# coding: utf8
from __future__ import unicode_literals


LOOKUP = {
    "aba": "abar",
    "ababa": "abar",
    "ababais": "abar",
    "ababan": "abar",
    "ababanes": "ababán",
    "ababas": "abar",
    "ababoles": "ababol",
    "ababábites": "ababábite",
    "abacera": "abacero",
    "abaceras": "abacero",
    "abaceros": "abacero",
    "abacerías": "abacería",
    "abaciales": "abacial",
    "abacás": "abacá",
    "abad": "abar",
    "abada": "abar",
    "abadas": "abar",
    "abadejos": "abadejo",
    "abadenga": "abadengo",
    "abadengas": "abadengo",
    "abadengos": "abadengo",
    "abades": "abad",
    "abadesas": "abadesa",
    "abadiados": "abadiado",
    "abadiatos": "abadiato",
    "abado": "abar",
    "abados": "abar",
    "abadías": "abadía",
    "abaja": "abajar",
    "abajaba": "abajar",
    "abajabais": "abajar",
    "abajaban": "abajar",
    "abajabas": "abajar",
    "abajad": "abajar",
    "abajada": "abajar",
    "abajadas": "abajar",
    "abajaderos": "abajadero",
    "abajado": "abajar",
    "abajados": "abajar",
    "abajamientos": "abajamiento",
    "abajamos": "abajar",
    "abajan": "abajar",
    "abajando": "abajar",
    "abajara": "abajar",
    "abajarais": "abajar",
    "abajaran": "abajar",
    "abajaras": "abajar",
    "abajare": "abajar",
    "abajareis": "abajar",
    "abajaremos": "abajar",
    "abajaren": "abajar",
    "abajares": "abajar",
    "abajaron": "abajar",
    "abajará": "abajar",
    "abajarán": "abajar",
    "abajarás": "abajar",
    "abajaré": "abajar",
    "abajaréis": "abajar",
    "abajaría": "abajar",
    "abajaríais": "abajar",
    "abajaríamos": "abajar",
    "abajarían": "abajar",
    "abajarías": "abajar",
    "abajas": "abajar",
    "abajase": "abajar",
    "abajaseis": "abajar",
    "abajasen": "abajar",
    "abajases": "abajar",
    "abajaste": "abajar",
    "abajasteis": "abajar",
    "abaje": "abajar",
    "abajemos": "abajar",
    "abajen": "abajar",
    "abajes": "abajar",
    "abajeña": "abajeño",
    "abajeñas": "abajeño",
    "abajeños": "abajeño",
    "abajo": "abajar",
    "abajábamos": "abajar",
    "abajáis": "abajar",
    "abajáramos": "abajar",
    "abajáremos": "abajar",
    "abajásemos": "abajar",
    "abajé": "abajar",
    "abajéis": "abajar",
    "abajó": "abajar",
    "abalance": "abalanzar",
    "abalancemos": "abalanzar",
    "abalancen": "abalanzar",
    "abalances": "abalanzar",
    "abalancé": "abalanzar",
    "abalancéis": "abalanzar",
    "abalanza": "abalanzar",
    "abalanzaba": "abalanzar",
    "abalanzabais": "abalanzar",
    "abalanzaban": "abalanzar",
    "abalanzabas": "abalanzar",
    "abalanzad": "abalanzar",
    "abalanzada": "abalanzar",
    "abalanzadas": "abalanzar",
    "abalanzado": "abalanzar",
    "abalanzados": "abalanzar",
    "abalanzamos": "abalanzar",
    "abalanzan": "abalanzar",
    "abalanzando": "abalanzar",
    "abalanzara": "abalanzar",
    "abalanzarais": "abalanzar",
    "abalanzaran": "abalanzar",
    "abalanzaras": "abalanzar",
    "abalanzare": "abalanzar",
    "abalanzareis": "abalanzar",
    "abalanzaremos": "abalanzar",
    "abalanzaren": "abalanzar",
    "abalanzares": "abalanzar",
    "abalanzaron": "abalanzar",
    "abalanzará": "abalanzar",
    "abalanzarán": "abalanzar",
    "abalanzarás": "abalanzar",
    "abalanzaré": "abalanzar",
    "abalanzaréis": "abalanzar",
    "abalanzaría": "abalanzar",
    "abalanzaríais": "abalanzar",
    "abalanzaríamos": "abalanzar",
    "abalanzarían": "abalanzar",
    "abalanzarías": "abalanzar",
    "abalanzas": "abalanzar",
    "abalanzase": "abalanzar",
    "abalanzaseis": "abalanzar",
    "abalanzasen": "abalanzar",
    "abalanzases": "abalanzar",
    "abalanzaste": "abalanzar",
    "abalanzasteis": "abalanzar",
    "abalanzo": "abalanzar",
    "abalanzábamos": "abalanzar",
    "abalanzáis": "abalanzar",
    "abalanzáramos": "abalanzar",
    "abalanzáremos": "abalanzar",
    "abalanzásemos": "abalanzar",
    "abalanzó": "abalanzar",
    "abalaustra": "abalaustrar",
    "abalaustraba": "abalaustrar",
    "abalaustrabais": "abalaustrar",
    "abalaustraban": "abalaustrar",
    "abalaustrabas": "abalaustrar",
    "abalaustrad": "abalaustrar",
    "abalaustrada": "abalaustrar",
    "abalaustradas": "abalaustrar",
    "abalaustrado": "abalaustrar",
    "abalaustrados": "abalaustrar",
    "abalaustramos": "abalaustrar",
    "abalaustran": "abalaustrar",
    "abalaustrando": "abalaustrar",
    "abalaustrara": "abalaustrar",
    "abalaustrarais": "abalaustrar",
    "abalaustraran": "abalaustrar",
    "abalaustraras": "abalaustrar",
    "abalaustrare": "abalaustrar",
    "abalaustrareis": "abalaustrar",
    "abalaustraremos": "abalaustrar",
    "abalaustraren": "abalaustrar",
    "abalaustrares": "abalaustrar",
    "abalaustraron": "abalaustrar",
    "abalaustrará": "abalaustrar",
    "abalaustrarán": "abalaustrar",
    "abalaustrarás": "abalaustrar",
    "abalaustraré": "abalaustrar",
    "abalaustraréis": "abalaustrar",
    "abalaustraría": "abalaustrar",
    "abalaustraríais": "abalaustrar",
    "abalaustraríamos": "abalaustrar",
    "abalaustrarían": "abalaustrar",
    "abalaustrarías": "abalaustrar",
    "abalaustras": "abalaustrar",
    "abalaustrase": "abalaustrar",
    "abalaustraseis": "abalaustrar",
    "abalaustrasen": "abalaustrar",
    "abalaustrases": "abalaustrar",
    "abalaustraste": "abalaustrar",
    "abalaustrasteis": "abalaustrar",
    "abalaustre": "abalaustrar",
    "abalaustremos": "abalaustrar",
    "abalaustren": "abalaustrar",
    "abalaustres": "abalaustrar",
    "abalaustro": "abalaustrar",
    "abalaustrábamos": "abalaustrar",
    "abalaustráis": "abalaustrar",
    "abalaustráramos": "abalaustrar",
    "abalaustráremos": "abalaustrar",
    "abalaustrásemos": "abalaustrar",
    "abalaustré": "abalaustrar",
    "abalaustréis": "abalaustrar",
    "abalaustró": "abalaustrar",
    "abalea": "abalear",
    "abaleaba": "abalear",
    "abaleabais": "abalear",
    "abaleaban": "abalear",
    "abaleabas": "abalear",
    "abalead": "abalear",
    "abaleada": "abalear",
    "abaleadas": "abalear",
    "abaleado": "abalear",
    "abaleadora": "abaleador",
    "abaleadoras": "abaleador",
    "abaleadores": "abaleador",
    "abaleados": "abalear",
    "abaleaduras": "abaleadura",
    "abaleamos": "abalear",
    "abalean": "abalear",
    "abaleando": "abalear",
    "abaleara": "abalear",
    "abalearais": "abalear",
    "abalearan": "abalear",
    "abalearas": "abalear",
    "abaleare": "abalear",
    "abaleareis": "abalear",
    "abalearemos": "abalear",
    "abalearen": "abalear",
    "abaleares": "abalear",
    "abalearon": "abalear",
    "abaleará": "abalear",
    "abalearán": "abalear",
    "abalearás": "abalear",
    "abalearé": "abalear",
    "abalearéis": "abalear",
    "abalearía": "abalear",
    "abalearíais": "abalear",
    "abalearíamos": "abalear",
    "abalearían": "abalear",
    "abalearías": "abalear",
    "abaleas": "abalear",
    "abalease": "abalear",
    "abaleaseis": "abalear",
    "abaleasen": "abalear",
    "abaleases": "abalear",
    "abaleaste": "abalear",
    "abaleasteis": "abalear",
    "abalee": "abalear",
    "abaleemos": "abalear",
    "abaleen": "abalear",
    "abalees": "abalear",
    "abaleo": "abalear",
    "abaleos": "abaleo",
    "abaleábamos": "abalear",
    "abaleáis": "abalear",
    "abaleáramos": "abalear",
    "abaleáremos": "abalear",
    "abaleásemos": "abalear",
    "abaleé": "abalear",
    "abaleéis": "abalear",
    "abaleó": "abalear",
    "abalice": "abalizar",
    "abalicemos": "abalizar",
    "abalicen": "abalizar",
    "abalices": "abalizar",
    "abalicé": "abalizar",
    "abalicéis": "abalizar",
    "abaliza": "abalizar",
    "abalizaba": "abalizar",
    "abalizabais": "abalizar",
    "abalizaban": "abalizar",
    "abalizabas": "abalizar",
    "abalizad": "abalizar",
    "abalizada": "abalizar",
    "abalizadas": "abalizar",
    "abalizado": "abalizar",
    "abalizados": "abalizar",
    "abalizamientos": "abalizamiento",
    "abalizamos": "abalizar",
    "abalizan": "abalizar",
    "abalizando": "abalizar",
    "abalizara": "abalizar",
    "abalizarais": "abalizar",
    "abalizaran": "abalizar",
    "abalizaras": "abalizar",
    "abalizare": "abalizar",
    "abalizareis": "abalizar",
    "abalizaremos": "abalizar",
    "abalizaren": "abalizar",
    "abalizares": "abalizar",
    "abalizaron": "abalizar",
    "abalizará": "abalizar",
    "abalizarán": "abalizar",
    "abalizarás": "abalizar",
    "abalizaré": "abalizar",
    "abalizaréis": "abalizar",
    "abalizaría": "abalizar",
    "abalizaríais": "abalizar",
    "abalizaríamos": "abalizar",
    "abalizarían": "abalizar",
    "abalizarías": "abalizar",
    "abalizas": "abalizar",
    "abalizase": "abalizar",
    "abalizaseis": "abalizar",
    "abalizasen": "abalizar",
    "abalizases": "abalizar",
    "abalizaste": "abalizar",
    "abalizasteis": "abalizar",
    "abalizo": "abalizar",
    "abalizábamos": "abalizar",
    "abalizáis": "abalizar",
    "abalizáramos": "abalizar",
    "abalizáremos": "abalizar",
    "abalizásemos": "abalizar",
    "abalizó": "abalizar",
    "aballa": "aballar",
    "aballaba": "aballar",
    "aballabais": "aballar",
    "aballaban": "aballar",
    "aballabas": "aballar",
    "aballad": "aballar",
    "aballada": "aballar",
    "aballadas": "aballar",
    "aballado": "aballar",
    "aballados": "aballar",
    "aballamos": "aballar",
    "aballan": "aballar",
    "aballando": "aballar",
    "aballara": "aballar",
    "aballarais": "aballar",
    "aballaran": "aballar",
    "aballaras": "aballar",
    "aballare": "aballar",
    "aballareis": "aballar",
    "aballaremos": "aballar",
    "aballaren": "aballar",
    "aballares": "aballar",
    "aballaron": "aballar",
    "aballará": "aballar",
    "aballarán": "aballar",
    "aballarás": "aballar",
    "aballaré": "aballar",
    "aballaréis": "aballar",
    "aballaría": "aballar",
    "aballaríais": "aballar",
    "aballaríamos": "aballar",
    "aballarían": "aballar",
    "aballarías": "aballar",
    "aballas": "aballar",
    "aballase": "aballar",
    "aballaseis": "aballar",
    "aballasen": "aballar",
    "aballases": "aballar",
    "aballaste": "aballar",
    "aballasteis": "aballar",
    "aballe": "aballar",
    "aballemos": "aballar",
    "aballen": "aballar",
    "aballes": "aballar",
    "aballesta": "aballestar",
    "aballestaba": "aballestar",
    "aballestabais": "aballestar",
    "aballestaban": "aballestar",
    "aballestabas": "aballestar",
    "aballestad": "aballestar",
    "aballestada": "aballestar",
    "aballestadas": "aballestar",
    "aballestado": "aballestar",
    "aballestados": "aballestar",
    "aballestamos": "aballestar",
    "aballestan": "aballestar",
    "aballestando": "aballestar",
    "aballestara": "aballestar",
    "aballestarais": "aballestar",
    "aballestaran": "aballestar",
    "aballestaras": "aballestar",
    "aballestare": "aballestar",
    "aballestareis": "aballestar",
    "aballestaremos": "aballestar",
    "aballestaren": "aballestar",
    "aballestares": "aballestar",
    "aballestaron": "aballestar",
    "aballestará": "aballestar",
    "aballestarán": "aballestar",
    "aballestarás": "aballestar",
    "aballestaré": "aballestar",
    "aballestaréis": "aballestar",
    "aballestaría": "aballestar",
    "aballestaríais": "aballestar",
    "aballestaríamos": "aballestar",
    "aballestarían": "aballestar",
    "aballestarías": "aballestar",
    "aballestas": "aballestar",
    "aballestase": "aballestar",
    "aballestaseis": "aballestar",
    "aballestasen": "aballestar",
    "aballestases": "aballestar",
    "aballestaste": "aballestar",
    "aballestasteis": "aballestar",
    "aballeste": "aballestar",
    "aballestemos": "aballestar",
    "aballesten": "aballestar",
    "aballestes": "aballestar",
    "aballesto": "aballestar",
    "aballestábamos": "aballestar",
    "aballestáis": "aballestar",
    "aballestáramos": "aballestar",
    "aballestáremos": "aballestar",
    "aballestásemos": "aballestar",
    "aballesté": "aballestar",
    "aballestéis": "aballestar",
    "aballestó": "aballestar",
    "aballo": "aballar",
    "aballábamos": "aballar",
    "aballáis": "aballar",
    "aballáramos": "aballar",
    "aballáremos": "aballar",
    "aballásemos": "aballar",
    "aballé": "aballar",
    "aballéis": "aballar",
    "aballó": "aballar",
    "abalorios": "abalorio",
    "abaluarta": "abaluartar",
    "abaluartaba": "abaluartar",
    "abaluartabais": "abaluartar",
    "abaluartaban": "abaluartar",
    "abaluartabas": "abaluartar",
    "abaluartad": "abaluartar",
    "abaluartada": "abaluartar",
    "abaluartadas": "abaluartar",
    "abaluartado": "abaluartar",
    "abaluartados": "abaluartar",
    "abaluartamos": "abaluartar",
    "abaluartan": "abaluartar",
    "abaluartando": "abaluartar",
    "abaluartara": "abaluartar",
    "abaluartarais": "abaluartar",
    "abaluartaran": "abaluartar",
    "abaluartaras": "abaluartar",
    "abaluartare": "abaluartar",
    "abaluartareis": "abaluartar",
    "abaluartaremos": "abaluartar",
    "abaluartaren": "abaluartar",
    "abaluartares": "abaluartar",
    "abaluartaron": "abaluartar",
    "abaluartará": "abaluartar",
    "abaluartarán": "abaluartar",
    "abaluartarás": "abaluartar",
    "abaluartaré": "abaluartar",
    "abaluartaréis": "abaluartar",
    "abaluartaría": "abaluartar",
    "abaluartaríais": "abaluartar",
    "abaluartaríamos": "abaluartar",
    "abaluartarían": "abaluartar",
    "abaluartarías": "abaluartar",
    "abaluartas": "abaluartar",
    "abaluartase": "abaluartar",
    "abaluartaseis": "abaluartar",
    "abaluartasen": "abaluartar",
    "abaluartases": "abaluartar",
    "abaluartaste": "abaluartar",
    "abaluartasteis": "abaluartar",
    "abaluarte": "abaluartar",
    "abaluartemos": "abaluartar",
    "abaluarten": "abaluartar",
    "abaluartes": "abaluartar",
    "abaluarto": "abaluartar",
    "abaluartábamos": "abaluartar",
    "abaluartáis": "abaluartar",
    "abaluartáramos": "abaluartar",
    "abaluartáremos": "abaluartar",
    "abaluartásemos": "abaluartar",
    "abaluarté": "abaluartar",
    "abaluartéis": "abaluartar",
    "abaluartó": "abaluartar",
    "abamos": "abar",
    "aban": "abar",
    "abana": "abanar",
    "abanaba": "abanar",
    "abanabais": "abanar",
    "abanaban": "abanar",
    "abanabas": "abanar",
    "abanad": "abanar",
    "abanada": "abanar",
    "abanadas": "abanar",
    "abanado": "abanar",
    "abanados": "abanar",
    "abanamos": "abanar",
    "abanan": "abanar",
    "abanando": "abanar",
    "abanara": "abanar",
    "abanarais": "abanar",
    "abanaran": "abanar",
    "abanaras": "abanar",
    "abanare": "abanar",
    "abanareis": "abanar",
    "abanaremos": "abanar",
    "abanaren": "abanar",
    "abanares": "abanar",
    "abanaron": "abanar",
    "abanará": "abanar",
    "abanarán": "abanar",
    "abanarás": "abanar",
    "abanaré": "abanar",
    "abanaréis": "abanar",
    "abanaría": "abanar",
    "abanaríais": "abanar",
    "abanaríamos": "abanar",
    "abanarían": "abanar",
    "abanarías": "abanar",
    "abanas": "abanar",
    "abanase": "abanar",
    "abanaseis": "abanar",
    "abanasen": "abanar",
    "abanases": "abanar",
    "abanaste": "abanar",
    "abanasteis": "abanar",
    "abandalice": "abandalizar",
    "abandalicemos": "abandalizar",
    "abandalicen": "abandalizar",
    "abandalices": "abandalizar",
    "abandalicé": "abandalizar",
    "abandalicéis": "abandalizar",
    "abandaliza": "abandalizar",
    "abandalizaba": "abandalizar",
    "abandalizabais": "abandalizar",
    "abandalizaban": "abandalizar",
    "abandalizabas": "abandalizar",
    "abandalizad": "abandalizar",
    "abandalizada": "abandalizar",
    "abandalizadas": "abandalizar",
    "abandalizado": "abandalizar",
    "abandalizados": "abandalizar",
    "abandalizamos": "abandalizar",
    "abandalizan": "abandalizar",
    "abandalizando": "abandalizar",
    "abandalizara": "abandalizar",
    "abandalizarais": "abandalizar",
    "abandalizaran": "abandalizar",
    "abandalizaras": "abandalizar",
    "abandalizare": "abandalizar",
    "abandalizareis": "abandalizar",
    "abandalizaremos": "abandalizar",
    "abandalizaren": "abandalizar",
    "abandalizares": "abandalizar",
    "abandalizaron": "abandalizar",
    "abandalizará": "abandalizar",
    "abandalizarán": "abandalizar",
    "abandalizarás": "abandalizar",
    "abandalizaré": "abandalizar",
    "abandalizaréis": "abandalizar",
    "abandalizaría": "abandalizar",
    "abandalizaríais": "abandalizar",
    "abandalizaríamos": "abandalizar",
    "abandalizarían": "abandalizar",
    "abandalizarías": "abandalizar",
    "abandalizas": "abandalizar",
    "abandalizase": "abandalizar",
    "abandalizaseis": "abandalizar",
    "abandalizasen": "abandalizar",
    "abandalizases": "abandalizar",
    "abandalizaste": "abandalizar",
    "abandalizasteis": "abandalizar",
    "abandalizo": "abandalizar",
    "abandalizábamos": "abandalizar",
    "abandalizáis": "abandalizar",
    "abandalizáramos": "abandalizar",
    "abandalizáremos": "abandalizar",
    "abandalizásemos": "abandalizar",
    "abandalizó": "abandalizar",
    "abandera": "abanderar",
    "abanderaba": "abanderar",
    "abanderabais": "abanderar",
    "abanderaban": "abanderar",
    "abanderabas": "abanderar",
    "abanderad": "abanderar",
    "abanderada": "abanderar",
    "abanderadas": "abanderar",
    "abanderado": "abanderar",
    "abanderados": "abanderar",
    "abanderamientos": "abanderamiento",
    "abanderamos": "abanderar",
    "abanderan": "abanderar",
    "abanderando": "abanderar",
    "abanderara": "abanderar",
    "abanderarais": "abanderar",
    "abanderaran": "abanderar",
    "abanderaras": "abanderar",
    "abanderare": "abanderar",
    "abanderareis": "abanderar",
    "abanderaremos": "abanderar",
    "abanderaren": "abanderar",
    "abanderares": "abanderar",
    "abanderaron": "abanderar",
    "abanderará": "abanderar",
    "abanderarán": "abanderar",
    "abanderarás": "abanderar",
    "abanderaré": "abanderar",
    "abanderaréis": "abanderar",
    "abanderaría": "abanderar",
    "abanderaríais": "abanderar",
    "abanderaríamos": "abanderar",
    "abanderarían": "abanderar",
    "abanderarías": "abanderar",
    "abanderas": "abanderar",
    "abanderase": "abanderar",
    "abanderaseis": "abanderar",
    "abanderasen": "abanderar",
    "abanderases": "abanderar",
    "abanderaste": "abanderar",
    "abanderasteis": "abanderar",
    "abandere": "abanderar",
    "abanderemos": "abanderar",
    "abanderen": "abanderar",
    "abanderes": "abanderar",
    "abanderice": "abanderizar",
    "abandericemos": "abanderizar",
    "abandericen": "abanderizar",
    "abanderices": "abanderizar",
    "abandericé": "abanderizar",
    "abandericéis": "abanderizar",
    "abanderiza": "abanderizar",
    "abanderizaba": "abanderizar",
    "abanderizabais": "abanderizar",
    "abanderizaban": "abanderizar",
    "abanderizabas": "abanderizar",
    "abanderizad": "abanderizar",
    "abanderizada": "abanderizar",
    "abanderizadas": "abanderizar",
    "abanderizado": "abanderizar",
    "abanderizados": "abanderizar",
    "abanderizamos": "abanderizar",
    "abanderizan": "abanderizar",
    "abanderizando": "abanderizar",
    "abanderizara": "abanderizar",
    "abanderizarais": "abanderizar",
    "abanderizaran": "abanderizar",
    "abanderizaras": "abanderizar",
    "abanderizare": "abanderizar",
    "abanderizareis": "abanderizar",
    "abanderizaremos": "abanderizar",
    "abanderizaren": "abanderizar",
    "abanderizares": "abanderizar",
    "abanderizaron": "abanderizar",
    "abanderizará": "abanderizar",
    "abanderizarán": "abanderizar",
    "abanderizarás": "abanderizar",
    "abanderizaré": "abanderizar",
    "abanderizaréis": "abanderizar",
    "abanderizaría": "abanderizar",
    "abanderizaríais": "abanderizar",
    "abanderizaríamos": "abanderizar",
    "abanderizarían": "abanderizar",
    "abanderizarías": "abanderizar",
    "abanderizas": "abanderizar",
    "abanderizase": "abanderizar",
    "abanderizaseis": "abanderizar",
    "abanderizasen": "abanderizar",
    "abanderizases": "abanderizar",
    "abanderizaste": "abanderizar",
    "abanderizasteis": "abanderizar",
    "abanderizo": "abanderizar",
    "abanderizábamos": "abanderizar",
    "abanderizáis": "abanderizar",
    "abanderizáramos": "abanderizar",
    "abanderizáremos": "abanderizar",
    "abanderizásemos": "abanderizar",
    "abanderizó": "abanderizar",
    "abandero": "abanderar",
    "abanderábamos": "abanderar",
    "abanderáis": "abanderar",
    "abanderáramos": "abanderar",
    "abanderáremos": "abanderar",
    "abanderásemos": "abanderar",
    "abanderé": "abanderar",
    "abanderéis": "abanderar",
    "abanderó": "abanderar",
    "abando": "abar",
    "abandona": "abandonar",
    "abandonaba": "abandonar",
    "abandonabais": "abandonar",
    "abandonaban": "abandonar",
    "abandonabas": "abandonar",
    "abandonad": "abandonar",
    "abandonada": "abandonar",
    "abandonadas": "abandonar",
    "abandonado": "abandonar",
    "abandonados": "abandonar",
    "abandonamos": "abandonar",
    "abandonan": "abandonar",
    "abandonando": "abandonar",
    "abandonara": "abandonar",
    "abandonarais": "abandonar",
    "abandonaran": "abandonar",
    "abandonaras": "abandonar",
    "abandonare": "abandonar",
    "abandonareis": "abandonar",
    "abandonaremos": "abandonar",
    "abandonaren": "abandonar",
    "abandonares": "abandonar",
    "abandonaron": "abandonar",
    "abandonará": "abandonar",
    "abandonarán": "abandonar",
    "abandonarás": "abandonar",
    "abandonaré": "abandonar",
    "abandonaréis": "abandonar",
    "abandonaría": "abandonar",
    "abandonaríais": "abandonar",
    "abandonaríamos": "abandonar",
    "abandonarían": "abandonar",
    "abandonarías": "abandonar",
    "abandonas": "abandonar",
    "abandonase": "abandonar",
    "abandonaseis": "abandonar",
    "abandonasen": "abandonar",
    "abandonases": "abandonar",
    "abandonaste": "abandonar",
    "abandonasteis": "abandonar",
    "abandone": "abandonar",
    "abandonemos": "abandonar",
    "abandonen": "abandonar",
    "abandones": "abandonar",
    "abandonismos": "abandonismo",
    "abandonistas": "abandonista",
    "abandono": "abandonar",
    "abandonos": "abandono",
    "abandonábamos": "abandonar",
    "abandonáis": "abandonar",
    "abandonáramos": "abandonar",
    "abandonáremos": "abandonar",
    "abandonásemos": "abandonar",
    "abandoné": "abandonar",
    "abandonéis": "abandonar",
    "abandonó": "abandonar",
    "abane": "abanar",
    "abanemos": "abanar",
    "abanen": "abanar",
    "abanes": "abanar",
    "abanica": "abanicar",
    "abanicaba": "abanicar",
    "abanicabais": "abanicar",
    "abanicaban": "abanicar",
    "abanicabas": "abanicar",
    "abanicad": "abanicar",
    "abanicada": "abanicar",
    "abanicadas": "abanicar",
    "abanicado": "abanicar",
    "abanicados": "abanicar",
    "abanicamos": "abanicar",
    "abanican": "abanicar",
    "abanicando": "abanicar",
    "abanicara": "abanicar",
    "abanicarais": "abanicar",
    "abanicaran": "abanicar",
    "abanicaras": "abanicar",
    "abanicare": "abanicar",
    "abanicareis": "abanicar",
    "abanicaremos": "abanicar",
    "abanicaren": "abanicar",
    "abanicares": "abanicar",
    "abanicaron": "abanicar",
    "abanicará": "abanicar",
    "abanicarán": "abanicar",
    "abanicarás": "abanicar",
    "abanicaré": "abanicar",
    "abanicaréis": "abanicar",
    "abanicaría": "abanicar",
    "abanicaríais": "abanicar",
    "abanicaríamos": "abanicar",
    "abanicarían": "abanicar",
    "abanicarías": "abanicar",
    "abanicas": "abanicar",
    "abanicase": "abanicar",
    "abanicaseis": "abanicar",
    "abanicasen": "abanicar",
    "abanicases": "abanicar",
    "abanicaste": "abanicar",
    "abanicasteis": "abanicar",
    "abanicazos": "abanicazo",
    "abanico": "abanicar",
    "abanicos": "abanico",
    "abanicábamos": "abanicar",
    "abanicáis": "abanicar",
    "abanicáramos": "abanicar",
    "abanicáremos": "abanicar",
    "abanicásemos": "abanicar",
    "abanicó": "abanicar",
    "abanillos": "abanillo",
    "abanique": "abanicar",
    "abaniquemos": "abanicar",
    "abaniquen": "abanicar",
    "abaniqueos": "abaniqueo",
    "abaniquera": "abaniquero",
    "abaniqueras": "abaniquero",
    "abaniqueros": "abaniquero",
    "abaniquerías": "abaniquería",
    "abaniques": "abanicar",
    "abaniqué": "abanicar",
    "abaniquéis": "abanicar",
    "abano": "abanar",
    "abanos": "abano",
    "abantos": "abanto",
    "abanábamos": "abanar",
    "abanáis": "abanar",
    "abanáramos": "abanar",
    "abanáremos": "abanar",
    "abanásemos": "abanar",
    "abané": "abanar",
    "abanéis": "abanar",
    "abanó": "abanar",
    "abara": "abar",
    "abarais": "abar",
    "abaran": "abar",
    "abaras": "abar",
    "abarata": "abaratar",
    "abarataba": "abaratar",
    "abaratabais": "abaratar",
    "abarataban": "abaratar",
    "abaratabas": "abaratar",
    "abaratad": "abaratar",
    "abaratada": "abaratar",
    "abaratadas": "abaratar",
    "abaratado": "abaratar",
    "abaratados": "abaratar",
    "abaratamientos": "abaratamiento",
    "abaratamos": "abaratar",
    "abaratan": "abaratar",
    "abaratando": "abaratar",
    "abaratara": "abaratar",
    "abaratarais": "abaratar",
    "abarataran": "abaratar",
    "abarataras": "abaratar",
    "abaratare": "abaratar",
    "abaratareis": "abaratar",
    "abarataremos": "abaratar",
    "abarataren": "abaratar",
    "abaratares": "abaratar",
    "abarataron": "abaratar",
    "abaratará": "abaratar",
    "abaratarán": "abaratar",
    "abaratarás": "abaratar",
    "abarataré": "abaratar",
    "abarataréis": "abaratar",
    "abarataría": "abaratar",
    "abarataríais": "abaratar",
    "abarataríamos": "abaratar",
    "abaratarían": "abaratar",
    "abaratarías": "abaratar",
    "abaratas": "abaratar",
    "abaratase": "abaratar",
    "abarataseis": "abaratar",
    "abaratasen": "abaratar",
    "abaratases": "abaratar",
    "abarataste": "abaratar",
    "abaratasteis": "abaratar",
    "abarate": "abaratar",
    "abaratemos": "abaratar",
    "abaraten": "abaratar",
    "abarates": "abaratar",
    "abarato": "abaratar",
    "abaratábamos": "abaratar",
    "abaratáis": "abaratar",
    "abaratáramos": "abaratar",
    "abaratáremos": "abaratar",
    "abaratásemos": "abaratar",
    "abaraté": "abaratar",
    "abaratéis": "abaratar",
    "abarató": "abaratar",
    "abarbecha": "abarbechar",
    "abarbechaba": "abarbechar",
    "abarbechabais": "abarbechar",
    "abarbechaban": "abarbechar",
    "abarbechabas": "abarbechar",
    "abarbechad": "abarbechar",
    "abarbechada": "abarbechar",
    "abarbechadas": "abarbechar",
    "abarbechado": "abarbechar",
    "abarbechados": "abarbechar",
    "abarbechamos": "abarbechar",
    "abarbechan": "abarbechar",
    "abarbechando": "abarbechar",
    "abarbechara": "abarbechar",
    "abarbecharais": "abarbechar",
    "abarbecharan": "abarbechar",
    "abarbecharas": "abarbechar",
    "abarbechare": "abarbechar",
    "abarbechareis": "abarbechar",
    "abarbecharemos": "abarbechar",
    "abarbecharen": "abarbechar",
    "abarbechares": "abarbechar",
    "abarbecharon": "abarbechar",
    "abarbechará": "abarbechar",
    "abarbecharán": "abarbechar",
    "abarbecharás": "abarbechar",
    "abarbecharé": "abarbechar",
    "abarbecharéis": "abarbechar",
    "abarbecharía": "abarbechar",
    "abarbecharíais": "abarbechar",
    "abarbecharíamos": "abarbechar",
    "abarbecharían": "abarbechar",
    "abarbecharías": "abarbechar",
    "abarbechas": "abarbechar",
    "abarbechase": "abarbechar",
    "abarbechaseis": "abarbechar",
    "abarbechasen": "abarbechar",
    "abarbechases": "abarbechar",
    "abarbechaste": "abarbechar",
    "abarbechasteis": "abarbechar",
    "abarbeche": "abarbechar",
    "abarbechemos": "abarbechar",
    "abarbechen": "abarbechar",
    "abarbeches": "abarbechar",
    "abarbecho": "abarbechar",
    "abarbechábamos": "abarbechar",
    "abarbecháis": "abarbechar",
    "abarbecháramos": "abarbechar",
    "abarbecháremos": "abarbechar",
    "abarbechásemos": "abarbechar",
    "abarbeché": "abarbechar",
    "abarbechéis": "abarbechar",
    "abarbechó": "abarbechar",
    "abarbeta": "abarbetar",
    "abarbetaba": "abarbetar",
    "abarbetabais": "abarbetar",
    "abarbetaban": "abarbetar",
    "abarbetabas": "abarbetar",
    "abarbetad": "abarbetar",
    "abarbetada": "abarbetar",
    "abarbetadas": "abarbetar",
    "abarbetado": "abarbetar",
    "abarbetados": "abarbetar",
    "abarbetamos": "abarbetar",
    "abarbetan": "abarbetar",
    "abarbetando": "abarbetar",
    "abarbetara": "abarbetar",
    "abarbetarais": "abarbetar",
    "abarbetaran": "abarbetar",
    "abarbetaras": "abarbetar",
    "abarbetare": "abarbetar",
    "abarbetareis": "abarbetar",
    "abarbetaremos": "abarbetar",
    "abarbetaren": "abarbetar",
    "abarbetares": "abarbetar",
    "abarbetaron": "abarbetar",
    "abarbetará": "abarbetar",
    "abarbetarán": "abarbetar",
    "abarbetarás": "abarbetar",
    "abarbetaré": "abarbetar",
    "abarbetaréis": "abarbetar",
    "abarbetaría": "abarbetar",
    "abarbetaríais": "abarbetar",
    "abarbetaríamos": "abarbetar",
    "abarbetarían": "abarbetar",
    "abarbetarías": "abarbetar",
    "abarbetas": "abarbetar",
    "abarbetase": "abarbetar",
    "abarbetaseis": "abarbetar",
    "abarbetasen": "abarbetar",
    "abarbetases": "abarbetar",
    "abarbetaste": "abarbetar",
    "abarbetasteis": "abarbetar",
    "abarbete": "abarbetar",
    "abarbetemos": "abarbetar",
    "abarbeten": "abarbetar",
    "abarbetes": "abarbetar",
    "abarbeto": "abarbetar",
    "abarbetábamos": "abarbetar",
    "abarbetáis": "abarbetar",
    "abarbetáramos": "abarbetar",
    "abarbetáremos": "abarbetar",
    "abarbetásemos": "abarbetar",
    "abarbeté": "abarbetar",
    "abarbetéis": "abarbetar",
    "abarbetó": "abarbetar",
    "abarca": "abarcar",
    "abarcaba": "abarcar",
    "abarcabais": "abarcar",
    "abarcaban": "abarcar",
    "abarcabas": "abarcar",
    "abarcables": "abarcable",
    "abarcad": "abarcar",
    "abarcada": "abarcar",
    "abarcadas": "abarcar",
    "abarcado": "abarcar",
    "abarcadora": "abarcador",
    "abarcadoras": "abarcador",
    "abarcadores": "abarcador",
    "abarcados": "abarcar",
    "abarcaduras": "abarcadura",
    "abarcamientos": "abarcamiento",
    "abarcamos": "abarcar",
    "abarcan": "abarcar",
    "abarcando": "abarcar",
    "abarcara": "abarcar",
    "abarcarais": "abarcar",
    "abarcaran": "abarcar",
    "abarcaras": "abarcar",
    "abarcare": "abarcar",
    "abarcareis": "abarcar",
    "abarcaremos": "abarcar",
    "abarcaren": "abarcar",
    "abarcares": "abarcar",
    "abarcaron": "abarcar",
    "abarcará": "abarcar",
    "abarcarán": "abarcar",
    "abarcarás": "abarcar",
    "abarcaré": "abarcar",
    "abarcaréis": "abarcar",
    "abarcaría": "abarcar",
    "abarcaríais": "abarcar",
    "abarcaríamos": "abarcar",
    "abarcarían": "abarcar",
    "abarcarías": "abarcar",
    "abarcas": "abarcar",
    "abarcase": "abarcar",
    "abarcaseis": "abarcar",
    "abarcasen": "abarcar",
    "abarcases": "abarcar",
    "abarcaste": "abarcar",
    "abarcasteis": "abarcar",
    "abarco": "abarcar",
    "abarcábamos": "abarcar",
    "abarcáis": "abarcar",
    "abarcáramos": "abarcar",
    "abarcáremos": "abarcar",
    "abarcásemos": "abarcar",
    "abarcó": "abarcar",
    "abare": "abar",
    "abareis": "abar",
    "abaremos": "abar",
    "abaren": "abar",
    "abares": "abar",
    "abaritonada": "abaritonado",
    "abaritonadas": "abaritonado",
    "abaritonados": "abaritonado",
    "abarloa": "abarloar",
    "abarloaba": "abarloar",
    "abarloabais": "abarloar",
    "abarloaban": "abarloar",
    "abarloabas": "abarloar",
    "abarload": "abarloar",
    "abarloada": "abarloar",
    "abarloadas": "abarloar",
    "abarloado": "abarloar",
    "abarloados": "abarloar",
    "abarloamientos": "abarloamiento",
    "abarloamos": "abarloar",
    "abarloan": "abarloar",
    "abarloando": "abarloar",
    "abarloara": "abarloar",
    "abarloarais": "abarloar",
    "abarloaran": "abarloar",
    "abarloaras": "abarloar",
    "abarloare": "abarloar",
    "abarloareis": "abarloar",
    "abarloaremos": "abarloar",
    "abarloaren": "abarloar",
    "abarloares": "abarloar",
    "abarloaron": "abarloar",
    "abarloará": "abarloar",
    "abarloarán": "abarloar",
    "abarloarás": "abarloar",
    "abarloaré": "abarloar",
    "abarloaréis": "abarloar",
    "abarloaría": "abarloar",
    "abarloaríais": "abarloar",
    "abarloaríamos": "abarloar",
    "abarloarían": "abarloar",
    "abarloarías": "abarloar",
    "abarloas": "abarloar",
    "abarloase": "abarloar",
    "abarloaseis": "abarloar",
    "abarloasen": "abarloar",
    "abarloases": "abarloar",
    "abarloaste": "abarloar",
    "abarloasteis": "abarloar",
    "abarloe": "abarloar",
    "abarloemos": "abarloar",
    "abarloen": "abarloar",
    "abarloes": "abarloar",
    "abarloo": "abarloar",
    "abarloábamos": "abarloar",
    "abarloáis": "abarloar",
    "abarloáramos": "abarloar",
    "abarloáremos": "abarloar",
    "abarloásemos": "abarloar",
    "abarloé": "abarloar",
    "abarloéis": "abarloar",
    "abarloó": "abarloar",
    "abaron": "abar",
    "abarque": "abarcar",
    "abarquemos": "abarcar",
    "abarquen": "abarcar",
    "abarques": "abarcar",
    "abarquilla": "abarquillar",
    "abarquillaba": "abarquillar",
    "abarquillabais": "abarquillar",
    "abarquillaban": "abarquillar",
    "abarquillabas": "abarquillar",
    "abarquillad": "abarquillar",
    "abarquillada": "abarquillar",
    "abarquilladas": "abarquillar",
    "abarquillado": "abarquillar",
    "abarquillados": "abarquillar",
    "abarquillamientos": "abarquillamiento",
    "abarquillamos": "abarquillar",
    "abarquillan": "abarquillar",
    "abarquillando": "abarquillar",
    "abarquillara": "abarquillar",
    "abarquillarais": "abarquillar",
    "abarquillaran": "abarquillar",
    "abarquillaras": "abarquillar",
    "abarquillare": "abarquillar",
    "abarquillareis": "abarquillar",
    "abarquillaremos": "abarquillar",
    "abarquillaren": "abarquillar",
    "abarquillares": "abarquillar",
    "abarquillaron": "abarquillar",
    "abarquillará": "abarquillar",
    "abarquillarán": "abarquillar",
    "abarquillarás": "abarquillar",
    "abarquillaré": "abarquillar",
    "abarquillaréis": "abarquillar",
    "abarquillaría": "abarquillar",
    "abarquillaríais": "abarquillar",
    "abarquillaríamos": "abarquillar",
    "abarquillarían": "abarquillar",
    "abarquillarías": "abarquillar",
    "abarquillas": "abarquillar",
    "abarquillase": "abarquillar",
    "abarquillaseis": "abarquillar",
    "abarquillasen": "abarquillar",
    "abarquillases": "abarquillar",
    "abarquillaste": "abarquillar",
    "abarquillasteis": "abarquillar",
    "abarquille": "abarquillar",
    "abarquillemos": "abarquillar",
    "abarquillen": "abarquillar",
    "abarquilles": "abarquillar",
    "abarquillo": "abarquillar",
    "abarquillábamos": "abarquillar",
    "abarquilláis": "abarquillar",
    "abarquilláramos": "abarquillar",
    "abarquilláremos": "abarquillar",
    "abarquillásemos": "abarquillar",
    "abarquillé": "abarquillar",
    "abarquilléis": "abarquillar",
    "abarquilló": "abarquillar",
    "abarqué": "abarcar",
    "abarquéis": "abarcar",
    "abarraca": "abarracar",
    "abarracaba": "abarracar",
    "abarracabais": "abarracar",
    "abarracaban": "abarracar",
    "abarracabas": "abarracar",
    "abarracad": "abarracar",
    "abarracada": "abarracar",
    "abarracadas": "abarracar",
    "abarracado": "abarracar",
    "abarracados": "abarracar",
    "abarracamos": "abarracar",
    "abarracan": "abarracar",
    "abarracando": "abarracar",
    "abarracara": "abarracar",
    "abarracarais": "abarracar",
    "abarracaran": "abarracar",
    "abarracaras": "abarracar",
    "abarracare": "abarracar",
    "abarracareis": "abarracar",
    "abarracaremos": "abarracar",
    "abarracaren": "abarracar",
    "abarracares": "abarracar",
    "abarracaron": "abarracar",
    "abarracará": "abarracar",
    "abarracarán": "abarracar",
    "abarracarás": "abarracar",
    "abarracaré": "abarracar",
    "abarracaréis": "abarracar",
    "abarracaría": "abarracar",
    "abarracaríais": "abarracar",
    "abarracaríamos": "abarracar",
    "abarracarían": "abarracar",
    "abarracarías": "abarracar",
    "abarracas": "abarracar",
    "abarracase": "abarracar",
    "abarracaseis": "abarracar",
    "abarracasen": "abarracar",
    "abarracases": "abarracar",
    "abarracaste": "abarracar",
    "abarracasteis": "abarracar",
    "abarraco": "abarracar",
    "abarracábamos": "abarracar",
    "abarracáis": "abarracar",
    "abarracáramos": "abarracar",
    "abarracáremos": "abarracar",
    "abarracásemos": "abarracar",
    "abarracó": "abarracar",
    "abarragana": "abarraganar",
    "abarraganaba": "abarraganar",
    "abarraganabais": "abarraganar",
    "abarraganaban": "abarraganar",
    "abarraganabas": "abarraganar",
    "abarraganad": "abarraganar",
    "abarraganada": "abarraganar",
    "abarraganadas": "abarraganar",
    "abarraganado": "abarraganar",
    "abarraganados": "abarraganar",
    "abarraganamientos": "abarraganamiento",
    "abarraganamos": "abarraganar",
    "abarraganan": "abarraganar",
    "abarraganando": "abarraganar",
    "abarraganara": "abarraganar",
    "abarraganarais": "abarraganar",
    "abarraganaran": "abarraganar",
    "abarraganaras": "abarraganar",
    "abarraganare": "abarraganar",
    "abarraganareis": "abarraganar",
    "abarraganaremos": "abarraganar",
    "abarraganaren": "abarraganar",
    "abarraganares": "abarraganar",
    "abarraganaron": "abarraganar",
    "abarraganará": "abarraganar",
    "abarraganarán": "abarraganar",
    "abarraganarás": "abarraganar",
    "abarraganaré": "abarraganar",
    "abarraganaréis": "abarraganar",
    "abarraganaría": "abarraganar",
    "abarraganaríais": "abarraganar",
    "abarraganaríamos": "abarraganar",
    "abarraganarían": "abarraganar",
    "abarraganarías": "abarraganar",
    "abarraganas": "abarraganar",
    "abarraganase": "abarraganar",
    "abarraganaseis": "abarraganar",
    "abarraganasen": "abarraganar",
    "abarraganases": "abarraganar",
    "abarraganaste": "abarraganar",
    "abarraganasteis": "abarraganar",
    "abarragane": "abarraganar",
    "abarraganemos": "abarraganar",
    "abarraganen": "abarraganar",
    "abarraganes": "abarraganar",
    "abarragano": "abarraganar",
    "abarraganábamos": "abarraganar",
    "abarraganáis": "abarraganar",
    "abarraganáramos": "abarraganar",
    "abarraganáremos": "abarraganar",
    "abarraganásemos": "abarraganar",
    "abarragané": "abarraganar",
    "abarraganéis": "abarraganar",
    "abarraganó": "abarraganar",
    "abarraja": "abarrajar",
    "abarrajaba": "abarrajar",
    "abarrajabais": "abarrajar",
    "abarrajaban": "abarrajar",
    "abarrajabas": "abarrajar",
    "abarrajad": "abarrajar",
    "abarrajada": "abarrajar",
    "abarrajadas": "abarrajar",
    "abarrajado": "abarrajar",
    "abarrajados": "abarrajar",
    "abarrajamos": "abarrajar",
    "abarrajan": "abarrajar",
    "abarrajando": "abarrajar",
    "abarrajara": "abarrajar",
    "abarrajarais": "abarrajar",
    "abarrajaran": "abarrajar",
    "abarrajaras": "abarrajar",
    "abarrajare": "abarrajar",
    "abarrajareis": "abarrajar",
    "abarrajaremos": "abarrajar",
    "abarrajaren": "abarrajar",
    "abarrajares": "abarrajar",
    "abarrajaron": "abarrajar",
    "abarrajará": "abarrajar",
    "abarrajarán": "abarrajar",
    "abarrajarás": "abarrajar",
    "abarrajaré": "abarrajar",
    "abarrajaréis": "abarrajar",
    "abarrajaría": "abarrajar",
    "abarrajaríais": "abarrajar",
    "abarrajaríamos": "abarrajar",
    "abarrajarían": "abarrajar",
    "abarrajarías": "abarrajar",
    "abarrajas": "abarrajar",
    "abarrajase": "abarrajar",
    "abarrajaseis": "abarrajar",
    "abarrajasen": "abarrajar",
    "abarrajases": "abarrajar",
    "abarrajaste": "abarrajar",
    "abarrajasteis": "abarrajar",
    "abarraje": "abarrajar",
    "abarrajemos": "abarrajar",
    "abarrajen": "abarrajar",
    "abarrajes": "abarrajar",
    "abarrajo": "abarrajar",
    "abarrajábamos": "abarrajar",
    "abarrajáis": "abarrajar",
    "abarrajáramos": "abarrajar",
    "abarrajáremos": "abarrajar",
    "abarrajásemos": "abarrajar",
    "abarrajé": "abarrajar",
    "abarrajéis": "abarrajar",
    "abarrajó": "abarrajar",
    "abarranca": "abarrancar",
    "abarrancaba": "abarrancar",
    "abarrancabais": "abarrancar",
    "abarrancaban": "abarrancar",
    "abarrancabas": "abarrancar",
    "abarrancad": "abarrancar",
    "abarrancada": "abarrancar",
    "abarrancadas": "abarrancar",
    "abarrancaderos": "abarrancadero",
    "abarrancado": "abarrancar",
    "abarrancados": "abarrancar",
    "abarrancamientos": "abarrancamiento",
    "abarrancamos": "abarrancar",
    "abarrancan": "abarrancar",
    "abarrancando": "abarrancar",
    "abarrancara": "abarrancar",
    "abarrancarais": "abarrancar",
    "abarrancaran": "abarrancar",
    "abarrancaras": "abarrancar",
    "abarrancare": "abarrancar",
    "abarrancareis": "abarrancar",
    "abarrancaremos": "abarrancar",
    "abarrancaren": "abarrancar",
    "abarrancares": "abarrancar",
    "abarrancaron": "abarrancar",
    "abarrancará": "abarrancar",
    "abarrancarán": "abarrancar",
    "abarrancarás": "abarrancar",
    "abarrancaré": "abarrancar",
    "abarrancaréis": "abarrancar",
    "abarrancaría": "abarrancar",
    "abarrancaríais": "abarrancar",
    "abarrancaríamos": "abarrancar",
    "abarrancarían": "abarrancar",
    "abarrancarías": "abarrancar",
    "abarrancas": "abarrancar",
    "abarrancase": "abarrancar",
    "abarrancaseis": "abarrancar",
    "abarrancasen": "abarrancar",
    "abarrancases": "abarrancar",
    "abarrancaste": "abarrancar",
    "abarrancasteis": "abarrancar",
    "abarranco": "abarrancar",
    "abarrancábamos": "abarrancar",
    "abarrancáis": "abarrancar",
    "abarrancáramos": "abarrancar",
    "abarrancáremos": "abarrancar",
    "abarrancásemos": "abarrancar",
    "abarrancó": "abarrancar",
    "abarranque": "abarrancar",
    "abarranquemos": "abarrancar",
    "abarranquen": "abarrancar",
    "abarranques": "abarrancar",
    "abarranqué": "abarrancar",
    "abarranquéis": "abarrancar",
    "abarraque": "abarracar",
    "abarraquemos": "abarracar",
    "abarraquen": "abarracar",
    "abarraques": "abarracar",
    "abarraqué": "abarracar",
    "abarraquéis": "abarracar",
    "abarrota": "abarrotar",
    "abarrotaba": "abarrotar",
    "abarrotabais": "abarrotar",
    "abarrotaban": "abarrotar",
    "abarrotabas": "abarrotar",
    "abarrotad": "abarrotar",
    "abarrotada": "abarrotar",
    "abarrotadas": "abarrotar",
    "abarrotado": "abarrotar",
    "abarrotados": "abarrotar",
    "abarrotamientos": "abarrotamiento",
    "abarrotamos": "abarrotar",
    "abarrotan": "abarrotar",
    "abarrotando": "abarrotar",
    "abarrotara": "abarrotar",
    "abarrotarais": "abarrotar",
    "abarrotaran": "abarrotar",
    "abarrotaras": "abarrotar",
    "abarrotare": "abarrotar",
    "abarrotareis": "abarrotar",
    "abarrotaremos": "abarrotar",
    "abarrotaren": "abarrotar",
    "abarrotares": "abarrotar",
    "abarrotaron": "abarrotar",
    "abarrotará": "abarrotar",
    "abarrotarán": "abarrotar",
    "abarrotarás": "abarrotar",
    "abarrotaré": "abarrotar",
    "abarrotaréis": "abarrotar",
    "abarrotaría": "abarrotar",
    "abarrotaríais": "abarrotar",
    "abarrotaríamos": "abarrotar",
    "abarrotarían": "abarrotar",
    "abarrotarías": "abarrotar",
    "abarrotas": "abarrotar",
    "abarrotase": "abarrotar",
    "abarrotaseis": "abarrotar",
    "abarrotasen": "abarrotar",
    "abarrotases": "abarrotar",
    "abarrotaste": "abarrotar",
    "abarrotasteis": "abarrotar",
    "abarrote": "abarrotar",
    "abarrotemos": "abarrotar",
    "abarroten": "abarrotar",
    "abarrotes": "abarrote",
    "abarroto": "abarrotar",
    "abarrotábamos": "abarrotar",
    "abarrotáis": "abarrotar",
    "abarrotáramos": "abarrotar",
    "abarrotáremos": "abarrotar",
    "abarrotásemos": "abarrotar",
    "abarroté": "abarrotar",
    "abarrotéis": "abarrotar",
    "abarrotó": "abarrotar",
    "abará": "abar",
    "abarán": "abar",
    "abarás": "abar",
    "abaré": "abar",
    "abaréis": "abar",
    "abaría": "abar",
    "abaríais": "abar",
    "abaríamos": "abar",
    "abarían": "abar",
    "abarías": "abar",
    "abas": "abar",
    "abase": "abar",
    "abaseis": "abar",
    "abasen": "abar",
    "abases": "abar",
    "abasidas": "abasida",
    "abasta": "abastar",
    "abastaba": "abastar",
    "abastabais": "abastar",
    "abastaban": "abastar",
    "abastabas": "abastar",
    "abastad": "abastar",
    "abastada": "abastar",
    "abastadas": "abastar",
    "abastado": "abastar",
    "abastados": "abastar",
    "abastamientos": "abastamiento",
    "abastamos": "abastar",
    "abastan": "abastar",
    "abastando": "abastar",
    "abastara": "abastar",
    "abastarais": "abastar",
    "abastaran": "abastar",
    "abastaras": "abastar",
    "abastare": "abastar",
    "abastareis": "abastar",
    "abastaremos": "abastar",
    "abastaren": "abastar",
    "abastares": "abastar",
    "abastaron": "abastar",
    "abastará": "abastar",
    "abastarán": "abastar",
    "abastarás": "abastar",
    "abastaré": "abastar",
    "abastaréis": "abastar",
    "abastaría": "abastar",
    "abastaríais": "abastar",
    "abastaríamos": "abastar",
    "abastarían": "abastar",
    "abastarías": "abastar",
    "abastas": "abastar",
    "abastase": "abastar",
    "abastaseis": "abastar",
    "abastasen": "abastar",
    "abastases": "abastar",
    "abastaste": "abastar",
    "abastasteis": "abastar",
    "abaste": "abastar",
    "abastece": "abastecer",
    "abasteced": "abastecer",
    "abastecedora": "abastecedor",
    "abastecedoras": "abastecedor",
    "abastecedores": "abastecedor",
    "abastecemos": "abastecer",
    "abastecen": "abastecer",
    "abasteceremos": "abastecer",
    "abastecerá": "abastecer",
    "abastecerán": "abastecer",
    "abastecerás": "abastecer",
    "abasteceré": "abastecer",
    "abasteceréis": "abastecer",
    "abastecería": "abastecer",
    "abasteceríais": "abastecer",
    "abasteceríamos": "abastecer",
    "abastecerían": "abastecer",
    "abastecerías": "abastecer",
    "abasteces": "abastecer",
    "abastecida": "abastecer",
    "abastecidas": "abastecer",
    "abastecido": "abastecer",
    "abastecidos": "abastecer",
    "abasteciendo": "abastecer",
    "abasteciera": "abastecer",
    "abastecierais": "abastecer",
    "abastecieran": "abastecer",
    "abastecieras": "abastecer",
    "abasteciere": "abastecer",
    "abasteciereis": "abastecer",
    "abastecieren": "abastecer",
    "abastecieres": "abastecer",
    "abastecieron": "abastecer",
    "abasteciese": "abastecer",
    "abastecieseis": "abastecer",
    "abasteciesen": "abastecer",
    "abastecieses": "abastecer",
    "abastecimientos": "abastecimiento",
    "abastecimos": "abastecer",
    "abasteciste": "abastecer",
    "abastecisteis": "abastecer",
    "abasteciéramos": "abastecer",
    "abasteciéremos": "abastecer",
    "abasteciésemos": "abastecer",
    "abasteció": "abastecer",
    "abastecéis": "abastecer",
    "abastecí": "abastecer",
    "abastecía": "abastecer",
    "abastecíais": "abastecer",
    "abastecíamos": "abastecer",
    "abastecían": "abastecer",
    "abastecías": "abastecer",
    "abasteis": "abar",
    "abastemos": "abastar",
    "abasten": "abastar",
    "abastera": "abastero",
    "abasteras": "abastero",
    "abasteros": "abastero",
    "abastes": "abastar",
    "abastezca": "abastecer",
    "abastezcamos": "abastecer",
    "abastezcan": "abastecer",
    "abastezcas": "abastecer",
    "abastezco": "abastecer",
    "abastezcáis": "abastecer",
    "abastiona": "abastionar",
    "abastionaba": "abastionar",
    "abastionabais": "abastionar",
    "abastionaban": "abastionar",
    "abastionabas": "abastionar",
    "abastionad": "abastionar",
    "abastionada": "abastionar",
    "abastionadas": "abastionar",
    "abastionado": "abastionar",
    "abastionados": "abastionar",
    "abastionamos": "abastionar",
    "abastionan": "abastionar",
    "abastionando": "abastionar",
    "abastionara": "abastionar",
    "abastionarais": "abastionar",
    "abastionaran": "abastionar",
    "abastionaras": "abastionar",
    "abastionare": "abastionar",
    "abastionareis": "abastionar",
    "abastionaremos": "abastionar",
    "abastionaren": "abastionar",
    "abastionares": "abastionar",
    "abastionaron": "abastionar",
    "abastionará": "abastionar",
    "abastionarán": "abastionar",
    "abastionarás": "abastionar",
    "abastionaré": "abastionar",
    "abastionaréis": "abastionar",
    "abastionaría": "abastionar",
    "abastionaríais": "abastionar",
    "abastionaríamos": "abastionar",
    "abastionarían": "abastionar",
    "abastionarías": "abastionar",
    "abastionas": "abastionar",
    "abastionase": "abastionar",
    "abastionaseis": "abastionar",
    "abastionasen": "abastionar",
    "abastionases": "abastionar",
    "abastionaste": "abastionar",
    "abastionasteis": "abastionar",
    "abastione": "abastionar",
    "abastionemos": "abastionar",
    "abastionen": "abastionar",
    "abastiones": "abastionar",
    "abastiono": "abastionar",
    "abastionábamos": "abastionar",
    "abastionáis": "abastionar",
    "abastionáramos": "abastionar",
    "abastionáremos": "abastionar",
    "abastionásemos": "abastionar",
    "abastioné": "abastionar",
    "abastionéis": "abastionar",
    "abastionó": "abastionar",
    "abasto": "abastar",
    "abastábamos": "abastar",
    "abastáis": "abastar",
    "abastáramos": "abastar",
    "abastáremos": "abastar",
    "abastásemos": "abastar",
    "abasté": "abastar",
    "abastéis": "abastar",
    "abastó": "abastar",
    "abasíes": "abasí",
    "abasís": "abasí",
    "abata": "abatir",
    "abatamos": "abatir",
    "abatan": "abatir",
    "abatana": "abatanar",
    "abatanaba": "abatanar",
    "abatanabais": "abatanar",
    "abatanaban": "abatanar",
    "abatanabas": "abatanar",
    "abatanad": "abatanar",
    "abatanada": "abatanar",
    "abatanadas": "abatanar",
    "abatanado": "abatanar",
    "abatanados": "abatanar",
    "abatanamos": "abatanar",
    "abatanan": "abatanar",
    "abatanando": "abatanar",
    "abatanara": "abatanar",
    "abatanarais": "abatanar",
    "abatanaran": "abatanar",
    "abatanaras": "abatanar",
    "abatanare": "abatanar",
    "abatanareis": "abatanar",
    "abatanaremos": "abatanar",
    "abatanaren": "abatanar",
    "abatanares": "abatanar",
    "abatanaron": "abatanar",
    "abatanará": "abatanar",
    "abatanarán": "abatanar",
    "abatanarás": "abatanar",
    "abatanaré": "abatanar",
    "abatanaréis": "abatanar",
    "abatanaría": "abatanar",
    "abatanaríais": "abatanar",
    "abatanaríamos": "abatanar",
    "abatanarían": "abatanar",
    "abatanarías": "abatanar",
    "abatanas": "abatanar",
    "abatanase": "abatanar",
    "abatanaseis": "abatanar",
    "abatanasen": "abatanar",
    "abatanases": "abatanar",
    "abatanaste": "abatanar",
    "abatanasteis": "abatanar",
    "abatane": "abatanar",
    "abatanemos": "abatanar",
    "abatanen": "abatanar",
    "abatanes": "abatanar",
    "abatano": "abatanar",
    "abatanábamos": "abatanar",
    "abatanáis": "abatanar",
    "abatanáramos": "abatanar",
    "abatanáremos": "abatanar",
    "abatanásemos": "abatanar",
    "abatané": "abatanar",
    "abatanéis": "abatanar",
    "abatanó": "abatanar",
    "abatas": "abatir",
    "abatata": "abatatar",
    "abatataba": "abatatar",
    "abatatabais": "abatatar",
    "abatataban": "abatatar",
    "abatatabas": "abatatar",
    "abatatad": "abatatar",
    "abatatada": "abatatar",
    "abatatadas": "abatatar",
    "abatatado": "abatatar",
    "abatatados": "abatatar",
    "abatatamos": "abatatar",
    "abatatan": "abatatar",
    "abatatando": "abatatar",
    "abatatara": "abatatar",
    "abatatarais": "abatatar",
    "abatataran": "abatatar",
    "abatataras": "abatatar",
    "abatatare": "abatatar",
    "abatatareis": "abatatar",
    "abatataremos": "abatatar",
    "abatataren": "abatatar",
    "abatatares": "abatatar",
    "abatataron": "abatatar",
    "abatatará": "abatatar",
    "abatatarán": "abatatar",
    "abatatarás": "abatatar",
    "abatataré": "abatatar",
    "abatataréis": "abatatar",
    "abatataría": "abatatar",
    "abatataríais": "abatatar",
    "abatataríamos": "abatatar",
    "abatatarían": "abatatar",
    "abatatarías": "abatatar",
    "abatatas": "abatatar",
    "abatatase": "abatatar",
    "abatataseis": "abatatar",
    "abatatasen": "abatatar",
    "abatatases": "abatatar",
    "abatataste": "abatatar",
    "abatatasteis": "abatatar",
    "abatate": "abatatar",
    "abatatemos": "abatatar",
    "abataten": "abatatar",
    "abatates": "abatatar",
    "abatato": "abatatar",
    "abatatábamos": "abatatar",
    "abatatáis": "abatatar",
    "abatatáramos": "abatatar",
    "abatatáremos": "abatatar",
    "abatatásemos": "abatatar",
    "abataté": "abatatar",
    "abatatéis": "abatatar",
    "abatató": "abatatar",
    "abate": "abatir",
    "abatelenguas": "abatelengua",
    "abaten": "abatir",
    "abates": "abatir",
    "abatibles": "abatible",
    "abatid": "abatir",
    "abatida": "abatir",
    "abatidas": "abatir",
    "abatido": "abatir",
    "abatidos": "abatir",
    "abatiendo": "abatir",
    "abatiera": "abatir",
    "abatierais": "abatir",
    "abatieran": "abatir",
    "abatieras": "abatir",
    "abatiere": "abatir",
    "abatiereis": "abatir",
    "abatieren": "abatir",
    "abatieres": "abatir",
    "abatieron": "abatir",
    "abatiese": "abatir",
    "abatieseis": "abatir",
    "abatiesen": "abatir",
    "abatieses": "abatir",
    "abatimientos": "abatimiento",
    "abatimos": "abatir",
    "abatiremos": "abatir",
    "abatirá": "abatir",
    "abatirán": "abatir",
    "abatirás": "abatir",
    "abatiré": "abatir",
    "abatiréis": "abatir",
    "abatiría": "abatir",
    "abatiríais": "abatir",
    "abatiríamos": "abatir",
    "abatirían": "abatir",
    "abatirías": "abatir",
    "abatiste": "abatir",
    "abatisteis": "abatir",
    "abatiéramos": "abatir",
    "abatiéremos": "abatir",
    "abatiésemos": "abatir",
    "abatió": "abatir",
    "abato": "abatir",
    "abatoja": "abatojar",
    "abatojaba": "abatojar",
    "abatojabais": "abatojar",
    "abatojaban": "abatojar",
    "abatojabas": "abatojar",
    "abatojad": "abatojar",
    "abatojada": "abatojar",
    "abatojadas": "abatojar",
    "abatojado": "abatojar",
    "abatojados": "abatojar",
    "abatojamos": "abatojar",
    "abatojan": "abatojar",
    "abatojando": "abatojar",
    "abatojara": "abatojar",
    "abatojarais": "abatojar",
    "abatojaran": "abatojar",
    "abatojaras": "abatojar",
    "abatojare": "abatojar",
    "abatojareis": "abatojar",
    "abatojaremos": "abatojar",
    "abatojaren": "abatojar",
    "abatojares": "abatojar",
    "abatojaron": "abatojar",
    "abatojará": "abatojar",
    "abatojarán": "abatojar",
    "abatojarás": "abatojar",
    "abatojaré": "abatojar",
    "abatojaréis": "abatojar",
    "abatojaría": "abatojar",
    "abatojaríais": "abatojar",
    "abatojaríamos": "abatojar",
    "abatojarían": "abatojar",
    "abatojarías": "abatojar",
    "abatojas": "abatojar",
    "abatojase": "abatojar",
    "abatojaseis": "abatojar",
    "abatojasen": "abatojar",
    "abatojases": "abatojar",
    "abatojaste": "abatojar",
    "abatojasteis": "abatojar",
    "abatoje": "abatojar",
    "abatojemos": "abatojar",
    "abatojen": "abatojar",
    "abatojes": "abatojar",
    "abatojo": "abatojar",
    "abatojábamos": "abatojar",
    "abatojáis": "abatojar",
    "abatojáramos": "abatojar",
    "abatojáremos": "abatojar",
    "abatojásemos": "abatojar",
    "abatojé": "abatojar",
    "abatojéis": "abatojar",
    "abatojó": "abatojar",
    "abatáis": "abatir",
    "abatí": "abatir",
    "abatía": "abatir",
    "abatíais": "abatir",
    "abatíamos": "abatir",
    "abatían": "abatir",
    "abatías": "abatir",
    "abatíes": "abatí",
    "abatís": "abatir",
    "abaxiales": "abaxial",
    "abayada": "abayado",
    "abayadas": "abayado",
    "abayados": "abayado",
    "abazones": "abazón",
    "abaña": "abañar",
    "abañaba": "abañar",
    "abañabais": "abañar",
    "abañaban": "abañar",
    "abañabas": "abañar",
    "abañad": "abañar",
    "abañada": "abañar",
    "abañadas": "abañar",
    "abañado": "abañar",
    "abañadora": "abañador",
    "abañadoras": "abañador",
    "abañadores": "abañador",
    "abañados": "abañar",
    "abañaduras": "abañadura",
    "abañamos": "abañar",
    "abañan": "abañar",
    "abañando": "abañar",
    "abañara": "abañar",
    "abañarais": "abañar",
    "abañaran": "abañar",
    "abañaras": "abañar",
    "abañare": "abañar",
    "abañareis": "abañar",
    "abañaremos": "abañar",
    "abañaren": "abañar",
    "abañares": "abañar",
    "abañaron": "abañar",
    "abañará": "abañar",
    "abañarán": "abañar",
    "abañarás": "abañar",
    "abañaré": "abañar",
    "abañaréis": "abañar",
    "abañaría": "abañar",
    "abañaríais": "abañar",
    "abañaríamos": "abañar",
    "abañarían": "abañar",
    "abañarías": "abañar",
    "abañas": "abañar",
    "abañase": "abañar",
    "abañaseis": "abañar",
    "abañasen": "abañar",
    "abañases": "abañar",
    "abañaste": "abañar",
    "abañasteis": "abañar",
    "abañe": "abañar",
    "abañemos": "abañar",
    "abañen": "abañar",
    "abañes": "abañar",
    "abaño": "abañar",
    "abañábamos": "abañar",
    "abañáis": "abañar",
    "abañáramos": "abañar",
    "abañáremos": "abañar",
    "abañásemos": "abañar",
    "abañé": "abañar",
    "abañéis": "abañar",
    "abañó": "abañar",
    "abbadíes": "abbadí",
    "abbadís": "abbadí",
    "abbas": "abba",
    "abbasíes": "abbasí",
    "abbasís": "abbasí",
    "abbevillienses": "abbevilliense",
    "abdalwadíes": "abdalwadí",
    "abdalwadís": "abdalwadí",
    "abdica": "abdicar",
    "abdicaba": "abdicar",
    "abdicabais": "abdicar",
    "abdicaban": "abdicar",
    "abdicabas": "abdicar",
    "abdicaciones": "abdicación",
    "abdicad": "abdicar",
    "abdicada": "abdicar",
    "abdicadas": "abdicar",
    "abdicado": "abdicar",
    "abdicados": "abdicar",
    "abdicamos": "abdicar",
    "abdican": "abdicar",
    "abdicando": "abdicar",
    "abdicara": "abdicar",
    "abdicarais": "abdicar",
    "abdicaran": "abdicar",
    "abdicaras": "abdicar",
    "abdicare": "abdicar",
    "abdicareis": "abdicar",
    "abdicaremos": "abdicar",
    "abdicaren": "abdicar",
    "abdicares": "abdicar",
    "abdicaron": "abdicar",
    "abdicará": "abdicar",
    "abdicarán": "abdicar",
    "abdicarás": "abdicar",
    "abdicaré": "abdicar",
    "abdicaréis": "abdicar",
    "abdicaría": "abdicar",
    "abdicaríais": "abdicar",
    "abdicaríamos": "abdicar",
    "abdicarían": "abdicar",
    "abdicarías": "abdicar",
    "abdicas": "abdicar",
    "abdicase": "abdicar",
    "abdicaseis": "abdicar",
    "abdicasen": "abdicar",
    "abdicases": "abdicar",
    "abdicaste": "abdicar",
    "abdicasteis": "abdicar",
    "abdico": "abdicar",
    "abdicábamos": "abdicar",
    "abdicáis": "abdicar",
    "abdicáramos": "abdicar",
    "abdicáremos": "abdicar",
    "abdicásemos": "abdicar",
    "abdicó": "abdicar",
    "abdique": "abdicar",
    "abdiquemos": "abdicar",
    "abdiquen": "abdicar",
    "abdiques": "abdicar",
    "abdiqué": "abdicar",
    "abdiquéis": "abdicar",
    "abdominales": "abdominal",
    "abdominogenitales": "abdominogenital",
    "abducciones": "abducción",
    "abductora": "abductor",
    "abductoras": "abductor",
    "abductores": "abductor",
    "abdómenes": "abdomen",
    "abe": "abar",
    "abecedarios": "abecedario",
    "abecés": "abecé",
    "abedules": "abedul",
    "abejares": "abejar",
    "abejarucos": "abejaruco",
    "abejas": "abeja",
    "abejera": "abejero",
    "abejeras": "abejero",
    "abejeros": "abejero",
    "abejones": "abejón",
    "abejorros": "abejorro",
    "abeliana": "abeliano",
    "abelianas": "abeliano",
    "abelianos": "abeliano",
    "abelitos": "abelito",
    "abellaca": "abellacar",
    "abellacaba": "abellacar",
    "abellacabais": "abellacar",
    "abellacaban": "abellacar",
    "abellacabas": "abellacar",
    "abellacad": "abellacar",
    "abellacada": "abellacar",
    "abellacadas": "abellacar",
    "abellacado": "abellacar",
    "abellacados": "abellacar",
    "abellacamos": "abellacar",
    "abellacan": "abellacar",
    "abellacando": "abellacar",
    "abellacara": "abellacar",
    "abellacarais": "abellacar",
    "abellacaran": "abellacar",
    "abellacaras": "abellacar",
    "abellacare": "abellacar",
    "abellacareis": "abellacar",
    "abellacaremos": "abellacar",
    "abellacaren": "abellacar",
    "abellacares": "abellacar",
    "abellacaron": "abellacar",
    "abellacará": "abellacar",
    "abellacarán": "abellacar",
    "abellacarás": "abellacar",
    "abellacaré": "abellacar",
    "abellacaréis": "abellacar",
    "abellacaría": "abellacar",
    "abellacaríais": "abellacar",
    "abellacaríamos": "abellacar",
    "abellacarían": "abellacar",
    "abellacarías": "abellacar",
    "abellacas": "abellacar",
    "abellacase": "abellacar",
    "abellacaseis": "abellacar",
    "abellacasen": "abellacar",
    "abellacases": "abellacar",
    "abellacaste": "abellacar",
    "abellacasteis": "abellacar",
    "abellaco": "abellacar",
    "abellacábamos": "abellacar",
    "abellacáis": "abellacar",
    "abellacáramos": "abellacar",
    "abellacáremos": "abellacar",
    "abellacásemos": "abellacar",
    "abellacó": "abellacar",
    "abellaque": "abellacar",
    "abellaquemos": "abellacar",
    "abellaquen": "abellacar",
    "abellaques": "abellacar",
    "abellaqué": "abellacar",
    "abellaquéis": "abellacar",
    "abellotada": "abellotado",
    "abellotadas": "abellotado",
    "abellotados": "abellotado",
    "abelmoschos": "abelmoscho",
    "abemola": "abemolar",
    "abemolaba": "abemolar",
    "abemolabais": "abemolar",
    "abemolaban": "abemolar",
    "abemolabas": "abemolar",
    "abemolad": "abemolar",
    "abemolada": "abemolar",
    "abemoladas": "abemolar",
    "abemolado": "abemolar",
    "abemolados": "abemolar",
    "abemolamos": "abemolar",
    "abemolan": "abemolar",
    "abemolando": "abemolar",
    "abemolara": "abemolar",
    "abemolarais": "abemolar",
    "abemolaran": "abemolar",
    "abemolaras": "abemolar",
    "abemolare": "abemolar",
    "abemolareis": "abemolar",
    "abemolaremos": "abemolar",
    "abemolaren": "abemolar",
    "abemolares": "abemolar",
    "abemolaron": "abemolar",
    "abemolará": "abemolar",
    "abemolarán": "abemolar",
    "abemolarás": "abemolar",
    "abemolaré": "abemolar",
    "abemolaréis": "abemolar",
    "abemolaría": "abemolar",
    "abemolaríais": "abemolar",
    "abemolaríamos": "abemolar",
    "abemolarían": "abemolar",
    "abemolarías": "abemolar",
    "abemolas": "abemolar",
    "abemolase": "abemolar",
    "abemolaseis": "abemolar",
    "abemolasen": "abemolar",
    "abemolases": "abemolar",
    "abemolaste": "abemolar",
    "abemolasteis": "abemolar",
    "abemole": "abemolar",
    "abemolemos": "abemolar",
    "abemolen": "abemolar",
    "abemoles": "abemolar",
    "abemolo": "abemolar",
    "abemolábamos": "abemolar",
    "abemoláis": "abemolar",
    "abemoláramos": "abemolar",
    "abemoláremos": "abemolar",
    "abemolásemos": "abemolar",
    "abemolé": "abemolar",
    "abemoléis": "abemolar",
    "abemoló": "abemolar",
    "abemos": "abar",
    "aben": "abar",
    "abencerrajes": "abencerraje",
    "abenuces": "abenuz",
    "aberenjenada": "aberenjenado",
    "aberenjenadas": "aberenjenado",
    "aberenjenados": "aberenjenado",
    "aberraba": "aberrar",
    "aberrabais": "aberrar",
    "aberraban": "aberrar",
    "aberrabas": "aberrar",
    "aberraciones": "aberración",
    "aberrad": "aberrar",
    "aberrada": "aberrar",
    "aberradas": "aberrar",
    "aberrado": "aberrar",
    "aberrados": "aberrar",
    "aberramos": "aberrar",
    "aberrando": "aberrar",
    "aberrantes": "aberrante",
    "aberrara": "aberrar",
    "aberrarais": "aberrar",
    "aberraran": "aberrar",
    "aberraras": "aberrar",
    "aberrare": "aberrar",
    "aberrareis": "aberrar",
    "aberraremos": "aberrar",
    "aberraren": "aberrar",
    "aberrares": "aberrar",
    "aberraron": "aberrar",
    "aberrará": "aberrar",
    "aberrarán": "aberrar",
    "aberrarás": "aberrar",
    "aberraré": "aberrar",
    "aberraréis": "aberrar",
    "aberraría": "aberrar",
    "aberraríais": "aberrar",
    "aberraríamos": "aberrar",
    "aberrarían": "aberrar",
    "aberrarías": "aberrar",
    "aberrase": "aberrar",
    "aberraseis": "aberrar",
    "aberrasen": "aberrar",
    "aberrases": "aberrar",
    "aberraste": "aberrar",
    "aberrasteis": "aberrar",
    "aberratios": "aberratio",
    "aberremos": "aberrar",
    "aberrábamos": "aberrar",
    "aberráis": "aberrar",
    "aberráramos": "aberrar",
    "aberráremos": "aberrar",
    "aberrásemos": "aberrar",
    "aberré": "aberrar",
    "aberréis": "aberrar",
    "aberró": "aberrar",
    "abertales": "abertal",
    "aberturas": "abertura",
    "abertzales": "abertzale",
    "abes": "abar",
    "abetales": "abetal",
    "abetalipoproteinemias": "abetalipoproteinemia",
    "abetares": "abetar",
    "abetinotes": "abetinote",
    "abetos": "abeto",
    "abetuna": "abetunar",
    "abetunaba": "abetunar",
    "abetunabais": "abetunar",
    "abetunaban": "abetunar",
    "abetunabas": "abetunar",
    "abetunad": "abetunar",
    "abetunada": "abetunar",
    "abetunadas": "abetunar",
    "abetunado": "abetunar",
    "abetunados": "abetunar",
    "abetunamos": "abetunar",
    "abetunan": "abetunar",
    "abetunando": "abetunar",
    "abetunara": "abetunar",
    "abetunarais": "abetunar",
    "abetunaran": "abetunar",
    "abetunaras": "abetunar",
    "abetunare": "abetunar",
    "abetunareis": "abetunar",
    "abetunaremos": "abetunar",
    "abetunaren": "abetunar",
    "abetunares": "abetunar",
    "abetunaron": "abetunar",
    "abetunará": "abetunar",
    "abetunarán": "abetunar",
    "abetunarás": "abetunar",
    "abetunaré": "abetunar",
    "abetunaréis": "abetunar",
    "abetunaría": "abetunar",
    "abetunaríais": "abetunar",
    "abetunaríamos": "abetunar",
    "abetunarían": "abetunar",
    "abetunarías": "abetunar",
    "abetunas": "abetunar",
    "abetunase": "abetunar",
    "abetunaseis": "abetunar",
    "abetunasen": "abetunar",
    "abetunases": "abetunar",
    "abetunaste": "abetunar",
    "abetunasteis": "abetunar",
    "abetune": "abetunar",
    "abetunemos": "abetunar",
    "abetunen": "abetunar",
    "abetunes": "abetunar",
    "abetuno": "abetunar",
    "abetunábamos": "abetunar",
    "abetunáis": "abetunar",
    "abetunáramos": "abetunar",
    "abetunáremos": "abetunar",
    "abetunásemos": "abetunar",
    "abetuné": "abetunar",
    "abetunéis": "abetunar",
    "abetunó": "abetunar",
    "abeyes": "abey",
    "abierra": "aberrar",
    "abierran": "aberrar",
    "abierras": "aberrar",
    "abierre": "aberrar",
    "abierren": "aberrar",
    "abierres": "aberrar",
    "abierro": "aberrar",
    "abierta": "abrir",
    "abiertas": "abrir",
    "abierto": "abrir",
    "abiertos": "abrir",
    "abietenos": "abieteno",
    "abietinatos": "abietinato",
    "abietáceas": "abietácea",
    "abigarra": "abigarrar",
    "abigarraba": "abigarrar",
    "abigarrabais": "abigarrar",
    "abigarraban": "abigarrar",
    "abigarrabas": "abigarrar",
    "abigarrad": "abigarrar",
    "abigarrada": "abigarrar",
    "abigarradas": "abigarrar",
    "abigarrado": "abigarrar",
    "abigarrados": "abigarrar",
    "abigarramientos": "abigarramiento",
    "abigarramos": "abigarrar",
    "abigarran": "abigarrar",
    "abigarrando": "abigarrar",
    "abigarrara": "abigarrar",
    "abigarrarais": "abigarrar",
    "abigarraran": "abigarrar",
    "abigarraras": "abigarrar",
    "abigarrare": "abigarrar",
    "abigarrareis": "abigarrar",
    "abigarraremos": "abigarrar",
    "abigarraren": "abigarrar",
    "abigarrares": "abigarrar",
    "abigarraron": "abigarrar",
    "abigarrará": "abigarrar",
    "abigarrarán": "abigarrar",
    "abigarrarás": "abigarrar",
    "abigarraré": "abigarrar",
    "abigarraréis": "abigarrar",
    "abigarraría": "abigarrar",
    "abigarraríais": "abigarrar",
    "abigarraríamos": "abigarrar",
    "abigarrarían": "abigarrar",
    "abigarrarías": "abigarrar",
    "abigarras": "abigarrar",
    "abigarrase": "abigarrar",
    "abigarraseis": "abigarrar",
    "abigarrasen": "abigarrar",
    "abigarrases": "abigarrar",
    "abigarraste": "abigarrar",
    "abigarrasteis": "abigarrar",
    "abigarre": "abigarrar",
    "abigarremos": "abigarrar",
    "abigarren": "abigarrar",
    "abigarres": "abigarrar",
    "abigarro": "abigarrar",
    "abigarrábamos": "abigarrar",
    "abigarráis": "abigarrar",
    "abigarráramos": "abigarrar",
    "abigarráremos": "abigarrar",
    "abigarrásemos": "abigarrar",
    "abigarré": "abigarrar",
    "abigarréis": "abigarrar",
    "abigarró": "abigarrar",
    "abigeatos": "abigeato",
    "abigeos": "abigeo",
    "abintestatos": "abintestato",
    "abipones": "abipón",
    "abisales": "abisal",
    "abisinia": "abisinio",
    "abisinias": "abisinio",
    "abisinios": "abisinio",
    "abisma": "abismar",
    "abismaba": "abismar",
    "abismabais": "abismar",
    "abismaban": "abismar",
    "abismabas": "abismar",
    "abismad": "abismar",
    "abismada": "abismar",
    "abismadas": "abismar",
    "abismado": "abismar",
    "abismados": "abismar",
    "abismales": "abismal",
    "abismamos": "abismar",
    "abisman": "abismar",
    "abismando": "abismar",
    "abismara": "abismar",
    "abismarais": "abismar",
    "abismaran": "abismar",
    "abismaras": "abismar",
    "abismare": "abismar",
    "abismareis": "abismar",
    "abismaremos": "abismar",
    "abismaren": "abismar",
    "abismares": "abismar",
    "abismaron": "abismar",
    "abismará": "abismar",
    "abismarán": "abismar",
    "abismarás": "abismar",
    "abismaré": "abismar",
    "abismaréis": "abismar",
    "abismaría": "abismar",
    "abismaríais": "abismar",
    "abismaríamos": "abismar",
    "abismarían": "abismar",
    "abismarías": "abismar",
    "abismas": "abismar",
    "abismase": "abismar",
    "abismaseis": "abismar",
    "abismasen": "abismar",
    "abismases": "abismar",
    "abismaste": "abismar",
    "abismasteis": "abismar",
    "abisme": "abismar",
    "abismemos": "abismar",
    "abismen": "abismar",
    "abismes": "abismar",
    "abismo": "abismar",
    "abismos": "abismo",
    "abismábamos": "abismar",
    "abismáis": "abismar",
    "abismáramos": "abismar",
    "abismáremos": "abismar",
    "abismásemos": "abismar",
    "abismática": "abismático",
    "abismáticas": "abismático",
    "abismáticos": "abismático",
    "abismé": "abismar",
    "abisméis": "abismar",
    "abismó": "abismar",
    "abita": "abitar",
    "abitaba": "abitar",
    "abitabais": "abitar",
    "abitaban": "abitar",
    "abitabas": "abitar",
    "abitad": "abitar",
    "abitada": "abitar",
    "abitadas": "abitar",
    "abitado": "abitar",
    "abitados": "abitar",
    "abitaduras": "abitadura",
    "abitamos": "abitar",
    "abitan": "abitar",
    "abitando": "abitar",
    "abitara": "abitar",
    "abitarais": "abitar",
    "abitaran": "abitar",
    "abitaras": "abitar",
    "abitare": "abitar",
    "abitareis": "abitar",
    "abitaremos": "abitar",
    "abitaren": "abitar",
    "abitares": "abitar",
    "abitaron": "abitar",
    "abitará": "abitar",
    "abitarán": "abitar",
    "abitarás": "abitar",
    "abitaré": "abitar",
    "abitaréis": "abitar",
    "abitaría": "abitar",
    "abitaríais": "abitar",
    "abitaríamos": "abitar",
    "abitarían": "abitar",
    "abitarías": "abitar",
    "abitas": "abitar",
    "abitase": "abitar",
    "abitaseis": "abitar",
    "abitasen": "abitar",
    "abitases": "abitar",
    "abitaste": "abitar",
    "abitasteis": "abitar",
    "abite": "abitar",
    "abitemos": "abitar",
    "abiten": "abitar",
    "abites": "abitar",
    "abito": "abitar",
    "abitones": "abitón",
    "abitábamos": "abitar",
    "abitáis": "abitar",
    "abitáramos": "abitar",
    "abitáremos": "abitar",
    "abitásemos": "abitar",
    "abité": "abitar",
    "abitéis": "abitar",
    "abitó": "abitar",
    "abiuretos": "abiureto",
    "abizcochada": "abizcochado",
    "abizcochadas": "abizcochado",
    "abizcochados": "abizcochado",
    "abiética": "abiético",
    "abiéticas": "abiético",
    "abiéticos": "abiético",
    "abiótica": "abiótico",
    "abióticas": "abiótico",
    "abióticos": "abiótico",
    "abjura": "abjurar",
    "abjuraba": "abjurar",
    "abjurabais": "abjurar",
    "abjuraban": "abjurar",
    "abjurabas": "abjurar",
    "abjuraciones": "abjuración",
    "abjurad": "abjurar",
    "abjurada": "abjurar",
    "abjuradas": "abjurar",
    "abjurado": "abjurar",
    "abjurados": "abjurar",
    "abjuramos": "abjurar",
    "abjuran": "abjurar",
    "abjurando": "abjurar",
    "abjurara": "abjurar",
    "abjurarais": "abjurar",
    "abjuraran": "abjurar",
    "abjuraras": "abjurar",
    "abjurare": "abjurar",
    "abjurareis": "abjurar",
    "abjuraremos": "abjurar",
    "abjuraren": "abjurar",
    "abjurares": "abjurar",
    "abjuraron": "abjurar",
    "abjurará": "abjurar",
    "abjurarán": "abjurar",
    "abjurarás": "abjurar",
    "abjuraré": "abjurar",
    "abjuraréis": "abjurar",
    "abjuraría": "abjurar",
    "abjuraríais": "abjurar",
    "abjuraríamos": "abjurar",
    "abjurarían": "abjurar",
    "abjurarías": "abjurar",
    "abjuras": "abjurar",
    "abjurase": "abjurar",
    "abjuraseis": "abjurar",
    "abjurasen": "abjurar",
    "abjurases": "abjurar",
    "abjuraste": "abjurar",
    "abjurasteis": "abjurar",
    "abjure": "abjurar",
    "abjuremos": "abjurar",
    "abjuren": "abjurar",
    "abjures": "abjurar",
    "abjuro": "abjurar",
    "abjurábamos": "abjurar",
    "abjuráis": "abjurar",
    "abjuráramos": "abjurar",
    "abjuráremos": "abjurar",
    "abjurásemos": "abjurar",
    "abjuré": "abjurar",
    "abjuréis": "abjurar",
    "abjuró": "abjurar",
    "ablaciones": "ablación",
    "ablactaciones": "ablactación",
    "ablanda": "ablandar",
    "ablandaba": "ablandar",
    "ablandabais": "ablandar",
    "ablandaban": "ablandar",
    "ablandabas": "ablandar",
    "ablandad": "ablandar",
    "ablandada": "ablandar",
    "ablandadas": "ablandar",
    "ablandado": "ablandar",
    "ablandadora": "ablandador",
    "ablandadoras": "ablandador",
    "ablandadores": "ablandador",
    "ablandados": "ablandar",
    "ablandaduras": "ablandadura",
    "ablandamientos": "ablandamiento",
    "ablandamos": "ablandar",
    "ablandan": "ablandar",
    "ablandando": "ablandar",
    "ablandantes": "ablandante",
    "ablandara": "ablandar",
    "ablandarais": "ablandar",
    "ablandaran": "ablandar",
    "ablandaras": "ablandar",
    "ablandare": "ablandar",
    "ablandareis": "ablandar",
    "ablandaremos": "ablandar",
    "ablandaren": "ablandar",
    "ablandares": "ablandar",
    "ablandaron": "ablandar",
    "ablandará": "ablandar",
    "ablandarán": "ablandar",
    "ablandarás": "ablandar",
    "ablandaré": "ablandar",
    "ablandaréis": "ablandar",
    "ablandaría": "ablandar",
    "ablandaríais": "ablandar",
    "ablandaríamos": "ablandar",
    "ablandarían": "ablandar",
    "ablandarías": "ablandar",
    "ablandas": "ablandar",
    "ablandase": "ablandar",
    "ablandaseis": "ablandar",
    "ablandasen": "ablandar",
    "ablandases": "ablandar",
    "ablandaste": "ablandar",
    "ablandasteis": "ablandar",
    "ablande": "ablandar",
    "ablandece": "ablandecer",
    "ablandeced": "ablandecer",
    "ablandecemos": "ablandecer",
    "ablandecen": "ablandecer",
    "ablandeceremos": "ablandecer",
    "ablandecerá": "ablandecer",
    "ablandecerán": "ablandecer",
    "ablandecerás": "ablandecer",
    "ablandeceré": "ablandecer",
    "ablandeceréis": "ablandecer",
    "ablandecería": "ablandecer",
    "ablandeceríais": "ablandecer",
    "ablandeceríamos": "ablandecer",
    "ablandecerían": "ablandecer",
    "ablandecerías": "ablandecer",
    "ablandeces": "ablandecer",
    "ablandecida": "ablandecer",
    "ablandecidas": "ablandecer",
    "ablandecido": "ablandecer",
    "ablandecidos": "ablandecer",
    "ablandeciendo": "ablandecer",
    "ablandeciera": "ablandecer",
    "ablandecierais": "ablandecer",
    "ablandecieran": "ablandecer",
    "ablandecieras": "ablandecer",
    "ablandeciere": "ablandecer",
    "ablandeciereis": "ablandecer",
    "ablandecieren": "ablandecer",
    "ablandecieres": "ablandecer",
    "ablandecieron": "ablandecer",
    "ablandeciese": "ablandecer",
    "ablandecieseis": "ablandecer",
    "ablandeciesen": "ablandecer",
    "ablandecieses": "ablandecer",
    "ablandecimos": "ablandecer",
    "ablandeciste": "ablandecer",
    "ablandecisteis": "ablandecer",
    "ablandeciéramos": "ablandecer",
    "ablandeciéremos": "ablandecer",
    "ablandeciésemos": "ablandecer",
    "ablandeció": "ablandecer",
    "ablandecéis": "ablandecer",
    "ablandecí": "ablandecer",
    "ablandecía": "ablandecer",
    "ablandecíais": "ablandecer",
    "ablandecíamos": "ablandecer",
    "ablandecían": "ablandecer",
    "ablandecías": "ablandecer",
    "ablandemos": "ablandar",
    "ablanden": "ablandar",
    "ablandes": "ablande",
    "ablandezca": "ablandecer",
    "ablandezcamos": "ablandecer",
    "ablandezcan": "ablandecer",
    "ablandezcas": "ablandecer",
    "ablandezco": "ablandecer",
    "ablandezcáis": "ablandecer",
    "ablando": "ablandar",
    "ablandábamos": "ablandar",
    "ablandáis": "ablandar",
    "ablandáramos": "ablandar",
    "ablandáremos": "ablandar",
    "ablandásemos": "ablandar",
    "ablandé": "ablandar",
    "ablandéis": "ablandar",
    "ablandó": "ablandar",
    "ablaqueaciones": "ablaqueación",
    "ablaques": "ablaque",
    "ablastinas": "ablastina",
    "ablativos": "ablativo",
    "ablefarias": "ablefaria",
    "ablepsias": "ablepsia",
    "abluciones": "ablución",
    "ablusada": "ablusado",
    "ablusadas": "ablusado",
    "ablusados": "ablusado",
    "abnegaciones": "abnegación",
    "abnegada": "abnegado",
    "abnegadas": "abnegado",
    "abnegados": "abnegado",
    "abo": "abar",
    "aboba": "abobar",
    "abobaba": "abobar",
    "abobabais": "abobar",
    "abobaban": "abobar",
    "abobabas": "abobar",
    "abobad": "abobar",
    "abobada": "abobar",
    "abobadas": "abobar",
    "abobado": "abobar",
    "abobados": "abobar",
    "abobamientos": "abobamiento",
    "abobamos": "abobar",
    "aboban": "abobar",
    "abobando": "abobar",
    "abobara": "abobar",
    "abobarais": "abobar",
    "abobaran": "abobar",
    "abobaras": "abobar",
    "abobare": "abobar",
    "abobareis": "abobar",
    "abobaremos": "abobar",
    "abobaren": "abobar",
    "abobares": "abobar",
    "abobaron": "abobar",
    "abobará": "abobar",
    "abobarán": "abobar",
    "abobarás": "abobar",
    "abobaré": "abobar",
    "abobaréis": "abobar",
    "abobaría": "abobar",
    "abobaríais": "abobar",
    "abobaríamos": "abobar",
    "abobarían": "abobar",
    "abobarías": "abobar",
    "abobas": "abobar",
    "abobase": "abobar",
    "abobaseis": "abobar",
    "abobasen": "abobar",
    "abobases": "abobar",
    "abobaste": "abobar",
    "abobasteis": "abobar",
    "abobe": "abobar",
    "abobemos": "abobar",
    "aboben": "abobar",
    "abobes": "abobar",
    "abobo": "abobar",
    "abobras": "abobra",
    "abobábamos": "abobar",
    "abobáis": "abobar",
    "abobáramos": "abobar",
    "abobáremos": "abobar",
    "abobásemos": "abobar",
    "abobé": "abobar",
    "abobéis": "abobar",
    "abobó": "abobar",
    "aboca": "abocar",
    "abocaba": "abocar",
    "abocabais": "abocar",
    "abocaban": "abocar",
    "abocabas": "abocar",
    "abocad": "abocar",
    "abocada": "abocar",
    "abocadas": "abocar",
    "abocadea": "abocadear",
    "abocadeaba": "abocadear",
    "abocadeabais": "abocadear",
    "abocadeaban": "abocadear",
    "abocadeabas": "abocadear",
    "abocadead": "abocadear",
    "abocadeada": "abocadear",
    "abocadeadas": "abocadear",
    "abocadeado": "abocadear",
    "abocadeados": "abocadear",
    "abocadeamos": "abocadear",
    "abocadean": "abocadear",
    "abocadeando": "abocadear",
    "abocadeara": "abocadear",
    "abocadearais": "abocadear",
    "abocadearan": "abocadear",
    "abocadearas": "abocadear",
    "abocadeare": "abocadear",
    "abocadeareis": "abocadear",
    "abocadearemos": "abocadear",
    "abocadearen": "abocadear",
    "abocadeares": "abocadear",
    "abocadearon": "abocadear",
    "abocadeará": "abocadear",
    "abocadearán": "abocadear",
    "abocadearás": "abocadear",
    "abocadearé": "abocadear",
    "abocadearéis": "abocadear",
    "abocadearía": "abocadear",
    "abocadearíais": "abocadear",
    "abocadearíamos": "abocadear",
    "abocadearían": "abocadear",
    "abocadearías": "abocadear",
    "abocadeas": "abocadear",
    "abocadease": "abocadear",
    "abocadeaseis": "abocadear",
    "abocadeasen": "abocadear",
    "abocadeases": "abocadear",
    "abocadeaste": "abocadear",
    "abocadeasteis": "abocadear",
    "abocadee": "abocadear",
    "abocadeemos": "abocadear",
    "abocadeen": "abocadear",
    "abocadees": "abocadear",
    "abocadeo": "abocadear",
    "abocadeábamos": "abocadear",
    "abocadeáis": "abocadear",
    "abocadeáramos": "abocadear",
    "abocadeáremos": "abocadear",
    "abocadeásemos": "abocadear",
    "abocadeé": "abocadear",
    "abocadeéis": "abocadear",
    "abocadeó": "abocadear",
    "abocado": "abocar",
    "abocados": "abocar",
    "abocamientos": "abocamiento",
    "abocamos": "abocar",
    "abocan": "abocar",
    "abocando": "abocar",
    "abocara": "abocar",
    "abocarais": "abocar",
    "abocaran": "abocar",
    "abocaras": "abocar",
    "abocarda": "abocardar",
    "abocardaba": "abocardar",
    "abocardabais": "abocardar",
    "abocardaban": "abocardar",
    "abocardabas": "abocardar",
    "abocardad": "abocardar",
    "abocardada": "abocardar",
    "abocardadas": "abocardar",
    "abocardado": "abocardar",
    "abocardadores": "abocardador",
    "abocardados": "abocardar",
    "abocardamos": "abocardar",
    "abocardan": "abocardar",
    "abocardando": "abocardar",
    "abocardara": "abocardar",
    "abocardarais": "abocardar",
    "abocardaran": "abocardar",
    "abocardaras": "abocardar",
    "abocardare": "abocardar",
    "abocardareis": "abocardar",
    "abocardaremos": "abocardar",
    "abocardaren": "abocardar",
    "abocardares": "abocardar",
    "abocardaron": "abocardar",
    "abocardará": "abocardar",
    "abocardarán": "abocardar",
    "abocardarás": "abocardar",
    "abocardaré": "abocardar",
    "abocardaréis": "abocardar",
    "abocardaría": "abocardar",
    "abocardaríais": "abocardar",
    "abocardaríamos": "abocardar",
    "abocardarían": "abocardar",
    "abocardarías": "abocardar",
    "abocardas": "abocardar",
    "abocardase": "abocardar",
    "abocardaseis": "abocardar",
    "abocardasen": "abocardar",
    "abocardases": "abocardar",
    "abocardaste": "abocardar",
    "abocardasteis": "abocardar",
    "abocarde": "abocardar",
    "abocardemos": "abocardar",
    "abocarden": "abocardar",
    "abocardes": "abocardar",
    "abocardo": "abocardar",
    "abocardos": "abocardo",
    "abocardábamos": "abocardar",
    "abocardáis": "abocardar",
    "abocardáramos": "abocardar",
    "abocardáremos": "abocardar",
    "abocardásemos": "abocardar",
    "abocardé": "abocardar",
    "abocardéis": "abocardar",
    "abocardó": "abocardar",
    "abocare": "abocar",
    "abocareis": "abocar",
    "abocaremos": "abocar",
    "abocaren": "abocar",
    "abocares": "abocar",
    "abocaron": "abocar",
    "abocará": "abocar",
    "abocarán": "abocar",
    "abocarás": "abocar",
    "abocaré": "abocar",
    "abocaréis": "abocar",
    "abocaría": "abocar",
    "abocaríais": "abocar",
    "abocaríamos": "abocar",
    "abocarían": "abocar",
    "abocarías": "abocar",
    "abocas": "abocar",
    "abocase": "abocar",
    "abocaseis": "abocar",
    "abocasen": "abocar",
    "abocases": "abocar",
    "abocaste": "abocar",
    "abocasteis": "abocar",
    "aboceta": "abocetar",
    "abocetaba": "abocetar",
    "abocetabais": "abocetar",
    "abocetaban": "abocetar",
    "abocetabas": "abocetar",
    "abocetad": "abocetar",
    "abocetada": "abocetar",
    "abocetadas": "abocetar",
    "abocetado": "abocetar",
    "abocetados": "abocetar",
    "abocetamos": "abocetar",
    "abocetan": "abocetar",
    "abocetando": "abocetar",
    "abocetara": "abocetar",
    "abocetarais": "abocetar",
    "abocetaran": "abocetar",
    "abocetaras": "abocetar",
    "abocetare": "abocetar",
    "abocetareis": "abocetar",
    "abocetaremos": "abocetar",
    "abocetaren": "abocetar",
    "abocetares": "abocetar",
    "abocetaron": "abocetar",
    "abocetará": "abocetar",
    "abocetarán": "abocetar",
    "abocetarás": "abocetar",
    "abocetaré": "abocetar",
    "abocetaréis": "abocetar",
    "abocetaría": "abocetar",
    "abocetaríais": "abocetar",
    "abocetaríamos": "abocetar",
    "abocetarían": "abocetar",
    "abocetarías": "abocetar",
    "abocetas": "abocetar",
    "abocetase": "abocetar",
    "abocetaseis": "abocetar",
    "abocetasen": "abocetar",
    "abocetases": "abocetar",
    "abocetaste": "abocetar",
    "abocetasteis": "abocetar",
    "abocete": "abocetar",
    "abocetemos": "abocetar",
    "aboceten": "abocetar",
    "abocetes": "abocetar",
    "aboceto": "abocetar",
    "abocetábamos": "abocetar",
    "abocetáis": "abocetar",
    "abocetáramos": "abocetar",
    "abocetáremos": "abocetar",
    "abocetásemos": "abocetar",
    "aboceté": "abocetar",
    "abocetéis": "abocetar",
    "abocetó": "abocetar",
    "abochorna": "abochornar",
    "abochornaba": "abochornar",
    "abochornabais": "abochornar",
    "abochornaban": "abochornar",
    "abochornabas": "abochornar",
    "abochornad": "abochornar",
    "abochornada": "abochornar",
    "abochornadas": "abochornar",
    "abochornado": "abochornar",
    "abochornados": "abochornar",
    "abochornamos": "abochornar",
    "abochornan": "abochornar",
    "abochornando": "abochornar",
    "abochornara": "abochornar",
    "abochornarais": "abochornar",
    "abochornaran": "abochornar",
    "abochornaras": "abochornar",
    "abochornare": "abochornar",
    "abochornareis": "abochornar",
    "abochornaremos": "abochornar",
    "abochornaren": "abochornar",
    "abochornares": "abochornar",
    "abochornaron": "abochornar",
    "abochornará": "abochornar",
    "abochornarán": "abochornar",
    "abochornarás": "abochornar",
    "abochornaré": "abochornar",
    "abochornaréis": "abochornar",
    "abochornaría": "abochornar",
    "abochornaríais": "abochornar",
    "abochornaríamos": "abochornar",
    "abochornarían": "abochornar",
    "abochornarías": "abochornar",
    "abochornas": "abochornar",
    "abochornase": "abochornar",
    "abochornaseis": "abochornar",
    "abochornasen": "abochornar",
    "abochornases": "abochornar",
    "abochornaste": "abochornar",
    "abochornasteis": "abochornar",
    "abochorne": "abochornar",
    "abochornemos": "abochornar",
    "abochornen": "abochornar",
    "abochornes": "abochornar",
    "abochorno": "abochornar",
    "abochornábamos": "abochornar",
    "abochornáis": "abochornar",
    "abochornáramos": "abochornar",
    "abochornáremos": "abochornar",
    "abochornásemos": "abochornar",
    "abochorné": "abochornar",
    "abochornéis": "abochornar",
    "abochornó": "abochornar",
    "abocina": "abocinar",
    "abocinaba": "abocinar",
    "abocinabais": "abocinar",
    "abocinaban": "abocinar",
    "abocinabas": "abocinar",
    "abocinad": "abocinar",
    "abocinada": "abocinar",
    "abocinadas": "abocinar",
    "abocinado": "abocinar",
    "abocinados": "abocinar",
    "abocinamientos": "abocinamiento",
    "abocinamos": "abocinar",
    "abocinan": "abocinar",
    "abocinando": "abocinar",
    "abocinara": "abocinar",
    "abocinarais": "abocinar",
    "abocinaran": "abocinar",
    "abocinaras": "abocinar",
    "abocinare": "abocinar",
    "abocinareis": "abocinar",
    "abocinaremos": "abocinar",
    "abocinaren": "abocinar",
    "abocinares": "abocinar",
    "abocinaron": "abocinar",
    "abocinará": "abocinar",
    "abocinarán": "abocinar",
    "abocinarás": "abocinar",
    "abocinaré": "abocinar",
    "abocinaréis": "abocinar",
    "abocinaría": "abocinar",
    "abocinaríais": "abocinar",
    "abocinaríamos": "abocinar",
    "abocinarían": "abocinar",
    "abocinarías": "abocinar",
    "abocinas": "abocinar",
    "abocinase": "abocinar",
    "abocinaseis": "abocinar",
    "abocinasen": "abocinar",
    "abocinases": "abocinar",
    "abocinaste": "abocinar",
    "abocinasteis": "abocinar",
    "abocine": "abocinar",
    "abocinemos": "abocinar",
    "abocinen": "abocinar",
    "abocines": "abocinar",
    "abocino": "abocinar",
    "abocinábamos": "abocinar",
    "abocináis": "abocinar",
    "abocináramos": "abocinar",
    "abocináremos": "abocinar",
    "abocinásemos": "abocinar",
    "abociné": "abocinar",
    "abocinéis": "abocinar",
    "abocinó": "abocinar",
    "aboco": "abocar",
    "abocábamos": "abocar",
    "abocáis": "abocar",
    "abocáramos": "abocar",
    "abocáremos": "abocar",
    "abocásemos": "abocar",
    "abocó": "abocar",
    "abofetea": "abofetear",
    "abofeteaba": "abofetear",
    "abofeteabais": "abofetear",
    "abofeteaban": "abofetear",
    "abofeteabas": "abofetear",
    "abofetead": "abofetear",
    "abofeteada": "abofetear",
    "abofeteadas": "abofetear",
    "abofeteado": "abofetear",
    "abofeteados": "abofetear",
    "abofeteamientos": "abofeteamiento",
    "abofeteamos": "abofetear",
    "abofetean": "abofetear",
    "abofeteando": "abofetear",
    "abofeteara": "abofetear",
    "abofetearais": "abofetear",
    "abofetearan": "abofetear",
    "abofetearas": "abofetear",
    "abofeteare": "abofetear",
    "abofeteareis": "abofetear",
    "abofetearemos": "abofetear",
    "abofetearen": "abofetear",
    "abofeteares": "abofetear",
    "abofetearon": "abofetear",
    "abofeteará": "abofetear",
    "abofetearán": "abofetear",
    "abofetearás": "abofetear",
    "abofetearé": "abofetear",
    "abofetearéis": "abofetear",
    "abofetearía": "abofetear",
    "abofetearíais": "abofetear",
    "abofetearíamos": "abofetear",
    "abofetearían": "abofetear",
    "abofetearías": "abofetear",
    "abofeteas": "abofetear",
    "abofetease": "abofetear",
    "abofeteaseis": "abofetear",
    "abofeteasen": "abofetear",
    "abofeteases": "abofetear",
    "abofeteaste": "abofetear",
    "abofeteasteis": "abofetear",
    "abofetee": "abofetear",
    "abofeteemos": "abofetear",
    "abofeteen": "abofetear",
    "abofetees": "abofetear",
    "abofeteo": "abofetear",
    "abofeteábamos": "abofetear",
    "abofeteáis": "abofetear",
    "abofeteáramos": "abofetear",
    "abofeteáremos": "abofetear",
    "abofeteásemos": "abofetear",
    "abofeteé": "abofetear",
    "abofeteéis": "abofetear",
    "abofeteó": "abofetear",
    "aboga": "abogar",
    "abogaba": "abogar",
    "abogabais": "abogar",
    "abogaban": "abogar",
    "abogabas": "abogar",
    "abogacías": "abogacía",
    "abogad": "abogar",
    "abogada": "abogar",
    "abogadas": "abogar",
    "abogado": "abogar",
    "abogados": "abogar",
    "abogamos": "abogar",
    "abogan": "abogar",
    "abogando": "abogar",
    "abogara": "abogar",
    "abogarais": "abogar",
    "abogaran": "abogar",
    "abogaras": "abogar",
    "abogare": "abogar",
    "abogareis": "abogar",
    "abogaremos": "abogar",
    "abogaren": "abogar",
    "abogares": "abogar",
    "abogaron": "abogar",
    "abogará": "abogar",
    "abogarán": "abogar",
    "abogarás": "abogar",
    "abogaré": "abogar",
    "abogaréis": "abogar",
    "abogaría": "abogar",
    "abogaríais": "abogar",
    "abogaríamos": "abogar",
    "abogarían": "abogar",
    "abogarías": "abogar",
    "abogas": "abogar",
    "abogase": "abogar",
    "abogaseis": "abogar",
    "abogasen": "abogar",
    "abogases": "abogar",
    "abogaste": "abogar",
    "abogasteis": "abogar",
    "abogo": "abogar",
    "abogue": "abogar",
    "aboguemos": "abogar",
    "aboguen": "abogar",
    "abogues": "abogar",
    "abogué": "abogar",
    "aboguéis": "abogar",
    "abogábamos": "abogar",
    "abogáis": "abogar",
    "abogáramos": "abogar",
    "abogáremos": "abogar",
    "abogásemos": "abogar",
    "abogó": "abogar",
    "abolengos": "abolengo",
    "aboliciones": "abolición",
    "abolicionismos": "abolicionismo",
    "abolicionistas": "abolicionista",
    "abolid": "abolir",
    "abolida": "abolir",
    "abolidas": "abolir",
    "abolido": "abolir",
    "abolidos": "abolir",
    "aboliendo": "abolir",
    "aboliera": "abolir",
    "abolierais": "abolir",
    "abolieran": "abolir",
    "abolieras": "abolir",
    "aboliere": "abolir",
    "aboliereis": "abolir",
    "abolieren": "abolir",
    "abolieres": "abolir",
    "abolieron": "abolir",
    "aboliese": "abolir",
    "abolieseis": "abolir",
    "aboliesen": "abolir",
    "abolieses": "abolir",
    "abolimos": "abolir",
    "aboliremos": "abolir",
    "abolirá": "abolir",
    "abolirán": "abolir",
    "abolirás": "abolir",
    "aboliré": "abolir",
    "aboliréis": "abolir",
    "aboliría": "abolir",
    "aboliríais": "abolir",
    "aboliríamos": "abolir",
    "abolirían": "abolir",
    "abolirías": "abolir",
    "aboliste": "abolir",
    "abolisteis": "abolir",
    "aboliéramos": "abolir",
    "aboliéremos": "abolir",
    "aboliésemos": "abolir",
    "abolió": "abolir",
    "abolla": "abollar",
    "abollaba": "abollar",
    "abollabais": "abollar",
    "abollaban": "abollar",
    "abollabas": "abollar",
    "abollad": "abollar",
    "abollada": "abollar",
    "abolladas": "abollar",
    "abollado": "abollar",
    "abollados": "abollar",
    "abolladuras": "abolladura",
    "abollamos": "abollar",
    "abollan": "abollar",
    "abollando": "abollar",
    "abollara": "abollar",
    "abollarais": "abollar",
    "abollaran": "abollar",
    "abollaras": "abollar",
    "abollare": "abollar",
    "abollareis": "abollar",
    "abollaremos": "abollar",
    "abollaren": "abollar",
    "abollares": "abollar",
    "abollaron": "abollar",
    "abollará": "abollar",
    "abollarán": "abollar",
    "abollarás": "abollar",
    "abollaré": "abollar",
    "abollaréis": "abollar",
    "abollaría": "abollar",
    "abollaríais": "abollar",
    "abollaríamos": "abollar",
    "abollarían": "abollar",
    "abollarías": "abollar",
    "abollas": "abollar",
    "abollase": "abollar",
    "abollaseis": "abollar",
    "abollasen": "abollar",
    "abollases": "abollar",
    "abollaste": "abollar",
    "abollasteis": "abollar",
    "abolle": "abollar",
    "abollemos": "abollar",
    "abollen": "abollar",
    "abolles": "abollar",
    "abollo": "abollar",
    "abollona": "abollonar",
    "abollonaba": "abollonar",
    "abollonabais": "abollonar",
    "abollonaban": "abollonar",
    "abollonabas": "abollonar",
    "abollonad": "abollonar",
    "abollonada": "abollonar",
    "abollonadas": "abollonar",
    "abollonado": "abollonar",
    "abollonados": "abollonar",
    "abollonamos": "abollonar",
    "abollonan": "abollonar",
    "abollonando": "abollonar",
    "abollonara": "abollonar",
    "abollonarais": "abollonar",
    "abollonaran": "abollonar",
    "abollonaras": "abollonar",
    "abollonare": "abollonar",
    "abollonareis": "abollonar",
    "abollonaremos": "abollonar",
    "abollonaren": "abollonar",
    "abollonares": "abollonar",
    "abollonaron": "abollonar",
    "abollonará": "abollonar",
    "abollonarán": "abollonar",
    "abollonarás": "abollonar",
    "abollonaré": "abollonar",
    "abollonaréis": "abollonar",
    "abollonaría": "abollonar",
    "abollonaríais": "abollonar",
    "abollonaríamos": "abollonar",
    "abollonarían": "abollonar",
    "abollonarías": "abollonar",
    "abollonas": "abollonar",
    "abollonase": "abollonar",
    "abollonaseis": "abollonar",
    "abollonasen": "abollonar",
    "abollonases": "abollonar",
    "abollonaste": "abollonar",
    "abollonasteis": "abollonar",
    "abollone": "abollonar",
    "abollonemos": "abollonar",
    "abollonen": "abollonar",
    "abollones": "abollonar",
    "abollono": "abollonar",
    "abollonábamos": "abollonar",
    "abollonáis": "abollonar",
    "abollonáramos": "abollonar",
    "abollonáremos": "abollonar",
    "abollonásemos": "abollonar",
    "abolloné": "abollonar",
    "abollonéis": "abollonar",
    "abollonó": "abollonar",
    "abollábamos": "abollar",
    "abolláis": "abollar",
    "abolláramos": "abollar",
    "abolláremos": "abollar",
    "abollásemos": "abollar",
    "abollé": "abollar",
    "abolléis": "abollar",
    "abolló": "abollar",
    "abolorios": "abolorio",
    "abolsa": "abolsar",
    "abolsaba": "abolsar",
    "abolsabais": "abolsar",
    "abolsaban": "abolsar",
    "abolsabas": "abolsar",
    "abolsad": "abolsar",
    "abolsada": "abolsar",
    "abolsadas": "abolsar",
    "abolsado": "abolsar",
    "abolsados": "abolsar",
    "abolsamientos": "abolsamiento",
    "abolsamos": "abolsar",
    "abolsan": "abolsar",
    "abolsando": "abolsar",
    "abolsara": "abolsar",
    "abolsarais": "abolsar",
    "abolsaran": "abolsar",
    "abolsaras": "abolsar",
    "abolsare": "abolsar",
    "abolsareis": "abolsar",
    "abolsaremos": "abolsar",
    "abolsaren": "abolsar",
    "abolsares": "abolsar",
    "abolsaron": "abolsar",
    "abolsará": "abolsar",
    "abolsarán": "abolsar",
    "abolsarás": "abolsar",
    "abolsaré": "abolsar",
    "abolsaréis": "abolsar",
    "abolsaría": "abolsar",
    "abolsaríais": "abolsar",
    "abolsaríamos": "abolsar",
    "abolsarían": "abolsar",
    "abolsarías": "abolsar",
    "abolsas": "abolsar",
    "abolsase": "abolsar",
    "abolsaseis": "abolsar",
    "abolsasen": "abolsar",
    "abolsases": "abolsar",
    "abolsaste": "abolsar",
    "abolsasteis": "abolsar",
    "abolse": "abolsar",
    "abolsemos": "abolsar",
    "abolsen": "abolsar",
    "abolses": "abolsar",
    "abolso": "abolsar",
    "abolsábamos": "abolsar",
    "abolsáis": "abolsar",
    "abolsáramos": "abolsar",
    "abolsáremos": "abolsar",
    "abolsásemos": "abolsar",
    "abolsé": "abolsar",
    "abolséis": "abolsar",
    "abolsó": "abolsar",
    "abolí": "abolir",
    "abolía": "abolir",
    "abolíais": "abolir",
    "abolíamos": "abolir",
    "abolían": "abolir",
    "abolías": "abolir",
    "abolís": "abolir",
    "abomasos": "abomaso",
    "abomba": "abombar",
    "abombaba": "abombar",
    "abombabais": "abombar",
    "abombaban": "abombar",
    "abombabas": "abombar",
    "abombad": "abombar",
    "abombada": "abombar",
    "abombadas": "abombar",
    "abombado": "abombar",
    "abombados": "abombar",
    "abombamientos": "abombamiento",
    "abombamos": "abombar",
    "abomban": "abombar",
    "abombando": "abombar",
    "abombara": "abombar",
    "abombarais": "abombar",
    "abombaran": "abombar",
    "abombaras": "abombar",
    "abombare": "abombar",
    "abombareis": "abombar",
    "abombaremos": "abombar",
    "abombaren": "abombar",
    "abombares": "abombar",
    "abombaron": "abombar",
    "abombará": "abombar",
    "abombarán": "abombar",
    "abombarás": "abombar",
    "abombaré": "abombar",
    "abombaréis": "abombar",
    "abombaría": "abombar",
    "abombaríais": "abombar",
    "abombaríamos": "abombar",
    "abombarían": "abombar",
    "abombarías": "abombar",
    "abombas": "abombar",
    "abombase": "abombar",
    "abombaseis": "abombar",
    "abombasen": "abombar",
    "abombases": "abombar",
    "abombaste": "abombar",
    "abombasteis": "abombar",
    "abombe": "abombar",
    "abombemos": "abombar",
    "abomben": "abombar",
    "abombes": "abombar",
    "abombo": "abombar",
    "abombábamos": "abombar",
    "abombáis": "abombar",
    "abombáramos": "abombar",
    "abombáremos": "abombar",
    "abombásemos": "abombar",
    "abombé": "abombar",
    "abombéis": "abombar",
    "abombó": "abombar",
    "abomina": "abominar",
    "abominaba": "abominar",
    "abominabais": "abominar",
    "abominaban": "abominar",
    "abominabas": "abominar",
    "abominables": "abominable",
    "abominaciones": "abominación",
    "abominad": "abominar",
    "abominada": "abominar",
    "abominadas": "abominar",
    "abominado": "abominar",
    "abominados": "abominar",
    "abominamos": "abominar",
    "abominan": "abominar",
    "abominando": "abominar",
    "abominara": "abominar",
    "abominarais": "abominar",
    "abominaran": "abominar",
    "abominaras": "abominar",
    "abominare": "abominar",
    "abominareis": "abominar",
    "abominaremos": "abominar",
    "abominaren": "abominar",
    "abominares": "abominar",
    "abominaron": "abominar",
    "abominará": "abominar",
    "abominarán": "abominar",
    "abominarás": "abominar",
    "abominaré": "abominar",
    "abominaréis": "abominar",
    "abominaría": "abominar",
    "abominaríais": "abominar",
    "abominaríamos": "abominar",
    "abominarían": "abominar",
    "abominarías": "abominar",
    "abominas": "abominar",
    "abominase": "abominar",
    "abominaseis": "abominar",
    "abominasen": "abominar",
    "abominases": "abominar",
    "abominaste": "abominar",
    "abominasteis": "abominar",
    "abomine": "abominar",
    "abominemos": "abominar",
    "abominen": "abominar",
    "abomines": "abominar",
    "abomino": "abominar",
    "abominábamos": "abominar",
    "abomináis": "abominar",
    "abomináramos": "abominar",
    "abomináremos": "abominar",
    "abominásemos": "abominar",
    "abominé": "abominar",
    "abominéis": "abominar",
    "abominó": "abominar",
    "abona": "abonar",
    "abonaba": "abonar",
    "abonabais": "abonar",
    "abonaban": "abonar",
    "abonabas": "abonar",
    "abonables": "abonable",
    "abonad": "abonar",
    "abonada": "abonar",
    "abonadas": "abonar",
    "abonado": "abonar",
    "abonadora": "abonador",
    "abonadoras": "abonador",
    "abonadores": "abonador",
    "abonados": "abonar",
    "abonamos": "abonar",
    "abonan": "abonar",
    "abonance": "abonanzar",
    "abonancemos": "abonanzar",
    "abonancen": "abonanzar",
    "abonances": "abonanzar",
    "abonancé": "abonanzar",
    "abonancéis": "abonanzar",
    "abonando": "abonar",
    "abonanza": "abonanzar",
    "abonanzaba": "abonanzar",
    "abonanzabais": "abonanzar",
    "abonanzaban": "abonanzar",
    "abonanzabas": "abonanzar",
    "abonanzad": "abonanzar",
    "abonanzada": "abonanzar",
    "abonanzadas": "abonanzar",
    "abonanzado": "abonanzar",
    "abonanzados": "abonanzar",
    "abonanzamos": "abonanzar",
    "abonanzan": "abonanzar",
    "abonanzando": "abonanzar",
    "abonanzara": "abonanzar",
    "abonanzarais": "abonanzar",
    "abonanzaran": "abonanzar",
    "abonanzaras": "abonanzar",
    "abonanzare": "abonanzar",
    "abonanzareis": "abonanzar",
    "abonanzaremos": "abonanzar",
    "abonanzaren": "abonanzar",
    "abonanzares": "abonanzar",
    "abonanzaron": "abonanzar",
    "abonanzará": "abonanzar",
    "abonanzarán": "abonanzar",
    "abonanzarás": "abonanzar",
    "abonanzaré": "abonanzar",
    "abonanzaréis": "abonanzar",
    "abonanzaría": "abonanzar",
    "abonanzaríais": "abonanzar",
    "abonanzaríamos": "abonanzar",
    "abonanzarían": "abonanzar",
    "abonanzarías": "abonanzar",
    "abonanzas": "abonanzar",
    "abonanzase": "abonanzar",
    "abonanzaseis": "abonanzar",
    "abonanzasen": "abonanzar",
    "abonanzases": "abonanzar",
    "abonanzaste": "abonanzar",
    "abonanzasteis": "abonanzar",
    "abonanzo": "abonanzar",
    "abonanzábamos": "abonanzar",
    "abonanzáis": "abonanzar",
    "abonanzáramos": "abonanzar",
    "abonanzáremos": "abonanzar",
    "abonanzásemos": "abonanzar",
    "abonanzó": "abonanzar",
    "abonara": "abonar",
    "abonarais": "abonar",
    "abonaran": "abonar",
    "abonaras": "abonar",
    "abonare": "abonar",
    "abonareis": "abonar",
    "abonaremos": "abonar",
    "abonaren": "abonar",
    "abonares": "abonar",
    "abonaron": "abonar",
    "abonará": "abonar",
    "abonarán": "abonar",
    "abonarás": "abonar",
    "abonaré": "abonar",
    "abonaréis": "abonar",
    "abonarés": "abonaré",
    "abonaría": "abonar",
    "abonaríais": "abonar",
    "abonaríamos": "abonar",
    "abonarían": "abonar",
    "abonarías": "abonar",
    "abonas": "abonar",
    "abonase": "abonar",
    "abonaseis": "abonar",
    "abonasen": "abonar",
    "abonases": "abonar",
    "abonaste": "abonar",
    "abonasteis": "abonar",
    "abone": "abonar",
    "abonemos": "abonar",
    "abonen": "abonar",
    "abones": "abonar",
    "abono": "abonar",
    "abonos": "abono",
    "abonábamos": "abonar",
    "abonáis": "abonar",
    "abonáramos": "abonar",
    "abonáremos": "abonar",
    "abonásemos": "abonar",
    "aboné": "abonar",
    "abonéis": "abonar",
    "abonó": "abonar",
    "aboque": "abocar",
    "aboquemos": "abocar",
    "aboquen": "abocar",
    "aboques": "abocar",
    "aboquilla": "aboquillar",
    "aboquillaba": "aboquillar",
    "aboquillabais": "aboquillar",
    "aboquillaban": "aboquillar",
    "aboquillabas": "aboquillar",
    "aboquillad": "aboquillar",
    "aboquillada": "aboquillar",
    "aboquilladas": "aboquillar",
    "aboquillado": "aboquillar",
    "aboquillados": "aboquillar",
    "aboquillamos": "aboquillar",
    "aboquillan": "aboquillar",
    "aboquillando": "aboquillar",
    "aboquillara": "aboquillar",
    "aboquillarais": "aboquillar",
    "aboquillaran": "aboquillar",
    "aboquillaras": "aboquillar",
    "aboquillare": "aboquillar",
    "aboquillareis": "aboquillar",
    "aboquillaremos": "aboquillar",
    "aboquillaren": "aboquillar",
    "aboquillares": "aboquillar",
    "aboquillaron": "aboquillar",
    "aboquillará": "aboquillar",
    "aboquillarán": "aboquillar",
    "aboquillarás": "aboquillar",
    "aboquillaré": "aboquillar",
    "aboquillaréis": "aboquillar",
    "aboquillaría": "aboquillar",
    "aboquillaríais": "aboquillar",
    "aboquillaríamos": "aboquillar",
    "aboquillarían": "aboquillar",
    "aboquillarías": "aboquillar",
    "aboquillas": "aboquillar",
    "aboquillase": "aboquillar",
    "aboquillaseis": "aboquillar",
    "aboquillasen": "aboquillar",
    "aboquillases": "aboquillar",
    "aboquillaste": "aboquillar",
    "aboquillasteis": "aboquillar",
    "aboquille": "aboquillar",
    "aboquillemos": "aboquillar",
    "aboquillen": "aboquillar",
    "aboquilles": "aboquillar",
    "aboquillo": "aboquillar",
    "aboquillábamos": "aboquillar",
    "aboquilláis": "aboquillar",
    "aboquilláramos": "aboquillar",
    "aboquilláremos": "aboquillar",
    "aboquillásemos": "aboquillar",
    "aboquillé": "aboquillar",
    "aboquilléis": "aboquillar",
    "aboquilló": "aboquillar",
    "aboqué": "abocar",
    "aboquéis": "abocar",
    "aborales": "aboral",
    "aborda": "abordar",
    "abordaba": "abordar",
    "abordabais": "abordar",
    "abordaban": "abordar",
    "abordabas": "abordar",
    "abordables": "abordable",
    "abordad": "abordar",
    "abordada": "abordar",
    "abordadas": "abordar",
    "abordado": "abordar",
    "abordadora": "abordador",
    "abordadoras": "abordador",
    "abordadores": "abordador",
    "abordados": "abordar",
    "abordajes": "abordaje",
    "abordamientos": "abordamiento",
    "abordamos": "abordar",
    "abordan": "abordar",
    "abordando": "abordar",
    "abordara": "abordar",
    "abordarais": "abordar",
    "abordaran": "abordar",
    "abordaras": "abordar",
    "abordare": "abordar",
    "abordareis": "abordar",
    "abordaremos": "abordar",
    "abordaren": "abordar",
    "abordares": "abordar",
    "abordaron": "abordar",
    "abordará": "abordar",
    "abordarán": "abordar",
    "abordarás": "abordar",
    "abordaré": "abordar",
    "abordaréis": "abordar",
    "abordaría": "abordar",
    "abordaríais": "abordar",
    "abordaríamos": "abordar",
    "abordarían": "abordar",
    "abordarías": "abordar",
    "abordas": "abordar",
    "abordase": "abordar",
    "abordaseis": "abordar",
    "abordasen": "abordar",
    "abordases": "abordar",
    "abordaste": "abordar",
    "abordasteis": "abordar",
    "aborde": "abordar",
    "abordemos": "abordar",
    "aborden": "abordar",
    "abordes": "abordar",
    "abordo": "abordar",
    "abordos": "abordo",
    "abordábamos": "abordar",
    "abordáis": "abordar",
    "abordáramos": "abordar",
    "abordáremos": "abordar",
    "abordásemos": "abordar",
    "abordé": "abordar",
    "abordéis": "abordar",
    "abordó": "abordar",
    "aborraja": "aborrajar",
    "aborrajaba": "aborrajar",
    "aborrajabais": "aborrajar",
    "aborrajaban": "aborrajar",
    "aborrajabas": "aborrajar",
    "aborrajad": "aborrajar",
    "aborrajada": "aborrajar",
    "aborrajadas": "aborrajar",
    "aborrajado": "aborrajar",
    "aborrajados": "aborrajar",
    "aborrajamos": "aborrajar",
    "aborrajan": "aborrajar",
    "aborrajando": "aborrajar",
    "aborrajara": "aborrajar",
    "aborrajarais": "aborrajar",
    "aborrajaran": "aborrajar",
    "aborrajaras": "aborrajar",
    "aborrajare": "aborrajar",
    "aborrajareis": "aborrajar",
    "aborrajaremos": "aborrajar",
    "aborrajaren": "aborrajar",
    "aborrajares": "aborrajar",
    "aborrajaron": "aborrajar",
    "aborrajará": "aborrajar",
    "aborrajarán": "aborrajar",
    "aborrajarás": "aborrajar",
    "aborrajaré": "aborrajar",
    "aborrajaréis": "aborrajar",
    "aborrajaría": "aborrajar",
    "aborrajaríais": "aborrajar",
    "aborrajaríamos": "aborrajar",
    "aborrajarían": "aborrajar",
    "aborrajarías": "aborrajar",
    "aborrajas": "aborrajar",
    "aborrajase": "aborrajar",
    "aborrajaseis": "aborrajar",
    "aborrajasen": "aborrajar",
    "aborrajases": "aborrajar",
    "aborrajaste": "aborrajar",
    "aborrajasteis": "aborrajar",
    "aborraje": "aborrajar",
    "aborrajemos": "aborrajar",
    "aborrajen": "aborrajar",
    "aborrajes": "aborrajar",
    "aborrajo": "aborrajar",
    "aborrajábamos": "aborrajar",
    "aborrajáis": "aborrajar",
    "aborrajáramos": "aborrajar",
    "aborrajáremos": "aborrajar",
    "aborrajásemos": "aborrajar",
    "aborrajé": "aborrajar",
    "aborrajéis": "aborrajar",
    "aborrajó": "aborrajar",
    "aborrasca": "aborrascar",
    "aborrascaba": "aborrascar",
    "aborrascabais": "aborrascar",
    "aborrascaban": "aborrascar",
    "aborrascabas": "aborrascar",
    "aborrascad": "aborrascar",
    "aborrascada": "aborrascar",
    "aborrascadas": "aborrascar",
    "aborrascado": "aborrascar",
    "aborrascados": "aborrascar",
    "aborrascamos": "aborrascar",
    "aborrascan": "aborrascar",
    "aborrascando": "aborrascar",
    "aborrascara": "aborrascar",
    "aborrascarais": "aborrascar",
    "aborrascaran": "aborrascar",
    "aborrascaras": "aborrascar",
    "aborrascare": "aborrascar",
    "aborrascareis": "aborrascar",
    "aborrascaremos": "aborrascar",
    "aborrascaren": "aborrascar",
    "aborrascares": "aborrascar",
    "aborrascaron": "aborrascar",
    "aborrascará": "aborrascar",
    "aborrascarán": "aborrascar",
    "aborrascarás": "aborrascar",
    "aborrascaré": "aborrascar",
    "aborrascaréis": "aborrascar",
    "aborrascaría": "aborrascar",
    "aborrascaríais": "aborrascar",
    "aborrascaríamos": "aborrascar",
    "aborrascarían": "aborrascar",
    "aborrascarías": "aborrascar",
    "aborrascas": "aborrascar",
    "aborrascase": "aborrascar",
    "aborrascaseis": "aborrascar",
    "aborrascasen": "aborrascar",
    "aborrascases": "aborrascar",
    "aborrascaste": "aborrascar",
    "aborrascasteis": "aborrascar",
    "aborrasco": "aborrascar",
    "aborrascábamos": "aborrascar",
    "aborrascáis": "aborrascar",
    "aborrascáramos": "aborrascar",
    "aborrascáremos": "aborrascar",
    "aborrascásemos": "aborrascar",
    "aborrascó": "aborrascar",
    "aborrasque": "aborrascar",
    "aborrasquemos": "aborrascar",
    "aborrasquen": "aborrascar",
    "aborrasques": "aborrascar",
    "aborrasqué": "aborrascar",
    "aborrasquéis": "aborrascar",
    "aborrece": "aborrecer",
    "aborreced": "aborrecer",
    "aborrecedora": "aborrecedor",
    "aborrecedoras": "aborrecedor",
    "aborrecedores": "aborrecedor",
    "aborrecemos": "aborrecer",
    "aborrecen": "aborrecer",
    "aborreceremos": "aborrecer",
    "aborrecerá": "aborrecer",
    "aborrecerán": "aborrecer",
    "aborrecerás": "aborrecer",
    "aborreceré": "aborrecer",
    "aborreceréis": "aborrecer",
    "aborrecería": "aborrecer",
    "aborreceríais": "aborrecer",
    "aborreceríamos": "aborrecer",
    "aborrecerían": "aborrecer",
    "aborrecerías": "aborrecer",
    "aborreces": "aborrecer",
    "aborrecibles": "aborrecible",
    "aborrecida": "aborrecer",
    "aborrecidas": "aborrecer",
    "aborrecido": "aborrecer",
    "aborrecidos": "aborrecer",
    "aborreciendo": "aborrecer",
    "aborreciera": "aborrecer",
    "aborrecierais": "aborrecer",
    "aborrecieran": "aborrecer",
    "aborrecieras": "aborrecer",
    "aborreciere": "aborrecer",
    "aborreciereis": "aborrecer",
    "aborrecieren": "aborrecer",
    "aborrecieres": "aborrecer",
    "aborrecieron": "aborrecer",
    "aborreciese": "aborrecer",
    "aborrecieseis": "aborrecer",
    "aborreciesen": "aborrecer",
    "aborrecieses": "aborrecer",
    "aborrecimientos": "aborrecimiento",
    "aborrecimos": "aborrecer",
    "aborreciste": "aborrecer",
    "aborrecisteis": "aborrecer",
    "aborreciéramos": "aborrecer",
    "aborreciéremos": "aborrecer",
    "aborreciésemos": "aborrecer",
    "aborreció": "aborrecer",
    "aborrecéis": "aborrecer",
    "aborrecí": "aborrecer",
    "aborrecía": "aborrecer",
    "aborrecíais": "aborrecer",
    "aborrecíamos": "aborrecer",
    "aborrecían": "aborrecer",
    "aborrecías": "aborrecer",
    "aborrega": "aborregar",
    "aborregaba": "aborregar",
    "aborregabais": "aborregar",
    "aborregaban": "aborregar",
    "aborregabas": "aborregar",
    "aborregad": "aborregar",
    "aborregada": "aborregar",
    "aborregadas": "aborregar",
    "aborregado": "aborregar",
    "aborregados": "aborregar",
    "aborregamientos": "aborregamiento",
    "aborregamos": "aborregar",
    "aborregan": "aborregar",
    "aborregando": "aborregar",
    "aborregara": "aborregar",
    "aborregarais": "aborregar",
    "aborregaran": "aborregar",
    "aborregaras": "aborregar",
    "aborregare": "aborregar",
    "aborregareis": "aborregar",
    "aborregaremos": "aborregar",
    "aborregaren": "aborregar",
    "aborregares": "aborregar",
    "aborregaron": "aborregar",
    "aborregará": "aborregar",
    "aborregarán": "aborregar",
    "aborregarás": "aborregar",
    "aborregaré": "aborregar",
    "aborregaréis": "aborregar",
    "aborregaría": "aborregar",
    "aborregaríais": "aborregar",
    "aborregaríamos": "aborregar",
    "aborregarían": "aborregar",
    "aborregarías": "aborregar",
    "aborregas": "aborregar",
    "aborregase": "aborregar",
    "aborregaseis": "aborregar",
    "aborregasen": "aborregar",
    "aborregases": "aborregar",
    "aborregaste": "aborregar",
    "aborregasteis": "aborregar",
    "aborrego": "aborregar",
    "aborregue": "aborregar",
    "aborreguemos": "aborregar",
    "aborreguen": "aborregar",
    "aborregues": "aborregar",
    "aborregué": "aborregar",
    "aborreguéis": "aborregar",
    "aborregábamos": "aborregar",
    "aborregáis": "aborregar",
    "aborregáramos": "aborregar",
    "aborregáremos": "aborregar",
    "aborregásemos": "aborregar",
    "aborregó": "aborregar",
    "aborrezca": "aborrecer",
    "aborrezcamos": "aborrecer",
    "aborrezcan": "aborrecer",
    "aborrezcas": "aborrecer",
    "aborrezco": "aborrecer",
    "aborrezcáis": "aborrecer",
    "aborta": "abortar",
    "abortaba": "abortar",
    "abortabais": "abortar",
    "abortaban": "abortar",
    "abortabas": "abortar",
    "abortad": "abortar",
    "abortada": "abortar",
    "abortadas": "abortar",
    "abortado": "abortar",
    "abortados": "abortar",
    "abortamientos": "abortamiento",
    "abortamos": "abortar",
    "abortan": "abortar",
    "abortando": "abortar",
    "abortara": "abortar",
    "abortarais": "abortar",
    "abortaran": "abortar",
    "abortaras": "abortar",
    "abortare": "abortar",
    "abortareis": "abortar",
    "abortaremos": "abortar",
    "abortaren": "abortar",
    "abortares": "abortar",
    "abortargamientos": "abortargamiento",
    "abortaron": "abortar",
    "abortará": "abortar",
    "abortarán": "abortar",
    "abortarás": "abortar",
    "abortaré": "abortar",
    "abortaréis": "abortar",
    "abortaría": "abortar",
    "abortaríais": "abortar",
    "abortaríamos": "abortar",
    "abortarían": "abortar",
    "abortarías": "abortar",
    "abortas": "abortar",
    "abortase": "abortar",
    "abortaseis": "abortar",
    "abortasen": "abortar",
    "abortases": "abortar",
    "abortaste": "abortar",
    "abortasteis": "abortar",
    "aborte": "abortar",
    "abortemos": "abortar",
    "aborten": "abortar",
    "abortes": "abortar",
    "abortistas": "abortista",
    "abortiva": "abortivo",
    "abortivas": "abortivo",
    "abortivos": "abortivo",
    "aborto": "abortar",
    "abortones": "abortón",
    "abortos": "aborto",
    "abortábamos": "abortar",
    "abortáis": "abortar",
    "abortáramos": "abortar",
    "abortáremos": "abortar",
    "abortásemos": "abortar",
    "aborté": "abortar",
    "abortéis": "abortar",
    "abortó": "abortar",
    "aboruja": "aborujar",
    "aborujaba": "aborujar",
    "aborujabais": "aborujar",
    "aborujaban": "aborujar",
    "aborujabas": "aborujar",
    "aborujad": "aborujar",
    "aborujada": "aborujar",
    "aborujadas": "aborujar",
    "aborujado": "aborujar",
    "aborujados": "aborujar",
    "aborujamos": "aborujar",
    "aborujan": "aborujar",
    "aborujando": "aborujar",
    "aborujara": "aborujar",
    "aborujarais": "aborujar",
    "aborujaran": "aborujar",
    "aborujaras": "aborujar",
    "aborujare": "aborujar",
    "aborujareis": "aborujar",
    "aborujaremos": "aborujar",
    "aborujaren": "aborujar",
    "aborujares": "aborujar",
    "aborujaron": "aborujar",
    "aborujará": "aborujar",
    "aborujarán": "aborujar",
    "aborujarás": "aborujar",
    "aborujaré": "aborujar",
    "aborujaréis": "aborujar",
    "aborujaría": "aborujar",
    "aborujaríais": "aborujar",
    "aborujaríamos": "aborujar",
    "aborujarían": "aborujar",
    "aborujarías": "aborujar",
    "aborujas": "aborujar",
    "aborujase": "aborujar",
    "aborujaseis": "aborujar",
    "aborujasen": "aborujar",
    "aborujases": "aborujar",
    "aborujaste": "aborujar",
    "aborujasteis": "aborujar",
    "aboruje": "aborujar",
    "aborujemos": "aborujar",
    "aborujen": "aborujar",
    "aborujes": "aborujar",
    "aborujo": "aborujar",
    "aborujábamos": "aborujar",
    "aborujáis": "aborujar",
    "aborujáramos": "aborujar",
    "aborujáremos": "aborujar",
    "aborujásemos": "aborujar",
    "aborujé": "aborujar",
    "aborujéis": "aborujar",
    "aborujó": "aborujar",
    "aborígenes": "aborigen",
    "abotaga": "abotagar",
    "abotagaba": "abotagar",
    "abotagabais": "abotagar",
    "abotagaban": "abotagar",
    "abotagabas": "abotagar",
    "abotagad": "abotagar",
    "abotagada": "abotagar",
    "abotagadas": "abotagar",
    "abotagado": "abotagar",
    "abotagados": "abotagar",
    "abotagamientos": "abotagamiento",
    "abotagamos": "abotagar",
    "abotagan": "abotagar",
    "abotagando": "abotagar",
    "abotagara": "abotagar",
    "abotagarais": "abotagar",
    "abotagaran": "abotagar",
    "abotagaras": "abotagar",
    "abotagare": "abotagar",
    "abotagareis": "abotagar",
    "abotagaremos": "abotagar",
    "abotagaren": "abotagar",
    "abotagares": "abotagar",
    "abotagaron": "abotagar",
    "abotagará": "abotagar",
    "abotagarán": "abotagar",
    "abotagarás": "abotagar",
    "abotagaré": "abotagar",
    "abotagaréis": "abotagar",
    "abotagaría": "abotagar",
    "abotagaríais": "abotagar",
    "abotagaríamos": "abotagar",
    "abotagarían": "abotagar",
    "abotagarías": "abotagar",
    "abotagas": "abotagar",
    "abotagase": "abotagar",
    "abotagaseis": "abotagar",
    "abotagasen": "abotagar",
    "abotagases": "abotagar",
    "abotagaste": "abotagar",
    "abotagasteis": "abotagar",
    "abotago": "abotagar",
    "abotague": "abotagar",
    "abotaguemos": "abotagar",
    "abotaguen": "abotagar",
    "abotagues": "abotagar",
    "abotagué": "abotagar",
    "abotaguéis": "abotagar",
    "abotagábamos": "abotagar",
    "abotagáis": "abotagar",
    "abotagáramos": "abotagar",
    "abotagáremos": "abotagar",
    "abotagásemos": "abotagar",
    "abotagó": "abotagar",
    "abotarga": "abotargar",
    "abotargaba": "abotargar",
    "abotargabais": "abotargar",
    "abotargaban": "abotargar",
    "abotargabas": "abotargar",
    "abotargad": "abotargar",
    "abotargada": "abotargar",
    "abotargadas": "abotargar",
    "abotargado": "abotargar",
    "abotargados": "abotargar",
    "abotargamientos": "abotargamiento",
    "abotargamos": "abotargar",
    "abotargan": "abotargar",
    "abotargando": "abotargar",
    "abotargara": "abotargar",
    "abotargarais": "abotargar",
    "abotargaran": "abotargar",
    "abotargaras": "abotargar",
    "abotargare": "abotargar",
    "abotargareis": "abotargar",
    "abotargaremos": "abotargar",
    "abotargaren": "abotargar",
    "abotargares": "abotargar",
    "abotargaron": "abotargar",
    "abotargará": "abotargar",
    "abotargarán": "abotargar",
    "abotargarás": "abotargar",
    "abotargaré": "abotargar",
    "abotargaréis": "abotargar",
    "abotargaría": "abotargar",
    "abotargaríais": "abotargar",
    "abotargaríamos": "abotargar",
    "abotargarían": "abotargar",
    "abotargarías": "abotargar",
    "abotargas": "abotargar",
    "abotargase": "abotargar",
    "abotargaseis": "abotargar",
    "abotargasen": "abotargar",
    "abotargases": "abotargar",
    "abotargaste": "abotargar",
    "abotargasteis": "abotargar",
    "abotargo": "abotargar",
    "abotargue": "abotargar",
    "abotarguemos": "abotargar",
    "abotarguen": "abotargar",
    "abotargues": "abotargar",
    "abotargué": "abotargar",
    "abotarguéis": "abotargar",
    "abotargábamos": "abotargar",
    "abotargáis": "abotargar",
    "abotargáramos": "abotargar",
    "abotargáremos": "abotargar",
    "abotargásemos": "abotargar",
    "abotargó": "abotargar",
    "abotinada": "abotinado",
    "abotinadas": "abotinado",
    "abotinados": "abotinado",
    "abotona": "abotonar",
    "abotonaba": "abotonar",
    "abotonabais": "abotonar",
    "abotonaban": "abotonar",
    "abotonabas": "abotonar",
    "abotonad": "abotonar",
    "abotonada": "abotonar",
    "abotonadas": "abotonar",
    "abotonado": "abotonar",
    "abotonadores": "abotonador",
    "abotonados": "abotonar",
    "abotonaduras": "abotonadura",
    "abotonamos": "abotonar",
    "abotonan": "abotonar",
    "abotonando": "abotonar",
    "abotonara": "abotonar",
    "abotonarais": "abotonar",
    "abotonaran": "abotonar",
    "abotonaras": "abotonar",
    "abotonare": "abotonar",
    "abotonareis": "abotonar",
    "abotonaremos": "abotonar",
    "abotonaren": "abotonar",
    "abotonares": "abotonar",
    "abotonaron": "abotonar",
    "abotonará": "abotonar",
    "abotonarán": "abotonar",
    "abotonarás": "abotonar",
    "abotonaré": "abotonar",
    "abotonaréis": "abotonar",
    "abotonaría": "abotonar",
    "abotonaríais": "abotonar",
    "abotonaríamos": "abotonar",
    "abotonarían": "abotonar",
    "abotonarías": "abotonar",
    "abotonas": "abotonar",
    "abotonase": "abotonar",
    "abotonaseis": "abotonar",
    "abotonasen": "abotonar",
    "abotonases": "abotonar",
    "abotonaste": "abotonar",
    "abotonasteis": "abotonar",
    "abotone": "abotonar",
    "abotonemos": "abotonar",
    "abotonen": "abotonar",
    "abotones": "abotonar",
    "abotono": "abotonar",
    "abotonábamos": "abotonar",
    "abotonáis": "abotonar",
    "abotonáramos": "abotonar",
    "abotonáremos": "abotonar",
    "abotonásemos": "abotonar",
    "abotoné": "abotonar",
    "abotonéis": "abotonar",
    "abotonó": "abotonar",
    "aboveda": "abovedar",
    "abovedaba": "abovedar",
    "abovedabais": "abovedar",
    "abovedaban": "abovedar",
    "abovedabas": "abovedar",
    "abovedad": "abovedar",
    "abovedada": "abovedar",
    "abovedadas": "abovedar",
    "abovedado": "abovedar",
    "abovedados": "abovedar",
    "abovedamientos": "abovedamiento",
    "abovedamos": "abovedar",
    "abovedan": "abovedar",
    "abovedando": "abovedar",
    "abovedara": "abovedar",
    "abovedarais": "abovedar",
    "abovedaran": "abovedar",
    "abovedaras": "abovedar",
    "abovedare": "abovedar",
    "abovedareis": "abovedar",
    "abovedaremos": "abovedar",
    "abovedaren": "abovedar",
    "abovedares": "abovedar",
    "abovedaron": "abovedar",
    "abovedará": "abovedar",
    "abovedarán": "abovedar",
    "abovedarás": "abovedar",
    "abovedaré": "abovedar",
    "abovedaréis": "abovedar",
    "abovedaría": "abovedar",
    "abovedaríais": "abovedar",
    "abovedaríamos": "abovedar",
    "abovedarían": "abovedar",
    "abovedarías": "abovedar",
    "abovedas": "abovedar",
    "abovedase": "abovedar",
    "abovedaseis": "abovedar",
    "abovedasen": "abovedar",
    "abovedases": "abovedar",
    "abovedaste": "abovedar",
    "abovedasteis": "abovedar",
    "abovede": "abovedar",
    "abovedemos": "abovedar",
    "aboveden": "abovedar",
    "abovedes": "abovedar",
    "abovedo": "abovedar",
    "abovedábamos": "abovedar",
    "abovedáis": "abovedar",
    "abovedáramos": "abovedar",
    "abovedáremos": "abovedar",
    "abovedásemos": "abovedar",
    "abovedé": "abovedar",
    "abovedéis": "abovedar",
    "abovedó": "abovedar",
    "aboya": "aboyar",
    "aboyaba": "aboyar",
    "aboyabais": "aboyar",
    "aboyaban": "aboyar",
    "aboyabas": "aboyar",
    "aboyad": "aboyar",
    "aboyada": "aboyar",
    "aboyadas": "aboyar",
    "aboyado": "aboyar",
    "aboyados": "aboyar",
    "aboyamos": "aboyar",
    "aboyan": "aboyar",
    "aboyando": "aboyar",
    "aboyara": "aboyar",
    "aboyarais": "aboyar",
    "aboyaran": "aboyar",
    "aboyaras": "aboyar",
    "aboyare": "aboyar",
    "aboyareis": "aboyar",
    "aboyaremos": "aboyar",
    "aboyaren": "aboyar",
    "aboyares": "aboyar",
    "aboyaron": "aboyar",
    "aboyará": "aboyar",
    "aboyarán": "aboyar",
    "aboyarás": "aboyar",
    "aboyaré": "aboyar",
    "aboyaréis": "aboyar",
    "aboyaría": "aboyar",
    "aboyaríais": "aboyar",
    "aboyaríamos": "aboyar",
    "aboyarían": "aboyar",
    "aboyarías": "aboyar",
    "aboyas": "aboyar",
    "aboyase": "aboyar",
    "aboyaseis": "aboyar",
    "aboyasen": "aboyar",
    "aboyases": "aboyar",
    "aboyaste": "aboyar",
    "aboyasteis": "aboyar",
    "aboye": "aboyar",
    "aboyemos": "aboyar",
    "aboyen": "aboyar",
    "aboyes": "aboyar",
    "aboyo": "aboyar",
    "aboyábamos": "aboyar",
    "aboyáis": "aboyar",
    "aboyáramos": "aboyar",
    "aboyáremos": "aboyar",
    "aboyásemos": "aboyar",
    "aboyé": "aboyar",
    "aboyéis": "aboyar",
    "aboyó": "aboyar",
    "abozala": "abozalar",
    "abozalaba": "abozalar",
    "abozalabais": "abozalar",
    "abozalaban": "abozalar",
    "abozalabas": "abozalar",
    "abozalad": "abozalar",
    "abozalada": "abozalar",
    "abozaladas": "abozalar",
    "abozalado": "abozalar",
    "abozalados": "abozalar",
    "abozalamos": "abozalar",
    "abozalan": "abozalar",
    "abozalando": "abozalar",
    "abozalara": "abozalar",
    "abozalarais": "abozalar",
    "abozalaran": "abozalar",
    "abozalaras": "abozalar",
    "abozalare": "abozalar",
    "abozalareis": "abozalar",
    "abozalaremos": "abozalar",
    "abozalaren": "abozalar",
    "abozalares": "abozalar",
    "abozalaron": "abozalar",
    "abozalará": "abozalar",
    "abozalarán": "abozalar",
    "abozalarás": "abozalar",
    "abozalaré": "abozalar",
    "abozalaréis": "abozalar",
    "abozalaría": "abozalar",
    "abozalaríais": "abozalar",
    "abozalaríamos": "abozalar",
    "abozalarían": "abozalar",
    "abozalarías": "abozalar",
    "abozalas": "abozalar",
    "abozalase": "abozalar",
    "abozalaseis": "abozalar",
    "abozalasen": "abozalar",
    "abozalases": "abozalar",
    "abozalaste": "abozalar",
    "abozalasteis": "abozalar",
    "abozale": "abozalar",
    "abozalemos": "abozalar",
    "abozalen": "abozalar",
    "abozales": "abozalar",
    "abozalo": "abozalar",
    "abozalábamos": "abozalar",
    "abozaláis": "abozalar",
    "abozaláramos": "abozalar",
    "abozaláremos": "abozalar",
    "abozalásemos": "abozalar",
    "abozalé": "abozalar",
    "abozaléis": "abozalar",
    "abozaló": "abozalar",
    "abr.": "abril",
    "abra": "abrir",
    "abracadabrantes": "abracadabrante",
    "abracadabras": "abracadabra",
    "abrace": "abrazar",
    "abracemos": "abrazar",
    "abracen": "abrazar",
    "abraces": "abrazar",
    "abracé": "abrazar",
    "abracéis": "abrazar",
    "abramos": "abrir",
    "abran": "abrir",
    "abras": "abrir",
    "abrasa": "abrasar",
    "abrasaba": "abrasar",
    "abrasabais": "abrasar",
    "abrasaban": "abrasar",
    "abrasabas": "abrasar",
    "abrasad": "abrasar",
    "abrasada": "abrasar",
    "abrasadas": "abrasar",
    "abrasado": "abrasar",
    "abrasadora": "abrasador",
    "abrasadoras": "abrasador",
    "abrasadores": "abrasador",
    "abrasados": "abrasar",
    "abrasamientos": "abrasamiento",
    "abrasamos": "abrasar",
    "abrasan": "abrasar",
    "abrasando": "abrasar",
    "abrasara": "abrasar",
    "abrasarais": "abrasar",
    "abrasaran": "abrasar",
    "abrasaras": "abrasar",
    "abrasare": "abrasar",
    "abrasareis": "abrasar",
    "abrasaremos": "abrasar",
    "abrasaren": "abrasar",
    "abrasares": "abrasar",
    "abrasaron": "abrasar",
    "abrasará": "abrasar",
    "abrasarán": "abrasar",
    "abrasarás": "abrasar",
    "abrasaré": "abrasar",
    "abrasaréis": "abrasar",
    "abrasaría": "abrasar",
    "abrasaríais": "abrasar",
    "abrasaríamos": "abrasar",
    "abrasarían": "abrasar",
    "abrasarías": "abrasar",
    "abrasas": "abrasar",
    "abrasase": "abrasar",
    "abrasaseis": "abrasar",
    "abrasasen": "abrasar",
    "abrasases": "abrasar",
    "abrasaste": "abrasar",
    "abrasasteis": "abrasar",
    "abrase": "abrasar",
    "abrasemos": "abrasar",
    "abrasen": "abrasar",
    "abrases": "abrasar",
    "abrasines": "abrasín",
    "abrasiones": "abrasión",
    "abrasiva": "abrasivo",
    "abrasivas": "abrasivo",
    "abrasividades": "abrasividad",
    "abrasivos": "abrasivo",
    "abraso": "abrasar",
    "abrasábamos": "abrasar",
    "abrasáis": "abrasar",
    "abrasáramos": "abrasar",
    "abrasáremos": "abrasar",
    "abrasásemos": "abrasar",
    "abrasé": "abrasar",
    "abraséis": "abrasar",
    "abrasímetros": "abrasímetro",
    "abrasó": "abrasar",
    "abraza": "abrazar",
    "abrazaba": "abrazar",
    "abrazabais": "abrazar",
    "abrazaban": "abrazar",
    "abrazabas": "abrazar",
    "abrazad": "abrazar",
    "abrazada": "abrazar",
    "abrazadas": "abrazar",
    "abrazaderas": "abrazadera",
    "abrazado": "abrazar",
    "abrazadora": "abrazador",
    "abrazadoras": "abrazador",
    "abrazadores": "abrazador",
    "abrazados": "abrazar",
    "abrazamientos": "abrazamiento",
    "abrazamos": "abrazar",
    "abrazan": "abrazar",
    "abrazando": "abrazar",
    "abrazara": "abrazar",
    "abrazarais": "abrazar",
    "abrazaran": "abrazar",
    "abrazaras": "abrazar",
    "abrazare": "abrazar",
    "abrazareis": "abrazar",
    "abrazaremos": "abrazar",
    "abrazaren": "abrazar",
    "abrazares": "abrazar",
    "abrazaron": "abrazar",
    "abrazará": "abrazar",
    "abrazarán": "abrazar",
    "abrazarás": "abrazar",
    "abrazaré": "abrazar",
    "abrazaréis": "abrazar",
    "abrazaría": "abrazar",
    "abrazaríais": "abrazar",
    "abrazaríamos": "abrazar",
    "abrazarían": "abrazar",
    "abrazarías": "abrazar",
    "abrazas": "abrazar",
    "abrazase": "abrazar",
    "abrazaseis": "abrazar",
    "abrazasen": "abrazar",
    "abrazases": "abrazar",
    "abrazaste": "abrazar",
    "abrazasteis": "abrazar",
    "abrazo": "abrazar",
    "abrazos": "abrazo",
    "abrazábamos": "abrazar",
    "abrazáis": "abrazar",
    "abrazáramos": "abrazar",
    "abrazáremos": "abrazar",
    "abrazásemos": "abrazar",
    "abrazó": "abrazar",
    "abre": "abrir",
    "abreaciones": "abreación",
    "abren": "abrir",
    "abrepuños": "abrepuño",
    "abres": "abrir",
    "abreva": "abrevar",
    "abrevaba": "abrevar",
    "abrevabais": "abrevar",
    "abrevaban": "abrevar",
    "abrevabas": "abrevar",
    "abrevad": "abrevar",
    "abrevada": "abrevar",
    "abrevadas": "abrevar",
    "abrevaderos": "abrevadero",
    "abrevado": "abrevar",
    "abrevadores": "abrevador",
    "abrevados": "abrevar",
    "abrevamos": "abrevar",
    "abrevan": "abrevar",
    "abrevando": "abrevar",
    "abrevara": "abrevar",
    "abrevarais": "abrevar",
    "abrevaran": "abrevar",
    "abrevaras": "abrevar",
    "abrevare": "abrevar",
    "abrevareis": "abrevar",
    "abrevaremos": "abrevar",
    "abrevaren": "abrevar",
    "abrevares": "abrevar",
    "abrevaron": "abrevar",
    "abrevará": "abrevar",
    "abrevarán": "abrevar",
    "abrevarás": "abrevar",
    "abrevaré": "abrevar",
    "abrevaréis": "abrevar",
    "abrevaría": "abrevar",
    "abrevaríais": "abrevar",
    "abrevaríamos": "abrevar",
    "abrevarían": "abrevar",
    "abrevarías": "abrevar",
    "abrevas": "abrevar",
    "abrevase": "abrevar",
    "abrevaseis": "abrevar",
    "abrevasen": "abrevar",
    "abrevases": "abrevar",
    "abrevaste": "abrevar",
    "abrevasteis": "abrevar",
    "abreve": "abrevar",
    "abrevemos": "abrevar",
    "abreven": "abrevar",
    "abreves": "abrevar",
    "abrevia": "abreviar",
    "abreviaba": "abreviar",
    "abreviabais": "abreviar",
    "abreviaban": "abreviar",
    "abreviabas": "abreviar",
    "abreviables": "abreviable",
    "abreviaciones": "abreviación",
    "abreviad": "abreviar",
    "abreviada": "abreviar",
    "abreviadas": "abreviar",
    "abreviado": "abreviar",
    "abreviados": "abreviar",
    "abreviamientos": "abreviamiento",
    "abreviamos": "abreviar",
    "abrevian": "abreviar",
    "abreviando": "abreviar",
    "abreviara": "abreviar",
    "abreviarais": "abreviar",
    "abreviaran": "abreviar",
    "abreviaras": "abreviar",
    "abreviare": "abreviar",
    "abreviareis": "abreviar",
    "abreviaremos": "abreviar",
    "abreviaren": "abreviar",
    "abreviares": "abreviar",
    "abreviaron": "abreviar",
    "abreviará": "abreviar",
    "abreviarán": "abreviar",
    "abreviarás": "abreviar",
    "abreviaré": "abreviar",
    "abreviaréis": "abreviar",
    "abreviaría": "abreviar",
    "abreviaríais": "abreviar",
    "abreviaríamos": "abreviar",
    "abreviarían": "abreviar",
    "abreviarías": "abreviar",
    "abrevias": "abreviar",
    "abreviase": "abreviar",
    "abreviaseis": "abreviar",
    "abreviasen": "abreviar",
    "abreviases": "abreviar",
    "abreviaste": "abreviar",
    "abreviasteis": "abreviar",
    "abreviaturas": "abreviatura",
    "abrevie": "abreviar",
    "abreviemos": "abreviar",
    "abrevien": "abreviar",
    "abrevies": "abreviar",
    "abrevio": "abreviar",
    "abreviábamos": "abreviar",
    "abreviáis": "abreviar",
    "abreviáramos": "abreviar",
    "abreviáremos": "abreviar",
    "abreviásemos": "abreviar",
    "abrevié": "abreviar",
    "abreviéis": "abreviar",
    "abrevió": "abreviar",
    "abrevo": "abrevar",
    "abrevábamos": "abrevar",
    "abreváis": "abrevar",
    "abreváramos": "abrevar",
    "abreváremos": "abrevar",
    "abrevásemos": "abrevar",
    "abrevé": "abrevar",
    "abrevéis": "abrevar",
    "abrevó": "abrevar",
    "abribona": "abribonar",
    "abribonaba": "abribonar",
    "abribonabais": "abribonar",
    "abribonaban": "abribonar",
    "abribonabas": "abribonar",
    "abribonad": "abribonar",
    "abribonada": "abribonar",
    "abribonadas": "abribonar",
    "abribonado": "abribonar",
    "abribonados": "abribonar",
    "abribonamos": "abribonar",
    "abribonan": "abribonar",
    "abribonando": "abribonar",
    "abribonara": "abribonar",
    "abribonarais": "abribonar",
    "abribonaran": "abribonar",
    "abribonaras": "abribonar",
    "abribonare": "abribonar",
    "abribonareis": "abribonar",
    "abribonaremos": "abribonar",
    "abribonaren": "abribonar",
    "abribonares": "abribonar",
    "abribonaron": "abribonar",
    "abribonará": "abribonar",
    "abribonarán": "abribonar",
    "abribonarás": "abribonar",
    "abribonaré": "abribonar",
    "abribonaréis": "abribonar",
    "abribonaría": "abribonar",
    "abribonaríais": "abribonar",
    "abribonaríamos": "abribonar",
    "abribonarían": "abribonar",
    "abribonarías": "abribonar",
    "abribonas": "abribonar",
    "abribonase": "abribonar",
    "abribonaseis": "abribonar",
    "abribonasen": "abribonar",
    "abribonases": "abribonar",
    "abribonaste": "abribonar",
    "abribonasteis": "abribonar",
    "abribone": "abribonar",
    "abribonemos": "abribonar",
    "abribonen": "abribonar",
    "abribones": "abribonar",
    "abribono": "abribonar",
    "abribonábamos": "abribonar",
    "abribonáis": "abribonar",
    "abribonáramos": "abribonar",
    "abribonáremos": "abribonar",
    "abribonásemos": "abribonar",
    "abriboné": "abribonar",
    "abribonéis": "abribonar",
    "abribonó": "abribonar",
    "abrid": "abrir",
    "abridera": "abridero",
    "abrideras": "abridero",
    "abrideros": "abridero",
    "abridora": "abridor",
    "abridoras": "abridora",
    "abridores": "abridor",
    "abriendo": "abrir",
    "abriera": "abrir",
    "abrierais": "abrir",
    "abrieran": "abrir",
    "abrieras": "abrir",
    "abriere": "abrir",
    "abriereis": "abrir",
    "abrieren": "abrir",
    "abrieres": "abrir",
    "abrieron": "abrir",
    "abriese": "abrir",
    "abrieseis": "abrir",
    "abriesen": "abrir",
    "abrieses": "abrir",
    "abriga": "abrigar",
    "abrigaba": "abrigar",
    "abrigabais": "abrigar",
    "abrigaban": "abrigar",
    "abrigabas": "abrigar",
    "abrigad": "abrigar",
    "abrigada": "abrigar",
    "abrigadas": "abrigar",
    "abrigaderos": "abrigadero",
    "abrigado": "abrigar",
    "abrigados": "abrigar",
    "abrigamos": "abrigar",
    "abrigan": "abrigar",
    "abrigando": "abrigar",
    "abrigara": "abrigar",
    "abrigarais": "abrigar",
    "abrigaran": "abrigar",
    "abrigaras": "abrigar",
    "abrigare": "abrigar",
    "abrigareis": "abrigar",
    "abrigaremos": "abrigar",
    "abrigaren": "abrigar",
    "abrigares": "abrigar",
    "abrigaron": "abrigar",
    "abrigará": "abrigar",
    "abrigarán": "abrigar",
    "abrigarás": "abrigar",
    "abrigaré": "abrigar",
    "abrigaréis": "abrigar",
    "abrigaría": "abrigar",
    "abrigaríais": "abrigar",
    "abrigaríamos": "abrigar",
    "abrigarían": "abrigar",
    "abrigarías": "abrigar",
    "abrigas": "abrigar",
    "abrigase": "abrigar",
    "abrigaseis": "abrigar",
    "abrigasen": "abrigar",
    "abrigases": "abrigar",
    "abrigaste": "abrigar",
    "abrigasteis": "abrigar",
    "abrigaños": "abrigaño",
    "abrigo": "abrigar",
    "abrigos": "abrigo",
    "abrigue": "abrigar",
    "abriguemos": "abrigar",
    "abriguen": "abrigar",
    "abrigues": "abrigar",
    "abrigué": "abrigar",
    "abriguéis": "abrigar",
    "abrigábamos": "abrigar",
    "abrigáis": "abrigar",
    "abrigáramos": "abrigar",
    "abrigáremos": "abrigar",
    "abrigásemos": "abrigar",
    "abrigó": "abrigar",
    "abriles": "abril",
    "abrileña": "abrileño",
    "abrileñas": "abrileño",
    "abrileños": "abrileño",
    "abrillanta": "abrillantar",
    "abrillantaba": "abrillantar",
    "abrillantabais": "abrillantar",
    "abrillantaban": "abrillantar",
    "abrillantabas": "abrillantar",
    "abrillantad": "abrillantar",
    "abrillantada": "abrillantar",
    "abrillantadas": "abrillantar",
    "abrillantado": "abrillantar",
    "abrillantadores": "abrillantador",
    "abrillantados": "abrillantar",
    "abrillantamientos": "abrillantamiento",
    "abrillantamos": "abrillantar",
    "abrillantan": "abrillantar",
    "abrillantando": "abrillantar",
    "abrillantara": "abrillantar",
    "abrillantarais": "abrillantar",
    "abrillantaran": "abrillantar",
    "abrillantaras": "abrillantar",
    "abrillantare": "abrillantar",
    "abrillantareis": "abrillantar",
    "abrillantaremos": "abrillantar",
    "abrillantaren": "abrillantar",
    "abrillantares": "abrillantar",
    "abrillantaron": "abrillantar",
    "abrillantará": "abrillantar",
    "abrillantarán": "abrillantar",
    "abrillantarás": "abrillantar",
    "abrillantaré": "abrillantar",
    "abrillantaréis": "abrillantar",
    "abrillantaría": "abrillantar",
    "abrillantaríais": "abrillantar",
    "abrillantaríamos": "abrillantar",
    "abrillantarían": "abrillantar",
    "abrillantarías": "abrillantar",
    "abrillantas": "abrillantar",
    "abrillantase": "abrillantar",
    "abrillantaseis": "abrillantar",
    "abrillantasen": "abrillantar",
    "abrillantases": "abrillantar",
    "abrillantaste": "abrillantar",
    "abrillantasteis": "abrillantar",
    "abrillante": "abrillantar",
    "abrillantemos": "abrillantar",
    "abrillanten": "abrillantar",
    "abrillantes": "abrillantar",
    "abrillanto": "abrillantar",
    "abrillantábamos": "abrillantar",
    "abrillantáis": "abrillantar",
    "abrillantáramos": "abrillantar",
    "abrillantáremos": "abrillantar",
    "abrillantásemos": "abrillantar",
    "abrillanté": "abrillantar",
    "abrillantéis": "abrillantar",
    "abrillantó": "abrillantar",
    "abrimos": "abrir",
    "abriremos": "abrir",
    "abrirá": "abrir",
    "abrirán": "abrir",
    "abrirás": "abrir",
    "abriré": "abrir",
    "abriréis": "abrir",
    "abriría": "abrir",
    "abriríais": "abrir",
    "abriríamos": "abrir",
    "abrirían": "abrir",
    "abrirías": "abrir",
    "abriste": "abrir",
    "abristeis": "abrir",
    "abriéramos": "abrir",
    "abriéremos": "abrir",
    "abriésemos": "abrir",
    "abrió": "abrir",
    "abro": "abrir",
    "abroca": "abrocar",
    "abrocaba": "abrocar",
    "abrocabais": "abrocar",
    "abrocaban": "abrocar",
    "abrocabas": "abrocar",
    "abrocad": "abrocar",
    "abrocada": "abrocar",
    "abrocadas": "abrocar",
    "abrocado": "abrocar",
    "abrocados": "abrocar",
    "abrocamos": "abrocar",
    "abrocan": "abrocar",
    "abrocando": "abrocar",
    "abrocara": "abrocar",
    "abrocarais": "abrocar",
    "abrocaran": "abrocar",
    "abrocaras": "abrocar",
    "abrocare": "abrocar",
    "abrocareis": "abrocar",
    "abrocaremos": "abrocar",
    "abrocaren": "abrocar",
    "abrocares": "abrocar",
    "abrocaron": "abrocar",
    "abrocará": "abrocar",
    "abrocarán": "abrocar",
    "abrocarás": "abrocar",
    "abrocaré": "abrocar",
    "abrocaréis": "abrocar",
    "abrocaría": "abrocar",
    "abrocaríais": "abrocar",
    "abrocaríamos": "abrocar",
    "abrocarían": "abrocar",
    "abrocarías": "abrocar",
    "abrocas": "abrocar",
    "abrocase": "abrocar",
    "abrocaseis": "abrocar",
    "abrocasen": "abrocar",
    "abrocases": "abrocar",
    "abrocaste": "abrocar",
    "abrocasteis": "abrocar",
    "abrocha": "abrochar",
    "abrochaba": "abrochar",
    "abrochabais": "abrochar",
    "abrochaban": "abrochar",
    "abrochabas": "abrochar",
    "abrochad": "abrochar",
    "abrochada": "abrochar",
    "abrochadas": "abrochar",
    "abrochado": "abrochar",
    "abrochadora": "abrochador",
    "abrochadoras": "abrochador",
    "abrochadores": "abrochador",
    "abrochados": "abrochar",
    "abrochaduras": "abrochadura",
    "abrochamientos": "abrochamiento",
    "abrochamos": "abrochar",
    "abrochan": "abrochar",
    "abrochando": "abrochar",
    "abrochara": "abrochar",
    "abrocharais": "abrochar",
    "abrocharan": "abrochar",
    "abrocharas": "abrochar",
    "abrochare": "abrochar",
    "abrochareis": "abrochar",
    "abrocharemos": "abrochar",
    "abrocharen": "abrochar",
    "abrochares": "abrochar",
    "abrocharon": "abrochar",
    "abrochará": "abrochar",
    "abrocharán": "abrochar",
    "abrocharás": "abrochar",
    "abrocharé": "abrochar",
    "abrocharéis": "abrochar",
    "abrocharía": "abrochar",
    "abrocharíais": "abrochar",
    "abrocharíamos": "abrochar",
    "abrocharían": "abrochar",
    "abrocharías": "abrochar",
    "abrochas": "abrochar",
    "abrochase": "abrochar",
    "abrochaseis": "abrochar",
    "abrochasen": "abrochar",
    "abrochases": "abrochar",
    "abrochaste": "abrochar",
    "abrochasteis": "abrochar",
    "abroche": "abrochar",
    "abrochemos": "abrochar",
    "abrochen": "abrochar",
    "abroches": "abrochar",
    "abrocho": "abrochar",
    "abrochábamos": "abrochar",
    "abrocháis": "abrochar",
    "abrocháramos": "abrochar",
    "abrocháremos": "abrochar",
    "abrochásemos": "abrochar",
    "abroché": "abrochar",
    "abrochéis": "abrochar",
    "abrochó": "abrochar",
    "abroco": "abrocar",
    "abrocábamos": "abrocar",
    "abrocáis": "abrocar",
    "abrocáramos": "abrocar",
    "abrocáremos": "abrocar",
    "abrocásemos": "abrocar",
    "abrocó": "abrocar",
    "abrocómidos": "abrocómido",
    "abroga": "abrogar",
    "abrogaba": "abrogar",
    "abrogabais": "abrogar",
    "abrogaban": "abrogar",
    "abrogabas": "abrogar",
    "abrogaciones": "abrogación",
    "abrogad": "abrogar",
    "abrogada": "abrogar",
    "abrogadas": "abrogar",
    "abrogado": "abrogar",
    "abrogados": "abrogar",
    "abrogamos": "abrogar",
    "abrogan": "abrogar",
    "abrogando": "abrogar",
    "abrogara": "abrogar",
    "abrogarais": "abrogar",
    "abrogaran": "abrogar",
    "abrogaras": "abrogar",
    "abrogare": "abrogar",
    "abrogareis": "abrogar",
    "abrogaremos": "abrogar",
    "abrogaren": "abrogar",
    "abrogares": "abrogar",
    "abrogaron": "abrogar",
    "abrogará": "abrogar",
    "abrogarán": "abrogar",
    "abrogarás": "abrogar",
    "abrogaré": "abrogar",
    "abrogaréis": "abrogar",
    "abrogaría": "abrogar",
    "abrogaríais": "abrogar",
    "abrogaríamos": "abrogar",
    "abrogarían": "abrogar",
    "abrogarías": "abrogar",
    "abrogas": "abrogar",
    "abrogase": "abrogar",
    "abrogaseis": "abrogar",
    "abrogasen": "abrogar",
    "abrogases": "abrogar",
    "abrogaste": "abrogar",
    "abrogasteis": "abrogar",
    "abrogatoria": "abrogatorio",
    "abrogatorias": "abrogatorio",
    "abrogatorios": "abrogatorio",
    "abrogo": "abrogar",
    "abrogue": "abrogar",
    "abroguemos": "abrogar",
    "abroguen": "abrogar",
    "abrogues": "abrogar",
    "abrogué": "abrogar",
    "abroguéis": "abrogar",
    "abrogábamos": "abrogar",
    "abrogáis": "abrogar",
    "abrogáramos": "abrogar",
    "abrogáremos": "abrogar",
    "abrogásemos": "abrogar",
    "abrogó": "abrogar",
    "abrojales": "abrojal",
    "abrojos": "abrojo",
    "abroma": "abromar",
    "abromaba": "abromar",
    "abromabais": "abromar",
    "abromaban": "abromar",
    "abromabas": "abromar",
    "abromad": "abromar",
    "abromada": "abromar",
    "abromadas": "abromar",
    "abromado": "abromar",
    "abromados": "abromar",
    "abromamos": "abromar",
    "abroman": "abromar",
    "abromando": "abromar",
    "abromara": "abromar",
    "abromarais": "abromar",
    "abromaran": "abromar",
    "abromaras": "abromar",
    "abromare": "abromar",
    "abromareis": "abromar",
    "abromaremos": "abromar",
    "abromaren": "abromar",
    "abromares": "abromar",
    "abromaron": "abromar",
    "abromará": "abromar",
    "abromarán": "abromar",
    "abromarás": "abromar",
    "abromaré": "abromar",
    "abromaréis": "abromar",
    "abromaría": "abromar",
    "abromaríais": "abromar",
    "abromaríamos": "abromar",
    "abromarían": "abromar",
    "abromarías": "abromar",
    "abromas": "abromar",
    "abromase": "abromar",
    "abromaseis": "abromar",
    "abromasen": "abromar",
    "abromases": "abromar",
    "abromaste": "abromar",
    "abromasteis": "abromar",
    "abrome": "abromar",
    "abromemos": "abromar",
    "abromen": "abromar",
    "abromes": "abromar",
    "abromo": "abromar",
    "abromábamos": "abromar",
    "abromáis": "abromar",
    "abromáramos": "abromar",
    "abromáremos": "abromar",
    "abromásemos": "abromar",
    "abromé": "abromar",
    "abroméis": "abromar",
    "abromó": "abromar",
    "abronca": "abroncar",
    "abroncaba": "abroncar",
    "abroncabais": "abroncar",
    "abroncaban": "abroncar",
    "abroncabas": "abroncar",
    "abroncad": "abroncar",
    "abroncada": "abroncar",
    "abroncadas": "abroncar",
    "abroncado": "abroncar",
    "abroncados": "abroncar",
    "abroncamos": "abroncar",
    "abroncan": "abroncar",
    "abroncando": "abroncar",
    "abroncara": "abroncar",
    "abroncarais": "abroncar",
    "abroncaran": "abroncar",
    "abroncaras": "abroncar",
    "abroncare": "abroncar",
    "abroncareis": "abroncar",
    "abroncaremos": "abroncar",
    "abroncaren": "abroncar",
    "abroncares": "abroncar",
    "abroncaron": "abroncar",
    "abroncará": "abroncar",
    "abroncarán": "abroncar",
    "abroncarás": "abroncar",
    "abroncaré": "abroncar",
    "abroncaréis": "abroncar",
    "abroncaría": "abroncar",
    "abroncaríais": "abroncar",
    "abroncaríamos": "abroncar",
    "abroncarían": "abroncar",
    "abroncarías": "abroncar",
    "abroncas": "abroncar",
    "abroncase": "abroncar",
    "abroncaseis": "abroncar",
    "abroncasen": "abroncar",
    "abroncases": "abroncar",
    "abroncaste": "abroncar",
    "abroncasteis": "abroncar",
    "abronco": "abroncar",
    "abroncábamos": "abroncar",
    "abroncáis": "abroncar",
    "abroncáramos": "abroncar",
    "abroncáremos": "abroncar",
    "abroncásemos": "abroncar",
    "abroncó": "abroncar",
    "abronque": "abroncar",
    "abronquemos": "abroncar",
    "abronquen": "abroncar",
    "abronques": "abroncar",
    "abronqué": "abroncar",
    "abronquéis": "abroncar",
    "abroque": "abrocar",
    "abroquela": "abroquelar",
    "abroquelaba": "abroquelar",
    "abroquelabais": "abroquelar",
    "abroquelaban": "abroquelar",
    "abroquelabas": "abroquelar",
    "abroquelad": "abroquelar",
    "abroquelada": "abroquelar",
    "abroqueladas": "abroquelar",
    "abroquelado": "abroquelar",
    "abroquelados": "abroquelar",
    "abroquelamos": "abroquelar",
    "abroquelan": "abroquelar",
    "abroquelando": "abroquelar",
    "abroquelara": "abroquelar",
    "abroquelarais": "abroquelar",
    "abroquelaran": "abroquelar",
    "abroquelaras": "abroquelar",
    "abroquelare": "abroquelar",
    "abroquelareis": "abroquelar",
    "abroquelaremos": "abroquelar",
    "abroquelaren": "abroquelar",
    "abroquelares": "abroquelar",
    "abroquelaron": "abroquelar",
    "abroquelará": "abroquelar",
    "abroquelarán": "abroquelar",
    "abroquelarás": "abroquelar",
    "abroquelaré": "abroquelar",
    "abroquelaréis": "abroquelar",
    "abroquelaría": "abroquelar",
    "abroquelaríais": "abroquelar",
    "abroquelaríamos": "abroquelar",
    "abroquelarían": "abroquelar",
    "abroquelarías": "abroquelar",
    "abroquelas": "abroquelar",
    "abroquelase": "abroquelar",
    "abroquelaseis": "abroquelar",
    "abroquelasen": "abroquelar",
    "abroquelases": "abroquelar",
    "abroquelaste": "abroquelar",
    "abroquelasteis": "abroquelar",
    "abroquele": "abroquelar",
    "abroquelemos": "abroquelar",
    "abroquelen": "abroquelar",
    "abroqueles": "abroquelar",
    "abroquelo": "abroquelar",
    "abroquelábamos": "abroquelar",
    "abroqueláis": "abroquelar",
    "abroqueláramos": "abroquelar",
    "abroqueláremos": "abroquelar",
    "abroquelásemos": "abroquelar",
    "abroquelé": "abroquelar",
    "abroqueléis": "abroquelar",
    "abroqueló": "abroquelar",
    "abroquemos": "abrocar",
    "abroquen": "abrocar",
    "abroques": "abrocar",
    "abroqué": "abrocar",
    "abroquéis": "abrocar",
    "abros": "abro",
    "abrotinas": "abrotina",
    "abrotoña": "abrotoñar",
    "abrotoñaba": "abrotoñar",
    "abrotoñabais": "abrotoñar",
    "abrotoñaban": "abrotoñar",
    "abrotoñabas": "abrotoñar",
    "abrotoñad": "abrotoñar",
    "abrotoñada": "abrotoñar",
    "abrotoñadas": "abrotoñar",
    "abrotoñado": "abrotoñar",
    "abrotoñados": "abrotoñar",
    "abrotoñamos": "abrotoñar",
    "abrotoñan": "abrotoñar",
    "abrotoñando": "abrotoñar",
    "abrotoñara": "abrotoñar",
    "abrotoñarais": "abrotoñar",
    "abrotoñaran": "abrotoñar",
    "abrotoñaras": "abrotoñar",
    "abrotoñare": "abrotoñar",
    "abrotoñareis": "abrotoñar",
    "abrotoñaremos": "abrotoñar",
    "abrotoñaren": "abrotoñar",
    "abrotoñares": "abrotoñar",
    "abrotoñaron": "abrotoñar",
    "abrotoñará": "abrotoñar",
    "abrotoñarán": "abrotoñar",
    "abrotoñarás": "abrotoñar",
    "abrotoñaré": "abrotoñar",
    "abrotoñaréis": "abrotoñar",
    "abrotoñaría": "abrotoñar",
    "abrotoñaríais": "abrotoñar",
    "abrotoñaríamos": "abrotoñar",
    "abrotoñarían": "abrotoñar",
    "abrotoñarías": "abrotoñar",
    "abrotoñas": "abrotoñar",
    "abrotoñase": "abrotoñar",
    "abrotoñaseis": "abrotoñar",
    "abrotoñasen": "abrotoñar",
    "abrotoñases": "abrotoñar",
    "abrotoñaste": "abrotoñar",
    "abrotoñasteis": "abrotoñar",
    "abrotoñe": "abrotoñar",
    "abrotoñemos": "abrotoñar",
    "abrotoñen": "abrotoñar",
    "abrotoñes": "abrotoñar",
    "abrotoño": "abrotoñar",
    "abrotoñábamos": "abrotoñar",
    "abrotoñáis": "abrotoñar",
    "abrotoñáramos": "abrotoñar",
    "abrotoñáremos": "abrotoñar",
    "abrotoñásemos": "abrotoñar",
    "abrotoñé": "abrotoñar",
    "abrotoñéis": "abrotoñar",
    "abrotoñó": "abrotoñar",
    "abruma": "abrumar",
    "abrumaba": "abrumar",
    "abrumabais": "abrumar",
    "abrumaban": "abrumar",
    "abrumabas": "abrumar",
    "abrumaciones": "abrumación",
    "abrumad": "abrumar",
    "abrumada": "abrumar",
    "abrumadas": "abrumar",
    "abrumado": "abrumar",
    "abrumadora": "abrumador",
    "abrumadoras": "abrumador",
    "abrumadores": "abrumador",
    "abrumados": "abrumar",
    "abrumamos": "abrumar",
    "abruman": "abrumar",
    "abrumando": "abrumar",
    "abrumara": "abrumar",
    "abrumarais": "abrumar",
    "abrumaran": "abrumar",
    "abrumaras": "abrumar",
    "abrumare": "abrumar",
    "abrumareis": "abrumar",
    "abrumaremos": "abrumar",
    "abrumaren": "abrumar",
    "abrumares": "abrumar",
    "abrumaron": "abrumar",
    "abrumará": "abrumar",
    "abrumarán": "abrumar",
    "abrumarás": "abrumar",
    "abrumaré": "abrumar",
    "abrumaréis": "abrumar",
    "abrumaría": "abrumar",
    "abrumaríais": "abrumar",
    "abrumaríamos": "abrumar",
    "abrumarían": "abrumar",
    "abrumarías": "abrumar",
    "abrumas": "abrumar",
    "abrumase": "abrumar",
    "abrumaseis": "abrumar",
    "abrumasen": "abrumar",
    "abrumases": "abrumar",
    "abrumaste": "abrumar",
    "abrumasteis": "abrumar",
    "abrume": "abrumar",
    "abrumemos": "abrumar",
    "abrumen": "abrumar",
    "abrumes": "abrumar",
    "abrumo": "abrumar",
    "abrumábamos": "abrumar",
    "abrumáis": "abrumar",
    "abrumáramos": "abrumar",
    "abrumáremos": "abrumar",
    "abrumásemos": "abrumar",
    "abrumé": "abrumar",
    "abruméis": "abrumar",
    "abrumó": "abrumar",
    "abrupta": "abrupto",
    "abruptas": "abrupto",
    "abruptos": "abrupto",
    "abrutada": "abrutado",
    "abrutadas": "abrutado",
    "abrutados": "abrutado",
    "abráis": "abrir",
    "abrí": "abrir",
    "abría": "abrir",
    "abríais": "abrir",
    "abríamos": "abrir",
    "abrían": "abrir",
    "abrías": "abrir",
    "abrís": "abrir",
    "abrótanos": "abrótano",
    "abrúptica": "abrúptico",
    "abrúpticas": "abrúptico",
    "abrúpticos": "abrúptico",
    "abscesos": "absceso",
    "abscisas": "abscisa",
    "abscisiones": "abscisión",
    "absentas": "absenta",
    "absentismos": "absentismo",
    "absentistas": "absentista",
    "absidales": "absidal",
    "absidiolas": "absidiola",
    "absidiolos": "absidiolo",
    "absintinas": "absintina",
    "absintios": "absintio",
    "absoluciones": "absolución",
    "absoluta": "absoluto",
    "absolutas": "absoluto",
    "absolutismos": "absolutismo",
    "absolutistas": "absolutista",
    "absolutoria": "absolutorio",
    "absolutorias": "absolutorio",
    "absolutorios": "absolutorio",
    "absolutos": "absoluto",
    "absolvamos": "absolver",
    "absolved": "absolver",
    "absolvedora": "absolvedor",
    "absolvedoras": "absolvedor",
    "absolvedores": "absolvedor",
    "absolvemos": "absolver",
    "absolveremos": "absolver",
    "absolverá": "absolver",
    "absolverán": "absolver",
    "absolverás": "absolver",
    "absolveré": "absolver",
    "absolveréis": "absolver",
    "absolvería": "absolver",
    "absolveríais": "absolver",
    "absolveríamos": "absolver",
    "absolverían": "absolver",
    "absolverías": "absolver",
    "absolviendo": "absolver",
    "absolviera": "absolver",
    "absolvierais": "absolver",
    "absolvieran": "absolver",
    "absolvieras": "absolver",
    "absolviere": "absolver",
    "absolviereis": "absolver",
    "absolvieren": "absolver",
    "absolvieres": "absolver",
    "absolvieron": "absolver",
    "absolviese": "absolver",
    "absolvieseis": "absolver",
    "absolviesen": "absolver",
    "absolvieses": "absolver",
    "absolvimos": "absolver",
    "absolviste": "absolver",
    "absolvisteis": "absolver",
    "absolviéramos": "absolver",
    "absolviéremos": "absolver",
    "absolviésemos": "absolver",
    "absolvió": "absolver",
    "absolváis": "absolver",
    "absolvéis": "absolver",
    "absolví": "absolver",
    "absolvía": "absolver",
    "absolvíais": "absolver",
    "absolvíamos": "absolver",
    "absolvían": "absolver",
    "absolvías": "absolver",
    "absorba": "absorber",
    "absorbamos": "absorber",
    "absorban": "absorber",
    "absorbas": "absorber",
    "absorbe": "absorber",
    "absorbed": "absorber",
    "absorbedores": "absorbedor",
    "absorbemos": "absorber",
    "absorben": "absorber",
    "absorbencias": "absorbencia",
    "absorbentes": "absorbente",
    "absorberemos": "absorber",
    "absorberá": "absorber",
    "absorberán": "absorber",
    "absorberás": "absorber",
    "absorberé": "absorber",
    "absorberéis": "absorber",
    "absorbería": "absorber",
    "absorberíais": "absorber",
    "absorberíamos": "absorber",
    "absorberían": "absorber",
    "absorberías": "absorber",
    "absorbes": "absorber",
    "absorbibilidades": "absorbibilidad",
    "absorbibles": "absorbible",
    "absorbida": "absorber",
    "absorbidas": "absorber",
    "absorbido": "absorber",
    "absorbidos": "absorber",
    "absorbiendo": "absorber",
    "absorbiera": "absorber",
    "absorbierais": "absorber",
    "absorbieran": "absorber",
    "absorbieras": "absorber",
    "absorbiere": "absorber",
    "absorbiereis": "absorber",
    "absorbieren": "absorber",
    "absorbieres": "absorber",
    "absorbieron": "absorber",
    "absorbiese": "absorber",
    "absorbieseis": "absorber",
    "absorbiesen": "absorber",
    "absorbieses": "absorber",
    "absorbimientos": "absorbimiento",
    "absorbimos": "absorber",
    "absorbiste": "absorber",
    "absorbisteis": "absorber",
    "absorbiéramos": "absorber",
    "absorbiéremos": "absorber",
    "absorbiésemos": "absorber",
    "absorbió": "absorber",
    "absorbo": "absorber",
    "absorbáis": "absorber",
    "absorbéis": "absorber",
    "absorbí": "absorber",
    "absorbía": "absorber",
    "absorbíais": "absorber",
    "absorbíamos": "absorber",
    "absorbían": "absorber",
    "absorbías": "absorber",
    "absorciometrías": "absorciometría",
    "absorciones": "absorción",
    "absorciómetros": "absorciómetro",
    "absorptancias": "absorptancia",
    "absorta": "absorto",
    "absortaba": "absortar",
    "absortabais": "absortar",
    "absortaban": "absortar",
    "absortabas": "absortar",
    "absortad": "absortar",
    "absortada": "absortar",
    "absortadas": "absortar",
    "absortado": "absortar",
    "absortados": "absortar",
    "absortamos": "absortar",
    "absortan": "absortar",
    "absortancias": "absortancia",
    "absortando": "absortar",
    "absortara": "absortar",
    "absortarais": "absortar",
    "absortaran": "absortar",
    "absortaras": "absortar",
    "absortare": "absortar",
    "absortareis": "absortar",
    "absortaremos": "absortar",
    "absortaren": "absortar",
    "absortares": "absortar",
    "absortaron": "absortar",
    "absortará": "absortar",
    "absortarán": "absortar",
    "absortarás": "absortar",
    "absortaré": "absortar",
    "absortaréis": "absortar",
    "absortaría": "absortar",
    "absortaríais": "absortar",
    "absortaríamos": "absortar",
    "absortarían": "absortar",
    "absortarías": "absortar",
    "absortas": "absorto",
    "absortase": "absortar",
    "absortaseis": "absortar",
    "absortasen": "absortar",
    "absortases": "absortar",
    "absortaste": "absortar",
    "absortasteis": "absortar",
    "absorte": "absortar",
    "absortemos": "absortar",
    "absorten": "absortar",
    "absortes": "absortar",
    "absorto": "absortar",
    "absortos": "absorto",
    "absortábamos": "absortar",
    "absortáis": "absortar",
    "absortáramos": "absortar",
    "absortáremos": "absortar",
    "absortásemos": "absortar",
    "absorté": "absortar",
    "absortéis": "absortar",
    "absortó": "absortar",
    "abstemia": "abstemio",
    "abstemias": "abstemio",
    "abstemios": "abstemio",
    "abstenciones": "abstención",
    "abstencionismos": "abstencionismo",
    "abstencionistas": "abstencionista",
    "abstendremos": "abstener",
    "abstendrá": "abstener",
    "abstendrán": "abstener",
    "abstendrás": "abstener",
    "abstendré": "abstener",
    "abstendréis": "abstener",
    "abstendría": "abstener",
    "abstendríais": "abstener",
    "abstendríamos": "abstener",
    "abstendrían": "abstener",
    "abstendrías": "abstener",
    "abstened": "abstener",
    "abstenemos": "abstener",
    "abstenga": "abstener",
    "abstengamos": "abstener",
    "abstengan": "abstener",
    "abstengas": "abstener",
    "abstengo": "abstener",
    "abstengáis": "abstener",
    "abstenida": "abstener",
    "abstenidas": "abstener",
    "abstenido": "abstener",
    "abstenidos": "abstener",
    "absteniendo": "abstener",
    "abstenéis": "abstener",
    "abstenía": "abstener",
    "absteníais": "abstener",
    "absteníamos": "abstener",
    "abstenían": "abstener",
    "abstenías": "abstener",
    "absterge": "absterger",
    "absterged": "absterger",
    "abstergemos": "absterger",
    "abstergen": "absterger",
    "abstergentes": "abstergente",
    "abstergeremos": "absterger",
    "abstergerá": "absterger",
    "abstergerán": "absterger",
    "abstergerás": "absterger",
    "abstergeré": "absterger",
    "abstergeréis": "absterger",
    "abstergería": "absterger",
    "abstergeríais": "absterger",
    "abstergeríamos": "absterger",
    "abstergerían": "absterger",
    "abstergerías": "absterger",
    "absterges": "absterger",
    "abstergida": "absterger",
    "abstergidas": "absterger",
    "abstergido": "absterger",
    "abstergidos": "absterger",
    "abstergiendo": "absterger",
    "abstergiera": "absterger",
    "abstergierais": "absterger",
    "abstergieran": "absterger",
    "abstergieras": "absterger",
    "abstergiere": "absterger",
    "abstergiereis": "absterger",
    "abstergieren": "absterger",
    "abstergieres": "absterger",
    "abstergieron": "absterger",
    "abstergiese": "absterger",
    "abstergieseis": "absterger",
    "abstergiesen": "absterger",
    "abstergieses": "absterger",
    "abstergimos": "absterger",
    "abstergiste": "absterger",
    "abstergisteis": "absterger",
    "abstergiéramos": "absterger",
    "abstergiéremos": "absterger",
    "abstergiésemos": "absterger",
    "abstergió": "absterger",
    "abstergéis": "absterger",
    "abstergí": "absterger",
    "abstergía": "absterger",
    "abstergíais": "absterger",
    "abstergíamos": "absterger",
    "abstergían": "absterger",
    "abstergías": "absterger",
    "absterja": "absterger",
    "absterjamos": "absterger",
    "absterjan": "absterger",
    "absterjas": "absterger",
    "absterjo": "absterger",
    "absterjáis": "absterger",
    "abstersiones": "abstersión",
    "abstersiva": "abstersivo",
    "abstersivas": "abstersivo",
    "abstersivos": "abstersivo",
    "abstiene": "abstener",
    "abstienen": "abstener",
    "abstienes": "abstener",
    "abstinencias": "abstinencia",
    "abstinentes": "abstinente",
    "abstracciones": "abstracción",
    "abstraccionismos": "abstraccionismo",
    "abstracta": "abstracto",
    "abstractas": "abstracto",
    "abstractos": "abstracto",
    "abstrae": "abstraer",
    "abstraed": "abstraer",
    "abstraemos": "abstraer",
    "abstraen": "abstraer",
    "abstraeremos": "abstraer",
    "abstraerá": "abstraer",
    "abstraerán": "abstraer",
    "abstraerás": "abstraer",
    "abstraeré": "abstraer",
    "abstraeréis": "abstraer",
    "abstraería": "abstraer",
    "abstraeríais": "abstraer",
    "abstraeríamos": "abstraer",
    "abstraerían": "abstraer",
    "abstraerías": "abstraer",
    "abstraes": "abstraer",
    "abstraiga": "abstraer",
    "abstraigamos": "abstraer",
    "abstraigan": "abstraer",
    "abstraigas": "abstraer",
    "abstraigo": "abstraer",
    "abstraigáis": "abstraer",
    "abstraje": "abstraer",
    "abstrajera": "abstraer",
    "abstrajerais": "abstraer",
    "abstrajeran": "abstraer",
    "abstrajeras": "abstraer",
    "abstrajere": "abstraer",
    "abstrajereis": "abstraer",
    "abstrajeren": "abstraer",
    "abstrajeres": "abstraer",
    "abstrajeron": "abstraer",
    "abstrajese": "abstraer",
    "abstrajeseis": "abstraer",
    "abstrajesen": "abstraer",
    "abstrajeses": "abstraer",
    "abstrajimos": "abstraer",
    "abstrajiste": "abstraer",
    "abstrajisteis": "abstraer",
    "abstrajo": "abstraer",
    "abstrajéramos": "abstraer",
    "abstrajéremos": "abstraer",
    "abstrajésemos": "abstraer",
    "abstrayendo": "abstraer",
    "abstraéis": "abstraer",
    "abstraía": "abstraer",
    "abstraíais": "abstraer",
    "abstraíamos": "abstraer",
    "abstraían": "abstraer",
    "abstraías": "abstraer",
    "abstraída": "abstraer",
    "abstraídas": "abstraer",
    "abstraído": "abstraer",
    "abstraídos": "abstraer",
    "abstrusa": "abstruso",
    "abstrusas": "abstruso",
    "abstrusos": "abstruso",
    "abstuve": "abstener",
    "abstuviera": "abstener",
    "abstuvierais": "abstener",
    "abstuvieran": "abstener",
    "abstuvieras": "abstener",
    "abstuviere": "abstener",
    "abstuviereis": "abstener",
    "abstuvieren": "abstener",
    "abstuvieres": "abstener",
    "abstuvieron": "abstener",
    "abstuviese": "abstener",
    "abstuvieseis": "abstener",
    "abstuviesen": "abstener",
    "abstuvieses": "abstener",
    "abstuvimos": "abstener",
    "abstuviste": "abstener",
    "abstuvisteis": "abstener",
    "abstuviéramos": "abstener",
    "abstuviéremos": "abstener",
    "abstuviésemos": "abstener",
    "abstuvo": "abstener",
    "abstén": "abstener",
    "absuelta": "absolver",
    "absueltas": "absolver",
    "absuelto": "absolver",
    "absueltos": "absolver",
    "absuelva": "absolver",
    "absuelvan": "absolver",
    "absuelvas": "absolver",
    "absuelve": "absolver",
    "absuelven": "absolver",
    "absuelves": "absolver",
    "absuelvo": "absolver",
    "absurda": "absurdo",
    "absurdas": "absurdo",
    "absurdidades": "absurdidad",
    "absurdos": "absurdo",
    "abubillas": "abubilla",
    "abuchea": "abuchear",
    "abucheaba": "abuchear",
    "abucheabais": "abuchear",
    "abucheaban": "abuchear",
    "abucheabas": "abuchear",
    "abuchead": "abuchear",
    "abucheada": "abuchear",
    "abucheadas": "abuchear",
    "abucheado": "abuchear",
    "abucheados": "abuchear",
    "abucheamos": "abuchear",
    "abuchean": "abuchear",
    "abucheando": "abuchear",
    "abucheara": "abuchear",
    "abuchearais": "abuchear",
    "abuchearan": "abuchear",
    "abuchearas": "abuchear",
    "abucheare": "abuchear",
    "abucheareis": "abuchear",
    "abuchearemos": "abuchear",
    "abuchearen": "abuchear",
    "abucheares": "abuchear",
    "abuchearon": "abuchear",
    "abucheará": "abuchear",
    "abuchearán": "abuchear",
    "abuchearás": "abuchear",
    "abuchearé": "abuchear",
    "abuchearéis": "abuchear",
    "abuchearía": "abuchear",
    "abuchearíais": "abuchear",
    "abuchearíamos": "abuchear",
    "abuchearían": "abuchear",
    "abuchearías": "abuchear",
    "abucheas": "abuchear",
    "abuchease": "abuchear",
    "abucheaseis": "abuchear",
    "abucheasen": "abuchear",
    "abucheases": "abuchear",
    "abucheaste": "abuchear",
    "abucheasteis": "abuchear",
    "abuchee": "abuchear",
    "abucheemos": "abuchear",
    "abucheen": "abuchear",
    "abuchees": "abuchear",
    "abucheo": "abuchear",
    "abucheos": "abucheo",
    "abucheábamos": "abuchear",
    "abucheáis": "abuchear",
    "abucheáramos": "abuchear",
    "abucheáremos": "abuchear",
    "abucheásemos": "abuchear",
    "abucheé": "abuchear",
    "abucheéis": "abuchear",
    "abucheó": "abuchear",
    "abuela": "abuelo",
    "abuelas": "abuelo",
    "abuelastra": "abuelastro",
    "abuelastras": "abuelastro",
    "abuelastros": "abuelastro",
    "abuelos": "abuelo",
    "abulenses": "abulense",
    "abulias": "abulia",
    "abullona": "abullonar",
    "abullonaba": "abullonar",
    "abullonabais": "abullonar",
    "abullonaban": "abullonar",
    "abullonabas": "abullonar",
    "abullonad": "abullonar",
    "abullonada": "abullonar",
    "abullonadas": "abullonar",
    "abullonado": "abullonar",
    "abullonados": "abullonar",
    "abullonamos": "abullonar",
    "abullonan": "abullonar",
    "abullonando": "abullonar",
    "abullonara": "abullonar",
    "abullonarais": "abullonar",
    "abullonaran": "abullonar",
    "abullonaras": "abullonar",
    "abullonare": "abullonar",
    "abullonareis": "abullonar",
    "abullonaremos": "abullonar",
    "abullonaren": "abullonar",
    "abullonares": "abullonar",
    "abullonaron": "abullonar",
    "abullonará": "abullonar",
    "abullonarán": "abullonar",
    "abullonarás": "abullonar",
    "abullonaré": "abullonar",
    "abullonaréis": "abullonar",
    "abullonaría": "abullonar",
    "abullonaríais": "abullonar",
    "abullonaríamos": "abullonar",
    "abullonarían": "abullonar",
    "abullonarías": "abullonar",
    "abullonas": "abullonar",
    "abullonase": "abullonar",
    "abullonaseis": "abullonar",
    "abullonasen": "abullonar",
    "abullonases": "abullonar",
    "abullonaste": "abullonar",
    "abullonasteis": "abullonar",
    "abullone": "abullonar",
    "abullonemos": "abullonar",
    "abullonen": "abullonar",
    "abullones": "abullonar",
    "abullono": "abullonar",
    "abullonábamos": "abullonar",
    "abullonáis": "abullonar",
    "abullonáramos": "abullonar",
    "abullonáremos": "abullonar",
    "abullonásemos": "abullonar",
    "abulloné": "abullonar",
    "abullonéis": "abullonar",
    "abullonó": "abullonar",
    "abulta": "abultar",
    "abultaba": "abultar",
    "abultabais": "abultar",
    "abultaban": "abultar",
    "abultabas": "abultar",
    "abultad": "abultar",
    "abultada": "abultar",
    "abultadas": "abultar",
    "abultado": "abultar",
    "abultados": "abultar",
    "abultamientos": "abultamiento",
    "abultamos": "abultar",
    "abultan": "abultar",
    "abultando": "abultar",
    "abultara": "abultar",
    "abultarais": "abultar",
    "abultaran": "abultar",
    "abultaras": "abultar",
    "abultare": "abultar",
    "abultareis": "abultar",
    "abultaremos": "abultar",
    "abultaren": "abultar",
    "abultares": "abultar",
    "abultaron": "abultar",
    "abultará": "abultar",
    "abultarán": "abultar",
    "abultarás": "abultar",
    "abultaré": "abultar",
    "abultaréis": "abultar",
    "abultaría": "abultar",
    "abultaríais": "abultar",
    "abultaríamos": "abultar",
    "abultarían": "abultar",
    "abultarías": "abultar",
    "abultas": "abultar",
    "abultase": "abultar",
    "abultaseis": "abultar",
    "abultasen": "abultar",
    "abultases": "abultar",
    "abultaste": "abultar",
    "abultasteis": "abultar",
    "abulte": "abultar",
    "abultemos": "abultar",
    "abulten": "abultar",
    "abultes": "abultar",
    "abulto": "abultar",
    "abultábamos": "abultar",
    "abultáis": "abultar",
    "abultáramos": "abultar",
    "abultáremos": "abultar",
    "abultásemos": "abultar",
    "abulté": "abultar",
    "abultéis": "abultar",
    "abultó": "abultar",
    "abunda": "abundar",
    "abundaba": "abundar",
    "abundabais": "abundar",
    "abundaban": "abundar",
    "abundabas": "abundar",
    "abundad": "abundar",
    "abundada": "abundar",
    "abundadas": "abundar",
    "abundado": "abundar",
    "abundados": "abundar",
    "abundamientos": "abundamiento",
    "abundamos": "abundar",
    "abundan": "abundar",
    "abundancias": "abundancia",
    "abundando": "abundar",
    "abundantes": "abundante",
    "abundara": "abundar",
    "abundarais": "abundar",
    "abundaran": "abundar",
    "abundaras": "abundar",
    "abundare": "abundar",
    "abundareis": "abundar",
    "abundaremos": "abundar",
    "abundaren": "abundar",
    "abundares": "abundar",
    "abundaron": "abundar",
    "abundará": "abundar",
    "abundarán": "abundar",
    "abundarás": "abundar",
    "abundaré": "abundar",
    "abundaréis": "abundar",
    "abundaría": "abundar",
    "abundaríais": "abundar",
    "abundaríamos": "abundar",
    "abundarían": "abundar",
    "abundarías": "abundar",
    "abundas": "abundar",
    "abundase": "abundar",
    "abundaseis": "abundar",
    "abundasen": "abundar",
    "abundases": "abundar",
    "abundaste": "abundar",
    "abundasteis": "abundar",
    "abunde": "abundar",
    "abundemos": "abundar",
    "abunden": "abundar",
    "abundes": "abundar",
    "abundo": "abundar",
    "abundosa": "abundoso",
    "abundosas": "abundoso",
    "abundosos": "abundoso",
    "abundábamos": "abundar",
    "abundáis": "abundar",
    "abundáramos": "abundar",
    "abundáremos": "abundar",
    "abundásemos": "abundar",
    "abundé": "abundar",
    "abundéis": "abundar",
    "abundó": "abundar",
    "abura": "aburar",
    "aburaba": "aburar",
    "aburabais": "aburar",
    "aburaban": "aburar",
    "aburabas": "aburar",
    "aburad": "aburar",
    "aburada": "aburar",
    "aburadas": "aburar",
    "aburado": "aburar",
    "aburados": "aburar",
    "aburamos": "aburar",
    "aburan": "aburar",
    "aburando": "aburar",
    "aburara": "aburar",
    "aburarais": "aburar",
    "aburaran": "aburar",
    "aburaras": "aburar",
    "aburare": "aburar",
    "aburareis": "aburar",
    "aburaremos": "aburar",
    "aburaren": "aburar",
    "aburares": "aburar",
    "aburaron": "aburar",
    "aburará": "aburar",
    "aburarán": "aburar",
    "aburarás": "aburar",
    "aburaré": "aburar",
    "aburaréis": "aburar",
    "aburaría": "aburar",
    "aburaríais": "aburar",
    "aburaríamos": "aburar",
    "aburarían": "aburar",
    "aburarías": "aburar",
    "aburas": "aburar",
    "aburase": "aburar",
    "aburaseis": "aburar",
    "aburasen": "aburar",
    "aburases": "aburar",
    "aburaste": "aburar",
    "aburasteis": "aburar",
    "abure": "aburar",
    "aburemos": "aburar",
    "aburen": "aburar",
    "abures": "aburar",
    "aburguesa": "aburguesar",
    "aburguesaba": "aburguesar",
    "aburguesabais": "aburguesar",
    "aburguesaban": "aburguesar",
    "aburguesabas": "aburguesar",
    "aburguesad": "aburguesar",
    "aburguesada": "aburguesar",
    "aburguesadas": "aburguesar",
    "aburguesado": "aburguesar",
    "aburguesados": "aburguesar",
    "aburguesamientos": "aburguesamiento",
    "aburguesamos": "aburguesar",
    "aburguesan": "aburguesar",
    "aburguesando": "aburguesar",
    "aburguesara": "aburguesar",
    "aburguesarais": "aburguesar",
    "aburguesaran": "aburguesar",
    "aburguesaras": "aburguesar",
    "aburguesare": "aburguesar",
    "aburguesareis": "aburguesar",
    "aburguesaremos": "aburguesar",
    "aburguesaren": "aburguesar",
    "aburguesares": "aburguesar",
    "aburguesaron": "aburguesar",
    "aburguesará": "aburguesar",
    "aburguesarán": "aburguesar",
    "aburguesarás": "aburguesar",
    "aburguesaré": "aburguesar",
    "aburguesaréis": "aburguesar",
    "aburguesaría": "aburguesar",
    "aburguesaríais": "aburguesar",
    "aburguesaríamos": "aburguesar",
    "aburguesarían": "aburguesar",
    "aburguesarías": "aburguesar",
    "aburguesas": "aburguesar",
    "aburguesase": "aburguesar",
    "aburguesaseis": "aburguesar",
    "aburguesasen": "aburguesar",
    "aburguesases": "aburguesar",
    "aburguesaste": "aburguesar",
    "aburguesasteis": "aburguesar",
    "aburguese": "aburguesar",
    "aburguesemos": "aburguesar",
    "aburguesen": "aburguesar",
    "aburgueses": "aburguesar",
    "aburgueso": "aburguesar",
    "aburguesábamos": "aburguesar",
    "aburguesáis": "aburguesar",
    "aburguesáramos": "aburguesar",
    "aburguesáremos": "aburguesar",
    "aburguesásemos": "aburguesar",
    "aburguesé": "aburguesar",
    "aburgueséis": "aburguesar",
    "aburguesó": "aburguesar",
    "aburo": "aburar",
    "aburra": "aburrir",
    "aburraba": "aburrar",
    "aburrabais": "aburrar",
    "aburraban": "aburrar",
    "aburrabas": "aburrar",
    "aburrad": "aburrar",
    "aburrada": "aburrar",
    "aburradas": "aburrar",
    "aburrado": "aburrar",
    "aburrados": "aburrar",
    "aburramos": "aburrir",
    "aburran": "aburrir",
    "aburrando": "aburrar",
    "aburrara": "aburrar",
    "aburrarais": "aburrar",
    "aburraran": "aburrar",
    "aburraras": "aburrar",
    "aburrare": "aburrar",
    "aburrareis": "aburrar",
    "aburraremos": "aburrar",
    "aburraren": "aburrar",
    "aburrares": "aburrar",
    "aburraron": "aburrar",
    "aburrará": "aburrar",
    "aburrarán": "aburrar",
    "aburrarás": "aburrar",
    "aburraré": "aburrar",
    "aburraréis": "aburrar",
    "aburraría": "aburrar",
    "aburraríais": "aburrar",
    "aburraríamos": "aburrar",
    "aburrarían": "aburrar",
    "aburrarías": "aburrar",
    "aburras": "aburrir",
    "aburrase": "aburrar",
    "aburraseis": "aburrar",
    "aburrasen": "aburrar",
    "aburrases": "aburrar",
    "aburraste": "aburrar",
    "aburrasteis": "aburrar",
    "aburre": "aburrir",
    "aburremos": "aburrar",
    "aburren": "aburrir",
    "aburres": "aburrir",
    "aburrid": "aburrir",
    "aburrida": "aburrir",
    "aburridas": "aburrir",
    "aburrido": "aburrir",
    "aburridos": "aburrir",
    "aburriendo": "aburrir",
    "aburriera": "aburrir",
    "aburrierais": "aburrir",
    "aburrieran": "aburrir",
    "aburrieras": "aburrir",
    "aburriere": "aburrir",
    "aburriereis": "aburrir",
    "aburrieren": "aburrir",
    "aburrieres": "aburrir",
    "aburrieron": "aburrir",
    "aburriese": "aburrir",
    "aburrieseis": "aburrir",
    "aburriesen": "aburrir",
    "aburrieses": "aburrir",
    "aburrimientos": "aburrimiento",
    "aburrimos": "aburrir",
    "aburriremos": "aburrir",
    "aburrirá": "aburrir",
    "aburrirán": "aburrir",
    "aburrirás": "aburrir",
    "aburriré": "aburrir",
    "aburriréis": "aburrir",
    "aburriría": "aburrir",
    "aburriríais": "aburrir",
    "aburriríamos": "aburrir",
    "aburrirían": "aburrir",
    "aburrirías": "aburrir",
    "aburriste": "aburrir",
    "aburristeis": "aburrir",
    "aburriéramos": "aburrir",
    "aburriéremos": "aburrir",
    "aburriésemos": "aburrir",
    "aburrió": "aburrir",
    "aburro": "aburrir",
    "aburrábamos": "aburrar",
    "aburráis": "aburrir",
    "aburráramos": "aburrar",
    "aburráremos": "aburrar",
    "aburrásemos": "aburrar",
    "aburré": "aburrar",
    "aburréis": "aburrar",
    "aburrí": "aburrir",
    "aburría": "aburrir",
    "aburríais": "aburrir",
    "aburríamos": "aburrir",
    "aburrían": "aburrir",
    "aburrías": "aburrir",
    "aburrís": "aburrir",
    "aburró": "aburrar",
    "aburuja": "aburujar",
    "aburujaba": "aburujar",
    "aburujabais": "aburujar",
    "aburujaban": "aburujar",
    "aburujabas": "aburujar",
    "aburujad": "aburujar",
    "aburujada": "aburujar",
    "aburujadas": "aburujar",
    "aburujado": "aburujar",
    "aburujados": "aburujar",
    "aburujamos": "aburujar",
    "aburujan": "aburujar",
    "aburujando": "aburujar",
    "aburujara": "aburujar",
    "aburujarais": "aburujar",
    "aburujaran": "aburujar",
    "aburujaras": "aburujar",
    "aburujare": "aburujar",
    "aburujareis": "aburujar",
    "aburujaremos": "aburujar",
    "aburujaren": "aburujar",
    "aburujares": "aburujar",
    "aburujaron": "aburujar",
    "aburujará": "aburujar",
    "aburujarán": "aburujar",
    "aburujarás": "aburujar",
    "aburujaré": "aburujar",
    "aburujaréis": "aburujar",
    "aburujaría": "aburujar",
    "aburujaríais": "aburujar",
    "aburujaríamos": "aburujar",
    "aburujarían": "aburujar",
    "aburujarías": "aburujar",
    "aburujas": "aburujar",
    "aburujase": "aburujar",
    "aburujaseis": "aburujar",
    "aburujasen": "aburujar",
    "aburujases": "aburujar",
    "aburujaste": "aburujar",
    "aburujasteis": "aburujar",
    "aburuje": "aburujar",
    "aburujemos": "aburujar",
    "aburujen": "aburujar",
    "aburujes": "aburujar",
    "aburujo": "aburujar",
    "aburujábamos": "aburujar",
    "aburujáis": "aburujar",
    "aburujáramos": "aburujar",
    "aburujáremos": "aburujar",
    "aburujásemos": "aburujar",
    "aburujé": "aburujar",
    "aburujéis": "aburujar",
    "aburujó": "aburujar",
    "aburábamos": "aburar",
    "aburáis": "aburar",
    "aburáramos": "aburar",
    "aburáremos": "aburar",
    "aburásemos": "aburar",
    "aburé": "aburar",
    "aburéis": "aburar",
    "aburó": "aburar",
    "abusa": "abusar",
    "abusaba": "abusar",
    "abusabais": "abusar",
    "abusaban": "abusar",
    "abusabas": "abusar",
    "abusad": "abusar",
    "abusada": "abusar",
    "abusadas": "abusar",
    "abusado": "abusar",
    "abusados": "abusar",
    "abusamos": "abusar",
    "abusan": "abusar",
    "abusando": "abusar",
    "abusara": "abusar",
    "abusarais": "abusar",
    "abusaran": "abusar",
    "abusaras": "abusar",
    "abusare": "abusar",
    "abusareis": "abusar",
    "abusaremos": "abusar",
    "abusaren": "abusar",
    "abusares": "abusar",
    "abusaron": "abusar",
    "abusará": "abusar",
    "abusarán": "abusar",
    "abusarás": "abusar",
    "abusaré": "abusar",
    "abusaréis": "abusar",
    "abusaría": "abusar",
    "abusaríais": "abusar",
    "abusaríamos": "abusar",
    "abusarían": "abusar",
    "abusarías": "abusar",
    "abusas": "abusar",
    "abusase": "abusar",
    "abusaseis": "abusar",
    "abusasen": "abusar",
    "abusases": "abusar",
    "abusaste": "abusar",
    "abusasteis": "abusar",
    "abuse": "abusar",
    "abusemos": "abusar",
    "abusen": "abusar",
    "abuses": "abusar",
    "abusiva": "abusivo",
    "abusivas": "abusivo",
    "abusivos": "abusivo",
    "abuso": "abusar",
    "abusona": "abusón",
    "abusonas": "abusón",
    "abusones": "abusón",
    "abusos": "abuso",
    "abusábamos": "abusar",
    "abusáis": "abusar",
    "abusáramos": "abusar",
    "abusáremos": "abusar",
    "abusásemos": "abusar",
    "abusé": "abusar",
    "abuséis": "abusar",
    "abusó": "abusar",
    "abuñolaba": "abuñolar",
    "abuñolabais": "abuñolar",
    "abuñolaban": "abuñolar",
    "abuñolabas": "abuñolar",
    "abuñolad": "abuñolar",
    "abuñolada": "abuñolar",
    "abuñoladas": "abuñolar",
    "abuñolado": "abuñolar",
    "abuñolados": "abuñolar",
    "abuñolamos": "abuñolar",
    "abuñolando": "abuñolar",
    "abuñolara": "abuñolar",
    "abuñolarais": "abuñolar",
    "abuñolaran": "abuñolar",
    "abuñolaras": "abuñolar",
    "abuñolare": "abuñolar",
    "abuñolareis": "abuñolar",
    "abuñolaremos": "abuñolar",
    "abuñolaren": "abuñolar",
    "abuñolares": "abuñolar",
    "abuñolaron": "abuñolar",
    "abuñolará": "abuñolar",
    "abuñolarán": "abuñolar",
    "abuñolarás": "abuñolar",
    "abuñolaré": "abuñolar",
    "abuñolaréis": "abuñolar",
    "abuñolaría": "abuñolar",
    "abuñolaríais": "abuñolar",
    "abuñolaríamos": "abuñolar",
    "abuñolarían": "abuñolar",
    "abuñolarías": "abuñolar",
    "abuñolase": "abuñolar",
    "abuñolaseis": "abuñolar",
    "abuñolasen": "abuñolar",
    "abuñolases": "abuñolar",
    "abuñolaste": "abuñolar",
    "abuñolasteis": "abuñolar",
    "abuñolemos": "abuñolar",
    "abuñolábamos": "abuñolar",
    "abuñoláis": "abuñolar",
    "abuñoláramos": "abuñolar",
    "abuñoláremos": "abuñolar",
    "abuñolásemos": "abuñolar",
    "abuñolé": "abuñolar",
    "abuñoléis": "abuñolar",
    "abuñoló": "abuñolar",
    "abuñuela": "abuñuelar",
    "abuñuelaba": "abuñuelar",
    "abuñuelabais": "abuñuelar",
    "abuñuelaban": "abuñuelar",
    "abuñuelabas": "abuñuelar",
    "abuñuelad": "abuñuelar",
    "abuñuelada": "abuñuelar",
    "abuñueladas": "abuñuelar",
    "abuñuelado": "abuñuelar",
    "abuñuelados": "abuñuelar",
    "abuñuelamos": "abuñuelar",
    "abuñuelan": "abuñuelar",
    "abuñuelando": "abuñuelar",
    "abuñuelara": "abuñuelar",
    "abuñuelarais": "abuñuelar",
    "abuñuelaran": "abuñuelar",
    "abuñuelaras": "abuñuelar",
    "abuñuelare": "abuñuelar",
    "abuñuelareis": "abuñuelar",
    "abuñuelaremos": "abuñuelar",
    "abuñuelaren": "abuñuelar",
    "abuñuelares": "abuñuelar",
    "abuñuelaron": "abuñuelar",
    "abuñuelará": "abuñuelar",
    "abuñuelarán": "abuñuelar",
    "abuñuelarás": "abuñuelar",
    "abuñuelaré": "abuñuelar",
    "abuñuelaréis": "abuñuelar",
    "abuñuelaría": "abuñuelar",
    "abuñuelaríais": "abuñuelar",
    "abuñuelaríamos": "abuñuelar",
    "abuñuelarían": "abuñuelar",
    "abuñuelarías": "abuñuelar",
    "abuñuelas": "abuñuelar",
    "abuñuelase": "abuñuelar",
    "abuñuelaseis": "abuñuelar",
    "abuñuelasen": "abuñuelar",
    "abuñuelases": "abuñuelar",
    "abuñuelaste": "abuñuelar",
    "abuñuelasteis": "abuñuelar",
    "abuñuele": "abuñuelar",
    "abuñuelemos": "abuñuelar",
    "abuñuelen": "abuñuelar",
    "abuñueles": "abuñuelar",
    "abuñuelo": "abuñuelar",
    "abuñuelábamos": "abuñuelar",
    "abuñueláis": "abuñuelar",
    "abuñueláramos": "abuñuelar",
    "abuñueláremos": "abuñuelar",
    "abuñuelásemos": "abuñuelar",
    "abuñuelé": "abuñuelar",
    "abuñueléis": "abuñuelar",
    "abuñueló": "abuñuelar",
    "abyecciones": "abyección",
    "abyecta": "abyecto",
    "abyectas": "abyecto",
    "abyectos": "abyecto",
    "abzimas": "abzima",
    "abábamos": "abar",
    "abáis": "abar",
    "abáramos": "abar",
    "abáremos": "abar",
    "abásemos": "abar",
    "abásidas": "abásida",
    "abé": "abar",
    "abéis": "abar",
    "abó": "abar",
    "abúlica": "abúlico",
    "abúlicas": "abúlico",
    "abúlicos": "abúlico",
    "acaba": "acabar",
    "acababa": "acabar",
    "acababais": "acabar",
    "acababan": "acabar",
    "acababas": "acabar",
    "acabables": "acabable",
    "acabad": "acabar",
    "acabada": "acabar",
    "acabadas": "acabar",
    "acabado": "acabar",
    "acabadora": "acabador",
    "acabadoras": "acabador",
    "acabadores": "acabador",
    "acabados": "acabar",
    "acaballa": "acaballar",
    "acaballaba": "acaballar",
    "acaballabais": "acaballar",
    "acaballaban": "acaballar",
    "acaballabas": "acaballar",
    "acaballad": "acaballar",
    "acaballada": "acaballar",
    "acaballadas": "acaballar",
    "acaballaderos": "acaballadero",
    "acaballado": "acaballar",
    "acaballados": "acaballar",
    "acaballamos": "acaballar",
    "acaballan": "acaballar",
    "acaballando": "acaballar",
    "acaballara": "acaballar",
    "acaballarais": "acaballar",
    "acaballaran": "acaballar",
    "acaballaras": "acaballar",
    "acaballare": "acaballar",
    "acaballareis": "acaballar",
    "acaballaremos": "acaballar",
    "acaballaren": "acaballar",
    "acaballares": "acaballar",
    "acaballaron": "acaballar",
    "acaballará": "acaballar",
    "acaballarán": "acaballar",
    "acaballarás": "acaballar",
    "acaballaré": "acaballar",
    "acaballaréis": "acaballar",
    "acaballaría": "acaballar",
    "acaballaríais": "acaballar",
    "acaballaríamos": "acaballar",
    "acaballarían": "acaballar",
    "acaballarías": "acaballar",
    "acaballas": "acaballar",
    "acaballase": "acaballar",
    "acaballaseis": "acaballar",
    "acaballasen": "acaballar",
    "acaballases": "acaballar",
    "acaballaste": "acaballar",
    "acaballasteis": "acaballar",
    "acaballe": "acaballar",
    "acaballemos": "acaballar",
    "acaballen": "acaballar",
    "acaballes": "acaballar",
    "acaballo": "acaballar",
    "acaballona": "acaballonar",
    "acaballonaba": "acaballonar",
    "acaballonabais": "acaballonar",
    "acaballonaban": "acaballonar",
    "acaballonabas": "acaballonar",
    "acaballonad": "acaballonar",
    "acaballonada": "acaballonar",
    "acaballonadas": "acaballonar",
    "acaballonado": "acaballonar",
    "acaballonados": "acaballonar",
    "acaballonamos": "acaballonar",
    "acaballonan": "acaballonar",
    "acaballonando": "acaballonar",
    "acaballonara": "acaballonar",
    "acaballonarais": "acaballonar",
    "acaballonaran": "acaballonar",
    "acaballonaras": "acaballonar",
    "acaballonare": "acaballonar",
    "acaballonareis": "acaballonar",
    "acaballonaremos": "acaballonar",
    "acaballonaren": "acaballonar",
    "acaballonares": "acaballonar",
    "acaballonaron": "acaballonar",
    "acaballonará": "acaballonar",
    "acaballonarán": "acaballonar",
    "acaballonarás": "acaballonar",
    "acaballonaré": "acaballonar",
    "acaballonaréis": "acaballonar",
    "acaballonaría": "acaballonar",
    "acaballonaríais": "acaballonar",
    "acaballonaríamos": "acaballonar",
    "acaballonarían": "acaballonar",
    "acaballonarías": "acaballonar",
    "acaballonas": "acaballonar",
    "acaballonase": "acaballonar",
    "acaballonaseis": "acaballonar",
    "acaballonasen": "acaballonar",
    "acaballonases": "acaballonar",
    "acaballonaste": "acaballonar",
    "acaballonasteis": "acaballonar",
    "acaballone": "acaballonar",
    "acaballonemos": "acaballonar",
    "acaballonen": "acaballonar",
    "acaballones": "acaballonar",
    "acaballono": "acaballonar",
    "acaballonábamos": "acaballonar",
    "acaballonáis": "acaballonar",
    "acaballonáramos": "acaballonar",
    "acaballonáremos": "acaballonar",
    "acaballonásemos": "acaballonar",
    "acaballoné": "acaballonar",
    "acaballonéis": "acaballonar",
    "acaballonó": "acaballonar",
    "acaballábamos": "acaballar",
    "acaballáis": "acaballar",
    "acaballáramos": "acaballar",
    "acaballáremos": "acaballar",
    "acaballásemos": "acaballar",
    "acaballé": "acaballar",
    "acaballéis": "acaballar",
    "acaballó": "acaballar",
    "acabamientos": "acabamiento",
    "acabamos": "acabar",
    "acaban": "acabar",
    "acabando": "acabar",
    "acabara": "acabar",
    "acabarais": "acabar",
    "acabaran": "acabar",
    "acabaras": "acabar",
    "acabare": "acabar",
    "acabareis": "acabar",
    "acabaremos": "acabar",
    "acabaren": "acabar",
    "acabares": "acabar",
    "acabaron": "acabar",
    "acabará": "acabar",
    "acabarán": "acabar",
    "acabarás": "acabar",
    "acabaré": "acabar",
    "acabaréis": "acabar",
    "acabaría": "acabar",
    "acabaríais": "acabar",
    "acabaríamos": "acabar",
    "acabarían": "acabar",
    "acabarías": "acabar",
    "acabas": "acabar",
    "acabase": "acabar",
    "acabaseis": "acabar",
    "acabasen": "acabar",
    "acabases": "acabar",
    "acabaste": "acabar",
    "acabasteis": "acabar",
    "acabe": "acabar",
    "acabemos": "acabar",
    "acaben": "acabar",
    "acabes": "acabar",
    "acabijos": "acabijo",
    "acabilda": "acabildar",
    "acabildaba": "acabildar",
    "acabildabais": "acabildar",
    "acabildaban": "acabildar",
    "acabildabas": "acabildar",
    "acabildad": "acabildar",
    "acabildada": "acabildar",
    "acabildadas": "acabildar",
    "acabildado": "acabildar",
    "acabildados": "acabildar",
    "acabildamos": "acabildar",
    "acabildan": "acabildar",
    "acabildando": "acabildar",
    "acabildara": "acabildar",
    "acabildarais": "acabildar",
    "acabildaran": "acabildar",
    "acabildaras": "acabildar",
    "acabildare": "acabildar",
    "acabildareis": "acabildar",
    "acabildaremos": "acabildar",
    "acabildaren": "acabildar",
    "acabildares": "acabildar",
    "acabildaron": "acabildar",
    "acabildará": "acabildar",
    "acabildarán": "acabildar",
    "acabildarás": "acabildar",
    "acabildaré": "acabildar",
    "acabildaréis": "acabildar",
    "acabildaría": "acabildar",
    "acabildaríais": "acabildar",
    "acabildaríamos": "acabildar",
    "acabildarían": "acabildar",
    "acabildarías": "acabildar",
    "acabildas": "acabildar",
    "acabildase": "acabildar",
    "acabildaseis": "acabildar",
    "acabildasen": "acabildar",
    "acabildases": "acabildar",
    "acabildaste": "acabildar",
    "acabildasteis": "acabildar",
    "acabilde": "acabildar",
    "acabildemos": "acabildar",
    "acabilden": "acabildar",
    "acabildes": "acabildar",
    "acabildo": "acabildar",
    "acabildábamos": "acabildar",
    "acabildáis": "acabildar",
    "acabildáramos": "acabildar",
    "acabildáremos": "acabildar",
    "acabildásemos": "acabildar",
    "acabildé": "acabildar",
    "acabildéis": "acabildar",
    "acabildó": "acabildar",
    "acabo": "acabar",
    "acabábamos": "acabar",
    "acabáis": "acabar",
    "acabáramos": "acabar",
    "acabáremos": "acabar",
    "acabásemos": "acabar",
    "acabé": "acabar",
    "acabéis": "acabar",
    "acabó": "acabar",
    "acabóses": "acabóse",
    "acacetinas": "acacetina",
    "acacheta": "acachetar",
    "acachetaba": "acachetar",
    "acachetabais": "acachetar",
    "acachetaban": "acachetar",
    "acachetabas": "acachetar",
    "acachetad": "acachetar",
    "acachetada": "acachetar",
    "acachetadas": "acachetar",
    "acachetado": "acachetar",
    "acachetados": "acachetar",
    "acachetamos": "acachetar",
    "acachetan": "acachetar",
    "acachetando": "acachetar",
    "acachetara": "acachetar",
    "acachetarais": "acachetar",
    "acachetaran": "acachetar",
    "acachetaras": "acachetar",
    "acachetare": "acachetar",
    "acachetareis": "acachetar",
    "acachetaremos": "acachetar",
    "acachetaren": "acachetar",
    "acachetares": "acachetar",
    "acachetaron": "acachetar",
    "acachetará": "acachetar",
    "acachetarán": "acachetar",
    "acachetarás": "acachetar",
    "acachetaré": "acachetar",
    "acachetaréis": "acachetar",
    "acachetaría": "acachetar",
    "acachetaríais": "acachetar",
    "acachetaríamos": "acachetar",
    "acachetarían": "acachetar",
    "acachetarías": "acachetar",
    "acachetas": "acachetar",
    "acachetase": "acachetar",
    "acachetaseis": "acachetar",
    "acachetasen": "acachetar",
    "acachetases": "acachetar",
    "acachetaste": "acachetar",
    "acachetasteis": "acachetar",
    "acachete": "acachetar",
    "acachetea": "acachetear",
    "acacheteaba": "acachetear",
    "acacheteabais": "acachetear",
    "acacheteaban": "acachetear",
    "acacheteabas": "acachetear",
    "acachetead": "acachetear",
    "acacheteada": "acachetear",
    "acacheteadas": "acachetear",
    "acacheteado": "acachetear",
    "acacheteados": "acachetear",
    "acacheteamos": "acachetear",
    "acachetean": "acachetear",
    "acacheteando": "acachetear",
    "acacheteara": "acachetear",
    "acachetearais": "acachetear",
    "acachetearan": "acachetear",
    "acachetearas": "acachetear",
    "acacheteare": "acachetear",
    "acacheteareis": "acachetear",
    "acachetearemos": "acachetear",
    "acachetearen": "acachetear",
    "acacheteares": "acachetear",
    "acachetearon": "acachetear",
    "acacheteará": "acachetear",
    "acachetearán": "acachetear",
    "acachetearás": "acachetear",
    "acachetearé": "acachetear",
    "acachetearéis": "acachetear",
    "acachetearía": "acachetear",
    "acachetearíais": "acachetear",
    "acachetearíamos": "acachetear",
    "acachetearían": "acachetear",
    "acachetearías": "acachetear",
    "acacheteas": "acachetear",
    "acachetease": "acachetear",
    "acacheteaseis": "acachetear",
    "acacheteasen": "acachetear",
    "acacheteases": "acachetear",
    "acacheteaste": "acachetear",
    "acacheteasteis": "acachetear",
    "acachetee": "acachetear",
    "acacheteemos": "acachetear",
    "acacheteen": "acachetear",
    "acachetees": "acachetear",
    "acachetemos": "acachetar",
    "acacheten": "acachetar",
    "acacheteo": "acachetear",
    "acachetes": "acachetar",
    "acacheteábamos": "acachetear",
    "acacheteáis": "acachetear",
    "acacheteáramos": "acachetear",
    "acacheteáremos": "acachetear",
    "acacheteásemos": "acachetear",
    "acacheteé": "acachetear",
    "acacheteéis": "acachetear",
    "acacheteó": "acachetear",
    "acacheto": "acachetar",
    "acachetábamos": "acachetar",
    "acachetáis": "acachetar",
    "acachetáramos": "acachetar",
    "acachetáremos": "acachetar",
    "acachetásemos": "acachetar",
    "acacheté": "acachetar",
    "acachetéis": "acachetar",
    "acachetó": "acachetar",
    "acacias": "acacia",
    "acaciinas": "acaciina",
    "academias": "academia",
    "academice": "academizar",
    "academicemos": "academizar",
    "academicen": "academizar",
    "academices": "academizar",
    "academicismos": "academicismo",
    "academicistas": "academicista",
    "academicé": "academizar",
    "academicéis": "academizar",
    "academismos": "academismo",
    "academistas": "academista",
    "academiza": "academizar",
    "academizaba": "academizar",
    "academizabais": "academizar",
    "academizaban": "academizar",
    "academizabas": "academizar",
    "academizad": "academizar",
    "academizada": "academizar",
    "academizadas": "academizar",
    "academizado": "academizar",
    "academizados": "academizar",
    "academizamos": "academizar",
    "academizan": "academizar",
    "academizando": "academizar",
    "academizara": "academizar",
    "academizarais": "academizar",
    "academizaran": "academizar",
    "academizaras": "academizar",
    "academizare": "academizar",
    "academizareis": "academizar",
    "academizaremos": "academizar",
    "academizaren": "academizar",
    "academizares": "academizar",
    "academizaron": "academizar",
    "academizará": "academizar",
    "academizarán": "academizar",
    "academizarás": "academizar",
    "academizaré": "academizar",
    "academizaréis": "academizar",
    "academizaría": "academizar",
    "academizaríais": "academizar",
    "academizaríamos": "academizar",
    "academizarían": "academizar",
    "academizarías": "academizar",
    "academizas": "academizar",
    "academizase": "academizar",
    "academizaseis": "academizar",
    "academizasen": "academizar",
    "academizases": "academizar",
    "academizaste": "academizar",
    "academizasteis": "academizar",
    "academizo": "academizar",
    "academizábamos": "academizar",
    "academizáis": "academizar",
    "academizáramos": "academizar",
    "academizáremos": "academizar",
    "academizásemos": "academizar",
    "academizó": "academizar",
    "académica": "académico",
    "académicas": "académico",
    "académicos": "académico",
    "acaece": "acaecer",
    "acaeced": "acaecer",
    "acaecedera": "acaecedero",
    "acaecederas": "acaecedero",
    "acaecederos": "acaecedero",
    "acaecemos": "acaecer",
    "acaecen": "acaecer",
    "acaeceremos": "acaecer",
    "acaecerá": "acaecer",
    "acaecerán": "acaecer",
    "acaecerás": "acaecer",
    "acaeceré": "acaecer",
    "acaeceréis": "acaecer",
    "acaecería": "acaecer",
    "acaeceríais": "acaecer",
    "acaeceríamos": "acaecer",
    "acaecerían": "acaecer",
    "acaecerías": "acaecer",
    "acaeces": "acaecer",
    "acaecida": "acaecer",
    "acaecidas": "acaecer",
    "acaecido": "acaecer",
    "acaecidos": "acaecer",
    "acaeciendo": "acaecer",
    "acaeciera": "acaecer",
    "acaecierais": "acaecer",
    "acaecieran": "acaecer",
    "acaecieras": "acaecer",
    "acaeciere": "acaecer",
    "acaeciereis": "acaecer",
    "acaecieren": "acaecer",
    "acaecieres": "acaecer",
    "acaecieron": "acaecer",
    "acaeciese": "acaecer",
    "acaecieseis": "acaecer",
    "acaeciesen": "acaecer",
    "acaecieses": "acaecer",
    "acaecimientos": "acaecimiento",
    "acaecimos": "acaecer",
    "acaeciste": "acaecer",
    "acaecisteis": "acaecer",
    "acaeciéramos": "acaecer",
    "acaeciéremos": "acaecer",
    "acaeciésemos": "acaecer",
    "acaeció": "acaecer",
    "acaecéis": "acaecer",
    "acaecí": "acaecer",
    "acaecía": "acaecer",
    "acaecíais": "acaecer",
    "acaecíamos": "acaecer",
    "acaecían": "acaecer",
    "acaecías": "acaecer",
    "acaezca": "acaecer",
    "acaezcamos": "acaecer",
    "acaezcan": "acaecer",
    "acaezcas": "acaecer",
    "acaezco": "acaecer",
    "acaezcáis": "acaecer",
    "acahuales": "acahual",
    "acahualillos": "acahualillo",
    "acahuates": "acahuate",
    "acahuites": "acahuite",
    "acajúes": "acajú",
    "acajús": "acajú",
    "acalabazada": "acalabazado",
    "acalabazadas": "acalabazado",
    "acalabazados": "acalabazado",
    "acalabrota": "acalabrotar",
    "acalabrotaba": "acalabrotar",
    "acalabrotabais": "acalabrotar",
    "acalabrotaban": "acalabrotar",
    "acalabrotabas": "acalabrotar",
    "acalabrotad": "acalabrotar",
    "acalabrotada": "acalabrotar",
    "acalabrotadas": "acalabrotar",
    "acalabrotado": "acalabrotar",
    "acalabrotados": "acalabrotar",
    "acalabrotamos": "acalabrotar",
    "acalabrotan": "acalabrotar",
    "acalabrotando": "acalabrotar",
    "acalabrotara": "acalabrotar",
    "acalabrotarais": "acalabrotar",
    "acalabrotaran": "acalabrotar",
    "acalabrotaras": "acalabrotar",
    "acalabrotare": "acalabrotar",
    "acalabrotareis": "acalabrotar",
    "acalabrotaremos": "acalabrotar",
    "acalabrotaren": "acalabrotar",
    "acalabrotares": "acalabrotar",
    "acalabrotaron": "acalabrotar",
    "acalabrotará": "acalabrotar",
    "acalabrotarán": "acalabrotar",
    "acalabrotarás": "acalabrotar",
    "acalabrotaré": "acalabrotar",
    "acalabrotaréis": "acalabrotar",
    "acalabrotaría": "acalabrotar",
    "acalabrotaríais": "acalabrotar",
    "acalabrotaríamos": "acalabrotar",
    "acalabrotarían": "acalabrotar",
    "acalabrotarías": "acalabrotar",
    "acalabrotas": "acalabrotar",
    "acalabrotase": "acalabrotar",
    "acalabrotaseis": "acalabrotar",
    "acalabrotasen": "acalabrotar",
    "acalabrotases": "acalabrotar",
    "acalabrotaste": "acalabrotar",
    "acalabrotasteis": "acalabrotar",
    "acalabrote": "acalabrotar",
    "acalabrotemos": "acalabrotar",
    "acalabroten": "acalabrotar",
    "acalabrotes": "acalabrotar",
    "acalabroto": "acalabrotar",
    "acalabrotábamos": "acalabrotar",
    "acalabrotáis": "acalabrotar",
    "acalabrotáramos": "acalabrotar",
    "acalabrotáremos": "acalabrotar",
    "acalabrotásemos": "acalabrotar",
    "acalabroté": "acalabrotar",
    "acalabrotéis": "acalabrotar",
    "acalabrotó": "acalabrotar",
    "acalambra": "acalambrar",
    "acalambraba": "acalambrar",
    "acalambrabais": "acalambrar",
    "acalambraban": "acalambrar",
    "acalambrabas": "acalambrar",
    "acalambrad": "acalambrar",
    "acalambrada": "acalambrar",
    "acalambradas": "acalambrar",
    "acalambrado": "acalambrar",
    "acalambrados": "acalambrar",
    "acalambramos": "acalambrar",
    "acalambran": "acalambrar",
    "acalambrando": "acalambrar",
    "acalambrara": "acalambrar",
    "acalambrarais": "acalambrar",
    "acalambraran": "acalambrar",
    "acalambraras": "acalambrar",
    "acalambrare": "acalambrar",
    "acalambrareis": "acalambrar",
    "acalambraremos": "acalambrar",
    "acalambraren": "acalambrar",
    "acalambrares": "acalambrar",
    "acalambraron": "acalambrar",
    "acalambrará": "acalambrar",
    "acalambrarán": "acalambrar",
    "acalambrarás": "acalambrar",
    "acalambraré": "acalambrar",
    "acalambraréis": "acalambrar",
    "acalambraría": "acalambrar",
    "acalambraríais": "acalambrar",
    "acalambraríamos": "acalambrar",
    "acalambrarían": "acalambrar",
    "acalambrarías": "acalambrar",
    "acalambras": "acalambrar",
    "acalambrase": "acalambrar",
    "acalambraseis": "acalambrar",
    "acalambrasen": "acalambrar",
    "acalambrases": "acalambrar",
    "acalambraste": "acalambrar",
    "acalambrasteis": "acalambrar",
    "acalambre": "acalambrar",
    "acalambremos": "acalambrar",
    "acalambren": "acalambrar",
    "acalambres": "acalambrar",
    "acalambro": "acalambrar",
    "acalambrábamos": "acalambrar",
    "acalambráis": "acalambrar",
    "acalambráramos": "acalambrar",
    "acalambráremos": "acalambrar",
    "acalambrásemos": "acalambrar",
    "acalambré": "acalambrar",
    "acalambréis": "acalambrar",
    "acalambró": "acalambrar",
    "acalefa": "acalefo",
    "acalefas": "acalefo",
    "acalefos": "acalefo",
    "acalentura": "acalenturar",
    "acalenturaba": "acalenturar",
    "acalenturabais": "acalenturar",
    "acalenturaban": "acalenturar",
    "acalenturabas": "acalenturar",
    "acalenturad": "acalenturar",
    "acalenturada": "acalenturar",
    "acalenturadas": "acalenturar",
    "acalenturado": "acalenturar",
    "acalenturados": "acalenturar",
    "acalenturamos": "acalenturar",
    "acalenturan": "acalenturar",
    "acalenturando": "acalenturar",
    "acalenturara": "acalenturar",
    "acalenturarais": "acalenturar",
    "acalenturaran": "acalenturar",
    "acalenturaras": "acalenturar",
    "acalenturare": "acalenturar",
    "acalenturareis": "acalenturar",
    "acalenturaremos": "acalenturar",
    "acalenturaren": "acalenturar",
    "acalenturares": "acalenturar",
    "acalenturaron": "acalenturar",
    "acalenturará": "acalenturar",
    "acalenturarán": "acalenturar",
    "acalenturarás": "acalenturar",
    "acalenturaré": "acalenturar",
    "acalenturaréis": "acalenturar",
    "acalenturaría": "acalenturar",
    "acalenturaríais": "acalenturar",
    "acalenturaríamos": "acalenturar",
    "acalenturarían": "acalenturar",
    "acalenturarías": "acalenturar",
    "acalenturas": "acalenturar",
    "acalenturase": "acalenturar",
    "acalenturaseis": "acalenturar",
    "acalenturasen": "acalenturar",
    "acalenturases": "acalenturar",
    "acalenturaste": "acalenturar",
    "acalenturasteis": "acalenturar",
    "acalenture": "acalenturar",
    "acalenturemos": "acalenturar",
    "acalenturen": "acalenturar",
    "acalentures": "acalenturar",
    "acalenturo": "acalenturar",
    "acalenturábamos": "acalenturar",
    "acalenturáis": "acalenturar",
    "acalenturáramos": "acalenturar",
    "acalenturáremos": "acalenturar",
    "acalenturásemos": "acalenturar",
    "acalenturé": "acalenturar",
    "acalenturéis": "acalenturar",
    "acalenturó": "acalenturar",
    "acalla": "acallar",
    "acallaba": "acallar",
    "acallabais": "acallar",
    "acallaban": "acallar",
    "acallabas": "acallar",
    "acallad": "acallar",
    "acallada": "acallar",
    "acalladas": "acallar",
    "acallado": "acallar",
    "acallados": "acallar",
    "acallamientos": "acallamiento",
    "acallamos": "acallar",
    "acallan": "acallar",
    "acallando": "acallar",
    "acallanta": "acallantar",
    "acallantaba": "acallantar",
    "acallantabais": "acallantar",
    "acallantaban": "acallantar",
    "acallantabas": "acallantar",
    "acallantad": "acallantar",
    "acallantada": "acallantar",
    "acallantadas": "acallantar",
    "acallantado": "acallantar",
    "acallantados": "acallantar",
    "acallantamos": "acallantar",
    "acallantan": "acallantar",
    "acallantando": "acallantar",
    "acallantara": "acallantar",
    "acallantarais": "acallantar",
    "acallantaran": "acallantar",
    "acallantaras": "acallantar",
    "acallantare": "acallantar",
    "acallantareis": "acallantar",
    "acallantaremos": "acallantar",
    "acallantaren": "acallantar",
    "acallantares": "acallantar",
    "acallantaron": "acallantar",
    "acallantará": "acallantar",
    "acallantarán": "acallantar",
    "acallantarás": "acallantar",
    "acallantaré": "acallantar",
    "acallantaréis": "acallantar",
    "acallantaría": "acallantar",
    "acallantaríais": "acallantar",
    "acallantaríamos": "acallantar",
    "acallantarían": "acallantar",
    "acallantarías": "acallantar",
    "acallantas": "acallantar",
    "acallantase": "acallantar",
    "acallantaseis": "acallantar",
    "acallantasen": "acallantar",
    "acallantases": "acallantar",
    "acallantaste": "acallantar",
    "acallantasteis": "acallantar",
    "acallante": "acallantar",
    "acallantemos": "acallantar",
    "acallanten": "acallantar",
    "acallantes": "acallantar",
    "acallanto": "acallantar",
    "acallantábamos": "acallantar",
    "acallantáis": "acallantar",
    "acallantáramos": "acallantar",
    "acallantáremos": "acallantar",
    "acallantásemos": "acallantar",
    "acallanté": "acallantar",
    "acallantéis": "acallantar",
    "acallantó": "acallantar",
    "acallara": "acallar",
    "acallarais": "acallar",
    "acallaran": "acallar",
    "acallaras": "acallar",
    "acallare": "acallar",
    "acallareis": "acallar",
    "acallaremos": "acallar",
    "acallaren": "acallar",
    "acallares": "acallar",
    "acallaron": "acallar",
    "acallará": "acallar",
    "acallarán": "acallar",
    "acallarás": "acallar",
    "acallaré": "acallar",
    "acallaréis": "acallar",
    "acallaría": "acallar",
    "acallaríais": "acallar",
    "acallaríamos": "acallar",
    "acallarían": "acallar",
    "acallarías": "acallar",
    "acallas": "acallar",
    "acallase": "acallar",
    "acallaseis": "acallar",
    "acallasen": "acallar",
    "acallases": "acallar",
    "acallaste": "acallar",
    "acallasteis": "acallar",
    "acalle": "acallar",
    "acallemos": "acallar",
    "acallen": "acallar",
    "acalles": "acallar",
    "acallo": "acallar",
    "acallábamos": "acallar",
    "acalláis": "acallar",
    "acalláramos": "acallar",
    "acalláremos": "acallar",
    "acallásemos": "acallar",
    "acallé": "acallar",
    "acalléis": "acallar",
    "acalló": "acallar",
    "acalora": "acalorar",
    "acaloraba": "acalorar",
    "acalorabais": "acalorar",
    "acaloraban": "acalorar",
    "acalorabas": "acalorar",
    "acalorad": "acalorar",
    "acalorada": "acalorar",
    "acaloradas": "acalorar",
    "acalorado": "acalorar",
    "acalorados": "acalorar",
    "acaloramientos": "acaloramiento",
    "acaloramos": "acalorar",
    "acaloran": "acalorar",
    "acalorando": "acalorar",
    "acalorara": "acalorar",
    "acalorarais": "acalorar",
    "acaloraran": "acalorar",
    "acaloraras": "acalorar",
    "acalorare": "acalorar",
    "acalorareis": "acalorar",
    "acaloraremos": "acalorar",
    "acaloraren": "acalorar",
    "acalorares": "acalorar",
    "acaloraron": "acalorar",
    "acalorará": "acalorar",
    "acalorarán": "acalorar",
    "acalorarás": "acalorar",
    "acaloraré": "acalorar",
    "acaloraréis": "acalorar",
    "acaloraría": "acalorar",
    "acaloraríais": "acalorar",
    "acaloraríamos": "acalorar",
    "acalorarían": "acalorar",
    "acalorarías": "acalorar",
    "acaloras": "acalorar",
    "acalorase": "acalorar",
    "acaloraseis": "acalorar",
    "acalorasen": "acalorar",
    "acalorases": "acalorar",
    "acaloraste": "acalorar",
    "acalorasteis": "acalorar",
    "acalore": "acalorar",
    "acaloremos": "acalorar",
    "acaloren": "acalorar",
    "acalores": "acalorar",
    "acaloro": "acalorar",
    "acaloros": "acaloro",
    "acalorábamos": "acalorar",
    "acaloráis": "acalorar",
    "acaloráramos": "acalorar",
    "acaloráremos": "acalorar",
    "acalorásemos": "acalorar",
    "acaloré": "acalorar",
    "acaloréis": "acalorar",
    "acaloró": "acalorar",
    "acama": "acamar",
    "acamaba": "acamar",
    "acamabais": "acamar",
    "acamaban": "acamar",
    "acamabas": "acamar",
    "acamad": "acamar",
    "acamada": "acamar",
    "acamadas": "acamar",
    "acamado": "acamar",
    "acamados": "acamar",
    "acamamos": "acamar",
    "acaman": "acamar",
    "acamando": "acamar",
    "acamara": "acamar",
    "acamarais": "acamar",
    "acamaran": "acamar",
    "acamaras": "acamar",
    "acamare": "acamar",
    "acamareis": "acamar",
    "acamaremos": "acamar",
    "acamaren": "acamar",
    "acamares": "acamar",
    "acamaron": "acamar",
    "acamará": "acamar",
    "acamarán": "acamar",
    "acamarás": "acamar",
    "acamaré": "acamar",
    "acamaréis": "acamar",
    "acamaría": "acamar",
    "acamaríais": "acamar",
    "acamaríamos": "acamar",
    "acamarían": "acamar",
    "acamarías": "acamar",
    "acamas": "acamar",
    "acamase": "acamar",
    "acamaseis": "acamar",
    "acamasen": "acamar",
    "acamases": "acamar",
    "acamaste": "acamar",
    "acamasteis": "acamar",
    "acame": "acamar",
    "acamellada": "acamellado",
    "acamelladas": "acamellado",
    "acamellados": "acamellado",
    "acamemos": "acamar",
    "acamen": "acamar",
    "acames": "acamar",
    "acamo": "acamar",
    "acampa": "acampar",
    "acampaba": "acampar",
    "acampabais": "acampar",
    "acampaban": "acampar",
    "acampabas": "acampar",
    "acampad": "acampar",
    "acampada": "acampar",
    "acampadas": "acampar",
    "acampado": "acampar",
    "acampados": "acampar",
    "acampamos": "acampar",
    "acampan": "acampar",
    "acampanada": "acampanado",
    "acampanadas": "acampanado",
    "acampanados": "acampanado",
    "acampando": "acampar",
    "acampara": "acampar",
    "acamparais": "acampar",
    "acamparan": "acampar",
    "acamparas": "acampar",
    "acampare": "acampar",
    "acampareis": "acampar",
    "acamparemos": "acampar",
    "acamparen": "acampar",
    "acampares": "acampar",
    "acamparon": "acampar",
    "acampará": "acampar",
    "acamparán": "acampar",
    "acamparás": "acampar",
    "acamparé": "acampar",
    "acamparéis": "acampar",
    "acamparía": "acampar",
    "acamparíais": "acampar",
    "acamparíamos": "acampar",
    "acamparían": "acampar",
    "acamparías": "acampar",
    "acampas": "acampar",
    "acampase": "acampar",
    "acampaseis": "acampar",
    "acampasen": "acampar",
    "acampases": "acampar",
    "acampaste": "acampar",
    "acampasteis": "acampar",
    "acampe": "acampar",
    "acampemos": "acampar",
    "acampen": "acampar",
    "acampes": "acampar",
    "acampo": "acampar",
    "acampábamos": "acampar",
    "acampáis": "acampar",
    "acampáramos": "acampar",
    "acampáremos": "acampar",
    "acampásemos": "acampar",
    "acampé": "acampar",
    "acampéis": "acampar",
    "acampó": "acampar",
    "acamábamos": "acamar",
    "acamáis": "acamar",
    "acamáramos": "acamar",
    "acamáremos": "acamar",
    "acamásemos": "acamar",
    "acamé": "acamar",
    "acaméis": "acamar",
    "acamó": "acamar",
    "acanala": "acanalar",
    "acanalaba": "acanalar",
    "acanalabais": "acanalar",
    "acanalaban": "acanalar",
    "acanalabas": "acanalar",
    "acanalad": "acanalar",
    "acanalada": "acanalar",
    "acanaladas": "acanalar",
    "acanalado": "acanalar",
    "acanaladoras": "acanaladora",
    "acanaladores": "acanalador",
    "acanalados": "acanalar",
    "acanaladuras": "acanaladura",
    "acanalamientos": "acanalamiento",
    "acanalamos": "acanalar",
    "acanalan": "acanalar",
    "acanalando": "acanalar",
    "acanalara": "acanalar",
    "acanalarais": "acanalar",
    "acanalaran": "acanalar",
    "acanalaras": "acanalar",
    "acanalare": "acanalar",
    "acanalareis": "acanalar",
    "acanalaremos": "acanalar",
    "acanalaren": "acanalar",
    "acanalares": "acanalar",
    "acanalaron": "acanalar",
    "acanalará": "acanalar",
    "acanalarán": "acanalar",
    "acanalarás": "acanalar",
    "acanalaré": "acanalar",
    "acanalaréis": "acanalar",
    "acanalaría": "acanalar",
    "acanalaríais": "acanalar",
    "acanalaríamos": "acanalar",
    "acanalarían": "acanalar",
    "acanalarías": "acanalar",
    "acanalas": "acanalar",
    "acanalase": "acanalar",
    "acanalaseis": "acanalar",
    "acanalasen": "acanalar",
    "acanalases": "acanalar",
    "acanalaste": "acanalar",
    "acanalasteis": "acanalar",
    "acanale": "acanalar",
    "acanalemos": "acanalar",
    "acanalen": "acanalar",
    "acanales": "acanalar",
    "acanalo": "acanalar",
    "acanalábamos": "acanalar",
    "acanaláis": "acanalar",
    "acanaláramos": "acanalar",
    "acanaláremos": "acanalar",
    "acanalásemos": "acanalar",
    "acanalé": "acanalar",
    "acanaléis": "acanalar",
    "acanaló": "acanalar",
    "acancha": "acanchar",
    "acanchaba": "acanchar",
    "acanchabais": "acanchar",
    "acanchaban": "acanchar",
    "acanchabas": "acanchar",
    "acanchad": "acanchar",
    "acanchada": "acanchar",
    "acanchadas": "acanchar",
    "acanchado": "acanchar",
    "acanchados": "acanchar",
    "acanchamos": "acanchar",
    "acanchan": "acanchar",
    "acanchando": "acanchar",
    "acanchara": "acanchar",
    "acancharais": "acanchar",
    "acancharan": "acanchar",
    "acancharas": "acanchar",
    "acanchare": "acanchar",
    "acanchareis": "acanchar",
    "acancharemos": "acanchar",
    "acancharen": "acanchar",
    "acanchares": "acanchar",
    "acancharon": "acanchar",
    "acanchará": "acanchar",
    "acancharán": "acanchar",
    "acancharás": "acanchar",
    "acancharé": "acanchar",
    "acancharéis": "acanchar",
    "acancharía": "acanchar",
    "acancharíais": "acanchar",
    "acancharíamos": "acanchar",
    "acancharían": "acanchar",
    "acancharías": "acanchar",
    "acanchas": "acanchar",
    "acanchase": "acanchar",
    "acanchaseis": "acanchar",
    "acanchasen": "acanchar",
    "acanchases": "acanchar",
    "acanchaste": "acanchar",
    "acanchasteis": "acanchar",
    "acanche": "acanchar",
    "acanchemos": "acanchar",
    "acanchen": "acanchar",
    "acanches": "acanchar",
    "acancho": "acanchar",
    "acanchábamos": "acanchar",
    "acancháis": "acanchar",
    "acancháramos": "acanchar",
    "acancháremos": "acanchar",
    "acanchásemos": "acanchar",
    "acanché": "acanchar",
    "acanchéis": "acanchar",
    "acanchó": "acanchar",
    "acanelada": "acanelado",
    "acaneladas": "acanelado",
    "acanelados": "acanelado",
    "acanelona": "acanelonar",
    "acanelonaba": "acanelonar",
    "acanelonabais": "acanelonar",
    "acanelonaban": "acanelonar",
    "acanelonabas": "acanelonar",
    "acanelonad": "acanelonar",
    "acanelonada": "acanelonar",
    "acanelonadas": "acanelonar",
    "acanelonado": "acanelonar",
    "acanelonados": "acanelonar",
    "acanelonamos": "acanelonar",
    "acanelonan": "acanelonar",
    "acanelonando": "acanelonar",
    "acanelonara": "acanelonar",
    "acanelonarais": "acanelonar",
    "acanelonaran": "acanelonar",
    "acanelonaras": "acanelonar",
    "acanelonare": "acanelonar",
    "acanelonareis": "acanelonar",
    "acanelonaremos": "acanelonar",
    "acanelonaren": "acanelonar",
    "acanelonares": "acanelonar",
    "acanelonaron": "acanelonar",
    "acanelonará": "acanelonar",
    "acanelonarán": "acanelonar",
    "acanelonarás": "acanelonar",
    "acanelonaré": "acanelonar",
    "acanelonaréis": "acanelonar",
    "acanelonaría": "acanelonar",
    "acanelonaríais": "acanelonar",
    "acanelonaríamos": "acanelonar",
    "acanelonarían": "acanelonar",
    "acanelonarías": "acanelonar",
    "acanelonas": "acanelonar",
    "acanelonase": "acanelonar",
    "acanelonaseis": "acanelonar",
    "acanelonasen": "acanelonar",
    "acanelonases": "acanelonar",
    "acanelonaste": "acanelonar",
    "acanelonasteis": "acanelonar",
    "acanelone": "acanelonar",
    "acanelonemos": "acanelonar",
    "acanelonen": "acanelonar",
    "acanelones": "acanelonar",
    "acanelono": "acanelonar",
    "acanelonábamos": "acanelonar",
    "acanelonáis": "acanelonar",
    "acanelonáramos": "acanelonar",
    "acanelonáremos": "acanelonar",
    "acanelonásemos": "acanelonar",
    "acaneloné": "acanelonar",
    "acanelonéis": "acanelonar",
    "acanelonó": "acanelonar",
    "acanillada": "acanillado",
    "acanilladas": "acanillado",
    "acanillados": "acanillado",
    "acanilladuras": "acanilladura",
    "acantila": "acantilar",
    "acantilaba": "acantilar",
    "acantilabais": "acantilar",
    "acantilaban": "acantilar",
    "acantilabas": "acantilar",
    "acantilad": "acantilar",
    "acantilada": "acantilar",
    "acantiladas": "acantilar",
    "acantilado": "acantilar",
    "acantilados": "acantilar",
    "acantilamos": "acantilar",
    "acantilan": "acantilar",
    "acantilando": "acantilar",
    "acantilara": "acantilar",
    "acantilarais": "acantilar",
    "acantilaran": "acantilar",
    "acantilaras": "acantilar",
    "acantilare": "acantilar",
    "acantilareis": "acantilar",
    "acantilaremos": "acantilar",
    "acantilaren": "acantilar",
    "acantilares": "acantilar",
    "acantilaron": "acantilar",
    "acantilará": "acantilar",
    "acantilarán": "acantilar",
    "acantilarás": "acantilar",
    "acantilaré": "acantilar",
    "acantilaréis": "acantilar",
    "acantilaría": "acantilar",
    "acantilaríais": "acantilar",
    "acantilaríamos": "acantilar",
    "acantilarían": "acantilar",
    "acantilarías": "acantilar",
    "acantilas": "acantilar",
    "acantilase": "acantilar",
    "acantilaseis": "acantilar",
    "acantilasen": "acantilar",
    "acantilases": "acantilar",
    "acantilaste": "acantilar",
    "acantilasteis": "acantilar",
    "acantile": "acantilar",
    "acantilemos": "acantilar",
    "acantilen": "acantilar",
    "acantiles": "acantilar",
    "acantilo": "acantilar",
    "acantilábamos": "acantilar",
    "acantiláis": "acantilar",
    "acantiláramos": "acantilar",
    "acantiláremos": "acantilar",
    "acantilásemos": "acantilar",
    "acantilé": "acantilar",
    "acantiléis": "acantilar",
    "acantiló": "acantilar",
    "acantocitos": "acantocito",
    "acantocéfalos": "acantocéfalo",
    "acantona": "acantonar",
    "acantonaba": "acantonar",
    "acantonabais": "acantonar",
    "acantonaban": "acantonar",
    "acantonabas": "acantonar",
    "acantonad": "acantonar",
    "acantonada": "acantonar",
    "acantonadas": "acantonar",
    "acantonado": "acantonar",
    "acantonados": "acantonar",
    "acantonamientos": "acantonamiento",
    "acantonamos": "acantonar",
    "acantonan": "acantonar",
    "acantonando": "acantonar",
    "acantonara": "acantonar",
    "acantonarais": "acantonar",
    "acantonaran": "acantonar",
    "acantonaras": "acantonar",
    "acantonare": "acantonar",
    "acantonareis": "acantonar",
    "acantonaremos": "acantonar",
    "acantonaren": "acantonar",
    "acantonares": "acantonar",
    "acantonaron": "acantonar",
    "acantonará": "acantonar",
    "acantonarán": "acantonar",
    "acantonarás": "acantonar",
    "acantonaré": "acantonar",
    "acantonaréis": "acantonar",
    "acantonaría": "acantonar",
    "acantonaríais": "acantonar",
    "acantonaríamos": "acantonar",
    "acantonarían": "acantonar",
    "acantonarías": "acantonar",
    "acantonas": "acantonar",
    "acantonase": "acantonar",
    "acantonaseis": "acantonar",
    "acantonasen": "acantonar",
    "acantonases": "acantonar",
    "acantonaste": "acantonar",
    "acantonasteis": "acantonar",
    "acantone": "acantonar",
    "acantonemos": "acantonar",
    "acantonen": "acantonar",
    "acantones": "acantonar",
    "acantono": "acantonar",
    "acantonábamos": "acantonar",
    "acantonáis": "acantonar",
    "acantonáramos": "acantonar",
    "acantonáremos": "acantonar",
    "acantonásemos": "acantonar",
    "acantoné": "acantonar",
    "acantonéis": "acantonar",
    "acantonó": "acantonar",
    "acantopterigios": "acantopterigio",
    "acantoquitónidos": "acantoquitónido",
    "acantos": "acanto",
    "acantosomas": "acantosoma",
    "acantáceas": "acantácea",
    "acantúridos": "acantúrido",
    "acaobada": "acaobado",
    "acaobadas": "acaobado",
    "acaobados": "acaobado",
    "acapara": "acaparar",
    "acaparaba": "acaparar",
    "acaparabais": "acaparar",
    "acaparaban": "acaparar",
    "acaparabas": "acaparar",
    "acaparad": "acaparar",
    "acaparada": "acaparar",
    "acaparadas": "acaparar",
    "acaparado": "acaparar",
    "acaparadora": "acaparador",
    "acaparadoras": "acaparador",
    "acaparadores": "acaparador",
    "acaparados": "acaparar",
    "acaparamientos": "acaparamiento",
    "acaparamos": "acaparar",
    "acaparan": "acaparar",
    "acaparando": "acaparar",
    "acaparara": "acaparar",
    "acapararais": "acaparar",
    "acapararan": "acaparar",
    "acapararas": "acaparar",
    "acaparare": "acaparar",
    "acaparareis": "acaparar",
    "acapararemos": "acaparar",
    "acapararen": "acaparar",
    "acaparares": "acaparar",
    "acapararon": "acaparar",
    "acaparará": "acaparar",
    "acapararán": "acaparar",
    "acapararás": "acaparar",
    "acapararé": "acaparar",
    "acapararéis": "acaparar",
    "acapararía": "acaparar",
    "acapararíais": "acaparar",
    "acapararíamos": "acaparar",
    "acapararían": "acaparar",
    "acapararías": "acaparar",
    "acaparas": "acaparar",
    "acaparase": "acaparar",
    "acaparaseis": "acaparar",
    "acaparasen": "acaparar",
    "acaparases": "acaparar",
    "acaparaste": "acaparar",
    "acaparasteis": "acaparar",
    "acapare": "acaparar",
    "acaparemos": "acaparar",
    "acaparen": "acaparar",
    "acapares": "acaparar",
    "acaparo": "acaparar",
    "acaparra": "acaparrar",
    "acaparraba": "acaparrar",
    "acaparrabais": "acaparrar",
    "acaparraban": "acaparrar",
    "acaparrabas": "acaparrar",
    "acaparrad": "acaparrar",
    "acaparrada": "acaparrar",
    "acaparradas": "acaparrar",
    "acaparrado": "acaparrar",
    "acaparrados": "acaparrar",
    "acaparramos": "acaparrar",
    "acaparran": "acaparrar",
    "acaparrando": "acaparrar",
    "acaparrara": "acaparrar",
    "acaparrarais": "acaparrar",
    "acaparraran": "acaparrar",
    "acaparraras": "acaparrar",
    "acaparrare": "acaparrar",
    "acaparrareis": "acaparrar",
    "acaparraremos": "acaparrar",
    "acaparraren": "acaparrar",
    "acaparrares": "acaparrar",
    "acaparraron": "acaparrar",
    "acaparrará": "acaparrar",
    "acaparrarán": "acaparrar",
    "acaparrarás": "acaparrar",
    "acaparraré": "acaparrar",
    "acaparraréis": "acaparrar",
    "acaparraría": "acaparrar",
    "acaparraríais": "acaparrar",
    "acaparraríamos": "acaparrar",
    "acaparrarían": "acaparrar",
    "acaparrarías": "acaparrar",
    "acaparras": "acaparrar",
    "acaparrase": "acaparrar",
    "acaparraseis": "acaparrar",
    "acaparrasen": "acaparrar",
    "acaparrases": "acaparrar",
    "acaparraste": "acaparrar",
    "acaparrasteis": "acaparrar",
    "acaparre": "acaparrar",
    "acaparremos": "acaparrar",
    "acaparren": "acaparrar",
    "acaparres": "acaparrar",
    "acaparro": "acaparrar",
    "acaparrábamos": "acaparrar",
    "acaparráis": "acaparrar",
    "acaparráramos": "acaparrar",
    "acaparráremos": "acaparrar",
    "acaparrásemos": "acaparrar",
    "acaparré": "acaparrar",
    "acaparréis": "acaparrar",
    "acaparró": "acaparrar",
    "acaparábamos": "acaparar",
    "acaparáis": "acaparar",
    "acaparáramos": "acaparar",
    "acaparáremos": "acaparar",
    "acaparásemos": "acaparar",
    "acaparé": "acaparar",
    "acaparéis": "acaparar",
    "acaparó": "acaparar",
    "acaponada": "acaponado",
    "acaponadas": "acaponado",
    "acaponados": "acaponado",
    "acara": "acarar",
    "acaraba": "acarar",
    "acarabais": "acarar",
    "acaraban": "acarar",
    "acarabas": "acarar",
    "acaracolada": "acaracolado",
    "acaracoladas": "acaracolado",
    "acaracolados": "acaracolado",
    "acarad": "acarar",
    "acarada": "acarar",
    "acaradas": "acarar",
    "acarado": "acarar",
    "acarados": "acarar",
    "acaramela": "acaramelar",
    "acaramelaba": "acaramelar",
    "acaramelabais": "acaramelar",
    "acaramelaban": "acaramelar",
    "acaramelabas": "acaramelar",
    "acaramelad": "acaramelar",
    "acaramelada": "acaramelar",
    "acarameladas": "acaramelar",
    "acaramelado": "acaramelar",
    "acaramelados": "acaramelar",
    "acaramelamientos": "acaramelamiento",
    "acaramelamos": "acaramelar",
    "acaramelan": "acaramelar",
    "acaramelando": "acaramelar",
    "acaramelara": "acaramelar",
    "acaramelarais": "acaramelar",
    "acaramelaran": "acaramelar",
    "acaramelaras": "acaramelar",
    "acaramelare": "acaramelar",
    "acaramelareis": "acaramelar",
    "acaramelaremos": "acaramelar",
    "acaramelaren": "acaramelar",
    "acaramelares": "acaramelar",
    "acaramelaron": "acaramelar",
    "acaramelará": "acaramelar",
    "acaramelarán": "acaramelar",
    "acaramelarás": "acaramelar",
    "acaramelaré": "acaramelar",
    "acaramelaréis": "acaramelar",
    "acaramelaría": "acaramelar",
    "acaramelaríais": "acaramelar",
    "acaramelaríamos": "acaramelar",
    "acaramelarían": "acaramelar",
    "acaramelarías": "acaramelar",
    "acaramelas": "acaramelar",
    "acaramelase": "acaramelar",
    "acaramelaseis": "acaramelar",
    "acaramelasen": "acaramelar",
    "acaramelases": "acaramelar",
    "acaramelaste": "acaramelar",
    "acaramelasteis": "acaramelar",
    "acaramele": "acaramelar",
    "acaramelemos": "acaramelar",
    "acaramelen": "acaramelar",
    "acarameles": "acaramelar",
    "acaramelo": "acaramelar",
    "acaramelábamos": "acaramelar",
    "acarameláis": "acaramelar",
    "acarameláramos": "acaramelar",
    "acarameláremos": "acaramelar",
    "acaramelásemos": "acaramelar",
    "acaramelé": "acaramelar",
    "acarameléis": "acaramelar",
    "acarameló": "acaramelar",
    "acaramos": "acarar",
    "acaran": "acarar",
    "acarando": "acarar",
    "acarara": "acarar",
    "acararais": "acarar",
    "acararan": "acarar",
    "acararas": "acarar",
    "acarare": "acarar",
    "acarareis": "acarar",
    "acararemos": "acarar",
    "acararen": "acarar",
    "acarares": "acarar",
    "acararon": "acarar",
    "acarará": "acarar",
    "acararán": "acarar",
    "acararás": "acarar",
    "acararé": "acarar",
    "acararéis": "acarar",
    "acararía": "acarar",
    "acararíais": "acarar",
    "acararíamos": "acarar",
    "acararían": "acarar",
    "acararías": "acarar",
    "acaras": "acarar",
    "acarase": "acarar",
    "acaraseis": "acarar",
    "acarasen": "acarar",
    "acarases": "acarar",
    "acaraste": "acarar",
    "acarasteis": "acarar",
    "acarbosas": "acarbosa",
    "acarcavamientos": "acarcavamiento",
    "acardenala": "acardenalar",
    "acardenalaba": "acardenalar",
    "acardenalabais": "acardenalar",
    "acardenalaban": "acardenalar",
    "acardenalabas": "acardenalar",
    "acardenalad": "acardenalar",
    "acardenalada": "acardenalar",
    "acardenaladas": "acardenalar",
    "acardenalado": "acardenalar",
    "acardenalados": "acardenalar",
    "acardenalamos": "acardenalar",
    "acardenalan": "acardenalar",
    "acardenalando": "acardenalar",
    "acardenalara": "acardenalar",
    "acardenalarais": "acardenalar",
    "acardenalaran": "acardenalar",
    "acardenalaras": "acardenalar",
    "acardenalare": "acardenalar",
    "acardenalareis": "acardenalar",
    "acardenalaremos": "acardenalar",
    "acardenalaren": "acardenalar",
    "acardenalares": "acardenalar",
    "acardenalaron": "acardenalar",
    "acardenalará": "acardenalar",
    "acardenalarán": "acardenalar",
    "acardenalarás": "acardenalar",
    "acardenalaré": "acardenalar",
    "acardenalaréis": "acardenalar",
    "acardenalaría": "acardenalar",
    "acardenalaríais": "acardenalar",
    "acardenalaríamos": "acardenalar",
    "acardenalarían": "acardenalar",
    "acardenalarías": "acardenalar",
    "acardenalas": "acardenalar",
    "acardenalase": "acardenalar",
    "acardenalaseis": "acardenalar",
    "acardenalasen": "acardenalar",
    "acardenalases": "acardenalar",
    "acardenalaste": "acardenalar",
    "acardenalasteis": "acardenalar",
    "acardenale": "acardenalar",
    "acardenalemos": "acardenalar",
    "acardenalen": "acardenalar",
    "acardenales": "acardenalar",
    "acardenalo": "acardenalar",
    "acardenalábamos": "acardenalar",
    "acardenaláis": "acardenalar",
    "acardenaláramos": "acardenalar",
    "acardenaláremos": "acardenalar",
    "acardenalásemos": "acardenalar",
    "acardenalé": "acardenalar",
    "acardenaléis": "acardenalar",
    "acardenaló": "acardenalar",
    "acare": "acarar",
    "acarea": "acarear",
    "acareaba": "acarear",
    "acareabais": "acarear",
    "acareaban": "acarear",
    "acareabas": "acarear",
    "acaread": "acarear",
    "acareada": "acarear",
    "acareadas": "acarear",
    "acareado": "acarear",
    "acareados": "acarear",
    "acareamientos": "acareamiento",
    "acareamos": "acarear",
    "acarean": "acarear",
    "acareando": "acarear",
    "acareara": "acarear",
    "acarearais": "acarear",
    "acarearan": "acarear",
    "acarearas": "acarear",
    "acareare": "acarear",
    "acareareis": "acarear",
    "acarearemos": "acarear",
    "acarearen": "acarear",
    "acareares": "acarear",
    "acarearon": "acarear",
    "acareará": "acarear",
    "acarearán": "acarear",
    "acarearás": "acarear",
    "acarearé": "acarear",
    "acarearéis": "acarear",
    "acarearía": "acarear",
    "acarearíais": "acarear",
    "acarearíamos": "acarear",
    "acarearían": "acarear",
    "acarearías": "acarear",
    "acareas": "acarear",
    "acarease": "acarear",
    "acareaseis": "acarear",
    "acareasen": "acarear",
    "acareases": "acarear",
    "acareaste": "acarear",
    "acareasteis": "acarear",
    "acaree": "acarear",
    "acareemos": "acarear",
    "acareen": "acarear",
    "acarees": "acarear",
    "acaremos": "acarar",
    "acaren": "acarar",
    "acareo": "acarear",
    "acares": "acarar",
    "acareábamos": "acarear",
    "acareáis": "acarear",
    "acareáramos": "acarear",
    "acareáremos": "acarear",
    "acareásemos": "acarear",
    "acareé": "acarear",
    "acareéis": "acarear",
    "acareó": "acarear",
    "acaricia": "acariciar",
    "acariciaba": "acariciar",
    "acariciabais": "acariciar",
    "acariciaban": "acariciar",
    "acariciabas": "acariciar",
    "acariciad": "acariciar",
    "acariciada": "acariciar",
    "acariciadas": "acariciar",
    "acariciado": "acariciar",
    "acariciadora": "acariciador",
    "acariciadoras": "acariciador",
    "acariciadores": "acariciador",
    "acariciados": "acariciar",
    "acariciamos": "acariciar",
    "acarician": "acariciar",
    "acariciando": "acariciar",
    "acariciantes": "acariciante",
    "acariciara": "acariciar",
    "acariciarais": "acariciar",
    "acariciaran": "acariciar",
    "acariciaras": "acariciar",
    "acariciare": "acariciar",
    "acariciareis": "acariciar",
    "acariciaremos": "acariciar",
    "acariciaren": "acariciar",
    "acariciares": "acariciar",
    "acariciaron": "acariciar",
    "acariciará": "acariciar",
    "acariciarán": "acariciar",
    "acariciarás": "acariciar",
    "acariciaré": "acariciar",
    "acariciaréis": "acariciar",
    "acariciaría": "acariciar",
    "acariciaríais": "acariciar",
    "acariciaríamos": "acariciar",
    "acariciarían": "acariciar",
    "acariciarías": "acariciar",
    "acaricias": "acariciar",
    "acariciase": "acariciar",
    "acariciaseis": "acariciar",
    "acariciasen": "acariciar",
    "acariciases": "acariciar",
    "acariciaste": "acariciar",
    "acariciasteis": "acariciar",
    "acaricidas": "acaricida",
    "acaricie": "acariciar",
    "acariciemos": "acariciar",
    "acaricien": "acariciar",
    "acaricies": "acariciar",
    "acaricio": "acariciar",
    "acariciábamos": "acariciar",
    "acariciáis": "acariciar",
    "acariciáramos": "acariciar",
    "acariciáremos": "acariciar",
    "acariciásemos": "acariciar",
    "acaricié": "acariciar",
    "acariciéis": "acariciar",
    "acarició": "acariciar",
    "acariocitos": "acariocito",
    "acarminada": "acarminado",
    "acarminadas": "acarminado",
    "acarminados": "acarminado",
    "acaro": "acarar",
    "acarra": "acarrar",
    "acarraba": "acarrar",
    "acarrabais": "acarrar",
    "acarraban": "acarrar",
    "acarrabas": "acarrar",
    "acarrad": "acarrar",
    "acarrada": "acarrar",
    "acarradas": "acarrar",
    "acarrado": "acarrar",
    "acarrados": "acarrar",
    "acarrala": "acarralar",
    "acarralaba": "acarralar",
    "acarralabais": "acarralar",
    "acarralaban": "acarralar",
    "acarralabas": "acarralar",
    "acarralad": "acarralar",
    "acarralada": "acarralar",
    "acarraladas": "acarralar",
    "acarralado": "acarralar",
    "acarralados": "acarralar",
    "acarralamos": "acarralar",
    "acarralan": "acarralar",
    "acarralando": "acarralar",
    "acarralara": "acarralar",
    "acarralarais": "acarralar",
    "acarralaran": "acarralar",
    "acarralaras": "acarralar",
    "acarralare": "acarralar",
    "acarralareis": "acarralar",
    "acarralaremos": "acarralar",
    "acarralaren": "acarralar",
    "acarralares": "acarralar",
    "acarralaron": "acarralar",
    "acarralará": "acarralar",
    "acarralarán": "acarralar",
    "acarralarás": "acarralar",
    "acarralaré": "acarralar",
    "acarralaréis": "acarralar",
    "acarralaría": "acarralar",
    "acarralaríais": "acarralar",
    "acarralaríamos": "acarralar",
    "acarralarían": "acarralar",
    "acarralarías": "acarralar",
    "acarralas": "acarralar",
    "acarralase": "acarralar",
    "acarralaseis": "acarralar",
    "acarralasen": "acarralar",
    "acarralases": "acarralar",
    "acarralaste": "acarralar",
    "acarralasteis": "acarralar",
    "acarrale": "acarralar",
    "acarralemos": "acarralar",
    "acarralen": "acarralar",
    "acarrales": "acarralar",
    "acarralo": "acarralar",
    "acarralábamos": "acarralar",
    "acarraláis": "acarralar",
    "acarraláramos": "acarralar",
    "acarraláremos": "acarralar",
    "acarralásemos": "acarralar",
    "acarralé": "acarralar",
    "acarraléis": "acarralar",
    "acarraló": "acarralar",
    "acarramos": "acarrar",
    "acarran": "acarrar",
    "acarrando": "acarrar",
    "acarrara": "acarrar",
    "acarrarais": "acarrar",
    "acarraran": "acarrar",
    "acarraras": "acarrar",
    "acarrare": "acarrar",
    "acarrareis": "acarrar",
    "acarraremos": "acarrar",
    "acarraren": "acarrar",
    "acarrares": "acarrar",
    "acarraron": "acarrar",
    "acarrará": "acarrar",
    "acarrarán": "acarrar",
    "acarrarás": "acarrar",
    "acarraré": "acarrar",
    "acarraréis": "acarrar",
    "acarraría": "acarrar",
    "acarraríais": "acarrar",
    "acarraríamos": "acarrar",
    "acarrarían": "acarrar",
    "acarrarías": "acarrar",
    "acarras": "acarrar",
    "acarrase": "acarrar",
    "acarraseis": "acarrar",
    "acarrasen": "acarrar",
    "acarrases": "acarrar",
    "acarraste": "acarrar",
    "acarrasteis": "acarrar",
    "acarre": "acarrar",
    "acarrea": "acarrear",
    "acarreaba": "acarrear",
    "acarreabais": "acarrear",
    "acarreaban": "acarrear",
    "acarreabas": "acarrear",
    "acarread": "acarrear",
    "acarreada": "acarrear",
    "acarreadas": "acarrear",
    "acarreado": "acarrear",
    "acarreadora": "acarreador",
    "acarreadoras": "acarreador",
    "acarreadores": "acarreador",
    "acarreados": "acarrear",
    "acarreamientos": "acarreamiento",
    "acarreamos": "acarrear",
    "acarrean": "acarrear",
    "acarreando": "acarrear",
    "acarreara": "acarrear",
    "acarrearais": "acarrear",
    "acarrearan": "acarrear",
    "acarrearas": "acarrear",
    "acarreare": "acarrear",
    "acarreareis": "acarrear",
    "acarrearemos": "acarrear",
    "acarrearen": "acarrear",
    "acarreares": "acarrear",
    "acarrearon": "acarrear",
    "acarreará": "acarrear",
    "acarrearán": "acarrear",
    "acarrearás": "acarrear",
    "acarrearé": "acarrear",
    "acarrearéis": "acarrear",
    "acarrearía": "acarrear",
    "acarrearíais": "acarrear",
    "acarrearíamos": "acarrear",
    "acarrearían": "acarrear",
    "acarrearías": "acarrear",
    "acarreas": "acarrear",
    "acarrease": "acarrear",
    "acarreaseis": "acarrear",
    "acarreasen": "acarrear",
    "acarreases": "acarrear",
    "acarreaste": "acarrear",
    "acarreasteis": "acarrear",
    "acarree": "acarrear",
    "acarreemos": "acarrear",
    "acarreen": "acarrear",
    "acarrees": "acarrear",
    "acarremos": "acarrar",
    "acarren": "acarrar",
    "acarreo": "acarrear",
    "acarreos": "acarreo",
    "acarres": "acarrar",
    "acarreábamos": "acarrear",
    "acarreáis": "acarrear",
    "acarreáramos": "acarrear",
    "acarreáremos": "acarrear",
    "acarreásemos": "acarrear",
    "acarreé": "acarrear",
    "acarreéis": "acarrear",
    "acarreó": "acarrear",
    "acarro": "acarrar",
    "acarroña": "acarroñar",
    "acarroñaba": "acarroñar",
    "acarroñabais": "acarroñar",
    "acarroñaban": "acarroñar",
    "acarroñabas": "acarroñar",
    "acarroñad": "acarroñar",
    "acarroñada": "acarroñar",
    "acarroñadas": "acarroñar",
    "acarroñado": "acarroñar",
    "acarroñados": "acarroñar",
    "acarroñamos": "acarroñar",
    "acarroñan": "acarroñar",
    "acarroñando": "acarroñar",
    "acarroñara": "acarroñar",
    "acarroñarais": "acarroñar",
    "acarroñaran": "acarroñar",
    "acarroñaras": "acarroñar",
    "acarroñare": "acarroñar",
    "acarroñareis": "acarroñar",
    "acarroñaremos": "acarroñar",
    "acarroñaren": "acarroñar",
    "acarroñares": "acarroñar",
    "acarroñaron": "acarroñar",
    "acarroñará": "acarroñar",
    "acarroñarán": "acarroñar",
    "acarroñarás": "acarroñar",
    "acarroñaré": "acarroñar",
    "acarroñaréis": "acarroñar",
    "acarroñaría": "acarroñar",
    "acarroñaríais": "acarroñar",
    "acarroñaríamos": "acarroñar",
    "acarroñarían": "acarroñar",
    "acarroñarías": "acarroñar",
    "acarroñas": "acarroñar",
    "acarroñase": "acarroñar",
    "acarroñaseis": "acarroñar",
    "acarroñasen": "acarroñar",
    "acarroñases": "acarroñar",
    "acarroñaste": "acarroñar",
    "acarroñasteis": "acarroñar",
    "acarroñe": "acarroñar",
    "acarroñemos": "acarroñar",
    "acarroñen": "acarroñar",
    "acarroñes": "acarroñar",
    "acarroño": "acarroñar",
    "acarroñábamos": "acarroñar",
    "acarroñáis": "acarroñar",
    "acarroñáramos": "acarroñar",
    "acarroñáremos": "acarroñar",
    "acarroñásemos": "acarroñar",
    "acarroñé": "acarroñar",
    "acarroñéis": "acarroñar",
    "acarroñó": "acarroñar",
    "acarrábamos": "acarrar",
    "acarráis": "acarrar",
    "acarráramos": "acarrar",
    "acarráremos": "acarrar",
    "acarrásemos": "acarrar",
    "acarré": "acarrar",
    "acarréis": "acarrar",
    "acarró": "acarrar",
    "acartona": "acartonar",
    "acartonaba": "acartonar",
    "acartonabais": "acartonar",
    "acartonaban": "acartonar",
    "acartonabas": "acartonar",
    "acartonad": "acartonar",
    "acartonada": "acartonar",
    "acartonadas": "acartonar",
    "acartonado": "acartonar",
    "acartonados": "acartonar",
    "acartonamientos": "acartonamiento",
    "acartonamos": "acartonar",
    "acartonan": "acartonar",
    "acartonando": "acartonar",
    "acartonara": "acartonar",
    "acartonarais": "acartonar",
    "acartonaran": "acartonar",
    "acartonaras": "acartonar",
    "acartonare": "acartonar",
    "acartonareis": "acartonar",
    "acartonaremos": "acartonar",
    "acartonaren": "acartonar",
    "acartonares": "acartonar",
    "acartonaron": "acartonar",
    "acartonará": "acartonar",
    "acartonarán": "acartonar",
    "acartonarás": "acartonar",
    "acartonaré": "acartonar",
    "acartonaréis": "acartonar",
    "acartonaría": "acartonar",
    "acartonaríais": "acartonar",
    "acartonaríamos": "acartonar",
    "acartonarían": "acartonar",
    "acartonarías": "acartonar",
    "acartonas": "acartonar",
    "acartonase": "acartonar",
    "acartonaseis": "acartonar",
    "acartonasen": "acartonar",
    "acartonases": "acartonar",
    "acartonaste": "acartonar",
    "acartonasteis": "acartonar",
    "acartone": "acartonar",
    "acartonemos": "acartonar",
    "acartonen": "acartonar",
    "acartones": "acartonar",
    "acartono": "acartonar",
    "acartonábamos": "acartonar",
    "acartonáis": "acartonar",
    "acartonáramos": "acartonar",
    "acartonáremos": "acartonar",
    "acartonásemos": "acartonar",
    "acartoné": "acartonar",
    "acartonéis": "acartonar",
    "acartonó": "acartonar",
    "acarábamos": "acarar",
    "acaráis": "acarar",
    "acaráramos": "acarar",
    "acaráremos": "acarar",
    "acarásemos": "acarar",
    "acaré": "acarar",
    "acaréis": "acarar",
    "acaró": "acarar",
    "acasos": "acaso",
    "acata": "acatar",
    "acataba": "acatar",
    "acatabais": "acatar",
    "acataban": "acatar",
    "acatabas": "acatar",
    "acatad": "acatar",
    "acatada": "acatar",
    "acatadas": "acatar",
    "acatado": "acatar",
    "acatadora": "acatador",
    "acatadoras": "acatador",
    "acatadores": "acatador",
    "acatados": "acatar",
    "acatalepsias": "acatalepsia",
    "acataléctica": "acataléctico",
    "acatalécticas": "acataléctico",
    "acatalécticos": "acataléctico",
    "acatamientos": "acatamiento",
    "acatamos": "acatar",
    "acatan": "acatar",
    "acatando": "acatar",
    "acatara": "acatar",
    "acatarais": "acatar",
    "acataran": "acatar",
    "acataras": "acatar",
    "acatare": "acatar",
    "acatareis": "acatar",
    "acataremos": "acatar",
    "acataren": "acatar",
    "acatares": "acatar",
    "acataron": "acatar",
    "acatarra": "acatarrar",
    "acatarraba": "acatarrar",
    "acatarrabais": "acatarrar",
    "acatarraban": "acatarrar",
    "acatarrabas": "acatarrar",
    "acatarrad": "acatarrar",
    "acatarrada": "acatarrar",
    "acatarradas": "acatarrar",
    "acatarrado": "acatarrar",
    "acatarrados": "acatarrar",
    "acatarramientos": "acatarramiento",
    "acatarramos": "acatarrar",
    "acatarran": "acatarrar",
    "acatarrando": "acatarrar",
    "acatarrara": "acatarrar",
    "acatarrarais": "acatarrar",
    "acatarraran": "acatarrar",
    "acatarraras": "acatarrar",
    "acatarrare": "acatarrar",
    "acatarrareis": "acatarrar",
    "acatarraremos": "acatarrar",
    "acatarraren": "acatarrar",
    "acatarrares": "acatarrar",
    "acatarraron": "acatarrar",
    "acatarrará": "acatarrar",
    "acatarrarán": "acatarrar",
    "acatarrarás": "acatarrar",
    "acatarraré": "acatarrar",
    "acatarraréis": "acatarrar",
    "acatarraría": "acatarrar",
    "acatarraríais": "acatarrar",
    "acatarraríamos": "acatarrar",
    "acatarrarían": "acatarrar",
    "acatarrarías": "acatarrar",
    "acatarras": "acatarrar",
    "acatarrase": "acatarrar",
    "acatarraseis": "acatarrar",
    "acatarrasen": "acatarrar",
    "acatarrases": "acatarrar",
    "acatarraste": "acatarrar",
    "acatarrasteis": "acatarrar",
    "acatarre": "acatarrar",
    "acatarremos": "acatarrar",
    "acatarren": "acatarrar",
    "acatarres": "acatarrar",
    "acatarro": "acatarrar",
    "acatarrábamos": "acatarrar",
    "acatarráis": "acatarrar",
    "acatarráramos": "acatarrar",
    "acatarráremos": "acatarrar",
    "acatarrásemos": "acatarrar",
    "acatarré": "acatarrar",
    "acatarréis": "acatarrar",
    "acatarró": "acatarrar",
    "acatará": "acatar",
    "acatarán": "acatar",
    "acatarás": "acatar",
    "acataré": "acatar",
    "acataréis": "acatar",
    "acataría": "acatar",
    "acataríais": "acatar",
    "acataríamos": "acatar",
    "acatarían": "acatar",
    "acatarías": "acatar",
    "acatas": "acatar",
    "acatase": "acatar",
    "acataseis": "acatar",
    "acatasen": "acatar",
    "acatases": "acatar",
    "acataste": "acatar",
    "acatasteis": "acatar",
    "acate": "acatar",
    "acatemos": "acatar",
    "acaten": "acatar",
    "acates": "acatar",
    "acato": "acatar",
    "acatos": "acato",
    "acatábamos": "acatar",
    "acatáis": "acatar",
    "acatáramos": "acatar",
    "acatáremos": "acatar",
    "acatásemos": "acatar",
    "acaté": "acatar",
    "acatéis": "acatar",
    "acató": "acatar",
    "acaudala": "acaudalar",
    "acaudalaba": "acaudalar",
    "acaudalabais": "acaudalar",
    "acaudalaban": "acaudalar",
    "acaudalabas": "acaudalar",
    "acaudalad": "acaudalar",
    "acaudalada": "acaudalar",
    "acaudaladas": "acaudalar",
    "acaudalado": "acaudalar",
    "acaudalados": "acaudalar",
    "acaudalamos": "acaudalar",
    "acaudalan": "acaudalar",
    "acaudalando": "acaudalar",
    "acaudalara": "acaudalar",
    "acaudalarais": "acaudalar",
    "acaudalaran": "acaudalar",
    "acaudalaras": "acaudalar",
    "acaudalare": "acaudalar",
    "acaudalareis": "acaudalar",
    "acaudalaremos": "acaudalar",
    "acaudalaren": "acaudalar",
    "acaudalares": "acaudalar",
    "acaudalaron": "acaudalar",
    "acaudalará": "acaudalar",
    "acaudalarán": "acaudalar",
    "acaudalarás": "acaudalar",
    "acaudalaré": "acaudalar",
    "acaudalaréis": "acaudalar",
    "acaudalaría": "acaudalar",
    "acaudalaríais": "acaudalar",
    "acaudalaríamos": "acaudalar",
    "acaudalarían": "acaudalar",
    "acaudalarías": "acaudalar",
    "acaudalas": "acaudalar",
    "acaudalase": "acaudalar",
    "acaudalaseis": "acaudalar",
    "acaudalasen": "acaudalar",
    "acaudalases": "acaudalar",
    "acaudalaste": "acaudalar",
    "acaudalasteis": "acaudalar",
    "acaudale": "acaudalar",
    "acaudalemos": "acaudalar",
    "acaudalen": "acaudalar",
    "acaudales": "acaudalar",
    "acaudalo": "acaudalar",
    "acaudalábamos": "acaudalar",
    "acaudaláis": "acaudalar",
    "acaudaláramos": "acaudalar",
    "acaudaláremos": "acaudalar",
    "acaudalásemos": "acaudalar",
    "acaudalé": "acaudalar",
    "acaudaléis": "acaudalar",
    "acaudaló": "acaudalar",
    "acaudilla": "acaudillar",
    "acaudillaba": "acaudillar",
    "acaudillabais": "acaudillar",
    "acaudillaban": "acaudillar",
    "acaudillabas": "acaudillar",
    "acaudillad": "acaudillar",
    "acaudillada": "acaudillar",
    "acaudilladas": "acaudillar",
    "acaudillado": "acaudillar",
    "acaudillados": "acaudillar",
    "acaudillamientos": "acaudillamiento",
    "acaudillamos": "acaudillar",
    "acaudillan": "acaudillar",
    "acaudillando": "acaudillar",
    "acaudillara": "acaudillar",
    "acaudillarais": "acaudillar",
    "acaudillaran": "acaudillar",
    "acaudillaras": "acaudillar",
    "acaudillare": "acaudillar",
    "acaudillareis": "acaudillar",
    "acaudillaremos": "acaudillar",
    "acaudillaren": "acaudillar",
    "acaudillares": "acaudillar",
    "acaudillaron": "acaudillar",
    "acaudillará": "acaudillar",
    "acaudillarán": "acaudillar",
    "acaudillarás": "acaudillar",
    "acaudillaré": "acaudillar",
    "acaudillaréis": "acaudillar",
    "acaudillaría": "acaudillar",
    "acaudillaríais": "acaudillar",
    "acaudillaríamos": "acaudillar",
    "acaudillarían": "acaudillar",
    "acaudillarías": "acaudillar",
    "acaudillas": "acaudillar",
    "acaudillase": "acaudillar",
    "acaudillaseis": "acaudillar",
    "acaudillasen": "acaudillar",
    "acaudillases": "acaudillar",
    "acaudillaste": "acaudillar",
    "acaudillasteis": "acaudillar",
    "acaudille": "acaudillar",
    "acaudillemos": "acaudillar",
    "acaudillen": "acaudillar",
    "acaudilles": "acaudillar",
    "acaudillo": "acaudillar",
    "acaudillábamos": "acaudillar",
    "acaudilláis": "acaudillar",
    "acaudilláramos": "acaudillar",
    "acaudilláremos": "acaudillar",
    "acaudillásemos": "acaudillar",
    "acaudillé": "acaudillar",
    "acaudilléis": "acaudillar",
    "acaudilló": "acaudillar",
    "acaules": "acaule",
    "acañonea": "acañonear",
    "acañoneaba": "acañonear",
    "acañoneabais": "acañonear",
    "acañoneaban": "acañonear",
    "acañoneabas": "acañonear",
    "acañonead": "acañonear",
    "acañoneada": "acañonear",
    "acañoneadas": "acañonear",
    "acañoneado": "acañonear",
    "acañoneados": "acañonear",
    "acañoneamos": "acañonear",
    "acañonean": "acañonear",
    "acañoneando": "acañonear",
    "acañoneara": "acañonear",
    "acañonearais": "acañonear",
    "acañonearan": "acañonear",
    "acañonearas": "acañonear",
    "acañoneare": "acañonear",
    "acañoneareis": "acañonear",
    "acañonearemos": "acañonear",
    "acañonearen": "acañonear",
    "acañoneares": "acañonear",
    "acañonearon": "acañonear",
    "acañoneará": "acañonear",
    "acañonearán": "acañonear",
    "acañonearás": "acañonear",
    "acañonearé": "acañonear",
    "acañonearéis": "acañonear",
    "acañonearía": "acañonear",
    "acañonearíais": "acañonear",
    "acañonearíamos": "acañonear",
    "acañonearían": "acañonear",
    "acañonearías": "acañonear",
    "acañoneas": "acañonear",
    "acañonease": "acañonear",
    "acañoneaseis": "acañonear",
    "acañoneasen": "acañonear",
    "acañoneases": "acañonear",
    "acañoneaste": "acañonear",
    "acañoneasteis": "acañonear",
    "acañonee": "acañonear",
    "acañoneemos": "acañonear",
    "acañoneen": "acañonear",
    "acañonees": "acañonear",
    "acañoneo": "acañonear",
    "acañoneábamos": "acañonear",
    "acañoneáis": "acañonear",
    "acañoneáramos": "acañonear",
    "acañoneáremos": "acañonear",
    "acañoneásemos": "acañonear",
    "acañoneé": "acañonear",
    "acañoneéis": "acañonear",
    "acañoneó": "acañonear",
    "acceda": "acceder",
    "accedamos": "acceder",
    "accedan": "acceder",
    "accedas": "acceder",
    "accede": "acceder",
    "acceded": "acceder",
    "accedemos": "acceder",
    "acceden": "acceder",
    "accederemos": "acceder",
    "accederá": "acceder",
    "accederán": "acceder",
    "accederás": "acceder",
    "accederé": "acceder",
    "accederéis": "acceder",
    "accedería": "acceder",
    "accederíais": "acceder",
    "accederíamos": "acceder",
    "accederían": "acceder",
    "accederías": "acceder",
    "accedes": "acceder",
    "accedida": "acceder",
    "accedidas": "acceder",
    "accedido": "acceder",
    "accedidos": "acceder",
    "accediendo": "acceder",
    "accediera": "acceder",
    "accedierais": "acceder",
    "accedieran": "acceder",
    "accedieras": "acceder",
    "accediere": "acceder",
    "accediereis": "acceder",
    "accedieren": "acceder",
    "accedieres": "acceder",
    "accedieron": "acceder",
    "accediese": "acceder",
    "accedieseis": "acceder",
    "accediesen": "acceder",
    "accedieses": "acceder",
    "accedimos": "acceder",
    "accediste": "acceder",
    "accedisteis": "acceder",
    "accediéramos": "acceder",
    "accediéremos": "acceder",
    "accediésemos": "acceder",
    "accedió": "acceder",
    "accedo": "acceder",
    "accedáis": "acceder",
    "accedéis": "acceder",
    "accedí": "acceder",
    "accedía": "acceder",
    "accedíais": "acceder",
    "accedíamos": "acceder",
    "accedían": "acceder",
    "accedías": "acceder",
    "accelerogramas": "accelerograma",
    "accesibilidades": "accesibilidad",
    "accesibles": "accesible",
    "accesionales": "accesional",
    "accesiones": "accesión",
    "accesoria": "accesorio",
    "accesorias": "accesorio",
    "accesoriedades": "accesoriedad",
    "accesorios": "accesorio",
    "accesos": "acceso",
    "accidenta": "accidentar",
    "accidentaba": "accidentar",
    "accidentabais": "accidentar",
    "accidentaban": "accidentar",
    "accidentabas": "accidentar",
    "accidentad": "accidentar",
    "accidentada": "accidentar",
    "accidentadas": "accidentar",
    "accidentado": "accidentar",
    "accidentados": "accidentar",
    "accidentales": "accidental",
    "accidentalidades": "accidentalidad",
    "accidentamos": "accidentar",
    "accidentan": "accidentar",
    "accidentando": "accidentar",
    "accidentara": "accidentar",
    "accidentarais": "accidentar",
    "accidentaran": "accidentar",
    "accidentaras": "accidentar",
    "accidentare": "accidentar",
    "accidentareis": "accidentar",
    "accidentaremos": "accidentar",
    "accidentaren": "accidentar",
    "accidentares": "accidentar",
    "accidentaron": "accidentar",
    "accidentará": "accidentar",
    "accidentarán": "accidentar",
    "accidentarás": "accidentar",
    "accidentaré": "accidentar",
    "accidentaréis": "accidentar",
    "accidentaría": "accidentar",
    "accidentaríais": "accidentar",
    "accidentaríamos": "accidentar",
    "accidentarían": "accidentar",
    "accidentarías": "accidentar",
    "accidentas": "accidentar",
    "accidentase": "accidentar",
    "accidentaseis": "accidentar",
    "accidentasen": "accidentar",
    "accidentases": "accidentar",
    "accidentaste": "accidentar",
    "accidentasteis": "accidentar",
    "accidente": "accidentar",
    "accidentemos": "accidentar",
    "accidenten": "accidentar",
    "accidentes": "accidente",
    "accidento": "accidentar",
    "accidentábamos": "accidentar",
    "accidentáis": "accidentar",
    "accidentáramos": "accidentar",
    "accidentáremos": "accidentar",
    "accidentásemos": "accidentar",
    "accidenté": "accidentar",
    "accidentéis": "accidentar",
    "accidentó": "accidentar",
    "acciona": "accionar",
    "accionaba": "accionar",
    "accionabais": "accionar",
    "accionaban": "accionar",
    "accionabas": "accionar",
    "accionables": "accionable",
    "accionad": "accionar",
    "accionada": "accionar",
    "accionadas": "accionar",
    "accionado": "accionar",
    "accionadores": "accionador",
    "accionados": "accionar",
    "accionamientos": "accionamiento",
    "accionamos": "accionar",
    "accionan": "accionar",
    "accionando": "accionar",
    "accionara": "accionar",
    "accionarais": "accionar",
    "accionaran": "accionar",
    "accionaras": "accionar",
    "accionare": "accionar",
    "accionareis": "accionar",
    "accionaremos": "accionar",
    "accionaren": "accionar",
    "accionares": "accionar",
    "accionaria": "accionario",
    "accionariados": "accionariado",
    "accionariales": "accionarial",
    "accionarias": "accionario",
    "accionarios": "accionario",
    "accionaron": "accionar",
    "accionará": "accionar",
    "accionarán": "accionar",
    "accionarás": "accionar",
    "accionaré": "accionar",
    "accionaréis": "accionar",
    "accionaría": "accionar",
    "accionaríais": "accionar",
    "accionaríamos": "accionar",
    "accionarían": "accionar",
    "accionarías": "accionar",
    "accionas": "accionar",
    "accionase": "accionar",
    "accionaseis": "accionar",
    "accionasen": "accionar",
    "accionases": "accionar",
    "accionaste": "accionar",
    "accionasteis": "accionar",
    "accione": "accionar",
    "accionemos": "accionar",
    "accionen": "accionar",
    "acciones": "accionar",
    "accionistas": "accionista",
    "acciono": "accionar",
    "accionábamos": "accionar",
    "accionáis": "accionar",
    "accionáramos": "accionar",
    "accionáremos": "accionar",
    "accionásemos": "accionar",
    "accioné": "accionar",
    "accionéis": "accionar",
    "accionó": "accionar",
    "accésits": "accésit",
    "accípitres": "accípitre",
    "acebales": "acebal",
    "acebedas": "acebeda",
    "acebedos": "acebedo",
    "acebollada": "acebollado",
    "acebolladas": "acebollado",
    "acebollados": "acebollado",
    "acebolladuras": "acebolladura",
    "acebos": "acebo",
    "acebrada": "acebrado",
    "acebradas": "acebrado",
    "acebrados": "acebrado",
    "acebuchales": "acebuchal",
    "acebuches": "acebuche",
    "acebuchinas": "acebuchina",
    "acebutololes": "acebutolol",
    "acece": "acezar",
    "acecemos": "acezar",
    "acecen": "acezar",
    "aceces": "acezar",
    "acecha": "acechar",
    "acechaba": "acechar",
    "acechabais": "acechar",
    "acechaban": "acechar",
    "acechabas": "acechar",
    "acechad": "acechar",
    "acechada": "acechar",
    "acechadas": "acechar",
    "acechaderas": "acechadera",
    "acechado": "acechar",
    "acechados": "acechar",
    "acechamos": "acechar",
    "acechan": "acechar",
    "acechando": "acechar",
    "acechara": "acechar",
    "acecharais": "acechar",
    "acecharan": "acechar",
    "acecharas": "acechar",
    "acechare": "acechar",
    "acechareis": "acechar",
    "acecharemos": "acechar",
    "acecharen": "acechar",
    "acechares": "acechar",
    "acecharon": "acechar",
    "acechará": "acechar",
    "acecharán": "acechar",
    "acecharás": "acechar",
    "acecharé": "acechar",
    "acecharéis": "acechar",
    "acecharía": "acechar",
    "acecharíais": "acechar",
    "acecharíamos": "acechar",
    "acecharían": "acechar",
    "acecharías": "acechar",
    "acechas": "acechar",
    "acechase": "acechar",
    "acechaseis": "acechar",
    "acechasen": "acechar",
    "acechases": "acechar",
    "acechaste": "acechar",
    "acechasteis": "acechar",
    "aceche": "acechar",
    "acechemos": "acechar",
    "acechen": "acechar",
    "aceches": "acechar",
    "acecho": "acechar",
    "acechos": "acecho",
    "acechábamos": "acechar",
    "acecháis": "acechar",
    "acecháramos": "acechar",
    "acecháremos": "acechar",
    "acechásemos": "acechar",
    "aceché": "acechar",
    "acechéis": "acechar",
    "acechó": "acechar",
    "acecina": "acecinar",
    "acecinaba": "acecinar",
    "acecinabais": "acecinar",
    "acecinaban": "acecinar",
    "acecinabas": "acecinar",
    "acecinad": "acecinar",
    "acecinada": "acecinar",
    "acecinadas": "acecinar",
    "acecinado": "acecinar",
    "acecinados": "acecinar",
    "acecinamos": "acecinar",
    "acecinan": "acecinar",
    "acecinando": "acecinar",
    "acecinara": "acecinar",
    "acecinarais": "acecinar",
    "acecinaran": "acecinar",
    "acecinaras": "acecinar",
    "acecinare": "acecinar",
    "acecinareis": "acecinar",
    "acecinaremos": "acecinar",
    "acecinaren": "acecinar",
    "acecinares": "acecinar",
    "acecinaron": "acecinar",
    "acecinará": "acecinar",
    "acecinarán": "acecinar",
    "acecinarás": "acecinar",
    "acecinaré": "acecinar",
    "acecinaréis": "acecinar",
    "acecinaría": "acecinar",
    "acecinaríais": "acecinar",
    "acecinaríamos": "acecinar",
    "acecinarían": "acecinar",
    "acecinarías": "acecinar",
    "acecinas": "acecinar",
    "acecinase": "acecinar",
    "acecinaseis": "acecinar",
    "acecinasen": "acecinar",
    "acecinases": "acecinar",
    "acecinaste": "acecinar",
    "acecinasteis": "acecinar",
    "acecine": "acecinar",
    "acecinemos": "acecinar",
    "acecinen": "acecinar",
    "acecines": "acecinar",
    "acecino": "acecinar",
    "acecinábamos": "acecinar",
    "acecináis": "acecinar",
    "acecináramos": "acecinar",
    "acecináremos": "acecinar",
    "acecinásemos": "acecinar",
    "aceciné": "acecinar",
    "acecinéis": "acecinar",
    "acecinó": "acecinar",
    "acecé": "acezar",
    "acecéis": "acezar",
    "aceda": "acedo",
    "acedaba": "acedar",
    "acedabais": "acedar",
    "acedaban": "acedar",
    "acedabas": "acedar",
    "acedad": "acedar",
    "acedada": "acedar",
    "acedadas": "acedar",
    "acedado": "acedar",
    "acedados": "acedar",
    "acedamos": "acedar",
    "acedan": "acedar",
    "acedando": "acedar",
    "acedara": "acedar",
    "acedarais": "acedar",
    "acedaran": "acedar",
    "acedaras": "acedar",
    "acedare": "acedar",
    "acedareis": "acedar",
    "acedaremos": "acedar",
    "acedaren": "acedar",
    "acedares": "acedar",
    "acedaron": "acedar",
    "acedará": "acedar",
    "acedarán": "acedar",
    "acedarás": "acedar",
    "acedaré": "acedar",
    "acedaréis": "acedar",
    "acedaría": "acedar",
    "acedaríais": "acedar",
    "acedaríamos": "acedar",
    "acedarían": "acedar",
    "acedarías": "acedar",
    "acedas": "acedo",
    "acedase": "acedar",
    "acedaseis": "acedar",
    "acedasen": "acedar",
    "acedases": "acedar",
    "acedaste": "acedar",
    "acedasteis": "acedar",
    "acede": "acedar",
    "acedemos": "acedar",
    "aceden": "acedar",
    "acederaques": "acederaque",
    "acederas": "acedera",
    "acederillas": "acederilla",
    "acederones": "acederón",
    "acedes": "acedar",
    "acedo": "acedar",
    "acedos": "acedo",
    "acedábamos": "acedar",
    "acedáis": "acedar",
    "acedáramos": "acedar",
    "acedáremos": "acedar",
    "acedásemos": "acedar",
    "acedé": "acedar",
    "acedéis": "acedar",
    "acedías": "acedía",
    "acedó": "acedar",
    "acefalismos": "acefalismo",
    "acefalías": "acefalía",
    "aceifas": "aceifa",
    "aceita": "aceitar",
    "aceitaba": "aceitar",
    "aceitabais": "aceitar",
    "aceitaban": "aceitar",
    "aceitabas": "aceitar",
    "aceitad": "aceitar",
    "aceitada": "aceitar",
    "aceitadas": "aceitar",
    "aceitado": "aceitar",
    "aceitados": "aceitar",
    "aceitamos": "aceitar",
    "aceitan": "aceitar",
    "aceitando": "aceitar",
    "aceitara": "aceitar",
    "aceitarais": "aceitar",
    "aceitaran": "aceitar",
    "aceitaras": "aceitar",
    "aceitare": "aceitar",
    "aceitareis": "aceitar",
    "aceitaremos": "aceitar",
    "aceitaren": "aceitar",
    "aceitares": "aceitar",
    "aceitaron": "aceitar",
    "aceitará": "aceitar",
    "aceitarán": "aceitar",
    "aceitarás": "aceitar",
    "aceitaré": "aceitar",
    "aceitaréis": "aceitar",
    "aceitaría": "aceitar",
    "aceitaríais": "aceitar",
    "aceitaríamos": "aceitar",
    "aceitarían": "aceitar",
    "aceitarías": "aceitar",
    "aceitas": "aceitar",
    "aceitase": "aceitar",
    "aceitaseis": "aceitar",
    "aceitasen": "aceitar",
    "aceitases": "aceitar",
    "aceitaste": "aceitar",
    "aceitasteis": "aceitar",
    "aceite": "aceitar",
    "aceitemos": "aceitar",
    "aceiten": "aceitar",
    "aceitera": "aceitero",
    "aceiteras": "aceitero",
    "aceiteros": "aceitero",
    "aceiterías": "aceitería",
    "aceites": "aceite",
    "aceitillos": "aceitillo",
    "aceito": "aceitar",
    "aceitones": "aceitón",
    "aceitosa": "aceitoso",
    "aceitosas": "aceitoso",
    "aceitosos": "aceitoso",
    "aceitunada": "aceitunado",
    "aceitunadas": "aceitunado",
    "aceitunados": "aceitunado",
    "aceitunas": "aceituna",
    "aceitunera": "aceitunero",
    "aceituneras": "aceitunero",
    "aceituneros": "aceitunero",
    "aceituniles": "aceitunil",
    "aceitunillos": "aceitunillo",
    "aceitunos": "aceituno",
    "aceitábamos": "aceitar",
    "aceitáis": "aceitar",
    "aceitáramos": "aceitar",
    "aceitáremos": "aceitar",
    "aceitásemos": "aceitar",
    "aceité": "aceitar",
    "aceitéis": "aceitar",
    "aceitó": "aceitar",
    "acelera": "acelerar",
    "aceleraba": "acelerar",
    "acelerabais": "acelerar",
    "aceleraban": "acelerar",
    "acelerabas": "acelerar",
    "aceleraciones": "aceleración",
    "acelerad": "acelerar",
    "acelerada": "acelerar",
    "aceleradas": "acelerar",
    "acelerado": "acelerar",
    "aceleradora": "acelerador",
    "aceleradoras": "acelerador",
    "aceleradores": "acelerador",
    "acelerados": "acelerar",
    "aceleramientos": "aceleramiento",
    "aceleramos": "acelerar",
    "aceleran": "acelerar",
    "acelerando": "acelerar",
    "acelerantes": "acelerante",
    "acelerara": "acelerar",
    "acelerarais": "acelerar",
    "aceleraran": "acelerar",
    "aceleraras": "acelerar",
    "acelerare": "acelerar",
    "acelerareis": "acelerar",
    "aceleraremos": "acelerar",
    "aceleraren": "acelerar",
    "acelerares": "acelerar",
    "aceleraron": "acelerar",
    "acelerará": "acelerar",
    "acelerarán": "acelerar",
    "acelerarás": "acelerar",
    "aceleraré": "acelerar",
    "aceleraréis": "acelerar",
    "aceleraría": "acelerar",
    "aceleraríais": "acelerar",
    "aceleraríamos": "acelerar",
    "acelerarían": "acelerar",
    "acelerarías": "acelerar",
    "aceleras": "acelerar",
    "acelerase": "acelerar",
    "aceleraseis": "acelerar",
    "acelerasen": "acelerar",
    "acelerases": "acelerar",
    "aceleraste": "acelerar",
    "acelerasteis": "acelerar",
    "acelere": "acelerar",
    "aceleremos": "acelerar",
    "aceleren": "acelerar",
    "aceleres": "acelerar",
    "acelerinas": "acelerina",
    "acelero": "acelerar",
    "acelerogramas": "acelerograma",
    "acelerones": "acelerón",
    "acelerábamos": "acelerar",
    "aceleráis": "acelerar",
    "aceleráramos": "acelerar",
    "aceleráremos": "acelerar",
    "acelerásemos": "acelerar",
    "aceleré": "acelerar",
    "aceleréis": "acelerar",
    "aceleró": "acelerar",
    "acelerómetros": "acelerómetro",
    "acelgas": "acelga",
    "acelulares": "acelular",
    "acemilera": "acemilero",
    "acemileras": "acemilero",
    "acemileros": "acemilero",
    "acemilerías": "acemilería",
    "acemitas": "acemita",
    "acemites": "acemite",
    "acenaftenoquinonas": "acenaftenoquinona",
    "acenaftenos": "acenafteno",
    "acendra": "acendrar",
    "acendraba": "acendrar",
    "acendrabais": "acendrar",
    "acendraban": "acendrar",
    "acendrabas": "acendrar",
    "acendrad": "acendrar",
    "acendrada": "acendrar",
    "acendradas": "acendrar",
    "acendrado": "acendrar",
    "acendrados": "acendrar",
    "acendramientos": "acendramiento",
    "acendramos": "acendrar",
    "acendran": "acendrar",
    "acendrando": "acendrar",
    "acendrara": "acendrar",
    "acendrarais": "acendrar",
    "acendraran": "acendrar",
    "acendraras": "acendrar",
    "acendrare": "acendrar",
    "acendrareis": "acendrar",
    "acendraremos": "acendrar",
    "acendraren": "acendrar",
    "acendrares": "acendrar",
    "acendraron": "acendrar",
    "acendrará": "acendrar",
    "acendrarán": "acendrar",
    "acendrarás": "acendrar",
    "acendraré": "acendrar",
    "acendraréis": "acendrar",
    "acendraría": "acendrar",
    "acendraríais": "acendrar",
    "acendraríamos": "acendrar",
    "acendrarían": "acendrar",
    "acendrarías": "acendrar",
    "acendras": "acendrar",
    "acendrase": "acendrar",
    "acendraseis": "acendrar",
    "acendrasen": "acendrar",
    "acendrases": "acendrar",
    "acendraste": "acendrar",
    "acendrasteis": "acendrar",
    "acendre": "acendrar",
    "acendremos": "acendrar",
    "acendren": "acendrar",
    "acendres": "acendrar",
    "acendro": "acendrar",
    "acendrábamos": "acendrar",
    "acendráis": "acendrar",
    "acendráramos": "acendrar",
    "acendráremos": "acendrar",
    "acendrásemos": "acendrar",
    "acendré": "acendrar",
    "acendréis": "acendrar",
    "acendró": "acendrar",
    "acensa": "acensar",
    "acensaba": "acensar",
    "acensabais": "acensar",
    "acensaban": "acensar",
    "acensabas": "acensar",
    "acensad": "acensar",
    "acensada": "acensar",
    "acensadas": "acensar",
    "acensado": "acensar",
    "acensadora": "acensador",
    "acensadoras": "acensador",
    "acensadores": "acensador",
    "acensados": "acensar",
    "acensamos": "acensar",
    "acensan": "acensar",
    "acensando": "acensar",
    "acensara": "acensar",
    "acensarais": "acensar",
    "acensaran": "acensar",
    "acensaras": "acensar",
    "acensare": "acensar",
    "acensareis": "acensar",
    "acensaremos": "acensar",
    "acensaren": "acensar",
    "acensares": "acensar",
    "acensaron": "acensar",
    "acensará": "acensar",
    "acensarán": "acensar",
    "acensarás": "acensar",
    "acensaré": "acensar",
    "acensaréis": "acensar",
    "acensaría": "acensar",
    "acensaríais": "acensar",
    "acensaríamos": "acensar",
    "acensarían": "acensar",
    "acensarías": "acensar",
    "acensas": "acensar",
    "acensase": "acensar",
    "acensaseis": "acensar",
    "acensasen": "acensar",
    "acensases": "acensar",
    "acensaste": "acensar",
    "acensasteis": "acensar",
    "acense": "acensar",
    "acensemos": "acensar",
    "acensen": "acensar",
    "acenses": "acensar",
    "acenso": "acensar",
    "acensuaba": "acensuar",
    "acensuabais": "acensuar",
    "acensuaban": "acensuar",
    "acensuabas": "acensuar",
    "acensuad": "acensuar",
    "acensuada": "acensuar",
    "acensuadas": "acensuar",
    "acensuado": "acensuar",
    "acensuados": "acensuar",
    "acensuamos": "acensuar",
    "acensuando": "acensuar",
    "acensuara": "acensuar",
    "acensuarais": "acensuar",
    "acensuaran": "acensuar",
    "acensuaras": "acensuar",
    "acensuare": "acensuar",
    "acensuareis": "acensuar",
    "acensuaremos": "acensuar",
    "acensuaren": "acensuar",
    "acensuares": "acensuar",
    "acensuaron": "acensuar",
    "acensuará": "acensuar",
    "acensuarán": "acensuar",
    "acensuarás": "acensuar",
    "acensuaré": "acensuar",
    "acensuaréis": "acensuar",
    "acensuaría": "acensuar",
    "acensuaríais": "acensuar",
    "acensuaríamos": "acensuar",
    "acensuarían": "acensuar",
    "acensuarías": "acensuar",
    "acensuase": "acensuar",
    "acensuaseis": "acensuar",
    "acensuasen": "acensuar",
    "acensuases": "acensuar",
    "acensuaste": "acensuar",
    "acensuasteis": "acensuar",
    "acensuemos": "acensuar",
    "acensuábamos": "acensuar",
    "acensuáis": "acensuar",
    "acensuáramos": "acensuar",
    "acensuáremos": "acensuar",
    "acensuásemos": "acensuar",
    "acensué": "acensuar",
    "acensuéis": "acensuar",
    "acensuó": "acensuar",
    "acensábamos": "acensar",
    "acensáis": "acensar",
    "acensáramos": "acensar",
    "acensáremos": "acensar",
    "acensásemos": "acensar",
    "acensé": "acensar",
    "acenséis": "acensar",
    "acensó": "acensar",
    "acensúa": "acensuar",
    "acensúan": "acensuar",
    "acensúas": "acensuar",
    "acensúe": "acensuar",
    "acensúen": "acensuar",
    "acensúes": "acensuar",
    "acensúo": "acensuar",
    "acentores": "acentor",
    "acentos": "acento",
    "acentuaba": "acentuar",
    "acentuabais": "acentuar",
    "acentuaban": "acentuar",
    "acentuabas": "acentuar",
    "acentuaciones": "acentuación",
    "acentuad": "acentuar",
    "acentuada": "acentuar",
    "acentuadas": "acentuar",
    "acentuado": "acentuar",
    "acentuados": "acentuar",
    "acentuales": "acentual",
    "acentuamos": "acentuar",
    "acentuando": "acentuar",
    "acentuara": "acentuar",
    "acentuarais": "acentuar",
    "acentuaran": "acentuar",
    "acentuaras": "acentuar",
    "acentuare": "acentuar",
    "acentuareis": "acentuar",
    "acentuaremos": "acentuar",
    "acentuaren": "acentuar",
    "acentuares": "acentuar",
    "acentuaron": "acentuar",
    "acentuará": "acentuar",
    "acentuarán": "acentuar",
    "acentuarás": "acentuar",
    "acentuaré": "acentuar",
    "acentuaréis": "acentuar",
    "acentuaría": "acentuar",
    "acentuaríais": "acentuar",
    "acentuaríamos": "acentuar",
    "acentuarían": "acentuar",
    "acentuarías": "acentuar",
    "acentuase": "acentuar",
    "acentuaseis": "acentuar",
    "acentuasen": "acentuar",
    "acentuases": "acentuar",
    "acentuaste": "acentuar",
    "acentuasteis": "acentuar",
    "acentuemos": "acentuar",
    "acentuábamos": "acentuar",
    "acentuáis": "acentuar",
    "acentuáramos": "acentuar",
    "acentuáremos": "acentuar",
    "acentuásemos": "acentuar",
    "acentué": "acentuar",
    "acentuéis": "acentuar",
    "acentuó": "acentuar",
    "acentúa": "acentuar",
    "acentúan": "acentuar",
    "acentúas": "acentuar",
    "acentúe": "acentuar",
    "acentúen": "acentuar",
    "acentúes": "acentuar",
    "acentúo": "acentuar",
    "acepa": "acepar",
    "acepaba": "acepar",
    "acepabais": "acepar",
    "acepaban": "acepar",
    "acepabas": "acepar",
    "acepad": "acepar",
    "acepada": "acepar",
    "acepadas": "acepar",
    "acepado": "acepar",
    "acepados": "acepar",
    "acepamos": "acepar",
    "acepan": "acepar",
    "acepando": "acepar",
    "acepara": "acepar",
    "aceparais": "acepar",
    "aceparan": "acepar",
    "aceparas": "acepar",
    "acepare": "acepar",
    "acepareis": "acepar",
    "aceparemos": "acepar",
    "aceparen": "acepar",
    "acepares": "acepar",
    "aceparon": "acepar",
    "acepará": "acepar",
    "aceparán": "acepar",
    "aceparás": "acepar",
    "aceparé": "acepar",
    "aceparéis": "acepar",
    "aceparía": "acepar",
    "aceparíais": "acepar",
    "aceparíamos": "acepar",
    "aceparían": "acepar",
    "aceparías": "acepar",
    "acepas": "acepar",
    "acepase": "acepar",
    "acepaseis": "acepar",
    "acepasen": "acepar",
    "acepases": "acepar",
    "acepaste": "acepar",
    "acepasteis": "acepar",
    "acepciones": "acepción",
    "acepe": "acepar",
    "acepemos": "acepar",
    "acepen": "acepar",
    "acepes": "acepar",
    "acepilla": "acepillar",
    "acepillaba": "acepillar",
    "acepillabais": "acepillar",
    "acepillaban": "acepillar",
    "acepillabas": "acepillar",
    "acepillad": "acepillar",
    "acepillada": "acepillar",
    "acepilladas": "acepillar",
    "acepillado": "acepillar",
    "acepilladoras": "acepilladora",
    "acepillados": "acepillar",
    "acepilladuras": "acepilladura",
    "acepillamos": "acepillar",
    "acepillan": "acepillar",
    "acepillando": "acepillar",
    "acepillara": "acepillar",
    "acepillarais": "acepillar",
    "acepillaran": "acepillar",
    "acepillaras": "acepillar",
    "acepillare": "acepillar",
    "acepillareis": "acepillar",
    "acepillaremos": "acepillar",
    "acepillaren": "acepillar",
    "acepillares": "acepillar",
    "acepillaron": "acepillar",
    "acepillará": "acepillar",
    "acepillarán": "acepillar",
    "acepillarás": "acepillar",
    "acepillaré": "acepillar",
    "acepillaréis": "acepillar",
    "acepillaría": "acepillar",
    "acepillaríais": "acepillar",
    "acepillaríamos": "acepillar",
    "acepillarían": "acepillar",
    "acepillarías": "acepillar",
    "acepillas": "acepillar",
    "acepillase": "acepillar",
    "acepillaseis": "acepillar",
    "acepillasen": "acepillar",
    "acepillases": "acepillar",
    "acepillaste": "acepillar",
    "acepillasteis": "acepillar",
    "acepille": "acepillar",
    "acepillemos": "acepillar",
    "acepillen": "acepillar",
    "acepilles": "acepillar",
    "acepillo": "acepillar",
    "acepillábamos": "acepillar",
    "acepilláis": "acepillar",
    "acepilláramos": "acepillar",
    "acepilláremos": "acepillar",
    "acepillásemos": "acepillar",
    "acepillé": "acepillar",
    "acepilléis": "acepillar",
    "acepilló": "acepillar",
    "acepo": "acepar",
    "acepta": "acepto",
    "aceptaba": "aceptar",
    "aceptabais": "aceptar",
    "aceptaban": "aceptar",
    "aceptabas": "aceptar",
    "aceptabilidades": "aceptabilidad",
    "aceptables": "aceptable",
    "aceptaciones": "aceptación",
    "aceptad": "aceptar",
    "aceptada": "aceptar",
    "aceptadas": "aceptar",
    "aceptado": "aceptar",
    "aceptadora": "aceptador",
    "aceptadoras": "aceptador",
    "aceptadores": "aceptador",
    "aceptados": "aceptar",
    "aceptamos": "aceptar",
    "aceptan": "aceptar",
    "aceptando": "aceptar",
    "aceptantes": "aceptante",
    "aceptara": "aceptar",
    "aceptarais": "aceptar",
    "aceptaran": "aceptar",
    "aceptaras": "aceptar",
    "aceptare": "aceptar",
    "aceptareis": "aceptar",
    "aceptaremos": "aceptar",
    "aceptaren": "aceptar",
    "aceptares": "aceptar",
    "aceptaron": "aceptar",
    "aceptará": "aceptar",
    "aceptarán": "aceptar",
    "aceptarás": "aceptar",
    "aceptaré": "aceptar",
    "aceptaréis": "aceptar",
    "aceptaría": "aceptar",
    "aceptaríais": "aceptar",
    "aceptaríamos": "aceptar",
    "aceptarían": "aceptar",
    "aceptarías": "aceptar",
    "aceptas": "acepto",
    "aceptase": "aceptar",
    "aceptaseis": "aceptar",
    "aceptasen": "aceptar",
    "aceptases": "aceptar",
    "aceptaste": "aceptar",
    "aceptasteis": "aceptar",
    "acepte": "aceptar",
    "aceptemos": "aceptar",
    "acepten": "aceptar",
    "aceptes": "aceptar",
    "acepto": "aceptar",
    "aceptora": "aceptor",
    "aceptoras": "aceptor",
    "aceptores": "aceptor",
    "aceptos": "acepto",
    "aceptábamos": "aceptar",
    "aceptáis": "aceptar",
    "aceptáramos": "aceptar",
    "aceptáremos": "aceptar",
    "aceptásemos": "aceptar",
    "acepté": "aceptar",
    "aceptéis": "aceptar",
    "aceptó": "aceptar",
    "acepábamos": "acepar",
    "acepáis": "acepar",
    "acepáramos": "acepar",
    "acepáremos": "acepar",
    "acepásemos": "acepar",
    "acepé": "acepar",
    "acepéis": "acepar",
    "acepó": "acepar",
    "acequia": "acequiar",
    "acequiaba": "acequiar",
    "acequiabais": "acequiar",
    "acequiaban": "acequiar",
    "acequiabas": "acequiar",
    "acequiad": "acequiar",
    "acequiada": "acequiar",
    "acequiadas": "acequiar",
    "acequiado": "acequiar",
    "acequiados": "acequiar",
    "acequiamos": "acequiar",
    "acequian": "acequiar",
    "acequiando": "acequiar",
    "acequiara": "acequiar",
    "acequiarais": "acequiar",
    "acequiaran": "acequiar",
    "acequiaras": "acequiar",
    "acequiare": "acequiar",
    "acequiareis": "acequiar",
    "acequiaremos": "acequiar",
    "acequiaren": "acequiar",
    "acequiares": "acequiar",
    "acequiaron": "acequiar",
    "acequiará": "acequiar",
    "acequiarán": "acequiar",
    "acequiarás": "acequiar",
    "acequiaré": "acequiar",
    "acequiaréis": "acequiar",
    "acequiaría": "acequiar",
    "acequiaríais": "acequiar",
    "acequiaríamos": "acequiar",
    "acequiarían": "acequiar",
    "acequiarías": "acequiar",
    "acequias": "acequiar",
    "acequiase": "acequiar",
    "acequiaseis": "acequiar",
    "acequiasen": "acequiar",
    "acequiases": "acequiar",
    "acequiaste": "acequiar",
    "acequiasteis": "acequiar",
    "acequie": "acequiar",
    "acequiemos": "acequiar",
    "acequien": "acequiar",
    "acequieros": "acequiero",
    "acequies": "acequiar",
    "acequio": "acequiar",
    "acequiábamos": "acequiar",
    "acequiáis": "acequiar",
    "acequiáramos": "acequiar",
    "acequiáremos": "acequiar",
    "acequiásemos": "acequiar",
    "acequié": "acequiar",
    "acequiéis": "acequiar",
    "acequió": "acequiar",
    "acera": "acerar",
    "aceraba": "acerar",
    "acerabais": "acerar",
    "aceraban": "acerar",
    "acerabas": "acerar",
    "aceraciones": "aceración",
    "acerad": "acerar",
    "acerada": "acerar",
    "aceradas": "acerar",
    "acerado": "acerar",
    "acerados": "acerar",
    "aceramos": "acerar",
    "aceran": "acerar",
    "acerando": "acerar",
    "acerara": "acerar",
    "acerarais": "acerar",
    "aceraran": "acerar",
    "aceraras": "acerar",
    "acerare": "acerar",
    "acerareis": "acerar",
    "aceraremos": "acerar",
    "aceraren": "acerar",
    "acerares": "acerar",
    "aceraron": "acerar",
    "acerará": "acerar",
    "acerarán": "acerar",
    "acerarás": "acerar",
    "aceraré": "acerar",
    "aceraréis": "acerar",
    "aceraría": "acerar",
    "aceraríais": "acerar",
    "aceraríamos": "acerar",
    "acerarían": "acerar",
    "acerarías": "acerar",
    "aceras": "acerar",
    "acerase": "acerar",
    "aceraseis": "acerar",
    "acerasen": "acerar",
    "acerases": "acerar",
    "aceraste": "acerar",
    "acerasteis": "acerar",
    "acerba": "acerbo",
    "acerbas": "acerbo",
    "acerbidades": "acerbidad",
    "acerbos": "acerbo",
    "acerca": "acercar",
    "acercaba": "acercar",
    "acercabais": "acercar",
    "acercaban": "acercar",
    "acercabas": "acercar",
    "acercad": "acercar",
    "acercada": "acercar",
    "acercadas": "acercar",
    "acercado": "acercar",
    "acercados": "acercar",
    "acercamientos": "acercamiento",
    "acercamos": "acercar",
    "acercan": "acercar",
    "acercando": "acercar",
    "acercara": "acercar",
    "acercarais": "acercar",
    "acercaran": "acercar",
    "acercaras": "acercar",
    "acercare": "acercar",
    "acercareis": "acercar",
    "acercaremos": "acercar",
    "acercaren": "acercar",
    "acercares": "acercar",
    "acercaron": "acercar",
    "acercará": "acercar",
    "acercarán": "acercar",
    "acercarás": "acercar",
    "acercaré": "acercar",
    "acercaréis": "acercar",
    "acercaría": "acercar",
    "acercaríais": "acercar",
    "acercaríamos": "acercar",
    "acercarían": "acercar",
    "acercarías": "acercar",
    "acercas": "acercar",
    "acercase": "acercar",
    "acercaseis": "acercar",
    "acercasen": "acercar",
    "acercases": "acercar",
    "acercaste": "acercar",
    "acercasteis": "acercar",
    "acerco": "acercar",
    "acercábamos": "acercar",
    "acercáis": "acercar",
    "acercáramos": "acercar",
    "acercáremos": "acercar",
    "acercásemos": "acercar",
    "acercó": "acercar",
    "acere": "acerar",
    "aceremos": "acerar",
    "aceren": "acerar",
    "acererías": "acerería",
    "aceres": "acerar",
    "acericos": "acerico",
    "acerillos": "acerillo",
    "acero": "acerar",
    "acerolas": "acerola",
    "acerolos": "acerolo",
    "aceros": "acero",
    "acerque": "acercar",
    "acerquemos": "acercar",
    "acerquen": "acercar",
    "acerques": "acercar",
    "acerqué": "acercar",
    "acerquéis": "acercar",
    "acerroja": "acerrojar",
    "acerrojaba": "acerrojar",
    "acerrojabais": "acerrojar",
    "acerrojaban": "acerrojar",
    "acerrojabas": "acerrojar",
    "acerrojad": "acerrojar",
    "acerrojada": "acerrojar",
    "acerrojadas": "acerrojar",
    "acerrojado": "acerrojar",
    "acerrojados": "acerrojar",
    "acerrojamos": "acerrojar",
    "acerrojan": "acerrojar",
    "acerrojando": "acerrojar",
    "acerrojara": "acerrojar",
    "acerrojarais": "acerrojar",
    "acerrojaran": "acerrojar",
    "acerrojaras": "acerrojar",
    "acerrojare": "acerrojar",
    "acerrojareis": "acerrojar",
    "acerrojaremos": "acerrojar",
    "acerrojaren": "acerrojar",
    "acerrojares": "acerrojar",
    "acerrojaron": "acerrojar",
    "acerrojará": "acerrojar",
    "acerrojarán": "acerrojar",
    "acerrojarás": "acerrojar",
    "acerrojaré": "acerrojar",
    "acerrojaréis": "acerrojar",
    "acerrojaría": "acerrojar",
    "acerrojaríais": "acerrojar",
    "acerrojaríamos": "acerrojar",
    "acerrojarían": "acerrojar",
    "acerrojarías": "acerrojar",
    "acerrojas": "acerrojar",
    "acerrojase": "acerrojar",
    "acerrojaseis": "acerrojar",
    "acerrojasen": "acerrojar",
    "acerrojases": "acerrojar",
    "acerrojaste": "acerrojar",
    "acerrojasteis": "acerrojar",
    "acerroje": "acerrojar",
    "acerrojemos": "acerrojar",
    "acerrojen": "acerrojar",
    "acerrojes": "acerrojar",
    "acerrojo": "acerrojar",
    "acerrojábamos": "acerrojar",
    "acerrojáis": "acerrojar",
    "acerrojáramos": "acerrojar",
    "acerrojáremos": "acerrojar",
    "acerrojásemos": "acerrojar",
    "acerrojé": "acerrojar",
    "acerrojéis": "acerrojar",
    "acerrojó": "acerrojar",
    "acertaba": "acertar",
    "acertabais": "acertar",
    "acertaban": "acertar",
    "acertabas": "acertar",
    "acertad": "acertar",
    "acertada": "acertar",
    "acertadas": "acertar",
    "acertado": "acertar",
    "acertados": "acertar",
    "acertamos": "acertar",
    "acertando": "acertar",
    "acertantes": "acertante",
    "acertara": "acertar",
    "acertarais": "acertar",
    "acertaran": "acertar",
    "acertaras": "acertar",
    "acertare": "acertar",
    "acertareis": "acertar",
    "acertaremos": "acertar",
    "acertaren": "acertar",
    "acertares": "acertar",
    "acertaron": "acertar",
    "acertará": "acertar",
    "acertarán": "acertar",
    "acertarás": "acertar",
    "acertaré": "acertar",
    "acertaréis": "acertar",
    "acertaría": "acertar",
    "acertaríais": "acertar",
    "acertaríamos": "acertar",
    "acertarían": "acertar",
    "acertarías": "acertar",
    "acertase": "acertar",
    "acertaseis": "acertar",
    "acertasen": "acertar",
    "acertases": "acertar",
    "acertaste": "acertar",
    "acertasteis": "acertar",
    "acertemos": "acertar",
    "acertijos": "acertijo",
    "acertábamos": "acertar",
    "acertáis": "acertar",
    "acertáramos": "acertar",
    "acertáremos": "acertar",
    "acertásemos": "acertar",
    "acerté": "acertar",
    "acertéis": "acertar",
    "acertó": "acertar",
    "aceruelos": "aceruelo",
    "acervos": "acervo",
    "acerábamos": "acerar",
    "aceráceas": "acerácea",
    "aceráis": "acerar",
    "aceráramos": "acerar",
    "aceráremos": "acerar",
    "acerásemos": "acerar",
    "aceré": "acerar",
    "aceréis": "acerar",
    "acerías": "acería",
    "aceró": "acerar",
    "aces": "ace",
    "acescencias": "acescencia",
    "acescentes": "acescente",
    "acesivas": "acesiva",
    "acetabulares": "acetabular",
    "acetaldehidasas": "acetaldehidasa",
    "acetaldehídos": "acetaldehído",
    "acetales": "acetal",
    "acetalizaciones": "acetalización",
    "acetamidas": "acetamida",
    "acetamidinas": "acetamidina",
    "acetanilidas": "acetanilida",
    "acetatopropionatos": "acetatopropionato",
    "acetatos": "acetato",
    "acetifica": "acetificar",
    "acetificaba": "acetificar",
    "acetificabais": "acetificar",
    "acetificaban": "acetificar",
    "acetificabas": "acetificar",
    "acetificaciones": "acetificación",
    "acetificad": "acetificar",
    "acetificada": "acetificar",
    "acetificadas": "acetificar",
    "acetificado": "acetificar",
    "acetificados": "acetificar",
    "acetificamos": "acetificar",
    "acetifican": "acetificar",
    "acetificando": "acetificar",
    "acetificantes": "acetificante",
    "acetificara": "acetificar",
    "acetificarais": "acetificar",
    "acetificaran": "acetificar",
    "acetificaras": "acetificar",
    "acetificare": "acetificar",
    "acetificareis": "acetificar",
    "acetificaremos": "acetificar",
    "acetificaren": "acetificar",
    "acetificares": "acetificar",
    "acetificaron": "acetificar",
    "acetificará": "acetificar",
    "acetificarán": "acetificar",
    "acetificarás": "acetificar",
    "acetificaré": "acetificar",
    "acetificaréis": "acetificar",
    "acetificaría": "acetificar",
    "acetificaríais": "acetificar",
    "acetificaríamos": "acetificar",
    "acetificarían": "acetificar",
    "acetificarías": "acetificar",
    "acetificas": "acetificar",
    "acetificase": "acetificar",
    "acetificaseis": "acetificar",
    "acetificasen": "acetificar",
    "acetificases": "acetificar",
    "acetificaste": "acetificar",
    "acetificasteis": "acetificar",
    "acetifico": "acetificar",
    "acetificábamos": "acetificar",
    "acetificáis": "acetificar",
    "acetificáramos": "acetificar",
    "acetificáremos": "acetificar",
    "acetificásemos": "acetificar",
    "acetificó": "acetificar",
    "acetifique": "acetificar",
    "acetifiquemos": "acetificar",
    "acetifiquen": "acetificar",
    "acetifiques": "acetificar",
    "acetifiqué": "acetificar",
    "acetifiquéis": "acetificar",
    "acetilacetonas": "acetilacetona",
    "acetilacetonatos": "acetilacetonato",
    "acetilaciones": "acetilación",
    "acetiladores": "acetilador",
    "acetilarsanilatos": "acetilarsanilato",
    "acetilcelulosas": "acetilcelulosa",
    "acetilcolinas": "acetilcolina",
    "acetilcolinesterasas": "acetilcolinesterasa",
    "acetilenos": "acetileno",
    "acetilfenilhidracinas": "acetilfenilhidracina",
    "acetilisoeugenoles": "acetilisoeugenol",
    "acetilsalicilatos": "acetilsalicilato",
    "acetilsalicílica": "acetilsalicílico",
    "acetilsalicílicas": "acetilsalicílico",
    "acetilsalicílicos": "acetilsalicílico",
    "acetiluros": "acetiluro",
    "acetilénica": "acetilénico",
    "acetilénicas": "acetilénico",
    "acetilénicos": "acetilénico",
    "acetinas": "acetina",
    "acetoacetanilidas": "acetoacetanilida",
    "acetoacetatos": "acetoacetato",
    "acetoacética": "acetoacético",
    "acetoacéticas": "acetoacético",
    "acetoacéticos": "acetoacético",
    "acetoarsenitos": "acetoarsenito",
    "acetobacterias": "acetobacteria",
    "acetobenzoatos": "acetobenzoato",
    "acetobromales": "acetobromal",
    "acetobromuros": "acetobromuro",
    "acetobutiratos": "acetobutirato",
    "acetofenonas": "acetofenona",
    "acetolos": "acetolo",
    "acetonaftonas": "acetonaftona",
    "acetonas": "acetona",
    "acetonemias": "acetonemia",
    "acetonurias": "acetonuria",
    "acetosa": "acetoso",
    "acetosas": "acetoso",
    "acetosidades": "acetosidad",
    "acetosos": "acetoso",
    "acetoxiestearatos": "acetoxiestearato",
    "acetoxilaciones": "acetoxilación",
    "acetoxilos": "acetoxilo",
    "acetoximas": "acetoxima",
    "acetoínas": "acetoína",
    "acetres": "acetre",
    "acetrina": "acetrinar",
    "acetrinaba": "acetrinar",
    "acetrinabais": "acetrinar",
    "acetrinaban": "acetrinar",
    "acetrinabas": "acetrinar",
    "acetrinad": "acetrinar",
    "acetrinada": "acetrinar",
    "acetrinadas": "acetrinar",
    "acetrinado": "acetrinar",
    "acetrinados": "acetrinar",
    "acetrinamos": "acetrinar",
    "acetrinan": "acetrinar",
    "acetrinando": "acetrinar",
    "acetrinara": "acetrinar",
    "acetrinarais": "acetrinar",
    "acetrinaran": "acetrinar",
    "acetrinaras": "acetrinar",
    "acetrinare": "acetrinar",
    "acetrinareis": "acetrinar",
    "acetrinaremos": "acetrinar",
    "acetrinaren": "acetrinar",
    "acetrinares": "acetrinar",
    "acetrinaron": "acetrinar",
    "acetrinará": "acetrinar",
    "acetrinarán": "acetrinar",
    "acetrinarás": "acetrinar",
    "acetrinaré": "acetrinar",
    "acetrinaréis": "acetrinar",
    "acetrinaría": "acetrinar",
    "acetrinaríais": "acetrinar",
    "acetrinaríamos": "acetrinar",
    "acetrinarían": "acetrinar",
    "acetrinarías": "acetrinar",
    "acetrinas": "acetrinar",
    "acetrinase": "acetrinar",
    "acetrinaseis": "acetrinar",
    "acetrinasen": "acetrinar",
    "acetrinases": "acetrinar",
    "acetrinaste": "acetrinar",
    "acetrinasteis": "acetrinar",
    "acetrine": "acetrinar",
    "acetrinemos": "acetrinar",
    "acetrinen": "acetrinar",
    "acetrines": "acetrinar",
    "acetrino": "acetrinar",
    "acetrinábamos": "acetrinar",
    "acetrináis": "acetrinar",
    "acetrináramos": "acetrinar",
    "acetrináremos": "acetrinar",
    "acetrinásemos": "acetrinar",
    "acetriné": "acetrinar",
    "acetrinéis": "acetrinar",
    "acetrinó": "acetrinar",
    "acetrizoatos": "acetrizoato",
    "acetábulos": "acetábulo",
    "acetímetros": "acetímetro",
    "acetómetros": "acetómetro",
    "acetónica": "acetónico",
    "acetónicas": "acetónico",
    "acetónicos": "acetónico",
    "aceza": "acezar",
    "acezaba": "acezar",
    "acezabais": "acezar",
    "acezaban": "acezar",
    "acezabas": "acezar",
    "acezad": "acezar",
    "acezada": "acezar",
    "acezadas": "acezar",
    "acezado": "acezar",
    "acezados": "acezar",
    "acezamos": "acezar",
    "acezan": "acezar",
    "acezando": "acezar",
    "acezara": "acezar",
    "acezarais": "acezar",
    "acezaran": "acezar",
    "acezaras": "acezar",
    "acezare": "acezar",
    "acezareis": "acezar",
    "acezaremos": "acezar",
    "acezaren": "acezar",
    "acezares": "acezar",
    "acezaron": "acezar",
    "acezará": "acezar",
    "acezarán": "acezar",
    "acezarás": "acezar",
    "acezaré": "acezar",
    "acezaréis": "acezar",
    "acezaría": "acezar",
    "acezaríais": "acezar",
    "acezaríamos": "acezar",
    "acezarían": "acezar",
    "acezarías": "acezar",
    "acezas": "acezar",
    "acezase": "acezar",
    "acezaseis": "acezar",
    "acezasen": "acezar",
    "acezases": "acezar",
    "acezaste": "acezar",
    "acezasteis": "acezar",
    "acezo": "acezar",
    "acezos": "acezo",
    "acezosa": "acezoso",
    "acezosas": "acezoso",
    "acezosos": "acezoso",
    "acezábamos": "acezar",
    "acezáis": "acezar",
    "acezáramos": "acezar",
    "acezáremos": "acezar",
    "acezásemos": "acezar",
    "acezó": "acezar",
    "aceñas": "aceña",
    "aceñera": "aceñero",
    "aceñeras": "aceñero",
    "aceñeros": "aceñero",
    "achabacana": "achabacanar",
    "achabacanaba": "achabacanar",
    "achabacanabais": "achabacanar",
    "achabacanaban": "achabacanar",
    "achabacanabas": "achabacanar",
    "achabacanad": "achabacanar",
    "achabacanada": "achabacanar",
    "achabacanadas": "achabacanar",
    "achabacanado": "achabacanar",
    "achabacanados": "achabacanar",
    "achabacanamientos": "achabacanamiento",
    "achabacanamos": "achabacanar",
    "achabacanan": "achabacanar",
    "achabacanando": "achabacanar",
    "achabacanara": "achabacanar",
    "achabacanarais": "achabacanar",
    "achabacanaran": "achabacanar",
    "achabacanaras": "achabacanar",
    "achabacanare": "achabacanar",
    "achabacanareis": "achabacanar",
    "achabacanaremos": "achabacanar",
    "achabacanaren": "achabacanar",
    "achabacanares": "achabacanar",
    "achabacanaron": "achabacanar",
    "achabacanará": "achabacanar",
    "achabacanarán": "achabacanar",
    "achabacanarás": "achabacanar",
    "achabacanaré": "achabacanar",
    "achabacanaréis": "achabacanar",
    "achabacanaría": "achabacanar",
    "achabacanaríais": "achabacanar",
    "achabacanaríamos": "achabacanar",
    "achabacanarían": "achabacanar",
    "achabacanarías": "achabacanar",
    "achabacanas": "achabacanar",
    "achabacanase": "achabacanar",
    "achabacanaseis": "achabacanar",
    "achabacanasen": "achabacanar",
    "achabacanases": "achabacanar",
    "achabacanaste": "achabacanar",
    "achabacanasteis": "achabacanar",
    "achabacane": "achabacanar",
    "achabacanemos": "achabacanar",
    "achabacanen": "achabacanar",
    "achabacanes": "achabacanar",
    "achabacano": "achabacanar",
    "achabacanábamos": "achabacanar",
    "achabacanáis": "achabacanar",
    "achabacanáramos": "achabacanar",
    "achabacanáremos": "achabacanar",
    "achabacanásemos": "achabacanar",
    "achabacané": "achabacanar",
    "achabacanéis": "achabacanar",
    "achabacanó": "achabacanar",
    "achaca": "achacar",
    "achacaba": "achacar",
    "achacabais": "achacar",
    "achacaban": "achacar",
    "achacabas": "achacar",
    "achacables": "achacable",
    "achacad": "achacar",
    "achacada": "achacar",
    "achacadas": "achacar",
    "achacado": "achacar",
    "achacados": "achacar",
    "achacamos": "achacar",
    "achacan": "achacar",
    "achacando": "achacar",
    "achacara": "achacar",
    "achacarais": "achacar",
    "achacaran": "achacar",
    "achacaras": "achacar",
    "achacare": "achacar",
    "achacareis": "achacar",
    "achacaremos": "achacar",
    "achacaren": "achacar",
    "achacares": "achacar",
    "achacaron": "achacar",
    "achacará": "achacar",
    "achacarán": "achacar",
    "achacarás": "achacar",
    "achacaré": "achacar",
    "achacaréis": "achacar",
    "achacaría": "achacar",
    "achacaríais": "achacar",
    "achacaríamos": "achacar",
    "achacarían": "achacar",
    "achacarías": "achacar",
    "achacas": "achacar",
    "achacase": "achacar",
    "achacaseis": "achacar",
    "achacasen": "achacar",
    "achacases": "achacar",
    "achacaste": "achacar",
    "achacasteis": "achacar",
    "achaco": "achacar",
    "achacosa": "achacoso",
    "achacosas": "achacoso",
    "achacosos": "achacoso",
    "achacábamos": "achacar",
    "achacáis": "achacar",
    "achacáramos": "achacar",
    "achacáremos": "achacar",
    "achacásemos": "achacar",
    "achacó": "achacar",
    "achaflana": "achaflanar",
    "achaflanaba": "achaflanar",
    "achaflanabais": "achaflanar",
    "achaflanaban": "achaflanar",
    "achaflanabas": "achaflanar",
    "achaflanad": "achaflanar",
    "achaflanada": "achaflanar",
    "achaflanadas": "achaflanar",
    "achaflanado": "achaflanar",
    "achaflanados": "achaflanar",
    "achaflanamos": "achaflanar",
    "achaflanan": "achaflanar",
    "achaflanando": "achaflanar",
    "achaflanara": "achaflanar",
    "achaflanarais": "achaflanar",
    "achaflanaran": "achaflanar",
    "achaflanaras": "achaflanar",
    "achaflanare": "achaflanar",
    "achaflanareis": "achaflanar",
    "achaflanaremos": "achaflanar",
    "achaflanaren": "achaflanar",
    "achaflanares": "achaflanar",
    "achaflanaron": "achaflanar",
    "achaflanará": "achaflanar",
    "achaflanarán": "achaflanar",
    "achaflanarás": "achaflanar",
    "achaflanaré": "achaflanar",
    "achaflanaréis": "achaflanar",
    "achaflanaría": "achaflanar",
    "achaflanaríais": "achaflanar",
    "achaflanaríamos": "achaflanar",
    "achaflanarían": "achaflanar",
    "achaflanarías": "achaflanar",
    "achaflanas": "achaflanar",
    "achaflanase": "achaflanar",
    "achaflanaseis": "achaflanar",
    "achaflanasen": "achaflanar",
    "achaflanases": "achaflanar",
    "achaflanaste": "achaflanar",
    "achaflanasteis": "achaflanar",
    "achaflane": "achaflanar",
    "achaflanemos": "achaflanar",
    "achaflanen": "achaflanar",
    "achaflanes": "achaflanar",
    "achaflano": "achaflanar",
    "achaflanábamos": "achaflanar",
    "achaflanáis": "achaflanar",
    "achaflanáramos": "achaflanar",
    "achaflanáremos": "achaflanar",
    "achaflanásemos": "achaflanar",
    "achaflané": "achaflanar",
    "achaflanéis": "achaflanar",
    "achaflanó": "achaflanar",
    "achajuana": "achajuanar",
    "achajuanaba": "achajuanar",
    "achajuanabais": "achajuanar",
    "achajuanaban": "achajuanar",
    "achajuanabas": "achajuanar",
    "achajuanad": "achajuanar",
    "achajuanada": "achajuanar",
    "achajuanadas": "achajuanar",
    "achajuanado": "achajuanar",
    "achajuanados": "achajuanar",
    "achajuanamos": "achajuanar",
    "achajuanan": "achajuanar",
    "achajuanando": "achajuanar",
    "achajuanara": "achajuanar",
    "achajuanarais": "achajuanar",
    "achajuanaran": "achajuanar",
    "achajuanaras": "achajuanar",
    "achajuanare": "achajuanar",
    "achajuanareis": "achajuanar",
    "achajuanaremos": "achajuanar",
    "achajuanaren": "achajuanar",
    "achajuanares": "achajuanar",
    "achajuanaron": "achajuanar",
    "achajuanará": "achajuanar",
    "achajuanarán": "achajuanar",
    "achajuanarás": "achajuanar",
    "achajuanaré": "achajuanar",
    "achajuanaréis": "achajuanar",
    "achajuanaría": "achajuanar",
    "achajuanaríais": "achajuanar",
    "achajuanaríamos": "achajuanar",
    "achajuanarían": "achajuanar",
    "achajuanarías": "achajuanar",
    "achajuanas": "achajuanar",
    "achajuanase": "achajuanar",
    "achajuanaseis": "achajuanar",
    "achajuanasen": "achajuanar",
    "achajuanases": "achajuanar",
    "achajuanaste": "achajuanar",
    "achajuanasteis": "achajuanar",
    "achajuane": "achajuanar",
    "achajuanemos": "achajuanar",
    "achajuanen": "achajuanar",
    "achajuanes": "achajuanar",
    "achajuano": "achajuanar",
    "achajuanábamos": "achajuanar",
    "achajuanáis": "achajuanar",
    "achajuanáramos": "achajuanar",
    "achajuanáremos": "achajuanar",
    "achajuanásemos": "achajuanar",
    "achajuané": "achajuanar",
    "achajuanéis": "achajuanar",
    "achajuanó": "achajuanar",
    "achampanada": "achampanado",
    "achampanadas": "achampanado",
    "achampanados": "achampanado",
    "achampañada": "achampañado",
    "achampañadas": "achampañado",
    "achampañados": "achampañado",
    "achanta": "achantar",
    "achantaba": "achantar",
    "achantabais": "achantar",
    "achantaban": "achantar",
    "achantabas": "achantar",
    "achantad": "achantar",
    "achantada": "achantar",
    "achantadas": "achantar",
    "achantado": "achantar",
    "achantados": "achantar",
    "achantamos": "achantar",
    "achantan": "achantar",
    "achantando": "achantar",
    "achantara": "achantar",
    "achantarais": "achantar",
    "achantaran": "achantar",
    "achantaras": "achantar",
    "achantare": "achantar",
    "achantareis": "achantar",
    "achantaremos": "achantar",
    "achantaren": "achantar",
    "achantares": "achantar",
    "achantaron": "achantar",
    "achantará": "achantar",
    "achantarán": "achantar",
    "achantarás": "achantar",
    "achantaré": "achantar",
    "achantaréis": "achantar",
    "achantaría": "achantar",
    "achantaríais": "achantar",
    "achantaríamos": "achantar",
    "achantarían": "achantar",
    "achantarías": "achantar",
    "achantas": "achantar",
    "achantase": "achantar",
    "achantaseis": "achantar",
    "achantasen": "achantar",
    "achantases": "achantar",
    "achantaste": "achantar",
    "achantasteis": "achantar",
    "achante": "achantar",
    "achantemos": "achantar",
    "achanten": "achantar",
    "achantes": "achantar",
    "achanto": "achantar",
    "achantábamos": "achantar",
    "achantáis": "achantar",
    "achantáramos": "achantar",
    "achantáremos": "achantar",
    "achantásemos": "achantar",
    "achanté": "achantar",
    "achantéis": "achantar",
    "achantó": "achantar",
    "achaparra": "achaparrar",
    "achaparraba": "achaparrar",
    "achaparrabais": "achaparrar",
    "achaparraban": "achaparrar",
    "achaparrabas": "achaparrar",
    "achaparrad": "achaparrar",
    "achaparrada": "achaparrar",
    "achaparradas": "achaparrar",
    "achaparrado": "achaparrar",
    "achaparrados": "achaparrar",
    "achaparramos": "achaparrar",
    "achaparran": "achaparrar",
    "achaparrando": "achaparrar",
    "achaparrara": "achaparrar",
    "achaparrarais": "achaparrar",
    "achaparraran": "achaparrar",
    "achaparraras": "achaparrar",
    "achaparrare": "achaparrar",
    "achaparrareis": "achaparrar",
    "achaparraremos": "achaparrar",
    "achaparraren": "achaparrar",
    "achaparrares": "achaparrar",
    "achaparraron": "achaparrar",
    "achaparrará": "achaparrar",
    "achaparrarán": "achaparrar",
    "achaparrarás": "achaparrar",
    "achaparraré": "achaparrar",
    "achaparraréis": "achaparrar",
    "achaparraría": "achaparrar",
    "achaparraríais": "achaparrar",
    "achaparraríamos": "achaparrar",
    "achaparrarían": "achaparrar",
    "achaparrarías": "achaparrar",
    "achaparras": "achaparrar",
    "achaparrase": "achaparrar",
    "achaparraseis": "achaparrar",
    "achaparrasen": "achaparrar",
    "achaparrases": "achaparrar",
    "achaparraste": "achaparrar",
    "achaparrasteis": "achaparrar",
    "achaparre": "achaparrar",
    "achaparremos": "achaparrar",
    "achaparren": "achaparrar",
    "achaparres": "achaparrar",
    "achaparro": "achaparrar",
    "achaparrábamos": "achaparrar",
    "achaparráis": "achaparrar",
    "achaparráramos": "achaparrar",
    "achaparráremos": "achaparrar",
    "achaparrásemos": "achaparrar",
    "achaparré": "achaparrar",
    "achaparréis": "achaparrar",
    "achaparró": "achaparrar",
    "achaque": "achacar",
    "achaquemos": "achacar",
    "achaquen": "achacar",
    "achaques": "achaque",
    "achaquienta": "achaquiento",
    "achaquientas": "achaquiento",
    "achaquientos": "achaquiento",
    "achaqué": "achacar",
    "achaquéis": "achacar",
    "acharola": "acharolar",
    "acharolaba": "acharolar",
    "acharolabais": "acharolar",
    "acharolaban": "acharolar",
    "acharolabas": "acharolar",
    "acharolad": "acharolar",
    "acharolada": "acharolar",
    "acharoladas": "acharolar",
    "acharolado": "acharolar",
    "acharolados": "acharolar",
    "acharolamos": "acharolar",
    "acharolan": "acharolar",
    "acharolando": "acharolar",
    "acharolara": "acharolar",
    "acharolarais": "acharolar",
    "acharolaran": "acharolar",
    "acharolaras": "acharolar",
    "acharolare": "acharolar",
    "acharolareis": "acharolar",
    "acharolaremos": "acharolar",
    "acharolaren": "acharolar",
    "acharolares": "acharolar",
    "acharolaron": "acharolar",
    "acharolará": "acharolar",
    "acharolarán": "acharolar",
    "acharolarás": "acharolar",
    "acharolaré": "acharolar",
    "acharolaréis": "acharolar",
    "acharolaría": "acharolar",
    "acharolaríais": "acharolar",
    "acharolaríamos": "acharolar",
    "acharolarían": "acharolar",
    "acharolarías": "acharolar",
    "acharolas": "acharolar",
    "acharolase": "acharolar",
    "acharolaseis": "acharolar",
    "acharolasen": "acharolar",
    "acharolases": "acharolar",
    "acharolaste": "acharolar",
    "acharolasteis": "acharolar",
    "acharole": "acharolar",
    "acharolemos": "acharolar",
    "acharolen": "acharolar",
    "acharoles": "acharolar",
    "acharolo": "acharolar",
    "acharolábamos": "acharolar",
    "acharoláis": "acharolar",
    "acharoláramos": "acharolar",
    "acharoláremos": "acharolar",
    "acharolásemos": "acharolar",
    "acharolé": "acharolar",
    "acharoléis": "acharolar",
    "acharoló": "acharolar",
    "achata": "achatar",
    "achataba": "achatar",
    "achatabais": "achatar",
    "achataban": "achatar",
    "achatabas": "achatar",
    "achatad": "achatar",
    "achatada": "achatar",
    "achatadas": "achatar",
    "achatado": "achatar",
    "achatados": "achatar",
    "achatamientos": "achatamiento",
    "achatamos": "achatar",
    "achatan": "achatar",
    "achatando": "achatar",
    "achatara": "achatar",
    "achatarais": "achatar",
    "achataran": "achatar",
    "achataras": "achatar",
    "achatare": "achatar",
    "achatareis": "achatar",
    "achataremos": "achatar",
    "achataren": "achatar",
    "achatares": "achatar",
    "achataron": "achatar",
    "achatará": "achatar",
    "achatarán": "achatar",
    "achatarás": "achatar",
    "achataré": "achatar",
    "achataréis": "achatar",
    "achataría": "achatar",
    "achataríais": "achatar",
    "achataríamos": "achatar",
    "achatarían": "achatar",
    "achatarías": "achatar",
    "achatas": "achatar",
    "achatase": "achatar",
    "achataseis": "achatar",
    "achatasen": "achatar",
    "achatases": "achatar",
    "achataste": "achatar",
    "achatasteis": "achatar",
    "achate": "achatar",
    "achatemos": "achatar",
    "achaten": "achatar",
    "achates": "achatar",
    "achato": "achatar",
    "achatábamos": "achatar",
    "achatáis": "achatar",
    "achatáramos": "achatar",
    "achatáremos": "achatar",
    "achatásemos": "achatar",
    "achaté": "achatar",
    "achatéis": "achatar",
    "acható": "achatar",
    "achica": "achicar",
    "achicaba": "achicar",
    "achicabais": "achicar",
    "achicaban": "achicar",
    "achicabas": "achicar",
    "achicad": "achicar",
    "achicada": "achicar",
    "achicadas": "achicar",
    "achicado": "achicar",
    "achicadora": "achicador",
    "achicadoras": "achicador",
    "achicadores": "achicador",
    "achicados": "achicar",
    "achicaduras": "achicadura",
    "achicamientos": "achicamiento",
    "achicamos": "achicar",
    "achican": "achicar",
    "achicando": "achicar",
    "achicara": "achicar",
    "achicarais": "achicar",
    "achicaran": "achicar",
    "achicaras": "achicar",
    "achicare": "achicar",
    "achicareis": "achicar",
    "achicaremos": "achicar",
    "achicaren": "achicar",
    "achicares": "achicar",
    "achicaron": "achicar",
    "achicará": "achicar",
    "achicarán": "achicar",
    "achicarás": "achicar",
    "achicaré": "achicar",
    "achicaréis": "achicar",
    "achicaría": "achicar",
    "achicaríais": "achicar",
    "achicaríamos": "achicar",
    "achicarían": "achicar",
    "achicarías": "achicar",
    "achicas": "achicar",
    "achicase": "achicar",
    "achicaseis": "achicar",
    "achicasen": "achicar",
    "achicases": "achicar",
    "achicaste": "achicar",
    "achicasteis": "achicar",
    "achicharra": "achicharrar",
    "achicharraba": "achicharrar",
    "achicharrabais": "achicharrar",
    "achicharraban": "achicharrar",
    "achicharrabas": "achicharrar",
    "achicharrad": "achicharrar",
    "achicharrada": "achicharrar",
    "achicharradas": "achicharrar",
    "achicharraderos": "achicharradero",
    "achicharrado": "achicharrar",
    "achicharrados": "achicharrar",
    "achicharramientos": "achicharramiento",
    "achicharramos": "achicharrar",
    "achicharran": "achicharrar",
    "achicharrando": "achicharrar",
    "achicharrantes": "achicharrante",
    "achicharrara": "achicharrar",
    "achicharrarais": "achicharrar",
    "achicharraran": "achicharrar",
    "achicharraras": "achicharrar",
    "achicharrare": "achicharrar",
    "achicharrareis": "achicharrar",
    "achicharraremos": "achicharrar",
    "achicharraren": "achicharrar",
    "achicharrares": "achicharrar",
    "achicharraron": "achicharrar",
    "achicharrará": "achicharrar",
    "achicharrarán": "achicharrar",
    "achicharrarás": "achicharrar",
    "achicharraré": "achicharrar",
    "achicharraréis": "achicharrar",
    "achicharraría": "achicharrar",
    "achicharraríais": "achicharrar",
    "achicharraríamos": "achicharrar",
    "achicharrarían": "achicharrar",
    "achicharrarías": "achicharrar",
    "achicharras": "achicharrar",
    "achicharrase": "achicharrar",
    "achicharraseis": "achicharrar",
    "achicharrasen": "achicharrar",
    "achicharrases": "achicharrar",
    "achicharraste": "achicharrar",
    "achicharrasteis": "achicharrar",
    "achicharre": "achicharrar",
    "achicharremos": "achicharrar",
    "achicharren": "achicharrar",
    "achicharres": "achicharrar",
    "achicharro": "achicharrar",
    "achicharrábamos": "achicharrar",
    "achicharráis": "achicharrar",
    "achicharráramos": "achicharrar",
    "achicharráremos": "achicharrar",
    "achicharrásemos": "achicharrar",
    "achicharré": "achicharrar",
    "achicharréis": "achicharrar",
    "achicharró": "achicharrar",
    "achico": "achicar",
    "achicorias": "achicoria",
    "achicábamos": "achicar",
    "achicáis": "achicar",
    "achicáramos": "achicar",
    "achicáremos": "achicar",
    "achicásemos": "achicar",
    "achicó": "achicar",
    "achigua": "achiguar",
    "achiguaba": "achiguar",
    "achiguabais": "achiguar",
    "achiguaban": "achiguar",
    "achiguabas": "achiguar",
    "achiguad": "achiguar",
    "achiguada": "achiguar",
    "achiguadas": "achiguar",
    "achiguado": "achiguar",
    "achiguados": "achiguar",
    "achiguamos": "achiguar",
    "achiguan": "achiguar",
    "achiguando": "achiguar",
    "achiguara": "achiguar",
    "achiguarais": "achiguar",
    "achiguaran": "achiguar",
    "achiguaras": "achiguar",
    "achiguare": "achiguar",
    "achiguareis": "achiguar",
    "achiguaremos": "achiguar",
    "achiguaren": "achiguar",
    "achiguares": "achiguar",
    "achiguaron": "achiguar",
    "achiguará": "achiguar",
    "achiguarán": "achiguar",
    "achiguarás": "achiguar",
    "achiguaré": "achiguar",
    "achiguaréis": "achiguar",
    "achiguaría": "achiguar",
    "achiguaríais": "achiguar",
    "achiguaríamos": "achiguar",
    "achiguarían": "achiguar",
    "achiguarías": "achiguar",
    "achiguas": "achiguar",
    "achiguase": "achiguar",
    "achiguaseis": "achiguar",
    "achiguasen": "achiguar",
    "achiguases": "achiguar",
    "achiguaste": "achiguar",
    "achiguasteis": "achiguar",
    "achiguo": "achiguar",
    "achiguábamos": "achiguar",
    "achiguáis": "achiguar",
    "achiguáramos": "achiguar",
    "achiguáremos": "achiguar",
    "achiguásemos": "achiguar",
    "achiguó": "achiguar",
    "achigüe": "achiguar",
    "achigüemos": "achiguar",
    "achigüen": "achiguar",
    "achigües": "achiguar",
    "achigüé": "achiguar",
    "achigüéis": "achiguar",
    "achinada": "achinado",
    "achinadas": "achinado",
    "achinados": "achinado",
    "achinamientos": "achinamiento",
    "achiotes": "achiote",
    "achique": "achicar",
    "achiquemos": "achicar",
    "achiquen": "achicar",
    "achiques": "achique",
    "achiquillada": "achiquillado",
    "achiquilladas": "achiquillado",
    "achiquillados": "achiquillado",
    "achiqué": "achicar",
    "achiquéis": "achicar",
    "achiras": "achira",
    "achispa": "achispar",
    "achispaba": "achispar",
    "achispabais": "achispar",
    "achispaban": "achispar",
    "achispabas": "achispar",
    "achispad": "achispar",
    "achispada": "achispar",
    "achispadas": "achispar",
    "achispado": "achispar",
    "achispados": "achispar",
    "achispamientos": "achispamiento",
    "achispamos": "achispar",
    "achispan": "achispar",
    "achispando": "achispar",
    "achispara": "achispar",
    "achisparais": "achispar",
    "achisparan": "achispar",
    "achisparas": "achispar",
    "achispare": "achispar",
    "achispareis": "achispar",
    "achisparemos": "achispar",
    "achisparen": "achispar",
    "achispares": "achispar",
    "achisparon": "achispar",
    "achispará": "achispar",
    "achisparán": "achispar",
    "achisparás": "achispar",
    "achisparé": "achispar",
    "achisparéis": "achispar",
    "achisparía": "achispar",
    "achisparíais": "achispar",
    "achisparíamos": "achispar",
    "achisparían": "achispar",
    "achisparías": "achispar",
    "achispas": "achispar",
    "achispase": "achispar",
    "achispaseis": "achispar",
    "achispasen": "achispar",
    "achispases": "achispar",
    "achispaste": "achispar",
    "achispasteis": "achispar",
    "achispe": "achispar",
    "achispemos": "achispar",
    "achispen": "achispar",
    "achispes": "achispar",
    "achispo": "achispar",
    "achispábamos": "achispar",
    "achispáis": "achispar",
    "achispáramos": "achispar",
    "achispáremos": "achispar",
    "achispásemos": "achispar",
    "achispé": "achispar",
    "achispéis": "achispar",
    "achispó": "achispar",
    "achoca": "achocar",
    "achocaba": "achocar",
    "achocabais": "achocar",
    "achocaban": "achocar",
    "achocabas": "achocar",
    "achocad": "achocar",
    "achocada": "achocar",
    "achocadas": "achocar",
    "achocado": "achocar",
    "achocados": "achocar",
    "achocamos": "achocar",
    "achocan": "achocar",
    "achocando": "achocar",
    "achocara": "achocar",
    "achocarais": "achocar",
    "achocaran": "achocar",
    "achocaras": "achocar",
    "achocare": "achocar",
    "achocareis": "achocar",
    "achocaremos": "achocar",
    "achocaren": "achocar",
    "achocares": "achocar",
    "achocaron": "achocar",
    "achocará": "achocar",
    "achocarán": "achocar",
    "achocarás": "achocar",
    "achocaré": "achocar",
    "achocaréis": "achocar",
    "achocaría": "achocar",
    "achocaríais": "achocar",
    "achocaríamos": "achocar",
    "achocarían": "achocar",
    "achocarías": "achocar",
    "achocas": "achocar",
    "achocase": "achocar",
    "achocaseis": "achocar",
    "achocasen": "achocar",
    "achocases": "achocar",
    "achocaste": "achocar",
    "achocasteis": "achocar",
    "achoco": "achocar",
    "achocolatada": "achocolatado",
    "achocolatadas": "achocolatado",
    "achocolatados": "achocolatado",
    "achocábamos": "achocar",
    "achocáis": "achocar",
    "achocáramos": "achocar",
    "achocáremos": "achocar",
    "achocásemos": "achocar",
    "achocó": "achocar",
    "achola": "acholar",
    "acholaba": "acholar",
    "acholabais": "acholar",
    "acholaban": "acholar",
    "acholabas": "acholar",
    "acholad": "acholar",
    "acholada": "acholar",
    "acholadas": "acholar",
    "acholado": "acholar",
    "acholados": "acholar",
    "acholamos": "acholar",
    "acholan": "acholar",
    "acholando": "acholar",
    "acholara": "acholar",
    "acholarais": "acholar",
    "acholaran": "acholar",
    "acholaras": "acholar",
    "acholare": "acholar",
    "acholareis": "acholar",
    "acholaremos": "acholar",
    "acholaren": "acholar",
    "acholares": "acholar",
    "acholaron": "acholar",
    "acholará": "acholar",
    "acholarán": "acholar",
    "acholarás": "acholar",
    "acholaré": "acholar",
    "acholaréis": "acholar",
    "acholaría": "acholar",
    "acholaríais": "acholar",
    "acholaríamos": "acholar",
    "acholarían": "acholar",
    "acholarías": "acholar",
    "acholas": "acholar",
    "acholase": "acholar",
    "acholaseis": "acholar",
    "acholasen": "acholar",
    "acholases": "acholar",
    "acholaste": "acholar",
    "acholasteis": "acholar",
    "achole": "acholar",
    "acholemos": "acholar",
    "acholen": "acholar",
    "acholes": "acholar",
    "acholo": "acholar",
    "acholábamos": "acholar",
    "acholáis": "acholar",
    "acholáramos": "acholar",
    "acholáremos": "acholar",
    "acholásemos": "acholar",
    "acholé": "acholar",
    "acholéis": "acholar",
    "acholó": "acholar",
    "achoque": "achocar",
    "achoquemos": "achocar",
    "achoquen": "achocar",
    "achoques": "achocar",
    "achoqué": "achocar",
    "achoquéis": "achocar",
    "achotes": "achote",
    "achubasca": "achubascar",
    "achubascaba": "achubascar",
    "achubascabais": "achubascar",
    "achubascaban": "achubascar",
    "achubascabas": "achubascar",
    "achubascad": "achubascar",
    "achubascada": "achubascar",
    "achubascadas": "achubascar",
    "achubascado": "achubascar",
    "achubascados": "achubascar",
    "achubascamos": "achubascar",
    "achubascan": "achubascar",
    "achubascando": "achubascar",
    "achubascara": "achubascar",
    "achubascarais": "achubascar",
    "achubascaran": "achubascar",
    "achubascaras": "achubascar",
    "achubascare": "achubascar",
    "achubascareis": "achubascar",
    "achubascaremos": "achubascar",
    "achubascaren": "achubascar",
    "achubascares": "achubascar",
    "achubascaron": "achubascar",
    "achubascará": "achubascar",
    "achubascarán": "achubascar",
    "achubascarás": "achubascar",
    "achubascaré": "achubascar",
    "achubascaréis": "achubascar",
    "achubascaría": "achubascar",
    "achubascaríais": "achubascar",
    "achubascaríamos": "achubascar",
    "achubascarían": "achubascar",
    "achubascarías": "achubascar",
    "achubascas": "achubascar",
    "achubascase": "achubascar",
    "achubascaseis": "achubascar",
    "achubascasen": "achubascar",
    "achubascases": "achubascar",
    "achubascaste": "achubascar",
    "achubascasteis": "achubascar",
    "achubasco": "achubascar",
    "achubascábamos": "achubascar",
    "achubascáis": "achubascar",
    "achubascáramos": "achubascar",
    "achubascáremos": "achubascar",
    "achubascásemos": "achubascar",
    "achubascó": "achubascar",
    "achubasque": "achubascar",
    "achubasquemos": "achubascar",
    "achubasquen": "achubascar",
    "achubasques": "achubascar",
    "achubasqué": "achubascar",
    "achubasquéis": "achubascar",
    "achucha": "achuchar",
    "achuchaba": "achuchar",
    "achuchabais": "achuchar",
    "achuchaban": "achuchar",
    "achuchabas": "achuchar",
    "achuchad": "achuchar",
    "achuchada": "achuchar",
    "achuchadas": "achuchar",
    "achuchado": "achuchar",
    "achuchados": "achuchar",
    "achuchamos": "achuchar",
    "achuchan": "achuchar",
    "achuchando": "achuchar",
    "achuchara": "achuchar",
    "achucharais": "achuchar",
    "achucharan": "achuchar",
    "achucharas": "achuchar",
    "achuchare": "achuchar",
    "achuchareis": "achuchar",
    "achucharemos": "achuchar",
    "achucharen": "achuchar",
    "achuchares": "achuchar",
    "achucharon": "achuchar",
    "achucharra": "achucharrar",
    "achucharraba": "achucharrar",
    "achucharrabais": "achucharrar",
    "achucharraban": "achucharrar",
    "achucharrabas": "achucharrar",
    "achucharrad": "achucharrar",
    "achucharrada": "achucharrar",
    "achucharradas": "achucharrar",
    "achucharrado": "achucharrar",
    "achucharrados": "achucharrar",
    "achucharramos": "achucharrar",
    "achucharran": "achucharrar",
    "achucharrando": "achucharrar",
    "achucharrara": "achucharrar",
    "achucharrarais": "achucharrar",
    "achucharraran": "achucharrar",
    "achucharraras": "achucharrar",
    "achucharrare": "achucharrar",
    "achucharrareis": "achucharrar",
    "achucharraremos": "achucharrar",
    "achucharraren": "achucharrar",
    "achucharrares": "achucharrar",
    "achucharraron": "achucharrar",
    "achucharrará": "achucharrar",
    "achucharrarán": "achucharrar",
    "achucharrarás": "achucharrar",
    "achucharraré": "achucharrar",
    "achucharraréis": "achucharrar",
    "achucharraría": "achucharrar",
    "achucharraríais": "achucharrar",
    "achucharraríamos": "achucharrar",
    "achucharrarían": "achucharrar",
    "achucharrarías": "achucharrar",
    "achucharras": "achucharrar",
    "achucharrase": "achucharrar",
    "achucharraseis": "achucharrar",
    "achucharrasen": "achucharrar",
    "achucharrases": "achucharrar",
    "achucharraste": "achucharrar",
    "achucharrasteis": "achucharrar",
    "achucharre": "achucharrar",
    "achucharremos": "achucharrar",
    "achucharren": "achucharrar",
    "achucharres": "achucharrar",
    "achucharro": "achucharrar",
    "achucharrábamos": "achucharrar",
    "achucharráis": "achucharrar",
    "achucharráramos": "achucharrar",
    "achucharráremos": "achucharrar",
    "achucharrásemos": "achucharrar",
    "achucharré": "achucharrar",
    "achucharréis": "achucharrar",
    "achucharró": "achucharrar",
    "achuchará": "achuchar",
    "achucharán": "achuchar",
    "achucharás": "achuchar",
    "achucharé": "achuchar",
    "achucharéis": "achuchar",
    "achucharía": "achuchar",
    "achucharíais": "achuchar",
    "achucharíamos": "achuchar",
    "achucharían": "achuchar",
    "achucharías": "achuchar",
    "achuchas": "achuchar",
    "achuchase": "achuchar",
    "achuchaseis": "achuchar",
    "achuchasen": "achuchar",
    "achuchases": "achuchar",
    "achuchaste": "achuchar",
    "achuchasteis": "achuchar",
    "achuche": "achuchar",
    "achuchemos": "achuchar",
    "achuchen": "achuchar",
    "achuches": "achuchar",
    "achucho": "achuchar",
    "achuchones": "achuchón",
    "achuchábamos": "achuchar",
    "achucháis": "achuchar",
    "achucháramos": "achuchar",
    "achucháremos": "achuchar",
    "achuchásemos": "achuchar",
    "achuché": "achuchar",
    "achuchéis": "achuchar",
    "achuchó": "achuchar",
    "achucuya": "achucuyar",
    "achucuyaba": "achucuyar",
    "achucuyabais": "achucuyar",
    "achucuyaban": "achucuyar",
    "achucuyabas": "achucuyar",
    "achucuyad": "achucuyar",
    "achucuyada": "achucuyar",
    "achucuyadas": "achucuyar",
    "achucuyado": "achucuyar",
    "achucuyados": "achucuyar",
    "achucuyamos": "achucuyar",
    "achucuyan": "achucuyar",
    "achucuyando": "achucuyar",
    "achucuyara": "achucuyar",
    "achucuyarais": "achucuyar",
    "achucuyaran": "achucuyar",
    "achucuyaras": "achucuyar",
    "achucuyare": "achucuyar",
    "achucuyareis": "achucuyar",
    "achucuyaremos": "achucuyar",
    "achucuyaren": "achucuyar",
    "achucuyares": "achucuyar",
    "achucuyaron": "achucuyar",
    "achucuyará": "achucuyar",
    "achucuyarán": "achucuyar",
    "achucuyarás": "achucuyar",
    "achucuyaré": "achucuyar",
    "achucuyaréis": "achucuyar",
    "achucuyaría": "achucuyar",
    "achucuyaríais": "achucuyar",
    "achucuyaríamos": "achucuyar",
    "achucuyarían": "achucuyar",
    "achucuyarías": "achucuyar",
    "achucuyas": "achucuyar",
    "achucuyase": "achucuyar",
    "achucuyaseis": "achucuyar",
    "achucuyasen": "achucuyar",
    "achucuyases": "achucuyar",
    "achucuyaste": "achucuyar",
    "achucuyasteis": "achucuyar",
    "achucuye": "achucuyar",
    "achucuyemos": "achucuyar",
    "achucuyen": "achucuyar",
    "achucuyes": "achucuyar",
    "achucuyo": "achucuyar",
    "achucuyábamos": "achucuyar",
    "achucuyáis": "achucuyar",
    "achucuyáramos": "achucuyar",
    "achucuyáremos": "achucuyar",
    "achucuyásemos": "achucuyar",
    "achucuyé": "achucuyar",
    "achucuyéis": "achucuyar",
    "achucuyó": "achucuyar",
    "achulada": "achulado",
    "achuladas": "achulado",
    "achulados": "achulado",
    "achulapada": "achulapado",
    "achulapadas": "achulapado",
    "achulapados": "achulapado",
    "achura": "achurar",
    "achuraba": "achurar",
    "achurabais": "achurar",
    "achuraban": "achurar",
    "achurabas": "achurar",
    "achurad": "achurar",
    "achurada": "achurar",
    "achuradas": "achurar",
    "achurado": "achurar",
    "achuradora": "achurador",
    "achuradoras": "achurador",
    "achuradores": "achurador",
    "achurados": "achurar",
    "achuramos": "achurar",
    "achuran": "achurar",
    "achurando": "achurar",
    "achurara": "achurar",
    "achurarais": "achurar",
    "achuraran": "achurar",
    "achuraras": "achurar",
    "achurare": "achurar",
    "achurareis": "achurar",
    "achuraremos": "achurar",
    "achuraren": "achurar",
    "achurares": "achurar",
    "achuraron": "achurar",
    "achurará": "achurar",
    "achurarán": "achurar",
    "achurarás": "achurar",
    "achuraré": "achurar",
    "achuraréis": "achurar",
    "achuraría": "achurar",
    "achuraríais": "achurar",
    "achuraríamos": "achurar",
    "achurarían": "achurar",
    "achurarías": "achurar",
    "achuras": "achurar",
    "achurase": "achurar",
    "achuraseis": "achurar",
    "achurasen": "achurar",
    "achurases": "achurar",
    "achuraste": "achurar",
    "achurasteis": "achurar",
    "achure": "achurar",
    "achuremos": "achurar",
    "achuren": "achurar",
    "achures": "achurar",
    "achuro": "achurar",
    "achurábamos": "achurar",
    "achuráis": "achurar",
    "achuráramos": "achurar",
    "achuráremos": "achurar",
    "achurásemos": "achurar",
    "achuré": "achurar",
    "achuréis": "achurar",
    "achuró": "achurar",
    "aciaga": "aciago",
    "aciagas": "aciago",
    "aciagos": "aciago",
    "aciales": "acial",
    "acianos": "aciano",
    "acibara": "acibarar",
    "acibaraba": "acibarar",
    "acibarabais": "acibarar",
    "acibaraban": "acibarar",
    "acibarabas": "acibarar",
    "acibarad": "acibarar",
    "acibarada": "acibarar",
    "acibaradas": "acibarar",
    "acibarado": "acibarar",
    "acibarados": "acibarar",
    "acibaramos": "acibarar",
    "acibaran": "acibarar",
    "acibarando": "acibarar",
    "acibarara": "acibarar",
    "acibararais": "acibarar",
    "acibararan": "acibarar",
    "acibararas": "acibarar",
    "acibarare": "acibarar",
    "acibarareis": "acibarar",
    "acibararemos": "acibarar",
    "acibararen": "acibarar",
    "acibarares": "acibarar",
    "acibararon": "acibarar",
    "acibarará": "acibarar",
    "acibararán": "acibarar",
    "acibararás": "acibarar",
    "acibararé": "acibarar",
    "acibararéis": "acibarar",
    "acibararía": "acibarar",
    "acibararíais": "acibarar",
    "acibararíamos": "acibarar",
    "acibararían": "acibarar",
    "acibararías": "acibarar",
    "acibaras": "acibarar",
    "acibarase": "acibarar",
    "acibaraseis": "acibarar",
    "acibarasen": "acibarar",
    "acibarases": "acibarar",
    "acibaraste": "acibarar",
    "acibarasteis": "acibarar",
    "acibare": "acibarar",
    "acibaremos": "acibarar",
    "acibaren": "acibarar",
    "acibares": "acibarar",
    "acibaro": "acibarar",
    "acibarábamos": "acibarar",
    "acibaráis": "acibarar",
    "acibaráramos": "acibarar",
    "acibaráremos": "acibarar",
    "acibarásemos": "acibarar",
    "acibaré": "acibarar",
    "acibaréis": "acibarar",
    "acibaró": "acibarar",
    "acibera": "aciberar",
    "aciberaba": "aciberar",
    "aciberabais": "aciberar",
    "aciberaban": "aciberar",
    "aciberabas": "aciberar",
    "aciberad": "aciberar",
    "aciberada": "aciberar",
    "aciberadas": "aciberar",
    "aciberado": "aciberar",
    "aciberados": "aciberar",
    "aciberamos": "aciberar",
    "aciberan": "aciberar",
    "aciberando": "aciberar",
    "aciberara": "aciberar",
    "aciberarais": "aciberar",
    "aciberaran": "aciberar",
    "aciberaras": "aciberar",
    "aciberare": "aciberar",
    "aciberareis": "aciberar",
    "aciberaremos": "aciberar",
    "aciberaren": "aciberar",
    "aciberares": "aciberar",
    "aciberaron": "aciberar",
    "aciberará": "aciberar",
    "aciberarán": "aciberar",
    "aciberarás": "aciberar",
    "aciberaré": "aciberar",
    "aciberaréis": "aciberar",
    "aciberaría": "aciberar",
    "aciberaríais": "aciberar",
    "aciberaríamos": "aciberar",
    "aciberarían": "aciberar",
    "aciberarías": "aciberar",
    "aciberas": "aciberar",
    "aciberase": "aciberar",
    "aciberaseis": "aciberar",
    "aciberasen": "aciberar",
    "aciberases": "aciberar",
    "aciberaste": "aciberar",
    "aciberasteis": "aciberar",
    "acibere": "aciberar",
    "aciberemos": "aciberar",
    "aciberen": "aciberar",
    "aciberes": "aciberar",
    "acibero": "aciberar",
    "aciberábamos": "aciberar",
    "aciberáis": "aciberar",
    "aciberáramos": "aciberar",
    "aciberáremos": "aciberar",
    "aciberásemos": "aciberar",
    "aciberé": "aciberar",
    "aciberéis": "aciberar",
    "aciberó": "aciberar",
    "acicala": "acicalar",
    "acicalaba": "acicalar",
    "acicalabais": "acicalar",
    "acicalaban": "acicalar",
    "acicalabas": "acicalar",
    "acicalad": "acicalar",
    "acicalada": "acicalar",
    "acicaladas": "acicalar",
    "acicalado": "acicalar",
    "acicalados": "acicalar",
    "acicaladuras": "acicaladura",
    "acicalamientos": "acicalamiento",
    "acicalamos": "acicalar",
    "acicalan": "acicalar",
    "acicalando": "acicalar",
    "acicalara": "acicalar",
    "acicalarais": "acicalar",
    "acicalaran": "acicalar",
    "acicalaras": "acicalar",
    "acicalare": "acicalar",
    "acicalareis": "acicalar",
    "acicalaremos": "acicalar",
    "acicalaren": "acicalar",
    "acicalares": "acicalar",
    "acicalaron": "acicalar",
    "acicalará": "acicalar",
    "acicalarán": "acicalar",
    "acicalarás": "acicalar",
    "acicalaré": "acicalar",
    "acicalaréis": "acicalar",
    "acicalaría": "acicalar",
    "acicalaríais": "acicalar",
    "acicalaríamos": "acicalar",
    "acicalarían": "acicalar",
    "acicalarías": "acicalar",
    "acicalas": "acicalar",
    "acicalase": "acicalar",
    "acicalaseis": "acicalar",
    "acicalasen": "acicalar",
    "acicalases": "acicalar",
    "acicalaste": "acicalar",
    "acicalasteis": "acicalar",
    "acicale": "acicalar",
    "acicalemos": "acicalar",
    "acicalen": "acicalar",
    "acicales": "acicalar",
    "acicalo": "acicalar",
    "acicalábamos": "acicalar",
    "acicaláis": "acicalar",
    "acicaláramos": "acicalar",
    "acicaláremos": "acicalar",
    "acicalásemos": "acicalar",
    "acicalé": "acicalar",
    "acicaléis": "acicalar",
    "acicaló": "acicalar",
    "acicatea": "acicatear",
    "acicateaba": "acicatear",
    "acicateabais": "acicatear",
    "acicateaban": "acicatear",
    "acicateabas": "acicatear",
    "acicatead": "acicatear",
    "acicateada": "acicatear",
    "acicateadas": "acicatear",
    "acicateado": "acicatear",
    "acicateados": "acicatear",
    "acicateamos": "acicatear",
    "acicatean": "acicatear",
    "acicateando": "acicatear",
    "acicateara": "acicatear",
    "acicatearais": "acicatear",
    "acicatearan": "acicatear",
    "acicatearas": "acicatear",
    "acicateare": "acicatear",
    "acicateareis": "acicatear",
    "acicatearemos": "acicatear",
    "acicatearen": "acicatear",
    "acicateares": "acicatear",
    "acicatearon": "acicatear",
    "acicateará": "acicatear",
    "acicatearán": "acicatear",
    "acicatearás": "acicatear",
    "acicatearé": "acicatear",
    "acicatearéis": "acicatear",
    "acicatearía": "acicatear",
    "acicatearíais": "acicatear",
    "acicatearíamos": "acicatear",
    "acicatearían": "acicatear",
    "acicatearías": "acicatear",
    "acicateas": "acicatear",
    "acicatease": "acicatear",
    "acicateaseis": "acicatear",
    "acicateasen": "acicatear",
    "acicateases": "acicatear",
    "acicateaste": "acicatear",
    "acicateasteis": "acicatear",
    "acicatee": "acicatear",
    "acicateemos": "acicatear",
    "acicateen": "acicatear",
    "acicatees": "acicatear",
    "acicateo": "acicatear",
    "acicates": "acicate",
    "acicateábamos": "acicatear",
    "acicateáis": "acicatear",
    "acicateáramos": "acicatear",
    "acicateáremos": "acicatear",
    "acicateásemos": "acicatear",
    "acicateé": "acicatear",
    "acicateéis": "acicatear",
    "acicateó": "acicatear",
    "aciches": "aciche",
    "aciculares": "acicular",
    "aciculifolia": "aciculifolio",
    "aciculifolias": "aciculifolio",
    "aciculifolios": "aciculifolio",
    "acida": "acidar",
    "acidaba": "acidar",
    "acidabais": "acidar",
    "acidaban": "acidar",
    "acidabas": "acidar",
    "acidad": "acidar",
    "acidada": "acidar",
    "acidadas": "acidar",
    "acidado": "acidar",
    "acidados": "acidar",
    "acidamos": "acidar",
    "acidan": "acidar",
    "acidando": "acidar",
    "acidara": "acidar",
    "acidarais": "acidar",
    "acidaran": "acidar",
    "acidaras": "acidar",
    "acidare": "acidar",
    "acidareis": "acidar",
    "acidaremos": "acidar",
    "acidaren": "acidar",
    "acidares": "acidar",
    "acidaron": "acidar",
    "acidará": "acidar",
    "acidarán": "acidar",
    "acidarás": "acidar",
    "acidaré": "acidar",
    "acidaréis": "acidar",
    "acidaría": "acidar",
    "acidaríais": "acidar",
    "acidaríamos": "acidar",
    "acidarían": "acidar",
    "acidarías": "acidar",
    "acidas": "acidar",
    "acidase": "acidar",
    "acidaseis": "acidar",
    "acidasen": "acidar",
    "acidases": "acidar",
    "acidaste": "acidar",
    "acidasteis": "acidar",
    "acide": "acidar",
    "acideces": "acidez",
    "acidemos": "acidar",
    "aciden": "acidar",
    "acides": "acidar",
    "acidias": "acidia",
    "acidifica": "acidificar",
    "acidificaba": "acidificar",
    "acidificabais": "acidificar",
    "acidificaban": "acidificar",
    "acidificabas": "acidificar",
    "acidificaciones": "acidificación",
    "acidificad": "acidificar",
    "acidificada": "acidificar",
    "acidificadas": "acidificar",
    "acidificado": "acidificar",
    "acidificados": "acidificar",
    "acidificamientos": "acidificamiento",
    "acidificamos": "acidificar",
    "acidifican": "acidificar",
    "acidificando": "acidificar",
    "acidificantes": "acidificante",
    "acidificara": "acidificar",
    "acidificarais": "acidificar",
    "acidificaran": "acidificar",
    "acidificaras": "acidificar",
    "acidificare": "acidificar",
    "acidificareis": "acidificar",
    "acidificaremos": "acidificar",
    "acidificaren": "acidificar",
    "acidificares": "acidificar",
    "acidificaron": "acidificar",
    "acidificará": "acidificar",
    "acidificarán": "acidificar",
    "acidificarás": "acidificar",
    "acidificaré": "acidificar",
    "acidificaréis": "acidificar",
    "acidificaría": "acidificar",
    "acidificaríais": "acidificar",
    "acidificaríamos": "acidificar",
    "acidificarían": "acidificar",
    "acidificarías": "acidificar",
    "acidificas": "acidificar",
    "acidificase": "acidificar",
    "acidificaseis": "acidificar",
    "acidificasen": "acidificar",
    "acidificases": "acidificar",
    "acidificaste": "acidificar",
    "acidificasteis": "acidificar",
    "acidifico": "acidificar",
    "acidificábamos": "acidificar",
    "acidificáis": "acidificar",
    "acidificáramos": "acidificar",
    "acidificáremos": "acidificar",
    "acidificásemos": "acidificar",
    "acidificó": "acidificar",
    "acidifique": "acidificar",
    "acidifiquemos": "acidificar",
    "acidifiquen": "acidificar",
    "acidifiques": "acidificar",
    "acidifiqué": "acidificar",
    "acidifiquéis": "acidificar",
    "acidimetrías": "acidimetría",
    "acidismos": "acidismo",
    "acidizaciones": "acidización",
    "acido": "acidar",
    "acidogénica": "acidogénico",
    "acidogénicas": "acidogénico",
    "acidogénicos": "acidogénico",
    "acidula": "acidular",
    "acidulaba": "acidular",
    "acidulabais": "acidular",
    "acidulaban": "acidular",
    "acidulabas": "acidular",
    "acidulaciones": "acidulación",
    "acidulad": "acidular",
    "acidulada": "acidular",
    "aciduladas": "acidular",
    "acidulado": "acidular",
    "acidulados": "acidular",
    "acidulamos": "acidular",
    "acidulan": "acidular",
    "acidulando": "acidular",
    "acidulantes": "acidulante",
    "acidulara": "acidular",
    "acidularais": "acidular",
    "acidularan": "acidular",
    "acidularas": "acidular",
    "acidulare": "acidular",
    "acidulareis": "acidular",
    "acidularemos": "acidular",
    "acidularen": "acidular",
    "acidulares": "acidular",
    "acidularon": "acidular",
    "acidulará": "acidular",
    "acidularán": "acidular",
    "acidularás": "acidular",
    "acidularé": "acidular",
    "acidularéis": "acidular",
    "acidularía": "acidular",
    "acidularíais": "acidular",
    "acidularíamos": "acidular",
    "acidularían": "acidular",
    "acidularías": "acidular",
    "acidulas": "acidular",
    "acidulase": "acidular",
    "acidulaseis": "acidular",
    "acidulasen": "acidular",
    "acidulases": "acidular",
    "acidulaste": "acidular",
    "acidulasteis": "acidular",
    "acidule": "acidular",
    "acidulemos": "acidular",
    "acidulen": "acidular",
    "acidules": "acidular",
    "acidulo": "acidular",
    "acidulábamos": "acidular",
    "aciduláis": "acidular",
    "aciduláramos": "acidular",
    "aciduláremos": "acidular",
    "acidulásemos": "acidular",
    "acidulé": "acidular",
    "aciduléis": "acidular",
    "aciduló": "acidular",
    "acidábamos": "acidar",
    "acidáis": "acidar",
    "acidáramos": "acidar",
    "acidáremos": "acidar",
    "acidásemos": "acidar",
    "acidé": "acidar",
    "acidéis": "acidar",
    "acidímetros": "acidímetro",
    "acidó": "acidar",
    "acidófila": "acidófilo",
    "acidófilas": "acidófilo",
    "acidófilos": "acidófilo",
    "acidógena": "acidógeno",
    "acidógenas": "acidógeno",
    "acidógenos": "acidógeno",
    "acidósica": "acidósico",
    "acidósicas": "acidósico",
    "acidósicos": "acidósico",
    "acierta": "acertar",
    "aciertan": "acertar",
    "aciertas": "acertar",
    "acierte": "acertar",
    "acierten": "acertar",
    "aciertes": "acertar",
    "acierto": "acertar",
    "aciertos": "acierto",
    "acigospermas": "acigosperma",
    "acigotos": "acigoto",
    "aciguata": "aciguatar",
    "aciguataba": "aciguatar",
    "aciguatabais": "aciguatar",
    "aciguataban": "aciguatar",
    "aciguatabas": "aciguatar",
    "aciguatad": "aciguatar",
    "aciguatada": "aciguatar",
    "aciguatadas": "aciguatar",
    "aciguatado": "aciguatar",
    "aciguatados": "aciguatar",
    "aciguatamos": "aciguatar",
    "aciguatan": "aciguatar",
    "aciguatando": "aciguatar",
    "aciguatara": "aciguatar",
    "aciguatarais": "aciguatar",
    "aciguataran": "aciguatar",
    "aciguataras": "aciguatar",
    "aciguatare": "aciguatar",
    "aciguatareis": "aciguatar",
    "aciguataremos": "aciguatar",
    "aciguataren": "aciguatar",
    "aciguatares": "aciguatar",
    "aciguataron": "aciguatar",
    "aciguatará": "aciguatar",
    "aciguatarán": "aciguatar",
    "aciguatarás": "aciguatar",
    "aciguataré": "aciguatar",
    "aciguataréis": "aciguatar",
    "aciguataría": "aciguatar",
    "aciguataríais": "aciguatar",
    "aciguataríamos": "aciguatar",
    "aciguatarían": "aciguatar",
    "aciguatarías": "aciguatar",
    "aciguatas": "aciguatar",
    "aciguatase": "aciguatar",
    "aciguataseis": "aciguatar",
    "aciguatasen": "aciguatar",
    "aciguatases": "aciguatar",
    "aciguataste": "aciguatar",
    "aciguatasteis": "aciguatar",
    "aciguate": "aciguatar",
    "aciguatemos": "aciguatar",
    "aciguaten": "aciguatar",
    "aciguates": "aciguatar",
    "aciguato": "aciguatar",
    "aciguatábamos": "aciguatar",
    "aciguatáis": "aciguatar",
    "aciguatáramos": "aciguatar",
    "aciguatáremos": "aciguatar",
    "aciguatásemos": "aciguatar",
    "aciguaté": "aciguatar",
    "aciguatéis": "aciguatar",
    "aciguató": "aciguatar",
    "acilaciones": "acilación",
    "aciloinas": "aciloina",
    "acilos": "acilo",
    "acimutales": "acimutal",
    "acimutes": "acimut",
    "acinesias": "acinesia",
    "acinetobácteres": "acinetobácter",
    "acinetos": "acineto",
    "acinética": "acinético",
    "acinéticas": "acinético",
    "acinéticos": "acinético",
    "aciones": "ación",
    "acipenséridos": "acipensérido",
    "acirates": "acirate",
    "acitrones": "acitrón",
    "aclama": "aclamar",
    "aclamaba": "aclamar",
    "aclamabais": "aclamar",
    "aclamaban": "aclamar",
    "aclamabas": "aclamar",
    "aclamaciones": "aclamación",
    "aclamad": "aclamar",
    "aclamada": "aclamar",
    "aclamadas": "aclamar",
    "aclamado": "aclamar",
    "aclamados": "aclamar",
    "aclamamos": "aclamar",
    "aclaman": "aclamar",
    "aclamando": "aclamar",
    "aclamara": "aclamar",
    "aclamarais": "aclamar",
    "aclamaran": "aclamar",
    "aclamaras": "aclamar",
    "aclamare": "aclamar",
    "aclamareis": "aclamar",
    "aclamaremos": "aclamar",
    "aclamaren": "aclamar",
    "aclamares": "aclamar",
    "aclamaron": "aclamar",
    "aclamará": "aclamar",
    "aclamarán": "aclamar",
    "aclamarás": "aclamar",
    "aclamaré": "aclamar",
    "aclamaréis": "aclamar",
    "aclamaría": "aclamar",
    "aclamaríais": "aclamar",
    "aclamaríamos": "aclamar",
    "aclamarían": "aclamar",
    "aclamarías": "aclamar",
    "aclamas": "aclamar",
    "aclamase": "aclamar",
    "aclamaseis": "aclamar",
    "aclamasen": "aclamar",
    "aclamases": "aclamar",
    "aclamaste": "aclamar",
    "aclamasteis": "aclamar",
    "aclame": "aclamar",
    "aclamemos": "aclamar",
    "aclamen": "aclamar",
    "aclames": "aclamar",
    "aclamo": "aclamar",
    "aclamábamos": "aclamar",
    "aclamáis": "aclamar",
    "aclamáramos": "aclamar",
    "aclamáremos": "aclamar",
    "aclamásemos": "aclamar",
    "aclamé": "aclamar",
    "aclaméis": "aclamar",
    "aclamídea": "aclamídeo",
    "aclamídeas": "aclamídeo",
    "aclamídeos": "aclamídeo",
    "aclamó": "aclamar",
    "aclara": "aclarar",
    "aclaraba": "aclarar",
    "aclarabais": "aclarar",
    "aclaraban": "aclarar",
    "aclarabas": "aclarar",
    "aclaraciones": "aclaración",
    "aclarad": "aclarar",
    "aclarada": "aclarar",
    "aclaradas": "aclarar",
    "aclarado": "aclarar",
    "aclaradoras": "aclaradora",
    "aclarados": "aclarar",
    "aclaramientos": "aclaramiento",
    "aclaramos": "aclarar",
    "aclaran": "aclarar",
    "aclarando": "aclarar",
    "aclarara": "aclarar",
    "aclararais": "aclarar",
    "aclararan": "aclarar",
    "aclararas": "aclarar",
    "aclarare": "aclarar",
    "aclarareis": "aclarar",
    "aclararemos": "aclarar",
    "aclararen": "aclarar",
    "aclarares": "aclarar",
    "aclararon": "aclarar",
    "aclarará": "aclarar",
    "aclararán": "aclarar",
    "aclararás": "aclarar",
    "aclararé": "aclarar",
    "aclararéis": "aclarar",
    "aclararía": "aclarar",
    "aclararíais": "aclarar",
    "aclararíamos": "aclarar",
    "aclararían": "aclarar",
    "aclararías": "aclarar",
    "aclaras": "aclarar",
    "aclarase": "aclarar",
    "aclaraseis": "aclarar",
    "aclarasen": "aclarar",
    "aclarases": "aclarar",
    "aclaraste": "aclarar",
    "aclarasteis": "aclarar",
    "aclaratoria": "aclaratorio",
    "aclaratorias": "aclaratorio",
    "aclaratorios": "aclaratorio",
    "aclare": "aclarar",
    "aclaremos": "aclarar",
    "aclaren": "aclarar",
    "aclareos": "aclareo",
    "aclares": "aclarar",
    "aclaro": "aclarar",
    "aclarábamos": "aclarar",
    "aclaráis": "aclarar",
    "aclaráramos": "aclarar",
    "aclaráremos": "aclarar",
    "aclarásemos": "aclarar",
    "aclaré": "aclarar",
    "aclaréis": "aclarar",
    "aclaró": "aclarar",
    "aclavelada": "aclavelado",
    "aclaveladas": "aclavelado",
    "aclavelados": "aclavelado",
    "acleida": "acleido",
    "acleidas": "acleido",
    "acleidos": "acleido",
    "aclimata": "aclimatar",
    "aclimataba": "aclimatar",
    "aclimatabais": "aclimatar",
    "aclimataban": "aclimatar",
    "aclimatabas": "aclimatar",
    "aclimatables": "aclimatable",
    "aclimataciones": "aclimatación",
    "aclimatad": "aclimatar",
    "aclimatada": "aclimatar",
    "aclimatadas": "aclimatar",
    "aclimatado": "aclimatar",
    "aclimatados": "aclimatar",
    "aclimatamos": "aclimatar",
    "aclimatan": "aclimatar",
    "aclimatando": "aclimatar",
    "aclimatara": "aclimatar",
    "aclimatarais": "aclimatar",
    "aclimataran": "aclimatar",
    "aclimataras": "aclimatar",
    "aclimatare": "aclimatar",
    "aclimatareis": "aclimatar",
    "aclimataremos": "aclimatar",
    "aclimataren": "aclimatar",
    "aclimatares": "aclimatar",
    "aclimataron": "aclimatar",
    "aclimatará": "aclimatar",
    "aclimatarán": "aclimatar",
    "aclimatarás": "aclimatar",
    "aclimataré": "aclimatar",
    "aclimataréis": "aclimatar",
    "aclimataría": "aclimatar",
    "aclimataríais": "aclimatar",
    "aclimataríamos": "aclimatar",
    "aclimatarían": "aclimatar",
    "aclimatarías": "aclimatar",
    "aclimatas": "aclimatar",
    "aclimatase": "aclimatar",
    "aclimataseis": "aclimatar",
    "aclimatasen": "aclimatar",
    "aclimatases": "aclimatar",
    "aclimataste": "aclimatar",
    "aclimatasteis": "aclimatar",
    "aclimate": "aclimatar",
    "aclimatemos": "aclimatar",
    "aclimaten": "aclimatar",
    "aclimates": "aclimatar",
    "aclimatizaciones": "aclimatización",
    "aclimato": "aclimatar",
    "aclimatábamos": "aclimatar",
    "aclimatáis": "aclimatar",
    "aclimatáramos": "aclimatar",
    "aclimatáremos": "aclimatar",
    "aclimatásemos": "aclimatar",
    "aclimaté": "aclimatar",
    "aclimatéis": "aclimatar",
    "aclimató": "aclimatar",
    "aclocaba": "aclocar",
    "aclocabais": "aclocar",
    "aclocaban": "aclocar",
    "aclocabas": "aclocar",
    "aclocad": "aclocar",
    "aclocada": "aclocar",
    "aclocadas": "aclocar",
    "aclocado": "aclocar",
    "aclocados": "aclocar",
    "aclocamos": "aclocar",
    "aclocando": "aclocar",
    "aclocara": "aclocar",
    "aclocarais": "aclocar",
    "aclocaran": "aclocar",
    "aclocaras": "aclocar",
    "aclocare": "aclocar",
    "aclocareis": "aclocar",
    "aclocaremos": "aclocar",
    "aclocaren": "aclocar",
    "aclocares": "aclocar",
    "aclocaron": "aclocar",
    "aclocará": "aclocar",
    "aclocarán": "aclocar",
    "aclocarás": "aclocar",
    "aclocaré": "aclocar",
    "aclocaréis": "aclocar",
    "aclocaría": "aclocar",
    "aclocaríais": "aclocar",
    "aclocaríamos": "aclocar",
    "aclocarían": "aclocar",
    "aclocarías": "aclocar",
    "aclocase": "aclocar",
    "aclocaseis": "aclocar",
    "aclocasen": "aclocar",
    "aclocases": "aclocar",
    "aclocaste": "aclocar",
    "aclocasteis": "aclocar",
    "aclocábamos": "aclocar",
    "aclocáis": "aclocar",
    "aclocáramos": "aclocar",
    "aclocáremos": "aclocar",
    "aclocásemos": "aclocar",
    "aclocó": "aclocar",
    "acloquemos": "aclocar",
    "acloqué": "aclocar",
    "acloquéis": "aclocar",
    "aclorhidrias": "aclorhidria",
    "aclorhídrica": "aclorhídrico",
    "aclorhídricas": "aclorhídrico",
    "aclorhídricos": "aclorhídrico",
    "aclueca": "aclocar",
    "acluecan": "aclocar",
    "acluecas": "aclocar",
    "aclueco": "aclocar",
    "aclueque": "aclocar",
    "acluequen": "aclocar",
    "aclueques": "aclocar",
    "aclínica": "aclínico",
    "aclínicas": "aclínico",
    "aclínicos": "aclínico",
    "acmeísmos": "acmeísmo",
    "acmeístas": "acmeísta",
    "acmés": "acmé",
    "acnodos": "acnodo",
    "acnés": "acné",
    "acobarda": "acobardar",
    "acobardaba": "acobardar",
    "acobardabais": "acobardar",
    "acobardaban": "acobardar",
    "acobardabas": "acobardar",
    "acobardad": "acobardar",
    "acobardada": "acobardar",
    "acobardadas": "acobardar",
    "acobardado": "acobardar",
    "acobardados": "acobardar",
    "acobardamientos": "acobardamiento",
    "acobardamos": "acobardar",
    "acobardan": "acobardar",
    "acobardando": "acobardar",
    "acobardara": "acobardar",
    "acobardarais": "acobardar",
    "acobardaran": "acobardar",
    "acobardaras": "acobardar",
    "acobardare": "acobardar",
    "acobardareis": "acobardar",
    "acobardaremos": "acobardar",
    "acobardaren": "acobardar",
    "acobardares": "acobardar",
    "acobardaron": "acobardar",
    "acobardará": "acobardar",
    "acobardarán": "acobardar",
    "acobardarás": "acobardar",
    "acobardaré": "acobardar",
    "acobardaréis": "acobardar",
    "acobardaría": "acobardar",
    "acobardaríais": "acobardar",
    "acobardaríamos": "acobardar",
    "acobardarían": "acobardar",
    "acobardarías": "acobardar",
    "acobardas": "acobardar",
    "acobardase": "acobardar",
    "acobardaseis": "acobardar",
    "acobardasen": "acobardar",
    "acobardases": "acobardar",
    "acobardaste": "acobardar",
    "acobardasteis": "acobardar",
    "acobarde": "acobardar",
    "acobardemos": "acobardar",
    "acobarden": "acobardar",
    "acobardes": "acobardar",
    "acobardo": "acobardar",
    "acobardábamos": "acobardar",
    "acobardáis": "acobardar",
    "acobardáramos": "acobardar",
    "acobardáremos": "acobardar",
    "acobardásemos": "acobardar",
    "acobardé": "acobardar",
    "acobardéis": "acobardar",
    "acobardó": "acobardar",
    "acobijos": "acobijo",
    "acobrada": "acobrado",
    "acobradas": "acobrado",
    "acobrados": "acobrado",
    "acocea": "acocear",
    "acoceaba": "acocear",
    "acoceabais": "acocear",
    "acoceaban": "acocear",
    "acoceabas": "acocear",
    "acocead": "acocear",
    "acoceada": "acocear",
    "acoceadas": "acocear",
    "acoceado": "acocear",
    "acoceados": "acocear",
    "acoceamientos": "acoceamiento",
    "acoceamos": "acocear",
    "acocean": "acocear",
    "acoceando": "acocear",
    "acoceara": "acocear",
    "acocearais": "acocear",
    "acocearan": "acocear",
    "acocearas": "acocear",
    "acoceare": "acocear",
    "acoceareis": "acocear",
    "acocearemos": "acocear",
    "acocearen": "acocear",
    "acoceares": "acocear",
    "acocearon": "acocear",
    "acoceará": "acocear",
    "acocearán": "acocear",
    "acocearás": "acocear",
    "acocearé": "acocear",
    "acocearéis": "acocear",
    "acocearía": "acocear",
    "acocearíais": "acocear",
    "acocearíamos": "acocear",
    "acocearían": "acocear",
    "acocearías": "acocear",
    "acoceas": "acocear",
    "acocease": "acocear",
    "acoceaseis": "acocear",
    "acoceasen": "acocear",
    "acoceases": "acocear",
    "acoceaste": "acocear",
    "acoceasteis": "acocear",
    "acocee": "acocear",
    "acoceemos": "acocear",
    "acoceen": "acocear",
    "acocees": "acocear",
    "acoceo": "acocear",
    "acoceábamos": "acocear",
    "acoceáis": "acocear",
    "acoceáramos": "acocear",
    "acoceáremos": "acocear",
    "acoceásemos": "acocear",
    "acoceé": "acocear",
    "acoceéis": "acocear",
    "acoceó": "acocear",
    "acochambra": "acochambrar",
    "acochambraba": "acochambrar",
    "acochambrabais": "acochambrar",
    "acochambraban": "acochambrar",
    "acochambrabas": "acochambrar",
    "acochambrad": "acochambrar",
    "acochambrada": "acochambrar",
    "acochambradas": "acochambrar",
    "acochambrado": "acochambrar",
    "acochambrados": "acochambrar",
    "acochambramos": "acochambrar",
    "acochambran": "acochambrar",
    "acochambrando": "acochambrar",
    "acochambrara": "acochambrar",
    "acochambrarais": "acochambrar",
    "acochambraran": "acochambrar",
    "acochambraras": "acochambrar",
    "acochambrare": "acochambrar",
    "acochambrareis": "acochambrar",
    "acochambraremos": "acochambrar",
    "acochambraren": "acochambrar",
    "acochambrares": "acochambrar",
    "acochambraron": "acochambrar",
    "acochambrará": "acochambrar",
    "acochambrarán": "acochambrar",
    "acochambrarás": "acochambrar",
    "acochambraré": "acochambrar",
    "acochambraréis": "acochambrar",
    "acochambraría": "acochambrar",
    "acochambraríais": "acochambrar",
    "acochambraríamos": "acochambrar",
    "acochambrarían": "acochambrar",
    "acochambrarías": "acochambrar",
    "acochambras": "acochambrar",
    "acochambrase": "acochambrar",
    "acochambraseis": "acochambrar",
    "acochambrasen": "acochambrar",
    "acochambrases": "acochambrar",
    "acochambraste": "acochambrar",
    "acochambrasteis": "acochambrar",
    "acochambre": "acochambrar",
    "acochambremos": "acochambrar",
    "acochambren": "acochambrar",
    "acochambres": "acochambrar",
    "acochambro": "acochambrar",
    "acochambrábamos": "acochambrar",
    "acochambráis": "acochambrar",
    "acochambráramos": "acochambrar",
    "acochambráremos": "acochambrar",
    "acochambrásemos": "acochambrar",
    "acochambré": "acochambrar",
    "acochambréis": "acochambrar",
    "acochambró": "acochambrar",
    "acochina": "acochinar",
    "acochinaba": "acochinar",
    "acochinabais": "acochinar",
    "acochinaban": "acochinar",
    "acochinabas": "acochinar",
    "acochinad": "acochinar",
    "acochinada": "acochinar",
    "acochinadas": "acochinar",
    "acochinado": "acochinar",
    "acochinados": "acochinar",
    "acochinamos": "acochinar",
    "acochinan": "acochinar",
    "acochinando": "acochinar",
    "acochinara": "acochinar",
    "acochinarais": "acochinar",
    "acochinaran": "acochinar",
    "acochinaras": "acochinar",
    "acochinare": "acochinar",
    "acochinareis": "acochinar",
    "acochinaremos": "acochinar",
    "acochinaren": "acochinar",
    "acochinares": "acochinar",
    "acochinaron": "acochinar",
    "acochinará": "acochinar",
    "acochinarán": "acochinar",
    "acochinarás": "acochinar",
    "acochinaré": "acochinar",
    "acochinaréis": "acochinar",
    "acochinaría": "acochinar",
    "acochinaríais": "acochinar",
    "acochinaríamos": "acochinar",
    "acochinarían": "acochinar",
    "acochinarías": "acochinar",
    "acochinas": "acochinar",
    "acochinase": "acochinar",
    "acochinaseis": "acochinar",
    "acochinasen": "acochinar",
    "acochinases": "acochinar",
    "acochinaste": "acochinar",
    "acochinasteis": "acochinar",
    "acochine": "acochinar",
    "acochinemos": "acochinar",
    "acochinen": "acochinar",
    "acochines": "acochinar",
    "acochino": "acochinar",
    "acochinábamos": "acochinar",
    "acochináis": "acochinar",
    "acochináramos": "acochinar",
    "acochináremos": "acochinar",
    "acochinásemos": "acochinar",
    "acochiné": "acochinar",
    "acochinéis": "acochinar",
    "acochinó": "acochinar",
    "acoda": "acodar",
    "acodaba": "acodar",
    "acodabais": "acodar",
    "acodaban": "acodar",
    "acodabas": "acodar",
    "acodad": "acodar",
    "acodada": "acodar",
    "acodadas": "acodar",
    "acodaderas": "acodadera",
    "acodado": "acodar",
    "acodados": "acodar",
    "acodaduras": "acodadura",
    "acodala": "acodalar",
    "acodalaba": "acodalar",
    "acodalabais": "acodalar",
    "acodalaban": "acodalar",
    "acodalabas": "acodalar",
    "acodalad": "acodalar",
    "acodalada": "acodalar",
    "acodaladas": "acodalar",
    "acodalado": "acodalar",
    "acodalados": "acodalar",
    "acodalamos": "acodalar",
    "acodalan": "acodalar",
    "acodalando": "acodalar",
    "acodalara": "acodalar",
    "acodalarais": "acodalar",
    "acodalaran": "acodalar",
    "acodalaras": "acodalar",
    "acodalare": "acodalar",
    "acodalareis": "acodalar",
    "acodalaremos": "acodalar",
    "acodalaren": "acodalar",
    "acodalares": "acodalar",
    "acodalaron": "acodalar",
    "acodalará": "acodalar",
    "acodalarán": "acodalar",
    "acodalarás": "acodalar",
    "acodalaré": "acodalar",
    "acodalaréis": "acodalar",
    "acodalaría": "acodalar",
    "acodalaríais": "acodalar",
    "acodalaríamos": "acodalar",
    "acodalarían": "acodalar",
    "acodalarías": "acodalar",
    "acodalas": "acodalar",
    "acodalase": "acodalar",
    "acodalaseis": "acodalar",
    "acodalasen": "acodalar",
    "acodalases": "acodalar",
    "acodalaste": "acodalar",
    "acodalasteis": "acodalar",
    "acodale": "acodalar",
    "acodalemos": "acodalar",
    "acodalen": "acodalar",
    "acodales": "acodalar",
    "acodalo": "acodalar",
    "acodalábamos": "acodalar",
    "acodaláis": "acodalar",
    "acodaláramos": "acodalar",
    "acodaláremos": "acodalar",
    "acodalásemos": "acodalar",
    "acodalé": "acodalar",
    "acodaléis": "acodalar",
    "acodaló": "acodalar",
    "acodamientos": "acodamiento",
    "acodamos": "acodar",
    "acodan": "acodar",
    "acodando": "acodar",
    "acodara": "acodar",
    "acodarais": "acodar",
    "acodaran": "acodar",
    "acodaras": "acodar",
    "acodare": "acodar",
    "acodareis": "acodar",
    "acodaremos": "acodar",
    "acodaren": "acodar",
    "acodares": "acodar",
    "acodaron": "acodar",
    "acodará": "acodar",
    "acodarán": "acodar",
    "acodarás": "acodar",
    "acodaré": "acodar",
    "acodaréis": "acodar",
    "acodaría": "acodar",
    "acodaríais": "acodar",
    "acodaríamos": "acodar",
    "acodarían": "acodar",
    "acodarías": "acodar",
    "acodas": "acodar",
    "acodase": "acodar",
    "acodaseis": "acodar",
    "acodasen": "acodar",
    "acodases": "acodar",
    "acodaste": "acodar",
    "acodasteis": "acodar",
    "acode": "acodar",
    "acodemos": "acodar",
    "acoden": "acodar",
    "acodera": "acoderar",
    "acoderaba": "acoderar",
    "acoderabais": "acoderar",
    "acoderaban": "acoderar",
    "acoderabas": "acoderar",
    "acoderad": "acoderar",
    "acoderada": "acoderar",
    "acoderadas": "acoderar",
    "acoderado": "acoderar",
    "acoderados": "acoderar",
    "acoderamientos": "acoderamiento",
    "acoderamos": "acoderar",
    "acoderan": "acoderar",
    "acoderando": "acoderar",
    "acoderara": "acoderar",
    "acoderarais": "acoderar",
    "acoderaran": "acoderar",
    "acoderaras": "acoderar",
    "acoderare": "acoderar",
    "acoderareis": "acoderar",
    "acoderaremos": "acoderar",
    "acoderaren": "acoderar",
    "acoderares": "acoderar",
    "acoderaron": "acoderar",
    "acoderará": "acoderar",
    "acoderarán": "acoderar",
    "acoderarás": "acoderar",
    "acoderaré": "acoderar",
    "acoderaréis": "acoderar",
    "acoderaría": "acoderar",
    "acoderaríais": "acoderar",
    "acoderaríamos": "acoderar",
    "acoderarían": "acoderar",
    "acoderarías": "acoderar",
    "acoderas": "acoderar",
    "acoderase": "acoderar",
    "acoderaseis": "acoderar",
    "acoderasen": "acoderar",
    "acoderases": "acoderar",
    "acoderaste": "acoderar",
    "acoderasteis": "acoderar",
    "acodere": "acoderar",
    "acoderemos": "acoderar",
    "acoderen": "acoderar",
    "acoderes": "acoderar",
    "acodero": "acoderar",
    "acoderábamos": "acoderar",
    "acoderáis": "acoderar",
    "acoderáramos": "acoderar",
    "acoderáremos": "acoderar",
    "acoderásemos": "acoderar",
    "acoderé": "acoderar",
    "acoderéis": "acoderar",
    "acoderó": "acoderar",
    "acodes": "acodar",
    "acodilla": "acodillar",
    "acodillaba": "acodillar",
    "acodillabais": "acodillar",
    "acodillaban": "acodillar",
    "acodillabas": "acodillar",
    "acodillad": "acodillar",
    "acodillada": "acodillar",
    "acodilladas": "acodillar",
    "acodillado": "acodillar",
    "acodillados": "acodillar",
    "acodillamos": "acodillar",
    "acodillan": "acodillar",
    "acodillando": "acodillar",
    "acodillara": "acodillar",
    "acodillarais": "acodillar",
    "acodillaran": "acodillar",
    "acodillaras": "acodillar",
    "acodillare": "acodillar",
    "acodillareis": "acodillar",
    "acodillaremos": "acodillar",
    "acodillaren": "acodillar",
    "acodillares": "acodillar",
    "acodillaron": "acodillar",
    "acodillará": "acodillar",
    "acodillarán": "acodillar",
    "acodillarás": "acodillar",
    "acodillaré": "acodillar",
    "acodillaréis": "acodillar",
    "acodillaría": "acodillar",
    "acodillaríais": "acodillar",
    "acodillaríamos": "acodillar",
    "acodillarían": "acodillar",
    "acodillarías": "acodillar",
    "acodillas": "acodillar",
    "acodillase": "acodillar",
    "acodillaseis": "acodillar",
    "acodillasen": "acodillar",
    "acodillases": "acodillar",
    "acodillaste": "acodillar",
    "acodillasteis": "acodillar",
    "acodille": "acodillar",
    "acodillemos": "acodillar",
    "acodillen": "acodillar",
    "acodilles": "acodillar",
    "acodillo": "acodillar",
    "acodillábamos": "acodillar",
    "acodilláis": "acodillar",
    "acodilláramos": "acodillar",
    "acodilláremos": "acodillar",
    "acodillásemos": "acodillar",
    "acodillé": "acodillar",
    "acodilléis": "acodillar",
    "acodilló": "acodillar",
    "acodo": "acodar",
    "acodos": "acodo",
    "acodábamos": "acodar",
    "acodáis": "acodar",
    "acodáramos": "acodar",
    "acodáremos": "acodar",
    "acodásemos": "acodar",
    "acodé": "acodar",
    "acodéis": "acodar",
    "acodó": "acodar",
    "acoge": "acoger",
    "acoged": "acoger",
    "acogedora": "acogedor",
    "acogedoras": "acogedor",
    "acogedores": "acogedor",
    "acogemos": "acoger",
    "acogen": "acoger",
    "acogeremos": "acoger",
    "acogerá": "acoger",
    "acogerán": "acoger",
    "acogerás": "acoger",
    "acogeré": "acoger",
    "acogeréis": "acoger",
    "acogería": "acoger",
    "acogeríais": "acoger",
    "acogeríamos": "acoger",
    "acogerían": "acoger",
    "acogerías": "acoger",
    "acoges": "acoger",
    "acogida": "acoger",
    "acogidas": "acogida",
    "acogido": "acoger",
    "acogidos": "acogido",
    "acogiendo": "acoger",
    "acogiera": "acoger",
    "acogierais": "acoger",
    "acogieran": "acoger",
    "acogieras": "acoger",
    "acogiere": "acoger",
    "acogiereis": "acoger",
    "acogieren": "acoger",
    "acogieres": "acoger",
    "acogieron": "acoger",
    "acogiese": "acoger",
    "acogieseis": "acoger",
    "acogiesen": "acoger",
    "acogieses": "acoger",
    "acogimientos": "acogimiento",
    "acogimos": "acoger",
    "acogiste": "acoger",
    "acogisteis": "acoger",
    "acogiéramos": "acoger",
    "acogiéremos": "acoger",
    "acogiésemos": "acoger",
    "acogió": "acoger",
    "acogolla": "acogollar",
    "acogollaba": "acogollar",
    "acogollabais": "acogollar",
    "acogollaban": "acogollar",
    "acogollabas": "acogollar",
    "acogollad": "acogollar",
    "acogollada": "acogollar",
    "acogolladas": "acogollar",
    "acogollado": "acogollar",
    "acogollados": "acogollar",
    "acogollamos": "acogollar",
    "acogollan": "acogollar",
    "acogollando": "acogollar",
    "acogollara": "acogollar",
    "acogollarais": "acogollar",
    "acogollaran": "acogollar",
    "acogollaras": "acogollar",
    "acogollare": "acogollar",
    "acogollareis": "acogollar",
    "acogollaremos": "acogollar",
    "acogollaren": "acogollar",
    "acogollares": "acogollar",
    "acogollaron": "acogollar",
    "acogollará": "acogollar",
    "acogollarán": "acogollar",
    "acogollarás": "acogollar",
    "acogollaré": "acogollar",
    "acogollaréis": "acogollar",
    "acogollaría": "acogollar",
    "acogollaríais": "acogollar",
    "acogollaríamos": "acogollar",
    "acogollarían": "acogollar",
    "acogollarías": "acogollar",
    "acogollas": "acogollar",
    "acogollase": "acogollar",
    "acogollaseis": "acogollar",
    "acogollasen": "acogollar",
    "acogollases": "acogollar",
    "acogollaste": "acogollar",
    "acogollasteis": "acogollar",
    "acogolle": "acogollar",
    "acogollemos": "acogollar",
    "acogollen": "acogollar",
    "acogolles": "acogollar",
    "acogollo": "acogollar",
    "acogollábamos": "acogollar",
    "acogolláis": "acogollar",
    "acogolláramos": "acogollar",
    "acogolláremos": "acogollar",
    "acogollásemos": "acogollar",
    "acogollé": "acogollar",
    "acogolléis": "acogollar",
    "acogolló": "acogollar",
    "acogombra": "acogombrar",
    "acogombraba": "acogombrar",
    "acogombrabais": "acogombrar",
    "acogombraban": "acogombrar",
    "acogombrabas": "acogombrar",
    "acogombrad": "acogombrar",
    "acogombrada": "acogombrar",
    "acogombradas": "acogombrar",
    "acogombrado": "acogombrar",
    "acogombrados": "acogombrar",
    "acogombramos": "acogombrar",
    "acogombran": "acogombrar",
    "acogombrando": "acogombrar",
    "acogombrara": "acogombrar",
    "acogombrarais": "acogombrar",
    "acogombraran": "acogombrar",
    "acogombraras": "acogombrar",
    "acogombrare": "acogombrar",
    "acogombrareis": "acogombrar",
    "acogombraremos": "acogombrar",
    "acogombraren": "acogombrar",
    "acogombrares": "acogombrar",
    "acogombraron": "acogombrar",
    "acogombrará": "acogombrar",
    "acogombrarán": "acogombrar",
    "acogombrarás": "acogombrar",
    "acogombraré": "acogombrar",
    "acogombraréis": "acogombrar",
    "acogombraría": "acogombrar",
    "acogombraríais": "acogombrar",
    "acogombraríamos": "acogombrar",
    "acogombrarían": "acogombrar",
    "acogombrarías": "acogombrar",
    "acogombras": "acogombrar",
    "acogombrase": "acogombrar",
    "acogombraseis": "acogombrar",
    "acogombrasen": "acogombrar",
    "acogombrases": "acogombrar",
    "acogombraste": "acogombrar",
    "acogombrasteis": "acogombrar",
    "acogombre": "acogombrar",
    "acogombremos": "acogombrar",
    "acogombren": "acogombrar",
    "acogombres": "acogombrar",
    "acogombro": "acogombrar",
    "acogombrábamos": "acogombrar",
    "acogombráis": "acogombrar",
    "acogombráramos": "acogombrar",
    "acogombráremos": "acogombrar",
    "acogombrásemos": "acogombrar",
    "acogombré": "acogombrar",
    "acogombréis": "acogombrar",
    "acogombró": "acogombrar",
    "acogota": "acogotar",
    "acogotaba": "acogotar",
    "acogotabais": "acogotar",
    "acogotaban": "acogotar",
    "acogotabas": "acogotar",
    "acogotad": "acogotar",
    "acogotada": "acogotar",
    "acogotadas": "acogotar",
    "acogotado": "acogotar",
    "acogotados": "acogotar",
    "acogotamos": "acogotar",
    "acogotan": "acogotar",
    "acogotando": "acogotar",
    "acogotara": "acogotar",
    "acogotarais": "acogotar",
    "acogotaran": "acogotar",
    "acogotaras": "acogotar",
    "acogotare": "acogotar",
    "acogotareis": "acogotar",
    "acogotaremos": "acogotar",
    "acogotaren": "acogotar",
    "acogotares": "acogotar",
    "acogotaron": "acogotar",
    "acogotará": "acogotar",
    "acogotarán": "acogotar",
    "acogotarás": "acogotar",
    "acogotaré": "acogotar",
    "acogotaréis": "acogotar",
    "acogotaría": "acogotar",
    "acogotaríais": "acogotar",
    "acogotaríamos": "acogotar",
    "acogotarían": "acogotar",
    "acogotarías": "acogotar",
    "acogotas": "acogotar",
    "acogotase": "acogotar",
    "acogotaseis": "acogotar",
    "acogotasen": "acogotar",
    "acogotases": "acogotar",
    "acogotaste": "acogotar",
    "acogotasteis": "acogotar",
    "acogote": "acogotar",
    "acogotemos": "acogotar",
    "acogoten": "acogotar",
    "acogotes": "acogotar",
    "acogoto": "acogotar",
    "acogotábamos": "acogotar",
    "acogotáis": "acogotar",
    "acogotáramos": "acogotar",
    "acogotáremos": "acogotar",
    "acogotásemos": "acogotar",
    "acogoté": "acogotar",
    "acogotéis": "acogotar",
    "acogotó": "acogotar",
    "acogéis": "acoger",
    "acogí": "acoger",
    "acogía": "acoger",
    "acogíais": "acoger",
    "acogíamos": "acoger",
    "acogían": "acoger",
    "acogías": "acoger",
    "acohombra": "acohombrar",
    "acohombraba": "acohombrar",
    "acohombrabais": "acohombrar",
    "acohombraban": "acohombrar",
    "acohombrabas": "acohombrar",
    "acohombrad": "acohombrar",
    "acohombrada": "acohombrar",
    "acohombradas": "acohombrar",
    "acohombrado": "acohombrar",
    "acohombrados": "acohombrar",
    "acohombramos": "acohombrar",
    "acohombran": "acohombrar",
    "acohombrando": "acohombrar",
    "acohombrara": "acohombrar",
    "acohombrarais": "acohombrar",
    "acohombraran": "acohombrar",
    "acohombraras": "acohombrar",
    "acohombrare": "acohombrar",
    "acohombrareis": "acohombrar",
    "acohombraremos": "acohombrar",
    "acohombraren": "acohombrar",
    "acohombrares": "acohombrar",
    "acohombraron": "acohombrar",
    "acohombrará": "acohombrar",
    "acohombrarán": "acohombrar",
    "acohombrarás": "acohombrar",
    "acohombraré": "acohombrar",
    "acohombraréis": "acohombrar",
    "acohombraría": "acohombrar",
    "acohombraríais": "acohombrar",
    "acohombraríamos": "acohombrar",
    "acohombrarían": "acohombrar",
    "acohombrarías": "acohombrar",
    "acohombras": "acohombrar",
    "acohombrase": "acohombrar",
    "acohombraseis": "acohombrar",
    "acohombrasen": "acohombrar",
    "acohombrases": "acohombrar",
    "acohombraste": "acohombrar",
    "acohombrasteis": "acohombrar",
    "acohombre": "acohombrar",
    "acohombremos": "acohombrar",
    "acohombren": "acohombrar",
    "acohombres": "acohombrar",
    "acohombro": "acohombrar",
    "acohombrábamos": "acohombrar",
    "acohombráis": "acohombrar",
    "acohombráramos": "acohombrar",
    "acohombráremos": "acohombrar",
    "acohombrásemos": "acohombrar",
    "acohombré": "acohombrar",
    "acohombréis": "acohombrar",
    "acohombró": "acohombrar",
    "acoita": "acoitar",
    "acoitaba": "acoitar",
    "acoitabais": "acoitar",
    "acoitaban": "acoitar",
    "acoitabas": "acoitar",
    "acoitad": "acoitar",
    "acoitada": "acoitar",
    "acoitadas": "acoitar",
    "acoitado": "acoitar",
    "acoitados": "acoitar",
    "acoitamos": "acoitar",
    "acoitan": "acoitar",
    "acoitando": "acoitar",
    "acoitara": "acoitar",
    "acoitarais": "acoitar",
    "acoitaran": "acoitar",
    "acoitaras": "acoitar",
    "acoitare": "acoitar",
    "acoitareis": "acoitar",
    "acoitaremos": "acoitar",
    "acoitaren": "acoitar",
    "acoitares": "acoitar",
    "acoitaron": "acoitar",
    "acoitará": "acoitar",
    "acoitarán": "acoitar",
    "acoitarás": "acoitar",
    "acoitaré": "acoitar",
    "acoitaréis": "acoitar",
    "acoitaría": "acoitar",
    "acoitaríais": "acoitar",
    "acoitaríamos": "acoitar",
    "acoitarían": "acoitar",
    "acoitarías": "acoitar",
    "acoitas": "acoitar",
    "acoitase": "acoitar",
    "acoitaseis": "acoitar",
    "acoitasen": "acoitar",
    "acoitases": "acoitar",
    "acoitaste": "acoitar",
    "acoitasteis": "acoitar",
    "acoite": "acoitar",
    "acoitemos": "acoitar",
    "acoiten": "acoitar",
    "acoites": "acoitar",
    "acoito": "acoitar",
    "acoitábamos": "acoitar",
    "acoitáis": "acoitar",
    "acoitáramos": "acoitar",
    "acoitáremos": "acoitar",
    "acoitásemos": "acoitar",
    "acoité": "acoitar",
    "acoitéis": "acoitar",
    "acoitó": "acoitar",
    "acoja": "acoger",
    "acojamos": "acoger",
    "acojan": "acoger",
    "acojas": "acoger",
    "acojidas": "acojida",
    "acojina": "acojinar",
    "acojinaba": "acojinar",
    "acojinabais": "acojinar",
    "acojinaban": "acojinar",
    "acojinabas": "acojinar",
    "acojinad": "acojinar",
    "acojinada": "acojinar",
    "acojinadas": "acojinar",
    "acojinado": "acojinar",
    "acojinados": "acojinar",
    "acojinamientos": "acojinamiento",
    "acojinamos": "acojinar",
    "acojinan": "acojinar",
    "acojinando": "acojinar",
    "acojinara": "acojinar",
    "acojinarais": "acojinar",
    "acojinaran": "acojinar",
    "acojinaras": "acojinar",
    "acojinare": "acojinar",
    "acojinareis": "acojinar",
    "acojinaremos": "acojinar",
    "acojinaren": "acojinar",
    "acojinares": "acojinar",
    "acojinaron": "acojinar",
    "acojinará": "acojinar",
    "acojinarán": "acojinar",
    "acojinarás": "acojinar",
    "acojinaré": "acojinar",
    "acojinaréis": "acojinar",
    "acojinaría": "acojinar",
    "acojinaríais": "acojinar",
    "acojinaríamos": "acojinar",
    "acojinarían": "acojinar",
    "acojinarías": "acojinar",
    "acojinas": "acojinar",
    "acojinase": "acojinar",
    "acojinaseis": "acojinar",
    "acojinasen": "acojinar",
    "acojinases": "acojinar",
    "acojinaste": "acojinar",
    "acojinasteis": "acojinar",
    "acojine": "acojinar",
    "acojinemos": "acojinar",
    "acojinen": "acojinar",
    "acojines": "acojinar",
    "acojino": "acojinar",
    "acojinábamos": "acojinar",
    "acojináis": "acojinar",
    "acojináramos": "acojinar",
    "acojináremos": "acojinar",
    "acojinásemos": "acojinar",
    "acojiné": "acojinar",
    "acojinéis": "acojinar",
    "acojinó": "acojinar",
    "acojo": "acoger",
    "acojona": "acojonar",
    "acojonaba": "acojonar",
    "acojonabais": "acojonar",
    "acojonaban": "acojonar",
    "acojonabas": "acojonar",
    "acojonad": "acojonar",
    "acojonada": "acojonar",
    "acojonadas": "acojonar",
    "acojonado": "acojonar",
    "acojonados": "acojonar",
    "acojonamos": "acojonar",
    "acojonan": "acojonar",
    "acojonando": "acojonar",
    "acojonantes": "acojonante",
    "acojonara": "acojonar",
    "acojonarais": "acojonar",
    "acojonaran": "acojonar",
    "acojonaras": "acojonar",
    "acojonare": "acojonar",
    "acojonareis": "acojonar",
    "acojonaremos": "acojonar",
    "acojonaren": "acojonar",
    "acojonares": "acojonar",
    "acojonaron": "acojonar",
    "acojonará": "acojonar",
    "acojonarán": "acojonar",
    "acojonarás": "acojonar",
    "acojonaré": "acojonar",
    "acojonaréis": "acojonar",
    "acojonaría": "acojonar",
    "acojonaríais": "acojonar",
    "acojonaríamos": "acojonar",
    "acojonarían": "acojonar",
    "acojonarías": "acojonar",
    "acojonas": "acojonar",
    "acojonase": "acojonar",
    "acojonaseis": "acojonar",
    "acojonasen": "acojonar",
    "acojonases": "acojonar",
    "acojonaste": "acojonar",
    "acojonasteis": "acojonar",
    "acojone": "acojonar",
    "acojonemos": "acojonar",
    "acojonen": "acojonar",
    "acojones": "acojonar",
    "acojono": "acojonar",
    "acojonábamos": "acojonar",
    "acojonáis": "acojonar",
    "acojonáramos": "acojonar",
    "acojonáremos": "acojonar",
    "acojonásemos": "acojonar",
    "acojoné": "acojonar",
    "acojonéis": "acojonar",
    "acojonó": "acojonar",
    "acojáis": "acoger",
    "acola": "acolar",
    "acolaba": "acolar",
    "acolabais": "acolar",
    "acolaban": "acolar",
    "acolabas": "acolar",
    "acolad": "acolar",
    "acolada": "acolar",
    "acoladas": "acolar",
    "acolado": "acolar",
    "acolados": "acolar",
    "acolamos": "acolar",
    "acolan": "acolar",
    "acolando": "acolar",
    "acolara": "acolar",
    "acolarais": "acolar",
    "acolaran": "acolar",
    "acolaras": "acolar",
    "acolare": "acolar",
    "acolareis": "acolar",
    "acolaremos": "acolar",
    "acolaren": "acolar",
    "acolares": "acolar",
    "acolaron": "acolar",
    "acolará": "acolar",
    "acolarán": "acolar",
    "acolarás": "acolar",
    "acolaré": "acolar",
    "acolaréis": "acolar",
    "acolaría": "acolar",
    "acolaríais": "acolar",
    "acolaríamos": "acolar",
    "acolarían": "acolar",
    "acolarías": "acolar",
    "acolas": "acolar",
    "acolase": "acolar",
    "acolaseis": "acolar",
    "acolasen": "acolar",
    "acolases": "acolar",
    "acolaste": "acolar",
    "acolasteis": "acolar",
    "acolcha": "acolchar",
    "acolchaba": "acolchar",
    "acolchabais": "acolchar",
    "acolchaban": "acolchar",
    "acolchabas": "acolchar",
    "acolchad": "acolchar",
    "acolchada": "acolchar",
    "acolchadas": "acolchar",
    "acolchado": "acolchar",
    "acolchados": "acolchar",
    "acolchamientos": "acolchamiento",
    "acolchamos": "acolchar",
    "acolchan": "acolchar",
    "acolchando": "acolchar",
    "acolchara": "acolchar",
    "acolcharais": "acolchar",
    "acolcharan": "acolchar",
    "acolcharas": "acolchar",
    "acolchare": "acolchar",
    "acolchareis": "acolchar",
    "acolcharemos": "acolchar",
    "acolcharen": "acolchar",
    "acolchares": "acolchar",
    "acolcharon": "acolchar",
    "acolchará": "acolchar",
    "acolcharán": "acolchar",
    "acolcharás": "acolchar",
    "acolcharé": "acolchar",
    "acolcharéis": "acolchar",
    "acolcharía": "acolchar",
    "acolcharíais": "acolchar",
    "acolcharíamos": "acolchar",
    "acolcharían": "acolchar",
    "acolcharías": "acolchar",
    "acolchas": "acolchar",
    "acolchase": "acolchar",
    "acolchaseis": "acolchar",
    "acolchasen": "acolchar",
    "acolchases": "acolchar",
    "acolchaste": "acolchar",
    "acolchasteis": "acolchar",
    "acolche": "acolchar",
    "acolchemos": "acolchar",
    "acolchen": "acolchar",
    "acolches": "acolchar",
    "acolchichis": "acolchichi",
    "acolchis": "acolchi",
    "acolcho": "acolchar",
    "acolchona": "acolchonar",
    "acolchonaba": "acolchonar",
    "acolchonabais": "acolchonar",
    "acolchonaban": "acolchonar",
    "acolchonabas": "acolchonar",
    "acolchonad": "acolchonar",
    "acolchonada": "acolchonar",
    "acolchonadas": "acolchonar",
    "acolchonado": "acolchonar",
    "acolchonados": "acolchonar",
    "acolchonamos": "acolchonar",
    "acolchonan": "acolchonar",
    "acolchonando": "acolchonar",
    "acolchonara": "acolchonar",
    "acolchonarais": "acolchonar",
    "acolchonaran": "acolchonar",
    "acolchonaras": "acolchonar",
    "acolchonare": "acolchonar",
    "acolchonareis": "acolchonar",
    "acolchonaremos": "acolchonar",
    "acolchonaren": "acolchonar",
    "acolchonares": "acolchonar",
    "acolchonaron": "acolchonar",
    "acolchonará": "acolchonar",
    "acolchonarán": "acolchonar",
    "acolchonarás": "acolchonar",
    "acolchonaré": "acolchonar",
    "acolchonaréis": "acolchonar",
    "acolchonaría": "acolchonar",
    "acolchonaríais": "acolchonar",
    "acolchonaríamos": "acolchonar",
    "acolchonarían": "acolchonar",
    "acolchonarías": "acolchonar",
    "acolchonas": "acolchonar",
    "acolchonase": "acolchonar",
    "acolchonaseis": "acolchonar",
    "acolchonasen": "acolchonar",
    "acolchonases": "acolchonar",
    "acolchonaste": "acolchonar",
    "acolchonasteis": "acolchonar",
    "acolchone": "acolchonar",
    "acolchonemos": "acolchonar",
    "acolchonen": "acolchonar",
    "acolchones": "acolchonar",
    "acolchono": "acolchonar",
    "acolchonábamos": "acolchonar",
    "acolchonáis": "acolchonar",
    "acolchonáramos": "acolchonar",
    "acolchonáremos": "acolchonar",
    "acolchonásemos": "acolchonar",
    "acolchoné": "acolchonar",
    "acolchonéis": "acolchonar",
    "acolchonó": "acolchonar",
    "acolchábamos": "acolchar",
    "acolcháis": "acolchar",
    "acolcháramos": "acolchar",
    "acolcháremos": "acolchar",
    "acolchásemos": "acolchar",
    "acolché": "acolchar",
    "acolchéis": "acolchar",
    "acolchó": "acolchar",
    "acole": "acolar",
    "acolemos": "acolar",
    "acolen": "acolar",
    "acoles": "acolar",
    "acolhuas": "acolhua",
    "acolita": "acolitar",
    "acolitaba": "acolitar",
    "acolitabais": "acolitar",
    "acolitaban": "acolitar",
    "acolitabas": "acolitar",
    "acolitad": "acolitar",
    "acolitada": "acolitar",
    "acolitadas": "acolitar",
    "acolitado": "acolitar",
    "acolitados": "acolitar",
    "acolitamos": "acolitar",
    "acolitan": "acolitar",
    "acolitando": "acolitar",
    "acolitara": "acolitar",
    "acolitarais": "acolitar",
    "acolitaran": "acolitar",
    "acolitaras": "acolitar",
    "acolitare": "acolitar",
    "acolitareis": "acolitar",
    "acolitaremos": "acolitar",
    "acolitaren": "acolitar",
    "acolitares": "acolitar",
    "acolitaron": "acolitar",
    "acolitará": "acolitar",
    "acolitarán": "acolitar",
    "acolitarás": "acolitar",
    "acolitaré": "acolitar",
    "acolitaréis": "acolitar",
    "acolitaría": "acolitar",
    "acolitaríais": "acolitar",
    "acolitaríamos": "acolitar",
    "acolitarían": "acolitar",
    "acolitarías": "acolitar",
    "acolitas": "acolitar",
    "acolitase": "acolitar",
    "acolitaseis": "acolitar",
    "acolitasen": "acolitar",
    "acolitases": "acolitar",
    "acolitaste": "acolitar",
    "acolitasteis": "acolitar",
    "acolite": "acolitar",
    "acolitemos": "acolitar",
    "acoliten": "acolitar",
    "acolites": "acolitar",
    "acolito": "acolitar",
    "acolitábamos": "acolitar",
    "acolitáis": "acolitar",
    "acolitáramos": "acolitar",
    "acolitáremos": "acolitar",
    "acolitásemos": "acolitar",
    "acolité": "acolitar",
    "acolitéis": "acolitar",
    "acolitó": "acolitar",
    "acollaba": "acollar",
    "acollabais": "acollar",
    "acollaban": "acollar",
    "acollabas": "acollar",
    "acollad": "acollar",
    "acollada": "acollar",
    "acolladas": "acollar",
    "acollado": "acollar",
    "acolladora": "acollador",
    "acolladoras": "acollador",
    "acolladores": "acollador",
    "acollados": "acollar",
    "acolladuras": "acolladura",
    "acollamos": "acollar",
    "acollando": "acollar",
    "acollara": "acollarar",
    "acollaraba": "acollarar",
    "acollarabais": "acollarar",
    "acollaraban": "acollarar",
    "acollarabas": "acollarar",
    "acollarad": "acollarar",
    "acollarada": "acollarar",
    "acollaradas": "acollarar",
    "acollarado": "acollarar",
    "acollarados": "acollarar",
    "acollarais": "acollar",
    "acollaramos": "acollarar",
    "acollaran": "acollarar",
    "acollarando": "acollarar",
    "acollarara": "acollarar",
    "acollararais": "acollarar",
    "acollararan": "acollarar",
    "acollararas": "acollarar",
    "acollarare": "acollarar",
    "acollarareis": "acollarar",
    "acollararemos": "acollarar",
    "acollararen": "acollarar",
    "acollarares": "acollarar",
    "acollararon": "acollarar",
    "acollarará": "acollarar",
    "acollararán": "acollarar",
    "acollararás": "acollarar",
    "acollararé": "acollarar",
    "acollararéis": "acollarar",
    "acollararía": "acollarar",
    "acollararíais": "acollarar",
    "acollararíamos": "acollarar",
    "acollararían": "acollarar",
    "acollararías": "acollarar",
    "acollaras": "acollarar",
    "acollarase": "acollarar",
    "acollaraseis": "acollarar",
    "acollarasen": "acollarar",
    "acollarases": "acollarar",
    "acollaraste": "acollarar",
    "acollarasteis": "acollarar",
    "acollare": "acollarar",
    "acollareis": "acollar",
    "acollaremos": "acollarar",
    "acollaren": "acollarar",
    "acollares": "acollarar",
    "acollaro": "acollarar",
    "acollaron": "acollar",
    "acollará": "acollar",
    "acollarábamos": "acollarar",
    "acollaráis": "acollarar",
    "acollarán": "acollar",
    "acollaráramos": "acollarar",
    "acollaráremos": "acollarar",
    "acollarás": "acollar",
    "acollarásemos": "acollarar",
    "acollaré": "acollarar",
    "acollaréis": "acollarar",
    "acollaría": "acollar",
    "acollaríais": "acollar",
    "acollaríamos": "acollar",
    "acollarían": "acollar",
    "acollarías": "acollar",
    "acollaró": "acollarar",
    "acollase": "acollar",
    "acollaseis": "acollar",
    "acollasen": "acollar",
    "acollases": "acollar",
    "acollaste": "acollar",
    "acollasteis": "acollar",
    "acollemos": "acollar",
    "acollona": "acollonar",
    "acollonaba": "acollonar",
    "acollonabais": "acollonar",
    "acollonaban": "acollonar",
    "acollonabas": "acollonar",
    "acollonad": "acollonar",
    "acollonada": "acollonar",
    "acollonadas": "acollonar",
    "acollonado": "acollonar",
    "acollonados": "acollonar",
    "acollonamos": "acollonar",
    "acollonan": "acollonar",
    "acollonando": "acollonar",
    "acollonara": "acollonar",
    "acollonarais": "acollonar",
    "acollonaran": "acollonar",
    "acollonaras": "acollonar",
    "acollonare": "acollonar",
    "acollonareis": "acollonar",
    "acollonaremos": "acollonar",
    "acollonaren": "acollonar",
    "acollonares": "acollonar",
    "acollonaron": "acollonar",
    "acollonará": "acollonar",
    "acollonarán": "acollonar",
    "acollonarás": "acollonar",
    "acollonaré": "acollonar",
    "acollonaréis": "acollonar",
    "acollonaría": "acollonar",
    "acollonaríais": "acollonar",
    "acollonaríamos": "acollonar",
    "acollonarían": "acollonar",
    "acollonarías": "acollonar",
    "acollonas": "acollonar",
    "acollonase": "acollonar",
    "acollonaseis": "acollonar",
    "acollonasen": "acollonar",
    "acollonases": "acollonar",
    "acollonaste": "acollonar",
    "acollonasteis": "acollonar",
    "acollone": "acollonar",
    "acollonemos": "acollonar",
    "acollonen": "acollonar",
    "acollones": "acollonar",
    "acollono": "acollonar",
    "acollonábamos": "acollonar",
    "acollonáis": "acollonar",
    "acollonáramos": "acollonar",
    "acollonáremos": "acollonar",
    "acollonásemos": "acollonar",
    "acolloné": "acollonar",
    "acollonéis": "acollonar",
    "acollonó": "acollonar",
    "acollábamos": "acollar",
    "acolláis": "acollar",
    "acolláramos": "acollar",
    "acolláremos": "acollar",
    "acollásemos": "acollar",
    "acollé": "acollar",
    "acolléis": "acollar",
    "acolló": "acollar",
    "acolo": "acolar",
    "acolábamos": "acolar",
    "acoláis": "acolar",
    "acoláramos": "acolar",
    "acoláremos": "acolar",
    "acolásemos": "acolar",
    "acolé": "acolar",
    "acoléis": "acolar",
    "acoló": "acolar",
    "acomas": "acoma",
    "acomedid": "acomedir",
    "acomedida": "acomedir",
    "acomedidas": "acomedir",
    "acomedido": "acomedir",
    "acomedidos": "acomedir",
    "acomedimos": "acomedir",
    "acomediremos": "acomedir",
    "acomedirá": "acomedir",
    "acomedirán": "acomedir",
    "acomedirás": "acomedir",
    "acomediré": "acomedir",
    "acomediréis": "acomedir",
    "acomediría": "acomedir",
    "acomediríais": "acomedir",
    "acomediríamos": "acomedir",
    "acomedirían": "acomedir",
    "acomedirías": "acomedir",
    "acomediste": "acomedir",
    "acomedisteis": "acomedir",
    "acomedí": "acomedir",
    "acomedía": "acomedir",
    "acomedíais": "acomedir",
    "acomedíamos": "acomedir",
    "acomedían": "acomedir",
    "acomedías": "acomedir",
    "acomedís": "acomedir",
    "acometa": "acometer",
    "acometamos": "acometer",
    "acometan": "acometer",
    "acometas": "acometer",
    "acomete": "acometer",
    "acometed": "acometer",
    "acometedora": "acometedor",
    "acometedoras": "acometedor",
    "acometedores": "acometedor",
    "acometemos": "acometer",
    "acometen": "acometer",
    "acometeremos": "acometer",
    "acometerá": "acometer",
    "acometerán": "acometer",
    "acometerás": "acometer",
    "acometeré": "acometer",
    "acometeréis": "acometer",
    "acometería": "acometer",
    "acometeríais": "acometer",
    "acometeríamos": "acometer",
    "acometerían": "acometer",
    "acometerías": "acometer",
    "acometes": "acometer",
    "acometida": "acometer",
    "acometidas": "acometida",
    "acometido": "acometer",
    "acometidos": "acometer",
    "acometiendo": "acometer",
    "acometiera": "acometer",
    "acometierais": "acometer",
    "acometieran": "acometer",
    "acometieras": "acometer",
    "acometiere": "acometer",
    "acometiereis": "acometer",
    "acometieren": "acometer",
    "acometieres": "acometer",
    "acometieron": "acometer",
    "acometiese": "acometer",
    "acometieseis": "acometer",
    "acometiesen": "acometer",
    "acometieses": "acometer",
    "acometimientos": "acometimiento",
    "acometimos": "acometer",
    "acometiste": "acometer",
    "acometisteis": "acometer",
    "acometividades": "acometividad",
    "acometiéramos": "acometer",
    "acometiéremos": "acometer",
    "acometiésemos": "acometer",
    "acometió": "acometer",
    "acometo": "acometer",
    "acometáis": "acometer",
    "acometéis": "acometer",
    "acometí": "acometer",
    "acometía": "acometer",
    "acometíais": "acometer",
    "acometíamos": "acometer",
    "acometían": "acometer",
    "acometías": "acometer",
    "acomida": "acomedir",
    "acomidamos": "acomedir",
    "acomidan": "acomedir",
    "acomidas": "acomedir",
    "acomide": "acomedir",
    "acomiden": "acomedir",
    "acomides": "acomedir",
    "acomidiendo": "acomedir",
    "acomidiera": "acomedir",
    "acomidierais": "acomedir",
    "acomidieran": "acomedir",
    "acomidieras": "acomedir",
    "acomidiere": "acomedir",
    "acomidiereis": "acomedir",
    "acomidieren": "acomedir",
    "acomidieres": "acomedir",
    "acomidieron": "acomedir",
    "acomidiese": "acomedir",
    "acomidieseis": "acomedir",
    "acomidiesen": "acomedir",
    "acomidieses": "acomedir",
    "acomidiéramos": "acomedir",
    "acomidiéremos": "acomedir",
    "acomidiésemos": "acomedir",
    "acomidió": "acomedir",
    "acomido": "acomedir",
    "acomidáis": "acomedir",
    "acomoda": "acomodar",
    "acomodaba": "acomodar",
    "acomodabais": "acomodar",
    "acomodaban": "acomodar",
    "acomodabas": "acomodar",
    "acomodables": "acomodable",
    "acomodaciones": "acomodación",
    "acomodad": "acomodar",
    "acomodada": "acomodar",
    "acomodadas": "acomodar",
    "acomodadiza": "acomodadizo",
    "acomodadizas": "acomodadizo",
    "acomodadizos": "acomodadizo",
    "acomodado": "acomodar",
    "acomodadora": "acomodador",
    "acomodadoras": "acomodador",
    "acomodadores": "acomodador",
    "acomodados": "acomodar",
    "acomodamientos": "acomodamiento",
    "acomodamos": "acomodar",
    "acomodan": "acomodar",
    "acomodando": "acomodar",
    "acomodara": "acomodar",
    "acomodarais": "acomodar",
    "acomodaran": "acomodar",
    "acomodaras": "acomodar",
    "acomodare": "acomodar",
    "acomodareis": "acomodar",
    "acomodaremos": "acomodar",
    "acomodaren": "acomodar",
    "acomodares": "acomodar",
    "acomodaron": "acomodar",
    "acomodará": "acomodar",
    "acomodarán": "acomodar",
    "acomodarás": "acomodar",
    "acomodaré": "acomodar",
    "acomodaréis": "acomodar",
    "acomodaría": "acomodar",
    "acomodaríais": "acomodar",
    "acomodaríamos": "acomodar",
    "acomodarían": "acomodar",
    "acomodarías": "acomodar",
    "acomodas": "acomodar",
    "acomodase": "acomodar",
    "acomodaseis": "acomodar",
    "acomodasen": "acomodar",
    "acomodases": "acomodar",
    "acomodaste": "acomodar",
    "acomodasteis": "acomodar",
    "acomodaticia": "acomodaticio",
    "acomodaticias": "acomodaticio",
    "acomodaticios": "acomodaticio",
    "acomodativa": "acomodativo",
    "acomodativas": "acomodativo",
    "acomodativos": "acomodativo",
    "acomode": "acomodar",
    "acomodemos": "acomodar",
    "acomoden": "acomodar",
    "acomodes": "acomodar",
    "acomodo": "acomodar",
    "acomodos": "acomodo",
    "acomodábamos": "acomodar",
    "acomodáis": "acomodar",
    "acomodáramos": "acomodar",
    "acomodáremos": "acomodar",
    "acomodásemos": "acomodar",
    "acomodé": "acomodar",
    "acomodéis": "acomodar",
    "acomodó": "acomodar",
    "acompasa": "acompasar",
    "acompasaba": "acompasar",
    "acompasabais": "acompasar",
    "acompasaban": "acompasar",
    "acompasabas": "acompasar",
    "acompasad": "acompasar",
    "acompasada": "acompasar",
    "acompasadas": "acompasar",
    "acompasado": "acompasar",
    "acompasados": "acompasar",
    "acompasamientos": "acompasamiento",
    "acompasamos": "acompasar",
    "acompasan": "acompasar",
    "acompasando": "acompasar",
    "acompasara": "acompasar",
    "acompasarais": "acompasar",
    "acompasaran": "acompasar",
    "acompasaras": "acompasar",
    "acompasare": "acompasar",
    "acompasareis": "acompasar",
    "acompasaremos": "acompasar",
    "acompasaren": "acompasar",
    "acompasares": "acompasar",
    "acompasaron": "acompasar",
    "acompasará": "acompasar",
    "acompasarán": "acompasar",
    "acompasarás": "acompasar",
    "acompasaré": "acompasar",
    "acompasaréis": "acompasar",
    "acompasaría": "acompasar",
    "acompasaríais": "acompasar",
    "acompasaríamos": "acompasar",
    "acompasarían": "acompasar",
    "acompasarías": "acompasar",
    "acompasas": "acompasar",
    "acompasase": "acompasar",
    "acompasaseis": "acompasar",
    "acompasasen": "acompasar",
    "acompasases": "acompasar",
    "acompasaste": "acompasar",
    "acompasasteis": "acompasar",
    "acompase": "acompasar",
    "acompasemos": "acompasar",
    "acompasen": "acompasar",
    "acompases": "acompasar",
    "acompaso": "acompasar",
    "acompasábamos": "acompasar",
    "acompasáis": "acompasar",
    "acompasáramos": "acompasar",
    "acompasáremos": "acompasar",
    "acompasásemos": "acompasar",
    "acompasé": "acompasar",
    "acompaséis": "acompasar",
    "acompasó": "acompasar",
    "acompaña": "acompañar",
    "acompañaba": "acompañar",
    "acompañabais": "acompañar",
    "acompañaban": "acompañar",
    "acompañabas": "acompañar",
    "acompañad": "acompañar",
    "acompañada": "acompañar",
    "acompañadas": "acompañar",
    "acompañado": "acompañar",
    "acompañadora": "acompañador",
    "acompañadoras": "acompañador",
    "acompañadores": "acompañador",
    "acompañados": "acompañar",
    "acompañamientos": "acompañamiento",
    "acompañamos": "acompañar",
    "acompañan": "acompañar",
    "acompañando": "acompañar",
    "acompañantas": "acompañanta",
    "acompañantes": "acompañante",
    "acompañara": "acompañar",
    "acompañarais": "acompañar",
    "acompañaran": "acompañar",
    "acompañaras": "acompañar",
    "acompañare": "acompañar",
    "acompañareis": "acompañar",
    "acompañaremos": "acompañar",
    "acompañaren": "acompañar",
    "acompañares": "acompañar",
    "acompañaron": "acompañar",
    "acompañará": "acompañar",
    "acompañarán": "acompañar",
    "acompañarás": "acompañar",
    "acompañaré": "acompañar",
    "acompañaréis": "acompañar",
    "acompañaría": "acompañar",
    "acompañaríais": "acompañar",
    "acompañaríamos": "acompañar",
    "acompañarían": "acompañar",
    "acompañarías": "acompañar",
    "acompañas": "acompañar",
    "acompañase": "acompañar",
    "acompañaseis": "acompañar",
    "acompañasen": "acompañar",
    "acompañases": "acompañar",
    "acompañaste": "acompañar",
    "acompañasteis": "acompañar",
    "acompañe": "acompañar",
    "acompañemos": "acompañar",
    "acompañen": "acompañar",
    "acompañes": "acompañar",
    "acompaño": "acompañar",
    "acompañábamos": "acompañar",
    "acompañáis": "acompañar",
    "acompañáramos": "acompañar",
    "acompañáremos": "acompañar",
    "acompañásemos": "acompañar",
    "acompañé": "acompañar",
    "acompañéis": "acompañar",
    "acompañó": "acompañar",
    "acompleja": "acomplejar",
    "acomplejaba": "acomplejar",
    "acomplejabais": "acomplejar",
    "acomplejaban": "acomplejar",
    "acomplejabas": "acomplejar",
    "acomplejad": "acomplejar",
    "acomplejada": "acomplejar",
    "acomplejadas": "acomplejar",
    "acomplejado": "acomplejar",
    "acomplejados": "acomplejar",
    "acomplejamos": "acomplejar",
    "acomplejan": "acomplejar",
    "acomplejando": "acomplejar",
    "acomplejara": "acomplejar",
    "acomplejarais": "acomplejar",
    "acomplejaran": "acomplejar",
    "acomplejaras": "acomplejar",
    "acomplejare": "acomplejar",
    "acomplejareis": "acomplejar",
    "acomplejaremos": "acomplejar",
    "acomplejaren": "acomplejar",
    "acomplejares": "acomplejar",
    "acomplejaron": "acomplejar",
    "acomplejará": "acomplejar",
    "acomplejarán": "acomplejar",
    "acomplejarás": "acomplejar",
    "acomplejaré": "acomplejar",
    "acomplejaréis": "acomplejar",
    "acomplejaría": "acomplejar",
    "acomplejaríais": "acomplejar",
    "acomplejaríamos": "acomplejar",
    "acomplejarían": "acomplejar",
    "acomplejarías": "acomplejar",
    "acomplejas": "acomplejar",
    "acomplejase": "acomplejar",
    "acomplejaseis": "acomplejar",
    "acomplejasen": "acomplejar",
    "acomplejases": "acomplejar",
    "acomplejaste": "acomplejar",
    "acomplejasteis": "acomplejar",
    "acompleje": "acomplejar",
    "acomplejemos": "acomplejar",
    "acomplejen": "acomplejar",
    "acomplejes": "acomplejar",
    "acomplejo": "acomplejar",
    "acomplejábamos": "acomplejar",
    "acomplejáis": "acomplejar",
    "acomplejáramos": "acomplejar",
    "acomplejáremos": "acomplejar",
    "acomplejásemos": "acomplejar",
    "acomplejé": "acomplejar",
    "acomplejéis": "acomplejar",
    "acomplejó": "acomplejar",
    "acomuna": "acomunar",
    "acomunaba": "acomunar",
    "acomunabais": "acomunar",
    "acomunaban": "acomunar",
    "acomunabas": "acomunar",
    "acomunad": "acomunar",
    "acomunada": "acomunar",
    "acomunadas": "acomunar",
    "acomunado": "acomunar",
    "acomunados": "acomunar",
    "acomunamos": "acomunar",
    "acomunan": "acomunar",
    "acomunando": "acomunar",
    "acomunara": "acomunar",
    "acomunarais": "acomunar",
    "acomunaran": "acomunar",
    "acomunaras": "acomunar",
    "acomunare": "acomunar",
    "acomunareis": "acomunar",
    "acomunaremos": "acomunar",
    "acomunaren": "acomunar",
    "acomunares": "acomunar",
    "acomunaron": "acomunar",
    "acomunará": "acomunar",
    "acomunarán": "acomunar",
    "acomunarás": "acomunar",
    "acomunaré": "acomunar",
    "acomunaréis": "acomunar",
    "acomunaría": "acomunar",
    "acomunaríais": "acomunar",
    "acomunaríamos": "acomunar",
    "acomunarían": "acomunar",
    "acomunarías": "acomunar",
    "acomunas": "acomunar",
    "acomunase": "acomunar",
    "acomunaseis": "acomunar",
    "acomunasen": "acomunar",
    "acomunases": "acomunar",
    "acomunaste": "acomunar",
    "acomunasteis": "acomunar",
    "acomune": "acomunar",
    "acomunemos": "acomunar",
    "acomunen": "acomunar",
    "acomunes": "acomunar",
    "acomuno": "acomunar",
    "acomunábamos": "acomunar",
    "acomunáis": "acomunar",
    "acomunáramos": "acomunar",
    "acomunáremos": "acomunar",
    "acomunásemos": "acomunar",
    "acomuné": "acomunar",
    "acomunéis": "acomunar",
    "acomunó": "acomunar",
    "aconcha": "aconchar",
    "aconchaba": "aconchar",
    "aconchababa": "aconchabar",
    "aconchababais": "aconchabar",
    "aconchababan": "aconchabar",
    "aconchababas": "aconchabar",
    "aconchabad": "aconchabar",
    "aconchabada": "aconchabar",
    "aconchabadas": "aconchabar",
    "aconchabado": "aconchabar",
    "aconchabados": "aconchabar",
    "aconchabais": "aconchar",
    "aconchabamos": "aconchabar",
    "aconchaban": "aconchar",
    "aconchabando": "aconchabar",
    "aconchabara": "aconchabar",
    "aconchabarais": "aconchabar",
    "aconchabaran": "aconchabar",
    "aconchabaras": "aconchabar",
    "aconchabare": "aconchabar",
    "aconchabareis": "aconchabar",
    "aconchabaremos": "aconchabar",
    "aconchabaren": "aconchabar",
    "aconchabares": "aconchabar",
    "aconchabaron": "aconchabar",
    "aconchabará": "aconchabar",
    "aconchabarán": "aconchabar",
    "aconchabarás": "aconchabar",
    "aconchabaré": "aconchabar",
    "aconchabaréis": "aconchabar",
    "aconchabaría": "aconchabar",
    "aconchabaríais": "aconchabar",
    "aconchabaríamos": "aconchabar",
    "aconchabarían": "aconchabar",
    "aconchabarías": "aconchabar",
    "aconchabas": "aconchar",
    "aconchabase": "aconchabar",
    "aconchabaseis": "aconchabar",
    "aconchabasen": "aconchabar",
    "aconchabases": "aconchabar",
    "aconchabaste": "aconchabar",
    "aconchabasteis": "aconchabar",
    "aconchabe": "aconchabar",
    "aconchabemos": "aconchabar",
    "aconchaben": "aconchabar",
    "aconchabes": "aconchabar",
    "aconchabo": "aconchabar",
    "aconchabábamos": "aconchabar",
    "aconchabáis": "aconchabar",
    "aconchabáramos": "aconchabar",
    "aconchabáremos": "aconchabar",
    "aconchabásemos": "aconchabar",
    "aconchabé": "aconchabar",
    "aconchabéis": "aconchabar",
    "aconchabó": "aconchabar",
    "aconchad": "aconchar",
    "aconchada": "aconchar",
    "aconchadas": "aconchar",
    "aconchado": "aconchar",
    "aconchados": "aconchar",
    "aconchamos": "aconchar",
    "aconchan": "aconchar",
    "aconchando": "aconchar",
    "aconchara": "aconchar",
    "aconcharais": "aconchar",
    "aconcharan": "aconchar",
    "aconcharas": "aconchar",
    "aconchare": "aconchar",
    "aconchareis": "aconchar",
    "aconcharemos": "aconchar",
    "aconcharen": "aconchar",
    "aconchares": "aconchar",
    "aconcharon": "aconchar",
    "aconchará": "aconchar",
    "aconcharán": "aconchar",
    "aconcharás": "aconchar",
    "aconcharé": "aconchar",
    "aconcharéis": "aconchar",
    "aconcharía": "aconchar",
    "aconcharíais": "aconchar",
    "aconcharíamos": "aconchar",
    "aconcharían": "aconchar",
    "aconcharías": "aconchar",
    "aconchas": "aconchar",
    "aconchase": "aconchar",
    "aconchaseis": "aconchar",
    "aconchasen": "aconchar",
    "aconchases": "aconchar",
    "aconchaste": "aconchar",
    "aconchasteis": "aconchar",
    "aconche": "aconchar",
    "aconchemos": "aconchar",
    "aconchen": "aconchar",
    "aconches": "aconchar",
    "aconcho": "aconchar",
    "aconchábamos": "aconchar",
    "aconcháis": "aconchar",
    "aconcháramos": "aconchar",
    "aconcháremos": "aconchar",
    "aconchásemos": "aconchar",
    "aconché": "aconchar",
    "aconchéis": "aconchar",
    "aconchó": "aconchar",
    "acondiciona": "acondicionar",
    "acondicionaba": "acondicionar",
    "acondicionabais": "acondicionar",
    "acondicionaban": "acondicionar",
    "acondicionabas": "acondicionar",
    "acondicionad": "acondicionar",
    "acondicionada": "acondicionar",
    "acondicionadas": "acondicionar",
    "acondicionado": "acondicionar",
    "acondicionadora": "acondicionador",
    "acondicionadoras": "acondicionador",
    "acondicionadores": "acondicionador",
    "acondicionados": "acondicionar",
    "acondicionamientos": "acondicionamiento",
    "acondicionamos": "acondicionar",
    "acondicionan": "acondicionar",
    "acondicionando": "acondicionar",
    "acondicionara": "acondicionar",
    "acondicionarais": "acondicionar",
    "acondicionaran": "acondicionar",
    "acondicionaras": "acondicionar",
    "acondicionare": "acondicionar",
    "acondicionareis": "acondicionar",
    "acondicionaremos": "acondicionar",
    "acondicionaren": "acondicionar",
    "acondicionares": "acondicionar",
    "acondicionaron": "acondicionar",
    "acondicionará": "acondicionar",
    "acondicionarán": "acondicionar",
    "acondicionarás": "acondicionar",
    "acondicionaré": "acondicionar",
    "acondicionaréis": "acondicionar",
    "acondicionaría": "acondicionar",
    "acondicionaríais": "acondicionar",
    "acondicionaríamos": "acondicionar",
    "acondicionarían": "acondicionar",
    "acondicionarías": "acondicionar",
    "acondicionas": "acondicionar",
    "acondicionase": "acondicionar",
    "acondicionaseis": "acondicionar",
    "acondicionasen": "acondicionar",
    "acondicionases": "acondicionar",
    "acondicionaste": "acondicionar",
    "acondicionasteis": "acondicionar",
    "acondicione": "acondicionar",
    "acondicionemos": "acondicionar",
    "acondicionen": "acondicionar",
    "acondiciones": "acondicionar",
    "acondiciono": "acondicionar",
    "acondicionábamos": "acondicionar",
    "acondicionáis": "acondicionar",
    "acondicionáramos": "acondicionar",
    "acondicionáremos": "acondicionar",
    "acondicionásemos": "acondicionar",
    "acondicioné": "acondicionar",
    "acondicionéis": "acondicionar",
    "acondicionó": "acondicionar",
    "acondroplasias": "acondroplasia",
    "acongoja": "acongojar",
    "acongojaba": "acongojar",
    "acongojabais": "acongojar",
    "acongojaban": "acongojar",
    "acongojabas": "acongojar",
    "acongojad": "acongojar",
    "acongojada": "acongojar",
    "acongojadas": "acongojar",
    "acongojado": "acongojar",
    "acongojados": "acongojar",
    "acongojamos": "acongojar",
    "acongojan": "acongojar",
    "acongojando": "acongojar",
    "acongojara": "acongojar",
    "acongojarais": "acongojar",
    "acongojaran": "acongojar",
    "acongojaras": "acongojar",
    "acongojare": "acongojar",
    "acongojareis": "acongojar",
    "acongojaremos": "acongojar",
    "acongojaren": "acongojar",
    "acongojares": "acongojar",
    "acongojaron": "acongojar",
    "acongojará": "acongojar",
    "acongojarán": "acongojar",
    "acongojarás": "acongojar",
    "acongojaré": "acongojar",
    "acongojaréis": "acongojar",
    "acongojaría": "acongojar",
    "acongojaríais": "acongojar",
    "acongojaríamos": "acongojar",
    "acongojarían": "acongojar",
    "acongojarías": "acongojar",
    "acongojas": "acongojar",
    "acongojase": "acongojar",
    "acongojaseis": "acongojar",
    "acongojasen": "acongojar",
    "acongojases": "acongojar",
    "acongojaste": "acongojar",
    "acongojasteis": "acongojar",
    "acongoje": "acongojar",
    "acongojemos": "acongojar",
    "acongojen": "acongojar",
    "acongojes": "acongojar",
    "acongojo": "acongojar",
    "acongojábamos": "acongojar",
    "acongojáis": "acongojar",
    "acongojáramos": "acongojar",
    "acongojáremos": "acongojar",
    "acongojásemos": "acongojar",
    "acongojé": "acongojar",
    "acongojéis": "acongojar",
    "acongojó": "acongojar",
    "aconitatos": "aconitato",
    "aconitinas": "aconitina",
    "aconseja": "aconsejar",
    "aconsejaba": "aconsejar",
    "aconsejabais": "aconsejar",
    "aconsejaban": "aconsejar",
    "aconsejabas": "aconsejar",
    "aconsejables": "aconsejable",
    "aconsejad": "aconsejar",
    "aconsejada": "aconsejar",
    "aconsejadas": "aconsejar",
    "aconsejado": "aconsejar",
    "aconsejadora": "aconsejador",
    "aconsejadoras": "aconsejador",
    "aconsejadores": "aconsejador",
    "aconsejados": "aconsejar",
    "aconsejamos": "aconsejar",
    "aconsejan": "aconsejar",
    "aconsejando": "aconsejar",
    "aconsejara": "aconsejar",
    "aconsejarais": "aconsejar",
    "aconsejaran": "aconsejar",
    "aconsejaras": "aconsejar",
    "aconsejare": "aconsejar",
    "aconsejareis": "aconsejar",
    "aconsejaremos": "aconsejar",
    "aconsejaren": "aconsejar",
    "aconsejares": "aconsejar",
    "aconsejaron": "aconsejar",
    "aconsejará": "aconsejar",
    "aconsejarán": "aconsejar",
    "aconsejarás": "aconsejar",
    "aconsejaré": "aconsejar",
    "aconsejaréis": "aconsejar",
    "aconsejaría": "aconsejar",
    "aconsejaríais": "aconsejar",
    "aconsejaríamos": "aconsejar",
    "aconsejarían": "aconsejar",
    "aconsejarías": "aconsejar",
    "aconsejas": "aconsejar",
    "aconsejase": "aconsejar",
    "aconsejaseis": "aconsejar",
    "aconsejasen": "aconsejar",
    "aconsejases": "aconsejar",
    "aconsejaste": "aconsejar",
    "aconsejasteis": "aconsejar",
    "aconseje": "aconsejar",
    "aconsejemos": "aconsejar",
    "aconsejen": "aconsejar",
    "aconsejes": "aconsejar",
    "aconsejo": "aconsejar",
    "aconsejábamos": "aconsejar",
    "aconsejáis": "aconsejar",
    "aconsejáramos": "aconsejar",
    "aconsejáremos": "aconsejar",
    "aconsejásemos": "aconsejar",
    "aconsejé": "aconsejar",
    "aconsejéis": "aconsejar",
    "aconsejó": "aconsejar",
    "aconsonanta": "aconsonantar",
    "aconsonantaba": "aconsonantar",
    "aconsonantabais": "aconsonantar",
    "aconsonantaban": "aconsonantar",
    "aconsonantabas": "aconsonantar",
    "aconsonantad": "aconsonantar",
    "aconsonantada": "aconsonantar",
    "aconsonantadas": "aconsonantar",
    "aconsonantado": "aconsonantar",
    "aconsonantados": "aconsonantar",
    "aconsonantamos": "aconsonantar",
    "aconsonantan": "aconsonantar",
    "aconsonantando": "aconsonantar",
    "aconsonantara": "aconsonantar",
    "aconsonantarais": "aconsonantar",
    "aconsonantaran": "aconsonantar",
    "aconsonantaras": "aconsonantar",
    "aconsonantare": "aconsonantar",
    "aconsonantareis": "aconsonantar",
    "aconsonantaremos": "aconsonantar",
    "aconsonantaren": "aconsonantar",
    "aconsonantares": "aconsonantar",
    "aconsonantaron": "aconsonantar",
    "aconsonantará": "aconsonantar",
    "aconsonantarán": "aconsonantar",
    "aconsonantarás": "aconsonantar",
    "aconsonantaré": "aconsonantar",
    "aconsonantaréis": "aconsonantar",
    "aconsonantaría": "aconsonantar",
    "aconsonantaríais": "aconsonantar",
    "aconsonantaríamos": "aconsonantar",
    "aconsonantarían": "aconsonantar",
    "aconsonantarías": "aconsonantar",
    "aconsonantas": "aconsonantar",
    "aconsonantase": "aconsonantar",
    "aconsonantaseis": "aconsonantar",
    "aconsonantasen": "aconsonantar",
    "aconsonantases": "aconsonantar",
    "aconsonantaste": "aconsonantar",
    "aconsonantasteis": "aconsonantar",
    "aconsonante": "aconsonantar",
    "aconsonantemos": "aconsonantar",
    "aconsonanten": "aconsonantar",
    "aconsonantes": "aconsonantar",
    "aconsonanto": "aconsonantar",
    "aconsonantábamos": "aconsonantar",
    "aconsonantáis": "aconsonantar",
    "aconsonantáramos": "aconsonantar",
    "aconsonantáremos": "aconsonantar",
    "aconsonantásemos": "aconsonantar",
    "aconsonanté": "aconsonantar",
    "aconsonantéis": "aconsonantar",
    "aconsonantó": "aconsonantar",
    "acontece": "acontecer",
    "aconteced": "acontecer",
    "acontecemos": "acontecer",
    "acontecen": "acontecer",
    "aconteceremos": "acontecer",
    "aconteceres": "acontecer",
    "acontecerá": "acontecer",
    "acontecerán": "acontecer",
    "acontecerás": "acontecer",
    "aconteceré": "acontecer",
    "aconteceréis": "acontecer",
    "acontecería": "acontecer",
    "aconteceríais": "acontecer",
    "aconteceríamos": "acontecer",
    "acontecerían": "acontecer",
    "acontecerías": "acontecer",
    "aconteces": "acontecer",
    "acontecida": "acontecer",
    "acontecidas": "acontecer",
    "acontecido": "acontecer",
    "acontecidos": "acontecer",
    "aconteciendo": "acontecer",
    "aconteciera": "acontecer",
    "acontecierais": "acontecer",
    "acontecieran": "acontecer",
    "acontecieras": "acontecer",
    "aconteciere": "acontecer",
    "aconteciereis": "acontecer",
    "acontecieren": "acontecer",
    "acontecieres": "acontecer",
    "acontecieron": "acontecer",
    "aconteciese": "acontecer",
    "acontecieseis": "acontecer",
    "aconteciesen": "acontecer",
    "acontecieses": "acontecer",
    "acontecimientos": "acontecimiento",
    "acontecimos": "acontecer",
    "aconteciste": "acontecer",
    "acontecisteis": "acontecer",
    "aconteciéramos": "acontecer",
    "aconteciéremos": "acontecer",
    "aconteciésemos": "acontecer",
    "aconteció": "acontecer",
    "acontecéis": "acontecer",
    "acontecí": "acontecer",
    "acontecía": "acontecer",
    "acontecíais": "acontecer",
    "acontecíamos": "acontecer",
    "acontecían": "acontecer",
    "acontecías": "acontecer",
    "acontezca": "acontecer",
    "acontezcamos": "acontecer",
    "acontezcan": "acontecer",
    "acontezcas": "acontecer",
    "acontezco": "acontecer",
    "acontezcáis": "acontecer",
    "aconítica": "aconítico",
    "aconíticas": "aconítico",
    "aconíticos": "aconítico",
    "acopa": "acopar",
    "acopaba": "acopar",
    "acopabais": "acopar",
    "acopaban": "acopar",
    "acopabas": "acopar",
    "acopad": "acopar",
    "acopada": "acopar",
    "acopadas": "acopar",
    "acopado": "acopar",
    "acopados": "acopar",
    "acopamos": "acopar",
    "acopan": "acopar",
    "acopando": "acopar",
    "acopara": "acopar",
    "acoparais": "acopar",
    "acoparan": "acopar",
    "acoparas": "acopar",
    "acopare": "acopar",
    "acopareis": "acopar",
    "acoparemos": "acopar",
    "acoparen": "acopar",
    "acopares": "acopar",
    "acoparon": "acopar",
    "acopará": "acopar",
    "acoparán": "acopar",
    "acoparás": "acopar",
    "acoparé": "acopar",
    "acoparéis": "acopar",
    "acoparía": "acopar",
    "acoparíais": "acopar",
    "acoparíamos": "acopar",
    "acoparían": "acopar",
    "acoparías": "acopar",
    "acopas": "acopar",
    "acopase": "acopar",
    "acopaseis": "acopar",
    "acopasen": "acopar",
    "acopases": "acopar",
    "acopaste": "acopar",
    "acopasteis": "acopar",
    "acope": "acopar",
    "acopemos": "acopar",
    "acopen": "acopar",
    "acopes": "acopar",
    "acopia": "acopiar",
    "acopiaba": "acopiar",
    "acopiabais": "acopiar",
    "acopiaban": "acopiar",
    "acopiabas": "acopiar",
    "acopiad": "acopiar",
    "acopiada": "acopiar",
    "acopiadas": "acopiar",
    "acopiado": "acopiar",
    "acopiados": "acopiar",
    "acopiamientos": "acopiamiento",
    "acopiamos": "acopiar",
    "acopian": "acopiar",
    "acopiando": "acopiar",
    "acopiara": "acopiar",
    "acopiarais": "acopiar",
    "acopiaran": "acopiar",
    "acopiaras": "acopiar",
    "acopiare": "acopiar",
    "acopiareis": "acopiar",
    "acopiaremos": "acopiar",
    "acopiaren": "acopiar",
    "acopiares": "acopiar",
    "acopiaron": "acopiar",
    "acopiará": "acopiar",
    "acopiarán": "acopiar",
    "acopiarás": "acopiar",
    "acopiaré": "acopiar",
    "acopiaréis": "acopiar",
    "acopiaría": "acopiar",
    "acopiaríais": "acopiar",
    "acopiaríamos": "acopiar",
    "acopiarían": "acopiar",
    "acopiarías": "acopiar",
    "acopias": "acopiar",
    "acopiase": "acopiar",
    "acopiaseis": "acopiar",
    "acopiasen": "acopiar",
    "acopiases": "acopiar",
    "acopiaste": "acopiar",
    "acopiasteis": "acopiar",
    "acopie": "acopiar",
    "acopiemos": "acopiar",
    "acopien": "acopiar",
    "acopies": "acopiar",
    "acopio": "acopiar",
    "acopios": "acopio",
    "acopiábamos": "acopiar",
    "acopiáis": "acopiar",
    "acopiáramos": "acopiar",
    "acopiáremos": "acopiar",
    "acopiásemos": "acopiar",
    "acopié": "acopiar",
    "acopiéis": "acopiar",
    "acopió": "acopiar",
    "acopla": "acoplar",
    "acoplaba": "acoplar",
    "acoplabais": "acoplar",
    "acoplaban": "acoplar",
    "acoplabas": "acoplar",
    "acoplad": "acoplar",
    "acoplada": "acoplar",
    "acopladas": "acoplar",
    "acoplado": "acoplar",
    "acopladores": "acoplador",
    "acoplados": "acoplar",
    "acopladuras": "acopladura",
    "acoplamientos": "acoplamiento",
    "acoplamos": "acoplar",
    "acoplan": "acoplar",
    "acoplando": "acoplar",
    "acoplara": "acoplar",
    "acoplarais": "acoplar",
    "acoplaran": "acoplar",
    "acoplaras": "acoplar",
    "acoplare": "acoplar",
    "acoplareis": "acoplar",
    "acoplaremos": "acoplar",
    "acoplaren": "acoplar",
    "acoplares": "acoplar",
    "acoplaron": "acoplar",
    "acoplará": "acoplar",
    "acoplarán": "acoplar",
    "acoplarás": "acoplar",
    "acoplaré": "acoplar",
    "acoplaréis": "acoplar",
    "acoplaría": "acoplar",
    "acoplaríais": "acoplar",
    "acoplaríamos": "acoplar",
    "acoplarían": "acoplar",
    "acoplarías": "acoplar",
    "acoplas": "acoplar",
    "acoplase": "acoplar",
    "acoplaseis": "acoplar",
    "acoplasen": "acoplar",
    "acoplases": "acoplar",
    "acoplaste": "acoplar",
    "acoplasteis": "acoplar",
    "acople": "acoplar",
    "acoplemos": "acoplar",
    "acoplen": "acoplar",
    "acoples": "acople",
    "acoplo": "acoplar",
    "acoplos": "acoplo",
    "acoplábamos": "acoplar",
    "acopláis": "acoplar",
    "acopláramos": "acoplar",
    "acopláremos": "acoplar",
    "acoplásemos": "acoplar",
    "acoplé": "acoplar",
    "acopléis": "acoplar",
    "acopló": "acoplar",
    "acopo": "acopar",
    "acopábamos": "acopar",
    "acopáis": "acopar",
    "acopáramos": "acopar",
    "acopáremos": "acopar",
    "acopásemos": "acopar",
    "acopé": "acopar",
    "acopéis": "acopar",
    "acopó": "acopar",
    "acoquina": "acoquinar",
    "acoquinaba": "acoquinar",
    "acoquinabais": "acoquinar",
    "acoquinaban": "acoquinar",
    "acoquinabas": "acoquinar",
    "acoquinad": "acoquinar",
    "acoquinada": "acoquinar",
    "acoquinadas": "acoquinar",
    "acoquinado": "acoquinar",
    "acoquinados": "acoquinar",
    "acoquinamientos": "acoquinamiento",
    "acoquinamos": "acoquinar",
    "acoquinan": "acoquinar",
    "acoquinando": "acoquinar",
    "acoquinara": "acoquinar",
    "acoquinarais": "acoquinar",
    "acoquinaran": "acoquinar",
    "acoquinaras": "acoquinar",
    "acoquinare": "acoquinar",
    "acoquinareis": "acoquinar",
    "acoquinaremos": "acoquinar",
    "acoquinaren": "acoquinar",
    "acoquinares": "acoquinar",
    "acoquinaron": "acoquinar",
    "acoquinará": "acoquinar",
    "acoquinarán": "acoquinar",
    "acoquinarás": "acoquinar",
    "acoquinaré": "acoquinar",
    "acoquinaréis": "acoquinar",
    "acoquinaría": "acoquinar",
    "acoquinaríais": "acoquinar",
    "acoquinaríamos": "acoquinar",
    "acoquinarían": "acoquinar",
    "acoquinarías": "acoquinar",
    "acoquinas": "acoquinar",
    "acoquinase": "acoquinar",
    "acoquinaseis": "acoquinar",
    "acoquinasen": "acoquinar",
    "acoquinases": "acoquinar",
    "acoquinaste": "acoquinar",
    "acoquinasteis": "acoquinar",
    "acoquine": "acoquinar",
    "acoquinemos": "acoquinar",
    "acoquinen": "acoquinar",
    "acoquines": "acoquinar",
    "acoquino": "acoquinar",
    "acoquinábamos": "acoquinar",
    "acoquináis": "acoquinar",
    "acoquináramos": "acoquinar",
    "acoquináremos": "acoquinar",
    "acoquinásemos": "acoquinar",
    "acoquiné": "acoquinar",
    "acoquinéis": "acoquinar",
    "acoquinó": "acoquinar",
    "acora": "acorar",
    "acoraba": "acorar",
    "acorabais": "acorar",
    "acoraban": "acorar",
    "acorabas": "acorar",
    "acorace": "acorazar",
    "acoracemos": "acorazar",
    "acoracen": "acorazar",
    "acoraces": "acorazar",
    "acoracé": "acorazar",
    "acoracéis": "acorazar",
    "acorad": "acorar",
    "acorada": "acorar",
    "acoradas": "acorar",
    "acorado": "acorar",
    "acorados": "acorar",
    "acoramos": "acorar",
    "acoran": "acorar",
    "acorando": "acorar",
    "acorara": "acorar",
    "acorarais": "acorar",
    "acoraran": "acorar",
    "acoraras": "acorar",
    "acorare": "acorar",
    "acorareis": "acorar",
    "acoraremos": "acorar",
    "acoraren": "acorar",
    "acorares": "acorar",
    "acoraron": "acorar",
    "acorará": "acorar",
    "acorarán": "acorar",
    "acorarás": "acorar",
    "acoraré": "acorar",
    "acoraréis": "acorar",
    "acoraría": "acorar",
    "acoraríais": "acorar",
    "acoraríamos": "acorar",
    "acorarían": "acorar",
    "acorarías": "acorar",
    "acoras": "acorar",
    "acorase": "acorar",
    "acoraseis": "acorar",
    "acorasen": "acorar",
    "acorases": "acorar",
    "acoraste": "acorar",
    "acorasteis": "acorar",
    "acoraza": "acorazar",
    "acorazaba": "acorazar",
    "acorazabais": "acorazar",
    "acorazaban": "acorazar",
    "acorazabas": "acorazar",
    "acorazad": "acorazar",
    "acorazada": "acorazar",
    "acorazadas": "acorazar",
    "acorazado": "acorazar",
    "acorazados": "acorazar",
    "acorazamientos": "acorazamiento",
    "acorazamos": "acorazar",
    "acorazan": "acorazar",
    "acorazando": "acorazar",
    "acorazara": "acorazar",
    "acorazarais": "acorazar",
    "acorazaran": "acorazar",
    "acorazaras": "acorazar",
    "acorazare": "acorazar",
    "acorazareis": "acorazar",
    "acorazaremos": "acorazar",
    "acorazaren": "acorazar",
    "acorazares": "acorazar",
    "acorazaron": "acorazar",
    "acorazará": "acorazar",
    "acorazarán": "acorazar",
    "acorazarás": "acorazar",
    "acorazaré": "acorazar",
    "acorazaréis": "acorazar",
    "acorazaría": "acorazar",
    "acorazaríais": "acorazar",
    "acorazaríamos": "acorazar",
    "acorazarían": "acorazar",
    "acorazarías": "acorazar",
    "acorazas": "acorazar",
    "acorazase": "acorazar",
    "acorazaseis": "acorazar",
    "acorazasen": "acorazar",
    "acorazases": "acorazar",
    "acorazaste": "acorazar",
    "acorazasteis": "acorazar",
    "acorazo": "acorazar",
    "acorazonada": "acorazonado",
    "acorazonadas": "acorazonado",
    "acorazonados": "acorazonado",
    "acorazábamos": "acorazar",
    "acorazáis": "acorazar",
    "acorazáramos": "acorazar",
    "acorazáremos": "acorazar",
    "acorazásemos": "acorazar",
    "acorazó": "acorazar",
    "acorcha": "acorchar",
    "acorchaba": "acorchar",
    "acorchabais": "acorchar",
    "acorchaban": "acorchar",
    "acorchabas": "acorchar",
    "acorchad": "acorchar",
    "acorchada": "acorchar",
    "acorchadas": "acorchar",
    "acorchado": "acorchar",
    "acorchados": "acorchar",
    "acorchamientos": "acorchamiento",
    "acorchamos": "acorchar",
    "acorchan": "acorchar",
    "acorchando": "acorchar",
    "acorchara": "acorchar",
    "acorcharais": "acorchar",
    "acorcharan": "acorchar",
    "acorcharas": "acorchar",
    "acorchare": "acorchar",
    "acorchareis": "acorchar",
    "acorcharemos": "acorchar",
    "acorcharen": "acorchar",
    "acorchares": "acorchar",
    "acorcharon": "acorchar",
    "acorchará": "acorchar",
    "acorcharán": "acorchar",
    "acorcharás": "acorchar",
    "acorcharé": "acorchar",
    "acorcharéis": "acorchar",
    "acorcharía": "acorchar",
    "acorcharíais": "acorchar",
    "acorcharíamos": "acorchar",
    "acorcharían": "acorchar",
    "acorcharías": "acorchar",
    "acorchas": "acorchar",
    "acorchase": "acorchar",
    "acorchaseis": "acorchar",
    "acorchasen": "acorchar",
    "acorchases": "acorchar",
    "acorchaste": "acorchar",
    "acorchasteis": "acorchar",
    "acorche": "acorchar",
    "acorchemos": "acorchar",
    "acorchen": "acorchar",
    "acorches": "acorchar",
    "acorcho": "acorchar",
    "acorchábamos": "acorchar",
    "acorcháis": "acorchar",
    "acorcháramos": "acorchar",
    "acorcháremos": "acorchar",
    "acorchásemos": "acorchar",
    "acorché": "acorchar",
    "acorchéis": "acorchar",
    "acorchó": "acorchar",
    "acordaba": "acordar",
    "acordabais": "acordar",
    "acordaban": "acordar",
    "acordabas": "acordar",
    "acordables": "acordable",
    "acordad": "acordar",
    "acordada": "acordar",
    "acordadas": "acordar",
    "acordado": "acordar",
    "acordados": "acordar",
    "acordamos": "acordar",
    "acordando": "acordar",
    "acordara": "acordar",
    "acordarais": "acordar",
    "acordaran": "acordar",
    "acordaras": "acordar",
    "acordare": "acordar",
    "acordareis": "acordar",
    "acordaremos": "acordar",
    "acordaren": "acordar",
    "acordares": "acordar",
    "acordaron": "acordar",
    "acordará": "acordar",
    "acordarán": "acordar",
    "acordarás": "acordar",
    "acordaré": "acordar",
    "acordaréis": "acordar",
    "acordaría": "acordar",
    "acordaríais": "acordar",
    "acordaríamos": "acordar",
    "acordarían": "acordar",
    "acordarías": "acordar",
    "acordase": "acordar",
    "acordaseis": "acordar",
    "acordasen": "acordar",
    "acordases": "acordar",
    "acordaste": "acordar",
    "acordasteis": "acordar",
    "acordela": "acordelar",
    "acordelaba": "acordelar",
    "acordelabais": "acordelar",
    "acordelaban": "acordelar",
    "acordelabas": "acordelar",
    "acordelad": "acordelar",
    "acordelada": "acordelar",
    "acordeladas": "acordelar",
    "acordelado": "acordelar",
    "acordelados": "acordelar",
    "acordelamos": "acordelar",
    "acordelan": "acordelar",
    "acordelando": "acordelar",
    "acordelara": "acordelar",
    "acordelarais": "acordelar",
    "acordelaran": "acordelar",
    "acordelaras": "acordelar",
    "acordelare": "acordelar",
    "acordelareis": "acordelar",
    "acordelaremos": "acordelar",
    "acordelaren": "acordelar",
    "acordelares": "acordelar",
    "acordelaron": "acordelar",
    "acordelará": "acordelar",
    "acordelarán": "acordelar",
    "acordelarás": "acordelar",
    "acordelaré": "acordelar",
    "acordelaréis": "acordelar",
    "acordelaría": "acordelar",
    "acordelaríais": "acordelar",
    "acordelaríamos": "acordelar",
    "acordelarían": "acordelar",
    "acordelarías": "acordelar",
    "acordelas": "acordelar",
    "acordelase": "acordelar",
    "acordelaseis": "acordelar",
    "acordelasen": "acordelar",
    "acordelases": "acordelar",
    "acordelaste": "acordelar",
    "acordelasteis": "acordelar",
    "acordele": "acordelar",
    "acordelemos": "acordelar",
    "acordelen": "acordelar",
    "acordeles": "acordelar",
    "acordelo": "acordelar",
    "acordelábamos": "acordelar",
    "acordeláis": "acordelar",
    "acordeláramos": "acordelar",
    "acordeláremos": "acordelar",
    "acordelásemos": "acordelar",
    "acordelé": "acordelar",
    "acordeléis": "acordelar",
    "acordeló": "acordelar",
    "acordemos": "acordar",
    "acordeona": "acordeonar",
    "acordeonaba": "acordeonar",
    "acordeonabais": "acordeonar",
    "acordeonaban": "acordeonar",
    "acordeonabas": "acordeonar",
    "acordeonad": "acordeonar",
    "acordeonada": "acordeonar",
    "acordeonadas": "acordeonar",
    "acordeonado": "acordeonar",
    "acordeonados": "acordeonar",
    "acordeonamos": "acordeonar",
    "acordeonan": "acordeonar",
    "acordeonando": "acordeonar",
    "acordeonara": "acordeonar",
    "acordeonarais": "acordeonar",
    "acordeonaran": "acordeonar",
    "acordeonaras": "acordeonar",
    "acordeonare": "acordeonar",
    "acordeonareis": "acordeonar",
    "acordeonaremos": "acordeonar",
    "acordeonaren": "acordeonar",
    "acordeonares": "acordeonar",
    "acordeonaron": "acordeonar",
    "acordeonará": "acordeonar",
    "acordeonarán": "acordeonar",
    "acordeonarás": "acordeonar",
    "acordeonaré": "acordeonar",
    "acordeonaréis": "acordeonar",
    "acordeonaría": "acordeonar",
    "acordeonaríais": "acordeonar",
    "acordeonaríamos": "acordeonar",
    "acordeonarían": "acordeonar",
    "acordeonarías": "acordeonar",
    "acordeonas": "acordeonar",
    "acordeonase": "acordeonar",
    "acordeonaseis": "acordeonar",
    "acordeonasen": "acordeonar",
    "acordeonases": "acordeonar",
    "acordeonaste": "acordeonar",
    "acordeonasteis": "acordeonar",
    "acordeone": "acordeonar",
    "acordeonemos": "acordeonar",
    "acordeonen": "acordeonar",
    "acordeones": "acordeonar",
    "acordeonistas": "acordeonista",
    "acordeono": "acordeonar",
    "acordeonábamos": "acordeonar",
    "acordeonáis": "acordeonar",
    "acordeonáramos": "acordeonar",
    "acordeonáremos": "acordeonar",
    "acordeonásemos": "acordeonar",
    "acordeoné": "acordeonar",
    "acordeonéis": "acordeonar",
    "acordeonó": "acordeonar",
    "acordes": "acorde",
    "acordona": "acordonar",
    "acordonaba": "acordonar",
    "acordonabais": "acordonar",
    "acordonaban": "acordonar",
    "acordonabas": "acordonar",
    "acordonad": "acordonar",
    "acordonada": "acordonar",
    "acordonadas": "acordonar",
    "acordonado": "acordonar",
    "acordonados": "acordonar",
    "acordonamientos": "acordonamiento",
    "acordonamos": "acordonar",
    "acordonan": "acordonar",
    "acordonando": "acordonar",
    "acordonara": "acordonar",
    "acordonarais": "acordonar",
    "acordonaran": "acordonar",
    "acordonaras": "acordonar",
    "acordonare": "acordonar",
    "acordonareis": "acordonar",
    "acordonaremos": "acordonar",
    "acordonaren": "acordonar",
    "acordonares": "acordonar",
    "acordonaron": "acordonar",
    "acordonará": "acordonar",
    "acordonarán": "acordonar",
    "acordonarás": "acordonar",
    "acordonaré": "acordonar",
    "acordonaréis": "acordonar",
    "acordonaría": "acordonar",
    "acordonaríais": "acordonar",
    "acordonaríamos": "acordonar",
    "acordonarían": "acordonar",
    "acordonarías": "acordonar",
    "acordonas": "acordonar",
    "acordonase": "acordonar",
    "acordonaseis": "acordonar",
    "acordonasen": "acordonar",
    "acordonases": "acordonar",
    "acordonaste": "acordonar",
    "acordonasteis": "acordonar",
    "acordone": "acordonar",
    "acordonemos": "acordonar",
    "acordonen": "acordonar",
    "acordones": "acordonar",
    "acordono": "acordonar",
    "acordonábamos": "acordonar",
    "acordonáis": "acordonar",
    "acordonáramos": "acordonar",
    "acordonáremos": "acordonar",
    "acordonásemos": "acordonar",
    "acordoné": "acordonar",
    "acordonéis": "acordonar",
    "acordonó": "acordonar",
    "acordábamos": "acordar",
    "acordáis": "acordar",
    "acordáramos": "acordar",
    "acordáremos": "acordar",
    "acordásemos": "acordar",
    "acordé": "acordar",
    "acordéis": "acordar",
    "acordó": "acordar",
    "acore": "acorar",
    "acoremos": "acorar",
    "acoren": "acorar",
    "acores": "acorar",
    "acornaba": "acornar",
    "acornabais": "acornar",
    "acornaban": "acornar",
    "acornabas": "acornar",
    "acornad": "acornar",
    "acornada": "acornar",
    "acornadas": "acornar",
    "acornado": "acornar",
    "acornados": "acornar",
    "acornamos": "acornar",
    "acornando": "acornar",
    "acornara": "acornar",
    "acornarais": "acornar",
    "acornaran": "acornar",
    "acornaras": "acornar",
    "acornare": "acornar",
    "acornareis": "acornar",
    "acornaremos": "acornar",
    "acornaren": "acornar",
    "acornares": "acornar",
    "acornaron": "acornar",
    "acornará": "acornar",
    "acornarán": "acornar",
    "acornarás": "acornar",
    "acornaré": "acornar",
    "acornaréis": "acornar",
    "acornaría": "acornar",
    "acornaríais": "acornar",
    "acornaríamos": "acornar",
    "acornarían": "acornar",
    "acornarías": "acornar",
    "acornase": "acornar",
    "acornaseis": "acornar",
    "acornasen": "acornar",
    "acornases": "acornar",
    "acornaste": "acornar",
    "acornasteis": "acornar",
    "acornea": "acornear",
    "acorneaba": "acornear",
    "acorneabais": "acornear",
    "acorneaban": "acornear",
    "acorneabas": "acornear",
    "acornead": "acornear",
    "acorneada": "acornear",
    "acorneadas": "acornear",
    "acorneado": "acornear",
    "acorneados": "acornear",
    "acorneamos": "acornear",
    "acornean": "acornear",
    "acorneando": "acornear",
    "acorneara": "acornear",
    "acornearais": "acornear",
    "acornearan": "acornear",
    "acornearas": "acornear",
    "acorneare": "acornear",
    "acorneareis": "acornear",
    "acornearemos": "acornear",
    "acornearen": "acornear",
    "acorneares": "acornear",
    "acornearon": "acornear",
    "acorneará": "acornear",
    "acornearán": "acornear",
    "acornearás": "acornear",
    "acornearé": "acornear",
    "acornearéis": "acornear",
    "acornearía": "acornear",
    "acornearíais": "acornear",
    "acornearíamos": "acornear",
    "acornearían": "acornear",
    "acornearías": "acornear",
    "acorneas": "acornear",
    "acornease": "acornear",
    "acorneaseis": "acornear",
    "acorneasen": "acornear",
    "acorneases": "acornear",
    "acorneaste": "acornear",
    "acorneasteis": "acornear",
    "acornee": "acornear",
    "acorneemos": "acornear",
    "acorneen": "acornear",
    "acornees": "acornear",
    "acornemos": "acornar",
    "acorneo": "acornear",
    "acorneábamos": "acornear",
    "acorneáis": "acornear",
    "acorneáramos": "acornear",
    "acorneáremos": "acornear",
    "acorneásemos": "acornear",
    "acorneé": "acornear",
    "acorneéis": "acornear",
    "acorneó": "acornear",
    "acornábamos": "acornar",
    "acornáis": "acornar",
    "acornáramos": "acornar",
    "acornáremos": "acornar",
    "acornásemos": "acornar",
    "acorné": "acornar",
    "acornéis": "acornar",
    "acornó": "acornar",
    "acoro": "acorar",
    "acorra": "acorrer",
    "acorrala": "acorralar",
    "acorralaba": "acorralar",
    "acorralabais": "acorralar",
    "acorralaban": "acorralar",
    "acorralabas": "acorralar",
    "acorralad": "acorralar",
    "acorralada": "acorralar",
    "acorraladas": "acorralar",
    "acorralado": "acorralar",
    "acorralados": "acorralar",
    "acorralamientos": "acorralamiento",
    "acorralamos": "acorralar",
    "acorralan": "acorralar",
    "acorralando": "acorralar",
    "acorralara": "acorralar",
    "acorralarais": "acorralar",
    "acorralaran": "acorralar",
    "acorralaras": "acorralar",
    "acorralare": "acorralar",
    "acorralareis": "acorralar",
    "acorralaremos": "acorralar",
    "acorralaren": "acorralar",
    "acorralares": "acorralar",
    "acorralaron": "acorralar",
    "acorralará": "acorralar",
    "acorralarán": "acorralar",
    "acorralarás": "acorralar",
    "acorralaré": "acorralar",
    "acorralaréis": "acorralar",
    "acorralaría": "acorralar",
    "acorralaríais": "acorralar",
    "acorralaríamos": "acorralar",
    "acorralarían": "acorralar",
    "acorralarías": "acorralar",
    "acorralas": "acorralar",
    "acorralase": "acorralar",
    "acorralaseis": "acorralar",
    "acorralasen": "acorralar",
    "acorralases": "acorralar",
    "acorralaste": "acorralar",
    "acorralasteis": "acorralar",
    "acorrale": "acorralar",
    "acorralemos": "acorralar",
    "acorralen": "acorralar",
    "acorrales": "acorralar",
    "acorralo": "acorralar",
    "acorralábamos": "acorralar",
    "acorraláis": "acorralar",
    "acorraláramos": "acorralar",
    "acorraláremos": "acorralar",
    "acorralásemos": "acorralar",
    "acorralé": "acorralar",
    "acorraléis": "acorralar",
    "acorraló": "acorralar",
    "acorramos": "acorrer",
    "acorran": "acorrer",
    "acorras": "acorrer",
    "acorre": "acorrer",
    "acorred": "acorrer",
    "acorremos": "acorrer",
    "acorren": "acorrer",
    "acorreremos": "acorrer",
    "acorrerá": "acorrer",
    "acorrerán": "acorrer",
    "acorrerás": "acorrer",
    "acorreré": "acorrer",
    "acorreréis": "acorrer",
    "acorrería": "acorrer",
    "acorreríais": "acorrer",
    "acorreríamos": "acorrer",
    "acorrerían": "acorrer",
    "acorrerías": "acorrer",
    "acorres": "acorrer",
    "acorrida": "acorrer",
    "acorridas": "acorrer",
    "acorrido": "acorrer",
    "acorridos": "acorrer",
    "acorriendo": "acorrer",
    "acorriera": "acorrer",
    "acorrierais": "acorrer",
    "acorrieran": "acorrer",
    "acorrieras": "acorrer",
    "acorriere": "acorrer",
    "acorriereis": "acorrer",
    "acorrieren": "acorrer",
    "acorrieres": "acorrer",
    "acorrieron": "acorrer",
    "acorriese": "acorrer",
    "acorrieseis": "acorrer",
    "acorriesen": "acorrer",
    "acorrieses": "acorrer",
    "acorrimos": "acorrer",
    "acorriste": "acorrer",
    "acorristeis": "acorrer",
    "acorriéramos": "acorrer",
    "acorriéremos": "acorrer",
    "acorriésemos": "acorrer",
    "acorrió": "acorrer",
    "acorro": "acorrer",
    "acorráis": "acorrer",
    "acorréis": "acorrer",
    "acorrí": "acorrer",
    "acorría": "acorrer",
    "acorríais": "acorrer",
    "acorríamos": "acorrer",
    "acorrían": "acorrer",
    "acorrías": "acorrer",
    "acorta": "acortar",
    "acortaba": "acortar",
    "acortabais": "acortar",
    "acortaban": "acortar",
    "acortabas": "acortar",
    "acortables": "acortable",
    "acortad": "acortar",
    "acortada": "acortar",
    "acortadas": "acortar",
    "acortado": "acortar",
    "acortados": "acortar",
    "acortamientos": "acortamiento",
    "acortamos": "acortar",
    "acortan": "acortar",
    "acortando": "acortar",
    "acortara": "acortar",
    "acortarais": "acortar",
    "acortaran": "acortar",
    "acortaras": "acortar",
    "acortare": "acortar",
    "acortareis": "acortar",
    "acortaremos": "acortar",
    "acortaren": "acortar",
    "acortares": "acortar",
    "acortaron": "acortar",
    "acortará": "acortar",
    "acortarán": "acortar",
    "acortarás": "acortar",
    "acortaré": "acortar",
    "acortaréis": "acortar",
    "acortaría": "acortar",
    "acortaríais": "acortar",
    "acortaríamos": "acortar",
    "acortarían": "acortar",
    "acortarías": "acortar",
    "acortas": "acortar",
    "acortase": "acortar",
    "acortaseis": "acortar",
    "acortasen": "acortar",
    "acortases": "acortar",
    "acortaste": "acortar",
    "acortasteis": "acortar",
    "acorte": "acortar",
    "acortemos": "acortar",
    "acorten": "acortar",
    "acortes": "acortar",
    "acorto": "acortar",
    "acortábamos": "acortar",
    "acortáis": "acortar",
    "acortáramos": "acortar",
    "acortáremos": "acortar",
    "acortásemos": "acortar",
    "acorté": "acortar",
    "acortéis": "acortar",
    "acortó": "acortar",
    "acorábamos": "acorar",
    "acoráis": "acorar",
    "acoráramos": "acorar",
    "acoráremos": "acorar",
    "acorásemos": "acorar",
    "acoré": "acorar",
    "acoréis": "acorar",
    "acoró": "acorar",
    "acos": "aco",
    "acosa": "acosar",
    "acosaba": "acosar",
    "acosabais": "acosar",
    "acosaban": "acosar",
    "acosabas": "acosar",
    "acosad": "acosar",
    "acosada": "acosar",
    "acosadas": "acosar",
    "acosado": "acosar",
    "acosadora": "acosador",
    "acosadoras": "acosador",
    "acosadores": "acosador",
    "acosados": "acosar",
    "acosamientos": "acosamiento",
    "acosamos": "acosar",
    "acosan": "acosar",
    "acosando": "acosar",
    "acosara": "acosar",
    "acosarais": "acosar",
    "acosaran": "acosar",
    "acosaras": "acosar",
    "acosare": "acosar",
    "acosareis": "acosar",
    "acosaremos": "acosar",
    "acosaren": "acosar",
    "acosares": "acosar",
    "acosaron": "acosar",
    "acosará": "acosar",
    "acosarán": "acosar",
    "acosarás": "acosar",
    "acosaré": "acosar",
    "acosaréis": "acosar",
    "acosaría": "acosar",
    "acosaríais": "acosar",
    "acosaríamos": "acosar",
    "acosarían": "acosar",
    "acosarías": "acosar",
    "acosas": "acosar",
    "acosase": "acosar",
    "acosaseis": "acosar",
    "acosasen": "acosar",
    "acosases": "acosar",
    "acosaste": "acosar",
    "acosasteis": "acosar",
    "acose": "acosar",
    "acosemos": "acosar",
    "acosen": "acosar",
    "acoses": "acosar",
    "acosija": "acosijar",
    "acosijaba": "acosijar",
    "acosijabais": "acosijar",
    "acosijaban": "acosijar",
    "acosijabas": "acosijar",
    "acosijad": "acosijar",
    "acosijada": "acosijar",
    "acosijadas": "acosijar",
    "acosijado": "acosijar",
    "acosijados": "acosijar",
    "acosijamos": "acosijar",
    "acosijan": "acosijar",
    "acosijando": "acosijar",
    "acosijara": "acosijar",
    "acosijarais": "acosijar",
    "acosijaran": "acosijar",
    "acosijaras": "acosijar",
    "acosijare": "acosijar",
    "acosijareis": "acosijar",
    "acosijaremos": "acosijar",
    "acosijaren": "acosijar",
    "acosijares": "acosijar",
    "acosijaron": "acosijar",
    "acosijará": "acosijar",
    "acosijarán": "acosijar",
    "acosijarás": "acosijar",
    "acosijaré": "acosijar",
    "acosijaréis": "acosijar",
    "acosijaría": "acosijar",
    "acosijaríais": "acosijar",
    "acosijaríamos": "acosijar",
    "acosijarían": "acosijar",
    "acosijarías": "acosijar",
    "acosijas": "acosijar",
    "acosijase": "acosijar",
    "acosijaseis": "acosijar",
    "acosijasen": "acosijar",
    "acosijases": "acosijar",
    "acosijaste": "acosijar",
    "acosijasteis": "acosijar",
    "acosije": "acosijar",
    "acosijemos": "acosijar",
    "acosijen": "acosijar",
    "acosijes": "acosijar",
    "acosijo": "acosijar",
    "acosijábamos": "acosijar",
    "acosijáis": "acosijar",
    "acosijáramos": "acosijar",
    "acosijáremos": "acosijar",
    "acosijásemos": "acosijar",
    "acosijé": "acosijar",
    "acosijéis": "acosijar",
    "acosijó": "acosijar",
    "acoso": "acosar",
    "acosos": "acoso",
    "acostaba": "acostar",
    "acostabais": "acostar",
    "acostaban": "acostar",
    "acostabas": "acostar",
    "acostad": "acostar",
    "acostada": "acostar",
    "acostadas": "acostar",
    "acostado": "acostar",
    "acostados": "acostar",
    "acostamientos": "acostamiento",
    "acostamos": "acostar",
    "acostando": "acostar",
    "acostara": "acostar",
    "acostarais": "acostar",
    "acostaran": "acostar",
    "acostaras": "acostar",
    "acostare": "acostar",
    "acostareis": "acostar",
    "acostaremos": "acostar",
    "acostaren": "acostar",
    "acostares": "acostar",
    "acostaron": "acostar",
    "acostará": "acostar",
    "acostarán": "acostar",
    "acostarás": "acostar",
    "acostaré": "acostar",
    "acostaréis": "acostar",
    "acostaría": "acostar",
    "acostaríais": "acostar",
    "acostaríamos": "acostar",
    "acostarían": "acostar",
    "acostarías": "acostar",
    "acostase": "acostar",
    "acostaseis": "acostar",
    "acostasen": "acostar",
    "acostases": "acostar",
    "acostaste": "acostar",
    "acostasteis": "acostar",
    "acostemos": "acostar",
    "acostumbra": "acostumbrar",
    "acostumbraba": "acostumbrar",
    "acostumbrabais": "acostumbrar",
    "acostumbraban": "acostumbrar",
    "acostumbrabas": "acostumbrar",
    "acostumbrad": "acostumbrar",
    "acostumbrada": "acostumbrar",
    "acostumbradas": "acostumbrar",
    "acostumbrado": "acostumbrar",
    "acostumbrados": "acostumbrar",
    "acostumbramos": "acostumbrar",
    "acostumbran": "acostumbrar",
    "acostumbrando": "acostumbrar",
    "acostumbrara": "acostumbrar",
    "acostumbrarais": "acostumbrar",
    "acostumbraran": "acostumbrar",
    "acostumbraras": "acostumbrar",
    "acostumbrare": "acostumbrar",
    "acostumbrareis": "acostumbrar",
    "acostumbraremos": "acostumbrar",
    "acostumbraren": "acostumbrar",
    "acostumbrares": "acostumbrar",
    "acostumbraron": "acostumbrar",
    "acostumbrará": "acostumbrar",
    "acostumbrarán": "acostumbrar",
    "acostumbrarás": "acostumbrar",
    "acostumbraré": "acostumbrar",
    "acostumbraréis": "acostumbrar",
    "acostumbraría": "acostumbrar",
    "acostumbraríais": "acostumbrar",
    "acostumbraríamos": "acostumbrar",
    "acostumbrarían": "acostumbrar",
    "acostumbrarías": "acostumbrar",
    "acostumbras": "acostumbrar",
    "acostumbrase": "acostumbrar",
    "acostumbraseis": "acostumbrar",
    "acostumbrasen": "acostumbrar",
    "acostumbrases": "acostumbrar",
    "acostumbraste": "acostumbrar",
    "acostumbrasteis": "acostumbrar",
    "acostumbre": "acostumbrar",
    "acostumbremos": "acostumbrar",
    "acostumbren": "acostumbrar",
    "acostumbres": "acostumbrar",
    "acostumbro": "acostumbrar",
    "acostumbrábamos": "acostumbrar",
    "acostumbráis": "acostumbrar",
    "acostumbráramos": "acostumbrar",
    "acostumbráremos": "acostumbrar",
    "acostumbrásemos": "acostumbrar",
    "acostumbré": "acostumbrar",
    "acostumbréis": "acostumbrar",
    "acostumbró": "acostumbrar",
    "acostábamos": "acostar",
    "acostáis": "acostar",
    "acostáramos": "acostar",
    "acostáremos": "acostar",
    "acostásemos": "acostar",
    "acosté": "acostar",
    "acostéis": "acostar",
    "acostó": "acostar",
    "acosábamos": "acosar",
    "acosáis": "acosar",
    "acosáramos": "acosar",
    "acosáremos": "acosar",
    "acosásemos": "acosar",
    "acosé": "acosar",
    "acoséis": "acosar",
    "acosó": "acosar",
    "acota": "acotar",
    "acotaba": "acotar",
    "acotabais": "acotar",
    "acotaban": "acotar",
    "acotabas": "acotar",
    "acotaciones": "acotación",
    "acotad": "acotar",
    "acotada": "acotar",
    "acotadas": "acotar",
    "acotado": "acotar",
    "acotados": "acotar",
    "acotamientos": "acotamiento",
    "acotamos": "acotar",
    "acotan": "acotar",
    "acotando": "acotar",
    "acotara": "acotar",
    "acotarais": "acotar",
    "acotaran": "acotar",
    "acotaras": "acotar",
    "acotare": "acotar",
    "acotareis": "acotar",
    "acotaremos": "acotar",
    "acotaren": "acotar",
    "acotares": "acotar",
    "acotaron": "acotar",
    "acotará": "acotar",
    "acotarán": "acotar",
    "acotarás": "acotar",
    "acotaré": "acotar",
    "acotaréis": "acotar",
    "acotaría": "acotar",
    "acotaríais": "acotar",
    "acotaríamos": "acotar",
    "acotarían": "acotar",
    "acotarías": "acotar",
    "acotas": "acotar",
    "acotase": "acotar",
    "acotaseis": "acotar",
    "acotasen": "acotar",
    "acotases": "acotar",
    "acotaste": "acotar",
    "acotasteis": "acotar",
    "acote": "acotar",
    "acoteja": "acotejar",
    "acotejaba": "acotejar",
    "acotejabais": "acotejar",
    "acotejaban": "acotejar",
    "acotejabas": "acotejar",
    "acotejad": "acotejar",
    "acotejada": "acotejar",
    "acotejadas": "acotejar",
    "acotejado": "acotejar",
    "acotejados": "acotejar",
    "acotejamos": "acotejar",
    "acotejan": "acotejar",
    "acotejando": "acotejar",
    "acotejara": "acotejar",
    "acotejarais": "acotejar",
    "acotejaran": "acotejar",
    "acotejaras": "acotejar",
    "acotejare": "acotejar",
    "acotejareis": "acotejar",
    "acotejaremos": "acotejar",
    "acotejaren": "acotejar",
    "acotejares": "acotejar",
    "acotejaron": "acotejar",
    "acotejará": "acotejar",
    "acotejarán": "acotejar",
    "acotejarás": "acotejar",
    "acotejaré": "acotejar",
    "acotejaréis": "acotejar",
    "acotejaría": "acotejar",
    "acotejaríais": "acotejar",
    "acotejaríamos": "acotejar",
    "acotejarían": "acotejar",
    "acotejarías": "acotejar",
    "acotejas": "acotejar",
    "acotejase": "acotejar",
    "acotejaseis": "acotejar",
    "acotejasen": "acotejar",
    "acotejases": "acotejar",
    "acotejaste": "acotejar",
    "acotejasteis": "acotejar",
    "acoteje": "acotejar",
    "acotejemos": "acotejar",
    "acotejen": "acotejar",
    "acotejes": "acotejar",
    "acotejo": "acotejar",
    "acotejábamos": "acotejar",
    "acotejáis": "acotejar",
    "acotejáramos": "acotejar",
    "acotejáremos": "acotejar",
    "acotejásemos": "acotejar",
    "acotejé": "acotejar",
    "acotejéis": "acotejar",
    "acotejó": "acotejar",
    "acotemos": "acotar",
    "acoten": "acotar",
    "acotes": "acotar",
    "acotiledónea": "acotiledóneo",
    "acotiledóneas": "acotiledóneo",
    "acotiledóneos": "acotiledóneo",
    "acotillos": "acotillo",
    "acoto": "acotar",
    "acotábamos": "acotar",
    "acotáis": "acotar",
    "acotáramos": "acotar",
    "acotáremos": "acotar",
    "acotásemos": "acotar",
    "acoté": "acotar",
    "acotéis": "acotar",
    "acotó": "acotar",
    "acoyunda": "acoyundar",
    "acoyundaba": "acoyundar",
    "acoyundabais": "acoyundar",
    "acoyundaban": "acoyundar",
    "acoyundabas": "acoyundar",
    "acoyundad": "acoyundar",
    "acoyundada": "acoyundar",
    "acoyundadas": "acoyundar",
    "acoyundado": "acoyundar",
    "acoyundados": "acoyundar",
    "acoyundamos": "acoyundar",
    "acoyundan": "acoyundar",
    "acoyundando": "acoyundar",
    "acoyundara": "acoyundar",
    "acoyundarais": "acoyundar",
    "acoyundaran": "acoyundar",
    "acoyundaras": "acoyundar",
    "acoyundare": "acoyundar",
    "acoyundareis": "acoyundar",
    "acoyundaremos": "acoyundar",
    "acoyundaren": "acoyundar",
    "acoyundares": "acoyundar",
    "acoyundaron": "acoyundar",
    "acoyundará": "acoyundar",
    "acoyundarán": "acoyundar",
    "acoyundarás": "acoyundar",
    "acoyundaré": "acoyundar",
    "acoyundaréis": "acoyundar",
    "acoyundaría": "acoyundar",
    "acoyundaríais": "acoyundar",
    "acoyundaríamos": "acoyundar",
    "acoyundarían": "acoyundar",
    "acoyundarías": "acoyundar",
    "acoyundas": "acoyundar",
    "acoyundase": "acoyundar",
    "acoyundaseis": "acoyundar",
    "acoyundasen": "acoyundar",
    "acoyundases": "acoyundar",
    "acoyundaste": "acoyundar",
    "acoyundasteis": "acoyundar",
    "acoyunde": "acoyundar",
    "acoyundemos": "acoyundar",
    "acoyunden": "acoyundar",
    "acoyundes": "acoyundar",
    "acoyundo": "acoyundar",
    "acoyundábamos": "acoyundar",
    "acoyundáis": "acoyundar",
    "acoyundáramos": "acoyundar",
    "acoyundáremos": "acoyundar",
    "acoyundásemos": "acoyundar",
    "acoyundé": "acoyundar",
    "acoyundéis": "acoyundar",
    "acoyundó": "acoyundar",
    "acoyunta": "acoyuntar",
    "acoyuntaba": "acoyuntar",
    "acoyuntabais": "acoyuntar",
    "acoyuntaban": "acoyuntar",
    "acoyuntabas": "acoyuntar",
    "acoyuntad": "acoyuntar",
    "acoyuntada": "acoyuntar",
    "acoyuntadas": "acoyuntar",
    "acoyuntado": "acoyuntar",
    "acoyuntados": "acoyuntar",
    "acoyuntamos": "acoyuntar",
    "acoyuntan": "acoyuntar",
    "acoyuntando": "acoyuntar",
    "acoyuntara": "acoyuntar",
    "acoyuntarais": "acoyuntar",
    "acoyuntaran": "acoyuntar",
    "acoyuntaras": "acoyuntar",
    "acoyuntare": "acoyuntar",
    "acoyuntareis": "acoyuntar",
    "acoyuntaremos": "acoyuntar",
    "acoyuntaren": "acoyuntar",
    "acoyuntares": "acoyuntar",
    "acoyuntaron": "acoyuntar",
    "acoyuntará": "acoyuntar",
    "acoyuntarán": "acoyuntar",
    "acoyuntarás": "acoyuntar",
    "acoyuntaré": "acoyuntar",
    "acoyuntaréis": "acoyuntar",
    "acoyuntaría": "acoyuntar",
    "acoyuntaríais": "acoyuntar",
    "acoyuntaríamos": "acoyuntar",
    "acoyuntarían": "acoyuntar",
    "acoyuntarías": "acoyuntar",
    "acoyuntas": "acoyuntar",
    "acoyuntase": "acoyuntar",
    "acoyuntaseis": "acoyuntar",
    "acoyuntasen": "acoyuntar",
    "acoyuntases": "acoyuntar",
    "acoyuntaste": "acoyuntar",
    "acoyuntasteis": "acoyuntar",
    "acoyunte": "acoyuntar",
    "acoyuntemos": "acoyuntar",
    "acoyunten": "acoyuntar",
    "acoyunteros": "acoyuntero",
    "acoyuntes": "acoyuntar",
    "acoyunto": "acoyuntar",
    "acoyuntábamos": "acoyuntar",
    "acoyuntáis": "acoyuntar",
    "acoyuntáramos": "acoyuntar",
    "acoyuntáremos": "acoyuntar",
    "acoyuntásemos": "acoyuntar",
    "acoyunté": "acoyuntar",
    "acoyuntéis": "acoyuntar",
    "acoyuntó": "acoyuntar",
    "acracias": "acracia",
    "acrece": "acrecer",
    "acreced": "acrecer",
    "acrecemos": "acrecer",
    "acrecen": "acrecer",
    "acrecencias": "acrecencia",
    "acrecentaba": "acrecentar",
    "acrecentabais": "acrecentar",
    "acrecentaban": "acrecentar",
    "acrecentabas": "acrecentar",
    "acrecentad": "acrecentar",
    "acrecentada": "acrecentar",
    "acrecentadas": "acrecentar",
    "acrecentado": "acrecentar",
    "acrecentadora": "acrecentador",
    "acrecentadoras": "acrecentador",
    "acrecentadores": "acrecentador",
    "acrecentados": "acrecentar",
    "acrecentamientos": "acrecentamiento",
    "acrecentamos": "acrecentar",
    "acrecentando": "acrecentar",
    "acrecentara": "acrecentar",
    "acrecentarais": "acrecentar",
    "acrecentaran": "acrecentar",
    "acrecentaras": "acrecentar",
    "acrecentare": "acrecentar",
    "acrecentareis": "acrecentar",
    "acrecentaremos": "acrecentar",
    "acrecentaren": "acrecentar",
    "acrecentares": "acrecentar",
    "acrecentaron": "acrecentar",
    "acrecentará": "acrecentar",
    "acrecentarán": "acrecentar",
    "acrecentarás": "acrecentar",
    "acrecentaré": "acrecentar",
    "acrecentaréis": "acrecentar",
    "acrecentaría": "acrecentar",
    "acrecentaríais": "acrecentar",
    "acrecentaríamos": "acrecentar",
    "acrecentarían": "acrecentar",
    "acrecentarías": "acrecentar",
    "acrecentase": "acrecentar",
    "acrecentaseis": "acrecentar",
    "acrecentasen": "acrecentar",
    "acrecentases": "acrecentar",
    "acrecentaste": "acrecentar",
    "acrecentasteis": "acrecentar",
    "acrecentemos": "acrecentar",
    "acrecentábamos": "acrecentar",
    "acrecentáis": "acrecentar",
    "acrecentáramos": "acrecentar",
    "acrecentáremos": "acrecentar",
    "acrecentásemos": "acrecentar",
    "acrecenté": "acrecentar",
    "acrecentéis": "acrecentar",
    "acrecentó": "acrecentar",
    "acreceremos": "acrecer",
    "acrecerá": "acrecer",
    "acrecerán": "acrecer",
    "acrecerás": "acrecer",
    "acreceré": "acrecer",
    "acreceréis": "acrecer",
    "acrecería": "acrecer",
    "acreceríais": "acrecer",
    "acreceríamos": "acrecer",
    "acrecerían": "acrecer",
    "acrecerías": "acrecer",
    "acreces": "acrecer",
    "acrecida": "acrecer",
    "acrecidas": "acrecer",
    "acrecido": "acrecer",
    "acrecidos": "acrecer",
    "acreciendo": "acrecer",
    "acrecienta": "acrecentar",
    "acrecientan": "acrecentar",
    "acrecientas": "acrecentar",
    "acreciente": "acrecentar",
    "acrecienten": "acrecentar",
    "acrecientes": "acrecentar",
    "acreciento": "acrecentar",
    "acreciera": "acrecer",
    "acrecierais": "acrecer",
    "acrecieran": "acrecer",
    "acrecieras": "acrecer",
    "acreciere": "acrecer",
    "acreciereis": "acrecer",
    "acrecieren": "acrecer",
    "acrecieres": "acrecer",
    "acrecieron": "acrecer",
    "acreciese": "acrecer",
    "acrecieseis": "acrecer",
    "acreciesen": "acrecer",
    "acrecieses": "acrecer",
    "acrecimientos": "acrecimiento",
    "acrecimos": "acrecer",
    "acreciones": "acreción",
    "acreciste": "acrecer",
    "acrecisteis": "acrecer",
    "acreciéramos": "acrecer",
    "acreciéremos": "acrecer",
    "acreciésemos": "acrecer",
    "acreció": "acrecer",
    "acrecéis": "acrecer",
    "acrecí": "acrecer",
    "acrecía": "acrecer",
    "acrecíais": "acrecer",
    "acrecíamos": "acrecer",
    "acrecían": "acrecer",
    "acrecías": "acrecer",
    "acredita": "acreditar",
    "acreditaba": "acreditar",
    "acreditabais": "acreditar",
    "acreditaban": "acreditar",
    "acreditabas": "acreditar",
    "acreditaciones": "acreditación",
    "acreditad": "acreditar",
    "acreditada": "acreditar",
    "acreditadas": "acreditar",
    "acreditado": "acreditar",
    "acreditados": "acreditar",
    "acreditamientos": "acreditamiento",
    "acreditamos": "acreditar",
    "acreditan": "acreditar",
    "acreditando": "acreditar",
    "acreditantes": "acreditante",
    "acreditara": "acreditar",
    "acreditarais": "acreditar",
    "acreditaran": "acreditar",
    "acreditaras": "acreditar",
    "acreditare": "acreditar",
    "acreditareis": "acreditar",
    "acreditaremos": "acreditar",
    "acreditaren": "acreditar",
    "acreditares": "acreditar",
    "acreditaron": "acreditar",
    "acreditará": "acreditar",
    "acreditarán": "acreditar",
    "acreditarás": "acreditar",
    "acreditaré": "acreditar",
    "acreditaréis": "acreditar",
    "acreditaría": "acreditar",
    "acreditaríais": "acreditar",
    "acreditaríamos": "acreditar",
    "acreditarían": "acreditar",
    "acreditarías": "acreditar",
    "acreditas": "acreditar",
    "acreditase": "acreditar",
    "acreditaseis": "acreditar",
    "acreditasen": "acreditar",
    "acreditases": "acreditar",
    "acreditaste": "acreditar",
    "acreditasteis": "acreditar",
    "acreditativa": "acreditativo",
    "acreditativas": "acreditativo",
    "acreditativos": "acreditativo",
    "acredite": "acreditar",
    "acreditemos": "acreditar",
    "acrediten": "acreditar",
    "acredites": "acreditar",
    "acredito": "acreditar",
    "acreditábamos": "acreditar",
    "acreditáis": "acreditar",
    "acreditáramos": "acreditar",
    "acreditáremos": "acreditar",
    "acreditásemos": "acreditar",
    "acredité": "acreditar",
    "acreditéis": "acreditar",
    "acreditó": "acreditar",
    "acreedora": "acreedor",
    "acreedoras": "acreedor",
    "acreedores": "acreedor",
    "acres": "acre",
    "acrescentes": "acrescente",
    "acrezca": "acrecer",
    "acrezcamos": "acrecer",
    "acrezcan": "acrecer",
    "acrezcas": "acrecer",
    "acrezco": "acrecer",
    "acrezcáis": "acrecer",
    "acriba": "acribar",
    "acribaba": "acribar",
    "acribabais": "acribar",
    "acribaban": "acribar",
    "acribabas": "acribar",
    "acribad": "acribar",
    "acribada": "acribar",
    "acribadas": "acribar",
    "acribado": "acribar",
    "acribadora": "acribador",
    "acribadoras": "acribador",
    "acribadores": "acribador",
    "acribados": "acribar",
    "acribaduras": "acribadura",
    "acribamos": "acribar",
    "acriban": "acribar",
    "acribando": "acribar",
    "acribara": "acribar",
    "acribarais": "acribar",
    "acribaran": "acribar",
    "acribaras": "acribar",
    "acribare": "acribar",
    "acribareis": "acribar",
    "acribaremos": "acribar",
    "acribaren": "acribar",
    "acribares": "acribar",
    "acribaron": "acribar",
    "acribará": "acribar",
    "acribarán": "acribar",
    "acribarás": "acribar",
    "acribaré": "acribar",
    "acribaréis": "acribar",
    "acribaría": "acribar",
    "acribaríais": "acribar",
    "acribaríamos": "acribar",
    "acribarían": "acribar",
    "acribarías": "acribar",
    "acribas": "acribar",
    "acribase": "acribar",
    "acribaseis": "acribar",
    "acribasen": "acribar",
    "acribases": "acribar",
    "acribaste": "acribar",
    "acribasteis": "acribar",
    "acribe": "acribar",
    "acribemos": "acribar",
    "acriben": "acribar",
    "acribes": "acribar",
    "acribilla": "acribillar",
    "acribillaba": "acribillar",
    "acribillabais": "acribillar",
    "acribillaban": "acribillar",
    "acribillabas": "acribillar",
    "acribillad": "acribillar",
    "acribillada": "acribillar",
    "acribilladas": "acribillar",
    "acribillado": "acribillar",
    "acribillados": "acribillar",
    "acribillamientos": "acribillamiento",
    "acribillamos": "acribillar",
    "acribillan": "acribillar",
    "acribillando": "acribillar",
    "acribillara": "acribillar",
    "acribillarais": "acribillar",
    "acribillaran": "acribillar",
    "acribillaras": "acribillar",
    "acribillare": "acribillar",
    "acribillareis": "acribillar",
    "acribillaremos": "acribillar",
    "acribillaren": "acribillar",
    "acribillares": "acribillar",
    "acribillaron": "acribillar",
    "acribillará": "acribillar",
    "acribillarán": "acribillar",
    "acribillarás": "acribillar",
    "acribillaré": "acribillar",
    "acribillaréis": "acribillar",
    "acribillaría": "acribillar",
    "acribillaríais": "acribillar",
    "acribillaríamos": "acribillar",
    "acribillarían": "acribillar",
    "acribillarías": "acribillar",
    "acribillas": "acribillar",
    "acribillase": "acribillar",
    "acribillaseis": "acribillar",
    "acribillasen": "acribillar",
    "acribillases": "acribillar",
    "acribillaste": "acribillar",
    "acribillasteis": "acribillar",
    "acribille": "acribillar",
    "acribillemos": "acribillar",
    "acribillen": "acribillar",
    "acribilles": "acribillar",
    "acribillo": "acribillar",
    "acribillábamos": "acribillar",
    "acribilláis": "acribillar",
    "acribilláramos": "acribillar",
    "acribilláremos": "acribillar",
    "acribillásemos": "acribillar",
    "acribillé": "acribillar",
    "acribilléis": "acribillar",
    "acribilló": "acribillar",
    "acribo": "acribar",
    "acribábamos": "acribar",
    "acribáis": "acribar",
    "acribáramos": "acribar",
    "acribáremos": "acribar",
    "acribásemos": "acribar",
    "acribé": "acribar",
    "acribéis": "acribar",
    "acribó": "acribar",
    "acridinas": "acridina",
    "acridonas": "acridona",
    "acriflavinas": "acriflavina",
    "acrilamidas": "acrilamida",
    "acrilatos": "acrilato",
    "acrilonitrilos": "acrilonitrilo",
    "acrilonitrílica": "acrilonitrílico",
    "acrilonitrílicas": "acrilonitrílico",
    "acrilonitrílicos": "acrilonitrílico",
    "acrimina": "acriminar",
    "acriminaba": "acriminar",
    "acriminabais": "acriminar",
    "acriminaban": "acriminar",
    "acriminabas": "acriminar",
    "acriminaciones": "acriminación",
    "acriminad": "acriminar",
    "acriminada": "acriminar",
    "acriminadas": "acriminar",
    "acriminado": "acriminar",
    "acriminadora": "acriminador",
    "acriminadoras": "acriminador",
    "acriminadores": "acriminador",
    "acriminados": "acriminar",
    "acriminamos": "acriminar",
    "acriminan": "acriminar",
    "acriminando": "acriminar",
    "acriminara": "acriminar",
    "acriminarais": "acriminar",
    "acriminaran": "acriminar",
    "acriminaras": "acriminar",
    "acriminare": "acriminar",
    "acriminareis": "acriminar",
    "acriminaremos": "acriminar",
    "acriminaren": "acriminar",
    "acriminares": "acriminar",
    "acriminaron": "acriminar",
    "acriminará": "acriminar",
    "acriminarán": "acriminar",
    "acriminarás": "acriminar",
    "acriminaré": "acriminar",
    "acriminaréis": "acriminar",
    "acriminaría": "acriminar",
    "acriminaríais": "acriminar",
    "acriminaríamos": "acriminar",
    "acriminarían": "acriminar",
    "acriminarías": "acriminar",
    "acriminas": "acriminar",
    "acriminase": "acriminar",
    "acriminaseis": "acriminar",
    "acriminasen": "acriminar",
    "acriminases": "acriminar",
    "acriminaste": "acriminar",
    "acriminasteis": "acriminar",
    "acrimine": "acriminar",
    "acriminemos": "acriminar",
    "acriminen": "acriminar",
    "acrimines": "acriminar",
    "acrimino": "acriminar",
    "acriminábamos": "acriminar",
    "acrimináis": "acriminar",
    "acrimináramos": "acriminar",
    "acrimináremos": "acriminar",
    "acriminásemos": "acriminar",
    "acriminé": "acriminar",
    "acriminéis": "acriminar",
    "acriminó": "acriminar",
    "acrimonias": "acrimonia",
    "acrimoniosa": "acrimonioso",
    "acrimoniosas": "acrimonioso",
    "acrimoniosos": "acrimonioso",
    "acriolla": "acriollar",
    "acriollaba": "acriollar",
    "acriollabais": "acriollar",
    "acriollaban": "acriollar",
    "acriollabas": "acriollar",
    "acriollad": "acriollar",
    "acriollada": "acriollar",
    "acriolladas": "acriollar",
    "acriollado": "acriollar",
    "acriollados": "acriollar",
    "acriollamos": "acriollar",
    "acriollan": "acriollar",
    "acriollando": "acriollar",
    "acriollara": "acriollar",
    "acriollarais": "acriollar",
    "acriollaran": "acriollar",
    "acriollaras": "acriollar",
    "acriollare": "acriollar",
    "acriollareis": "acriollar",
    "acriollaremos": "acriollar",
    "acriollaren": "acriollar",
    "acriollares": "acriollar",
    "acriollaron": "acriollar",
    "acriollará": "acriollar",
    "acriollarán": "acriollar",
    "acriollarás": "acriollar",
    "acriollaré": "acriollar",
    "acriollaréis": "acriollar",
    "acriollaría": "acriollar",
    "acriollaríais": "acriollar",
    "acriollaríamos": "acriollar",
    "acriollarían": "acriollar",
    "acriollarías": "acriollar",
    "acriollas": "acriollar",
    "acriollase": "acriollar",
    "acriollaseis": "acriollar",
    "acriollasen": "acriollar",
    "acriollases": "acriollar",
    "acriollaste": "acriollar",
    "acriollasteis": "acriollar",
    "acriolle": "acriollar",
    "acriollemos": "acriollar",
    "acriollen": "acriollar",
    "acriolles": "acriollar",
    "acriollo": "acriollar",
    "acriollábamos": "acriollar",
    "acriolláis": "acriollar",
    "acriolláramos": "acriollar",
    "acriolláremos": "acriollar",
    "acriollásemos": "acriollar",
    "acriollé": "acriollar",
    "acriolléis": "acriollar",
    "acriolló": "acriollar",
    "acrisola": "acrisolar",
    "acrisolaba": "acrisolar",
    "acrisolabais": "acrisolar",
    "acrisolaban": "acrisolar",
    "acrisolabas": "acrisolar",
    "acrisolad": "acrisolar",
    "acrisolada": "acrisolar",
    "acrisoladas": "acrisolar",
    "acrisolado": "acrisolar",
    "acrisolados": "acrisolar",
    "acrisolamos": "acrisolar",
    "acrisolan": "acrisolar",
    "acrisolando": "acrisolar",
    "acrisolara": "acrisolar",
    "acrisolarais": "acrisolar",
    "acrisolaran": "acrisolar",
    "acrisolaras": "acrisolar",
    "acrisolare": "acrisolar",
    "acrisolareis": "acrisolar",
    "acrisolaremos": "acrisolar",
    "acrisolaren": "acrisolar",
    "acrisolares": "acrisolar",
    "acrisolaron": "acrisolar",
    "acrisolará": "acrisolar",
    "acrisolarán": "acrisolar",
    "acrisolarás": "acrisolar",
    "acrisolaré": "acrisolar",
    "acrisolaréis": "acrisolar",
    "acrisolaría": "acrisolar",
    "acrisolaríais": "acrisolar",
    "acrisolaríamos": "acrisolar",
    "acrisolarían": "acrisolar",
    "acrisolarías": "acrisolar",
    "acrisolas": "acrisolar",
    "acrisolase": "acrisolar",
    "acrisolaseis": "acrisolar",
    "acrisolasen": "acrisolar",
    "acrisolases": "acrisolar",
    "acrisolaste": "acrisolar",
    "acrisolasteis": "acrisolar",
    "acrisole": "acrisolar",
    "acrisolemos": "acrisolar",
    "acrisolen": "acrisolar",
    "acrisoles": "acrisolar",
    "acrisolo": "acrisolar",
    "acrisolábamos": "acrisolar",
    "acrisoláis": "acrisolar",
    "acrisoláramos": "acrisolar",
    "acrisoláremos": "acrisolar",
    "acrisolásemos": "acrisolar",
    "acrisolé": "acrisolar",
    "acrisoléis": "acrisolar",
    "acrisoló": "acrisolar",
    "acristala": "acristalar",
    "acristalaba": "acristalar",
    "acristalabais": "acristalar",
    "acristalaban": "acristalar",
    "acristalabas": "acristalar",
    "acristalad": "acristalar",
    "acristalada": "acristalar",
    "acristaladas": "acristalar",
    "acristalado": "acristalar",
    "acristalados": "acristalar",
    "acristalamientos": "acristalamiento",
    "acristalamos": "acristalar",
    "acristalan": "acristalar",
    "acristalando": "acristalar",
    "acristalara": "acristalar",
    "acristalarais": "acristalar",
    "acristalaran": "acristalar",
    "acristalaras": "acristalar",
    "acristalare": "acristalar",
    "acristalareis": "acristalar",
    "acristalaremos": "acristalar",
    "acristalaren": "acristalar",
    "acristalares": "acristalar",
    "acristalaron": "acristalar",
    "acristalará": "acristalar",
    "acristalarán": "acristalar",
    "acristalarás": "acristalar",
    "acristalaré": "acristalar",
    "acristalaréis": "acristalar",
    "acristalaría": "acristalar",
    "acristalaríais": "acristalar",
    "acristalaríamos": "acristalar",
    "acristalarían": "acristalar",
    "acristalarías": "acristalar",
    "acristalas": "acristalar",
    "acristalase": "acristalar",
    "acristalaseis": "acristalar",
    "acristalasen": "acristalar",
    "acristalases": "acristalar",
    "acristalaste": "acristalar",
    "acristalasteis": "acristalar",
    "acristale": "acristalar",
    "acristalemos": "acristalar",
    "acristalen": "acristalar",
    "acristales": "acristalar",
    "acristalo": "acristalar",
    "acristalábamos": "acristalar",
    "acristaláis": "acristalar",
    "acristaláramos": "acristalar",
    "acristaláremos": "acristalar",
    "acristalásemos": "acristalar",
    "acristalé": "acristalar",
    "acristaléis": "acristalar",
    "acristaló": "acristalar",
    "acristiana": "acristianar",
    "acristianaba": "acristianar",
    "acristianabais": "acristianar",
    "acristianaban": "acristianar",
    "acristianabas": "acristianar",
    "acristianad": "acristianar",
    "acristianada": "acristianar",
    "acristianadas": "acristianar",
    "acristianado": "acristianar",
    "acristianados": "acristianar",
    "acristianamos": "acristianar",
    "acristianan": "acristianar",
    "acristianando": "acristianar",
    "acristianara": "acristianar",
    "acristianarais": "acristianar",
    "acristianaran": "acristianar",
    "acristianaras": "acristianar",
    "acristianare": "acristianar",
    "acristianareis": "acristianar",
    "acristianaremos": "acristianar",
    "acristianaren": "acristianar",
    "acristianares": "acristianar",
    "acristianaron": "acristianar",
    "acristianará": "acristianar",
    "acristianarán": "acristianar",
    "acristianarás": "acristianar",
    "acristianaré": "acristianar",
    "acristianaréis": "acristianar",
    "acristianaría": "acristianar",
    "acristianaríais": "acristianar",
    "acristianaríamos": "acristianar",
    "acristianarían": "acristianar",
    "acristianarías": "acristianar",
    "acristianas": "acristianar",
    "acristianase": "acristianar",
    "acristianaseis": "acristianar",
    "acristianasen": "acristianar",
    "acristianases": "acristianar",
    "acristianaste": "acristianar",
    "acristianasteis": "acristianar",
    "acristiane": "acristianar",
    "acristianemos": "acristianar",
    "acristianen": "acristianar",
    "acristianes": "acristianar",
    "acristiano": "acristianar",
    "acristianábamos": "acristianar",
    "acristianáis": "acristianar",
    "acristianáramos": "acristianar",
    "acristianáremos": "acristianar",
    "acristianásemos": "acristianar",
    "acristiané": "acristianar",
    "acristianéis": "acristianar",
    "acristianó": "acristianar",
    "acritudes": "acritud",
    "acrobacias": "acrobacia",
    "acrobatismos": "acrobatismo",
    "acroblastos": "acroblasto",
    "acrobática": "acrobático",
    "acrobáticas": "acrobático",
    "acrobáticos": "acrobático",
    "acrocefalias": "acrocefalia",
    "acrocéfala": "acrocéfalo",
    "acrocéfalas": "acrocéfalo",
    "acrocéfalos": "acrocéfalo",
    "acrocéntrica": "acrocéntrico",
    "acrocéntricas": "acrocéntrico",
    "acrocéntricos": "acrocéntrico",
    "acrodextrinas": "acrodextrina",
    "acrodinias": "acrodinia",
    "acrodonta": "acrodonto",
    "acrodontas": "acrodonto",
    "acrodontos": "acrodonto",
    "acrogénica": "acrogénico",
    "acrogénicas": "acrogénico",
    "acrogénicos": "acrogénico",
    "acroitas": "acroita",
    "acroleínas": "acroleína",
    "acromatice": "acromatizar",
    "acromaticemos": "acromatizar",
    "acromaticen": "acromatizar",
    "acromatices": "acromatizar",
    "acromaticé": "acromatizar",
    "acromaticéis": "acromatizar",
    "acromatismos": "acromatismo",
    "acromatiza": "acromatizar",
    "acromatizaba": "acromatizar",
    "acromatizabais": "acromatizar",
    "acromatizaban": "acromatizar",
    "acromatizabas": "acromatizar",
    "acromatizad": "acromatizar",
    "acromatizada": "acromatizar",
    "acromatizadas": "acromatizar",
    "acromatizado": "acromatizar",
    "acromatizados": "acromatizar",
    "acromatizamos": "acromatizar",
    "acromatizan": "acromatizar",
    "acromatizando": "acromatizar",
    "acromatizara": "acromatizar",
    "acromatizarais": "acromatizar",
    "acromatizaran": "acromatizar",
    "acromatizaras": "acromatizar",
    "acromatizare": "acromatizar",
    "acromatizareis": "acromatizar",
    "acromatizaremos": "acromatizar",
    "acromatizaren": "acromatizar",
    "acromatizares": "acromatizar",
    "acromatizaron": "acromatizar",
    "acromatizará": "acromatizar",
    "acromatizarán": "acromatizar",
    "acromatizarás": "acromatizar",
    "acromatizaré": "acromatizar",
    "acromatizaréis": "acromatizar",
    "acromatizaría": "acromatizar",
    "acromatizaríais": "acromatizar",
    "acromatizaríamos": "acromatizar",
    "acromatizarían": "acromatizar",
    "acromatizarías": "acromatizar",
    "acromatizas": "acromatizar",
    "acromatizase": "acromatizar",
    "acromatizaseis": "acromatizar",
    "acromatizasen": "acromatizar",
    "acromatizases": "acromatizar",
    "acromatizaste": "acromatizar",
    "acromatizasteis": "acromatizar",
    "acromatizo": "acromatizar",
    "acromatizábamos": "acromatizar",
    "acromatizáis": "acromatizar",
    "acromatizáramos": "acromatizar",
    "acromatizáremos": "acromatizar",
    "acromatizásemos": "acromatizar",
    "acromatizó": "acromatizar",
    "acromatopsias": "acromatopsia",
    "acromegalias": "acromegalia",
    "acromegálica": "acromegálico",
    "acromegálicas": "acromegálico",
    "acromegálicos": "acromegálico",
    "acromiales": "acromial",
    "acromias": "acromia",
    "acromioclaviculares": "acromioclavicular",
    "acromiocoraidea": "acromiocoraideo",
    "acromiocoraideas": "acromiocoraideo",
    "acromiocoraideos": "acromiocoraideo",
    "acromiones": "acromion",
    "acromiotorácica": "acromiotorácico",
    "acromiotorácicas": "acromiotorácico",
    "acromiotorácicos": "acromiotorácico",
    "acromática": "acromático",
    "acromáticas": "acromático",
    "acromáticos": "acromático",
    "acromías": "acromía",
    "acrones": "acron",
    "acronimias": "acronimia",
    "acrosinas": "acrosina",
    "acrosomas": "acrosoma",
    "acrostolios": "acrostolio",
    "acrosómica": "acrosómico",
    "acrosómicas": "acrosómico",
    "acrosómicos": "acrosómico",
    "acroteras": "acrotera",
    "acroterias": "acroteria",
    "acroterios": "acroterio",
    "acrática": "acrático",
    "acráticas": "acrático",
    "acráticos": "acrático",
    "acrídidos": "acrídido",
    "acrílica": "acrílico",
    "acrílicas": "acrílico",
    "acrílicos": "acrílico",
    "acróbatas": "acróbata",
    "acrómiones": "acromion",
    "acrónica": "acrónico",
    "acrónicas": "acrónico",
    "acrónicos": "acrónico",
    "acrónimos": "acrónimo",
    "acróstica": "acróstico",
    "acrósticas": "acróstico",
    "acrósticos": "acróstico",
    "actas": "acta",
    "actinas": "actina",
    "actiniarios": "actiniario",
    "actinias": "actinia",
    "actinidiáceas": "actinidiácea",
    "actinios": "actinio",
    "actinismos": "actinismo",
    "actinobiologías": "actinobiología",
    "actinoblastos": "actinoblasto",
    "actinolitas": "actinolita",
    "actinologías": "actinología",
    "actinometrías": "actinometría",
    "actinomicetales": "actinomicetal",
    "actinomicetos": "actinomiceto",
    "actinomicinas": "actinomicina",
    "actinomorfa": "actinomorfo",
    "actinomorfas": "actinomorfo",
    "actinomorfos": "actinomorfo",
    "actinométrica": "actinométrico",
    "actinométricas": "actinométrico",
    "actinométricos": "actinométrico",
    "actinones": "actinón",
    "actinotas": "actinota",
    "actinoterapias": "actinoterapia",
    "actinómetros": "actinómetro",
    "actiones": "actione",
    "actios": "actio",
    "actitudes": "actitud",
    "activa": "activo",
    "activaba": "activar",
    "activabais": "activar",
    "activaban": "activar",
    "activabas": "activar",
    "activables": "activable",
    "activaciones": "activación",
    "activad": "activar",
    "activada": "activar",
    "activadas": "activar",
    "activado": "activar",
    "activadora": "activador",
    "activadoras": "activador",
    "activadores": "activador",
    "activados": "activar",
    "activamos": "activar",
    "activan": "activar",
    "activando": "activar",
    "activantes": "activante",
    "activara": "activar",
    "activarais": "activar",
    "activaran": "activar",
    "activaras": "activar",
    "activare": "activar",
    "activareis": "activar",
    "activaremos": "activar",
    "activaren": "activar",
    "activares": "activar",
    "activaron": "activar",
    "activará": "activar",
    "activarán": "activar",
    "activarás": "activar",
    "activaré": "activar",
    "activaréis": "activar",
    "activaría": "activar",
    "activaríais": "activar",
    "activaríamos": "activar",
    "activarían": "activar",
    "activarías": "activar",
    "activas": "activo",
    "activase": "activar",
    "activaseis": "activar",
    "activasen": "activar",
    "activases": "activar",
    "activaste": "activar",
    "activasteis": "activar",
    "active": "activar",
    "activemos": "activar",
    "activen": "activar",
    "actives": "activar",
    "actividades": "actividad",
    "activismos": "activismo",
    "activistas": "activista",
    "activo": "activar",
    "activos": "activo",
    "activábamos": "activar",
    "activáis": "activar",
    "activáramos": "activar",
    "activáremos": "activar",
    "activásemos": "activar",
    "activé": "activar",
    "activéis": "activar",
    "activó": "activar",
    "actomiosinas": "actomiosina",
    "actoras": "actora",
    "actores": "actor",
    "actos": "acto",
    "actrices": "actriz",
    "actuaba": "actuar",
    "actuabais": "actuar",
    "actuaban": "actuar",
    "actuabas": "actuar",
    "actuaciones": "actuación",
    "actuad": "actuar",
    "actuada": "actuar",
    "actuadas": "actuar",
    "actuado": "actuar",
    "actuadores": "actuador",
    "actuados": "actuar",
    "actuales": "actual",
    "actualice": "actualizar",
    "actualicemos": "actualizar",
    "actualicen": "actualizar",
    "actualices": "actualizar",
    "actualicé": "actualizar",
    "actualicéis": "actualizar",
    "actualidades": "actualidad",
    "actualismos": "actualismo",
    "actualiza": "actualizar",
    "actualizaba": "actualizar",
    "actualizabais": "actualizar",
    "actualizaban": "actualizar",
    "actualizabas": "actualizar",
    "actualizables": "actualizable",
    "actualizaciones": "actualización",
    "actualizad": "actualizar",
    "actualizada": "actualizar",
    "actualizadas": "actualizar",
    "actualizado": "actualizar",
    "actualizadora": "actualizador",
    "actualizadoras": "actualizador",
    "actualizadores": "actualizador",
    "actualizados": "actualizar",
    "actualizamos": "actualizar",
    "actualizan": "actualizar",
    "actualizando": "actualizar",
    "actualizara": "actualizar",
    "actualizarais": "actualizar",
    "actualizaran": "actualizar",
    "actualizaras": "actualizar",
    "actualizare": "actualizar",
    "actualizareis": "actualizar",
    "actualizaremos": "actualizar",
    "actualizaren": "actualizar",
    "actualizares": "actualizar",
    "actualizaron": "actualizar",
    "actualizará": "actualizar",
    "actualizarán": "actualizar",
    "actualizarás": "actualizar",
    "actualizaré": "actualizar",
    "actualizaréis": "actualizar",
    "actualizaría": "actualizar",
    "actualizaríais": "actualizar",
    "actualizaríamos": "actualizar",
    "actualizarían": "actualizar",
    "actualizarías": "actualizar",
    "actualizas": "actualizar",
    "actualizase": "actualizar",
    "actualizaseis": "actualizar",
    "actualizasen": "actualizar",
    "actualizases": "actualizar",
    "actualizaste": "actualizar",
    "actualizasteis": "actualizar",
    "actualizo": "actualizar",
    "actualizábamos": "actualizar",
    "actualizáis": "actualizar",
    "actualizáramos": "actualizar",
    "actualizáremos": "actualizar",
    "actualizásemos": "actualizar",
    "actualizó": "actualizar",
    "actuamos": "actuar",
    "actuando": "actuar",
    "actuantes": "actuante",
    "actuara": "actuar",
    "actuarais": "actuar",
    "actuaran": "actuar",
    "actuaras": "actuar",
    "actuare": "actuar",
    "actuareis": "actuar",
    "actuaremos": "actuar",
    "actuaren": "actuar",
    "actuares": "actuar",
    "actuaria": "actuario",
    "actuariada": "actuariado",
    "actuariadas": "actuariado",
    "actuariados": "actuariado",
    "actuariales": "actuarial",
    "actuarias": "actuario",
    "actuarios": "actuario",
    "actuaron": "actuar",
    "actuará": "actuar",
    "actuarán": "actuar",
    "actuarás": "actuar",
    "actuaré": "actuar",
    "actuaréis": "actuar",
    "actuaría": "actuar",
    "actuaríais": "actuar",
    "actuaríamos": "actuar",
    "actuarían": "actuar",
    "actuarías": "actuar",
    "actuase": "actuar",
    "actuaseis": "actuar",
    "actuasen": "actuar",
    "actuases": "actuar",
    "actuaste": "actuar",
    "actuasteis": "actuar",
    "actuemos": "actuar",
    "actuábamos": "actuar",
    "actuáis": "actuar",
    "actuáramos": "actuar",
    "actuáremos": "actuar",
    "actuásemos": "actuar",
    "actué": "actuar",
    "actuéis": "actuar",
    "actuó": "actuar",
    "actínica": "actínico",
    "actínicas": "actínico",
    "actínicos": "actínico",
    "actínida": "actínido",
    "actínidas": "actínido",
    "actínidos": "actínido",
    "actúa": "actuar",
    "actúan": "actuar",
    "actúas": "actuar",
    "actúe": "actuar",
    "actúen": "actuar",
    "actúes": "actuar",
    "actúo": "actuar",
    "acuadrilla": "acuadrillar",
    "acuadrillaba": "acuadrillar",
    "acuadrillabais": "acuadrillar",
    "acuadrillaban": "acuadrillar",
    "acuadrillabas": "acuadrillar",
    "acuadrillad": "acuadrillar",
    "acuadrillada": "acuadrillar",
    "acuadrilladas": "acuadrillar",
    "acuadrillado": "acuadrillar",
    "acuadrillados": "acuadrillar",
    "acuadrillamos": "acuadrillar",
    "acuadrillan": "acuadrillar",
    "acuadrillando": "acuadrillar",
    "acuadrillara": "acuadrillar",
    "acuadrillarais": "acuadrillar",
    "acuadrillaran": "acuadrillar",
    "acuadrillaras": "acuadrillar",
    "acuadrillare": "acuadrillar",
    "acuadrillareis": "acuadrillar",
    "acuadrillaremos": "acuadrillar",
    "acuadrillaren": "acuadrillar",
    "acuadrillares": "acuadrillar",
    "acuadrillaron": "acuadrillar",
    "acuadrillará": "acuadrillar",
    "acuadrillarán": "acuadrillar",
    "acuadrillarás": "acuadrillar",
    "acuadrillaré": "acuadrillar",
    "acuadrillaréis": "acuadrillar",
    "acuadrillaría": "acuadrillar",
    "acuadrillaríais": "acuadrillar",
    "acuadrillaríamos": "acuadrillar",
    "acuadrillarían": "acuadrillar",
    "acuadrillarías": "acuadrillar",
    "acuadrillas": "acuadrillar",
    "acuadrillase": "acuadrillar",
    "acuadrillaseis": "acuadrillar",
    "acuadrillasen": "acuadrillar",
    "acuadrillases": "acuadrillar",
    "acuadrillaste": "acuadrillar",
    "acuadrillasteis": "acuadrillar",
    "acuadrille": "acuadrillar",
    "acuadrillemos": "acuadrillar",
    "acuadrillen": "acuadrillar",
    "acuadrilles": "acuadrillar",
    "acuadrillo": "acuadrillar",
    "acuadrillábamos": "acuadrillar",
    "acuadrilláis": "acuadrillar",
    "acuadrilláramos": "acuadrillar",
    "acuadrilláremos": "acuadrillar",
    "acuadrillásemos": "acuadrillar",
    "acuadrillé": "acuadrillar",
    "acuadrilléis": "acuadrillar",
    "acuadrilló": "acuadrillar",
    "acuafortistas": "acuafortista",
    "acuahoteles": "acuahotel",
    "acuantia": "acuantiar",
    "acuantiaba": "acuantiar",
    "acuantiabais": "acuantiar",
    "acuantiaban": "acuantiar",
    "acuantiabas": "acuantiar",
    "acuantiad": "acuantiar",
    "acuantiada": "acuantiar",
    "acuantiadas": "acuantiar",
    "acuantiado": "acuantiar",
    "acuantiados": "acuantiar",
    "acuantiamos": "acuantiar",
    "acuantian": "acuantiar",
    "acuantiando": "acuantiar",
    "acuantiara": "acuantiar",
    "acuantiarais": "acuantiar",
    "acuantiaran": "acuantiar",
    "acuantiaras": "acuantiar",
    "acuantiare": "acuantiar",
    "acuantiareis": "acuantiar",
    "acuantiaremos": "acuantiar",
    "acuantiaren": "acuantiar",
    "acuantiares": "acuantiar",
    "acuantiaron": "acuantiar",
    "acuantiará": "acuantiar",
    "acuantiarán": "acuantiar",
    "acuantiarás": "acuantiar",
    "acuantiaré": "acuantiar",
    "acuantiaréis": "acuantiar",
    "acuantiaría": "acuantiar",
    "acuantiaríais": "acuantiar",
    "acuantiaríamos": "acuantiar",
    "acuantiarían": "acuantiar",
    "acuantiarías": "acuantiar",
    "acuantias": "acuantiar",
    "acuantiase": "acuantiar",
    "acuantiaseis": "acuantiar",
    "acuantiasen": "acuantiar",
    "acuantiases": "acuantiar",
    "acuantiaste": "acuantiar",
    "acuantiasteis": "acuantiar",
    "acuantie": "acuantiar",
    "acuantiemos": "acuantiar",
    "acuantien": "acuantiar",
    "acuanties": "acuantiar",
    "acuantio": "acuantiar",
    "acuantiábamos": "acuantiar",
    "acuantiáis": "acuantiar",
    "acuantiáramos": "acuantiar",
    "acuantiáremos": "acuantiar",
    "acuantiásemos": "acuantiar",
    "acuantié": "acuantiar",
    "acuantiéis": "acuantiar",
    "acuantió": "acuantiar",
    "acuaplanos": "acuaplano",
    "acuarelas": "acuarela",
    "acuarelistas": "acuarelista",
    "acuarios": "acuario",
    "acuartela": "acuartelar",
    "acuartelaba": "acuartelar",
    "acuartelabais": "acuartelar",
    "acuartelaban": "acuartelar",
    "acuartelabas": "acuartelar",
    "acuartelad": "acuartelar",
    "acuartelada": "acuartelar",
    "acuarteladas": "acuartelar",
    "acuartelado": "acuartelar",
    "acuartelados": "acuartelar",
    "acuartelamientos": "acuartelamiento",
    "acuartelamos": "acuartelar",
    "acuartelan": "acuartelar",
    "acuartelando": "acuartelar",
    "acuartelara": "acuartelar",
    "acuartelarais": "acuartelar",
    "acuartelaran": "acuartelar",
    "acuartelaras": "acuartelar",
    "acuartelare": "acuartelar",
    "acuartelareis": "acuartelar",
    "acuartelaremos": "acuartelar",
    "acuartelaren": "acuartelar",
    "acuartelares": "acuartelar",
    "acuartelaron": "acuartelar",
    "acuartelará": "acuartelar",
    "acuartelarán": "acuartelar",
    "acuartelarás": "acuartelar",
    "acuartelaré": "acuartelar",
    "acuartelaréis": "acuartelar",
    "acuartelaría": "acuartelar",
    "acuartelaríais": "acuartelar",
    "acuartelaríamos": "acuartelar",
    "acuartelarían": "acuartelar",
    "acuartelarías": "acuartelar",
    "acuartelas": "acuartelar",
    "acuartelase": "acuartelar",
    "acuartelaseis": "acuartelar",
    "acuartelasen": "acuartelar",
    "acuartelases": "acuartelar",
    "acuartelaste": "acuartelar",
    "acuartelasteis": "acuartelar",
    "acuartele": "acuartelar",
    "acuartelemos": "acuartelar",
    "acuartelen": "acuartelar",
    "acuarteles": "acuartelar",
    "acuartelo": "acuartelar",
    "acuartelábamos": "acuartelar",
    "acuarteláis": "acuartelar",
    "acuarteláramos": "acuartelar",
    "acuarteláremos": "acuartelar",
    "acuartelásemos": "acuartelar",
    "acuartelé": "acuartelar",
    "acuarteléis": "acuartelar",
    "acuarteló": "acuartelar",
    "acuartilla": "acuartillar",
    "acuartillaba": "acuartillar",
    "acuartillabais": "acuartillar",
    "acuartillaban": "acuartillar",
    "acuartillabas": "acuartillar",
    "acuartillad": "acuartillar",
    "acuartillada": "acuartillar",
    "acuartilladas": "acuartillar",
    "acuartillado": "acuartillar",
    "acuartillados": "acuartillar",
    "acuartillamos": "acuartillar",
    "acuartillan": "acuartillar",
    "acuartillando": "acuartillar",
    "acuartillara": "acuartillar",
    "acuartillarais": "acuartillar",
    "acuartillaran": "acuartillar",
    "acuartillaras": "acuartillar",
    "acuartillare": "acuartillar",
    "acuartillareis": "acuartillar",
    "acuartillaremos": "acuartillar",
    "acuartillaren": "acuartillar",
    "acuartillares": "acuartillar",
    "acuartillaron": "acuartillar",
    "acuartillará": "acuartillar",
    "acuartillarán": "acuartillar",
    "acuartillarás": "acuartillar",
    "acuartillaré": "acuartillar",
    "acuartillaréis": "acuartillar",
    "acuartillaría": "acuartillar",
    "acuartillaríais": "acuartillar",
    "acuartillaríamos": "acuartillar",
    "acuartillarían": "acuartillar",
    "acuartillarías": "acuartillar",
    "acuartillas": "acuartillar",
    "acuartillase": "acuartillar",
    "acuartillaseis": "acuartillar",
    "acuartillasen": "acuartillar",
    "acuartillases": "acuartillar",
    "acuartillaste": "acuartillar",
    "acuartillasteis": "acuartillar",
    "acuartille": "acuartillar",
    "acuartillemos": "acuartillar",
    "acuartillen": "acuartillar",
    "acuartilles": "acuartillar",
    "acuartillo": "acuartillar",
    "acuartillábamos": "acuartillar",
    "acuartilláis": "acuartillar",
    "acuartilláramos": "acuartillar",
    "acuartilláremos": "acuartillar",
    "acuartillásemos": "acuartillar",
    "acuartillé": "acuartillar",
    "acuartilléis": "acuartillar",
    "acuartilló": "acuartillar",
    "acuatice": "acuatizar",
    "acuaticemos": "acuatizar",
    "acuaticen": "acuatizar",
    "acuatices": "acuatizar",
    "acuaticé": "acuatizar",
    "acuaticéis": "acuatizar",
    "acuatintas": "acuatinta",
    "acuatiza": "acuatizar",
    "acuatizaba": "acuatizar",
    "acuatizabais": "acuatizar",
    "acuatizaban": "acuatizar",
    "acuatizabas": "acuatizar",
    "acuatizad": "acuatizar",
    "acuatizada": "acuatizar",
    "acuatizadas": "acuatizar",
    "acuatizado": "acuatizar",
    "acuatizados": "acuatizar",
    "acuatizamos": "acuatizar",
    "acuatizan": "acuatizar",
    "acuatizando": "acuatizar",
    "acuatizara": "acuatizar",
    "acuatizarais": "acuatizar",
    "acuatizaran": "acuatizar",
    "acuatizaras": "acuatizar",
    "acuatizare": "acuatizar",
    "acuatizareis": "acuatizar",
    "acuatizaremos": "acuatizar",
    "acuatizaren": "acuatizar",
    "acuatizares": "acuatizar",
    "acuatizaron": "acuatizar",
    "acuatizará": "acuatizar",
    "acuatizarán": "acuatizar",
    "acuatizarás": "acuatizar",
    "acuatizaré": "acuatizar",
    "acuatizaréis": "acuatizar",
    "acuatizaría": "acuatizar",
    "acuatizaríais": "acuatizar",
    "acuatizaríamos": "acuatizar",
    "acuatizarían": "acuatizar",
    "acuatizarías": "acuatizar",
    "acuatizas": "acuatizar",
    "acuatizase": "acuatizar",
    "acuatizaseis": "acuatizar",
    "acuatizasen": "acuatizar",
    "acuatizases": "acuatizar",
    "acuatizaste": "acuatizar",
    "acuatizasteis": "acuatizar",
    "acuatizo": "acuatizar",
    "acuatizábamos": "acuatizar",
    "acuatizáis": "acuatizar",
    "acuatizáramos": "acuatizar",
    "acuatizáremos": "acuatizar",
    "acuatizásemos": "acuatizar",
    "acuatizó": "acuatizar",
    "acucha": "acuchar",
    "acuchaba": "acuchar",
    "acuchabais": "acuchar",
    "acuchaban": "acuchar",
    "acuchabas": "acuchar",
    "acuchad": "acuchar",
    "acuchada": "acuchar",
    "acuchadas": "acuchar",
    "acuchado": "acuchar",
    "acuchados": "acuchar",
    "acuchamos": "acuchar",
    "acuchan": "acuchar",
    "acuchando": "acuchar",
    "acuchara": "acuchar",
    "acucharada": "acucharado",
    "acucharadas": "acucharado",
    "acucharados": "acucharado",
    "acucharais": "acuchar",
    "acucharan": "acuchar",
    "acucharas": "acuchar",
    "acuchare": "acuchar",
    "acuchareis": "acuchar",
    "acucharemos": "acuchar",
    "acucharen": "acuchar",
    "acuchares": "acuchar",
    "acucharon": "acuchar",
    "acuchará": "acuchar",
    "acucharán": "acuchar",
    "acucharás": "acuchar",
    "acucharé": "acuchar",
    "acucharéis": "acuchar",
    "acucharía": "acuchar",
    "acucharíais": "acuchar",
    "acucharíamos": "acuchar",
    "acucharían": "acuchar",
    "acucharías": "acuchar",
    "acuchas": "acuchar",
    "acuchase": "acuchar",
    "acuchaseis": "acuchar",
    "acuchasen": "acuchar",
    "acuchases": "acuchar",
    "acuchaste": "acuchar",
    "acuchasteis": "acuchar",
    "acuche": "acuchar",
    "acuchemos": "acuchar",
    "acuchen": "acuchar",
    "acuches": "acuchar",
    "acuchilla": "acuchillar",
    "acuchillaba": "acuchillar",
    "acuchillabais": "acuchillar",
    "acuchillaban": "acuchillar",
    "acuchillabas": "acuchillar",
    "acuchillad": "acuchillar",
    "acuchillada": "acuchillar",
    "acuchilladas": "acuchillar",
    "acuchillado": "acuchillar",
    "acuchilladora": "acuchillador",
    "acuchilladoras": "acuchillador",
    "acuchilladores": "acuchillador",
    "acuchillados": "acuchillar",
    "acuchillamos": "acuchillar",
    "acuchillan": "acuchillar",
    "acuchillando": "acuchillar",
    "acuchillara": "acuchillar",
    "acuchillarais": "acuchillar",
    "acuchillaran": "acuchillar",
    "acuchillaras": "acuchillar",
    "acuchillare": "acuchillar",
    "acuchillareis": "acuchillar",
    "acuchillaremos": "acuchillar",
    "acuchillaren": "acuchillar",
    "acuchillares": "acuchillar",
    "acuchillaron": "acuchillar",
    "acuchillará": "acuchillar",
    "acuchillarán": "acuchillar",
    "acuchillarás": "acuchillar",
    "acuchillaré": "acuchillar",
    "acuchillaréis": "acuchillar",
    "acuchillaría": "acuchillar",
    "acuchillaríais": "acuchillar",
    "acuchillaríamos": "acuchillar",
    "acuchillarían": "acuchillar",
    "acuchillarías": "acuchillar",
    "acuchillas": "acuchillar",
    "acuchillase": "acuchillar",
    "acuchillaseis": "acuchillar",
    "acuchillasen": "acuchillar",
    "acuchillases": "acuchillar",
    "acuchillaste": "acuchillar",
    "acuchillasteis": "acuchillar",
    "acuchille": "acuchillar",
    "acuchillemos": "acuchillar",
    "acuchillen": "acuchillar",
    "acuchilles": "acuchillar",
    "acuchillo": "acuchillar",
    "acuchillábamos": "acuchillar",
    "acuchilláis": "acuchillar",
    "acuchilláramos": "acuchillar",
    "acuchilláremos": "acuchillar",
    "acuchillásemos": "acuchillar",
    "acuchillé": "acuchillar",
    "acuchilléis": "acuchillar",
    "acuchilló": "acuchillar",
    "acucho": "acuchar",
    "acuchábamos": "acuchar",
    "acucháis": "acuchar",
    "acucháramos": "acuchar",
    "acucháremos": "acuchar",
    "acuchásemos": "acuchar",
    "acuché": "acuchar",
    "acuchéis": "acuchar",
    "acuchó": "acuchar",
    "acucia": "acuciar",
    "acuciaba": "acuciar",
    "acuciabais": "acuciar",
    "acuciaban": "acuciar",
    "acuciabas": "acuciar",
    "acuciad": "acuciar",
    "acuciada": "acuciar",
    "acuciadas": "acuciar",
    "acuciado": "acuciar",
    "acuciadora": "acuciador",
    "acuciadoras": "acuciador",
    "acuciadores": "acuciador",
    "acuciados": "acuciar",
    "acuciamientos": "acuciamiento",
    "acuciamos": "acuciar",
    "acucian": "acuciar",
    "acuciando": "acuciar",
    "acuciantes": "acuciante",
    "acuciara": "acuciar",
    "acuciarais": "acuciar",
    "acuciaran": "acuciar",
    "acuciaras": "acuciar",
    "acuciare": "acuciar",
    "acuciareis": "acuciar",
    "acuciaremos": "acuciar",
    "acuciaren": "acuciar",
    "acuciares": "acuciar",
    "acuciaron": "acuciar",
    "acuciará": "acuciar",
    "acuciarán": "acuciar",
    "acuciarás": "acuciar",
    "acuciaré": "acuciar",
    "acuciaréis": "acuciar",
    "acuciaría": "acuciar",
    "acuciaríais": "acuciar",
    "acuciaríamos": "acuciar",
    "acuciarían": "acuciar",
    "acuciarías": "acuciar",
    "acucias": "acuciar",
    "acuciase": "acuciar",
    "acuciaseis": "acuciar",
    "acuciasen": "acuciar",
    "acuciases": "acuciar",
    "acuciaste": "acuciar",
    "acuciasteis": "acuciar",
    "acucie": "acuciar",
    "acuciemos": "acuciar",
    "acucien": "acuciar",
    "acucies": "acuciar",
    "acucio": "acuciar",
    "acuciosa": "acucioso",
    "acuciosas": "acucioso",
    "acuciosidades": "acuciosidad",
    "acuciosos": "acucioso",
    "acuciábamos": "acuciar",
    "acuciáis": "acuciar",
    "acuciáramos": "acuciar",
    "acuciáremos": "acuciar",
    "acuciásemos": "acuciar",
    "acucié": "acuciar",
    "acuciéis": "acuciar",
    "acució": "acuciar",
    "acuclilla": "acuclillar",
    "acuclillaba": "acuclillar",
    "acuclillabais": "acuclillar",
    "acuclillaban": "acuclillar",
    "acuclillabas": "acuclillar",
    "acuclillad": "acuclillar",
    "acuclillada": "acuclillar",
    "acuclilladas": "acuclillar",
    "acuclillado": "acuclillar",
    "acuclillados": "acuclillar",
    "acuclillamos": "acuclillar",
    "acuclillan": "acuclillar",
    "acuclillando": "acuclillar",
    "acuclillara": "acuclillar",
    "acuclillarais": "acuclillar",
    "acuclillaran": "acuclillar",
    "acuclillaras": "acuclillar",
    "acuclillare": "acuclillar",
    "acuclillareis": "acuclillar",
    "acuclillaremos": "acuclillar",
    "acuclillaren": "acuclillar",
    "acuclillares": "acuclillar",
    "acuclillaron": "acuclillar",
    "acuclillará": "acuclillar",
    "acuclillarán": "acuclillar",
    "acuclillarás": "acuclillar",
    "acuclillaré": "acuclillar",
    "acuclillaréis": "acuclillar",
    "acuclillaría": "acuclillar",
    "acuclillaríais": "acuclillar",
    "acuclillaríamos": "acuclillar",
    "acuclillarían": "acuclillar",
    "acuclillarías": "acuclillar",
    "acuclillas": "acuclillar",
    "acuclillase": "acuclillar",
    "acuclillaseis": "acuclillar",
    "acuclillasen": "acuclillar",
    "acuclillases": "acuclillar",
    "acuclillaste": "acuclillar",
    "acuclillasteis": "acuclillar",
    "acuclille": "acuclillar",
    "acuclillemos": "acuclillar",
    "acuclillen": "acuclillar",
    "acuclilles": "acuclillar",
    "acuclillo": "acuclillar",
    "acuclillábamos": "acuclillar",
    "acuclilláis": "acuclillar",
    "acuclilláramos": "acuclillar",
    "acuclilláremos": "acuclillar",
    "acuclillásemos": "acuclillar",
    "acuclillé": "acuclillar",
    "acuclilléis": "acuclillar",
    "acuclilló": "acuclillar",
    "acuda": "acudir",
    "acudamos": "acudir",
    "acudan": "acudir",
    "acudas": "acudir",
    "acude": "acudir",
    "acuden": "acudir",
    "acudes": "acudir",
    "acudid": "acudir",
    "acudida": "acudir",
    "acudidas": "acudir",
    "acudido": "acudir",
    "acudidos": "acudir",
    "acudiendo": "acudir",
    "acudiera": "acudir",
    "acudierais": "acudir",
    "acudieran": "acudir",
    "acudieras": "acudir",
    "acudiere": "acudir",
    "acudiereis": "acudir",
    "acudieren": "acudir",
    "acudieres": "acudir",
    "acudieron": "acudir",
    "acudiese": "acudir",
    "acudieseis": "acudir",
    "acudiesen": "acudir",
    "acudieses": "acudir",
    "acudimientos": "acudimiento",
    "acudimos": "acudir",
    "acudiremos": "acudir",
    "acudirá": "acudir",
    "acudirán": "acudir",
    "acudirás": "acudir",
    "acudiré": "acudir",
    "acudiréis": "acudir",
    "acudiría": "acudir",
    "acudiríais": "acudir",
    "acudiríamos": "acudir",
    "acudirían": "acudir",
    "acudirías": "acudir",
    "acudiste": "acudir",
    "acudisteis": "acudir",
    "acudiéramos": "acudir",
    "acudiéremos": "acudir",
    "acudiésemos": "acudir",
    "acudió": "acudir",
    "acudo": "acudir",
    "acudáis": "acudir",
    "acudí": "acudir",
    "acudía": "acudir",
    "acudíais": "acudir",
    "acudíamos": "acudir",
    "acudían": "acudir",
    "acudías": "acudir",
    "acudís": "acudir",
    "acueductos": "acueducto",
    "acuella": "acollar",
    "acuellan": "acollar",
    "acuellas": "acollar",
    "acuelle": "acollar",
    "acuellen": "acollar",
    "acuelles": "acollar",
    "acuello": "acollar",
    "acuerda": "acordar",
    "acuerdan": "acordar",
    "acuerdas": "acordar",
    "acuerde": "acordar",
    "acuerden": "acordar",
    "acuerdes": "acordar",
    "acuerdo": "acordar",
    "acuerdos": "acuerdo",
    "acuerna": "acornar",
    "acuernan": "acornar",
    "acuernas": "acornar",
    "acuerne": "acornar",
    "acuernen": "acornar",
    "acuernes": "acornar",
    "acuerno": "acornar",
    "acuesta": "acostar",
    "acuestan": "acostar",
    "acuestas": "acostar",
    "acueste": "acostar",
    "acuesten": "acostar",
    "acuestes": "acostar",
    "acuesto": "acostar",
    "acuiculturas": "acuicultura",
    "acuidades": "acuidad",
    "acuita": "acuitar",
    "acuitaba": "acuitar",
    "acuitabais": "acuitar",
    "acuitaban": "acuitar",
    "acuitabas": "acuitar",
    "acuitad": "acuitar",
    "acuitada": "acuitar",
    "acuitadas": "acuitar",
    "acuitado": "acuitar",
    "acuitados": "acuitar",
    "acuitamos": "acuitar",
    "acuitan": "acuitar",
    "acuitando": "acuitar",
    "acuitara": "acuitar",
    "acuitarais": "acuitar",
    "acuitaran": "acuitar",
    "acuitaras": "acuitar",
    "acuitare": "acuitar",
    "acuitareis": "acuitar",
    "acuitaremos": "acuitar",
    "acuitaren": "acuitar",
    "acuitares": "acuitar",
    "acuitaron": "acuitar",
    "acuitará": "acuitar",
    "acuitarán": "acuitar",
    "acuitarás": "acuitar",
    "acuitaré": "acuitar",
    "acuitaréis": "acuitar",
    "acuitaría": "acuitar",
    "acuitaríais": "acuitar",
    "acuitaríamos": "acuitar",
    "acuitarían": "acuitar",
    "acuitarías": "acuitar",
    "acuitas": "acuitar",
    "acuitase": "acuitar",
    "acuitaseis": "acuitar",
    "acuitasen": "acuitar",
    "acuitases": "acuitar",
    "acuitaste": "acuitar",
    "acuitasteis": "acuitar",
    "acuite": "acuitar",
    "acuitemos": "acuitar",
    "acuiten": "acuitar",
    "acuites": "acuitar",
    "acuito": "acuitar",
    "acuitábamos": "acuitar",
    "acuitáis": "acuitar",
    "acuitáramos": "acuitar",
    "acuitáremos": "acuitar",
    "acuitásemos": "acuitar",
    "acuité": "acuitar",
    "acuitéis": "acuitar",
    "acuitó": "acuitar",
    "acula": "acular",
    "aculaba": "acular",
    "aculabais": "acular",
    "aculaban": "acular",
    "aculabas": "acular",
    "aculad": "acular",
    "aculada": "acular",
    "aculadas": "acular",
    "aculado": "acular",
    "aculados": "acular",
    "aculamos": "acular",
    "aculan": "acular",
    "aculando": "acular",
    "aculara": "acular",
    "acularais": "acular",
    "acularan": "acular",
    "acularas": "acular",
    "aculare": "acular",
    "aculareis": "acular",
    "acularemos": "acular",
    "acularen": "acular",
    "aculares": "acular",
    "acularon": "acular",
    "aculará": "acular",
    "acularán": "acular",
    "acularás": "acular",
    "acularé": "acular",
    "acularéis": "acular",
    "acularía": "acular",
    "acularíais": "acular",
    "acularíamos": "acular",
    "acularían": "acular",
    "acularías": "acular",
    "aculas": "acular",
    "aculase": "acular",
    "aculaseis": "acular",
    "aculasen": "acular",
    "aculases": "acular",
    "aculaste": "acular",
    "aculasteis": "acular",
    "acule": "acular",
    "aculebrinada": "aculebrinado",
    "aculebrinadas": "aculebrinado",
    "aculebrinados": "aculebrinado",
    "aculemos": "acular",
    "aculen": "acular",
    "acules": "acular",
    "aculo": "acular",
    "aculturaciónes": "aculturación",
    "aculábamos": "acular",
    "aculáis": "acular",
    "aculáramos": "acular",
    "aculáremos": "acular",
    "aculásemos": "acular",
    "aculé": "acular",
    "aculéis": "acular",
    "aculó": "acular",
    "acumetrías": "acumetría",
    "acuminada": "acuminado",
    "acuminadas": "acuminado",
    "acuminados": "acuminado",
    "acumula": "acumular",
    "acumulaba": "acumular",
    "acumulabais": "acumular",
    "acumulaban": "acumular",
    "acumulabas": "acumular",
    "acumulables": "acumulable",
    "acumulaciones": "acumulación",
    "acumulad": "acumular",
    "acumulada": "acumular",
    "acumuladas": "acumular",
    "acumulado": "acumular",
    "acumuladora": "acumulador",
    "acumuladoras": "acumulador",
    "acumuladores": "acumulador",
    "acumulados": "acumular",
    "acumulamientos": "acumulamiento",
    "acumulamos": "acumular",
    "acumulan": "acumular",
    "acumulando": "acumular",
    "acumulara": "acumular",
    "acumularais": "acumular",
    "acumularan": "acumular",
    "acumularas": "acumular",
    "acumulare": "acumular",
    "acumulareis": "acumular",
    "acumularemos": "acumular",
    "acumularen": "acumular",
    "acumulares": "acumular",
    "acumularon": "acumular",
    "acumulará": "acumular",
    "acumularán": "acumular",
    "acumularás": "acumular",
    "acumularé": "acumular",
    "acumularéis": "acumular",
    "acumularía": "acumular",
    "acumularíais": "acumular",
    "acumularíamos": "acumular",
    "acumularían": "acumular",
    "acumularías": "acumular",
    "acumulas": "acumular",
    "acumulase": "acumular",
    "acumulaseis": "acumular",
    "acumulasen": "acumular",
    "acumulases": "acumular",
    "acumulaste": "acumular",
    "acumulasteis": "acumular",
    "acumulativa": "acumulativo",
    "acumulativas": "acumulativo",
    "acumulativos": "acumulativo",
    "acumule": "acumular",
    "acumulemos": "acumular",
    "acumulen": "acumular",
    "acumules": "acumular",
    "acumulo": "acumular",
    "acumulábamos": "acumular",
    "acumuláis": "acumular",
    "acumuláramos": "acumular",
    "acumuláremos": "acumular",
    "acumulásemos": "acumular",
    "acumulé": "acumular",
    "acumuléis": "acumular",
    "acumuló": "acumular",
    "acuna": "acunar",
    "acunaba": "acunar",
    "acunabais": "acunar",
    "acunaban": "acunar",
    "acunabas": "acunar",
    "acunad": "acunar",
    "acunada": "acunar",
    "acunadas": "acunar",
    "acunado": "acunar",
    "acunados": "acunar",
    "acunamos": "acunar",
    "acunan": "acunar",
    "acunando": "acunar",
    "acunara": "acunar",
    "acunarais": "acunar",
    "acunaran": "acunar",
    "acunaras": "acunar",
    "acunare": "acunar",
    "acunareis": "acunar",
    "acunaremos": "acunar",
    "acunaren": "acunar",
    "acunares": "acunar",
    "acunaron": "acunar",
    "acunará": "acunar",
    "acunarán": "acunar",
    "acunarás": "acunar",
    "acunaré": "acunar",
    "acunaréis": "acunar",
    "acunaría": "acunar",
    "acunaríais": "acunar",
    "acunaríamos": "acunar",
    "acunarían": "acunar",
    "acunarías": "acunar",
    "acunas": "acunar",
    "acunase": "acunar",
    "acunaseis": "acunar",
    "acunasen": "acunar",
    "acunases": "acunar",
    "acunaste": "acunar",
    "acunasteis": "acunar",
    "acune": "acunar",
    "acunemos": "acunar",
    "acunen": "acunar",
    "acunes": "acunar",
    "acuno": "acunar",
    "acunábamos": "acunar",
    "acunáis": "acunar",
    "acunáramos": "acunar",
    "acunáremos": "acunar",
    "acunásemos": "acunar",
    "acuné": "acunar",
    "acunéis": "acunar",
    "acunó": "acunar",
    "acuosa": "acuoso",
    "acuosas": "acuoso",
    "acuosidades": "acuosidad",
    "acuosos": "acuoso",
    "acupuntora": "acupuntor",
    "acupuntoras": "acupuntor",
    "acupuntores": "acupuntor",
    "acupunturas": "acupuntura",
    "acupunturistas": "acupunturista",
    "acurruca": "acurrucar",
    "acurrucaba": "acurrucar",
    "acurrucabais": "acurrucar",
    "acurrucaban": "acurrucar",
    "acurrucabas": "acurrucar",
    "acurrucad": "acurrucar",
    "acurrucada": "acurrucar",
    "acurrucadas": "acurrucar",
    "acurrucado": "acurrucar",
    "acurrucados": "acurrucar",
    "acurrucamientos": "acurrucamiento",
    "acurrucamos": "acurrucar",
    "acurrucan": "acurrucar",
    "acurrucando": "acurrucar",
    "acurrucara": "acurrucar",
    "acurrucarais": "acurrucar",
    "acurrucaran": "acurrucar",
    "acurrucaras": "acurrucar",
    "acurrucare": "acurrucar",
    "acurrucareis": "acurrucar",
    "acurrucaremos": "acurrucar",
    "acurrucaren": "acurrucar",
    "acurrucares": "acurrucar",
    "acurrucaron": "acurrucar",
    "acurrucará": "acurrucar",
    "acurrucarán": "acurrucar",
    "acurrucarás": "acurrucar",
    "acurrucaré": "acurrucar",
    "acurrucaréis": "acurrucar",
    "acurrucaría": "acurrucar",
    "acurrucaríais": "acurrucar",
    "acurrucaríamos": "acurrucar",
    "acurrucarían": "acurrucar",
    "acurrucarías": "acurrucar",
    "acurrucas": "acurrucar",
    "acurrucase": "acurrucar",
    "acurrucaseis": "acurrucar",
    "acurrucasen": "acurrucar",
    "acurrucases": "acurrucar",
    "acurrucaste": "acurrucar",
    "acurrucasteis": "acurrucar",
    "acurruco": "acurrucar",
    "acurrucábamos": "acurrucar",
    "acurrucáis": "acurrucar",
    "acurrucáramos": "acurrucar",
    "acurrucáremos": "acurrucar",
    "acurrucásemos": "acurrucar",
    "acurrucó": "acurrucar",
    "acurrulla": "acurrullar",
    "acurrullaba": "acurrullar",
    "acurrullabais": "acurrullar",
    "acurrullaban": "acurrullar",
    "acurrullabas": "acurrullar",
    "acurrullad": "acurrullar",
    "acurrullada": "acurrullar",
    "acurrulladas": "acurrullar",
    "acurrullado": "acurrullar",
    "acurrullados": "acurrullar",
    "acurrullamos": "acurrullar",
    "acurrullan": "acurrullar",
    "acurrullando": "acurrullar",
    "acurrullara": "acurrullar",
    "acurrullarais": "acurrullar",
    "acurrullaran": "acurrullar",
    "acurrullaras": "acurrullar",
    "acurrullare": "acurrullar",
    "acurrullareis": "acurrullar",
    "acurrullaremos": "acurrullar",
    "acurrullaren": "acurrullar",
    "acurrullares": "acurrullar",
    "acurrullaron": "acurrullar",
    "acurrullará": "acurrullar",
    "acurrullarán": "acurrullar",
    "acurrullarás": "acurrullar",
    "acurrullaré": "acurrullar",
    "acurrullaréis": "acurrullar",
    "acurrullaría": "acurrullar",
    "acurrullaríais": "acurrullar",
    "acurrullaríamos": "acurrullar",
    "acurrullarían": "acurrullar",
    "acurrullarías": "acurrullar",
    "acurrullas": "acurrullar",
    "acurrullase": "acurrullar",
    "acurrullaseis": "acurrullar",
    "acurrullasen": "acurrullar",
    "acurrullases": "acurrullar",
    "acurrullaste": "acurrullar",
    "acurrullasteis": "acurrullar",
    "acurrulle": "acurrullar",
    "acurrullemos": "acurrullar",
    "acurrullen": "acurrullar",
    "acurrulles": "acurrullar",
    "acurrullo": "acurrullar",
    "acurrullábamos": "acurrullar",
    "acurrulláis": "acurrullar",
    "acurrulláramos": "acurrullar",
    "acurrulláremos": "acurrullar",
    "acurrullásemos": "acurrullar",
    "acurrullé": "acurrullar",
    "acurrulléis": "acurrullar",
    "acurrulló": "acurrullar",
    "acurruque": "acurrucar",
    "acurruquemos": "acurrucar",
    "acurruquen": "acurrucar",
    "acurruques": "acurrucar",
    "acurruqué": "acurrucar",
    "acurruquéis": "acurrucar",
    "acusa": "acusar",
    "acusaba": "acusar",
    "acusabais": "acusar",
    "acusaban": "acusar",
    "acusabas": "acusar",
    "acusaciones": "acusación",
    "acusad": "acusar",
    "acusada": "acusar",
    "acusadas": "acusar",
    "acusado": "acusar",
    "acusadora": "acusador",
    "acusadoras": "acusador",
    "acusadores": "acusador",
    "acusados": "acusar",
    "acusamos": "acusar",
    "acusan": "acusar",
    "acusando": "acusar",
    "acusara": "acusar",
    "acusarais": "acusar",
    "acusaran": "acusar",
    "acusaras": "acusar",
    "acusare": "acusar",
    "acusareis": "acusar",
    "acusaremos": "acusar",
    "acusaren": "acusar",
    "acusares": "acusar",
    "acusaron": "acusar",
    "acusará": "acusar",
    "acusarán": "acusar",
    "acusarás": "acusar",
    "acusaré": "acusar",
    "acusaréis": "acusar",
    "acusaría": "acusar",
    "acusaríais": "acusar",
    "acusaríamos": "acusar",
    "acusarían": "acusar",
    "acusarías": "acusar",
    "acusas": "acusar",
    "acusase": "acusar",
    "acusaseis": "acusar",
    "acusasen": "acusar",
    "acusases": "acusar",
    "acusaste": "acusar",
    "acusasteis": "acusar",
    "acusativa": "acusativo",
    "acusativas": "acusativo",
    "acusativos": "acusativo",
    "acusatoria": "acusatorio",
    "acusatorias": "acusatorio",
    "acusatorios": "acusatorio",
    "acuse": "acusar",
    "acusemos": "acusar",
    "acusen": "acusar",
    "acuses": "acuse",
    "acusetes": "acusete",
    "acusicas": "acusica",
    "acuso": "acusar",
    "acusona": "acusón",
    "acusonas": "acusón",
    "acusones": "acusón",
    "acusábamos": "acusar",
    "acusáis": "acusar",
    "acusáramos": "acusar",
    "acusáremos": "acusar",
    "acusásemos": "acusar",
    "acusé": "acusar",
    "acuséis": "acusar",
    "acusó": "acusar",
    "acutángula": "acutángulo",
    "acutángulas": "acutángulo",
    "acutángulos": "acutángulo",
    "acuámeros": "acuámero",
    "acuática": "acuático",
    "acuáticas": "acuático",
    "acuáticos": "acuático",
    "acuátiles": "acuátil",
    "acuífera": "acuífero",
    "acuíferas": "acuífero",
    "acuíferos": "acuífero",
    "acuña": "acuñar",
    "acuñaba": "acuñar",
    "acuñabais": "acuñar",
    "acuñaban": "acuñar",
    "acuñabas": "acuñar",
    "acuñaciones": "acuñación",
    "acuñad": "acuñar",
    "acuñada": "acuñar",
    "acuñadas": "acuñar",
    "acuñado": "acuñar",
    "acuñadora": "acuñador",
    "acuñadoras": "acuñador",
    "acuñadores": "acuñador",
    "acuñados": "acuñar",
    "acuñamientos": "acuñamiento",
    "acuñamos": "acuñar",
    "acuñan": "acuñar",
    "acuñando": "acuñar",
    "acuñara": "acuñar",
    "acuñarais": "acuñar",
    "acuñaran": "acuñar",
    "acuñaras": "acuñar",
    "acuñare": "acuñar",
    "acuñareis": "acuñar",
    "acuñaremos": "acuñar",
    "acuñaren": "acuñar",
    "acuñares": "acuñar",
    "acuñaron": "acuñar",
    "acuñará": "acuñar",
    "acuñarán": "acuñar",
    "acuñarás": "acuñar",
    "acuñaré": "acuñar",
    "acuñaréis": "acuñar",
    "acuñaría": "acuñar",
    "acuñaríais": "acuñar",
    "acuñaríamos": "acuñar",
    "acuñarían": "acuñar",
    "acuñarías": "acuñar",
    "acuñas": "acuñar",
    "acuñase": "acuñar",
    "acuñaseis": "acuñar",
    "acuñasen": "acuñar",
    "acuñases": "acuñar",
    "acuñaste": "acuñar",
    "acuñasteis": "acuñar",
    "acuñe": "acuñar",
    "acuñemos": "acuñar",
    "acuñen": "acuñar",
    "acuñes": "acuñar",
    "acuño": "acuñar",
    "acuñábamos": "acuñar",
    "acuñáis": "acuñar",
    "acuñáramos": "acuñar",
    "acuñáremos": "acuñar",
    "acuñásemos": "acuñar",
    "acuñé": "acuñar",
    "acuñéis": "acuñar",
    "acuñó": "acuñar",
    "acápites": "acápite",
    "acéfala": "acéfalo",
    "acéfalas": "acéfalo",
    "acéfalos": "acéfalo",
    "acémilas": "acémila",
    "acérrima": "acérrimo",
    "acérrimas": "acérrimo",
    "acérrimos": "acérrimo",
    "acética": "acético",
    "acéticas": "acético",
    "acéticos": "acético",
    "acíbares": "acíbar",
    "acíclica": "acíclico",
    "acíclicas": "acíclico",
    "acíclicos": "acíclico",
    "acículas": "acícula",
    "acídica": "acídico",
    "acídicas": "acídico",
    "acídicos": "acídico",
    "acídula": "acídulo",
    "acídulas": "acídulo",
    "acídulos": "acídulo",
    "acólitos": "acólito",
    "acónitos": "acónito",
    "acúfenos": "acúfeno",
    "acústica": "acústico",
    "acústicas": "acústico",
    "acústico-fonética": "acústico-fonético",
    "acústico-fonéticas": "acústico-fonético",
    "acústico-fonéticos": "acústico-fonético",
    "acústico-óptica": "acústico-óptico",
    "acústico-ópticas": "acústico-óptico",
    "acústico-ópticos": "acústico-óptico",
    "acústicos": "acústico",
    "ada": "adir",
    "adagios": "adagio",
    "adalides": "adalid",
    "adama": "adamar",
    "adamaba": "adamar",
    "adamabais": "adamar",
    "adamaban": "adamar",
    "adamabas": "adamar",
    "adamad": "adamar",
    "adamada": "adamar",
    "adamadas": "adamar",
    "adamado": "adamar",
    "adamados": "adamar",
    "adamamos": "adamar",
    "adaman": "adamar",
    "adamando": "adamar",
    "adamantina": "adamantino",
    "adamantinas": "adamantino",
    "adamantinomas": "adamantinoma",
    "adamantinos": "adamantino",
    "adamantoblastomas": "adamantoblastoma",
    "adamara": "adamar",
    "adamarais": "adamar",
    "adamaran": "adamar",
    "adamaras": "adamar",
    "adamare": "adamar",
    "adamareis": "adamar",
    "adamaremos": "adamar",
    "adamaren": "adamar",
    "adamares": "adamar",
    "adamaron": "adamar",
    "adamará": "adamar",
    "adamarán": "adamar",
    "adamarás": "adamar",
    "adamaré": "adamar",
    "adamaréis": "adamar",
    "adamaría": "adamar",
    "adamaríais": "adamar",
    "adamaríamos": "adamar",
    "adamarían": "adamar",
    "adamarías": "adamar",
    "adamas": "adamar",
    "adamasca": "adamascar",
    "adamascaba": "adamascar",
    "adamascabais": "adamascar",
    "adamascaban": "adamascar",
    "adamascabas": "adamascar",
    "adamascad": "adamascar",
    "adamascada": "adamascar",
    "adamascadas": "adamascar",
    "adamascado": "adamascar",
    "adamascados": "adamascar",
    "adamascamos": "adamascar",
    "adamascan": "adamascar",
    "adamascando": "adamascar",
    "adamascara": "adamascar",
    "adamascarais": "adamascar",
    "adamascaran": "adamascar",
    "adamascaras": "adamascar",
    "adamascare": "adamascar",
    "adamascareis": "adamascar",
    "adamascaremos": "adamascar",
    "adamascaren": "adamascar",
    "adamascares": "adamascar",
    "adamascaron": "adamascar",
    "adamascará": "adamascar",
    "adamascarán": "adamascar",
    "adamascarás": "adamascar",
    "adamascaré": "adamascar",
    "adamascaréis": "adamascar",
    "adamascaría": "adamascar",
    "adamascaríais": "adamascar",
    "adamascaríamos": "adamascar",
    "adamascarían": "adamascar",
    "adamascarías": "adamascar",
    "adamascas": "adamascar",
    "adamascase": "adamascar",
    "adamascaseis": "adamascar",
    "adamascasen": "adamascar",
    "adamascases": "adamascar",
    "adamascaste": "adamascar",
    "adamascasteis": "adamascar",
    "adamasco": "adamascar",
    "adamascábamos": "adamascar",
    "adamascáis": "adamascar",
    "adamascáramos": "adamascar",
    "adamascáremos": "adamascar",
    "adamascásemos": "adamascar",
    "adamascó": "adamascar",
    "adamase": "adamar",
    "adamaseis": "adamar",
    "adamasen": "adamar",
    "adamases": "adamar",
    "adamasque": "adamascar",
    "adamasquemos": "adamascar",
    "adamasquen": "adamascar",
    "adamasques": "adamascar",
    "adamasqué": "adamascar",
    "adamasquéis": "adamascar",
    "adamaste": "adamar",
    "adamasteis": "adamar",
    "adame": "adamar",
    "adamemos": "adamar",
    "adamen": "adamar",
    "adames": "adamar",
    "adamismos": "adamismo",
    "adamitas": "adamita",
    "adamo": "adamar",
    "adamos": "adir",
    "adamábamos": "adamar",
    "adamáis": "adamar",
    "adamáramos": "adamar",
    "adamáremos": "adamar",
    "adamásemos": "adamar",
    "adamé": "adamar",
    "adaméis": "adamar",
    "adamó": "adamar",
    "adan": "adir",
    "adanes": "adán",
    "adapta": "adaptar",
    "adaptaba": "adaptar",
    "adaptabais": "adaptar",
    "adaptaban": "adaptar",
    "adaptabas": "adaptar",
    "adaptabilidades": "adaptabilidad",
    "adaptables": "adaptable",
    "adaptaciones": "adaptación",
    "adaptad": "adaptar",
    "adaptada": "adaptar",
    "adaptadas": "adaptar",
    "adaptado": "adaptar",
    "adaptadora": "adaptador",
    "adaptadoras": "adaptador",
    "adaptadores": "adaptador",
    "adaptados": "adaptar",
    "adaptamos": "adaptar",
    "adaptan": "adaptar",
    "adaptando": "adaptar",
    "adaptara": "adaptar",
    "adaptarais": "adaptar",
    "adaptaran": "adaptar",
    "adaptaras": "adaptar",
    "adaptare": "adaptar",
    "adaptareis": "adaptar",
    "adaptaremos": "adaptar",
    "adaptaren": "adaptar",
    "adaptares": "adaptar",
    "adaptaron": "adaptar",
    "adaptará": "adaptar",
    "adaptarán": "adaptar",
    "adaptarás": "adaptar",
    "adaptaré": "adaptar",
    "adaptaréis": "adaptar",
    "adaptaría": "adaptar",
    "adaptaríais": "adaptar",
    "adaptaríamos": "adaptar",
    "adaptarían": "adaptar",
    "adaptarías": "adaptar",
    "adaptas": "adaptar",
    "adaptase": "adaptar",
    "adaptaseis": "adaptar",
    "adaptasen": "adaptar",
    "adaptases": "adaptar",
    "adaptaste": "adaptar",
    "adaptasteis": "adaptar",
    "adaptativa": "adaptativo",
    "adaptativas": "adaptativo",
    "adaptativos": "adaptativo",
    "adapte": "adaptar",
    "adaptemos": "adaptar",
    "adapten": "adaptar",
    "adaptes": "adaptar",
    "adapto": "adaptar",
    "adaptábamos": "adaptar",
    "adaptáis": "adaptar",
    "adaptáramos": "adaptar",
    "adaptáremos": "adaptar",
    "adaptásemos": "adaptar",
    "adapté": "adaptar",
    "adaptéis": "adaptar",
    "adaptó": "adaptar",
    "adaptómetros": "adaptómetro",
    "adarajas": "adaraja",
    "adarces": "adarce",
    "adardea": "adardear",
    "adardeaba": "adardear",
    "adardeabais": "adardear",
    "adardeaban": "adardear",
    "adardeabas": "adardear",
    "adardead": "adardear",
    "adardeada": "adardear",
    "adardeadas": "adardear",
    "adardeado": "adardear",
    "adardeados": "adardear",
    "adardeamos": "adardear",
    "adardean": "adardear",
    "adardeando": "adardear",
    "adardeara": "adardear",
    "adardearais": "adardear",
    "adardearan": "adardear",
    "adardearas": "adardear",
    "adardeare": "adardear",
    "adardeareis": "adardear",
    "adardearemos": "adardear",
    "adardearen": "adardear",
    "adardeares": "adardear",
    "adardearon": "adardear",
    "adardeará": "adardear",
    "adardearán": "adardear",
    "adardearás": "adardear",
    "adardearé": "adardear",
    "adardearéis": "adardear",
    "adardearía": "adardear",
    "adardearíais": "adardear",
    "adardearíamos": "adardear",
    "adardearían": "adardear",
    "adardearías": "adardear",
    "adardeas": "adardear",
    "adardease": "adardear",
    "adardeaseis": "adardear",
    "adardeasen": "adardear",
    "adardeases": "adardear",
    "adardeaste": "adardear",
    "adardeasteis": "adardear",
    "adardee": "adardear",
    "adardeemos": "adardear",
    "adardeen": "adardear",
    "adardees": "adardear",
    "adardeo": "adardear",
    "adardeábamos": "adardear",
    "adardeáis": "adardear",
    "adardeáramos": "adardear",
    "adardeáremos": "adardear",
    "adardeásemos": "adardear",
    "adardeé": "adardear",
    "adardeéis": "adardear",
    "adardeó": "adardear",
    "adarga": "adargar",
    "adargaba": "adargar",
    "adargabais": "adargar",
    "adargaban": "adargar",
    "adargabas": "adargar",
    "adargad": "adargar",
    "adargada": "adargar",
    "adargadas": "adargar",
    "adargado": "adargar",
    "adargados": "adargar",
    "adargamos": "adargar",
    "adargan": "adargar",
    "adargando": "adargar",
    "adargara": "adargar",
    "adargarais": "adargar",
    "adargaran": "adargar",
    "adargaras": "adargar",
    "adargare": "adargar",
    "adargareis": "adargar",
    "adargaremos": "adargar",
    "adargaren": "adargar",
    "adargares": "adargar",
    "adargaron": "adargar",
    "adargará": "adargar",
    "adargarán": "adargar",
    "adargarás": "adargar",
    "adargaré": "adargar",
    "adargaréis": "adargar",
    "adargaría": "adargar",
    "adargaríais": "adargar",
    "adargaríamos": "adargar",
    "adargarían": "adargar",
    "adargarías": "adargar",
    "adargas": "adargar",
    "adargase": "adargar",
    "adargaseis": "adargar",
    "adargasen": "adargar",
    "adargases": "adargar",
    "adargaste": "adargar",
    "adargasteis": "adargar",
    "adargo": "adargar",
    "adargue": "adargar",
    "adarguemos": "adargar",
    "adarguen": "adargar",
    "adargueros": "adarguero",
    "adargues": "adargar",
    "adargué": "adargar",
    "adarguéis": "adargar",
    "adargábamos": "adargar",
    "adargáis": "adargar",
    "adargáramos": "adargar",
    "adargáremos": "adargar",
    "adargásemos": "adargar",
    "adargó": "adargar",
    "adarmes": "adarme",
    "adarva": "adarvar",
    "adarvaba": "adarvar",
    "adarvabais": "adarvar",
    "adarvaban": "adarvar",
    "adarvabas": "adarvar",
    "adarvad": "adarvar",
    "adarvada": "adarvar",
    "adarvadas": "adarvar",
    "adarvado": "adarvar",
    "adarvados": "adarvar",
    "adarvamos": "adarvar",
    "adarvan": "adarvar",
    "adarvando": "adarvar",
    "adarvara": "adarvar",
    "adarvarais": "adarvar",
    "adarvaran": "adarvar",
    "adarvaras": "adarvar",
    "adarvare": "adarvar",
    "adarvareis": "adarvar",
    "adarvaremos": "adarvar",
    "adarvaren": "adarvar",
    "adarvares": "adarvar",
    "adarvaron": "adarvar",
    "adarvará": "adarvar",
    "adarvarán": "adarvar",
    "adarvarás": "adarvar",
    "adarvaré": "adarvar",
    "adarvaréis": "adarvar",
    "adarvaría": "adarvar",
    "adarvaríais": "adarvar",
    "adarvaríamos": "adarvar",
    "adarvarían": "adarvar",
    "adarvarías": "adarvar",
    "adarvas": "adarvar",
    "adarvase": "adarvar",
    "adarvaseis": "adarvar",
    "adarvasen": "adarvar",
    "adarvases": "adarvar",
    "adarvaste": "adarvar",
    "adarvasteis": "adarvar",
    "adarve": "adarvar",
    "adarvemos": "adarvar",
    "adarven": "adarvar",
    "adarves": "adarve",
    "adarvo": "adarvar",
    "adarvábamos": "adarvar",
    "adarváis": "adarvar",
    "adarváramos": "adarvar",
    "adarváremos": "adarvar",
    "adarvásemos": "adarvar",
    "adarvé": "adarvar",
    "adarvéis": "adarvar",
    "adarvó": "adarvar",
    "adas": "adir",
    "adazas": "adaza",
    "addendas": "addenda",
    "adductos": "adducto",
    "ade": "adir",
    "adecena": "adecenar",
    "adecenaba": "adecenar",
    "adecenabais": "adecenar",
    "adecenaban": "adecenar",
    "adecenabas": "adecenar",
    "adecenad": "adecenar",
    "adecenada": "adecenar",
    "adecenadas": "adecenar",
    "adecenado": "adecenar",
    "adecenados": "adecenar",
    "adecenamos": "adecenar",
    "adecenan": "adecenar",
    "adecenando": "adecenar",
    "adecenara": "adecenar",
    "adecenarais": "adecenar",
    "adecenaran": "adecenar",
    "adecenaras": "adecenar",
    "adecenare": "adecenar",
    "adecenareis": "adecenar",
    "adecenaremos": "adecenar",
    "adecenaren": "adecenar",
    "adecenares": "adecenar",
    "adecenaron": "adecenar",
    "adecenará": "adecenar",
    "adecenarán": "adecenar",
    "adecenarás": "adecenar",
    "adecenaré": "adecenar",
    "adecenaréis": "adecenar",
    "adecenaría": "adecenar",
    "adecenaríais": "adecenar",
    "adecenaríamos": "adecenar",
    "adecenarían": "adecenar",
    "adecenarías": "adecenar",
    "adecenas": "adecenar",
    "adecenase": "adecenar",
    "adecenaseis": "adecenar",
    "adecenasen": "adecenar",
    "adecenases": "adecenar",
    "adecenaste": "adecenar",
    "adecenasteis": "adecenar",
    "adecene": "adecenar",
    "adecenemos": "adecenar",
    "adecenen": "adecenar",
    "adecenes": "adecenar",
    "adeceno": "adecenar",
    "adecenta": "adecentar",
    "adecentaba": "adecentar",
    "adecentabais": "adecentar",
    "adecentaban": "adecentar",
    "adecentabas": "adecentar",
    "adecentad": "adecentar",
    "adecentada": "adecentar",
    "adecentadas": "adecentar",
    "adecentado": "adecentar",
    "adecentados": "adecentar",
    "adecentamientos": "adecentamiento",
    "adecentamos": "adecentar",
    "adecentan": "adecentar",
    "adecentando": "adecentar",
    "adecentara": "adecentar",
    "adecentarais": "adecentar",
    "adecentaran": "adecentar",
    "adecentaras": "adecentar",
    "adecentare": "adecentar",
    "adecentareis": "adecentar",
    "adecentaremos": "adecentar",
    "adecentaren": "adecentar",
    "adecentares": "adecentar",
    "adecentaron": "adecentar",
    "adecentará": "adecentar",
    "adecentarán": "adecentar",
    "adecentarás": "adecentar",
    "adecentaré": "adecentar",
    "adecentaréis": "adecentar",
    "adecentaría": "adecentar",
    "adecentaríais": "adecentar",
    "adecentaríamos": "adecentar",
    "adecentarían": "adecentar",
    "adecentarías": "adecentar",
    "adecentas": "adecentar",
    "adecentase": "adecentar",
    "adecentaseis": "adecentar",
    "adecentasen": "adecentar",
    "adecentases": "adecentar",
    "adecentaste": "adecentar",
    "adecentasteis": "adecentar",
    "adecente": "adecentar",
    "adecentemos": "adecentar",
    "adecenten": "adecentar",
    "adecentes": "adecentar",
    "adecento": "adecentar",
    "adecentábamos": "adecentar",
    "adecentáis": "adecentar",
    "adecentáramos": "adecentar",
    "adecentáremos": "adecentar",
    "adecentásemos": "adecentar",
    "adecenté": "adecentar",
    "adecentéis": "adecentar",
    "adecentó": "adecentar",
    "adecenábamos": "adecenar",
    "adecenáis": "adecenar",
    "adecenáramos": "adecenar",
    "adecenáremos": "adecenar",
    "adecenásemos": "adecenar",
    "adecené": "adecenar",
    "adecenéis": "adecenar",
    "adecenó": "adecenar",
    "adecuaba": "adecuar",
    "adecuabais": "adecuar",
    "adecuaban": "adecuar",
    "adecuabas": "adecuar",
    "adecuaciones": "adecuación",
    "adecuad": "adecuar",
    "adecuada": "adecuar",
    "adecuadas": "adecuar",
    "adecuado": "adecuar",
    "adecuados": "adecuar",
    "adecuamos": "adecuar",
    "adecuando": "adecuar",
    "adecuara": "adecuar",
    "adecuarais": "adecuar",
    "adecuaran": "adecuar",
    "adecuaras": "adecuar",
    "adecuare": "adecuar",
    "adecuareis": "adecuar",
    "adecuaremos": "adecuar",
    "adecuaren": "adecuar",
    "adecuares": "adecuar",
    "adecuaron": "adecuar",
    "adecuará": "adecuar",
    "adecuarán": "adecuar",
    "adecuarás": "adecuar",
    "adecuaré": "adecuar",
    "adecuaréis": "adecuar",
    "adecuaría": "adecuar",
    "adecuaríais": "adecuar",
    "adecuaríamos": "adecuar",
    "adecuarían": "adecuar",
    "adecuarías": "adecuar",
    "adecuase": "adecuar",
    "adecuaseis": "adecuar",
    "adecuasen": "adecuar",
    "adecuases": "adecuar",
    "adecuaste": "adecuar",
    "adecuasteis": "adecuar",
    "adecuemos": "adecuar",
    "adecuábamos": "adecuar",
    "adecuáis": "adecuar",
    "adecuáramos": "adecuar",
    "adecuáremos": "adecuar",
    "adecuásemos": "adecuar",
    "adecué": "adecuar",
    "adecuéis": "adecuar",
    "adecuó": "adecuar",
    "adecúa": "adecuar",
    "adecúan": "adecuar",
    "adecúas": "adecuar",
    "adecúe": "adecuar",
    "adecúen": "adecuar",
    "adecúes": "adecuar",
    "adecúo": "adecuar",
    "adefagias": "adefagia",
    "adefesios": "adefesio",
    "adehalas": "adehala",
    "adelanta": "adelantar",
    "adelantaba": "adelantar",
    "adelantabais": "adelantar",
    "adelantaban": "adelantar",
    "adelantabas": "adelantar",
    "adelantad": "adelantar",
    "adelantada": "adelantar",
    "adelantadas": "adelantar",
    "adelantado": "adelantar",
    "adelantadores": "adelantador",
    "adelantados": "adelantar",
    "adelantamientos": "adelantamiento",
    "adelantamos": "adelantar",
    "adelantan": "adelantar",
    "adelantando": "adelantar",
    "adelantara": "adelantar",
    "adelantarais": "adelantar",
    "adelantaran": "adelantar",
    "adelantaras": "adelantar",
    "adelantare": "adelantar",
    "adelantareis": "adelantar",
    "adelantaremos": "adelantar",
    "adelantaren": "adelantar",
    "adelantares": "adelantar",
    "adelantaron": "adelantar",
    "adelantará": "adelantar",
    "adelantarán": "adelantar",
    "adelantarás": "adelantar",
    "adelantaré": "adelantar",
    "adelantaréis": "adelantar",
    "adelantaría": "adelantar",
    "adelantaríais": "adelantar",
    "adelantaríamos": "adelantar",
    "adelantarían": "adelantar",
    "adelantarías": "adelantar",
    "adelantas": "adelantar",
    "adelantase": "adelantar",
    "adelantaseis": "adelantar",
    "adelantasen": "adelantar",
    "adelantases": "adelantar",
    "adelantaste": "adelantar",
    "adelantasteis": "adelantar",
    "adelante": "adelantar",
    "adelantemos": "adelantar",
    "adelanten": "adelantar",
    "adelantes": "adelantar",
    "adelanto": "adelantar",
    "adelantos": "adelanto",
    "adelantábamos": "adelantar",
    "adelantáis": "adelantar",
    "adelantáramos": "adelantar",
    "adelantáremos": "adelantar",
    "adelantásemos": "adelantar",
    "adelanté": "adelantar",
    "adelantéis": "adelantar",
    "adelantó": "adelantar",
    "adelfales": "adelfal",
    "adelfas": "adelfa",
    "adelgace": "adelgazar",
    "adelgacemos": "adelgazar",
    "adelgacen": "adelgazar",
    "adelgaces": "adelgazar",
    "adelgacé": "adelgazar",
    "adelgacéis": "adelgazar",
    "adelgaza": "adelgazar",
    "adelgazaba": "adelgazar",
    "adelgazabais": "adelgazar",
    "adelgazaban": "adelgazar",
    "adelgazabas": "adelgazar",
    "adelgazad": "adelgazar",
    "adelgazada": "adelgazar",
    "adelgazadas": "adelgazar",
    "adelgazado": "adelgazar",
    "adelgazadora": "adelgazador",
    "adelgazadoras": "adelgazador",
    "adelgazadores": "adelgazador",
    "adelgazados": "adelgazar",
    "adelgazamientos": "adelgazamiento",
    "adelgazamos": "adelgazar",
    "adelgazan": "adelgazar",
    "adelgazando": "adelgazar",
    "adelgazara": "adelgazar",
    "adelgazarais": "adelgazar",
    "adelgazaran": "adelgazar",
    "adelgazaras": "adelgazar",
    "adelgazare": "adelgazar",
    "adelgazareis": "adelgazar",
    "adelgazaremos": "adelgazar",
    "adelgazaren": "adelgazar",
    "adelgazares": "adelgazar",
    "adelgazaron": "adelgazar",
    "adelgazará": "adelgazar",
    "adelgazarán": "adelgazar",
    "adelgazarás": "adelgazar",
    "adelgazaré": "adelgazar",
    "adelgazaréis": "adelgazar",
    "adelgazaría": "adelgazar",
    "adelgazaríais": "adelgazar",
    "adelgazaríamos": "adelgazar",
    "adelgazarían": "adelgazar",
    "adelgazarías": "adelgazar",
    "adelgazas": "adelgazar",
    "adelgazase": "adelgazar",
    "adelgazaseis": "adelgazar",
    "adelgazasen": "adelgazar",
    "adelgazases": "adelgazar",
    "adelgazaste": "adelgazar",
    "adelgazasteis": "adelgazar",
    "adelgazo": "adelgazar",
    "adelgazábamos": "adelgazar",
    "adelgazáis": "adelgazar",
    "adelgazáramos": "adelgazar",
    "adelgazáremos": "adelgazar",
    "adelgazásemos": "adelgazar",
    "adelgazó": "adelgazar",
    "adelta": "adelto",
    "adeltas": "adelto",
    "adeltos": "adelto",
    "adema": "ademar",
    "ademaba": "ademar",
    "ademabais": "ademar",
    "ademaban": "ademar",
    "ademabas": "ademar",
    "ademad": "ademar",
    "ademada": "ademar",
    "ademadas": "ademar",
    "ademado": "ademar",
    "ademados": "ademar",
    "ademamos": "ademar",
    "ademan": "ademar",
    "ademando": "ademar",
    "ademanes": "ademán",
    "ademara": "ademar",
    "ademarais": "ademar",
    "ademaran": "ademar",
    "ademaras": "ademar",
    "ademare": "ademar",
    "ademareis": "ademar",
    "ademaremos": "ademar",
    "ademaren": "ademar",
    "ademares": "ademar",
    "ademaron": "ademar",
    "ademará": "ademar",
    "ademarán": "ademar",
    "ademarás": "ademar",
    "ademaré": "ademar",
    "ademaréis": "ademar",
    "ademaría": "ademar",
    "ademaríais": "ademar",
    "ademaríamos": "ademar",
    "ademarían": "ademar",
    "ademarías": "ademar",
    "ademas": "ademar",
    "ademase": "ademar",
    "ademaseis": "ademar",
    "ademasen": "ademar",
    "ademases": "ademar",
    "ademaste": "ademar",
    "ademasteis": "ademar",
    "ademe": "ademar",
    "adememos": "ademar",
    "ademen": "ademar",
    "ademes": "ademe",
    "ademo": "ademar",
    "ademábamos": "ademar",
    "ademáis": "ademar",
    "ademánes": "ademán",
    "ademáramos": "ademar",
    "ademáremos": "ademar",
    "ademásemos": "ademar",
    "ademé": "ademar",
    "ademéis": "ademar",
    "ademó": "ademar",
    "aden": "adir",
    "adendas": "adenda",
    "adeninas": "adenina",
    "adenoameloblastomas": "adenoameloblastoma",
    "adenocromos": "adenocromo",
    "adenogalias": "adenogalia",
    "adenogramas": "adenograma",
    "adenoidea": "adenoideo",
    "adenoideas": "adenoideo",
    "adenoidectomías": "adenoidectomía",
    "adenoideos": "adenoideo",
    "adenoides": "adenoide",
    "adenologías": "adenología",
    "adenomas": "adenoma",
    "adenopatías": "adenopatía",
    "adenosinas": "adenosina",
    "adenosindifosfatos": "adenosindifosfato",
    "adenosinmonofosfatos": "adenosinmonofosfato",
    "adenosintrifosfatasas": "adenosintrifosfatasa",
    "adenosintrifosfatos": "adenosintrifosfato",
    "adentra": "adentrar",
    "adentraba": "adentrar",
    "adentrabais": "adentrar",
    "adentraban": "adentrar",
    "adentrabas": "adentrar",
    "adentrad": "adentrar",
    "adentrada": "adentrar",
    "adentradas": "adentrar",
    "adentrado": "adentrar",
    "adentrados": "adentrar",
    "adentramientos": "adentramiento",
    "adentramos": "adentrar",
    "adentran": "adentrar",
    "adentrando": "adentrar",
    "adentrara": "adentrar",
    "adentrarais": "adentrar",
    "adentraran": "adentrar",
    "adentraras": "adentrar",
    "adentrare": "adentrar",
    "adentrareis": "adentrar",
    "adentraremos": "adentrar",
    "adentraren": "adentrar",
    "adentrares": "adentrar",
    "adentraron": "adentrar",
    "adentrará": "adentrar",
    "adentrarán": "adentrar",
    "adentrarás": "adentrar",
    "adentraré": "adentrar",
    "adentraréis": "adentrar",
    "adentraría": "adentrar",
    "adentraríais": "adentrar",
    "adentraríamos": "adentrar",
    "adentrarían": "adentrar",
    "adentrarías": "adentrar",
    "adentras": "adentrar",
    "adentrase": "adentrar",
    "adentraseis": "adentrar",
    "adentrasen": "adentrar",
    "adentrases": "adentrar",
    "adentraste": "adentrar",
    "adentrasteis": "adentrar",
    "adentre": "adentrar",
    "adentremos": "adentrar",
    "adentren": "adentrar",
    "adentres": "adentrar",
    "adentro": "adentrar",
    "adentrábamos": "adentrar",
    "adentráis": "adentrar",
    "adentráramos": "adentrar",
    "adentráremos": "adentrar",
    "adentrásemos": "adentrar",
    "adentré": "adentrar",
    "adentréis": "adentrar",
    "adentró": "adentrar",
    "adenílica": "adenílico",
    "adenílicas": "adenílico",
    "adenílicos": "adenílico",
    "adepta": "adepto",
    "adeptas": "adepto",
    "adeptos": "adepto",
    "aderece": "aderezar",
    "aderecemos": "aderezar",
    "aderecen": "aderezar",
    "adereces": "aderezar",
    "aderecé": "aderezar",
    "aderecéis": "aderezar",
    "adereza": "aderezar",
    "aderezaba": "aderezar",
    "aderezabais": "aderezar",
    "aderezaban": "aderezar",
    "aderezabas": "aderezar",
    "aderezad": "aderezar",
    "aderezada": "aderezar",
    "aderezadas": "aderezar",
    "aderezado": "aderezar",
    "aderezados": "aderezar",
    "aderezamientos": "aderezamiento",
    "aderezamos": "aderezar",
    "aderezan": "aderezar",
    "aderezando": "aderezar",
    "aderezara": "aderezar",
    "aderezarais": "aderezar",
    "aderezaran": "aderezar",
    "aderezaras": "aderezar",
    "aderezare": "aderezar",
    "aderezareis": "aderezar",
    "aderezaremos": "aderezar",
    "aderezaren": "aderezar",
    "aderezares": "aderezar",
    "aderezaron": "aderezar",
    "aderezará": "aderezar",
    "aderezarán": "aderezar",
    "aderezarás": "aderezar",
    "aderezaré": "aderezar",
    "aderezaréis": "aderezar",
    "aderezaría": "aderezar",
    "aderezaríais": "aderezar",
    "aderezaríamos": "aderezar",
    "aderezarían": "aderezar",
    "aderezarías": "aderezar",
    "aderezas": "aderezar",
    "aderezase": "aderezar",
    "aderezaseis": "aderezar",
    "aderezasen": "aderezar",
    "aderezases": "aderezar",
    "aderezaste": "aderezar",
    "aderezasteis": "aderezar",
    "aderezo": "aderezar",
    "aderezos": "aderezo",
    "aderezábamos": "aderezar",
    "aderezáis": "aderezar",
    "aderezáramos": "aderezar",
    "aderezáremos": "aderezar",
    "aderezásemos": "aderezar",
    "aderezó": "aderezar",
    "ades": "adir",
    "adestra": "adestrar",
    "adestraba": "adestrar",
    "adestrabais": "adestrar",
    "adestraban": "adestrar",
    "adestrabas": "adestrar",
    "adestrad": "adestrar",
    "adestrada": "adestrar",
    "adestradas": "adestrar",
    "adestrado": "adestrar",
    "adestrados": "adestrar",
    "adestramos": "adestrar",
    "adestran": "adestrar",
    "adestrando": "adestrar",
    "adestrara": "adestrar",
    "adestrarais": "adestrar",
    "adestraran": "adestrar",
    "adestraras": "adestrar",
    "adestrare": "adestrar",
    "adestrareis": "adestrar",
    "adestraremos": "adestrar",
    "adestraren": "adestrar",
    "adestrares": "adestrar",
    "adestraron": "adestrar",
    "adestrará": "adestrar",
    "adestrarán": "adestrar",
    "adestrarás": "adestrar",
    "adestraré": "adestrar",
    "adestraréis": "adestrar",
    "adestraría": "adestrar",
    "adestraríais": "adestrar",
    "adestraríamos": "adestrar",
    "adestrarían": "adestrar",
    "adestrarías": "adestrar",
    "adestras": "adestrar",
    "adestrase": "adestrar",
    "adestraseis": "adestrar",
    "adestrasen": "adestrar",
    "adestrases": "adestrar",
    "adestraste": "adestrar",
    "adestrasteis": "adestrar",
    "adestre": "adestrar",
    "adestremos": "adestrar",
    "adestren": "adestrar",
    "adestres": "adestrar",
    "adestro": "adestrar",
    "adestrábamos": "adestrar",
    "adestráis": "adestrar",
    "adestráramos": "adestrar",
    "adestráremos": "adestrar",
    "adestrásemos": "adestrar",
    "adestré": "adestrar",
    "adestréis": "adestrar",
    "adestró": "adestrar",
    "adeuda": "adeudar",
    "adeudaba": "adeudar",
    "adeudabais": "adeudar",
    "adeudaban": "adeudar",
    "adeudabas": "adeudar",
    "adeudad": "adeudar",
    "adeudada": "adeudar",
    "adeudadas": "adeudar",
    "adeudado": "adeudar",
    "adeudados": "adeudar",
    "adeudamos": "adeudar",
    "adeudan": "adeudar",
    "adeudando": "adeudar",
    "adeudara": "adeudar",
    "adeudarais": "adeudar",
    "adeudaran": "adeudar",
    "adeudaras": "adeudar",
    "adeudare": "adeudar",
    "adeudareis": "adeudar",
    "adeudaremos": "adeudar",
    "adeudaren": "adeudar",
    "adeudares": "adeudar",
    "adeudaron": "adeudar",
    "adeudará": "adeudar",
    "adeudarán": "adeudar",
    "adeudarás": "adeudar",
    "adeudaré": "adeudar",
    "adeudaréis": "adeudar",
    "adeudaría": "adeudar",
    "adeudaríais": "adeudar",
    "adeudaríamos": "adeudar",
    "adeudarían": "adeudar",
    "adeudarías": "adeudar",
    "adeudas": "adeudar",
    "adeudase": "adeudar",
    "adeudaseis": "adeudar",
    "adeudasen": "adeudar",
    "adeudases": "adeudar",
    "adeudaste": "adeudar",
    "adeudasteis": "adeudar",
    "adeude": "adeudar",
    "adeudemos": "adeudar",
    "adeuden": "adeudar",
    "adeudes": "adeudar",
    "adeudo": "adeudar",
    "adeudos": "adeudo",
    "adeudábamos": "adeudar",
    "adeudáis": "adeudar",
    "adeudáramos": "adeudar",
    "adeudáremos": "adeudar",
    "adeudásemos": "adeudar",
    "adeudé": "adeudar",
    "adeudéis": "adeudar",
    "adeudó": "adeudar",
    "adherencias": "adherencia",
    "adherentes": "adherente",
    "adherid": "adherir",
    "adherida": "adherir",
    "adheridas": "adherir",
    "adherido": "adherir",
    "adheridos": "adherir",
    "adherimos": "adherir",
    "adheriremos": "adherir",
    "adherirá": "adherir",
    "adherirán": "adherir",
    "adherirás": "adherir",
    "adheriré": "adherir",
    "adheriréis": "adherir",
    "adheriría": "adherir",
    "adheriríais": "adherir",
    "adheriríamos": "adherir",
    "adherirían": "adherir",
    "adherirías": "adherir",
    "adheriste": "adherir",
    "adheristeis": "adherir",
    "adherí": "adherir",
    "adhería": "adherir",
    "adheríais": "adherir",
    "adheríamos": "adherir",
    "adherían": "adherir",
    "adherías": "adherir",
    "adherís": "adherir",
    "adhesiones": "adhesión",
    "adhesiva": "adhesivo",
    "adhesivas": "adhesivo",
    "adhesividades": "adhesividad",
    "adhesivos": "adhesivo",
    "adhiera": "adherir",
    "adhieran": "adherir",
    "adhieras": "adherir",
    "adhiere": "adherir",
    "adhieren": "adherir",
    "adhieres": "adherir",
    "adhiero": "adherir",
    "adhiramos": "adherir",
    "adhiriendo": "adherir",
    "adhiriera": "adherir",
    "adhirierais": "adherir",
    "adhirieran": "adherir",
    "adhirieras": "adherir",
    "adhiriere": "adherir",
    "adhiriereis": "adherir",
    "adhirieren": "adherir",
    "adhirieres": "adherir",
    "adhirieron": "adherir",
    "adhiriese": "adherir",
    "adhirieseis": "adherir",
    "adhiriesen": "adherir",
    "adhirieses": "adherir",
    "adhiriéramos": "adherir",
    "adhiriéremos": "adherir",
    "adhiriésemos": "adherir",
    "adhirió": "adherir",
    "adhiráis": "adherir",
    "adiabática": "adiabático",
    "adiabáticas": "adiabático",
    "adiabáticos": "adiabático",
    "adiactínica": "adiactínico",
    "adiactínicas": "adiactínico",
    "adiactínicos": "adiactínico",
    "adiamantada": "adiamantado",
    "adiamantadas": "adiamantado",
    "adiamantados": "adiamantado",
    "adiamientos": "adiamiento",
    "adicciones": "adicción",
    "adiciona": "adicionar",
    "adicionaba": "adicionar",
    "adicionabais": "adicionar",
    "adicionaban": "adicionar",
    "adicionabas": "adicionar",
    "adicionables": "adicionable",
    "adicionad": "adicionar",
    "adicionada": "adicionar",
    "adicionadas": "adicionar",
    "adicionado": "adicionar",
    "adicionadora": "adicionador",
    "adicionadoras": "adicionador",
    "adicionadores": "adicionador",
    "adicionados": "adicionar",
    "adicionales": "adicional",
    "adicionamos": "adicionar",
    "adicionan": "adicionar",
    "adicionando": "adicionar",
    "adicionara": "adicionar",
    "adicionarais": "adicionar",
    "adicionaran": "adicionar",
    "adicionaras": "adicionar",
    "adicionare": "adicionar",
    "adicionareis": "adicionar",
    "adicionaremos": "adicionar",
    "adicionaren": "adicionar",
    "adicionares": "adicionar",
    "adicionaron": "adicionar",
    "adicionará": "adicionar",
    "adicionarán": "adicionar",
    "adicionarás": "adicionar",
    "adicionaré": "adicionar",
    "adicionaréis": "adicionar",
    "adicionaría": "adicionar",
    "adicionaríais": "adicionar",
    "adicionaríamos": "adicionar",
    "adicionarían": "adicionar",
    "adicionarías": "adicionar",
    "adicionas": "adicionar",
    "adicionase": "adicionar",
    "adicionaseis": "adicionar",
    "adicionasen": "adicionar",
    "adicionases": "adicionar",
    "adicionaste": "adicionar",
    "adicionasteis": "adicionar",
    "adicione": "adicionar",
    "adicionemos": "adicionar",
    "adicionen": "adicionar",
    "adiciones": "adicionar",
    "adiciono": "adicionar",
    "adicionábamos": "adicionar",
    "adicionáis": "adicionar",
    "adicionáramos": "adicionar",
    "adicionáremos": "adicionar",
    "adicionásemos": "adicionar",
    "adicioné": "adicionar",
    "adicionéis": "adicionar",
    "adicionó": "adicionar",
    "adicta": "adicto",
    "adictas": "adicto",
    "adictiva": "adictivo",
    "adictivas": "adictivo",
    "adictivos": "adictivo",
    "adictos": "adicto",
    "adid": "adir",
    "adida": "adir",
    "adidas": "adir",
    "adido": "adir",
    "adidos": "adir",
    "adiendo": "adir",
    "adiera": "adir",
    "adierais": "adir",
    "adieran": "adir",
    "adieras": "adir",
    "adiere": "adir",
    "adiereis": "adir",
    "adieren": "adir",
    "adieres": "adir",
    "adieron": "adir",
    "adiese": "adir",
    "adieseis": "adir",
    "adiesen": "adir",
    "adieses": "adir",
    "adiestra": "adiestrar",
    "adiestraba": "adiestrar",
    "adiestrabais": "adiestrar",
    "adiestraban": "adiestrar",
    "adiestrabas": "adiestrar",
    "adiestrad": "adiestrar",
    "adiestrada": "adiestrar",
    "adiestradas": "adiestrar",
    "adiestrado": "adiestrar",
    "adiestradora": "adiestrador",
    "adiestradoras": "adiestrador",
    "adiestradores": "adiestrador",
    "adiestrados": "adiestrar",
    "adiestramientos": "adiestramiento",
    "adiestramos": "adiestrar",
    "adiestran": "adiestrar",
    "adiestrando": "adiestrar",
    "adiestrara": "adiestrar",
    "adiestrarais": "adiestrar",
    "adiestraran": "adiestrar",
    "adiestraras": "adiestrar",
    "adiestrare": "adiestrar",
    "adiestrareis": "adiestrar",
    "adiestraremos": "adiestrar",
    "adiestraren": "adiestrar",
    "adiestrares": "adiestrar",
    "adiestraron": "adiestrar",
    "adiestrará": "adiestrar",
    "adiestrarán": "adiestrar",
    "adiestrarás": "adiestrar",
    "adiestraré": "adiestrar",
    "adiestraréis": "adiestrar",
    "adiestraría": "adiestrar",
    "adiestraríais": "adiestrar",
    "adiestraríamos": "adiestrar",
    "adiestrarían": "adiestrar",
    "adiestrarías": "adiestrar",
    "adiestras": "adiestrar",
    "adiestrase": "adiestrar",
    "adiestraseis": "adiestrar",
    "adiestrasen": "adiestrar",
    "adiestrases": "adiestrar",
    "adiestraste": "adiestrar",
    "adiestrasteis": "adiestrar",
    "adiestre": "adiestrar",
    "adiestremos": "adiestrar",
    "adiestren": "adiestrar",
    "adiestres": "adiestrar",
    "adiestro": "adiestrar",
    "adiestrábamos": "adiestrar",
    "adiestráis": "adiestrar",
    "adiestráramos": "adiestrar",
    "adiestráremos": "adiestrar",
    "adiestrásemos": "adiestrar",
    "adiestré": "adiestrar",
    "adiestréis": "adiestrar",
    "adiestró": "adiestrar",
    "adieta": "adietar",
    "adietaba": "adietar",
    "adietabais": "adietar",
    "adietaban": "adietar",
    "adietabas": "adietar",
    "adietad": "adietar",
    "adietada": "adietar",
    "adietadas": "adietar",
    "adietado": "adietar",
    "adietados": "adietar",
    "adietamos": "adietar",
    "adietan": "adietar",
    "adietando": "adietar",
    "adietara": "adietar",
    "adietarais": "adietar",
    "adietaran": "adietar",
    "adietaras": "adietar",
    "adietare": "adietar",
    "adietareis": "adietar",
    "adietaremos": "adietar",
    "adietaren": "adietar",
    "adietares": "adietar",
    "adietaron": "adietar",
    "adietará": "adietar",
    "adietarán": "adietar",
    "adietarás": "adietar",
    "adietaré": "adietar",
    "adietaréis": "adietar",
    "adietaría": "adietar",
    "adietaríais": "adietar",
    "adietaríamos": "adietar",
    "adietarían": "adietar",
    "adietarías": "adietar",
    "adietas": "adietar",
    "adietase": "adietar",
    "adietaseis": "adietar",
    "adietasen": "adietar",
    "adietases": "adietar",
    "adietaste": "adietar",
    "adietasteis": "adietar",
    "adiete": "adietar",
    "adietemos": "adietar",
    "adieten": "adietar",
    "adietes": "adietar",
    "adieto": "adietar",
    "adietábamos": "adietar",
    "adietáis": "adietar",
    "adietáramos": "adietar",
    "adietáremos": "adietar",
    "adietásemos": "adietar",
    "adieté": "adietar",
    "adietéis": "adietar",
    "adietó": "adietar",
    "adimensionales": "adimensional",
    "adimos": "adir",
    "adinamias": "adinamia",
    "adinera": "adinerar",
    "adineraba": "adinerar",
    "adinerabais": "adinerar",
    "adineraban": "adinerar",
    "adinerabas": "adinerar",
    "adinerad": "adinerar",
    "adinerada": "adinerar",
    "adineradas": "adinerar",
    "adinerado": "adinerar",
    "adinerados": "adinerar",
    "adineramos": "adinerar",
    "adineran": "adinerar",
    "adinerando": "adinerar",
    "adinerara": "adinerar",
    "adinerarais": "adinerar",
    "adineraran": "adinerar",
    "adineraras": "adinerar",
    "adinerare": "adinerar",
    "adinerareis": "adinerar",
    "adineraremos": "adinerar",
    "adineraren": "adinerar",
    "adinerares": "adinerar",
    "adineraron": "adinerar",
    "adinerará": "adinerar",
    "adinerarán": "adinerar",
    "adinerarás": "adinerar",
    "adineraré": "adinerar",
    "adineraréis": "adinerar",
    "adineraría": "adinerar",
    "adineraríais": "adinerar",
    "adineraríamos": "adinerar",
    "adinerarían": "adinerar",
    "adinerarías": "adinerar",
    "adineras": "adinerar",
    "adinerase": "adinerar",
    "adineraseis": "adinerar",
    "adinerasen": "adinerar",
    "adinerases": "adinerar",
    "adineraste": "adinerar",
    "adinerasteis": "adinerar",
    "adinere": "adinerar",
    "adineremos": "adinerar",
    "adineren": "adinerar",
    "adineres": "adinerar",
    "adinero": "adinerar",
    "adinerábamos": "adinerar",
    "adineráis": "adinerar",
    "adineráramos": "adinerar",
    "adineráremos": "adinerar",
    "adinerásemos": "adinerar",
    "adineré": "adinerar",
    "adineréis": "adinerar",
    "adineró": "adinerar",
    "adintelada": "adintelado",
    "adinteladas": "adintelado",
    "adintelados": "adintelado",
    "adinámica": "adinámico",
    "adinámicas": "adinámico",
    "adinámicos": "adinámico",
    "adioses": "adiós",
    "adipatos": "adipato",
    "adipoblastos": "adipoblasto",
    "adipociras": "adipocira",
    "adipocitos": "adipocito",
    "adiponitrilos": "adiponitrilo",
    "adiposa": "adiposo",
    "adiposas": "adiposo",
    "adiposidades": "adiposidad",
    "adiposos": "adiposo",
    "adipsias": "adipsia",
    "adiremos": "adir",
    "adirá": "adir",
    "adirán": "adir",
    "adirás": "adir",
    "adiré": "adir",
    "adiréis": "adir",
    "adiría": "adir",
    "adiríais": "adir",
    "adiríamos": "adir",
    "adirían": "adir",
    "adirías": "adir",
    "adiste": "adir",
    "adisteis": "adir",
    "aditamentos": "aditamento",
    "aditamientos": "aditamiento",
    "aditiva": "aditivo",
    "aditivas": "aditivo",
    "aditividades": "aditividad",
    "aditivos": "aditivo",
    "adivas": "adiva",
    "adives": "adive",
    "adivina": "adivino",
    "adivinaba": "adivinar",
    "adivinabais": "adivinar",
    "adivinaban": "adivinar",
    "adivinabas": "adivinar",
    "adivinables": "adivinable",
    "adivinaciones": "adivinación",
    "adivinad": "adivinar",
    "adivinada": "adivinar",
    "adivinadas": "adivinar",
    "adivinado": "adivinar",
    "adivinadora": "adivinador",
    "adivinadoras": "adivinador",
    "adivinadores": "adivinador",
    "adivinados": "adivinar",
    "adivinamientos": "adivinamiento",
    "adivinamos": "adivinar",
    "adivinan": "adivinar",
    "adivinando": "adivinar",
    "adivinanzas": "adivinanza",
    "adivinara": "adivinar",
    "adivinarais": "adivinar",
    "adivinaran": "adivinar",
    "adivinaras": "adivinar",
    "adivinare": "adivinar",
    "adivinareis": "adivinar",
    "adivinaremos": "adivinar",
    "adivinaren": "adivinar",
    "adivinares": "adivinar",
    "adivinaron": "adivinar",
    "adivinará": "adivinar",
    "adivinarán": "adivinar",
    "adivinarás": "adivinar",
    "adivinaré": "adivinar",
    "adivinaréis": "adivinar",
    "adivinaría": "adivinar",
    "adivinaríais": "adivinar",
    "adivinaríamos": "adivinar",
    "adivinarían": "adivinar",
    "adivinarías": "adivinar",
    "adivinas": "adivino",
    "adivinase": "adivinar",
    "adivinaseis": "adivinar",
    "adivinasen": "adivinar",
    "adivinases": "adivinar",
    "adivinaste": "adivinar",
    "adivinasteis": "adivinar",
    "adivinatoria": "adivinatorio",
    "adivinatorias": "adivinatorio",
    "adivinatorios": "adivinatorio",
    "adivine": "adivinar",
    "adivinemos": "adivinar",
    "adivinen": "adivinar",
    "adivines": "adivinar",
    "adivino": "adivinar",
    "adivinos": "adivino",
    "adivinábamos": "adivinar",
    "adivináis": "adivinar",
    "adivináramos": "adivinar",
    "adivináremos": "adivinar",
    "adivinásemos": "adivinar",
    "adiviné": "adivinar",
    "adivinéis": "adivinar",
    "adivinó": "adivinar",
    "adiéramos": "adir",
    "adiéremos": "adir",
    "adiésemos": "adir",
    "adió": "adir",
    "adjetiva": "adjetivo",
    "adjetivaba": "adjetivar",
    "adjetivabais": "adjetivar",
    "adjetivaban": "adjetivar",
    "adjetivabas": "adjetivar",
    "adjetivaciones": "adjetivación",
    "adjetivad": "adjetivar",
    "adjetivada": "adjetivar",
    "adjetivadas": "adjetivar",
    "adjetivado": "adjetivar",
    "adjetivadora": "adjetivador",
    "adjetivadoras": "adjetivador",
    "adjetivadores": "adjetivador",
    "adjetivados": "adjetivar",
    "adjetivales": "adjetival",
    "adjetivamos": "adjetivar",
    "adjetivan": "adjetivar",
    "adjetivando": "adjetivar",
    "adjetivara": "adjetivar",
    "adjetivarais": "adjetivar",
    "adjetivaran": "adjetivar",
    "adjetivaras": "adjetivar",
    "adjetivare": "adjetivar",
    "adjetivareis": "adjetivar",
    "adjetivaremos": "adjetivar",
    "adjetivaren": "adjetivar",
    "adjetivares": "adjetivar",
    "adjetivaron": "adjetivar",
    "adjetivará": "adjetivar",
    "adjetivarán": "adjetivar",
    "adjetivarás": "adjetivar",
    "adjetivaré": "adjetivar",
    "adjetivaréis": "adjetivar",
    "adjetivaría": "adjetivar",
    "adjetivaríais": "adjetivar",
    "adjetivaríamos": "adjetivar",
    "adjetivarían": "adjetivar",
    "adjetivarías": "adjetivar",
    "adjetivas": "adjetivo",
    "adjetivase": "adjetivar",
    "adjetivaseis": "adjetivar",
    "adjetivasen": "adjetivar",
    "adjetivases": "adjetivar",
    "adjetivaste": "adjetivar",
    "adjetivasteis": "adjetivar",
    "adjetive": "adjetivar",
    "adjetivemos": "adjetivar",
    "adjetiven": "adjetivar",
    "adjetives": "adjetivar",
    "adjetivizaciones": "adjetivización",
    "adjetivo": "adjetivar",
    "adjetivos": "adjetivo",
    "adjetivábamos": "adjetivar",
    "adjetiváis": "adjetivar",
    "adjetiváramos": "adjetivar",
    "adjetiváremos": "adjetivar",
    "adjetivásemos": "adjetivar",
    "adjetivé": "adjetivar",
    "adjetivéis": "adjetivar",
    "adjetivó": "adjetivar",
    "adjudica": "adjudicar",
    "adjudicaba": "adjudicar",
    "adjudicabais": "adjudicar",
    "adjudicaban": "adjudicar",
    "adjudicabas": "adjudicar",
    "adjudicables": "adjudicable",
    "adjudicaciones": "adjudicación",
    "adjudicad": "adjudicar",
    "adjudicada": "adjudicar",
    "adjudicadas": "adjudicar",
    "adjudicado": "adjudicar",
    "adjudicadora": "adjudicador",
    "adjudicadoras": "adjudicador",
    "adjudicadores": "adjudicador",
    "adjudicados": "adjudicar",
    "adjudicamos": "adjudicar",
    "adjudican": "adjudicar",
    "adjudicando": "adjudicar",
    "adjudicara": "adjudicar",
    "adjudicarais": "adjudicar",
    "adjudicaran": "adjudicar",
    "adjudicaras": "adjudicar",
    "adjudicare": "adjudicar",
    "adjudicareis": "adjudicar",
    "adjudicaremos": "adjudicar",
    "adjudicaren": "adjudicar",
    "adjudicares": "adjudicar",
    "adjudicaron": "adjudicar",
    "adjudicará": "adjudicar",
    "adjudicarán": "adjudicar",
    "adjudicarás": "adjudicar",
    "adjudicaré": "adjudicar",
    "adjudicaréis": "adjudicar",
    "adjudicaría": "adjudicar",
    "adjudicaríais": "adjudicar",
    "adjudicaríamos": "adjudicar",
    "adjudicarían": "adjudicar",
    "adjudicarías": "adjudicar",
    "adjudicas": "adjudicar",
    "adjudicase": "adjudicar",
    "adjudicaseis": "adjudicar",
    "adjudicasen": "adjudicar",
    "adjudicases": "adjudicar",
    "adjudicaste": "adjudicar",
    "adjudicasteis": "adjudicar",
    "adjudicataria": "adjudicatario",
    "adjudicatarias": "adjudicatario",
    "adjudicatarios": "adjudicatario",
    "adjudico": "adjudicar",
    "adjudicábamos": "adjudicar",
    "adjudicáis": "adjudicar",
    "adjudicáramos": "adjudicar",
    "adjudicáremos": "adjudicar",
    "adjudicásemos": "adjudicar",
    "adjudicó": "adjudicar",
    "adjudique": "adjudicar",
    "adjudiquemos": "adjudicar",
    "adjudiquen": "adjudicar",
    "adjudiques": "adjudicar",
    "adjudiqué": "adjudicar",
    "adjudiquéis": "adjudicar",
    "adjunciones": "adjunción",
    "adjunta": "adjunto",
    "adjuntaba": "adjuntar",
    "adjuntabais": "adjuntar",
    "adjuntaban": "adjuntar",
    "adjuntabas": "adjuntar",
    "adjuntad": "adjuntar",
    "adjuntada": "adjuntar",
    "adjuntadas": "adjuntar",
    "adjuntado": "adjuntar",
    "adjuntados": "adjuntar",
    "adjuntamos": "adjuntar",
    "adjuntan": "adjuntar",
    "adjuntando": "adjuntar",
    "adjuntara": "adjuntar",
    "adjuntarais": "adjuntar",
    "adjuntaran": "adjuntar",
    "adjuntaras": "adjuntar",
    "adjuntare": "adjuntar",
    "adjuntareis": "adjuntar",
    "adjuntaremos": "adjuntar",
    "adjuntaren": "adjuntar",
    "adjuntares": "adjuntar",
    "adjuntaron": "adjuntar",
    "adjuntará": "adjuntar",
    "adjuntarán": "adjuntar",
    "adjuntarás": "adjuntar",
    "adjuntaré": "adjuntar",
    "adjuntaréis": "adjuntar",
    "adjuntaría": "adjuntar",
    "adjuntaríais": "adjuntar",
    "adjuntaríamos": "adjuntar",
    "adjuntarían": "adjuntar",
    "adjuntarías": "adjuntar",
    "adjuntas": "adjunto",
    "adjuntase": "adjuntar",
    "adjuntaseis": "adjuntar",
    "adjuntasen": "adjuntar",
    "adjuntases": "adjuntar",
    "adjuntaste": "adjuntar",
    "adjuntasteis": "adjuntar",
    "adjunte": "adjuntar",
    "adjuntemos": "adjuntar",
    "adjunten": "adjuntar",
    "adjuntes": "adjuntar",
    "adjunto": "adjuntar",
    "adjuntos": "adjunto",
    "adjuntábamos": "adjuntar",
    "adjuntáis": "adjuntar",
    "adjuntáramos": "adjuntar",
    "adjuntáremos": "adjuntar",
    "adjuntásemos": "adjuntar",
    "adjunté": "adjuntar",
    "adjuntéis": "adjuntar",
    "adjuntó": "adjuntar",
    "adjura": "adjurar",
    "adjuraba": "adjurar",
    "adjurabais": "adjurar",
    "adjuraban": "adjurar",
    "adjurabas": "adjurar",
    "adjuraciones": "adjuración",
    "adjurad": "adjurar",
    "adjurada": "adjurar",
    "adjuradas": "adjurar",
    "adjurado": "adjurar",
    "adjurados": "adjurar",
    "adjuramos": "adjurar",
    "adjuran": "adjurar",
    "adjurando": "adjurar",
    "adjurara": "adjurar",
    "adjurarais": "adjurar",
    "adjuraran": "adjurar",
    "adjuraras": "adjurar",
    "adjurare": "adjurar",
    "adjurareis": "adjurar",
    "adjuraremos": "adjurar",
    "adjuraren": "adjurar",
    "adjurares": "adjurar",
    "adjuraron": "adjurar",
    "adjurará": "adjurar",
    "adjurarán": "adjurar",
    "adjurarás": "adjurar",
    "adjuraré": "adjurar",
    "adjuraréis": "adjurar",
    "adjuraría": "adjurar",
    "adjuraríais": "adjurar",
    "adjuraríamos": "adjurar",
    "adjurarían": "adjurar",
    "adjurarías": "adjurar",
    "adjuras": "adjurar",
    "adjurase": "adjurar",
    "adjuraseis": "adjurar",
    "adjurasen": "adjurar",
    "adjurases": "adjurar",
    "adjuraste": "adjurar",
    "adjurasteis": "adjurar",
    "adjure": "adjurar",
    "adjuremos": "adjurar",
    "adjuren": "adjurar",
    "adjures": "adjurar",
    "adjuro": "adjurar",
    "adjurábamos": "adjurar",
    "adjuráis": "adjurar",
    "adjuráramos": "adjurar",
    "adjuráremos": "adjurar",
    "adjurásemos": "adjurar",
    "adjuré": "adjurar",
    "adjuréis": "adjurar",
    "adjuró": "adjurar",
    "adlativa": "adlativo",
    "adlativas": "adlativo",
    "adlativos": "adlativo",
    "adláteres": "adlátere",
    "admantoblastomas": "admantoblastoma",
    "adminicula": "adminicular",
    "adminiculaba": "adminicular",
    "adminiculabais": "adminicular",
    "adminiculaban": "adminicular",
    "adminiculabas": "adminicular",
    "adminiculad": "adminicular",
    "adminiculada": "adminicular",
    "adminiculadas": "adminicular",
    "adminiculado": "adminicular",
    "adminiculados": "adminicular",
    "adminiculamos": "adminicular",
    "adminiculan": "adminicular",
    "adminiculando": "adminicular",
    "adminiculara": "adminicular",
    "adminicularais": "adminicular",
    "adminicularan": "adminicular",
    "adminicularas": "adminicular",
    "adminiculare": "adminicular",
    "adminiculareis": "adminicular",
    "adminicularemos": "adminicular",
    "adminicularen": "adminicular",
    "adminiculares": "adminicular",
    "adminicularon": "adminicular",
    "adminiculará": "adminicular",
    "adminicularán": "adminicular",
    "adminicularás": "adminicular",
    "adminicularé": "adminicular",
    "adminicularéis": "adminicular",
    "adminicularía": "adminicular",
    "adminicularíais": "adminicular",
    "adminicularíamos": "adminicular",
    "adminicularían": "adminicular",
    "adminicularías": "adminicular",
    "adminiculas": "adminicular",
    "adminiculase": "adminicular",
    "adminiculaseis": "adminicular",
    "adminiculasen": "adminicular",
    "adminiculases": "adminicular",
    "adminiculaste": "adminicular",
    "adminiculasteis": "adminicular",
    "adminicule": "adminicular",
    "adminiculemos": "adminicular",
    "adminiculen": "adminicular",
    "adminicules": "adminicular",
    "adminiculo": "adminicular",
    "adminiculábamos": "adminicular",
    "adminiculáis": "adminicular",
    "adminiculáramos": "adminicular",
    "adminiculáremos": "adminicular",
    "adminiculásemos": "adminicular",
    "adminiculé": "adminicular",
    "adminiculéis": "adminicular",
    "adminiculó": "adminicular",
    "administra": "administrar",
    "administraba": "administrar",
    "administrabais": "administrar",
    "administraban": "administrar",
    "administrabas": "administrar",
    "administrables": "administrable",
    "administraciones": "administración",
    "administrad": "administrar",
    "administrada": "administrar",
    "administradas": "administrar",
    "administrado": "administrar",
    "administradora": "administrador",
    "administradoras": "administrador",
    "administradores": "administrador",
    "administrados": "administrar",
    "administramos": "administrar",
    "administran": "administrar",
    "administrando": "administrar",
    "administrara": "administrar",
    "administrarais": "administrar",
    "administraran": "administrar",
    "administraras": "administrar",
    "administrare": "administrar",
    "administrareis": "administrar",
    "administraremos": "administrar",
    "administraren": "administrar",
    "administrares": "administrar",
    "administraron": "administrar",
    "administrará": "administrar",
    "administrarán": "administrar",
    "administrarás": "administrar",
    "administraré": "administrar",
    "administraréis": "administrar",
    "administraría": "administrar",
    "administraríais": "administrar",
    "administraríamos": "administrar",
    "administrarían": "administrar",
    "administrarías": "administrar",
    "administras": "administrar",
    "administrase": "administrar",
    "administraseis": "administrar",
    "administrasen": "administrar",
    "administrases": "administrar",
    "administraste": "administrar",
    "administrasteis": "administrar",
    "administrativa": "administrativo",
    "administrativas": "administrativo",
    "administrativos": "administrativo",
    "administre": "administrar",
    "administremos": "administrar",
    "administren": "administrar",
    "administres": "administrar",
    "administro": "administrar",
    "administrábamos": "administrar",
    "administráis": "administrar",
    "administráramos": "administrar",
    "administráremos": "administrar",
    "administrásemos": "administrar",
    "administré": "administrar",
    "administréis": "administrar",
    "administró": "administrar",
    "adminículos": "adminículo",
    "admira": "admirar",
    "admiraba": "admirar",
    "admirabais": "admirar",
    "admiraban": "admirar",
    "admirabas": "admirar",
    "admirables": "admirable",
    "admiraciones": "admiración",
    "admirad": "admirar",
    "admirada": "admirar",
    "admiradas": "admirar",
    "admirado": "admirar",
    "admiradora": "admirador",
    "admiradoras": "admirador",
    "admiradores": "admirador",
    "admirados": "admirar",
    "admiramos": "admirar",
    "admiran": "admirar",
    "admirando": "admirar",
    "admirara": "admirar",
    "admirarais": "admirar",
    "admiraran": "admirar",
    "admiraras": "admirar",
    "admirare": "admirar",
    "admirareis": "admirar",
    "admiraremos": "admirar",
    "admiraren": "admirar",
    "admirares": "admirar",
    "admiraron": "admirar",
    "admirará": "admirar",
    "admirarán": "admirar",
    "admirarás": "admirar",
    "admiraré": "admirar",
    "admiraréis": "admirar",
    "admiraría": "admirar",
    "admiraríais": "admirar",
    "admiraríamos": "admirar",
    "admirarían": "admirar",
    "admirarías": "admirar",
    "admiras": "admirar",
    "admirase": "admirar",
    "admiraseis": "admirar",
    "admirasen": "admirar",
    "admirases": "admirar",
    "admiraste": "admirar",
    "admirasteis": "admirar",
    "admirativa": "admirativo",
    "admirativas": "admirativo",
    "admirativos": "admirativo",
    "admire": "admirar",
    "admiremos": "admirar",
    "admiren": "admirar",
    "admires": "admirar",
    "admiro": "admirar",
    "admirábamos": "admirar",
    "admiráis": "admirar",
    "admiráramos": "admirar",
    "admiráremos": "admirar",
    "admirásemos": "admirar",
    "admiré": "admirar",
    "admiréis": "admirar",
    "admiró": "admirar",
    "admisibilidades": "admisibilidad",
    "admisibles": "admisible",
    "admisiones": "admisión",
    "admita": "admitir",
    "admitamos": "admitir",
    "admitan": "admitir",
    "admitancias": "admitancia",
    "admitas": "admitir",
    "admite": "admitir",
    "admiten": "admitir",
    "admites": "admitir",
    "admitid": "admitir",
    "admitida": "admitir",
    "admitidas": "admitir",
    "admitido": "admitir",
    "admitidos": "admitir",
    "admitiendo": "admitir",
    "admitiera": "admitir",
    "admitierais": "admitir",
    "admitieran": "admitir",
    "admitieras": "admitir",
    "admitiere": "admitir",
    "admitiereis": "admitir",
    "admitieren": "admitir",
    "admitieres": "admitir",
    "admitieron": "admitir",
    "admitiese": "admitir",
    "admitieseis": "admitir",
    "admitiesen": "admitir",
    "admitieses": "admitir",
    "admitimos": "admitir",
    "admitiremos": "admitir",
    "admitirá": "admitir",
    "admitirán": "admitir",
    "admitirás": "admitir",
    "admitiré": "admitir",
    "admitiréis": "admitir",
    "admitiría": "admitir",
    "admitiríais": "admitir",
    "admitiríamos": "admitir",
    "admitirían": "admitir",
    "admitirías": "admitir",
    "admitiste": "admitir",
    "admitisteis": "admitir",
    "admitiéramos": "admitir",
    "admitiéremos": "admitir",
    "admitiésemos": "admitir",
    "admitió": "admitir",
    "admito": "admitir",
    "admitáis": "admitir",
    "admití": "admitir",
    "admitía": "admitir",
    "admitíais": "admitir",
    "admitíamos": "admitir",
    "admitían": "admitir",
    "admitías": "admitir",
    "admitís": "admitir",
    "admoniciones": "admonición",
    "admonitora": "admonitor",
    "admonitoras": "admonitor",
    "admonitores": "admonitor",
    "admonitoria": "admonitorio",
    "admonitorias": "admonitorio",
    "admonitorios": "admonitorio",
    "admón": "administración",
    "adnata": "adnato",
    "adnatas": "adnato",
    "adnatos": "adnato",
    "ado": "adir",
    "adoba": "adobar",
    "adobaba": "adobar",
    "adobabais": "adobar",
    "adobaban": "adobar",
    "adobabas": "adobar",
    "adobad": "adobar",
    "adobada": "adobar",
    "adobadas": "adobar",
    "adobado": "adobar",
    "adobadora": "adobador",
    "adobadoras": "adobador",
    "adobadores": "adobador",
    "adobados": "adobar",
    "adobamientos": "adobamiento",
    "adobamos": "adobar",
    "adoban": "adobar",
    "adobando": "adobar",
    "adobara": "adobar",
    "adobarais": "adobar",
    "adobaran": "adobar",
    "adobaras": "adobar",
    "adobare": "adobar",
    "adobareis": "adobar",
    "adobaremos": "adobar",
    "adobaren": "adobar",
    "adobares": "adobar",
    "adobaron": "adobar",
    "adobará": "adobar",
    "adobarán": "adobar",
    "adobarás": "adobar",
    "adobaré": "adobar",
    "adobaréis": "adobar",
    "adobaría": "adobar",
    "adobaríais": "adobar",
    "adobaríamos": "adobar",
    "adobarían": "adobar",
    "adobarías": "adobar",
    "adobas": "adobar",
    "adobase": "adobar",
    "adobaseis": "adobar",
    "adobasen": "adobar",
    "adobases": "adobar",
    "adobaste": "adobar",
    "adobasteis": "adobar",
    "adobe": "adobar",
    "adobemos": "adobar",
    "adoben": "adobar",
    "adoberas": "adobera",
    "adoberías": "adobería",
    "adobes": "adobe",
    "adobo": "adobar",
    "adobos": "adobo",
    "adobábamos": "adobar",
    "adobáis": "adobar",
    "adobáramos": "adobar",
    "adobáremos": "adobar",
    "adobásemos": "adobar",
    "adobé": "adobar",
    "adobéis": "adobar",
    "adobó": "adobar",
    "adocena": "adocenar",
    "adocenaba": "adocenar",
    "adocenabais": "adocenar",
    "adocenaban": "adocenar",
    "adocenabas": "adocenar",
    "adocenad": "adocenar",
    "adocenada": "adocenar",
    "adocenadas": "adocenar",
    "adocenado": "adocenar",
    "adocenados": "adocenar",
    "adocenamientos": "adocenamiento",
    "adocenamos": "adocenar",
    "adocenan": "adocenar",
    "adocenando": "adocenar",
    "adocenara": "adocenar",
    "adocenarais": "adocenar",
    "adocenaran": "adocenar",
    "adocenaras": "adocenar",
    "adocenare": "adocenar",
    "adocenareis": "adocenar",
    "adocenaremos": "adocenar",
    "adocenaren": "adocenar",
    "adocenares": "adocenar",
    "adocenaron": "adocenar",
    "adocenará": "adocenar",
    "adocenarán": "adocenar",
    "adocenarás": "adocenar",
    "adocenaré": "adocenar",
    "adocenaréis": "adocenar",
    "adocenaría": "adocenar",
    "adocenaríais": "adocenar",
    "adocenaríamos": "adocenar",
    "adocenarían": "adocenar",
    "adocenarías": "adocenar",
    "adocenas": "adocenar",
    "adocenase": "adocenar",
    "adocenaseis": "adocenar",
    "adocenasen": "adocenar",
    "adocenases": "adocenar",
    "adocenaste": "adocenar",
    "adocenasteis": "adocenar",
    "adocene": "adocenar",
    "adocenemos": "adocenar",
    "adocenen": "adocenar",
    "adocenes": "adocenar",
    "adoceno": "adocenar",
    "adocenábamos": "adocenar",
    "adocenáis": "adocenar",
    "adocenáramos": "adocenar",
    "adocenáremos": "adocenar",
    "adocenásemos": "adocenar",
    "adocené": "adocenar",
    "adocenéis": "adocenar",
    "adocenó": "adocenar",
    "adoctrina": "adoctrinar",
    "adoctrinaba": "adoctrinar",
    "adoctrinabais": "adoctrinar",
    "adoctrinaban": "adoctrinar",
    "adoctrinabas": "adoctrinar",
    "adoctrinad": "adoctrinar",
    "adoctrinada": "adoctrinar",
    "adoctrinadas": "adoctrinar",
    "adoctrinado": "adoctrinar",
    "adoctrinados": "adoctrinar",
    "adoctrinamientos": "adoctrinamiento",
    "adoctrinamos": "adoctrinar",
    "adoctrinan": "adoctrinar",
    "adoctrinando": "adoctrinar",
    "adoctrinara": "adoctrinar",
    "adoctrinarais": "adoctrinar",
    "adoctrinaran": "adoctrinar",
    "adoctrinaras": "adoctrinar",
    "adoctrinare": "adoctrinar",
    "adoctrinareis": "adoctrinar",
    "adoctrinaremos": "adoctrinar",
    "adoctrinaren": "adoctrinar",
    "adoctrinares": "adoctrinar",
    "adoctrinaron": "adoctrinar",
    "adoctrinará": "adoctrinar",
    "adoctrinarán": "adoctrinar",
    "adoctrinarás": "adoctrinar",
    "adoctrinaré": "adoctrinar",
    "adoctrinaréis": "adoctrinar",
    "adoctrinaría": "adoctrinar",
    "adoctrinaríais": "adoctrinar",
    "adoctrinaríamos": "adoctrinar",
    "adoctrinarían": "adoctrinar",
    "adoctrinarías": "adoctrinar",
    "adoctrinas": "adoctrinar",
    "adoctrinase": "adoctrinar",
    "adoctrinaseis": "adoctrinar",
    "adoctrinasen": "adoctrinar",
    "adoctrinases": "adoctrinar",
    "adoctrinaste": "adoctrinar",
    "adoctrinasteis": "adoctrinar",
    "adoctrine": "adoctrinar",
    "adoctrinemos": "adoctrinar",
    "adoctrinen": "adoctrinar",
    "adoctrines": "adoctrinar",
    "adoctrino": "adoctrinar",
    "adoctrinábamos": "adoctrinar",
    "adoctrináis": "adoctrinar",
    "adoctrináramos": "adoctrinar",
    "adoctrináremos": "adoctrinar",
    "adoctrinásemos": "adoctrinar",
    "adoctriné": "adoctrinar",
    "adoctrinéis": "adoctrinar",
    "adoctrinó": "adoctrinar",
    "adolece": "adolecer",
    "adoleced": "adolecer",
    "adolecemos": "adolecer",
    "adolecen": "adolecer",
    "adoleceremos": "adolecer",
    "adolecerá": "adolecer",
    "adolecerán": "adolecer",
    "adolecerás": "adolecer",
    "adoleceré": "adolecer",
    "adoleceréis": "adolecer",
    "adolecería": "adolecer",
    "adoleceríais": "adolecer",
    "adoleceríamos": "adolecer",
    "adolecerían": "adolecer",
    "adolecerías": "adolecer",
    "adoleces": "adolecer",
    "adolecida": "adolecer",
    "adolecidas": "adolecer",
    "adolecido": "adolecer",
    "adolecidos": "adolecer",
    "adoleciendo": "adolecer",
    "adoleciera": "adolecer",
    "adolecierais": "adolecer",
    "adolecieran": "adolecer",
    "adolecieras": "adolecer",
    "adoleciere": "adolecer",
    "adoleciereis": "adolecer",
    "adolecieren": "adolecer",
    "adolecieres": "adolecer",
    "adolecieron": "adolecer",
    "adoleciese": "adolecer",
    "adolecieseis": "adolecer",
    "adoleciesen": "adolecer",
    "adolecieses": "adolecer",
    "adolecimientos": "adolecimiento",
    "adolecimos": "adolecer",
    "adoleciste": "adolecer",
    "adolecisteis": "adolecer",
    "adoleciéramos": "adolecer",
    "adoleciéremos": "adolecer",
    "adoleciésemos": "adolecer",
    "adoleció": "adolecer",
    "adolecéis": "adolecer",
    "adolecí": "adolecer",
    "adolecía": "adolecer",
    "adolecíais": "adolecer",
    "adolecíamos": "adolecer",
    "adolecían": "adolecer",
    "adolecías": "adolecer",
    "adolescencias": "adolescencia",
    "adolescentes": "adolescente",
    "adolezca": "adolecer",
    "adolezcamos": "adolecer",
    "adolezcan": "adolecer",
    "adolezcas": "adolecer",
    "adolezco": "adolecer",
    "adolezcáis": "adolecer",
    "adopcionanismos": "adopcionanismo",
    "adopciones": "adopción",
    "adopcionismos": "adopcionismo",
    "adopcionistas": "adopcionista",
    "adopta": "adoptar",
    "adoptaba": "adoptar",
    "adoptabais": "adoptar",
    "adoptaban": "adoptar",
    "adoptabas": "adoptar",
    "adoptabilidades": "adoptabilidad",
    "adoptables": "adoptable",
    "adoptad": "adoptar",
    "adoptada": "adoptar",
    "adoptadas": "adoptar",
    "adoptado": "adoptar",
    "adoptados": "adoptar",
    "adoptamos": "adoptar",
    "adoptan": "adoptar",
    "adoptanda": "adoptando",
    "adoptandas": "adoptando",
    "adoptando": "adoptar",
    "adoptandos": "adoptando",
    "adoptantes": "adoptante",
    "adoptara": "adoptar",
    "adoptarais": "adoptar",
    "adoptaran": "adoptar",
    "adoptaras": "adoptar",
    "adoptare": "adoptar",
    "adoptareis": "adoptar",
    "adoptaremos": "adoptar",
    "adoptaren": "adoptar",
    "adoptares": "adoptar",
    "adoptaron": "adoptar",
    "adoptará": "adoptar",
    "adoptarán": "adoptar",
    "adoptarás": "adoptar",
    "adoptaré": "adoptar",
    "adoptaréis": "adoptar",
    "adoptaría": "adoptar",
    "adoptaríais": "adoptar",
    "adoptaríamos": "adoptar",
    "adoptarían": "adoptar",
    "adoptarías": "adoptar",
    "adoptas": "adoptar",
    "adoptase": "adoptar",
    "adoptaseis": "adoptar",
    "adoptasen": "adoptar",
    "adoptases": "adoptar",
    "adoptaste": "adoptar",
    "adoptasteis": "adoptar",
    "adopte": "adoptar",
    "adoptemos": "adoptar",
    "adopten": "adoptar",
    "adoptes": "adoptar",
    "adoptiva": "adoptivo",
    "adoptivas": "adoptivo",
    "adoptivos": "adoptivo",
    "adopto": "adoptar",
    "adoptábamos": "adoptar",
    "adoptáis": "adoptar",
    "adoptáramos": "adoptar",
    "adoptáremos": "adoptar",
    "adoptásemos": "adoptar",
    "adopté": "adoptar",
    "adoptéis": "adoptar",
    "adoptó": "adoptar",
    "adoquina": "adoquinar",
    "adoquinaba": "adoquinar",
    "adoquinabais": "adoquinar",
    "adoquinaban": "adoquinar",
    "adoquinabas": "adoquinar",
    "adoquinad": "adoquinar",
    "adoquinada": "adoquinar",
    "adoquinadas": "adoquinar",
    "adoquinado": "adoquinar",
    "adoquinados": "adoquinar",
    "adoquinamos": "adoquinar",
    "adoquinan": "adoquinar",
    "adoquinando": "adoquinar",
    "adoquinara": "adoquinar",
    "adoquinarais": "adoquinar",
    "adoquinaran": "adoquinar",
    "adoquinaras": "adoquinar",
    "adoquinare": "adoquinar",
    "adoquinareis": "adoquinar",
    "adoquinaremos": "adoquinar",
    "adoquinaren": "adoquinar",
    "adoquinares": "adoquinar",
    "adoquinaron": "adoquinar",
    "adoquinará": "adoquinar",
    "adoquinarán": "adoquinar",
    "adoquinarás": "adoquinar",
    "adoquinaré": "adoquinar",
    "adoquinaréis": "adoquinar",
    "adoquinaría": "adoquinar",
    "adoquinaríais": "adoquinar",
    "adoquinaríamos": "adoquinar",
    "adoquinarían": "adoquinar",
    "adoquinarías": "adoquinar",
    "adoquinas": "adoquinar",
    "adoquinase": "adoquinar",
    "adoquinaseis": "adoquinar",
    "adoquinasen": "adoquinar",
    "adoquinases": "adoquinar",
    "adoquinaste": "adoquinar",
    "adoquinasteis": "adoquinar",
    "adoquine": "adoquinar",
    "adoquinemos": "adoquinar",
    "adoquinen": "adoquinar",
    "adoquines": "adoquinar",
    "adoquino": "adoquinar",
    "adoquinábamos": "adoquinar",
    "adoquináis": "adoquinar",
    "adoquináramos": "adoquinar",
    "adoquináremos": "adoquinar",
    "adoquinásemos": "adoquinar",
    "adoquiné": "adoquinar",
    "adoquinéis": "adoquinar",
    "adoquinó": "adoquinar",
    "adora": "adorar",
    "adoraba": "adorar",
    "adorabais": "adorar",
    "adoraban": "adorar",
    "adorabas": "adorar",
    "adorables": "adorable",
    "adoraciones": "adoración",
    "adorad": "adorar",
    "adorada": "adorar",
    "adoradas": "adorar",
    "adorado": "adorar",
    "adoradora": "adorador",
    "adoradoras": "adorador",
    "adoradores": "adorador",
    "adorados": "adorar",
    "adoramos": "adorar",
    "adoran": "adorar",
    "adorando": "adorar",
    "adorara": "adorar",
    "adorarais": "adorar",
    "adoraran": "adorar",
    "adoraras": "adorar",
    "adorare": "adorar",
    "adorareis": "adorar",
    "adoraremos": "adorar",
    "adoraren": "adorar",
    "adorares": "adorar",
    "adoraron": "adorar",
    "adorará": "adorar",
    "adorarán": "adorar",
    "adorarás": "adorar",
    "adoraré": "adorar",
    "adoraréis": "adorar",
    "adoraría": "adorar",
    "adoraríais": "adorar",
    "adoraríamos": "adorar",
    "adorarían": "adorar",
    "adorarías": "adorar",
    "adoras": "adorar",
    "adorase": "adorar",
    "adoraseis": "adorar",
    "adorasen": "adorar",
    "adorases": "adorar",
    "adoraste": "adorar",
    "adorasteis": "adorar",
    "adoratorios": "adoratorio",
    "adoratrices": "adoratriz",
    "adore": "adorar",
    "adoremos": "adorar",
    "adoren": "adorar",
    "adores": "adorar",
    "adormece": "adormecer",
    "adormeced": "adormecer",
    "adormecedora": "adormecedor",
    "adormecedoras": "adormecedor",
    "adormecedores": "adormecedor",
    "adormecemos": "adormecer",
    "adormecen": "adormecer",
    "adormeceremos": "adormecer",
    "adormecerá": "adormecer",
    "adormecerán": "adormecer",
    "adormecerás": "adormecer",
    "adormeceré": "adormecer",
    "adormeceréis": "adormecer",
    "adormecería": "adormecer",
    "adormeceríais": "adormecer",
    "adormeceríamos": "adormecer",
    "adormecerían": "adormecer",
    "adormecerías": "adormecer",
    "adormeces": "adormecer",
    "adormecida": "adormecer",
    "adormecidas": "adormecer",
    "adormecido": "adormecer",
    "adormecidos": "adormecer",
    "adormeciendo": "adormecer",
    "adormeciera": "adormecer",
    "adormecierais": "adormecer",
    "adormecieran": "adormecer",
    "adormecieras": "adormecer",
    "adormeciere": "adormecer",
    "adormeciereis": "adormecer",
    "adormecieren": "adormecer",
    "adormecieres": "adormecer",
    "adormecieron": "adormecer",
    "adormeciese": "adormecer",
    "adormecieseis": "adormecer",
    "adormeciesen": "adormecer",
    "adormecieses": "adormecer",
    "adormecimientos": "adormecimiento",
    "adormecimos": "adormecer",
    "adormeciste": "adormecer",
    "adormecisteis": "adormecer",
    "adormeciéramos": "adormecer",
    "adormeciéremos": "adormecer",
    "adormeciésemos": "adormecer",
    "adormeció": "adormecer",
    "adormecéis": "adormecer",
    "adormecí": "adormecer",
    "adormecía": "adormecer",
    "adormecíais": "adormecer",
    "adormecíamos": "adormecer",
    "adormecían": "adormecer",
    "adormecías": "adormecer",
    "adormezca": "adormecer",
    "adormezcamos": "adormecer",
    "adormezcan": "adormecer",
    "adormezcas": "adormecer",
    "adormezco": "adormecer",
    "adormezcáis": "adormecer",
    "adormideras": "adormidera",
    "adormila": "adormilar",
    "adormilaba": "adormilar",
    "adormilabais": "adormilar",
    "adormilaban": "adormilar",
    "adormilabas": "adormilar",
    "adormilad": "adormilar",
    "adormilada": "adormilar",
    "adormiladas": "adormilar",
    "adormilado": "adormilar",
    "adormilados": "adormilar",
    "adormilamientos": "adormilamiento",
    "adormilamos": "adormilar",
    "adormilan": "adormilar",
    "adormilando": "adormilar",
    "adormilara": "adormilar",
    "adormilarais": "adormilar",
    "adormilaran": "adormilar",
    "adormilaras": "adormilar",
    "adormilare": "adormilar",
    "adormilareis": "adormilar",
    "adormilaremos": "adormilar",
    "adormilaren": "adormilar",
    "adormilares": "adormilar",
    "adormilaron": "adormilar",
    "adormilará": "adormilar",
    "adormilarán": "adormilar",
    "adormilarás": "adormilar",
    "adormilaré": "adormilar",
    "adormilaréis": "adormilar",
    "adormilaría": "adormilar",
    "adormilaríais": "adormilar",
    "adormilaríamos": "adormilar",
    "adormilarían": "adormilar",
    "adormilarías": "adormilar",
    "adormilas": "adormilar",
    "adormilase": "adormilar",
    "adormilaseis": "adormilar",
    "adormilasen": "adormilar",
    "adormilases": "adormilar",
    "adormilaste": "adormilar",
    "adormilasteis": "adormilar",
    "adormile": "adormilar",
    "adormilemos": "adormilar",
    "adormilen": "adormilar",
    "adormiles": "adormilar",
    "adormilo": "adormilar",
    "adormilábamos": "adormilar",
    "adormiláis": "adormilar",
    "adormiláramos": "adormilar",
    "adormiláremos": "adormilar",
    "adormilásemos": "adormilar",
    "adormilé": "adormilar",
    "adormiléis": "adormilar",
    "adormiló": "adormilar",
    "adormita": "adormitar",
    "adormitaba": "adormitar",
    "adormitabais": "adormitar",
    "adormitaban": "adormitar",
    "adormitabas": "adormitar",
    "adormitad": "adormitar",
    "adormitada": "adormitar",
    "adormitadas": "adormitar",
    "adormitado": "adormitar",
    "adormitados": "adormitar",
    "adormitamos": "adormitar",
    "adormitan": "adormitar",
    "adormitando": "adormitar",
    "adormitara": "adormitar",
    "adormitarais": "adormitar",
    "adormitaran": "adormitar",
    "adormitaras": "adormitar",
    "adormitare": "adormitar",
    "adormitareis": "adormitar",
    "adormitaremos": "adormitar",
    "adormitaren": "adormitar",
    "adormitares": "adormitar",
    "adormitaron": "adormitar",
    "adormitará": "adormitar",
    "adormitarán": "adormitar",
    "adormitarás": "adormitar",
    "adormitaré": "adormitar",
    "adormitaréis": "adormitar",
    "adormitaría": "adormitar",
    "adormitaríais": "adormitar",
    "adormitaríamos": "adormitar",
    "adormitarían": "adormitar",
    "adormitarías": "adormitar",
    "adormitas": "adormitar",
    "adormitase": "adormitar",
    "adormitaseis": "adormitar",
    "adormitasen": "adormitar",
    "adormitases": "adormitar",
    "adormitaste": "adormitar",
    "adormitasteis": "adormitar",
    "adormite": "adormitar",
    "adormitemos": "adormitar",
    "adormiten": "adormitar",
    "adormites": "adormitar",
    "adormito": "adormitar",
    "adormitábamos": "adormitar",
    "adormitáis": "adormitar",
    "adormitáramos": "adormitar",
    "adormitáremos": "adormitar",
    "adormitásemos": "adormitar",
    "adormité": "adormitar",
    "adormitéis": "adormitar",
    "adormitó": "adormitar",
    "adorna": "adornar",
    "adornaba": "adornar",
    "adornabais": "adornar",
    "adornaban": "adornar",
    "adornabas": "adornar",
    "adornad": "adornar",
    "adornada": "adornar",
    "adornadas": "adornar",
    "adornado": "adornar",
    "adornados": "adornar",
    "adornamientos": "adornamiento",
    "adornamos": "adornar",
    "adornan": "adornar",
    "adornando": "adornar",
    "adornara": "adornar",
    "adornarais": "adornar",
    "adornaran": "adornar",
    "adornaras": "adornar",
    "adornare": "adornar",
    "adornareis": "adornar",
    "adornaremos": "adornar",
    "adornaren": "adornar",
    "adornares": "adornar",
    "adornaron": "adornar",
    "adornará": "adornar",
    "adornarán": "adornar",
    "adornarás": "adornar",
    "adornaré": "adornar",
    "adornaréis": "adornar",
    "adornaría": "adornar",
    "adornaríais": "adornar",
    "adornaríamos": "adornar",
    "adornarían": "adornar",
    "adornarías": "adornar",
    "adornas": "adornar",
    "adornase": "adornar",
    "adornaseis": "adornar",
    "adornasen": "adornar",
    "adornases": "adornar",
    "adornaste": "adornar",
    "adornasteis": "adornar",
    "adorne": "adornar",
    "adornemos": "adornar",
    "adornen": "adornar",
    "adornes": "adornar",
    "adorno": "adornar",
    "adornos": "adorno",
    "adornábamos": "adornar",
    "adornáis": "adornar",
    "adornáramos": "adornar",
    "adornáremos": "adornar",
    "adornásemos": "adornar",
    "adorné": "adornar",
    "adornéis": "adornar",
    "adornó": "adornar",
    "adoro": "adorar",
    "adorábamos": "adorar",
    "adoráis": "adorar",
    "adoráramos": "adorar",
    "adoráremos": "adorar",
    "adorásemos": "adorar",
    "adoré": "adorar",
    "adoréis": "adorar",
    "adoró": "adorar",
    "adosa": "adosar",
    "adosaba": "adosar",
    "adosabais": "adosar",
    "adosaban": "adosar",
    "adosabas": "adosar",
    "adosad": "adosar",
    "adosada": "adosar",
    "adosadas": "adosar",
    "adosado": "adosar",
    "adosados": "adosar",
    "adosamientos": "adosamiento",
    "adosamos": "adosar",
    "adosan": "adosar",
    "adosando": "adosar",
    "adosara": "adosar",
    "adosarais": "adosar",
    "adosaran": "adosar",
    "adosaras": "adosar",
    "adosare": "adosar",
    "adosareis": "adosar",
    "adosaremos": "adosar",
    "adosaren": "adosar",
    "adosares": "adosar",
    "adosaron": "adosar",
    "adosará": "adosar",
    "adosarán": "adosar",
    "adosarás": "adosar",
    "adosaré": "adosar",
    "adosaréis": "adosar",
    "adosaría": "adosar",
    "adosaríais": "adosar",
    "adosaríamos": "adosar",
    "adosarían": "adosar",
    "adosarías": "adosar",
    "adosas": "adosar",
    "adosase": "adosar",
    "adosaseis": "adosar",
    "adosasen": "adosar",
    "adosases": "adosar",
    "adosaste": "adosar",
    "adosasteis": "adosar",
    "adose": "adosar",
    "adosemos": "adosar",
    "adosen": "adosar",
    "adoses": "adosar",
    "adoso": "adosar",
    "adosábamos": "adosar",
    "adosáis": "adosar",
    "adosáramos": "adosar",
    "adosáremos": "adosar",
    "adosásemos": "adosar",
    "adosé": "adosar",
    "adoséis": "adosar",
    "adosó": "adosar",
    "adquiera": "adquirir",
    "adquieran": "adquirir",
    "adquieras": "adquirir",
    "adquiere": "adquirir",
    "adquieren": "adquirir",
    "adquieres": "adquirir",
    "adquiero": "adquirir",
    "adquiramos": "adquirir",
    "adquirentes": "adquirente",
    "adquiribles": "adquirible",
    "adquirid": "adquirir",
    "adquirida": "adquirir",
    "adquiridas": "adquirir",
    "adquirido": "adquirir",
    "adquiridos": "adquirir",
    "adquiriendo": "adquirir",
    "adquirientes": "adquiriente",
    "adquiriera": "adquirir",
    "adquirierais": "adquirir",
    "adquirieran": "adquirir",
    "adquirieras": "adquirir",
    "adquiriere": "adquirir",
    "adquiriereis": "adquirir",
    "adquirieren": "adquirir",
    "adquirieres": "adquirir",
    "adquirieron": "adquirir",
    "adquiriese": "adquirir",
    "adquirieseis": "adquirir",
    "adquiriesen": "adquirir",
    "adquirieses": "adquirir",
    "adquirimos": "adquirir",
    "adquiriremos": "adquirir",
    "adquirirá": "adquirir",
    "adquirirán": "adquirir",
    "adquirirás": "adquirir",
    "adquiriré": "adquirir",
    "adquiriréis": "adquirir",
    "adquiriría": "adquirir",
    "adquiriríais": "adquirir",
    "adquiriríamos": "adquirir",
    "adquirirían": "adquirir",
    "adquirirías": "adquirir",
    "adquiriste": "adquirir",
    "adquiristeis": "adquirir",
    "adquiriéramos": "adquirir",
    "adquiriéremos": "adquirir",
    "adquiriésemos": "adquirir",
    "adquirió": "adquirir",
    "adquiráis": "adquirir",
    "adquirí": "adquirir",
    "adquiría": "adquirir",
    "adquiríais": "adquirir",
    "adquiríamos": "adquirir",
    "adquirían": "adquirir",
    "adquirías": "adquirir",
    "adquirís": "adquirir",
    "adquisiciones": "adquisición",
    "adquisidora": "adquisidor",
    "adquisidoras": "adquisidor",
    "adquisidores": "adquisidor",
    "adquisitiva": "adquisitivo",
    "adquisitivas": "adquisitivo",
    "adquisitivos": "adquisitivo",
    "adquisividades": "adquisividad",
    "adragantes": "adragante",
    "adragantos": "adraganto",
    "adrales": "adral",
    "adrares": "adrar",
    "adras": "adra",
    "adrenales": "adrenal",
    "adrenalinas": "adrenalina",
    "adrenocorticales": "adrenocortical",
    "adrenocorticotrópica": "adrenocorticotrópico",
    "adrenocorticotrópicas": "adrenocorticotrópico",
    "adrenocorticotrópicos": "adrenocorticotrópico",
    "adrenocromos": "adrenocromo",
    "adrenosteronas": "adrenosterona",
    "adresinas": "adresina",
    "adrice": "adrizar",
    "adricemos": "adrizar",
    "adricen": "adrizar",
    "adrices": "adrizar",
    "adricé": "adrizar",
    "adricéis": "adrizar",
    "adriza": "adrizar",
    "adrizaba": "adrizar",
    "adrizabais": "adrizar",
    "adrizaban": "adrizar",
    "adrizabas": "adrizar",
    "adrizad": "adrizar",
    "adrizada": "adrizar",
    "adrizadas": "adrizar",
    "adrizado": "adrizar",
    "adrizados": "adrizar",
    "adrizamientos": "adrizamiento",
    "adrizamos": "adrizar",
    "adrizan": "adrizar",
    "adrizando": "adrizar",
    "adrizara": "adrizar",
    "adrizarais": "adrizar",
    "adrizaran": "adrizar",
    "adrizaras": "adrizar",
    "adrizare": "adrizar",
    "adrizareis": "adrizar",
    "adrizaremos": "adrizar",
    "adrizaren": "adrizar",
    "adrizares": "adrizar",
    "adrizaron": "adrizar",
    "adrizará": "adrizar",
    "adrizarán": "adrizar",
    "adrizarás": "adrizar",
    "adrizaré": "adrizar",
    "adrizaréis": "adrizar",
    "adrizaría": "adrizar",
    "adrizaríais": "adrizar",
    "adrizaríamos": "adrizar",
    "adrizarían": "adrizar",
    "adrizarías": "adrizar",
    "adrizas": "adrizar",
    "adrizase": "adrizar",
    "adrizaseis": "adrizar",
    "adrizasen": "adrizar",
    "adrizases": "adrizar",
    "adrizaste": "adrizar",
    "adrizasteis": "adrizar",
    "adrizo": "adrizar",
    "adrizábamos": "adrizar",
    "adrizáis": "adrizar",
    "adrizáramos": "adrizar",
    "adrizáremos": "adrizar",
    "adrizásemos": "adrizar",
    "adrizó": "adrizar",
    "adriática": "adriático",
    "adriáticas": "adriático",
    "adriáticos": "adriático",
    "adroga": "adrogar",
    "adrogaba": "adrogar",
    "adrogabais": "adrogar",
    "adrogaban": "adrogar",
    "adrogabas": "adrogar",
    "adrogad": "adrogar",
    "adrogada": "adrogar",
    "adrogadas": "adrogar",
    "adrogado": "adrogar",
    "adrogados": "adrogar",
    "adrogamos": "adrogar",
    "adrogan": "adrogar",
    "adrogando": "adrogar",
    "adrogara": "adrogar",
    "adrogarais": "adrogar",
    "adrogaran": "adrogar",
    "adrogaras": "adrogar",
    "adrogare": "adrogar",
    "adrogareis": "adrogar",
    "adrogaremos": "adrogar",
    "adrogaren": "adrogar",
    "adrogares": "adrogar",
    "adrogaron": "adrogar",
    "adrogará": "adrogar",
    "adrogarán": "adrogar",
    "adrogarás": "adrogar",
    "adrogaré": "adrogar",
    "adrogaréis": "adrogar",
    "adrogaría": "adrogar",
    "adrogaríais": "adrogar",
    "adrogaríamos": "adrogar",
    "adrogarían": "adrogar",
    "adrogarías": "adrogar",
    "adrogas": "adrogar",
    "adrogase": "adrogar",
    "adrogaseis": "adrogar",
    "adrogasen": "adrogar",
    "adrogases": "adrogar",
    "adrogaste": "adrogar",
    "adrogasteis": "adrogar",
    "adrogo": "adrogar",
    "adrogue": "adrogar",
    "adroguemos": "adrogar",
    "adroguen": "adrogar",
    "adrogues": "adrogar",
    "adrogué": "adrogar",
    "adroguéis": "adrogar",
    "adrogábamos": "adrogar",
    "adrogáis": "adrogar",
    "adrogáramos": "adrogar",
    "adrogáremos": "adrogar",
    "adrogásemos": "adrogar",
    "adrogó": "adrogar",
    "adscriba": "adscribir",
    "adscribamos": "adscribir",
    "adscriban": "adscribir",
    "adscribas": "adscribir",
    "adscribe": "adscribir",
    "adscriben": "adscribir",
    "adscribes": "adscribir",
    "adscribid": "adscribir",
    "adscribiendo": "adscribir",
    "adscribiera": "adscribir",
    "adscribierais": "adscribir",
    "adscribieran": "adscribir",
    "adscribieras": "adscribir",
    "adscribiere": "adscribir",
    "adscribiereis": "adscribir",
    "adscribieren": "adscribir",
    "adscribieres": "adscribir",
    "adscribieron": "adscribir",
    "adscribiese": "adscribir",
    "adscribieseis": "adscribir",
    "adscribiesen": "adscribir",
    "adscribieses": "adscribir",
    "adscribimos": "adscribir",
    "adscribiremos": "adscribir",
    "adscribirá": "adscribir",
    "adscribirán": "adscribir",
    "adscribirás": "adscribir",
    "adscribiré": "adscribir",
    "adscribiréis": "adscribir",
    "adscribiría": "adscribir",
    "adscribiríais": "adscribir",
    "adscribiríamos": "adscribir",
    "adscribirían": "adscribir",
    "adscribirías": "adscribir",
    "adscribiste": "adscribir",
    "adscribisteis": "adscribir",
    "adscribiéramos": "adscribir",
    "adscribiéremos": "adscribir",
    "adscribiésemos": "adscribir",
    "adscribió": "adscribir",
    "adscribo": "adscribir",
    "adscribáis": "adscribir",
    "adscribí": "adscribir",
    "adscribía": "adscribir",
    "adscribíais": "adscribir",
    "adscribíamos": "adscribir",
    "adscribían": "adscribir",
    "adscribías": "adscribir",
    "adscribís": "adscribir",
    "adscripciones": "adscripción",
    "adscrita": "adscribir",
    "adscritas": "adscribir",
    "adscrito": "adscribir",
    "adscritos": "adscribir",
    "adsorba": "adsorber",
    "adsorbabilidades": "adsorbabilidad",
    "adsorbamos": "adsorber",
    "adsorban": "adsorber",
    "adsorbas": "adsorber",
    "adsorbatos": "adsorbato",
    "adsorbe": "adsorber",
    "adsorbed": "adsorber",
    "adsorbedores": "adsorbedor",
    "adsorbemos": "adsorber",
    "adsorben": "adsorber",
    "adsorbentes": "adsorbente",
    "adsorberemos": "adsorber",
    "adsorberá": "adsorber",
    "adsorberán": "adsorber",
    "adsorberás": "adsorber",
    "adsorberé": "adsorber",
    "adsorberéis": "adsorber",
    "adsorbería": "adsorber",
    "adsorberíais": "adsorber",
    "adsorberíamos": "adsorber",
    "adsorberían": "adsorber",
    "adsorberías": "adsorber",
    "adsorbes": "adsorber",
    "adsorbida": "adsorber",
    "adsorbidas": "adsorber",
    "adsorbido": "adsorber",
    "adsorbidos": "adsorber",
    "adsorbiendo": "adsorber",
    "adsorbiera": "adsorber",
    "adsorbierais": "adsorber",
    "adsorbieran": "adsorber",
    "adsorbieras": "adsorber",
    "adsorbiere": "adsorber",
    "adsorbiereis": "adsorber",
    "adsorbieren": "adsorber",
    "adsorbieres": "adsorber",
    "adsorbieron": "adsorber",
    "adsorbiese": "adsorber",
    "adsorbieseis": "adsorber",
    "adsorbiesen": "adsorber",
    "adsorbieses": "adsorber",
    "adsorbimos": "adsorber",
    "adsorbiste": "adsorber",
    "adsorbisteis": "adsorber",
    "adsorbiéramos": "adsorber",
    "adsorbiéremos": "adsorber",
    "adsorbiésemos": "adsorber",
    "adsorbió": "adsorber",
    "adsorbo": "adsorber",
    "adsorbáis": "adsorber",
    "adsorbéis": "adsorber",
    "adsorbí": "adsorber",
    "adsorbía": "adsorber",
    "adsorbíais": "adsorber",
    "adsorbíamos": "adsorber",
    "adsorbían": "adsorber",
    "adsorbías": "adsorber",
    "adsorciones": "adsorción",
    "adstratos": "adstrato",
    "adstringe": "adstringir",
    "adstringen": "adstringir",
    "adstringes": "adstringir",
    "adstringid": "adstringir",
    "adstringida": "adstringir",
    "adstringidas": "adstringir",
    "adstringido": "adstringir",
    "adstringidos": "adstringir",
    "adstringiendo": "adstringir",
    "adstringiera": "adstringir",
    "adstringierais": "adstringir",
    "adstringieran": "adstringir",
    "adstringieras": "adstringir",
    "adstringiere": "adstringir",
    "adstringiereis": "adstringir",
    "adstringieren": "adstringir",
    "adstringieres": "adstringir",
    "adstringieron": "adstringir",
    "adstringiese": "adstringir",
    "adstringieseis": "adstringir",
    "adstringiesen": "adstringir",
    "adstringieses": "adstringir",
    "adstringimos": "adstringir",
    "adstringiremos": "adstringir",
    "adstringirá": "adstringir",
    "adstringirán": "adstringir",
    "adstringirás": "adstringir",
    "adstringiré": "adstringir",
    "adstringiréis": "adstringir",
    "adstringiría": "adstringir",
    "adstringiríais": "adstringir",
    "adstringiríamos": "adstringir",
    "adstringirían": "adstringir",
    "adstringirías": "adstringir",
    "adstringiste": "adstringir",
    "adstringisteis": "adstringir",
    "adstringiéramos": "adstringir",
    "adstringiéremos": "adstringir",
    "adstringiésemos": "adstringir",
    "adstringió": "adstringir",
    "adstringí": "adstringir",
    "adstringía": "adstringir",
    "adstringíais": "adstringir",
    "adstringíamos": "adstringir",
    "adstringían": "adstringir",
    "adstringías": "adstringir",
    "adstringís": "adstringir",
    "adstrinja": "adstringir",
    "adstrinjamos": "adstringir",
    "adstrinjan": "adstringir",
    "adstrinjas": "adstringir",
    "adstrinjo": "adstringir",
    "adstrinjáis": "adstringir",
    "aduana": "aduanar",
    "aduanaba": "aduanar",
    "aduanabais": "aduanar",
    "aduanaban": "aduanar",
    "aduanabas": "aduanar",
    "aduanad": "aduanar",
    "aduanada": "aduanar",
    "aduanadas": "aduanar",
    "aduanado": "aduanar",
    "aduanados": "aduanar",
    "aduanales": "aduanal",
    "aduanamos": "aduanar",
    "aduanan": "aduanar",
    "aduanando": "aduanar",
    "aduanara": "aduanar",
    "aduanarais": "aduanar",
    "aduanaran": "aduanar",
    "aduanaras": "aduanar",
    "aduanare": "aduanar",
    "aduanareis": "aduanar",
    "aduanaremos": "aduanar",
    "aduanaren": "aduanar",
    "aduanares": "aduanar",
    "aduanaron": "aduanar",
    "aduanará": "aduanar",
    "aduanarán": "aduanar",
    "aduanarás": "aduanar",
    "aduanaré": "aduanar",
    "aduanaréis": "aduanar",
    "aduanaría": "aduanar",
    "aduanaríais": "aduanar",
    "aduanaríamos": "aduanar",
    "aduanarían": "aduanar",
    "aduanarías": "aduanar",
    "aduanas": "aduanar",
    "aduanase": "aduanar",
    "aduanaseis": "aduanar",
    "aduanasen": "aduanar",
    "aduanases": "aduanar",
    "aduanaste": "aduanar",
    "aduanasteis": "aduanar",
    "aduane": "aduanar",
    "aduanemos": "aduanar",
    "aduanen": "aduanar",
    "aduanera": "aduanero",
    "aduaneras": "aduanero",
    "aduaneros": "aduanero",
    "aduanes": "aduanar",
    "aduano": "aduanar",
    "aduanábamos": "aduanar",
    "aduanáis": "aduanar",
    "aduanáramos": "aduanar",
    "aduanáremos": "aduanar",
    "aduanásemos": "aduanar",
    "aduané": "aduanar",
    "aduanéis": "aduanar",
    "aduanó": "aduanar",
    "aduares": "aduar",
    "aducciones": "aducción",
    "aduce": "aducir",
    "aducen": "aducir",
    "aduces": "aducir",
    "aducid": "aducir",
    "aducida": "aducir",
    "aducidas": "aducir",
    "aducido": "aducir",
    "aducidos": "aducir",
    "aduciendo": "aducir",
    "aducimos": "aducir",
    "aduciremos": "aducir",
    "aducirá": "aducir",
    "aducirán": "aducir",
    "aducirás": "aducir",
    "aduciré": "aducir",
    "aduciréis": "aducir",
    "aduciría": "aducir",
    "aduciríais": "aducir",
    "aduciríamos": "aducir",
    "aducirían": "aducir",
    "aducirías": "aducir",
    "aductora": "aductor",
    "aductoras": "aductor",
    "aductores": "aductor",
    "aductos": "aducto",
    "aducía": "aducir",
    "aducíais": "aducir",
    "aducíamos": "aducir",
    "aducían": "aducir",
    "aducías": "aducir",
    "aducís": "aducir",
    "adueña": "adueñar",
    "adueñaba": "adueñar",
    "adueñabais": "adueñar",
    "adueñaban": "adueñar",
    "adueñabas": "adueñar",
    "adueñad": "adueñar",
    "adueñada": "adueñar",
    "adueñadas": "adueñar",
    "adueñado": "adueñar",
    "adueñados": "adueñar",
    "adueñamos": "adueñar",
    "adueñan": "adueñar",
    "adueñando": "adueñar",
    "adueñara": "adueñar",
    "adueñarais": "adueñar",
    "adueñaran": "adueñar",
    "adueñaras": "adueñar",
    "adueñare": "adueñar",
    "adueñareis": "adueñar",
    "adueñaremos": "adueñar",
    "adueñaren": "adueñar",
    "adueñares": "adueñar",
    "adueñaron": "adueñar",
    "adueñará": "adueñar",
    "adueñarán": "adueñar",
    "adueñarás": "adueñar",
    "adueñaré": "adueñar",
    "adueñaréis": "adueñar",
    "adueñaría": "adueñar",
    "adueñaríais": "adueñar",
    "adueñaríamos": "adueñar",
    "adueñarían": "adueñar",
    "adueñarías": "adueñar",
    "adueñas": "adueñar",
    "adueñase": "adueñar",
    "adueñaseis": "adueñar",
    "adueñasen": "adueñar",
    "adueñases": "adueñar",
    "adueñaste": "adueñar",
    "adueñasteis": "adueñar",
    "adueñe": "adueñar",
    "adueñemos": "adueñar",
    "adueñen": "adueñar",
    "adueñes": "adueñar",
    "adueño": "adueñar",
    "adueñábamos": "adueñar",
    "adueñáis": "adueñar",
    "adueñáramos": "adueñar",
    "adueñáremos": "adueñar",
    "adueñásemos": "adueñar",
    "adueñé": "adueñar",
    "adueñéis": "adueñar",
    "adueñó": "adueñar",
    "adufera": "adufero",
    "aduferas": "adufero",
    "aduferos": "adufero",
    "adufes": "adufe",
    "adufres": "adufre",
    "aduja": "adujar",
    "adujaba": "adujar",
    "adujabais": "adujar",
    "adujaban": "adujar",
    "adujabas": "adujar",
    "adujad": "adujar",
    "adujada": "adujar",
    "adujadas": "adujar",
    "adujado": "adujar",
    "adujados": "adujar",
    "adujamos": "adujar",
    "adujan": "adujar",
    "adujando": "adujar",
    "adujara": "adujar",
    "adujarais": "adujar",
    "adujaran": "adujar",
    "adujaras": "adujar",
    "adujare": "adujar",
    "adujareis": "adujar",
    "adujaremos": "adujar",
    "adujaren": "adujar",
    "adujares": "adujar",
    "adujaron": "adujar",
    "adujará": "adujar",
    "adujarán": "adujar",
    "adujarás": "adujar",
    "adujaré": "adujar",
    "adujaréis": "adujar",
    "adujaría": "adujar",
    "adujaríais": "adujar",
    "adujaríamos": "adujar",
    "adujarían": "adujar",
    "adujarías": "adujar",
    "adujas": "adujar",
    "adujase": "adujar",
    "adujaseis": "adujar",
    "adujasen": "adujar",
    "adujases": "adujar",
    "adujaste": "adujar",
    "adujasteis": "adujar",
    "aduje": "adujar",
    "adujemos": "adujar",
    "adujen": "adujar",
    "adujera": "aducir",
    "adujerais": "aducir",
    "adujeran": "aducir",
    "adujeras": "aducir",
    "adujere": "aducir",
    "adujereis": "aducir",
    "adujeren": "aducir",
    "adujeres": "aducir",
    "adujeron": "aducir",
    "adujes": "adujar",
    "adujese": "aducir",
    "adujeseis": "aducir",
    "adujesen": "aducir",
    "adujeses": "aducir",
    "adujimos": "aducir",
    "adujiste": "aducir",
    "adujisteis": "aducir",
    "adujo": "adujar",
    "adujábamos": "adujar",
    "adujáis": "adujar",
    "adujáramos": "adujar",
    "adujáremos": "adujar",
    "adujásemos": "adujar",
    "adujé": "adujar",
    "adujéis": "adujar",
    "adujéramos": "aducir",
    "adujéremos": "aducir",
    "adujésemos": "aducir",
    "adujó": "adujar",
    "adula": "adular",
    "adulaba": "adular",
    "adulabais": "adular",
    "adulaban": "adular",
    "adulabas": "adular",
    "adulaciones": "adulación",
    "adulad": "adular",
    "adulada": "adular",
    "aduladas": "adular",
    "adulado": "adular",
    "aduladora": "adulador",
    "aduladoras": "adulador",
    "aduladores": "adulador",
    "adulados": "adular",
    "adulamos": "adular",
    "adulan": "adular",
    "adulando": "adular",
    "adulara": "adular",
    "adularais": "adular",
    "adularan": "adular",
    "adularas": "adular",
    "adulare": "adular",
    "adulareis": "adular",
    "adularemos": "adular",
    "adularen": "adular",
    "adulares": "adular",
    "adularescencias": "adularescencia",
    "adularias": "adularia",
    "adularon": "adular",
    "adulará": "adular",
    "adularán": "adular",
    "adularás": "adular",
    "adularé": "adular",
    "adularéis": "adular",
    "adularía": "adular",
    "adularíais": "adular",
    "adularíamos": "adular",
    "adularían": "adular",
    "adularías": "adular",
    "adulas": "adular",
    "adulase": "adular",
    "adulaseis": "adular",
    "adulasen": "adular",
    "adulases": "adular",
    "adulaste": "adular",
    "adulasteis": "adular",
    "adulatoria": "adulatorio",
    "adulatorias": "adulatorio",
    "adulatorios": "adulatorio",
    "adulce": "adulzar",
    "adulcemos": "adulzar",
    "adulcen": "adulzar",
    "adulces": "adulzar",
    "adulcé": "adulzar",
    "adulcéis": "adulzar",
    "adule": "adular",
    "adulemos": "adular",
    "adulen": "adular",
    "adules": "adular",
    "adulo": "adular",
    "adulona": "adulón",
    "adulonas": "adulón",
    "adulones": "adulón",
    "adulta": "adulto",
    "adultas": "adulto",
    "adulteces": "adultez",
    "adultera": "adulterar",
    "adulteraba": "adulterar",
    "adulterabais": "adulterar",
    "adulteraban": "adulterar",
    "adulterabas": "adulterar",
    "adulterables": "adulterable",
    "adulteraciones": "adulteración",
    "adulterad": "adulterar",
    "adulterada": "adulterar",
    "adulteradas": "adulterar",
    "adulterado": "adulterar",
    "adulteradora": "adulterador",
    "adulteradoras": "adulterador",
    "adulteradores": "adulterador",
    "adulterados": "adulterar",
    "adulteramos": "adulterar",
    "adulteran": "adulterar",
    "adulterando": "adulterar",
    "adulterara": "adulterar",
    "adulterarais": "adulterar",
    "adulteraran": "adulterar",
    "adulteraras": "adulterar",
    "adulterare": "adulterar",
    "adulterareis": "adulterar",
    "adulteraremos": "adulterar",
    "adulteraren": "adulterar",
    "adulterares": "adulterar",
    "adulteraron": "adulterar",
    "adulterará": "adulterar",
    "adulterarán": "adulterar",
    "adulterarás": "adulterar",
    "adulteraré": "adulterar",
    "adulteraréis": "adulterar",
    "adulteraría": "adulterar",
    "adulteraríais": "adulterar",
    "adulteraríamos": "adulterar",
    "adulterarían": "adulterar",
    "adulterarías": "adulterar",
    "adulteras": "adulterar",
    "adulterase": "adulterar",
    "adulteraseis": "adulterar",
    "adulterasen": "adulterar",
    "adulterases": "adulterar",
    "adulteraste": "adulterar",
    "adulterasteis": "adulterar",
    "adultere": "adulterar",
    "adulteremos": "adulterar",
    "adulteren": "adulterar",
    "adulteres": "adulterar",
    "adulterina": "adulterino",
    "adulterinas": "adulterino",
    "adulterinos": "adulterino",
    "adulterios": "adulterio",
    "adultero": "adulterar",
    "adulterábamos": "adulterar",
    "adulteráis": "adulterar",
    "adulteráramos": "adulterar",
    "adulteráremos": "adulterar",
    "adulterásemos": "adulterar",
    "adulteré": "adulterar",
    "adulteréis": "adulterar",
    "adulteró": "adulterar",
    "adultos": "adulto",
    "adulza": "adulzar",
    "adulzaba": "adulzar",
    "adulzabais": "adulzar",
    "adulzaban": "adulzar",
    "adulzabas": "adulzar",
    "adulzad": "adulzar",
    "adulzada": "adulzar",
    "adulzadas": "adulzar",
    "adulzado": "adulzar",
    "adulzados": "adulzar",
    "adulzamos": "adulzar",
    "adulzan": "adulzar",
    "adulzando": "adulzar",
    "adulzara": "adulzar",
    "adulzarais": "adulzar",
    "adulzaran": "adulzar",
    "adulzaras": "adulzar",
    "adulzare": "adulzar",
    "adulzareis": "adulzar",
    "adulzaremos": "adulzar",
    "adulzaren": "adulzar",
    "adulzares": "adulzar",
    "adulzaron": "adulzar",
    "adulzará": "adulzar",
    "adulzarán": "adulzar",
    "adulzarás": "adulzar",
    "adulzaré": "adulzar",
    "adulzaréis": "adulzar",
    "adulzaría": "adulzar",
    "adulzaríais": "adulzar",
    "adulzaríamos": "adulzar",
    "adulzarían": "adulzar",
    "adulzarías": "adulzar",
    "adulzas": "adulzar",
    "adulzase": "adulzar",
    "adulzaseis": "adulzar",
    "adulzasen": "adulzar",
    "adulzases": "adulzar",
    "adulzaste": "adulzar",
    "adulzasteis": "adulzar",
    "adulzo": "adulzar",
    "adulzora": "adulzorar",
    "adulzoraba": "adulzorar",
    "adulzorabais": "adulzorar",
    "adulzoraban": "adulzorar",
    "adulzorabas": "adulzorar",
    "adulzorad": "adulzorar",
    "adulzorada": "adulzorar",
    "adulzoradas": "adulzorar",
    "adulzorado": "adulzorar",
    "adulzorados": "adulzorar",
    "adulzoramos": "adulzorar",
    "adulzoran": "adulzorar",
    "adulzorando": "adulzorar",
    "adulzorara": "adulzorar",
    "adulzorarais": "adulzorar",
    "adulzoraran": "adulzorar",
    "adulzoraras": "adulzorar",
    "adulzorare": "adulzorar",
    "adulzorareis": "adulzorar",
    "adulzoraremos": "adulzorar",
    "adulzoraren": "adulzorar",
    "adulzorares": "adulzorar",
    "adulzoraron": "adulzorar",
    "adulzorará": "adulzorar",
    "adulzorarán": "adulzorar",
    "adulzorarás": "adulzorar",
    "adulzoraré": "adulzorar",
    "adulzoraréis": "adulzorar",
    "adulzoraría": "adulzorar",
    "adulzoraríais": "adulzorar",
    "adulzoraríamos": "adulzorar",
    "adulzorarían": "adulzorar",
    "adulzorarías": "adulzorar",
    "adulzoras": "adulzorar",
    "adulzorase": "adulzorar",
    "adulzoraseis": "adulzorar",
    "adulzorasen": "adulzorar",
    "adulzorases": "adulzorar",
    "adulzoraste": "adulzorar",
    "adulzorasteis": "adulzorar",
    "adulzore": "adulzorar",
    "adulzoremos": "adulzorar",
    "adulzoren": "adulzorar",
    "adulzores": "adulzorar",
    "adulzoro": "adulzorar",
    "adulzorábamos": "adulzorar",
    "adulzoráis": "adulzorar",
    "adulzoráramos": "adulzorar",
    "adulzoráremos": "adulzorar",
    "adulzorásemos": "adulzorar",
    "adulzoré": "adulzorar",
    "adulzoréis": "adulzorar",
    "adulzoró": "adulzorar",
    "adulzábamos": "adulzar",
    "adulzáis": "adulzar",
    "adulzáramos": "adulzar",
    "adulzáremos": "adulzar",
    "adulzásemos": "adulzar",
    "adulzó": "adulzar",
    "adulábamos": "adular",
    "aduláis": "adular",
    "aduláramos": "adular",
    "aduláremos": "adular",
    "adulásemos": "adular",
    "adulé": "adular",
    "aduléis": "adular",
    "aduló": "adular",
    "adumbraciones": "adumbración",
    "aduna": "adunar",
    "adunaba": "adunar",
    "adunabais": "adunar",
    "adunaban": "adunar",
    "adunabas": "adunar",
    "adunad": "adunar",
    "adunada": "adunar",
    "adunadas": "adunar",
    "adunado": "adunar",
    "adunados": "adunar",
    "adunamos": "adunar",
    "adunan": "adunar",
    "adunando": "adunar",
    "adunara": "adunar",
    "adunarais": "adunar",
    "adunaran": "adunar",
    "adunaras": "adunar",
    "adunare": "adunar",
    "adunareis": "adunar",
    "adunaremos": "adunar",
    "adunaren": "adunar",
    "adunares": "adunar",
    "adunaron": "adunar",
    "adunará": "adunar",
    "adunarán": "adunar",
    "adunarás": "adunar",
    "adunaré": "adunar",
    "adunaréis": "adunar",
    "adunaría": "adunar",
    "adunaríais": "adunar",
    "adunaríamos": "adunar",
    "adunarían": "adunar",
    "adunarías": "adunar",
    "adunas": "adunar",
    "adunase": "adunar",
    "adunaseis": "adunar",
    "adunasen": "adunar",
    "adunases": "adunar",
    "adunaste": "adunar",
    "adunasteis": "adunar",
    "adune": "adunar",
    "adunemos": "adunar",
    "adunen": "adunar",
    "adunes": "adunar",
    "aduno": "adunar",
    "adunábamos": "adunar",
    "adunáis": "adunar",
    "adunáramos": "adunar",
    "adunáremos": "adunar",
    "adunásemos": "adunar",
    "aduné": "adunar",
    "adunéis": "adunar",
    "adunó": "adunar",
    "adusta": "adusto",
    "adustas": "adusto",
    "adusteces": "adustez",
    "adustos": "adusto",
    "aduzca": "aducir",
    "aduzcamos": "aducir",
    "aduzcan": "aducir",
    "aduzcas": "aducir",
    "aduzco": "aducir",
    "aduzcáis": "aducir",
    "advaitas": "advaita",
    "advecciones": "advección",
    "adven": "advenir",
    "advendremos": "advenir",
    "advendrá": "advenir",
    "advendrán": "advenir",
    "advendrás": "advenir",
    "advendré": "advenir",
    "advendréis": "advenir",
    "advendría": "advenir",
    "advendríais": "advenir",
    "advendríamos": "advenir",
    "advendrían": "advenir",
    "advendrías": "advenir",
    "advenediza": "advenedizo",
    "advenedizas": "advenedizo",
    "advenedizos": "advenedizo",
    "advenga": "advenir",
    "advengamos": "advenir",
    "advengan": "advenir",
    "advengas": "advenir",
    "advengo": "advenir",
    "advengáis": "advenir",
    "advenid": "advenir",
    "advenida": "advenir",
    "advenidas": "advenir",
    "advenido": "advenir",
    "advenidos": "advenir",
    "advenimientos": "advenimiento",
    "advenimos": "advenir",
    "adventicia": "adventicio",
    "adventicias": "adventicio",
    "adventicios": "adventicio",
    "adventismos": "adventismo",
    "adventistas": "adventista",
    "advenía": "advenir",
    "adveníais": "advenir",
    "adveníamos": "advenir",
    "advenían": "advenir",
    "advenías": "advenir",
    "advenís": "advenir",
    "advera": "adverar",
    "adveraba": "adverar",
    "adverabais": "adverar",
    "adveraban": "adverar",
    "adverabas": "adverar",
    "adveraciones": "adveración",
    "adverad": "adverar",
    "adverada": "adverar",
    "adveradas": "adverar",
    "adverado": "adverar",
    "adveradora": "adverador",
    "adveradoras": "adverador",
    "adveradores": "adverador",
    "adverados": "adverar",
    "adveramos": "adverar",
    "adveran": "adverar",
    "adverando": "adverar",
    "adverara": "adverar",
    "adverarais": "adverar",
    "adveraran": "adverar",
    "adveraras": "adverar",
    "adverare": "adverar",
    "adverareis": "adverar",
    "adveraremos": "adverar",
    "adveraren": "adverar",
    "adverares": "adverar",
    "adveraron": "adverar",
    "adverará": "adverar",
    "adverarán": "adverar",
    "adverarás": "adverar",
    "adveraré": "adverar",
    "adveraréis": "adverar",
    "adveraría": "adverar",
    "adveraríais": "adverar",
    "adveraríamos": "adverar",
    "adverarían": "adverar",
    "adverarías": "adverar",
    "adveras": "adverar",
    "adverase": "adverar",
    "adveraseis": "adverar",
    "adverasen": "adverar",
    "adverases": "adverar",
    "adveraste": "adverar",
    "adverasteis": "adverar",
    "adverbiales": "adverbial",
    "adverbialice": "adverbializar",
    "adverbialicemos": "adverbializar",
    "adverbialicen": "adverbializar",
    "adverbialices": "adverbializar",
    "adverbialicé": "adverbializar",
    "adverbialicéis": "adverbializar",
    "adverbializa": "adverbializar",
    "adverbializaba": "adverbializar",
    "adverbializabais": "adverbializar",
    "adverbializaban": "adverbializar",
    "adverbializabas": "adverbializar",
    "adverbializaciones": "adverbialización",
    "adverbializad": "adverbializar",
    "adverbializada": "adverbializar",
    "adverbializadas": "adverbializar",
    "adverbializado": "adverbializar",
    "adverbializadora": "adverbializador",
    "adverbializadoras": "adverbializador",
    "adverbializadores": "adverbializador",
    "adverbializados": "adverbializar",
    "adverbializamos": "adverbializar",
    "adverbializan": "adverbializar",
    "adverbializando": "adverbializar",
    "adverbializara": "adverbializar",
    "adverbializarais": "adverbializar",
    "adverbializaran": "adverbializar",
    "adverbializaras": "adverbializar",
    "adverbializare": "adverbializar",
    "adverbializareis": "adverbializar",
    "adverbializaremos": "adverbializar",
    "adverbializaren": "adverbializar",
    "adverbializares": "adverbializar",
    "adverbializaron": "adverbializar",
    "adverbializará": "adverbializar",
    "adverbializarán": "adverbializar",
    "adverbializarás": "adverbializar",
    "adverbializaré": "adverbializar",
    "adverbializaréis": "adverbializar",
    "adverbializaría": "adverbializar",
    "adverbializaríais": "adverbializar",
    "adverbializaríamos": "adverbializar",
    "adverbializarían": "adverbializar",
    "adverbializarías": "adverbializar",
    "adverbializas": "adverbializar",
    "adverbializase": "adverbializar",
    "adverbializaseis": "adverbializar",
    "adverbializasen": "adverbializar",
    "adverbializases": "adverbializar",
    "adverbializaste": "adverbializar",
    "adverbializasteis": "adverbializar",
    "adverbializo": "adverbializar",
    "adverbializábamos": "adverbializar",
    "adverbializáis": "adverbializar",
    "adverbializáramos": "adverbializar",
    "adverbializáremos": "adverbializar",
    "adverbializásemos": "adverbializar",
    "adverbializó": "adverbializar",
    "adverbios": "adverbio",
    "advere": "adverar",
    "adveremos": "adverar",
    "adveren": "adverar",
    "adveres": "adverar",
    "advero": "adverar",
    "adversa": "adverso",
    "adversaria": "adversario",
    "adversarias": "adversario",
    "adversarios": "adversario",
    "adversas": "adverso",
    "adversativa": "adversativo",
    "adversativas": "adversativo",
    "adversativos": "adversativo",
    "adversidades": "adversidad",
    "adversos": "adverso",
    "advertencias": "advertencia",
    "advertid": "advertir",
    "advertida": "advertir",
    "advertidas": "advertir",
    "advertido": "advertir",
    "advertidos": "advertir",
    "advertimientos": "advertimiento",
    "advertimos": "advertir",
    "advertiremos": "advertir",
    "advertirá": "advertir",
    "advertirán": "advertir",
    "advertirás": "advertir",
    "advertiré": "advertir",
    "advertiréis": "advertir",
    "advertiría": "advertir",
    "advertiríais": "advertir",
    "advertiríamos": "advertir",
    "advertirían": "advertir",
    "advertirías": "advertir",
    "advertiste": "advertir",
    "advertisteis": "advertir",
    "advertí": "advertir",
    "advertía": "advertir",
    "advertíais": "advertir",
    "advertíamos": "advertir",
    "advertían": "advertir",
    "advertías": "advertir",
    "advertís": "advertir",
    "adverábamos": "adverar",
    "adveráis": "adverar",
    "adveráramos": "adverar",
    "adveráremos": "adverar",
    "adverásemos": "adverar",
    "adveré": "adverar",
    "adveréis": "adverar",
    "adveró": "adverar",
    "adviene": "advenir",
    "advienen": "advenir",
    "advienes": "advenir",
    "advientos": "adviento",
    "advierta": "advertir",
    "adviertan": "advertir",
    "adviertas": "advertir",
    "advierte": "advertir",
    "advierten": "advertir",
    "adviertes": "advertir",
    "advierto": "advertir",
    "advine": "advenir",
    "adviniendo": "advenir",
    "adviniera": "advenir",
    "advinierais": "advenir",
    "advinieran": "advenir",
    "advinieras": "advenir",
    "adviniere": "advenir",
    "adviniereis": "advenir",
    "advinieren": "advenir",
    "advinieres": "advenir",
    "advinieron": "advenir",
    "adviniese": "advenir",
    "advinieseis": "advenir",
    "adviniesen": "advenir",
    "advinieses": "advenir",
    "advinimos": "advenir",
    "adviniste": "advenir",
    "advinisteis": "advenir",
    "adviniéramos": "advenir",
    "adviniéremos": "advenir",
    "adviniésemos": "advenir",
    "advino": "advenir",
    "advirtamos": "advertir",
    "advirtiendo": "advertir",
    "advirtiera": "advertir",
    "advirtierais": "advertir",
    "advirtieran": "advertir",
    "advirtieras": "advertir",
    "advirtiere": "advertir",
    "advirtiereis": "advertir",
    "advirtieren": "advertir",
    "advirtieres": "advertir",
    "advirtieron": "advertir",
    "advirtiese": "advertir",
    "advirtieseis": "advertir",
    "advirtiesen": "advertir",
    "advirtieses": "advertir",
    "advirtiéramos": "advertir",
    "advirtiéremos": "advertir",
    "advirtiésemos": "advertir",
    "advirtió": "advertir",
    "advirtáis": "advertir",
    "advocaciones": "advocación",
    "adyacencias": "adyacencia",
    "adyacentes": "adyacente",
    "adyuvantes": "adyuvante",
    "adáis": "adir",
    "adí": "adir",
    "adía": "adir",
    "adíais": "adir",
    "adíamos": "adir",
    "adían": "adir",
    "adías": "adir",
    "adípica": "adípico",
    "adípicas": "adípico",
    "adípicos": "adípico",
    "adís": "adir",
    "adónica": "adónico",
    "adónicas": "adónico",
    "adónicos": "adónico",
    "adúcares": "adúcar",
    "adúltera": "adúltero",
    "adúlteras": "adúltero",
    "adúlteros": "adúltero",
    "aedos": "aedo",
    "aeraciones": "aeración",
    "aeriformes": "aeriforme",
    "aerobia": "aerobio",
    "aerobias": "aerobio",
    "aerobiologías": "aerobiología",
    "aerobios": "aerobio",
    "aerobioscopios": "aerobioscopio",
    "aerobuses": "aerobús",
    "aerocartógrafa": "aerocartógrafo",
    "aerocartógrafas": "aerocartógrafo",
    "aerocartógrafos": "aerocartógrafo",
    "aeroclasificadores": "aeroclasificador",
    "aeroclubes": "aeroclub",
    "aeroclubs": "aeroclub",
    "aerocondensadores": "aerocondensador",
    "aerodeslizadores": "aerodeslizador",
    "aerodinamómetros": "aerodinamómetro",
    "aerodinos": "aerodino",
    "aerodinámica": "aerodinámico",
    "aerodinámicas": "aerodinámico",
    "aerodinámicos": "aerodinámico",
    "aerodoncias": "aerodoncia",
    "aerodontalgias": "aerodontalgia",
    "aeroelasticidades": "aeroelasticidad",
    "aeroespaciales": "aeroespacial",
    "aeroestaciones": "aeroestación",
    "aerofagias": "aerofagia",
    "aerofaros": "aerofaro",
    "aeroflotaciones": "aeroflotación",
    "aerofotografías": "aerofotografía",
    "aerofotográfica": "aerofotográfico",
    "aerofotográficas": "aerofotográfico",
    "aerofotográficos": "aerofotográfico",
    "aerogeneradores": "aerogenerador",
    "aerografías": "aerografía",
    "aerogramas": "aerograma",
    "aerohidráulica": "aerohidráulico",
    "aerohidráulicas": "aerohidráulico",
    "aerohidráulicos": "aerohidráulico",
    "aerolitos": "aerolito",
    "aerologías": "aerología",
    "aerolíneas": "aerolínea",
    "aerolítica": "aerolítico",
    "aerolíticas": "aerolítico",
    "aerolíticos": "aerolítico",
    "aerológica": "aerológico",
    "aerológicas": "aerológico",
    "aerológicos": "aerológico",
    "aeromancias": "aeromancia",
    "aerometrías": "aerometría",
    "aeromodelismos": "aeromodelismo",
    "aeromodelistas": "aeromodelista",
    "aeromodelos": "aeromodelo",
    "aeromotores": "aeromotor",
    "aeronautas": "aeronauta",
    "aeronavales": "aeronaval",
    "aeronavegabilidades": "aeronavegabilidad",
    "aeronavegables": "aeronavegable",
    "aeronaves": "aeronave",
    "aeronomías": "aeronomía",
    "aeronáutica": "aeronáutico",
    "aeronáuticas": "aeronáutico",
    "aeronáuticos": "aeronáutico",
    "aeroplanos": "aeroplano",
    "aeroportuaria": "aeroportuario",
    "aeroportuarias": "aeroportuario",
    "aeroportuarios": "aeroportuario",
    "aeropostales": "aeropostal",
    "aeropuertos": "aeropuerto",
    "aeroscopios": "aeroscopio",
    "aerosoles": "aerosol",
    "aerostaciones": "aerostación",
    "aerostática": "aerostático",
    "aerostáticas": "aerostático",
    "aerostáticos": "aerostático",
    "aerotaxis": "aerotaxi",
    "aeroterapias": "aeroterapia",
    "aerotermodinámicas": "aerotermodinámica",
    "aerotermos": "aerotermo",
    "aeroterrestres": "aeroterrestre",
    "aerotimpánica": "aerotimpánico",
    "aerotimpánicas": "aerotimpánico",
    "aerotimpánicos": "aerotimpánico",
    "aerotrenes": "aerotrén",
    "aerotriangulaciones": "aerotriangulación",
    "aerovías": "aerovía",
    "aerícolas": "aerícola",
    "aerífera": "aerífero",
    "aeríferas": "aerífero",
    "aeríferos": "aerífero",
    "aeróbica": "aeróbico",
    "aeróbicas": "aeróbico",
    "aeróbicos": "aeróbico",
    "aeróbics": "aeróbic",
    "aeródromos": "aeródromo",
    "aerógena": "aerógeno",
    "aerógenas": "aerógeno",
    "aerógenos": "aerógeno",
    "aerógrafos": "aerógrafo",
    "aerómetros": "aerómetro",
    "aeróscopos": "aeróscopo",
    "aeróstatos": "aeróstato",
    "aes": "a",
    "aetas": "aeta",
    "afabilidades": "afabilidad",
    "afables": "afable",
    "afacetada": "afacetado",
    "afacetadas": "afacetado",
    "afacetados": "afacetado",
    "afama": "afamar",
    "afamaba": "afamar",
    "afamabais": "afamar",
    "afamaban": "afamar",
    "afamabas": "afamar",
    "afamad": "afamar",
    "afamada": "afamar",
    "afamadas": "afamar",
    "afamado": "afamar",
    "afamados": "afamar",
    "afamamos": "afamar",
    "afaman": "afamar",
    "afamando": "afamar",
    "afamara": "afamar",
    "afamarais": "afamar",
    "afamaran": "afamar",
    "afamaras": "afamar",
    "afamare": "afamar",
    "afamareis": "afamar",
    "afamaremos": "afamar",
    "afamaren": "afamar",
    "afamares": "afamar",
    "afamaron": "afamar",
    "afamará": "afamar",
    "afamarán": "afamar",
    "afamarás": "afamar",
    "afamaré": "afamar",
    "afamaréis": "afamar",
    "afamaría": "afamar",
    "afamaríais": "afamar",
    "afamaríamos": "afamar",
    "afamarían": "afamar",
    "afamarías": "afamar",
    "afamas": "afamar",
    "afamase": "afamar",
    "afamaseis": "afamar",
    "afamasen": "afamar",
    "afamases": "afamar",
    "afamaste": "afamar",
    "afamasteis": "afamar",
    "afame": "afamar",
    "afamemos": "afamar",
    "afamen": "afamar",
    "afames": "afamar",
    "afamo": "afamar",
    "afamábamos": "afamar",
    "afamáis": "afamar",
    "afamáramos": "afamar",
    "afamáremos": "afamar",
    "afamásemos": "afamar",
    "afamé": "afamar",
    "afaméis": "afamar",
    "afamó": "afamar",
    "afana": "afanar",
    "afanaba": "afanar",
    "afanabais": "afanar",
    "afanaban": "afanar",
    "afanabas": "afanar",
    "afanad": "afanar",
    "afanada": "afanar",
    "afanadas": "afanar",
    "afanado": "afanar",
    "afanados": "afanar",
    "afanamientos": "afanamiento",
    "afanamos": "afanar",
    "afanan": "afanar",
    "afanando": "afanar",
    "afanara": "afanar",
    "afanarais": "afanar",
    "afanaran": "afanar",
    "afanaras": "afanar",
    "afanare": "afanar",
    "afanareis": "afanar",
    "afanaremos": "afanar",
    "afanaren": "afanar",
    "afanares": "afanar",
    "afanaron": "afanar",
    "afanará": "afanar",
    "afanarán": "afanar",
    "afanarás": "afanar",
    "afanaré": "afanar",
    "afanaréis": "afanar",
    "afanaría": "afanar",
    "afanaríais": "afanar",
    "afanaríamos": "afanar",
    "afanarían": "afanar",
    "afanarías": "afanar",
    "afanas": "afanar",
    "afanase": "afanar",
    "afanaseis": "afanar",
    "afanasen": "afanar",
    "afanases": "afanar",
    "afanaste": "afanar",
    "afanasteis": "afanar",
    "afane": "afanar",
    "afanemos": "afanar",
    "afanen": "afanar",
    "afanes": "afanar",
    "afano": "afanar",
    "afanosa": "afanoso",
    "afanosas": "afanoso",
    "afanosos": "afanoso",
    "afanábamos": "afanar",
    "afanáis": "afanar",
    "afanáramos": "afanar",
    "afanáremos": "afanar",
    "afanásemos": "afanar",
    "afané": "afanar",
    "afanéis": "afanar",
    "afanípteros": "afaníptero",
    "afanó": "afanar",
    "afaquias": "afaquia",
    "afarolada": "afarolado",
    "afaroladas": "afarolado",
    "afarolados": "afarolado",
    "afasias": "afasia",
    "afea": "afear",
    "afeaba": "afear",
    "afeabais": "afear",
    "afeaban": "afear",
    "afeabas": "afear",
    "afead": "afear",
    "afeada": "afear",
    "afeadas": "afear",
    "afeado": "afear",
    "afeados": "afear",
    "afeamientos": "afeamiento",
    "afeamos": "afear",
    "afean": "afear",
    "afeando": "afear",
    "afeara": "afear",
    "afearais": "afear",
    "afearan": "afear",
    "afearas": "afear",
    "afeare": "afear",
    "afeareis": "afear",
    "afearemos": "afear",
    "afearen": "afear",
    "afeares": "afear",
    "afearon": "afear",
    "afeará": "afear",
    "afearán": "afear",
    "afearás": "afear",
    "afearé": "afear",
    "afearéis": "afear",
    "afearía": "afear",
    "afearíais": "afear",
    "afearíamos": "afear",
    "afearían": "afear",
    "afearías": "afear",
    "afeas": "afear",
    "afease": "afear",
    "afeaseis": "afear",
    "afeasen": "afear",
    "afeases": "afear",
    "afeaste": "afear",
    "afeasteis": "afear",
    "afecciones": "afección",
    "afecta": "afecto",
    "afectaba": "afectar",
    "afectabais": "afectar",
    "afectaban": "afectar",
    "afectabas": "afectar",
    "afectaciones": "afectación",
    "afectad": "afectar",
    "afectada": "afectar",
    "afectadas": "afectar",
    "afectado": "afectar",
    "afectados": "afectar",
    "afectamos": "afectar",
    "afectan": "afectar",
    "afectando": "afectar",
    "afectara": "afectar",
    "afectarais": "afectar",
    "afectaran": "afectar",
    "afectaras": "afectar",
    "afectare": "afectar",
    "afectareis": "afectar",
    "afectaremos": "afectar",
    "afectaren": "afectar",
    "afectares": "afectar",
    "afectaron": "afectar",
    "afectará": "afectar",
    "afectarán": "afectar",
    "afectarás": "afectar",
    "afectaré": "afectar",
    "afectaréis": "afectar",
    "afectaría": "afectar",
    "afectaríais": "afectar",
    "afectaríamos": "afectar",
    "afectarían": "afectar",
    "afectarías": "afectar",
    "afectas": "afecto",
    "afectase": "afectar",
    "afectaseis": "afectar",
    "afectasen": "afectar",
    "afectases": "afectar",
    "afectaste": "afectar",
    "afectasteis": "afectar",
    "afecte": "afectar",
    "afectemos": "afectar",
    "afecten": "afectar",
    "afectes": "afectar",
    "afectiva": "afectivo",
    "afectivas": "afectivo",
    "afectividades": "afectividad",
    "afectivos": "afectivo",
    "afecto": "afectar",
    "afectos": "afecto",
    "afectuosa": "afectuoso",
    "afectuosas": "afectuoso",
    "afectuosidades": "afectuosidad",
    "afectuosos": "afectuoso",
    "afectábamos": "afectar",
    "afectáis": "afectar",
    "afectáramos": "afectar",
    "afectáremos": "afectar",
    "afectásemos": "afectar",
    "afecté": "afectar",
    "afectéis": "afectar",
    "afectó": "afectar",
    "afee": "afear",
    "afeemos": "afear",
    "afeen": "afear",
    "afees": "afear",
    "afeita": "afeitar",
    "afeitaba": "afeitar",
    "afeitabais": "afeitar",
    "afeitaban": "afeitar",
    "afeitabas": "afeitar",
    "afeitad": "afeitar",
    "afeitada": "afeitar",
    "afeitadas": "afeitar",
    "afeitado": "afeitar",
    "afeitadoras": "afeitadora",
    "afeitados": "afeitar",
    "afeitamos": "afeitar",
    "afeitan": "afeitar",
    "afeitando": "afeitar",
    "afeitara": "afeitar",
    "afeitarais": "afeitar",
    "afeitaran": "afeitar",
    "afeitaras": "afeitar",
    "afeitare": "afeitar",
    "afeitareis": "afeitar",
    "afeitaremos": "afeitar",
    "afeitaren": "afeitar",
    "afeitares": "afeitar",
    "afeitaron": "afeitar",
    "afeitará": "afeitar",
    "afeitarán": "afeitar",
    "afeitarás": "afeitar",
    "afeitaré": "afeitar",
    "afeitaréis": "afeitar",
    "afeitaría": "afeitar",
    "afeitaríais": "afeitar",
    "afeitaríamos": "afeitar",
    "afeitarían": "afeitar",
    "afeitarías": "afeitar",
    "afeitas": "afeitar",
    "afeitase": "afeitar",
    "afeitaseis": "afeitar",
    "afeitasen": "afeitar",
    "afeitases": "afeitar",
    "afeitaste": "afeitar",
    "afeitasteis": "afeitar",
    "afeite": "afeitar",
    "afeitemos": "afeitar",
    "afeiten": "afeitar",
    "afeites": "afeitar",
    "afeito": "afeitar",
    "afeitábamos": "afeitar",
    "afeitáis": "afeitar",
    "afeitáramos": "afeitar",
    "afeitáremos": "afeitar",
    "afeitásemos": "afeitar",
    "afeité": "afeitar",
    "afeitéis": "afeitar",
    "afeitó": "afeitar",
    "afelios": "afelio",
    "afelpa": "afelpar",
    "afelpaba": "afelpar",
    "afelpabais": "afelpar",
    "afelpaban": "afelpar",
    "afelpabas": "afelpar",
    "afelpad": "afelpar",
    "afelpada": "afelpar",
    "afelpadas": "afelpar",
    "afelpado": "afelpar",
    "afelpados": "afelpar",
    "afelpamos": "afelpar",
    "afelpan": "afelpar",
    "afelpando": "afelpar",
    "afelpara": "afelpar",
    "afelparais": "afelpar",
    "afelparan": "afelpar",
    "afelparas": "afelpar",
    "afelpare": "afelpar",
    "afelpareis": "afelpar",
    "afelparemos": "afelpar",
    "afelparen": "afelpar",
    "afelpares": "afelpar",
    "afelparon": "afelpar",
    "afelpará": "afelpar",
    "afelparán": "afelpar",
    "afelparás": "afelpar",
    "afelparé": "afelpar",
    "afelparéis": "afelpar",
    "afelparía": "afelpar",
    "afelparíais": "afelpar",
    "afelparíamos": "afelpar",
    "afelparían": "afelpar",
    "afelparías": "afelpar",
    "afelpas": "afelpar",
    "afelpase": "afelpar",
    "afelpaseis": "afelpar",
    "afelpasen": "afelpar",
    "afelpases": "afelpar",
    "afelpaste": "afelpar",
    "afelpasteis": "afelpar",
    "afelpe": "afelpar",
    "afelpemos": "afelpar",
    "afelpen": "afelpar",
    "afelpes": "afelpar",
    "afelpo": "afelpar",
    "afelpábamos": "afelpar",
    "afelpáis": "afelpar",
    "afelpáramos": "afelpar",
    "afelpáremos": "afelpar",
    "afelpásemos": "afelpar",
    "afelpé": "afelpar",
    "afelpéis": "afelpar",
    "afelpó": "afelpar",
    "afemias": "afemia",
    "afemina": "afeminar",
    "afeminaba": "afeminar",
    "afeminabais": "afeminar",
    "afeminaban": "afeminar",
    "afeminabas": "afeminar",
    "afeminaciones": "afeminación",
    "afeminad": "afeminar",
    "afeminada": "afeminar",
    "afeminadas": "afeminar",
    "afeminado": "afeminar",
    "afeminados": "afeminar",
    "afeminamientos": "afeminamiento",
    "afeminamos": "afeminar",
    "afeminan": "afeminar",
    "afeminando": "afeminar",
    "afeminara": "afeminar",
    "afeminarais": "afeminar",
    "afeminaran": "afeminar",
    "afeminaras": "afeminar",
    "afeminare": "afeminar",
    "afeminareis": "afeminar",
    "afeminaremos": "afeminar",
    "afeminaren": "afeminar",
    "afeminares": "afeminar",
    "afeminaron": "afeminar",
    "afeminará": "afeminar",
    "afeminarán": "afeminar",
    "afeminarás": "afeminar",
    "afeminaré": "afeminar",
    "afeminaréis": "afeminar",
    "afeminaría": "afeminar",
    "afeminaríais": "afeminar",
    "afeminaríamos": "afeminar",
    "afeminarían": "afeminar",
    "afeminarías": "afeminar",
    "afeminas": "afeminar",
    "afeminase": "afeminar",
    "afeminaseis": "afeminar",
    "afeminasen": "afeminar",
    "afeminases": "afeminar",
    "afeminaste": "afeminar",
    "afeminasteis": "afeminar",
    "afemine": "afeminar",
    "afeminemos": "afeminar",
    "afeminen": "afeminar",
    "afemines": "afeminar",
    "afemino": "afeminar",
    "afeminábamos": "afeminar",
    "afemináis": "afeminar",
    "afemináramos": "afeminar",
    "afemináremos": "afeminar",
    "afeminásemos": "afeminar",
    "afeminé": "afeminar",
    "afeminéis": "afeminar",
    "afeminó": "afeminar",
    "afeo": "afear",
    "aferentes": "aferente",
    "aferra": "aferrar",
    "aferraba": "aferrar",
    "aferrabais": "aferrar",
    "aferraban": "aferrar",
    "aferrabas": "aferrar",
    "aferrad": "aferrar",
    "aferrada": "aferrar",
    "aferradas": "aferrar",
    "aferrado": "aferrar",
    "aferrados": "aferrar",
    "aferramientos": "aferramiento",
    "aferramos": "aferrar",
    "aferran": "aferrar",
    "aferrando": "aferrar",
    "aferrara": "aferrar",
    "aferrarais": "aferrar",
    "aferraran": "aferrar",
    "aferraras": "aferrar",
    "aferrare": "aferrar",
    "aferrareis": "aferrar",
    "aferraremos": "aferrar",
    "aferraren": "aferrar",
    "aferrares": "aferrar",
    "aferraron": "aferrar",
    "aferrará": "aferrar",
    "aferrarán": "aferrar",
    "aferrarás": "aferrar",
    "aferraré": "aferrar",
    "aferraréis": "aferrar",
    "aferraría": "aferrar",
    "aferraríais": "aferrar",
    "aferraríamos": "aferrar",
    "aferrarían": "aferrar",
    "aferrarías": "aferrar",
    "aferras": "aferrar",
    "aferrase": "aferrar",
    "aferraseis": "aferrar",
    "aferrasen": "aferrar",
    "aferrases": "aferrar",
    "aferraste": "aferrar",
    "aferrasteis": "aferrar",
    "aferre": "aferrar",
    "aferremos": "aferrar",
    "aferren": "aferrar",
    "aferres": "aferrar",
    "aferro": "aferrar",
    "aferrábamos": "aferrar",
    "aferráis": "aferrar",
    "aferráramos": "aferrar",
    "aferráremos": "aferrar",
    "aferrásemos": "aferrar",
    "aferré": "aferrar",
    "aferréis": "aferrar",
    "aferró": "aferrar",
    "afeábamos": "afear",
    "afeáis": "afear",
    "afeáramos": "afear",
    "afeáremos": "afear",
    "afeásemos": "afear",
    "afeé": "afear",
    "afeéis": "afear",
    "afeó": "afear",
    "affaires": "affaire",
    "afgana": "afgano",
    "afganas": "afgano",
    "afganis": "afgani",
    "afganos": "afgano",
    "afghana": "afghano",
    "afghanas": "afghano",
    "afghanis": "afghani",
    "afghanos": "afghano",
    "afiance": "afianzar",
    "afiancemos": "afianzar",
    "afiancen": "afianzar",
    "afiances": "afianzar",
    "afiancé": "afianzar",
    "afiancéis": "afianzar",
    "afianza": "afianzar",
    "afianzaba": "afianzar",
    "afianzabais": "afianzar",
    "afianzaban": "afianzar",
    "afianzabas": "afianzar",
    "afianzad": "afianzar",
    "afianzada": "afianzar",
    "afianzadas": "afianzar",
    "afianzado": "afianzar",
    "afianzadora": "afianzador",
    "afianzadoras": "afianzador",
    "afianzadores": "afianzador",
    "afianzados": "afianzar",
    "afianzamientos": "afianzamiento",
    "afianzamos": "afianzar",
    "afianzan": "afianzar",
    "afianzando": "afianzar",
    "afianzara": "afianzar",
    "afianzarais": "afianzar",
    "afianzaran": "afianzar",
    "afianzaras": "afianzar",
    "afianzare": "afianzar",
    "afianzareis": "afianzar",
    "afianzaremos": "afianzar",
    "afianzaren": "afianzar",
    "afianzares": "afianzar",
    "afianzaron": "afianzar",
    "afianzará": "afianzar",
    "afianzarán": "afianzar",
    "afianzarás": "afianzar",
    "afianzaré": "afianzar",
    "afianzaréis": "afianzar",
    "afianzaría": "afianzar",
    "afianzaríais": "afianzar",
    "afianzaríamos": "afianzar",
    "afianzarían": "afianzar",
    "afianzarías": "afianzar",
    "afianzas": "afianzar",
    "afianzase": "afianzar",
    "afianzaseis": "afianzar",
    "afianzasen": "afianzar",
    "afianzases": "afianzar",
    "afianzaste": "afianzar",
    "afianzasteis": "afianzar",
    "afianzo": "afianzar",
    "afianzábamos": "afianzar",
    "afianzáis": "afianzar",
    "afianzáramos": "afianzar",
    "afianzáremos": "afianzar",
    "afianzásemos": "afianzar",
    "afianzó": "afianzar",
    "afibrinogenemias": "afibrinogenemia",
    "afiches": "afiche",
    "aficiona": "aficionar",
    "aficionaba": "aficionar",
    "aficionabais": "aficionar",
    "aficionaban": "aficionar",
    "aficionabas": "aficionar",
    "aficionad": "aficionar",
    "aficionada": "aficionar",
    "aficionadas": "aficionar",
    "aficionado": "aficionar",
    "aficionados": "aficionar",
    "aficionamos": "aficionar",
    "aficionan": "aficionar",
    "aficionando": "aficionar",
    "aficionara": "aficionar",
    "aficionarais": "aficionar",
    "aficionaran": "aficionar",
    "aficionaras": "aficionar",
    "aficionare": "aficionar",
    "aficionareis": "aficionar",
    "aficionaremos": "aficionar",
    "aficionaren": "aficionar",
    "aficionares": "aficionar",
    "aficionaron": "aficionar",
    "aficionará": "aficionar",
    "aficionarán": "aficionar",
    "aficionarás": "aficionar",
    "aficionaré": "aficionar",
    "aficionaréis": "aficionar",
    "aficionaría": "aficionar",
    "aficionaríais": "aficionar",
    "aficionaríamos": "aficionar",
    "aficionarían": "aficionar",
    "aficionarías": "aficionar",
    "aficionas": "aficionar",
    "aficionase": "aficionar",
    "aficionaseis": "aficionar",
    "aficionasen": "aficionar",
    "aficionases": "aficionar",
    "aficionaste": "aficionar",
    "aficionasteis": "aficionar",
    "aficione": "aficionar",
    "aficionemos": "aficionar",
    "aficionen": "aficionar",
    "aficiones": "aficionar",
    "aficiono": "aficionar",
    "aficionábamos": "aficionar",
    "aficionáis": "aficionar",
    "aficionáramos": "aficionar",
    "aficionáremos": "aficionar",
    "aficionásemos": "aficionar",
    "aficioné": "aficionar",
    "aficionéis": "aficionar",
    "aficionó": "aficionar",
    "afidavits": "afidavit",
    "afidávits": "afidávit",
    "afiebra": "afiebrar",
    "afiebraba": "afiebrar",
    "afiebrabais": "afiebrar",
    "afiebraban": "afiebrar",
    "afiebrabas": "afiebrar",
    "afiebrad": "afiebrar",
    "afiebrada": "afiebrar",
    "afiebradas": "afiebrar",
    "afiebrado": "afiebrar",
    "afiebrados": "afiebrar",
    "afiebramos": "afiebrar",
    "afiebran": "afiebrar",
    "afiebrando": "afiebrar",
    "afiebrara": "afiebrar",
    "afiebrarais": "afiebrar",
    "afiebraran": "afiebrar",
    "afiebraras": "afiebrar",
    "afiebrare": "afiebrar",
    "afiebrareis": "afiebrar",
    "afiebraremos": "afiebrar",
    "afiebraren": "afiebrar",
    "afiebrares": "afiebrar",
    "afiebraron": "afiebrar",
    "afiebrará": "afiebrar",
    "afiebrarán": "afiebrar",
    "afiebrarás": "afiebrar",
    "afiebraré": "afiebrar",
    "afiebraréis": "afiebrar",
    "afiebraría": "afiebrar",
    "afiebraríais": "afiebrar",
    "afiebraríamos": "afiebrar",
    "afiebrarían": "afiebrar",
    "afiebrarías": "afiebrar",
    "afiebras": "afiebrar",
    "afiebrase": "afiebrar",
    "afiebraseis": "afiebrar",
    "afiebrasen": "afiebrar",
    "afiebrases": "afiebrar",
    "afiebraste": "afiebrar",
    "afiebrasteis": "afiebrar",
    "afiebre": "afiebrar",
    "afiebremos": "afiebrar",
    "afiebren": "afiebrar",
    "afiebres": "afiebrar",
    "afiebro": "afiebrar",
    "afiebrábamos": "afiebrar",
    "afiebráis": "afiebrar",
    "afiebráramos": "afiebrar",
    "afiebráremos": "afiebrar",
    "afiebrásemos": "afiebrar",
    "afiebré": "afiebrar",
    "afiebréis": "afiebrar",
    "afiebró": "afiebrar",
    "afija": "afijar",
    "afijaba": "afijar",
    "afijabais": "afijar",
    "afijaban": "afijar",
    "afijabas": "afijar",
    "afijaciones": "afijación",
    "afijad": "afijar",
    "afijada": "afijar",
    "afijadas": "afijar",
    "afijado": "afijar",
    "afijados": "afijar",
    "afijales": "afijal",
    "afijamos": "afijar",
    "afijan": "afijar",
    "afijando": "afijar",
    "afijara": "afijar",
    "afijarais": "afijar",
    "afijaran": "afijar",
    "afijaras": "afijar",
    "afijare": "afijar",
    "afijareis": "afijar",
    "afijaremos": "afijar",
    "afijaren": "afijar",
    "afijares": "afijar",
    "afijaron": "afijar",
    "afijará": "afijar",
    "afijarán": "afijar",
    "afijarás": "afijar",
    "afijaré": "afijar",
    "afijaréis": "afijar",
    "afijaría": "afijar",
    "afijaríais": "afijar",
    "afijaríamos": "afijar",
    "afijarían": "afijar",
    "afijarías": "afijar",
    "afijas": "afijar",
    "afijase": "afijar",
    "afijaseis": "afijar",
    "afijasen": "afijar",
    "afijases": "afijar",
    "afijaste": "afijar",
    "afijasteis": "afijar",
    "afije": "afijar",
    "afijemos": "afijar",
    "afijen": "afijar",
    "afijes": "afijar",
    "afijo": "afijar",
    "afijos": "afijo",
    "afijábamos": "afijar",
    "afijáis": "afijar",
    "afijáramos": "afijar",
    "afijáremos": "afijar",
    "afijásemos": "afijar",
    "afijé": "afijar",
    "afijéis": "afijar",
    "afijó": "afijar",
    "afila": "afilar",
    "afilaba": "afilar",
    "afilabais": "afilar",
    "afilaban": "afilar",
    "afilabas": "afilar",
    "afilad": "afilar",
    "afilada": "afilar",
    "afiladas": "afilar",
    "afilado": "afilar",
    "afiladora": "afilador",
    "afiladoras": "afiladora",
    "afiladores": "afilador",
    "afilados": "afilar",
    "afiladuras": "afiladura",
    "afilamientos": "afilamiento",
    "afilamos": "afilar",
    "afilan": "afilar",
    "afilando": "afilar",
    "afilara": "afilar",
    "afilarais": "afilar",
    "afilaran": "afilar",
    "afilaras": "afilar",
    "afilare": "afilar",
    "afilareis": "afilar",
    "afilaremos": "afilar",
    "afilaren": "afilar",
    "afilares": "afilar",
    "afilaron": "afilar",
    "afilará": "afilar",
    "afilarán": "afilar",
    "afilarás": "afilar",
    "afilaré": "afilar",
    "afilaréis": "afilar",
    "afilaría": "afilar",
    "afilaríais": "afilar",
    "afilaríamos": "afilar",
    "afilarían": "afilar",
    "afilarías": "afilar",
    "afilas": "afilar",
    "afilase": "afilar",
    "afilaseis": "afilar",
    "afilasen": "afilar",
    "afilases": "afilar",
    "afilaste": "afilar",
    "afilasteis": "afilar",
    "afile": "afilar",
    "afilemos": "afilar",
    "afilen": "afilar",
    "afiles": "afilar",
    "afilia": "afiliar",
    "afiliaba": "afiliar",
    "afiliabais": "afiliar",
    "afiliaban": "afiliar",
    "afiliabas": "afiliar",
    "afiliaciones": "afiliación",
    "afiliad": "afiliar",
    "afiliada": "afiliar",
    "afiliadas": "afiliar",
    "afiliado": "afiliar",
    "afiliados": "afiliar",
    "afiliamos": "afiliar",
    "afilian": "afiliar",
    "afiliando": "afiliar",
    "afiliara": "afiliar",
    "afiliarais": "afiliar",
    "afiliaran": "afiliar",
    "afiliaras": "afiliar",
    "afiliare": "afiliar",
    "afiliareis": "afiliar",
    "afiliaremos": "afiliar",
    "afiliaren": "afiliar",
    "afiliares": "afiliar",
    "afiliaron": "afiliar",
    "afiliará": "afiliar",
    "afiliarán": "afiliar",
    "afiliarás": "afiliar",
    "afiliaré": "afiliar",
    "afiliaréis": "afiliar",
    "afiliaría": "afiliar",
    "afiliaríais": "afiliar",
    "afiliaríamos": "afiliar",
    "afiliarían": "afiliar",
    "afiliarías": "afiliar",
    "afilias": "afiliar",
    "afiliase": "afiliar",
    "afiliaseis": "afiliar",
    "afiliasen": "afiliar",
    "afiliases": "afiliar",
    "afiliaste": "afiliar",
    "afiliasteis": "afiliar",
    "afilie": "afiliar",
    "afiliemos": "afiliar",
    "afilien": "afiliar",
    "afilies": "afiliar",
    "afiligrana": "afiligranar",
    "afiligranaba": "afiligranar",
    "afiligranabais": "afiligranar",
    "afiligranaban": "afiligranar",
    "afiligranabas": "afiligranar",
    "afiligranad": "afiligranar",
    "afiligranada": "afiligranar",
    "afiligranadas": "afiligranar",
    "afiligranado": "afiligranar",
    "afiligranados": "afiligranar",
    "afiligranamos": "afiligranar",
    "afiligranan": "afiligranar",
    "afiligranando": "afiligranar",
    "afiligranara": "afiligranar",
    "afiligranarais": "afiligranar",
    "afiligranaran": "afiligranar",
    "afiligranaras": "afiligranar",
    "afiligranare": "afiligranar",
    "afiligranareis": "afiligranar",
    "afiligranaremos": "afiligranar",
    "afiligranaren": "afiligranar",
    "afiligranares": "afiligranar",
    "afiligranaron": "afiligranar",
    "afiligranará": "afiligranar",
    "afiligranarán": "afiligranar",
    "afiligranarás": "afiligranar",
    "afiligranaré": "afiligranar",
    "afiligranaréis": "afiligranar",
    "afiligranaría": "afiligranar",
    "afiligranaríais": "afiligranar",
    "afiligranaríamos": "afiligranar",
    "afiligranarían": "afiligranar",
    "afiligranarías": "afiligranar",
    "afiligranas": "afiligranar",
    "afiligranase": "afiligranar",
    "afiligranaseis": "afiligranar",
    "afiligranasen": "afiligranar",
    "afiligranases": "afiligranar",
    "afiligranaste": "afiligranar",
    "afiligranasteis": "afiligranar",
    "afiligrane": "afiligranar",
    "afiligranemos": "afiligranar",
    "afiligranen": "afiligranar",
    "afiligranes": "afiligranar",
    "afiligrano": "afiligranar",
    "afiligranábamos": "afiligranar",
    "afiligranáis": "afiligranar",
    "afiligranáramos": "afiligranar",
    "afiligranáremos": "afiligranar",
    "afiligranásemos": "afiligranar",
    "afiligrané": "afiligranar",
    "afiligranéis": "afiligranar",
    "afiligranó": "afiligranar",
    "afilio": "afiliar",
    "afiliábamos": "afiliar",
    "afiliáis": "afiliar",
    "afiliáramos": "afiliar",
    "afiliáremos": "afiliar",
    "afiliásemos": "afiliar",
    "afilié": "afiliar",
    "afiliéis": "afiliar",
    "afilió": "afiliar",
    "afilo": "afilar",
    "afilones": "afilón",
    "afilábamos": "afilar",
    "afiláctica": "afiláctico",
    "afilácticas": "afiláctico",
    "afilácticos": "afiláctico",
    "afiláis": "afilar",
    "afiláramos": "afilar",
    "afiláremos": "afilar",
    "afilásemos": "afilar",
    "afilé": "afilar",
    "afiléis": "afilar",
    "afiló": "afilar",
    "afina": "afinar",
    "afinaba": "afinar",
    "afinabais": "afinar",
    "afinaban": "afinar",
    "afinabas": "afinar",
    "afinaciones": "afinación",
    "afinad": "afinar",
    "afinada": "afinar",
    "afinadas": "afinar",
    "afinado": "afinar",
    "afinadora": "afinador",
    "afinadoras": "afinador",
    "afinadores": "afinador",
    "afinados": "afinar",
    "afinaduras": "afinadura",
    "afinamientos": "afinamiento",
    "afinamos": "afinar",
    "afinan": "afinar",
    "afinando": "afinar",
    "afinara": "afinar",
    "afinarais": "afinar",
    "afinaran": "afinar",
    "afinaras": "afinar",
    "afinare": "afinar",
    "afinareis": "afinar",
    "afinaremos": "afinar",
    "afinaren": "afinar",
    "afinares": "afinar",
    "afinaron": "afinar",
    "afinará": "afinar",
    "afinarán": "afinar",
    "afinarás": "afinar",
    "afinaré": "afinar",
    "afinaréis": "afinar",
    "afinaría": "afinar",
    "afinaríais": "afinar",
    "afinaríamos": "afinar",
    "afinarían": "afinar",
    "afinarías": "afinar",
    "afinas": "afinar",
    "afinase": "afinar",
    "afinaseis": "afinar",
    "afinasen": "afinar",
    "afinases": "afinar",
    "afinaste": "afinar",
    "afinasteis": "afinar",
    "afinca": "afincar",
    "afincaba": "afincar",
    "afincabais": "afincar",
    "afincaban": "afincar",
    "afincabas": "afincar",
    "afincad": "afincar",
    "afincada": "afincar",
    "afincadas": "afincar",
    "afincado": "afincar",
    "afincados": "afincar",
    "afincamos": "afincar",
    "afincan": "afincar",
    "afincando": "afincar",
    "afincara": "afincar",
    "afincarais": "afincar",
    "afincaran": "afincar",
    "afincaras": "afincar",
    "afincare": "afincar",
    "afincareis": "afincar",
    "afincaremos": "afincar",
    "afincaren": "afincar",
    "afincares": "afincar",
    "afincaron": "afincar",
    "afincará": "afincar",
    "afincarán": "afincar",
    "afincarás": "afincar",
    "afincaré": "afincar",
    "afincaréis": "afincar",
    "afincaría": "afincar",
    "afincaríais": "afincar",
    "afincaríamos": "afincar",
    "afincarían": "afincar",
    "afincarías": "afincar",
    "afincas": "afincar",
    "afincase": "afincar",
    "afincaseis": "afincar",
    "afincasen": "afincar",
    "afincases": "afincar",
    "afincaste": "afincar",
    "afincasteis": "afincar",
    "afinco": "afincar",
    "afincábamos": "afincar",
    "afincáis": "afincar",
    "afincáramos": "afincar",
    "afincáremos": "afincar",
    "afincásemos": "afincar",
    "afincó": "afincar",
    "afine": "afinar",
    "afinemos": "afinar",
    "afinen": "afinar",
    "afines": "afinar",
    "afinidades": "afinidad",
    "afino": "afinar",
    "afinos": "afino",
    "afinque": "afincar",
    "afinquemos": "afincar",
    "afinquen": "afincar",
    "afinques": "afincar",
    "afinqué": "afincar",
    "afinquéis": "afincar",
    "afinábamos": "afinar",
    "afináis": "afinar",
    "afináramos": "afinar",
    "afináremos": "afinar",
    "afinásemos": "afinar",
    "afiné": "afinar",
    "afinéis": "afinar",
    "afinó": "afinar",
    "afirma": "afirmar",
    "afirmaba": "afirmar",
    "afirmabais": "afirmar",
    "afirmaban": "afirmar",
    "afirmabas": "afirmar",
    "afirmaciones": "afirmación",
    "afirmad": "afirmar",
    "afirmada": "afirmar",
    "afirmadas": "afirmar",
    "afirmado": "afirmar",
    "afirmados": "afirmar",
    "afirmamos": "afirmar",
    "afirman": "afirmar",
    "afirmando": "afirmar",
    "afirmara": "afirmar",
    "afirmarais": "afirmar",
    "afirmaran": "afirmar",
    "afirmaras": "afirmar",
    "afirmare": "afirmar",
    "afirmareis": "afirmar",
    "afirmaremos": "afirmar",
    "afirmaren": "afirmar",
    "afirmares": "afirmar",
    "afirmaron": "afirmar",
    "afirmará": "afirmar",
    "afirmarán": "afirmar",
    "afirmarás": "afirmar",
    "afirmaré": "afirmar",
    "afirmaréis": "afirmar",
    "afirmaría": "afirmar",
    "afirmaríais": "afirmar",
    "afirmaríamos": "afirmar",
    "afirmarían": "afirmar",
    "afirmarías": "afirmar",
    "afirmas": "afirmar",
    "afirmase": "afirmar",
    "afirmaseis": "afirmar",
    "afirmasen": "afirmar",
    "afirmases": "afirmar",
    "afirmaste": "afirmar",
    "afirmasteis": "afirmar",
    "afirmativa": "afirmativo",
    "afirmativas": "afirmativo",
    "afirmativos": "afirmativo",
    "afirme": "afirmar",
    "afirmemos": "afirmar",
    "afirmen": "afirmar",
    "afirmes": "afirmar",
    "afirmo": "afirmar",
    "afirmábamos": "afirmar",
    "afirmáis": "afirmar",
    "afirmáramos": "afirmar",
    "afirmáremos": "afirmar",
    "afirmásemos": "afirmar",
    "afirmé": "afirmar",
    "afirméis": "afirmar",
    "afirmó": "afirmar",
    "afistula": "afistular",
    "afistulaba": "afistular",
    "afistulabais": "afistular",
    "afistulaban": "afistular",
    "afistulabas": "afistular",
    "afistulad": "afistular",
    "afistulada": "afistular",
    "afistuladas": "afistular",
    "afistulado": "afistular",
    "afistulados": "afistular",
    "afistulamos": "afistular",
    "afistulan": "afistular",
    "afistulando": "afistular",
    "afistulara": "afistular",
    "afistularais": "afistular",
    "afistularan": "afistular",
    "afistularas": "afistular",
    "afistulare": "afistular",
    "afistulareis": "afistular",
    "afistularemos": "afistular",
    "afistularen": "afistular",
    "afistulares": "afistular",
    "afistularon": "afistular",
    "afistulará": "afistular",
    "afistularán": "afistular",
    "afistularás": "afistular",
    "afistularé": "afistular",
    "afistularéis": "afistular",
    "afistularía": "afistular",
    "afistularíais": "afistular",
    "afistularíamos": "afistular",
    "afistularían": "afistular",
    "afistularías": "afistular",
    "afistulas": "afistular",
    "afistulase": "afistular",
    "afistulaseis": "afistular",
    "afistulasen": "afistular",
    "afistulases": "afistular",
    "afistulaste": "afistular",
    "afistulasteis": "afistular",
    "afistule": "afistular",
    "afistulemos": "afistular",
    "afistulen": "afistular",
    "afistules": "afistular",
    "afistulo": "afistular",
    "afistulábamos": "afistular",
    "afistuláis": "afistular",
    "afistuláramos": "afistular",
    "afistuláremos": "afistular",
    "afistulásemos": "afistular",
    "afistulé": "afistular",
    "afistuléis": "afistular",
    "afistuló": "afistular",
    "aflatoxinas": "aflatoxina",
    "aflautada": "aflautado",
    "aflautadas": "aflautado",
    "aflautados": "aflautado",
    "aflicciones": "aflicción",
    "aflictiva": "aflictivo",
    "aflictivas": "aflictivo",
    "aflictivos": "aflictivo",
    "aflige": "afligir",
    "afligen": "afligir",
    "afliges": "afligir",
    "afligid": "afligir",
    "afligida": "afligir",
    "afligidas": "afligir",
    "afligido": "afligir",
    "afligidos": "afligir",
    "afligiendo": "afligir",
    "afligiera": "afligir",
    "afligierais": "afligir",
    "afligieran": "afligir",
    "afligieras": "afligir",
    "afligiere": "afligir",
    "afligiereis": "afligir",
    "afligieren": "afligir",
    "afligieres": "afligir",
    "afligieron": "afligir",
    "afligiese": "afligir",
    "afligieseis": "afligir",
    "afligiesen": "afligir",
    "afligieses": "afligir",
    "afligimientos": "afligimiento",
    "afligimos": "afligir",
    "afligiremos": "afligir",
    "afligirá": "afligir",
    "afligirán": "afligir",
    "afligirás": "afligir",
    "afligiré": "afligir",
    "afligiréis": "afligir",
    "afligiría": "afligir",
    "afligiríais": "afligir",
    "afligiríamos": "afligir",
    "afligirían": "afligir",
    "afligirías": "afligir",
    "afligiste": "afligir",
    "afligisteis": "afligir",
    "afligiéramos": "afligir",
    "afligiéremos": "afligir",
    "afligiésemos": "afligir",
    "afligió": "afligir",
    "afligí": "afligir",
    "afligía": "afligir",
    "afligíais": "afligir",
    "afligíamos": "afligir",
    "afligían": "afligir",
    "afligías": "afligir",
    "afligís": "afligir",
    "aflija": "afligir",
    "aflijamos": "afligir",
    "aflijan": "afligir",
    "aflijas": "afligir",
    "aflijo": "afligir",
    "aflijáis": "afligir",
    "afloja": "aflojar",
    "aflojaba": "aflojar",
    "aflojabais": "aflojar",
    "aflojaban": "aflojar",
    "aflojabas": "aflojar",
    "aflojad": "aflojar",
    "aflojada": "aflojar",
    "aflojadas": "aflojar",
    "aflojado": "aflojar",
    "aflojados": "aflojar",
    "aflojamientos": "aflojamiento",
    "aflojamos": "aflojar",
    "aflojan": "aflojar",
    "aflojando": "aflojar",
    "aflojara": "aflojar",
    "aflojarais": "aflojar",
    "aflojaran": "aflojar",
    "aflojaras": "aflojar",
    "aflojare": "aflojar",
    "aflojareis": "aflojar",
    "aflojaremos": "aflojar",
    "aflojaren": "aflojar",
    "aflojares": "aflojar",
    "aflojaron": "aflojar",
    "aflojará": "aflojar",
    "aflojarán": "aflojar",
    "aflojarás": "aflojar",
    "aflojaré": "aflojar",
    "aflojaréis": "aflojar",
    "aflojaría": "aflojar",
    "aflojaríais": "aflojar",
    "aflojaríamos": "aflojar",
    "aflojarían": "aflojar",
    "aflojarías": "aflojar",
    "aflojas": "aflojar",
    "aflojase": "aflojar",
    "aflojaseis": "aflojar",
    "aflojasen": "aflojar",
    "aflojases": "aflojar",
    "aflojaste": "aflojar",
    "aflojasteis": "aflojar",
    "afloje": "aflojar",
    "aflojemos": "aflojar",
    "aflojen": "aflojar",
    "aflojes": "aflojar",
    "aflojo": "aflojar",
    "aflojábamos": "aflojar",
    "aflojáis": "aflojar",
    "aflojáramos": "aflojar",
    "aflojáremos": "aflojar",
    "aflojásemos": "aflojar",
    "aflojé": "aflojar",
    "aflojéis": "aflojar",
    "aflojó": "aflojar",
    "aflora": "aflorar",
    "afloraba": "aflorar",
    "aflorabais": "aflorar",
    "afloraban": "aflorar",
    "aflorabas": "aflorar",
    "afloraciones": "afloración",
    "aflorad": "aflorar",
    "aflorada": "aflorar",
    "afloradas": "aflorar",
    "aflorado": "aflorar",
    "aflorados": "aflorar",
    "afloramientos": "afloramiento",
    "afloramos": "aflorar",
    "afloran": "aflorar",
    "aflorando": "aflorar",
    "aflorara": "aflorar",
    "aflorarais": "aflorar",
    "afloraran": "aflorar",
    "afloraras": "aflorar",
    "aflorare": "aflorar",
    "aflorareis": "aflorar",
    "afloraremos": "aflorar",
    "afloraren": "aflorar",
    "aflorares": "aflorar",
    "afloraron": "aflorar",
    "aflorará": "aflorar",
    "aflorarán": "aflorar",
    "aflorarás": "aflorar",
    "afloraré": "aflorar",
    "afloraréis": "aflorar",
    "afloraría": "aflorar",
    "afloraríais": "aflorar",
    "afloraríamos": "aflorar",
    "aflorarían": "aflorar",
    "aflorarías": "aflorar",
    "afloras": "aflorar",
    "aflorase": "aflorar",
    "afloraseis": "aflorar",
    "aflorasen": "aflorar",
    "aflorases": "aflorar",
    "afloraste": "aflorar",
    "aflorasteis": "aflorar",
    "aflore": "aflorar",
    "afloremos": "aflorar",
    "afloren": "aflorar",
    "aflores": "aflorar",
    "afloro": "aflorar",
    "aflorábamos": "aflorar",
    "afloráis": "aflorar",
    "afloráramos": "aflorar",
    "afloráremos": "aflorar",
    "aflorásemos": "aflorar",
    "afloré": "aflorar",
    "afloréis": "aflorar",
    "afloró": "aflorar",
    "afluencias": "afluencia",
    "afluentes": "afluente",
    "afluid": "afluir",
    "afluida": "afluir",
    "afluidas": "afluir",
    "afluido": "afluir",
    "afluidos": "afluir",
    "afluimos": "afluir",
    "afluiremos": "afluir",
    "afluirá": "afluir",
    "afluirán": "afluir",
    "afluirás": "afluir",
    "afluiré": "afluir",
    "afluiréis": "afluir",
    "afluiría": "afluir",
    "afluiríais": "afluir",
    "afluiríamos": "afluir",
    "afluirían": "afluir",
    "afluirías": "afluir",
    "afluiste": "afluir",
    "afluisteis": "afluir",
    "aflujos": "aflujo",
    "afluya": "afluir",
    "afluyamos": "afluir",
    "afluyan": "afluir",
    "afluyas": "afluir",
    "afluye": "afluir",
    "afluyen": "afluir",
    "afluyendo": "afluir",
    "afluyera": "afluir",
    "afluyerais": "afluir",
    "afluyeran": "afluir",
    "afluyeras": "afluir",
    "afluyere": "afluir",
    "afluyereis": "afluir",
    "afluyeren": "afluir",
    "afluyeres": "afluir",
    "afluyeron": "afluir",
    "afluyes": "afluir",
    "afluyese": "afluir",
    "afluyeseis": "afluir",
    "afluyesen": "afluir",
    "afluyeses": "afluir",
    "afluyo": "afluir",
    "afluyáis": "afluir",
    "afluyéramos": "afluir",
    "afluyéremos": "afluir",
    "afluyésemos": "afluir",
    "afluyó": "afluir",
    "afluí": "afluir",
    "afluía": "afluir",
    "afluíais": "afluir",
    "afluíamos": "afluir",
    "afluían": "afluir",
    "afluías": "afluir",
    "afluís": "afluir",
    "afocales": "afocal",
    "afogara": "afogarar",
    "afogaraba": "afogarar",
    "afogarabais": "afogarar",
    "afogaraban": "afogarar",
    "afogarabas": "afogarar",
    "afogarad": "afogarar",
    "afogarada": "afogarar",
    "afogaradas": "afogarar",
    "afogarado": "afogarar",
    "afogarados": "afogarar",
    "afogaramos": "afogarar",
    "afogaran": "afogarar",
    "afogarando": "afogarar",
    "afogarara": "afogarar",
    "afogararais": "afogarar",
    "afogararan": "afogarar",
    "afogararas": "afogarar",
    "afogarare": "afogarar",
    "afogarareis": "afogarar",
    "afogararemos": "afogarar",
    "afogararen": "afogarar",
    "afogarares": "afogarar",
    "afogararon": "afogarar",
    "afogarará": "afogarar",
    "afogararán": "afogarar",
    "afogararás": "afogarar",
    "afogararé": "afogarar",
    "afogararéis": "afogarar",
    "afogararía": "afogarar",
    "afogararíais": "afogarar",
    "afogararíamos": "afogarar",
    "afogararían": "afogarar",
    "afogararías": "afogarar",
    "afogaras": "afogarar",
    "afogarase": "afogarar",
    "afogaraseis": "afogarar",
    "afogarasen": "afogarar",
    "afogarases": "afogarar",
    "afogaraste": "afogarar",
    "afogarasteis": "afogarar",
    "afogare": "afogarar",
    "afogaremos": "afogarar",
    "afogaren": "afogarar",
    "afogares": "afogarar",
    "afogaro": "afogarar",
    "afogarábamos": "afogarar",
    "afogaráis": "afogarar",
    "afogaráramos": "afogarar",
    "afogaráremos": "afogarar",
    "afogarásemos": "afogarar",
    "afogaré": "afogarar",
    "afogaréis": "afogarar",
    "afogaró": "afogarar",
    "afollaba": "afollar",
    "afollabais": "afollar",
    "afollaban": "afollar",
    "afollabas": "afollar",
    "afollad": "afollar",
    "afollada": "afollar",
    "afolladas": "afollar",
    "afollado": "afollar",
    "afollados": "afollar",
    "afollamos": "afollar",
    "afollando": "afollar",
    "afollara": "afollar",
    "afollarais": "afollar",
    "afollaran": "afollar",
    "afollaras": "afollar",
    "afollare": "afollar",
    "afollareis": "afollar",
    "afollaremos": "afollar",
    "afollaren": "afollar",
    "afollares": "afollar",
    "afollaron": "afollar",
    "afollará": "afollar",
    "afollarán": "afollar",
    "afollarás": "afollar",
    "afollaré": "afollar",
    "afollaréis": "afollar",
    "afollaría": "afollar",
    "afollaríais": "afollar",
    "afollaríamos": "afollar",
    "afollarían": "afollar",
    "afollarías": "afollar",
    "afollase": "afollar",
    "afollaseis": "afollar",
    "afollasen": "afollar",
    "afollases": "afollar",
    "afollaste": "afollar",
    "afollasteis": "afollar",
    "afollemos": "afollar",
    "afollábamos": "afollar",
    "afolláis": "afollar",
    "afolláramos": "afollar",
    "afolláremos": "afollar",
    "afollásemos": "afollar",
    "afollé": "afollar",
    "afolléis": "afollar",
    "afolló": "afollar",
    "afonías": "afonía",
    "aforaba": "aforar",
    "aforabais": "aforar",
    "aforaban": "aforar",
    "aforabas": "aforar",
    "aforad": "aforar",
    "aforada": "aforar",
    "aforadas": "aforar",
    "aforado": "aforar",
    "aforadora": "aforador",
    "aforadoras": "aforador",
    "aforadores": "aforador",
    "aforados": "aforar",
    "aforamientos": "aforamiento",
    "aforamos": "aforar",
    "aforando": "aforar",
    "aforara": "aforar",
    "aforarais": "aforar",
    "aforaran": "aforar",
    "aforaras": "aforar",
    "aforare": "aforar",
    "aforareis": "aforar",
    "aforaremos": "aforar",
    "aforaren": "aforar",
    "aforares": "aforar",
    "aforaron": "aforar",
    "aforará": "aforar",
    "aforarán": "aforar",
    "aforarás": "aforar",
    "aforaré": "aforar",
    "aforaréis": "aforar",
    "aforaría": "aforar",
    "aforaríais": "aforar",
    "aforaríamos": "aforar",
    "aforarían": "aforar",
    "aforarías": "aforar",
    "aforase": "aforar",
    "aforaseis": "aforar",
    "aforasen": "aforar",
    "aforases": "aforar",
    "aforaste": "aforar",
    "aforasteis": "aforar",
    "aforemos": "aforar",
    "aforestaciones": "aforestación",
    "aforismos": "aforismo",
    "aforos": "aforo",
    "aforra": "aforrar",
    "aforraba": "aforrar",
    "aforrabais": "aforrar",
    "aforraban": "aforrar",
    "aforrabas": "aforrar",
    "aforrad": "aforrar",
    "aforrada": "aforrar",
    "aforradas": "aforrar",
    "aforrado": "aforrar",
    "aforrados": "aforrar",
    "aforramos": "aforrar",
    "aforran": "aforrar",
    "aforrando": "aforrar",
    "aforrara": "aforrar",
    "aforrarais": "aforrar",
    "aforraran": "aforrar",
    "aforraras": "aforrar",
    "aforrare": "aforrar",
    "aforrareis": "aforrar",
    "aforraremos": "aforrar",
    "aforraren": "aforrar",
    "aforrares": "aforrar",
    "aforraron": "aforrar",
    "aforrará": "aforrar",
    "aforrarán": "aforrar",
    "aforrarás": "aforrar",
    "aforraré": "aforrar",
    "aforraréis": "aforrar",
    "aforraría": "aforrar",
    "aforraríais": "aforrar",
    "aforraríamos": "aforrar",
    "aforrarían": "aforrar",
    "aforrarías": "aforrar",
    "aforras": "aforrar",
    "aforrase": "aforrar",
    "aforraseis": "aforrar",
    "aforrasen": "aforrar",
    "aforrases": "aforrar",
    "aforraste": "aforrar",
    "aforrasteis": "aforrar",
    "aforre": "aforrar",
    "aforremos": "aforrar",
    "aforren": "aforrar",
    "aforres": "aforrar",
    "aforro": "aforrar",
    "aforros": "aforro",
    "aforrábamos": "aforrar",
    "aforráis": "aforrar",
    "aforráramos": "aforrar",
    "aforráremos": "aforrar",
    "aforrásemos": "aforrar",
    "aforré": "aforrar",
    "aforréis": "aforrar",
    "aforró": "aforrar",
    "afortunada": "afortunado",
    "afortunadas": "afortunado",
    "afortunados": "afortunado",
    "aforábamos": "aforar",
    "aforáis": "aforar",
    "aforáramos": "aforar",
    "aforáremos": "aforar",
    "aforásemos": "aforar",
    "aforé": "aforar",
    "aforéis": "aforar",
    "aforística": "aforístico",
    "aforísticas": "aforístico",
    "aforísticos": "aforístico",
    "aforó": "aforar",
    "afosca": "afoscar",
    "afoscaba": "afoscar",
    "afoscabais": "afoscar",
    "afoscaban": "afoscar",
    "afoscabas": "afoscar",
    "afoscad": "afoscar",
    "afoscada": "afoscar",
    "afoscadas": "afoscar",
    "afoscado": "afoscar",
    "afoscados": "afoscar",
    "afoscamos": "afoscar",
    "afoscan": "afoscar",
    "afoscando": "afoscar",
    "afoscara": "afoscar",
    "afoscarais": "afoscar",
    "afoscaran": "afoscar",
    "afoscaras": "afoscar",
    "afoscare": "afoscar",
    "afoscareis": "afoscar",
    "afoscaremos": "afoscar",
    "afoscaren": "afoscar",
    "afoscares": "afoscar",
    "afoscaron": "afoscar",
    "afoscará": "afoscar",
    "afoscarán": "afoscar",
    "afoscarás": "afoscar",
    "afoscaré": "afoscar",
    "afoscaréis": "afoscar",
    "afoscaría": "afoscar",
    "afoscaríais": "afoscar",
    "afoscaríamos": "afoscar",
    "afoscarían": "afoscar",
    "afoscarías": "afoscar",
    "afoscas": "afoscar",
    "afoscase": "afoscar",
    "afoscaseis": "afoscar",
    "afoscasen": "afoscar",
    "afoscases": "afoscar",
    "afoscaste": "afoscar",
    "afoscasteis": "afoscar",
    "afosco": "afoscar",
    "afoscábamos": "afoscar",
    "afoscáis": "afoscar",
    "afoscáramos": "afoscar",
    "afoscáremos": "afoscar",
    "afoscásemos": "afoscar",
    "afoscó": "afoscar",
    "afosque": "afoscar",
    "afosquemos": "afoscar",
    "afosquen": "afoscar",
    "afosques": "afoscar",
    "afosqué": "afoscar",
    "afosquéis": "afoscar",
    "afotestesias": "afotestesia",
    "afra": "afro",
    "afraila": "afrailar",
    "afrailaba": "afrailar",
    "afrailabais": "afrailar",
    "afrailaban": "afrailar",
    "afrailabas": "afrailar",
    "afrailad": "afrailar",
    "afrailada": "afrailar",
    "afrailadas": "afrailar",
    "afrailado": "afrailar",
    "afrailados": "afrailar",
    "afrailamientos": "afrailamiento",
    "afrailamos": "afrailar",
    "afrailan": "afrailar",
    "afrailando": "afrailar",
    "afrailara": "afrailar",
    "afrailarais": "afrailar",
    "afrailaran": "afrailar",
    "afrailaras": "afrailar",
    "afrailare": "afrailar",
    "afrailareis": "afrailar",
    "afrailaremos": "afrailar",
    "afrailaren": "afrailar",
    "afrailares": "afrailar",
    "afrailaron": "afrailar",
    "afrailará": "afrailar",
    "afrailarán": "afrailar",
    "afrailarás": "afrailar",
    "afrailaré": "afrailar",
    "afrailaréis": "afrailar",
    "afrailaría": "afrailar",
    "afrailaríais": "afrailar",
    "afrailaríamos": "afrailar",
    "afrailarían": "afrailar",
    "afrailarías": "afrailar",
    "afrailas": "afrailar",
    "afrailase": "afrailar",
    "afrailaseis": "afrailar",
    "afrailasen": "afrailar",
    "afrailases": "afrailar",
    "afrailaste": "afrailar",
    "afrailasteis": "afrailar",
    "afraile": "afrailar",
    "afrailemos": "afrailar",
    "afrailen": "afrailar",
    "afrailes": "afrailar",
    "afrailo": "afrailar",
    "afrailábamos": "afrailar",
    "afrailáis": "afrailar",
    "afrailáramos": "afrailar",
    "afrailáremos": "afrailar",
    "afrailásemos": "afrailar",
    "afrailé": "afrailar",
    "afrailéis": "afrailar",
    "afrailó": "afrailar",
    "afrancesa": "afrancesar",
    "afrancesaba": "afrancesar",
    "afrancesabais": "afrancesar",
    "afrancesaban": "afrancesar",
    "afrancesabas": "afrancesar",
    "afrancesad": "afrancesar",
    "afrancesada": "afrancesar",
    "afrancesadas": "afrancesar",
    "afrancesado": "afrancesar",
    "afrancesados": "afrancesar",
    "afrancesamientos": "afrancesamiento",
    "afrancesamos": "afrancesar",
    "afrancesan": "afrancesar",
    "afrancesando": "afrancesar",
    "afrancesara": "afrancesar",
    "afrancesarais": "afrancesar",
    "afrancesaran": "afrancesar",
    "afrancesaras": "afrancesar",
    "afrancesare": "afrancesar",
    "afrancesareis": "afrancesar",
    "afrancesaremos": "afrancesar",
    "afrancesaren": "afrancesar",
    "afrancesares": "afrancesar",
    "afrancesaron": "afrancesar",
    "afrancesará": "afrancesar",
    "afrancesarán": "afrancesar",
    "afrancesarás": "afrancesar",
    "afrancesaré": "afrancesar",
    "afrancesaréis": "afrancesar",
    "afrancesaría": "afrancesar",
    "afrancesaríais": "afrancesar",
    "afrancesaríamos": "afrancesar",
    "afrancesarían": "afrancesar",
    "afrancesarías": "afrancesar",
    "afrancesas": "afrancesar",
    "afrancesase": "afrancesar",
    "afrancesaseis": "afrancesar",
    "afrancesasen": "afrancesar",
    "afrancesases": "afrancesar",
    "afrancesaste": "afrancesar",
    "afrancesasteis": "afrancesar",
    "afrancese": "afrancesar",
    "afrancesemos": "afrancesar",
    "afrancesen": "afrancesar",
    "afranceses": "afrancesar",
    "afranceso": "afrancesar",
    "afrancesábamos": "afrancesar",
    "afrancesáis": "afrancesar",
    "afrancesáramos": "afrancesar",
    "afrancesáremos": "afrancesar",
    "afrancesásemos": "afrancesar",
    "afrancesé": "afrancesar",
    "afranceséis": "afrancesar",
    "afrancesó": "afrancesar",
    "afras": "afro",
    "afrechos": "afrecho",
    "afrenilla": "afrenillar",
    "afrenillaba": "afrenillar",
    "afrenillabais": "afrenillar",
    "afrenillaban": "afrenillar",
    "afrenillabas": "afrenillar",
    "afrenillad": "afrenillar",
    "afrenillada": "afrenillar",
    "afrenilladas": "afrenillar",
    "afrenillado": "afrenillar",
    "afrenillados": "afrenillar",
    "afrenillamos": "afrenillar",
    "afrenillan": "afrenillar",
    "afrenillando": "afrenillar",
    "afrenillara": "afrenillar",
    "afrenillarais": "afrenillar",
    "afrenillaran": "afrenillar",
    "afrenillaras": "afrenillar",
    "afrenillare": "afrenillar",
    "afrenillareis": "afrenillar",
    "afrenillaremos": "afrenillar",
    "afrenillaren": "afrenillar",
    "afrenillares": "afrenillar",
    "afrenillaron": "afrenillar",
    "afrenillará": "afrenillar",
    "afrenillarán": "afrenillar",
    "afrenillarás": "afrenillar",
    "afrenillaré": "afrenillar",
    "afrenillaréis": "afrenillar",
    "afrenillaría": "afrenillar",
    "afrenillaríais": "afrenillar",
    "afrenillaríamos": "afrenillar",
    "afrenillarían": "afrenillar",
    "afrenillarías": "afrenillar",
    "afrenillas": "afrenillar",
    "afrenillase": "afrenillar",
    "afrenillaseis": "afrenillar",
    "afrenillasen": "afrenillar",
    "afrenillases": "afrenillar",
    "afrenillaste": "afrenillar",
    "afrenillasteis": "afrenillar",
    "afrenille": "afrenillar",
    "afrenillemos": "afrenillar",
    "afrenillen": "afrenillar",
    "afrenilles": "afrenillar",
    "afrenillo": "afrenillar",
    "afrenillábamos": "afrenillar",
    "afrenilláis": "afrenillar",
    "afrenilláramos": "afrenillar",
    "afrenilláremos": "afrenillar",
    "afrenillásemos": "afrenillar",
    "afrenillé": "afrenillar",
    "afrenilléis": "afrenillar",
    "afrenilló": "afrenillar",
    "afrenta": "afrentar",
    "afrentaba": "afrentar",
    "afrentabais": "afrentar",
    "afrentaban": "afrentar",
    "afrentabas": "afrentar",
    "afrentad": "afrentar",
    "afrentada": "afrentar",
    "afrentadas": "afrentar",
    "afrentado": "afrentar",
    "afrentadora": "afrentador",
    "afrentadoras": "afrentador",
    "afrentadores": "afrentador",
    "afrentados": "afrentar",
    "afrentamos": "afrentar",
    "afrentan": "afrentar",
    "afrentando": "afrentar",
    "afrentara": "afrentar",
    "afrentarais": "afrentar",
    "afrentaran": "afrentar",
    "afrentaras": "afrentar",
    "afrentare": "afrentar",
    "afrentareis": "afrentar",
    "afrentaremos": "afrentar",
    "afrentaren": "afrentar",
    "afrentares": "afrentar",
    "afrentaron": "afrentar",
    "afrentará": "afrentar",
    "afrentarán": "afrentar",
    "afrentarás": "afrentar",
    "afrentaré": "afrentar",
    "afrentaréis": "afrentar",
    "afrentaría": "afrentar",
    "afrentaríais": "afrentar",
    "afrentaríamos": "afrentar",
    "afrentarían": "afrentar",
    "afrentarías": "afrentar",
    "afrentas": "afrentar",
    "afrentase": "afrentar",
    "afrentaseis": "afrentar",
    "afrentasen": "afrentar",
    "afrentases": "afrentar",
    "afrentaste": "afrentar",
    "afrentasteis": "afrentar",
    "afrente": "afrentar",
    "afrentemos": "afrentar",
    "afrenten": "afrentar",
    "afrentes": "afrentar",
    "afrento": "afrentar",
    "afrentosa": "afrentoso",
    "afrentosas": "afrentoso",
    "afrentosos": "afrentoso",
    "afrentábamos": "afrentar",
    "afrentáis": "afrentar",
    "afrentáramos": "afrentar",
    "afrentáremos": "afrentar",
    "afrentásemos": "afrentar",
    "afrenté": "afrentar",
    "afrentéis": "afrentar",
    "afrentó": "afrentar",
    "afreta": "afretar",
    "afretaba": "afretar",
    "afretabais": "afretar",
    "afretaban": "afretar",
    "afretabas": "afretar",
    "afretad": "afretar",
    "afretada": "afretar",
    "afretadas": "afretar",
    "afretado": "afretar",
    "afretados": "afretar",
    "afretamos": "afretar",
    "afretan": "afretar",
    "afretando": "afretar",
    "afretara": "afretar",
    "afretarais": "afretar",
    "afretaran": "afretar",
    "afretaras": "afretar",
    "afretare": "afretar",
    "afretareis": "afretar",
    "afretaremos": "afretar",
    "afretaren": "afretar",
    "afretares": "afretar",
    "afretaron": "afretar",
    "afretará": "afretar",
    "afretarán": "afretar",
    "afretarás": "afretar",
    "afretaré": "afretar",
    "afretaréis": "afretar",
    "afretaría": "afretar",
    "afretaríais": "afretar",
    "afretaríamos": "afretar",
    "afretarían": "afretar",
    "afretarías": "afretar",
    "afretas": "afretar",
    "afretase": "afretar",
    "afretaseis": "afretar",
    "afretasen": "afretar",
    "afretases": "afretar",
    "afretaste": "afretar",
    "afretasteis": "afretar",
    "afrete": "afretar",
    "afretemos": "afretar",
    "afreten": "afretar",
    "afretes": "afretar",
    "afreto": "afretar",
    "afretábamos": "afretar",
    "afretáis": "afretar",
    "afretáramos": "afretar",
    "afretáremos": "afretar",
    "afretásemos": "afretar",
    "afreté": "afretar",
    "afretéis": "afretar",
    "afretó": "afretar",
    "africaciones": "africación",
    "africada": "africado",
    "africadas": "africado",
    "africados": "africado",
    "africana": "africano",
    "africanas": "africano",
    "africanice": "africanizar",
    "africanicemos": "africanizar",
    "africanicen": "africanizar",
    "africanices": "africanizar",
    "africanicé": "africanizar",
    "africanicéis": "africanizar",
    "africanismos": "africanismo",
    "africanistas": "africanista",
    "africaniza": "africanizar",
    "africanizaba": "africanizar",
    "africanizabais": "africanizar",
    "africanizaban": "africanizar",
    "africanizabas": "africanizar",
    "africanizaciones": "africanización",
    "africanizad": "africanizar",
    "africanizada": "africanizar",
    "africanizadas": "africanizar",
    "africanizado": "africanizar",
    "africanizados": "africanizar",
    "africanizamos": "africanizar",
    "africanizan": "africanizar",
    "africanizando": "africanizar",
    "africanizara": "africanizar",
    "africanizarais": "africanizar",
    "africanizaran": "africanizar",
    "africanizaras": "africanizar",
    "africanizare": "africanizar",
    "africanizareis": "africanizar",
    "africanizaremos": "africanizar",
    "africanizaren": "africanizar",
    "africanizares": "africanizar",
    "africanizaron": "africanizar",
    "africanizará": "africanizar",
    "africanizarán": "africanizar",
    "africanizarás": "africanizar",
    "africanizaré": "africanizar",
    "africanizaréis": "africanizar",
    "africanizaría": "africanizar",
    "africanizaríais": "africanizar",
    "africanizaríamos": "africanizar",
    "africanizarían": "africanizar",
    "africanizarías": "africanizar",
    "africanizas": "africanizar",
    "africanizase": "africanizar",
    "africanizaseis": "africanizar",
    "africanizasen": "africanizar",
    "africanizases": "africanizar",
    "africanizaste": "africanizar",
    "africanizasteis": "africanizar",
    "africanizo": "africanizar",
    "africanizábamos": "africanizar",
    "africanizáis": "africanizar",
    "africanizáramos": "africanizar",
    "africanizáremos": "africanizar",
    "africanizásemos": "africanizar",
    "africanizó": "africanizar",
    "africanos": "africano",
    "afrikaners": "afrikaner",
    "afroamericana": "afroamericano",
    "afroamericanas": "afroamericano",
    "afroamericanos": "afroamericano",
    "afroasiática": "afroasiático",
    "afroasiáticas": "afroasiático",
    "afroasiáticos": "afroasiático",
    "afrocubana": "afrocubano",
    "afrocubanas": "afrocubano",
    "afrocubanos": "afrocubano",
    "afrodisíaca": "afrodisíaco",
    "afrodisíacas": "afrodisíaco",
    "afrodisíacos": "afrodisíaco",
    "afronta": "afrontar",
    "afrontaba": "afrontar",
    "afrontabais": "afrontar",
    "afrontaban": "afrontar",
    "afrontabas": "afrontar",
    "afrontad": "afrontar",
    "afrontada": "afrontar",
    "afrontadas": "afrontar",
    "afrontado": "afrontar",
    "afrontados": "afrontar",
    "afrontamientos": "afrontamiento",
    "afrontamos": "afrontar",
    "afrontan": "afrontar",
    "afrontando": "afrontar",
    "afrontara": "afrontar",
    "afrontarais": "afrontar",
    "afrontaran": "afrontar",
    "afrontaras": "afrontar",
    "afrontare": "afrontar",
    "afrontareis": "afrontar",
    "afrontaremos": "afrontar",
    "afrontaren": "afrontar",
    "afrontares": "afrontar",
    "afrontaron": "afrontar",
    "afrontará": "afrontar",
    "afrontarán": "afrontar",
    "afrontarás": "afrontar",
    "afrontaré": "afrontar",
    "afrontaréis": "afrontar",
    "afrontaría": "afrontar",
    "afrontaríais": "afrontar",
    "afrontaríamos": "afrontar",
    "afrontarían": "afrontar",
    "afrontarías": "afrontar",
    "afrontas": "afrontar",
    "afrontase": "afrontar",
    "afrontaseis": "afrontar",
    "afrontasen": "afrontar",
    "afrontases": "afrontar",
    "afrontaste": "afrontar",
    "afrontasteis": "afrontar",
    "afronte": "afrontar",
    "afrontemos": "afrontar",
    "afronten": "afrontar",
    "afrontes": "afrontar",
    "afrontila": "afrontilar",
    "afrontilaba": "afrontilar",
    "afrontilabais": "afrontilar",
    "afrontilaban": "afrontilar",
    "afrontilabas": "afrontilar",
    "afrontilad": "afrontilar",
    "afrontilada": "afrontilar",
    "afrontiladas": "afrontilar",
    "afrontilado": "afrontilar",
    "afrontilados": "afrontilar",
    "afrontilamos": "afrontilar",
    "afrontilan": "afrontilar",
    "afrontilando": "afrontilar",
    "afrontilara": "afrontilar",
    "afrontilarais": "afrontilar",
    "afrontilaran": "afrontilar",
    "afrontilaras": "afrontilar",
    "afrontilare": "afrontilar",
    "afrontilareis": "afrontilar",
    "afrontilaremos": "afrontilar",
    "afrontilaren": "afrontilar",
    "afrontilares": "afrontilar",
    "afrontilaron": "afrontilar",
    "afrontilará": "afrontilar",
    "afrontilarán": "afrontilar",
    "afrontilarás": "afrontilar",
    "afrontilaré": "afrontilar",
    "afrontilaréis": "afrontilar",
    "afrontilaría": "afrontilar",
    "afrontilaríais": "afrontilar",
    "afrontilaríamos": "afrontilar",
    "afrontilarían": "afrontilar",
    "afrontilarías": "afrontilar",
    "afrontilas": "afrontilar",
    "afrontilase": "afrontilar",
    "afrontilaseis": "afrontilar",
    "afrontilasen": "afrontilar",
    "afrontilases": "afrontilar",
    "afrontilaste": "afrontilar",
    "afrontilasteis": "afrontilar",
    "afrontile": "afrontilar",
    "afrontilemos": "afrontilar",
    "afrontilen": "afrontilar",
    "afrontiles": "afrontilar",
    "afrontilo": "afrontilar",
    "afrontilábamos": "afrontilar",
    "afrontiláis": "afrontilar",
    "afrontiláramos": "afrontilar",
    "afrontiláremos": "afrontilar",
    "afrontilásemos": "afrontilar",
    "afrontilé": "afrontilar",
    "afrontiléis": "afrontilar",
    "afrontiló": "afrontilar",
    "afronto": "afrontar",
    "afrontábamos": "afrontar",
    "afrontáis": "afrontar",
    "afrontáramos": "afrontar",
    "afrontáremos": "afrontar",
    "afrontásemos": "afrontar",
    "afronté": "afrontar",
    "afrontéis": "afrontar",
    "afrontó": "afrontar",
    "afrormosias": "afrormosia",
    "afros": "afro",
    "aftas": "afta",
    "aftoides": "aftoide",
    "aftosa": "aftoso",
    "aftosas": "aftoso",
    "aftosos": "aftoso",
    "afuella": "afollar",
    "afuellan": "afollar",
    "afuellas": "afollar",
    "afuelle": "afollar",
    "afuellen": "afollar",
    "afuelles": "afollar",
    "afuello": "afollar",
    "afuera": "aforar",
    "afueran": "aforar",
    "afueras": "aforar",
    "afuere": "aforar",
    "afueren": "aforar",
    "afueres": "aforar",
    "afuero": "aforar",
    "afuetea": "afuetear",
    "afueteaba": "afuetear",
    "afueteabais": "afuetear",
    "afueteaban": "afuetear",
    "afueteabas": "afuetear",
    "afuetead": "afuetear",
    "afueteada": "afuetear",
    "afueteadas": "afuetear",
    "afueteado": "afuetear",
    "afueteados": "afuetear",
    "afueteamos": "afuetear",
    "afuetean": "afuetear",
    "afueteando": "afuetear",
    "afueteara": "afuetear",
    "afuetearais": "afuetear",
    "afuetearan": "afuetear",
    "afuetearas": "afuetear",
    "afueteare": "afuetear",
    "afueteareis": "afuetear",
    "afuetearemos": "afuetear",
    "afuetearen": "afuetear",
    "afueteares": "afuetear",
    "afuetearon": "afuetear",
    "afueteará": "afuetear",
    "afuetearán": "afuetear",
    "afuetearás": "afuetear",
    "afuetearé": "afuetear",
    "afuetearéis": "afuetear",
    "afuetearía": "afuetear",
    "afuetearíais": "afuetear",
    "afuetearíamos": "afuetear",
    "afuetearían": "afuetear",
    "afuetearías": "afuetear",
    "afueteas": "afuetear",
    "afuetease": "afuetear",
    "afueteaseis": "afuetear",
    "afueteasen": "afuetear",
    "afueteases": "afuetear",
    "afueteaste": "afuetear",
    "afueteasteis": "afuetear",
    "afuetee": "afuetear",
    "afueteemos": "afuetear",
    "afueteen": "afuetear",
    "afuetees": "afuetear",
    "afueteo": "afuetear",
    "afueteábamos": "afuetear",
    "afueteáis": "afuetear",
    "afueteáramos": "afuetear",
    "afueteáremos": "afuetear",
    "afueteásemos": "afuetear",
    "afueteé": "afuetear",
    "afueteéis": "afuetear",
    "afueteó": "afuetear",
    "afufa": "afufar",
    "afufaba": "afufar",
    "afufabais": "afufar",
    "afufaban": "afufar",
    "afufabas": "afufar",
    "afufad": "afufar",
    "afufada": "afufar",
    "afufadas": "afufar",
    "afufado": "afufar",
    "afufados": "afufar",
    "afufamos": "afufar",
    "afufan": "afufar",
    "afufando": "afufar",
    "afufara": "afufar",
    "afufarais": "afufar",
    "afufaran": "afufar",
    "afufaras": "afufar",
    "afufare": "afufar",
    "afufareis": "afufar",
    "afufaremos": "afufar",
    "afufaren": "afufar",
    "afufares": "afufar",
    "afufaron": "afufar",
    "afufará": "afufar",
    "afufarán": "afufar",
    "afufarás": "afufar",
    "afufaré": "afufar",
    "afufaréis": "afufar",
    "afufaría": "afufar",
    "afufaríais": "afufar",
    "afufaríamos": "afufar",
    "afufarían": "afufar",
    "afufarías": "afufar",
    "afufas": "afufar",
    "afufase": "afufar",
    "afufaseis": "afufar",
    "afufasen": "afufar",
    "afufases": "afufar",
    "afufaste": "afufar",
    "afufasteis": "afufar",
    "afufe": "afufar",
    "afufemos": "afufar",
    "afufen": "afufar",
    "afufes": "afufar",
    "afufo": "afufar",
    "afufábamos": "afufar",
    "afufáis": "afufar",
    "afufáramos": "afufar",
    "afufáremos": "afufar",
    "afufásemos": "afufar",
    "afufé": "afufar",
    "afuféis": "afufar",
    "afufó": "afufar",
    "afustes": "afuste",
    "afutra": "afutrar",
    "afutraba": "afutrar",
    "afutrabais": "afutrar",
    "afutraban": "afutrar",
    "afutrabas": "afutrar",
    "afutrad": "afutrar",
    "afutrada": "afutrar",
    "afutradas": "afutrar",
    "afutrado": "afutrar",
    "afutrados": "afutrar",
    "afutramos": "afutrar",
    "afutran": "afutrar",
    "afutrando": "afutrar",
    "afutrara": "afutrar",
    "afutrarais": "afutrar",
    "afutraran": "afutrar",
    "afutraras": "afutrar",
    "afutrare": "afutrar",
    "afutrareis": "afutrar",
    "afutraremos": "afutrar",
    "afutraren": "afutrar",
    "afutrares": "afutrar",
    "afutraron": "afutrar",
    "afutrará": "afutrar",
    "afutrarán": "afutrar",
    "afutrarás": "afutrar",
    "afutraré": "afutrar",
    "afutraréis": "afutrar",
    "afutraría": "afutrar",
    "afutraríais": "afutrar",
    "afutraríamos": "afutrar",
    "afutrarían": "afutrar",
    "afutrarías": "afutrar",
    "afutras": "afutrar",
    "afutrase": "afutrar",
    "afutraseis": "afutrar",
    "afutrasen": "afutrar",
    "afutrases": "afutrar",
    "afutraste": "afutrar",
    "afutrasteis": "afutrar",
    "afutre": "afutrar",
    "afutremos": "afutrar",
    "afutren": "afutrar",
    "afutres": "afutrar",
    "afutro": "afutrar",
    "afutrábamos": "afutrar",
    "afutráis": "afutrar",
    "afutráramos": "afutrar",
    "afutráremos": "afutrar",
    "afutrásemos": "afutrar",
    "afutré": "afutrar",
    "afutréis": "afutrar",
    "afutró": "afutrar",
    "afásica": "afásico",
    "afásicas": "afásico",
    "afásicos": "afásico",
    "afónica": "afónico",
    "afónicas": "afónico",
    "afónicos": "afónico",
    "afótica": "afótico",
    "afóticas": "afótico",
    "afóticos": "afótico",
    "agacha": "agachar",
    "agachaba": "agachar",
    "agachabais": "agachar",
    "agachaban": "agachar",
    "agachabas": "agachar",
    "agachad": "agachar",
    "agachada": "agachar",
    "agachadas": "agachar",
    "agachadizas": "agachadiza",
    "agachado": "agachar",
    "agachados": "agachar",
    "agachamos": "agachar",
    "agachan": "agachar",
    "agachando": "agachar",
    "agachara": "agachar",
    "agacharais": "agachar",
    "agacharan": "agachar",
    "agacharas": "agachar",
    "agachare": "agachar",
    "agachareis": "agachar",
    "agacharemos": "agachar",
    "agacharen": "agachar",
    "agachares": "agachar",
    "agacharon": "agachar",
    "agachará": "agachar",
    "agacharán": "agachar",
    "agacharás": "agachar",
    "agacharé": "agachar",
    "agacharéis": "agachar",
    "agacharía": "agachar",
    "agacharíais": "agachar",
    "agacharíamos": "agachar",
    "agacharían": "agachar",
    "agacharías": "agachar",
    "agachas": "agachar",
    "agachase": "agachar",
    "agachaseis": "agachar",
    "agachasen": "agachar",
    "agachases": "agachar",
    "agachaste": "agachar",
    "agachasteis": "agachar",
    "agache": "agachar",
    "agachemos": "agachar",
    "agachen": "agachar",
    "agaches": "agachar",
    "agacho": "agachar",
    "agachábamos": "agachar",
    "agacháis": "agachar",
    "agacháramos": "agachar",
    "agacháremos": "agachar",
    "agachásemos": "agachar",
    "agaché": "agachar",
    "agachéis": "agachar",
    "agachó": "agachar",
    "agalactias": "agalactia",
    "agallas": "agalla",
    "agallones": "agallón",
    "agalmatolitas": "agalmatolita",
    "agametoblastos": "agametoblasto",
    "agametos": "agameto",
    "agamias": "agamia",
    "agamita": "agamitar",
    "agamitaba": "agamitar",
    "agamitabais": "agamitar",
    "agamitaban": "agamitar",
    "agamitabas": "agamitar",
    "agamitad": "agamitar",
    "agamitada": "agamitar",
    "agamitadas": "agamitar",
    "agamitado": "agamitar",
    "agamitados": "agamitar",
    "agamitamos": "agamitar",
    "agamitan": "agamitar",
    "agamitando": "agamitar",
    "agamitara": "agamitar",
    "agamitarais": "agamitar",
    "agamitaran": "agamitar",
    "agamitaras": "agamitar",
    "agamitare": "agamitar",
    "agamitareis": "agamitar",
    "agamitaremos": "agamitar",
    "agamitaren": "agamitar",
    "agamitares": "agamitar",
    "agamitaron": "agamitar",
    "agamitará": "agamitar",
    "agamitarán": "agamitar",
    "agamitarás": "agamitar",
    "agamitaré": "agamitar",
    "agamitaréis": "agamitar",
    "agamitaría": "agamitar",
    "agamitaríais": "agamitar",
    "agamitaríamos": "agamitar",
    "agamitarían": "agamitar",
    "agamitarías": "agamitar",
    "agamitas": "agamitar",
    "agamitase": "agamitar",
    "agamitaseis": "agamitar",
    "agamitasen": "agamitar",
    "agamitases": "agamitar",
    "agamitaste": "agamitar",
    "agamitasteis": "agamitar",
    "agamite": "agamitar",
    "agamitemos": "agamitar",
    "agamiten": "agamitar",
    "agamites": "agamitar",
    "agamito": "agamitar",
    "agamitábamos": "agamitar",
    "agamitáis": "agamitar",
    "agamitáramos": "agamitar",
    "agamitáremos": "agamitar",
    "agamitásemos": "agamitar",
    "agamité": "agamitar",
    "agamitéis": "agamitar",
    "agamitó": "agamitar",
    "agammaglobulinemias": "agammaglobulinemia",
    "agamíes": "agamí",
    "agamís": "agamí",
    "agapantos": "agapanto",
    "agaragares": "agaragar",
    "agarba": "agarbar",
    "agarbaba": "agarbar",
    "agarbabais": "agarbar",
    "agarbaban": "agarbar",
    "agarbabas": "agarbar",
    "agarbad": "agarbar",
    "agarbada": "agarbar",
    "agarbadas": "agarbar",
    "agarbado": "agarbar",
    "agarbados": "agarbar",
    "agarbamos": "agarbar",
    "agarban": "agarbar",
    "agarbando": "agarbar",
    "agarbara": "agarbar",
    "agarbarais": "agarbar",
    "agarbaran": "agarbar",
    "agarbaras": "agarbar",
    "agarbare": "agarbar",
    "agarbareis": "agarbar",
    "agarbaremos": "agarbar",
    "agarbaren": "agarbar",
    "agarbares": "agarbar",
    "agarbaron": "agarbar",
    "agarbará": "agarbar",
    "agarbarán": "agarbar",
    "agarbarás": "agarbar",
    "agarbaré": "agarbar",
    "agarbaréis": "agarbar",
    "agarbaría": "agarbar",
    "agarbaríais": "agarbar",
    "agarbaríamos": "agarbar",
    "agarbarían": "agarbar",
    "agarbarías": "agarbar",
    "agarbas": "agarbar",
    "agarbase": "agarbar",
    "agarbaseis": "agarbar",
    "agarbasen": "agarbar",
    "agarbases": "agarbar",
    "agarbaste": "agarbar",
    "agarbasteis": "agarbar",
    "agarbe": "agarbar",
    "agarbemos": "agarbar",
    "agarben": "agarbar",
    "agarbes": "agarbar",
    "agarbilla": "agarbillar",
    "agarbillaba": "agarbillar",
    "agarbillabais": "agarbillar",
    "agarbillaban": "agarbillar",
    "agarbillabas": "agarbillar",
    "agarbillad": "agarbillar",
    "agarbillada": "agarbillar",
    "agarbilladas": "agarbillar",
    "agarbillado": "agarbillar",
    "agarbillados": "agarbillar",
    "agarbillamos": "agarbillar",
    "agarbillan": "agarbillar",
    "agarbillando": "agarbillar",
    "agarbillara": "agarbillar",
    "agarbillarais": "agarbillar",
    "agarbillaran": "agarbillar",
    "agarbillaras": "agarbillar",
    "agarbillare": "agarbillar",
    "agarbillareis": "agarbillar",
    "agarbillaremos": "agarbillar",
    "agarbillaren": "agarbillar",
    "agarbillares": "agarbillar",
    "agarbillaron": "agarbillar",
    "agarbillará": "agarbillar",
    "agarbillarán": "agarbillar",
    "agarbillarás": "agarbillar",
    "agarbillaré": "agarbillar",
    "agarbillaréis": "agarbillar",
    "agarbillaría": "agarbillar",
    "agarbillaríais": "agarbillar",
    "agarbillaríamos": "agarbillar",
    "agarbillarían": "agarbillar",
    "agarbillarías": "agarbillar",
    "agarbillas": "agarbillar",
    "agarbillase": "agarbillar",
    "agarbillaseis": "agarbillar",
    "agarbillasen": "agarbillar",
    "agarbillases": "agarbillar",
    "agarbillaste": "agarbillar",
    "agarbillasteis": "agarbillar",
    "agarbille": "agarbillar",
    "agarbillemos": "agarbillar",
    "agarbillen": "agarbillar",
    "agarbilles": "agarbillar",
    "agarbillo": "agarbillar",
    "agarbillábamos": "agarbillar",
    "agarbilláis": "agarbillar",
    "agarbilláramos": "agarbillar",
    "agarbilláremos": "agarbillar",
    "agarbillásemos": "agarbillar",
    "agarbillé": "agarbillar",
    "agarbilléis": "agarbillar",
    "agarbilló": "agarbillar",
    "agarbo": "agarbar",
    "agarbábamos": "agarbar",
    "agarbáis": "agarbar",
    "agarbáramos": "agarbar",
    "agarbáremos": "agarbar",
    "agarbásemos": "agarbar",
    "agarbé": "agarbar",
    "agarbéis": "agarbar",
    "agarbó": "agarbar",
    "agarena": "agareno",
    "agarenas": "agareno",
    "agarenos": "agareno",
    "agares": "agar",
    "agaricáceas": "agaricácea",
    "agarosas": "agarosa",
    "agarra": "agarrar",
    "agarraba": "agarrar",
    "agarrabais": "agarrar",
    "agarraban": "agarrar",
    "agarrabas": "agarrar",
    "agarrad": "agarrar",
    "agarrada": "agarrar",
    "agarradas": "agarrar",
    "agarraderas": "agarradera",
    "agarraderos": "agarradero",
    "agarrado": "agarrar",
    "agarradores": "agarrador",
    "agarrados": "agarrar",
    "agarramos": "agarrar",
    "agarran": "agarrar",
    "agarrando": "agarrar",
    "agarrara": "agarrar",
    "agarrarais": "agarrar",
    "agarraran": "agarrar",
    "agarraras": "agarrar",
    "agarrare": "agarrar",
    "agarrareis": "agarrar",
    "agarraremos": "agarrar",
    "agarraren": "agarrar",
    "agarrares": "agarrar",
    "agarraron": "agarrar",
    "agarrará": "agarrar",
    "agarrarán": "agarrar",
    "agarrarás": "agarrar",
    "agarraré": "agarrar",
    "agarraréis": "agarrar",
    "agarraría": "agarrar",
    "agarraríais": "agarrar",
    "agarraríamos": "agarrar",
    "agarrarían": "agarrar",
    "agarrarías": "agarrar",
    "agarras": "agarrar",
    "agarrase": "agarrar",
    "agarraseis": "agarrar",
    "agarrasen": "agarrar",
    "agarrases": "agarrar",
    "agarraste": "agarrar",
    "agarrasteis": "agarrar",
    "agarre": "agarrar",
    "agarremos": "agarrar",
    "agarren": "agarrar",
    "agarres": "agarre",
    "agarro": "agarrar",
    "agarrocha": "agarrochar",
    "agarrochaba": "agarrochar",
    "agarrochabais": "agarrochar",
    "agarrochaban": "agarrochar",
    "agarrochabas": "agarrochar",
    "agarrochad": "agarrochar",
    "agarrochada": "agarrochar",
    "agarrochadas": "agarrochar",
    "agarrochado": "agarrochar",
    "agarrochados": "agarrochar",
    "agarrochamos": "agarrochar",
    "agarrochan": "agarrochar",
    "agarrochando": "agarrochar",
    "agarrochara": "agarrochar",
    "agarrocharais": "agarrochar",
    "agarrocharan": "agarrochar",
    "agarrocharas": "agarrochar",
    "agarrochare": "agarrochar",
    "agarrochareis": "agarrochar",
    "agarrocharemos": "agarrochar",
    "agarrocharen": "agarrochar",
    "agarrochares": "agarrochar",
    "agarrocharon": "agarrochar",
    "agarrochará": "agarrochar",
    "agarrocharán": "agarrochar",
    "agarrocharás": "agarrochar",
    "agarrocharé": "agarrochar",
    "agarrocharéis": "agarrochar",
    "agarrocharía": "agarrochar",
    "agarrocharíais": "agarrochar",
    "agarrocharíamos": "agarrochar",
    "agarrocharían": "agarrochar",
    "agarrocharías": "agarrochar",
    "agarrochas": "agarrochar",
    "agarrochase": "agarrochar",
    "agarrochaseis": "agarrochar",
    "agarrochasen": "agarrochar",
    "agarrochases": "agarrochar",
    "agarrochaste": "agarrochar",
    "agarrochasteis": "agarrochar",
    "agarroche": "agarrochar",
    "agarrochemos": "agarrochar",
    "agarrochen": "agarrochar",
    "agarroches": "agarrochar",
    "agarrocho": "agarrochar",
    "agarrochábamos": "agarrochar",
    "agarrocháis": "agarrochar",
    "agarrocháramos": "agarrochar",
    "agarrocháremos": "agarrochar",
    "agarrochásemos": "agarrochar",
    "agarroché": "agarrochar",
    "agarrochéis": "agarrochar",
    "agarrochó": "agarrochar",
    "agarrones": "agarrón",
    "agarrota": "agarrotar",
    "agarrotaba": "agarrotar",
    "agarrotabais": "agarrotar",
    "agarrotaban": "agarrotar",
    "agarrotabas": "agarrotar",
    "agarrotad": "agarrotar",
    "agarrotada": "agarrotar",
    "agarrotadas": "agarrotar",
    "agarrotado": "agarrotar",
    "agarrotados": "agarrotar",
    "agarrotamientos": "agarrotamiento",
    "agarrotamos": "agarrotar",
    "agarrotan": "agarrotar",
    "agarrotando": "agarrotar",
    "agarrotara": "agarrotar",
    "agarrotarais": "agarrotar",
    "agarrotaran": "agarrotar",
    "agarrotaras": "agarrotar",
    "agarrotare": "agarrotar",
    "agarrotareis": "agarrotar",
    "agarrotaremos": "agarrotar",
    "agarrotaren": "agarrotar",
    "agarrotares": "agarrotar",
    "agarrotaron": "agarrotar",
    "agarrotará": "agarrotar",
    "agarrotarán": "agarrotar",
    "agarrotarás": "agarrotar",
    "agarrotaré": "agarrotar",
    "agarrotaréis": "agarrotar",
    "agarrotaría": "agarrotar",
    "agarrotaríais": "agarrotar",
    "agarrotaríamos": "agarrotar",
    "agarrotarían": "agarrotar",
    "agarrotarías": "agarrotar",
    "agarrotas": "agarrotar",
    "agarrotase": "agarrotar",
    "agarrotaseis": "agarrotar",
    "agarrotasen": "agarrotar",
    "agarrotases": "agarrotar",
    "agarrotaste": "agarrotar",
    "agarrotasteis": "agarrotar",
    "agarrote": "agarrotar",
    "agarrotea": "agarrotear",
    "agarroteaba": "agarrotear",
    "agarroteabais": "agarrotear",
    "agarroteaban": "agarrotear",
    "agarroteabas": "agarrotear",
    "agarrotead": "agarrotear",
    "agarroteada": "agarrotear",
    "agarroteadas": "agarrotear",
    "agarroteado": "agarrotear",
    "agarroteados": "agarrotear",
    "agarroteamos": "agarrotear",
    "agarrotean": "agarrotear",
    "agarroteando": "agarrotear",
    "agarroteara": "agarrotear",
    "agarrotearais": "agarrotear",
    "agarrotearan": "agarrotear",
    "agarrotearas": "agarrotear",
    "agarroteare": "agarrotear",
    "agarroteareis": "agarrotear",
    "agarrotearemos": "agarrotear",
    "agarrotearen": "agarrotear",
    "agarroteares": "agarrotear",
    "agarrotearon": "agarrotear",
    "agarroteará": "agarrotear",
    "agarrotearán": "agarrotear",
    "agarrotearás": "agarrotear",
    "agarrotearé": "agarrotear",
    "agarrotearéis": "agarrotear",
    "agarrotearía": "agarrotear",
    "agarrotearíais": "agarrotear",
    "agarrotearíamos": "agarrotear",
    "agarrotearían": "agarrotear",
    "agarrotearías": "agarrotear",
    "agarroteas": "agarrotear",
    "agarrotease": "agarrotear",
    "agarroteaseis": "agarrotear",
    "agarroteasen": "agarrotear",
    "agarroteases": "agarrotear",
    "agarroteaste": "agarrotear",
    "agarroteasteis": "agarrotear",
    "agarrotee": "agarrotear",
    "agarroteemos": "agarrotear",
    "agarroteen": "agarrotear",
    "agarrotees": "agarrotear",
    "agarrotemos": "agarrotar",
    "agarroten": "agarrotar",
    "agarroteo": "agarrotear",
    "agarrotes": "agarrotar",
    "agarroteábamos": "agarrotear",
    "agarroteáis": "agarrotear",
    "agarroteáramos": "agarrotear",
    "agarroteáremos": "agarrotear",
    "agarroteásemos": "agarrotear",
    "agarroteé": "agarrotear",
    "agarroteéis": "agarrotear",
    "agarroteó": "agarrotear",
    "agarroto": "agarrotar",
    "agarrotábamos": "agarrotar",
    "agarrotáis": "agarrotar",
    "agarrotáramos": "agarrotar",
    "agarrotáremos": "agarrotar",
    "agarrotásemos": "agarrotar",
    "agarroté": "agarrotar",
    "agarrotéis": "agarrotar",
    "agarrotó": "agarrotar",
    "agarrábamos": "agarrar",
    "agarráis": "agarrar",
    "agarráramos": "agarrar",
    "agarráremos": "agarrar",
    "agarrásemos": "agarrar",
    "agarré": "agarrar",
    "agarréis": "agarrar",
    "agarró": "agarrar",
    "agas": "aga",
    "agasaja": "agasajar",
    "agasajaba": "agasajar",
    "agasajabais": "agasajar",
    "agasajaban": "agasajar",
    "agasajabas": "agasajar",
    "agasajad": "agasajar",
    "agasajada": "agasajar",
    "agasajadas": "agasajar",
    "agasajado": "agasajar",
    "agasajados": "agasajar",
    "agasajamos": "agasajar",
    "agasajan": "agasajar",
    "agasajando": "agasajar",
    "agasajara": "agasajar",
    "agasajarais": "agasajar",
    "agasajaran": "agasajar",
    "agasajaras": "agasajar",
    "agasajare": "agasajar",
    "agasajareis": "agasajar",
    "agasajaremos": "agasajar",
    "agasajaren": "agasajar",
    "agasajares": "agasajar",
    "agasajaron": "agasajar",
    "agasajará": "agasajar",
    "agasajarán": "agasajar",
    "agasajarás": "agasajar",
    "agasajaré": "agasajar",
    "agasajaréis": "agasajar",
    "agasajaría": "agasajar",
    "agasajaríais": "agasajar",
    "agasajaríamos": "agasajar",
    "agasajarían": "agasajar",
    "agasajarías": "agasajar",
    "agasajas": "agasajar",
    "agasajase": "agasajar",
    "agasajaseis": "agasajar",
    "agasajasen": "agasajar",
    "agasajases": "agasajar",
    "agasajaste": "agasajar",
    "agasajasteis": "agasajar",
    "agasaje": "agasajar",
    "agasajemos": "agasajar",
    "agasajen": "agasajar",
    "agasajes": "agasajar",
    "agasajo": "agasajar",
    "agasajos": "agasajo",
    "agasajábamos": "agasajar",
    "agasajáis": "agasajar",
    "agasajáramos": "agasajar",
    "agasajáremos": "agasajar",
    "agasajásemos": "agasajar",
    "agasajé": "agasajar",
    "agasajéis": "agasajar",
    "agasajó": "agasajar",
    "agateadores": "agateador",
    "agavanzos": "agavanzo",
    "agavilla": "agavillar",
    "agavillaba": "agavillar",
    "agavillabais": "agavillar",
    "agavillaban": "agavillar",
    "agavillabas": "agavillar",
    "agavillad": "agavillar",
    "agavillada": "agavillar",
    "agavilladas": "agavillar",
    "agavillado": "agavillar",
    "agavilladora": "agavillador",
    "agavilladoras": "agavillador",
    "agavilladores": "agavillador",
    "agavillados": "agavillar",
    "agavillamos": "agavillar",
    "agavillan": "agavillar",
    "agavillando": "agavillar",
    "agavillara": "agavillar",
    "agavillarais": "agavillar",
    "agavillaran": "agavillar",
    "agavillaras": "agavillar",
    "agavillare": "agavillar",
    "agavillareis": "agavillar",
    "agavillaremos": "agavillar",
    "agavillaren": "agavillar",
    "agavillares": "agavillar",
    "agavillaron": "agavillar",
    "agavillará": "agavillar",
    "agavillarán": "agavillar",
    "agavillarás": "agavillar",
    "agavillaré": "agavillar",
    "agavillaréis": "agavillar",
    "agavillaría": "agavillar",
    "agavillaríais": "agavillar",
    "agavillaríamos": "agavillar",
    "agavillarían": "agavillar",
    "agavillarías": "agavillar",
    "agavillas": "agavillar",
    "agavillase": "agavillar",
    "agavillaseis": "agavillar",
    "agavillasen": "agavillar",
    "agavillases": "agavillar",
    "agavillaste": "agavillar",
    "agavillasteis": "agavillar",
    "agaville": "agavillar",
    "agavillemos": "agavillar",
    "agavillen": "agavillar",
    "agavilles": "agavillar",
    "agavillo": "agavillar",
    "agavillábamos": "agavillar",
    "agavilláis": "agavillar",
    "agavilláramos": "agavillar",
    "agavilláremos": "agavillar",
    "agavillásemos": "agavillar",
    "agavillé": "agavillar",
    "agavilléis": "agavillar",
    "agavilló": "agavillar",
    "agazapa": "agazapar",
    "agazapaba": "agazapar",
    "agazapabais": "agazapar",
    "agazapaban": "agazapar",
    "agazapabas": "agazapar",
    "agazapad": "agazapar",
    "agazapada": "agazapar",
    "agazapadas": "agazapar",
    "agazapado": "agazapar",
    "agazapados": "agazapar",
    "agazapamos": "agazapar",
    "agazapan": "agazapar",
    "agazapando": "agazapar",
    "agazapara": "agazapar",
    "agazaparais": "agazapar",
    "agazaparan": "agazapar",
    "agazaparas": "agazapar",
    "agazapare": "agazapar",
    "agazapareis": "agazapar",
    "agazaparemos": "agazapar",
    "agazaparen": "agazapar",
    "agazapares": "agazapar",
    "agazaparon": "agazapar",
    "agazapará": "agazapar",
    "agazaparán": "agazapar",
    "agazaparás": "agazapar",
    "agazaparé": "agazapar",
    "agazaparéis": "agazapar",
    "agazaparía": "agazapar",
    "agazaparíais": "agazapar",
    "agazaparíamos": "agazapar",
    "agazaparían": "agazapar",
    "agazaparías": "agazapar",
    "agazapas": "agazapar",
    "agazapase": "agazapar",
    "agazapaseis": "agazapar",
    "agazapasen": "agazapar",
    "agazapases": "agazapar",
    "agazapaste": "agazapar",
    "agazapasteis": "agazapar",
    "agazape": "agazapar",
    "agazapemos": "agazapar",
    "agazapen": "agazapar",
    "agazapes": "agazapar",
    "agazapo": "agazapar",
    "agazapábamos": "agazapar",
    "agazapáis": "agazapar",
    "agazapáramos": "agazapar",
    "agazapáremos": "agazapar",
    "agazapásemos": "agazapar",
    "agazapé": "agazapar",
    "agazapéis": "agazapar",
    "agazapó": "agazapar",
    "agencia": "agenciar",
    "agenciaba": "agenciar",
    "agenciabais": "agenciar",
    "agenciaban": "agenciar",
    "agenciabas": "agenciar",
    "agenciad": "agenciar",
    "agenciada": "agenciar",
    "agenciadas": "agenciar",
    "agenciado": "agenciar",
    "agenciados": "agenciar",
    "agenciamos": "agenciar",
    "agencian": "agenciar",
    "agenciando": "agenciar",
    "agenciara": "agenciar",
    "agenciarais": "agenciar",
    "agenciaran": "agenciar",
    "agenciaras": "agenciar",
    "agenciare": "agenciar",
    "agenciareis": "agenciar",
    "agenciaremos": "agenciar",
    "agenciaren": "agenciar",
    "agenciares": "agenciar",
    "agenciaron": "agenciar",
    "agenciará": "agenciar",
    "agenciarán": "agenciar",
    "agenciarás": "agenciar",
    "agenciaré": "agenciar",
    "agenciaréis": "agenciar",
    "agenciaría": "agenciar",
    "agenciaríais": "agenciar",
    "agenciaríamos": "agenciar",
    "agenciarían": "agenciar",
    "agenciarías": "agenciar",
    "agencias": "agenciar",
    "agenciase": "agenciar",
    "agenciaseis": "agenciar",
    "agenciasen": "agenciar",
    "agenciases": "agenciar",
    "agenciaste": "agenciar",
    "agenciasteis": "agenciar",
    "agencie": "agenciar",
    "agenciemos": "agenciar",
    "agencien": "agenciar",
    "agencies": "agenciar",
    "agencio": "agenciar",
    "agenciábamos": "agenciar",
    "agenciáis": "agenciar",
    "agenciáramos": "agenciar",
    "agenciáremos": "agenciar",
    "agenciásemos": "agenciar",
    "agencié": "agenciar",
    "agenciéis": "agenciar",
    "agenció": "agenciar",
    "agendas": "agenda",
    "agenesias": "agenesia",
    "agentes": "agente",
    "agentiva": "agentivo",
    "agentivas": "agentivo",
    "agentivos": "agentivo",
    "agenésica": "agenésico",
    "agenésicas": "agenésico",
    "agenésicos": "agenésico",
    "agerasias": "agerasia",
    "agermana": "agermanar",
    "agermanaba": "agermanar",
    "agermanabais": "agermanar",
    "agermanaban": "agermanar",
    "agermanabas": "agermanar",
    "agermanad": "agermanar",
    "agermanada": "agermanar",
    "agermanadas": "agermanar",
    "agermanado": "agermanar",
    "agermanados": "agermanar",
    "agermanamos": "agermanar",
    "agermanan": "agermanar",
    "agermanando": "agermanar",
    "agermanara": "agermanar",
    "agermanarais": "agermanar",
    "agermanaran": "agermanar",
    "agermanaras": "agermanar",
    "agermanare": "agermanar",
    "agermanareis": "agermanar",
    "agermanaremos": "agermanar",
    "agermanaren": "agermanar",
    "agermanares": "agermanar",
    "agermanaron": "agermanar",
    "agermanará": "agermanar",
    "agermanarán": "agermanar",
    "agermanarás": "agermanar",
    "agermanaré": "agermanar",
    "agermanaréis": "agermanar",
    "agermanaría": "agermanar",
    "agermanaríais": "agermanar",
    "agermanaríamos": "agermanar",
    "agermanarían": "agermanar",
    "agermanarías": "agermanar",
    "agermanas": "agermanar",
    "agermanase": "agermanar",
    "agermanaseis": "agermanar",
    "agermanasen": "agermanar",
    "agermanases": "agermanar",
    "agermanaste": "agermanar",
    "agermanasteis": "agermanar",
    "agermane": "agermanar",
    "agermanemos": "agermanar",
    "agermanen": "agermanar",
    "agermanes": "agermanar",
    "agermano": "agermanar",
    "agermanábamos": "agermanar",
    "agermanáis": "agermanar",
    "agermanáramos": "agermanar",
    "agermanáremos": "agermanar",
    "agermanásemos": "agermanar",
    "agermané": "agermanar",
    "agermanéis": "agermanar",
    "agermanó": "agermanar",
    "ageusias": "ageusia",
    "ageustias": "ageustia",
    "aggiornamentos": "aggiornamento",
    "agiadas": "agiada",
    "agiganta": "agigantar",
    "agigantaba": "agigantar",
    "agigantabais": "agigantar",
    "agigantaban": "agigantar",
    "agigantabas": "agigantar",
    "agigantad": "agigantar",
    "agigantada": "agigantar",
    "agigantadas": "agigantar",
    "agigantado": "agigantar",
    "agigantados": "agigantar",
    "agigantamos": "agigantar",
    "agigantan": "agigantar",
    "agigantando": "agigantar",
    "agigantara": "agigantar",
    "agigantarais": "agigantar",
    "agigantaran": "agigantar",
    "agigantaras": "agigantar",
    "agigantare": "agigantar",
    "agigantareis": "agigantar",
    "agigantaremos": "agigantar",
    "agigantaren": "agigantar",
    "agigantares": "agigantar",
    "agigantaron": "agigantar",
    "agigantará": "agigantar",
    "agigantarán": "agigantar",
    "agigantarás": "agigantar",
    "agigantaré": "agigantar",
    "agigantaréis": "agigantar",
    "agigantaría": "agigantar",
    "agigantaríais": "agigantar",
    "agigantaríamos": "agigantar",
    "agigantarían": "agigantar",
    "agigantarías": "agigantar",
    "agigantas": "agigantar",
    "agigantase": "agigantar",
    "agigantaseis": "agigantar",
    "agigantasen": "agigantar",
    "agigantases": "agigantar",
    "agigantaste": "agigantar",
    "agigantasteis": "agigantar",
    "agigante": "agigantar",
    "agigantemos": "agigantar",
    "agiganten": "agigantar",
    "agigantes": "agigantar",
    "agiganto": "agigantar",
    "agigantábamos": "agigantar",
    "agigantáis": "agigantar",
    "agigantáramos": "agigantar",
    "agigantáremos": "agigantar",
    "agigantásemos": "agigantar",
    "agiganté": "agigantar",
    "agigantéis": "agigantar",
    "agigantó": "agigantar",
    "agilice": "agilizar",
    "agilicemos": "agilizar",
    "agilicen": "agilizar",
    "agilices": "agilizar",
    "agilicé": "agilizar",
    "agilicéis": "agilizar",
    "agilidades": "agilidad",
    "agiliza": "agilizar",
    "agilizaba": "agilizar",
    "agilizabais": "agilizar",
    "agilizaban": "agilizar",
    "agilizabas": "agilizar",
    "agilizaciones": "agilización",
    "agilizad": "agilizar",
    "agilizada": "agilizar",
    "agilizadas": "agilizar",
    "agilizado": "agilizar",
    "agilizados": "agilizar",
    "agilizamos": "agilizar",
    "agilizan": "agilizar",
    "agilizando": "agilizar",
    "agilizara": "agilizar",
    "agilizarais": "agilizar",
    "agilizaran": "agilizar",
    "agilizaras": "agilizar",
    "agilizare": "agilizar",
    "agilizareis": "agilizar",
    "agilizaremos": "agilizar",
    "agilizaren": "agilizar",
    "agilizares": "agilizar",
    "agilizaron": "agilizar",
    "agilizará": "agilizar",
    "agilizarán": "agilizar",
    "agilizarás": "agilizar",
    "agilizaré": "agilizar",
    "agilizaréis": "agilizar",
    "agilizaría": "agilizar",
    "agilizaríais": "agilizar",
    "agilizaríamos": "agilizar",
    "agilizarían": "agilizar",
    "agilizarías": "agilizar",
    "agilizas": "agilizar",
    "agilizase": "agilizar",
    "agilizaseis": "agilizar",
    "agilizasen": "agilizar",
    "agilizases": "agilizar",
    "agilizaste": "agilizar",
    "agilizasteis": "agilizar",
    "agilizo": "agilizar",
    "agilizábamos": "agilizar",
    "agilizáis": "agilizar",
    "agilizáramos": "agilizar",
    "agilizáremos": "agilizar",
    "agilizásemos": "agilizar",
    "agilizó": "agilizar",
    "agios": "agio",
    "agiotadora": "agiotador",
    "agiotadoras": "agiotador",
    "agiotadores": "agiotador",
    "agiotajes": "agiotaje",
    "agiotistas": "agiotista",
    "agita": "agitar",
    "agitaba": "agitar",
    "agitabais": "agitar",
    "agitaban": "agitar",
    "agitabas": "agitar",
    "agitaciones": "agitación",
    "agitad": "agitar",
    "agitada": "agitar",
    "agitadas": "agitar",
    "agitado": "agitar",
    "agitadora": "agitador",
    "agitadoras": "agitador",
    "agitadores": "agitador",
    "agitados": "agitar",
    "agitamos": "agitar",
    "agitan": "agitar",
    "agitanada": "agitanado",
    "agitanadas": "agitanado",
    "agitanados": "agitanado",
    "agitando": "agitar",
    "agitara": "agitar",
    "agitarais": "agitar",
    "agitaran": "agitar",
    "agitaras": "agitar",
    "agitare": "agitar",
    "agitareis": "agitar",
    "agitaremos": "agitar",
    "agitaren": "agitar",
    "agitares": "agitar",
    "agitaron": "agitar",
    "agitará": "agitar",
    "agitarán": "agitar",
    "agitarás": "agitar",
    "agitaré": "agitar",
    "agitaréis": "agitar",
    "agitaría": "agitar",
    "agitaríais": "agitar",
    "agitaríamos": "agitar",
    "agitarían": "agitar",
    "agitarías": "agitar",
    "agitas": "agitar",
    "agitase": "agitar",
    "agitaseis": "agitar",
    "agitasen": "agitar",
    "agitases": "agitar",
    "agitaste": "agitar",
    "agitasteis": "agitar",
    "agite": "agitar",
    "agitemos": "agitar",
    "agiten": "agitar",
    "agites": "agitar",
    "agito": "agitar",
    "agitábamos": "agitar",
    "agitáis": "agitar",
    "agitáramos": "agitar",
    "agitáremos": "agitar",
    "agitásemos": "agitar",
    "agité": "agitar",
    "agitéis": "agitar",
    "agitó": "agitar",
    "aglabíes": "aglabí",
    "aglabís": "aglabí",
    "aglicosúrica": "aglicosúrico",
    "aglicosúricas": "aglicosúrico",
    "aglicosúricos": "aglicosúrico",
    "aglifos": "aglifo",
    "aglomera": "aglomerar",
    "aglomeraba": "aglomerar",
    "aglomerabais": "aglomerar",
    "aglomeraban": "aglomerar",
    "aglomerabas": "aglomerar",
    "aglomeraciones": "aglomeración",
    "aglomerad": "aglomerar",
    "aglomerada": "aglomerar",
    "aglomeradas": "aglomerar",
    "aglomerado": "aglomerar",
    "aglomerados": "aglomerar",
    "aglomeramos": "aglomerar",
    "aglomeran": "aglomerar",
    "aglomerando": "aglomerar",
    "aglomerantes": "aglomerante",
    "aglomerara": "aglomerar",
    "aglomerarais": "aglomerar",
    "aglomeraran": "aglomerar",
    "aglomeraras": "aglomerar",
    "aglomerare": "aglomerar",
    "aglomerareis": "aglomerar",
    "aglomeraremos": "aglomerar",
    "aglomeraren": "aglomerar",
    "aglomerares": "aglomerar",
    "aglomeraron": "aglomerar",
    "aglomerará": "aglomerar",
    "aglomerarán": "aglomerar",
    "aglomerarás": "aglomerar",
    "aglomeraré": "aglomerar",
    "aglomeraréis": "aglomerar",
    "aglomeraría": "aglomerar",
    "aglomeraríais": "aglomerar",
    "aglomeraríamos": "aglomerar",
    "aglomerarían": "aglomerar",
    "aglomerarías": "aglomerar",
    "aglomeras": "aglomerar",
    "aglomerase": "aglomerar",
    "aglomeraseis": "aglomerar",
    "aglomerasen": "aglomerar",
    "aglomerases": "aglomerar",
    "aglomeraste": "aglomerar",
    "aglomerasteis": "aglomerar",
    "aglomere": "aglomerar",
    "aglomeremos": "aglomerar",
    "aglomeren": "aglomerar",
    "aglomeres": "aglomerar",
    "aglomero": "aglomerar",
    "aglomerábamos": "aglomerar",
    "aglomeráis": "aglomerar",
    "aglomeráramos": "aglomerar",
    "aglomeráremos": "aglomerar",
    "aglomerásemos": "aglomerar",
    "aglomeré": "aglomerar",
    "aglomeréis": "aglomerar",
    "aglomeró": "aglomerar",
    "aglosias": "aglosia",
    "aglosostomías": "aglosostomía",
    "agluitininas": "agluitinina",
    "aglutina": "aglutinar",
    "aglutinaba": "aglutinar",
    "aglutinabais": "aglutinar",
    "aglutinaban": "aglutinar",
    "aglutinabas": "aglutinar",
    "aglutinaciones": "aglutinación",
    "aglutinad": "aglutinar",
    "aglutinada": "aglutinar",
    "aglutinadas": "aglutinar",
    "aglutinado": "aglutinar",
    "aglutinadora": "aglutinador",
    "aglutinadoras": "aglutinador",
    "aglutinadores": "aglutinador",
    "aglutinados": "aglutinar",
    "aglutinamientos": "aglutinamiento",
    "aglutinamos": "aglutinar",
    "aglutinan": "aglutinar",
    "aglutinando": "aglutinar",
    "aglutinantes": "aglutinante",
    "aglutinara": "aglutinar",
    "aglutinarais": "aglutinar",
    "aglutinaran": "aglutinar",
    "aglutinaras": "aglutinar",
    "aglutinare": "aglutinar",
    "aglutinareis": "aglutinar",
    "aglutinaremos": "aglutinar",
    "aglutinaren": "aglutinar",
    "aglutinares": "aglutinar",
    "aglutinaron": "aglutinar",
    "aglutinará": "aglutinar",
    "aglutinarán": "aglutinar",
    "aglutinarás": "aglutinar",
    "aglutinaré": "aglutinar",
    "aglutinaréis": "aglutinar",
    "aglutinaría": "aglutinar",
    "aglutinaríais": "aglutinar",
    "aglutinaríamos": "aglutinar",
    "aglutinarían": "aglutinar",
    "aglutinarías": "aglutinar",
    "aglutinas": "aglutinar",
    "aglutinase": "aglutinar",
    "aglutinaseis": "aglutinar",
    "aglutinasen": "aglutinar",
    "aglutinases": "aglutinar",
    "aglutinaste": "aglutinar",
    "aglutinasteis": "aglutinar",
    "aglutinativa": "aglutinativo",
    "aglutinativas": "aglutinativo",
    "aglutinativos": "aglutinativo",
    "aglutine": "aglutinar",
    "aglutinemos": "aglutinar",
    "aglutinen": "aglutinar",
    "aglutines": "aglutinar",
    "aglutininas": "aglutinina",
    "aglutino": "aglutinar",
    "aglutinábamos": "aglutinar",
    "aglutináis": "aglutinar",
    "aglutináramos": "aglutinar",
    "aglutináremos": "aglutinar",
    "aglutinásemos": "aglutinar",
    "aglutiné": "aglutinar",
    "aglutinéis": "aglutinar",
    "aglutinó": "aglutinar",
    "aglutinógenos": "aglutinógeno",
    "agnacias": "agnacia",
    "agnaciones": "agnación",
    "agnada": "agnado",
    "agnadas": "agnado",
    "agnados": "agnado",
    "agnatias": "agnatia",
    "agnaticia": "agnaticio",
    "agnaticias": "agnaticio",
    "agnaticios": "agnaticio",
    "agnatos": "agnato",
    "agnogénica": "agnogénico",
    "agnogénicas": "agnogénico",
    "agnogénicos": "agnogénico",
    "agnosias": "agnosia",
    "agnosticismos": "agnosticismo",
    "agnusdéis": "agnusdéi",
    "agnómenes": "agnomen",
    "agnóstica": "agnóstico",
    "agnósticas": "agnóstico",
    "agnósticos": "agnóstico",
    "ago.": "agosto",
    "agobia": "agobiar",
    "agobiaba": "agobiar",
    "agobiabais": "agobiar",
    "agobiaban": "agobiar",
    "agobiabas": "agobiar",
    "agobiad": "agobiar",
    "agobiada": "agobiar",
    "agobiadas": "agobiar",
    "agobiado": "agobiar",
    "agobiadora": "agobiador",
    "agobiadoras": "agobiador",
    "agobiadores": "agobiador",
    "agobiados": "agobiar",
    "agobiamos": "agobiar",
    "agobian": "agobiar",
    "agobiando": "agobiar",
    "agobiantes": "agobiante",
    "agobiara": "agobiar",
    "agobiarais": "agobiar",
    "agobiaran": "agobiar",
    "agobiaras": "agobiar",
    "agobiare": "agobiar",
    "agobiareis": "agobiar",
    "agobiaremos": "agobiar",
    "agobiaren": "agobiar",
    "agobiares": "agobiar",
    "agobiaron": "agobiar",
    "agobiará": "agobiar",
    "agobiarán": "agobiar",
    "agobiarás": "agobiar",
    "agobiaré": "agobiar",
    "agobiaréis": "agobiar",
    "agobiaría": "agobiar",
    "agobiaríais": "agobiar",
    "agobiaríamos": "agobiar",
    "agobiarían": "agobiar",
    "agobiarías": "agobiar",
    "agobias": "agobiar",
    "agobiase": "agobiar",
    "agobiaseis": "agobiar",
    "agobiasen": "agobiar",
    "agobiases": "agobiar",
    "agobiaste": "agobiar",
    "agobiasteis": "agobiar",
    "agobie": "agobiar",
    "agobiemos": "agobiar",
    "agobien": "agobiar",
    "agobies": "agobiar",
    "agobio": "agobiar",
    "agobios": "agobio",
    "agobiábamos": "agobiar",
    "agobiáis": "agobiar",
    "agobiáramos": "agobiar",
    "agobiáremos": "agobiar",
    "agobiásemos": "agobiar",
    "agobié": "agobiar",
    "agobiéis": "agobiar",
    "agobió": "agobiar",
    "agogías": "agogía",
    "agolpa": "agolpar",
    "agolpaba": "agolpar",
    "agolpabais": "agolpar",
    "agolpaban": "agolpar",
    "agolpabas": "agolpar",
    "agolpad": "agolpar",
    "agolpada": "agolpar",
    "agolpadas": "agolpar",
    "agolpado": "agolpar",
    "agolpados": "agolpar",
    "agolpamientos": "agolpamiento",
    "agolpamos": "agolpar",
    "agolpan": "agolpar",
    "agolpando": "agolpar",
    "agolpara": "agolpar",
    "agolparais": "agolpar",
    "agolparan": "agolpar",
    "agolparas": "agolpar",
    "agolpare": "agolpar",
    "agolpareis": "agolpar",
    "agolparemos": "agolpar",
    "agolparen": "agolpar",
    "agolpares": "agolpar",
    "agolparon": "agolpar",
    "agolpará": "agolpar",
    "agolparán": "agolpar",
    "agolparás": "agolpar",
    "agolparé": "agolpar",
    "agolparéis": "agolpar",
    "agolparía": "agolpar",
    "agolparíais": "agolpar",
    "agolparíamos": "agolpar",
    "agolparían": "agolpar",
    "agolparías": "agolpar",
    "agolpas": "agolpar",
    "agolpase": "agolpar",
    "agolpaseis": "agolpar",
    "agolpasen": "agolpar",
    "agolpases": "agolpar",
    "agolpaste": "agolpar",
    "agolpasteis": "agolpar",
    "agolpe": "agolpar",
    "agolpemos": "agolpar",
    "agolpen": "agolpar",
    "agolpes": "agolpar",
    "agolpo": "agolpar",
    "agolpábamos": "agolpar",
    "agolpáis": "agolpar",
    "agolpáramos": "agolpar",
    "agolpáremos": "agolpar",
    "agolpásemos": "agolpar",
    "agolpé": "agolpar",
    "agolpéis": "agolpar",
    "agolpó": "agolpar",
    "agonales": "agonal",
    "agonice": "agonizar",
    "agonicemos": "agonizar",
    "agonicen": "agonizar",
    "agonices": "agonizar",
    "agonicé": "agonizar",
    "agonicéis": "agonizar",
    "agonistas": "agonista",
    "agoniza": "agonizar",
    "agonizaba": "agonizar",
    "agonizabais": "agonizar",
    "agonizaban": "agonizar",
    "agonizabas": "agonizar",
    "agonizad": "agonizar",
    "agonizada": "agonizar",
    "agonizadas": "agonizar",
    "agonizado": "agonizar",
    "agonizados": "agonizar",
    "agonizamos": "agonizar",
    "agonizan": "agonizar",
    "agonizando": "agonizar",
    "agonizantes": "agonizante",
    "agonizara": "agonizar",
    "agonizarais": "agonizar",
    "agonizaran": "agonizar",
    "agonizaras": "agonizar",
    "agonizare": "agonizar",
    "agonizareis": "agonizar",
    "agonizaremos": "agonizar",
    "agonizaren": "agonizar",
    "agonizares": "agonizar",
    "agonizaron": "agonizar",
    "agonizará": "agonizar",
    "agonizarán": "agonizar",
    "agonizarás": "agonizar",
    "agonizaré": "agonizar",
    "agonizaréis": "agonizar",
    "agonizaría": "agonizar",
    "agonizaríais": "agonizar",
    "agonizaríamos": "agonizar",
    "agonizarían": "agonizar",
    "agonizarías": "agonizar",
    "agonizas": "agonizar",
    "agonizase": "agonizar",
    "agonizaseis": "agonizar",
    "agonizasen": "agonizar",
    "agonizases": "agonizar",
    "agonizaste": "agonizar",
    "agonizasteis": "agonizar",
    "agonizo": "agonizar",
    "agonizábamos": "agonizar",
    "agonizáis": "agonizar",
    "agonizáramos": "agonizar",
    "agonizáremos": "agonizar",
    "agonizásemos": "agonizar",
    "agonizó": "agonizar",
    "agonías": "agonía",
    "agonística": "agonístico",
    "agonísticas": "agonístico",
    "agonísticos": "agonístico",
    "agoraba": "agorar",
    "agorabais": "agorar",
    "agoraban": "agorar",
    "agorabas": "agorar",
    "agorad": "agorar",
    "agorada": "agorar",
    "agoradas": "agorar",
    "agorado": "agorar",
    "agorados": "agorar",
    "agorafobias": "agorafobia",
    "agoramos": "agorar",
    "agorando": "agorar",
    "agorara": "agorar",
    "agorarais": "agorar",
    "agoraran": "agorar",
    "agoraras": "agorar",
    "agorare": "agorar",
    "agorareis": "agorar",
    "agoraremos": "agorar",
    "agoraren": "agorar",
    "agorares": "agorar",
    "agoraron": "agorar",
    "agorará": "agorar",
    "agorarán": "agorar",
    "agorarás": "agorar",
    "agoraré": "agorar",
    "agoraréis": "agorar",
    "agoraría": "agorar",
    "agoraríais": "agorar",
    "agoraríamos": "agorar",
    "agorarían": "agorar",
    "agorarías": "agorar",
    "agorase": "agorar",
    "agoraseis": "agorar",
    "agorasen": "agorar",
    "agorases": "agorar",
    "agoraste": "agorar",
    "agorasteis": "agorar",
    "agoremos": "agorar",
    "agorera": "agorero",
    "agoreras": "agorero",
    "agoreros": "agorero",
    "agorgoja": "agorgojar",
    "agorgojaba": "agorgojar",
    "agorgojabais": "agorgojar",
    "agorgojaban": "agorgojar",
    "agorgojabas": "agorgojar",
    "agorgojad": "agorgojar",
    "agorgojada": "agorgojar",
    "agorgojadas": "agorgojar",
    "agorgojado": "agorgojar",
    "agorgojados": "agorgojar",
    "agorgojamos": "agorgojar",
    "agorgojan": "agorgojar",
    "agorgojando": "agorgojar",
    "agorgojara": "agorgojar",
    "agorgojarais": "agorgojar",
    "agorgojaran": "agorgojar",
    "agorgojaras": "agorgojar",
    "agorgojare": "agorgojar",
    "agorgojareis": "agorgojar",
    "agorgojaremos": "agorgojar",
    "agorgojaren": "agorgojar",
    "agorgojares": "agorgojar",
    "agorgojaron": "agorgojar",
    "agorgojará": "agorgojar",
    "agorgojarán": "agorgojar",
    "agorgojarás": "agorgojar",
    "agorgojaré": "agorgojar",
    "agorgojaréis": "agorgojar",
    "agorgojaría": "agorgojar",
    "agorgojaríais": "agorgojar",
    "agorgojaríamos": "agorgojar",
    "agorgojarían": "agorgojar",
    "agorgojarías": "agorgojar",
    "agorgojas": "agorgojar",
    "agorgojase": "agorgojar",
    "agorgojaseis": "agorgojar",
    "agorgojasen": "agorgojar",
    "agorgojases": "agorgojar",
    "agorgojaste": "agorgojar",
    "agorgojasteis": "agorgojar",
    "agorgoje": "agorgojar",
    "agorgojemos": "agorgojar",
    "agorgojen": "agorgojar",
    "agorgojes": "agorgojar",
    "agorgojo": "agorgojar",
    "agorgojábamos": "agorgojar",
    "agorgojáis": "agorgojar",
    "agorgojáramos": "agorgojar",
    "agorgojáremos": "agorgojar",
    "agorgojásemos": "agorgojar",
    "agorgojé": "agorgojar",
    "agorgojéis": "agorgojar",
    "agorgojó": "agorgojar",
    "agorábamos": "agorar",
    "agoráis": "agorar",
    "agoráramos": "agorar",
    "agoráremos": "agorar",
    "agorásemos": "agorar",
    "agoré": "agorar",
    "agoréis": "agorar",
    "agoró": "agorar",
    "agosta": "agostar",
    "agostaba": "agostar",
    "agostabais": "agostar",
    "agostaban": "agostar",
    "agostabas": "agostar",
    "agostad": "agostar",
    "agostada": "agostar",
    "agostadas": "agostar",
    "agostaderos": "agostadero",
    "agostado": "agostar",
    "agostadora": "agostador",
    "agostadoras": "agostador",
    "agostadores": "agostador",
    "agostados": "agostar",
    "agostamientos": "agostamiento",
    "agostamos": "agostar",
    "agostan": "agostar",
    "agostando": "agostar",
    "agostara": "agostar",
    "agostarais": "agostar",
    "agostaran": "agostar",
    "agostaras": "agostar",
    "agostare": "agostar",
    "agostareis": "agostar",
    "agostaremos": "agostar",
    "agostaren": "agostar",
    "agostares": "agostar",
    "agostaron": "agostar",
    "agostará": "agostar",
    "agostarán": "agostar",
    "agostarás": "agostar",
    "agostaré": "agostar",
    "agostaréis": "agostar",
    "agostaría": "agostar",
    "agostaríais": "agostar",
    "agostaríamos": "agostar",
    "agostarían": "agostar",
    "agostarías": "agostar",
    "agostas": "agostar",
    "agostase": "agostar",
    "agostaseis": "agostar",
    "agostasen": "agostar",
    "agostases": "agostar",
    "agostaste": "agostar",
    "agostasteis": "agostar",
    "agoste": "agostar",
    "agostemos": "agostar",
    "agosten": "agostar",
    "agostera": "agostero",
    "agosteras": "agostero",
    "agosteros": "agostero",
    "agostes": "agostar",
    "agosto": "agostar",
    "agostos": "agosto",
    "agostábamos": "agostar",
    "agostáis": "agostar",
    "agostáramos": "agostar",
    "agostáremos": "agostar",
    "agostásemos": "agostar",
    "agosté": "agostar",
    "agostéis": "agostar",
    "agostó": "agostar",
    "agota": "agotar",
    "agotaba": "agotar",
    "agotabais": "agotar",
    "agotaban": "agotar",
    "agotabas": "agotar",
    "agotables": "agotable",
    "agotad": "agotar",
    "agotada": "agotar",
    "agotadas": "agotar",
    "agotado": "agotar",
    "agotadora": "agotador",
    "agotadoras": "agotador",
    "agotadores": "agotador",
    "agotados": "agotar",
    "agotamientos": "agotamiento",
    "agotamos": "agotar",
    "agotan": "agotar",
    "agotando": "agotar",
    "agotara": "agotar",
    "agotarais": "agotar",
    "agotaran": "agotar",
    "agotaras": "agotar",
    "agotare": "agotar",
    "agotareis": "agotar",
    "agotaremos": "agotar",
    "agotaren": "agotar",
    "agotares": "agotar",
    "agotaron": "agotar",
    "agotará": "agotar",
    "agotarán": "agotar",
    "agotarás": "agotar",
    "agotaré": "agotar",
    "agotaréis": "agotar",
    "agotaría": "agotar",
    "agotaríais": "agotar",
    "agotaríamos": "agotar",
    "agotarían": "agotar",
    "agotarías": "agotar",
    "agotas": "agotar",
    "agotase": "agotar",
    "agotaseis": "agotar",
    "agotasen": "agotar",
    "agotases": "agotar",
    "agotaste": "agotar",
    "agotasteis": "agotar",
    "agote": "agotar",
    "agotemos": "agotar",
    "agoten": "agotar",
    "agotes": "agote",
    "agoto": "agotar",
    "agotábamos": "agotar",
    "agotáis": "agotar",
    "agotáramos": "agotar",
    "agotáremos": "agotar",
    "agotásemos": "agotar",
    "agoté": "agotar",
    "agotéis": "agotar",
    "agotó": "agotar",
    "agracejos": "agracejo",
    "agracera": "agracero",
    "agraceras": "agracero",
    "agraceros": "agracero",
    "agraces": "agraz",
    "agraceña": "agraceño",
    "agraceñas": "agraceño",
    "agraceños": "agraceño",
    "agracia": "agraciar",
    "agraciaba": "agraciar",
    "agraciabais": "agraciar",
    "agraciaban": "agraciar",
    "agraciabas": "agraciar",
    "agraciad": "agraciar",
    "agraciada": "agraciar",
    "agraciadas": "agraciar",
    "agraciado": "agraciar",
    "agraciados": "agraciar",
    "agraciamos": "agraciar",
    "agracian": "agraciar",
    "agraciando": "agraciar",
    "agraciara": "agraciar",
    "agraciarais": "agraciar",
    "agraciaran": "agraciar",
    "agraciaras": "agraciar",
    "agraciare": "agraciar",
    "agraciareis": "agraciar",
    "agraciaremos": "agraciar",
    "agraciaren": "agraciar",
    "agraciares": "agraciar",
    "agraciaron": "agraciar",
    "agraciará": "agraciar",
    "agraciarán": "agraciar",
    "agraciarás": "agraciar",
    "agraciaré": "agraciar",
    "agraciaréis": "agraciar",
    "agraciaría": "agraciar",
    "agraciaríais": "agraciar",
    "agraciaríamos": "agraciar",
    "agraciarían": "agraciar",
    "agraciarías": "agraciar",
    "agracias": "agraciar",
    "agraciase": "agraciar",
    "agraciaseis": "agraciar",
    "agraciasen": "agraciar",
    "agraciases": "agraciar",
    "agraciaste": "agraciar",
    "agraciasteis": "agraciar",
    "agracie": "agraciar",
    "agraciemos": "agraciar",
    "agracien": "agraciar",
    "agracies": "agraciar",
    "agracio": "agraciar",
    "agraciábamos": "agraciar",
    "agraciáis": "agraciar",
    "agraciáramos": "agraciar",
    "agraciáremos": "agraciar",
    "agraciásemos": "agraciar",
    "agracié": "agraciar",
    "agraciéis": "agraciar",
    "agració": "agraciar",
    "agrada": "agradar",
    "agradaba": "agradar",
    "agradabais": "agradar",
    "agradaban": "agradar",
    "agradabas": "agradar",
    "agradables": "agradable",
    "agradad": "agradar",
    "agradada": "agradar",
    "agradadas": "agradar",
    "agradado": "agradar",
    "agradados": "agradar",
    "agradamos": "agradar",
    "agradan": "agradar",
    "agradando": "agradar",
    "agradara": "agradar",
    "agradarais": "agradar",
    "agradaran": "agradar",
    "agradaras": "agradar",
    "agradare": "agradar",
    "agradareis": "agradar",
    "agradaremos": "agradar",
    "agradaren": "agradar",
    "agradares": "agradar",
    "agradaron": "agradar",
    "agradará": "agradar",
    "agradarán": "agradar",
    "agradarás": "agradar",
    "agradaré": "agradar",
    "agradaréis": "agradar",
    "agradaría": "agradar",
    "agradaríais": "agradar",
    "agradaríamos": "agradar",
    "agradarían": "agradar",
    "agradarías": "agradar",
    "agradas": "agradar",
    "agradase": "agradar",
    "agradaseis": "agradar",
    "agradasen": "agradar",
    "agradases": "agradar",
    "agradaste": "agradar",
    "agradasteis": "agradar",
    "agrade": "agradar",
    "agradece": "agradecer",
    "agradeced": "agradecer",
    "agradecemos": "agradecer",
    "agradecen": "agradecer",
    "agradeceremos": "agradecer",
    "agradecerá": "agradecer",
    "agradecerán": "agradecer",
    "agradecerás": "agradecer",
    "agradeceré": "agradecer",
    "agradeceréis": "agradecer",
    "agradecería": "agradecer",
    "agradeceríais": "agradecer",
    "agradeceríamos": "agradecer",
    "agradecerían": "agradecer",
    "agradecerías": "agradecer",
    "agradeces": "agradecer",
    "agradecida": "agradecer",
    "agradecidas": "agradecer",
    "agradecido": "agradecer",
    "agradecidos": "agradecer",
    "agradeciendo": "agradecer",
    "agradeciera": "agradecer",
    "agradecierais": "agradecer",
    "agradecieran": "agradecer",
    "agradecieras": "agradecer",
    "agradeciere": "agradecer",
    "agradeciereis": "agradecer",
    "agradecieren": "agradecer",
    "agradecieres": "agradecer",
    "agradecieron": "agradecer",
    "agradeciese": "agradecer",
    "agradecieseis": "agradecer",
    "agradeciesen": "agradecer",
    "agradecieses": "agradecer",
    "agradecimientos": "agradecimiento",
    "agradecimos": "agradecer",
    "agradeciste": "agradecer",
    "agradecisteis": "agradecer",
    "agradeciéramos": "agradecer",
    "agradeciéremos": "agradecer",
    "agradeciésemos": "agradecer",
    "agradeció": "agradecer",
    "agradecéis": "agradecer",
    "agradecí": "agradecer",
    "agradecía": "agradecer",
    "agradecíais": "agradecer",
    "agradecíamos": "agradecer",
    "agradecían": "agradecer",
    "agradecías": "agradecer",
    "agrademos": "agradar",
    "agraden": "agradar",
    "agrades": "agradar",
    "agradezca": "agradecer",
    "agradezcamos": "agradecer",
    "agradezcan": "agradecer",
    "agradezcas": "agradecer",
    "agradezco": "agradecer",
    "agradezcáis": "agradecer",
    "agrado": "agradar",
    "agrados": "agrado",
    "agradábamos": "agradar",
    "agradáis": "agradar",
    "agradáramos": "agradar",
    "agradáremos": "agradar",
    "agradásemos": "agradar",
    "agradé": "agradar",
    "agradéis": "agradar",
    "agradó": "agradar",
    "agrafías": "agrafía",
    "agrama": "agramar",
    "agramaba": "agramar",
    "agramabais": "agramar",
    "agramaban": "agramar",
    "agramabas": "agramar",
    "agramad": "agramar",
    "agramada": "agramar",
    "agramadas": "agramar",
    "agramaderas": "agramadera",
    "agramado": "agramar",
    "agramadora": "agramador",
    "agramadoras": "agramador",
    "agramadores": "agramador",
    "agramados": "agramar",
    "agramamos": "agramar",
    "agraman": "agramar",
    "agramando": "agramar",
    "agramara": "agramar",
    "agramarais": "agramar",
    "agramaran": "agramar",
    "agramaras": "agramar",
    "agramare": "agramar",
    "agramareis": "agramar",
    "agramaremos": "agramar",
    "agramaren": "agramar",
    "agramares": "agramar",
    "agramaron": "agramar",
    "agramará": "agramar",
    "agramarán": "agramar",
    "agramarás": "agramar",
    "agramaré": "agramar",
    "agramaréis": "agramar",
    "agramaría": "agramar",
    "agramaríais": "agramar",
    "agramaríamos": "agramar",
    "agramarían": "agramar",
    "agramarías": "agramar",
    "agramas": "agramar",
    "agramase": "agramar",
    "agramaseis": "agramar",
    "agramasen": "agramar",
    "agramases": "agramar",
    "agramaste": "agramar",
    "agramasteis": "agramar",
    "agramaticales": "agramatical",
    "agramaticalidades": "agramaticalidad",
    "agrame": "agramar",
    "agramemos": "agramar",
    "agramen": "agramar",
    "agrames": "agramar",
    "agramila": "agramilar",
    "agramilaba": "agramilar",
    "agramilabais": "agramilar",
    "agramilaban": "agramilar",
    "agramilabas": "agramilar",
    "agramilad": "agramilar",
    "agramilada": "agramilar",
    "agramiladas": "agramilar",
    "agramilado": "agramilar",
    "agramilados": "agramilar",
    "agramilamos": "agramilar",
    "agramilan": "agramilar",
    "agramilando": "agramilar",
    "agramilara": "agramilar",
    "agramilarais": "agramilar",
    "agramilaran": "agramilar",
    "agramilaras": "agramilar",
    "agramilare": "agramilar",
    "agramilareis": "agramilar",
    "agramilaremos": "agramilar",
    "agramilaren": "agramilar",
    "agramilares": "agramilar",
    "agramilaron": "agramilar",
    "agramilará": "agramilar",
    "agramilarán": "agramilar",
    "agramilarás": "agramilar",
    "agramilaré": "agramilar",
    "agramilaréis": "agramilar",
    "agramilaría": "agramilar",
    "agramilaríais": "agramilar",
    "agramilaríamos": "agramilar",
    "agramilarían": "agramilar",
    "agramilarías": "agramilar",
    "agramilas": "agramilar",
    "agramilase": "agramilar",
    "agramilaseis": "agramilar",
    "agramilasen": "agramilar",
    "agramilases": "agramilar",
    "agramilaste": "agramilar",
    "agramilasteis": "agramilar",
    "agramile": "agramilar",
    "agramilemos": "agramilar",
    "agramilen": "agramilar",
    "agramiles": "agramilar",
    "agramilo": "agramilar",
    "agramilábamos": "agramilar",
    "agramiláis": "agramilar",
    "agramiláramos": "agramilar",
    "agramiláremos": "agramilar",
    "agramilásemos": "agramilar",
    "agramilé": "agramilar",
    "agramiléis": "agramilar",
    "agramiló": "agramilar",
    "agramizas": "agramiza",
    "agramo": "agramar",
    "agramontesa": "agramontés",
    "agramontesas": "agramontés",
    "agramonteses": "agramontés",
    "agramábamos": "agramar",
    "agramáis": "agramar",
    "agramáramos": "agramar",
    "agramáremos": "agramar",
    "agramásemos": "agramar",
    "agramé": "agramar",
    "agraméis": "agramar",
    "agramó": "agramar",
    "agranda": "agrandar",
    "agrandaba": "agrandar",
    "agrandabais": "agrandar",
    "agrandaban": "agrandar",
    "agrandabas": "agrandar",
    "agrandad": "agrandar",
    "agrandada": "agrandar",
    "agrandadas": "agrandar",
    "agrandado": "agrandar",
    "agrandados": "agrandar",
    "agrandamientos": "agrandamiento",
    "agrandamos": "agrandar",
    "agrandan": "agrandar",
    "agrandando": "agrandar",
    "agrandara": "agrandar",
    "agrandarais": "agrandar",
    "agrandaran": "agrandar",
    "agrandaras": "agrandar",
    "agrandare": "agrandar",
    "agrandareis": "agrandar",
    "agrandaremos": "agrandar",
    "agrandaren": "agrandar",
    "agrandares": "agrandar",
    "agrandaron": "agrandar",
    "agrandará": "agrandar",
    "agrandarán": "agrandar",
    "agrandarás": "agrandar",
    "agrandaré": "agrandar",
    "agrandaréis": "agrandar",
    "agrandaría": "agrandar",
    "agrandaríais": "agrandar",
    "agrandaríamos": "agrandar",
    "agrandarían": "agrandar",
    "agrandarías": "agrandar",
    "agrandas": "agrandar",
    "agrandase": "agrandar",
    "agrandaseis": "agrandar",
    "agrandasen": "agrandar",
    "agrandases": "agrandar",
    "agrandaste": "agrandar",
    "agrandasteis": "agrandar",
    "agrande": "agrandar",
    "agrandemos": "agrandar",
    "agranden": "agrandar",
    "agrandes": "agrandar",
    "agrando": "agrandar",
    "agrandábamos": "agrandar",
    "agrandáis": "agrandar",
    "agrandáramos": "agrandar",
    "agrandáremos": "agrandar",
    "agrandásemos": "agrandar",
    "agrandé": "agrandar",
    "agrandéis": "agrandar",
    "agrandó": "agrandar",
    "agranuja": "agranujar",
    "agranujaba": "agranujar",
    "agranujabais": "agranujar",
    "agranujaban": "agranujar",
    "agranujabas": "agranujar",
    "agranujad": "agranujar",
    "agranujada": "agranujar",
    "agranujadas": "agranujar",
    "agranujado": "agranujar",
    "agranujados": "agranujar",
    "agranujamos": "agranujar",
    "agranujan": "agranujar",
    "agranujando": "agranujar",
    "agranujara": "agranujar",
    "agranujarais": "agranujar",
    "agranujaran": "agranujar",
    "agranujaras": "agranujar",
    "agranujare": "agranujar",
    "agranujareis": "agranujar",
    "agranujaremos": "agranujar",
    "agranujaren": "agranujar",
    "agranujares": "agranujar",
    "agranujaron": "agranujar",
    "agranujará": "agranujar",
    "agranujarán": "agranujar",
    "agranujarás": "agranujar",
    "agranujaré": "agranujar",
    "agranujaréis": "agranujar",
    "agranujaría": "agranujar",
    "agranujaríais": "agranujar",
    "agranujaríamos": "agranujar",
    "agranujarían": "agranujar",
    "agranujarías": "agranujar",
    "agranujas": "agranujar",
    "agranujase": "agranujar",
    "agranujaseis": "agranujar",
    "agranujasen": "agranujar",
    "agranujases": "agranujar",
    "agranujaste": "agranujar",
    "agranujasteis": "agranujar",
    "agranuje": "agranujar",
    "agranujemos": "agranujar",
    "agranujen": "agranujar",
    "agranujes": "agranujar",
    "agranujo": "agranujar",
    "agranujábamos": "agranujar",
    "agranujáis": "agranujar",
    "agranujáramos": "agranujar",
    "agranujáremos": "agranujar",
    "agranujásemos": "agranujar",
    "agranujé": "agranujar",
    "agranujéis": "agranujar",
    "agranujó": "agranujar",
    "agranulares": "agranular",
    "agraria": "agrario",
    "agrarias": "agrario",
    "agrarios": "agrario",
    "agrarismos": "agrarismo",
    "agrava": "agravar",
    "agravaba": "agravar",
    "agravabais": "agravar",
    "agravaban": "agravar",
    "agravabas": "agravar",
    "agravaciones": "agravación",
    "agravad": "agravar",
    "agravada": "agravar",
    "agravadas": "agravar",
    "agravado": "agravar",
    "agravadora": "agravador",
    "agravadoras": "agravador",
    "agravadores": "agravador",
    "agravados": "agravar",
    "agravamientos": "agravamiento",
    "agravamos": "agravar",
    "agravan": "agravar",
    "agravando": "agravar",
    "agravantes": "agravante",
    "agravara": "agravar",
    "agravarais": "agravar",
    "agravaran": "agravar",
    "agravaras": "agravar",
    "agravare": "agravar",
    "agravareis": "agravar",
    "agravaremos": "agravar",
    "agravaren": "agravar",
    "agravares": "agravar",
    "agravaron": "agravar",
    "agravará": "agravar",
    "agravarán": "agravar",
    "agravarás": "agravar",
    "agravaré": "agravar",
    "agravaréis": "agravar",
    "agravaría": "agravar",
    "agravaríais": "agravar",
    "agravaríamos": "agravar",
    "agravarían": "agravar",
    "agravarías": "agravar",
    "agravas": "agravar",
    "agravase": "agravar",
    "agravaseis": "agravar",
    "agravasen": "agravar",
    "agravases": "agravar",
    "agravaste": "agravar",
    "agravasteis": "agravar",
    "agrave": "agravar",
    "agravemos": "agravar",
    "agraven": "agravar",
    "agraves": "agravar",
    "agravia": "agraviar",
    "agraviaba": "agraviar",
    "agraviabais": "agraviar",
    "agraviaban": "agraviar",
    "agraviabas": "agraviar",
    "agraviad": "agraviar",
    "agraviada": "agraviar",
    "agraviadas": "agraviar",
    "agraviado": "agraviar",
    "agraviadora": "agraviador",
    "agraviadoras": "agraviador",
    "agraviadores": "agraviador",
    "agraviados": "agraviar",
    "agraviamos": "agraviar",
    "agravian": "agraviar",
    "agraviando": "agraviar",
    "agraviara": "agraviar",
    "agraviarais": "agraviar",
    "agraviaran": "agraviar",
    "agraviaras": "agraviar",
    "agraviare": "agraviar",
    "agraviareis": "agraviar",
    "agraviaremos": "agraviar",
    "agraviaren": "agraviar",
    "agraviares": "agraviar",
    "agraviaron": "agraviar",
    "agraviará": "agraviar",
    "agraviarán": "agraviar",
    "agraviarás": "agraviar",
    "agraviaré": "agraviar",
    "agraviaréis": "agraviar",
    "agraviaría": "agraviar",
    "agraviaríais": "agraviar",
    "agraviaríamos": "agraviar",
    "agraviarían": "agraviar",
    "agraviarías": "agraviar",
    "agravias": "agraviar",
    "agraviase": "agraviar",
    "agraviaseis": "agraviar",
    "agraviasen": "agraviar",
    "agraviases": "agraviar",
    "agraviaste": "agraviar",
    "agraviasteis": "agraviar",
    "agravie": "agraviar",
    "agraviemos": "agraviar",
    "agravien": "agraviar",
    "agravies": "agraviar",
    "agravio": "agraviar",
    "agravios": "agravio",
    "agraviosa": "agravioso",
    "agraviosas": "agravioso",
    "agraviosos": "agravioso",
    "agraviábamos": "agraviar",
    "agraviáis": "agraviar",
    "agraviáramos": "agraviar",
    "agraviáremos": "agraviar",
    "agraviásemos": "agraviar",
    "agravié": "agraviar",
    "agraviéis": "agraviar",
    "agravió": "agraviar",
    "agravo": "agravar",
    "agravábamos": "agravar",
    "agraváis": "agravar",
    "agraváramos": "agravar",
    "agraváremos": "agravar",
    "agravásemos": "agravar",
    "agravé": "agravar",
    "agravéis": "agravar",
    "agravó": "agravar",
    "agrazadas": "agrazada",
    "agrazones": "agrazón",
    "agreda": "agredir",
    "agredamos": "agredir",
    "agredan": "agredir",
    "agredas": "agredir",
    "agrede": "agredir",
    "agreden": "agredir",
    "agredes": "agredir",
    "agredid": "agredir",
    "agredida": "agredir",
    "agredidas": "agredir",
    "agredido": "agredir",
    "agredidos": "agredir",
    "agrediendo": "agredir",
    "agrediera": "agredir",
    "agredierais": "agredir",
    "agredieran": "agredir",
    "agredieras": "agredir",
    "agrediere": "agredir",
    "agrediereis": "agredir",
    "agredieren": "agredir",
    "agredieres": "agredir",
    "agredieron": "agredir",
    "agrediese": "agredir",
    "agredieseis": "agredir",
    "agrediesen": "agredir",
    "agredieses": "agredir",
    "agredimos": "agredir",
    "agrediremos": "agredir",
    "agredirá": "agredir",
    "agredirán": "agredir",
    "agredirás": "agredir",
    "agrediré": "agredir",
    "agrediréis": "agredir",
    "agrediría": "agredir",
    "agrediríais": "agredir",
    "agrediríamos": "agredir",
    "agredirían": "agredir",
    "agredirías": "agredir",
    "agrediste": "agredir",
    "agredisteis": "agredir",
    "agrediéramos": "agredir",
    "agrediéremos": "agredir",
    "agrediésemos": "agredir",
    "agredió": "agredir",
    "agredo": "agredir",
    "agredáis": "agredir",
    "agredí": "agredir",
    "agredía": "agredir",
    "agredíais": "agredir",
    "agredíamos": "agredir",
    "agredían": "agredir",
    "agredías": "agredir",
    "agredís": "agredir",
    "agrega": "agregar",
    "agregaba": "agregar",
    "agregabais": "agregar",
    "agregaban": "agregar",
    "agregabas": "agregar",
    "agregaciones": "agregación",
    "agregad": "agregar",
    "agregada": "agregar",
    "agregadas": "agregar",
    "agregado": "agregar",
    "agregadora": "agregador",
    "agregadoras": "agregador",
    "agregadores": "agregador",
    "agregados": "agregar",
    "agregamos": "agregar",
    "agregan": "agregar",
    "agregando": "agregar",
    "agregara": "agregar",
    "agregarais": "agregar",
    "agregaran": "agregar",
    "agregaras": "agregar",
    "agregare": "agregar",
    "agregareis": "agregar",
    "agregaremos": "agregar",
    "agregaren": "agregar",
    "agregares": "agregar",
    "agregaron": "agregar",
    "agregará": "agregar",
    "agregarán": "agregar",
    "agregarás": "agregar",
    "agregaré": "agregar",
    "agregaréis": "agregar",
    "agregaría": "agregar",
    "agregaríais": "agregar",
    "agregaríamos": "agregar",
    "agregarían": "agregar",
    "agregarías": "agregar",
    "agregas": "agregar",
    "agregase": "agregar",
    "agregaseis": "agregar",
    "agregasen": "agregar",
    "agregases": "agregar",
    "agregaste": "agregar",
    "agregasteis": "agregar",
    "agregativa": "agregativo",
    "agregativas": "agregativo",
    "agregativos": "agregativo",
    "agrego": "agregar",
    "agregue": "agregar",
    "agreguemos": "agregar",
    "agreguen": "agregar",
    "agregues": "agregar",
    "agregué": "agregar",
    "agreguéis": "agregar",
    "agregábamos": "agregar",
    "agregáis": "agregar",
    "agregáramos": "agregar",
    "agregáremos": "agregar",
    "agregásemos": "agregar",
    "agregó": "agregar",
    "agremanes": "agremán",
    "agremia": "agremiar",
    "agremiaba": "agremiar",
    "agremiabais": "agremiar",
    "agremiaban": "agremiar",
    "agremiabas": "agremiar",
    "agremiaciones": "agremiación",
    "agremiad": "agremiar",
    "agremiada": "agremiar",
    "agremiadas": "agremiar",
    "agremiado": "agremiar",
    "agremiados": "agremiar",
    "agremiamos": "agremiar",
    "agremian": "agremiar",
    "agremiando": "agremiar",
    "agremiara": "agremiar",
    "agremiarais": "agremiar",
    "agremiaran": "agremiar",
    "agremiaras": "agremiar",
    "agremiare": "agremiar",
    "agremiareis": "agremiar",
    "agremiaremos": "agremiar",
    "agremiaren": "agremiar",
    "agremiares": "agremiar",
    "agremiaron": "agremiar",
    "agremiará": "agremiar",
    "agremiarán": "agremiar",
    "agremiarás": "agremiar",
    "agremiaré": "agremiar",
    "agremiaréis": "agremiar",
    "agremiaría": "agremiar",
    "agremiaríais": "agremiar",
    "agremiaríamos": "agremiar",
    "agremiarían": "agremiar",
    "agremiarías": "agremiar",
    "agremias": "agremiar",
    "agremiase": "agremiar",
    "agremiaseis": "agremiar",
    "agremiasen": "agremiar",
    "agremiases": "agremiar",
    "agremiaste": "agremiar",
    "agremiasteis": "agremiar",
    "agremie": "agremiar",
    "agremiemos": "agremiar",
    "agremien": "agremiar",
    "agremies": "agremiar",
    "agremio": "agremiar",
    "agremiábamos": "agremiar",
    "agremiáis": "agremiar",
    "agremiáramos": "agremiar",
    "agremiáremos": "agremiar",
    "agremiásemos": "agremiar",
    "agremié": "agremiar",
    "agremiéis": "agremiar",
    "agremió": "agremiar",
    "agresinas": "agresina",
    "agresinógenos": "agresinógeno",
    "agresiones": "agresión",
    "agresiva": "agresivo",
    "agresivas": "agresivo",
    "agresividades": "agresividad",
    "agresivos": "agresivo",
    "agresora": "agresor",
    "agresoras": "agresor",
    "agresores": "agresor",
    "agrestes": "agreste",
    "agretopos": "agretopo",
    "agria": "agrio",
    "agriaba": "agriar",
    "agriabais": "agriar",
    "agriaban": "agriar",
    "agriabas": "agriar",
    "agriad": "agriar",
    "agriada": "agriar",
    "agriadas": "agriar",
    "agriado": "agriar",
    "agriados": "agriar",
    "agriamientos": "agriamiento",
    "agriamos": "agriar",
    "agrian": "agriar",
    "agriando": "agriar",
    "agriara": "agriar",
    "agriarais": "agriar",
    "agriaran": "agriar",
    "agriaras": "agriar",
    "agriare": "agriar",
    "agriareis": "agriar",
    "agriaremos": "agriar",
    "agriaren": "agriar",
    "agriares": "agriar",
    "agriaron": "agriar",
    "agriará": "agriar",
    "agriarán": "agriar",
    "agriarás": "agriar",
    "agriaré": "agriar",
    "agriaréis": "agriar",
    "agriaría": "agriar",
    "agriaríais": "agriar",
    "agriaríamos": "agriar",
    "agriarían": "agriar",
    "agriarías": "agriar",
    "agrias": "agrio",
    "agriase": "agriar",
    "agriaseis": "agriar",
    "agriasen": "agriar",
    "agriases": "agriar",
    "agriaste": "agriar",
    "agriasteis": "agriar",
    "agricultora": "agricultor",
    "agricultoras": "agricultor",
    "agricultores": "agricultor",
    "agriculturas": "agricultura",
    "agridulces": "agridulce",
    "agrie": "agriar",
    "agriemos": "agriar",
    "agrien": "agriar",
    "agrieras": "agriera",
    "agries": "agriar",
    "agrieta": "agrietar",
    "agrietaba": "agrietar",
    "agrietabais": "agrietar",
    "agrietaban": "agrietar",
    "agrietabas": "agrietar",
    "agrietad": "agrietar",
    "agrietada": "agrietar",
    "agrietadas": "agrietar",
    "agrietado": "agrietar",
    "agrietados": "agrietar",
    "agrietamientos": "agrietamiento",
    "agrietamos": "agrietar",
    "agrietan": "agrietar",
    "agrietando": "agrietar",
    "agrietara": "agrietar",
    "agrietarais": "agrietar",
    "agrietaran": "agrietar",
    "agrietaras": "agrietar",
    "agrietare": "agrietar",
    "agrietareis": "agrietar",
    "agrietaremos": "agrietar",
    "agrietaren": "agrietar",
    "agrietares": "agrietar",
    "agrietaron": "agrietar",
    "agrietará": "agrietar",
    "agrietarán": "agrietar",
    "agrietarás": "agrietar",
    "agrietaré": "agrietar",
    "agrietaréis": "agrietar",
    "agrietaría": "agrietar",
    "agrietaríais": "agrietar",
    "agrietaríamos": "agrietar",
    "agrietarían": "agrietar",
    "agrietarías": "agrietar",
    "agrietas": "agrietar",
    "agrietase": "agrietar",
    "agrietaseis": "agrietar",
    "agrietasen": "agrietar",
    "agrietases": "agrietar",
    "agrietaste": "agrietar",
    "agrietasteis": "agrietar",
    "agriete": "agrietar",
    "agrietemos": "agrietar",
    "agrieten": "agrietar",
    "agrietes": "agrietar",
    "agrieto": "agrietar",
    "agrietábamos": "agrietar",
    "agrietáis": "agrietar",
    "agrietáramos": "agrietar",
    "agrietáremos": "agrietar",
    "agrietásemos": "agrietar",
    "agrieté": "agrietar",
    "agrietéis": "agrietar",
    "agrietó": "agrietar",
    "agrimensora": "agrimensor",
    "agrimensoras": "agrimensor",
    "agrimensores": "agrimensor",
    "agrimensuras": "agrimensura",
    "agrimonias": "agrimonia",
    "agrio": "agriar",
    "agriones": "agrión",
    "agrios": "agrio",
    "agriotipos": "agriotipo",
    "agrisada": "agrisado",
    "agrisadas": "agrisado",
    "agrisados": "agrisado",
    "agriábamos": "agriar",
    "agriáis": "agriar",
    "agriáramos": "agriar",
    "agriáremos": "agriar",
    "agriásemos": "agriar",
    "agrié": "agriar",
    "agriéis": "agriar",
    "agrió": "agriar",
    "agroalimentaria": "agroalimentario",
    "agroalimentarias": "agroalimentario",
    "agroalimentarios": "agroalimentario",
    "agroambientales": "agroambiental",
    "agrobiologías": "agrobiología",
    "agrociudades": "agrociudad",
    "agroclimática": "agroclimático",
    "agroclimáticas": "agroclimático",
    "agroclimáticos": "agroclimático",
    "agroforestales": "agroforestal",
    "agrogeologías": "agrogeología",
    "agroindustriales": "agroindustrial",
    "agroindustrias": "agroindustria",
    "agrologías": "agrología",
    "agrometeorologías": "agrometeorología",
    "agronometrías": "agronometría",
    "agronomías": "agronomía",
    "agronómica": "agronómico",
    "agronómicas": "agronómico",
    "agronómicos": "agronómico",
    "agropecuaria": "agropecuario",
    "agropecuarias": "agropecuario",
    "agropecuarios": "agropecuario",
    "agros": "agro",
    "agroturismos": "agroturismo",
    "agrupa": "agrupar",
    "agrupaba": "agrupar",
    "agrupabais": "agrupar",
    "agrupaban": "agrupar",
    "agrupabas": "agrupar",
    "agrupables": "agrupable",
    "agrupaciones": "agrupación",
    "agrupad": "agrupar",
    "agrupada": "agrupar",
    "agrupadas": "agrupar",
    "agrupado": "agrupar",
    "agrupadora": "agrupador",
    "agrupadoras": "agrupador",
    "agrupadores": "agrupador",
    "agrupados": "agrupar",
    "agrupamientos": "agrupamiento",
    "agrupamos": "agrupar",
    "agrupan": "agrupar",
    "agrupando": "agrupar",
    "agrupara": "agrupar",
    "agruparais": "agrupar",
    "agruparan": "agrupar",
    "agruparas": "agrupar",
    "agrupare": "agrupar",
    "agrupareis": "agrupar",
    "agruparemos": "agrupar",
    "agruparen": "agrupar",
    "agrupares": "agrupar",
    "agruparon": "agrupar",
    "agrupará": "agrupar",
    "agruparán": "agrupar",
    "agruparás": "agrupar",
    "agruparé": "agrupar",
    "agruparéis": "agrupar",
    "agruparía": "agrupar",
    "agruparíais": "agrupar",
    "agruparíamos": "agrupar",
    "agruparían": "agrupar",
    "agruparías": "agrupar",
    "agrupas": "agrupar",
    "agrupase": "agrupar",
    "agrupaseis": "agrupar",
    "agrupasen": "agrupar",
    "agrupases": "agrupar",
    "agrupaste": "agrupar",
    "agrupasteis": "agrupar",
    "agrupe": "agrupar",
    "agrupemos": "agrupar",
    "agrupen": "agrupar",
    "agrupes": "agrupar",
    "agrupo": "agrupar",
    "agrupábamos": "agrupar",
    "agrupáis": "agrupar",
    "agrupáramos": "agrupar",
    "agrupáremos": "agrupar",
    "agrupásemos": "agrupar",
    "agrupé": "agrupar",
    "agrupéis": "agrupar",
    "agrupó": "agrupar",
    "agruras": "agrura",
    "agrícolas": "agrícola",
    "agrónoma": "agrónomo",
    "agrónomas": "agrónomo",
    "agrónomos": "agrónomo",
    "agua": "aguar",
    "aguaba": "aguar",
    "aguabais": "aguar",
    "aguaban": "aguar",
    "aguabas": "aguar",
    "aguacatee": "aguacate",
    "aguacatees": "aguacate",
    "aguacates": "aguacate",
    "aguace": "aguazar",
    "aguacemos": "aguazar",
    "aguacen": "aguazar",
    "aguaceros": "aguacero",
    "aguaces": "aguazar",
    "aguacha": "aguachar",
    "aguachaba": "aguachar",
    "aguachabais": "aguachar",
    "aguachaban": "aguachar",
    "aguachabas": "aguachar",
    "aguachad": "aguachar",
    "aguachada": "aguachar",
    "aguachadas": "aguachar",
    "aguachado": "aguachar",
    "aguachados": "aguachar",
    "aguachamos": "aguachar",
    "aguachan": "aguachar",
    "aguachando": "aguachar",
    "aguachara": "aguachar",
    "aguacharais": "aguachar",
    "aguacharan": "aguachar",
    "aguacharas": "aguachar",
    "aguachare": "aguachar",
    "aguachareis": "aguachar",
    "aguacharemos": "aguachar",
    "aguacharen": "aguachar",
    "aguachares": "aguachar",
    "aguacharna": "aguacharnar",
    "aguacharnaba": "aguacharnar",
    "aguacharnabais": "aguacharnar",
    "aguacharnaban": "aguacharnar",
    "aguacharnabas": "aguacharnar",
    "aguacharnad": "aguacharnar",
    "aguacharnada": "aguacharnar",
    "aguacharnadas": "aguacharnar",
    "aguacharnado": "aguacharnar",
    "aguacharnados": "aguacharnar",
    "aguacharnamos": "aguacharnar",
    "aguacharnan": "aguacharnar",
    "aguacharnando": "aguacharnar",
    "aguacharnara": "aguacharnar",
    "aguacharnarais": "aguacharnar",
    "aguacharnaran": "aguacharnar",
    "aguacharnaras": "aguacharnar",
    "aguacharnare": "aguacharnar",
    "aguacharnareis": "aguacharnar",
    "aguacharnaremos": "aguacharnar",
    "aguacharnaren": "aguacharnar",
    "aguacharnares": "aguacharnar",
    "aguacharnaron": "aguacharnar",
    "aguacharnará": "aguacharnar",
    "aguacharnarán": "aguacharnar",
    "aguacharnarás": "aguacharnar",
    "aguacharnaré": "aguacharnar",
    "aguacharnaréis": "aguacharnar",
    "aguacharnaría": "aguacharnar",
    "aguacharnaríais": "aguacharnar",
    "aguacharnaríamos": "aguacharnar",
    "aguacharnarían": "aguacharnar",
    "aguacharnarías": "aguacharnar",
    "aguacharnas": "aguacharnar",
    "aguacharnase": "aguacharnar",
    "aguacharnaseis": "aguacharnar",
    "aguacharnasen": "aguacharnar",
    "aguacharnases": "aguacharnar",
    "aguacharnaste": "aguacharnar",
    "aguacharnasteis": "aguacharnar",
    "aguacharne": "aguacharnar",
    "aguacharnemos": "aguacharnar",
    "aguacharnen": "aguacharnar",
    "aguacharnes": "aguacharnar",
    "aguacharno": "aguacharnar",
    "aguacharnábamos": "aguacharnar",
    "aguacharnáis": "aguacharnar",
    "aguacharnáramos": "aguacharnar",
    "aguacharnáremos": "aguacharnar",
    "aguacharnásemos": "aguacharnar",
    "aguacharné": "aguacharnar",
    "aguacharnéis": "aguacharnar",
    "aguacharnó": "aguacharnar",
    "aguacharon": "aguachar",
    "aguachará": "aguachar",
    "aguacharán": "aguachar",
    "aguacharás": "aguachar",
    "aguacharé": "aguachar",
    "aguacharéis": "aguachar",
    "aguacharía": "aguachar",
    "aguacharíais": "aguachar",
    "aguacharíamos": "aguachar",
    "aguacharían": "aguachar",
    "aguacharías": "aguachar",
    "aguachas": "aguachar",
    "aguachase": "aguachar",
    "aguachaseis": "aguachar",
    "aguachasen": "aguachar",
    "aguachases": "aguachar",
    "aguachaste": "aguachar",
    "aguachasteis": "aguachar",
    "aguache": "aguachar",
    "aguachemos": "aguachar",
    "aguachen": "aguachar",
    "aguachenta": "aguachento",
    "aguachentas": "aguachento",
    "aguachentos": "aguachento",
    "aguaches": "aguachar",
    "aguachina": "aguachinar",
    "aguachinaba": "aguachinar",
    "aguachinabais": "aguachinar",
    "aguachinaban": "aguachinar",
    "aguachinabas": "aguachinar",
    "aguachinad": "aguachinar",
    "aguachinada": "aguachinar",
    "aguachinadas": "aguachinar",
    "aguachinado": "aguachinar",
    "aguachinados": "aguachinar",
    "aguachinamos": "aguachinar",
    "aguachinan": "aguachinar",
    "aguachinando": "aguachinar",
    "aguachinara": "aguachinar",
    "aguachinarais": "aguachinar",
    "aguachinaran": "aguachinar",
    "aguachinaras": "aguachinar",
    "aguachinare": "aguachinar",
    "aguachinareis": "aguachinar",
    "aguachinaremos": "aguachinar",
    "aguachinaren": "aguachinar",
    "aguachinares": "aguachinar",
    "aguachinaron": "aguachinar",
    "aguachinará": "aguachinar",
    "aguachinarán": "aguachinar",
    "aguachinarás": "aguachinar",
    "aguachinaré": "aguachinar",
    "aguachinaréis": "aguachinar",
    "aguachinaría": "aguachinar",
    "aguachinaríais": "aguachinar",
    "aguachinaríamos": "aguachinar",
    "aguachinarían": "aguachinar",
    "aguachinarías": "aguachinar",
    "aguachinas": "aguachinar",
    "aguachinase": "aguachinar",
    "aguachinaseis": "aguachinar",
    "aguachinasen": "aguachinar",
    "aguachinases": "aguachinar",
    "aguachinaste": "aguachinar",
    "aguachinasteis": "aguachinar",
    "aguachine": "aguachinar",
    "aguachinemos": "aguachinar",
    "aguachinen": "aguachinar",
    "aguachines": "aguachinar",
    "aguachino": "aguachinar",
    "aguachinábamos": "aguachinar",
    "aguachináis": "aguachinar",
    "aguachináramos": "aguachinar",
    "aguachináremos": "aguachinar",
    "aguachinásemos": "aguachinar",
    "aguachiné": "aguachinar",
    "aguachinéis": "aguachinar",
    "aguachinó": "aguachinar",
    "aguachirles": "aguachirle",
    "aguacho": "aguachar",
    "aguachábamos": "aguachar",
    "aguacháis": "aguachar",
    "aguacháramos": "aguachar",
    "aguacháremos": "aguachar",
    "aguachásemos": "aguachar",
    "aguaché": "aguachar",
    "aguachéis": "aguachar",
    "aguachó": "aguachar",
    "aguaciberas": "aguacibera",
    "aguacé": "aguazar",
    "aguacéis": "aguazar",
    "aguad": "aguar",
    "aguada": "aguar",
    "aguadas": "aguar",
    "aguadera": "aguadero",
    "aguaderas": "aguadero",
    "aguaderos": "aguadero",
    "aguado": "aguar",
    "aguadora": "aguador",
    "aguadoras": "aguador",
    "aguadores": "aguador",
    "aguados": "aguar",
    "aguaduchos": "aguaducho",
    "aguafuertes": "aguafuerte",
    "aguagomas": "aguagoma",
    "aguaita": "aguaitar",
    "aguaitaba": "aguaitar",
    "aguaitabais": "aguaitar",
    "aguaitaban": "aguaitar",
    "aguaitabas": "aguaitar",
    "aguaitad": "aguaitar",
    "aguaitada": "aguaitar",
    "aguaitadas": "aguaitar",
    "aguaitado": "aguaitar",
    "aguaitados": "aguaitar",
    "aguaitamos": "aguaitar",
    "aguaitan": "aguaitar",
    "aguaitando": "aguaitar",
    "aguaitara": "aguaitar",
    "aguaitarais": "aguaitar",
    "aguaitaran": "aguaitar",
    "aguaitaras": "aguaitar",
    "aguaitare": "aguaitar",
    "aguaitareis": "aguaitar",
    "aguaitaremos": "aguaitar",
    "aguaitaren": "aguaitar",
    "aguaitares": "aguaitar",
    "aguaitaron": "aguaitar",
    "aguaitará": "aguaitar",
    "aguaitarán": "aguaitar",
    "aguaitarás": "aguaitar",
    "aguaitaré": "aguaitar",
    "aguaitaréis": "aguaitar",
    "aguaitaría": "aguaitar",
    "aguaitaríais": "aguaitar",
    "aguaitaríamos": "aguaitar",
    "aguaitarían": "aguaitar",
    "aguaitarías": "aguaitar",
    "aguaitas": "aguaitar",
    "aguaitase": "aguaitar",
    "aguaitaseis": "aguaitar",
    "aguaitasen": "aguaitar",
    "aguaitases": "aguaitar",
    "aguaitaste": "aguaitar",
    "aguaitasteis": "aguaitar",
    "aguaite": "aguaitar",
    "aguaitemos": "aguaitar",
    "aguaiten": "aguaitar",
    "aguaites": "aguaitar",
    "aguaito": "aguaitar",
    "aguaitábamos": "aguaitar",
    "aguaitáis": "aguaitar",
    "aguaitáramos": "aguaitar",
    "aguaitáremos": "aguaitar",
    "aguaitásemos": "aguaitar",
    "aguaité": "aguaitar",
    "aguaitéis": "aguaitar",
    "aguaitó": "aguaitar",
    "aguajes": "aguaje",
    "aguamaniles": "aguamanil",
    "aguamarinas": "aguamarina",
    "aguametrías": "aguametría",
    "aguamieles": "aguamiel",
    "aguamos": "aguar",
    "aguan": "aguar",
    "aguando": "aguar",
    "aguanieves": "aguanieve",
    "aguanosa": "aguanoso",
    "aguanosas": "aguanoso",
    "aguanosidades": "aguanosidad",
    "aguanosos": "aguanoso",
    "aguanta": "aguantar",
    "aguantaba": "aguantar",
    "aguantabais": "aguantar",
    "aguantaban": "aguantar",
    "aguantabas": "aguantar",
    "aguantables": "aguantable",
    "aguantad": "aguantar",
    "aguantada": "aguantar",
    "aguantadas": "aguantar",
    "aguantado": "aguantar",
    "aguantados": "aguantar",
    "aguantamos": "aguantar",
    "aguantan": "aguantar",
    "aguantando": "aguantar",
    "aguantara": "aguantar",
    "aguantarais": "aguantar",
    "aguantaran": "aguantar",
    "aguantaras": "aguantar",
    "aguantare": "aguantar",
    "aguantareis": "aguantar",
    "aguantaremos": "aguantar",
    "aguantaren": "aguantar",
    "aguantares": "aguantar",
    "aguantaron": "aguantar",
    "aguantará": "aguantar",
    "aguantarán": "aguantar",
    "aguantarás": "aguantar",
    "aguantaré": "aguantar",
    "aguantaréis": "aguantar",
    "aguantaría": "aguantar",
    "aguantaríais": "aguantar",
    "aguantaríamos": "aguantar",
    "aguantarían": "aguantar",
    "aguantarías": "aguantar",
    "aguantas": "aguantar",
    "aguantase": "aguantar",
    "aguantaseis": "aguantar",
    "aguantasen": "aguantar",
    "aguantases": "aguantar",
    "aguantaste": "aguantar",
    "aguantasteis": "aguantar",
    "aguante": "aguantar",
    "aguantemos": "aguantar",
    "aguanten": "aguantar",
    "aguantes": "aguante",
    "aguanto": "aguantar",
    "aguantábamos": "aguantar",
    "aguantáis": "aguantar",
    "aguantáramos": "aguantar",
    "aguantáremos": "aguantar",
    "aguantásemos": "aguantar",
    "aguanté": "aguantar",
    "aguantéis": "aguantar",
    "aguantó": "aguantar",
    "aguapacles": "aguapacle",
    "aguapiés": "aguapié",
    "aguapés": "aguapé",
    "aguara": "aguar",
    "aguaraguazúes": "aguaraguazú",
    "aguaraguazús": "aguaraguazú",
    "aguarais": "aguar",
    "aguaran": "aguar",
    "aguaras": "aguar",
    "aguarda": "aguardar",
    "aguardaba": "aguardar",
    "aguardabais": "aguardar",
    "aguardaban": "aguardar",
    "aguardabas": "aguardar",
    "aguardad": "aguardar",
    "aguardada": "aguardar",
    "aguardadas": "aguardar",
    "aguardado": "aguardar",
    "aguardados": "aguardar",
    "aguardamos": "aguardar",
    "aguardan": "aguardar",
    "aguardando": "aguardar",
    "aguardara": "aguardar",
    "aguardarais": "aguardar",
    "aguardaran": "aguardar",
    "aguardaras": "aguardar",
    "aguardare": "aguardar",
    "aguardareis": "aguardar",
    "aguardaremos": "aguardar",
    "aguardaren": "aguardar",
    "aguardares": "aguardar",
    "aguardaron": "aguardar",
    "aguardará": "aguardar",
    "aguardarán": "aguardar",
    "aguardarás": "aguardar",
    "aguardaré": "aguardar",
    "aguardaréis": "aguardar",
    "aguardaría": "aguardar",
    "aguardaríais": "aguardar",
    "aguardaríamos": "aguardar",
    "aguardarían": "aguardar",
    "aguardarías": "aguardar",
    "aguardas": "aguardar",
    "aguardase": "aguardar",
    "aguardaseis": "aguardar",
    "aguardasen": "aguardar",
    "aguardases": "aguardar",
    "aguardaste": "aguardar",
    "aguardasteis": "aguardar",
    "aguarde": "aguardar",
    "aguardemos": "aguardar",
    "aguarden": "aguardar",
    "aguardentosa": "aguardentoso",
    "aguardentosas": "aguardentoso",
    "aguardentosos": "aguardentoso",
    "aguardes": "aguardar",
    "aguardientes": "aguardiente",
    "aguardo": "aguardar",
    "aguardábamos": "aguardar",
    "aguardáis": "aguardar",
    "aguardáramos": "aguardar",
    "aguardáremos": "aguardar",
    "aguardásemos": "aguardar",
    "aguardé": "aguardar",
    "aguardéis": "aguardar",
    "aguardó": "aguardar",
    "aguare": "aguar",
    "aguareis": "aguar",
    "aguaremos": "aguar",
    "aguaren": "aguar",
    "aguares": "aguar",
    "aguaron": "aguar",
    "aguarrases": "aguarrás",
    "aguará": "aguar",
    "aguarán": "aguar",
    "aguarás": "aguar",
    "aguaré": "aguar",
    "aguaréis": "aguar",
    "aguaría": "aguar",
    "aguaríais": "aguar",
    "aguaríamos": "aguar",
    "aguarían": "aguar",
    "aguarías": "aguar",
    "aguas": "aguar",
    "aguase": "aguar",
    "aguaseis": "aguar",
    "aguasen": "aguar",
    "aguases": "aguar",
    "aguaste": "aguar",
    "aguasteis": "aguar",
    "aguatintas": "aguatinta",
    "aguatochas": "aguatocha",
    "aguayes": "aguay",
    "aguaza": "aguazar",
    "aguazaba": "aguazar",
    "aguazabais": "aguazar",
    "aguazaban": "aguazar",
    "aguazabas": "aguazar",
    "aguazad": "aguazar",
    "aguazada": "aguazar",
    "aguazadas": "aguazar",
    "aguazado": "aguazar",
    "aguazados": "aguazar",
    "aguazales": "aguazal",
    "aguazamos": "aguazar",
    "aguazan": "aguazar",
    "aguazando": "aguazar",
    "aguazara": "aguazar",
    "aguazarais": "aguazar",
    "aguazaran": "aguazar",
    "aguazaras": "aguazar",
    "aguazare": "aguazar",
    "aguazareis": "aguazar",
    "aguazaremos": "aguazar",
    "aguazaren": "aguazar",
    "aguazares": "aguazar",
    "aguazaron": "aguazar",
    "aguazará": "aguazar",
    "aguazarán": "aguazar",
    "aguazarás": "aguazar",
    "aguazaré": "aguazar",
    "aguazaréis": "aguazar",
    "aguazaría": "aguazar",
    "aguazaríais": "aguazar",
    "aguazaríamos": "aguazar",
    "aguazarían": "aguazar",
    "aguazarías": "aguazar",
    "aguazas": "aguazar",
    "aguazase": "aguazar",
    "aguazaseis": "aguazar",
    "aguazasen": "aguazar",
    "aguazases": "aguazar",
    "aguazaste": "aguazar",
    "aguazasteis": "aguazar",
    "aguazo": "aguazar",
    "aguazos": "aguazo",
    "aguazábamos": "aguazar",
    "aguazáis": "aguazar",
    "aguazáramos": "aguazar",
    "aguazáremos": "aguazar",
    "aguazásemos": "aguazar",
    "aguazó": "aguazar",
    "aguce": "aguzar",
    "agucemos": "aguzar",
    "agucen": "aguzar",
    "aguces": "aguzar",
    "agucé": "aguzar",
    "agucéis": "aguzar",
    "aguda": "agudo",
    "agudas": "agudo",
    "agudezas": "agudeza",
    "agudice": "agudizar",
    "agudicemos": "agudizar",
    "agudicen": "agudizar",
    "agudices": "agudizar",
    "agudicé": "agudizar",
    "agudicéis": "agudizar",
    "agudiza": "agudizar",
    "agudizaba": "agudizar",
    "agudizabais": "agudizar",
    "agudizaban": "agudizar",
    "agudizabas": "agudizar",
    "agudizaciones": "agudización",
    "agudizad": "agudizar",
    "agudizada": "agudizar",
    "agudizadas": "agudizar",
    "agudizado": "agudizar",
    "agudizados": "agudizar",
    "agudizamientos": "agudizamiento",
    "agudizamos": "agudizar",
    "agudizan": "agudizar",
    "agudizando": "agudizar",
    "agudizara": "agudizar",
    "agudizarais": "agudizar",
    "agudizaran": "agudizar",
    "agudizaras": "agudizar",
    "agudizare": "agudizar",
    "agudizareis": "agudizar",
    "agudizaremos": "agudizar",
    "agudizaren": "agudizar",
    "agudizares": "agudizar",
    "agudizaron": "agudizar",
    "agudizará": "agudizar",
    "agudizarán": "agudizar",
    "agudizarás": "agudizar",
    "agudizaré": "agudizar",
    "agudizaréis": "agudizar",
    "agudizaría": "agudizar",
    "agudizaríais": "agudizar",
    "agudizaríamos": "agudizar",
    "agudizarían": "agudizar",
    "agudizarías": "agudizar",
    "agudizas": "agudizar",
    "agudizase": "agudizar",
    "agudizaseis": "agudizar",
    "agudizasen": "agudizar",
    "agudizases": "agudizar",
    "agudizaste": "agudizar",
    "agudizasteis": "agudizar",
    "agudizo": "agudizar",
    "agudizábamos": "agudizar",
    "agudizáis": "agudizar",
    "agudizáramos": "agudizar",
    "agudizáremos": "agudizar",
    "agudizásemos": "agudizar",
    "agudizó": "agudizar",
    "agudos": "agudo",
    "aguerra": "aguerrir",
    "aguerramos": "aguerrir",
    "aguerran": "aguerrir",
    "aguerras": "aguerrir",
    "aguerre": "aguerrir",
    "aguerren": "aguerrir",
    "aguerres": "aguerrir",
    "aguerrid": "aguerrir",
    "aguerrida": "aguerrir",
    "aguerridas": "aguerrir",
    "aguerrido": "aguerrir",
    "aguerridos": "aguerrir",
    "aguerriendo": "aguerrir",
    "aguerriera": "aguerrir",
    "aguerrierais": "aguerrir",
    "aguerrieran": "aguerrir",
    "aguerrieras": "aguerrir",
    "aguerriere": "aguerrir",
    "aguerriereis": "aguerrir",
    "aguerrieren": "aguerrir",
    "aguerrieres": "aguerrir",
    "aguerrieron": "aguerrir",
    "aguerriese": "aguerrir",
    "aguerrieseis": "aguerrir",
    "aguerriesen": "aguerrir",
    "aguerrieses": "aguerrir",
    "aguerrimos": "aguerrir",
    "aguerriremos": "aguerrir",
    "aguerrirá": "aguerrir",
    "aguerrirán": "aguerrir",
    "aguerrirás": "aguerrir",
    "aguerriré": "aguerrir",
    "aguerriréis": "aguerrir",
    "aguerriría": "aguerrir",
    "aguerriríais": "aguerrir",
    "aguerriríamos": "aguerrir",
    "aguerrirían": "aguerrir",
    "aguerrirías": "aguerrir",
    "aguerriste": "aguerrir",
    "aguerristeis": "aguerrir",
    "aguerriéramos": "aguerrir",
    "aguerriéremos": "aguerrir",
    "aguerriésemos": "aguerrir",
    "aguerrió": "aguerrir",
    "aguerro": "aguerrir",
    "aguerráis": "aguerrir",
    "aguerrí": "aguerrir",
    "aguerría": "aguerrir",
    "aguerríais": "aguerrir",
    "aguerríamos": "aguerrir",
    "aguerrían": "aguerrir",
    "aguerrías": "aguerrir",
    "aguerrís": "aguerrir",
    "aguija": "aguijar",
    "aguijaba": "aguijar",
    "aguijabais": "aguijar",
    "aguijaban": "aguijar",
    "aguijabas": "aguijar",
    "aguijad": "aguijar",
    "aguijada": "aguijar",
    "aguijadas": "aguijar",
    "aguijado": "aguijar",
    "aguijados": "aguijar",
    "aguijaduras": "aguijadura",
    "aguijamos": "aguijar",
    "aguijan": "aguijar",
    "aguijando": "aguijar",
    "aguijara": "aguijar",
    "aguijarais": "aguijar",
    "aguijaran": "aguijar",
    "aguijaras": "aguijar",
    "aguijare": "aguijar",
    "aguijareis": "aguijar",
    "aguijaremos": "aguijar",
    "aguijaren": "aguijar",
    "aguijares": "aguijar",
    "aguijaron": "aguijar",
    "aguijará": "aguijar",
    "aguijarán": "aguijar",
    "aguijarás": "aguijar",
    "aguijaré": "aguijar",
    "aguijaréis": "aguijar",
    "aguijaría": "aguijar",
    "aguijaríais": "aguijar",
    "aguijaríamos": "aguijar",
    "aguijarían": "aguijar",
    "aguijarías": "aguijar",
    "aguijas": "aguijar",
    "aguijase": "aguijar",
    "aguijaseis": "aguijar",
    "aguijasen": "aguijar",
    "aguijases": "aguijar",
    "aguijaste": "aguijar",
    "aguijasteis": "aguijar",
    "aguije": "aguijar",
    "aguijemos": "aguijar",
    "aguijen": "aguijar",
    "aguijes": "aguijar",
    "aguijo": "aguijar",
    "aguijonadas": "aguijonada",
    "aguijonazos": "aguijonazo",
    "aguijonea": "aguijonear",
    "aguijoneaba": "aguijonear",
    "aguijoneabais": "aguijonear",
    "aguijoneaban": "aguijonear",
    "aguijoneabas": "aguijonear",
    "aguijonead": "aguijonear",
    "aguijoneada": "aguijonear",
    "aguijoneadas": "aguijonear",
    "aguijoneado": "aguijonear",
    "aguijoneados": "aguijonear",
    "aguijoneamos": "aguijonear",
    "aguijonean": "aguijonear",
    "aguijoneando": "aguijonear",
    "aguijoneara": "aguijonear",
    "aguijonearais": "aguijonear",
    "aguijonearan": "aguijonear",
    "aguijonearas": "aguijonear",
    "aguijoneare": "aguijonear",
    "aguijoneareis": "aguijonear",
    "aguijonearemos": "aguijonear",
    "aguijonearen": "aguijonear",
    "aguijoneares": "aguijonear",
    "aguijonearon": "aguijonear",
    "aguijoneará": "aguijonear",
    "aguijonearán": "aguijonear",
    "aguijonearás": "aguijonear",
    "aguijonearé": "aguijonear",
    "aguijonearéis": "aguijonear",
    "aguijonearía": "aguijonear",
    "aguijonearíais": "aguijonear",
    "aguijonearíamos": "aguijonear",
    "aguijonearían": "aguijonear",
    "aguijonearías": "aguijonear",
    "aguijoneas": "aguijonear",
    "aguijonease": "aguijonear",
    "aguijoneaseis": "aguijonear",
    "aguijoneasen": "aguijonear",
    "aguijoneases": "aguijonear",
    "aguijoneaste": "aguijonear",
    "aguijoneasteis": "aguijonear",
    "aguijonee": "aguijonear",
    "aguijoneemos": "aguijonear",
    "aguijoneen": "aguijonear",
    "aguijonees": "aguijonear",
    "aguijoneo": "aguijonear",
    "aguijones": "aguijón",
    "aguijoneábamos": "aguijonear",
    "aguijoneáis": "aguijonear",
    "aguijoneáramos": "aguijonear",
    "aguijoneáremos": "aguijonear",
    "aguijoneásemos": "aguijonear",
    "aguijoneé": "aguijonear",
    "aguijoneéis": "aguijonear",
    "aguijoneó": "aguijonear",
    "aguijábamos": "aguijar",
    "aguijáis": "aguijar",
    "aguijáramos": "aguijar",
    "aguijáremos": "aguijar",
    "aguijásemos": "aguijar",
    "aguijé": "aguijar",
    "aguijéis": "aguijar",
    "aguijó": "aguijar",
    "aguileras": "aguilera",
    "aguileña": "aguileño",
    "aguileñas": "aguileño",
    "aguileños": "aguileño",
    "aguilones": "aguilón",
    "aguiluchos": "aguilucho",
    "aguinaldos": "aguinaldo",
    "aguizga": "aguizgar",
    "aguizgaba": "aguizgar",
    "aguizgabais": "aguizgar",
    "aguizgaban": "aguizgar",
    "aguizgabas": "aguizgar",
    "aguizgad": "aguizgar",
    "aguizgada": "aguizgar",
    "aguizgadas": "aguizgar",
    "aguizgado": "aguizgar",
    "aguizgados": "aguizgar",
    "aguizgamos": "aguizgar",
    "aguizgan": "aguizgar",
    "aguizgando": "aguizgar",
    "aguizgara": "aguizgar",
    "aguizgarais": "aguizgar",
    "aguizgaran": "aguizgar",
    "aguizgaras": "aguizgar",
    "aguizgare": "aguizgar",
    "aguizgareis": "aguizgar",
    "aguizgaremos": "aguizgar",
    "aguizgaren": "aguizgar",
    "aguizgares": "aguizgar",
    "aguizgaron": "aguizgar",
    "aguizgará": "aguizgar",
    "aguizgarán": "aguizgar",
    "aguizgarás": "aguizgar",
    "aguizgaré": "aguizgar",
    "aguizgaréis": "aguizgar",
    "aguizgaría": "aguizgar",
    "aguizgaríais": "aguizgar",
    "aguizgaríamos": "aguizgar",
    "aguizgarían": "aguizgar",
    "aguizgarías": "aguizgar",
    "aguizgas": "aguizgar",
    "aguizgase": "aguizgar",
    "aguizgaseis": "aguizgar",
    "aguizgasen": "aguizgar",
    "aguizgases": "aguizgar",
    "aguizgaste": "aguizgar",
    "aguizgasteis": "aguizgar",
    "aguizgo": "aguizgar",
    "aguizgue": "aguizgar",
    "aguizguemos": "aguizgar",
    "aguizguen": "aguizgar",
    "aguizgues": "aguizgar",
    "aguizgué": "aguizgar",
    "aguizguéis": "aguizgar",
    "aguizgábamos": "aguizgar",
    "aguizgáis": "aguizgar",
    "aguizgáramos": "aguizgar",
    "aguizgáremos": "aguizgar",
    "aguizgásemos": "aguizgar",
    "aguizgó": "aguizgar",
    "agujas": "aguja",
    "agujera": "agujerar",
    "agujeraba": "agujerar",
    "agujerabais": "agujerar",
    "agujeraban": "agujerar",
    "agujerabas": "agujerar",
    "agujerad": "agujerar",
    "agujerada": "agujerar",
    "agujeradas": "agujerar",
    "agujerado": "agujerar",
    "agujerados": "agujerar",
    "agujeramos": "agujerar",
    "agujeran": "agujerar",
    "agujerando": "agujerar",
    "agujerara": "agujerar",
    "agujerarais": "agujerar",
    "agujeraran": "agujerar",
    "agujeraras": "agujerar",
    "agujerare": "agujerar",
    "agujerareis": "agujerar",
    "agujeraremos": "agujerar",
    "agujeraren": "agujerar",
    "agujerares": "agujerar",
    "agujeraron": "agujerar",
    "agujerará": "agujerar",
    "agujerarán": "agujerar",
    "agujerarás": "agujerar",
    "agujeraré": "agujerar",
    "agujeraréis": "agujerar",
    "agujeraría": "agujerar",
    "agujeraríais": "agujerar",
    "agujeraríamos": "agujerar",
    "agujerarían": "agujerar",
    "agujerarías": "agujerar",
    "agujeras": "agujerar",
    "agujerase": "agujerar",
    "agujeraseis": "agujerar",
    "agujerasen": "agujerar",
    "agujerases": "agujerar",
    "agujeraste": "agujerar",
    "agujerasteis": "agujerar",
    "agujere": "agujerar",
    "agujerea": "agujerear",
    "agujereaba": "agujerear",
    "agujereabais": "agujerear",
    "agujereaban": "agujerear",
    "agujereabas": "agujerear",
    "agujeread": "agujerear",
    "agujereada": "agujerear",
    "agujereadas": "agujerear",
    "agujereado": "agujerear",
    "agujereados": "agujerear",
    "agujereamientos": "agujereamiento",
    "agujereamos": "agujerear",
    "agujerean": "agujerear",
    "agujereando": "agujerear",
    "agujereara": "agujerear",
    "agujerearais": "agujerear",
    "agujerearan": "agujerear",
    "agujerearas": "agujerear",
    "agujereare": "agujerear",
    "agujereareis": "agujerear",
    "agujerearemos": "agujerear",
    "agujerearen": "agujerear",
    "agujereares": "agujerear",
    "agujerearon": "agujerear",
    "agujereará": "agujerear",
    "agujerearán": "agujerear",
    "agujerearás": "agujerear",
    "agujerearé": "agujerear",
    "agujerearéis": "agujerear",
    "agujerearía": "agujerear",
    "agujerearíais": "agujerear",
    "agujerearíamos": "agujerear",
    "agujerearían": "agujerear",
    "agujerearías": "agujerear",
    "agujereas": "agujerear",
    "agujerease": "agujerear",
    "agujereaseis": "agujerear",
    "agujereasen": "agujerear",
    "agujereases": "agujerear",
    "agujereaste": "agujerear",
    "agujereasteis": "agujerear",
    "agujeree": "agujerear",
    "agujereemos": "agujerear",
    "agujereen": "agujerear",
    "agujerees": "agujerear",
    "agujeremos": "agujerar",
    "agujeren": "agujerar",
    "agujereo": "agujerear",
    "agujeres": "agujerar",
    "agujereábamos": "agujerear",
    "agujereáis": "agujerear",
    "agujereáramos": "agujerear",
    "agujereáremos": "agujerear",
    "agujereásemos": "agujerear",
    "agujereé": "agujerear",
    "agujereéis": "agujerear",
    "agujereó": "agujerear",
    "agujero": "agujerar",
    "agujeros": "agujero",
    "agujerábamos": "agujerar",
    "agujeráis": "agujerar",
    "agujeráramos": "agujerar",
    "agujeráremos": "agujerar",
    "agujerásemos": "agujerar",
    "agujeré": "agujerar",
    "agujeréis": "agujerar",
    "agujeró": "agujerar",
    "agujones": "agujón",
    "aguo": "aguar",
    "agusana": "agusanar",
    "agusanaba": "agusanar",
    "agusanabais": "agusanar",
    "agusanaban": "agusanar",
    "agusanabas": "agusanar",
    "agusanad": "agusanar",
    "agusanada": "agusanar",
    "agusanadas": "agusanar",
    "agusanado": "agusanar",
    "agusanados": "agusanar",
    "agusanamientos": "agusanamiento",
    "agusanamos": "agusanar",
    "agusanan": "agusanar",
    "agusanando": "agusanar",
    "agusanara": "agusanar",
    "agusanarais": "agusanar",
    "agusanaran": "agusanar",
    "agusanaras": "agusanar",
    "agusanare": "agusanar",
    "agusanareis": "agusanar",
    "agusanaremos": "agusanar",
    "agusanaren": "agusanar",
    "agusanares": "agusanar",
    "agusanaron": "agusanar",
    "agusanará": "agusanar",
    "agusanarán": "agusanar",
    "agusanarás": "agusanar",
    "agusanaré": "agusanar",
    "agusanaréis": "agusanar",
    "agusanaría": "agusanar",
    "agusanaríais": "agusanar",
    "agusanaríamos": "agusanar",
    "agusanarían": "agusanar",
    "agusanarías": "agusanar",
    "agusanas": "agusanar",
    "agusanase": "agusanar",
    "agusanaseis": "agusanar",
    "agusanasen": "agusanar",
    "agusanases": "agusanar",
    "agusanaste": "agusanar",
    "agusanasteis": "agusanar",
    "agusane": "agusanar",
    "agusanemos": "agusanar",
    "agusanen": "agusanar",
    "agusanes": "agusanar",
    "agusano": "agusanar",
    "agusanábamos": "agusanar",
    "agusanáis": "agusanar",
    "agusanáramos": "agusanar",
    "agusanáremos": "agusanar",
    "agusanásemos": "agusanar",
    "agusané": "agusanar",
    "agusanéis": "agusanar",
    "agusanó": "agusanar",
    "agutíes": "agutí",
    "agutís": "agutí",
    "aguza": "aguzar",
    "aguzaba": "aguzar",
    "aguzabais": "aguzar",
    "aguzaban": "aguzar",
    "aguzabas": "aguzar",
    "aguzad": "aguzar",
    "aguzada": "aguzar",
    "aguzadas": "aguzar",
    "aguzado": "aguzar",
    "aguzados": "aguzar",
    "aguzamientos": "aguzamiento",
    "aguzamos": "aguzar",
    "aguzan": "aguzar",
    "aguzando": "aguzar",
    "aguzara": "aguzar",
    "aguzarais": "aguzar",
    "aguzaran": "aguzar",
    "aguzaras": "aguzar",
    "aguzare": "aguzar",
    "aguzareis": "aguzar",
    "aguzaremos": "aguzar",
    "aguzaren": "aguzar",
    "aguzares": "aguzar",
    "aguzaron": "aguzar",
    "aguzará": "aguzar",
    "aguzarán": "aguzar",
    "aguzarás": "aguzar",
    "aguzaré": "aguzar",
    "aguzaréis": "aguzar",
    "aguzaría": "aguzar",
    "aguzaríais": "aguzar",
    "aguzaríamos": "aguzar",
    "aguzarían": "aguzar",
    "aguzarías": "aguzar",
    "aguzas": "aguzar",
    "aguzase": "aguzar",
    "aguzaseis": "aguzar",
    "aguzasen": "aguzar",
    "aguzases": "aguzar",
    "aguzaste": "aguzar",
    "aguzasteis": "aguzar",
    "aguzo": "aguzar",
    "aguzábamos": "aguzar",
    "aguzáis": "aguzar",
    "aguzáramos": "aguzar",
    "aguzáremos": "aguzar",
    "aguzásemos": "aguzar",
    "aguzó": "aguzar",
    "aguábamos": "aguar",
    "aguáis": "aguar",
    "aguáramos": "aguar",
    "aguáremos": "aguar",
    "aguásemos": "aguar",
    "aguó": "aguar",
    "agámidos": "agámido",
    "agás": "agá",
    "agónica": "agónico",
    "agónicas": "agónico",
    "agónicos": "agónico",
    "agüe": "aguar",
    "agüemos": "aguar",
    "agüen": "aguar",
    "agüera": "agorar",
    "agüeran": "agorar",
    "agüeras": "agorar",
    "agüere": "agorar",
    "agüeren": "agorar",
    "agüeres": "agorar",
    "agüero": "agorar",
    "agüeros": "agüero",
    "agües": "aguar",
    "agüistas": "agüista",
    "agüé": "aguar",
    "agüéis": "aguar",
    "ahecha": "ahechar",
    "ahechaba": "ahechar",
    "ahechabais": "ahechar",
    "ahechaban": "ahechar",
    "ahechabas": "ahechar",
    "ahechad": "ahechar",
    "ahechada": "ahechar",
    "ahechadas": "ahechar",
    "ahechado": "ahechar",
    "ahechados": "ahechar",
    "ahechaduras": "ahechadura",
    "ahechamos": "ahechar",
    "ahechan": "ahechar",
    "ahechando": "ahechar",
    "ahechara": "ahechar",
    "ahecharais": "ahechar",
    "ahecharan": "ahechar",
    "ahecharas": "ahechar",
    "ahechare": "ahechar",
    "ahechareis": "ahechar",
    "ahecharemos": "ahechar",
    "ahecharen": "ahechar",
    "ahechares": "ahechar",
    "ahecharon": "ahechar",
    "ahechará": "ahechar",
    "ahecharán": "ahechar",
    "ahecharás": "ahechar",
    "ahecharé": "ahechar",
    "ahecharéis": "ahechar",
    "ahecharía": "ahechar",
    "ahecharíais": "ahechar",
    "ahecharíamos": "ahechar",
    "ahecharían": "ahechar",
    "ahecharías": "ahechar",
    "ahechas": "ahechar",
    "ahechase": "ahechar",
    "ahechaseis": "ahechar",
    "ahechasen": "ahechar",
    "ahechases": "ahechar",
    "ahechaste": "ahechar",
    "ahechasteis": "ahechar",
    "aheche": "ahechar",
    "ahechemos": "ahechar",
    "ahechen": "ahechar",
    "aheches": "ahechar",
    "ahecho": "ahechar",
    "ahechos": "ahecho",
    "ahechábamos": "ahechar",
    "ahecháis": "ahechar",
    "ahecháramos": "ahechar",
    "ahecháremos": "ahechar",
    "ahechásemos": "ahechar",
    "aheché": "ahechar",
    "ahechéis": "ahechar",
    "ahechó": "ahechar",
    "ahelea": "ahelear",
    "aheleaba": "ahelear",
    "aheleabais": "ahelear",
    "aheleaban": "ahelear",
    "aheleabas": "ahelear",
    "ahelead": "ahelear",
    "aheleada": "ahelear",
    "aheleadas": "ahelear",
    "aheleado": "ahelear",
    "aheleados": "ahelear",
    "aheleamos": "ahelear",
    "ahelean": "ahelear",
    "aheleando": "ahelear",
    "aheleara": "ahelear",
    "ahelearais": "ahelear",
    "ahelearan": "ahelear",
    "ahelearas": "ahelear",
    "aheleare": "ahelear",
    "aheleareis": "ahelear",
    "ahelearemos": "ahelear",
    "ahelearen": "ahelear",
    "aheleares": "ahelear",
    "ahelearon": "ahelear",
    "aheleará": "ahelear",
    "ahelearán": "ahelear",
    "ahelearás": "ahelear",
    "ahelearé": "ahelear",
    "ahelearéis": "ahelear",
    "ahelearía": "ahelear",
    "ahelearíais": "ahelear",
    "ahelearíamos": "ahelear",
    "ahelearían": "ahelear",
    "ahelearías": "ahelear",
    "aheleas": "ahelear",
    "ahelease": "ahelear",
    "aheleaseis": "ahelear",
    "aheleasen": "ahelear",
    "aheleases": "ahelear",
    "aheleaste": "ahelear",
    "aheleasteis": "ahelear",
    "ahelee": "ahelear",
    "aheleemos": "ahelear",
    "aheleen": "ahelear",
    "ahelees": "ahelear",
    "aheleo": "ahelear",
    "aheleábamos": "ahelear",
    "aheleáis": "ahelear",
    "aheleáramos": "ahelear",
    "aheleáremos": "ahelear",
    "aheleásemos": "ahelear",
    "aheleé": "ahelear",
    "aheleéis": "ahelear",
    "aheleó": "ahelear",
    "aherroja": "aherrojar",
    "aherrojaba": "aherrojar",
    "aherrojabais": "aherrojar",
    "aherrojaban": "aherrojar",
    "aherrojabas": "aherrojar",
    "aherrojad": "aherrojar",
    "aherrojada": "aherrojar",
    "aherrojadas": "aherrojar",
    "aherrojado": "aherrojar",
    "aherrojados": "aherrojar",
    "aherrojamientos": "aherrojamiento",
    "aherrojamos": "aherrojar",
    "aherrojan": "aherrojar",
    "aherrojando": "aherrojar",
    "aherrojara": "aherrojar",
    "aherrojarais": "aherrojar",
    "aherrojaran": "aherrojar",
    "aherrojaras": "aherrojar",
    "aherrojare": "aherrojar",
    "aherrojareis": "aherrojar",
    "aherrojaremos": "aherrojar",
    "aherrojaren": "aherrojar",
    "aherrojares": "aherrojar",
    "aherrojaron": "aherrojar",
    "aherrojará": "aherrojar",
    "aherrojarán": "aherrojar",
    "aherrojarás": "aherrojar",
    "aherrojaré": "aherrojar",
    "aherrojaréis": "aherrojar",
    "aherrojaría": "aherrojar",
    "aherrojaríais": "aherrojar",
    "aherrojaríamos": "aherrojar",
    "aherrojarían": "aherrojar",
    "aherrojarías": "aherrojar",
    "aherrojas": "aherrojar",
    "aherrojase": "aherrojar",
    "aherrojaseis": "aherrojar",
    "aherrojasen": "aherrojar",
    "aherrojases": "aherrojar",
    "aherrojaste": "aherrojar",
    "aherrojasteis": "aherrojar",
    "aherroje": "aherrojar",
    "aherrojemos": "aherrojar",
    "aherrojen": "aherrojar",
    "aherrojes": "aherrojar",
    "aherrojo": "aherrojar",
    "aherrojábamos": "aherrojar",
    "aherrojáis": "aherrojar",
    "aherrojáramos": "aherrojar",
    "aherrojáremos": "aherrojar",
    "aherrojásemos": "aherrojar",
    "aherrojé": "aherrojar",
    "aherrojéis": "aherrojar",
    "aherrojó": "aherrojar",
    "aherrumbra": "aherrumbrar",
    "aherrumbraba": "aherrumbrar",
    "aherrumbrabais": "aherrumbrar",
    "aherrumbraban": "aherrumbrar",
    "aherrumbrabas": "aherrumbrar",
    "aherrumbrad": "aherrumbrar",
    "aherrumbrada": "aherrumbrar",
    "aherrumbradas": "aherrumbrar",
    "aherrumbrado": "aherrumbrar",
    "aherrumbrados": "aherrumbrar",
    "aherrumbramos": "aherrumbrar",
    "aherrumbran": "aherrumbrar",
    "aherrumbrando": "aherrumbrar",
    "aherrumbrara": "aherrumbrar",
    "aherrumbrarais": "aherrumbrar",
    "aherrumbraran": "aherrumbrar",
    "aherrumbraras": "aherrumbrar",
    "aherrumbrare": "aherrumbrar",
    "aherrumbrareis": "aherrumbrar",
    "aherrumbraremos": "aherrumbrar",
    "aherrumbraren": "aherrumbrar",
    "aherrumbrares": "aherrumbrar",
    "aherrumbraron": "aherrumbrar",
    "aherrumbrará": "aherrumbrar",
    "aherrumbrarán": "aherrumbrar",
    "aherrumbrarás": "aherrumbrar",
    "aherrumbraré": "aherrumbrar",
    "aherrumbraréis": "aherrumbrar",
    "aherrumbraría": "aherrumbrar",
    "aherrumbraríais": "aherrumbrar",
    "aherrumbraríamos": "aherrumbrar",
    "aherrumbrarían": "aherrumbrar",
    "aherrumbrarías": "aherrumbrar",
    "aherrumbras": "aherrumbrar",
    "aherrumbrase": "aherrumbrar",
    "aherrumbraseis": "aherrumbrar",
    "aherrumbrasen": "aherrumbrar",
    "aherrumbrases": "aherrumbrar",
    "aherrumbraste": "aherrumbrar",
    "aherrumbrasteis": "aherrumbrar",
    "aherrumbre": "aherrumbrar",
    "aherrumbremos": "aherrumbrar",
    "aherrumbren": "aherrumbrar",
    "aherrumbres": "aherrumbrar",
    "aherrumbro": "aherrumbrar",
    "aherrumbrábamos": "aherrumbrar",
    "aherrumbráis": "aherrumbrar",
    "aherrumbráramos": "aherrumbrar",
    "aherrumbráremos": "aherrumbrar",
    "aherrumbrásemos": "aherrumbrar",
    "aherrumbré": "aherrumbrar",
    "aherrumbréis": "aherrumbrar",
    "aherrumbró": "aherrumbrar",
    "ahervora": "ahervorar",
    "ahervoraba": "ahervorar",
    "ahervorabais": "ahervorar",
    "ahervoraban": "ahervorar",
    "ahervorabas": "ahervorar",
    "ahervorad": "ahervorar",
    "ahervorada": "ahervorar",
    "ahervoradas": "ahervorar",
    "ahervorado": "ahervorar",
    "ahervorados": "ahervorar",
    "ahervoramos": "ahervorar",
    "ahervoran": "ahervorar",
    "ahervorando": "ahervorar",
    "ahervorara": "ahervorar",
    "ahervorarais": "ahervorar",
    "ahervoraran": "ahervorar",
    "ahervoraras": "ahervorar",
    "ahervorare": "ahervorar",
    "ahervorareis": "ahervorar",
    "ahervoraremos": "ahervorar",
    "ahervoraren": "ahervorar",
    "ahervorares": "ahervorar",
    "ahervoraron": "ahervorar",
    "ahervorará": "ahervorar",
    "ahervorarán": "ahervorar",
    "ahervorarás": "ahervorar",
    "ahervoraré": "ahervorar",
    "ahervoraréis": "ahervorar",
    "ahervoraría": "ahervorar",
    "ahervoraríais": "ahervorar",
    "ahervoraríamos": "ahervorar",
    "ahervorarían": "ahervorar",
    "ahervorarías": "ahervorar",
    "ahervoras": "ahervorar",
    "ahervorase": "ahervorar",
    "ahervoraseis": "ahervorar",
    "ahervorasen": "ahervorar",
    "ahervorases": "ahervorar",
    "ahervoraste": "ahervorar",
    "ahervorasteis": "ahervorar",
    "ahervore": "ahervorar",
    "ahervoremos": "ahervorar",
    "ahervoren": "ahervorar",
    "ahervores": "ahervorar",
    "ahervoro": "ahervorar",
    "ahervorábamos": "ahervorar",
    "ahervoráis": "ahervorar",
    "ahervoráramos": "ahervorar",
    "ahervoráremos": "ahervorar",
    "ahervorásemos": "ahervorar",
    "ahervoré": "ahervorar",
    "ahervoréis": "ahervorar",
    "ahervoró": "ahervorar",
    "ahijaba": "ahijar",
    "ahijabais": "ahijar",
    "ahijaban": "ahijar",
    "ahijabas": "ahijar",
    "ahijad": "ahijar",
    "ahijada": "ahijar",
    "ahijadas": "ahijar",
    "ahijado": "ahijar",
    "ahijados": "ahijar",
    "ahijamos": "ahijar",
    "ahijando": "ahijar",
    "ahijara": "ahijar",
    "ahijarais": "ahijar",
    "ahijaran": "ahijar",
    "ahijaras": "ahijar",
    "ahijare": "ahijar",
    "ahijareis": "ahijar",
    "ahijaremos": "ahijar",
    "ahijaren": "ahijar",
    "ahijares": "ahijar",
    "ahijaron": "ahijar",
    "ahijará": "ahijar",
    "ahijarán": "ahijar",
    "ahijarás": "ahijar",
    "ahijaré": "ahijar",
    "ahijaréis": "ahijar",
    "ahijaría": "ahijar",
    "ahijaríais": "ahijar",
    "ahijaríamos": "ahijar",
    "ahijarían": "ahijar",
    "ahijarías": "ahijar",
    "ahijase": "ahijar",
    "ahijaseis": "ahijar",
    "ahijasen": "ahijar",
    "ahijases": "ahijar",
    "ahijaste": "ahijar",
    "ahijasteis": "ahijar",
    "ahijemos": "ahijar",
    "ahijábamos": "ahijar",
    "ahijáis": "ahijar",
    "ahijáramos": "ahijar",
    "ahijáremos": "ahijar",
    "ahijásemos": "ahijar",
    "ahijé": "ahijar",
    "ahijéis": "ahijar",
    "ahijó": "ahijar",
    "ahilaba": "ahilar",
    "ahilabais": "ahilar",
    "ahilaban": "ahilar",
    "ahilabas": "ahilar",
    "ahilad": "ahilar",
    "ahilada": "ahilar",
    "ahiladas": "ahilar",
    "ahilado": "ahilar",
    "ahilados": "ahilar",
    "ahilamientos": "ahilamiento",
    "ahilamos": "ahilar",
    "ahilando": "ahilar",
    "ahilara": "ahilar",
    "ahilarais": "ahilar",
    "ahilaran": "ahilar",
    "ahilaras": "ahilar",
    "ahilare": "ahilar",
    "ahilareis": "ahilar",
    "ahilaremos": "ahilar",
    "ahilaren": "ahilar",
    "ahilares": "ahilar",
    "ahilaron": "ahilar",
    "ahilará": "ahilar",
    "ahilarán": "ahilar",
    "ahilarás": "ahilar",
    "ahilaré": "ahilar",
    "ahilaréis": "ahilar",
    "ahilaría": "ahilar",
    "ahilaríais": "ahilar",
    "ahilaríamos": "ahilar",
    "ahilarían": "ahilar",
    "ahilarías": "ahilar",
    "ahilase": "ahilar",
    "ahilaseis": "ahilar",
    "ahilasen": "ahilar",
    "ahilases": "ahilar",
    "ahilaste": "ahilar",
    "ahilasteis": "ahilar",
    "ahilemos": "ahilar",
    "ahilábamos": "ahilar",
    "ahiláis": "ahilar",
    "ahiláramos": "ahilar",
    "ahiláremos": "ahilar",
    "ahilásemos": "ahilar",
    "ahilé": "ahilar",
    "ahiléis": "ahilar",
    "ahiló": "ahilar",
    "ahincaba": "ahincar",
    "ahincabais": "ahincar",
    "ahincaban": "ahincar",
    "ahincabas": "ahincar",
    "ahincad": "ahincar",
    "ahincada": "ahincar",
    "ahincadas": "ahincar",
    "ahincado": "ahincar",
    "ahincados": "ahincar",
    "ahincamos": "ahincar",
    "ahincando": "ahincar",
    "ahincara": "ahincar",
    "ahincarais": "ahincar",
    "ahincaran": "ahincar",
    "ahincaras": "ahincar",
    "ahincare": "ahincar",
    "ahincareis": "ahincar",
    "ahincaremos": "ahincar",
    "ahincaren": "ahincar",
    "ahincares": "ahincar",
    "ahincaron": "ahincar",
    "ahincará": "ahincar",
    "ahincarán": "ahincar",
    "ahincarás": "ahincar",
    "ahincaré": "ahincar",
    "ahincaréis": "ahincar",
    "ahincaría": "ahincar",
    "ahincaríais": "ahincar",
    "ahincaríamos": "ahincar",
    "ahincarían": "ahincar",
    "ahincarías": "ahincar",
    "ahincase": "ahincar",
    "ahincaseis": "ahincar",
    "ahincasen": "ahincar",
    "ahincases": "ahincar",
    "ahincaste": "ahincar",
    "ahincasteis": "ahincar",
    "ahincábamos": "ahincar",
    "ahincáis": "ahincar",
    "ahincáramos": "ahincar",
    "ahincáremos": "ahincar",
    "ahincásemos": "ahincar",
    "ahincó": "ahincar",
    "ahinoja": "ahinojar",
    "ahinojaba": "ahinojar",
    "ahinojabais": "ahinojar",
    "ahinojaban": "ahinojar",
    "ahinojabas": "ahinojar",
    "ahinojad": "ahinojar",
    "ahinojada": "ahinojar",
    "ahinojadas": "ahinojar",
    "ahinojado": "ahinojar",
    "ahinojados": "ahinojar",
    "ahinojamos": "ahinojar",
    "ahinojan": "ahinojar",
    "ahinojando": "ahinojar",
    "ahinojara": "ahinojar",
    "ahinojarais": "ahinojar",
    "ahinojaran": "ahinojar",
    "ahinojaras": "ahinojar",
    "ahinojare": "ahinojar",
    "ahinojareis": "ahinojar",
    "ahinojaremos": "ahinojar",
    "ahinojaren": "ahinojar",
    "ahinojares": "ahinojar",
    "ahinojaron": "ahinojar",
    "ahinojará": "ahinojar",
    "ahinojarán": "ahinojar",
    "ahinojarás": "ahinojar",
    "ahinojaré": "ahinojar",
    "ahinojaréis": "ahinojar",
    "ahinojaría": "ahinojar",
    "ahinojaríais": "ahinojar",
    "ahinojaríamos": "ahinojar",
    "ahinojarían": "ahinojar",
    "ahinojarías": "ahinojar",
    "ahinojas": "ahinojar",
    "ahinojase": "ahinojar",
    "ahinojaseis": "ahinojar",
    "ahinojasen": "ahinojar",
    "ahinojases": "ahinojar",
    "ahinojaste": "ahinojar",
    "ahinojasteis": "ahinojar",
    "ahinoje": "ahinojar",
    "ahinojemos": "ahinojar",
    "ahinojen": "ahinojar",
    "ahinojes": "ahinojar",
    "ahinojo": "ahinojar",
    "ahinojábamos": "ahinojar",
    "ahinojáis": "ahinojar",
    "ahinojáramos": "ahinojar",
    "ahinojáremos": "ahinojar",
    "ahinojásemos": "ahinojar",
    "ahinojé": "ahinojar",
    "ahinojéis": "ahinojar",
    "ahinojó": "ahinojar",
    "ahinquemos": "ahincar",
    "ahinqué": "ahincar",
    "ahinquéis": "ahincar",
    "ahitaba": "ahitar",
    "ahitabais": "ahitar",
    "ahitaban": "ahitar",
    "ahitabas": "ahitar",
    "ahitad": "ahitar",
    "ahitada": "ahitar",
    "ahitadas": "ahitar",
    "ahitado": "ahitar",
    "ahitados": "ahitar",
    "ahitamos": "ahitar",
    "ahitando": "ahitar",
    "ahitara": "ahitar",
    "ahitarais": "ahitar",
    "ahitaran": "ahitar",
    "ahitaras": "ahitar",
    "ahitare": "ahitar",
    "ahitareis": "ahitar",
    "ahitaremos": "ahitar",
    "ahitaren": "ahitar",
    "ahitares": "ahitar",
    "ahitaron": "ahitar",
    "ahitará": "ahitar",
    "ahitarán": "ahitar",
    "ahitarás": "ahitar",
    "ahitaré": "ahitar",
    "ahitaréis": "ahitar",
    "ahitaría": "ahitar",
    "ahitaríais": "ahitar",
    "ahitaríamos": "ahitar",
    "ahitarían": "ahitar",
    "ahitarías": "ahitar",
    "ahitase": "ahitar",
    "ahitaseis": "ahitar",
    "ahitasen": "ahitar",
    "ahitases": "ahitar",
    "ahitaste": "ahitar",
    "ahitasteis": "ahitar",
    "ahitemos": "ahitar",
    "ahiteras": "ahitera",
    "ahitábamos": "ahitar",
    "ahitáis": "ahitar",
    "ahitáramos": "ahitar",
    "ahitáremos": "ahitar",
    "ahitásemos": "ahitar",
    "ahité": "ahitar",
    "ahitéis": "ahitar",
    "ahitó": "ahitar",
    "ahocica": "ahocicar",
    "ahocicaba": "ahocicar",
    "ahocicabais": "ahocicar",
    "ahocicaban": "ahocicar",
    "ahocicabas": "ahocicar",
    "ahocicad": "ahocicar",
    "ahocicada": "ahocicar",
    "ahocicadas": "ahocicar",
    "ahocicado": "ahocicar",
    "ahocicados": "ahocicar",
    "ahocicamos": "ahocicar",
    "ahocican": "ahocicar",
    "ahocicando": "ahocicar",
    "ahocicara": "ahocicar",
    "ahocicarais": "ahocicar",
    "ahocicaran": "ahocicar",
    "ahocicaras": "ahocicar",
    "ahocicare": "ahocicar",
    "ahocicareis": "ahocicar",
    "ahocicaremos": "ahocicar",
    "ahocicaren": "ahocicar",
    "ahocicares": "ahocicar",
    "ahocicaron": "ahocicar",
    "ahocicará": "ahocicar",
    "ahocicarán": "ahocicar",
    "ahocicarás": "ahocicar",
    "ahocicaré": "ahocicar",
    "ahocicaréis": "ahocicar",
    "ahocicaría": "ahocicar",
    "ahocicaríais": "ahocicar",
    "ahocicaríamos": "ahocicar",
    "ahocicarían": "ahocicar",
    "ahocicarías": "ahocicar",
    "ahocicas": "ahocicar",
    "ahocicase": "ahocicar",
    "ahocicaseis": "ahocicar",
    "ahocicasen": "ahocicar",
    "ahocicases": "ahocicar",
    "ahocicaste": "ahocicar",
    "ahocicasteis": "ahocicar",
    "ahocico": "ahocicar",
    "ahocicábamos": "ahocicar",
    "ahocicáis": "ahocicar",
    "ahocicáramos": "ahocicar",
    "ahocicáremos": "ahocicar",
    "ahocicásemos": "ahocicar",
    "ahocicó": "ahocicar",
    "ahocina": "ahocinar",
    "ahocinaba": "ahocinar",
    "ahocinabais": "ahocinar",
    "ahocinaban": "ahocinar",
    "ahocinabas": "ahocinar",
    "ahocinad": "ahocinar",
    "ahocinada": "ahocinar",
    "ahocinadas": "ahocinar",
    "ahocinado": "ahocinar",
    "ahocinados": "ahocinar",
    "ahocinamos": "ahocinar",
    "ahocinan": "ahocinar",
    "ahocinando": "ahocinar",
    "ahocinara": "ahocinar",
    "ahocinarais": "ahocinar",
    "ahocinaran": "ahocinar",
    "ahocinaras": "ahocinar",
    "ahocinare": "ahocinar",
    "ahocinareis": "ahocinar",
    "ahocinaremos": "ahocinar",
    "ahocinaren": "ahocinar",
    "ahocinares": "ahocinar",
    "ahocinaron": "ahocinar",
    "ahocinará": "ahocinar",
    "ahocinarán": "ahocinar",
    "ahocinarás": "ahocinar",
    "ahocinaré": "ahocinar",
    "ahocinaréis": "ahocinar",
    "ahocinaría": "ahocinar",
    "ahocinaríais": "ahocinar",
    "ahocinaríamos": "ahocinar",
    "ahocinarían": "ahocinar",
    "ahocinarías": "ahocinar",
    "ahocinas": "ahocinar",
    "ahocinase": "ahocinar",
    "ahocinaseis": "ahocinar",
    "ahocinasen": "ahocinar",
    "ahocinases": "ahocinar",
    "ahocinaste": "ahocinar",
    "ahocinasteis": "ahocinar",
    "ahocine": "ahocinar",
    "ahocinemos": "ahocinar",
    "ahocinen": "ahocinar",
    "ahocines": "ahocinar",
    "ahocino": "ahocinar",
    "ahocinábamos": "ahocinar",
    "ahocináis": "ahocinar",
    "ahocináramos": "ahocinar",
    "ahocináremos": "ahocinar",
    "ahocinásemos": "ahocinar",
    "ahociné": "ahocinar",
    "ahocinéis": "ahocinar",
    "ahocinó": "ahocinar",
    "ahocique": "ahocicar",
    "ahociquemos": "ahocicar",
    "ahociquen": "ahocicar",
    "ahociques": "ahocicar",
    "ahociqué": "ahocicar",
    "ahociquéis": "ahocicar",
    "ahoga": "ahogar",
    "ahogaba": "ahogar",
    "ahogabais": "ahogar",
    "ahogaban": "ahogar",
    "ahogabas": "ahogar",
    "ahogad": "ahogar",
    "ahogada": "ahogar",
    "ahogadas": "ahogar",
    "ahogaderos": "ahogadero",
    "ahogadiza": "ahogadizo",
    "ahogadizas": "ahogadizo",
    "ahogadizos": "ahogadizo",
    "ahogado": "ahogar",
    "ahogados": "ahogar",
    "ahogamientos": "ahogamiento",
    "ahogamos": "ahogar",
    "ahogan": "ahogar",
    "ahogando": "ahogar",
    "ahogara": "ahogar",
    "ahogarais": "ahogar",
    "ahogaran": "ahogar",
    "ahogaras": "ahogar",
    "ahogare": "ahogar",
    "ahogareis": "ahogar",
    "ahogaremos": "ahogar",
    "ahogaren": "ahogar",
    "ahogares": "ahogar",
    "ahogaron": "ahogar",
    "ahogará": "ahogar",
    "ahogarán": "ahogar",
    "ahogarás": "ahogar",
    "ahogaré": "ahogar",
    "ahogaréis": "ahogar",
    "ahogaría": "ahogar",
    "ahogaríais": "ahogar",
    "ahogaríamos": "ahogar",
    "ahogarían": "ahogar",
    "ahogarías": "ahogar",
    "ahogas": "ahogar",
    "ahogase": "ahogar",
    "ahogaseis": "ahogar",
    "ahogasen": "ahogar",
    "ahogases": "ahogar",
    "ahogaste": "ahogar",
    "ahogasteis": "ahogar",
    "ahogo": "ahogar",
    "ahogos": "ahogo",
    "ahogue": "ahogar",
    "ahoguemos": "ahogar",
    "ahoguen": "ahogar",
    "ahogues": "ahogar",
    "ahogué": "ahogar",
    "ahoguéis": "ahogar",
    "ahoguíos": "ahoguío",
    "ahogábamos": "ahogar",
    "ahogáis": "ahogar",
    "ahogáramos": "ahogar",
    "ahogáremos": "ahogar",
    "ahogásemos": "ahogar",
    "ahogó": "ahogar",
    "ahonda": "ahondar",
    "ahondaba": "ahondar",
    "ahondabais": "ahondar",
    "ahondaban": "ahondar",
    "ahondabas": "ahondar",
    "ahondad": "ahondar",
    "ahondada": "ahondar",
    "ahondadas": "ahondar",
    "ahondado": "ahondar",
    "ahondados": "ahondar",
    "ahondamientos": "ahondamiento",
    "ahondamos": "ahondar",
    "ahondan": "ahondar",
    "ahondando": "ahondar",
    "ahondara": "ahondar",
    "ahondarais": "ahondar",
    "ahondaran": "ahondar",
    "ahondaras": "ahondar",
    "ahondare": "ahondar",
    "ahondareis": "ahondar",
    "ahondaremos": "ahondar",
    "ahondaren": "ahondar",
    "ahondares": "ahondar",
    "ahondaron": "ahondar",
    "ahondará": "ahondar",
    "ahondarán": "ahondar",
    "ahondarás": "ahondar",
    "ahondaré": "ahondar",
    "ahondaréis": "ahondar",
    "ahondaría": "ahondar",
    "ahondaríais": "ahondar",
    "ahondaríamos": "ahondar",
    "ahondarían": "ahondar",
    "ahondarías": "ahondar",
    "ahondas": "ahondar",
    "ahondase": "ahondar",
    "ahondaseis": "ahondar",
    "ahondasen": "ahondar",
    "ahondases": "ahondar",
    "ahondaste": "ahondar",
    "ahondasteis": "ahondar",
    "ahonde": "ahondar",
    "ahondemos": "ahondar",
    "ahonden": "ahondar",
    "ahondes": "ahonde",
    "ahondo": "ahondar",
    "ahondábamos": "ahondar",
    "ahondáis": "ahondar",
    "ahondáramos": "ahondar",
    "ahondáremos": "ahondar",
    "ahondásemos": "ahondar",
    "ahondé": "ahondar",
    "ahondéis": "ahondar",
    "ahondó": "ahondar",
    "ahorca": "ahorcar",
    "ahorcaba": "ahorcar",
    "ahorcabais": "ahorcar",
    "ahorcaban": "ahorcar",
    "ahorcabas": "ahorcar",
    "ahorcad": "ahorcar",
    "ahorcada": "ahorcar",
    "ahorcadas": "ahorcar",
    "ahorcado": "ahorcar",
    "ahorcados": "ahorcar",
    "ahorcaduras": "ahorcadura",
    "ahorcaja": "ahorcajar",
    "ahorcajaba": "ahorcajar",
    "ahorcajabais": "ahorcajar",
    "ahorcajaban": "ahorcajar",
    "ahorcajabas": "ahorcajar",
    "ahorcajad": "ahorcajar",
    "ahorcajada": "ahorcajar",
    "ahorcajadas": "ahorcajar",
    "ahorcajado": "ahorcajar",
    "ahorcajados": "ahorcajar",
    "ahorcajamos": "ahorcajar",
    "ahorcajan": "ahorcajar",
    "ahorcajando": "ahorcajar",
    "ahorcajara": "ahorcajar",
    "ahorcajarais": "ahorcajar",
    "ahorcajaran": "ahorcajar",
    "ahorcajaras": "ahorcajar",
    "ahorcajare": "ahorcajar",
    "ahorcajareis": "ahorcajar",
    "ahorcajaremos": "ahorcajar",
    "ahorcajaren": "ahorcajar",
    "ahorcajares": "ahorcajar",
    "ahorcajaron": "ahorcajar",
    "ahorcajará": "ahorcajar",
    "ahorcajarán": "ahorcajar",
    "ahorcajarás": "ahorcajar",
    "ahorcajaré": "ahorcajar",
    "ahorcajaréis": "ahorcajar",
    "ahorcajaría": "ahorcajar",
    "ahorcajaríais": "ahorcajar",
    "ahorcajaríamos": "ahorcajar",
    "ahorcajarían": "ahorcajar",
    "ahorcajarías": "ahorcajar",
    "ahorcajas": "ahorcajar",
    "ahorcajase": "ahorcajar",
    "ahorcajaseis": "ahorcajar",
    "ahorcajasen": "ahorcajar",
    "ahorcajases": "ahorcajar",
    "ahorcajaste": "ahorcajar",
    "ahorcajasteis": "ahorcajar",
    "ahorcaje": "ahorcajar",
    "ahorcajemos": "ahorcajar",
    "ahorcajen": "ahorcajar",
    "ahorcajes": "ahorcajar",
    "ahorcajo": "ahorcajar",
    "ahorcajábamos": "ahorcajar",
    "ahorcajáis": "ahorcajar",
    "ahorcajáramos": "ahorcajar",
    "ahorcajáremos": "ahorcajar",
    "ahorcajásemos": "ahorcajar",
    "ahorcajé": "ahorcajar",
    "ahorcajéis": "ahorcajar",
    "ahorcajó": "ahorcajar",
    "ahorcamientos": "ahorcamiento",
    "ahorcamos": "ahorcar",
    "ahorcan": "ahorcar",
    "ahorcando": "ahorcar",
    "ahorcara": "ahorcar",
    "ahorcarais": "ahorcar",
    "ahorcaran": "ahorcar",
    "ahorcaras": "ahorcar",
    "ahorcare": "ahorcar",
    "ahorcareis": "ahorcar",
    "ahorcaremos": "ahorcar",
    "ahorcaren": "ahorcar",
    "ahorcares": "ahorcar",
    "ahorcaron": "ahorcar",
    "ahorcará": "ahorcar",
    "ahorcarán": "ahorcar",
    "ahorcarás": "ahorcar",
    "ahorcaré": "ahorcar",
    "ahorcaréis": "ahorcar",
    "ahorcaría": "ahorcar",
    "ahorcaríais": "ahorcar",
    "ahorcaríamos": "ahorcar",
    "ahorcarían": "ahorcar",
    "ahorcarías": "ahorcar",
    "ahorcas": "ahorcar",
    "ahorcase": "ahorcar",
    "ahorcaseis": "ahorcar",
    "ahorcasen": "ahorcar",
    "ahorcases": "ahorcar",
    "ahorcaste": "ahorcar",
    "ahorcasteis": "ahorcar",
    "ahorco": "ahorcar",
    "ahorcábamos": "ahorcar",
    "ahorcáis": "ahorcar",
    "ahorcáramos": "ahorcar",
    "ahorcáremos": "ahorcar",
    "ahorcásemos": "ahorcar",
    "ahorcó": "ahorcar",
    "ahorma": "ahormar",
    "ahormaba": "ahormar",
    "ahormabais": "ahormar",
    "ahormaban": "ahormar",
    "ahormabas": "ahormar",
    "ahormad": "ahormar",
    "ahormada": "ahormar",
    "ahormadas": "ahormar",
    "ahormado": "ahormar",
    "ahormados": "ahormar",
    "ahormamos": "ahormar",
    "ahorman": "ahormar",
    "ahormando": "ahormar",
    "ahormara": "ahormar",
    "ahormarais": "ahormar",
    "ahormaran": "ahormar",
    "ahormaras": "ahormar",
    "ahormare": "ahormar",
    "ahormareis": "ahormar",
    "ahormaremos": "ahormar",
    "ahormaren": "ahormar",
    "ahormares": "ahormar",
    "ahormaron": "ahormar",
    "ahormará": "ahormar",
    "ahormarán": "ahormar",
    "ahormarás": "ahormar",
    "ahormaré": "ahormar",
    "ahormaréis": "ahormar",
    "ahormaría": "ahormar",
    "ahormaríais": "ahormar",
    "ahormaríamos": "ahormar",
    "ahormarían": "ahormar",
    "ahormarías": "ahormar",
    "ahormas": "ahormar",
    "ahormase": "ahormar",
    "ahormaseis": "ahormar",
    "ahormasen": "ahormar",
    "ahormases": "ahormar",
    "ahormaste": "ahormar",
    "ahormasteis": "ahormar",
    "ahorme": "ahormar",
    "ahormemos": "ahormar",
    "ahormen": "ahormar",
    "ahormes": "ahormar",
    "ahormo": "ahormar",
    "ahormábamos": "ahormar",
    "ahormáis": "ahormar",
    "ahormáramos": "ahormar",
    "ahormáremos": "ahormar",
    "ahormásemos": "ahormar",
    "ahormé": "ahormar",
    "ahorméis": "ahormar",
    "ahormó": "ahormar",
    "ahornaga": "ahornagar",
    "ahornagaba": "ahornagar",
    "ahornagabais": "ahornagar",
    "ahornagaban": "ahornagar",
    "ahornagabas": "ahornagar",
    "ahornagad": "ahornagar",
    "ahornagada": "ahornagar",
    "ahornagadas": "ahornagar",
    "ahornagado": "ahornagar",
    "ahornagados": "ahornagar",
    "ahornagamos": "ahornagar",
    "ahornagan": "ahornagar",
    "ahornagando": "ahornagar",
    "ahornagara": "ahornagar",
    "ahornagarais": "ahornagar",
    "ahornagaran": "ahornagar",
    "ahornagaras": "ahornagar",
    "ahornagare": "ahornagar",
    "ahornagareis": "ahornagar",
    "ahornagaremos": "ahornagar",
    "ahornagaren": "ahornagar",
    "ahornagares": "ahornagar",
    "ahornagaron": "ahornagar",
    "ahornagará": "ahornagar",
    "ahornagarán": "ahornagar",
    "ahornagarás": "ahornagar",
    "ahornagaré": "ahornagar",
    "ahornagaréis": "ahornagar",
    "ahornagaría": "ahornagar",
    "ahornagaríais": "ahornagar",
    "ahornagaríamos": "ahornagar",
    "ahornagarían": "ahornagar",
    "ahornagarías": "ahornagar",
    "ahornagas": "ahornagar",
    "ahornagase": "ahornagar",
    "ahornagaseis": "ahornagar",
    "ahornagasen": "ahornagar",
    "ahornagases": "ahornagar",
    "ahornagaste": "ahornagar",
    "ahornagasteis": "ahornagar",
    "ahornago": "ahornagar",
    "ahornague": "ahornagar",
    "ahornaguemos": "ahornagar",
    "ahornaguen": "ahornagar",
    "ahornagues": "ahornagar",
    "ahornagué": "ahornagar",
    "ahornaguéis": "ahornagar",
    "ahornagábamos": "ahornagar",
    "ahornagáis": "ahornagar",
    "ahornagáramos": "ahornagar",
    "ahornagáremos": "ahornagar",
    "ahornagásemos": "ahornagar",
    "ahornagó": "ahornagar",
    "ahorque": "ahorcar",
    "ahorquemos": "ahorcar",
    "ahorquen": "ahorcar",
    "ahorques": "ahorcar",
    "ahorquilla": "ahorquillar",
    "ahorquillaba": "ahorquillar",
    "ahorquillabais": "ahorquillar",
    "ahorquillaban": "ahorquillar",
    "ahorquillabas": "ahorquillar",
    "ahorquillad": "ahorquillar",
    "ahorquillada": "ahorquillar",
    "ahorquilladas": "ahorquillar",
    "ahorquillado": "ahorquillar",
    "ahorquillados": "ahorquillar",
    "ahorquillamos": "ahorquillar",
    "ahorquillan": "ahorquillar",
    "ahorquillando": "ahorquillar",
    "ahorquillara": "ahorquillar",
    "ahorquillarais": "ahorquillar",
    "ahorquillaran": "ahorquillar",
    "ahorquillaras": "ahorquillar",
    "ahorquillare": "ahorquillar",
    "ahorquillareis": "ahorquillar",
    "ahorquillaremos": "ahorquillar",
    "ahorquillaren": "ahorquillar",
    "ahorquillares": "ahorquillar",
    "ahorquillaron": "ahorquillar",
    "ahorquillará": "ahorquillar",
    "ahorquillarán": "ahorquillar",
    "ahorquillarás": "ahorquillar",
    "ahorquillaré": "ahorquillar",
    "ahorquillaréis": "ahorquillar",
    "ahorquillaría": "ahorquillar",
    "ahorquillaríais": "ahorquillar",
    "ahorquillaríamos": "ahorquillar",
    "ahorquillarían": "ahorquillar",
    "ahorquillarías": "ahorquillar",
    "ahorquillas": "ahorquillar",
    "ahorquillase": "ahorquillar",
    "ahorquillaseis": "ahorquillar",
    "ahorquillasen": "ahorquillar",
    "ahorquillases": "ahorquillar",
    "ahorquillaste": "ahorquillar",
    "ahorquillasteis": "ahorquillar",
    "ahorquille": "ahorquillar",
    "ahorquillemos": "ahorquillar",
    "ahorquillen": "ahorquillar",
    "ahorquilles": "ahorquillar",
    "ahorquillo": "ahorquillar",
    "ahorquillábamos": "ahorquillar",
    "ahorquilláis": "ahorquillar",
    "ahorquilláramos": "ahorquillar",
    "ahorquilláremos": "ahorquillar",
    "ahorquillásemos": "ahorquillar",
    "ahorquillé": "ahorquillar",
    "ahorquilléis": "ahorquillar",
    "ahorquilló": "ahorquillar",
    "ahorqué": "ahorcar",
    "ahorquéis": "ahorcar",
    "ahorra": "ahorrar",
    "ahorraba": "ahorrar",
    "ahorrabais": "ahorrar",
    "ahorraban": "ahorrar",
    "ahorrabas": "ahorrar",
    "ahorrad": "ahorrar",
    "ahorrada": "ahorrar",
    "ahorradas": "ahorrar",
    "ahorrado": "ahorrar",
    "ahorradora": "ahorrador",
    "ahorradoras": "ahorrador",
    "ahorradores": "ahorrador",
    "ahorrados": "ahorrar",
    "ahorramientos": "ahorramiento",
    "ahorramos": "ahorrar",
    "ahorran": "ahorrar",
    "ahorrando": "ahorrar",
    "ahorrara": "ahorrar",
    "ahorrarais": "ahorrar",
    "ahorraran": "ahorrar",
    "ahorraras": "ahorrar",
    "ahorrare": "ahorrar",
    "ahorrareis": "ahorrar",
    "ahorraremos": "ahorrar",
    "ahorraren": "ahorrar",
    "ahorrares": "ahorrar",
    "ahorraron": "ahorrar",
    "ahorrará": "ahorrar",
    "ahorrarán": "ahorrar",
    "ahorrarás": "ahorrar",
    "ahorraré": "ahorrar",
    "ahorraréis": "ahorrar",
    "ahorraría": "ahorrar",
    "ahorraríais": "ahorrar",
    "ahorraríamos": "ahorrar",
    "ahorrarían": "ahorrar",
    "ahorrarías": "ahorrar",
    "ahorras": "ahorrar",
    "ahorrase": "ahorrar",
    "ahorraseis": "ahorrar",
    "ahorrasen": "ahorrar",
    "ahorrases": "ahorrar",
    "ahorraste": "ahorrar",
    "ahorrasteis": "ahorrar",
    "ahorrativa": "ahorrativo",
    "ahorrativas": "ahorrativo",
    "ahorrativos": "ahorrativo",
    "ahorre": "ahorrar",
    "ahorremos": "ahorrar",
    "ahorren": "ahorrar",
    "ahorres": "ahorrar",
    "ahorristas": "ahorrista",
    "ahorro": "ahorrar",
    "ahorros": "ahorro",
    "ahorrábamos": "ahorrar",
    "ahorráis": "ahorrar",
    "ahorráramos": "ahorrar",
    "ahorráremos": "ahorrar",
    "ahorrásemos": "ahorrar",
    "ahorré": "ahorrar",
    "ahorréis": "ahorrar",
    "ahorró": "ahorrar",
    "ahoyaduras": "ahoyadura",
    "ahuchaba": "ahuchar",
    "ahuchabais": "ahuchar",
    "ahuchaban": "ahuchar",
    "ahuchabas": "ahuchar",
    "ahuchad": "ahuchar",
    "ahuchada": "ahuchar",
    "ahuchadas": "ahuchar",
    "ahuchado": "ahuchar",
    "ahuchados": "ahuchar",
    "ahuchamos": "ahuchar",
    "ahuchando": "ahuchar",
    "ahuchara": "ahuchar",
    "ahucharais": "ahuchar",
    "ahucharan": "ahuchar",
    "ahucharas": "ahuchar",
    "ahuchare": "ahuchar",
    "ahuchareis": "ahuchar",
    "ahucharemos": "ahuchar",
    "ahucharen": "ahuchar",
    "ahuchares": "ahuchar",
    "ahucharon": "ahuchar",
    "ahuchará": "ahuchar",
    "ahucharán": "ahuchar",
    "ahucharás": "ahuchar",
    "ahucharé": "ahuchar",
    "ahucharéis": "ahuchar",
    "ahucharía": "ahuchar",
    "ahucharíais": "ahuchar",
    "ahucharíamos": "ahuchar",
    "ahucharían": "ahuchar",
    "ahucharías": "ahuchar",
    "ahuchase": "ahuchar",
    "ahuchaseis": "ahuchar",
    "ahuchasen": "ahuchar",
    "ahuchases": "ahuchar",
    "ahuchaste": "ahuchar",
    "ahuchasteis": "ahuchar",
    "ahuchemos": "ahuchar",
    "ahuchábamos": "ahuchar",
    "ahucháis": "ahuchar",
    "ahucháramos": "ahuchar",
    "ahucháremos": "ahuchar",
    "ahuchásemos": "ahuchar",
    "ahuché": "ahuchar",
    "ahuchéis": "ahuchar",
    "ahuchó": "ahuchar",
    "ahueca": "ahuecar",
    "ahuecaba": "ahuecar",
    "ahuecabais": "ahuecar",
    "ahuecaban": "ahuecar",
    "ahuecabas": "ahuecar",
    "ahuecad": "ahuecar",
    "ahuecada": "ahuecar",
    "ahuecadas": "ahuecar",
    "ahuecado": "ahuecar",
    "ahuecados": "ahuecar",
    "ahuecamientos": "ahuecamiento",
    "ahuecamos": "ahuecar",
    "ahuecan": "ahuecar",
    "ahuecando": "ahuecar",
    "ahuecara": "ahuecar",
    "ahuecarais": "ahuecar",
    "ahuecaran": "ahuecar",
    "ahuecaras": "ahuecar",
    "ahuecare": "ahuecar",
    "ahuecareis": "ahuecar",
    "ahuecaremos": "ahuecar",
    "ahuecaren": "ahuecar",
    "ahuecares": "ahuecar",
    "ahuecaron": "ahuecar",
    "ahuecará": "ahuecar",
    "ahuecarán": "ahuecar",
    "ahuecarás": "ahuecar",
    "ahuecaré": "ahuecar",
    "ahuecaréis": "ahuecar",
    "ahuecaría": "ahuecar",
    "ahuecaríais": "ahuecar",
    "ahuecaríamos": "ahuecar",
    "ahuecarían": "ahuecar",
    "ahuecarías": "ahuecar",
    "ahuecas": "ahuecar",
    "ahuecase": "ahuecar",
    "ahuecaseis": "ahuecar",
    "ahuecasen": "ahuecar",
    "ahuecases": "ahuecar",
    "ahuecaste": "ahuecar",
    "ahuecasteis": "ahuecar",
    "ahueco": "ahuecar",
    "ahuecábamos": "ahuecar",
    "ahuecáis": "ahuecar",
    "ahuecáramos": "ahuecar",
    "ahuecáremos": "ahuecar",
    "ahuecásemos": "ahuecar",
    "ahuecó": "ahuecar",
    "ahuehuetes": "ahuehuete",
    "ahuehués": "ahuehué",
    "ahueque": "ahuecar",
    "ahuequemos": "ahuecar",
    "ahuequen": "ahuecar",
    "ahueques": "ahuecar",
    "ahuequé": "ahuecar",
    "ahuequéis": "ahuecar",
    "ahueva": "ahuevar",
    "ahuevaba": "ahuevar",
    "ahuevabais": "ahuevar",
    "ahuevaban": "ahuevar",
    "ahuevabas": "ahuevar",
    "ahuevad": "ahuevar",
    "ahuevada": "ahuevar",
    "ahuevadas": "ahuevar",
    "ahuevado": "ahuevar",
    "ahuevados": "ahuevar",
    "ahuevamos": "ahuevar",
    "ahuevan": "ahuevar",
    "ahuevando": "ahuevar",
    "ahuevara": "ahuevar",
    "ahuevarais": "ahuevar",
    "ahuevaran": "ahuevar",
    "ahuevaras": "ahuevar",
    "ahuevare": "ahuevar",
    "ahuevareis": "ahuevar",
    "ahuevaremos": "ahuevar",
    "ahuevaren": "ahuevar",
    "ahuevares": "ahuevar",
    "ahuevaron": "ahuevar",
    "ahuevará": "ahuevar",
    "ahuevarán": "ahuevar",
    "ahuevarás": "ahuevar",
    "ahuevaré": "ahuevar",
    "ahuevaréis": "ahuevar",
    "ahuevaría": "ahuevar",
    "ahuevaríais": "ahuevar",
    "ahuevaríamos": "ahuevar",
    "ahuevarían": "ahuevar",
    "ahuevarías": "ahuevar",
    "ahuevas": "ahuevar",
    "ahuevase": "ahuevar",
    "ahuevaseis": "ahuevar",
    "ahuevasen": "ahuevar",
    "ahuevases": "ahuevar",
    "ahuevaste": "ahuevar",
    "ahuevasteis": "ahuevar",
    "ahueve": "ahuevar",
    "ahuevemos": "ahuevar",
    "ahueven": "ahuevar",
    "ahueves": "ahuevar",
    "ahuevo": "ahuevar",
    "ahuevábamos": "ahuevar",
    "ahueváis": "ahuevar",
    "ahueváramos": "ahuevar",
    "ahueváremos": "ahuevar",
    "ahuevásemos": "ahuevar",
    "ahuevé": "ahuevar",
    "ahuevéis": "ahuevar",
    "ahuevó": "ahuevar",
    "ahumaba": "ahumar",
    "ahumabais": "ahumar",
    "ahumaban": "ahumar",
    "ahumabas": "ahumar",
    "ahumad": "ahumar",
    "ahumada": "ahumar",
    "ahumadas": "ahumar",
    "ahumado": "ahumar",
    "ahumados": "ahumar",
    "ahumamos": "ahumar",
    "ahumando": "ahumar",
    "ahumara": "ahumar",
    "ahumarais": "ahumar",
    "ahumaran": "ahumar",
    "ahumaras": "ahumar",
    "ahumare": "ahumar",
    "ahumareis": "ahumar",
    "ahumaremos": "ahumar",
    "ahumaren": "ahumar",
    "ahumares": "ahumar",
    "ahumaron": "ahumar",
    "ahumará": "ahumar",
    "ahumarán": "ahumar",
    "ahumarás": "ahumar",
    "ahumaré": "ahumar",
    "ahumaréis": "ahumar",
    "ahumaría": "ahumar",
    "ahumaríais": "ahumar",
    "ahumaríamos": "ahumar",
    "ahumarían": "ahumar",
    "ahumarías": "ahumar",
    "ahumase": "ahumar",
    "ahumaseis": "ahumar",
    "ahumasen": "ahumar",
    "ahumases": "ahumar",
    "ahumaste": "ahumar",
    "ahumasteis": "ahumar",
    "ahumemos": "ahumar",
    "ahumábamos": "ahumar",
    "ahumáis": "ahumar",
    "ahumáramos": "ahumar",
    "ahumáremos": "ahumar",
    "ahumásemos": "ahumar",
    "ahumé": "ahumar",
    "ahuméis": "ahumar",
    "ahumó": "ahumar",
    "ahusaba": "ahusar",
    "ahusabais": "ahusar",
    "ahusaban": "ahusar",
    "ahusabas": "ahusar",
    "ahusad": "ahusar",
    "ahusada": "ahusar",
    "ahusadas": "ahusar",
    "ahusado": "ahusar",
    "ahusados": "ahusar",
    "ahusamos": "ahusar",
    "ahusando": "ahusar",
    "ahusara": "ahusar",
    "ahusarais": "ahusar",
    "ahusaran": "ahusar",
    "ahusaras": "ahusar",
    "ahusare": "ahusar",
    "ahusareis": "ahusar",
    "ahusaremos": "ahusar",
    "ahusaren": "ahusar",
    "ahusares": "ahusar",
    "ahusaron": "ahusar",
    "ahusará": "ahusar",
    "ahusarán": "ahusar",
    "ahusarás": "ahusar",
    "ahusaré": "ahusar",
    "ahusaréis": "ahusar",
    "ahusaría": "ahusar",
    "ahusaríais": "ahusar",
    "ahusaríamos": "ahusar",
    "ahusarían": "ahusar",
    "ahusarías": "ahusar",
    "ahusase": "ahusar",
    "ahusaseis": "ahusar",
    "ahusasen": "ahusar",
    "ahusases": "ahusar",
    "ahusaste": "ahusar",
    "ahusasteis": "ahusar",
    "ahusemos": "ahusar",
    "ahusábamos": "ahusar",
    "ahusáis": "ahusar",
    "ahusáramos": "ahusar",
    "ahusáremos": "ahusar",
    "ahusásemos": "ahusar",
    "ahusé": "ahusar",
    "ahuséis": "ahusar",
    "ahusó": "ahusar",
    "ahuyenta": "ahuyentar",
    "ahuyentaba": "ahuyentar",
    "ahuyentabais": "ahuyentar",
    "ahuyentaban": "ahuyentar",
    "ahuyentabas": "ahuyentar",
    "ahuyentad": "ahuyentar",
    "ahuyentada": "ahuyentar",
    "ahuyentadas": "ahuyentar",
    "ahuyentado": "ahuyentar",
    "ahuyentados": "ahuyentar",
    "ahuyentamientos": "ahuyentamiento",
    "ahuyentamos": "ahuyentar",
    "ahuyentan": "ahuyentar",
    "ahuyentando": "ahuyentar",
    "ahuyentara": "ahuyentar",
    "ahuyentarais": "ahuyentar",
    "ahuyentaran": "ahuyentar",
    "ahuyentaras": "ahuyentar",
    "ahuyentare": "ahuyentar",
    "ahuyentareis": "ahuyentar",
    "ahuyentaremos": "ahuyentar",
    "ahuyentaren": "ahuyentar",
    "ahuyentares": "ahuyentar",
    "ahuyentaron": "ahuyentar",
    "ahuyentará": "ahuyentar",
    "ahuyentarán": "ahuyentar",
    "ahuyentarás": "ahuyentar",
    "ahuyentaré": "ahuyentar",
    "ahuyentaréis": "ahuyentar",
    "ahuyentaría": "ahuyentar",
    "ahuyentaríais": "ahuyentar",
    "ahuyentaríamos": "ahuyentar",
    "ahuyentarían": "ahuyentar",
    "ahuyentarías": "ahuyentar",
    "ahuyentas": "ahuyentar",
    "ahuyentase": "ahuyentar",
    "ahuyentaseis": "ahuyentar",
    "ahuyentasen": "ahuyentar",
    "ahuyentases": "ahuyentar",
    "ahuyentaste": "ahuyentar",
    "ahuyentasteis": "ahuyentar",
    "ahuyente": "ahuyentar",
    "ahuyentemos": "ahuyentar",
    "ahuyenten": "ahuyentar",
    "ahuyentes": "ahuyentar",
    "ahuyento": "ahuyentar",
    "ahuyentábamos": "ahuyentar",
    "ahuyentáis": "ahuyentar",
    "ahuyentáramos": "ahuyentar",
    "ahuyentáremos": "ahuyentar",
    "ahuyentásemos": "ahuyentar",
    "ahuyenté": "ahuyentar",
    "ahuyentéis": "ahuyentar",
    "ahuyentó": "ahuyentar",
    "ahíja": "ahijar",
    "ahíjan": "ahijar",
    "ahíjas": "ahijar",
    "ahíje": "ahijar",
    "ahíjen": "ahijar",
    "ahíjes": "ahijar",
    "ahíjo": "ahijar",
    "ahíla": "ahilar",
    "ahílan": "ahilar",
    "ahílas": "ahilar",
    "ahíle": "ahilar",
    "ahílen": "ahilar",
    "ahíles": "ahilar",
    "ahílo": "ahilar",
    "ahílos": "ahílo",
    "ahínca": "ahincar",
    "ahíncan": "ahincar",
    "ahíncas": "ahincar",
    "ahínco": "ahincar",
    "ahíncos": "ahínco",
    "ahínque": "ahincar",
    "ahínquen": "ahincar",
    "ahínques": "ahincar",
    "ahíta": "ahíto",
    "ahítan": "ahitar",
    "ahítas": "ahíto",
    "ahíte": "ahitar",
    "ahíten": "ahitar",
    "ahítes": "ahitar",
    "ahíto": "ahitar",
    "ahítos": "ahíto",
    "ahúcha": "ahuchar",
    "ahúchan": "ahuchar",
    "ahúchas": "ahuchar",
    "ahúche": "ahuchar",
    "ahúchen": "ahuchar",
    "ahúches": "ahuchar",
    "ahúcho": "ahuchar",
    "ahúma": "ahumar",
    "ahúman": "ahumar",
    "ahúmas": "ahumar",
    "ahúme": "ahumar",
    "ahúmen": "ahumar",
    "ahúmes": "ahumar",
    "ahúmo": "ahumar",
    "ahúsa": "ahusar",
    "ahúsan": "ahusar",
    "ahúsas": "ahusar",
    "ahúse": "ahusar",
    "ahúsen": "ahusar",
    "ahúses": "ahusar",
    "ahúso": "ahusar",
    "aiatol·làs": "aiatol·là",
    "aifardas": "aifarda",
    "ailantos": "ailanto",
    "aimarás": "aimará",
    "aina": "aino",
    "ainas": "aino",
    "aindamáis": "además",
    "ainos": "aino",
    "ainus": "ainu",
    "airaba": "airar",
    "airabais": "airar",
    "airaban": "airar",
    "airabas": "airar",
    "airad": "airar",
    "airada": "airar",
    "airadas": "airar",
    "airado": "airar",
    "airados": "airar",
    "airamientos": "airamiento",
    "airamos": "airar",
    "airando": "airar",
    "airara": "airar",
    "airarais": "airar",
    "airaran": "airar",
    "airaras": "airar",
    "airare": "airar",
    "airareis": "airar",
    "airaremos": "airar",
    "airaren": "airar",
    "airares": "airar",
    "airaron": "airar",
    "airará": "airar",
    "airarán": "airar",
    "airarás": "airar",
    "airaré": "airar",
    "airaréis": "airar",
    "airaría": "airar",
    "airaríais": "airar",
    "airaríamos": "airar",
    "airarían": "airar",
    "airarías": "airar",
    "airase": "airar",
    "airaseis": "airar",
    "airasen": "airar",
    "airases": "airar",
    "airaste": "airar",
    "airasteis": "airar",
    "airbags": "airbag",
    "airea": "airear",
    "aireaba": "airear",
    "aireabais": "airear",
    "aireaban": "airear",
    "aireabas": "airear",
    "aireaciones": "aireación",
    "airead": "airear",
    "aireada": "airear",
    "aireadas": "airear",
    "aireado": "airear",
    "aireadoras": "aireadora",
    "aireadores": "aireador",
    "aireados": "airear",
    "aireamos": "airear",
    "airean": "airear",
    "aireando": "airear",
    "aireara": "airear",
    "airearais": "airear",
    "airearan": "airear",
    "airearas": "airear",
    "aireare": "airear",
    "aireareis": "airear",
    "airearemos": "airear",
    "airearen": "airear",
    "aireares": "airear",
    "airearon": "airear",
    "aireará": "airear",
    "airearán": "airear",
    "airearás": "airear",
    "airearé": "airear",
    "airearéis": "airear",
    "airearía": "airear",
    "airearíais": "airear",
    "airearíamos": "airear",
    "airearían": "airear",
    "airearías": "airear",
    "aireas": "airear",
    "airease": "airear",
    "aireaseis": "airear",
    "aireasen": "airear",
    "aireases": "airear",
    "aireaste": "airear",
    "aireasteis": "airear",
    "airee": "airear",
    "aireemos": "airear",
    "aireen": "airear",
    "airees": "airear",
    "airemos": "airar",
    "aireo": "airear",
    "aireos": "aireo",
    "aires": "aire",
    "aireábamos": "airear",
    "aireáis": "airear",
    "aireáramos": "airear",
    "aireáremos": "airear",
    "aireásemos": "airear",
    "aireé": "airear",
    "aireéis": "airear",
    "aireó": "airear",
    "airones": "airón",
    "airosa": "airoso",
    "airosas": "airoso",
    "airosidades": "airosidad",
    "airosos": "airoso",
    "airábamos": "airar",
    "airáis": "airar",
    "airáramos": "airar",
    "airáremos": "airar",
    "airásemos": "airar",
    "airé": "airar",
    "airéis": "airar",
    "airó": "airar",
    "aislaba": "aislar",
    "aislabais": "aislar",
    "aislaban": "aislar",
    "aislabas": "aislar",
    "aislacionismos": "aislacionismo",
    "aislacionistas": "aislacionista",
    "aislad": "aislar",
    "aislada": "aislar",
    "aisladas": "aislar",
    "aislado": "aislar",
    "aisladora": "aislador",
    "aisladoras": "aislador",
    "aisladores": "aislador",
    "aislados": "aislar",
    "aislamientos": "aislamiento",
    "aislamos": "aislar",
    "aislando": "aislar",
    "aislantes": "aislante",
    "aislara": "aislar",
    "aislarais": "aislar",
    "aislaran": "aislar",
    "aislaras": "aislar",
    "aislare": "aislar",
    "aislareis": "aislar",
    "aislaremos": "aislar",
    "aislaren": "aislar",
    "aislares": "aislar",
    "aislaron": "aislar",
    "aislará": "aislar",
    "aislarán": "aislar",
    "aislarás": "aislar",
    "aislaré": "aislar",
    "aislaréis": "aislar",
    "aislaría": "aislar",
    "aislaríais": "aislar",
    "aislaríamos": "aislar",
    "aislarían": "aislar",
    "aislarías": "aislar",
    "aislase": "aislar",
    "aislaseis": "aislar",
    "aislasen": "aislar",
    "aislases": "aislar",
    "aislaste": "aislar",
    "aislasteis": "aislar",
    "aislemos": "aislar",
    "aislábamos": "aislar",
    "aisláis": "aislar",
    "aisláramos": "aislar",
    "aisláremos": "aislar",
    "aislásemos": "aislar",
    "aislé": "aislar",
    "aisléis": "aislar",
    "aisló": "aislar",
    "aizcolaris": "aizcolari",
    "aizkolaris": "aizkolari",
    "aizoáceas": "aizoácea",
    "aja": "ajar",
    "ajaba": "ajar",
    "ajabais": "ajar",
    "ajaban": "ajar",
    "ajabas": "ajar",
    "ajad": "ajar",
    "ajada": "ajar",
    "ajadas": "ajar",
    "ajado": "ajar",
    "ajados": "ajar",
    "ajamientos": "ajamiento",
    "ajamona": "ajamonar",
    "ajamonaba": "ajamonar",
    "ajamonabais": "ajamonar",
    "ajamonaban": "ajamonar",
    "ajamonabas": "ajamonar",
    "ajamonad": "ajamonar",
    "ajamonada": "ajamonar",
    "ajamonadas": "ajamonar",
    "ajamonado": "ajamonar",
    "ajamonados": "ajamonar",
    "ajamonamos": "ajamonar",
    "ajamonan": "ajamonar",
    "ajamonando": "ajamonar",
    "ajamonara": "ajamonar",
    "ajamonarais": "ajamonar",
    "ajamonaran": "ajamonar",
    "ajamonaras": "ajamonar",
    "ajamonare": "ajamonar",
    "ajamonareis": "ajamonar",
    "ajamonaremos": "ajamonar",
    "ajamonaren": "ajamonar",
    "ajamonares": "ajamonar",
    "ajamonaron": "ajamonar",
    "ajamonará": "ajamonar",
    "ajamonarán": "ajamonar",
    "ajamonarás": "ajamonar",
    "ajamonaré": "ajamonar",
    "ajamonaréis": "ajamonar",
    "ajamonaría": "ajamonar",
    "ajamonaríais": "ajamonar",
    "ajamonaríamos": "ajamonar",
    "ajamonarían": "ajamonar",
    "ajamonarías": "ajamonar",
    "ajamonas": "ajamonar",
    "ajamonase": "ajamonar",
    "ajamonaseis": "ajamonar",
    "ajamonasen": "ajamonar",
    "ajamonases": "ajamonar",
    "ajamonaste": "ajamonar",
    "ajamonasteis": "ajamonar",
    "ajamone": "ajamonar",
    "ajamonemos": "ajamonar",
    "ajamonen": "ajamonar",
    "ajamones": "ajamonar",
    "ajamono": "ajamonar",
    "ajamonábamos": "ajamonar",
    "ajamonáis": "ajamonar",
    "ajamonáramos": "ajamonar",
    "ajamonáremos": "ajamonar",
    "ajamonásemos": "ajamonar",
    "ajamoné": "ajamonar",
    "ajamonéis": "ajamonar",
    "ajamonó": "ajamonar",
    "ajamos": "ajar",
    "ajan": "ajar",
    "ajando": "ajar",
    "ajaqueca": "ajaquecar",
    "ajaquecaba": "ajaquecar",
    "ajaquecabais": "ajaquecar",
    "ajaquecaban": "ajaquecar",
    "ajaquecabas": "ajaquecar",
    "ajaquecad": "ajaquecar",
    "ajaquecada": "ajaquecar",
    "ajaquecadas": "ajaquecar",
    "ajaquecado": "ajaquecar",
    "ajaquecados": "ajaquecar",
    "ajaquecamos": "ajaquecar",
    "ajaquecan": "ajaquecar",
    "ajaquecando": "ajaquecar",
    "ajaquecara": "ajaquecar",
    "ajaquecarais": "ajaquecar",
    "ajaquecaran": "ajaquecar",
    "ajaquecaras": "ajaquecar",
    "ajaquecare": "ajaquecar",
    "ajaquecareis": "ajaquecar",
    "ajaquecaremos": "ajaquecar",
    "ajaquecaren": "ajaquecar",
    "ajaquecares": "ajaquecar",
    "ajaquecaron": "ajaquecar",
    "ajaquecará": "ajaquecar",
    "ajaquecarán": "ajaquecar",
    "ajaquecarás": "ajaquecar",
    "ajaquecaré": "ajaquecar",
    "ajaquecaréis": "ajaquecar",
    "ajaquecaría": "ajaquecar",
    "ajaquecaríais": "ajaquecar",
    "ajaquecaríamos": "ajaquecar",
    "ajaquecarían": "ajaquecar",
    "ajaquecarías": "ajaquecar",
    "ajaquecas": "ajaquecar",
    "ajaquecase": "ajaquecar",
    "ajaquecaseis": "ajaquecar",
    "ajaquecasen": "ajaquecar",
    "ajaquecases": "ajaquecar",
    "ajaquecaste": "ajaquecar",
    "ajaquecasteis": "ajaquecar",
    "ajaqueco": "ajaquecar",
    "ajaquecábamos": "ajaquecar",
    "ajaquecáis": "ajaquecar",
    "ajaquecáramos": "ajaquecar",
    "ajaquecáremos": "ajaquecar",
    "ajaquecásemos": "ajaquecar",
    "ajaquecó": "ajaquecar",
    "ajaqueque": "ajaquecar",
    "ajaquequemos": "ajaquecar",
    "ajaquequen": "ajaquecar",
    "ajaqueques": "ajaquecar",
    "ajaquequé": "ajaquecar",
    "ajaquequéis": "ajaquecar",
    "ajara": "ajar",
    "ajaracas": "ajaraca",
    "ajarafes": "ajarafe",
    "ajarais": "ajar",
    "ajaran": "ajar",
    "ajaras": "ajar",
    "ajardina": "ajardinar",
    "ajardinaba": "ajardinar",
    "ajardinabais": "ajardinar",
    "ajardinaban": "ajardinar",
    "ajardinabas": "ajardinar",
    "ajardinad": "ajardinar",
    "ajardinada": "ajardinar",
    "ajardinadas": "ajardinar",
    "ajardinado": "ajardinar",
    "ajardinados": "ajardinar",
    "ajardinamientos": "ajardinamiento",
    "ajardinamos": "ajardinar",
    "ajardinan": "ajardinar",
    "ajardinando": "ajardinar",
    "ajardinara": "ajardinar",
    "ajardinarais": "ajardinar",
    "ajardinaran": "ajardinar",
    "ajardinaras": "ajardinar",
    "ajardinare": "ajardinar",
    "ajardinareis": "ajardinar",
    "ajardinaremos": "ajardinar",
    "ajardinaren": "ajardinar",
    "ajardinares": "ajardinar",
    "ajardinaron": "ajardinar",
    "ajardinará": "ajardinar",
    "ajardinarán": "ajardinar",
    "ajardinarás": "ajardinar",
    "ajardinaré": "ajardinar",
    "ajardinaréis": "ajardinar",
    "ajardinaría": "ajardinar",
    "ajardinaríais": "ajardinar",
    "ajardinaríamos": "ajardinar",
    "ajardinarían": "ajardinar",
    "ajardinarías": "ajardinar",
    "ajardinas": "ajardinar",
    "ajardinase": "ajardinar",
    "ajardinaseis": "ajardinar",
    "ajardinasen": "ajardinar",
    "ajardinases": "ajardinar",
    "ajardinaste": "ajardinar",
    "ajardinasteis": "ajardinar",
    "ajardine": "ajardinar",
    "ajardinemos": "ajardinar",
    "ajardinen": "ajardinar",
    "ajardines": "ajardinar",
    "ajardino": "ajardinar",
    "ajardinábamos": "ajardinar",
    "ajardináis": "ajardinar",
    "ajardináramos": "ajardinar",
    "ajardináremos": "ajardinar",
    "ajardinásemos": "ajardinar",
    "ajardiné": "ajardinar",
    "ajardinéis": "ajardinar",
    "ajardinó": "ajardinar",
    "ajare": "ajar",
    "ajareis": "ajar",
    "ajaremos": "ajar",
    "ajaren": "ajar",
    "ajares": "ajar",
    "ajaron": "ajar",
    "ajará": "ajar",
    "ajarán": "ajar",
    "ajarás": "ajar",
    "ajaré": "ajar",
    "ajaréis": "ajar",
    "ajaría": "ajar",
    "ajaríais": "ajar",
    "ajaríamos": "ajar",
    "ajarían": "ajar",
    "ajarías": "ajar",
    "ajas": "ajar",
    "ajase": "ajar",
    "ajaseis": "ajar",
    "ajasen": "ajar",
    "ajases": "ajar",
    "ajaste": "ajar",
    "ajasteis": "ajar",
    "aje": "ajar",
    "ajea": "ajear",
    "ajeaba": "ajear",
    "ajeabais": "ajear",
    "ajeaban": "ajear",
    "ajeabas": "ajear",
    "ajead": "ajear",
    "ajeada": "ajear",
    "ajeadas": "ajear",
    "ajeado": "ajear",
    "ajeados": "ajear",
    "ajeamos": "ajear",
    "ajean": "ajear",
    "ajeando": "ajear",
    "ajeara": "ajear",
    "ajearais": "ajear",
    "ajearan": "ajear",
    "ajearas": "ajear",
    "ajeare": "ajear",
    "ajeareis": "ajear",
    "ajearemos": "ajear",
    "ajearen": "ajear",
    "ajeares": "ajear",
    "ajearon": "ajear",
    "ajeará": "ajear",
    "ajearán": "ajear",
    "ajearás": "ajear",
    "ajearé": "ajear",
    "ajearéis": "ajear",
    "ajearía": "ajear",
    "ajearíais": "ajear",
    "ajearíamos": "ajear",
    "ajearían": "ajear",
    "ajearías": "ajear",
    "ajeas": "ajear",
    "ajease": "ajear",
    "ajeaseis": "ajear",
    "ajeasen": "ajear",
    "ajeases": "ajear",
    "ajeaste": "ajear",
    "ajeasteis": "ajear",
    "ajedreas": "ajedrea",
    "ajedreces": "ajedrez",
    "ajedrecistas": "ajedrecista",
    "ajedrecística": "ajedrecístico",
    "ajedrecísticas": "ajedrecístico",
    "ajedrecísticos": "ajedrecístico",
    "ajedrezada": "ajedrezado",
    "ajedrezadas": "ajedrezado",
    "ajedrezados": "ajedrezado",
    "ajee": "ajear",
    "ajeemos": "ajear",
    "ajeen": "ajear",
    "ajees": "ajear",
    "ajemos": "ajar",
    "ajen": "ajar",
    "ajena": "ajeno",
    "ajenas": "ajeno",
    "ajenjos": "ajenjo",
    "ajenos": "ajeno",
    "ajeo": "ajear",
    "ajes": "aje",
    "ajetes": "ajete",
    "ajetrea": "ajetrear",
    "ajetreaba": "ajetrear",
    "ajetreabais": "ajetrear",
    "ajetreaban": "ajetrear",
    "ajetreabas": "ajetrear",
    "ajetread": "ajetrear",
    "ajetreada": "ajetrear",
    "ajetreadas": "ajetrear",
    "ajetreado": "ajetrear",
    "ajetreados": "ajetrear",
    "ajetreamos": "ajetrear",
    "ajetrean": "ajetrear",
    "ajetreando": "ajetrear",
    "ajetreara": "ajetrear",
    "ajetrearais": "ajetrear",
    "ajetrearan": "ajetrear",
    "ajetrearas": "ajetrear",
    "ajetreare": "ajetrear",
    "ajetreareis": "ajetrear",
    "ajetrearemos": "ajetrear",
    "ajetrearen": "ajetrear",
    "ajetreares": "ajetrear",
    "ajetrearon": "ajetrear",
    "ajetreará": "ajetrear",
    "ajetrearán": "ajetrear",
    "ajetrearás": "ajetrear",
    "ajetrearé": "ajetrear",
    "ajetrearéis": "ajetrear",
    "ajetrearía": "ajetrear",
    "ajetrearíais": "ajetrear",
    "ajetrearíamos": "ajetrear",
    "ajetrearían": "ajetrear",
    "ajetrearías": "ajetrear",
    "ajetreas": "ajetrear",
    "ajetrease": "ajetrear",
    "ajetreaseis": "ajetrear",
    "ajetreasen": "ajetrear",
    "ajetreases": "ajetrear",
    "ajetreaste": "ajetrear",
    "ajetreasteis": "ajetrear",
    "ajetree": "ajetrear",
    "ajetreemos": "ajetrear",
    "ajetreen": "ajetrear",
    "ajetrees": "ajetrear",
    "ajetreo": "ajetrear",
    "ajetreos": "ajetreo",
    "ajetreábamos": "ajetrear",
    "ajetreáis": "ajetrear",
    "ajetreáramos": "ajetrear",
    "ajetreáremos": "ajetrear",
    "ajetreásemos": "ajetrear",
    "ajetreé": "ajetrear",
    "ajetreéis": "ajetrear",
    "ajetreó": "ajetrear",
    "ajeábamos": "ajear",
    "ajeáis": "ajear",
    "ajeáramos": "ajear",
    "ajeáremos": "ajear",
    "ajeásemos": "ajear",
    "ajeé": "ajear",
    "ajeéis": "ajear",
    "ajeó": "ajear",
    "ajiaceites": "ajiaceite",
    "ajiacos": "ajiaco",
    "ajicolas": "ajicola",
    "ajilimojes": "ajilimoje",
    "ajilimójilis": "ajilimójili",
    "ajillos": "ajillo",
    "ajimeces": "ajimez",
    "ajipuerros": "ajipuerro",
    "ajo": "ajar",
    "ajoaceites": "ajoaceite",
    "ajoarrieros": "ajoarriero",
    "ajoba": "ajobar",
    "ajobaba": "ajobar",
    "ajobabais": "ajobar",
    "ajobaban": "ajobar",
    "ajobabas": "ajobar",
    "ajobad": "ajobar",
    "ajobada": "ajobar",
    "ajobadas": "ajobar",
    "ajobado": "ajobar",
    "ajobados": "ajobar",
    "ajobamos": "ajobar",
    "ajoban": "ajobar",
    "ajobando": "ajobar",
    "ajobara": "ajobar",
    "ajobarais": "ajobar",
    "ajobaran": "ajobar",
    "ajobaras": "ajobar",
    "ajobare": "ajobar",
    "ajobareis": "ajobar",
    "ajobaremos": "ajobar",
    "ajobaren": "ajobar",
    "ajobares": "ajobar",
    "ajobaron": "ajobar",
    "ajobará": "ajobar",
    "ajobarán": "ajobar",
    "ajobarás": "ajobar",
    "ajobaré": "ajobar",
    "ajobaréis": "ajobar",
    "ajobaría": "ajobar",
    "ajobaríais": "ajobar",
    "ajobaríamos": "ajobar",
    "ajobarían": "ajobar",
    "ajobarías": "ajobar",
    "ajobas": "ajobar",
    "ajobase": "ajobar",
    "ajobaseis": "ajobar",
    "ajobasen": "ajobar",
    "ajobases": "ajobar",
    "ajobaste": "ajobar",
    "ajobasteis": "ajobar",
    "ajobe": "ajobar",
    "ajobemos": "ajobar",
    "ajoben": "ajobar",
    "ajobes": "ajobar",
    "ajobo": "ajobar",
    "ajobábamos": "ajobar",
    "ajobáis": "ajobar",
    "ajobáramos": "ajobar",
    "ajobáremos": "ajobar",
    "ajobásemos": "ajobar",
    "ajobé": "ajobar",
    "ajobéis": "ajobar",
    "ajobó": "ajobar",
    "ajonjeras": "ajonjera",
    "ajonjes": "ajonje",
    "ajonjolíes": "ajonjolí",
    "ajonjolís": "ajonjolí",
    "ajoraba": "ajorar",
    "ajorabais": "ajorar",
    "ajoraban": "ajorar",
    "ajorabas": "ajorar",
    "ajorad": "ajorar",
    "ajorada": "ajorar",
    "ajoradas": "ajorar",
    "ajorado": "ajorar",
    "ajorados": "ajorar",
    "ajoramos": "ajorar",
    "ajorando": "ajorar",
    "ajorara": "ajorar",
    "ajorarais": "ajorar",
    "ajoraran": "ajorar",
    "ajoraras": "ajorar",
    "ajorare": "ajorar",
    "ajorareis": "ajorar",
    "ajoraremos": "ajorar",
    "ajoraren": "ajorar",
    "ajorares": "ajorar",
    "ajoraron": "ajorar",
    "ajorará": "ajorar",
    "ajorarán": "ajorar",
    "ajorarás": "ajorar",
    "ajoraré": "ajorar",
    "ajoraréis": "ajorar",
    "ajoraría": "ajorar",
    "ajoraríais": "ajorar",
    "ajoraríamos": "ajorar",
    "ajorarían": "ajorar",
    "ajorarías": "ajorar",
    "ajorase": "ajorar",
    "ajoraseis": "ajorar",
    "ajorasen": "ajorar",
    "ajorases": "ajorar",
    "ajoraste": "ajorar",
    "ajorasteis": "ajorar",
    "ajorcas": "ajorca",
    "ajoremos": "ajorar",
    "ajornala": "ajornalar",
    "ajornalaba": "ajornalar",
    "ajornalabais": "ajornalar",
    "ajornalaban": "ajornalar",
    "ajornalabas": "ajornalar",
    "ajornalad": "ajornalar",
    "ajornalada": "ajornalar",
    "ajornaladas": "ajornalar",
    "ajornalado": "ajornalar",
    "ajornalados": "ajornalar",
    "ajornalamos": "ajornalar",
    "ajornalan": "ajornalar",
    "ajornalando": "ajornalar",
    "ajornalara": "ajornalar",
    "ajornalarais": "ajornalar",
    "ajornalaran": "ajornalar",
    "ajornalaras": "ajornalar",
    "ajornalare": "ajornalar",
    "ajornalareis": "ajornalar",
    "ajornalaremos": "ajornalar",
    "ajornalaren": "ajornalar",
    "ajornalares": "ajornalar",
    "ajornalaron": "ajornalar",
    "ajornalará": "ajornalar",
    "ajornalarán": "ajornalar",
    "ajornalarás": "ajornalar",
    "ajornalaré": "ajornalar",
    "ajornalaréis": "ajornalar",
    "ajornalaría": "ajornalar",
    "ajornalaríais": "ajornalar",
    "ajornalaríamos": "ajornalar",
    "ajornalarían": "ajornalar",
    "ajornalarías": "ajornalar",
    "ajornalas": "ajornalar",
    "ajornalase": "ajornalar",
    "ajornalaseis": "ajornalar",
    "ajornalasen": "ajornalar",
    "ajornalases": "ajornalar",
    "ajornalaste": "ajornalar",
    "ajornalasteis": "ajornalar",
    "ajornale": "ajornalar",
    "ajornalemos": "ajornalar",
    "ajornalen": "ajornalar",
    "ajornales": "ajornalar",
    "ajornalo": "ajornalar",
    "ajornalábamos": "ajornalar",
    "ajornaláis": "ajornalar",
    "ajornaláramos": "ajornalar",
    "ajornaláremos": "ajornalar",
    "ajornalásemos": "ajornalar",
    "ajornalé": "ajornalar",
    "ajornaléis": "ajornalar",
    "ajornaló": "ajornalar",
    "ajorábamos": "ajorar",
    "ajoráis": "ajorar",
    "ajoráramos": "ajorar",
    "ajoráremos": "ajorar",
    "ajorásemos": "ajorar",
    "ajoré": "ajorar",
    "ajoréis": "ajorar",
    "ajoró": "ajorar",
    "ajos": "ajo",
    "ajovares": "ajovar",
    "ajuara": "ajuarar",
    "ajuaraba": "ajuarar",
    "ajuarabais": "ajuarar",
    "ajuaraban": "ajuarar",
    "ajuarabas": "ajuarar",
    "ajuarad": "ajuarar",
    "ajuarada": "ajuarar",
    "ajuaradas": "ajuarar",
    "ajuarado": "ajuarar",
    "ajuarados": "ajuarar",
    "ajuaramos": "ajuarar",
    "ajuaran": "ajuarar",
    "ajuarando": "ajuarar",
    "ajuarara": "ajuarar",
    "ajuararais": "ajuarar",
    "ajuararan": "ajuarar",
    "ajuararas": "ajuarar",
    "ajuarare": "ajuarar",
    "ajuarareis": "ajuarar",
    "ajuararemos": "ajuarar",
    "ajuararen": "ajuarar",
    "ajuarares": "ajuarar",
    "ajuararon": "ajuarar",
    "ajuarará": "ajuarar",
    "ajuararán": "ajuarar",
    "ajuararás": "ajuarar",
    "ajuararé": "ajuarar",
    "ajuararéis": "ajuarar",
    "ajuararía": "ajuarar",
    "ajuararíais": "ajuarar",
    "ajuararíamos": "ajuarar",
    "ajuararían": "ajuarar",
    "ajuararías": "ajuarar",
    "ajuaras": "ajuarar",
    "ajuarase": "ajuarar",
    "ajuaraseis": "ajuarar",
    "ajuarasen": "ajuarar",
    "ajuarases": "ajuarar",
    "ajuaraste": "ajuarar",
    "ajuarasteis": "ajuarar",
    "ajuare": "ajuarar",
    "ajuaremos": "ajuarar",
    "ajuaren": "ajuarar",
    "ajuares": "ajuarar",
    "ajuaro": "ajuarar",
    "ajuarábamos": "ajuarar",
    "ajuaráis": "ajuarar",
    "ajuaráramos": "ajuarar",
    "ajuaráremos": "ajuarar",
    "ajuarásemos": "ajuarar",
    "ajuaré": "ajuarar",
    "ajuaréis": "ajuarar",
    "ajuaró": "ajuarar",
    "ajudiada": "ajudiado",
    "ajudiadas": "ajudiado",
    "ajudiados": "ajudiado",
    "ajuera": "ajorar",
    "ajueran": "ajorar",
    "ajueras": "ajorar",
    "ajuere": "ajorar",
    "ajueren": "ajorar",
    "ajueres": "ajorar",
    "ajuero": "ajorar",
    "ajuicia": "ajuiciar",
    "ajuiciaba": "ajuiciar",
    "ajuiciabais": "ajuiciar",
    "ajuiciaban": "ajuiciar",
    "ajuiciabas": "ajuiciar",
    "ajuiciad": "ajuiciar",
    "ajuiciada": "ajuiciar",
    "ajuiciadas": "ajuiciar",
    "ajuiciado": "ajuiciar",
    "ajuiciados": "ajuiciar",
    "ajuiciamos": "ajuiciar",
    "ajuician": "ajuiciar",
    "ajuiciando": "ajuiciar",
    "ajuiciara": "ajuiciar",
    "ajuiciarais": "ajuiciar",
    "ajuiciaran": "ajuiciar",
    "ajuiciaras": "ajuiciar",
    "ajuiciare": "ajuiciar",
    "ajuiciareis": "ajuiciar",
    "ajuiciaremos": "ajuiciar",
    "ajuiciaren": "ajuiciar",
    "ajuiciares": "ajuiciar",
    "ajuiciaron": "ajuiciar",
    "ajuiciará": "ajuiciar",
    "ajuiciarán": "ajuiciar",
    "ajuiciarás": "ajuiciar",
    "ajuiciaré": "ajuiciar",
    "ajuiciaréis": "ajuiciar",
    "ajuiciaría": "ajuiciar",
    "ajuiciaríais": "ajuiciar",
    "ajuiciaríamos": "ajuiciar",
    "ajuiciarían": "ajuiciar",
    "ajuiciarías": "ajuiciar",
    "ajuicias": "ajuiciar",
    "ajuiciase": "ajuiciar",
    "ajuiciaseis": "ajuiciar",
    "ajuiciasen": "ajuiciar",
    "ajuiciases": "ajuiciar",
    "ajuiciaste": "ajuiciar",
    "ajuiciasteis": "ajuiciar",
    "ajuicie": "ajuiciar",
    "ajuiciemos": "ajuiciar",
    "ajuicien": "ajuiciar",
    "ajuicies": "ajuiciar",
    "ajuicio": "ajuiciar",
    "ajuiciábamos": "ajuiciar",
    "ajuiciáis": "ajuiciar",
    "ajuiciáramos": "ajuiciar",
    "ajuiciáremos": "ajuiciar",
    "ajuiciásemos": "ajuiciar",
    "ajuicié": "ajuiciar",
    "ajuiciéis": "ajuiciar",
    "ajuició": "ajuiciar",
    "ajuma": "ajumar",
    "ajumaba": "ajumar",
    "ajumabais": "ajumar",
    "ajumaban": "ajumar",
    "ajumabas": "ajumar",
    "ajumad": "ajumar",
    "ajumada": "ajumar",
    "ajumadas": "ajumar",
    "ajumado": "ajumar",
    "ajumados": "ajumar",
    "ajumamos": "ajumar",
    "ajuman": "ajumar",
    "ajumando": "ajumar",
    "ajumara": "ajumar",
    "ajumarais": "ajumar",
    "ajumaran": "ajumar",
    "ajumaras": "ajumar",
    "ajumare": "ajumar",
    "ajumareis": "ajumar",
    "ajumaremos": "ajumar",
    "ajumaren": "ajumar",
    "ajumares": "ajumar",
    "ajumaron": "ajumar",
    "ajumará": "ajumar",
    "ajumarán": "ajumar",
    "ajumarás": "ajumar",
    "ajumaré": "ajumar",
    "ajumaréis": "ajumar",
    "ajumaría": "ajumar",
    "ajumaríais": "ajumar",
    "ajumaríamos": "ajumar",
    "ajumarían": "ajumar",
    "ajumarías": "ajumar",
    "ajumas": "ajumar",
    "ajumase": "ajumar",
    "ajumaseis": "ajumar",
    "ajumasen": "ajumar",
    "ajumases": "ajumar",
    "ajumaste": "ajumar",
    "ajumasteis": "ajumar",
    "ajume": "ajumar",
    "ajumemos": "ajumar",
    "ajumen": "ajumar",
    "ajumes": "ajumar",
    "ajumo": "ajumar",
    "ajumábamos": "ajumar",
    "ajumáis": "ajumar",
    "ajumáramos": "ajumar",
    "ajumáremos": "ajumar",
    "ajumásemos": "ajumar",
    "ajumé": "ajumar",
    "ajuméis": "ajumar",
    "ajumó": "ajumar",
    "ajusta": "ajustar",
    "ajustaba": "ajustar",
    "ajustabais": "ajustar",
    "ajustaban": "ajustar",
    "ajustabas": "ajustar",
    "ajustables": "ajustable",
    "ajustad": "ajustar",
    "ajustada": "ajustar",
    "ajustadas": "ajustar",
    "ajustado": "ajustar",
    "ajustadora": "ajustador",
    "ajustadoras": "ajustador",
    "ajustadores": "ajustador",
    "ajustados": "ajustar",
    "ajustamientos": "ajustamiento",
    "ajustamos": "ajustar",
    "ajustan": "ajustar",
    "ajustando": "ajustar",
    "ajustara": "ajustar",
    "ajustarais": "ajustar",
    "ajustaran": "ajustar",
    "ajustaras": "ajustar",
    "ajustare": "ajustar",
    "ajustareis": "ajustar",
    "ajustaremos": "ajustar",
    "ajustaren": "ajustar",
    "ajustares": "ajustar",
    "ajustaron": "ajustar",
    "ajustará": "ajustar",
    "ajustarán": "ajustar",
    "ajustarás": "ajustar",
    "ajustaré": "ajustar",
    "ajustaréis": "ajustar",
    "ajustaría": "ajustar",
    "ajustaríais": "ajustar",
    "ajustaríamos": "ajustar",
    "ajustarían": "ajustar",
    "ajustarías": "ajustar",
    "ajustas": "ajustar",
    "ajustase": "ajustar",
    "ajustaseis": "ajustar",
    "ajustasen": "ajustar",
    "ajustases": "ajustar",
    "ajustaste": "ajustar",
    "ajustasteis": "ajustar",
    "ajuste": "ajustar",
    "ajustemos": "ajustar",
    "ajusten": "ajustar",
    "ajustes": "ajuste",
    "ajusticia": "ajusticiar",
    "ajusticiaba": "ajusticiar",
    "ajusticiabais": "ajusticiar",
    "ajusticiaban": "ajusticiar",
    "ajusticiabas": "ajusticiar",
    "ajusticiad": "ajusticiar",
    "ajusticiada": "ajusticiar",
    "ajusticiadas": "ajusticiar",
    "ajusticiado": "ajusticiar",
    "ajusticiados": "ajusticiar",
    "ajusticiamientos": "ajusticiamiento",
    "ajusticiamos": "ajusticiar",
    "ajustician": "ajusticiar",
    "ajusticiando": "ajusticiar",
    "ajusticiara": "ajusticiar",
    "ajusticiarais": "ajusticiar",
    "ajusticiaran": "ajusticiar",
    "ajusticiaras": "ajusticiar",
    "ajusticiare": "ajusticiar",
    "ajusticiareis": "ajusticiar",
    "ajusticiaremos": "ajusticiar",
    "ajusticiaren": "ajusticiar",
    "ajusticiares": "ajusticiar",
    "ajusticiaron": "ajusticiar",
    "ajusticiará": "ajusticiar",
    "ajusticiarán": "ajusticiar",
    "ajusticiarás": "ajusticiar",
    "ajusticiaré": "ajusticiar",
    "ajusticiaréis": "ajusticiar",
    "ajusticiaría": "ajusticiar",
    "ajusticiaríais": "ajusticiar",
    "ajusticiaríamos": "ajusticiar",
    "ajusticiarían": "ajusticiar",
    "ajusticiarías": "ajusticiar",
    "ajusticias": "ajusticiar",
    "ajusticiase": "ajusticiar",
    "ajusticiaseis": "ajusticiar",
    "ajusticiasen": "ajusticiar",
    "ajusticiases": "ajusticiar",
    "ajusticiaste": "ajusticiar",
    "ajusticiasteis": "ajusticiar",
    "ajusticie": "ajusticiar",
    "ajusticiemos": "ajusticiar",
    "ajusticien": "ajusticiar",
    "ajusticies": "ajusticiar",
    "ajusticio": "ajusticiar",
    "ajusticiábamos": "ajusticiar",
    "ajusticiáis": "ajusticiar",
    "ajusticiáramos": "ajusticiar",
    "ajusticiáremos": "ajusticiar",
    "ajusticiásemos": "ajusticiar",
    "ajusticié": "ajusticiar",
    "ajusticiéis": "ajusticiar",
    "ajustició": "ajusticiar",
    "ajusto": "ajustar",
    "ajustábamos": "ajustar",
    "ajustáis": "ajustar",
    "ajustáramos": "ajustar",
    "ajustáremos": "ajustar",
    "ajustásemos": "ajustar",
    "ajusté": "ajustar",
    "ajustéis": "ajustar",
    "ajustó": "ajustar",
    "ajábamos": "ajar",
    "ajáis": "ajar",
    "ajáramos": "ajar",
    "ajáremos": "ajar",
    "ajásemos": "ajar",
    "ajé": "ajar",
    "ajéis": "ajar",
    "ajíes": "ají",
    "ajís": "ají",
    "ajó": "ajar",
    "akkadia": "akkadio",
    "akkadias": "akkadio",
    "akkadios": "akkadio",
    "alaba": "alabar",
    "alababa": "alabar",
    "alababais": "alabar",
    "alababan": "alabar",
    "alababas": "alabar",
    "alabad": "alabar",
    "alabada": "alabar",
    "alabadas": "alabar",
    "alabado": "alabar",
    "alabadora": "alabador",
    "alabadoras": "alabador",
    "alabadores": "alabador",
    "alabados": "alabar",
    "alabamos": "alabar",
    "alaban": "alabar",
    "alabanditas": "alabandita",
    "alabando": "alabar",
    "alabanzas": "alabanza",
    "alabara": "alabar",
    "alabarais": "alabar",
    "alabaran": "alabar",
    "alabaras": "alabar",
    "alabardas": "alabarda",
    "alabardera": "alabardero",
    "alabarderas": "alabardero",
    "alabarderos": "alabardero",
    "alabare": "alabar",
    "alabareis": "alabar",
    "alabaremos": "alabar",
    "alabaren": "alabar",
    "alabares": "alabar",
    "alabaron": "alabar",
    "alabará": "alabar",
    "alabarán": "alabar",
    "alabarás": "alabar",
    "alabaré": "alabar",
    "alabaréis": "alabar",
    "alabaría": "alabar",
    "alabaríais": "alabar",
    "alabaríamos": "alabar",
    "alabarían": "alabar",
    "alabarías": "alabar",
    "alabas": "alabar",
    "alabase": "alabar",
    "alabaseis": "alabar",
    "alabasen": "alabar",
    "alabases": "alabar",
    "alabaste": "alabar",
    "alabasteis": "alabar",
    "alabastrada": "alabastrado",
    "alabastradas": "alabastrado",
    "alabastrados": "alabastrado",
    "alabastrina": "alabastrino",
    "alabastrinas": "alabastrino",
    "alabastrinos": "alabastrino",
    "alabastros": "alabastro",
    "alabe": "alabar",
    "alabea": "alabear",
    "alabeaba": "alabear",
    "alabeabais": "alabear",
    "alabeaban": "alabear",
    "alabeabas": "alabear",
    "alabead": "alabear",
    "alabeada": "alabear",
    "alabeadas": "alabear",
    "alabeado": "alabear",
    "alabeados": "alabear",
    "alabeamos": "alabear",
    "alabean": "alabear",
    "alabeando": "alabear",
    "alabeara": "alabear",
    "alabearais": "alabear",
    "alabearan": "alabear",
    "alabearas": "alabear",
    "alabeare": "alabear",
    "alabeareis": "alabear",
    "alabearemos": "alabear",
    "alabearen": "alabear",
    "alabeares": "alabear",
    "alabearon": "alabear",
    "alabeará": "alabear",
    "alabearán": "alabear",
    "alabearás": "alabear",
    "alabearé": "alabear",
    "alabearéis": "alabear",
    "alabearía": "alabear",
    "alabearíais": "alabear",
    "alabearíamos": "alabear",
    "alabearían": "alabear",
    "alabearías": "alabear",
    "alabeas": "alabear",
    "alabease": "alabear",
    "alabeaseis": "alabear",
    "alabeasen": "alabear",
    "alabeases": "alabear",
    "alabeaste": "alabear",
    "alabeasteis": "alabear",
    "alabee": "alabear",
    "alabeemos": "alabear",
    "alabeen": "alabear",
    "alabees": "alabear",
    "alabemos": "alabar",
    "alaben": "alabar",
    "alabeo": "alabear",
    "alabeos": "alabeo",
    "alabes": "alabar",
    "alabeábamos": "alabear",
    "alabeáis": "alabear",
    "alabeáramos": "alabear",
    "alabeáremos": "alabear",
    "alabeásemos": "alabear",
    "alabeé": "alabear",
    "alabeéis": "alabear",
    "alabeó": "alabear",
    "alabo": "alabar",
    "alabábamos": "alabar",
    "alabáis": "alabar",
    "alabáramos": "alabar",
    "alabáremos": "alabar",
    "alabásemos": "alabar",
    "alabé": "alabar",
    "alabéis": "alabar",
    "alabó": "alabar",
    "alacalufes": "alacaluf",
    "alacenas": "alacena",
    "alacraneras": "alacranera",
    "alacranes": "alacrán",
    "alacridades": "alacridad",
    "alada": "alado",
    "aladares": "aladar",
    "aladas": "alado",
    "aladiernas": "aladierna",
    "alados": "alado",
    "aladroques": "aladroque",
    "alaga": "alagar",
    "alagaba": "alagar",
    "alagabais": "alagar",
    "alagaban": "alagar",
    "alagabas": "alagar",
    "alagad": "alagar",
    "alagada": "alagar",
    "alagadas": "alagar",
    "alagado": "alagar",
    "alagados": "alagar",
    "alagamos": "alagar",
    "alagan": "alagar",
    "alagando": "alagar",
    "alagara": "alagar",
    "alagarais": "alagar",
    "alagaran": "alagar",
    "alagaras": "alagar",
    "alagare": "alagar",
    "alagareis": "alagar",
    "alagaremos": "alagar",
    "alagaren": "alagar",
    "alagares": "alagar",
    "alagaron": "alagar",
    "alagarta": "alagartar",
    "alagartaba": "alagartar",
    "alagartabais": "alagartar",
    "alagartaban": "alagartar",
    "alagartabas": "alagartar",
    "alagartad": "alagartar",
    "alagartada": "alagartar",
    "alagartadas": "alagartar",
    "alagartado": "alagartar",
    "alagartados": "alagartar",
    "alagartamos": "alagartar",
    "alagartan": "alagartar",
    "alagartando": "alagartar",
    "alagartara": "alagartar",
    "alagartarais": "alagartar",
    "alagartaran": "alagartar",
    "alagartaras": "alagartar",
    "alagartare": "alagartar",
    "alagartareis": "alagartar",
    "alagartaremos": "alagartar",
    "alagartaren": "alagartar",
    "alagartares": "alagartar",
    "alagartaron": "alagartar",
    "alagartará": "alagartar",
    "alagartarán": "alagartar",
    "alagartarás": "alagartar",
    "alagartaré": "alagartar",
    "alagartaréis": "alagartar",
    "alagartaría": "alagartar",
    "alagartaríais": "alagartar",
    "alagartaríamos": "alagartar",
    "alagartarían": "alagartar",
    "alagartarías": "alagartar",
    "alagartas": "alagartar",
    "alagartase": "alagartar",
    "alagartaseis": "alagartar",
    "alagartasen": "alagartar",
    "alagartases": "alagartar",
    "alagartaste": "alagartar",
    "alagartasteis": "alagartar",
    "alagarte": "alagartar",
    "alagartemos": "alagartar",
    "alagarten": "alagartar",
    "alagartes": "alagartar",
    "alagarto": "alagartar",
    "alagartábamos": "alagartar",
    "alagartáis": "alagartar",
    "alagartáramos": "alagartar",
    "alagartáremos": "alagartar",
    "alagartásemos": "alagartar",
    "alagarté": "alagartar",
    "alagartéis": "alagartar",
    "alagartó": "alagartar",
    "alagará": "alagar",
    "alagarán": "alagar",
    "alagarás": "alagar",
    "alagaré": "alagar",
    "alagaréis": "alagar",
    "alagaría": "alagar",
    "alagaríais": "alagar",
    "alagaríamos": "alagar",
    "alagarían": "alagar",
    "alagarías": "alagar",
    "alagas": "alagar",
    "alagase": "alagar",
    "alagaseis": "alagar",
    "alagasen": "alagar",
    "alagases": "alagar",
    "alagaste": "alagar",
    "alagasteis": "alagar",
    "alago": "alagar",
    "alague": "alagar",
    "alaguemos": "alagar",
    "alaguen": "alagar",
    "alagues": "alagar",
    "alagué": "alagar",
    "alaguéis": "alagar",
    "alagábamos": "alagar",
    "alagáis": "alagar",
    "alagáramos": "alagar",
    "alagáremos": "alagar",
    "alagásemos": "alagar",
    "alagó": "alagar",
    "alahuitas": "alahuita",
    "alajúes": "alajú",
    "alajús": "alajú",
    "alajúses": "alajú",
    "alalias": "alalia",
    "alalás": "alalá",
    "alamanes": "alamán",
    "alamares": "alamar",
    "alambica": "alambicar",
    "alambicaba": "alambicar",
    "alambicabais": "alambicar",
    "alambicaban": "alambicar",
    "alambicabas": "alambicar",
    "alambicad": "alambicar",
    "alambicada": "alambicar",
    "alambicadas": "alambicar",
    "alambicado": "alambicar",
    "alambicados": "alambicar",
    "alambicamientos": "alambicamiento",
    "alambicamos": "alambicar",
    "alambican": "alambicar",
    "alambicando": "alambicar",
    "alambicara": "alambicar",
    "alambicarais": "alambicar",
    "alambicaran": "alambicar",
    "alambicaras": "alambicar",
    "alambicare": "alambicar",
    "alambicareis": "alambicar",
    "alambicaremos": "alambicar",
    "alambicaren": "alambicar",
    "alambicares": "alambicar",
    "alambicaron": "alambicar",
    "alambicará": "alambicar",
    "alambicarán": "alambicar",
    "alambicarás": "alambicar",
    "alambicaré": "alambicar",
    "alambicaréis": "alambicar",
    "alambicaría": "alambicar",
    "alambicaríais": "alambicar",
    "alambicaríamos": "alambicar",
    "alambicarían": "alambicar",
    "alambicarías": "alambicar",
    "alambicas": "alambicar",
    "alambicase": "alambicar",
    "alambicaseis": "alambicar",
    "alambicasen": "alambicar",
    "alambicases": "alambicar",
    "alambicaste": "alambicar",
    "alambicasteis": "alambicar",
    "alambico": "alambicar",
    "alambicábamos": "alambicar",
    "alambicáis": "alambicar",
    "alambicáramos": "alambicar",
    "alambicáremos": "alambicar",
    "alambicásemos": "alambicar",
    "alambicó": "alambicar",
    "alambique": "alambicar",
    "alambiquemos": "alambicar",
    "alambiquen": "alambicar",
    "alambiques": "alambique",
    "alambiqué": "alambicar",
    "alambiquéis": "alambicar",
    "alambores": "alambor",
    "alambra": "alambrar",
    "alambraba": "alambrar",
    "alambrabais": "alambrar",
    "alambraban": "alambrar",
    "alambrabas": "alambrar",
    "alambrad": "alambrar",
    "alambrada": "alambrar",
    "alambradas": "alambrar",
    "alambrado": "alambrar",
    "alambrados": "alambrar",
    "alambramos": "alambrar",
    "alambran": "alambrar",
    "alambrando": "alambrar",
    "alambrara": "alambrar",
    "alambrarais": "alambrar",
    "alambraran": "alambrar",
    "alambraras": "alambrar",
    "alambrare": "alambrar",
    "alambrareis": "alambrar",
    "alambraremos": "alambrar",
    "alambraren": "alambrar",
    "alambrares": "alambrar",
    "alambraron": "alambrar",
    "alambrará": "alambrar",
    "alambrarán": "alambrar",
    "alambrarás": "alambrar",
    "alambraré": "alambrar",
    "alambraréis": "alambrar",
    "alambraría": "alambrar",
    "alambraríais": "alambrar",
    "alambraríamos": "alambrar",
    "alambrarían": "alambrar",
    "alambrarías": "alambrar",
    "alambras": "alambrar",
    "alambrase": "alambrar",
    "alambraseis": "alambrar",
    "alambrasen": "alambrar",
    "alambrases": "alambrar",
    "alambraste": "alambrar",
    "alambrasteis": "alambrar",
    "alambre": "alambrar",
    "alambrea": "alambrear",
    "alambreaba": "alambrear",
    "alambreabais": "alambrear",
    "alambreaban": "alambrear",
    "alambreabas": "alambrear",
    "alambread": "alambrear",
    "alambreada": "alambrear",
    "alambreadas": "alambrear",
    "alambreado": "alambrear",
    "alambreados": "alambrear",
    "alambreamos": "alambrear",
    "alambrean": "alambrear",
    "alambreando": "alambrear",
    "alambreara": "alambrear",
    "alambrearais": "alambrear",
    "alambrearan": "alambrear",
    "alambrearas": "alambrear",
    "alambreare": "alambrear",
    "alambreareis": "alambrear",
    "alambrearemos": "alambrear",
    "alambrearen": "alambrear",
    "alambreares": "alambrear",
    "alambrearon": "alambrear",
    "alambreará": "alambrear",
    "alambrearán": "alambrear",
    "alambrearás": "alambrear",
    "alambrearé": "alambrear",
    "alambrearéis": "alambrear",
    "alambrearía": "alambrear",
    "alambrearíais": "alambrear",
    "alambrearíamos": "alambrear",
    "alambrearían": "alambrear",
    "alambrearías": "alambrear",
    "alambreas": "alambrear",
    "alambrease": "alambrear",
    "alambreaseis": "alambrear",
    "alambreasen": "alambrear",
    "alambreases": "alambrear",
    "alambreaste": "alambrear",
    "alambreasteis": "alambrear",
    "alambree": "alambrear",
    "alambreemos": "alambrear",
    "alambreen": "alambrear",
    "alambrees": "alambrear",
    "alambremos": "alambrar",
    "alambren": "alambrar",
    "alambreo": "alambrear",
    "alambreras": "alambrera",
    "alambres": "alambre",
    "alambreábamos": "alambrear",
    "alambreáis": "alambrear",
    "alambreáramos": "alambrear",
    "alambreáremos": "alambrear",
    "alambreásemos": "alambrear",
    "alambreé": "alambrear",
    "alambreéis": "alambrear",
    "alambreó": "alambrear",
    "alambro": "alambrar",
    "alambrábamos": "alambrar",
    "alambráis": "alambrar",
    "alambráramos": "alambrar",
    "alambráremos": "alambrar",
    "alambrásemos": "alambrar",
    "alambré": "alambrar",
    "alambréis": "alambrar",
    "alambró": "alambrar",
    "alamedas": "alameda",
    "alamines": "alamín",
    "alampa": "alampar",
    "alampaba": "alampar",
    "alampabais": "alampar",
    "alampaban": "alampar",
    "alampabas": "alampar",
    "alampad": "alampar",
    "alampada": "alampar",
    "alampadas": "alampar",
    "alampado": "alampar",
    "alampados": "alampar",
    "alampamos": "alampar",
    "alampan": "alampar",
    "alampando": "alampar",
    "alampara": "alampar",
    "alamparais": "alampar",
    "alamparan": "alampar",
    "alamparas": "alampar",
    "alampare": "alampar",
    "alampareis": "alampar",
    "alamparemos": "alampar",
    "alamparen": "alampar",
    "alampares": "alampar",
    "alamparon": "alampar",
    "alampará": "alampar",
    "alamparán": "alampar",
    "alamparás": "alampar",
    "alamparé": "alampar",
    "alamparéis": "alampar",
    "alamparía": "alampar",
    "alamparíais": "alampar",
    "alamparíamos": "alampar",
    "alamparían": "alampar",
    "alamparías": "alampar",
    "alampas": "alampar",
    "alampase": "alampar",
    "alampaseis": "alampar",
    "alampasen": "alampar",
    "alampases": "alampar",
    "alampaste": "alampar",
    "alampasteis": "alampar",
    "alampe": "alampar",
    "alampemos": "alampar",
    "alampen": "alampar",
    "alampes": "alampar",
    "alampo": "alampar",
    "alampábamos": "alampar",
    "alampáis": "alampar",
    "alampáramos": "alampar",
    "alampáremos": "alampar",
    "alampásemos": "alampar",
    "alampé": "alampar",
    "alampéis": "alampar",
    "alampó": "alampar",
    "alana": "alano",
    "alanas": "alano",
    "alancea": "alancear",
    "alanceaba": "alancear",
    "alanceabais": "alancear",
    "alanceaban": "alancear",
    "alanceabas": "alancear",
    "alancead": "alancear",
    "alanceada": "alancear",
    "alanceadas": "alancear",
    "alanceado": "alancear",
    "alanceados": "alancear",
    "alanceamos": "alancear",
    "alancean": "alancear",
    "alanceando": "alancear",
    "alanceara": "alancear",
    "alancearais": "alancear",
    "alancearan": "alancear",
    "alancearas": "alancear",
    "alanceare": "alancear",
    "alanceareis": "alancear",
    "alancearemos": "alancear",
    "alancearen": "alancear",
    "alanceares": "alancear",
    "alancearon": "alancear",
    "alanceará": "alancear",
    "alancearán": "alancear",
    "alancearás": "alancear",
    "alancearé": "alancear",
    "alancearéis": "alancear",
    "alancearía": "alancear",
    "alancearíais": "alancear",
    "alancearíamos": "alancear",
    "alancearían": "alancear",
    "alancearías": "alancear",
    "alanceas": "alancear",
    "alancease": "alancear",
    "alanceaseis": "alancear",
    "alanceasen": "alancear",
    "alanceases": "alancear",
    "alanceaste": "alancear",
    "alanceasteis": "alancear",
    "alancee": "alancear",
    "alanceemos": "alancear",
    "alanceen": "alancear",
    "alancees": "alancear",
    "alanceo": "alancear",
    "alanceábamos": "alancear",
    "alanceáis": "alancear",
    "alanceáramos": "alancear",
    "alanceáremos": "alancear",
    "alanceásemos": "alancear",
    "alanceé": "alancear",
    "alanceéis": "alancear",
    "alanceó": "alancear",
    "alaninas": "alanina",
    "alanitas": "alanita",
    "alanos": "alano",
    "alantoides": "alantoide",
    "alantoínas": "alantoína",
    "alardea": "alardear",
    "alardeaba": "alardear",
    "alardeabais": "alardear",
    "alardeaban": "alardear",
    "alardeabas": "alardear",
    "alardead": "alardear",
    "alardeada": "alardear",
    "alardeadas": "alardear",
    "alardeado": "alardear",
    "alardeados": "alardear",
    "alardeamos": "alardear",
    "alardean": "alardear",
    "alardeando": "alardear",
    "alardeara": "alardear",
    "alardearais": "alardear",
    "alardearan": "alardear",
    "alardearas": "alardear",
    "alardeare": "alardear",
    "alardeareis": "alardear",
    "alardearemos": "alardear",
    "alardearen": "alardear",
    "alardeares": "alardear",
    "alardearon": "alardear",
    "alardeará": "alardear",
    "alardearán": "alardear",
    "alardearás": "alardear",
    "alardearé": "alardear",
    "alardearéis": "alardear",
    "alardearía": "alardear",
    "alardearíais": "alardear",
    "alardearíamos": "alardear",
    "alardearían": "alardear",
    "alardearías": "alardear",
    "alardeas": "alardear",
    "alardease": "alardear",
    "alardeaseis": "alardear",
    "alardeasen": "alardear",
    "alardeases": "alardear",
    "alardeaste": "alardear",
    "alardeasteis": "alardear",
    "alardee": "alardear",
    "alardeemos": "alardear",
    "alardeen": "alardear",
    "alardees": "alardear",
    "alardeo": "alardear",
    "alardeos": "alardeo",
    "alardes": "alarde",
    "alardeábamos": "alardear",
    "alardeáis": "alardear",
    "alardeáramos": "alardear",
    "alardeáremos": "alardear",
    "alardeásemos": "alardear",
    "alardeé": "alardear",
    "alardeéis": "alardear",
    "alardeó": "alardear",
    "alares": "alar",
    "alarga": "alargar",
    "alargaba": "alargar",
    "alargabais": "alargar",
    "alargaban": "alargar",
    "alargabas": "alargar",
    "alargad": "alargar",
    "alargada": "alargar",
    "alargadas": "alargar",
    "alargaderas": "alargadera",
    "alargado": "alargar",
    "alargadores": "alargador",
    "alargados": "alargar",
    "alargamientos": "alargamiento",
    "alargamos": "alargar",
    "alargan": "alargar",
    "alargando": "alargar",
    "alargara": "alargar",
    "alargarais": "alargar",
    "alargaran": "alargar",
    "alargaras": "alargar",
    "alargare": "alargar",
    "alargareis": "alargar",
    "alargaremos": "alargar",
    "alargaren": "alargar",
    "alargares": "alargar",
    "alargaron": "alargar",
    "alargará": "alargar",
    "alargarán": "alargar",
    "alargarás": "alargar",
    "alargaré": "alargar",
    "alargaréis": "alargar",
    "alargaría": "alargar",
    "alargaríais": "alargar",
    "alargaríamos": "alargar",
    "alargarían": "alargar",
    "alargarías": "alargar",
    "alargas": "alargar",
    "alargase": "alargar",
    "alargaseis": "alargar",
    "alargasen": "alargar",
    "alargases": "alargar",
    "alargaste": "alargar",
    "alargasteis": "alargar",
    "alargo": "alargar",
    "alargue": "alargar",
    "alarguemos": "alargar",
    "alarguen": "alargar",
    "alargues": "alargue",
    "alargué": "alargar",
    "alarguéis": "alargar",
    "alargábamos": "alargar",
    "alargáis": "alargar",
    "alargáramos": "alargar",
    "alargáremos": "alargar",
    "alargásemos": "alargar",
    "alargó": "alargar",
    "alarias": "alaria",
    "alaridos": "alarido",
    "alarifes": "alarife",
    "alarma": "alarmar",
    "alarmaba": "alarmar",
    "alarmabais": "alarmar",
    "alarmaban": "alarmar",
    "alarmabas": "alarmar",
    "alarmad": "alarmar",
    "alarmada": "alarmar",
    "alarmadas": "alarmar",
    "alarmado": "alarmar",
    "alarmados": "alarmar",
    "alarmamos": "alarmar",
    "alarman": "alarmar",
    "alarmando": "alarmar",
    "alarmantes": "alarmante",
    "alarmara": "alarmar",
    "alarmarais": "alarmar",
    "alarmaran": "alarmar",
    "alarmaras": "alarmar",
    "alarmare": "alarmar",
    "alarmareis": "alarmar",
    "alarmaremos": "alarmar",
    "alarmaren": "alarmar",
    "alarmares": "alarmar",
    "alarmaron": "alarmar",
    "alarmará": "alarmar",
    "alarmarán": "alarmar",
    "alarmarás": "alarmar",
    "alarmaré": "alarmar",
    "alarmaréis": "alarmar",
    "alarmaría": "alarmar",
    "alarmaríais": "alarmar",
    "alarmaríamos": "alarmar",
    "alarmarían": "alarmar",
    "alarmarías": "alarmar",
    "alarmas": "alarmar",
    "alarmase": "alarmar",
    "alarmaseis": "alarmar",
    "alarmasen": "alarmar",
    "alarmases": "alarmar",
    "alarmaste": "alarmar",
    "alarmasteis": "alarmar",
    "alarme": "alarmar",
    "alarmemos": "alarmar",
    "alarmen": "alarmar",
    "alarmes": "alarmar",
    "alarmismos": "alarmismo",
    "alarmistas": "alarmista",
    "alarmo": "alarmar",
    "alarmábamos": "alarmar",
    "alarmáis": "alarmar",
    "alarmáramos": "alarmar",
    "alarmáremos": "alarmar",
    "alarmásemos": "alarmar",
    "alarmé": "alarmar",
    "alarméis": "alarmar",
    "alarmó": "alarmar",
    "alaroces": "alaroz",
    "alas": "ala",
    "alas-pívot": "ala-pívot",
    "alastra": "alastrar",
    "alastraba": "alastrar",
    "alastrabais": "alastrar",
    "alastraban": "alastrar",
    "alastrabas": "alastrar",
    "alastrad": "alastrar",
    "alastrada": "alastrar",
    "alastradas": "alastrar",
    "alastrado": "alastrar",
    "alastrados": "alastrar",
    "alastramos": "alastrar",
    "alastran": "alastrar",
    "alastrando": "alastrar",
    "alastrara": "alastrar",
    "alastrarais": "alastrar",
    "alastraran": "alastrar",
    "alastraras": "alastrar",
    "alastrare": "alastrar",
    "alastrareis": "alastrar",
    "alastraremos": "alastrar",
    "alastraren": "alastrar",
    "alastrares": "alastrar",
    "alastraron": "alastrar",
    "alastrará": "alastrar",
    "alastrarán": "alastrar",
    "alastrarás": "alastrar",
    "alastraré": "alastrar",
    "alastraréis": "alastrar",
    "alastraría": "alastrar",
    "alastraríais": "alastrar",
    "alastraríamos": "alastrar",
    "alastrarían": "alastrar",
    "alastrarías": "alastrar",
    "alastras": "alastrar",
    "alastrase": "alastrar",
    "alastraseis": "alastrar",
    "alastrasen": "alastrar",
    "alastrases": "alastrar",
    "alastraste": "alastrar",
    "alastrasteis": "alastrar",
    "alastre": "alastrar",
    "alastremos": "alastrar",
    "alastren": "alastrar",
    "alastres": "alastrar",
    "alastro": "alastrar",
    "alastrábamos": "alastrar",
    "alastráis": "alastrar",
    "alastráramos": "alastrar",
    "alastráremos": "alastrar",
    "alastrásemos": "alastrar",
    "alastré": "alastrar",
    "alastréis": "alastrar",
    "alastró": "alastrar",
    "alauitas": "alauita",
    "alavenses": "alavense",
    "alavesa": "alavés",
    "alavesas": "alavés",
    "alaveses": "alavés",
    "alavésa": "alavés",
    "alavésas": "alavés",
    "alavéses": "alavés",
    "alazana": "alazán",
    "alazanas": "alazán",
    "alazanes": "alazán",
    "alazores": "alazor",
    "albacaras": "albacara",
    "albaceas": "albacea",
    "albaceazgos": "albaceazgo",
    "albacetenses": "albacetense",
    "albaceteña": "albaceteño",
    "albaceteñas": "albaceteño",
    "albaceteños": "albaceteño",
    "albacoras": "albacora",
    "albahacas": "albahaca",
    "albalás": "albalá",
    "albana": "albano",
    "albanas": "albano",
    "albanegas": "albanega",
    "albanesa": "albanés",
    "albanesas": "albanés",
    "albaneses": "albanés",
    "albanos": "albano",
    "albanésa": "albanés",
    "albanésas": "albanés",
    "albanéses": "albanés",
    "albaranes": "albarán",
    "albarda": "albardar",
    "albardaba": "albardar",
    "albardabais": "albardar",
    "albardaban": "albardar",
    "albardabas": "albardar",
    "albardad": "albardar",
    "albardada": "albardar",
    "albardadas": "albardar",
    "albardado": "albardar",
    "albardados": "albardar",
    "albardamos": "albardar",
    "albardan": "albardar",
    "albardando": "albardar",
    "albardanea": "albardanear",
    "albardaneaba": "albardanear",
    "albardaneabais": "albardanear",
    "albardaneaban": "albardanear",
    "albardaneabas": "albardanear",
    "albardanead": "albardanear",
    "albardaneada": "albardanear",
    "albardaneadas": "albardanear",
    "albardaneado": "albardanear",
    "albardaneados": "albardanear",
    "albardaneamos": "albardanear",
    "albardanean": "albardanear",
    "albardaneando": "albardanear",
    "albardaneara": "albardanear",
    "albardanearais": "albardanear",
    "albardanearan": "albardanear",
    "albardanearas": "albardanear",
    "albardaneare": "albardanear",
    "albardaneareis": "albardanear",
    "albardanearemos": "albardanear",
    "albardanearen": "albardanear",
    "albardaneares": "albardanear",
    "albardanearon": "albardanear",
    "albardaneará": "albardanear",
    "albardanearán": "albardanear",
    "albardanearás": "albardanear",
    "albardanearé": "albardanear",
    "albardanearéis": "albardanear",
    "albardanearía": "albardanear",
    "albardanearíais": "albardanear",
    "albardanearíamos": "albardanear",
    "albardanearían": "albardanear",
    "albardanearías": "albardanear",
    "albardaneas": "albardanear",
    "albardanease": "albardanear",
    "albardaneaseis": "albardanear",
    "albardaneasen": "albardanear",
    "albardaneases": "albardanear",
    "albardaneaste": "albardanear",
    "albardaneasteis": "albardanear",
    "albardanee": "albardanear",
    "albardaneemos": "albardanear",
    "albardaneen": "albardanear",
    "albardanees": "albardanear",
    "albardaneo": "albardanear",
    "albardaneábamos": "albardanear",
    "albardaneáis": "albardanear",
    "albardaneáramos": "albardanear",
    "albardaneáremos": "albardanear",
    "albardaneásemos": "albardanear",
    "albardaneé": "albardanear",
    "albardaneéis": "albardanear",
    "albardaneó": "albardanear",
    "albardara": "albardar",
    "albardarais": "albardar",
    "albardaran": "albardar",
    "albardaras": "albardar",
    "albardare": "albardar",
    "albardareis": "albardar",
    "albardaremos": "albardar",
    "albardaren": "albardar",
    "albardares": "albardar",
    "albardaron": "albardar",
    "albardará": "albardar",
    "albardarán": "albardar",
    "albardarás": "albardar",
    "albardaré": "albardar",
    "albardaréis": "albardar",
    "albardaría": "albardar",
    "albardaríais": "albardar",
    "albardaríamos": "albardar",
    "albardarían": "albardar",
    "albardarías": "albardar",
    "albardas": "albardar",
    "albardase": "albardar",
    "albardaseis": "albardar",
    "albardasen": "albardar",
    "albardases": "albardar",
    "albardaste": "albardar",
    "albardasteis": "albardar",
    "albarde": "albardar",
    "albardemos": "albardar",
    "albarden": "albardar",
    "albarderos": "albardero",
    "albarderías": "albardería",
    "albardes": "albardar",
    "albardillas": "albardilla",
    "albardines": "albardín",
    "albardo": "albardar",
    "albardoneros": "albardonero",
    "albardonerías": "albardonería",
    "albardones": "albardón",
    "albardábamos": "albardar",
    "albardáis": "albardar",
    "albardáramos": "albardar",
    "albardáremos": "albardar",
    "albardásemos": "albardar",
    "albardé": "albardar",
    "albardéis": "albardar",
    "albardó": "albardar",
    "albarelos": "albarelo",
    "albares": "albar",
    "albaricoqueros": "albaricoquero",
    "albaricoques": "albaricoque",
    "albarillos": "albarillo",
    "albariza": "albarizo",
    "albarizas": "albarizo",
    "albarizos": "albarizo",
    "albarradas": "albarrada",
    "albarranas": "albarrana",
    "albarsas": "albarsa",
    "albas": "alba",
    "albayaldes": "albayalde",
    "albazana": "albazano",
    "albazanas": "albazano",
    "albazanos": "albazano",
    "albañales": "albañal",
    "albañilerías": "albañilería",
    "albañiles": "albañil",
    "albea": "albear",
    "albeaba": "albear",
    "albeabais": "albear",
    "albeaban": "albear",
    "albeabas": "albear",
    "albead": "albear",
    "albeada": "albear",
    "albeadas": "albear",
    "albeado": "albear",
    "albeados": "albear",
    "albeamos": "albear",
    "albean": "albear",
    "albeando": "albear",
    "albeara": "albear",
    "albearais": "albear",
    "albearan": "albear",
    "albearas": "albear",
    "albeare": "albear",
    "albeareis": "albear",
    "albearemos": "albear",
    "albearen": "albear",
    "albeares": "albear",
    "albearon": "albear",
    "albeará": "albear",
    "albearán": "albear",
    "albearás": "albear",
    "albearé": "albear",
    "albearéis": "albear",
    "albearía": "albear",
    "albearíais": "albear",
    "albearíamos": "albear",
    "albearían": "albear",
    "albearías": "albear",
    "albeas": "albear",
    "albease": "albear",
    "albeaseis": "albear",
    "albeasen": "albear",
    "albeases": "albear",
    "albeaste": "albear",
    "albeasteis": "albear",
    "albedos": "albedo",
    "albedríos": "albedrío",
    "albee": "albear",
    "albeemos": "albear",
    "albeen": "albear",
    "albees": "albear",
    "albeiterías": "albeitería",
    "albengalas": "albengala",
    "albeo": "albear",
    "albera": "albero",
    "alberas": "albero",
    "albercas": "alberca",
    "alberga": "albergar",
    "albergaba": "albergar",
    "albergabais": "albergar",
    "albergaban": "albergar",
    "albergabas": "albergar",
    "albergad": "albergar",
    "albergada": "albergar",
    "albergadas": "albergar",
    "albergado": "albergar",
    "albergados": "albergar",
    "albergamos": "albergar",
    "albergan": "albergar",
    "albergando": "albergar",
    "albergara": "albergar",
    "albergarais": "albergar",
    "albergaran": "albergar",
    "albergaras": "albergar",
    "albergare": "albergar",
    "albergareis": "albergar",
    "albergaremos": "albergar",
    "albergaren": "albergar",
    "albergares": "albergar",
    "albergaron": "albergar",
    "albergará": "albergar",
    "albergarán": "albergar",
    "albergarás": "albergar",
    "albergaré": "albergar",
    "albergaréis": "albergar",
    "albergaría": "albergar",
    "albergaríais": "albergar",
    "albergaríamos": "albergar",
    "albergarían": "albergar",
    "albergarías": "albergar",
    "albergas": "albergar",
    "albergase": "albergar",
    "albergaseis": "albergar",
    "albergasen": "albergar",
    "albergases": "albergar",
    "albergaste": "albergar",
    "albergasteis": "albergar",
    "albergo": "albergar",
    "albergue": "albergar",
    "alberguemos": "albergar",
    "alberguen": "albergar",
    "albergues": "albergue",
    "albergué": "albergar",
    "alberguéis": "albergar",
    "albergábamos": "albergar",
    "albergáis": "albergar",
    "albergáramos": "albergar",
    "albergáremos": "albergar",
    "albergásemos": "albergar",
    "albergó": "albergar",
    "alberos": "albero",
    "albertitas": "albertita",
    "albeábamos": "albear",
    "albeáis": "albear",
    "albeáramos": "albear",
    "albeáremos": "albear",
    "albeásemos": "albear",
    "albeé": "albear",
    "albeéis": "albear",
    "albeó": "albear",
    "albicantes": "albicante",
    "albigenses": "albigense",
    "albilla": "albillo",
    "albillas": "albillo",
    "albillos": "albillo",
    "albina": "albino",
    "albinas": "albino",
    "albines": "albín",
    "albinismos": "albinismo",
    "albinos": "albino",
    "albistas": "albista",
    "albitas": "albita",
    "alboaires": "alboaire",
    "albogones": "albogón",
    "albogues": "albogue",
    "albondiguillas": "albondiguilla",
    "alboradas": "alborada",
    "alborea": "alborear",
    "alboreaba": "alborear",
    "alboreabais": "alborear",
    "alboreaban": "alborear",
    "alboreabas": "alborear",
    "alboread": "alborear",
    "alboreada": "alborear",
    "alboreadas": "alborear",
    "alboreado": "alborear",
    "alboreados": "alborear",
    "alboreamos": "alborear",
    "alborean": "alborear",
    "alboreando": "alborear",
    "alboreara": "alborear",
    "alborearais": "alborear",
    "alborearan": "alborear",
    "alborearas": "alborear",
    "alboreare": "alborear",
    "alboreareis": "alborear",
    "alborearemos": "alborear",
    "alborearen": "alborear",
    "alboreares": "alborear",
    "alborearon": "alborear",
    "alboreará": "alborear",
    "alborearán": "alborear",
    "alborearás": "alborear",
    "alborearé": "alborear",
    "alborearéis": "alborear",
    "alborearía": "alborear",
    "alborearíais": "alborear",
    "alborearíamos": "alborear",
    "alborearían": "alborear",
    "alborearías": "alborear",
    "alboreas": "alborear",
    "alborease": "alborear",
    "alboreaseis": "alborear",
    "alboreasen": "alborear",
    "alboreases": "alborear",
    "alboreaste": "alborear",
    "alboreasteis": "alborear",
    "alboree": "alborear",
    "alboreemos": "alborear",
    "alboreen": "alborear",
    "alborees": "alborear",
    "alboreo": "alborear",
    "alboreos": "alboreo",
    "albores": "albor",
    "alboreábamos": "alborear",
    "alboreáis": "alborear",
    "alboreáramos": "alborear",
    "alboreáremos": "alborear",
    "alboreásemos": "alborear",
    "alboreé": "alborear",
    "alboreéis": "alborear",
    "alboreó": "alborear",
    "albornoces": "albornoz",
    "alboroce": "alborozar",
    "alborocemos": "alborozar",
    "alborocen": "alborozar",
    "alboroces": "alborozar",
    "alborocé": "alborozar",
    "alborocéis": "alborozar",
    "alboronías": "alboronía",
    "alborota": "alborotar",
    "alborotaba": "alborotar",
    "alborotabais": "alborotar",
    "alborotaban": "alborotar",
    "alborotabas": "alborotar",
    "alborotad": "alborotar",
    "alborotada": "alborotar",
    "alborotadas": "alborotar",
    "alborotadiza": "alborotadizo",
    "alborotadizas": "alborotadizo",
    "alborotadizos": "alborotadizo",
    "alborotado": "alborotar",
    "alborotadora": "alborotador",
    "alborotadoras": "alborotador",
    "alborotadores": "alborotador",
    "alborotados": "alborotar",
    "alborotamientos": "alborotamiento",
    "alborotamos": "alborotar",
    "alborotan": "alborotar",
    "alborotando": "alborotar",
    "alborotara": "alborotar",
    "alborotarais": "alborotar",
    "alborotaran": "alborotar",
    "alborotaras": "alborotar",
    "alborotare": "alborotar",
    "alborotareis": "alborotar",
    "alborotaremos": "alborotar",
    "alborotaren": "alborotar",
    "alborotares": "alborotar",
    "alborotaron": "alborotar",
    "alborotará": "alborotar",
    "alborotarán": "alborotar",
    "alborotarás": "alborotar",
    "alborotaré": "alborotar",
    "alborotaréis": "alborotar",
    "alborotaría": "alborotar",
    "alborotaríais": "alborotar",
    "alborotaríamos": "alborotar",
    "alborotarían": "alborotar",
    "alborotarías": "alborotar",
    "alborotas": "alborotar",
    "alborotase": "alborotar",
    "alborotaseis": "alborotar",
    "alborotasen": "alborotar",
    "alborotases": "alborotar",
    "alborotaste": "alborotar",
    "alborotasteis": "alborotar",
    "alborote": "alborotar",
    "alborotemos": "alborotar",
    "alboroten": "alborotar",
    "alborotes": "alborotar",
    "alboroto": "alborotar",
    "alborotos": "alboroto",
    "alborotábamos": "alborotar",
    "alborotáis": "alborotar",
    "alborotáramos": "alborotar",
    "alborotáremos": "alborotar",
    "alborotásemos": "alborotar",
    "alboroté": "alborotar",
    "alborotéis": "alborotar",
    "alborotó": "alborotar",
    "alboroza": "alborozar",
    "alborozaba": "alborozar",
    "alborozabais": "alborozar",
    "alborozaban": "alborozar",
    "alborozabas": "alborozar",
    "alborozad": "alborozar",
    "alborozada": "alborozar",
    "alborozadas": "alborozar",
    "alborozado": "alborozar",
    "alborozados": "alborozar",
    "alborozamos": "alborozar",
    "alborozan": "alborozar",
    "alborozando": "alborozar",
    "alborozara": "alborozar",
    "alborozarais": "alborozar",
    "alborozaran": "alborozar",
    "alborozaras": "alborozar",
    "alborozare": "alborozar",
    "alborozareis": "alborozar",
    "alborozaremos": "alborozar",
    "alborozaren": "alborozar",
    "alborozares": "alborozar",
    "alborozaron": "alborozar",
    "alborozará": "alborozar",
    "alborozarán": "alborozar",
    "alborozarás": "alborozar",
    "alborozaré": "alborozar",
    "alborozaréis": "alborozar",
    "alborozaría": "alborozar",
    "alborozaríais": "alborozar",
    "alborozaríamos": "alborozar",
    "alborozarían": "alborozar",
    "alborozarías": "alborozar",
    "alborozas": "alborozar",
    "alborozase": "alborozar",
    "alborozaseis": "alborozar",
    "alborozasen": "alborozar",
    "alborozases": "alborozar",
    "alborozaste": "alborozar",
    "alborozasteis": "alborozar",
    "alborozo": "alborozar",
    "alborozos": "alborozo",
    "alborozábamos": "alborozar",
    "alborozáis": "alborozar",
    "alborozáramos": "alborozar",
    "alborozáremos": "alborozar",
    "alborozásemos": "alborozar",
    "alborozó": "alborozar",
    "albuferas": "albufera",
    "albugos": "albugo",
    "albugínea": "albugíneo",
    "albugíneas": "albugíneo",
    "albugíneos": "albugíneo",
    "albuheras": "albuhera",
    "albuminemias": "albuminemia",
    "albuminoidea": "albuminoideo",
    "albuminoideas": "albuminoideo",
    "albuminoideos": "albuminoideo",
    "albuminoides": "albuminoide",
    "albuminosa": "albuminoso",
    "albuminosas": "albuminoso",
    "albuminosos": "albuminoso",
    "albuminurias": "albuminuria",
    "albuminúrica": "albuminúrico",
    "albuminúricas": "albuminúrico",
    "albuminúricos": "albuminúrico",
    "albumosas": "albumosa",
    "alburas": "albura",
    "alburea": "alburear",
    "albureaba": "alburear",
    "albureabais": "alburear",
    "albureaban": "alburear",
    "albureabas": "alburear",
    "alburead": "alburear",
    "albureada": "alburear",
    "albureadas": "alburear",
    "albureado": "alburear",
    "albureados": "alburear",
    "albureamos": "alburear",
    "alburean": "alburear",
    "albureando": "alburear",
    "albureara": "alburear",
    "alburearais": "alburear",
    "alburearan": "alburear",
    "alburearas": "alburear",
    "albureare": "alburear",
    "albureareis": "alburear",
    "alburearemos": "alburear",
    "alburearen": "alburear",
    "albureares": "alburear",
    "alburearon": "alburear",
    "albureará": "alburear",
    "alburearán": "alburear",
    "alburearás": "alburear",
    "alburearé": "alburear",
    "alburearéis": "alburear",
    "alburearía": "alburear",
    "alburearíais": "alburear",
    "alburearíamos": "alburear",
    "alburearían": "alburear",
    "alburearías": "alburear",
    "albureas": "alburear",
    "alburease": "alburear",
    "albureaseis": "alburear",
    "albureasen": "alburear",
    "albureases": "alburear",
    "albureaste": "alburear",
    "albureasteis": "alburear",
    "alburee": "alburear",
    "albureemos": "alburear",
    "albureen": "alburear",
    "alburees": "alburear",
    "albureo": "alburear",
    "albures": "albur",
    "albureábamos": "alburear",
    "albureáis": "alburear",
    "albureáramos": "alburear",
    "albureáremos": "alburear",
    "albureásemos": "alburear",
    "albureé": "alburear",
    "albureéis": "alburear",
    "albureó": "alburear",
    "alburnos": "alburno",
    "albéitares": "albéitar",
    "albéntolas": "albéntola",
    "albérchigos": "albérchigo",
    "albóndigas": "albóndiga",
    "albúmenes": "albumen",
    "albúminas": "albúmina",
    "alcabalas": "alcabala",
    "alcabaleros": "alcabalero",
    "alcabores": "alcabor",
    "alcaceles": "alcacel",
    "alcaceres": "alcacer",
    "alcachofales": "alcachofal",
    "alcachofares": "alcachofar",
    "alcachofas": "alcachofa",
    "alcachoferas": "alcachofera",
    "alcaciles": "alcacil",
    "alcacíes": "alcací",
    "alcacís": "alcací",
    "alcadienos": "alcadieno",
    "alcahace": "alcahazar",
    "alcahacemos": "alcahazar",
    "alcahacen": "alcahazar",
    "alcahaces": "alcahazar",
    "alcahacé": "alcahazar",
    "alcahacéis": "alcahazar",
    "alcahaza": "alcahazar",
    "alcahazaba": "alcahazar",
    "alcahazabais": "alcahazar",
    "alcahazaban": "alcahazar",
    "alcahazabas": "alcahazar",
    "alcahazad": "alcahazar",
    "alcahazada": "alcahazar",
    "alcahazadas": "alcahazar",
    "alcahazado": "alcahazar",
    "alcahazados": "alcahazar",
    "alcahazamos": "alcahazar",
    "alcahazan": "alcahazar",
    "alcahazando": "alcahazar",
    "alcahazara": "alcahazar",
    "alcahazarais": "alcahazar",
    "alcahazaran": "alcahazar",
    "alcahazaras": "alcahazar",
    "alcahazare": "alcahazar",
    "alcahazareis": "alcahazar",
    "alcahazaremos": "alcahazar",
    "alcahazaren": "alcahazar",
    "alcahazares": "alcahazar",
    "alcahazaron": "alcahazar",
    "alcahazará": "alcahazar",
    "alcahazarán": "alcahazar",
    "alcahazarás": "alcahazar",
    "alcahazaré": "alcahazar",
    "alcahazaréis": "alcahazar",
    "alcahazaría": "alcahazar",
    "alcahazaríais": "alcahazar",
    "alcahazaríamos": "alcahazar",
    "alcahazarían": "alcahazar",
    "alcahazarías": "alcahazar",
    "alcahazas": "alcahazar",
    "alcahazase": "alcahazar",
    "alcahazaseis": "alcahazar",
    "alcahazasen": "alcahazar",
    "alcahazases": "alcahazar",
    "alcahazaste": "alcahazar",
    "alcahazasteis": "alcahazar",
    "alcahazo": "alcahazar",
    "alcahazábamos": "alcahazar",
    "alcahazáis": "alcahazar",
    "alcahazáramos": "alcahazar",
    "alcahazáremos": "alcahazar",
    "alcahazásemos": "alcahazar",
    "alcahazó": "alcahazar",
    "alcahueta": "alcahuete",
    "alcahuetas": "alcahuete",
    "alcahuetea": "alcahuetear",
    "alcahueteaba": "alcahuetear",
    "alcahueteabais": "alcahuetear",
    "alcahueteaban": "alcahuetear",
    "alcahueteabas": "alcahuetear",
    "alcahuetead": "alcahuetear",
    "alcahueteada": "alcahuetear",
    "alcahueteadas": "alcahuetear",
    "alcahueteado": "alcahuetear",
    "alcahueteados": "alcahuetear",
    "alcahueteamos": "alcahuetear",
    "alcahuetean": "alcahuetear",
    "alcahueteando": "alcahuetear",
    "alcahueteara": "alcahuetear",
    "alcahuetearais": "alcahuetear",
    "alcahuetearan": "alcahuetear",
    "alcahuetearas": "alcahuetear",
    "alcahueteare": "alcahuetear",
    "alcahueteareis": "alcahuetear",
    "alcahuetearemos": "alcahuetear",
    "alcahuetearen": "alcahuetear",
    "alcahueteares": "alcahuetear",
    "alcahuetearon": "alcahuetear",
    "alcahueteará": "alcahuetear",
    "alcahuetearán": "alcahuetear",
    "alcahuetearás": "alcahuetear",
    "alcahuetearé": "alcahuetear",
    "alcahuetearéis": "alcahuetear",
    "alcahuetearía": "alcahuetear",
    "alcahuetearíais": "alcahuetear",
    "alcahuetearíamos": "alcahuetear",
    "alcahuetearían": "alcahuetear",
    "alcahuetearías": "alcahuetear",
    "alcahueteas": "alcahuetear",
    "alcahuetease": "alcahuetear",
    "alcahueteaseis": "alcahuetear",
    "alcahueteasen": "alcahuetear",
    "alcahueteases": "alcahuetear",
    "alcahueteaste": "alcahuetear",
    "alcahueteasteis": "alcahuetear",
    "alcahuetee": "alcahuetear",
    "alcahueteemos": "alcahuetear",
    "alcahueteen": "alcahuetear",
    "alcahuetees": "alcahuetear",
    "alcahueteo": "alcahuetear",
    "alcahueterías": "alcahuetería",
    "alcahuetes": "alcahuete",
    "alcahueteábamos": "alcahuetear",
    "alcahueteáis": "alcahuetear",
    "alcahueteáramos": "alcahuetear",
    "alcahueteáremos": "alcahuetear",
    "alcahueteásemos": "alcahuetear",
    "alcahueteé": "alcahuetear",
    "alcahueteéis": "alcahuetear",
    "alcahueteó": "alcahuetear",
    "alcaica": "alcaico",
    "alcaicas": "alcaico",
    "alcaicerías": "alcaicería",
    "alcaicos": "alcaico",
    "alcaides": "alcaide",
    "alcaidesas": "alcaidesa",
    "alcaidías": "alcaidía",
    "alcaldables": "alcaldable",
    "alcaldadas": "alcaldada",
    "alcaldes": "alcalde",
    "alcaldesas": "alcaldesa",
    "alcaldías": "alcaldía",
    "alcalescencias": "alcalescencia",
    "alcalice": "alcalizar",
    "alcalicemos": "alcalizar",
    "alcalicen": "alcalizar",
    "alcalices": "alcalizar",
    "alcalicé": "alcalizar",
    "alcalicéis": "alcalizar",
    "alcalimetrías": "alcalimetría",
    "alcalina": "alcalino",
    "alcalinas": "alcalino",
    "alcalinidades": "alcalinidad",
    "alcalinizaciones": "alcalinización",
    "alcalinos": "alcalino",
    "alcalinotérrea": "alcalinotérreo",
    "alcalinotérreas": "alcalinotérreo",
    "alcalinotérreos": "alcalinotérreo",
    "alcaliza": "alcalizar",
    "alcalizaba": "alcalizar",
    "alcalizabais": "alcalizar",
    "alcalizaban": "alcalizar",
    "alcalizabas": "alcalizar",
    "alcalizaciones": "alcalización",
    "alcalizad": "alcalizar",
    "alcalizada": "alcalizar",
    "alcalizadas": "alcalizar",
    "alcalizado": "alcalizar",
    "alcalizados": "alcalizar",
    "alcalizamos": "alcalizar",
    "alcalizan": "alcalizar",
    "alcalizando": "alcalizar",
    "alcalizara": "alcalizar",
    "alcalizarais": "alcalizar",
    "alcalizaran": "alcalizar",
    "alcalizaras": "alcalizar",
    "alcalizare": "alcalizar",
    "alcalizareis": "alcalizar",
    "alcalizaremos": "alcalizar",
    "alcalizaren": "alcalizar",
    "alcalizares": "alcalizar",
    "alcalizaron": "alcalizar",
    "alcalizará": "alcalizar",
    "alcalizarán": "alcalizar",
    "alcalizarás": "alcalizar",
    "alcalizaré": "alcalizar",
    "alcalizaréis": "alcalizar",
    "alcalizaría": "alcalizar",
    "alcalizaríais": "alcalizar",
    "alcalizaríamos": "alcalizar",
    "alcalizarían": "alcalizar",
    "alcalizarías": "alcalizar",
    "alcalizas": "alcalizar",
    "alcalizase": "alcalizar",
    "alcalizaseis": "alcalizar",
    "alcalizasen": "alcalizar",
    "alcalizases": "alcalizar",
    "alcalizaste": "alcalizar",
    "alcalizasteis": "alcalizar",
    "alcalizo": "alcalizar",
    "alcalizábamos": "alcalizar",
    "alcalizáis": "alcalizar",
    "alcalizáramos": "alcalizar",
    "alcalizáremos": "alcalizar",
    "alcalizásemos": "alcalizar",
    "alcalizó": "alcalizar",
    "alcaloides": "alcaloide",
    "alcalímetros": "alcalímetro",
    "alcamonías": "alcamonía",
    "alcance": "alcanzar",
    "alcancemos": "alcanzar",
    "alcancen": "alcanzar",
    "alcances": "alcanzar",
    "alcancé": "alcanzar",
    "alcancéis": "alcanzar",
    "alcancías": "alcancía",
    "alcanfora": "alcanforar",
    "alcanforaba": "alcanforar",
    "alcanforabais": "alcanforar",
    "alcanforaban": "alcanforar",
    "alcanforabas": "alcanforar",
    "alcanforad": "alcanforar",
    "alcanforada": "alcanforar",
    "alcanforadas": "alcanforar",
    "alcanforado": "alcanforar",
    "alcanforados": "alcanforar",
    "alcanforamos": "alcanforar",
    "alcanforan": "alcanforar",
    "alcanforando": "alcanforar",
    "alcanforara": "alcanforar",
    "alcanforarais": "alcanforar",
    "alcanforaran": "alcanforar",
    "alcanforaras": "alcanforar",
    "alcanforare": "alcanforar",
    "alcanforareis": "alcanforar",
    "alcanforaremos": "alcanforar",
    "alcanforaren": "alcanforar",
    "alcanforares": "alcanforar",
    "alcanforaron": "alcanforar",
    "alcanforará": "alcanforar",
    "alcanforarán": "alcanforar",
    "alcanforarás": "alcanforar",
    "alcanforaré": "alcanforar",
    "alcanforaréis": "alcanforar",
    "alcanforaría": "alcanforar",
    "alcanforaríais": "alcanforar",
    "alcanforaríamos": "alcanforar",
    "alcanforarían": "alcanforar",
    "alcanforarías": "alcanforar",
    "alcanforas": "alcanforar",
    "alcanforase": "alcanforar",
    "alcanforaseis": "alcanforar",
    "alcanforasen": "alcanforar",
    "alcanforases": "alcanforar",
    "alcanforaste": "alcanforar",
    "alcanforasteis": "alcanforar",
    "alcanfore": "alcanforar",
    "alcanforemos": "alcanforar",
    "alcanforen": "alcanforar",
    "alcanforeros": "alcanforero",
    "alcanfores": "alcanforar",
    "alcanforo": "alcanforar",
    "alcanforábamos": "alcanforar",
    "alcanforáis": "alcanforar",
    "alcanforáramos": "alcanforar",
    "alcanforáremos": "alcanforar",
    "alcanforásemos": "alcanforar",
    "alcanforé": "alcanforar",
    "alcanforéis": "alcanforar",
    "alcanforó": "alcanforar",
    "alcanos": "alcano",
    "alcanosulfónica": "alcanosulfónico",
    "alcanosulfónicas": "alcanosulfónico",
    "alcanosulfónicos": "alcanosulfónico",
    "alcantarilla": "alcantarillar",
    "alcantarillaba": "alcantarillar",
    "alcantarillabais": "alcantarillar",
    "alcantarillaban": "alcantarillar",
    "alcantarillabas": "alcantarillar",
    "alcantarillad": "alcantarillar",
    "alcantarillada": "alcantarillar",
    "alcantarilladas": "alcantarillar",
    "alcantarillado": "alcantarillar",
    "alcantarillados": "alcantarillar",
    "alcantarillamos": "alcantarillar",
    "alcantarillan": "alcantarillar",
    "alcantarillando": "alcantarillar",
    "alcantarillara": "alcantarillar",
    "alcantarillarais": "alcantarillar",
    "alcantarillaran": "alcantarillar",
    "alcantarillaras": "alcantarillar",
    "alcantarillare": "alcantarillar",
    "alcantarillareis": "alcantarillar",
    "alcantarillaremos": "alcantarillar",
    "alcantarillaren": "alcantarillar",
    "alcantarillares": "alcantarillar",
    "alcantarillaron": "alcantarillar",
    "alcantarillará": "alcantarillar",
    "alcantarillarán": "alcantarillar",
    "alcantarillarás": "alcantarillar",
    "alcantarillaré": "alcantarillar",
    "alcantarillaréis": "alcantarillar",
    "alcantarillaría": "alcantarillar",
    "alcantarillaríais": "alcantarillar",
    "alcantarillaríamos": "alcantarillar",
    "alcantarillarían": "alcantarillar",
    "alcantarillarías": "alcantarillar",
    "alcantarillas": "alcantarillar",
    "alcantarillase": "alcantarillar",
    "alcantarillaseis": "alcantarillar",
    "alcantarillasen": "alcantarillar",
    "alcantarillases": "alcantarillar",
    "alcantarillaste": "alcantarillar",
    "alcantarillasteis": "alcantarillar",
    "alcantarille": "alcantarillar",
    "alcantarillemos": "alcantarillar",
    "alcantarillen": "alcantarillar",
    "alcantarilles": "alcantarillar",
    "alcantarillo": "alcantarillar",
    "alcantarillábamos": "alcantarillar",
    "alcantarilláis": "alcantarillar",
    "alcantarilláramos": "alcantarillar",
    "alcantarilláremos": "alcantarillar",
    "alcantarillásemos": "alcantarillar",
    "alcantarillé": "alcantarillar",
    "alcantarilléis": "alcantarillar",
    "alcantarilló": "alcantarillar",
    "alcanza": "alcanzar",
    "alcanzaba": "alcanzar",
    "alcanzabais": "alcanzar",
    "alcanzaban": "alcanzar",
    "alcanzabas": "alcanzar",
    "alcanzables": "alcanzable",
    "alcanzad": "alcanzar",
    "alcanzada": "alcanzar",
    "alcanzadas": "alcanzar",
    "alcanzado": "alcanzar",
    "alcanzados": "alcanzar",
    "alcanzamos": "alcanzar",
    "alcanzan": "alcanzar",
    "alcanzando": "alcanzar",
    "alcanzara": "alcanzar",
    "alcanzarais": "alcanzar",
    "alcanzaran": "alcanzar",
    "alcanzaras": "alcanzar",
    "alcanzare": "alcanzar",
    "alcanzareis": "alcanzar",
    "alcanzaremos": "alcanzar",
    "alcanzaren": "alcanzar",
    "alcanzares": "alcanzar",
    "alcanzaron": "alcanzar",
    "alcanzará": "alcanzar",
    "alcanzarán": "alcanzar",
    "alcanzarás": "alcanzar",
    "alcanzaré": "alcanzar",
    "alcanzaréis": "alcanzar",
    "alcanzaría": "alcanzar",
    "alcanzaríais": "alcanzar",
    "alcanzaríamos": "alcanzar",
    "alcanzarían": "alcanzar",
    "alcanzarías": "alcanzar",
    "alcanzas": "alcanzar",
    "alcanzase": "alcanzar",
    "alcanzaseis": "alcanzar",
    "alcanzasen": "alcanzar",
    "alcanzases": "alcanzar",
    "alcanzaste": "alcanzar",
    "alcanzasteis": "alcanzar",
    "alcanzo": "alcanzar",
    "alcanzábamos": "alcanzar",
    "alcanzáis": "alcanzar",
    "alcanzáramos": "alcanzar",
    "alcanzáremos": "alcanzar",
    "alcanzásemos": "alcanzar",
    "alcanzó": "alcanzar",
    "alcaparras": "alcaparra",
    "alcaparreras": "alcaparrera",
    "alcaravanes": "alcaraván",
    "alcaraveas": "alcaravea",
    "alcarchofas": "alcarchofa",
    "alcarciles": "alcarcil",
    "alcarracera": "alcarracero",
    "alcarraceras": "alcarracero",
    "alcarraceros": "alcarracero",
    "alcarracerías": "alcarracería",
    "alcarrazas": "alcarraza",
    "alcarreña": "alcarreño",
    "alcarreñas": "alcarreño",
    "alcarreños": "alcarreño",
    "alcarrias": "alcarria",
    "alcas": "alca",
    "alcatraces": "alcatraz",
    "alcauciles": "alcaucil",
    "alcaucíes": "alcaucí",
    "alcaucís": "alcaucí",
    "alcaudones": "alcaudón",
    "alcayatas": "alcayata",
    "alcazabas": "alcazaba",
    "alce": "alzar",
    "alcemos": "alzar",
    "alcen": "alzar",
    "alces": "alzar",
    "alcionarios": "alcionario",
    "alciones": "alción",
    "alcistas": "alcista",
    "alcmeónidas": "alcmeónida",
    "alcobas": "alcoba",
    "alcohilaciones": "alcohilación",
    "alcohilatos": "alcohilato",
    "alcohilos": "alcohilo",
    "alcohola": "alcoholar",
    "alcoholaba": "alcoholar",
    "alcoholabais": "alcoholar",
    "alcoholaban": "alcoholar",
    "alcoholabas": "alcoholar",
    "alcoholad": "alcoholar",
    "alcoholada": "alcoholar",
    "alcoholadas": "alcoholar",
    "alcoholado": "alcoholar",
    "alcoholados": "alcoholar",
    "alcoholamos": "alcoholar",
    "alcoholan": "alcoholar",
    "alcoholando": "alcoholar",
    "alcoholara": "alcoholar",
    "alcoholarais": "alcoholar",
    "alcoholaran": "alcoholar",
    "alcoholaras": "alcoholar",
    "alcoholare": "alcoholar",
    "alcoholareis": "alcoholar",
    "alcoholaremos": "alcoholar",
    "alcoholaren": "alcoholar",
    "alcoholares": "alcoholar",
    "alcoholaron": "alcoholar",
    "alcoholará": "alcoholar",
    "alcoholarán": "alcoholar",
    "alcoholarás": "alcoholar",
    "alcoholaré": "alcoholar",
    "alcoholaréis": "alcoholar",
    "alcoholaría": "alcoholar",
    "alcoholaríais": "alcoholar",
    "alcoholaríamos": "alcoholar",
    "alcoholarían": "alcoholar",
    "alcoholarías": "alcoholar",
    "alcoholas": "alcoholar",
    "alcoholase": "alcoholar",
    "alcoholaseis": "alcoholar",
    "alcoholasen": "alcoholar",
    "alcoholases": "alcoholar",
    "alcoholaste": "alcoholar",
    "alcoholasteis": "alcoholar",
    "alcoholatos": "alcoholato",
    "alcoholaturas": "alcoholatura",
    "alcoholaturos": "alcoholaturo",
    "alcohole": "alcoholar",
    "alcoholemias": "alcoholemia",
    "alcoholemos": "alcoholar",
    "alcoholen": "alcoholar",
    "alcoholes": "alcoholar",
    "alcoholice": "alcoholizar",
    "alcoholicemos": "alcoholizar",
    "alcoholicen": "alcoholizar",
    "alcoholices": "alcoholizar",
    "alcoholicé": "alcoholizar",
    "alcoholicéis": "alcoholizar",
    "alcoholimetrías": "alcoholimetría",
    "alcoholismos": "alcoholismo",
    "alcoholiza": "alcoholizar",
    "alcoholizaba": "alcoholizar",
    "alcoholizabais": "alcoholizar",
    "alcoholizaban": "alcoholizar",
    "alcoholizabas": "alcoholizar",
    "alcoholizaciones": "alcoholización",
    "alcoholizad": "alcoholizar",
    "alcoholizada": "alcoholizar",
    "alcoholizadas": "alcoholizar",
    "alcoholizado": "alcoholizar",
    "alcoholizados": "alcoholizar",
    "alcoholizamos": "alcoholizar",
    "alcoholizan": "alcoholizar",
    "alcoholizando": "alcoholizar",
    "alcoholizara": "alcoholizar",
    "alcoholizarais": "alcoholizar",
    "alcoholizaran": "alcoholizar",
    "alcoholizaras": "alcoholizar",
    "alcoholizare": "alcoholizar",
    "alcoholizareis": "alcoholizar",
    "alcoholizaremos": "alcoholizar",
    "alcoholizaren": "alcoholizar",
    "alcoholizares": "alcoholizar",
    "alcoholizaron": "alcoholizar",
    "alcoholizará": "alcoholizar",
    "alcoholizarán": "alcoholizar",
    "alcoholizarás": "alcoholizar",
    "alcoholizaré": "alcoholizar",
    "alcoholizaréis": "alcoholizar",
    "alcoholizaría": "alcoholizar",
    "alcoholizaríais": "alcoholizar",
    "alcoholizaríamos": "alcoholizar",
    "alcoholizarían": "alcoholizar",
    "alcoholizarías": "alcoholizar",
    "alcoholizas": "alcoholizar",
    "alcoholizase": "alcoholizar",
    "alcoholizaseis": "alcoholizar",
    "alcoholizasen": "alcoholizar",
    "alcoholizases": "alcoholizar",
    "alcoholizaste": "alcoholizar",
    "alcoholizasteis": "alcoholizar",
    "alcoholizo": "alcoholizar",
    "alcoholizábamos": "alcoholizar",
    "alcoholizáis": "alcoholizar",
    "alcoholizáramos": "alcoholizar",
    "alcoholizáremos": "alcoholizar",
    "alcoholizásemos": "alcoholizar",
    "alcoholizó": "alcoholizar",
    "alcoholo": "alcoholar",
    "alcoholábamos": "alcoholar",
    "alcoholáis": "alcoholar",
    "alcoholáramos": "alcoholar",
    "alcoholáremos": "alcoholar",
    "alcoholásemos": "alcoholar",
    "alcoholé": "alcoholar",
    "alcoholéis": "alcoholar",
    "alcoholímetros": "alcoholímetro",
    "alcoholó": "alcoholar",
    "alcoholómetros": "alcoholómetro",
    "alcohílos": "alcohílo",
    "alcohólica": "alcohólico",
    "alcohólicas": "alcohólico",
    "alcohólicos": "alcohólico",
    "alcohómetros": "alcohómetro",
    "alcorce": "alcorzar",
    "alcorcemos": "alcorzar",
    "alcorcen": "alcorzar",
    "alcorces": "alcorzar",
    "alcorcé": "alcorzar",
    "alcorcéis": "alcorzar",
    "alcores": "alcor",
    "alcornocales": "alcornocal",
    "alcornoques": "alcornoque",
    "alcornoqueña": "alcornoqueño",
    "alcornoqueñas": "alcornoqueño",
    "alcornoqueños": "alcornoqueño",
    "alcorques": "alcorque",
    "alcorza": "alcorzar",
    "alcorzaba": "alcorzar",
    "alcorzabais": "alcorzar",
    "alcorzaban": "alcorzar",
    "alcorzabas": "alcorzar",
    "alcorzad": "alcorzar",
    "alcorzada": "alcorzar",
    "alcorzadas": "alcorzar",
    "alcorzado": "alcorzar",
    "alcorzados": "alcorzar",
    "alcorzamos": "alcorzar",
    "alcorzan": "alcorzar",
    "alcorzando": "alcorzar",
    "alcorzara": "alcorzar",
    "alcorzarais": "alcorzar",
    "alcorzaran": "alcorzar",
    "alcorzaras": "alcorzar",
    "alcorzare": "alcorzar",
    "alcorzareis": "alcorzar",
    "alcorzaremos": "alcorzar",
    "alcorzaren": "alcorzar",
    "alcorzares": "alcorzar",
    "alcorzaron": "alcorzar",
    "alcorzará": "alcorzar",
    "alcorzarán": "alcorzar",
    "alcorzarás": "alcorzar",
    "alcorzaré": "alcorzar",
    "alcorzaréis": "alcorzar",
    "alcorzaría": "alcorzar",
    "alcorzaríais": "alcorzar",
    "alcorzaríamos": "alcorzar",
    "alcorzarían": "alcorzar",
    "alcorzarías": "alcorzar",
    "alcorzas": "alcorzar",
    "alcorzase": "alcorzar",
    "alcorzaseis": "alcorzar",
    "alcorzasen": "alcorzar",
    "alcorzases": "alcorzar",
    "alcorzaste": "alcorzar",
    "alcorzasteis": "alcorzar",
    "alcorzo": "alcorzar",
    "alcorzábamos": "alcorzar",
    "alcorzáis": "alcorzar",
    "alcorzáramos": "alcorzar",
    "alcorzáremos": "alcorzar",
    "alcorzásemos": "alcorzar",
    "alcorzó": "alcorzar",
    "alcosoles": "alcosol",
    "alcotanas": "alcotana",
    "alcotanes": "alcotán",
    "alcoxilos": "alcoxilo",
    "alcubillas": "alcubilla",
    "alcurnias": "alcurnia",
    "alcuzadas": "alcuzada",
    "alcuzas": "alcuza",
    "alcuzcuces": "alcuzcuz",
    "alcándaras": "alcándara",
    "alcázares": "alcázar",
    "alcé": "alzar",
    "alcéis": "alzar",
    "alcídica": "alcídico",
    "alcídicas": "alcídico",
    "alcídicos": "alcídico",
    "aldabas": "aldaba",
    "aldabazos": "aldabazo",
    "aldabea": "aldabear",
    "aldabeaba": "aldabear",
    "aldabeabais": "aldabear",
    "aldabeaban": "aldabear",
    "aldabeabas": "aldabear",
    "aldabead": "aldabear",
    "aldabeada": "aldabear",
    "aldabeadas": "aldabear",
    "aldabeado": "aldabear",
    "aldabeados": "aldabear",
    "aldabeamos": "aldabear",
    "aldabean": "aldabear",
    "aldabeando": "aldabear",
    "aldabeara": "aldabear",
    "aldabearais": "aldabear",
    "aldabearan": "aldabear",
    "aldabearas": "aldabear",
    "aldabeare": "aldabear",
    "aldabeareis": "aldabear",
    "aldabearemos": "aldabear",
    "aldabearen": "aldabear",
    "aldabeares": "aldabear",
    "aldabearon": "aldabear",
    "aldabeará": "aldabear",
    "aldabearán": "aldabear",
    "aldabearás": "aldabear",
    "aldabearé": "aldabear",
    "aldabearéis": "aldabear",
    "aldabearía": "aldabear",
    "aldabearíais": "aldabear",
    "aldabearíamos": "aldabear",
    "aldabearían": "aldabear",
    "aldabearías": "aldabear",
    "aldabeas": "aldabear",
    "aldabease": "aldabear",
    "aldabeaseis": "aldabear",
    "aldabeasen": "aldabear",
    "aldabeases": "aldabear",
    "aldabeaste": "aldabear",
    "aldabeasteis": "aldabear",
    "aldabee": "aldabear",
    "aldabeemos": "aldabear",
    "aldabeen": "aldabear",
    "aldabees": "aldabear",
    "aldabeo": "aldabear",
    "aldabeos": "aldabeo",
    "aldabeábamos": "aldabear",
    "aldabeáis": "aldabear",
    "aldabeáramos": "aldabear",
    "aldabeáremos": "aldabear",
    "aldabeásemos": "aldabear",
    "aldabeé": "aldabear",
    "aldabeéis": "aldabear",
    "aldabeó": "aldabear",
    "aldabillas": "aldabilla",
    "aldabones": "aldabón",
    "aldeana": "aldeano",
    "aldeanas": "aldeano",
    "aldeanismos": "aldeanismo",
    "aldeanos": "aldeano",
    "aldeas": "aldea",
    "aldehuelas": "aldehuela",
    "aldehídos": "aldehído",
    "alderredor": "alrededor",
    "aldiminas": "aldimina",
    "aldina": "aldino",
    "aldinas": "aldino",
    "aldinos": "aldino",
    "aldoles": "aldol",
    "aldolizaciones": "aldolización",
    "aldopentosas": "aldopentosa",
    "aldosas": "aldosa",
    "aldosteronas": "aldosterona",
    "aldoximas": "aldoxima",
    "aldrinas": "aldrina",
    "alea": "alear",
    "aleaba": "alear",
    "aleabais": "alear",
    "aleaban": "alear",
    "aleabas": "alear",
    "aleaciones": "aleación",
    "alead": "alear",
    "aleada": "alear",
    "aleadas": "alear",
    "aleado": "alear",
    "aleados": "alear",
    "aleamos": "alear",
    "alean": "alear",
    "aleando": "alear",
    "aleara": "alear",
    "alearais": "alear",
    "alearan": "alear",
    "alearas": "alear",
    "aleare": "alear",
    "aleareis": "alear",
    "alearemos": "alear",
    "alearen": "alear",
    "aleares": "alear",
    "alearon": "alear",
    "aleará": "alear",
    "alearán": "alear",
    "alearás": "alear",
    "alearé": "alear",
    "alearéis": "alear",
    "alearía": "alear",
    "alearíais": "alear",
    "alearíamos": "alear",
    "alearían": "alear",
    "alearías": "alear",
    "aleas": "alear",
    "alease": "alear",
    "aleaseis": "alear",
    "aleasen": "alear",
    "aleases": "alear",
    "aleaste": "alear",
    "aleasteis": "alear",
    "aleatoria": "aleatorio",
    "aleatorias": "aleatorio",
    "aleatoriedades": "aleatoriedad",
    "aleatorios": "aleatorio",
    "aleatorizaciones": "aleatorización",
    "alebraba": "alebrar",
    "alebrabais": "alebrar",
    "alebraban": "alebrar",
    "alebrabas": "alebrar",
    "alebrad": "alebrar",
    "alebrada": "alebrar",
    "alebradas": "alebrar",
    "alebrado": "alebrar",
    "alebrados": "alebrar",
    "alebramos": "alebrar",
    "alebrando": "alebrar",
    "alebrara": "alebrar",
    "alebrarais": "alebrar",
    "alebraran": "alebrar",
    "alebraras": "alebrar",
    "alebrare": "alebrar",
    "alebrareis": "alebrar",
    "alebraremos": "alebrar",
    "alebraren": "alebrar",
    "alebrares": "alebrar",
    "alebraron": "alebrar",
    "alebrará": "alebrar",
    "alebrarán": "alebrar",
    "alebrarás": "alebrar",
    "alebraré": "alebrar",
    "alebraréis": "alebrar",
    "alebraría": "alebrar",
    "alebraríais": "alebrar",
    "alebraríamos": "alebrar",
    "alebrarían": "alebrar",
    "alebrarías": "alebrar",
    "alebrase": "alebrar",
    "alebraseis": "alebrar",
    "alebrasen": "alebrar",
    "alebrases": "alebrar",
    "alebrasta": "alebrastar",
    "alebrastaba": "alebrastar",
    "alebrastabais": "alebrastar",
    "alebrastaban": "alebrastar",
    "alebrastabas": "alebrastar",
    "alebrastad": "alebrastar",
    "alebrastada": "alebrastar",
    "alebrastadas": "alebrastar",
    "alebrastado": "alebrastar",
    "alebrastados": "alebrastar",
    "alebrastamos": "alebrastar",
    "alebrastan": "alebrastar",
    "alebrastando": "alebrastar",
    "alebrastara": "alebrastar",
    "alebrastarais": "alebrastar",
    "alebrastaran": "alebrastar",
    "alebrastaras": "alebrastar",
    "alebrastare": "alebrastar",
    "alebrastareis": "alebrastar",
    "alebrastaremos": "alebrastar",
    "alebrastaren": "alebrastar",
    "alebrastares": "alebrastar",
    "alebrastaron": "alebrastar",
    "alebrastará": "alebrastar",
    "alebrastarán": "alebrastar",
    "alebrastarás": "alebrastar",
    "alebrastaré": "alebrastar",
    "alebrastaréis": "alebrastar",
    "alebrastaría": "alebrastar",
    "alebrastaríais": "alebrastar",
    "alebrastaríamos": "alebrastar",
    "alebrastarían": "alebrastar",
    "alebrastarías": "alebrastar",
    "alebrastas": "alebrastar",
    "alebrastase": "alebrastar",
    "alebrastaseis": "alebrastar",
    "alebrastasen": "alebrastar",
    "alebrastases": "alebrastar",
    "alebrastaste": "alebrastar",
    "alebrastasteis": "alebrastar",
    "alebraste": "alebrastar",
    "alebrasteis": "alebrar",
    "alebrastemos": "alebrastar",
    "alebrasten": "alebrastar",
    "alebrastes": "alebrastar",
    "alebrasto": "alebrastar",
    "alebrastábamos": "alebrastar",
    "alebrastáis": "alebrastar",
    "alebrastáramos": "alebrastar",
    "alebrastáremos": "alebrastar",
    "alebrastásemos": "alebrastar",
    "alebrasté": "alebrastar",
    "alebrastéis": "alebrastar",
    "alebrastó": "alebrastar",
    "alebremos": "alebrar",
    "alebresta": "alebrestar",
    "alebrestaba": "alebrestar",
    "alebrestabais": "alebrestar",
    "alebrestaban": "alebrestar",
    "alebrestabas": "alebrestar",
    "alebrestad": "alebrestar",
    "alebrestada": "alebrestar",
    "alebrestadas": "alebrestar",
    "alebrestado": "alebrestar",
    "alebrestados": "alebrestar",
    "alebrestamos": "alebrestar",
    "alebrestan": "alebrestar",
    "alebrestando": "alebrestar",
    "alebrestara": "alebrestar",
    "alebrestarais": "alebrestar",
    "alebrestaran": "alebrestar",
    "alebrestaras": "alebrestar",
    "alebrestare": "alebrestar",
    "alebrestareis": "alebrestar",
    "alebrestaremos": "alebrestar",
    "alebrestaren": "alebrestar",
    "alebrestares": "alebrestar",
    "alebrestaron": "alebrestar",
    "alebrestará": "alebrestar",
    "alebrestarán": "alebrestar",
    "alebrestarás": "alebrestar",
    "alebrestaré": "alebrestar",
    "alebrestaréis": "alebrestar",
    "alebrestaría": "alebrestar",
    "alebrestaríais": "alebrestar",
    "alebrestaríamos": "alebrestar",
    "alebrestarían": "alebrestar",
    "alebrestarías": "alebrestar",
    "alebrestas": "alebrestar",
    "alebrestase": "alebrestar",
    "alebrestaseis": "alebrestar",
    "alebrestasen": "alebrestar",
    "alebrestases": "alebrestar",
    "alebrestaste": "alebrestar",
    "alebrestasteis": "alebrestar",
    "alebreste": "alebrestar",
    "alebrestemos": "alebrestar",
    "alebresten": "alebrestar",
    "alebrestes": "alebrestar",
    "alebresto": "alebrestar",
    "alebrestábamos": "alebrestar",
    "alebrestáis": "alebrestar",
    "alebrestáramos": "alebrestar",
    "alebrestáremos": "alebrestar",
    "alebrestásemos": "alebrestar",
    "alebresté": "alebrestar",
    "alebrestéis": "alebrestar",
    "alebrestó": "alebrestar",
    "alebrona": "alebronar",
    "alebronaba": "alebronar",
    "alebronabais": "alebronar",
    "alebronaban": "alebronar",
    "alebronabas": "alebronar",
    "alebronad": "alebronar",
    "alebronada": "alebronar",
    "alebronadas": "alebronar",
    "alebronado": "alebronar",
    "alebronados": "alebronar",
    "alebronamos": "alebronar",
    "alebronan": "alebronar",
    "alebronando": "alebronar",
    "alebronara": "alebronar",
    "alebronarais": "alebronar",
    "alebronaran": "alebronar",
    "alebronaras": "alebronar",
    "alebronare": "alebronar",
    "alebronareis": "alebronar",
    "alebronaremos": "alebronar",
    "alebronaren": "alebronar",
    "alebronares": "alebronar",
    "alebronaron": "alebronar",
    "alebronará": "alebronar",
    "alebronarán": "alebronar",
    "alebronarás": "alebronar",
    "alebronaré": "alebronar",
    "alebronaréis": "alebronar",
    "alebronaría": "alebronar",
    "alebronaríais": "alebronar",
    "alebronaríamos": "alebronar",
    "alebronarían": "alebronar",
    "alebronarías": "alebronar",
    "alebronas": "alebronar",
    "alebronase": "alebronar",
    "alebronaseis": "alebronar",
    "alebronasen": "alebronar",
    "alebronases": "alebronar",
    "alebronaste": "alebronar",
    "alebronasteis": "alebronar",
    "alebrone": "alebronar",
    "alebronemos": "alebronar",
    "alebronen": "alebronar",
    "alebrones": "alebronar",
    "alebrono": "alebronar",
    "alebronábamos": "alebronar",
    "alebronáis": "alebronar",
    "alebronáramos": "alebronar",
    "alebronáremos": "alebronar",
    "alebronásemos": "alebronar",
    "alebroné": "alebronar",
    "alebronéis": "alebronar",
    "alebronó": "alebronar",
    "alebrábamos": "alebrar",
    "alebráis": "alebrar",
    "alebráramos": "alebrar",
    "alebráremos": "alebrar",
    "alebrásemos": "alebrar",
    "alebré": "alebrar",
    "alebréis": "alebrar",
    "alebró": "alebrar",
    "alecciona": "aleccionar",
    "aleccionaba": "aleccionar",
    "aleccionabais": "aleccionar",
    "aleccionaban": "aleccionar",
    "aleccionabas": "aleccionar",
    "aleccionad": "aleccionar",
    "aleccionada": "aleccionar",
    "aleccionadas": "aleccionar",
    "aleccionado": "aleccionar",
    "aleccionadora": "aleccionador",
    "aleccionadoras": "aleccionador",
    "aleccionadores": "aleccionador",
    "aleccionados": "aleccionar",
    "aleccionamientos": "aleccionamiento",
    "aleccionamos": "aleccionar",
    "aleccionan": "aleccionar",
    "aleccionando": "aleccionar",
    "aleccionara": "aleccionar",
    "aleccionarais": "aleccionar",
    "aleccionaran": "aleccionar",
    "aleccionaras": "aleccionar",
    "aleccionare": "aleccionar",
    "aleccionareis": "aleccionar",
    "aleccionaremos": "aleccionar",
    "aleccionaren": "aleccionar",
    "aleccionares": "aleccionar",
    "aleccionaron": "aleccionar",
    "aleccionará": "aleccionar",
    "aleccionarán": "aleccionar",
    "aleccionarás": "aleccionar",
    "aleccionaré": "aleccionar",
    "aleccionaréis": "aleccionar",
    "aleccionaría": "aleccionar",
    "aleccionaríais": "aleccionar",
    "aleccionaríamos": "aleccionar",
    "aleccionarían": "aleccionar",
    "aleccionarías": "aleccionar",
    "aleccionas": "aleccionar",
    "aleccionase": "aleccionar",
    "aleccionaseis": "aleccionar",
    "aleccionasen": "aleccionar",
    "aleccionases": "aleccionar",
    "aleccionaste": "aleccionar",
    "aleccionasteis": "aleccionar",
    "aleccione": "aleccionar",
    "aleccionemos": "aleccionar",
    "aleccionen": "aleccionar",
    "alecciones": "aleccionar",
    "alecciono": "aleccionar",
    "aleccionábamos": "aleccionar",
    "aleccionáis": "aleccionar",
    "aleccionáramos": "aleccionar",
    "aleccionáremos": "aleccionar",
    "aleccionásemos": "aleccionar",
    "aleccioné": "aleccionar",
    "aleccionéis": "aleccionar",
    "aleccionó": "aleccionar",
    "alechuga": "alechugar",
    "alechugaba": "alechugar",
    "alechugabais": "alechugar",
    "alechugaban": "alechugar",
    "alechugabas": "alechugar",
    "alechugad": "alechugar",
    "alechugada": "alechugar",
    "alechugadas": "alechugar",
    "alechugado": "alechugar",
    "alechugados": "alechugar",
    "alechugamos": "alechugar",
    "alechugan": "alechugar",
    "alechugando": "alechugar",
    "alechugara": "alechugar",
    "alechugarais": "alechugar",
    "alechugaran": "alechugar",
    "alechugaras": "alechugar",
    "alechugare": "alechugar",
    "alechugareis": "alechugar",
    "alechugaremos": "alechugar",
    "alechugaren": "alechugar",
    "alechugares": "alechugar",
    "alechugaron": "alechugar",
    "alechugará": "alechugar",
    "alechugarán": "alechugar",
    "alechugarás": "alechugar",
    "alechugaré": "alechugar",
    "alechugaréis": "alechugar",
    "alechugaría": "alechugar",
    "alechugaríais": "alechugar",
    "alechugaríamos": "alechugar",
    "alechugarían": "alechugar",
    "alechugarías": "alechugar",
    "alechugas": "alechugar",
    "alechugase": "alechugar",
    "alechugaseis": "alechugar",
    "alechugasen": "alechugar",
    "alechugases": "alechugar",
    "alechugaste": "alechugar",
    "alechugasteis": "alechugar",
    "alechugo": "alechugar",
    "alechugue": "alechugar",
    "alechuguemos": "alechugar",
    "alechuguen": "alechugar",
    "alechugues": "alechugar",
    "alechugué": "alechugar",
    "alechuguéis": "alechugar",
    "alechugábamos": "alechugar",
    "alechugáis": "alechugar",
    "alechugáramos": "alechugar",
    "alechugáremos": "alechugar",
    "alechugásemos": "alechugar",
    "alechugó": "alechugar",
    "alecrines": "alecrín",
    "aledaña": "aledaño",
    "aledañas": "aledaño",
    "aledaños": "aledaño",
    "alee": "alear",
    "aleemos": "alear",
    "aleen": "alear",
    "alees": "alear",
    "alefatos": "alefato",
    "alefrices": "alefriz",
    "alega": "alegar",
    "alegaba": "alegar",
    "alegabais": "alegar",
    "alegaban": "alegar",
    "alegabas": "alegar",
    "alegaciones": "alegación",
    "alegad": "alegar",
    "alegada": "alegar",
    "alegadas": "alegar",
    "alegado": "alegar",
    "alegadora": "alegador",
    "alegadoras": "alegador",
    "alegadores": "alegador",
    "alegados": "alegar",
    "alegamos": "alegar",
    "alegan": "alegar",
    "alegando": "alegar",
    "alegara": "alegar",
    "alegarais": "alegar",
    "alegaran": "alegar",
    "alegaras": "alegar",
    "alegare": "alegar",
    "alegareis": "alegar",
    "alegaremos": "alegar",
    "alegaren": "alegar",
    "alegares": "alegar",
    "alegaron": "alegar",
    "alegará": "alegar",
    "alegarán": "alegar",
    "alegarás": "alegar",
    "alegaré": "alegar",
    "alegaréis": "alegar",
    "alegaría": "alegar",
    "alegaríais": "alegar",
    "alegaríamos": "alegar",
    "alegarían": "alegar",
    "alegarías": "alegar",
    "alegas": "alegar",
    "alegase": "alegar",
    "alegaseis": "alegar",
    "alegasen": "alegar",
    "alegases": "alegar",
    "alegaste": "alegar",
    "alegasteis": "alegar",
    "alegatos": "alegato",
    "alego": "alegar",
    "alegorice": "alegorizar",
    "alegoricemos": "alegorizar",
    "alegoricen": "alegorizar",
    "alegorices": "alegorizar",
    "alegoricé": "alegorizar",
    "alegoricéis": "alegorizar",
    "alegorismos": "alegorismo",
    "alegoristas": "alegorista",
    "alegoriza": "alegorizar",
    "alegorizaba": "alegorizar",
    "alegorizabais": "alegorizar",
    "alegorizaban": "alegorizar",
    "alegorizabas": "alegorizar",
    "alegorizad": "alegorizar",
    "alegorizada": "alegorizar",
    "alegorizadas": "alegorizar",
    "alegorizado": "alegorizar",
    "alegorizados": "alegorizar",
    "alegorizamos": "alegorizar",
    "alegorizan": "alegorizar",
    "alegorizando": "alegorizar",
    "alegorizara": "alegorizar",
    "alegorizarais": "alegorizar",
    "alegorizaran": "alegorizar",
    "alegorizaras": "alegorizar",
    "alegorizare": "alegorizar",
    "alegorizareis": "alegorizar",
    "alegorizaremos": "alegorizar",
    "alegorizaren": "alegorizar",
    "alegorizares": "alegorizar",
    "alegorizaron": "alegorizar",
    "alegorizará": "alegorizar",
    "alegorizarán": "alegorizar",
    "alegorizarás": "alegorizar",
    "alegorizaré": "alegorizar",
    "alegorizaréis": "alegorizar",
    "alegorizaría": "alegorizar",
    "alegorizaríais": "alegorizar",
    "alegorizaríamos": "alegorizar",
    "alegorizarían": "alegorizar",
    "alegorizarías": "alegorizar",
    "alegorizas": "alegorizar",
    "alegorizase": "alegorizar",
    "alegorizaseis": "alegorizar",
    "alegorizasen": "alegorizar",
    "alegorizases": "alegorizar",
    "alegorizaste": "alegorizar",
    "alegorizasteis": "alegorizar",
    "alegorizo": "alegorizar",
    "alegorizábamos": "alegorizar",
    "alegorizáis": "alegorizar",
    "alegorizáramos": "alegorizar",
    "alegorizáremos": "alegorizar",
    "alegorizásemos": "alegorizar",
    "alegorizó": "alegorizar",
    "alegorías": "alegoría",
    "alegra": "alegrar",
    "alegraba": "alegrar",
    "alegrabais": "alegrar",
    "alegraban": "alegrar",
    "alegrabas": "alegrar",
    "alegrad": "alegrar",
    "alegrada": "alegrar",
    "alegradas": "alegrar",
    "alegrado": "alegrar",
    "alegrados": "alegrar",
    "alegramos": "alegrar",
    "alegran": "alegrar",
    "alegrando": "alegrar",
    "alegrara": "alegrar",
    "alegrarais": "alegrar",
    "alegraran": "alegrar",
    "alegraras": "alegrar",
    "alegrare": "alegrar",
    "alegrareis": "alegrar",
    "alegraremos": "alegrar",
    "alegraren": "alegrar",
    "alegrares": "alegrar",
    "alegraron": "alegrar",
    "alegrará": "alegrar",
    "alegrarán": "alegrar",
    "alegrarás": "alegrar",
    "alegraré": "alegrar",
    "alegraréis": "alegrar",
    "alegraría": "alegrar",
    "alegraríais": "alegrar",
    "alegraríamos": "alegrar",
    "alegrarían": "alegrar",
    "alegrarías": "alegrar",
    "alegras": "alegrar",
    "alegrase": "alegrar",
    "alegraseis": "alegrar",
    "alegrasen": "alegrar",
    "alegrases": "alegrar",
    "alegraste": "alegrar",
    "alegrasteis": "alegrar",
    "alegre": "alegrar",
    "alegremos": "alegrar",
    "alegren": "alegrar",
    "alegres": "alegre",
    "alegretos": "alegreto",
    "alegro": "alegrar",
    "alegrones": "alegrón",
    "alegros": "alegro",
    "alegrábamos": "alegrar",
    "alegráis": "alegrar",
    "alegráramos": "alegrar",
    "alegráremos": "alegrar",
    "alegrásemos": "alegrar",
    "alegré": "alegrar",
    "alegréis": "alegrar",
    "alegrías": "alegría",
    "alegró": "alegrar",
    "alegue": "alegar",
    "aleguemos": "alegar",
    "aleguen": "alegar",
    "alegues": "alegar",
    "alegué": "alegar",
    "aleguéis": "alegar",
    "alegábamos": "alegar",
    "alegáis": "alegar",
    "alegáramos": "alegar",
    "alegáremos": "alegar",
    "alegásemos": "alegar",
    "alegó": "alegar",
    "alegórica": "alegórico",
    "alegóricas": "alegórico",
    "alegóricos": "alegórico",
    "aleja": "alejar",
    "alejaba": "alejar",
    "alejabais": "alejar",
    "alejaban": "alejar",
    "alejabas": "alejar",
    "alejad": "alejar",
    "alejada": "alejar",
    "alejadas": "alejar",
    "alejado": "alejar",
    "alejados": "alejar",
    "alejamientos": "alejamiento",
    "alejamos": "alejar",
    "alejan": "alejar",
    "alejando": "alejar",
    "alejandrina": "alejandrino",
    "alejandrinas": "alejandrino",
    "alejandrinos": "alejandrino",
    "alejandritas": "alejandrita",
    "alejara": "alejar",
    "alejarais": "alejar",
    "alejaran": "alejar",
    "alejaras": "alejar",
    "alejare": "alejar",
    "alejareis": "alejar",
    "alejaremos": "alejar",
    "alejaren": "alejar",
    "alejares": "alejar",
    "alejaron": "alejar",
    "alejará": "alejar",
    "alejarán": "alejar",
    "alejarás": "alejar",
    "alejaré": "alejar",
    "alejaréis": "alejar",
    "alejaría": "alejar",
    "alejaríais": "alejar",
    "alejaríamos": "alejar",
    "alejarían": "alejar",
    "alejarías": "alejar",
    "alejas": "alejar",
    "alejase": "alejar",
    "alejaseis": "alejar",
    "alejasen": "alejar",
    "alejases": "alejar",
    "alejaste": "alejar",
    "alejasteis": "alejar",
    "aleje": "alejar",
    "alejemos": "alejar",
    "alejen": "alejar",
    "alejes": "alejar",
    "alejo": "alejar",
    "alejábamos": "alejar",
    "alejáis": "alejar",
    "alejáramos": "alejar",
    "alejáremos": "alejar",
    "alejásemos": "alejar",
    "alejé": "alejar",
    "alejéis": "alejar",
    "alejó": "alejar",
    "alela": "alelar",
    "alelaba": "alelar",
    "alelabais": "alelar",
    "alelaban": "alelar",
    "alelabas": "alelar",
    "alelad": "alelar",
    "alelada": "alelar",
    "aleladas": "alelar",
    "alelado": "alelar",
    "alelados": "alelar",
    "alelamientos": "alelamiento",
    "alelamos": "alelar",
    "alelan": "alelar",
    "alelando": "alelar",
    "alelara": "alelar",
    "alelarais": "alelar",
    "alelaran": "alelar",
    "alelaras": "alelar",
    "alelare": "alelar",
    "alelareis": "alelar",
    "alelaremos": "alelar",
    "alelaren": "alelar",
    "alelares": "alelar",
    "alelaron": "alelar",
    "alelará": "alelar",
    "alelarán": "alelar",
    "alelarás": "alelar",
    "alelaré": "alelar",
    "alelaréis": "alelar",
    "alelaría": "alelar",
    "alelaríais": "alelar",
    "alelaríamos": "alelar",
    "alelarían": "alelar",
    "alelarías": "alelar",
    "alelas": "alelar",
    "alelase": "alelar",
    "alelaseis": "alelar",
    "alelasen": "alelar",
    "alelases": "alelar",
    "alelaste": "alelar",
    "alelasteis": "alelar",
    "alele": "alelar",
    "alelemos": "alelar",
    "alelen": "alelar",
    "aleles": "alelar",
    "alelismos": "alelismo",
    "alelo": "alelar",
    "alelomorfismos": "alelomorfismo",
    "alelomorfos": "alelomorfo",
    "alelos": "alelo",
    "alelotipos": "alelotipo",
    "aleluyas": "aleluya",
    "alelábamos": "alelar",
    "aleláis": "alelar",
    "aleláramos": "alelar",
    "aleláremos": "alelar",
    "alelásemos": "alelar",
    "alelé": "alelar",
    "aleléis": "alelar",
    "aleló": "alelar",
    "alemana": "alemán",
    "alemanas": "alemán",
    "alemandas": "alemanda",
    "alemanes": "alemán",
    "alemána": "alemán",
    "alemánas": "alemán",
    "alemánes": "alemán",
    "alenos": "aleno",
    "alentaba": "alentar",
    "alentabais": "alentar",
    "alentaban": "alentar",
    "alentabas": "alentar",
    "alentad": "alentar",
    "alentada": "alentar",
    "alentadas": "alentar",
    "alentado": "alentar",
    "alentadora": "alentador",
    "alentadoras": "alentador",
    "alentadores": "alentador",
    "alentados": "alentar",
    "alentamos": "alentar",
    "alentando": "alentar",
    "alentara": "alentar",
    "alentarais": "alentar",
    "alentaran": "alentar",
    "alentaras": "alentar",
    "alentare": "alentar",
    "alentareis": "alentar",
    "alentaremos": "alentar",
    "alentaren": "alentar",
    "alentares": "alentar",
    "alentaron": "alentar",
    "alentará": "alentar",
    "alentarán": "alentar",
    "alentarás": "alentar",
    "alentaré": "alentar",
    "alentaréis": "alentar",
    "alentaría": "alentar",
    "alentaríais": "alentar",
    "alentaríamos": "alentar",
    "alentarían": "alentar",
    "alentarías": "alentar",
    "alentase": "alentar",
    "alentaseis": "alentar",
    "alentasen": "alentar",
    "alentases": "alentar",
    "alentaste": "alentar",
    "alentasteis": "alentar",
    "alentemos": "alentar",
    "alentábamos": "alentar",
    "alentáis": "alentar",
    "alentáramos": "alentar",
    "alentáremos": "alentar",
    "alentásemos": "alentar",
    "alenté": "alentar",
    "alentéis": "alentar",
    "alentó": "alentar",
    "aleo": "alear",
    "alerces": "alerce",
    "alergenos": "alergeno",
    "alergias": "alergia",
    "alergizaciones": "alergización",
    "alergologías": "alergología",
    "alergénica": "alergénico",
    "alergénicas": "alergénico",
    "alergénicos": "alergénico",
    "alergóloga": "alergólogo",
    "alergólogas": "alergólogo",
    "alergólogos": "alergólogo",
    "alerones": "alerón",
    "aleros": "alero",
    "aleros-pívot": "alero-pívot",
    "alerta": "alertar",
    "alertaba": "alertar",
    "alertabais": "alertar",
    "alertaban": "alertar",
    "alertabas": "alertar",
    "alertad": "alertar",
    "alertada": "alertar",
    "alertadas": "alertar",
    "alertado": "alertar",
    "alertados": "alertar",
    "alertamos": "alertar",
    "alertan": "alertar",
    "alertando": "alertar",
    "alertara": "alertar",
    "alertarais": "alertar",
    "alertaran": "alertar",
    "alertaras": "alertar",
    "alertare": "alertar",
    "alertareis": "alertar",
    "alertaremos": "alertar",
    "alertaren": "alertar",
    "alertares": "alertar",
    "alertaron": "alertar",
    "alertará": "alertar",
    "alertarán": "alertar",
    "alertarás": "alertar",
    "alertaré": "alertar",
    "alertaréis": "alertar",
    "alertaría": "alertar",
    "alertaríais": "alertar",
    "alertaríamos": "alertar",
    "alertarían": "alertar",
    "alertarías": "alertar",
    "alertas": "alertar",
    "alertase": "alertar",
    "alertaseis": "alertar",
    "alertasen": "alertar",
    "alertases": "alertar",
    "alertaste": "alertar",
    "alertasteis": "alertar",
    "alerte": "alertar",
    "alertemos": "alertar",
    "alerten": "alertar",
    "alertes": "alertar",
    "alerto": "alertar",
    "alertábamos": "alertar",
    "alertáis": "alertar",
    "alertáramos": "alertar",
    "alertáremos": "alertar",
    "alertásemos": "alertar",
    "alerté": "alertar",
    "alertéis": "alertar",
    "alertó": "alertar",
    "ales": "ale",
    "aletadas": "aletada",
    "aletarga": "aletargar",
    "aletargaba": "aletargar",
    "aletargabais": "aletargar",
    "aletargaban": "aletargar",
    "aletargabas": "aletargar",
    "aletargad": "aletargar",
    "aletargada": "aletargar",
    "aletargadas": "aletargar",
    "aletargado": "aletargar",
    "aletargados": "aletargar",
    "aletargamientos": "aletargamiento",
    "aletargamos": "aletargar",
    "aletargan": "aletargar",
    "aletargando": "aletargar",
    "aletargara": "aletargar",
    "aletargarais": "aletargar",
    "aletargaran": "aletargar",
    "aletargaras": "aletargar",
    "aletargare": "aletargar",
    "aletargareis": "aletargar",
    "aletargaremos": "aletargar",
    "aletargaren": "aletargar",
    "aletargares": "aletargar",
    "aletargaron": "aletargar",
    "aletargará": "aletargar",
    "aletargarán": "aletargar",
    "aletargarás": "aletargar",
    "aletargaré": "aletargar",
    "aletargaréis": "aletargar",
    "aletargaría": "aletargar",
    "aletargaríais": "aletargar",
    "aletargaríamos": "aletargar",
    "aletargarían": "aletargar",
    "aletargarías": "aletargar",
    "aletargas": "aletargar",
    "aletargase": "aletargar",
    "aletargaseis": "aletargar",
    "aletargasen": "aletargar",
    "aletargases": "aletargar",
    "aletargaste": "aletargar",
    "aletargasteis": "aletargar",
    "aletargo": "aletargar",
    "aletargue": "aletargar",
    "aletarguemos": "aletargar",
    "aletarguen": "aletargar",
    "aletargues": "aletargar",
    "aletargué": "aletargar",
    "aletarguéis": "aletargar",
    "aletargábamos": "aletargar",
    "aletargáis": "aletargar",
    "aletargáramos": "aletargar",
    "aletargáremos": "aletargar",
    "aletargásemos": "aletargar",
    "aletargó": "aletargar",
    "aletas": "aleta",
    "aletazos": "aletazo",
    "aletea": "aletear",
    "aleteaba": "aletear",
    "aleteabais": "aletear",
    "aleteaban": "aletear",
    "aleteabas": "aletear",
    "aletead": "aletear",
    "aleteada": "aletear",
    "aleteadas": "aletear",
    "aleteado": "aletear",
    "aleteados": "aletear",
    "aleteamos": "aletear",
    "aletean": "aletear",
    "aleteando": "aletear",
    "aleteara": "aletear",
    "aletearais": "aletear",
    "aletearan": "aletear",
    "aletearas": "aletear",
    "aleteare": "aletear",
    "aleteareis": "aletear",
    "aletearemos": "aletear",
    "aletearen": "aletear",
    "aleteares": "aletear",
    "aletearon": "aletear",
    "aleteará": "aletear",
    "aletearán": "aletear",
    "aletearás": "aletear",
    "aletearé": "aletear",
    "aletearéis": "aletear",
    "aletearía": "aletear",
    "aletearíais": "aletear",
    "aletearíamos": "aletear",
    "aletearían": "aletear",
    "aletearías": "aletear",
    "aleteas": "aletear",
    "aletease": "aletear",
    "aleteaseis": "aletear",
    "aleteasen": "aletear",
    "aleteases": "aletear",
    "aleteaste": "aletear",
    "aleteasteis": "aletear",
    "aletee": "aletear",
    "aleteemos": "aletear",
    "aleteen": "aletear",
    "aletees": "aletear",
    "aleteo": "aletear",
    "aleteos": "aleteo",
    "aleteábamos": "aletear",
    "aleteáis": "aletear",
    "aleteáramos": "aletear",
    "aleteáremos": "aletear",
    "aleteásemos": "aletear",
    "aleteé": "aletear",
    "aleteéis": "aletear",
    "aleteó": "aletear",
    "aletrinas": "aletrina",
    "aleucémica": "aleucémico",
    "aleucémicas": "aleucémico",
    "aleucémicos": "aleucémico",
    "aleuronas": "aleurona",
    "aleutas": "aleuta",
    "aleutianos": "aleutiano",
    "aleves": "aleve",
    "alevina": "alevín",
    "alevinas": "alevín",
    "alevines": "alevín",
    "alevosa": "alevoso",
    "alevosas": "alevoso",
    "alevosos": "alevoso",
    "alevosías": "alevosía",
    "alexias": "alexia",
    "alexinas": "alexina",
    "alexitimias": "alexitimia",
    "aleyas": "aleya",
    "alezos": "alezo",
    "aleábamos": "alear",
    "aleáis": "alear",
    "aleáramos": "alear",
    "aleáremos": "alear",
    "aleásemos": "alear",
    "aleé": "alear",
    "aleéis": "alear",
    "aleó": "alear",
    "alfabetice": "alfabetizar",
    "alfabeticemos": "alfabetizar",
    "alfabeticen": "alfabetizar",
    "alfabetices": "alfabetizar",
    "alfabeticé": "alfabetizar",
    "alfabeticéis": "alfabetizar",
    "alfabetiza": "alfabetizar",
    "alfabetizaba": "alfabetizar",
    "alfabetizabais": "alfabetizar",
    "alfabetizaban": "alfabetizar",
    "alfabetizabas": "alfabetizar",
    "alfabetizaciones": "alfabetización",
    "alfabetizad": "alfabetizar",
    "alfabetizada": "alfabetizar",
    "alfabetizadas": "alfabetizar",
    "alfabetizado": "alfabetizar",
    "alfabetizados": "alfabetizar",
    "alfabetizamos": "alfabetizar",
    "alfabetizan": "alfabetizar",
    "alfabetizando": "alfabetizar",
    "alfabetizara": "alfabetizar",
    "alfabetizarais": "alfabetizar",
    "alfabetizaran": "alfabetizar",
    "alfabetizaras": "alfabetizar",
    "alfabetizare": "alfabetizar",
    "alfabetizareis": "alfabetizar",
    "alfabetizaremos": "alfabetizar",
    "alfabetizaren": "alfabetizar",
    "alfabetizares": "alfabetizar",
    "alfabetizaron": "alfabetizar",
    "alfabetizará": "alfabetizar",
    "alfabetizarán": "alfabetizar",
    "alfabetizarás": "alfabetizar",
    "alfabetizaré": "alfabetizar",
    "alfabetizaréis": "alfabetizar",
    "alfabetizaría": "alfabetizar",
    "alfabetizaríais": "alfabetizar",
    "alfabetizaríamos": "alfabetizar",
    "alfabetizarían": "alfabetizar",
    "alfabetizarías": "alfabetizar",
    "alfabetizas": "alfabetizar",
    "alfabetizase": "alfabetizar",
    "alfabetizaseis": "alfabetizar",
    "alfabetizasen": "alfabetizar",
    "alfabetizases": "alfabetizar",
    "alfabetizaste": "alfabetizar",
    "alfabetizasteis": "alfabetizar",
    "alfabetizo": "alfabetizar",
    "alfabetizábamos": "alfabetizar",
    "alfabetizáis": "alfabetizar",
    "alfabetizáramos": "alfabetizar",
    "alfabetizáremos": "alfabetizar",
    "alfabetizásemos": "alfabetizar",
    "alfabetizó": "alfabetizar",
    "alfabetos": "alfabeto",
    "alfabética": "alfabético",
    "alfabéticas": "alfabético",
    "alfabéticos": "alfabético",
    "alfafetoproteínas": "alfafetoproteína",
    "alfaglobulinas": "alfaglobulina",
    "alfaguaras": "alfaguara",
    "alfajores": "alfajor",
    "alfalfa": "alfalfar",
    "alfalfaba": "alfalfar",
    "alfalfabais": "alfalfar",
    "alfalfaban": "alfalfar",
    "alfalfabas": "alfalfar",
    "alfalfad": "alfalfar",
    "alfalfada": "alfalfar",
    "alfalfadas": "alfalfar",
    "alfalfado": "alfalfar",
    "alfalfados": "alfalfar",
    "alfalfamos": "alfalfar",
    "alfalfan": "alfalfar",
    "alfalfando": "alfalfar",
    "alfalfara": "alfalfar",
    "alfalfarais": "alfalfar",
    "alfalfaran": "alfalfar",
    "alfalfaras": "alfalfar",
    "alfalfare": "alfalfar",
    "alfalfareis": "alfalfar",
    "alfalfaremos": "alfalfar",
    "alfalfaren": "alfalfar",
    "alfalfares": "alfalfar",
    "alfalfaron": "alfalfar",
    "alfalfará": "alfalfar",
    "alfalfarán": "alfalfar",
    "alfalfarás": "alfalfar",
    "alfalfaré": "alfalfar",
    "alfalfaréis": "alfalfar",
    "alfalfaría": "alfalfar",
    "alfalfaríais": "alfalfar",
    "alfalfaríamos": "alfalfar",
    "alfalfarían": "alfalfar",
    "alfalfarías": "alfalfar",
    "alfalfas": "alfalfar",
    "alfalfase": "alfalfar",
    "alfalfaseis": "alfalfar",
    "alfalfasen": "alfalfar",
    "alfalfases": "alfalfar",
    "alfalfaste": "alfalfar",
    "alfalfasteis": "alfalfar",
    "alfalfe": "alfalfar",
    "alfalfemos": "alfalfar",
    "alfalfen": "alfalfar",
    "alfalfes": "alfalfar",
    "alfalfo": "alfalfar",
    "alfalfábamos": "alfalfar",
    "alfalfáis": "alfalfar",
    "alfalfáramos": "alfalfar",
    "alfalfáremos": "alfalfar",
    "alfalfásemos": "alfalfar",
    "alfalfé": "alfalfar",
    "alfalféis": "alfalfar",
    "alfalfó": "alfalfar",
    "alfanas": "alfana",
    "alfaneques": "alfaneque",
    "alfanjes": "alfanje",
    "alfanumérica": "alfanumérico",
    "alfanuméricas": "alfanumérico",
    "alfanuméricos": "alfanumérico",
    "alfaques": "alfaque",
    "alfaquíes": "alfaquí",
    "alfaquís": "alfaquí",
    "alfaraces": "alfaraz",
    "alfardas": "alfarda",
    "alfardones": "alfardón",
    "alfaremes": "alfareme",
    "alfarera": "alfarero",
    "alfareras": "alfarero",
    "alfareros": "alfarero",
    "alfarerías": "alfarería",
    "alfares": "alfar",
    "alfarjes": "alfarje",
    "alfarjías": "alfarjía",
    "alfarrace": "alfarrazar",
    "alfarracemos": "alfarrazar",
    "alfarracen": "alfarrazar",
    "alfarraces": "alfarrazar",
    "alfarracé": "alfarrazar",
    "alfarracéis": "alfarrazar",
    "alfarraza": "alfarrazar",
    "alfarrazaba": "alfarrazar",
    "alfarrazabais": "alfarrazar",
    "alfarrazaban": "alfarrazar",
    "alfarrazabas": "alfarrazar",
    "alfarrazad": "alfarrazar",
    "alfarrazada": "alfarrazar",
    "alfarrazadas": "alfarrazar",
    "alfarrazado": "alfarrazar",
    "alfarrazados": "alfarrazar",
    "alfarrazamos": "alfarrazar",
    "alfarrazan": "alfarrazar",
    "alfarrazando": "alfarrazar",
    "alfarrazara": "alfarrazar",
    "alfarrazarais": "alfarrazar",
    "alfarrazaran": "alfarrazar",
    "alfarrazaras": "alfarrazar",
    "alfarrazare": "alfarrazar",
    "alfarrazareis": "alfarrazar",
    "alfarrazaremos": "alfarrazar",
    "alfarrazaren": "alfarrazar",
    "alfarrazares": "alfarrazar",
    "alfarrazaron": "alfarrazar",
    "alfarrazará": "alfarrazar",
    "alfarrazarán": "alfarrazar",
    "alfarrazarás": "alfarrazar",
    "alfarrazaré": "alfarrazar",
    "alfarrazaréis": "alfarrazar",
    "alfarrazaría": "alfarrazar",
    "alfarrazaríais": "alfarrazar",
    "alfarrazaríamos": "alfarrazar",
    "alfarrazarían": "alfarrazar",
    "alfarrazarías": "alfarrazar",
    "alfarrazas": "alfarrazar",
    "alfarrazase": "alfarrazar",
    "alfarrazaseis": "alfarrazar",
    "alfarrazasen": "alfarrazar",
    "alfarrazases": "alfarrazar",
    "alfarrazaste": "alfarrazar",
    "alfarrazasteis": "alfarrazar",
    "alfarrazo": "alfarrazar",
    "alfarrazábamos": "alfarrazar",
    "alfarrazáis": "alfarrazar",
    "alfarrazáramos": "alfarrazar",
    "alfarrazáremos": "alfarrazar",
    "alfarrazásemos": "alfarrazar",
    "alfarrazó": "alfarrazar",
    "alfas": "alfa",
    "alferecías": "alferecía",
    "alfeñica": "alfeñicar",
    "alfeñicaba": "alfeñicar",
    "alfeñicabais": "alfeñicar",
    "alfeñicaban": "alfeñicar",
    "alfeñicabas": "alfeñicar",
    "alfeñicad": "alfeñicar",
    "alfeñicada": "alfeñicar",
    "alfeñicadas": "alfeñicar",
    "alfeñicado": "alfeñicar",
    "alfeñicados": "alfeñicar",
    "alfeñicamos": "alfeñicar",
    "alfeñican": "alfeñicar",
    "alfeñicando": "alfeñicar",
    "alfeñicara": "alfeñicar",
    "alfeñicarais": "alfeñicar",
    "alfeñicaran": "alfeñicar",
    "alfeñicaras": "alfeñicar",
    "alfeñicare": "alfeñicar",
    "alfeñicareis": "alfeñicar",
    "alfeñicaremos": "alfeñicar",
    "alfeñicaren": "alfeñicar",
    "alfeñicares": "alfeñicar",
    "alfeñicaron": "alfeñicar",
    "alfeñicará": "alfeñicar",
    "alfeñicarán": "alfeñicar",
    "alfeñicarás": "alfeñicar",
    "alfeñicaré": "alfeñicar",
    "alfeñicaréis": "alfeñicar",
    "alfeñicaría": "alfeñicar",
    "alfeñicaríais": "alfeñicar",
    "alfeñicaríamos": "alfeñicar",
    "alfeñicarían": "alfeñicar",
    "alfeñicarías": "alfeñicar",
    "alfeñicas": "alfeñicar",
    "alfeñicase": "alfeñicar",
    "alfeñicaseis": "alfeñicar",
    "alfeñicasen": "alfeñicar",
    "alfeñicases": "alfeñicar",
    "alfeñicaste": "alfeñicar",
    "alfeñicasteis": "alfeñicar",
    "alfeñico": "alfeñicar",
    "alfeñicábamos": "alfeñicar",
    "alfeñicáis": "alfeñicar",
    "alfeñicáramos": "alfeñicar",
    "alfeñicáremos": "alfeñicar",
    "alfeñicásemos": "alfeñicar",
    "alfeñicó": "alfeñicar",
    "alfeñique": "alfeñicar",
    "alfeñiquemos": "alfeñicar",
    "alfeñiquen": "alfeñicar",
    "alfeñiques": "alfeñique",
    "alfeñiqué": "alfeñicar",
    "alfeñiquéis": "alfeñicar",
    "alfices": "alfiz",
    "alfilerazos": "alfilerazo",
    "alfileres": "alfiler",
    "alfilerillos": "alfilerillo",
    "alfiles": "alfil",
    "alfileteros": "alfiletero",
    "alfoces": "alfoz",
    "alfolíes": "alfolí",
    "alfolís": "alfolí",
    "alfombra": "alfombrar",
    "alfombraba": "alfombrar",
    "alfombrabais": "alfombrar",
    "alfombraban": "alfombrar",
    "alfombrabas": "alfombrar",
    "alfombrad": "alfombrar",
    "alfombrada": "alfombrar",
    "alfombradas": "alfombrar",
    "alfombrado": "alfombrar",
    "alfombrados": "alfombrar",
    "alfombramos": "alfombrar",
    "alfombran": "alfombrar",
    "alfombrando": "alfombrar",
    "alfombrara": "alfombrar",
    "alfombrarais": "alfombrar",
    "alfombraran": "alfombrar",
    "alfombraras": "alfombrar",
    "alfombrare": "alfombrar",
    "alfombrareis": "alfombrar",
    "alfombraremos": "alfombrar",
    "alfombraren": "alfombrar",
    "alfombrares": "alfombrar",
    "alfombraron": "alfombrar",
    "alfombrará": "alfombrar",
    "alfombrarán": "alfombrar",
    "alfombrarás": "alfombrar",
    "alfombraré": "alfombrar",
    "alfombraréis": "alfombrar",
    "alfombraría": "alfombrar",
    "alfombraríais": "alfombrar",
    "alfombraríamos": "alfombrar",
    "alfombrarían": "alfombrar",
    "alfombrarías": "alfombrar",
    "alfombras": "alfombrar",
    "alfombrase": "alfombrar",
    "alfombraseis": "alfombrar",
    "alfombrasen": "alfombrar",
    "alfombrases": "alfombrar",
    "alfombraste": "alfombrar",
    "alfombrasteis": "alfombrar",
    "alfombre": "alfombrar",
    "alfombremos": "alfombrar",
    "alfombren": "alfombrar",
    "alfombres": "alfombrar",
    "alfombrillas": "alfombrilla",
    "alfombro": "alfombrar",
    "alfombrábamos": "alfombrar",
    "alfombráis": "alfombrar",
    "alfombráramos": "alfombrar",
    "alfombráremos": "alfombrar",
    "alfombrásemos": "alfombrar",
    "alfombré": "alfombrar",
    "alfombréis": "alfombrar",
    "alfombró": "alfombrar",
    "alfondegas": "alfondega",
    "alfonsina": "alfonsino",
    "alfonsinas": "alfonsino",
    "alfonsinos": "alfonsino",
    "alfonsismos": "alfonsismo",
    "alfonsíes": "alfonsí",
    "alfonsís": "alfonsí",
    "alforce": "alforzar",
    "alforcemos": "alforzar",
    "alforcen": "alforzar",
    "alforces": "alforzar",
    "alforcé": "alforzar",
    "alforcéis": "alforzar",
    "alforfones": "alforfón",
    "alforjas": "alforja",
    "alforjera": "alforjero",
    "alforjeras": "alforjero",
    "alforjeros": "alforjero",
    "alforvas": "alforva",
    "alforza": "alforzar",
    "alforzaba": "alforzar",
    "alforzabais": "alforzar",
    "alforzaban": "alforzar",
    "alforzabas": "alforzar",
    "alforzad": "alforzar",
    "alforzada": "alforzar",
    "alforzadas": "alforzar",
    "alforzado": "alforzar",
    "alforzados": "alforzar",
    "alforzamos": "alforzar",
    "alforzan": "alforzar",
    "alforzando": "alforzar",
    "alforzara": "alforzar",
    "alforzarais": "alforzar",
    "alforzaran": "alforzar",
    "alforzaras": "alforzar",
    "alforzare": "alforzar",
    "alforzareis": "alforzar",
    "alforzaremos": "alforzar",
    "alforzaren": "alforzar",
    "alforzares": "alforzar",
    "alforzaron": "alforzar",
    "alforzará": "alforzar",
    "alforzarán": "alforzar",
    "alforzarás": "alforzar",
    "alforzaré": "alforzar",
    "alforzaréis": "alforzar",
    "alforzaría": "alforzar",
    "alforzaríais": "alforzar",
    "alforzaríamos": "alforzar",
    "alforzarían": "alforzar",
    "alforzarías": "alforzar",
    "alforzas": "alforzar",
    "alforzase": "alforzar",
    "alforzaseis": "alforzar",
    "alforzasen": "alforzar",
    "alforzases": "alforzar",
    "alforzaste": "alforzar",
    "alforzasteis": "alforzar",
    "alforzo": "alforzar",
    "alforzábamos": "alforzar",
    "alforzáis": "alforzar",
    "alforzáramos": "alforzar",
    "alforzáremos": "alforzar",
    "alforzásemos": "alforzar",
    "alforzó": "alforzar",
    "alféizares": "alféizar",
    "alféreces": "alférez",
    "alfóncigos": "alfóncigo",
    "algales": "algal",
    "algalia": "algaliar",
    "algaliaba": "algaliar",
    "algaliabais": "algaliar",
    "algaliaban": "algaliar",
    "algaliabas": "algaliar",
    "algaliad": "algaliar",
    "algaliada": "algaliar",
    "algaliadas": "algaliar",
    "algaliado": "algaliar",
    "algaliados": "algaliar",
    "algaliamos": "algaliar",
    "algalian": "algaliar",
    "algaliando": "algaliar",
    "algaliara": "algaliar",
    "algaliarais": "algaliar",
    "algaliaran": "algaliar",
    "algaliaras": "algaliar",
    "algaliare": "algaliar",
    "algaliareis": "algaliar",
    "algaliaremos": "algaliar",
    "algaliaren": "algaliar",
    "algaliares": "algaliar",
    "algaliaron": "algaliar",
    "algaliará": "algaliar",
    "algaliarán": "algaliar",
    "algaliarás": "algaliar",
    "algaliaré": "algaliar",
    "algaliaréis": "algaliar",
    "algaliaría": "algaliar",
    "algaliaríais": "algaliar",
    "algaliaríamos": "algaliar",
    "algaliarían": "algaliar",
    "algaliarías": "algaliar",
    "algalias": "algaliar",
    "algaliase": "algaliar",
    "algaliaseis": "algaliar",
    "algaliasen": "algaliar",
    "algaliases": "algaliar",
    "algaliaste": "algaliar",
    "algaliasteis": "algaliar",
    "algalie": "algaliar",
    "algaliemos": "algaliar",
    "algalien": "algaliar",
    "algalies": "algaliar",
    "algalio": "algaliar",
    "algaliábamos": "algaliar",
    "algaliáis": "algaliar",
    "algaliáramos": "algaliar",
    "algaliáremos": "algaliar",
    "algaliásemos": "algaliar",
    "algalié": "algaliar",
    "algaliéis": "algaliar",
    "algalió": "algaliar",
    "algarabías": "algarabía",
    "algaradas": "algarada",
    "algaras": "algara",
    "algarazas": "algaraza",
    "algares": "algar",
    "algarradas": "algarrada",
    "algarrobales": "algarrobal",
    "algarrobas": "algarroba",
    "algarroberos": "algarrobero",
    "algarrobos": "algarrobo",
    "algas": "alga",
    "algavaros": "algavaro",
    "algazaras": "algazara",
    "algazules": "algazul",
    "algebraica": "algebraico",
    "algebraicas": "algebraico",
    "algebraicos": "algebraico",
    "algebristas": "algebrista",
    "algecireña": "algecireño",
    "algecireñas": "algecireño",
    "algecireños": "algecireño",
    "algesias": "algesia",
    "algesiógena": "algesiógeno",
    "algesiógenas": "algesiógeno",
    "algesiógenos": "algesiógeno",
    "algiacusias": "algiacusia",
    "algias": "algia",
    "algideces": "algidez",
    "alginatos": "alginato",
    "algodona": "algodonar",
    "algodonaba": "algodonar",
    "algodonabais": "algodonar",
    "algodonaban": "algodonar",
    "algodonabas": "algodonar",
    "algodonad": "algodonar",
    "algodonada": "algodonar",
    "algodonadas": "algodonar",
    "algodonado": "algodonar",
    "algodonados": "algodonar",
    "algodonales": "algodonal",
    "algodonamos": "algodonar",
    "algodonan": "algodonar",
    "algodonando": "algodonar",
    "algodonara": "algodonar",
    "algodonarais": "algodonar",
    "algodonaran": "algodonar",
    "algodonaras": "algodonar",
    "algodonare": "algodonar",
    "algodonareis": "algodonar",
    "algodonaremos": "algodonar",
    "algodonaren": "algodonar",
    "algodonares": "algodonar",
    "algodonaron": "algodonar",
    "algodonará": "algodonar",
    "algodonarán": "algodonar",
    "algodonarás": "algodonar",
    "algodonaré": "algodonar",
    "algodonaréis": "algodonar",
    "algodonaría": "algodonar",
    "algodonaríais": "algodonar",
    "algodonaríamos": "algodonar",
    "algodonarían": "algodonar",
    "algodonarías": "algodonar",
    "algodonas": "algodonar",
    "algodonase": "algodonar",
    "algodonaseis": "algodonar",
    "algodonasen": "algodonar",
    "algodonases": "algodonar",
    "algodonaste": "algodonar",
    "algodonasteis": "algodonar",
    "algodoncillos": "algodoncillo",
    "algodone": "algodonar",
    "algodonemos": "algodonar",
    "algodonen": "algodonar",
    "algodonera": "algodonero",
    "algodoneras": "algodonero",
    "algodoneros": "algodonero",
    "algodones": "algodonar",
    "algodonitas": "algodonita",
    "algodonizaciones": "algodonización",
    "algodonizaciónes": "algodonización",
    "algodono": "algodonar",
    "algodonosa": "algodonoso",
    "algodonosas": "algodonoso",
    "algodonosos": "algodonoso",
    "algodonábamos": "algodonar",
    "algodonáis": "algodonar",
    "algodonáramos": "algodonar",
    "algodonáremos": "algodonar",
    "algodonásemos": "algodonar",
    "algodoné": "algodonar",
    "algodonéis": "algodonar",
    "algodonó": "algodonar",
    "algonquina": "algonquino",
    "algonquinas": "algonquino",
    "algonquinos": "algonquino",
    "algoritmias": "algoritmia",
    "algoritmos": "algoritmo",
    "algorítmica": "algorítmico",
    "algorítmicas": "algorítmico",
    "algorítmicos": "algorítmico",
    "alguacilazgos": "alguacilazgo",
    "alguacilerías": "alguacilería",
    "alguaciles": "alguacil",
    "alguacilesas": "alguacilesa",
    "alguacilesca": "alguacilesco",
    "alguacilescas": "alguacilesco",
    "alguacilescos": "alguacilesco",
    "alguacilillos": "alguacilillo",
    "alguicidas": "alguicida",
    "alguna": "alguno",
    "algunas": "alguno",
    "algunos": "alguno",
    "algébrica": "algébrico",
    "algébricas": "algébrico",
    "algébricos": "algébrico",
    "algésica": "algésico",
    "algésicas": "algésico",
    "algésicos": "algésico",
    "algínica": "algínico",
    "algínicas": "algínico",
    "algínicos": "algínico",
    "algún": "alguno",
    "alhaja": "alhajar",
    "alhajaba": "alhajar",
    "alhajabais": "alhajar",
    "alhajaban": "alhajar",
    "alhajabas": "alhajar",
    "alhajad": "alhajar",
    "alhajada": "alhajar",
    "alhajadas": "alhajar",
    "alhajado": "alhajar",
    "alhajados": "alhajar",
    "alhajamos": "alhajar",
    "alhajan": "alhajar",
    "alhajando": "alhajar",
    "alhajara": "alhajar",
    "alhajarais": "alhajar",
    "alhajaran": "alhajar",
    "alhajaras": "alhajar",
    "alhajare": "alhajar",
    "alhajareis": "alhajar",
    "alhajaremos": "alhajar",
    "alhajaren": "alhajar",
    "alhajares": "alhajar",
    "alhajaron": "alhajar",
    "alhajará": "alhajar",
    "alhajarán": "alhajar",
    "alhajarás": "alhajar",
    "alhajaré": "alhajar",
    "alhajaréis": "alhajar",
    "alhajaría": "alhajar",
    "alhajaríais": "alhajar",
    "alhajaríamos": "alhajar",
    "alhajarían": "alhajar",
    "alhajarías": "alhajar",
    "alhajas": "alhajar",
    "alhajase": "alhajar",
    "alhajaseis": "alhajar",
    "alhajasen": "alhajar",
    "alhajases": "alhajar",
    "alhajaste": "alhajar",
    "alhajasteis": "alhajar",
    "alhaje": "alhajar",
    "alhajemos": "alhajar",
    "alhajen": "alhajar",
    "alhajes": "alhajar",
    "alhajo": "alhajar",
    "alhajábamos": "alhajar",
    "alhajáis": "alhajar",
    "alhajáramos": "alhajar",
    "alhajáremos": "alhajar",
    "alhajásemos": "alhajar",
    "alhajé": "alhajar",
    "alhajéis": "alhajar",
    "alhajó": "alhajar",
    "alhamíes": "alhamí",
    "alhamís": "alhamí",
    "alharacas": "alharaca",
    "alhelíes": "alhelí",
    "alhelís": "alhelí",
    "alheña": "alheñar",
    "alheñaba": "alheñar",
    "alheñabais": "alheñar",
    "alheñaban": "alheñar",
    "alheñabas": "alheñar",
    "alheñad": "alheñar",
    "alheñada": "alheñar",
    "alheñadas": "alheñar",
    "alheñado": "alheñar",
    "alheñados": "alheñar",
    "alheñamos": "alheñar",
    "alheñan": "alheñar",
    "alheñando": "alheñar",
    "alheñara": "alheñar",
    "alheñarais": "alheñar",
    "alheñaran": "alheñar",
    "alheñaras": "alheñar",
    "alheñare": "alheñar",
    "alheñareis": "alheñar",
    "alheñaremos": "alheñar",
    "alheñaren": "alheñar",
    "alheñares": "alheñar",
    "alheñaron": "alheñar",
    "alheñará": "alheñar",
    "alheñarán": "alheñar",
    "alheñarás": "alheñar",
    "alheñaré": "alheñar",
    "alheñaréis": "alheñar",
    "alheñaría": "alheñar",
    "alheñaríais": "alheñar",
    "alheñaríamos": "alheñar",
    "alheñarían": "alheñar",
    "alheñarías": "alheñar",
    "alheñas": "alheñar",
    "alheñase": "alheñar",
    "alheñaseis": "alheñar",
    "alheñasen": "alheñar",
    "alheñases": "alheñar",
    "alheñaste": "alheñar",
    "alheñasteis": "alheñar",
    "alheñe": "alheñar",
    "alheñemos": "alheñar",
    "alheñen": "alheñar",
    "alheñes": "alheñar",
    "alheño": "alheñar",
    "alheñábamos": "alheñar",
    "alheñáis": "alheñar",
    "alheñáramos": "alheñar",
    "alheñáremos": "alheñar",
    "alheñásemos": "alheñar",
    "alheñé": "alheñar",
    "alheñéis": "alheñar",
    "alheñó": "alheñar",
    "alhorres": "alhorre",
    "alhucemas": "alhucema",
    "alhóndigas": "alhóndiga",
    "aliaba": "aliar",
    "aliabais": "aliar",
    "aliaban": "aliar",
    "aliabas": "aliar",
    "aliad": "aliar",
    "aliada": "aliar",
    "aliadas": "aliar",
    "aliado": "aliar",
    "aliados": "aliar",
    "aliadófila": "aliadófilo",
    "aliadófilas": "aliadófilo",
    "aliadófilos": "aliadófilo",
    "aliagas": "aliaga",
    "aliamos": "aliar",
    "aliando": "aliar",
    "alianzas": "alianza",
    "aliara": "aliar",
    "aliarais": "aliar",
    "aliaran": "aliar",
    "aliaras": "aliar",
    "aliare": "aliar",
    "aliareis": "aliar",
    "aliaremos": "aliar",
    "aliaren": "aliar",
    "aliares": "aliar",
    "aliaron": "aliar",
    "aliará": "aliar",
    "aliarán": "aliar",
    "aliarás": "aliar",
    "aliaré": "aliar",
    "aliaréis": "aliar",
    "aliaría": "aliar",
    "aliaríais": "aliar",
    "aliaríamos": "aliar",
    "aliarían": "aliar",
    "aliarías": "aliar",
    "aliase": "aliar",
    "aliaseis": "aliar",
    "aliasen": "aliar",
    "aliases": "aliar",
    "aliaste": "aliar",
    "aliasteis": "aliar",
    "alicantina": "alicantino",
    "alicantinas": "alicantino",
    "alicantinos": "alicantino",
    "alicata": "alicatar",
    "alicataba": "alicatar",
    "alicatabais": "alicatar",
    "alicataban": "alicatar",
    "alicatabas": "alicatar",
    "alicatad": "alicatar",
    "alicatada": "alicatar",
    "alicatadas": "alicatar",
    "alicatado": "alicatar",
    "alicatados": "alicatar",
    "alicatamos": "alicatar",
    "alicatan": "alicatar",
    "alicatando": "alicatar",
    "alicatara": "alicatar",
    "alicatarais": "alicatar",
    "alicataran": "alicatar",
    "alicataras": "alicatar",
    "alicatare": "alicatar",
    "alicatareis": "alicatar",
    "alicataremos": "alicatar",
    "alicataren": "alicatar",
    "alicatares": "alicatar",
    "alicataron": "alicatar",
    "alicatará": "alicatar",
    "alicatarán": "alicatar",
    "alicatarás": "alicatar",
    "alicataré": "alicatar",
    "alicataréis": "alicatar",
    "alicataría": "alicatar",
    "alicataríais": "alicatar",
    "alicataríamos": "alicatar",
    "alicatarían": "alicatar",
    "alicatarías": "alicatar",
    "alicatas": "alicatar",
    "alicatase": "alicatar",
    "alicataseis": "alicatar",
    "alicatasen": "alicatar",
    "alicatases": "alicatar",
    "alicataste": "alicatar",
    "alicatasteis": "alicatar",
    "alicate": "alicatar",
    "alicatemos": "alicatar",
    "alicaten": "alicatar",
    "alicates": "alicatar",
    "alicato": "alicatar",
    "alicatábamos": "alicatar",
    "alicatáis": "alicatar",
    "alicatáramos": "alicatar",
    "alicatáremos": "alicatar",
    "alicatásemos": "alicatar",
    "alicaté": "alicatar",
    "alicatéis": "alicatar",
    "alicató": "alicatar",
    "alicaída": "alicaído",
    "alicaídas": "alicaído",
    "alicaídos": "alicaído",
    "aliceres": "alicer",
    "alicientes": "aliciente",
    "alicorta": "alicortar",
    "alicortaba": "alicortar",
    "alicortabais": "alicortar",
    "alicortaban": "alicortar",
    "alicortabas": "alicortar",
    "alicortad": "alicortar",
    "alicortada": "alicortar",
    "alicortadas": "alicortar",
    "alicortado": "alicortar",
    "alicortados": "alicortar",
    "alicortamos": "alicortar",
    "alicortan": "alicortar",
    "alicortando": "alicortar",
    "alicortara": "alicortar",
    "alicortarais": "alicortar",
    "alicortaran": "alicortar",
    "alicortaras": "alicortar",
    "alicortare": "alicortar",
    "alicortareis": "alicortar",
    "alicortaremos": "alicortar",
    "alicortaren": "alicortar",
    "alicortares": "alicortar",
    "alicortaron": "alicortar",
    "alicortará": "alicortar",
    "alicortarán": "alicortar",
    "alicortarás": "alicortar",
    "alicortaré": "alicortar",
    "alicortaréis": "alicortar",
    "alicortaría": "alicortar",
    "alicortaríais": "alicortar",
    "alicortaríamos": "alicortar",
    "alicortarían": "alicortar",
    "alicortarías": "alicortar",
    "alicortas": "alicortar",
    "alicortase": "alicortar",
    "alicortaseis": "alicortar",
    "alicortasen": "alicortar",
    "alicortases": "alicortar",
    "alicortaste": "alicortar",
    "alicortasteis": "alicortar",
    "alicorte": "alicortar",
    "alicortemos": "alicortar",
    "alicorten": "alicortar",
    "alicortes": "alicortar",
    "alicorto": "alicortar",
    "alicortábamos": "alicortar",
    "alicortáis": "alicortar",
    "alicortáramos": "alicortar",
    "alicortáremos": "alicortar",
    "alicortásemos": "alicortar",
    "alicorté": "alicortar",
    "alicortéis": "alicortar",
    "alicortó": "alicortar",
    "alicíclica": "alicíclico",
    "alicíclicas": "alicíclico",
    "alicíclicos": "alicíclico",
    "alidadas": "alidada",
    "aliebra": "alebrar",
    "aliebran": "alebrar",
    "aliebras": "alebrar",
    "aliebre": "alebrar",
    "aliebren": "alebrar",
    "aliebres": "alebrar",
    "aliebro": "alebrar",
    "aliemos": "aliar",
    "aliena": "alienar",
    "alienaba": "alienar",
    "alienabais": "alienar",
    "alienaban": "alienar",
    "alienabas": "alienar",
    "alienabilidades": "alienabilidad",
    "alienables": "alienable",
    "alienaciones": "alienación",
    "alienad": "alienar",
    "alienada": "alienar",
    "alienadas": "alienar",
    "alienado": "alienar",
    "alienados": "alienar",
    "alienamos": "alienar",
    "alienan": "alienar",
    "alienando": "alienar",
    "alienara": "alienar",
    "alienarais": "alienar",
    "alienaran": "alienar",
    "alienaras": "alienar",
    "alienare": "alienar",
    "alienareis": "alienar",
    "alienaremos": "alienar",
    "alienaren": "alienar",
    "alienares": "alienar",
    "alienaron": "alienar",
    "alienará": "alienar",
    "alienarán": "alienar",
    "alienarás": "alienar",
    "alienaré": "alienar",
    "alienaréis": "alienar",
    "alienaría": "alienar",
    "alienaríais": "alienar",
    "alienaríamos": "alienar",
    "alienarían": "alienar",
    "alienarías": "alienar",
    "alienas": "alienar",
    "alienase": "alienar",
    "alienaseis": "alienar",
    "alienasen": "alienar",
    "alienases": "alienar",
    "alienaste": "alienar",
    "alienasteis": "alienar",
    "aliene": "alienar",
    "alienemos": "alienar",
    "alienen": "alienar",
    "alienes": "alienar",
    "alienidades": "alienidad",
    "alienistas": "alienista",
    "alieno": "alienar",
    "alienta": "alentar",
    "alientan": "alentar",
    "alientas": "alentar",
    "aliente": "alentar",
    "alienten": "alentar",
    "alientes": "alentar",
    "aliento": "alentar",
    "alientos": "aliento",
    "alienábamos": "alienar",
    "alienáis": "alienar",
    "alienáramos": "alienar",
    "alienáremos": "alienar",
    "alienásemos": "alienar",
    "aliené": "alienar",
    "alienéis": "alienar",
    "alienó": "alienar",
    "alifafes": "alifafe",
    "alifatos": "alifato",
    "alifática": "alifático",
    "alifáticas": "alifático",
    "alifáticos": "alifático",
    "aligaciones": "aligación",
    "aligera": "aligerar",
    "aligeraba": "aligerar",
    "aligerabais": "aligerar",
    "aligeraban": "aligerar",
    "aligerabas": "aligerar",
    "aligerad": "aligerar",
    "aligerada": "aligerar",
    "aligeradas": "aligerar",
    "aligerado": "aligerar",
    "aligerados": "aligerar",
    "aligeramientos": "aligeramiento",
    "aligeramos": "aligerar",
    "aligeran": "aligerar",
    "aligerando": "aligerar",
    "aligerara": "aligerar",
    "aligerarais": "aligerar",
    "aligeraran": "aligerar",
    "aligeraras": "aligerar",
    "aligerare": "aligerar",
    "aligerareis": "aligerar",
    "aligeraremos": "aligerar",
    "aligeraren": "aligerar",
    "aligerares": "aligerar",
    "aligeraron": "aligerar",
    "aligerará": "aligerar",
    "aligerarán": "aligerar",
    "aligerarás": "aligerar",
    "aligeraré": "aligerar",
    "aligeraréis": "aligerar",
    "aligeraría": "aligerar",
    "aligeraríais": "aligerar",
    "aligeraríamos": "aligerar",
    "aligerarían": "aligerar",
    "aligerarías": "aligerar",
    "aligeras": "aligerar",
    "aligerase": "aligerar",
    "aligeraseis": "aligerar",
    "aligerasen": "aligerar",
    "aligerases": "aligerar",
    "aligeraste": "aligerar",
    "aligerasteis": "aligerar",
    "aligere": "aligerar",
    "aligeremos": "aligerar",
    "aligeren": "aligerar",
    "aligeres": "aligerar",
    "aligero": "aligerar",
    "aligerábamos": "aligerar",
    "aligeráis": "aligerar",
    "aligeráramos": "aligerar",
    "aligeráremos": "aligerar",
    "aligerásemos": "aligerar",
    "aligeré": "aligerar",
    "aligeréis": "aligerar",
    "aligeró": "aligerar",
    "aligotes": "aligote",
    "aligustres": "aligustre",
    "aligátores": "aligátor",
    "alija": "alijar",
    "alijaba": "alijar",
    "alijabais": "alijar",
    "alijaban": "alijar",
    "alijabas": "alijar",
    "alijad": "alijar",
    "alijada": "alijar",
    "alijadas": "alijar",
    "alijado": "alijar",
    "alijadora": "alijador",
    "alijadoras": "alijador",
    "alijadores": "alijador",
    "alijados": "alijar",
    "alijamientos": "alijamiento",
    "alijamos": "alijar",
    "alijan": "alijar",
    "alijando": "alijar",
    "alijara": "alijarar",
    "alijaraba": "alijarar",
    "alijarabais": "alijarar",
    "alijaraban": "alijarar",
    "alijarabas": "alijarar",
    "alijarad": "alijarar",
    "alijarada": "alijarar",
    "alijaradas": "alijarar",
    "alijarado": "alijarar",
    "alijarados": "alijarar",
    "alijarais": "alijar",
    "alijaramos": "alijarar",
    "alijaran": "alijarar",
    "alijarando": "alijarar",
    "alijarara": "alijarar",
    "alijararais": "alijarar",
    "alijararan": "alijarar",
    "alijararas": "alijarar",
    "alijarare": "alijarar",
    "alijarareis": "alijarar",
    "alijararemos": "alijarar",
    "alijararen": "alijarar",
    "alijarares": "alijarar",
    "alijararon": "alijarar",
    "alijarará": "alijarar",
    "alijararán": "alijarar",
    "alijararás": "alijarar",
    "alijararé": "alijarar",
    "alijararéis": "alijarar",
    "alijararía": "alijarar",
    "alijararíais": "alijarar",
    "alijararíamos": "alijarar",
    "alijararían": "alijarar",
    "alijararías": "alijarar",
    "alijaras": "alijarar",
    "alijarase": "alijarar",
    "alijaraseis": "alijarar",
    "alijarasen": "alijarar",
    "alijarases": "alijarar",
    "alijaraste": "alijarar",
    "alijarasteis": "alijarar",
    "alijare": "alijarar",
    "alijareis": "alijar",
    "alijaremos": "alijarar",
    "alijaren": "alijarar",
    "alijarera": "alijarero",
    "alijareras": "alijarero",
    "alijareros": "alijarero",
    "alijares": "alijarar",
    "alijaro": "alijarar",
    "alijaron": "alijar",
    "alijará": "alijar",
    "alijarábamos": "alijarar",
    "alijaráis": "alijarar",
    "alijarán": "alijar",
    "alijaráramos": "alijarar",
    "alijaráremos": "alijarar",
    "alijarás": "alijar",
    "alijarásemos": "alijarar",
    "alijaré": "alijarar",
    "alijaréis": "alijarar",
    "alijaría": "alijar",
    "alijaríais": "alijar",
    "alijaríamos": "alijar",
    "alijarían": "alijar",
    "alijarías": "alijar",
    "alijaró": "alijarar",
    "alijas": "alijar",
    "alijase": "alijar",
    "alijaseis": "alijar",
    "alijasen": "alijar",
    "alijases": "alijar",
    "alijaste": "alijar",
    "alijasteis": "alijar",
    "alije": "alijar",
    "alijemos": "alijar",
    "alijen": "alijar",
    "alijes": "alijar",
    "alijo": "alijar",
    "alijos": "alijo",
    "alijábamos": "alijar",
    "alijáis": "alijar",
    "alijáramos": "alijar",
    "alijáremos": "alijar",
    "alijásemos": "alijar",
    "alijé": "alijar",
    "alijéis": "alijar",
    "alijó": "alijar",
    "alilos": "alilo",
    "aliltioureas": "aliltiourea",
    "alimañas": "alimaña",
    "alimañera": "alimañero",
    "alimañeras": "alimañero",
    "alimañeros": "alimañero",
    "alimenta": "alimentar",
    "alimentaba": "alimentar",
    "alimentabais": "alimentar",
    "alimentaban": "alimentar",
    "alimentabas": "alimentar",
    "alimentaciones": "alimentación",
    "alimentad": "alimentar",
    "alimentada": "alimentar",
    "alimentadas": "alimentar",
    "alimentado": "alimentar",
    "alimentadora": "alimentador",
    "alimentadoras": "alimentador",
    "alimentadores": "alimentador",
    "alimentados": "alimentar",
    "alimentamos": "alimentar",
    "alimentan": "alimentar",
    "alimentando": "alimentar",
    "alimentara": "alimentar",
    "alimentarais": "alimentar",
    "alimentaran": "alimentar",
    "alimentaras": "alimentar",
    "alimentare": "alimentar",
    "alimentareis": "alimentar",
    "alimentaremos": "alimentar",
    "alimentaren": "alimentar",
    "alimentares": "alimentar",
    "alimentaria": "alimentario",
    "alimentarias": "alimentario",
    "alimentarios": "alimentario",
    "alimentaron": "alimentar",
    "alimentará": "alimentar",
    "alimentarán": "alimentar",
    "alimentarás": "alimentar",
    "alimentaré": "alimentar",
    "alimentaréis": "alimentar",
    "alimentaría": "alimentar",
    "alimentaríais": "alimentar",
    "alimentaríamos": "alimentar",
    "alimentarían": "alimentar",
    "alimentarías": "alimentar",
    "alimentas": "alimentar",
    "alimentase": "alimentar",
    "alimentaseis": "alimentar",
    "alimentasen": "alimentar",
    "alimentases": "alimentar",
    "alimentaste": "alimentar",
    "alimentasteis": "alimentar",
    "alimente": "alimentar",
    "alimentemos": "alimentar",
    "alimenten": "alimentar",
    "alimentes": "alimentar",
    "alimenticia": "alimenticio",
    "alimenticias": "alimenticio",
    "alimenticios": "alimenticio",
    "alimento": "alimentar",
    "alimentos": "alimento",
    "alimentábamos": "alimentar",
    "alimentáis": "alimentar",
    "alimentáramos": "alimentar",
    "alimentáremos": "alimentar",
    "alimentásemos": "alimentar",
    "alimenté": "alimentar",
    "alimentéis": "alimentar",
    "alimentó": "alimentar",
    "alimoches": "alimoche",
    "alimona": "alimonar",
    "alimonaba": "alimonar",
    "alimonabais": "alimonar",
    "alimonaban": "alimonar",
    "alimonabas": "alimonar",
    "alimonad": "alimonar",
    "alimonada": "alimonar",
    "alimonadas": "alimonar",
    "alimonado": "alimonar",
    "alimonados": "alimonar",
    "alimonamos": "alimonar",
    "alimonan": "alimonar",
    "alimonando": "alimonar",
    "alimonara": "alimonar",
    "alimonarais": "alimonar",
    "alimonaran": "alimonar",
    "alimonaras": "alimonar",
    "alimonare": "alimonar",
    "alimonareis": "alimonar",
    "alimonaremos": "alimonar",
    "alimonaren": "alimonar",
    "alimonares": "alimonar",
    "alimonaron": "alimonar",
    "alimonará": "alimonar",
    "alimonarán": "alimonar",
    "alimonarás": "alimonar",
    "alimonaré": "alimonar",
    "alimonaréis": "alimonar",
    "alimonaría": "alimonar",
    "alimonaríais": "alimonar",
    "alimonaríamos": "alimonar",
    "alimonarían": "alimonar",
    "alimonarías": "alimonar",
    "alimonas": "alimonar",
    "alimonase": "alimonar",
    "alimonaseis": "alimonar",
    "alimonasen": "alimonar",
    "alimonases": "alimonar",
    "alimonaste": "alimonar",
    "alimonasteis": "alimonar",
    "alimone": "alimonar",
    "alimonemos": "alimonar",
    "alimonen": "alimonar",
    "alimones": "alimonar",
    "alimono": "alimonar",
    "alimonábamos": "alimonar",
    "alimonáis": "alimonar",
    "alimonáramos": "alimonar",
    "alimonáremos": "alimonar",
    "alimonásemos": "alimonar",
    "alimoné": "alimonar",
    "alimonéis": "alimonar",
    "alimonó": "alimonar",
    "alinda": "alindar",
    "alindaba": "alindar",
    "alindabais": "alindar",
    "alindaban": "alindar",
    "alindabas": "alindar",
    "alindad": "alindar",
    "alindada": "alindar",
    "alindadas": "alindar",
    "alindado": "alindar",
    "alindados": "alindar",
    "alindamos": "alindar",
    "alindan": "alindar",
    "alindando": "alindar",
    "alindara": "alindar",
    "alindarais": "alindar",
    "alindaran": "alindar",
    "alindaras": "alindar",
    "alindare": "alindar",
    "alindareis": "alindar",
    "alindaremos": "alindar",
    "alindaren": "alindar",
    "alindares": "alindar",
    "alindaron": "alindar",
    "alindará": "alindar",
    "alindarán": "alindar",
    "alindarás": "alindar",
    "alindaré": "alindar",
    "alindaréis": "alindar",
    "alindaría": "alindar",
    "alindaríais": "alindar",
    "alindaríamos": "alindar",
    "alindarían": "alindar",
    "alindarías": "alindar",
    "alindas": "alindar",
    "alindase": "alindar",
    "alindaseis": "alindar",
    "alindasen": "alindar",
    "alindases": "alindar",
    "alindaste": "alindar",
    "alindasteis": "alindar",
    "alinde": "alindar",
    "alindemos": "alindar",
    "alinden": "alindar",
    "alindera": "alinderar",
    "alinderaba": "alinderar",
    "alinderabais": "alinderar",
    "alinderaban": "alinderar",
    "alinderabas": "alinderar",
    "alinderad": "alinderar",
    "alinderada": "alinderar",
    "alinderadas": "alinderar",
    "alinderado": "alinderar",
    "alinderados": "alinderar",
    "alinderamos": "alinderar",
    "alinderan": "alinderar",
    "alinderando": "alinderar",
    "alinderara": "alinderar",
    "alinderarais": "alinderar",
    "alinderaran": "alinderar",
    "alinderaras": "alinderar",
    "alinderare": "alinderar",
    "alinderareis": "alinderar",
    "alinderaremos": "alinderar",
    "alinderaren": "alinderar",
    "alinderares": "alinderar",
    "alinderaron": "alinderar",
    "alinderará": "alinderar",
    "alinderarán": "alinderar",
    "alinderarás": "alinderar",
    "alinderaré": "alinderar",
    "alinderaréis": "alinderar",
    "alinderaría": "alinderar",
    "alinderaríais": "alinderar",
    "alinderaríamos": "alinderar",
    "alinderarían": "alinderar",
    "alinderarías": "alinderar",
    "alinderas": "alinderar",
    "alinderase": "alinderar",
    "alinderaseis": "alinderar",
    "alinderasen": "alinderar",
    "alinderases": "alinderar",
    "alinderaste": "alinderar",
    "alinderasteis": "alinderar",
    "alindere": "alinderar",
    "alinderemos": "alinderar",
    "alinderen": "alinderar",
    "alinderes": "alinderar",
    "alindero": "alinderar",
    "alinderábamos": "alinderar",
    "alinderáis": "alinderar",
    "alinderáramos": "alinderar",
    "alinderáremos": "alinderar",
    "alinderásemos": "alinderar",
    "alinderé": "alinderar",
    "alinderéis": "alinderar",
    "alinderó": "alinderar",
    "alindes": "alindar",
    "alindo": "alindar",
    "alindábamos": "alindar",
    "alindáis": "alindar",
    "alindáramos": "alindar",
    "alindáremos": "alindar",
    "alindásemos": "alindar",
    "alindé": "alindar",
    "alindéis": "alindar",
    "alindó": "alindar",
    "alinea": "alinear",
    "alineaba": "alinear",
    "alineabais": "alinear",
    "alineaban": "alinear",
    "alineabas": "alinear",
    "alineabilidades": "alineabilidad",
    "alineaciones": "alineación",
    "alinead": "alinear",
    "alineada": "alinear",
    "alineadas": "alinear",
    "alineado": "alinear",
    "alineadores": "alineador",
    "alineados": "alinear",
    "alineamientos": "alineamiento",
    "alineamos": "alinear",
    "alinean": "alinear",
    "alineando": "alinear",
    "alineara": "alinear",
    "alinearais": "alinear",
    "alinearan": "alinear",
    "alinearas": "alinear",
    "alineare": "alinear",
    "alineareis": "alinear",
    "alinearemos": "alinear",
    "alinearen": "alinear",
    "alineares": "alinear",
    "alinearon": "alinear",
    "alineará": "alinear",
    "alinearán": "alinear",
    "alinearás": "alinear",
    "alinearé": "alinear",
    "alinearéis": "alinear",
    "alinearía": "alinear",
    "alinearíais": "alinear",
    "alinearíamos": "alinear",
    "alinearían": "alinear",
    "alinearías": "alinear",
    "alineas": "alinear",
    "alinease": "alinear",
    "alineaseis": "alinear",
    "alineasen": "alinear",
    "alineases": "alinear",
    "alineaste": "alinear",
    "alineasteis": "alinear",
    "alinee": "alinear",
    "alineemos": "alinear",
    "alineen": "alinear",
    "alinees": "alinear",
    "alineo": "alinear",
    "alineábamos": "alinear",
    "alineáis": "alinear",
    "alineáramos": "alinear",
    "alineáremos": "alinear",
    "alineásemos": "alinear",
    "alineé": "alinear",
    "alineéis": "alinear",
    "alineó": "alinear",
    "aliolis": "alioli",
    "alirones": "alirón",
    "alisa": "alisar",
    "alisaba": "alisar",
    "alisabais": "alisar",
    "alisaban": "alisar",
    "alisabas": "alisar",
    "alisad": "alisar",
    "alisada": "alisar",
    "alisadas": "alisar",
    "alisado": "alisar",
    "alisadora": "alisador",
    "alisadoras": "alisador",
    "alisadores": "alisador",
    "alisados": "alisar",
    "alisaduras": "alisadura",
    "alisales": "alisal",
    "alisamientos": "alisamiento",
    "alisamos": "alisar",
    "alisan": "alisar",
    "alisando": "alisar",
    "alisara": "alisar",
    "alisarais": "alisar",
    "alisaran": "alisar",
    "alisaras": "alisar",
    "alisare": "alisar",
    "alisareis": "alisar",
    "alisaremos": "alisar",
    "alisaren": "alisar",
    "alisares": "alisar",
    "alisaron": "alisar",
    "alisará": "alisar",
    "alisarán": "alisar",
    "alisarás": "alisar",
    "alisaré": "alisar",
    "alisaréis": "alisar",
    "alisaría": "alisar",
    "alisaríais": "alisar",
    "alisaríamos": "alisar",
    "alisarían": "alisar",
    "alisarías": "alisar",
    "alisas": "alisar",
    "alisase": "alisar",
    "alisaseis": "alisar",
    "alisasen": "alisar",
    "alisases": "alisar",
    "alisaste": "alisar",
    "alisasteis": "alisar",
    "alise": "alisar",
    "alisedas": "aliseda",
    "alisemos": "alisar",
    "alisen": "alisar",
    "alises": "alisar",
    "alisia": "alisio",
    "alisias": "alisio",
    "alisios": "alisio",
    "alismatáceas": "alismatácea",
    "aliso": "alisar",
    "alisos": "aliso",
    "alista": "alistar",
    "alistaba": "alistar",
    "alistabais": "alistar",
    "alistaban": "alistar",
    "alistabas": "alistar",
    "alistad": "alistar",
    "alistada": "alistar",
    "alistadas": "alistar",
    "alistado": "alistar",
    "alistados": "alistar",
    "alistamientos": "alistamiento",
    "alistamos": "alistar",
    "alistan": "alistar",
    "alistando": "alistar",
    "alistara": "alistar",
    "alistarais": "alistar",
    "alistaran": "alistar",
    "alistaras": "alistar",
    "alistare": "alistar",
    "alistareis": "alistar",
    "alistaremos": "alistar",
    "alistaren": "alistar",
    "alistares": "alistar",
    "alistaron": "alistar",
    "alistará": "alistar",
    "alistarán": "alistar",
    "alistarás": "alistar",
    "alistaré": "alistar",
    "alistaréis": "alistar",
    "alistaría": "alistar",
    "alistaríais": "alistar",
    "alistaríamos": "alistar",
    "alistarían": "alistar",
    "alistarías": "alistar",
    "alistas": "alistar",
    "alistase": "alistar",
    "alistaseis": "alistar",
    "alistasen": "alistar",
    "alistases": "alistar",
    "alistaste": "alistar",
    "alistasteis": "alistar",
    "aliste": "alistar",
    "alistemos": "alistar",
    "alisten": "alistar",
    "alistes": "alistar",
    "alisto": "alistar",
    "alistábamos": "alistar",
    "alistáis": "alistar",
    "alistáramos": "alistar",
    "alistáremos": "alistar",
    "alistásemos": "alistar",
    "alisté": "alistar",
    "alistéis": "alistar",
    "alistó": "alistar",
    "alisábamos": "alisar",
    "alisáis": "alisar",
    "alisáramos": "alisar",
    "alisáremos": "alisar",
    "alisásemos": "alisar",
    "alisé": "alisar",
    "aliséis": "alisar",
    "alisó": "alisar",
    "alitanes": "alitán",
    "alitas": "alita",
    "aliteraciones": "aliteración",
    "aliterada": "aliterado",
    "aliteradas": "aliterado",
    "aliterados": "aliterado",
    "alitizaciones": "alitización",
    "alivia": "aliviar",
    "aliviaba": "aliviar",
    "aliviabais": "aliviar",
    "aliviaban": "aliviar",
    "aliviabas": "aliviar",
    "aliviad": "aliviar",
    "aliviada": "aliviar",
    "aliviadas": "aliviar",
    "aliviaderos": "aliviadero",
    "aliviado": "aliviar",
    "aliviados": "aliviar",
    "aliviamos": "aliviar",
    "alivian": "aliviar",
    "aliviana": "alivianar",
    "alivianaba": "alivianar",
    "alivianabais": "alivianar",
    "alivianaban": "alivianar",
    "alivianabas": "alivianar",
    "alivianad": "alivianar",
    "alivianada": "alivianar",
    "alivianadas": "alivianar",
    "alivianado": "alivianar",
    "alivianados": "alivianar",
    "alivianamos": "alivianar",
    "alivianan": "alivianar",
    "alivianando": "alivianar",
    "alivianara": "alivianar",
    "alivianarais": "alivianar",
    "alivianaran": "alivianar",
    "alivianaras": "alivianar",
    "alivianare": "alivianar",
    "alivianareis": "alivianar",
    "alivianaremos": "alivianar",
    "alivianaren": "alivianar",
    "alivianares": "alivianar",
    "alivianaron": "alivianar",
    "alivianará": "alivianar",
    "alivianarán": "alivianar",
    "alivianarás": "alivianar",
    "alivianaré": "alivianar",
    "alivianaréis": "alivianar",
    "alivianaría": "alivianar",
    "alivianaríais": "alivianar",
    "alivianaríamos": "alivianar",
    "alivianarían": "alivianar",
    "alivianarías": "alivianar",
    "alivianas": "alivianar",
    "alivianase": "alivianar",
    "alivianaseis": "alivianar",
    "alivianasen": "alivianar",
    "alivianases": "alivianar",
    "alivianaste": "alivianar",
    "alivianasteis": "alivianar",
    "aliviando": "aliviar",
    "aliviane": "alivianar",
    "alivianemos": "alivianar",
    "alivianen": "alivianar",
    "alivianes": "alivianar",
    "aliviano": "alivianar",
    "alivianábamos": "alivianar",
    "alivianáis": "alivianar",
    "alivianáramos": "alivianar",
    "alivianáremos": "alivianar",
    "alivianásemos": "alivianar",
    "aliviané": "alivianar",
    "alivianéis": "alivianar",
    "alivianó": "alivianar",
    "aliviara": "aliviar",
    "aliviarais": "aliviar",
    "aliviaran": "aliviar",
    "aliviaras": "aliviar",
    "aliviare": "aliviar",
    "aliviareis": "aliviar",
    "aliviaremos": "aliviar",
    "aliviaren": "aliviar",
    "aliviares": "aliviar",
    "aliviaron": "aliviar",
    "aliviará": "aliviar",
    "aliviarán": "aliviar",
    "aliviarás": "aliviar",
    "aliviaré": "aliviar",
    "aliviaréis": "aliviar",
    "aliviaría": "aliviar",
    "aliviaríais": "aliviar",
    "aliviaríamos": "aliviar",
    "aliviarían": "aliviar",
    "aliviarías": "aliviar",
    "alivias": "aliviar",
    "aliviase": "aliviar",
    "aliviaseis": "aliviar",
    "aliviasen": "aliviar",
    "aliviases": "aliviar",
    "aliviaste": "aliviar",
    "aliviasteis": "aliviar",
    "alivie": "aliviar",
    "aliviemos": "aliviar",
    "alivien": "aliviar",
    "alivies": "aliviar",
    "alivio": "aliviar",
    "alivios": "alivio",
    "aliviábamos": "aliviar",
    "aliviáis": "aliviar",
    "aliviáramos": "aliviar",
    "aliviáremos": "aliviar",
    "aliviásemos": "aliviar",
    "alivié": "aliviar",
    "aliviéis": "aliviar",
    "alivió": "aliviar",
    "alizares": "alizar",
    "alizarinas": "alizarina",
    "aliábamos": "aliar",
    "aliácea": "aliáceo",
    "aliáceas": "aliáceo",
    "aliáceos": "aliáceo",
    "aliáis": "aliar",
    "aliáramos": "aliar",
    "aliáremos": "aliar",
    "aliásemos": "aliar",
    "alié": "aliar",
    "aliéis": "aliar",
    "aliña": "aliñar",
    "aliñaba": "aliñar",
    "aliñabais": "aliñar",
    "aliñaban": "aliñar",
    "aliñabas": "aliñar",
    "aliñad": "aliñar",
    "aliñada": "aliñar",
    "aliñadas": "aliñar",
    "aliñado": "aliñar",
    "aliñados": "aliñar",
    "aliñamos": "aliñar",
    "aliñan": "aliñar",
    "aliñando": "aliñar",
    "aliñara": "aliñar",
    "aliñarais": "aliñar",
    "aliñaran": "aliñar",
    "aliñaras": "aliñar",
    "aliñare": "aliñar",
    "aliñareis": "aliñar",
    "aliñaremos": "aliñar",
    "aliñaren": "aliñar",
    "aliñares": "aliñar",
    "aliñaron": "aliñar",
    "aliñará": "aliñar",
    "aliñarán": "aliñar",
    "aliñarás": "aliñar",
    "aliñaré": "aliñar",
    "aliñaréis": "aliñar",
    "aliñaría": "aliñar",
    "aliñaríais": "aliñar",
    "aliñaríamos": "aliñar",
    "aliñarían": "aliñar",
    "aliñarías": "aliñar",
    "aliñas": "aliñar",
    "aliñase": "aliñar",
    "aliñaseis": "aliñar",
    "aliñasen": "aliñar",
    "aliñases": "aliñar",
    "aliñaste": "aliñar",
    "aliñasteis": "aliñar",
    "aliñe": "aliñar",
    "aliñemos": "aliñar",
    "aliñen": "aliñar",
    "aliñes": "aliñar",
    "aliño": "aliñar",
    "aliños": "aliño",
    "aliñábamos": "aliñar",
    "aliñáis": "aliñar",
    "aliñáramos": "aliñar",
    "aliñáremos": "aliñar",
    "aliñásemos": "aliñar",
    "aliñé": "aliñar",
    "aliñéis": "aliñar",
    "aliñó": "aliñar",
    "alió": "aliar",
    "aljabas": "aljaba",
    "aljamas": "aljama",
    "aljamiada": "aljamiado",
    "aljamiadas": "aljamiado",
    "aljamiados": "aljamiado",
    "aljamías": "aljamía",
    "aljibes": "aljibe",
    "aljofainas": "aljofaina",
    "aljofara": "aljofarar",
    "aljofaraba": "aljofarar",
    "aljofarabais": "aljofarar",
    "aljofaraban": "aljofarar",
    "aljofarabas": "aljofarar",
    "aljofarad": "aljofarar",
    "aljofarada": "aljofarar",
    "aljofaradas": "aljofarar",
    "aljofarado": "aljofarar",
    "aljofarados": "aljofarar",
    "aljofaramos": "aljofarar",
    "aljofaran": "aljofarar",
    "aljofarando": "aljofarar",
    "aljofarara": "aljofarar",
    "aljofararais": "aljofarar",
    "aljofararan": "aljofarar",
    "aljofararas": "aljofarar",
    "aljofarare": "aljofarar",
    "aljofarareis": "aljofarar",
    "aljofararemos": "aljofarar",
    "aljofararen": "aljofarar",
    "aljofarares": "aljofarar",
    "aljofararon": "aljofarar",
    "aljofarará": "aljofarar",
    "aljofararán": "aljofarar",
    "aljofararás": "aljofarar",
    "aljofararé": "aljofarar",
    "aljofararéis": "aljofarar",
    "aljofararía": "aljofarar",
    "aljofararíais": "aljofarar",
    "aljofararíamos": "aljofarar",
    "aljofararían": "aljofarar",
    "aljofararías": "aljofarar",
    "aljofaras": "aljofarar",
    "aljofarase": "aljofarar",
    "aljofaraseis": "aljofarar",
    "aljofarasen": "aljofarar",
    "aljofarases": "aljofarar",
    "aljofaraste": "aljofarar",
    "aljofarasteis": "aljofarar",
    "aljofare": "aljofarar",
    "aljofaremos": "aljofarar",
    "aljofaren": "aljofarar",
    "aljofares": "aljofarar",
    "aljofaro": "aljofarar",
    "aljofarábamos": "aljofarar",
    "aljofaráis": "aljofarar",
    "aljofaráramos": "aljofarar",
    "aljofaráremos": "aljofarar",
    "aljofarásemos": "aljofarar",
    "aljofaré": "aljofarar",
    "aljofaréis": "aljofarar",
    "aljofaró": "aljofarar",
    "aljofifa": "aljofifar",
    "aljofifaba": "aljofifar",
    "aljofifabais": "aljofifar",
    "aljofifaban": "aljofifar",
    "aljofifabas": "aljofifar",
    "aljofifad": "aljofifar",
    "aljofifada": "aljofifar",
    "aljofifadas": "aljofifar",
    "aljofifado": "aljofifar",
    "aljofifados": "aljofifar",
    "aljofifamos": "aljofifar",
    "aljofifan": "aljofifar",
    "aljofifando": "aljofifar",
    "aljofifara": "aljofifar",
    "aljofifarais": "aljofifar",
    "aljofifaran": "aljofifar",
    "aljofifaras": "aljofifar",
    "aljofifare": "aljofifar",
    "aljofifareis": "aljofifar",
    "aljofifaremos": "aljofifar",
    "aljofifaren": "aljofifar",
    "aljofifares": "aljofifar",
    "aljofifaron": "aljofifar",
    "aljofifará": "aljofifar",
    "aljofifarán": "aljofifar",
    "aljofifarás": "aljofifar",
    "aljofifaré": "aljofifar",
    "aljofifaréis": "aljofifar",
    "aljofifaría": "aljofifar",
    "aljofifaríais": "aljofifar",
    "aljofifaríamos": "aljofifar",
    "aljofifarían": "aljofifar",
    "aljofifarías": "aljofifar",
    "aljofifas": "aljofifar",
    "aljofifase": "aljofifar",
    "aljofifaseis": "aljofifar",
    "aljofifasen": "aljofifar",
    "aljofifases": "aljofifar",
    "aljofifaste": "aljofifar",
    "aljofifasteis": "aljofifar",
    "aljofife": "aljofifar",
    "aljofifemos": "aljofifar",
    "aljofifen": "aljofifar",
    "aljofifes": "aljofifar",
    "aljofifo": "aljofifar",
    "aljofifábamos": "aljofifar",
    "aljofifáis": "aljofifar",
    "aljofifáramos": "aljofifar",
    "aljofifáremos": "aljofifar",
    "aljofifásemos": "aljofifar",
    "aljofifé": "aljofifar",
    "aljofiféis": "aljofifar",
    "aljofifó": "aljofifar",
    "aljubas": "aljuba",
    "aljófares": "aljófar",
    "allana": "allanar",
    "allanaba": "allanar",
    "allanabais": "allanar",
    "allanaban": "allanar",
    "allanabas": "allanar",
    "allanad": "allanar",
    "allanada": "allanar",
    "allanadas": "allanar",
    "allanado": "allanar",
    "allanadora": "allanador",
    "allanadoras": "allanador",
    "allanadores": "allanador",
    "allanados": "allanar",
    "allanamientos": "allanamiento",
    "allanamos": "allanar",
    "allanan": "allanar",
    "allanando": "allanar",
    "allanara": "allanar",
    "allanarais": "allanar",
    "allanaran": "allanar",
    "allanaras": "allanar",
    "allanare": "allanar",
    "allanareis": "allanar",
    "allanaremos": "allanar",
    "allanaren": "allanar",
    "allanares": "allanar",
    "allanaron": "allanar",
    "allanará": "allanar",
    "allanarán": "allanar",
    "allanarás": "allanar",
    "allanaré": "allanar",
    "allanaréis": "allanar",
    "allanaría": "allanar",
    "allanaríais": "allanar",
    "allanaríamos": "allanar",
    "allanarían": "allanar",
    "allanarías": "allanar",
    "allanas": "allanar",
    "allanase": "allanar",
    "allanaseis": "allanar",
    "allanasen": "allanar",
    "allanases": "allanar",
    "allanaste": "allanar",
    "allanasteis": "allanar",
    "allane": "allanar",
    "allanemos": "allanar",
    "allanen": "allanar",
    "allanes": "allanar",
    "allano": "allanar",
    "allanábamos": "allanar",
    "allanáis": "allanar",
    "allanáramos": "allanar",
    "allanáremos": "allanar",
    "allanásemos": "allanar",
    "allané": "allanar",
    "allanéis": "allanar",
    "allanó": "allanar",
    "allega": "allegar",
    "allegaba": "allegar",
    "allegabais": "allegar",
    "allegaban": "allegar",
    "allegabas": "allegar",
    "allegad": "allegar",
    "allegada": "allegar",
    "allegadas": "allegar",
    "allegado": "allegar",
    "allegadora": "allegador",
    "allegadoras": "allegador",
    "allegadores": "allegador",
    "allegados": "allegar",
    "allegamientos": "allegamiento",
    "allegamos": "allegar",
    "allegan": "allegar",
    "allegando": "allegar",
    "allegara": "allegar",
    "allegarais": "allegar",
    "allegaran": "allegar",
    "allegaras": "allegar",
    "allegare": "allegar",
    "allegareis": "allegar",
    "allegaremos": "allegar",
    "allegaren": "allegar",
    "allegares": "allegar",
    "allegaron": "allegar",
    "allegará": "allegar",
    "allegarán": "allegar",
    "allegarás": "allegar",
    "allegaré": "allegar",
    "allegaréis": "allegar",
    "allegaría": "allegar",
    "allegaríais": "allegar",
    "allegaríamos": "allegar",
    "allegarían": "allegar",
    "allegarías": "allegar",
    "allegas": "allegar",
    "allegase": "allegar",
    "allegaseis": "allegar",
    "allegasen": "allegar",
    "allegases": "allegar",
    "allegaste": "allegar",
    "allegasteis": "allegar",
    "allego": "allegar",
    "allegrettos": "allegretto",
    "allegros": "allegro",
    "allegue": "allegar",
    "alleguemos": "allegar",
    "alleguen": "allegar",
    "allegues": "allegar",
    "allegué": "allegar",
    "alleguéis": "allegar",
    "allegábamos": "allegar",
    "allegáis": "allegar",
    "allegáramos": "allegar",
    "allegáremos": "allegar",
    "allegásemos": "allegar",
    "allegó": "allegar",
    "almacena": "almacenar",
    "almacenaba": "almacenar",
    "almacenabais": "almacenar",
    "almacenaban": "almacenar",
    "almacenabas": "almacenar",
    "almacenad": "almacenar",
    "almacenada": "almacenar",
    "almacenadas": "almacenar",
    "almacenado": "almacenar",
    "almacenados": "almacenar",
    "almacenajes": "almacenaje",
    "almacenamientos": "almacenamiento",
    "almacenamos": "almacenar",
    "almacenan": "almacenar",
    "almacenando": "almacenar",
    "almacenara": "almacenar",
    "almacenarais": "almacenar",
    "almacenaran": "almacenar",
    "almacenaras": "almacenar",
    "almacenare": "almacenar",
    "almacenareis": "almacenar",
    "almacenaremos": "almacenar",
    "almacenaren": "almacenar",
    "almacenares": "almacenar",
    "almacenaron": "almacenar",
    "almacenará": "almacenar",
    "almacenarán": "almacenar",
    "almacenarás": "almacenar",
    "almacenaré": "almacenar",
    "almacenaréis": "almacenar",
    "almacenaría": "almacenar",
    "almacenaríais": "almacenar",
    "almacenaríamos": "almacenar",
    "almacenarían": "almacenar",
    "almacenarías": "almacenar",
    "almacenas": "almacenar",
    "almacenase": "almacenar",
    "almacenaseis": "almacenar",
    "almacenasen": "almacenar",
    "almacenases": "almacenar",
    "almacenaste": "almacenar",
    "almacenasteis": "almacenar",
    "almacene": "almacenar",
    "almacenemos": "almacenar",
    "almacenen": "almacenar",
    "almacenera": "almacenero",
    "almaceneras": "almacenero",
    "almaceneros": "almacenero",
    "almacenes": "almacenar",
    "almacenistas": "almacenista",
    "almaceno": "almacenar",
    "almacenábamos": "almacenar",
    "almacenáis": "almacenar",
    "almacenáramos": "almacenar",
    "almacenáremos": "almacenar",
    "almacenásemos": "almacenar",
    "almacené": "almacenar",
    "almacenéis": "almacenar",
    "almacenó": "almacenar",
    "almadia": "almadiar",
    "almadiaba": "almadiar",
    "almadiabais": "almadiar",
    "almadiaban": "almadiar",
    "almadiabas": "almadiar",
    "almadiad": "almadiar",
    "almadiada": "almadiar",
    "almadiadas": "almadiar",
    "almadiado": "almadiar",
    "almadiados": "almadiar",
    "almadiamos": "almadiar",
    "almadian": "almadiar",
    "almadiando": "almadiar",
    "almadiara": "almadiar",
    "almadiarais": "almadiar",
    "almadiaran": "almadiar",
    "almadiaras": "almadiar",
    "almadiare": "almadiar",
    "almadiareis": "almadiar",
    "almadiaremos": "almadiar",
    "almadiaren": "almadiar",
    "almadiares": "almadiar",
    "almadiaron": "almadiar",
    "almadiará": "almadiar",
    "almadiarán": "almadiar",
    "almadiarás": "almadiar",
    "almadiaré": "almadiar",
    "almadiaréis": "almadiar",
    "almadiaría": "almadiar",
    "almadiaríais": "almadiar",
    "almadiaríamos": "almadiar",
    "almadiarían": "almadiar",
    "almadiarías": "almadiar",
    "almadias": "almadiar",
    "almadiase": "almadiar",
    "almadiaseis": "almadiar",
    "almadiasen": "almadiar",
    "almadiases": "almadiar",
    "almadiaste": "almadiar",
    "almadiasteis": "almadiar",
    "almadie": "almadiar",
    "almadiemos": "almadiar",
    "almadien": "almadiar",
    "almadies": "almadiar",
    "almadio": "almadiar",
    "almadiábamos": "almadiar",
    "almadiáis": "almadiar",
    "almadiáramos": "almadiar",
    "almadiáremos": "almadiar",
    "almadiásemos": "almadiar",
    "almadié": "almadiar",
    "almadiéis": "almadiar",
    "almadió": "almadiar",
    "almadrabas": "almadraba",
    "almadrabera": "almadrabero",
    "almadraberas": "almadrabero",
    "almadraberos": "almadrabero",
    "almadreñas": "almadreña",
    "almadreñera": "almadreñero",
    "almadreñeras": "almadreñero",
    "almadreñeros": "almadreñero",
    "almadías": "almadía",
    "almagestos": "almagesto",
    "almagra": "almagrar",
    "almagraba": "almagrar",
    "almagrabais": "almagrar",
    "almagraban": "almagrar",
    "almagrabas": "almagrar",
    "almagrad": "almagrar",
    "almagrada": "almagrar",
    "almagradas": "almagrar",
    "almagrado": "almagrar",
    "almagrados": "almagrar",
    "almagraduras": "almagradura",
    "almagrales": "almagral",
    "almagramos": "almagrar",
    "almagran": "almagrar",
    "almagrando": "almagrar",
    "almagrara": "almagrar",
    "almagrarais": "almagrar",
    "almagraran": "almagrar",
    "almagraras": "almagrar",
    "almagrare": "almagrar",
    "almagrareis": "almagrar",
    "almagraremos": "almagrar",
    "almagraren": "almagrar",
    "almagrares": "almagrar",
    "almagraron": "almagrar",
    "almagrará": "almagrar",
    "almagrarán": "almagrar",
    "almagrarás": "almagrar",
    "almagraré": "almagrar",
    "almagraréis": "almagrar",
    "almagraría": "almagrar",
    "almagraríais": "almagrar",
    "almagraríamos": "almagrar",
    "almagrarían": "almagrar",
    "almagrarías": "almagrar",
    "almagras": "almagrar",
    "almagrase": "almagrar",
    "almagraseis": "almagrar",
    "almagrasen": "almagrar",
    "almagrases": "almagrar",
    "almagraste": "almagrar",
    "almagrasteis": "almagrar",
    "almagre": "almagrar",
    "almagremos": "almagrar",
    "almagren": "almagrar",
    "almagrera": "almagrero",
    "almagreras": "almagrero",
    "almagreros": "almagrero",
    "almagres": "almagre",
    "almagro": "almagrar",
    "almagrábamos": "almagrar",
    "almagráis": "almagrar",
    "almagráramos": "almagrar",
    "almagráremos": "almagrar",
    "almagrásemos": "almagrar",
    "almagré": "almagrar",
    "almagréis": "almagrar",
    "almagró": "almagrar",
    "almaizales": "almaizal",
    "almaizares": "almaizar",
    "almalafas": "almalafa",
    "almanaques": "almanaque",
    "almandinas": "almandina",
    "almandinos": "almandino",
    "almantas": "almanta",
    "almaradas": "almarada",
    "almarbata": "almarbatar",
    "almarbataba": "almarbatar",
    "almarbatabais": "almarbatar",
    "almarbataban": "almarbatar",
    "almarbatabas": "almarbatar",
    "almarbatad": "almarbatar",
    "almarbatada": "almarbatar",
    "almarbatadas": "almarbatar",
    "almarbatado": "almarbatar",
    "almarbatados": "almarbatar",
    "almarbatamos": "almarbatar",
    "almarbatan": "almarbatar",
    "almarbatando": "almarbatar",
    "almarbatara": "almarbatar",
    "almarbatarais": "almarbatar",
    "almarbataran": "almarbatar",
    "almarbataras": "almarbatar",
    "almarbatare": "almarbatar",
    "almarbatareis": "almarbatar",
    "almarbataremos": "almarbatar",
    "almarbataren": "almarbatar",
    "almarbatares": "almarbatar",
    "almarbataron": "almarbatar",
    "almarbatará": "almarbatar",
    "almarbatarán": "almarbatar",
    "almarbatarás": "almarbatar",
    "almarbataré": "almarbatar",
    "almarbataréis": "almarbatar",
    "almarbataría": "almarbatar",
    "almarbataríais": "almarbatar",
    "almarbataríamos": "almarbatar",
    "almarbatarían": "almarbatar",
    "almarbatarías": "almarbatar",
    "almarbatas": "almarbatar",
    "almarbatase": "almarbatar",
    "almarbataseis": "almarbatar",
    "almarbatasen": "almarbatar",
    "almarbatases": "almarbatar",
    "almarbataste": "almarbatar",
    "almarbatasteis": "almarbatar",
    "almarbate": "almarbatar",
    "almarbatemos": "almarbatar",
    "almarbaten": "almarbatar",
    "almarbates": "almarbatar",
    "almarbato": "almarbatar",
    "almarbatábamos": "almarbatar",
    "almarbatáis": "almarbatar",
    "almarbatáramos": "almarbatar",
    "almarbatáremos": "almarbatar",
    "almarbatásemos": "almarbatar",
    "almarbaté": "almarbatar",
    "almarbatéis": "almarbatar",
    "almarbató": "almarbatar",
    "almarchas": "almarcha",
    "almargas": "almarga",
    "almarjales": "almarjal",
    "almarjos": "almarjo",
    "almarrajas": "almarraja",
    "almarrás": "almarrá",
    "almas": "alma",
    "almazaras": "almazara",
    "almazarera": "almazarero",
    "almazareras": "almazarero",
    "almazareros": "almazarero",
    "almeces": "almez",
    "almejas": "almeja",
    "almejones": "almejón",
    "almena": "almenar",
    "almenaba": "almenar",
    "almenabais": "almenar",
    "almenaban": "almenar",
    "almenabas": "almenar",
    "almenad": "almenar",
    "almenada": "almenar",
    "almenadas": "almenar",
    "almenado": "almenar",
    "almenados": "almenar",
    "almenajes": "almenaje",
    "almenamos": "almenar",
    "almenan": "almenar",
    "almenando": "almenar",
    "almenara": "almenar",
    "almenarais": "almenar",
    "almenaran": "almenar",
    "almenaras": "almenara",
    "almenare": "almenar",
    "almenareis": "almenar",
    "almenaremos": "almenar",
    "almenaren": "almenar",
    "almenares": "almenar",
    "almenaron": "almenar",
    "almenará": "almenar",
    "almenarán": "almenar",
    "almenarás": "almenar",
    "almenaré": "almenar",
    "almenaréis": "almenar",
    "almenaría": "almenar",
    "almenaríais": "almenar",
    "almenaríamos": "almenar",
    "almenarían": "almenar",
    "almenarías": "almenar",
    "almenas": "almenar",
    "almenase": "almenar",
    "almenaseis": "almenar",
    "almenasen": "almenar",
    "almenases": "almenar",
    "almenaste": "almenar",
    "almenasteis": "almenar",
    "almendrada": "almendrado",
    "almendradas": "almendrado",
    "almendrados": "almendrado",
    "almendras": "almendra",
    "almendrillas": "almendrilla",
    "almendrones": "almendrón",
    "almendros": "almendro",
    "almendrucos": "almendruco",
    "almene": "almenar",
    "almenemos": "almenar",
    "almenen": "almenar",
    "almenes": "almenar",
    "almeno": "almenar",
    "almenábamos": "almenar",
    "almenáis": "almenar",
    "almenáramos": "almenar",
    "almenáremos": "almenar",
    "almenásemos": "almenar",
    "almené": "almenar",
    "almenéis": "almenar",
    "almenó": "almenar",
    "almerienses": "almeriense",
    "almetes": "almete",
    "almiares": "almiar",
    "almibara": "almibarar",
    "almibaraba": "almibarar",
    "almibarabais": "almibarar",
    "almibaraban": "almibarar",
    "almibarabas": "almibarar",
    "almibarad": "almibarar",
    "almibarada": "almibarar",
    "almibaradas": "almibarar",
    "almibarado": "almibarar",
    "almibarados": "almibarar",
    "almibaramos": "almibarar",
    "almibaran": "almibarar",
    "almibarando": "almibarar",
    "almibarara": "almibarar",
    "almibararais": "almibarar",
    "almibararan": "almibarar",
    "almibararas": "almibarar",
    "almibarare": "almibarar",
    "almibarareis": "almibarar",
    "almibararemos": "almibarar",
    "almibararen": "almibarar",
    "almibarares": "almibarar",
    "almibararon": "almibarar",
    "almibarará": "almibarar",
    "almibararán": "almibarar",
    "almibararás": "almibarar",
    "almibararé": "almibarar",
    "almibararéis": "almibarar",
    "almibararía": "almibarar",
    "almibararíais": "almibarar",
    "almibararíamos": "almibarar",
    "almibararían": "almibarar",
    "almibararías": "almibarar",
    "almibaras": "almibarar",
    "almibarase": "almibarar",
    "almibaraseis": "almibarar",
    "almibarasen": "almibarar",
    "almibarases": "almibarar",
    "almibaraste": "almibarar",
    "almibarasteis": "almibarar",
    "almibare": "almibarar",
    "almibaremos": "almibarar",
    "almibaren": "almibarar",
    "almibares": "almibarar",
    "almibaro": "almibarar",
    "almibarábamos": "almibarar",
    "almibaráis": "almibarar",
    "almibaráramos": "almibarar",
    "almibaráremos": "almibarar",
    "almibarásemos": "almibarar",
    "almibaré": "almibarar",
    "almibaréis": "almibarar",
    "almibaró": "almibarar",
    "almicantaradas": "almicantarada",
    "almicantarats": "almicantarat",
    "almidona": "almidonar",
    "almidonaba": "almidonar",
    "almidonabais": "almidonar",
    "almidonaban": "almidonar",
    "almidonabas": "almidonar",
    "almidonad": "almidonar",
    "almidonada": "almidonar",
    "almidonadas": "almidonar",
    "almidonado": "almidonar",
    "almidonados": "almidonar",
    "almidonamos": "almidonar",
    "almidonan": "almidonar",
    "almidonando": "almidonar",
    "almidonara": "almidonar",
    "almidonarais": "almidonar",
    "almidonaran": "almidonar",
    "almidonaras": "almidonar",
    "almidonare": "almidonar",
    "almidonareis": "almidonar",
    "almidonaremos": "almidonar",
    "almidonaren": "almidonar",
    "almidonares": "almidonar",
    "almidonaron": "almidonar",
    "almidonará": "almidonar",
    "almidonarán": "almidonar",
    "almidonarás": "almidonar",
    "almidonaré": "almidonar",
    "almidonaréis": "almidonar",
    "almidonaría": "almidonar",
    "almidonaríais": "almidonar",
    "almidonaríamos": "almidonar",
    "almidonarían": "almidonar",
    "almidonarías": "almidonar",
    "almidonas": "almidonar",
    "almidonase": "almidonar",
    "almidonaseis": "almidonar",
    "almidonasen": "almidonar",
    "almidonases": "almidonar",
    "almidonaste": "almidonar",
    "almidonasteis": "almidonar",
    "almidone": "almidonar",
    "almidonemos": "almidonar",
    "almidonen": "almidonar",
    "almidones": "almidonar",
    "almidono": "almidonar",
    "almidonábamos": "almidonar",
    "almidonáis": "almidonar",
    "almidonáramos": "almidonar",
    "almidonáremos": "almidonar",
    "almidonásemos": "almidonar",
    "almidoné": "almidonar",
    "almidonéis": "almidonar",
    "almidonó": "almidonar",
    "almillas": "almilla",
    "almimbares": "almimbar",
    "alminares": "alminar",
    "almiquíes": "almiquí",
    "almiquís": "almiquí",
    "almirantas": "almiranta",
    "almirantazgos": "almirantazgo",
    "almirantes": "almirante",
    "almireces": "almirez",
    "almizates": "almizate",
    "almizcates": "almizcate",
    "almizcla": "almizclar",
    "almizclaba": "almizclar",
    "almizclabais": "almizclar",
    "almizclaban": "almizclar",
    "almizclabas": "almizclar",
    "almizclad": "almizclar",
    "almizclada": "almizclar",
    "almizcladas": "almizclar",
    "almizclado": "almizclar",
    "almizclados": "almizclar",
    "almizclamos": "almizclar",
    "almizclan": "almizclar",
    "almizclando": "almizclar",
    "almizclara": "almizclar",
    "almizclarais": "almizclar",
    "almizclaran": "almizclar",
    "almizclaras": "almizclar",
    "almizclare": "almizclar",
    "almizclareis": "almizclar",
    "almizclaremos": "almizclar",
    "almizclaren": "almizclar",
    "almizclares": "almizclar",
    "almizclaron": "almizclar",
    "almizclará": "almizclar",
    "almizclarán": "almizclar",
    "almizclarás": "almizclar",
    "almizclaré": "almizclar",
    "almizclaréis": "almizclar",
    "almizclaría": "almizclar",
    "almizclaríais": "almizclar",
    "almizclaríamos": "almizclar",
    "almizclarían": "almizclar",
    "almizclarías": "almizclar",
    "almizclas": "almizclar",
    "almizclase": "almizclar",
    "almizclaseis": "almizclar",
    "almizclasen": "almizclar",
    "almizclases": "almizclar",
    "almizclaste": "almizclar",
    "almizclasteis": "almizclar",
    "almizcle": "almizclar",
    "almizclemos": "almizclar",
    "almizclen": "almizclar",
    "almizclera": "almizclero",
    "almizcleras": "almizclero",
    "almizcleros": "almizclero",
    "almizcles": "almizcle",
    "almizcleña": "almizcleño",
    "almizcleñas": "almizcleño",
    "almizcleños": "almizcleño",
    "almizclo": "almizclar",
    "almizclábamos": "almizclar",
    "almizcláis": "almizclar",
    "almizcláramos": "almizclar",
    "almizcláremos": "almizclar",
    "almizclásemos": "almizclar",
    "almizclé": "almizclar",
    "almizcléis": "almizclar",
    "almizcló": "almizclar",
    "almocafres": "almocafre",
    "almocarbes": "almocarbe",
    "almocatracías": "almocatracía",
    "almocríes": "almocrí",
    "almocrís": "almocrí",
    "almocárabes": "almocárabe",
    "almoflates": "almoflate",
    "almofreces": "almofrez",
    "almogavarea": "almogavarear",
    "almogavareaba": "almogavarear",
    "almogavareabais": "almogavarear",
    "almogavareaban": "almogavarear",
    "almogavareabas": "almogavarear",
    "almogavaread": "almogavarear",
    "almogavareada": "almogavarear",
    "almogavareadas": "almogavarear",
    "almogavareado": "almogavarear",
    "almogavareados": "almogavarear",
    "almogavareamos": "almogavarear",
    "almogavarean": "almogavarear",
    "almogavareando": "almogavarear",
    "almogavareara": "almogavarear",
    "almogavarearais": "almogavarear",
    "almogavarearan": "almogavarear",
    "almogavarearas": "almogavarear",
    "almogavareare": "almogavarear",
    "almogavareareis": "almogavarear",
    "almogavarearemos": "almogavarear",
    "almogavarearen": "almogavarear",
    "almogavareares": "almogavarear",
    "almogavarearon": "almogavarear",
    "almogavareará": "almogavarear",
    "almogavarearán": "almogavarear",
    "almogavarearás": "almogavarear",
    "almogavarearé": "almogavarear",
    "almogavarearéis": "almogavarear",
    "almogavarearía": "almogavarear",
    "almogavarearíais": "almogavarear",
    "almogavarearíamos": "almogavarear",
    "almogavarearían": "almogavarear",
    "almogavarearías": "almogavarear",
    "almogavareas": "almogavarear",
    "almogavarease": "almogavarear",
    "almogavareaseis": "almogavarear",
    "almogavareasen": "almogavarear",
    "almogavareases": "almogavarear",
    "almogavareaste": "almogavarear",
    "almogavareasteis": "almogavarear",
    "almogavaree": "almogavarear",
    "almogavareemos": "almogavarear",
    "almogavareen": "almogavarear",
    "almogavarees": "almogavarear",
    "almogavareo": "almogavarear",
    "almogavareábamos": "almogavarear",
    "almogavareáis": "almogavarear",
    "almogavareáramos": "almogavarear",
    "almogavareáremos": "almogavarear",
    "almogavareásemos": "almogavarear",
    "almogavareé": "almogavarear",
    "almogavareéis": "almogavarear",
    "almogavareó": "almogavarear",
    "almogávares": "almogávar",
    "almohace": "almohazar",
    "almohacemos": "almohazar",
    "almohacen": "almohazar",
    "almohaces": "almohazar",
    "almohacé": "almohazar",
    "almohacéis": "almohazar",
    "almohadas": "almohada",
    "almohades": "almohade",
    "almohadilla": "almohadillar",
    "almohadillaba": "almohadillar",
    "almohadillabais": "almohadillar",
    "almohadillaban": "almohadillar",
    "almohadillabas": "almohadillar",
    "almohadillad": "almohadillar",
    "almohadillada": "almohadillar",
    "almohadilladas": "almohadillar",
    "almohadillado": "almohadillar",
    "almohadillados": "almohadillar",
    "almohadillamos": "almohadillar",
    "almohadillan": "almohadillar",
    "almohadillando": "almohadillar",
    "almohadillara": "almohadillar",
    "almohadillarais": "almohadillar",
    "almohadillaran": "almohadillar",
    "almohadillaras": "almohadillar",
    "almohadillare": "almohadillar",
    "almohadillareis": "almohadillar",
    "almohadillaremos": "almohadillar",
    "almohadillaren": "almohadillar",
    "almohadillares": "almohadillar",
    "almohadillaron": "almohadillar",
    "almohadillará": "almohadillar",
    "almohadillarán": "almohadillar",
    "almohadillarás": "almohadillar",
    "almohadillaré": "almohadillar",
    "almohadillaréis": "almohadillar",
    "almohadillaría": "almohadillar",
    "almohadillaríais": "almohadillar",
    "almohadillaríamos": "almohadillar",
    "almohadillarían": "almohadillar",
    "almohadillarías": "almohadillar",
    "almohadillas": "almohadillar",
    "almohadillase": "almohadillar",
    "almohadillaseis": "almohadillar",
    "almohadillasen": "almohadillar",
    "almohadillases": "almohadillar",
    "almohadillaste": "almohadillar",
    "almohadillasteis": "almohadillar",
    "almohadille": "almohadillar",
    "almohadillemos": "almohadillar",
    "almohadillen": "almohadillar",
    "almohadilles": "almohadillar",
    "almohadillo": "almohadillar",
    "almohadillábamos": "almohadillar",
    "almohadilláis": "almohadillar",
    "almohadilláramos": "almohadillar",
    "almohadilláremos": "almohadillar",
    "almohadillásemos": "almohadillar",
    "almohadillé": "almohadillar",
    "almohadilléis": "almohadillar",
    "almohadilló": "almohadillar",
    "almohadones": "almohadón",
    "almohaza": "almohazar",
    "almohazaba": "almohazar",
    "almohazabais": "almohazar",
    "almohazaban": "almohazar",
    "almohazabas": "almohazar",
    "almohazad": "almohazar",
    "almohazada": "almohazar",
    "almohazadas": "almohazar",
    "almohazado": "almohazar",
    "almohazados": "almohazar",
    "almohazamos": "almohazar",
    "almohazan": "almohazar",
    "almohazando": "almohazar",
    "almohazara": "almohazar",
    "almohazarais": "almohazar",
    "almohazaran": "almohazar",
    "almohazaras": "almohazar",
    "almohazare": "almohazar",
    "almohazareis": "almohazar",
    "almohazaremos": "almohazar",
    "almohazaren": "almohazar",
    "almohazares": "almohazar",
    "almohazaron": "almohazar",
    "almohazará": "almohazar",
    "almohazarán": "almohazar",
    "almohazarás": "almohazar",
    "almohazaré": "almohazar",
    "almohazaréis": "almohazar",
    "almohazaría": "almohazar",
    "almohazaríais": "almohazar",
    "almohazaríamos": "almohazar",
    "almohazarían": "almohazar",
    "almohazarías": "almohazar",
    "almohazas": "almohazar",
    "almohazase": "almohazar",
    "almohazaseis": "almohazar",
    "almohazasen": "almohazar",
    "almohazases": "almohazar",
    "almohazaste": "almohazar",
    "almohazasteis": "almohazar",
    "almohazo": "almohazar",
    "almohazábamos": "almohazar",
    "almohazáis": "almohazar",
    "almohazáramos": "almohazar",
    "almohazáremos": "almohazar",
    "almohazásemos": "almohazar",
    "almohazó": "almohazar",
    "almojarifazgos": "almojarifazgo",
    "almojarifes": "almojarife",
    "almojayas": "almojaya",
    "almonedas": "almoneda",
    "almorcemos": "almorzar",
    "almorcé": "almorzar",
    "almorcéis": "almorzar",
    "almorranas": "almorrana",
    "almortas": "almorta",
    "almorzaba": "almorzar",
    "almorzabais": "almorzar",
    "almorzaban": "almorzar",
    "almorzabas": "almorzar",
    "almorzad": "almorzar",
    "almorzada": "almorzar",
    "almorzadas": "almorzar",
    "almorzado": "almorzar",
    "almorzados": "almorzar",
    "almorzamos": "almorzar",
    "almorzando": "almorzar",
    "almorzara": "almorzar",
    "almorzarais": "almorzar",
    "almorzaran": "almorzar",
    "almorzaras": "almorzar",
    "almorzare": "almorzar",
    "almorzareis": "almorzar",
    "almorzaremos": "almorzar",
    "almorzaren": "almorzar",
    "almorzares": "almorzar",
    "almorzaron": "almorzar",
    "almorzará": "almorzar",
    "almorzarán": "almorzar",
    "almorzarás": "almorzar",
    "almorzaré": "almorzar",
    "almorzaréis": "almorzar",
    "almorzaría": "almorzar",
    "almorzaríais": "almorzar",
    "almorzaríamos": "almorzar",
    "almorzarían": "almorzar",
    "almorzarías": "almorzar",
    "almorzase": "almorzar",
    "almorzaseis": "almorzar",
    "almorzasen": "almorzar",
    "almorzases": "almorzar",
    "almorzaste": "almorzar",
    "almorzasteis": "almorzar",
    "almorzábamos": "almorzar",
    "almorzáis": "almorzar",
    "almorzáramos": "almorzar",
    "almorzáremos": "almorzar",
    "almorzásemos": "almorzar",
    "almorzó": "almorzar",
    "almorávides": "almorávide",
    "almotacenes": "almotacén",
    "almudadas": "almudada",
    "almudes": "almud",
    "almudines": "almudín",
    "almudíes": "almudí",
    "almudís": "almudí",
    "almuecines": "almuecín",
    "almuerce": "almorzar",
    "almuercen": "almorzar",
    "almuerces": "almorzar",
    "almuerza": "almorzar",
    "almuerzan": "almorzar",
    "almuerzas": "almorzar",
    "almuerzo": "almorzar",
    "almuerzos": "almuerzo",
    "almuédanos": "almuédano",
    "almácigas": "almáciga",
    "almácigos": "almácigo",
    "almádenas": "almádena",
    "almíbares": "almíbar",
    "almóndigas": "almóndiga",
    "aloanticuerpos": "aloanticuerpo",
    "aloantisueros": "aloantisuero",
    "aloantígenos": "aloantígeno",
    "alobaras": "alobara",
    "alocada": "alocado",
    "alocadas": "alocado",
    "alocados": "alocado",
    "alocromática": "alocromático",
    "alocromáticas": "alocromático",
    "alocromáticos": "alocromático",
    "alocuciones": "alocución",
    "alodiales": "alodial",
    "alodios": "alodio",
    "aloestérica": "aloestérico",
    "aloestéricas": "aloestérico",
    "aloestéricos": "aloestérico",
    "alofanas": "alofana",
    "alofonías": "alofonía",
    "alofénica": "alofénico",
    "alofénicas": "alofénico",
    "alofénicos": "alofénico",
    "alofónica": "alofónico",
    "alofónicas": "alofónico",
    "alofónicos": "alofónico",
    "alogamias": "alogamia",
    "alogeneica": "alogeneico",
    "alogeneicas": "alogeneico",
    "alogeneicos": "alogeneico",
    "alogenética": "alogenético",
    "alogenéticas": "alogenético",
    "alogenéticos": "alogenético",
    "alogrupos": "alogrupo",
    "alogénica": "alogénico",
    "alogénicas": "alogénico",
    "alogénicos": "alogénico",
    "aloinjertos": "aloinjerto",
    "aloinmunidades": "aloinmunidad",
    "aloinmunizaciones": "aloinmunización",
    "aloisomerismos": "aloisomerismo",
    "aloja": "alojar",
    "alojaba": "alojar",
    "alojabais": "alojar",
    "alojaban": "alojar",
    "alojabas": "alojar",
    "alojad": "alojar",
    "alojada": "alojar",
    "alojadas": "alojar",
    "alojado": "alojar",
    "alojados": "alojar",
    "alojamientos": "alojamiento",
    "alojamos": "alojar",
    "alojan": "alojar",
    "alojando": "alojar",
    "alojara": "alojar",
    "alojarais": "alojar",
    "alojaran": "alojar",
    "alojaras": "alojar",
    "alojare": "alojar",
    "alojareis": "alojar",
    "alojaremos": "alojar",
    "alojaren": "alojar",
    "alojares": "alojar",
    "alojaron": "alojar",
    "alojará": "alojar",
    "alojarán": "alojar",
    "alojarás": "alojar",
    "alojaré": "alojar",
    "alojaréis": "alojar",
    "alojaría": "alojar",
    "alojaríais": "alojar",
    "alojaríamos": "alojar",
    "alojarían": "alojar",
    "alojarías": "alojar",
    "alojas": "alojar",
    "alojase": "alojar",
    "alojaseis": "alojar",
    "alojasen": "alojar",
    "alojases": "alojar",
    "alojaste": "alojar",
    "alojasteis": "alojar",
    "aloje": "alojar",
    "alojemos": "alojar",
    "alojen": "alojar",
    "alojes": "alojar",
    "alojo": "alojar",
    "alojábamos": "alojar",
    "alojáis": "alojar",
    "alojáramos": "alojar",
    "alojáremos": "alojar",
    "alojásemos": "alojar",
    "alojé": "alojar",
    "alojéis": "alojar",
    "alojó": "alojar",
    "aloma": "alomar",
    "alomaba": "alomar",
    "alomabais": "alomar",
    "alomaban": "alomar",
    "alomabas": "alomar",
    "alomad": "alomar",
    "alomada": "alomar",
    "alomadas": "alomar",
    "alomado": "alomar",
    "alomadora": "alomador",
    "alomadoras": "alomador",
    "alomadores": "alomador",
    "alomados": "alomar",
    "alomamos": "alomar",
    "aloman": "alomar",
    "alomando": "alomar",
    "alomara": "alomar",
    "alomarais": "alomar",
    "alomaran": "alomar",
    "alomaras": "alomar",
    "alomare": "alomar",
    "alomareis": "alomar",
    "alomaremos": "alomar",
    "alomaren": "alomar",
    "alomares": "alomar",
    "alomaron": "alomar",
    "alomará": "alomar",
    "alomarán": "alomar",
    "alomarás": "alomar",
    "alomaré": "alomar",
    "alomaréis": "alomar",
    "alomaría": "alomar",
    "alomaríais": "alomar",
    "alomaríamos": "alomar",
    "alomarían": "alomar",
    "alomarías": "alomar",
    "alomas": "alomar",
    "alomase": "alomar",
    "alomaseis": "alomar",
    "alomasen": "alomar",
    "alomases": "alomar",
    "alomaste": "alomar",
    "alomasteis": "alomar",
    "alome": "alomar",
    "alomemos": "alomar",
    "alomen": "alomar",
    "alomes": "alomar",
    "alomo": "alomar",
    "alomorfa": "alomorfo",
    "alomorfas": "alomorfo",
    "alomorfismos": "alomorfismo",
    "alomorfos": "alomorfo",
    "alomorfías": "alomorfía",
    "alomábamos": "alomar",
    "alomáis": "alomar",
    "alomáramos": "alomar",
    "alomáremos": "alomar",
    "alomásemos": "alomar",
    "alomé": "alomar",
    "aloméis": "alomar",
    "alomérica": "alomérico",
    "aloméricas": "alomérico",
    "aloméricos": "alomérico",
    "alomó": "alomar",
    "alomórfica": "alomórfico",
    "alomórficas": "alomórfico",
    "alomórficos": "alomórfico",
    "alondras": "alondra",
    "alones": "alón",
    "alongamientos": "alongamiento",
    "alopatías": "alopatía",
    "alopecias": "alopecia",
    "alopoliploides": "alopoliploide",
    "alopática": "alopático",
    "alopáticas": "alopático",
    "alopáticos": "alopático",
    "alopátrica": "alopátrico",
    "alopátricas": "alopátrico",
    "alopátricos": "alopátrico",
    "aloques": "aloque",
    "alorreactividades": "alorreactividad",
    "alosas": "alosa",
    "alosomas": "alosoma",
    "alota": "alotar",
    "alotaba": "alotar",
    "alotabais": "alotar",
    "alotaban": "alotar",
    "alotabas": "alotar",
    "alotad": "alotar",
    "alotada": "alotar",
    "alotadas": "alotar",
    "alotado": "alotar",
    "alotados": "alotar",
    "alotamos": "alotar",
    "alotan": "alotar",
    "alotando": "alotar",
    "alotara": "alotar",
    "alotarais": "alotar",
    "alotaran": "alotar",
    "alotaras": "alotar",
    "alotare": "alotar",
    "alotareis": "alotar",
    "alotaremos": "alotar",
    "alotaren": "alotar",
    "alotares": "alotar",
    "alotaron": "alotar",
    "alotará": "alotar",
    "alotarán": "alotar",
    "alotarás": "alotar",
    "alotaré": "alotar",
    "alotaréis": "alotar",
    "alotaría": "alotar",
    "alotaríais": "alotar",
    "alotaríamos": "alotar",
    "alotarían": "alotar",
    "alotarías": "alotar",
    "alotas": "alotar",
    "alotase": "alotar",
    "alotaseis": "alotar",
    "alotasen": "alotar",
    "alotases": "alotar",
    "alotaste": "alotar",
    "alotasteis": "alotar",
    "alote": "alotar",
    "alotemos": "alotar",
    "aloten": "alotar",
    "alotes": "alotar",
    "alotipias": "alotipia",
    "alotipos": "alotipo",
    "aloto": "alotar",
    "alotropismos": "alotropismo",
    "alotropías": "alotropía",
    "alotrópica": "alotrópico",
    "alotrópicas": "alotrópico",
    "alotrópicos": "alotrópico",
    "alotábamos": "alotar",
    "alotáis": "alotar",
    "alotáramos": "alotar",
    "alotáremos": "alotar",
    "alotásemos": "alotar",
    "aloté": "alotar",
    "alotéis": "alotar",
    "alotígena": "alotígeno",
    "alotígenas": "alotígeno",
    "alotígenos": "alotígeno",
    "alotípica": "alotípico",
    "alotípicas": "alotípico",
    "alotípicos": "alotípico",
    "alotó": "alotar",
    "aloxacinas": "aloxacina",
    "aloínas": "aloína",
    "alpacas": "alpaca",
    "alpargatas": "alpargata",
    "alpargatera": "alpargatero",
    "alpargateras": "alpargatero",
    "alpargateros": "alpargatero",
    "alpargaterías": "alpargatería",
    "alpechines": "alpechín",
    "alpendes": "alpende",
    "alpenstocks": "alpenstock",
    "alpestres": "alpestre",
    "alpina": "alpino",
    "alpinas": "alpino",
    "alpinismos": "alpinismo",
    "alpinistas": "alpinista",
    "alpinos": "alpino",
    "alpistes": "alpiste",
    "alqueces": "alquez",
    "alquenilos": "alquenilo",
    "alquenos": "alqueno",
    "alquermes": "alquerme",
    "alquerques": "alquerque",
    "alquerías": "alquería",
    "alquiblas": "alquibla",
    "alquila": "alquilar",
    "alquilaba": "alquilar",
    "alquilabais": "alquilar",
    "alquilaban": "alquilar",
    "alquilabas": "alquilar",
    "alquilad": "alquilar",
    "alquilada": "alquilar",
    "alquiladas": "alquilar",
    "alquilado": "alquilar",
    "alquiladora": "alquilador",
    "alquiladoras": "alquilador",
    "alquiladores": "alquilador",
    "alquilados": "alquilar",
    "alquilamos": "alquilar",
    "alquilan": "alquilar",
    "alquilando": "alquilar",
    "alquilara": "alquilar",
    "alquilarais": "alquilar",
    "alquilaran": "alquilar",
    "alquilaras": "alquilar",
    "alquilare": "alquilar",
    "alquilareis": "alquilar",
    "alquilaremos": "alquilar",
    "alquilaren": "alquilar",
    "alquilares": "alquilar",
    "alquilaron": "alquilar",
    "alquilará": "alquilar",
    "alquilarán": "alquilar",
    "alquilarás": "alquilar",
    "alquilaré": "alquilar",
    "alquilaréis": "alquilar",
    "alquilaría": "alquilar",
    "alquilaríais": "alquilar",
    "alquilaríamos": "alquilar",
    "alquilarían": "alquilar",
    "alquilarías": "alquilar",
    "alquilas": "alquilar",
    "alquilase": "alquilar",
    "alquilaseis": "alquilar",
    "alquilasen": "alquilar",
    "alquilases": "alquilar",
    "alquilaste": "alquilar",
    "alquilasteis": "alquilar",
    "alquile": "alquilar",
    "alquilemos": "alquilar",
    "alquilen": "alquilar",
    "alquilenos": "alquileno",
    "alquileres": "alquiler",
    "alquiles": "alquilar",
    "alquilo": "alquilar",
    "alquilos": "alquilo",
    "alquilábamos": "alquilar",
    "alquiláis": "alquilar",
    "alquiláramos": "alquilar",
    "alquiláremos": "alquilar",
    "alquilásemos": "alquilar",
    "alquilé": "alquilar",
    "alquiléis": "alquilar",
    "alquiló": "alquilar",
    "alquimias": "alquimia",
    "alquimistas": "alquimista",
    "alquinos": "alquino",
    "alquitara": "alquitarar",
    "alquitaraba": "alquitarar",
    "alquitarabais": "alquitarar",
    "alquitaraban": "alquitarar",
    "alquitarabas": "alquitarar",
    "alquitarad": "alquitarar",
    "alquitarada": "alquitarar",
    "alquitaradas": "alquitarar",
    "alquitarado": "alquitarar",
    "alquitarados": "alquitarar",
    "alquitaramos": "alquitarar",
    "alquitaran": "alquitarar",
    "alquitarando": "alquitarar",
    "alquitarara": "alquitarar",
    "alquitararais": "alquitarar",
    "alquitararan": "alquitarar",
    "alquitararas": "alquitarar",
    "alquitarare": "alquitarar",
    "alquitarareis": "alquitarar",
    "alquitararemos": "alquitarar",
    "alquitararen": "alquitarar",
    "alquitarares": "alquitarar",
    "alquitararon": "alquitarar",
    "alquitarará": "alquitarar",
    "alquitararán": "alquitarar",
    "alquitararás": "alquitarar",
    "alquitararé": "alquitarar",
    "alquitararéis": "alquitarar",
    "alquitararía": "alquitarar",
    "alquitararíais": "alquitarar",
    "alquitararíamos": "alquitarar",
    "alquitararían": "alquitarar",
    "alquitararías": "alquitarar",
    "alquitaras": "alquitarar",
    "alquitarase": "alquitarar",
    "alquitaraseis": "alquitarar",
    "alquitarasen": "alquitarar",
    "alquitarases": "alquitarar",
    "alquitaraste": "alquitarar",
    "alquitarasteis": "alquitarar",
    "alquitare": "alquitarar",
    "alquitaremos": "alquitarar",
    "alquitaren": "alquitarar",
    "alquitares": "alquitarar",
    "alquitaro": "alquitarar",
    "alquitarábamos": "alquitarar",
    "alquitaráis": "alquitarar",
    "alquitaráramos": "alquitarar",
    "alquitaráremos": "alquitarar",
    "alquitarásemos": "alquitarar",
    "alquitaré": "alquitarar",
    "alquitaréis": "alquitarar",
    "alquitaró": "alquitarar",
    "alquitrana": "alquitranar",
    "alquitranaba": "alquitranar",
    "alquitranabais": "alquitranar",
    "alquitranaban": "alquitranar",
    "alquitranabas": "alquitranar",
    "alquitranad": "alquitranar",
    "alquitranada": "alquitranar",
    "alquitranadas": "alquitranar",
    "alquitranado": "alquitranar",
    "alquitranados": "alquitranar",
    "alquitranamos": "alquitranar",
    "alquitranan": "alquitranar",
    "alquitranando": "alquitranar",
    "alquitranara": "alquitranar",
    "alquitranarais": "alquitranar",
    "alquitranaran": "alquitranar",
    "alquitranaras": "alquitranar",
    "alquitranare": "alquitranar",
    "alquitranareis": "alquitranar",
    "alquitranaremos": "alquitranar",
    "alquitranaren": "alquitranar",
    "alquitranares": "alquitranar",
    "alquitranaron": "alquitranar",
    "alquitranará": "alquitranar",
    "alquitranarán": "alquitranar",
    "alquitranarás": "alquitranar",
    "alquitranaré": "alquitranar",
    "alquitranaréis": "alquitranar",
    "alquitranaría": "alquitranar",
    "alquitranaríais": "alquitranar",
    "alquitranaríamos": "alquitranar",
    "alquitranarían": "alquitranar",
    "alquitranarías": "alquitranar",
    "alquitranas": "alquitranar",
    "alquitranase": "alquitranar",
    "alquitranaseis": "alquitranar",
    "alquitranasen": "alquitranar",
    "alquitranases": "alquitranar",
    "alquitranaste": "alquitranar",
    "alquitranasteis": "alquitranar",
    "alquitrane": "alquitranar",
    "alquitranemos": "alquitranar",
    "alquitranen": "alquitranar",
    "alquitranes": "alquitranar",
    "alquitrano": "alquitranar",
    "alquitranábamos": "alquitranar",
    "alquitranáis": "alquitranar",
    "alquitranáramos": "alquitranar",
    "alquitranáremos": "alquitranar",
    "alquitranásemos": "alquitranar",
    "alquitrané": "alquitranar",
    "alquitranéis": "alquitranar",
    "alquitranó": "alquitranar",
    "alquídica": "alquídico",
    "alquídicas": "alquídico",
    "alquídicos": "alquídico",
    "alquímica": "alquímico",
    "alquímicas": "alquímico",
    "alquímicos": "alquímico",
    "alrededores": "alrededor",
    "alrotas": "alrota",
    "alsaciana": "alsaciano",
    "alsacianas": "alsaciano",
    "alsacianos": "alsaciano",
    "alta": "alto",
    "altaica": "altaico",
    "altaicas": "altaico",
    "altaicoes": "altaico",
    "altaicos": "altaico",
    "altanera": "altanero",
    "altaneras": "altanero",
    "altaneros": "altanero",
    "altanerías": "altanería",
    "altaplanicies": "altaplanicie",
    "altares": "altar",
    "altas": "alto",
    "altavoces": "altavoz",
    "altea": "altear",
    "alteaba": "altear",
    "alteabais": "altear",
    "alteaban": "altear",
    "alteabas": "altear",
    "altead": "altear",
    "alteada": "altear",
    "alteadas": "altear",
    "alteado": "altear",
    "alteados": "altear",
    "alteamos": "altear",
    "altean": "altear",
    "alteando": "altear",
    "alteara": "altear",
    "altearais": "altear",
    "altearan": "altear",
    "altearas": "altear",
    "alteare": "altear",
    "alteareis": "altear",
    "altearemos": "altear",
    "altearen": "altear",
    "alteares": "altear",
    "altearon": "altear",
    "alteará": "altear",
    "altearán": "altear",
    "altearás": "altear",
    "altearé": "altear",
    "altearéis": "altear",
    "altearía": "altear",
    "altearíais": "altear",
    "altearíamos": "altear",
    "altearían": "altear",
    "altearías": "altear",
    "alteas": "altear",
    "altease": "altear",
    "alteaseis": "altear",
    "alteasen": "altear",
    "alteases": "altear",
    "alteaste": "altear",
    "alteasteis": "altear",
    "altee": "altear",
    "alteemos": "altear",
    "alteen": "altear",
    "altees": "altear",
    "alteo": "altear",
    "altera": "alterar",
    "alteraba": "alterar",
    "alterabais": "alterar",
    "alteraban": "alterar",
    "alterabas": "alterar",
    "alterabilidades": "alterabilidad",
    "alterables": "alterable",
    "alteraciones": "alteración",
    "alterad": "alterar",
    "alterada": "alterar",
    "alteradas": "alterar",
    "alterado": "alterar",
    "alteradora": "alterador",
    "alteradoras": "alterador",
    "alteradores": "alterador",
    "alterados": "alterar",
    "alteramos": "alterar",
    "alteran": "alterar",
    "alterando": "alterar",
    "alterara": "alterar",
    "alterarais": "alterar",
    "alteraran": "alterar",
    "alteraras": "alterar",
    "alterare": "alterar",
    "alterareis": "alterar",
    "alteraremos": "alterar",
    "alteraren": "alterar",
    "alterares": "alterar",
    "alteraron": "alterar",
    "alterará": "alterar",
    "alterarán": "alterar",
    "alterarás": "alterar",
    "alteraré": "alterar",
    "alteraréis": "alterar",
    "alteraría": "alterar",
    "alteraríais": "alterar",
    "alteraríamos": "alterar",
    "alterarían": "alterar",
    "alterarías": "alterar",
    "alteras": "alterar",
    "alterase": "alterar",
    "alteraseis": "alterar",
    "alterasen": "alterar",
    "alterases": "alterar",
    "alteraste": "alterar",
    "alterasteis": "alterar",
    "alterca": "altercar",
    "altercaba": "altercar",
    "altercabais": "altercar",
    "altercaban": "altercar",
    "altercabas": "altercar",
    "altercad": "altercar",
    "altercada": "altercar",
    "altercadas": "altercar",
    "altercado": "altercar",
    "altercadora": "altercador",
    "altercadoras": "altercador",
    "altercadores": "altercador",
    "altercados": "altercar",
    "altercamos": "altercar",
    "altercan": "altercar",
    "altercando": "altercar",
    "altercara": "altercar",
    "altercarais": "altercar",
    "altercaran": "altercar",
    "altercaras": "altercar",
    "altercare": "altercar",
    "altercareis": "altercar",
    "altercaremos": "altercar",
    "altercaren": "altercar",
    "altercares": "altercar",
    "altercaron": "altercar",
    "altercará": "altercar",
    "altercarán": "altercar",
    "altercarás": "altercar",
    "altercaré": "altercar",
    "altercaréis": "altercar",
    "altercaría": "altercar",
    "altercaríais": "altercar",
    "altercaríamos": "altercar",
    "altercarían": "altercar",
    "altercarías": "altercar",
    "altercas": "altercar",
    "altercase": "altercar",
    "altercaseis": "altercar",
    "altercasen": "altercar",
    "altercases": "altercar",
    "altercaste": "altercar",
    "altercasteis": "altercar",
    "alterco": "altercar",
    "altercábamos": "altercar",
    "altercáis": "altercar",
    "altercáramos": "altercar",
    "altercáremos": "altercar",
    "altercásemos": "altercar",
    "altercó": "altercar",
    "altere": "alterar",
    "alteremos": "alterar",
    "alteren": "alterar",
    "alteres": "alterar",
    "alteridades": "alteridad",
    "alteritas": "alterita",
    "alterna": "alterno",
    "alternaba": "alternar",
    "alternabais": "alternar",
    "alternaban": "alternar",
    "alternabas": "alternar",
    "alternaciones": "alternación",
    "alternad": "alternar",
    "alternada": "alternar",
    "alternadas": "alternar",
    "alternado": "alternar",
    "alternadores": "alternador",
    "alternados": "alternar",
    "alternamos": "alternar",
    "alternan": "alternar",
    "alternancias": "alternancia",
    "alternando": "alternar",
    "alternantes": "alternante",
    "alternara": "alternar",
    "alternarais": "alternar",
    "alternaran": "alternar",
    "alternaras": "alternar",
    "alternare": "alternar",
    "alternareis": "alternar",
    "alternaremos": "alternar",
    "alternaren": "alternar",
    "alternares": "alternar",
    "alternaron": "alternar",
    "alternará": "alternar",
    "alternarán": "alternar",
    "alternarás": "alternar",
    "alternaré": "alternar",
    "alternaréis": "alternar",
    "alternaría": "alternar",
    "alternaríais": "alternar",
    "alternaríamos": "alternar",
    "alternarían": "alternar",
    "alternarías": "alternar",
    "alternas": "alterno",
    "alternase": "alternar",
    "alternaseis": "alternar",
    "alternasen": "alternar",
    "alternases": "alternar",
    "alternaste": "alternar",
    "alternasteis": "alternar",
    "alternativa": "alternativo",
    "alternativas": "alternativo",
    "alternatividades": "alternatividad",
    "alternativos": "alternativo",
    "alterne": "alternar",
    "alternemos": "alternar",
    "alternen": "alternar",
    "alternes": "alternar",
    "alterno": "alternar",
    "alternos": "alterno",
    "alternábamos": "alternar",
    "alternáis": "alternar",
    "alternáramos": "alternar",
    "alternáremos": "alternar",
    "alternásemos": "alternar",
    "alterné": "alternar",
    "alternéis": "alternar",
    "alternó": "alternar",
    "altero": "alterar",
    "alterque": "altercar",
    "alterquemos": "altercar",
    "alterquen": "altercar",
    "alterques": "altercar",
    "alterqué": "altercar",
    "alterquéis": "altercar",
    "alterábamos": "alterar",
    "alteráis": "alterar",
    "alteráramos": "alterar",
    "alteráremos": "alterar",
    "alterásemos": "alterar",
    "alteré": "alterar",
    "alteréis": "alterar",
    "alteró": "alterar",
    "altezas": "alteza",
    "alteábamos": "altear",
    "alteáis": "altear",
    "alteáramos": "altear",
    "alteáremos": "altear",
    "alteásemos": "altear",
    "alteé": "altear",
    "alteéis": "altear",
    "alteó": "altear",
    "altibajos": "altibajo",
    "altillanos": "altillano",
    "altillanuras": "altillanura",
    "altillos": "altillo",
    "altilocuencias": "altilocuencia",
    "altilocuentes": "altilocuente",
    "altimetrías": "altimetría",
    "altimétrica": "altimétrico",
    "altimétricas": "altimétrico",
    "altimétricos": "altimétrico",
    "altipampas": "altipampa",
    "altiplanaciones": "altiplanación",
    "altiplanicies": "altiplanicie",
    "altiplanos": "altiplano",
    "altisonancias": "altisonancia",
    "altisonantes": "altisonante",
    "altitudes": "altitud",
    "altitudinales": "altitudinal",
    "altiva": "altivo",
    "altivaba": "altivar",
    "altivabais": "altivar",
    "altivaban": "altivar",
    "altivabas": "altivar",
    "altivad": "altivar",
    "altivada": "altivar",
    "altivadas": "altivar",
    "altivado": "altivar",
    "altivados": "altivar",
    "altivamos": "altivar",
    "altivan": "altivar",
    "altivando": "altivar",
    "altivara": "altivar",
    "altivarais": "altivar",
    "altivaran": "altivar",
    "altivaras": "altivar",
    "altivare": "altivar",
    "altivareis": "altivar",
    "altivaremos": "altivar",
    "altivaren": "altivar",
    "altivares": "altivar",
    "altivaron": "altivar",
    "altivará": "altivar",
    "altivarán": "altivar",
    "altivarás": "altivar",
    "altivaré": "altivar",
    "altivaréis": "altivar",
    "altivaría": "altivar",
    "altivaríais": "altivar",
    "altivaríamos": "altivar",
    "altivarían": "altivar",
    "altivarías": "altivar",
    "altivas": "altivo",
    "altivase": "altivar",
    "altivaseis": "altivar",
    "altivasen": "altivar",
    "altivases": "altivar",
    "altivaste": "altivar",
    "altivasteis": "altivar",
    "altive": "altivar",
    "altivece": "altivecer",
    "altiveced": "altivecer",
    "altivecemos": "altivecer",
    "altivecen": "altivecer",
    "altiveceremos": "altivecer",
    "altivecerá": "altivecer",
    "altivecerán": "altivecer",
    "altivecerás": "altivecer",
    "altiveceré": "altivecer",
    "altiveceréis": "altivecer",
    "altivecería": "altivecer",
    "altiveceríais": "altivecer",
    "altiveceríamos": "altivecer",
    "altivecerían": "altivecer",
    "altivecerías": "altivecer",
    "altiveces": "altivez",
    "altivecida": "altivecer",
    "altivecidas": "altivecer",
    "altivecido": "altivecer",
    "altivecidos": "altivecer",
    "altiveciendo": "altivecer",
    "altiveciera": "altivecer",
    "altivecierais": "altivecer",
    "altivecieran": "altivecer",
    "altivecieras": "altivecer",
    "altiveciere": "altivecer",
    "altiveciereis": "altivecer",
    "altivecieren": "altivecer",
    "altivecieres": "altivecer",
    "altivecieron": "altivecer",
    "altiveciese": "altivecer",
    "altivecieseis": "altivecer",
    "altiveciesen": "altivecer",
    "altivecieses": "altivecer",
    "altivecimos": "altivecer",
    "altiveciste": "altivecer",
    "altivecisteis": "altivecer",
    "altiveciéramos": "altivecer",
    "altiveciéremos": "altivecer",
    "altiveciésemos": "altivecer",
    "altiveció": "altivecer",
    "altivecéis": "altivecer",
    "altivecí": "altivecer",
    "altivecía": "altivecer",
    "altivecíais": "altivecer",
    "altivecíamos": "altivecer",
    "altivecían": "altivecer",
    "altivecías": "altivecer",
    "altivemos": "altivar",
    "altiven": "altivar",
    "altives": "altivar",
    "altivezca": "altivecer",
    "altivezcamos": "altivecer",
    "altivezcan": "altivecer",
    "altivezcas": "altivecer",
    "altivezco": "altivecer",
    "altivezcáis": "altivecer",
    "altivo": "altivar",
    "altivos": "altivo",
    "altivábamos": "altivar",
    "altiváis": "altivar",
    "altiváramos": "altivar",
    "altiváremos": "altivar",
    "altivásemos": "altivar",
    "altivé": "altivar",
    "altivéis": "altivar",
    "altivó": "altivar",
    "altizidas": "altizida",
    "altocúmulos": "altocúmulo",
    "altoestratos": "altoestrato",
    "altoparlantes": "altoparlante",
    "altos": "alto",
    "altostratos": "altostrato",
    "altozanos": "altozano",
    "altramuces": "altramuz",
    "altrosas": "altrosa",
    "altruismos": "altruismo",
    "altruistas": "altruista",
    "altruísmos": "altruísmo",
    "alturas": "altura",
    "altímetros": "altímetro",
    "altísona": "altísono",
    "altísonas": "altísono",
    "altísonos": "altísono",
    "alubias": "alubia",
    "alucia": "aluciar",
    "aluciaba": "aluciar",
    "aluciabais": "aluciar",
    "aluciaban": "aluciar",
    "aluciabas": "aluciar",
    "aluciad": "aluciar",
    "aluciada": "aluciar",
    "aluciadas": "aluciar",
    "aluciado": "aluciar",
    "aluciados": "aluciar",
    "aluciamos": "aluciar",
    "alucian": "aluciar",
    "aluciando": "aluciar",
    "aluciara": "aluciar",
    "aluciarais": "aluciar",
    "aluciaran": "aluciar",
    "aluciaras": "aluciar",
    "aluciare": "aluciar",
    "aluciareis": "aluciar",
    "aluciaremos": "aluciar",
    "aluciaren": "aluciar",
    "aluciares": "aluciar",
    "aluciaron": "aluciar",
    "aluciará": "aluciar",
    "aluciarán": "aluciar",
    "aluciarás": "aluciar",
    "aluciaré": "aluciar",
    "aluciaréis": "aluciar",
    "aluciaría": "aluciar",
    "aluciaríais": "aluciar",
    "aluciaríamos": "aluciar",
    "aluciarían": "aluciar",
    "aluciarías": "aluciar",
    "alucias": "aluciar",
    "aluciase": "aluciar",
    "aluciaseis": "aluciar",
    "aluciasen": "aluciar",
    "aluciases": "aluciar",
    "aluciaste": "aluciar",
    "aluciasteis": "aluciar",
    "alucie": "aluciar",
    "aluciemos": "aluciar",
    "alucien": "aluciar",
    "alucies": "aluciar",
    "alucina": "alucinar",
    "alucinaba": "alucinar",
    "alucinabais": "alucinar",
    "alucinaban": "alucinar",
    "alucinabas": "alucinar",
    "alucinaciones": "alucinación",
    "alucinad": "alucinar",
    "alucinada": "alucinar",
    "alucinadas": "alucinar",
    "alucinado": "alucinar",
    "alucinadora": "alucinador",
    "alucinadoras": "alucinador",
    "alucinadores": "alucinador",
    "alucinados": "alucinar",
    "alucinamientos": "alucinamiento",
    "alucinamos": "alucinar",
    "alucinan": "alucinar",
    "alucinando": "alucinar",
    "alucinantes": "alucinante",
    "alucinara": "alucinar",
    "alucinarais": "alucinar",
    "alucinaran": "alucinar",
    "alucinaras": "alucinar",
    "alucinare": "alucinar",
    "alucinareis": "alucinar",
    "alucinaremos": "alucinar",
    "alucinaren": "alucinar",
    "alucinares": "alucinar",
    "alucinaron": "alucinar",
    "alucinará": "alucinar",
    "alucinarán": "alucinar",
    "alucinarás": "alucinar",
    "alucinaré": "alucinar",
    "alucinaréis": "alucinar",
    "alucinaría": "alucinar",
    "alucinaríais": "alucinar",
    "alucinaríamos": "alucinar",
    "alucinarían": "alucinar",
    "alucinarías": "alucinar",
    "alucinas": "alucinar",
    "alucinase": "alucinar",
    "alucinaseis": "alucinar",
    "alucinasen": "alucinar",
    "alucinases": "alucinar",
    "alucinaste": "alucinar",
    "alucinasteis": "alucinar",
    "alucinatoria": "alucinatorio",
    "alucinatorias": "alucinatorio",
    "alucinatorios": "alucinatorio",
    "alucine": "alucinar",
    "alucinemos": "alucinar",
    "alucinen": "alucinar",
    "alucines": "alucinar",
    "alucino": "alucinar",
    "alucinábamos": "alucinar",
    "alucináis": "alucinar",
    "alucináramos": "alucinar",
    "alucináremos": "alucinar",
    "alucinásemos": "alucinar",
    "aluciné": "alucinar",
    "alucinéis": "alucinar",
    "alucinó": "alucinar",
    "alucinógena": "alucinógeno",
    "alucinógenas": "alucinógeno",
    "alucinógenos": "alucinógeno",
    "alucio": "aluciar",
    "aluciábamos": "aluciar",
    "aluciáis": "aluciar",
    "aluciáramos": "aluciar",
    "aluciáremos": "aluciar",
    "aluciásemos": "aluciar",
    "alucié": "aluciar",
    "aluciéis": "aluciar",
    "alució": "aluciar",
    "aluda": "aludir",
    "aludamos": "aludir",
    "aludan": "aludir",
    "aludas": "aludir",
    "alude": "aludir",
    "aluden": "aludir",
    "aludes": "aludir",
    "aludid": "aludir",
    "aludida": "aludir",
    "aludidas": "aludir",
    "aludido": "aludir",
    "aludidos": "aludir",
    "aludiendo": "aludir",
    "aludiera": "aludir",
    "aludierais": "aludir",
    "aludieran": "aludir",
    "aludieras": "aludir",
    "aludiere": "aludir",
    "aludiereis": "aludir",
    "aludieren": "aludir",
    "aludieres": "aludir",
    "aludieron": "aludir",
    "aludiese": "aludir",
    "aludieseis": "aludir",
    "aludiesen": "aludir",
    "aludieses": "aludir",
    "aludimos": "aludir",
    "aludiremos": "aludir",
    "aludirá": "aludir",
    "aludirán": "aludir",
    "aludirás": "aludir",
    "aludiré": "aludir",
    "aludiréis": "aludir",
    "aludiría": "aludir",
    "aludiríais": "aludir",
    "aludiríamos": "aludir",
    "aludirían": "aludir",
    "aludirías": "aludir",
    "aludiste": "aludir",
    "aludisteis": "aludir",
    "aludiéramos": "aludir",
    "aludiéremos": "aludir",
    "aludiésemos": "aludir",
    "aludió": "aludir",
    "aludo": "aludir",
    "aludáis": "aludir",
    "aludí": "aludir",
    "aludía": "aludir",
    "aludíais": "aludir",
    "aludíamos": "aludir",
    "aludían": "aludir",
    "aludías": "aludir",
    "aludís": "aludir",
    "aluja": "alujar",
    "alujaba": "alujar",
    "alujabais": "alujar",
    "alujaban": "alujar",
    "alujabas": "alujar",
    "alujad": "alujar",
    "alujada": "alujar",
    "alujadas": "alujar",
    "alujado": "alujar",
    "alujados": "alujar",
    "alujamos": "alujar",
    "alujan": "alujar",
    "alujando": "alujar",
    "alujara": "alujar",
    "alujarais": "alujar",
    "alujaran": "alujar",
    "alujaras": "alujar",
    "alujare": "alujar",
    "alujareis": "alujar",
    "alujaremos": "alujar",
    "alujaren": "alujar",
    "alujares": "alujar",
    "alujaron": "alujar",
    "alujará": "alujar",
    "alujarán": "alujar",
    "alujarás": "alujar",
    "alujaré": "alujar",
    "alujaréis": "alujar",
    "alujaría": "alujar",
    "alujaríais": "alujar",
    "alujaríamos": "alujar",
    "alujarían": "alujar",
    "alujarías": "alujar",
    "alujas": "alujar",
    "alujase": "alujar",
    "alujaseis": "alujar",
    "alujasen": "alujar",
    "alujases": "alujar",
    "alujaste": "alujar",
    "alujasteis": "alujar",
    "aluje": "alujar",
    "alujemos": "alujar",
    "alujen": "alujar",
    "alujes": "alujar",
    "alujo": "alujar",
    "alujábamos": "alujar",
    "alujáis": "alujar",
    "alujáramos": "alujar",
    "alujáremos": "alujar",
    "alujásemos": "alujar",
    "alujé": "alujar",
    "alujéis": "alujar",
    "alujó": "alujar",
    "alumbra": "alumbrar",
    "alumbraba": "alumbrar",
    "alumbrabais": "alumbrar",
    "alumbraban": "alumbrar",
    "alumbrabas": "alumbrar",
    "alumbrad": "alumbrar",
    "alumbrada": "alumbrar",
    "alumbradas": "alumbrar",
    "alumbrado": "alumbrar",
    "alumbrados": "alumbrar",
    "alumbramientos": "alumbramiento",
    "alumbramos": "alumbrar",
    "alumbran": "alumbrar",
    "alumbrando": "alumbrar",
    "alumbrara": "alumbrar",
    "alumbrarais": "alumbrar",
    "alumbraran": "alumbrar",
    "alumbraras": "alumbrar",
    "alumbrare": "alumbrar",
    "alumbrareis": "alumbrar",
    "alumbraremos": "alumbrar",
    "alumbraren": "alumbrar",
    "alumbrares": "alumbrar",
    "alumbraron": "alumbrar",
    "alumbrará": "alumbrar",
    "alumbrarán": "alumbrar",
    "alumbrarás": "alumbrar",
    "alumbraré": "alumbrar",
    "alumbraréis": "alumbrar",
    "alumbraría": "alumbrar",
    "alumbraríais": "alumbrar",
    "alumbraríamos": "alumbrar",
    "alumbrarían": "alumbrar",
    "alumbrarías": "alumbrar",
    "alumbras": "alumbrar",
    "alumbrase": "alumbrar",
    "alumbraseis": "alumbrar",
    "alumbrasen": "alumbrar",
    "alumbrases": "alumbrar",
    "alumbraste": "alumbrar",
    "alumbrasteis": "alumbrar",
    "alumbre": "alumbrar",
    "alumbremos": "alumbrar",
    "alumbren": "alumbrar",
    "alumbres": "alumbre",
    "alumbro": "alumbrar",
    "alumbrábamos": "alumbrar",
    "alumbráis": "alumbrar",
    "alumbráramos": "alumbrar",
    "alumbráremos": "alumbrar",
    "alumbrásemos": "alumbrar",
    "alumbré": "alumbrar",
    "alumbréis": "alumbrar",
    "alumbró": "alumbrar",
    "alumes": "alum",
    "aluminados": "aluminado",
    "aluminatos": "aluminato",
    "aluminios": "aluminio",
    "aluminitas": "aluminita",
    "aluminizaciones": "aluminización",
    "aluminosa": "aluminoso",
    "aluminosas": "aluminoso",
    "aluminosos": "aluminoso",
    "aluminotermias": "aluminotermia",
    "aluminotérmica": "aluminotérmico",
    "aluminotérmicas": "aluminotérmico",
    "aluminotérmicos": "aluminotérmico",
    "aluminurados": "aluminurado",
    "alumna": "alumno",
    "alumnados": "alumnado",
    "alumnas": "alumno",
    "alumnos": "alumno",
    "aluna": "alunar",
    "alunaba": "alunar",
    "alunabais": "alunar",
    "alunaban": "alunar",
    "alunabas": "alunar",
    "alunad": "alunar",
    "alunada": "alunar",
    "alunadas": "alunar",
    "alunado": "alunar",
    "alunados": "alunar",
    "alunamos": "alunar",
    "alunan": "alunar",
    "alunando": "alunar",
    "alunara": "alunar",
    "alunarais": "alunar",
    "alunaran": "alunar",
    "alunaras": "alunar",
    "alunare": "alunar",
    "alunareis": "alunar",
    "alunaremos": "alunar",
    "alunaren": "alunar",
    "alunares": "alunar",
    "alunaron": "alunar",
    "alunará": "alunar",
    "alunarán": "alunar",
    "alunarás": "alunar",
    "alunaré": "alunar",
    "alunaréis": "alunar",
    "alunaría": "alunar",
    "alunaríais": "alunar",
    "alunaríamos": "alunar",
    "alunarían": "alunar",
    "alunarías": "alunar",
    "alunas": "alunar",
    "alunase": "alunar",
    "alunaseis": "alunar",
    "alunasen": "alunar",
    "alunases": "alunar",
    "alunaste": "alunar",
    "alunasteis": "alunar",
    "alune": "alunar",
    "alunemos": "alunar",
    "alunen": "alunar",
    "alunes": "alunar",
    "alunice": "alunizar",
    "alunicemos": "alunizar",
    "alunicen": "alunizar",
    "alunices": "alunizar",
    "alunicé": "alunizar",
    "alunicéis": "alunizar",
    "alunitas": "alunita",
    "aluniza": "alunizar",
    "alunizaba": "alunizar",
    "alunizabais": "alunizar",
    "alunizaban": "alunizar",
    "alunizabas": "alunizar",
    "alunizad": "alunizar",
    "alunizada": "alunizar",
    "alunizadas": "alunizar",
    "alunizado": "alunizar",
    "alunizados": "alunizar",
    "alunizajes": "alunizaje",
    "alunizamos": "alunizar",
    "alunizan": "alunizar",
    "alunizando": "alunizar",
    "alunizara": "alunizar",
    "alunizarais": "alunizar",
    "alunizaran": "alunizar",
    "alunizaras": "alunizar",
    "alunizare": "alunizar",
    "alunizareis": "alunizar",
    "alunizaremos": "alunizar",
    "alunizaren": "alunizar",
    "alunizares": "alunizar",
    "alunizaron": "alunizar",
    "alunizará": "alunizar",
    "alunizarán": "alunizar",
    "alunizarás": "alunizar",
    "alunizaré": "alunizar",
    "alunizaréis": "alunizar",
    "alunizaría": "alunizar",
    "alunizaríais": "alunizar",
    "alunizaríamos": "alunizar",
    "alunizarían": "alunizar",
    "alunizarías": "alunizar",
    "alunizas": "alunizar",
    "alunizase": "alunizar",
    "alunizaseis": "alunizar",
    "alunizasen": "alunizar",
    "alunizases": "alunizar",
    "alunizaste": "alunizar",
    "alunizasteis": "alunizar",
    "alunizo": "alunizar",
    "alunizábamos": "alunizar",
    "alunizáis": "alunizar",
    "alunizáramos": "alunizar",
    "alunizáremos": "alunizar",
    "alunizásemos": "alunizar",
    "alunizó": "alunizar",
    "aluno": "alunar",
    "alunábamos": "alunar",
    "alunáis": "alunar",
    "alunáramos": "alunar",
    "alunáremos": "alunar",
    "alunásemos": "alunar",
    "aluné": "alunar",
    "alunéis": "alunar",
    "alunó": "alunar",
    "alusiones": "alusión",
    "alusiva": "alusivo",
    "alusivas": "alusivo",
    "alusivos": "alusivo",
    "alustra": "alustrar",
    "alustraba": "alustrar",
    "alustrabais": "alustrar",
    "alustraban": "alustrar",
    "alustrabas": "alustrar",
    "alustrad": "alustrar",
    "alustrada": "alustrar",
    "alustradas": "alustrar",
    "alustrado": "alustrar",
    "alustrados": "alustrar",
    "alustramos": "alustrar",
    "alustran": "alustrar",
    "alustrando": "alustrar",
    "alustrara": "alustrar",
    "alustrarais": "alustrar",
    "alustraran": "alustrar",
    "alustraras": "alustrar",
    "alustrare": "alustrar",
    "alustrareis": "alustrar",
    "alustraremos": "alustrar",
    "alustraren": "alustrar",
    "alustrares": "alustrar",
    "alustraron": "alustrar",
    "alustrará": "alustrar",
    "alustrarán": "alustrar",
    "alustrarás": "alustrar",
    "alustraré": "alustrar",
    "alustraréis": "alustrar",
    "alustraría": "alustrar",
    "alustraríais": "alustrar",
    "alustraríamos": "alustrar",
    "alustrarían": "alustrar",
    "alustrarías": "alustrar",
    "alustras": "alustrar",
    "alustrase": "alustrar",
    "alustraseis": "alustrar",
    "alustrasen": "alustrar",
    "alustrases": "alustrar",
    "alustraste": "alustrar",
    "alustrasteis": "alustrar",
    "alustre": "alustrar",
    "alustremos": "alustrar",
    "alustren": "alustrar",
    "alustres": "alustrar",
    "alustro": "alustrar",
    "alustrábamos": "alustrar",
    "alustráis": "alustrar",
    "alustráramos": "alustrar",
    "alustráremos": "alustrar",
    "alustrásemos": "alustrar",
    "alustré": "alustrar",
    "alustréis": "alustrar",
    "alustró": "alustrar",
    "aluviales": "aluvial",
    "aluvionamientos": "aluvionamiento",
    "aluviones": "aluvión",
    "alveolaciones": "alveolación",
    "alveolalgias": "alveolalgia",
    "alveolares": "alveolar",
    "alveolectomías": "alveolectomía",
    "alveoloclasias": "alveoloclasia",
    "alveolodentales": "alveolodental",
    "alveolodentaria": "alveolodentario",
    "alveolodentarias": "alveolodentario",
    "alveolodentarios": "alveolodentario",
    "alveologingivales": "alveologingival",
    "alveololinguales": "alveololingual",
    "alveolomerotomías": "alveolomerotomía",
    "alveoloplastias": "alveoloplastia",
    "alveolos": "alveolo",
    "alveolotomías": "alveolotomía",
    "alvéolos": "alvéolo",
    "alza": "alzar",
    "alzaba": "alzar",
    "alzabais": "alzar",
    "alzaban": "alzar",
    "alzabas": "alzar",
    "alzacolas": "alzacola",
    "alzacuellos": "alzacuello",
    "alzad": "alzar",
    "alzada": "alzar",
    "alzadas": "alzar",
    "alzado": "alzar",
    "alzados": "alzar",
    "alzamientos": "alzamiento",
    "alzamos": "alzar",
    "alzan": "alzar",
    "alzando": "alzar",
    "alzapaños": "alzapaño",
    "alzaprima": "alzaprimar",
    "alzaprimaba": "alzaprimar",
    "alzaprimabais": "alzaprimar",
    "alzaprimaban": "alzaprimar",
    "alzaprimabas": "alzaprimar",
    "alzaprimad": "alzaprimar",
    "alzaprimada": "alzaprimar",
    "alzaprimadas": "alzaprimar",
    "alzaprimado": "alzaprimar",
    "alzaprimados": "alzaprimar",
    "alzaprimamos": "alzaprimar",
    "alzapriman": "alzaprimar",
    "alzaprimando": "alzaprimar",
    "alzaprimara": "alzaprimar",
    "alzaprimarais": "alzaprimar",
    "alzaprimaran": "alzaprimar",
    "alzaprimaras": "alzaprimar",
    "alzaprimare": "alzaprimar",
    "alzaprimareis": "alzaprimar",
    "alzaprimaremos": "alzaprimar",
    "alzaprimaren": "alzaprimar",
    "alzaprimares": "alzaprimar",
    "alzaprimaron": "alzaprimar",
    "alzaprimará": "alzaprimar",
    "alzaprimarán": "alzaprimar",
    "alzaprimarás": "alzaprimar",
    "alzaprimaré": "alzaprimar",
    "alzaprimaréis": "alzaprimar",
    "alzaprimaría": "alzaprimar",
    "alzaprimaríais": "alzaprimar",
    "alzaprimaríamos": "alzaprimar",
    "alzaprimarían": "alzaprimar",
    "alzaprimarías": "alzaprimar",
    "alzaprimas": "alzaprimar",
    "alzaprimase": "alzaprimar",
    "alzaprimaseis": "alzaprimar",
    "alzaprimasen": "alzaprimar",
    "alzaprimases": "alzaprimar",
    "alzaprimaste": "alzaprimar",
    "alzaprimasteis": "alzaprimar",
    "alzaprime": "alzaprimar",
    "alzaprimemos": "alzaprimar",
    "alzaprimen": "alzaprimar",
    "alzaprimes": "alzaprimar",
    "alzaprimo": "alzaprimar",
    "alzaprimábamos": "alzaprimar",
    "alzaprimáis": "alzaprimar",
    "alzaprimáramos": "alzaprimar",
    "alzaprimáremos": "alzaprimar",
    "alzaprimásemos": "alzaprimar",
    "alzaprimé": "alzaprimar",
    "alzapriméis": "alzaprimar",
    "alzaprimó": "alzaprimar",
    "alzara": "alzar",
    "alzarais": "alzar",
    "alzaran": "alzar",
    "alzaras": "alzar",
    "alzare": "alzar",
    "alzareis": "alzar",
    "alzaremos": "alzar",
    "alzaren": "alzar",
    "alzares": "alzar",
    "alzaron": "alzar",
    "alzará": "alzar",
    "alzarán": "alzar",
    "alzarás": "alzar",
    "alzaré": "alzar",
    "alzaréis": "alzar",
    "alzaría": "alzar",
    "alzaríais": "alzar",
    "alzaríamos": "alzar",
    "alzarían": "alzar",
    "alzarías": "alzar",
    "alzas": "alzar",
    "alzase": "alzar",
    "alzaseis": "alzar",
    "alzasen": "alzar",
    "alzases": "alzar",
    "alzaste": "alzar",
    "alzasteis": "alzar",
    "alzo": "alzar",
    "alzos": "alzo",
    "alzábamos": "alzar",
    "alzáis": "alzar",
    "alzáramos": "alzar",
    "alzáremos": "alzar",
    "alzásemos": "alzar",
    "alzó": "alzar",
    "alámbrica": "alámbrico",
    "alámbricas": "alámbrico",
    "alámbricos": "alámbrico",
    "aláteres": "alátere",
    "aláudidos": "aláudido",
    "alélica": "alélico",
    "alélicas": "alélico",
    "alélicos": "alélico",
    "alérgica": "alérgico",
    "alérgicas": "alérgico",
    "alérgicos": "alérgico",
    "alía": "aliar",
    "alían": "aliar",
    "alías": "aliar",
    "alícuotas": "alícuota",
    "alíe": "aliar",
    "alíen": "aliar",
    "alíes": "aliar",
    "alílica": "alílico",
    "alílicas": "alílico",
    "alílicos": "alílico",
    "alío": "aliar",
    "alítica": "alítico",
    "alíticas": "alítico",
    "alíticos": "alítico",
    "alóctona": "alóctono",
    "alóctonas": "alóctono",
    "alóctonos": "alóctono",
    "alófonos": "alófono",
    "alógena": "alógeno",
    "alógenas": "alógeno",
    "alógenos": "alógeno",
    "alópatas": "alópata",
    "alótropos": "alótropo",
    "alúminas": "alúmina",
    "ama": "amar",
    "amaba": "amar",
    "amabais": "amar",
    "amaban": "amar",
    "amabas": "amar",
    "amabilidades": "amabilidad",
    "amables": "amable",
    "amacayos": "amacayo",
    "amachambra": "amachambrar",
    "amachambraba": "amachambrar",
    "amachambrabais": "amachambrar",
    "amachambraban": "amachambrar",
    "amachambrabas": "amachambrar",
    "amachambrad": "amachambrar",
    "amachambrada": "amachambrar",
    "amachambradas": "amachambrar",
    "amachambrado": "amachambrar",
    "amachambrados": "amachambrar",
    "amachambramos": "amachambrar",
    "amachambran": "amachambrar",
    "amachambrando": "amachambrar",
    "amachambrara": "amachambrar",
    "amachambrarais": "amachambrar",
    "amachambraran": "amachambrar",
    "amachambraras": "amachambrar",
    "amachambrare": "amachambrar",
    "amachambrareis": "amachambrar",
    "amachambraremos": "amachambrar",
    "amachambraren": "amachambrar",
    "amachambrares": "amachambrar",
    "amachambraron": "amachambrar",
    "amachambrará": "amachambrar",
    "amachambrarán": "amachambrar",
    "amachambrarás": "amachambrar",
    "amachambraré": "amachambrar",
    "amachambraréis": "amachambrar",
    "amachambraría": "amachambrar",
    "amachambraríais": "amachambrar",
    "amachambraríamos": "amachambrar",
    "amachambrarían": "amachambrar",
    "amachambrarías": "amachambrar",
    "amachambras": "amachambrar",
    "amachambrase": "amachambrar",
    "amachambraseis": "amachambrar",
    "amachambrasen": "amachambrar",
    "amachambrases": "amachambrar",
    "amachambraste": "amachambrar",
    "amachambrasteis": "amachambrar",
    "amachambre": "amachambrar",
    "amachambremos": "amachambrar",
    "amachambren": "amachambrar",
    "amachambres": "amachambrar",
    "amachambro": "amachambrar",
    "amachambrábamos": "amachambrar",
    "amachambráis": "amachambrar",
    "amachambráramos": "amachambrar",
    "amachambráremos": "amachambrar",
    "amachambrásemos": "amachambrar",
    "amachambré": "amachambrar",
    "amachambréis": "amachambrar",
    "amachambró": "amachambrar",
    "amachetea": "amachetear",
    "amacheteaba": "amachetear",
    "amacheteabais": "amachetear",
    "amacheteaban": "amachetear",
    "amacheteabas": "amachetear",
    "amachetead": "amachetear",
    "amacheteada": "amachetear",
    "amacheteadas": "amachetear",
    "amacheteado": "amachetear",
    "amacheteados": "amachetear",
    "amacheteamos": "amachetear",
    "amachetean": "amachetear",
    "amacheteando": "amachetear",
    "amacheteara": "amachetear",
    "amachetearais": "amachetear",
    "amachetearan": "amachetear",
    "amachetearas": "amachetear",
    "amacheteare": "amachetear",
    "amacheteareis": "amachetear",
    "amachetearemos": "amachetear",
    "amachetearen": "amachetear",
    "amacheteares": "amachetear",
    "amachetearon": "amachetear",
    "amacheteará": "amachetear",
    "amachetearán": "amachetear",
    "amachetearás": "amachetear",
    "amachetearé": "amachetear",
    "amachetearéis": "amachetear",
    "amachetearía": "amachetear",
    "amachetearíais": "amachetear",
    "amachetearíamos": "amachetear",
    "amachetearían": "amachetear",
    "amachetearías": "amachetear",
    "amacheteas": "amachetear",
    "amachetease": "amachetear",
    "amacheteaseis": "amachetear",
    "amacheteasen": "amachetear",
    "amacheteases": "amachetear",
    "amacheteaste": "amachetear",
    "amacheteasteis": "amachetear",
    "amachetee": "amachetear",
    "amacheteemos": "amachetear",
    "amacheteen": "amachetear",
    "amachetees": "amachetear",
    "amacheteo": "amachetear",
    "amacheteábamos": "amachetear",
    "amacheteáis": "amachetear",
    "amacheteáramos": "amachetear",
    "amacheteáremos": "amachetear",
    "amacheteásemos": "amachetear",
    "amacheteé": "amachetear",
    "amacheteéis": "amachetear",
    "amacheteó": "amachetear",
    "amachina": "amachinar",
    "amachinaba": "amachinar",
    "amachinabais": "amachinar",
    "amachinaban": "amachinar",
    "amachinabas": "amachinar",
    "amachinad": "amachinar",
    "amachinada": "amachinar",
    "amachinadas": "amachinar",
    "amachinado": "amachinar",
    "amachinados": "amachinar",
    "amachinamos": "amachinar",
    "amachinan": "amachinar",
    "amachinando": "amachinar",
    "amachinara": "amachinar",
    "amachinarais": "amachinar",
    "amachinaran": "amachinar",
    "amachinaras": "amachinar",
    "amachinare": "amachinar",
    "amachinareis": "amachinar",
    "amachinaremos": "amachinar",
    "amachinaren": "amachinar",
    "amachinares": "amachinar",
    "amachinaron": "amachinar",
    "amachinará": "amachinar",
    "amachinarán": "amachinar",
    "amachinarás": "amachinar",
    "amachinaré": "amachinar",
    "amachinaréis": "amachinar",
    "amachinaría": "amachinar",
    "amachinaríais": "amachinar",
    "amachinaríamos": "amachinar",
    "amachinarían": "amachinar",
    "amachinarías": "amachinar",
    "amachinas": "amachinar",
    "amachinase": "amachinar",
    "amachinaseis": "amachinar",
    "amachinasen": "amachinar",
    "amachinases": "amachinar",
    "amachinaste": "amachinar",
    "amachinasteis": "amachinar",
    "amachine": "amachinar",
    "amachinemos": "amachinar",
    "amachinen": "amachinar",
    "amachines": "amachinar",
    "amachino": "amachinar",
    "amachinábamos": "amachinar",
    "amachináis": "amachinar",
    "amachináramos": "amachinar",
    "amachináremos": "amachinar",
    "amachinásemos": "amachinar",
    "amachiné": "amachinar",
    "amachinéis": "amachinar",
    "amachinó": "amachinar",
    "amacice": "amacizar",
    "amacicemos": "amacizar",
    "amacicen": "amacizar",
    "amacices": "amacizar",
    "amacicé": "amacizar",
    "amacicéis": "amacizar",
    "amaciones": "amación",
    "amaciza": "amacizar",
    "amacizaba": "amacizar",
    "amacizabais": "amacizar",
    "amacizaban": "amacizar",
    "amacizabas": "amacizar",
    "amacizad": "amacizar",
    "amacizada": "amacizar",
    "amacizadas": "amacizar",
    "amacizado": "amacizar",
    "amacizados": "amacizar",
    "amacizamos": "amacizar",
    "amacizan": "amacizar",
    "amacizando": "amacizar",
    "amacizara": "amacizar",
    "amacizarais": "amacizar",
    "amacizaran": "amacizar",
    "amacizaras": "amacizar",
    "amacizare": "amacizar",
    "amacizareis": "amacizar",
    "amacizaremos": "amacizar",
    "amacizaren": "amacizar",
    "amacizares": "amacizar",
    "amacizaron": "amacizar",
    "amacizará": "amacizar",
    "amacizarán": "amacizar",
    "amacizarás": "amacizar",
    "amacizaré": "amacizar",
    "amacizaréis": "amacizar",
    "amacizaría": "amacizar",
    "amacizaríais": "amacizar",
    "amacizaríamos": "amacizar",
    "amacizarían": "amacizar",
    "amacizarías": "amacizar",
    "amacizas": "amacizar",
    "amacizase": "amacizar",
    "amacizaseis": "amacizar",
    "amacizasen": "amacizar",
    "amacizases": "amacizar",
    "amacizaste": "amacizar",
    "amacizasteis": "amacizar",
    "amacizo": "amacizar",
    "amacizábamos": "amacizar",
    "amacizáis": "amacizar",
    "amacizáramos": "amacizar",
    "amacizáremos": "amacizar",
    "amacizásemos": "amacizar",
    "amacizó": "amacizar",
    "amacolla": "amacollar",
    "amacollaba": "amacollar",
    "amacollabais": "amacollar",
    "amacollaban": "amacollar",
    "amacollabas": "amacollar",
    "amacollad": "amacollar",
    "amacollada": "amacollar",
    "amacolladas": "amacollar",
    "amacollado": "amacollar",
    "amacollados": "amacollar",
    "amacollamos": "amacollar",
    "amacollan": "amacollar",
    "amacollando": "amacollar",
    "amacollara": "amacollar",
    "amacollarais": "amacollar",
    "amacollaran": "amacollar",
    "amacollaras": "amacollar",
    "amacollare": "amacollar",
    "amacollareis": "amacollar",
    "amacollaremos": "amacollar",
    "amacollaren": "amacollar",
    "amacollares": "amacollar",
    "amacollaron": "amacollar",
    "amacollará": "amacollar",
    "amacollarán": "amacollar",
    "amacollarás": "amacollar",
    "amacollaré": "amacollar",
    "amacollaréis": "amacollar",
    "amacollaría": "amacollar",
    "amacollaríais": "amacollar",
    "amacollaríamos": "amacollar",
    "amacollarían": "amacollar",
    "amacollarías": "amacollar",
    "amacollas": "amacollar",
    "amacollase": "amacollar",
    "amacollaseis": "amacollar",
    "amacollasen": "amacollar",
    "amacollases": "amacollar",
    "amacollaste": "amacollar",
    "amacollasteis": "amacollar",
    "amacolle": "amacollar",
    "amacollemos": "amacollar",
    "amacollen": "amacollar",
    "amacolles": "amacollar",
    "amacollo": "amacollar",
    "amacollábamos": "amacollar",
    "amacolláis": "amacollar",
    "amacolláramos": "amacollar",
    "amacolláremos": "amacollar",
    "amacollásemos": "amacollar",
    "amacollé": "amacollar",
    "amacolléis": "amacollar",
    "amacolló": "amacollar",
    "amad": "amar",
    "amada": "amar",
    "amadas": "amar",
    "amadingas": "amadinga",
    "amado": "amar",
    "amados": "amar",
    "amadriga": "amadrigar",
    "amadrigaba": "amadrigar",
    "amadrigabais": "amadrigar",
    "amadrigaban": "amadrigar",
    "amadrigabas": "amadrigar",
    "amadrigad": "amadrigar",
    "amadrigada": "amadrigar",
    "amadrigadas": "amadrigar",
    "amadrigado": "amadrigar",
    "amadrigados": "amadrigar",
    "amadrigamos": "amadrigar",
    "amadrigan": "amadrigar",
    "amadrigando": "amadrigar",
    "amadrigara": "amadrigar",
    "amadrigarais": "amadrigar",
    "amadrigaran": "amadrigar",
    "amadrigaras": "amadrigar",
    "amadrigare": "amadrigar",
    "amadrigareis": "amadrigar",
    "amadrigaremos": "amadrigar",
    "amadrigaren": "amadrigar",
    "amadrigares": "amadrigar",
    "amadrigaron": "amadrigar",
    "amadrigará": "amadrigar",
    "amadrigarán": "amadrigar",
    "amadrigarás": "amadrigar",
    "amadrigaré": "amadrigar",
    "amadrigaréis": "amadrigar",
    "amadrigaría": "amadrigar",
    "amadrigaríais": "amadrigar",
    "amadrigaríamos": "amadrigar",
    "amadrigarían": "amadrigar",
    "amadrigarías": "amadrigar",
    "amadrigas": "amadrigar",
    "amadrigase": "amadrigar",
    "amadrigaseis": "amadrigar",
    "amadrigasen": "amadrigar",
    "amadrigases": "amadrigar",
    "amadrigaste": "amadrigar",
    "amadrigasteis": "amadrigar",
    "amadrigo": "amadrigar",
    "amadrigue": "amadrigar",
    "amadriguemos": "amadrigar",
    "amadriguen": "amadrigar",
    "amadrigues": "amadrigar",
    "amadrigué": "amadrigar",
    "amadriguéis": "amadrigar",
    "amadrigábamos": "amadrigar",
    "amadrigáis": "amadrigar",
    "amadrigáramos": "amadrigar",
    "amadrigáremos": "amadrigar",
    "amadrigásemos": "amadrigar",
    "amadrigó": "amadrigar",
    "amadrina": "amadrinar",
    "amadrinaba": "amadrinar",
    "amadrinabais": "amadrinar",
    "amadrinaban": "amadrinar",
    "amadrinabas": "amadrinar",
    "amadrinad": "amadrinar",
    "amadrinada": "amadrinar",
    "amadrinadas": "amadrinar",
    "amadrinado": "amadrinar",
    "amadrinados": "amadrinar",
    "amadrinamientos": "amadrinamiento",
    "amadrinamos": "amadrinar",
    "amadrinan": "amadrinar",
    "amadrinando": "amadrinar",
    "amadrinara": "amadrinar",
    "amadrinarais": "amadrinar",
    "amadrinaran": "amadrinar",
    "amadrinaras": "amadrinar",
    "amadrinare": "amadrinar",
    "amadrinareis": "amadrinar",
    "amadrinaremos": "amadrinar",
    "amadrinaren": "amadrinar",
    "amadrinares": "amadrinar",
    "amadrinaron": "amadrinar",
    "amadrinará": "amadrinar",
    "amadrinarán": "amadrinar",
    "amadrinarás": "amadrinar",
    "amadrinaré": "amadrinar",
    "amadrinaréis": "amadrinar",
    "amadrinaría": "amadrinar",
    "amadrinaríais": "amadrinar",
    "amadrinaríamos": "amadrinar",
    "amadrinarían": "amadrinar",
    "amadrinarías": "amadrinar",
    "amadrinas": "amadrinar",
    "amadrinase": "amadrinar",
    "amadrinaseis": "amadrinar",
    "amadrinasen": "amadrinar",
    "amadrinases": "amadrinar",
    "amadrinaste": "amadrinar",
    "amadrinasteis": "amadrinar",
    "amadrine": "amadrinar",
    "amadrinemos": "amadrinar",
    "amadrinen": "amadrinar",
    "amadrines": "amadrinar",
    "amadrino": "amadrinar",
    "amadrinábamos": "amadrinar",
    "amadrináis": "amadrinar",
    "amadrináramos": "amadrinar",
    "amadrináremos": "amadrinar",
    "amadrinásemos": "amadrinar",
    "amadriné": "amadrinar",
    "amadrinéis": "amadrinar",
    "amadrinó": "amadrinar",
    "amaestra": "amaestrar",
    "amaestraba": "amaestrar",
    "amaestrabais": "amaestrar",
    "amaestraban": "amaestrar",
    "amaestrabas": "amaestrar",
    "amaestrad": "amaestrar",
    "amaestrada": "amaestrar",
    "amaestradas": "amaestrar",
    "amaestrado": "amaestrar",
    "amaestrados": "amaestrar",
    "amaestramientos": "amaestramiento",
    "amaestramos": "amaestrar",
    "amaestran": "amaestrar",
    "amaestrando": "amaestrar",
    "amaestrara": "amaestrar",
    "amaestrarais": "amaestrar",
    "amaestraran": "amaestrar",
    "amaestraras": "amaestrar",
    "amaestrare": "amaestrar",
    "amaestrareis": "amaestrar",
    "amaestraremos": "amaestrar",
    "amaestraren": "amaestrar",
    "amaestrares": "amaestrar",
    "amaestraron": "amaestrar",
    "amaestrará": "amaestrar",
    "amaestrarán": "amaestrar",
    "amaestrarás": "amaestrar",
    "amaestraré": "amaestrar",
    "amaestraréis": "amaestrar",
    "amaestraría": "amaestrar",
    "amaestraríais": "amaestrar",
    "amaestraríamos": "amaestrar",
    "amaestrarían": "amaestrar",
    "amaestrarías": "amaestrar",
    "amaestras": "amaestrar",
    "amaestrase": "amaestrar",
    "amaestraseis": "amaestrar",
    "amaestrasen": "amaestrar",
    "amaestrases": "amaestrar",
    "amaestraste": "amaestrar",
    "amaestrasteis": "amaestrar",
    "amaestre": "amaestrar",
    "amaestremos": "amaestrar",
    "amaestren": "amaestrar",
    "amaestres": "amaestrar",
    "amaestro": "amaestrar",
    "amaestrábamos": "amaestrar",
    "amaestráis": "amaestrar",
    "amaestráramos": "amaestrar",
    "amaestráremos": "amaestrar",
    "amaestrásemos": "amaestrar",
    "amaestré": "amaestrar",
    "amaestréis": "amaestrar",
    "amaestró": "amaestrar",
    "amaga": "amagar",
    "amagaba": "amagar",
    "amagabais": "amagar",
    "amagaban": "amagar",
    "amagabas": "amagar",
    "amagad": "amagar",
    "amagada": "amagar",
    "amagadas": "amagar",
    "amagado": "amagar",
    "amagados": "amagar",
    "amagaduras": "amagadura",
    "amagamos": "amagar",
    "amagan": "amagar",
    "amagando": "amagar",
    "amagara": "amagar",
    "amagarais": "amagar",
    "amagaran": "amagar",
    "amagaras": "amagar",
    "amagare": "amagar",
    "amagareis": "amagar",
    "amagaremos": "amagar",
    "amagaren": "amagar",
    "amagares": "amagar",
    "amagaron": "amagar",
    "amagará": "amagar",
    "amagarán": "amagar",
    "amagarás": "amagar",
    "amagaré": "amagar",
    "amagaréis": "amagar",
    "amagaría": "amagar",
    "amagaríais": "amagar",
    "amagaríamos": "amagar",
    "amagarían": "amagar",
    "amagarías": "amagar",
    "amagas": "amagar",
    "amagase": "amagar",
    "amagaseis": "amagar",
    "amagasen": "amagar",
    "amagases": "amagar",
    "amagaste": "amagar",
    "amagasteis": "amagar",
    "amagnética": "amagnético",
    "amagnéticas": "amagnético",
    "amagnéticos": "amagnético",
    "amago": "amagar",
    "amagos": "amago",
    "amague": "amagar",
    "amaguemos": "amagar",
    "amaguen": "amagar",
    "amagues": "amagar",
    "amagué": "amagar",
    "amaguéis": "amagar",
    "amagábamos": "amagar",
    "amagáis": "amagar",
    "amagáramos": "amagar",
    "amagáremos": "amagar",
    "amagásemos": "amagar",
    "amagó": "amagar",
    "amaina": "amainar",
    "amainaba": "amainar",
    "amainabais": "amainar",
    "amainaban": "amainar",
    "amainabas": "amainar",
    "amainad": "amainar",
    "amainada": "amainar",
    "amainadas": "amainar",
    "amainado": "amainar",
    "amainados": "amainar",
    "amainamos": "amainar",
    "amainan": "amainar",
    "amainando": "amainar",
    "amainara": "amainar",
    "amainarais": "amainar",
    "amainaran": "amainar",
    "amainaras": "amainar",
    "amainare": "amainar",
    "amainareis": "amainar",
    "amainaremos": "amainar",
    "amainaren": "amainar",
    "amainares": "amainar",
    "amainaron": "amainar",
    "amainará": "amainar",
    "amainarán": "amainar",
    "amainarás": "amainar",
    "amainaré": "amainar",
    "amainaréis": "amainar",
    "amainaría": "amainar",
    "amainaríais": "amainar",
    "amainaríamos": "amainar",
    "amainarían": "amainar",
    "amainarías": "amainar",
    "amainas": "amainar",
    "amainase": "amainar",
    "amainaseis": "amainar",
    "amainasen": "amainar",
    "amainases": "amainar",
    "amainaste": "amainar",
    "amainasteis": "amainar",
    "amaine": "amainar",
    "amainemos": "amainar",
    "amainen": "amainar",
    "amaines": "amainar",
    "amaino": "amainar",
    "amainábamos": "amainar",
    "amaináis": "amainar",
    "amaináramos": "amainar",
    "amaináremos": "amainar",
    "amainásemos": "amainar",
    "amainé": "amainar",
    "amainéis": "amainar",
    "amainó": "amainar",
    "amajada": "amajadar",
    "amajadaba": "amajadar",
    "amajadabais": "amajadar",
    "amajadaban": "amajadar",
    "amajadabas": "amajadar",
    "amajadad": "amajadar",
    "amajadada": "amajadar",
    "amajadadas": "amajadar",
    "amajadado": "amajadar",
    "amajadados": "amajadar",
    "amajadamos": "amajadar",
    "amajadan": "amajadar",
    "amajadando": "amajadar",
    "amajadara": "amajadar",
    "amajadarais": "amajadar",
    "amajadaran": "amajadar",
    "amajadaras": "amajadar",
    "amajadare": "amajadar",
    "amajadareis": "amajadar",
    "amajadaremos": "amajadar",
    "amajadaren": "amajadar",
    "amajadares": "amajadar",
    "amajadaron": "amajadar",
    "amajadará": "amajadar",
    "amajadarán": "amajadar",
    "amajadarás": "amajadar",
    "amajadaré": "amajadar",
    "amajadaréis": "amajadar",
    "amajadaría": "amajadar",
    "amajadaríais": "amajadar",
    "amajadaríamos": "amajadar",
    "amajadarían": "amajadar",
    "amajadarías": "amajadar",
    "amajadas": "amajadar",
    "amajadase": "amajadar",
    "amajadaseis": "amajadar",
    "amajadasen": "amajadar",
    "amajadases": "amajadar",
    "amajadaste": "amajadar",
    "amajadasteis": "amajadar",
    "amajade": "amajadar",
    "amajademos": "amajadar",
    "amajaden": "amajadar",
    "amajades": "amajadar",
    "amajado": "amajadar",
    "amajadábamos": "amajadar",
    "amajadáis": "amajadar",
    "amajadáramos": "amajadar",
    "amajadáremos": "amajadar",
    "amajadásemos": "amajadar",
    "amajadé": "amajadar",
    "amajadéis": "amajadar",
    "amajadó": "amajadar",
    "amalaya": "amalayar",
    "amalayaba": "amalayar",
    "amalayabais": "amalayar",
    "amalayaban": "amalayar",
    "amalayabas": "amalayar",
    "amalayad": "amalayar",
    "amalayada": "amalayar",
    "amalayadas": "amalayar",
    "amalayado": "amalayar",
    "amalayados": "amalayar",
    "amalayamos": "amalayar",
    "amalayan": "amalayar",
    "amalayando": "amalayar",
    "amalayara": "amalayar",
    "amalayarais": "amalayar",
    "amalayaran": "amalayar",
    "amalayaras": "amalayar",
    "amalayare": "amalayar",
    "amalayareis": "amalayar",
    "amalayaremos": "amalayar",
    "amalayaren": "amalayar",
    "amalayares": "amalayar",
    "amalayaron": "amalayar",
    "amalayará": "amalayar",
    "amalayarán": "amalayar",
    "amalayarás": "amalayar",
    "amalayaré": "amalayar",
    "amalayaréis": "amalayar",
    "amalayaría": "amalayar",
    "amalayaríais": "amalayar",
    "amalayaríamos": "amalayar",
    "amalayarían": "amalayar",
    "amalayarías": "amalayar",
    "amalayas": "amalayar",
    "amalayase": "amalayar",
    "amalayaseis": "amalayar",
    "amalayasen": "amalayar",
    "amalayases": "amalayar",
    "amalayaste": "amalayar",
    "amalayasteis": "amalayar",
    "amalaye": "amalayar",
    "amalayemos": "amalayar",
    "amalayen": "amalayar",
    "amalayes": "amalayar",
    "amalayo": "amalayar",
    "amalayábamos": "amalayar",
    "amalayáis": "amalayar",
    "amalayáramos": "amalayar",
    "amalayáremos": "amalayar",
    "amalayásemos": "amalayar",
    "amalayé": "amalayar",
    "amalayéis": "amalayar",
    "amalayó": "amalayar",
    "amalgama": "amalgamar",
    "amalgamaba": "amalgamar",
    "amalgamabais": "amalgamar",
    "amalgamaban": "amalgamar",
    "amalgamabas": "amalgamar",
    "amalgamaciones": "amalgamación",
    "amalgamad": "amalgamar",
    "amalgamada": "amalgamar",
    "amalgamadas": "amalgamar",
    "amalgamado": "amalgamar",
    "amalgamadores": "amalgamador",
    "amalgamados": "amalgamar",
    "amalgamamientos": "amalgamamiento",
    "amalgamamos": "amalgamar",
    "amalgaman": "amalgamar",
    "amalgamando": "amalgamar",
    "amalgamara": "amalgamar",
    "amalgamarais": "amalgamar",
    "amalgamaran": "amalgamar",
    "amalgamaras": "amalgamar",
    "amalgamare": "amalgamar",
    "amalgamareis": "amalgamar",
    "amalgamaremos": "amalgamar",
    "amalgamaren": "amalgamar",
    "amalgamares": "amalgamar",
    "amalgamaron": "amalgamar",
    "amalgamará": "amalgamar",
    "amalgamarán": "amalgamar",
    "amalgamarás": "amalgamar",
    "amalgamaré": "amalgamar",
    "amalgamaréis": "amalgamar",
    "amalgamaría": "amalgamar",
    "amalgamaríais": "amalgamar",
    "amalgamaríamos": "amalgamar",
    "amalgamarían": "amalgamar",
    "amalgamarías": "amalgamar",
    "amalgamas": "amalgamar",
    "amalgamase": "amalgamar",
    "amalgamaseis": "amalgamar",
    "amalgamasen": "amalgamar",
    "amalgamases": "amalgamar",
    "amalgamaste": "amalgamar",
    "amalgamasteis": "amalgamar",
    "amalgame": "amalgamar",
    "amalgamemos": "amalgamar",
    "amalgamen": "amalgamar",
    "amalgames": "amalgamar",
    "amalgamo": "amalgamar",
    "amalgamábamos": "amalgamar",
    "amalgamáis": "amalgamar",
    "amalgamáramos": "amalgamar",
    "amalgamáremos": "amalgamar",
    "amalgamásemos": "amalgamar",
    "amalgamé": "amalgamar",
    "amalgaméis": "amalgamar",
    "amalgamó": "amalgamar",
    "amamanta": "amamantar",
    "amamantaba": "amamantar",
    "amamantabais": "amamantar",
    "amamantaban": "amamantar",
    "amamantabas": "amamantar",
    "amamantad": "amamantar",
    "amamantada": "amamantar",
    "amamantadas": "amamantar",
    "amamantado": "amamantar",
    "amamantados": "amamantar",
    "amamantamientos": "amamantamiento",
    "amamantamos": "amamantar",
    "amamantan": "amamantar",
    "amamantando": "amamantar",
    "amamantara": "amamantar",
    "amamantarais": "amamantar",
    "amamantaran": "amamantar",
    "amamantaras": "amamantar",
    "amamantare": "amamantar",
    "amamantareis": "amamantar",
    "amamantaremos": "amamantar",
    "amamantaren": "amamantar",
    "amamantares": "amamantar",
    "amamantaron": "amamantar",
    "amamantará": "amamantar",
    "amamantarán": "amamantar",
    "amamantarás": "amamantar",
    "amamantaré": "amamantar",
    "amamantaréis": "amamantar",
    "amamantaría": "amamantar",
    "amamantaríais": "amamantar",
    "amamantaríamos": "amamantar",
    "amamantarían": "amamantar",
    "amamantarías": "amamantar",
    "amamantas": "amamantar",
    "amamantase": "amamantar",
    "amamantaseis": "amamantar",
    "amamantasen": "amamantar",
    "amamantases": "amamantar",
    "amamantaste": "amamantar",
    "amamantasteis": "amamantar",
    "amamante": "amamantar",
    "amamantemos": "amamantar",
    "amamanten": "amamantar",
    "amamantes": "amamantar",
    "amamanto": "amamantar",
    "amamantábamos": "amamantar",
    "amamantáis": "amamantar",
    "amamantáramos": "amamantar",
    "amamantáremos": "amamantar",
    "amamantásemos": "amamantar",
    "amamanté": "amamantar",
    "amamantéis": "amamantar",
    "amamantó": "amamantar",
    "amamos": "amar",
    "aman": "amar",
    "amanceba": "amancebar",
    "amancebaba": "amancebar",
    "amancebabais": "amancebar",
    "amancebaban": "amancebar",
    "amancebabas": "amancebar",
    "amancebad": "amancebar",
    "amancebada": "amancebar",
    "amancebadas": "amancebar",
    "amancebado": "amancebar",
    "amancebados": "amancebar",
    "amancebamientos": "amancebamiento",
    "amancebamos": "amancebar",
    "amanceban": "amancebar",
    "amancebando": "amancebar",
    "amancebara": "amancebar",
    "amancebarais": "amancebar",
    "amancebaran": "amancebar",
    "amancebaras": "amancebar",
    "amancebare": "amancebar",
    "amancebareis": "amancebar",
    "amancebaremos": "amancebar",
    "amancebaren": "amancebar",
    "amancebares": "amancebar",
    "amancebaron": "amancebar",
    "amancebará": "amancebar",
    "amancebarán": "amancebar",
    "amancebarás": "amancebar",
    "amancebaré": "amancebar",
    "amancebaréis": "amancebar",
    "amancebaría": "amancebar",
    "amancebaríais": "amancebar",
    "amancebaríamos": "amancebar",
    "amancebarían": "amancebar",
    "amancebarías": "amancebar",
    "amancebas": "amancebar",
    "amancebase": "amancebar",
    "amancebaseis": "amancebar",
    "amancebasen": "amancebar",
    "amancebases": "amancebar",
    "amancebaste": "amancebar",
    "amancebasteis": "amancebar",
    "amancebe": "amancebar",
    "amancebemos": "amancebar",
    "amanceben": "amancebar",
    "amancebes": "amancebar",
    "amancebo": "amancebar",
    "amancebábamos": "amancebar",
    "amancebáis": "amancebar",
    "amancebáramos": "amancebar",
    "amancebáremos": "amancebar",
    "amancebásemos": "amancebar",
    "amancebé": "amancebar",
    "amancebéis": "amancebar",
    "amancebó": "amancebar",
    "amancilla": "amancillar",
    "amancillaba": "amancillar",
    "amancillabais": "amancillar",
    "amancillaban": "amancillar",
    "amancillabas": "amancillar",
    "amancillad": "amancillar",
    "amancillada": "amancillar",
    "amancilladas": "amancillar",
    "amancillado": "amancillar",
    "amancillados": "amancillar",
    "amancillamos": "amancillar",
    "amancillan": "amancillar",
    "amancillando": "amancillar",
    "amancillara": "amancillar",
    "amancillarais": "amancillar",
    "amancillaran": "amancillar",
    "amancillaras": "amancillar",
    "amancillare": "amancillar",
    "amancillareis": "amancillar",
    "amancillaremos": "amancillar",
    "amancillaren": "amancillar",
    "amancillares": "amancillar",
    "amancillaron": "amancillar",
    "amancillará": "amancillar",
    "amancillarán": "amancillar",
    "amancillarás": "amancillar",
    "amancillaré": "amancillar",
    "amancillaréis": "amancillar",
    "amancillaría": "amancillar",
    "amancillaríais": "amancillar",
    "amancillaríamos": "amancillar",
    "amancillarían": "amancillar",
    "amancillarías": "amancillar",
    "amancillas": "amancillar",
    "amancillase": "amancillar",
    "amancillaseis": "amancillar",
    "amancillasen": "amancillar",
    "amancillases": "amancillar",
    "amancillaste": "amancillar",
    "amancillasteis": "amancillar",
    "amancille": "amancillar",
    "amancillemos": "amancillar",
    "amancillen": "amancillar",
    "amancilles": "amancillar",
    "amancillo": "amancillar",
    "amancillábamos": "amancillar",
    "amancilláis": "amancillar",
    "amancilláramos": "amancillar",
    "amancilláremos": "amancillar",
    "amancillásemos": "amancillar",
    "amancillé": "amancillar",
    "amancilléis": "amancillar",
    "amancilló": "amancillar",
    "amando": "amar",
    "amanece": "amanecer",
    "amaneced": "amanecer",
    "amanecemos": "amanecer",
    "amanecen": "amanecer",
    "amaneceremos": "amanecer",
    "amaneceres": "amanecer",
    "amanecerá": "amanecer",
    "amanecerán": "amanecer",
    "amanecerás": "amanecer",
    "amaneceré": "amanecer",
    "amaneceréis": "amanecer",
    "amanecería": "amanecer",
    "amaneceríais": "amanecer",
    "amaneceríamos": "amanecer",
    "amanecerían": "amanecer",
    "amanecerías": "amanecer",
    "amaneces": "amanecer",
    "amanecida": "amanecer",
    "amanecidas": "amanecida",
    "amanecido": "amanecer",
    "amanecidos": "amanecer",
    "amaneciendo": "amanecer",
    "amaneciera": "amanecer",
    "amanecierais": "amanecer",
    "amanecieran": "amanecer",
    "amanecieras": "amanecer",
    "amaneciere": "amanecer",
    "amaneciereis": "amanecer",
    "amanecieren": "amanecer",
    "amanecieres": "amanecer",
    "amanecieron": "amanecer",
    "amaneciese": "amanecer",
    "amanecieseis": "amanecer",
    "amaneciesen": "amanecer",
    "amanecieses": "amanecer",
    "amanecimos": "amanecer",
    "amaneciste": "amanecer",
    "amanecisteis": "amanecer",
    "amaneciéramos": "amanecer",
    "amaneciéremos": "amanecer",
    "amaneciésemos": "amanecer",
    "amaneció": "amanecer",
    "amanecéis": "amanecer",
    "amanecí": "amanecer",
    "amanecía": "amanecer",
    "amanecíais": "amanecer",
    "amanecíamos": "amanecer",
    "amanecían": "amanecer",
    "amanecías": "amanecer",
    "amanera": "amanerar",
    "amaneraba": "amanerar",
    "amanerabais": "amanerar",
    "amaneraban": "amanerar",
    "amanerabas": "amanerar",
    "amanerad": "amanerar",
    "amanerada": "amanerar",
    "amaneradas": "amanerar",
    "amanerado": "amanerar",
    "amanerados": "amanerar",
    "amaneramientos": "amaneramiento",
    "amaneramos": "amanerar",
    "amaneran": "amanerar",
    "amanerando": "amanerar",
    "amanerara": "amanerar",
    "amanerarais": "amanerar",
    "amaneraran": "amanerar",
    "amaneraras": "amanerar",
    "amanerare": "amanerar",
    "amanerareis": "amanerar",
    "amaneraremos": "amanerar",
    "amaneraren": "amanerar",
    "amanerares": "amanerar",
    "amaneraron": "amanerar",
    "amanerará": "amanerar",
    "amanerarán": "amanerar",
    "amanerarás": "amanerar",
    "amaneraré": "amanerar",
    "amaneraréis": "amanerar",
    "amaneraría": "amanerar",
    "amaneraríais": "amanerar",
    "amaneraríamos": "amanerar",
    "amanerarían": "amanerar",
    "amanerarías": "amanerar",
    "amaneras": "amanerar",
    "amanerase": "amanerar",
    "amaneraseis": "amanerar",
    "amanerasen": "amanerar",
    "amanerases": "amanerar",
    "amaneraste": "amanerar",
    "amanerasteis": "amanerar",
    "amanere": "amanerar",
    "amaneremos": "amanerar",
    "amaneren": "amanerar",
    "amaneres": "amanerar",
    "amanero": "amanerar",
    "amanerábamos": "amanerar",
    "amaneráis": "amanerar",
    "amaneráramos": "amanerar",
    "amaneráremos": "amanerar",
    "amanerásemos": "amanerar",
    "amaneré": "amanerar",
    "amaneréis": "amanerar",
    "amaneró": "amanerar",
    "amanes": "amán",
    "amanezca": "amanecer",
    "amanezcamos": "amanecer",
    "amanezcan": "amanecer",
    "amanezcas": "amanecer",
    "amanezco": "amanecer",
    "amanezcáis": "amanecer",
    "amanoja": "amanojar",
    "amanojaba": "amanojar",
    "amanojabais": "amanojar",
    "amanojaban": "amanojar",
    "amanojabas": "amanojar",
    "amanojad": "amanojar",
    "amanojada": "amanojar",
    "amanojadas": "amanojar",
    "amanojado": "amanojar",
    "amanojados": "amanojar",
    "amanojamos": "amanojar",
    "amanojan": "amanojar",
    "amanojando": "amanojar",
    "amanojara": "amanojar",
    "amanojarais": "amanojar",
    "amanojaran": "amanojar",
    "amanojaras": "amanojar",
    "amanojare": "amanojar",
    "amanojareis": "amanojar",
    "amanojaremos": "amanojar",
    "amanojaren": "amanojar",
    "amanojares": "amanojar",
    "amanojaron": "amanojar",
    "amanojará": "amanojar",
    "amanojarán": "amanojar",
    "amanojarás": "amanojar",
    "amanojaré": "amanojar",
    "amanojaréis": "amanojar",
    "amanojaría": "amanojar",
    "amanojaríais": "amanojar",
    "amanojaríamos": "amanojar",
    "amanojarían": "amanojar",
    "amanojarías": "amanojar",
    "amanojas": "amanojar",
    "amanojase": "amanojar",
    "amanojaseis": "amanojar",
    "amanojasen": "amanojar",
    "amanojases": "amanojar",
    "amanojaste": "amanojar",
    "amanojasteis": "amanojar",
    "amanoje": "amanojar",
    "amanojemos": "amanojar",
    "amanojen": "amanojar",
    "amanojes": "amanojar",
    "amanojo": "amanojar",
    "amanojábamos": "amanojar",
    "amanojáis": "amanojar",
    "amanojáramos": "amanojar",
    "amanojáremos": "amanojar",
    "amanojásemos": "amanojar",
    "amanojé": "amanojar",
    "amanojéis": "amanojar",
    "amanojó": "amanojar",
    "amansa": "amansar",
    "amansaba": "amansar",
    "amansabais": "amansar",
    "amansaban": "amansar",
    "amansabas": "amansar",
    "amansad": "amansar",
    "amansada": "amansar",
    "amansadas": "amansar",
    "amansado": "amansar",
    "amansadora": "amansador",
    "amansadoras": "amansador",
    "amansadores": "amansador",
    "amansados": "amansar",
    "amansamientos": "amansamiento",
    "amansamos": "amansar",
    "amansan": "amansar",
    "amansando": "amansar",
    "amansara": "amansar",
    "amansarais": "amansar",
    "amansaran": "amansar",
    "amansaras": "amansar",
    "amansare": "amansar",
    "amansareis": "amansar",
    "amansaremos": "amansar",
    "amansaren": "amansar",
    "amansares": "amansar",
    "amansaron": "amansar",
    "amansará": "amansar",
    "amansarán": "amansar",
    "amansarás": "amansar",
    "amansaré": "amansar",
    "amansaréis": "amansar",
    "amansaría": "amansar",
    "amansaríais": "amansar",
    "amansaríamos": "amansar",
    "amansarían": "amansar",
    "amansarías": "amansar",
    "amansas": "amansar",
    "amansase": "amansar",
    "amansaseis": "amansar",
    "amansasen": "amansar",
    "amansases": "amansar",
    "amansaste": "amansar",
    "amansasteis": "amansar",
    "amanse": "amansar",
    "amansemos": "amansar",
    "amansen": "amansar",
    "amanses": "amansar",
    "amanso": "amansar",
    "amansábamos": "amansar",
    "amansáis": "amansar",
    "amansáramos": "amansar",
    "amansáremos": "amansar",
    "amansásemos": "amansar",
    "amansé": "amansar",
    "amanséis": "amansar",
    "amansó": "amansar",
    "amantes": "amante",
    "amantillos": "amantillo",
    "amanuenses": "amanuense",
    "amapolas": "amapola",
    "amara": "amarar",
    "amaraba": "amarar",
    "amarabais": "amarar",
    "amaraban": "amarar",
    "amarabas": "amarar",
    "amarad": "amarar",
    "amarada": "amarar",
    "amaradas": "amarar",
    "amarado": "amarar",
    "amarados": "amarar",
    "amarais": "amar",
    "amarajes": "amaraje",
    "amaramos": "amarar",
    "amaran": "amarar",
    "amarando": "amarar",
    "amarantos": "amaranto",
    "amarara": "amarar",
    "amararais": "amarar",
    "amararan": "amarar",
    "amararas": "amarar",
    "amarare": "amarar",
    "amarareis": "amarar",
    "amararemos": "amarar",
    "amararen": "amarar",
    "amarares": "amarar",
    "amararon": "amarar",
    "amarará": "amarar",
    "amararán": "amarar",
    "amararás": "amarar",
    "amararé": "amarar",
    "amararéis": "amarar",
    "amararía": "amarar",
    "amararíais": "amarar",
    "amararíamos": "amarar",
    "amararían": "amarar",
    "amararías": "amarar",
    "amaras": "amarar",
    "amarase": "amarar",
    "amaraseis": "amarar",
    "amarasen": "amarar",
    "amarases": "amarar",
    "amaraste": "amarar",
    "amarasteis": "amarar",
    "amarchanta": "amarchantar",
    "amarchantaba": "amarchantar",
    "amarchantabais": "amarchantar",
    "amarchantaban": "amarchantar",
    "amarchantabas": "amarchantar",
    "amarchantad": "amarchantar",
    "amarchantada": "amarchantar",
    "amarchantadas": "amarchantar",
    "amarchantado": "amarchantar",
    "amarchantados": "amarchantar",
    "amarchantamos": "amarchantar",
    "amarchantan": "amarchantar",
    "amarchantando": "amarchantar",
    "amarchantara": "amarchantar",
    "amarchantarais": "amarchantar",
    "amarchantaran": "amarchantar",
    "amarchantaras": "amarchantar",
    "amarchantare": "amarchantar",
    "amarchantareis": "amarchantar",
    "amarchantaremos": "amarchantar",
    "amarchantaren": "amarchantar",
    "amarchantares": "amarchantar",
    "amarchantaron": "amarchantar",
    "amarchantará": "amarchantar",
    "amarchantarán": "amarchantar",
    "amarchantarás": "amarchantar",
    "amarchantaré": "amarchantar",
    "amarchantaréis": "amarchantar",
    "amarchantaría": "amarchantar",
    "amarchantaríais": "amarchantar",
    "amarchantaríamos": "amarchantar",
    "amarchantarían": "amarchantar",
    "amarchantarías": "amarchantar",
    "amarchantas": "amarchantar",
    "amarchantase": "amarchantar",
    "amarchantaseis": "amarchantar",
    "amarchantasen": "amarchantar",
    "amarchantases": "amarchantar",
    "amarchantaste": "amarchantar",
    "amarchantasteis": "amarchantar",
    "amarchante": "amarchantar",
    "amarchantemos": "amarchantar",
    "amarchanten": "amarchantar",
    "amarchantes": "amarchantar",
    "amarchanto": "amarchantar",
    "amarchantábamos": "amarchantar",
    "amarchantáis": "amarchantar",
    "amarchantáramos": "amarchantar",
    "amarchantáremos": "amarchantar",
    "amarchantásemos": "amarchantar",
    "amarchanté": "amarchantar",
    "amarchantéis": "amarchantar",
    "amarchantó": "amarchantar",
    "amare": "amarar",
    "amareis": "amar",
    "amaremos": "amarar",
    "amaren": "amarar",
    "amares": "amarar",
    "amarga": "amargo",
    "amargaba": "amargar",
    "amargabais": "amargar",
    "amargaban": "amargar",
    "amargabas": "amargar",
    "amargad": "amargar",
    "amargada": "amargar",
    "amargadas": "amargar",
    "amargado": "amargar",
    "amargados": "amargar",
    "amargamos": "amargar",
    "amargan": "amargar",
    "amargando": "amargar",
    "amargara": "amargar",
    "amargarais": "amargar",
    "amargaran": "amargar",
    "amargaras": "amargar",
    "amargare": "amargar",
    "amargareis": "amargar",
    "amargaremos": "amargar",
    "amargaren": "amargar",
    "amargares": "amargar",
    "amargaron": "amargar",
    "amargará": "amargar",
    "amargarán": "amargar",
    "amargarás": "amargar",
    "amargaré": "amargar",
    "amargaréis": "amargar",
    "amargaría": "amargar",
    "amargaríais": "amargar",
    "amargaríamos": "amargar",
    "amargarían": "amargar",
    "amargarías": "amargar",
    "amargas": "amargo",
    "amargase": "amargar",
    "amargaseis": "amargar",
    "amargasen": "amargar",
    "amargases": "amargar",
    "amargaste": "amargar",
    "amargasteis": "amargar",
    "amargo": "amargar",
    "amargores": "amargor",
    "amargos": "amargo",
    "amargue": "amargar",
    "amarguemos": "amargar",
    "amarguen": "amargar",
    "amargues": "amargar",
    "amarguras": "amargura",
    "amargué": "amargar",
    "amarguéis": "amargar",
    "amargábamos": "amargar",
    "amargáis": "amargar",
    "amargáramos": "amargar",
    "amargáremos": "amargar",
    "amargásemos": "amargar",
    "amargó": "amargar",
    "amarilla": "amarillo",
    "amarillas": "amarillo",
    "amarillea": "amarillear",
    "amarilleaba": "amarillear",
    "amarilleabais": "amarillear",
    "amarilleaban": "amarillear",
    "amarilleabas": "amarillear",
    "amarillead": "amarillear",
    "amarilleada": "amarillear",
    "amarilleadas": "amarillear",
    "amarilleado": "amarillear",
    "amarilleados": "amarillear",
    "amarilleamientos": "amarilleamiento",
    "amarilleamos": "amarillear",
    "amarillean": "amarillear",
    "amarilleando": "amarillear",
    "amarilleara": "amarillear",
    "amarillearais": "amarillear",
    "amarillearan": "amarillear",
    "amarillearas": "amarillear",
    "amarilleare": "amarillear",
    "amarilleareis": "amarillear",
    "amarillearemos": "amarillear",
    "amarillearen": "amarillear",
    "amarilleares": "amarillear",
    "amarillearon": "amarillear",
    "amarilleará": "amarillear",
    "amarillearán": "amarillear",
    "amarillearás": "amarillear",
    "amarillearé": "amarillear",
    "amarillearéis": "amarillear",
    "amarillearía": "amarillear",
    "amarillearíais": "amarillear",
    "amarillearíamos": "amarillear",
    "amarillearían": "amarillear",
    "amarillearías": "amarillear",
    "amarilleas": "amarillear",
    "amarillease": "amarillear",
    "amarilleaseis": "amarillear",
    "amarilleasen": "amarillear",
    "amarilleases": "amarillear",
    "amarilleaste": "amarillear",
    "amarilleasteis": "amarillear",
    "amarillece": "amarillecer",
    "amarilleced": "amarillecer",
    "amarillecemos": "amarillecer",
    "amarillecen": "amarillecer",
    "amarilleceremos": "amarillecer",
    "amarillecerá": "amarillecer",
    "amarillecerán": "amarillecer",
    "amarillecerás": "amarillecer",
    "amarilleceré": "amarillecer",
    "amarilleceréis": "amarillecer",
    "amarillecería": "amarillecer",
    "amarilleceríais": "amarillecer",
    "amarilleceríamos": "amarillecer",
    "amarillecerían": "amarillecer",
    "amarillecerías": "amarillecer",
    "amarilleces": "amarillez",
    "amarillecida": "amarillecer",
    "amarillecidas": "amarillecer",
    "amarillecido": "amarillecer",
    "amarillecidos": "amarillecer",
    "amarilleciendo": "amarillecer",
    "amarilleciera": "amarillecer",
    "amarillecierais": "amarillecer",
    "amarillecieran": "amarillecer",
    "amarillecieras": "amarillecer",
    "amarilleciere": "amarillecer",
    "amarilleciereis": "amarillecer",
    "amarillecieren": "amarillecer",
    "amarillecieres": "amarillecer",
    "amarillecieron": "amarillecer",
    "amarilleciese": "amarillecer",
    "amarillecieseis": "amarillecer",
    "amarilleciesen": "amarillecer",
    "amarillecieses": "amarillecer",
    "amarillecimos": "amarillecer",
    "amarilleciste": "amarillecer",
    "amarillecisteis": "amarillecer",
    "amarilleciéramos": "amarillecer",
    "amarilleciéremos": "amarillecer",
    "amarilleciésemos": "amarillecer",
    "amarilleció": "amarillecer",
    "amarillecéis": "amarillecer",
    "amarillecí": "amarillecer",
    "amarillecía": "amarillecer",
    "amarillecíais": "amarillecer",
    "amarillecíamos": "amarillecer",
    "amarillecían": "amarillecer",
    "amarillecías": "amarillecer",
    "amarillee": "amarillear",
    "amarilleemos": "amarillear",
    "amarilleen": "amarillear",
    "amarillees": "amarillear",
    "amarillenta": "amarillento",
    "amarillentas": "amarillento",
    "amarillentos": "amarillento",
    "amarilleo": "amarillear",
    "amarilleos": "amarilleo",
    "amarillezca": "amarillecer",
    "amarillezcamos": "amarillecer",
    "amarillezcan": "amarillecer",
    "amarillezcas": "amarillecer",
    "amarillezco": "amarillecer",
    "amarillezcáis": "amarillecer",
    "amarilleábamos": "amarillear",
    "amarilleáis": "amarillear",
    "amarilleáramos": "amarillear",
    "amarilleáremos": "amarillear",
    "amarilleásemos": "amarillear",
    "amarilleé": "amarillear",
    "amarilleéis": "amarillear",
    "amarilleó": "amarillear",
    "amarillistas": "amarillista",
    "amarillos": "amarillo",
    "amarina": "amarinar",
    "amarinaba": "amarinar",
    "amarinabais": "amarinar",
    "amarinaban": "amarinar",
    "amarinabas": "amarinar",
    "amarinad": "amarinar",
    "amarinada": "amarinar",
    "amarinadas": "amarinar",
    "amarinado": "amarinar",
    "amarinados": "amarinar",
    "amarinamos": "amarinar",
    "amarinan": "amarinar",
    "amarinando": "amarinar",
    "amarinara": "amarinar",
    "amarinarais": "amarinar",
    "amarinaran": "amarinar",
    "amarinaras": "amarinar",
    "amarinare": "amarinar",
    "amarinareis": "amarinar",
    "amarinaremos": "amarinar",
    "amarinaren": "amarinar",
    "amarinares": "amarinar",
    "amarinaron": "amarinar",
    "amarinará": "amarinar",
    "amarinarán": "amarinar",
    "amarinarás": "amarinar",
    "amarinaré": "amarinar",
    "amarinaréis": "amarinar",
    "amarinaría": "amarinar",
    "amarinaríais": "amarinar",
    "amarinaríamos": "amarinar",
    "amarinarían": "amarinar",
    "amarinarías": "amarinar",
    "amarinas": "amarinar",
    "amarinase": "amarinar",
    "amarinaseis": "amarinar",
    "amarinasen": "amarinar",
    "amarinases": "amarinar",
    "amarinaste": "amarinar",
    "amarinasteis": "amarinar",
    "amarine": "amarinar",
    "amarinemos": "amarinar",
    "amarinen": "amarinar",
    "amarines": "amarinar",
    "amarino": "amarinar",
    "amarinábamos": "amarinar",
    "amarináis": "amarinar",
    "amarináramos": "amarinar",
    "amarináremos": "amarinar",
    "amarinásemos": "amarinar",
    "amariné": "amarinar",
    "amarinéis": "amarinar",
    "amarinó": "amarinar",
    "amariposada": "amariposado",
    "amariposadas": "amariposado",
    "amariposados": "amariposado",
    "amaro": "amarar",
    "amaroma": "amaromar",
    "amaromaba": "amaromar",
    "amaromabais": "amaromar",
    "amaromaban": "amaromar",
    "amaromabas": "amaromar",
    "amaromad": "amaromar",
    "amaromada": "amaromar",
    "amaromadas": "amaromar",
    "amaromado": "amaromar",
    "amaromados": "amaromar",
    "amaromamos": "amaromar",
    "amaroman": "amaromar",
    "amaromando": "amaromar",
    "amaromara": "amaromar",
    "amaromarais": "amaromar",
    "amaromaran": "amaromar",
    "amaromaras": "amaromar",
    "amaromare": "amaromar",
    "amaromareis": "amaromar",
    "amaromaremos": "amaromar",
    "amaromaren": "amaromar",
    "amaromares": "amaromar",
    "amaromaron": "amaromar",
    "amaromará": "amaromar",
    "amaromarán": "amaromar",
    "amaromarás": "amaromar",
    "amaromaré": "amaromar",
    "amaromaréis": "amaromar",
    "amaromaría": "amaromar",
    "amaromaríais": "amaromar",
    "amaromaríamos": "amaromar",
    "amaromarían": "amaromar",
    "amaromarías": "amaromar",
    "amaromas": "amaromar",
    "amaromase": "amaromar",
    "amaromaseis": "amaromar",
    "amaromasen": "amaromar",
    "amaromases": "amaromar",
    "amaromaste": "amaromar",
    "amaromasteis": "amaromar",
    "amarome": "amaromar",
    "amaromemos": "amaromar",
    "amaromen": "amaromar",
    "amaromes": "amaromar",
    "amaromo": "amaromar",
    "amaromábamos": "amaromar",
    "amaromáis": "amaromar",
    "amaromáramos": "amaromar",
    "amaromáremos": "amaromar",
    "amaromásemos": "amaromar",
    "amaromé": "amaromar",
    "amaroméis": "amaromar",
    "amaromó": "amaromar",
    "amaron": "amar",
    "amaros": "amaro",
    "amarra": "amarrar",
    "amarraba": "amarrar",
    "amarrabais": "amarrar",
    "amarraban": "amarrar",
    "amarrabas": "amarrar",
    "amarrad": "amarrar",
    "amarrada": "amarrar",
    "amarradas": "amarrar",
    "amarraderos": "amarradero",
    "amarrado": "amarrar",
    "amarrados": "amarrar",
    "amarraduras": "amarradura",
    "amarrajes": "amarraje",
    "amarramos": "amarrar",
    "amarran": "amarrar",
    "amarrando": "amarrar",
    "amarrara": "amarrar",
    "amarrarais": "amarrar",
    "amarraran": "amarrar",
    "amarraras": "amarrar",
    "amarrare": "amarrar",
    "amarrareis": "amarrar",
    "amarraremos": "amarrar",
    "amarraren": "amarrar",
    "amarrares": "amarrar",
    "amarraron": "amarrar",
    "amarrará": "amarrar",
    "amarrarán": "amarrar",
    "amarrarás": "amarrar",
    "amarraré": "amarrar",
    "amarraréis": "amarrar",
    "amarraría": "amarrar",
    "amarraríais": "amarrar",
    "amarraríamos": "amarrar",
    "amarrarían": "amarrar",
    "amarrarías": "amarrar",
    "amarras": "amarrar",
    "amarrase": "amarrar",
    "amarraseis": "amarrar",
    "amarrasen": "amarrar",
    "amarrases": "amarrar",
    "amarraste": "amarrar",
    "amarrasteis": "amarrar",
    "amarre": "amarrar",
    "amarremos": "amarrar",
    "amarren": "amarrar",
    "amarres": "amarre",
    "amarro": "amarrar",
    "amarronada": "amarronado",
    "amarronadas": "amarronado",
    "amarronados": "amarronado",
    "amarrábamos": "amarrar",
    "amarráis": "amarrar",
    "amarráramos": "amarrar",
    "amarráremos": "amarrar",
    "amarrásemos": "amarrar",
    "amarré": "amarrar",
    "amarréis": "amarrar",
    "amarró": "amarrar",
    "amartela": "amartelar",
    "amartelaba": "amartelar",
    "amartelabais": "amartelar",
    "amartelaban": "amartelar",
    "amartelabas": "amartelar",
    "amartelad": "amartelar",
    "amartelada": "amartelar",
    "amarteladas": "amartelar",
    "amartelado": "amartelar",
    "amartelados": "amartelar",
    "amartelamientos": "amartelamiento",
    "amartelamos": "amartelar",
    "amartelan": "amartelar",
    "amartelando": "amartelar",
    "amartelara": "amartelar",
    "amartelarais": "amartelar",
    "amartelaran": "amartelar",
    "amartelaras": "amartelar",
    "amartelare": "amartelar",
    "amartelareis": "amartelar",
    "amartelaremos": "amartelar",
    "amartelaren": "amartelar",
    "amartelares": "amartelar",
    "amartelaron": "amartelar",
    "amartelará": "amartelar",
    "amartelarán": "amartelar",
    "amartelarás": "amartelar",
    "amartelaré": "amartelar",
    "amartelaréis": "amartelar",
    "amartelaría": "amartelar",
    "amartelaríais": "amartelar",
    "amartelaríamos": "amartelar",
    "amartelarían": "amartelar",
    "amartelarías": "amartelar",
    "amartelas": "amartelar",
    "amartelase": "amartelar",
    "amartelaseis": "amartelar",
    "amartelasen": "amartelar",
    "amartelases": "amartelar",
    "amartelaste": "amartelar",
    "amartelasteis": "amartelar",
    "amartele": "amartelar",
    "amartelemos": "amartelar",
    "amartelen": "amartelar",
    "amarteles": "amartelar",
    "amartelo": "amartelar",
    "amartelábamos": "amartelar",
    "amarteláis": "amartelar",
    "amarteláramos": "amartelar",
    "amarteláremos": "amartelar",
    "amartelásemos": "amartelar",
    "amartelé": "amartelar",
    "amarteléis": "amartelar",
    "amarteló": "amartelar",
    "amartilla": "amartillar",
    "amartillaba": "amartillar",
    "amartillabais": "amartillar",
    "amartillaban": "amartillar",
    "amartillabas": "amartillar",
    "amartillad": "amartillar",
    "amartillada": "amartillar",
    "amartilladas": "amartillar",
    "amartillado": "amartillar",
    "amartillados": "amartillar",
    "amartillamos": "amartillar",
    "amartillan": "amartillar",
    "amartillando": "amartillar",
    "amartillara": "amartillar",
    "amartillarais": "amartillar",
    "amartillaran": "amartillar",
    "amartillaras": "amartillar",
    "amartillare": "amartillar",
    "amartillareis": "amartillar",
    "amartillaremos": "amartillar",
    "amartillaren": "amartillar",
    "amartillares": "amartillar",
    "amartillaron": "amartillar",
    "amartillará": "amartillar",
    "amartillarán": "amartillar",
    "amartillarás": "amartillar",
    "amartillaré": "amartillar",
    "amartillaréis": "amartillar",
    "amartillaría": "amartillar",
    "amartillaríais": "amartillar",
    "amartillaríamos": "amartillar",
    "amartillarían": "amartillar",
    "amartillarías": "amartillar",
    "amartillas": "amartillar",
    "amartillase": "amartillar",
    "amartillaseis": "amartillar",
    "amartillasen": "amartillar",
    "amartillases": "amartillar",
    "amartillaste": "amartillar",
    "amartillasteis": "amartillar",
    "amartille": "amartillar",
    "amartillemos": "amartillar",
    "amartillen": "amartillar",
    "amartilles": "amartillar",
    "amartillo": "amartillar",
    "amartillábamos": "amartillar",
    "amartilláis": "amartillar",
    "amartilláramos": "amartillar",
    "amartilláremos": "amartillar",
    "amartillásemos": "amartillar",
    "amartillé": "amartillar",
    "amartilléis": "amartillar",
    "amartilló": "amartillar",
    "amará": "amar",
    "amarábamos": "amarar",
    "amaráis": "amarar",
    "amarán": "amar",
    "amaráramos": "amarar",
    "amaráremos": "amarar",
    "amarás": "amar",
    "amarásemos": "amarar",
    "amaré": "amarar",
    "amaréis": "amarar",
    "amaría": "amar",
    "amaríais": "amar",
    "amaríamos": "amar",
    "amarían": "amar",
    "amarías": "amar",
    "amaró": "amarar",
    "amas": "amar",
    "amasa": "amasar",
    "amasaba": "amasar",
    "amasabais": "amasar",
    "amasaban": "amasar",
    "amasabas": "amasar",
    "amasad": "amasar",
    "amasada": "amasar",
    "amasadas": "amasar",
    "amasaderas": "amasadera",
    "amasaderos": "amasadero",
    "amasado": "amasar",
    "amasadora": "amasador",
    "amasadoras": "amasadora",
    "amasadores": "amasador",
    "amasados": "amasar",
    "amasaduras": "amasadura",
    "amasakes": "amasake",
    "amasamientos": "amasamiento",
    "amasamos": "amasar",
    "amasan": "amasar",
    "amasando": "amasar",
    "amasara": "amasar",
    "amasarais": "amasar",
    "amasaran": "amasar",
    "amasaras": "amasar",
    "amasare": "amasar",
    "amasareis": "amasar",
    "amasaremos": "amasar",
    "amasaren": "amasar",
    "amasares": "amasar",
    "amasaron": "amasar",
    "amasará": "amasar",
    "amasarán": "amasar",
    "amasarás": "amasar",
    "amasaré": "amasar",
    "amasaréis": "amasar",
    "amasaría": "amasar",
    "amasaríais": "amasar",
    "amasaríamos": "amasar",
    "amasarían": "amasar",
    "amasarías": "amasar",
    "amasas": "amasar",
    "amasase": "amasar",
    "amasaseis": "amasar",
    "amasasen": "amasar",
    "amasases": "amasar",
    "amasaste": "amasar",
    "amasasteis": "amasar",
    "amase": "amasar",
    "amaseis": "amar",
    "amasemos": "amasar",
    "amasen": "amasar",
    "amases": "amasar",
    "amasijos": "amasijo",
    "amaso": "amasar",
    "amaste": "amar",
    "amasteis": "amar",
    "amasábamos": "amasar",
    "amasáis": "amasar",
    "amasáramos": "amasar",
    "amasáremos": "amasar",
    "amasásemos": "amasar",
    "amasé": "amasar",
    "amaséis": "amasar",
    "amasó": "amasar",
    "amates": "amate",
    "amateurismos": "amateurismo",
    "amateurs": "amateur",
    "amatistas": "amatista",
    "amatoria": "amatorio",
    "amatorias": "amatorio",
    "amatorios": "amatorio",
    "amaurótica": "amaurótico",
    "amauróticas": "amaurótico",
    "amauróticos": "amaurótico",
    "amayorazga": "amayorazgar",
    "amayorazgaba": "amayorazgar",
    "amayorazgabais": "amayorazgar",
    "amayorazgaban": "amayorazgar",
    "amayorazgabas": "amayorazgar",
    "amayorazgad": "amayorazgar",
    "amayorazgada": "amayorazgar",
    "amayorazgadas": "amayorazgar",
    "amayorazgado": "amayorazgar",
    "amayorazgados": "amayorazgar",
    "amayorazgamos": "amayorazgar",
    "amayorazgan": "amayorazgar",
    "amayorazgando": "amayorazgar",
    "amayorazgara": "amayorazgar",
    "amayorazgarais": "amayorazgar",
    "amayorazgaran": "amayorazgar",
    "amayorazgaras": "amayorazgar",
    "amayorazgare": "amayorazgar",
    "amayorazgareis": "amayorazgar",
    "amayorazgaremos": "amayorazgar",
    "amayorazgaren": "amayorazgar",
    "amayorazgares": "amayorazgar",
    "amayorazgaron": "amayorazgar",
    "amayorazgará": "amayorazgar",
    "amayorazgarán": "amayorazgar",
    "amayorazgarás": "amayorazgar",
    "amayorazgaré": "amayorazgar",
    "amayorazgaréis": "amayorazgar",
    "amayorazgaría": "amayorazgar",
    "amayorazgaríais": "amayorazgar",
    "amayorazgaríamos": "amayorazgar",
    "amayorazgarían": "amayorazgar",
    "amayorazgarías": "amayorazgar",
    "amayorazgas": "amayorazgar",
    "amayorazgase": "amayorazgar",
    "amayorazgaseis": "amayorazgar",
    "amayorazgasen": "amayorazgar",
    "amayorazgases": "amayorazgar",
    "amayorazgaste": "amayorazgar",
    "amayorazgasteis": "amayorazgar",
    "amayorazgo": "amayorazgar",
    "amayorazgue": "amayorazgar",
    "amayorazguemos": "amayorazgar",
    "amayorazguen": "amayorazgar",
    "amayorazgues": "amayorazgar",
    "amayorazgué": "amayorazgar",
    "amayorazguéis": "amayorazgar",
    "amayorazgábamos": "amayorazgar",
    "amayorazgáis": "amayorazgar",
    "amayorazgáramos": "amayorazgar",
    "amayorazgáremos": "amayorazgar",
    "amayorazgásemos": "amayorazgar",
    "amayorazgó": "amayorazgar",
    "amazacota": "amazacotar",
    "amazacotaba": "amazacotar",
    "amazacotabais": "amazacotar",
    "amazacotaban": "amazacotar",
    "amazacotabas": "amazacotar",
    "amazacotad": "amazacotar",
    "amazacotada": "amazacotar",
    "amazacotadas": "amazacotar",
    "amazacotado": "amazacotar",
    "amazacotados": "amazacotar",
    "amazacotamientos": "amazacotamiento",
    "amazacotamos": "amazacotar",
    "amazacotan": "amazacotar",
    "amazacotando": "amazacotar",
    "amazacotara": "amazacotar",
    "amazacotarais": "amazacotar",
    "amazacotaran": "amazacotar",
    "amazacotaras": "amazacotar",
    "amazacotare": "amazacotar",
    "amazacotareis": "amazacotar",
    "amazacotaremos": "amazacotar",
    "amazacotaren": "amazacotar",
    "amazacotares": "amazacotar",
    "amazacotaron": "amazacotar",
    "amazacotará": "amazacotar",
    "amazacotarán": "amazacotar",
    "amazacotarás": "amazacotar",
    "amazacotaré": "amazacotar",
    "amazacotaréis": "amazacotar",
    "amazacotaría": "amazacotar",
    "amazacotaríais": "amazacotar",
    "amazacotaríamos": "amazacotar",
    "amazacotarían": "amazacotar",
    "amazacotarías": "amazacotar",
    "amazacotas": "amazacotar",
    "amazacotase": "amazacotar",
    "amazacotaseis": "amazacotar",
    "amazacotasen": "amazacotar",
    "amazacotases": "amazacotar",
    "amazacotaste": "amazacotar",
    "amazacotasteis": "amazacotar",
    "amazacote": "amazacotar",
    "amazacotemos": "amazacotar",
    "amazacoten": "amazacotar",
    "amazacotes": "amazacotar",
    "amazacoto": "amazacotar",
    "amazacotábamos": "amazacotar",
    "amazacotáis": "amazacotar",
    "amazacotáramos": "amazacotar",
    "amazacotáremos": "amazacotar",
    "amazacotásemos": "amazacotar",
    "amazacoté": "amazacotar",
    "amazacotéis": "amazacotar",
    "amazacotó": "amazacotar",
    "amazonas": "amazona",
    "amazonitas": "amazonita",
    "amazónica": "amazónico",
    "amazónicas": "amazónico",
    "amazónicos": "amazónico",
    "amaña": "amañar",
    "amañaba": "amañar",
    "amañabais": "amañar",
    "amañaban": "amañar",
    "amañabas": "amañar",
    "amañad": "amañar",
    "amañada": "amañar",
    "amañadas": "amañar",
    "amañado": "amañar",
    "amañados": "amañar",
    "amañamos": "amañar",
    "amañan": "amañar",
    "amañando": "amañar",
    "amañara": "amañar",
    "amañarais": "amañar",
    "amañaran": "amañar",
    "amañaras": "amañar",
    "amañare": "amañar",
    "amañareis": "amañar",
    "amañaremos": "amañar",
    "amañaren": "amañar",
    "amañares": "amañar",
    "amañaron": "amañar",
    "amañará": "amañar",
    "amañarán": "amañar",
    "amañarás": "amañar",
    "amañaré": "amañar",
    "amañaréis": "amañar",
    "amañaría": "amañar",
    "amañaríais": "amañar",
    "amañaríamos": "amañar",
    "amañarían": "amañar",
    "amañarías": "amañar",
    "amañas": "amañar",
    "amañase": "amañar",
    "amañaseis": "amañar",
    "amañasen": "amañar",
    "amañases": "amañar",
    "amañaste": "amañar",
    "amañasteis": "amañar",
    "amañe": "amañar",
    "amañemos": "amañar",
    "amañen": "amañar",
    "amañes": "amañar",
    "amaño": "amañar",
    "amaños": "amaño",
    "amañábamos": "amañar",
    "amañáis": "amañar",
    "amañáramos": "amañar",
    "amañáremos": "amañar",
    "amañásemos": "amañar",
    "amañé": "amañar",
    "amañéis": "amañar",
    "amañó": "amañar",
    "ambara": "ambarar",
    "ambaraba": "ambarar",
    "ambarabais": "ambarar",
    "ambaraban": "ambarar",
    "ambarabas": "ambarar",
    "ambarad": "ambarar",
    "ambarada": "ambarar",
    "ambaradas": "ambarar",
    "ambarado": "ambarar",
    "ambarados": "ambarar",
    "ambaramos": "ambarar",
    "ambaran": "ambarar",
    "ambarando": "ambarar",
    "ambarara": "ambarar",
    "ambararais": "ambarar",
    "ambararan": "ambarar",
    "ambararas": "ambarar",
    "ambarare": "ambarar",
    "ambarareis": "ambarar",
    "ambararemos": "ambarar",
    "ambararen": "ambarar",
    "ambarares": "ambarar",
    "ambararon": "ambarar",
    "ambarará": "ambarar",
    "ambararán": "ambarar",
    "ambararás": "ambarar",
    "ambararé": "ambarar",
    "ambararéis": "ambarar",
    "ambararía": "ambarar",
    "ambararíais": "ambarar",
    "ambararíamos": "ambarar",
    "ambararían": "ambarar",
    "ambararías": "ambarar",
    "ambaras": "ambarar",
    "ambarase": "ambarar",
    "ambaraseis": "ambarar",
    "ambarasen": "ambarar",
    "ambarases": "ambarar",
    "ambaraste": "ambarar",
    "ambarasteis": "ambarar",
    "ambare": "ambarar",
    "ambaremos": "ambarar",
    "ambaren": "ambarar",
    "ambares": "ambarar",
    "ambarina": "ambarino",
    "ambarinas": "ambarino",
    "ambarinos": "ambarino",
    "ambaro": "ambarar",
    "ambarábamos": "ambarar",
    "ambaráis": "ambarar",
    "ambaráramos": "ambarar",
    "ambaráremos": "ambarar",
    "ambarásemos": "ambarar",
    "ambaré": "ambarar",
    "ambaréis": "ambarar",
    "ambaró": "ambarar",
    "ambas": "ambos",
    "amberina": "amberino",
    "amberinas": "amberino",
    "amberinos": "amberino",
    "ambiciona": "ambicionar",
    "ambicionaba": "ambicionar",
    "ambicionabais": "ambicionar",
    "ambicionaban": "ambicionar",
    "ambicionabas": "ambicionar",
    "ambicionad": "ambicionar",
    "ambicionada": "ambicionar",
    "ambicionadas": "ambicionar",
    "ambicionado": "ambicionar",
    "ambicionados": "ambicionar",
    "ambicionamos": "ambicionar",
    "ambicionan": "ambicionar",
    "ambicionando": "ambicionar",
    "ambicionara": "ambicionar",
    "ambicionarais": "ambicionar",
    "ambicionaran": "ambicionar",
    "ambicionaras": "ambicionar",
    "ambicionare": "ambicionar",
    "ambicionareis": "ambicionar",
    "ambicionaremos": "ambicionar",
    "ambicionaren": "ambicionar",
    "ambicionares": "ambicionar",
    "ambicionaron": "ambicionar",
    "ambicionará": "ambicionar",
    "ambicionarán": "ambicionar",
    "ambicionarás": "ambicionar",
    "ambicionaré": "ambicionar",
    "ambicionaréis": "ambicionar",
    "ambicionaría": "ambicionar",
    "ambicionaríais": "ambicionar",
    "ambicionaríamos": "ambicionar",
    "ambicionarían": "ambicionar",
    "ambicionarías": "ambicionar",
    "ambicionas": "ambicionar",
    "ambicionase": "ambicionar",
    "ambicionaseis": "ambicionar",
    "ambicionasen": "ambicionar",
    "ambicionases": "ambicionar",
    "ambicionaste": "ambicionar",
    "ambicionasteis": "ambicionar",
    "ambicione": "ambicionar",
    "ambicionemos": "ambicionar",
    "ambicionen": "ambicionar",
    "ambiciones": "ambicionar",
    "ambiciono": "ambicionar",
    "ambicionábamos": "ambicionar",
    "ambicionáis": "ambicionar",
    "ambicionáramos": "ambicionar",
    "ambicionáremos": "ambicionar",
    "ambicionásemos": "ambicionar",
    "ambicioné": "ambicionar",
    "ambicionéis": "ambicionar",
    "ambicionó": "ambicionar",
    "ambiciosa": "ambicioso",
    "ambiciosas": "ambicioso",
    "ambiciosos": "ambicioso",
    "ambidextra": "ambidextro",
    "ambidextras": "ambidextro",
    "ambidextros": "ambidextro",
    "ambidiestra": "ambidiestro",
    "ambidiestras": "ambidiestro",
    "ambidiestros": "ambidiestro",
    "ambienta": "ambientar",
    "ambientaba": "ambientar",
    "ambientabais": "ambientar",
    "ambientaban": "ambientar",
    "ambientabas": "ambientar",
    "ambientaciones": "ambientación",
    "ambientad": "ambientar",
    "ambientada": "ambientar",
    "ambientadas": "ambientar",
    "ambientado": "ambientar",
    "ambientadores": "ambientador",
    "ambientados": "ambientar",
    "ambientales": "ambiental",
    "ambientalismos": "ambientalismo",
    "ambientalistas": "ambientalista",
    "ambientamos": "ambientar",
    "ambientan": "ambientar",
    "ambientando": "ambientar",
    "ambientara": "ambientar",
    "ambientarais": "ambientar",
    "ambientaran": "ambientar",
    "ambientaras": "ambientar",
    "ambientare": "ambientar",
    "ambientareis": "ambientar",
    "ambientaremos": "ambientar",
    "ambientaren": "ambientar",
    "ambientares": "ambientar",
    "ambientaron": "ambientar",
    "ambientará": "ambientar",
    "ambientarán": "ambientar",
    "ambientarás": "ambientar",
    "ambientaré": "ambientar",
    "ambientaréis": "ambientar",
    "ambientaría": "ambientar",
    "ambientaríais": "ambientar",
    "ambientaríamos": "ambientar",
    "ambientarían": "ambientar",
    "ambientarías": "ambientar",
    "ambientas": "ambientar",
    "ambientase": "ambientar",
    "ambientaseis": "ambientar",
    "ambientasen": "ambientar",
    "ambientases": "ambientar",
    "ambientaste": "ambientar",
    "ambientasteis": "ambientar",
    "ambiente": "ambientar",
    "ambientemos": "ambientar",
    "ambienten": "ambientar",
    "ambientes": "ambiente",
    "ambiento": "ambientar",
    "ambientábamos": "ambientar",
    "ambientáis": "ambientar",
    "ambientáramos": "ambientar",
    "ambientáremos": "ambientar",
    "ambientásemos": "ambientar",
    "ambienté": "ambientar",
    "ambientéis": "ambientar",
    "ambientó": "ambientar",
    "ambigua": "ambiguo",
    "ambiguas": "ambiguo",
    "ambiguos": "ambiguo",
    "ambigús": "ambigú",
    "ambigüedades": "ambigüedad",
    "ambilineales": "ambilineal",
    "ambilocales": "ambilocal",
    "ambivalencias": "ambivalencia",
    "ambivalentes": "ambivalente",
    "ambla": "amblar",
    "amblaba": "amblar",
    "amblabais": "amblar",
    "amblaban": "amblar",
    "amblabas": "amblar",
    "amblad": "amblar",
    "amblada": "amblar",
    "ambladas": "amblar",
    "amblado": "amblar",
    "ambladora": "amblador",
    "ambladoras": "amblador",
    "ambladores": "amblador",
    "amblados": "amblar",
    "ambladuras": "ambladura",
    "amblamos": "amblar",
    "amblan": "amblar",
    "amblando": "amblar",
    "amblara": "amblar",
    "amblarais": "amblar",
    "amblaran": "amblar",
    "amblaras": "amblar",
    "amblare": "amblar",
    "amblareis": "amblar",
    "amblaremos": "amblar",
    "amblaren": "amblar",
    "amblares": "amblar",
    "amblaron": "amblar",
    "amblará": "amblar",
    "amblarán": "amblar",
    "amblarás": "amblar",
    "amblaré": "amblar",
    "amblaréis": "amblar",
    "amblaría": "amblar",
    "amblaríais": "amblar",
    "amblaríamos": "amblar",
    "amblarían": "amblar",
    "amblarías": "amblar",
    "amblas": "amblar",
    "amblase": "amblar",
    "amblaseis": "amblar",
    "amblasen": "amblar",
    "amblases": "amblar",
    "amblaste": "amblar",
    "amblasteis": "amblar",
    "amble": "amblar",
    "amblemos": "amblar",
    "amblen": "amblar",
    "ambles": "amblar",
    "ambligonitas": "ambligonita",
    "ambliopías": "ambliopía",
    "amblioscopios": "amblioscopio",
    "amblo": "amblar",
    "amblábamos": "amblar",
    "ambláis": "amblar",
    "ambláramos": "amblar",
    "ambláremos": "amblar",
    "amblásemos": "amblar",
    "amblé": "amblar",
    "ambléis": "amblar",
    "amblíopes": "amblíope",
    "ambló": "amblar",
    "amboceptora": "amboceptor",
    "amboceptoras": "amboceptor",
    "amboceptores": "amboceptor",
    "ambones": "ambón",
    "ambroides": "ambroide",
    "ambrosiana": "ambrosiano",
    "ambrosianas": "ambrosiano",
    "ambrosianos": "ambrosiano",
    "ambrosías": "ambrosía",
    "ambuestas": "ambuesta",
    "ambula": "ambular",
    "ambulaba": "ambular",
    "ambulabais": "ambular",
    "ambulaban": "ambular",
    "ambulabas": "ambular",
    "ambulacrales": "ambulacral",
    "ambulacros": "ambulacro",
    "ambulad": "ambular",
    "ambulada": "ambular",
    "ambuladas": "ambular",
    "ambulado": "ambular",
    "ambulados": "ambular",
    "ambulamos": "ambular",
    "ambulan": "ambular",
    "ambulancias": "ambulancia",
    "ambulando": "ambular",
    "ambulantes": "ambulante",
    "ambulara": "ambular",
    "ambularais": "ambular",
    "ambularan": "ambular",
    "ambularas": "ambular",
    "ambulare": "ambular",
    "ambulareis": "ambular",
    "ambularemos": "ambular",
    "ambularen": "ambular",
    "ambulares": "ambular",
    "ambularon": "ambular",
    "ambulará": "ambular",
    "ambularán": "ambular",
    "ambularás": "ambular",
    "ambularé": "ambular",
    "ambularéis": "ambular",
    "ambularía": "ambular",
    "ambularíais": "ambular",
    "ambularíamos": "ambular",
    "ambularían": "ambular",
    "ambularías": "ambular",
    "ambulas": "ambular",
    "ambulase": "ambular",
    "ambulaseis": "ambular",
    "ambulasen": "ambular",
    "ambulases": "ambular",
    "ambulaste": "ambular",
    "ambulasteis": "ambular",
    "ambulatoria": "ambulatorio",
    "ambulatorias": "ambulatorio",
    "ambulatorios": "ambulatorio",
    "ambule": "ambular",
    "ambulemos": "ambular",
    "ambulen": "ambular",
    "ambules": "ambular",
    "ambulo": "ambular",
    "ambulábamos": "ambular",
    "ambuláis": "ambular",
    "ambuláramos": "ambular",
    "ambuláremos": "ambular",
    "ambulásemos": "ambular",
    "ambulé": "ambular",
    "ambuléis": "ambular",
    "ambuló": "ambular",
    "ame": "amar",
    "amebas": "ameba",
    "ameboides": "ameboide",
    "amecha": "amechar",
    "amechaba": "amechar",
    "amechabais": "amechar",
    "amechaban": "amechar",
    "amechabas": "amechar",
    "amechad": "amechar",
    "amechada": "amechar",
    "amechadas": "amechar",
    "amechado": "amechar",
    "amechados": "amechar",
    "amechamos": "amechar",
    "amechan": "amechar",
    "amechando": "amechar",
    "amechara": "amechar",
    "amecharais": "amechar",
    "amecharan": "amechar",
    "amecharas": "amechar",
    "amechare": "amechar",
    "amechareis": "amechar",
    "amecharemos": "amechar",
    "amecharen": "amechar",
    "amechares": "amechar",
    "amecharon": "amechar",
    "amechará": "amechar",
    "amecharán": "amechar",
    "amecharás": "amechar",
    "amecharé": "amechar",
    "amecharéis": "amechar",
    "amecharía": "amechar",
    "amecharíais": "amechar",
    "amecharíamos": "amechar",
    "amecharían": "amechar",
    "amecharías": "amechar",
    "amechas": "amechar",
    "amechase": "amechar",
    "amechaseis": "amechar",
    "amechasen": "amechar",
    "amechases": "amechar",
    "amechaste": "amechar",
    "amechasteis": "amechar",
    "ameche": "amechar",
    "amechemos": "amechar",
    "amechen": "amechar",
    "ameches": "amechar",
    "amecho": "amechar",
    "amechábamos": "amechar",
    "amecháis": "amechar",
    "amecháramos": "amechar",
    "amecháremos": "amechar",
    "amechásemos": "amechar",
    "ameché": "amechar",
    "amechéis": "amechar",
    "amechó": "amechar",
    "amedrenta": "amedrentar",
    "amedrentaba": "amedrentar",
    "amedrentabais": "amedrentar",
    "amedrentaban": "amedrentar",
    "amedrentabas": "amedrentar",
    "amedrentad": "amedrentar",
    "amedrentada": "amedrentar",
    "amedrentadas": "amedrentar",
    "amedrentado": "amedrentar",
    "amedrentados": "amedrentar",
    "amedrentamientos": "amedrentamiento",
    "amedrentamos": "amedrentar",
    "amedrentan": "amedrentar",
    "amedrentando": "amedrentar",
    "amedrentara": "amedrentar",
    "amedrentarais": "amedrentar",
    "amedrentaran": "amedrentar",
    "amedrentaras": "amedrentar",
    "amedrentare": "amedrentar",
    "amedrentareis": "amedrentar",
    "amedrentaremos": "amedrentar",
    "amedrentaren": "amedrentar",
    "amedrentares": "amedrentar",
    "amedrentaron": "amedrentar",
    "amedrentará": "amedrentar",
    "amedrentarán": "amedrentar",
    "amedrentarás": "amedrentar",
    "amedrentaré": "amedrentar",
    "amedrentaréis": "amedrentar",
    "amedrentaría": "amedrentar",
    "amedrentaríais": "amedrentar",
    "amedrentaríamos": "amedrentar",
    "amedrentarían": "amedrentar",
    "amedrentarías": "amedrentar",
    "amedrentas": "amedrentar",
    "amedrentase": "amedrentar",
    "amedrentaseis": "amedrentar",
    "amedrentasen": "amedrentar",
    "amedrentases": "amedrentar",
    "amedrentaste": "amedrentar",
    "amedrentasteis": "amedrentar",
    "amedrente": "amedrentar",
    "amedrentemos": "amedrentar",
    "amedrenten": "amedrentar",
    "amedrentes": "amedrentar",
    "amedrento": "amedrentar",
    "amedrentábamos": "amedrentar",
    "amedrentáis": "amedrentar",
    "amedrentáramos": "amedrentar",
    "amedrentáremos": "amedrentar",
    "amedrentásemos": "amedrentar",
    "amedrenté": "amedrentar",
    "amedrentéis": "amedrentar",
    "amedrentó": "amedrentar",
    "amelga": "amelgar",
    "amelgaba": "amelgar",
    "amelgabais": "amelgar",
    "amelgaban": "amelgar",
    "amelgabas": "amelgar",
    "amelgad": "amelgar",
    "amelgada": "amelgar",
    "amelgadas": "amelgar",
    "amelgado": "amelgar",
    "amelgados": "amelgar",
    "amelgamos": "amelgar",
    "amelgan": "amelgar",
    "amelgando": "amelgar",
    "amelgara": "amelgar",
    "amelgarais": "amelgar",
    "amelgaran": "amelgar",
    "amelgaras": "amelgar",
    "amelgare": "amelgar",
    "amelgareis": "amelgar",
    "amelgaremos": "amelgar",
    "amelgaren": "amelgar",
    "amelgares": "amelgar",
    "amelgaron": "amelgar",
    "amelgará": "amelgar",
    "amelgarán": "amelgar",
    "amelgarás": "amelgar",
    "amelgaré": "amelgar",
    "amelgaréis": "amelgar",
    "amelgaría": "amelgar",
    "amelgaríais": "amelgar",
    "amelgaríamos": "amelgar",
    "amelgarían": "amelgar",
    "amelgarías": "amelgar",
    "amelgas": "amelgar",
    "amelgase": "amelgar",
    "amelgaseis": "amelgar",
    "amelgasen": "amelgar",
    "amelgases": "amelgar",
    "amelgaste": "amelgar",
    "amelgasteis": "amelgar",
    "amelgo": "amelgar",
    "amelgue": "amelgar",
    "amelguemos": "amelgar",
    "amelguen": "amelgar",
    "amelgues": "amelgar",
    "amelgué": "amelgar",
    "amelguéis": "amelgar",
    "amelgábamos": "amelgar",
    "amelgáis": "amelgar",
    "amelgáramos": "amelgar",
    "amelgáremos": "amelgar",
    "amelgásemos": "amelgar",
    "amelgó": "amelgar",
    "ameloblastomas": "ameloblastoma",
    "ameloblastos": "ameloblasto",
    "ameloblástica": "ameloblástico",
    "ameloblásticas": "ameloblástico",
    "ameloblásticos": "ameloblástico",
    "amelodentinales": "amelodentinal",
    "amelopatías": "amelopatía",
    "amelos": "amelo",
    "amelógena": "amelógeno",
    "amelógenas": "amelógeno",
    "amelógenos": "amelógeno",
    "amemos": "amar",
    "amen": "amar",
    "amena": "ameno",
    "amenace": "amenazar",
    "amenacemos": "amenazar",
    "amenacen": "amenazar",
    "amenaces": "amenazar",
    "amenacé": "amenazar",
    "amenacéis": "amenazar",
    "amenas": "ameno",
    "amenaza": "amenazar",
    "amenazaba": "amenazar",
    "amenazabais": "amenazar",
    "amenazaban": "amenazar",
    "amenazabas": "amenazar",
    "amenazad": "amenazar",
    "amenazada": "amenazar",
    "amenazadas": "amenazar",
    "amenazado": "amenazar",
    "amenazadora": "amenazador",
    "amenazadoras": "amenazador",
    "amenazadores": "amenazador",
    "amenazados": "amenazar",
    "amenazamos": "amenazar",
    "amenazan": "amenazar",
    "amenazando": "amenazar",
    "amenazantes": "amenazante",
    "amenazara": "amenazar",
    "amenazarais": "amenazar",
    "amenazaran": "amenazar",
    "amenazaras": "amenazar",
    "amenazare": "amenazar",
    "amenazareis": "amenazar",
    "amenazaremos": "amenazar",
    "amenazaren": "amenazar",
    "amenazares": "amenazar",
    "amenazaron": "amenazar",
    "amenazará": "amenazar",
    "amenazarán": "amenazar",
    "amenazarás": "amenazar",
    "amenazaré": "amenazar",
    "amenazaréis": "amenazar",
    "amenazaría": "amenazar",
    "amenazaríais": "amenazar",
    "amenazaríamos": "amenazar",
    "amenazarían": "amenazar",
    "amenazarías": "amenazar",
    "amenazas": "amenazar",
    "amenazase": "amenazar",
    "amenazaseis": "amenazar",
    "amenazasen": "amenazar",
    "amenazases": "amenazar",
    "amenazaste": "amenazar",
    "amenazasteis": "amenazar",
    "amenazo": "amenazar",
    "amenazábamos": "amenazar",
    "amenazáis": "amenazar",
    "amenazáramos": "amenazar",
    "amenazáremos": "amenazar",
    "amenazásemos": "amenazar",
    "amenazó": "amenazar",
    "amencias": "amencia",
    "amenes": "amén",
    "amengua": "amenguar",
    "amenguaba": "amenguar",
    "amenguabais": "amenguar",
    "amenguaban": "amenguar",
    "amenguabas": "amenguar",
    "amenguad": "amenguar",
    "amenguada": "amenguar",
    "amenguadas": "amenguar",
    "amenguado": "amenguar",
    "amenguados": "amenguar",
    "amenguamos": "amenguar",
    "amenguan": "amenguar",
    "amenguando": "amenguar",
    "amenguara": "amenguar",
    "amenguarais": "amenguar",
    "amenguaran": "amenguar",
    "amenguaras": "amenguar",
    "amenguare": "amenguar",
    "amenguareis": "amenguar",
    "amenguaremos": "amenguar",
    "amenguaren": "amenguar",
    "amenguares": "amenguar",
    "amenguaron": "amenguar",
    "amenguará": "amenguar",
    "amenguarán": "amenguar",
    "amenguarás": "amenguar",
    "amenguaré": "amenguar",
    "amenguaréis": "amenguar",
    "amenguaría": "amenguar",
    "amenguaríais": "amenguar",
    "amenguaríamos": "amenguar",
    "amenguarían": "amenguar",
    "amenguarías": "amenguar",
    "amenguas": "amenguar",
    "amenguase": "amenguar",
    "amenguaseis": "amenguar",
    "amenguasen": "amenguar",
    "amenguases": "amenguar",
    "amenguaste": "amenguar",
    "amenguasteis": "amenguar",
    "amenguo": "amenguar",
    "amenguábamos": "amenguar",
    "amenguáis": "amenguar",
    "amenguáramos": "amenguar",
    "amenguáremos": "amenguar",
    "amenguásemos": "amenguar",
    "amenguó": "amenguar",
    "amengüe": "amenguar",
    "amengüemos": "amenguar",
    "amengüen": "amenguar",
    "amengües": "amenguar",
    "amengüé": "amenguar",
    "amengüéis": "amenguar",
    "amenice": "amenizar",
    "amenicemos": "amenizar",
    "amenicen": "amenizar",
    "amenices": "amenizar",
    "amenicé": "amenizar",
    "amenicéis": "amenizar",
    "amenidades": "amenidad",
    "ameniza": "amenizar",
    "amenizaba": "amenizar",
    "amenizabais": "amenizar",
    "amenizaban": "amenizar",
    "amenizabas": "amenizar",
    "amenizad": "amenizar",
    "amenizada": "amenizar",
    "amenizadas": "amenizar",
    "amenizado": "amenizar",
    "amenizados": "amenizar",
    "amenizamos": "amenizar",
    "amenizan": "amenizar",
    "amenizando": "amenizar",
    "amenizara": "amenizar",
    "amenizarais": "amenizar",
    "amenizaran": "amenizar",
    "amenizaras": "amenizar",
    "amenizare": "amenizar",
    "amenizareis": "amenizar",
    "amenizaremos": "amenizar",
    "amenizaren": "amenizar",
    "amenizares": "amenizar",
    "amenizaron": "amenizar",
    "amenizará": "amenizar",
    "amenizarán": "amenizar",
    "amenizarás": "amenizar",
    "amenizaré": "amenizar",
    "amenizaréis": "amenizar",
    "amenizaría": "amenizar",
    "amenizaríais": "amenizar",
    "amenizaríamos": "amenizar",
    "amenizarían": "amenizar",
    "amenizarías": "amenizar",
    "amenizas": "amenizar",
    "amenizase": "amenizar",
    "amenizaseis": "amenizar",
    "amenizasen": "amenizar",
    "amenizases": "amenizar",
    "amenizaste": "amenizar",
    "amenizasteis": "amenizar",
    "amenizo": "amenizar",
    "amenizábamos": "amenizar",
    "amenizáis": "amenizar",
    "amenizáramos": "amenizar",
    "amenizáremos": "amenizar",
    "amenizásemos": "amenizar",
    "amenizó": "amenizar",
    "amenorreas": "amenorrea",
    "amenos": "ameno",
    "amentos": "amento",
    "amentácea": "amentáceo",
    "amentáceas": "amentáceo",
    "amentáceos": "amentáceo",
    "amera": "amerar",
    "ameraba": "amerar",
    "amerabais": "amerar",
    "ameraban": "amerar",
    "amerabas": "amerar",
    "amerad": "amerar",
    "amerada": "amerar",
    "ameradas": "amerar",
    "amerado": "amerar",
    "amerados": "amerar",
    "ameramos": "amerar",
    "ameran": "amerar",
    "amerando": "amerar",
    "amerara": "amerar",
    "amerarais": "amerar",
    "ameraran": "amerar",
    "ameraras": "amerar",
    "amerare": "amerar",
    "amerareis": "amerar",
    "ameraremos": "amerar",
    "ameraren": "amerar",
    "amerares": "amerar",
    "ameraron": "amerar",
    "amerará": "amerar",
    "amerarán": "amerar",
    "amerarás": "amerar",
    "ameraré": "amerar",
    "ameraréis": "amerar",
    "ameraría": "amerar",
    "ameraríais": "amerar",
    "ameraríamos": "amerar",
    "amerarían": "amerar",
    "amerarías": "amerar",
    "ameras": "amerar",
    "amerase": "amerar",
    "ameraseis": "amerar",
    "amerasen": "amerar",
    "amerases": "amerar",
    "ameraste": "amerar",
    "amerasteis": "amerar",
    "amere": "amerar",
    "ameremos": "amerar",
    "ameren": "amerar",
    "amerengada": "amerengado",
    "amerengadas": "amerengado",
    "amerengados": "amerengado",
    "ameres": "amerar",
    "americana": "americano",
    "americanas": "americano",
    "americanice": "americanizar",
    "americanicemos": "americanizar",
    "americanicen": "americanizar",
    "americanices": "americanizar",
    "americanicé": "americanizar",
    "americanicéis": "americanizar",
    "americanismos": "americanismo",
    "americanistas": "americanista",
    "americaniza": "americanizar",
    "americanizaba": "americanizar",
    "americanizabais": "americanizar",
    "americanizaban": "americanizar",
    "americanizabas": "americanizar",
    "americanizaciones": "americanización",
    "americanizad": "americanizar",
    "americanizada": "americanizar",
    "americanizadas": "americanizar",
    "americanizado": "americanizar",
    "americanizados": "americanizar",
    "americanizamos": "americanizar",
    "americanizan": "americanizar",
    "americanizando": "americanizar",
    "americanizara": "americanizar",
    "americanizarais": "americanizar",
    "americanizaran": "americanizar",
    "americanizaras": "americanizar",
    "americanizare": "americanizar",
    "americanizareis": "americanizar",
    "americanizaremos": "americanizar",
    "americanizaren": "americanizar",
    "americanizares": "americanizar",
    "americanizaron": "americanizar",
    "americanizará": "americanizar",
    "americanizarán": "americanizar",
    "americanizarás": "americanizar",
    "americanizaré": "americanizar",
    "americanizaréis": "americanizar",
    "americanizaría": "americanizar",
    "americanizaríais": "americanizar",
    "americanizaríamos": "americanizar",
    "americanizarían": "americanizar",
    "americanizarías": "americanizar",
    "americanizas": "americanizar",
    "americanizase": "americanizar",
    "americanizaseis": "americanizar",
    "americanizasen": "americanizar",
    "americanizases": "americanizar",
    "americanizaste": "americanizar",
    "americanizasteis": "americanizar",
    "americanizo": "americanizar",
    "americanizábamos": "americanizar",
    "americanizáis": "americanizar",
    "americanizáramos": "americanizar",
    "americanizáremos": "americanizar",
    "americanizásemos": "americanizar",
    "americanizó": "americanizar",
    "americanos": "americano",
    "americe": "amerizar",
    "americemos": "amerizar",
    "americen": "amerizar",
    "americes": "amerizar",
    "americios": "americio",
    "americé": "amerizar",
    "americéis": "amerizar",
    "amerindia": "amerindio",
    "amerindias": "amerindio",
    "amerindios": "amerindio",
    "amerita": "ameritar",
    "ameritaba": "ameritar",
    "ameritabais": "ameritar",
    "ameritaban": "ameritar",
    "ameritabas": "ameritar",
    "ameritad": "ameritar",
    "ameritada": "ameritar",
    "ameritadas": "ameritar",
    "ameritado": "ameritar",
    "ameritados": "ameritar",
    "ameritamos": "ameritar",
    "ameritan": "ameritar",
    "ameritando": "ameritar",
    "ameritara": "ameritar",
    "ameritarais": "ameritar",
    "ameritaran": "ameritar",
    "ameritaras": "ameritar",
    "ameritare": "ameritar",
    "ameritareis": "ameritar",
    "ameritaremos": "ameritar",
    "ameritaren": "ameritar",
    "ameritares": "ameritar",
    "ameritaron": "ameritar",
    "ameritará": "ameritar",
    "ameritarán": "ameritar",
    "ameritarás": "ameritar",
    "ameritaré": "ameritar",
    "ameritaréis": "ameritar",
    "ameritaría": "ameritar",
    "ameritaríais": "ameritar",
    "ameritaríamos": "ameritar",
    "ameritarían": "ameritar",
    "ameritarías": "ameritar",
    "ameritas": "ameritar",
    "ameritase": "ameritar",
    "ameritaseis": "ameritar",
    "ameritasen": "ameritar",
    "ameritases": "ameritar",
    "ameritaste": "ameritar",
    "ameritasteis": "ameritar",
    "amerite": "ameritar",
    "ameritemos": "ameritar",
    "ameriten": "ameritar",
    "amerites": "ameritar",
    "amerito": "ameritar",
    "ameritábamos": "ameritar",
    "ameritáis": "ameritar",
    "ameritáramos": "ameritar",
    "ameritáremos": "ameritar",
    "ameritásemos": "ameritar",
    "amerité": "ameritar",
    "ameritéis": "ameritar",
    "ameritó": "ameritar",
    "ameriza": "amerizar",
    "amerizaba": "amerizar",
    "amerizabais": "amerizar",
    "amerizaban": "amerizar",
    "amerizabas": "amerizar",
    "amerizad": "amerizar",
    "amerizada": "amerizar",
    "amerizadas": "amerizar",
    "amerizado": "amerizar",
    "amerizados": "amerizar",
    "amerizajes": "amerizaje",
    "amerizamos": "amerizar",
    "amerizan": "amerizar",
    "amerizando": "amerizar",
    "amerizara": "amerizar",
    "amerizarais": "amerizar",
    "amerizaran": "amerizar",
    "amerizaras": "amerizar",
    "amerizare": "amerizar",
    "amerizareis": "amerizar",
    "amerizaremos": "amerizar",
    "amerizaren": "amerizar",
    "amerizares": "amerizar",
    "amerizaron": "amerizar",
    "amerizará": "amerizar",
    "amerizarán": "amerizar",
    "amerizarás": "amerizar",
    "amerizaré": "amerizar",
    "amerizaréis": "amerizar",
    "amerizaría": "amerizar",
    "amerizaríais": "amerizar",
    "amerizaríamos": "amerizar",
    "amerizarían": "amerizar",
    "amerizarías": "amerizar",
    "amerizas": "amerizar",
    "amerizase": "amerizar",
    "amerizaseis": "amerizar",
    "amerizasen": "amerizar",
    "amerizases": "amerizar",
    "amerizaste": "amerizar",
    "amerizasteis": "amerizar",
    "amerizo": "amerizar",
    "amerizábamos": "amerizar",
    "amerizáis": "amerizar",
    "amerizáramos": "amerizar",
    "amerizáremos": "amerizar",
    "amerizásemos": "amerizar",
    "amerizó": "amerizar",
    "amero": "amerar",
    "amerábamos": "amerar",
    "ameráis": "amerar",
    "ameráramos": "amerar",
    "ameráremos": "amerar",
    "amerásemos": "amerar",
    "ameré": "amerar",
    "ameréis": "amerar",
    "ameró": "amerar",
    "ames": "amar",
    "amestizada": "amestizado",
    "amestizadas": "amestizado",
    "amestizados": "amestizado",
    "ametopterinas": "ametopterina",
    "ametralla": "ametrallar",
    "ametrallaba": "ametrallar",
    "ametrallabais": "ametrallar",
    "ametrallaban": "ametrallar",
    "ametrallabas": "ametrallar",
    "ametrallad": "ametrallar",
    "ametrallada": "ametrallar",
    "ametralladas": "ametrallar",
    "ametrallado": "ametrallar",
    "ametralladora": "ametrallador",
    "ametralladoras": "ametralladora",
    "ametralladores": "ametrallador",
    "ametrallados": "ametrallar",
    "ametrallamientos": "ametrallamiento",
    "ametrallamos": "ametrallar",
    "ametrallan": "ametrallar",
    "ametrallando": "ametrallar",
    "ametrallara": "ametrallar",
    "ametrallarais": "ametrallar",
    "ametrallaran": "ametrallar",
    "ametrallaras": "ametrallar",
    "ametrallare": "ametrallar",
    "ametrallareis": "ametrallar",
    "ametrallaremos": "ametrallar",
    "ametrallaren": "ametrallar",
    "ametrallares": "ametrallar",
    "ametrallaron": "ametrallar",
    "ametrallará": "ametrallar",
    "ametrallarán": "ametrallar",
    "ametrallarás": "ametrallar",
    "ametrallaré": "ametrallar",
    "ametrallaréis": "ametrallar",
    "ametrallaría": "ametrallar",
    "ametrallaríais": "ametrallar",
    "ametrallaríamos": "ametrallar",
    "ametrallarían": "ametrallar",
    "ametrallarías": "ametrallar",
    "ametrallas": "ametrallar",
    "ametrallase": "ametrallar",
    "ametrallaseis": "ametrallar",
    "ametrallasen": "ametrallar",
    "ametrallases": "ametrallar",
    "ametrallaste": "ametrallar",
    "ametrallasteis": "ametrallar",
    "ametralle": "ametrallar",
    "ametrallemos": "ametrallar",
    "ametrallen": "ametrallar",
    "ametralles": "ametrallar",
    "ametrallo": "ametrallar",
    "ametrallábamos": "ametrallar",
    "ametralláis": "ametrallar",
    "ametralláramos": "ametrallar",
    "ametralláremos": "ametrallar",
    "ametrallásemos": "ametrallar",
    "ametrallé": "ametrallar",
    "ametralléis": "ametrallar",
    "ametralló": "ametrallar",
    "ametropías": "ametropía",
    "ametrópica": "ametrópico",
    "ametrópicas": "ametrópico",
    "ametrópicos": "ametrópico",
    "amezquinda": "amezquindar",
    "amezquindaba": "amezquindar",
    "amezquindabais": "amezquindar",
    "amezquindaban": "amezquindar",
    "amezquindabas": "amezquindar",
    "amezquindad": "amezquindar",
    "amezquindada": "amezquindar",
    "amezquindadas": "amezquindar",
    "amezquindado": "amezquindar",
    "amezquindados": "amezquindar",
    "amezquindamos": "amezquindar",
    "amezquindan": "amezquindar",
    "amezquindando": "amezquindar",
    "amezquindara": "amezquindar",
    "amezquindarais": "amezquindar",
    "amezquindaran": "amezquindar",
    "amezquindaras": "amezquindar",
    "amezquindare": "amezquindar",
    "amezquindareis": "amezquindar",
    "amezquindaremos": "amezquindar",
    "amezquindaren": "amezquindar",
    "amezquindares": "amezquindar",
    "amezquindaron": "amezquindar",
    "amezquindará": "amezquindar",
    "amezquindarán": "amezquindar",
    "amezquindarás": "amezquindar",
    "amezquindaré": "amezquindar",
    "amezquindaréis": "amezquindar",
    "amezquindaría": "amezquindar",
    "amezquindaríais": "amezquindar",
    "amezquindaríamos": "amezquindar",
    "amezquindarían": "amezquindar",
    "amezquindarías": "amezquindar",
    "amezquindas": "amezquindar",
    "amezquindase": "amezquindar",
    "amezquindaseis": "amezquindar",
    "amezquindasen": "amezquindar",
    "amezquindases": "amezquindar",
    "amezquindaste": "amezquindar",
    "amezquindasteis": "amezquindar",
    "amezquinde": "amezquindar",
    "amezquindemos": "amezquindar",
    "amezquinden": "amezquindar",
    "amezquindes": "amezquindar",
    "amezquindo": "amezquindar",
    "amezquindábamos": "amezquindar",
    "amezquindáis": "amezquindar",
    "amezquindáramos": "amezquindar",
    "amezquindáremos": "amezquindar",
    "amezquindásemos": "amezquindar",
    "amezquindé": "amezquindar",
    "amezquindéis": "amezquindar",
    "amezquindó": "amezquindar",
    "amháricos": "amhárico",
    "amiantos": "amianto",
    "amibas": "amiba",
    "amicales": "amical",
    "amicrones": "amicrón",
    "amidas": "amida",
    "amidinas": "amidina",
    "amiga": "amigo",
    "amigaba": "amigar",
    "amigabais": "amigar",
    "amigaban": "amigar",
    "amigabas": "amigar",
    "amigabilidades": "amigabilidad",
    "amigables": "amigable",
    "amigad": "amigar",
    "amigada": "amigar",
    "amigadas": "amigar",
    "amigado": "amigar",
    "amigados": "amigar",
    "amigamos": "amigar",
    "amigan": "amigar",
    "amigando": "amigar",
    "amigara": "amigar",
    "amigarais": "amigar",
    "amigaran": "amigar",
    "amigaras": "amigar",
    "amigare": "amigar",
    "amigareis": "amigar",
    "amigaremos": "amigar",
    "amigaren": "amigar",
    "amigares": "amigar",
    "amigaron": "amigar",
    "amigará": "amigar",
    "amigarán": "amigar",
    "amigarás": "amigar",
    "amigaré": "amigar",
    "amigaréis": "amigar",
    "amigaría": "amigar",
    "amigaríais": "amigar",
    "amigaríamos": "amigar",
    "amigarían": "amigar",
    "amigarías": "amigar",
    "amigas": "amigo",
    "amigase": "amigar",
    "amigaseis": "amigar",
    "amigasen": "amigar",
    "amigases": "amigar",
    "amigaste": "amigar",
    "amigasteis": "amigar",
    "amigdalares": "amigdalar",
    "amigdalectomías": "amigdalectomía",
    "amigdalina": "amigdalino",
    "amigdalinas": "amigdalino",
    "amigdalinos": "amigdalino",
    "amigdaloglosa": "amigdalogloso",
    "amigdaloglosas": "amigdalogloso",
    "amigdaloglosos": "amigdalogloso",
    "amigdaloides": "amigdaloide",
    "amigdalopatías": "amigdalopatía",
    "amigdalotomías": "amigdalotomía",
    "amigo": "amigar",
    "amigos": "amigo",
    "amigotes": "amigote",
    "amigue": "amigar",
    "amiguemos": "amigar",
    "amiguen": "amigar",
    "amigues": "amigar",
    "amiguismos": "amiguismo",
    "amigué": "amigar",
    "amiguéis": "amigar",
    "amigábamos": "amigar",
    "amigáis": "amigar",
    "amigáramos": "amigar",
    "amigáremos": "amigar",
    "amigásemos": "amigar",
    "amigó": "amigar",
    "amiiformes": "amiiforme",
    "amilana": "amilanar",
    "amilanaba": "amilanar",
    "amilanabais": "amilanar",
    "amilanaban": "amilanar",
    "amilanabas": "amilanar",
    "amilanad": "amilanar",
    "amilanada": "amilanar",
    "amilanadas": "amilanar",
    "amilanado": "amilanar",
    "amilanados": "amilanar",
    "amilanamientos": "amilanamiento",
    "amilanamos": "amilanar",
    "amilanan": "amilanar",
    "amilanando": "amilanar",
    "amilanara": "amilanar",
    "amilanarais": "amilanar",
    "amilanaran": "amilanar",
    "amilanaras": "amilanar",
    "amilanare": "amilanar",
    "amilanareis": "amilanar",
    "amilanaremos": "amilanar",
    "amilanaren": "amilanar",
    "amilanares": "amilanar",
    "amilanaron": "amilanar",
    "amilanará": "amilanar",
    "amilanarán": "amilanar",
    "amilanarás": "amilanar",
    "amilanaré": "amilanar",
    "amilanaréis": "amilanar",
    "amilanaría": "amilanar",
    "amilanaríais": "amilanar",
    "amilanaríamos": "amilanar",
    "amilanarían": "amilanar",
    "amilanarías": "amilanar",
    "amilanas": "amilanar",
    "amilanase": "amilanar",
    "amilanaseis": "amilanar",
    "amilanasen": "amilanar",
    "amilanases": "amilanar",
    "amilanaste": "amilanar",
    "amilanasteis": "amilanar",
    "amilane": "amilanar",
    "amilanemos": "amilanar",
    "amilanen": "amilanar",
    "amilanes": "amilanar",
    "amilano": "amilanar",
    "amilanábamos": "amilanar",
    "amilanáis": "amilanar",
    "amilanáramos": "amilanar",
    "amilanáremos": "amilanar",
    "amilanásemos": "amilanar",
    "amilané": "amilanar",
    "amilanéis": "amilanar",
    "amilanó": "amilanar",
    "amilasas": "amilasa",
    "amilenos": "amileno",
    "amillara": "amillarar",
    "amillaraba": "amillarar",
    "amillarabais": "amillarar",
    "amillaraban": "amillarar",
    "amillarabas": "amillarar",
    "amillarad": "amillarar",
    "amillarada": "amillarar",
    "amillaradas": "amillarar",
    "amillarado": "amillarar",
    "amillarados": "amillarar",
    "amillaramientos": "amillaramiento",
    "amillaramos": "amillarar",
    "amillaran": "amillarar",
    "amillarando": "amillarar",
    "amillarara": "amillarar",
    "amillararais": "amillarar",
    "amillararan": "amillarar",
    "amillararas": "amillarar",
    "amillarare": "amillarar",
    "amillarareis": "amillarar",
    "amillararemos": "amillarar",
    "amillararen": "amillarar",
    "amillarares": "amillarar",
    "amillararon": "amillarar",
    "amillarará": "amillarar",
    "amillararán": "amillarar",
    "amillararás": "amillarar",
    "amillararé": "amillarar",
    "amillararéis": "amillarar",
    "amillararía": "amillarar",
    "amillararíais": "amillarar",
    "amillararíamos": "amillarar",
    "amillararían": "amillarar",
    "amillararías": "amillarar",
    "amillaras": "amillarar",
    "amillarase": "amillarar",
    "amillaraseis": "amillarar",
    "amillarasen": "amillarar",
    "amillarases": "amillarar",
    "amillaraste": "amillarar",
    "amillarasteis": "amillarar",
    "amillare": "amillarar",
    "amillaremos": "amillarar",
    "amillaren": "amillarar",
    "amillares": "amillarar",
    "amillaro": "amillarar",
    "amillarábamos": "amillarar",
    "amillaráis": "amillarar",
    "amillaráramos": "amillarar",
    "amillaráremos": "amillarar",
    "amillarásemos": "amillarar",
    "amillaré": "amillarar",
    "amillaréis": "amillarar",
    "amillaró": "amillarar",
    "amiloplastos": "amiloplasto",
    "amilopsinas": "amilopsina",
    "amiloridas": "amilorida",
    "amilos": "amilo",
    "amilosas": "amilosa",
    "amilácea": "amiláceo",
    "amiláceas": "amiláceo",
    "amiláceos": "amiláceo",
    "aminaciones": "aminación",
    "aminada": "aminado",
    "aminadas": "aminado",
    "aminados": "aminado",
    "aminas": "amina",
    "aminoacetatos": "aminoacetato",
    "aminobenzoatos": "aminobenzoato",
    "aminopterinas": "aminopterina",
    "aminora": "aminorar",
    "aminoraba": "aminorar",
    "aminorabais": "aminorar",
    "aminoraban": "aminorar",
    "aminorabas": "aminorar",
    "aminoraciones": "aminoración",
    "aminorad": "aminorar",
    "aminorada": "aminorar",
    "aminoradas": "aminorar",
    "aminorado": "aminorar",
    "aminorados": "aminorar",
    "aminoramientos": "aminoramiento",
    "aminoramos": "aminorar",
    "aminoran": "aminorar",
    "aminorando": "aminorar",
    "aminorara": "aminorar",
    "aminorarais": "aminorar",
    "aminoraran": "aminorar",
    "aminoraras": "aminorar",
    "aminorare": "aminorar",
    "aminorareis": "aminorar",
    "aminoraremos": "aminorar",
    "aminoraren": "aminorar",
    "aminorares": "aminorar",
    "aminoraron": "aminorar",
    "aminorará": "aminorar",
    "aminorarán": "aminorar",
    "aminorarás": "aminorar",
    "aminoraré": "aminorar",
    "aminoraréis": "aminorar",
    "aminoraría": "aminorar",
    "aminoraríais": "aminorar",
    "aminoraríamos": "aminorar",
    "aminorarían": "aminorar",
    "aminorarías": "aminorar",
    "aminoras": "aminorar",
    "aminorase": "aminorar",
    "aminoraseis": "aminorar",
    "aminorasen": "aminorar",
    "aminorases": "aminorar",
    "aminoraste": "aminorar",
    "aminorasteis": "aminorar",
    "aminore": "aminorar",
    "aminoremos": "aminorar",
    "aminoren": "aminorar",
    "aminores": "aminorar",
    "aminoro": "aminorar",
    "aminorábamos": "aminorar",
    "aminoráis": "aminorar",
    "aminoráramos": "aminorar",
    "aminoráremos": "aminorar",
    "aminorásemos": "aminorar",
    "aminoré": "aminorar",
    "aminoréis": "aminorar",
    "aminoró": "aminorar",
    "aminoácidos": "aminoácido",
    "amista": "amistar",
    "amistaba": "amistar",
    "amistabais": "amistar",
    "amistaban": "amistar",
    "amistabas": "amistar",
    "amistad": "amistar",
    "amistada": "amistar",
    "amistadas": "amistar",
    "amistades": "amistad",
    "amistado": "amistar",
    "amistados": "amistar",
    "amistamos": "amistar",
    "amistan": "amistar",
    "amistando": "amistar",
    "amistara": "amistar",
    "amistarais": "amistar",
    "amistaran": "amistar",
    "amistaras": "amistar",
    "amistare": "amistar",
    "amistareis": "amistar",
    "amistaremos": "amistar",
    "amistaren": "amistar",
    "amistares": "amistar",
    "amistaron": "amistar",
    "amistará": "amistar",
    "amistarán": "amistar",
    "amistarás": "amistar",
    "amistaré": "amistar",
    "amistaréis": "amistar",
    "amistaría": "amistar",
    "amistaríais": "amistar",
    "amistaríamos": "amistar",
    "amistarían": "amistar",
    "amistarías": "amistar",
    "amistas": "amistar",
    "amistase": "amistar",
    "amistaseis": "amistar",
    "amistasen": "amistar",
    "amistases": "amistar",
    "amistaste": "amistar",
    "amistasteis": "amistar",
    "amiste": "amistar",
    "amistemos": "amistar",
    "amisten": "amistar",
    "amistes": "amistar",
    "amisto": "amistar",
    "amistosa": "amistoso",
    "amistosas": "amistoso",
    "amistosos": "amistoso",
    "amistábamos": "amistar",
    "amistáis": "amistar",
    "amistáramos": "amistar",
    "amistáremos": "amistar",
    "amistásemos": "amistar",
    "amisté": "amistar",
    "amistéis": "amistar",
    "amistó": "amistar",
    "amitolocales": "amitolocal",
    "amitos": "amito",
    "amitótica": "amitótico",
    "amitóticas": "amitótico",
    "amitóticos": "amitótico",
    "amixias": "amixia",
    "ammocetes": "ammocete",
    "ammonitas": "ammonita",
    "amnesias": "amnesia",
    "amnistiaba": "amnistiar",
    "amnistiabais": "amnistiar",
    "amnistiaban": "amnistiar",
    "amnistiabas": "amnistiar",
    "amnistiad": "amnistiar",
    "amnistiada": "amnistiar",
    "amnistiadas": "amnistiar",
    "amnistiado": "amnistiar",
    "amnistiados": "amnistiar",
    "amnistiamos": "amnistiar",
    "amnistiando": "amnistiar",
    "amnistiara": "amnistiar",
    "amnistiarais": "amnistiar",
    "amnistiaran": "amnistiar",
    "amnistiaras": "amnistiar",
    "amnistiare": "amnistiar",
    "amnistiareis": "amnistiar",
    "amnistiaremos": "amnistiar",
    "amnistiaren": "amnistiar",
    "amnistiares": "amnistiar",
    "amnistiaron": "amnistiar",
    "amnistiará": "amnistiar",
    "amnistiarán": "amnistiar",
    "amnistiarás": "amnistiar",
    "amnistiaré": "amnistiar",
    "amnistiaréis": "amnistiar",
    "amnistiaría": "amnistiar",
    "amnistiaríais": "amnistiar",
    "amnistiaríamos": "amnistiar",
    "amnistiarían": "amnistiar",
    "amnistiarías": "amnistiar",
    "amnistiase": "amnistiar",
    "amnistiaseis": "amnistiar",
    "amnistiasen": "amnistiar",
    "amnistiases": "amnistiar",
    "amnistiaste": "amnistiar",
    "amnistiasteis": "amnistiar",
    "amnistiemos": "amnistiar",
    "amnistiábamos": "amnistiar",
    "amnistiáis": "amnistiar",
    "amnistiáramos": "amnistiar",
    "amnistiáremos": "amnistiar",
    "amnistiásemos": "amnistiar",
    "amnistié": "amnistiar",
    "amnistiéis": "amnistiar",
    "amnistió": "amnistiar",
    "amnistía": "amnistiar",
    "amnistían": "amnistiar",
    "amnistías": "amnistiar",
    "amnistíe": "amnistiar",
    "amnistíen": "amnistiar",
    "amnistíes": "amnistiar",
    "amnistío": "amnistiar",
    "amniótica": "amniótico",
    "amnióticas": "amniótico",
    "amnióticos": "amniótico",
    "amnésica": "amnésico",
    "amnésicas": "amnésico",
    "amnésicos": "amnésico",
    "amo": "amar",
    "amoblaba": "amoblar",
    "amoblabais": "amoblar",
    "amoblaban": "amoblar",
    "amoblabas": "amoblar",
    "amoblad": "amoblar",
    "amoblada": "amoblar",
    "amobladas": "amoblar",
    "amoblado": "amoblar",
    "amoblados": "amoblar",
    "amoblamos": "amoblar",
    "amoblando": "amoblar",
    "amoblara": "amoblar",
    "amoblarais": "amoblar",
    "amoblaran": "amoblar",
    "amoblaras": "amoblar",
    "amoblare": "amoblar",
    "amoblareis": "amoblar",
    "amoblaremos": "amoblar",
    "amoblaren": "amoblar",
    "amoblares": "amoblar",
    "amoblaron": "amoblar",
    "amoblará": "amoblar",
    "amoblarán": "amoblar",
    "amoblarás": "amoblar",
    "amoblaré": "amoblar",
    "amoblaréis": "amoblar",
    "amoblaría": "amoblar",
    "amoblaríais": "amoblar",
    "amoblaríamos": "amoblar",
    "amoblarían": "amoblar",
    "amoblarías": "amoblar",
    "amoblase": "amoblar",
    "amoblaseis": "amoblar",
    "amoblasen": "amoblar",
    "amoblases": "amoblar",
    "amoblaste": "amoblar",
    "amoblasteis": "amoblar",
    "amoblemos": "amoblar",
    "amoblábamos": "amoblar",
    "amobláis": "amoblar",
    "amobláramos": "amoblar",
    "amobláremos": "amoblar",
    "amoblásemos": "amoblar",
    "amoblé": "amoblar",
    "amobléis": "amoblar",
    "amobló": "amoblar",
    "amocha": "amochar",
    "amochaba": "amochar",
    "amochabais": "amochar",
    "amochaban": "amochar",
    "amochabas": "amochar",
    "amochad": "amochar",
    "amochada": "amochar",
    "amochadas": "amochar",
    "amochado": "amochar",
    "amochados": "amochar",
    "amochamos": "amochar",
    "amochan": "amochar",
    "amochando": "amochar",
    "amochara": "amochar",
    "amocharais": "amochar",
    "amocharan": "amochar",
    "amocharas": "amochar",
    "amochare": "amochar",
    "amochareis": "amochar",
    "amocharemos": "amochar",
    "amocharen": "amochar",
    "amochares": "amochar",
    "amocharon": "amochar",
    "amochará": "amochar",
    "amocharán": "amochar",
    "amocharás": "amochar",
    "amocharé": "amochar",
    "amocharéis": "amochar",
    "amocharía": "amochar",
    "amocharíais": "amochar",
    "amocharíamos": "amochar",
    "amocharían": "amochar",
    "amocharías": "amochar",
    "amochas": "amochar",
    "amochase": "amochar",
    "amochaseis": "amochar",
    "amochasen": "amochar",
    "amochases": "amochar",
    "amochaste": "amochar",
    "amochasteis": "amochar",
    "amoche": "amochar",
    "amochemos": "amochar",
    "amochen": "amochar",
    "amoches": "amochar",
    "amocho": "amochar",
    "amochábamos": "amochar",
    "amocháis": "amochar",
    "amocháramos": "amochar",
    "amocháremos": "amochar",
    "amochásemos": "amochar",
    "amoché": "amochar",
    "amochéis": "amochar",
    "amochó": "amochar",
    "amodorra": "amodorrar",
    "amodorraba": "amodorrar",
    "amodorrabais": "amodorrar",
    "amodorraban": "amodorrar",
    "amodorrabas": "amodorrar",
    "amodorrad": "amodorrar",
    "amodorrada": "amodorrar",
    "amodorradas": "amodorrar",
    "amodorrado": "amodorrar",
    "amodorrados": "amodorrar",
    "amodorramientos": "amodorramiento",
    "amodorramos": "amodorrar",
    "amodorran": "amodorrar",
    "amodorrando": "amodorrar",
    "amodorrara": "amodorrar",
    "amodorrarais": "amodorrar",
    "amodorraran": "amodorrar",
    "amodorraras": "amodorrar",
    "amodorrare": "amodorrar",
    "amodorrareis": "amodorrar",
    "amodorraremos": "amodorrar",
    "amodorraren": "amodorrar",
    "amodorrares": "amodorrar",
    "amodorraron": "amodorrar",
    "amodorrará": "amodorrar",
    "amodorrarán": "amodorrar",
    "amodorrarás": "amodorrar",
    "amodorraré": "amodorrar",
    "amodorraréis": "amodorrar",
    "amodorraría": "amodorrar",
    "amodorraríais": "amodorrar",
    "amodorraríamos": "amodorrar",
    "amodorrarían": "amodorrar",
    "amodorrarías": "amodorrar",
    "amodorras": "amodorrar",
    "amodorrase": "amodorrar",
    "amodorraseis": "amodorrar",
    "amodorrasen": "amodorrar",
    "amodorrases": "amodorrar",
    "amodorraste": "amodorrar",
    "amodorrasteis": "amodorrar",
    "amodorre": "amodorrar",
    "amodorremos": "amodorrar",
    "amodorren": "amodorrar",
    "amodorres": "amodorrar",
    "amodorro": "amodorrar",
    "amodorrábamos": "amodorrar",
    "amodorráis": "amodorrar",
    "amodorráramos": "amodorrar",
    "amodorráremos": "amodorrar",
    "amodorrásemos": "amodorrar",
    "amodorré": "amodorrar",
    "amodorréis": "amodorrar",
    "amodorró": "amodorrar",
    "amohinaba": "amohinar",
    "amohinabais": "amohinar",
    "amohinaban": "amohinar",
    "amohinabas": "amohinar",
    "amohinad": "amohinar",
    "amohinada": "amohinar",
    "amohinadas": "amohinar",
    "amohinado": "amohinar",
    "amohinados": "amohinar",
    "amohinamos": "amohinar",
    "amohinando": "amohinar",
    "amohinara": "amohinar",
    "amohinarais": "amohinar",
    "amohinaran": "amohinar",
    "amohinaras": "amohinar",
    "amohinare": "amohinar",
    "amohinareis": "amohinar",
    "amohinaremos": "amohinar",
    "amohinaren": "amohinar",
    "amohinares": "amohinar",
    "amohinaron": "amohinar",
    "amohinará": "amohinar",
    "amohinarán": "amohinar",
    "amohinarás": "amohinar",
    "amohinaré": "amohinar",
    "amohinaréis": "amohinar",
    "amohinaría": "amohinar",
    "amohinaríais": "amohinar",
    "amohinaríamos": "amohinar",
    "amohinarían": "amohinar",
    "amohinarías": "amohinar",
    "amohinase": "amohinar",
    "amohinaseis": "amohinar",
    "amohinasen": "amohinar",
    "amohinases": "amohinar",
    "amohinaste": "amohinar",
    "amohinasteis": "amohinar",
    "amohinemos": "amohinar",
    "amohinábamos": "amohinar",
    "amohináis": "amohinar",
    "amohináramos": "amohinar",
    "amohináremos": "amohinar",
    "amohinásemos": "amohinar",
    "amohiné": "amohinar",
    "amohinéis": "amohinar",
    "amohinó": "amohinar",
    "amohína": "amohinar",
    "amohínan": "amohinar",
    "amohínas": "amohinar",
    "amohíne": "amohinar",
    "amohínen": "amohinar",
    "amohínes": "amohinar",
    "amohíno": "amohinar",
    "amojama": "amojamar",
    "amojamaba": "amojamar",
    "amojamabais": "amojamar",
    "amojamaban": "amojamar",
    "amojamabas": "amojamar",
    "amojamad": "amojamar",
    "amojamada": "amojamar",
    "amojamadas": "amojamar",
    "amojamado": "amojamar",
    "amojamados": "amojamar",
    "amojamamos": "amojamar",
    "amojaman": "amojamar",
    "amojamando": "amojamar",
    "amojamara": "amojamar",
    "amojamarais": "amojamar",
    "amojamaran": "amojamar",
    "amojamaras": "amojamar",
    "amojamare": "amojamar",
    "amojamareis": "amojamar",
    "amojamaremos": "amojamar",
    "amojamaren": "amojamar",
    "amojamares": "amojamar",
    "amojamaron": "amojamar",
    "amojamará": "amojamar",
    "amojamarán": "amojamar",
    "amojamarás": "amojamar",
    "amojamaré": "amojamar",
    "amojamaréis": "amojamar",
    "amojamaría": "amojamar",
    "amojamaríais": "amojamar",
    "amojamaríamos": "amojamar",
    "amojamarían": "amojamar",
    "amojamarías": "amojamar",
    "amojamas": "amojamar",
    "amojamase": "amojamar",
    "amojamaseis": "amojamar",
    "amojamasen": "amojamar",
    "amojamases": "amojamar",
    "amojamaste": "amojamar",
    "amojamasteis": "amojamar",
    "amojame": "amojamar",
    "amojamemos": "amojamar",
    "amojamen": "amojamar",
    "amojames": "amojamar",
    "amojamo": "amojamar",
    "amojamábamos": "amojamar",
    "amojamáis": "amojamar",
    "amojamáramos": "amojamar",
    "amojamáremos": "amojamar",
    "amojamásemos": "amojamar",
    "amojamé": "amojamar",
    "amojaméis": "amojamar",
    "amojamó": "amojamar",
    "amojela": "amojelar",
    "amojelaba": "amojelar",
    "amojelabais": "amojelar",
    "amojelaban": "amojelar",
    "amojelabas": "amojelar",
    "amojelad": "amojelar",
    "amojelada": "amojelar",
    "amojeladas": "amojelar",
    "amojelado": "amojelar",
    "amojelados": "amojelar",
    "amojelamos": "amojelar",
    "amojelan": "amojelar",
    "amojelando": "amojelar",
    "amojelara": "amojelar",
    "amojelarais": "amojelar",
    "amojelaran": "amojelar",
    "amojelaras": "amojelar",
    "amojelare": "amojelar",
    "amojelareis": "amojelar",
    "amojelaremos": "amojelar",
    "amojelaren": "amojelar",
    "amojelares": "amojelar",
    "amojelaron": "amojelar",
    "amojelará": "amojelar",
    "amojelarán": "amojelar",
    "amojelarás": "amojelar",
    "amojelaré": "amojelar",
    "amojelaréis": "amojelar",
    "amojelaría": "amojelar",
    "amojelaríais": "amojelar",
    "amojelaríamos": "amojelar",
    "amojelarían": "amojelar",
    "amojelarías": "amojelar",
    "amojelas": "amojelar",
    "amojelase": "amojelar",
    "amojelaseis": "amojelar",
    "amojelasen": "amojelar",
    "amojelases": "amojelar",
    "amojelaste": "amojelar",
    "amojelasteis": "amojelar",
    "amojele": "amojelar",
    "amojelemos": "amojelar",
    "amojelen": "amojelar",
    "amojeles": "amojelar",
    "amojelo": "amojelar",
    "amojelábamos": "amojelar",
    "amojeláis": "amojelar",
    "amojeláramos": "amojelar",
    "amojeláremos": "amojelar",
    "amojelásemos": "amojelar",
    "amojelé": "amojelar",
    "amojeléis": "amojelar",
    "amojeló": "amojelar",
    "amojona": "amojonar",
    "amojonaba": "amojonar",
    "amojonabais": "amojonar",
    "amojonaban": "amojonar",
    "amojonabas": "amojonar",
    "amojonad": "amojonar",
    "amojonada": "amojonar",
    "amojonadas": "amojonar",
    "amojonado": "amojonar",
    "amojonados": "amojonar",
    "amojonamientos": "amojonamiento",
    "amojonamos": "amojonar",
    "amojonan": "amojonar",
    "amojonando": "amojonar",
    "amojonara": "amojonar",
    "amojonarais": "amojonar",
    "amojonaran": "amojonar",
    "amojonaras": "amojonar",
    "amojonare": "amojonar",
    "amojonareis": "amojonar",
    "amojonaremos": "amojonar",
    "amojonaren": "amojonar",
    "amojonares": "amojonar",
    "amojonaron": "amojonar",
    "amojonará": "amojonar",
    "amojonarán": "amojonar",
    "amojonarás": "amojonar",
    "amojonaré": "amojonar",
    "amojonaréis": "amojonar",
    "amojonaría": "amojonar",
    "amojonaríais": "amojonar",
    "amojonaríamos": "amojonar",
    "amojonarían": "amojonar",
    "amojonarías": "amojonar",
    "amojonas": "amojonar",
    "amojonase": "amojonar",
    "amojonaseis": "amojonar",
    "amojonasen": "amojonar",
    "amojonases": "amojonar",
    "amojonaste": "amojonar",
    "amojonasteis": "amojonar",
    "amojone": "amojonar",
    "amojonemos": "amojonar",
    "amojonen": "amojonar",
    "amojones": "amojonar",
    "amojono": "amojonar",
    "amojonábamos": "amojonar",
    "amojonáis": "amojonar",
    "amojonáramos": "amojonar",
    "amojonáremos": "amojonar",
    "amojonásemos": "amojonar",
    "amojoné": "amojonar",
    "amojonéis": "amojonar",
    "amojonó": "amojonar",
    "amolaba": "amolar",
    "amolabais": "amolar",
    "amolaban": "amolar",
    "amolabas": "amolar",
    "amolad": "amolar",
    "amolada": "amolar",
    "amoladas": "amolar",
    "amoladeras": "amoladera",
    "amolado": "amolar",
    "amoladores": "amolador",
    "amolados": "amolar",
    "amoladuras": "amoladura",
    "amolamos": "amolar",
    "amolanchines": "amolanchín",
    "amolando": "amolar",
    "amolara": "amolar",
    "amolarais": "amolar",
    "amolaran": "amolar",
    "amolaras": "amolar",
    "amolare": "amolar",
    "amolareis": "amolar",
    "amolaremos": "amolar",
    "amolaren": "amolar",
    "amolares": "amolar",
    "amolaron": "amolar",
    "amolará": "amolar",
    "amolarán": "amolar",
    "amolarás": "amolar",
    "amolaré": "amolar",
    "amolaréis": "amolar",
    "amolaría": "amolar",
    "amolaríais": "amolar",
    "amolaríamos": "amolar",
    "amolarían": "amolar",
    "amolarías": "amolar",
    "amolase": "amolar",
    "amolaseis": "amolar",
    "amolasen": "amolar",
    "amolases": "amolar",
    "amolaste": "amolar",
    "amolasteis": "amolar",
    "amolda": "amoldar",
    "amoldaba": "amoldar",
    "amoldabais": "amoldar",
    "amoldaban": "amoldar",
    "amoldabas": "amoldar",
    "amoldad": "amoldar",
    "amoldada": "amoldar",
    "amoldadas": "amoldar",
    "amoldado": "amoldar",
    "amoldados": "amoldar",
    "amoldamientos": "amoldamiento",
    "amoldamos": "amoldar",
    "amoldan": "amoldar",
    "amoldando": "amoldar",
    "amoldara": "amoldar",
    "amoldarais": "amoldar",
    "amoldaran": "amoldar",
    "amoldaras": "amoldar",
    "amoldare": "amoldar",
    "amoldareis": "amoldar",
    "amoldaremos": "amoldar",
    "amoldaren": "amoldar",
    "amoldares": "amoldar",
    "amoldaron": "amoldar",
    "amoldará": "amoldar",
    "amoldarán": "amoldar",
    "amoldarás": "amoldar",
    "amoldaré": "amoldar",
    "amoldaréis": "amoldar",
    "amoldaría": "amoldar",
    "amoldaríais": "amoldar",
    "amoldaríamos": "amoldar",
    "amoldarían": "amoldar",
    "amoldarías": "amoldar",
    "amoldas": "amoldar",
    "amoldase": "amoldar",
    "amoldaseis": "amoldar",
    "amoldasen": "amoldar",
    "amoldases": "amoldar",
    "amoldaste": "amoldar",
    "amoldasteis": "amoldar",
    "amolde": "amoldar",
    "amoldemos": "amoldar",
    "amolden": "amoldar",
    "amoldes": "amoldar",
    "amoldo": "amoldar",
    "amoldábamos": "amoldar",
    "amoldáis": "amoldar",
    "amoldáramos": "amoldar",
    "amoldáremos": "amoldar",
    "amoldásemos": "amoldar",
    "amoldé": "amoldar",
    "amoldéis": "amoldar",
    "amoldó": "amoldar",
    "amolemos": "amolar",
    "amolla": "amollar",
    "amollaba": "amollar",
    "amollabais": "amollar",
    "amollaban": "amollar",
    "amollabas": "amollar",
    "amollad": "amollar",
    "amollada": "amollar",
    "amolladas": "amollar",
    "amollado": "amollar",
    "amollados": "amollar",
    "amollamos": "amollar",
    "amollan": "amollar",
    "amollando": "amollar",
    "amollara": "amollar",
    "amollarais": "amollar",
    "amollaran": "amollar",
    "amollaras": "amollar",
    "amollare": "amollar",
    "amollareis": "amollar",
    "amollaremos": "amollar",
    "amollaren": "amollar",
    "amollares": "amollar",
    "amollaron": "amollar",
    "amollará": "amollar",
    "amollarán": "amollar",
    "amollarás": "amollar",
    "amollaré": "amollar",
    "amollaréis": "amollar",
    "amollaría": "amollar",
    "amollaríais": "amollar",
    "amollaríamos": "amollar",
    "amollarían": "amollar",
    "amollarías": "amollar",
    "amollas": "amollar",
    "amollase": "amollar",
    "amollaseis": "amollar",
    "amollasen": "amollar",
    "amollases": "amollar",
    "amollaste": "amollar",
    "amollasteis": "amollar",
    "amolle": "amollar",
    "amollemos": "amollar",
    "amollen": "amollar",
    "amolles": "amollar",
    "amollo": "amollar",
    "amollábamos": "amollar",
    "amolláis": "amollar",
    "amolláramos": "amollar",
    "amolláremos": "amollar",
    "amollásemos": "amollar",
    "amollé": "amollar",
    "amolléis": "amollar",
    "amolló": "amollar",
    "amolábamos": "amolar",
    "amoláis": "amolar",
    "amoláramos": "amolar",
    "amoláremos": "amolar",
    "amolásemos": "amolar",
    "amolé": "amolar",
    "amoléis": "amolar",
    "amoló": "amolar",
    "amona": "amonar",
    "amonaba": "amonar",
    "amonabais": "amonar",
    "amonaban": "amonar",
    "amonabas": "amonar",
    "amonad": "amonar",
    "amonada": "amonar",
    "amonadas": "amonar",
    "amonado": "amonar",
    "amonados": "amonar",
    "amonales": "amonal",
    "amonamos": "amonar",
    "amonan": "amonar",
    "amonando": "amonar",
    "amonara": "amonar",
    "amonarais": "amonar",
    "amonaran": "amonar",
    "amonaras": "amonar",
    "amonare": "amonar",
    "amonareis": "amonar",
    "amonaremos": "amonar",
    "amonaren": "amonar",
    "amonares": "amonar",
    "amonaron": "amonar",
    "amonará": "amonar",
    "amonarán": "amonar",
    "amonarás": "amonar",
    "amonaré": "amonar",
    "amonaréis": "amonar",
    "amonaría": "amonar",
    "amonaríais": "amonar",
    "amonaríamos": "amonar",
    "amonarían": "amonar",
    "amonarías": "amonar",
    "amonas": "amonar",
    "amonase": "amonar",
    "amonaseis": "amonar",
    "amonasen": "amonar",
    "amonases": "amonar",
    "amonaste": "amonar",
    "amonasteis": "amonar",
    "amone": "amonar",
    "amoneda": "amonedar",
    "amonedaba": "amonedar",
    "amonedabais": "amonedar",
    "amonedaban": "amonedar",
    "amonedabas": "amonedar",
    "amonedaciones": "amonedación",
    "amonedad": "amonedar",
    "amonedada": "amonedar",
    "amonedadas": "amonedar",
    "amonedado": "amonedar",
    "amonedados": "amonedar",
    "amonedamos": "amonedar",
    "amonedan": "amonedar",
    "amonedando": "amonedar",
    "amonedara": "amonedar",
    "amonedarais": "amonedar",
    "amonedaran": "amonedar",
    "amonedaras": "amonedar",
    "amonedare": "amonedar",
    "amonedareis": "amonedar",
    "amonedaremos": "amonedar",
    "amonedaren": "amonedar",
    "amonedares": "amonedar",
    "amonedaron": "amonedar",
    "amonedará": "amonedar",
    "amonedarán": "amonedar",
    "amonedarás": "amonedar",
    "amonedaré": "amonedar",
    "amonedaréis": "amonedar",
    "amonedaría": "amonedar",
    "amonedaríais": "amonedar",
    "amonedaríamos": "amonedar",
    "amonedarían": "amonedar",
    "amonedarías": "amonedar",
    "amonedas": "amonedar",
    "amonedase": "amonedar",
    "amonedaseis": "amonedar",
    "amonedasen": "amonedar",
    "amonedases": "amonedar",
    "amonedaste": "amonedar",
    "amonedasteis": "amonedar",
    "amonede": "amonedar",
    "amonedemos": "amonedar",
    "amoneden": "amonedar",
    "amonedes": "amonedar",
    "amonedo": "amonedar",
    "amonedábamos": "amonedar",
    "amonedáis": "amonedar",
    "amonedáramos": "amonedar",
    "amonedáremos": "amonedar",
    "amonedásemos": "amonedar",
    "amonedé": "amonedar",
    "amonedéis": "amonedar",
    "amonedó": "amonedar",
    "amonemos": "amonar",
    "amonen": "amonar",
    "amones": "amonar",
    "amonesta": "amonestar",
    "amonestaba": "amonestar",
    "amonestabais": "amonestar",
    "amonestaban": "amonestar",
    "amonestabas": "amonestar",
    "amonestaciones": "amonestación",
    "amonestad": "amonestar",
    "amonestada": "amonestar",
    "amonestadas": "amonestar",
    "amonestado": "amonestar",
    "amonestados": "amonestar",
    "amonestamientos": "amonestamiento",
    "amonestamos": "amonestar",
    "amonestan": "amonestar",
    "amonestando": "amonestar",
    "amonestara": "amonestar",
    "amonestarais": "amonestar",
    "amonestaran": "amonestar",
    "amonestaras": "amonestar",
    "amonestare": "amonestar",
    "amonestareis": "amonestar",
    "amonestaremos": "amonestar",
    "amonestaren": "amonestar",
    "amonestares": "amonestar",
    "amonestaron": "amonestar",
    "amonestará": "amonestar",
    "amonestarán": "amonestar",
    "amonestarás": "amonestar",
    "amonestaré": "amonestar",
    "amonestaréis": "amonestar",
    "amonestaría": "amonestar",
    "amonestaríais": "amonestar",
    "amonestaríamos": "amonestar",
    "amonestarían": "amonestar",
    "amonestarías": "amonestar",
    "amonestas": "amonestar",
    "amonestase": "amonestar",
    "amonestaseis": "amonestar",
    "amonestasen": "amonestar",
    "amonestases": "amonestar",
    "amonestaste": "amonestar",
    "amonestasteis": "amonestar",
    "amoneste": "amonestar",
    "amonestemos": "amonestar",
    "amonesten": "amonestar",
    "amonestes": "amonestar",
    "amonesto": "amonestar",
    "amonestábamos": "amonestar",
    "amonestáis": "amonestar",
    "amonestáramos": "amonestar",
    "amonestáremos": "amonestar",
    "amonestásemos": "amonestar",
    "amonesté": "amonestar",
    "amonestéis": "amonestar",
    "amonestó": "amonestar",
    "amoniacales": "amoniacal",
    "amoniaciones": "amoniación",
    "amoniacos": "amoniaco",
    "amonificaciones": "amonificación",
    "amonios": "amonio",
    "amonizaciones": "amonización",
    "amono": "amonar",
    "amontillada": "amontillado",
    "amontilladas": "amontillado",
    "amontillados": "amontillado",
    "amontona": "amontonar",
    "amontonaba": "amontonar",
    "amontonabais": "amontonar",
    "amontonaban": "amontonar",
    "amontonabas": "amontonar",
    "amontonad": "amontonar",
    "amontonada": "amontonar",
    "amontonadas": "amontonar",
    "amontonado": "amontonar",
    "amontonados": "amontonar",
    "amontonamientos": "amontonamiento",
    "amontonamos": "amontonar",
    "amontonan": "amontonar",
    "amontonando": "amontonar",
    "amontonara": "amontonar",
    "amontonarais": "amontonar",
    "amontonaran": "amontonar",
    "amontonaras": "amontonar",
    "amontonare": "amontonar",
    "amontonareis": "amontonar",
    "amontonaremos": "amontonar",
    "amontonaren": "amontonar",
    "amontonares": "amontonar",
    "amontonaron": "amontonar",
    "amontonará": "amontonar",
    "amontonarán": "amontonar",
    "amontonarás": "amontonar",
    "amontonaré": "amontonar",
    "amontonaréis": "amontonar",
    "amontonaría": "amontonar",
    "amontonaríais": "amontonar",
    "amontonaríamos": "amontonar",
    "amontonarían": "amontonar",
    "amontonarías": "amontonar",
    "amontonas": "amontonar",
    "amontonase": "amontonar",
    "amontonaseis": "amontonar",
    "amontonasen": "amontonar",
    "amontonases": "amontonar",
    "amontonaste": "amontonar",
    "amontonasteis": "amontonar",
    "amontone": "amontonar",
    "amontonemos": "amontonar",
    "amontonen": "amontonar",
    "amontones": "amontonar",
    "amontono": "amontonar",
    "amontonábamos": "amontonar",
    "amontonáis": "amontonar",
    "amontonáramos": "amontonar",
    "amontonáremos": "amontonar",
    "amontonásemos": "amontonar",
    "amontoné": "amontonar",
    "amontonéis": "amontonar",
    "amontonó": "amontonar",
    "amonábamos": "amonar",
    "amonáis": "amonar",
    "amonáramos": "amonar",
    "amonáremos": "amonar",
    "amonásemos": "amonar",
    "amoné": "amonar",
    "amonéis": "amonar",
    "amoníacos": "amoníaco",
    "amonó": "amonar",
    "amorales": "amoral",
    "amoralidades": "amoralidad",
    "amorata": "amoratar",
    "amorataba": "amoratar",
    "amoratabais": "amoratar",
    "amorataban": "amoratar",
    "amoratabas": "amoratar",
    "amoratad": "amoratar",
    "amoratada": "amoratar",
    "amoratadas": "amoratar",
    "amoratado": "amoratar",
    "amoratados": "amoratar",
    "amoratamos": "amoratar",
    "amoratan": "amoratar",
    "amoratando": "amoratar",
    "amoratara": "amoratar",
    "amoratarais": "amoratar",
    "amorataran": "amoratar",
    "amorataras": "amoratar",
    "amoratare": "amoratar",
    "amoratareis": "amoratar",
    "amorataremos": "amoratar",
    "amorataren": "amoratar",
    "amoratares": "amoratar",
    "amorataron": "amoratar",
    "amoratará": "amoratar",
    "amoratarán": "amoratar",
    "amoratarás": "amoratar",
    "amorataré": "amoratar",
    "amorataréis": "amoratar",
    "amorataría": "amoratar",
    "amorataríais": "amoratar",
    "amorataríamos": "amoratar",
    "amoratarían": "amoratar",
    "amoratarías": "amoratar",
    "amoratas": "amoratar",
    "amoratase": "amoratar",
    "amorataseis": "amoratar",
    "amoratasen": "amoratar",
    "amoratases": "amoratar",
    "amorataste": "amoratar",
    "amoratasteis": "amoratar",
    "amorate": "amoratar",
    "amoratemos": "amoratar",
    "amoraten": "amoratar",
    "amorates": "amoratar",
    "amorato": "amoratar",
    "amoratábamos": "amoratar",
    "amoratáis": "amoratar",
    "amoratáramos": "amoratar",
    "amoratáremos": "amoratar",
    "amoratásemos": "amoratar",
    "amoraté": "amoratar",
    "amoratéis": "amoratar",
    "amorató": "amoratar",
    "amorca": "amorcar",
    "amorcaba": "amorcar",
    "amorcabais": "amorcar",
    "amorcaban": "amorcar",
    "amorcabas": "amorcar",
    "amorcad": "amorcar",
    "amorcada": "amorcar",
    "amorcadas": "amorcar",
    "amorcado": "amorcar",
    "amorcados": "amorcar",
    "amorcamos": "amorcar",
    "amorcan": "amorcar",
    "amorcando": "amorcar",
    "amorcara": "amorcar",
    "amorcarais": "amorcar",
    "amorcaran": "amorcar",
    "amorcaras": "amorcar",
    "amorcare": "amorcar",
    "amorcareis": "amorcar",
    "amorcaremos": "amorcar",
    "amorcaren": "amorcar",
    "amorcares": "amorcar",
    "amorcaron": "amorcar",
    "amorcará": "amorcar",
    "amorcarán": "amorcar",
    "amorcarás": "amorcar",
    "amorcaré": "amorcar",
    "amorcaréis": "amorcar",
    "amorcaría": "amorcar",
    "amorcaríais": "amorcar",
    "amorcaríamos": "amorcar",
    "amorcarían": "amorcar",
    "amorcarías": "amorcar",
    "amorcas": "amorcar",
    "amorcase": "amorcar",
    "amorcaseis": "amorcar",
    "amorcasen": "amorcar",
    "amorcases": "amorcar",
    "amorcaste": "amorcar",
    "amorcasteis": "amorcar",
    "amorcillos": "amorcillo",
    "amorco": "amorcar",
    "amorcábamos": "amorcar",
    "amorcáis": "amorcar",
    "amorcáramos": "amorcar",
    "amorcáremos": "amorcar",
    "amorcásemos": "amorcar",
    "amorcó": "amorcar",
    "amordace": "amordazar",
    "amordacemos": "amordazar",
    "amordacen": "amordazar",
    "amordaces": "amordazar",
    "amordacé": "amordazar",
    "amordacéis": "amordazar",
    "amordaza": "amordazar",
    "amordazaba": "amordazar",
    "amordazabais": "amordazar",
    "amordazaban": "amordazar",
    "amordazabas": "amordazar",
    "amordazad": "amordazar",
    "amordazada": "amordazar",
    "amordazadas": "amordazar",
    "amordazado": "amordazar",
    "amordazados": "amordazar",
    "amordazamientos": "amordazamiento",
    "amordazamos": "amordazar",
    "amordazan": "amordazar",
    "amordazando": "amordazar",
    "amordazara": "amordazar",
    "amordazarais": "amordazar",
    "amordazaran": "amordazar",
    "amordazaras": "amordazar",
    "amordazare": "amordazar",
    "amordazareis": "amordazar",
    "amordazaremos": "amordazar",
    "amordazaren": "amordazar",
    "amordazares": "amordazar",
    "amordazaron": "amordazar",
    "amordazará": "amordazar",
    "amordazarán": "amordazar",
    "amordazarás": "amordazar",
    "amordazaré": "amordazar",
    "amordazaréis": "amordazar",
    "amordazaría": "amordazar",
    "amordazaríais": "amordazar",
    "amordazaríamos": "amordazar",
    "amordazarían": "amordazar",
    "amordazarías": "amordazar",
    "amordazas": "amordazar",
    "amordazase": "amordazar",
    "amordazaseis": "amordazar",
    "amordazasen": "amordazar",
    "amordazases": "amordazar",
    "amordazaste": "amordazar",
    "amordazasteis": "amordazar",
    "amordazo": "amordazar",
    "amordazábamos": "amordazar",
    "amordazáis": "amordazar",
    "amordazáramos": "amordazar",
    "amordazáremos": "amordazar",
    "amordazásemos": "amordazar",
    "amordazó": "amordazar",
    "amorece": "amorecer",
    "amoreced": "amorecer",
    "amorecemos": "amorecer",
    "amorecen": "amorecer",
    "amoreceremos": "amorecer",
    "amorecerá": "amorecer",
    "amorecerán": "amorecer",
    "amorecerás": "amorecer",
    "amoreceré": "amorecer",
    "amoreceréis": "amorecer",
    "amorecería": "amorecer",
    "amoreceríais": "amorecer",
    "amoreceríamos": "amorecer",
    "amorecerían": "amorecer",
    "amorecerías": "amorecer",
    "amoreces": "amorecer",
    "amorecida": "amorecer",
    "amorecidas": "amorecer",
    "amorecido": "amorecer",
    "amorecidos": "amorecer",
    "amoreciendo": "amorecer",
    "amoreciera": "amorecer",
    "amorecierais": "amorecer",
    "amorecieran": "amorecer",
    "amorecieras": "amorecer",
    "amoreciere": "amorecer",
    "amoreciereis": "amorecer",
    "amorecieren": "amorecer",
    "amorecieres": "amorecer",
    "amorecieron": "amorecer",
    "amoreciese": "amorecer",
    "amorecieseis": "amorecer",
    "amoreciesen": "amorecer",
    "amorecieses": "amorecer",
    "amorecimos": "amorecer",
    "amoreciste": "amorecer",
    "amorecisteis": "amorecer",
    "amoreciéramos": "amorecer",
    "amoreciéremos": "amorecer",
    "amoreciésemos": "amorecer",
    "amoreció": "amorecer",
    "amorecéis": "amorecer",
    "amorecí": "amorecer",
    "amorecía": "amorecer",
    "amorecíais": "amorecer",
    "amorecíamos": "amorecer",
    "amorecían": "amorecer",
    "amorecías": "amorecer",
    "amores": "amor",
    "amorezca": "amorecer",
    "amorezcamos": "amorecer",
    "amorezcan": "amorecer",
    "amorezcas": "amorecer",
    "amorezco": "amorecer",
    "amorezcáis": "amorecer",
    "amorfa": "amorfo",
    "amorfas": "amorfo",
    "amorfias": "amorfia",
    "amorfos": "amorfo",
    "amorilla": "amorillar",
    "amorillaba": "amorillar",
    "amorillabais": "amorillar",
    "amorillaban": "amorillar",
    "amorillabas": "amorillar",
    "amorillad": "amorillar",
    "amorillada": "amorillar",
    "amorilladas": "amorillar",
    "amorillado": "amorillar",
    "amorillados": "amorillar",
    "amorillamos": "amorillar",
    "amorillan": "amorillar",
    "amorillando": "amorillar",
    "amorillara": "amorillar",
    "amorillarais": "amorillar",
    "amorillaran": "amorillar",
    "amorillaras": "amorillar",
    "amorillare": "amorillar",
    "amorillareis": "amorillar",
    "amorillaremos": "amorillar",
    "amorillaren": "amorillar",
    "amorillares": "amorillar",
    "amorillaron": "amorillar",
    "amorillará": "amorillar",
    "amorillarán": "amorillar",
    "amorillarás": "amorillar",
    "amorillaré": "amorillar",
    "amorillaréis": "amorillar",
    "amorillaría": "amorillar",
    "amorillaríais": "amorillar",
    "amorillaríamos": "amorillar",
    "amorillarían": "amorillar",
    "amorillarías": "amorillar",
    "amorillas": "amorillar",
    "amorillase": "amorillar",
    "amorillaseis": "amorillar",
    "amorillasen": "amorillar",
    "amorillases": "amorillar",
    "amorillaste": "amorillar",
    "amorillasteis": "amorillar",
    "amorille": "amorillar",
    "amorillemos": "amorillar",
    "amorillen": "amorillar",
    "amorilles": "amorillar",
    "amorillo": "amorillar",
    "amorillábamos": "amorillar",
    "amorilláis": "amorillar",
    "amorilláramos": "amorillar",
    "amorilláremos": "amorillar",
    "amorillásemos": "amorillar",
    "amorillé": "amorillar",
    "amorilléis": "amorillar",
    "amorilló": "amorillar",
    "amoritas": "amorita",
    "amorosa": "amoroso",
    "amorosas": "amoroso",
    "amorosos": "amoroso",
    "amorque": "amorcar",
    "amorquemos": "amorcar",
    "amorquen": "amorcar",
    "amorques": "amorcar",
    "amorqué": "amorcar",
    "amorquéis": "amorcar",
    "amorra": "amorrar",
    "amorraba": "amorrar",
    "amorrabais": "amorrar",
    "amorraban": "amorrar",
    "amorrabas": "amorrar",
    "amorrad": "amorrar",
    "amorrada": "amorrar",
    "amorradas": "amorrar",
    "amorrado": "amorrar",
    "amorrados": "amorrar",
    "amorramientos": "amorramiento",
    "amorramos": "amorrar",
    "amorran": "amorrar",
    "amorrando": "amorrar",
    "amorrara": "amorrar",
    "amorrarais": "amorrar",
    "amorraran": "amorrar",
    "amorraras": "amorrar",
    "amorrare": "amorrar",
    "amorrareis": "amorrar",
    "amorraremos": "amorrar",
    "amorraren": "amorrar",
    "amorrares": "amorrar",
    "amorraron": "amorrar",
    "amorrará": "amorrar",
    "amorrarán": "amorrar",
    "amorrarás": "amorrar",
    "amorraré": "amorrar",
    "amorraréis": "amorrar",
    "amorraría": "amorrar",
    "amorraríais": "amorrar",
    "amorraríamos": "amorrar",
    "amorrarían": "amorrar",
    "amorrarías": "amorrar",
    "amorras": "amorrar",
    "amorrase": "amorrar",
    "amorraseis": "amorrar",
    "amorrasen": "amorrar",
    "amorrases": "amorrar",
    "amorraste": "amorrar",
    "amorrasteis": "amorrar",
    "amorre": "amorrar",
    "amorrea": "amorreo",
    "amorreas": "amorreo",
    "amorremos": "amorrar",
    "amorren": "amorrar",
    "amorreos": "amorreo",
    "amorres": "amorrar",
    "amorritas": "amorrita",
    "amorro": "amorrar",
    "amorrona": "amorronar",
    "amorronaba": "amorronar",
    "amorronabais": "amorronar",
    "amorronaban": "amorronar",
    "amorronabas": "amorronar",
    "amorronad": "amorronar",
    "amorronada": "amorronar",
    "amorronadas": "amorronar",
    "amorronado": "amorronar",
    "amorronados": "amorronar",
    "amorronamos": "amorronar",
    "amorronan": "amorronar",
    "amorronando": "amorronar",
    "amorronara": "amorronar",
    "amorronarais": "amorronar",
    "amorronaran": "amorronar",
    "amorronaras": "amorronar",
    "amorronare": "amorronar",
    "amorronareis": "amorronar",
    "amorronaremos": "amorronar",
    "amorronaren": "amorronar",
    "amorronares": "amorronar",
    "amorronaron": "amorronar",
    "amorronará": "amorronar",
    "amorronarán": "amorronar",
    "amorronarás": "amorronar",
    "amorronaré": "amorronar",
    "amorronaréis": "amorronar",
    "amorronaría": "amorronar",
    "amorronaríais": "amorronar",
    "amorronaríamos": "amorronar",
    "amorronarían": "amorronar",
    "amorronarías": "amorronar",
    "amorronas": "amorronar",
    "amorronase": "amorronar",
    "amorronaseis": "amorronar",
    "amorronasen": "amorronar",
    "amorronases": "amorronar",
    "amorronaste": "amorronar",
    "amorronasteis": "amorronar",
    "amorrone": "amorronar",
    "amorronemos": "amorronar",
    "amorronen": "amorronar",
    "amorrones": "amorronar",
    "amorrono": "amorronar",
    "amorronábamos": "amorronar",
    "amorronáis": "amorronar",
    "amorronáramos": "amorronar",
    "amorronáremos": "amorronar",
    "amorronásemos": "amorronar",
    "amorroné": "amorronar",
    "amorronéis": "amorronar",
    "amorronó": "amorronar",
    "amorrábamos": "amorrar",
    "amorráis": "amorrar",
    "amorráramos": "amorrar",
    "amorráremos": "amorrar",
    "amorrásemos": "amorrar",
    "amorré": "amorrar",
    "amorréis": "amorrar",
    "amorró": "amorrar",
    "amortaja": "amortajar",
    "amortajaba": "amortajar",
    "amortajabais": "amortajar",
    "amortajaban": "amortajar",
    "amortajabas": "amortajar",
    "amortajad": "amortajar",
    "amortajada": "amortajar",
    "amortajadas": "amortajar",
    "amortajado": "amortajar",
    "amortajadora": "amortajador",
    "amortajadoras": "amortajador",
    "amortajadores": "amortajador",
    "amortajados": "amortajar",
    "amortajamientos": "amortajamiento",
    "amortajamos": "amortajar",
    "amortajan": "amortajar",
    "amortajando": "amortajar",
    "amortajara": "amortajar",
    "amortajarais": "amortajar",
    "amortajaran": "amortajar",
    "amortajaras": "amortajar",
    "amortajare": "amortajar",
    "amortajareis": "amortajar",
    "amortajaremos": "amortajar",
    "amortajaren": "amortajar",
    "amortajares": "amortajar",
    "amortajaron": "amortajar",
    "amortajará": "amortajar",
    "amortajarán": "amortajar",
    "amortajarás": "amortajar",
    "amortajaré": "amortajar",
    "amortajaréis": "amortajar",
    "amortajaría": "amortajar",
    "amortajaríais": "amortajar",
    "amortajaríamos": "amortajar",
    "amortajarían": "amortajar",
    "amortajarías": "amortajar",
    "amortajas": "amortajar",
    "amortajase": "amortajar",
    "amortajaseis": "amortajar",
    "amortajasen": "amortajar",
    "amortajases": "amortajar",
    "amortajaste": "amortajar",
    "amortajasteis": "amortajar",
    "amortaje": "amortajar",
    "amortajemos": "amortajar",
    "amortajen": "amortajar",
    "amortajes": "amortajar",
    "amortajo": "amortajar",
    "amortajábamos": "amortajar",
    "amortajáis": "amortajar",
    "amortajáramos": "amortajar",
    "amortajáremos": "amortajar",
    "amortajásemos": "amortajar",
    "amortajé": "amortajar",
    "amortajéis": "amortajar",
    "amortajó": "amortajar",
    "amortece": "amortecer",
    "amorteced": "amortecer",
    "amortecemos": "amortecer",
    "amortecen": "amortecer",
    "amorteceremos": "amortecer",
    "amortecerá": "amortecer",
    "amortecerán": "amortecer",
    "amortecerás": "amortecer",
    "amorteceré": "amortecer",
    "amorteceréis": "amortecer",
    "amortecería": "amortecer",
    "amorteceríais": "amortecer",
    "amorteceríamos": "amortecer",
    "amortecerían": "amortecer",
    "amortecerías": "amortecer",
    "amorteces": "amortecer",
    "amortecida": "amortecer",
    "amortecidas": "amortecer",
    "amortecido": "amortecer",
    "amortecidos": "amortecer",
    "amorteciendo": "amortecer",
    "amorteciera": "amortecer",
    "amortecierais": "amortecer",
    "amortecieran": "amortecer",
    "amortecieras": "amortecer",
    "amorteciere": "amortecer",
    "amorteciereis": "amortecer",
    "amortecieren": "amortecer",
    "amortecieres": "amortecer",
    "amortecieron": "amortecer",
    "amorteciese": "amortecer",
    "amortecieseis": "amortecer",
    "amorteciesen": "amortecer",
    "amortecieses": "amortecer",
    "amortecimos": "amortecer",
    "amorteciste": "amortecer",
    "amortecisteis": "amortecer",
    "amorteciéramos": "amortecer",
    "amorteciéremos": "amortecer",
    "amorteciésemos": "amortecer",
    "amorteció": "amortecer",
    "amortecéis": "amortecer",
    "amortecí": "amortecer",
    "amortecía": "amortecer",
    "amortecíais": "amortecer",
    "amortecíamos": "amortecer",
    "amortecían": "amortecer",
    "amortecías": "amortecer",
    "amortezca": "amortecer",
    "amortezcamos": "amortecer",
    "amortezcan": "amortecer",
    "amortezcas": "amortecer",
    "amortezco": "amortecer",
    "amortezcáis": "amortecer",
    "amortice": "amortizar",
    "amorticemos": "amortizar",
    "amorticen": "amortizar",
    "amortices": "amortizar",
    "amorticé": "amortizar",
    "amorticéis": "amortizar",
    "amortigua": "amortiguar",
    "amortiguaba": "amortiguar",
    "amortiguabais": "amortiguar",
    "amortiguaban": "amortiguar",
    "amortiguabas": "amortiguar",
    "amortiguaciones": "amortiguación",
    "amortiguad": "amortiguar",
    "amortiguada": "amortiguar",
    "amortiguadas": "amortiguar",
    "amortiguado": "amortiguar",
    "amortiguadora": "amortiguador",
    "amortiguadoras": "amortiguador",
    "amortiguadores": "amortiguador",
    "amortiguados": "amortiguar",
    "amortiguamientos": "amortiguamiento",
    "amortiguamos": "amortiguar",
    "amortiguan": "amortiguar",
    "amortiguando": "amortiguar",
    "amortiguara": "amortiguar",
    "amortiguarais": "amortiguar",
    "amortiguaran": "amortiguar",
    "amortiguaras": "amortiguar",
    "amortiguare": "amortiguar",
    "amortiguareis": "amortiguar",
    "amortiguaremos": "amortiguar",
    "amortiguaren": "amortiguar",
    "amortiguares": "amortiguar",
    "amortiguaron": "amortiguar",
    "amortiguará": "amortiguar",
    "amortiguarán": "amortiguar",
    "amortiguarás": "amortiguar",
    "amortiguaré": "amortiguar",
    "amortiguaréis": "amortiguar",
    "amortiguaría": "amortiguar",
    "amortiguaríais": "amortiguar",
    "amortiguaríamos": "amortiguar",
    "amortiguarían": "amortiguar",
    "amortiguarías": "amortiguar",
    "amortiguas": "amortiguar",
    "amortiguase": "amortiguar",
    "amortiguaseis": "amortiguar",
    "amortiguasen": "amortiguar",
    "amortiguases": "amortiguar",
    "amortiguaste": "amortiguar",
    "amortiguasteis": "amortiguar",
    "amortiguo": "amortiguar",
    "amortiguábamos": "amortiguar",
    "amortiguáis": "amortiguar",
    "amortiguáramos": "amortiguar",
    "amortiguáremos": "amortiguar",
    "amortiguásemos": "amortiguar",
    "amortiguó": "amortiguar",
    "amortigüe": "amortiguar",
    "amortigüemos": "amortiguar",
    "amortigüen": "amortiguar",
    "amortigües": "amortiguar",
    "amortigüé": "amortiguar",
    "amortigüéis": "amortiguar",
    "amortiza": "amortizar",
    "amortizaba": "amortizar",
    "amortizabais": "amortizar",
    "amortizaban": "amortizar",
    "amortizabas": "amortizar",
    "amortizables": "amortizable",
    "amortizaciones": "amortización",
    "amortizad": "amortizar",
    "amortizada": "amortizar",
    "amortizadas": "amortizar",
    "amortizado": "amortizar",
    "amortizados": "amortizar",
    "amortizamos": "amortizar",
    "amortizan": "amortizar",
    "amortizando": "amortizar",
    "amortizara": "amortizar",
    "amortizarais": "amortizar",
    "amortizaran": "amortizar",
    "amortizaras": "amortizar",
    "amortizare": "amortizar",
    "amortizareis": "amortizar",
    "amortizaremos": "amortizar",
    "amortizaren": "amortizar",
    "amortizares": "amortizar",
    "amortizaron": "amortizar",
    "amortizará": "amortizar",
    "amortizarán": "amortizar",
    "amortizarás": "amortizar",
    "amortizaré": "amortizar",
    "amortizaréis": "amortizar",
    "amortizaría": "amortizar",
    "amortizaríais": "amortizar",
    "amortizaríamos": "amortizar",
    "amortizarían": "amortizar",
    "amortizarías": "amortizar",
    "amortizas": "amortizar",
    "amortizase": "amortizar",
    "amortizaseis": "amortizar",
    "amortizasen": "amortizar",
    "amortizases": "amortizar",
    "amortizaste": "amortizar",
    "amortizasteis": "amortizar",
    "amortizo": "amortizar",
    "amortizábamos": "amortizar",
    "amortizáis": "amortizar",
    "amortizáramos": "amortizar",
    "amortizáremos": "amortizar",
    "amortizásemos": "amortizar",
    "amortizó": "amortizar",
    "amoríos": "amorío",
    "amos": "amo",
    "amosca": "amoscar",
    "amoscaba": "amoscar",
    "amoscabais": "amoscar",
    "amoscaban": "amoscar",
    "amoscabas": "amoscar",
    "amoscad": "amoscar",
    "amoscada": "amoscar",
    "amoscadas": "amoscar",
    "amoscado": "amoscar",
    "amoscados": "amoscar",
    "amoscamientos": "amoscamiento",
    "amoscamos": "amoscar",
    "amoscan": "amoscar",
    "amoscando": "amoscar",
    "amoscara": "amoscar",
    "amoscarais": "amoscar",
    "amoscaran": "amoscar",
    "amoscaras": "amoscar",
    "amoscare": "amoscar",
    "amoscareis": "amoscar",
    "amoscaremos": "amoscar",
    "amoscaren": "amoscar",
    "amoscares": "amoscar",
    "amoscaron": "amoscar",
    "amoscará": "amoscar",
    "amoscarán": "amoscar",
    "amoscarás": "amoscar",
    "amoscaré": "amoscar",
    "amoscaréis": "amoscar",
    "amoscaría": "amoscar",
    "amoscaríais": "amoscar",
    "amoscaríamos": "amoscar",
    "amoscarían": "amoscar",
    "amoscarías": "amoscar",
    "amoscas": "amoscar",
    "amoscase": "amoscar",
    "amoscaseis": "amoscar",
    "amoscasen": "amoscar",
    "amoscases": "amoscar",
    "amoscaste": "amoscar",
    "amoscasteis": "amoscar",
    "amosco": "amoscar",
    "amoscábamos": "amoscar",
    "amoscáis": "amoscar",
    "amoscáramos": "amoscar",
    "amoscáremos": "amoscar",
    "amoscásemos": "amoscar",
    "amoscó": "amoscar",
    "amosque": "amoscar",
    "amosquemos": "amoscar",
    "amosquen": "amoscar",
    "amosques": "amoscar",
    "amosqué": "amoscar",
    "amosquéis": "amoscar",
    "amostace": "amostazar",
    "amostacemos": "amostazar",
    "amostacen": "amostazar",
    "amostaces": "amostazar",
    "amostacé": "amostazar",
    "amostacéis": "amostazar",
    "amostaza": "amostazar",
    "amostazaba": "amostazar",
    "amostazabais": "amostazar",
    "amostazaban": "amostazar",
    "amostazabas": "amostazar",
    "amostazad": "amostazar",
    "amostazada": "amostazar",
    "amostazadas": "amostazar",
    "amostazado": "amostazar",
    "amostazados": "amostazar",
    "amostazamos": "amostazar",
    "amostazan": "amostazar",
    "amostazando": "amostazar",
    "amostazara": "amostazar",
    "amostazarais": "amostazar",
    "amostazaran": "amostazar",
    "amostazaras": "amostazar",
    "amostazare": "amostazar",
    "amostazareis": "amostazar",
    "amostazaremos": "amostazar",
    "amostazaren": "amostazar",
    "amostazares": "amostazar",
    "amostazaron": "amostazar",
    "amostazará": "amostazar",
    "amostazarán": "amostazar",
    "amostazarás": "amostazar",
    "amostazaré": "amostazar",
    "amostazaréis": "amostazar",
    "amostazaría": "amostazar",
    "amostazaríais": "amostazar",
    "amostazaríamos": "amostazar",
    "amostazarían": "amostazar",
    "amostazarías": "amostazar",
    "amostazas": "amostazar",
    "amostazase": "amostazar",
    "amostazaseis": "amostazar",
    "amostazasen": "amostazar",
    "amostazases": "amostazar",
    "amostazaste": "amostazar",
    "amostazasteis": "amostazar",
    "amostazo": "amostazar",
    "amostazábamos": "amostazar",
    "amostazáis": "amostazar",
    "amostazáramos": "amostazar",
    "amostazáremos": "amostazar",
    "amostazásemos": "amostazar",
    "amostazó": "amostazar",
    "amotina": "amotinar",
    "amotinaba": "amotinar",
    "amotinabais": "amotinar",
    "amotinaban": "amotinar",
    "amotinabas": "amotinar",
    "amotinad": "amotinar",
    "amotinada": "amotinar",
    "amotinadas": "amotinar",
    "amotinado": "amotinar",
    "amotinadora": "amotinador",
    "amotinadoras": "amotinador",
    "amotinadores": "amotinador",
    "amotinados": "amotinar",
    "amotinamientos": "amotinamiento",
    "amotinamos": "amotinar",
    "amotinan": "amotinar",
    "amotinando": "amotinar",
    "amotinara": "amotinar",
    "amotinarais": "amotinar",
    "amotinaran": "amotinar",
    "amotinaras": "amotinar",
    "amotinare": "amotinar",
    "amotinareis": "amotinar",
    "amotinaremos": "amotinar",
    "amotinaren": "amotinar",
    "amotinares": "amotinar",
    "amotinaron": "amotinar",
    "amotinará": "amotinar",
    "amotinarán": "amotinar",
    "amotinarás": "amotinar",
    "amotinaré": "amotinar",
    "amotinaréis": "amotinar",
    "amotinaría": "amotinar",
    "amotinaríais": "amotinar",
    "amotinaríamos": "amotinar",
    "amotinarían": "amotinar",
    "amotinarías": "amotinar",
    "amotinas": "amotinar",
    "amotinase": "amotinar",
    "amotinaseis": "amotinar",
    "amotinasen": "amotinar",
    "amotinases": "amotinar",
    "amotinaste": "amotinar",
    "amotinasteis": "amotinar",
    "amotine": "amotinar",
    "amotinemos": "amotinar",
    "amotinen": "amotinar",
    "amotines": "amotinar",
    "amotino": "amotinar",
    "amotinábamos": "amotinar",
    "amotináis": "amotinar",
    "amotináramos": "amotinar",
    "amotináremos": "amotinar",
    "amotinásemos": "amotinar",
    "amotiné": "amotinar",
    "amotinéis": "amotinar",
    "amotinó": "amotinar",
    "amovamos": "amover",
    "amoved": "amover",
    "amovemos": "amover",
    "amoveremos": "amover",
    "amoverá": "amover",
    "amoverán": "amover",
    "amoverás": "amover",
    "amoveré": "amover",
    "amoveréis": "amover",
    "amovería": "amover",
    "amoveríais": "amover",
    "amoveríamos": "amover",
    "amoverían": "amover",
    "amoverías": "amover",
    "amovibilidades": "amovibilidad",
    "amovibles": "amovible",
    "amovida": "amover",
    "amovidas": "amover",
    "amovido": "amover",
    "amovidos": "amover",
    "amoviendo": "amover",
    "amoviera": "amover",
    "amovierais": "amover",
    "amovieran": "amover",
    "amovieras": "amover",
    "amoviere": "amover",
    "amoviereis": "amover",
    "amovieren": "amover",
    "amovieres": "amover",
    "amovieron": "amover",
    "amoviese": "amover",
    "amovieseis": "amover",
    "amoviesen": "amover",
    "amovieses": "amover",
    "amovilidades": "amovilidad",
    "amovimos": "amover",
    "amoviste": "amover",
    "amovisteis": "amover",
    "amoviéramos": "amover",
    "amoviéremos": "amover",
    "amoviésemos": "amover",
    "amovió": "amover",
    "amováis": "amover",
    "amovéis": "amover",
    "amoví": "amover",
    "amovía": "amover",
    "amovíais": "amover",
    "amovíamos": "amover",
    "amovían": "amover",
    "amovías": "amover",
    "ampara": "amparar",
    "amparaba": "amparar",
    "amparabais": "amparar",
    "amparaban": "amparar",
    "amparabas": "amparar",
    "amparad": "amparar",
    "amparada": "amparar",
    "amparadas": "amparar",
    "amparado": "amparar",
    "amparadora": "amparador",
    "amparadoras": "amparador",
    "amparadores": "amparador",
    "amparados": "amparar",
    "amparamos": "amparar",
    "amparan": "amparar",
    "amparando": "amparar",
    "amparara": "amparar",
    "ampararais": "amparar",
    "ampararan": "amparar",
    "ampararas": "amparar",
    "amparare": "amparar",
    "amparareis": "amparar",
    "ampararemos": "amparar",
    "ampararen": "amparar",
    "amparares": "amparar",
    "ampararon": "amparar",
    "amparará": "amparar",
    "ampararán": "amparar",
    "ampararás": "amparar",
    "ampararé": "amparar",
    "ampararéis": "amparar",
    "ampararía": "amparar",
    "ampararíais": "amparar",
    "ampararíamos": "amparar",
    "ampararían": "amparar",
    "ampararías": "amparar",
    "amparas": "amparar",
    "amparase": "amparar",
    "amparaseis": "amparar",
    "amparasen": "amparar",
    "amparases": "amparar",
    "amparaste": "amparar",
    "amparasteis": "amparar",
    "ampare": "amparar",
    "amparemos": "amparar",
    "amparen": "amparar",
    "ampares": "amparar",
    "amparo": "amparar",
    "amparos": "amparo",
    "amparábamos": "amparar",
    "amparáis": "amparar",
    "amparáramos": "amparar",
    "amparáremos": "amparar",
    "amparásemos": "amparar",
    "amparé": "amparar",
    "amparéis": "amparar",
    "amparó": "amparar",
    "ampelografías": "ampelografía",
    "ampelográfica": "ampelográfico",
    "ampelográficas": "ampelográfico",
    "ampelográficos": "ampelográfico",
    "ampelógrafa": "ampelógrafo",
    "ampelógrafas": "ampelógrafo",
    "ampelógrafos": "ampelógrafo",
    "amperes": "ampere",
    "amperiana": "amperiano",
    "amperianas": "amperiano",
    "amperianos": "amperiano",
    "amperihorímetros": "amperihorímetro",
    "amperimétrica": "amperimétrico",
    "amperimétricas": "amperimétrico",
    "amperimétricos": "amperimétrico",
    "amperio": "ampere",
    "amperios": "ampere",
    "amperometrías": "amperometría",
    "ampersands": "ampersand",
    "amperímetros": "amperímetro",
    "ampicilinas": "ampicilina",
    "amplexicaula": "amplexicaulo",
    "amplexicaulas": "amplexicaulo",
    "amplexicaulos": "amplexicaulo",
    "amplia": "amplio",
    "ampliaba": "ampliar",
    "ampliabais": "ampliar",
    "ampliaban": "ampliar",
    "ampliabas": "ampliar",
    "ampliables": "ampliable",
    "ampliaciones": "ampliación",
    "ampliad": "ampliar",
    "ampliada": "ampliar",
    "ampliadas": "ampliar",
    "ampliado": "ampliar",
    "ampliadora": "ampliador",
    "ampliadoras": "ampliador",
    "ampliadores": "ampliador",
    "ampliados": "ampliar",
    "ampliamos": "ampliar",
    "ampliando": "ampliar",
    "ampliara": "ampliar",
    "ampliarais": "ampliar",
    "ampliaran": "ampliar",
    "ampliaras": "ampliar",
    "ampliare": "ampliar",
    "ampliareis": "ampliar",
    "ampliaremos": "ampliar",
    "ampliaren": "ampliar",
    "ampliares": "ampliar",
    "ampliaron": "ampliar",
    "ampliará": "ampliar",
    "ampliarán": "ampliar",
    "ampliarás": "ampliar",
    "ampliaré": "ampliar",
    "ampliaréis": "ampliar",
    "ampliaría": "ampliar",
    "ampliaríais": "ampliar",
    "ampliaríamos": "ampliar",
    "ampliarían": "ampliar",
    "ampliarías": "ampliar",
    "amplias": "amplio",
    "ampliase": "ampliar",
    "ampliaseis": "ampliar",
    "ampliasen": "ampliar",
    "ampliases": "ampliar",
    "ampliaste": "ampliar",
    "ampliasteis": "ampliar",
    "ampliatoria": "ampliatorio",
    "ampliatorias": "ampliatorio",
    "ampliatorios": "ampliatorio",
    "amplidinas": "amplidina",
    "ampliemos": "ampliar",
    "amplifica": "amplificar",
    "amplificaba": "amplificar",
    "amplificabais": "amplificar",
    "amplificaban": "amplificar",
    "amplificabas": "amplificar",
    "amplificaciones": "amplificación",
    "amplificad": "amplificar",
    "amplificada": "amplificar",
    "amplificadas": "amplificar",
    "amplificado": "amplificar",
    "amplificadora": "amplificador",
    "amplificadoras": "amplificador",
    "amplificadores": "amplificador",
    "amplificados": "amplificar",
    "amplificamos": "amplificar",
    "amplifican": "amplificar",
    "amplificando": "amplificar",
    "amplificara": "amplificar",
    "amplificarais": "amplificar",
    "amplificaran": "amplificar",
    "amplificaras": "amplificar",
    "amplificare": "amplificar",
    "amplificareis": "amplificar",
    "amplificaremos": "amplificar",
    "amplificaren": "amplificar",
    "amplificares": "amplificar",
    "amplificaron": "amplificar",
    "amplificará": "amplificar",
    "amplificarán": "amplificar",
    "amplificarás": "amplificar",
    "amplificaré": "amplificar",
    "amplificaréis": "amplificar",
    "amplificaría": "amplificar",
    "amplificaríais": "amplificar",
    "amplificaríamos": "amplificar",
    "amplificarían": "amplificar",
    "amplificarías": "amplificar",
    "amplificas": "amplificar",
    "amplificase": "amplificar",
    "amplificaseis": "amplificar",
    "amplificasen": "amplificar",
    "amplificases": "amplificar",
    "amplificaste": "amplificar",
    "amplificasteis": "amplificar",
    "amplifico": "amplificar",
    "amplificábamos": "amplificar",
    "amplificáis": "amplificar",
    "amplificáramos": "amplificar",
    "amplificáremos": "amplificar",
    "amplificásemos": "amplificar",
    "amplificó": "amplificar",
    "amplifique": "amplificar",
    "amplifiquemos": "amplificar",
    "amplifiquen": "amplificar",
    "amplifiques": "amplificar",
    "amplifiqué": "amplificar",
    "amplifiquéis": "amplificar",
    "amplios": "amplio",
    "amplitudes": "amplitud",
    "ampliábamos": "ampliar",
    "ampliáis": "ampliar",
    "ampliáramos": "ampliar",
    "ampliáremos": "ampliar",
    "ampliásemos": "ampliar",
    "amplié": "ampliar",
    "ampliéis": "ampliar",
    "amplió": "ampliar",
    "amplía": "ampliar",
    "amplían": "ampliar",
    "amplías": "ampliar",
    "amplíe": "ampliar",
    "amplíen": "ampliar",
    "amplíes": "ampliar",
    "amplío": "ampliar",
    "ampolla": "ampollar",
    "ampollaba": "ampollar",
    "ampollabais": "ampollar",
    "ampollaban": "ampollar",
    "ampollabas": "ampollar",
    "ampollad": "ampollar",
    "ampollada": "ampollar",
    "ampolladas": "ampollar",
    "ampollado": "ampollar",
    "ampollados": "ampollar",
    "ampollamos": "ampollar",
    "ampollan": "ampollar",
    "ampollando": "ampollar",
    "ampollara": "ampollar",
    "ampollarais": "ampollar",
    "ampollaran": "ampollar",
    "ampollaras": "ampollar",
    "ampollare": "ampollar",
    "ampollareis": "ampollar",
    "ampollaremos": "ampollar",
    "ampollaren": "ampollar",
    "ampollares": "ampollar",
    "ampollaron": "ampollar",
    "ampollará": "ampollar",
    "ampollarán": "ampollar",
    "ampollarás": "ampollar",
    "ampollaré": "ampollar",
    "ampollaréis": "ampollar",
    "ampollaría": "ampollar",
    "ampollaríais": "ampollar",
    "ampollaríamos": "ampollar",
    "ampollarían": "ampollar",
    "ampollarías": "ampollar",
    "ampollas": "ampollar",
    "ampollase": "ampollar",
    "ampollaseis": "ampollar",
    "ampollasen": "ampollar",
    "ampollases": "ampollar",
    "ampollaste": "ampollar",
    "ampollasteis": "ampollar",
    "ampolle": "ampollar",
    "ampollemos": "ampollar",
    "ampollen": "ampollar",
    "ampolles": "ampollar",
    "ampolletas": "ampolleta",
    "ampollo": "ampollar",
    "ampollábamos": "ampollar",
    "ampolláis": "ampollar",
    "ampolláramos": "ampollar",
    "ampolláremos": "ampollar",
    "ampollásemos": "ampollar",
    "ampollé": "ampollar",
    "ampolléis": "ampollar",
    "ampolló": "ampollar",
    "ampos": "ampo",
    "ampulares": "ampular",
    "ampulosa": "ampuloso",
    "ampulosas": "ampuloso",
    "ampulosidades": "ampulosidad",
    "ampulosos": "ampuloso",
    "ampurdanesa": "ampurdanés",
    "ampurdanesas": "ampurdanés",
    "ampurdaneses": "ampurdanés",
    "amputa": "amputar",
    "amputaba": "amputar",
    "amputabais": "amputar",
    "amputaban": "amputar",
    "amputabas": "amputar",
    "amputaciones": "amputación",
    "amputaciónes": "amputación",
    "amputad": "amputar",
    "amputada": "amputar",
    "amputadas": "amputar",
    "amputado": "amputar",
    "amputados": "amputar",
    "amputamos": "amputar",
    "amputan": "amputar",
    "amputando": "amputar",
    "amputara": "amputar",
    "amputarais": "amputar",
    "amputaran": "amputar",
    "amputaras": "amputar",
    "amputare": "amputar",
    "amputareis": "amputar",
    "amputaremos": "amputar",
    "amputaren": "amputar",
    "amputares": "amputar",
    "amputaron": "amputar",
    "amputará": "amputar",
    "amputarán": "amputar",
    "amputarás": "amputar",
    "amputaré": "amputar",
    "amputaréis": "amputar",
    "amputaría": "amputar",
    "amputaríais": "amputar",
    "amputaríamos": "amputar",
    "amputarían": "amputar",
    "amputarías": "amputar",
    "amputas": "amputar",
    "amputase": "amputar",
    "amputaseis": "amputar",
    "amputasen": "amputar",
    "amputases": "amputar",
    "amputaste": "amputar",
    "amputasteis": "amputar",
    "ampute": "amputar",
    "amputemos": "amputar",
    "amputen": "amputar",
    "amputes": "amputar",
    "amputo": "amputar",
    "amputábamos": "amputar",
    "amputáis": "amputar",
    "amputáramos": "amputar",
    "amputáremos": "amputar",
    "amputásemos": "amputar",
    "amputé": "amputar",
    "amputéis": "amputar",
    "amputó": "amputar",
    "amstrongs": "amstrong",
    "amuebla": "amueblar",
    "amueblaba": "amueblar",
    "amueblabais": "amueblar",
    "amueblaban": "amueblar",
    "amueblabas": "amueblar",
    "amueblad": "amueblar",
    "amueblada": "amueblar",
    "amuebladas": "amueblar",
    "amueblado": "amueblar",
    "amueblados": "amueblar",
    "amueblamientos": "amueblamiento",
    "amueblamos": "amueblar",
    "amueblan": "amueblar",
    "amueblando": "amueblar",
    "amueblara": "amueblar",
    "amueblarais": "amueblar",
    "amueblaran": "amueblar",
    "amueblaras": "amueblar",
    "amueblare": "amueblar",
    "amueblareis": "amueblar",
    "amueblaremos": "amueblar",
    "amueblaren": "amueblar",
    "amueblares": "amueblar",
    "amueblaron": "amueblar",
    "amueblará": "amueblar",
    "amueblarán": "amueblar",
    "amueblarás": "amueblar",
    "amueblaré": "amueblar",
    "amueblaréis": "amueblar",
    "amueblaría": "amueblar",
    "amueblaríais": "amueblar",
    "amueblaríamos": "amueblar",
    "amueblarían": "amueblar",
    "amueblarías": "amueblar",
    "amueblas": "amueblar",
    "amueblase": "amueblar",
    "amueblaseis": "amueblar",
    "amueblasen": "amueblar",
    "amueblases": "amueblar",
    "amueblaste": "amueblar",
    "amueblasteis": "amueblar",
    "amueble": "amueblar",
    "amueblemos": "amueblar",
    "amueblen": "amueblar",
    "amuebles": "amueblar",
    "amueblo": "amueblar",
    "amueblábamos": "amueblar",
    "amuebláis": "amueblar",
    "amuebláramos": "amueblar",
    "amuebláremos": "amueblar",
    "amueblásemos": "amueblar",
    "amueblé": "amueblar",
    "amuebléis": "amueblar",
    "amuebló": "amueblar",
    "amuela": "amolar",
    "amuelan": "amolar",
    "amuelas": "amolar",
    "amuele": "amolar",
    "amuelen": "amolar",
    "amueles": "amolar",
    "amuelo": "amolar",
    "amueva": "amover",
    "amuevan": "amover",
    "amuevas": "amover",
    "amueve": "amover",
    "amueven": "amover",
    "amueves": "amover",
    "amuevo": "amover",
    "amugrona": "amugronar",
    "amugronaba": "amugronar",
    "amugronabais": "amugronar",
    "amugronaban": "amugronar",
    "amugronabas": "amugronar",
    "amugronad": "amugronar",
    "amugronada": "amugronar",
    "amugronadas": "amugronar",
    "amugronado": "amugronar",
    "amugronados": "amugronar",
    "amugronamos": "amugronar",
    "amugronan": "amugronar",
    "amugronando": "amugronar",
    "amugronara": "amugronar",
    "amugronarais": "amugronar",
    "amugronaran": "amugronar",
    "amugronaras": "amugronar",
    "amugronare": "amugronar",
    "amugronareis": "amugronar",
    "amugronaremos": "amugronar",
    "amugronaren": "amugronar",
    "amugronares": "amugronar",
    "amugronaron": "amugronar",
    "amugronará": "amugronar",
    "amugronarán": "amugronar",
    "amugronarás": "amugronar",
    "amugronaré": "amugronar",
    "amugronaréis": "amugronar",
    "amugronaría": "amugronar",
    "amugronaríais": "amugronar",
    "amugronaríamos": "amugronar",
    "amugronarían": "amugronar",
    "amugronarías": "amugronar",
    "amugronas": "amugronar",
    "amugronase": "amugronar",
    "amugronaseis": "amugronar",
    "amugronasen": "amugronar",
    "amugronases": "amugronar",
    "amugronaste": "amugronar",
    "amugronasteis": "amugronar",
    "amugrone": "amugronar",
    "amugronemos": "amugronar",
    "amugronen": "amugronar",
    "amugrones": "amugronar",
    "amugrono": "amugronar",
    "amugronábamos": "amugronar",
    "amugronáis": "amugronar",
    "amugronáramos": "amugronar",
    "amugronáremos": "amugronar",
    "amugronásemos": "amugronar",
    "amugroné": "amugronar",
    "amugronéis": "amugronar",
    "amugronó": "amugronar",
    "amuletos": "amuleto",
    "amuniciona": "amunicionar",
    "amunicionaba": "amunicionar",
    "amunicionabais": "amunicionar",
    "amunicionaban": "amunicionar",
    "amunicionabas": "amunicionar",
    "amunicionad": "amunicionar",
    "amunicionada": "amunicionar",
    "amunicionadas": "amunicionar",
    "amunicionado": "amunicionar",
    "amunicionados": "amunicionar",
    "amunicionamos": "amunicionar",
    "amunicionan": "amunicionar",
    "amunicionando": "amunicionar",
    "amunicionara": "amunicionar",
    "amunicionarais": "amunicionar",
    "amunicionaran": "amunicionar",
    "amunicionaras": "amunicionar",
    "amunicionare": "amunicionar",
    "amunicionareis": "amunicionar",
    "amunicionaremos": "amunicionar",
    "amunicionaren": "amunicionar",
    "amunicionares": "amunicionar",
    "amunicionaron": "amunicionar",
    "amunicionará": "amunicionar",
    "amunicionarán": "amunicionar",
    "amunicionarás": "amunicionar",
    "amunicionaré": "amunicionar",
    "amunicionaréis": "amunicionar",
    "amunicionaría": "amunicionar",
    "amunicionaríais": "amunicionar",
    "amunicionaríamos": "amunicionar",
    "amunicionarían": "amunicionar",
    "amunicionarías": "amunicionar",
    "amunicionas": "amunicionar",
    "amunicionase": "amunicionar",
    "amunicionaseis": "amunicionar",
    "amunicionasen": "amunicionar",
    "amunicionases": "amunicionar",
    "amunicionaste": "amunicionar",
    "amunicionasteis": "amunicionar",
    "amunicione": "amunicionar",
    "amunicionemos": "amunicionar",
    "amunicionen": "amunicionar",
    "amuniciones": "amunicionar",
    "amuniciono": "amunicionar",
    "amunicionábamos": "amunicionar",
    "amunicionáis": "amunicionar",
    "amunicionáramos": "amunicionar",
    "amunicionáremos": "amunicionar",
    "amunicionásemos": "amunicionar",
    "amunicioné": "amunicionar",
    "amunicionéis": "amunicionar",
    "amunicionó": "amunicionar",
    "amura": "amurar",
    "amuraba": "amurar",
    "amurabais": "amurar",
    "amuraban": "amurar",
    "amurabas": "amurar",
    "amurad": "amurar",
    "amurada": "amurar",
    "amuradas": "amurar",
    "amurado": "amurar",
    "amurados": "amurar",
    "amuralla": "amurallar",
    "amurallaba": "amurallar",
    "amurallabais": "amurallar",
    "amurallaban": "amurallar",
    "amurallabas": "amurallar",
    "amurallad": "amurallar",
    "amurallada": "amurallar",
    "amuralladas": "amurallar",
    "amurallado": "amurallar",
    "amurallados": "amurallar",
    "amurallamientos": "amurallamiento",
    "amurallamos": "amurallar",
    "amurallan": "amurallar",
    "amurallando": "amurallar",
    "amurallara": "amurallar",
    "amurallarais": "amurallar",
    "amurallaran": "amurallar",
    "amurallaras": "amurallar",
    "amurallare": "amurallar",
    "amurallareis": "amurallar",
    "amurallaremos": "amurallar",
    "amurallaren": "amurallar",
    "amurallares": "amurallar",
    "amurallaron": "amurallar",
    "amurallará": "amurallar",
    "amurallarán": "amurallar",
    "amurallarás": "amurallar",
    "amurallaré": "amurallar",
    "amurallaréis": "amurallar",
    "amurallaría": "amurallar",
    "amurallaríais": "amurallar",
    "amurallaríamos": "amurallar",
    "amurallarían": "amurallar",
    "amurallarías": "amurallar",
    "amurallas": "amurallar",
    "amurallase": "amurallar",
    "amurallaseis": "amurallar",
    "amurallasen": "amurallar",
    "amurallases": "amurallar",
    "amurallaste": "amurallar",
    "amurallasteis": "amurallar",
    "amuralle": "amurallar",
    "amurallemos": "amurallar",
    "amurallen": "amurallar",
    "amuralles": "amurallar",
    "amurallo": "amurallar",
    "amurallábamos": "amurallar",
    "amuralláis": "amurallar",
    "amuralláramos": "amurallar",
    "amuralláremos": "amurallar",
    "amurallásemos": "amurallar",
    "amurallé": "amurallar",
    "amuralléis": "amurallar",
    "amuralló": "amurallar",
    "amuramos": "amurar",
    "amuran": "amurar",
    "amurando": "amurar",
    "amurara": "amurar",
    "amurarais": "amurar",
    "amuraran": "amurar",
    "amuraras": "amurar",
    "amurare": "amurar",
    "amurareis": "amurar",
    "amuraremos": "amurar",
    "amuraren": "amurar",
    "amurares": "amurar",
    "amuraron": "amurar",
    "amurará": "amurar",
    "amurarán": "amurar",
    "amurarás": "amurar",
    "amuraré": "amurar",
    "amuraréis": "amurar",
    "amuraría": "amurar",
    "amuraríais": "amurar",
    "amuraríamos": "amurar",
    "amurarían": "amurar",
    "amurarías": "amurar",
    "amuras": "amurar",
    "amurase": "amurar",
    "amuraseis": "amurar",
    "amurasen": "amurar",
    "amurases": "amurar",
    "amuraste": "amurar",
    "amurasteis": "amurar",
    "amurca": "amurcar",
    "amurcaba": "amurcar",
    "amurcabais": "amurcar",
    "amurcaban": "amurcar",
    "amurcabas": "amurcar",
    "amurcad": "amurcar",
    "amurcada": "amurcar",
    "amurcadas": "amurcar",
    "amurcado": "amurcar",
    "amurcados": "amurcar",
    "amurcamos": "amurcar",
    "amurcan": "amurcar",
    "amurcando": "amurcar",
    "amurcara": "amurcar",
    "amurcarais": "amurcar",
    "amurcaran": "amurcar",
    "amurcaras": "amurcar",
    "amurcare": "amurcar",
    "amurcareis": "amurcar",
    "amurcaremos": "amurcar",
    "amurcaren": "amurcar",
    "amurcares": "amurcar",
    "amurcaron": "amurcar",
    "amurcará": "amurcar",
    "amurcarán": "amurcar",
    "amurcarás": "amurcar",
    "amurcaré": "amurcar",
    "amurcaréis": "amurcar",
    "amurcaría": "amurcar",
    "amurcaríais": "amurcar",
    "amurcaríamos": "amurcar",
    "amurcarían": "amurcar",
    "amurcarías": "amurcar",
    "amurcas": "amurcar",
    "amurcase": "amurcar",
    "amurcaseis": "amurcar",
    "amurcasen": "amurcar",
    "amurcases": "amurcar",
    "amurcaste": "amurcar",
    "amurcasteis": "amurcar",
    "amurco": "amurcar",
    "amurcábamos": "amurcar",
    "amurcáis": "amurcar",
    "amurcáramos": "amurcar",
    "amurcáremos": "amurcar",
    "amurcásemos": "amurcar",
    "amurcó": "amurcar",
    "amure": "amurar",
    "amuremos": "amurar",
    "amuren": "amurar",
    "amures": "amurar",
    "amuro": "amurar",
    "amurque": "amurcar",
    "amurquemos": "amurcar",
    "amurquen": "amurcar",
    "amurques": "amurcar",
    "amurqué": "amurcar",
    "amurquéis": "amurcar",
    "amurra": "amurrar",
    "amurraba": "amurrar",
    "amurrabais": "amurrar",
    "amurraban": "amurrar",
    "amurrabas": "amurrar",
    "amurrad": "amurrar",
    "amurrada": "amurrar",
    "amurradas": "amurrar",
    "amurrado": "amurrar",
    "amurrados": "amurrar",
    "amurramos": "amurrar",
    "amurran": "amurrar",
    "amurrando": "amurrar",
    "amurrara": "amurrar",
    "amurrarais": "amurrar",
    "amurraran": "amurrar",
    "amurraras": "amurrar",
    "amurrare": "amurrar",
    "amurrareis": "amurrar",
    "amurraremos": "amurrar",
    "amurraren": "amurrar",
    "amurrares": "amurrar",
    "amurraron": "amurrar",
    "amurrará": "amurrar",
    "amurrarán": "amurrar",
    "amurrarás": "amurrar",
    "amurraré": "amurrar",
    "amurraréis": "amurrar",
    "amurraría": "amurrar",
    "amurraríais": "amurrar",
    "amurraríamos": "amurrar",
    "amurrarían": "amurrar",
    "amurrarías": "amurrar",
    "amurras": "amurrar",
    "amurrase": "amurrar",
    "amurraseis": "amurrar",
    "amurrasen": "amurrar",
    "amurrases": "amurrar",
    "amurraste": "amurrar",
    "amurrasteis": "amurrar",
    "amurre": "amurrar",
    "amurremos": "amurrar",
    "amurren": "amurrar",
    "amurres": "amurrar",
    "amurria": "amurriar",
    "amurriaba": "amurriar",
    "amurriabais": "amurriar",
    "amurriaban": "amurriar",
    "amurriabas": "amurriar",
    "amurriad": "amurriar",
    "amurriada": "amurriar",
    "amurriadas": "amurriar",
    "amurriado": "amurriar",
    "amurriados": "amurriar",
    "amurriamos": "amurriar",
    "amurrian": "amurriar",
    "amurriando": "amurriar",
    "amurriara": "amurriar",
    "amurriarais": "amurriar",
    "amurriaran": "amurriar",
    "amurriaras": "amurriar",
    "amurriare": "amurriar",
    "amurriareis": "amurriar",
    "amurriaremos": "amurriar",
    "amurriaren": "amurriar",
    "amurriares": "amurriar",
    "amurriaron": "amurriar",
    "amurriará": "amurriar",
    "amurriarán": "amurriar",
    "amurriarás": "amurriar",
    "amurriaré": "amurriar",
    "amurriaréis": "amurriar",
    "amurriaría": "amurriar",
    "amurriaríais": "amurriar",
    "amurriaríamos": "amurriar",
    "amurriarían": "amurriar",
    "amurriarías": "amurriar",
    "amurrias": "amurriar",
    "amurriase": "amurriar",
    "amurriaseis": "amurriar",
    "amurriasen": "amurriar",
    "amurriases": "amurriar",
    "amurriaste": "amurriar",
    "amurriasteis": "amurriar",
    "amurrie": "amurriar",
    "amurriemos": "amurriar",
    "amurrien": "amurriar",
    "amurries": "amurriar",
    "amurrio": "amurriar",
    "amurriábamos": "amurriar",
    "amurriáis": "amurriar",
    "amurriáramos": "amurriar",
    "amurriáremos": "amurriar",
    "amurriásemos": "amurriar",
    "amurrié": "amurriar",
    "amurriéis": "amurriar",
    "amurriña": "amurriñar",
    "amurriñaba": "amurriñar",
    "amurriñabais": "amurriñar",
    "amurriñaban": "amurriñar",
    "amurriñabas": "amurriñar",
    "amurriñad": "amurriñar",
    "amurriñada": "amurriñar",
    "amurriñadas": "amurriñar",
    "amurriñado": "amurriñar",
    "amurriñados": "amurriñar",
    "amurriñamos": "amurriñar",
    "amurriñan": "amurriñar",
    "amurriñando": "amurriñar",
    "amurriñara": "amurriñar",
    "amurriñarais": "amurriñar",
    "amurriñaran": "amurriñar",
    "amurriñaras": "amurriñar",
    "amurriñare": "amurriñar",
    "amurriñareis": "amurriñar",
    "amurriñaremos": "amurriñar",
    "amurriñaren": "amurriñar",
    "amurriñares": "amurriñar",
    "amurriñaron": "amurriñar",
    "amurriñará": "amurriñar",
    "amurriñarán": "amurriñar",
    "amurriñarás": "amurriñar",
    "amurriñaré": "amurriñar",
    "amurriñaréis": "amurriñar",
    "amurriñaría": "amurriñar",
    "amurriñaríais": "amurriñar",
    "amurriñaríamos": "amurriñar",
    "amurriñarían": "amurriñar",
    "amurriñarías": "amurriñar",
    "amurriñas": "amurriñar",
    "amurriñase": "amurriñar",
    "amurriñaseis": "amurriñar",
    "amurriñasen": "amurriñar",
    "amurriñases": "amurriñar",
    "amurriñaste": "amurriñar",
    "amurriñasteis": "amurriñar",
    "amurriñe": "amurriñar",
    "amurriñemos": "amurriñar",
    "amurriñen": "amurriñar",
    "amurriñes": "amurriñar",
    "amurriño": "amurriñar",
    "amurriñábamos": "amurriñar",
    "amurriñáis": "amurriñar",
    "amurriñáramos": "amurriñar",
    "amurriñáremos": "amurriñar",
    "amurriñásemos": "amurriñar",
    "amurriñé": "amurriñar",
    "amurriñéis": "amurriñar",
    "amurriñó": "amurriñar",
    "amurrió": "amurriar",
    "amurro": "amurrar",
    "amurrábamos": "amurrar",
    "amurráis": "amurrar",
    "amurráramos": "amurrar",
    "amurráremos": "amurrar",
    "amurrásemos": "amurrar",
    "amurré": "amurrar",
    "amurréis": "amurrar",
    "amurró": "amurrar",
    "amurábamos": "amurar",
    "amuráis": "amurar",
    "amuráramos": "amurar",
    "amuráremos": "amurar",
    "amurásemos": "amurar",
    "amuré": "amurar",
    "amuréis": "amurar",
    "amuró": "amurar",
    "amusga": "amusgar",
    "amusgaba": "amusgar",
    "amusgabais": "amusgar",
    "amusgaban": "amusgar",
    "amusgabas": "amusgar",
    "amusgad": "amusgar",
    "amusgada": "amusgar",
    "amusgadas": "amusgar",
    "amusgado": "amusgar",
    "amusgados": "amusgar",
    "amusgamos": "amusgar",
    "amusgan": "amusgar",
    "amusgando": "amusgar",
    "amusgara": "amusgar",
    "amusgarais": "amusgar",
    "amusgaran": "amusgar",
    "amusgaras": "amusgar",
    "amusgare": "amusgar",
    "amusgareis": "amusgar",
    "amusgaremos": "amusgar",
    "amusgaren": "amusgar",
    "amusgares": "amusgar",
    "amusgaron": "amusgar",
    "amusgará": "amusgar",
    "amusgarán": "amusgar",
    "amusgarás": "amusgar",
    "amusgaré": "amusgar",
    "amusgaréis": "amusgar",
    "amusgaría": "amusgar",
    "amusgaríais": "amusgar",
    "amusgaríamos": "amusgar",
    "amusgarían": "amusgar",
    "amusgarías": "amusgar",
    "amusgas": "amusgar",
    "amusgase": "amusgar",
    "amusgaseis": "amusgar",
    "amusgasen": "amusgar",
    "amusgases": "amusgar",
    "amusgaste": "amusgar",
    "amusgasteis": "amusgar",
    "amusgo": "amusgar",
    "amusgue": "amusgar",
    "amusguemos": "amusgar",
    "amusguen": "amusgar",
    "amusgues": "amusgar",
    "amusgué": "amusgar",
    "amusguéis": "amusgar",
    "amusgábamos": "amusgar",
    "amusgáis": "amusgar",
    "amusgáramos": "amusgar",
    "amusgáremos": "amusgar",
    "amusgásemos": "amusgar",
    "amusgó": "amusgar",
    "amusias": "amusia",
    "amustia": "amustiar",
    "amustiaba": "amustiar",
    "amustiabais": "amustiar",
    "amustiaban": "amustiar",
    "amustiabas": "amustiar",
    "amustiad": "amustiar",
    "amustiada": "amustiar",
    "amustiadas": "amustiar",
    "amustiado": "amustiar",
    "amustiados": "amustiar",
    "amustiamos": "amustiar",
    "amustian": "amustiar",
    "amustiando": "amustiar",
    "amustiara": "amustiar",
    "amustiarais": "amustiar",
    "amustiaran": "amustiar",
    "amustiaras": "amustiar",
    "amustiare": "amustiar",
    "amustiareis": "amustiar",
    "amustiaremos": "amustiar",
    "amustiaren": "amustiar",
    "amustiares": "amustiar",
    "amustiaron": "amustiar",
    "amustiará": "amustiar",
    "amustiarán": "amustiar",
    "amustiarás": "amustiar",
    "amustiaré": "amustiar",
    "amustiaréis": "amustiar",
    "amustiaría": "amustiar",
    "amustiaríais": "amustiar",
    "amustiaríamos": "amustiar",
    "amustiarían": "amustiar",
    "amustiarías": "amustiar",
    "amustias": "amustiar",
    "amustiase": "amustiar",
    "amustiaseis": "amustiar",
    "amustiasen": "amustiar",
    "amustiases": "amustiar",
    "amustiaste": "amustiar",
    "amustiasteis": "amustiar",
    "amustie": "amustiar",
    "amustiemos": "amustiar",
    "amustien": "amustiar",
    "amusties": "amustiar",
    "amustio": "amustiar",
    "amustiábamos": "amustiar",
    "amustiáis": "amustiar",
    "amustiáramos": "amustiar",
    "amustiáremos": "amustiar",
    "amustiásemos": "amustiar",
    "amustié": "amustiar",
    "amustiéis": "amustiar",
    "amustió": "amustiar",
    "amábamos": "amar",
    "amáis": "amar",
    "amáramos": "amar",
    "amáremos": "amar",
    "amásemos": "amar",
    "amé": "amar",
    "améis": "amar",
    "amétropes": "amétrope",
    "amíctica": "amíctico",
    "amícticas": "amíctico",
    "amícticos": "amíctico",
    "amígdalas": "amígdala",
    "amínica": "amínico",
    "amínicas": "amínico",
    "amínicos": "amínico",
    "amó": "amar",
    "amónica": "amónico",
    "amónicas": "amónico",
    "amónicos": "amónico",
    "anabaptismos": "anabaptismo",
    "anabaptistas": "anabaptista",
    "anabasinas": "anabasina",
    "anabergitas": "anabergita",
    "anabolismos": "anabolismo",
    "anabolizantes": "anabolizante",
    "anabática": "anabático",
    "anabáticas": "anabático",
    "anabáticos": "anabático",
    "anacarada": "anacarado",
    "anacaradas": "anacarado",
    "anacarados": "anacarado",
    "anacardos": "anacardo",
    "anaclítica": "anaclítico",
    "anaclíticas": "anaclítico",
    "anaclíticos": "anaclítico",
    "anacolutos": "anacoluto",
    "anacondas": "anaconda",
    "anacoretas": "anacoreta",
    "anacorética": "anacorético",
    "anacoréticas": "anacorético",
    "anacoréticos": "anacorético",
    "anacos": "anaco",
    "anacreónticas": "anacreóntica",
    "anacromasias": "anacromasia",
    "anacronismos": "anacronismo",
    "anacrusas": "anacrusa",
    "anacrónica": "anacrónico",
    "anacrónicas": "anacrónico",
    "anacrónicos": "anacrónico",
    "anadea": "anadear",
    "anadeaba": "anadear",
    "anadeabais": "anadear",
    "anadeaban": "anadear",
    "anadeabas": "anadear",
    "anadead": "anadear",
    "anadeada": "anadear",
    "anadeadas": "anadear",
    "anadeado": "anadear",
    "anadeados": "anadear",
    "anadeamos": "anadear",
    "anadean": "anadear",
    "anadeando": "anadear",
    "anadeara": "anadear",
    "anadearais": "anadear",
    "anadearan": "anadear",
    "anadearas": "anadear",
    "anadeare": "anadear",
    "anadeareis": "anadear",
    "anadearemos": "anadear",
    "anadearen": "anadear",
    "anadeares": "anadear",
    "anadearon": "anadear",
    "anadeará": "anadear",
    "anadearán": "anadear",
    "anadearás": "anadear",
    "anadearé": "anadear",
    "anadearéis": "anadear",
    "anadearía": "anadear",
    "anadearíais": "anadear",
    "anadearíamos": "anadear",
    "anadearían": "anadear",
    "anadearías": "anadear",
    "anadeas": "anadear",
    "anadease": "anadear",
    "anadeaseis": "anadear",
    "anadeasen": "anadear",
    "anadeases": "anadear",
    "anadeaste": "anadear",
    "anadeasteis": "anadear",
    "anadee": "anadear",
    "anadeemos": "anadear",
    "anadeen": "anadear",
    "anadees": "anadear",
    "anadeo": "anadear",
    "anadeábamos": "anadear",
    "anadeáis": "anadear",
    "anadeáramos": "anadear",
    "anadeáremos": "anadear",
    "anadeásemos": "anadear",
    "anadeé": "anadear",
    "anadeéis": "anadear",
    "anadeó": "anadear",
    "anaerobia": "anaerobio",
    "anaerobias": "anaerobio",
    "anaerobios": "anaerobio",
    "anaeróbica": "anaeróbico",
    "anaeróbicas": "anaeróbico",
    "anaeróbicos": "anaeróbico",
    "anafases": "anafase",
    "anafes": "anafe",
    "anafilactogénica": "anafilactogénico",
    "anafilactogénicas": "anafilactogénico",
    "anafilactogénicos": "anafilactogénico",
    "anafilactoides": "anafilactoide",
    "anafilactotoxinas": "anafilactotoxina",
    "anafilactógenos": "anafilactógeno",
    "anafilatoxinas": "anafilatoxina",
    "anafilaxias": "anafilaxia",
    "anafiláctica": "anafiláctico",
    "anafilácticas": "anafiláctico",
    "anafilácticos": "anafiláctico",
    "anafiláxises": "anafilaxis",
    "anafrodisiaca": "anafrodisíaco",
    "anafrodisiacas": "anafrodisíaco",
    "anafrodisiacos": "anafrodisíaco",
    "anafrodisias": "anafrodisia",
    "anafrodisíaca": "anafrodisíaco",
    "anafrodisíacas": "anafrodisíaco",
    "anafrodisíacos": "anafrodisíaco",
    "anafroditas": "anafrodita",
    "anafórica": "anafórico",
    "anafóricas": "anafórico",
    "anafóricos": "anafórico",
    "anaglifos": "anaglifo",
    "anaglifoscopios": "anaglifoscopio",
    "anagliptografías": "anagliptografía",
    "anaglíptica": "anaglíptico",
    "anaglípticas": "anaglíptico",
    "anaglípticos": "anaglíptico",
    "anagoges": "anagoge",
    "anagogías": "anagogía",
    "anagramas": "anagrama",
    "analcimas": "analcima",
    "analectas": "analecta",
    "anales": "anal",
    "analfabeta": "analfabeto",
    "analfabetas": "analfabeto",
    "analfabetismos": "analfabetismo",
    "analfabetos": "analfabeto",
    "analgesias": "analgesia",
    "analgésica": "analgésico",
    "analgésicas": "analgésico",
    "analgésicos": "analgésico",
    "analice": "analizar",
    "analicemos": "analizar",
    "analicen": "analizar",
    "analices": "analizar",
    "analicé": "analizar",
    "analicéis": "analizar",
    "analistas": "analista",
    "analiticidades": "analiticidad",
    "analiza": "analizar",
    "analizaba": "analizar",
    "analizabais": "analizar",
    "analizaban": "analizar",
    "analizabas": "analizar",
    "analizables": "analizable",
    "analizad": "analizar",
    "analizada": "analizar",
    "analizadas": "analizar",
    "analizado": "analizar",
    "analizadora": "analizador",
    "analizadoras": "analizador",
    "analizadores": "analizador",
    "analizados": "analizar",
    "analizamos": "analizar",
    "analizan": "analizar",
    "analizando": "analizar",
    "analizara": "analizar",
    "analizarais": "analizar",
    "analizaran": "analizar",
    "analizaras": "analizar",
    "analizare": "analizar",
    "analizareis": "analizar",
    "analizaremos": "analizar",
    "analizaren": "analizar",
    "analizares": "analizar",
    "analizaron": "analizar",
    "analizará": "analizar",
    "analizarán": "analizar",
    "analizarás": "analizar",
    "analizaré": "analizar",
    "analizaréis": "analizar",
    "analizaría": "analizar",
    "analizaríais": "analizar",
    "analizaríamos": "analizar",
    "analizarían": "analizar",
    "analizarías": "analizar",
    "analizas": "analizar",
    "analizase": "analizar",
    "analizaseis": "analizar",
    "analizasen": "analizar",
    "analizases": "analizar",
    "analizaste": "analizar",
    "analizasteis": "analizar",
    "analizo": "analizar",
    "analizábamos": "analizar",
    "analizáis": "analizar",
    "analizáramos": "analizar",
    "analizáremos": "analizar",
    "analizásemos": "analizar",
    "analizó": "analizar",
    "analogías": "analogía",
    "analépticos": "analéptico",
    "analérgica": "analérgico",
    "analérgicas": "analérgico",
    "analérgicos": "analérgico",
    "analítica": "analítico",
    "analíticas": "analítico",
    "analíticos": "analítico",
    "analógica": "analógico",
    "analógicas": "analógico",
    "analógicos": "analógico",
    "anamórfica": "anamórfico",
    "anamórficas": "anamórfico",
    "anamórficos": "anamórfico",
    "ananases": "ananás",
    "anapelos": "anapelo",
    "anapestos": "anapesto",
    "anaplastias": "anaplastia",
    "anapéstica": "anapéstico",
    "anapésticas": "anapéstico",
    "anapésticos": "anapéstico",
    "anaquelerías": "anaquelería",
    "anaqueles": "anaquel",
    "anaranjada": "anaranjado",
    "anaranjadas": "anaranjado",
    "anaranjados": "anaranjado",
    "anarcosindicalismos": "anarcosindicalismo",
    "anarcosindicalistas": "anarcosindicalista",
    "anarmónica": "anarmónico",
    "anarmónicas": "anarmónico",
    "anarmónicos": "anarmónico",
    "anarquice": "anarquizar",
    "anarquicemos": "anarquizar",
    "anarquicen": "anarquizar",
    "anarquices": "anarquizar",
    "anarquicé": "anarquizar",
    "anarquicéis": "anarquizar",
    "anarquismos": "anarquismo",
    "anarquistas": "anarquista",
    "anarquiza": "anarquizar",
    "anarquizaba": "anarquizar",
    "anarquizabais": "anarquizar",
    "anarquizaban": "anarquizar",
    "anarquizabas": "anarquizar",
    "anarquizad": "anarquizar",
    "anarquizada": "anarquizar",
    "anarquizadas": "anarquizar",
    "anarquizado": "anarquizar",
    "anarquizados": "anarquizar",
    "anarquizamos": "anarquizar",
    "anarquizan": "anarquizar",
    "anarquizando": "anarquizar",
    "anarquizara": "anarquizar",
    "anarquizarais": "anarquizar",
    "anarquizaran": "anarquizar",
    "anarquizaras": "anarquizar",
    "anarquizare": "anarquizar",
    "anarquizareis": "anarquizar",
    "anarquizaremos": "anarquizar",
    "anarquizaren": "anarquizar",
    "anarquizares": "anarquizar",
    "anarquizaron": "anarquizar",
    "anarquizará": "anarquizar",
    "anarquizarán": "anarquizar",
    "anarquizarás": "anarquizar",
    "anarquizaré": "anarquizar",
    "anarquizaréis": "anarquizar",
    "anarquizaría": "anarquizar",
    "anarquizaríais": "anarquizar",
    "anarquizaríamos": "anarquizar",
    "anarquizarían": "anarquizar",
    "anarquizarías": "anarquizar",
    "anarquizas": "anarquizar",
    "anarquizase": "anarquizar",
    "anarquizaseis": "anarquizar",
    "anarquizasen": "anarquizar",
    "anarquizases": "anarquizar",
    "anarquizaste": "anarquizar",
    "anarquizasteis": "anarquizar",
    "anarquizo": "anarquizar",
    "anarquizábamos": "anarquizar",
    "anarquizáis": "anarquizar",
    "anarquizáramos": "anarquizar",
    "anarquizáremos": "anarquizar",
    "anarquizásemos": "anarquizar",
    "anarquizó": "anarquizar",
    "anarquías": "anarquía",
    "anas": "ana",
    "anastigmática": "anastigmático",
    "anastigmáticas": "anastigmático",
    "anastigmáticos": "anastigmático",
    "anastomosa": "anastomosar",
    "anastomosaba": "anastomosar",
    "anastomosabais": "anastomosar",
    "anastomosaban": "anastomosar",
    "anastomosabas": "anastomosar",
    "anastomosad": "anastomosar",
    "anastomosada": "anastomosar",
    "anastomosadas": "anastomosar",
    "anastomosado": "anastomosar",
    "anastomosados": "anastomosar",
    "anastomosamos": "anastomosar",
    "anastomosan": "anastomosar",
    "anastomosando": "anastomosar",
    "anastomosara": "anastomosar",
    "anastomosarais": "anastomosar",
    "anastomosaran": "anastomosar",
    "anastomosaras": "anastomosar",
    "anastomosare": "anastomosar",
    "anastomosareis": "anastomosar",
    "anastomosaremos": "anastomosar",
    "anastomosaren": "anastomosar",
    "anastomosares": "anastomosar",
    "anastomosaron": "anastomosar",
    "anastomosará": "anastomosar",
    "anastomosarán": "anastomosar",
    "anastomosarás": "anastomosar",
    "anastomosaré": "anastomosar",
    "anastomosaréis": "anastomosar",
    "anastomosaría": "anastomosar",
    "anastomosaríais": "anastomosar",
    "anastomosaríamos": "anastomosar",
    "anastomosarían": "anastomosar",
    "anastomosarías": "anastomosar",
    "anastomosas": "anastomosar",
    "anastomosase": "anastomosar",
    "anastomosaseis": "anastomosar",
    "anastomosasen": "anastomosar",
    "anastomosases": "anastomosar",
    "anastomosaste": "anastomosar",
    "anastomosasteis": "anastomosar",
    "anastomose": "anastomosar",
    "anastomosemos": "anastomosar",
    "anastomosen": "anastomosar",
    "anastomoses": "anastomosar",
    "anastomoso": "anastomosar",
    "anastomosábamos": "anastomosar",
    "anastomosáis": "anastomosar",
    "anastomosáramos": "anastomosar",
    "anastomosáremos": "anastomosar",
    "anastomosásemos": "anastomosar",
    "anastomosé": "anastomosar",
    "anastomoséis": "anastomosar",
    "anastomosó": "anastomosar",
    "anastomótica": "anastomótico",
    "anastomóticas": "anastomótico",
    "anastomóticos": "anastomótico",
    "anatas": "anata",
    "anatasas": "anatasa",
    "anatemas": "anatema",
    "anatematice": "anatematizar",
    "anatematicemos": "anatematizar",
    "anatematicen": "anatematizar",
    "anatematices": "anatematizar",
    "anatematicé": "anatematizar",
    "anatematicéis": "anatematizar",
    "anatematiza": "anatematizar",
    "anatematizaba": "anatematizar",
    "anatematizabais": "anatematizar",
    "anatematizaban": "anatematizar",
    "anatematizabas": "anatematizar",
    "anatematizad": "anatematizar",
    "anatematizada": "anatematizar",
    "anatematizadas": "anatematizar",
    "anatematizado": "anatematizar",
    "anatematizadora": "anatematizador",
    "anatematizadoras": "anatematizador",
    "anatematizadores": "anatematizador",
    "anatematizados": "anatematizar",
    "anatematizamos": "anatematizar",
    "anatematizan": "anatematizar",
    "anatematizando": "anatematizar",
    "anatematizara": "anatematizar",
    "anatematizarais": "anatematizar",
    "anatematizaran": "anatematizar",
    "anatematizaras": "anatematizar",
    "anatematizare": "anatematizar",
    "anatematizareis": "anatematizar",
    "anatematizaremos": "anatematizar",
    "anatematizaren": "anatematizar",
    "anatematizares": "anatematizar",
    "anatematizaron": "anatematizar",
    "anatematizará": "anatematizar",
    "anatematizarán": "anatematizar",
    "anatematizarás": "anatematizar",
    "anatematizaré": "anatematizar",
    "anatematizaréis": "anatematizar",
    "anatematizaría": "anatematizar",
    "anatematizaríais": "anatematizar",
    "anatematizaríamos": "anatematizar",
    "anatematizarían": "anatematizar",
    "anatematizarías": "anatematizar",
    "anatematizas": "anatematizar",
    "anatematizase": "anatematizar",
    "anatematizaseis": "anatematizar",
    "anatematizasen": "anatematizar",
    "anatematizases": "anatematizar",
    "anatematizaste": "anatematizar",
    "anatematizasteis": "anatematizar",
    "anatematizo": "anatematizar",
    "anatematizábamos": "anatematizar",
    "anatematizáis": "anatematizar",
    "anatematizáramos": "anatematizar",
    "anatematizáremos": "anatematizar",
    "anatematizásemos": "anatematizar",
    "anatematizó": "anatematizar",
    "anatexias": "anatexia",
    "anatocismos": "anatocismo",
    "anatomice": "anatomizar",
    "anatomicemos": "anatomizar",
    "anatomicen": "anatomizar",
    "anatomices": "anatomizar",
    "anatomicoforenses": "anatomicoforense",
    "anatomicé": "anatomizar",
    "anatomicéis": "anatomizar",
    "anatomistas": "anatomista",
    "anatomiza": "anatomizar",
    "anatomizaba": "anatomizar",
    "anatomizabais": "anatomizar",
    "anatomizaban": "anatomizar",
    "anatomizabas": "anatomizar",
    "anatomizad": "anatomizar",
    "anatomizada": "anatomizar",
    "anatomizadas": "anatomizar",
    "anatomizado": "anatomizar",
    "anatomizados": "anatomizar",
    "anatomizamos": "anatomizar",
    "anatomizan": "anatomizar",
    "anatomizando": "anatomizar",
    "anatomizara": "anatomizar",
    "anatomizarais": "anatomizar",
    "anatomizaran": "anatomizar",
    "anatomizaras": "anatomizar",
    "anatomizare": "anatomizar",
    "anatomizareis": "anatomizar",
    "anatomizaremos": "anatomizar",
    "anatomizaren": "anatomizar",
    "anatomizares": "anatomizar",
    "anatomizaron": "anatomizar",
    "anatomizará": "anatomizar",
    "anatomizarán": "anatomizar",
    "anatomizarás": "anatomizar",
    "anatomizaré": "anatomizar",
    "anatomizaréis": "anatomizar",
    "anatomizaría": "anatomizar",
    "anatomizaríais": "anatomizar",
    "anatomizaríamos": "anatomizar",
    "anatomizarían": "anatomizar",
    "anatomizarías": "anatomizar",
    "anatomizas": "anatomizar",
    "anatomizase": "anatomizar",
    "anatomizaseis": "anatomizar",
    "anatomizasen": "anatomizar",
    "anatomizases": "anatomizar",
    "anatomizaste": "anatomizar",
    "anatomizasteis": "anatomizar",
    "anatomizo": "anatomizar",
    "anatomizábamos": "anatomizar",
    "anatomizáis": "anatomizar",
    "anatomizáramos": "anatomizar",
    "anatomizáremos": "anatomizar",
    "anatomizásemos": "anatomizar",
    "anatomizó": "anatomizar",
    "anatomías": "anatomía",
    "anatos": "anato",
    "anatoxinas": "anatoxina",
    "anatómica": "anatómico",
    "anatómicas": "anatómico",
    "anatómicos": "anatómico",
    "ancas": "anca",
    "ancestrales": "ancestral",
    "ancestros": "ancestro",
    "ancha": "ancho",
    "anchas": "ancho",
    "anchoas": "anchoa",
    "anchos": "ancho",
    "anchovetas": "anchoveta",
    "anchuras": "anchura",
    "anchurosa": "anchuroso",
    "anchurosas": "anchuroso",
    "anchurosos": "anchuroso",
    "anciana": "anciano",
    "ancianas": "anciano",
    "ancianidades": "ancianidad",
    "ancianos": "anciano",
    "ancla": "anclar",
    "anclaba": "anclar",
    "anclabais": "anclar",
    "anclaban": "anclar",
    "anclabas": "anclar",
    "anclad": "anclar",
    "anclada": "anclar",
    "ancladas": "anclar",
    "ancladeros": "ancladero",
    "anclado": "anclar",
    "anclados": "anclar",
    "anclajes": "anclaje",
    "anclamos": "anclar",
    "anclan": "anclar",
    "anclando": "anclar",
    "anclara": "anclar",
    "anclarais": "anclar",
    "anclaran": "anclar",
    "anclaras": "anclar",
    "anclare": "anclar",
    "anclareis": "anclar",
    "anclaremos": "anclar",
    "anclaren": "anclar",
    "anclares": "anclar",
    "anclaron": "anclar",
    "anclará": "anclar",
    "anclarán": "anclar",
    "anclarás": "anclar",
    "anclaré": "anclar",
    "anclaréis": "anclar",
    "anclaría": "anclar",
    "anclaríais": "anclar",
    "anclaríamos": "anclar",
    "anclarían": "anclar",
    "anclarías": "anclar",
    "anclas": "anclar",
    "anclase": "anclar",
    "anclaseis": "anclar",
    "anclasen": "anclar",
    "anclases": "anclar",
    "anclaste": "anclar",
    "anclasteis": "anclar",
    "ancle": "anclar",
    "anclemos": "anclar",
    "anclen": "anclar",
    "ancles": "anclar",
    "anclo": "anclar",
    "anclotes": "anclote",
    "anclábamos": "anclar",
    "ancláis": "anclar",
    "ancláramos": "anclar",
    "ancláremos": "anclar",
    "anclásemos": "anclar",
    "anclé": "anclar",
    "ancléis": "anclar",
    "ancló": "anclar",
    "ancones": "ancón",
    "ancora": "ancorar",
    "ancoraba": "ancorar",
    "ancorabais": "ancorar",
    "ancoraban": "ancorar",
    "ancorabas": "ancorar",
    "ancorad": "ancorar",
    "ancorada": "ancorar",
    "ancoradas": "ancorar",
    "ancorado": "ancorar",
    "ancorados": "ancorar",
    "ancoramos": "ancorar",
    "ancoran": "ancorar",
    "ancorando": "ancorar",
    "ancorara": "ancorar",
    "ancorarais": "ancorar",
    "ancoraran": "ancorar",
    "ancoraras": "ancorar",
    "ancorare": "ancorar",
    "ancorareis": "ancorar",
    "ancoraremos": "ancorar",
    "ancoraren": "ancorar",
    "ancorares": "ancorar",
    "ancoraron": "ancorar",
    "ancorará": "ancorar",
    "ancorarán": "ancorar",
    "ancorarás": "ancorar",
    "ancoraré": "ancorar",
    "ancoraréis": "ancorar",
    "ancoraría": "ancorar",
    "ancoraríais": "ancorar",
    "ancoraríamos": "ancorar",
    "ancorarían": "ancorar",
    "ancorarías": "ancorar",
    "ancoras": "ancorar",
    "ancorase": "ancorar",
    "ancoraseis": "ancorar",
    "ancorasen": "ancorar",
    "ancorases": "ancorar",
    "ancoraste": "ancorar",
    "ancorasteis": "ancorar",
    "ancorcas": "ancorca",
    "ancore": "ancorar",
    "ancoreles": "ancorel",
    "ancoremos": "ancorar",
    "ancoren": "ancorar",
    "ancores": "ancorar",
    "ancoro": "ancorar",
    "ancorábamos": "ancorar",
    "ancoráis": "ancorar",
    "ancoráramos": "ancorar",
    "ancoráremos": "ancorar",
    "ancorásemos": "ancorar",
    "ancoré": "ancorar",
    "ancoréis": "ancorar",
    "ancoró": "ancorar",
    "ancuviñas": "ancuviña",
    "ancónea": "ancóneo",
    "ancóneas": "ancóneo",
    "ancóneos": "ancóneo",
    "anda": "andar",
    "andaba": "andar",
    "andabais": "andar",
    "andaban": "andar",
    "andabas": "andar",
    "andad": "andar",
    "andada": "andar",
    "andadas": "andar",
    "andado": "andar",
    "andadora": "andador",
    "andadoras": "andador",
    "andadores": "andador",
    "andados": "andar",
    "andaduras": "andadura",
    "andaluca": "andaluz",
    "andalucas": "andaluz",
    "andaluces": "andaluz",
    "andalucistas": "andalucista",
    "andalucitas": "andalucita",
    "andaluza": "andaluz",
    "andaluzadas": "andaluzada",
    "andaluzas": "andaluz",
    "andaluzes": "andaluz",
    "andamiadas": "andamiada",
    "andamiajes": "andamiaje",
    "andamios": "andamio",
    "andamos": "andar",
    "andan": "andar",
    "andanadas": "andanada",
    "andanas": "andana",
    "andando": "andar",
    "andantes": "andante",
    "andantinos": "andantino",
    "andanzas": "andanza",
    "andarajes": "andaraje",
    "andaremos": "andar",
    "andares": "andar",
    "andariega": "andariego",
    "andariegas": "andariego",
    "andariegos": "andariego",
    "andarina": "andarín",
    "andarinas": "andarín",
    "andarines": "andarín",
    "andariveles": "andarivel",
    "andará": "andar",
    "andarán": "andar",
    "andarás": "andar",
    "andaré": "andar",
    "andaréis": "andar",
    "andaría": "andar",
    "andaríais": "andar",
    "andaríamos": "andar",
    "andarían": "andar",
    "andarías": "andar",
    "andas": "andar",
    "ande": "andar",
    "andeles": "andel",
    "andemos": "andar",
    "anden": "andar",
    "andenes": "andén",
    "andes": "andar",
    "andesinas": "andesina",
    "andesitas": "andesita",
    "andesítica": "andesítico",
    "andesíticas": "andesítico",
    "andesíticos": "andesítico",
    "andideportiva": "andideportivo",
    "andideportivas": "andideportivo",
    "andideportivos": "andideportivo",
    "andina": "andino",
    "andinas": "andino",
    "andinos": "andino",
    "ando": "andar",
    "andorgas": "andorga",
    "andorrana": "andorrano",
    "andorranas": "andorrano",
    "andorranos": "andorrano",
    "andorrea": "andorrear",
    "andorreaba": "andorrear",
    "andorreabais": "andorrear",
    "andorreaban": "andorrear",
    "andorreabas": "andorrear",
    "andorread": "andorrear",
    "andorreada": "andorrear",
    "andorreadas": "andorrear",
    "andorreado": "andorrear",
    "andorreados": "andorrear",
    "andorreamos": "andorrear",
    "andorrean": "andorrear",
    "andorreando": "andorrear",
    "andorreara": "andorrear",
    "andorrearais": "andorrear",
    "andorrearan": "andorrear",
    "andorrearas": "andorrear",
    "andorreare": "andorrear",
    "andorreareis": "andorrear",
    "andorrearemos": "andorrear",
    "andorrearen": "andorrear",
    "andorreares": "andorrear",
    "andorrearon": "andorrear",
    "andorreará": "andorrear",
    "andorrearán": "andorrear",
    "andorrearás": "andorrear",
    "andorrearé": "andorrear",
    "andorrearéis": "andorrear",
    "andorrearía": "andorrear",
    "andorrearíais": "andorrear",
    "andorrearíamos": "andorrear",
    "andorrearían": "andorrear",
    "andorrearías": "andorrear",
    "andorreas": "andorrear",
    "andorrease": "andorrear",
    "andorreaseis": "andorrear",
    "andorreasen": "andorrear",
    "andorreases": "andorrear",
    "andorreaste": "andorrear",
    "andorreasteis": "andorrear",
    "andorree": "andorrear",
    "andorreemos": "andorrear",
    "andorreen": "andorrear",
    "andorrees": "andorrear",
    "andorreo": "andorrear",
    "andorreábamos": "andorrear",
    "andorreáis": "andorrear",
    "andorreáramos": "andorrear",
    "andorreáremos": "andorrear",
    "andorreásemos": "andorrear",
    "andorreé": "andorrear",
    "andorreéis": "andorrear",
    "andorreó": "andorrear",
    "andraditas": "andradita",
    "andrajos": "andrajo",
    "andrajosa": "andrajoso",
    "andrajosas": "andrajoso",
    "andrajosos": "andrajoso",
    "androceos": "androceo",
    "androfobias": "androfobia",
    "androides": "androide",
    "andrologías": "andrología",
    "andronas": "androna",
    "andropausias": "andropausia",
    "androsteronas": "androsterona",
    "andrógenos": "andrógeno",
    "andrógina": "andrógino",
    "andróginas": "andrógino",
    "andróginos": "andrógino",
    "andróminas": "andrómina",
    "andullos": "andullo",
    "andurriales": "andurrial",
    "anduve": "andar",
    "anduviera": "andar",
    "anduvierais": "andar",
    "anduvieran": "andar",
    "anduvieras": "andar",
    "anduviere": "andar",
    "anduviereis": "andar",
    "anduvieren": "andar",
    "anduvieres": "andar",
    "anduvieron": "andar",
    "anduviese": "andar",
    "anduvieseis": "andar",
    "anduviesen": "andar",
    "anduvieses": "andar",
    "anduvimos": "andar",
    "anduviste": "andar",
    "anduvisteis": "andar",
    "anduviéramos": "andar",
    "anduviéremos": "andar",
    "anduviésemos": "andar",
    "anduvo": "andar",
    "andábamos": "andar",
    "andáis": "andar",
    "andéis": "andar",
    "aneajes": "aneaje",
    "aneas": "anea",
    "aneblaba": "aneblar",
    "aneblabais": "aneblar",
    "aneblaban": "aneblar",
    "aneblabas": "aneblar",
    "aneblad": "aneblar",
    "aneblada": "aneblar",
    "anebladas": "aneblar",
    "aneblado": "aneblar",
    "aneblados": "aneblar",
    "aneblamos": "aneblar",
    "aneblando": "aneblar",
    "aneblara": "aneblar",
    "aneblarais": "aneblar",
    "aneblaran": "aneblar",
    "aneblaras": "aneblar",
    "aneblare": "aneblar",
    "aneblareis": "aneblar",
    "aneblaremos": "aneblar",
    "aneblaren": "aneblar",
    "aneblares": "aneblar",
    "aneblaron": "aneblar",
    "aneblará": "aneblar",
    "aneblarán": "aneblar",
    "aneblarás": "aneblar",
    "aneblaré": "aneblar",
    "aneblaréis": "aneblar",
    "aneblaría": "aneblar",
    "aneblaríais": "aneblar",
    "aneblaríamos": "aneblar",
    "aneblarían": "aneblar",
    "aneblarías": "aneblar",
    "aneblase": "aneblar",
    "aneblaseis": "aneblar",
    "aneblasen": "aneblar",
    "aneblases": "aneblar",
    "aneblaste": "aneblar",
    "aneblasteis": "aneblar",
    "aneblemos": "aneblar",
    "aneblábamos": "aneblar",
    "anebláis": "aneblar",
    "anebláramos": "aneblar",
    "anebláremos": "aneblar",
    "aneblásemos": "aneblar",
    "aneblé": "aneblar",
    "anebléis": "aneblar",
    "anebló": "aneblar",
    "anecdotarios": "anecdotario",
    "anecdotistas": "anecdotista",
    "anecdótica": "anecdótico",
    "anecdóticas": "anecdótico",
    "anecdóticos": "anecdótico",
    "anecoica": "anecoico",
    "anecoicas": "anecoico",
    "anecoicos": "anecoico",
    "anecoides": "anecoide",
    "anega": "anegar",
    "anegaba": "anegar",
    "anegabais": "anegar",
    "anegaban": "anegar",
    "anegabas": "anegar",
    "anegaciones": "anegación",
    "anegad": "anegar",
    "anegada": "anegar",
    "anegadas": "anegar",
    "anegado": "anegar",
    "anegados": "anegar",
    "anegamos": "anegar",
    "anegan": "anegar",
    "anegando": "anegar",
    "anegara": "anegar",
    "anegarais": "anegar",
    "anegaran": "anegar",
    "anegaras": "anegar",
    "anegare": "anegar",
    "anegareis": "anegar",
    "anegaremos": "anegar",
    "anegaren": "anegar",
    "anegares": "anegar",
    "anegaron": "anegar",
    "anegará": "anegar",
    "anegarán": "anegar",
    "anegarás": "anegar",
    "anegaré": "anegar",
    "anegaréis": "anegar",
    "anegaría": "anegar",
    "anegaríais": "anegar",
    "anegaríamos": "anegar",
    "anegarían": "anegar",
    "anegarías": "anegar",
    "anegas": "anegar",
    "anegase": "anegar",
    "anegaseis": "anegar",
    "anegasen": "anegar",
    "anegases": "anegar",
    "anegaste": "anegar",
    "anegasteis": "anegar",
    "anego": "anegar",
    "anegue": "anegar",
    "aneguemos": "anegar",
    "aneguen": "anegar",
    "anegues": "anegar",
    "anegué": "anegar",
    "aneguéis": "anegar",
    "anegábamos": "anegar",
    "anegáis": "anegar",
    "anegáramos": "anegar",
    "anegáremos": "anegar",
    "anegásemos": "anegar",
    "anegó": "anegar",
    "aneja": "anejo",
    "anejaba": "anejar",
    "anejabais": "anejar",
    "anejaban": "anejar",
    "anejabas": "anejar",
    "anejad": "anejar",
    "anejada": "anejar",
    "anejadas": "anejar",
    "anejado": "anejar",
    "anejados": "anejar",
    "anejamos": "anejar",
    "anejan": "anejar",
    "anejando": "anejar",
    "anejara": "anejar",
    "anejarais": "anejar",
    "anejaran": "anejar",
    "anejaras": "anejar",
    "anejare": "anejar",
    "anejareis": "anejar",
    "anejaremos": "anejar",
    "anejaren": "anejar",
    "anejares": "anejar",
    "anejaron": "anejar",
    "anejará": "anejar",
    "anejarán": "anejar",
    "anejarás": "anejar",
    "anejaré": "anejar",
    "anejaréis": "anejar",
    "anejaría": "anejar",
    "anejaríais": "anejar",
    "anejaríamos": "anejar",
    "anejarían": "anejar",
    "anejarías": "anejar",
    "anejas": "anejo",
    "anejase": "anejar",
    "anejaseis": "anejar",
    "anejasen": "anejar",
    "anejases": "anejar",
    "anejaste": "anejar",
    "anejasteis": "anejar",
    "aneje": "anejar",
    "anejemos": "anejar",
    "anejen": "anejar",
    "anejes": "anejar",
    "anejo": "anejar",
    "anejos": "anejo",
    "anejábamos": "anejar",
    "anejáis": "anejar",
    "anejáramos": "anejar",
    "anejáremos": "anejar",
    "anejásemos": "anejar",
    "anejé": "anejar",
    "anejéis": "anejar",
    "anejó": "anejar",
    "anelasticidades": "anelasticidad",
    "aneldos": "aneldo",
    "anemias": "anemia",
    "anemografías": "anemografía",
    "anemográfica": "anemográfico",
    "anemográficas": "anemográfico",
    "anemográficos": "anemográfico",
    "anemometrías": "anemometría",
    "anemométrica": "anemométrico",
    "anemométricas": "anemométrico",
    "anemométricos": "anemométrico",
    "anemones": "anemone",
    "anemoscopios": "anemoscopio",
    "anemócoras": "anemócora",
    "anemófila": "anemófilo",
    "anemófilas": "anemófilo",
    "anemófilos": "anemófilo",
    "anemógrafos": "anemógrafo",
    "anemómetros": "anemómetro",
    "anergias": "anergia",
    "anergías": "anergía",
    "aneroides": "aneroide",
    "anestesia": "anestesiar",
    "anestesiaba": "anestesiar",
    "anestesiabais": "anestesiar",
    "anestesiaban": "anestesiar",
    "anestesiabas": "anestesiar",
    "anestesiad": "anestesiar",
    "anestesiada": "anestesiar",
    "anestesiadas": "anestesiar",
    "anestesiado": "anestesiar",
    "anestesiados": "anestesiar",
    "anestesiamos": "anestesiar",
    "anestesian": "anestesiar",
    "anestesiando": "anestesiar",
    "anestesiara": "anestesiar",
    "anestesiarais": "anestesiar",
    "anestesiaran": "anestesiar",
    "anestesiaras": "anestesiar",
    "anestesiare": "anestesiar",
    "anestesiareis": "anestesiar",
    "anestesiaremos": "anestesiar",
    "anestesiaren": "anestesiar",
    "anestesiares": "anestesiar",
    "anestesiaron": "anestesiar",
    "anestesiará": "anestesiar",
    "anestesiarán": "anestesiar",
    "anestesiarás": "anestesiar",
    "anestesiaré": "anestesiar",
    "anestesiaréis": "anestesiar",
    "anestesiaría": "anestesiar",
    "anestesiaríais": "anestesiar",
    "anestesiaríamos": "anestesiar",
    "anestesiarían": "anestesiar",
    "anestesiarías": "anestesiar",
    "anestesias": "anestesiar",
    "anestesiase": "anestesiar",
    "anestesiaseis": "anestesiar",
    "anestesiasen": "anestesiar",
    "anestesiases": "anestesiar",
    "anestesiaste": "anestesiar",
    "anestesiasteis": "anestesiar",
    "anestesie": "anestesiar",
    "anestesiemos": "anestesiar",
    "anestesien": "anestesiar",
    "anestesies": "anestesiar",
    "anestesio": "anestesiar",
    "anestesiologías": "anestesiología",
    "anestesistas": "anestesista",
    "anestesiábamos": "anestesiar",
    "anestesiáis": "anestesiar",
    "anestesiáramos": "anestesiar",
    "anestesiáremos": "anestesiar",
    "anestesiásemos": "anestesiar",
    "anestesié": "anestesiar",
    "anestesiéis": "anestesiar",
    "anestesió": "anestesiar",
    "anestesióloga": "anestesiólogo",
    "anestesiólogas": "anestesiólogo",
    "anestesiólogos": "anestesiólogo",
    "anestésica": "anestésico",
    "anestésicas": "anestésico",
    "anestésicos": "anestésico",
    "anetoles": "anetol",
    "aneuploidias": "aneuploidia",
    "aneurismas": "aneurisma",
    "aneurismática": "aneurismático",
    "aneurismáticas": "aneurismático",
    "aneurismáticos": "aneurismático",
    "anexa": "anexo",
    "anexaba": "anexar",
    "anexabais": "anexar",
    "anexaban": "anexar",
    "anexabas": "anexar",
    "anexad": "anexar",
    "anexada": "anexar",
    "anexadas": "anexar",
    "anexado": "anexar",
    "anexados": "anexar",
    "anexamos": "anexar",
    "anexan": "anexar",
    "anexando": "anexar",
    "anexara": "anexar",
    "anexarais": "anexar",
    "anexaran": "anexar",
    "anexaras": "anexar",
    "anexare": "anexar",
    "anexareis": "anexar",
    "anexaremos": "anexar",
    "anexaren": "anexar",
    "anexares": "anexar",
    "anexaron": "anexar",
    "anexará": "anexar",
    "anexarán": "anexar",
    "anexarás": "anexar",
    "anexaré": "anexar",
    "anexaréis": "anexar",
    "anexaría": "anexar",
    "anexaríais": "anexar",
    "anexaríamos": "anexar",
    "anexarían": "anexar",
    "anexarías": "anexar",
    "anexas": "anexo",
    "anexase": "anexar",
    "anexaseis": "anexar",
    "anexasen": "anexar",
    "anexases": "anexar",
    "anexaste": "anexar",
    "anexasteis": "anexar",
    "anexe": "anexar",
    "anexemos": "anexar",
    "anexen": "anexar",
    "anexes": "anexar",
    "anexiona": "anexionar",
    "anexionaba": "anexionar",
    "anexionabais": "anexionar",
    "anexionaban": "anexionar",
    "anexionabas": "anexionar",
    "anexionad": "anexionar",
    "anexionada": "anexionar",
    "anexionadas": "anexionar",
    "anexionado": "anexionar",
    "anexionados": "anexionar",
    "anexionamos": "anexionar",
    "anexionan": "anexionar",
    "anexionando": "anexionar",
    "anexionara": "anexionar",
    "anexionarais": "anexionar",
    "anexionaran": "anexionar",
    "anexionaras": "anexionar",
    "anexionare": "anexionar",
    "anexionareis": "anexionar",
    "anexionaremos": "anexionar",
    "anexionaren": "anexionar",
    "anexionares": "anexionar",
    "anexionaron": "anexionar",
    "anexionará": "anexionar",
    "anexionarán": "anexionar",
    "anexionarás": "anexionar",
    "anexionaré": "anexionar",
    "anexionaréis": "anexionar",
    "anexionaría": "anexionar",
    "anexionaríais": "anexionar",
    "anexionaríamos": "anexionar",
    "anexionarían": "anexionar",
    "anexionarías": "anexionar",
    "anexionas": "anexionar",
    "anexionase": "anexionar",
    "anexionaseis": "anexionar",
    "anexionasen": "anexionar",
    "anexionases": "anexionar",
    "anexionaste": "anexionar",
    "anexionasteis": "anexionar",
    "anexione": "anexionar",
    "anexionemos": "anexionar",
    "anexionen": "anexionar",
    "anexiones": "anexionar",
    "anexionismos": "anexionismo",
    "anexionistas": "anexionista",
    "anexiono": "anexionar",
    "anexionábamos": "anexionar",
    "anexionáis": "anexionar",
    "anexionáramos": "anexionar",
    "anexionáremos": "anexionar",
    "anexionásemos": "anexionar",
    "anexioné": "anexionar",
    "anexionéis": "anexionar",
    "anexionó": "anexionar",
    "anexo": "anexar",
    "anexos": "anexo",
    "anexábamos": "anexar",
    "anexáis": "anexar",
    "anexáramos": "anexar",
    "anexáremos": "anexar",
    "anexásemos": "anexar",
    "anexé": "anexar",
    "anexéis": "anexar",
    "anexó": "anexar",
    "anfetaminas": "anfetamina",
    "anfiasteres": "anfiaster",
    "anfibia": "anfibio",
    "anfibias": "anfibio",
    "anfibios": "anfibio",
    "anfibolitas": "anfibolita",
    "anfibologías": "anfibología",
    "anfibológica": "anfibológico",
    "anfibológicas": "anfibológico",
    "anfibológicos": "anfibológico",
    "anfictionados": "anfictionado",
    "anfictiones": "anfictión",
    "anfictionías": "anfictionía",
    "anfictiónica": "anfictiónico",
    "anfictiónicas": "anfictiónico",
    "anfictiónicos": "anfictiónico",
    "anfidiploidias": "anfidiploidia",
    "anfidrómica": "anfidrómico",
    "anfidrómicas": "anfidrómico",
    "anfidrómicos": "anfidrómico",
    "anfigonias": "anfigonia",
    "anfipróstila": "anfipróstilo",
    "anfipróstilas": "anfipróstilo",
    "anfipróstilos": "anfipróstilo",
    "anfipática": "anfipático",
    "anfipáticas": "anfipático",
    "anfipáticos": "anfipático",
    "anfiscia": "anfiscio",
    "anfiscias": "anfiscio",
    "anfiscios": "anfiscio",
    "anfiteatros": "anfiteatro",
    "anfitriona": "anfitrión",
    "anfitrionas": "anfitrión",
    "anfitriones": "anfitrión",
    "anfogenias": "anfogenia",
    "anfolita": "anfolito",
    "anfolitas": "anfolito",
    "anfolitos": "anfolito",
    "anforiscos": "anforisco",
    "anfotérica": "anfotérico",
    "anfotéricas": "anfotérico",
    "anfotéricos": "anfotérico",
    "anfractuosa": "anfractuoso",
    "anfractuosas": "anfractuoso",
    "anfractuosidades": "anfractuosidad",
    "anfractuosos": "anfractuoso",
    "anfíboles": "anfíbol",
    "anfíbracos": "anfíbraco",
    "anfólitos": "anfólito",
    "anfótera": "anfótero",
    "anfóteras": "anfótero",
    "anfóteros": "anfótero",
    "angarias": "angaria",
    "angarías": "angaría",
    "angazos": "angazo",
    "angelicales": "angelical",
    "angelitos": "angelito",
    "angelolatrías": "angelolatría",
    "angelologías": "angelología",
    "angelotes": "angelote",
    "angevina": "angevino",
    "angevinas": "angevino",
    "angevinos": "angevino",
    "anginas": "angina",
    "anginosa": "anginoso",
    "anginosas": "anginoso",
    "anginosos": "anginoso",
    "angioblastos": "angioblasto",
    "angioedemas": "angioedema",
    "angiofibromas": "angiofibroma",
    "angiografías": "angiografía",
    "angiográfica": "angiográfico",
    "angiográficas": "angiográfico",
    "angiográficos": "angiográfico",
    "angiologías": "angiología",
    "angiológica": "angiológico",
    "angiológicas": "angiológico",
    "angiológicos": "angiológico",
    "angiomas": "angioma",
    "angioneurótica": "angioneurótico",
    "angioneuróticas": "angioneurótico",
    "angioneuróticos": "angioneurótico",
    "angioscotomas": "angioscotoma",
    "angioscotometrías": "angioscotometría",
    "angiospermas": "angiosperma",
    "angiotensinas": "angiotensina",
    "angióloga": "angiólogo",
    "angiólogas": "angiólogo",
    "angiólogos": "angiólogo",
    "angla": "anglo",
    "anglas": "anglo",
    "anglesitas": "anglesita",
    "anglicana": "anglicano",
    "anglicanas": "anglicano",
    "anglicanismos": "anglicanismo",
    "anglicanos": "anglicano",
    "anglicismos": "anglicismo",
    "anglicistas": "anglicista",
    "angloamericana": "angloamericano",
    "angloamericanas": "angloamericano",
    "angloamericanos": "angloamericano",
    "anglofilias": "anglofilia",
    "anglofobias": "anglofobia",
    "anglonormanda": "anglonormando",
    "anglonormandas": "anglonormando",
    "anglonormandos": "anglonormando",
    "angloparlantes": "angloparlante",
    "anglos": "anglo",
    "anglosajona": "anglosajón",
    "anglosajonas": "anglosajón",
    "anglosajones": "anglosajón",
    "anglófila": "anglófilo",
    "anglófilas": "anglófilo",
    "anglófilos": "anglófilo",
    "anglófoba": "anglófobo",
    "anglófobas": "anglófobo",
    "anglófobos": "anglófobo",
    "anglófona": "anglófono",
    "anglófonas": "anglófono",
    "anglófonos": "anglófono",
    "angolenses": "angolense",
    "angolesa": "angolés",
    "angolesas": "angolés",
    "angoleses": "angolés",
    "angoleña": "angoleño",
    "angoleñas": "angoleño",
    "angoleños": "angoleño",
    "angoras": "angora",
    "angorras": "angorra",
    "angosta": "angosto",
    "angostaba": "angostar",
    "angostabais": "angostar",
    "angostaban": "angostar",
    "angostabas": "angostar",
    "angostad": "angostar",
    "angostada": "angostar",
    "angostadas": "angostar",
    "angostado": "angostar",
    "angostados": "angostar",
    "angostamientos": "angostamiento",
    "angostamos": "angostar",
    "angostan": "angostar",
    "angostando": "angostar",
    "angostara": "angostar",
    "angostarais": "angostar",
    "angostaran": "angostar",
    "angostaras": "angostar",
    "angostare": "angostar",
    "angostareis": "angostar",
    "angostaremos": "angostar",
    "angostaren": "angostar",
    "angostares": "angostar",
    "angostaron": "angostar",
    "angostará": "angostar",
    "angostarán": "angostar",
    "angostarás": "angostar",
    "angostaré": "angostar",
    "angostaréis": "angostar",
    "angostaría": "angostar",
    "angostaríais": "angostar",
    "angostaríamos": "angostar",
    "angostarían": "angostar",
    "angostarías": "angostar",
    "angostas": "angosto",
    "angostase": "angostar",
    "angostaseis": "angostar",
    "angostasen": "angostar",
    "angostases": "angostar",
    "angostaste": "angostar",
    "angostasteis": "angostar",
    "angoste": "angostar",
    "angostemos": "angostar",
    "angosten": "angostar",
    "angostes": "angostar",
    "angosto": "angostar",
    "angostos": "angosto",
    "angosturas": "angostura",
    "angostábamos": "angostar",
    "angostáis": "angostar",
    "angostáramos": "angostar",
    "angostáremos": "angostar",
    "angostásemos": "angostar",
    "angosté": "angostar",
    "angostéis": "angostar",
    "angostó": "angostar",
    "angras": "angra",
    "angrelada": "angrelado",
    "angreladas": "angrelado",
    "angrelados": "angrelado",
    "angstromes": "angstrom",
    "angstromios": "angstromio",
    "anguilas": "anguila",
    "anguiliculturas": "anguilicultura",
    "anguitistas": "anguitista",
    "angulada": "angulado",
    "anguladas": "angulado",
    "angulados": "angulado",
    "angulares": "angular",
    "angulas": "angula",
    "angulemas": "angulema",
    "angulosa": "anguloso",
    "angulosas": "anguloso",
    "angulosos": "anguloso",
    "angustia": "angustiar",
    "angustiaba": "angustiar",
    "angustiabais": "angustiar",
    "angustiaban": "angustiar",
    "angustiabas": "angustiar",
    "angustiad": "angustiar",
    "angustiada": "angustiar",
    "angustiadas": "angustiar",
    "angustiado": "angustiar",
    "angustiados": "angustiar",
    "angustiamos": "angustiar",
    "angustian": "angustiar",
    "angustiando": "angustiar",
    "angustiara": "angustiar",
    "angustiarais": "angustiar",
    "angustiaran": "angustiar",
    "angustiaras": "angustiar",
    "angustiare": "angustiar",
    "angustiareis": "angustiar",
    "angustiaremos": "angustiar",
    "angustiaren": "angustiar",
    "angustiares": "angustiar",
    "angustiaron": "angustiar",
    "angustiará": "angustiar",
    "angustiarán": "angustiar",
    "angustiarás": "angustiar",
    "angustiaré": "angustiar",
    "angustiaréis": "angustiar",
    "angustiaría": "angustiar",
    "angustiaríais": "angustiar",
    "angustiaríamos": "angustiar",
    "angustiarían": "angustiar",
    "angustiarías": "angustiar",
    "angustias": "angustiar",
    "angustiase": "angustiar",
    "angustiaseis": "angustiar",
    "angustiasen": "angustiar",
    "angustiases": "angustiar",
    "angustiaste": "angustiar",
    "angustiasteis": "angustiar",
    "angustie": "angustiar",
    "angustiemos": "angustiar",
    "angustien": "angustiar",
    "angusties": "angustiar",
    "angustio": "angustiar",
    "angustiosa": "angustioso",
    "angustiosas": "angustioso",
    "angustiosos": "angustioso",
    "angustiábamos": "angustiar",
    "angustiáis": "angustiar",
    "angustiáramos": "angustiar",
    "angustiáremos": "angustiar",
    "angustiásemos": "angustiar",
    "angustié": "angustiar",
    "angustiéis": "angustiar",
    "angustió": "angustiar",
    "angélica": "angélico",
    "angélicas": "angélico",
    "angélicos": "angélico",
    "anhedrales": "anhedral",
    "anhela": "anhelar",
    "anhelaba": "anhelar",
    "anhelabais": "anhelar",
    "anhelaban": "anhelar",
    "anhelabas": "anhelar",
    "anhelaciones": "anhelación",
    "anhelad": "anhelar",
    "anhelada": "anhelar",
    "anheladas": "anhelar",
    "anhelado": "anhelar",
    "anhelados": "anhelar",
    "anhelamos": "anhelar",
    "anhelan": "anhelar",
    "anhelando": "anhelar",
    "anhelantes": "anhelante",
    "anhelara": "anhelar",
    "anhelarais": "anhelar",
    "anhelaran": "anhelar",
    "anhelaras": "anhelar",
    "anhelare": "anhelar",
    "anhelareis": "anhelar",
    "anhelaremos": "anhelar",
    "anhelaren": "anhelar",
    "anhelares": "anhelar",
    "anhelaron": "anhelar",
    "anhelará": "anhelar",
    "anhelarán": "anhelar",
    "anhelarás": "anhelar",
    "anhelaré": "anhelar",
    "anhelaréis": "anhelar",
    "anhelaría": "anhelar",
    "anhelaríais": "anhelar",
    "anhelaríamos": "anhelar",
    "anhelarían": "anhelar",
    "anhelarías": "anhelar",
    "anhelas": "anhelar",
    "anhelase": "anhelar",
    "anhelaseis": "anhelar",
    "anhelasen": "anhelar",
    "anhelases": "anhelar",
    "anhelaste": "anhelar",
    "anhelasteis": "anhelar",
    "anhele": "anhelar",
    "anhelemos": "anhelar",
    "anhelen": "anhelar",
    "anheles": "anhelar",
    "anhelo": "anhelar",
    "anhelos": "anhelo",
    "anhelábamos": "anhelar",
    "anheláis": "anhelar",
    "anheláramos": "anhelar",
    "anheláremos": "anhelar",
    "anhelásemos": "anhelar",
    "anhelé": "anhelar",
    "anheléis": "anhelar",
    "anheló": "anhelar",
    "anhidra": "anhidro",
    "anhidras": "anhidro",
    "anhidritas": "anhidrita",
    "anhidros": "anhidro",
    "anhisterética": "anhisterético",
    "anhisteréticas": "anhisterético",
    "anhisteréticos": "anhisterético",
    "anhídrica": "anhídrico",
    "anhídricas": "anhídrico",
    "anhídricos": "anhídrico",
    "anhídridos": "anhídrido",
    "anida": "anidar",
    "anidaba": "anidar",
    "anidabais": "anidar",
    "anidaban": "anidar",
    "anidabas": "anidar",
    "anidables": "anidable",
    "anidaciones": "anidación",
    "anidad": "anidar",
    "anidada": "anidar",
    "anidadas": "anidar",
    "anidado": "anidar",
    "anidados": "anidar",
    "anidamientos": "anidamiento",
    "anidamos": "anidar",
    "anidan": "anidar",
    "anidando": "anidar",
    "anidara": "anidar",
    "anidarais": "anidar",
    "anidaran": "anidar",
    "anidaras": "anidar",
    "anidare": "anidar",
    "anidareis": "anidar",
    "anidaremos": "anidar",
    "anidaren": "anidar",
    "anidares": "anidar",
    "anidaron": "anidar",
    "anidará": "anidar",
    "anidarán": "anidar",
    "anidarás": "anidar",
    "anidaré": "anidar",
    "anidaréis": "anidar",
    "anidaría": "anidar",
    "anidaríais": "anidar",
    "anidaríamos": "anidar",
    "anidarían": "anidar",
    "anidarías": "anidar",
    "anidas": "anidar",
    "anidase": "anidar",
    "anidaseis": "anidar",
    "anidasen": "anidar",
    "anidases": "anidar",
    "anidaste": "anidar",
    "anidasteis": "anidar",
    "anide": "anidar",
    "anidemos": "anidar",
    "aniden": "anidar",
    "anides": "anidar",
    "anido": "anidar",
    "anidábamos": "anidar",
    "anidáis": "anidar",
    "anidáramos": "anidar",
    "anidáremos": "anidar",
    "anidásemos": "anidar",
    "anidé": "anidar",
    "anidéis": "anidar",
    "anidó": "anidar",
    "aniebla": "anieblar",
    "anieblaba": "anieblar",
    "anieblabais": "anieblar",
    "anieblaban": "anieblar",
    "anieblabas": "anieblar",
    "anieblad": "anieblar",
    "anieblada": "anieblar",
    "aniebladas": "anieblar",
    "anieblado": "anieblar",
    "anieblados": "anieblar",
    "anieblamos": "anieblar",
    "anieblan": "anieblar",
    "anieblando": "anieblar",
    "anieblara": "anieblar",
    "anieblarais": "anieblar",
    "anieblaran": "anieblar",
    "anieblaras": "anieblar",
    "anieblare": "anieblar",
    "anieblareis": "anieblar",
    "anieblaremos": "anieblar",
    "anieblaren": "anieblar",
    "anieblares": "anieblar",
    "anieblaron": "anieblar",
    "anieblará": "anieblar",
    "anieblarán": "anieblar",
    "anieblarás": "anieblar",
    "anieblaré": "anieblar",
    "anieblaréis": "anieblar",
    "anieblaría": "anieblar",
    "anieblaríais": "anieblar",
    "anieblaríamos": "anieblar",
    "anieblarían": "anieblar",
    "anieblarías": "anieblar",
    "anieblas": "anieblar",
    "anieblase": "anieblar",
    "anieblaseis": "anieblar",
    "anieblasen": "anieblar",
    "anieblases": "anieblar",
    "anieblaste": "anieblar",
    "anieblasteis": "anieblar",
    "anieble": "anieblar",
    "anieblemos": "anieblar",
    "anieblen": "anieblar",
    "aniebles": "anieblar",
    "anieblo": "anieblar",
    "anieblábamos": "anieblar",
    "aniebláis": "anieblar",
    "aniebláramos": "anieblar",
    "aniebláremos": "anieblar",
    "anieblásemos": "anieblar",
    "anieblé": "anieblar",
    "aniebléis": "anieblar",
    "aniebló": "anieblar",
    "anihila": "anihilar",
    "anihilaba": "anihilar",
    "anihilabais": "anihilar",
    "anihilaban": "anihilar",
    "anihilabas": "anihilar",
    "anihilad": "anihilar",
    "anihilada": "anihilar",
    "anihiladas": "anihilar",
    "anihilado": "anihilar",
    "anihilados": "anihilar",
    "anihilamos": "anihilar",
    "anihilan": "anihilar",
    "anihilando": "anihilar",
    "anihilara": "anihilar",
    "anihilarais": "anihilar",
    "anihilaran": "anihilar",
    "anihilaras": "anihilar",
    "anihilare": "anihilar",
    "anihilareis": "anihilar",
    "anihilaremos": "anihilar",
    "anihilaren": "anihilar",
    "anihilares": "anihilar",
    "anihilaron": "anihilar",
    "anihilará": "anihilar",
    "anihilarán": "anihilar",
    "anihilarás": "anihilar",
    "anihilaré": "anihilar",
    "anihilaréis": "anihilar",
    "anihilaría": "anihilar",
    "anihilaríais": "anihilar",
    "anihilaríamos": "anihilar",
    "anihilarían": "anihilar",
    "anihilarías": "anihilar",
    "anihilas": "anihilar",
    "anihilase": "anihilar",
    "anihilaseis": "anihilar",
    "anihilasen": "anihilar",
    "anihilases": "anihilar",
    "anihilaste": "anihilar",
    "anihilasteis": "anihilar",
    "anihile": "anihilar",
    "anihilemos": "anihilar",
    "anihilen": "anihilar",
    "anihiles": "anihilar",
    "anihilo": "anihilar",
    "anihilábamos": "anihilar",
    "anihiláis": "anihilar",
    "anihiláramos": "anihilar",
    "anihiláremos": "anihilar",
    "anihilásemos": "anihilar",
    "anihilé": "anihilar",
    "anihiléis": "anihilar",
    "anihiló": "anihilar",
    "anilidas": "anilida",
    "anilinas": "anilina",
    "anilla": "anillar",
    "anillaba": "anillar",
    "anillabais": "anillar",
    "anillaban": "anillar",
    "anillabas": "anillar",
    "anillad": "anillar",
    "anillada": "anillar",
    "anilladas": "anillar",
    "anillado": "anillar",
    "anillados": "anillar",
    "anillamos": "anillar",
    "anillan": "anillar",
    "anillando": "anillar",
    "anillara": "anillar",
    "anillarais": "anillar",
    "anillaran": "anillar",
    "anillaras": "anillar",
    "anillare": "anillar",
    "anillareis": "anillar",
    "anillaremos": "anillar",
    "anillaren": "anillar",
    "anillares": "anillar",
    "anillaron": "anillar",
    "anillará": "anillar",
    "anillarán": "anillar",
    "anillarás": "anillar",
    "anillaré": "anillar",
    "anillaréis": "anillar",
    "anillaría": "anillar",
    "anillaríais": "anillar",
    "anillaríamos": "anillar",
    "anillarían": "anillar",
    "anillarías": "anillar",
    "anillas": "anillar",
    "anillase": "anillar",
    "anillaseis": "anillar",
    "anillasen": "anillar",
    "anillases": "anillar",
    "anillaste": "anillar",
    "anillasteis": "anillar",
    "anille": "anillar",
    "anillemos": "anillar",
    "anillen": "anillar",
    "anilles": "anillar",
    "anillo": "anillar",
    "anillos": "anillo",
    "anillábamos": "anillar",
    "anilláis": "anillar",
    "anilláramos": "anillar",
    "anilláremos": "anillar",
    "anillásemos": "anillar",
    "anillé": "anillar",
    "anilléis": "anillar",
    "anilló": "anillar",
    "anima": "animar",
    "animaba": "animar",
    "animabais": "animar",
    "animaban": "animar",
    "animabas": "animar",
    "animaciones": "animación",
    "animad": "animar",
    "animada": "animar",
    "animadas": "animar",
    "animado": "animar",
    "animadora": "animador",
    "animadoras": "animador",
    "animadores": "animador",
    "animados": "animar",
    "animadversiones": "animadversión",
    "animaladas": "animalada",
    "animales": "animal",
    "animalice": "animalizar",
    "animalicemos": "animalizar",
    "animalicen": "animalizar",
    "animalices": "animalizar",
    "animalicé": "animalizar",
    "animalicéis": "animalizar",
    "animalidades": "animalidad",
    "animaliza": "animalizar",
    "animalizaba": "animalizar",
    "animalizabais": "animalizar",
    "animalizaban": "animalizar",
    "animalizabas": "animalizar",
    "animalizaciones": "animalización",
    "animalizad": "animalizar",
    "animalizada": "animalizar",
    "animalizadas": "animalizar",
    "animalizado": "animalizar",
    "animalizados": "animalizar",
    "animalizamos": "animalizar",
    "animalizan": "animalizar",
    "animalizando": "animalizar",
    "animalizara": "animalizar",
    "animalizarais": "animalizar",
    "animalizaran": "animalizar",
    "animalizaras": "animalizar",
    "animalizare": "animalizar",
    "animalizareis": "animalizar",
    "animalizaremos": "animalizar",
    "animalizaren": "animalizar",
    "animalizares": "animalizar",
    "animalizaron": "animalizar",
    "animalizará": "animalizar",
    "animalizarán": "animalizar",
    "animalizarás": "animalizar",
    "animalizaré": "animalizar",
    "animalizaréis": "animalizar",
    "animalizaría": "animalizar",
    "animalizaríais": "animalizar",
    "animalizaríamos": "animalizar",
    "animalizarían": "animalizar",
    "animalizarías": "animalizar",
    "animalizas": "animalizar",
    "animalizase": "animalizar",
    "animalizaseis": "animalizar",
    "animalizasen": "animalizar",
    "animalizases": "animalizar",
    "animalizaste": "animalizar",
    "animalizasteis": "animalizar",
    "animalizo": "animalizar",
    "animalizábamos": "animalizar",
    "animalizáis": "animalizar",
    "animalizáramos": "animalizar",
    "animalizáremos": "animalizar",
    "animalizásemos": "animalizar",
    "animalizó": "animalizar",
    "animamos": "animar",
    "animan": "animar",
    "animando": "animar",
    "animara": "animar",
    "animarais": "animar",
    "animaran": "animar",
    "animaras": "animar",
    "animare": "animar",
    "animareis": "animar",
    "animaremos": "animar",
    "animaren": "animar",
    "animares": "animar",
    "animaron": "animar",
    "animará": "animar",
    "animarán": "animar",
    "animarás": "animar",
    "animaré": "animar",
    "animaréis": "animar",
    "animaría": "animar",
    "animaríais": "animar",
    "animaríamos": "animar",
    "animarían": "animar",
    "animarías": "animar",
    "animas": "animar",
    "animase": "animar",
    "animaseis": "animar",
    "animasen": "animar",
    "animases": "animar",
    "animaste": "animar",
    "animasteis": "animar",
    "animatismos": "animatismo",
    "anime": "animar",
    "animemos": "animar",
    "animen": "animar",
    "animes": "animar",
    "animismos": "animismo",
    "animo": "animar",
    "animosa": "animoso",
    "animosas": "animoso",
    "animosidades": "animosidad",
    "animosos": "animoso",
    "animábamos": "animar",
    "animáis": "animar",
    "animáramos": "animar",
    "animáremos": "animar",
    "animásemos": "animar",
    "animé": "animar",
    "animéis": "animar",
    "animó": "animar",
    "aniones": "anión",
    "aniquila": "aniquilar",
    "aniquilaba": "aniquilar",
    "aniquilabais": "aniquilar",
    "aniquilaban": "aniquilar",
    "aniquilabas": "aniquilar",
    "aniquilaciones": "aniquilación",
    "aniquilad": "aniquilar",
    "aniquilada": "aniquilar",
    "aniquiladas": "aniquilar",
    "aniquilado": "aniquilar",
    "aniquiladora": "aniquilador",
    "aniquiladoras": "aniquilador",
    "aniquiladores": "aniquilador",
    "aniquilados": "aniquilar",
    "aniquilamientos": "aniquilamiento",
    "aniquilamos": "aniquilar",
    "aniquilan": "aniquilar",
    "aniquilando": "aniquilar",
    "aniquilara": "aniquilar",
    "aniquilarais": "aniquilar",
    "aniquilaran": "aniquilar",
    "aniquilaras": "aniquilar",
    "aniquilare": "aniquilar",
    "aniquilareis": "aniquilar",
    "aniquilaremos": "aniquilar",
    "aniquilaren": "aniquilar",
    "aniquilares": "aniquilar",
    "aniquilaron": "aniquilar",
    "aniquilará": "aniquilar",
    "aniquilarán": "aniquilar",
    "aniquilarás": "aniquilar",
    "aniquilaré": "aniquilar",
    "aniquilaréis": "aniquilar",
    "aniquilaría": "aniquilar",
    "aniquilaríais": "aniquilar",
    "aniquilaríamos": "aniquilar",
    "aniquilarían": "aniquilar",
    "aniquilarías": "aniquilar",
    "aniquilas": "aniquilar",
    "aniquilase": "aniquilar",
    "aniquilaseis": "aniquilar",
    "aniquilasen": "aniquilar",
    "aniquilases": "aniquilar",
    "aniquilaste": "aniquilar",
    "aniquilasteis": "aniquilar",
    "aniquile": "aniquilar",
    "aniquilemos": "aniquilar",
    "aniquilen": "aniquilar",
    "aniquiles": "aniquilar",
    "aniquilo": "aniquilar",
    "aniquilábamos": "aniquilar",
    "aniquiláis": "aniquilar",
    "aniquiláramos": "aniquilar",
    "aniquiláremos": "aniquilar",
    "aniquilásemos": "aniquilar",
    "aniquilé": "aniquilar",
    "aniquiléis": "aniquilar",
    "aniquiló": "aniquilar",
    "aniridias": "aniridia",
    "anisa": "anisar",
    "anisaba": "anisar",
    "anisabais": "anisar",
    "anisaban": "anisar",
    "anisabas": "anisar",
    "anisad": "anisar",
    "anisada": "anisar",
    "anisadas": "anisar",
    "anisado": "anisar",
    "anisados": "anisar",
    "anisamos": "anisar",
    "anisan": "anisar",
    "anisando": "anisar",
    "anisara": "anisar",
    "anisarais": "anisar",
    "anisaran": "anisar",
    "anisaras": "anisar",
    "anisare": "anisar",
    "anisareis": "anisar",
    "anisaremos": "anisar",
    "anisaren": "anisar",
    "anisares": "anisar",
    "anisaron": "anisar",
    "anisará": "anisar",
    "anisarán": "anisar",
    "anisarás": "anisar",
    "anisaré": "anisar",
    "anisaréis": "anisar",
    "anisaría": "anisar",
    "anisaríais": "anisar",
    "anisaríamos": "anisar",
    "anisarían": "anisar",
    "anisarías": "anisar",
    "anisas": "anisar",
    "anisase": "anisar",
    "anisaseis": "anisar",
    "anisasen": "anisar",
    "anisases": "anisar",
    "anisaste": "anisar",
    "anisasteis": "anisar",
    "anise": "anisar",
    "aniseiconías": "aniseiconía",
    "aniseicónica": "aniseicónico",
    "aniseicónicas": "aniseicónico",
    "aniseicónicos": "aniseicónico",
    "anisemos": "anisar",
    "anisen": "anisar",
    "anises": "anisar",
    "anisetes": "anisete",
    "anisidinas": "anisidina",
    "aniso": "anisar",
    "anisoacomodaciones": "anisoacomodación",
    "anisocorias": "anisocoria",
    "anisognata": "anisognato",
    "anisognatas": "anisognato",
    "anisognatos": "anisognato",
    "anisoiconías": "anisoiconía",
    "anisoles": "anisol",
    "anisometropías": "anisometropía",
    "anisomérica": "anisomérico",
    "anisoméricas": "anisomérico",
    "anisoméricos": "anisomérico",
    "anisométrica": "anisométrico",
    "anisométricas": "anisométrico",
    "anisométricos": "anisométrico",
    "anisopétala": "anisopétalo",
    "anisopétalas": "anisopétalo",
    "anisopétalos": "anisopétalo",
    "anisotropias": "anisotropia",
    "anisotropías": "anisotropía",
    "anisotrópica": "anisotrópico",
    "anisotrópicas": "anisotrópico",
    "anisotrópicos": "anisotrópico",
    "anisotónica": "anisotónico",
    "anisotónicas": "anisotónico",
    "anisotónicos": "anisotónico",
    "anisábamos": "anisar",
    "anisáis": "anisar",
    "anisáramos": "anisar",
    "anisáremos": "anisar",
    "anisásemos": "anisar",
    "anisé": "anisar",
    "aniséis": "anisar",
    "anisó": "anisar",
    "anisócrona": "anisócrono",
    "anisócronas": "anisócrono",
    "anisócronos": "anisócrono",
    "anisómera": "anisómero",
    "anisómeras": "anisómero",
    "anisómeros": "anisómero",
    "anisótropa": "anisótropo",
    "anisótropas": "anisótropo",
    "anisótropos": "anisótropo",
    "aniversaria": "aniversario",
    "aniversarias": "aniversario",
    "aniversarios": "aniversario",
    "aniñada": "aniñado",
    "aniñadas": "aniñado",
    "aniñados": "aniñado",
    "aniónica": "aniónico",
    "aniónicas": "aniónico",
    "aniónicos": "aniónico",
    "anobidos": "anobido",
    "anochece": "anochecer",
    "anocheced": "anochecer",
    "anochecemos": "anochecer",
    "anochecen": "anochecer",
    "anocheceremos": "anochecer",
    "anocheceres": "anochecer",
    "anochecerá": "anochecer",
    "anochecerán": "anochecer",
    "anochecerás": "anochecer",
    "anocheceré": "anochecer",
    "anocheceréis": "anochecer",
    "anochecería": "anochecer",
    "anocheceríais": "anochecer",
    "anocheceríamos": "anochecer",
    "anochecerían": "anochecer",
    "anochecerías": "anochecer",
    "anocheces": "anochecer",
    "anochecida": "anochecer",
    "anochecidas": "anochecida",
    "anochecido": "anochecer",
    "anochecidos": "anochecer",
    "anocheciendo": "anochecer",
    "anocheciera": "anochecer",
    "anochecierais": "anochecer",
    "anochecieran": "anochecer",
    "anochecieras": "anochecer",
    "anocheciere": "anochecer",
    "anocheciereis": "anochecer",
    "anochecieren": "anochecer",
    "anochecieres": "anochecer",
    "anochecieron": "anochecer",
    "anocheciese": "anochecer",
    "anochecieseis": "anochecer",
    "anocheciesen": "anochecer",
    "anochecieses": "anochecer",
    "anochecimos": "anochecer",
    "anocheciste": "anochecer",
    "anochecisteis": "anochecer",
    "anocheciéramos": "anochecer",
    "anocheciéremos": "anochecer",
    "anocheciésemos": "anochecer",
    "anocheció": "anochecer",
    "anochecéis": "anochecer",
    "anochecí": "anochecer",
    "anochecía": "anochecer",
    "anochecíais": "anochecer",
    "anochecíamos": "anochecer",
    "anochecían": "anochecer",
    "anochecías": "anochecer",
    "anochezca": "anochecer",
    "anochezcamos": "anochecer",
    "anochezcan": "anochecer",
    "anochezcas": "anochecer",
    "anochezco": "anochecer",
    "anochezcáis": "anochecer",
    "anococcígea": "anococcígeo",
    "anococcígeas": "anococcígeo",
    "anococcígeos": "anococcígeo",
    "anocutánea": "anocutáneo",
    "anocutáneas": "anocutáneo",
    "anocutáneos": "anocutáneo",
    "anodice": "anodizar",
    "anodicemos": "anodizar",
    "anodicen": "anodizar",
    "anodices": "anodizar",
    "anodicé": "anodizar",
    "anodicéis": "anodizar",
    "anodina": "anodino",
    "anodinaba": "anodinar",
    "anodinabais": "anodinar",
    "anodinaban": "anodinar",
    "anodinabas": "anodinar",
    "anodinad": "anodinar",
    "anodinada": "anodinar",
    "anodinadas": "anodinar",
    "anodinado": "anodinar",
    "anodinados": "anodinar",
    "anodinamos": "anodinar",
    "anodinan": "anodinar",
    "anodinando": "anodinar",
    "anodinara": "anodinar",
    "anodinarais": "anodinar",
    "anodinaran": "anodinar",
    "anodinaras": "anodinar",
    "anodinare": "anodinar",
    "anodinareis": "anodinar",
    "anodinaremos": "anodinar",
    "anodinaren": "anodinar",
    "anodinares": "anodinar",
    "anodinaron": "anodinar",
    "anodinará": "anodinar",
    "anodinarán": "anodinar",
    "anodinarás": "anodinar",
    "anodinaré": "anodinar",
    "anodinaréis": "anodinar",
    "anodinaría": "anodinar",
    "anodinaríais": "anodinar",
    "anodinaríamos": "anodinar",
    "anodinarían": "anodinar",
    "anodinarías": "anodinar",
    "anodinas": "anodino",
    "anodinase": "anodinar",
    "anodinaseis": "anodinar",
    "anodinasen": "anodinar",
    "anodinases": "anodinar",
    "anodinaste": "anodinar",
    "anodinasteis": "anodinar",
    "anodine": "anodinar",
    "anodinemos": "anodinar",
    "anodinen": "anodinar",
    "anodines": "anodinar",
    "anodino": "anodinar",
    "anodinos": "anodino",
    "anodinábamos": "anodinar",
    "anodináis": "anodinar",
    "anodináramos": "anodinar",
    "anodináremos": "anodinar",
    "anodinásemos": "anodinar",
    "anodiné": "anodinar",
    "anodinéis": "anodinar",
    "anodinó": "anodinar",
    "anodiza": "anodizar",
    "anodizaba": "anodizar",
    "anodizabais": "anodizar",
    "anodizaban": "anodizar",
    "anodizabas": "anodizar",
    "anodizaciones": "anodización",
    "anodizad": "anodizar",
    "anodizada": "anodizar",
    "anodizadas": "anodizar",
    "anodizado": "anodizar",
    "anodizados": "anodizar",
    "anodizamos": "anodizar",
    "anodizan": "anodizar",
    "anodizando": "anodizar",
    "anodizara": "anodizar",
    "anodizarais": "anodizar",
    "anodizaran": "anodizar",
    "anodizaras": "anodizar",
    "anodizare": "anodizar",
    "anodizareis": "anodizar",
    "anodizaremos": "anodizar",
    "anodizaren": "anodizar",
    "anodizares": "anodizar",
    "anodizaron": "anodizar",
    "anodizará": "anodizar",
    "anodizarán": "anodizar",
    "anodizarás": "anodizar",
    "anodizaré": "anodizar",
    "anodizaréis": "anodizar",
    "anodizaría": "anodizar",
    "anodizaríais": "anodizar",
    "anodizaríamos": "anodizar",
    "anodizarían": "anodizar",
    "anodizarías": "anodizar",
    "anodizas": "anodizar",
    "anodizase": "anodizar",
    "anodizaseis": "anodizar",
    "anodizasen": "anodizar",
    "anodizases": "anodizar",
    "anodizaste": "anodizar",
    "anodizasteis": "anodizar",
    "anodizo": "anodizar",
    "anodizábamos": "anodizar",
    "anodizáis": "anodizar",
    "anodizáramos": "anodizar",
    "anodizáremos": "anodizar",
    "anodizásemos": "anodizar",
    "anodizó": "anodizar",
    "anodoncias": "anodoncia",
    "anoftalmías": "anoftalmía",
    "anolitos": "anolito",
    "anomalopías": "anomalopía",
    "anomaloscopios": "anomaloscopio",
    "anomalías": "anomalía",
    "anomias": "anomia",
    "anomías": "anomía",
    "anonada": "anonadar",
    "anonadaba": "anonadar",
    "anonadabais": "anonadar",
    "anonadaban": "anonadar",
    "anonadabas": "anonadar",
    "anonadaciones": "anonadación",
    "anonadad": "anonadar",
    "anonadada": "anonadar",
    "anonadadas": "anonadar",
    "anonadado": "anonadar",
    "anonadados": "anonadar",
    "anonadamientos": "anonadamiento",
    "anonadamos": "anonadar",
    "anonadan": "anonadar",
    "anonadando": "anonadar",
    "anonadara": "anonadar",
    "anonadarais": "anonadar",
    "anonadaran": "anonadar",
    "anonadaras": "anonadar",
    "anonadare": "anonadar",
    "anonadareis": "anonadar",
    "anonadaremos": "anonadar",
    "anonadaren": "anonadar",
    "anonadares": "anonadar",
    "anonadaron": "anonadar",
    "anonadará": "anonadar",
    "anonadarán": "anonadar",
    "anonadarás": "anonadar",
    "anonadaré": "anonadar",
    "anonadaréis": "anonadar",
    "anonadaría": "anonadar",
    "anonadaríais": "anonadar",
    "anonadaríamos": "anonadar",
    "anonadarían": "anonadar",
    "anonadarías": "anonadar",
    "anonadas": "anonadar",
    "anonadase": "anonadar",
    "anonadaseis": "anonadar",
    "anonadasen": "anonadar",
    "anonadases": "anonadar",
    "anonadaste": "anonadar",
    "anonadasteis": "anonadar",
    "anonade": "anonadar",
    "anonademos": "anonadar",
    "anonaden": "anonadar",
    "anonades": "anonadar",
    "anonado": "anonadar",
    "anonadábamos": "anonadar",
    "anonadáis": "anonadar",
    "anonadáramos": "anonadar",
    "anonadáremos": "anonadar",
    "anonadásemos": "anonadar",
    "anonadé": "anonadar",
    "anonadéis": "anonadar",
    "anonadó": "anonadar",
    "anonimatos": "anonimato",
    "anonimias": "anonimia",
    "anonimizaciones": "anonimización",
    "anopsias": "anopsia",
    "anoracs": "anorac",
    "anoraks": "anorak",
    "anorexias": "anorexia",
    "anorgasmias": "anorgasmia",
    "anormales": "anormal",
    "anormalidades": "anormalidad",
    "anorrectales": "anorrectal",
    "anortitas": "anortita",
    "anortoclasas": "anortoclasa",
    "anortopías": "anortopía",
    "anortositas": "anortosita",
    "anos": "ano",
    "anosmias": "anosmia",
    "anota": "anotar",
    "anotaba": "anotar",
    "anotabais": "anotar",
    "anotaban": "anotar",
    "anotabas": "anotar",
    "anotaciones": "anotación",
    "anotad": "anotar",
    "anotada": "anotar",
    "anotadas": "anotar",
    "anotado": "anotar",
    "anotadora": "anotador",
    "anotadoras": "anotador",
    "anotadores": "anotador",
    "anotados": "anotar",
    "anotamos": "anotar",
    "anotan": "anotar",
    "anotando": "anotar",
    "anotara": "anotar",
    "anotarais": "anotar",
    "anotaran": "anotar",
    "anotaras": "anotar",
    "anotare": "anotar",
    "anotareis": "anotar",
    "anotaremos": "anotar",
    "anotaren": "anotar",
    "anotares": "anotar",
    "anotaron": "anotar",
    "anotará": "anotar",
    "anotarán": "anotar",
    "anotarás": "anotar",
    "anotaré": "anotar",
    "anotaréis": "anotar",
    "anotaría": "anotar",
    "anotaríais": "anotar",
    "anotaríamos": "anotar",
    "anotarían": "anotar",
    "anotarías": "anotar",
    "anotas": "anotar",
    "anotase": "anotar",
    "anotaseis": "anotar",
    "anotasen": "anotar",
    "anotases": "anotar",
    "anotaste": "anotar",
    "anotasteis": "anotar",
    "anote": "anotar",
    "anotemos": "anotar",
    "anoten": "anotar",
    "anotes": "anotar",
    "anotias": "anotia",
    "anoto": "anotar",
    "anotropías": "anotropía",
    "anotábamos": "anotar",
    "anotáis": "anotar",
    "anotáramos": "anotar",
    "anotáremos": "anotar",
    "anotásemos": "anotar",
    "anoté": "anotar",
    "anotéis": "anotar",
    "anotó": "anotar",
    "anoxemias": "anoxemia",
    "anoxias": "anoxia",
    "anqueritas": "anquerita",
    "anquiloglosias": "anquiloglosia",
    "anquilosa": "anquilosar",
    "anquilosaba": "anquilosar",
    "anquilosabais": "anquilosar",
    "anquilosaban": "anquilosar",
    "anquilosabas": "anquilosar",
    "anquilosad": "anquilosar",
    "anquilosada": "anquilosar",
    "anquilosadas": "anquilosar",
    "anquilosado": "anquilosar",
    "anquilosados": "anquilosar",
    "anquilosamientos": "anquilosamiento",
    "anquilosamos": "anquilosar",
    "anquilosan": "anquilosar",
    "anquilosando": "anquilosar",
    "anquilosantes": "anquilosante",
    "anquilosara": "anquilosar",
    "anquilosarais": "anquilosar",
    "anquilosaran": "anquilosar",
    "anquilosaras": "anquilosar",
    "anquilosare": "anquilosar",
    "anquilosareis": "anquilosar",
    "anquilosaremos": "anquilosar",
    "anquilosaren": "anquilosar",
    "anquilosares": "anquilosar",
    "anquilosaron": "anquilosar",
    "anquilosará": "anquilosar",
    "anquilosarán": "anquilosar",
    "anquilosarás": "anquilosar",
    "anquilosaré": "anquilosar",
    "anquilosaréis": "anquilosar",
    "anquilosaría": "anquilosar",
    "anquilosaríais": "anquilosar",
    "anquilosaríamos": "anquilosar",
    "anquilosarían": "anquilosar",
    "anquilosarías": "anquilosar",
    "anquilosas": "anquilosar",
    "anquilosase": "anquilosar",
    "anquilosaseis": "anquilosar",
    "anquilosasen": "anquilosar",
    "anquilosases": "anquilosar",
    "anquilosaste": "anquilosar",
    "anquilosasteis": "anquilosar",
    "anquilose": "anquilosar",
    "anquilosemos": "anquilosar",
    "anquilosen": "anquilosar",
    "anquiloses": "anquilosar",
    "anquiloso": "anquilosar",
    "anquilostomas": "anquilostoma",
    "anquilosábamos": "anquilosar",
    "anquilosáis": "anquilosar",
    "anquilosáramos": "anquilosar",
    "anquilosáremos": "anquilosar",
    "anquilosásemos": "anquilosar",
    "anquilosé": "anquilosar",
    "anquiloséis": "anquilosar",
    "anquilosó": "anquilosar",
    "anquirinas": "anquirina",
    "ansas": "ansa",
    "anserina": "anserino",
    "anserinas": "anserino",
    "anserinos": "anserino",
    "anseática": "anseático",
    "anseáticas": "anseático",
    "anseáticos": "anseático",
    "ansiaba": "ansiar",
    "ansiabais": "ansiar",
    "ansiaban": "ansiar",
    "ansiabas": "ansiar",
    "ansiad": "ansiar",
    "ansiada": "ansiar",
    "ansiadas": "ansiar",
    "ansiado": "ansiar",
    "ansiados": "ansiar",
    "ansiamos": "ansiar",
    "ansiando": "ansiar",
    "ansiara": "ansiar",
    "ansiarais": "ansiar",
    "ansiaran": "ansiar",
    "ansiaras": "ansiar",
    "ansiare": "ansiar",
    "ansiareis": "ansiar",
    "ansiaremos": "ansiar",
    "ansiaren": "ansiar",
    "ansiares": "ansiar",
    "ansiaron": "ansiar",
    "ansiará": "ansiar",
    "ansiarán": "ansiar",
    "ansiarás": "ansiar",
    "ansiaré": "ansiar",
    "ansiaréis": "ansiar",
    "ansiaría": "ansiar",
    "ansiaríais": "ansiar",
    "ansiaríamos": "ansiar",
    "ansiarían": "ansiar",
    "ansiarías": "ansiar",
    "ansias": "ansia",
    "ansiase": "ansiar",
    "ansiaseis": "ansiar",
    "ansiasen": "ansiar",
    "ansiases": "ansiar",
    "ansiaste": "ansiar",
    "ansiasteis": "ansiar",
    "ansiedades": "ansiedad",
    "ansiemos": "ansiar",
    "ansiosa": "ansioso",
    "ansiosas": "ansioso",
    "ansiosos": "ansioso",
    "ansiábamos": "ansiar",
    "ansiáis": "ansiar",
    "ansiáramos": "ansiar",
    "ansiáremos": "ansiar",
    "ansiásemos": "ansiar",
    "ansié": "ansiar",
    "ansiéis": "ansiar",
    "ansió": "ansiar",
    "ansía": "ansiar",
    "ansían": "ansiar",
    "ansías": "ansiar",
    "ansíe": "ansiar",
    "ansíen": "ansiar",
    "ansíes": "ansiar",
    "ansío": "ansiar",
    "antagalla": "antagallar",
    "antagallaba": "antagallar",
    "antagallabais": "antagallar",
    "antagallaban": "antagallar",
    "antagallabas": "antagallar",
    "antagallad": "antagallar",
    "antagallada": "antagallar",
    "antagalladas": "antagallar",
    "antagallado": "antagallar",
    "antagallados": "antagallar",
    "antagallamos": "antagallar",
    "antagallan": "antagallar",
    "antagallando": "antagallar",
    "antagallara": "antagallar",
    "antagallarais": "antagallar",
    "antagallaran": "antagallar",
    "antagallaras": "antagallar",
    "antagallare": "antagallar",
    "antagallareis": "antagallar",
    "antagallaremos": "antagallar",
    "antagallaren": "antagallar",
    "antagallares": "antagallar",
    "antagallaron": "antagallar",
    "antagallará": "antagallar",
    "antagallarán": "antagallar",
    "antagallarás": "antagallar",
    "antagallaré": "antagallar",
    "antagallaréis": "antagallar",
    "antagallaría": "antagallar",
    "antagallaríais": "antagallar",
    "antagallaríamos": "antagallar",
    "antagallarían": "antagallar",
    "antagallarías": "antagallar",
    "antagallas": "antagallar",
    "antagallase": "antagallar",
    "antagallaseis": "antagallar",
    "antagallasen": "antagallar",
    "antagallases": "antagallar",
    "antagallaste": "antagallar",
    "antagallasteis": "antagallar",
    "antagalle": "antagallar",
    "antagallemos": "antagallar",
    "antagallen": "antagallar",
    "antagalles": "antagallar",
    "antagallo": "antagallar",
    "antagallábamos": "antagallar",
    "antagalláis": "antagallar",
    "antagalláramos": "antagallar",
    "antagalláremos": "antagallar",
    "antagallásemos": "antagallar",
    "antagallé": "antagallar",
    "antagalléis": "antagallar",
    "antagalló": "antagallar",
    "antagonismos": "antagonismo",
    "antagonistas": "antagonista",
    "antagónica": "antagónico",
    "antagónicas": "antagónico",
    "antagónicos": "antagónico",
    "antas": "anta",
    "antañona": "antañón",
    "antañonas": "antañón",
    "antañones": "antañón",
    "antealtares": "antealtar",
    "antebrazos": "antebrazo",
    "anteca": "anteco",
    "antecas": "anteco",
    "anteceda": "anteceder",
    "antecedamos": "anteceder",
    "antecedan": "anteceder",
    "antecedas": "anteceder",
    "antecede": "anteceder",
    "anteceded": "anteceder",
    "antecedemos": "anteceder",
    "anteceden": "anteceder",
    "antecedencias": "antecedencia",
    "antecedentes": "antecedente",
    "antecederemos": "anteceder",
    "antecederá": "anteceder",
    "antecederán": "anteceder",
    "antecederás": "anteceder",
    "antecederé": "anteceder",
    "antecederéis": "anteceder",
    "antecedería": "anteceder",
    "antecederíais": "anteceder",
    "antecederíamos": "anteceder",
    "antecederían": "anteceder",
    "antecederías": "anteceder",
    "antecedes": "anteceder",
    "antecedida": "anteceder",
    "antecedidas": "anteceder",
    "antecedido": "anteceder",
    "antecedidos": "anteceder",
    "antecediendo": "anteceder",
    "antecediera": "anteceder",
    "antecedierais": "anteceder",
    "antecedieran": "anteceder",
    "antecedieras": "anteceder",
    "antecediere": "anteceder",
    "antecediereis": "anteceder",
    "antecedieren": "anteceder",
    "antecedieres": "anteceder",
    "antecedieron": "anteceder",
    "antecediese": "anteceder",
    "antecedieseis": "anteceder",
    "antecediesen": "anteceder",
    "antecedieses": "anteceder",
    "antecedimos": "anteceder",
    "antecediste": "anteceder",
    "antecedisteis": "anteceder",
    "antecediéramos": "anteceder",
    "antecediéremos": "anteceder",
    "antecediésemos": "anteceder",
    "antecedió": "anteceder",
    "antecedo": "anteceder",
    "antecedáis": "anteceder",
    "antecedéis": "anteceder",
    "antecedí": "anteceder",
    "antecedía": "anteceder",
    "antecedíais": "anteceder",
    "antecedíamos": "anteceder",
    "antecedían": "anteceder",
    "antecedías": "anteceder",
    "antecesora": "antecesor",
    "antecesoras": "antecesor",
    "antecesores": "antecesor",
    "antecocinas": "antecocina",
    "antecoros": "antecoro",
    "antecos": "anteco",
    "antecámaras": "antecámara",
    "antedata": "antedatar",
    "antedataba": "antedatar",
    "antedatabais": "antedatar",
    "antedataban": "antedatar",
    "antedatabas": "antedatar",
    "antedataciones": "antedatación",
    "antedatad": "antedatar",
    "antedatada": "antedatar",
    "antedatadas": "antedatar",
    "antedatado": "antedatar",
    "antedatados": "antedatar",
    "antedatamos": "antedatar",
    "antedatan": "antedatar",
    "antedatando": "antedatar",
    "antedatara": "antedatar",
    "antedatarais": "antedatar",
    "antedataran": "antedatar",
    "antedataras": "antedatar",
    "antedatare": "antedatar",
    "antedatareis": "antedatar",
    "antedataremos": "antedatar",
    "antedataren": "antedatar",
    "antedatares": "antedatar",
    "antedataron": "antedatar",
    "antedatará": "antedatar",
    "antedatarán": "antedatar",
    "antedatarás": "antedatar",
    "antedataré": "antedatar",
    "antedataréis": "antedatar",
    "antedataría": "antedatar",
    "antedataríais": "antedatar",
    "antedataríamos": "antedatar",
    "antedatarían": "antedatar",
    "antedatarías": "antedatar",
    "antedatas": "antedatar",
    "antedatase": "antedatar",
    "antedataseis": "antedatar",
    "antedatasen": "antedatar",
    "antedatases": "antedatar",
    "antedataste": "antedatar",
    "antedatasteis": "antedatar",
    "antedate": "antedatar",
    "antedatemos": "antedatar",
    "antedaten": "antedatar",
    "antedates": "antedatar",
    "antedato": "antedatar",
    "antedatábamos": "antedatar",
    "antedatáis": "antedatar",
    "antedatáramos": "antedatar",
    "antedatáremos": "antedatar",
    "antedatásemos": "antedatar",
    "antedaté": "antedatar",
    "antedatéis": "antedatar",
    "antedató": "antedatar",
    "antedecid": "antedecir",
    "antedecimos": "antedecir",
    "antedecía": "antedecir",
    "antedecíais": "antedecir",
    "antedecíamos": "antedecir",
    "antedecían": "antedecir",
    "antedecías": "antedecir",
    "antedecís": "antedecir",
    "antedi": "antedecir",
    "antedice": "antedecir",
    "antedicen": "antedecir",
    "antedices": "antedecir",
    "antedicha": "antedecir",
    "antedichas": "antedecir",
    "antedicho": "antedecir",
    "antedichos": "antedecir",
    "antediciendo": "antedecir",
    "antediga": "antedecir",
    "antedigamos": "antedecir",
    "antedigan": "antedecir",
    "antedigas": "antedecir",
    "antedigo": "antedecir",
    "antedigáis": "antedecir",
    "antedije": "antedecir",
    "antedijera": "antedecir",
    "antedijerais": "antedecir",
    "antedijeran": "antedecir",
    "antedijeras": "antedecir",
    "antedijere": "antedecir",
    "antedijereis": "antedecir",
    "antedijeren": "antedecir",
    "antedijeres": "antedecir",
    "antedijeron": "antedecir",
    "antedijese": "antedecir",
    "antedijeseis": "antedecir",
    "antedijesen": "antedecir",
    "antedijeses": "antedecir",
    "antedijimos": "antedecir",
    "antedijiste": "antedecir",
    "antedijisteis": "antedecir",
    "antedijo": "antedecir",
    "antedijéramos": "antedecir",
    "antedijéremos": "antedecir",
    "antedijésemos": "antedecir",
    "antediluviana": "antediluviano",
    "antediluvianas": "antediluviano",
    "antediluvianos": "antediluviano",
    "antediremos": "antedecir",
    "antedirá": "antedecir",
    "antedirán": "antedecir",
    "antedirás": "antedecir",
    "antediré": "antedecir",
    "antediréis": "antedecir",
    "antediría": "antedecir",
    "antediríais": "antedecir",
    "antediríamos": "antedecir",
    "antedirían": "antedecir",
    "antedirías": "antedecir",
    "antefijas": "antefija",
    "antefirmas": "antefirma",
    "antegoniales": "antegonial",
    "antegréens": "antegreen",
    "anteiglesias": "anteiglesia",
    "antejuicios": "antejuicio",
    "antelaciones": "antelación",
    "antellevones": "antellevón",
    "antemanos": "antemano",
    "antemeridiana": "antemeridiano",
    "antemeridianas": "antemeridiano",
    "antemeridianos": "antemeridiano",
    "antemurales": "antemural",
    "antemuros": "antemuro",
    "antenas": "antena",
    "antenombres": "antenombre",
    "antenupciales": "antenupcial",
    "anteojeras": "anteojera",
    "anteojos": "anteojo",
    "antepalcos": "antepalco",
    "antepasada": "antepasado",
    "antepasadas": "antepasado",
    "antepasados": "antepasado",
    "antepaíses": "antepaís",
    "antepechos": "antepecho",
    "antepenúltima": "antepenúltimo",
    "antepenúltimas": "antepenúltimo",
    "antepenúltimos": "antepenúltimo",
    "antepondremos": "anteponer",
    "antepondrá": "anteponer",
    "antepondrán": "anteponer",
    "antepondrás": "anteponer",
    "antepondré": "anteponer",
    "antepondréis": "anteponer",
    "antepondría": "anteponer",
    "antepondríais": "anteponer",
    "antepondríamos": "anteponer",
    "antepondrían": "anteponer",
    "antepondrías": "anteponer",
    "antepone": "anteponer",
    "anteponed": "anteponer",
    "anteponemos": "anteponer",
    "anteponen": "anteponer",
    "antepones": "anteponer",
    "anteponga": "anteponer",
    "antepongamos": "anteponer",
    "antepongan": "anteponer",
    "antepongas": "anteponer",
    "antepongo": "anteponer",
    "antepongáis": "anteponer",
    "anteponiendo": "anteponer",
    "anteponéis": "anteponer",
    "anteponía": "anteponer",
    "anteponíais": "anteponer",
    "anteponíamos": "anteponer",
    "anteponían": "anteponer",
    "anteponías": "anteponer",
    "anteportadas": "anteportada",
    "anteposiciones": "anteposición",
    "anteproyectos": "anteproyecto",
    "antepuertas": "antepuerta",
    "antepuertos": "antepuerto",
    "antepuesta": "anteponer",
    "antepuestas": "anteponer",
    "antepuesto": "anteponer",
    "antepuestos": "anteponer",
    "antepuse": "anteponer",
    "antepusiera": "anteponer",
    "antepusierais": "anteponer",
    "antepusieran": "anteponer",
    "antepusieras": "anteponer",
    "antepusiere": "anteponer",
    "antepusiereis": "anteponer",
    "antepusieren": "anteponer",
    "antepusieres": "anteponer",
    "antepusieron": "anteponer",
    "antepusiese": "anteponer",
    "antepusieseis": "anteponer",
    "antepusiesen": "anteponer",
    "antepusieses": "anteponer",
    "antepusimos": "anteponer",
    "antepusiste": "anteponer",
    "antepusisteis": "anteponer",
    "antepusiéramos": "anteponer",
    "antepusiéremos": "anteponer",
    "antepusiésemos": "anteponer",
    "antepuso": "anteponer",
    "antepón": "anteponer",
    "anteras": "antera",
    "anteridiales": "anteridial",
    "anteridios": "anteridio",
    "anteriores": "anterior",
    "anterioridades": "anterioridad",
    "anterolaterales": "anterolateral",
    "anteroposteriores": "anteroposterior",
    "antesalas": "antesala",
    "antevea": "antever",
    "anteveamos": "antever",
    "antevean": "antever",
    "anteveas": "antever",
    "anteved": "antever",
    "antevemos": "antever",
    "anteven": "antevenir",
    "antevendremos": "antevenir",
    "antevendrá": "antevenir",
    "antevendrán": "antevenir",
    "antevendrás": "antevenir",
    "antevendré": "antevenir",
    "antevendréis": "antevenir",
    "antevendría": "antevenir",
    "antevendríais": "antevenir",
    "antevendríamos": "antevenir",
    "antevendrían": "antevenir",
    "antevendrías": "antevenir",
    "antevenga": "antevenir",
    "antevengamos": "antevenir",
    "antevengan": "antevenir",
    "antevengas": "antevenir",
    "antevengo": "antevenir",
    "antevengáis": "antevenir",
    "antevenid": "antevenir",
    "antevenida": "antevenir",
    "antevenidas": "antevenir",
    "antevenido": "antevenir",
    "antevenidos": "antevenir",
    "antevenimos": "antevenir",
    "antevenía": "antevenir",
    "anteveníais": "antevenir",
    "anteveníamos": "antevenir",
    "antevenían": "antevenir",
    "antevenías": "antevenir",
    "antevenís": "antevenir",
    "anteveo": "antever",
    "anteveremos": "antever",
    "anteverá": "antever",
    "anteverán": "antever",
    "anteverás": "antever",
    "anteveré": "antever",
    "anteveréis": "antever",
    "antevería": "antever",
    "anteveríais": "antever",
    "anteveríamos": "antever",
    "anteverían": "antever",
    "anteverías": "antever",
    "anteveáis": "antever",
    "anteveía": "antever",
    "anteveíais": "antever",
    "anteveíamos": "antever",
    "anteveían": "antever",
    "anteveías": "antever",
    "anteviendo": "antever",
    "anteviene": "antevenir",
    "antevienen": "antevenir",
    "antevienes": "antevenir",
    "anteviera": "antever",
    "antevierais": "antever",
    "antevieran": "antever",
    "antevieras": "antever",
    "anteviere": "antever",
    "anteviereis": "antever",
    "antevieren": "antever",
    "antevieres": "antever",
    "antevieron": "antever",
    "anteviese": "antever",
    "antevieseis": "antever",
    "anteviesen": "antever",
    "antevieses": "antever",
    "antevimos": "antever",
    "antevine": "antevenir",
    "anteviniendo": "antevenir",
    "anteviniera": "antevenir",
    "antevinierais": "antevenir",
    "antevinieran": "antevenir",
    "antevinieras": "antevenir",
    "anteviniere": "antevenir",
    "anteviniereis": "antevenir",
    "antevinieren": "antevenir",
    "antevinieres": "antevenir",
    "antevinieron": "antevenir",
    "anteviniese": "antevenir",
    "antevinieseis": "antevenir",
    "anteviniesen": "antevenir",
    "antevinieses": "antevenir",
    "antevinimos": "antevenir",
    "anteviniste": "antevenir",
    "antevinisteis": "antevenir",
    "anteviniéramos": "antevenir",
    "anteviniéremos": "antevenir",
    "anteviniésemos": "antevenir",
    "antevino": "antevenir",
    "antevista": "antever",
    "antevistas": "antever",
    "anteviste": "antever",
    "antevisteis": "antever",
    "antevisto": "antever",
    "antevistos": "antever",
    "anteviéramos": "antever",
    "anteviéremos": "antever",
    "anteviésemos": "antever",
    "antevió": "antever",
    "antevé": "antever",
    "antevéis": "antever",
    "antevén": "antever",
    "antevés": "antever",
    "anteví": "antever",
    "antevísperas": "antevíspera",
    "antiabolicionismos": "antiabolicionismo",
    "antiabolicionistas": "antiabolicionista",
    "antiabrasiva": "antiabrasivo",
    "antiabrasivas": "antiabrasivo",
    "antiabrasivos": "antiabrasivo",
    "antiacadémica": "antiacadémico",
    "antiacadémicas": "antiacadémico",
    "antiacadémicos": "antiacadémico",
    "antiadherentes": "antiadherente",
    "antiadhesivos": "antiadhesivo",
    "antiafrodisíacos": "antiafrodisíaco",
    "antialbumosas": "antialbumosa",
    "antialcalina": "antialcalino",
    "antialcalinas": "antialcalino",
    "antialcalinos": "antialcalino",
    "antialcohólica": "antialcohólico",
    "antialcohólicas": "antialcohólico",
    "antialcohólicos": "antialcohólico",
    "antialdoximas": "antialdoxima",
    "antialotipos": "antialotipo",
    "antiamericana": "antiamericano",
    "antiamericanas": "antiamericano",
    "antiamericanos": "antiamericano",
    "antiarrugas": "antiarruga",
    "antiatómica": "antiatómico",
    "antiatómicas": "antiatómico",
    "antiatómicos": "antiatómico",
    "antiaérea": "antiaéreo",
    "antiaéreas": "antiaéreo",
    "antiaéreos": "antiaéreo",
    "antibalanceadora": "antibalanceador",
    "antibalanceadoras": "antibalanceador",
    "antibalanceadores": "antibalanceador",
    "antibarionas": "antibariona",
    "antibiótica": "antibiótico",
    "antibióticas": "antibiótico",
    "antibióticos": "antibiótico",
    "antiblindada": "antiblindado",
    "antiblindadas": "antiblindado",
    "antiblindados": "antiblindado",
    "antibloqueos": "antibloqueo",
    "anticadencias": "anticadencia",
    "anticancerígena": "anticancerígeno",
    "anticancerígenas": "anticancerígeno",
    "anticancerígenos": "anticancerígeno",
    "anticapacitativa": "anticapacitativo",
    "anticapacitativas": "anticapacitativo",
    "anticapacitativos": "anticapacitativo",
    "anticapacitiva": "anticapacitivo",
    "anticapacitivas": "anticapacitivo",
    "anticapacitivos": "anticapacitivo",
    "anticapilares": "anticapilar",
    "anticarbonillas": "anticarbonilla",
    "anticariógena": "anticariógeno",
    "anticariógenas": "anticariógeno",
    "anticariógenos": "anticariógeno",
    "anticastristas": "anticastrista",
    "anticatalanistas": "anticatalanista",
    "anticatalizadores": "anticatalizador",
    "anticelulítica": "anticelulítico",
    "anticelulíticas": "anticelulítico",
    "anticelulíticos": "anticelulítico",
    "anticiclones": "anticiclón",
    "anticiclónica": "anticiclónico",
    "anticiclónicas": "anticiclónico",
    "anticiclónicos": "anticiclónico",
    "anticipa": "anticipar",
    "anticipaba": "anticipar",
    "anticipabais": "anticipar",
    "anticipaban": "anticipar",
    "anticipabas": "anticipar",
    "anticipaciones": "anticipación",
    "anticipad": "anticipar",
    "anticipada": "anticipar",
    "anticipadas": "anticipar",
    "anticipado": "anticipar",
    "anticipados": "anticipar",
    "anticipamos": "anticipar",
    "anticipan": "anticipar",
    "anticipando": "anticipar",
    "anticipara": "anticipar",
    "anticiparais": "anticipar",
    "anticiparan": "anticipar",
    "anticiparas": "anticipar",
    "anticipare": "anticipar",
    "anticipareis": "anticipar",
    "anticiparemos": "anticipar",
    "anticiparen": "anticipar",
    "anticipares": "anticipar",
    "anticiparon": "anticipar",
    "anticipará": "anticipar",
    "anticiparán": "anticipar",
    "anticiparás": "anticipar",
    "anticiparé": "anticipar",
    "anticiparéis": "anticipar",
    "anticiparía": "anticipar",
    "anticiparíais": "anticipar",
    "anticiparíamos": "anticipar",
    "anticiparían": "anticipar",
    "anticiparías": "anticipar",
    "anticipas": "anticipar",
    "anticipase": "anticipar",
    "anticipaseis": "anticipar",
    "anticipasen": "anticipar",
    "anticipases": "anticipar",
    "anticipaste": "anticipar",
    "anticipasteis": "anticipar",
    "anticipativa": "anticipativo",
    "anticipativas": "anticipativo",
    "anticipativos": "anticipativo",
    "anticipe": "anticipar",
    "anticipemos": "anticipar",
    "anticipen": "anticipar",
    "anticipes": "anticipar",
    "anticipo": "anticipar",
    "anticipos": "anticipo",
    "anticipábamos": "anticipar",
    "anticipáis": "anticipar",
    "anticipáramos": "anticipar",
    "anticipáremos": "anticipar",
    "anticipásemos": "anticipar",
    "anticipé": "anticipar",
    "anticipéis": "anticipar",
    "anticipó": "anticipar",
    "anticlericales": "anticlerical",
    "anticlericalismos": "anticlericalismo",
    "anticlinales": "anticlinal",
    "anticloros": "anticloro",
    "anticlímaxes": "anticlímax",
    "anticoagulantes": "anticoagulante",
    "anticoagulinas": "anticoagulina",
    "anticolonialismos": "anticolonialismo",
    "anticolonialistas": "anticolonialista",
    "anticompetitiva": "anticompetitivo",
    "anticompetitivas": "anticompetitivo",
    "anticompetitivos": "anticompetitivo",
    "anticomplementaria": "anticomplementario",
    "anticomplementarias": "anticomplementario",
    "anticomplementarios": "anticomplementario",
    "anticomunismos": "anticomunismo",
    "anticomunistas": "anticomunista",
    "anticonceptiva": "anticonceptivo",
    "anticonceptivas": "anticonceptivo",
    "anticonceptivos": "anticonceptivo",
    "anticondensaciones": "anticondensación",
    "anticongelantes": "anticongelante",
    "anticonstitucionales": "anticonstitucional",
    "anticonsumistas": "anticonsumista",
    "anticontaminantes": "anticontaminante",
    "anticonvulsivos": "anticonvulsivo",
    "anticorrosiones": "anticorrosión",
    "anticorrosiva": "anticorrosivo",
    "anticorrosivas": "anticorrosivo",
    "anticorrosivos": "anticorrosivo",
    "anticresistas": "anticresista",
    "anticristos": "anticristo",
    "anticrética": "anticrético",
    "anticréticas": "anticrético",
    "anticréticos": "anticrético",
    "anticuaba": "anticuar",
    "anticuabais": "anticuar",
    "anticuaban": "anticuar",
    "anticuabas": "anticuar",
    "anticuad": "anticuar",
    "anticuada": "anticuar",
    "anticuadas": "anticuar",
    "anticuado": "anticuar",
    "anticuados": "anticuar",
    "anticuamos": "anticuar",
    "anticuando": "anticuar",
    "anticuara": "anticuar",
    "anticuarais": "anticuar",
    "anticuaran": "anticuar",
    "anticuaras": "anticuar",
    "anticuare": "anticuar",
    "anticuareis": "anticuar",
    "anticuaremos": "anticuar",
    "anticuaren": "anticuar",
    "anticuares": "anticuar",
    "anticuaria": "anticuario",
    "anticuarias": "anticuario",
    "anticuarios": "anticuario",
    "anticuaron": "anticuar",
    "anticuará": "anticuar",
    "anticuarán": "anticuar",
    "anticuarás": "anticuar",
    "anticuaré": "anticuar",
    "anticuaréis": "anticuar",
    "anticuaría": "anticuar",
    "anticuaríais": "anticuar",
    "anticuaríamos": "anticuar",
    "anticuarían": "anticuar",
    "anticuarías": "anticuar",
    "anticuase": "anticuar",
    "anticuaseis": "anticuar",
    "anticuasen": "anticuar",
    "anticuases": "anticuar",
    "anticuaste": "anticuar",
    "anticuasteis": "anticuar",
    "anticuemos": "anticuar",
    "anticuerpos": "anticuerpo",
    "anticuábamos": "anticuar",
    "anticuáis": "anticuar",
    "anticuáramos": "anticuar",
    "anticuáremos": "anticuar",
    "anticuásemos": "anticuar",
    "anticué": "anticuar",
    "anticuéis": "anticuar",
    "anticuó": "anticuar",
    "anticátodos": "anticátodo",
    "anticíclica": "anticíclico",
    "anticíclicas": "anticíclico",
    "anticíclicos": "anticíclico",
    "anticúa": "anticuar",
    "anticúan": "anticuar",
    "anticúas": "anticuar",
    "anticúe": "anticuar",
    "anticúen": "anticuar",
    "anticúes": "anticuar",
    "anticúo": "anticuar",
    "antideflagrantes": "antideflagrante",
    "antidemocrática": "antidemocrático",
    "antidemocráticas": "antidemocrático",
    "antidemocráticos": "antidemocrático",
    "antidemócratas": "antidemócrata",
    "antideportiva": "antideportivo",
    "antideportivas": "antideportivo",
    "antideportivos": "antideportivo",
    "antidepresivos": "antidepresivo",
    "antiderivadas": "antiderivada",
    "antiderrapantes": "antiderrapante",
    "antidesgastes": "antidesgaste",
    "antideslizantes": "antideslizante",
    "antideslumbrantes": "antideslumbrante",
    "antidesvanecedora": "antidesvanecedor",
    "antidesvanecedoras": "antidesvanecedor",
    "antidesvanecedores": "antidesvanecedor",
    "antidetonantes": "antidetonante",
    "antidiluviana": "antidiluviano",
    "antidiluvianas": "antidiluviano",
    "antidiluvianos": "antidiluviano",
    "antidopings": "antidoping",
    "antidunas": "antiduna",
    "antieconómica": "antieconómico",
    "antieconómicas": "antieconómico",
    "antieconómicos": "antieconómico",
    "antienzimas": "antienzima",
    "antiepicentros": "antiepicentro",
    "antierosiva": "antierosivo",
    "antierosivas": "antierosivo",
    "antierosivos": "antierosivo",
    "antiestática": "antiestático",
    "antiestáticas": "antiestático",
    "antiestáticos": "antiestático",
    "antiestética": "antiestético",
    "antiestéticas": "antiestético",
    "antiestéticos": "antiestético",
    "antieuropea": "antieuropeo",
    "antieuropeas": "antieuropeo",
    "antieuropeos": "antieuropeo",
    "antiexplosiva": "antiexplosivo",
    "antiexplosivas": "antiexplosivo",
    "antiexplosivos": "antiexplosivo",
    "antifaces": "antifaz",
    "antifascismos": "antifascismo",
    "antifascistas": "antifascista",
    "antifederalistas": "antifederalista",
    "antifeministas": "antifeminista",
    "antiferias": "antiferia",
    "antiferromagnetismos": "antiferromagnetismo",
    "antiferromagnética": "antiferromagnético",
    "antiferromagnéticas": "antiferromagnético",
    "antiferromagnéticos": "antiferromagnético",
    "antifibrinolisinas": "antifibrinolisina",
    "antifloculantes": "antifloculante",
    "antifonales": "antifonal",
    "antifonarios": "antifonario",
    "antifranquistas": "antifranquista",
    "antifricciones": "antifricción",
    "antifuncionales": "antifuncional",
    "antifúngicos": "antifúngico",
    "antigasa": "antigás",
    "antigasas": "antigás",
    "antigases": "antigás",
    "antigenicidades": "antigenicidad",
    "antiglobulinas": "antiglobulina",
    "antigoritas": "antigorita",
    "antigua": "antiguo",
    "antiguaba": "antiguar",
    "antiguabais": "antiguar",
    "antiguaban": "antiguar",
    "antiguabas": "antiguar",
    "antiguad": "antiguar",
    "antiguada": "antiguar",
    "antiguadas": "antiguar",
    "antiguado": "antiguar",
    "antiguados": "antiguar",
    "antiguallas": "antigualla",
    "antiguamos": "antiguar",
    "antiguan": "antiguar",
    "antiguando": "antiguar",
    "antiguara": "antiguar",
    "antiguarais": "antiguar",
    "antiguaran": "antiguar",
    "antiguaras": "antiguar",
    "antiguare": "antiguar",
    "antiguareis": "antiguar",
    "antiguaremos": "antiguar",
    "antiguaren": "antiguar",
    "antiguares": "antiguar",
    "antiguaron": "antiguar",
    "antiguará": "antiguar",
    "antiguarán": "antiguar",
    "antiguarás": "antiguar",
    "antiguaré": "antiguar",
    "antiguaréis": "antiguar",
    "antiguaría": "antiguar",
    "antiguaríais": "antiguar",
    "antiguaríamos": "antiguar",
    "antiguarían": "antiguar",
    "antiguarías": "antiguar",
    "antiguas": "antiguo",
    "antiguase": "antiguar",
    "antiguaseis": "antiguar",
    "antiguasen": "antiguar",
    "antiguases": "antiguar",
    "antiguaste": "antiguar",
    "antiguasteis": "antiguar",
    "antigubernamentales": "antigubernamental",
    "antiguo": "antiguar",
    "antiguos": "antiguo",
    "antiguábamos": "antiguar",
    "antiguáis": "antiguar",
    "antiguáramos": "antiguar",
    "antiguáremos": "antiguar",
    "antiguásemos": "antiguar",
    "antiguó": "antiguar",
    "antigénica": "antigénico",
    "antigénicas": "antigénico",
    "antigénicos": "antigénico",
    "antigüe": "antiguar",
    "antigüedades": "antigüedad",
    "antigüemos": "antiguar",
    "antigüen": "antiguar",
    "antigües": "antiguar",
    "antigüé": "antiguar",
    "antigüéis": "antiguar",
    "antihelmínticos": "antihelmíntico",
    "antiherrumbrosa": "antiherrumbroso",
    "antiherrumbrosas": "antiherrumbroso",
    "antiherrumbrosos": "antiherrumbroso",
    "antihigiénica": "antihigiénico",
    "antihigiénicas": "antihigiénico",
    "antihigiénicos": "antihigiénico",
    "antihiperglucemiantes": "antihiperglucemiante",
    "antihipoglucemiantes": "antihipoglucemiante",
    "antihistaminas": "antihistamina",
    "antihistamínica": "antihistamínico",
    "antihistamínicas": "antihistamínico",
    "antihistamínicos": "antihistamínico",
    "antihiática": "antihiático",
    "antihiáticas": "antihiático",
    "antihiáticos": "antihiático",
    "antihormonas": "antihormona",
    "antihumana": "antihumano",
    "antihumanas": "antihumano",
    "antihumanos": "antihumano",
    "antihéroes": "antihéroe",
    "antiidiotipos": "antiidiotipo",
    "antiidiotopos": "antiidiotopo",
    "antiincrustantes": "antiincrustante",
    "antiinductiva": "antiinductivo",
    "antiinductivas": "antiinductivo",
    "antiinductivos": "antiinductivo",
    "antiinflacionaria": "antiinflacionario",
    "antiinflacionarias": "antiinflacionario",
    "antiinflacionarios": "antiinflacionario",
    "antijuricidades": "antijuricidad",
    "antijuridicidades": "antijuridicidad",
    "antijurídica": "antijurídico",
    "antijurídicas": "antijurídico",
    "antijurídicos": "antijurídico",
    "antiliberales": "antiliberal",
    "antilinfocitaria": "antilinfocitario",
    "antilinfocitarias": "antilinfocitario",
    "antilinfocitarios": "antilinfocitario",
    "antilinfocítica": "antilinfocítico",
    "antilinfocíticas": "antilinfocítico",
    "antilinfocíticos": "antilinfocítico",
    "antilocales": "antilocal",
    "antilogaritmos": "antilogaritmo",
    "antilogios": "antilogio",
    "antilogías": "antilogía",
    "antilógica": "antilógico",
    "antilógicas": "antilógico",
    "antilógicos": "antilógico",
    "antimagnética": "antimagnético",
    "antimagnéticas": "antimagnético",
    "antimagnéticos": "antimagnético",
    "antimaquinismos": "antimaquinismo",
    "antimaterias": "antimateria",
    "antimeridianos": "antimeridiano",
    "antimetabolitos": "antimetabolito",
    "antimicinas": "antimicina",
    "antimicrobiana": "antimicrobiano",
    "antimicrobianas": "antimicrobiano",
    "antimicrobianos": "antimicrobiano",
    "antimilitarismos": "antimilitarismo",
    "antimilitaristas": "antimilitarista",
    "antimitótica": "antimitótico",
    "antimitóticas": "antimitótico",
    "antimitóticos": "antimitótico",
    "antimodas": "antimoda",
    "antimohos": "antimoho",
    "antimoniales": "antimonial",
    "antimoniatos": "antimoniato",
    "antimonios": "antimonio",
    "antimoniosa": "antimonioso",
    "antimoniosas": "antimonioso",
    "antimoniosos": "antimonioso",
    "antimonitas": "antimonita",
    "antimoniuros": "antimoniuro",
    "antimonopolios": "antimonopolio",
    "antimonárquica": "antimonárquico",
    "antimonárquicas": "antimonárquico",
    "antimonárquicos": "antimonárquico",
    "antimorales": "antimoral",
    "antimunicipalistas": "antimunicipalista",
    "antinacionalistas": "antinacionalista",
    "antinaturales": "antinatural",
    "antineutrones": "antineutrón",
    "antinodos": "antinodo",
    "antinomias": "antinomia",
    "antinomías": "antinomía",
    "antinormativa": "antinormativo",
    "antinormativas": "antinormativo",
    "antinormativos": "antinormativo",
    "antinorteamericana": "antinorteamericano",
    "antinorteamericanas": "antinorteamericano",
    "antinorteamericanos": "antinorteamericano",
    "antinucleares": "antinuclear",
    "antinómica": "antinómico",
    "antinómicas": "antinómico",
    "antinómicos": "antinómico",
    "antioccidentales": "antioccidental",
    "antiofídica": "antiofídico",
    "antiofídicas": "antiofídico",
    "antiofídicos": "antiofídico",
    "antioxidantes": "antioxidante",
    "antiozonizantes": "antiozonizante",
    "antipalúdica": "antipalúdico",
    "antipalúdicas": "antipalúdico",
    "antipalúdicos": "antipalúdico",
    "antipapas": "antipapa",
    "antiparalela": "antiparalelo",
    "antiparalelas": "antiparalelo",
    "antiparalelos": "antiparalelo",
    "antiparas": "antipara",
    "antiparasitaria": "antiparasitario",
    "antiparasitarias": "antiparasitario",
    "antiparasitarios": "antiparasitario",
    "antiparlamentaria": "antiparlamentario",
    "antiparlamentarias": "antiparlamentario",
    "antiparlamentarios": "antiparlamentario",
    "antipartículas": "antipartícula",
    "antipatriotas": "antipatriota",
    "antipatriótica": "antipatriótico",
    "antipatrióticas": "antipatriótico",
    "antipatrióticos": "antipatriótico",
    "antipatías": "antipatía",
    "antipedagógica": "antipedagógico",
    "antipedagógicas": "antipedagógico",
    "antipedagógicos": "antipedagógico",
    "antipendios": "antipendio",
    "antiperistaltismos": "antiperistaltismo",
    "antiperistáltica": "antiperistáltico",
    "antiperistálticas": "antiperistáltico",
    "antiperistálticos": "antiperistáltico",
    "antipersonas": "antipersona",
    "antipirinas": "antipirina",
    "antipiréticos": "antipirético",
    "antiplasminas": "antiplasmina",
    "antipolítica": "antipolítico",
    "antipolíticas": "antipolítico",
    "antipolíticos": "antipolítico",
    "antipressings": "antipressing",
    "antiproteccionistas": "antiproteccionista",
    "antiprotones": "antiprotón",
    "antipsórica": "antipsórico",
    "antipsóricas": "antipsórico",
    "antipsóricos": "antipsórico",
    "antipática": "antipático",
    "antipáticas": "antipático",
    "antipáticos": "antipático",
    "antireflexiva": "antireflexivo",
    "antireflexivas": "antireflexivo",
    "antireflexivos": "antireflexivo",
    "antirracismos": "antirracismo",
    "antirreflectantes": "antirreflectante",
    "antirreflectora": "antirreflector",
    "antirreflectoras": "antirreflector",
    "antirreflectores": "antirreflector",
    "antirreflejantes": "antirreflejante",
    "antirreglamentaria": "antirreglamentario",
    "antirreglamentarias": "antirreglamentario",
    "antirreglamentarios": "antirreglamentario",
    "antirreligiosa": "antirreligioso",
    "antirreligiosas": "antirreligioso",
    "antirreligiosos": "antirreligioso",
    "antirresonancias": "antirresonancia",
    "antirresonantes": "antirresonante",
    "antirreumáticos": "antirreumático",
    "antirrevolucionaria": "antirrevolucionario",
    "antirrevolucionarias": "antirrevolucionario",
    "antirrevolucionarios": "antirrevolucionario",
    "antirrobos": "antirrobo",
    "antirroja": "antirrojo",
    "antirrojas": "antirrojo",
    "antirrojos": "antirrojo",
    "antirrotatoria": "antirrotatorio",
    "antirrotatorias": "antirrotatorio",
    "antirrotatorios": "antirrotatorio",
    "antirrábica": "antirrábico",
    "antirrábicas": "antirrábico",
    "antirrábicos": "antirrábico",
    "antiscia": "antiscio",
    "antiscias": "antiscio",
    "antiscios": "antiscio",
    "antiselecciones": "antiselección",
    "antisemitas": "antisemita",
    "antisemitismos": "antisemitismo",
    "antisemítica": "antisemítico",
    "antisemíticas": "antisemítico",
    "antisemíticos": "antisemítico",
    "antisepsias": "antisepsia",
    "antisialagoga": "antisialagogo",
    "antisialagogas": "antisialagogo",
    "antisialagogos": "antisialagogo",
    "antisifilítica": "antisifilítico",
    "antisifilíticas": "antisifilítico",
    "antisifilíticos": "antisifilítico",
    "antisimétrica": "antisimétrico",
    "antisimétricas": "antisimétrico",
    "antisimétricos": "antisimétrico",
    "antisiálica": "antisiálico",
    "antisiálicas": "antisiálico",
    "antisiálicos": "antisiálico",
    "antisobornos": "antisoborno",
    "antisociales": "antisocial",
    "antisolares": "antisolar",
    "antisonora": "antisonoro",
    "antisonoras": "antisonoro",
    "antisonoros": "antisonoro",
    "antispastos": "antispasto",
    "antistrofas": "antistrofa",
    "antisubmarina": "antisubmarino",
    "antisubmarinas": "antisubmarino",
    "antisubmarinos": "antisubmarino",
    "antisueros": "antisuero",
    "antiséptica": "antiséptico",
    "antisépticas": "antiséptico",
    "antisépticos": "antiséptico",
    "antisísmica": "antisísmico",
    "antisísmicas": "antisísmico",
    "antisísmicos": "antisísmico",
    "antitabaquistas": "antitabaquista",
    "antitanques": "antitanque",
    "antiterroristas": "antiterrorista",
    "antitoxinas": "antitoxina",
    "antitragos": "antitrago",
    "antitranspirantes": "antitranspirante",
    "antitrinitaria": "antitrinitario",
    "antitrinitarias": "antitrinitario",
    "antitrinitarios": "antitrinitario",
    "antitrombinas": "antitrombina",
    "antituberculosa": "antituberculoso",
    "antituberculosas": "antituberculoso",
    "antituberculosos": "antituberculoso",
    "antitusígenos": "antitusígeno",
    "antitérmicos": "antitérmico",
    "antitética": "antitético",
    "antitéticas": "antitético",
    "antitéticos": "antitético",
    "antivalencias": "antivalencia",
    "antivenenos": "antiveneno",
    "antivenérea": "antivenéreo",
    "antivenéreas": "antivenéreo",
    "antivenéreos": "antivenéreo",
    "antivibratoria": "antivibratorio",
    "antivibratorias": "antivibratorio",
    "antivibratorios": "antivibratorio",
    "antivirales": "antiviral",
    "antivitaminas": "antivitamina",
    "antivírica": "antivírico",
    "antivíricas": "antivírico",
    "antivíricos": "antivírico",
    "antiácida": "antiácido",
    "antiácidas": "antiácido",
    "antiácidos": "antiácido",
    "antofilitas": "antofilita",
    "antoja": "antojar",
    "antojaba": "antojar",
    "antojabais": "antojar",
    "antojaban": "antojar",
    "antojabas": "antojar",
    "antojad": "antojar",
    "antojada": "antojar",
    "antojadas": "antojar",
    "antojadiza": "antojadizo",
    "antojadizas": "antojadizo",
    "antojadizos": "antojadizo",
    "antojado": "antojar",
    "antojados": "antojar",
    "antojamos": "antojar",
    "antojan": "antojar",
    "antojando": "antojar",
    "antojara": "antojar",
    "antojarais": "antojar",
    "antojaran": "antojar",
    "antojaras": "antojar",
    "antojare": "antojar",
    "antojareis": "antojar",
    "antojaremos": "antojar",
    "antojaren": "antojar",
    "antojares": "antojar",
    "antojaron": "antojar",
    "antojará": "antojar",
    "antojarán": "antojar",
    "antojarás": "antojar",
    "antojaré": "antojar",
    "antojaréis": "antojar",
    "antojaría": "antojar",
    "antojaríais": "antojar",
    "antojaríamos": "antojar",
    "antojarían": "antojar",
    "antojarías": "antojar",
    "antojas": "antojar",
    "antojase": "antojar",
    "antojaseis": "antojar",
    "antojasen": "antojar",
    "antojases": "antojar",
    "antojaste": "antojar",
    "antojasteis": "antojar",
    "antoje": "antojar",
    "antojemos": "antojar",
    "antojen": "antojar",
    "antojes": "antojar",
    "antojo": "antojar",
    "antojos": "antojo",
    "antojábamos": "antojar",
    "antojáis": "antojar",
    "antojáramos": "antojar",
    "antojáremos": "antojar",
    "antojásemos": "antojar",
    "antojé": "antojar",
    "antojéis": "antojar",
    "antojó": "antojar",
    "antologías": "antología",
    "antológica": "antológico",
    "antológicas": "antológico",
    "antológicos": "antológico",
    "antoniana": "antoniano",
    "antonianas": "antoniano",
    "antonianos": "antoniano",
    "antonimias": "antonimia",
    "antoninianos": "antoniniano",
    "antonomasias": "antonomasia",
    "antorchas": "antorcha",
    "antorchitas": "antorchita",
    "antozoarios": "antozoario",
    "antozoos": "antozoo",
    "antracenos": "antraceno",
    "antracitas": "antracita",
    "antracologías": "antracología",
    "antraflavinas": "antraflavina",
    "antrales": "antral",
    "antranilatos": "antranilato",
    "antranoles": "antranol",
    "antranílica": "antranílico",
    "antranílicas": "antranílico",
    "antranílicos": "antranílico",
    "antraquinonas": "antraquinona",
    "antrarrobinas": "antrarrobina",
    "antraxes": "antrax",
    "antrectomías": "antrectomía",
    "antrocoanales": "antrocoanal",
    "antropocentrismos": "antropocentrismo",
    "antropocéntrica": "antropocéntrico",
    "antropocéntricas": "antropocéntrico",
    "antropocéntricos": "antropocéntrico",
    "antropofagias": "antropofagia",
    "antropogeografías": "antropogeografía",
    "antropografías": "antropografía",
    "antropográfica": "antropográfico",
    "antropográficas": "antropográfico",
    "antropográficos": "antropográfico",
    "antropogénica": "antropogénico",
    "antropogénicas": "antropogénico",
    "antropogénicos": "antropogénico",
    "antropoidea": "antropoideo",
    "antropoideas": "antropoideo",
    "antropoideos": "antropoideo",
    "antropoides": "antropoide",
    "antropolatrías": "antropolatría",
    "antropologías": "antropología",
    "antropológica": "antropológico",
    "antropológicas": "antropológico",
    "antropológicos": "antropológico",
    "antropometrías": "antropometría",
    "antropomorfismos": "antropomorfismo",
    "antropomorfos": "antropomorfo",
    "antropométrica": "antropométrico",
    "antropométricas": "antropométrico",
    "antropométricos": "antropométrico",
    "antropomórfica": "antropomórfico",
    "antropomórficas": "antropomórfico",
    "antropomórficos": "antropomórfico",
    "antroponimias": "antroponimia",
    "antroponímica": "antroponímico",
    "antroponímicas": "antroponímico",
    "antroponímicos": "antroponímico",
    "antropopitecos": "antropopiteco",
    "antropófaga": "antropófago",
    "antropófagas": "antropófago",
    "antropófagos": "antropófago",
    "antropófila": "antropófilo",
    "antropófilas": "antropófilo",
    "antropófilos": "antropófilo",
    "antropóloga": "antropólogo",
    "antropólogas": "antropólogo",
    "antropólogos": "antropólogo",
    "antropómetros": "antropómetro",
    "antropónimos": "antropónimo",
    "antros": "antro",
    "antrotomías": "antrotomía",
    "antrueja": "antruejar",
    "antruejaba": "antruejar",
    "antruejabais": "antruejar",
    "antruejaban": "antruejar",
    "antruejabas": "antruejar",
    "antruejad": "antruejar",
    "antruejada": "antruejar",
    "antruejadas": "antruejar",
    "antruejado": "antruejar",
    "antruejados": "antruejar",
    "antruejamos": "antruejar",
    "antruejan": "antruejar",
    "antruejando": "antruejar",
    "antruejara": "antruejar",
    "antruejarais": "antruejar",
    "antruejaran": "antruejar",
    "antruejaras": "antruejar",
    "antruejare": "antruejar",
    "antruejareis": "antruejar",
    "antruejaremos": "antruejar",
    "antruejaren": "antruejar",
    "antruejares": "antruejar",
    "antruejaron": "antruejar",
    "antruejará": "antruejar",
    "antruejarán": "antruejar",
    "antruejarás": "antruejar",
    "antruejaré": "antruejar",
    "antruejaréis": "antruejar",
    "antruejaría": "antruejar",
    "antruejaríais": "antruejar",
    "antruejaríamos": "antruejar",
    "antruejarían": "antruejar",
    "antruejarías": "antruejar",
    "antruejas": "antruejar",
    "antruejase": "antruejar",
    "antruejaseis": "antruejar",
    "antruejasen": "antruejar",
    "antruejases": "antruejar",
    "antruejaste": "antruejar",
    "antruejasteis": "antruejar",
    "antrueje": "antruejar",
    "antruejemos": "antruejar",
    "antruejen": "antruejar",
    "antruejes": "antruejar",
    "antruejo": "antruejar",
    "antruejábamos": "antruejar",
    "antruejáis": "antruejar",
    "antruejáramos": "antruejar",
    "antruejáremos": "antruejar",
    "antruejásemos": "antruejar",
    "antruejé": "antruejar",
    "antruejéis": "antruejar",
    "antruejó": "antruejar",
    "antrópica": "antrópico",
    "antrópicas": "antrópico",
    "antrópicos": "antrópico",
    "antártica": "antártico",
    "antárticas": "antártico",
    "antárticos": "antártico",
    "antídotos": "antídoto",
    "antífonas": "antífona",
    "antígenos": "antígeno",
    "antílopes": "antílope",
    "antípodas": "antípoda",
    "antónimos": "antónimo",
    "anuales": "anual",
    "anualidades": "anualidad",
    "anualizaciones": "anualización",
    "anualizada": "anualizado",
    "anualizadas": "anualizado",
    "anualizados": "anualizado",
    "anuarios": "anuario",
    "anubada": "anubado",
    "anubadas": "anubado",
    "anubados": "anubado",
    "anubarrada": "anubarrado",
    "anubarradas": "anubarrado",
    "anubarrados": "anubarrado",
    "anubla": "anublar",
    "anublaba": "anublar",
    "anublabais": "anublar",
    "anublaban": "anublar",
    "anublabas": "anublar",
    "anublad": "anublar",
    "anublada": "anublar",
    "anubladas": "anublar",
    "anublado": "anublar",
    "anublados": "anublar",
    "anublamos": "anublar",
    "anublan": "anublar",
    "anublando": "anublar",
    "anublara": "anublar",
    "anublarais": "anublar",
    "anublaran": "anublar",
    "anublaras": "anublar",
    "anublare": "anublar",
    "anublareis": "anublar",
    "anublaremos": "anublar",
    "anublaren": "anublar",
    "anublares": "anublar",
    "anublaron": "anublar",
    "anublará": "anublar",
    "anublarán": "anublar",
    "anublarás": "anublar",
    "anublaré": "anublar",
    "anublaréis": "anublar",
    "anublaría": "anublar",
    "anublaríais": "anublar",
    "anublaríamos": "anublar",
    "anublarían": "anublar",
    "anublarías": "anublar",
    "anublas": "anublar",
    "anublase": "anublar",
    "anublaseis": "anublar",
    "anublasen": "anublar",
    "anublases": "anublar",
    "anublaste": "anublar",
    "anublasteis": "anublar",
    "anuble": "anublar",
    "anublemos": "anublar",
    "anublen": "anublar",
    "anubles": "anublar",
    "anublo": "anublar",
    "anublábamos": "anublar",
    "anubláis": "anublar",
    "anubláramos": "anublar",
    "anubláremos": "anublar",
    "anublásemos": "anublar",
    "anublé": "anublar",
    "anubléis": "anublar",
    "anubló": "anublar",
    "anuda": "anudar",
    "anudaba": "anudar",
    "anudabais": "anudar",
    "anudaban": "anudar",
    "anudabas": "anudar",
    "anudad": "anudar",
    "anudada": "anudar",
    "anudadas": "anudar",
    "anudado": "anudar",
    "anudadora": "anudador",
    "anudadoras": "anudador",
    "anudadores": "anudador",
    "anudados": "anudar",
    "anudamientos": "anudamiento",
    "anudamos": "anudar",
    "anudan": "anudar",
    "anudando": "anudar",
    "anudara": "anudar",
    "anudarais": "anudar",
    "anudaran": "anudar",
    "anudaras": "anudar",
    "anudare": "anudar",
    "anudareis": "anudar",
    "anudaremos": "anudar",
    "anudaren": "anudar",
    "anudares": "anudar",
    "anudaron": "anudar",
    "anudará": "anudar",
    "anudarán": "anudar",
    "anudarás": "anudar",
    "anudaré": "anudar",
    "anudaréis": "anudar",
    "anudaría": "anudar",
    "anudaríais": "anudar",
    "anudaríamos": "anudar",
    "anudarían": "anudar",
    "anudarías": "anudar",
    "anudas": "anudar",
    "anudase": "anudar",
    "anudaseis": "anudar",
    "anudasen": "anudar",
    "anudases": "anudar",
    "anudaste": "anudar",
    "anudasteis": "anudar",
    "anude": "anudar",
    "anudemos": "anudar",
    "anuden": "anudar",
    "anudes": "anudar",
    "anudo": "anudar",
    "anudábamos": "anudar",
    "anudáis": "anudar",
    "anudáramos": "anudar",
    "anudáremos": "anudar",
    "anudásemos": "anudar",
    "anudé": "anudar",
    "anudéis": "anudar",
    "anudó": "anudar",
    "anuencias": "anuencia",
    "anuentes": "anuente",
    "anula": "anular",
    "anulaba": "anular",
    "anulabais": "anular",
    "anulaban": "anular",
    "anulabas": "anular",
    "anulabilidades": "anulabilidad",
    "anulaciones": "anulación",
    "anulad": "anular",
    "anulada": "anular",
    "anuladas": "anular",
    "anulado": "anular",
    "anuladora": "anulador",
    "anuladoras": "anulador",
    "anuladores": "anulador",
    "anulados": "anular",
    "anulamos": "anular",
    "anulan": "anular",
    "anulando": "anular",
    "anulara": "anular",
    "anularais": "anular",
    "anularan": "anular",
    "anularas": "anular",
    "anulare": "anular",
    "anulareis": "anular",
    "anularemos": "anular",
    "anularen": "anular",
    "anulares": "anular",
    "anularon": "anular",
    "anulará": "anular",
    "anularán": "anular",
    "anularás": "anular",
    "anularé": "anular",
    "anularéis": "anular",
    "anularía": "anular",
    "anularíais": "anular",
    "anularíamos": "anular",
    "anularían": "anular",
    "anularías": "anular",
    "anulas": "anular",
    "anulase": "anular",
    "anulaseis": "anular",
    "anulasen": "anular",
    "anulases": "anular",
    "anulaste": "anular",
    "anulasteis": "anular",
    "anulativa": "anulativo",
    "anulativas": "anulativo",
    "anulativos": "anulativo",
    "anulatoria": "anulatorio",
    "anulatorias": "anulatorio",
    "anulatorios": "anulatorio",
    "anule": "anular",
    "anulemos": "anular",
    "anulen": "anular",
    "anules": "anular",
    "anulo": "anular",
    "anulábamos": "anular",
    "anuláis": "anular",
    "anuláramos": "anular",
    "anuláremos": "anular",
    "anulásemos": "anular",
    "anulé": "anular",
    "anuléis": "anular",
    "anuló": "anular",
    "anuncia": "anunciar",
    "anunciaba": "anunciar",
    "anunciabais": "anunciar",
    "anunciaban": "anunciar",
    "anunciabas": "anunciar",
    "anunciaciones": "anunciación",
    "anunciad": "anunciar",
    "anunciada": "anunciar",
    "anunciadas": "anunciar",
    "anunciado": "anunciar",
    "anunciadora": "anunciador",
    "anunciadoras": "anunciador",
    "anunciadores": "anunciador",
    "anunciados": "anunciar",
    "anunciamos": "anunciar",
    "anuncian": "anunciar",
    "anunciando": "anunciar",
    "anunciantes": "anunciante",
    "anunciara": "anunciar",
    "anunciarais": "anunciar",
    "anunciaran": "anunciar",
    "anunciaras": "anunciar",
    "anunciare": "anunciar",
    "anunciareis": "anunciar",
    "anunciaremos": "anunciar",
    "anunciaren": "anunciar",
    "anunciares": "anunciar",
    "anunciaron": "anunciar",
    "anunciará": "anunciar",
    "anunciarán": "anunciar",
    "anunciarás": "anunciar",
    "anunciaré": "anunciar",
    "anunciaréis": "anunciar",
    "anunciaría": "anunciar",
    "anunciaríais": "anunciar",
    "anunciaríamos": "anunciar",
    "anunciarían": "anunciar",
    "anunciarías": "anunciar",
    "anuncias": "anunciar",
    "anunciase": "anunciar",
    "anunciaseis": "anunciar",
    "anunciasen": "anunciar",
    "anunciases": "anunciar",
    "anunciaste": "anunciar",
    "anunciasteis": "anunciar",
    "anuncie": "anunciar",
    "anunciemos": "anunciar",
    "anuncien": "anunciar",
    "anuncies": "anunciar",
    "anuncio": "anunciar",
    "anuncios": "anuncio",
    "anunciábamos": "anunciar",
    "anunciáis": "anunciar",
    "anunciáramos": "anunciar",
    "anunciáremos": "anunciar",
    "anunciásemos": "anunciar",
    "anuncié": "anunciar",
    "anunciéis": "anunciar",
    "anunció": "anunciar",
    "anurias": "anuria",
    "anuros": "anuro",
    "anversos": "anverso",
    "anzuelos": "anzuelo",
    "anáforas": "anáfora",
    "anáglifos": "anáglifo",
    "análoga": "análogo",
    "análogas": "análogo",
    "análogos": "análogo",
    "anárquica": "anárquico",
    "anárquicas": "anárquico",
    "anárquicos": "anárquico",
    "anástrofes": "anástrofe",
    "anécdotas": "anécdota",
    "anélidos": "anélido",
    "anémica": "anémico",
    "anémicas": "anémico",
    "anémicos": "anémico",
    "anémonas": "anémona",
    "anímica": "anímico",
    "anímicas": "anímico",
    "anímicos": "anímico",
    "anódica": "anódico",
    "anódicas": "anódico",
    "anódicos": "anódico",
    "anómala": "anómalo",
    "anómalas": "anómalo",
    "anómalos": "anómalo",
    "anómica": "anómico",
    "anómicas": "anómico",
    "anómicos": "anómico",
    "anónima": "anónimo",
    "anónimas": "anónimo",
    "anónimos": "anónimo",
    "anóxica": "anóxico",
    "anóxicas": "anóxico",
    "anóxicos": "anóxico",
    "anúa": "anúo",
    "anúas": "anúo",
    "anúlicos": "anúlico",
    "anúos": "anúo",
    "aoja": "aojar",
    "aojaba": "aojar",
    "aojabais": "aojar",
    "aojaban": "aojar",
    "aojabas": "aojar",
    "aojad": "aojar",
    "aojada": "aojar",
    "aojadas": "aojar",
    "aojado": "aojar",
    "aojadora": "aojador",
    "aojadoras": "aojador",
    "aojadores": "aojador",
    "aojados": "aojar",
    "aojaduras": "aojadura",
    "aojamientos": "aojamiento",
    "aojamos": "aojar",
    "aojan": "aojar",
    "aojando": "aojar",
    "aojara": "aojar",
    "aojarais": "aojar",
    "aojaran": "aojar",
    "aojaras": "aojar",
    "aojare": "aojar",
    "aojareis": "aojar",
    "aojaremos": "aojar",
    "aojaren": "aojar",
    "aojares": "aojar",
    "aojaron": "aojar",
    "aojará": "aojar",
    "aojarán": "aojar",
    "aojarás": "aojar",
    "aojaré": "aojar",
    "aojaréis": "aojar",
    "aojaría": "aojar",
    "aojaríais": "aojar",
    "aojaríamos": "aojar",
    "aojarían": "aojar",
    "aojarías": "aojar",
    "aojas": "aojar",
    "aojase": "aojar",
    "aojaseis": "aojar",
    "aojasen": "aojar",
    "aojases": "aojar",
    "aojaste": "aojar",
    "aojasteis": "aojar",
    "aoje": "aojar",
    "aojemos": "aojar",
    "aojen": "aojar",
    "aojes": "aojar",
    "aojo": "aojar",
    "aojábamos": "aojar",
    "aojáis": "aojar",
    "aojáramos": "aojar",
    "aojáremos": "aojar",
    "aojásemos": "aojar",
    "aojé": "aojar",
    "aojéis": "aojar",
    "aojó": "aojar",
    "aoristos": "aoristo",
    "aortas": "aorta",
    "aorticoabdominales": "aorticoabdominal",
    "aorticorrenales": "aorticorrenal",
    "aorticotorácica": "aorticotorácico",
    "aorticotorácicas": "aorticotorácico",
    "aorticotorácicos": "aorticotorácico",
    "aorística": "aorístico",
    "aorísticas": "aorístico",
    "aorísticos": "aorístico",
    "aova": "aovar",
    "aovaba": "aovar",
    "aovabais": "aovar",
    "aovaban": "aovar",
    "aovabas": "aovar",
    "aovad": "aovar",
    "aovada": "aovar",
    "aovadas": "aovar",
    "aovado": "aovar",
    "aovados": "aovar",
    "aovamos": "aovar",
    "aovan": "aovar",
    "aovando": "aovar",
    "aovara": "aovar",
    "aovarais": "aovar",
    "aovaran": "aovar",
    "aovaras": "aovar",
    "aovare": "aovar",
    "aovareis": "aovar",
    "aovaremos": "aovar",
    "aovaren": "aovar",
    "aovares": "aovar",
    "aovaron": "aovar",
    "aovará": "aovar",
    "aovarán": "aovar",
    "aovarás": "aovar",
    "aovaré": "aovar",
    "aovaréis": "aovar",
    "aovaría": "aovar",
    "aovaríais": "aovar",
    "aovaríamos": "aovar",
    "aovarían": "aovar",
    "aovarías": "aovar",
    "aovas": "aovar",
    "aovase": "aovar",
    "aovaseis": "aovar",
    "aovasen": "aovar",
    "aovases": "aovar",
    "aovaste": "aovar",
    "aovasteis": "aovar",
    "aove": "aovar",
    "aovemos": "aovar",
    "aoven": "aovar",
    "aoves": "aovar",
    "aovilla": "aovillar",
    "aovillaba": "aovillar",
    "aovillabais": "aovillar",
    "aovillaban": "aovillar",
    "aovillabas": "aovillar",
    "aovillad": "aovillar",
    "aovillada": "aovillar",
    "aovilladas": "aovillar",
    "aovillado": "aovillar",
    "aovillados": "aovillar",
    "aovillamos": "aovillar",
    "aovillan": "aovillar",
    "aovillando": "aovillar",
    "aovillara": "aovillar",
    "aovillarais": "aovillar",
    "aovillaran": "aovillar",
    "aovillaras": "aovillar",
    "aovillare": "aovillar",
    "aovillareis": "aovillar",
    "aovillaremos": "aovillar",
    "aovillaren": "aovillar",
    "aovillares": "aovillar",
    "aovillaron": "aovillar",
    "aovillará": "aovillar",
    "aovillarán": "aovillar",
    "aovillarás": "aovillar",
    "aovillaré": "aovillar",
    "aovillaréis": "aovillar",
    "aovillaría": "aovillar",
    "aovillaríais": "aovillar",
    "aovillaríamos": "aovillar",
    "aovillarían": "aovillar",
    "aovillarías": "aovillar",
    "aovillas": "aovillar",
    "aovillase": "aovillar",
    "aovillaseis": "aovillar",
    "aovillasen": "aovillar",
    "aovillases": "aovillar",
    "aovillaste": "aovillar",
    "aovillasteis": "aovillar",
    "aoville": "aovillar",
    "aovillemos": "aovillar",
    "aovillen": "aovillar",
    "aovilles": "aovillar",
    "aovillo": "aovillar",
    "aovillábamos": "aovillar",
    "aovilláis": "aovillar",
    "aovilláramos": "aovillar",
    "aovilláremos": "aovillar",
    "aovillásemos": "aovillar",
    "aovillé": "aovillar",
    "aovilléis": "aovillar",
    "aovilló": "aovillar",
    "aovo": "aovar",
    "aovábamos": "aovar",
    "aováis": "aovar",
    "aováramos": "aovar",
    "aováremos": "aovar",
    "aovásemos": "aovar",
    "aové": "aovar",
    "aovéis": "aovar",
    "aovó": "aovar",
    "apabila": "apabilar",
    "apabilaba": "apabilar",
    "apabilabais": "apabilar",
    "apabilaban": "apabilar",
    "apabilabas": "apabilar",
    "apabilad": "apabilar",
    "apabilada": "apabilar",
    "apabiladas": "apabilar",
    "apabilado": "apabilar",
    "apabilados": "apabilar",
    "apabilamos": "apabilar",
    "apabilan": "apabilar",
    "apabilando": "apabilar",
    "apabilara": "apabilar",
    "apabilarais": "apabilar",
    "apabilaran": "apabilar",
    "apabilaras": "apabilar",
    "apabilare": "apabilar",
    "apabilareis": "apabilar",
    "apabilaremos": "apabilar",
    "apabilaren": "apabilar",
    "apabilares": "apabilar",
    "apabilaron": "apabilar",
    "apabilará": "apabilar",
    "apabilarán": "apabilar",
    "apabilarás": "apabilar",
    "apabilaré": "apabilar",
    "apabilaréis": "apabilar",
    "apabilaría": "apabilar",
    "apabilaríais": "apabilar",
    "apabilaríamos": "apabilar",
    "apabilarían": "apabilar",
    "apabilarías": "apabilar",
    "apabilas": "apabilar",
    "apabilase": "apabilar",
    "apabilaseis": "apabilar",
    "apabilasen": "apabilar",
    "apabilases": "apabilar",
    "apabilaste": "apabilar",
    "apabilasteis": "apabilar",
    "apabile": "apabilar",
    "apabilemos": "apabilar",
    "apabilen": "apabilar",
    "apabiles": "apabilar",
    "apabilo": "apabilar",
    "apabilábamos": "apabilar",
    "apabiláis": "apabilar",
    "apabiláramos": "apabilar",
    "apabiláremos": "apabilar",
    "apabilásemos": "apabilar",
    "apabilé": "apabilar",
    "apabiléis": "apabilar",
    "apabiló": "apabilar",
    "apabulla": "apabullar",
    "apabullaba": "apabullar",
    "apabullabais": "apabullar",
    "apabullaban": "apabullar",
    "apabullabas": "apabullar",
    "apabullad": "apabullar",
    "apabullada": "apabullar",
    "apabulladas": "apabullar",
    "apabullado": "apabullar",
    "apabullados": "apabullar",
    "apabullamientos": "apabullamiento",
    "apabullamos": "apabullar",
    "apabullan": "apabullar",
    "apabullando": "apabullar",
    "apabullantes": "apabullante",
    "apabullara": "apabullar",
    "apabullarais": "apabullar",
    "apabullaran": "apabullar",
    "apabullaras": "apabullar",
    "apabullare": "apabullar",
    "apabullareis": "apabullar",
    "apabullaremos": "apabullar",
    "apabullaren": "apabullar",
    "apabullares": "apabullar",
    "apabullaron": "apabullar",
    "apabullará": "apabullar",
    "apabullarán": "apabullar",
    "apabullarás": "apabullar",
    "apabullaré": "apabullar",
    "apabullaréis": "apabullar",
    "apabullaría": "apabullar",
    "apabullaríais": "apabullar",
    "apabullaríamos": "apabullar",
    "apabullarían": "apabullar",
    "apabullarías": "apabullar",
    "apabullas": "apabullar",
    "apabullase": "apabullar",
    "apabullaseis": "apabullar",
    "apabullasen": "apabullar",
    "apabullases": "apabullar",
    "apabullaste": "apabullar",
    "apabullasteis": "apabullar",
    "apabulle": "apabullar",
    "apabullemos": "apabullar",
    "apabullen": "apabullar",
    "apabulles": "apabullar",
    "apabullo": "apabullar",
    "apabullos": "apabullo",
    "apabullábamos": "apabullar",
    "apabulláis": "apabullar",
    "apabulláramos": "apabullar",
    "apabulláremos": "apabullar",
    "apabullásemos": "apabullar",
    "apabullé": "apabullar",
    "apabulléis": "apabullar",
    "apabulló": "apabullar",
    "apacentaba": "apacentar",
    "apacentabais": "apacentar",
    "apacentaban": "apacentar",
    "apacentabas": "apacentar",
    "apacentad": "apacentar",
    "apacentada": "apacentar",
    "apacentadas": "apacentar",
    "apacentaderos": "apacentadero",
    "apacentado": "apacentar",
    "apacentadora": "apacentador",
    "apacentadoras": "apacentador",
    "apacentadores": "apacentador",
    "apacentados": "apacentar",
    "apacentamientos": "apacentamiento",
    "apacentamos": "apacentar",
    "apacentando": "apacentar",
    "apacentara": "apacentar",
    "apacentarais": "apacentar",
    "apacentaran": "apacentar",
    "apacentaras": "apacentar",
    "apacentare": "apacentar",
    "apacentareis": "apacentar",
    "apacentaremos": "apacentar",
    "apacentaren": "apacentar",
    "apacentares": "apacentar",
    "apacentaron": "apacentar",
    "apacentará": "apacentar",
    "apacentarán": "apacentar",
    "apacentarás": "apacentar",
    "apacentaré": "apacentar",
    "apacentaréis": "apacentar",
    "apacentaría": "apacentar",
    "apacentaríais": "apacentar",
    "apacentaríamos": "apacentar",
    "apacentarían": "apacentar",
    "apacentarías": "apacentar",
    "apacentase": "apacentar",
    "apacentaseis": "apacentar",
    "apacentasen": "apacentar",
    "apacentases": "apacentar",
    "apacentaste": "apacentar",
    "apacentasteis": "apacentar",
    "apacentemos": "apacentar",
    "apacentábamos": "apacentar",
    "apacentáis": "apacentar",
    "apacentáramos": "apacentar",
    "apacentáremos": "apacentar",
    "apacentásemos": "apacentar",
    "apacenté": "apacentar",
    "apacentéis": "apacentar",
    "apacentó": "apacentar",
    "apaches": "apache",
    "apachurra": "apachurrar",
    "apachurraba": "apachurrar",
    "apachurrabais": "apachurrar",
    "apachurraban": "apachurrar",
    "apachurrabas": "apachurrar",
    "apachurrad": "apachurrar",
    "apachurrada": "apachurrar",
    "apachurradas": "apachurrar",
    "apachurrado": "apachurrar",
    "apachurrados": "apachurrar",
    "apachurramos": "apachurrar",
    "apachurran": "apachurrar",
    "apachurrando": "apachurrar",
    "apachurrara": "apachurrar",
    "apachurrarais": "apachurrar",
    "apachurraran": "apachurrar",
    "apachurraras": "apachurrar",
    "apachurrare": "apachurrar",
    "apachurrareis": "apachurrar",
    "apachurraremos": "apachurrar",
    "apachurraren": "apachurrar",
    "apachurrares": "apachurrar",
    "apachurraron": "apachurrar",
    "apachurrará": "apachurrar",
    "apachurrarán": "apachurrar",
    "apachurrarás": "apachurrar",
    "apachurraré": "apachurrar",
    "apachurraréis": "apachurrar",
    "apachurraría": "apachurrar",
    "apachurraríais": "apachurrar",
    "apachurraríamos": "apachurrar",
    "apachurrarían": "apachurrar",
    "apachurrarías": "apachurrar",
    "apachurras": "apachurrar",
    "apachurrase": "apachurrar",
    "apachurraseis": "apachurrar",
    "apachurrasen": "apachurrar",
    "apachurrases": "apachurrar",
    "apachurraste": "apachurrar",
    "apachurrasteis": "apachurrar",
    "apachurre": "apachurrar",
    "apachurremos": "apachurrar",
    "apachurren": "apachurrar",
    "apachurres": "apachurrar",
    "apachurro": "apachurrar",
    "apachurrábamos": "apachurrar",
    "apachurráis": "apachurrar",
    "apachurráramos": "apachurrar",
    "apachurráremos": "apachurrar",
    "apachurrásemos": "apachurrar",
    "apachurré": "apachurrar",
    "apachurréis": "apachurrar",
    "apachurró": "apachurrar",
    "apacibilidades": "apacibilidad",
    "apacibles": "apacible",
    "apacienta": "apacentar",
    "apacientan": "apacentar",
    "apacientas": "apacentar",
    "apaciente": "apacentar",
    "apacienten": "apacentar",
    "apacientes": "apacentar",
    "apaciento": "apacentar",
    "apacigua": "apaciguar",
    "apaciguaba": "apaciguar",
    "apaciguabais": "apaciguar",
    "apaciguaban": "apaciguar",
    "apaciguabas": "apaciguar",
    "apaciguad": "apaciguar",
    "apaciguada": "apaciguar",
    "apaciguadas": "apaciguar",
    "apaciguado": "apaciguar",
    "apaciguadora": "apaciguador",
    "apaciguadoras": "apaciguador",
    "apaciguadores": "apaciguador",
    "apaciguados": "apaciguar",
    "apaciguamientos": "apaciguamiento",
    "apaciguamos": "apaciguar",
    "apaciguan": "apaciguar",
    "apaciguando": "apaciguar",
    "apaciguantes": "apaciguante",
    "apaciguara": "apaciguar",
    "apaciguarais": "apaciguar",
    "apaciguaran": "apaciguar",
    "apaciguaras": "apaciguar",
    "apaciguare": "apaciguar",
    "apaciguareis": "apaciguar",
    "apaciguaremos": "apaciguar",
    "apaciguaren": "apaciguar",
    "apaciguares": "apaciguar",
    "apaciguaron": "apaciguar",
    "apaciguará": "apaciguar",
    "apaciguarán": "apaciguar",
    "apaciguarás": "apaciguar",
    "apaciguaré": "apaciguar",
    "apaciguaréis": "apaciguar",
    "apaciguaría": "apaciguar",
    "apaciguaríais": "apaciguar",
    "apaciguaríamos": "apaciguar",
    "apaciguarían": "apaciguar",
    "apaciguarías": "apaciguar",
    "apaciguas": "apaciguar",
    "apaciguase": "apaciguar",
    "apaciguaseis": "apaciguar",
    "apaciguasen": "apaciguar",
    "apaciguases": "apaciguar",
    "apaciguaste": "apaciguar",
    "apaciguasteis": "apaciguar",
    "apaciguo": "apaciguar",
    "apaciguábamos": "apaciguar",
    "apaciguáis": "apaciguar",
    "apaciguáramos": "apaciguar",
    "apaciguáremos": "apaciguar",
    "apaciguásemos": "apaciguar",
    "apaciguó": "apaciguar",
    "apacigüe": "apaciguar",
    "apacigüemos": "apaciguar",
    "apacigüen": "apaciguar",
    "apacigües": "apaciguar",
    "apacigüé": "apaciguar",
    "apacigüéis": "apaciguar",
    "apadanas": "apadana",
    "apadrina": "apadrinar",
    "apadrinaba": "apadrinar",
    "apadrinabais": "apadrinar",
    "apadrinaban": "apadrinar",
    "apadrinabas": "apadrinar",
    "apadrinad": "apadrinar",
    "apadrinada": "apadrinar",
    "apadrinadas": "apadrinar",
    "apadrinado": "apadrinar",
    "apadrinados": "apadrinar",
    "apadrinamientos": "apadrinamiento",
    "apadrinamos": "apadrinar",
    "apadrinan": "apadrinar",
    "apadrinando": "apadrinar",
    "apadrinara": "apadrinar",
    "apadrinarais": "apadrinar",
    "apadrinaran": "apadrinar",
    "apadrinaras": "apadrinar",
    "apadrinare": "apadrinar",
    "apadrinareis": "apadrinar",
    "apadrinaremos": "apadrinar",
    "apadrinaren": "apadrinar",
    "apadrinares": "apadrinar",
    "apadrinaron": "apadrinar",
    "apadrinará": "apadrinar",
    "apadrinarán": "apadrinar",
    "apadrinarás": "apadrinar",
    "apadrinaré": "apadrinar",
    "apadrinaréis": "apadrinar",
    "apadrinaría": "apadrinar",
    "apadrinaríais": "apadrinar",
    "apadrinaríamos": "apadrinar",
    "apadrinarían": "apadrinar",
    "apadrinarías": "apadrinar",
    "apadrinas": "apadrinar",
    "apadrinase": "apadrinar",
    "apadrinaseis": "apadrinar",
    "apadrinasen": "apadrinar",
    "apadrinases": "apadrinar",
    "apadrinaste": "apadrinar",
    "apadrinasteis": "apadrinar",
    "apadrine": "apadrinar",
    "apadrinemos": "apadrinar",
    "apadrinen": "apadrinar",
    "apadrines": "apadrinar",
    "apadrino": "apadrinar",
    "apadrinábamos": "apadrinar",
    "apadrináis": "apadrinar",
    "apadrináramos": "apadrinar",
    "apadrináremos": "apadrinar",
    "apadrinásemos": "apadrinar",
    "apadriné": "apadrinar",
    "apadrinéis": "apadrinar",
    "apadrinó": "apadrinar",
    "apaga": "apagar",
    "apagaba": "apagar",
    "apagabais": "apagar",
    "apagaban": "apagar",
    "apagabas": "apagar",
    "apagables": "apagable",
    "apagad": "apagar",
    "apagada": "apagar",
    "apagadas": "apagar",
    "apagadiza": "apagadizo",
    "apagadizas": "apagadizo",
    "apagadizos": "apagadizo",
    "apagado": "apagar",
    "apagadora": "apagador",
    "apagadoras": "apagador",
    "apagadores": "apagador",
    "apagados": "apagar",
    "apagamientos": "apagamiento",
    "apagamos": "apagar",
    "apagan": "apagar",
    "apagando": "apagar",
    "apagapenoles": "apagapenol",
    "apagara": "apagar",
    "apagarais": "apagar",
    "apagaran": "apagar",
    "apagaras": "apagar",
    "apagare": "apagar",
    "apagareis": "apagar",
    "apagaremos": "apagar",
    "apagaren": "apagar",
    "apagares": "apagar",
    "apagaron": "apagar",
    "apagará": "apagar",
    "apagarán": "apagar",
    "apagarás": "apagar",
    "apagaré": "apagar",
    "apagaréis": "apagar",
    "apagaría": "apagar",
    "apagaríais": "apagar",
    "apagaríamos": "apagar",
    "apagarían": "apagar",
    "apagarías": "apagar",
    "apagas": "apagar",
    "apagase": "apagar",
    "apagaseis": "apagar",
    "apagasen": "apagar",
    "apagases": "apagar",
    "apagaste": "apagar",
    "apagasteis": "apagar",
    "apago": "apagar",
    "apagones": "apagón",
    "apague": "apagar",
    "apaguemos": "apagar",
    "apaguen": "apagar",
    "apagues": "apagar",
    "apagué": "apagar",
    "apaguéis": "apagar",
    "apagábamos": "apagar",
    "apagáis": "apagar",
    "apagáramos": "apagar",
    "apagáremos": "apagar",
    "apagásemos": "apagar",
    "apagó": "apagar",
    "apainelada": "apainelado",
    "apaineladas": "apainelado",
    "apainelados": "apainelado",
    "apaisada": "apaisado",
    "apaisadas": "apaisado",
    "apaisados": "apaisado",
    "apalabra": "apalabrar",
    "apalabraba": "apalabrar",
    "apalabrabais": "apalabrar",
    "apalabraban": "apalabrar",
    "apalabrabas": "apalabrar",
    "apalabrad": "apalabrar",
    "apalabrada": "apalabrar",
    "apalabradas": "apalabrar",
    "apalabrado": "apalabrar",
    "apalabrados": "apalabrar",
    "apalabramos": "apalabrar",
    "apalabran": "apalabrar",
    "apalabrando": "apalabrar",
    "apalabrara": "apalabrar",
    "apalabrarais": "apalabrar",
    "apalabraran": "apalabrar",
    "apalabraras": "apalabrar",
    "apalabrare": "apalabrar",
    "apalabrareis": "apalabrar",
    "apalabraremos": "apalabrar",
    "apalabraren": "apalabrar",
    "apalabrares": "apalabrar",
    "apalabraron": "apalabrar",
    "apalabrará": "apalabrar",
    "apalabrarán": "apalabrar",
    "apalabrarás": "apalabrar",
    "apalabraré": "apalabrar",
    "apalabraréis": "apalabrar",
    "apalabraría": "apalabrar",
    "apalabraríais": "apalabrar",
    "apalabraríamos": "apalabrar",
    "apalabrarían": "apalabrar",
    "apalabrarías": "apalabrar",
    "apalabras": "apalabrar",
    "apalabrase": "apalabrar",
    "apalabraseis": "apalabrar",
    "apalabrasen": "apalabrar",
    "apalabrases": "apalabrar",
    "apalabraste": "apalabrar",
    "apalabrasteis": "apalabrar",
    "apalabre": "apalabrar",
    "apalabremos": "apalabrar",
    "apalabren": "apalabrar",
    "apalabres": "apalabrar",
    "apalabro": "apalabrar",
    "apalabrábamos": "apalabrar",
    "apalabráis": "apalabrar",
    "apalabráramos": "apalabrar",
    "apalabráremos": "apalabrar",
    "apalabrásemos": "apalabrar",
    "apalabré": "apalabrar",
    "apalabréis": "apalabrar",
    "apalabró": "apalabrar",
    "apalachienses": "apalachiense",
    "apalanca": "apalancar",
    "apalancaba": "apalancar",
    "apalancabais": "apalancar",
    "apalancaban": "apalancar",
    "apalancabas": "apalancar",
    "apalancad": "apalancar",
    "apalancada": "apalancar",
    "apalancadas": "apalancar",
    "apalancado": "apalancar",
    "apalancados": "apalancar",
    "apalancamientos": "apalancamiento",
    "apalancamos": "apalancar",
    "apalancan": "apalancar",
    "apalancando": "apalancar",
    "apalancara": "apalancar",
    "apalancarais": "apalancar",
    "apalancaran": "apalancar",
    "apalancaras": "apalancar",
    "apalancare": "apalancar",
    "apalancareis": "apalancar",
    "apalancaremos": "apalancar",
    "apalancaren": "apalancar",
    "apalancares": "apalancar",
    "apalancaron": "apalancar",
    "apalancará": "apalancar",
    "apalancarán": "apalancar",
    "apalancarás": "apalancar",
    "apalancaré": "apalancar",
    "apalancaréis": "apalancar",
    "apalancaría": "apalancar",
    "apalancaríais": "apalancar",
    "apalancaríamos": "apalancar",
    "apalancarían": "apalancar",
    "apalancarías": "apalancar",
    "apalancas": "apalancar",
    "apalancase": "apalancar",
    "apalancaseis": "apalancar",
    "apalancasen": "apalancar",
    "apalancases": "apalancar",
    "apalancaste": "apalancar",
    "apalancasteis": "apalancar",
    "apalanco": "apalancar",
    "apalancábamos": "apalancar",
    "apalancáis": "apalancar",
    "apalancáramos": "apalancar",
    "apalancáremos": "apalancar",
    "apalancásemos": "apalancar",
    "apalancó": "apalancar",
    "apalanque": "apalancar",
    "apalanquemos": "apalancar",
    "apalanquen": "apalancar",
    "apalanques": "apalancar",
    "apalanqué": "apalancar",
    "apalanquéis": "apalancar",
    "apalea": "apalear",
    "apaleaba": "apalear",
    "apaleabais": "apalear",
    "apaleaban": "apalear",
    "apaleabas": "apalear",
    "apalead": "apalear",
    "apaleada": "apalear",
    "apaleadas": "apalear",
    "apaleado": "apalear",
    "apaleados": "apalear",
    "apaleamientos": "apaleamiento",
    "apaleamos": "apalear",
    "apalean": "apalear",
    "apaleando": "apalear",
    "apaleara": "apalear",
    "apalearais": "apalear",
    "apalearan": "apalear",
    "apalearas": "apalear",
    "apaleare": "apalear",
    "apaleareis": "apalear",
    "apalearemos": "apalear",
    "apalearen": "apalear",
    "apaleares": "apalear",
    "apalearon": "apalear",
    "apaleará": "apalear",
    "apalearán": "apalear",
    "apalearás": "apalear",
    "apalearé": "apalear",
    "apalearéis": "apalear",
    "apalearía": "apalear",
    "apalearíais": "apalear",
    "apalearíamos": "apalear",
    "apalearían": "apalear",
    "apalearías": "apalear",
    "apaleas": "apalear",
    "apalease": "apalear",
    "apaleaseis": "apalear",
    "apaleasen": "apalear",
    "apaleases": "apalear",
    "apaleaste": "apalear",
    "apaleasteis": "apalear",
    "apalee": "apalear",
    "apaleemos": "apalear",
    "apaleen": "apalear",
    "apalees": "apalear",
    "apaleo": "apalear",
    "apaleos": "apaleo",
    "apaleábamos": "apalear",
    "apaleáis": "apalear",
    "apaleáramos": "apalear",
    "apaleáremos": "apalear",
    "apaleásemos": "apalear",
    "apaleé": "apalear",
    "apaleéis": "apalear",
    "apaleó": "apalear",
    "apana": "apanar",
    "apanaba": "apanar",
    "apanabais": "apanar",
    "apanaban": "apanar",
    "apanabas": "apanar",
    "apanad": "apanar",
    "apanada": "apanar",
    "apanadas": "apanar",
    "apanado": "apanar",
    "apanados": "apanar",
    "apanalada": "apanalado",
    "apanaladas": "apanalado",
    "apanalados": "apanalado",
    "apanamos": "apanar",
    "apanan": "apanar",
    "apanando": "apanar",
    "apanara": "apanar",
    "apanarais": "apanar",
    "apanaran": "apanar",
    "apanaras": "apanar",
    "apanare": "apanar",
    "apanareis": "apanar",
    "apanaremos": "apanar",
    "apanaren": "apanar",
    "apanares": "apanar",
    "apanaron": "apanar",
    "apanará": "apanar",
    "apanarán": "apanar",
    "apanarás": "apanar",
    "apanaré": "apanar",
    "apanaréis": "apanar",
    "apanaría": "apanar",
    "apanaríais": "apanar",
    "apanaríamos": "apanar",
    "apanarían": "apanar",
    "apanarías": "apanar",
    "apanas": "apanar",
    "apanase": "apanar",
    "apanaseis": "apanar",
    "apanasen": "apanar",
    "apanases": "apanar",
    "apanaste": "apanar",
    "apanasteis": "apanar",
    "apanda": "apandar",
    "apandaba": "apandar",
    "apandabais": "apandar",
    "apandaban": "apandar",
    "apandabas": "apandar",
    "apandad": "apandar",
    "apandada": "apandar",
    "apandadas": "apandar",
    "apandado": "apandar",
    "apandados": "apandar",
    "apandamos": "apandar",
    "apandan": "apandar",
    "apandando": "apandar",
    "apandara": "apandar",
    "apandarais": "apandar",
    "apandaran": "apandar",
    "apandaras": "apandar",
    "apandare": "apandar",
    "apandareis": "apandar",
    "apandaremos": "apandar",
    "apandaren": "apandar",
    "apandares": "apandar",
    "apandaron": "apandar",
    "apandará": "apandar",
    "apandarán": "apandar",
    "apandarás": "apandar",
    "apandaré": "apandar",
    "apandaréis": "apandar",
    "apandaría": "apandar",
    "apandaríais": "apandar",
    "apandaríamos": "apandar",
    "apandarían": "apandar",
    "apandarías": "apandar",
    "apandas": "apandar",
    "apandase": "apandar",
    "apandaseis": "apandar",
    "apandasen": "apandar",
    "apandases": "apandar",
    "apandaste": "apandar",
    "apandasteis": "apandar",
    "apande": "apandar",
    "apandemos": "apandar",
    "apanden": "apandar",
    "apandes": "apandar",
    "apandilla": "apandillar",
    "apandillaba": "apandillar",
    "apandillabais": "apandillar",
    "apandillaban": "apandillar",
    "apandillabas": "apandillar",
    "apandillad": "apandillar",
    "apandillada": "apandillar",
    "apandilladas": "apandillar",
    "apandillado": "apandillar",
    "apandillados": "apandillar",
    "apandillamientos": "apandillamiento",
    "apandillamos": "apandillar",
    "apandillan": "apandillar",
    "apandillando": "apandillar",
    "apandillara": "apandillar",
    "apandillarais": "apandillar",
    "apandillaran": "apandillar",
    "apandillaras": "apandillar",
    "apandillare": "apandillar",
    "apandillareis": "apandillar",
    "apandillaremos": "apandillar",
    "apandillaren": "apandillar",
    "apandillares": "apandillar",
    "apandillaron": "apandillar",
    "apandillará": "apandillar",
    "apandillarán": "apandillar",
    "apandillarás": "apandillar",
    "apandillaré": "apandillar",
    "apandillaréis": "apandillar",
    "apandillaría": "apandillar",
    "apandillaríais": "apandillar",
    "apandillaríamos": "apandillar",
    "apandillarían": "apandillar",
    "apandillarías": "apandillar",
    "apandillas": "apandillar",
    "apandillase": "apandillar",
    "apandillaseis": "apandillar",
    "apandillasen": "apandillar",
    "apandillases": "apandillar",
    "apandillaste": "apandillar",
    "apandillasteis": "apandillar",
    "apandille": "apandillar",
    "apandillemos": "apandillar",
    "apandillen": "apandillar",
    "apandilles": "apandillar",
    "apandillo": "apandillar",
    "apandillábamos": "apandillar",
    "apandilláis": "apandillar",
    "apandilláramos": "apandillar",
    "apandilláremos": "apandillar",
    "apandillásemos": "apandillar",
    "apandillé": "apandillar",
    "apandilléis": "apandillar",
    "apandilló": "apandillar",
    "apando": "apandar",
    "apandonga": "apandongar",
    "apandongaba": "apandongar",
    "apandongabais": "apandongar",
    "apandongaban": "apandongar",
    "apandongabas": "apandongar",
    "apandongad": "apandongar",
    "apandongada": "apandongar",
    "apandongadas": "apandongar",
    "apandongado": "apandongar",
    "apandongados": "apandongar",
    "apandongamos": "apandongar",
    "apandongan": "apandongar",
    "apandongando": "apandongar",
    "apandongara": "apandongar",
    "apandongarais": "apandongar",
    "apandongaran": "apandongar",
    "apandongaras": "apandongar",
    "apandongare": "apandongar",
    "apandongareis": "apandongar",
    "apandongaremos": "apandongar",
    "apandongaren": "apandongar",
    "apandongares": "apandongar",
    "apandongaron": "apandongar",
    "apandongará": "apandongar",
    "apandongarán": "apandongar",
    "apandongarás": "apandongar",
    "apandongaré": "apandongar",
    "apandongaréis": "apandongar",
    "apandongaría": "apandongar",
    "apandongaríais": "apandongar",
    "apandongaríamos": "apandongar",
    "apandongarían": "apandongar",
    "apandongarías": "apandongar",
    "apandongas": "apandongar",
    "apandongase": "apandongar",
    "apandongaseis": "apandongar",
    "apandongasen": "apandongar",
    "apandongases": "apandongar",
    "apandongaste": "apandongar",
    "apandongasteis": "apandongar",
    "apandongo": "apandongar",
    "apandongue": "apandongar",
    "apandonguemos": "apandongar",
    "apandonguen": "apandongar",
    "apandongues": "apandongar",
    "apandongué": "apandongar",
    "apandonguéis": "apandongar",
    "apandongábamos": "apandongar",
    "apandongáis": "apandongar",
    "apandongáramos": "apandongar",
    "apandongáremos": "apandongar",
    "apandongásemos": "apandongar",
    "apandongó": "apandongar",
    "apandábamos": "apandar",
    "apandáis": "apandar",
    "apandáramos": "apandar",
    "apandáremos": "apandar",
    "apandásemos": "apandar",
    "apandé": "apandar",
    "apandéis": "apandar",
    "apandó": "apandar",
    "apane": "apanar",
    "apanemos": "apanar",
    "apanen": "apanar",
    "apanes": "apanar",
    "apano": "apanar",
    "apantalla": "apantallar",
    "apantallaba": "apantallar",
    "apantallabais": "apantallar",
    "apantallaban": "apantallar",
    "apantallabas": "apantallar",
    "apantallad": "apantallar",
    "apantallada": "apantallar",
    "apantalladas": "apantallar",
    "apantallado": "apantallar",
    "apantallados": "apantallar",
    "apantallamientos": "apantallamiento",
    "apantallamos": "apantallar",
    "apantallan": "apantallar",
    "apantallando": "apantallar",
    "apantallara": "apantallar",
    "apantallarais": "apantallar",
    "apantallaran": "apantallar",
    "apantallaras": "apantallar",
    "apantallare": "apantallar",
    "apantallareis": "apantallar",
    "apantallaremos": "apantallar",
    "apantallaren": "apantallar",
    "apantallares": "apantallar",
    "apantallaron": "apantallar",
    "apantallará": "apantallar",
    "apantallarán": "apantallar",
    "apantallarás": "apantallar",
    "apantallaré": "apantallar",
    "apantallaréis": "apantallar",
    "apantallaría": "apantallar",
    "apantallaríais": "apantallar",
    "apantallaríamos": "apantallar",
    "apantallarían": "apantallar",
    "apantallarías": "apantallar",
    "apantallas": "apantallar",
    "apantallase": "apantallar",
    "apantallaseis": "apantallar",
    "apantallasen": "apantallar",
    "apantallases": "apantallar",
    "apantallaste": "apantallar",
    "apantallasteis": "apantallar",
    "apantalle": "apantallar",
    "apantallemos": "apantallar",
    "apantallen": "apantallar",
    "apantalles": "apantallar",
    "apantallo": "apantallar",
    "apantallábamos": "apantallar",
    "apantalláis": "apantallar",
    "apantalláramos": "apantallar",
    "apantalláremos": "apantallar",
    "apantallásemos": "apantallar",
    "apantallé": "apantallar",
    "apantalléis": "apantallar",
    "apantalló": "apantallar",
    "apantana": "apantanar",
    "apantanaba": "apantanar",
    "apantanabais": "apantanar",
    "apantanaban": "apantanar",
    "apantanabas": "apantanar",
    "apantanad": "apantanar",
    "apantanada": "apantanar",
    "apantanadas": "apantanar",
    "apantanado": "apantanar",
    "apantanados": "apantanar",
    "apantanamientos": "apantanamiento",
    "apantanamos": "apantanar",
    "apantanan": "apantanar",
    "apantanando": "apantanar",
    "apantanara": "apantanar",
    "apantanarais": "apantanar",
    "apantanaran": "apantanar",
    "apantanaras": "apantanar",
    "apantanare": "apantanar",
    "apantanareis": "apantanar",
    "apantanaremos": "apantanar",
    "apantanaren": "apantanar",
    "apantanares": "apantanar",
    "apantanaron": "apantanar",
    "apantanará": "apantanar",
    "apantanarán": "apantanar",
    "apantanarás": "apantanar",
    "apantanaré": "apantanar",
    "apantanaréis": "apantanar",
    "apantanaría": "apantanar",
    "apantanaríais": "apantanar",
    "apantanaríamos": "apantanar",
    "apantanarían": "apantanar",
    "apantanarías": "apantanar",
    "apantanas": "apantanar",
    "apantanase": "apantanar",
    "apantanaseis": "apantanar",
    "apantanasen": "apantanar",
    "apantanases": "apantanar",
    "apantanaste": "apantanar",
    "apantanasteis": "apantanar",
    "apantane": "apantanar",
    "apantanemos": "apantanar",
    "apantanen": "apantanar",
    "apantanes": "apantanar",
    "apantano": "apantanar",
    "apantanábamos": "apantanar",
    "apantanáis": "apantanar",
    "apantanáramos": "apantanar",
    "apantanáremos": "apantanar",
    "apantanásemos": "apantanar",
    "apantané": "apantanar",
    "apantanéis": "apantanar",
    "apantanó": "apantanar",
    "apanábamos": "apanar",
    "apanáis": "apanar",
    "apanáramos": "apanar",
    "apanáremos": "apanar",
    "apanásemos": "apanar",
    "apané": "apanar",
    "apanéis": "apanar",
    "apanó": "apanar",
    "apara": "aparar",
    "aparaba": "aparar",
    "aparabais": "aparar",
    "aparaban": "aparar",
    "aparabas": "aparar",
    "aparad": "aparar",
    "aparada": "aparar",
    "aparadas": "aparar",
    "aparado": "aparar",
    "aparadores": "aparador",
    "aparados": "aparar",
    "aparamentas": "aparamenta",
    "aparamos": "aparar",
    "aparan": "aparar",
    "aparando": "aparar",
    "aparara": "aparar",
    "apararais": "aparar",
    "apararan": "aparar",
    "apararas": "aparar",
    "aparare": "aparar",
    "aparareis": "aparar",
    "apararemos": "aparar",
    "apararen": "aparar",
    "aparares": "aparar",
    "apararon": "aparar",
    "aparará": "aparar",
    "apararán": "aparar",
    "apararás": "aparar",
    "apararé": "aparar",
    "apararéis": "aparar",
    "apararía": "aparar",
    "apararíais": "aparar",
    "apararíamos": "aparar",
    "apararían": "aparar",
    "apararías": "aparar",
    "aparas": "aparar",
    "aparase": "aparar",
    "aparaseis": "aparar",
    "aparasen": "aparar",
    "aparases": "aparar",
    "aparaste": "aparar",
    "aparasteis": "aparar",
    "aparatologías": "aparatología",
    "aparatos": "aparato",
    "aparatosa": "aparatoso",
    "aparatosas": "aparatoso",
    "aparatosidades": "aparatosidad",
    "aparatosos": "aparatoso",
    "aparca": "aparcar",
    "aparcaba": "aparcar",
    "aparcabais": "aparcar",
    "aparcaban": "aparcar",
    "aparcabas": "aparcar",
    "aparcad": "aparcar",
    "aparcada": "aparcar",
    "aparcadas": "aparcar",
    "aparcado": "aparcar",
    "aparcados": "aparcar",
    "aparcamientos": "aparcamiento",
    "aparcamos": "aparcar",
    "aparcan": "aparcar",
    "aparcando": "aparcar",
    "aparcara": "aparcar",
    "aparcarais": "aparcar",
    "aparcaran": "aparcar",
    "aparcaras": "aparcar",
    "aparcare": "aparcar",
    "aparcareis": "aparcar",
    "aparcaremos": "aparcar",
    "aparcaren": "aparcar",
    "aparcares": "aparcar",
    "aparcaron": "aparcar",
    "aparcará": "aparcar",
    "aparcarán": "aparcar",
    "aparcarás": "aparcar",
    "aparcaré": "aparcar",
    "aparcaréis": "aparcar",
    "aparcaría": "aparcar",
    "aparcaríais": "aparcar",
    "aparcaríamos": "aparcar",
    "aparcarían": "aparcar",
    "aparcarías": "aparcar",
    "aparcas": "aparcar",
    "aparcase": "aparcar",
    "aparcaseis": "aparcar",
    "aparcasen": "aparcar",
    "aparcases": "aparcar",
    "aparcaste": "aparcar",
    "aparcasteis": "aparcar",
    "aparcera": "aparcero",
    "aparceras": "aparcero",
    "aparceros": "aparcero",
    "aparcerías": "aparcería",
    "aparco": "aparcar",
    "aparcábamos": "aparcar",
    "aparcáis": "aparcar",
    "aparcáramos": "aparcar",
    "aparcáremos": "aparcar",
    "aparcásemos": "aparcar",
    "aparcó": "aparcar",
    "apare": "aparar",
    "aparea": "aparear",
    "apareaba": "aparear",
    "apareabais": "aparear",
    "apareaban": "aparear",
    "apareabas": "aparear",
    "aparead": "aparear",
    "apareada": "aparear",
    "apareadas": "aparear",
    "apareado": "aparear",
    "apareados": "aparear",
    "apareamientos": "apareamiento",
    "apareamos": "aparear",
    "aparean": "aparear",
    "apareando": "aparear",
    "apareara": "aparear",
    "aparearais": "aparear",
    "aparearan": "aparear",
    "aparearas": "aparear",
    "apareare": "aparear",
    "apareareis": "aparear",
    "aparearemos": "aparear",
    "aparearen": "aparear",
    "apareares": "aparear",
    "aparearon": "aparear",
    "apareará": "aparear",
    "aparearán": "aparear",
    "aparearás": "aparear",
    "aparearé": "aparear",
    "aparearéis": "aparear",
    "aparearía": "aparear",
    "aparearíais": "aparear",
    "aparearíamos": "aparear",
    "aparearían": "aparear",
    "aparearías": "aparear",
    "apareas": "aparear",
    "aparease": "aparear",
    "apareaseis": "aparear",
    "apareasen": "aparear",
    "apareases": "aparear",
    "apareaste": "aparear",
    "apareasteis": "aparear",
    "aparece": "aparecer",
    "apareced": "aparecer",
    "aparecemos": "aparecer",
    "aparecen": "aparecer",
    "apareceremos": "aparecer",
    "aparecerá": "aparecer",
    "aparecerán": "aparecer",
    "aparecerás": "aparecer",
    "apareceré": "aparecer",
    "apareceréis": "aparecer",
    "aparecería": "aparecer",
    "apareceríais": "aparecer",
    "apareceríamos": "aparecer",
    "aparecerían": "aparecer",
    "aparecerías": "aparecer",
    "apareces": "aparecer",
    "aparecida": "aparecer",
    "aparecidas": "aparecer",
    "aparecido": "aparecer",
    "aparecidos": "aparecido",
    "apareciendo": "aparecer",
    "apareciera": "aparecer",
    "aparecierais": "aparecer",
    "aparecieran": "aparecer",
    "aparecieras": "aparecer",
    "apareciere": "aparecer",
    "apareciereis": "aparecer",
    "aparecieren": "aparecer",
    "aparecieres": "aparecer",
    "aparecieron": "aparecer",
    "apareciese": "aparecer",
    "aparecieseis": "aparecer",
    "apareciesen": "aparecer",
    "aparecieses": "aparecer",
    "aparecimos": "aparecer",
    "apareciste": "aparecer",
    "aparecisteis": "aparecer",
    "apareciéramos": "aparecer",
    "apareciéremos": "aparecer",
    "apareciésemos": "aparecer",
    "apareció": "aparecer",
    "aparecéis": "aparecer",
    "aparecí": "aparecer",
    "aparecía": "aparecer",
    "aparecíais": "aparecer",
    "aparecíamos": "aparecer",
    "aparecían": "aparecer",
    "aparecías": "aparecer",
    "aparee": "aparear",
    "apareemos": "aparear",
    "apareen": "aparear",
    "aparees": "aparear",
    "apareja": "aparejar",
    "aparejaba": "aparejar",
    "aparejabais": "aparejar",
    "aparejaban": "aparejar",
    "aparejabas": "aparejar",
    "aparejad": "aparejar",
    "aparejada": "aparejar",
    "aparejadas": "aparejar",
    "aparejado": "aparejar",
    "aparejadora": "aparejador",
    "aparejadoras": "aparejador",
    "aparejadores": "aparejador",
    "aparejados": "aparejar",
    "aparejamientos": "aparejamiento",
    "aparejamos": "aparejar",
    "aparejan": "aparejar",
    "aparejando": "aparejar",
    "aparejara": "aparejar",
    "aparejarais": "aparejar",
    "aparejaran": "aparejar",
    "aparejaras": "aparejar",
    "aparejare": "aparejar",
    "aparejareis": "aparejar",
    "aparejaremos": "aparejar",
    "aparejaren": "aparejar",
    "aparejares": "aparejar",
    "aparejaron": "aparejar",
    "aparejará": "aparejar",
    "aparejarán": "aparejar",
    "aparejarás": "aparejar",
    "aparejaré": "aparejar",
    "aparejaréis": "aparejar",
    "aparejaría": "aparejar",
    "aparejaríais": "aparejar",
    "aparejaríamos": "aparejar",
    "aparejarían": "aparejar",
    "aparejarías": "aparejar",
    "aparejas": "aparejar",
    "aparejase": "aparejar",
    "aparejaseis": "aparejar",
    "aparejasen": "aparejar",
    "aparejases": "aparejar",
    "aparejaste": "aparejar",
    "aparejasteis": "aparejar",
    "apareje": "aparejar",
    "aparejemos": "aparejar",
    "aparejen": "aparejar",
    "aparejes": "aparejar",
    "aparejo": "aparejar",
    "aparejos": "aparejo",
    "aparejábamos": "aparejar",
    "aparejáis": "aparejar",
    "aparejáramos": "aparejar",
    "aparejáremos": "aparejar",
    "aparejásemos": "aparejar",
    "aparejé": "aparejar",
    "aparejéis": "aparejar",
    "aparejó": "aparejar",
    "aparellajes": "aparellaje",
    "aparemos": "aparar",
    "aparen": "aparar",
    "aparenciales": "aparencial",
    "aparenta": "aparentar",
    "aparentaba": "aparentar",
    "aparentabais": "aparentar",
    "aparentaban": "aparentar",
    "aparentabas": "aparentar",
    "aparentad": "aparentar",
    "aparentada": "aparentar",
    "aparentadas": "aparentar",
    "aparentado": "aparentar",
    "aparentados": "aparentar",
    "aparentamos": "aparentar",
    "aparentan": "aparentar",
    "aparentando": "aparentar",
    "aparentara": "aparentar",
    "aparentarais": "aparentar",
    "aparentaran": "aparentar",
    "aparentaras": "aparentar",
    "aparentare": "aparentar",
    "aparentareis": "aparentar",
    "aparentaremos": "aparentar",
    "aparentaren": "aparentar",
    "aparentares": "aparentar",
    "aparentaron": "aparentar",
    "aparentará": "aparentar",
    "aparentarán": "aparentar",
    "aparentarás": "aparentar",
    "aparentaré": "aparentar",
    "aparentaréis": "aparentar",
    "aparentaría": "aparentar",
    "aparentaríais": "aparentar",
    "aparentaríamos": "aparentar",
    "aparentarían": "aparentar",
    "aparentarías": "aparentar",
    "aparentas": "aparentar",
    "aparentase": "aparentar",
    "aparentaseis": "aparentar",
    "aparentasen": "aparentar",
    "aparentases": "aparentar",
    "aparentaste": "aparentar",
    "aparentasteis": "aparentar",
    "aparente": "aparentar",
    "aparentemos": "aparentar",
    "aparenten": "aparentar",
    "aparentes": "aparente",
    "aparento": "aparentar",
    "aparentábamos": "aparentar",
    "aparentáis": "aparentar",
    "aparentáramos": "aparentar",
    "aparentáremos": "aparentar",
    "aparentásemos": "aparentar",
    "aparenté": "aparentar",
    "aparentéis": "aparentar",
    "aparentó": "aparentar",
    "apareo": "aparear",
    "apares": "aparar",
    "aparezca": "aparecer",
    "aparezcamos": "aparecer",
    "aparezcan": "aparecer",
    "aparezcas": "aparecer",
    "aparezco": "aparecer",
    "aparezcáis": "aparecer",
    "apareábamos": "aparear",
    "apareáis": "aparear",
    "apareáramos": "aparear",
    "apareáremos": "aparear",
    "apareásemos": "aparear",
    "apareé": "aparear",
    "apareéis": "aparear",
    "apareó": "aparear",
    "apariciones": "aparición",
    "apariencias": "apariencia",
    "aparo": "aparar",
    "aparque": "aparcar",
    "aparquemos": "aparcar",
    "aparquen": "aparcar",
    "aparques": "aparcar",
    "aparqué": "aparcar",
    "aparquéis": "aparcar",
    "aparra": "aparrar",
    "aparraba": "aparrar",
    "aparrabais": "aparrar",
    "aparraban": "aparrar",
    "aparrabas": "aparrar",
    "aparrad": "aparrar",
    "aparrada": "aparrar",
    "aparradas": "aparrar",
    "aparrado": "aparrar",
    "aparrados": "aparrar",
    "aparramos": "aparrar",
    "aparran": "aparrar",
    "aparrando": "aparrar",
    "aparrara": "aparrar",
    "aparrarais": "aparrar",
    "aparraran": "aparrar",
    "aparraras": "aparrar",
    "aparrare": "aparrar",
    "aparrareis": "aparrar",
    "aparraremos": "aparrar",
    "aparraren": "aparrar",
    "aparrares": "aparrar",
    "aparraron": "aparrar",
    "aparrará": "aparrar",
    "aparrarán": "aparrar",
    "aparrarás": "aparrar",
    "aparraré": "aparrar",
    "aparraréis": "aparrar",
    "aparraría": "aparrar",
    "aparraríais": "aparrar",
    "aparraríamos": "aparrar",
    "aparrarían": "aparrar",
    "aparrarías": "aparrar",
    "aparras": "aparrar",
    "aparrase": "aparrar",
    "aparraseis": "aparrar",
    "aparrasen": "aparrar",
    "aparrases": "aparrar",
    "aparraste": "aparrar",
    "aparrasteis": "aparrar",
    "aparre": "aparrar",
    "aparremos": "aparrar",
    "aparren": "aparrar",
    "aparres": "aparrar",
    "aparro": "aparrar",
    "aparroquia": "aparroquiar",
    "aparroquiaba": "aparroquiar",
    "aparroquiabais": "aparroquiar",
    "aparroquiaban": "aparroquiar",
    "aparroquiabas": "aparroquiar",
    "aparroquiad": "aparroquiar",
    "aparroquiada": "aparroquiar",
    "aparroquiadas": "aparroquiar",
    "aparroquiado": "aparroquiar",
    "aparroquiados": "aparroquiar",
    "aparroquiamos": "aparroquiar",
    "aparroquian": "aparroquiar",
    "aparroquiando": "aparroquiar",
    "aparroquiara": "aparroquiar",
    "aparroquiarais": "aparroquiar",
    "aparroquiaran": "aparroquiar",
    "aparroquiaras": "aparroquiar",
    "aparroquiare": "aparroquiar",
    "aparroquiareis": "aparroquiar",
    "aparroquiaremos": "aparroquiar",
    "aparroquiaren": "aparroquiar",
    "aparroquiares": "aparroquiar",
    "aparroquiaron": "aparroquiar",
    "aparroquiará": "aparroquiar",
    "aparroquiarán": "aparroquiar",
    "aparroquiarás": "aparroquiar",
    "aparroquiaré": "aparroquiar",
    "aparroquiaréis": "aparroquiar",
    "aparroquiaría": "aparroquiar",
    "aparroquiaríais": "aparroquiar",
    "aparroquiaríamos": "aparroquiar",
    "aparroquiarían": "aparroquiar",
    "aparroquiarías": "aparroquiar",
    "aparroquias": "aparroquiar",
    "aparroquiase": "aparroquiar",
    "aparroquiaseis": "aparroquiar",
    "aparroquiasen": "aparroquiar",
    "aparroquiases": "aparroquiar",
    "aparroquiaste": "aparroquiar",
    "aparroquiasteis": "aparroquiar",
    "aparroquie": "aparroquiar",
    "aparroquiemos": "aparroquiar",
    "aparroquien": "aparroquiar",
    "aparroquies": "aparroquiar",
    "aparroquio": "aparroquiar",
    "aparroquiábamos": "aparroquiar",
    "aparroquiáis": "aparroquiar",
    "aparroquiáramos": "aparroquiar",
    "aparroquiáremos": "aparroquiar",
    "aparroquiásemos": "aparroquiar",
    "aparroquié": "aparroquiar",
    "aparroquiéis": "aparroquiar",
    "aparroquió": "aparroquiar",
    "aparrábamos": "aparrar",
    "aparráis": "aparrar",
    "aparráramos": "aparrar",
    "aparráremos": "aparrar",
    "aparrásemos": "aparrar",
    "aparré": "aparrar",
    "aparréis": "aparrar",
    "aparró": "aparrar",
    "aparta": "apartar",
    "apartaba": "apartar",
    "apartabais": "apartar",
    "apartaban": "apartar",
    "apartabas": "apartar",
    "apartad": "apartar",
    "apartada": "apartar",
    "apartadas": "apartar",
    "apartadera": "apartadero",
    "apartaderas": "apartadero",
    "apartaderos": "apartadero",
    "apartadijos": "apartadijo",
    "apartadiza": "apartadizo",
    "apartadizas": "apartadizo",
    "apartadizos": "apartadizo",
    "apartado": "apartar",
    "apartadora": "apartador",
    "apartadoras": "apartador",
    "apartadores": "apartador",
    "apartados": "apartar",
    "apartahoteles": "apartahotel",
    "apartamentos": "apartamento",
    "apartamientos": "apartamiento",
    "apartamos": "apartar",
    "apartan": "apartar",
    "apartando": "apartar",
    "apartara": "apartar",
    "apartarais": "apartar",
    "apartaran": "apartar",
    "apartaras": "apartar",
    "apartare": "apartar",
    "apartareis": "apartar",
    "apartaremos": "apartar",
    "apartaren": "apartar",
    "apartares": "apartar",
    "apartaron": "apartar",
    "apartará": "apartar",
    "apartarán": "apartar",
    "apartarás": "apartar",
    "apartaré": "apartar",
    "apartaréis": "apartar",
    "apartaría": "apartar",
    "apartaríais": "apartar",
    "apartaríamos": "apartar",
    "apartarían": "apartar",
    "apartarías": "apartar",
    "apartas": "apartar",
    "apartase": "apartar",
    "apartaseis": "apartar",
    "apartasen": "apartar",
    "apartases": "apartar",
    "apartaste": "apartar",
    "apartasteis": "apartar",
    "aparte": "apartar",
    "apartemos": "apartar",
    "aparten": "apartar",
    "apartes": "aparte",
    "aparthoteles": "aparthotel",
    "aparto": "apartar",
    "apartábamos": "apartar",
    "apartáis": "apartar",
    "apartáramos": "apartar",
    "apartáremos": "apartar",
    "apartásemos": "apartar",
    "aparté": "apartar",
    "apartéis": "apartar",
    "apartó": "apartar",
    "aparva": "aparvar",
    "aparvaba": "aparvar",
    "aparvabais": "aparvar",
    "aparvaban": "aparvar",
    "aparvabas": "aparvar",
    "aparvad": "aparvar",
    "aparvada": "aparvar",
    "aparvadas": "aparvar",
    "aparvado": "aparvar",
    "aparvados": "aparvar",
    "aparvamos": "aparvar",
    "aparvan": "aparvar",
    "aparvando": "aparvar",
    "aparvara": "aparvar",
    "aparvarais": "aparvar",
    "aparvaran": "aparvar",
    "aparvaras": "aparvar",
    "aparvare": "aparvar",
    "aparvareis": "aparvar",
    "aparvaremos": "aparvar",
    "aparvaren": "aparvar",
    "aparvares": "aparvar",
    "aparvaron": "aparvar",
    "aparvará": "aparvar",
    "aparvarán": "aparvar",
    "aparvarás": "aparvar",
    "aparvaré": "aparvar",
    "aparvaréis": "aparvar",
    "aparvaría": "aparvar",
    "aparvaríais": "aparvar",
    "aparvaríamos": "aparvar",
    "aparvarían": "aparvar",
    "aparvarías": "aparvar",
    "aparvas": "aparvar",
    "aparvase": "aparvar",
    "aparvaseis": "aparvar",
    "aparvasen": "aparvar",
    "aparvases": "aparvar",
    "aparvaste": "aparvar",
    "aparvasteis": "aparvar",
    "aparve": "aparvar",
    "aparvemos": "aparvar",
    "aparven": "aparvar",
    "aparves": "aparvar",
    "aparvo": "aparvar",
    "aparvábamos": "aparvar",
    "aparváis": "aparvar",
    "aparváramos": "aparvar",
    "aparváremos": "aparvar",
    "aparvásemos": "aparvar",
    "aparvé": "aparvar",
    "aparvéis": "aparvar",
    "aparvó": "aparvar",
    "aparábamos": "aparar",
    "aparáis": "aparar",
    "aparáramos": "aparar",
    "aparáremos": "aparar",
    "aparásemos": "aparar",
    "aparé": "aparar",
    "aparéis": "aparar",
    "aparó": "aparar",
    "apasiona": "apasionar",
    "apasionaba": "apasionar",
    "apasionabais": "apasionar",
    "apasionaban": "apasionar",
    "apasionabas": "apasionar",
    "apasionad": "apasionar",
    "apasionada": "apasionar",
    "apasionadas": "apasionar",
    "apasionado": "apasionar",
    "apasionados": "apasionar",
    "apasionamientos": "apasionamiento",
    "apasionamos": "apasionar",
    "apasionan": "apasionar",
    "apasionando": "apasionar",
    "apasionantes": "apasionante",
    "apasionara": "apasionar",
    "apasionarais": "apasionar",
    "apasionaran": "apasionar",
    "apasionaras": "apasionar",
    "apasionare": "apasionar",
    "apasionareis": "apasionar",
    "apasionaremos": "apasionar",
    "apasionaren": "apasionar",
    "apasionares": "apasionar",
    "apasionaron": "apasionar",
    "apasionará": "apasionar",
    "apasionarán": "apasionar",
    "apasionarás": "apasionar",
    "apasionaré": "apasionar",
    "apasionaréis": "apasionar",
    "apasionaría": "apasionar",
    "apasionaríais": "apasionar",
    "apasionaríamos": "apasionar",
    "apasionarían": "apasionar",
    "apasionarías": "apasionar",
    "apasionas": "apasionar",
    "apasionase": "apasionar",
    "apasionaseis": "apasionar",
    "apasionasen": "apasionar",
    "apasionases": "apasionar",
    "apasionaste": "apasionar",
    "apasionasteis": "apasionar",
    "apasione": "apasionar",
    "apasionemos": "apasionar",
    "apasionen": "apasionar",
    "apasiones": "apasionar",
    "apasiono": "apasionar",
    "apasionábamos": "apasionar",
    "apasionáis": "apasionar",
    "apasionáramos": "apasionar",
    "apasionáremos": "apasionar",
    "apasionásemos": "apasionar",
    "apasioné": "apasionar",
    "apasionéis": "apasionar",
    "apasionó": "apasionar",
    "apasta": "apastar",
    "apastaba": "apastar",
    "apastabais": "apastar",
    "apastaban": "apastar",
    "apastabas": "apastar",
    "apastad": "apastar",
    "apastada": "apastar",
    "apastadas": "apastar",
    "apastado": "apastar",
    "apastados": "apastar",
    "apastamos": "apastar",
    "apastan": "apastar",
    "apastando": "apastar",
    "apastara": "apastar",
    "apastarais": "apastar",
    "apastaran": "apastar",
    "apastaras": "apastar",
    "apastare": "apastar",
    "apastareis": "apastar",
    "apastaremos": "apastar",
    "apastaren": "apastar",
    "apastares": "apastar",
    "apastaron": "apastar",
    "apastará": "apastar",
    "apastarán": "apastar",
    "apastarás": "apastar",
    "apastaré": "apastar",
    "apastaréis": "apastar",
    "apastaría": "apastar",
    "apastaríais": "apastar",
    "apastaríamos": "apastar",
    "apastarían": "apastar",
    "apastarías": "apastar",
    "apastas": "apastar",
    "apastase": "apastar",
    "apastaseis": "apastar",
    "apastasen": "apastar",
    "apastases": "apastar",
    "apastaste": "apastar",
    "apastasteis": "apastar",
    "apaste": "apastar",
    "apastemos": "apastar",
    "apasten": "apastar",
    "apastes": "apastar",
    "apasto": "apastar",
    "apastábamos": "apastar",
    "apastáis": "apastar",
    "apastáramos": "apastar",
    "apastáremos": "apastar",
    "apastásemos": "apastar",
    "apasté": "apastar",
    "apastéis": "apastar",
    "apastó": "apastar",
    "apates": "apate",
    "apatitos": "apatito",
    "apatridias": "apatridia",
    "apatías": "apatía",
    "apaula": "apaular",
    "apaulaba": "apaular",
    "apaulabais": "apaular",
    "apaulaban": "apaular",
    "apaulabas": "apaular",
    "apaulad": "apaular",
    "apaulada": "apaular",
    "apauladas": "apaular",
    "apaulado": "apaular",
    "apaulados": "apaular",
    "apaulamos": "apaular",
    "apaulan": "apaular",
    "apaulando": "apaular",
    "apaulara": "apaular",
    "apaularais": "apaular",
    "apaularan": "apaular",
    "apaularas": "apaular",
    "apaulare": "apaular",
    "apaulareis": "apaular",
    "apaularemos": "apaular",
    "apaularen": "apaular",
    "apaulares": "apaular",
    "apaularon": "apaular",
    "apaulará": "apaular",
    "apaularán": "apaular",
    "apaularás": "apaular",
    "apaularé": "apaular",
    "apaularéis": "apaular",
    "apaularía": "apaular",
    "apaularíais": "apaular",
    "apaularíamos": "apaular",
    "apaularían": "apaular",
    "apaularías": "apaular",
    "apaulas": "apaular",
    "apaulase": "apaular",
    "apaulaseis": "apaular",
    "apaulasen": "apaular",
    "apaulases": "apaular",
    "apaulaste": "apaular",
    "apaulasteis": "apaular",
    "apaule": "apaular",
    "apaulemos": "apaular",
    "apaulen": "apaular",
    "apaules": "apaular",
    "apaulilla": "apaulillar",
    "apaulillaba": "apaulillar",
    "apaulillabais": "apaulillar",
    "apaulillaban": "apaulillar",
    "apaulillabas": "apaulillar",
    "apaulillad": "apaulillar",
    "apaulillada": "apaulillar",
    "apaulilladas": "apaulillar",
    "apaulillado": "apaulillar",
    "apaulillados": "apaulillar",
    "apaulillamos": "apaulillar",
    "apaulillan": "apaulillar",
    "apaulillando": "apaulillar",
    "apaulillara": "apaulillar",
    "apaulillarais": "apaulillar",
    "apaulillaran": "apaulillar",
    "apaulillaras": "apaulillar",
    "apaulillare": "apaulillar",
    "apaulillareis": "apaulillar",
    "apaulillaremos": "apaulillar",
    "apaulillaren": "apaulillar",
    "apaulillares": "apaulillar",
    "apaulillaron": "apaulillar",
    "apaulillará": "apaulillar",
    "apaulillarán": "apaulillar",
    "apaulillarás": "apaulillar",
    "apaulillaré": "apaulillar",
    "apaulillaréis": "apaulillar",
    "apaulillaría": "apaulillar",
    "apaulillaríais": "apaulillar",
    "apaulillaríamos": "apaulillar",
    "apaulillarían": "apaulillar",
    "apaulillarías": "apaulillar",
    "apaulillas": "apaulillar",
    "apaulillase": "apaulillar",
    "apaulillaseis": "apaulillar",
    "apaulillasen": "apaulillar",
    "apaulillases": "apaulillar",
    "apaulillaste": "apaulillar",
    "apaulillasteis": "apaulillar",
    "apaulille": "apaulillar",
    "apaulillemos": "apaulillar",
    "apaulillen": "apaulillar",
    "apaulilles": "apaulillar",
    "apaulillo": "apaulillar",
    "apaulillábamos": "apaulillar",
    "apaulilláis": "apaulillar",
    "apaulilláramos": "apaulillar",
    "apaulilláremos": "apaulillar",
    "apaulillásemos": "apaulillar",
    "apaulillé": "apaulillar",
    "apaulilléis": "apaulillar",
    "apaulilló": "apaulillar",
    "apaulo": "apaular",
    "apaulábamos": "apaular",
    "apauláis": "apaular",
    "apauláramos": "apaular",
    "apauláremos": "apaular",
    "apaulásemos": "apaular",
    "apaulé": "apaular",
    "apauléis": "apaular",
    "apauló": "apaular",
    "apaña": "apañar",
    "apañaba": "apañar",
    "apañabais": "apañar",
    "apañaban": "apañar",
    "apañabas": "apañar",
    "apañad": "apañar",
    "apañada": "apañar",
    "apañadas": "apañar",
    "apañado": "apañar",
    "apañados": "apañar",
    "apañaduras": "apañadura",
    "apañamos": "apañar",
    "apañan": "apañar",
    "apañando": "apañar",
    "apañara": "apañar",
    "apañarais": "apañar",
    "apañaran": "apañar",
    "apañaras": "apañar",
    "apañare": "apañar",
    "apañareis": "apañar",
    "apañaremos": "apañar",
    "apañaren": "apañar",
    "apañares": "apañar",
    "apañaron": "apañar",
    "apañará": "apañar",
    "apañarán": "apañar",
    "apañarás": "apañar",
    "apañaré": "apañar",
    "apañaréis": "apañar",
    "apañaría": "apañar",
    "apañaríais": "apañar",
    "apañaríamos": "apañar",
    "apañarían": "apañar",
    "apañarías": "apañar",
    "apañas": "apañar",
    "apañase": "apañar",
    "apañaseis": "apañar",
    "apañasen": "apañar",
    "apañases": "apañar",
    "apañaste": "apañar",
    "apañasteis": "apañar",
    "apañe": "apañar",
    "apañemos": "apañar",
    "apañen": "apañar",
    "apañes": "apañar",
    "apaño": "apañar",
    "apaños": "apaño",
    "apañusca": "apañuscar",
    "apañuscaba": "apañuscar",
    "apañuscabais": "apañuscar",
    "apañuscaban": "apañuscar",
    "apañuscabas": "apañuscar",
    "apañuscad": "apañuscar",
    "apañuscada": "apañuscar",
    "apañuscadas": "apañuscar",
    "apañuscado": "apañuscar",
    "apañuscadora": "apañuscador",
    "apañuscadoras": "apañuscador",
    "apañuscadores": "apañuscador",
    "apañuscados": "apañuscar",
    "apañuscamos": "apañuscar",
    "apañuscan": "apañuscar",
    "apañuscando": "apañuscar",
    "apañuscara": "apañuscar",
    "apañuscarais": "apañuscar",
    "apañuscaran": "apañuscar",
    "apañuscaras": "apañuscar",
    "apañuscare": "apañuscar",
    "apañuscareis": "apañuscar",
    "apañuscaremos": "apañuscar",
    "apañuscaren": "apañuscar",
    "apañuscares": "apañuscar",
    "apañuscaron": "apañuscar",
    "apañuscará": "apañuscar",
    "apañuscarán": "apañuscar",
    "apañuscarás": "apañuscar",
    "apañuscaré": "apañuscar",
    "apañuscaréis": "apañuscar",
    "apañuscaría": "apañuscar",
    "apañuscaríais": "apañuscar",
    "apañuscaríamos": "apañuscar",
    "apañuscarían": "apañuscar",
    "apañuscarías": "apañuscar",
    "apañuscas": "apañuscar",
    "apañuscase": "apañuscar",
    "apañuscaseis": "apañuscar",
    "apañuscasen": "apañuscar",
    "apañuscases": "apañuscar",
    "apañuscaste": "apañuscar",
    "apañuscasteis": "apañuscar",
    "apañusco": "apañuscar",
    "apañuscábamos": "apañuscar",
    "apañuscáis": "apañuscar",
    "apañuscáramos": "apañuscar",
    "apañuscáremos": "apañuscar",
    "apañuscásemos": "apañuscar",
    "apañuscó": "apañuscar",
    "apañusque": "apañuscar",
    "apañusquemos": "apañuscar",
    "apañusquen": "apañuscar",
    "apañusques": "apañuscar",
    "apañusqué": "apañuscar",
    "apañusquéis": "apañuscar",
    "apañábamos": "apañar",
    "apañáis": "apañar",
    "apañáramos": "apañar",
    "apañáremos": "apañar",
    "apañásemos": "apañar",
    "apañé": "apañar",
    "apañéis": "apañar",
    "apañó": "apañar",
    "apea": "apear",
    "apeaba": "apear",
    "apeabais": "apear",
    "apeaban": "apear",
    "apeabas": "apear",
    "apead": "apear",
    "apeada": "apear",
    "apeadas": "apear",
    "apeaderos": "apeadero",
    "apeado": "apear",
    "apeados": "apear",
    "apeala": "apealar",
    "apealaba": "apealar",
    "apealabais": "apealar",
    "apealaban": "apealar",
    "apealabas": "apealar",
    "apealad": "apealar",
    "apealada": "apealar",
    "apealadas": "apealar",
    "apealado": "apealar",
    "apealados": "apealar",
    "apealamos": "apealar",
    "apealan": "apealar",
    "apealando": "apealar",
    "apealara": "apealar",
    "apealarais": "apealar",
    "apealaran": "apealar",
    "apealaras": "apealar",
    "apealare": "apealar",
    "apealareis": "apealar",
    "apealaremos": "apealar",
    "apealaren": "apealar",
    "apealares": "apealar",
    "apealaron": "apealar",
    "apealará": "apealar",
    "apealarán": "apealar",
    "apealarás": "apealar",
    "apealaré": "apealar",
    "apealaréis": "apealar",
    "apealaría": "apealar",
    "apealaríais": "apealar",
    "apealaríamos": "apealar",
    "apealarían": "apealar",
    "apealarías": "apealar",
    "apealas": "apealar",
    "apealase": "apealar",
    "apealaseis": "apealar",
    "apealasen": "apealar",
    "apealases": "apealar",
    "apealaste": "apealar",
    "apealasteis": "apealar",
    "apeale": "apealar",
    "apealemos": "apealar",
    "apealen": "apealar",
    "apeales": "apealar",
    "apealo": "apealar",
    "apealábamos": "apealar",
    "apealáis": "apealar",
    "apealáramos": "apealar",
    "apealáremos": "apealar",
    "apealásemos": "apealar",
    "apealé": "apealar",
    "apealéis": "apealar",
    "apealó": "apealar",
    "apeamientos": "apeamiento",
    "apeamos": "apear",
    "apean": "apear",
    "apeando": "apear",
    "apeara": "apear",
    "apearais": "apear",
    "apearan": "apear",
    "apearas": "apear",
    "apeare": "apear",
    "apeareis": "apear",
    "apearemos": "apear",
    "apearen": "apear",
    "apeares": "apear",
    "apearon": "apear",
    "apeará": "apear",
    "apearán": "apear",
    "apearás": "apear",
    "apearé": "apear",
    "apearéis": "apear",
    "apearía": "apear",
    "apearíais": "apear",
    "apearíamos": "apear",
    "apearían": "apear",
    "apearías": "apear",
    "apeas": "apear",
    "apease": "apear",
    "apeaseis": "apear",
    "apeasen": "apear",
    "apeases": "apear",
    "apeaste": "apear",
    "apeasteis": "apear",
    "apecha": "apechar",
    "apechaba": "apechar",
    "apechabais": "apechar",
    "apechaban": "apechar",
    "apechabas": "apechar",
    "apechad": "apechar",
    "apechada": "apechar",
    "apechadas": "apechar",
    "apechado": "apechar",
    "apechados": "apechar",
    "apechamos": "apechar",
    "apechan": "apechar",
    "apechando": "apechar",
    "apechara": "apechar",
    "apecharais": "apechar",
    "apecharan": "apechar",
    "apecharas": "apechar",
    "apechare": "apechar",
    "apechareis": "apechar",
    "apecharemos": "apechar",
    "apecharen": "apechar",
    "apechares": "apechar",
    "apecharon": "apechar",
    "apechará": "apechar",
    "apecharán": "apechar",
    "apecharás": "apechar",
    "apecharé": "apechar",
    "apecharéis": "apechar",
    "apecharía": "apechar",
    "apecharíais": "apechar",
    "apecharíamos": "apechar",
    "apecharían": "apechar",
    "apecharías": "apechar",
    "apechas": "apechar",
    "apechase": "apechar",
    "apechaseis": "apechar",
    "apechasen": "apechar",
    "apechases": "apechar",
    "apechaste": "apechar",
    "apechasteis": "apechar",
    "apeche": "apechar",
    "apechemos": "apechar",
    "apechen": "apechar",
    "apeches": "apechar",
    "apecho": "apechar",
    "apechuga": "apechugar",
    "apechugaba": "apechugar",
    "apechugabais": "apechugar",
    "apechugaban": "apechugar",
    "apechugabas": "apechugar",
    "apechugad": "apechugar",
    "apechugada": "apechugar",
    "apechugadas": "apechugar",
    "apechugado": "apechugar",
    "apechugados": "apechugar",
    "apechugamos": "apechugar",
    "apechugan": "apechugar",
    "apechugando": "apechugar",
    "apechugara": "apechugar",
    "apechugarais": "apechugar",
    "apechugaran": "apechugar",
    "apechugaras": "apechugar",
    "apechugare": "apechugar",
    "apechugareis": "apechugar",
    "apechugaremos": "apechugar",
    "apechugaren": "apechugar",
    "apechugares": "apechugar",
    "apechugaron": "apechugar",
    "apechugará": "apechugar",
    "apechugarán": "apechugar",
    "apechugarás": "apechugar",
    "apechugaré": "apechugar",
    "apechugaréis": "apechugar",
    "apechugaría": "apechugar",
    "apechugaríais": "apechugar",
    "apechugaríamos": "apechugar",
    "apechugarían": "apechugar",
    "apechugarías": "apechugar",
    "apechugas": "apechugar",
    "apechugase": "apechugar",
    "apechugaseis": "apechugar",
    "apechugasen": "apechugar",
    "apechugases": "apechugar",
    "apechugaste": "apechugar",
    "apechugasteis": "apechugar",
    "apechugo": "apechugar",
    "apechugue": "apechugar",
    "apechuguemos": "apechugar",
    "apechuguen": "apechugar",
    "apechugues": "apechugar",
    "apechugué": "apechugar",
    "apechuguéis": "apechugar",
    "apechugábamos": "apechugar",
    "apechugáis": "apechugar",
    "apechugáramos": "apechugar",
    "apechugáremos": "apechugar",
    "apechugásemos": "apechugar",
    "apechugó": "apechugar",
    "apechábamos": "apechar",
    "apecháis": "apechar",
    "apecháramos": "apechar",
    "apecháremos": "apechar",
    "apechásemos": "apechar",
    "apeché": "apechar",
    "apechéis": "apechar",
    "apechó": "apechar",
    "apectomías": "apectomía",
    "apedace": "apedazar",
    "apedacemos": "apedazar",
    "apedacen": "apedazar",
    "apedaces": "apedazar",
    "apedacé": "apedazar",
    "apedacéis": "apedazar",
    "apedaza": "apedazar",
    "apedazaba": "apedazar",
    "apedazabais": "apedazar",
    "apedazaban": "apedazar",
    "apedazabas": "apedazar",
    "apedazad": "apedazar",
    "apedazada": "apedazar",
    "apedazadas": "apedazar",
    "apedazado": "apedazar",
    "apedazados": "apedazar",
    "apedazamos": "apedazar",
    "apedazan": "apedazar",
    "apedazando": "apedazar",
    "apedazara": "apedazar",
    "apedazarais": "apedazar",
    "apedazaran": "apedazar",
    "apedazaras": "apedazar",
    "apedazare": "apedazar",
    "apedazareis": "apedazar",
    "apedazaremos": "apedazar",
    "apedazaren": "apedazar",
    "apedazares": "apedazar",
    "apedazaron": "apedazar",
    "apedazará": "apedazar",
    "apedazarán": "apedazar",
    "apedazarás": "apedazar",
    "apedazaré": "apedazar",
    "apedazaréis": "apedazar",
    "apedazaría": "apedazar",
    "apedazaríais": "apedazar",
    "apedazaríamos": "apedazar",
    "apedazarían": "apedazar",
    "apedazarías": "apedazar",
    "apedazas": "apedazar",
    "apedazase": "apedazar",
    "apedazaseis": "apedazar",
    "apedazasen": "apedazar",
    "apedazases": "apedazar",
    "apedazaste": "apedazar",
    "apedazasteis": "apedazar",
    "apedazo": "apedazar",
    "apedazábamos": "apedazar",
    "apedazáis": "apedazar",
    "apedazáramos": "apedazar",
    "apedazáremos": "apedazar",
    "apedazásemos": "apedazar",
    "apedazó": "apedazar",
    "apedrea": "apedrear",
    "apedreaba": "apedrear",
    "apedreabais": "apedrear",
    "apedreaban": "apedrear",
    "apedreabas": "apedrear",
    "apedread": "apedrear",
    "apedreada": "apedrear",
    "apedreadas": "apedrear",
    "apedreado": "apedrear",
    "apedreadora": "apedreador",
    "apedreadoras": "apedreador",
    "apedreadores": "apedreador",
    "apedreados": "apedrear",
    "apedreamientos": "apedreamiento",
    "apedreamos": "apedrear",
    "apedrean": "apedrear",
    "apedreando": "apedrear",
    "apedreara": "apedrear",
    "apedrearais": "apedrear",
    "apedrearan": "apedrear",
    "apedrearas": "apedrear",
    "apedreare": "apedrear",
    "apedreareis": "apedrear",
    "apedrearemos": "apedrear",
    "apedrearen": "apedrear",
    "apedreares": "apedrear",
    "apedrearon": "apedrear",
    "apedreará": "apedrear",
    "apedrearán": "apedrear",
    "apedrearás": "apedrear",
    "apedrearé": "apedrear",
    "apedrearéis": "apedrear",
    "apedrearía": "apedrear",
    "apedrearíais": "apedrear",
    "apedrearíamos": "apedrear",
    "apedrearían": "apedrear",
    "apedrearías": "apedrear",
    "apedreas": "apedrear",
    "apedrease": "apedrear",
    "apedreaseis": "apedrear",
    "apedreasen": "apedrear",
    "apedreases": "apedrear",
    "apedreaste": "apedrear",
    "apedreasteis": "apedrear",
    "apedree": "apedrear",
    "apedreemos": "apedrear",
    "apedreen": "apedrear",
    "apedrees": "apedrear",
    "apedreo": "apedrear",
    "apedreos": "apedreo",
    "apedreábamos": "apedrear",
    "apedreáis": "apedrear",
    "apedreáramos": "apedrear",
    "apedreáremos": "apedrear",
    "apedreásemos": "apedrear",
    "apedreé": "apedrear",
    "apedreéis": "apedrear",
    "apedreó": "apedrear",
    "apee": "apear",
    "apeemos": "apear",
    "apeen": "apear",
    "apees": "apear",
    "apegos": "apego",
    "apeguala": "apegualar",
    "apegualaba": "apegualar",
    "apegualabais": "apegualar",
    "apegualaban": "apegualar",
    "apegualabas": "apegualar",
    "apegualad": "apegualar",
    "apegualada": "apegualar",
    "apegualadas": "apegualar",
    "apegualado": "apegualar",
    "apegualados": "apegualar",
    "apegualamos": "apegualar",
    "apegualan": "apegualar",
    "apegualando": "apegualar",
    "apegualara": "apegualar",
    "apegualarais": "apegualar",
    "apegualaran": "apegualar",
    "apegualaras": "apegualar",
    "apegualare": "apegualar",
    "apegualareis": "apegualar",
    "apegualaremos": "apegualar",
    "apegualaren": "apegualar",
    "apegualares": "apegualar",
    "apegualaron": "apegualar",
    "apegualará": "apegualar",
    "apegualarán": "apegualar",
    "apegualarás": "apegualar",
    "apegualaré": "apegualar",
    "apegualaréis": "apegualar",
    "apegualaría": "apegualar",
    "apegualaríais": "apegualar",
    "apegualaríamos": "apegualar",
    "apegualarían": "apegualar",
    "apegualarías": "apegualar",
    "apegualas": "apegualar",
    "apegualase": "apegualar",
    "apegualaseis": "apegualar",
    "apegualasen": "apegualar",
    "apegualases": "apegualar",
    "apegualaste": "apegualar",
    "apegualasteis": "apegualar",
    "apeguale": "apegualar",
    "apegualemos": "apegualar",
    "apegualen": "apegualar",
    "apeguales": "apegualar",
    "apegualo": "apegualar",
    "apegualábamos": "apegualar",
    "apegualáis": "apegualar",
    "apegualáramos": "apegualar",
    "apegualáremos": "apegualar",
    "apegualásemos": "apegualar",
    "apegualé": "apegualar",
    "apegualéis": "apegualar",
    "apegualó": "apegualar",
    "apela": "apelar",
    "apelaba": "apelar",
    "apelabais": "apelar",
    "apelaban": "apelar",
    "apelabas": "apelar",
    "apelables": "apelable",
    "apelaciones": "apelación",
    "apelad": "apelar",
    "apelada": "apelar",
    "apeladas": "apelar",
    "apelado": "apelar",
    "apelados": "apelar",
    "apelambra": "apelambrar",
    "apelambraba": "apelambrar",
    "apelambrabais": "apelambrar",
    "apelambraban": "apelambrar",
    "apelambrabas": "apelambrar",
    "apelambrad": "apelambrar",
    "apelambrada": "apelambrar",
    "apelambradas": "apelambrar",
    "apelambrado": "apelambrar",
    "apelambrados": "apelambrar",
    "apelambramos": "apelambrar",
    "apelambran": "apelambrar",
    "apelambrando": "apelambrar",
    "apelambrara": "apelambrar",
    "apelambrarais": "apelambrar",
    "apelambraran": "apelambrar",
    "apelambraras": "apelambrar",
    "apelambrare": "apelambrar",
    "apelambrareis": "apelambrar",
    "apelambraremos": "apelambrar",
    "apelambraren": "apelambrar",
    "apelambrares": "apelambrar",
    "apelambraron": "apelambrar",
    "apelambrará": "apelambrar",
    "apelambrarán": "apelambrar",
    "apelambrarás": "apelambrar",
    "apelambraré": "apelambrar",
    "apelambraréis": "apelambrar",
    "apelambraría": "apelambrar",
    "apelambraríais": "apelambrar",
    "apelambraríamos": "apelambrar",
    "apelambrarían": "apelambrar",
    "apelambrarías": "apelambrar",
    "apelambras": "apelambrar",
    "apelambrase": "apelambrar",
    "apelambraseis": "apelambrar",
    "apelambrasen": "apelambrar",
    "apelambrases": "apelambrar",
    "apelambraste": "apelambrar",
    "apelambrasteis": "apelambrar",
    "apelambre": "apelambrar",
    "apelambremos": "apelambrar",
    "apelambren": "apelambrar",
    "apelambres": "apelambrar",
    "apelambro": "apelambrar",
    "apelambrábamos": "apelambrar",
    "apelambráis": "apelambrar",
    "apelambráramos": "apelambrar",
    "apelambráremos": "apelambrar",
    "apelambrásemos": "apelambrar",
    "apelambré": "apelambrar",
    "apelambréis": "apelambrar",
    "apelambró": "apelambrar",
    "apelamos": "apelar",
    "apelan": "apelar",
    "apelando": "apelar",
    "apelantes": "apelante",
    "apelara": "apelar",
    "apelarais": "apelar",
    "apelaran": "apelar",
    "apelaras": "apelar",
    "apelare": "apelar",
    "apelareis": "apelar",
    "apelaremos": "apelar",
    "apelaren": "apelar",
    "apelares": "apelar",
    "apelaron": "apelar",
    "apelará": "apelar",
    "apelarán": "apelar",
    "apelarás": "apelar",
    "apelaré": "apelar",
    "apelaréis": "apelar",
    "apelaría": "apelar",
    "apelaríais": "apelar",
    "apelaríamos": "apelar",
    "apelarían": "apelar",
    "apelarías": "apelar",
    "apelas": "apelar",
    "apelase": "apelar",
    "apelaseis": "apelar",
    "apelasen": "apelar",
    "apelases": "apelar",
    "apelaste": "apelar",
    "apelasteis": "apelar",
    "apelativa": "apelativo",
    "apelativas": "apelativo",
    "apelativos": "apelativo",
    "apelda": "apeldar",
    "apeldaba": "apeldar",
    "apeldabais": "apeldar",
    "apeldaban": "apeldar",
    "apeldabas": "apeldar",
    "apeldad": "apeldar",
    "apeldada": "apeldar",
    "apeldadas": "apeldar",
    "apeldado": "apeldar",
    "apeldados": "apeldar",
    "apeldamos": "apeldar",
    "apeldan": "apeldar",
    "apeldando": "apeldar",
    "apeldara": "apeldar",
    "apeldarais": "apeldar",
    "apeldaran": "apeldar",
    "apeldaras": "apeldar",
    "apeldare": "apeldar",
    "apeldareis": "apeldar",
    "apeldaremos": "apeldar",
    "apeldaren": "apeldar",
    "apeldares": "apeldar",
    "apeldaron": "apeldar",
    "apeldará": "apeldar",
    "apeldarán": "apeldar",
    "apeldarás": "apeldar",
    "apeldaré": "apeldar",
    "apeldaréis": "apeldar",
    "apeldaría": "apeldar",
    "apeldaríais": "apeldar",
    "apeldaríamos": "apeldar",
    "apeldarían": "apeldar",
    "apeldarías": "apeldar",
    "apeldas": "apeldar",
    "apeldase": "apeldar",
    "apeldaseis": "apeldar",
    "apeldasen": "apeldar",
    "apeldases": "apeldar",
    "apeldaste": "apeldar",
    "apeldasteis": "apeldar",
    "apelde": "apeldar",
    "apeldemos": "apeldar",
    "apelden": "apeldar",
    "apeldes": "apelde",
    "apeldo": "apeldar",
    "apeldábamos": "apeldar",
    "apeldáis": "apeldar",
    "apeldáramos": "apeldar",
    "apeldáremos": "apeldar",
    "apeldásemos": "apeldar",
    "apeldé": "apeldar",
    "apeldéis": "apeldar",
    "apeldó": "apeldar",
    "apele": "apelar",
    "apelemos": "apelar",
    "apelen": "apelar",
    "apeles": "apelar",
    "apella": "apellar",
    "apellaba": "apellar",
    "apellabais": "apellar",
    "apellaban": "apellar",
    "apellabas": "apellar",
    "apellad": "apellar",
    "apellada": "apellar",
    "apelladas": "apellar",
    "apellado": "apellar",
    "apellados": "apellar",
    "apellamos": "apellar",
    "apellan": "apellar",
    "apellando": "apellar",
    "apellara": "apellar",
    "apellarais": "apellar",
    "apellaran": "apellar",
    "apellaras": "apellar",
    "apellare": "apellar",
    "apellareis": "apellar",
    "apellaremos": "apellar",
    "apellaren": "apellar",
    "apellares": "apellar",
    "apellaron": "apellar",
    "apellará": "apellar",
    "apellarán": "apellar",
    "apellarás": "apellar",
    "apellaré": "apellar",
    "apellaréis": "apellar",
    "apellaría": "apellar",
    "apellaríais": "apellar",
    "apellaríamos": "apellar",
    "apellarían": "apellar",
    "apellarías": "apellar",
    "apellas": "apellar",
    "apellase": "apellar",
    "apellaseis": "apellar",
    "apellasen": "apellar",
    "apellases": "apellar",
    "apellaste": "apellar",
    "apellasteis": "apellar",
    "apelle": "apellar",
    "apellemos": "apellar",
    "apellen": "apellar",
    "apelles": "apellar",
    "apellida": "apellidar",
    "apellidaba": "apellidar",
    "apellidabais": "apellidar",
    "apellidaban": "apellidar",
    "apellidabas": "apellidar",
    "apellidad": "apellidar",
    "apellidada": "apellidar",
    "apellidadas": "apellidar",
    "apellidado": "apellidar",
    "apellidados": "apellidar",
    "apellidamos": "apellidar",
    "apellidan": "apellidar",
    "apellidando": "apellidar",
    "apellidara": "apellidar",
    "apellidarais": "apellidar",
    "apellidaran": "apellidar",
    "apellidaras": "apellidar",
    "apellidare": "apellidar",
    "apellidareis": "apellidar",
    "apellidaremos": "apellidar",
    "apellidaren": "apellidar",
    "apellidares": "apellidar",
    "apellidaron": "apellidar",
    "apellidará": "apellidar",
    "apellidarán": "apellidar",
    "apellidarás": "apellidar",
    "apellidaré": "apellidar",
    "apellidaréis": "apellidar",
    "apellidaría": "apellidar",
    "apellidaríais": "apellidar",
    "apellidaríamos": "apellidar",
    "apellidarían": "apellidar",
    "apellidarías": "apellidar",
    "apellidas": "apellidar",
    "apellidase": "apellidar",
    "apellidaseis": "apellidar",
    "apellidasen": "apellidar",
    "apellidases": "apellidar",
    "apellidaste": "apellidar",
    "apellidasteis": "apellidar",
    "apellide": "apellidar",
    "apellidemos": "apellidar",
    "apelliden": "apellidar",
    "apellides": "apellidar",
    "apellido": "apellidar",
    "apellidos": "apellido",
    "apellidábamos": "apellidar",
    "apellidáis": "apellidar",
    "apellidáramos": "apellidar",
    "apellidáremos": "apellidar",
    "apellidásemos": "apellidar",
    "apellidé": "apellidar",
    "apellidéis": "apellidar",
    "apellidó": "apellidar",
    "apello": "apellar",
    "apellábamos": "apellar",
    "apelláis": "apellar",
    "apelláramos": "apellar",
    "apelláremos": "apellar",
    "apellásemos": "apellar",
    "apellé": "apellar",
    "apelléis": "apellar",
    "apelló": "apellar",
    "apelmace": "apelmazar",
    "apelmacemos": "apelmazar",
    "apelmacen": "apelmazar",
    "apelmaces": "apelmazar",
    "apelmacé": "apelmazar",
    "apelmacéis": "apelmazar",
    "apelmaza": "apelmazar",
    "apelmazaba": "apelmazar",
    "apelmazabais": "apelmazar",
    "apelmazaban": "apelmazar",
    "apelmazabas": "apelmazar",
    "apelmazad": "apelmazar",
    "apelmazada": "apelmazar",
    "apelmazadas": "apelmazar",
    "apelmazado": "apelmazar",
    "apelmazados": "apelmazar",
    "apelmazamientos": "apelmazamiento",
    "apelmazamos": "apelmazar",
    "apelmazan": "apelmazar",
    "apelmazando": "apelmazar",
    "apelmazara": "apelmazar",
    "apelmazarais": "apelmazar",
    "apelmazaran": "apelmazar",
    "apelmazaras": "apelmazar",
    "apelmazare": "apelmazar",
    "apelmazareis": "apelmazar",
    "apelmazaremos": "apelmazar",
    "apelmazaren": "apelmazar",
    "apelmazares": "apelmazar",
    "apelmazaron": "apelmazar",
    "apelmazará": "apelmazar",
    "apelmazarán": "apelmazar",
    "apelmazarás": "apelmazar",
    "apelmazaré": "apelmazar",
    "apelmazaréis": "apelmazar",
    "apelmazaría": "apelmazar",
    "apelmazaríais": "apelmazar",
    "apelmazaríamos": "apelmazar",
    "apelmazarían": "apelmazar",
    "apelmazarías": "apelmazar",
    "apelmazas": "apelmazar",
    "apelmazase": "apelmazar",
    "apelmazaseis": "apelmazar",
    "apelmazasen": "apelmazar",
    "apelmazases": "apelmazar",
    "apelmazaste": "apelmazar",
    "apelmazasteis": "apelmazar",
    "apelmazo": "apelmazar",
    "apelmazábamos": "apelmazar",
    "apelmazáis": "apelmazar",
    "apelmazáramos": "apelmazar",
    "apelmazáremos": "apelmazar",
    "apelmazásemos": "apelmazar",
    "apelmazó": "apelmazar",
    "apelo": "apelar",
    "apelotona": "apelotonar",
    "apelotonaba": "apelotonar",
    "apelotonabais": "apelotonar",
    "apelotonaban": "apelotonar",
    "apelotonabas": "apelotonar",
    "apelotonad": "apelotonar",
    "apelotonada": "apelotonar",
    "apelotonadas": "apelotonar",
    "apelotonado": "apelotonar",
    "apelotonados": "apelotonar",
    "apelotonamientos": "apelotonamiento",
    "apelotonamos": "apelotonar",
    "apelotonan": "apelotonar",
    "apelotonando": "apelotonar",
    "apelotonara": "apelotonar",
    "apelotonarais": "apelotonar",
    "apelotonaran": "apelotonar",
    "apelotonaras": "apelotonar",
    "apelotonare": "apelotonar",
    "apelotonareis": "apelotonar",
    "apelotonaremos": "apelotonar",
    "apelotonaren": "apelotonar",
    "apelotonares": "apelotonar",
    "apelotonaron": "apelotonar",
    "apelotonará": "apelotonar",
    "apelotonarán": "apelotonar",
    "apelotonarás": "apelotonar",
    "apelotonaré": "apelotonar",
    "apelotonaréis": "apelotonar",
    "apelotonaría": "apelotonar",
    "apelotonaríais": "apelotonar",
    "apelotonaríamos": "apelotonar",
    "apelotonarían": "apelotonar",
    "apelotonarías": "apelotonar",
    "apelotonas": "apelotonar",
    "apelotonase": "apelotonar",
    "apelotonaseis": "apelotonar",
    "apelotonasen": "apelotonar",
    "apelotonases": "apelotonar",
    "apelotonaste": "apelotonar",
    "apelotonasteis": "apelotonar",
    "apelotone": "apelotonar",
    "apelotonemos": "apelotonar",
    "apelotonen": "apelotonar",
    "apelotones": "apelotonar",
    "apelotono": "apelotonar",
    "apelotonábamos": "apelotonar",
    "apelotonáis": "apelotonar",
    "apelotonáramos": "apelotonar",
    "apelotonáremos": "apelotonar",
    "apelotonásemos": "apelotonar",
    "apelotoné": "apelotonar",
    "apelotonéis": "apelotonar",
    "apelotonó": "apelotonar",
    "apelábamos": "apelar",
    "apeláis": "apelar",
    "apeláramos": "apelar",
    "apeláremos": "apelar",
    "apelásemos": "apelar",
    "apelé": "apelar",
    "apeléis": "apelar",
    "apeló": "apelar",
    "apena": "apenar",
    "apenaba": "apenar",
    "apenabais": "apenar",
    "apenaban": "apenar",
    "apenabas": "apenar",
    "apenad": "apenar",
    "apenada": "apenar",
    "apenadas": "apenar",
    "apenado": "apenar",
    "apenados": "apenar",
    "apenamientos": "apenamiento",
    "apenamos": "apenar",
    "apenan": "apenar",
    "apenando": "apenar",
    "apenara": "apenar",
    "apenarais": "apenar",
    "apenaran": "apenar",
    "apenaras": "apenar",
    "apenare": "apenar",
    "apenareis": "apenar",
    "apenaremos": "apenar",
    "apenaren": "apenar",
    "apenares": "apenar",
    "apenaron": "apenar",
    "apenará": "apenar",
    "apenarán": "apenar",
    "apenarás": "apenar",
    "apenaré": "apenar",
    "apenaréis": "apenar",
    "apenaría": "apenar",
    "apenaríais": "apenar",
    "apenaríamos": "apenar",
    "apenarían": "apenar",
    "apenarías": "apenar",
    "apenas": "apenar",
    "apenase": "apenar",
    "apenaseis": "apenar",
    "apenasen": "apenar",
    "apenases": "apenar",
    "apenaste": "apenar",
    "apenasteis": "apenar",
    "apenca": "apencar",
    "apencaba": "apencar",
    "apencabais": "apencar",
    "apencaban": "apencar",
    "apencabas": "apencar",
    "apencad": "apencar",
    "apencada": "apencar",
    "apencadas": "apencar",
    "apencado": "apencar",
    "apencados": "apencar",
    "apencamos": "apencar",
    "apencan": "apencar",
    "apencando": "apencar",
    "apencara": "apencar",
    "apencarais": "apencar",
    "apencaran": "apencar",
    "apencaras": "apencar",
    "apencare": "apencar",
    "apencareis": "apencar",
    "apencaremos": "apencar",
    "apencaren": "apencar",
    "apencares": "apencar",
    "apencaron": "apencar",
    "apencará": "apencar",
    "apencarán": "apencar",
    "apencarás": "apencar",
    "apencaré": "apencar",
    "apencaréis": "apencar",
    "apencaría": "apencar",
    "apencaríais": "apencar",
    "apencaríamos": "apencar",
    "apencarían": "apencar",
    "apencarías": "apencar",
    "apencas": "apencar",
    "apencase": "apencar",
    "apencaseis": "apencar",
    "apencasen": "apencar",
    "apencases": "apencar",
    "apencaste": "apencar",
    "apencasteis": "apencar",
    "apenco": "apencar",
    "apencábamos": "apencar",
    "apencáis": "apencar",
    "apencáramos": "apencar",
    "apencáremos": "apencar",
    "apencásemos": "apencar",
    "apencó": "apencar",
    "apendiculares": "apendicular",
    "apene": "apenar",
    "apenemos": "apenar",
    "apenen": "apenar",
    "apenes": "apenar",
    "apeno": "apenar",
    "apenque": "apencar",
    "apenquemos": "apencar",
    "apenquen": "apencar",
    "apenques": "apencar",
    "apenqué": "apencar",
    "apenquéis": "apencar",
    "apenábamos": "apenar",
    "apenáis": "apenar",
    "apenáramos": "apenar",
    "apenáremos": "apenar",
    "apenásemos": "apenar",
    "apené": "apenar",
    "apenéis": "apenar",
    "apenó": "apenar",
    "apeo": "apear",
    "apeona": "apeonar",
    "apeonaba": "apeonar",
    "apeonabais": "apeonar",
    "apeonaban": "apeonar",
    "apeonabas": "apeonar",
    "apeonad": "apeonar",
    "apeonada": "apeonar",
    "apeonadas": "apeonar",
    "apeonado": "apeonar",
    "apeonados": "apeonar",
    "apeonamos": "apeonar",
    "apeonan": "apeonar",
    "apeonando": "apeonar",
    "apeonara": "apeonar",
    "apeonarais": "apeonar",
    "apeonaran": "apeonar",
    "apeonaras": "apeonar",
    "apeonare": "apeonar",
    "apeonareis": "apeonar",
    "apeonaremos": "apeonar",
    "apeonaren": "apeonar",
    "apeonares": "apeonar",
    "apeonaron": "apeonar",
    "apeonará": "apeonar",
    "apeonarán": "apeonar",
    "apeonarás": "apeonar",
    "apeonaré": "apeonar",
    "apeonaréis": "apeonar",
    "apeonaría": "apeonar",
    "apeonaríais": "apeonar",
    "apeonaríamos": "apeonar",
    "apeonarían": "apeonar",
    "apeonarías": "apeonar",
    "apeonas": "apeonar",
    "apeonase": "apeonar",
    "apeonaseis": "apeonar",
    "apeonasen": "apeonar",
    "apeonases": "apeonar",
    "apeonaste": "apeonar",
    "apeonasteis": "apeonar",
    "apeone": "apeonar",
    "apeonemos": "apeonar",
    "apeonen": "apeonar",
    "apeones": "apeonar",
    "apeono": "apeonar",
    "apeonábamos": "apeonar",
    "apeonáis": "apeonar",
    "apeonáramos": "apeonar",
    "apeonáremos": "apeonar",
    "apeonásemos": "apeonar",
    "apeoné": "apeonar",
    "apeonéis": "apeonar",
    "apeonó": "apeonar",
    "apeos": "apeo",
    "apepsias": "apepsia",
    "apera": "aperar",
    "aperaba": "aperar",
    "aperabais": "aperar",
    "aperaban": "aperar",
    "aperabas": "aperar",
    "aperad": "aperar",
    "aperada": "aperar",
    "aperadas": "aperar",
    "aperado": "aperar",
    "aperadores": "aperador",
    "aperados": "aperar",
    "aperamos": "aperar",
    "aperan": "aperar",
    "aperando": "aperar",
    "aperara": "aperar",
    "aperarais": "aperar",
    "aperaran": "aperar",
    "aperaras": "aperar",
    "aperare": "aperar",
    "aperareis": "aperar",
    "aperaremos": "aperar",
    "aperaren": "aperar",
    "aperares": "aperar",
    "aperaron": "aperar",
    "aperará": "aperar",
    "aperarán": "aperar",
    "aperarás": "aperar",
    "aperaré": "aperar",
    "aperaréis": "aperar",
    "aperaría": "aperar",
    "aperaríais": "aperar",
    "aperaríamos": "aperar",
    "aperarían": "aperar",
    "aperarías": "aperar",
    "aperas": "aperar",
    "aperase": "aperar",
    "aperaseis": "aperar",
    "aperasen": "aperar",
    "aperases": "aperar",
    "aperaste": "aperar",
    "aperasteis": "aperar",
    "apercebimientos": "apercebimiento",
    "apercepciones": "apercepción",
    "aperciba": "apercibir",
    "apercibamos": "apercibir",
    "aperciban": "apercibir",
    "apercibas": "apercibir",
    "apercibe": "apercibir",
    "aperciben": "apercibir",
    "apercibes": "apercibir",
    "apercibid": "apercibir",
    "apercibida": "apercibir",
    "apercibidas": "apercibir",
    "apercibido": "apercibir",
    "apercibidos": "apercibir",
    "apercibiendo": "apercibir",
    "apercibiera": "apercibir",
    "apercibierais": "apercibir",
    "apercibieran": "apercibir",
    "apercibieras": "apercibir",
    "apercibiere": "apercibir",
    "apercibiereis": "apercibir",
    "apercibieren": "apercibir",
    "apercibieres": "apercibir",
    "apercibieron": "apercibir",
    "apercibiese": "apercibir",
    "apercibieseis": "apercibir",
    "apercibiesen": "apercibir",
    "apercibieses": "apercibir",
    "apercibimientos": "apercibimiento",
    "apercibimos": "apercibir",
    "apercibiremos": "apercibir",
    "apercibirá": "apercibir",
    "apercibirán": "apercibir",
    "apercibirás": "apercibir",
    "apercibiré": "apercibir",
    "apercibiréis": "apercibir",
    "apercibiría": "apercibir",
    "apercibiríais": "apercibir",
    "apercibiríamos": "apercibir",
    "apercibirían": "apercibir",
    "apercibirías": "apercibir",
    "apercibiste": "apercibir",
    "apercibisteis": "apercibir",
    "apercibiéramos": "apercibir",
    "apercibiéremos": "apercibir",
    "apercibiésemos": "apercibir",
    "apercibió": "apercibir",
    "apercibo": "apercibir",
    "apercibáis": "apercibir",
    "apercibí": "apercibir",
    "apercibía": "apercibir",
    "apercibíais": "apercibir",
    "apercibíamos": "apercibir",
    "apercibían": "apercibir",
    "apercibías": "apercibir",
    "apercibís": "apercibir",
    "apercollaba": "apercollar",
    "apercollabais": "apercollar",
    "apercollaban": "apercollar",
    "apercollabas": "apercollar",
    "apercollad": "apercollar",
    "apercollada": "apercollar",
    "apercolladas": "apercollar",
    "apercollado": "apercollar",
    "apercollados": "apercollar",
    "apercollamos": "apercollar",
    "apercollando": "apercollar",
    "apercollara": "apercollar",
    "apercollarais": "apercollar",
    "apercollaran": "apercollar",
    "apercollaras": "apercollar",
    "apercollare": "apercollar",
    "apercollareis": "apercollar",
    "apercollaremos": "apercollar",
    "apercollaren": "apercollar",
    "apercollares": "apercollar",
    "apercollaron": "apercollar",
    "apercollará": "apercollar",
    "apercollarán": "apercollar",
    "apercollarás": "apercollar",
    "apercollaré": "apercollar",
    "apercollaréis": "apercollar",
    "apercollaría": "apercollar",
    "apercollaríais": "apercollar",
    "apercollaríamos": "apercollar",
    "apercollarían": "apercollar",
    "apercollarías": "apercollar",
    "apercollase": "apercollar",
    "apercollaseis": "apercollar",
    "apercollasen": "apercollar",
    "apercollases": "apercollar",
    "apercollaste": "apercollar",
    "apercollasteis": "apercollar",
    "apercollemos": "apercollar",
    "apercollábamos": "apercollar",
    "apercolláis": "apercollar",
    "apercolláramos": "apercollar",
    "apercolláremos": "apercollar",
    "apercollásemos": "apercollar",
    "apercollé": "apercollar",
    "apercolléis": "apercollar",
    "apercolló": "apercollar",
    "apercuella": "apercollar",
    "apercuellan": "apercollar",
    "apercuellas": "apercollar",
    "apercuelle": "apercollar",
    "apercuellen": "apercollar",
    "apercuelles": "apercollar",
    "apercuello": "apercollar",
    "apere": "aperar",
    "aperemos": "aperar",
    "aperen": "aperar",
    "aperes": "aperar",
    "apergamina": "apergaminar",
    "apergaminaba": "apergaminar",
    "apergaminabais": "apergaminar",
    "apergaminaban": "apergaminar",
    "apergaminabas": "apergaminar",
    "apergaminad": "apergaminar",
    "apergaminada": "apergaminar",
    "apergaminadas": "apergaminar",
    "apergaminado": "apergaminar",
    "apergaminados": "apergaminar",
    "apergaminamos": "apergaminar",
    "apergaminan": "apergaminar",
    "apergaminando": "apergaminar",
    "apergaminara": "apergaminar",
    "apergaminarais": "apergaminar",
    "apergaminaran": "apergaminar",
    "apergaminaras": "apergaminar",
    "apergaminare": "apergaminar",
    "apergaminareis": "apergaminar",
    "apergaminaremos": "apergaminar",
    "apergaminaren": "apergaminar",
    "apergaminares": "apergaminar",
    "apergaminaron": "apergaminar",
    "apergaminará": "apergaminar",
    "apergaminarán": "apergaminar",
    "apergaminarás": "apergaminar",
    "apergaminaré": "apergaminar",
    "apergaminaréis": "apergaminar",
    "apergaminaría": "apergaminar",
    "apergaminaríais": "apergaminar",
    "apergaminaríamos": "apergaminar",
    "apergaminarían": "apergaminar",
    "apergaminarías": "apergaminar",
    "apergaminas": "apergaminar",
    "apergaminase": "apergaminar",
    "apergaminaseis": "apergaminar",
    "apergaminasen": "apergaminar",
    "apergaminases": "apergaminar",
    "apergaminaste": "apergaminar",
    "apergaminasteis": "apergaminar",
    "apergamine": "apergaminar",
    "apergaminemos": "apergaminar",
    "apergaminen": "apergaminar",
    "apergamines": "apergaminar",
    "apergamino": "apergaminar",
    "apergaminábamos": "apergaminar",
    "apergamináis": "apergaminar",
    "apergamináramos": "apergaminar",
    "apergamináremos": "apergaminar",
    "apergaminásemos": "apergaminar",
    "apergaminé": "apergaminar",
    "apergaminéis": "apergaminar",
    "apergaminó": "apergaminar",
    "aperitiva": "aperitivo",
    "aperitivas": "aperitivo",
    "aperitivos": "aperitivo",
    "aperiódica": "aperiódico",
    "aperiódicas": "aperiódico",
    "aperiódicos": "aperiódico",
    "apernaba": "apernar",
    "apernabais": "apernar",
    "apernaban": "apernar",
    "apernabas": "apernar",
    "apernad": "apernar",
    "apernada": "apernar",
    "apernadas": "apernar",
    "apernado": "apernar",
    "apernadora": "apernador",
    "apernadoras": "apernador",
    "apernadores": "apernador",
    "apernados": "apernar",
    "apernamos": "apernar",
    "apernando": "apernar",
    "apernara": "apernar",
    "apernarais": "apernar",
    "apernaran": "apernar",
    "apernaras": "apernar",
    "apernare": "apernar",
    "apernareis": "apernar",
    "apernaremos": "apernar",
    "apernaren": "apernar",
    "apernares": "apernar",
    "apernaron": "apernar",
    "apernará": "apernar",
    "apernarán": "apernar",
    "apernarás": "apernar",
    "apernaré": "apernar",
    "apernaréis": "apernar",
    "apernaría": "apernar",
    "apernaríais": "apernar",
    "apernaríamos": "apernar",
    "apernarían": "apernar",
    "apernarías": "apernar",
    "apernase": "apernar",
    "apernaseis": "apernar",
    "apernasen": "apernar",
    "apernases": "apernar",
    "apernaste": "apernar",
    "apernasteis": "apernar",
    "apernemos": "apernar",
    "apernábamos": "apernar",
    "apernáis": "apernar",
    "apernáramos": "apernar",
    "apernáremos": "apernar",
    "apernásemos": "apernar",
    "aperné": "apernar",
    "apernéis": "apernar",
    "apernó": "apernar",
    "apero": "aperar",
    "aperos": "apero",
    "aperrea": "aperrear",
    "aperreaba": "aperrear",
    "aperreabais": "aperrear",
    "aperreaban": "aperrear",
    "aperreabas": "aperrear",
    "aperread": "aperrear",
    "aperreada": "aperrear",
    "aperreadas": "aperrear",
    "aperreado": "aperrear",
    "aperreados": "aperrear",
    "aperreamos": "aperrear",
    "aperrean": "aperrear",
    "aperreando": "aperrear",
    "aperreara": "aperrear",
    "aperrearais": "aperrear",
    "aperrearan": "aperrear",
    "aperrearas": "aperrear",
    "aperreare": "aperrear",
    "aperreareis": "aperrear",
    "aperrearemos": "aperrear",
    "aperrearen": "aperrear",
    "aperreares": "aperrear",
    "aperrearon": "aperrear",
    "aperreará": "aperrear",
    "aperrearán": "aperrear",
    "aperrearás": "aperrear",
    "aperrearé": "aperrear",
    "aperrearéis": "aperrear",
    "aperrearía": "aperrear",
    "aperrearíais": "aperrear",
    "aperrearíamos": "aperrear",
    "aperrearían": "aperrear",
    "aperrearías": "aperrear",
    "aperreas": "aperrear",
    "aperrease": "aperrear",
    "aperreaseis": "aperrear",
    "aperreasen": "aperrear",
    "aperreases": "aperrear",
    "aperreaste": "aperrear",
    "aperreasteis": "aperrear",
    "aperree": "aperrear",
    "aperreemos": "aperrear",
    "aperreen": "aperrear",
    "aperrees": "aperrear",
    "aperreo": "aperrear",
    "aperreos": "aperreo",
    "aperreábamos": "aperrear",
    "aperreáis": "aperrear",
    "aperreáramos": "aperrear",
    "aperreáremos": "aperrear",
    "aperreásemos": "aperrear",
    "aperreé": "aperrear",
    "aperreéis": "aperrear",
    "aperreó": "aperrear",
    "apersona": "apersonar",
    "apersonaba": "apersonar",
    "apersonabais": "apersonar",
    "apersonaban": "apersonar",
    "apersonabas": "apersonar",
    "apersonad": "apersonar",
    "apersonada": "apersonar",
    "apersonadas": "apersonar",
    "apersonado": "apersonar",
    "apersonados": "apersonar",
    "apersonamos": "apersonar",
    "apersonan": "apersonar",
    "apersonando": "apersonar",
    "apersonara": "apersonar",
    "apersonarais": "apersonar",
    "apersonaran": "apersonar",
    "apersonaras": "apersonar",
    "apersonare": "apersonar",
    "apersonareis": "apersonar",
    "apersonaremos": "apersonar",
    "apersonaren": "apersonar",
    "apersonares": "apersonar",
    "apersonaron": "apersonar",
    "apersonará": "apersonar",
    "apersonarán": "apersonar",
    "apersonarás": "apersonar",
    "apersonaré": "apersonar",
    "apersonaréis": "apersonar",
    "apersonaría": "apersonar",
    "apersonaríais": "apersonar",
    "apersonaríamos": "apersonar",
    "apersonarían": "apersonar",
    "apersonarías": "apersonar",
    "apersonas": "apersonar",
    "apersonase": "apersonar",
    "apersonaseis": "apersonar",
    "apersonasen": "apersonar",
    "apersonases": "apersonar",
    "apersonaste": "apersonar",
    "apersonasteis": "apersonar",
    "apersone": "apersonar",
    "apersonemos": "apersonar",
    "apersonen": "apersonar",
    "apersones": "apersonar",
    "apersono": "apersonar",
    "apersonábamos": "apersonar",
    "apersonáis": "apersonar",
    "apersonáramos": "apersonar",
    "apersonáremos": "apersonar",
    "apersonásemos": "apersonar",
    "apersoné": "apersonar",
    "apersonéis": "apersonar",
    "apersonó": "apersonar",
    "aperturas": "apertura",
    "aperturistas": "aperturista",
    "aperábamos": "aperar",
    "aperáis": "aperar",
    "aperáramos": "aperar",
    "aperáremos": "aperar",
    "aperásemos": "aperar",
    "aperé": "aperar",
    "aperéis": "aperar",
    "aperó": "aperar",
    "apesadumbra": "apesadumbrar",
    "apesadumbraba": "apesadumbrar",
    "apesadumbrabais": "apesadumbrar",
    "apesadumbraban": "apesadumbrar",
    "apesadumbrabas": "apesadumbrar",
    "apesadumbrad": "apesadumbrar",
    "apesadumbrada": "apesadumbrar",
    "apesadumbradas": "apesadumbrar",
    "apesadumbrado": "apesadumbrar",
    "apesadumbrados": "apesadumbrar",
    "apesadumbramientos": "apesadumbramiento",
    "apesadumbramos": "apesadumbrar",
    "apesadumbran": "apesadumbrar",
    "apesadumbrando": "apesadumbrar",
    "apesadumbrara": "apesadumbrar",
    "apesadumbrarais": "apesadumbrar",
    "apesadumbraran": "apesadumbrar",
    "apesadumbraras": "apesadumbrar",
    "apesadumbrare": "apesadumbrar",
    "apesadumbrareis": "apesadumbrar",
    "apesadumbraremos": "apesadumbrar",
    "apesadumbraren": "apesadumbrar",
    "apesadumbrares": "apesadumbrar",
    "apesadumbraron": "apesadumbrar",
    "apesadumbrará": "apesadumbrar",
    "apesadumbrarán": "apesadumbrar",
    "apesadumbrarás": "apesadumbrar",
    "apesadumbraré": "apesadumbrar",
    "apesadumbraréis": "apesadumbrar",
    "apesadumbraría": "apesadumbrar",
    "apesadumbraríais": "apesadumbrar",
    "apesadumbraríamos": "apesadumbrar",
    "apesadumbrarían": "apesadumbrar",
    "apesadumbrarías": "apesadumbrar",
    "apesadumbras": "apesadumbrar",
    "apesadumbrase": "apesadumbrar",
    "apesadumbraseis": "apesadumbrar",
    "apesadumbrasen": "apesadumbrar",
    "apesadumbrases": "apesadumbrar",
    "apesadumbraste": "apesadumbrar",
    "apesadumbrasteis": "apesadumbrar",
    "apesadumbre": "apesadumbrar",
    "apesadumbremos": "apesadumbrar",
    "apesadumbren": "apesadumbrar",
    "apesadumbres": "apesadumbrar",
    "apesadumbro": "apesadumbrar",
    "apesadumbrábamos": "apesadumbrar",
    "apesadumbráis": "apesadumbrar",
    "apesadumbráramos": "apesadumbrar",
    "apesadumbráremos": "apesadumbrar",
    "apesadumbrásemos": "apesadumbrar",
    "apesadumbré": "apesadumbrar",
    "apesadumbréis": "apesadumbrar",
    "apesadumbró": "apesadumbrar",
    "apesara": "apesarar",
    "apesaraba": "apesarar",
    "apesarabais": "apesarar",
    "apesaraban": "apesarar",
    "apesarabas": "apesarar",
    "apesarad": "apesarar",
    "apesarada": "apesarar",
    "apesaradas": "apesarar",
    "apesarado": "apesarar",
    "apesarados": "apesarar",
    "apesaramos": "apesarar",
    "apesaran": "apesarar",
    "apesarando": "apesarar",
    "apesarara": "apesarar",
    "apesararais": "apesarar",
    "apesararan": "apesarar",
    "apesararas": "apesarar",
    "apesarare": "apesarar",
    "apesarareis": "apesarar",
    "apesararemos": "apesarar",
    "apesararen": "apesarar",
    "apesarares": "apesarar",
    "apesararon": "apesarar",
    "apesarará": "apesarar",
    "apesararán": "apesarar",
    "apesararás": "apesarar",
    "apesararé": "apesarar",
    "apesararéis": "apesarar",
    "apesararía": "apesarar",
    "apesararíais": "apesarar",
    "apesararíamos": "apesarar",
    "apesararían": "apesarar",
    "apesararías": "apesarar",
    "apesaras": "apesarar",
    "apesarase": "apesarar",
    "apesaraseis": "apesarar",
    "apesarasen": "apesarar",
    "apesarases": "apesarar",
    "apesaraste": "apesarar",
    "apesarasteis": "apesarar",
    "apesare": "apesarar",
    "apesaremos": "apesarar",
    "apesaren": "apesarar",
    "apesares": "apesarar",
    "apesaro": "apesarar",
    "apesarábamos": "apesarar",
    "apesaráis": "apesarar",
    "apesaráramos": "apesarar",
    "apesaráremos": "apesarar",
    "apesarásemos": "apesarar",
    "apesaré": "apesarar",
    "apesaréis": "apesarar",
    "apesaró": "apesarar",
    "apesga": "apesgar",
    "apesgaba": "apesgar",
    "apesgabais": "apesgar",
    "apesgaban": "apesgar",
    "apesgabas": "apesgar",
    "apesgad": "apesgar",
    "apesgada": "apesgar",
    "apesgadas": "apesgar",
    "apesgado": "apesgar",
    "apesgados": "apesgar",
    "apesgamos": "apesgar",
    "apesgan": "apesgar",
    "apesgando": "apesgar",
    "apesgara": "apesgar",
    "apesgarais": "apesgar",
    "apesgaran": "apesgar",
    "apesgaras": "apesgar",
    "apesgare": "apesgar",
    "apesgareis": "apesgar",
    "apesgaremos": "apesgar",
    "apesgaren": "apesgar",
    "apesgares": "apesgar",
    "apesgaron": "apesgar",
    "apesgará": "apesgar",
    "apesgarán": "apesgar",
    "apesgarás": "apesgar",
    "apesgaré": "apesgar",
    "apesgaréis": "apesgar",
    "apesgaría": "apesgar",
    "apesgaríais": "apesgar",
    "apesgaríamos": "apesgar",
    "apesgarían": "apesgar",
    "apesgarías": "apesgar",
    "apesgas": "apesgar",
    "apesgase": "apesgar",
    "apesgaseis": "apesgar",
    "apesgasen": "apesgar",
    "apesgases": "apesgar",
    "apesgaste": "apesgar",
    "apesgasteis": "apesgar",
    "apesgo": "apesgar",
    "apesgue": "apesgar",
    "apesguemos": "apesgar",
    "apesguen": "apesgar",
    "apesgues": "apesgar",
    "apesgué": "apesgar",
    "apesguéis": "apesgar",
    "apesgábamos": "apesgar",
    "apesgáis": "apesgar",
    "apesgáramos": "apesgar",
    "apesgáremos": "apesgar",
    "apesgásemos": "apesgar",
    "apesgó": "apesgar",
    "apesta": "apestar",
    "apestaba": "apestar",
    "apestabais": "apestar",
    "apestaban": "apestar",
    "apestabas": "apestar",
    "apestad": "apestar",
    "apestada": "apestar",
    "apestadas": "apestar",
    "apestado": "apestar",
    "apestados": "apestar",
    "apestamientos": "apestamiento",
    "apestamos": "apestar",
    "apestan": "apestar",
    "apestando": "apestar",
    "apestara": "apestar",
    "apestarais": "apestar",
    "apestaran": "apestar",
    "apestaras": "apestar",
    "apestare": "apestar",
    "apestareis": "apestar",
    "apestaremos": "apestar",
    "apestaren": "apestar",
    "apestares": "apestar",
    "apestaron": "apestar",
    "apestará": "apestar",
    "apestarán": "apestar",
    "apestarás": "apestar",
    "apestaré": "apestar",
    "apestaréis": "apestar",
    "apestaría": "apestar",
    "apestaríais": "apestar",
    "apestaríamos": "apestar",
    "apestarían": "apestar",
    "apestarías": "apestar",
    "apestas": "apestar",
    "apestase": "apestar",
    "apestaseis": "apestar",
    "apestasen": "apestar",
    "apestases": "apestar",
    "apestaste": "apestar",
    "apestasteis": "apestar",
    "apeste": "apestar",
    "apestemos": "apestar",
    "apesten": "apestar",
    "apestes": "apestar",
    "apesto": "apestar",
    "apestosa": "apestoso",
    "apestosas": "apestoso",
    "apestosos": "apestoso",
    "apestábamos": "apestar",
    "apestáis": "apestar",
    "apestáramos": "apestar",
    "apestáremos": "apestar",
    "apestásemos": "apestar",
    "apesté": "apestar",
    "apestéis": "apestar",
    "apestó": "apestar",
    "apetece": "apetecer",
    "apeteced": "apetecer",
    "apetecedora": "apetecedor",
    "apetecedoras": "apetecedor",
    "apetecedores": "apetecedor",
    "apetecemos": "apetecer",
    "apetecen": "apetecer",
    "apeteceremos": "apetecer",
    "apetecerá": "apetecer",
    "apetecerán": "apetecer",
    "apetecerás": "apetecer",
    "apeteceré": "apetecer",
    "apeteceréis": "apetecer",
    "apetecería": "apetecer",
    "apeteceríais": "apetecer",
    "apeteceríamos": "apetecer",
    "apetecerían": "apetecer",
    "apetecerías": "apetecer",
    "apeteces": "apetecer",
    "apetecibilidades": "apetecibilidad",
    "apetecibles": "apetecible",
    "apetecida": "apetecer",
    "apetecidas": "apetecer",
    "apetecido": "apetecer",
    "apetecidos": "apetecer",
    "apeteciendo": "apetecer",
    "apeteciera": "apetecer",
    "apetecierais": "apetecer",
    "apetecieran": "apetecer",
    "apetecieras": "apetecer",
    "apeteciere": "apetecer",
    "apeteciereis": "apetecer",
    "apetecieren": "apetecer",
    "apetecieres": "apetecer",
    "apetecieron": "apetecer",
    "apeteciese": "apetecer",
    "apetecieseis": "apetecer",
    "apeteciesen": "apetecer",
    "apetecieses": "apetecer",
    "apetecimos": "apetecer",
    "apeteciste": "apetecer",
    "apetecisteis": "apetecer",
    "apeteciéramos": "apetecer",
    "apeteciéremos": "apetecer",
    "apeteciésemos": "apetecer",
    "apeteció": "apetecer",
    "apetecéis": "apetecer",
    "apetecí": "apetecer",
    "apetecía": "apetecer",
    "apetecíais": "apetecer",
    "apetecíamos": "apetecer",
    "apetecían": "apetecer",
    "apetecías": "apetecer",
    "apetencias": "apetencia",
    "apetezca": "apetecer",
    "apetezcamos": "apetecer",
    "apetezcan": "apetecer",
    "apetezcas": "apetecer",
    "apetezco": "apetecer",
    "apetezcáis": "apetecer",
    "apetitiva": "apetitivo",
    "apetitivas": "apetitivo",
    "apetitivos": "apetitivo",
    "apetitos": "apetito",
    "apetitosa": "apetitoso",
    "apetitosas": "apetitoso",
    "apetitosidades": "apetitosidad",
    "apetitosos": "apetitoso",
    "apeábamos": "apear",
    "apeáis": "apear",
    "apeáramos": "apear",
    "apeáremos": "apear",
    "apeásemos": "apear",
    "apeé": "apear",
    "apeéis": "apear",
    "apeñusca": "apeñuscar",
    "apeñuscaba": "apeñuscar",
    "apeñuscabais": "apeñuscar",
    "apeñuscaban": "apeñuscar",
    "apeñuscabas": "apeñuscar",
    "apeñuscad": "apeñuscar",
    "apeñuscada": "apeñuscar",
    "apeñuscadas": "apeñuscar",
    "apeñuscado": "apeñuscar",
    "apeñuscados": "apeñuscar",
    "apeñuscamos": "apeñuscar",
    "apeñuscan": "apeñuscar",
    "apeñuscando": "apeñuscar",
    "apeñuscara": "apeñuscar",
    "apeñuscarais": "apeñuscar",
    "apeñuscaran": "apeñuscar",
    "apeñuscaras": "apeñuscar",
    "apeñuscare": "apeñuscar",
    "apeñuscareis": "apeñuscar",
    "apeñuscaremos": "apeñuscar",
    "apeñuscaren": "apeñuscar",
    "apeñuscares": "apeñuscar",
    "apeñuscaron": "apeñuscar",
    "apeñuscará": "apeñuscar",
    "apeñuscarán": "apeñuscar",
    "apeñuscarás": "apeñuscar",
    "apeñuscaré": "apeñuscar",
    "apeñuscaréis": "apeñuscar",
    "apeñuscaría": "apeñuscar",
    "apeñuscaríais": "apeñuscar",
    "apeñuscaríamos": "apeñuscar",
    "apeñuscarían": "apeñuscar",
    "apeñuscarías": "apeñuscar",
    "apeñuscas": "apeñuscar",
    "apeñuscase": "apeñuscar",
    "apeñuscaseis": "apeñuscar",
    "apeñuscasen": "apeñuscar",
    "apeñuscases": "apeñuscar",
    "apeñuscaste": "apeñuscar",
    "apeñuscasteis": "apeñuscar",
    "apeñusco": "apeñuscar",
    "apeñuscábamos": "apeñuscar",
    "apeñuscáis": "apeñuscar",
    "apeñuscáramos": "apeñuscar",
    "apeñuscáremos": "apeñuscar",
    "apeñuscásemos": "apeñuscar",
    "apeñuscó": "apeñuscar",
    "apeñusque": "apeñuscar",
    "apeñusquemos": "apeñuscar",
    "apeñusquen": "apeñuscar",
    "apeñusques": "apeñuscar",
    "apeñusqué": "apeñuscar",
    "apeñusquéis": "apeñuscar",
    "apeó": "apear",
    "apiada": "apiadar",
    "apiadaba": "apiadar",
    "apiadabais": "apiadar",
    "apiadaban": "apiadar",
    "apiadabas": "apiadar",
    "apiadaciones": "apiadación",
    "apiadad": "apiadar",
    "apiadada": "apiadar",
    "apiadadas": "apiadar",
    "apiadado": "apiadar",
    "apiadados": "apiadar",
    "apiadamos": "apiadar",
    "apiadan": "apiadar",
    "apiadando": "apiadar",
    "apiadara": "apiadar",
    "apiadarais": "apiadar",
    "apiadaran": "apiadar",
    "apiadaras": "apiadar",
    "apiadare": "apiadar",
    "apiadareis": "apiadar",
    "apiadaremos": "apiadar",
    "apiadaren": "apiadar",
    "apiadares": "apiadar",
    "apiadaron": "apiadar",
    "apiadará": "apiadar",
    "apiadarán": "apiadar",
    "apiadarás": "apiadar",
    "apiadaré": "apiadar",
    "apiadaréis": "apiadar",
    "apiadaría": "apiadar",
    "apiadaríais": "apiadar",
    "apiadaríamos": "apiadar",
    "apiadarían": "apiadar",
    "apiadarías": "apiadar",
    "apiadas": "apiadar",
    "apiadase": "apiadar",
    "apiadaseis": "apiadar",
    "apiadasen": "apiadar",
    "apiadases": "apiadar",
    "apiadaste": "apiadar",
    "apiadasteis": "apiadar",
    "apiade": "apiadar",
    "apiademos": "apiadar",
    "apiaden": "apiadar",
    "apiades": "apiadar",
    "apiado": "apiadar",
    "apiadábamos": "apiadar",
    "apiadáis": "apiadar",
    "apiadáramos": "apiadar",
    "apiadáremos": "apiadar",
    "apiadásemos": "apiadar",
    "apiadé": "apiadar",
    "apiadéis": "apiadar",
    "apiadó": "apiadar",
    "apiana": "apianar",
    "apianaba": "apianar",
    "apianabais": "apianar",
    "apianaban": "apianar",
    "apianabas": "apianar",
    "apianad": "apianar",
    "apianada": "apianar",
    "apianadas": "apianar",
    "apianado": "apianar",
    "apianados": "apianar",
    "apianamos": "apianar",
    "apianan": "apianar",
    "apianando": "apianar",
    "apianara": "apianar",
    "apianarais": "apianar",
    "apianaran": "apianar",
    "apianaras": "apianar",
    "apianare": "apianar",
    "apianareis": "apianar",
    "apianaremos": "apianar",
    "apianaren": "apianar",
    "apianares": "apianar",
    "apianaron": "apianar",
    "apianará": "apianar",
    "apianarán": "apianar",
    "apianarás": "apianar",
    "apianaré": "apianar",
    "apianaréis": "apianar",
    "apianaría": "apianar",
    "apianaríais": "apianar",
    "apianaríamos": "apianar",
    "apianarían": "apianar",
    "apianarías": "apianar",
    "apianas": "apianar",
    "apianase": "apianar",
    "apianaseis": "apianar",
    "apianasen": "apianar",
    "apianases": "apianar",
    "apianaste": "apianar",
    "apianasteis": "apianar",
    "apiane": "apianar",
    "apianemos": "apianar",
    "apianen": "apianar",
    "apianes": "apianar",
    "apiano": "apianar",
    "apianábamos": "apianar",
    "apianáis": "apianar",
    "apianáramos": "apianar",
    "apianáremos": "apianar",
    "apianásemos": "apianar",
    "apiané": "apianar",
    "apianéis": "apianar",
    "apianó": "apianar",
    "apiaraderos": "apiaradero",
    "apicales": "apical",
    "apicara": "apicarar",
    "apicaraba": "apicarar",
    "apicarabais": "apicarar",
    "apicaraban": "apicarar",
    "apicarabas": "apicarar",
    "apicarad": "apicarar",
    "apicarada": "apicarar",
    "apicaradas": "apicarar",
    "apicarado": "apicarar",
    "apicarados": "apicarar",
    "apicaramos": "apicarar",
    "apicaran": "apicarar",
    "apicarando": "apicarar",
    "apicarara": "apicarar",
    "apicararais": "apicarar",
    "apicararan": "apicarar",
    "apicararas": "apicarar",
    "apicarare": "apicarar",
    "apicarareis": "apicarar",
    "apicararemos": "apicarar",
    "apicararen": "apicarar",
    "apicarares": "apicarar",
    "apicararon": "apicarar",
    "apicarará": "apicarar",
    "apicararán": "apicarar",
    "apicararás": "apicarar",
    "apicararé": "apicarar",
    "apicararéis": "apicarar",
    "apicararía": "apicarar",
    "apicararíais": "apicarar",
    "apicararíamos": "apicarar",
    "apicararían": "apicarar",
    "apicararías": "apicarar",
    "apicaras": "apicarar",
    "apicarase": "apicarar",
    "apicaraseis": "apicarar",
    "apicarasen": "apicarar",
    "apicarases": "apicarar",
    "apicaraste": "apicarar",
    "apicarasteis": "apicarar",
    "apicare": "apicarar",
    "apicaremos": "apicarar",
    "apicaren": "apicarar",
    "apicares": "apicarar",
    "apicaro": "apicarar",
    "apicarábamos": "apicarar",
    "apicaráis": "apicarar",
    "apicaráramos": "apicarar",
    "apicaráremos": "apicarar",
    "apicarásemos": "apicarar",
    "apicaré": "apicarar",
    "apicaréis": "apicarar",
    "apicaró": "apicarar",
    "apicoalveolares": "apicoalveolar",
    "apicodentales": "apicodental",
    "apicultora": "apicultor",
    "apicultoras": "apicultor",
    "apicultores": "apicultor",
    "apiculturas": "apicultura",
    "apierna": "apernar",
    "apiernan": "apernar",
    "apiernas": "apernar",
    "apierne": "apernar",
    "apiernen": "apernar",
    "apiernes": "apernar",
    "apierno": "apernar",
    "apila": "apilar",
    "apilaba": "apilar",
    "apilabais": "apilar",
    "apilaban": "apilar",
    "apilabas": "apilar",
    "apilabilidades": "apilabilidad",
    "apilables": "apilable",
    "apilad": "apilar",
    "apilada": "apilar",
    "apiladas": "apilar",
    "apilado": "apilar",
    "apiladoras": "apiladora",
    "apiladores": "apilador",
    "apilados": "apilar",
    "apilamientos": "apilamiento",
    "apilamos": "apilar",
    "apilan": "apilar",
    "apilando": "apilar",
    "apilara": "apilar",
    "apilarais": "apilar",
    "apilaran": "apilar",
    "apilaras": "apilar",
    "apilare": "apilar",
    "apilareis": "apilar",
    "apilaremos": "apilar",
    "apilaren": "apilar",
    "apilares": "apilar",
    "apilaron": "apilar",
    "apilará": "apilar",
    "apilarán": "apilar",
    "apilarás": "apilar",
    "apilaré": "apilar",
    "apilaréis": "apilar",
    "apilaría": "apilar",
    "apilaríais": "apilar",
    "apilaríamos": "apilar",
    "apilarían": "apilar",
    "apilarías": "apilar",
    "apilas": "apilar",
    "apilase": "apilar",
    "apilaseis": "apilar",
    "apilasen": "apilar",
    "apilases": "apilar",
    "apilaste": "apilar",
    "apilasteis": "apilar",
    "apile": "apilar",
    "apilemos": "apilar",
    "apilen": "apilar",
    "apiles": "apilar",
    "apilgua": "apilguar",
    "apilguaba": "apilguar",
    "apilguabais": "apilguar",
    "apilguaban": "apilguar",
    "apilguabas": "apilguar",
    "apilguad": "apilguar",
    "apilguada": "apilguar",
    "apilguadas": "apilguar",
    "apilguado": "apilguar",
    "apilguados": "apilguar",
    "apilguamos": "apilguar",
    "apilguan": "apilguar",
    "apilguando": "apilguar",
    "apilguara": "apilguar",
    "apilguarais": "apilguar",
    "apilguaran": "apilguar",
    "apilguaras": "apilguar",
    "apilguare": "apilguar",
    "apilguareis": "apilguar",
    "apilguaremos": "apilguar",
    "apilguaren": "apilguar",
    "apilguares": "apilguar",
    "apilguaron": "apilguar",
    "apilguará": "apilguar",
    "apilguarán": "apilguar",
    "apilguarás": "apilguar",
    "apilguaré": "apilguar",
    "apilguaréis": "apilguar",
    "apilguaría": "apilguar",
    "apilguaríais": "apilguar",
    "apilguaríamos": "apilguar",
    "apilguarían": "apilguar",
    "apilguarías": "apilguar",
    "apilguas": "apilguar",
    "apilguase": "apilguar",
    "apilguaseis": "apilguar",
    "apilguasen": "apilguar",
    "apilguases": "apilguar",
    "apilguaste": "apilguar",
    "apilguasteis": "apilguar",
    "apilguo": "apilguar",
    "apilguábamos": "apilguar",
    "apilguáis": "apilguar",
    "apilguáramos": "apilguar",
    "apilguáremos": "apilguar",
    "apilguásemos": "apilguar",
    "apilguó": "apilguar",
    "apilgüe": "apilguar",
    "apilgüemos": "apilguar",
    "apilgüen": "apilguar",
    "apilgües": "apilguar",
    "apilgüé": "apilguar",
    "apilgüéis": "apilguar",
    "apilo": "apilar",
    "apilábamos": "apilar",
    "apiláis": "apilar",
    "apiláramos": "apilar",
    "apiláremos": "apilar",
    "apilásemos": "apilar",
    "apilé": "apilar",
    "apiléis": "apilar",
    "apiló": "apilar",
    "apimpolla": "apimpollar",
    "apimpollaba": "apimpollar",
    "apimpollabais": "apimpollar",
    "apimpollaban": "apimpollar",
    "apimpollabas": "apimpollar",
    "apimpollad": "apimpollar",
    "apimpollada": "apimpollar",
    "apimpolladas": "apimpollar",
    "apimpollado": "apimpollar",
    "apimpollados": "apimpollar",
    "apimpollamos": "apimpollar",
    "apimpollan": "apimpollar",
    "apimpollando": "apimpollar",
    "apimpollara": "apimpollar",
    "apimpollarais": "apimpollar",
    "apimpollaran": "apimpollar",
    "apimpollaras": "apimpollar",
    "apimpollare": "apimpollar",
    "apimpollareis": "apimpollar",
    "apimpollaremos": "apimpollar",
    "apimpollaren": "apimpollar",
    "apimpollares": "apimpollar",
    "apimpollaron": "apimpollar",
    "apimpollará": "apimpollar",
    "apimpollarán": "apimpollar",
    "apimpollarás": "apimpollar",
    "apimpollaré": "apimpollar",
    "apimpollaréis": "apimpollar",
    "apimpollaría": "apimpollar",
    "apimpollaríais": "apimpollar",
    "apimpollaríamos": "apimpollar",
    "apimpollarían": "apimpollar",
    "apimpollarías": "apimpollar",
    "apimpollas": "apimpollar",
    "apimpollase": "apimpollar",
    "apimpollaseis": "apimpollar",
    "apimpollasen": "apimpollar",
    "apimpollases": "apimpollar",
    "apimpollaste": "apimpollar",
    "apimpollasteis": "apimpollar",
    "apimpolle": "apimpollar",
    "apimpollemos": "apimpollar",
    "apimpollen": "apimpollar",
    "apimpolles": "apimpollar",
    "apimpollo": "apimpollar",
    "apimpollábamos": "apimpollar",
    "apimpolláis": "apimpollar",
    "apimpolláramos": "apimpollar",
    "apimpolláremos": "apimpollar",
    "apimpollásemos": "apimpollar",
    "apimpollé": "apimpollar",
    "apimpolléis": "apimpollar",
    "apimpolló": "apimpollar",
    "apiola": "apiolar",
    "apiolaba": "apiolar",
    "apiolabais": "apiolar",
    "apiolaban": "apiolar",
    "apiolabas": "apiolar",
    "apiolad": "apiolar",
    "apiolada": "apiolar",
    "apioladas": "apiolar",
    "apiolado": "apiolar",
    "apiolados": "apiolar",
    "apiolamos": "apiolar",
    "apiolan": "apiolar",
    "apiolando": "apiolar",
    "apiolara": "apiolar",
    "apiolarais": "apiolar",
    "apiolaran": "apiolar",
    "apiolaras": "apiolar",
    "apiolare": "apiolar",
    "apiolareis": "apiolar",
    "apiolaremos": "apiolar",
    "apiolaren": "apiolar",
    "apiolares": "apiolar",
    "apiolaron": "apiolar",
    "apiolará": "apiolar",
    "apiolarán": "apiolar",
    "apiolarás": "apiolar",
    "apiolaré": "apiolar",
    "apiolaréis": "apiolar",
    "apiolaría": "apiolar",
    "apiolaríais": "apiolar",
    "apiolaríamos": "apiolar",
    "apiolarían": "apiolar",
    "apiolarías": "apiolar",
    "apiolas": "apiolar",
    "apiolase": "apiolar",
    "apiolaseis": "apiolar",
    "apiolasen": "apiolar",
    "apiolases": "apiolar",
    "apiolaste": "apiolar",
    "apiolasteis": "apiolar",
    "apiole": "apiolar",
    "apiolemos": "apiolar",
    "apiolen": "apiolar",
    "apioles": "apiolar",
    "apiolo": "apiolar",
    "apiolábamos": "apiolar",
    "apioláis": "apiolar",
    "apioláramos": "apiolar",
    "apioláremos": "apiolar",
    "apiolásemos": "apiolar",
    "apiolé": "apiolar",
    "apioléis": "apiolar",
    "apioló": "apiolar",
    "apios": "apio",
    "apipa": "apipar",
    "apipaba": "apipar",
    "apipabais": "apipar",
    "apipaban": "apipar",
    "apipabas": "apipar",
    "apipad": "apipar",
    "apipada": "apipar",
    "apipadas": "apipar",
    "apipado": "apipar",
    "apipados": "apipar",
    "apipamos": "apipar",
    "apipan": "apipar",
    "apipando": "apipar",
    "apipara": "apipar",
    "apiparais": "apipar",
    "apiparan": "apipar",
    "apiparas": "apipar",
    "apipare": "apipar",
    "apipareis": "apipar",
    "apiparemos": "apipar",
    "apiparen": "apipar",
    "apipares": "apipar",
    "apiparon": "apipar",
    "apipará": "apipar",
    "apiparán": "apipar",
    "apiparás": "apipar",
    "apiparé": "apipar",
    "apiparéis": "apipar",
    "apiparía": "apipar",
    "apiparíais": "apipar",
    "apiparíamos": "apipar",
    "apiparían": "apipar",
    "apiparías": "apipar",
    "apipas": "apipar",
    "apipase": "apipar",
    "apipaseis": "apipar",
    "apipasen": "apipar",
    "apipases": "apipar",
    "apipaste": "apipar",
    "apipasteis": "apipar",
    "apipe": "apipar",
    "apipemos": "apipar",
    "apipen": "apipar",
    "apipes": "apipar",
    "apipo": "apipar",
    "apipábamos": "apipar",
    "apipáis": "apipar",
    "apipáramos": "apipar",
    "apipáremos": "apipar",
    "apipásemos": "apipar",
    "apipé": "apipar",
    "apipéis": "apipar",
    "apipó": "apipar",
    "apires": "apir",
    "apirexias": "apirexia",
    "apirgüina": "apirgüinar",
    "apirgüinaba": "apirgüinar",
    "apirgüinabais": "apirgüinar",
    "apirgüinaban": "apirgüinar",
    "apirgüinabas": "apirgüinar",
    "apirgüinad": "apirgüinar",
    "apirgüinada": "apirgüinar",
    "apirgüinadas": "apirgüinar",
    "apirgüinado": "apirgüinar",
    "apirgüinados": "apirgüinar",
    "apirgüinamos": "apirgüinar",
    "apirgüinan": "apirgüinar",
    "apirgüinando": "apirgüinar",
    "apirgüinara": "apirgüinar",
    "apirgüinarais": "apirgüinar",
    "apirgüinaran": "apirgüinar",
    "apirgüinaras": "apirgüinar",
    "apirgüinare": "apirgüinar",
    "apirgüinareis": "apirgüinar",
    "apirgüinaremos": "apirgüinar",
    "apirgüinaren": "apirgüinar",
    "apirgüinares": "apirgüinar",
    "apirgüinaron": "apirgüinar",
    "apirgüinará": "apirgüinar",
    "apirgüinarán": "apirgüinar",
    "apirgüinarás": "apirgüinar",
    "apirgüinaré": "apirgüinar",
    "apirgüinaréis": "apirgüinar",
    "apirgüinaría": "apirgüinar",
    "apirgüinaríais": "apirgüinar",
    "apirgüinaríamos": "apirgüinar",
    "apirgüinarían": "apirgüinar",
    "apirgüinarías": "apirgüinar",
    "apirgüinas": "apirgüinar",
    "apirgüinase": "apirgüinar",
    "apirgüinaseis": "apirgüinar",
    "apirgüinasen": "apirgüinar",
    "apirgüinases": "apirgüinar",
    "apirgüinaste": "apirgüinar",
    "apirgüinasteis": "apirgüinar",
    "apirgüine": "apirgüinar",
    "apirgüinemos": "apirgüinar",
    "apirgüinen": "apirgüinar",
    "apirgüines": "apirgüinar",
    "apirgüino": "apirgüinar",
    "apirgüinábamos": "apirgüinar",
    "apirgüináis": "apirgüinar",
    "apirgüináramos": "apirgüinar",
    "apirgüináremos": "apirgüinar",
    "apirgüinásemos": "apirgüinar",
    "apirgüiné": "apirgüinar",
    "apirgüinéis": "apirgüinar",
    "apirgüinó": "apirgüinar",
    "apiris": "apiri",
    "apirética": "apirético",
    "apiréticas": "apirético",
    "apiréticos": "apirético",
    "apisona": "apisonar",
    "apisonaba": "apisonar",
    "apisonabais": "apisonar",
    "apisonaban": "apisonar",
    "apisonabas": "apisonar",
    "apisonad": "apisonar",
    "apisonada": "apisonar",
    "apisonadas": "apisonar",
    "apisonado": "apisonar",
    "apisonadoras": "apisonadora",
    "apisonados": "apisonar",
    "apisonamientos": "apisonamiento",
    "apisonamos": "apisonar",
    "apisonan": "apisonar",
    "apisonando": "apisonar",
    "apisonara": "apisonar",
    "apisonarais": "apisonar",
    "apisonaran": "apisonar",
    "apisonaras": "apisonar",
    "apisonare": "apisonar",
    "apisonareis": "apisonar",
    "apisonaremos": "apisonar",
    "apisonaren": "apisonar",
    "apisonares": "apisonar",
    "apisonaron": "apisonar",
    "apisonará": "apisonar",
    "apisonarán": "apisonar",
    "apisonarás": "apisonar",
    "apisonaré": "apisonar",
    "apisonaréis": "apisonar",
    "apisonaría": "apisonar",
    "apisonaríais": "apisonar",
    "apisonaríamos": "apisonar",
    "apisonarían": "apisonar",
    "apisonarías": "apisonar",
    "apisonas": "apisonar",
    "apisonase": "apisonar",
    "apisonaseis": "apisonar",
    "apisonasen": "apisonar",
    "apisonases": "apisonar",
    "apisonaste": "apisonar",
    "apisonasteis": "apisonar",
    "apisone": "apisonar",
    "apisonemos": "apisonar",
    "apisonen": "apisonar",
    "apisones": "apisonar",
    "apisono": "apisonar",
    "apisonábamos": "apisonar",
    "apisonáis": "apisonar",
    "apisonáramos": "apisonar",
    "apisonáremos": "apisonar",
    "apisonásemos": "apisonar",
    "apisoné": "apisonar",
    "apisonéis": "apisonar",
    "apisonó": "apisonar",
    "apitona": "apitonar",
    "apitonaba": "apitonar",
    "apitonabais": "apitonar",
    "apitonaban": "apitonar",
    "apitonabas": "apitonar",
    "apitonad": "apitonar",
    "apitonada": "apitonar",
    "apitonadas": "apitonar",
    "apitonado": "apitonar",
    "apitonados": "apitonar",
    "apitonamientos": "apitonamiento",
    "apitonamos": "apitonar",
    "apitonan": "apitonar",
    "apitonando": "apitonar",
    "apitonara": "apitonar",
    "apitonarais": "apitonar",
    "apitonaran": "apitonar",
    "apitonaras": "apitonar",
    "apitonare": "apitonar",
    "apitonareis": "apitonar",
    "apitonaremos": "apitonar",
    "apitonaren": "apitonar",
    "apitonares": "apitonar",
    "apitonaron": "apitonar",
    "apitonará": "apitonar",
    "apitonarán": "apitonar",
    "apitonarás": "apitonar",
    "apitonaré": "apitonar",
    "apitonaréis": "apitonar",
    "apitonaría": "apitonar",
    "apitonaríais": "apitonar",
    "apitonaríamos": "apitonar",
    "apitonarían": "apitonar",
    "apitonarías": "apitonar",
    "apitonas": "apitonar",
    "apitonase": "apitonar",
    "apitonaseis": "apitonar",
    "apitonasen": "apitonar",
    "apitonases": "apitonar",
    "apitonaste": "apitonar",
    "apitonasteis": "apitonar",
    "apitone": "apitonar",
    "apitonemos": "apitonar",
    "apitonen": "apitonar",
    "apitones": "apitonar",
    "apitono": "apitonar",
    "apitonábamos": "apitonar",
    "apitonáis": "apitonar",
    "apitonáramos": "apitonar",
    "apitonáremos": "apitonar",
    "apitonásemos": "apitonar",
    "apitoné": "apitonar",
    "apitonéis": "apitonar",
    "apitonó": "apitonar",
    "apizarrada": "apizarrado",
    "apizarradas": "apizarrado",
    "apizarrados": "apizarrado",
    "apiña": "apiñar",
    "apiñaba": "apiñar",
    "apiñabais": "apiñar",
    "apiñaban": "apiñar",
    "apiñabas": "apiñar",
    "apiñad": "apiñar",
    "apiñada": "apiñar",
    "apiñadas": "apiñar",
    "apiñado": "apiñar",
    "apiñados": "apiñar",
    "apiñaduras": "apiñadura",
    "apiñamientos": "apiñamiento",
    "apiñamos": "apiñar",
    "apiñan": "apiñar",
    "apiñando": "apiñar",
    "apiñara": "apiñar",
    "apiñarais": "apiñar",
    "apiñaran": "apiñar",
    "apiñaras": "apiñar",
    "apiñare": "apiñar",
    "apiñareis": "apiñar",
    "apiñaremos": "apiñar",
    "apiñaren": "apiñar",
    "apiñares": "apiñar",
    "apiñaron": "apiñar",
    "apiñará": "apiñar",
    "apiñarán": "apiñar",
    "apiñarás": "apiñar",
    "apiñaré": "apiñar",
    "apiñaréis": "apiñar",
    "apiñaría": "apiñar",
    "apiñaríais": "apiñar",
    "apiñaríamos": "apiñar",
    "apiñarían": "apiñar",
    "apiñarías": "apiñar",
    "apiñas": "apiñar",
    "apiñase": "apiñar",
    "apiñaseis": "apiñar",
    "apiñasen": "apiñar",
    "apiñases": "apiñar",
    "apiñaste": "apiñar",
    "apiñasteis": "apiñar",
    "apiñe": "apiñar",
    "apiñemos": "apiñar",
    "apiñen": "apiñar",
    "apiñes": "apiñar",
    "apiño": "apiñar",
    "apiñábamos": "apiñar",
    "apiñáis": "apiñar",
    "apiñáramos": "apiñar",
    "apiñáremos": "apiñar",
    "apiñásemos": "apiñar",
    "apiñé": "apiñar",
    "apiñéis": "apiñar",
    "apiñó": "apiñar",
    "aplaca": "aplacar",
    "aplacaba": "aplacar",
    "aplacabais": "aplacar",
    "aplacaban": "aplacar",
    "aplacabas": "aplacar",
    "aplacables": "aplacable",
    "aplacad": "aplacar",
    "aplacada": "aplacar",
    "aplacadas": "aplacar",
    "aplacado": "aplacar",
    "aplacadora": "aplacador",
    "aplacadoras": "aplacador",
    "aplacadores": "aplacador",
    "aplacados": "aplacar",
    "aplacamientos": "aplacamiento",
    "aplacamos": "aplacar",
    "aplacan": "aplacar",
    "aplacando": "aplacar",
    "aplacara": "aplacar",
    "aplacarais": "aplacar",
    "aplacaran": "aplacar",
    "aplacaras": "aplacar",
    "aplacare": "aplacar",
    "aplacareis": "aplacar",
    "aplacaremos": "aplacar",
    "aplacaren": "aplacar",
    "aplacares": "aplacar",
    "aplacaron": "aplacar",
    "aplacará": "aplacar",
    "aplacarán": "aplacar",
    "aplacarás": "aplacar",
    "aplacaré": "aplacar",
    "aplacaréis": "aplacar",
    "aplacaría": "aplacar",
    "aplacaríais": "aplacar",
    "aplacaríamos": "aplacar",
    "aplacarían": "aplacar",
    "aplacarías": "aplacar",
    "aplacas": "aplacar",
    "aplacase": "aplacar",
    "aplacaseis": "aplacar",
    "aplacasen": "aplacar",
    "aplacases": "aplacar",
    "aplacaste": "aplacar",
    "aplacasteis": "aplacar",
    "aplace": "aplazar",
    "aplacemos": "aplazar",
    "aplacen": "aplazar",
    "aplacerada": "aplacerado",
    "aplaceradas": "aplacerado",
    "aplacerados": "aplacerado",
    "aplaces": "aplazar",
    "aplaco": "aplacar",
    "aplacábamos": "aplacar",
    "aplacáis": "aplacar",
    "aplacáramos": "aplacar",
    "aplacáremos": "aplacar",
    "aplacásemos": "aplacar",
    "aplacé": "aplazar",
    "aplacéis": "aplazar",
    "aplacó": "aplacar",
    "aplana": "aplanar",
    "aplanaba": "aplanar",
    "aplanabais": "aplanar",
    "aplanaban": "aplanar",
    "aplanabas": "aplanar",
    "aplanad": "aplanar",
    "aplanada": "aplanar",
    "aplanadas": "aplanar",
    "aplanado": "aplanar",
    "aplanadora": "aplanador",
    "aplanadoras": "aplanadora",
    "aplanadores": "aplanador",
    "aplanados": "aplanar",
    "aplanamientos": "aplanamiento",
    "aplanamos": "aplanar",
    "aplanan": "aplanar",
    "aplanando": "aplanar",
    "aplanara": "aplanar",
    "aplanarais": "aplanar",
    "aplanaran": "aplanar",
    "aplanaras": "aplanar",
    "aplanare": "aplanar",
    "aplanareis": "aplanar",
    "aplanaremos": "aplanar",
    "aplanaren": "aplanar",
    "aplanares": "aplanar",
    "aplanaron": "aplanar",
    "aplanará": "aplanar",
    "aplanarán": "aplanar",
    "aplanarás": "aplanar",
    "aplanaré": "aplanar",
    "aplanaréis": "aplanar",
    "aplanaría": "aplanar",
    "aplanaríais": "aplanar",
    "aplanaríamos": "aplanar",
    "aplanarían": "aplanar",
    "aplanarías": "aplanar",
    "aplanas": "aplanar",
    "aplanase": "aplanar",
    "aplanaseis": "aplanar",
    "aplanasen": "aplanar",
    "aplanases": "aplanar",
    "aplanaste": "aplanar",
    "aplanasteis": "aplanar",
    "aplane": "aplanar",
    "aplanemos": "aplanar",
    "aplanen": "aplanar",
    "aplanes": "aplanar",
    "aplano": "aplanar",
    "aplantilla": "aplantillar",
    "aplantillaba": "aplantillar",
    "aplantillabais": "aplantillar",
    "aplantillaban": "aplantillar",
    "aplantillabas": "aplantillar",
    "aplantillad": "aplantillar",
    "aplantillada": "aplantillar",
    "aplantilladas": "aplantillar",
    "aplantillado": "aplantillar",
    "aplantillados": "aplantillar",
    "aplantillamos": "aplantillar",
    "aplantillan": "aplantillar",
    "aplantillando": "aplantillar",
    "aplantillara": "aplantillar",
    "aplantillarais": "aplantillar",
    "aplantillaran": "aplantillar",
    "aplantillaras": "aplantillar",
    "aplantillare": "aplantillar",
    "aplantillareis": "aplantillar",
    "aplantillaremos": "aplantillar",
    "aplantillaren": "aplantillar",
    "aplantillares": "aplantillar",
    "aplantillaron": "aplantillar",
    "aplantillará": "aplantillar",
    "aplantillarán": "aplantillar",
    "aplantillarás": "aplantillar",
    "aplantillaré": "aplantillar",
    "aplantillaréis": "aplantillar",
    "aplantillaría": "aplantillar",
    "aplantillaríais": "aplantillar",
    "aplantillaríamos": "aplantillar",
    "aplantillarían": "aplantillar",
    "aplantillarías": "aplantillar",
    "aplantillas": "aplantillar",
    "aplantillase": "aplantillar",
    "aplantillaseis": "aplantillar",
    "aplantillasen": "aplantillar",
    "aplantillases": "aplantillar",
    "aplantillaste": "aplantillar",
    "aplantillasteis": "aplantillar",
    "aplantille": "aplantillar",
    "aplantillemos": "aplantillar",
    "aplantillen": "aplantillar",
    "aplantilles": "aplantillar",
    "aplantillo": "aplantillar",
    "aplantillábamos": "aplantillar",
    "aplantilláis": "aplantillar",
    "aplantilláramos": "aplantillar",
    "aplantilláremos": "aplantillar",
    "aplantillásemos": "aplantillar",
    "aplantillé": "aplantillar",
    "aplantilléis": "aplantillar",
    "aplantilló": "aplantillar",
    "aplanábamos": "aplanar",
    "aplanáis": "aplanar",
    "aplanáramos": "aplanar",
    "aplanáremos": "aplanar",
    "aplanásemos": "aplanar",
    "aplanática": "aplanático",
    "aplanáticas": "aplanático",
    "aplanáticos": "aplanático",
    "aplané": "aplanar",
    "aplanéis": "aplanar",
    "aplanética": "aplanético",
    "aplanéticas": "aplanético",
    "aplanéticos": "aplanético",
    "aplanó": "aplanar",
    "aplaque": "aplacar",
    "aplaquemos": "aplacar",
    "aplaquen": "aplacar",
    "aplaques": "aplacar",
    "aplaqué": "aplacar",
    "aplaquéis": "aplacar",
    "aplasias": "aplasia",
    "aplasta": "aplastar",
    "aplastaba": "aplastar",
    "aplastabais": "aplastar",
    "aplastaban": "aplastar",
    "aplastabas": "aplastar",
    "aplastad": "aplastar",
    "aplastada": "aplastar",
    "aplastadas": "aplastar",
    "aplastado": "aplastar",
    "aplastados": "aplastar",
    "aplastamientos": "aplastamiento",
    "aplastamos": "aplastar",
    "aplastan": "aplastar",
    "aplastando": "aplastar",
    "aplastantes": "aplastante",
    "aplastara": "aplastar",
    "aplastarais": "aplastar",
    "aplastaran": "aplastar",
    "aplastaras": "aplastar",
    "aplastare": "aplastar",
    "aplastareis": "aplastar",
    "aplastaremos": "aplastar",
    "aplastaren": "aplastar",
    "aplastares": "aplastar",
    "aplastaron": "aplastar",
    "aplastará": "aplastar",
    "aplastarán": "aplastar",
    "aplastarás": "aplastar",
    "aplastaré": "aplastar",
    "aplastaréis": "aplastar",
    "aplastaría": "aplastar",
    "aplastaríais": "aplastar",
    "aplastaríamos": "aplastar",
    "aplastarían": "aplastar",
    "aplastarías": "aplastar",
    "aplastas": "aplastar",
    "aplastase": "aplastar",
    "aplastaseis": "aplastar",
    "aplastasen": "aplastar",
    "aplastases": "aplastar",
    "aplastaste": "aplastar",
    "aplastasteis": "aplastar",
    "aplaste": "aplastar",
    "aplastemos": "aplastar",
    "aplasten": "aplastar",
    "aplastes": "aplastar",
    "aplasto": "aplastar",
    "aplastábamos": "aplastar",
    "aplastáis": "aplastar",
    "aplastáramos": "aplastar",
    "aplastáremos": "aplastar",
    "aplastásemos": "aplastar",
    "aplasté": "aplastar",
    "aplastéis": "aplastar",
    "aplastó": "aplastar",
    "aplatana": "aplatanar",
    "aplatanaba": "aplatanar",
    "aplatanabais": "aplatanar",
    "aplatanaban": "aplatanar",
    "aplatanabas": "aplatanar",
    "aplatanad": "aplatanar",
    "aplatanada": "aplatanar",
    "aplatanadas": "aplatanar",
    "aplatanado": "aplatanar",
    "aplatanados": "aplatanar",
    "aplatanamientos": "aplatanamiento",
    "aplatanamos": "aplatanar",
    "aplatanan": "aplatanar",
    "aplatanando": "aplatanar",
    "aplatanara": "aplatanar",
    "aplatanarais": "aplatanar",
    "aplatanaran": "aplatanar",
    "aplatanaras": "aplatanar",
    "aplatanare": "aplatanar",
    "aplatanareis": "aplatanar",
    "aplatanaremos": "aplatanar",
    "aplatanaren": "aplatanar",
    "aplatanares": "aplatanar",
    "aplatanaron": "aplatanar",
    "aplatanará": "aplatanar",
    "aplatanarán": "aplatanar",
    "aplatanarás": "aplatanar",
    "aplatanaré": "aplatanar",
    "aplatanaréis": "aplatanar",
    "aplatanaría": "aplatanar",
    "aplatanaríais": "aplatanar",
    "aplatanaríamos": "aplatanar",
    "aplatanarían": "aplatanar",
    "aplatanarías": "aplatanar",
    "aplatanas": "aplatanar",
    "aplatanase": "aplatanar",
    "aplatanaseis": "aplatanar",
    "aplatanasen": "aplatanar",
    "aplatanases": "aplatanar",
    "aplatanaste": "aplatanar",
    "aplatanasteis": "aplatanar",
    "aplatane": "aplatanar",
    "aplatanemos": "aplatanar",
    "aplatanen": "aplatanar",
    "aplatanes": "aplatanar",
    "aplatano": "aplatanar",
    "aplatanábamos": "aplatanar",
    "aplatanáis": "aplatanar",
    "aplatanáramos": "aplatanar",
    "aplatanáremos": "aplatanar",
    "aplatanásemos": "aplatanar",
    "aplatané": "aplatanar",
    "aplatanéis": "aplatanar",
    "aplatanó": "aplatanar",
    "aplauda": "aplaudir",
    "aplaudamos": "aplaudir",
    "aplaudan": "aplaudir",
    "aplaudas": "aplaudir",
    "aplaude": "aplaudir",
    "aplauden": "aplaudir",
    "aplaudes": "aplaudir",
    "aplaudid": "aplaudir",
    "aplaudida": "aplaudir",
    "aplaudidas": "aplaudir",
    "aplaudido": "aplaudir",
    "aplaudidora": "aplaudidor",
    "aplaudidoras": "aplaudidor",
    "aplaudidores": "aplaudidor",
    "aplaudidos": "aplaudir",
    "aplaudiendo": "aplaudir",
    "aplaudiera": "aplaudir",
    "aplaudierais": "aplaudir",
    "aplaudieran": "aplaudir",
    "aplaudieras": "aplaudir",
    "aplaudiere": "aplaudir",
    "aplaudiereis": "aplaudir",
    "aplaudieren": "aplaudir",
    "aplaudieres": "aplaudir",
    "aplaudieron": "aplaudir",
    "aplaudiese": "aplaudir",
    "aplaudieseis": "aplaudir",
    "aplaudiesen": "aplaudir",
    "aplaudieses": "aplaudir",
    "aplaudimos": "aplaudir",
    "aplaudiremos": "aplaudir",
    "aplaudirá": "aplaudir",
    "aplaudirán": "aplaudir",
    "aplaudirás": "aplaudir",
    "aplaudiré": "aplaudir",
    "aplaudiréis": "aplaudir",
    "aplaudiría": "aplaudir",
    "aplaudiríais": "aplaudir",
    "aplaudiríamos": "aplaudir",
    "aplaudirían": "aplaudir",
    "aplaudirías": "aplaudir",
    "aplaudiste": "aplaudir",
    "aplaudisteis": "aplaudir",
    "aplaudiéramos": "aplaudir",
    "aplaudiéremos": "aplaudir",
    "aplaudiésemos": "aplaudir",
    "aplaudió": "aplaudir",
    "aplaudo": "aplaudir",
    "aplaudáis": "aplaudir",
    "aplaudí": "aplaudir",
    "aplaudía": "aplaudir",
    "aplaudíais": "aplaudir",
    "aplaudíamos": "aplaudir",
    "aplaudían": "aplaudir",
    "aplaudías": "aplaudir",
    "aplaudís": "aplaudir",
    "aplausos": "aplauso",
    "aplaza": "aplazar",
    "aplazaba": "aplazar",
    "aplazabais": "aplazar",
    "aplazaban": "aplazar",
    "aplazabas": "aplazar",
    "aplazables": "aplazable",
    "aplazad": "aplazar",
    "aplazada": "aplazar",
    "aplazadas": "aplazar",
    "aplazado": "aplazar",
    "aplazados": "aplazar",
    "aplazamientos": "aplazamiento",
    "aplazamos": "aplazar",
    "aplazan": "aplazar",
    "aplazando": "aplazar",
    "aplazara": "aplazar",
    "aplazarais": "aplazar",
    "aplazaran": "aplazar",
    "aplazaras": "aplazar",
    "aplazare": "aplazar",
    "aplazareis": "aplazar",
    "aplazaremos": "aplazar",
    "aplazaren": "aplazar",
    "aplazares": "aplazar",
    "aplazaron": "aplazar",
    "aplazará": "aplazar",
    "aplazarán": "aplazar",
    "aplazarás": "aplazar",
    "aplazaré": "aplazar",
    "aplazaréis": "aplazar",
    "aplazaría": "aplazar",
    "aplazaríais": "aplazar",
    "aplazaríamos": "aplazar",
    "aplazarían": "aplazar",
    "aplazarías": "aplazar",
    "aplazas": "aplazar",
    "aplazase": "aplazar",
    "aplazaseis": "aplazar",
    "aplazasen": "aplazar",
    "aplazases": "aplazar",
    "aplazaste": "aplazar",
    "aplazasteis": "aplazar",
    "aplazo": "aplazar",
    "aplazábamos": "aplazar",
    "aplazáis": "aplazar",
    "aplazáramos": "aplazar",
    "aplazáremos": "aplazar",
    "aplazásemos": "aplazar",
    "aplazó": "aplazar",
    "aplebeya": "aplebeyar",
    "aplebeyaba": "aplebeyar",
    "aplebeyabais": "aplebeyar",
    "aplebeyaban": "aplebeyar",
    "aplebeyabas": "aplebeyar",
    "aplebeyad": "aplebeyar",
    "aplebeyada": "aplebeyar",
    "aplebeyadas": "aplebeyar",
    "aplebeyado": "aplebeyar",
    "aplebeyados": "aplebeyar",
    "aplebeyamos": "aplebeyar",
    "aplebeyan": "aplebeyar",
    "aplebeyando": "aplebeyar",
    "aplebeyara": "aplebeyar",
    "aplebeyarais": "aplebeyar",
    "aplebeyaran": "aplebeyar",
    "aplebeyaras": "aplebeyar",
    "aplebeyare": "aplebeyar",
    "aplebeyareis": "aplebeyar",
    "aplebeyaremos": "aplebeyar",
    "aplebeyaren": "aplebeyar",
    "aplebeyares": "aplebeyar",
    "aplebeyaron": "aplebeyar",
    "aplebeyará": "aplebeyar",
    "aplebeyarán": "aplebeyar",
    "aplebeyarás": "aplebeyar",
    "aplebeyaré": "aplebeyar",
    "aplebeyaréis": "aplebeyar",
    "aplebeyaría": "aplebeyar",
    "aplebeyaríais": "aplebeyar",
    "aplebeyaríamos": "aplebeyar",
    "aplebeyarían": "aplebeyar",
    "aplebeyarías": "aplebeyar",
    "aplebeyas": "aplebeyar",
    "aplebeyase": "aplebeyar",
    "aplebeyaseis": "aplebeyar",
    "aplebeyasen": "aplebeyar",
    "aplebeyases": "aplebeyar",
    "aplebeyaste": "aplebeyar",
    "aplebeyasteis": "aplebeyar",
    "aplebeye": "aplebeyar",
    "aplebeyemos": "aplebeyar",
    "aplebeyen": "aplebeyar",
    "aplebeyes": "aplebeyar",
    "aplebeyo": "aplebeyar",
    "aplebeyábamos": "aplebeyar",
    "aplebeyáis": "aplebeyar",
    "aplebeyáramos": "aplebeyar",
    "aplebeyáremos": "aplebeyar",
    "aplebeyásemos": "aplebeyar",
    "aplebeyé": "aplebeyar",
    "aplebeyéis": "aplebeyar",
    "aplebeyó": "aplebeyar",
    "aplica": "aplicar",
    "aplicaba": "aplicar",
    "aplicabais": "aplicar",
    "aplicaban": "aplicar",
    "aplicabas": "aplicar",
    "aplicabilidades": "aplicabilidad",
    "aplicables": "aplicable",
    "aplicaciones": "aplicación",
    "aplicad": "aplicar",
    "aplicada": "aplicar",
    "aplicadas": "aplicar",
    "aplicado": "aplicar",
    "aplicadora": "aplicador",
    "aplicadoras": "aplicador",
    "aplicadores": "aplicador",
    "aplicados": "aplicar",
    "aplicamos": "aplicar",
    "aplican": "aplicar",
    "aplicando": "aplicar",
    "aplicara": "aplicar",
    "aplicarais": "aplicar",
    "aplicaran": "aplicar",
    "aplicaras": "aplicar",
    "aplicare": "aplicar",
    "aplicareis": "aplicar",
    "aplicaremos": "aplicar",
    "aplicaren": "aplicar",
    "aplicares": "aplicar",
    "aplicaron": "aplicar",
    "aplicará": "aplicar",
    "aplicarán": "aplicar",
    "aplicarás": "aplicar",
    "aplicaré": "aplicar",
    "aplicaréis": "aplicar",
    "aplicaría": "aplicar",
    "aplicaríais": "aplicar",
    "aplicaríamos": "aplicar",
    "aplicarían": "aplicar",
    "aplicarías": "aplicar",
    "aplicas": "aplicar",
    "aplicase": "aplicar",
    "aplicaseis": "aplicar",
    "aplicasen": "aplicar",
    "aplicases": "aplicar",
    "aplicaste": "aplicar",
    "aplicasteis": "aplicar",
    "aplicativa": "aplicativo",
    "aplicativas": "aplicativo",
    "aplicativos": "aplicativo",
    "aplico": "aplicar",
    "aplicábamos": "aplicar",
    "aplicáis": "aplicar",
    "aplicáramos": "aplicar",
    "aplicáremos": "aplicar",
    "aplicásemos": "aplicar",
    "aplicó": "aplicar",
    "aplique": "aplicar",
    "apliquemos": "aplicar",
    "apliquen": "aplicar",
    "apliques": "aplique",
    "apliqué": "aplicar",
    "apliquéis": "aplicar",
    "aplitas": "aplita",
    "aploma": "aplomar",
    "aplomaba": "aplomar",
    "aplomabais": "aplomar",
    "aplomaban": "aplomar",
    "aplomabas": "aplomar",
    "aplomad": "aplomar",
    "aplomada": "aplomar",
    "aplomadas": "aplomar",
    "aplomado": "aplomar",
    "aplomados": "aplomar",
    "aplomamos": "aplomar",
    "aploman": "aplomar",
    "aplomando": "aplomar",
    "aplomara": "aplomar",
    "aplomarais": "aplomar",
    "aplomaran": "aplomar",
    "aplomaras": "aplomar",
    "aplomare": "aplomar",
    "aplomareis": "aplomar",
    "aplomaremos": "aplomar",
    "aplomaren": "aplomar",
    "aplomares": "aplomar",
    "aplomaron": "aplomar",
    "aplomará": "aplomar",
    "aplomarán": "aplomar",
    "aplomarás": "aplomar",
    "aplomaré": "aplomar",
    "aplomaréis": "aplomar",
    "aplomaría": "aplomar",
    "aplomaríais": "aplomar",
    "aplomaríamos": "aplomar",
    "aplomarían": "aplomar",
    "aplomarías": "aplomar",
    "aplomas": "aplomar",
    "aplomase": "aplomar",
    "aplomaseis": "aplomar",
    "aplomasen": "aplomar",
    "aplomases": "aplomar",
    "aplomaste": "aplomar",
    "aplomasteis": "aplomar",
    "aplome": "aplomar",
    "aplomemos": "aplomar",
    "aplomen": "aplomar",
    "aplomes": "aplomar",
    "aplomo": "aplomar",
    "aplomos": "aplomo",
    "aplomábamos": "aplomar",
    "aplomáis": "aplomar",
    "aplomáramos": "aplomar",
    "aplomáremos": "aplomar",
    "aplomásemos": "aplomar",
    "aplomé": "aplomar",
    "aploméis": "aplomar",
    "aplomó": "aplomar",
    "aplustros": "aplustro",
    "aplásica": "aplásico",
    "aplásicas": "aplásico",
    "aplásicos": "aplásico",
    "apneas": "apnea",
    "apoastros": "apoastro",
    "apoca": "apocar",
    "apocaba": "apocar",
    "apocabais": "apocar",
    "apocaban": "apocar",
    "apocabas": "apocar",
    "apocad": "apocar",
    "apocada": "apocar",
    "apocadas": "apocar",
    "apocado": "apocar",
    "apocados": "apocar",
    "apocalíptica": "apocalíptico",
    "apocalípticas": "apocalíptico",
    "apocalípticos": "apocalíptico",
    "apocamientos": "apocamiento",
    "apocamos": "apocar",
    "apocan": "apocar",
    "apocando": "apocar",
    "apocara": "apocar",
    "apocarais": "apocar",
    "apocaran": "apocar",
    "apocaras": "apocar",
    "apocare": "apocar",
    "apocareis": "apocar",
    "apocaremos": "apocar",
    "apocaren": "apocar",
    "apocares": "apocar",
    "apocaron": "apocar",
    "apocará": "apocar",
    "apocarán": "apocar",
    "apocarás": "apocar",
    "apocaré": "apocar",
    "apocaréis": "apocar",
    "apocaría": "apocar",
    "apocaríais": "apocar",
    "apocaríamos": "apocar",
    "apocarían": "apocar",
    "apocarías": "apocar",
    "apocas": "apocar",
    "apocase": "apocar",
    "apocaseis": "apocar",
    "apocasen": "apocar",
    "apocases": "apocar",
    "apocaste": "apocar",
    "apocasteis": "apocar",
    "apoco": "apocar",
    "apocopa": "apocopar",
    "apocopaba": "apocopar",
    "apocopabais": "apocopar",
    "apocopaban": "apocopar",
    "apocopabas": "apocopar",
    "apocopad": "apocopar",
    "apocopada": "apocopar",
    "apocopadas": "apocopar",
    "apocopado": "apocopar",
    "apocopados": "apocopar",
    "apocopamos": "apocopar",
    "apocopan": "apocopar",
    "apocopando": "apocopar",
    "apocopara": "apocopar",
    "apocoparais": "apocopar",
    "apocoparan": "apocopar",
    "apocoparas": "apocopar",
    "apocopare": "apocopar",
    "apocopareis": "apocopar",
    "apocoparemos": "apocopar",
    "apocoparen": "apocopar",
    "apocopares": "apocopar",
    "apocoparon": "apocopar",
    "apocopará": "apocopar",
    "apocoparán": "apocopar",
    "apocoparás": "apocopar",
    "apocoparé": "apocopar",
    "apocoparéis": "apocopar",
    "apocoparía": "apocopar",
    "apocoparíais": "apocopar",
    "apocoparíamos": "apocopar",
    "apocoparían": "apocopar",
    "apocoparías": "apocopar",
    "apocopas": "apocopar",
    "apocopase": "apocopar",
    "apocopaseis": "apocopar",
    "apocopasen": "apocopar",
    "apocopases": "apocopar",
    "apocopaste": "apocopar",
    "apocopasteis": "apocopar",
    "apocope": "apocopar",
    "apocopemos": "apocopar",
    "apocopen": "apocopar",
    "apocopes": "apocopar",
    "apocopo": "apocopar",
    "apocopábamos": "apocopar",
    "apocopáis": "apocopar",
    "apocopáramos": "apocopar",
    "apocopáremos": "apocopar",
    "apocopásemos": "apocopar",
    "apocopé": "apocopar",
    "apocopéis": "apocopar",
    "apocopó": "apocopar",
    "apocromática": "apocromático",
    "apocromáticas": "apocromático",
    "apocromáticos": "apocromático",
    "apocábamos": "apocar",
    "apocáis": "apocar",
    "apocáramos": "apocar",
    "apocáremos": "apocar",
    "apocásemos": "apocar",
    "apocó": "apocar",
    "apoda": "apodar",
    "apodaba": "apodar",
    "apodabais": "apodar",
    "apodaban": "apodar",
    "apodabas": "apodar",
    "apodad": "apodar",
    "apodada": "apodar",
    "apodadas": "apodar",
    "apodado": "apodar",
    "apodados": "apodar",
    "apodamos": "apodar",
    "apodan": "apodar",
    "apodando": "apodar",
    "apodara": "apodar",
    "apodarais": "apodar",
    "apodaran": "apodar",
    "apodaras": "apodar",
    "apodare": "apodar",
    "apodareis": "apodar",
    "apodaremos": "apodar",
    "apodaren": "apodar",
    "apodares": "apodar",
    "apodaron": "apodar",
    "apodará": "apodar",
    "apodarán": "apodar",
    "apodarás": "apodar",
    "apodaré": "apodar",
    "apodaréis": "apodar",
    "apodaría": "apodar",
    "apodaríais": "apodar",
    "apodaríamos": "apodar",
    "apodarían": "apodar",
    "apodarías": "apodar",
    "apodas": "apodar",
    "apodase": "apodar",
    "apodaseis": "apodar",
    "apodasen": "apodar",
    "apodases": "apodar",
    "apodaste": "apodar",
    "apodasteis": "apodar",
    "apode": "apodar",
    "apodemos": "apodar",
    "apoden": "apodar",
    "apodera": "apoderar",
    "apoderaba": "apoderar",
    "apoderabais": "apoderar",
    "apoderaban": "apoderar",
    "apoderabas": "apoderar",
    "apoderad": "apoderar",
    "apoderada": "apoderar",
    "apoderadas": "apoderar",
    "apoderado": "apoderar",
    "apoderados": "apoderar",
    "apoderamientos": "apoderamiento",
    "apoderamos": "apoderar",
    "apoderan": "apoderar",
    "apoderando": "apoderar",
    "apoderara": "apoderar",
    "apoderarais": "apoderar",
    "apoderaran": "apoderar",
    "apoderaras": "apoderar",
    "apoderare": "apoderar",
    "apoderareis": "apoderar",
    "apoderaremos": "apoderar",
    "apoderaren": "apoderar",
    "apoderares": "apoderar",
    "apoderaron": "apoderar",
    "apoderará": "apoderar",
    "apoderarán": "apoderar",
    "apoderarás": "apoderar",
    "apoderaré": "apoderar",
    "apoderaréis": "apoderar",
    "apoderaría": "apoderar",
    "apoderaríais": "apoderar",
    "apoderaríamos": "apoderar",
    "apoderarían": "apoderar",
    "apoderarías": "apoderar",
    "apoderas": "apoderar",
    "apoderase": "apoderar",
    "apoderaseis": "apoderar",
    "apoderasen": "apoderar",
    "apoderases": "apoderar",
    "apoderaste": "apoderar",
    "apoderasteis": "apoderar",
    "apodere": "apoderar",
    "apoderemos": "apoderar",
    "apoderen": "apoderar",
    "apoderes": "apoderar",
    "apodero": "apoderar",
    "apoderábamos": "apoderar",
    "apoderáis": "apoderar",
    "apoderáramos": "apoderar",
    "apoderáremos": "apoderar",
    "apoderásemos": "apoderar",
    "apoderé": "apoderar",
    "apoderéis": "apoderar",
    "apoderó": "apoderar",
    "apodes": "apodar",
    "apodisaciones": "apodisación",
    "apodo": "apodar",
    "apodos": "apodo",
    "apodábamos": "apodar",
    "apodáis": "apodar",
    "apodáramos": "apodar",
    "apodáremos": "apodar",
    "apodásemos": "apodar",
    "apodé": "apodar",
    "apodéis": "apodar",
    "apodíctica": "apodíctico",
    "apodícticas": "apodíctico",
    "apodícticos": "apodíctico",
    "apodó": "apodar",
    "apoenzimas": "apoenzima",
    "apoferritinas": "apoferritina",
    "apofilitas": "apofilita",
    "apofonías": "apofonía",
    "apofánticas": "apofántica",
    "apogamias": "apogamia",
    "apogeos": "apogeo",
    "apolilla": "apolillar",
    "apolillaba": "apolillar",
    "apolillabais": "apolillar",
    "apolillaban": "apolillar",
    "apolillabas": "apolillar",
    "apolillad": "apolillar",
    "apolillada": "apolillar",
    "apolilladas": "apolillar",
    "apolillado": "apolillar",
    "apolillados": "apolillar",
    "apolilladuras": "apolilladura",
    "apolillamientos": "apolillamiento",
    "apolillamos": "apolillar",
    "apolillan": "apolillar",
    "apolillando": "apolillar",
    "apolillara": "apolillar",
    "apolillarais": "apolillar",
    "apolillaran": "apolillar",
    "apolillaras": "apolillar",
    "apolillare": "apolillar",
    "apolillareis": "apolillar",
    "apolillaremos": "apolillar",
    "apolillaren": "apolillar",
    "apolillares": "apolillar",
    "apolillaron": "apolillar",
    "apolillará": "apolillar",
    "apolillarán": "apolillar",
    "apolillarás": "apolillar",
    "apolillaré": "apolillar",
    "apolillaréis": "apolillar",
    "apolillaría": "apolillar",
    "apolillaríais": "apolillar",
    "apolillaríamos": "apolillar",
    "apolillarían": "apolillar",
    "apolillarías": "apolillar",
    "apolillas": "apolillar",
    "apolillase": "apolillar",
    "apolillaseis": "apolillar",
    "apolillasen": "apolillar",
    "apolillases": "apolillar",
    "apolillaste": "apolillar",
    "apolillasteis": "apolillar",
    "apolille": "apolillar",
    "apolillemos": "apolillar",
    "apolillen": "apolillar",
    "apolilles": "apolillar",
    "apolillo": "apolillar",
    "apolillábamos": "apolillar",
    "apolilláis": "apolillar",
    "apolilláramos": "apolillar",
    "apolilláremos": "apolillar",
    "apolillásemos": "apolillar",
    "apolillé": "apolillar",
    "apolilléis": "apolillar",
    "apolilló": "apolillar",
    "apolinarismos": "apolinarismo",
    "apolinaristas": "apolinarista",
    "apoliticismos": "apoliticismo",
    "apologistas": "apologista",
    "apologética": "apologético",
    "apologéticas": "apologético",
    "apologéticos": "apologético",
    "apologías": "apología",
    "apoltrona": "apoltronar",
    "apoltronaba": "apoltronar",
    "apoltronabais": "apoltronar",
    "apoltronaban": "apoltronar",
    "apoltronabas": "apoltronar",
    "apoltronad": "apoltronar",
    "apoltronada": "apoltronar",
    "apoltronadas": "apoltronar",
    "apoltronado": "apoltronar",
    "apoltronados": "apoltronar",
    "apoltronamientos": "apoltronamiento",
    "apoltronamos": "apoltronar",
    "apoltronan": "apoltronar",
    "apoltronando": "apoltronar",
    "apoltronara": "apoltronar",
    "apoltronarais": "apoltronar",
    "apoltronaran": "apoltronar",
    "apoltronaras": "apoltronar",
    "apoltronare": "apoltronar",
    "apoltronareis": "apoltronar",
    "apoltronaremos": "apoltronar",
    "apoltronaren": "apoltronar",
    "apoltronares": "apoltronar",
    "apoltronaron": "apoltronar",
    "apoltronará": "apoltronar",
    "apoltronarán": "apoltronar",
    "apoltronarás": "apoltronar",
    "apoltronaré": "apoltronar",
    "apoltronaréis": "apoltronar",
    "apoltronaría": "apoltronar",
    "apoltronaríais": "apoltronar",
    "apoltronaríamos": "apoltronar",
    "apoltronarían": "apoltronar",
    "apoltronarías": "apoltronar",
    "apoltronas": "apoltronar",
    "apoltronase": "apoltronar",
    "apoltronaseis": "apoltronar",
    "apoltronasen": "apoltronar",
    "apoltronases": "apoltronar",
    "apoltronaste": "apoltronar",
    "apoltronasteis": "apoltronar",
    "apoltrone": "apoltronar",
    "apoltronemos": "apoltronar",
    "apoltronen": "apoltronar",
    "apoltrones": "apoltronar",
    "apoltrono": "apoltronar",
    "apoltronábamos": "apoltronar",
    "apoltronáis": "apoltronar",
    "apoltronáramos": "apoltronar",
    "apoltronáremos": "apoltronar",
    "apoltronásemos": "apoltronar",
    "apoltroné": "apoltronar",
    "apoltronéis": "apoltronar",
    "apoltronó": "apoltronar",
    "apolvilla": "apolvillar",
    "apolvillaba": "apolvillar",
    "apolvillabais": "apolvillar",
    "apolvillaban": "apolvillar",
    "apolvillabas": "apolvillar",
    "apolvillad": "apolvillar",
    "apolvillada": "apolvillar",
    "apolvilladas": "apolvillar",
    "apolvillado": "apolvillar",
    "apolvillados": "apolvillar",
    "apolvillamos": "apolvillar",
    "apolvillan": "apolvillar",
    "apolvillando": "apolvillar",
    "apolvillara": "apolvillar",
    "apolvillarais": "apolvillar",
    "apolvillaran": "apolvillar",
    "apolvillaras": "apolvillar",
    "apolvillare": "apolvillar",
    "apolvillareis": "apolvillar",
    "apolvillaremos": "apolvillar",
    "apolvillaren": "apolvillar",
    "apolvillares": "apolvillar",
    "apolvillaron": "apolvillar",
    "apolvillará": "apolvillar",
    "apolvillarán": "apolvillar",
    "apolvillarás": "apolvillar",
    "apolvillaré": "apolvillar",
    "apolvillaréis": "apolvillar",
    "apolvillaría": "apolvillar",
    "apolvillaríais": "apolvillar",
    "apolvillaríamos": "apolvillar",
    "apolvillarían": "apolvillar",
    "apolvillarías": "apolvillar",
    "apolvillas": "apolvillar",
    "apolvillase": "apolvillar",
    "apolvillaseis": "apolvillar",
    "apolvillasen": "apolvillar",
    "apolvillases": "apolvillar",
    "apolvillaste": "apolvillar",
    "apolvillasteis": "apolvillar",
    "apolville": "apolvillar",
    "apolvillemos": "apolvillar",
    "apolvillen": "apolvillar",
    "apolvilles": "apolvillar",
    "apolvillo": "apolvillar",
    "apolvillábamos": "apolvillar",
    "apolvilláis": "apolvillar",
    "apolvilláramos": "apolvillar",
    "apolvilláremos": "apolvillar",
    "apolvillásemos": "apolvillar",
    "apolvillé": "apolvillar",
    "apolvilléis": "apolvillar",
    "apolvilló": "apolvillar",
    "apolínea": "apolíneo",
    "apolíneas": "apolíneo",
    "apolíneos": "apolíneo",
    "apolítica": "apolítico",
    "apolíticas": "apolítico",
    "apolíticos": "apolítico",
    "apológica": "apológico",
    "apológicas": "apológico",
    "apológicos": "apológico",
    "apomace": "apomazar",
    "apomacemos": "apomazar",
    "apomacen": "apomazar",
    "apomaces": "apomazar",
    "apomacé": "apomazar",
    "apomacéis": "apomazar",
    "apomaza": "apomazar",
    "apomazaba": "apomazar",
    "apomazabais": "apomazar",
    "apomazaban": "apomazar",
    "apomazabas": "apomazar",
    "apomazad": "apomazar",
    "apomazada": "apomazar",
    "apomazadas": "apomazar",
    "apomazado": "apomazar",
    "apomazados": "apomazar",
    "apomazamos": "apomazar",
    "apomazan": "apomazar",
    "apomazando": "apomazar",
    "apomazara": "apomazar",
    "apomazarais": "apomazar",
    "apomazaran": "apomazar",
    "apomazaras": "apomazar",
    "apomazare": "apomazar",
    "apomazareis": "apomazar",
    "apomazaremos": "apomazar",
    "apomazaren": "apomazar",
    "apomazares": "apomazar",
    "apomazaron": "apomazar",
    "apomazará": "apomazar",
    "apomazarán": "apomazar",
    "apomazarás": "apomazar",
    "apomazaré": "apomazar",
    "apomazaréis": "apomazar",
    "apomazaría": "apomazar",
    "apomazaríais": "apomazar",
    "apomazaríamos": "apomazar",
    "apomazarían": "apomazar",
    "apomazarías": "apomazar",
    "apomazas": "apomazar",
    "apomazase": "apomazar",
    "apomazaseis": "apomazar",
    "apomazasen": "apomazar",
    "apomazases": "apomazar",
    "apomazaste": "apomazar",
    "apomazasteis": "apomazar",
    "apomazo": "apomazar",
    "apomazábamos": "apomazar",
    "apomazáis": "apomazar",
    "apomazáramos": "apomazar",
    "apomazáremos": "apomazar",
    "apomazásemos": "apomazar",
    "apomazó": "apomazar",
    "apomorfinas": "apomorfina",
    "aponeurótica": "aponeurótico",
    "aponeuróticas": "aponeurótico",
    "aponeuróticos": "aponeurótico",
    "apontoca": "apontocar",
    "apontocaba": "apontocar",
    "apontocabais": "apontocar",
    "apontocaban": "apontocar",
    "apontocabas": "apontocar",
    "apontocad": "apontocar",
    "apontocada": "apontocar",
    "apontocadas": "apontocar",
    "apontocado": "apontocar",
    "apontocados": "apontocar",
    "apontocamos": "apontocar",
    "apontocan": "apontocar",
    "apontocando": "apontocar",
    "apontocara": "apontocar",
    "apontocarais": "apontocar",
    "apontocaran": "apontocar",
    "apontocaras": "apontocar",
    "apontocare": "apontocar",
    "apontocareis": "apontocar",
    "apontocaremos": "apontocar",
    "apontocaren": "apontocar",
    "apontocares": "apontocar",
    "apontocaron": "apontocar",
    "apontocará": "apontocar",
    "apontocarán": "apontocar",
    "apontocarás": "apontocar",
    "apontocaré": "apontocar",
    "apontocaréis": "apontocar",
    "apontocaría": "apontocar",
    "apontocaríais": "apontocar",
    "apontocaríamos": "apontocar",
    "apontocarían": "apontocar",
    "apontocarías": "apontocar",
    "apontocas": "apontocar",
    "apontocase": "apontocar",
    "apontocaseis": "apontocar",
    "apontocasen": "apontocar",
    "apontocases": "apontocar",
    "apontocaste": "apontocar",
    "apontocasteis": "apontocar",
    "apontoco": "apontocar",
    "apontocábamos": "apontocar",
    "apontocáis": "apontocar",
    "apontocáramos": "apontocar",
    "apontocáremos": "apontocar",
    "apontocásemos": "apontocar",
    "apontocó": "apontocar",
    "apontoque": "apontocar",
    "apontoquemos": "apontocar",
    "apontoquen": "apontocar",
    "apontoques": "apontocar",
    "apontoqué": "apontocar",
    "apontoquéis": "apontocar",
    "apopa": "apopar",
    "apopaba": "apopar",
    "apopabais": "apopar",
    "apopaban": "apopar",
    "apopabas": "apopar",
    "apopad": "apopar",
    "apopada": "apopar",
    "apopadas": "apopar",
    "apopado": "apopar",
    "apopados": "apopar",
    "apopamos": "apopar",
    "apopan": "apopar",
    "apopando": "apopar",
    "apopara": "apopar",
    "apoparais": "apopar",
    "apoparan": "apopar",
    "apoparas": "apopar",
    "apopare": "apopar",
    "apopareis": "apopar",
    "apoparemos": "apopar",
    "apoparen": "apopar",
    "apopares": "apopar",
    "apoparon": "apopar",
    "apopará": "apopar",
    "apoparán": "apopar",
    "apoparás": "apopar",
    "apoparé": "apopar",
    "apoparéis": "apopar",
    "apoparía": "apopar",
    "apoparíais": "apopar",
    "apoparíamos": "apopar",
    "apoparían": "apopar",
    "apoparías": "apopar",
    "apopas": "apopar",
    "apopase": "apopar",
    "apopaseis": "apopar",
    "apopasen": "apopar",
    "apopases": "apopar",
    "apopaste": "apopar",
    "apopasteis": "apopar",
    "apope": "apopar",
    "apopemos": "apopar",
    "apopen": "apopar",
    "apopes": "apopar",
    "apoplastos": "apoplasto",
    "apoplejías": "apoplejía",
    "apoplética": "apoplético",
    "apopléticas": "apoplético",
    "apopléticos": "apoplético",
    "apopo": "apopar",
    "apoproteínas": "apoproteína",
    "apopábamos": "apopar",
    "apopáis": "apopar",
    "apopáramos": "apopar",
    "apopáremos": "apopar",
    "apopásemos": "apopar",
    "apopé": "apopar",
    "apopéis": "apopar",
    "apopó": "apopar",
    "apoque": "apocar",
    "apoquemos": "apocar",
    "apoquen": "apocar",
    "apoques": "apocar",
    "apoquina": "apoquinar",
    "apoquinaba": "apoquinar",
    "apoquinabais": "apoquinar",
    "apoquinaban": "apoquinar",
    "apoquinabas": "apoquinar",
    "apoquinad": "apoquinar",
    "apoquinada": "apoquinar",
    "apoquinadas": "apoquinar",
    "apoquinado": "apoquinar",
    "apoquinados": "apoquinar",
    "apoquinamientos": "apoquinamiento",
    "apoquinamos": "apoquinar",
    "apoquinan": "apoquinar",
    "apoquinando": "apoquinar",
    "apoquinara": "apoquinar",
    "apoquinarais": "apoquinar",
    "apoquinaran": "apoquinar",
    "apoquinaras": "apoquinar",
    "apoquinare": "apoquinar",
    "apoquinareis": "apoquinar",
    "apoquinaremos": "apoquinar",
    "apoquinaren": "apoquinar",
    "apoquinares": "apoquinar",
    "apoquinaron": "apoquinar",
    "apoquinará": "apoquinar",
    "apoquinarán": "apoquinar",
    "apoquinarás": "apoquinar",
    "apoquinaré": "apoquinar",
    "apoquinaréis": "apoquinar",
    "apoquinaría": "apoquinar",
    "apoquinaríais": "apoquinar",
    "apoquinaríamos": "apoquinar",
    "apoquinarían": "apoquinar",
    "apoquinarías": "apoquinar",
    "apoquinas": "apoquinar",
    "apoquinase": "apoquinar",
    "apoquinaseis": "apoquinar",
    "apoquinasen": "apoquinar",
    "apoquinases": "apoquinar",
    "apoquinaste": "apoquinar",
    "apoquinasteis": "apoquinar",
    "apoquine": "apoquinar",
    "apoquinemos": "apoquinar",
    "apoquinen": "apoquinar",
    "apoquines": "apoquinar",
    "apoquino": "apoquinar",
    "apoquinábamos": "apoquinar",
    "apoquináis": "apoquinar",
    "apoquináramos": "apoquinar",
    "apoquináremos": "apoquinar",
    "apoquinásemos": "apoquinar",
    "apoquiné": "apoquinar",
    "apoquinéis": "apoquinar",
    "apoquinó": "apoquinar",
    "apoqué": "apocar",
    "apoquéis": "apocar",
    "aporca": "aporcar",
    "aporcaba": "aporcar",
    "aporcabais": "aporcar",
    "aporcaban": "aporcar",
    "aporcabas": "aporcar",
    "aporcad": "aporcar",
    "aporcada": "aporcar",
    "aporcadas": "aporcar",
    "aporcado": "aporcar",
    "aporcadores": "aporcador",
    "aporcados": "aporcar",
    "aporcaduras": "aporcadura",
    "aporcamos": "aporcar",
    "aporcan": "aporcar",
    "aporcando": "aporcar",
    "aporcara": "aporcar",
    "aporcarais": "aporcar",
    "aporcaran": "aporcar",
    "aporcaras": "aporcar",
    "aporcare": "aporcar",
    "aporcareis": "aporcar",
    "aporcaremos": "aporcar",
    "aporcaren": "aporcar",
    "aporcares": "aporcar",
    "aporcaron": "aporcar",
    "aporcará": "aporcar",
    "aporcarán": "aporcar",
    "aporcarás": "aporcar",
    "aporcaré": "aporcar",
    "aporcaréis": "aporcar",
    "aporcaría": "aporcar",
    "aporcaríais": "aporcar",
    "aporcaríamos": "aporcar",
    "aporcarían": "aporcar",
    "aporcarías": "aporcar",
    "aporcas": "aporcar",
    "aporcase": "aporcar",
    "aporcaseis": "aporcar",
    "aporcasen": "aporcar",
    "aporcases": "aporcar",
    "aporcaste": "aporcar",
    "aporcasteis": "aporcar",
    "aporco": "aporcar",
    "aporcábamos": "aporcar",
    "aporcáis": "aporcar",
    "aporcáramos": "aporcar",
    "aporcáremos": "aporcar",
    "aporcásemos": "aporcar",
    "aporcó": "aporcar",
    "aporemas": "aporema",
    "aporque": "aporcar",
    "aporquemos": "aporcar",
    "aporquen": "aporcar",
    "aporques": "aporcar",
    "aporqué": "aporcar",
    "aporquéis": "aporcar",
    "aporra": "aporrar",
    "aporraba": "aporrar",
    "aporrabais": "aporrar",
    "aporraban": "aporrar",
    "aporrabas": "aporrar",
    "aporrad": "aporrar",
    "aporrada": "aporrar",
    "aporradas": "aporrar",
    "aporrado": "aporrar",
    "aporrados": "aporrar",
    "aporramos": "aporrar",
    "aporran": "aporrar",
    "aporrando": "aporrar",
    "aporrara": "aporrar",
    "aporrarais": "aporrar",
    "aporraran": "aporrar",
    "aporraras": "aporrar",
    "aporrare": "aporrar",
    "aporrareis": "aporrar",
    "aporraremos": "aporrar",
    "aporraren": "aporrar",
    "aporrares": "aporrar",
    "aporraron": "aporrar",
    "aporrará": "aporrar",
    "aporrarán": "aporrar",
    "aporrarás": "aporrar",
    "aporraré": "aporrar",
    "aporraréis": "aporrar",
    "aporraría": "aporrar",
    "aporraríais": "aporrar",
    "aporraríamos": "aporrar",
    "aporrarían": "aporrar",
    "aporrarías": "aporrar",
    "aporras": "aporrar",
    "aporrase": "aporrar",
    "aporraseis": "aporrar",
    "aporrasen": "aporrar",
    "aporrases": "aporrar",
    "aporraste": "aporrar",
    "aporrasteis": "aporrar",
    "aporre": "aporrar",
    "aporrea": "aporrear",
    "aporreaba": "aporrear",
    "aporreabais": "aporrear",
    "aporreaban": "aporrear",
    "aporreabas": "aporrear",
    "aporread": "aporrear",
    "aporreada": "aporrear",
    "aporreadas": "aporrear",
    "aporreado": "aporrear",
    "aporreadora": "aporreador",
    "aporreadoras": "aporreador",
    "aporreadores": "aporreador",
    "aporreados": "aporrear",
    "aporreaduras": "aporreadura",
    "aporreamientos": "aporreamiento",
    "aporreamos": "aporrear",
    "aporrean": "aporrear",
    "aporreando": "aporrear",
    "aporreara": "aporrear",
    "aporrearais": "aporrear",
    "aporrearan": "aporrear",
    "aporrearas": "aporrear",
    "aporreare": "aporrear",
    "aporreareis": "aporrear",
    "aporrearemos": "aporrear",
    "aporrearen": "aporrear",
    "aporreares": "aporrear",
    "aporrearon": "aporrear",
    "aporreará": "aporrear",
    "aporrearán": "aporrear",
    "aporrearás": "aporrear",
    "aporrearé": "aporrear",
    "aporrearéis": "aporrear",
    "aporrearía": "aporrear",
    "aporrearíais": "aporrear",
    "aporrearíamos": "aporrear",
    "aporrearían": "aporrear",
    "aporrearías": "aporrear",
    "aporreas": "aporrear",
    "aporrease": "aporrear",
    "aporreaseis": "aporrear",
    "aporreasen": "aporrear",
    "aporreases": "aporrear",
    "aporreaste": "aporrear",
    "aporreasteis": "aporrear",
    "aporree": "aporrear",
    "aporreemos": "aporrear",
    "aporreen": "aporrear",
    "aporrees": "aporrear",
    "aporremos": "aporrar",
    "aporren": "aporrar",
    "aporreo": "aporrear",
    "aporreos": "aporreo",
    "aporres": "aporrar",
    "aporreábamos": "aporrear",
    "aporreáis": "aporrear",
    "aporreáramos": "aporrear",
    "aporreáremos": "aporrear",
    "aporreásemos": "aporrear",
    "aporreé": "aporrear",
    "aporreéis": "aporrear",
    "aporreó": "aporrear",
    "aporro": "aporrar",
    "aporrábamos": "aporrar",
    "aporráis": "aporrar",
    "aporráramos": "aporrar",
    "aporráremos": "aporrar",
    "aporrásemos": "aporrar",
    "aporré": "aporrar",
    "aporréis": "aporrar",
    "aporró": "aporrar",
    "aporta": "aportar",
    "aportaba": "aportar",
    "aportabais": "aportar",
    "aportaban": "aportar",
    "aportabas": "aportar",
    "aportables": "aportable",
    "aportaciones": "aportación",
    "aportad": "aportar",
    "aportada": "aportar",
    "aportadas": "aportar",
    "aportaderas": "aportadera",
    "aportado": "aportar",
    "aportados": "aportar",
    "aportamos": "aportar",
    "aportan": "aportar",
    "aportando": "aportar",
    "aportantes": "aportante",
    "aportara": "aportar",
    "aportarais": "aportar",
    "aportaran": "aportar",
    "aportaras": "aportar",
    "aportare": "aportar",
    "aportareis": "aportar",
    "aportaremos": "aportar",
    "aportaren": "aportar",
    "aportares": "aportar",
    "aportaron": "aportar",
    "aportará": "aportar",
    "aportarán": "aportar",
    "aportarás": "aportar",
    "aportaré": "aportar",
    "aportaréis": "aportar",
    "aportaría": "aportar",
    "aportaríais": "aportar",
    "aportaríamos": "aportar",
    "aportarían": "aportar",
    "aportarías": "aportar",
    "aportas": "aportar",
    "aportase": "aportar",
    "aportaseis": "aportar",
    "aportasen": "aportar",
    "aportases": "aportar",
    "aportaste": "aportar",
    "aportasteis": "aportar",
    "aporte": "aportar",
    "aportemos": "aportar",
    "aporten": "aportar",
    "aportes": "aporte",
    "aportilla": "aportillar",
    "aportillaba": "aportillar",
    "aportillabais": "aportillar",
    "aportillaban": "aportillar",
    "aportillabas": "aportillar",
    "aportillad": "aportillar",
    "aportillada": "aportillar",
    "aportilladas": "aportillar",
    "aportillado": "aportillar",
    "aportillados": "aportillar",
    "aportillamos": "aportillar",
    "aportillan": "aportillar",
    "aportillando": "aportillar",
    "aportillara": "aportillar",
    "aportillarais": "aportillar",
    "aportillaran": "aportillar",
    "aportillaras": "aportillar",
    "aportillare": "aportillar",
    "aportillareis": "aportillar",
    "aportillaremos": "aportillar",
    "aportillaren": "aportillar",
    "aportillares": "aportillar",
    "aportillaron": "aportillar",
    "aportillará": "aportillar",
    "aportillarán": "aportillar",
    "aportillarás": "aportillar",
    "aportillaré": "aportillar",
    "aportillaréis": "aportillar",
    "aportillaría": "aportillar",
    "aportillaríais": "aportillar",
    "aportillaríamos": "aportillar",
    "aportillarían": "aportillar",
    "aportillarías": "aportillar",
    "aportillas": "aportillar",
    "aportillase": "aportillar",
    "aportillaseis": "aportillar",
    "aportillasen": "aportillar",
    "aportillases": "aportillar",
    "aportillaste": "aportillar",
    "aportillasteis": "aportillar",
    "aportille": "aportillar",
    "aportillemos": "aportillar",
    "aportillen": "aportillar",
    "aportilles": "aportillar",
    "aportillo": "aportillar",
    "aportillábamos": "aportillar",
    "aportilláis": "aportillar",
    "aportilláramos": "aportillar",
    "aportilláremos": "aportillar",
    "aportillásemos": "aportillar",
    "aportillé": "aportillar",
    "aportilléis": "aportillar",
    "aportilló": "aportillar",
    "aporto": "aportar",
    "aportábamos": "aportar",
    "aportáis": "aportar",
    "aportáramos": "aportar",
    "aportáremos": "aportar",
    "aportásemos": "aportar",
    "aporté": "aportar",
    "aportéis": "aportar",
    "aportó": "aportar",
    "aporías": "aporía",
    "aposenta": "aposentar",
    "aposentaba": "aposentar",
    "aposentabais": "aposentar",
    "aposentaban": "aposentar",
    "aposentabas": "aposentar",
    "aposentad": "aposentar",
    "aposentada": "aposentar",
    "aposentadas": "aposentar",
    "aposentado": "aposentar",
    "aposentadora": "aposentador",
    "aposentadoras": "aposentador",
    "aposentadores": "aposentador",
    "aposentados": "aposentar",
    "aposentadurías": "aposentaduría",
    "aposentamientos": "aposentamiento",
    "aposentamos": "aposentar",
    "aposentan": "aposentar",
    "aposentando": "aposentar",
    "aposentara": "aposentar",
    "aposentarais": "aposentar",
    "aposentaran": "aposentar",
    "aposentaras": "aposentar",
    "aposentare": "aposentar",
    "aposentareis": "aposentar",
    "aposentaremos": "aposentar",
    "aposentaren": "aposentar",
    "aposentares": "aposentar",
    "aposentaron": "aposentar",
    "aposentará": "aposentar",
    "aposentarán": "aposentar",
    "aposentarás": "aposentar",
    "aposentaré": "aposentar",
    "aposentaréis": "aposentar",
    "aposentaría": "aposentar",
    "aposentaríais": "aposentar",
    "aposentaríamos": "aposentar",
    "aposentarían": "aposentar",
    "aposentarías": "aposentar",
    "aposentas": "aposentar",
    "aposentase": "aposentar",
    "aposentaseis": "aposentar",
    "aposentasen": "aposentar",
    "aposentases": "aposentar",
    "aposentaste": "aposentar",
    "aposentasteis": "aposentar",
    "aposente": "aposentar",
    "aposentemos": "aposentar",
    "aposenten": "aposentar",
    "aposentes": "aposentar",
    "aposento": "aposentar",
    "aposentos": "aposento",
    "aposentábamos": "aposentar",
    "aposentáis": "aposentar",
    "aposentáramos": "aposentar",
    "aposentáremos": "aposentar",
    "aposentásemos": "aposentar",
    "aposenté": "aposentar",
    "aposentéis": "aposentar",
    "aposentó": "aposentar",
    "aposesiona": "aposesionar",
    "aposesionaba": "aposesionar",
    "aposesionabais": "aposesionar",
    "aposesionaban": "aposesionar",
    "aposesionabas": "aposesionar",
    "aposesionad": "aposesionar",
    "aposesionada": "aposesionar",
    "aposesionadas": "aposesionar",
    "aposesionado": "aposesionar",
    "aposesionados": "aposesionar",
    "aposesionamos": "aposesionar",
    "aposesionan": "aposesionar",
    "aposesionando": "aposesionar",
    "aposesionara": "aposesionar",
    "aposesionarais": "aposesionar",
    "aposesionaran": "aposesionar",
    "aposesionaras": "aposesionar",
    "aposesionare": "aposesionar",
    "aposesionareis": "aposesionar",
    "aposesionaremos": "aposesionar",
    "aposesionaren": "aposesionar",
    "aposesionares": "aposesionar",
    "aposesionaron": "aposesionar",
    "aposesionará": "aposesionar",
    "aposesionarán": "aposesionar",
    "aposesionarás": "aposesionar",
    "aposesionaré": "aposesionar",
    "aposesionaréis": "aposesionar",
    "aposesionaría": "aposesionar",
    "aposesionaríais": "aposesionar",
    "aposesionaríamos": "aposesionar",
    "aposesionarían": "aposesionar",
    "aposesionarías": "aposesionar",
    "aposesionas": "aposesionar",
    "aposesionase": "aposesionar",
    "aposesionaseis": "aposesionar",
    "aposesionasen": "aposesionar",
    "aposesionases": "aposesionar",
    "aposesionaste": "aposesionar",
    "aposesionasteis": "aposesionar",
    "aposesione": "aposesionar",
    "aposesionemos": "aposesionar",
    "aposesionen": "aposesionar",
    "aposesiones": "aposesionar",
    "aposesiono": "aposesionar",
    "aposesionábamos": "aposesionar",
    "aposesionáis": "aposesionar",
    "aposesionáramos": "aposesionar",
    "aposesionáremos": "aposesionar",
    "aposesionásemos": "aposesionar",
    "aposesioné": "aposesionar",
    "aposesionéis": "aposesionar",
    "aposesionó": "aposesionar",
    "aposiciones": "aposición",
    "aposta": "apostar",
    "apostaba": "apostar",
    "apostabais": "apostar",
    "apostaban": "apostar",
    "apostabas": "apostar",
    "apostad": "apostar",
    "apostada": "apostar",
    "apostadas": "apostar",
    "apostaderos": "apostadero",
    "apostado": "apostar",
    "apostados": "apostar",
    "apostamos": "apostar",
    "apostan": "apostar",
    "apostando": "apostar",
    "apostantes": "apostante",
    "apostara": "apostar",
    "apostarais": "apostar",
    "apostaran": "apostar",
    "apostaras": "apostar",
    "apostare": "apostar",
    "apostareis": "apostar",
    "apostaremos": "apostar",
    "apostaren": "apostar",
    "apostares": "apostar",
    "apostaron": "apostar",
    "apostará": "apostar",
    "apostarán": "apostar",
    "apostarás": "apostar",
    "apostaré": "apostar",
    "apostaréis": "apostar",
    "apostaría": "apostar",
    "apostaríais": "apostar",
    "apostaríamos": "apostar",
    "apostarían": "apostar",
    "apostarías": "apostar",
    "apostas": "apostar",
    "apostase": "apostar",
    "apostaseis": "apostar",
    "apostasen": "apostar",
    "apostases": "apostar",
    "apostaste": "apostar",
    "apostasteis": "apostar",
    "apostasías": "apostasía",
    "apostata": "apostatar",
    "apostataba": "apostatar",
    "apostatabais": "apostatar",
    "apostataban": "apostatar",
    "apostatabas": "apostatar",
    "apostataciones": "apostatación",
    "apostatad": "apostatar",
    "apostatada": "apostatar",
    "apostatadas": "apostatar",
    "apostatado": "apostatar",
    "apostatados": "apostatar",
    "apostatamos": "apostatar",
    "apostatan": "apostatar",
    "apostatando": "apostatar",
    "apostatara": "apostatar",
    "apostatarais": "apostatar",
    "apostataran": "apostatar",
    "apostataras": "apostatar",
    "apostatare": "apostatar",
    "apostatareis": "apostatar",
    "apostataremos": "apostatar",
    "apostataren": "apostatar",
    "apostatares": "apostatar",
    "apostataron": "apostatar",
    "apostatará": "apostatar",
    "apostatarán": "apostatar",
    "apostatarás": "apostatar",
    "apostataré": "apostatar",
    "apostataréis": "apostatar",
    "apostataría": "apostatar",
    "apostataríais": "apostatar",
    "apostataríamos": "apostatar",
    "apostatarían": "apostatar",
    "apostatarías": "apostatar",
    "apostatas": "apostatar",
    "apostatase": "apostatar",
    "apostataseis": "apostatar",
    "apostatasen": "apostatar",
    "apostatases": "apostatar",
    "apostataste": "apostatar",
    "apostatasteis": "apostatar",
    "apostate": "apostatar",
    "apostatemos": "apostatar",
    "apostaten": "apostatar",
    "apostates": "apostatar",
    "apostato": "apostatar",
    "apostatábamos": "apostatar",
    "apostatáis": "apostatar",
    "apostatáramos": "apostatar",
    "apostatáremos": "apostatar",
    "apostatásemos": "apostatar",
    "apostaté": "apostatar",
    "apostatéis": "apostatar",
    "apostató": "apostatar",
    "aposte": "apostar",
    "apostema": "apostemar",
    "apostemaba": "apostemar",
    "apostemabais": "apostemar",
    "apostemaban": "apostemar",
    "apostemabas": "apostemar",
    "apostemad": "apostemar",
    "apostemada": "apostemar",
    "apostemadas": "apostemar",
    "apostemado": "apostemar",
    "apostemados": "apostemar",
    "apostemamos": "apostemar",
    "aposteman": "apostemar",
    "apostemando": "apostemar",
    "apostemara": "apostemar",
    "apostemarais": "apostemar",
    "apostemaran": "apostemar",
    "apostemaras": "apostemar",
    "apostemare": "apostemar",
    "apostemareis": "apostemar",
    "apostemaremos": "apostemar",
    "apostemaren": "apostemar",
    "apostemares": "apostemar",
    "apostemaron": "apostemar",
    "apostemará": "apostemar",
    "apostemarán": "apostemar",
    "apostemarás": "apostemar",
    "apostemaré": "apostemar",
    "apostemaréis": "apostemar",
    "apostemaría": "apostemar",
    "apostemaríais": "apostemar",
    "apostemaríamos": "apostemar",
    "apostemarían": "apostemar",
    "apostemarías": "apostemar",
    "apostemas": "apostemar",
    "apostemase": "apostemar",
    "apostemaseis": "apostemar",
    "apostemasen": "apostemar",
    "apostemases": "apostemar",
    "apostemaste": "apostemar",
    "apostemasteis": "apostemar",
    "aposteme": "apostemar",
    "apostememos": "apostemar",
    "apostemen": "apostemar",
    "apostemes": "apostemar",
    "apostemo": "apostemar",
    "apostemos": "apostar",
    "apostemábamos": "apostemar",
    "apostemáis": "apostemar",
    "apostemáramos": "apostemar",
    "apostemáremos": "apostemar",
    "apostemásemos": "apostemar",
    "apostemé": "apostemar",
    "aposteméis": "apostemar",
    "apostemó": "apostemar",
    "aposten": "apostar",
    "apostes": "apostar",
    "apostilla": "apostillar",
    "apostillaba": "apostillar",
    "apostillabais": "apostillar",
    "apostillaban": "apostillar",
    "apostillabas": "apostillar",
    "apostillad": "apostillar",
    "apostillada": "apostillar",
    "apostilladas": "apostillar",
    "apostillado": "apostillar",
    "apostillados": "apostillar",
    "apostillamos": "apostillar",
    "apostillan": "apostillar",
    "apostillando": "apostillar",
    "apostillara": "apostillar",
    "apostillarais": "apostillar",
    "apostillaran": "apostillar",
    "apostillaras": "apostillar",
    "apostillare": "apostillar",
    "apostillareis": "apostillar",
    "apostillaremos": "apostillar",
    "apostillaren": "apostillar",
    "apostillares": "apostillar",
    "apostillaron": "apostillar",
    "apostillará": "apostillar",
    "apostillarán": "apostillar",
    "apostillarás": "apostillar",
    "apostillaré": "apostillar",
    "apostillaréis": "apostillar",
    "apostillaría": "apostillar",
    "apostillaríais": "apostillar",
    "apostillaríamos": "apostillar",
    "apostillarían": "apostillar",
    "apostillarías": "apostillar",
    "apostillas": "apostillar",
    "apostillase": "apostillar",
    "apostillaseis": "apostillar",
    "apostillasen": "apostillar",
    "apostillases": "apostillar",
    "apostillaste": "apostillar",
    "apostillasteis": "apostillar",
    "apostille": "apostillar",
    "apostillemos": "apostillar",
    "apostillen": "apostillar",
    "apostilles": "apostillar",
    "apostillo": "apostillar",
    "apostillábamos": "apostillar",
    "apostilláis": "apostillar",
    "apostilláramos": "apostillar",
    "apostilláremos": "apostillar",
    "apostillásemos": "apostillar",
    "apostillé": "apostillar",
    "apostilléis": "apostillar",
    "apostilló": "apostillar",
    "aposto": "apostar",
    "apostolados": "apostolado",
    "apostolicidades": "apostolicidad",
    "apostrofa": "apostrofar",
    "apostrofaba": "apostrofar",
    "apostrofabais": "apostrofar",
    "apostrofaban": "apostrofar",
    "apostrofabas": "apostrofar",
    "apostrofad": "apostrofar",
    "apostrofada": "apostrofar",
    "apostrofadas": "apostrofar",
    "apostrofado": "apostrofar",
    "apostrofados": "apostrofar",
    "apostrofamos": "apostrofar",
    "apostrofan": "apostrofar",
    "apostrofando": "apostrofar",
    "apostrofara": "apostrofar",
    "apostrofarais": "apostrofar",
    "apostrofaran": "apostrofar",
    "apostrofaras": "apostrofar",
    "apostrofare": "apostrofar",
    "apostrofareis": "apostrofar",
    "apostrofaremos": "apostrofar",
    "apostrofaren": "apostrofar",
    "apostrofares": "apostrofar",
    "apostrofaron": "apostrofar",
    "apostrofará": "apostrofar",
    "apostrofarán": "apostrofar",
    "apostrofarás": "apostrofar",
    "apostrofaré": "apostrofar",
    "apostrofaréis": "apostrofar",
    "apostrofaría": "apostrofar",
    "apostrofaríais": "apostrofar",
    "apostrofaríamos": "apostrofar",
    "apostrofarían": "apostrofar",
    "apostrofarías": "apostrofar",
    "apostrofas": "apostrofar",
    "apostrofase": "apostrofar",
    "apostrofaseis": "apostrofar",
    "apostrofasen": "apostrofar",
    "apostrofases": "apostrofar",
    "apostrofaste": "apostrofar",
    "apostrofasteis": "apostrofar",
    "apostrofe": "apostrofar",
    "apostrofemos": "apostrofar",
    "apostrofen": "apostrofar",
    "apostrofes": "apostrofar",
    "apostrofo": "apostrofar",
    "apostrofábamos": "apostrofar",
    "apostrofáis": "apostrofar",
    "apostrofáramos": "apostrofar",
    "apostrofáremos": "apostrofar",
    "apostrofásemos": "apostrofar",
    "apostrofé": "apostrofar",
    "apostroféis": "apostrofar",
    "apostrofó": "apostrofar",
    "aposturas": "apostura",
    "apostábamos": "apostar",
    "apostáis": "apostar",
    "apostáramos": "apostar",
    "apostáremos": "apostar",
    "apostásemos": "apostar",
    "aposté": "apostar",
    "apostéis": "apostar",
    "apostó": "apostar",
    "apostólica": "apostólico",
    "apostólicas": "apostólico",
    "apostólicos": "apostólico",
    "apotegmas": "apotegma",
    "apotemas": "apotema",
    "apoteósica": "apoteósico",
    "apoteósicas": "apoteósico",
    "apoteósicos": "apoteósico",
    "apotrera": "apotrerar",
    "apotreraba": "apotrerar",
    "apotrerabais": "apotrerar",
    "apotreraban": "apotrerar",
    "apotrerabas": "apotrerar",
    "apotrerad": "apotrerar",
    "apotrerada": "apotrerar",
    "apotreradas": "apotrerar",
    "apotrerado": "apotrerar",
    "apotrerados": "apotrerar",
    "apotreramos": "apotrerar",
    "apotreran": "apotrerar",
    "apotrerando": "apotrerar",
    "apotrerara": "apotrerar",
    "apotrerarais": "apotrerar",
    "apotreraran": "apotrerar",
    "apotreraras": "apotrerar",
    "apotrerare": "apotrerar",
    "apotrerareis": "apotrerar",
    "apotreraremos": "apotrerar",
    "apotreraren": "apotrerar",
    "apotrerares": "apotrerar",
    "apotreraron": "apotrerar",
    "apotrerará": "apotrerar",
    "apotrerarán": "apotrerar",
    "apotrerarás": "apotrerar",
    "apotreraré": "apotrerar",
    "apotreraréis": "apotrerar",
    "apotreraría": "apotrerar",
    "apotreraríais": "apotrerar",
    "apotreraríamos": "apotrerar",
    "apotrerarían": "apotrerar",
    "apotrerarías": "apotrerar",
    "apotreras": "apotrerar",
    "apotrerase": "apotrerar",
    "apotreraseis": "apotrerar",
    "apotrerasen": "apotrerar",
    "apotrerases": "apotrerar",
    "apotreraste": "apotrerar",
    "apotrerasteis": "apotrerar",
    "apotrere": "apotrerar",
    "apotreremos": "apotrerar",
    "apotreren": "apotrerar",
    "apotreres": "apotrerar",
    "apotrero": "apotrerar",
    "apotrerábamos": "apotrerar",
    "apotreráis": "apotrerar",
    "apotreráramos": "apotrerar",
    "apotreráremos": "apotrerar",
    "apotrerásemos": "apotrerar",
    "apotreré": "apotrerar",
    "apotreréis": "apotrerar",
    "apotreró": "apotrerar",
    "apoya": "apoyar",
    "apoyaba": "apoyar",
    "apoyabais": "apoyar",
    "apoyaban": "apoyar",
    "apoyabas": "apoyar",
    "apoyad": "apoyar",
    "apoyada": "apoyar",
    "apoyadas": "apoyar",
    "apoyado": "apoyar",
    "apoyados": "apoyar",
    "apoyaduras": "apoyadura",
    "apoyamos": "apoyar",
    "apoyan": "apoyar",
    "apoyando": "apoyar",
    "apoyara": "apoyar",
    "apoyarais": "apoyar",
    "apoyaran": "apoyar",
    "apoyaras": "apoyar",
    "apoyare": "apoyar",
    "apoyareis": "apoyar",
    "apoyaremos": "apoyar",
    "apoyaren": "apoyar",
    "apoyares": "apoyar",
    "apoyaron": "apoyar",
    "apoyará": "apoyar",
    "apoyarán": "apoyar",
    "apoyarás": "apoyar",
    "apoyaré": "apoyar",
    "apoyaréis": "apoyar",
    "apoyaría": "apoyar",
    "apoyaríais": "apoyar",
    "apoyaríamos": "apoyar",
    "apoyarían": "apoyar",
    "apoyarías": "apoyar",
    "apoyas": "apoyar",
    "apoyase": "apoyar",
    "apoyaseis": "apoyar",
    "apoyasen": "apoyar",
    "apoyases": "apoyar",
    "apoyaste": "apoyar",
    "apoyasteis": "apoyar",
    "apoyaturas": "apoyatura",
    "apoye": "apoyar",
    "apoyemos": "apoyar",
    "apoyen": "apoyar",
    "apoyes": "apoyar",
    "apoyo": "apoyar",
    "apoyos": "apoyo",
    "apoyábamos": "apoyar",
    "apoyáis": "apoyar",
    "apoyáramos": "apoyar",
    "apoyáremos": "apoyar",
    "apoyásemos": "apoyar",
    "apoyé": "apoyar",
    "apoyéis": "apoyar",
    "apoyó": "apoyar",
    "approaches": "approach",
    "apraxias": "apraxia",
    "aprecia": "apreciar",
    "apreciaba": "apreciar",
    "apreciabais": "apreciar",
    "apreciaban": "apreciar",
    "apreciabas": "apreciar",
    "apreciabilidades": "apreciabilidad",
    "apreciables": "apreciable",
    "apreciaciones": "apreciación",
    "apreciad": "apreciar",
    "apreciada": "apreciar",
    "apreciadas": "apreciar",
    "apreciado": "apreciar",
    "apreciadora": "apreciador",
    "apreciadoras": "apreciador",
    "apreciadores": "apreciador",
    "apreciados": "apreciar",
    "apreciamos": "apreciar",
    "aprecian": "apreciar",
    "apreciando": "apreciar",
    "apreciara": "apreciar",
    "apreciarais": "apreciar",
    "apreciaran": "apreciar",
    "apreciaras": "apreciar",
    "apreciare": "apreciar",
    "apreciareis": "apreciar",
    "apreciaremos": "apreciar",
    "apreciaren": "apreciar",
    "apreciares": "apreciar",
    "apreciaron": "apreciar",
    "apreciará": "apreciar",
    "apreciarán": "apreciar",
    "apreciarás": "apreciar",
    "apreciaré": "apreciar",
    "apreciaréis": "apreciar",
    "apreciaría": "apreciar",
    "apreciaríais": "apreciar",
    "apreciaríamos": "apreciar",
    "apreciarían": "apreciar",
    "apreciarías": "apreciar",
    "aprecias": "apreciar",
    "apreciase": "apreciar",
    "apreciaseis": "apreciar",
    "apreciasen": "apreciar",
    "apreciases": "apreciar",
    "apreciaste": "apreciar",
    "apreciasteis": "apreciar",
    "apreciativa": "apreciativo",
    "apreciativas": "apreciativo",
    "apreciativos": "apreciativo",
    "aprecie": "apreciar",
    "apreciemos": "apreciar",
    "aprecien": "apreciar",
    "aprecies": "apreciar",
    "aprecio": "apreciar",
    "aprecios": "aprecio",
    "apreciábamos": "apreciar",
    "apreciáis": "apreciar",
    "apreciáramos": "apreciar",
    "apreciáremos": "apreciar",
    "apreciásemos": "apreciar",
    "aprecié": "apreciar",
    "apreciéis": "apreciar",
    "apreció": "apreciar",
    "aprehenda": "aprehender",
    "aprehendamos": "aprehender",
    "aprehendan": "aprehender",
    "aprehendas": "aprehender",
    "aprehende": "aprehender",
    "aprehended": "aprehender",
    "aprehendemos": "aprehender",
    "aprehenden": "aprehender",
    "aprehenderemos": "aprehender",
    "aprehenderá": "aprehender",
    "aprehenderán": "aprehender",
    "aprehenderás": "aprehender",
    "aprehenderé": "aprehender",
    "aprehenderéis": "aprehender",
    "aprehendería": "aprehender",
    "aprehenderíais": "aprehender",
    "aprehenderíamos": "aprehender",
    "aprehenderían": "aprehender",
    "aprehenderías": "aprehender",
    "aprehendes": "aprehender",
    "aprehendida": "aprehender",
    "aprehendidas": "aprehender",
    "aprehendido": "aprehender",
    "aprehendidos": "aprehender",
    "aprehendiendo": "aprehender",
    "aprehendiera": "aprehender",
    "aprehendierais": "aprehender",
    "aprehendieran": "aprehender",
    "aprehendieras": "aprehender",
    "aprehendiere": "aprehender",
    "aprehendiereis": "aprehender",
    "aprehendieren": "aprehender",
    "aprehendieres": "aprehender",
    "aprehendieron": "aprehender",
    "aprehendiese": "aprehender",
    "aprehendieseis": "aprehender",
    "aprehendiesen": "aprehender",
    "aprehendieses": "aprehender",
    "aprehendimos": "aprehender",
    "aprehendiste": "aprehender",
    "aprehendisteis": "aprehender",
    "aprehendiéramos": "aprehender",
    "aprehendiéremos": "aprehender",
    "aprehendiésemos": "aprehender",
    "aprehendió": "aprehender",
    "aprehendo": "aprehender",
    "aprehendáis": "aprehender",
    "aprehendéis": "aprehender",
    "aprehendí": "aprehender",
    "aprehendía": "aprehender",
    "aprehendíais": "aprehender",
    "aprehendíamos": "aprehender",
    "aprehendían": "aprehender",
    "aprehendías": "aprehender",
    "aprehensibles": "aprehensible",
    "aprehensiones": "aprehensión",
    "aprehensiva": "aprehensivo",
    "aprehensivas": "aprehensivo",
    "aprehensivos": "aprehensivo",
    "aprehensora": "aprehensor",
    "aprehensoras": "aprehensor",
    "aprehensores": "aprehensor",
    "apremia": "apremiar",
    "apremiaba": "apremiar",
    "apremiabais": "apremiar",
    "apremiaban": "apremiar",
    "apremiabas": "apremiar",
    "apremiad": "apremiar",
    "apremiada": "apremiar",
    "apremiadas": "apremiar",
    "apremiado": "apremiar",
    "apremiados": "apremiar",
    "apremiamos": "apremiar",
    "apremian": "apremiar",
    "apremiando": "apremiar",
    "apremiantes": "apremiante",
    "apremiara": "apremiar",
    "apremiarais": "apremiar",
    "apremiaran": "apremiar",
    "apremiaras": "apremiar",
    "apremiare": "apremiar",
    "apremiareis": "apremiar",
    "apremiaremos": "apremiar",
    "apremiaren": "apremiar",
    "apremiares": "apremiar",
    "apremiaron": "apremiar",
    "apremiará": "apremiar",
    "apremiarán": "apremiar",
    "apremiarás": "apremiar",
    "apremiaré": "apremiar",
    "apremiaréis": "apremiar",
    "apremiaría": "apremiar",
    "apremiaríais": "apremiar",
    "apremiaríamos": "apremiar",
    "apremiarían": "apremiar",
    "apremiarías": "apremiar",
    "apremias": "apremiar",
    "apremiase": "apremiar",
    "apremiaseis": "apremiar",
    "apremiasen": "apremiar",
    "apremiases": "apremiar",
    "apremiaste": "apremiar",
    "apremiasteis": "apremiar",
    "apremie": "apremiar",
    "apremiemos": "apremiar",
    "apremien": "apremiar",
    "apremies": "apremiar",
    "apremio": "apremiar",
    "apremios": "apremio",
    "apremiábamos": "apremiar",
    "apremiáis": "apremiar",
    "apremiáramos": "apremiar",
    "apremiáremos": "apremiar",
    "apremiásemos": "apremiar",
    "apremié": "apremiar",
    "apremiéis": "apremiar",
    "apremió": "apremiar",
    "aprenda": "aprender",
    "aprendamos": "aprender",
    "aprendan": "aprender",
    "aprendas": "aprender",
    "aprende": "aprender",
    "aprended": "aprender",
    "aprendemos": "aprender",
    "aprenden": "aprender",
    "aprenderemos": "aprender",
    "aprenderá": "aprender",
    "aprenderán": "aprender",
    "aprenderás": "aprender",
    "aprenderé": "aprender",
    "aprenderéis": "aprender",
    "aprendería": "aprender",
    "aprenderíais": "aprender",
    "aprenderíamos": "aprender",
    "aprenderían": "aprender",
    "aprenderías": "aprender",
    "aprendes": "aprender",
    "aprendices": "aprendiz",
    "aprendida": "aprender",
    "aprendidas": "aprender",
    "aprendido": "aprender",
    "aprendidos": "aprender",
    "aprendiendo": "aprender",
    "aprendiera": "aprender",
    "aprendierais": "aprender",
    "aprendieran": "aprender",
    "aprendieras": "aprender",
    "aprendiere": "aprender",
    "aprendiereis": "aprender",
    "aprendieren": "aprender",
    "aprendieres": "aprender",
    "aprendieron": "aprender",
    "aprendiese": "aprender",
    "aprendieseis": "aprender",
    "aprendiesen": "aprender",
    "aprendieses": "aprender",
    "aprendimos": "aprender",
    "aprendiste": "aprender",
    "aprendisteis": "aprender",
    "aprendiza": "aprendiz",
    "aprendizajes": "aprendizaje",
    "aprendizas": "aprendiz",
    "aprendiéramos": "aprender",
    "aprendiéremos": "aprender",
    "aprendiésemos": "aprender",
    "aprendió": "aprender",
    "aprendo": "aprender",
    "aprendáis": "aprender",
    "aprendéis": "aprender",
    "aprendí": "aprender",
    "aprendía": "aprender",
    "aprendíais": "aprender",
    "aprendíamos": "aprender",
    "aprendían": "aprender",
    "aprendías": "aprender",
    "aprensa": "aprensar",
    "aprensaba": "aprensar",
    "aprensabais": "aprensar",
    "aprensaban": "aprensar",
    "aprensabas": "aprensar",
    "aprensad": "aprensar",
    "aprensada": "aprensar",
    "aprensadas": "aprensar",
    "aprensado": "aprensar",
    "aprensados": "aprensar",
    "aprensamos": "aprensar",
    "aprensan": "aprensar",
    "aprensando": "aprensar",
    "aprensara": "aprensar",
    "aprensarais": "aprensar",
    "aprensaran": "aprensar",
    "aprensaras": "aprensar",
    "aprensare": "aprensar",
    "aprensareis": "aprensar",
    "aprensaremos": "aprensar",
    "aprensaren": "aprensar",
    "aprensares": "aprensar",
    "aprensaron": "aprensar",
    "aprensará": "aprensar",
    "aprensarán": "aprensar",
    "aprensarás": "aprensar",
    "aprensaré": "aprensar",
    "aprensaréis": "aprensar",
    "aprensaría": "aprensar",
    "aprensaríais": "aprensar",
    "aprensaríamos": "aprensar",
    "aprensarían": "aprensar",
    "aprensarías": "aprensar",
    "aprensas": "aprensar",
    "aprensase": "aprensar",
    "aprensaseis": "aprensar",
    "aprensasen": "aprensar",
    "aprensases": "aprensar",
    "aprensaste": "aprensar",
    "aprensasteis": "aprensar",
    "aprense": "aprensar",
    "aprensemos": "aprensar",
    "aprensen": "aprensar",
    "aprenses": "aprensar",
    "aprensiones": "aprensión",
    "aprensiva": "aprensivo",
    "aprensivas": "aprensivo",
    "aprensivos": "aprensivo",
    "aprenso": "aprensar",
    "aprensábamos": "aprensar",
    "aprensáis": "aprensar",
    "aprensáramos": "aprensar",
    "aprensáremos": "aprensar",
    "aprensásemos": "aprensar",
    "aprensé": "aprensar",
    "aprenséis": "aprensar",
    "aprensó": "aprensar",
    "apresa": "apresar",
    "apresaba": "apresar",
    "apresabais": "apresar",
    "apresaban": "apresar",
    "apresabas": "apresar",
    "apresad": "apresar",
    "apresada": "apresar",
    "apresadas": "apresar",
    "apresado": "apresar",
    "apresadora": "apresador",
    "apresadoras": "apresador",
    "apresadores": "apresador",
    "apresados": "apresar",
    "apresamientos": "apresamiento",
    "apresamos": "apresar",
    "apresan": "apresar",
    "apresando": "apresar",
    "apresara": "apresar",
    "apresarais": "apresar",
    "apresaran": "apresar",
    "apresaras": "apresar",
    "apresare": "apresar",
    "apresareis": "apresar",
    "apresaremos": "apresar",
    "apresaren": "apresar",
    "apresares": "apresar",
    "apresaron": "apresar",
    "apresará": "apresar",
    "apresarán": "apresar",
    "apresarás": "apresar",
    "apresaré": "apresar",
    "apresaréis": "apresar",
    "apresaría": "apresar",
    "apresaríais": "apresar",
    "apresaríamos": "apresar",
    "apresarían": "apresar",
    "apresarías": "apresar",
    "apresas": "apresar",
    "apresase": "apresar",
    "apresaseis": "apresar",
    "apresasen": "apresar",
    "apresases": "apresar",
    "apresaste": "apresar",
    "apresasteis": "apresar",
    "aprese": "apresar",
    "apresemos": "apresar",
    "apresen": "apresar",
    "apreses": "apresar",
    "apreso": "apresar",
    "apresta": "aprestar",
    "aprestaba": "aprestar",
    "aprestabais": "aprestar",
    "aprestaban": "aprestar",
    "aprestabas": "aprestar",
    "aprestad": "aprestar",
    "aprestada": "aprestar",
    "aprestadas": "aprestar",
    "aprestado": "aprestar",
    "aprestados": "aprestar",
    "aprestamos": "aprestar",
    "aprestan": "aprestar",
    "aprestando": "aprestar",
    "aprestara": "aprestar",
    "aprestarais": "aprestar",
    "aprestaran": "aprestar",
    "aprestaras": "aprestar",
    "aprestare": "aprestar",
    "aprestareis": "aprestar",
    "aprestaremos": "aprestar",
    "aprestaren": "aprestar",
    "aprestares": "aprestar",
    "aprestaron": "aprestar",
    "aprestará": "aprestar",
    "aprestarán": "aprestar",
    "aprestarás": "aprestar",
    "aprestaré": "aprestar",
    "aprestaréis": "aprestar",
    "aprestaría": "aprestar",
    "aprestaríais": "aprestar",
    "aprestaríamos": "aprestar",
    "aprestarían": "aprestar",
    "aprestarías": "aprestar",
    "aprestas": "aprestar",
    "aprestase": "aprestar",
    "aprestaseis": "aprestar",
    "aprestasen": "aprestar",
    "aprestases": "aprestar",
    "aprestaste": "aprestar",
    "aprestasteis": "aprestar",
    "apreste": "aprestar",
    "aprestemos": "aprestar",
    "apresten": "aprestar",
    "aprestes": "aprestar",
    "apresto": "aprestar",
    "aprestos": "apresto",
    "aprestábamos": "aprestar",
    "aprestáis": "aprestar",
    "aprestáramos": "aprestar",
    "aprestáremos": "aprestar",
    "aprestásemos": "aprestar",
    "apresté": "aprestar",
    "aprestéis": "aprestar",
    "aprestó": "aprestar",
    "apresura": "apresurar",
    "apresuraba": "apresurar",
    "apresurabais": "apresurar",
    "apresuraban": "apresurar",
    "apresurabas": "apresurar",
    "apresurad": "apresurar",
    "apresurada": "apresurar",
    "apresuradas": "apresurar",
    "apresurado": "apresurar",
    "apresurados": "apresurar",
    "apresuramientos": "apresuramiento",
    "apresuramos": "apresurar",
    "apresuran": "apresurar",
    "apresurando": "apresurar",
    "apresurara": "apresurar",
    "apresurarais": "apresurar",
    "apresuraran": "apresurar",
    "apresuraras": "apresurar",
    "apresurare": "apresurar",
    "apresurareis": "apresurar",
    "apresuraremos": "apresurar",
    "apresuraren": "apresurar",
    "apresurares": "apresurar",
    "apresuraron": "apresurar",
    "apresurará": "apresurar",
    "apresurarán": "apresurar",
    "apresurarás": "apresurar",
    "apresuraré": "apresurar",
    "apresuraréis": "apresurar",
    "apresuraría": "apresurar",
    "apresuraríais": "apresurar",
    "apresuraríamos": "apresurar",
    "apresurarían": "apresurar",
    "apresurarías": "apresurar",
    "apresuras": "apresurar",
    "apresurase": "apresurar",
    "apresuraseis": "apresurar",
    "apresurasen": "apresurar",
    "apresurases": "apresurar",
    "apresuraste": "apresurar",
    "apresurasteis": "apresurar",
    "apresure": "apresurar",
    "apresuremos": "apresurar",
    "apresuren": "apresurar",
    "apresures": "apresurar",
    "apresuro": "apresurar",
    "apresurábamos": "apresurar",
    "apresuráis": "apresurar",
    "apresuráramos": "apresurar",
    "apresuráremos": "apresurar",
    "apresurásemos": "apresurar",
    "apresuré": "apresurar",
    "apresuréis": "apresurar",
    "apresuró": "apresurar",
    "apresábamos": "apresar",
    "apresáis": "apresar",
    "apresáramos": "apresar",
    "apresáremos": "apresar",
    "apresásemos": "apresar",
    "apresé": "apresar",
    "apreséis": "apresar",
    "apresó": "apresar",
    "apretaba": "apretar",
    "apretabais": "apretar",
    "apretaban": "apretar",
    "apretabas": "apretar",
    "apretad": "apretar",
    "apretada": "apretar",
    "apretadas": "apretar",
    "apretado": "apretar",
    "apretadora": "apretador",
    "apretadoras": "apretador",
    "apretadores": "apretador",
    "apretados": "apretar",
    "apretaduras": "apretadura",
    "apretamientos": "apretamiento",
    "apretamos": "apretar",
    "apretando": "apretar",
    "apretara": "apretar",
    "apretarais": "apretar",
    "apretaran": "apretar",
    "apretaras": "apretar",
    "apretare": "apretar",
    "apretareis": "apretar",
    "apretaremos": "apretar",
    "apretaren": "apretar",
    "apretares": "apretar",
    "apretaron": "apretar",
    "apretará": "apretar",
    "apretarán": "apretar",
    "apretarás": "apretar",
    "apretaré": "apretar",
    "apretaréis": "apretar",
    "apretaría": "apretar",
    "apretaríais": "apretar",
    "apretaríamos": "apretar",
    "apretarían": "apretar",
    "apretarías": "apretar",
    "apretase": "apretar",
    "apretaseis": "apretar",
    "apretasen": "apretar",
    "apretases": "apretar",
    "apretaste": "apretar",
    "apretasteis": "apretar",
    "apretemos": "apretar",
    "apretones": "apretón",
    "apretuja": "apretujar",
    "apretujaba": "apretujar",
    "apretujabais": "apretujar",
    "apretujaban": "apretujar",
    "apretujabas": "apretujar",
    "apretujad": "apretujar",
    "apretujada": "apretujar",
    "apretujadas": "apretujar",
    "apretujado": "apretujar",
    "apretujados": "apretujar",
    "apretujamos": "apretujar",
    "apretujan": "apretujar",
    "apretujando": "apretujar",
    "apretujara": "apretujar",
    "apretujarais": "apretujar",
    "apretujaran": "apretujar",
    "apretujaras": "apretujar",
    "apretujare": "apretujar",
    "apretujareis": "apretujar",
    "apretujaremos": "apretujar",
    "apretujaren": "apretujar",
    "apretujares": "apretujar",
    "apretujaron": "apretujar",
    "apretujará": "apretujar",
    "apretujarán": "apretujar",
    "apretujarás": "apretujar",
    "apretujaré": "apretujar",
    "apretujaréis": "apretujar",
    "apretujaría": "apretujar",
    "apretujaríais": "apretujar",
    "apretujaríamos": "apretujar",
    "apretujarían": "apretujar",
    "apretujarías": "apretujar",
    "apretujas": "apretujar",
    "apretujase": "apretujar",
    "apretujaseis": "apretujar",
    "apretujasen": "apretujar",
    "apretujases": "apretujar",
    "apretujaste": "apretujar",
    "apretujasteis": "apretujar",
    "apretuje": "apretujar",
    "apretujemos": "apretujar",
    "apretujen": "apretujar",
    "apretujes": "apretujar",
    "apretujo": "apretujar",
    "apretujones": "apretujón",
    "apretujábamos": "apretujar",
    "apretujáis": "apretujar",
    "apretujáramos": "apretujar",
    "apretujáremos": "apretujar",
    "apretujásemos": "apretujar",
    "apretujé": "apretujar",
    "apretujéis": "apretujar",
    "apretujó": "apretujar",
    "apreturas": "apretura",
    "apretábamos": "apretar",
    "apretáis": "apretar",
    "apretáramos": "apretar",
    "apretáremos": "apretar",
    "apretásemos": "apretar",
    "apreté": "apretar",
    "apretéis": "apretar",
    "apretó": "apretar",
    "aprieta": "apretar",
    "aprietan": "apretar",
    "aprietas": "apretar",
    "apriete": "apretar",
    "aprieten": "apretar",
    "aprietes": "apretar",
    "aprieto": "apretar",
    "aprietos": "aprieto",
    "apriorismos": "apriorismo",
    "apriorística": "apriorístico",
    "apriorísticas": "apriorístico",
    "apriorísticos": "apriorístico",
    "aprisca": "apriscar",
    "apriscaba": "apriscar",
    "apriscabais": "apriscar",
    "apriscaban": "apriscar",
    "apriscabas": "apriscar",
    "apriscad": "apriscar",
    "apriscada": "apriscar",
    "apriscadas": "apriscar",
    "apriscado": "apriscar",
    "apriscados": "apriscar",
    "apriscamos": "apriscar",
    "apriscan": "apriscar",
    "apriscando": "apriscar",
    "apriscara": "apriscar",
    "apriscarais": "apriscar",
    "apriscaran": "apriscar",
    "apriscaras": "apriscar",
    "apriscare": "apriscar",
    "apriscareis": "apriscar",
    "apriscaremos": "apriscar",
    "apriscaren": "apriscar",
    "apriscares": "apriscar",
    "apriscaron": "apriscar",
    "apriscará": "apriscar",
    "apriscarán": "apriscar",
    "apriscarás": "apriscar",
    "apriscaré": "apriscar",
    "apriscaréis": "apriscar",
    "apriscaría": "apriscar",
    "apriscaríais": "apriscar",
    "apriscaríamos": "apriscar",
    "apriscarían": "apriscar",
    "apriscarías": "apriscar",
    "apriscas": "apriscar",
    "apriscase": "apriscar",
    "apriscaseis": "apriscar",
    "apriscasen": "apriscar",
    "apriscases": "apriscar",
    "apriscaste": "apriscar",
    "apriscasteis": "apriscar",
    "aprisco": "apriscar",
    "apriscos": "aprisco",
    "apriscábamos": "apriscar",
    "apriscáis": "apriscar",
    "apriscáramos": "apriscar",
    "apriscáremos": "apriscar",
    "apriscásemos": "apriscar",
    "apriscó": "apriscar",
    "aprisiona": "aprisionar",
    "aprisionaba": "aprisionar",
    "aprisionabais": "aprisionar",
    "aprisionaban": "aprisionar",
    "aprisionabas": "aprisionar",
    "aprisionad": "aprisionar",
    "aprisionada": "aprisionar",
    "aprisionadas": "aprisionar",
    "aprisionado": "aprisionar",
    "aprisionados": "aprisionar",
    "aprisionamientos": "aprisionamiento",
    "aprisionamos": "aprisionar",
    "aprisionan": "aprisionar",
    "aprisionando": "aprisionar",
    "aprisionara": "aprisionar",
    "aprisionarais": "aprisionar",
    "aprisionaran": "aprisionar",
    "aprisionaras": "aprisionar",
    "aprisionare": "aprisionar",
    "aprisionareis": "aprisionar",
    "aprisionaremos": "aprisionar",
    "aprisionaren": "aprisionar",
    "aprisionares": "aprisionar",
    "aprisionaron": "aprisionar",
    "aprisionará": "aprisionar",
    "aprisionarán": "aprisionar",
    "aprisionarás": "aprisionar",
    "aprisionaré": "aprisionar",
    "aprisionaréis": "aprisionar",
    "aprisionaría": "aprisionar",
    "aprisionaríais": "aprisionar",
    "aprisionaríamos": "aprisionar",
    "aprisionarían": "aprisionar",
    "aprisionarías": "aprisionar",
    "aprisionas": "aprisionar",
    "aprisionase": "aprisionar",
    "aprisionaseis": "aprisionar",
    "aprisionasen": "aprisionar",
    "aprisionases": "aprisionar",
    "aprisionaste": "aprisionar",
    "aprisionasteis": "aprisionar",
    "aprisione": "aprisionar",
    "aprisionemos": "aprisionar",
    "aprisionen": "aprisionar",
    "aprisiones": "aprisionar",
    "aprisiono": "aprisionar",
    "aprisionábamos": "aprisionar",
    "aprisionáis": "aprisionar",
    "aprisionáramos": "aprisionar",
    "aprisionáremos": "aprisionar",
    "aprisionásemos": "aprisionar",
    "aprisioné": "aprisionar",
    "aprisionéis": "aprisionar",
    "aprisionó": "aprisionar",
    "aprismos": "aprismo",
    "aprisque": "apriscar",
    "aprisquemos": "apriscar",
    "aprisquen": "apriscar",
    "aprisques": "apriscar",
    "aprisqué": "apriscar",
    "aprisquéis": "apriscar",
    "apristas": "aprista",
    "aproa": "aproar",
    "aproaba": "aproar",
    "aproabais": "aproar",
    "aproaban": "aproar",
    "aproabas": "aproar",
    "aproad": "aproar",
    "aproada": "aproar",
    "aproadas": "aproar",
    "aproado": "aproar",
    "aproados": "aproar",
    "aproamos": "aproar",
    "aproan": "aproar",
    "aproando": "aproar",
    "aproara": "aproar",
    "aproarais": "aproar",
    "aproaran": "aproar",
    "aproaras": "aproar",
    "aproare": "aproar",
    "aproareis": "aproar",
    "aproaremos": "aproar",
    "aproaren": "aproar",
    "aproares": "aproar",
    "aproaron": "aproar",
    "aproará": "aproar",
    "aproarán": "aproar",
    "aproarás": "aproar",
    "aproaré": "aproar",
    "aproaréis": "aproar",
    "aproaría": "aproar",
    "aproaríais": "aproar",
    "aproaríamos": "aproar",
    "aproarían": "aproar",
    "aproarías": "aproar",
    "aproas": "aproar",
    "aproase": "aproar",
    "aproaseis": "aproar",
    "aproasen": "aproar",
    "aproases": "aproar",
    "aproaste": "aproar",
    "aproasteis": "aproar",
    "aprobaba": "aprobar",
    "aprobabais": "aprobar",
    "aprobaban": "aprobar",
    "aprobabas": "aprobar",
    "aprobaciones": "aprobación",
    "aprobad": "aprobar",
    "aprobada": "aprobar",
    "aprobadas": "aprobar",
    "aprobado": "aprobar",
    "aprobadora": "aprobador",
    "aprobadoras": "aprobador",
    "aprobadores": "aprobador",
    "aprobados": "aprobar",
    "aprobamos": "aprobar",
    "aprobando": "aprobar",
    "aprobara": "aprobar",
    "aprobarais": "aprobar",
    "aprobaran": "aprobar",
    "aprobaras": "aprobar",
    "aprobare": "aprobar",
    "aprobareis": "aprobar",
    "aprobaremos": "aprobar",
    "aprobaren": "aprobar",
    "aprobares": "aprobar",
    "aprobaron": "aprobar",
    "aprobará": "aprobar",
    "aprobarán": "aprobar",
    "aprobarás": "aprobar",
    "aprobaré": "aprobar",
    "aprobaréis": "aprobar",
    "aprobaría": "aprobar",
    "aprobaríais": "aprobar",
    "aprobaríamos": "aprobar",
    "aprobarían": "aprobar",
    "aprobarías": "aprobar",
    "aprobase": "aprobar",
    "aprobaseis": "aprobar",
    "aprobasen": "aprobar",
    "aprobases": "aprobar",
    "aprobaste": "aprobar",
    "aprobasteis": "aprobar",
    "aprobativa": "aprobativo",
    "aprobativas": "aprobativo",
    "aprobativos": "aprobativo",
    "aprobatoria": "aprobatorio",
    "aprobatorias": "aprobatorio",
    "aprobatorios": "aprobatorio",
    "aprobemos": "aprobar",
    "aproblemática": "aproblemático",
    "aproblemáticas": "aproblemático",
    "aproblemáticos": "aproblemático",
    "aprobábamos": "aprobar",
    "aprobáis": "aprobar",
    "aprobáramos": "aprobar",
    "aprobáremos": "aprobar",
    "aprobásemos": "aprobar",
    "aprobé": "aprobar",
    "aprobéis": "aprobar",
    "aprobó": "aprobar",
    "aproe": "aproar",
    "aproemos": "aproar",
    "aproen": "aproar",
    "aproes": "aproar",
    "apronta": "aprontar",
    "aprontaba": "aprontar",
    "aprontabais": "aprontar",
    "aprontaban": "aprontar",
    "aprontabas": "aprontar",
    "aprontad": "aprontar",
    "aprontada": "aprontar",
    "aprontadas": "aprontar",
    "aprontado": "aprontar",
    "aprontados": "aprontar",
    "aprontamientos": "aprontamiento",
    "aprontamos": "aprontar",
    "aprontan": "aprontar",
    "aprontando": "aprontar",
    "aprontara": "aprontar",
    "aprontarais": "aprontar",
    "aprontaran": "aprontar",
    "aprontaras": "aprontar",
    "aprontare": "aprontar",
    "aprontareis": "aprontar",
    "aprontaremos": "aprontar",
    "aprontaren": "aprontar",
    "aprontares": "aprontar",
    "aprontaron": "aprontar",
    "aprontará": "aprontar",
    "aprontarán": "aprontar",
    "aprontarás": "aprontar",
    "aprontaré": "aprontar",
    "aprontaréis": "aprontar",
    "aprontaría": "aprontar",
    "aprontaríais": "aprontar",
    "aprontaríamos": "aprontar",
    "aprontarían": "aprontar",
    "aprontarías": "aprontar",
    "aprontas": "aprontar",
    "aprontase": "aprontar",
    "aprontaseis": "aprontar",
    "aprontasen": "aprontar",
    "aprontases": "aprontar",
    "aprontaste": "aprontar",
    "aprontasteis": "aprontar",
    "apronte": "aprontar",
    "aprontemos": "aprontar",
    "apronten": "aprontar",
    "aprontes": "aprontar",
    "apronto": "aprontar",
    "aprontábamos": "aprontar",
    "aprontáis": "aprontar",
    "aprontáramos": "aprontar",
    "aprontáremos": "aprontar",
    "aprontásemos": "aprontar",
    "apronté": "aprontar",
    "aprontéis": "aprontar",
    "aprontó": "aprontar",
    "aproo": "aproar",
    "apropia": "apropiar",
    "apropiaba": "apropiar",
    "apropiabais": "apropiar",
    "apropiaban": "apropiar",
    "apropiabas": "apropiar",
    "apropiaciones": "apropiación",
    "apropiad": "apropiar",
    "apropiada": "apropiar",
    "apropiadas": "apropiar",
    "apropiado": "apropiar",
    "apropiadora": "apropiador",
    "apropiadoras": "apropiador",
    "apropiadores": "apropiador",
    "apropiados": "apropiar",
    "apropiamos": "apropiar",
    "apropian": "apropiar",
    "apropiando": "apropiar",
    "apropiara": "apropiar",
    "apropiarais": "apropiar",
    "apropiaran": "apropiar",
    "apropiaras": "apropiar",
    "apropiare": "apropiar",
    "apropiareis": "apropiar",
    "apropiaremos": "apropiar",
    "apropiaren": "apropiar",
    "apropiares": "apropiar",
    "apropiaron": "apropiar",
    "apropiará": "apropiar",
    "apropiarán": "apropiar",
    "apropiarás": "apropiar",
    "apropiaré": "apropiar",
    "apropiaréis": "apropiar",
    "apropiaría": "apropiar",
    "apropiaríais": "apropiar",
    "apropiaríamos": "apropiar",
    "apropiarían": "apropiar",
    "apropiarías": "apropiar",
    "apropias": "apropiar",
    "apropiase": "apropiar",
    "apropiaseis": "apropiar",
    "apropiasen": "apropiar",
    "apropiases": "apropiar",
    "apropiaste": "apropiar",
    "apropiasteis": "apropiar",
    "apropie": "apropiar",
    "apropiemos": "apropiar",
    "apropien": "apropiar",
    "apropies": "apropiar",
    "apropincuaba": "apropincuar",
    "apropincuabais": "apropincuar",
    "apropincuaban": "apropincuar",
    "apropincuabas": "apropincuar",
    "apropincuad": "apropincuar",
    "apropincuada": "apropincuar",
    "apropincuadas": "apropincuar",
    "apropincuado": "apropincuar",
    "apropincuados": "apropincuar",
    "apropincuamos": "apropincuar",
    "apropincuando": "apropincuar",
    "apropincuara": "apropincuar",
    "apropincuarais": "apropincuar",
    "apropincuaran": "apropincuar",
    "apropincuaras": "apropincuar",
    "apropincuare": "apropincuar",
    "apropincuareis": "apropincuar",
    "apropincuaremos": "apropincuar",
    "apropincuaren": "apropincuar",
    "apropincuares": "apropincuar",
    "apropincuaron": "apropincuar",
    "apropincuará": "apropincuar",
    "apropincuarán": "apropincuar",
    "apropincuarás": "apropincuar",
    "apropincuaré": "apropincuar",
    "apropincuaréis": "apropincuar",
    "apropincuaría": "apropincuar",
    "apropincuaríais": "apropincuar",
    "apropincuaríamos": "apropincuar",
    "apropincuarían": "apropincuar",
    "apropincuarías": "apropincuar",
    "apropincuase": "apropincuar",
    "apropincuaseis": "apropincuar",
    "apropincuasen": "apropincuar",
    "apropincuases": "apropincuar",
    "apropincuaste": "apropincuar",
    "apropincuasteis": "apropincuar",
    "apropincuemos": "apropincuar",
    "apropincuábamos": "apropincuar",
    "apropincuáis": "apropincuar",
    "apropincuáramos": "apropincuar",
    "apropincuáremos": "apropincuar",
    "apropincuásemos": "apropincuar",
    "apropincué": "apropincuar",
    "apropincuéis": "apropincuar",
    "apropincuó": "apropincuar",
    "apropincúa": "apropincuar",
    "apropincúan": "apropincuar",
    "apropincúas": "apropincuar",
    "apropincúe": "apropincuar",
    "apropincúen": "apropincuar",
    "apropincúes": "apropincuar",
    "apropincúo": "apropincuar",
    "apropio": "apropiar",
    "apropiábamos": "apropiar",
    "apropiáis": "apropiar",
    "apropiáramos": "apropiar",
    "apropiáremos": "apropiar",
    "apropiásemos": "apropiar",
    "apropié": "apropiar",
    "apropiéis": "apropiar",
    "apropió": "apropiar",
    "apropósitos": "apropósito",
    "aprovecha": "aprovechar",
    "aprovechaba": "aprovechar",
    "aprovechabais": "aprovechar",
    "aprovechaban": "aprovechar",
    "aprovechabas": "aprovechar",
    "aprovechables": "aprovechable",
    "aprovechad": "aprovechar",
    "aprovechada": "aprovechar",
    "aprovechadas": "aprovechar",
    "aprovechado": "aprovechar",
    "aprovechadora": "aprovechador",
    "aprovechadoras": "aprovechador",
    "aprovechadores": "aprovechador",
    "aprovechados": "aprovechar",
    "aprovechamientos": "aprovechamiento",
    "aprovechamos": "aprovechar",
    "aprovechan": "aprovechar",
    "aprovechando": "aprovechar",
    "aprovechara": "aprovechar",
    "aprovecharais": "aprovechar",
    "aprovecharan": "aprovechar",
    "aprovecharas": "aprovechar",
    "aprovechare": "aprovechar",
    "aprovechareis": "aprovechar",
    "aprovecharemos": "aprovechar",
    "aprovecharen": "aprovechar",
    "aprovechares": "aprovechar",
    "aprovecharon": "aprovechar",
    "aprovechará": "aprovechar",
    "aprovecharán": "aprovechar",
    "aprovecharás": "aprovechar",
    "aprovecharé": "aprovechar",
    "aprovecharéis": "aprovechar",
    "aprovecharía": "aprovechar",
    "aprovecharíais": "aprovechar",
    "aprovecharíamos": "aprovechar",
    "aprovecharían": "aprovechar",
    "aprovecharías": "aprovechar",
    "aprovechas": "aprovechar",
    "aprovechase": "aprovechar",
    "aprovechaseis": "aprovechar",
    "aprovechasen": "aprovechar",
    "aprovechases": "aprovechar",
    "aprovechaste": "aprovechar",
    "aprovechasteis": "aprovechar",
    "aproveche": "aprovechar",
    "aprovechemos": "aprovechar",
    "aprovechen": "aprovechar",
    "aproveches": "aprovechar",
    "aprovecho": "aprovechar",
    "aprovechábamos": "aprovechar",
    "aprovecháis": "aprovechar",
    "aprovecháramos": "aprovechar",
    "aprovecháremos": "aprovechar",
    "aprovechásemos": "aprovechar",
    "aproveché": "aprovechar",
    "aprovechéis": "aprovechar",
    "aprovechó": "aprovechar",
    "aprovisiona": "aprovisionar",
    "aprovisionaba": "aprovisionar",
    "aprovisionabais": "aprovisionar",
    "aprovisionaban": "aprovisionar",
    "aprovisionabas": "aprovisionar",
    "aprovisionad": "aprovisionar",
    "aprovisionada": "aprovisionar",
    "aprovisionadas": "aprovisionar",
    "aprovisionado": "aprovisionar",
    "aprovisionados": "aprovisionar",
    "aprovisionamientos": "aprovisionamiento",
    "aprovisionamos": "aprovisionar",
    "aprovisionan": "aprovisionar",
    "aprovisionando": "aprovisionar",
    "aprovisionara": "aprovisionar",
    "aprovisionarais": "aprovisionar",
    "aprovisionaran": "aprovisionar",
    "aprovisionaras": "aprovisionar",
    "aprovisionare": "aprovisionar",
    "aprovisionareis": "aprovisionar",
    "aprovisionaremos": "aprovisionar",
    "aprovisionaren": "aprovisionar",
    "aprovisionares": "aprovisionar",
    "aprovisionaron": "aprovisionar",
    "aprovisionará": "aprovisionar",
    "aprovisionarán": "aprovisionar",
    "aprovisionarás": "aprovisionar",
    "aprovisionaré": "aprovisionar",
    "aprovisionaréis": "aprovisionar",
    "aprovisionaría": "aprovisionar",
    "aprovisionaríais": "aprovisionar",
    "aprovisionaríamos": "aprovisionar",
    "aprovisionarían": "aprovisionar",
    "aprovisionarías": "aprovisionar",
    "aprovisionas": "aprovisionar",
    "aprovisionase": "aprovisionar",
    "aprovisionaseis": "aprovisionar",
    "aprovisionasen": "aprovisionar",
    "aprovisionases": "aprovisionar",
    "aprovisionaste": "aprovisionar",
    "aprovisionasteis": "aprovisionar",
    "aprovisione": "aprovisionar",
    "aprovisionemos": "aprovisionar",
    "aprovisionen": "aprovisionar",
    "aprovisiones": "aprovisionar",
    "aprovisiono": "aprovisionar",
    "aprovisionábamos": "aprovisionar",
    "aprovisionáis": "aprovisionar",
    "aprovisionáramos": "aprovisionar",
    "aprovisionáremos": "aprovisionar",
    "aprovisionásemos": "aprovisionar",
    "aprovisioné": "aprovisionar",
    "aprovisionéis": "aprovisionar",
    "aprovisionó": "aprovisionar",
    "aproxima": "aproximar",
    "aproximaba": "aproximar",
    "aproximabais": "aproximar",
    "aproximaban": "aproximar",
    "aproximabas": "aproximar",
    "aproximaciones": "aproximación",
    "aproximad": "aproximar",
    "aproximada": "aproximar",
    "aproximadas": "aproximar",
    "aproximado": "aproximar",
    "aproximados": "aproximar",
    "aproximamos": "aproximar",
    "aproximan": "aproximar",
    "aproximando": "aproximar",
    "aproximantes": "aproximante",
    "aproximara": "aproximar",
    "aproximarais": "aproximar",
    "aproximaran": "aproximar",
    "aproximaras": "aproximar",
    "aproximare": "aproximar",
    "aproximareis": "aproximar",
    "aproximaremos": "aproximar",
    "aproximaren": "aproximar",
    "aproximares": "aproximar",
    "aproximaron": "aproximar",
    "aproximará": "aproximar",
    "aproximarán": "aproximar",
    "aproximarás": "aproximar",
    "aproximaré": "aproximar",
    "aproximaréis": "aproximar",
    "aproximaría": "aproximar",
    "aproximaríais": "aproximar",
    "aproximaríamos": "aproximar",
    "aproximarían": "aproximar",
    "aproximarías": "aproximar",
    "aproximas": "aproximar",
    "aproximase": "aproximar",
    "aproximaseis": "aproximar",
    "aproximasen": "aproximar",
    "aproximases": "aproximar",
    "aproximaste": "aproximar",
    "aproximasteis": "aproximar",
    "aproximativa": "aproximativo",
    "aproximativas": "aproximativo",
    "aproximativos": "aproximativo",
    "aproxime": "aproximar",
    "aproximemos": "aproximar",
    "aproximen": "aproximar",
    "aproximes": "aproximar",
    "aproximo": "aproximar",
    "aproximábamos": "aproximar",
    "aproximáis": "aproximar",
    "aproximáramos": "aproximar",
    "aproximáremos": "aproximar",
    "aproximásemos": "aproximar",
    "aproximé": "aproximar",
    "aproximéis": "aproximar",
    "aproximó": "aproximar",
    "aproábamos": "aproar",
    "aproáis": "aproar",
    "aproáramos": "aproar",
    "aproáremos": "aproar",
    "aproásemos": "aproar",
    "aproé": "aproar",
    "aproéis": "aproar",
    "aproó": "aproar",
    "aprueba": "aprobar",
    "aprueban": "aprobar",
    "apruebas": "aprobar",
    "apruebe": "aprobar",
    "aprueben": "aprobar",
    "apruebes": "aprobar",
    "apruebo": "aprobar",
    "aprótica": "aprótico",
    "apróticas": "aprótico",
    "apróticos": "aprótico",
    "apta": "apto",
    "aptaba": "aptar",
    "aptabais": "aptar",
    "aptaban": "aptar",
    "aptabas": "aptar",
    "aptad": "aptar",
    "aptada": "aptar",
    "aptadas": "aptar",
    "aptado": "aptar",
    "aptados": "aptar",
    "aptamos": "aptar",
    "aptan": "aptar",
    "aptando": "aptar",
    "aptara": "aptar",
    "aptarais": "aptar",
    "aptaran": "aptar",
    "aptaras": "aptar",
    "aptare": "aptar",
    "aptareis": "aptar",
    "aptaremos": "aptar",
    "aptaren": "aptar",
    "aptares": "aptar",
    "aptaron": "aptar",
    "aptará": "aptar",
    "aptarán": "aptar",
    "aptarás": "aptar",
    "aptaré": "aptar",
    "aptaréis": "aptar",
    "aptaría": "aptar",
    "aptaríais": "aptar",
    "aptaríamos": "aptar",
    "aptarían": "aptar",
    "aptarías": "aptar",
    "aptas": "apto",
    "aptase": "aptar",
    "aptaseis": "aptar",
    "aptasen": "aptar",
    "aptases": "aptar",
    "aptaste": "aptar",
    "aptasteis": "aptar",
    "apte": "aptar",
    "aptemos": "aptar",
    "apten": "aptar",
    "aptes": "aptar",
    "aptialismos": "aptialismo",
    "aptitudes": "aptitud",
    "apto": "aptar",
    "aptos": "apto",
    "aptábamos": "aptar",
    "aptáis": "aptar",
    "aptáramos": "aptar",
    "aptáremos": "aptar",
    "aptásemos": "aptar",
    "apté": "aptar",
    "aptéis": "aptar",
    "aptó": "aptar",
    "apuesta": "apuesto",
    "apuestan": "apostar",
    "apuestas": "apuesto",
    "apueste": "apostar",
    "apuesten": "apostar",
    "apuestes": "apostar",
    "apuesto": "apostar",
    "apuestos": "apuesto",
    "apulgara": "apulgarar",
    "apulgaraba": "apulgarar",
    "apulgarabais": "apulgarar",
    "apulgaraban": "apulgarar",
    "apulgarabas": "apulgarar",
    "apulgarad": "apulgarar",
    "apulgarada": "apulgarar",
    "apulgaradas": "apulgarar",
    "apulgarado": "apulgarar",
    "apulgarados": "apulgarar",
    "apulgaramos": "apulgarar",
    "apulgaran": "apulgarar",
    "apulgarando": "apulgarar",
    "apulgarara": "apulgarar",
    "apulgararais": "apulgarar",
    "apulgararan": "apulgarar",
    "apulgararas": "apulgarar",
    "apulgarare": "apulgarar",
    "apulgarareis": "apulgarar",
    "apulgararemos": "apulgarar",
    "apulgararen": "apulgarar",
    "apulgarares": "apulgarar",
    "apulgararon": "apulgarar",
    "apulgarará": "apulgarar",
    "apulgararán": "apulgarar",
    "apulgararás": "apulgarar",
    "apulgararé": "apulgarar",
    "apulgararéis": "apulgarar",
    "apulgararía": "apulgarar",
    "apulgararíais": "apulgarar",
    "apulgararíamos": "apulgarar",
    "apulgararían": "apulgarar",
    "apulgararías": "apulgarar",
    "apulgaras": "apulgarar",
    "apulgarase": "apulgarar",
    "apulgaraseis": "apulgarar",
    "apulgarasen": "apulgarar",
    "apulgarases": "apulgarar",
    "apulgaraste": "apulgarar",
    "apulgarasteis": "apulgarar",
    "apulgare": "apulgarar",
    "apulgaremos": "apulgarar",
    "apulgaren": "apulgarar",
    "apulgares": "apulgarar",
    "apulgaro": "apulgarar",
    "apulgarábamos": "apulgarar",
    "apulgaráis": "apulgarar",
    "apulgaráramos": "apulgarar",
    "apulgaráremos": "apulgarar",
    "apulgarásemos": "apulgarar",
    "apulgaré": "apulgarar",
    "apulgaréis": "apulgarar",
    "apulgaró": "apulgarar",
    "apuna": "apunar",
    "apunaba": "apunar",
    "apunabais": "apunar",
    "apunaban": "apunar",
    "apunabas": "apunar",
    "apunad": "apunar",
    "apunada": "apunar",
    "apunadas": "apunar",
    "apunado": "apunar",
    "apunados": "apunar",
    "apunamos": "apunar",
    "apunan": "apunar",
    "apunando": "apunar",
    "apunara": "apunar",
    "apunarais": "apunar",
    "apunaran": "apunar",
    "apunaras": "apunar",
    "apunare": "apunar",
    "apunareis": "apunar",
    "apunaremos": "apunar",
    "apunaren": "apunar",
    "apunares": "apunar",
    "apunaron": "apunar",
    "apunará": "apunar",
    "apunarán": "apunar",
    "apunarás": "apunar",
    "apunaré": "apunar",
    "apunaréis": "apunar",
    "apunaría": "apunar",
    "apunaríais": "apunar",
    "apunaríamos": "apunar",
    "apunarían": "apunar",
    "apunarías": "apunar",
    "apunas": "apunar",
    "apunase": "apunar",
    "apunaseis": "apunar",
    "apunasen": "apunar",
    "apunases": "apunar",
    "apunaste": "apunar",
    "apunasteis": "apunar",
    "apune": "apunar",
    "apunemos": "apunar",
    "apunen": "apunar",
    "apunes": "apunar",
    "apuno": "apunar",
    "apunta": "apuntar",
    "apuntaba": "apuntar",
    "apuntabais": "apuntar",
    "apuntaban": "apuntar",
    "apuntabas": "apuntar",
    "apuntaciones": "apuntación",
    "apuntad": "apuntar",
    "apuntada": "apuntar",
    "apuntadas": "apuntar",
    "apuntado": "apuntar",
    "apuntadora": "apuntador",
    "apuntadoras": "apuntador",
    "apuntadores": "apuntador",
    "apuntados": "apuntar",
    "apuntala": "apuntalar",
    "apuntalaba": "apuntalar",
    "apuntalabais": "apuntalar",
    "apuntalaban": "apuntalar",
    "apuntalabas": "apuntalar",
    "apuntalad": "apuntalar",
    "apuntalada": "apuntalar",
    "apuntaladas": "apuntalar",
    "apuntalado": "apuntalar",
    "apuntalados": "apuntalar",
    "apuntalamientos": "apuntalamiento",
    "apuntalamos": "apuntalar",
    "apuntalan": "apuntalar",
    "apuntalando": "apuntalar",
    "apuntalara": "apuntalar",
    "apuntalarais": "apuntalar",
    "apuntalaran": "apuntalar",
    "apuntalaras": "apuntalar",
    "apuntalare": "apuntalar",
    "apuntalareis": "apuntalar",
    "apuntalaremos": "apuntalar",
    "apuntalaren": "apuntalar",
    "apuntalares": "apuntalar",
    "apuntalaron": "apuntalar",
    "apuntalará": "apuntalar",
    "apuntalarán": "apuntalar",
    "apuntalarás": "apuntalar",
    "apuntalaré": "apuntalar",
    "apuntalaréis": "apuntalar",
    "apuntalaría": "apuntalar",
    "apuntalaríais": "apuntalar",
    "apuntalaríamos": "apuntalar",
    "apuntalarían": "apuntalar",
    "apuntalarías": "apuntalar",
    "apuntalas": "apuntalar",
    "apuntalase": "apuntalar",
    "apuntalaseis": "apuntalar",
    "apuntalasen": "apuntalar",
    "apuntalases": "apuntalar",
    "apuntalaste": "apuntalar",
    "apuntalasteis": "apuntalar",
    "apuntale": "apuntalar",
    "apuntalemos": "apuntalar",
    "apuntalen": "apuntalar",
    "apuntales": "apuntalar",
    "apuntalo": "apuntalar",
    "apuntalábamos": "apuntalar",
    "apuntaláis": "apuntalar",
    "apuntaláramos": "apuntalar",
    "apuntaláremos": "apuntalar",
    "apuntalásemos": "apuntalar",
    "apuntalé": "apuntalar",
    "apuntaléis": "apuntalar",
    "apuntaló": "apuntalar",
    "apuntamientos": "apuntamiento",
    "apuntamos": "apuntar",
    "apuntan": "apuntar",
    "apuntando": "apuntar",
    "apuntara": "apuntar",
    "apuntarais": "apuntar",
    "apuntaran": "apuntar",
    "apuntaras": "apuntar",
    "apuntare": "apuntar",
    "apuntareis": "apuntar",
    "apuntaremos": "apuntar",
    "apuntaren": "apuntar",
    "apuntares": "apuntar",
    "apuntaron": "apuntar",
    "apuntará": "apuntar",
    "apuntarán": "apuntar",
    "apuntarás": "apuntar",
    "apuntaré": "apuntar",
    "apuntaréis": "apuntar",
    "apuntaría": "apuntar",
    "apuntaríais": "apuntar",
    "apuntaríamos": "apuntar",
    "apuntarían": "apuntar",
    "apuntarías": "apuntar",
    "apuntas": "apuntar",
    "apuntase": "apuntar",
    "apuntaseis": "apuntar",
    "apuntasen": "apuntar",
    "apuntases": "apuntar",
    "apuntaste": "apuntar",
    "apuntasteis": "apuntar",
    "apunte": "apuntar",
    "apuntemos": "apuntar",
    "apunten": "apuntar",
    "apuntes": "apunte",
    "apuntilla": "apuntillar",
    "apuntillaba": "apuntillar",
    "apuntillabais": "apuntillar",
    "apuntillaban": "apuntillar",
    "apuntillabas": "apuntillar",
    "apuntillad": "apuntillar",
    "apuntillada": "apuntillar",
    "apuntilladas": "apuntillar",
    "apuntillado": "apuntillar",
    "apuntillados": "apuntillar",
    "apuntillamos": "apuntillar",
    "apuntillan": "apuntillar",
    "apuntillando": "apuntillar",
    "apuntillara": "apuntillar",
    "apuntillarais": "apuntillar",
    "apuntillaran": "apuntillar",
    "apuntillaras": "apuntillar",
    "apuntillare": "apuntillar",
    "apuntillareis": "apuntillar",
    "apuntillaremos": "apuntillar",
    "apuntillaren": "apuntillar",
    "apuntillares": "apuntillar",
    "apuntillaron": "apuntillar",
    "apuntillará": "apuntillar",
    "apuntillarán": "apuntillar",
    "apuntillarás": "apuntillar",
    "apuntillaré": "apuntillar",
    "apuntillaréis": "apuntillar",
    "apuntillaría": "apuntillar",
    "apuntillaríais": "apuntillar",
    "apuntillaríamos": "apuntillar",
    "apuntillarían": "apuntillar",
    "apuntillarías": "apuntillar",
    "apuntillas": "apuntillar",
    "apuntillase": "apuntillar",
    "apuntillaseis": "apuntillar",
    "apuntillasen": "apuntillar",
    "apuntillases": "apuntillar",
    "apuntillaste": "apuntillar",
    "apuntillasteis": "apuntillar",
    "apuntille": "apuntillar",
    "apuntillemos": "apuntillar",
    "apuntillen": "apuntillar",
    "apuntilles": "apuntillar",
    "apuntillo": "apuntillar",
    "apuntillábamos": "apuntillar",
    "apuntilláis": "apuntillar",
    "apuntilláramos": "apuntillar",
    "apuntilláremos": "apuntillar",
    "apuntillásemos": "apuntillar",
    "apuntillé": "apuntillar",
    "apuntilléis": "apuntillar",
    "apuntilló": "apuntillar",
    "apunto": "apuntar",
    "apuntábamos": "apuntar",
    "apuntáis": "apuntar",
    "apuntáramos": "apuntar",
    "apuntáremos": "apuntar",
    "apuntásemos": "apuntar",
    "apunté": "apuntar",
    "apuntéis": "apuntar",
    "apuntó": "apuntar",
    "apunábamos": "apunar",
    "apunáis": "apunar",
    "apunáramos": "apunar",
    "apunáremos": "apunar",
    "apunásemos": "apunar",
    "apuné": "apunar",
    "apunéis": "apunar",
    "apunó": "apunar",
    "apura": "apurar",
    "apuraba": "apurar",
    "apurabais": "apurar",
    "apuraban": "apurar",
    "apurabas": "apurar",
    "apuraciones": "apuración",
    "apurad": "apurar",
    "apurada": "apurar",
    "apuradas": "apurar",
    "apurado": "apurar",
    "apuradora": "apurador",
    "apuradoras": "apurador",
    "apuradores": "apurador",
    "apurados": "apurar",
    "apuramientos": "apuramiento",
    "apuramos": "apurar",
    "apuran": "apurar",
    "apurando": "apurar",
    "apurara": "apurar",
    "apurarais": "apurar",
    "apuraran": "apurar",
    "apuraras": "apurar",
    "apurare": "apurar",
    "apurareis": "apurar",
    "apuraremos": "apurar",
    "apuraren": "apurar",
    "apurares": "apurar",
    "apuraron": "apurar",
    "apurará": "apurar",
    "apurarán": "apurar",
    "apurarás": "apurar",
    "apuraré": "apurar",
    "apuraréis": "apurar",
    "apuraría": "apurar",
    "apuraríais": "apurar",
    "apuraríamos": "apurar",
    "apurarían": "apurar",
    "apurarías": "apurar",
    "apuras": "apurar",
    "apurase": "apurar",
    "apuraseis": "apurar",
    "apurasen": "apurar",
    "apurases": "apurar",
    "apuraste": "apurar",
    "apurasteis": "apurar",
    "apure": "apurar",
    "apuremos": "apurar",
    "apuren": "apurar",
    "apures": "apure",
    "apuro": "apurar",
    "apuros": "apuro",
    "apurábamos": "apurar",
    "apuráis": "apurar",
    "apuráramos": "apurar",
    "apuráremos": "apurar",
    "apurásemos": "apurar",
    "apuré": "apurar",
    "apuréis": "apurar",
    "apuró": "apurar",
    "apuña": "apuñar",
    "apuñaba": "apuñar",
    "apuñabais": "apuñar",
    "apuñaban": "apuñar",
    "apuñabas": "apuñar",
    "apuñad": "apuñar",
    "apuñada": "apuñar",
    "apuñadas": "apuñar",
    "apuñado": "apuñar",
    "apuñados": "apuñar",
    "apuñala": "apuñalar",
    "apuñalaba": "apuñalar",
    "apuñalabais": "apuñalar",
    "apuñalaban": "apuñalar",
    "apuñalabas": "apuñalar",
    "apuñalad": "apuñalar",
    "apuñalada": "apuñalar",
    "apuñaladas": "apuñalar",
    "apuñalado": "apuñalar",
    "apuñalados": "apuñalar",
    "apuñalamientos": "apuñalamiento",
    "apuñalamos": "apuñalar",
    "apuñalan": "apuñalar",
    "apuñalando": "apuñalar",
    "apuñalara": "apuñalar",
    "apuñalarais": "apuñalar",
    "apuñalaran": "apuñalar",
    "apuñalaras": "apuñalar",
    "apuñalare": "apuñalar",
    "apuñalareis": "apuñalar",
    "apuñalaremos": "apuñalar",
    "apuñalaren": "apuñalar",
    "apuñalares": "apuñalar",
    "apuñalaron": "apuñalar",
    "apuñalará": "apuñalar",
    "apuñalarán": "apuñalar",
    "apuñalarás": "apuñalar",
    "apuñalaré": "apuñalar",
    "apuñalaréis": "apuñalar",
    "apuñalaría": "apuñalar",
    "apuñalaríais": "apuñalar",
    "apuñalaríamos": "apuñalar",
    "apuñalarían": "apuñalar",
    "apuñalarías": "apuñalar",
    "apuñalas": "apuñalar",
    "apuñalase": "apuñalar",
    "apuñalaseis": "apuñalar",
    "apuñalasen": "apuñalar",
    "apuñalases": "apuñalar",
    "apuñalaste": "apuñalar",
    "apuñalasteis": "apuñalar",
    "apuñale": "apuñalar",
    "apuñalemos": "apuñalar",
    "apuñalen": "apuñalar",
    "apuñales": "apuñalar",
    "apuñalo": "apuñalar",
    "apuñalábamos": "apuñalar",
    "apuñaláis": "apuñalar",
    "apuñaláramos": "apuñalar",
    "apuñaláremos": "apuñalar",
    "apuñalásemos": "apuñalar",
    "apuñalé": "apuñalar",
    "apuñaléis": "apuñalar",
    "apuñaló": "apuñalar",
    "apuñamos": "apuñar",
    "apuñan": "apuñar",
    "apuñando": "apuñar",
    "apuñara": "apuñar",
    "apuñarais": "apuñar",
    "apuñaran": "apuñar",
    "apuñaras": "apuñar",
    "apuñare": "apuñar",
    "apuñareis": "apuñar",
    "apuñaremos": "apuñar",
    "apuñaren": "apuñar",
    "apuñares": "apuñar",
    "apuñaron": "apuñar",
    "apuñará": "apuñar",
    "apuñarán": "apuñar",
    "apuñarás": "apuñar",
    "apuñaré": "apuñar",
    "apuñaréis": "apuñar",
    "apuñaría": "apuñar",
    "apuñaríais": "apuñar",
    "apuñaríamos": "apuñar",
    "apuñarían": "apuñar",
    "apuñarías": "apuñar",
    "apuñas": "apuñar",
    "apuñase": "apuñar",
    "apuñaseis": "apuñar",
    "apuñasen": "apuñar",
    "apuñases": "apuñar",
    "apuñaste": "apuñar",
    "apuñasteis": "apuñar",
    "apuñe": "apuñar",
    "apuñea": "apuñear",
    "apuñeaba": "apuñear",
    "apuñeabais": "apuñear",
    "apuñeaban": "apuñear",
    "apuñeabas": "apuñear",
    "apuñead": "apuñear",
    "apuñeada": "apuñear",
    "apuñeadas": "apuñear",
    "apuñeado": "apuñear",
    "apuñeados": "apuñear",
    "apuñeamos": "apuñear",
    "apuñean": "apuñear",
    "apuñeando": "apuñear",
    "apuñeara": "apuñear",
    "apuñearais": "apuñear",
    "apuñearan": "apuñear",
    "apuñearas": "apuñear",
    "apuñeare": "apuñear",
    "apuñeareis": "apuñear",
    "apuñearemos": "apuñear",
    "apuñearen": "apuñear",
    "apuñeares": "apuñear",
    "apuñearon": "apuñear",
    "apuñeará": "apuñear",
    "apuñearán": "apuñear",
    "apuñearás": "apuñear",
    "apuñearé": "apuñear",
    "apuñearéis": "apuñear",
    "apuñearía": "apuñear",
    "apuñearíais": "apuñear",
    "apuñearíamos": "apuñear",
    "apuñearían": "apuñear",
    "apuñearías": "apuñear",
    "apuñeas": "apuñear",
    "apuñease": "apuñear",
    "apuñeaseis": "apuñear",
    "apuñeasen": "apuñear",
    "apuñeases": "apuñear",
    "apuñeaste": "apuñear",
    "apuñeasteis": "apuñear",
    "apuñee": "apuñear",
    "apuñeemos": "apuñear",
    "apuñeen": "apuñear",
    "apuñees": "apuñear",
    "apuñemos": "apuñar",
    "apuñen": "apuñar",
    "apuñeo": "apuñear",
    "apuñes": "apuñar",
    "apuñetea": "apuñetear",
    "apuñeteaba": "apuñetear",
    "apuñeteabais": "apuñetear",
    "apuñeteaban": "apuñetear",
    "apuñeteabas": "apuñetear",
    "apuñetead": "apuñetear",
    "apuñeteada": "apuñetear",
    "apuñeteadas": "apuñetear",
    "apuñeteado": "apuñetear",
    "apuñeteados": "apuñetear",
    "apuñeteamos": "apuñetear",
    "apuñetean": "apuñetear",
    "apuñeteando": "apuñetear",
    "apuñeteara": "apuñetear",
    "apuñetearais": "apuñetear",
    "apuñetearan": "apuñetear",
    "apuñetearas": "apuñetear",
    "apuñeteare": "apuñetear",
    "apuñeteareis": "apuñetear",
    "apuñetearemos": "apuñetear",
    "apuñetearen": "apuñetear",
    "apuñeteares": "apuñetear",
    "apuñetearon": "apuñetear",
    "apuñeteará": "apuñetear",
    "apuñetearán": "apuñetear",
    "apuñetearás": "apuñetear",
    "apuñetearé": "apuñetear",
    "apuñetearéis": "apuñetear",
    "apuñetearía": "apuñetear",
    "apuñetearíais": "apuñetear",
    "apuñetearíamos": "apuñetear",
    "apuñetearían": "apuñetear",
    "apuñetearías": "apuñetear",
    "apuñeteas": "apuñetear",
    "apuñetease": "apuñetear",
    "apuñeteaseis": "apuñetear",
    "apuñeteasen": "apuñetear",
    "apuñeteases": "apuñetear",
    "apuñeteaste": "apuñetear",
    "apuñeteasteis": "apuñetear",
    "apuñetee": "apuñetear",
    "apuñeteemos": "apuñetear",
    "apuñeteen": "apuñetear",
    "apuñetees": "apuñetear",
    "apuñeteo": "apuñetear",
    "apuñeteábamos": "apuñetear",
    "apuñeteáis": "apuñetear",
    "apuñeteáramos": "apuñetear",
    "apuñeteáremos": "apuñetear",
    "apuñeteásemos": "apuñetear",
    "apuñeteé": "apuñetear",
    "apuñeteéis": "apuñetear",
    "apuñeteó": "apuñetear",
    "apuñeábamos": "apuñear",
    "apuñeáis": "apuñear",
    "apuñeáramos": "apuñear",
    "apuñeáremos": "apuñear",
    "apuñeásemos": "apuñear",
    "apuñeé": "apuñear",
    "apuñeéis": "apuñear",
    "apuñeó": "apuñear",
    "apuño": "apuñar",
    "apuñábamos": "apuñar",
    "apuñáis": "apuñar",
    "apuñáramos": "apuñar",
    "apuñáremos": "apuñar",
    "apuñásemos": "apuñar",
    "apuñé": "apuñar",
    "apuñéis": "apuñar",
    "apuñó": "apuñar",
    "apática": "apático",
    "apáticas": "apático",
    "apáticos": "apático",
    "apátridas": "apátrida",
    "apéndices": "apéndice",
    "apétala": "apétalo",
    "apétalas": "apétalo",
    "apétalos": "apétalo",
    "apócopes": "apócope",
    "apócrifa": "apócrifo",
    "apócrifas": "apócrifo",
    "apócrifos": "apócrifo",
    "apófiges": "apófige",
    "apógrafos": "apógrafo",
    "apólogos": "apólogo",
    "apósitos": "apósito",
    "apóstatas": "apóstata",
    "apóstoles": "apóstol",
    "apóstrofes": "apóstrofe",
    "apóstrofos": "apóstrofo",
    "aquea": "aqueo",
    "aqueas": "aqueo",
    "aqueilias": "aqueilia",
    "aqueja": "aquejar",
    "aquejaba": "aquejar",
    "aquejabais": "aquejar",
    "aquejaban": "aquejar",
    "aquejabas": "aquejar",
    "aquejad": "aquejar",
    "aquejada": "aquejar",
    "aquejadas": "aquejar",
    "aquejado": "aquejar",
    "aquejados": "aquejar",
    "aquejamos": "aquejar",
    "aquejan": "aquejar",
    "aquejando": "aquejar",
    "aquejara": "aquejar",
    "aquejarais": "aquejar",
    "aquejaran": "aquejar",
    "aquejaras": "aquejar",
    "aquejare": "aquejar",
    "aquejareis": "aquejar",
    "aquejaremos": "aquejar",
    "aquejaren": "aquejar",
    "aquejares": "aquejar",
    "aquejaron": "aquejar",
    "aquejará": "aquejar",
    "aquejarán": "aquejar",
    "aquejarás": "aquejar",
    "aquejaré": "aquejar",
    "aquejaréis": "aquejar",
    "aquejaría": "aquejar",
    "aquejaríais": "aquejar",
    "aquejaríamos": "aquejar",
    "aquejarían": "aquejar",
    "aquejarías": "aquejar",
    "aquejas": "aquejar",
    "aquejase": "aquejar",
    "aquejaseis": "aquejar",
    "aquejasen": "aquejar",
    "aquejases": "aquejar",
    "aquejaste": "aquejar",
    "aquejasteis": "aquejar",
    "aqueje": "aquejar",
    "aquejemos": "aquejar",
    "aquejen": "aquejar",
    "aquejes": "aquejar",
    "aquejo": "aquejar",
    "aquejábamos": "aquejar",
    "aquejáis": "aquejar",
    "aquejáramos": "aquejar",
    "aquejáremos": "aquejar",
    "aquejásemos": "aquejar",
    "aquejé": "aquejar",
    "aquejéis": "aquejar",
    "aquejó": "aquejar",
    "aquelarres": "aquelarre",
    "aquella": "aquel",
    "aquellas": "aquel",
    "aquello": "aquel",
    "aquellos": "aquel",
    "aqueménidas": "aqueménida",
    "aquenios": "aquenio",
    "aqueos": "aqueo",
    "aquerencia": "aquerenciar",
    "aquerenciaba": "aquerenciar",
    "aquerenciabais": "aquerenciar",
    "aquerenciaban": "aquerenciar",
    "aquerenciabas": "aquerenciar",
    "aquerenciad": "aquerenciar",
    "aquerenciada": "aquerenciar",
    "aquerenciadas": "aquerenciar",
    "aquerenciado": "aquerenciar",
    "aquerenciados": "aquerenciar",
    "aquerenciamos": "aquerenciar",
    "aquerencian": "aquerenciar",
    "aquerenciando": "aquerenciar",
    "aquerenciara": "aquerenciar",
    "aquerenciarais": "aquerenciar",
    "aquerenciaran": "aquerenciar",
    "aquerenciaras": "aquerenciar",
    "aquerenciare": "aquerenciar",
    "aquerenciareis": "aquerenciar",
    "aquerenciaremos": "aquerenciar",
    "aquerenciaren": "aquerenciar",
    "aquerenciares": "aquerenciar",
    "aquerenciaron": "aquerenciar",
    "aquerenciará": "aquerenciar",
    "aquerenciarán": "aquerenciar",
    "aquerenciarás": "aquerenciar",
    "aquerenciaré": "aquerenciar",
    "aquerenciaréis": "aquerenciar",
    "aquerenciaría": "aquerenciar",
    "aquerenciaríais": "aquerenciar",
    "aquerenciaríamos": "aquerenciar",
    "aquerenciarían": "aquerenciar",
    "aquerenciarías": "aquerenciar",
    "aquerencias": "aquerenciar",
    "aquerenciase": "aquerenciar",
    "aquerenciaseis": "aquerenciar",
    "aquerenciasen": "aquerenciar",
    "aquerenciases": "aquerenciar",
    "aquerenciaste": "aquerenciar",
    "aquerenciasteis": "aquerenciar",
    "aquerencie": "aquerenciar",
    "aquerenciemos": "aquerenciar",
    "aquerencien": "aquerenciar",
    "aquerencies": "aquerenciar",
    "aquerencio": "aquerenciar",
    "aquerenciábamos": "aquerenciar",
    "aquerenciáis": "aquerenciar",
    "aquerenciáramos": "aquerenciar",
    "aquerenciáremos": "aquerenciar",
    "aquerenciásemos": "aquerenciar",
    "aquerencié": "aquerenciar",
    "aquerenciéis": "aquerenciar",
    "aquerenció": "aquerenciar",
    "aquiescencias": "aquiescencia",
    "aquiescentes": "aquiescente",
    "aquieta": "aquietar",
    "aquietaba": "aquietar",
    "aquietabais": "aquietar",
    "aquietaban": "aquietar",
    "aquietabas": "aquietar",
    "aquietad": "aquietar",
    "aquietada": "aquietar",
    "aquietadas": "aquietar",
    "aquietado": "aquietar",
    "aquietadora": "aquietador",
    "aquietadoras": "aquietador",
    "aquietadores": "aquietador",
    "aquietados": "aquietar",
    "aquietamientos": "aquietamiento",
    "aquietamos": "aquietar",
    "aquietan": "aquietar",
    "aquietando": "aquietar",
    "aquietara": "aquietar",
    "aquietarais": "aquietar",
    "aquietaran": "aquietar",
    "aquietaras": "aquietar",
    "aquietare": "aquietar",
    "aquietareis": "aquietar",
    "aquietaremos": "aquietar",
    "aquietaren": "aquietar",
    "aquietares": "aquietar",
    "aquietaron": "aquietar",
    "aquietará": "aquietar",
    "aquietarán": "aquietar",
    "aquietarás": "aquietar",
    "aquietaré": "aquietar",
    "aquietaréis": "aquietar",
    "aquietaría": "aquietar",
    "aquietaríais": "aquietar",
    "aquietaríamos": "aquietar",
    "aquietarían": "aquietar",
    "aquietarías": "aquietar",
    "aquietas": "aquietar",
    "aquietase": "aquietar",
    "aquietaseis": "aquietar",
    "aquietasen": "aquietar",
    "aquietases": "aquietar",
    "aquietaste": "aquietar",
    "aquietasteis": "aquietar",
    "aquiete": "aquietar",
    "aquietemos": "aquietar",
    "aquieten": "aquietar",
    "aquietes": "aquietar",
    "aquieto": "aquietar",
    "aquietábamos": "aquietar",
    "aquietáis": "aquietar",
    "aquietáramos": "aquietar",
    "aquietáremos": "aquietar",
    "aquietásemos": "aquietar",
    "aquieté": "aquietar",
    "aquietéis": "aquietar",
    "aquietó": "aquietar",
    "aquilata": "aquilatar",
    "aquilataba": "aquilatar",
    "aquilatabais": "aquilatar",
    "aquilataban": "aquilatar",
    "aquilatabas": "aquilatar",
    "aquilatad": "aquilatar",
    "aquilatada": "aquilatar",
    "aquilatadas": "aquilatar",
    "aquilatado": "aquilatar",
    "aquilatados": "aquilatar",
    "aquilatamientos": "aquilatamiento",
    "aquilatamos": "aquilatar",
    "aquilatan": "aquilatar",
    "aquilatando": "aquilatar",
    "aquilatara": "aquilatar",
    "aquilatarais": "aquilatar",
    "aquilataran": "aquilatar",
    "aquilataras": "aquilatar",
    "aquilatare": "aquilatar",
    "aquilatareis": "aquilatar",
    "aquilataremos": "aquilatar",
    "aquilataren": "aquilatar",
    "aquilatares": "aquilatar",
    "aquilataron": "aquilatar",
    "aquilatará": "aquilatar",
    "aquilatarán": "aquilatar",
    "aquilatarás": "aquilatar",
    "aquilataré": "aquilatar",
    "aquilataréis": "aquilatar",
    "aquilataría": "aquilatar",
    "aquilataríais": "aquilatar",
    "aquilataríamos": "aquilatar",
    "aquilatarían": "aquilatar",
    "aquilatarías": "aquilatar",
    "aquilatas": "aquilatar",
    "aquilatase": "aquilatar",
    "aquilataseis": "aquilatar",
    "aquilatasen": "aquilatar",
    "aquilatases": "aquilatar",
    "aquilataste": "aquilatar",
    "aquilatasteis": "aquilatar",
    "aquilate": "aquilatar",
    "aquilatemos": "aquilatar",
    "aquilaten": "aquilatar",
    "aquilates": "aquilatar",
    "aquilato": "aquilatar",
    "aquilatábamos": "aquilatar",
    "aquilatáis": "aquilatar",
    "aquilatáramos": "aquilatar",
    "aquilatáremos": "aquilatar",
    "aquilatásemos": "aquilatar",
    "aquilaté": "aquilatar",
    "aquilatéis": "aquilatar",
    "aquilató": "aquilatar",
    "aquilina": "aquilino",
    "aquilinas": "aquilino",
    "aquilinos": "aquilino",
    "aquillada": "aquillado",
    "aquilladas": "aquillado",
    "aquillados": "aquillado",
    "aquilones": "aquilón",
    "aquitana": "aquitano",
    "aquitanas": "aquitano",
    "aquitanos": "aquitano",
    "aquél": "aquel",
    "aquélla": "aquel",
    "aquéllas": "aquel",
    "aquéllos": "aquel",
    "ara": "arar",
    "araba": "arar",
    "arabais": "arar",
    "araban": "arar",
    "arabas": "arar",
    "arabesca": "arabesco",
    "arabescas": "arabesco",
    "arabescos": "arabesco",
    "arabice": "arabizar",
    "arabicemos": "arabizar",
    "arabicen": "arabizar",
    "arabices": "arabizar",
    "arabicé": "arabizar",
    "arabicéis": "arabizar",
    "arabinosas": "arabinosa",
    "arabismos": "arabismo",
    "arabistas": "arabista",
    "arabiza": "arabizar",
    "arabizaba": "arabizar",
    "arabizabais": "arabizar",
    "arabizaban": "arabizar",
    "arabizabas": "arabizar",
    "arabizaciones": "arabización",
    "arabizad": "arabizar",
    "arabizada": "arabizar",
    "arabizadas": "arabizar",
    "arabizado": "arabizar",
    "arabizados": "arabizar",
    "arabizamos": "arabizar",
    "arabizan": "arabizar",
    "arabizando": "arabizar",
    "arabizantes": "arabizante",
    "arabizara": "arabizar",
    "arabizarais": "arabizar",
    "arabizaran": "arabizar",
    "arabizaras": "arabizar",
    "arabizare": "arabizar",
    "arabizareis": "arabizar",
    "arabizaremos": "arabizar",
    "arabizaren": "arabizar",
    "arabizares": "arabizar",
    "arabizaron": "arabizar",
    "arabizará": "arabizar",
    "arabizarán": "arabizar",
    "arabizarás": "arabizar",
    "arabizaré": "arabizar",
    "arabizaréis": "arabizar",
    "arabizaría": "arabizar",
    "arabizaríais": "arabizar",
    "arabizaríamos": "arabizar",
    "arabizarían": "arabizar",
    "arabizarías": "arabizar",
    "arabizas": "arabizar",
    "arabizase": "arabizar",
    "arabizaseis": "arabizar",
    "arabizasen": "arabizar",
    "arabizases": "arabizar",
    "arabizaste": "arabizar",
    "arabizasteis": "arabizar",
    "arabizo": "arabizar",
    "arabizábamos": "arabizar",
    "arabizáis": "arabizar",
    "arabizáramos": "arabizar",
    "arabizáremos": "arabizar",
    "arabizásemos": "arabizar",
    "arabizó": "arabizar",
    "arables": "arable",
    "araboberéberes": "araboberéber",
    "aracnoidea": "aracnoideo",
    "aracnoideas": "aracnoideo",
    "aracnoideos": "aracnoideo",
    "aracnologías": "aracnología",
    "aracnóideses": "aracnoides",
    "aracnóloga": "aracnólogo",
    "aracnólogas": "aracnólogo",
    "aracnólogos": "aracnólogo",
    "arad": "arar",
    "arada": "arar",
    "aradas": "arar",
    "arado": "arar",
    "aradora": "arador",
    "aradoras": "arador",
    "aradores": "arador",
    "arados": "arar",
    "aragonesa": "aragonés",
    "aragonesas": "aragonés",
    "aragoneses": "aragonés",
    "aragonesismos": "aragonesismo",
    "aragonitos": "aragonito",
    "arahuacos": "arahuaco",
    "araliáceas": "araliácea",
    "arambeles": "arambel",
    "aramea": "arameo",
    "arameas": "arameo",
    "arameos": "arameo",
    "aramos": "arar",
    "aran": "arar",
    "arancelaria": "arancelario",
    "arancelarias": "arancelario",
    "arancelarios": "arancelario",
    "aranceles": "arancel",
    "arandas": "aranda",
    "arandelas": "arandela",
    "arando": "arar",
    "araneidos": "araneido",
    "aranesa": "aranés",
    "aranesas": "aranés",
    "araneses": "aranés",
    "araos": "arao",
    "arapaha": "arapaho",
    "arapahas": "arapaho",
    "arapahos": "arapaho",
    "arapaimas": "arapaima",
    "araquidatos": "araquidato",
    "araquídica": "araquídico",
    "araquídicas": "araquídico",
    "araquídicos": "araquídico",
    "arara": "arar",
    "ararais": "arar",
    "araran": "arar",
    "araras": "arar",
    "arare": "arar",
    "arareis": "arar",
    "araremos": "arar",
    "araren": "arar",
    "arares": "arar",
    "araron": "arar",
    "arará": "arar",
    "ararán": "arar",
    "ararás": "arar",
    "araré": "arar",
    "araréis": "arar",
    "araría": "arar",
    "araríais": "arar",
    "araríamos": "arar",
    "ararían": "arar",
    "ararías": "arar",
    "aras": "arar",
    "arase": "arar",
    "araseis": "arar",
    "arasen": "arar",
    "arases": "arar",
    "araste": "arar",
    "arasteis": "arar",
    "araucana": "araucano",
    "araucanas": "araucano",
    "araucanos": "araucano",
    "araucarias": "araucaria",
    "araña": "arañar",
    "arañaba": "arañar",
    "arañabais": "arañar",
    "arañaban": "arañar",
    "arañabas": "arañar",
    "arañad": "arañar",
    "arañada": "arañar",
    "arañadas": "arañar",
    "arañado": "arañar",
    "arañadora": "arañador",
    "arañadoras": "arañador",
    "arañadores": "arañador",
    "arañados": "arañar",
    "arañamientos": "arañamiento",
    "arañamos": "arañar",
    "arañan": "arañar",
    "arañando": "arañar",
    "arañara": "arañar",
    "arañarais": "arañar",
    "arañaran": "arañar",
    "arañaras": "arañar",
    "arañare": "arañar",
    "arañareis": "arañar",
    "arañaremos": "arañar",
    "arañaren": "arañar",
    "arañares": "arañar",
    "arañaron": "arañar",
    "arañará": "arañar",
    "arañarán": "arañar",
    "arañarás": "arañar",
    "arañaré": "arañar",
    "arañaréis": "arañar",
    "arañaría": "arañar",
    "arañaríais": "arañar",
    "arañaríamos": "arañar",
    "arañarían": "arañar",
    "arañarías": "arañar",
    "arañas": "arañar",
    "arañase": "arañar",
    "arañaseis": "arañar",
    "arañasen": "arañar",
    "arañases": "arañar",
    "arañaste": "arañar",
    "arañasteis": "arañar",
    "arañazos": "arañazo",
    "arañe": "arañar",
    "arañemos": "arañar",
    "arañen": "arañar",
    "arañes": "arañar",
    "araño": "arañar",
    "arañuelas": "arañuela",
    "arañuelos": "arañuelo",
    "arañábamos": "arañar",
    "arañáis": "arañar",
    "arañáramos": "arañar",
    "arañáremos": "arañar",
    "arañásemos": "arañar",
    "arañé": "arañar",
    "arañéis": "arañar",
    "arañó": "arañar",
    "arbequina": "arbequino",
    "arbequinas": "arbequino",
    "arbequines": "arbequín",
    "arbequinos": "arbequino",
    "arbitra": "arbitrar",
    "arbitraba": "arbitrar",
    "arbitrabais": "arbitrar",
    "arbitraban": "arbitrar",
    "arbitrabas": "arbitrar",
    "arbitrad": "arbitrar",
    "arbitrada": "arbitrar",
    "arbitradas": "arbitrar",
    "arbitrado": "arbitrar",
    "arbitrados": "arbitrar",
    "arbitrajes": "arbitraje",
    "arbitrales": "arbitral",
    "arbitramentos": "arbitramento",
    "arbitramientos": "arbitramiento",
    "arbitramos": "arbitrar",
    "arbitran": "arbitrar",
    "arbitrando": "arbitrar",
    "arbitrara": "arbitrar",
    "arbitrarais": "arbitrar",
    "arbitraran": "arbitrar",
    "arbitraras": "arbitrar",
    "arbitrare": "arbitrar",
    "arbitrareis": "arbitrar",
    "arbitraremos": "arbitrar",
    "arbitraren": "arbitrar",
    "arbitrares": "arbitrar",
    "arbitraria": "arbitrario",
    "arbitrarias": "arbitrario",
    "arbitrariedades": "arbitrariedad",
    "arbitrarios": "arbitrario",
    "arbitraron": "arbitrar",
    "arbitrará": "arbitrar",
    "arbitrarán": "arbitrar",
    "arbitrarás": "arbitrar",
    "arbitraré": "arbitrar",
    "arbitraréis": "arbitrar",
    "arbitraría": "arbitrar",
    "arbitraríais": "arbitrar",
    "arbitraríamos": "arbitrar",
    "arbitrarían": "arbitrar",
    "arbitrarías": "arbitrar",
    "arbitras": "arbitrar",
    "arbitrase": "arbitrar",
    "arbitraseis": "arbitrar",
    "arbitrasen": "arbitrar",
    "arbitrases": "arbitrar",
    "arbitraste": "arbitrar",
    "arbitrasteis": "arbitrar",
    "arbitrativa": "arbitrativo",
    "arbitrativas": "arbitrativo",
    "arbitrativos": "arbitrativo",
    "arbitre": "arbitrar",
    "arbitremos": "arbitrar",
    "arbitren": "arbitrar",
    "arbitres": "arbitrar",
    "arbitrios": "arbitrio",
    "arbitrismos": "arbitrismo",
    "arbitristas": "arbitrista",
    "arbitro": "arbitrar",
    "arbitrábamos": "arbitrar",
    "arbitráis": "arbitrar",
    "arbitráramos": "arbitrar",
    "arbitráremos": "arbitrar",
    "arbitrásemos": "arbitrar",
    "arbitré": "arbitrar",
    "arbitréis": "arbitrar",
    "arbitró": "arbitrar",
    "arbola": "arbolar",
    "arbolaba": "arbolar",
    "arbolabais": "arbolar",
    "arbolaban": "arbolar",
    "arbolabas": "arbolar",
    "arbolad": "arbolar",
    "arbolada": "arbolar",
    "arboladas": "arbolar",
    "arbolado": "arbolar",
    "arbolados": "arbolar",
    "arboladuras": "arboladura",
    "arbolamos": "arbolar",
    "arbolan": "arbolar",
    "arbolando": "arbolar",
    "arbolara": "arbolar",
    "arbolarais": "arbolar",
    "arbolaran": "arbolar",
    "arbolaras": "arbolar",
    "arbolare": "arbolar",
    "arbolareis": "arbolar",
    "arbolaremos": "arbolar",
    "arbolaren": "arbolar",
    "arbolares": "arbolar",
    "arbolaron": "arbolar",
    "arbolará": "arbolar",
    "arbolarán": "arbolar",
    "arbolarás": "arbolar",
    "arbolaré": "arbolar",
    "arbolaréis": "arbolar",
    "arbolaría": "arbolar",
    "arbolaríais": "arbolar",
    "arbolaríamos": "arbolar",
    "arbolarían": "arbolar",
    "arbolarías": "arbolar",
    "arbolas": "arbolar",
    "arbolase": "arbolar",
    "arbolaseis": "arbolar",
    "arbolasen": "arbolar",
    "arbolases": "arbolar",
    "arbolaste": "arbolar",
    "arbolasteis": "arbolar",
    "arbole": "arbolar",
    "arbolece": "arbolecer",
    "arboleced": "arbolecer",
    "arbolecemos": "arbolecer",
    "arbolecen": "arbolecer",
    "arboleceremos": "arbolecer",
    "arbolecerá": "arbolecer",
    "arbolecerán": "arbolecer",
    "arbolecerás": "arbolecer",
    "arboleceré": "arbolecer",
    "arboleceréis": "arbolecer",
    "arbolecería": "arbolecer",
    "arboleceríais": "arbolecer",
    "arboleceríamos": "arbolecer",
    "arbolecerían": "arbolecer",
    "arbolecerías": "arbolecer",
    "arboleces": "arbolecer",
    "arbolecida": "arbolecer",
    "arbolecidas": "arbolecer",
    "arbolecido": "arbolecer",
    "arbolecidos": "arbolecer",
    "arboleciendo": "arbolecer",
    "arboleciera": "arbolecer",
    "arbolecierais": "arbolecer",
    "arbolecieran": "arbolecer",
    "arbolecieras": "arbolecer",
    "arboleciere": "arbolecer",
    "arboleciereis": "arbolecer",
    "arbolecieren": "arbolecer",
    "arbolecieres": "arbolecer",
    "arbolecieron": "arbolecer",
    "arboleciese": "arbolecer",
    "arbolecieseis": "arbolecer",
    "arboleciesen": "arbolecer",
    "arbolecieses": "arbolecer",
    "arbolecimos": "arbolecer",
    "arboleciste": "arbolecer",
    "arbolecisteis": "arbolecer",
    "arboleciéramos": "arbolecer",
    "arboleciéremos": "arbolecer",
    "arboleciésemos": "arbolecer",
    "arboleció": "arbolecer",
    "arbolecéis": "arbolecer",
    "arbolecí": "arbolecer",
    "arbolecía": "arbolecer",
    "arbolecíais": "arbolecer",
    "arbolecíamos": "arbolecer",
    "arbolecían": "arbolecer",
    "arbolecías": "arbolecer",
    "arboledas": "arboleda",
    "arbolemos": "arbolar",
    "arbolen": "arbolar",
    "arboles": "arbolar",
    "arboletes": "arbolete",
    "arbolezca": "arbolecer",
    "arbolezcamos": "arbolecer",
    "arbolezcan": "arbolecer",
    "arbolezcas": "arbolecer",
    "arbolezco": "arbolecer",
    "arbolezcáis": "arbolecer",
    "arbolistas": "arbolista",
    "arbolo": "arbolar",
    "arbolábamos": "arbolar",
    "arboláis": "arbolar",
    "arboláramos": "arbolar",
    "arboláremos": "arbolar",
    "arbolásemos": "arbolar",
    "arbolé": "arbolar",
    "arboléis": "arbolar",
    "arboló": "arbolar",
    "arborece": "arborecer",
    "arboreced": "arborecer",
    "arborecemos": "arborecer",
    "arborecen": "arborecer",
    "arboreceremos": "arborecer",
    "arborecerá": "arborecer",
    "arborecerán": "arborecer",
    "arborecerás": "arborecer",
    "arboreceré": "arborecer",
    "arboreceréis": "arborecer",
    "arborecería": "arborecer",
    "arboreceríais": "arborecer",
    "arboreceríamos": "arborecer",
    "arborecerían": "arborecer",
    "arborecerías": "arborecer",
    "arboreces": "arborecer",
    "arborecida": "arborecer",
    "arborecidas": "arborecer",
    "arborecido": "arborecer",
    "arborecidos": "arborecer",
    "arboreciendo": "arborecer",
    "arboreciera": "arborecer",
    "arborecierais": "arborecer",
    "arborecieran": "arborecer",
    "arborecieras": "arborecer",
    "arboreciere": "arborecer",
    "arboreciereis": "arborecer",
    "arborecieren": "arborecer",
    "arborecieres": "arborecer",
    "arborecieron": "arborecer",
    "arboreciese": "arborecer",
    "arborecieseis": "arborecer",
    "arboreciesen": "arborecer",
    "arborecieses": "arborecer",
    "arborecimos": "arborecer",
    "arboreciste": "arborecer",
    "arborecisteis": "arborecer",
    "arboreciéramos": "arborecer",
    "arboreciéremos": "arborecer",
    "arboreciésemos": "arborecer",
    "arboreció": "arborecer",
    "arborecéis": "arborecer",
    "arborecí": "arborecer",
    "arborecía": "arborecer",
    "arborecíais": "arborecer",
    "arborecíamos": "arborecer",
    "arborecían": "arborecer",
    "arborecías": "arborecer",
    "arborescencias": "arborescencia",
    "arborescentes": "arborescente",
    "arborezca": "arborecer",
    "arborezcamos": "arborecer",
    "arborezcan": "arborecer",
    "arborezcas": "arborecer",
    "arborezco": "arborecer",
    "arborezcáis": "arborecer",
    "arboricultora": "arboricultor",
    "arboricultoras": "arboricultor",
    "arboricultores": "arboricultor",
    "arboriculturas": "arboricultura",
    "arboriformes": "arboriforme",
    "arborizaciones": "arborización",
    "arborícolas": "arborícola",
    "arbotantes": "arbotante",
    "arbustiva": "arbustivo",
    "arbustivas": "arbustivo",
    "arbustivos": "arbustivo",
    "arbustos": "arbusto",
    "arbórea": "arbóreo",
    "arbóreas": "arbóreo",
    "arbóreos": "arbóreo",
    "arcabucea": "arcabucear",
    "arcabuceaba": "arcabucear",
    "arcabuceabais": "arcabucear",
    "arcabuceaban": "arcabucear",
    "arcabuceabas": "arcabucear",
    "arcabucead": "arcabucear",
    "arcabuceada": "arcabucear",
    "arcabuceadas": "arcabucear",
    "arcabuceado": "arcabucear",
    "arcabuceados": "arcabucear",
    "arcabuceamos": "arcabucear",
    "arcabucean": "arcabucear",
    "arcabuceando": "arcabucear",
    "arcabuceara": "arcabucear",
    "arcabucearais": "arcabucear",
    "arcabucearan": "arcabucear",
    "arcabucearas": "arcabucear",
    "arcabuceare": "arcabucear",
    "arcabuceareis": "arcabucear",
    "arcabucearemos": "arcabucear",
    "arcabucearen": "arcabucear",
    "arcabuceares": "arcabucear",
    "arcabucearon": "arcabucear",
    "arcabuceará": "arcabucear",
    "arcabucearán": "arcabucear",
    "arcabucearás": "arcabucear",
    "arcabucearé": "arcabucear",
    "arcabucearéis": "arcabucear",
    "arcabucearía": "arcabucear",
    "arcabucearíais": "arcabucear",
    "arcabucearíamos": "arcabucear",
    "arcabucearían": "arcabucear",
    "arcabucearías": "arcabucear",
    "arcabuceas": "arcabucear",
    "arcabucease": "arcabucear",
    "arcabuceaseis": "arcabucear",
    "arcabuceasen": "arcabucear",
    "arcabuceases": "arcabucear",
    "arcabuceaste": "arcabucear",
    "arcabuceasteis": "arcabucear",
    "arcabucee": "arcabucear",
    "arcabuceemos": "arcabucear",
    "arcabuceen": "arcabucear",
    "arcabucees": "arcabucear",
    "arcabuceo": "arcabucear",
    "arcabucera": "arcabucero",
    "arcabuceras": "arcabucero",
    "arcabuceros": "arcabucero",
    "arcabucerías": "arcabucería",
    "arcabuces": "arcabuz",
    "arcabuceábamos": "arcabucear",
    "arcabuceáis": "arcabucear",
    "arcabuceáramos": "arcabucear",
    "arcabuceáremos": "arcabucear",
    "arcabuceásemos": "arcabucear",
    "arcabuceé": "arcabucear",
    "arcabuceéis": "arcabucear",
    "arcabuceó": "arcabucear",
    "arcabuzazos": "arcabuzazo",
    "arcadas": "arcada",
    "arcadi": "arcadia",
    "arcaduces": "arcaduz",
    "arcaica": "arcaico",
    "arcaicas": "arcaico",
    "arcaicemos": "arcaizar",
    "arcaicos": "arcaico",
    "arcaicé": "arcaizar",
    "arcaicéis": "arcaizar",
    "arcaizaba": "arcaizar",
    "arcaizabais": "arcaizar",
    "arcaizaban": "arcaizar",
    "arcaizabas": "arcaizar",
    "arcaizad": "arcaizar",
    "arcaizada": "arcaizar",
    "arcaizadas": "arcaizar",
    "arcaizado": "arcaizar",
    "arcaizados": "arcaizar",
    "arcaizamos": "arcaizar",
    "arcaizando": "arcaizar",
    "arcaizantes": "arcaizante",
    "arcaizara": "arcaizar",
    "arcaizarais": "arcaizar",
    "arcaizaran": "arcaizar",
    "arcaizaras": "arcaizar",
    "arcaizare": "arcaizar",
    "arcaizareis": "arcaizar",
    "arcaizaremos": "arcaizar",
    "arcaizaren": "arcaizar",
    "arcaizares": "arcaizar",
    "arcaizaron": "arcaizar",
    "arcaizará": "arcaizar",
    "arcaizarán": "arcaizar",
    "arcaizarás": "arcaizar",
    "arcaizaré": "arcaizar",
    "arcaizaréis": "arcaizar",
    "arcaizaría": "arcaizar",
    "arcaizaríais": "arcaizar",
    "arcaizaríamos": "arcaizar",
    "arcaizarían": "arcaizar",
    "arcaizarías": "arcaizar",
    "arcaizase": "arcaizar",
    "arcaizaseis": "arcaizar",
    "arcaizasen": "arcaizar",
    "arcaizases": "arcaizar",
    "arcaizaste": "arcaizar",
    "arcaizasteis": "arcaizar",
    "arcaizábamos": "arcaizar",
    "arcaizáis": "arcaizar",
    "arcaizáramos": "arcaizar",
    "arcaizáremos": "arcaizar",
    "arcaizásemos": "arcaizar",
    "arcaizó": "arcaizar",
    "arcana": "arcano",
    "arcanas": "arcano",
    "arcanos": "arcano",
    "arcas": "arca",
    "arcaíce": "arcaizar",
    "arcaícen": "arcaizar",
    "arcaíces": "arcaizar",
    "arcaísmos": "arcaísmo",
    "arcaíza": "arcaizar",
    "arcaízan": "arcaizar",
    "arcaízas": "arcaizar",
    "arcaízo": "arcaizar",
    "arcedianatos": "arcedianato",
    "arcedianos": "arcediano",
    "arcedos": "arcedo",
    "arcenes": "arcén",
    "arces": "arce",
    "archeros": "archero",
    "archibebés": "archibebé",
    "archicofrades": "archicofrade",
    "archicofradías": "archicofradía",
    "archidiáconos": "archidiácono",
    "archiducados": "archiducado",
    "archiducales": "archiducal",
    "archiduques": "archiduque",
    "archiduquesas": "archiduquesa",
    "archienemiga": "archienemigo",
    "archienemigas": "archienemigo",
    "archienemigos": "archienemigo",
    "archifonemas": "archifonema",
    "archifonemática": "archifonemático",
    "archifonemáticas": "archifonemático",
    "archifonemáticos": "archifonemático",
    "archilaúdes": "archilaúd",
    "archimandritas": "archimandrita",
    "archipelágica": "archipelágico",
    "archipelágicas": "archipelágico",
    "archipelágicos": "archipelágico",
    "archipiélagos": "archipiélago",
    "archiva": "archivar",
    "archivaba": "archivar",
    "archivabais": "archivar",
    "archivaban": "archivar",
    "archivabas": "archivar",
    "archivaciones": "archivación",
    "archivad": "archivar",
    "archivada": "archivar",
    "archivadas": "archivar",
    "archivado": "archivar",
    "archivadora": "archivador",
    "archivadoras": "archivador",
    "archivadores": "archivador",
    "archivados": "archivar",
    "archivamos": "archivar",
    "archivan": "archivar",
    "archivando": "archivar",
    "archivara": "archivar",
    "archivarais": "archivar",
    "archivaran": "archivar",
    "archivaras": "archivar",
    "archivare": "archivar",
    "archivareis": "archivar",
    "archivaremos": "archivar",
    "archivaren": "archivar",
    "archivares": "archivar",
    "archivaron": "archivar",
    "archivará": "archivar",
    "archivarán": "archivar",
    "archivarás": "archivar",
    "archivaré": "archivar",
    "archivaréis": "archivar",
    "archivaría": "archivar",
    "archivaríais": "archivar",
    "archivaríamos": "archivar",
    "archivarían": "archivar",
    "archivarías": "archivar",
    "archivas": "archivar",
    "archivase": "archivar",
    "archivaseis": "archivar",
    "archivasen": "archivar",
    "archivases": "archivar",
    "archivaste": "archivar",
    "archivasteis": "archivar",
    "archive": "archivar",
    "archivemos": "archivar",
    "archiven": "archivar",
    "archivera": "archivero",
    "archiveras": "archivero",
    "archiveros": "archivero",
    "archives": "archivar",
    "archivistas": "archivista",
    "archivo": "archivar",
    "archivoltas": "archivolta",
    "archivos": "archivo",
    "archivábamos": "archivar",
    "archiváis": "archivar",
    "archiváramos": "archivar",
    "archiváremos": "archivar",
    "archivásemos": "archivar",
    "archivé": "archivar",
    "archivéis": "archivar",
    "archivística": "archivístico",
    "archivísticas": "archivístico",
    "archivísticos": "archivístico",
    "archivó": "archivar",
    "arciformes": "arciforme",
    "arcilla": "arcillar",
    "arcillaba": "arcillar",
    "arcillabais": "arcillar",
    "arcillaban": "arcillar",
    "arcillabas": "arcillar",
    "arcillad": "arcillar",
    "arcillada": "arcillar",
    "arcilladas": "arcillar",
    "arcillado": "arcillar",
    "arcillados": "arcillar",
    "arcillamos": "arcillar",
    "arcillan": "arcillar",
    "arcillando": "arcillar",
    "arcillara": "arcillar",
    "arcillarais": "arcillar",
    "arcillaran": "arcillar",
    "arcillaras": "arcillar",
    "arcillare": "arcillar",
    "arcillareis": "arcillar",
    "arcillaremos": "arcillar",
    "arcillaren": "arcillar",
    "arcillares": "arcillar",
    "arcillaron": "arcillar",
    "arcillará": "arcillar",
    "arcillarán": "arcillar",
    "arcillarás": "arcillar",
    "arcillaré": "arcillar",
    "arcillaréis": "arcillar",
    "arcillaría": "arcillar",
    "arcillaríais": "arcillar",
    "arcillaríamos": "arcillar",
    "arcillarían": "arcillar",
    "arcillarías": "arcillar",
    "arcillas": "arcillar",
    "arcillase": "arcillar",
    "arcillaseis": "arcillar",
    "arcillasen": "arcillar",
    "arcillases": "arcillar",
    "arcillaste": "arcillar",
    "arcillasteis": "arcillar",
    "arcille": "arcillar",
    "arcillemos": "arcillar",
    "arcillen": "arcillar",
    "arcilles": "arcillar",
    "arcillo": "arcillar",
    "arcillosa": "arcilloso",
    "arcillosas": "arcilloso",
    "arcillosos": "arcilloso",
    "arcillábamos": "arcillar",
    "arcilláis": "arcillar",
    "arcilláramos": "arcillar",
    "arcilláremos": "arcillar",
    "arcillásemos": "arcillar",
    "arcillé": "arcillar",
    "arcilléis": "arcillar",
    "arcilló": "arcillar",
    "arcilosa": "arciloso",
    "arcilosas": "arciloso",
    "arcilosos": "arciloso",
    "arciones": "arción",
    "arciprestazgos": "arciprestazgo",
    "arciprestes": "arcipreste",
    "arcocosenos": "arcocoseno",
    "arcones": "arcón",
    "arcontes": "arconte",
    "arcos": "arco",
    "arcosas": "arcosa",
    "arcosenos": "arcoseno",
    "arcosolios": "arcosolio",
    "arcuada": "arcuado",
    "arcuadas": "arcuado",
    "arcuados": "arcuado",
    "arcángeles": "arcángel",
    "arda": "arder",
    "ardamos": "arder",
    "ardan": "arder",
    "ardas": "arder",
    "arde": "arder",
    "arded": "arder",
    "ardemos": "arder",
    "arden": "arder",
    "arderemos": "arder",
    "arderá": "arder",
    "arderán": "arder",
    "arderás": "arder",
    "arderé": "arder",
    "arderéis": "arder",
    "ardería": "arder",
    "arderíais": "arder",
    "arderíamos": "arder",
    "arderían": "arder",
    "arderías": "arder",
    "ardes": "arder",
    "ardida": "arder",
    "ardidas": "arder",
    "ardides": "ardid",
    "ardido": "arder",
    "ardidos": "arder",
    "ardiendo": "arder",
    "ardientes": "ardiente",
    "ardiera": "arder",
    "ardierais": "arder",
    "ardieran": "arder",
    "ardieras": "arder",
    "ardiere": "arder",
    "ardiereis": "arder",
    "ardieren": "arder",
    "ardieres": "arder",
    "ardieron": "arder",
    "ardiese": "arder",
    "ardieseis": "arder",
    "ardiesen": "arder",
    "ardieses": "arder",
    "ardillas": "ardilla",
    "ardimientos": "ardimiento",
    "ardimos": "arder",
    "ardiste": "arder",
    "ardisteis": "arder",
    "ardites": "ardite",
    "ardiéramos": "arder",
    "ardiéremos": "arder",
    "ardiésemos": "arder",
    "ardió": "arder",
    "ardo": "arder",
    "ardores": "ardor",
    "ardorosa": "ardoroso",
    "ardorosas": "ardoroso",
    "ardorosos": "ardoroso",
    "ardua": "arduo",
    "arduas": "arduo",
    "arduidades": "arduidad",
    "arduos": "arduo",
    "ardáis": "arder",
    "ardéis": "arder",
    "ardí": "arder",
    "ardía": "arder",
    "ardíais": "arder",
    "ardíamos": "arder",
    "ardían": "arder",
    "ardías": "arder",
    "are": "arar",
    "arefacciones": "arefacción",
    "arela": "arelar",
    "arelaba": "arelar",
    "arelabais": "arelar",
    "arelaban": "arelar",
    "arelabas": "arelar",
    "arelad": "arelar",
    "arelada": "arelar",
    "areladas": "arelar",
    "arelado": "arelar",
    "arelados": "arelar",
    "arelamos": "arelar",
    "arelan": "arelar",
    "arelando": "arelar",
    "arelara": "arelar",
    "arelarais": "arelar",
    "arelaran": "arelar",
    "arelaras": "arelar",
    "arelare": "arelar",
    "arelareis": "arelar",
    "arelaremos": "arelar",
    "arelaren": "arelar",
    "arelares": "arelar",
    "arelaron": "arelar",
    "arelará": "arelar",
    "arelarán": "arelar",
    "arelarás": "arelar",
    "arelaré": "arelar",
    "arelaréis": "arelar",
    "arelaría": "arelar",
    "arelaríais": "arelar",
    "arelaríamos": "arelar",
    "arelarían": "arelar",
    "arelarías": "arelar",
    "arelas": "arelar",
    "arelase": "arelar",
    "arelaseis": "arelar",
    "arelasen": "arelar",
    "arelases": "arelar",
    "arelaste": "arelar",
    "arelasteis": "arelar",
    "arele": "arelar",
    "arelemos": "arelar",
    "arelen": "arelar",
    "areles": "arelar",
    "arelo": "arelar",
    "arelábamos": "arelar",
    "areláis": "arelar",
    "areláramos": "arelar",
    "areláremos": "arelar",
    "arelásemos": "arelar",
    "arelé": "arelar",
    "areléis": "arelar",
    "areló": "arelar",
    "aremos": "arar",
    "aren": "arar",
    "arena": "arenar",
    "arenaba": "arenar",
    "arenabais": "arenar",
    "arenaban": "arenar",
    "arenabas": "arenar",
    "arenad": "arenar",
    "arenada": "arenar",
    "arenadas": "arenar",
    "arenado": "arenar",
    "arenadores": "arenador",
    "arenados": "arenar",
    "arenales": "arenal",
    "arenamos": "arenar",
    "arenan": "arenar",
    "arenando": "arenar",
    "arenara": "arenar",
    "arenarais": "arenar",
    "arenaran": "arenar",
    "arenaras": "arenar",
    "arenare": "arenar",
    "arenareis": "arenar",
    "arenaremos": "arenar",
    "arenaren": "arenar",
    "arenares": "arenar",
    "arenaron": "arenar",
    "arenará": "arenar",
    "arenarán": "arenar",
    "arenarás": "arenar",
    "arenaré": "arenar",
    "arenaréis": "arenar",
    "arenaría": "arenar",
    "arenaríais": "arenar",
    "arenaríamos": "arenar",
    "arenarían": "arenar",
    "arenarías": "arenar",
    "arenas": "arenar",
    "arenase": "arenar",
    "arenaseis": "arenar",
    "arenasen": "arenar",
    "arenases": "arenar",
    "arenaste": "arenar",
    "arenasteis": "arenar",
    "arenca": "arencar",
    "arencaba": "arencar",
    "arencabais": "arencar",
    "arencaban": "arencar",
    "arencabas": "arencar",
    "arencad": "arencar",
    "arencada": "arencar",
    "arencadas": "arencar",
    "arencado": "arencar",
    "arencados": "arencar",
    "arencamos": "arencar",
    "arencan": "arencar",
    "arencando": "arencar",
    "arencara": "arencar",
    "arencarais": "arencar",
    "arencaran": "arencar",
    "arencaras": "arencar",
    "arencare": "arencar",
    "arencareis": "arencar",
    "arencaremos": "arencar",
    "arencaren": "arencar",
    "arencares": "arencar",
    "arencaron": "arencar",
    "arencará": "arencar",
    "arencarán": "arencar",
    "arencarás": "arencar",
    "arencaré": "arencar",
    "arencaréis": "arencar",
    "arencaría": "arencar",
    "arencaríais": "arencar",
    "arencaríamos": "arencar",
    "arencarían": "arencar",
    "arencarías": "arencar",
    "arencas": "arencar",
    "arencase": "arencar",
    "arencaseis": "arencar",
    "arencasen": "arencar",
    "arencases": "arencar",
    "arencaste": "arencar",
    "arencasteis": "arencar",
    "arenco": "arencar",
    "arencábamos": "arencar",
    "arencáis": "arencar",
    "arencáramos": "arencar",
    "arencáremos": "arencar",
    "arencásemos": "arencar",
    "arencó": "arencar",
    "arene": "arenar",
    "arenemos": "arenar",
    "arenen": "arenar",
    "areneros": "arenero",
    "arenes": "arenar",
    "arenga": "arengar",
    "arengaba": "arengar",
    "arengabais": "arengar",
    "arengaban": "arengar",
    "arengabas": "arengar",
    "arengad": "arengar",
    "arengada": "arengar",
    "arengadas": "arengar",
    "arengado": "arengar",
    "arengados": "arengar",
    "arengamos": "arengar",
    "arengan": "arengar",
    "arengando": "arengar",
    "arengara": "arengar",
    "arengarais": "arengar",
    "arengaran": "arengar",
    "arengaras": "arengar",
    "arengare": "arengar",
    "arengareis": "arengar",
    "arengaremos": "arengar",
    "arengaren": "arengar",
    "arengares": "arengar",
    "arengaron": "arengar",
    "arengará": "arengar",
    "arengarán": "arengar",
    "arengarás": "arengar",
    "arengaré": "arengar",
    "arengaréis": "arengar",
    "arengaría": "arengar",
    "arengaríais": "arengar",
    "arengaríamos": "arengar",
    "arengarían": "arengar",
    "arengarías": "arengar",
    "arengas": "arengar",
    "arengase": "arengar",
    "arengaseis": "arengar",
    "arengasen": "arengar",
    "arengases": "arengar",
    "arengaste": "arengar",
    "arengasteis": "arengar",
    "arengo": "arengar",
    "arengue": "arengar",
    "arenguemos": "arengar",
    "arenguen": "arengar",
    "arengues": "arengar",
    "arengué": "arengar",
    "arenguéis": "arengar",
    "arengábamos": "arengar",
    "arengáis": "arengar",
    "arengáramos": "arengar",
    "arengáremos": "arengar",
    "arengásemos": "arengar",
    "arengó": "arengar",
    "arenillas": "arenilla",
    "arenisca": "arenisco",
    "areniscas": "arenisco",
    "areniscos": "arenisco",
    "areno": "arenar",
    "arenosa": "arenoso",
    "arenosas": "arenoso",
    "arenosoles": "arenosol",
    "arenosos": "arenoso",
    "arenque": "arencar",
    "arenquemos": "arencar",
    "arenquen": "arencar",
    "arenques": "arenque",
    "arenqué": "arencar",
    "arenquéis": "arencar",
    "arenábamos": "arenar",
    "arenácea": "arenáceo",
    "arenáceas": "arenáceo",
    "arenáceos": "arenáceo",
    "arenáis": "arenar",
    "arenáramos": "arenar",
    "arenáremos": "arenar",
    "arenásemos": "arenar",
    "arené": "arenar",
    "arenéis": "arenar",
    "arenícolas": "arenícola",
    "arenó": "arenar",
    "areolares": "areolar",
    "areolas": "areola",
    "areopagitas": "areopagita",
    "areopicnómetros": "areopicnómetro",
    "areosístila": "areosístilo",
    "areosístilas": "areosístilo",
    "areosístilos": "areosístilo",
    "arepas": "arepa",
    "ares": "arar",
    "aretes": "arete",
    "areómetros": "areómetro",
    "areópagos": "areópago",
    "areóstila": "areóstilo",
    "areóstilas": "areóstilo",
    "areóstilos": "areóstilo",
    "arfa": "arfar",
    "arfaba": "arfar",
    "arfabais": "arfar",
    "arfaban": "arfar",
    "arfabas": "arfar",
    "arfad": "arfar",
    "arfada": "arfar",
    "arfadas": "arfar",
    "arfado": "arfar",
    "arfados": "arfar",
    "arfamos": "arfar",
    "arfan": "arfar",
    "arfando": "arfar",
    "arfara": "arfar",
    "arfarais": "arfar",
    "arfaran": "arfar",
    "arfaras": "arfar",
    "arfare": "arfar",
    "arfareis": "arfar",
    "arfaremos": "arfar",
    "arfaren": "arfar",
    "arfares": "arfar",
    "arfaron": "arfar",
    "arfará": "arfar",
    "arfarán": "arfar",
    "arfarás": "arfar",
    "arfaré": "arfar",
    "arfaréis": "arfar",
    "arfaría": "arfar",
    "arfaríais": "arfar",
    "arfaríamos": "arfar",
    "arfarían": "arfar",
    "arfarías": "arfar",
    "arfas": "arfar",
    "arfase": "arfar",
    "arfaseis": "arfar",
    "arfasen": "arfar",
    "arfases": "arfar",
    "arfaste": "arfar",
    "arfasteis": "arfar",
    "arfe": "arfar",
    "arfemos": "arfar",
    "arfen": "arfar",
    "arfes": "arfar",
    "arfo": "arfar",
    "arfábamos": "arfar",
    "arfáis": "arfar",
    "arfáramos": "arfar",
    "arfáremos": "arfar",
    "arfásemos": "arfar",
    "arfé": "arfar",
    "arféis": "arfar",
    "arfó": "arfar",
    "argadillos": "argadillo",
    "argalleras": "argallera",
    "argalíes": "argalí",
    "argalís": "argalí",
    "argamandijos": "argamandijo",
    "argamasa": "argamasar",
    "argamasaba": "argamasar",
    "argamasabais": "argamasar",
    "argamasaban": "argamasar",
    "argamasabas": "argamasar",
    "argamasad": "argamasar",
    "argamasada": "argamasar",
    "argamasadas": "argamasar",
    "argamasado": "argamasar",
    "argamasados": "argamasar",
    "argamasamos": "argamasar",
    "argamasan": "argamasar",
    "argamasando": "argamasar",
    "argamasara": "argamasar",
    "argamasarais": "argamasar",
    "argamasaran": "argamasar",
    "argamasaras": "argamasar",
    "argamasare": "argamasar",
    "argamasareis": "argamasar",
    "argamasaremos": "argamasar",
    "argamasaren": "argamasar",
    "argamasares": "argamasar",
    "argamasaron": "argamasar",
    "argamasará": "argamasar",
    "argamasarán": "argamasar",
    "argamasarás": "argamasar",
    "argamasaré": "argamasar",
    "argamasaréis": "argamasar",
    "argamasaría": "argamasar",
    "argamasaríais": "argamasar",
    "argamasaríamos": "argamasar",
    "argamasarían": "argamasar",
    "argamasarías": "argamasar",
    "argamasas": "argamasar",
    "argamasase": "argamasar",
    "argamasaseis": "argamasar",
    "argamasasen": "argamasar",
    "argamasases": "argamasar",
    "argamasaste": "argamasar",
    "argamasasteis": "argamasar",
    "argamase": "argamasar",
    "argamasemos": "argamasar",
    "argamasen": "argamasar",
    "argamases": "argamasar",
    "argamaso": "argamasar",
    "argamasábamos": "argamasar",
    "argamasáis": "argamasar",
    "argamasáramos": "argamasar",
    "argamasáremos": "argamasar",
    "argamasásemos": "argamasar",
    "argamasé": "argamasar",
    "argamaséis": "argamasar",
    "argamasó": "argamasar",
    "arganeles": "arganel",
    "arganeos": "arganeo",
    "arganes": "argán",
    "argelina": "argelino",
    "argelinas": "argelino",
    "argelinos": "argelino",
    "argenta": "argentar",
    "argentaba": "argentar",
    "argentabais": "argentar",
    "argentaban": "argentar",
    "argentabas": "argentar",
    "argentad": "argentar",
    "argentada": "argentar",
    "argentadas": "argentar",
    "argentado": "argentar",
    "argentados": "argentar",
    "argentamos": "argentar",
    "argentan": "argentar",
    "argentando": "argentar",
    "argentara": "argentar",
    "argentarais": "argentar",
    "argentaran": "argentar",
    "argentaras": "argentar",
    "argentare": "argentar",
    "argentareis": "argentar",
    "argentaremos": "argentar",
    "argentaren": "argentar",
    "argentares": "argentar",
    "argentaria": "argentario",
    "argentarias": "argentario",
    "argentarios": "argentario",
    "argentaron": "argentar",
    "argentará": "argentar",
    "argentarán": "argentar",
    "argentarás": "argentar",
    "argentaré": "argentar",
    "argentaréis": "argentar",
    "argentaría": "argentar",
    "argentaríais": "argentar",
    "argentaríamos": "argentar",
    "argentarían": "argentar",
    "argentarías": "argentar",
    "argentas": "argentar",
    "argentase": "argentar",
    "argentaseis": "argentar",
    "argentasen": "argentar",
    "argentases": "argentar",
    "argentaste": "argentar",
    "argentasteis": "argentar",
    "argente": "argentar",
    "argentemos": "argentar",
    "argenten": "argentar",
    "argentera": "argentero",
    "argenteras": "argentero",
    "argenteros": "argentero",
    "argenterías": "argentería",
    "argentes": "argentar",
    "argentina": "argentino",
    "argentinas": "argentino",
    "argentinismos": "argentinismo",
    "argentinos": "argentino",
    "argentitas": "argentita",
    "argento": "argentar",
    "argentosa": "argentoso",
    "argentosas": "argentoso",
    "argentosos": "argentoso",
    "argentábamos": "argentar",
    "argentáis": "argentar",
    "argentáramos": "argentar",
    "argentáremos": "argentar",
    "argentásemos": "argentar",
    "argenté": "argentar",
    "argentéis": "argentar",
    "argentífera": "argentífero",
    "argentíferas": "argentífero",
    "argentíferos": "argentífero",
    "argentó": "argentar",
    "argilizaciones": "argilización",
    "argiloturbaciones": "argiloturbación",
    "arginasas": "arginasa",
    "argininas": "arginina",
    "argiva": "argivo",
    "argivas": "argivo",
    "argivos": "argivo",
    "argoles": "argol",
    "argollas": "argolla",
    "argonautas": "argonauta",
    "argones": "argón",
    "argotes": "argot",
    "argots": "argot",
    "argucias": "argucia",
    "argumenta": "argumentar",
    "argumentaba": "argumentar",
    "argumentabais": "argumentar",
    "argumentaban": "argumentar",
    "argumentabas": "argumentar",
    "argumentaciones": "argumentación",
    "argumentad": "argumentar",
    "argumentada": "argumentar",
    "argumentadas": "argumentar",
    "argumentado": "argumentar",
    "argumentados": "argumentar",
    "argumentales": "argumental",
    "argumentamos": "argumentar",
    "argumentan": "argumentar",
    "argumentando": "argumentar",
    "argumentara": "argumentar",
    "argumentarais": "argumentar",
    "argumentaran": "argumentar",
    "argumentaras": "argumentar",
    "argumentare": "argumentar",
    "argumentareis": "argumentar",
    "argumentaremos": "argumentar",
    "argumentaren": "argumentar",
    "argumentares": "argumentar",
    "argumentaron": "argumentar",
    "argumentará": "argumentar",
    "argumentarán": "argumentar",
    "argumentarás": "argumentar",
    "argumentaré": "argumentar",
    "argumentaréis": "argumentar",
    "argumentaría": "argumentar",
    "argumentaríais": "argumentar",
    "argumentaríamos": "argumentar",
    "argumentarían": "argumentar",
    "argumentarías": "argumentar",
    "argumentas": "argumentar",
    "argumentase": "argumentar",
    "argumentaseis": "argumentar",
    "argumentasen": "argumentar",
    "argumentases": "argumentar",
    "argumentaste": "argumentar",
    "argumentasteis": "argumentar",
    "argumentativa": "argumentativo",
    "argumentativas": "argumentativo",
    "argumentativos": "argumentativo",
    "argumente": "argumentar",
    "argumentemos": "argumentar",
    "argumenten": "argumentar",
    "argumentes": "argumentar",
    "argumentistas": "argumentista",
    "argumento": "argumentar",
    "argumentos": "argumento",
    "argumentábamos": "argumentar",
    "argumentáis": "argumentar",
    "argumentáramos": "argumentar",
    "argumentáremos": "argumentar",
    "argumentásemos": "argumentar",
    "argumenté": "argumentar",
    "argumentéis": "argumentar",
    "argumentó": "argumentar",
    "arguya": "argüir",
    "arguyamos": "argüir",
    "arguyan": "argüir",
    "arguyas": "argüir",
    "arguye": "argüir",
    "arguyen": "argüir",
    "arguyendo": "argüir",
    "arguyera": "argüir",
    "arguyerais": "argüir",
    "arguyeran": "argüir",
    "arguyeras": "argüir",
    "arguyere": "argüir",
    "arguyereis": "argüir",
    "arguyeren": "argüir",
    "arguyeres": "argüir",
    "arguyeron": "argüir",
    "arguyes": "argüir",
    "arguyese": "argüir",
    "arguyeseis": "argüir",
    "arguyesen": "argüir",
    "arguyeses": "argüir",
    "arguyo": "argüir",
    "arguyáis": "argüir",
    "arguyéramos": "argüir",
    "arguyéremos": "argüir",
    "arguyésemos": "argüir",
    "arguyó": "argüir",
    "argárica": "argárico",
    "argáricas": "argárico",
    "argáricos": "argárico",
    "argéntea": "argénteo",
    "argénteas": "argénteo",
    "argénteos": "argénteo",
    "argílica": "argílico",
    "argílicas": "argílico",
    "argílicos": "argílico",
    "argótica": "argótico",
    "argóticas": "argótico",
    "argóticos": "argótico",
    "argüid": "argüir",
    "argüida": "argüir",
    "argüidas": "argüir",
    "argüido": "argüir",
    "argüidos": "argüir",
    "argüimos": "argüir",
    "argüiremos": "argüir",
    "argüirá": "argüir",
    "argüirán": "argüir",
    "argüirás": "argüir",
    "argüiré": "argüir",
    "argüiréis": "argüir",
    "argüiría": "argüir",
    "argüiríais": "argüir",
    "argüiríamos": "argüir",
    "argüirían": "argüir",
    "argüirías": "argüir",
    "argüiste": "argüir",
    "argüisteis": "argüir",
    "argüí": "argüir",
    "argüía": "argüir",
    "argüíais": "argüir",
    "argüíamos": "argüir",
    "argüían": "argüir",
    "argüías": "argüir",
    "argüís": "argüir",
    "aria": "ario",
    "arias": "ario",
    "arica": "aricar",
    "aricaba": "aricar",
    "aricabais": "aricar",
    "aricaban": "aricar",
    "aricabas": "aricar",
    "aricad": "aricar",
    "aricada": "aricar",
    "aricadas": "aricar",
    "aricado": "aricar",
    "aricados": "aricar",
    "aricamos": "aricar",
    "arican": "aricar",
    "aricando": "aricar",
    "aricara": "aricar",
    "aricarais": "aricar",
    "aricaran": "aricar",
    "aricaras": "aricar",
    "aricare": "aricar",
    "aricareis": "aricar",
    "aricaremos": "aricar",
    "aricaren": "aricar",
    "aricares": "aricar",
    "aricaron": "aricar",
    "aricará": "aricar",
    "aricarán": "aricar",
    "aricarás": "aricar",
    "aricaré": "aricar",
    "aricaréis": "aricar",
    "aricaría": "aricar",
    "aricaríais": "aricar",
    "aricaríamos": "aricar",
    "aricarían": "aricar",
    "aricarías": "aricar",
    "aricas": "aricar",
    "aricase": "aricar",
    "aricaseis": "aricar",
    "aricasen": "aricar",
    "aricases": "aricar",
    "aricaste": "aricar",
    "aricasteis": "aricar",
    "arico": "aricar",
    "aricábamos": "aricar",
    "aricáis": "aricar",
    "aricáramos": "aricar",
    "aricáremos": "aricar",
    "aricásemos": "aricar",
    "aricó": "aricar",
    "aridades": "aridad",
    "aridece": "aridecer",
    "arideced": "aridecer",
    "aridecemos": "aridecer",
    "aridecen": "aridecer",
    "arideceremos": "aridecer",
    "aridecerá": "aridecer",
    "aridecerán": "aridecer",
    "aridecerás": "aridecer",
    "arideceré": "aridecer",
    "arideceréis": "aridecer",
    "aridecería": "aridecer",
    "arideceríais": "aridecer",
    "arideceríamos": "aridecer",
    "aridecerían": "aridecer",
    "aridecerías": "aridecer",
    "arideces": "aridez",
    "aridecida": "aridecer",
    "aridecidas": "aridecer",
    "aridecido": "aridecer",
    "aridecidos": "aridecer",
    "arideciendo": "aridecer",
    "arideciera": "aridecer",
    "aridecierais": "aridecer",
    "aridecieran": "aridecer",
    "aridecieras": "aridecer",
    "arideciere": "aridecer",
    "arideciereis": "aridecer",
    "aridecieren": "aridecer",
    "aridecieres": "aridecer",
    "aridecieron": "aridecer",
    "arideciese": "aridecer",
    "aridecieseis": "aridecer",
    "arideciesen": "aridecer",
    "aridecieses": "aridecer",
    "aridecimos": "aridecer",
    "arideciste": "aridecer",
    "aridecisteis": "aridecer",
    "arideciéramos": "aridecer",
    "arideciéremos": "aridecer",
    "arideciésemos": "aridecer",
    "arideció": "aridecer",
    "aridecéis": "aridecer",
    "aridecí": "aridecer",
    "aridecía": "aridecer",
    "aridecíais": "aridecer",
    "aridecíamos": "aridecer",
    "aridecían": "aridecer",
    "aridecías": "aridecer",
    "aridezca": "aridecer",
    "aridezcamos": "aridecer",
    "aridezcan": "aridecer",
    "aridezcas": "aridecer",
    "aridezco": "aridecer",
    "aridezcáis": "aridecer",
    "arietas": "arieta",
    "arietes": "ariete",
    "arilaciones": "arilación",
    "arilos": "arilo",
    "arios": "ario",
    "arique": "aricar",
    "ariquemos": "aricar",
    "ariquen": "aricar",
    "ariques": "aricar",
    "ariqué": "aricar",
    "ariquéis": "aricar",
    "arisca": "arisco",
    "ariscas": "arisco",
    "ariscos": "arisco",
    "aristada": "aristado",
    "aristadas": "aristado",
    "aristados": "aristado",
    "aristas": "arista",
    "aristocracias": "aristocracia",
    "aristocratice": "aristocratizar",
    "aristocraticemos": "aristocratizar",
    "aristocraticen": "aristocratizar",
    "aristocratices": "aristocratizar",
    "aristocraticé": "aristocratizar",
    "aristocraticéis": "aristocratizar",
    "aristocratiza": "aristocratizar",
    "aristocratizaba": "aristocratizar",
    "aristocratizabais": "aristocratizar",
    "aristocratizaban": "aristocratizar",
    "aristocratizabas": "aristocratizar",
    "aristocratizaciones": "aristocratización",
    "aristocratizad": "aristocratizar",
    "aristocratizada": "aristocratizar",
    "aristocratizadas": "aristocratizar",
    "aristocratizado": "aristocratizar",
    "aristocratizados": "aristocratizar",
    "aristocratizamos": "aristocratizar",
    "aristocratizan": "aristocratizar",
    "aristocratizando": "aristocratizar",
    "aristocratizara": "aristocratizar",
    "aristocratizarais": "aristocratizar",
    "aristocratizaran": "aristocratizar",
    "aristocratizaras": "aristocratizar",
    "aristocratizare": "aristocratizar",
    "aristocratizareis": "aristocratizar",
    "aristocratizaremos": "aristocratizar",
    "aristocratizaren": "aristocratizar",
    "aristocratizares": "aristocratizar",
    "aristocratizaron": "aristocratizar",
    "aristocratizará": "aristocratizar",
    "aristocratizarán": "aristocratizar",
    "aristocratizarás": "aristocratizar",
    "aristocratizaré": "aristocratizar",
    "aristocratizaréis": "aristocratizar",
    "aristocratizaría": "aristocratizar",
    "aristocratizaríais": "aristocratizar",
    "aristocratizaríamos": "aristocratizar",
    "aristocratizarían": "aristocratizar",
    "aristocratizarías": "aristocratizar",
    "aristocratizas": "aristocratizar",
    "aristocratizase": "aristocratizar",
    "aristocratizaseis": "aristocratizar",
    "aristocratizasen": "aristocratizar",
    "aristocratizases": "aristocratizar",
    "aristocratizaste": "aristocratizar",
    "aristocratizasteis": "aristocratizar",
    "aristocratizo": "aristocratizar",
    "aristocratizábamos": "aristocratizar",
    "aristocratizáis": "aristocratizar",
    "aristocratizáramos": "aristocratizar",
    "aristocratizáremos": "aristocratizar",
    "aristocratizásemos": "aristocratizar",
    "aristocratizó": "aristocratizar",
    "aristocrática": "aristocrático",
    "aristocráticas": "aristocrático",
    "aristocráticos": "aristocrático",
    "aristotelismos": "aristotelismo",
    "aristotélica": "aristotélico",
    "aristotélicas": "aristotélico",
    "aristotélicos": "aristotélico",
    "aristócratas": "aristócrata",
    "aritenoepiglótica": "aritenoepiglótico",
    "aritenoepiglóticas": "aritenoepiglótico",
    "aritenoepiglóticos": "aritenoepiglótico",
    "aritenoidea": "aritenoideo",
    "aritenoideas": "aritenoideo",
    "aritenoidectomías": "aritenoidectomía",
    "aritenoideos": "aritenoideo",
    "aritenoidopexias": "aritenoidopexia",
    "aritenopexias": "aritenopexia",
    "aritmética": "aritmético",
    "aritméticas": "aritmético",
    "aritméticos": "aritmético",
    "aritmógrafos": "aritmógrafo",
    "aritmómetros": "aritmómetro",
    "arlequines": "arlequín",
    "arma": "armar",
    "armaba": "armar",
    "armabais": "armar",
    "armaban": "armar",
    "armabas": "armar",
    "armad": "armar",
    "armada": "armar",
    "armadas": "armar",
    "armadijos": "armadijo",
    "armadillos": "armadillo",
    "armado": "armar",
    "armadora": "armador",
    "armadoras": "armador",
    "armadores": "armador",
    "armados": "armar",
    "armaduras": "armadura",
    "armadías": "armadía",
    "armajales": "armajal",
    "armamentismos": "armamentismo",
    "armamentistas": "armamentista",
    "armamentos": "armamento",
    "armamentística": "armamentístico",
    "armamentísticas": "armamentístico",
    "armamentísticos": "armamentístico",
    "armamos": "armar",
    "arman": "armar",
    "armando": "armar",
    "armara": "armar",
    "armarais": "armar",
    "armaran": "armar",
    "armaras": "armar",
    "armare": "armar",
    "armareis": "armar",
    "armaremos": "armar",
    "armaren": "armar",
    "armares": "armar",
    "armarios": "armario",
    "armaron": "armar",
    "armará": "armar",
    "armarán": "armar",
    "armarás": "armar",
    "armaré": "armar",
    "armaréis": "armar",
    "armaría": "armar",
    "armaríais": "armar",
    "armaríamos": "armar",
    "armarían": "armar",
    "armarías": "armar",
    "armas": "armar",
    "armase": "armar",
    "armaseis": "armar",
    "armasen": "armar",
    "armases": "armar",
    "armaste": "armar",
    "armasteis": "armar",
    "armatolos": "armatolo",
    "armatostes": "armatoste",
    "armazones": "armazón",
    "armazónes": "armazón",
    "armañaces": "armañac",
    "arme": "armar",
    "armellas": "armella",
    "armemos": "armar",
    "armen": "armar",
    "armenia": "armenio",
    "armenias": "armenio",
    "armenios": "armenio",
    "armenoides": "armenoide",
    "armera": "armero",
    "armeras": "armero",
    "armeros": "armero",
    "armerías": "armería",
    "armes": "armar",
    "armillas": "armilla",
    "armisticios": "armisticio",
    "armiñada": "armiñado",
    "armiñadas": "armiñado",
    "armiñados": "armiñado",
    "armiños": "armiño",
    "armo": "armar",
    "armones": "armón",
    "armonice": "armonizar",
    "armonicemos": "armonizar",
    "armonicen": "armonizar",
    "armonices": "armonizar",
    "armonicistas": "armonicista",
    "armonicé": "armonizar",
    "armonicéis": "armonizar",
    "armonios": "armonio",
    "armoniosa": "armonioso",
    "armoniosas": "armonioso",
    "armoniosos": "armonioso",
    "armonistas": "armonista",
    "armoniza": "armonizar",
    "armonizaba": "armonizar",
    "armonizabais": "armonizar",
    "armonizaban": "armonizar",
    "armonizabas": "armonizar",
    "armonizaciones": "armonización",
    "armonizad": "armonizar",
    "armonizada": "armonizar",
    "armonizadas": "armonizar",
    "armonizado": "armonizar",
    "armonizados": "armonizar",
    "armonizamos": "armonizar",
    "armonizan": "armonizar",
    "armonizando": "armonizar",
    "armonizara": "armonizar",
    "armonizarais": "armonizar",
    "armonizaran": "armonizar",
    "armonizaras": "armonizar",
    "armonizare": "armonizar",
    "armonizareis": "armonizar",
    "armonizaremos": "armonizar",
    "armonizaren": "armonizar",
    "armonizares": "armonizar",
    "armonizaron": "armonizar",
    "armonizará": "armonizar",
    "armonizarán": "armonizar",
    "armonizarás": "armonizar",
    "armonizaré": "armonizar",
    "armonizaréis": "armonizar",
    "armonizaría": "armonizar",
    "armonizaríais": "armonizar",
    "armonizaríamos": "armonizar",
    "armonizarían": "armonizar",
    "armonizarías": "armonizar",
    "armonizas": "armonizar",
    "armonizase": "armonizar",
    "armonizaseis": "armonizar",
    "armonizasen": "armonizar",
    "armonizases": "armonizar",
    "armonizaste": "armonizar",
    "armonizasteis": "armonizar",
    "armonizo": "armonizar",
    "armonizábamos": "armonizar",
    "armonizáis": "armonizar",
    "armonizáramos": "armonizar",
    "armonizáremos": "armonizar",
    "armonizásemos": "armonizar",
    "armonizó": "armonizar",
    "armonías": "armonía",
    "armoricana": "armoricano",
    "armoricanas": "armoricano",
    "armoricanos": "armoricano",
    "armábamos": "armar",
    "armáis": "armar",
    "armáramos": "armar",
    "armáremos": "armar",
    "armásemos": "armar",
    "armé": "armar",
    "arméis": "armar",
    "armó": "armar",
    "armónica": "armónico",
    "armónicas": "armónico",
    "armónicos": "armónico",
    "armóniums": "armónium",
    "arnas": "arna",
    "arneses": "arnés",
    "aro": "arar",
    "aroma": "aromar",
    "aromaba": "aromar",
    "aromabais": "aromar",
    "aromaban": "aromar",
    "aromabas": "aromar",
    "aromad": "aromar",
    "aromada": "aromar",
    "aromadas": "aromar",
    "aromado": "aromar",
    "aromados": "aromar",
    "aromamos": "aromar",
    "aroman": "aromar",
    "aromando": "aromar",
    "aromara": "aromar",
    "aromarais": "aromar",
    "aromaran": "aromar",
    "aromaras": "aromar",
    "aromare": "aromar",
    "aromareis": "aromar",
    "aromaremos": "aromar",
    "aromaren": "aromar",
    "aromares": "aromar",
    "aromaron": "aromar",
    "aromará": "aromar",
    "aromarán": "aromar",
    "aromarás": "aromar",
    "aromaré": "aromar",
    "aromaréis": "aromar",
    "aromaría": "aromar",
    "aromaríais": "aromar",
    "aromaríamos": "aromar",
    "aromarían": "aromar",
    "aromarías": "aromar",
    "aromas": "aromar",
    "aromase": "aromar",
    "aromaseis": "aromar",
    "aromasen": "aromar",
    "aromases": "aromar",
    "aromaste": "aromar",
    "aromasteis": "aromar",
    "aromatice": "aromatizar",
    "aromaticemos": "aromatizar",
    "aromaticen": "aromatizar",
    "aromatices": "aromatizar",
    "aromaticidades": "aromaticidad",
    "aromaticé": "aromatizar",
    "aromaticéis": "aromatizar",
    "aromatiza": "aromatizar",
    "aromatizaba": "aromatizar",
    "aromatizabais": "aromatizar",
    "aromatizaban": "aromatizar",
    "aromatizabas": "aromatizar",
    "aromatizaciones": "aromatización",
    "aromatizad": "aromatizar",
    "aromatizada": "aromatizar",
    "aromatizadas": "aromatizar",
    "aromatizado": "aromatizar",
    "aromatizados": "aromatizar",
    "aromatizamos": "aromatizar",
    "aromatizan": "aromatizar",
    "aromatizando": "aromatizar",
    "aromatizara": "aromatizar",
    "aromatizarais": "aromatizar",
    "aromatizaran": "aromatizar",
    "aromatizaras": "aromatizar",
    "aromatizare": "aromatizar",
    "aromatizareis": "aromatizar",
    "aromatizaremos": "aromatizar",
    "aromatizaren": "aromatizar",
    "aromatizares": "aromatizar",
    "aromatizaron": "aromatizar",
    "aromatizará": "aromatizar",
    "aromatizarán": "aromatizar",
    "aromatizarás": "aromatizar",
    "aromatizaré": "aromatizar",
    "aromatizaréis": "aromatizar",
    "aromatizaría": "aromatizar",
    "aromatizaríais": "aromatizar",
    "aromatizaríamos": "aromatizar",
    "aromatizarían": "aromatizar",
    "aromatizarías": "aromatizar",
    "aromatizas": "aromatizar",
    "aromatizase": "aromatizar",
    "aromatizaseis": "aromatizar",
    "aromatizasen": "aromatizar",
    "aromatizases": "aromatizar",
    "aromatizaste": "aromatizar",
    "aromatizasteis": "aromatizar",
    "aromatizo": "aromatizar",
    "aromatizábamos": "aromatizar",
    "aromatizáis": "aromatizar",
    "aromatizáramos": "aromatizar",
    "aromatizáremos": "aromatizar",
    "aromatizásemos": "aromatizar",
    "aromatizó": "aromatizar",
    "arome": "aromar",
    "aromemos": "aromar",
    "aromen": "aromar",
    "aromes": "aromar",
    "aromo": "aromar",
    "aromos": "aromo",
    "aromábamos": "aromar",
    "aromáis": "aromar",
    "aromáramos": "aromar",
    "aromáremos": "aromar",
    "aromásemos": "aromar",
    "aromática": "aromático",
    "aromáticas": "aromático",
    "aromáticos": "aromático",
    "aromé": "aromar",
    "aroméis": "aromar",
    "aromó": "aromar",
    "aros": "aro",
    "arpa": "arpar",
    "arpaba": "arpar",
    "arpabais": "arpar",
    "arpaban": "arpar",
    "arpabas": "arpar",
    "arpad": "arpar",
    "arpada": "arpar",
    "arpadas": "arpar",
    "arpado": "arpar",
    "arpados": "arpar",
    "arpaduras": "arpadura",
    "arpamos": "arpar",
    "arpan": "arpar",
    "arpando": "arpar",
    "arpara": "arpar",
    "arparais": "arpar",
    "arparan": "arpar",
    "arparas": "arpar",
    "arpare": "arpar",
    "arpareis": "arpar",
    "arparemos": "arpar",
    "arparen": "arpar",
    "arpares": "arpar",
    "arparon": "arpar",
    "arpará": "arpar",
    "arparán": "arpar",
    "arparás": "arpar",
    "arparé": "arpar",
    "arparéis": "arpar",
    "arparía": "arpar",
    "arparíais": "arpar",
    "arparíamos": "arpar",
    "arparían": "arpar",
    "arparías": "arpar",
    "arpas": "arpar",
    "arpase": "arpar",
    "arpaseis": "arpar",
    "arpasen": "arpar",
    "arpases": "arpar",
    "arpaste": "arpar",
    "arpasteis": "arpar",
    "arpe": "arpar",
    "arpegia": "arpegiar",
    "arpegiaba": "arpegiar",
    "arpegiabais": "arpegiar",
    "arpegiaban": "arpegiar",
    "arpegiabas": "arpegiar",
    "arpegiad": "arpegiar",
    "arpegiada": "arpegiar",
    "arpegiadas": "arpegiar",
    "arpegiado": "arpegiar",
    "arpegiados": "arpegiar",
    "arpegiamos": "arpegiar",
    "arpegian": "arpegiar",
    "arpegiando": "arpegiar",
    "arpegiara": "arpegiar",
    "arpegiarais": "arpegiar",
    "arpegiaran": "arpegiar",
    "arpegiaras": "arpegiar",
    "arpegiare": "arpegiar",
    "arpegiareis": "arpegiar",
    "arpegiaremos": "arpegiar",
    "arpegiaren": "arpegiar",
    "arpegiares": "arpegiar",
    "arpegiaron": "arpegiar",
    "arpegiará": "arpegiar",
    "arpegiarán": "arpegiar",
    "arpegiarás": "arpegiar",
    "arpegiaré": "arpegiar",
    "arpegiaréis": "arpegiar",
    "arpegiaría": "arpegiar",
    "arpegiaríais": "arpegiar",
    "arpegiaríamos": "arpegiar",
    "arpegiarían": "arpegiar",
    "arpegiarías": "arpegiar",
    "arpegias": "arpegiar",
    "arpegiase": "arpegiar",
    "arpegiaseis": "arpegiar",
    "arpegiasen": "arpegiar",
    "arpegiases": "arpegiar",
    "arpegiaste": "arpegiar",
    "arpegiasteis": "arpegiar",
    "arpegie": "arpegiar",
    "arpegiemos": "arpegiar",
    "arpegien": "arpegiar",
    "arpegies": "arpegiar",
    "arpegio": "arpegiar",
    "arpegios": "arpegio",
    "arpegiábamos": "arpegiar",
    "arpegiáis": "arpegiar",
    "arpegiáramos": "arpegiar",
    "arpegiáremos": "arpegiar",
    "arpegiásemos": "arpegiar",
    "arpegié": "arpegiar",
    "arpegiéis": "arpegiar",
    "arpegió": "arpegiar",
    "arpemos": "arpar",
    "arpen": "arpar",
    "arpeos": "arpeo",
    "arpes": "arpar",
    "arpilla": "arpillar",
    "arpillaba": "arpillar",
    "arpillabais": "arpillar",
    "arpillaban": "arpillar",
    "arpillabas": "arpillar",
    "arpillad": "arpillar",
    "arpillada": "arpillar",
    "arpilladas": "arpillar",
    "arpillado": "arpillar",
    "arpillados": "arpillar",
    "arpillamos": "arpillar",
    "arpillan": "arpillar",
    "arpillando": "arpillar",
    "arpillara": "arpillar",
    "arpillarais": "arpillar",
    "arpillaran": "arpillar",
    "arpillaras": "arpillar",
    "arpillare": "arpillar",
    "arpillareis": "arpillar",
    "arpillaremos": "arpillar",
    "arpillaren": "arpillar",
    "arpillares": "arpillar",
    "arpillaron": "arpillar",
    "arpillará": "arpillar",
    "arpillarán": "arpillar",
    "arpillarás": "arpillar",
    "arpillaré": "arpillar",
    "arpillaréis": "arpillar",
    "arpillaría": "arpillar",
    "arpillaríais": "arpillar",
    "arpillaríamos": "arpillar",
    "arpillarían": "arpillar",
    "arpillarías": "arpillar",
    "arpillas": "arpillar",
    "arpillase": "arpillar",
    "arpillaseis": "arpillar",
    "arpillasen": "arpillar",
    "arpillases": "arpillar",
    "arpillaste": "arpillar",
    "arpillasteis": "arpillar",
    "arpille": "arpillar",
    "arpillemos": "arpillar",
    "arpillen": "arpillar",
    "arpilleras": "arpillera",
    "arpilles": "arpillar",
    "arpillo": "arpillar",
    "arpillábamos": "arpillar",
    "arpilláis": "arpillar",
    "arpilláramos": "arpillar",
    "arpilláremos": "arpillar",
    "arpillásemos": "arpillar",
    "arpillé": "arpillar",
    "arpilléis": "arpillar",
    "arpilló": "arpillar",
    "arpistas": "arpista",
    "arpo": "arpar",
    "arpona": "arponar",
    "arponaba": "arponar",
    "arponabais": "arponar",
    "arponaban": "arponar",
    "arponabas": "arponar",
    "arponad": "arponar",
    "arponada": "arponar",
    "arponadas": "arponar",
    "arponado": "arponar",
    "arponados": "arponar",
    "arponamos": "arponar",
    "arponan": "arponar",
    "arponando": "arponar",
    "arponara": "arponar",
    "arponarais": "arponar",
    "arponaran": "arponar",
    "arponaras": "arponar",
    "arponare": "arponar",
    "arponareis": "arponar",
    "arponaremos": "arponar",
    "arponaren": "arponar",
    "arponares": "arponar",
    "arponaron": "arponar",
    "arponará": "arponar",
    "arponarán": "arponar",
    "arponarás": "arponar",
    "arponaré": "arponar",
    "arponaréis": "arponar",
    "arponaría": "arponar",
    "arponaríais": "arponar",
    "arponaríamos": "arponar",
    "arponarían": "arponar",
    "arponarías": "arponar",
    "arponas": "arponar",
    "arponase": "arponar",
    "arponaseis": "arponar",
    "arponasen": "arponar",
    "arponases": "arponar",
    "arponaste": "arponar",
    "arponasteis": "arponar",
    "arpone": "arponar",
    "arponea": "arponear",
    "arponeaba": "arponear",
    "arponeabais": "arponear",
    "arponeaban": "arponear",
    "arponeabas": "arponear",
    "arponead": "arponear",
    "arponeada": "arponear",
    "arponeadas": "arponear",
    "arponeado": "arponear",
    "arponeados": "arponear",
    "arponeamos": "arponear",
    "arponean": "arponear",
    "arponeando": "arponear",
    "arponeara": "arponear",
    "arponearais": "arponear",
    "arponearan": "arponear",
    "arponearas": "arponear",
    "arponeare": "arponear",
    "arponeareis": "arponear",
    "arponearemos": "arponear",
    "arponearen": "arponear",
    "arponeares": "arponear",
    "arponearon": "arponear",
    "arponeará": "arponear",
    "arponearán": "arponear",
    "arponearás": "arponear",
    "arponearé": "arponear",
    "arponearéis": "arponear",
    "arponearía": "arponear",
    "arponearíais": "arponear",
    "arponearíamos": "arponear",
    "arponearían": "arponear",
    "arponearías": "arponear",
    "arponeas": "arponear",
    "arponease": "arponear",
    "arponeaseis": "arponear",
    "arponeasen": "arponear",
    "arponeases": "arponear",
    "arponeaste": "arponear",
    "arponeasteis": "arponear",
    "arponee": "arponear",
    "arponeemos": "arponear",
    "arponeen": "arponear",
    "arponees": "arponear",
    "arponemos": "arponar",
    "arponen": "arponar",
    "arponeo": "arponear",
    "arponera": "arponero",
    "arponeras": "arponero",
    "arponeros": "arponero",
    "arpones": "arponar",
    "arponeábamos": "arponear",
    "arponeáis": "arponear",
    "arponeáramos": "arponear",
    "arponeáremos": "arponear",
    "arponeásemos": "arponear",
    "arponeé": "arponear",
    "arponeéis": "arponear",
    "arponeó": "arponear",
    "arpono": "arponar",
    "arponábamos": "arponar",
    "arponáis": "arponar",
    "arponáramos": "arponar",
    "arponáremos": "arponar",
    "arponásemos": "arponar",
    "arponé": "arponar",
    "arponéis": "arponar",
    "arponó": "arponar",
    "arpábamos": "arpar",
    "arpáis": "arpar",
    "arpáramos": "arpar",
    "arpáremos": "arpar",
    "arpásemos": "arpar",
    "arpé": "arpar",
    "arpéis": "arpar",
    "arpías": "arpía",
    "arpó": "arpar",
    "arquea": "arquear",
    "arqueaba": "arquear",
    "arqueabais": "arquear",
    "arqueaban": "arquear",
    "arqueabas": "arquear",
    "arquead": "arquear",
    "arqueada": "arquear",
    "arqueadas": "arquear",
    "arqueado": "arquear",
    "arqueadores": "arqueador",
    "arqueados": "arquear",
    "arqueajes": "arqueaje",
    "arqueamientos": "arqueamiento",
    "arqueamos": "arquear",
    "arquean": "arquear",
    "arqueando": "arquear",
    "arqueara": "arquear",
    "arquearais": "arquear",
    "arquearan": "arquear",
    "arquearas": "arquear",
    "arqueare": "arquear",
    "arqueareis": "arquear",
    "arquearemos": "arquear",
    "arquearen": "arquear",
    "arqueares": "arquear",
    "arquearon": "arquear",
    "arqueará": "arquear",
    "arquearán": "arquear",
    "arquearás": "arquear",
    "arquearé": "arquear",
    "arquearéis": "arquear",
    "arquearía": "arquear",
    "arquearíais": "arquear",
    "arquearíamos": "arquear",
    "arquearían": "arquear",
    "arquearías": "arquear",
    "arqueas": "arquear",
    "arquease": "arquear",
    "arqueaseis": "arquear",
    "arqueasen": "arquear",
    "arqueases": "arquear",
    "arqueaste": "arquear",
    "arqueasteis": "arquear",
    "arquee": "arquear",
    "arqueemos": "arquear",
    "arqueen": "arquear",
    "arquees": "arquear",
    "arquegoniada": "arquegoniado",
    "arquegoniadas": "arquegoniado",
    "arquegoniados": "arquegoniado",
    "arquegoniales": "arquegonial",
    "arquegonios": "arquegonio",
    "arqueo": "arquear",
    "arqueobotánicas": "arqueobotánica",
    "arqueobotánicos": "arqueobotánico",
    "arqueociática": "arqueociático",
    "arqueociáticas": "arqueociático",
    "arqueociáticos": "arqueociático",
    "arqueografías": "arqueografía",
    "arqueologías": "arqueología",
    "arqueolítica": "arqueolítico",
    "arqueolíticas": "arqueolítico",
    "arqueolíticos": "arqueolítico",
    "arqueológica": "arqueológico",
    "arqueológicas": "arqueológico",
    "arqueológicos": "arqueológico",
    "arqueomagnetismos": "arqueomagnetismo",
    "arqueos": "arqueo",
    "arqueozoologías": "arqueozoología",
    "arquera": "arquero",
    "arqueras": "arquero",
    "arqueros": "arquero",
    "arquerías": "arquería",
    "arquetas": "arqueta",
    "arquetipos": "arquetipo",
    "arquetípica": "arquetípico",
    "arquetípicas": "arquetípico",
    "arquetípicos": "arquetípico",
    "arqueábamos": "arquear",
    "arqueáis": "arquear",
    "arqueáramos": "arquear",
    "arqueáremos": "arquear",
    "arqueásemos": "arquear",
    "arqueé": "arquear",
    "arqueéis": "arquear",
    "arqueó": "arquear",
    "arqueóloga": "arqueólogo",
    "arqueólogas": "arqueólogo",
    "arqueólogos": "arqueólogo",
    "arquibancos": "arquibanco",
    "arquillos": "arquillo",
    "arquimediana": "arquimediano",
    "arquimedianas": "arquimediano",
    "arquimedianos": "arquimediano",
    "arquimesas": "arquimesa",
    "arquimicetes": "arquimicete",
    "arquisinagogos": "arquisinagogo",
    "arquitecta": "arquitecto",
    "arquitectas": "arquitecto",
    "arquitectos": "arquitecto",
    "arquitecturales": "arquitectural",
    "arquitecturas": "arquitectura",
    "arquitectónica": "arquitectónico",
    "arquitectónicas": "arquitectónico",
    "arquitectónicos": "arquitectónico",
    "arquitrabes": "arquitrabe",
    "arquivoltas": "arquivolta",
    "arquíatras": "arquíatra",
    "arrabalera": "arrabalero",
    "arrabaleras": "arrabalero",
    "arrabaleros": "arrabalero",
    "arrabales": "arrabal",
    "arrabios": "arrabio",
    "arrabás": "arrabá",
    "arracadas": "arracada",
    "arracima": "arracimar",
    "arracimaba": "arracimar",
    "arracimabais": "arracimar",
    "arracimaban": "arracimar",
    "arracimabas": "arracimar",
    "arracimad": "arracimar",
    "arracimada": "arracimar",
    "arracimadas": "arracimar",
    "arracimado": "arracimar",
    "arracimados": "arracimar",
    "arracimamos": "arracimar",
    "arraciman": "arracimar",
    "arracimando": "arracimar",
    "arracimara": "arracimar",
    "arracimarais": "arracimar",
    "arracimaran": "arracimar",
    "arracimaras": "arracimar",
    "arracimare": "arracimar",
    "arracimareis": "arracimar",
    "arracimaremos": "arracimar",
    "arracimaren": "arracimar",
    "arracimares": "arracimar",
    "arracimaron": "arracimar",
    "arracimará": "arracimar",
    "arracimarán": "arracimar",
    "arracimarás": "arracimar",
    "arracimaré": "arracimar",
    "arracimaréis": "arracimar",
    "arracimaría": "arracimar",
    "arracimaríais": "arracimar",
    "arracimaríamos": "arracimar",
    "arracimarían": "arracimar",
    "arracimarías": "arracimar",
    "arracimas": "arracimar",
    "arracimase": "arracimar",
    "arracimaseis": "arracimar",
    "arracimasen": "arracimar",
    "arracimases": "arracimar",
    "arracimaste": "arracimar",
    "arracimasteis": "arracimar",
    "arracime": "arracimar",
    "arracimemos": "arracimar",
    "arracimen": "arracimar",
    "arracimes": "arracimar",
    "arracimo": "arracimar",
    "arracimábamos": "arracimar",
    "arracimáis": "arracimar",
    "arracimáramos": "arracimar",
    "arracimáremos": "arracimar",
    "arracimásemos": "arracimar",
    "arracimé": "arracimar",
    "arraciméis": "arracimar",
    "arracimó": "arracimar",
    "arraclanes": "arraclán",
    "arraiga": "arraigar",
    "arraigaba": "arraigar",
    "arraigabais": "arraigar",
    "arraigaban": "arraigar",
    "arraigabas": "arraigar",
    "arraigad": "arraigar",
    "arraigada": "arraigar",
    "arraigadas": "arraigar",
    "arraigado": "arraigar",
    "arraigados": "arraigar",
    "arraigamos": "arraigar",
    "arraigan": "arraigar",
    "arraigando": "arraigar",
    "arraigara": "arraigar",
    "arraigarais": "arraigar",
    "arraigaran": "arraigar",
    "arraigaras": "arraigar",
    "arraigare": "arraigar",
    "arraigareis": "arraigar",
    "arraigaremos": "arraigar",
    "arraigaren": "arraigar",
    "arraigares": "arraigar",
    "arraigaron": "arraigar",
    "arraigará": "arraigar",
    "arraigarán": "arraigar",
    "arraigarás": "arraigar",
    "arraigaré": "arraigar",
    "arraigaréis": "arraigar",
    "arraigaría": "arraigar",
    "arraigaríais": "arraigar",
    "arraigaríamos": "arraigar",
    "arraigarían": "arraigar",
    "arraigarías": "arraigar",
    "arraigas": "arraigar",
    "arraigase": "arraigar",
    "arraigaseis": "arraigar",
    "arraigasen": "arraigar",
    "arraigases": "arraigar",
    "arraigaste": "arraigar",
    "arraigasteis": "arraigar",
    "arraigo": "arraigar",
    "arraigos": "arraigo",
    "arraigue": "arraigar",
    "arraiguemos": "arraigar",
    "arraiguen": "arraigar",
    "arraigues": "arraigar",
    "arraigué": "arraigar",
    "arraiguéis": "arraigar",
    "arraigábamos": "arraigar",
    "arraigáis": "arraigar",
    "arraigáramos": "arraigar",
    "arraigáremos": "arraigar",
    "arraigásemos": "arraigar",
    "arraigó": "arraigar",
    "arrakes": "arrak",
    "arrambla": "arramblar",
    "arramblaba": "arramblar",
    "arramblabais": "arramblar",
    "arramblaban": "arramblar",
    "arramblabas": "arramblar",
    "arramblad": "arramblar",
    "arramblada": "arramblar",
    "arrambladas": "arramblar",
    "arramblado": "arramblar",
    "arramblados": "arramblar",
    "arramblamos": "arramblar",
    "arramblan": "arramblar",
    "arramblando": "arramblar",
    "arramblara": "arramblar",
    "arramblarais": "arramblar",
    "arramblaran": "arramblar",
    "arramblaras": "arramblar",
    "arramblare": "arramblar",
    "arramblareis": "arramblar",
    "arramblaremos": "arramblar",
    "arramblaren": "arramblar",
    "arramblares": "arramblar",
    "arramblaron": "arramblar",
    "arramblará": "arramblar",
    "arramblarán": "arramblar",
    "arramblarás": "arramblar",
    "arramblaré": "arramblar",
    "arramblaréis": "arramblar",
    "arramblaría": "arramblar",
    "arramblaríais": "arramblar",
    "arramblaríamos": "arramblar",
    "arramblarían": "arramblar",
    "arramblarías": "arramblar",
    "arramblas": "arramblar",
    "arramblase": "arramblar",
    "arramblaseis": "arramblar",
    "arramblasen": "arramblar",
    "arramblases": "arramblar",
    "arramblaste": "arramblar",
    "arramblasteis": "arramblar",
    "arramble": "arramblar",
    "arramblemos": "arramblar",
    "arramblen": "arramblar",
    "arrambles": "arramblar",
    "arramblo": "arramblar",
    "arramblábamos": "arramblar",
    "arrambláis": "arramblar",
    "arrambláramos": "arramblar",
    "arrambláremos": "arramblar",
    "arramblásemos": "arramblar",
    "arramblé": "arramblar",
    "arrambléis": "arramblar",
    "arrambló": "arramblar",
    "arrampla": "arramplar",
    "arramplaba": "arramplar",
    "arramplabais": "arramplar",
    "arramplaban": "arramplar",
    "arramplabas": "arramplar",
    "arramplad": "arramplar",
    "arramplada": "arramplar",
    "arrampladas": "arramplar",
    "arramplado": "arramplar",
    "arramplados": "arramplar",
    "arramplamos": "arramplar",
    "arramplan": "arramplar",
    "arramplando": "arramplar",
    "arramplara": "arramplar",
    "arramplarais": "arramplar",
    "arramplaran": "arramplar",
    "arramplaras": "arramplar",
    "arramplare": "arramplar",
    "arramplareis": "arramplar",
    "arramplaremos": "arramplar",
    "arramplaren": "arramplar",
    "arramplares": "arramplar",
    "arramplaron": "arramplar",
    "arramplará": "arramplar",
    "arramplarán": "arramplar",
    "arramplarás": "arramplar",
    "arramplaré": "arramplar",
    "arramplaréis": "arramplar",
    "arramplaría": "arramplar",
    "arramplaríais": "arramplar",
    "arramplaríamos": "arramplar",
    "arramplarían": "arramplar",
    "arramplarías": "arramplar",
    "arramplas": "arramplar",
    "arramplase": "arramplar",
    "arramplaseis": "arramplar",
    "arramplasen": "arramplar",
    "arramplases": "arramplar",
    "arramplaste": "arramplar",
    "arramplasteis": "arramplar",
    "arrample": "arramplar",
    "arramplemos": "arramplar",
    "arramplen": "arramplar",
    "arramples": "arramplar",
    "arramplo": "arramplar",
    "arramplábamos": "arramplar",
    "arrampláis": "arramplar",
    "arrampláramos": "arramplar",
    "arrampláremos": "arramplar",
    "arramplásemos": "arramplar",
    "arramplé": "arramplar",
    "arrampléis": "arramplar",
    "arrampló": "arramplar",
    "arranca": "arrancar",
    "arrancaba": "arrancar",
    "arrancabais": "arrancar",
    "arrancaban": "arrancar",
    "arrancabas": "arrancar",
    "arrancad": "arrancar",
    "arrancada": "arrancar",
    "arrancadas": "arrancar",
    "arrancaderas": "arrancadera",
    "arrancado": "arrancar",
    "arrancadora": "arrancador",
    "arrancadoras": "arrancadora",
    "arrancadores": "arrancador",
    "arrancados": "arrancar",
    "arrancaduras": "arrancadura",
    "arrancamientos": "arrancamiento",
    "arrancamos": "arrancar",
    "arrancan": "arrancar",
    "arrancando": "arrancar",
    "arrancara": "arrancar",
    "arrancarais": "arrancar",
    "arrancaran": "arrancar",
    "arrancaras": "arrancar",
    "arrancare": "arrancar",
    "arrancareis": "arrancar",
    "arrancaremos": "arrancar",
    "arrancaren": "arrancar",
    "arrancares": "arrancar",
    "arrancaron": "arrancar",
    "arrancará": "arrancar",
    "arrancarán": "arrancar",
    "arrancarás": "arrancar",
    "arrancaré": "arrancar",
    "arrancaréis": "arrancar",
    "arrancaría": "arrancar",
    "arrancaríais": "arrancar",
    "arrancaríamos": "arrancar",
    "arrancarían": "arrancar",
    "arrancarías": "arrancar",
    "arrancas": "arrancar",
    "arrancase": "arrancar",
    "arrancaseis": "arrancar",
    "arrancasen": "arrancar",
    "arrancases": "arrancar",
    "arrancasiegas": "arrancasiega",
    "arrancaste": "arrancar",
    "arrancasteis": "arrancar",
    "arrancha": "arranchar",
    "arranchaba": "arranchar",
    "arranchabais": "arranchar",
    "arranchaban": "arranchar",
    "arranchabas": "arranchar",
    "arranchad": "arranchar",
    "arranchada": "arranchar",
    "arranchadas": "arranchar",
    "arranchado": "arranchar",
    "arranchados": "arranchar",
    "arranchamos": "arranchar",
    "arranchan": "arranchar",
    "arranchando": "arranchar",
    "arranchara": "arranchar",
    "arrancharais": "arranchar",
    "arrancharan": "arranchar",
    "arrancharas": "arranchar",
    "arranchare": "arranchar",
    "arranchareis": "arranchar",
    "arrancharemos": "arranchar",
    "arrancharen": "arranchar",
    "arranchares": "arranchar",
    "arrancharon": "arranchar",
    "arranchará": "arranchar",
    "arrancharán": "arranchar",
    "arrancharás": "arranchar",
    "arrancharé": "arranchar",
    "arrancharéis": "arranchar",
    "arrancharía": "arranchar",
    "arrancharíais": "arranchar",
    "arrancharíamos": "arranchar",
    "arrancharían": "arranchar",
    "arrancharías": "arranchar",
    "arranchas": "arranchar",
    "arranchase": "arranchar",
    "arranchaseis": "arranchar",
    "arranchasen": "arranchar",
    "arranchases": "arranchar",
    "arranchaste": "arranchar",
    "arranchasteis": "arranchar",
    "arranche": "arranchar",
    "arranchemos": "arranchar",
    "arranchen": "arranchar",
    "arranches": "arranchar",
    "arrancho": "arranchar",
    "arranchábamos": "arranchar",
    "arrancháis": "arranchar",
    "arrancháramos": "arranchar",
    "arrancháremos": "arranchar",
    "arranchásemos": "arranchar",
    "arranché": "arranchar",
    "arranchéis": "arranchar",
    "arranchó": "arranchar",
    "arranco": "arrancar",
    "arrancábamos": "arrancar",
    "arrancáis": "arrancar",
    "arrancáramos": "arrancar",
    "arrancáremos": "arrancar",
    "arrancásemos": "arrancar",
    "arrancó": "arrancar",
    "arranque": "arrancar",
    "arranquemos": "arrancar",
    "arranquen": "arrancar",
    "arranqueras": "arranquera",
    "arranques": "arranque",
    "arranqué": "arrancar",
    "arranquéis": "arrancar",
    "arrapiezos": "arrapiezo",
    "arrasa": "arrasar",
    "arrasaba": "arrasar",
    "arrasabais": "arrasar",
    "arrasaban": "arrasar",
    "arrasabas": "arrasar",
    "arrasad": "arrasar",
    "arrasada": "arrasar",
    "arrasadas": "arrasar",
    "arrasado": "arrasar",
    "arrasados": "arrasar",
    "arrasamientos": "arrasamiento",
    "arrasamos": "arrasar",
    "arrasan": "arrasar",
    "arrasando": "arrasar",
    "arrasara": "arrasar",
    "arrasarais": "arrasar",
    "arrasaran": "arrasar",
    "arrasaras": "arrasar",
    "arrasare": "arrasar",
    "arrasareis": "arrasar",
    "arrasaremos": "arrasar",
    "arrasaren": "arrasar",
    "arrasares": "arrasar",
    "arrasaron": "arrasar",
    "arrasará": "arrasar",
    "arrasarán": "arrasar",
    "arrasarás": "arrasar",
    "arrasaré": "arrasar",
    "arrasaréis": "arrasar",
    "arrasaría": "arrasar",
    "arrasaríais": "arrasar",
    "arrasaríamos": "arrasar",
    "arrasarían": "arrasar",
    "arrasarías": "arrasar",
    "arrasas": "arrasar",
    "arrasase": "arrasar",
    "arrasaseis": "arrasar",
    "arrasasen": "arrasar",
    "arrasases": "arrasar",
    "arrasaste": "arrasar",
    "arrasasteis": "arrasar",
    "arrase": "arrasar",
    "arrasemos": "arrasar",
    "arrasen": "arrasar",
    "arrases": "arrasar",
    "arraso": "arrasar",
    "arrastra": "arrastrar",
    "arrastraba": "arrastrar",
    "arrastrabais": "arrastrar",
    "arrastraban": "arrastrar",
    "arrastrabas": "arrastrar",
    "arrastrad": "arrastrar",
    "arrastrada": "arrastrar",
    "arrastradas": "arrastrar",
    "arrastraderas": "arrastradera",
    "arrastraderos": "arrastradero",
    "arrastradiza": "arrastradizo",
    "arrastradizas": "arrastradizo",
    "arrastradizos": "arrastradizo",
    "arrastrado": "arrastrar",
    "arrastrados": "arrastrar",
    "arrastramientos": "arrastramiento",
    "arrastramos": "arrastrar",
    "arrastran": "arrastrar",
    "arrastrando": "arrastrar",
    "arrastrara": "arrastrar",
    "arrastrarais": "arrastrar",
    "arrastraran": "arrastrar",
    "arrastraras": "arrastrar",
    "arrastrare": "arrastrar",
    "arrastrareis": "arrastrar",
    "arrastraremos": "arrastrar",
    "arrastraren": "arrastrar",
    "arrastrares": "arrastrar",
    "arrastraron": "arrastrar",
    "arrastrará": "arrastrar",
    "arrastrarán": "arrastrar",
    "arrastrarás": "arrastrar",
    "arrastraré": "arrastrar",
    "arrastraréis": "arrastrar",
    "arrastraría": "arrastrar",
    "arrastraríais": "arrastrar",
    "arrastraríamos": "arrastrar",
    "arrastrarían": "arrastrar",
    "arrastrarías": "arrastrar",
    "arrastras": "arrastrar",
    "arrastrase": "arrastrar",
    "arrastraseis": "arrastrar",
    "arrastrasen": "arrastrar",
    "arrastrases": "arrastrar",
    "arrastraste": "arrastrar",
    "arrastrasteis": "arrastrar",
    "arrastre": "arrastrar",
    "arrastremos": "arrastrar",
    "arrastren": "arrastrar",
    "arrastrera": "arrastrero",
    "arrastreras": "arrastrero",
    "arrastreros": "arrastrero",
    "arrastres": "arrastre",
    "arrastro": "arrastrar",
    "arrastrábamos": "arrastrar",
    "arrastráis": "arrastrar",
    "arrastráramos": "arrastrar",
    "arrastráremos": "arrastrar",
    "arrastrásemos": "arrastrar",
    "arrastré": "arrastrar",
    "arrastréis": "arrastrar",
    "arrastró": "arrastrar",
    "arrasábamos": "arrasar",
    "arrasáis": "arrasar",
    "arrasáramos": "arrasar",
    "arrasáremos": "arrasar",
    "arrasásemos": "arrasar",
    "arrasé": "arrasar",
    "arraséis": "arrasar",
    "arrasó": "arrasar",
    "arrates": "arrate",
    "arratonada": "arratonado",
    "arratonadas": "arratonado",
    "arratonados": "arratonado",
    "arrayanales": "arrayanal",
    "arrayanes": "arrayán",
    "arrea": "arrear",
    "arreaba": "arrear",
    "arreabais": "arrear",
    "arreaban": "arrear",
    "arreabas": "arrear",
    "arread": "arrear",
    "arreada": "arrear",
    "arreadas": "arrear",
    "arreado": "arrear",
    "arreados": "arrear",
    "arreamos": "arrear",
    "arrean": "arrear",
    "arreando": "arrear",
    "arreara": "arrear",
    "arrearais": "arrear",
    "arrearan": "arrear",
    "arrearas": "arrear",
    "arreare": "arrear",
    "arreareis": "arrear",
    "arrearemos": "arrear",
    "arrearen": "arrear",
    "arreares": "arrear",
    "arrearon": "arrear",
    "arreará": "arrear",
    "arrearán": "arrear",
    "arrearás": "arrear",
    "arrearé": "arrear",
    "arrearéis": "arrear",
    "arrearía": "arrear",
    "arrearíais": "arrear",
    "arrearíamos": "arrear",
    "arrearían": "arrear",
    "arrearías": "arrear",
    "arreas": "arrear",
    "arrease": "arrear",
    "arreaseis": "arrear",
    "arreasen": "arrear",
    "arreases": "arrear",
    "arreaste": "arrear",
    "arreasteis": "arrear",
    "arrebata": "arrebatar",
    "arrebataba": "arrebatar",
    "arrebatabais": "arrebatar",
    "arrebataban": "arrebatar",
    "arrebatabas": "arrebatar",
    "arrebatad": "arrebatar",
    "arrebatada": "arrebatar",
    "arrebatadas": "arrebatar",
    "arrebatadiza": "arrebatadizo",
    "arrebatadizas": "arrebatadizo",
    "arrebatadizos": "arrebatadizo",
    "arrebatado": "arrebatar",
    "arrebatadora": "arrebatador",
    "arrebatadoras": "arrebatador",
    "arrebatadores": "arrebatador",
    "arrebatados": "arrebatar",
    "arrebatamientos": "arrebatamiento",
    "arrebatamos": "arrebatar",
    "arrebatan": "arrebatar",
    "arrebatando": "arrebatar",
    "arrebatara": "arrebatar",
    "arrebatarais": "arrebatar",
    "arrebataran": "arrebatar",
    "arrebataras": "arrebatar",
    "arrebatare": "arrebatar",
    "arrebatareis": "arrebatar",
    "arrebataremos": "arrebatar",
    "arrebataren": "arrebatar",
    "arrebatares": "arrebatar",
    "arrebataron": "arrebatar",
    "arrebatará": "arrebatar",
    "arrebatarán": "arrebatar",
    "arrebatarás": "arrebatar",
    "arrebataré": "arrebatar",
    "arrebataréis": "arrebatar",
    "arrebataría": "arrebatar",
    "arrebataríais": "arrebatar",
    "arrebataríamos": "arrebatar",
    "arrebatarían": "arrebatar",
    "arrebatarías": "arrebatar",
    "arrebatas": "arrebatar",
    "arrebatase": "arrebatar",
    "arrebataseis": "arrebatar",
    "arrebatasen": "arrebatar",
    "arrebatases": "arrebatar",
    "arrebataste": "arrebatar",
    "arrebatasteis": "arrebatar",
    "arrebate": "arrebatar",
    "arrebatemos": "arrebatar",
    "arrebaten": "arrebatar",
    "arrebates": "arrebatar",
    "arrebatiñas": "arrebatiña",
    "arrebato": "arrebatar",
    "arrebatos": "arrebato",
    "arrebatábamos": "arrebatar",
    "arrebatáis": "arrebatar",
    "arrebatáramos": "arrebatar",
    "arrebatáremos": "arrebatar",
    "arrebatásemos": "arrebatar",
    "arrebaté": "arrebatar",
    "arrebatéis": "arrebatar",
    "arrebató": "arrebatar",
    "arrebaña": "arrebañar",
    "arrebañaba": "arrebañar",
    "arrebañabais": "arrebañar",
    "arrebañaban": "arrebañar",
    "arrebañabas": "arrebañar",
    "arrebañad": "arrebañar",
    "arrebañada": "arrebañar",
    "arrebañadas": "arrebañar",
    "arrebañado": "arrebañar",
    "arrebañados": "arrebañar",
    "arrebañamos": "arrebañar",
    "arrebañan": "arrebañar",
    "arrebañando": "arrebañar",
    "arrebañara": "arrebañar",
    "arrebañarais": "arrebañar",
    "arrebañaran": "arrebañar",
    "arrebañaras": "arrebañar",
    "arrebañare": "arrebañar",
    "arrebañareis": "arrebañar",
    "arrebañaremos": "arrebañar",
    "arrebañaren": "arrebañar",
    "arrebañares": "arrebañar",
    "arrebañaron": "arrebañar",
    "arrebañará": "arrebañar",
    "arrebañarán": "arrebañar",
    "arrebañarás": "arrebañar",
    "arrebañaré": "arrebañar",
    "arrebañaréis": "arrebañar",
    "arrebañaría": "arrebañar",
    "arrebañaríais": "arrebañar",
    "arrebañaríamos": "arrebañar",
    "arrebañarían": "arrebañar",
    "arrebañarías": "arrebañar",
    "arrebañas": "arrebañar",
    "arrebañase": "arrebañar",
    "arrebañaseis": "arrebañar",
    "arrebañasen": "arrebañar",
    "arrebañases": "arrebañar",
    "arrebañaste": "arrebañar",
    "arrebañasteis": "arrebañar",
    "arrebañe": "arrebañar",
    "arrebañemos": "arrebañar",
    "arrebañen": "arrebañar",
    "arrebañes": "arrebañar",
    "arrebaño": "arrebañar",
    "arrebañábamos": "arrebañar",
    "arrebañáis": "arrebañar",
    "arrebañáramos": "arrebañar",
    "arrebañáremos": "arrebañar",
    "arrebañásemos": "arrebañar",
    "arrebañé": "arrebañar",
    "arrebañéis": "arrebañar",
    "arrebañó": "arrebañar",
    "arreboce": "arrebozar",
    "arrebocemos": "arrebozar",
    "arrebocen": "arrebozar",
    "arreboces": "arrebozar",
    "arrebocé": "arrebozar",
    "arrebocéis": "arrebozar",
    "arrebola": "arrebolar",
    "arrebolaba": "arrebolar",
    "arrebolabais": "arrebolar",
    "arrebolaban": "arrebolar",
    "arrebolabas": "arrebolar",
    "arrebolad": "arrebolar",
    "arrebolada": "arrebolar",
    "arreboladas": "arrebolar",
    "arrebolado": "arrebolar",
    "arrebolados": "arrebolar",
    "arrebolamos": "arrebolar",
    "arrebolan": "arrebolar",
    "arrebolando": "arrebolar",
    "arrebolara": "arrebolar",
    "arrebolarais": "arrebolar",
    "arrebolaran": "arrebolar",
    "arrebolaras": "arrebolar",
    "arrebolare": "arrebolar",
    "arrebolareis": "arrebolar",
    "arrebolaremos": "arrebolar",
    "arrebolaren": "arrebolar",
    "arrebolares": "arrebolar",
    "arrebolaron": "arrebolar",
    "arrebolará": "arrebolar",
    "arrebolarán": "arrebolar",
    "arrebolarás": "arrebolar",
    "arrebolaré": "arrebolar",
    "arrebolaréis": "arrebolar",
    "arrebolaría": "arrebolar",
    "arrebolaríais": "arrebolar",
    "arrebolaríamos": "arrebolar",
    "arrebolarían": "arrebolar",
    "arrebolarías": "arrebolar",
    "arrebolas": "arrebolar",
    "arrebolase": "arrebolar",
    "arrebolaseis": "arrebolar",
    "arrebolasen": "arrebolar",
    "arrebolases": "arrebolar",
    "arrebolaste": "arrebolar",
    "arrebolasteis": "arrebolar",
    "arrebole": "arrebolar",
    "arrebolemos": "arrebolar",
    "arrebolen": "arrebolar",
    "arreboles": "arrebolar",
    "arrebolo": "arrebolar",
    "arrebolábamos": "arrebolar",
    "arreboláis": "arrebolar",
    "arreboláramos": "arrebolar",
    "arreboláremos": "arrebolar",
    "arrebolásemos": "arrebolar",
    "arrebolé": "arrebolar",
    "arreboléis": "arrebolar",
    "arreboló": "arrebolar",
    "arreboza": "arrebozar",
    "arrebozaba": "arrebozar",
    "arrebozabais": "arrebozar",
    "arrebozaban": "arrebozar",
    "arrebozabas": "arrebozar",
    "arrebozad": "arrebozar",
    "arrebozada": "arrebozar",
    "arrebozadas": "arrebozar",
    "arrebozado": "arrebozar",
    "arrebozados": "arrebozar",
    "arrebozamos": "arrebozar",
    "arrebozan": "arrebozar",
    "arrebozando": "arrebozar",
    "arrebozara": "arrebozar",
    "arrebozarais": "arrebozar",
    "arrebozaran": "arrebozar",
    "arrebozaras": "arrebozar",
    "arrebozare": "arrebozar",
    "arrebozareis": "arrebozar",
    "arrebozaremos": "arrebozar",
    "arrebozaren": "arrebozar",
    "arrebozares": "arrebozar",
    "arrebozaron": "arrebozar",
    "arrebozará": "arrebozar",
    "arrebozarán": "arrebozar",
    "arrebozarás": "arrebozar",
    "arrebozaré": "arrebozar",
    "arrebozaréis": "arrebozar",
    "arrebozaría": "arrebozar",
    "arrebozaríais": "arrebozar",
    "arrebozaríamos": "arrebozar",
    "arrebozarían": "arrebozar",
    "arrebozarías": "arrebozar",
    "arrebozas": "arrebozar",
    "arrebozase": "arrebozar",
    "arrebozaseis": "arrebozar",
    "arrebozasen": "arrebozar",
    "arrebozases": "arrebozar",
    "arrebozaste": "arrebozar",
    "arrebozasteis": "arrebozar",
    "arrebozo": "arrebozar",
    "arrebozos": "arrebozo",
    "arrebozábamos": "arrebozar",
    "arrebozáis": "arrebozar",
    "arrebozáramos": "arrebozar",
    "arrebozáremos": "arrebozar",
    "arrebozásemos": "arrebozar",
    "arrebozó": "arrebozar",
    "arrebuja": "arrebujar",
    "arrebujaba": "arrebujar",
    "arrebujabais": "arrebujar",
    "arrebujaban": "arrebujar",
    "arrebujabas": "arrebujar",
    "arrebujad": "arrebujar",
    "arrebujada": "arrebujar",
    "arrebujadas": "arrebujar",
    "arrebujado": "arrebujar",
    "arrebujados": "arrebujar",
    "arrebujamos": "arrebujar",
    "arrebujan": "arrebujar",
    "arrebujando": "arrebujar",
    "arrebujara": "arrebujar",
    "arrebujarais": "arrebujar",
    "arrebujaran": "arrebujar",
    "arrebujaras": "arrebujar",
    "arrebujare": "arrebujar",
    "arrebujareis": "arrebujar",
    "arrebujaremos": "arrebujar",
    "arrebujaren": "arrebujar",
    "arrebujares": "arrebujar",
    "arrebujaron": "arrebujar",
    "arrebujará": "arrebujar",
    "arrebujarán": "arrebujar",
    "arrebujarás": "arrebujar",
    "arrebujaré": "arrebujar",
    "arrebujaréis": "arrebujar",
    "arrebujaría": "arrebujar",
    "arrebujaríais": "arrebujar",
    "arrebujaríamos": "arrebujar",
    "arrebujarían": "arrebujar",
    "arrebujarías": "arrebujar",
    "arrebujas": "arrebujar",
    "arrebujase": "arrebujar",
    "arrebujaseis": "arrebujar",
    "arrebujasen": "arrebujar",
    "arrebujases": "arrebujar",
    "arrebujaste": "arrebujar",
    "arrebujasteis": "arrebujar",
    "arrebuje": "arrebujar",
    "arrebujemos": "arrebujar",
    "arrebujen": "arrebujar",
    "arrebujes": "arrebujar",
    "arrebujo": "arrebujar",
    "arrebujábamos": "arrebujar",
    "arrebujáis": "arrebujar",
    "arrebujáramos": "arrebujar",
    "arrebujáremos": "arrebujar",
    "arrebujásemos": "arrebujar",
    "arrebujé": "arrebujar",
    "arrebujéis": "arrebujar",
    "arrebujó": "arrebujar",
    "arrechuchos": "arrechucho",
    "arrecia": "arreciar",
    "arreciaba": "arreciar",
    "arreciabais": "arreciar",
    "arreciaban": "arreciar",
    "arreciabas": "arreciar",
    "arreciad": "arreciar",
    "arreciada": "arreciar",
    "arreciadas": "arreciar",
    "arreciado": "arreciar",
    "arreciados": "arreciar",
    "arreciamos": "arreciar",
    "arrecian": "arreciar",
    "arreciando": "arreciar",
    "arreciara": "arreciar",
    "arreciarais": "arreciar",
    "arreciaran": "arreciar",
    "arreciaras": "arreciar",
    "arreciare": "arreciar",
    "arreciareis": "arreciar",
    "arreciaremos": "arreciar",
    "arreciaren": "arreciar",
    "arreciares": "arreciar",
    "arreciaron": "arreciar",
    "arreciará": "arreciar",
    "arreciarán": "arreciar",
    "arreciarás": "arreciar",
    "arreciaré": "arreciar",
    "arreciaréis": "arreciar",
    "arreciaría": "arreciar",
    "arreciaríais": "arreciar",
    "arreciaríamos": "arreciar",
    "arreciarían": "arreciar",
    "arreciarías": "arreciar",
    "arrecias": "arreciar",
    "arreciase": "arreciar",
    "arreciaseis": "arreciar",
    "arreciasen": "arreciar",
    "arreciases": "arreciar",
    "arreciaste": "arreciar",
    "arreciasteis": "arreciar",
    "arrecie": "arreciar",
    "arreciemos": "arreciar",
    "arrecien": "arreciar",
    "arrecies": "arreciar",
    "arrecifes": "arrecife",
    "arrecio": "arreciar",
    "arreciábamos": "arreciar",
    "arreciáis": "arreciar",
    "arreciáramos": "arreciar",
    "arreciáremos": "arreciar",
    "arreciásemos": "arreciar",
    "arrecié": "arreciar",
    "arreciéis": "arreciar",
    "arreció": "arreciar",
    "arredila": "arredilar",
    "arredilaba": "arredilar",
    "arredilabais": "arredilar",
    "arredilaban": "arredilar",
    "arredilabas": "arredilar",
    "arredilad": "arredilar",
    "arredilada": "arredilar",
    "arrediladas": "arredilar",
    "arredilado": "arredilar",
    "arredilados": "arredilar",
    "arredilamos": "arredilar",
    "arredilan": "arredilar",
    "arredilando": "arredilar",
    "arredilara": "arredilar",
    "arredilarais": "arredilar",
    "arredilaran": "arredilar",
    "arredilaras": "arredilar",
    "arredilare": "arredilar",
    "arredilareis": "arredilar",
    "arredilaremos": "arredilar",
    "arredilaren": "arredilar",
    "arredilares": "arredilar",
    "arredilaron": "arredilar",
    "arredilará": "arredilar",
    "arredilarán": "arredilar",
    "arredilarás": "arredilar",
    "arredilaré": "arredilar",
    "arredilaréis": "arredilar",
    "arredilaría": "arredilar",
    "arredilaríais": "arredilar",
    "arredilaríamos": "arredilar",
    "arredilarían": "arredilar",
    "arredilarías": "arredilar",
    "arredilas": "arredilar",
    "arredilase": "arredilar",
    "arredilaseis": "arredilar",
    "arredilasen": "arredilar",
    "arredilases": "arredilar",
    "arredilaste": "arredilar",
    "arredilasteis": "arredilar",
    "arredile": "arredilar",
    "arredilemos": "arredilar",
    "arredilen": "arredilar",
    "arrediles": "arredilar",
    "arredilo": "arredilar",
    "arredilábamos": "arredilar",
    "arrediláis": "arredilar",
    "arrediláramos": "arredilar",
    "arrediláremos": "arredilar",
    "arredilásemos": "arredilar",
    "arredilé": "arredilar",
    "arrediléis": "arredilar",
    "arrediló": "arredilar",
    "arredra": "arredrar",
    "arredraba": "arredrar",
    "arredrabais": "arredrar",
    "arredraban": "arredrar",
    "arredrabas": "arredrar",
    "arredrad": "arredrar",
    "arredrada": "arredrar",
    "arredradas": "arredrar",
    "arredrado": "arredrar",
    "arredrados": "arredrar",
    "arredramientos": "arredramiento",
    "arredramos": "arredrar",
    "arredran": "arredrar",
    "arredrando": "arredrar",
    "arredrara": "arredrar",
    "arredrarais": "arredrar",
    "arredraran": "arredrar",
    "arredraras": "arredrar",
    "arredrare": "arredrar",
    "arredrareis": "arredrar",
    "arredraremos": "arredrar",
    "arredraren": "arredrar",
    "arredrares": "arredrar",
    "arredraron": "arredrar",
    "arredrará": "arredrar",
    "arredrarán": "arredrar",
    "arredrarás": "arredrar",
    "arredraré": "arredrar",
    "arredraréis": "arredrar",
    "arredraría": "arredrar",
    "arredraríais": "arredrar",
    "arredraríamos": "arredrar",
    "arredrarían": "arredrar",
    "arredrarías": "arredrar",
    "arredras": "arredrar",
    "arredrase": "arredrar",
    "arredraseis": "arredrar",
    "arredrasen": "arredrar",
    "arredrases": "arredrar",
    "arredraste": "arredrar",
    "arredrasteis": "arredrar",
    "arredre": "arredrar",
    "arredremos": "arredrar",
    "arredren": "arredrar",
    "arredres": "arredrar",
    "arredro": "arredrar",
    "arredrábamos": "arredrar",
    "arredráis": "arredrar",
    "arredráramos": "arredrar",
    "arredráremos": "arredrar",
    "arredrásemos": "arredrar",
    "arredré": "arredrar",
    "arredréis": "arredrar",
    "arredró": "arredrar",
    "arree": "arrear",
    "arreemos": "arrear",
    "arreen": "arrear",
    "arrees": "arrear",
    "arreflexias": "arreflexia",
    "arregace": "arregazar",
    "arregacemos": "arregazar",
    "arregacen": "arregazar",
    "arregaces": "arregazar",
    "arregacé": "arregazar",
    "arregacéis": "arregazar",
    "arregaza": "arregazar",
    "arregazaba": "arregazar",
    "arregazabais": "arregazar",
    "arregazaban": "arregazar",
    "arregazabas": "arregazar",
    "arregazad": "arregazar",
    "arregazada": "arregazar",
    "arregazadas": "arregazar",
    "arregazado": "arregazar",
    "arregazados": "arregazar",
    "arregazamos": "arregazar",
    "arregazan": "arregazar",
    "arregazando": "arregazar",
    "arregazara": "arregazar",
    "arregazarais": "arregazar",
    "arregazaran": "arregazar",
    "arregazaras": "arregazar",
    "arregazare": "arregazar",
    "arregazareis": "arregazar",
    "arregazaremos": "arregazar",
    "arregazaren": "arregazar",
    "arregazares": "arregazar",
    "arregazaron": "arregazar",
    "arregazará": "arregazar",
    "arregazarán": "arregazar",
    "arregazarás": "arregazar",
    "arregazaré": "arregazar",
    "arregazaréis": "arregazar",
    "arregazaría": "arregazar",
    "arregazaríais": "arregazar",
    "arregazaríamos": "arregazar",
    "arregazarían": "arregazar",
    "arregazarías": "arregazar",
    "arregazas": "arregazar",
    "arregazase": "arregazar",
    "arregazaseis": "arregazar",
    "arregazasen": "arregazar",
    "arregazases": "arregazar",
    "arregazaste": "arregazar",
    "arregazasteis": "arregazar",
    "arregazo": "arregazar",
    "arregazábamos": "arregazar",
    "arregazáis": "arregazar",
    "arregazáramos": "arregazar",
    "arregazáremos": "arregazar",
    "arregazásemos": "arregazar",
    "arregazó": "arregazar",
    "arregla": "arreglar",
    "arreglaba": "arreglar",
    "arreglabais": "arreglar",
    "arreglaban": "arreglar",
    "arreglabas": "arreglar",
    "arreglad": "arreglar",
    "arreglada": "arreglar",
    "arregladas": "arreglar",
    "arreglado": "arreglar",
    "arregladora": "arreglador",
    "arregladoras": "arreglador",
    "arregladores": "arreglador",
    "arreglados": "arreglar",
    "arreglamos": "arreglar",
    "arreglan": "arreglar",
    "arreglando": "arreglar",
    "arreglara": "arreglar",
    "arreglarais": "arreglar",
    "arreglaran": "arreglar",
    "arreglaras": "arreglar",
    "arreglare": "arreglar",
    "arreglareis": "arreglar",
    "arreglaremos": "arreglar",
    "arreglaren": "arreglar",
    "arreglares": "arreglar",
    "arreglaron": "arreglar",
    "arreglará": "arreglar",
    "arreglarán": "arreglar",
    "arreglarás": "arreglar",
    "arreglaré": "arreglar",
    "arreglaréis": "arreglar",
    "arreglaría": "arreglar",
    "arreglaríais": "arreglar",
    "arreglaríamos": "arreglar",
    "arreglarían": "arreglar",
    "arreglarías": "arreglar",
    "arreglas": "arreglar",
    "arreglase": "arreglar",
    "arreglaseis": "arreglar",
    "arreglasen": "arreglar",
    "arreglases": "arreglar",
    "arreglaste": "arreglar",
    "arreglasteis": "arreglar",
    "arregle": "arreglar",
    "arreglemos": "arreglar",
    "arreglen": "arreglar",
    "arregles": "arreglar",
    "arreglistas": "arreglista",
    "arreglo": "arreglar",
    "arreglos": "arreglo",
    "arreglábamos": "arreglar",
    "arregláis": "arreglar",
    "arregláramos": "arreglar",
    "arregláremos": "arreglar",
    "arreglásemos": "arreglar",
    "arreglé": "arreglar",
    "arregléis": "arreglar",
    "arregló": "arreglar",
    "arregosta": "arregostar",
    "arregostaba": "arregostar",
    "arregostabais": "arregostar",
    "arregostaban": "arregostar",
    "arregostabas": "arregostar",
    "arregostad": "arregostar",
    "arregostada": "arregostar",
    "arregostadas": "arregostar",
    "arregostado": "arregostar",
    "arregostados": "arregostar",
    "arregostamos": "arregostar",
    "arregostan": "arregostar",
    "arregostando": "arregostar",
    "arregostara": "arregostar",
    "arregostarais": "arregostar",
    "arregostaran": "arregostar",
    "arregostaras": "arregostar",
    "arregostare": "arregostar",
    "arregostareis": "arregostar",
    "arregostaremos": "arregostar",
    "arregostaren": "arregostar",
    "arregostares": "arregostar",
    "arregostaron": "arregostar",
    "arregostará": "arregostar",
    "arregostarán": "arregostar",
    "arregostarás": "arregostar",
    "arregostaré": "arregostar",
    "arregostaréis": "arregostar",
    "arregostaría": "arregostar",
    "arregostaríais": "arregostar",
    "arregostaríamos": "arregostar",
    "arregostarían": "arregostar",
    "arregostarías": "arregostar",
    "arregostas": "arregostar",
    "arregostase": "arregostar",
    "arregostaseis": "arregostar",
    "arregostasen": "arregostar",
    "arregostases": "arregostar",
    "arregostaste": "arregostar",
    "arregostasteis": "arregostar",
    "arregoste": "arregostar",
    "arregostemos": "arregostar",
    "arregosten": "arregostar",
    "arregostes": "arregostar",
    "arregosto": "arregostar",
    "arregostos": "arregosto",
    "arregostábamos": "arregostar",
    "arregostáis": "arregostar",
    "arregostáramos": "arregostar",
    "arregostáremos": "arregostar",
    "arregostásemos": "arregostar",
    "arregosté": "arregostar",
    "arregostéis": "arregostar",
    "arregostó": "arregostar",
    "arreica": "arreico",
    "arreicas": "arreico",
    "arreicos": "arreico",
    "arrejaca": "arrejacar",
    "arrejacaba": "arrejacar",
    "arrejacabais": "arrejacar",
    "arrejacaban": "arrejacar",
    "arrejacabas": "arrejacar",
    "arrejacad": "arrejacar",
    "arrejacada": "arrejacar",
    "arrejacadas": "arrejacar",
    "arrejacado": "arrejacar",
    "arrejacados": "arrejacar",
    "arrejacamos": "arrejacar",
    "arrejacan": "arrejacar",
    "arrejacando": "arrejacar",
    "arrejacara": "arrejacar",
    "arrejacarais": "arrejacar",
    "arrejacaran": "arrejacar",
    "arrejacaras": "arrejacar",
    "arrejacare": "arrejacar",
    "arrejacareis": "arrejacar",
    "arrejacaremos": "arrejacar",
    "arrejacaren": "arrejacar",
    "arrejacares": "arrejacar",
    "arrejacaron": "arrejacar",
    "arrejacará": "arrejacar",
    "arrejacarán": "arrejacar",
    "arrejacarás": "arrejacar",
    "arrejacaré": "arrejacar",
    "arrejacaréis": "arrejacar",
    "arrejacaría": "arrejacar",
    "arrejacaríais": "arrejacar",
    "arrejacaríamos": "arrejacar",
    "arrejacarían": "arrejacar",
    "arrejacarías": "arrejacar",
    "arrejacas": "arrejacar",
    "arrejacase": "arrejacar",
    "arrejacaseis": "arrejacar",
    "arrejacasen": "arrejacar",
    "arrejacases": "arrejacar",
    "arrejacaste": "arrejacar",
    "arrejacasteis": "arrejacar",
    "arrejaco": "arrejacar",
    "arrejacábamos": "arrejacar",
    "arrejacáis": "arrejacar",
    "arrejacáramos": "arrejacar",
    "arrejacáremos": "arrejacar",
    "arrejacásemos": "arrejacar",
    "arrejacó": "arrejacar",
    "arrejadas": "arrejada",
    "arrejaque": "arrejacar",
    "arrejaquemos": "arrejacar",
    "arrejaquen": "arrejacar",
    "arrejaques": "arrejaque",
    "arrejaqué": "arrejacar",
    "arrejaquéis": "arrejacar",
    "arrejera": "arrejerar",
    "arrejeraba": "arrejerar",
    "arrejerabais": "arrejerar",
    "arrejeraban": "arrejerar",
    "arrejerabas": "arrejerar",
    "arrejerad": "arrejerar",
    "arrejerada": "arrejerar",
    "arrejeradas": "arrejerar",
    "arrejerado": "arrejerar",
    "arrejerados": "arrejerar",
    "arrejeramos": "arrejerar",
    "arrejeran": "arrejerar",
    "arrejerando": "arrejerar",
    "arrejerara": "arrejerar",
    "arrejerarais": "arrejerar",
    "arrejeraran": "arrejerar",
    "arrejeraras": "arrejerar",
    "arrejerare": "arrejerar",
    "arrejerareis": "arrejerar",
    "arrejeraremos": "arrejerar",
    "arrejeraren": "arrejerar",
    "arrejerares": "arrejerar",
    "arrejeraron": "arrejerar",
    "arrejerará": "arrejerar",
    "arrejerarán": "arrejerar",
    "arrejerarás": "arrejerar",
    "arrejeraré": "arrejerar",
    "arrejeraréis": "arrejerar",
    "arrejeraría": "arrejerar",
    "arrejeraríais": "arrejerar",
    "arrejeraríamos": "arrejerar",
    "arrejerarían": "arrejerar",
    "arrejerarías": "arrejerar",
    "arrejeras": "arrejerar",
    "arrejerase": "arrejerar",
    "arrejeraseis": "arrejerar",
    "arrejerasen": "arrejerar",
    "arrejerases": "arrejerar",
    "arrejeraste": "arrejerar",
    "arrejerasteis": "arrejerar",
    "arrejere": "arrejerar",
    "arrejeremos": "arrejerar",
    "arrejeren": "arrejerar",
    "arrejeres": "arrejerar",
    "arrejero": "arrejerar",
    "arrejerábamos": "arrejerar",
    "arrejeráis": "arrejerar",
    "arrejeráramos": "arrejerar",
    "arrejeráremos": "arrejerar",
    "arrejerásemos": "arrejerar",
    "arrejeré": "arrejerar",
    "arrejeréis": "arrejerar",
    "arrejeró": "arrejerar",
    "arrejunta": "arrejuntar",
    "arrejuntaba": "arrejuntar",
    "arrejuntabais": "arrejuntar",
    "arrejuntaban": "arrejuntar",
    "arrejuntabas": "arrejuntar",
    "arrejuntad": "arrejuntar",
    "arrejuntada": "arrejuntar",
    "arrejuntadas": "arrejuntar",
    "arrejuntado": "arrejuntar",
    "arrejuntados": "arrejuntar",
    "arrejuntamos": "arrejuntar",
    "arrejuntan": "arrejuntar",
    "arrejuntando": "arrejuntar",
    "arrejuntara": "arrejuntar",
    "arrejuntarais": "arrejuntar",
    "arrejuntaran": "arrejuntar",
    "arrejuntaras": "arrejuntar",
    "arrejuntare": "arrejuntar",
    "arrejuntareis": "arrejuntar",
    "arrejuntaremos": "arrejuntar",
    "arrejuntaren": "arrejuntar",
    "arrejuntares": "arrejuntar",
    "arrejuntaron": "arrejuntar",
    "arrejuntará": "arrejuntar",
    "arrejuntarán": "arrejuntar",
    "arrejuntarás": "arrejuntar",
    "arrejuntaré": "arrejuntar",
    "arrejuntaréis": "arrejuntar",
    "arrejuntaría": "arrejuntar",
    "arrejuntaríais": "arrejuntar",
    "arrejuntaríamos": "arrejuntar",
    "arrejuntarían": "arrejuntar",
    "arrejuntarías": "arrejuntar",
    "arrejuntas": "arrejuntar",
    "arrejuntase": "arrejuntar",
    "arrejuntaseis": "arrejuntar",
    "arrejuntasen": "arrejuntar",
    "arrejuntases": "arrejuntar",
    "arrejuntaste": "arrejuntar",
    "arrejuntasteis": "arrejuntar",
    "arrejunte": "arrejuntar",
    "arrejuntemos": "arrejuntar",
    "arrejunten": "arrejuntar",
    "arrejuntes": "arrejuntar",
    "arrejunto": "arrejuntar",
    "arrejuntábamos": "arrejuntar",
    "arrejuntáis": "arrejuntar",
    "arrejuntáramos": "arrejuntar",
    "arrejuntáremos": "arrejuntar",
    "arrejuntásemos": "arrejuntar",
    "arrejunté": "arrejuntar",
    "arrejuntéis": "arrejuntar",
    "arrejuntó": "arrejuntar",
    "arreligiosa": "arreligioso",
    "arreligiosas": "arreligioso",
    "arreligiosidades": "arreligiosidad",
    "arreligiosos": "arreligioso",
    "arrellana": "arrellanar",
    "arrellanaba": "arrellanar",
    "arrellanabais": "arrellanar",
    "arrellanaban": "arrellanar",
    "arrellanabas": "arrellanar",
    "arrellanad": "arrellanar",
    "arrellanada": "arrellanar",
    "arrellanadas": "arrellanar",
    "arrellanado": "arrellanar",
    "arrellanados": "arrellanar",
    "arrellanamos": "arrellanar",
    "arrellanan": "arrellanar",
    "arrellanando": "arrellanar",
    "arrellanara": "arrellanar",
    "arrellanarais": "arrellanar",
    "arrellanaran": "arrellanar",
    "arrellanaras": "arrellanar",
    "arrellanare": "arrellanar",
    "arrellanareis": "arrellanar",
    "arrellanaremos": "arrellanar",
    "arrellanaren": "arrellanar",
    "arrellanares": "arrellanar",
    "arrellanaron": "arrellanar",
    "arrellanará": "arrellanar",
    "arrellanarán": "arrellanar",
    "arrellanarás": "arrellanar",
    "arrellanaré": "arrellanar",
    "arrellanaréis": "arrellanar",
    "arrellanaría": "arrellanar",
    "arrellanaríais": "arrellanar",
    "arrellanaríamos": "arrellanar",
    "arrellanarían": "arrellanar",
    "arrellanarías": "arrellanar",
    "arrellanas": "arrellanar",
    "arrellanase": "arrellanar",
    "arrellanaseis": "arrellanar",
    "arrellanasen": "arrellanar",
    "arrellanases": "arrellanar",
    "arrellanaste": "arrellanar",
    "arrellanasteis": "arrellanar",
    "arrellane": "arrellanar",
    "arrellanemos": "arrellanar",
    "arrellanen": "arrellanar",
    "arrellanes": "arrellanar",
    "arrellano": "arrellanar",
    "arrellanábamos": "arrellanar",
    "arrellanáis": "arrellanar",
    "arrellanáramos": "arrellanar",
    "arrellanáremos": "arrellanar",
    "arrellanásemos": "arrellanar",
    "arrellané": "arrellanar",
    "arrellanéis": "arrellanar",
    "arrellanó": "arrellanar",
    "arremanga": "arremangar",
    "arremangaba": "arremangar",
    "arremangabais": "arremangar",
    "arremangaban": "arremangar",
    "arremangabas": "arremangar",
    "arremangad": "arremangar",
    "arremangada": "arremangar",
    "arremangadas": "arremangar",
    "arremangado": "arremangar",
    "arremangados": "arremangar",
    "arremangamos": "arremangar",
    "arremangan": "arremangar",
    "arremangando": "arremangar",
    "arremangara": "arremangar",
    "arremangarais": "arremangar",
    "arremangaran": "arremangar",
    "arremangaras": "arremangar",
    "arremangare": "arremangar",
    "arremangareis": "arremangar",
    "arremangaremos": "arremangar",
    "arremangaren": "arremangar",
    "arremangares": "arremangar",
    "arremangaron": "arremangar",
    "arremangará": "arremangar",
    "arremangarán": "arremangar",
    "arremangarás": "arremangar",
    "arremangaré": "arremangar",
    "arremangaréis": "arremangar",
    "arremangaría": "arremangar",
    "arremangaríais": "arremangar",
    "arremangaríamos": "arremangar",
    "arremangarían": "arremangar",
    "arremangarías": "arremangar",
    "arremangas": "arremangar",
    "arremangase": "arremangar",
    "arremangaseis": "arremangar",
    "arremangasen": "arremangar",
    "arremangases": "arremangar",
    "arremangaste": "arremangar",
    "arremangasteis": "arremangar",
    "arremango": "arremangar",
    "arremangos": "arremango",
    "arremangue": "arremangar",
    "arremanguemos": "arremangar",
    "arremanguen": "arremangar",
    "arremangues": "arremangar",
    "arremangué": "arremangar",
    "arremanguéis": "arremangar",
    "arremangábamos": "arremangar",
    "arremangáis": "arremangar",
    "arremangáramos": "arremangar",
    "arremangáremos": "arremangar",
    "arremangásemos": "arremangar",
    "arremangó": "arremangar",
    "arremeta": "arremeter",
    "arremetamos": "arremeter",
    "arremetan": "arremeter",
    "arremetas": "arremeter",
    "arremete": "arremeter",
    "arremeted": "arremeter",
    "arremetederos": "arremetedero",
    "arremetedora": "arremetedor",
    "arremetedoras": "arremetedor",
    "arremetedores": "arremetedor",
    "arremetemos": "arremeter",
    "arremeten": "arremeter",
    "arremeteremos": "arremeter",
    "arremeterá": "arremeter",
    "arremeterán": "arremeter",
    "arremeterás": "arremeter",
    "arremeteré": "arremeter",
    "arremeteréis": "arremeter",
    "arremetería": "arremeter",
    "arremeteríais": "arremeter",
    "arremeteríamos": "arremeter",
    "arremeterían": "arremeter",
    "arremeterías": "arremeter",
    "arremetes": "arremeter",
    "arremetida": "arremeter",
    "arremetidas": "arremetida",
    "arremetido": "arremeter",
    "arremetidos": "arremeter",
    "arremetiendo": "arremeter",
    "arremetiera": "arremeter",
    "arremetierais": "arremeter",
    "arremetieran": "arremeter",
    "arremetieras": "arremeter",
    "arremetiere": "arremeter",
    "arremetiereis": "arremeter",
    "arremetieren": "arremeter",
    "arremetieres": "arremeter",
    "arremetieron": "arremeter",
    "arremetiese": "arremeter",
    "arremetieseis": "arremeter",
    "arremetiesen": "arremeter",
    "arremetieses": "arremeter",
    "arremetimientos": "arremetimiento",
    "arremetimos": "arremeter",
    "arremetiste": "arremeter",
    "arremetisteis": "arremeter",
    "arremetiéramos": "arremeter",
    "arremetiéremos": "arremeter",
    "arremetiésemos": "arremeter",
    "arremetió": "arremeter",
    "arremeto": "arremeter",
    "arremetáis": "arremeter",
    "arremetéis": "arremeter",
    "arremetí": "arremeter",
    "arremetía": "arremeter",
    "arremetíais": "arremeter",
    "arremetíamos": "arremeter",
    "arremetían": "arremeter",
    "arremetías": "arremeter",
    "arremolina": "arremolinar",
    "arremolinaba": "arremolinar",
    "arremolinabais": "arremolinar",
    "arremolinaban": "arremolinar",
    "arremolinabas": "arremolinar",
    "arremolinad": "arremolinar",
    "arremolinada": "arremolinar",
    "arremolinadas": "arremolinar",
    "arremolinado": "arremolinar",
    "arremolinados": "arremolinar",
    "arremolinamientos": "arremolinamiento",
    "arremolinamos": "arremolinar",
    "arremolinan": "arremolinar",
    "arremolinando": "arremolinar",
    "arremolinara": "arremolinar",
    "arremolinarais": "arremolinar",
    "arremolinaran": "arremolinar",
    "arremolinaras": "arremolinar",
    "arremolinare": "arremolinar",
    "arremolinareis": "arremolinar",
    "arremolinaremos": "arremolinar",
    "arremolinaren": "arremolinar",
    "arremolinares": "arremolinar",
    "arremolinaron": "arremolinar",
    "arremolinará": "arremolinar",
    "arremolinarán": "arremolinar",
    "arremolinarás": "arremolinar",
    "arremolinaré": "arremolinar",
    "arremolinaréis": "arremolinar",
    "arremolinaría": "arremolinar",
    "arremolinaríais": "arremolinar",
    "arremolinaríamos": "arremolinar",
    "arremolinarían": "arremolinar",
    "arremolinarías": "arremolinar",
    "arremolinas": "arremolinar",
    "arremolinase": "arremolinar",
    "arremolinaseis": "arremolinar",
    "arremolinasen": "arremolinar",
    "arremolinases": "arremolinar",
    "arremolinaste": "arremolinar",
    "arremolinasteis": "arremolinar",
    "arremoline": "arremolinar",
    "arremolinemos": "arremolinar",
    "arremolinen": "arremolinar",
    "arremolines": "arremolinar",
    "arremolino": "arremolinar",
    "arremolinábamos": "arremolinar",
    "arremolináis": "arremolinar",
    "arremolináramos": "arremolinar",
    "arremolináremos": "arremolinar",
    "arremolinásemos": "arremolinar",
    "arremoliné": "arremolinar",
    "arremolinéis": "arremolinar",
    "arremolinó": "arremolinar",
    "arrempuja": "arrempujar",
    "arrempujaba": "arrempujar",
    "arrempujabais": "arrempujar",
    "arrempujaban": "arrempujar",
    "arrempujabas": "arrempujar",
    "arrempujad": "arrempujar",
    "arrempujada": "arrempujar",
    "arrempujadas": "arrempujar",
    "arrempujado": "arrempujar",
    "arrempujados": "arrempujar",
    "arrempujamos": "arrempujar",
    "arrempujan": "arrempujar",
    "arrempujando": "arrempujar",
    "arrempujara": "arrempujar",
    "arrempujarais": "arrempujar",
    "arrempujaran": "arrempujar",
    "arrempujaras": "arrempujar",
    "arrempujare": "arrempujar",
    "arrempujareis": "arrempujar",
    "arrempujaremos": "arrempujar",
    "arrempujaren": "arrempujar",
    "arrempujares": "arrempujar",
    "arrempujaron": "arrempujar",
    "arrempujará": "arrempujar",
    "arrempujarán": "arrempujar",
    "arrempujarás": "arrempujar",
    "arrempujaré": "arrempujar",
    "arrempujaréis": "arrempujar",
    "arrempujaría": "arrempujar",
    "arrempujaríais": "arrempujar",
    "arrempujaríamos": "arrempujar",
    "arrempujarían": "arrempujar",
    "arrempujarías": "arrempujar",
    "arrempujas": "arrempujar",
    "arrempujase": "arrempujar",
    "arrempujaseis": "arrempujar",
    "arrempujasen": "arrempujar",
    "arrempujases": "arrempujar",
    "arrempujaste": "arrempujar",
    "arrempujasteis": "arrempujar",
    "arrempuje": "arrempujar",
    "arrempujemos": "arrempujar",
    "arrempujen": "arrempujar",
    "arrempujes": "arrempujar",
    "arrempujo": "arrempujar",
    "arrempujábamos": "arrempujar",
    "arrempujáis": "arrempujar",
    "arrempujáramos": "arrempujar",
    "arrempujáremos": "arrempujar",
    "arrempujásemos": "arrempujar",
    "arrempujé": "arrempujar",
    "arrempujéis": "arrempujar",
    "arrempujó": "arrempujar",
    "arrendaba": "arrendar",
    "arrendabais": "arrendar",
    "arrendaban": "arrendar",
    "arrendabas": "arrendar",
    "arrendables": "arrendable",
    "arrendad": "arrendar",
    "arrendada": "arrendar",
    "arrendadas": "arrendar",
    "arrendaderos": "arrendadero",
    "arrendado": "arrendar",
    "arrendadora": "arrendador",
    "arrendadoras": "arrendador",
    "arrendadores": "arrendador",
    "arrendados": "arrendar",
    "arrendajos": "arrendajo",
    "arrendamientos": "arrendamiento",
    "arrendamos": "arrendar",
    "arrendando": "arrendar",
    "arrendara": "arrendar",
    "arrendarais": "arrendar",
    "arrendaran": "arrendar",
    "arrendaras": "arrendar",
    "arrendare": "arrendar",
    "arrendareis": "arrendar",
    "arrendaremos": "arrendar",
    "arrendaren": "arrendar",
    "arrendares": "arrendar",
    "arrendaron": "arrendar",
    "arrendará": "arrendar",
    "arrendarán": "arrendar",
    "arrendarás": "arrendar",
    "arrendaré": "arrendar",
    "arrendaréis": "arrendar",
    "arrendaría": "arrendar",
    "arrendaríais": "arrendar",
    "arrendaríamos": "arrendar",
    "arrendarían": "arrendar",
    "arrendarías": "arrendar",
    "arrendase": "arrendar",
    "arrendaseis": "arrendar",
    "arrendasen": "arrendar",
    "arrendases": "arrendar",
    "arrendaste": "arrendar",
    "arrendasteis": "arrendar",
    "arrendataria": "arrendatario",
    "arrendatarias": "arrendatario",
    "arrendatarios": "arrendatario",
    "arrendaticia": "arrendaticio",
    "arrendaticias": "arrendaticio",
    "arrendaticios": "arrendaticio",
    "arrendemos": "arrendar",
    "arrendábamos": "arrendar",
    "arrendáis": "arrendar",
    "arrendáramos": "arrendar",
    "arrendáremos": "arrendar",
    "arrendásemos": "arrendar",
    "arrendé": "arrendar",
    "arrendéis": "arrendar",
    "arrendó": "arrendar",
    "arrenquines": "arrenquín",
    "arreo": "arrear",
    "arreos": "arreo",
    "arrepanchiga": "arrepanchigar",
    "arrepanchigaba": "arrepanchigar",
    "arrepanchigabais": "arrepanchigar",
    "arrepanchigaban": "arrepanchigar",
    "arrepanchigabas": "arrepanchigar",
    "arrepanchigad": "arrepanchigar",
    "arrepanchigada": "arrepanchigar",
    "arrepanchigadas": "arrepanchigar",
    "arrepanchigado": "arrepanchigar",
    "arrepanchigados": "arrepanchigar",
    "arrepanchigamos": "arrepanchigar",
    "arrepanchigan": "arrepanchigar",
    "arrepanchigando": "arrepanchigar",
    "arrepanchigara": "arrepanchigar",
    "arrepanchigarais": "arrepanchigar",
    "arrepanchigaran": "arrepanchigar",
    "arrepanchigaras": "arrepanchigar",
    "arrepanchigare": "arrepanchigar",
    "arrepanchigareis": "arrepanchigar",
    "arrepanchigaremos": "arrepanchigar",
    "arrepanchigaren": "arrepanchigar",
    "arrepanchigares": "arrepanchigar",
    "arrepanchigaron": "arrepanchigar",
    "arrepanchigará": "arrepanchigar",
    "arrepanchigarán": "arrepanchigar",
    "arrepanchigarás": "arrepanchigar",
    "arrepanchigaré": "arrepanchigar",
    "arrepanchigaréis": "arrepanchigar",
    "arrepanchigaría": "arrepanchigar",
    "arrepanchigaríais": "arrepanchigar",
    "arrepanchigaríamos": "arrepanchigar",
    "arrepanchigarían": "arrepanchigar",
    "arrepanchigarías": "arrepanchigar",
    "arrepanchigas": "arrepanchigar",
    "arrepanchigase": "arrepanchigar",
    "arrepanchigaseis": "arrepanchigar",
    "arrepanchigasen": "arrepanchigar",
    "arrepanchigases": "arrepanchigar",
    "arrepanchigaste": "arrepanchigar",
    "arrepanchigasteis": "arrepanchigar",
    "arrepanchigo": "arrepanchigar",
    "arrepanchigue": "arrepanchigar",
    "arrepanchiguemos": "arrepanchigar",
    "arrepanchiguen": "arrepanchigar",
    "arrepanchigues": "arrepanchigar",
    "arrepanchigué": "arrepanchigar",
    "arrepanchiguéis": "arrepanchigar",
    "arrepanchigábamos": "arrepanchigar",
    "arrepanchigáis": "arrepanchigar",
    "arrepanchigáramos": "arrepanchigar",
    "arrepanchigáremos": "arrepanchigar",
    "arrepanchigásemos": "arrepanchigar",
    "arrepanchigó": "arrepanchigar",
    "arrepentid": "arrepentir",
    "arrepentida": "arrepentir",
    "arrepentidas": "arrepentir",
    "arrepentido": "arrepentir",
    "arrepentidos": "arrepentir",
    "arrepentimientos": "arrepentimiento",
    "arrepentimos": "arrepentir",
    "arrepentiremos": "arrepentir",
    "arrepentirá": "arrepentir",
    "arrepentirán": "arrepentir",
    "arrepentirás": "arrepentir",
    "arrepentiré": "arrepentir",
    "arrepentiréis": "arrepentir",
    "arrepentiría": "arrepentir",
    "arrepentiríais": "arrepentir",
    "arrepentiríamos": "arrepentir",
    "arrepentirían": "arrepentir",
    "arrepentirías": "arrepentir",
    "arrepentiste": "arrepentir",
    "arrepentisteis": "arrepentir",
    "arrepentí": "arrepentir",
    "arrepentía": "arrepentir",
    "arrepentíais": "arrepentir",
    "arrepentíamos": "arrepentir",
    "arrepentían": "arrepentir",
    "arrepentías": "arrepentir",
    "arrepentís": "arrepentir",
    "arrepienta": "arrepentir",
    "arrepientan": "arrepentir",
    "arrepientas": "arrepentir",
    "arrepiente": "arrepentir",
    "arrepienten": "arrepentir",
    "arrepientes": "arrepentir",
    "arrepiento": "arrepentir",
    "arrepintamos": "arrepentir",
    "arrepintiendo": "arrepentir",
    "arrepintiera": "arrepentir",
    "arrepintierais": "arrepentir",
    "arrepintieran": "arrepentir",
    "arrepintieras": "arrepentir",
    "arrepintiere": "arrepentir",
    "arrepintiereis": "arrepentir",
    "arrepintieren": "arrepentir",
    "arrepintieres": "arrepentir",
    "arrepintieron": "arrepentir",
    "arrepintiese": "arrepentir",
    "arrepintieseis": "arrepentir",
    "arrepintiesen": "arrepentir",
    "arrepintieses": "arrepentir",
    "arrepintiéramos": "arrepentir",
    "arrepintiéremos": "arrepentir",
    "arrepintiésemos": "arrepentir",
    "arrepintió": "arrepentir",
    "arrepintáis": "arrepentir",
    "arrepápalos": "arrepápalo",
    "arrequesona": "arrequesonar",
    "arrequesonaba": "arrequesonar",
    "arrequesonabais": "arrequesonar",
    "arrequesonaban": "arrequesonar",
    "arrequesonabas": "arrequesonar",
    "arrequesonad": "arrequesonar",
    "arrequesonada": "arrequesonar",
    "arrequesonadas": "arrequesonar",
    "arrequesonado": "arrequesonar",
    "arrequesonados": "arrequesonar",
    "arrequesonamos": "arrequesonar",
    "arrequesonan": "arrequesonar",
    "arrequesonando": "arrequesonar",
    "arrequesonara": "arrequesonar",
    "arrequesonarais": "arrequesonar",
    "arrequesonaran": "arrequesonar",
    "arrequesonaras": "arrequesonar",
    "arrequesonare": "arrequesonar",
    "arrequesonareis": "arrequesonar",
    "arrequesonaremos": "arrequesonar",
    "arrequesonaren": "arrequesonar",
    "arrequesonares": "arrequesonar",
    "arrequesonaron": "arrequesonar",
    "arrequesonará": "arrequesonar",
    "arrequesonarán": "arrequesonar",
    "arrequesonarás": "arrequesonar",
    "arrequesonaré": "arrequesonar",
    "arrequesonaréis": "arrequesonar",
    "arrequesonaría": "arrequesonar",
    "arrequesonaríais": "arrequesonar",
    "arrequesonaríamos": "arrequesonar",
    "arrequesonarían": "arrequesonar",
    "arrequesonarías": "arrequesonar",
    "arrequesonas": "arrequesonar",
    "arrequesonase": "arrequesonar",
    "arrequesonaseis": "arrequesonar",
    "arrequesonasen": "arrequesonar",
    "arrequesonases": "arrequesonar",
    "arrequesonaste": "arrequesonar",
    "arrequesonasteis": "arrequesonar",
    "arrequesone": "arrequesonar",
    "arrequesonemos": "arrequesonar",
    "arrequesonen": "arrequesonar",
    "arrequesones": "arrequesonar",
    "arrequesono": "arrequesonar",
    "arrequesonábamos": "arrequesonar",
    "arrequesonáis": "arrequesonar",
    "arrequesonáramos": "arrequesonar",
    "arrequesonáremos": "arrequesonar",
    "arrequesonásemos": "arrequesonar",
    "arrequesoné": "arrequesonar",
    "arrequesonéis": "arrequesonar",
    "arrequesonó": "arrequesonar",
    "arrequifes": "arrequife",
    "arrequives": "arrequive",
    "arresta": "arrestar",
    "arrestaba": "arrestar",
    "arrestabais": "arrestar",
    "arrestaban": "arrestar",
    "arrestabas": "arrestar",
    "arrestad": "arrestar",
    "arrestada": "arrestar",
    "arrestadas": "arrestar",
    "arrestado": "arrestar",
    "arrestados": "arrestar",
    "arrestamos": "arrestar",
    "arrestan": "arrestar",
    "arrestando": "arrestar",
    "arrestara": "arrestar",
    "arrestarais": "arrestar",
    "arrestaran": "arrestar",
    "arrestaras": "arrestar",
    "arrestare": "arrestar",
    "arrestareis": "arrestar",
    "arrestaremos": "arrestar",
    "arrestaren": "arrestar",
    "arrestares": "arrestar",
    "arrestaron": "arrestar",
    "arrestará": "arrestar",
    "arrestarán": "arrestar",
    "arrestarás": "arrestar",
    "arrestaré": "arrestar",
    "arrestaréis": "arrestar",
    "arrestaría": "arrestar",
    "arrestaríais": "arrestar",
    "arrestaríamos": "arrestar",
    "arrestarían": "arrestar",
    "arrestarías": "arrestar",
    "arrestas": "arrestar",
    "arrestase": "arrestar",
    "arrestaseis": "arrestar",
    "arrestasen": "arrestar",
    "arrestases": "arrestar",
    "arrestaste": "arrestar",
    "arrestasteis": "arrestar",
    "arreste": "arrestar",
    "arrestemos": "arrestar",
    "arresten": "arrestar",
    "arrestes": "arrestar",
    "arresto": "arrestar",
    "arrestos": "arresto",
    "arrestábamos": "arrestar",
    "arrestáis": "arrestar",
    "arrestáramos": "arrestar",
    "arrestáremos": "arrestar",
    "arrestásemos": "arrestar",
    "arresté": "arrestar",
    "arrestéis": "arrestar",
    "arrestó": "arrestar",
    "arrezaga": "arrezagar",
    "arrezagaba": "arrezagar",
    "arrezagabais": "arrezagar",
    "arrezagaban": "arrezagar",
    "arrezagabas": "arrezagar",
    "arrezagad": "arrezagar",
    "arrezagada": "arrezagar",
    "arrezagadas": "arrezagar",
    "arrezagado": "arrezagar",
    "arrezagados": "arrezagar",
    "arrezagamos": "arrezagar",
    "arrezagan": "arrezagar",
    "arrezagando": "arrezagar",
    "arrezagara": "arrezagar",
    "arrezagarais": "arrezagar",
    "arrezagaran": "arrezagar",
    "arrezagaras": "arrezagar",
    "arrezagare": "arrezagar",
    "arrezagareis": "arrezagar",
    "arrezagaremos": "arrezagar",
    "arrezagaren": "arrezagar",
    "arrezagares": "arrezagar",
    "arrezagaron": "arrezagar",
    "arrezagará": "arrezagar",
    "arrezagarán": "arrezagar",
    "arrezagarás": "arrezagar",
    "arrezagaré": "arrezagar",
    "arrezagaréis": "arrezagar",
    "arrezagaría": "arrezagar",
    "arrezagaríais": "arrezagar",
    "arrezagaríamos": "arrezagar",
    "arrezagarían": "arrezagar",
    "arrezagarías": "arrezagar",
    "arrezagas": "arrezagar",
    "arrezagase": "arrezagar",
    "arrezagaseis": "arrezagar",
    "arrezagasen": "arrezagar",
    "arrezagases": "arrezagar",
    "arrezagaste": "arrezagar",
    "arrezagasteis": "arrezagar",
    "arrezago": "arrezagar",
    "arrezague": "arrezagar",
    "arrezaguemos": "arrezagar",
    "arrezaguen": "arrezagar",
    "arrezagues": "arrezagar",
    "arrezagué": "arrezagar",
    "arrezaguéis": "arrezagar",
    "arrezagábamos": "arrezagar",
    "arrezagáis": "arrezagar",
    "arrezagáramos": "arrezagar",
    "arrezagáremos": "arrezagar",
    "arrezagásemos": "arrezagar",
    "arrezagó": "arrezagar",
    "arreábamos": "arrear",
    "arreáis": "arrear",
    "arreáramos": "arrear",
    "arreáremos": "arrear",
    "arreásemos": "arrear",
    "arreé": "arrear",
    "arreéis": "arrear",
    "arreó": "arrear",
    "arriaba": "arriar",
    "arriabais": "arriar",
    "arriaban": "arriar",
    "arriabas": "arriar",
    "arriad": "arriar",
    "arriada": "arriar",
    "arriadas": "arriar",
    "arriado": "arriar",
    "arriados": "arriar",
    "arriamos": "arriar",
    "arriana": "arriano",
    "arrianas": "arriano",
    "arriando": "arriar",
    "arrianismos": "arrianismo",
    "arrianos": "arriano",
    "arriara": "arriar",
    "arriarais": "arriar",
    "arriaran": "arriar",
    "arriaras": "arriar",
    "arriare": "arriar",
    "arriareis": "arriar",
    "arriaremos": "arriar",
    "arriaren": "arriar",
    "arriares": "arriar",
    "arriaron": "arriar",
    "arriará": "arriar",
    "arriarán": "arriar",
    "arriarás": "arriar",
    "arriaré": "arriar",
    "arriaréis": "arriar",
    "arriaría": "arriar",
    "arriaríais": "arriar",
    "arriaríamos": "arriar",
    "arriarían": "arriar",
    "arriarías": "arriar",
    "arriase": "arriar",
    "arriaseis": "arriar",
    "arriasen": "arriar",
    "arriases": "arriar",
    "arriaste": "arriar",
    "arriasteis": "arriar",
    "arriatas": "arriata",
    "arriates": "arriate",
    "arriba": "arribar",
    "arribaba": "arribar",
    "arribabais": "arribar",
    "arribaban": "arribar",
    "arribabas": "arribar",
    "arribad": "arribar",
    "arribada": "arribar",
    "arribadas": "arribar",
    "arribado": "arribar",
    "arribados": "arribar",
    "arribamos": "arribar",
    "arriban": "arribar",
    "arribando": "arribar",
    "arribara": "arribar",
    "arribarais": "arribar",
    "arribaran": "arribar",
    "arribaras": "arribar",
    "arribare": "arribar",
    "arribareis": "arribar",
    "arribaremos": "arribar",
    "arribaren": "arribar",
    "arribares": "arribar",
    "arribaron": "arribar",
    "arribará": "arribar",
    "arribarán": "arribar",
    "arribarás": "arribar",
    "arribaré": "arribar",
    "arribaréis": "arribar",
    "arribaría": "arribar",
    "arribaríais": "arribar",
    "arribaríamos": "arribar",
    "arribarían": "arribar",
    "arribarías": "arribar",
    "arribas": "arribar",
    "arribase": "arribar",
    "arribaseis": "arribar",
    "arribasen": "arribar",
    "arribases": "arribar",
    "arribaste": "arribar",
    "arribasteis": "arribar",
    "arribazones": "arribazón",
    "arribe": "arribar",
    "arribemos": "arribar",
    "arriben": "arribar",
    "arribes": "arribar",
    "arribeña": "arribeño",
    "arribeñas": "arribeño",
    "arribeños": "arribeño",
    "arribismos": "arribismo",
    "arribistas": "arribista",
    "arribo": "arribar",
    "arribos": "arribo",
    "arribábamos": "arribar",
    "arribáis": "arribar",
    "arribáramos": "arribar",
    "arribáremos": "arribar",
    "arribásemos": "arribar",
    "arribé": "arribar",
    "arribéis": "arribar",
    "arribó": "arribar",
    "arrice": "arrizar",
    "arricemos": "arrizar",
    "arricen": "arrizar",
    "arrices": "arrizar",
    "arriceses": "arricés",
    "arricé": "arrizar",
    "arricéis": "arrizar",
    "arrida": "arridar",
    "arridaba": "arridar",
    "arridabais": "arridar",
    "arridaban": "arridar",
    "arridabas": "arridar",
    "arridad": "arridar",
    "arridada": "arridar",
    "arridadas": "arridar",
    "arridado": "arridar",
    "arridados": "arridar",
    "arridamos": "arridar",
    "arridan": "arridar",
    "arridando": "arridar",
    "arridara": "arridar",
    "arridarais": "arridar",
    "arridaran": "arridar",
    "arridaras": "arridar",
    "arridare": "arridar",
    "arridareis": "arridar",
    "arridaremos": "arridar",
    "arridaren": "arridar",
    "arridares": "arridar",
    "arridaron": "arridar",
    "arridará": "arridar",
    "arridarán": "arridar",
    "arridarás": "arridar",
    "arridaré": "arridar",
    "arridaréis": "arridar",
    "arridaría": "arridar",
    "arridaríais": "arridar",
    "arridaríamos": "arridar",
    "arridarían": "arridar",
    "arridarías": "arridar",
    "arridas": "arridar",
    "arridase": "arridar",
    "arridaseis": "arridar",
    "arridasen": "arridar",
    "arridases": "arridar",
    "arridaste": "arridar",
    "arridasteis": "arridar",
    "arride": "arridar",
    "arridemos": "arridar",
    "arriden": "arridar",
    "arrides": "arridar",
    "arrido": "arridar",
    "arridábamos": "arridar",
    "arridáis": "arridar",
    "arridáramos": "arridar",
    "arridáremos": "arridar",
    "arridásemos": "arridar",
    "arridé": "arridar",
    "arridéis": "arridar",
    "arridó": "arridar",
    "arriemos": "arriar",
    "arrienda": "arrendar",
    "arriendan": "arrendar",
    "arriendas": "arrendar",
    "arriende": "arrendar",
    "arrienden": "arrendar",
    "arriendes": "arrendar",
    "arriendo": "arrendar",
    "arriendos": "arriendo",
    "arriera": "arriero",
    "arrieras": "arriero",
    "arrieros": "arriero",
    "arrierías": "arriería",
    "arriesga": "arriesgar",
    "arriesgaba": "arriesgar",
    "arriesgabais": "arriesgar",
    "arriesgaban": "arriesgar",
    "arriesgabas": "arriesgar",
    "arriesgad": "arriesgar",
    "arriesgada": "arriesgar",
    "arriesgadas": "arriesgar",
    "arriesgado": "arriesgar",
    "arriesgados": "arriesgar",
    "arriesgamos": "arriesgar",
    "arriesgan": "arriesgar",
    "arriesgando": "arriesgar",
    "arriesgara": "arriesgar",
    "arriesgarais": "arriesgar",
    "arriesgaran": "arriesgar",
    "arriesgaras": "arriesgar",
    "arriesgare": "arriesgar",
    "arriesgareis": "arriesgar",
    "arriesgaremos": "arriesgar",
    "arriesgaren": "arriesgar",
    "arriesgares": "arriesgar",
    "arriesgaron": "arriesgar",
    "arriesgará": "arriesgar",
    "arriesgarán": "arriesgar",
    "arriesgarás": "arriesgar",
    "arriesgaré": "arriesgar",
    "arriesgaréis": "arriesgar",
    "arriesgaría": "arriesgar",
    "arriesgaríais": "arriesgar",
    "arriesgaríamos": "arriesgar",
    "arriesgarían": "arriesgar",
    "arriesgarías": "arriesgar",
    "arriesgas": "arriesgar",
    "arriesgase": "arriesgar",
    "arriesgaseis": "arriesgar",
    "arriesgasen": "arriesgar",
    "arriesgases": "arriesgar",
    "arriesgaste": "arriesgar",
    "arriesgasteis": "arriesgar",
    "arriesgo": "arriesgar",
    "arriesgue": "arriesgar",
    "arriesguemos": "arriesgar",
    "arriesguen": "arriesgar",
    "arriesgues": "arriesgar",
    "arriesgué": "arriesgar",
    "arriesguéis": "arriesgar",
    "arriesgábamos": "arriesgar",
    "arriesgáis": "arriesgar",
    "arriesgáramos": "arriesgar",
    "arriesgáremos": "arriesgar",
    "arriesgásemos": "arriesgar",
    "arriesgó": "arriesgar",
    "arrima": "arrimar",
    "arrimaba": "arrimar",
    "arrimabais": "arrimar",
    "arrimaban": "arrimar",
    "arrimabas": "arrimar",
    "arrimad": "arrimar",
    "arrimada": "arrimar",
    "arrimadas": "arrimar",
    "arrimaderos": "arrimadero",
    "arrimadiza": "arrimadizo",
    "arrimadizas": "arrimadizo",
    "arrimadizos": "arrimadizo",
    "arrimado": "arrimar",
    "arrimadora": "arrimador",
    "arrimadoras": "arrimador",
    "arrimadores": "arrimador",
    "arrimados": "arrimar",
    "arrimamos": "arrimar",
    "arriman": "arrimar",
    "arrimando": "arrimar",
    "arrimara": "arrimar",
    "arrimarais": "arrimar",
    "arrimaran": "arrimar",
    "arrimaras": "arrimar",
    "arrimare": "arrimar",
    "arrimareis": "arrimar",
    "arrimaremos": "arrimar",
    "arrimaren": "arrimar",
    "arrimares": "arrimar",
    "arrimaron": "arrimar",
    "arrimará": "arrimar",
    "arrimarán": "arrimar",
    "arrimarás": "arrimar",
    "arrimaré": "arrimar",
    "arrimaréis": "arrimar",
    "arrimaría": "arrimar",
    "arrimaríais": "arrimar",
    "arrimaríamos": "arrimar",
    "arrimarían": "arrimar",
    "arrimarías": "arrimar",
    "arrimas": "arrimar",
    "arrimase": "arrimar",
    "arrimaseis": "arrimar",
    "arrimasen": "arrimar",
    "arrimases": "arrimar",
    "arrimaste": "arrimar",
    "arrimasteis": "arrimar",
    "arrime": "arrimar",
    "arrimemos": "arrimar",
    "arrimen": "arrimar",
    "arrimes": "arrime",
    "arrimo": "arrimar",
    "arrimona": "arrimón",
    "arrimonas": "arrimón",
    "arrimones": "arrimón",
    "arrimos": "arrimo",
    "arrimábamos": "arrimar",
    "arrimáis": "arrimar",
    "arrimáramos": "arrimar",
    "arrimáremos": "arrimar",
    "arrimásemos": "arrimar",
    "arrimé": "arrimar",
    "arriméis": "arrimar",
    "arrimó": "arrimar",
    "arrincona": "arrinconar",
    "arrinconaba": "arrinconar",
    "arrinconabais": "arrinconar",
    "arrinconaban": "arrinconar",
    "arrinconabas": "arrinconar",
    "arrinconad": "arrinconar",
    "arrinconada": "arrinconar",
    "arrinconadas": "arrinconar",
    "arrinconado": "arrinconar",
    "arrinconados": "arrinconar",
    "arrinconamientos": "arrinconamiento",
    "arrinconamos": "arrinconar",
    "arrinconan": "arrinconar",
    "arrinconando": "arrinconar",
    "arrinconara": "arrinconar",
    "arrinconarais": "arrinconar",
    "arrinconaran": "arrinconar",
    "arrinconaras": "arrinconar",
    "arrinconare": "arrinconar",
    "arrinconareis": "arrinconar",
    "arrinconaremos": "arrinconar",
    "arrinconaren": "arrinconar",
    "arrinconares": "arrinconar",
    "arrinconaron": "arrinconar",
    "arrinconará": "arrinconar",
    "arrinconarán": "arrinconar",
    "arrinconarás": "arrinconar",
    "arrinconaré": "arrinconar",
    "arrinconaréis": "arrinconar",
    "arrinconaría": "arrinconar",
    "arrinconaríais": "arrinconar",
    "arrinconaríamos": "arrinconar",
    "arrinconarían": "arrinconar",
    "arrinconarías": "arrinconar",
    "arrinconas": "arrinconar",
    "arrinconase": "arrinconar",
    "arrinconaseis": "arrinconar",
    "arrinconasen": "arrinconar",
    "arrinconases": "arrinconar",
    "arrinconaste": "arrinconar",
    "arrinconasteis": "arrinconar",
    "arrincone": "arrinconar",
    "arrinconemos": "arrinconar",
    "arrinconen": "arrinconar",
    "arrincones": "arrinconar",
    "arrincono": "arrinconar",
    "arrinconábamos": "arrinconar",
    "arrinconáis": "arrinconar",
    "arrinconáramos": "arrinconar",
    "arrinconáremos": "arrinconar",
    "arrinconásemos": "arrinconar",
    "arrinconé": "arrinconar",
    "arrinconéis": "arrinconar",
    "arrinconó": "arrinconar",
    "arriostra": "arriostrar",
    "arriostraba": "arriostrar",
    "arriostrabais": "arriostrar",
    "arriostraban": "arriostrar",
    "arriostrabas": "arriostrar",
    "arriostrad": "arriostrar",
    "arriostrada": "arriostrar",
    "arriostradas": "arriostrar",
    "arriostrado": "arriostrar",
    "arriostrados": "arriostrar",
    "arriostramientos": "arriostramiento",
    "arriostramos": "arriostrar",
    "arriostran": "arriostrar",
    "arriostrando": "arriostrar",
    "arriostrara": "arriostrar",
    "arriostrarais": "arriostrar",
    "arriostraran": "arriostrar",
    "arriostraras": "arriostrar",
    "arriostrare": "arriostrar",
    "arriostrareis": "arriostrar",
    "arriostraremos": "arriostrar",
    "arriostraren": "arriostrar",
    "arriostrares": "arriostrar",
    "arriostraron": "arriostrar",
    "arriostrará": "arriostrar",
    "arriostrarán": "arriostrar",
    "arriostrarás": "arriostrar",
    "arriostraré": "arriostrar",
    "arriostraréis": "arriostrar",
    "arriostraría": "arriostrar",
    "arriostraríais": "arriostrar",
    "arriostraríamos": "arriostrar",
    "arriostrarían": "arriostrar",
    "arriostrarías": "arriostrar",
    "arriostras": "arriostrar",
    "arriostrase": "arriostrar",
    "arriostraseis": "arriostrar",
    "arriostrasen": "arriostrar",
    "arriostrases": "arriostrar",
    "arriostraste": "arriostrar",
    "arriostrasteis": "arriostrar",
    "arriostre": "arriostrar",
    "arriostremos": "arriostrar",
    "arriostren": "arriostrar",
    "arriostres": "arriostrar",
    "arriostro": "arriostrar",
    "arriostrábamos": "arriostrar",
    "arriostráis": "arriostrar",
    "arriostráramos": "arriostrar",
    "arriostráremos": "arriostrar",
    "arriostrásemos": "arriostrar",
    "arriostré": "arriostrar",
    "arriostréis": "arriostrar",
    "arriostró": "arriostrar",
    "arrisca": "arriscar",
    "arriscaba": "arriscar",
    "arriscabais": "arriscar",
    "arriscaban": "arriscar",
    "arriscabas": "arriscar",
    "arriscad": "arriscar",
    "arriscada": "arriscar",
    "arriscadas": "arriscar",
    "arriscado": "arriscar",
    "arriscadora": "arriscador",
    "arriscadoras": "arriscador",
    "arriscadores": "arriscador",
    "arriscados": "arriscar",
    "arriscamos": "arriscar",
    "arriscan": "arriscar",
    "arriscando": "arriscar",
    "arriscara": "arriscar",
    "arriscarais": "arriscar",
    "arriscaran": "arriscar",
    "arriscaras": "arriscar",
    "arriscare": "arriscar",
    "arriscareis": "arriscar",
    "arriscaremos": "arriscar",
    "arriscaren": "arriscar",
    "arriscares": "arriscar",
    "arriscaron": "arriscar",
    "arriscará": "arriscar",
    "arriscarán": "arriscar",
    "arriscarás": "arriscar",
    "arriscaré": "arriscar",
    "arriscaréis": "arriscar",
    "arriscaría": "arriscar",
    "arriscaríais": "arriscar",
    "arriscaríamos": "arriscar",
    "arriscarían": "arriscar",
    "arriscarías": "arriscar",
    "arriscas": "arriscar",
    "arriscase": "arriscar",
    "arriscaseis": "arriscar",
    "arriscasen": "arriscar",
    "arriscases": "arriscar",
    "arriscaste": "arriscar",
    "arriscasteis": "arriscar",
    "arrisco": "arriscar",
    "arriscábamos": "arriscar",
    "arriscáis": "arriscar",
    "arriscáramos": "arriscar",
    "arriscáremos": "arriscar",
    "arriscásemos": "arriscar",
    "arriscó": "arriscar",
    "arrisque": "arriscar",
    "arrisquemos": "arriscar",
    "arrisquen": "arriscar",
    "arrisques": "arriscar",
    "arrisqué": "arriscar",
    "arrisquéis": "arriscar",
    "arritmias": "arritmia",
    "arriza": "arrizar",
    "arrizaba": "arrizar",
    "arrizabais": "arrizar",
    "arrizaban": "arrizar",
    "arrizabas": "arrizar",
    "arrizad": "arrizar",
    "arrizada": "arrizar",
    "arrizadas": "arrizar",
    "arrizado": "arrizar",
    "arrizados": "arrizar",
    "arrizamos": "arrizar",
    "arrizan": "arrizar",
    "arrizando": "arrizar",
    "arrizara": "arrizar",
    "arrizarais": "arrizar",
    "arrizaran": "arrizar",
    "arrizaras": "arrizar",
    "arrizare": "arrizar",
    "arrizareis": "arrizar",
    "arrizaremos": "arrizar",
    "arrizaren": "arrizar",
    "arrizares": "arrizar",
    "arrizaron": "arrizar",
    "arrizará": "arrizar",
    "arrizarán": "arrizar",
    "arrizarás": "arrizar",
    "arrizaré": "arrizar",
    "arrizaréis": "arrizar",
    "arrizaría": "arrizar",
    "arrizaríais": "arrizar",
    "arrizaríamos": "arrizar",
    "arrizarían": "arrizar",
    "arrizarías": "arrizar",
    "arrizas": "arrizar",
    "arrizase": "arrizar",
    "arrizaseis": "arrizar",
    "arrizasen": "arrizar",
    "arrizases": "arrizar",
    "arrizaste": "arrizar",
    "arrizasteis": "arrizar",
    "arrizo": "arrizar",
    "arrizábamos": "arrizar",
    "arrizáis": "arrizar",
    "arrizáramos": "arrizar",
    "arrizáremos": "arrizar",
    "arrizásemos": "arrizar",
    "arrizó": "arrizar",
    "arriábamos": "arriar",
    "arriáis": "arriar",
    "arriáramos": "arriar",
    "arriáremos": "arriar",
    "arriásemos": "arriar",
    "arrié": "arriar",
    "arriéis": "arriar",
    "arriñonada": "arriñonado",
    "arriñonadas": "arriñonado",
    "arriñonados": "arriñonado",
    "arrió": "arriar",
    "arroba": "arrobar",
    "arrobaba": "arrobar",
    "arrobabais": "arrobar",
    "arrobaban": "arrobar",
    "arrobabas": "arrobar",
    "arrobad": "arrobar",
    "arrobada": "arrobar",
    "arrobadas": "arrobar",
    "arrobadiza": "arrobadizo",
    "arrobadizas": "arrobadizo",
    "arrobadizos": "arrobadizo",
    "arrobado": "arrobar",
    "arrobadora": "arrobador",
    "arrobadoras": "arrobador",
    "arrobadores": "arrobador",
    "arrobados": "arrobar",
    "arrobamientos": "arrobamiento",
    "arrobamos": "arrobar",
    "arroban": "arrobar",
    "arrobando": "arrobar",
    "arrobara": "arrobar",
    "arrobarais": "arrobar",
    "arrobaran": "arrobar",
    "arrobaras": "arrobar",
    "arrobare": "arrobar",
    "arrobareis": "arrobar",
    "arrobaremos": "arrobar",
    "arrobaren": "arrobar",
    "arrobares": "arrobar",
    "arrobaron": "arrobar",
    "arrobará": "arrobar",
    "arrobarán": "arrobar",
    "arrobarás": "arrobar",
    "arrobaré": "arrobar",
    "arrobaréis": "arrobar",
    "arrobaría": "arrobar",
    "arrobaríais": "arrobar",
    "arrobaríamos": "arrobar",
    "arrobarían": "arrobar",
    "arrobarías": "arrobar",
    "arrobas": "arrobar",
    "arrobase": "arrobar",
    "arrobaseis": "arrobar",
    "arrobasen": "arrobar",
    "arrobases": "arrobar",
    "arrobaste": "arrobar",
    "arrobasteis": "arrobar",
    "arrobe": "arrobar",
    "arrobemos": "arrobar",
    "arroben": "arrobar",
    "arrobes": "arrobar",
    "arrobo": "arrobar",
    "arrobos": "arrobo",
    "arrobábamos": "arrobar",
    "arrobáis": "arrobar",
    "arrobáramos": "arrobar",
    "arrobáremos": "arrobar",
    "arrobásemos": "arrobar",
    "arrobé": "arrobar",
    "arrobéis": "arrobar",
    "arrobó": "arrobar",
    "arrocabes": "arrocabe",
    "arrocada": "arrocado",
    "arrocadas": "arrocado",
    "arrocados": "arrocado",
    "arrocera": "arrocero",
    "arroceras": "arrocero",
    "arroceros": "arrocero",
    "arroces": "arroz",
    "arrochela": "arrochelar",
    "arrochelaba": "arrochelar",
    "arrochelabais": "arrochelar",
    "arrochelaban": "arrochelar",
    "arrochelabas": "arrochelar",
    "arrochelad": "arrochelar",
    "arrochelada": "arrochelar",
    "arrocheladas": "arrochelar",
    "arrochelado": "arrochelar",
    "arrochelados": "arrochelar",
    "arrochelamos": "arrochelar",
    "arrochelan": "arrochelar",
    "arrochelando": "arrochelar",
    "arrochelara": "arrochelar",
    "arrochelarais": "arrochelar",
    "arrochelaran": "arrochelar",
    "arrochelaras": "arrochelar",
    "arrochelare": "arrochelar",
    "arrochelareis": "arrochelar",
    "arrochelaremos": "arrochelar",
    "arrochelaren": "arrochelar",
    "arrochelares": "arrochelar",
    "arrochelaron": "arrochelar",
    "arrochelará": "arrochelar",
    "arrochelarán": "arrochelar",
    "arrochelarás": "arrochelar",
    "arrochelaré": "arrochelar",
    "arrochelaréis": "arrochelar",
    "arrochelaría": "arrochelar",
    "arrochelaríais": "arrochelar",
    "arrochelaríamos": "arrochelar",
    "arrochelarían": "arrochelar",
    "arrochelarías": "arrochelar",
    "arrochelas": "arrochelar",
    "arrochelase": "arrochelar",
    "arrochelaseis": "arrochelar",
    "arrochelasen": "arrochelar",
    "arrochelases": "arrochelar",
    "arrochelaste": "arrochelar",
    "arrochelasteis": "arrochelar",
    "arrochele": "arrochelar",
    "arrochelemos": "arrochelar",
    "arrochelen": "arrochelar",
    "arrocheles": "arrochelar",
    "arrochelo": "arrochelar",
    "arrochelábamos": "arrochelar",
    "arrocheláis": "arrochelar",
    "arrocheláramos": "arrochelar",
    "arrocheláremos": "arrochelar",
    "arrochelásemos": "arrochelar",
    "arrochelé": "arrochelar",
    "arrocheléis": "arrochelar",
    "arrocheló": "arrochelar",
    "arrocina": "arrocinar",
    "arrocinaba": "arrocinar",
    "arrocinabais": "arrocinar",
    "arrocinaban": "arrocinar",
    "arrocinabas": "arrocinar",
    "arrocinad": "arrocinar",
    "arrocinada": "arrocinar",
    "arrocinadas": "arrocinar",
    "arrocinado": "arrocinar",
    "arrocinados": "arrocinar",
    "arrocinamos": "arrocinar",
    "arrocinan": "arrocinar",
    "arrocinando": "arrocinar",
    "arrocinara": "arrocinar",
    "arrocinarais": "arrocinar",
    "arrocinaran": "arrocinar",
    "arrocinaras": "arrocinar",
    "arrocinare": "arrocinar",
    "arrocinareis": "arrocinar",
    "arrocinaremos": "arrocinar",
    "arrocinaren": "arrocinar",
    "arrocinares": "arrocinar",
    "arrocinaron": "arrocinar",
    "arrocinará": "arrocinar",
    "arrocinarán": "arrocinar",
    "arrocinarás": "arrocinar",
    "arrocinaré": "arrocinar",
    "arrocinaréis": "arrocinar",
    "arrocinaría": "arrocinar",
    "arrocinaríais": "arrocinar",
    "arrocinaríamos": "arrocinar",
    "arrocinarían": "arrocinar",
    "arrocinarías": "arrocinar",
    "arrocinas": "arrocinar",
    "arrocinase": "arrocinar",
    "arrocinaseis": "arrocinar",
    "arrocinasen": "arrocinar",
    "arrocinases": "arrocinar",
    "arrocinaste": "arrocinar",
    "arrocinasteis": "arrocinar",
    "arrocine": "arrocinar",
    "arrocinemos": "arrocinar",
    "arrocinen": "arrocinar",
    "arrocines": "arrocinar",
    "arrocino": "arrocinar",
    "arrocinábamos": "arrocinar",
    "arrocináis": "arrocinar",
    "arrocináramos": "arrocinar",
    "arrocináremos": "arrocinar",
    "arrocinásemos": "arrocinar",
    "arrociné": "arrocinar",
    "arrocinéis": "arrocinar",
    "arrocinó": "arrocinar",
    "arrodaja": "arrodajar",
    "arrodajaba": "arrodajar",
    "arrodajabais": "arrodajar",
    "arrodajaban": "arrodajar",
    "arrodajabas": "arrodajar",
    "arrodajad": "arrodajar",
    "arrodajada": "arrodajar",
    "arrodajadas": "arrodajar",
    "arrodajado": "arrodajar",
    "arrodajados": "arrodajar",
    "arrodajamos": "arrodajar",
    "arrodajan": "arrodajar",
    "arrodajando": "arrodajar",
    "arrodajara": "arrodajar",
    "arrodajarais": "arrodajar",
    "arrodajaran": "arrodajar",
    "arrodajaras": "arrodajar",
    "arrodajare": "arrodajar",
    "arrodajareis": "arrodajar",
    "arrodajaremos": "arrodajar",
    "arrodajaren": "arrodajar",
    "arrodajares": "arrodajar",
    "arrodajaron": "arrodajar",
    "arrodajará": "arrodajar",
    "arrodajarán": "arrodajar",
    "arrodajarás": "arrodajar",
    "arrodajaré": "arrodajar",
    "arrodajaréis": "arrodajar",
    "arrodajaría": "arrodajar",
    "arrodajaríais": "arrodajar",
    "arrodajaríamos": "arrodajar",
    "arrodajarían": "arrodajar",
    "arrodajarías": "arrodajar",
    "arrodajas": "arrodajar",
    "arrodajase": "arrodajar",
    "arrodajaseis": "arrodajar",
    "arrodajasen": "arrodajar",
    "arrodajases": "arrodajar",
    "arrodajaste": "arrodajar",
    "arrodajasteis": "arrodajar",
    "arrodaje": "arrodajar",
    "arrodajemos": "arrodajar",
    "arrodajen": "arrodajar",
    "arrodajes": "arrodajar",
    "arrodajo": "arrodajar",
    "arrodajábamos": "arrodajar",
    "arrodajáis": "arrodajar",
    "arrodajáramos": "arrodajar",
    "arrodajáremos": "arrodajar",
    "arrodajásemos": "arrodajar",
    "arrodajé": "arrodajar",
    "arrodajéis": "arrodajar",
    "arrodajó": "arrodajar",
    "arrodilla": "arrodillar",
    "arrodillaba": "arrodillar",
    "arrodillabais": "arrodillar",
    "arrodillaban": "arrodillar",
    "arrodillabas": "arrodillar",
    "arrodillad": "arrodillar",
    "arrodillada": "arrodillar",
    "arrodilladas": "arrodillar",
    "arrodillado": "arrodillar",
    "arrodillados": "arrodillar",
    "arrodillamientos": "arrodillamiento",
    "arrodillamos": "arrodillar",
    "arrodillan": "arrodillar",
    "arrodillando": "arrodillar",
    "arrodillara": "arrodillar",
    "arrodillarais": "arrodillar",
    "arrodillaran": "arrodillar",
    "arrodillaras": "arrodillar",
    "arrodillare": "arrodillar",
    "arrodillareis": "arrodillar",
    "arrodillaremos": "arrodillar",
    "arrodillaren": "arrodillar",
    "arrodillares": "arrodillar",
    "arrodillaron": "arrodillar",
    "arrodillará": "arrodillar",
    "arrodillarán": "arrodillar",
    "arrodillarás": "arrodillar",
    "arrodillaré": "arrodillar",
    "arrodillaréis": "arrodillar",
    "arrodillaría": "arrodillar",
    "arrodillaríais": "arrodillar",
    "arrodillaríamos": "arrodillar",
    "arrodillarían": "arrodillar",
    "arrodillarías": "arrodillar",
    "arrodillas": "arrodillar",
    "arrodillase": "arrodillar",
    "arrodillaseis": "arrodillar",
    "arrodillasen": "arrodillar",
    "arrodillases": "arrodillar",
    "arrodillaste": "arrodillar",
    "arrodillasteis": "arrodillar",
    "arrodille": "arrodillar",
    "arrodillemos": "arrodillar",
    "arrodillen": "arrodillar",
    "arrodilles": "arrodillar",
    "arrodillo": "arrodillar",
    "arrodillábamos": "arrodillar",
    "arrodilláis": "arrodillar",
    "arrodilláramos": "arrodillar",
    "arrodilláremos": "arrodillar",
    "arrodillásemos": "arrodillar",
    "arrodillé": "arrodillar",
    "arrodilléis": "arrodillar",
    "arrodilló": "arrodillar",
    "arrodriga": "arrodrigar",
    "arrodrigaba": "arrodrigar",
    "arrodrigabais": "arrodrigar",
    "arrodrigaban": "arrodrigar",
    "arrodrigabas": "arrodrigar",
    "arrodrigad": "arrodrigar",
    "arrodrigada": "arrodrigar",
    "arrodrigadas": "arrodrigar",
    "arrodrigado": "arrodrigar",
    "arrodrigados": "arrodrigar",
    "arrodrigamos": "arrodrigar",
    "arrodrigan": "arrodrigar",
    "arrodrigando": "arrodrigar",
    "arrodrigara": "arrodrigar",
    "arrodrigarais": "arrodrigar",
    "arrodrigaran": "arrodrigar",
    "arrodrigaras": "arrodrigar",
    "arrodrigare": "arrodrigar",
    "arrodrigareis": "arrodrigar",
    "arrodrigaremos": "arrodrigar",
    "arrodrigaren": "arrodrigar",
    "arrodrigares": "arrodrigar",
    "arrodrigaron": "arrodrigar",
    "arrodrigará": "arrodrigar",
    "arrodrigarán": "arrodrigar",
    "arrodrigarás": "arrodrigar",
    "arrodrigaré": "arrodrigar",
    "arrodrigaréis": "arrodrigar",
    "arrodrigaría": "arrodrigar",
    "arrodrigaríais": "arrodrigar",
    "arrodrigaríamos": "arrodrigar",
    "arrodrigarían": "arrodrigar",
    "arrodrigarías": "arrodrigar",
    "arrodrigas": "arrodrigar",
    "arrodrigase": "arrodrigar",
    "arrodrigaseis": "arrodrigar",
    "arrodrigasen": "arrodrigar",
    "arrodrigases": "arrodrigar",
    "arrodrigaste": "arrodrigar",
    "arrodrigasteis": "arrodrigar",
    "arrodrigo": "arrodrigar",
    "arrodrigona": "arrodrigonar",
    "arrodrigonaba": "arrodrigonar",
    "arrodrigonabais": "arrodrigonar",
    "arrodrigonaban": "arrodrigonar",
    "arrodrigonabas": "arrodrigonar",
    "arrodrigonad": "arrodrigonar",
    "arrodrigonada": "arrodrigonar",
    "arrodrigonadas": "arrodrigonar",
    "arrodrigonado": "arrodrigonar",
    "arrodrigonados": "arrodrigonar",
    "arrodrigonamos": "arrodrigonar",
    "arrodrigonan": "arrodrigonar",
    "arrodrigonando": "arrodrigonar",
    "arrodrigonara": "arrodrigonar",
    "arrodrigonarais": "arrodrigonar",
    "arrodrigonaran": "arrodrigonar",
    "arrodrigonaras": "arrodrigonar",
    "arrodrigonare": "arrodrigonar",
    "arrodrigonareis": "arrodrigonar",
    "arrodrigonaremos": "arrodrigonar",
    "arrodrigonaren": "arrodrigonar",
    "arrodrigonares": "arrodrigonar",
    "arrodrigonaron": "arrodrigonar",
    "arrodrigonará": "arrodrigonar",
    "arrodrigonarán": "arrodrigonar",
    "arrodrigonarás": "arrodrigonar",
    "arrodrigonaré": "arrodrigonar",
    "arrodrigonaréis": "arrodrigonar",
    "arrodrigonaría": "arrodrigonar",
    "arrodrigonaríais": "arrodrigonar",
    "arrodrigonaríamos": "arrodrigonar",
    "arrodrigonarían": "arrodrigonar",
    "arrodrigonarías": "arrodrigonar",
    "arrodrigonas": "arrodrigonar",
    "arrodrigonase": "arrodrigonar",
    "arrodrigonaseis": "arrodrigonar",
    "arrodrigonasen": "arrodrigonar",
    "arrodrigonases": "arrodrigonar",
    "arrodrigonaste": "arrodrigonar",
    "arrodrigonasteis": "arrodrigonar",
    "arrodrigone": "arrodrigonar",
    "arrodrigonemos": "arrodrigonar",
    "arrodrigonen": "arrodrigonar",
    "arrodrigones": "arrodrigonar",
    "arrodrigono": "arrodrigonar",
    "arrodrigonábamos": "arrodrigonar",
    "arrodrigonáis": "arrodrigonar",
    "arrodrigonáramos": "arrodrigonar",
    "arrodrigonáremos": "arrodrigonar",
    "arrodrigonásemos": "arrodrigonar",
    "arrodrigoné": "arrodrigonar",
    "arrodrigonéis": "arrodrigonar",
    "arrodrigonó": "arrodrigonar",
    "arrodrigue": "arrodrigar",
    "arrodriguemos": "arrodrigar",
    "arrodriguen": "arrodrigar",
    "arrodrigues": "arrodrigar",
    "arrodrigué": "arrodrigar",
    "arrodriguéis": "arrodrigar",
    "arrodrigábamos": "arrodrigar",
    "arrodrigáis": "arrodrigar",
    "arrodrigáramos": "arrodrigar",
    "arrodrigáremos": "arrodrigar",
    "arrodrigásemos": "arrodrigar",
    "arrodrigó": "arrodrigar",
    "arroga": "arrogar",
    "arrogaba": "arrogar",
    "arrogabais": "arrogar",
    "arrogaban": "arrogar",
    "arrogabas": "arrogar",
    "arrogaciones": "arrogación",
    "arrogad": "arrogar",
    "arrogada": "arrogar",
    "arrogadas": "arrogar",
    "arrogado": "arrogar",
    "arrogadora": "arrogador",
    "arrogadoras": "arrogador",
    "arrogadores": "arrogador",
    "arrogados": "arrogar",
    "arrogamos": "arrogar",
    "arrogan": "arrogar",
    "arrogancias": "arrogancia",
    "arrogando": "arrogar",
    "arrogantes": "arrogante",
    "arrogara": "arrogar",
    "arrogarais": "arrogar",
    "arrogaran": "arrogar",
    "arrogaras": "arrogar",
    "arrogare": "arrogar",
    "arrogareis": "arrogar",
    "arrogaremos": "arrogar",
    "arrogaren": "arrogar",
    "arrogares": "arrogar",
    "arrogaron": "arrogar",
    "arrogará": "arrogar",
    "arrogarán": "arrogar",
    "arrogarás": "arrogar",
    "arrogaré": "arrogar",
    "arrogaréis": "arrogar",
    "arrogaría": "arrogar",
    "arrogaríais": "arrogar",
    "arrogaríamos": "arrogar",
    "arrogarían": "arrogar",
    "arrogarías": "arrogar",
    "arrogas": "arrogar",
    "arrogase": "arrogar",
    "arrogaseis": "arrogar",
    "arrogasen": "arrogar",
    "arrogases": "arrogar",
    "arrogaste": "arrogar",
    "arrogasteis": "arrogar",
    "arrogo": "arrogar",
    "arrogue": "arrogar",
    "arroguemos": "arrogar",
    "arroguen": "arrogar",
    "arrogues": "arrogar",
    "arrogué": "arrogar",
    "arroguéis": "arrogar",
    "arrogábamos": "arrogar",
    "arrogáis": "arrogar",
    "arrogáramos": "arrogar",
    "arrogáremos": "arrogar",
    "arrogásemos": "arrogar",
    "arrogó": "arrogar",
    "arroja": "arrojar",
    "arrojaba": "arrojar",
    "arrojabais": "arrojar",
    "arrojaban": "arrojar",
    "arrojabas": "arrojar",
    "arrojad": "arrojar",
    "arrojada": "arrojar",
    "arrojadas": "arrojar",
    "arrojadiza": "arrojadizo",
    "arrojadizas": "arrojadizo",
    "arrojadizos": "arrojadizo",
    "arrojado": "arrojar",
    "arrojadora": "arrojador",
    "arrojadoras": "arrojador",
    "arrojadores": "arrojador",
    "arrojados": "arrojar",
    "arrojamos": "arrojar",
    "arrojan": "arrojar",
    "arrojando": "arrojar",
    "arrojara": "arrojar",
    "arrojarais": "arrojar",
    "arrojaran": "arrojar",
    "arrojaras": "arrojar",
    "arrojare": "arrojar",
    "arrojareis": "arrojar",
    "arrojaremos": "arrojar",
    "arrojaren": "arrojar",
    "arrojares": "arrojar",
    "arrojaron": "arrojar",
    "arrojará": "arrojar",
    "arrojarán": "arrojar",
    "arrojarás": "arrojar",
    "arrojaré": "arrojar",
    "arrojaréis": "arrojar",
    "arrojaría": "arrojar",
    "arrojaríais": "arrojar",
    "arrojaríamos": "arrojar",
    "arrojarían": "arrojar",
    "arrojarías": "arrojar",
    "arrojas": "arrojar",
    "arrojase": "arrojar",
    "arrojaseis": "arrojar",
    "arrojasen": "arrojar",
    "arrojases": "arrojar",
    "arrojaste": "arrojar",
    "arrojasteis": "arrojar",
    "arroje": "arrojar",
    "arrojemos": "arrojar",
    "arrojen": "arrojar",
    "arrojes": "arrojar",
    "arrojo": "arrojar",
    "arrojos": "arrojo",
    "arrojábamos": "arrojar",
    "arrojáis": "arrojar",
    "arrojáramos": "arrojar",
    "arrojáremos": "arrojar",
    "arrojásemos": "arrojar",
    "arrojé": "arrojar",
    "arrojéis": "arrojar",
    "arrojó": "arrojar",
    "arrolla": "arrollar",
    "arrollaba": "arrollar",
    "arrollabais": "arrollar",
    "arrollaban": "arrollar",
    "arrollabas": "arrollar",
    "arrollad": "arrollar",
    "arrollada": "arrollar",
    "arrolladas": "arrollar",
    "arrollado": "arrollar",
    "arrolladora": "arrollador",
    "arrolladoras": "arrollador",
    "arrolladores": "arrollador",
    "arrollados": "arrollar",
    "arrollamientos": "arrollamiento",
    "arrollamos": "arrollar",
    "arrollan": "arrollar",
    "arrollando": "arrollar",
    "arrollara": "arrollar",
    "arrollarais": "arrollar",
    "arrollaran": "arrollar",
    "arrollaras": "arrollar",
    "arrollare": "arrollar",
    "arrollareis": "arrollar",
    "arrollaremos": "arrollar",
    "arrollaren": "arrollar",
    "arrollares": "arrollar",
    "arrollaron": "arrollar",
    "arrollará": "arrollar",
    "arrollarán": "arrollar",
    "arrollarás": "arrollar",
    "arrollaré": "arrollar",
    "arrollaréis": "arrollar",
    "arrollaría": "arrollar",
    "arrollaríais": "arrollar",
    "arrollaríamos": "arrollar",
    "arrollarían": "arrollar",
    "arrollarías": "arrollar",
    "arrollas": "arrollar",
    "arrollase": "arrollar",
    "arrollaseis": "arrollar",
    "arrollasen": "arrollar",
    "arrollases": "arrollar",
    "arrollaste": "arrollar",
    "arrollasteis": "arrollar",
    "arrolle": "arrollar",
    "arrollemos": "arrollar",
    "arrollen": "arrollar",
    "arrolles": "arrollar",
    "arrollo": "arrollar",
    "arrollábamos": "arrollar",
    "arrolláis": "arrollar",
    "arrolláramos": "arrollar",
    "arrolláremos": "arrollar",
    "arrollásemos": "arrollar",
    "arrollé": "arrollar",
    "arrolléis": "arrollar",
    "arrolló": "arrollar",
    "arroma": "arromar",
    "arromaba": "arromar",
    "arromabais": "arromar",
    "arromaban": "arromar",
    "arromabas": "arromar",
    "arromad": "arromar",
    "arromada": "arromar",
    "arromadas": "arromar",
    "arromado": "arromar",
    "arromados": "arromar",
    "arromamos": "arromar",
    "arroman": "arromar",
    "arromance": "arromanzar",
    "arromancemos": "arromanzar",
    "arromancen": "arromanzar",
    "arromances": "arromanzar",
    "arromancé": "arromanzar",
    "arromancéis": "arromanzar",
    "arromando": "arromar",
    "arromanza": "arromanzar",
    "arromanzaba": "arromanzar",
    "arromanzabais": "arromanzar",
    "arromanzaban": "arromanzar",
    "arromanzabas": "arromanzar",
    "arromanzad": "arromanzar",
    "arromanzada": "arromanzar",
    "arromanzadas": "arromanzar",
    "arromanzado": "arromanzar",
    "arromanzados": "arromanzar",
    "arromanzamos": "arromanzar",
    "arromanzan": "arromanzar",
    "arromanzando": "arromanzar",
    "arromanzara": "arromanzar",
    "arromanzarais": "arromanzar",
    "arromanzaran": "arromanzar",
    "arromanzaras": "arromanzar",
    "arromanzare": "arromanzar",
    "arromanzareis": "arromanzar",
    "arromanzaremos": "arromanzar",
    "arromanzaren": "arromanzar",
    "arromanzares": "arromanzar",
    "arromanzaron": "arromanzar",
    "arromanzará": "arromanzar",
    "arromanzarán": "arromanzar",
    "arromanzarás": "arromanzar",
    "arromanzaré": "arromanzar",
    "arromanzaréis": "arromanzar",
    "arromanzaría": "arromanzar",
    "arromanzaríais": "arromanzar",
    "arromanzaríamos": "arromanzar",
    "arromanzarían": "arromanzar",
    "arromanzarías": "arromanzar",
    "arromanzas": "arromanzar",
    "arromanzase": "arromanzar",
    "arromanzaseis": "arromanzar",
    "arromanzasen": "arromanzar",
    "arromanzases": "arromanzar",
    "arromanzaste": "arromanzar",
    "arromanzasteis": "arromanzar",
    "arromanzo": "arromanzar",
    "arromanzábamos": "arromanzar",
    "arromanzáis": "arromanzar",
    "arromanzáramos": "arromanzar",
    "arromanzáremos": "arromanzar",
    "arromanzásemos": "arromanzar",
    "arromanzó": "arromanzar",
    "arromara": "arromar",
    "arromarais": "arromar",
    "arromaran": "arromar",
    "arromaras": "arromar",
    "arromare": "arromar",
    "arromareis": "arromar",
    "arromaremos": "arromar",
    "arromaren": "arromar",
    "arromares": "arromar",
    "arromaron": "arromar",
    "arromará": "arromar",
    "arromarán": "arromar",
    "arromarás": "arromar",
    "arromaré": "arromar",
    "arromaréis": "arromar",
    "arromaría": "arromar",
    "arromaríais": "arromar",
    "arromaríamos": "arromar",
    "arromarían": "arromar",
    "arromarías": "arromar",
    "arromas": "arromar",
    "arromase": "arromar",
    "arromaseis": "arromar",
    "arromasen": "arromar",
    "arromases": "arromar",
    "arromaste": "arromar",
    "arromasteis": "arromar",
    "arrome": "arromar",
    "arromemos": "arromar",
    "arromen": "arromar",
    "arromes": "arromar",
    "arromo": "arromar",
    "arrompa": "arromper",
    "arrompamos": "arromper",
    "arrompan": "arromper",
    "arrompas": "arromper",
    "arrompe": "arromper",
    "arromped": "arromper",
    "arrompemos": "arromper",
    "arrompen": "arromper",
    "arromperemos": "arromper",
    "arromperá": "arromper",
    "arromperán": "arromper",
    "arromperás": "arromper",
    "arromperé": "arromper",
    "arromperéis": "arromper",
    "arrompería": "arromper",
    "arromperíais": "arromper",
    "arromperíamos": "arromper",
    "arromperían": "arromper",
    "arromperías": "arromper",
    "arrompes": "arromper",
    "arrompida": "arromper",
    "arrompidas": "arromper",
    "arrompido": "arromper",
    "arrompidos": "arrompido",
    "arrompiendo": "arromper",
    "arrompiera": "arromper",
    "arrompierais": "arromper",
    "arrompieran": "arromper",
    "arrompieras": "arromper",
    "arrompiere": "arromper",
    "arrompiereis": "arromper",
    "arrompieren": "arromper",
    "arrompieres": "arromper",
    "arrompieron": "arromper",
    "arrompiese": "arromper",
    "arrompieseis": "arromper",
    "arrompiesen": "arromper",
    "arrompieses": "arromper",
    "arrompimos": "arromper",
    "arrompiste": "arromper",
    "arrompisteis": "arromper",
    "arrompiéramos": "arromper",
    "arrompiéremos": "arromper",
    "arrompiésemos": "arromper",
    "arrompió": "arromper",
    "arrompo": "arromper",
    "arrompáis": "arromper",
    "arrompéis": "arromper",
    "arrompí": "arromper",
    "arrompía": "arromper",
    "arrompíais": "arromper",
    "arrompíamos": "arromper",
    "arrompían": "arromper",
    "arrompías": "arromper",
    "arromábamos": "arromar",
    "arromáis": "arromar",
    "arromáramos": "arromar",
    "arromáremos": "arromar",
    "arromásemos": "arromar",
    "arromé": "arromar",
    "arroméis": "arromar",
    "arromó": "arromar",
    "arronce": "arronzar",
    "arroncemos": "arronzar",
    "arroncen": "arronzar",
    "arronces": "arronzar",
    "arroncé": "arronzar",
    "arroncéis": "arronzar",
    "arronza": "arronzar",
    "arronzaba": "arronzar",
    "arronzabais": "arronzar",
    "arronzaban": "arronzar",
    "arronzabas": "arronzar",
    "arronzad": "arronzar",
    "arronzada": "arronzar",
    "arronzadas": "arronzar",
    "arronzado": "arronzar",
    "arronzados": "arronzar",
    "arronzamos": "arronzar",
    "arronzan": "arronzar",
    "arronzando": "arronzar",
    "arronzara": "arronzar",
    "arronzarais": "arronzar",
    "arronzaran": "arronzar",
    "arronzaras": "arronzar",
    "arronzare": "arronzar",
    "arronzareis": "arronzar",
    "arronzaremos": "arronzar",
    "arronzaren": "arronzar",
    "arronzares": "arronzar",
    "arronzaron": "arronzar",
    "arronzará": "arronzar",
    "arronzarán": "arronzar",
    "arronzarás": "arronzar",
    "arronzaré": "arronzar",
    "arronzaréis": "arronzar",
    "arronzaría": "arronzar",
    "arronzaríais": "arronzar",
    "arronzaríamos": "arronzar",
    "arronzarían": "arronzar",
    "arronzarías": "arronzar",
    "arronzas": "arronzar",
    "arronzase": "arronzar",
    "arronzaseis": "arronzar",
    "arronzasen": "arronzar",
    "arronzases": "arronzar",
    "arronzaste": "arronzar",
    "arronzasteis": "arronzar",
    "arronzo": "arronzar",
    "arronzábamos": "arronzar",
    "arronzáis": "arronzar",
    "arronzáramos": "arronzar",
    "arronzáremos": "arronzar",
    "arronzásemos": "arronzar",
    "arronzó": "arronzar",
    "arropa": "arropar",
    "arropaba": "arropar",
    "arropabais": "arropar",
    "arropaban": "arropar",
    "arropabas": "arropar",
    "arropad": "arropar",
    "arropada": "arropar",
    "arropadas": "arropar",
    "arropado": "arropar",
    "arropadora": "arropador",
    "arropadoras": "arropador",
    "arropadores": "arropador",
    "arropados": "arropar",
    "arropamientos": "arropamiento",
    "arropamos": "arropar",
    "arropan": "arropar",
    "arropando": "arropar",
    "arropara": "arropar",
    "arroparais": "arropar",
    "arroparan": "arropar",
    "arroparas": "arropar",
    "arropare": "arropar",
    "arropareis": "arropar",
    "arroparemos": "arropar",
    "arroparen": "arropar",
    "arropares": "arropar",
    "arroparon": "arropar",
    "arropará": "arropar",
    "arroparán": "arropar",
    "arroparás": "arropar",
    "arroparé": "arropar",
    "arroparéis": "arropar",
    "arroparía": "arropar",
    "arroparíais": "arropar",
    "arroparíamos": "arropar",
    "arroparían": "arropar",
    "arroparías": "arropar",
    "arropas": "arropar",
    "arropase": "arropar",
    "arropaseis": "arropar",
    "arropasen": "arropar",
    "arropases": "arropar",
    "arropaste": "arropar",
    "arropasteis": "arropar",
    "arrope": "arropar",
    "arropemos": "arropar",
    "arropen": "arropar",
    "arroperas": "arropera",
    "arropes": "arrope",
    "arropo": "arropar",
    "arropábamos": "arropar",
    "arropáis": "arropar",
    "arropáramos": "arropar",
    "arropáremos": "arropar",
    "arropásemos": "arropar",
    "arropé": "arropar",
    "arropéis": "arropar",
    "arropó": "arropar",
    "arrostra": "arrostrar",
    "arrostraba": "arrostrar",
    "arrostrabais": "arrostrar",
    "arrostraban": "arrostrar",
    "arrostrabas": "arrostrar",
    "arrostrad": "arrostrar",
    "arrostrada": "arrostrar",
    "arrostradas": "arrostrar",
    "arrostrado": "arrostrar",
    "arrostrados": "arrostrar",
    "arrostramos": "arrostrar",
    "arrostran": "arrostrar",
    "arrostrando": "arrostrar",
    "arrostrara": "arrostrar",
    "arrostrarais": "arrostrar",
    "arrostraran": "arrostrar",
    "arrostraras": "arrostrar",
    "arrostrare": "arrostrar",
    "arrostrareis": "arrostrar",
    "arrostraremos": "arrostrar",
    "arrostraren": "arrostrar",
    "arrostrares": "arrostrar",
    "arrostraron": "arrostrar",
    "arrostrará": "arrostrar",
    "arrostrarán": "arrostrar",
    "arrostrarás": "arrostrar",
    "arrostraré": "arrostrar",
    "arrostraréis": "arrostrar",
    "arrostraría": "arrostrar",
    "arrostraríais": "arrostrar",
    "arrostraríamos": "arrostrar",
    "arrostrarían": "arrostrar",
    "arrostrarías": "arrostrar",
    "arrostras": "arrostrar",
    "arrostrase": "arrostrar",
    "arrostraseis": "arrostrar",
    "arrostrasen": "arrostrar",
    "arrostrases": "arrostrar",
    "arrostraste": "arrostrar",
    "arrostrasteis": "arrostrar",
    "arrostre": "arrostrar",
    "arrostremos": "arrostrar",
    "arrostren": "arrostrar",
    "arrostres": "arrostrar",
    "arrostro": "arrostrar",
    "arrostrábamos": "arrostrar",
    "arrostráis": "arrostrar",
    "arrostráramos": "arrostrar",
    "arrostráremos": "arrostrar",
    "arrostrásemos": "arrostrar",
    "arrostré": "arrostrar",
    "arrostréis": "arrostrar",
    "arrostró": "arrostrar",
    "arrota": "arromper",
    "arrotas": "arromper",
    "arroto": "arromper",
    "arrotos": "arromper",
    "arroya": "arroyar",
    "arroyaba": "arroyar",
    "arroyabais": "arroyar",
    "arroyaban": "arroyar",
    "arroyabas": "arroyar",
    "arroyad": "arroyar",
    "arroyada": "arroyar",
    "arroyadas": "arroyar",
    "arroyado": "arroyar",
    "arroyados": "arroyar",
    "arroyamientos": "arroyamiento",
    "arroyamos": "arroyar",
    "arroyan": "arroyar",
    "arroyando": "arroyar",
    "arroyara": "arroyar",
    "arroyarais": "arroyar",
    "arroyaran": "arroyar",
    "arroyaras": "arroyar",
    "arroyare": "arroyar",
    "arroyareis": "arroyar",
    "arroyaremos": "arroyar",
    "arroyaren": "arroyar",
    "arroyares": "arroyar",
    "arroyaron": "arroyar",
    "arroyará": "arroyar",
    "arroyarán": "arroyar",
    "arroyarás": "arroyar",
    "arroyaré": "arroyar",
    "arroyaréis": "arroyar",
    "arroyaría": "arroyar",
    "arroyaríais": "arroyar",
    "arroyaríamos": "arroyar",
    "arroyarían": "arroyar",
    "arroyarías": "arroyar",
    "arroyas": "arroyar",
    "arroyase": "arroyar",
    "arroyaseis": "arroyar",
    "arroyasen": "arroyar",
    "arroyases": "arroyar",
    "arroyaste": "arroyar",
    "arroyasteis": "arroyar",
    "arroye": "arroyar",
    "arroyemos": "arroyar",
    "arroyen": "arroyar",
    "arroyes": "arroyar",
    "arroyo": "arroyar",
    "arroyos": "arroyo",
    "arroyuelos": "arroyuelo",
    "arroyábamos": "arroyar",
    "arroyáis": "arroyar",
    "arroyáramos": "arroyar",
    "arroyáremos": "arroyar",
    "arroyásemos": "arroyar",
    "arroyé": "arroyar",
    "arroyéis": "arroyar",
    "arroyó": "arroyar",
    "arrozales": "arrozal",
    "arruaba": "arruar",
    "arruabais": "arruar",
    "arruaban": "arruar",
    "arruabas": "arruar",
    "arruad": "arruar",
    "arruada": "arruar",
    "arruadas": "arruar",
    "arruado": "arruar",
    "arruados": "arruar",
    "arruamos": "arruar",
    "arruando": "arruar",
    "arruara": "arruar",
    "arruarais": "arruar",
    "arruaran": "arruar",
    "arruaras": "arruar",
    "arruare": "arruar",
    "arruareis": "arruar",
    "arruaremos": "arruar",
    "arruaren": "arruar",
    "arruares": "arruar",
    "arruaron": "arruar",
    "arruará": "arruar",
    "arruarán": "arruar",
    "arruarás": "arruar",
    "arruaré": "arruar",
    "arruaréis": "arruar",
    "arruaría": "arruar",
    "arruaríais": "arruar",
    "arruaríamos": "arruar",
    "arruarían": "arruar",
    "arruarías": "arruar",
    "arruase": "arruar",
    "arruaseis": "arruar",
    "arruasen": "arruar",
    "arruases": "arruar",
    "arruaste": "arruar",
    "arruasteis": "arruar",
    "arruemos": "arruar",
    "arrufa": "arrufar",
    "arrufaba": "arrufar",
    "arrufabais": "arrufar",
    "arrufaban": "arrufar",
    "arrufabas": "arrufar",
    "arrufad": "arrufar",
    "arrufada": "arrufar",
    "arrufadas": "arrufar",
    "arrufado": "arrufar",
    "arrufados": "arrufar",
    "arrufamos": "arrufar",
    "arrufan": "arrufar",
    "arrufando": "arrufar",
    "arrufara": "arrufar",
    "arrufarais": "arrufar",
    "arrufaran": "arrufar",
    "arrufaras": "arrufar",
    "arrufare": "arrufar",
    "arrufareis": "arrufar",
    "arrufaremos": "arrufar",
    "arrufaren": "arrufar",
    "arrufares": "arrufar",
    "arrufaron": "arrufar",
    "arrufará": "arrufar",
    "arrufarán": "arrufar",
    "arrufarás": "arrufar",
    "arrufaré": "arrufar",
    "arrufaréis": "arrufar",
    "arrufaría": "arrufar",
    "arrufaríais": "arrufar",
    "arrufaríamos": "arrufar",
    "arrufarían": "arrufar",
    "arrufarías": "arrufar",
    "arrufas": "arrufar",
    "arrufase": "arrufar",
    "arrufaseis": "arrufar",
    "arrufasen": "arrufar",
    "arrufases": "arrufar",
    "arrufaste": "arrufar",
    "arrufasteis": "arrufar",
    "arrufe": "arrufar",
    "arrufemos": "arrufar",
    "arrufen": "arrufar",
    "arrufes": "arrufar",
    "arrufianada": "arrufianado",
    "arrufianadas": "arrufianado",
    "arrufianados": "arrufianado",
    "arrufo": "arrufar",
    "arrufos": "arrufo",
    "arrufábamos": "arrufar",
    "arrufáis": "arrufar",
    "arrufáramos": "arrufar",
    "arrufáremos": "arrufar",
    "arrufásemos": "arrufar",
    "arrufé": "arrufar",
    "arruféis": "arrufar",
    "arrufó": "arrufar",
    "arruga": "arrugar",
    "arrugaba": "arrugar",
    "arrugabais": "arrugar",
    "arrugaban": "arrugar",
    "arrugabas": "arrugar",
    "arrugables": "arrugable",
    "arrugad": "arrugar",
    "arrugada": "arrugar",
    "arrugadas": "arrugar",
    "arrugado": "arrugar",
    "arrugados": "arrugar",
    "arrugamientos": "arrugamiento",
    "arrugamos": "arrugar",
    "arrugan": "arrugar",
    "arrugando": "arrugar",
    "arrugara": "arrugar",
    "arrugarais": "arrugar",
    "arrugaran": "arrugar",
    "arrugaras": "arrugar",
    "arrugare": "arrugar",
    "arrugareis": "arrugar",
    "arrugaremos": "arrugar",
    "arrugaren": "arrugar",
    "arrugares": "arrugar",
    "arrugaron": "arrugar",
    "arrugará": "arrugar",
    "arrugarán": "arrugar",
    "arrugarás": "arrugar",
    "arrugaré": "arrugar",
    "arrugaréis": "arrugar",
    "arrugaría": "arrugar",
    "arrugaríais": "arrugar",
    "arrugaríamos": "arrugar",
    "arrugarían": "arrugar",
    "arrugarías": "arrugar",
    "arrugas": "arrugar",
    "arrugase": "arrugar",
    "arrugaseis": "arrugar",
    "arrugasen": "arrugar",
    "arrugases": "arrugar",
    "arrugaste": "arrugar",
    "arrugasteis": "arrugar",
    "arrugo": "arrugar",
    "arrugue": "arrugar",
    "arruguemos": "arrugar",
    "arruguen": "arrugar",
    "arrugues": "arrugar",
    "arruguillas": "arruguilla",
    "arrugué": "arrugar",
    "arruguéis": "arrugar",
    "arrugábamos": "arrugar",
    "arrugáis": "arrugar",
    "arrugáramos": "arrugar",
    "arrugáremos": "arrugar",
    "arrugásemos": "arrugar",
    "arrugó": "arrugar",
    "arruina": "arruinar",
    "arruinaba": "arruinar",
    "arruinabais": "arruinar",
    "arruinaban": "arruinar",
    "arruinabas": "arruinar",
    "arruinad": "arruinar",
    "arruinada": "arruinar",
    "arruinadas": "arruinar",
    "arruinado": "arruinar",
    "arruinados": "arruinar",
    "arruinamientos": "arruinamiento",
    "arruinamos": "arruinar",
    "arruinan": "arruinar",
    "arruinando": "arruinar",
    "arruinara": "arruinar",
    "arruinarais": "arruinar",
    "arruinaran": "arruinar",
    "arruinaras": "arruinar",
    "arruinare": "arruinar",
    "arruinareis": "arruinar",
    "arruinaremos": "arruinar",
    "arruinaren": "arruinar",
    "arruinares": "arruinar",
    "arruinaron": "arruinar",
    "arruinará": "arruinar",
    "arruinarán": "arruinar",
    "arruinarás": "arruinar",
    "arruinaré": "arruinar",
    "arruinaréis": "arruinar",
    "arruinaría": "arruinar",
    "arruinaríais": "arruinar",
    "arruinaríamos": "arruinar",
    "arruinarían": "arruinar",
    "arruinarías": "arruinar",
    "arruinas": "arruinar",
    "arruinase": "arruinar",
    "arruinaseis": "arruinar",
    "arruinasen": "arruinar",
    "arruinases": "arruinar",
    "arruinaste": "arruinar",
    "arruinasteis": "arruinar",
    "arruine": "arruinar",
    "arruinemos": "arruinar",
    "arruinen": "arruinar",
    "arruines": "arruinar",
    "arruino": "arruinar",
    "arruinábamos": "arruinar",
    "arruináis": "arruinar",
    "arruináramos": "arruinar",
    "arruináremos": "arruinar",
    "arruinásemos": "arruinar",
    "arruiné": "arruinar",
    "arruinéis": "arruinar",
    "arruinó": "arruinar",
    "arrulla": "arrullar",
    "arrullaba": "arrullar",
    "arrullabais": "arrullar",
    "arrullaban": "arrullar",
    "arrullabas": "arrullar",
    "arrullad": "arrullar",
    "arrullada": "arrullar",
    "arrulladas": "arrullar",
    "arrullado": "arrullar",
    "arrulladora": "arrullador",
    "arrulladoras": "arrullador",
    "arrulladores": "arrullador",
    "arrullados": "arrullar",
    "arrullamos": "arrullar",
    "arrullan": "arrullar",
    "arrullando": "arrullar",
    "arrullara": "arrullar",
    "arrullarais": "arrullar",
    "arrullaran": "arrullar",
    "arrullaras": "arrullar",
    "arrullare": "arrullar",
    "arrullareis": "arrullar",
    "arrullaremos": "arrullar",
    "arrullaren": "arrullar",
    "arrullares": "arrullar",
    "arrullaron": "arrullar",
    "arrullará": "arrullar",
    "arrullarán": "arrullar",
    "arrullarás": "arrullar",
    "arrullaré": "arrullar",
    "arrullaréis": "arrullar",
    "arrullaría": "arrullar",
    "arrullaríais": "arrullar",
    "arrullaríamos": "arrullar",
    "arrullarían": "arrullar",
    "arrullarías": "arrullar",
    "arrullas": "arrullar",
    "arrullase": "arrullar",
    "arrullaseis": "arrullar",
    "arrullasen": "arrullar",
    "arrullases": "arrullar",
    "arrullaste": "arrullar",
    "arrullasteis": "arrullar",
    "arrulle": "arrullar",
    "arrullemos": "arrullar",
    "arrullen": "arrullar",
    "arrulles": "arrullar",
    "arrullo": "arrullar",
    "arrullos": "arrullo",
    "arrullábamos": "arrullar",
    "arrulláis": "arrullar",
    "arrulláramos": "arrullar",
    "arrulláremos": "arrullar",
    "arrullásemos": "arrullar",
    "arrullé": "arrullar",
    "arrulléis": "arrullar",
    "arrulló": "arrullar",
    "arruma": "arrumar",
    "arrumaba": "arrumar",
    "arrumabais": "arrumar",
    "arrumaban": "arrumar",
    "arrumabas": "arrumar",
    "arrumacos": "arrumaco",
    "arrumad": "arrumar",
    "arrumada": "arrumar",
    "arrumadas": "arrumar",
    "arrumado": "arrumar",
    "arrumados": "arrumar",
    "arrumamos": "arrumar",
    "arruman": "arrumar",
    "arrumando": "arrumar",
    "arrumara": "arrumar",
    "arrumarais": "arrumar",
    "arrumaran": "arrumar",
    "arrumaras": "arrumar",
    "arrumare": "arrumar",
    "arrumareis": "arrumar",
    "arrumaremos": "arrumar",
    "arrumaren": "arrumar",
    "arrumares": "arrumar",
    "arrumaron": "arrumar",
    "arrumará": "arrumar",
    "arrumarán": "arrumar",
    "arrumarás": "arrumar",
    "arrumaré": "arrumar",
    "arrumaréis": "arrumar",
    "arrumaría": "arrumar",
    "arrumaríais": "arrumar",
    "arrumaríamos": "arrumar",
    "arrumarían": "arrumar",
    "arrumarías": "arrumar",
    "arrumas": "arrumar",
    "arrumase": "arrumar",
    "arrumaseis": "arrumar",
    "arrumasen": "arrumar",
    "arrumases": "arrumar",
    "arrumaste": "arrumar",
    "arrumasteis": "arrumar",
    "arrumazones": "arrumazón",
    "arrumba": "arrumbar",
    "arrumbaba": "arrumbar",
    "arrumbabais": "arrumbar",
    "arrumbaban": "arrumbar",
    "arrumbabas": "arrumbar",
    "arrumbaciones": "arrumbación",
    "arrumbad": "arrumbar",
    "arrumbada": "arrumbar",
    "arrumbadas": "arrumbar",
    "arrumbado": "arrumbar",
    "arrumbadora": "arrumbador",
    "arrumbadoras": "arrumbador",
    "arrumbadores": "arrumbador",
    "arrumbados": "arrumbar",
    "arrumbamientos": "arrumbamiento",
    "arrumbamos": "arrumbar",
    "arrumban": "arrumbar",
    "arrumbando": "arrumbar",
    "arrumbara": "arrumbar",
    "arrumbarais": "arrumbar",
    "arrumbaran": "arrumbar",
    "arrumbaras": "arrumbar",
    "arrumbare": "arrumbar",
    "arrumbareis": "arrumbar",
    "arrumbaremos": "arrumbar",
    "arrumbaren": "arrumbar",
    "arrumbares": "arrumbar",
    "arrumbaron": "arrumbar",
    "arrumbará": "arrumbar",
    "arrumbarán": "arrumbar",
    "arrumbarás": "arrumbar",
    "arrumbaré": "arrumbar",
    "arrumbaréis": "arrumbar",
    "arrumbaría": "arrumbar",
    "arrumbaríais": "arrumbar",
    "arrumbaríamos": "arrumbar",
    "arrumbarían": "arrumbar",
    "arrumbarías": "arrumbar",
    "arrumbas": "arrumbar",
    "arrumbase": "arrumbar",
    "arrumbaseis": "arrumbar",
    "arrumbasen": "arrumbar",
    "arrumbases": "arrumbar",
    "arrumbaste": "arrumbar",
    "arrumbasteis": "arrumbar",
    "arrumbe": "arrumbar",
    "arrumbemos": "arrumbar",
    "arrumben": "arrumbar",
    "arrumbes": "arrumbar",
    "arrumbo": "arrumbar",
    "arrumbábamos": "arrumbar",
    "arrumbáis": "arrumbar",
    "arrumbáramos": "arrumbar",
    "arrumbáremos": "arrumbar",
    "arrumbásemos": "arrumbar",
    "arrumbé": "arrumbar",
    "arrumbéis": "arrumbar",
    "arrumbó": "arrumbar",
    "arrume": "arrumar",
    "arrumemos": "arrumar",
    "arrumen": "arrumar",
    "arrumes": "arrumar",
    "arrumo": "arrumar",
    "arrumábamos": "arrumar",
    "arrumáis": "arrumar",
    "arrumáramos": "arrumar",
    "arrumáremos": "arrumar",
    "arrumásemos": "arrumar",
    "arrumé": "arrumar",
    "arruméis": "arrumar",
    "arrumó": "arrumar",
    "arrurruces": "arrurruz",
    "arruruces": "arruruz",
    "arruábamos": "arruar",
    "arruáis": "arruar",
    "arruáramos": "arruar",
    "arruáremos": "arruar",
    "arruásemos": "arruar",
    "arrué": "arruar",
    "arruéis": "arruar",
    "arruó": "arruar",
    "arráeces": "arráez",
    "arría": "arriar",
    "arrían": "arriar",
    "arrías": "arriar",
    "arríe": "arriar",
    "arríen": "arriar",
    "arríes": "arriar",
    "arrío": "arriar",
    "arrítmica": "arrítmico",
    "arrítmicas": "arrítmico",
    "arrítmicos": "arrítmico",
    "arrúa": "arruar",
    "arrúan": "arruar",
    "arrúas": "arruar",
    "arrúe": "arruar",
    "arrúen": "arruar",
    "arrúes": "arruar",
    "arrúo": "arruar",
    "arsanilatos": "arsanilato",
    "arsanílica": "arsanílico",
    "arsanílicas": "arsanílico",
    "arsanílicos": "arsanílico",
    "arsenales": "arsenal",
    "arseniatos": "arseniato",
    "arsenicales": "arsenical",
    "arsenitos": "arsenito",
    "arseniuros": "arseniuro",
    "arsenopiritas": "arsenopirita",
    "arsfenaminas": "arsfenamina",
    "arsinas": "arsina",
    "arsácidas": "arsácida",
    "arsénicos": "arsénico",
    "art": "artículo",
    "artefactos": "artefacto",
    "artejos": "artejo",
    "arteles": "artel",
    "artemisas": "artemisa",
    "artera": "artero",
    "arteras": "artero",
    "arteriales": "arterial",
    "arterias": "arteria",
    "arteriolas": "arteriola",
    "arteriosa": "arterioso",
    "arteriosas": "arterioso",
    "arteriosclerótica": "arteriosclerótico",
    "arterioscleróticas": "arteriosclerótico",
    "arterioscleróticos": "arteriosclerótico",
    "arteriosos": "arterioso",
    "arteriovenosa": "arteriovenoso",
    "arteriovenosas": "arteriovenoso",
    "arteriovenosos": "arteriovenoso",
    "arteros": "artero",
    "artes": "arte",
    "artesana": "artesano",
    "artesanados": "artesanado",
    "artesanales": "artesanal",
    "artesanas": "artesano",
    "artesanos": "artesano",
    "artesanías": "artesanía",
    "artesas": "artesa",
    "artesiana": "artesiano",
    "artesianas": "artesiano",
    "artesianos": "artesiano",
    "artesona": "artesonar",
    "artesonaba": "artesonar",
    "artesonabais": "artesonar",
    "artesonaban": "artesonar",
    "artesonabas": "artesonar",
    "artesonad": "artesonar",
    "artesonada": "artesonar",
    "artesonadas": "artesonar",
    "artesonado": "artesonar",
    "artesonados": "artesonar",
    "artesonamos": "artesonar",
    "artesonan": "artesonar",
    "artesonando": "artesonar",
    "artesonara": "artesonar",
    "artesonarais": "artesonar",
    "artesonaran": "artesonar",
    "artesonaras": "artesonar",
    "artesonare": "artesonar",
    "artesonareis": "artesonar",
    "artesonaremos": "artesonar",
    "artesonaren": "artesonar",
    "artesonares": "artesonar",
    "artesonaron": "artesonar",
    "artesonará": "artesonar",
    "artesonarán": "artesonar",
    "artesonarás": "artesonar",
    "artesonaré": "artesonar",
    "artesonaréis": "artesonar",
    "artesonaría": "artesonar",
    "artesonaríais": "artesonar",
    "artesonaríamos": "artesonar",
    "artesonarían": "artesonar",
    "artesonarías": "artesonar",
    "artesonas": "artesonar",
    "artesonase": "artesonar",
    "artesonaseis": "artesonar",
    "artesonasen": "artesonar",
    "artesonases": "artesonar",
    "artesonaste": "artesonar",
    "artesonasteis": "artesonar",
    "artesone": "artesonar",
    "artesonemos": "artesonar",
    "artesonen": "artesonar",
    "artesones": "artesonar",
    "artesono": "artesonar",
    "artesonábamos": "artesonar",
    "artesonáis": "artesonar",
    "artesonáramos": "artesonar",
    "artesonáremos": "artesonar",
    "artesonásemos": "artesonar",
    "artesoné": "artesonar",
    "artesonéis": "artesonar",
    "artesonó": "artesonar",
    "articula": "articular",
    "articulaba": "articular",
    "articulabais": "articular",
    "articulaban": "articular",
    "articulabas": "articular",
    "articulaciones": "articulación",
    "articulad": "articular",
    "articulada": "articular",
    "articuladas": "articular",
    "articulado": "articular",
    "articuladora": "articulador",
    "articuladoras": "articulador",
    "articuladores": "articulador",
    "articulados": "articular",
    "articulamos": "articular",
    "articulan": "articular",
    "articulando": "articular",
    "articulara": "articular",
    "articularais": "articular",
    "articularan": "articular",
    "articularas": "articular",
    "articulare": "articular",
    "articulareis": "articular",
    "articularemos": "articular",
    "articularen": "articular",
    "articulares": "articular",
    "articularon": "articular",
    "articulará": "articular",
    "articularán": "articular",
    "articularás": "articular",
    "articularé": "articular",
    "articularéis": "articular",
    "articularía": "articular",
    "articularíais": "articular",
    "articularíamos": "articular",
    "articularían": "articular",
    "articularías": "articular",
    "articulas": "articular",
    "articulase": "articular",
    "articulaseis": "articular",
    "articulasen": "articular",
    "articulases": "articular",
    "articulaste": "articular",
    "articulasteis": "articular",
    "articulatoria": "articulatorio",
    "articulatorias": "articulatorio",
    "articulatorios": "articulatorio",
    "articule": "articular",
    "articulemos": "articular",
    "articulen": "articular",
    "articules": "articular",
    "articulistas": "articulista",
    "articulo": "articular",
    "articulábamos": "articular",
    "articuláis": "articular",
    "articuláramos": "articular",
    "articuláremos": "articular",
    "articulásemos": "articular",
    "articulé": "articular",
    "articuléis": "articular",
    "articuló": "articular",
    "artificera": "artificero",
    "artificeras": "artificero",
    "artificeros": "artificero",
    "artificiales": "artificial",
    "artificialidades": "artificialidad",
    "artificiera": "artificiero",
    "artificieras": "artificiero",
    "artificieros": "artificiero",
    "artificios": "artificio",
    "artificiosa": "artificioso",
    "artificiosas": "artificioso",
    "artificiosidades": "artificiosidad",
    "artificiosos": "artificioso",
    "artiga": "artigar",
    "artigaba": "artigar",
    "artigabais": "artigar",
    "artigaban": "artigar",
    "artigabas": "artigar",
    "artigad": "artigar",
    "artigada": "artigar",
    "artigadas": "artigar",
    "artigado": "artigar",
    "artigados": "artigar",
    "artigamos": "artigar",
    "artigan": "artigar",
    "artigando": "artigar",
    "artigara": "artigar",
    "artigarais": "artigar",
    "artigaran": "artigar",
    "artigaras": "artigar",
    "artigare": "artigar",
    "artigareis": "artigar",
    "artigaremos": "artigar",
    "artigaren": "artigar",
    "artigares": "artigar",
    "artigaron": "artigar",
    "artigará": "artigar",
    "artigarán": "artigar",
    "artigarás": "artigar",
    "artigaré": "artigar",
    "artigaréis": "artigar",
    "artigaría": "artigar",
    "artigaríais": "artigar",
    "artigaríamos": "artigar",
    "artigarían": "artigar",
    "artigarías": "artigar",
    "artigas": "artigar",
    "artigase": "artigar",
    "artigaseis": "artigar",
    "artigasen": "artigar",
    "artigases": "artigar",
    "artigaste": "artigar",
    "artigasteis": "artigar",
    "artigo": "artigar",
    "artigue": "artigar",
    "artiguemos": "artigar",
    "artiguen": "artigar",
    "artigues": "artigar",
    "artigué": "artigar",
    "artiguéis": "artigar",
    "artigábamos": "artigar",
    "artigáis": "artigar",
    "artigáramos": "artigar",
    "artigáremos": "artigar",
    "artigásemos": "artigar",
    "artigó": "artigar",
    "artilla": "artillar",
    "artillaba": "artillar",
    "artillabais": "artillar",
    "artillaban": "artillar",
    "artillabas": "artillar",
    "artillad": "artillar",
    "artillada": "artillar",
    "artilladas": "artillar",
    "artillado": "artillar",
    "artillados": "artillar",
    "artillamos": "artillar",
    "artillan": "artillar",
    "artillando": "artillar",
    "artillara": "artillar",
    "artillarais": "artillar",
    "artillaran": "artillar",
    "artillaras": "artillar",
    "artillare": "artillar",
    "artillareis": "artillar",
    "artillaremos": "artillar",
    "artillaren": "artillar",
    "artillares": "artillar",
    "artillaron": "artillar",
    "artillará": "artillar",
    "artillarán": "artillar",
    "artillarás": "artillar",
    "artillaré": "artillar",
    "artillaréis": "artillar",
    "artillaría": "artillar",
    "artillaríais": "artillar",
    "artillaríamos": "artillar",
    "artillarían": "artillar",
    "artillarías": "artillar",
    "artillas": "artillar",
    "artillase": "artillar",
    "artillaseis": "artillar",
    "artillasen": "artillar",
    "artillases": "artillar",
    "artillaste": "artillar",
    "artillasteis": "artillar",
    "artille": "artillar",
    "artillemos": "artillar",
    "artillen": "artillar",
    "artillera": "artillero",
    "artilleras": "artillero",
    "artilleros": "artillero",
    "artillerías": "artillería",
    "artilles": "artillar",
    "artillo": "artillar",
    "artillábamos": "artillar",
    "artilláis": "artillar",
    "artilláramos": "artillar",
    "artilláremos": "artillar",
    "artillásemos": "artillar",
    "artillé": "artillar",
    "artilléis": "artillar",
    "artilló": "artillar",
    "artilugios": "artilugio",
    "artimañas": "artimaña",
    "artiniana": "artiniano",
    "artinianas": "artiniano",
    "artinianos": "artiniano",
    "artiodáctilos": "artiodáctilo",
    "artistas": "artista",
    "artoplastias": "artoplastia",
    "artralgias": "artralgia",
    "artritismos": "artritismo",
    "artrodias": "artrodia",
    "artrografías": "artrografía",
    "artrologías": "artrología",
    "artropatías": "artropatía",
    "artroplastias": "artroplastia",
    "artroscopias": "artroscopia",
    "artrotomías": "artrotomía",
    "artrítica": "artrítico",
    "artríticas": "artrítico",
    "artríticos": "artrítico",
    "artrópodos": "artrópodo",
    "artículos": "artículo",
    "artífices": "artífice",
    "artística": "artístico",
    "artísticas": "artístico",
    "artísticos": "artístico",
    "artúrica": "artúrico",
    "artúricas": "artúrico",
    "artúricos": "artúrico",
    "aruntas": "arunta",
    "arvejas": "arveja",
    "arvenses": "arvense",
    "arvicultora": "arvicultor",
    "arvicultoras": "arvicultor",
    "arvicultores": "arvicultor",
    "arviculturas": "arvicultura",
    "arxivística": "arxivístico",
    "arxivísticas": "arxivístico",
    "arxivísticos": "arxivístico",
    "arzobispados": "arzobispado",
    "arzobispales": "arzobispal",
    "arzobispos": "arzobispo",
    "arzones": "arzón",
    "arábamos": "arar",
    "arábica": "arábico",
    "arábicas": "arábico",
    "arábicos": "arábico",
    "arábiga": "arábigo",
    "arábigas": "arábigo",
    "arábigos": "arábigo",
    "aráceas": "arácea",
    "arácnidos": "arácnido",
    "aráis": "arar",
    "arándanos": "arándano",
    "aráquidos": "aráquido",
    "aráramos": "arar",
    "aráremos": "arar",
    "arásemos": "arar",
    "aré": "arar",
    "aréis": "arar",
    "arénica": "arénico",
    "arénicas": "arénico",
    "arénicos": "arénico",
    "aréolas": "aréola",
    "arévacos": "arévaco",
    "aró": "arar",
    "arúspices": "arúspice",
    "asa": "asar",
    "asaba": "asar",
    "asabais": "asar",
    "asaban": "asar",
    "asabas": "asar",
    "asaces": "asaz",
    "asad": "asar",
    "asada": "asar",
    "asadas": "asar",
    "asadera": "asadero",
    "asaderas": "asadero",
    "asaderos": "asadero",
    "asado": "asar",
    "asadores": "asador",
    "asados": "asar",
    "asaduras": "asadura",
    "asaeta": "asaetar",
    "asaetaba": "asaetar",
    "asaetabais": "asaetar",
    "asaetaban": "asaetar",
    "asaetabas": "asaetar",
    "asaetad": "asaetar",
    "asaetada": "asaetar",
    "asaetadas": "asaetar",
    "asaetado": "asaetar",
    "asaetados": "asaetar",
    "asaetamos": "asaetar",
    "asaetan": "asaetar",
    "asaetando": "asaetar",
    "asaetara": "asaetar",
    "asaetarais": "asaetar",
    "asaetaran": "asaetar",
    "asaetaras": "asaetar",
    "asaetare": "asaetar",
    "asaetareis": "asaetar",
    "asaetaremos": "asaetar",
    "asaetaren": "asaetar",
    "asaetares": "asaetar",
    "asaetaron": "asaetar",
    "asaetará": "asaetar",
    "asaetarán": "asaetar",
    "asaetarás": "asaetar",
    "asaetaré": "asaetar",
    "asaetaréis": "asaetar",
    "asaetaría": "asaetar",
    "asaetaríais": "asaetar",
    "asaetaríamos": "asaetar",
    "asaetarían": "asaetar",
    "asaetarías": "asaetar",
    "asaetas": "asaetar",
    "asaetase": "asaetar",
    "asaetaseis": "asaetar",
    "asaetasen": "asaetar",
    "asaetases": "asaetar",
    "asaetaste": "asaetar",
    "asaetasteis": "asaetar",
    "asaete": "asaetar",
    "asaetemos": "asaetar",
    "asaeten": "asaetar",
    "asaetes": "asaetar",
    "asaeto": "asaetar",
    "asaetábamos": "asaetar",
    "asaetáis": "asaetar",
    "asaetáramos": "asaetar",
    "asaetáremos": "asaetar",
    "asaetásemos": "asaetar",
    "asaeté": "asaetar",
    "asaetéis": "asaetar",
    "asaetó": "asaetar",
    "asainetea": "asainetear",
    "asaineteaba": "asainetear",
    "asaineteabais": "asainetear",
    "asaineteaban": "asainetear",
    "asaineteabas": "asainetear",
    "asainetead": "asainetear",
    "asaineteada": "asainetear",
    "asaineteadas": "asainetear",
    "asaineteado": "asainetear",
    "asaineteados": "asainetear",
    "asaineteamos": "asainetear",
    "asainetean": "asainetear",
    "asaineteando": "asainetear",
    "asaineteara": "asainetear",
    "asainetearais": "asainetear",
    "asainetearan": "asainetear",
    "asainetearas": "asainetear",
    "asaineteare": "asainetear",
    "asaineteareis": "asainetear",
    "asainetearemos": "asainetear",
    "asainetearen": "asainetear",
    "asaineteares": "asainetear",
    "asainetearon": "asainetear",
    "asaineteará": "asainetear",
    "asainetearán": "asainetear",
    "asainetearás": "asainetear",
    "asainetearé": "asainetear",
    "asainetearéis": "asainetear",
    "asainetearía": "asainetear",
    "asainetearíais": "asainetear",
    "asainetearíamos": "asainetear",
    "asainetearían": "asainetear",
    "asainetearías": "asainetear",
    "asaineteas": "asainetear",
    "asainetease": "asainetear",
    "asaineteaseis": "asainetear",
    "asaineteasen": "asainetear",
    "asaineteases": "asainetear",
    "asaineteaste": "asainetear",
    "asaineteasteis": "asainetear",
    "asainetee": "asainetear",
    "asaineteemos": "asainetear",
    "asaineteen": "asainetear",
    "asainetees": "asainetear",
    "asaineteo": "asainetear",
    "asaineteábamos": "asainetear",
    "asaineteáis": "asainetear",
    "asaineteáramos": "asainetear",
    "asaineteáremos": "asainetear",
    "asaineteásemos": "asainetear",
    "asaineteé": "asainetear",
    "asaineteéis": "asainetear",
    "asaineteó": "asainetear",
    "asalaria": "asalariar",
    "asalariaba": "asalariar",
    "asalariabais": "asalariar",
    "asalariaban": "asalariar",
    "asalariabas": "asalariar",
    "asalariad": "asalariar",
    "asalariada": "asalariar",
    "asalariadas": "asalariar",
    "asalariado": "asalariar",
    "asalariados": "asalariar",
    "asalariamos": "asalariar",
    "asalarian": "asalariar",
    "asalariando": "asalariar",
    "asalariara": "asalariar",
    "asalariarais": "asalariar",
    "asalariaran": "asalariar",
    "asalariaras": "asalariar",
    "asalariare": "asalariar",
    "asalariareis": "asalariar",
    "asalariaremos": "asalariar",
    "asalariaren": "asalariar",
    "asalariares": "asalariar",
    "asalariaron": "asalariar",
    "asalariará": "asalariar",
    "asalariarán": "asalariar",
    "asalariarás": "asalariar",
    "asalariaré": "asalariar",
    "asalariaréis": "asalariar",
    "asalariaría": "asalariar",
    "asalariaríais": "asalariar",
    "asalariaríamos": "asalariar",
    "asalariarían": "asalariar",
    "asalariarías": "asalariar",
    "asalarias": "asalariar",
    "asalariase": "asalariar",
    "asalariaseis": "asalariar",
    "asalariasen": "asalariar",
    "asalariases": "asalariar",
    "asalariaste": "asalariar",
    "asalariasteis": "asalariar",
    "asalarie": "asalariar",
    "asalariemos": "asalariar",
    "asalarien": "asalariar",
    "asalaries": "asalariar",
    "asalario": "asalariar",
    "asalariábamos": "asalariar",
    "asalariáis": "asalariar",
    "asalariáramos": "asalariar",
    "asalariáremos": "asalariar",
    "asalariásemos": "asalariar",
    "asalarié": "asalariar",
    "asalariéis": "asalariar",
    "asalarió": "asalariar",
    "asalmera": "asalmerar",
    "asalmeraba": "asalmerar",
    "asalmerabais": "asalmerar",
    "asalmeraban": "asalmerar",
    "asalmerabas": "asalmerar",
    "asalmerad": "asalmerar",
    "asalmerada": "asalmerar",
    "asalmeradas": "asalmerar",
    "asalmerado": "asalmerar",
    "asalmerados": "asalmerar",
    "asalmeramos": "asalmerar",
    "asalmeran": "asalmerar",
    "asalmerando": "asalmerar",
    "asalmerara": "asalmerar",
    "asalmerarais": "asalmerar",
    "asalmeraran": "asalmerar",
    "asalmeraras": "asalmerar",
    "asalmerare": "asalmerar",
    "asalmerareis": "asalmerar",
    "asalmeraremos": "asalmerar",
    "asalmeraren": "asalmerar",
    "asalmerares": "asalmerar",
    "asalmeraron": "asalmerar",
    "asalmerará": "asalmerar",
    "asalmerarán": "asalmerar",
    "asalmerarás": "asalmerar",
    "asalmeraré": "asalmerar",
    "asalmeraréis": "asalmerar",
    "asalmeraría": "asalmerar",
    "asalmeraríais": "asalmerar",
    "asalmeraríamos": "asalmerar",
    "asalmerarían": "asalmerar",
    "asalmerarías": "asalmerar",
    "asalmeras": "asalmerar",
    "asalmerase": "asalmerar",
    "asalmeraseis": "asalmerar",
    "asalmerasen": "asalmerar",
    "asalmerases": "asalmerar",
    "asalmeraste": "asalmerar",
    "asalmerasteis": "asalmerar",
    "asalmere": "asalmerar",
    "asalmeremos": "asalmerar",
    "asalmeren": "asalmerar",
    "asalmeres": "asalmerar",
    "asalmero": "asalmerar",
    "asalmerábamos": "asalmerar",
    "asalmeráis": "asalmerar",
    "asalmeráramos": "asalmerar",
    "asalmeráremos": "asalmerar",
    "asalmerásemos": "asalmerar",
    "asalmeré": "asalmerar",
    "asalmeréis": "asalmerar",
    "asalmeró": "asalmerar",
    "asalmonada": "asalmonado",
    "asalmonadas": "asalmonado",
    "asalmonados": "asalmonado",
    "asalta": "asaltar",
    "asaltaba": "asaltar",
    "asaltabais": "asaltar",
    "asaltaban": "asaltar",
    "asaltabas": "asaltar",
    "asaltad": "asaltar",
    "asaltada": "asaltar",
    "asaltadas": "asaltar",
    "asaltado": "asaltar",
    "asaltados": "asaltar",
    "asaltamos": "asaltar",
    "asaltan": "asaltar",
    "asaltando": "asaltar",
    "asaltantes": "asaltante",
    "asaltara": "asaltar",
    "asaltarais": "asaltar",
    "asaltaran": "asaltar",
    "asaltaras": "asaltar",
    "asaltare": "asaltar",
    "asaltareis": "asaltar",
    "asaltaremos": "asaltar",
    "asaltaren": "asaltar",
    "asaltares": "asaltar",
    "asaltaron": "asaltar",
    "asaltará": "asaltar",
    "asaltarán": "asaltar",
    "asaltarás": "asaltar",
    "asaltaré": "asaltar",
    "asaltaréis": "asaltar",
    "asaltaría": "asaltar",
    "asaltaríais": "asaltar",
    "asaltaríamos": "asaltar",
    "asaltarían": "asaltar",
    "asaltarías": "asaltar",
    "asaltas": "asaltar",
    "asaltase": "asaltar",
    "asaltaseis": "asaltar",
    "asaltasen": "asaltar",
    "asaltases": "asaltar",
    "asaltaste": "asaltar",
    "asaltasteis": "asaltar",
    "asalte": "asaltar",
    "asaltemos": "asaltar",
    "asalten": "asaltar",
    "asaltes": "asaltar",
    "asalto": "asaltar",
    "asaltos": "asalto",
    "asaltábamos": "asaltar",
    "asaltáis": "asaltar",
    "asaltáramos": "asaltar",
    "asaltáremos": "asaltar",
    "asaltásemos": "asaltar",
    "asalté": "asaltar",
    "asaltéis": "asaltar",
    "asaltó": "asaltar",
    "asamblearia": "asambleario",
    "asamblearias": "asambleario",
    "asamblearios": "asambleario",
    "asambleas": "asamblea",
    "asambleístas": "asambleísta",
    "asamos": "asar",
    "asan": "asar",
    "asando": "asar",
    "asara": "asar",
    "asarais": "asar",
    "asaran": "asar",
    "asaras": "asar",
    "asardinada": "asardinado",
    "asardinadas": "asardinado",
    "asardinados": "asardinado",
    "asare": "asar",
    "asareis": "asar",
    "asaremos": "asar",
    "asaren": "asar",
    "asares": "asar",
    "asargada": "asargado",
    "asargadas": "asargado",
    "asargados": "asargado",
    "asaron": "asar",
    "asará": "asar",
    "asarán": "asar",
    "asarás": "asar",
    "asaré": "asar",
    "asaréis": "asar",
    "asaría": "asar",
    "asaríais": "asar",
    "asaríamos": "asar",
    "asarían": "asar",
    "asarías": "asar",
    "asas": "asar",
    "asase": "asar",
    "asaseis": "asar",
    "asasen": "asar",
    "asases": "asar",
    "asaste": "asar",
    "asasteis": "asar",
    "asbestos": "asbesto",
    "asbolanas": "asbolana",
    "ascaridoles": "ascaridol",
    "ascendamos": "ascender",
    "ascended": "ascender",
    "ascendemos": "ascender",
    "ascendencias": "ascendencia",
    "ascendentes": "ascendente",
    "ascenderemos": "ascender",
    "ascenderá": "ascender",
    "ascenderán": "ascender",
    "ascenderás": "ascender",
    "ascenderé": "ascender",
    "ascenderéis": "ascender",
    "ascendería": "ascender",
    "ascenderíais": "ascender",
    "ascenderíamos": "ascender",
    "ascenderían": "ascender",
    "ascenderías": "ascender",
    "ascendida": "ascender",
    "ascendidas": "ascender",
    "ascendido": "ascender",
    "ascendidos": "ascender",
    "ascendiendo": "ascender",
    "ascendientes": "ascendiente",
    "ascendiera": "ascender",
    "ascendierais": "ascender",
    "ascendieran": "ascender",
    "ascendieras": "ascender",
    "ascendiere": "ascender",
    "ascendiereis": "ascender",
    "ascendieren": "ascender",
    "ascendieres": "ascender",
    "ascendieron": "ascender",
    "ascendiese": "ascender",
    "ascendieseis": "ascender",
    "ascendiesen": "ascender",
    "ascendieses": "ascender",
    "ascendimos": "ascender",
    "ascendiste": "ascender",
    "ascendisteis": "ascender",
    "ascendiéramos": "ascender",
    "ascendiéremos": "ascender",
    "ascendiésemos": "ascender",
    "ascendió": "ascender",
    "ascendáis": "ascender",
    "ascendéis": "ascender",
    "ascendí": "ascender",
    "ascendía": "ascender",
    "ascendíais": "ascender",
    "ascendíamos": "ascender",
    "ascendían": "ascender",
    "ascendías": "ascender",
    "ascensionales": "ascensional",
    "ascensiones": "ascensión",
    "ascensionistas": "ascensionista",
    "ascensores": "ascensor",
    "ascensoristas": "ascensorista",
    "ascensos": "ascenso",
    "ascetas": "asceta",
    "ascetismos": "ascetismo",
    "ascidias": "ascidia",
    "ascienda": "ascender",
    "asciendan": "ascender",
    "asciendas": "ascender",
    "asciende": "ascender",
    "ascienden": "ascender",
    "asciendes": "ascender",
    "asciendo": "ascender",
    "asclepiadáceas": "asclepiadácea",
    "ascomicetes": "ascomicete",
    "ascomicetos": "ascomiceto",
    "ascorbatos": "ascorbato",
    "ascos": "asco",
    "ascuas": "ascua",
    "ascética": "ascético",
    "ascéticas": "ascético",
    "ascéticos": "ascético",
    "ascítica": "ascítico",
    "ascíticas": "ascítico",
    "ascíticos": "ascítico",
    "ascórbica": "ascórbico",
    "ascórbicas": "ascórbico",
    "ascórbicos": "ascórbico",
    "ase": "asir",
    "asea": "asear",
    "aseaba": "asear",
    "aseabais": "asear",
    "aseaban": "asear",
    "aseabas": "asear",
    "asead": "asear",
    "aseada": "asear",
    "aseadas": "asear",
    "aseado": "asear",
    "aseados": "asear",
    "aseamos": "asear",
    "asean": "asear",
    "aseando": "asear",
    "aseara": "asear",
    "asearais": "asear",
    "asearan": "asear",
    "asearas": "asear",
    "aseare": "asear",
    "aseareis": "asear",
    "asearemos": "asear",
    "asearen": "asear",
    "aseares": "asear",
    "asearon": "asear",
    "aseará": "asear",
    "asearán": "asear",
    "asearás": "asear",
    "asearé": "asear",
    "asearéis": "asear",
    "asearía": "asear",
    "asearíais": "asear",
    "asearíamos": "asear",
    "asearían": "asear",
    "asearías": "asear",
    "aseas": "asear",
    "asease": "asear",
    "aseaseis": "asear",
    "aseasen": "asear",
    "aseases": "asear",
    "aseaste": "asear",
    "aseasteis": "asear",
    "asecha": "asechar",
    "asechaba": "asechar",
    "asechabais": "asechar",
    "asechaban": "asechar",
    "asechabas": "asechar",
    "asechad": "asechar",
    "asechada": "asechar",
    "asechadas": "asechar",
    "asechado": "asechar",
    "asechados": "asechar",
    "asechamos": "asechar",
    "asechan": "asechar",
    "asechando": "asechar",
    "asechanzas": "asechanza",
    "asechara": "asechar",
    "asecharais": "asechar",
    "asecharan": "asechar",
    "asecharas": "asechar",
    "asechare": "asechar",
    "asechareis": "asechar",
    "asecharemos": "asechar",
    "asecharen": "asechar",
    "asechares": "asechar",
    "asecharon": "asechar",
    "asechará": "asechar",
    "asecharán": "asechar",
    "asecharás": "asechar",
    "asecharé": "asechar",
    "asecharéis": "asechar",
    "asecharía": "asechar",
    "asecharíais": "asechar",
    "asecharíamos": "asechar",
    "asecharían": "asechar",
    "asecharías": "asechar",
    "asechas": "asechar",
    "asechase": "asechar",
    "asechaseis": "asechar",
    "asechasen": "asechar",
    "asechases": "asechar",
    "asechaste": "asechar",
    "asechasteis": "asechar",
    "aseche": "asechar",
    "asechemos": "asechar",
    "asechen": "asechar",
    "aseches": "asechar",
    "asecho": "asechar",
    "asechos": "asecho",
    "asechábamos": "asechar",
    "asecháis": "asechar",
    "asecháramos": "asechar",
    "asecháremos": "asechar",
    "asechásemos": "asechar",
    "aseché": "asechar",
    "asechéis": "asechar",
    "asechó": "asechar",
    "aseda": "asedar",
    "asedaba": "asedar",
    "asedabais": "asedar",
    "asedaban": "asedar",
    "asedabas": "asedar",
    "asedad": "asedar",
    "asedada": "asedar",
    "asedadas": "asedar",
    "asedado": "asedar",
    "asedados": "asedar",
    "asedamos": "asedar",
    "asedan": "asedar",
    "asedando": "asedar",
    "asedara": "asedar",
    "asedarais": "asedar",
    "asedaran": "asedar",
    "asedaras": "asedar",
    "asedare": "asedar",
    "asedareis": "asedar",
    "asedaremos": "asedar",
    "asedaren": "asedar",
    "asedares": "asedar",
    "asedaron": "asedar",
    "asedará": "asedar",
    "asedarán": "asedar",
    "asedarás": "asedar",
    "asedaré": "asedar",
    "asedaréis": "asedar",
    "asedaría": "asedar",
    "asedaríais": "asedar",
    "asedaríamos": "asedar",
    "asedarían": "asedar",
    "asedarías": "asedar",
    "asedas": "asedar",
    "asedase": "asedar",
    "asedaseis": "asedar",
    "asedasen": "asedar",
    "asedases": "asedar",
    "asedaste": "asedar",
    "asedasteis": "asedar",
    "asede": "asedar",
    "asedemos": "asedar",
    "aseden": "asedar",
    "asedes": "asedar",
    "asedia": "asediar",
    "asediaba": "asediar",
    "asediabais": "asediar",
    "asediaban": "asediar",
    "asediabas": "asediar",
    "asediad": "asediar",
    "asediada": "asediar",
    "asediadas": "asediar",
    "asediado": "asediar",
    "asediadora": "asediador",
    "asediadoras": "asediador",
    "asediadores": "asediador",
    "asediados": "asediar",
    "asediamos": "asediar",
    "asedian": "asediar",
    "asediando": "asediar",
    "asediantes": "asediante",
    "asediara": "asediar",
    "asediarais": "asediar",
    "asediaran": "asediar",
    "asediaras": "asediar",
    "asediare": "asediar",
    "asediareis": "asediar",
    "asediaremos": "asediar",
    "asediaren": "asediar",
    "asediares": "asediar",
    "asediaron": "asediar",
    "asediará": "asediar",
    "asediarán": "asediar",
    "asediarás": "asediar",
    "asediaré": "asediar",
    "asediaréis": "asediar",
    "asediaría": "asediar",
    "asediaríais": "asediar",
    "asediaríamos": "asediar",
    "asediarían": "asediar",
    "asediarías": "asediar",
    "asedias": "asediar",
    "asediase": "asediar",
    "asediaseis": "asediar",
    "asediasen": "asediar",
    "asediases": "asediar",
    "asediaste": "asediar",
    "asediasteis": "asediar",
    "asedie": "asediar",
    "asediemos": "asediar",
    "asedien": "asediar",
    "asedies": "asediar",
    "asedio": "asediar",
    "asedios": "asedio",
    "asediábamos": "asediar",
    "asediáis": "asediar",
    "asediáramos": "asediar",
    "asediáremos": "asediar",
    "asediásemos": "asediar",
    "asedié": "asediar",
    "asediéis": "asediar",
    "asedió": "asediar",
    "asedo": "asedar",
    "asedábamos": "asedar",
    "asedáis": "asedar",
    "asedáramos": "asedar",
    "asedáremos": "asedar",
    "asedásemos": "asedar",
    "asedé": "asedar",
    "asedéis": "asedar",
    "asedó": "asedar",
    "asee": "asear",
    "aseemos": "asear",
    "aseen": "asear",
    "asees": "asear",
    "aseglara": "aseglarar",
    "aseglaraba": "aseglarar",
    "aseglarabais": "aseglarar",
    "aseglaraban": "aseglarar",
    "aseglarabas": "aseglarar",
    "aseglarad": "aseglarar",
    "aseglarada": "aseglarar",
    "aseglaradas": "aseglarar",
    "aseglarado": "aseglarar",
    "aseglarados": "aseglarar",
    "aseglaramos": "aseglarar",
    "aseglaran": "aseglarar",
    "aseglarando": "aseglarar",
    "aseglarara": "aseglarar",
    "aseglararais": "aseglarar",
    "aseglararan": "aseglarar",
    "aseglararas": "aseglarar",
    "aseglarare": "aseglarar",
    "aseglarareis": "aseglarar",
    "aseglararemos": "aseglarar",
    "aseglararen": "aseglarar",
    "aseglarares": "aseglarar",
    "aseglararon": "aseglarar",
    "aseglarará": "aseglarar",
    "aseglararán": "aseglarar",
    "aseglararás": "aseglarar",
    "aseglararé": "aseglarar",
    "aseglararéis": "aseglarar",
    "aseglararía": "aseglarar",
    "aseglararíais": "aseglarar",
    "aseglararíamos": "aseglarar",
    "aseglararían": "aseglarar",
    "aseglararías": "aseglarar",
    "aseglaras": "aseglarar",
    "aseglarase": "aseglarar",
    "aseglaraseis": "aseglarar",
    "aseglarasen": "aseglarar",
    "aseglarases": "aseglarar",
    "aseglaraste": "aseglarar",
    "aseglarasteis": "aseglarar",
    "aseglare": "aseglarar",
    "aseglaremos": "aseglarar",
    "aseglaren": "aseglarar",
    "aseglares": "aseglarar",
    "aseglarice": "aseglarizar",
    "aseglaricemos": "aseglarizar",
    "aseglaricen": "aseglarizar",
    "aseglarices": "aseglarizar",
    "aseglaricé": "aseglarizar",
    "aseglaricéis": "aseglarizar",
    "aseglariza": "aseglarizar",
    "aseglarizaba": "aseglarizar",
    "aseglarizabais": "aseglarizar",
    "aseglarizaban": "aseglarizar",
    "aseglarizabas": "aseglarizar",
    "aseglarizad": "aseglarizar",
    "aseglarizada": "aseglarizar",
    "aseglarizadas": "aseglarizar",
    "aseglarizado": "aseglarizar",
    "aseglarizados": "aseglarizar",
    "aseglarizamos": "aseglarizar",
    "aseglarizan": "aseglarizar",
    "aseglarizando": "aseglarizar",
    "aseglarizara": "aseglarizar",
    "aseglarizarais": "aseglarizar",
    "aseglarizaran": "aseglarizar",
    "aseglarizaras": "aseglarizar",
    "aseglarizare": "aseglarizar",
    "aseglarizareis": "aseglarizar",
    "aseglarizaremos": "aseglarizar",
    "aseglarizaren": "aseglarizar",
    "aseglarizares": "aseglarizar",
    "aseglarizaron": "aseglarizar",
    "aseglarizará": "aseglarizar",
    "aseglarizarán": "aseglarizar",
    "aseglarizarás": "aseglarizar",
    "aseglarizaré": "aseglarizar",
    "aseglarizaréis": "aseglarizar",
    "aseglarizaría": "aseglarizar",
    "aseglarizaríais": "aseglarizar",
    "aseglarizaríamos": "aseglarizar",
    "aseglarizarían": "aseglarizar",
    "aseglarizarías": "aseglarizar",
    "aseglarizas": "aseglarizar",
    "aseglarizase": "aseglarizar",
    "aseglarizaseis": "aseglarizar",
    "aseglarizasen": "aseglarizar",
    "aseglarizases": "aseglarizar",
    "aseglarizaste": "aseglarizar",
    "aseglarizasteis": "aseglarizar",
    "aseglarizo": "aseglarizar",
    "aseglarizábamos": "aseglarizar",
    "aseglarizáis": "aseglarizar",
    "aseglarizáramos": "aseglarizar",
    "aseglarizáremos": "aseglarizar",
    "aseglarizásemos": "aseglarizar",
    "aseglarizó": "aseglarizar",
    "aseglaro": "aseglarar",
    "aseglarábamos": "aseglarar",
    "aseglaráis": "aseglarar",
    "aseglaráramos": "aseglarar",
    "aseglaráremos": "aseglarar",
    "aseglarásemos": "aseglarar",
    "aseglaré": "aseglarar",
    "aseglaréis": "aseglarar",
    "aseglaró": "aseglarar",
    "asegura": "asegurar",
    "aseguraba": "asegurar",
    "asegurabais": "asegurar",
    "aseguraban": "asegurar",
    "asegurabas": "asegurar",
    "asegurabilidades": "asegurabilidad",
    "asegurables": "asegurable",
    "aseguraciones": "aseguración",
    "asegurad": "asegurar",
    "asegurada": "asegurar",
    "aseguradas": "asegurar",
    "asegurado": "asegurar",
    "aseguradora": "asegurador",
    "aseguradoras": "aseguradora",
    "aseguradores": "asegurador",
    "asegurados": "asegurar",
    "aseguramientos": "aseguramiento",
    "aseguramos": "asegurar",
    "aseguran": "asegurar",
    "asegurando": "asegurar",
    "asegurara": "asegurar",
    "asegurarais": "asegurar",
    "aseguraran": "asegurar",
    "aseguraras": "asegurar",
    "asegurare": "asegurar",
    "asegurareis": "asegurar",
    "aseguraremos": "asegurar",
    "aseguraren": "asegurar",
    "asegurares": "asegurar",
    "aseguraron": "asegurar",
    "asegurará": "asegurar",
    "asegurarán": "asegurar",
    "asegurarás": "asegurar",
    "aseguraré": "asegurar",
    "aseguraréis": "asegurar",
    "aseguraría": "asegurar",
    "aseguraríais": "asegurar",
    "aseguraríamos": "asegurar",
    "asegurarían": "asegurar",
    "asegurarías": "asegurar",
    "aseguras": "asegurar",
    "asegurase": "asegurar",
    "aseguraseis": "asegurar",
    "asegurasen": "asegurar",
    "asegurases": "asegurar",
    "aseguraste": "asegurar",
    "asegurasteis": "asegurar",
    "asegure": "asegurar",
    "aseguremos": "asegurar",
    "aseguren": "asegurar",
    "asegures": "asegurar",
    "aseguro": "asegurar",
    "asegurábamos": "asegurar",
    "aseguráis": "asegurar",
    "aseguráramos": "asegurar",
    "aseguráremos": "asegurar",
    "asegurásemos": "asegurar",
    "aseguré": "asegurar",
    "aseguréis": "asegurar",
    "aseguró": "asegurar",
    "aseidades": "aseidad",
    "asemeja": "asemejar",
    "asemejaba": "asemejar",
    "asemejabais": "asemejar",
    "asemejaban": "asemejar",
    "asemejabas": "asemejar",
    "asemejad": "asemejar",
    "asemejada": "asemejar",
    "asemejadas": "asemejar",
    "asemejado": "asemejar",
    "asemejados": "asemejar",
    "asemejamos": "asemejar",
    "asemejan": "asemejar",
    "asemejando": "asemejar",
    "asemejara": "asemejar",
    "asemejarais": "asemejar",
    "asemejaran": "asemejar",
    "asemejaras": "asemejar",
    "asemejare": "asemejar",
    "asemejareis": "asemejar",
    "asemejaremos": "asemejar",
    "asemejaren": "asemejar",
    "asemejares": "asemejar",
    "asemejaron": "asemejar",
    "asemejará": "asemejar",
    "asemejarán": "asemejar",
    "asemejarás": "asemejar",
    "asemejaré": "asemejar",
    "asemejaréis": "asemejar",
    "asemejaría": "asemejar",
    "asemejaríais": "asemejar",
    "asemejaríamos": "asemejar",
    "asemejarían": "asemejar",
    "asemejarías": "asemejar",
    "asemejas": "asemejar",
    "asemejase": "asemejar",
    "asemejaseis": "asemejar",
    "asemejasen": "asemejar",
    "asemejases": "asemejar",
    "asemejaste": "asemejar",
    "asemejasteis": "asemejar",
    "asemeje": "asemejar",
    "asemejemos": "asemejar",
    "asemejen": "asemejar",
    "asemejes": "asemejar",
    "asemejo": "asemejar",
    "asemejábamos": "asemejar",
    "asemejáis": "asemejar",
    "asemejáramos": "asemejar",
    "asemejáremos": "asemejar",
    "asemejásemos": "asemejar",
    "asemejé": "asemejar",
    "asemejéis": "asemejar",
    "asemejó": "asemejar",
    "asemos": "asar",
    "asen": "asir",
    "asenderea": "asenderear",
    "asendereaba": "asenderear",
    "asendereabais": "asenderear",
    "asendereaban": "asenderear",
    "asendereabas": "asenderear",
    "asenderead": "asenderear",
    "asendereada": "asenderear",
    "asendereadas": "asenderear",
    "asendereado": "asenderear",
    "asendereados": "asenderear",
    "asendereamos": "asenderear",
    "asenderean": "asenderear",
    "asendereando": "asenderear",
    "asendereara": "asenderear",
    "asenderearais": "asenderear",
    "asenderearan": "asenderear",
    "asenderearas": "asenderear",
    "asendereare": "asenderear",
    "asendereareis": "asenderear",
    "asenderearemos": "asenderear",
    "asenderearen": "asenderear",
    "asendereares": "asenderear",
    "asenderearon": "asenderear",
    "asendereará": "asenderear",
    "asenderearán": "asenderear",
    "asenderearás": "asenderear",
    "asenderearé": "asenderear",
    "asenderearéis": "asenderear",
    "asenderearía": "asenderear",
    "asenderearíais": "asenderear",
    "asenderearíamos": "asenderear",
    "asenderearían": "asenderear",
    "asenderearías": "asenderear",
    "asendereas": "asenderear",
    "asenderease": "asenderear",
    "asendereaseis": "asenderear",
    "asendereasen": "asenderear",
    "asendereases": "asenderear",
    "asendereaste": "asenderear",
    "asendereasteis": "asenderear",
    "asenderee": "asenderear",
    "asendereemos": "asenderear",
    "asendereen": "asenderear",
    "asenderees": "asenderear",
    "asendereo": "asenderear",
    "asendereábamos": "asenderear",
    "asendereáis": "asenderear",
    "asendereáramos": "asenderear",
    "asendereáremos": "asenderear",
    "asendereásemos": "asenderear",
    "asendereé": "asenderear",
    "asendereéis": "asenderear",
    "asendereó": "asenderear",
    "asensos": "asenso",
    "asentaba": "asentar",
    "asentabais": "asentar",
    "asentaban": "asentar",
    "asentabas": "asentar",
    "asentad": "asentar",
    "asentada": "asentar",
    "asentadas": "asentar",
    "asentado": "asentar",
    "asentadores": "asentador",
    "asentados": "asentar",
    "asentamientos": "asentamiento",
    "asentamos": "asentar",
    "asentando": "asentar",
    "asentara": "asentar",
    "asentarais": "asentar",
    "asentaran": "asentar",
    "asentaras": "asentar",
    "asentare": "asentar",
    "asentareis": "asentar",
    "asentaremos": "asentar",
    "asentaren": "asentar",
    "asentares": "asentar",
    "asentaron": "asentar",
    "asentará": "asentar",
    "asentarán": "asentar",
    "asentarás": "asentar",
    "asentaré": "asentar",
    "asentaréis": "asentar",
    "asentaría": "asentar",
    "asentaríais": "asentar",
    "asentaríamos": "asentar",
    "asentarían": "asentar",
    "asentarías": "asentar",
    "asentase": "asentar",
    "asentaseis": "asentar",
    "asentasen": "asentar",
    "asentases": "asentar",
    "asentaste": "asentar",
    "asentasteis": "asentar",
    "asentemos": "asentar",
    "asentid": "asentir",
    "asentida": "asentir",
    "asentidas": "asentir",
    "asentido": "asentir",
    "asentidos": "asentir",
    "asentimientos": "asentimiento",
    "asentimos": "asentir",
    "asentiremos": "asentir",
    "asentirá": "asentir",
    "asentirán": "asentir",
    "asentirás": "asentir",
    "asentiré": "asentir",
    "asentiréis": "asentir",
    "asentiría": "asentir",
    "asentiríais": "asentir",
    "asentiríamos": "asentir",
    "asentirían": "asentir",
    "asentirías": "asentir",
    "asentiste": "asentir",
    "asentisteis": "asentir",
    "asentábamos": "asentar",
    "asentáis": "asentar",
    "asentáramos": "asentar",
    "asentáremos": "asentar",
    "asentásemos": "asentar",
    "asenté": "asentar",
    "asentéis": "asentar",
    "asentí": "asentir",
    "asentía": "asentir",
    "asentíais": "asentir",
    "asentíamos": "asentir",
    "asentían": "asentir",
    "asentías": "asentir",
    "asentís": "asentir",
    "asentó": "asentar",
    "aseo": "asear",
    "aseos": "aseo",
    "asepsias": "asepsia",
    "asequibilidades": "asequibilidad",
    "asequibles": "asequible",
    "aserciones": "aserción",
    "aserena": "aserenar",
    "aserenaba": "aserenar",
    "aserenabais": "aserenar",
    "aserenaban": "aserenar",
    "aserenabas": "aserenar",
    "aserenad": "aserenar",
    "aserenada": "aserenar",
    "aserenadas": "aserenar",
    "aserenado": "aserenar",
    "aserenados": "aserenar",
    "aserenamos": "aserenar",
    "aserenan": "aserenar",
    "aserenando": "aserenar",
    "aserenara": "aserenar",
    "aserenarais": "aserenar",
    "aserenaran": "aserenar",
    "aserenaras": "aserenar",
    "aserenare": "aserenar",
    "aserenareis": "aserenar",
    "aserenaremos": "aserenar",
    "aserenaren": "aserenar",
    "aserenares": "aserenar",
    "aserenaron": "aserenar",
    "aserenará": "aserenar",
    "aserenarán": "aserenar",
    "aserenarás": "aserenar",
    "aserenaré": "aserenar",
    "aserenaréis": "aserenar",
    "aserenaría": "aserenar",
    "aserenaríais": "aserenar",
    "aserenaríamos": "aserenar",
    "aserenarían": "aserenar",
    "aserenarías": "aserenar",
    "aserenas": "aserenar",
    "aserenase": "aserenar",
    "aserenaseis": "aserenar",
    "aserenasen": "aserenar",
    "aserenases": "aserenar",
    "aserenaste": "aserenar",
    "aserenasteis": "aserenar",
    "aserene": "aserenar",
    "aserenemos": "aserenar",
    "aserenen": "aserenar",
    "aserenes": "aserenar",
    "asereno": "aserenar",
    "aserenábamos": "aserenar",
    "aserenáis": "aserenar",
    "aserenáramos": "aserenar",
    "aserenáremos": "aserenar",
    "aserenásemos": "aserenar",
    "aserené": "aserenar",
    "aserenéis": "aserenar",
    "aserenó": "aserenar",
    "aseria": "aseriar",
    "aseriaba": "aseriar",
    "aseriabais": "aseriar",
    "aseriaban": "aseriar",
    "aseriabas": "aseriar",
    "aseriad": "aseriar",
    "aseriada": "aseriar",
    "aseriadas": "aseriar",
    "aseriado": "aseriar",
    "aseriados": "aseriar",
    "aseriamos": "aseriar",
    "aserian": "aseriar",
    "aseriando": "aseriar",
    "aseriara": "aseriar",
    "aseriarais": "aseriar",
    "aseriaran": "aseriar",
    "aseriaras": "aseriar",
    "aseriare": "aseriar",
    "aseriareis": "aseriar",
    "aseriaremos": "aseriar",
    "aseriaren": "aseriar",
    "aseriares": "aseriar",
    "aseriaron": "aseriar",
    "aseriará": "aseriar",
    "aseriarán": "aseriar",
    "aseriarás": "aseriar",
    "aseriaré": "aseriar",
    "aseriaréis": "aseriar",
    "aseriaría": "aseriar",
    "aseriaríais": "aseriar",
    "aseriaríamos": "aseriar",
    "aseriarían": "aseriar",
    "aseriarías": "aseriar",
    "aserias": "aseriar",
    "aseriase": "aseriar",
    "aseriaseis": "aseriar",
    "aseriasen": "aseriar",
    "aseriases": "aseriar",
    "aseriaste": "aseriar",
    "aseriasteis": "aseriar",
    "aserie": "aseriar",
    "aseriemos": "aseriar",
    "aserien": "aseriar",
    "aseries": "aseriar",
    "aserio": "aseriar",
    "aseriábamos": "aseriar",
    "aseriáis": "aseriar",
    "aseriáramos": "aseriar",
    "aseriáremos": "aseriar",
    "aseriásemos": "aseriar",
    "aserié": "aseriar",
    "aseriéis": "aseriar",
    "aserió": "aseriar",
    "aserraba": "aserrar",
    "aserrabais": "aserrar",
    "aserraban": "aserrar",
    "aserrabas": "aserrar",
    "aserrad": "aserrar",
    "aserrada": "aserrar",
    "aserradas": "aserrar",
    "aserraderos": "aserradero",
    "aserradiza": "aserradizo",
    "aserradizas": "aserradizo",
    "aserradizos": "aserradizo",
    "aserrado": "aserrar",
    "aserradora": "aserrador",
    "aserradoras": "aserradora",
    "aserradores": "aserrador",
    "aserrados": "aserrar",
    "aserraduras": "aserradura",
    "aserramos": "aserrar",
    "aserrando": "aserrar",
    "aserrara": "aserrar",
    "aserrarais": "aserrar",
    "aserraran": "aserrar",
    "aserraras": "aserrar",
    "aserrare": "aserrar",
    "aserrareis": "aserrar",
    "aserraremos": "aserrar",
    "aserraren": "aserrar",
    "aserrares": "aserrar",
    "aserraron": "aserrar",
    "aserrará": "aserrar",
    "aserrarán": "aserrar",
    "aserrarás": "aserrar",
    "aserraré": "aserrar",
    "aserraréis": "aserrar",
    "aserraría": "aserrar",
    "aserraríais": "aserrar",
    "aserraríamos": "aserrar",
    "aserrarían": "aserrar",
    "aserrarías": "aserrar",
    "aserrase": "aserrar",
    "aserraseis": "aserrar",
    "aserrasen": "aserrar",
    "aserrases": "aserrar",
    "aserraste": "aserrar",
    "aserrasteis": "aserrar",
    "aserremos": "aserrar",
    "aserrucha": "aserruchar",
    "aserruchaba": "aserruchar",
    "aserruchabais": "aserruchar",
    "aserruchaban": "aserruchar",
    "aserruchabas": "aserruchar",
    "aserruchad": "aserruchar",
    "aserruchada": "aserruchar",
    "aserruchadas": "aserruchar",
    "aserruchado": "aserruchar",
    "aserruchados": "aserruchar",
    "aserruchamos": "aserruchar",
    "aserruchan": "aserruchar",
    "aserruchando": "aserruchar",
    "aserruchara": "aserruchar",
    "aserrucharais": "aserruchar",
    "aserrucharan": "aserruchar",
    "aserrucharas": "aserruchar",
    "aserruchare": "aserruchar",
    "aserruchareis": "aserruchar",
    "aserrucharemos": "aserruchar",
    "aserrucharen": "aserruchar",
    "aserruchares": "aserruchar",
    "aserrucharon": "aserruchar",
    "aserruchará": "aserruchar",
    "aserrucharán": "aserruchar",
    "aserrucharás": "aserruchar",
    "aserrucharé": "aserruchar",
    "aserrucharéis": "aserruchar",
    "aserrucharía": "aserruchar",
    "aserrucharíais": "aserruchar",
    "aserrucharíamos": "aserruchar",
    "aserrucharían": "aserruchar",
    "aserrucharías": "aserruchar",
    "aserruchas": "aserruchar",
    "aserruchase": "aserruchar",
    "aserruchaseis": "aserruchar",
    "aserruchasen": "aserruchar",
    "aserruchases": "aserruchar",
    "aserruchaste": "aserruchar",
    "aserruchasteis": "aserruchar",
    "aserruche": "aserruchar",
    "aserruchemos": "aserruchar",
    "aserruchen": "aserruchar",
    "aserruches": "aserruchar",
    "aserrucho": "aserruchar",
    "aserruchábamos": "aserruchar",
    "aserrucháis": "aserruchar",
    "aserrucháramos": "aserruchar",
    "aserrucháremos": "aserruchar",
    "aserruchásemos": "aserruchar",
    "aserruché": "aserruchar",
    "aserruchéis": "aserruchar",
    "aserruchó": "aserruchar",
    "aserrábamos": "aserrar",
    "aserráis": "aserrar",
    "aserráramos": "aserrar",
    "aserráremos": "aserrar",
    "aserrásemos": "aserrar",
    "aserré": "aserrar",
    "aserréis": "aserrar",
    "aserró": "aserrar",
    "asertiva": "asertivo",
    "asertivas": "asertivo",
    "asertivos": "asertivo",
    "asertos": "aserto",
    "asertórica": "asertórico",
    "asertóricas": "asertórico",
    "asertóricos": "asertórico",
    "ases": "asir",
    "asesa": "asesar",
    "asesaba": "asesar",
    "asesabais": "asesar",
    "asesaban": "asesar",
    "asesabas": "asesar",
    "asesad": "asesar",
    "asesada": "asesar",
    "asesadas": "asesar",
    "asesado": "asesar",
    "asesados": "asesar",
    "asesamos": "asesar",
    "asesan": "asesar",
    "asesando": "asesar",
    "asesara": "asesar",
    "asesarais": "asesar",
    "asesaran": "asesar",
    "asesaras": "asesar",
    "asesare": "asesar",
    "asesareis": "asesar",
    "asesaremos": "asesar",
    "asesaren": "asesar",
    "asesares": "asesar",
    "asesaron": "asesar",
    "asesará": "asesar",
    "asesarán": "asesar",
    "asesarás": "asesar",
    "asesaré": "asesar",
    "asesaréis": "asesar",
    "asesaría": "asesar",
    "asesaríais": "asesar",
    "asesaríamos": "asesar",
    "asesarían": "asesar",
    "asesarías": "asesar",
    "asesas": "asesar",
    "asesase": "asesar",
    "asesaseis": "asesar",
    "asesasen": "asesar",
    "asesases": "asesar",
    "asesaste": "asesar",
    "asesasteis": "asesar",
    "asese": "asesar",
    "asesemos": "asesar",
    "asesen": "asesar",
    "aseses": "asesar",
    "asesina": "asesino",
    "asesinaba": "asesinar",
    "asesinabais": "asesinar",
    "asesinaban": "asesinar",
    "asesinabas": "asesinar",
    "asesinad": "asesinar",
    "asesinada": "asesinar",
    "asesinadas": "asesinar",
    "asesinado": "asesinar",
    "asesinados": "asesinar",
    "asesinamos": "asesinar",
    "asesinan": "asesinar",
    "asesinando": "asesinar",
    "asesinara": "asesinar",
    "asesinarais": "asesinar",
    "asesinaran": "asesinar",
    "asesinaras": "asesinar",
    "asesinare": "asesinar",
    "asesinareis": "asesinar",
    "asesinaremos": "asesinar",
    "asesinaren": "asesinar",
    "asesinares": "asesinar",
    "asesinaron": "asesinar",
    "asesinará": "asesinar",
    "asesinarán": "asesinar",
    "asesinarás": "asesinar",
    "asesinaré": "asesinar",
    "asesinaréis": "asesinar",
    "asesinaría": "asesinar",
    "asesinaríais": "asesinar",
    "asesinaríamos": "asesinar",
    "asesinarían": "asesinar",
    "asesinarías": "asesinar",
    "asesinas": "asesino",
    "asesinase": "asesinar",
    "asesinaseis": "asesinar",
    "asesinasen": "asesinar",
    "asesinases": "asesinar",
    "asesinaste": "asesinar",
    "asesinasteis": "asesinar",
    "asesinatos": "asesinato",
    "asesine": "asesinar",
    "asesinemos": "asesinar",
    "asesinen": "asesinar",
    "asesines": "asesinar",
    "asesino": "asesinar",
    "asesinos": "asesino",
    "asesinábamos": "asesinar",
    "asesináis": "asesinar",
    "asesináramos": "asesinar",
    "asesináremos": "asesinar",
    "asesinásemos": "asesinar",
    "asesiné": "asesinar",
    "asesinéis": "asesinar",
    "asesinó": "asesinar",
    "aseso": "asesar",
    "asesora": "asesorar",
    "asesoraba": "asesorar",
    "asesorabais": "asesorar",
    "asesoraban": "asesorar",
    "asesorabas": "asesorar",
    "asesorad": "asesorar",
    "asesorada": "asesorar",
    "asesoradas": "asesorar",
    "asesorado": "asesorar",
    "asesorados": "asesorar",
    "asesoramientos": "asesoramiento",
    "asesoramos": "asesorar",
    "asesoran": "asesorar",
    "asesorando": "asesorar",
    "asesorara": "asesorar",
    "asesorarais": "asesorar",
    "asesoraran": "asesorar",
    "asesoraras": "asesorar",
    "asesorare": "asesorar",
    "asesorareis": "asesorar",
    "asesoraremos": "asesorar",
    "asesoraren": "asesorar",
    "asesorares": "asesorar",
    "asesoraron": "asesorar",
    "asesorará": "asesorar",
    "asesorarán": "asesorar",
    "asesorarás": "asesorar",
    "asesoraré": "asesorar",
    "asesoraréis": "asesorar",
    "asesoraría": "asesorar",
    "asesoraríais": "asesorar",
    "asesoraríamos": "asesorar",
    "asesorarían": "asesorar",
    "asesorarías": "asesorar",
    "asesoras": "asesorar",
    "asesorase": "asesorar",
    "asesoraseis": "asesorar",
    "asesorasen": "asesorar",
    "asesorases": "asesorar",
    "asesoraste": "asesorar",
    "asesorasteis": "asesorar",
    "asesore": "asesorar",
    "asesoremos": "asesorar",
    "asesoren": "asesorar",
    "asesores": "asesorar",
    "asesoro": "asesorar",
    "asesorábamos": "asesorar",
    "asesoráis": "asesorar",
    "asesoráramos": "asesorar",
    "asesoráremos": "asesorar",
    "asesorásemos": "asesorar",
    "asesoré": "asesorar",
    "asesoréis": "asesorar",
    "asesorías": "asesoría",
    "asesoró": "asesorar",
    "asestaba": "asestar",
    "asestabais": "asestar",
    "asestaban": "asestar",
    "asestabas": "asestar",
    "asestad": "asestar",
    "asestada": "asestar",
    "asestadas": "asestar",
    "asestado": "asestar",
    "asestados": "asestar",
    "asestaduras": "asestadura",
    "asestamos": "asestar",
    "asestando": "asestar",
    "asestara": "asestar",
    "asestarais": "asestar",
    "asestaran": "asestar",
    "asestaras": "asestar",
    "asestare": "asestar",
    "asestareis": "asestar",
    "asestaremos": "asestar",
    "asestaren": "asestar",
    "asestares": "asestar",
    "asestaron": "asestar",
    "asestará": "asestar",
    "asestarán": "asestar",
    "asestarás": "asestar",
    "asestaré": "asestar",
    "asestaréis": "asestar",
    "asestaría": "asestar",
    "asestaríais": "asestar",
    "asestaríamos": "asestar",
    "asestarían": "asestar",
    "asestarías": "asestar",
    "asestase": "asestar",
    "asestaseis": "asestar",
    "asestasen": "asestar",
    "asestases": "asestar",
    "asestaste": "asestar",
    "asestasteis": "asestar",
    "asestemos": "asestar",
    "asestábamos": "asestar",
    "asestáis": "asestar",
    "asestáramos": "asestar",
    "asestáremos": "asestar",
    "asestásemos": "asestar",
    "asesté": "asestar",
    "asestéis": "asestar",
    "asestó": "asestar",
    "asesábamos": "asesar",
    "asesáis": "asesar",
    "asesáramos": "asesar",
    "asesáremos": "asesar",
    "asesásemos": "asesar",
    "asesé": "asesar",
    "aseséis": "asesar",
    "asesó": "asesar",
    "asetada": "asetado",
    "asetadas": "asetado",
    "asetados": "asetado",
    "asevera": "aseverar",
    "aseveraba": "aseverar",
    "aseverabais": "aseverar",
    "aseveraban": "aseverar",
    "aseverabas": "aseverar",
    "aseveraciones": "aseveración",
    "aseverad": "aseverar",
    "aseverada": "aseverar",
    "aseveradas": "aseverar",
    "aseverado": "aseverar",
    "aseverados": "aseverar",
    "aseveramos": "aseverar",
    "aseveran": "aseverar",
    "aseverando": "aseverar",
    "aseverara": "aseverar",
    "aseverarais": "aseverar",
    "aseveraran": "aseverar",
    "aseveraras": "aseverar",
    "aseverare": "aseverar",
    "aseverareis": "aseverar",
    "aseveraremos": "aseverar",
    "aseveraren": "aseverar",
    "aseverares": "aseverar",
    "aseveraron": "aseverar",
    "aseverará": "aseverar",
    "aseverarán": "aseverar",
    "aseverarás": "aseverar",
    "aseveraré": "aseverar",
    "aseveraréis": "aseverar",
    "aseveraría": "aseverar",
    "aseveraríais": "aseverar",
    "aseveraríamos": "aseverar",
    "aseverarían": "aseverar",
    "aseverarías": "aseverar",
    "aseveras": "aseverar",
    "aseverase": "aseverar",
    "aseveraseis": "aseverar",
    "aseverasen": "aseverar",
    "aseverases": "aseverar",
    "aseveraste": "aseverar",
    "aseverasteis": "aseverar",
    "aseverativa": "aseverativo",
    "aseverativas": "aseverativo",
    "aseverativos": "aseverativo",
    "asevere": "aseverar",
    "aseveremos": "aseverar",
    "aseveren": "aseverar",
    "aseveres": "aseverar",
    "asevero": "aseverar",
    "aseverábamos": "aseverar",
    "aseveráis": "aseverar",
    "aseveráramos": "aseverar",
    "aseveráremos": "aseverar",
    "aseverásemos": "aseverar",
    "aseveré": "aseverar",
    "aseveréis": "aseverar",
    "aseveró": "aseverar",
    "asexuada": "asexuado",
    "asexuadas": "asexuado",
    "asexuados": "asexuado",
    "asexuales": "asexual",
    "aseábamos": "asear",
    "aseáis": "asear",
    "aseáramos": "asear",
    "aseáremos": "asear",
    "aseásemos": "asear",
    "aseé": "asear",
    "aseéis": "asear",
    "aseó": "asear",
    "asfalta": "asfaltar",
    "asfaltaba": "asfaltar",
    "asfaltabais": "asfaltar",
    "asfaltaban": "asfaltar",
    "asfaltabas": "asfaltar",
    "asfaltad": "asfaltar",
    "asfaltada": "asfaltar",
    "asfaltadas": "asfaltar",
    "asfaltado": "asfaltar",
    "asfaltados": "asfaltar",
    "asfaltamos": "asfaltar",
    "asfaltan": "asfaltar",
    "asfaltando": "asfaltar",
    "asfaltara": "asfaltar",
    "asfaltarais": "asfaltar",
    "asfaltaran": "asfaltar",
    "asfaltaras": "asfaltar",
    "asfaltare": "asfaltar",
    "asfaltareis": "asfaltar",
    "asfaltaremos": "asfaltar",
    "asfaltaren": "asfaltar",
    "asfaltares": "asfaltar",
    "asfaltaron": "asfaltar",
    "asfaltará": "asfaltar",
    "asfaltarán": "asfaltar",
    "asfaltarás": "asfaltar",
    "asfaltaré": "asfaltar",
    "asfaltaréis": "asfaltar",
    "asfaltaría": "asfaltar",
    "asfaltaríais": "asfaltar",
    "asfaltaríamos": "asfaltar",
    "asfaltarían": "asfaltar",
    "asfaltarías": "asfaltar",
    "asfaltas": "asfaltar",
    "asfaltase": "asfaltar",
    "asfaltaseis": "asfaltar",
    "asfaltasen": "asfaltar",
    "asfaltases": "asfaltar",
    "asfaltaste": "asfaltar",
    "asfaltasteis": "asfaltar",
    "asfalte": "asfaltar",
    "asfaltemos": "asfaltar",
    "asfalten": "asfaltar",
    "asfaltenos": "asfalteno",
    "asfaltes": "asfaltar",
    "asfaltitas": "asfaltita",
    "asfalto": "asfaltar",
    "asfaltos": "asfalto",
    "asfaltábamos": "asfaltar",
    "asfaltáis": "asfaltar",
    "asfaltáramos": "asfaltar",
    "asfaltáremos": "asfaltar",
    "asfaltásemos": "asfaltar",
    "asfalté": "asfaltar",
    "asfaltéis": "asfaltar",
    "asfaltó": "asfaltar",
    "asfixia": "asfixiar",
    "asfixiaba": "asfixiar",
    "asfixiabais": "asfixiar",
    "asfixiaban": "asfixiar",
    "asfixiabas": "asfixiar",
    "asfixiad": "asfixiar",
    "asfixiada": "asfixiar",
    "asfixiadas": "asfixiar",
    "asfixiado": "asfixiar",
    "asfixiados": "asfixiar",
    "asfixiamos": "asfixiar",
    "asfixian": "asfixiar",
    "asfixiando": "asfixiar",
    "asfixiantes": "asfixiante",
    "asfixiara": "asfixiar",
    "asfixiarais": "asfixiar",
    "asfixiaran": "asfixiar",
    "asfixiaras": "asfixiar",
    "asfixiare": "asfixiar",
    "asfixiareis": "asfixiar",
    "asfixiaremos": "asfixiar",
    "asfixiaren": "asfixiar",
    "asfixiares": "asfixiar",
    "asfixiaron": "asfixiar",
    "asfixiará": "asfixiar",
    "asfixiarán": "asfixiar",
    "asfixiarás": "asfixiar",
    "asfixiaré": "asfixiar",
    "asfixiaréis": "asfixiar",
    "asfixiaría": "asfixiar",
    "asfixiaríais": "asfixiar",
    "asfixiaríamos": "asfixiar",
    "asfixiarían": "asfixiar",
    "asfixiarías": "asfixiar",
    "asfixias": "asfixiar",
    "asfixiase": "asfixiar",
    "asfixiaseis": "asfixiar",
    "asfixiasen": "asfixiar",
    "asfixiases": "asfixiar",
    "asfixiaste": "asfixiar",
    "asfixiasteis": "asfixiar",
    "asfixie": "asfixiar",
    "asfixiemos": "asfixiar",
    "asfixien": "asfixiar",
    "asfixies": "asfixiar",
    "asfixio": "asfixiar",
    "asfixiábamos": "asfixiar",
    "asfixiáis": "asfixiar",
    "asfixiáramos": "asfixiar",
    "asfixiáremos": "asfixiar",
    "asfixiásemos": "asfixiar",
    "asfixié": "asfixiar",
    "asfixiéis": "asfixiar",
    "asfixió": "asfixiar",
    "asfáltica": "asfáltico",
    "asfálticas": "asfáltico",
    "asfálticos": "asfáltico",
    "asférica": "asférico",
    "asféricas": "asférico",
    "asféricos": "asférico",
    "asga": "asir",
    "asgamos": "asir",
    "asgan": "asir",
    "asgas": "asir",
    "asgo": "asir",
    "asgáis": "asir",
    "asialias": "asialia",
    "asianismos": "asianismo",
    "asibila": "asibilar",
    "asibilaba": "asibilar",
    "asibilabais": "asibilar",
    "asibilaban": "asibilar",
    "asibilabas": "asibilar",
    "asibilaciones": "asibilación",
    "asibilad": "asibilar",
    "asibilada": "asibilar",
    "asibiladas": "asibilar",
    "asibilado": "asibilar",
    "asibilados": "asibilar",
    "asibilamos": "asibilar",
    "asibilan": "asibilar",
    "asibilando": "asibilar",
    "asibilara": "asibilar",
    "asibilarais": "asibilar",
    "asibilaran": "asibilar",
    "asibilaras": "asibilar",
    "asibilare": "asibilar",
    "asibilareis": "asibilar",
    "asibilaremos": "asibilar",
    "asibilaren": "asibilar",
    "asibilares": "asibilar",
    "asibilaron": "asibilar",
    "asibilará": "asibilar",
    "asibilarán": "asibilar",
    "asibilarás": "asibilar",
    "asibilaré": "asibilar",
    "asibilaréis": "asibilar",
    "asibilaría": "asibilar",
    "asibilaríais": "asibilar",
    "asibilaríamos": "asibilar",
    "asibilarían": "asibilar",
    "asibilarías": "asibilar",
    "asibilas": "asibilar",
    "asibilase": "asibilar",
    "asibilaseis": "asibilar",
    "asibilasen": "asibilar",
    "asibilases": "asibilar",
    "asibilaste": "asibilar",
    "asibilasteis": "asibilar",
    "asibile": "asibilar",
    "asibilemos": "asibilar",
    "asibilen": "asibilar",
    "asibiles": "asibilar",
    "asibilo": "asibilar",
    "asibilábamos": "asibilar",
    "asibiláis": "asibilar",
    "asibiláramos": "asibilar",
    "asibiláremos": "asibilar",
    "asibilásemos": "asibilar",
    "asibilé": "asibilar",
    "asibiléis": "asibilar",
    "asibiló": "asibilar",
    "asibles": "asible",
    "asid": "asir",
    "asida": "asir",
    "asidas": "asir",
    "asideros": "asidero",
    "asido": "asir",
    "asidos": "asir",
    "asidua": "asiduo",
    "asiduas": "asiduo",
    "asiduidades": "asiduidad",
    "asiduos": "asiduo",
    "asiendo": "asir",
    "asienta": "asentir",
    "asientan": "asentir",
    "asientas": "asentir",
    "asiente": "asentir",
    "asienten": "asentir",
    "asientes": "asentir",
    "asiento": "asentir",
    "asientos": "asiento",
    "asiera": "asir",
    "asierais": "asir",
    "asieran": "asir",
    "asieras": "asir",
    "asiere": "asir",
    "asiereis": "asir",
    "asieren": "asir",
    "asieres": "asir",
    "asieron": "asir",
    "asierra": "aserrar",
    "asierran": "aserrar",
    "asierras": "aserrar",
    "asierre": "aserrar",
    "asierren": "aserrar",
    "asierres": "aserrar",
    "asierro": "aserrar",
    "asiese": "asir",
    "asieseis": "asir",
    "asiesen": "asir",
    "asieses": "asir",
    "asiesta": "asestar",
    "asiestan": "asestar",
    "asiestas": "asestar",
    "asieste": "asestar",
    "asiesten": "asestar",
    "asiestes": "asestar",
    "asiesto": "asestar",
    "asigna": "asignar",
    "asignaba": "asignar",
    "asignabais": "asignar",
    "asignaban": "asignar",
    "asignabas": "asignar",
    "asignables": "asignable",
    "asignaciones": "asignación",
    "asignad": "asignar",
    "asignada": "asignar",
    "asignadas": "asignar",
    "asignado": "asignar",
    "asignadora": "asignador",
    "asignadoras": "asignador",
    "asignadores": "asignador",
    "asignados": "asignar",
    "asignamos": "asignar",
    "asignan": "asignar",
    "asignando": "asignar",
    "asignara": "asignar",
    "asignarais": "asignar",
    "asignaran": "asignar",
    "asignaras": "asignar",
    "asignare": "asignar",
    "asignareis": "asignar",
    "asignaremos": "asignar",
    "asignaren": "asignar",
    "asignares": "asignar",
    "asignaron": "asignar",
    "asignará": "asignar",
    "asignarán": "asignar",
    "asignarás": "asignar",
    "asignaré": "asignar",
    "asignaréis": "asignar",
    "asignaría": "asignar",
    "asignaríais": "asignar",
    "asignaríamos": "asignar",
    "asignarían": "asignar",
    "asignarías": "asignar",
    "asignas": "asignar",
    "asignase": "asignar",
    "asignaseis": "asignar",
    "asignasen": "asignar",
    "asignases": "asignar",
    "asignaste": "asignar",
    "asignasteis": "asignar",
    "asignataria": "asignatario",
    "asignatarias": "asignatario",
    "asignatarios": "asignatario",
    "asignaturas": "asignatura",
    "asigne": "asignar",
    "asignemos": "asignar",
    "asignen": "asignar",
    "asignes": "asignar",
    "asigno": "asignar",
    "asignábamos": "asignar",
    "asignáis": "asignar",
    "asignáramos": "asignar",
    "asignáremos": "asignar",
    "asignásemos": "asignar",
    "asigné": "asignar",
    "asignéis": "asignar",
    "asignó": "asignar",
    "asila": "asilar",
    "asilaba": "asilar",
    "asilabais": "asilar",
    "asilaban": "asilar",
    "asilabas": "asilar",
    "asilad": "asilar",
    "asilada": "asilar",
    "asiladas": "asilar",
    "asilado": "asilar",
    "asilados": "asilar",
    "asilamos": "asilar",
    "asilan": "asilar",
    "asilando": "asilar",
    "asilara": "asilar",
    "asilarais": "asilar",
    "asilaran": "asilar",
    "asilaras": "asilar",
    "asilare": "asilar",
    "asilareis": "asilar",
    "asilaremos": "asilar",
    "asilaren": "asilar",
    "asilares": "asilar",
    "asilaron": "asilar",
    "asilará": "asilar",
    "asilarán": "asilar",
    "asilarás": "asilar",
    "asilaré": "asilar",
    "asilaréis": "asilar",
    "asilaría": "asilar",
    "asilaríais": "asilar",
    "asilaríamos": "asilar",
    "asilarían": "asilar",
    "asilarías": "asilar",
    "asilas": "asilar",
    "asilase": "asilar",
    "asilaseis": "asilar",
    "asilasen": "asilar",
    "asilases": "asilar",
    "asilaste": "asilar",
    "asilasteis": "asilar",
    "asile": "asilar",
    "asilemos": "asilar",
    "asilen": "asilar",
    "asiles": "asilar",
    "asilo": "asilar",
    "asilos": "asilo",
    "asilvestrada": "asilvestrado",
    "asilvestradas": "asilvestrado",
    "asilvestrados": "asilvestrado",
    "asilábamos": "asilar",
    "asilábica": "asilábico",
    "asilábicas": "asilábico",
    "asilábicos": "asilábico",
    "asiláis": "asilar",
    "asiláramos": "asilar",
    "asiláremos": "asilar",
    "asilásemos": "asilar",
    "asilé": "asilar",
    "asiléis": "asilar",
    "asiló": "asilar",
    "asimetrías": "asimetría",
    "asimila": "asimilar",
    "asimilaba": "asimilar",
    "asimilabais": "asimilar",
    "asimilaban": "asimilar",
    "asimilabas": "asimilar",
    "asimilables": "asimilable",
    "asimilaciones": "asimilación",
    "asimilad": "asimilar",
    "asimilada": "asimilar",
    "asimiladas": "asimilar",
    "asimilado": "asimilar",
    "asimilados": "asimilar",
    "asimilamos": "asimilar",
    "asimilan": "asimilar",
    "asimilando": "asimilar",
    "asimilara": "asimilar",
    "asimilarais": "asimilar",
    "asimilaran": "asimilar",
    "asimilaras": "asimilar",
    "asimilare": "asimilar",
    "asimilareis": "asimilar",
    "asimilaremos": "asimilar",
    "asimilaren": "asimilar",
    "asimilares": "asimilar",
    "asimilaron": "asimilar",
    "asimilará": "asimilar",
    "asimilarán": "asimilar",
    "asimilarás": "asimilar",
    "asimilaré": "asimilar",
    "asimilaréis": "asimilar",
    "asimilaría": "asimilar",
    "asimilaríais": "asimilar",
    "asimilaríamos": "asimilar",
    "asimilarían": "asimilar",
    "asimilarías": "asimilar",
    "asimilas": "asimilar",
    "asimilase": "asimilar",
    "asimilaseis": "asimilar",
    "asimilasen": "asimilar",
    "asimilases": "asimilar",
    "asimilaste": "asimilar",
    "asimilasteis": "asimilar",
    "asimilativa": "asimilativo",
    "asimilativas": "asimilativo",
    "asimilativos": "asimilativo",
    "asimilatoria": "asimilatorio",
    "asimilatorias": "asimilatorio",
    "asimilatorios": "asimilatorio",
    "asimile": "asimilar",
    "asimilemos": "asimilar",
    "asimilen": "asimilar",
    "asimiles": "asimilar",
    "asimilistas": "asimilista",
    "asimilo": "asimilar",
    "asimilábamos": "asimilar",
    "asimiláis": "asimilar",
    "asimiláramos": "asimilar",
    "asimiláremos": "asimilar",
    "asimilásemos": "asimilar",
    "asimilé": "asimilar",
    "asimiléis": "asimilar",
    "asimiló": "asimilar",
    "asimos": "asir",
    "asimplada": "asimplado",
    "asimpladas": "asimplado",
    "asimplados": "asimplado",
    "asimétrica": "asimétrico",
    "asimétricas": "asimétrico",
    "asimétricos": "asimétrico",
    "asina": "así",
    "asincronismos": "asincronismo",
    "asincrónica": "asincrónico",
    "asincrónicas": "asincrónico",
    "asincrónicos": "asincrónico",
    "asindética": "asindético",
    "asindéticas": "asindético",
    "asindéticos": "asindético",
    "asintamos": "asentir",
    "asintiendo": "asentir",
    "asintiera": "asentir",
    "asintierais": "asentir",
    "asintieran": "asentir",
    "asintieras": "asentir",
    "asintiere": "asentir",
    "asintiereis": "asentir",
    "asintieren": "asentir",
    "asintieres": "asentir",
    "asintieron": "asentir",
    "asintiese": "asentir",
    "asintieseis": "asentir",
    "asintiesen": "asentir",
    "asintieses": "asentir",
    "asintiéramos": "asentir",
    "asintiéremos": "asentir",
    "asintiésemos": "asentir",
    "asintió": "asentir",
    "asintáis": "asentir",
    "asintótica": "asintótico",
    "asintóticas": "asintótico",
    "asintóticos": "asintótico",
    "asiremos": "asir",
    "asiria": "asirio",
    "asirias": "asirio",
    "asiriologías": "asiriología",
    "asirios": "asirio",
    "asirióloga": "asiriólogo",
    "asiriólogas": "asiriólogo",
    "asiriólogos": "asiriólogo",
    "asirá": "asir",
    "asirán": "asir",
    "asirás": "asir",
    "asiré": "asir",
    "asiréis": "asir",
    "asiría": "asir",
    "asiríais": "asir",
    "asiríamos": "asir",
    "asirían": "asir",
    "asirías": "asir",
    "asista": "asistir",
    "asistamos": "asistir",
    "asistan": "asistir",
    "asistas": "asistir",
    "asiste": "asistir",
    "asisteis": "asir",
    "asisten": "asistir",
    "asistenciales": "asistencial",
    "asistencias": "asistencia",
    "asistenta": "asistente",
    "asistentas": "asistente",
    "asistentes": "asistente",
    "asistes": "asistir",
    "asistid": "asistir",
    "asistida": "asistir",
    "asistidas": "asistir",
    "asistido": "asistir",
    "asistidos": "asistir",
    "asistiendo": "asistir",
    "asistiera": "asistir",
    "asistierais": "asistir",
    "asistieran": "asistir",
    "asistieras": "asistir",
    "asistiere": "asistir",
    "asistiereis": "asistir",
    "asistieren": "asistir",
    "asistieres": "asistir",
    "asistieron": "asistir",
    "asistiese": "asistir",
    "asistieseis": "asistir",
    "asistiesen": "asistir",
    "asistieses": "asistir",
    "asistimos": "asistir",
    "asistiremos": "asistir",
    "asistirá": "asistir",
    "asistirán": "asistir",
    "asistirás": "asistir",
    "asistiré": "asistir",
    "asistiréis": "asistir",
    "asistiría": "asistir",
    "asistiríais": "asistir",
    "asistiríamos": "asistir",
    "asistirían": "asistir",
    "asistirías": "asistir",
    "asististe": "asistir",
    "asististeis": "asistir",
    "asistiéramos": "asistir",
    "asistiéremos": "asistir",
    "asistiésemos": "asistir",
    "asistió": "asistir",
    "asisto": "asistir",
    "asistolias": "asistolia",
    "asistáis": "asistir",
    "asistí": "asistir",
    "asistía": "asistir",
    "asistíais": "asistir",
    "asistíamos": "asistir",
    "asistían": "asistir",
    "asistías": "asistir",
    "asistís": "asistir",
    "asiática": "asiático",
    "asiáticas": "asiático",
    "asiáticos": "asiático",
    "asiéramos": "asir",
    "asiéremos": "asir",
    "asiésemos": "asir",
    "asió": "asir",
    "asmas": "asma",
    "asmática": "asmático",
    "asmáticas": "asmático",
    "asmáticos": "asmático",
    "asna": "asno",
    "asnales": "asnal",
    "asnas": "asno",
    "asnillas": "asnilla",
    "asnos": "asno",
    "aso": "asar",
    "asobarca": "asobarcar",
    "asobarcaba": "asobarcar",
    "asobarcabais": "asobarcar",
    "asobarcaban": "asobarcar",
    "asobarcabas": "asobarcar",
    "asobarcad": "asobarcar",
    "asobarcada": "asobarcar",
    "asobarcadas": "asobarcar",
    "asobarcado": "asobarcar",
    "asobarcados": "asobarcar",
    "asobarcamos": "asobarcar",
    "asobarcan": "asobarcar",
    "asobarcando": "asobarcar",
    "asobarcara": "asobarcar",
    "asobarcarais": "asobarcar",
    "asobarcaran": "asobarcar",
    "asobarcaras": "asobarcar",
    "asobarcare": "asobarcar",
    "asobarcareis": "asobarcar",
    "asobarcaremos": "asobarcar",
    "asobarcaren": "asobarcar",
    "asobarcares": "asobarcar",
    "asobarcaron": "asobarcar",
    "asobarcará": "asobarcar",
    "asobarcarán": "asobarcar",
    "asobarcarás": "asobarcar",
    "asobarcaré": "asobarcar",
    "asobarcaréis": "asobarcar",
    "asobarcaría": "asobarcar",
    "asobarcaríais": "asobarcar",
    "asobarcaríamos": "asobarcar",
    "asobarcarían": "asobarcar",
    "asobarcarías": "asobarcar",
    "asobarcas": "asobarcar",
    "asobarcase": "asobarcar",
    "asobarcaseis": "asobarcar",
    "asobarcasen": "asobarcar",
    "asobarcases": "asobarcar",
    "asobarcaste": "asobarcar",
    "asobarcasteis": "asobarcar",
    "asobarco": "asobarcar",
    "asobarcábamos": "asobarcar",
    "asobarcáis": "asobarcar",
    "asobarcáramos": "asobarcar",
    "asobarcáremos": "asobarcar",
    "asobarcásemos": "asobarcar",
    "asobarcó": "asobarcar",
    "asobarque": "asobarcar",
    "asobarquemos": "asobarcar",
    "asobarquen": "asobarcar",
    "asobarques": "asobarcar",
    "asobarqué": "asobarcar",
    "asobarquéis": "asobarcar",
    "asobina": "asobinar",
    "asobinaba": "asobinar",
    "asobinabais": "asobinar",
    "asobinaban": "asobinar",
    "asobinabas": "asobinar",
    "asobinad": "asobinar",
    "asobinada": "asobinar",
    "asobinadas": "asobinar",
    "asobinado": "asobinar",
    "asobinados": "asobinar",
    "asobinamos": "asobinar",
    "asobinan": "asobinar",
    "asobinando": "asobinar",
    "asobinara": "asobinar",
    "asobinarais": "asobinar",
    "asobinaran": "asobinar",
    "asobinaras": "asobinar",
    "asobinare": "asobinar",
    "asobinareis": "asobinar",
    "asobinaremos": "asobinar",
    "asobinaren": "asobinar",
    "asobinares": "asobinar",
    "asobinaron": "asobinar",
    "asobinará": "asobinar",
    "asobinarán": "asobinar",
    "asobinarás": "asobinar",
    "asobinaré": "asobinar",
    "asobinaréis": "asobinar",
    "asobinaría": "asobinar",
    "asobinaríais": "asobinar",
    "asobinaríamos": "asobinar",
    "asobinarían": "asobinar",
    "asobinarías": "asobinar",
    "asobinas": "asobinar",
    "asobinase": "asobinar",
    "asobinaseis": "asobinar",
    "asobinasen": "asobinar",
    "asobinases": "asobinar",
    "asobinaste": "asobinar",
    "asobinasteis": "asobinar",
    "asobine": "asobinar",
    "asobinemos": "asobinar",
    "asobinen": "asobinar",
    "asobines": "asobinar",
    "asobino": "asobinar",
    "asobinábamos": "asobinar",
    "asobináis": "asobinar",
    "asobináramos": "asobinar",
    "asobináremos": "asobinar",
    "asobinásemos": "asobinar",
    "asobiné": "asobinar",
    "asobinéis": "asobinar",
    "asobinó": "asobinar",
    "asocia": "asociar",
    "asociaba": "asociar",
    "asociabais": "asociar",
    "asociaban": "asociar",
    "asociabas": "asociar",
    "asociables": "asociable",
    "asociaciones": "asociación",
    "asociacionismos": "asociacionismo",
    "asociacionistas": "asociacionista",
    "asociad": "asociar",
    "asociada": "asociar",
    "asociadas": "asociar",
    "asociado": "asociar",
    "asociados": "asociar",
    "asociales": "asocial",
    "asocializaciones": "asocialización",
    "asociamos": "asociar",
    "asocian": "asociar",
    "asociando": "asociar",
    "asociara": "asociar",
    "asociarais": "asociar",
    "asociaran": "asociar",
    "asociaras": "asociar",
    "asociare": "asociar",
    "asociareis": "asociar",
    "asociaremos": "asociar",
    "asociaren": "asociar",
    "asociares": "asociar",
    "asociaron": "asociar",
    "asociará": "asociar",
    "asociarán": "asociar",
    "asociarás": "asociar",
    "asociaré": "asociar",
    "asociaréis": "asociar",
    "asociaría": "asociar",
    "asociaríais": "asociar",
    "asociaríamos": "asociar",
    "asociarían": "asociar",
    "asociarías": "asociar",
    "asocias": "asociar",
    "asociase": "asociar",
    "asociaseis": "asociar",
    "asociasen": "asociar",
    "asociases": "asociar",
    "asociaste": "asociar",
    "asociasteis": "asociar",
    "asociativa": "asociativo",
    "asociativas": "asociativo",
    "asociatividades": "asociatividad",
    "asociativos": "asociativo",
    "asocie": "asociar",
    "asociemos": "asociar",
    "asocien": "asociar",
    "asocies": "asociar",
    "asocio": "asociar",
    "asociábamos": "asociar",
    "asociáis": "asociar",
    "asociáramos": "asociar",
    "asociáremos": "asociar",
    "asociásemos": "asociar",
    "asocié": "asociar",
    "asociéis": "asociar",
    "asoció": "asociar",
    "asola": "asolar",
    "asolaba": "asolar",
    "asolabais": "asolar",
    "asolaban": "asolar",
    "asolabas": "asolar",
    "asolaciones": "asolación",
    "asolad": "asolar",
    "asolada": "asolar",
    "asoladas": "asolar",
    "asolado": "asolar",
    "asoladora": "asolador",
    "asoladoras": "asolador",
    "asoladores": "asolador",
    "asolados": "asolar",
    "asolamientos": "asolamiento",
    "asolamos": "asolar",
    "asolan": "asolar",
    "asolana": "asolanar",
    "asolanaba": "asolanar",
    "asolanabais": "asolanar",
    "asolanaban": "asolanar",
    "asolanabas": "asolanar",
    "asolanad": "asolanar",
    "asolanada": "asolanar",
    "asolanadas": "asolanar",
    "asolanado": "asolanar",
    "asolanados": "asolanar",
    "asolanamos": "asolanar",
    "asolanan": "asolanar",
    "asolanando": "asolanar",
    "asolanara": "asolanar",
    "asolanarais": "asolanar",
    "asolanaran": "asolanar",
    "asolanaras": "asolanar",
    "asolanare": "asolanar",
    "asolanareis": "asolanar",
    "asolanaremos": "asolanar",
    "asolanaren": "asolanar",
    "asolanares": "asolanar",
    "asolanaron": "asolanar",
    "asolanará": "asolanar",
    "asolanarán": "asolanar",
    "asolanarás": "asolanar",
    "asolanaré": "asolanar",
    "asolanaréis": "asolanar",
    "asolanaría": "asolanar",
    "asolanaríais": "asolanar",
    "asolanaríamos": "asolanar",
    "asolanarían": "asolanar",
    "asolanarías": "asolanar",
    "asolanas": "asolanar",
    "asolanase": "asolanar",
    "asolanaseis": "asolanar",
    "asolanasen": "asolanar",
    "asolanases": "asolanar",
    "asolanaste": "asolanar",
    "asolanasteis": "asolanar",
    "asolando": "asolar",
    "asolane": "asolanar",
    "asolanemos": "asolanar",
    "asolanen": "asolanar",
    "asolanes": "asolanar",
    "asolano": "asolanar",
    "asolanábamos": "asolanar",
    "asolanáis": "asolanar",
    "asolanáramos": "asolanar",
    "asolanáremos": "asolanar",
    "asolanásemos": "asolanar",
    "asolané": "asolanar",
    "asolanéis": "asolanar",
    "asolanó": "asolanar",
    "asolara": "asolar",
    "asolarais": "asolar",
    "asolaran": "asolar",
    "asolaras": "asolar",
    "asolare": "asolar",
    "asolareis": "asolar",
    "asolaremos": "asolar",
    "asolaren": "asolar",
    "asolares": "asolar",
    "asolaron": "asolar",
    "asolará": "asolar",
    "asolarán": "asolar",
    "asolarás": "asolar",
    "asolaré": "asolar",
    "asolaréis": "asolar",
    "asolaría": "asolar",
    "asolaríais": "asolar",
    "asolaríamos": "asolar",
    "asolarían": "asolar",
    "asolarías": "asolar",
    "asolas": "asolar",
    "asolase": "asolar",
    "asolaseis": "asolar",
    "asolasen": "asolar",
    "asolases": "asolar",
    "asolaste": "asolar",
    "asolasteis": "asolar",
    "asole": "asolar",
    "asolea": "asolear",
    "asoleaba": "asolear",
    "asoleabais": "asolear",
    "asoleaban": "asolear",
    "asoleabas": "asolear",
    "asolead": "asolear",
    "asoleada": "asolear",
    "asoleadas": "asolear",
    "asoleado": "asolear",
    "asoleados": "asolear",
    "asoleamos": "asolear",
    "asolean": "asolear",
    "asoleando": "asolear",
    "asoleara": "asolear",
    "asolearais": "asolear",
    "asolearan": "asolear",
    "asolearas": "asolear",
    "asoleare": "asolear",
    "asoleareis": "asolear",
    "asolearemos": "asolear",
    "asolearen": "asolear",
    "asoleares": "asolear",
    "asolearon": "asolear",
    "asoleará": "asolear",
    "asolearán": "asolear",
    "asolearás": "asolear",
    "asolearé": "asolear",
    "asolearéis": "asolear",
    "asolearía": "asolear",
    "asolearíais": "asolear",
    "asolearíamos": "asolear",
    "asolearían": "asolear",
    "asolearías": "asolear",
    "asoleas": "asolear",
    "asolease": "asolear",
    "asoleaseis": "asolear",
    "asoleasen": "asolear",
    "asoleases": "asolear",
    "asoleaste": "asolear",
    "asoleasteis": "asolear",
    "asolee": "asolear",
    "asoleemos": "asolear",
    "asoleen": "asolear",
    "asolees": "asolear",
    "asolemos": "asolar",
    "asolen": "asolar",
    "asoleo": "asolear",
    "asoles": "asolar",
    "asoleábamos": "asolear",
    "asoleáis": "asolear",
    "asoleáramos": "asolear",
    "asoleáremos": "asolear",
    "asoleásemos": "asolear",
    "asoleé": "asolear",
    "asoleéis": "asolear",
    "asoleó": "asolear",
    "asolo": "asolar",
    "asolábamos": "asolar",
    "asoláis": "asolar",
    "asoláramos": "asolar",
    "asoláremos": "asolar",
    "asolásemos": "asolar",
    "asolé": "asolar",
    "asoléis": "asolar",
    "asoló": "asolar",
    "asoma": "asomar",
    "asomaba": "asomar",
    "asomabais": "asomar",
    "asomaban": "asomar",
    "asomabas": "asomar",
    "asomad": "asomar",
    "asomada": "asomar",
    "asomadas": "asomar",
    "asomado": "asomar",
    "asomados": "asomar",
    "asomamos": "asomar",
    "asoman": "asomar",
    "asomando": "asomar",
    "asomara": "asomar",
    "asomarais": "asomar",
    "asomaran": "asomar",
    "asomaras": "asomar",
    "asomare": "asomar",
    "asomareis": "asomar",
    "asomaremos": "asomar",
    "asomaren": "asomar",
    "asomares": "asomar",
    "asomaron": "asomar",
    "asomará": "asomar",
    "asomarán": "asomar",
    "asomarás": "asomar",
    "asomaré": "asomar",
    "asomaréis": "asomar",
    "asomaría": "asomar",
    "asomaríais": "asomar",
    "asomaríamos": "asomar",
    "asomarían": "asomar",
    "asomarías": "asomar",
    "asomas": "asomar",
    "asomase": "asomar",
    "asomaseis": "asomar",
    "asomasen": "asomar",
    "asomases": "asomar",
    "asomaste": "asomar",
    "asomasteis": "asomar",
    "asombra": "asombrar",
    "asombraba": "asombrar",
    "asombrabais": "asombrar",
    "asombraban": "asombrar",
    "asombrabas": "asombrar",
    "asombrad": "asombrar",
    "asombrada": "asombrar",
    "asombradas": "asombrar",
    "asombrado": "asombrar",
    "asombrados": "asombrar",
    "asombramos": "asombrar",
    "asombran": "asombrar",
    "asombrando": "asombrar",
    "asombrara": "asombrar",
    "asombrarais": "asombrar",
    "asombraran": "asombrar",
    "asombraras": "asombrar",
    "asombrare": "asombrar",
    "asombrareis": "asombrar",
    "asombraremos": "asombrar",
    "asombraren": "asombrar",
    "asombrares": "asombrar",
    "asombraron": "asombrar",
    "asombrará": "asombrar",
    "asombrarán": "asombrar",
    "asombrarás": "asombrar",
    "asombraré": "asombrar",
    "asombraréis": "asombrar",
    "asombraría": "asombrar",
    "asombraríais": "asombrar",
    "asombraríamos": "asombrar",
    "asombrarían": "asombrar",
    "asombrarías": "asombrar",
    "asombras": "asombrar",
    "asombrase": "asombrar",
    "asombraseis": "asombrar",
    "asombrasen": "asombrar",
    "asombrases": "asombrar",
    "asombraste": "asombrar",
    "asombrasteis": "asombrar",
    "asombre": "asombrar",
    "asombremos": "asombrar",
    "asombren": "asombrar",
    "asombres": "asombrar",
    "asombro": "asombrar",
    "asombros": "asombro",
    "asombrosa": "asombroso",
    "asombrosas": "asombroso",
    "asombrosos": "asombroso",
    "asombrábamos": "asombrar",
    "asombráis": "asombrar",
    "asombráramos": "asombrar",
    "asombráremos": "asombrar",
    "asombrásemos": "asombrar",
    "asombré": "asombrar",
    "asombréis": "asombrar",
    "asombró": "asombrar",
    "asome": "asomar",
    "asomemos": "asomar",
    "asomen": "asomar",
    "asomes": "asomar",
    "asomo": "asomar",
    "asomos": "asomo",
    "asomábamos": "asomar",
    "asomáis": "asomar",
    "asomáramos": "asomar",
    "asomáremos": "asomar",
    "asomásemos": "asomar",
    "asomé": "asomar",
    "asoméis": "asomar",
    "asomó": "asomar",
    "asonaba": "asonar",
    "asonabais": "asonar",
    "asonaban": "asonar",
    "asonabas": "asonar",
    "asonad": "asonar",
    "asonada": "asonar",
    "asonadas": "asonar",
    "asonado": "asonar",
    "asonados": "asonar",
    "asonamos": "asonar",
    "asonancias": "asonancia",
    "asonando": "asonar",
    "asonanta": "asonantar",
    "asonantaba": "asonantar",
    "asonantabais": "asonantar",
    "asonantaban": "asonantar",
    "asonantabas": "asonantar",
    "asonantad": "asonantar",
    "asonantada": "asonantar",
    "asonantadas": "asonantar",
    "asonantado": "asonantar",
    "asonantados": "asonantar",
    "asonantamos": "asonantar",
    "asonantan": "asonantar",
    "asonantando": "asonantar",
    "asonantara": "asonantar",
    "asonantarais": "asonantar",
    "asonantaran": "asonantar",
    "asonantaras": "asonantar",
    "asonantare": "asonantar",
    "asonantareis": "asonantar",
    "asonantaremos": "asonantar",
    "asonantaren": "asonantar",
    "asonantares": "asonantar",
    "asonantaron": "asonantar",
    "asonantará": "asonantar",
    "asonantarán": "asonantar",
    "asonantarás": "asonantar",
    "asonantaré": "asonantar",
    "asonantaréis": "asonantar",
    "asonantaría": "asonantar",
    "asonantaríais": "asonantar",
    "asonantaríamos": "asonantar",
    "asonantarían": "asonantar",
    "asonantarías": "asonantar",
    "asonantas": "asonantar",
    "asonantase": "asonantar",
    "asonantaseis": "asonantar",
    "asonantasen": "asonantar",
    "asonantases": "asonantar",
    "asonantaste": "asonantar",
    "asonantasteis": "asonantar",
    "asonante": "asonantar",
    "asonantemos": "asonantar",
    "asonanten": "asonantar",
    "asonantes": "asonante",
    "asonanto": "asonantar",
    "asonantábamos": "asonantar",
    "asonantáis": "asonantar",
    "asonantáramos": "asonantar",
    "asonantáremos": "asonantar",
    "asonantásemos": "asonantar",
    "asonanté": "asonantar",
    "asonantéis": "asonantar",
    "asonantó": "asonantar",
    "asonara": "asonar",
    "asonarais": "asonar",
    "asonaran": "asonar",
    "asonaras": "asonar",
    "asonare": "asonar",
    "asonareis": "asonar",
    "asonaremos": "asonar",
    "asonaren": "asonar",
    "asonares": "asonar",
    "asonaron": "asonar",
    "asonará": "asonar",
    "asonarán": "asonar",
    "asonarás": "asonar",
    "asonaré": "asonar",
    "asonaréis": "asonar",
    "asonaría": "asonar",
    "asonaríais": "asonar",
    "asonaríamos": "asonar",
    "asonarían": "asonar",
    "asonarías": "asonar",
    "asonase": "asonar",
    "asonaseis": "asonar",
    "asonasen": "asonar",
    "asonases": "asonar",
    "asonaste": "asonar",
    "asonasteis": "asonar",
    "asonemos": "asonar",
    "asonábamos": "asonar",
    "asonáis": "asonar",
    "asonántica": "asonántico",
    "asonánticas": "asonántico",
    "asonánticos": "asonántico",
    "asonáramos": "asonar",
    "asonáremos": "asonar",
    "asonásemos": "asonar",
    "asoné": "asonar",
    "asonéis": "asonar",
    "asonó": "asonar",
    "asorocha": "asorochar",
    "asorochaba": "asorochar",
    "asorochabais": "asorochar",
    "asorochaban": "asorochar",
    "asorochabas": "asorochar",
    "asorochad": "asorochar",
    "asorochada": "asorochar",
    "asorochadas": "asorochar",
    "asorochado": "asorochar",
    "asorochados": "asorochar",
    "asorochamos": "asorochar",
    "asorochan": "asorochar",
    "asorochando": "asorochar",
    "asorochara": "asorochar",
    "asorocharais": "asorochar",
    "asorocharan": "asorochar",
    "asorocharas": "asorochar",
    "asorochare": "asorochar",
    "asorochareis": "asorochar",
    "asorocharemos": "asorochar",
    "asorocharen": "asorochar",
    "asorochares": "asorochar",
    "asorocharon": "asorochar",
    "asorochará": "asorochar",
    "asorocharán": "asorochar",
    "asorocharás": "asorochar",
    "asorocharé": "asorochar",
    "asorocharéis": "asorochar",
    "asorocharía": "asorochar",
    "asorocharíais": "asorochar",
    "asorocharíamos": "asorochar",
    "asorocharían": "asorochar",
    "asorocharías": "asorochar",
    "asorochas": "asorochar",
    "asorochase": "asorochar",
    "asorochaseis": "asorochar",
    "asorochasen": "asorochar",
    "asorochases": "asorochar",
    "asorochaste": "asorochar",
    "asorochasteis": "asorochar",
    "asoroche": "asorochar",
    "asorochemos": "asorochar",
    "asorochen": "asorochar",
    "asoroches": "asorochar",
    "asorocho": "asorochar",
    "asorochábamos": "asorochar",
    "asorocháis": "asorochar",
    "asorocháramos": "asorochar",
    "asorocháremos": "asorochar",
    "asorochásemos": "asorochar",
    "asoroché": "asorochar",
    "asorochéis": "asorochar",
    "asorochó": "asorochar",
    "aspa": "aspar",
    "aspaba": "aspar",
    "aspabais": "aspar",
    "aspaban": "aspar",
    "aspabas": "aspar",
    "aspad": "aspar",
    "aspada": "aspar",
    "aspadas": "aspar",
    "aspaderas": "aspadera",
    "aspado": "aspar",
    "aspadora": "aspador",
    "aspadoras": "aspador",
    "aspadores": "aspador",
    "aspados": "aspar",
    "aspamos": "aspar",
    "aspan": "aspar",
    "aspando": "aspar",
    "aspara": "aspar",
    "asparais": "aspar",
    "asparan": "aspar",
    "asparas": "aspar",
    "aspare": "aspar",
    "aspareis": "aspar",
    "asparemos": "aspar",
    "asparen": "aspar",
    "aspares": "aspar",
    "asparon": "aspar",
    "aspará": "aspar",
    "asparán": "aspar",
    "asparás": "aspar",
    "asparé": "aspar",
    "asparéis": "aspar",
    "asparía": "aspar",
    "asparíais": "aspar",
    "asparíamos": "aspar",
    "asparían": "aspar",
    "asparías": "aspar",
    "aspas": "aspar",
    "aspase": "aspar",
    "aspaseis": "aspar",
    "aspasen": "aspar",
    "aspases": "aspar",
    "aspaste": "aspar",
    "aspasteis": "aspar",
    "aspaventaba": "aspaventar",
    "aspaventabais": "aspaventar",
    "aspaventaban": "aspaventar",
    "aspaventabas": "aspaventar",
    "aspaventad": "aspaventar",
    "aspaventada": "aspaventar",
    "aspaventadas": "aspaventar",
    "aspaventado": "aspaventar",
    "aspaventados": "aspaventar",
    "aspaventamos": "aspaventar",
    "aspaventando": "aspaventar",
    "aspaventara": "aspaventar",
    "aspaventarais": "aspaventar",
    "aspaventaran": "aspaventar",
    "aspaventaras": "aspaventar",
    "aspaventare": "aspaventar",
    "aspaventareis": "aspaventar",
    "aspaventaremos": "aspaventar",
    "aspaventaren": "aspaventar",
    "aspaventares": "aspaventar",
    "aspaventaron": "aspaventar",
    "aspaventará": "aspaventar",
    "aspaventarán": "aspaventar",
    "aspaventarás": "aspaventar",
    "aspaventaré": "aspaventar",
    "aspaventaréis": "aspaventar",
    "aspaventaría": "aspaventar",
    "aspaventaríais": "aspaventar",
    "aspaventaríamos": "aspaventar",
    "aspaventarían": "aspaventar",
    "aspaventarías": "aspaventar",
    "aspaventase": "aspaventar",
    "aspaventaseis": "aspaventar",
    "aspaventasen": "aspaventar",
    "aspaventases": "aspaventar",
    "aspaventaste": "aspaventar",
    "aspaventasteis": "aspaventar",
    "aspaventemos": "aspaventar",
    "aspaventera": "aspaventero",
    "aspaventeras": "aspaventero",
    "aspaventeros": "aspaventero",
    "aspaventábamos": "aspaventar",
    "aspaventáis": "aspaventar",
    "aspaventáramos": "aspaventar",
    "aspaventáremos": "aspaventar",
    "aspaventásemos": "aspaventar",
    "aspaventé": "aspaventar",
    "aspaventéis": "aspaventar",
    "aspaventó": "aspaventar",
    "aspavienta": "aspaventar",
    "aspavientan": "aspaventar",
    "aspavientas": "aspaventar",
    "aspaviente": "aspaventar",
    "aspavienten": "aspaventar",
    "aspavientes": "aspaventar",
    "aspaviento": "aspaventar",
    "aspavientos": "aspaviento",
    "aspe": "aspar",
    "aspectos": "aspecto",
    "aspectuales": "aspectual",
    "aspemos": "aspar",
    "aspen": "aspar",
    "asperaciones": "asperación",
    "asperea": "asperear",
    "aspereaba": "asperear",
    "aspereabais": "asperear",
    "aspereaban": "asperear",
    "aspereabas": "asperear",
    "asperead": "asperear",
    "aspereada": "asperear",
    "aspereadas": "asperear",
    "aspereado": "asperear",
    "aspereados": "asperear",
    "aspereamos": "asperear",
    "asperean": "asperear",
    "aspereando": "asperear",
    "aspereara": "asperear",
    "asperearais": "asperear",
    "asperearan": "asperear",
    "asperearas": "asperear",
    "aspereare": "asperear",
    "aspereareis": "asperear",
    "asperearemos": "asperear",
    "asperearen": "asperear",
    "aspereares": "asperear",
    "asperearon": "asperear",
    "aspereará": "asperear",
    "asperearán": "asperear",
    "asperearás": "asperear",
    "asperearé": "asperear",
    "asperearéis": "asperear",
    "asperearía": "asperear",
    "asperearíais": "asperear",
    "asperearíamos": "asperear",
    "asperearían": "asperear",
    "asperearías": "asperear",
    "aspereas": "asperear",
    "asperease": "asperear",
    "aspereaseis": "asperear",
    "aspereasen": "asperear",
    "aspereases": "asperear",
    "aspereaste": "asperear",
    "aspereasteis": "asperear",
    "asperee": "asperear",
    "aspereemos": "asperear",
    "aspereen": "asperear",
    "asperees": "asperear",
    "aspereo": "asperear",
    "asperezas": "aspereza",
    "aspereábamos": "asperear",
    "aspereáis": "asperear",
    "aspereáramos": "asperear",
    "aspereáremos": "asperear",
    "aspereásemos": "asperear",
    "aspereé": "asperear",
    "aspereéis": "asperear",
    "aspereó": "asperear",
    "asperge": "asperger",
    "asperged": "asperger",
    "aspergemos": "asperger",
    "aspergen": "asperger",
    "aspergeremos": "asperger",
    "aspergerá": "asperger",
    "aspergerán": "asperger",
    "aspergerás": "asperger",
    "aspergeré": "asperger",
    "aspergeréis": "asperger",
    "aspergería": "asperger",
    "aspergeríais": "asperger",
    "aspergeríamos": "asperger",
    "aspergerían": "asperger",
    "aspergerías": "asperger",
    "asperges": "asperger",
    "aspergida": "asperger",
    "aspergidas": "asperger",
    "aspergido": "asperger",
    "aspergidos": "asperger",
    "aspergiendo": "asperger",
    "aspergiera": "asperger",
    "aspergierais": "asperger",
    "aspergieran": "asperger",
    "aspergieras": "asperger",
    "aspergiere": "asperger",
    "aspergiereis": "asperger",
    "aspergieren": "asperger",
    "aspergieres": "asperger",
    "aspergieron": "asperger",
    "aspergiese": "asperger",
    "aspergieseis": "asperger",
    "aspergiesen": "asperger",
    "aspergieses": "asperger",
    "aspergimos": "asperger",
    "aspergiste": "asperger",
    "aspergisteis": "asperger",
    "aspergiéramos": "asperger",
    "aspergiéremos": "asperger",
    "aspergiésemos": "asperger",
    "aspergió": "asperger",
    "aspergéis": "asperger",
    "aspergí": "asperger",
    "aspergía": "asperger",
    "aspergíais": "asperger",
    "aspergíamos": "asperger",
    "aspergían": "asperger",
    "aspergías": "asperger",
    "asperillas": "asperilla",
    "asperillos": "asperillo",
    "asperja": "asperjar",
    "asperjaba": "asperjar",
    "asperjabais": "asperjar",
    "asperjaban": "asperjar",
    "asperjabas": "asperjar",
    "asperjad": "asperjar",
    "asperjada": "asperjar",
    "asperjadas": "asperjar",
    "asperjado": "asperjar",
    "asperjados": "asperjar",
    "asperjamos": "asperjar",
    "asperjan": "asperjar",
    "asperjando": "asperjar",
    "asperjara": "asperjar",
    "asperjarais": "asperjar",
    "asperjaran": "asperjar",
    "asperjaras": "asperjar",
    "asperjare": "asperjar",
    "asperjareis": "asperjar",
    "asperjaremos": "asperjar",
    "asperjaren": "asperjar",
    "asperjares": "asperjar",
    "asperjaron": "asperjar",
    "asperjará": "asperjar",
    "asperjarán": "asperjar",
    "asperjarás": "asperjar",
    "asperjaré": "asperjar",
    "asperjaréis": "asperjar",
    "asperjaría": "asperjar",
    "asperjaríais": "asperjar",
    "asperjaríamos": "asperjar",
    "asperjarían": "asperjar",
    "asperjarías": "asperjar",
    "asperjas": "asperjar",
    "asperjase": "asperjar",
    "asperjaseis": "asperjar",
    "asperjasen": "asperjar",
    "asperjases": "asperjar",
    "asperjaste": "asperjar",
    "asperjasteis": "asperjar",
    "asperje": "asperjar",
    "asperjemos": "asperjar",
    "asperjen": "asperjar",
    "asperjes": "asperjar",
    "asperjo": "asperjar",
    "asperjábamos": "asperjar",
    "asperjáis": "asperjar",
    "asperjáramos": "asperjar",
    "asperjáremos": "asperjar",
    "asperjásemos": "asperjar",
    "asperjé": "asperjar",
    "asperjéis": "asperjar",
    "asperjó": "asperjar",
    "asperones": "asperón",
    "aspersiones": "aspersión",
    "aspersora": "aspersor",
    "aspersoras": "aspersor",
    "aspersores": "aspersor",
    "aspersorios": "aspersorio",
    "aspes": "aspar",
    "aspilleras": "aspillera",
    "aspira": "aspirar",
    "aspiraba": "aspirar",
    "aspirabais": "aspirar",
    "aspiraban": "aspirar",
    "aspirabas": "aspirar",
    "aspiraciones": "aspiración",
    "aspirad": "aspirar",
    "aspirada": "aspirar",
    "aspiradas": "aspirar",
    "aspirado": "aspirar",
    "aspiradora": "aspirador",
    "aspiradoras": "aspirador",
    "aspiradores": "aspirador",
    "aspirados": "aspirar",
    "aspiramos": "aspirar",
    "aspiran": "aspirar",
    "aspirando": "aspirar",
    "aspirantes": "aspirante",
    "aspirara": "aspirar",
    "aspirarais": "aspirar",
    "aspiraran": "aspirar",
    "aspiraras": "aspirar",
    "aspirare": "aspirar",
    "aspirareis": "aspirar",
    "aspiraremos": "aspirar",
    "aspiraren": "aspirar",
    "aspirares": "aspirar",
    "aspiraron": "aspirar",
    "aspirará": "aspirar",
    "aspirarán": "aspirar",
    "aspirarás": "aspirar",
    "aspiraré": "aspirar",
    "aspiraréis": "aspirar",
    "aspiraría": "aspirar",
    "aspiraríais": "aspirar",
    "aspiraríamos": "aspirar",
    "aspirarían": "aspirar",
    "aspirarías": "aspirar",
    "aspiras": "aspirar",
    "aspirase": "aspirar",
    "aspiraseis": "aspirar",
    "aspirasen": "aspirar",
    "aspirases": "aspirar",
    "aspiraste": "aspirar",
    "aspirasteis": "aspirar",
    "aspire": "aspirar",
    "aspiremos": "aspirar",
    "aspiren": "aspirar",
    "aspires": "aspirar",
    "aspirinas": "aspirina",
    "aspiro": "aspirar",
    "aspiropsicrómetros": "aspiropsicrómetro",
    "aspirotermómetros": "aspirotermómetro",
    "aspirábamos": "aspirar",
    "aspiráis": "aspirar",
    "aspiráramos": "aspirar",
    "aspiráremos": "aspirar",
    "aspirásemos": "aspirar",
    "aspiré": "aspirar",
    "aspiréis": "aspirar",
    "aspiró": "aspirar",
    "aspo": "aspar",
    "aspábamos": "aspar",
    "aspáis": "aspar",
    "aspáramos": "aspar",
    "aspáremos": "aspar",
    "aspártica": "aspártico",
    "aspárticas": "aspártico",
    "aspárticos": "aspártico",
    "aspásemos": "aspar",
    "aspé": "aspar",
    "aspéis": "aspar",
    "aspó": "aspar",
    "asquea": "asquear",
    "asqueaba": "asquear",
    "asqueabais": "asquear",
    "asqueaban": "asquear",
    "asqueabas": "asquear",
    "asquead": "asquear",
    "asqueada": "asquear",
    "asqueadas": "asquear",
    "asqueado": "asquear",
    "asqueados": "asquear",
    "asqueamientos": "asqueamiento",
    "asqueamos": "asquear",
    "asquean": "asquear",
    "asqueando": "asquear",
    "asqueantes": "asqueante",
    "asqueara": "asquear",
    "asquearais": "asquear",
    "asquearan": "asquear",
    "asquearas": "asquear",
    "asqueare": "asquear",
    "asqueareis": "asquear",
    "asquearemos": "asquear",
    "asquearen": "asquear",
    "asqueares": "asquear",
    "asquearon": "asquear",
    "asqueará": "asquear",
    "asquearán": "asquear",
    "asquearás": "asquear",
    "asquearé": "asquear",
    "asquearéis": "asquear",
    "asquearía": "asquear",
    "asquearíais": "asquear",
    "asquearíamos": "asquear",
    "asquearían": "asquear",
    "asquearías": "asquear",
    "asqueas": "asquear",
    "asquease": "asquear",
    "asqueaseis": "asquear",
    "asqueasen": "asquear",
    "asqueases": "asquear",
    "asqueaste": "asquear",
    "asqueasteis": "asquear",
    "asquee": "asquear",
    "asqueemos": "asquear",
    "asqueen": "asquear",
    "asquees": "asquear",
    "asqueo": "asquear",
    "asquerosa": "asqueroso",
    "asquerosas": "asqueroso",
    "asquerosidades": "asquerosidad",
    "asquerosos": "asqueroso",
    "asqueábamos": "asquear",
    "asqueáis": "asquear",
    "asqueáramos": "asquear",
    "asqueáremos": "asquear",
    "asqueásemos": "asquear",
    "asqueé": "asquear",
    "asqueéis": "asquear",
    "asqueó": "asquear",
    "astables": "astable",
    "astada": "astado",
    "astadas": "astado",
    "astados": "astado",
    "astas": "asta",
    "astatinos": "astatino",
    "astatos": "astato",
    "astenias": "astenia",
    "astenopías": "astenopía",
    "astenosferas": "astenosfera",
    "asteres": "aster",
    "asteriscos": "asterisco",
    "asterismos": "asterismo",
    "asteroideos": "asteroideo",
    "asteroides": "asteroide",
    "astigmatismos": "astigmatismo",
    "astigmática": "astigmático",
    "astigmáticas": "astigmático",
    "astigmáticos": "astigmático",
    "astiles": "astil",
    "astilla": "astillar",
    "astillaba": "astillar",
    "astillabais": "astillar",
    "astillaban": "astillar",
    "astillabas": "astillar",
    "astillad": "astillar",
    "astillada": "astillar",
    "astilladas": "astillar",
    "astillado": "astillar",
    "astilladoras": "astilladora",
    "astillados": "astillar",
    "astillamientos": "astillamiento",
    "astillamos": "astillar",
    "astillan": "astillar",
    "astillando": "astillar",
    "astillara": "astillar",
    "astillarais": "astillar",
    "astillaran": "astillar",
    "astillaras": "astillar",
    "astillare": "astillar",
    "astillareis": "astillar",
    "astillaremos": "astillar",
    "astillaren": "astillar",
    "astillares": "astillar",
    "astillaron": "astillar",
    "astillará": "astillar",
    "astillarán": "astillar",
    "astillarás": "astillar",
    "astillaré": "astillar",
    "astillaréis": "astillar",
    "astillaría": "astillar",
    "astillaríais": "astillar",
    "astillaríamos": "astillar",
    "astillarían": "astillar",
    "astillarías": "astillar",
    "astillas": "astillar",
    "astillase": "astillar",
    "astillaseis": "astillar",
    "astillasen": "astillar",
    "astillases": "astillar",
    "astillaste": "astillar",
    "astillasteis": "astillar",
    "astillazos": "astillazo",
    "astille": "astillar",
    "astillemos": "astillar",
    "astillen": "astillar",
    "astilleros": "astillero",
    "astilles": "astillar",
    "astillo": "astillar",
    "astillosa": "astilloso",
    "astillosas": "astilloso",
    "astillosos": "astilloso",
    "astillábamos": "astillar",
    "astilláis": "astillar",
    "astilláramos": "astillar",
    "astilláremos": "astillar",
    "astillásemos": "astillar",
    "astillé": "astillar",
    "astilléis": "astillar",
    "astilló": "astillar",
    "astracanadas": "astracanada",
    "astracanes": "astracán",
    "astragalina": "astragalino",
    "astragalinas": "astragalino",
    "astragalinos": "astragalino",
    "astragalocalcaneoescafoidea": "astragalocalcaneoescafoideo",
    "astragalocalcaneoescafoideas": "astragalocalcaneoescafoideo",
    "astragalocalcaneoescafoideos": "astragalocalcaneoescafoideo",
    "astragalocalcaneonaviculares": "astragalocalcaneonavicular",
    "astragalocalcánea": "astragalocalcáneo",
    "astragalocalcáneas": "astragalocalcáneo",
    "astragalocalcáneos": "astragalocalcáneo",
    "astragaloescafoidea": "astragaloescafoideo",
    "astragaloescafoideas": "astragaloescafoideo",
    "astragaloescafoideos": "astragaloescafoideo",
    "astrales": "astral",
    "astreñid": "astreñir",
    "astreñida": "astreñir",
    "astreñidas": "astreñir",
    "astreñido": "astreñir",
    "astreñidos": "astreñir",
    "astreñimos": "astreñir",
    "astreñiremos": "astreñir",
    "astreñirá": "astreñir",
    "astreñirán": "astreñir",
    "astreñirás": "astreñir",
    "astreñiré": "astreñir",
    "astreñiréis": "astreñir",
    "astreñiría": "astreñir",
    "astreñiríais": "astreñir",
    "astreñiríamos": "astreñir",
    "astreñirían": "astreñir",
    "astreñirías": "astreñir",
    "astreñiste": "astreñir",
    "astreñisteis": "astreñir",
    "astreñí": "astreñir",
    "astreñía": "astreñir",
    "astreñíais": "astreñir",
    "astreñíamos": "astreñir",
    "astreñían": "astreñir",
    "astreñías": "astreñir",
    "astreñís": "astreñir",
    "astricciones": "astricción",
    "astricta": "astringir",
    "astrictas": "astringir",
    "astrictiva": "astrictivo",
    "astrictivas": "astrictivo",
    "astrictivos": "astrictivo",
    "astricto": "astringir",
    "astrictos": "astringir",
    "astringe": "astringir",
    "astringen": "astringir",
    "astringentes": "astringente",
    "astringes": "astringir",
    "astringid": "astringir",
    "astringida": "astringir",
    "astringidas": "astringir",
    "astringido": "astringir",
    "astringidos": "astringir",
    "astringiendo": "astringir",
    "astringiera": "astringir",
    "astringierais": "astringir",
    "astringieran": "astringir",
    "astringieras": "astringir",
    "astringiere": "astringir",
    "astringiereis": "astringir",
    "astringieren": "astringir",
    "astringieres": "astringir",
    "astringieron": "astringir",
    "astringiese": "astringir",
    "astringieseis": "astringir",
    "astringiesen": "astringir",
    "astringieses": "astringir",
    "astringimos": "astringir",
    "astringiremos": "astringir",
    "astringirá": "astringir",
    "astringirán": "astringir",
    "astringirás": "astringir",
    "astringiré": "astringir",
    "astringiréis": "astringir",
    "astringiría": "astringir",
    "astringiríais": "astringir",
    "astringiríamos": "astringir",
    "astringirían": "astringir",
    "astringirías": "astringir",
    "astringiste": "astringir",
    "astringisteis": "astringir",
    "astringiéramos": "astringir",
    "astringiéremos": "astringir",
    "astringiésemos": "astringir",
    "astringió": "astringir",
    "astringí": "astringir",
    "astringía": "astringir",
    "astringíais": "astringir",
    "astringíamos": "astringir",
    "astringían": "astringir",
    "astringías": "astringir",
    "astringís": "astringir",
    "astrinja": "astringir",
    "astrinjamos": "astringir",
    "astrinjan": "astringir",
    "astrinjas": "astringir",
    "astrinjo": "astringir",
    "astrinjáis": "astringir",
    "astriña": "astriñir",
    "astriñamos": "astriñir",
    "astriñan": "astriñir",
    "astriñas": "astriñir",
    "astriñe": "astriñir",
    "astriñen": "astriñir",
    "astriñendo": "astriñir",
    "astriñera": "astriñir",
    "astriñerais": "astriñir",
    "astriñeran": "astriñir",
    "astriñeras": "astriñir",
    "astriñere": "astriñir",
    "astriñereis": "astriñir",
    "astriñeren": "astriñir",
    "astriñeres": "astriñir",
    "astriñeron": "astriñir",
    "astriñes": "astriñir",
    "astriñese": "astriñir",
    "astriñeseis": "astriñir",
    "astriñesen": "astriñir",
    "astriñeses": "astriñir",
    "astriñid": "astriñir",
    "astriñida": "astriñir",
    "astriñidas": "astriñir",
    "astriñido": "astriñir",
    "astriñidos": "astriñir",
    "astriñimos": "astriñir",
    "astriñiremos": "astriñir",
    "astriñirá": "astriñir",
    "astriñirán": "astriñir",
    "astriñirás": "astriñir",
    "astriñiré": "astriñir",
    "astriñiréis": "astriñir",
    "astriñiría": "astriñir",
    "astriñiríais": "astriñir",
    "astriñiríamos": "astriñir",
    "astriñirían": "astriñir",
    "astriñirías": "astriñir",
    "astriñiste": "astriñir",
    "astriñisteis": "astriñir",
    "astriño": "astriñir",
    "astriñáis": "astriñir",
    "astriñéramos": "astriñir",
    "astriñéremos": "astriñir",
    "astriñésemos": "astriñir",
    "astriñí": "astriñir",
    "astriñía": "astriñir",
    "astriñíais": "astriñir",
    "astriñíamos": "astriñir",
    "astriñían": "astriñir",
    "astriñías": "astriñir",
    "astriñís": "astriñir",
    "astriñó": "astriñir",
    "astrocitos": "astrocito",
    "astrodinámicas": "astrodinámica",
    "astrofotografías": "astrofotografía",
    "astrofotometrías": "astrofotometría",
    "astrofísica": "astrofísico",
    "astrofísicas": "astrofísico",
    "astrofísicos": "astrofísico",
    "astrogeologías": "astrogeología",
    "astrografías": "astrografía",
    "astroides": "astroide",
    "astrolabios": "astrolabio",
    "astrolitos": "astrolito",
    "astrologa": "astrologar",
    "astrologaba": "astrologar",
    "astrologabais": "astrologar",
    "astrologaban": "astrologar",
    "astrologabas": "astrologar",
    "astrologad": "astrologar",
    "astrologada": "astrologar",
    "astrologadas": "astrologar",
    "astrologado": "astrologar",
    "astrologados": "astrologar",
    "astrologamos": "astrologar",
    "astrologan": "astrologar",
    "astrologando": "astrologar",
    "astrologara": "astrologar",
    "astrologarais": "astrologar",
    "astrologaran": "astrologar",
    "astrologaras": "astrologar",
    "astrologare": "astrologar",
    "astrologareis": "astrologar",
    "astrologaremos": "astrologar",
    "astrologaren": "astrologar",
    "astrologares": "astrologar",
    "astrologaron": "astrologar",
    "astrologará": "astrologar",
    "astrologarán": "astrologar",
    "astrologarás": "astrologar",
    "astrologaré": "astrologar",
    "astrologaréis": "astrologar",
    "astrologaría": "astrologar",
    "astrologaríais": "astrologar",
    "astrologaríamos": "astrologar",
    "astrologarían": "astrologar",
    "astrologarías": "astrologar",
    "astrologas": "astrologar",
    "astrologase": "astrologar",
    "astrologaseis": "astrologar",
    "astrologasen": "astrologar",
    "astrologases": "astrologar",
    "astrologaste": "astrologar",
    "astrologasteis": "astrologar",
    "astrologo": "astrologar",
    "astrologue": "astrologar",
    "astrologuemos": "astrologar",
    "astrologuen": "astrologar",
    "astrologues": "astrologar",
    "astrologué": "astrologar",
    "astrologuéis": "astrologar",
    "astrologábamos": "astrologar",
    "astrologáis": "astrologar",
    "astrologáramos": "astrologar",
    "astrologáremos": "astrologar",
    "astrologásemos": "astrologar",
    "astrologías": "astrología",
    "astrologó": "astrologar",
    "astrológica": "astrológico",
    "astrológicas": "astrológico",
    "astrológicos": "astrológico",
    "astrometrías": "astrometría",
    "astronautas": "astronauta",
    "astronaves": "astronave",
    "astronomías": "astronomía",
    "astronáuticas": "astronáutica",
    "astronómica": "astronómico",
    "astronómicas": "astronómico",
    "astronómicos": "astronómico",
    "astronónica": "astronónico",
    "astronónicas": "astronónico",
    "astronónicos": "astronónico",
    "astros": "astro",
    "astrosa": "astroso",
    "astrosas": "astroso",
    "astrosos": "astroso",
    "astrágalos": "astrágalo",
    "astrógrafos": "astrógrafo",
    "astróloga": "astrólogo",
    "astrólogas": "astrólogo",
    "astrólogos": "astrólogo",
    "astrónoma": "astrónomo",
    "astrónomas": "astrónomo",
    "astrónomos": "astrónomo",
    "astucias": "astucia",
    "astures": "astur",
    "asturiana": "asturiano",
    "asturianas": "asturiano",
    "asturianismos": "asturianismo",
    "asturianos": "asturiano",
    "astuta": "astuto",
    "astutas": "astuto",
    "astutos": "astuto",
    "astática": "astático",
    "astáticas": "astático",
    "astáticos": "astático",
    "asténica": "asténico",
    "asténicas": "asténico",
    "asténicos": "asténico",
    "asuela": "asolar",
    "asuelan": "asolar",
    "asuelas": "asolar",
    "asuele": "asolar",
    "asuelen": "asolar",
    "asueles": "asolar",
    "asuelo": "asolar",
    "asuena": "asonar",
    "asuenan": "asonar",
    "asuenas": "asonar",
    "asuene": "asonar",
    "asuenen": "asonar",
    "asuenes": "asonar",
    "asueno": "asonar",
    "asuetos": "asueto",
    "asuma": "asumir",
    "asumamos": "asumir",
    "asuman": "asumir",
    "asumas": "asumir",
    "asume": "asumir",
    "asumen": "asumir",
    "asumes": "asumir",
    "asumibles": "asumible",
    "asumid": "asumir",
    "asumida": "asumir",
    "asumidas": "asumir",
    "asumido": "asumir",
    "asumidos": "asumir",
    "asumiendo": "asumir",
    "asumiera": "asumir",
    "asumierais": "asumir",
    "asumieran": "asumir",
    "asumieras": "asumir",
    "asumiere": "asumir",
    "asumiereis": "asumir",
    "asumieren": "asumir",
    "asumieres": "asumir",
    "asumieron": "asumir",
    "asumiese": "asumir",
    "asumieseis": "asumir",
    "asumiesen": "asumir",
    "asumieses": "asumir",
    "asumimos": "asumir",
    "asumiremos": "asumir",
    "asumirá": "asumir",
    "asumirán": "asumir",
    "asumirás": "asumir",
    "asumiré": "asumir",
    "asumiréis": "asumir",
    "asumiría": "asumir",
    "asumiríais": "asumir",
    "asumiríamos": "asumir",
    "asumirían": "asumir",
    "asumirías": "asumir",
    "asumiste": "asumir",
    "asumisteis": "asumir",
    "asumiéramos": "asumir",
    "asumiéremos": "asumir",
    "asumiésemos": "asumir",
    "asumió": "asumir",
    "asumo": "asumir",
    "asumáis": "asumir",
    "asumí": "asumir",
    "asumía": "asumir",
    "asumíais": "asumir",
    "asumíamos": "asumir",
    "asumían": "asumir",
    "asumías": "asumir",
    "asumís": "asumir",
    "asunciones": "asunción",
    "asunciónes": "asunción",
    "asuntos": "asunto",
    "asura": "asurar",
    "asuraba": "asurar",
    "asurabais": "asurar",
    "asuraban": "asurar",
    "asurabas": "asurar",
    "asurad": "asurar",
    "asurada": "asurar",
    "asuradas": "asurar",
    "asurado": "asurar",
    "asurados": "asurar",
    "asuramos": "asurar",
    "asuran": "asurar",
    "asurando": "asurar",
    "asurara": "asurar",
    "asurarais": "asurar",
    "asuraran": "asurar",
    "asuraras": "asurar",
    "asurare": "asurar",
    "asurareis": "asurar",
    "asuraremos": "asurar",
    "asuraren": "asurar",
    "asurares": "asurar",
    "asuraron": "asurar",
    "asurará": "asurar",
    "asurarán": "asurar",
    "asurarás": "asurar",
    "asuraré": "asurar",
    "asuraréis": "asurar",
    "asuraría": "asurar",
    "asuraríais": "asurar",
    "asuraríamos": "asurar",
    "asurarían": "asurar",
    "asurarías": "asurar",
    "asuras": "asurar",
    "asurase": "asurar",
    "asuraseis": "asurar",
    "asurasen": "asurar",
    "asurases": "asurar",
    "asuraste": "asurar",
    "asurasteis": "asurar",
    "asurca": "asurcar",
    "asurcaba": "asurcar",
    "asurcabais": "asurcar",
    "asurcaban": "asurcar",
    "asurcabas": "asurcar",
    "asurcad": "asurcar",
    "asurcada": "asurcar",
    "asurcadas": "asurcar",
    "asurcado": "asurcar",
    "asurcados": "asurcar",
    "asurcamos": "asurcar",
    "asurcan": "asurcar",
    "asurcando": "asurcar",
    "asurcara": "asurcar",
    "asurcarais": "asurcar",
    "asurcaran": "asurcar",
    "asurcaras": "asurcar",
    "asurcare": "asurcar",
    "asurcareis": "asurcar",
    "asurcaremos": "asurcar",
    "asurcaren": "asurcar",
    "asurcares": "asurcar",
    "asurcaron": "asurcar",
    "asurcará": "asurcar",
    "asurcarán": "asurcar",
    "asurcarás": "asurcar",
    "asurcaré": "asurcar",
    "asurcaréis": "asurcar",
    "asurcaría": "asurcar",
    "asurcaríais": "asurcar",
    "asurcaríamos": "asurcar",
    "asurcarían": "asurcar",
    "asurcarías": "asurcar",
    "asurcas": "asurcar",
    "asurcase": "asurcar",
    "asurcaseis": "asurcar",
    "asurcasen": "asurcar",
    "asurcases": "asurcar",
    "asurcaste": "asurcar",
    "asurcasteis": "asurcar",
    "asurco": "asurcar",
    "asurcábamos": "asurcar",
    "asurcáis": "asurcar",
    "asurcáramos": "asurcar",
    "asurcáremos": "asurcar",
    "asurcásemos": "asurcar",
    "asurcó": "asurcar",
    "asure": "asurar",
    "asuremos": "asurar",
    "asuren": "asurar",
    "asures": "asurar",
    "asuro": "asurar",
    "asurque": "asurcar",
    "asurquemos": "asurcar",
    "asurquen": "asurcar",
    "asurques": "asurcar",
    "asurqué": "asurcar",
    "asurquéis": "asurcar",
    "asurábamos": "asurar",
    "asuráis": "asurar",
    "asuráramos": "asurar",
    "asuráremos": "asurar",
    "asurásemos": "asurar",
    "asuré": "asurar",
    "asuréis": "asurar",
    "asuró": "asurar",
    "asusta": "asustar",
    "asustaba": "asustar",
    "asustabais": "asustar",
    "asustaban": "asustar",
    "asustabas": "asustar",
    "asustad": "asustar",
    "asustada": "asustar",
    "asustadas": "asustar",
    "asustadiza": "asustadizo",
    "asustadizas": "asustadizo",
    "asustadizos": "asustadizo",
    "asustado": "asustar",
    "asustados": "asustar",
    "asustamos": "asustar",
    "asustan": "asustar",
    "asustando": "asustar",
    "asustara": "asustar",
    "asustarais": "asustar",
    "asustaran": "asustar",
    "asustaras": "asustar",
    "asustare": "asustar",
    "asustareis": "asustar",
    "asustaremos": "asustar",
    "asustaren": "asustar",
    "asustares": "asustar",
    "asustaron": "asustar",
    "asustará": "asustar",
    "asustarán": "asustar",
    "asustarás": "asustar",
    "asustaré": "asustar",
    "asustaréis": "asustar",
    "asustaría": "asustar",
    "asustaríais": "asustar",
    "asustaríamos": "asustar",
    "asustarían": "asustar",
    "asustarías": "asustar",
    "asustas": "asustar",
    "asustase": "asustar",
    "asustaseis": "asustar",
    "asustasen": "asustar",
    "asustases": "asustar",
    "asustaste": "asustar",
    "asustasteis": "asustar",
    "asuste": "asustar",
    "asustemos": "asustar",
    "asusten": "asustar",
    "asustes": "asustar",
    "asusto": "asustar",
    "asustábamos": "asustar",
    "asustáis": "asustar",
    "asustáramos": "asustar",
    "asustáremos": "asustar",
    "asustásemos": "asustar",
    "asusté": "asustar",
    "asustéis": "asustar",
    "asustó": "asustar",
    "asutila": "asutilar",
    "asutilaba": "asutilar",
    "asutilabais": "asutilar",
    "asutilaban": "asutilar",
    "asutilabas": "asutilar",
    "asutilad": "asutilar",
    "asutilada": "asutilar",
    "asutiladas": "asutilar",
    "asutilado": "asutilar",
    "asutilados": "asutilar",
    "asutilamos": "asutilar",
    "asutilan": "asutilar",
    "asutilando": "asutilar",
    "asutilara": "asutilar",
    "asutilarais": "asutilar",
    "asutilaran": "asutilar",
    "asutilaras": "asutilar",
    "asutilare": "asutilar",
    "asutilareis": "asutilar",
    "asutilaremos": "asutilar",
    "asutilaren": "asutilar",
    "asutilares": "asutilar",
    "asutilaron": "asutilar",
    "asutilará": "asutilar",
    "asutilarán": "asutilar",
    "asutilarás": "asutilar",
    "asutilaré": "asutilar",
    "asutilaréis": "asutilar",
    "asutilaría": "asutilar",
    "asutilaríais": "asutilar",
    "asutilaríamos": "asutilar",
    "asutilarían": "asutilar",
    "asutilarías": "asutilar",
    "asutilas": "asutilar",
    "asutilase": "asutilar",
    "asutilaseis": "asutilar",
    "asutilasen": "asutilar",
    "asutilases": "asutilar",
    "asutilaste": "asutilar",
    "asutilasteis": "asutilar",
    "asutile": "asutilar",
    "asutilemos": "asutilar",
    "asutilen": "asutilar",
    "asutiles": "asutilar",
    "asutilo": "asutilar",
    "asutilábamos": "asutilar",
    "asutiláis": "asutilar",
    "asutiláramos": "asutilar",
    "asutiláremos": "asutilar",
    "asutilásemos": "asutilar",
    "asutilé": "asutilar",
    "asutiléis": "asutilar",
    "asutiló": "asutilar",
    "asábamos": "asar",
    "asáis": "asar",
    "asáramos": "asar",
    "asáremos": "asar",
    "asásemos": "asar",
    "asé": "asar",
    "aséis": "asar",
    "asépala": "asépalo",
    "asépalas": "asépalo",
    "asépalos": "asépalo",
    "aséptica": "aséptico",
    "asépticas": "aséptico",
    "asépticos": "aséptico",
    "así": "asir",
    "asía": "asir",
    "asíais": "asir",
    "asíamos": "asir",
    "asían": "asir",
    "asías": "asir",
    "asín": "así",
    "asíncrona": "asíncrono",
    "asíncronas": "asíncrono",
    "asíncronos": "asíncrono",
    "asíndetones": "asíndeton",
    "asíndetons": "asíndeton",
    "asíntotas": "asíntota",
    "asís": "asir",
    "asísmica": "asísmico",
    "asísmicas": "asísmico",
    "asísmicos": "asísmico",
    "asó": "asar",
    "ata": "atar",
    "ataba": "atar",
    "atabais": "atar",
    "atabalea": "atabalear",
    "atabaleaba": "atabalear",
    "atabaleabais": "atabalear",
    "atabaleaban": "atabalear",
    "atabaleabas": "atabalear",
    "atabalead": "atabalear",
    "atabaleada": "atabalear",
    "atabaleadas": "atabalear",
    "atabaleado": "atabalear",
    "atabaleados": "atabalear",
    "atabaleamos": "atabalear",
    "atabalean": "atabalear",
    "atabaleando": "atabalear",
    "atabaleara": "atabalear",
    "atabalearais": "atabalear",
    "atabalearan": "atabalear",
    "atabalearas": "atabalear",
    "atabaleare": "atabalear",
    "atabaleareis": "atabalear",
    "atabalearemos": "atabalear",
    "atabalearen": "atabalear",
    "atabaleares": "atabalear",
    "atabalearon": "atabalear",
    "atabaleará": "atabalear",
    "atabalearán": "atabalear",
    "atabalearás": "atabalear",
    "atabalearé": "atabalear",
    "atabalearéis": "atabalear",
    "atabalearía": "atabalear",
    "atabalearíais": "atabalear",
    "atabalearíamos": "atabalear",
    "atabalearían": "atabalear",
    "atabalearías": "atabalear",
    "atabaleas": "atabalear",
    "atabalease": "atabalear",
    "atabaleaseis": "atabalear",
    "atabaleasen": "atabalear",
    "atabaleases": "atabalear",
    "atabaleaste": "atabalear",
    "atabaleasteis": "atabalear",
    "atabalee": "atabalear",
    "atabaleemos": "atabalear",
    "atabaleen": "atabalear",
    "atabalees": "atabalear",
    "atabaleo": "atabalear",
    "atabaleros": "atabalero",
    "atabales": "atabal",
    "atabaleábamos": "atabalear",
    "atabaleáis": "atabalear",
    "atabaleáramos": "atabalear",
    "atabaleáremos": "atabalear",
    "atabaleásemos": "atabalear",
    "atabaleé": "atabalear",
    "atabaleéis": "atabalear",
    "atabaleó": "atabalear",
    "ataban": "atar",
    "atabas": "atar",
    "atabernada": "atabernado",
    "atabernadas": "atabernado",
    "atabernados": "atabernado",
    "atabes": "atabe",
    "atabilla": "atabillar",
    "atabillaba": "atabillar",
    "atabillabais": "atabillar",
    "atabillaban": "atabillar",
    "atabillabas": "atabillar",
    "atabillad": "atabillar",
    "atabillada": "atabillar",
    "atabilladas": "atabillar",
    "atabillado": "atabillar",
    "atabillados": "atabillar",
    "atabillamos": "atabillar",
    "atabillan": "atabillar",
    "atabillando": "atabillar",
    "atabillara": "atabillar",
    "atabillarais": "atabillar",
    "atabillaran": "atabillar",
    "atabillaras": "atabillar",
    "atabillare": "atabillar",
    "atabillareis": "atabillar",
    "atabillaremos": "atabillar",
    "atabillaren": "atabillar",
    "atabillares": "atabillar",
    "atabillaron": "atabillar",
    "atabillará": "atabillar",
    "atabillarán": "atabillar",
    "atabillarás": "atabillar",
    "atabillaré": "atabillar",
    "atabillaréis": "atabillar",
    "atabillaría": "atabillar",
    "atabillaríais": "atabillar",
    "atabillaríamos": "atabillar",
    "atabillarían": "atabillar",
    "atabillarías": "atabillar",
    "atabillas": "atabillar",
    "atabillase": "atabillar",
    "atabillaseis": "atabillar",
    "atabillasen": "atabillar",
    "atabillases": "atabillar",
    "atabillaste": "atabillar",
    "atabillasteis": "atabillar",
    "atabille": "atabillar",
    "atabillemos": "atabillar",
    "atabillen": "atabillar",
    "atabilles": "atabillar",
    "atabillo": "atabillar",
    "atabillábamos": "atabillar",
    "atabilláis": "atabillar",
    "atabilláramos": "atabillar",
    "atabilláremos": "atabillar",
    "atabillásemos": "atabillar",
    "atabillé": "atabillar",
    "atabilléis": "atabillar",
    "atabilló": "atabillar",
    "atabla": "atablar",
    "atablaba": "atablar",
    "atablabais": "atablar",
    "atablaban": "atablar",
    "atablabas": "atablar",
    "atablad": "atablar",
    "atablada": "atablar",
    "atabladas": "atablar",
    "atabladeras": "atabladera",
    "atablado": "atablar",
    "atablados": "atablar",
    "atablamos": "atablar",
    "atablan": "atablar",
    "atablando": "atablar",
    "atablara": "atablar",
    "atablarais": "atablar",
    "atablaran": "atablar",
    "atablaras": "atablar",
    "atablare": "atablar",
    "atablareis": "atablar",
    "atablaremos": "atablar",
    "atablaren": "atablar",
    "atablares": "atablar",
    "atablaron": "atablar",
    "atablará": "atablar",
    "atablarán": "atablar",
    "atablarás": "atablar",
    "atablaré": "atablar",
    "atablaréis": "atablar",
    "atablaría": "atablar",
    "atablaríais": "atablar",
    "atablaríamos": "atablar",
    "atablarían": "atablar",
    "atablarías": "atablar",
    "atablas": "atablar",
    "atablase": "atablar",
    "atablaseis": "atablar",
    "atablasen": "atablar",
    "atablases": "atablar",
    "atablaste": "atablar",
    "atablasteis": "atablar",
    "atable": "atablar",
    "atablemos": "atablar",
    "atablen": "atablar",
    "atables": "atablar",
    "atablo": "atablar",
    "atablábamos": "atablar",
    "atabláis": "atablar",
    "atabláramos": "atablar",
    "atabláremos": "atablar",
    "atablásemos": "atablar",
    "atablé": "atablar",
    "atabléis": "atablar",
    "atabló": "atablar",
    "ataca": "atacar",
    "atacaba": "atacar",
    "atacabais": "atacar",
    "atacaban": "atacar",
    "atacabas": "atacar",
    "atacables": "atacable",
    "atacad": "atacar",
    "atacada": "atacar",
    "atacadas": "atacar",
    "atacaderas": "atacadera",
    "atacado": "atacar",
    "atacadora": "atacador",
    "atacadoras": "atacador",
    "atacadores": "atacador",
    "atacados": "atacar",
    "atacaduras": "atacadura",
    "atacamitas": "atacamita",
    "atacamos": "atacar",
    "atacan": "atacar",
    "atacando": "atacar",
    "atacantes": "atacante",
    "atacara": "atacar",
    "atacarais": "atacar",
    "atacaran": "atacar",
    "atacaras": "atacar",
    "atacare": "atacar",
    "atacareis": "atacar",
    "atacaremos": "atacar",
    "atacaren": "atacar",
    "atacares": "atacar",
    "atacaron": "atacar",
    "atacará": "atacar",
    "atacarán": "atacar",
    "atacarás": "atacar",
    "atacaré": "atacar",
    "atacaréis": "atacar",
    "atacaría": "atacar",
    "atacaríais": "atacar",
    "atacaríamos": "atacar",
    "atacarían": "atacar",
    "atacarías": "atacar",
    "atacas": "atacar",
    "atacase": "atacar",
    "atacaseis": "atacar",
    "atacasen": "atacar",
    "atacases": "atacar",
    "atacaste": "atacar",
    "atacasteis": "atacar",
    "ataco": "atacar",
    "atacábamos": "atacar",
    "atacáis": "atacar",
    "atacáramos": "atacar",
    "atacáremos": "atacar",
    "atacásemos": "atacar",
    "atacó": "atacar",
    "atad": "atar",
    "atada": "atar",
    "atadas": "atar",
    "ataderos": "atadero",
    "atadijos": "atadijo",
    "atado": "atar",
    "atadora": "atador",
    "atadoras": "atadora",
    "atadores": "atador",
    "atados": "atar",
    "ataduras": "atadura",
    "atafaga": "atafagar",
    "atafagaba": "atafagar",
    "atafagabais": "atafagar",
    "atafagaban": "atafagar",
    "atafagabas": "atafagar",
    "atafagad": "atafagar",
    "atafagada": "atafagar",
    "atafagadas": "atafagar",
    "atafagado": "atafagar",
    "atafagados": "atafagar",
    "atafagamos": "atafagar",
    "atafagan": "atafagar",
    "atafagando": "atafagar",
    "atafagara": "atafagar",
    "atafagarais": "atafagar",
    "atafagaran": "atafagar",
    "atafagaras": "atafagar",
    "atafagare": "atafagar",
    "atafagareis": "atafagar",
    "atafagaremos": "atafagar",
    "atafagaren": "atafagar",
    "atafagares": "atafagar",
    "atafagaron": "atafagar",
    "atafagará": "atafagar",
    "atafagarán": "atafagar",
    "atafagarás": "atafagar",
    "atafagaré": "atafagar",
    "atafagaréis": "atafagar",
    "atafagaría": "atafagar",
    "atafagaríais": "atafagar",
    "atafagaríamos": "atafagar",
    "atafagarían": "atafagar",
    "atafagarías": "atafagar",
    "atafagas": "atafagar",
    "atafagase": "atafagar",
    "atafagaseis": "atafagar",
    "atafagasen": "atafagar",
    "atafagases": "atafagar",
    "atafagaste": "atafagar",
    "atafagasteis": "atafagar",
    "atafago": "atafagar",
    "atafague": "atafagar",
    "atafaguemos": "atafagar",
    "atafaguen": "atafagar",
    "atafagues": "atafagar",
    "atafagué": "atafagar",
    "atafaguéis": "atafagar",
    "atafagábamos": "atafagar",
    "atafagáis": "atafagar",
    "atafagáramos": "atafagar",
    "atafagáremos": "atafagar",
    "atafagásemos": "atafagar",
    "atafagó": "atafagar",
    "atagalla": "atagallar",
    "atagallaba": "atagallar",
    "atagallabais": "atagallar",
    "atagallaban": "atagallar",
    "atagallabas": "atagallar",
    "atagallad": "atagallar",
    "atagallada": "atagallar",
    "atagalladas": "atagallar",
    "atagallado": "atagallar",
    "atagallados": "atagallar",
    "atagallamos": "atagallar",
    "atagallan": "atagallar",
    "atagallando": "atagallar",
    "atagallara": "atagallar",
    "atagallarais": "atagallar",
    "atagallaran": "atagallar",
    "atagallaras": "atagallar",
    "atagallare": "atagallar",
    "atagallareis": "atagallar",
    "atagallaremos": "atagallar",
    "atagallaren": "atagallar",
    "atagallares": "atagallar",
    "atagallaron": "atagallar",
    "atagallará": "atagallar",
    "atagallarán": "atagallar",
    "atagallarás": "atagallar",
    "atagallaré": "atagallar",
    "atagallaréis": "atagallar",
    "atagallaría": "atagallar",
    "atagallaríais": "atagallar",
    "atagallaríamos": "atagallar",
    "atagallarían": "atagallar",
    "atagallarías": "atagallar",
    "atagallas": "atagallar",
    "atagallase": "atagallar",
    "atagallaseis": "atagallar",
    "atagallasen": "atagallar",
    "atagallases": "atagallar",
    "atagallaste": "atagallar",
    "atagallasteis": "atagallar",
    "atagalle": "atagallar",
    "atagallemos": "atagallar",
    "atagallen": "atagallar",
    "atagalles": "atagallar",
    "atagallo": "atagallar",
    "atagallábamos": "atagallar",
    "atagalláis": "atagallar",
    "atagalláramos": "atagallar",
    "atagalláremos": "atagallar",
    "atagallásemos": "atagallar",
    "atagallé": "atagallar",
    "atagalléis": "atagallar",
    "atagalló": "atagallar",
    "ataguías": "ataguía",
    "ataharres": "ataharre",
    "ataira": "atairar",
    "atairaba": "atairar",
    "atairabais": "atairar",
    "atairaban": "atairar",
    "atairabas": "atairar",
    "atairad": "atairar",
    "atairada": "atairar",
    "atairadas": "atairar",
    "atairado": "atairar",
    "atairados": "atairar",
    "atairamos": "atairar",
    "atairan": "atairar",
    "atairando": "atairar",
    "atairara": "atairar",
    "atairarais": "atairar",
    "atairaran": "atairar",
    "atairaras": "atairar",
    "atairare": "atairar",
    "atairareis": "atairar",
    "atairaremos": "atairar",
    "atairaren": "atairar",
    "atairares": "atairar",
    "atairaron": "atairar",
    "atairará": "atairar",
    "atairarán": "atairar",
    "atairarás": "atairar",
    "atairaré": "atairar",
    "atairaréis": "atairar",
    "atairaría": "atairar",
    "atairaríais": "atairar",
    "atairaríamos": "atairar",
    "atairarían": "atairar",
    "atairarías": "atairar",
    "atairas": "atairar",
    "atairase": "atairar",
    "atairaseis": "atairar",
    "atairasen": "atairar",
    "atairases": "atairar",
    "atairaste": "atairar",
    "atairasteis": "atairar",
    "ataire": "atairar",
    "atairemos": "atairar",
    "atairen": "atairar",
    "ataires": "ataire",
    "atairo": "atairar",
    "atairábamos": "atairar",
    "atairáis": "atairar",
    "atairáramos": "atairar",
    "atairáremos": "atairar",
    "atairásemos": "atairar",
    "atairé": "atairar",
    "atairéis": "atairar",
    "atairó": "atairar",
    "ataja": "atajar",
    "atajaba": "atajar",
    "atajabais": "atajar",
    "atajaban": "atajar",
    "atajabas": "atajar",
    "atajad": "atajar",
    "atajada": "atajar",
    "atajadas": "atajar",
    "atajadizos": "atajadizo",
    "atajado": "atajar",
    "atajadora": "atajador",
    "atajadoras": "atajador",
    "atajadores": "atajador",
    "atajados": "atajar",
    "atajamientos": "atajamiento",
    "atajamos": "atajar",
    "atajan": "atajar",
    "atajando": "atajar",
    "atajara": "atajar",
    "atajarais": "atajar",
    "atajaran": "atajar",
    "atajaras": "atajar",
    "atajare": "atajar",
    "atajareis": "atajar",
    "atajaremos": "atajar",
    "atajaren": "atajar",
    "atajares": "atajar",
    "atajaron": "atajar",
    "atajará": "atajar",
    "atajarán": "atajar",
    "atajarás": "atajar",
    "atajaré": "atajar",
    "atajaréis": "atajar",
    "atajaría": "atajar",
    "atajaríais": "atajar",
    "atajaríamos": "atajar",
    "atajarían": "atajar",
    "atajarías": "atajar",
    "atajas": "atajar",
    "atajase": "atajar",
    "atajaseis": "atajar",
    "atajasen": "atajar",
    "atajases": "atajar",
    "atajaste": "atajar",
    "atajasteis": "atajar",
    "ataje": "atajar",
    "atajemos": "atajar",
    "atajen": "atajar",
    "atajes": "atajar",
    "atajo": "atajar",
    "atajos": "atajo",
    "atajábamos": "atajar",
    "atajáis": "atajar",
    "atajáramos": "atajar",
    "atajáremos": "atajar",
    "atajásemos": "atajar",
    "atajé": "atajar",
    "atajéis": "atajar",
    "atajó": "atajar",
    "atalaja": "atalajar",
    "atalajaba": "atalajar",
    "atalajabais": "atalajar",
    "atalajaban": "atalajar",
    "atalajabas": "atalajar",
    "atalajad": "atalajar",
    "atalajada": "atalajar",
    "atalajadas": "atalajar",
    "atalajado": "atalajar",
    "atalajados": "atalajar",
    "atalajamos": "atalajar",
    "atalajan": "atalajar",
    "atalajando": "atalajar",
    "atalajara": "atalajar",
    "atalajarais": "atalajar",
    "atalajaran": "atalajar",
    "atalajaras": "atalajar",
    "atalajare": "atalajar",
    "atalajareis": "atalajar",
    "atalajaremos": "atalajar",
    "atalajaren": "atalajar",
    "atalajares": "atalajar",
    "atalajaron": "atalajar",
    "atalajará": "atalajar",
    "atalajarán": "atalajar",
    "atalajarás": "atalajar",
    "atalajaré": "atalajar",
    "atalajaréis": "atalajar",
    "atalajaría": "atalajar",
    "atalajaríais": "atalajar",
    "atalajaríamos": "atalajar",
    "atalajarían": "atalajar",
    "atalajarías": "atalajar",
    "atalajas": "atalajar",
    "atalajase": "atalajar",
    "atalajaseis": "atalajar",
    "atalajasen": "atalajar",
    "atalajases": "atalajar",
    "atalajaste": "atalajar",
    "atalajasteis": "atalajar",
    "atalaje": "atalajar",
    "atalajemos": "atalajar",
    "atalajen": "atalajar",
    "atalajes": "atalaje",
    "atalajo": "atalajar",
    "atalajábamos": "atalajar",
    "atalajáis": "atalajar",
    "atalajáramos": "atalajar",
    "atalajáremos": "atalajar",
    "atalajásemos": "atalajar",
    "atalajé": "atalajar",
    "atalajéis": "atalajar",
    "atalajó": "atalajar",
    "atalanta": "atalantar",
    "atalantaba": "atalantar",
    "atalantabais": "atalantar",
    "atalantaban": "atalantar",
    "atalantabas": "atalantar",
    "atalantad": "atalantar",
    "atalantada": "atalantar",
    "atalantadas": "atalantar",
    "atalantado": "atalantar",
    "atalantados": "atalantar",
    "atalantamos": "atalantar",
    "atalantan": "atalantar",
    "atalantando": "atalantar",
    "atalantara": "atalantar",
    "atalantarais": "atalantar",
    "atalantaran": "atalantar",
    "atalantaras": "atalantar",
    "atalantare": "atalantar",
    "atalantareis": "atalantar",
    "atalantaremos": "atalantar",
    "atalantaren": "atalantar",
    "atalantares": "atalantar",
    "atalantaron": "atalantar",
    "atalantará": "atalantar",
    "atalantarán": "atalantar",
    "atalantarás": "atalantar",
    "atalantaré": "atalantar",
    "atalantaréis": "atalantar",
    "atalantaría": "atalantar",
    "atalantaríais": "atalantar",
    "atalantaríamos": "atalantar",
    "atalantarían": "atalantar",
    "atalantarías": "atalantar",
    "atalantas": "atalantar",
    "atalantase": "atalantar",
    "atalantaseis": "atalantar",
    "atalantasen": "atalantar",
    "atalantases": "atalantar",
    "atalantaste": "atalantar",
    "atalantasteis": "atalantar",
    "atalante": "atalantar",
    "atalantemos": "atalantar",
    "atalanten": "atalantar",
    "atalantes": "atalantar",
    "atalanto": "atalantar",
    "atalantábamos": "atalantar",
    "atalantáis": "atalantar",
    "atalantáramos": "atalantar",
    "atalantáremos": "atalantar",
    "atalantásemos": "atalantar",
    "atalanté": "atalantar",
    "atalantéis": "atalantar",
    "atalantó": "atalantar",
    "atalaya": "atalayar",
    "atalayaba": "atalayar",
    "atalayabais": "atalayar",
    "atalayaban": "atalayar",
    "atalayabas": "atalayar",
    "atalayad": "atalayar",
    "atalayada": "atalayar",
    "atalayadas": "atalayar",
    "atalayado": "atalayar",
    "atalayados": "atalayar",
    "atalayamos": "atalayar",
    "atalayan": "atalayar",
    "atalayando": "atalayar",
    "atalayara": "atalayar",
    "atalayarais": "atalayar",
    "atalayaran": "atalayar",
    "atalayaras": "atalayar",
    "atalayare": "atalayar",
    "atalayareis": "atalayar",
    "atalayaremos": "atalayar",
    "atalayaren": "atalayar",
    "atalayares": "atalayar",
    "atalayaron": "atalayar",
    "atalayará": "atalayar",
    "atalayarán": "atalayar",
    "atalayarás": "atalayar",
    "atalayaré": "atalayar",
    "atalayaréis": "atalayar",
    "atalayaría": "atalayar",
    "atalayaríais": "atalayar",
    "atalayaríamos": "atalayar",
    "atalayarían": "atalayar",
    "atalayarías": "atalayar",
    "atalayas": "atalayar",
    "atalayase": "atalayar",
    "atalayaseis": "atalayar",
    "atalayasen": "atalayar",
    "atalayases": "atalayar",
    "atalayaste": "atalayar",
    "atalayasteis": "atalayar",
    "atalaye": "atalayar",
    "atalayemos": "atalayar",
    "atalayen": "atalayar",
    "atalayes": "atalayar",
    "atalayo": "atalayar",
    "atalayábamos": "atalayar",
    "atalayáis": "atalayar",
    "atalayáramos": "atalayar",
    "atalayáremos": "atalayar",
    "atalayásemos": "atalayar",
    "atalayé": "atalayar",
    "atalayéis": "atalayar",
    "atalayó": "atalayar",
    "ataluce": "ataluzar",
    "atalucemos": "ataluzar",
    "atalucen": "ataluzar",
    "ataluces": "ataluzar",
    "atalucé": "ataluzar",
    "atalucéis": "ataluzar",
    "ataluda": "ataludar",
    "ataludaba": "ataludar",
    "ataludabais": "ataludar",
    "ataludaban": "ataludar",
    "ataludabas": "ataludar",
    "ataludad": "ataludar",
    "ataludada": "ataludar",
    "ataludadas": "ataludar",
    "ataludado": "ataludar",
    "ataludados": "ataludar",
    "ataludamos": "ataludar",
    "ataludan": "ataludar",
    "ataludando": "ataludar",
    "ataludara": "ataludar",
    "ataludarais": "ataludar",
    "ataludaran": "ataludar",
    "ataludaras": "ataludar",
    "ataludare": "ataludar",
    "ataludareis": "ataludar",
    "ataludaremos": "ataludar",
    "ataludaren": "ataludar",
    "ataludares": "ataludar",
    "ataludaron": "ataludar",
    "ataludará": "ataludar",
    "ataludarán": "ataludar",
    "ataludarás": "ataludar",
    "ataludaré": "ataludar",
    "ataludaréis": "ataludar",
    "ataludaría": "ataludar",
    "ataludaríais": "ataludar",
    "ataludaríamos": "ataludar",
    "ataludarían": "ataludar",
    "ataludarías": "ataludar",
    "ataludas": "ataludar",
    "ataludase": "ataludar",
    "ataludaseis": "ataludar",
    "ataludasen": "ataludar",
    "ataludases": "ataludar",
    "ataludaste": "ataludar",
    "ataludasteis": "ataludar",
    "atalude": "ataludar",
    "ataludemos": "ataludar",
    "ataluden": "ataludar",
    "ataludes": "ataludar",
    "ataludo": "ataludar",
    "ataludábamos": "ataludar",
    "ataludáis": "ataludar",
    "ataludáramos": "ataludar",
    "ataludáremos": "ataludar",
    "ataludásemos": "ataludar",
    "ataludé": "ataludar",
    "ataludéis": "ataludar",
    "ataludó": "ataludar",
    "ataluza": "ataluzar",
    "ataluzaba": "ataluzar",
    "ataluzabais": "ataluzar",
    "ataluzaban": "ataluzar",
    "ataluzabas": "ataluzar",
    "ataluzad": "ataluzar",
    "ataluzada": "ataluzar",
    "ataluzadas": "ataluzar",
    "ataluzado": "ataluzar",
    "ataluzados": "ataluzar",
    "ataluzamos": "ataluzar",
    "ataluzan": "ataluzar",
    "ataluzando": "ataluzar",
    "ataluzara": "ataluzar",
    "ataluzarais": "ataluzar",
    "ataluzaran": "ataluzar",
    "ataluzaras": "ataluzar",
    "ataluzare": "ataluzar",
    "ataluzareis": "ataluzar",
    "ataluzaremos": "ataluzar",
    "ataluzaren": "ataluzar",
    "ataluzares": "ataluzar",
    "ataluzaron": "ataluzar",
    "ataluzará": "ataluzar",
    "ataluzarán": "ataluzar",
    "ataluzarás": "ataluzar",
    "ataluzaré": "ataluzar",
    "ataluzaréis": "ataluzar",
    "ataluzaría": "ataluzar",
    "ataluzaríais": "ataluzar",
    "ataluzaríamos": "ataluzar",
    "ataluzarían": "ataluzar",
    "ataluzarías": "ataluzar",
    "ataluzas": "ataluzar",
    "ataluzase": "ataluzar",
    "ataluzaseis": "ataluzar",
    "ataluzasen": "ataluzar",
    "ataluzases": "ataluzar",
    "ataluzaste": "ataluzar",
    "ataluzasteis": "ataluzar",
    "ataluzo": "ataluzar",
    "ataluzábamos": "ataluzar",
    "ataluzáis": "ataluzar",
    "ataluzáramos": "ataluzar",
    "ataluzáremos": "ataluzar",
    "ataluzásemos": "ataluzar",
    "ataluzó": "ataluzar",
    "atamanes": "atamán",
    "atamos": "atar",
    "atan": "atar",
    "atanasias": "atanasia",
    "atando": "atar",
    "atanores": "atanor",
    "atapa": "atapar",
    "atapaba": "atapar",
    "atapabais": "atapar",
    "atapaban": "atapar",
    "atapabas": "atapar",
    "atapad": "atapar",
    "atapada": "atapar",
    "atapadas": "atapar",
    "atapado": "atapar",
    "atapados": "atapar",
    "atapamos": "atapar",
    "atapan": "atapar",
    "atapando": "atapar",
    "atapara": "atapar",
    "ataparais": "atapar",
    "ataparan": "atapar",
    "ataparas": "atapar",
    "atapare": "atapar",
    "atapareis": "atapar",
    "ataparemos": "atapar",
    "ataparen": "atapar",
    "atapares": "atapar",
    "ataparon": "atapar",
    "atapará": "atapar",
    "ataparán": "atapar",
    "ataparás": "atapar",
    "ataparé": "atapar",
    "ataparéis": "atapar",
    "ataparía": "atapar",
    "ataparíais": "atapar",
    "ataparíamos": "atapar",
    "ataparían": "atapar",
    "ataparías": "atapar",
    "atapas": "atapar",
    "atapasca": "atapasco",
    "atapascas": "atapasco",
    "atapascos": "atapasco",
    "atapase": "atapar",
    "atapaseis": "atapar",
    "atapasen": "atapar",
    "atapases": "atapar",
    "atapaste": "atapar",
    "atapasteis": "atapar",
    "atape": "atapar",
    "atapemos": "atapar",
    "atapen": "atapar",
    "atapes": "atapar",
    "atapo": "atapar",
    "atapábamos": "atapar",
    "atapáis": "atapar",
    "atapáramos": "atapar",
    "atapáremos": "atapar",
    "atapásemos": "atapar",
    "atapé": "atapar",
    "atapéis": "atapar",
    "atapó": "atapar",
    "ataque": "atacar",
    "ataquemos": "atacar",
    "ataquen": "atacar",
    "ataques": "ataque",
    "ataquizas": "ataquiza",
    "ataqué": "atacar",
    "ataquéis": "atacar",
    "atara": "atar",
    "atarace": "atarazar",
    "ataracea": "ataracear",
    "ataraceaba": "ataracear",
    "ataraceabais": "ataracear",
    "ataraceaban": "ataracear",
    "ataraceabas": "ataracear",
    "ataracead": "ataracear",
    "ataraceada": "ataracear",
    "ataraceadas": "ataracear",
    "ataraceado": "ataracear",
    "ataraceados": "ataracear",
    "ataraceamos": "ataracear",
    "ataracean": "ataracear",
    "ataraceando": "ataracear",
    "ataraceara": "ataracear",
    "ataracearais": "ataracear",
    "ataracearan": "ataracear",
    "ataracearas": "ataracear",
    "ataraceare": "ataracear",
    "ataraceareis": "ataracear",
    "ataracearemos": "ataracear",
    "ataracearen": "ataracear",
    "ataraceares": "ataracear",
    "ataracearon": "ataracear",
    "ataraceará": "ataracear",
    "ataracearán": "ataracear",
    "ataracearás": "ataracear",
    "ataracearé": "ataracear",
    "ataracearéis": "ataracear",
    "ataracearía": "ataracear",
    "ataracearíais": "ataracear",
    "ataracearíamos": "ataracear",
    "ataracearían": "ataracear",
    "ataracearías": "ataracear",
    "ataraceas": "ataracear",
    "ataracease": "ataracear",
    "ataraceaseis": "ataracear",
    "ataraceasen": "ataracear",
    "ataraceases": "ataracear",
    "ataraceaste": "ataracear",
    "ataraceasteis": "ataracear",
    "ataracee": "ataracear",
    "ataraceemos": "ataracear",
    "ataraceen": "ataracear",
    "ataracees": "ataracear",
    "ataracemos": "atarazar",
    "ataracen": "atarazar",
    "ataraceo": "ataracear",
    "ataraces": "atarazar",
    "ataraceábamos": "ataracear",
    "ataraceáis": "ataracear",
    "ataraceáramos": "ataracear",
    "ataraceáremos": "ataracear",
    "ataraceásemos": "ataracear",
    "ataraceé": "ataracear",
    "ataraceéis": "ataracear",
    "ataraceó": "ataracear",
    "ataracé": "atarazar",
    "ataracéis": "atarazar",
    "atarais": "atar",
    "ataran": "atar",
    "ataranta": "atarantar",
    "atarantaba": "atarantar",
    "atarantabais": "atarantar",
    "atarantaban": "atarantar",
    "atarantabas": "atarantar",
    "atarantad": "atarantar",
    "atarantada": "atarantar",
    "atarantadas": "atarantar",
    "atarantado": "atarantar",
    "atarantados": "atarantar",
    "atarantamientos": "atarantamiento",
    "atarantamos": "atarantar",
    "atarantan": "atarantar",
    "atarantando": "atarantar",
    "atarantara": "atarantar",
    "atarantarais": "atarantar",
    "atarantaran": "atarantar",
    "atarantaras": "atarantar",
    "atarantare": "atarantar",
    "atarantareis": "atarantar",
    "atarantaremos": "atarantar",
    "atarantaren": "atarantar",
    "atarantares": "atarantar",
    "atarantaron": "atarantar",
    "atarantará": "atarantar",
    "atarantarán": "atarantar",
    "atarantarás": "atarantar",
    "atarantaré": "atarantar",
    "atarantaréis": "atarantar",
    "atarantaría": "atarantar",
    "atarantaríais": "atarantar",
    "atarantaríamos": "atarantar",
    "atarantarían": "atarantar",
    "atarantarías": "atarantar",
    "atarantas": "atarantar",
    "atarantase": "atarantar",
    "atarantaseis": "atarantar",
    "atarantasen": "atarantar",
    "atarantases": "atarantar",
    "atarantaste": "atarantar",
    "atarantasteis": "atarantar",
    "atarante": "atarantar",
    "atarantemos": "atarantar",
    "ataranten": "atarantar",
    "atarantes": "atarantar",
    "ataranto": "atarantar",
    "atarantábamos": "atarantar",
    "atarantáis": "atarantar",
    "atarantáramos": "atarantar",
    "atarantáremos": "atarantar",
    "atarantásemos": "atarantar",
    "ataranté": "atarantar",
    "atarantéis": "atarantar",
    "atarantó": "atarantar",
    "ataras": "atar",
    "ataraxias": "ataraxia",
    "ataraza": "atarazar",
    "atarazaba": "atarazar",
    "atarazabais": "atarazar",
    "atarazaban": "atarazar",
    "atarazabas": "atarazar",
    "atarazad": "atarazar",
    "atarazada": "atarazar",
    "atarazadas": "atarazar",
    "atarazado": "atarazar",
    "atarazados": "atarazar",
    "atarazamos": "atarazar",
    "atarazan": "atarazar",
    "atarazanas": "atarazana",
    "atarazando": "atarazar",
    "atarazara": "atarazar",
    "atarazarais": "atarazar",
    "atarazaran": "atarazar",
    "atarazaras": "atarazar",
    "atarazare": "atarazar",
    "atarazareis": "atarazar",
    "atarazaremos": "atarazar",
    "atarazaren": "atarazar",
    "atarazares": "atarazar",
    "atarazaron": "atarazar",
    "atarazará": "atarazar",
    "atarazarán": "atarazar",
    "atarazarás": "atarazar",
    "atarazaré": "atarazar",
    "atarazaréis": "atarazar",
    "atarazaría": "atarazar",
    "atarazaríais": "atarazar",
    "atarazaríamos": "atarazar",
    "atarazarían": "atarazar",
    "atarazarías": "atarazar",
    "atarazas": "atarazar",
    "atarazase": "atarazar",
    "atarazaseis": "atarazar",
    "atarazasen": "atarazar",
    "atarazases": "atarazar",
    "atarazaste": "atarazar",
    "atarazasteis": "atarazar",
    "atarazo": "atarazar",
    "atarazábamos": "atarazar",
    "atarazáis": "atarazar",
    "atarazáramos": "atarazar",
    "atarazáremos": "atarazar",
    "atarazásemos": "atarazar",
    "atarazó": "atarazar",
    "atardece": "atardecer",
    "atardeced": "atardecer",
    "atardecemos": "atardecer",
    "atardecen": "atardecer",
    "atardeceremos": "atardecer",
    "atardeceres": "atardecer",
    "atardecerá": "atardecer",
    "atardecerán": "atardecer",
    "atardecerás": "atardecer",
    "atardeceré": "atardecer",
    "atardeceréis": "atardecer",
    "atardecería": "atardecer",
    "atardeceríais": "atardecer",
    "atardeceríamos": "atardecer",
    "atardecerían": "atardecer",
    "atardecerías": "atardecer",
    "atardeces": "atardecer",
    "atardecida": "atardecer",
    "atardecidas": "atardecer",
    "atardecido": "atardecer",
    "atardecidos": "atardecer",
    "atardeciendo": "atardecer",
    "atardeciera": "atardecer",
    "atardecierais": "atardecer",
    "atardecieran": "atardecer",
    "atardecieras": "atardecer",
    "atardeciere": "atardecer",
    "atardeciereis": "atardecer",
    "atardecieren": "atardecer",
    "atardecieres": "atardecer",
    "atardecieron": "atardecer",
    "atardeciese": "atardecer",
    "atardecieseis": "atardecer",
    "atardeciesen": "atardecer",
    "atardecieses": "atardecer",
    "atardecimos": "atardecer",
    "atardeciste": "atardecer",
    "atardecisteis": "atardecer",
    "atardeciéramos": "atardecer",
    "atardeciéremos": "atardecer",
    "atardeciésemos": "atardecer",
    "atardeció": "atardecer",
    "atardecéis": "atardecer",
    "atardecí": "atardecer",
    "atardecía": "atardecer",
    "atardecíais": "atardecer",
    "atardecíamos": "atardecer",
    "atardecían": "atardecer",
    "atardecías": "atardecer",
    "atardezca": "atardecer",
    "atardezcamos": "atardecer",
    "atardezcan": "atardecer",
    "atardezcas": "atardecer",
    "atardezco": "atardecer",
    "atardezcáis": "atardecer",
    "atare": "atar",
    "atarea": "atarear",
    "atareaba": "atarear",
    "atareabais": "atarear",
    "atareaban": "atarear",
    "atareabas": "atarear",
    "ataread": "atarear",
    "atareada": "atarear",
    "atareadas": "atarear",
    "atareado": "atarear",
    "atareados": "atarear",
    "atareamos": "atarear",
    "atarean": "atarear",
    "atareando": "atarear",
    "atareara": "atarear",
    "atarearais": "atarear",
    "atarearan": "atarear",
    "atarearas": "atarear",
    "atareare": "atarear",
    "atareareis": "atarear",
    "atarearemos": "atarear",
    "atarearen": "atarear",
    "atareares": "atarear",
    "atarearon": "atarear",
    "atareará": "atarear",
    "atarearán": "atarear",
    "atarearás": "atarear",
    "atarearé": "atarear",
    "atarearéis": "atarear",
    "atarearía": "atarear",
    "atarearíais": "atarear",
    "atarearíamos": "atarear",
    "atarearían": "atarear",
    "atarearías": "atarear",
    "atareas": "atarear",
    "atarease": "atarear",
    "atareaseis": "atarear",
    "atareasen": "atarear",
    "atareases": "atarear",
    "atareaste": "atarear",
    "atareasteis": "atarear",
    "ataree": "atarear",
    "atareemos": "atarear",
    "atareen": "atarear",
    "atarees": "atarear",
    "atareis": "atar",
    "ataremos": "atar",
    "ataren": "atar",
    "atareo": "atarear",
    "atares": "atar",
    "atareábamos": "atarear",
    "atareáis": "atarear",
    "atareáramos": "atarear",
    "atareáremos": "atarear",
    "atareásemos": "atarear",
    "atareé": "atarear",
    "atareéis": "atarear",
    "atareó": "atarear",
    "atarjeas": "atarjea",
    "ataron": "atar",
    "atarquina": "atarquinar",
    "atarquinaba": "atarquinar",
    "atarquinabais": "atarquinar",
    "atarquinaban": "atarquinar",
    "atarquinabas": "atarquinar",
    "atarquinad": "atarquinar",
    "atarquinada": "atarquinar",
    "atarquinadas": "atarquinar",
    "atarquinado": "atarquinar",
    "atarquinados": "atarquinar",
    "atarquinamos": "atarquinar",
    "atarquinan": "atarquinar",
    "atarquinando": "atarquinar",
    "atarquinara": "atarquinar",
    "atarquinarais": "atarquinar",
    "atarquinaran": "atarquinar",
    "atarquinaras": "atarquinar",
    "atarquinare": "atarquinar",
    "atarquinareis": "atarquinar",
    "atarquinaremos": "atarquinar",
    "atarquinaren": "atarquinar",
    "atarquinares": "atarquinar",
    "atarquinaron": "atarquinar",
    "atarquinará": "atarquinar",
    "atarquinarán": "atarquinar",
    "atarquinarás": "atarquinar",
    "atarquinaré": "atarquinar",
    "atarquinaréis": "atarquinar",
    "atarquinaría": "atarquinar",
    "atarquinaríais": "atarquinar",
    "atarquinaríamos": "atarquinar",
    "atarquinarían": "atarquinar",
    "atarquinarías": "atarquinar",
    "atarquinas": "atarquinar",
    "atarquinase": "atarquinar",
    "atarquinaseis": "atarquinar",
    "atarquinasen": "atarquinar",
    "atarquinases": "atarquinar",
    "atarquinaste": "atarquinar",
    "atarquinasteis": "atarquinar",
    "atarquine": "atarquinar",
    "atarquinemos": "atarquinar",
    "atarquinen": "atarquinar",
    "atarquines": "atarquinar",
    "atarquino": "atarquinar",
    "atarquinábamos": "atarquinar",
    "atarquináis": "atarquinar",
    "atarquináramos": "atarquinar",
    "atarquináremos": "atarquinar",
    "atarquinásemos": "atarquinar",
    "atarquiné": "atarquinar",
    "atarquinéis": "atarquinar",
    "atarquinó": "atarquinar",
    "atarraga": "atarragar",
    "atarragaba": "atarragar",
    "atarragabais": "atarragar",
    "atarragaban": "atarragar",
    "atarragabas": "atarragar",
    "atarragad": "atarragar",
    "atarragada": "atarragar",
    "atarragadas": "atarragar",
    "atarragado": "atarragar",
    "atarragados": "atarragar",
    "atarragamos": "atarragar",
    "atarragan": "atarragar",
    "atarragando": "atarragar",
    "atarragara": "atarragar",
    "atarragarais": "atarragar",
    "atarragaran": "atarragar",
    "atarragaras": "atarragar",
    "atarragare": "atarragar",
    "atarragareis": "atarragar",
    "atarragaremos": "atarragar",
    "atarragaren": "atarragar",
    "atarragares": "atarragar",
    "atarragaron": "atarragar",
    "atarragará": "atarragar",
    "atarragarán": "atarragar",
    "atarragarás": "atarragar",
    "atarragaré": "atarragar",
    "atarragaréis": "atarragar",
    "atarragaría": "atarragar",
    "atarragaríais": "atarragar",
    "atarragaríamos": "atarragar",
    "atarragarían": "atarragar",
    "atarragarías": "atarragar",
    "atarragas": "atarragar",
    "atarragase": "atarragar",
    "atarragaseis": "atarragar",
    "atarragasen": "atarragar",
    "atarragases": "atarragar",
    "atarragaste": "atarragar",
    "atarragasteis": "atarragar",
    "atarrago": "atarragar",
    "atarrague": "atarragar",
    "atarraguemos": "atarragar",
    "atarraguen": "atarragar",
    "atarragues": "atarragar",
    "atarragué": "atarragar",
    "atarraguéis": "atarragar",
    "atarragábamos": "atarragar",
    "atarragáis": "atarragar",
    "atarragáramos": "atarragar",
    "atarragáremos": "atarragar",
    "atarragásemos": "atarragar",
    "atarragó": "atarragar",
    "atarraja": "atarrajar",
    "atarrajaba": "atarrajar",
    "atarrajabais": "atarrajar",
    "atarrajaban": "atarrajar",
    "atarrajabas": "atarrajar",
    "atarrajad": "atarrajar",
    "atarrajada": "atarrajar",
    "atarrajadas": "atarrajar",
    "atarrajado": "atarrajar",
    "atarrajados": "atarrajar",
    "atarrajamos": "atarrajar",
    "atarrajan": "atarrajar",
    "atarrajando": "atarrajar",
    "atarrajara": "atarrajar",
    "atarrajarais": "atarrajar",
    "atarrajaran": "atarrajar",
    "atarrajaras": "atarrajar",
    "atarrajare": "atarrajar",
    "atarrajareis": "atarrajar",
    "atarrajaremos": "atarrajar",
    "atarrajaren": "atarrajar",
    "atarrajares": "atarrajar",
    "atarrajaron": "atarrajar",
    "atarrajará": "atarrajar",
    "atarrajarán": "atarrajar",
    "atarrajarás": "atarrajar",
    "atarrajaré": "atarrajar",
    "atarrajaréis": "atarrajar",
    "atarrajaría": "atarrajar",
    "atarrajaríais": "atarrajar",
    "atarrajaríamos": "atarrajar",
    "atarrajarían": "atarrajar",
    "atarrajarías": "atarrajar",
    "atarrajas": "atarrajar",
    "atarrajase": "atarrajar",
    "atarrajaseis": "atarrajar",
    "atarrajasen": "atarrajar",
    "atarrajases": "atarrajar",
    "atarrajaste": "atarrajar",
    "atarrajasteis": "atarrajar",
    "atarraje": "atarrajar",
    "atarrajemos": "atarrajar",
    "atarrajen": "atarrajar",
    "atarrajes": "atarrajar",
    "atarrajo": "atarrajar",
    "atarrajábamos": "atarrajar",
    "atarrajáis": "atarrajar",
    "atarrajáramos": "atarrajar",
    "atarrajáremos": "atarrajar",
    "atarrajásemos": "atarrajar",
    "atarrajé": "atarrajar",
    "atarrajéis": "atarrajar",
    "atarrajó": "atarrajar",
    "atarrayas": "atarraya",
    "ataruga": "atarugar",
    "atarugaba": "atarugar",
    "atarugabais": "atarugar",
    "atarugaban": "atarugar",
    "atarugabas": "atarugar",
    "atarugad": "atarugar",
    "atarugada": "atarugar",
    "atarugadas": "atarugar",
    "atarugado": "atarugar",
    "atarugados": "atarugar",
    "atarugamos": "atarugar",
    "atarugan": "atarugar",
    "atarugando": "atarugar",
    "atarugara": "atarugar",
    "atarugarais": "atarugar",
    "atarugaran": "atarugar",
    "atarugaras": "atarugar",
    "atarugare": "atarugar",
    "atarugareis": "atarugar",
    "atarugaremos": "atarugar",
    "atarugaren": "atarugar",
    "atarugares": "atarugar",
    "atarugaron": "atarugar",
    "atarugará": "atarugar",
    "atarugarán": "atarugar",
    "atarugarás": "atarugar",
    "atarugaré": "atarugar",
    "atarugaréis": "atarugar",
    "atarugaría": "atarugar",
    "atarugaríais": "atarugar",
    "atarugaríamos": "atarugar",
    "atarugarían": "atarugar",
    "atarugarías": "atarugar",
    "atarugas": "atarugar",
    "atarugase": "atarugar",
    "atarugaseis": "atarugar",
    "atarugasen": "atarugar",
    "atarugases": "atarugar",
    "atarugaste": "atarugar",
    "atarugasteis": "atarugar",
    "atarugo": "atarugar",
    "atarugue": "atarugar",
    "ataruguemos": "atarugar",
    "ataruguen": "atarugar",
    "atarugues": "atarugar",
    "atarugué": "atarugar",
    "ataruguéis": "atarugar",
    "atarugábamos": "atarugar",
    "atarugáis": "atarugar",
    "atarugáramos": "atarugar",
    "atarugáremos": "atarugar",
    "atarugásemos": "atarugar",
    "atarugó": "atarugar",
    "atará": "atar",
    "atarán": "atar",
    "atarás": "atar",
    "ataré": "atar",
    "ataréis": "atar",
    "ataría": "atar",
    "ataríais": "atar",
    "ataríamos": "atar",
    "atarían": "atar",
    "atarías": "atar",
    "atas": "atar",
    "atasaja": "atasajar",
    "atasajaba": "atasajar",
    "atasajabais": "atasajar",
    "atasajaban": "atasajar",
    "atasajabas": "atasajar",
    "atasajad": "atasajar",
    "atasajada": "atasajar",
    "atasajadas": "atasajar",
    "atasajado": "atasajar",
    "atasajados": "atasajar",
    "atasajamos": "atasajar",
    "atasajan": "atasajar",
    "atasajando": "atasajar",
    "atasajara": "atasajar",
    "atasajarais": "atasajar",
    "atasajaran": "atasajar",
    "atasajaras": "atasajar",
    "atasajare": "atasajar",
    "atasajareis": "atasajar",
    "atasajaremos": "atasajar",
    "atasajaren": "atasajar",
    "atasajares": "atasajar",
    "atasajaron": "atasajar",
    "atasajará": "atasajar",
    "atasajarán": "atasajar",
    "atasajarás": "atasajar",
    "atasajaré": "atasajar",
    "atasajaréis": "atasajar",
    "atasajaría": "atasajar",
    "atasajaríais": "atasajar",
    "atasajaríamos": "atasajar",
    "atasajarían": "atasajar",
    "atasajarías": "atasajar",
    "atasajas": "atasajar",
    "atasajase": "atasajar",
    "atasajaseis": "atasajar",
    "atasajasen": "atasajar",
    "atasajases": "atasajar",
    "atasajaste": "atasajar",
    "atasajasteis": "atasajar",
    "atasaje": "atasajar",
    "atasajemos": "atasajar",
    "atasajen": "atasajar",
    "atasajes": "atasajar",
    "atasajo": "atasajar",
    "atasajábamos": "atasajar",
    "atasajáis": "atasajar",
    "atasajáramos": "atasajar",
    "atasajáremos": "atasajar",
    "atasajásemos": "atasajar",
    "atasajé": "atasajar",
    "atasajéis": "atasajar",
    "atasajó": "atasajar",
    "atasca": "atascar",
    "atascaba": "atascar",
    "atascabais": "atascar",
    "atascaban": "atascar",
    "atascabas": "atascar",
    "atascad": "atascar",
    "atascada": "atascar",
    "atascadas": "atascar",
    "atascaderos": "atascadero",
    "atascado": "atascar",
    "atascados": "atascar",
    "atascamientos": "atascamiento",
    "atascamos": "atascar",
    "atascan": "atascar",
    "atascando": "atascar",
    "atascara": "atascar",
    "atascarais": "atascar",
    "atascaran": "atascar",
    "atascaras": "atascar",
    "atascare": "atascar",
    "atascareis": "atascar",
    "atascaremos": "atascar",
    "atascaren": "atascar",
    "atascares": "atascar",
    "atascaron": "atascar",
    "atascará": "atascar",
    "atascarán": "atascar",
    "atascarás": "atascar",
    "atascaré": "atascar",
    "atascaréis": "atascar",
    "atascaría": "atascar",
    "atascaríais": "atascar",
    "atascaríamos": "atascar",
    "atascarían": "atascar",
    "atascarías": "atascar",
    "atascas": "atascar",
    "atascase": "atascar",
    "atascaseis": "atascar",
    "atascasen": "atascar",
    "atascases": "atascar",
    "atascaste": "atascar",
    "atascasteis": "atascar",
    "atasco": "atascar",
    "atascos": "atasco",
    "atascábamos": "atascar",
    "atascáis": "atascar",
    "atascáramos": "atascar",
    "atascáremos": "atascar",
    "atascásemos": "atascar",
    "atascó": "atascar",
    "atase": "atar",
    "ataseis": "atar",
    "atasen": "atar",
    "atases": "atar",
    "atasque": "atascar",
    "atasquemos": "atascar",
    "atasquen": "atascar",
    "atasques": "atascar",
    "atasqué": "atascar",
    "atasquéis": "atascar",
    "ataste": "atar",
    "atasteis": "atar",
    "ataujías": "ataujía",
    "atauriques": "ataurique",
    "ataviaba": "ataviar",
    "ataviabais": "ataviar",
    "ataviaban": "ataviar",
    "ataviabas": "ataviar",
    "ataviad": "ataviar",
    "ataviada": "ataviar",
    "ataviadas": "ataviar",
    "ataviado": "ataviar",
    "ataviados": "ataviar",
    "ataviamos": "ataviar",
    "ataviando": "ataviar",
    "ataviara": "ataviar",
    "ataviarais": "ataviar",
    "ataviaran": "ataviar",
    "ataviaras": "ataviar",
    "ataviare": "ataviar",
    "ataviareis": "ataviar",
    "ataviaremos": "ataviar",
    "ataviaren": "ataviar",
    "ataviares": "ataviar",
    "ataviaron": "ataviar",
    "ataviará": "ataviar",
    "ataviarán": "ataviar",
    "ataviarás": "ataviar",
    "ataviaré": "ataviar",
    "ataviaréis": "ataviar",
    "ataviaría": "ataviar",
    "ataviaríais": "ataviar",
    "ataviaríamos": "ataviar",
    "ataviarían": "ataviar",
    "ataviarías": "ataviar",
    "ataviase": "ataviar",
    "ataviaseis": "ataviar",
    "ataviasen": "ataviar",
    "ataviases": "ataviar",
    "ataviaste": "ataviar",
    "ataviasteis": "ataviar",
    "ataviemos": "ataviar",
    "atavismos": "atavismo",
    "ataviábamos": "ataviar",
    "ataviáis": "ataviar",
    "ataviáramos": "ataviar",
    "ataviáremos": "ataviar",
    "ataviásemos": "ataviar",
    "atavié": "ataviar",
    "ataviéis": "ataviar",
    "atavió": "ataviar",
    "atavía": "ataviar",
    "atavían": "ataviar",
    "atavías": "ataviar",
    "atavíe": "ataviar",
    "atavíen": "ataviar",
    "atavíes": "ataviar",
    "atavío": "ataviar",
    "atavíos": "atavío",
    "ataxias": "ataxia",
    "ataña": "atañer",
    "atañamos": "atañer",
    "atañan": "atañer",
    "atañas": "atañer",
    "atañe": "atañer",
    "atañed": "atañer",
    "atañemos": "atañer",
    "atañen": "atañer",
    "atañendo": "atañer",
    "atañera": "atañer",
    "atañerais": "atañer",
    "atañeran": "atañer",
    "atañeras": "atañer",
    "atañere": "atañer",
    "atañereis": "atañer",
    "atañeremos": "atañer",
    "atañeren": "atañer",
    "atañeres": "atañer",
    "atañeron": "atañer",
    "atañerá": "atañer",
    "atañerán": "atañer",
    "atañerás": "atañer",
    "atañeré": "atañer",
    "atañeréis": "atañer",
    "atañería": "atañer",
    "atañeríais": "atañer",
    "atañeríamos": "atañer",
    "atañerían": "atañer",
    "atañerías": "atañer",
    "atañes": "atañer",
    "atañese": "atañer",
    "atañeseis": "atañer",
    "atañesen": "atañer",
    "atañeses": "atañer",
    "atañida": "atañer",
    "atañidas": "atañer",
    "atañido": "atañer",
    "atañidos": "atañer",
    "atañimos": "atañer",
    "atañiste": "atañer",
    "atañisteis": "atañer",
    "ataño": "atañer",
    "atañáis": "atañer",
    "atañéis": "atañer",
    "atañéramos": "atañer",
    "atañéremos": "atañer",
    "atañésemos": "atañer",
    "atañí": "atañer",
    "atañía": "atañer",
    "atañíais": "atañer",
    "atañíamos": "atañer",
    "atañían": "atañer",
    "atañías": "atañer",
    "atañó": "atañer",
    "ataúdes": "ataúd",
    "ate": "atar",
    "atea": "ateo",
    "ateas": "ateo",
    "atece": "atezar",
    "atecemos": "atezar",
    "atecen": "atezar",
    "ateces": "atezar",
    "atecé": "atezar",
    "atecéis": "atezar",
    "atedia": "atediar",
    "atediaba": "atediar",
    "atediabais": "atediar",
    "atediaban": "atediar",
    "atediabas": "atediar",
    "atediad": "atediar",
    "atediada": "atediar",
    "atediadas": "atediar",
    "atediado": "atediar",
    "atediados": "atediar",
    "atediamos": "atediar",
    "atedian": "atediar",
    "atediando": "atediar",
    "atediara": "atediar",
    "atediarais": "atediar",
    "atediaran": "atediar",
    "atediaras": "atediar",
    "atediare": "atediar",
    "atediareis": "atediar",
    "atediaremos": "atediar",
    "atediaren": "atediar",
    "atediares": "atediar",
    "atediaron": "atediar",
    "atediará": "atediar",
    "atediarán": "atediar",
    "atediarás": "atediar",
    "atediaré": "atediar",
    "atediaréis": "atediar",
    "atediaría": "atediar",
    "atediaríais": "atediar",
    "atediaríamos": "atediar",
    "atediarían": "atediar",
    "atediarías": "atediar",
    "atedias": "atediar",
    "atediase": "atediar",
    "atediaseis": "atediar",
    "atediasen": "atediar",
    "atediases": "atediar",
    "atediaste": "atediar",
    "atediasteis": "atediar",
    "atedie": "atediar",
    "atediemos": "atediar",
    "atedien": "atediar",
    "atedies": "atediar",
    "atedio": "atediar",
    "atediábamos": "atediar",
    "atediáis": "atediar",
    "atediáramos": "atediar",
    "atediáremos": "atediar",
    "atediásemos": "atediar",
    "atedié": "atediar",
    "atediéis": "atediar",
    "atedió": "atediar",
    "atelajes": "atelaje",
    "atelanas": "atelana",
    "atemorice": "atemorizar",
    "atemoricemos": "atemorizar",
    "atemoricen": "atemorizar",
    "atemorices": "atemorizar",
    "atemoricé": "atemorizar",
    "atemoricéis": "atemorizar",
    "atemoriza": "atemorizar",
    "atemorizaba": "atemorizar",
    "atemorizabais": "atemorizar",
    "atemorizaban": "atemorizar",
    "atemorizabas": "atemorizar",
    "atemorizad": "atemorizar",
    "atemorizada": "atemorizar",
    "atemorizadas": "atemorizar",
    "atemorizado": "atemorizar",
    "atemorizados": "atemorizar",
    "atemorizamos": "atemorizar",
    "atemorizan": "atemorizar",
    "atemorizando": "atemorizar",
    "atemorizara": "atemorizar",
    "atemorizarais": "atemorizar",
    "atemorizaran": "atemorizar",
    "atemorizaras": "atemorizar",
    "atemorizare": "atemorizar",
    "atemorizareis": "atemorizar",
    "atemorizaremos": "atemorizar",
    "atemorizaren": "atemorizar",
    "atemorizares": "atemorizar",
    "atemorizaron": "atemorizar",
    "atemorizará": "atemorizar",
    "atemorizarán": "atemorizar",
    "atemorizarás": "atemorizar",
    "atemorizaré": "atemorizar",
    "atemorizaréis": "atemorizar",
    "atemorizaría": "atemorizar",
    "atemorizaríais": "atemorizar",
    "atemorizaríamos": "atemorizar",
    "atemorizarían": "atemorizar",
    "atemorizarías": "atemorizar",
    "atemorizas": "atemorizar",
    "atemorizase": "atemorizar",
    "atemorizaseis": "atemorizar",
    "atemorizasen": "atemorizar",
    "atemorizases": "atemorizar",
    "atemorizaste": "atemorizar",
    "atemorizasteis": "atemorizar",
    "atemorizo": "atemorizar",
    "atemorizábamos": "atemorizar",
    "atemorizáis": "atemorizar",
    "atemorizáramos": "atemorizar",
    "atemorizáremos": "atemorizar",
    "atemorizásemos": "atemorizar",
    "atemorizó": "atemorizar",
    "atemos": "atar",
    "atempera": "atemperar",
    "atemperaba": "atemperar",
    "atemperabais": "atemperar",
    "atemperaban": "atemperar",
    "atemperabas": "atemperar",
    "atemperaciones": "atemperación",
    "atemperad": "atemperar",
    "atemperada": "atemperar",
    "atemperadas": "atemperar",
    "atemperado": "atemperar",
    "atemperadores": "atemperador",
    "atemperados": "atemperar",
    "atemperamos": "atemperar",
    "atemperan": "atemperar",
    "atemperando": "atemperar",
    "atemperara": "atemperar",
    "atemperarais": "atemperar",
    "atemperaran": "atemperar",
    "atemperaras": "atemperar",
    "atemperare": "atemperar",
    "atemperareis": "atemperar",
    "atemperaremos": "atemperar",
    "atemperaren": "atemperar",
    "atemperares": "atemperar",
    "atemperaron": "atemperar",
    "atemperará": "atemperar",
    "atemperarán": "atemperar",
    "atemperarás": "atemperar",
    "atemperaré": "atemperar",
    "atemperaréis": "atemperar",
    "atemperaría": "atemperar",
    "atemperaríais": "atemperar",
    "atemperaríamos": "atemperar",
    "atemperarían": "atemperar",
    "atemperarías": "atemperar",
    "atemperas": "atemperar",
    "atemperase": "atemperar",
    "atemperaseis": "atemperar",
    "atemperasen": "atemperar",
    "atemperases": "atemperar",
    "atemperaste": "atemperar",
    "atemperasteis": "atemperar",
    "atempere": "atemperar",
    "atemperemos": "atemperar",
    "atemperen": "atemperar",
    "atemperes": "atemperar",
    "atempero": "atemperar",
    "atemperábamos": "atemperar",
    "atemperáis": "atemperar",
    "atemperáramos": "atemperar",
    "atemperáremos": "atemperar",
    "atemperásemos": "atemperar",
    "atemperé": "atemperar",
    "atemperéis": "atemperar",
    "atemperó": "atemperar",
    "aten": "atar",
    "atenace": "atenazar",
    "atenacea": "atenacear",
    "atenaceaba": "atenacear",
    "atenaceabais": "atenacear",
    "atenaceaban": "atenacear",
    "atenaceabas": "atenacear",
    "atenacead": "atenacear",
    "atenaceada": "atenacear",
    "atenaceadas": "atenacear",
    "atenaceado": "atenacear",
    "atenaceados": "atenacear",
    "atenaceamos": "atenacear",
    "atenacean": "atenacear",
    "atenaceando": "atenacear",
    "atenaceara": "atenacear",
    "atenacearais": "atenacear",
    "atenacearan": "atenacear",
    "atenacearas": "atenacear",
    "atenaceare": "atenacear",
    "atenaceareis": "atenacear",
    "atenacearemos": "atenacear",
    "atenacearen": "atenacear",
    "atenaceares": "atenacear",
    "atenacearon": "atenacear",
    "atenaceará": "atenacear",
    "atenacearán": "atenacear",
    "atenacearás": "atenacear",
    "atenacearé": "atenacear",
    "atenacearéis": "atenacear",
    "atenacearía": "atenacear",
    "atenacearíais": "atenacear",
    "atenacearíamos": "atenacear",
    "atenacearían": "atenacear",
    "atenacearías": "atenacear",
    "atenaceas": "atenacear",
    "atenacease": "atenacear",
    "atenaceaseis": "atenacear",
    "atenaceasen": "atenacear",
    "atenaceases": "atenacear",
    "atenaceaste": "atenacear",
    "atenaceasteis": "atenacear",
    "atenacee": "atenacear",
    "atenaceemos": "atenacear",
    "atenaceen": "atenacear",
    "atenacees": "atenacear",
    "atenacemos": "atenazar",
    "atenacen": "atenazar",
    "atenaceo": "atenacear",
    "atenaces": "atenazar",
    "atenaceábamos": "atenacear",
    "atenaceáis": "atenacear",
    "atenaceáramos": "atenacear",
    "atenaceáremos": "atenacear",
    "atenaceásemos": "atenacear",
    "atenaceé": "atenacear",
    "atenaceéis": "atenacear",
    "atenaceó": "atenacear",
    "atenacé": "atenazar",
    "atenacéis": "atenazar",
    "atenaza": "atenazar",
    "atenazaba": "atenazar",
    "atenazabais": "atenazar",
    "atenazaban": "atenazar",
    "atenazabas": "atenazar",
    "atenazad": "atenazar",
    "atenazada": "atenazar",
    "atenazadas": "atenazar",
    "atenazado": "atenazar",
    "atenazadora": "atenazador",
    "atenazadoras": "atenazador",
    "atenazadores": "atenazador",
    "atenazados": "atenazar",
    "atenazamientos": "atenazamiento",
    "atenazamos": "atenazar",
    "atenazan": "atenazar",
    "atenazando": "atenazar",
    "atenazara": "atenazar",
    "atenazarais": "atenazar",
    "atenazaran": "atenazar",
    "atenazaras": "atenazar",
    "atenazare": "atenazar",
    "atenazareis": "atenazar",
    "atenazaremos": "atenazar",
    "atenazaren": "atenazar",
    "atenazares": "atenazar",
    "atenazaron": "atenazar",
    "atenazará": "atenazar",
    "atenazarán": "atenazar",
    "atenazarás": "atenazar",
    "atenazaré": "atenazar",
    "atenazaréis": "atenazar",
    "atenazaría": "atenazar",
    "atenazaríais": "atenazar",
    "atenazaríamos": "atenazar",
    "atenazarían": "atenazar",
    "atenazarías": "atenazar",
    "atenazas": "atenazar",
    "atenazase": "atenazar",
    "atenazaseis": "atenazar",
    "atenazasen": "atenazar",
    "atenazases": "atenazar",
    "atenazaste": "atenazar",
    "atenazasteis": "atenazar",
    "atenazo": "atenazar",
    "atenazábamos": "atenazar",
    "atenazáis": "atenazar",
    "atenazáramos": "atenazar",
    "atenazáremos": "atenazar",
    "atenazásemos": "atenazar",
    "atenazó": "atenazar",
    "atenciones": "atención",
    "atendamos": "atender",
    "atended": "atender",
    "atendedora": "atendedor",
    "atendedoras": "atendedor",
    "atendedores": "atendedor",
    "atendemos": "atender",
    "atenderemos": "atender",
    "atenderá": "atender",
    "atenderán": "atender",
    "atenderás": "atender",
    "atenderé": "atender",
    "atenderéis": "atender",
    "atendería": "atender",
    "atenderíais": "atender",
    "atenderíamos": "atender",
    "atenderían": "atender",
    "atenderías": "atender",
    "atendibles": "atendible",
    "atendida": "atender",
    "atendidas": "atender",
    "atendido": "atender",
    "atendidos": "atender",
    "atendiendo": "atender",
    "atendiera": "atender",
    "atendierais": "atender",
    "atendieran": "atender",
    "atendieras": "atender",
    "atendiere": "atender",
    "atendiereis": "atender",
    "atendieren": "atender",
    "atendieres": "atender",
    "atendieron": "atender",
    "atendiese": "atender",
    "atendieseis": "atender",
    "atendiesen": "atender",
    "atendieses": "atender",
    "atendimos": "atender",
    "atendiste": "atender",
    "atendisteis": "atender",
    "atendiéramos": "atender",
    "atendiéremos": "atender",
    "atendiésemos": "atender",
    "atendió": "atender",
    "atendremos": "atener",
    "atendrá": "atener",
    "atendrán": "atener",
    "atendrás": "atener",
    "atendré": "atener",
    "atendréis": "atener",
    "atendría": "atener",
    "atendríais": "atener",
    "atendríamos": "atener",
    "atendrían": "atener",
    "atendrías": "atener",
    "atendáis": "atender",
    "atendéis": "atender",
    "atendí": "atender",
    "atendía": "atender",
    "atendíais": "atender",
    "atendíamos": "atender",
    "atendían": "atender",
    "atendías": "atender",
    "atenebra": "atenebrar",
    "atenebraba": "atenebrar",
    "atenebrabais": "atenebrar",
    "atenebraban": "atenebrar",
    "atenebrabas": "atenebrar",
    "atenebrad": "atenebrar",
    "atenebrada": "atenebrar",
    "atenebradas": "atenebrar",
    "atenebrado": "atenebrar",
    "atenebrados": "atenebrar",
    "atenebramos": "atenebrar",
    "atenebran": "atenebrar",
    "atenebrando": "atenebrar",
    "atenebrara": "atenebrar",
    "atenebrarais": "atenebrar",
    "atenebraran": "atenebrar",
    "atenebraras": "atenebrar",
    "atenebrare": "atenebrar",
    "atenebrareis": "atenebrar",
    "atenebraremos": "atenebrar",
    "atenebraren": "atenebrar",
    "atenebrares": "atenebrar",
    "atenebraron": "atenebrar",
    "atenebrará": "atenebrar",
    "atenebrarán": "atenebrar",
    "atenebrarás": "atenebrar",
    "atenebraré": "atenebrar",
    "atenebraréis": "atenebrar",
    "atenebraría": "atenebrar",
    "atenebraríais": "atenebrar",
    "atenebraríamos": "atenebrar",
    "atenebrarían": "atenebrar",
    "atenebrarías": "atenebrar",
    "atenebras": "atenebrar",
    "atenebrase": "atenebrar",
    "atenebraseis": "atenebrar",
    "atenebrasen": "atenebrar",
    "atenebrases": "atenebrar",
    "atenebraste": "atenebrar",
    "atenebrasteis": "atenebrar",
    "atenebre": "atenebrar",
    "atenebremos": "atenebrar",
    "atenebren": "atenebrar",
    "atenebres": "atenebrar",
    "atenebro": "atenebrar",
    "atenebrábamos": "atenebrar",
    "atenebráis": "atenebrar",
    "atenebráramos": "atenebrar",
    "atenebráremos": "atenebrar",
    "atenebrásemos": "atenebrar",
    "atenebré": "atenebrar",
    "atenebréis": "atenebrar",
    "atenebró": "atenebrar",
    "atened": "atener",
    "atenemos": "atener",
    "ateneos": "ateneo",
    "ateneístas": "ateneísta",
    "atenga": "atener",
    "atengamos": "atener",
    "atengan": "atener",
    "atengas": "atener",
    "atengo": "atener",
    "atengáis": "atener",
    "atenida": "atener",
    "atenidas": "atener",
    "atenido": "atener",
    "atenidos": "atener",
    "ateniendo": "atener",
    "atenienses": "ateniense",
    "atenololes": "atenolol",
    "atenta": "atento",
    "atentaba": "atentar",
    "atentabais": "atentar",
    "atentaban": "atentar",
    "atentabas": "atentar",
    "atentad": "atentar",
    "atentada": "atentar",
    "atentadas": "atentar",
    "atentado": "atentar",
    "atentados": "atentar",
    "atentamos": "atentar",
    "atentan": "atentar",
    "atentando": "atentar",
    "atentara": "atentar",
    "atentarais": "atentar",
    "atentaran": "atentar",
    "atentaras": "atentar",
    "atentare": "atentar",
    "atentareis": "atentar",
    "atentaremos": "atentar",
    "atentaren": "atentar",
    "atentares": "atentar",
    "atentaron": "atentar",
    "atentará": "atentar",
    "atentarán": "atentar",
    "atentarás": "atentar",
    "atentaré": "atentar",
    "atentaréis": "atentar",
    "atentaría": "atentar",
    "atentaríais": "atentar",
    "atentaríamos": "atentar",
    "atentarían": "atentar",
    "atentarías": "atentar",
    "atentas": "atento",
    "atentase": "atentar",
    "atentaseis": "atentar",
    "atentasen": "atentar",
    "atentases": "atentar",
    "atentaste": "atentar",
    "atentasteis": "atentar",
    "atentatoria": "atentatorio",
    "atentatorias": "atentatorio",
    "atentatorios": "atentatorio",
    "atente": "atentar",
    "atentemos": "atentar",
    "atenten": "atentar",
    "atentes": "atentar",
    "atento": "atentar",
    "atentos": "atento",
    "atentábamos": "atentar",
    "atentáis": "atentar",
    "atentáramos": "atentar",
    "atentáremos": "atentar",
    "atentásemos": "atentar",
    "atenté": "atentar",
    "atentéis": "atentar",
    "atentó": "atentar",
    "atenuaba": "atenuar",
    "atenuabais": "atenuar",
    "atenuaban": "atenuar",
    "atenuabas": "atenuar",
    "atenuaciones": "atenuación",
    "atenuad": "atenuar",
    "atenuada": "atenuar",
    "atenuadas": "atenuar",
    "atenuado": "atenuar",
    "atenuadora": "atenuador",
    "atenuadoras": "atenuador",
    "atenuadores": "atenuador",
    "atenuados": "atenuar",
    "atenuamos": "atenuar",
    "atenuando": "atenuar",
    "atenuantes": "atenuante",
    "atenuara": "atenuar",
    "atenuarais": "atenuar",
    "atenuaran": "atenuar",
    "atenuaras": "atenuar",
    "atenuare": "atenuar",
    "atenuareis": "atenuar",
    "atenuaremos": "atenuar",
    "atenuaren": "atenuar",
    "atenuares": "atenuar",
    "atenuaron": "atenuar",
    "atenuará": "atenuar",
    "atenuarán": "atenuar",
    "atenuarás": "atenuar",
    "atenuaré": "atenuar",
    "atenuaréis": "atenuar",
    "atenuaría": "atenuar",
    "atenuaríais": "atenuar",
    "atenuaríamos": "atenuar",
    "atenuarían": "atenuar",
    "atenuarías": "atenuar",
    "atenuase": "atenuar",
    "atenuaseis": "atenuar",
    "atenuasen": "atenuar",
    "atenuases": "atenuar",
    "atenuaste": "atenuar",
    "atenuasteis": "atenuar",
    "atenuemos": "atenuar",
    "atenuábamos": "atenuar",
    "atenuáis": "atenuar",
    "atenuáramos": "atenuar",
    "atenuáremos": "atenuar",
    "atenuásemos": "atenuar",
    "atenué": "atenuar",
    "atenuéis": "atenuar",
    "atenuó": "atenuar",
    "atenéis": "atener",
    "atenía": "atener",
    "ateníais": "atener",
    "ateníamos": "atener",
    "atenían": "atener",
    "atenías": "atener",
    "atenúa": "atenuar",
    "atenúan": "atenuar",
    "atenúas": "atenuar",
    "atenúe": "atenuar",
    "atenúen": "atenuar",
    "atenúes": "atenuar",
    "atenúo": "atenuar",
    "ateos": "ateo",
    "atera": "aterir",
    "ateramos": "aterir",
    "ateran": "aterir",
    "ateras": "aterir",
    "aterciopelada": "aterciopelado",
    "aterciopeladas": "aterciopelado",
    "aterciopelados": "aterciopelado",
    "atere": "aterir",
    "ateren": "aterir",
    "ateres": "aterir",
    "aterid": "aterir",
    "aterida": "aterir",
    "ateridas": "aterir",
    "aterido": "aterir",
    "ateridos": "aterir",
    "ateriendo": "aterir",
    "ateriera": "aterir",
    "aterierais": "aterir",
    "aterieran": "aterir",
    "aterieras": "aterir",
    "ateriere": "aterir",
    "ateriereis": "aterir",
    "aterieren": "aterir",
    "aterieres": "aterir",
    "aterieron": "aterir",
    "ateriese": "aterir",
    "aterieseis": "aterir",
    "ateriesen": "aterir",
    "aterieses": "aterir",
    "aterimientos": "aterimiento",
    "aterimos": "aterir",
    "ateriremos": "aterir",
    "aterirá": "aterir",
    "aterirán": "aterir",
    "aterirás": "aterir",
    "ateriré": "aterir",
    "ateriréis": "aterir",
    "ateriría": "aterir",
    "ateriríais": "aterir",
    "ateriríamos": "aterir",
    "aterirían": "aterir",
    "aterirías": "aterir",
    "ateriste": "aterir",
    "ateristeis": "aterir",
    "ateriéramos": "aterir",
    "ateriéremos": "aterir",
    "ateriésemos": "aterir",
    "aterió": "aterir",
    "atero": "aterir",
    "ateromas": "ateroma",
    "aterotrombótica": "aterotrombótico",
    "aterotrombóticas": "aterotrombótico",
    "aterotrombóticos": "aterotrombótico",
    "aterra": "aterrar",
    "aterraba": "aterrar",
    "aterrabais": "aterrar",
    "aterraban": "aterrar",
    "aterrabas": "aterrar",
    "aterrad": "aterrar",
    "aterrada": "aterrar",
    "aterradas": "aterrar",
    "aterrado": "aterrar",
    "aterradora": "aterrador",
    "aterradoras": "aterrador",
    "aterradores": "aterrador",
    "aterrados": "aterrar",
    "aterraja": "aterrajar",
    "aterrajaba": "aterrajar",
    "aterrajabais": "aterrajar",
    "aterrajaban": "aterrajar",
    "aterrajabas": "aterrajar",
    "aterrajad": "aterrajar",
    "aterrajada": "aterrajar",
    "aterrajadas": "aterrajar",
    "aterrajado": "aterrajar",
    "aterrajados": "aterrajar",
    "aterrajamos": "aterrajar",
    "aterrajan": "aterrajar",
    "aterrajando": "aterrajar",
    "aterrajara": "aterrajar",
    "aterrajarais": "aterrajar",
    "aterrajaran": "aterrajar",
    "aterrajaras": "aterrajar",
    "aterrajare": "aterrajar",
    "aterrajareis": "aterrajar",
    "aterrajaremos": "aterrajar",
    "aterrajaren": "aterrajar",
    "aterrajares": "aterrajar",
    "aterrajaron": "aterrajar",
    "aterrajará": "aterrajar",
    "aterrajarán": "aterrajar",
    "aterrajarás": "aterrajar",
    "aterrajaré": "aterrajar",
    "aterrajaréis": "aterrajar",
    "aterrajaría": "aterrajar",
    "aterrajaríais": "aterrajar",
    "aterrajaríamos": "aterrajar",
    "aterrajarían": "aterrajar",
    "aterrajarías": "aterrajar",
    "aterrajas": "aterrajar",
    "aterrajase": "aterrajar",
    "aterrajaseis": "aterrajar",
    "aterrajasen": "aterrajar",
    "aterrajases": "aterrajar",
    "aterrajaste": "aterrajar",
    "aterrajasteis": "aterrajar",
    "aterraje": "aterrajar",
    "aterrajemos": "aterrajar",
    "aterrajen": "aterrajar",
    "aterrajes": "aterraje",
    "aterrajo": "aterrajar",
    "aterrajábamos": "aterrajar",
    "aterrajáis": "aterrajar",
    "aterrajáramos": "aterrajar",
    "aterrajáremos": "aterrajar",
    "aterrajásemos": "aterrajar",
    "aterrajé": "aterrajar",
    "aterrajéis": "aterrajar",
    "aterrajó": "aterrajar",
    "aterramientos": "aterramiento",
    "aterramos": "aterrar",
    "aterran": "aterrar",
    "aterrando": "aterrar",
    "aterrara": "aterrar",
    "aterrarais": "aterrar",
    "aterraran": "aterrar",
    "aterraras": "aterrar",
    "aterrare": "aterrar",
    "aterrareis": "aterrar",
    "aterraremos": "aterrar",
    "aterraren": "aterrar",
    "aterrares": "aterrar",
    "aterraron": "aterrar",
    "aterrará": "aterrar",
    "aterrarán": "aterrar",
    "aterrarás": "aterrar",
    "aterraré": "aterrar",
    "aterraréis": "aterrar",
    "aterraría": "aterrar",
    "aterraríais": "aterrar",
    "aterraríamos": "aterrar",
    "aterrarían": "aterrar",
    "aterrarías": "aterrar",
    "aterras": "aterrar",
    "aterrase": "aterrar",
    "aterraseis": "aterrar",
    "aterrasen": "aterrar",
    "aterrases": "aterrar",
    "aterraste": "aterrar",
    "aterrasteis": "aterrar",
    "aterre": "aterrar",
    "aterremos": "aterrar",
    "aterren": "aterrar",
    "aterres": "aterrar",
    "aterrice": "aterrizar",
    "aterricemos": "aterrizar",
    "aterricen": "aterrizar",
    "aterrices": "aterrizar",
    "aterricé": "aterrizar",
    "aterricéis": "aterrizar",
    "aterriza": "aterrizar",
    "aterrizaba": "aterrizar",
    "aterrizabais": "aterrizar",
    "aterrizaban": "aterrizar",
    "aterrizabas": "aterrizar",
    "aterrizad": "aterrizar",
    "aterrizada": "aterrizar",
    "aterrizadas": "aterrizar",
    "aterrizado": "aterrizar",
    "aterrizados": "aterrizar",
    "aterrizajes": "aterrizaje",
    "aterrizamos": "aterrizar",
    "aterrizan": "aterrizar",
    "aterrizando": "aterrizar",
    "aterrizara": "aterrizar",
    "aterrizarais": "aterrizar",
    "aterrizaran": "aterrizar",
    "aterrizaras": "aterrizar",
    "aterrizare": "aterrizar",
    "aterrizareis": "aterrizar",
    "aterrizaremos": "aterrizar",
    "aterrizaren": "aterrizar",
    "aterrizares": "aterrizar",
    "aterrizaron": "aterrizar",
    "aterrizará": "aterrizar",
    "aterrizarán": "aterrizar",
    "aterrizarás": "aterrizar",
    "aterrizaré": "aterrizar",
    "aterrizaréis": "aterrizar",
    "aterrizaría": "aterrizar",
    "aterrizaríais": "aterrizar",
    "aterrizaríamos": "aterrizar",
    "aterrizarían": "aterrizar",
    "aterrizarías": "aterrizar",
    "aterrizas": "aterrizar",
    "aterrizase": "aterrizar",
    "aterrizaseis": "aterrizar",
    "aterrizasen": "aterrizar",
    "aterrizases": "aterrizar",
    "aterrizaste": "aterrizar",
    "aterrizasteis": "aterrizar",
    "aterrizo": "aterrizar",
    "aterrizábamos": "aterrizar",
    "aterrizáis": "aterrizar",
    "aterrizáramos": "aterrizar",
    "aterrizáremos": "aterrizar",
    "aterrizásemos": "aterrizar",
    "aterrizó": "aterrizar",
    "aterro": "aterrar",
    "aterrona": "aterronar",
    "aterronaba": "aterronar",
    "aterronabais": "aterronar",
    "aterronaban": "aterronar",
    "aterronabas": "aterronar",
    "aterronad": "aterronar",
    "aterronada": "aterronar",
    "aterronadas": "aterronar",
    "aterronado": "aterronar",
    "aterronados": "aterronar",
    "aterronamos": "aterronar",
    "aterronan": "aterronar",
    "aterronando": "aterronar",
    "aterronara": "aterronar",
    "aterronarais": "aterronar",
    "aterronaran": "aterronar",
    "aterronaras": "aterronar",
    "aterronare": "aterronar",
    "aterronareis": "aterronar",
    "aterronaremos": "aterronar",
    "aterronaren": "aterronar",
    "aterronares": "aterronar",
    "aterronaron": "aterronar",
    "aterronará": "aterronar",
    "aterronarán": "aterronar",
    "aterronarás": "aterronar",
    "aterronaré": "aterronar",
    "aterronaréis": "aterronar",
    "aterronaría": "aterronar",
    "aterronaríais": "aterronar",
    "aterronaríamos": "aterronar",
    "aterronarían": "aterronar",
    "aterronarías": "aterronar",
    "aterronas": "aterronar",
    "aterronase": "aterronar",
    "aterronaseis": "aterronar",
    "aterronasen": "aterronar",
    "aterronases": "aterronar",
    "aterronaste": "aterronar",
    "aterronasteis": "aterronar",
    "aterrone": "aterronar",
    "aterronemos": "aterronar",
    "aterronen": "aterronar",
    "aterrones": "aterronar",
    "aterrono": "aterronar",
    "aterronábamos": "aterronar",
    "aterronáis": "aterronar",
    "aterronáramos": "aterronar",
    "aterronáremos": "aterronar",
    "aterronásemos": "aterronar",
    "aterroné": "aterronar",
    "aterronéis": "aterronar",
    "aterronó": "aterronar",
    "aterrorice": "aterrorizar",
    "aterroricemos": "aterrorizar",
    "aterroricen": "aterrorizar",
    "aterrorices": "aterrorizar",
    "aterroricé": "aterrorizar",
    "aterroricéis": "aterrorizar",
    "aterroriza": "aterrorizar",
    "aterrorizaba": "aterrorizar",
    "aterrorizabais": "aterrorizar",
    "aterrorizaban": "aterrorizar",
    "aterrorizabas": "aterrorizar",
    "aterrorizad": "aterrorizar",
    "aterrorizada": "aterrorizar",
    "aterrorizadas": "aterrorizar",
    "aterrorizado": "aterrorizar",
    "aterrorizados": "aterrorizar",
    "aterrorizamos": "aterrorizar",
    "aterrorizan": "aterrorizar",
    "aterrorizando": "aterrorizar",
    "aterrorizara": "aterrorizar",
    "aterrorizarais": "aterrorizar",
    "aterrorizaran": "aterrorizar",
    "aterrorizaras": "aterrorizar",
    "aterrorizare": "aterrorizar",
    "aterrorizareis": "aterrorizar",
    "aterrorizaremos": "aterrorizar",
    "aterrorizaren": "aterrorizar",
    "aterrorizares": "aterrorizar",
    "aterrorizaron": "aterrorizar",
    "aterrorizará": "aterrorizar",
    "aterrorizarán": "aterrorizar",
    "aterrorizarás": "aterrorizar",
    "aterrorizaré": "aterrorizar",
    "aterrorizaréis": "aterrorizar",
    "aterrorizaría": "aterrorizar",
    "aterrorizaríais": "aterrorizar",
    "aterrorizaríamos": "aterrorizar",
    "aterrorizarían": "aterrorizar",
    "aterrorizarías": "aterrorizar",
    "aterrorizas": "aterrorizar",
    "aterrorizase": "aterrorizar",
    "aterrorizaseis": "aterrorizar",
    "aterrorizasen": "aterrorizar",
    "aterrorizases": "aterrorizar",
    "aterrorizaste": "aterrorizar",
    "aterrorizasteis": "aterrorizar",
    "aterrorizo": "aterrorizar",
    "aterrorizábamos": "aterrorizar",
    "aterrorizáis": "aterrorizar",
    "aterrorizáramos": "aterrorizar",
    "aterrorizáremos": "aterrorizar",
    "aterrorizásemos": "aterrorizar",
    "aterrorizó": "aterrorizar",
    "aterrábamos": "aterrar",
    "aterráis": "aterrar",
    "aterráramos": "aterrar",
    "aterráremos": "aterrar",
    "aterrásemos": "aterrar",
    "aterré": "aterrar",
    "aterréis": "aterrar",
    "aterró": "aterrar",
    "ateráis": "aterir",
    "aterí": "aterir",
    "atería": "aterir",
    "ateríais": "aterir",
    "ateríamos": "aterir",
    "aterían": "aterir",
    "aterías": "aterir",
    "aterís": "aterir",
    "ates": "atar",
    "atesaba": "atesar",
    "atesabais": "atesar",
    "atesaban": "atesar",
    "atesabas": "atesar",
    "atesad": "atesar",
    "atesada": "atesar",
    "atesadas": "atesar",
    "atesado": "atesar",
    "atesados": "atesar",
    "atesamos": "atesar",
    "atesando": "atesar",
    "atesara": "atesar",
    "atesarais": "atesar",
    "atesaran": "atesar",
    "atesaras": "atesar",
    "atesare": "atesar",
    "atesareis": "atesar",
    "atesaremos": "atesar",
    "atesaren": "atesar",
    "atesares": "atesar",
    "atesaron": "atesar",
    "atesará": "atesar",
    "atesarán": "atesar",
    "atesarás": "atesar",
    "atesaré": "atesar",
    "atesaréis": "atesar",
    "atesaría": "atesar",
    "atesaríais": "atesar",
    "atesaríamos": "atesar",
    "atesarían": "atesar",
    "atesarías": "atesar",
    "atesase": "atesar",
    "atesaseis": "atesar",
    "atesasen": "atesar",
    "atesases": "atesar",
    "atesaste": "atesar",
    "atesasteis": "atesar",
    "atesemos": "atesar",
    "atesora": "atesorar",
    "atesoraba": "atesorar",
    "atesorabais": "atesorar",
    "atesoraban": "atesorar",
    "atesorabas": "atesorar",
    "atesorad": "atesorar",
    "atesorada": "atesorar",
    "atesoradas": "atesorar",
    "atesorado": "atesorar",
    "atesorados": "atesorar",
    "atesoramientos": "atesoramiento",
    "atesoramos": "atesorar",
    "atesoran": "atesorar",
    "atesorando": "atesorar",
    "atesorara": "atesorar",
    "atesorarais": "atesorar",
    "atesoraran": "atesorar",
    "atesoraras": "atesorar",
    "atesorare": "atesorar",
    "atesorareis": "atesorar",
    "atesoraremos": "atesorar",
    "atesoraren": "atesorar",
    "atesorares": "atesorar",
    "atesoraron": "atesorar",
    "atesorará": "atesorar",
    "atesorarán": "atesorar",
    "atesorarás": "atesorar",
    "atesoraré": "atesorar",
    "atesoraréis": "atesorar",
    "atesoraría": "atesorar",
    "atesoraríais": "atesorar",
    "atesoraríamos": "atesorar",
    "atesorarían": "atesorar",
    "atesorarías": "atesorar",
    "atesoras": "atesorar",
    "atesorase": "atesorar",
    "atesoraseis": "atesorar",
    "atesorasen": "atesorar",
    "atesorases": "atesorar",
    "atesoraste": "atesorar",
    "atesorasteis": "atesorar",
    "atesore": "atesorar",
    "atesoremos": "atesorar",
    "atesoren": "atesorar",
    "atesores": "atesorar",
    "atesoro": "atesorar",
    "atesorábamos": "atesorar",
    "atesoráis": "atesorar",
    "atesoráramos": "atesorar",
    "atesoráremos": "atesorar",
    "atesorásemos": "atesorar",
    "atesoré": "atesorar",
    "atesoréis": "atesorar",
    "atesoró": "atesorar",
    "atesta": "atestar",
    "atestaba": "atestar",
    "atestabais": "atestar",
    "atestaban": "atestar",
    "atestabas": "atestar",
    "atestaciones": "atestación",
    "atestad": "atestar",
    "atestada": "atestar",
    "atestadas": "atestar",
    "atestado": "atestar",
    "atestados": "atestar",
    "atestaduras": "atestadura",
    "atestamientos": "atestamiento",
    "atestamos": "atestar",
    "atestan": "atestar",
    "atestando": "atestar",
    "atestara": "atestar",
    "atestarais": "atestar",
    "atestaran": "atestar",
    "atestaras": "atestar",
    "atestare": "atestar",
    "atestareis": "atestar",
    "atestaremos": "atestar",
    "atestaren": "atestar",
    "atestares": "atestar",
    "atestaron": "atestar",
    "atestará": "atestar",
    "atestarán": "atestar",
    "atestarás": "atestar",
    "atestaré": "atestar",
    "atestaréis": "atestar",
    "atestaría": "atestar",
    "atestaríais": "atestar",
    "atestaríamos": "atestar",
    "atestarían": "atestar",
    "atestarías": "atestar",
    "atestas": "atestar",
    "atestase": "atestar",
    "atestaseis": "atestar",
    "atestasen": "atestar",
    "atestases": "atestar",
    "atestaste": "atestar",
    "atestasteis": "atestar",
    "ateste": "atestar",
    "atestemos": "atestar",
    "atesten": "atestar",
    "atestes": "atestar",
    "atestigua": "atestiguar",
    "atestiguaba": "atestiguar",
    "atestiguabais": "atestiguar",
    "atestiguaban": "atestiguar",
    "atestiguabas": "atestiguar",
    "atestiguaciones": "atestiguación",
    "atestiguad": "atestiguar",
    "atestiguada": "atestiguar",
    "atestiguadas": "atestiguar",
    "atestiguado": "atestiguar",
    "atestiguados": "atestiguar",
    "atestiguamientos": "atestiguamiento",
    "atestiguamos": "atestiguar",
    "atestiguan": "atestiguar",
    "atestiguando": "atestiguar",
    "atestiguara": "atestiguar",
    "atestiguarais": "atestiguar",
    "atestiguaran": "atestiguar",
    "atestiguaras": "atestiguar",
    "atestiguare": "atestiguar",
    "atestiguareis": "atestiguar",
    "atestiguaremos": "atestiguar",
    "atestiguaren": "atestiguar",
    "atestiguares": "atestiguar",
    "atestiguaron": "atestiguar",
    "atestiguará": "atestiguar",
    "atestiguarán": "atestiguar",
    "atestiguarás": "atestiguar",
    "atestiguaré": "atestiguar",
    "atestiguaréis": "atestiguar",
    "atestiguaría": "atestiguar",
    "atestiguaríais": "atestiguar",
    "atestiguaríamos": "atestiguar",
    "atestiguarían": "atestiguar",
    "atestiguarías": "atestiguar",
    "atestiguas": "atestiguar",
    "atestiguase": "atestiguar",
    "atestiguaseis": "atestiguar",
    "atestiguasen": "atestiguar",
    "atestiguases": "atestiguar",
    "atestiguaste": "atestiguar",
    "atestiguasteis": "atestiguar",
    "atestiguo": "atestiguar",
    "atestiguábamos": "atestiguar",
    "atestiguáis": "atestiguar",
    "atestiguáramos": "atestiguar",
    "atestiguáremos": "atestiguar",
    "atestiguásemos": "atestiguar",
    "atestiguó": "atestiguar",
    "atestigüe": "atestiguar",
    "atestigüemos": "atestiguar",
    "atestigüen": "atestiguar",
    "atestigües": "atestiguar",
    "atestigüé": "atestiguar",
    "atestigüéis": "atestiguar",
    "atesto": "atestar",
    "atestábamos": "atestar",
    "atestáis": "atestar",
    "atestáramos": "atestar",
    "atestáremos": "atestar",
    "atestásemos": "atestar",
    "atesté": "atestar",
    "atestéis": "atestar",
    "atestó": "atestar",
    "atesábamos": "atesar",
    "atesáis": "atesar",
    "atesáramos": "atesar",
    "atesáremos": "atesar",
    "atesásemos": "atesar",
    "atesé": "atesar",
    "ateséis": "atesar",
    "atesó": "atesar",
    "ateza": "atezar",
    "atezaba": "atezar",
    "atezabais": "atezar",
    "atezaban": "atezar",
    "atezabas": "atezar",
    "atezad": "atezar",
    "atezada": "atezar",
    "atezadas": "atezar",
    "atezado": "atezar",
    "atezados": "atezar",
    "atezamos": "atezar",
    "atezan": "atezar",
    "atezando": "atezar",
    "atezara": "atezar",
    "atezarais": "atezar",
    "atezaran": "atezar",
    "atezaras": "atezar",
    "atezare": "atezar",
    "atezareis": "atezar",
    "atezaremos": "atezar",
    "atezaren": "atezar",
    "atezares": "atezar",
    "atezaron": "atezar",
    "atezará": "atezar",
    "atezarán": "atezar",
    "atezarás": "atezar",
    "atezaré": "atezar",
    "atezaréis": "atezar",
    "atezaría": "atezar",
    "atezaríais": "atezar",
    "atezaríamos": "atezar",
    "atezarían": "atezar",
    "atezarías": "atezar",
    "atezas": "atezar",
    "atezase": "atezar",
    "atezaseis": "atezar",
    "atezasen": "atezar",
    "atezases": "atezar",
    "atezaste": "atezar",
    "atezasteis": "atezar",
    "atezo": "atezar",
    "atezábamos": "atezar",
    "atezáis": "atezar",
    "atezáramos": "atezar",
    "atezáremos": "atezar",
    "atezásemos": "atezar",
    "atezó": "atezar",
    "ateísmos": "ateísmo",
    "ateístas": "ateísta",
    "atiba": "atibar",
    "atibaba": "atibar",
    "atibabais": "atibar",
    "atibaban": "atibar",
    "atibabas": "atibar",
    "atibad": "atibar",
    "atibada": "atibar",
    "atibadas": "atibar",
    "atibado": "atibar",
    "atibados": "atibar",
    "atibamos": "atibar",
    "atiban": "atibar",
    "atibando": "atibar",
    "atibara": "atibar",
    "atibarais": "atibar",
    "atibaran": "atibar",
    "atibaras": "atibar",
    "atibare": "atibar",
    "atibareis": "atibar",
    "atibaremos": "atibar",
    "atibaren": "atibar",
    "atibares": "atibar",
    "atibaron": "atibar",
    "atibará": "atibar",
    "atibarán": "atibar",
    "atibarás": "atibar",
    "atibaré": "atibar",
    "atibaréis": "atibar",
    "atibaría": "atibar",
    "atibaríais": "atibar",
    "atibaríamos": "atibar",
    "atibarían": "atibar",
    "atibarías": "atibar",
    "atibas": "atibar",
    "atibase": "atibar",
    "atibaseis": "atibar",
    "atibasen": "atibar",
    "atibases": "atibar",
    "atibaste": "atibar",
    "atibasteis": "atibar",
    "atibe": "atibar",
    "atibemos": "atibar",
    "atiben": "atibar",
    "atibes": "atibar",
    "atibo": "atibar",
    "atiborra": "atiborrar",
    "atiborraba": "atiborrar",
    "atiborrabais": "atiborrar",
    "atiborraban": "atiborrar",
    "atiborrabas": "atiborrar",
    "atiborrad": "atiborrar",
    "atiborrada": "atiborrar",
    "atiborradas": "atiborrar",
    "atiborrado": "atiborrar",
    "atiborrados": "atiborrar",
    "atiborramientos": "atiborramiento",
    "atiborramos": "atiborrar",
    "atiborran": "atiborrar",
    "atiborrando": "atiborrar",
    "atiborrara": "atiborrar",
    "atiborrarais": "atiborrar",
    "atiborraran": "atiborrar",
    "atiborraras": "atiborrar",
    "atiborrare": "atiborrar",
    "atiborrareis": "atiborrar",
    "atiborraremos": "atiborrar",
    "atiborraren": "atiborrar",
    "atiborrares": "atiborrar",
    "atiborraron": "atiborrar",
    "atiborrará": "atiborrar",
    "atiborrarán": "atiborrar",
    "atiborrarás": "atiborrar",
    "atiborraré": "atiborrar",
    "atiborraréis": "atiborrar",
    "atiborraría": "atiborrar",
    "atiborraríais": "atiborrar",
    "atiborraríamos": "atiborrar",
    "atiborrarían": "atiborrar",
    "atiborrarías": "atiborrar",
    "atiborras": "atiborrar",
    "atiborrase": "atiborrar",
    "atiborraseis": "atiborrar",
    "atiborrasen": "atiborrar",
    "atiborrases": "atiborrar",
    "atiborraste": "atiborrar",
    "atiborrasteis": "atiborrar",
    "atiborre": "atiborrar",
    "atiborremos": "atiborrar",
    "atiborren": "atiborrar",
    "atiborres": "atiborrar",
    "atiborro": "atiborrar",
    "atiborrábamos": "atiborrar",
    "atiborráis": "atiborrar",
    "atiborráramos": "atiborrar",
    "atiborráremos": "atiborrar",
    "atiborrásemos": "atiborrar",
    "atiborré": "atiborrar",
    "atiborréis": "atiborrar",
    "atiborró": "atiborrar",
    "atibábamos": "atibar",
    "atibáis": "atibar",
    "atibáramos": "atibar",
    "atibáremos": "atibar",
    "atibásemos": "atibar",
    "atibé": "atibar",
    "atibéis": "atibar",
    "atibó": "atibar",
    "atice": "atizar",
    "aticemos": "atizar",
    "aticen": "atizar",
    "atices": "atizar",
    "aticismos": "aticismo",
    "aticistas": "aticista",
    "aticoantrotomías": "aticoantrotomía",
    "aticotomías": "aticotomía",
    "aticé": "atizar",
    "aticéis": "atizar",
    "atienda": "atender",
    "atiendan": "atender",
    "atiendas": "atender",
    "atiende": "atender",
    "atienden": "atender",
    "atiendes": "atender",
    "atiendo": "atender",
    "atiene": "atener",
    "atienen": "atener",
    "atienes": "atener",
    "atierra": "aterrar",
    "atierran": "aterrar",
    "atierras": "aterrar",
    "atierre": "aterrar",
    "atierren": "aterrar",
    "atierres": "atierre",
    "atierro": "aterrar",
    "atiesa": "atiesar",
    "atiesaba": "atiesar",
    "atiesabais": "atiesar",
    "atiesaban": "atiesar",
    "atiesabas": "atiesar",
    "atiesad": "atiesar",
    "atiesada": "atiesar",
    "atiesadas": "atiesar",
    "atiesado": "atiesar",
    "atiesados": "atiesar",
    "atiesamientos": "atiesamiento",
    "atiesamos": "atiesar",
    "atiesan": "atiesar",
    "atiesando": "atiesar",
    "atiesara": "atiesar",
    "atiesarais": "atiesar",
    "atiesaran": "atiesar",
    "atiesaras": "atiesar",
    "atiesare": "atiesar",
    "atiesareis": "atiesar",
    "atiesaremos": "atiesar",
    "atiesaren": "atiesar",
    "atiesares": "atiesar",
    "atiesaron": "atiesar",
    "atiesará": "atiesar",
    "atiesarán": "atiesar",
    "atiesarás": "atiesar",
    "atiesaré": "atiesar",
    "atiesaréis": "atiesar",
    "atiesaría": "atiesar",
    "atiesaríais": "atiesar",
    "atiesaríamos": "atiesar",
    "atiesarían": "atiesar",
    "atiesarías": "atiesar",
    "atiesas": "atiesar",
    "atiesase": "atiesar",
    "atiesaseis": "atiesar",
    "atiesasen": "atiesar",
    "atiesases": "atiesar",
    "atiesaste": "atiesar",
    "atiesasteis": "atiesar",
    "atiese": "atiesar",
    "atiesemos": "atiesar",
    "atiesen": "atiesar",
    "atieses": "atiesar",
    "atieso": "atiesar",
    "atiesta": "atestar",
    "atiestan": "atestar",
    "atiestas": "atestar",
    "atieste": "atestar",
    "atiesten": "atestar",
    "atiestes": "atestar",
    "atiesto": "atestar",
    "atiesábamos": "atiesar",
    "atiesáis": "atiesar",
    "atiesáramos": "atiesar",
    "atiesáremos": "atiesar",
    "atiesásemos": "atiesar",
    "atiesé": "atiesar",
    "atieséis": "atiesar",
    "atiesó": "atiesar",
    "atifles": "atifle",
    "atigrada": "atigrado",
    "atigradas": "atigrado",
    "atigrados": "atigrado",
    "atilanas": "atilana",
    "atilda": "atildar",
    "atildaba": "atildar",
    "atildabais": "atildar",
    "atildaban": "atildar",
    "atildabas": "atildar",
    "atildad": "atildar",
    "atildada": "atildar",
    "atildadas": "atildar",
    "atildado": "atildar",
    "atildados": "atildar",
    "atildaduras": "atildadura",
    "atildamientos": "atildamiento",
    "atildamos": "atildar",
    "atildan": "atildar",
    "atildando": "atildar",
    "atildara": "atildar",
    "atildarais": "atildar",
    "atildaran": "atildar",
    "atildaras": "atildar",
    "atildare": "atildar",
    "atildareis": "atildar",
    "atildaremos": "atildar",
    "atildaren": "atildar",
    "atildares": "atildar",
    "atildaron": "atildar",
    "atildará": "atildar",
    "atildarán": "atildar",
    "atildarás": "atildar",
    "atildaré": "atildar",
    "atildaréis": "atildar",
    "atildaría": "atildar",
    "atildaríais": "atildar",
    "atildaríamos": "atildar",
    "atildarían": "atildar",
    "atildarías": "atildar",
    "atildas": "atildar",
    "atildase": "atildar",
    "atildaseis": "atildar",
    "atildasen": "atildar",
    "atildases": "atildar",
    "atildaste": "atildar",
    "atildasteis": "atildar",
    "atilde": "atildar",
    "atildemos": "atildar",
    "atilden": "atildar",
    "atildes": "atildar",
    "atildo": "atildar",
    "atildábamos": "atildar",
    "atildáis": "atildar",
    "atildáramos": "atildar",
    "atildáremos": "atildar",
    "atildásemos": "atildar",
    "atildé": "atildar",
    "atildéis": "atildar",
    "atildó": "atildar",
    "atina": "atinar",
    "atinaba": "atinar",
    "atinabais": "atinar",
    "atinaban": "atinar",
    "atinabas": "atinar",
    "atinad": "atinar",
    "atinada": "atinar",
    "atinadas": "atinar",
    "atinado": "atinar",
    "atinados": "atinar",
    "atinamos": "atinar",
    "atinan": "atinar",
    "atinando": "atinar",
    "atinara": "atinar",
    "atinarais": "atinar",
    "atinaran": "atinar",
    "atinaras": "atinar",
    "atinare": "atinar",
    "atinareis": "atinar",
    "atinaremos": "atinar",
    "atinaren": "atinar",
    "atinares": "atinar",
    "atinaron": "atinar",
    "atinará": "atinar",
    "atinarán": "atinar",
    "atinarás": "atinar",
    "atinaré": "atinar",
    "atinaréis": "atinar",
    "atinaría": "atinar",
    "atinaríais": "atinar",
    "atinaríamos": "atinar",
    "atinarían": "atinar",
    "atinarías": "atinar",
    "atinas": "atinar",
    "atinase": "atinar",
    "atinaseis": "atinar",
    "atinasen": "atinar",
    "atinases": "atinar",
    "atinaste": "atinar",
    "atinasteis": "atinar",
    "atincona": "atinconar",
    "atinconaba": "atinconar",
    "atinconabais": "atinconar",
    "atinconaban": "atinconar",
    "atinconabas": "atinconar",
    "atinconad": "atinconar",
    "atinconada": "atinconar",
    "atinconadas": "atinconar",
    "atinconado": "atinconar",
    "atinconados": "atinconar",
    "atinconamos": "atinconar",
    "atinconan": "atinconar",
    "atinconando": "atinconar",
    "atinconara": "atinconar",
    "atinconarais": "atinconar",
    "atinconaran": "atinconar",
    "atinconaras": "atinconar",
    "atinconare": "atinconar",
    "atinconareis": "atinconar",
    "atinconaremos": "atinconar",
    "atinconaren": "atinconar",
    "atinconares": "atinconar",
    "atinconaron": "atinconar",
    "atinconará": "atinconar",
    "atinconarán": "atinconar",
    "atinconarás": "atinconar",
    "atinconaré": "atinconar",
    "atinconaréis": "atinconar",
    "atinconaría": "atinconar",
    "atinconaríais": "atinconar",
    "atinconaríamos": "atinconar",
    "atinconarían": "atinconar",
    "atinconarías": "atinconar",
    "atinconas": "atinconar",
    "atinconase": "atinconar",
    "atinconaseis": "atinconar",
    "atinconasen": "atinconar",
    "atinconases": "atinconar",
    "atinconaste": "atinconar",
    "atinconasteis": "atinconar",
    "atincone": "atinconar",
    "atinconemos": "atinconar",
    "atinconen": "atinconar",
    "atincones": "atinconar",
    "atincono": "atinconar",
    "atinconábamos": "atinconar",
    "atinconáis": "atinconar",
    "atinconáramos": "atinconar",
    "atinconáremos": "atinconar",
    "atinconásemos": "atinconar",
    "atinconé": "atinconar",
    "atinconéis": "atinconar",
    "atinconó": "atinconar",
    "atine": "atinar",
    "atinemos": "atinar",
    "atinen": "atinar",
    "atinentes": "atinente",
    "atines": "atinar",
    "atino": "atinar",
    "atinábamos": "atinar",
    "atináis": "atinar",
    "atináramos": "atinar",
    "atináremos": "atinar",
    "atinásemos": "atinar",
    "atiné": "atinar",
    "atinéis": "atinar",
    "atinó": "atinar",
    "atipicidades": "atipicidad",
    "atipla": "atiplar",
    "atiplaba": "atiplar",
    "atiplabais": "atiplar",
    "atiplaban": "atiplar",
    "atiplabas": "atiplar",
    "atiplad": "atiplar",
    "atiplada": "atiplar",
    "atipladas": "atiplar",
    "atiplado": "atiplar",
    "atiplados": "atiplar",
    "atiplamos": "atiplar",
    "atiplan": "atiplar",
    "atiplando": "atiplar",
    "atiplara": "atiplar",
    "atiplarais": "atiplar",
    "atiplaran": "atiplar",
    "atiplaras": "atiplar",
    "atiplare": "atiplar",
    "atiplareis": "atiplar",
    "atiplaremos": "atiplar",
    "atiplaren": "atiplar",
    "atiplares": "atiplar",
    "atiplaron": "atiplar",
    "atiplará": "atiplar",
    "atiplarán": "atiplar",
    "atiplarás": "atiplar",
    "atiplaré": "atiplar",
    "atiplaréis": "atiplar",
    "atiplaría": "atiplar",
    "atiplaríais": "atiplar",
    "atiplaríamos": "atiplar",
    "atiplarían": "atiplar",
    "atiplarías": "atiplar",
    "atiplas": "atiplar",
    "atiplase": "atiplar",
    "atiplaseis": "atiplar",
    "atiplasen": "atiplar",
    "atiplases": "atiplar",
    "atiplaste": "atiplar",
    "atiplasteis": "atiplar",
    "atiple": "atiplar",
    "atiplemos": "atiplar",
    "atiplen": "atiplar",
    "atiples": "atiplar",
    "atiplo": "atiplar",
    "atiplábamos": "atiplar",
    "atipláis": "atiplar",
    "atipláramos": "atiplar",
    "atipláremos": "atiplar",
    "atiplásemos": "atiplar",
    "atiplé": "atiplar",
    "atipléis": "atiplar",
    "atipló": "atiplar",
    "atiranta": "atirantar",
    "atirantaba": "atirantar",
    "atirantabais": "atirantar",
    "atirantaban": "atirantar",
    "atirantabas": "atirantar",
    "atirantad": "atirantar",
    "atirantada": "atirantar",
    "atirantadas": "atirantar",
    "atirantado": "atirantar",
    "atirantados": "atirantar",
    "atirantamos": "atirantar",
    "atirantan": "atirantar",
    "atirantando": "atirantar",
    "atirantara": "atirantar",
    "atirantarais": "atirantar",
    "atirantaran": "atirantar",
    "atirantaras": "atirantar",
    "atirantare": "atirantar",
    "atirantareis": "atirantar",
    "atirantaremos": "atirantar",
    "atirantaren": "atirantar",
    "atirantares": "atirantar",
    "atirantaron": "atirantar",
    "atirantará": "atirantar",
    "atirantarán": "atirantar",
    "atirantarás": "atirantar",
    "atirantaré": "atirantar",
    "atirantaréis": "atirantar",
    "atirantaría": "atirantar",
    "atirantaríais": "atirantar",
    "atirantaríamos": "atirantar",
    "atirantarían": "atirantar",
    "atirantarías": "atirantar",
    "atirantas": "atirantar",
    "atirantase": "atirantar",
    "atirantaseis": "atirantar",
    "atirantasen": "atirantar",
    "atirantases": "atirantar",
    "atirantaste": "atirantar",
    "atirantasteis": "atirantar",
    "atirante": "atirantar",
    "atirantemos": "atirantar",
    "atiranten": "atirantar",
    "atirantes": "atirantar",
    "atiranto": "atirantar",
    "atirantábamos": "atirantar",
    "atirantáis": "atirantar",
    "atirantáramos": "atirantar",
    "atirantáremos": "atirantar",
    "atirantásemos": "atirantar",
    "atiranté": "atirantar",
    "atirantéis": "atirantar",
    "atirantó": "atirantar",
    "atisba": "atisbar",
    "atisbaba": "atisbar",
    "atisbabais": "atisbar",
    "atisbaban": "atisbar",
    "atisbabas": "atisbar",
    "atisbad": "atisbar",
    "atisbada": "atisbar",
    "atisbadas": "atisbar",
    "atisbado": "atisbar",
    "atisbadora": "atisbador",
    "atisbadoras": "atisbador",
    "atisbadores": "atisbador",
    "atisbados": "atisbar",
    "atisbaduras": "atisbadura",
    "atisbamos": "atisbar",
    "atisban": "atisbar",
    "atisbando": "atisbar",
    "atisbara": "atisbar",
    "atisbarais": "atisbar",
    "atisbaran": "atisbar",
    "atisbaras": "atisbar",
    "atisbare": "atisbar",
    "atisbareis": "atisbar",
    "atisbaremos": "atisbar",
    "atisbaren": "atisbar",
    "atisbares": "atisbar",
    "atisbaron": "atisbar",
    "atisbará": "atisbar",
    "atisbarán": "atisbar",
    "atisbarás": "atisbar",
    "atisbaré": "atisbar",
    "atisbaréis": "atisbar",
    "atisbaría": "atisbar",
    "atisbaríais": "atisbar",
    "atisbaríamos": "atisbar",
    "atisbarían": "atisbar",
    "atisbarías": "atisbar",
    "atisbas": "atisbar",
    "atisbase": "atisbar",
    "atisbaseis": "atisbar",
    "atisbasen": "atisbar",
    "atisbases": "atisbar",
    "atisbaste": "atisbar",
    "atisbasteis": "atisbar",
    "atisbe": "atisbar",
    "atisbemos": "atisbar",
    "atisben": "atisbar",
    "atisbes": "atisbar",
    "atisbo": "atisbar",
    "atisbos": "atisbo",
    "atisbábamos": "atisbar",
    "atisbáis": "atisbar",
    "atisbáramos": "atisbar",
    "atisbáremos": "atisbar",
    "atisbásemos": "atisbar",
    "atisbé": "atisbar",
    "atisbéis": "atisbar",
    "atisbó": "atisbar",
    "atiza": "atizar",
    "atizaba": "atizar",
    "atizabais": "atizar",
    "atizaban": "atizar",
    "atizabas": "atizar",
    "atizad": "atizar",
    "atizada": "atizar",
    "atizadas": "atizar",
    "atizaderos": "atizadero",
    "atizado": "atizar",
    "atizadora": "atizador",
    "atizadoras": "atizador",
    "atizadores": "atizador",
    "atizados": "atizar",
    "atizamientos": "atizamiento",
    "atizamos": "atizar",
    "atizan": "atizar",
    "atizando": "atizar",
    "atizara": "atizar",
    "atizarais": "atizar",
    "atizaran": "atizar",
    "atizaras": "atizar",
    "atizare": "atizar",
    "atizareis": "atizar",
    "atizaremos": "atizar",
    "atizaren": "atizar",
    "atizares": "atizar",
    "atizaron": "atizar",
    "atizará": "atizar",
    "atizarán": "atizar",
    "atizarás": "atizar",
    "atizaré": "atizar",
    "atizaréis": "atizar",
    "atizaría": "atizar",
    "atizaríais": "atizar",
    "atizaríamos": "atizar",
    "atizarían": "atizar",
    "atizarías": "atizar",
    "atizas": "atizar",
    "atizase": "atizar",
    "atizaseis": "atizar",
    "atizasen": "atizar",
    "atizases": "atizar",
    "atizaste": "atizar",
    "atizasteis": "atizar",
    "atizo": "atizar",
    "atizona": "atizonar",
    "atizonaba": "atizonar",
    "atizonabais": "atizonar",
    "atizonaban": "atizonar",
    "atizonabas": "atizonar",
    "atizonad": "atizonar",
    "atizonada": "atizonar",
    "atizonadas": "atizonar",
    "atizonado": "atizonar",
    "atizonados": "atizonar",
    "atizonamos": "atizonar",
    "atizonan": "atizonar",
    "atizonando": "atizonar",
    "atizonara": "atizonar",
    "atizonarais": "atizonar",
    "atizonaran": "atizonar",
    "atizonaras": "atizonar",
    "atizonare": "atizonar",
    "atizonareis": "atizonar",
    "atizonaremos": "atizonar",
    "atizonaren": "atizonar",
    "atizonares": "atizonar",
    "atizonaron": "atizonar",
    "atizonará": "atizonar",
    "atizonarán": "atizonar",
    "atizonarás": "atizonar",
    "atizonaré": "atizonar",
    "atizonaréis": "atizonar",
    "atizonaría": "atizonar",
    "atizonaríais": "atizonar",
    "atizonaríamos": "atizonar",
    "atizonarían": "atizonar",
    "atizonarías": "atizonar",
    "atizonas": "atizonar",
    "atizonase": "atizonar",
    "atizonaseis": "atizonar",
    "atizonasen": "atizonar",
    "atizonases": "atizonar",
    "atizonaste": "atizonar",
    "atizonasteis": "atizonar",
    "atizone": "atizonar",
    "atizonemos": "atizonar",
    "atizonen": "atizonar",
    "atizones": "atizonar",
    "atizono": "atizonar",
    "atizonábamos": "atizonar",
    "atizonáis": "atizonar",
    "atizonáramos": "atizonar",
    "atizonáremos": "atizonar",
    "atizonásemos": "atizonar",
    "atizoné": "atizonar",
    "atizonéis": "atizonar",
    "atizonó": "atizonar",
    "atizábamos": "atizar",
    "atizáis": "atizar",
    "atizáramos": "atizar",
    "atizáremos": "atizar",
    "atizásemos": "atizar",
    "atizó": "atizar",
    "atlantes": "atlante",
    "atlantoaxiales": "atlantoaxial",
    "atlantooccipitales": "atlantooccipital",
    "atletas": "atleta",
    "atletismos": "atletismo",
    "atlántica": "atlántico",
    "atlánticas": "atlántico",
    "atlánticos": "atlántico",
    "atlética": "atlético",
    "atléticas": "atlético",
    "atléticos": "atlético",
    "atm": "atmósfera",
    "atmanes": "atman",
    "atmidómetros": "atmidómetro",
    "atmosférica": "atmosférico",
    "atmosféricas": "atmosférico",
    "atmosféricos": "atmosférico",
    "atmófila": "atmófilo",
    "atmófilas": "atmófilo",
    "atmófilos": "atmófilo",
    "atmósferas": "atmósfera",
    "ato": "atar",
    "atoa": "atoar",
    "atoaba": "atoar",
    "atoabais": "atoar",
    "atoaban": "atoar",
    "atoabas": "atoar",
    "atoad": "atoar",
    "atoada": "atoar",
    "atoadas": "atoar",
    "atoado": "atoar",
    "atoados": "atoar",
    "atoamientos": "atoamiento",
    "atoamos": "atoar",
    "atoan": "atoar",
    "atoando": "atoar",
    "atoara": "atoar",
    "atoarais": "atoar",
    "atoaran": "atoar",
    "atoaras": "atoar",
    "atoare": "atoar",
    "atoareis": "atoar",
    "atoaremos": "atoar",
    "atoaren": "atoar",
    "atoares": "atoar",
    "atoaron": "atoar",
    "atoará": "atoar",
    "atoarán": "atoar",
    "atoarás": "atoar",
    "atoaré": "atoar",
    "atoaréis": "atoar",
    "atoaría": "atoar",
    "atoaríais": "atoar",
    "atoaríamos": "atoar",
    "atoarían": "atoar",
    "atoarías": "atoar",
    "atoas": "atoar",
    "atoase": "atoar",
    "atoaseis": "atoar",
    "atoasen": "atoar",
    "atoases": "atoar",
    "atoaste": "atoar",
    "atoasteis": "atoar",
    "atocha": "atochar",
    "atochaba": "atochar",
    "atochabais": "atochar",
    "atochaban": "atochar",
    "atochabas": "atochar",
    "atochad": "atochar",
    "atochada": "atochar",
    "atochadas": "atochar",
    "atochado": "atochar",
    "atochados": "atochar",
    "atochales": "atochal",
    "atochamos": "atochar",
    "atochan": "atochar",
    "atochando": "atochar",
    "atochara": "atochar",
    "atocharais": "atochar",
    "atocharan": "atochar",
    "atocharas": "atochar",
    "atochare": "atochar",
    "atochareis": "atochar",
    "atocharemos": "atochar",
    "atocharen": "atochar",
    "atochares": "atochar",
    "atocharon": "atochar",
    "atochará": "atochar",
    "atocharán": "atochar",
    "atocharás": "atochar",
    "atocharé": "atochar",
    "atocharéis": "atochar",
    "atocharía": "atochar",
    "atocharíais": "atochar",
    "atocharíamos": "atochar",
    "atocharían": "atochar",
    "atocharías": "atochar",
    "atochas": "atochar",
    "atochase": "atochar",
    "atochaseis": "atochar",
    "atochasen": "atochar",
    "atochases": "atochar",
    "atochaste": "atochar",
    "atochasteis": "atochar",
    "atoche": "atochar",
    "atochemos": "atochar",
    "atochen": "atochar",
    "atoches": "atochar",
    "atocho": "atochar",
    "atochábamos": "atochar",
    "atocháis": "atochar",
    "atocháramos": "atochar",
    "atocháremos": "atochar",
    "atochásemos": "atochar",
    "atoché": "atochar",
    "atochéis": "atochar",
    "atochó": "atochar",
    "atocina": "atocinar",
    "atocinaba": "atocinar",
    "atocinabais": "atocinar",
    "atocinaban": "atocinar",
    "atocinabas": "atocinar",
    "atocinad": "atocinar",
    "atocinada": "atocinar",
    "atocinadas": "atocinar",
    "atocinado": "atocinar",
    "atocinados": "atocinar",
    "atocinamientos": "atocinamiento",
    "atocinamos": "atocinar",
    "atocinan": "atocinar",
    "atocinando": "atocinar",
    "atocinara": "atocinar",
    "atocinarais": "atocinar",
    "atocinaran": "atocinar",
    "atocinaras": "atocinar",
    "atocinare": "atocinar",
    "atocinareis": "atocinar",
    "atocinaremos": "atocinar",
    "atocinaren": "atocinar",
    "atocinares": "atocinar",
    "atocinaron": "atocinar",
    "atocinará": "atocinar",
    "atocinarán": "atocinar",
    "atocinarás": "atocinar",
    "atocinaré": "atocinar",
    "atocinaréis": "atocinar",
    "atocinaría": "atocinar",
    "atocinaríais": "atocinar",
    "atocinaríamos": "atocinar",
    "atocinarían": "atocinar",
    "atocinarías": "atocinar",
    "atocinas": "atocinar",
    "atocinase": "atocinar",
    "atocinaseis": "atocinar",
    "atocinasen": "atocinar",
    "atocinases": "atocinar",
    "atocinaste": "atocinar",
    "atocinasteis": "atocinar",
    "atocine": "atocinar",
    "atocinemos": "atocinar",
    "atocinen": "atocinar",
    "atocines": "atocinar",
    "atocino": "atocinar",
    "atocinábamos": "atocinar",
    "atocináis": "atocinar",
    "atocináramos": "atocinar",
    "atocináremos": "atocinar",
    "atocinásemos": "atocinar",
    "atociné": "atocinar",
    "atocinéis": "atocinar",
    "atocinó": "atocinar",
    "atoe": "atoar",
    "atoemos": "atoar",
    "atoen": "atoar",
    "atoes": "atoar",
    "atoles": "atole",
    "atolla": "atollar",
    "atollaba": "atollar",
    "atollabais": "atollar",
    "atollaban": "atollar",
    "atollabas": "atollar",
    "atollad": "atollar",
    "atollada": "atollar",
    "atolladas": "atollar",
    "atolladeros": "atolladero",
    "atollado": "atollar",
    "atollados": "atollar",
    "atollamos": "atollar",
    "atollan": "atollar",
    "atollando": "atollar",
    "atollara": "atollar",
    "atollarais": "atollar",
    "atollaran": "atollar",
    "atollaras": "atollar",
    "atollare": "atollar",
    "atollareis": "atollar",
    "atollaremos": "atollar",
    "atollaren": "atollar",
    "atollares": "atollar",
    "atollaron": "atollar",
    "atollará": "atollar",
    "atollarán": "atollar",
    "atollarás": "atollar",
    "atollaré": "atollar",
    "atollaréis": "atollar",
    "atollaría": "atollar",
    "atollaríais": "atollar",
    "atollaríamos": "atollar",
    "atollarían": "atollar",
    "atollarías": "atollar",
    "atollas": "atollar",
    "atollase": "atollar",
    "atollaseis": "atollar",
    "atollasen": "atollar",
    "atollases": "atollar",
    "atollaste": "atollar",
    "atollasteis": "atollar",
    "atolle": "atollar",
    "atollemos": "atollar",
    "atollen": "atollar",
    "atolles": "atollar",
    "atollo": "atollar",
    "atollábamos": "atollar",
    "atolláis": "atollar",
    "atolláramos": "atollar",
    "atolláremos": "atollar",
    "atollásemos": "atollar",
    "atollé": "atollar",
    "atolléis": "atollar",
    "atolló": "atollar",
    "atolondra": "atolondrar",
    "atolondraba": "atolondrar",
    "atolondrabais": "atolondrar",
    "atolondraban": "atolondrar",
    "atolondrabas": "atolondrar",
    "atolondrad": "atolondrar",
    "atolondrada": "atolondrar",
    "atolondradas": "atolondrar",
    "atolondrado": "atolondrar",
    "atolondrados": "atolondrar",
    "atolondramientos": "atolondramiento",
    "atolondramos": "atolondrar",
    "atolondran": "atolondrar",
    "atolondrando": "atolondrar",
    "atolondrara": "atolondrar",
    "atolondrarais": "atolondrar",
    "atolondraran": "atolondrar",
    "atolondraras": "atolondrar",
    "atolondrare": "atolondrar",
    "atolondrareis": "atolondrar",
    "atolondraremos": "atolondrar",
    "atolondraren": "atolondrar",
    "atolondrares": "atolondrar",
    "atolondraron": "atolondrar",
    "atolondrará": "atolondrar",
    "atolondrarán": "atolondrar",
    "atolondrarás": "atolondrar",
    "atolondraré": "atolondrar",
    "atolondraréis": "atolondrar",
    "atolondraría": "atolondrar",
    "atolondraríais": "atolondrar",
    "atolondraríamos": "atolondrar",
    "atolondrarían": "atolondrar",
    "atolondrarías": "atolondrar",
    "atolondras": "atolondrar",
    "atolondrase": "atolondrar",
    "atolondraseis": "atolondrar",
    "atolondrasen": "atolondrar",
    "atolondrases": "atolondrar",
    "atolondraste": "atolondrar",
    "atolondrasteis": "atolondrar",
    "atolondre": "atolondrar",
    "atolondremos": "atolondrar",
    "atolondren": "atolondrar",
    "atolondres": "atolondrar",
    "atolondro": "atolondrar",
    "atolondrábamos": "atolondrar",
    "atolondráis": "atolondrar",
    "atolondráramos": "atolondrar",
    "atolondráremos": "atolondrar",
    "atolondrásemos": "atolondrar",
    "atolondré": "atolondrar",
    "atolondréis": "atolondrar",
    "atolondró": "atolondrar",
    "atolones": "atolón",
    "atomice": "atomizar",
    "atomicemos": "atomizar",
    "atomicen": "atomizar",
    "atomices": "atomizar",
    "atomicidades": "atomicidad",
    "atomicé": "atomizar",
    "atomicéis": "atomizar",
    "atomismos": "atomismo",
    "atomistas": "atomista",
    "atomiza": "atomizar",
    "atomizaba": "atomizar",
    "atomizabais": "atomizar",
    "atomizaban": "atomizar",
    "atomizabas": "atomizar",
    "atomizaciones": "atomización",
    "atomizad": "atomizar",
    "atomizada": "atomizar",
    "atomizadas": "atomizar",
    "atomizado": "atomizar",
    "atomizadores": "atomizador",
    "atomizados": "atomizar",
    "atomizamos": "atomizar",
    "atomizan": "atomizar",
    "atomizando": "atomizar",
    "atomizara": "atomizar",
    "atomizarais": "atomizar",
    "atomizaran": "atomizar",
    "atomizaras": "atomizar",
    "atomizare": "atomizar",
    "atomizareis": "atomizar",
    "atomizaremos": "atomizar",
    "atomizaren": "atomizar",
    "atomizares": "atomizar",
    "atomizaron": "atomizar",
    "atomizará": "atomizar",
    "atomizarán": "atomizar",
    "atomizarás": "atomizar",
    "atomizaré": "atomizar",
    "atomizaréis": "atomizar",
    "atomizaría": "atomizar",
    "atomizaríais": "atomizar",
    "atomizaríamos": "atomizar",
    "atomizarían": "atomizar",
    "atomizarías": "atomizar",
    "atomizas": "atomizar",
    "atomizase": "atomizar",
    "atomizaseis": "atomizar",
    "atomizasen": "atomizar",
    "atomizases": "atomizar",
    "atomizaste": "atomizar",
    "atomizasteis": "atomizar",
    "atomizo": "atomizar",
    "atomizábamos": "atomizar",
    "atomizáis": "atomizar",
    "atomizáramos": "atomizar",
    "atomizáremos": "atomizar",
    "atomizásemos": "atomizar",
    "atomizó": "atomizar",
    "atomística": "atomístico",
    "atomísticas": "atomístico",
    "atomísticos": "atomístico",
    "atonales": "atonal",
    "atonalidades": "atonalidad",
    "atonda": "atondar",
    "atondaba": "atondar",
    "atondabais": "atondar",
    "atondaban": "atondar",
    "atondabas": "atondar",
    "atondad": "atondar",
    "atondada": "atondar",
    "atondadas": "atondar",
    "atondado": "atondar",
    "atondados": "atondar",
    "atondamos": "atondar",
    "atondan": "atondar",
    "atondando": "atondar",
    "atondara": "atondar",
    "atondarais": "atondar",
    "atondaran": "atondar",
    "atondaras": "atondar",
    "atondare": "atondar",
    "atondareis": "atondar",
    "atondaremos": "atondar",
    "atondaren": "atondar",
    "atondares": "atondar",
    "atondaron": "atondar",
    "atondará": "atondar",
    "atondarán": "atondar",
    "atondarás": "atondar",
    "atondaré": "atondar",
    "atondaréis": "atondar",
    "atondaría": "atondar",
    "atondaríais": "atondar",
    "atondaríamos": "atondar",
    "atondarían": "atondar",
    "atondarías": "atondar",
    "atondas": "atondar",
    "atondase": "atondar",
    "atondaseis": "atondar",
    "atondasen": "atondar",
    "atondases": "atondar",
    "atondaste": "atondar",
    "atondasteis": "atondar",
    "atonde": "atondar",
    "atondemos": "atondar",
    "atonden": "atondar",
    "atondes": "atondar",
    "atondo": "atondar",
    "atondábamos": "atondar",
    "atondáis": "atondar",
    "atondáramos": "atondar",
    "atondáremos": "atondar",
    "atondásemos": "atondar",
    "atondé": "atondar",
    "atondéis": "atondar",
    "atondó": "atondar",
    "atonicidades": "atonicidad",
    "atonta": "atontar",
    "atontaba": "atontar",
    "atontabais": "atontar",
    "atontaban": "atontar",
    "atontabas": "atontar",
    "atontad": "atontar",
    "atontada": "atontar",
    "atontadas": "atontar",
    "atontado": "atontar",
    "atontados": "atontar",
    "atontamientos": "atontamiento",
    "atontamos": "atontar",
    "atontan": "atontar",
    "atontando": "atontar",
    "atontara": "atontar",
    "atontarais": "atontar",
    "atontaran": "atontar",
    "atontaras": "atontar",
    "atontare": "atontar",
    "atontareis": "atontar",
    "atontaremos": "atontar",
    "atontaren": "atontar",
    "atontares": "atontar",
    "atontaron": "atontar",
    "atontará": "atontar",
    "atontarán": "atontar",
    "atontarás": "atontar",
    "atontaré": "atontar",
    "atontaréis": "atontar",
    "atontaría": "atontar",
    "atontaríais": "atontar",
    "atontaríamos": "atontar",
    "atontarían": "atontar",
    "atontarías": "atontar",
    "atontas": "atontar",
    "atontase": "atontar",
    "atontaseis": "atontar",
    "atontasen": "atontar",
    "atontases": "atontar",
    "atontaste": "atontar",
    "atontasteis": "atontar",
    "atonte": "atontar",
    "atontemos": "atontar",
    "atonten": "atontar",
    "atontes": "atontar",
    "atonto": "atontar",
    "atontábamos": "atontar",
    "atontáis": "atontar",
    "atontáramos": "atontar",
    "atontáremos": "atontar",
    "atontásemos": "atontar",
    "atonté": "atontar",
    "atontéis": "atontar",
    "atontó": "atontar",
    "atonías": "atonía",
    "atoo": "atoar",
    "atopias": "atopia",
    "atopías": "atopía",
    "atora": "atorar",
    "atoraba": "atorar",
    "atorabais": "atorar",
    "atoraban": "atorar",
    "atorabas": "atorar",
    "atorad": "atorar",
    "atorada": "atorar",
    "atoradas": "atorar",
    "atorado": "atorar",
    "atorados": "atorar",
    "atoramientos": "atoramiento",
    "atoramos": "atorar",
    "atoran": "atorar",
    "atorando": "atorar",
    "atorara": "atorar",
    "atorarais": "atorar",
    "atoraran": "atorar",
    "atoraras": "atorar",
    "atorare": "atorar",
    "atorareis": "atorar",
    "atoraremos": "atorar",
    "atoraren": "atorar",
    "atorares": "atorar",
    "atoraron": "atorar",
    "atorará": "atorar",
    "atorarán": "atorar",
    "atorarás": "atorar",
    "atoraré": "atorar",
    "atoraréis": "atorar",
    "atoraría": "atorar",
    "atoraríais": "atorar",
    "atoraríamos": "atorar",
    "atorarían": "atorar",
    "atorarías": "atorar",
    "atoras": "atorar",
    "atorase": "atorar",
    "atoraseis": "atorar",
    "atorasen": "atorar",
    "atorases": "atorar",
    "atoraste": "atorar",
    "atorasteis": "atorar",
    "atore": "atorar",
    "atoremos": "atorar",
    "atoren": "atorar",
    "atores": "atorar",
    "atormenta": "atormentar",
    "atormentaba": "atormentar",
    "atormentabais": "atormentar",
    "atormentaban": "atormentar",
    "atormentabas": "atormentar",
    "atormentad": "atormentar",
    "atormentada": "atormentar",
    "atormentadas": "atormentar",
    "atormentado": "atormentar",
    "atormentadora": "atormentador",
    "atormentadoras": "atormentador",
    "atormentadores": "atormentador",
    "atormentados": "atormentar",
    "atormentamos": "atormentar",
    "atormentan": "atormentar",
    "atormentando": "atormentar",
    "atormentara": "atormentar",
    "atormentarais": "atormentar",
    "atormentaran": "atormentar",
    "atormentaras": "atormentar",
    "atormentare": "atormentar",
    "atormentareis": "atormentar",
    "atormentaremos": "atormentar",
    "atormentaren": "atormentar",
    "atormentares": "atormentar",
    "atormentaron": "atormentar",
    "atormentará": "atormentar",
    "atormentarán": "atormentar",
    "atormentarás": "atormentar",
    "atormentaré": "atormentar",
    "atormentaréis": "atormentar",
    "atormentaría": "atormentar",
    "atormentaríais": "atormentar",
    "atormentaríamos": "atormentar",
    "atormentarían": "atormentar",
    "atormentarías": "atormentar",
    "atormentas": "atormentar",
    "atormentase": "atormentar",
    "atormentaseis": "atormentar",
    "atormentasen": "atormentar",
    "atormentases": "atormentar",
    "atormentaste": "atormentar",
    "atormentasteis": "atormentar",
    "atormente": "atormentar",
    "atormentemos": "atormentar",
    "atormenten": "atormentar",
    "atormentes": "atormentar",
    "atormento": "atormentar",
    "atormentábamos": "atormentar",
    "atormentáis": "atormentar",
    "atormentáramos": "atormentar",
    "atormentáremos": "atormentar",
    "atormentásemos": "atormentar",
    "atormenté": "atormentar",
    "atormentéis": "atormentar",
    "atormentó": "atormentar",
    "atornilla": "atornillar",
    "atornillaba": "atornillar",
    "atornillabais": "atornillar",
    "atornillaban": "atornillar",
    "atornillabas": "atornillar",
    "atornillad": "atornillar",
    "atornillada": "atornillar",
    "atornilladas": "atornillar",
    "atornillado": "atornillar",
    "atornilladores": "atornillador",
    "atornillados": "atornillar",
    "atornillamientos": "atornillamiento",
    "atornillamos": "atornillar",
    "atornillan": "atornillar",
    "atornillando": "atornillar",
    "atornillara": "atornillar",
    "atornillarais": "atornillar",
    "atornillaran": "atornillar",
    "atornillaras": "atornillar",
    "atornillare": "atornillar",
    "atornillareis": "atornillar",
    "atornillaremos": "atornillar",
    "atornillaren": "atornillar",
    "atornillares": "atornillar",
    "atornillaron": "atornillar",
    "atornillará": "atornillar",
    "atornillarán": "atornillar",
    "atornillarás": "atornillar",
    "atornillaré": "atornillar",
    "atornillaréis": "atornillar",
    "atornillaría": "atornillar",
    "atornillaríais": "atornillar",
    "atornillaríamos": "atornillar",
    "atornillarían": "atornillar",
    "atornillarías": "atornillar",
    "atornillas": "atornillar",
    "atornillase": "atornillar",
    "atornillaseis": "atornillar",
    "atornillasen": "atornillar",
    "atornillases": "atornillar",
    "atornillaste": "atornillar",
    "atornillasteis": "atornillar",
    "atornille": "atornillar",
    "atornillemos": "atornillar",
    "atornillen": "atornillar",
    "atornilles": "atornillar",
    "atornillo": "atornillar",
    "atornillábamos": "atornillar",
    "atornilláis": "atornillar",
    "atornilláramos": "atornillar",
    "atornilláremos": "atornillar",
    "atornillásemos": "atornillar",
    "atornillé": "atornillar",
    "atornilléis": "atornillar",
    "atornilló": "atornillar",
    "atoro": "atorar",
    "atortola": "atortolar",
    "atortolaba": "atortolar",
    "atortolabais": "atortolar",
    "atortolaban": "atortolar",
    "atortolabas": "atortolar",
    "atortolad": "atortolar",
    "atortolada": "atortolar",
    "atortoladas": "atortolar",
    "atortolado": "atortolar",
    "atortolados": "atortolar",
    "atortolamos": "atortolar",
    "atortolan": "atortolar",
    "atortolando": "atortolar",
    "atortolara": "atortolar",
    "atortolarais": "atortolar",
    "atortolaran": "atortolar",
    "atortolaras": "atortolar",
    "atortolare": "atortolar",
    "atortolareis": "atortolar",
    "atortolaremos": "atortolar",
    "atortolaren": "atortolar",
    "atortolares": "atortolar",
    "atortolaron": "atortolar",
    "atortolará": "atortolar",
    "atortolarán": "atortolar",
    "atortolarás": "atortolar",
    "atortolaré": "atortolar",
    "atortolaréis": "atortolar",
    "atortolaría": "atortolar",
    "atortolaríais": "atortolar",
    "atortolaríamos": "atortolar",
    "atortolarían": "atortolar",
    "atortolarías": "atortolar",
    "atortolas": "atortolar",
    "atortolase": "atortolar",
    "atortolaseis": "atortolar",
    "atortolasen": "atortolar",
    "atortolases": "atortolar",
    "atortolaste": "atortolar",
    "atortolasteis": "atortolar",
    "atortole": "atortolar",
    "atortolemos": "atortolar",
    "atortolen": "atortolar",
    "atortoles": "atortolar",
    "atortolo": "atortolar",
    "atortolábamos": "atortolar",
    "atortoláis": "atortolar",
    "atortoláramos": "atortolar",
    "atortoláremos": "atortolar",
    "atortolásemos": "atortolar",
    "atortolé": "atortolar",
    "atortoléis": "atortolar",
    "atortoló": "atortolar",
    "atortora": "atortorar",
    "atortoraba": "atortorar",
    "atortorabais": "atortorar",
    "atortoraban": "atortorar",
    "atortorabas": "atortorar",
    "atortorad": "atortorar",
    "atortorada": "atortorar",
    "atortoradas": "atortorar",
    "atortorado": "atortorar",
    "atortorados": "atortorar",
    "atortoramos": "atortorar",
    "atortoran": "atortorar",
    "atortorando": "atortorar",
    "atortorara": "atortorar",
    "atortorarais": "atortorar",
    "atortoraran": "atortorar",
    "atortoraras": "atortorar",
    "atortorare": "atortorar",
    "atortorareis": "atortorar",
    "atortoraremos": "atortorar",
    "atortoraren": "atortorar",
    "atortorares": "atortorar",
    "atortoraron": "atortorar",
    "atortorará": "atortorar",
    "atortorarán": "atortorar",
    "atortorarás": "atortorar",
    "atortoraré": "atortorar",
    "atortoraréis": "atortorar",
    "atortoraría": "atortorar",
    "atortoraríais": "atortorar",
    "atortoraríamos": "atortorar",
    "atortorarían": "atortorar",
    "atortorarías": "atortorar",
    "atortoras": "atortorar",
    "atortorase": "atortorar",
    "atortoraseis": "atortorar",
    "atortorasen": "atortorar",
    "atortorases": "atortorar",
    "atortoraste": "atortorar",
    "atortorasteis": "atortorar",
    "atortore": "atortorar",
    "atortoremos": "atortorar",
    "atortoren": "atortorar",
    "atortores": "atortorar",
    "atortoro": "atortorar",
    "atortorábamos": "atortorar",
    "atortoráis": "atortorar",
    "atortoráramos": "atortorar",
    "atortoráremos": "atortorar",
    "atortorásemos": "atortorar",
    "atortoré": "atortorar",
    "atortoréis": "atortorar",
    "atortoró": "atortorar",
    "atorábamos": "atorar",
    "atoráis": "atorar",
    "atoráramos": "atorar",
    "atoráremos": "atorar",
    "atorásemos": "atorar",
    "atoré": "atorar",
    "atoréis": "atorar",
    "atoró": "atorar",
    "atosiga": "atosigar",
    "atosigaba": "atosigar",
    "atosigabais": "atosigar",
    "atosigaban": "atosigar",
    "atosigabas": "atosigar",
    "atosigad": "atosigar",
    "atosigada": "atosigar",
    "atosigadas": "atosigar",
    "atosigado": "atosigar",
    "atosigados": "atosigar",
    "atosigamientos": "atosigamiento",
    "atosigamos": "atosigar",
    "atosigan": "atosigar",
    "atosigando": "atosigar",
    "atosigara": "atosigar",
    "atosigarais": "atosigar",
    "atosigaran": "atosigar",
    "atosigaras": "atosigar",
    "atosigare": "atosigar",
    "atosigareis": "atosigar",
    "atosigaremos": "atosigar",
    "atosigaren": "atosigar",
    "atosigares": "atosigar",
    "atosigaron": "atosigar",
    "atosigará": "atosigar",
    "atosigarán": "atosigar",
    "atosigarás": "atosigar",
    "atosigaré": "atosigar",
    "atosigaréis": "atosigar",
    "atosigaría": "atosigar",
    "atosigaríais": "atosigar",
    "atosigaríamos": "atosigar",
    "atosigarían": "atosigar",
    "atosigarías": "atosigar",
    "atosigas": "atosigar",
    "atosigase": "atosigar",
    "atosigaseis": "atosigar",
    "atosigasen": "atosigar",
    "atosigases": "atosigar",
    "atosigaste": "atosigar",
    "atosigasteis": "atosigar",
    "atosigo": "atosigar",
    "atosigue": "atosigar",
    "atosiguemos": "atosigar",
    "atosiguen": "atosigar",
    "atosigues": "atosigar",
    "atosigué": "atosigar",
    "atosiguéis": "atosigar",
    "atosigábamos": "atosigar",
    "atosigáis": "atosigar",
    "atosigáramos": "atosigar",
    "atosigáremos": "atosigar",
    "atosigásemos": "atosigar",
    "atosigó": "atosigar",
    "atoábamos": "atoar",
    "atoáis": "atoar",
    "atoáramos": "atoar",
    "atoáremos": "atoar",
    "atoásemos": "atoar",
    "atoé": "atoar",
    "atoéis": "atoar",
    "atoó": "atoar",
    "atrabanca": "atrabancar",
    "atrabancaba": "atrabancar",
    "atrabancabais": "atrabancar",
    "atrabancaban": "atrabancar",
    "atrabancabas": "atrabancar",
    "atrabancad": "atrabancar",
    "atrabancada": "atrabancar",
    "atrabancadas": "atrabancar",
    "atrabancado": "atrabancar",
    "atrabancados": "atrabancar",
    "atrabancamos": "atrabancar",
    "atrabancan": "atrabancar",
    "atrabancando": "atrabancar",
    "atrabancara": "atrabancar",
    "atrabancarais": "atrabancar",
    "atrabancaran": "atrabancar",
    "atrabancaras": "atrabancar",
    "atrabancare": "atrabancar",
    "atrabancareis": "atrabancar",
    "atrabancaremos": "atrabancar",
    "atrabancaren": "atrabancar",
    "atrabancares": "atrabancar",
    "atrabancaron": "atrabancar",
    "atrabancará": "atrabancar",
    "atrabancarán": "atrabancar",
    "atrabancarás": "atrabancar",
    "atrabancaré": "atrabancar",
    "atrabancaréis": "atrabancar",
    "atrabancaría": "atrabancar",
    "atrabancaríais": "atrabancar",
    "atrabancaríamos": "atrabancar",
    "atrabancarían": "atrabancar",
    "atrabancarías": "atrabancar",
    "atrabancas": "atrabancar",
    "atrabancase": "atrabancar",
    "atrabancaseis": "atrabancar",
    "atrabancasen": "atrabancar",
    "atrabancases": "atrabancar",
    "atrabancaste": "atrabancar",
    "atrabancasteis": "atrabancar",
    "atrabanco": "atrabancar",
    "atrabancos": "atrabanco",
    "atrabancábamos": "atrabancar",
    "atrabancáis": "atrabancar",
    "atrabancáramos": "atrabancar",
    "atrabancáremos": "atrabancar",
    "atrabancásemos": "atrabancar",
    "atrabancó": "atrabancar",
    "atrabanque": "atrabancar",
    "atrabanquemos": "atrabancar",
    "atrabanquen": "atrabancar",
    "atrabanques": "atrabancar",
    "atrabanqué": "atrabancar",
    "atrabanquéis": "atrabancar",
    "atrabiliaria": "atrabiliario",
    "atrabiliarias": "atrabiliario",
    "atrabiliarios": "atrabiliario",
    "atraca": "atracar",
    "atracaba": "atracar",
    "atracabais": "atracar",
    "atracaban": "atracar",
    "atracabas": "atracar",
    "atracad": "atracar",
    "atracada": "atracar",
    "atracadas": "atracar",
    "atracaderos": "atracadero",
    "atracado": "atracar",
    "atracadora": "atracador",
    "atracadoras": "atracador",
    "atracadores": "atracador",
    "atracados": "atracar",
    "atracamos": "atracar",
    "atracan": "atracar",
    "atracando": "atracar",
    "atracara": "atracar",
    "atracarais": "atracar",
    "atracaran": "atracar",
    "atracaras": "atracar",
    "atracare": "atracar",
    "atracareis": "atracar",
    "atracaremos": "atracar",
    "atracaren": "atracar",
    "atracares": "atracar",
    "atracaron": "atracar",
    "atracará": "atracar",
    "atracarán": "atracar",
    "atracarás": "atracar",
    "atracaré": "atracar",
    "atracaréis": "atracar",
    "atracaría": "atracar",
    "atracaríais": "atracar",
    "atracaríamos": "atracar",
    "atracarían": "atracar",
    "atracarías": "atracar",
    "atracas": "atracar",
    "atracase": "atracar",
    "atracaseis": "atracar",
    "atracasen": "atracar",
    "atracases": "atracar",
    "atracaste": "atracar",
    "atracasteis": "atracar",
    "atracciones": "atracción",
    "atraco": "atracar",
    "atracones": "atracón",
    "atracos": "atraco",
    "atractiva": "atractivo",
    "atractivas": "atractivo",
    "atractividades": "atractividad",
    "atractivos": "atractivo",
    "atracábamos": "atracar",
    "atracáis": "atracar",
    "atracáramos": "atracar",
    "atracáremos": "atracar",
    "atracásemos": "atracar",
    "atracó": "atracar",
    "atrae": "atraer",
    "atraed": "atraer",
    "atraemos": "atraer",
    "atraen": "atraer",
    "atraeremos": "atraer",
    "atraerá": "atraer",
    "atraerán": "atraer",
    "atraerás": "atraer",
    "atraeré": "atraer",
    "atraeréis": "atraer",
    "atraería": "atraer",
    "atraeríais": "atraer",
    "atraeríamos": "atraer",
    "atraerían": "atraer",
    "atraerías": "atraer",
    "atraes": "atraer",
    "atrafaga": "atrafagar",
    "atrafagaba": "atrafagar",
    "atrafagabais": "atrafagar",
    "atrafagaban": "atrafagar",
    "atrafagabas": "atrafagar",
    "atrafagad": "atrafagar",
    "atrafagada": "atrafagar",
    "atrafagadas": "atrafagar",
    "atrafagado": "atrafagar",
    "atrafagados": "atrafagar",
    "atrafagamos": "atrafagar",
    "atrafagan": "atrafagar",
    "atrafagando": "atrafagar",
    "atrafagara": "atrafagar",
    "atrafagarais": "atrafagar",
    "atrafagaran": "atrafagar",
    "atrafagaras": "atrafagar",
    "atrafagare": "atrafagar",
    "atrafagareis": "atrafagar",
    "atrafagaremos": "atrafagar",
    "atrafagaren": "atrafagar",
    "atrafagares": "atrafagar",
    "atrafagaron": "atrafagar",
    "atrafagará": "atrafagar",
    "atrafagarán": "atrafagar",
    "atrafagarás": "atrafagar",
    "atrafagaré": "atrafagar",
    "atrafagaréis": "atrafagar",
    "atrafagaría": "atrafagar",
    "atrafagaríais": "atrafagar",
    "atrafagaríamos": "atrafagar",
    "atrafagarían": "atrafagar",
    "atrafagarías": "atrafagar",
    "atrafagas": "atrafagar",
    "atrafagase": "atrafagar",
    "atrafagaseis": "atrafagar",
    "atrafagasen": "atrafagar",
    "atrafagases": "atrafagar",
    "atrafagaste": "atrafagar",
    "atrafagasteis": "atrafagar",
    "atrafago": "atrafagar",
    "atrafague": "atrafagar",
    "atrafaguemos": "atrafagar",
    "atrafaguen": "atrafagar",
    "atrafagues": "atrafagar",
    "atrafagué": "atrafagar",
    "atrafaguéis": "atrafagar",
    "atrafagábamos": "atrafagar",
    "atrafagáis": "atrafagar",
    "atrafagáramos": "atrafagar",
    "atrafagáremos": "atrafagar",
    "atrafagásemos": "atrafagar",
    "atrafagó": "atrafagar",
    "atraganta": "atragantar",
    "atragantaba": "atragantar",
    "atragantabais": "atragantar",
    "atragantaban": "atragantar",
    "atragantabas": "atragantar",
    "atragantad": "atragantar",
    "atragantada": "atragantar",
    "atragantadas": "atragantar",
    "atragantado": "atragantar",
    "atragantados": "atragantar",
    "atragantamientos": "atragantamiento",
    "atragantamos": "atragantar",
    "atragantan": "atragantar",
    "atragantando": "atragantar",
    "atragantara": "atragantar",
    "atragantarais": "atragantar",
    "atragantaran": "atragantar",
    "atragantaras": "atragantar",
    "atragantare": "atragantar",
    "atragantareis": "atragantar",
    "atragantaremos": "atragantar",
    "atragantaren": "atragantar",
    "atragantares": "atragantar",
    "atragantaron": "atragantar",
    "atragantará": "atragantar",
    "atragantarán": "atragantar",
    "atragantarás": "atragantar",
    "atragantaré": "atragantar",
    "atragantaréis": "atragantar",
    "atragantaría": "atragantar",
    "atragantaríais": "atragantar",
    "atragantaríamos": "atragantar",
    "atragantarían": "atragantar",
    "atragantarías": "atragantar",
    "atragantas": "atragantar",
    "atragantase": "atragantar",
    "atragantaseis": "atragantar",
    "atragantasen": "atragantar",
    "atragantases": "atragantar",
    "atragantaste": "atragantar",
    "atragantasteis": "atragantar",
    "atragante": "atragantar",
    "atragantemos": "atragantar",
    "atraganten": "atragantar",
    "atragantes": "atragantar",
    "atraganto": "atragantar",
    "atragantábamos": "atragantar",
    "atragantáis": "atragantar",
    "atragantáramos": "atragantar",
    "atragantáremos": "atragantar",
    "atragantásemos": "atragantar",
    "atraganté": "atragantar",
    "atragantéis": "atragantar",
    "atragantó": "atragantar",
    "atraiga": "atraer",
    "atraigamos": "atraer",
    "atraigan": "atraer",
    "atraigas": "atraer",
    "atraigo": "atraer",
    "atraigáis": "atraer",
    "atraillaba": "atraillar",
    "atraillabais": "atraillar",
    "atraillaban": "atraillar",
    "atraillabas": "atraillar",
    "atraillad": "atraillar",
    "atraillada": "atraillar",
    "atrailladas": "atraillar",
    "atraillado": "atraillar",
    "atraillados": "atraillar",
    "atraillamos": "atraillar",
    "atraillando": "atraillar",
    "atraillara": "atraillar",
    "atraillarais": "atraillar",
    "atraillaran": "atraillar",
    "atraillaras": "atraillar",
    "atraillare": "atraillar",
    "atraillareis": "atraillar",
    "atraillaremos": "atraillar",
    "atraillaren": "atraillar",
    "atraillares": "atraillar",
    "atraillaron": "atraillar",
    "atraillará": "atraillar",
    "atraillarán": "atraillar",
    "atraillarás": "atraillar",
    "atraillaré": "atraillar",
    "atraillaréis": "atraillar",
    "atraillaría": "atraillar",
    "atraillaríais": "atraillar",
    "atraillaríamos": "atraillar",
    "atraillarían": "atraillar",
    "atraillarías": "atraillar",
    "atraillase": "atraillar",
    "atraillaseis": "atraillar",
    "atraillasen": "atraillar",
    "atraillases": "atraillar",
    "atraillaste": "atraillar",
    "atraillasteis": "atraillar",
    "atraillemos": "atraillar",
    "atraillábamos": "atraillar",
    "atrailláis": "atraillar",
    "atrailláramos": "atraillar",
    "atrailláremos": "atraillar",
    "atraillásemos": "atraillar",
    "atraillé": "atraillar",
    "atrailléis": "atraillar",
    "atrailló": "atraillar",
    "atraje": "atraer",
    "atrajera": "atraer",
    "atrajerais": "atraer",
    "atrajeran": "atraer",
    "atrajeras": "atraer",
    "atrajere": "atraer",
    "atrajereis": "atraer",
    "atrajeren": "atraer",
    "atrajeres": "atraer",
    "atrajeron": "atraer",
    "atrajese": "atraer",
    "atrajeseis": "atraer",
    "atrajesen": "atraer",
    "atrajeses": "atraer",
    "atrajimos": "atraer",
    "atrajiste": "atraer",
    "atrajisteis": "atraer",
    "atrajo": "atraer",
    "atrajéramos": "atraer",
    "atrajéremos": "atraer",
    "atrajésemos": "atraer",
    "atrampa": "atrampar",
    "atrampaba": "atrampar",
    "atrampabais": "atrampar",
    "atrampaban": "atrampar",
    "atrampabas": "atrampar",
    "atrampad": "atrampar",
    "atrampada": "atrampar",
    "atrampadas": "atrampar",
    "atrampado": "atrampar",
    "atrampados": "atrampar",
    "atrampamos": "atrampar",
    "atrampan": "atrampar",
    "atrampando": "atrampar",
    "atrampara": "atrampar",
    "atramparais": "atrampar",
    "atramparan": "atrampar",
    "atramparas": "atrampar",
    "atrampare": "atrampar",
    "atrampareis": "atrampar",
    "atramparemos": "atrampar",
    "atramparen": "atrampar",
    "atrampares": "atrampar",
    "atramparon": "atrampar",
    "atrampará": "atrampar",
    "atramparán": "atrampar",
    "atramparás": "atrampar",
    "atramparé": "atrampar",
    "atramparéis": "atrampar",
    "atramparía": "atrampar",
    "atramparíais": "atrampar",
    "atramparíamos": "atrampar",
    "atramparían": "atrampar",
    "atramparías": "atrampar",
    "atrampas": "atrampar",
    "atrampase": "atrampar",
    "atrampaseis": "atrampar",
    "atrampasen": "atrampar",
    "atrampases": "atrampar",
    "atrampaste": "atrampar",
    "atrampasteis": "atrampar",
    "atrampe": "atrampar",
    "atrampemos": "atrampar",
    "atrampen": "atrampar",
    "atrampes": "atrampar",
    "atrampo": "atrampar",
    "atrampábamos": "atrampar",
    "atrampáis": "atrampar",
    "atrampáramos": "atrampar",
    "atrampáremos": "atrampar",
    "atrampásemos": "atrampar",
    "atrampé": "atrampar",
    "atrampéis": "atrampar",
    "atrampó": "atrampar",
    "atranca": "atrancar",
    "atrancaba": "atrancar",
    "atrancabais": "atrancar",
    "atrancaban": "atrancar",
    "atrancabas": "atrancar",
    "atrancad": "atrancar",
    "atrancada": "atrancar",
    "atrancadas": "atrancar",
    "atrancado": "atrancar",
    "atrancados": "atrancar",
    "atrancamientos": "atrancamiento",
    "atrancamos": "atrancar",
    "atrancan": "atrancar",
    "atrancando": "atrancar",
    "atrancara": "atrancar",
    "atrancarais": "atrancar",
    "atrancaran": "atrancar",
    "atrancaras": "atrancar",
    "atrancare": "atrancar",
    "atrancareis": "atrancar",
    "atrancaremos": "atrancar",
    "atrancaren": "atrancar",
    "atrancares": "atrancar",
    "atrancaron": "atrancar",
    "atrancará": "atrancar",
    "atrancarán": "atrancar",
    "atrancarás": "atrancar",
    "atrancaré": "atrancar",
    "atrancaréis": "atrancar",
    "atrancaría": "atrancar",
    "atrancaríais": "atrancar",
    "atrancaríamos": "atrancar",
    "atrancarían": "atrancar",
    "atrancarías": "atrancar",
    "atrancas": "atrancar",
    "atrancase": "atrancar",
    "atrancaseis": "atrancar",
    "atrancasen": "atrancar",
    "atrancases": "atrancar",
    "atrancaste": "atrancar",
    "atrancasteis": "atrancar",
    "atranco": "atrancar",
    "atrancos": "atranco",
    "atrancábamos": "atrancar",
    "atrancáis": "atrancar",
    "atrancáramos": "atrancar",
    "atrancáremos": "atrancar",
    "atrancásemos": "atrancar",
    "atrancó": "atrancar",
    "atranque": "atrancar",
    "atranquemos": "atrancar",
    "atranquen": "atrancar",
    "atranques": "atrancar",
    "atranqué": "atrancar",
    "atranquéis": "atrancar",
    "atrapa": "atrapar",
    "atrapaba": "atrapar",
    "atrapabais": "atrapar",
    "atrapaban": "atrapar",
    "atrapabas": "atrapar",
    "atrapad": "atrapar",
    "atrapada": "atrapar",
    "atrapadas": "atrapar",
    "atrapado": "atrapar",
    "atrapados": "atrapar",
    "atrapamos": "atrapar",
    "atrapan": "atrapar",
    "atrapando": "atrapar",
    "atrapara": "atrapar",
    "atraparais": "atrapar",
    "atraparan": "atrapar",
    "atraparas": "atrapar",
    "atrapare": "atrapar",
    "atrapareis": "atrapar",
    "atraparemos": "atrapar",
    "atraparen": "atrapar",
    "atrapares": "atrapar",
    "atraparon": "atrapar",
    "atrapará": "atrapar",
    "atraparán": "atrapar",
    "atraparás": "atrapar",
    "atraparé": "atrapar",
    "atraparéis": "atrapar",
    "atraparía": "atrapar",
    "atraparíais": "atrapar",
    "atraparíamos": "atrapar",
    "atraparían": "atrapar",
    "atraparías": "atrapar",
    "atrapas": "atrapar",
    "atrapase": "atrapar",
    "atrapaseis": "atrapar",
    "atrapasen": "atrapar",
    "atrapases": "atrapar",
    "atrapaste": "atrapar",
    "atrapasteis": "atrapar",
    "atrape": "atrapar",
    "atrapemos": "atrapar",
    "atrapen": "atrapar",
    "atrapes": "atrapar",
    "atrapo": "atrapar",
    "atrapábamos": "atrapar",
    "atrapáis": "atrapar",
    "atrapáramos": "atrapar",
    "atrapáremos": "atrapar",
    "atrapásemos": "atrapar",
    "atrapé": "atrapar",
    "atrapéis": "atrapar",
    "atrapó": "atrapar",
    "atraque": "atracar",
    "atraquemos": "atracar",
    "atraquen": "atracar",
    "atraques": "atraque",
    "atraqué": "atracar",
    "atraquéis": "atracar",
    "atrasa": "atrasar",
    "atrasaba": "atrasar",
    "atrasabais": "atrasar",
    "atrasaban": "atrasar",
    "atrasabas": "atrasar",
    "atrasad": "atrasar",
    "atrasada": "atrasar",
    "atrasadas": "atrasar",
    "atrasado": "atrasar",
    "atrasados": "atrasar",
    "atrasamos": "atrasar",
    "atrasan": "atrasar",
    "atrasando": "atrasar",
    "atrasara": "atrasar",
    "atrasarais": "atrasar",
    "atrasaran": "atrasar",
    "atrasaras": "atrasar",
    "atrasare": "atrasar",
    "atrasareis": "atrasar",
    "atrasaremos": "atrasar",
    "atrasaren": "atrasar",
    "atrasares": "atrasar",
    "atrasaron": "atrasar",
    "atrasará": "atrasar",
    "atrasarán": "atrasar",
    "atrasarás": "atrasar",
    "atrasaré": "atrasar",
    "atrasaréis": "atrasar",
    "atrasaría": "atrasar",
    "atrasaríais": "atrasar",
    "atrasaríamos": "atrasar",
    "atrasarían": "atrasar",
    "atrasarías": "atrasar",
    "atrasas": "atrasar",
    "atrasase": "atrasar",
    "atrasaseis": "atrasar",
    "atrasasen": "atrasar",
    "atrasases": "atrasar",
    "atrasaste": "atrasar",
    "atrasasteis": "atrasar",
    "atrase": "atrasar",
    "atrasemos": "atrasar",
    "atrasen": "atrasar",
    "atrases": "atrasar",
    "atraso": "atrasar",
    "atrasos": "atraso",
    "atrasábamos": "atrasar",
    "atrasáis": "atrasar",
    "atrasáramos": "atrasar",
    "atrasáremos": "atrasar",
    "atrasásemos": "atrasar",
    "atrasé": "atrasar",
    "atraséis": "atrasar",
    "atrasó": "atrasar",
    "atravesaba": "atravesar",
    "atravesabais": "atravesar",
    "atravesaban": "atravesar",
    "atravesabas": "atravesar",
    "atravesad": "atravesar",
    "atravesada": "atravesar",
    "atravesadas": "atravesar",
    "atravesado": "atravesar",
    "atravesados": "atravesar",
    "atravesamos": "atravesar",
    "atravesando": "atravesar",
    "atravesara": "atravesar",
    "atravesarais": "atravesar",
    "atravesaran": "atravesar",
    "atravesaras": "atravesar",
    "atravesare": "atravesar",
    "atravesareis": "atravesar",
    "atravesaremos": "atravesar",
    "atravesaren": "atravesar",
    "atravesares": "atravesar",
    "atravesaron": "atravesar",
    "atravesará": "atravesar",
    "atravesarán": "atravesar",
    "atravesarás": "atravesar",
    "atravesaré": "atravesar",
    "atravesaréis": "atravesar",
    "atravesaría": "atravesar",
    "atravesaríais": "atravesar",
    "atravesaríamos": "atravesar",
    "atravesarían": "atravesar",
    "atravesarías": "atravesar",
    "atravesase": "atravesar",
    "atravesaseis": "atravesar",
    "atravesasen": "atravesar",
    "atravesases": "atravesar",
    "atravesaste": "atravesar",
    "atravesasteis": "atravesar",
    "atravesemos": "atravesar",
    "atravesábamos": "atravesar",
    "atravesáis": "atravesar",
    "atravesáramos": "atravesar",
    "atravesáremos": "atravesar",
    "atravesásemos": "atravesar",
    "atravesé": "atravesar",
    "atraveséis": "atravesar",
    "atravesó": "atravesar",
    "atraviesa": "atravesar",
    "atraviesan": "atravesar",
    "atraviesas": "atravesar",
    "atraviese": "atravesar",
    "atraviesen": "atravesar",
    "atravieses": "atravesar",
    "atravieso": "atravesar",
    "atrayendo": "atraer",
    "atrayentes": "atrayente",
    "atraéis": "atraer",
    "atraía": "atraer",
    "atraíais": "atraer",
    "atraíamos": "atraer",
    "atraían": "atraer",
    "atraías": "atraer",
    "atraída": "atraer",
    "atraídas": "atraer",
    "atraído": "atraer",
    "atraídos": "atraer",
    "atraílla": "atraillar",
    "atraíllan": "atraillar",
    "atraíllas": "atraillar",
    "atraílle": "atraillar",
    "atraíllen": "atraillar",
    "atraílles": "atraillar",
    "atraíllo": "atraillar",
    "atrepsias": "atrepsia",
    "atresias": "atresia",
    "atresnala": "atresnalar",
    "atresnalaba": "atresnalar",
    "atresnalabais": "atresnalar",
    "atresnalaban": "atresnalar",
    "atresnalabas": "atresnalar",
    "atresnalad": "atresnalar",
    "atresnalada": "atresnalar",
    "atresnaladas": "atresnalar",
    "atresnalado": "atresnalar",
    "atresnalados": "atresnalar",
    "atresnalamos": "atresnalar",
    "atresnalan": "atresnalar",
    "atresnalando": "atresnalar",
    "atresnalara": "atresnalar",
    "atresnalarais": "atresnalar",
    "atresnalaran": "atresnalar",
    "atresnalaras": "atresnalar",
    "atresnalare": "atresnalar",
    "atresnalareis": "atresnalar",
    "atresnalaremos": "atresnalar",
    "atresnalaren": "atresnalar",
    "atresnalares": "atresnalar",
    "atresnalaron": "atresnalar",
    "atresnalará": "atresnalar",
    "atresnalarán": "atresnalar",
    "atresnalarás": "atresnalar",
    "atresnalaré": "atresnalar",
    "atresnalaréis": "atresnalar",
    "atresnalaría": "atresnalar",
    "atresnalaríais": "atresnalar",
    "atresnalaríamos": "atresnalar",
    "atresnalarían": "atresnalar",
    "atresnalarías": "atresnalar",
    "atresnalas": "atresnalar",
    "atresnalase": "atresnalar",
    "atresnalaseis": "atresnalar",
    "atresnalasen": "atresnalar",
    "atresnalases": "atresnalar",
    "atresnalaste": "atresnalar",
    "atresnalasteis": "atresnalar",
    "atresnale": "atresnalar",
    "atresnalemos": "atresnalar",
    "atresnalen": "atresnalar",
    "atresnales": "atresnalar",
    "atresnalo": "atresnalar",
    "atresnalábamos": "atresnalar",
    "atresnaláis": "atresnalar",
    "atresnaláramos": "atresnalar",
    "atresnaláremos": "atresnalar",
    "atresnalásemos": "atresnalar",
    "atresnalé": "atresnalar",
    "atresnaléis": "atresnalar",
    "atresnaló": "atresnalar",
    "atreva": "atrever",
    "atrevamos": "atrever",
    "atrevan": "atrever",
    "atrevas": "atrever",
    "atreve": "atrever",
    "atreved": "atrever",
    "atrevemos": "atrever",
    "atreven": "atrever",
    "atreveremos": "atrever",
    "atreverá": "atrever",
    "atreverán": "atrever",
    "atreverás": "atrever",
    "atreveré": "atrever",
    "atreveréis": "atrever",
    "atrevería": "atrever",
    "atreveríais": "atrever",
    "atreveríamos": "atrever",
    "atreverían": "atrever",
    "atreverías": "atrever",
    "atreves": "atrever",
    "atrevida": "atrevido",
    "atrevidas": "atrevido",
    "atrevido": "atrever",
    "atrevidos": "atrevido",
    "atreviendo": "atrever",
    "atreviera": "atrever",
    "atrevierais": "atrever",
    "atrevieran": "atrever",
    "atrevieras": "atrever",
    "atreviere": "atrever",
    "atreviereis": "atrever",
    "atrevieren": "atrever",
    "atrevieres": "atrever",
    "atrevieron": "atrever",
    "atreviese": "atrever",
    "atrevieseis": "atrever",
    "atreviesen": "atrever",
    "atrevieses": "atrever",
    "atrevimientos": "atrevimiento",
    "atrevimos": "atrever",
    "atreviste": "atrever",
    "atrevisteis": "atrever",
    "atreviéramos": "atrever",
    "atreviéremos": "atrever",
    "atreviésemos": "atrever",
    "atrevió": "atrever",
    "atrevo": "atrever",
    "atreváis": "atrever",
    "atrevéis": "atrever",
    "atreví": "atrever",
    "atrevía": "atrever",
    "atrevíais": "atrever",
    "atrevíamos": "atrever",
    "atrevían": "atrever",
    "atrevías": "atrever",
    "atrezzistas": "atrezzista",
    "atrezzos": "atrezzo",
    "atribuciones": "atribución",
    "atribuibles": "atribuible",
    "atribuid": "atribuir",
    "atribuida": "atribuir",
    "atribuidas": "atribuir",
    "atribuido": "atribuir",
    "atribuidos": "atribuir",
    "atribuimos": "atribuir",
    "atribuiremos": "atribuir",
    "atribuirá": "atribuir",
    "atribuirán": "atribuir",
    "atribuirás": "atribuir",
    "atribuiré": "atribuir",
    "atribuiréis": "atribuir",
    "atribuiría": "atribuir",
    "atribuiríais": "atribuir",
    "atribuiríamos": "atribuir",
    "atribuirían": "atribuir",
    "atribuirías": "atribuir",
    "atribuiste": "atribuir",
    "atribuisteis": "atribuir",
    "atribula": "atribular",
    "atribulaba": "atribular",
    "atribulabais": "atribular",
    "atribulaban": "atribular",
    "atribulabas": "atribular",
    "atribulaciones": "atribulación",
    "atribulad": "atribular",
    "atribulada": "atribular",
    "atribuladas": "atribular",
    "atribulado": "atribular",
    "atribulados": "atribular",
    "atribulamos": "atribular",
    "atribulan": "atribular",
    "atribulando": "atribular",
    "atribulara": "atribular",
    "atribularais": "atribular",
    "atribularan": "atribular",
    "atribularas": "atribular",
    "atribulare": "atribular",
    "atribulareis": "atribular",
    "atribularemos": "atribular",
    "atribularen": "atribular",
    "atribulares": "atribular",
    "atribularon": "atribular",
    "atribulará": "atribular",
    "atribularán": "atribular",
    "atribularás": "atribular",
    "atribularé": "atribular",
    "atribularéis": "atribular",
    "atribularía": "atribular",
    "atribularíais": "atribular",
    "atribularíamos": "atribular",
    "atribularían": "atribular",
    "atribularías": "atribular",
    "atribulas": "atribular",
    "atribulase": "atribular",
    "atribulaseis": "atribular",
    "atribulasen": "atribular",
    "atribulases": "atribular",
    "atribulaste": "atribular",
    "atribulasteis": "atribular",
    "atribule": "atribular",
    "atribulemos": "atribular",
    "atribulen": "atribular",
    "atribules": "atribular",
    "atribulo": "atribular",
    "atribulábamos": "atribular",
    "atribuláis": "atribular",
    "atribuláramos": "atribular",
    "atribuláremos": "atribular",
    "atribulásemos": "atribular",
    "atribulé": "atribular",
    "atribuléis": "atribular",
    "atribuló": "atribular",
    "atributiva": "atributivo",
    "atributivas": "atributivo",
    "atributivos": "atributivo",
    "atributos": "atributo",
    "atribuya": "atribuir",
    "atribuyamos": "atribuir",
    "atribuyan": "atribuir",
    "atribuyas": "atribuir",
    "atribuye": "atribuir",
    "atribuyen": "atribuir",
    "atribuyendo": "atribuir",
    "atribuyera": "atribuir",
    "atribuyerais": "atribuir",
    "atribuyeran": "atribuir",
    "atribuyeras": "atribuir",
    "atribuyere": "atribuir",
    "atribuyereis": "atribuir",
    "atribuyeren": "atribuir",
    "atribuyeres": "atribuir",
    "atribuyeron": "atribuir",
    "atribuyes": "atribuir",
    "atribuyese": "atribuir",
    "atribuyeseis": "atribuir",
    "atribuyesen": "atribuir",
    "atribuyeses": "atribuir",
    "atribuyo": "atribuir",
    "atribuyáis": "atribuir",
    "atribuyéramos": "atribuir",
    "atribuyéremos": "atribuir",
    "atribuyésemos": "atribuir",
    "atribuyó": "atribuir",
    "atribuí": "atribuir",
    "atribuía": "atribuir",
    "atribuíais": "atribuir",
    "atribuíamos": "atribuir",
    "atribuían": "atribuir",
    "atribuías": "atribuir",
    "atribuís": "atribuir",
    "atriciones": "atrición",
    "atridas": "atrida",
    "atriles": "atril",
    "atrinca": "atrincar",
    "atrincaba": "atrincar",
    "atrincabais": "atrincar",
    "atrincaban": "atrincar",
    "atrincabas": "atrincar",
    "atrincad": "atrincar",
    "atrincada": "atrincar",
    "atrincadas": "atrincar",
    "atrincado": "atrincar",
    "atrincados": "atrincar",
    "atrincamos": "atrincar",
    "atrincan": "atrincar",
    "atrincando": "atrincar",
    "atrincara": "atrincar",
    "atrincarais": "atrincar",
    "atrincaran": "atrincar",
    "atrincaras": "atrincar",
    "atrincare": "atrincar",
    "atrincareis": "atrincar",
    "atrincaremos": "atrincar",
    "atrincaren": "atrincar",
    "atrincares": "atrincar",
    "atrincaron": "atrincar",
    "atrincará": "atrincar",
    "atrincarán": "atrincar",
    "atrincarás": "atrincar",
    "atrincaré": "atrincar",
    "atrincaréis": "atrincar",
    "atrincaría": "atrincar",
    "atrincaríais": "atrincar",
    "atrincaríamos": "atrincar",
    "atrincarían": "atrincar",
    "atrincarías": "atrincar",
    "atrincas": "atrincar",
    "atrincase": "atrincar",
    "atrincaseis": "atrincar",
    "atrincasen": "atrincar",
    "atrincases": "atrincar",
    "atrincaste": "atrincar",
    "atrincasteis": "atrincar",
    "atrinchera": "atrincherar",
    "atrincheraba": "atrincherar",
    "atrincherabais": "atrincherar",
    "atrincheraban": "atrincherar",
    "atrincherabas": "atrincherar",
    "atrincherad": "atrincherar",
    "atrincherada": "atrincherar",
    "atrincheradas": "atrincherar",
    "atrincherado": "atrincherar",
    "atrincherados": "atrincherar",
    "atrincheramientos": "atrincheramiento",
    "atrincheramos": "atrincherar",
    "atrincheran": "atrincherar",
    "atrincherando": "atrincherar",
    "atrincherara": "atrincherar",
    "atrincherarais": "atrincherar",
    "atrincheraran": "atrincherar",
    "atrincheraras": "atrincherar",
    "atrincherare": "atrincherar",
    "atrincherareis": "atrincherar",
    "atrincheraremos": "atrincherar",
    "atrincheraren": "atrincherar",
    "atrincherares": "atrincherar",
    "atrincheraron": "atrincherar",
    "atrincherará": "atrincherar",
    "atrincherarán": "atrincherar",
    "atrincherarás": "atrincherar",
    "atrincheraré": "atrincherar",
    "atrincheraréis": "atrincherar",
    "atrincheraría": "atrincherar",
    "atrincheraríais": "atrincherar",
    "atrincheraríamos": "atrincherar",
    "atrincherarían": "atrincherar",
    "atrincherarías": "atrincherar",
    "atrincheras": "atrincherar",
    "atrincherase": "atrincherar",
    "atrincheraseis": "atrincherar",
    "atrincherasen": "atrincherar",
    "atrincherases": "atrincherar",
    "atrincheraste": "atrincherar",
    "atrincherasteis": "atrincherar",
    "atrinchere": "atrincherar",
    "atrincheremos": "atrincherar",
    "atrincheren": "atrincherar",
    "atrincheres": "atrincherar",
    "atrinchero": "atrincherar",
    "atrincherábamos": "atrincherar",
    "atrincheráis": "atrincherar",
    "atrincheráramos": "atrincherar",
    "atrincheráremos": "atrincherar",
    "atrincherásemos": "atrincherar",
    "atrincheré": "atrincherar",
    "atrincheréis": "atrincherar",
    "atrincheró": "atrincherar",
    "atrinco": "atrincar",
    "atrincábamos": "atrincar",
    "atrincáis": "atrincar",
    "atrincáramos": "atrincar",
    "atrincáremos": "atrincar",
    "atrincásemos": "atrincar",
    "atrincó": "atrincar",
    "atrinque": "atrincar",
    "atrinquemos": "atrincar",
    "atrinquen": "atrincar",
    "atrinques": "atrincar",
    "atrinqué": "atrincar",
    "atrinquéis": "atrincar",
    "atrios": "atrio",
    "atrioventriculares": "atrioventricular",
    "atrita": "atrito",
    "atritas": "atrito",
    "atritos": "atrito",
    "atroces": "atroz",
    "atrocha": "atrochar",
    "atrochaba": "atrochar",
    "atrochabais": "atrochar",
    "atrochaban": "atrochar",
    "atrochabas": "atrochar",
    "atrochad": "atrochar",
    "atrochada": "atrochar",
    "atrochadas": "atrochar",
    "atrochado": "atrochar",
    "atrochados": "atrochar",
    "atrochamos": "atrochar",
    "atrochan": "atrochar",
    "atrochando": "atrochar",
    "atrochara": "atrochar",
    "atrocharais": "atrochar",
    "atrocharan": "atrochar",
    "atrocharas": "atrochar",
    "atrochare": "atrochar",
    "atrochareis": "atrochar",
    "atrocharemos": "atrochar",
    "atrocharen": "atrochar",
    "atrochares": "atrochar",
    "atrocharon": "atrochar",
    "atrochará": "atrochar",
    "atrocharán": "atrochar",
    "atrocharás": "atrochar",
    "atrocharé": "atrochar",
    "atrocharéis": "atrochar",
    "atrocharía": "atrochar",
    "atrocharíais": "atrochar",
    "atrocharíamos": "atrochar",
    "atrocharían": "atrochar",
    "atrocharías": "atrochar",
    "atrochas": "atrochar",
    "atrochase": "atrochar",
    "atrochaseis": "atrochar",
    "atrochasen": "atrochar",
    "atrochases": "atrochar",
    "atrochaste": "atrochar",
    "atrochasteis": "atrochar",
    "atroche": "atrochar",
    "atrochemos": "atrochar",
    "atrochen": "atrochar",
    "atroches": "atrochar",
    "atrocho": "atrochar",
    "atrochábamos": "atrochar",
    "atrocháis": "atrochar",
    "atrocháramos": "atrochar",
    "atrocháremos": "atrochar",
    "atrochásemos": "atrochar",
    "atroché": "atrochar",
    "atrochéis": "atrochar",
    "atrochó": "atrochar",
    "atrocidades": "atrocidad",
    "atrofia": "atrofiar",
    "atrofiaba": "atrofiar",
    "atrofiabais": "atrofiar",
    "atrofiaban": "atrofiar",
    "atrofiabas": "atrofiar",
    "atrofiad": "atrofiar",
    "atrofiada": "atrofiar",
    "atrofiadas": "atrofiar",
    "atrofiado": "atrofiar",
    "atrofiados": "atrofiar",
    "atrofiamos": "atrofiar",
    "atrofian": "atrofiar",
    "atrofiando": "atrofiar",
    "atrofiara": "atrofiar",
    "atrofiarais": "atrofiar",
    "atrofiaran": "atrofiar",
    "atrofiaras": "atrofiar",
    "atrofiare": "atrofiar",
    "atrofiareis": "atrofiar",
    "atrofiaremos": "atrofiar",
    "atrofiaren": "atrofiar",
    "atrofiares": "atrofiar",
    "atrofiaron": "atrofiar",
    "atrofiará": "atrofiar",
    "atrofiarán": "atrofiar",
    "atrofiarás": "atrofiar",
    "atrofiaré": "atrofiar",
    "atrofiaréis": "atrofiar",
    "atrofiaría": "atrofiar",
    "atrofiaríais": "atrofiar",
    "atrofiaríamos": "atrofiar",
    "atrofiarían": "atrofiar",
    "atrofiarías": "atrofiar",
    "atrofias": "atrofiar",
    "atrofiase": "atrofiar",
    "atrofiaseis": "atrofiar",
    "atrofiasen": "atrofiar",
    "atrofiases": "atrofiar",
    "atrofiaste": "atrofiar",
    "atrofiasteis": "atrofiar",
    "atrofie": "atrofiar",
    "atrofiemos": "atrofiar",
    "atrofien": "atrofiar",
    "atrofies": "atrofiar",
    "atrofio": "atrofiar",
    "atrofiábamos": "atrofiar",
    "atrofiáis": "atrofiar",
    "atrofiáramos": "atrofiar",
    "atrofiáremos": "atrofiar",
    "atrofiásemos": "atrofiar",
    "atrofié": "atrofiar",
    "atrofiéis": "atrofiar",
    "atrofió": "atrofiar",
    "atroja": "atrojar",
    "atrojaba": "atrojar",
    "atrojabais": "atrojar",
    "atrojaban": "atrojar",
    "atrojabas": "atrojar",
    "atrojad": "atrojar",
    "atrojada": "atrojar",
    "atrojadas": "atrojar",
    "atrojado": "atrojar",
    "atrojados": "atrojar",
    "atrojamos": "atrojar",
    "atrojan": "atrojar",
    "atrojando": "atrojar",
    "atrojara": "atrojar",
    "atrojarais": "atrojar",
    "atrojaran": "atrojar",
    "atrojaras": "atrojar",
    "atrojare": "atrojar",
    "atrojareis": "atrojar",
    "atrojaremos": "atrojar",
    "atrojaren": "atrojar",
    "atrojares": "atrojar",
    "atrojaron": "atrojar",
    "atrojará": "atrojar",
    "atrojarán": "atrojar",
    "atrojarás": "atrojar",
    "atrojaré": "atrojar",
    "atrojaréis": "atrojar",
    "atrojaría": "atrojar",
    "atrojaríais": "atrojar",
    "atrojaríamos": "atrojar",
    "atrojarían": "atrojar",
    "atrojarías": "atrojar",
    "atrojas": "atrojar",
    "atrojase": "atrojar",
    "atrojaseis": "atrojar",
    "atrojasen": "atrojar",
    "atrojases": "atrojar",
    "atrojaste": "atrojar",
    "atrojasteis": "atrojar",
    "atroje": "atrojar",
    "atrojemos": "atrojar",
    "atrojen": "atrojar",
    "atrojes": "atrojar",
    "atrojo": "atrojar",
    "atrojábamos": "atrojar",
    "atrojáis": "atrojar",
    "atrojáramos": "atrojar",
    "atrojáremos": "atrojar",
    "atrojásemos": "atrojar",
    "atrojé": "atrojar",
    "atrojéis": "atrojar",
    "atrojó": "atrojar",
    "atrompetada": "atrompetado",
    "atrompetadas": "atrompetado",
    "atrompetados": "atrompetado",
    "atronaba": "atronar",
    "atronabais": "atronar",
    "atronaban": "atronar",
    "atronabas": "atronar",
    "atronad": "atronar",
    "atronada": "atronar",
    "atronadas": "atronar",
    "atronado": "atronar",
    "atronadora": "atronador",
    "atronadoras": "atronador",
    "atronadores": "atronador",
    "atronados": "atronar",
    "atronamientos": "atronamiento",
    "atronamos": "atronar",
    "atronando": "atronar",
    "atronara": "atronar",
    "atronarais": "atronar",
    "atronaran": "atronar",
    "atronaras": "atronar",
    "atronare": "atronar",
    "atronareis": "atronar",
    "atronaremos": "atronar",
    "atronaren": "atronar",
    "atronares": "atronar",
    "atronaron": "atronar",
    "atronará": "atronar",
    "atronarán": "atronar",
    "atronarás": "atronar",
    "atronaré": "atronar",
    "atronaréis": "atronar",
    "atronaría": "atronar",
    "atronaríais": "atronar",
    "atronaríamos": "atronar",
    "atronarían": "atronar",
    "atronarías": "atronar",
    "atronase": "atronar",
    "atronaseis": "atronar",
    "atronasen": "atronar",
    "atronases": "atronar",
    "atronaste": "atronar",
    "atronasteis": "atronar",
    "atronemos": "atronar",
    "atronera": "atronerar",
    "atroneraba": "atronerar",
    "atronerabais": "atronerar",
    "atroneraban": "atronerar",
    "atronerabas": "atronerar",
    "atronerad": "atronerar",
    "atronerada": "atronerar",
    "atroneradas": "atronerar",
    "atronerado": "atronerar",
    "atronerados": "atronerar",
    "atroneramos": "atronerar",
    "atroneran": "atronerar",
    "atronerando": "atronerar",
    "atronerara": "atronerar",
    "atronerarais": "atronerar",
    "atroneraran": "atronerar",
    "atroneraras": "atronerar",
    "atronerare": "atronerar",
    "atronerareis": "atronerar",
    "atroneraremos": "atronerar",
    "atroneraren": "atronerar",
    "atronerares": "atronerar",
    "atroneraron": "atronerar",
    "atronerará": "atronerar",
    "atronerarán": "atronerar",
    "atronerarás": "atronerar",
    "atroneraré": "atronerar",
    "atroneraréis": "atronerar",
    "atroneraría": "atronerar",
    "atroneraríais": "atronerar",
    "atroneraríamos": "atronerar",
    "atronerarían": "atronerar",
    "atronerarías": "atronerar",
    "atroneras": "atronerar",
    "atronerase": "atronerar",
    "atroneraseis": "atronerar",
    "atronerasen": "atronerar",
    "atronerases": "atronerar",
    "atroneraste": "atronerar",
    "atronerasteis": "atronerar",
    "atronere": "atronerar",
    "atroneremos": "atronerar",
    "atroneren": "atronerar",
    "atroneres": "atronerar",
    "atronero": "atronerar",
    "atronerábamos": "atronerar",
    "atroneráis": "atronerar",
    "atroneráramos": "atronerar",
    "atroneráremos": "atronerar",
    "atronerásemos": "atronerar",
    "atroneré": "atronerar",
    "atroneréis": "atronerar",
    "atroneró": "atronerar",
    "atronábamos": "atronar",
    "atronáis": "atronar",
    "atronáramos": "atronar",
    "atronáremos": "atronar",
    "atronásemos": "atronar",
    "atroné": "atronar",
    "atronéis": "atronar",
    "atronó": "atronar",
    "atropa": "atropar",
    "atropaba": "atropar",
    "atropabais": "atropar",
    "atropaban": "atropar",
    "atropabas": "atropar",
    "atropad": "atropar",
    "atropada": "atropar",
    "atropadas": "atropar",
    "atropado": "atropar",
    "atropados": "atropar",
    "atropamos": "atropar",
    "atropan": "atropar",
    "atropando": "atropar",
    "atropara": "atropar",
    "atroparais": "atropar",
    "atroparan": "atropar",
    "atroparas": "atropar",
    "atropare": "atropar",
    "atropareis": "atropar",
    "atroparemos": "atropar",
    "atroparen": "atropar",
    "atropares": "atropar",
    "atroparon": "atropar",
    "atropará": "atropar",
    "atroparán": "atropar",
    "atroparás": "atropar",
    "atroparé": "atropar",
    "atroparéis": "atropar",
    "atroparía": "atropar",
    "atroparíais": "atropar",
    "atroparíamos": "atropar",
    "atroparían": "atropar",
    "atroparías": "atropar",
    "atropas": "atropar",
    "atropase": "atropar",
    "atropaseis": "atropar",
    "atropasen": "atropar",
    "atropases": "atropar",
    "atropaste": "atropar",
    "atropasteis": "atropar",
    "atrope": "atropar",
    "atropella": "atropellar",
    "atropellaba": "atropellar",
    "atropellabais": "atropellar",
    "atropellaban": "atropellar",
    "atropellabas": "atropellar",
    "atropellad": "atropellar",
    "atropellada": "atropellar",
    "atropelladas": "atropellar",
    "atropellado": "atropellar",
    "atropelladora": "atropellador",
    "atropelladoras": "atropellador",
    "atropelladores": "atropellador",
    "atropellados": "atropellar",
    "atropellamientos": "atropellamiento",
    "atropellamos": "atropellar",
    "atropellan": "atropellar",
    "atropellando": "atropellar",
    "atropellara": "atropellar",
    "atropellarais": "atropellar",
    "atropellaran": "atropellar",
    "atropellaras": "atropellar",
    "atropellare": "atropellar",
    "atropellareis": "atropellar",
    "atropellaremos": "atropellar",
    "atropellaren": "atropellar",
    "atropellares": "atropellar",
    "atropellaron": "atropellar",
    "atropellará": "atropellar",
    "atropellarán": "atropellar",
    "atropellarás": "atropellar",
    "atropellaré": "atropellar",
    "atropellaréis": "atropellar",
    "atropellaría": "atropellar",
    "atropellaríais": "atropellar",
    "atropellaríamos": "atropellar",
    "atropellarían": "atropellar",
    "atropellarías": "atropellar",
    "atropellas": "atropellar",
    "atropellase": "atropellar",
    "atropellaseis": "atropellar",
    "atropellasen": "atropellar",
    "atropellases": "atropellar",
    "atropellaste": "atropellar",
    "atropellasteis": "atropellar",
    "atropelle": "atropellar",
    "atropellemos": "atropellar",
    "atropellen": "atropellar",
    "atropelles": "atropellar",
    "atropello": "atropellar",
    "atropellos": "atropello",
    "atropellábamos": "atropellar",
    "atropelláis": "atropellar",
    "atropelláramos": "atropellar",
    "atropelláremos": "atropellar",
    "atropellásemos": "atropellar",
    "atropellé": "atropellar",
    "atropelléis": "atropellar",
    "atropelló": "atropellar",
    "atropemos": "atropar",
    "atropen": "atropar",
    "atropes": "atropar",
    "atropinas": "atropina",
    "atropo": "atropar",
    "atropábamos": "atropar",
    "atropáis": "atropar",
    "atropáramos": "atropar",
    "atropáremos": "atropar",
    "atropásemos": "atropar",
    "atropé": "atropar",
    "atropéis": "atropar",
    "atropó": "atropar",
    "atrozes": "atroz",
    "atruchada": "atruchado",
    "atruchadas": "atruchado",
    "atruchados": "atruchado",
    "atruena": "atronar",
    "atruenan": "atronar",
    "atruenas": "atronar",
    "atruene": "atronar",
    "atruenen": "atronar",
    "atruenes": "atronar",
    "atrueno": "atronar",
    "atrófica": "atrófico",
    "atróficas": "atrófico",
    "atróficos": "atrófico",
    "atuendos": "atuendo",
    "atuera": "atorar",
    "atueran": "atorar",
    "atueras": "atorar",
    "atuere": "atorar",
    "atueren": "atorar",
    "atueres": "atorar",
    "atuero": "atorar",
    "atufa": "atufar",
    "atufaba": "atufar",
    "atufabais": "atufar",
    "atufaban": "atufar",
    "atufabas": "atufar",
    "atufad": "atufar",
    "atufada": "atufar",
    "atufadas": "atufar",
    "atufado": "atufar",
    "atufados": "atufar",
    "atufamientos": "atufamiento",
    "atufamos": "atufar",
    "atufan": "atufar",
    "atufando": "atufar",
    "atufara": "atufar",
    "atufarais": "atufar",
    "atufaran": "atufar",
    "atufaras": "atufar",
    "atufare": "atufar",
    "atufareis": "atufar",
    "atufaremos": "atufar",
    "atufaren": "atufar",
    "atufares": "atufar",
    "atufaron": "atufar",
    "atufará": "atufar",
    "atufarán": "atufar",
    "atufarás": "atufar",
    "atufaré": "atufar",
    "atufaréis": "atufar",
    "atufaría": "atufar",
    "atufaríais": "atufar",
    "atufaríamos": "atufar",
    "atufarían": "atufar",
    "atufarías": "atufar",
    "atufas": "atufar",
    "atufase": "atufar",
    "atufaseis": "atufar",
    "atufasen": "atufar",
    "atufases": "atufar",
    "atufaste": "atufar",
    "atufasteis": "atufar",
    "atufe": "atufar",
    "atufemos": "atufar",
    "atufen": "atufar",
    "atufes": "atufar",
    "atufo": "atufar",
    "atufos": "atufo",
    "atufábamos": "atufar",
    "atufáis": "atufar",
    "atufáramos": "atufar",
    "atufáremos": "atufar",
    "atufásemos": "atufar",
    "atufé": "atufar",
    "atuféis": "atufar",
    "atufó": "atufar",
    "atunera": "atunero",
    "atuneras": "atunero",
    "atuneros": "atunero",
    "atunes": "atún",
    "atura": "aturar",
    "aturaba": "aturar",
    "aturabais": "aturar",
    "aturaban": "aturar",
    "aturabas": "aturar",
    "aturad": "aturar",
    "aturada": "aturar",
    "aturadas": "aturar",
    "aturado": "aturar",
    "aturados": "aturar",
    "aturamos": "aturar",
    "aturan": "aturar",
    "aturando": "aturar",
    "aturara": "aturar",
    "aturarais": "aturar",
    "aturaran": "aturar",
    "aturaras": "aturar",
    "aturare": "aturar",
    "aturareis": "aturar",
    "aturaremos": "aturar",
    "aturaren": "aturar",
    "aturares": "aturar",
    "aturaron": "aturar",
    "aturará": "aturar",
    "aturarán": "aturar",
    "aturarás": "aturar",
    "aturaré": "aturar",
    "aturaréis": "aturar",
    "aturaría": "aturar",
    "aturaríais": "aturar",
    "aturaríamos": "aturar",
    "aturarían": "aturar",
    "aturarías": "aturar",
    "aturas": "aturar",
    "aturase": "aturar",
    "aturaseis": "aturar",
    "aturasen": "aturar",
    "aturases": "aturar",
    "aturaste": "aturar",
    "aturasteis": "aturar",
    "aturbona": "aturbonar",
    "aturbonaba": "aturbonar",
    "aturbonabais": "aturbonar",
    "aturbonaban": "aturbonar",
    "aturbonabas": "aturbonar",
    "aturbonad": "aturbonar",
    "aturbonada": "aturbonar",
    "aturbonadas": "aturbonar",
    "aturbonado": "aturbonar",
    "aturbonados": "aturbonar",
    "aturbonamos": "aturbonar",
    "aturbonan": "aturbonar",
    "aturbonando": "aturbonar",
    "aturbonara": "aturbonar",
    "aturbonarais": "aturbonar",
    "aturbonaran": "aturbonar",
    "aturbonaras": "aturbonar",
    "aturbonare": "aturbonar",
    "aturbonareis": "aturbonar",
    "aturbonaremos": "aturbonar",
    "aturbonaren": "aturbonar",
    "aturbonares": "aturbonar",
    "aturbonaron": "aturbonar",
    "aturbonará": "aturbonar",
    "aturbonarán": "aturbonar",
    "aturbonarás": "aturbonar",
    "aturbonaré": "aturbonar",
    "aturbonaréis": "aturbonar",
    "aturbonaría": "aturbonar",
    "aturbonaríais": "aturbonar",
    "aturbonaríamos": "aturbonar",
    "aturbonarían": "aturbonar",
    "aturbonarías": "aturbonar",
    "aturbonas": "aturbonar",
    "aturbonase": "aturbonar",
    "aturbonaseis": "aturbonar",
    "aturbonasen": "aturbonar",
    "aturbonases": "aturbonar",
    "aturbonaste": "aturbonar",
    "aturbonasteis": "aturbonar",
    "aturbone": "aturbonar",
    "aturbonemos": "aturbonar",
    "aturbonen": "aturbonar",
    "aturbones": "aturbonar",
    "aturbono": "aturbonar",
    "aturbonábamos": "aturbonar",
    "aturbonáis": "aturbonar",
    "aturbonáramos": "aturbonar",
    "aturbonáremos": "aturbonar",
    "aturbonásemos": "aturbonar",
    "aturboné": "aturbonar",
    "aturbonéis": "aturbonar",
    "aturbonó": "aturbonar",
    "aturda": "aturdir",
    "aturdamos": "aturdir",
    "aturdan": "aturdir",
    "aturdas": "aturdir",
    "aturde": "aturdir",
    "aturden": "aturdir",
    "aturdes": "aturdir",
    "aturdid": "aturdir",
    "aturdida": "aturdir",
    "aturdidas": "aturdir",
    "aturdido": "aturdir",
    "aturdidora": "aturdidor",
    "aturdidoras": "aturdidor",
    "aturdidores": "aturdidor",
    "aturdidos": "aturdir",
    "aturdiendo": "aturdir",
    "aturdiera": "aturdir",
    "aturdierais": "aturdir",
    "aturdieran": "aturdir",
    "aturdieras": "aturdir",
    "aturdiere": "aturdir",
    "aturdiereis": "aturdir",
    "aturdieren": "aturdir",
    "aturdieres": "aturdir",
    "aturdieron": "aturdir",
    "aturdiese": "aturdir",
    "aturdieseis": "aturdir",
    "aturdiesen": "aturdir",
    "aturdieses": "aturdir",
    "aturdimientos": "aturdimiento",
    "aturdimos": "aturdir",
    "aturdiremos": "aturdir",
    "aturdirá": "aturdir",
    "aturdirán": "aturdir",
    "aturdirás": "aturdir",
    "aturdiré": "aturdir",
    "aturdiréis": "aturdir",
    "aturdiría": "aturdir",
    "aturdiríais": "aturdir",
    "aturdiríamos": "aturdir",
    "aturdirían": "aturdir",
    "aturdirías": "aturdir",
    "aturdiste": "aturdir",
    "aturdisteis": "aturdir",
    "aturdiéramos": "aturdir",
    "aturdiéremos": "aturdir",
    "aturdiésemos": "aturdir",
    "aturdió": "aturdir",
    "aturdo": "aturdir",
    "aturdáis": "aturdir",
    "aturdí": "aturdir",
    "aturdía": "aturdir",
    "aturdíais": "aturdir",
    "aturdíamos": "aturdir",
    "aturdían": "aturdir",
    "aturdías": "aturdir",
    "aturdís": "aturdir",
    "ature": "aturar",
    "aturemos": "aturar",
    "aturen": "aturar",
    "atures": "aturar",
    "aturo": "aturar",
    "aturrulla": "aturrullar",
    "aturrullaba": "aturrullar",
    "aturrullabais": "aturrullar",
    "aturrullaban": "aturrullar",
    "aturrullabas": "aturrullar",
    "aturrullad": "aturrullar",
    "aturrullada": "aturrullar",
    "aturrulladas": "aturrullar",
    "aturrullado": "aturrullar",
    "aturrullados": "aturrullar",
    "aturrullamos": "aturrullar",
    "aturrullan": "aturrullar",
    "aturrullando": "aturrullar",
    "aturrullara": "aturrullar",
    "aturrullarais": "aturrullar",
    "aturrullaran": "aturrullar",
    "aturrullaras": "aturrullar",
    "aturrullare": "aturrullar",
    "aturrullareis": "aturrullar",
    "aturrullaremos": "aturrullar",
    "aturrullaren": "aturrullar",
    "aturrullares": "aturrullar",
    "aturrullaron": "aturrullar",
    "aturrullará": "aturrullar",
    "aturrullarán": "aturrullar",
    "aturrullarás": "aturrullar",
    "aturrullaré": "aturrullar",
    "aturrullaréis": "aturrullar",
    "aturrullaría": "aturrullar",
    "aturrullaríais": "aturrullar",
    "aturrullaríamos": "aturrullar",
    "aturrullarían": "aturrullar",
    "aturrullarías": "aturrullar",
    "aturrullas": "aturrullar",
    "aturrullase": "aturrullar",
    "aturrullaseis": "aturrullar",
    "aturrullasen": "aturrullar",
    "aturrullases": "aturrullar",
    "aturrullaste": "aturrullar",
    "aturrullasteis": "aturrullar",
    "aturrulle": "aturrullar",
    "aturrullemos": "aturrullar",
    "aturrullen": "aturrullar",
    "aturrulles": "aturrullar",
    "aturrullo": "aturrullar",
    "aturrullábamos": "aturrullar",
    "aturrulláis": "aturrullar",
    "aturrulláramos": "aturrullar",
    "aturrulláremos": "aturrullar",
    "aturrullásemos": "aturrullar",
    "aturrullé": "aturrullar",
    "aturrulléis": "aturrullar",
    "aturrulló": "aturrullar",
    "aturulla": "aturullar",
    "aturullaba": "aturullar",
    "aturullabais": "aturullar",
    "aturullaban": "aturullar",
    "aturullabas": "aturullar",
    "aturullad": "aturullar",
    "aturullada": "aturullar",
    "aturulladas": "aturullar",
    "aturullado": "aturullar",
    "aturullados": "aturullar",
    "aturullamientos": "aturullamiento",
    "aturullamos": "aturullar",
    "aturullan": "aturullar",
    "aturullando": "aturullar",
    "aturullara": "aturullar",
    "aturullarais": "aturullar",
    "aturullaran": "aturullar",
    "aturullaras": "aturullar",
    "aturullare": "aturullar",
    "aturullareis": "aturullar",
    "aturullaremos": "aturullar",
    "aturullaren": "aturullar",
    "aturullares": "aturullar",
    "aturullaron": "aturullar",
    "aturullará": "aturullar",
    "aturullarán": "aturullar",
    "aturullarás": "aturullar",
    "aturullaré": "aturullar",
    "aturullaréis": "aturullar",
    "aturullaría": "aturullar",
    "aturullaríais": "aturullar",
    "aturullaríamos": "aturullar",
    "aturullarían": "aturullar",
    "aturullarías": "aturullar",
    "aturullas": "aturullar",
    "aturullase": "aturullar",
    "aturullaseis": "aturullar",
    "aturullasen": "aturullar",
    "aturullases": "aturullar",
    "aturullaste": "aturullar",
    "aturullasteis": "aturullar",
    "aturulle": "aturullar",
    "aturullemos": "aturullar",
    "aturullen": "aturullar",
    "aturulles": "aturullar",
    "aturullo": "aturullar",
    "aturullábamos": "aturullar",
    "aturulláis": "aturullar",
    "aturulláramos": "aturullar",
    "aturulláremos": "aturullar",
    "aturullásemos": "aturullar",
    "aturullé": "aturullar",
    "aturulléis": "aturullar",
    "aturulló": "aturullar",
    "aturábamos": "aturar",
    "aturáis": "aturar",
    "aturáramos": "aturar",
    "aturáremos": "aturar",
    "aturásemos": "aturar",
    "aturé": "aturar",
    "aturéis": "aturar",
    "aturó": "aturar",
    "atusa": "atusar",
    "atusaba": "atusar",
    "atusabais": "atusar",
    "atusaban": "atusar",
    "atusabas": "atusar",
    "atusad": "atusar",
    "atusada": "atusar",
    "atusadas": "atusar",
    "atusado": "atusar",
    "atusadora": "atusador",
    "atusadoras": "atusador",
    "atusadores": "atusador",
    "atusados": "atusar",
    "atusamos": "atusar",
    "atusan": "atusar",
    "atusando": "atusar",
    "atusara": "atusar",
    "atusarais": "atusar",
    "atusaran": "atusar",
    "atusaras": "atusar",
    "atusare": "atusar",
    "atusareis": "atusar",
    "atusaremos": "atusar",
    "atusaren": "atusar",
    "atusares": "atusar",
    "atusaron": "atusar",
    "atusará": "atusar",
    "atusarán": "atusar",
    "atusarás": "atusar",
    "atusaré": "atusar",
    "atusaréis": "atusar",
    "atusaría": "atusar",
    "atusaríais": "atusar",
    "atusaríamos": "atusar",
    "atusarían": "atusar",
    "atusarías": "atusar",
    "atusas": "atusar",
    "atusase": "atusar",
    "atusaseis": "atusar",
    "atusasen": "atusar",
    "atusases": "atusar",
    "atusaste": "atusar",
    "atusasteis": "atusar",
    "atuse": "atusar",
    "atusemos": "atusar",
    "atusen": "atusar",
    "atuses": "atusar",
    "atuso": "atusar",
    "atusábamos": "atusar",
    "atusáis": "atusar",
    "atusáramos": "atusar",
    "atusáremos": "atusar",
    "atusásemos": "atusar",
    "atusé": "atusar",
    "atuséis": "atusar",
    "atusó": "atusar",
    "atuve": "atener",
    "atuviera": "atener",
    "atuvierais": "atener",
    "atuvieran": "atener",
    "atuvieras": "atener",
    "atuviere": "atener",
    "atuviereis": "atener",
    "atuvieren": "atener",
    "atuvieres": "atener",
    "atuvieron": "atener",
    "atuviese": "atener",
    "atuvieseis": "atener",
    "atuviesen": "atener",
    "atuvieses": "atener",
    "atuvimos": "atener",
    "atuviste": "atener",
    "atuvisteis": "atener",
    "atuviéramos": "atener",
    "atuviéremos": "atener",
    "atuviésemos": "atener",
    "atuvo": "atener",
    "atábamos": "atar",
    "atáctica": "atáctico",
    "atácticas": "atáctico",
    "atácticos": "atáctico",
    "atáis": "atar",
    "atálidas": "atálida",
    "atáramos": "atar",
    "atáremos": "atar",
    "atásemos": "atar",
    "atávica": "atávico",
    "atávicas": "atávico",
    "atávicos": "atávico",
    "atáxica": "atáxico",
    "atáxicas": "atáxico",
    "atáxicos": "atáxico",
    "até": "atar",
    "atéis": "atar",
    "atén": "atener",
    "atérmana": "atérmano",
    "atérmanas": "atérmano",
    "atérmanos": "atérmano",
    "atérmica": "atérmico",
    "atérmicas": "atérmico",
    "atérmicos": "atérmico",
    "atíncares": "atíncar",
    "atípica": "atípico",
    "atípicas": "atípico",
    "atípicos": "atípico",
    "ató": "atar",
    "atómica": "atómico",
    "atómicas": "atómico",
    "atómicos": "atómico",
    "atónica": "atónico",
    "atónicas": "atónico",
    "atónicos": "atónico",
    "atónita": "atónito",
    "atónitas": "atónito",
    "atónitos": "atónito",
    "aucubas": "aucuba",
    "audaces": "audaz",
    "audacias": "audacia",
    "audazes": "audaz",
    "audibilidades": "audibilidad",
    "audibles": "audible",
    "audiciones": "audición",
    "audiencias": "audiencia",
    "audio-visuales": "audio-visual",
    "audioanalgesias": "audioanalgesia",
    "audioconferencias": "audioconferencia",
    "audiofrecuencias": "audiofrecuencia",
    "audiografías": "audiografía",
    "audiogramas": "audiograma",
    "audiogénica": "audiogénico",
    "audiogénicas": "audiogénico",
    "audiogénicos": "audiogénico",
    "audiologías": "audiología",
    "audiomensajerías": "audiomensajería",
    "audiometrías": "audiometría",
    "audioprotesistas": "audioprotesista",
    "audios": "audio",
    "audiovisuales": "audiovisual",
    "audita": "auditar",
    "auditaba": "auditar",
    "auditabais": "auditar",
    "auditaban": "auditar",
    "auditabas": "auditar",
    "auditad": "auditar",
    "auditada": "auditar",
    "auditadas": "auditar",
    "auditado": "auditar",
    "auditados": "auditar",
    "auditamos": "auditar",
    "auditan": "auditar",
    "auditando": "auditar",
    "auditara": "auditar",
    "auditarais": "auditar",
    "auditaran": "auditar",
    "auditaras": "auditar",
    "auditare": "auditar",
    "auditareis": "auditar",
    "auditaremos": "auditar",
    "auditaren": "auditar",
    "auditares": "auditar",
    "auditaron": "auditar",
    "auditará": "auditar",
    "auditarán": "auditar",
    "auditarás": "auditar",
    "auditaré": "auditar",
    "auditaréis": "auditar",
    "auditaría": "auditar",
    "auditaríais": "auditar",
    "auditaríamos": "auditar",
    "auditarían": "auditar",
    "auditarías": "auditar",
    "auditas": "auditar",
    "auditase": "auditar",
    "auditaseis": "auditar",
    "auditasen": "auditar",
    "auditases": "auditar",
    "auditaste": "auditar",
    "auditasteis": "auditar",
    "audite": "auditar",
    "auditemos": "auditar",
    "auditen": "auditar",
    "audites": "auditar",
    "auditiva": "auditivo",
    "auditivas": "auditivo",
    "auditivos": "auditivo",
    "audito": "auditar",
    "auditora": "auditor",
    "auditoras": "auditor",
    "auditores": "auditor",
    "auditoria": "auditorio",
    "auditorias": "auditorio",
    "auditorios": "auditorio",
    "auditorías": "auditoría",
    "auditábamos": "auditar",
    "auditáis": "auditar",
    "auditáramos": "auditar",
    "auditáremos": "auditar",
    "auditásemos": "auditar",
    "audité": "auditar",
    "auditéis": "auditar",
    "auditó": "auditar",
    "auditóriums": "auditórium",
    "audiófonos": "audiófono",
    "audiómetros": "audiómetro",
    "audífonos": "audífono",
    "audímetros": "audímetro",
    "augelitas": "augelita",
    "auges": "auge",
    "augitas": "augita",
    "augura": "augurar",
    "auguraba": "augurar",
    "augurabais": "augurar",
    "auguraban": "augurar",
    "augurabas": "augurar",
    "auguraciones": "auguración",
    "augurad": "augurar",
    "augurada": "augurar",
    "auguradas": "augurar",
    "augurado": "augurar",
    "auguradora": "augurador",
    "auguradoras": "augurador",
    "auguradores": "augurador",
    "augurados": "augurar",
    "augurales": "augural",
    "auguramos": "augurar",
    "auguran": "augurar",
    "augurando": "augurar",
    "augurara": "augurar",
    "augurarais": "augurar",
    "auguraran": "augurar",
    "auguraras": "augurar",
    "augurare": "augurar",
    "augurareis": "augurar",
    "auguraremos": "augurar",
    "auguraren": "augurar",
    "augurares": "augurar",
    "auguraron": "augurar",
    "augurará": "augurar",
    "augurarán": "augurar",
    "augurarás": "augurar",
    "auguraré": "augurar",
    "auguraréis": "augurar",
    "auguraría": "augurar",
    "auguraríais": "augurar",
    "auguraríamos": "augurar",
    "augurarían": "augurar",
    "augurarías": "augurar",
    "auguras": "augurar",
    "augurase": "augurar",
    "auguraseis": "augurar",
    "augurasen": "augurar",
    "augurases": "augurar",
    "auguraste": "augurar",
    "augurasteis": "augurar",
    "augure": "augurar",
    "auguremos": "augurar",
    "auguren": "augurar",
    "augures": "augurar",
    "augurios": "augurio",
    "auguro": "augurar",
    "augurábamos": "augurar",
    "auguráis": "augurar",
    "auguráramos": "augurar",
    "auguráremos": "augurar",
    "augurásemos": "augurar",
    "auguré": "augurar",
    "auguréis": "augurar",
    "auguró": "augurar",
    "augusta": "augusto",
    "augustales": "augustal",
    "augustas": "augusto",
    "augustos": "augusto",
    "aulagas": "aulaga",
    "aulas": "aula",
    "aullaba": "aullar",
    "aullabais": "aullar",
    "aullaban": "aullar",
    "aullabas": "aullar",
    "aullad": "aullar",
    "aullada": "aullar",
    "aulladas": "aullar",
    "aullado": "aullar",
    "aulladora": "aullador",
    "aulladoras": "aullador",
    "aulladores": "aullador",
    "aullados": "aullar",
    "aullamos": "aullar",
    "aullando": "aullar",
    "aullara": "aullar",
    "aullarais": "aullar",
    "aullaran": "aullar",
    "aullaras": "aullar",
    "aullare": "aullar",
    "aullareis": "aullar",
    "aullaremos": "aullar",
    "aullaren": "aullar",
    "aullares": "aullar",
    "aullaron": "aullar",
    "aullará": "aullar",
    "aullarán": "aullar",
    "aullarás": "aullar",
    "aullaré": "aullar",
    "aullaréis": "aullar",
    "aullaría": "aullar",
    "aullaríais": "aullar",
    "aullaríamos": "aullar",
    "aullarían": "aullar",
    "aullarías": "aullar",
    "aullase": "aullar",
    "aullaseis": "aullar",
    "aullasen": "aullar",
    "aullases": "aullar",
    "aullaste": "aullar",
    "aullasteis": "aullar",
    "aullemos": "aullar",
    "aullidos": "aullido",
    "aullábamos": "aullar",
    "aulláis": "aullar",
    "aulláramos": "aullar",
    "aulláremos": "aullar",
    "aullásemos": "aullar",
    "aullé": "aullar",
    "aulléis": "aullar",
    "aulló": "aullar",
    "aumenta": "aumentar",
    "aumentaba": "aumentar",
    "aumentabais": "aumentar",
    "aumentaban": "aumentar",
    "aumentabas": "aumentar",
    "aumentad": "aumentar",
    "aumentada": "aumentar",
    "aumentadas": "aumentar",
    "aumentado": "aumentar",
    "aumentadora": "aumentador",
    "aumentadoras": "aumentador",
    "aumentadores": "aumentador",
    "aumentados": "aumentar",
    "aumentamos": "aumentar",
    "aumentan": "aumentar",
    "aumentando": "aumentar",
    "aumentara": "aumentar",
    "aumentarais": "aumentar",
    "aumentaran": "aumentar",
    "aumentaras": "aumentar",
    "aumentare": "aumentar",
    "aumentareis": "aumentar",
    "aumentaremos": "aumentar",
    "aumentaren": "aumentar",
    "aumentares": "aumentar",
    "aumentaron": "aumentar",
    "aumentará": "aumentar",
    "aumentarán": "aumentar",
    "aumentarás": "aumentar",
    "aumentaré": "aumentar",
    "aumentaréis": "aumentar",
    "aumentaría": "aumentar",
    "aumentaríais": "aumentar",
    "aumentaríamos": "aumentar",
    "aumentarían": "aumentar",
    "aumentarías": "aumentar",
    "aumentas": "aumentar",
    "aumentase": "aumentar",
    "aumentaseis": "aumentar",
    "aumentasen": "aumentar",
    "aumentases": "aumentar",
    "aumentaste": "aumentar",
    "aumentasteis": "aumentar",
    "aumentativa": "aumentativo",
    "aumentativas": "aumentativo",
    "aumentativos": "aumentativo",
    "aumente": "aumentar",
    "aumentemos": "aumentar",
    "aumenten": "aumentar",
    "aumentes": "aumentar",
    "aumento": "aumentar",
    "aumentos": "aumento",
    "aumentábamos": "aumentar",
    "aumentáis": "aumentar",
    "aumentáramos": "aumentar",
    "aumentáremos": "aumentar",
    "aumentásemos": "aumentar",
    "aumenté": "aumentar",
    "aumentéis": "aumentar",
    "aumentó": "aumentar",
    "aunaba": "aunar",
    "aunabais": "aunar",
    "aunaban": "aunar",
    "aunabas": "aunar",
    "aunad": "aunar",
    "aunada": "aunar",
    "aunadas": "aunar",
    "aunado": "aunar",
    "aunados": "aunar",
    "aunamos": "aunar",
    "aunando": "aunar",
    "aunara": "aunar",
    "aunarais": "aunar",
    "aunaran": "aunar",
    "aunaras": "aunar",
    "aunare": "aunar",
    "aunareis": "aunar",
    "aunaremos": "aunar",
    "aunaren": "aunar",
    "aunares": "aunar",
    "aunaron": "aunar",
    "aunará": "aunar",
    "aunarán": "aunar",
    "aunarás": "aunar",
    "aunaré": "aunar",
    "aunaréis": "aunar",
    "aunaría": "aunar",
    "aunaríais": "aunar",
    "aunaríamos": "aunar",
    "aunarían": "aunar",
    "aunarías": "aunar",
    "aunase": "aunar",
    "aunaseis": "aunar",
    "aunasen": "aunar",
    "aunases": "aunar",
    "aunaste": "aunar",
    "aunasteis": "aunar",
    "aunemos": "aunar",
    "aunábamos": "aunar",
    "aunáis": "aunar",
    "aunáramos": "aunar",
    "aunáremos": "aunar",
    "aunásemos": "aunar",
    "auné": "aunar",
    "aunéis": "aunar",
    "aunó": "aunar",
    "aupaba": "aupar",
    "aupabais": "aupar",
    "aupaban": "aupar",
    "aupabas": "aupar",
    "aupad": "aupar",
    "aupada": "aupar",
    "aupadas": "aupar",
    "aupado": "aupar",
    "aupados": "aupar",
    "aupamos": "aupar",
    "aupando": "aupar",
    "aupara": "aupar",
    "auparais": "aupar",
    "auparan": "aupar",
    "auparas": "aupar",
    "aupare": "aupar",
    "aupareis": "aupar",
    "auparemos": "aupar",
    "auparen": "aupar",
    "aupares": "aupar",
    "auparon": "aupar",
    "aupará": "aupar",
    "auparán": "aupar",
    "auparás": "aupar",
    "auparé": "aupar",
    "auparéis": "aupar",
    "auparía": "aupar",
    "auparíais": "aupar",
    "auparíamos": "aupar",
    "auparían": "aupar",
    "auparías": "aupar",
    "aupase": "aupar",
    "aupaseis": "aupar",
    "aupasen": "aupar",
    "aupases": "aupar",
    "aupaste": "aupar",
    "aupasteis": "aupar",
    "aupemos": "aupar",
    "aupábamos": "aupar",
    "aupáis": "aupar",
    "aupáramos": "aupar",
    "aupáremos": "aupar",
    "aupásemos": "aupar",
    "aupé": "aupar",
    "aupéis": "aupar",
    "aupó": "aupar",
    "auquénidos": "auquénido",
    "aurales": "aural",
    "auraminas": "auramina",
    "auras": "aura",
    "aureola": "aureolar",
    "aureolaba": "aureolar",
    "aureolabais": "aureolar",
    "aureolaban": "aureolar",
    "aureolabas": "aureolar",
    "aureolad": "aureolar",
    "aureolada": "aureolar",
    "aureoladas": "aureolar",
    "aureolado": "aureolar",
    "aureolados": "aureolar",
    "aureolamos": "aureolar",
    "aureolan": "aureolar",
    "aureolando": "aureolar",
    "aureolara": "aureolar",
    "aureolarais": "aureolar",
    "aureolaran": "aureolar",
    "aureolaras": "aureolar",
    "aureolare": "aureolar",
    "aureolareis": "aureolar",
    "aureolaremos": "aureolar",
    "aureolaren": "aureolar",
    "aureolares": "aureolar",
    "aureolaron": "aureolar",
    "aureolará": "aureolar",
    "aureolarán": "aureolar",
    "aureolarás": "aureolar",
    "aureolaré": "aureolar",
    "aureolaréis": "aureolar",
    "aureolaría": "aureolar",
    "aureolaríais": "aureolar",
    "aureolaríamos": "aureolar",
    "aureolarían": "aureolar",
    "aureolarías": "aureolar",
    "aureolas": "aureolar",
    "aureolase": "aureolar",
    "aureolaseis": "aureolar",
    "aureolasen": "aureolar",
    "aureolases": "aureolar",
    "aureolaste": "aureolar",
    "aureolasteis": "aureolar",
    "aureole": "aureolar",
    "aureolemos": "aureolar",
    "aureolen": "aureolar",
    "aureoles": "aureolar",
    "aureolo": "aureolar",
    "aureolábamos": "aureolar",
    "aureoláis": "aureolar",
    "aureoláramos": "aureolar",
    "aureoláremos": "aureolar",
    "aureolásemos": "aureolar",
    "aureolé": "aureolar",
    "aureoléis": "aureolar",
    "aureoló": "aureolar",
    "auriculares": "auricular",
    "auriculocervicales": "auriculocervical",
    "auriculotemporales": "auriculotemporal",
    "auriculoventriculares": "auriculoventricular",
    "auriformes": "auriforme",
    "aurigas": "auriga",
    "aurinas": "aurina",
    "auriñacienses": "auriñaciense",
    "aurorales": "auroral",
    "auroras": "aurora",
    "aurosa": "auroso",
    "aurosas": "auroso",
    "aurosomas": "aurosoma",
    "aurosos": "auroso",
    "aurrescus": "aurrescu",
    "aurículas": "aurícula",
    "aurífera": "aurífero",
    "auríferas": "aurífero",
    "auríferos": "aurífero",
    "ausculta": "auscultar",
    "auscultaba": "auscultar",
    "auscultabais": "auscultar",
    "auscultaban": "auscultar",
    "auscultabas": "auscultar",
    "auscultaciones": "auscultación",
    "auscultad": "auscultar",
    "auscultada": "auscultar",
    "auscultadas": "auscultar",
    "auscultado": "auscultar",
    "auscultados": "auscultar",
    "auscultamientos": "auscultamiento",
    "auscultamos": "auscultar",
    "auscultan": "auscultar",
    "auscultando": "auscultar",
    "auscultara": "auscultar",
    "auscultarais": "auscultar",
    "auscultaran": "auscultar",
    "auscultaras": "auscultar",
    "auscultare": "auscultar",
    "auscultareis": "auscultar",
    "auscultaremos": "auscultar",
    "auscultaren": "auscultar",
    "auscultares": "auscultar",
    "auscultaron": "auscultar",
    "auscultará": "auscultar",
    "auscultarán": "auscultar",
    "auscultarás": "auscultar",
    "auscultaré": "auscultar",
    "auscultaréis": "auscultar",
    "auscultaría": "auscultar",
    "auscultaríais": "auscultar",
    "auscultaríamos": "auscultar",
    "auscultarían": "auscultar",
    "auscultarías": "auscultar",
    "auscultas": "auscultar",
    "auscultase": "auscultar",
    "auscultaseis": "auscultar",
    "auscultasen": "auscultar",
    "auscultases": "auscultar",
    "auscultaste": "auscultar",
    "auscultasteis": "auscultar",
    "ausculte": "auscultar",
    "auscultemos": "auscultar",
    "ausculten": "auscultar",
    "auscultes": "auscultar",
    "ausculto": "auscultar",
    "auscultábamos": "auscultar",
    "auscultáis": "auscultar",
    "auscultáramos": "auscultar",
    "auscultáremos": "auscultar",
    "auscultásemos": "auscultar",
    "ausculté": "auscultar",
    "auscultéis": "auscultar",
    "auscultó": "auscultar",
    "ausencias": "ausencia",
    "ausenta": "ausentar",
    "ausentaba": "ausentar",
    "ausentabais": "ausentar",
    "ausentaban": "ausentar",
    "ausentabas": "ausentar",
    "ausentad": "ausentar",
    "ausentada": "ausentar",
    "ausentadas": "ausentar",
    "ausentado": "ausentar",
    "ausentados": "ausentar",
    "ausentamos": "ausentar",
    "ausentan": "ausentar",
    "ausentando": "ausentar",
    "ausentara": "ausentar",
    "ausentarais": "ausentar",
    "ausentaran": "ausentar",
    "ausentaras": "ausentar",
    "ausentare": "ausentar",
    "ausentareis": "ausentar",
    "ausentaremos": "ausentar",
    "ausentaren": "ausentar",
    "ausentares": "ausentar",
    "ausentaron": "ausentar",
    "ausentará": "ausentar",
    "ausentarán": "ausentar",
    "ausentarás": "ausentar",
    "ausentaré": "ausentar",
    "ausentaréis": "ausentar",
    "ausentaría": "ausentar",
    "ausentaríais": "ausentar",
    "ausentaríamos": "ausentar",
    "ausentarían": "ausentar",
    "ausentarías": "ausentar",
    "ausentas": "ausentar",
    "ausentase": "ausentar",
    "ausentaseis": "ausentar",
    "ausentasen": "ausentar",
    "ausentases": "ausentar",
    "ausentaste": "ausentar",
    "ausentasteis": "ausentar",
    "ausente": "ausentar",
    "ausentemos": "ausentar",
    "ausenten": "ausentar",
    "ausentes": "ausente",
    "ausentismos": "ausentismo",
    "ausento": "ausentar",
    "ausentábamos": "ausentar",
    "ausentáis": "ausentar",
    "ausentáramos": "ausentar",
    "ausentáremos": "ausentar",
    "ausentásemos": "ausentar",
    "ausenté": "ausentar",
    "ausentéis": "ausentar",
    "ausentó": "ausentar",
    "auspicia": "auspiciar",
    "auspiciaba": "auspiciar",
    "auspiciabais": "auspiciar",
    "auspiciaban": "auspiciar",
    "auspiciabas": "auspiciar",
    "auspiciad": "auspiciar",
    "auspiciada": "auspiciar",
    "auspiciadas": "auspiciar",
    "auspiciado": "auspiciar",
    "auspiciados": "auspiciar",
    "auspiciamos": "auspiciar",
    "auspician": "auspiciar",
    "auspiciando": "auspiciar",
    "auspiciara": "auspiciar",
    "auspiciarais": "auspiciar",
    "auspiciaran": "auspiciar",
    "auspiciaras": "auspiciar",
    "auspiciare": "auspiciar",
    "auspiciareis": "auspiciar",
    "auspiciaremos": "auspiciar",
    "auspiciaren": "auspiciar",
    "auspiciares": "auspiciar",
    "auspiciaron": "auspiciar",
    "auspiciará": "auspiciar",
    "auspiciarán": "auspiciar",
    "auspiciarás": "auspiciar",
    "auspiciaré": "auspiciar",
    "auspiciaréis": "auspiciar",
    "auspiciaría": "auspiciar",
    "auspiciaríais": "auspiciar",
    "auspiciaríamos": "auspiciar",
    "auspiciarían": "auspiciar",
    "auspiciarías": "auspiciar",
    "auspicias": "auspiciar",
    "auspiciase": "auspiciar",
    "auspiciaseis": "auspiciar",
    "auspiciasen": "auspiciar",
    "auspiciases": "auspiciar",
    "auspiciaste": "auspiciar",
    "auspiciasteis": "auspiciar",
    "auspicie": "auspiciar",
    "auspiciemos": "auspiciar",
    "auspicien": "auspiciar",
    "auspicies": "auspiciar",
    "auspicio": "auspiciar",
    "auspicios": "auspicio",
    "auspiciosa": "auspicioso",
    "auspiciosas": "auspicioso",
    "auspiciosos": "auspicioso",
    "auspiciábamos": "auspiciar",
    "auspiciáis": "auspiciar",
    "auspiciáramos": "auspiciar",
    "auspiciáremos": "auspiciar",
    "auspiciásemos": "auspiciar",
    "auspicié": "auspiciar",
    "auspiciéis": "auspiciar",
    "auspició": "auspiciar",
    "austemples": "austemple",
    "austeniconformada": "austeniconformado",
    "austeniconformadas": "austeniconformado",
    "austeniconformados": "austeniconformado",
    "austenitas": "austenita",
    "austenitizaciones": "austenitización",
    "austenítica": "austenítico",
    "austeníticas": "austenítico",
    "austeníticos": "austenítico",
    "austera": "austero",
    "austeras": "austero",
    "austeridades": "austeridad",
    "austeros": "austero",
    "australes": "austral",
    "australiana": "australiano",
    "australianas": "australiano",
    "australianos": "australiano",
    "australitas": "australita",
    "australoides": "australoide",
    "australopitécidos": "australopitécido",
    "austriaca": "austriaco",
    "austriacas": "austriaco",
    "austriacos": "austriaco",
    "austrohúngara": "austrohúngaro",
    "austrohúngaras": "austrohúngaro",
    "austrohúngaros": "austrohúngaro",
    "austromarxismos": "austromarxismo",
    "austros": "austro",
    "austríaca": "austríaco",
    "austríacas": "austríaco",
    "austríacos": "austríaco",
    "autarquías": "autarquía",
    "autentica": "autenticar",
    "autenticaba": "autenticar",
    "autenticabais": "autenticar",
    "autenticaban": "autenticar",
    "autenticabas": "autenticar",
    "autenticaciones": "autenticación",
    "autenticad": "autenticar",
    "autenticada": "autenticar",
    "autenticadas": "autenticar",
    "autenticado": "autenticar",
    "autenticados": "autenticar",
    "autenticamos": "autenticar",
    "autentican": "autenticar",
    "autenticando": "autenticar",
    "autenticara": "autenticar",
    "autenticarais": "autenticar",
    "autenticaran": "autenticar",
    "autenticaras": "autenticar",
    "autenticare": "autenticar",
    "autenticareis": "autenticar",
    "autenticaremos": "autenticar",
    "autenticaren": "autenticar",
    "autenticares": "autenticar",
    "autenticaron": "autenticar",
    "autenticará": "autenticar",
    "autenticarán": "autenticar",
    "autenticarás": "autenticar",
    "autenticaré": "autenticar",
    "autenticaréis": "autenticar",
    "autenticaría": "autenticar",
    "autenticaríais": "autenticar",
    "autenticaríamos": "autenticar",
    "autenticarían": "autenticar",
    "autenticarías": "autenticar",
    "autenticas": "autenticar",
    "autenticase": "autenticar",
    "autenticaseis": "autenticar",
    "autenticasen": "autenticar",
    "autenticases": "autenticar",
    "autenticaste": "autenticar",
    "autenticasteis": "autenticar",
    "autenticidades": "autenticidad",
    "autentico": "autenticar",
    "autenticábamos": "autenticar",
    "autenticáis": "autenticar",
    "autenticáramos": "autenticar",
    "autenticáremos": "autenticar",
    "autenticásemos": "autenticar",
    "autenticó": "autenticar",
    "autentifica": "autentificar",
    "autentificaba": "autentificar",
    "autentificabais": "autentificar",
    "autentificaban": "autentificar",
    "autentificabas": "autentificar",
    "autentificaciones": "autentificación",
    "autentificad": "autentificar",
    "autentificada": "autentificar",
    "autentificadas": "autentificar",
    "autentificado": "autentificar",
    "autentificados": "autentificar",
    "autentificamos": "autentificar",
    "autentifican": "autentificar",
    "autentificando": "autentificar",
    "autentificara": "autentificar",
    "autentificarais": "autentificar",
    "autentificaran": "autentificar",
    "autentificaras": "autentificar",
    "autentificare": "autentificar",
    "autentificareis": "autentificar",
    "autentificaremos": "autentificar",
    "autentificaren": "autentificar",
    "autentificares": "autentificar",
    "autentificaron": "autentificar",
    "autentificará": "autentificar",
    "autentificarán": "autentificar",
    "autentificarás": "autentificar",
    "autentificaré": "autentificar",
    "autentificaréis": "autentificar",
    "autentificaría": "autentificar",
    "autentificaríais": "autentificar",
    "autentificaríamos": "autentificar",
    "autentificarían": "autentificar",
    "autentificarías": "autentificar",
    "autentificas": "autentificar",
    "autentificase": "autentificar",
    "autentificaseis": "autentificar",
    "autentificasen": "autentificar",
    "autentificases": "autentificar",
    "autentificaste": "autentificar",
    "autentificasteis": "autentificar",
    "autentifico": "autentificar",
    "autentificábamos": "autentificar",
    "autentificáis": "autentificar",
    "autentificáramos": "autentificar",
    "autentificáremos": "autentificar",
    "autentificásemos": "autentificar",
    "autentificó": "autentificar",
    "autentifique": "autentificar",
    "autentifiquemos": "autentificar",
    "autentifiquen": "autentificar",
    "autentifiques": "autentificar",
    "autentifiqué": "autentificar",
    "autentifiquéis": "autentificar",
    "autentique": "autenticar",
    "autentiquemos": "autenticar",
    "autentiquen": "autenticar",
    "autentiques": "autenticar",
    "autentiqué": "autenticar",
    "autentiquéis": "autenticar",
    "autillos": "autillo",
    "autismos": "autismo",
    "autistas": "autista",
    "auto-regulaciones": "auto-regulación",
    "autoabastecimientos": "autoabastecimiento",
    "autoacusaciones": "autoacusación",
    "autoadaptables": "autoadaptable",
    "autoadaptaciones": "autoadaptación",
    "autoadaptadora": "autoadaptador",
    "autoadaptadoras": "autoadaptador",
    "autoadaptadores": "autoadaptador",
    "autoadhesiva": "autoadhesivo",
    "autoadhesivas": "autoadhesivo",
    "autoadhesivos": "autoadhesivo",
    "autoafirma": "autoafirmar",
    "autoafirmaba": "autoafirmar",
    "autoafirmabais": "autoafirmar",
    "autoafirmaban": "autoafirmar",
    "autoafirmabas": "autoafirmar",
    "autoafirmaciones": "autoafirmación",
    "autoafirmad": "autoafirmar",
    "autoafirmada": "autoafirmar",
    "autoafirmadas": "autoafirmar",
    "autoafirmado": "autoafirmar",
    "autoafirmados": "autoafirmar",
    "autoafirmamos": "autoafirmar",
    "autoafirman": "autoafirmar",
    "autoafirmando": "autoafirmar",
    "autoafirmara": "autoafirmar",
    "autoafirmarais": "autoafirmar",
    "autoafirmaran": "autoafirmar",
    "autoafirmaras": "autoafirmar",
    "autoafirmare": "autoafirmar",
    "autoafirmareis": "autoafirmar",
    "autoafirmaremos": "autoafirmar",
    "autoafirmaren": "autoafirmar",
    "autoafirmares": "autoafirmar",
    "autoafirmaron": "autoafirmar",
    "autoafirmará": "autoafirmar",
    "autoafirmarán": "autoafirmar",
    "autoafirmarás": "autoafirmar",
    "autoafirmaré": "autoafirmar",
    "autoafirmaréis": "autoafirmar",
    "autoafirmaría": "autoafirmar",
    "autoafirmaríais": "autoafirmar",
    "autoafirmaríamos": "autoafirmar",
    "autoafirmarían": "autoafirmar",
    "autoafirmarías": "autoafirmar",
    "autoafirmas": "autoafirmar",
    "autoafirmase": "autoafirmar",
    "autoafirmaseis": "autoafirmar",
    "autoafirmasen": "autoafirmar",
    "autoafirmases": "autoafirmar",
    "autoafirmaste": "autoafirmar",
    "autoafirmasteis": "autoafirmar",
    "autoafirme": "autoafirmar",
    "autoafirmemos": "autoafirmar",
    "autoafirmen": "autoafirmar",
    "autoafirmes": "autoafirmar",
    "autoafirmo": "autoafirmar",
    "autoafirmábamos": "autoafirmar",
    "autoafirmáis": "autoafirmar",
    "autoafirmáramos": "autoafirmar",
    "autoafirmáremos": "autoafirmar",
    "autoafirmásemos": "autoafirmar",
    "autoafirmé": "autoafirmar",
    "autoafirméis": "autoafirmar",
    "autoafirmó": "autoafirmar",
    "autoaglutinaciones": "autoaglutinación",
    "autoaglutininas": "autoaglutinina",
    "autoajustadora": "autoajustador",
    "autoajustadoras": "autoajustador",
    "autoajustadores": "autoajustador",
    "autoalimentadora": "autoalimentador",
    "autoalimentadoras": "autoalimentador",
    "autoalimentadores": "autoalimentador",
    "autoanalizadores": "autoanalizador",
    "autoanticuerpos": "autoanticuerpo",
    "autoantígenos": "autoantígeno",
    "autoaprendizajes": "autoaprendizaje",
    "autoarrancadora": "autoarrancador",
    "autoarrancadoras": "autoarrancador",
    "autoarrancadores": "autoarrancador",
    "autoasistencias": "autoasistencia",
    "autoayudas": "autoayuda",
    "autobiografías": "autobiografía",
    "autobiográfica": "autobiográfico",
    "autobiográficas": "autobiográfico",
    "autobiográficos": "autobiográfico",
    "autobiógrafa": "autobiógrafo",
    "autobiógrafas": "autobiógrafo",
    "autobiógrafos": "autobiógrafo",
    "autobombas": "autobomba",
    "autobombos": "autobombo",
    "autobuses": "autobús",
    "autocalentamientos": "autocalentamiento",
    "autocalibradora": "autocalibrador",
    "autocalibradoras": "autocalibrador",
    "autocalibradores": "autocalibrador",
    "autocamiones": "autocamión",
    "autocares": "autocar",
    "autocargables": "autocargable",
    "autocastigos": "autocastigo",
    "autocatalítica": "autocatalítico",
    "autocatalíticas": "autocatalítico",
    "autocatalíticos": "autocatalítico",
    "autocentradora": "autocentrador",
    "autocentradoras": "autocentrador",
    "autocentradores": "autocentrador",
    "autocentrantes": "autocentrante",
    "autocerradora": "autocerrador",
    "autocerradoras": "autocerrador",
    "autocerradores": "autocerrador",
    "autochoques": "autochoque",
    "autocines": "autocine",
    "autoclaves": "autoclave",
    "autocombustiones": "autocombustión",
    "autocompensadora": "autocompensador",
    "autocompensadoras": "autocompensador",
    "autocompensadores": "autocompensador",
    "autoconciencias": "autoconciencia",
    "autoconductora": "autoconductor",
    "autoconductoras": "autoconductor",
    "autoconductores": "autoconductor",
    "autoconfianzas": "autoconfianza",
    "autoconfiguraciones": "autoconfiguración",
    "autoconfiguradores": "autoconfigurador",
    "autoconsumos": "autoconsumo",
    "autocontendremos": "autocontener",
    "autocontendrá": "autocontener",
    "autocontendrán": "autocontener",
    "autocontendrás": "autocontener",
    "autocontendré": "autocontener",
    "autocontendréis": "autocontener",
    "autocontendría": "autocontener",
    "autocontendríais": "autocontener",
    "autocontendríamos": "autocontener",
    "autocontendrían": "autocontener",
    "autocontendrías": "autocontener",
    "autocontened": "autocontener",
    "autocontenemos": "autocontener",
    "autocontenga": "autocontener",
    "autocontengamos": "autocontener",
    "autocontengan": "autocontener",
    "autocontengas": "autocontener",
    "autocontengo": "autocontener",
    "autocontengáis": "autocontener",
    "autocontenida": "autocontener",
    "autocontenidas": "autocontener",
    "autocontenido": "autocontener",
    "autocontenidos": "autocontener",
    "autoconteniendo": "autocontener",
    "autocontenéis": "autocontener",
    "autocontenía": "autocontener",
    "autoconteníais": "autocontener",
    "autoconteníamos": "autocontener",
    "autocontenían": "autocontener",
    "autocontenías": "autocontener",
    "autocontiene": "autocontener",
    "autocontienen": "autocontener",
    "autocontienes": "autocontener",
    "autocontrolada": "autocontrolado",
    "autocontroladas": "autocontrolado",
    "autocontrolados": "autocontrolado",
    "autocontroles": "autocontrol",
    "autocontuve": "autocontener",
    "autocontuviera": "autocontener",
    "autocontuvierais": "autocontener",
    "autocontuvieran": "autocontener",
    "autocontuvieras": "autocontener",
    "autocontuviere": "autocontener",
    "autocontuviereis": "autocontener",
    "autocontuvieren": "autocontener",
    "autocontuvieres": "autocontener",
    "autocontuvieron": "autocontener",
    "autocontuviese": "autocontener",
    "autocontuvieseis": "autocontener",
    "autocontuviesen": "autocontener",
    "autocontuvieses": "autocontener",
    "autocontuvimos": "autocontener",
    "autocontuviste": "autocontener",
    "autocontuvisteis": "autocontener",
    "autocontuviéramos": "autocontener",
    "autocontuviéremos": "autocontener",
    "autocontuviésemos": "autocontener",
    "autocontuvo": "autocontener",
    "autocontén": "autocontener",
    "autocopistas": "autocopista",
    "autocorrecciones": "autocorrección",
    "autocorrelaciones": "autocorrelación",
    "autocorreladores": "autocorrelador",
    "autocracias": "autocracia",
    "autocrática": "autocrático",
    "autocráticas": "autocrático",
    "autocráticos": "autocrático",
    "autocríticas": "autocrítica",
    "autoctonías": "autoctonía",
    "autodefensas": "autodefensa",
    "autodefina": "autodefinir",
    "autodefinamos": "autodefinir",
    "autodefinan": "autodefinir",
    "autodefinas": "autodefinir",
    "autodefine": "autodefinir",
    "autodefinen": "autodefinir",
    "autodefines": "autodefinir",
    "autodefinibles": "autodefinible",
    "autodefinid": "autodefinir",
    "autodefinida": "autodefinir",
    "autodefinidas": "autodefinir",
    "autodefinido": "autodefinir",
    "autodefinidos": "autodefinir",
    "autodefiniendo": "autodefinir",
    "autodefiniera": "autodefinir",
    "autodefinierais": "autodefinir",
    "autodefinieran": "autodefinir",
    "autodefinieras": "autodefinir",
    "autodefiniere": "autodefinir",
    "autodefiniereis": "autodefinir",
    "autodefinieren": "autodefinir",
    "autodefinieres": "autodefinir",
    "autodefinieron": "autodefinir",
    "autodefiniese": "autodefinir",
    "autodefinieseis": "autodefinir",
    "autodefiniesen": "autodefinir",
    "autodefinieses": "autodefinir",
    "autodefinimos": "autodefinir",
    "autodefiniremos": "autodefinir",
    "autodefinirá": "autodefinir",
    "autodefinirán": "autodefinir",
    "autodefinirás": "autodefinir",
    "autodefiniré": "autodefinir",
    "autodefiniréis": "autodefinir",
    "autodefiniría": "autodefinir",
    "autodefiniríais": "autodefinir",
    "autodefiniríamos": "autodefinir",
    "autodefinirían": "autodefinir",
    "autodefinirías": "autodefinir",
    "autodefiniste": "autodefinir",
    "autodefinisteis": "autodefinir",
    "autodefinitoria": "autodefinitorio",
    "autodefinitorias": "autodefinitorio",
    "autodefinitorios": "autodefinitorio",
    "autodefiniéramos": "autodefinir",
    "autodefiniéremos": "autodefinir",
    "autodefiniésemos": "autodefinir",
    "autodefinió": "autodefinir",
    "autodefino": "autodefinir",
    "autodefináis": "autodefinir",
    "autodefiní": "autodefinir",
    "autodefinía": "autodefinir",
    "autodefiníais": "autodefinir",
    "autodefiníamos": "autodefinir",
    "autodefinían": "autodefinir",
    "autodefinías": "autodefinir",
    "autodefinís": "autodefinir",
    "autodegradaciones": "autodegradación",
    "autodepuraciones": "autodepuración",
    "autodesarrollos": "autodesarrollo",
    "autodescargables": "autodescargable",
    "autodestrucciones": "autodestrucción",
    "autodestructibles": "autodestructible",
    "autodestructiva": "autodestructivo",
    "autodestructivas": "autodestructivo",
    "autodestructivos": "autodestructivo",
    "autodeterminaciones": "autodeterminación",
    "autodiagnósticos": "autodiagnóstico",
    "autodidactas": "autodidacta",
    "autodidactos": "autodidacto",
    "autodidáctica": "autodidáctico",
    "autodidácticas": "autodidáctico",
    "autodidácticos": "autodidáctico",
    "autodifusiones": "autodifusión",
    "autodina": "autodino",
    "autodinas": "autodino",
    "autodinos": "autodino",
    "autodinámica": "autodinámico",
    "autodinámicas": "autodinámico",
    "autodinámicos": "autodinámico",
    "autodirecciones": "autodirección",
    "autodisciplinas": "autodisciplina",
    "autodocumentada": "autodocumentado",
    "autodocumentadas": "autodocumentado",
    "autodocumentados": "autodocumentado",
    "autodominios": "autodominio",
    "autoecologías": "autoecología",
    "autoeducaciones": "autoeducación",
    "autoempleos": "autoempleo",
    "autoencendidos": "autoencendido",
    "autoengaños": "autoengaño",
    "autoequilibradora": "autoequilibrador",
    "autoequilibradoras": "autoequilibrador",
    "autoequilibradores": "autoequilibrador",
    "autoescuelas": "autoescuela",
    "autoestables": "autoestable",
    "autoestopistas": "autoestopista",
    "autoevaluaciones": "autoevaluación",
    "autoexcitaciones": "autoexcitación",
    "autoexplicativa": "autoexplicativo",
    "autoexplicativas": "autoexplicativo",
    "autoexplicativos": "autoexplicativo",
    "autoextintora": "autoextintor",
    "autoextintoras": "autoextintor",
    "autoextintores": "autoextintor",
    "autofagias": "autofagia",
    "autofagosomas": "autofagosoma",
    "autofinanciaciones": "autofinanciación",
    "autofinanciada": "autofinanciado",
    "autofinanciadas": "autofinanciado",
    "autofinanciados": "autofinanciado",
    "autofinanciamientos": "autofinanciamiento",
    "autofonías": "autofonía",
    "autoformaciones": "autoformación",
    "autofunciones": "autofunción",
    "autofundentes": "autofundente",
    "autofágica": "autofágico",
    "autofágicas": "autofágico",
    "autofágicos": "autofágico",
    "autogamias": "autogamia",
    "autogestionada": "autogestionado",
    "autogestionadas": "autogestionado",
    "autogestionados": "autogestionado",
    "autogestionaria": "autogestionario",
    "autogestionarias": "autogestionario",
    "autogestionarios": "autogestionario",
    "autogestiones": "autogestión",
    "autogiros": "autogiro",
    "autogobiernos": "autogobierno",
    "autogoles": "autogol",
    "autogolpes": "autogolpe",
    "autografiaba": "autografiar",
    "autografiabais": "autografiar",
    "autografiaban": "autografiar",
    "autografiabas": "autografiar",
    "autografiad": "autografiar",
    "autografiada": "autografiar",
    "autografiadas": "autografiar",
    "autografiado": "autografiar",
    "autografiados": "autografiar",
    "autografiamos": "autografiar",
    "autografiando": "autografiar",
    "autografiara": "autografiar",
    "autografiarais": "autografiar",
    "autografiaran": "autografiar",
    "autografiaras": "autografiar",
    "autografiare": "autografiar",
    "autografiareis": "autografiar",
    "autografiaremos": "autografiar",
    "autografiaren": "autografiar",
    "autografiares": "autografiar",
    "autografiaron": "autografiar",
    "autografiará": "autografiar",
    "autografiarán": "autografiar",
    "autografiarás": "autografiar",
    "autografiaré": "autografiar",
    "autografiaréis": "autografiar",
    "autografiaría": "autografiar",
    "autografiaríais": "autografiar",
    "autografiaríamos": "autografiar",
    "autografiarían": "autografiar",
    "autografiarías": "autografiar",
    "autografiase": "autografiar",
    "autografiaseis": "autografiar",
    "autografiasen": "autografiar",
    "autografiases": "autografiar",
    "autografiaste": "autografiar",
    "autografiasteis": "autografiar",
    "autografiemos": "autografiar",
    "autografiábamos": "autografiar",
    "autografiáis": "autografiar",
    "autografiáramos": "autografiar",
    "autografiáremos": "autografiar",
    "autografiásemos": "autografiar",
    "autografié": "autografiar",
    "autografiéis": "autografiar",
    "autografió": "autografiar",
    "autografía": "autografiar",
    "autografían": "autografiar",
    "autografías": "autografiar",
    "autografíe": "autografiar",
    "autografíen": "autografiar",
    "autografíes": "autografiar",
    "autografío": "autografiar",
    "autográfica": "autográfico",
    "autográficas": "autográfico",
    "autográficos": "autográfico",
    "autoguiada": "autoguiado",
    "autoguiadas": "autoguiado",
    "autoguiados": "autoguiado",
    "autohemoterapias": "autohemoterapia",
    "autoheterodina": "autoheterodino",
    "autoheterodinas": "autoheterodino",
    "autoheterodinos": "autoheterodino",
    "autoigualadora": "autoigualador",
    "autoigualadoras": "autoigualador",
    "autoigualadores": "autoigualador",
    "autoinducciones": "autoinducción",
    "autoinductancias": "autoinductancia",
    "autoinductiva": "autoinductivo",
    "autoinductivas": "autoinductivo",
    "autoinductivos": "autoinductivo",
    "autoinflamaciones": "autoinflamación",
    "autoinjertos": "autoinjerto",
    "autoinmunes": "autoinmune",
    "autoinmunidades": "autoinmunidad",
    "autoinmunitaria": "autoinmunitario",
    "autoinmunitarias": "autoinmunitario",
    "autoinmunitarios": "autoinmunitario",
    "autoinmunizaciones": "autoinmunización",
    "autoinstrucciones": "autoinstrucción",
    "autoliquidaciones": "autoliquidación",
    "autolisosomas": "autolisosoma",
    "autolubricantes": "autolubricante",
    "autolubrificaciones": "autolubrificación",
    "automaciones": "automación",
    "automatice": "automatizar",
    "automaticemos": "automatizar",
    "automaticen": "automatizar",
    "automatices": "automatizar",
    "automaticidades": "automaticidad",
    "automaticé": "automatizar",
    "automaticéis": "automatizar",
    "automatismos": "automatismo",
    "automatiza": "automatizar",
    "automatizaba": "automatizar",
    "automatizabais": "automatizar",
    "automatizaban": "automatizar",
    "automatizabas": "automatizar",
    "automatizables": "automatizable",
    "automatizaciones": "automatización",
    "automatizad": "automatizar",
    "automatizada": "automatizar",
    "automatizadas": "automatizar",
    "automatizado": "automatizar",
    "automatizados": "automatizar",
    "automatizamos": "automatizar",
    "automatizan": "automatizar",
    "automatizando": "automatizar",
    "automatizara": "automatizar",
    "automatizarais": "automatizar",
    "automatizaran": "automatizar",
    "automatizaras": "automatizar",
    "automatizare": "automatizar",
    "automatizareis": "automatizar",
    "automatizaremos": "automatizar",
    "automatizaren": "automatizar",
    "automatizares": "automatizar",
    "automatizaron": "automatizar",
    "automatizará": "automatizar",
    "automatizarán": "automatizar",
    "automatizarás": "automatizar",
    "automatizaré": "automatizar",
    "automatizaréis": "automatizar",
    "automatizaría": "automatizar",
    "automatizaríais": "automatizar",
    "automatizaríamos": "automatizar",
    "automatizarían": "automatizar",
    "automatizarías": "automatizar",
    "automatizas": "automatizar",
    "automatizase": "automatizar",
    "automatizaseis": "automatizar",
    "automatizasen": "automatizar",
    "automatizases": "automatizar",
    "automatizaste": "automatizar",
    "automatizasteis": "automatizar",
    "automatizo": "automatizar",
    "automatizábamos": "automatizar",
    "automatizáis": "automatizar",
    "automatizáramos": "automatizar",
    "automatizáremos": "automatizar",
    "automatizásemos": "automatizar",
    "automatizó": "automatizar",
    "automociones": "automoción",
    "automorfismos": "automorfismo",
    "automotora": "automotor",
    "automotoras": "automotor",
    "automotores": "automotor",
    "automotrices": "automotriz",
    "automovilismos": "automovilismo",
    "automovilistas": "automovilista",
    "automovilística": "automovilístico",
    "automovilísticas": "automovilístico",
    "automovilísticos": "automovilístico",
    "automática": "automático",
    "automáticas": "automático",
    "automáticos": "automático",
    "automóviles": "automóvil",
    "autoniveladoras": "autoniveladora",
    "autonomice": "autonomizar",
    "autonomicemos": "autonomizar",
    "autonomicen": "autonomizar",
    "autonomices": "autonomizar",
    "autonomicé": "autonomizar",
    "autonomicéis": "autonomizar",
    "autonomismos": "autonomismo",
    "autonomistas": "autonomista",
    "autonomiza": "autonomizar",
    "autonomizaba": "autonomizar",
    "autonomizabais": "autonomizar",
    "autonomizaban": "autonomizar",
    "autonomizabas": "autonomizar",
    "autonomizad": "autonomizar",
    "autonomizada": "autonomizar",
    "autonomizadas": "autonomizar",
    "autonomizado": "autonomizar",
    "autonomizados": "autonomizar",
    "autonomizamos": "autonomizar",
    "autonomizan": "autonomizar",
    "autonomizando": "autonomizar",
    "autonomizara": "autonomizar",
    "autonomizarais": "autonomizar",
    "autonomizaran": "autonomizar",
    "autonomizaras": "autonomizar",
    "autonomizare": "autonomizar",
    "autonomizareis": "autonomizar",
    "autonomizaremos": "autonomizar",
    "autonomizaren": "autonomizar",
    "autonomizares": "autonomizar",
    "autonomizaron": "autonomizar",
    "autonomizará": "autonomizar",
    "autonomizarán": "autonomizar",
    "autonomizarás": "autonomizar",
    "autonomizaré": "autonomizar",
    "autonomizaréis": "autonomizar",
    "autonomizaría": "autonomizar",
    "autonomizaríais": "autonomizar",
    "autonomizaríamos": "autonomizar",
    "autonomizarían": "autonomizar",
    "autonomizarías": "autonomizar",
    "autonomizas": "autonomizar",
    "autonomizase": "autonomizar",
    "autonomizaseis": "autonomizar",
    "autonomizasen": "autonomizar",
    "autonomizases": "autonomizar",
    "autonomizaste": "autonomizar",
    "autonomizasteis": "autonomizar",
    "autonomizo": "autonomizar",
    "autonomizábamos": "autonomizar",
    "autonomizáis": "autonomizar",
    "autonomizáramos": "autonomizar",
    "autonomizáremos": "autonomizar",
    "autonomizásemos": "autonomizar",
    "autonomizó": "autonomizar",
    "autonomías": "autonomía",
    "autonómica": "autonómico",
    "autonómicas": "autonómico",
    "autonómicos": "autonómico",
    "autooscilaciones": "autooscilación",
    "autoosciladora": "autooscilador",
    "autoosciladoras": "autooscilador",
    "autoosciladores": "autooscilador",
    "autooxidaciones": "autooxidación",
    "autopases": "autopase",
    "autopistas": "autopista",
    "autoplastias": "autoplastia",
    "autopolarizada": "autopolarizado",
    "autopolarizadas": "autopolarizado",
    "autopolarizados": "autopolarizado",
    "autopolimerizaciones": "autopolimerización",
    "autoproducciones": "autoproducción",
    "autoproductora": "autoproductor",
    "autoproductoras": "autoproductor",
    "autoproductores": "autoproductor",
    "autoprogramables": "autoprogramable",
    "autopropulsada": "autopropulsado",
    "autopropulsadas": "autopropulsado",
    "autopropulsados": "autopropulsado",
    "autopropulsiones": "autopropulsión",
    "autopropulsora": "autopropulsor",
    "autopropulsoras": "autopropulsor",
    "autopropulsores": "autopropulsor",
    "autoprotecciones": "autoprotección",
    "autopsias": "autopsia",
    "autopullmans": "autopullman",
    "autora": "autor",
    "autoras": "autor",
    "autoregresiones": "autoregresión",
    "autoregulaciones": "autoregulación",
    "autores": "autor",
    "autorice": "autorizar",
    "autoricemos": "autorizar",
    "autoricen": "autorizar",
    "autorices": "autorizar",
    "autoricé": "autorizar",
    "autoricéis": "autorizar",
    "autoridades": "autoridad",
    "autoritaria": "autoritario",
    "autoritarias": "autoritario",
    "autoritarios": "autoritario",
    "autoritarismos": "autoritarismo",
    "autoriza": "autorizar",
    "autorizaba": "autorizar",
    "autorizabais": "autorizar",
    "autorizaban": "autorizar",
    "autorizabas": "autorizar",
    "autorizaciones": "autorización",
    "autorizad": "autorizar",
    "autorizada": "autorizar",
    "autorizadas": "autorizar",
    "autorizado": "autorizar",
    "autorizadora": "autorizador",
    "autorizadoras": "autorizador",
    "autorizadores": "autorizador",
    "autorizados": "autorizar",
    "autorizamientos": "autorizamiento",
    "autorizamos": "autorizar",
    "autorizan": "autorizar",
    "autorizando": "autorizar",
    "autorizara": "autorizar",
    "autorizarais": "autorizar",
    "autorizaran": "autorizar",
    "autorizaras": "autorizar",
    "autorizare": "autorizar",
    "autorizareis": "autorizar",
    "autorizaremos": "autorizar",
    "autorizaren": "autorizar",
    "autorizares": "autorizar",
    "autorizaron": "autorizar",
    "autorizará": "autorizar",
    "autorizarán": "autorizar",
    "autorizarás": "autorizar",
    "autorizaré": "autorizar",
    "autorizaréis": "autorizar",
    "autorizaría": "autorizar",
    "autorizaríais": "autorizar",
    "autorizaríamos": "autorizar",
    "autorizarían": "autorizar",
    "autorizarías": "autorizar",
    "autorizas": "autorizar",
    "autorizase": "autorizar",
    "autorizaseis": "autorizar",
    "autorizasen": "autorizar",
    "autorizases": "autorizar",
    "autorizaste": "autorizar",
    "autorizasteis": "autorizar",
    "autorizo": "autorizar",
    "autorizábamos": "autorizar",
    "autorizáis": "autorizar",
    "autorizáramos": "autorizar",
    "autorizáremos": "autorizar",
    "autorizásemos": "autorizar",
    "autorizó": "autorizar",
    "autorradiografías": "autorradiografía",
    "autorradios": "autorradio",
    "autorreactiva": "autorreactivo",
    "autorreactivas": "autorreactivo",
    "autorreactivos": "autorreactivo",
    "autorrectificadora": "autorrectificador",
    "autorrectificadoras": "autorrectificador",
    "autorrectificadores": "autorrectificador",
    "autorrefrigerada": "autorrefrigerado",
    "autorrefrigeradas": "autorrefrigerado",
    "autorrefrigerados": "autorrefrigerado",
    "autorregeneradora": "autorregenerador",
    "autorregeneradoras": "autorregenerador",
    "autorregeneradores": "autorregenerador",
    "autorregistradora": "autorregistrador",
    "autorregistradoras": "autorregistrador",
    "autorregistradores": "autorregistrador",
    "autorregresiva": "autorregresivo",
    "autorregresivas": "autorregresivo",
    "autorregresivos": "autorregresivo",
    "autorregula": "autorregular",
    "autorregulaba": "autorregular",
    "autorregulabais": "autorregular",
    "autorregulaban": "autorregular",
    "autorregulabas": "autorregular",
    "autorregulaciones": "autorregulación",
    "autorregulad": "autorregular",
    "autorregulada": "autorregular",
    "autorreguladas": "autorregular",
    "autorregulado": "autorregular",
    "autorreguladora": "autorregulador",
    "autorreguladoras": "autorregulador",
    "autorreguladores": "autorregulador",
    "autorregulados": "autorregular",
    "autorregulamos": "autorregular",
    "autorregulan": "autorregular",
    "autorregulando": "autorregular",
    "autorregulara": "autorregular",
    "autorregularais": "autorregular",
    "autorregularan": "autorregular",
    "autorregularas": "autorregular",
    "autorregulare": "autorregular",
    "autorregulareis": "autorregular",
    "autorregularemos": "autorregular",
    "autorregularen": "autorregular",
    "autorregulares": "autorregular",
    "autorregularon": "autorregular",
    "autorregulará": "autorregular",
    "autorregularán": "autorregular",
    "autorregularás": "autorregular",
    "autorregularé": "autorregular",
    "autorregularéis": "autorregular",
    "autorregularía": "autorregular",
    "autorregularíais": "autorregular",
    "autorregularíamos": "autorregular",
    "autorregularían": "autorregular",
    "autorregularías": "autorregular",
    "autorregulas": "autorregular",
    "autorregulase": "autorregular",
    "autorregulaseis": "autorregular",
    "autorregulasen": "autorregular",
    "autorregulases": "autorregular",
    "autorregulaste": "autorregular",
    "autorregulasteis": "autorregular",
    "autorregulatoria": "autorregulatorio",
    "autorregulatorias": "autorregulatorio",
    "autorregulatorios": "autorregulatorio",
    "autorregule": "autorregular",
    "autorregulemos": "autorregular",
    "autorregulen": "autorregular",
    "autorregules": "autorregular",
    "autorregulo": "autorregular",
    "autorregulábamos": "autorregular",
    "autorreguláis": "autorregular",
    "autorreguláramos": "autorregular",
    "autorreguláremos": "autorregular",
    "autorregulásemos": "autorregular",
    "autorregulé": "autorregular",
    "autorreguléis": "autorregular",
    "autorreguló": "autorregular",
    "autorretenciones": "autorretención",
    "autorretenedora": "autorretenedor",
    "autorretenedoras": "autorretenedor",
    "autorretenedores": "autorretenedor",
    "autorretratos": "autorretrato",
    "autorías": "autoría",
    "autos": "auto",
    "autosatisfacciones": "autosatisfacción",
    "autosaturaciones": "autosaturación",
    "autoseguros": "autoseguro",
    "autoservicios": "autoservicio",
    "autosomas": "autosoma",
    "autostopistas": "autostopista",
    "autosucesiones": "autosucesión",
    "autosuficiencias": "autosuficiencia",
    "autosuficientes": "autosuficiente",
    "autosugestiona": "autosugestionar",
    "autosugestionaba": "autosugestionar",
    "autosugestionabais": "autosugestionar",
    "autosugestionaban": "autosugestionar",
    "autosugestionabas": "autosugestionar",
    "autosugestionad": "autosugestionar",
    "autosugestionada": "autosugestionar",
    "autosugestionadas": "autosugestionar",
    "autosugestionado": "autosugestionar",
    "autosugestionados": "autosugestionar",
    "autosugestionamos": "autosugestionar",
    "autosugestionan": "autosugestionar",
    "autosugestionando": "autosugestionar",
    "autosugestionara": "autosugestionar",
    "autosugestionarais": "autosugestionar",
    "autosugestionaran": "autosugestionar",
    "autosugestionaras": "autosugestionar",
    "autosugestionare": "autosugestionar",
    "autosugestionareis": "autosugestionar",
    "autosugestionaremos": "autosugestionar",
    "autosugestionaren": "autosugestionar",
    "autosugestionares": "autosugestionar",
    "autosugestionaron": "autosugestionar",
    "autosugestionará": "autosugestionar",
    "autosugestionarán": "autosugestionar",
    "autosugestionarás": "autosugestionar",
    "autosugestionaré": "autosugestionar",
    "autosugestionaréis": "autosugestionar",
    "autosugestionaría": "autosugestionar",
    "autosugestionaríais": "autosugestionar",
    "autosugestionaríamos": "autosugestionar",
    "autosugestionarían": "autosugestionar",
    "autosugestionarías": "autosugestionar",
    "autosugestionas": "autosugestionar",
    "autosugestionase": "autosugestionar",
    "autosugestionaseis": "autosugestionar",
    "autosugestionasen": "autosugestionar",
    "autosugestionases": "autosugestionar",
    "autosugestionaste": "autosugestionar",
    "autosugestionasteis": "autosugestionar",
    "autosugestione": "autosugestionar",
    "autosugestionemos": "autosugestionar",
    "autosugestionen": "autosugestionar",
    "autosugestiones": "autosugestionar",
    "autosugestiono": "autosugestionar",
    "autosugestionábamos": "autosugestionar",
    "autosugestionáis": "autosugestionar",
    "autosugestionáramos": "autosugestionar",
    "autosugestionáremos": "autosugestionar",
    "autosugestionásemos": "autosugestionar",
    "autosugestioné": "autosugestionar",
    "autosugestionéis": "autosugestionar",
    "autosugestionó": "autosugestionar",
    "autotolerancias": "autotolerancia",
    "autotomías": "autotomía",
    "autotoques": "autotoque",
    "autotransductores": "autotransductor",
    "autotransformadores": "autotransformador",
    "autovacunas": "autovacuna",
    "autovaloraciones": "autovaloración",
    "autovalores": "autovalor",
    "autovectores": "autovector",
    "autoventiladora": "autoventilador",
    "autoventiladoras": "autoventilador",
    "autoventiladores": "autoventilador",
    "autovolquetes": "autovolquete",
    "autovías": "autovía",
    "autunitas": "autunita",
    "autárquica": "autárquico",
    "autárquicas": "autárquico",
    "autárquicos": "autárquico",
    "auténtica": "auténtico",
    "auténticas": "auténtico",
    "auténticos": "auténtico",
    "autística": "autístico",
    "autísticas": "autístico",
    "autísticos": "autístico",
    "autócratas": "autócrata",
    "autóctona": "autóctono",
    "autóctonas": "autóctono",
    "autóctonos": "autóctono",
    "autódromos": "autódromo",
    "autógama": "autógamo",
    "autógamas": "autógamo",
    "autógamos": "autógamo",
    "autógena": "autógeno",
    "autógenas": "autógeno",
    "autógenos": "autógeno",
    "autógrafa": "autógrafo",
    "autógrafas": "autógrafo",
    "autógrafos": "autógrafo",
    "autómatas": "autómata",
    "autónoma": "autónomo",
    "autónomas": "autónomo",
    "autónomos": "autónomo",
    "autótrofa": "autótrofo",
    "autótrofas": "autótrofo",
    "autótrofos": "autótrofo",
    "auxilia": "auxiliar",
    "auxiliaba": "auxiliar",
    "auxiliabais": "auxiliar",
    "auxiliaban": "auxiliar",
    "auxiliabas": "auxiliar",
    "auxiliad": "auxiliar",
    "auxiliada": "auxiliar",
    "auxiliadas": "auxiliar",
    "auxiliado": "auxiliar",
    "auxiliadora": "auxiliador",
    "auxiliadoras": "auxiliador",
    "auxiliadores": "auxiliador",
    "auxiliados": "auxiliar",
    "auxiliamos": "auxiliar",
    "auxilian": "auxiliar",
    "auxiliando": "auxiliar",
    "auxiliara": "auxiliar",
    "auxiliarais": "auxiliar",
    "auxiliaran": "auxiliar",
    "auxiliaras": "auxiliar",
    "auxiliare": "auxiliar",
    "auxiliareis": "auxiliar",
    "auxiliaremos": "auxiliar",
    "auxiliaren": "auxiliar",
    "auxiliares": "auxiliar",
    "auxiliaron": "auxiliar",
    "auxiliará": "auxiliar",
    "auxiliarán": "auxiliar",
    "auxiliarás": "auxiliar",
    "auxiliaré": "auxiliar",
    "auxiliaréis": "auxiliar",
    "auxiliaría": "auxiliar",
    "auxiliaríais": "auxiliar",
    "auxiliaríamos": "auxiliar",
    "auxiliarían": "auxiliar",
    "auxiliarías": "auxiliaría",
    "auxilias": "auxiliar",
    "auxiliase": "auxiliar",
    "auxiliaseis": "auxiliar",
    "auxiliasen": "auxiliar",
    "auxiliases": "auxiliar",
    "auxiliaste": "auxiliar",
    "auxiliasteis": "auxiliar",
    "auxilie": "auxiliar",
    "auxiliemos": "auxiliar",
    "auxilien": "auxiliar",
    "auxilies": "auxiliar",
    "auxilio": "auxiliar",
    "auxilios": "auxilio",
    "auxiliábamos": "auxiliar",
    "auxiliáis": "auxiliar",
    "auxiliáramos": "auxiliar",
    "auxiliáremos": "auxiliar",
    "auxiliásemos": "auxiliar",
    "auxilié": "auxiliar",
    "auxiliéis": "auxiliar",
    "auxilió": "auxiliar",
    "auxinas": "auxina",
    "auxocitos": "auxocito",
    "auxocromos": "auxocromo",
    "avada": "avadar",
    "avadaba": "avadar",
    "avadabais": "avadar",
    "avadaban": "avadar",
    "avadabas": "avadar",
    "avadad": "avadar",
    "avadada": "avadar",
    "avadadas": "avadar",
    "avadado": "avadar",
    "avadados": "avadar",
    "avadamos": "avadar",
    "avadan": "avadar",
    "avadando": "avadar",
    "avadara": "avadar",
    "avadarais": "avadar",
    "avadaran": "avadar",
    "avadaras": "avadar",
    "avadare": "avadar",
    "avadareis": "avadar",
    "avadaremos": "avadar",
    "avadaren": "avadar",
    "avadares": "avadar",
    "avadaron": "avadar",
    "avadará": "avadar",
    "avadarán": "avadar",
    "avadarás": "avadar",
    "avadaré": "avadar",
    "avadaréis": "avadar",
    "avadaría": "avadar",
    "avadaríais": "avadar",
    "avadaríamos": "avadar",
    "avadarían": "avadar",
    "avadarías": "avadar",
    "avadas": "avadar",
    "avadase": "avadar",
    "avadaseis": "avadar",
    "avadasen": "avadar",
    "avadases": "avadar",
    "avadaste": "avadar",
    "avadasteis": "avadar",
    "avade": "avadar",
    "avademos": "avadar",
    "avaden": "avadar",
    "avades": "avadar",
    "avado": "avadar",
    "avadábamos": "avadar",
    "avadáis": "avadar",
    "avadáramos": "avadar",
    "avadáremos": "avadar",
    "avadásemos": "avadar",
    "avadé": "avadar",
    "avadéis": "avadar",
    "avadó": "avadar",
    "avaha": "avahar",
    "avahaba": "avahar",
    "avahabais": "avahar",
    "avahaban": "avahar",
    "avahabas": "avahar",
    "avahad": "avahar",
    "avahada": "avahar",
    "avahadas": "avahar",
    "avahado": "avahar",
    "avahados": "avahar",
    "avahamos": "avahar",
    "avahan": "avahar",
    "avahando": "avahar",
    "avahara": "avahar",
    "avaharais": "avahar",
    "avaharan": "avahar",
    "avaharas": "avahar",
    "avahare": "avahar",
    "avahareis": "avahar",
    "avaharemos": "avahar",
    "avaharen": "avahar",
    "avahares": "avahar",
    "avaharon": "avahar",
    "avahará": "avahar",
    "avaharán": "avahar",
    "avaharás": "avahar",
    "avaharé": "avahar",
    "avaharéis": "avahar",
    "avaharía": "avahar",
    "avaharíais": "avahar",
    "avaharíamos": "avahar",
    "avaharían": "avahar",
    "avaharías": "avahar",
    "avahas": "avahar",
    "avahase": "avahar",
    "avahaseis": "avahar",
    "avahasen": "avahar",
    "avahases": "avahar",
    "avahaste": "avahar",
    "avahasteis": "avahar",
    "avahe": "avahar",
    "avahemos": "avahar",
    "avahen": "avahar",
    "avahes": "avahar",
    "avaho": "avahar",
    "avahábamos": "avahar",
    "avaháis": "avahar",
    "avaháramos": "avahar",
    "avaháremos": "avahar",
    "avahásemos": "avahar",
    "avahé": "avahar",
    "avahéis": "avahar",
    "avahó": "avahar",
    "avala": "avalar",
    "avalaba": "avalar",
    "avalabais": "avalar",
    "avalaban": "avalar",
    "avalabas": "avalar",
    "avalad": "avalar",
    "avalada": "avalar",
    "avaladas": "avalar",
    "avalado": "avalar",
    "avaladora": "avalador",
    "avaladoras": "avalador",
    "avaladores": "avalador",
    "avalados": "avalar",
    "avalamos": "avalar",
    "avalan": "avalar",
    "avalanchas": "avalancha",
    "avalando": "avalar",
    "avalara": "avalar",
    "avalarais": "avalar",
    "avalaran": "avalar",
    "avalaras": "avalar",
    "avalare": "avalar",
    "avalareis": "avalar",
    "avalaremos": "avalar",
    "avalaren": "avalar",
    "avalares": "avalar",
    "avalaron": "avalar",
    "avalará": "avalar",
    "avalarán": "avalar",
    "avalarás": "avalar",
    "avalaré": "avalar",
    "avalaréis": "avalar",
    "avalaría": "avalar",
    "avalaríais": "avalar",
    "avalaríamos": "avalar",
    "avalarían": "avalar",
    "avalarías": "avalar",
    "avalas": "avalar",
    "avalase": "avalar",
    "avalaseis": "avalar",
    "avalasen": "avalar",
    "avalases": "avalar",
    "avalaste": "avalar",
    "avalasteis": "avalar",
    "avale": "avalar",
    "avalemos": "avalar",
    "avalen": "avalar",
    "avales": "avalar",
    "avalistas": "avalista",
    "avalo": "avalar",
    "avalora": "avalorar",
    "avaloraba": "avalorar",
    "avalorabais": "avalorar",
    "avaloraban": "avalorar",
    "avalorabas": "avalorar",
    "avalorad": "avalorar",
    "avalorada": "avalorar",
    "avaloradas": "avalorar",
    "avalorado": "avalorar",
    "avalorados": "avalorar",
    "avaloramos": "avalorar",
    "avaloran": "avalorar",
    "avalorando": "avalorar",
    "avalorara": "avalorar",
    "avalorarais": "avalorar",
    "avaloraran": "avalorar",
    "avaloraras": "avalorar",
    "avalorare": "avalorar",
    "avalorareis": "avalorar",
    "avaloraremos": "avalorar",
    "avaloraren": "avalorar",
    "avalorares": "avalorar",
    "avaloraron": "avalorar",
    "avalorará": "avalorar",
    "avalorarán": "avalorar",
    "avalorarás": "avalorar",
    "avaloraré": "avalorar",
    "avaloraréis": "avalorar",
    "avaloraría": "avalorar",
    "avaloraríais": "avalorar",
    "avaloraríamos": "avalorar",
    "avalorarían": "avalorar",
    "avalorarías": "avalorar",
    "avaloras": "avalorar",
    "avalorase": "avalorar",
    "avaloraseis": "avalorar",
    "avalorasen": "avalorar",
    "avalorases": "avalorar",
    "avaloraste": "avalorar",
    "avalorasteis": "avalorar",
    "avalore": "avalorar",
    "avaloremos": "avalorar",
    "avaloren": "avalorar",
    "avalores": "avalorar",
    "avaloro": "avalorar",
    "avalorábamos": "avalorar",
    "avaloráis": "avalorar",
    "avaloráramos": "avalorar",
    "avaloráremos": "avalorar",
    "avalorásemos": "avalorar",
    "avaloré": "avalorar",
    "avaloréis": "avalorar",
    "avaloró": "avalorar",
    "avaluaba": "avaluar",
    "avaluabais": "avaluar",
    "avaluaban": "avaluar",
    "avaluabas": "avaluar",
    "avaluaciones": "avaluación",
    "avaluad": "avaluar",
    "avaluada": "avaluar",
    "avaluadas": "avaluar",
    "avaluado": "avaluar",
    "avaluados": "avaluar",
    "avaluamos": "avaluar",
    "avaluando": "avaluar",
    "avaluara": "avaluar",
    "avaluarais": "avaluar",
    "avaluaran": "avaluar",
    "avaluaras": "avaluar",
    "avaluare": "avaluar",
    "avaluareis": "avaluar",
    "avaluaremos": "avaluar",
    "avaluaren": "avaluar",
    "avaluares": "avaluar",
    "avaluaron": "avaluar",
    "avaluará": "avaluar",
    "avaluarán": "avaluar",
    "avaluarás": "avaluar",
    "avaluaré": "avaluar",
    "avaluaréis": "avaluar",
    "avaluaría": "avaluar",
    "avaluaríais": "avaluar",
    "avaluaríamos": "avaluar",
    "avaluarían": "avaluar",
    "avaluarías": "avaluar",
    "avaluase": "avaluar",
    "avaluaseis": "avaluar",
    "avaluasen": "avaluar",
    "avaluases": "avaluar",
    "avaluaste": "avaluar",
    "avaluasteis": "avaluar",
    "avaluemos": "avaluar",
    "avaluábamos": "avaluar",
    "avaluáis": "avaluar",
    "avaluáramos": "avaluar",
    "avaluáremos": "avaluar",
    "avaluásemos": "avaluar",
    "avalué": "avaluar",
    "avaluéis": "avaluar",
    "avaluó": "avaluar",
    "avalábamos": "avalar",
    "avaláis": "avalar",
    "avaláramos": "avalar",
    "avaláremos": "avalar",
    "avalásemos": "avalar",
    "avalé": "avalar",
    "avaléis": "avalar",
    "avaló": "avalar",
    "avalúa": "avaluar",
    "avalúan": "avaluar",
    "avalúas": "avaluar",
    "avalúe": "avaluar",
    "avalúen": "avaluar",
    "avalúes": "avaluar",
    "avalúo": "avaluar",
    "avalúos": "avalúo",
    "avambrazos": "avambrazo",
    "avance": "avanzar",
    "avancemos": "avanzar",
    "avancen": "avanzar",
    "avances": "avanzar",
    "avancé": "avanzar",
    "avancéis": "avanzar",
    "avanece": "avanecer",
    "avaneced": "avanecer",
    "avanecemos": "avanecer",
    "avanecen": "avanecer",
    "avaneceremos": "avanecer",
    "avanecerá": "avanecer",
    "avanecerán": "avanecer",
    "avanecerás": "avanecer",
    "avaneceré": "avanecer",
    "avaneceréis": "avanecer",
    "avanecería": "avanecer",
    "avaneceríais": "avanecer",
    "avaneceríamos": "avanecer",
    "avanecerían": "avanecer",
    "avanecerías": "avanecer",
    "avaneces": "avanecer",
    "avanecida": "avanecer",
    "avanecidas": "avanecer",
    "avanecido": "avanecer",
    "avanecidos": "avanecer",
    "avaneciendo": "avanecer",
    "avaneciera": "avanecer",
    "avanecierais": "avanecer",
    "avanecieran": "avanecer",
    "avanecieras": "avanecer",
    "avaneciere": "avanecer",
    "avaneciereis": "avanecer",
    "avanecieren": "avanecer",
    "avanecieres": "avanecer",
    "avanecieron": "avanecer",
    "avaneciese": "avanecer",
    "avanecieseis": "avanecer",
    "avaneciesen": "avanecer",
    "avanecieses": "avanecer",
    "avanecimos": "avanecer",
    "avaneciste": "avanecer",
    "avanecisteis": "avanecer",
    "avaneciéramos": "avanecer",
    "avaneciéremos": "avanecer",
    "avaneciésemos": "avanecer",
    "avaneció": "avanecer",
    "avanecéis": "avanecer",
    "avanecí": "avanecer",
    "avanecía": "avanecer",
    "avanecíais": "avanecer",
    "avanecíamos": "avanecer",
    "avanecían": "avanecer",
    "avanecías": "avanecer",
    "avanezca": "avanecer",
    "avanezcamos": "avanecer",
    "avanezcan": "avanecer",
    "avanezcas": "avanecer",
    "avanezco": "avanecer",
    "avanezcáis": "avanecer",
    "avanti": "avante",
    "avantpiquetes": "avantpiquet",
    "avantrenes": "avantrén",
    "avanza": "avanzar",
    "avanzaba": "avanzar",
    "avanzabais": "avanzar",
    "avanzaban": "avanzar",
    "avanzabas": "avanzar",
    "avanzad": "avanzar",
    "avanzada": "avanzar",
    "avanzadas": "avanzar",
    "avanzadillas": "avanzadilla",
    "avanzado": "avanzar",
    "avanzados": "avanzar",
    "avanzamos": "avanzar",
    "avanzan": "avanzar",
    "avanzando": "avanzar",
    "avanzara": "avanzar",
    "avanzarais": "avanzar",
    "avanzaran": "avanzar",
    "avanzaras": "avanzar",
    "avanzare": "avanzar",
    "avanzareis": "avanzar",
    "avanzaremos": "avanzar",
    "avanzaren": "avanzar",
    "avanzares": "avanzar",
    "avanzaron": "avanzar",
    "avanzará": "avanzar",
    "avanzarán": "avanzar",
    "avanzarás": "avanzar",
    "avanzaré": "avanzar",
    "avanzaréis": "avanzar",
    "avanzaría": "avanzar",
    "avanzaríais": "avanzar",
    "avanzaríamos": "avanzar",
    "avanzarían": "avanzar",
    "avanzarías": "avanzar",
    "avanzas": "avanzar",
    "avanzase": "avanzar",
    "avanzaseis": "avanzar",
    "avanzasen": "avanzar",
    "avanzases": "avanzar",
    "avanzaste": "avanzar",
    "avanzasteis": "avanzar",
    "avanzo": "avanzar",
    "avanzábamos": "avanzar",
    "avanzáis": "avanzar",
    "avanzáramos": "avanzar",
    "avanzáremos": "avanzar",
    "avanzásemos": "avanzar",
    "avanzó": "avanzar",
    "avara": "avaro",
    "avaras": "avaro",
    "avaricias": "avaricia",
    "avariciosa": "avaricioso",
    "avariciosas": "avaricioso",
    "avariciosos": "avaricioso",
    "avarienta": "avariento",
    "avarientas": "avariento",
    "avarientos": "avariento",
    "avaros": "avaro",
    "avasalla": "avasallar",
    "avasallaba": "avasallar",
    "avasallabais": "avasallar",
    "avasallaban": "avasallar",
    "avasallabas": "avasallar",
    "avasallad": "avasallar",
    "avasallada": "avasallar",
    "avasalladas": "avasallar",
    "avasallado": "avasallar",
    "avasalladora": "avasallador",
    "avasalladoras": "avasallador",
    "avasalladores": "avasallador",
    "avasallados": "avasallar",
    "avasallamientos": "avasallamiento",
    "avasallamos": "avasallar",
    "avasallan": "avasallar",
    "avasallando": "avasallar",
    "avasallara": "avasallar",
    "avasallarais": "avasallar",
    "avasallaran": "avasallar",
    "avasallaras": "avasallar",
    "avasallare": "avasallar",
    "avasallareis": "avasallar",
    "avasallaremos": "avasallar",
    "avasallaren": "avasallar",
    "avasallares": "avasallar",
    "avasallaron": "avasallar",
    "avasallará": "avasallar",
    "avasallarán": "avasallar",
    "avasallarás": "avasallar",
    "avasallaré": "avasallar",
    "avasallaréis": "avasallar",
    "avasallaría": "avasallar",
    "avasallaríais": "avasallar",
    "avasallaríamos": "avasallar",
    "avasallarían": "avasallar",
    "avasallarías": "avasallar",
    "avasallas": "avasallar",
    "avasallase": "avasallar",
    "avasallaseis": "avasallar",
    "avasallasen": "avasallar",
    "avasallases": "avasallar",
    "avasallaste": "avasallar",
    "avasallasteis": "avasallar",
    "avasalle": "avasallar",
    "avasallemos": "avasallar",
    "avasallen": "avasallar",
    "avasalles": "avasallar",
    "avasallo": "avasallar",
    "avasallábamos": "avasallar",
    "avasalláis": "avasallar",
    "avasalláramos": "avasallar",
    "avasalláremos": "avasallar",
    "avasallásemos": "avasallar",
    "avasallé": "avasallar",
    "avasalléis": "avasallar",
    "avasalló": "avasallar",
    "avataras": "avatara",
    "avatares": "avatar",
    "avece": "avezar",
    "avecemos": "avezar",
    "avecen": "avezar",
    "aveces": "avezar",
    "avechuchos": "avechucho",
    "avecina": "avecinar",
    "avecinaba": "avecinar",
    "avecinabais": "avecinar",
    "avecinaban": "avecinar",
    "avecinabas": "avecinar",
    "avecinad": "avecinar",
    "avecinada": "avecinar",
    "avecinadas": "avecinar",
    "avecinado": "avecinar",
    "avecinados": "avecinar",
    "avecinamos": "avecinar",
    "avecinan": "avecinar",
    "avecinando": "avecinar",
    "avecinara": "avecinar",
    "avecinarais": "avecinar",
    "avecinaran": "avecinar",
    "avecinaras": "avecinar",
    "avecinare": "avecinar",
    "avecinareis": "avecinar",
    "avecinaremos": "avecinar",
    "avecinaren": "avecinar",
    "avecinares": "avecinar",
    "avecinaron": "avecinar",
    "avecinará": "avecinar",
    "avecinarán": "avecinar",
    "avecinarás": "avecinar",
    "avecinaré": "avecinar",
    "avecinaréis": "avecinar",
    "avecinaría": "avecinar",
    "avecinaríais": "avecinar",
    "avecinaríamos": "avecinar",
    "avecinarían": "avecinar",
    "avecinarías": "avecinar",
    "avecinas": "avecinar",
    "avecinase": "avecinar",
    "avecinaseis": "avecinar",
    "avecinasen": "avecinar",
    "avecinases": "avecinar",
    "avecinaste": "avecinar",
    "avecinasteis": "avecinar",
    "avecinda": "avecindar",
    "avecindaba": "avecindar",
    "avecindabais": "avecindar",
    "avecindaban": "avecindar",
    "avecindabas": "avecindar",
    "avecindad": "avecindar",
    "avecindada": "avecindar",
    "avecindadas": "avecindar",
    "avecindado": "avecindar",
    "avecindados": "avecindar",
    "avecindamientos": "avecindamiento",
    "avecindamos": "avecindar",
    "avecindan": "avecindar",
    "avecindando": "avecindar",
    "avecindara": "avecindar",
    "avecindarais": "avecindar",
    "avecindaran": "avecindar",
    "avecindaras": "avecindar",
    "avecindare": "avecindar",
    "avecindareis": "avecindar",
    "avecindaremos": "avecindar",
    "avecindaren": "avecindar",
    "avecindares": "avecindar",
    "avecindaron": "avecindar",
    "avecindará": "avecindar",
    "avecindarán": "avecindar",
    "avecindarás": "avecindar",
    "avecindaré": "avecindar",
    "avecindaréis": "avecindar",
    "avecindaría": "avecindar",
    "avecindaríais": "avecindar",
    "avecindaríamos": "avecindar",
    "avecindarían": "avecindar",
    "avecindarías": "avecindar",
    "avecindas": "avecindar",
    "avecindase": "avecindar",
    "avecindaseis": "avecindar",
    "avecindasen": "avecindar",
    "avecindases": "avecindar",
    "avecindaste": "avecindar",
    "avecindasteis": "avecindar",
    "avecinde": "avecindar",
    "avecindemos": "avecindar",
    "avecinden": "avecindar",
    "avecindes": "avecindar",
    "avecindo": "avecindar",
    "avecindábamos": "avecindar",
    "avecindáis": "avecindar",
    "avecindáramos": "avecindar",
    "avecindáremos": "avecindar",
    "avecindásemos": "avecindar",
    "avecindé": "avecindar",
    "avecindéis": "avecindar",
    "avecindó": "avecindar",
    "avecine": "avecinar",
    "avecinemos": "avecinar",
    "avecinen": "avecinar",
    "avecines": "avecinar",
    "avecino": "avecinar",
    "avecinábamos": "avecinar",
    "avecináis": "avecinar",
    "avecináramos": "avecinar",
    "avecináremos": "avecinar",
    "avecinásemos": "avecinar",
    "aveciné": "avecinar",
    "avecinéis": "avecinar",
    "avecinó": "avecinar",
    "avecé": "avezar",
    "avecéis": "avezar",
    "avefrías": "avefría",
    "avejenta": "avejentar",
    "avejentaba": "avejentar",
    "avejentabais": "avejentar",
    "avejentaban": "avejentar",
    "avejentabas": "avejentar",
    "avejentad": "avejentar",
    "avejentada": "avejentar",
    "avejentadas": "avejentar",
    "avejentado": "avejentar",
    "avejentados": "avejentar",
    "avejentamientos": "avejentamiento",
    "avejentamos": "avejentar",
    "avejentan": "avejentar",
    "avejentando": "avejentar",
    "avejentara": "avejentar",
    "avejentarais": "avejentar",
    "avejentaran": "avejentar",
    "avejentaras": "avejentar",
    "avejentare": "avejentar",
    "avejentareis": "avejentar",
    "avejentaremos": "avejentar",
    "avejentaren": "avejentar",
    "avejentares": "avejentar",
    "avejentaron": "avejentar",
    "avejentará": "avejentar",
    "avejentarán": "avejentar",
    "avejentarás": "avejentar",
    "avejentaré": "avejentar",
    "avejentaréis": "avejentar",
    "avejentaría": "avejentar",
    "avejentaríais": "avejentar",
    "avejentaríamos": "avejentar",
    "avejentarían": "avejentar",
    "avejentarías": "avejentar",
    "avejentas": "avejentar",
    "avejentase": "avejentar",
    "avejentaseis": "avejentar",
    "avejentasen": "avejentar",
    "avejentases": "avejentar",
    "avejentaste": "avejentar",
    "avejentasteis": "avejentar",
    "avejente": "avejentar",
    "avejentemos": "avejentar",
    "avejenten": "avejentar",
    "avejentes": "avejentar",
    "avejento": "avejentar",
    "avejentábamos": "avejentar",
    "avejentáis": "avejentar",
    "avejentáramos": "avejentar",
    "avejentáremos": "avejentar",
    "avejentásemos": "avejentar",
    "avejenté": "avejentar",
    "avejentéis": "avejentar",
    "avejentó": "avejentar",
    "avejiga": "avejigar",
    "avejigaba": "avejigar",
    "avejigabais": "avejigar",
    "avejigaban": "avejigar",
    "avejigabas": "avejigar",
    "avejigad": "avejigar",
    "avejigada": "avejigar",
    "avejigadas": "avejigar",
    "avejigado": "avejigar",
    "avejigados": "avejigar",
    "avejigamos": "avejigar",
    "avejigan": "avejigar",
    "avejigando": "avejigar",
    "avejigara": "avejigar",
    "avejigarais": "avejigar",
    "avejigaran": "avejigar",
    "avejigaras": "avejigar",
    "avejigare": "avejigar",
    "avejigareis": "avejigar",
    "avejigaremos": "avejigar",
    "avejigaren": "avejigar",
    "avejigares": "avejigar",
    "avejigaron": "avejigar",
    "avejigará": "avejigar",
    "avejigarán": "avejigar",
    "avejigarás": "avejigar",
    "avejigaré": "avejigar",
    "avejigaréis": "avejigar",
    "avejigaría": "avejigar",
    "avejigaríais": "avejigar",
    "avejigaríamos": "avejigar",
    "avejigarían": "avejigar",
    "avejigarías": "avejigar",
    "avejigas": "avejigar",
    "avejigase": "avejigar",
    "avejigaseis": "avejigar",
    "avejigasen": "avejigar",
    "avejigases": "avejigar",
    "avejigaste": "avejigar",
    "avejigasteis": "avejigar",
    "avejigo": "avejigar",
    "avejigue": "avejigar",
    "avejiguemos": "avejigar",
    "avejiguen": "avejigar",
    "avejigues": "avejigar",
    "avejigué": "avejigar",
    "avejiguéis": "avejigar",
    "avejigábamos": "avejigar",
    "avejigáis": "avejigar",
    "avejigáramos": "avejigar",
    "avejigáremos": "avejigar",
    "avejigásemos": "avejigar",
    "avejigó": "avejigar",
    "avellana": "avellanar",
    "avellanaba": "avellanar",
    "avellanabais": "avellanar",
    "avellanaban": "avellanar",
    "avellanabas": "avellanar",
    "avellanad": "avellanar",
    "avellanada": "avellanar",
    "avellanadas": "avellanar",
    "avellanado": "avellanar",
    "avellanadores": "avellanador",
    "avellanados": "avellanar",
    "avellanamos": "avellanar",
    "avellanan": "avellanar",
    "avellanando": "avellanar",
    "avellanara": "avellanar",
    "avellanarais": "avellanar",
    "avellanaran": "avellanar",
    "avellanaras": "avellanar",
    "avellanare": "avellanar",
    "avellanareis": "avellanar",
    "avellanaremos": "avellanar",
    "avellanaren": "avellanar",
    "avellanares": "avellanar",
    "avellanaron": "avellanar",
    "avellanará": "avellanar",
    "avellanarán": "avellanar",
    "avellanarás": "avellanar",
    "avellanaré": "avellanar",
    "avellanaréis": "avellanar",
    "avellanaría": "avellanar",
    "avellanaríais": "avellanar",
    "avellanaríamos": "avellanar",
    "avellanarían": "avellanar",
    "avellanarías": "avellanar",
    "avellanas": "avellanar",
    "avellanase": "avellanar",
    "avellanaseis": "avellanar",
    "avellanasen": "avellanar",
    "avellanases": "avellanar",
    "avellanaste": "avellanar",
    "avellanasteis": "avellanar",
    "avellane": "avellanar",
    "avellanedas": "avellaneda",
    "avellanemos": "avellanar",
    "avellanen": "avellanar",
    "avellaneros": "avellanero",
    "avellanes": "avellanar",
    "avellano": "avellanar",
    "avellanos": "avellano",
    "avellanábamos": "avellanar",
    "avellanáis": "avellanar",
    "avellanáramos": "avellanar",
    "avellanáremos": "avellanar",
    "avellanásemos": "avellanar",
    "avellané": "avellanar",
    "avellanéis": "avellanar",
    "avellanó": "avellanar",
    "avemarías": "avemaría",
    "aven": "avenir",
    "avena": "avenar",
    "avenaba": "avenar",
    "avenabais": "avenar",
    "avenaban": "avenar",
    "avenabas": "avenar",
    "avenad": "avenar",
    "avenada": "avenar",
    "avenadas": "avenar",
    "avenado": "avenar",
    "avenados": "avenar",
    "avenamientos": "avenamiento",
    "avenamos": "avenar",
    "avenan": "avenar",
    "avenando": "avenar",
    "avenara": "avenar",
    "avenarais": "avenar",
    "avenaran": "avenar",
    "avenaras": "avenar",
    "avenare": "avenar",
    "avenareis": "avenar",
    "avenaremos": "avenar",
    "avenaren": "avenar",
    "avenares": "avenar",
    "avenaron": "avenar",
    "avenará": "avenar",
    "avenarán": "avenar",
    "avenarás": "avenar",
    "avenaré": "avenar",
    "avenaréis": "avenar",
    "avenaría": "avenar",
    "avenaríais": "avenar",
    "avenaríamos": "avenar",
    "avenarían": "avenar",
    "avenarías": "avenar",
    "avenas": "avenar",
    "avenase": "avenar",
    "avenaseis": "avenar",
    "avenasen": "avenar",
    "avenases": "avenar",
    "avenaste": "avenar",
    "avenasteis": "avenar",
    "avenates": "avenate",
    "avendremos": "avenir",
    "avendrá": "avenir",
    "avendrán": "avenir",
    "avendrás": "avenir",
    "avendré": "avenir",
    "avendréis": "avenir",
    "avendría": "avenir",
    "avendríais": "avenir",
    "avendríamos": "avenir",
    "avendrían": "avenir",
    "avendrías": "avenir",
    "avene": "avenar",
    "avenemos": "avenar",
    "avenen": "avenar",
    "avenencias": "avenencia",
    "avenes": "avenar",
    "avenga": "avenir",
    "avengamos": "avenir",
    "avengan": "avenir",
    "avengas": "avenir",
    "avengo": "avenir",
    "avengáis": "avenir",
    "avenid": "avenir",
    "avenida": "avenir",
    "avenidas": "avenir",
    "avenido": "avenir",
    "avenidos": "avenir",
    "avenimos": "avenir",
    "aveno": "avenar",
    "aventaba": "aventar",
    "aventabais": "aventar",
    "aventaban": "aventar",
    "aventabas": "aventar",
    "aventad": "aventar",
    "aventada": "aventar",
    "aventadas": "aventar",
    "aventado": "aventar",
    "aventadora": "aventador",
    "aventadoras": "aventadora",
    "aventadores": "aventador",
    "aventados": "aventar",
    "aventaduras": "aventadura",
    "aventaja": "aventajar",
    "aventajaba": "aventajar",
    "aventajabais": "aventajar",
    "aventajaban": "aventajar",
    "aventajabas": "aventajar",
    "aventajad": "aventajar",
    "aventajada": "aventajar",
    "aventajadas": "aventajar",
    "aventajado": "aventajar",
    "aventajados": "aventajar",
    "aventajamos": "aventajar",
    "aventajan": "aventajar",
    "aventajando": "aventajar",
    "aventajara": "aventajar",
    "aventajarais": "aventajar",
    "aventajaran": "aventajar",
    "aventajaras": "aventajar",
    "aventajare": "aventajar",
    "aventajareis": "aventajar",
    "aventajaremos": "aventajar",
    "aventajaren": "aventajar",
    "aventajares": "aventajar",
    "aventajaron": "aventajar",
    "aventajará": "aventajar",
    "aventajarán": "aventajar",
    "aventajarás": "aventajar",
    "aventajaré": "aventajar",
    "aventajaréis": "aventajar",
    "aventajaría": "aventajar",
    "aventajaríais": "aventajar",
    "aventajaríamos": "aventajar",
    "aventajarían": "aventajar",
    "aventajarías": "aventajar",
    "aventajas": "aventajar",
    "aventajase": "aventajar",
    "aventajaseis": "aventajar",
    "aventajasen": "aventajar",
    "aventajases": "aventajar",
    "aventajaste": "aventajar",
    "aventajasteis": "aventajar",
    "aventaje": "aventajar",
    "aventajemos": "aventajar",
    "aventajen": "aventajar",
    "aventajes": "aventajar",
    "aventajo": "aventajar",
    "aventajábamos": "aventajar",
    "aventajáis": "aventajar",
    "aventajáramos": "aventajar",
    "aventajáremos": "aventajar",
    "aventajásemos": "aventajar",
    "aventajé": "aventajar",
    "aventajéis": "aventajar",
    "aventajó": "aventajar",
    "aventamientos": "aventamiento",
    "aventamos": "aventar",
    "aventando": "aventar",
    "aventara": "aventar",
    "aventarais": "aventar",
    "aventaran": "aventar",
    "aventaras": "aventar",
    "aventare": "aventar",
    "aventareis": "aventar",
    "aventaremos": "aventar",
    "aventaren": "aventar",
    "aventares": "aventar",
    "aventaron": "aventar",
    "aventará": "aventar",
    "aventarán": "aventar",
    "aventarás": "aventar",
    "aventaré": "aventar",
    "aventaréis": "aventar",
    "aventaría": "aventar",
    "aventaríais": "aventar",
    "aventaríamos": "aventar",
    "aventarían": "aventar",
    "aventarías": "aventar",
    "aventase": "aventar",
    "aventaseis": "aventar",
    "aventasen": "aventar",
    "aventases": "aventar",
    "aventaste": "aventar",
    "aventasteis": "aventar",
    "aventemos": "aventar",
    "aventura": "aventurar",
    "aventuraba": "aventurar",
    "aventurabais": "aventurar",
    "aventuraban": "aventurar",
    "aventurabas": "aventurar",
    "aventurad": "aventurar",
    "aventurada": "aventurar",
    "aventuradas": "aventurar",
    "aventurado": "aventurar",
    "aventurados": "aventurar",
    "aventuramos": "aventurar",
    "aventuran": "aventurar",
    "aventurando": "aventurar",
    "aventurara": "aventurar",
    "aventurarais": "aventurar",
    "aventuraran": "aventurar",
    "aventuraras": "aventurar",
    "aventurare": "aventurar",
    "aventurareis": "aventurar",
    "aventuraremos": "aventurar",
    "aventuraren": "aventurar",
    "aventurares": "aventurar",
    "aventuraron": "aventurar",
    "aventurará": "aventurar",
    "aventurarán": "aventurar",
    "aventurarás": "aventurar",
    "aventuraré": "aventurar",
    "aventuraréis": "aventurar",
    "aventuraría": "aventurar",
    "aventuraríais": "aventurar",
    "aventuraríamos": "aventurar",
    "aventurarían": "aventurar",
    "aventurarías": "aventurar",
    "aventuras": "aventurar",
    "aventurase": "aventurar",
    "aventuraseis": "aventurar",
    "aventurasen": "aventurar",
    "aventurases": "aventurar",
    "aventuraste": "aventurar",
    "aventurasteis": "aventurar",
    "aventure": "aventurar",
    "aventuremos": "aventurar",
    "aventuren": "aventurar",
    "aventurera": "aventurero",
    "aventureras": "aventurero",
    "aventureros": "aventurero",
    "aventures": "aventurar",
    "aventurina": "aventurino",
    "aventurinada": "aventurinado",
    "aventurinadas": "aventurinado",
    "aventurinados": "aventurinado",
    "aventurinas": "aventurino",
    "aventurinos": "aventurino",
    "aventuro": "aventurar",
    "aventurábamos": "aventurar",
    "aventuráis": "aventurar",
    "aventuráramos": "aventurar",
    "aventuráremos": "aventurar",
    "aventurásemos": "aventurar",
    "aventuré": "aventurar",
    "aventuréis": "aventurar",
    "aventuró": "aventurar",
    "aventábamos": "aventar",
    "aventáis": "aventar",
    "aventáramos": "aventar",
    "aventáremos": "aventar",
    "aventásemos": "aventar",
    "aventé": "aventar",
    "aventéis": "aventar",
    "aventó": "aventar",
    "avenábamos": "avenar",
    "avenáis": "avenar",
    "avenáramos": "avenar",
    "avenáremos": "avenar",
    "avenásemos": "avenar",
    "avené": "avenar",
    "avenéis": "avenar",
    "avenía": "avenir",
    "aveníais": "avenir",
    "aveníamos": "avenir",
    "avenían": "avenir",
    "avenías": "avenir",
    "avenís": "avenir",
    "avenó": "avenar",
    "avergoncemos": "avergonzar",
    "avergoncé": "avergonzar",
    "avergoncéis": "avergonzar",
    "avergonzaba": "avergonzar",
    "avergonzabais": "avergonzar",
    "avergonzaban": "avergonzar",
    "avergonzabas": "avergonzar",
    "avergonzad": "avergonzar",
    "avergonzada": "avergonzar",
    "avergonzadas": "avergonzar",
    "avergonzado": "avergonzar",
    "avergonzados": "avergonzar",
    "avergonzamos": "avergonzar",
    "avergonzando": "avergonzar",
    "avergonzara": "avergonzar",
    "avergonzarais": "avergonzar",
    "avergonzaran": "avergonzar",
    "avergonzaras": "avergonzar",
    "avergonzare": "avergonzar",
    "avergonzareis": "avergonzar",
    "avergonzaremos": "avergonzar",
    "avergonzaren": "avergonzar",
    "avergonzares": "avergonzar",
    "avergonzaron": "avergonzar",
    "avergonzará": "avergonzar",
    "avergonzarán": "avergonzar",
    "avergonzarás": "avergonzar",
    "avergonzaré": "avergonzar",
    "avergonzaréis": "avergonzar",
    "avergonzaría": "avergonzar",
    "avergonzaríais": "avergonzar",
    "avergonzaríamos": "avergonzar",
    "avergonzarían": "avergonzar",
    "avergonzarías": "avergonzar",
    "avergonzase": "avergonzar",
    "avergonzaseis": "avergonzar",
    "avergonzasen": "avergonzar",
    "avergonzases": "avergonzar",
    "avergonzaste": "avergonzar",
    "avergonzasteis": "avergonzar",
    "avergonzábamos": "avergonzar",
    "avergonzáis": "avergonzar",
    "avergonzáramos": "avergonzar",
    "avergonzáremos": "avergonzar",
    "avergonzásemos": "avergonzar",
    "avergonzó": "avergonzar",
    "avergüence": "avergonzar",
    "avergüencen": "avergonzar",
    "avergüences": "avergonzar",
    "avergüenza": "avergonzar",
    "avergüenzan": "avergonzar",
    "avergüenzas": "avergonzar",
    "avergüenzo": "avergonzar",
    "averiaba": "averiar",
    "averiabais": "averiar",
    "averiaban": "averiar",
    "averiabas": "averiar",
    "averiad": "averiar",
    "averiada": "averiar",
    "averiadas": "averiar",
    "averiado": "averiar",
    "averiados": "averiar",
    "averiamos": "averiar",
    "averiando": "averiar",
    "averiara": "averiar",
    "averiarais": "averiar",
    "averiaran": "averiar",
    "averiaras": "averiar",
    "averiare": "averiar",
    "averiareis": "averiar",
    "averiaremos": "averiar",
    "averiaren": "averiar",
    "averiares": "averiar",
    "averiaron": "averiar",
    "averiará": "averiar",
    "averiarán": "averiar",
    "averiarás": "averiar",
    "averiaré": "averiar",
    "averiaréis": "averiar",
    "averiaría": "averiar",
    "averiaríais": "averiar",
    "averiaríamos": "averiar",
    "averiarían": "averiar",
    "averiarías": "averiar",
    "averiase": "averiar",
    "averiaseis": "averiar",
    "averiasen": "averiar",
    "averiases": "averiar",
    "averiaste": "averiar",
    "averiasteis": "averiar",
    "averiemos": "averiar",
    "averigua": "averiguar",
    "averiguaba": "averiguar",
    "averiguabais": "averiguar",
    "averiguaban": "averiguar",
    "averiguabas": "averiguar",
    "averiguables": "averiguable",
    "averiguaciones": "averiguación",
    "averiguad": "averiguar",
    "averiguada": "averiguar",
    "averiguadas": "averiguar",
    "averiguado": "averiguar",
    "averiguadora": "averiguador",
    "averiguadoras": "averiguador",
    "averiguadores": "averiguador",
    "averiguados": "averiguar",
    "averiguamientos": "averiguamiento",
    "averiguamos": "averiguar",
    "averiguan": "averiguar",
    "averiguando": "averiguar",
    "averiguara": "averiguar",
    "averiguarais": "averiguar",
    "averiguaran": "averiguar",
    "averiguaras": "averiguar",
    "averiguare": "averiguar",
    "averiguareis": "averiguar",
    "averiguaremos": "averiguar",
    "averiguaren": "averiguar",
    "averiguares": "averiguar",
    "averiguaron": "averiguar",
    "averiguará": "averiguar",
    "averiguarán": "averiguar",
    "averiguarás": "averiguar",
    "averiguaré": "averiguar",
    "averiguaréis": "averiguar",
    "averiguaría": "averiguar",
    "averiguaríais": "averiguar",
    "averiguaríamos": "averiguar",
    "averiguarían": "averiguar",
    "averiguarías": "averiguar",
    "averiguas": "averiguar",
    "averiguase": "averiguar",
    "averiguaseis": "averiguar",
    "averiguasen": "averiguar",
    "averiguases": "averiguar",
    "averiguaste": "averiguar",
    "averiguasteis": "averiguar",
    "averiguo": "averiguar",
    "averiguábamos": "averiguar",
    "averiguáis": "averiguar",
    "averiguáramos": "averiguar",
    "averiguáremos": "averiguar",
    "averiguásemos": "averiguar",
    "averiguó": "averiguar",
    "averigüe": "averiguar",
    "averigüemos": "averiguar",
    "averigüen": "averiguar",
    "averigües": "averiguar",
    "averigüé": "averiguar",
    "averigüéis": "averiguar",
    "averiábamos": "averiar",
    "averiáis": "averiar",
    "averiáramos": "averiar",
    "averiáremos": "averiar",
    "averiásemos": "averiar",
    "averié": "averiar",
    "averiéis": "averiar",
    "averió": "averiar",
    "avernos": "averno",
    "averroísmos": "averroísmo",
    "averroístas": "averroísta",
    "averrugada": "averrugado",
    "averrugadas": "averrugado",
    "averrugados": "averrugado",
    "aversiones": "aversión",
    "aversiónes": "aversión",
    "avería": "averiar",
    "averían": "averiar",
    "averías": "averiar",
    "averíe": "averiar",
    "averíen": "averiar",
    "averíes": "averiar",
    "averío": "averiar",
    "averíos": "averío",
    "aves": "ave",
    "avestruces": "avestruz",
    "avetorillos": "avetorillo",
    "avetoros": "avetoro",
    "aveza": "avezar",
    "avezaba": "avezar",
    "avezabais": "avezar",
    "avezaban": "avezar",
    "avezabas": "avezar",
    "avezad": "avezar",
    "avezada": "avezar",
    "avezadas": "avezar",
    "avezado": "avezar",
    "avezados": "avezar",
    "avezamos": "avezar",
    "avezan": "avezar",
    "avezando": "avezar",
    "avezara": "avezar",
    "avezarais": "avezar",
    "avezaran": "avezar",
    "avezaras": "avezar",
    "avezare": "avezar",
    "avezareis": "avezar",
    "avezaremos": "avezar",
    "avezaren": "avezar",
    "avezares": "avezar",
    "avezaron": "avezar",
    "avezará": "avezar",
    "avezarán": "avezar",
    "avezarás": "avezar",
    "avezaré": "avezar",
    "avezaréis": "avezar",
    "avezaría": "avezar",
    "avezaríais": "avezar",
    "avezaríamos": "avezar",
    "avezarían": "avezar",
    "avezarías": "avezar",
    "avezas": "avezar",
    "avezase": "avezar",
    "avezaseis": "avezar",
    "avezasen": "avezar",
    "avezases": "avezar",
    "avezaste": "avezar",
    "avezasteis": "avezar",
    "avezo": "avezar",
    "avezábamos": "avezar",
    "avezáis": "avezar",
    "avezáramos": "avezar",
    "avezáremos": "avezar",
    "avezásemos": "avezar",
    "avezó": "avezar",
    "aviaba": "aviar",
    "aviabais": "aviar",
    "aviaban": "aviar",
    "aviabas": "aviar",
    "aviaciones": "aviación",
    "aviad": "aviar",
    "aviada": "aviar",
    "aviadas": "aviar",
    "aviado": "aviar",
    "aviadora": "aviador",
    "aviadoras": "aviador",
    "aviadores": "aviador",
    "aviados": "aviar",
    "aviamos": "aviar",
    "aviando": "aviar",
    "aviara": "aviar",
    "aviarais": "aviar",
    "aviaran": "aviar",
    "aviaras": "aviar",
    "aviare": "aviar",
    "aviareis": "aviar",
    "aviaremos": "aviar",
    "aviaren": "aviar",
    "aviares": "aviar",
    "aviaria": "aviario",
    "aviarias": "aviario",
    "aviarios": "aviario",
    "aviaron": "aviar",
    "aviará": "aviar",
    "aviarán": "aviar",
    "aviarás": "aviar",
    "aviaré": "aviar",
    "aviaréis": "aviar",
    "aviaría": "aviar",
    "aviaríais": "aviar",
    "aviaríamos": "aviar",
    "aviarían": "aviar",
    "aviarías": "aviar",
    "aviase": "aviar",
    "aviaseis": "aviar",
    "aviasen": "aviar",
    "aviases": "aviar",
    "aviaste": "aviar",
    "aviasteis": "aviar",
    "avicia": "aviciar",
    "aviciaba": "aviciar",
    "aviciabais": "aviciar",
    "aviciaban": "aviciar",
    "aviciabas": "aviciar",
    "aviciad": "aviciar",
    "aviciada": "aviciar",
    "aviciadas": "aviciar",
    "aviciado": "aviciar",
    "aviciados": "aviciar",
    "aviciamos": "aviciar",
    "avician": "aviciar",
    "aviciando": "aviciar",
    "aviciara": "aviciar",
    "aviciarais": "aviciar",
    "aviciaran": "aviciar",
    "aviciaras": "aviciar",
    "aviciare": "aviciar",
    "aviciareis": "aviciar",
    "aviciaremos": "aviciar",
    "aviciaren": "aviciar",
    "aviciares": "aviciar",
    "aviciaron": "aviciar",
    "aviciará": "aviciar",
    "aviciarán": "aviciar",
    "aviciarás": "aviciar",
    "aviciaré": "aviciar",
    "aviciaréis": "aviciar",
    "aviciaría": "aviciar",
    "aviciaríais": "aviciar",
    "aviciaríamos": "aviciar",
    "aviciarían": "aviciar",
    "aviciarías": "aviciar",
    "avicias": "aviciar",
    "aviciase": "aviciar",
    "aviciaseis": "aviciar",
    "aviciasen": "aviciar",
    "aviciases": "aviciar",
    "aviciaste": "aviciar",
    "aviciasteis": "aviciar",
    "avicie": "aviciar",
    "aviciemos": "aviciar",
    "avicien": "aviciar",
    "avicies": "aviciar",
    "avicio": "aviciar",
    "aviciábamos": "aviciar",
    "aviciáis": "aviciar",
    "aviciáramos": "aviciar",
    "aviciáremos": "aviciar",
    "aviciásemos": "aviciar",
    "avicié": "aviciar",
    "aviciéis": "aviciar",
    "avició": "aviciar",
    "avicultora": "avicultor",
    "avicultoras": "avicultor",
    "avicultores": "avicultor",
    "aviculturas": "avicultura",
    "avideces": "avidez",
    "avidinas": "avidina",
    "avieja": "aviejar",
    "aviejaba": "aviejar",
    "aviejabais": "aviejar",
    "aviejaban": "aviejar",
    "aviejabas": "aviejar",
    "aviejad": "aviejar",
    "aviejada": "aviejar",
    "aviejadas": "aviejar",
    "aviejado": "aviejar",
    "aviejados": "aviejar",
    "aviejamientos": "aviejamiento",
    "aviejamos": "aviejar",
    "aviejan": "aviejar",
    "aviejando": "aviejar",
    "aviejara": "aviejar",
    "aviejarais": "aviejar",
    "aviejaran": "aviejar",
    "aviejaras": "aviejar",
    "aviejare": "aviejar",
    "aviejareis": "aviejar",
    "aviejaremos": "aviejar",
    "aviejaren": "aviejar",
    "aviejares": "aviejar",
    "aviejaron": "aviejar",
    "aviejará": "aviejar",
    "aviejarán": "aviejar",
    "aviejarás": "aviejar",
    "aviejaré": "aviejar",
    "aviejaréis": "aviejar",
    "aviejaría": "aviejar",
    "aviejaríais": "aviejar",
    "aviejaríamos": "aviejar",
    "aviejarían": "aviejar",
    "aviejarías": "aviejar",
    "aviejas": "aviejar",
    "aviejase": "aviejar",
    "aviejaseis": "aviejar",
    "aviejasen": "aviejar",
    "aviejases": "aviejar",
    "aviejaste": "aviejar",
    "aviejasteis": "aviejar",
    "avieje": "aviejar",
    "aviejemos": "aviejar",
    "aviejen": "aviejar",
    "aviejes": "aviejar",
    "aviejo": "aviejar",
    "aviejábamos": "aviejar",
    "aviejáis": "aviejar",
    "aviejáramos": "aviejar",
    "aviejáremos": "aviejar",
    "aviejásemos": "aviejar",
    "aviejé": "aviejar",
    "aviejéis": "aviejar",
    "aviejó": "aviejar",
    "aviemos": "aviar",
    "aviene": "avenir",
    "avienen": "avenir",
    "avienes": "avenir",
    "avienta": "aventar",
    "avientan": "aventar",
    "avientas": "avienta",
    "aviente": "aventar",
    "avienten": "aventar",
    "avientes": "aventar",
    "aviento": "aventar",
    "avientos": "aviento",
    "aviesa": "avieso",
    "aviesas": "avieso",
    "aviesos": "avieso",
    "avifaunas": "avifauna",
    "avigora": "avigorar",
    "avigoraba": "avigorar",
    "avigorabais": "avigorar",
    "avigoraban": "avigorar",
    "avigorabas": "avigorar",
    "avigorad": "avigorar",
    "avigorada": "avigorar",
    "avigoradas": "avigorar",
    "avigorado": "avigorar",
    "avigorados": "avigorar",
    "avigoramos": "avigorar",
    "avigoran": "avigorar",
    "avigorando": "avigorar",
    "avigorara": "avigorar",
    "avigorarais": "avigorar",
    "avigoraran": "avigorar",
    "avigoraras": "avigorar",
    "avigorare": "avigorar",
    "avigorareis": "avigorar",
    "avigoraremos": "avigorar",
    "avigoraren": "avigorar",
    "avigorares": "avigorar",
    "avigoraron": "avigorar",
    "avigorará": "avigorar",
    "avigorarán": "avigorar",
    "avigorarás": "avigorar",
    "avigoraré": "avigorar",
    "avigoraréis": "avigorar",
    "avigoraría": "avigorar",
    "avigoraríais": "avigorar",
    "avigoraríamos": "avigorar",
    "avigorarían": "avigorar",
    "avigorarías": "avigorar",
    "avigoras": "avigorar",
    "avigorase": "avigorar",
    "avigoraseis": "avigorar",
    "avigorasen": "avigorar",
    "avigorases": "avigorar",
    "avigoraste": "avigorar",
    "avigorasteis": "avigorar",
    "avigore": "avigorar",
    "avigoremos": "avigorar",
    "avigoren": "avigorar",
    "avigores": "avigorar",
    "avigoro": "avigorar",
    "avigorábamos": "avigorar",
    "avigoráis": "avigorar",
    "avigoráramos": "avigorar",
    "avigoráremos": "avigorar",
    "avigorásemos": "avigorar",
    "avigoré": "avigorar",
    "avigoréis": "avigorar",
    "avigoró": "avigorar",
    "avilanta": "avilantar",
    "avilantaba": "avilantar",
    "avilantabais": "avilantar",
    "avilantaban": "avilantar",
    "avilantabas": "avilantar",
    "avilantad": "avilantar",
    "avilantada": "avilantar",
    "avilantadas": "avilantar",
    "avilantado": "avilantar",
    "avilantados": "avilantar",
    "avilantamos": "avilantar",
    "avilantan": "avilantar",
    "avilantando": "avilantar",
    "avilantara": "avilantar",
    "avilantarais": "avilantar",
    "avilantaran": "avilantar",
    "avilantaras": "avilantar",
    "avilantare": "avilantar",
    "avilantareis": "avilantar",
    "avilantaremos": "avilantar",
    "avilantaren": "avilantar",
    "avilantares": "avilantar",
    "avilantaron": "avilantar",
    "avilantará": "avilantar",
    "avilantarán": "avilantar",
    "avilantarás": "avilantar",
    "avilantaré": "avilantar",
    "avilantaréis": "avilantar",
    "avilantaría": "avilantar",
    "avilantaríais": "avilantar",
    "avilantaríamos": "avilantar",
    "avilantarían": "avilantar",
    "avilantarías": "avilantar",
    "avilantas": "avilantar",
    "avilantase": "avilantar",
    "avilantaseis": "avilantar",
    "avilantasen": "avilantar",
    "avilantases": "avilantar",
    "avilantaste": "avilantar",
    "avilantasteis": "avilantar",
    "avilante": "avilantar",
    "avilanteces": "avilantez",
    "avilantemos": "avilantar",
    "avilanten": "avilantar",
    "avilantes": "avilantar",
    "avilanto": "avilantar",
    "avilantábamos": "avilantar",
    "avilantáis": "avilantar",
    "avilantáramos": "avilantar",
    "avilantáremos": "avilantar",
    "avilantásemos": "avilantar",
    "avilanté": "avilantar",
    "avilantéis": "avilantar",
    "avilantó": "avilantar",
    "avilenses": "avilense",
    "avilesa": "avilés",
    "avilesas": "avilés",
    "avileses": "avilés",
    "avillana": "avillanar",
    "avillanaba": "avillanar",
    "avillanabais": "avillanar",
    "avillanaban": "avillanar",
    "avillanabas": "avillanar",
    "avillanad": "avillanar",
    "avillanada": "avillanar",
    "avillanadas": "avillanar",
    "avillanado": "avillanar",
    "avillanados": "avillanar",
    "avillanamientos": "avillanamiento",
    "avillanamos": "avillanar",
    "avillanan": "avillanar",
    "avillanando": "avillanar",
    "avillanara": "avillanar",
    "avillanarais": "avillanar",
    "avillanaran": "avillanar",
    "avillanaras": "avillanar",
    "avillanare": "avillanar",
    "avillanareis": "avillanar",
    "avillanaremos": "avillanar",
    "avillanaren": "avillanar",
    "avillanares": "avillanar",
    "avillanaron": "avillanar",
    "avillanará": "avillanar",
    "avillanarán": "avillanar",
    "avillanarás": "avillanar",
    "avillanaré": "avillanar",
    "avillanaréis": "avillanar",
    "avillanaría": "avillanar",
    "avillanaríais": "avillanar",
    "avillanaríamos": "avillanar",
    "avillanarían": "avillanar",
    "avillanarías": "avillanar",
    "avillanas": "avillanar",
    "avillanase": "avillanar",
    "avillanaseis": "avillanar",
    "avillanasen": "avillanar",
    "avillanases": "avillanar",
    "avillanaste": "avillanar",
    "avillanasteis": "avillanar",
    "avillane": "avillanar",
    "avillanemos": "avillanar",
    "avillanen": "avillanar",
    "avillanes": "avillanar",
    "avillano": "avillanar",
    "avillanábamos": "avillanar",
    "avillanáis": "avillanar",
    "avillanáramos": "avillanar",
    "avillanáremos": "avillanar",
    "avillanásemos": "avillanar",
    "avillané": "avillanar",
    "avillanéis": "avillanar",
    "avillanó": "avillanar",
    "avinagra": "avinagrar",
    "avinagraba": "avinagrar",
    "avinagrabais": "avinagrar",
    "avinagraban": "avinagrar",
    "avinagrabas": "avinagrar",
    "avinagrad": "avinagrar",
    "avinagrada": "avinagrar",
    "avinagradas": "avinagrar",
    "avinagrado": "avinagrar",
    "avinagrados": "avinagrar",
    "avinagramientos": "avinagramiento",
    "avinagramos": "avinagrar",
    "avinagran": "avinagrar",
    "avinagrando": "avinagrar",
    "avinagrara": "avinagrar",
    "avinagrarais": "avinagrar",
    "avinagraran": "avinagrar",
    "avinagraras": "avinagrar",
    "avinagrare": "avinagrar",
    "avinagrareis": "avinagrar",
    "avinagraremos": "avinagrar",
    "avinagraren": "avinagrar",
    "avinagrares": "avinagrar",
    "avinagraron": "avinagrar",
    "avinagrará": "avinagrar",
    "avinagrarán": "avinagrar",
    "avinagrarás": "avinagrar",
    "avinagraré": "avinagrar",
    "avinagraréis": "avinagrar",
    "avinagraría": "avinagrar",
    "avinagraríais": "avinagrar",
    "avinagraríamos": "avinagrar",
    "avinagrarían": "avinagrar",
    "avinagrarías": "avinagrar",
    "avinagras": "avinagrar",
    "avinagrase": "avinagrar",
    "avinagraseis": "avinagrar",
    "avinagrasen": "avinagrar",
    "avinagrases": "avinagrar",
    "avinagraste": "avinagrar",
    "avinagrasteis": "avinagrar",
    "avinagre": "avinagrar",
    "avinagremos": "avinagrar",
    "avinagren": "avinagrar",
    "avinagres": "avinagrar",
    "avinagro": "avinagrar",
    "avinagrábamos": "avinagrar",
    "avinagráis": "avinagrar",
    "avinagráramos": "avinagrar",
    "avinagráremos": "avinagrar",
    "avinagrásemos": "avinagrar",
    "avinagré": "avinagrar",
    "avinagréis": "avinagrar",
    "avinagró": "avinagrar",
    "avine": "avenir",
    "aviniendo": "avenir",
    "aviniera": "avenir",
    "avinierais": "avenir",
    "avinieran": "avenir",
    "avinieras": "avenir",
    "aviniere": "avenir",
    "aviniereis": "avenir",
    "avinieren": "avenir",
    "avinieres": "avenir",
    "avinieron": "avenir",
    "aviniese": "avenir",
    "avinieseis": "avenir",
    "aviniesen": "avenir",
    "avinieses": "avenir",
    "avinimos": "avenir",
    "aviniste": "avenir",
    "avinisteis": "avenir",
    "aviniéramos": "avenir",
    "aviniéremos": "avenir",
    "aviniésemos": "avenir",
    "avino": "avenir",
    "aviones": "avión",
    "avionetas": "avioneta",
    "avisa": "avisar",
    "avisaba": "avisar",
    "avisabais": "avisar",
    "avisaban": "avisar",
    "avisabas": "avisar",
    "avisad": "avisar",
    "avisada": "avisar",
    "avisadas": "avisar",
    "avisado": "avisar",
    "avisadora": "avisador",
    "avisadoras": "avisador",
    "avisadores": "avisador",
    "avisados": "avisar",
    "avisamos": "avisar",
    "avisan": "avisar",
    "avisando": "avisar",
    "avisara": "avisar",
    "avisarais": "avisar",
    "avisaran": "avisar",
    "avisaras": "avisar",
    "avisare": "avisar",
    "avisareis": "avisar",
    "avisaremos": "avisar",
    "avisaren": "avisar",
    "avisares": "avisar",
    "avisaron": "avisar",
    "avisará": "avisar",
    "avisarán": "avisar",
    "avisarás": "avisar",
    "avisaré": "avisar",
    "avisaréis": "avisar",
    "avisaría": "avisar",
    "avisaríais": "avisar",
    "avisaríamos": "avisar",
    "avisarían": "avisar",
    "avisarías": "avisar",
    "avisas": "avisar",
    "avisase": "avisar",
    "avisaseis": "avisar",
    "avisasen": "avisar",
    "avisases": "avisar",
    "avisaste": "avisar",
    "avisasteis": "avisar",
    "avise": "avisar",
    "avisemos": "avisar",
    "avisen": "avisar",
    "avises": "avisar",
    "aviso": "avisar",
    "avisos": "aviso",
    "avispa": "avispar",
    "avispaba": "avispar",
    "avispabais": "avispar",
    "avispaban": "avispar",
    "avispabas": "avispar",
    "avispad": "avispar",
    "avispada": "avispar",
    "avispadas": "avispar",
    "avispado": "avispar",
    "avispados": "avispar",
    "avispamos": "avispar",
    "avispan": "avispar",
    "avispando": "avispar",
    "avispara": "avispar",
    "avisparais": "avispar",
    "avisparan": "avispar",
    "avisparas": "avispar",
    "avispare": "avispar",
    "avispareis": "avispar",
    "avisparemos": "avispar",
    "avisparen": "avispar",
    "avispares": "avispar",
    "avisparon": "avispar",
    "avispará": "avispar",
    "avisparán": "avispar",
    "avisparás": "avispar",
    "avisparé": "avispar",
    "avisparéis": "avispar",
    "avisparía": "avispar",
    "avisparíais": "avispar",
    "avisparíamos": "avispar",
    "avisparían": "avispar",
    "avisparías": "avispar",
    "avispas": "avispar",
    "avispase": "avispar",
    "avispaseis": "avispar",
    "avispasen": "avispar",
    "avispases": "avispar",
    "avispaste": "avispar",
    "avispasteis": "avispar",
    "avispe": "avispar",
    "avispemos": "avispar",
    "avispen": "avispar",
    "avisperos": "avispero",
    "avispes": "avispar",
    "avispo": "avispar",
    "avispones": "avispón",
    "avispábamos": "avispar",
    "avispáis": "avispar",
    "avispáramos": "avispar",
    "avispáremos": "avispar",
    "avispásemos": "avispar",
    "avispé": "avispar",
    "avispéis": "avispar",
    "avispó": "avispar",
    "avista": "avistar",
    "avistaba": "avistar",
    "avistabais": "avistar",
    "avistaban": "avistar",
    "avistabas": "avistar",
    "avistad": "avistar",
    "avistada": "avistar",
    "avistadas": "avistar",
    "avistado": "avistar",
    "avistados": "avistar",
    "avistamientos": "avistamiento",
    "avistamos": "avistar",
    "avistan": "avistar",
    "avistando": "avistar",
    "avistara": "avistar",
    "avistarais": "avistar",
    "avistaran": "avistar",
    "avistaras": "avistar",
    "avistare": "avistar",
    "avistareis": "avistar",
    "avistaremos": "avistar",
    "avistaren": "avistar",
    "avistares": "avistar",
    "avistaron": "avistar",
    "avistará": "avistar",
    "avistarán": "avistar",
    "avistarás": "avistar",
    "avistaré": "avistar",
    "avistaréis": "avistar",
    "avistaría": "avistar",
    "avistaríais": "avistar",
    "avistaríamos": "avistar",
    "avistarían": "avistar",
    "avistarías": "avistar",
    "avistas": "avistar",
    "avistase": "avistar",
    "avistaseis": "avistar",
    "avistasen": "avistar",
    "avistases": "avistar",
    "avistaste": "avistar",
    "avistasteis": "avistar",
    "aviste": "avistar",
    "avistemos": "avistar",
    "avisten": "avistar",
    "avistes": "avistar",
    "avisto": "avistar",
    "avistábamos": "avistar",
    "avistáis": "avistar",
    "avistáramos": "avistar",
    "avistáremos": "avistar",
    "avistásemos": "avistar",
    "avisté": "avistar",
    "avistéis": "avistar",
    "avistó": "avistar",
    "avisábamos": "avisar",
    "avisáis": "avisar",
    "avisáramos": "avisar",
    "avisáremos": "avisar",
    "avisásemos": "avisar",
    "avisé": "avisar",
    "aviséis": "avisar",
    "avisó": "avisar",
    "avitualla": "avituallar",
    "avituallaba": "avituallar",
    "avituallabais": "avituallar",
    "avituallaban": "avituallar",
    "avituallabas": "avituallar",
    "avituallad": "avituallar",
    "avituallada": "avituallar",
    "avitualladas": "avituallar",
    "avituallado": "avituallar",
    "avitualladora": "avituallador",
    "avitualladoras": "avituallador",
    "avitualladores": "avituallador",
    "avituallados": "avituallar",
    "avituallamientos": "avituallamiento",
    "avituallamos": "avituallar",
    "avituallan": "avituallar",
    "avituallando": "avituallar",
    "avituallara": "avituallar",
    "avituallarais": "avituallar",
    "avituallaran": "avituallar",
    "avituallaras": "avituallar",
    "avituallare": "avituallar",
    "avituallareis": "avituallar",
    "avituallaremos": "avituallar",
    "avituallaren": "avituallar",
    "avituallares": "avituallar",
    "avituallaron": "avituallar",
    "avituallará": "avituallar",
    "avituallarán": "avituallar",
    "avituallarás": "avituallar",
    "avituallaré": "avituallar",
    "avituallaréis": "avituallar",
    "avituallaría": "avituallar",
    "avituallaríais": "avituallar",
    "avituallaríamos": "avituallar",
    "avituallarían": "avituallar",
    "avituallarías": "avituallar",
    "avituallas": "avituallar",
    "avituallase": "avituallar",
    "avituallaseis": "avituallar",
    "avituallasen": "avituallar",
    "avituallases": "avituallar",
    "avituallaste": "avituallar",
    "avituallasteis": "avituallar",
    "avitualle": "avituallar",
    "avituallemos": "avituallar",
    "avituallen": "avituallar",
    "avitualles": "avituallar",
    "avituallo": "avituallar",
    "avituallábamos": "avituallar",
    "avitualláis": "avituallar",
    "avitualláramos": "avituallar",
    "avitualláremos": "avituallar",
    "avituallásemos": "avituallar",
    "avituallé": "avituallar",
    "avitualléis": "avituallar",
    "avitualló": "avituallar",
    "aviva": "avivar",
    "avivaba": "avivar",
    "avivabais": "avivar",
    "avivaban": "avivar",
    "avivabas": "avivar",
    "avivad": "avivar",
    "avivada": "avivar",
    "avivadas": "avivar",
    "avivado": "avivar",
    "avivadora": "avivador",
    "avivadoras": "avivador",
    "avivadores": "avivador",
    "avivados": "avivar",
    "avivamientos": "avivamiento",
    "avivamos": "avivar",
    "avivan": "avivar",
    "avivando": "avivar",
    "avivara": "avivar",
    "avivarais": "avivar",
    "avivaran": "avivar",
    "avivaras": "avivar",
    "avivare": "avivar",
    "avivareis": "avivar",
    "avivaremos": "avivar",
    "avivaren": "avivar",
    "avivares": "avivar",
    "avivaron": "avivar",
    "avivará": "avivar",
    "avivarán": "avivar",
    "avivarás": "avivar",
    "avivaré": "avivar",
    "avivaréis": "avivar",
    "avivaría": "avivar",
    "avivaríais": "avivar",
    "avivaríamos": "avivar",
    "avivarían": "avivar",
    "avivarías": "avivar",
    "avivas": "avivar",
    "avivase": "avivar",
    "avivaseis": "avivar",
    "avivasen": "avivar",
    "avivases": "avivar",
    "avivaste": "avivar",
    "avivasteis": "avivar",
    "avive": "avivar",
    "avivemos": "avivar",
    "aviven": "avivar",
    "avives": "avivar",
    "avivo": "avivar",
    "avivábamos": "avivar",
    "aviváis": "avivar",
    "aviváramos": "avivar",
    "aviváremos": "avivar",
    "avivásemos": "avivar",
    "avivé": "avivar",
    "avivéis": "avivar",
    "avivó": "avivar",
    "avizora": "avizorar",
    "avizoraba": "avizorar",
    "avizorabais": "avizorar",
    "avizoraban": "avizorar",
    "avizorabas": "avizorar",
    "avizorad": "avizorar",
    "avizorada": "avizorar",
    "avizoradas": "avizorar",
    "avizorado": "avizorar",
    "avizoradora": "avizorador",
    "avizoradoras": "avizorador",
    "avizoradores": "avizorador",
    "avizorados": "avizorar",
    "avizoramientos": "avizoramiento",
    "avizoramos": "avizorar",
    "avizoran": "avizorar",
    "avizorando": "avizorar",
    "avizorara": "avizorar",
    "avizorarais": "avizorar",
    "avizoraran": "avizorar",
    "avizoraras": "avizorar",
    "avizorare": "avizorar",
    "avizorareis": "avizorar",
    "avizoraremos": "avizorar",
    "avizoraren": "avizorar",
    "avizorares": "avizorar",
    "avizoraron": "avizorar",
    "avizorará": "avizorar",
    "avizorarán": "avizorar",
    "avizorarás": "avizorar",
    "avizoraré": "avizorar",
    "avizoraréis": "avizorar",
    "avizoraría": "avizorar",
    "avizoraríais": "avizorar",
    "avizoraríamos": "avizorar",
    "avizorarían": "avizorar",
    "avizorarías": "avizorar",
    "avizoras": "avizorar",
    "avizorase": "avizorar",
    "avizoraseis": "avizorar",
    "avizorasen": "avizorar",
    "avizorases": "avizorar",
    "avizoraste": "avizorar",
    "avizorasteis": "avizorar",
    "avizore": "avizorar",
    "avizoremos": "avizorar",
    "avizoren": "avizorar",
    "avizores": "avizorar",
    "avizoro": "avizorar",
    "avizorábamos": "avizorar",
    "avizoráis": "avizorar",
    "avizoráramos": "avizorar",
    "avizoráremos": "avizorar",
    "avizorásemos": "avizorar",
    "avizoré": "avizorar",
    "avizoréis": "avizorar",
    "avizoró": "avizorar",
    "aviábamos": "aviar",
    "aviáis": "aviar",
    "aviáramos": "aviar",
    "aviáremos": "aviar",
    "aviásemos": "aviar",
    "avié": "aviar",
    "aviéis": "aviar",
    "avió": "aviar",
    "avoca": "avocar",
    "avocaba": "avocar",
    "avocabais": "avocar",
    "avocaban": "avocar",
    "avocabas": "avocar",
    "avocaciones": "avocación",
    "avocad": "avocar",
    "avocada": "avocar",
    "avocadas": "avocar",
    "avocado": "avocar",
    "avocados": "avocar",
    "avocamientos": "avocamiento",
    "avocamos": "avocar",
    "avocan": "avocar",
    "avocando": "avocar",
    "avocara": "avocar",
    "avocarais": "avocar",
    "avocaran": "avocar",
    "avocaras": "avocar",
    "avocare": "avocar",
    "avocareis": "avocar",
    "avocaremos": "avocar",
    "avocaren": "avocar",
    "avocares": "avocar",
    "avocaron": "avocar",
    "avocará": "avocar",
    "avocarán": "avocar",
    "avocarás": "avocar",
    "avocaré": "avocar",
    "avocaréis": "avocar",
    "avocaría": "avocar",
    "avocaríais": "avocar",
    "avocaríamos": "avocar",
    "avocarían": "avocar",
    "avocarías": "avocar",
    "avocas": "avocar",
    "avocase": "avocar",
    "avocaseis": "avocar",
    "avocasen": "avocar",
    "avocases": "avocar",
    "avocaste": "avocar",
    "avocasteis": "avocar",
    "avocetas": "avoceta",
    "avoco": "avocar",
    "avocábamos": "avocar",
    "avocáis": "avocar",
    "avocáramos": "avocar",
    "avocáremos": "avocar",
    "avocásemos": "avocar",
    "avocó": "avocar",
    "avoque": "avocar",
    "avoquemos": "avocar",
    "avoquen": "avocar",
    "avoques": "avocar",
    "avoqué": "avocar",
    "avoquéis": "avocar",
    "avulsiones": "avulsión",
    "avunculocales": "avunculocal",
    "avutardas": "avutarda",
    "avéstica": "avéstico",
    "avésticas": "avéstico",
    "avésticos": "avéstico",
    "avía": "aviar",
    "avían": "aviar",
    "avías": "aviar",
    "avícolas": "avícola",
    "avíe": "aviar",
    "avíen": "aviar",
    "avíes": "aviar",
    "avío": "aviar",
    "avíos": "avío",
    "axiales": "axial",
    "axilares": "axilar",
    "axilas": "axila",
    "axiles": "axil",
    "axinitas": "axinita",
    "axiodistales": "axiodistal",
    "axiolinguales": "axiolingual",
    "axiologías": "axiología",
    "axiomas": "axioma",
    "axiomatizaciones": "axiomatización",
    "axiomesiales": "axiomesial",
    "axiomática": "axiomático",
    "axiomáticas": "axiomático",
    "axiomáticos": "axiomático",
    "axiómetros": "axiómetro",
    "axolemas": "axolema",
    "axolotes": "axolote",
    "axonemas": "axonema",
    "axones": "axón",
    "axonometrías": "axonometría",
    "axonométrica": "axonométrico",
    "axonométricas": "axonométrico",
    "axonométricos": "axonométrico",
    "axénica": "axénico",
    "axénicas": "axénico",
    "axénicos": "axénico",
    "axónica": "axónico",
    "axónicas": "axónico",
    "axónicos": "axónico",
    "axópodos": "axópodo",
    "aya": "ayo",
    "ayas": "ayo",
    "ayatollahs": "ayatollah",
    "ayatolás": "ayatolá",
    "ayea": "ayear",
    "ayeaba": "ayear",
    "ayeabais": "ayear",
    "ayeaban": "ayear",
    "ayeabas": "ayear",
    "ayead": "ayear",
    "ayeada": "ayear",
    "ayeadas": "ayear",
    "ayeado": "ayear",
    "ayeados": "ayear",
    "ayeamos": "ayear",
    "ayean": "ayear",
    "ayeando": "ayear",
    "ayeara": "ayear",
    "ayearais": "ayear",
    "ayearan": "ayear",
    "ayearas": "ayear",
    "ayeare": "ayear",
    "ayeareis": "ayear",
    "ayearemos": "ayear",
    "ayearen": "ayear",
    "ayeares": "ayear",
    "ayearon": "ayear",
    "ayeará": "ayear",
    "ayearán": "ayear",
    "ayearás": "ayear",
    "ayearé": "ayear",
    "ayearéis": "ayear",
    "ayearía": "ayear",
    "ayearíais": "ayear",
    "ayearíamos": "ayear",
    "ayearían": "ayear",
    "ayearías": "ayear",
    "ayeas": "ayear",
    "ayease": "ayear",
    "ayeaseis": "ayear",
    "ayeasen": "ayear",
    "ayeases": "ayear",
    "ayeaste": "ayear",
    "ayeasteis": "ayear",
    "ayee": "ayear",
    "ayeemos": "ayear",
    "ayeen": "ayear",
    "ayees": "ayear",
    "ayeo": "ayear",
    "ayeres": "ayer",
    "ayeábamos": "ayear",
    "ayeáis": "ayear",
    "ayeáramos": "ayear",
    "ayeáremos": "ayear",
    "ayeásemos": "ayear",
    "ayeé": "ayear",
    "ayeéis": "ayear",
    "ayeó": "ayear",
    "ayllúes": "ayllú",
    "ayllús": "ayllú",
    "ayos": "ayo",
    "ayotes": "ayote",
    "ayuda": "ayudar",
    "ayudaba": "ayudar",
    "ayudabais": "ayudar",
    "ayudaban": "ayudar",
    "ayudabas": "ayudar",
    "ayudad": "ayudar",
    "ayudada": "ayudar",
    "ayudadas": "ayudar",
    "ayudado": "ayudar",
    "ayudadora": "ayudador",
    "ayudadoras": "ayudador",
    "ayudadores": "ayudador",
    "ayudados": "ayudar",
    "ayudamos": "ayudar",
    "ayudan": "ayudar",
    "ayudando": "ayudar",
    "ayudantes": "ayudante",
    "ayudantías": "ayudantía",
    "ayudara": "ayudar",
    "ayudarais": "ayudar",
    "ayudaran": "ayudar",
    "ayudaras": "ayudar",
    "ayudare": "ayudar",
    "ayudareis": "ayudar",
    "ayudaremos": "ayudar",
    "ayudaren": "ayudar",
    "ayudares": "ayudar",
    "ayudaron": "ayudar",
    "ayudará": "ayudar",
    "ayudarán": "ayudar",
    "ayudarás": "ayudar",
    "ayudaré": "ayudar",
    "ayudaréis": "ayudar",
    "ayudaría": "ayudar",
    "ayudaríais": "ayudar",
    "ayudaríamos": "ayudar",
    "ayudarían": "ayudar",
    "ayudarías": "ayudar",
    "ayudas": "ayudar",
    "ayudase": "ayudar",
    "ayudaseis": "ayudar",
    "ayudasen": "ayudar",
    "ayudases": "ayudar",
    "ayudaste": "ayudar",
    "ayudasteis": "ayudar",
    "ayude": "ayudar",
    "ayudemos": "ayudar",
    "ayuden": "ayudar",
    "ayudes": "ayudar",
    "ayudo": "ayudar",
    "ayudábamos": "ayudar",
    "ayudáis": "ayudar",
    "ayudáramos": "ayudar",
    "ayudáremos": "ayudar",
    "ayudásemos": "ayudar",
    "ayudé": "ayudar",
    "ayudéis": "ayudar",
    "ayudó": "ayudar",
    "ayuna": "ayuno",
    "ayunaba": "ayunar",
    "ayunabais": "ayunar",
    "ayunaban": "ayunar",
    "ayunabas": "ayunar",
    "ayunad": "ayunar",
    "ayunada": "ayunar",
    "ayunadas": "ayunar",
    "ayunado": "ayunar",
    "ayunados": "ayunar",
    "ayunamos": "ayunar",
    "ayunan": "ayunar",
    "ayunando": "ayunar",
    "ayunara": "ayunar",
    "ayunarais": "ayunar",
    "ayunaran": "ayunar",
    "ayunaras": "ayunar",
    "ayunare": "ayunar",
    "ayunareis": "ayunar",
    "ayunaremos": "ayunar",
    "ayunaren": "ayunar",
    "ayunares": "ayunar",
    "ayunaron": "ayunar",
    "ayunará": "ayunar",
    "ayunarán": "ayunar",
    "ayunarás": "ayunar",
    "ayunaré": "ayunar",
    "ayunaréis": "ayunar",
    "ayunaría": "ayunar",
    "ayunaríais": "ayunar",
    "ayunaríamos": "ayunar",
    "ayunarían": "ayunar",
    "ayunarías": "ayunar",
    "ayunas": "ayuno",
    "ayunase": "ayunar",
    "ayunaseis": "ayunar",
    "ayunasen": "ayunar",
    "ayunases": "ayunar",
    "ayunaste": "ayunar",
    "ayunasteis": "ayunar",
    "ayune": "ayunar",
    "ayunemos": "ayunar",
    "ayunen": "ayunar",
    "ayunes": "ayunar",
    "ayuno": "ayunar",
    "ayunos": "ayuno",
    "ayuntamientos": "ayuntamiento",
    "ayunábamos": "ayunar",
    "ayunáis": "ayunar",
    "ayunáramos": "ayunar",
    "ayunáremos": "ayunar",
    "ayunásemos": "ayunar",
    "ayuné": "ayunar",
    "ayunéis": "ayunar",
    "ayunó": "ayunar",
    "ayusta": "ayustar",
    "ayustaba": "ayustar",
    "ayustabais": "ayustar",
    "ayustaban": "ayustar",
    "ayustabas": "ayustar",
    "ayustad": "ayustar",
    "ayustada": "ayustar",
    "ayustadas": "ayustar",
    "ayustado": "ayustar",
    "ayustados": "ayustar",
    "ayustamos": "ayustar",
    "ayustan": "ayustar",
    "ayustando": "ayustar",
    "ayustara": "ayustar",
    "ayustarais": "ayustar",
    "ayustaran": "ayustar",
    "ayustaras": "ayustar",
    "ayustare": "ayustar",
    "ayustareis": "ayustar",
    "ayustaremos": "ayustar",
    "ayustaren": "ayustar",
    "ayustares": "ayustar",
    "ayustaron": "ayustar",
    "ayustará": "ayustar",
    "ayustarán": "ayustar",
    "ayustarás": "ayustar",
    "ayustaré": "ayustar",
    "ayustaréis": "ayustar",
    "ayustaría": "ayustar",
    "ayustaríais": "ayustar",
    "ayustaríamos": "ayustar",
    "ayustarían": "ayustar",
    "ayustarías": "ayustar",
    "ayustas": "ayustar",
    "ayustase": "ayustar",
    "ayustaseis": "ayustar",
    "ayustasen": "ayustar",
    "ayustases": "ayustar",
    "ayustaste": "ayustar",
    "ayustasteis": "ayustar",
    "ayuste": "ayustar",
    "ayustemos": "ayustar",
    "ayusten": "ayustar",
    "ayustes": "ayuste",
    "ayusto": "ayustar",
    "ayustábamos": "ayustar",
    "ayustáis": "ayustar",
    "ayustáramos": "ayustar",
    "ayustáremos": "ayustar",
    "ayustásemos": "ayustar",
    "ayusté": "ayustar",
    "ayustéis": "ayustar",
    "ayustó": "ayustar",
    "ayyubíes": "ayyubí",
    "ayyubís": "ayyubí",
    "azabaches": "azabache",
    "azacana": "azacanar",
    "azacanaba": "azacanar",
    "azacanabais": "azacanar",
    "azacanaban": "azacanar",
    "azacanabas": "azacanar",
    "azacanad": "azacanar",
    "azacanada": "azacanar",
    "azacanadas": "azacanar",
    "azacanado": "azacanar",
    "azacanados": "azacanar",
    "azacanamos": "azacanar",
    "azacanan": "azacanar",
    "azacanando": "azacanar",
    "azacanara": "azacanar",
    "azacanarais": "azacanar",
    "azacanaran": "azacanar",
    "azacanaras": "azacanar",
    "azacanare": "azacanar",
    "azacanareis": "azacanar",
    "azacanaremos": "azacanar",
    "azacanaren": "azacanar",
    "azacanares": "azacanar",
    "azacanaron": "azacanar",
    "azacanará": "azacanar",
    "azacanarán": "azacanar",
    "azacanarás": "azacanar",
    "azacanaré": "azacanar",
    "azacanaréis": "azacanar",
    "azacanaría": "azacanar",
    "azacanaríais": "azacanar",
    "azacanaríamos": "azacanar",
    "azacanarían": "azacanar",
    "azacanarías": "azacanar",
    "azacanas": "azacanar",
    "azacanase": "azacanar",
    "azacanaseis": "azacanar",
    "azacanasen": "azacanar",
    "azacanases": "azacanar",
    "azacanaste": "azacanar",
    "azacanasteis": "azacanar",
    "azacane": "azacanar",
    "azacanemos": "azacanar",
    "azacanen": "azacanar",
    "azacaneos": "azacaneo",
    "azacanes": "azacanar",
    "azacano": "azacanar",
    "azacanábamos": "azacanar",
    "azacanáis": "azacanar",
    "azacanáramos": "azacanar",
    "azacanáremos": "azacanar",
    "azacanásemos": "azacanar",
    "azacané": "azacanar",
    "azacanéis": "azacanar",
    "azacanó": "azacanar",
    "azadas": "azada",
    "azadillas": "azadilla",
    "azadonazos": "azadonazo",
    "azadones": "azadón",
    "azafatas": "azafata",
    "azafates": "azafate",
    "azafeas": "azafea",
    "azafrana": "azafranar",
    "azafranaba": "azafranar",
    "azafranabais": "azafranar",
    "azafranaban": "azafranar",
    "azafranabas": "azafranar",
    "azafranad": "azafranar",
    "azafranada": "azafranar",
    "azafranadas": "azafranar",
    "azafranado": "azafranar",
    "azafranados": "azafranar",
    "azafranales": "azafranal",
    "azafranamos": "azafranar",
    "azafranan": "azafranar",
    "azafranando": "azafranar",
    "azafranara": "azafranar",
    "azafranarais": "azafranar",
    "azafranaran": "azafranar",
    "azafranaras": "azafranar",
    "azafranare": "azafranar",
    "azafranareis": "azafranar",
    "azafranaremos": "azafranar",
    "azafranaren": "azafranar",
    "azafranares": "azafranar",
    "azafranaron": "azafranar",
    "azafranará": "azafranar",
    "azafranarán": "azafranar",
    "azafranarás": "azafranar",
    "azafranaré": "azafranar",
    "azafranaréis": "azafranar",
    "azafranaría": "azafranar",
    "azafranaríais": "azafranar",
    "azafranaríamos": "azafranar",
    "azafranarían": "azafranar",
    "azafranarías": "azafranar",
    "azafranas": "azafranar",
    "azafranase": "azafranar",
    "azafranaseis": "azafranar",
    "azafranasen": "azafranar",
    "azafranases": "azafranar",
    "azafranaste": "azafranar",
    "azafranasteis": "azafranar",
    "azafrane": "azafranar",
    "azafranemos": "azafranar",
    "azafranen": "azafranar",
    "azafranes": "azafranar",
    "azafrano": "azafranar",
    "azafranábamos": "azafranar",
    "azafranáis": "azafranar",
    "azafranáramos": "azafranar",
    "azafranáremos": "azafranar",
    "azafranásemos": "azafranar",
    "azafrané": "azafranar",
    "azafranéis": "azafranar",
    "azafranó": "azafranar",
    "azahares": "azahar",
    "azaleas": "azalea",
    "azanahoriates": "azanahoriate",
    "azancas": "azanca",
    "azanorias": "azanoria",
    "azara": "azarar",
    "azaraba": "azarar",
    "azarabais": "azarar",
    "azaraban": "azarar",
    "azarabas": "azarar",
    "azarad": "azarar",
    "azarada": "azarar",
    "azaradas": "azarar",
    "azarado": "azarar",
    "azarados": "azarar",
    "azaramos": "azarar",
    "azaran": "azarar",
    "azaranda": "azarandar",
    "azarandaba": "azarandar",
    "azarandabais": "azarandar",
    "azarandaban": "azarandar",
    "azarandabas": "azarandar",
    "azarandad": "azarandar",
    "azarandada": "azarandar",
    "azarandadas": "azarandar",
    "azarandado": "azarandar",
    "azarandados": "azarandar",
    "azarandamos": "azarandar",
    "azarandan": "azarandar",
    "azarandando": "azarandar",
    "azarandara": "azarandar",
    "azarandarais": "azarandar",
    "azarandaran": "azarandar",
    "azarandaras": "azarandar",
    "azarandare": "azarandar",
    "azarandareis": "azarandar",
    "azarandaremos": "azarandar",
    "azarandaren": "azarandar",
    "azarandares": "azarandar",
    "azarandaron": "azarandar",
    "azarandará": "azarandar",
    "azarandarán": "azarandar",
    "azarandarás": "azarandar",
    "azarandaré": "azarandar",
    "azarandaréis": "azarandar",
    "azarandaría": "azarandar",
    "azarandaríais": "azarandar",
    "azarandaríamos": "azarandar",
    "azarandarían": "azarandar",
    "azarandarías": "azarandar",
    "azarandas": "azarandar",
    "azarandase": "azarandar",
    "azarandaseis": "azarandar",
    "azarandasen": "azarandar",
    "azarandases": "azarandar",
    "azarandaste": "azarandar",
    "azarandasteis": "azarandar",
    "azarande": "azarandar",
    "azarandemos": "azarandar",
    "azaranden": "azarandar",
    "azarandes": "azarandar",
    "azarando": "azarar",
    "azarandábamos": "azarandar",
    "azarandáis": "azarandar",
    "azarandáramos": "azarandar",
    "azarandáremos": "azarandar",
    "azarandásemos": "azarandar",
    "azarandé": "azarandar",
    "azarandéis": "azarandar",
    "azarandó": "azarandar",
    "azarara": "azarar",
    "azararais": "azarar",
    "azararan": "azarar",
    "azararas": "azarar",
    "azarare": "azarar",
    "azarareis": "azarar",
    "azararemos": "azarar",
    "azararen": "azarar",
    "azarares": "azarar",
    "azararon": "azarar",
    "azarará": "azarar",
    "azararán": "azarar",
    "azararás": "azarar",
    "azararé": "azarar",
    "azararéis": "azarar",
    "azararía": "azarar",
    "azararíais": "azarar",
    "azararíamos": "azarar",
    "azararían": "azarar",
    "azararías": "azarar",
    "azaras": "azarar",
    "azarase": "azarar",
    "azaraseis": "azarar",
    "azarasen": "azarar",
    "azarases": "azarar",
    "azaraste": "azarar",
    "azarasteis": "azarar",
    "azarbes": "azarbe",
    "azarcones": "azarcón",
    "azare": "azarar",
    "azarea": "azarear",
    "azareaba": "azarear",
    "azareabais": "azarear",
    "azareaban": "azarear",
    "azareabas": "azarear",
    "azaread": "azarear",
    "azareada": "azarear",
    "azareadas": "azarear",
    "azareado": "azarear",
    "azareados": "azarear",
    "azareamos": "azarear",
    "azarean": "azarear",
    "azareando": "azarear",
    "azareara": "azarear",
    "azarearais": "azarear",
    "azarearan": "azarear",
    "azarearas": "azarear",
    "azareare": "azarear",
    "azareareis": "azarear",
    "azarearemos": "azarear",
    "azarearen": "azarear",
    "azareares": "azarear",
    "azarearon": "azarear",
    "azareará": "azarear",
    "azarearán": "azarear",
    "azarearás": "azarear",
    "azarearé": "azarear",
    "azarearéis": "azarear",
    "azarearía": "azarear",
    "azarearíais": "azarear",
    "azarearíamos": "azarear",
    "azarearían": "azarear",
    "azarearías": "azarear",
    "azareas": "azarear",
    "azarease": "azarear",
    "azareaseis": "azarear",
    "azareasen": "azarear",
    "azareases": "azarear",
    "azareaste": "azarear",
    "azareasteis": "azarear",
    "azaree": "azarear",
    "azareemos": "azarear",
    "azareen": "azarear",
    "azarees": "azarear",
    "azaremos": "azarar",
    "azaren": "azarar",
    "azareo": "azarear",
    "azares": "azarar",
    "azareábamos": "azarear",
    "azareáis": "azarear",
    "azareáramos": "azarear",
    "azareáremos": "azarear",
    "azareásemos": "azarear",
    "azareé": "azarear",
    "azareéis": "azarear",
    "azareó": "azarear",
    "azarjas": "azarja",
    "azaro": "azarar",
    "azarosa": "azaroso",
    "azarosas": "azaroso",
    "azarosos": "azaroso",
    "azarábamos": "azarar",
    "azaráis": "azarar",
    "azaráramos": "azarar",
    "azaráremos": "azarar",
    "azarásemos": "azarar",
    "azaré": "azarar",
    "azaréis": "azarar",
    "azaró": "azarar",
    "azaserinas": "azaserina",
    "azatioprinas": "azatioprina",
    "azconas": "azcona",
    "azelatos": "azelato",
    "azeotropos": "azeotropo",
    "azeotrópica": "azeotrópico",
    "azeotrópicas": "azeotrópico",
    "azeotrópicos": "azeotrópico",
    "azerbaijana": "azerbaijano",
    "azerbaijanas": "azerbaijano",
    "azerbaijanos": "azerbaijano",
    "azerbaiyana": "azerbaiyano",
    "azerbaiyanas": "azerbaiyano",
    "azerbaiyanos": "azerbaiyano",
    "azerbayana": "azerbayano",
    "azerbayanas": "azerbayano",
    "azerbayanos": "azerbayano",
    "azidas": "azida",
    "azimutales": "azimutal",
    "azimutes": "azimut",
    "azinas": "azina",
    "aznachos": "aznacho",
    "aznallos": "aznallo",
    "aznares": "aznar",
    "azoa": "azoar",
    "azoaba": "azoar",
    "azoabais": "azoar",
    "azoaban": "azoar",
    "azoabas": "azoar",
    "azoad": "azoar",
    "azoada": "azoar",
    "azoadas": "azoar",
    "azoado": "azoar",
    "azoados": "azoar",
    "azoamos": "azoar",
    "azoan": "azoar",
    "azoando": "azoar",
    "azoara": "azoar",
    "azoarais": "azoar",
    "azoaran": "azoar",
    "azoaras": "azoar",
    "azoare": "azoar",
    "azoareis": "azoar",
    "azoaremos": "azoar",
    "azoaren": "azoar",
    "azoares": "azoar",
    "azoaron": "azoar",
    "azoará": "azoar",
    "azoarán": "azoar",
    "azoarás": "azoar",
    "azoaré": "azoar",
    "azoaréis": "azoar",
    "azoaría": "azoar",
    "azoaríais": "azoar",
    "azoaríamos": "azoar",
    "azoarían": "azoar",
    "azoarías": "azoar",
    "azoas": "azoar",
    "azoase": "azoar",
    "azoaseis": "azoar",
    "azoasen": "azoar",
    "azoases": "azoar",
    "azoaste": "azoar",
    "azoasteis": "azoar",
    "azobencenos": "azobenceno",
    "azoe": "azoar",
    "azoemos": "azoar",
    "azoen": "azoar",
    "azoes": "azoar",
    "azofeninas": "azofenina",
    "azoga": "azogar",
    "azogaba": "azogar",
    "azogabais": "azogar",
    "azogaban": "azogar",
    "azogabas": "azogar",
    "azogad": "azogar",
    "azogada": "azogar",
    "azogadas": "azogar",
    "azogado": "azogar",
    "azogados": "azogar",
    "azogamientos": "azogamiento",
    "azogamos": "azogar",
    "azogan": "azogar",
    "azogando": "azogar",
    "azogara": "azogar",
    "azogarais": "azogar",
    "azogaran": "azogar",
    "azogaras": "azogar",
    "azogare": "azogar",
    "azogareis": "azogar",
    "azogaremos": "azogar",
    "azogaren": "azogar",
    "azogares": "azogar",
    "azogaron": "azogar",
    "azogará": "azogar",
    "azogarán": "azogar",
    "azogarás": "azogar",
    "azogaré": "azogar",
    "azogaréis": "azogar",
    "azogaría": "azogar",
    "azogaríais": "azogar",
    "azogaríamos": "azogar",
    "azogarían": "azogar",
    "azogarías": "azogar",
    "azogas": "azogar",
    "azogase": "azogar",
    "azogaseis": "azogar",
    "azogasen": "azogar",
    "azogases": "azogar",
    "azogaste": "azogar",
    "azogasteis": "azogar",
    "azogo": "azogar",
    "azogue": "azogar",
    "azoguemos": "azogar",
    "azoguen": "azogar",
    "azogues": "azogue",
    "azogué": "azogar",
    "azoguéis": "azogar",
    "azogábamos": "azogar",
    "azogáis": "azogar",
    "azogáramos": "azogar",
    "azogáremos": "azogar",
    "azogásemos": "azogar",
    "azogó": "azogar",
    "azoica": "azoico",
    "azoicas": "azoico",
    "azoicos": "azoico",
    "azoimidas": "azoimida",
    "azolaba": "azolar",
    "azolabais": "azolar",
    "azolaban": "azolar",
    "azolabas": "azolar",
    "azolad": "azolar",
    "azolada": "azolar",
    "azoladas": "azolar",
    "azolado": "azolar",
    "azolados": "azolar",
    "azolamos": "azolar",
    "azolando": "azolar",
    "azolara": "azolar",
    "azolarais": "azolar",
    "azolaran": "azolar",
    "azolaras": "azolar",
    "azolare": "azolar",
    "azolareis": "azolar",
    "azolaremos": "azolar",
    "azolaren": "azolar",
    "azolares": "azolar",
    "azolaron": "azolar",
    "azolará": "azolar",
    "azolarán": "azolar",
    "azolarás": "azolar",
    "azolaré": "azolar",
    "azolaréis": "azolar",
    "azolaría": "azolar",
    "azolaríais": "azolar",
    "azolaríamos": "azolar",
    "azolarían": "azolar",
    "azolarías": "azolar",
    "azolase": "azolar",
    "azolaseis": "azolar",
    "azolasen": "azolar",
    "azolases": "azolar",
    "azolaste": "azolar",
    "azolasteis": "azolar",
    "azolemos": "azolar",
    "azolva": "azolvar",
    "azolvaba": "azolvar",
    "azolvabais": "azolvar",
    "azolvaban": "azolvar",
    "azolvabas": "azolvar",
    "azolvad": "azolvar",
    "azolvada": "azolvar",
    "azolvadas": "azolvar",
    "azolvado": "azolvar",
    "azolvados": "azolvar",
    "azolvamos": "azolvar",
    "azolvan": "azolvar",
    "azolvando": "azolvar",
    "azolvara": "azolvar",
    "azolvarais": "azolvar",
    "azolvaran": "azolvar",
    "azolvaras": "azolvar",
    "azolvare": "azolvar",
    "azolvareis": "azolvar",
    "azolvaremos": "azolvar",
    "azolvaren": "azolvar",
    "azolvares": "azolvar",
    "azolvaron": "azolvar",
    "azolvará": "azolvar",
    "azolvarán": "azolvar",
    "azolvarás": "azolvar",
    "azolvaré": "azolvar",
    "azolvaréis": "azolvar",
    "azolvaría": "azolvar",
    "azolvaríais": "azolvar",
    "azolvaríamos": "azolvar",
    "azolvarían": "azolvar",
    "azolvarías": "azolvar",
    "azolvas": "azolvar",
    "azolvase": "azolvar",
    "azolvaseis": "azolvar",
    "azolvasen": "azolvar",
    "azolvases": "azolvar",
    "azolvaste": "azolvar",
    "azolvasteis": "azolvar",
    "azolve": "azolvar",
    "azolvemos": "azolvar",
    "azolven": "azolvar",
    "azolves": "azolvar",
    "azolvo": "azolvar",
    "azolvábamos": "azolvar",
    "azolváis": "azolvar",
    "azolváramos": "azolvar",
    "azolváremos": "azolvar",
    "azolvásemos": "azolvar",
    "azolvé": "azolvar",
    "azolvéis": "azolvar",
    "azolvó": "azolvar",
    "azolábamos": "azolar",
    "azoláis": "azolar",
    "azoláramos": "azolar",
    "azoláremos": "azolar",
    "azolásemos": "azolar",
    "azolé": "azolar",
    "azoléis": "azolar",
    "azoló": "azolar",
    "azonales": "azonal",
    "azoo": "azoar",
    "azoospermias": "azoospermia",
    "azora": "azorar",
    "azoraba": "azorar",
    "azorabais": "azorar",
    "azoraban": "azorar",
    "azorabas": "azorar",
    "azorad": "azorar",
    "azorada": "azorar",
    "azoradas": "azorar",
    "azorado": "azorar",
    "azorados": "azorar",
    "azoramientos": "azoramiento",
    "azoramos": "azorar",
    "azoran": "azorar",
    "azorando": "azorar",
    "azorara": "azorar",
    "azorarais": "azorar",
    "azoraran": "azorar",
    "azoraras": "azorar",
    "azorare": "azorar",
    "azorareis": "azorar",
    "azoraremos": "azorar",
    "azoraren": "azorar",
    "azorares": "azorar",
    "azoraron": "azorar",
    "azorará": "azorar",
    "azorarán": "azorar",
    "azorarás": "azorar",
    "azoraré": "azorar",
    "azoraréis": "azorar",
    "azoraría": "azorar",
    "azoraríais": "azorar",
    "azoraríamos": "azorar",
    "azorarían": "azorar",
    "azorarías": "azorar",
    "azoras": "azorar",
    "azorase": "azorar",
    "azoraseis": "azorar",
    "azorasen": "azorar",
    "azorases": "azorar",
    "azoraste": "azorar",
    "azorasteis": "azorar",
    "azore": "azorar",
    "azoremos": "azorar",
    "azoren": "azorar",
    "azores": "azorar",
    "azoreña": "azoreño",
    "azoreñas": "azoreño",
    "azoreños": "azoreño",
    "azoro": "azorar",
    "azoros": "azoro",
    "azorra": "azorrar",
    "azorraba": "azorrar",
    "azorrabais": "azorrar",
    "azorraban": "azorrar",
    "azorrabas": "azorrar",
    "azorrad": "azorrar",
    "azorrada": "azorrar",
    "azorradas": "azorrar",
    "azorrado": "azorrar",
    "azorrados": "azorrar",
    "azorramientos": "azorramiento",
    "azorramos": "azorrar",
    "azorran": "azorrar",
    "azorrando": "azorrar",
    "azorrara": "azorrar",
    "azorrarais": "azorrar",
    "azorraran": "azorrar",
    "azorraras": "azorrar",
    "azorrare": "azorrar",
    "azorrareis": "azorrar",
    "azorraremos": "azorrar",
    "azorraren": "azorrar",
    "azorrares": "azorrar",
    "azorraron": "azorrar",
    "azorrará": "azorrar",
    "azorrarán": "azorrar",
    "azorrarás": "azorrar",
    "azorraré": "azorrar",
    "azorraréis": "azorrar",
    "azorraría": "azorrar",
    "azorraríais": "azorrar",
    "azorraríamos": "azorrar",
    "azorrarían": "azorrar",
    "azorrarías": "azorrar",
    "azorras": "azorrar",
    "azorrase": "azorrar",
    "azorraseis": "azorrar",
    "azorrasen": "azorrar",
    "azorrases": "azorrar",
    "azorraste": "azorrar",
    "azorrasteis": "azorrar",
    "azorre": "azorrar",
    "azorremos": "azorrar",
    "azorren": "azorrar",
    "azorres": "azorrar",
    "azorro": "azorrar",
    "azorrábamos": "azorrar",
    "azorráis": "azorrar",
    "azorráramos": "azorrar",
    "azorráremos": "azorrar",
    "azorrásemos": "azorrar",
    "azorré": "azorrar",
    "azorréis": "azorrar",
    "azorró": "azorrar",
    "azorábamos": "azorar",
    "azoráis": "azorar",
    "azoráramos": "azorar",
    "azoráremos": "azorar",
    "azorásemos": "azorar",
    "azoré": "azorar",
    "azoréis": "azorar",
    "azoró": "azorar",
    "azota": "azotar",
    "azotaba": "azotar",
    "azotabais": "azotar",
    "azotaban": "azotar",
    "azotabas": "azotar",
    "azotad": "azotar",
    "azotada": "azotar",
    "azotadas": "azotar",
    "azotado": "azotar",
    "azotadora": "azotador",
    "azotadoras": "azotador",
    "azotadores": "azotador",
    "azotados": "azotar",
    "azotainas": "azotaina",
    "azotamientos": "azotamiento",
    "azotamos": "azotar",
    "azotan": "azotar",
    "azotando": "azotar",
    "azotara": "azotar",
    "azotarais": "azotar",
    "azotaran": "azotar",
    "azotaras": "azotar",
    "azotare": "azotar",
    "azotareis": "azotar",
    "azotaremos": "azotar",
    "azotaren": "azotar",
    "azotares": "azotar",
    "azotaron": "azotar",
    "azotará": "azotar",
    "azotarán": "azotar",
    "azotarás": "azotar",
    "azotaré": "azotar",
    "azotaréis": "azotar",
    "azotaría": "azotar",
    "azotaríais": "azotar",
    "azotaríamos": "azotar",
    "azotarían": "azotar",
    "azotarías": "azotar",
    "azotas": "azotar",
    "azotasas": "azotasa",
    "azotase": "azotar",
    "azotaseis": "azotar",
    "azotasen": "azotar",
    "azotases": "azotar",
    "azotaste": "azotar",
    "azotasteis": "azotar",
    "azote": "azotar",
    "azoteas": "azotea",
    "azotemos": "azotar",
    "azoten": "azotar",
    "azotes": "azote",
    "azotinas": "azotina",
    "azoto": "azotar",
    "azoturias": "azoturia",
    "azotábamos": "azotar",
    "azotáis": "azotar",
    "azotáramos": "azotar",
    "azotáremos": "azotar",
    "azotásemos": "azotar",
    "azoté": "azotar",
    "azotéis": "azotar",
    "azotó": "azotar",
    "azotómetros": "azotómetro",
    "azoábamos": "azoar",
    "azoáis": "azoar",
    "azoáramos": "azoar",
    "azoáremos": "azoar",
    "azoásemos": "azoar",
    "azoé": "azoar",
    "azoéis": "azoar",
    "azoó": "azoar",
    "azoúrias": "azoúria",
    "aztecas": "azteca",
    "azucara": "azucarar",
    "azucaraba": "azucarar",
    "azucarabais": "azucarar",
    "azucaraban": "azucarar",
    "azucarabas": "azucarar",
    "azucarad": "azucarar",
    "azucarada": "azucarar",
    "azucaradas": "azucarar",
    "azucarado": "azucarar",
    "azucarados": "azucarar",
    "azucaramientos": "azucaramiento",
    "azucaramos": "azucarar",
    "azucaran": "azucarar",
    "azucarando": "azucarar",
    "azucarara": "azucarar",
    "azucararais": "azucarar",
    "azucararan": "azucarar",
    "azucararas": "azucarar",
    "azucarare": "azucarar",
    "azucarareis": "azucarar",
    "azucararemos": "azucarar",
    "azucararen": "azucarar",
    "azucarares": "azucarar",
    "azucararon": "azucarar",
    "azucarará": "azucarar",
    "azucararán": "azucarar",
    "azucararás": "azucarar",
    "azucararé": "azucarar",
    "azucararéis": "azucarar",
    "azucararía": "azucarar",
    "azucararíais": "azucarar",
    "azucararíamos": "azucarar",
    "azucararían": "azucarar",
    "azucararías": "azucarar",
    "azucaras": "azucarar",
    "azucarase": "azucarar",
    "azucaraseis": "azucarar",
    "azucarasen": "azucarar",
    "azucarases": "azucarar",
    "azucaraste": "azucarar",
    "azucarasteis": "azucarar",
    "azucare": "azucarar",
    "azucaremos": "azucarar",
    "azucaren": "azucarar",
    "azucarera": "azucarero",
    "azucareras": "azucarero",
    "azucareros": "azucarero",
    "azucarerías": "azucarería",
    "azucares": "azucarar",
    "azucarillos": "azucarillo",
    "azucaro": "azucarar",
    "azucarábamos": "azucarar",
    "azucaráis": "azucarar",
    "azucaráramos": "azucarar",
    "azucaráremos": "azucarar",
    "azucarásemos": "azucarar",
    "azucaré": "azucarar",
    "azucaréis": "azucarar",
    "azucaró": "azucarar",
    "azuce": "azuzar",
    "azucemos": "azuzar",
    "azucen": "azuzar",
    "azucenas": "azucena",
    "azuces": "azuzar",
    "azuches": "azuche",
    "azucé": "azuzar",
    "azucéis": "azuzar",
    "azudas": "azuda",
    "azudes": "azud",
    "azuela": "azolar",
    "azuelan": "azolar",
    "azuelas": "azuela",
    "azuele": "azolar",
    "azuelen": "azolar",
    "azueles": "azolar",
    "azuelo": "azolar",
    "azufaifas": "azufaifa",
    "azufaifos": "azufaifo",
    "azufra": "azufrar",
    "azufraba": "azufrar",
    "azufrabais": "azufrar",
    "azufraban": "azufrar",
    "azufrabas": "azufrar",
    "azufrad": "azufrar",
    "azufrada": "azufrar",
    "azufradas": "azufrar",
    "azufrado": "azufrar",
    "azufradora": "azufrador",
    "azufradoras": "azufrador",
    "azufradores": "azufrador",
    "azufrados": "azufrar",
    "azuframientos": "azuframiento",
    "azuframos": "azufrar",
    "azufran": "azufrar",
    "azufrando": "azufrar",
    "azufrara": "azufrar",
    "azufrarais": "azufrar",
    "azufraran": "azufrar",
    "azufraras": "azufrar",
    "azufrare": "azufrar",
    "azufrareis": "azufrar",
    "azufraremos": "azufrar",
    "azufraren": "azufrar",
    "azufrares": "azufrar",
    "azufraron": "azufrar",
    "azufrará": "azufrar",
    "azufrarán": "azufrar",
    "azufrarás": "azufrar",
    "azufraré": "azufrar",
    "azufraréis": "azufrar",
    "azufraría": "azufrar",
    "azufraríais": "azufrar",
    "azufraríamos": "azufrar",
    "azufrarían": "azufrar",
    "azufrarías": "azufrar",
    "azufras": "azufrar",
    "azufrase": "azufrar",
    "azufraseis": "azufrar",
    "azufrasen": "azufrar",
    "azufrases": "azufrar",
    "azufraste": "azufrar",
    "azufrasteis": "azufrar",
    "azufre": "azufrar",
    "azufremos": "azufrar",
    "azufren": "azufrar",
    "azufreras": "azufrera",
    "azufres": "azufre",
    "azufro": "azufrar",
    "azufrones": "azufrón",
    "azufrosa": "azufroso",
    "azufrosas": "azufroso",
    "azufrosos": "azufroso",
    "azufrábamos": "azufrar",
    "azufráis": "azufrar",
    "azufráramos": "azufrar",
    "azufráremos": "azufrar",
    "azufrásemos": "azufrar",
    "azufré": "azufrar",
    "azufréis": "azufrar",
    "azufró": "azufrar",
    "azula": "azular",
    "azulaba": "azular",
    "azulabais": "azular",
    "azulaban": "azular",
    "azulabas": "azular",
    "azulad": "azular",
    "azulada": "azular",
    "azuladas": "azular",
    "azulado": "azular",
    "azulados": "azular",
    "azulamos": "azular",
    "azulan": "azular",
    "azulando": "azular",
    "azulara": "azular",
    "azularais": "azular",
    "azularan": "azular",
    "azularas": "azular",
    "azulare": "azular",
    "azulareis": "azular",
    "azularemos": "azular",
    "azularen": "azular",
    "azulares": "azular",
    "azularon": "azular",
    "azulará": "azular",
    "azularán": "azular",
    "azularás": "azular",
    "azularé": "azular",
    "azularéis": "azular",
    "azularía": "azular",
    "azularíais": "azular",
    "azularíamos": "azular",
    "azularían": "azular",
    "azularías": "azular",
    "azulas": "azular",
    "azulase": "azular",
    "azulaseis": "azular",
    "azulasen": "azular",
    "azulases": "azular",
    "azulaste": "azular",
    "azulasteis": "azular",
    "azule": "azular",
    "azulea": "azulear",
    "azuleaba": "azulear",
    "azuleabais": "azulear",
    "azuleaban": "azulear",
    "azuleabas": "azulear",
    "azulead": "azulear",
    "azuleada": "azulear",
    "azuleadas": "azulear",
    "azuleado": "azulear",
    "azuleados": "azulear",
    "azuleamos": "azulear",
    "azulean": "azulear",
    "azuleando": "azulear",
    "azuleara": "azulear",
    "azulearais": "azulear",
    "azulearan": "azulear",
    "azulearas": "azulear",
    "azuleare": "azulear",
    "azuleareis": "azulear",
    "azulearemos": "azulear",
    "azulearen": "azulear",
    "azuleares": "azulear",
    "azulearon": "azulear",
    "azuleará": "azulear",
    "azulearán": "azulear",
    "azulearás": "azulear",
    "azulearé": "azulear",
    "azulearéis": "azulear",
    "azulearía": "azulear",
    "azulearíais": "azulear",
    "azulearíamos": "azulear",
    "azulearían": "azulear",
    "azulearías": "azulear",
    "azuleas": "azulear",
    "azulease": "azulear",
    "azuleaseis": "azulear",
    "azuleasen": "azulear",
    "azuleases": "azulear",
    "azuleaste": "azulear",
    "azuleasteis": "azulear",
    "azulee": "azulear",
    "azuleemos": "azulear",
    "azuleen": "azulear",
    "azulees": "azulear",
    "azulejea": "azulejear",
    "azulejeaba": "azulejear",
    "azulejeabais": "azulejear",
    "azulejeaban": "azulejear",
    "azulejeabas": "azulejear",
    "azulejead": "azulejear",
    "azulejeada": "azulejear",
    "azulejeadas": "azulejear",
    "azulejeado": "azulejear",
    "azulejeados": "azulejear",
    "azulejeamos": "azulejear",
    "azulejean": "azulejear",
    "azulejeando": "azulejear",
    "azulejeara": "azulejear",
    "azulejearais": "azulejear",
    "azulejearan": "azulejear",
    "azulejearas": "azulejear",
    "azulejeare": "azulejear",
    "azulejeareis": "azulejear",
    "azulejearemos": "azulejear",
    "azulejearen": "azulejear",
    "azulejeares": "azulejear",
    "azulejearon": "azulejear",
    "azulejeará": "azulejear",
    "azulejearán": "azulejear",
    "azulejearás": "azulejear",
    "azulejearé": "azulejear",
    "azulejearéis": "azulejear",
    "azulejearía": "azulejear",
    "azulejearíais": "azulejear",
    "azulejearíamos": "azulejear",
    "azulejearían": "azulejear",
    "azulejearías": "azulejear",
    "azulejeas": "azulejear",
    "azulejease": "azulejear",
    "azulejeaseis": "azulejear",
    "azulejeasen": "azulejear",
    "azulejeases": "azulejear",
    "azulejeaste": "azulejear",
    "azulejeasteis": "azulejear",
    "azulejee": "azulejear",
    "azulejeemos": "azulejear",
    "azulejeen": "azulejear",
    "azulejees": "azulejear",
    "azulejeo": "azulejear",
    "azulejerías": "azulejería",
    "azulejeábamos": "azulejear",
    "azulejeáis": "azulejear",
    "azulejeáramos": "azulejear",
    "azulejeáremos": "azulejear",
    "azulejeásemos": "azulejear",
    "azulejeé": "azulejear",
    "azulejeéis": "azulejear",
    "azulejeó": "azulejear",
    "azulejos": "azulejo",
    "azulemos": "azular",
    "azulen": "azular",
    "azuleo": "azulear",
    "azules": "azular",
    "azuletes": "azulete",
    "azuleábamos": "azulear",
    "azuleáis": "azulear",
    "azuleáramos": "azulear",
    "azuleáremos": "azulear",
    "azuleásemos": "azulear",
    "azuleé": "azulear",
    "azuleéis": "azulear",
    "azuleó": "azulear",
    "azulgranas": "azulgrana",
    "azulitos": "azulito",
    "azulo": "azular",
    "azulábamos": "azular",
    "azuláis": "azular",
    "azuláramos": "azular",
    "azuláremos": "azular",
    "azulásemos": "azular",
    "azulé": "azular",
    "azuléis": "azular",
    "azuló": "azular",
    "azuma": "azumar",
    "azumaba": "azumar",
    "azumabais": "azumar",
    "azumaban": "azumar",
    "azumabas": "azumar",
    "azumad": "azumar",
    "azumada": "azumar",
    "azumadas": "azumar",
    "azumado": "azumar",
    "azumados": "azumar",
    "azumamos": "azumar",
    "azuman": "azumar",
    "azumando": "azumar",
    "azumara": "azumar",
    "azumarais": "azumar",
    "azumaran": "azumar",
    "azumaras": "azumar",
    "azumare": "azumar",
    "azumareis": "azumar",
    "azumaremos": "azumar",
    "azumaren": "azumar",
    "azumares": "azumar",
    "azumaron": "azumar",
    "azumará": "azumar",
    "azumarán": "azumar",
    "azumarás": "azumar",
    "azumaré": "azumar",
    "azumaréis": "azumar",
    "azumaría": "azumar",
    "azumaríais": "azumar",
    "azumaríamos": "azumar",
    "azumarían": "azumar",
    "azumarías": "azumar",
    "azumas": "azumar",
    "azumase": "azumar",
    "azumaseis": "azumar",
    "azumasen": "azumar",
    "azumases": "azumar",
    "azumaste": "azumar",
    "azumasteis": "azumar",
    "azumbres": "azumbre",
    "azume": "azumar",
    "azumemos": "azumar",
    "azumen": "azumar",
    "azumes": "azumar",
    "azumo": "azumar",
    "azumábamos": "azumar",
    "azumáis": "azumar",
    "azumáramos": "azumar",
    "azumáremos": "azumar",
    "azumásemos": "azumar",
    "azumé": "azumar",
    "azuméis": "azumar",
    "azumó": "azumar",
    "azures": "azur",
    "azuritas": "azurita",
    "azurmalaquitas": "azurmalaquita",
    "azurrona": "azurronar",
    "azurronaba": "azurronar",
    "azurronabais": "azurronar",
    "azurronaban": "azurronar",
    "azurronabas": "azurronar",
    "azurronad": "azurronar",
    "azurronada": "azurronar",
    "azurronadas": "azurronar",
    "azurronado": "azurronar",
    "azurronados": "azurronar",
    "azurronamos": "azurronar",
    "azurronan": "azurronar",
    "azurronando": "azurronar",
    "azurronara": "azurronar",
    "azurronarais": "azurronar",
    "azurronaran": "azurronar",
    "azurronaras": "azurronar",
    "azurronare": "azurronar",
    "azurronareis": "azurronar",
    "azurronaremos": "azurronar",
    "azurronaren": "azurronar",
    "azurronares": "azurronar",
    "azurronaron": "azurronar",
    "azurronará": "azurronar",
    "azurronarán": "azurronar",
    "azurronarás": "azurronar",
    "azurronaré": "azurronar",
    "azurronaréis": "azurronar",
    "azurronaría": "azurronar",
    "azurronaríais": "azurronar",
    "azurronaríamos": "azurronar",
    "azurronarían": "azurronar",
    "azurronarías": "azurronar",
    "azurronas": "azurronar",
    "azurronase": "azurronar",
    "azurronaseis": "azurronar",
    "azurronasen": "azurronar",
    "azurronases": "azurronar",
    "azurronaste": "azurronar",
    "azurronasteis": "azurronar",
    "azurrone": "azurronar",
    "azurronemos": "azurronar",
    "azurronen": "azurronar",
    "azurrones": "azurronar",
    "azurrono": "azurronar",
    "azurronábamos": "azurronar",
    "azurronáis": "azurronar",
    "azurronáramos": "azurronar",
    "azurronáremos": "azurronar",
    "azurronásemos": "azurronar",
    "azurroné": "azurronar",
    "azurronéis": "azurronar",
    "azurronó": "azurronar",
    "azurófila": "azurófilo",
    "azurófilas": "azurófilo",
    "azurófilos": "azurófilo",
    "azuza": "azuzar",
    "azuzaba": "azuzar",
    "azuzabais": "azuzar",
    "azuzaban": "azuzar",
    "azuzabas": "azuzar",
    "azuzad": "azuzar",
    "azuzada": "azuzar",
    "azuzadas": "azuzar",
    "azuzado": "azuzar",
    "azuzados": "azuzar",
    "azuzamientos": "azuzamiento",
    "azuzamos": "azuzar",
    "azuzan": "azuzar",
    "azuzando": "azuzar",
    "azuzara": "azuzar",
    "azuzarais": "azuzar",
    "azuzaran": "azuzar",
    "azuzaras": "azuzar",
    "azuzare": "azuzar",
    "azuzareis": "azuzar",
    "azuzaremos": "azuzar",
    "azuzaren": "azuzar",
    "azuzares": "azuzar",
    "azuzaron": "azuzar",
    "azuzará": "azuzar",
    "azuzarán": "azuzar",
    "azuzarás": "azuzar",
    "azuzaré": "azuzar",
    "azuzaréis": "azuzar",
    "azuzaría": "azuzar",
    "azuzaríais": "azuzar",
    "azuzaríamos": "azuzar",
    "azuzarían": "azuzar",
    "azuzarías": "azuzar",
    "azuzas": "azuzar",
    "azuzase": "azuzar",
    "azuzaseis": "azuzar",
    "azuzasen": "azuzar",
    "azuzases": "azuzar",
    "azuzaste": "azuzar",
    "azuzasteis": "azuzar",
    "azuzo": "azuzar",
    "azuzábamos": "azuzar",
    "azuzáis": "azuzar",
    "azuzáramos": "azuzar",
    "azuzáremos": "azuzar",
    "azuzásemos": "azuzar",
    "azuzó": "azuzar",
    "azófares": "azófar",
    "azúcares": "azúcar",
    "aérea": "aéreo",
    "aéreas": "aéreo",
    "aéreos": "aéreo",
    "aíra": "airar",
    "aíran": "airar",
    "aíras": "airar",
    "aíre": "airar",
    "aíren": "airar",
    "aíres": "airar",
    "aíro": "airar",
    "aísla": "aislar",
    "aíslan": "aislar",
    "aíslas": "aislar",
    "aísle": "aislar",
    "aíslen": "aislar",
    "aísles": "aislar",
    "aíslo": "aislar",
    "añada": "añadir",
    "añadamos": "añadir",
    "añadan": "añadir",
    "añadas": "añadir",
    "añade": "añadir",
    "añaden": "añadir",
    "añades": "añadir",
    "añadid": "añadir",
    "añadida": "añadir",
    "añadidas": "añadir",
    "añadido": "añadir",
    "añadidos": "añadir",
    "añadiduras": "añadidura",
    "añadiendo": "añadir",
    "añadiera": "añadir",
    "añadierais": "añadir",
    "añadieran": "añadir",
    "añadieras": "añadir",
    "añadiere": "añadir",
    "añadiereis": "añadir",
    "añadieren": "añadir",
    "añadieres": "añadir",
    "añadieron": "añadir",
    "añadiese": "añadir",
    "añadieseis": "añadir",
    "añadiesen": "añadir",
    "añadieses": "añadir",
    "añadimos": "añadir",
    "añadiremos": "añadir",
    "añadirá": "añadir",
    "añadirán": "añadir",
    "añadirás": "añadir",
    "añadiré": "añadir",
    "añadiréis": "añadir",
    "añadiría": "añadir",
    "añadiríais": "añadir",
    "añadiríamos": "añadir",
    "añadirían": "añadir",
    "añadirías": "añadir",
    "añadiste": "añadir",
    "añadisteis": "añadir",
    "añadiéramos": "añadir",
    "añadiéremos": "añadir",
    "añadiésemos": "añadir",
    "añadió": "añadir",
    "añado": "añadir",
    "añadáis": "añadir",
    "añadí": "añadir",
    "añadía": "añadir",
    "añadíais": "añadir",
    "añadíamos": "añadir",
    "añadían": "añadir",
    "añadías": "añadir",
    "añadís": "añadir",
    "añafileros": "añafilero",
    "añafiles": "añafil",
    "añagazas": "añagaza",
    "añalejos": "añalejo",
    "añales": "añal",
    "añeja": "añejo",
    "añejaba": "añejar",
    "añejabais": "añejar",
    "añejaban": "añejar",
    "añejabas": "añejar",
    "añejad": "añejar",
    "añejada": "añejar",
    "añejadas": "añejar",
    "añejado": "añejar",
    "añejados": "añejar",
    "añejamientos": "añejamiento",
    "añejamos": "añejar",
    "añejan": "añejar",
    "añejando": "añejar",
    "añejara": "añejar",
    "añejarais": "añejar",
    "añejaran": "añejar",
    "añejaras": "añejar",
    "añejare": "añejar",
    "añejareis": "añejar",
    "añejaremos": "añejar",
    "añejaren": "añejar",
    "añejares": "añejar",
    "añejaron": "añejar",
    "añejará": "añejar",
    "añejarán": "añejar",
    "añejarás": "añejar",
    "añejaré": "añejar",
    "añejaréis": "añejar",
    "añejaría": "añejar",
    "añejaríais": "añejar",
    "añejaríamos": "añejar",
    "añejarían": "añejar",
    "añejarías": "añejar",
    "añejas": "añejo",
    "añejase": "añejar",
    "añejaseis": "añejar",
    "añejasen": "añejar",
    "añejases": "añejar",
    "añejaste": "añejar",
    "añejasteis": "añejar",
    "añeje": "añejar",
    "añejeces": "añejez",
    "añejemos": "añejar",
    "añejen": "añejar",
    "añejes": "añejar",
    "añejo": "añejar",
    "añejos": "añejo",
    "añejábamos": "añejar",
    "añejáis": "añejar",
    "añejáramos": "añejar",
    "añejáremos": "añejar",
    "añejásemos": "añejar",
    "añejé": "añejar",
    "añejéis": "añejar",
    "añejó": "añejar",
    "añicos": "añico",
    "añilerías": "añilería",
    "añiles": "añil",
    "añitos": "añito",
    "añoja": "añojo",
    "añojales": "añojal",
    "añojas": "añojo",
    "añojos": "añojo",
    "añora": "añorar",
    "añoraba": "añorar",
    "añorabais": "añorar",
    "añoraban": "añorar",
    "añorabas": "añorar",
    "añorad": "añorar",
    "añorada": "añorar",
    "añoradas": "añorar",
    "añorado": "añorar",
    "añorados": "añorar",
    "añoramientos": "añoramiento",
    "añoramos": "añorar",
    "añoran": "añorar",
    "añorando": "añorar",
    "añoranzas": "añoranza",
    "añorara": "añorar",
    "añorarais": "añorar",
    "añoraran": "añorar",
    "añoraras": "añorar",
    "añorare": "añorar",
    "añorareis": "añorar",
    "añoraremos": "añorar",
    "añoraren": "añorar",
    "añorares": "añorar",
    "añoraron": "añorar",
    "añorará": "añorar",
    "añorarán": "añorar",
    "añorarás": "añorar",
    "añoraré": "añorar",
    "añoraréis": "añorar",
    "añoraría": "añorar",
    "añoraríais": "añorar",
    "añoraríamos": "añorar",
    "añorarían": "añorar",
    "añorarías": "añorar",
    "añoras": "añorar",
    "añorase": "añorar",
    "añoraseis": "añorar",
    "añorasen": "añorar",
    "añorases": "añorar",
    "añoraste": "añorar",
    "añorasteis": "añorar",
    "añore": "añorar",
    "añoremos": "añorar",
    "añoren": "añorar",
    "añores": "añorar",
    "añoro": "añorar",
    "añorábamos": "añorar",
    "añoráis": "añorar",
    "añoráramos": "añorar",
    "añoráremos": "añorar",
    "añorásemos": "añorar",
    "añoré": "añorar",
    "añoréis": "añorar",
    "añoró": "añorar",
    "años": "año",
    "aórtica": "aórtico",
    "aórticas": "aórtico",
    "aórticos": "aórtico",
    "aúlla": "aullar",
    "aúllan": "aullar",
    "aúllas": "aullar",
    "aúlle": "aullar",
    "aúllen": "aullar",
    "aúlles": "aullar",
    "aúllo": "aullar",
    "aúllos": "aúllo",
    "aún": "aun",
    "aúna": "aunar",
    "aúnan": "aunar",
    "aúnas": "aunar",
    "aúne": "aunar",
    "aúnen": "aunar",
    "aúnes": "aunar",
    "aúno": "aunar",
    "aúpa": "aupar",
    "aúpan": "aupar",
    "aúpas": "aupar",
    "aúpe": "aupar",
    "aúpen": "aupar",
    "aúpes": "aupar",
    "aúpo": "aupar",
    "babaderos": "babadero",
    "babadores": "babador",
    "babas": "baba",
    "babea": "babear",
    "babeaba": "babear",
    "babeabais": "babear",
    "babeaban": "babear",
    "babeabas": "babear",
    "babead": "babear",
    "babeada": "babear",
    "babeadas": "babear",
    "babeado": "babear",
    "babeados": "babear",
    "babeamos": "babear",
    "babean": "babear",
    "babeando": "babear",
    "babeara": "babear",
    "babearais": "babear",
    "babearan": "babear",
    "babearas": "babear",
    "babeare": "babear",
    "babeareis": "babear",
    "babearemos": "babear",
    "babearen": "babear",
    "babeares": "babear",
    "babearon": "babear",
    "babeará": "babear",
    "babearán": "babear",
    "babearás": "babear",
    "babearé": "babear",
    "babearéis": "babear",
    "babearía": "babear",
    "babearíais": "babear",
    "babearíamos": "babear",
    "babearían": "babear",
    "babearías": "babear",
    "babeas": "babear",
    "babease": "babear",
    "babeaseis": "babear",
    "babeasen": "babear",
    "babeases": "babear",
    "babeaste": "babear",
    "babeasteis": "babear",
    "babee": "babear",
    "babeemos": "babear",
    "babeen": "babear",
    "babees": "babear",
    "babeles": "babel",
    "babeo": "babear",
    "babeos": "babeo",
    "baberas": "babera",
    "baberos": "babero",
    "babeábamos": "babear",
    "babeáis": "babear",
    "babeáramos": "babear",
    "babeáremos": "babear",
    "babeásemos": "babear",
    "babeé": "babear",
    "babeéis": "babear",
    "babeó": "babear",
    "babiecas": "babieca",
    "babillas": "babilla",
    "babilonia": "babilonio",
    "babilonias": "babilonio",
    "babilonios": "babilonio",
    "babilónica": "babilónico",
    "babilónicas": "babilónico",
    "babilónicos": "babilónico",
    "babirusas": "babirusa",
    "babis": "babi",
    "babismos": "babismo",
    "bables": "bable",
    "babores": "babor",
    "babosa": "baboso",
    "babosas": "baboso",
    "babosea": "babosear",
    "baboseaba": "babosear",
    "baboseabais": "babosear",
    "baboseaban": "babosear",
    "baboseabas": "babosear",
    "babosead": "babosear",
    "baboseada": "babosear",
    "baboseadas": "babosear",
    "baboseado": "babosear",
    "baboseados": "babosear",
    "baboseamos": "babosear",
    "babosean": "babosear",
    "baboseando": "babosear",
    "baboseara": "babosear",
    "babosearais": "babosear",
    "babosearan": "babosear",
    "babosearas": "babosear",
    "baboseare": "babosear",
    "baboseareis": "babosear",
    "babosearemos": "babosear",
    "babosearen": "babosear",
    "baboseares": "babosear",
    "babosearon": "babosear",
    "baboseará": "babosear",
    "babosearán": "babosear",
    "babosearás": "babosear",
    "babosearé": "babosear",
    "babosearéis": "babosear",
    "babosearía": "babosear",
    "babosearíais": "babosear",
    "babosearíamos": "babosear",
    "babosearían": "babosear",
    "babosearías": "babosear",
    "baboseas": "babosear",
    "babosease": "babosear",
    "baboseaseis": "babosear",
    "baboseasen": "babosear",
    "baboseases": "babosear",
    "baboseaste": "babosear",
    "baboseasteis": "babosear",
    "babosee": "babosear",
    "baboseemos": "babosear",
    "baboseen": "babosear",
    "babosees": "babosear",
    "baboseo": "babosear",
    "baboseos": "baboseo",
    "baboseábamos": "babosear",
    "baboseáis": "babosear",
    "baboseáramos": "babosear",
    "baboseáremos": "babosear",
    "baboseásemos": "babosear",
    "baboseé": "babosear",
    "baboseéis": "babosear",
    "baboseó": "babosear",
    "babositas": "babosita",
    "babosos": "baboso",
    "babuchas": "babucha",
    "babuchera": "babuchero",
    "babucheras": "babuchero",
    "babucheros": "babuchero",
    "babuina": "babuino",
    "babuinas": "babuino",
    "babuinos": "babuino",
    "babuvismos": "babuvismo",
    "babélica": "babélico",
    "babélicas": "babélico",
    "babélicos": "babélico",
    "bacaladera": "bacaladero",
    "bacaladeras": "bacaladero",
    "bacaladeros": "bacaladero",
    "bacaladillas": "bacaladilla",
    "bacalaos": "bacalao",
    "bacanales": "bacanal",
    "bacantes": "bacante",
    "bacarrás": "bacarrá",
    "bacarás": "bacará",
    "bacas": "baca",
    "baceras": "bacera",
    "bacetas": "baceta",
    "bachea": "bachear",
    "bacheaba": "bachear",
    "bacheabais": "bachear",
    "bacheaban": "bachear",
    "bacheabas": "bachear",
    "bachead": "bachear",
    "bacheada": "bachear",
    "bacheadas": "bachear",
    "bacheado": "bachear",
    "bacheados": "bachear",
    "bacheamos": "bachear",
    "bachean": "bachear",
    "bacheando": "bachear",
    "bacheara": "bachear",
    "bachearais": "bachear",
    "bachearan": "bachear",
    "bachearas": "bachear",
    "bacheare": "bachear",
    "bacheareis": "bachear",
    "bachearemos": "bachear",
    "bachearen": "bachear",
    "bacheares": "bachear",
    "bachearon": "bachear",
    "bacheará": "bachear",
    "bachearán": "bachear",
    "bachearás": "bachear",
    "bachearé": "bachear",
    "bachearéis": "bachear",
    "bachearía": "bachear",
    "bachearíais": "bachear",
    "bachearíamos": "bachear",
    "bachearían": "bachear",
    "bachearías": "bachear",
    "bacheas": "bachear",
    "bachease": "bachear",
    "bacheaseis": "bachear",
    "bacheasen": "bachear",
    "bacheases": "bachear",
    "bacheaste": "bachear",
    "bacheasteis": "bachear",
    "bachee": "bachear",
    "bacheemos": "bachear",
    "bacheen": "bachear",
    "bachees": "bachear",
    "bacheo": "bachear",
    "bacheos": "bacheo",
    "baches": "bache",
    "bacheábamos": "bachear",
    "bacheáis": "bachear",
    "bacheáramos": "bachear",
    "bacheáremos": "bachear",
    "bacheásemos": "bachear",
    "bacheé": "bachear",
    "bacheéis": "bachear",
    "bacheó": "bachear",
    "bachilleratos": "bachillerato",
    "bachillerea": "bachillerear",
    "bachillereaba": "bachillerear",
    "bachillereabais": "bachillerear",
    "bachillereaban": "bachillerear",
    "bachillereabas": "bachillerear",
    "bachilleread": "bachillerear",
    "bachillereada": "bachillerear",
    "bachillereadas": "bachillerear",
    "bachillereado": "bachillerear",
    "bachillereados": "bachillerear",
    "bachillereamos": "bachillerear",
    "bachillerean": "bachillerear",
    "bachillereando": "bachillerear",
    "bachillereara": "bachillerear",
    "bachillerearais": "bachillerear",
    "bachillerearan": "bachillerear",
    "bachillerearas": "bachillerear",
    "bachillereare": "bachillerear",
    "bachillereareis": "bachillerear",
    "bachillerearemos": "bachillerear",
    "bachillerearen": "bachillerear",
    "bachillereares": "bachillerear",
    "bachillerearon": "bachillerear",
    "bachillereará": "bachillerear",
    "bachillerearán": "bachillerear",
    "bachillerearás": "bachillerear",
    "bachillerearé": "bachillerear",
    "bachillerearéis": "bachillerear",
    "bachillerearía": "bachillerear",
    "bachillerearíais": "bachillerear",
    "bachillerearíamos": "bachillerear",
    "bachillerearían": "bachillerear",
    "bachillerearías": "bachillerear",
    "bachillereas": "bachillerear",
    "bachillerease": "bachillerear",
    "bachillereaseis": "bachillerear",
    "bachillereasen": "bachillerear",
    "bachillereases": "bachillerear",
    "bachillereaste": "bachillerear",
    "bachillereasteis": "bachillerear",
    "bachilleree": "bachillerear",
    "bachillereemos": "bachillerear",
    "bachillereen": "bachillerear",
    "bachillerees": "bachillerear",
    "bachillereo": "bachillerear",
    "bachilleres": "bachiller",
    "bachillereábamos": "bachillerear",
    "bachillereáis": "bachillerear",
    "bachillereáramos": "bachillerear",
    "bachillereáremos": "bachillerear",
    "bachillereásemos": "bachillerear",
    "bachillereé": "bachillerear",
    "bachillereéis": "bachillerear",
    "bachillereó": "bachillerear",
    "bacilos": "bacilo",
    "baciloscopias": "baciloscopia",
    "bacines": "bacín",
    "bacinetes": "bacinete",
    "bacitracinas": "bacitracina",
    "backgammones": "backgammon",
    "backgammons": "backgammon",
    "backgamons": "backgamon",
    "backspaces": "backspace",
    "backswings": "backswing",
    "backups": "backup",
    "bacones": "bacon",
    "bacons": "bacon",
    "bacteriana": "bacteriano",
    "bacterianas": "bacteriano",
    "bacterianos": "bacteriano",
    "bacterias": "bacteria",
    "bactericidas": "bactericida",
    "bactericidinas": "bactericidina",
    "bacteriemias": "bacteriemia",
    "bacteriocinas": "bacteriocina",
    "bacteriolisinas": "bacteriolisina",
    "bacteriologías": "bacteriología",
    "bacteriológica": "bacteriológico",
    "bacteriológicas": "bacteriológico",
    "bacteriológicos": "bacteriológico",
    "bacteriorrodopsinas": "bacteriorrodopsina",
    "bacteriostatos": "bacteriostato",
    "bacteriostática": "bacteriostático",
    "bacteriostáticas": "bacteriostático",
    "bacteriostáticos": "bacteriostático",
    "bacteriófagos": "bacteriófago",
    "bacteriógena": "bacteriógeno",
    "bacteriógenas": "bacteriógeno",
    "bacteriógenos": "bacteriógeno",
    "bacterióloga": "bacteriólogo",
    "bacteriólogas": "bacteriólogo",
    "bacteriólogos": "bacteriólogo",
    "baculiformes": "baculiforme",
    "bacías": "bacía",
    "badajadas": "badajada",
    "badajazos": "badajazo",
    "badajea": "badajear",
    "badajeaba": "badajear",
    "badajeabais": "badajear",
    "badajeaban": "badajear",
    "badajeabas": "badajear",
    "badajead": "badajear",
    "badajeada": "badajear",
    "badajeadas": "badajear",
    "badajeado": "badajear",
    "badajeados": "badajear",
    "badajeamos": "badajear",
    "badajean": "badajear",
    "badajeando": "badajear",
    "badajeara": "badajear",
    "badajearais": "badajear",
    "badajearan": "badajear",
    "badajearas": "badajear",
    "badajeare": "badajear",
    "badajeareis": "badajear",
    "badajearemos": "badajear",
    "badajearen": "badajear",
    "badajeares": "badajear",
    "badajearon": "badajear",
    "badajeará": "badajear",
    "badajearán": "badajear",
    "badajearás": "badajear",
    "badajearé": "badajear",
    "badajearéis": "badajear",
    "badajearía": "badajear",
    "badajearíais": "badajear",
    "badajearíamos": "badajear",
    "badajearían": "badajear",
    "badajearías": "badajear",
    "badajeas": "badajear",
    "badajease": "badajear",
    "badajeaseis": "badajear",
    "badajeasen": "badajear",
    "badajeases": "badajear",
    "badajeaste": "badajear",
    "badajeasteis": "badajear",
    "badajee": "badajear",
    "badajeemos": "badajear",
    "badajeen": "badajear",
    "badajees": "badajear",
    "badajeo": "badajear",
    "badajeábamos": "badajear",
    "badajeáis": "badajear",
    "badajeáramos": "badajear",
    "badajeáremos": "badajear",
    "badajeásemos": "badajear",
    "badajeé": "badajear",
    "badajeéis": "badajear",
    "badajeó": "badajear",
    "badajos": "badajo",
    "badales": "badal",
    "badalonesa": "badalonés",
    "badalonesas": "badalonés",
    "badaloneses": "badalonés",
    "badanas": "badana",
    "badeas": "badea",
    "badeleyitas": "badeleyita",
    "badenes": "badén",
    "badernas": "baderna",
    "badianes": "badián",
    "badilas": "badila",
    "badilazos": "badilazo",
    "badilejos": "badilejo",
    "badiles": "badil",
    "badmintones": "badminton",
    "badmintons": "badminton",
    "badulaques": "badulaque",
    "baffles": "baffle",
    "baga": "bagar",
    "bagaba": "bagar",
    "bagabais": "bagar",
    "bagaban": "bagar",
    "bagabas": "bagar",
    "bagaceras": "bagacera",
    "bagad": "bagar",
    "bagada": "bagar",
    "bagadas": "bagar",
    "bagado": "bagar",
    "bagados": "bagar",
    "bagajes": "bagaje",
    "bagamos": "bagar",
    "bagan": "bagar",
    "bagando": "bagar",
    "bagara": "bagar",
    "bagarais": "bagar",
    "bagaran": "bagar",
    "bagaras": "bagar",
    "bagare": "bagar",
    "bagareis": "bagar",
    "bagaremos": "bagar",
    "bagaren": "bagar",
    "bagares": "bagar",
    "bagarina": "bagarino",
    "bagarinas": "bagarino",
    "bagarinos": "bagarino",
    "bagaron": "bagar",
    "bagará": "bagar",
    "bagarán": "bagar",
    "bagarás": "bagar",
    "bagaré": "bagar",
    "bagaréis": "bagar",
    "bagaría": "bagar",
    "bagaríais": "bagar",
    "bagaríamos": "bagar",
    "bagarían": "bagar",
    "bagarías": "bagar",
    "bagas": "bagar",
    "bagase": "bagar",
    "bagaseis": "bagar",
    "bagasen": "bagar",
    "bagases": "bagar",
    "bagaste": "bagar",
    "bagasteis": "bagar",
    "bagatelas": "bagatela",
    "bagazos": "bagazo",
    "bago": "bagar",
    "bagratíes": "bagratí",
    "bagratís": "bagratí",
    "bagres": "bagre",
    "bagrátidas": "bagrátida",
    "bague": "bagar",
    "baguemos": "bagar",
    "baguen": "bagar",
    "bagues": "bagar",
    "bagurenses": "bagurense",
    "bagué": "bagar",
    "baguéis": "bagar",
    "bagábamos": "bagar",
    "bagáis": "bagar",
    "bagáramos": "bagar",
    "bagáremos": "bagar",
    "bagásemos": "bagar",
    "bagó": "bagar",
    "bahameña": "bahameño",
    "bahameñas": "bahameño",
    "bahameños": "bahameño",
    "bahaíes": "bahaí",
    "bahaís": "bahaí",
    "bahorrinas": "bahorrina",
    "bahts": "baht",
    "bahías": "bahía",
    "baila": "bailar",
    "bailaba": "bailar",
    "bailabais": "bailar",
    "bailaban": "bailar",
    "bailabas": "bailar",
    "bailables": "bailable",
    "bailad": "bailar",
    "bailada": "bailar",
    "bailadas": "bailar",
    "bailado": "bailar",
    "bailadora": "bailador",
    "bailadoras": "bailador",
    "bailadores": "bailador",
    "bailados": "bailar",
    "bailamos": "bailar",
    "bailan": "bailar",
    "bailando": "bailar",
    "bailara": "bailar",
    "bailarais": "bailar",
    "bailaran": "bailar",
    "bailaras": "bailar",
    "bailare": "bailar",
    "bailareis": "bailar",
    "bailaremos": "bailar",
    "bailaren": "bailar",
    "bailares": "bailar",
    "bailarina": "bailarín",
    "bailarinas": "bailarín",
    "bailarines": "bailarín",
    "bailaron": "bailar",
    "bailará": "bailar",
    "bailarán": "bailar",
    "bailarás": "bailar",
    "bailaré": "bailar",
    "bailaréis": "bailar",
    "bailaría": "bailar",
    "bailaríais": "bailar",
    "bailaríamos": "bailar",
    "bailarían": "bailar",
    "bailarías": "bailar",
    "bailas": "bailar",
    "bailase": "bailar",
    "bailaseis": "bailar",
    "bailasen": "bailar",
    "bailases": "bailar",
    "bailaste": "bailar",
    "bailasteis": "bailar",
    "baile": "bailar",
    "bailemos": "bailar",
    "bailen": "bailar",
    "bailes": "baile",
    "bailo": "bailar",
    "bailotea": "bailotear",
    "bailoteaba": "bailotear",
    "bailoteabais": "bailotear",
    "bailoteaban": "bailotear",
    "bailoteabas": "bailotear",
    "bailotead": "bailotear",
    "bailoteada": "bailotear",
    "bailoteadas": "bailotear",
    "bailoteado": "bailotear",
    "bailoteados": "bailotear",
    "bailoteamos": "bailotear",
    "bailotean": "bailotear",
    "bailoteando": "bailotear",
    "bailoteara": "bailotear",
    "bailotearais": "bailotear",
    "bailotearan": "bailotear",
    "bailotearas": "bailotear",
    "bailoteare": "bailotear",
    "bailoteareis": "bailotear",
    "bailotearemos": "bailotear",
    "bailotearen": "bailotear",
    "bailoteares": "bailotear",
    "bailotearon": "bailotear",
    "bailoteará": "bailotear",
    "bailotearán": "bailotear",
    "bailotearás": "bailotear",
    "bailotearé": "bailotear",
    "bailotearéis": "bailotear",
    "bailotearía": "bailotear",
    "bailotearíais": "bailotear",
    "bailotearíamos": "bailotear",
    "bailotearían": "bailotear",
    "bailotearías": "bailotear",
    "bailoteas": "bailotear",
    "bailotease": "bailotear",
    "bailoteaseis": "bailotear",
    "bailoteasen": "bailotear",
    "bailoteases": "bailotear",
    "bailoteaste": "bailotear",
    "bailoteasteis": "bailotear",
    "bailotee": "bailotear",
    "bailoteemos": "bailotear",
    "bailoteen": "bailotear",
    "bailotees": "bailotear",
    "bailoteo": "bailotear",
    "bailoteos": "bailoteo",
    "bailoteábamos": "bailotear",
    "bailoteáis": "bailotear",
    "bailoteáramos": "bailotear",
    "bailoteáremos": "bailotear",
    "bailoteásemos": "bailotear",
    "bailoteé": "bailotear",
    "bailoteéis": "bailotear",
    "bailoteó": "bailotear",
    "bailábamos": "bailar",
    "bailáis": "bailar",
    "bailáramos": "bailar",
    "bailáremos": "bailar",
    "bailásemos": "bailar",
    "bailé": "bailar",
    "bailéis": "bailar",
    "bailías": "bailía",
    "bailó": "bailar",
    "bainitas": "bainita",
    "bainítica": "bainítico",
    "bainíticas": "bainítico",
    "bainíticos": "bainítico",
    "baiveles": "baivel",
    "baja": "bajo",
    "bajaba": "bajar",
    "bajabais": "bajar",
    "bajaban": "bajar",
    "bajabas": "bajar",
    "bajad": "bajar",
    "bajada": "bajar",
    "bajadas": "bajar",
    "bajado": "bajar",
    "bajados": "bajar",
    "bajalenguas": "bajalengua",
    "bajamares": "bajamar",
    "bajamos": "bajar",
    "bajan": "bajar",
    "bajando": "bajar",
    "bajantes": "bajante",
    "bajara": "bajar",
    "bajarais": "bajar",
    "bajaran": "bajar",
    "bajaras": "bajar",
    "bajare": "bajar",
    "bajareis": "bajar",
    "bajaremos": "bajar",
    "bajaren": "bajar",
    "bajares": "bajar",
    "bajaron": "bajar",
    "bajará": "bajar",
    "bajarán": "bajar",
    "bajarás": "bajar",
    "bajaré": "bajar",
    "bajaréis": "bajar",
    "bajaría": "bajar",
    "bajaríais": "bajar",
    "bajaríamos": "bajar",
    "bajarían": "bajar",
    "bajarías": "bajar",
    "bajas": "bajo",
    "bajase": "bajar",
    "bajaseis": "bajar",
    "bajasen": "bajar",
    "bajases": "bajar",
    "bajaste": "bajar",
    "bajasteis": "bajar",
    "baje": "bajar",
    "bajeles": "bajel",
    "bajemos": "bajar",
    "bajen": "bajar",
    "bajera": "bajero",
    "bajeras": "bajero",
    "bajeros": "bajero",
    "bajes": "bajar",
    "bajetes": "bajete",
    "bajezas": "bajeza",
    "bajistas": "bajista",
    "bajita": "bajito",
    "bajitas": "bajito",
    "bajitos": "bajito",
    "bajo": "bajar",
    "bajocardas": "bajocarda",
    "bajonazos": "bajonazo",
    "bajoncillos": "bajoncillo",
    "bajones": "bajón",
    "bajonistas": "bajonista",
    "bajoplatos": "bajoplato",
    "bajorrelieves": "bajorrelieve",
    "bajos": "bajo",
    "bajuras": "bajura",
    "bajábamos": "bajar",
    "bajáis": "bajar",
    "bajáramos": "bajar",
    "bajáremos": "bajar",
    "bajásemos": "bajar",
    "bajé": "bajar",
    "bajéis": "bajar",
    "bajíos": "bajío",
    "bajó": "bajar",
    "bala": "balar",
    "balaba": "balar",
    "balabais": "balar",
    "balaban": "balar",
    "balabas": "balar",
    "balaceras": "balacera",
    "balad": "balar",
    "balada": "balar",
    "baladas": "balar",
    "balado": "balar",
    "balados": "balar",
    "baladrona": "baladrón",
    "baladronadas": "baladronada",
    "baladronas": "baladrón",
    "baladronea": "baladronear",
    "baladroneaba": "baladronear",
    "baladroneabais": "baladronear",
    "baladroneaban": "baladronear",
    "baladroneabas": "baladronear",
    "baladronead": "baladronear",
    "baladroneada": "baladronear",
    "baladroneadas": "baladronear",
    "baladroneado": "baladronear",
    "baladroneados": "baladronear",
    "baladroneamos": "baladronear",
    "baladronean": "baladronear",
    "baladroneando": "baladronear",
    "baladroneara": "baladronear",
    "baladronearais": "baladronear",
    "baladronearan": "baladronear",
    "baladronearas": "baladronear",
    "baladroneare": "baladronear",
    "baladroneareis": "baladronear",
    "baladronearemos": "baladronear",
    "baladronearen": "baladronear",
    "baladroneares": "baladronear",
    "baladronearon": "baladronear",
    "baladroneará": "baladronear",
    "baladronearán": "baladronear",
    "baladronearás": "baladronear",
    "baladronearé": "baladronear",
    "baladronearéis": "baladronear",
    "baladronearía": "baladronear",
    "baladronearíais": "baladronear",
    "baladronearíamos": "baladronear",
    "baladronearían": "baladronear",
    "baladronearías": "baladronear",
    "baladroneas": "baladronear",
    "baladronease": "baladronear",
    "baladroneaseis": "baladronear",
    "baladroneasen": "baladronear",
    "baladroneases": "baladronear",
    "baladroneaste": "baladronear",
    "baladroneasteis": "baladronear",
    "baladronee": "baladronear",
    "baladroneemos": "baladronear",
    "baladroneen": "baladronear",
    "baladronees": "baladronear",
    "baladroneo": "baladronear",
    "baladrones": "baladrón",
    "baladroneábamos": "baladronear",
    "baladroneáis": "baladronear",
    "baladroneáramos": "baladronear",
    "baladroneáremos": "baladronear",
    "baladroneásemos": "baladronear",
    "baladroneé": "baladronear",
    "baladroneéis": "baladronear",
    "baladroneó": "baladronear",
    "baladíes": "baladí",
    "baladís": "baladí",
    "balagueros": "balaguero",
    "balajes": "balaje",
    "balalaicas": "balalaica",
    "balamos": "balar",
    "balan": "balar",
    "balancea": "balancear",
    "balanceaba": "balancear",
    "balanceabais": "balancear",
    "balanceaban": "balancear",
    "balanceabas": "balancear",
    "balancead": "balancear",
    "balanceada": "balancear",
    "balanceadas": "balancear",
    "balanceado": "balancear",
    "balanceadora": "balanceador",
    "balanceadoras": "balanceador",
    "balanceadores": "balanceador",
    "balanceados": "balancear",
    "balanceamos": "balancear",
    "balancean": "balancear",
    "balanceando": "balancear",
    "balanceantes": "balanceante",
    "balanceara": "balancear",
    "balancearais": "balancear",
    "balancearan": "balancear",
    "balancearas": "balancear",
    "balanceare": "balancear",
    "balanceareis": "balancear",
    "balancearemos": "balancear",
    "balancearen": "balancear",
    "balanceares": "balancear",
    "balancearon": "balancear",
    "balanceará": "balancear",
    "balancearán": "balancear",
    "balancearás": "balancear",
    "balancearé": "balancear",
    "balancearéis": "balancear",
    "balancearía": "balancear",
    "balancearíais": "balancear",
    "balancearíamos": "balancear",
    "balancearían": "balancear",
    "balancearías": "balancear",
    "balanceas": "balancear",
    "balancease": "balancear",
    "balanceaseis": "balancear",
    "balanceasen": "balancear",
    "balanceases": "balancear",
    "balanceaste": "balancear",
    "balanceasteis": "balancear",
    "balancee": "balancear",
    "balanceemos": "balancear",
    "balanceen": "balancear",
    "balancees": "balancear",
    "balanceo": "balancear",
    "balanceos": "balanceo",
    "balancera": "balancero",
    "balanceras": "balancero",
    "balanceros": "balancero",
    "balances": "balance",
    "balanceábamos": "balancear",
    "balanceáis": "balancear",
    "balanceáramos": "balancear",
    "balanceáremos": "balancear",
    "balanceásemos": "balancear",
    "balanceé": "balancear",
    "balanceéis": "balancear",
    "balanceó": "balancear",
    "balancines": "balancín",
    "balancímetros": "balancímetro",
    "balando": "balar",
    "balandranes": "balandrán",
    "balandras": "balandra",
    "balandristas": "balandrista",
    "balandros": "balandro",
    "balanos": "balano",
    "balanzaria": "balanzario",
    "balanzarias": "balanzario",
    "balanzarios": "balanzario",
    "balanzas": "balanza",
    "balanzones": "balanzón",
    "balara": "balar",
    "balarais": "balar",
    "balaran": "balar",
    "balaras": "balar",
    "balare": "balar",
    "balareis": "balar",
    "balaremos": "balar",
    "balaren": "balar",
    "balares": "balar",
    "balaron": "balar",
    "balarrasas": "balarrasa",
    "balará": "balar",
    "balarán": "balar",
    "balarás": "balar",
    "balaré": "balar",
    "balaréis": "balar",
    "balaría": "balar",
    "balaríais": "balar",
    "balaríamos": "balar",
    "balarían": "balar",
    "balarías": "balar",
    "balas": "balar",
    "balase": "balar",
    "balaseis": "balar",
    "balasen": "balar",
    "balases": "balar",
    "balastajes": "balastaje",
    "balaste": "balar",
    "balasteis": "balar",
    "balastos": "balasto",
    "balatas": "balata",
    "balates": "balate",
    "balaustradas": "balaustrada",
    "balaustras": "balaustra",
    "balaustres": "balaustre",
    "balays": "balay",
    "balazos": "balazo",
    "balaústas": "balaústa",
    "balaústres": "balaústre",
    "balboas": "balboa",
    "balbuca": "balbucir",
    "balbucamos": "balbucir",
    "balbucan": "balbucir",
    "balbucas": "balbucir",
    "balbuce": "balbucir",
    "balbucea": "balbucear",
    "balbuceaba": "balbucear",
    "balbuceabais": "balbucear",
    "balbuceaban": "balbucear",
    "balbuceabas": "balbucear",
    "balbucead": "balbucear",
    "balbuceada": "balbucear",
    "balbuceadas": "balbucear",
    "balbuceado": "balbucear",
    "balbuceados": "balbucear",
    "balbuceamos": "balbucear",
    "balbucean": "balbucear",
    "balbuceando": "balbucear",
    "balbuceantes": "balbuceante",
    "balbuceara": "balbucear",
    "balbucearais": "balbucear",
    "balbucearan": "balbucear",
    "balbucearas": "balbucear",
    "balbuceare": "balbucear",
    "balbuceareis": "balbucear",
    "balbucearemos": "balbucear",
    "balbucearen": "balbucear",
    "balbuceares": "balbucear",
    "balbucearon": "balbucear",
    "balbuceará": "balbucear",
    "balbucearán": "balbucear",
    "balbucearás": "balbucear",
    "balbucearé": "balbucear",
    "balbucearéis": "balbucear",
    "balbucearía": "balbucear",
    "balbucearíais": "balbucear",
    "balbucearíamos": "balbucear",
    "balbucearían": "balbucear",
    "balbucearías": "balbucear",
    "balbuceas": "balbucear",
    "balbucease": "balbucear",
    "balbuceaseis": "balbucear",
    "balbuceasen": "balbucear",
    "balbuceases": "balbucear",
    "balbuceaste": "balbucear",
    "balbuceasteis": "balbucear",
    "balbucee": "balbucear",
    "balbuceemos": "balbucear",
    "balbuceen": "balbucear",
    "balbucees": "balbucear",
    "balbucen": "balbucir",
    "balbucencias": "balbucencia",
    "balbuceo": "balbucear",
    "balbuceos": "balbuceo",
    "balbuces": "balbucir",
    "balbuceábamos": "balbucear",
    "balbuceáis": "balbucear",
    "balbuceáramos": "balbucear",
    "balbuceáremos": "balbucear",
    "balbuceásemos": "balbucear",
    "balbuceé": "balbucear",
    "balbuceéis": "balbucear",
    "balbuceó": "balbucear",
    "balbucid": "balbucir",
    "balbucida": "balbucir",
    "balbucidas": "balbucir",
    "balbucido": "balbucir",
    "balbucidos": "balbucir",
    "balbuciendo": "balbucir",
    "balbuciera": "balbucir",
    "balbucierais": "balbucir",
    "balbucieran": "balbucir",
    "balbucieras": "balbucir",
    "balbuciere": "balbucir",
    "balbuciereis": "balbucir",
    "balbucieren": "balbucir",
    "balbucieres": "balbucir",
    "balbucieron": "balbucir",
    "balbuciese": "balbucir",
    "balbucieseis": "balbucir",
    "balbuciesen": "balbucir",
    "balbucieses": "balbucir",
    "balbucimos": "balbucir",
    "balbuciremos": "balbucir",
    "balbucirá": "balbucir",
    "balbucirán": "balbucir",
    "balbucirás": "balbucir",
    "balbuciré": "balbucir",
    "balbuciréis": "balbucir",
    "balbuciría": "balbucir",
    "balbuciríais": "balbucir",
    "balbuciríamos": "balbucir",
    "balbucirían": "balbucir",
    "balbucirías": "balbucir",
    "balbuciste": "balbucir",
    "balbucisteis": "balbucir",
    "balbuciéramos": "balbucir",
    "balbuciéremos": "balbucir",
    "balbuciésemos": "balbucir",
    "balbució": "balbucir",
    "balbuco": "balbucir",
    "balbucáis": "balbucir",
    "balbucí": "balbucir",
    "balbucía": "balbucir",
    "balbucíais": "balbucir",
    "balbucíamos": "balbucir",
    "balbucían": "balbucir",
    "balbucías": "balbucir",
    "balbucís": "balbucir",
    "balcanice": "balcanizar",
    "balcanicemos": "balcanizar",
    "balcanicen": "balcanizar",
    "balcanices": "balcanizar",
    "balcanicé": "balcanizar",
    "balcanicéis": "balcanizar",
    "balcaniza": "balcanizar",
    "balcanizaba": "balcanizar",
    "balcanizabais": "balcanizar",
    "balcanizaban": "balcanizar",
    "balcanizabas": "balcanizar",
    "balcanizaciones": "balcanización",
    "balcanizad": "balcanizar",
    "balcanizada": "balcanizar",
    "balcanizadas": "balcanizar",
    "balcanizado": "balcanizar",
    "balcanizados": "balcanizar",
    "balcanizamos": "balcanizar",
    "balcanizan": "balcanizar",
    "balcanizando": "balcanizar",
    "balcanizara": "balcanizar",
    "balcanizarais": "balcanizar",
    "balcanizaran": "balcanizar",
    "balcanizaras": "balcanizar",
    "balcanizare": "balcanizar",
    "balcanizareis": "balcanizar",
    "balcanizaremos": "balcanizar",
    "balcanizaren": "balcanizar",
    "balcanizares": "balcanizar",
    "balcanizaron": "balcanizar",
    "balcanizará": "balcanizar",
    "balcanizarán": "balcanizar",
    "balcanizarás": "balcanizar",
    "balcanizaré": "balcanizar",
    "balcanizaréis": "balcanizar",
    "balcanizaría": "balcanizar",
    "balcanizaríais": "balcanizar",
    "balcanizaríamos": "balcanizar",
    "balcanizarían": "balcanizar",
    "balcanizarías": "balcanizar",
    "balcanizas": "balcanizar",
    "balcanizase": "balcanizar",
    "balcanizaseis": "balcanizar",
    "balcanizasen": "balcanizar",
    "balcanizases": "balcanizar",
    "balcanizaste": "balcanizar",
    "balcanizasteis": "balcanizar",
    "balcanizo": "balcanizar",
    "balcanizábamos": "balcanizar",
    "balcanizáis": "balcanizar",
    "balcanizáramos": "balcanizar",
    "balcanizáremos": "balcanizar",
    "balcanizásemos": "balcanizar",
    "balcanizó": "balcanizar",
    "balconajes": "balconaje",
    "balconcillos": "balconcillo",
    "balconea": "balconear",
    "balconeaba": "balconear",
    "balconeabais": "balconear",
    "balconeaban": "balconear",
    "balconeabas": "balconear",
    "balconead": "balconear",
    "balconeada": "balconear",
    "balconeadas": "balconear",
    "balconeado": "balconear",
    "balconeados": "balconear",
    "balconeamos": "balconear",
    "balconean": "balconear",
    "balconeando": "balconear",
    "balconeara": "balconear",
    "balconearais": "balconear",
    "balconearan": "balconear",
    "balconearas": "balconear",
    "balconeare": "balconear",
    "balconeareis": "balconear",
    "balconearemos": "balconear",
    "balconearen": "balconear",
    "balconeares": "balconear",
    "balconearon": "balconear",
    "balconeará": "balconear",
    "balconearán": "balconear",
    "balconearás": "balconear",
    "balconearé": "balconear",
    "balconearéis": "balconear",
    "balconearía": "balconear",
    "balconearíais": "balconear",
    "balconearíamos": "balconear",
    "balconearían": "balconear",
    "balconearías": "balconear",
    "balconeas": "balconear",
    "balconease": "balconear",
    "balconeaseis": "balconear",
    "balconeasen": "balconear",
    "balconeases": "balconear",
    "balconeaste": "balconear",
    "balconeasteis": "balconear",
    "balconee": "balconear",
    "balconeemos": "balconear",
    "balconeen": "balconear",
    "balconees": "balconear",
    "balconeo": "balconear",
    "balcones": "balcón",
    "balconeábamos": "balconear",
    "balconeáis": "balconear",
    "balconeáramos": "balconear",
    "balconeáremos": "balconear",
    "balconeásemos": "balconear",
    "balconeé": "balconear",
    "balconeéis": "balconear",
    "balconeó": "balconear",
    "balcánica": "balcánico",
    "balcánicas": "balcánico",
    "balcánicos": "balcánico",
    "balda": "baldar",
    "baldaba": "baldar",
    "baldabais": "baldar",
    "baldaban": "baldar",
    "baldabas": "baldar",
    "baldad": "baldar",
    "baldada": "baldar",
    "baldadas": "baldar",
    "baldado": "baldar",
    "baldados": "baldar",
    "baldaduras": "baldadura",
    "baldamientos": "baldamiento",
    "baldamos": "baldar",
    "baldan": "baldar",
    "baldando": "baldar",
    "baldaquines": "baldaquín",
    "baldaquinos": "baldaquino",
    "baldara": "baldar",
    "baldarais": "baldar",
    "baldaran": "baldar",
    "baldaras": "baldar",
    "baldare": "baldar",
    "baldareis": "baldar",
    "baldaremos": "baldar",
    "baldaren": "baldar",
    "baldares": "baldar",
    "baldaron": "baldar",
    "baldará": "baldar",
    "baldarán": "baldar",
    "baldarás": "baldar",
    "baldaré": "baldar",
    "baldaréis": "baldar",
    "baldaría": "baldar",
    "baldaríais": "baldar",
    "baldaríamos": "baldar",
    "baldarían": "baldar",
    "baldarías": "baldar",
    "baldas": "baldar",
    "baldase": "baldar",
    "baldaseis": "baldar",
    "baldasen": "baldar",
    "baldases": "baldar",
    "baldaste": "baldar",
    "baldasteis": "baldar",
    "balde": "baldar",
    "baldea": "baldear",
    "baldeaba": "baldear",
    "baldeabais": "baldear",
    "baldeaban": "baldear",
    "baldeabas": "baldear",
    "baldead": "baldear",
    "baldeada": "baldear",
    "baldeadas": "baldear",
    "baldeado": "baldear",
    "baldeados": "baldear",
    "baldeamos": "baldear",
    "baldean": "baldear",
    "baldeando": "baldear",
    "baldeara": "baldear",
    "baldearais": "baldear",
    "baldearan": "baldear",
    "baldearas": "baldear",
    "baldeare": "baldear",
    "baldeareis": "baldear",
    "baldearemos": "baldear",
    "baldearen": "baldear",
    "baldeares": "baldear",
    "baldearon": "baldear",
    "baldeará": "baldear",
    "baldearán": "baldear",
    "baldearás": "baldear",
    "baldearé": "baldear",
    "baldearéis": "baldear",
    "baldearía": "baldear",
    "baldearíais": "baldear",
    "baldearíamos": "baldear",
    "baldearían": "baldear",
    "baldearías": "baldear",
    "baldeas": "baldear",
    "baldease": "baldear",
    "baldeaseis": "baldear",
    "baldeasen": "baldear",
    "baldeases": "baldear",
    "baldeaste": "baldear",
    "baldeasteis": "baldear",
    "baldee": "baldear",
    "baldeemos": "baldear",
    "baldeen": "baldear",
    "baldees": "baldear",
    "baldemos": "baldar",
    "balden": "baldar",
    "baldeo": "baldear",
    "baldeos": "baldeo",
    "baldes": "balde",
    "baldeábamos": "baldear",
    "baldeáis": "baldear",
    "baldeáramos": "baldear",
    "baldeáremos": "baldear",
    "baldeásemos": "baldear",
    "baldeé": "baldear",
    "baldeéis": "baldear",
    "baldeó": "baldear",
    "baldo": "baldar",
    "baldona": "baldonar",
    "baldonaba": "baldonar",
    "baldonabais": "baldonar",
    "baldonaban": "baldonar",
    "baldonabas": "baldonar",
    "baldonad": "baldonar",
    "baldonada": "baldonar",
    "baldonadas": "baldonar",
    "baldonado": "baldonar",
    "baldonados": "baldonar",
    "baldonamos": "baldonar",
    "baldonan": "baldonar",
    "baldonando": "baldonar",
    "baldonara": "baldonar",
    "baldonarais": "baldonar",
    "baldonaran": "baldonar",
    "baldonaras": "baldonar",
    "baldonare": "baldonar",
    "baldonareis": "baldonar",
    "baldonaremos": "baldonar",
    "baldonaren": "baldonar",
    "baldonares": "baldonar",
    "baldonaron": "baldonar",
    "baldonará": "baldonar",
    "baldonarán": "baldonar",
    "baldonarás": "baldonar",
    "baldonaré": "baldonar",
    "baldonaréis": "baldonar",
    "baldonaría": "baldonar",
    "baldonaríais": "baldonar",
    "baldonaríamos": "baldonar",
    "baldonarían": "baldonar",
    "baldonarías": "baldonar",
    "baldonas": "baldonar",
    "baldonase": "baldonar",
    "baldonaseis": "baldonar",
    "baldonasen": "baldonar",
    "baldonases": "baldonar",
    "baldonaste": "baldonar",
    "baldonasteis": "baldonar",
    "baldone": "baldonar",
    "baldonea": "baldonear",
    "baldoneaba": "baldonear",
    "baldoneabais": "baldonear",
    "baldoneaban": "baldonear",
    "baldoneabas": "baldonear",
    "baldonead": "baldonear",
    "baldoneada": "baldonear",
    "baldoneadas": "baldonear",
    "baldoneado": "baldonear",
    "baldoneados": "baldonear",
    "baldoneamos": "baldonear",
    "baldonean": "baldonear",
    "baldoneando": "baldonear",
    "baldoneara": "baldonear",
    "baldonearais": "baldonear",
    "baldonearan": "baldonear",
    "baldonearas": "baldonear",
    "baldoneare": "baldonear",
    "baldoneareis": "baldonear",
    "baldonearemos": "baldonear",
    "baldonearen": "baldonear",
    "baldoneares": "baldonear",
    "baldonearon": "baldonear",
    "baldoneará": "baldonear",
    "baldonearán": "baldonear",
    "baldonearás": "baldonear",
    "baldonearé": "baldonear",
    "baldonearéis": "baldonear",
    "baldonearía": "baldonear",
    "baldonearíais": "baldonear",
    "baldonearíamos": "baldonear",
    "baldonearían": "baldonear",
    "baldonearías": "baldonear",
    "baldoneas": "baldonear",
    "baldonease": "baldonear",
    "baldoneaseis": "baldonear",
    "baldoneasen": "baldonear",
    "baldoneases": "baldonear",
    "baldoneaste": "baldonear",
    "baldoneasteis": "baldonear",
    "baldonee": "baldonear",
    "baldoneemos": "baldonear",
    "baldoneen": "baldonear",
    "baldonees": "baldonear",
    "baldonemos": "baldonar",
    "baldonen": "baldonar",
    "baldoneo": "baldonear",
    "baldones": "baldonar",
    "baldoneábamos": "baldonear",
    "baldoneáis": "baldonear",
    "baldoneáramos": "baldonear",
    "baldoneáremos": "baldonear",
    "baldoneásemos": "baldonear",
    "baldoneé": "baldonear",
    "baldoneéis": "baldonear",
    "baldoneó": "baldonear",
    "baldono": "baldonar",
    "baldonábamos": "baldonar",
    "baldonáis": "baldonar",
    "baldonáramos": "baldonar",
    "baldonáremos": "baldonar",
    "baldonásemos": "baldonar",
    "baldoné": "baldonar",
    "baldonéis": "baldonar",
    "baldonó": "baldonar",
    "baldosa": "baldosar",
    "baldosaba": "baldosar",
    "baldosabais": "baldosar",
    "baldosaban": "baldosar",
    "baldosabas": "baldosar",
    "baldosad": "baldosar",
    "baldosada": "baldosar",
    "baldosadas": "baldosar",
    "baldosado": "baldosar",
    "baldosadora": "baldosador",
    "baldosadoras": "baldosador",
    "baldosadores": "baldosador",
    "baldosados": "baldosar",
    "baldosamos": "baldosar",
    "baldosan": "baldosar",
    "baldosando": "baldosar",
    "baldosara": "baldosar",
    "baldosarais": "baldosar",
    "baldosaran": "baldosar",
    "baldosaras": "baldosar",
    "baldosare": "baldosar",
    "baldosareis": "baldosar",
    "baldosaremos": "baldosar",
    "baldosaren": "baldosar",
    "baldosares": "baldosar",
    "baldosaron": "baldosar",
    "baldosará": "baldosar",
    "baldosarán": "baldosar",
    "baldosarás": "baldosar",
    "baldosaré": "baldosar",
    "baldosaréis": "baldosar",
    "baldosaría": "baldosar",
    "baldosaríais": "baldosar",
    "baldosaríamos": "baldosar",
    "baldosarían": "baldosar",
    "baldosarías": "baldosar",
    "baldosas": "baldosar",
    "baldosase": "baldosar",
    "baldosaseis": "baldosar",
    "baldosasen": "baldosar",
    "baldosases": "baldosar",
    "baldosaste": "baldosar",
    "baldosasteis": "baldosar",
    "baldose": "baldosar",
    "baldosemos": "baldosar",
    "baldosen": "baldosar",
    "baldoses": "baldosar",
    "baldosines": "baldosín",
    "baldoso": "baldosar",
    "baldosábamos": "baldosar",
    "baldosáis": "baldosar",
    "baldosáramos": "baldosar",
    "baldosáremos": "baldosar",
    "baldosásemos": "baldosar",
    "baldosé": "baldosar",
    "baldoséis": "baldosar",
    "baldosó": "baldosar",
    "balduques": "balduque",
    "baldábamos": "baldar",
    "baldáis": "baldar",
    "baldáramos": "baldar",
    "baldáremos": "baldar",
    "baldásemos": "baldar",
    "baldé": "baldar",
    "baldéis": "baldar",
    "baldía": "baldío",
    "baldías": "baldío",
    "baldíos": "baldío",
    "baldó": "baldar",
    "bale": "balar",
    "balea": "balear",
    "baleaba": "balear",
    "baleabais": "balear",
    "baleaban": "balear",
    "baleabas": "balear",
    "balead": "balear",
    "baleada": "balear",
    "baleadas": "balear",
    "baleado": "balear",
    "baleados": "balear",
    "baleamos": "balear",
    "balean": "balear",
    "baleando": "balear",
    "baleara": "balear",
    "balearais": "balear",
    "balearan": "balear",
    "balearas": "balear",
    "baleare": "balear",
    "baleareis": "balear",
    "balearemos": "balear",
    "balearen": "balear",
    "baleares": "balear",
    "balearon": "balear",
    "balears": "balear",
    "baleará": "balear",
    "balearán": "balear",
    "balearás": "balear",
    "balearé": "balear",
    "balearéis": "balear",
    "balearía": "balear",
    "balearíais": "balear",
    "balearíamos": "balear",
    "balearían": "balear",
    "balearías": "balear",
    "baleas": "balear",
    "balease": "balear",
    "baleaseis": "balear",
    "baleasen": "balear",
    "baleases": "balear",
    "baleaste": "balear",
    "baleasteis": "balear",
    "balee": "balear",
    "baleemos": "balear",
    "baleen": "balear",
    "balees": "balear",
    "balemos": "balar",
    "balen": "balar",
    "balenoptéridos": "balenoptérido",
    "baleo": "balear",
    "baleos": "baleo",
    "baleros": "balero",
    "balerías": "balería",
    "baleríos": "balerío",
    "bales": "balar",
    "baleábamos": "balear",
    "baleáis": "balear",
    "baleáramos": "balear",
    "baleáremos": "balear",
    "baleárica": "baleárico",
    "baleáricas": "baleárico",
    "baleáricos": "baleárico",
    "baleásemos": "balear",
    "baleé": "balear",
    "baleéis": "balear",
    "baleó": "balear",
    "balice": "balizar",
    "balicemos": "balizar",
    "balicen": "balizar",
    "balices": "balizar",
    "balicé": "balizar",
    "balicéis": "balizar",
    "balidos": "balido",
    "balines": "balín",
    "balistas": "balista",
    "balita": "balitar",
    "balitaba": "balitar",
    "balitabais": "balitar",
    "balitaban": "balitar",
    "balitabas": "balitar",
    "balitad": "balitar",
    "balitada": "balitar",
    "balitadas": "balitar",
    "balitaderas": "balitadera",
    "balitado": "balitar",
    "balitados": "balitar",
    "balitamos": "balitar",
    "balitan": "balitar",
    "balitando": "balitar",
    "balitara": "balitar",
    "balitarais": "balitar",
    "balitaran": "balitar",
    "balitaras": "balitar",
    "balitare": "balitar",
    "balitareis": "balitar",
    "balitaremos": "balitar",
    "balitaren": "balitar",
    "balitares": "balitar",
    "balitaron": "balitar",
    "balitará": "balitar",
    "balitarán": "balitar",
    "balitarás": "balitar",
    "balitaré": "balitar",
    "balitaréis": "balitar",
    "balitaría": "balitar",
    "balitaríais": "balitar",
    "balitaríamos": "balitar",
    "balitarían": "balitar",
    "balitarías": "balitar",
    "balitas": "balitar",
    "balitase": "balitar",
    "balitaseis": "balitar",
    "balitasen": "balitar",
    "balitases": "balitar",
    "balitaste": "balitar",
    "balitasteis": "balitar",
    "balite": "balitar",
    "balitea": "balitear",
    "baliteaba": "balitear",
    "baliteabais": "balitear",
    "baliteaban": "balitear",
    "baliteabas": "balitear",
    "balitead": "balitear",
    "baliteada": "balitear",
    "baliteadas": "balitear",
    "baliteado": "balitear",
    "baliteados": "balitear",
    "baliteamos": "balitear",
    "balitean": "balitear",
    "baliteando": "balitear",
    "baliteara": "balitear",
    "balitearais": "balitear",
    "balitearan": "balitear",
    "balitearas": "balitear",
    "baliteare": "balitear",
    "baliteareis": "balitear",
    "balitearemos": "balitear",
    "balitearen": "balitear",
    "baliteares": "balitear",
    "balitearon": "balitear",
    "baliteará": "balitear",
    "balitearán": "balitear",
    "balitearás": "balitear",
    "balitearé": "balitear",
    "balitearéis": "balitear",
    "balitearía": "balitear",
    "balitearíais": "balitear",
    "balitearíamos": "balitear",
    "balitearían": "balitear",
    "balitearías": "balitear",
    "baliteas": "balitear",
    "balitease": "balitear",
    "baliteaseis": "balitear",
    "baliteasen": "balitear",
    "baliteases": "balitear",
    "baliteaste": "balitear",
    "baliteasteis": "balitear",
    "balitee": "balitear",
    "baliteemos": "balitear",
    "baliteen": "balitear",
    "balitees": "balitear",
    "balitemos": "balitar",
    "baliten": "balitar",
    "baliteo": "balitear",
    "balites": "balitar",
    "baliteábamos": "balitear",
    "baliteáis": "balitear",
    "baliteáramos": "balitear",
    "baliteáremos": "balitear",
    "baliteásemos": "balitear",
    "baliteé": "balitear",
    "baliteéis": "balitear",
    "baliteó": "balitear",
    "balito": "balitar",
    "balitábamos": "balitar",
    "balitáis": "balitar",
    "balitáramos": "balitar",
    "balitáremos": "balitar",
    "balitásemos": "balitar",
    "balité": "balitar",
    "balitéis": "balitar",
    "balitó": "balitar",
    "baliza": "balizar",
    "balizaba": "balizar",
    "balizabais": "balizar",
    "balizaban": "balizar",
    "balizabas": "balizar",
    "balizad": "balizar",
    "balizada": "balizar",
    "balizadas": "balizar",
    "balizado": "balizar",
    "balizadora": "balizador",
    "balizadoras": "balizador",
    "balizadores": "balizador",
    "balizados": "balizar",
    "balizajes": "balizaje",
    "balizamientos": "balizamiento",
    "balizamos": "balizar",
    "balizan": "balizar",
    "balizando": "balizar",
    "balizara": "balizar",
    "balizarais": "balizar",
    "balizaran": "balizar",
    "balizaras": "balizar",
    "balizare": "balizar",
    "balizareis": "balizar",
    "balizaremos": "balizar",
    "balizaren": "balizar",
    "balizares": "balizar",
    "balizaron": "balizar",
    "balizará": "balizar",
    "balizarán": "balizar",
    "balizarás": "balizar",
    "balizaré": "balizar",
    "balizaréis": "balizar",
    "balizaría": "balizar",
    "balizaríais": "balizar",
    "balizaríamos": "balizar",
    "balizarían": "balizar",
    "balizarías": "balizar",
    "balizas": "balizar",
    "balizase": "balizar",
    "balizaseis": "balizar",
    "balizasen": "balizar",
    "balizases": "balizar",
    "balizaste": "balizar",
    "balizasteis": "balizar",
    "balizo": "balizar",
    "balizábamos": "balizar",
    "balizáis": "balizar",
    "balizáramos": "balizar",
    "balizáremos": "balizar",
    "balizásemos": "balizar",
    "balizó": "balizar",
    "ballenas": "ballena",
    "ballenatos": "ballenato",
    "ballenera": "ballenero",
    "balleneras": "ballenero",
    "balleneros": "ballenero",
    "ballestadas": "ballestada",
    "ballestas": "ballesta",
    "ballestazos": "ballestazo",
    "ballestea": "ballestear",
    "ballesteaba": "ballestear",
    "ballesteabais": "ballestear",
    "ballesteaban": "ballestear",
    "ballesteabas": "ballestear",
    "ballestead": "ballestear",
    "ballesteada": "ballestear",
    "ballesteadas": "ballestear",
    "ballesteado": "ballestear",
    "ballesteados": "ballestear",
    "ballesteamos": "ballestear",
    "ballestean": "ballestear",
    "ballesteando": "ballestear",
    "ballesteara": "ballestear",
    "ballestearais": "ballestear",
    "ballestearan": "ballestear",
    "ballestearas": "ballestear",
    "ballesteare": "ballestear",
    "ballesteareis": "ballestear",
    "ballestearemos": "ballestear",
    "ballestearen": "ballestear",
    "ballesteares": "ballestear",
    "ballestearon": "ballestear",
    "ballesteará": "ballestear",
    "ballestearán": "ballestear",
    "ballestearás": "ballestear",
    "ballestearé": "ballestear",
    "ballestearéis": "ballestear",
    "ballestearía": "ballestear",
    "ballestearíais": "ballestear",
    "ballestearíamos": "ballestear",
    "ballestearían": "ballestear",
    "ballestearías": "ballestear",
    "ballesteas": "ballestear",
    "ballestease": "ballestear",
    "ballesteaseis": "ballestear",
    "ballesteasen": "ballestear",
    "ballesteases": "ballestear",
    "ballesteaste": "ballestear",
    "ballesteasteis": "ballestear",
    "ballestee": "ballestear",
    "ballesteemos": "ballestear",
    "ballesteen": "ballestear",
    "ballestees": "ballestear",
    "ballesteo": "ballestear",
    "ballestera": "ballestero",
    "ballesteras": "ballestero",
    "ballesteros": "ballestero",
    "ballesterías": "ballestería",
    "ballesteábamos": "ballestear",
    "ballesteáis": "ballestear",
    "ballesteáramos": "ballestear",
    "ballesteáremos": "ballestear",
    "ballesteásemos": "ballestear",
    "ballesteé": "ballestear",
    "ballesteéis": "ballestear",
    "ballesteó": "ballestear",
    "ballestillas": "ballestilla",
    "ballestones": "ballestón",
    "ballestrinques": "ballestrinque",
    "ballets": "ballet",
    "ballicos": "ballico",
    "ballings": "balling",
    "balluecas": "ballueca",
    "balnearia": "balneario",
    "balnearias": "balneario",
    "balnearios": "balneario",
    "balneoterapias": "balneoterapia",
    "balo": "balar",
    "balompiés": "balompié",
    "balompédica": "balompédico",
    "balompédicas": "balompédico",
    "balompédicos": "balompédico",
    "baloncestistas": "baloncestista",
    "baloncestos": "baloncesto",
    "balones": "balón",
    "balonmanistas": "balonmanista",
    "balonmanos": "balonmano",
    "balonvoleas": "balonvolea",
    "balota": "balotar",
    "balotaba": "balotar",
    "balotabais": "balotar",
    "balotaban": "balotar",
    "balotabas": "balotar",
    "balotad": "balotar",
    "balotada": "balotar",
    "balotadas": "balotar",
    "balotado": "balotar",
    "balotados": "balotar",
    "balotamos": "balotar",
    "balotan": "balotar",
    "balotando": "balotar",
    "balotara": "balotar",
    "balotarais": "balotar",
    "balotaran": "balotar",
    "balotaras": "balotar",
    "balotare": "balotar",
    "balotareis": "balotar",
    "balotaremos": "balotar",
    "balotaren": "balotar",
    "balotares": "balotar",
    "balotaron": "balotar",
    "balotará": "balotar",
    "balotarán": "balotar",
    "balotarás": "balotar",
    "balotaré": "balotar",
    "balotaréis": "balotar",
    "balotaría": "balotar",
    "balotaríais": "balotar",
    "balotaríamos": "balotar",
    "balotarían": "balotar",
    "balotarías": "balotar",
    "balotas": "balotar",
    "balotase": "balotar",
    "balotaseis": "balotar",
    "balotasen": "balotar",
    "balotases": "balotar",
    "balotaste": "balotar",
    "balotasteis": "balotar",
    "balote": "balotar",
    "balotemos": "balotar",
    "baloten": "balotar",
    "balotes": "balotar",
    "baloto": "balotar",
    "balotábamos": "balotar",
    "balotáis": "balotar",
    "balotáramos": "balotar",
    "balotáremos": "balotar",
    "balotásemos": "balotar",
    "baloté": "balotar",
    "balotéis": "balotar",
    "balotó": "balotar",
    "balsadera": "balsadero",
    "balsaderas": "balsadero",
    "balsaderos": "balsadero",
    "balsameras": "balsamera",
    "balsameros": "balsamero",
    "balsaminas": "balsamina",
    "balsas": "balsa",
    "balsera": "balsero",
    "balseras": "balsero",
    "balseros": "balsero",
    "balsos": "balso",
    "balsámica": "balsámico",
    "balsámicas": "balsámico",
    "balsámicos": "balsámico",
    "balta": "balto",
    "baltas": "balto",
    "baltos": "balto",
    "baluartes": "baluarte",
    "balumas": "baluma",
    "balumbas": "balumba",
    "balumbos": "balumbo",
    "balábamos": "balar",
    "baláis": "balar",
    "baláramos": "balar",
    "baláremos": "balar",
    "balásemos": "balar",
    "balé": "balar",
    "baléis": "balar",
    "balénidos": "balénido",
    "balística": "balístico",
    "balísticas": "balístico",
    "balísticos": "balístico",
    "baló": "balar",
    "bambalea": "bambalear",
    "bambaleaba": "bambalear",
    "bambaleabais": "bambalear",
    "bambaleaban": "bambalear",
    "bambaleabas": "bambalear",
    "bambalead": "bambalear",
    "bambaleada": "bambalear",
    "bambaleadas": "bambalear",
    "bambaleado": "bambalear",
    "bambaleados": "bambalear",
    "bambaleamos": "bambalear",
    "bambalean": "bambalear",
    "bambaleando": "bambalear",
    "bambaleara": "bambalear",
    "bambalearais": "bambalear",
    "bambalearan": "bambalear",
    "bambalearas": "bambalear",
    "bambaleare": "bambalear",
    "bambaleareis": "bambalear",
    "bambalearemos": "bambalear",
    "bambalearen": "bambalear",
    "bambaleares": "bambalear",
    "bambalearon": "bambalear",
    "bambaleará": "bambalear",
    "bambalearán": "bambalear",
    "bambalearás": "bambalear",
    "bambalearé": "bambalear",
    "bambalearéis": "bambalear",
    "bambalearía": "bambalear",
    "bambalearíais": "bambalear",
    "bambalearíamos": "bambalear",
    "bambalearían": "bambalear",
    "bambalearías": "bambalear",
    "bambaleas": "bambalear",
    "bambalease": "bambalear",
    "bambaleaseis": "bambalear",
    "bambaleasen": "bambalear",
    "bambaleases": "bambalear",
    "bambaleaste": "bambalear",
    "bambaleasteis": "bambalear",
    "bambalee": "bambalear",
    "bambaleemos": "bambalear",
    "bambaleen": "bambalear",
    "bambalees": "bambalear",
    "bambaleo": "bambalear",
    "bambaleos": "bambaleo",
    "bambaleábamos": "bambalear",
    "bambaleáis": "bambalear",
    "bambaleáramos": "bambalear",
    "bambaleáremos": "bambalear",
    "bambaleásemos": "bambalear",
    "bambaleé": "bambalear",
    "bambaleéis": "bambalear",
    "bambaleó": "bambalear",
    "bambalinas": "bambalina",
    "bambalinones": "bambalinón",
    "bambaras": "bambara",
    "bambarrias": "bambarria",
    "bambas": "bamba",
    "bambochadas": "bambochada",
    "bambolea": "bambolear",
    "bamboleaba": "bambolear",
    "bamboleabais": "bambolear",
    "bamboleaban": "bambolear",
    "bamboleabas": "bambolear",
    "bambolead": "bambolear",
    "bamboleada": "bambolear",
    "bamboleadas": "bambolear",
    "bamboleado": "bambolear",
    "bamboleados": "bambolear",
    "bamboleamos": "bambolear",
    "bambolean": "bambolear",
    "bamboleando": "bambolear",
    "bamboleara": "bambolear",
    "bambolearais": "bambolear",
    "bambolearan": "bambolear",
    "bambolearas": "bambolear",
    "bamboleare": "bambolear",
    "bamboleareis": "bambolear",
    "bambolearemos": "bambolear",
    "bambolearen": "bambolear",
    "bamboleares": "bambolear",
    "bambolearon": "bambolear",
    "bamboleará": "bambolear",
    "bambolearán": "bambolear",
    "bambolearás": "bambolear",
    "bambolearé": "bambolear",
    "bambolearéis": "bambolear",
    "bambolearía": "bambolear",
    "bambolearíais": "bambolear",
    "bambolearíamos": "bambolear",
    "bambolearían": "bambolear",
    "bambolearías": "bambolear",
    "bamboleas": "bambolear",
    "bambolease": "bambolear",
    "bamboleaseis": "bambolear",
    "bamboleasen": "bambolear",
    "bamboleases": "bambolear",
    "bamboleaste": "bambolear",
    "bamboleasteis": "bambolear",
    "bambolee": "bambolear",
    "bamboleemos": "bambolear",
    "bamboleen": "bambolear",
    "bambolees": "bambolear",
    "bamboleo": "bambolear",
    "bamboleos": "bamboleo",
    "bamboleábamos": "bambolear",
    "bamboleáis": "bambolear",
    "bamboleáramos": "bambolear",
    "bamboleáremos": "bambolear",
    "bamboleásemos": "bambolear",
    "bamboleé": "bambolear",
    "bamboleéis": "bambolear",
    "bamboleó": "bambolear",
    "bambonea": "bambonear",
    "bamboneaba": "bambonear",
    "bamboneabais": "bambonear",
    "bamboneaban": "bambonear",
    "bamboneabas": "bambonear",
    "bambonead": "bambonear",
    "bamboneada": "bambonear",
    "bamboneadas": "bambonear",
    "bamboneado": "bambonear",
    "bamboneados": "bambonear",
    "bamboneamos": "bambonear",
    "bambonean": "bambonear",
    "bamboneando": "bambonear",
    "bamboneara": "bambonear",
    "bambonearais": "bambonear",
    "bambonearan": "bambonear",
    "bambonearas": "bambonear",
    "bamboneare": "bambonear",
    "bamboneareis": "bambonear",
    "bambonearemos": "bambonear",
    "bambonearen": "bambonear",
    "bamboneares": "bambonear",
    "bambonearon": "bambonear",
    "bamboneará": "bambonear",
    "bambonearán": "bambonear",
    "bambonearás": "bambonear",
    "bambonearé": "bambonear",
    "bambonearéis": "bambonear",
    "bambonearía": "bambonear",
    "bambonearíais": "bambonear",
    "bambonearíamos": "bambonear",
    "bambonearían": "bambonear",
    "bambonearías": "bambonear",
    "bamboneas": "bambonear",
    "bambonease": "bambonear",
    "bamboneaseis": "bambonear",
    "bamboneasen": "bambonear",
    "bamboneases": "bambonear",
    "bamboneaste": "bambonear",
    "bamboneasteis": "bambonear",
    "bambonee": "bambonear",
    "bamboneemos": "bambonear",
    "bamboneen": "bambonear",
    "bambonees": "bambonear",
    "bamboneo": "bambonear",
    "bamboneábamos": "bambonear",
    "bamboneáis": "bambonear",
    "bamboneáramos": "bambonear",
    "bamboneáremos": "bambonear",
    "bamboneásemos": "bambonear",
    "bamboneé": "bambonear",
    "bamboneéis": "bambonear",
    "bamboneó": "bambonear",
    "bambucos": "bambuco",
    "bambúes": "bambú",
    "bambús": "bambú",
    "bambúses": "bambú",
    "banales": "banal",
    "banalidades": "banalidad",
    "bananas": "banana",
    "bananera": "bananero",
    "bananeras": "bananero",
    "bananeros": "bananero",
    "bananos": "banano",
    "banastas": "banasta",
    "banastera": "banastero",
    "banasteras": "banastero",
    "banasteros": "banastero",
    "banastos": "banasto",
    "bancadas": "bancada",
    "bancales": "bancal",
    "bancaria": "bancario",
    "bancarias": "bancario",
    "bancarios": "bancario",
    "bancarrotas": "bancarrota",
    "bancas": "banca",
    "bancazas": "bancaza",
    "bancocracias": "bancocracia",
    "bancones": "bancón",
    "bancos": "banco",
    "bandadas": "bandada",
    "bandas": "banda",
    "bandazos": "bandazo",
    "bandea": "bandear",
    "bandeaba": "bandear",
    "bandeabais": "bandear",
    "bandeaban": "bandear",
    "bandeabas": "bandear",
    "bandead": "bandear",
    "bandeada": "bandear",
    "bandeadas": "bandear",
    "bandeado": "bandear",
    "bandeados": "bandear",
    "bandeamos": "bandear",
    "bandean": "bandear",
    "bandeando": "bandear",
    "bandeara": "bandear",
    "bandearais": "bandear",
    "bandearan": "bandear",
    "bandearas": "bandear",
    "bandeare": "bandear",
    "bandeareis": "bandear",
    "bandearemos": "bandear",
    "bandearen": "bandear",
    "bandeares": "bandear",
    "bandearon": "bandear",
    "bandeará": "bandear",
    "bandearán": "bandear",
    "bandearás": "bandear",
    "bandearé": "bandear",
    "bandearéis": "bandear",
    "bandearía": "bandear",
    "bandearíais": "bandear",
    "bandearíamos": "bandear",
    "bandearían": "bandear",
    "bandearías": "bandear",
    "bandeas": "bandear",
    "bandease": "bandear",
    "bandeaseis": "bandear",
    "bandeasen": "bandear",
    "bandeases": "bandear",
    "bandeaste": "bandear",
    "bandeasteis": "bandear",
    "bandee": "bandear",
    "bandeemos": "bandear",
    "bandeen": "bandear",
    "bandees": "bandear",
    "bandejas": "bandeja",
    "bandeo": "bandear",
    "banderas": "bandera",
    "banderice": "banderizar",
    "bandericemos": "banderizar",
    "bandericen": "banderizar",
    "banderices": "banderizar",
    "bandericé": "banderizar",
    "bandericéis": "banderizar",
    "banderillas": "banderilla",
    "banderillea": "banderillear",
    "banderilleaba": "banderillear",
    "banderilleabais": "banderillear",
    "banderilleaban": "banderillear",
    "banderilleabas": "banderillear",
    "banderillead": "banderillear",
    "banderilleada": "banderillear",
    "banderilleadas": "banderillear",
    "banderilleado": "banderillear",
    "banderilleados": "banderillear",
    "banderilleamos": "banderillear",
    "banderillean": "banderillear",
    "banderilleando": "banderillear",
    "banderilleara": "banderillear",
    "banderillearais": "banderillear",
    "banderillearan": "banderillear",
    "banderillearas": "banderillear",
    "banderilleare": "banderillear",
    "banderilleareis": "banderillear",
    "banderillearemos": "banderillear",
    "banderillearen": "banderillear",
    "banderilleares": "banderillear",
    "banderillearon": "banderillear",
    "banderilleará": "banderillear",
    "banderillearán": "banderillear",
    "banderillearás": "banderillear",
    "banderillearé": "banderillear",
    "banderillearéis": "banderillear",
    "banderillearía": "banderillear",
    "banderillearíais": "banderillear",
    "banderillearíamos": "banderillear",
    "banderillearían": "banderillear",
    "banderillearías": "banderillear",
    "banderilleas": "banderillear",
    "banderillease": "banderillear",
    "banderilleaseis": "banderillear",
    "banderilleasen": "banderillear",
    "banderilleases": "banderillear",
    "banderilleaste": "banderillear",
    "banderilleasteis": "banderillear",
    "banderillee": "banderillear",
    "banderilleemos": "banderillear",
    "banderilleen": "banderillear",
    "banderillees": "banderillear",
    "banderilleo": "banderillear",
    "banderillera": "banderillero",
    "banderilleras": "banderillero",
    "banderilleros": "banderillero",
    "banderilleábamos": "banderillear",
    "banderilleáis": "banderillear",
    "banderilleáramos": "banderillear",
    "banderilleáremos": "banderillear",
    "banderilleásemos": "banderillear",
    "banderilleé": "banderillear",
    "banderilleéis": "banderillear",
    "banderilleó": "banderillear",
    "banderines": "banderín",
    "banderitas": "banderita",
    "banderiza": "banderizar",
    "banderizaba": "banderizar",
    "banderizabais": "banderizar",
    "banderizaban": "banderizar",
    "banderizabas": "banderizar",
    "banderizad": "banderizar",
    "banderizada": "banderizar",
    "banderizadas": "banderizar",
    "banderizado": "banderizar",
    "banderizados": "banderizar",
    "banderizamos": "banderizar",
    "banderizan": "banderizar",
    "banderizando": "banderizar",
    "banderizara": "banderizar",
    "banderizarais": "banderizar",
    "banderizaran": "banderizar",
    "banderizaras": "banderizar",
    "banderizare": "banderizar",
    "banderizareis": "banderizar",
    "banderizaremos": "banderizar",
    "banderizaren": "banderizar",
    "banderizares": "banderizar",
    "banderizaron": "banderizar",
    "banderizará": "banderizar",
    "banderizarán": "banderizar",
    "banderizarás": "banderizar",
    "banderizaré": "banderizar",
    "banderizaréis": "banderizar",
    "banderizaría": "banderizar",
    "banderizaríais": "banderizar",
    "banderizaríamos": "banderizar",
    "banderizarían": "banderizar",
    "banderizarías": "banderizar",
    "banderizas": "banderizar",
    "banderizase": "banderizar",
    "banderizaseis": "banderizar",
    "banderizasen": "banderizar",
    "banderizases": "banderizar",
    "banderizaste": "banderizar",
    "banderizasteis": "banderizar",
    "banderizo": "banderizar",
    "banderizábamos": "banderizar",
    "banderizáis": "banderizar",
    "banderizáramos": "banderizar",
    "banderizáremos": "banderizar",
    "banderizásemos": "banderizar",
    "banderizó": "banderizar",
    "banderolas": "banderola",
    "banderías": "bandería",
    "bandeábamos": "bandear",
    "bandeáis": "bandear",
    "bandeáramos": "bandear",
    "bandeáremos": "bandear",
    "bandeásemos": "bandear",
    "bandeé": "bandear",
    "bandeéis": "bandear",
    "bandeó": "bandear",
    "bandicutes": "bandicut",
    "bandicuts": "bandicut",
    "bandida": "bandido",
    "bandidajes": "bandidaje",
    "bandidas": "bandido",
    "bandidos": "bandido",
    "bandines": "bandín",
    "bandolas": "bandola",
    "bandolera": "bandolero",
    "bandoleras": "bandolero",
    "bandolerismos": "bandolerismo",
    "bandoleros": "bandolero",
    "bandolinas": "bandolina",
    "bandolones": "bandolón",
    "bandolonistas": "bandolonista",
    "bandos": "bando",
    "bandujos": "bandujo",
    "bandurrias": "bandurria",
    "bandurristas": "bandurrista",
    "banjos": "banjo",
    "banquea": "banquear",
    "banqueaba": "banquear",
    "banqueabais": "banquear",
    "banqueaban": "banquear",
    "banqueabas": "banquear",
    "banquead": "banquear",
    "banqueada": "banquear",
    "banqueadas": "banquear",
    "banqueado": "banquear",
    "banqueados": "banquear",
    "banqueamos": "banquear",
    "banquean": "banquear",
    "banqueando": "banquear",
    "banqueara": "banquear",
    "banquearais": "banquear",
    "banquearan": "banquear",
    "banquearas": "banquear",
    "banqueare": "banquear",
    "banqueareis": "banquear",
    "banquearemos": "banquear",
    "banquearen": "banquear",
    "banqueares": "banquear",
    "banquearon": "banquear",
    "banqueará": "banquear",
    "banquearán": "banquear",
    "banquearás": "banquear",
    "banquearé": "banquear",
    "banquearéis": "banquear",
    "banquearía": "banquear",
    "banquearíais": "banquear",
    "banquearíamos": "banquear",
    "banquearían": "banquear",
    "banquearías": "banquear",
    "banqueas": "banquear",
    "banquease": "banquear",
    "banqueaseis": "banquear",
    "banqueasen": "banquear",
    "banqueases": "banquear",
    "banqueaste": "banquear",
    "banqueasteis": "banquear",
    "banquee": "banquear",
    "banqueemos": "banquear",
    "banqueen": "banquear",
    "banquees": "banquear",
    "banqueo": "banquear",
    "banqueos": "banqueo",
    "banquera": "banquero",
    "banqueras": "banquero",
    "banqueros": "banquero",
    "banquetas": "banqueta",
    "banquetea": "banquetear",
    "banqueteaba": "banquetear",
    "banqueteabais": "banquetear",
    "banqueteaban": "banquetear",
    "banqueteabas": "banquetear",
    "banquetead": "banquetear",
    "banqueteada": "banquetear",
    "banqueteadas": "banquetear",
    "banqueteado": "banquetear",
    "banqueteadora": "banqueteador",
    "banqueteadoras": "banqueteador",
    "banqueteadores": "banqueteador",
    "banqueteados": "banquetear",
    "banqueteamos": "banquetear",
    "banquetean": "banquetear",
    "banqueteando": "banquetear",
    "banqueteara": "banquetear",
    "banquetearais": "banquetear",
    "banquetearan": "banquetear",
    "banquetearas": "banquetear",
    "banqueteare": "banquetear",
    "banqueteareis": "banquetear",
    "banquetearemos": "banquetear",
    "banquetearen": "banquetear",
    "banqueteares": "banquetear",
    "banquetearon": "banquetear",
    "banqueteará": "banquetear",
    "banquetearán": "banquetear",
    "banquetearás": "banquetear",
    "banquetearé": "banquetear",
    "banquetearéis": "banquetear",
    "banquetearía": "banquetear",
    "banquetearíais": "banquetear",
    "banquetearíamos": "banquetear",
    "banquetearían": "banquetear",
    "banquetearías": "banquetear",
    "banqueteas": "banquetear",
    "banquetease": "banquetear",
    "banqueteaseis": "banquetear",
    "banqueteasen": "banquetear",
    "banqueteases": "banquetear",
    "banqueteaste": "banquetear",
    "banqueteasteis": "banquetear",
    "banquetee": "banquetear",
    "banqueteemos": "banquetear",
    "banqueteen": "banquetear",
    "banquetees": "banquetear",
    "banqueteo": "banquetear",
    "banquetes": "banquete",
    "banqueteábamos": "banquetear",
    "banqueteáis": "banquetear",
    "banqueteáramos": "banquetear",
    "banqueteáremos": "banquetear",
    "banqueteásemos": "banquetear",
    "banqueteé": "banquetear",
    "banqueteéis": "banquetear",
    "banqueteó": "banquetear",
    "banqueábamos": "banquear",
    "banqueáis": "banquear",
    "banqueáramos": "banquear",
    "banqueáremos": "banquear",
    "banqueásemos": "banquear",
    "banqueé": "banquear",
    "banqueéis": "banquear",
    "banqueó": "banquear",
    "banquiales": "banquial",
    "banquillos": "banquillo",
    "banquisas": "banquisa",
    "bantines": "bantín",
    "bantues": "bantu",
    "bantus": "bantu",
    "bantustanes": "bantustán",
    "bantúes": "bantú",
    "bantús": "bantú",
    "bantúses": "bantú",
    "banzos": "banzo",
    "baobabs": "baobab",
    "baos": "bao",
    "baptisterios": "baptisterio",
    "baquea": "baquear",
    "baqueaba": "baquear",
    "baqueabais": "baquear",
    "baqueaban": "baquear",
    "baqueabas": "baquear",
    "baquead": "baquear",
    "baqueada": "baquear",
    "baqueadas": "baquear",
    "baqueado": "baquear",
    "baqueados": "baquear",
    "baqueamos": "baquear",
    "baquean": "baquear",
    "baqueando": "baquear",
    "baqueara": "baquear",
    "baquearais": "baquear",
    "baquearan": "baquear",
    "baquearas": "baquear",
    "baqueare": "baquear",
    "baqueareis": "baquear",
    "baquearemos": "baquear",
    "baquearen": "baquear",
    "baqueares": "baquear",
    "baquearon": "baquear",
    "baqueará": "baquear",
    "baquearán": "baquear",
    "baquearás": "baquear",
    "baquearé": "baquear",
    "baquearéis": "baquear",
    "baquearía": "baquear",
    "baquearíais": "baquear",
    "baquearíamos": "baquear",
    "baquearían": "baquear",
    "baquearías": "baquear",
    "baqueas": "baquear",
    "baquease": "baquear",
    "baqueaseis": "baquear",
    "baqueasen": "baquear",
    "baqueases": "baquear",
    "baqueaste": "baquear",
    "baqueasteis": "baquear",
    "baquee": "baquear",
    "baqueemos": "baquear",
    "baqueen": "baquear",
    "baquees": "baquear",
    "baquelitas": "baquelita",
    "baqueo": "baquear",
    "baques": "baque",
    "baquetas": "baqueta",
    "baquetazos": "baquetazo",
    "baquetea": "baquetear",
    "baqueteaba": "baquetear",
    "baqueteabais": "baquetear",
    "baqueteaban": "baquetear",
    "baqueteabas": "baquetear",
    "baquetead": "baquetear",
    "baqueteada": "baquetear",
    "baqueteadas": "baquetear",
    "baqueteado": "baquetear",
    "baqueteados": "baquetear",
    "baqueteamos": "baquetear",
    "baquetean": "baquetear",
    "baqueteando": "baquetear",
    "baqueteara": "baquetear",
    "baquetearais": "baquetear",
    "baquetearan": "baquetear",
    "baquetearas": "baquetear",
    "baqueteare": "baquetear",
    "baqueteareis": "baquetear",
    "baquetearemos": "baquetear",
    "baquetearen": "baquetear",
    "baqueteares": "baquetear",
    "baquetearon": "baquetear",
    "baqueteará": "baquetear",
    "baquetearán": "baquetear",
    "baquetearás": "baquetear",
    "baquetearé": "baquetear",
    "baquetearéis": "baquetear",
    "baquetearía": "baquetear",
    "baquetearíais": "baquetear",
    "baquetearíamos": "baquetear",
    "baquetearían": "baquetear",
    "baquetearías": "baquetear",
    "baqueteas": "baquetear",
    "baquetease": "baquetear",
    "baqueteaseis": "baquetear",
    "baqueteasen": "baquetear",
    "baqueteases": "baquetear",
    "baqueteaste": "baquetear",
    "baqueteasteis": "baquetear",
    "baquetee": "baquetear",
    "baqueteemos": "baquetear",
    "baqueteen": "baquetear",
    "baquetees": "baquetear",
    "baqueteo": "baquetear",
    "baqueteos": "baqueteo",
    "baqueteábamos": "baquetear",
    "baqueteáis": "baquetear",
    "baqueteáramos": "baquetear",
    "baqueteáremos": "baquetear",
    "baqueteásemos": "baquetear",
    "baqueteé": "baquetear",
    "baqueteéis": "baquetear",
    "baqueteó": "baquetear",
    "baquetillas": "baquetilla",
    "baqueábamos": "baquear",
    "baqueáis": "baquear",
    "baqueáramos": "baquear",
    "baqueáremos": "baquear",
    "baqueásemos": "baquear",
    "baqueé": "baquear",
    "baqueéis": "baquear",
    "baqueó": "baquear",
    "baquiaba": "baquiar",
    "baquiabais": "baquiar",
    "baquiaban": "baquiar",
    "baquiabas": "baquiar",
    "baquiad": "baquiar",
    "baquiada": "baquiar",
    "baquiadas": "baquiar",
    "baquiado": "baquiar",
    "baquiados": "baquiar",
    "baquiamos": "baquiar",
    "baquiana": "baquiano",
    "baquianas": "baquiano",
    "baquiando": "baquiar",
    "baquianos": "baquiano",
    "baquiara": "baquiar",
    "baquiarais": "baquiar",
    "baquiaran": "baquiar",
    "baquiaras": "baquiar",
    "baquiare": "baquiar",
    "baquiareis": "baquiar",
    "baquiaremos": "baquiar",
    "baquiaren": "baquiar",
    "baquiares": "baquiar",
    "baquiaron": "baquiar",
    "baquiará": "baquiar",
    "baquiarán": "baquiar",
    "baquiarás": "baquiar",
    "baquiaré": "baquiar",
    "baquiaréis": "baquiar",
    "baquiaría": "baquiar",
    "baquiaríais": "baquiar",
    "baquiaríamos": "baquiar",
    "baquiarían": "baquiar",
    "baquiarías": "baquiar",
    "baquiase": "baquiar",
    "baquiaseis": "baquiar",
    "baquiasen": "baquiar",
    "baquiases": "baquiar",
    "baquiaste": "baquiar",
    "baquiasteis": "baquiar",
    "baquiemos": "baquiar",
    "baquios": "baquio",
    "baquiábamos": "baquiar",
    "baquiáis": "baquiar",
    "baquiáramos": "baquiar",
    "baquiáremos": "baquiar",
    "baquiásemos": "baquiar",
    "baquié": "baquiar",
    "baquiéis": "baquiar",
    "baquió": "baquiar",
    "baquía": "baquiar",
    "baquían": "baquiar",
    "baquías": "baquiar",
    "baquíe": "baquiar",
    "baquíen": "baquiar",
    "baquíes": "baquiar",
    "baquío": "baquiar",
    "barahúndas": "barahúnda",
    "baraja": "barajar",
    "barajaba": "barajar",
    "barajabais": "barajar",
    "barajaban": "barajar",
    "barajabas": "barajar",
    "barajad": "barajar",
    "barajada": "barajar",
    "barajadas": "barajar",
    "barajado": "barajar",
    "barajados": "barajar",
    "barajamos": "barajar",
    "barajan": "barajar",
    "barajando": "barajar",
    "barajara": "barajar",
    "barajarais": "barajar",
    "barajaran": "barajar",
    "barajaras": "barajar",
    "barajare": "barajar",
    "barajareis": "barajar",
    "barajaremos": "barajar",
    "barajaren": "barajar",
    "barajares": "barajar",
    "barajaron": "barajar",
    "barajará": "barajar",
    "barajarán": "barajar",
    "barajarás": "barajar",
    "barajaré": "barajar",
    "barajaréis": "barajar",
    "barajaría": "barajar",
    "barajaríais": "barajar",
    "barajaríamos": "barajar",
    "barajarían": "barajar",
    "barajarías": "barajar",
    "barajas": "barajar",
    "barajase": "barajar",
    "barajaseis": "barajar",
    "barajasen": "barajar",
    "barajases": "barajar",
    "barajaste": "barajar",
    "barajasteis": "barajar",
    "baraje": "barajar",
    "barajemos": "barajar",
    "barajen": "barajar",
    "barajes": "barajar",
    "barajo": "barajar",
    "barajones": "barajón",
    "barajábamos": "barajar",
    "barajáis": "barajar",
    "barajáramos": "barajar",
    "barajáremos": "barajar",
    "barajásemos": "barajar",
    "barajé": "barajar",
    "barajéis": "barajar",
    "barajó": "barajar",
    "barandados": "barandado",
    "barandajes": "barandaje",
    "barandales": "barandal",
    "barandas": "baranda",
    "barandillas": "barandilla",
    "barata": "barato",
    "baratas": "barato",
    "baratea": "baratear",
    "barateaba": "baratear",
    "barateabais": "baratear",
    "barateaban": "baratear",
    "barateabas": "baratear",
    "baratead": "baratear",
    "barateada": "baratear",
    "barateadas": "baratear",
    "barateado": "baratear",
    "barateados": "baratear",
    "barateamos": "baratear",
    "baratean": "baratear",
    "barateando": "baratear",
    "barateara": "baratear",
    "baratearais": "baratear",
    "baratearan": "baratear",
    "baratearas": "baratear",
    "barateare": "baratear",
    "barateareis": "baratear",
    "baratearemos": "baratear",
    "baratearen": "baratear",
    "barateares": "baratear",
    "baratearon": "baratear",
    "barateará": "baratear",
    "baratearán": "baratear",
    "baratearás": "baratear",
    "baratearé": "baratear",
    "baratearéis": "baratear",
    "baratearía": "baratear",
    "baratearíais": "baratear",
    "baratearíamos": "baratear",
    "baratearían": "baratear",
    "baratearías": "baratear",
    "barateas": "baratear",
    "baratease": "baratear",
    "barateaseis": "baratear",
    "barateasen": "baratear",
    "barateases": "baratear",
    "barateaste": "baratear",
    "barateasteis": "baratear",
    "baratee": "baratear",
    "barateemos": "baratear",
    "barateen": "baratear",
    "baratees": "baratear",
    "barateo": "baratear",
    "baraterías": "baratería",
    "barateábamos": "baratear",
    "barateáis": "baratear",
    "barateáramos": "baratear",
    "barateáremos": "baratear",
    "barateásemos": "baratear",
    "barateé": "baratear",
    "barateéis": "baratear",
    "barateó": "baratear",
    "baratijas": "baratija",
    "baratillos": "baratillo",
    "baratos": "barato",
    "baraturas": "baratura",
    "baraúndas": "baraúnda",
    "barba": "barbar",
    "barbaba": "barbar",
    "barbabais": "barbar",
    "barbaban": "barbar",
    "barbabas": "barbar",
    "barbacanas": "barbacana",
    "barbacoas": "barbacoa",
    "barbad": "barbar",
    "barbada": "barbar",
    "barbadas": "barbar",
    "barbado": "barbar",
    "barbados": "barbar",
    "barbamos": "barbar",
    "barban": "barbar",
    "barbando": "barbar",
    "barbara": "barbar",
    "barbarais": "barbar",
    "barbaran": "barbar",
    "barbaras": "barbar",
    "barbare": "barbar",
    "barbarea": "barbarear",
    "barbareaba": "barbarear",
    "barbareabais": "barbarear",
    "barbareaban": "barbarear",
    "barbareabas": "barbarear",
    "barbaread": "barbarear",
    "barbareada": "barbarear",
    "barbareadas": "barbarear",
    "barbareado": "barbarear",
    "barbareados": "barbarear",
    "barbareamos": "barbarear",
    "barbarean": "barbarear",
    "barbareando": "barbarear",
    "barbareara": "barbarear",
    "barbarearais": "barbarear",
    "barbarearan": "barbarear",
    "barbarearas": "barbarear",
    "barbareare": "barbarear",
    "barbareareis": "barbarear",
    "barbarearemos": "barbarear",
    "barbarearen": "barbarear",
    "barbareares": "barbarear",
    "barbarearon": "barbarear",
    "barbareará": "barbarear",
    "barbarearán": "barbarear",
    "barbarearás": "barbarear",
    "barbarearé": "barbarear",
    "barbarearéis": "barbarear",
    "barbarearía": "barbarear",
    "barbarearíais": "barbarear",
    "barbarearíamos": "barbarear",
    "barbarearían": "barbarear",
    "barbarearías": "barbarear",
    "barbareas": "barbarear",
    "barbarease": "barbarear",
    "barbareaseis": "barbarear",
    "barbareasen": "barbarear",
    "barbareases": "barbarear",
    "barbareaste": "barbarear",
    "barbareasteis": "barbarear",
    "barbaree": "barbarear",
    "barbareemos": "barbarear",
    "barbareen": "barbarear",
    "barbarees": "barbarear",
    "barbareis": "barbar",
    "barbaremos": "barbar",
    "barbaren": "barbar",
    "barbareo": "barbarear",
    "barbares": "barbar",
    "barbareábamos": "barbarear",
    "barbareáis": "barbarear",
    "barbareáramos": "barbarear",
    "barbareáremos": "barbarear",
    "barbareásemos": "barbarear",
    "barbareé": "barbarear",
    "barbareéis": "barbarear",
    "barbareó": "barbarear",
    "barbarice": "barbarizar",
    "barbaricemos": "barbarizar",
    "barbaricen": "barbarizar",
    "barbarices": "barbarizar",
    "barbaricé": "barbarizar",
    "barbaricéis": "barbarizar",
    "barbaridades": "barbaridad",
    "barbaries": "barbarie",
    "barbarismos": "barbarismo",
    "barbariza": "barbarizar",
    "barbarizaba": "barbarizar",
    "barbarizabais": "barbarizar",
    "barbarizaban": "barbarizar",
    "barbarizabas": "barbarizar",
    "barbarizad": "barbarizar",
    "barbarizada": "barbarizar",
    "barbarizadas": "barbarizar",
    "barbarizado": "barbarizar",
    "barbarizados": "barbarizar",
    "barbarizamos": "barbarizar",
    "barbarizan": "barbarizar",
    "barbarizando": "barbarizar",
    "barbarizara": "barbarizar",
    "barbarizarais": "barbarizar",
    "barbarizaran": "barbarizar",
    "barbarizaras": "barbarizar",
    "barbarizare": "barbarizar",
    "barbarizareis": "barbarizar",
    "barbarizaremos": "barbarizar",
    "barbarizaren": "barbarizar",
    "barbarizares": "barbarizar",
    "barbarizaron": "barbarizar",
    "barbarizará": "barbarizar",
    "barbarizarán": "barbarizar",
    "barbarizarás": "barbarizar",
    "barbarizaré": "barbarizar",
    "barbarizaréis": "barbarizar",
    "barbarizaría": "barbarizar",
    "barbarizaríais": "barbarizar",
    "barbarizaríamos": "barbarizar",
    "barbarizarían": "barbarizar",
    "barbarizarías": "barbarizar",
    "barbarizas": "barbarizar",
    "barbarizase": "barbarizar",
    "barbarizaseis": "barbarizar",
    "barbarizasen": "barbarizar",
    "barbarizases": "barbarizar",
    "barbarizaste": "barbarizar",
    "barbarizasteis": "barbarizar",
    "barbarizo": "barbarizar",
    "barbarizábamos": "barbarizar",
    "barbarizáis": "barbarizar",
    "barbarizáramos": "barbarizar",
    "barbarizáremos": "barbarizar",
    "barbarizásemos": "barbarizar",
    "barbarizó": "barbarizar",
    "barbaron": "barbar",
    "barbará": "barbar",
    "barbarán": "barbar",
    "barbarás": "barbar",
    "barbaré": "barbar",
    "barbaréis": "barbar",
    "barbaría": "barbar",
    "barbaríais": "barbar",
    "barbaríamos": "barbar",
    "barbarían": "barbar",
    "barbarías": "barbar",
    "barbas": "barbar",
    "barbase": "barbar",
    "barbaseis": "barbar",
    "barbasen": "barbar",
    "barbases": "barbar",
    "barbaste": "barbar",
    "barbasteis": "barbar",
    "barbastelos": "barbastelo",
    "barbe": "barbar",
    "barbea": "barbear",
    "barbeaba": "barbear",
    "barbeabais": "barbear",
    "barbeaban": "barbear",
    "barbeabas": "barbear",
    "barbead": "barbear",
    "barbeada": "barbear",
    "barbeadas": "barbear",
    "barbeado": "barbear",
    "barbeados": "barbear",
    "barbeamos": "barbear",
    "barbean": "barbear",
    "barbeando": "barbear",
    "barbeara": "barbear",
    "barbearais": "barbear",
    "barbearan": "barbear",
    "barbearas": "barbear",
    "barbeare": "barbear",
    "barbeareis": "barbear",
    "barbearemos": "barbear",
    "barbearen": "barbear",
    "barbeares": "barbear",
    "barbearon": "barbear",
    "barbeará": "barbear",
    "barbearán": "barbear",
    "barbearás": "barbear",
    "barbearé": "barbear",
    "barbearéis": "barbear",
    "barbearía": "barbear",
    "barbearíais": "barbear",
    "barbearíamos": "barbear",
    "barbearían": "barbear",
    "barbearías": "barbear",
    "barbeas": "barbear",
    "barbease": "barbear",
    "barbeaseis": "barbear",
    "barbeasen": "barbear",
    "barbeases": "barbear",
    "barbeaste": "barbear",
    "barbeasteis": "barbear",
    "barbecha": "barbechar",
    "barbechaba": "barbechar",
    "barbechabais": "barbechar",
    "barbechaban": "barbechar",
    "barbechabas": "barbechar",
    "barbechad": "barbechar",
    "barbechada": "barbechar",
    "barbechadas": "barbechar",
    "barbechado": "barbechar",
    "barbechados": "barbechar",
    "barbechamos": "barbechar",
    "barbechan": "barbechar",
    "barbechando": "barbechar",
    "barbechara": "barbechar",
    "barbecharais": "barbechar",
    "barbecharan": "barbechar",
    "barbecharas": "barbechar",
    "barbechare": "barbechar",
    "barbechareis": "barbechar",
    "barbecharemos": "barbechar",
    "barbecharen": "barbechar",
    "barbechares": "barbechar",
    "barbecharon": "barbechar",
    "barbechará": "barbechar",
    "barbecharán": "barbechar",
    "barbecharás": "barbechar",
    "barbecharé": "barbechar",
    "barbecharéis": "barbechar",
    "barbecharía": "barbechar",
    "barbecharíais": "barbechar",
    "barbecharíamos": "barbechar",
    "barbecharían": "barbechar",
    "barbecharías": "barbechar",
    "barbechas": "barbechar",
    "barbechase": "barbechar",
    "barbechaseis": "barbechar",
    "barbechasen": "barbechar",
    "barbechases": "barbechar",
    "barbechaste": "barbechar",
    "barbechasteis": "barbechar",
    "barbeche": "barbechar",
    "barbechemos": "barbechar",
    "barbechen": "barbechar",
    "barbecheras": "barbechera",
    "barbeches": "barbechar",
    "barbecho": "barbechar",
    "barbechos": "barbecho",
    "barbechábamos": "barbechar",
    "barbecháis": "barbechar",
    "barbecháramos": "barbechar",
    "barbecháremos": "barbechar",
    "barbechásemos": "barbechar",
    "barbeché": "barbechar",
    "barbechéis": "barbechar",
    "barbechó": "barbechar",
    "barbee": "barbear",
    "barbeemos": "barbear",
    "barbeen": "barbear",
    "barbees": "barbear",
    "barbemos": "barbar",
    "barben": "barbar",
    "barbeo": "barbear",
    "barbera": "barbero",
    "barberas": "barbero",
    "barberos": "barbero",
    "barberías": "barbería",
    "barbes": "barbar",
    "barbetas": "barbeta",
    "barbeábamos": "barbear",
    "barbeáis": "barbear",
    "barbeáramos": "barbear",
    "barbeáremos": "barbear",
    "barbeásemos": "barbear",
    "barbeé": "barbear",
    "barbeéis": "barbear",
    "barbeó": "barbear",
    "barbiana": "barbián",
    "barbianas": "barbián",
    "barbianes": "barbián",
    "barbicachos": "barbicacho",
    "barbihecha": "barbihecho",
    "barbihechas": "barbihecho",
    "barbihechos": "barbihecho",
    "barbilampinnna": "barbilampinnno",
    "barbilampinnnas": "barbilampinnno",
    "barbilampinnnos": "barbilampinnno",
    "barbilampiña": "barbilampiño",
    "barbilampiñas": "barbilampiño",
    "barbilampiños": "barbilampiño",
    "barbilinda": "barbilindo",
    "barbilindas": "barbilindo",
    "barbilindos": "barbilindo",
    "barbillas": "barbilla",
    "barbilleras": "barbillera",
    "barbillones": "barbillón",
    "barbilucia": "barbilucio",
    "barbilucias": "barbilucio",
    "barbilucios": "barbilucio",
    "barbiponientes": "barbiponiente",
    "barbipungentes": "barbipungente",
    "barbiquejos": "barbiquejo",
    "barbirrucia": "barbirrucio",
    "barbirrucias": "barbirrucio",
    "barbirrucios": "barbirrucio",
    "barbis": "barbi",
    "barbitaheña": "barbitaheño",
    "barbitaheñas": "barbitaheño",
    "barbitaheños": "barbitaheño",
    "barbitales": "barbital",
    "barbitonas": "barbitona",
    "barbiturismos": "barbiturismo",
    "barbitúrica": "barbitúrico",
    "barbitúricas": "barbitúrico",
    "barbitúricos": "barbitúrico",
    "barbo": "barbar",
    "barbones": "barbón",
    "barboquejos": "barboquejo",
    "barbos": "barbo",
    "barbota": "barbotar",
    "barbotaba": "barbotar",
    "barbotabais": "barbotar",
    "barbotaban": "barbotar",
    "barbotabas": "barbotar",
    "barbotad": "barbotar",
    "barbotada": "barbotar",
    "barbotadas": "barbotar",
    "barbotado": "barbotar",
    "barbotados": "barbotar",
    "barbotamos": "barbotar",
    "barbotan": "barbotar",
    "barbotando": "barbotar",
    "barbotara": "barbotar",
    "barbotarais": "barbotar",
    "barbotaran": "barbotar",
    "barbotaras": "barbotar",
    "barbotare": "barbotar",
    "barbotareis": "barbotar",
    "barbotaremos": "barbotar",
    "barbotaren": "barbotar",
    "barbotares": "barbotar",
    "barbotaron": "barbotar",
    "barbotará": "barbotar",
    "barbotarán": "barbotar",
    "barbotarás": "barbotar",
    "barbotaré": "barbotar",
    "barbotaréis": "barbotar",
    "barbotaría": "barbotar",
    "barbotaríais": "barbotar",
    "barbotaríamos": "barbotar",
    "barbotarían": "barbotar",
    "barbotarías": "barbotar",
    "barbotas": "barbotar",
    "barbotase": "barbotar",
    "barbotaseis": "barbotar",
    "barbotasen": "barbotar",
    "barbotases": "barbotar",
    "barbotaste": "barbotar",
    "barbotasteis": "barbotar",
    "barbote": "barbotar",
    "barbotea": "barbotear",
    "barboteaba": "barbotear",
    "barboteabais": "barbotear",
    "barboteaban": "barbotear",
    "barboteabas": "barbotear",
    "barbotead": "barbotear",
    "barboteada": "barbotear",
    "barboteadas": "barbotear",
    "barboteado": "barbotear",
    "barboteados": "barbotear",
    "barboteamos": "barbotear",
    "barbotean": "barbotear",
    "barboteando": "barbotear",
    "barboteara": "barbotear",
    "barbotearais": "barbotear",
    "barbotearan": "barbotear",
    "barbotearas": "barbotear",
    "barboteare": "barbotear",
    "barboteareis": "barbotear",
    "barbotearemos": "barbotear",
    "barbotearen": "barbotear",
    "barboteares": "barbotear",
    "barbotearon": "barbotear",
    "barboteará": "barbotear",
    "barbotearán": "barbotear",
    "barbotearás": "barbotear",
    "barbotearé": "barbotear",
    "barbotearéis": "barbotear",
    "barbotearía": "barbotear",
    "barbotearíais": "barbotear",
    "barbotearíamos": "barbotear",
    "barbotearían": "barbotear",
    "barbotearías": "barbotear",
    "barboteas": "barbotear",
    "barbotease": "barbotear",
    "barboteaseis": "barbotear",
    "barboteasen": "barbotear",
    "barboteases": "barbotear",
    "barboteaste": "barbotear",
    "barboteasteis": "barbotear",
    "barbotee": "barbotear",
    "barboteemos": "barbotear",
    "barboteen": "barbotear",
    "barbotees": "barbotear",
    "barbotemos": "barbotar",
    "barboten": "barbotar",
    "barboteo": "barbotear",
    "barboteos": "barboteo",
    "barbotes": "barbote",
    "barboteábamos": "barbotear",
    "barboteáis": "barbotear",
    "barboteáramos": "barbotear",
    "barboteáremos": "barbotear",
    "barboteásemos": "barbotear",
    "barboteé": "barbotear",
    "barboteéis": "barbotear",
    "barboteó": "barbotear",
    "barbotinas": "barbotina",
    "barbotines": "barbotín",
    "barboto": "barbotar",
    "barbotábamos": "barbotar",
    "barbotáis": "barbotar",
    "barbotáramos": "barbotar",
    "barbotáremos": "barbotar",
    "barbotásemos": "barbotar",
    "barboté": "barbotar",
    "barbotéis": "barbotar",
    "barbotó": "barbotar",
    "barboures": "barbour",
    "barbours": "barbour",
    "barbuda": "barbudo",
    "barbudas": "barbudo",
    "barbudos": "barbudo",
    "barbulla": "barbullar",
    "barbullaba": "barbullar",
    "barbullabais": "barbullar",
    "barbullaban": "barbullar",
    "barbullabas": "barbullar",
    "barbullad": "barbullar",
    "barbullada": "barbullar",
    "barbulladas": "barbullar",
    "barbullado": "barbullar",
    "barbullados": "barbullar",
    "barbullamos": "barbullar",
    "barbullan": "barbullar",
    "barbullando": "barbullar",
    "barbullara": "barbullar",
    "barbullarais": "barbullar",
    "barbullaran": "barbullar",
    "barbullaras": "barbullar",
    "barbullare": "barbullar",
    "barbullareis": "barbullar",
    "barbullaremos": "barbullar",
    "barbullaren": "barbullar",
    "barbullares": "barbullar",
    "barbullaron": "barbullar",
    "barbullará": "barbullar",
    "barbullarán": "barbullar",
    "barbullarás": "barbullar",
    "barbullaré": "barbullar",
    "barbullaréis": "barbullar",
    "barbullaría": "barbullar",
    "barbullaríais": "barbullar",
    "barbullaríamos": "barbullar",
    "barbullarían": "barbullar",
    "barbullarías": "barbullar",
    "barbullas": "barbullar",
    "barbullase": "barbullar",
    "barbullaseis": "barbullar",
    "barbullasen": "barbullar",
    "barbullases": "barbullar",
    "barbullaste": "barbullar",
    "barbullasteis": "barbullar",
    "barbulle": "barbullar",
    "barbullemos": "barbullar",
    "barbullen": "barbullar",
    "barbulles": "barbullar",
    "barbullidos": "barbullido",
    "barbullo": "barbullar",
    "barbullona": "barbullón",
    "barbullonas": "barbullón",
    "barbullones": "barbullón",
    "barbullábamos": "barbullar",
    "barbulláis": "barbullar",
    "barbulláramos": "barbullar",
    "barbulláremos": "barbullar",
    "barbullásemos": "barbullar",
    "barbullé": "barbullar",
    "barbulléis": "barbullar",
    "barbulló": "barbullar",
    "barbuquejos": "barbuquejo",
    "barbábamos": "barbar",
    "barbáis": "barbar",
    "barbáramos": "barbar",
    "barbáremos": "barbar",
    "barbárica": "barbárico",
    "barbáricas": "barbárico",
    "barbáricos": "barbárico",
    "barbásemos": "barbar",
    "barbé": "barbar",
    "barbéis": "barbar",
    "barbó": "barbar",
    "barcadas": "barcada",
    "barcajes": "barcaje",
    "barcales": "barcal",
    "barcarolas": "barcarola",
    "barcas": "barca",
    "barcazas": "barcaza",
    "barcelonenses": "barcelonense",
    "barcelonesa": "barcelonés",
    "barcelonesas": "barcelonés",
    "barceloneses": "barcelonés",
    "barcelonismos": "barcelonismo",
    "barcelonistas": "barcelonista",
    "barchilona": "barchilón",
    "barchilonas": "barchilón",
    "barchilones": "barchilón",
    "barcias": "barcia",
    "barcina": "barcino",
    "barcinas": "barcino",
    "barcinos": "barcino",
    "barcolongos": "barcolongo",
    "barcoluengos": "barcoluengo",
    "barcones": "barcón",
    "barcos": "barco",
    "bardagueras": "bardaguera",
    "bardales": "bardal",
    "bardas": "barda",
    "bardiotas": "bardiota",
    "bardos": "bardo",
    "bareinesa": "bareinés",
    "bareinesas": "bareinés",
    "bareineses": "bareinés",
    "baremos": "baremo",
    "bares": "bar",
    "barestesias": "barestesia",
    "bargueños": "bargueño",
    "barias": "baria",
    "baricentros": "baricentro",
    "baricéntrica": "baricéntrico",
    "baricéntricas": "baricéntrico",
    "baricéntricos": "baricéntrico",
    "bariones": "barión",
    "barios": "bario",
    "barisferas": "barisfera",
    "baritas": "barita",
    "baritinas": "baritina",
    "barizos": "barizo",
    "barjanes": "barján",
    "barjuletas": "barjuleta",
    "barloa": "barloar",
    "barloaba": "barloar",
    "barloabais": "barloar",
    "barloaban": "barloar",
    "barloabas": "barloar",
    "barload": "barloar",
    "barloada": "barloar",
    "barloadas": "barloar",
    "barloado": "barloar",
    "barloados": "barloar",
    "barloamos": "barloar",
    "barloan": "barloar",
    "barloando": "barloar",
    "barloara": "barloar",
    "barloarais": "barloar",
    "barloaran": "barloar",
    "barloaras": "barloar",
    "barloare": "barloar",
    "barloareis": "barloar",
    "barloaremos": "barloar",
    "barloaren": "barloar",
    "barloares": "barloar",
    "barloaron": "barloar",
    "barloará": "barloar",
    "barloarán": "barloar",
    "barloarás": "barloar",
    "barloaré": "barloar",
    "barloaréis": "barloar",
    "barloaría": "barloar",
    "barloaríais": "barloar",
    "barloaríamos": "barloar",
    "barloarían": "barloar",
    "barloarías": "barloar",
    "barloas": "barloar",
    "barloase": "barloar",
    "barloaseis": "barloar",
    "barloasen": "barloar",
    "barloases": "barloar",
    "barloaste": "barloar",
    "barloasteis": "barloar",
    "barloe": "barloar",
    "barloemos": "barloar",
    "barloen": "barloar",
    "barloes": "barloar",
    "barloo": "barloar",
    "barloventea": "barloventear",
    "barloventeaba": "barloventear",
    "barloventeabais": "barloventear",
    "barloventeaban": "barloventear",
    "barloventeabas": "barloventear",
    "barloventead": "barloventear",
    "barloventeada": "barloventear",
    "barloventeadas": "barloventear",
    "barloventeado": "barloventear",
    "barloventeados": "barloventear",
    "barloventeamos": "barloventear",
    "barloventean": "barloventear",
    "barloventeando": "barloventear",
    "barloventeara": "barloventear",
    "barloventearais": "barloventear",
    "barloventearan": "barloventear",
    "barloventearas": "barloventear",
    "barloventeare": "barloventear",
    "barloventeareis": "barloventear",
    "barloventearemos": "barloventear",
    "barloventearen": "barloventear",
    "barloventeares": "barloventear",
    "barloventearon": "barloventear",
    "barloventeará": "barloventear",
    "barloventearán": "barloventear",
    "barloventearás": "barloventear",
    "barloventearé": "barloventear",
    "barloventearéis": "barloventear",
    "barloventearía": "barloventear",
    "barloventearíais": "barloventear",
    "barloventearíamos": "barloventear",
    "barloventearían": "barloventear",
    "barloventearías": "barloventear",
    "barloventeas": "barloventear",
    "barloventease": "barloventear",
    "barloventeaseis": "barloventear",
    "barloventeasen": "barloventear",
    "barloventeases": "barloventear",
    "barloventeaste": "barloventear",
    "barloventeasteis": "barloventear",
    "barloventee": "barloventear",
    "barloventeemos": "barloventear",
    "barloventeen": "barloventear",
    "barloventees": "barloventear",
    "barloventeo": "barloventear",
    "barloventeábamos": "barloventear",
    "barloventeáis": "barloventear",
    "barloventeáramos": "barloventear",
    "barloventeáremos": "barloventear",
    "barloventeásemos": "barloventear",
    "barloventeé": "barloventear",
    "barloventeéis": "barloventear",
    "barloventeó": "barloventear",
    "barloventos": "barlovento",
    "barloábamos": "barloar",
    "barloáis": "barloar",
    "barloáramos": "barloar",
    "barloáremos": "barloar",
    "barloásemos": "barloar",
    "barloé": "barloar",
    "barloéis": "barloar",
    "barloó": "barloar",
    "barmanes": "barman",
    "barmans": "barman",
    "barnabitas": "barnabita",
    "barnaclas": "barnacla",
    "barnes": "barn",
    "barnice": "barnizar",
    "barnicemos": "barnizar",
    "barnicen": "barnizar",
    "barnices": "barnizar",
    "barnicé": "barnizar",
    "barnicéis": "barnizar",
    "barniza": "barnizar",
    "barnizaba": "barnizar",
    "barnizabais": "barnizar",
    "barnizaban": "barnizar",
    "barnizabas": "barnizar",
    "barnizad": "barnizar",
    "barnizada": "barnizar",
    "barnizadas": "barnizar",
    "barnizado": "barnizar",
    "barnizadora": "barnizador",
    "barnizadoras": "barnizador",
    "barnizadores": "barnizador",
    "barnizados": "barnizar",
    "barnizamos": "barnizar",
    "barnizan": "barnizar",
    "barnizando": "barnizar",
    "barnizara": "barnizar",
    "barnizarais": "barnizar",
    "barnizaran": "barnizar",
    "barnizaras": "barnizar",
    "barnizare": "barnizar",
    "barnizareis": "barnizar",
    "barnizaremos": "barnizar",
    "barnizaren": "barnizar",
    "barnizares": "barnizar",
    "barnizaron": "barnizar",
    "barnizará": "barnizar",
    "barnizarán": "barnizar",
    "barnizarás": "barnizar",
    "barnizaré": "barnizar",
    "barnizaréis": "barnizar",
    "barnizaría": "barnizar",
    "barnizaríais": "barnizar",
    "barnizaríamos": "barnizar",
    "barnizarían": "barnizar",
    "barnizarías": "barnizar",
    "barnizas": "barnizar",
    "barnizase": "barnizar",
    "barnizaseis": "barnizar",
    "barnizasen": "barnizar",
    "barnizases": "barnizar",
    "barnizaste": "barnizar",
    "barnizasteis": "barnizar",
    "barnizo": "barnizar",
    "barnizábamos": "barnizar",
    "barnizáis": "barnizar",
    "barnizáramos": "barnizar",
    "barnizáremos": "barnizar",
    "barnizásemos": "barnizar",
    "barnizó": "barnizar",
    "barns": "barn",
    "baroclínica": "baroclínico",
    "baroclínicas": "baroclínico",
    "baroclínicos": "baroclínico",
    "barodifusiones": "barodifusión",
    "barodontalgias": "barodontalgia",
    "barogramas": "barograma",
    "barometrías": "barometría",
    "barométrica": "barométrico",
    "barométricas": "barométrico",
    "barométricos": "barométrico",
    "barones": "barón",
    "baronesas": "baronesa",
    "baronets": "baronet",
    "baronías": "baronía",
    "baroscopios": "baroscopio",
    "barotermógrafos": "barotermógrafo",
    "barotermómetros": "barotermómetro",
    "barotraumatismos": "barotraumatismo",
    "barotraumática": "barotraumático",
    "barotraumáticas": "barotraumático",
    "barotraumáticos": "barotraumático",
    "barotrópica": "barotrópico",
    "barotrópicas": "barotrópico",
    "barotrópicos": "barotrópico",
    "barquea": "barquear",
    "barqueaba": "barquear",
    "barqueabais": "barquear",
    "barqueaban": "barquear",
    "barqueabas": "barquear",
    "barquead": "barquear",
    "barqueada": "barquear",
    "barqueadas": "barquear",
    "barqueado": "barquear",
    "barqueados": "barquear",
    "barqueamos": "barquear",
    "barquean": "barquear",
    "barqueando": "barquear",
    "barqueara": "barquear",
    "barquearais": "barquear",
    "barquearan": "barquear",
    "barquearas": "barquear",
    "barqueare": "barquear",
    "barqueareis": "barquear",
    "barquearemos": "barquear",
    "barquearen": "barquear",
    "barqueares": "barquear",
    "barquearon": "barquear",
    "barqueará": "barquear",
    "barquearán": "barquear",
    "barquearás": "barquear",
    "barquearé": "barquear",
    "barquearéis": "barquear",
    "barquearía": "barquear",
    "barquearíais": "barquear",
    "barquearíamos": "barquear",
    "barquearían": "barquear",
    "barquearías": "barquear",
    "barqueas": "barquear",
    "barquease": "barquear",
    "barqueaseis": "barquear",
    "barqueasen": "barquear",
    "barqueases": "barquear",
    "barqueaste": "barquear",
    "barqueasteis": "barquear",
    "barquee": "barquear",
    "barqueemos": "barquear",
    "barqueen": "barquear",
    "barquees": "barquear",
    "barqueo": "barquear",
    "barqueos": "barqueo",
    "barquera": "barquero",
    "barqueras": "barquero",
    "barqueros": "barquero",
    "barqueábamos": "barquear",
    "barqueáis": "barquear",
    "barqueáramos": "barquear",
    "barqueáremos": "barquear",
    "barqueásemos": "barquear",
    "barqueé": "barquear",
    "barqueéis": "barquear",
    "barqueó": "barquear",
    "barquillas": "barquilla",
    "barquillera": "barquillero",
    "barquilleras": "barquillero",
    "barquilleros": "barquillero",
    "barquillos": "barquillo",
    "barquinazos": "barquinazo",
    "barquines": "barquín",
    "barquitas": "barquita",
    "barquías": "barquía",
    "barra": "barrer",
    "barraba": "barrar",
    "barrabais": "barrar",
    "barraban": "barrar",
    "barrabas": "barrar",
    "barrabasadas": "barrabasada",
    "barracas": "barraca",
    "barracones": "barracón",
    "barracudas": "barracuda",
    "barrad": "barrar",
    "barrada": "barrar",
    "barradas": "barrar",
    "barrado": "barrar",
    "barrados": "barrar",
    "barraganas": "barragana",
    "barraganerías": "barraganería",
    "barraganes": "barragán",
    "barraganetes": "barraganete",
    "barramos": "barrer",
    "barran": "barrer",
    "barrancales": "barrancal",
    "barrancas": "barranca",
    "barrancos": "barranco",
    "barrancosa": "barrancoso",
    "barrancosas": "barrancoso",
    "barrancosos": "barrancoso",
    "barrando": "barrar",
    "barranqueras": "barranquera",
    "barranquismos": "barranquismo",
    "barraquera": "barraquero",
    "barraqueras": "barraquero",
    "barraqueros": "barraquero",
    "barrara": "barrar",
    "barrarais": "barrar",
    "barraran": "barrar",
    "barraras": "barrar",
    "barrare": "barrar",
    "barrareis": "barrar",
    "barraremos": "barrar",
    "barraren": "barrar",
    "barrares": "barrar",
    "barraron": "barrar",
    "barrará": "barrar",
    "barrarán": "barrar",
    "barrarás": "barrar",
    "barraré": "barrar",
    "barraréis": "barrar",
    "barraría": "barrar",
    "barraríais": "barrar",
    "barraríamos": "barrar",
    "barrarían": "barrar",
    "barrarías": "barrar",
    "barras": "barrer",
    "barrase": "barrar",
    "barraseis": "barrar",
    "barrasen": "barrar",
    "barrases": "barrar",
    "barraste": "barrar",
    "barrasteis": "barrar",
    "barre": "barrer",
    "barrea": "barrear",
    "barreaba": "barrear",
    "barreabais": "barrear",
    "barreaban": "barrear",
    "barreabas": "barrear",
    "barread": "barrear",
    "barreada": "barrear",
    "barreadas": "barrear",
    "barreado": "barrear",
    "barreados": "barrear",
    "barreamos": "barrear",
    "barrean": "barrear",
    "barreando": "barrear",
    "barreara": "barrear",
    "barrearais": "barrear",
    "barrearan": "barrear",
    "barrearas": "barrear",
    "barreare": "barrear",
    "barreareis": "barrear",
    "barrearemos": "barrear",
    "barrearen": "barrear",
    "barreares": "barrear",
    "barrearon": "barrear",
    "barreará": "barrear",
    "barrearán": "barrear",
    "barrearás": "barrear",
    "barrearé": "barrear",
    "barrearéis": "barrear",
    "barrearía": "barrear",
    "barrearíais": "barrear",
    "barrearíamos": "barrear",
    "barrearían": "barrear",
    "barrearías": "barrear",
    "barreas": "barrear",
    "barrease": "barrear",
    "barreaseis": "barrear",
    "barreasen": "barrear",
    "barreases": "barrear",
    "barreaste": "barrear",
    "barreasteis": "barrear",
    "barred": "barrer",
    "barredas": "barreda",
    "barredera": "barredero",
    "barrederas": "barredero",
    "barrederos": "barredero",
    "barredoras": "barredora",
    "barreduras": "barredura",
    "barree": "barrear",
    "barreemos": "barrear",
    "barreen": "barrear",
    "barrees": "barrear",
    "barremos": "barrer",
    "barren": "barrer",
    "barrena": "barrenar",
    "barrenaba": "barrenar",
    "barrenabais": "barrenar",
    "barrenaban": "barrenar",
    "barrenabas": "barrenar",
    "barrenad": "barrenar",
    "barrenada": "barrenar",
    "barrenadas": "barrenar",
    "barrenado": "barrenar",
    "barrenadora": "barrenador",
    "barrenadoras": "barrenador",
    "barrenadores": "barrenador",
    "barrenados": "barrenar",
    "barrenamos": "barrenar",
    "barrenan": "barrenar",
    "barrenando": "barrenar",
    "barrenara": "barrenar",
    "barrenarais": "barrenar",
    "barrenaran": "barrenar",
    "barrenaras": "barrenar",
    "barrenare": "barrenar",
    "barrenareis": "barrenar",
    "barrenaremos": "barrenar",
    "barrenaren": "barrenar",
    "barrenares": "barrenar",
    "barrenaron": "barrenar",
    "barrenará": "barrenar",
    "barrenarán": "barrenar",
    "barrenarás": "barrenar",
    "barrenaré": "barrenar",
    "barrenaréis": "barrenar",
    "barrenaría": "barrenar",
    "barrenaríais": "barrenar",
    "barrenaríamos": "barrenar",
    "barrenarían": "barrenar",
    "barrenarías": "barrenar",
    "barrenas": "barrenar",
    "barrenase": "barrenar",
    "barrenaseis": "barrenar",
    "barrenasen": "barrenar",
    "barrenases": "barrenar",
    "barrenaste": "barrenar",
    "barrenasteis": "barrenar",
    "barrendera": "barrendero",
    "barrenderas": "barrendero",
    "barrenderos": "barrendero",
    "barrene": "barrenar",
    "barrenemos": "barrenar",
    "barrenen": "barrenar",
    "barrenera": "barrenero",
    "barreneras": "barrenero",
    "barreneros": "barrenero",
    "barrenes": "barrenar",
    "barrenillos": "barrenillo",
    "barreno": "barrenar",
    "barrenos": "barreno",
    "barrenábamos": "barrenar",
    "barrenáis": "barrenar",
    "barrenáramos": "barrenar",
    "barrenáremos": "barrenar",
    "barrenásemos": "barrenar",
    "barrené": "barrenar",
    "barrenéis": "barrenar",
    "barrenó": "barrenar",
    "barreo": "barrear",
    "barreos": "barreo",
    "barreras": "barrera",
    "barreremos": "barrer",
    "barreros": "barrero",
    "barrerá": "barrer",
    "barrerán": "barrer",
    "barrerás": "barrer",
    "barreré": "barrer",
    "barreréis": "barrer",
    "barrería": "barrer",
    "barreríais": "barrer",
    "barreríamos": "barrer",
    "barrerían": "barrer",
    "barrerías": "barrer",
    "barres": "barrer",
    "barretas": "barreta",
    "barretea": "barretear",
    "barreteaba": "barretear",
    "barreteabais": "barretear",
    "barreteaban": "barretear",
    "barreteabas": "barretear",
    "barretead": "barretear",
    "barreteada": "barretear",
    "barreteadas": "barretear",
    "barreteado": "barretear",
    "barreteados": "barretear",
    "barreteamos": "barretear",
    "barretean": "barretear",
    "barreteando": "barretear",
    "barreteara": "barretear",
    "barretearais": "barretear",
    "barretearan": "barretear",
    "barretearas": "barretear",
    "barreteare": "barretear",
    "barreteareis": "barretear",
    "barretearemos": "barretear",
    "barretearen": "barretear",
    "barreteares": "barretear",
    "barretearon": "barretear",
    "barreteará": "barretear",
    "barretearán": "barretear",
    "barretearás": "barretear",
    "barretearé": "barretear",
    "barretearéis": "barretear",
    "barretearía": "barretear",
    "barretearíais": "barretear",
    "barretearíamos": "barretear",
    "barretearían": "barretear",
    "barretearías": "barretear",
    "barreteas": "barretear",
    "barretease": "barretear",
    "barreteaseis": "barretear",
    "barreteasen": "barretear",
    "barreteases": "barretear",
    "barreteaste": "barretear",
    "barreteasteis": "barretear",
    "barretee": "barretear",
    "barreteemos": "barretear",
    "barreteen": "barretear",
    "barretees": "barretear",
    "barreteo": "barretear",
    "barretera": "barretero",
    "barreteras": "barretero",
    "barreteros": "barretero",
    "barreteábamos": "barretear",
    "barreteáis": "barretear",
    "barreteáramos": "barretear",
    "barreteáremos": "barretear",
    "barreteásemos": "barretear",
    "barreteé": "barretear",
    "barreteéis": "barretear",
    "barreteó": "barretear",
    "barretinas": "barretina",
    "barreábamos": "barrear",
    "barreáis": "barrear",
    "barreáramos": "barrear",
    "barreáremos": "barrear",
    "barreásemos": "barrear",
    "barreé": "barrear",
    "barreéis": "barrear",
    "barreños": "barreño",
    "barreó": "barrear",
    "barriadas": "barriada",
    "barricadas": "barricada",
    "barricas": "barrica",
    "barrida": "barrer",
    "barridas": "barrer",
    "barrido": "barrer",
    "barridos": "barrido",
    "barriendo": "barrer",
    "barriera": "barrer",
    "barrierais": "barrer",
    "barrieran": "barrer",
    "barrieras": "barrer",
    "barriere": "barrer",
    "barriereis": "barrer",
    "barrieren": "barrer",
    "barrieres": "barrer",
    "barrieron": "barrer",
    "barriese": "barrer",
    "barrieseis": "barrer",
    "barriesen": "barrer",
    "barrieses": "barrer",
    "barrigas": "barriga",
    "barrigona": "barrigón",
    "barrigonas": "barrigón",
    "barrigones": "barrigón",
    "barriguda": "barrigudo",
    "barrigudas": "barrigudo",
    "barrigudos": "barrigudo",
    "barrigueras": "barriguera",
    "barrilera": "barrilero",
    "barrileras": "barrilero",
    "barrileros": "barrilero",
    "barrilerías": "barrilería",
    "barriles": "barril",
    "barriletes": "barrilete",
    "barrillas": "barrilla",
    "barrillos": "barrillo",
    "barrilámenes": "barrilamen",
    "barrimos": "barrer",
    "barriobajera": "barriobajero",
    "barriobajeras": "barriobajero",
    "barriobajeros": "barriobajero",
    "barrios": "barrio",
    "barristas": "barrista",
    "barriste": "barrer",
    "barristeis": "barrer",
    "barritas": "barrita",
    "barrizales": "barrizal",
    "barriéramos": "barrer",
    "barriéremos": "barrer",
    "barriésemos": "barrer",
    "barrió": "barrer",
    "barro": "barrer",
    "barroca": "barroco",
    "barrocas": "barroco",
    "barrocos": "barroco",
    "barrones": "barrón",
    "barroquismos": "barroquismo",
    "barros": "barro",
    "barrosa": "barroso",
    "barrosas": "barroso",
    "barrosos": "barroso",
    "barrotes": "barrote",
    "barruecos": "barrueco",
    "barrujos": "barrujo",
    "barrumbadas": "barrumbada",
    "barrunta": "barruntar",
    "barruntaba": "barruntar",
    "barruntabais": "barruntar",
    "barruntaban": "barruntar",
    "barruntabas": "barruntar",
    "barruntad": "barruntar",
    "barruntada": "barruntar",
    "barruntadas": "barruntar",
    "barruntado": "barruntar",
    "barruntados": "barruntar",
    "barruntamientos": "barruntamiento",
    "barruntamos": "barruntar",
    "barruntan": "barruntar",
    "barruntando": "barruntar",
    "barruntara": "barruntar",
    "barruntarais": "barruntar",
    "barruntaran": "barruntar",
    "barruntaras": "barruntar",
    "barruntare": "barruntar",
    "barruntareis": "barruntar",
    "barruntaremos": "barruntar",
    "barruntaren": "barruntar",
    "barruntares": "barruntar",
    "barruntaron": "barruntar",
    "barruntará": "barruntar",
    "barruntarán": "barruntar",
    "barruntarás": "barruntar",
    "barruntaré": "barruntar",
    "barruntaréis": "barruntar",
    "barruntaría": "barruntar",
    "barruntaríais": "barruntar",
    "barruntaríamos": "barruntar",
    "barruntarían": "barruntar",
    "barruntarías": "barruntar",
    "barruntas": "barruntar",
    "barruntase": "barruntar",
    "barruntaseis": "barruntar",
    "barruntasen": "barruntar",
    "barruntases": "barruntar",
    "barruntaste": "barruntar",
    "barruntasteis": "barruntar",
    "barrunte": "barruntar",
    "barruntemos": "barruntar",
    "barrunten": "barruntar",
    "barruntes": "barrunte",
    "barrunto": "barruntar",
    "barruntos": "barrunto",
    "barruntábamos": "barruntar",
    "barruntáis": "barruntar",
    "barruntáramos": "barruntar",
    "barruntáremos": "barruntar",
    "barruntásemos": "barruntar",
    "barrunté": "barruntar",
    "barruntéis": "barruntar",
    "barruntó": "barruntar",
    "barrábamos": "barrar",
    "barráis": "barrer",
    "barráramos": "barrar",
    "barráremos": "barrar",
    "barrásemos": "barrar",
    "barré": "barrar",
    "barréis": "barrer",
    "barrí": "barrer",
    "barría": "barrer",
    "barríais": "barrer",
    "barríamos": "barrer",
    "barrían": "barrer",
    "barrías": "barrer",
    "barró": "barrar",
    "bartolea": "bartolear",
    "bartoleaba": "bartolear",
    "bartoleabais": "bartolear",
    "bartoleaban": "bartolear",
    "bartoleabas": "bartolear",
    "bartolead": "bartolear",
    "bartoleada": "bartolear",
    "bartoleadas": "bartolear",
    "bartoleado": "bartolear",
    "bartoleados": "bartolear",
    "bartoleamos": "bartolear",
    "bartolean": "bartolear",
    "bartoleando": "bartolear",
    "bartoleara": "bartolear",
    "bartolearais": "bartolear",
    "bartolearan": "bartolear",
    "bartolearas": "bartolear",
    "bartoleare": "bartolear",
    "bartoleareis": "bartolear",
    "bartolearemos": "bartolear",
    "bartolearen": "bartolear",
    "bartoleares": "bartolear",
    "bartolearon": "bartolear",
    "bartoleará": "bartolear",
    "bartolearán": "bartolear",
    "bartolearás": "bartolear",
    "bartolearé": "bartolear",
    "bartolearéis": "bartolear",
    "bartolearía": "bartolear",
    "bartolearíais": "bartolear",
    "bartolearíamos": "bartolear",
    "bartolearían": "bartolear",
    "bartolearías": "bartolear",
    "bartoleas": "bartolear",
    "bartolease": "bartolear",
    "bartoleaseis": "bartolear",
    "bartoleasen": "bartolear",
    "bartoleases": "bartolear",
    "bartoleaste": "bartolear",
    "bartoleasteis": "bartolear",
    "bartolee": "bartolear",
    "bartoleemos": "bartolear",
    "bartoleen": "bartolear",
    "bartolees": "bartolear",
    "bartoleo": "bartolear",
    "bartoleábamos": "bartolear",
    "bartoleáis": "bartolear",
    "bartoleáramos": "bartolear",
    "bartoleáremos": "bartolear",
    "bartoleásemos": "bartolear",
    "bartoleé": "bartolear",
    "bartoleéis": "bartolear",
    "bartoleó": "bartolear",
    "bartolillos": "bartolillo",
    "bartrinas": "bartrina",
    "bartulea": "bartulear",
    "bartuleaba": "bartulear",
    "bartuleabais": "bartulear",
    "bartuleaban": "bartulear",
    "bartuleabas": "bartulear",
    "bartulead": "bartulear",
    "bartuleada": "bartulear",
    "bartuleadas": "bartulear",
    "bartuleado": "bartulear",
    "bartuleados": "bartulear",
    "bartuleamos": "bartulear",
    "bartulean": "bartulear",
    "bartuleando": "bartulear",
    "bartuleara": "bartulear",
    "bartulearais": "bartulear",
    "bartulearan": "bartulear",
    "bartulearas": "bartulear",
    "bartuleare": "bartulear",
    "bartuleareis": "bartulear",
    "bartulearemos": "bartulear",
    "bartulearen": "bartulear",
    "bartuleares": "bartulear",
    "bartulearon": "bartulear",
    "bartuleará": "bartulear",
    "bartulearán": "bartulear",
    "bartulearás": "bartulear",
    "bartulearé": "bartulear",
    "bartulearéis": "bartulear",
    "bartulearía": "bartulear",
    "bartulearíais": "bartulear",
    "bartulearíamos": "bartulear",
    "bartulearían": "bartulear",
    "bartulearías": "bartulear",
    "bartuleas": "bartulear",
    "bartulease": "bartulear",
    "bartuleaseis": "bartulear",
    "bartuleasen": "bartulear",
    "bartuleases": "bartulear",
    "bartuleaste": "bartulear",
    "bartuleasteis": "bartulear",
    "bartulee": "bartulear",
    "bartuleemos": "bartulear",
    "bartuleen": "bartulear",
    "bartulees": "bartulear",
    "bartuleo": "bartulear",
    "bartuleábamos": "bartulear",
    "bartuleáis": "bartulear",
    "bartuleáramos": "bartulear",
    "bartuleáremos": "bartulear",
    "bartuleásemos": "bartulear",
    "bartuleé": "bartulear",
    "bartuleéis": "bartulear",
    "bartuleó": "bartulear",
    "barucas": "baruca",
    "barullera": "barullero",
    "barulleras": "barullero",
    "barulleros": "barullero",
    "barullos": "barullo",
    "barzonea": "barzonear",
    "barzoneaba": "barzonear",
    "barzoneabais": "barzonear",
    "barzoneaban": "barzonear",
    "barzoneabas": "barzonear",
    "barzonead": "barzonear",
    "barzoneada": "barzonear",
    "barzoneadas": "barzonear",
    "barzoneado": "barzonear",
    "barzoneados": "barzonear",
    "barzoneamos": "barzonear",
    "barzonean": "barzonear",
    "barzoneando": "barzonear",
    "barzoneara": "barzonear",
    "barzonearais": "barzonear",
    "barzonearan": "barzonear",
    "barzonearas": "barzonear",
    "barzoneare": "barzonear",
    "barzoneareis": "barzonear",
    "barzonearemos": "barzonear",
    "barzonearen": "barzonear",
    "barzoneares": "barzonear",
    "barzonearon": "barzonear",
    "barzoneará": "barzonear",
    "barzonearán": "barzonear",
    "barzonearás": "barzonear",
    "barzonearé": "barzonear",
    "barzonearéis": "barzonear",
    "barzonearía": "barzonear",
    "barzonearíais": "barzonear",
    "barzonearíamos": "barzonear",
    "barzonearían": "barzonear",
    "barzonearías": "barzonear",
    "barzoneas": "barzonear",
    "barzonease": "barzonear",
    "barzoneaseis": "barzonear",
    "barzoneasen": "barzonear",
    "barzoneases": "barzonear",
    "barzoneaste": "barzonear",
    "barzoneasteis": "barzonear",
    "barzonee": "barzonear",
    "barzoneemos": "barzonear",
    "barzoneen": "barzonear",
    "barzonees": "barzonear",
    "barzoneo": "barzonear",
    "barzones": "barzón",
    "barzoneábamos": "barzonear",
    "barzoneáis": "barzonear",
    "barzoneáramos": "barzonear",
    "barzoneáremos": "barzonear",
    "barzoneásemos": "barzonear",
    "barzoneé": "barzonear",
    "barzoneéis": "barzonear",
    "barzoneó": "barzonear",
    "barítona": "barítono",
    "barítonas": "barítono",
    "barítonos": "barítono",
    "barógrafos": "barógrafo",
    "barómetros": "barómetro",
    "baróscopos": "baróscopo",
    "bas": "ba",
    "basa": "basar",
    "basaba": "basar",
    "basabais": "basar",
    "basaban": "basar",
    "basabas": "basar",
    "basad": "basar",
    "basada": "basar",
    "basadas": "basar",
    "basado": "basar",
    "basados": "basar",
    "basales": "basal",
    "basaltos": "basalto",
    "basamentos": "basamento",
    "basamos": "basar",
    "basan": "basar",
    "basando": "basar",
    "basara": "basar",
    "basarais": "basar",
    "basaran": "basar",
    "basaras": "basar",
    "basare": "basar",
    "basareis": "basar",
    "basaremos": "basar",
    "basaren": "basar",
    "basares": "basar",
    "basaron": "basar",
    "basará": "basar",
    "basarán": "basar",
    "basarás": "basar",
    "basaré": "basar",
    "basaréis": "basar",
    "basaría": "basar",
    "basaríais": "basar",
    "basaríamos": "basar",
    "basarían": "basar",
    "basarías": "basar",
    "basas": "basar",
    "basase": "basar",
    "basaseis": "basar",
    "basasen": "basar",
    "basases": "basar",
    "basaste": "basar",
    "basasteis": "basar",
    "bascas": "basca",
    "bascosa": "bascoso",
    "bascosas": "bascoso",
    "bascosidades": "bascosidad",
    "bascosos": "bascoso",
    "bascula": "bascular",
    "basculaba": "bascular",
    "basculabais": "bascular",
    "basculaban": "bascular",
    "basculabas": "bascular",
    "basculables": "basculable",
    "basculaciones": "basculación",
    "basculad": "bascular",
    "basculada": "bascular",
    "basculadas": "bascular",
    "basculado": "bascular",
    "basculadora": "basculador",
    "basculadoras": "basculador",
    "basculadores": "basculador",
    "basculados": "bascular",
    "basculajes": "basculaje",
    "basculamos": "bascular",
    "basculan": "bascular",
    "basculando": "bascular",
    "basculantes": "basculante",
    "basculara": "bascular",
    "bascularais": "bascular",
    "bascularan": "bascular",
    "bascularas": "bascular",
    "basculare": "bascular",
    "basculareis": "bascular",
    "bascularemos": "bascular",
    "bascularen": "bascular",
    "basculares": "bascular",
    "bascularon": "bascular",
    "basculará": "bascular",
    "bascularán": "bascular",
    "bascularás": "bascular",
    "bascularé": "bascular",
    "bascularéis": "bascular",
    "bascularía": "bascular",
    "bascularíais": "bascular",
    "bascularíamos": "bascular",
    "bascularían": "bascular",
    "bascularías": "bascular",
    "basculas": "bascular",
    "basculase": "bascular",
    "basculaseis": "bascular",
    "basculasen": "bascular",
    "basculases": "bascular",
    "basculaste": "bascular",
    "basculasteis": "bascular",
    "bascule": "bascular",
    "basculemos": "bascular",
    "basculen": "bascular",
    "bascules": "bascular",
    "basculo": "bascular",
    "basculábamos": "bascular",
    "basculáis": "bascular",
    "basculáramos": "bascular",
    "basculáremos": "bascular",
    "basculásemos": "bascular",
    "basculé": "bascular",
    "basculéis": "bascular",
    "basculó": "bascular",
    "bascuñanas": "bascuñana",
    "base": "basar",
    "basemos": "basar",
    "basen": "basar",
    "bases": "base",
    "basiales": "basial",
    "basialveolares": "basialveolar",
    "basicidades": "basicidad",
    "basicraneales": "basicraneal",
    "basidiomicetes": "basidiomicete",
    "basidios": "basidio",
    "basidiósporas": "basidióspora",
    "basifaciales": "basifacial",
    "basilares": "basilar",
    "basilia": "basilio",
    "basilias": "basilio",
    "basilicales": "basilical",
    "basilicona": "basilicón",
    "basiliconas": "basilicón",
    "basilicones": "basilicón",
    "basilios": "basilio",
    "basiliscos": "basilisco",
    "basimétrica": "basimétrico",
    "basimétricas": "basimétrico",
    "basimétricos": "basimétrico",
    "basioglosa": "basiogloso",
    "basioglosas": "basiogloso",
    "basioglosos": "basiogloso",
    "basiones": "basión",
    "basivertebrales": "basivertebral",
    "baskets": "basket",
    "baso": "basar",
    "basocitos": "basocito",
    "basofilias": "basofilia",
    "basolaterales": "basolateral",
    "basquea": "basquear",
    "basqueaba": "basquear",
    "basqueabais": "basquear",
    "basqueaban": "basquear",
    "basqueabas": "basquear",
    "basquead": "basquear",
    "basqueada": "basquear",
    "basqueadas": "basquear",
    "basqueado": "basquear",
    "basqueados": "basquear",
    "basqueamos": "basquear",
    "basquean": "basquear",
    "basqueando": "basquear",
    "basqueara": "basquear",
    "basquearais": "basquear",
    "basquearan": "basquear",
    "basquearas": "basquear",
    "basqueare": "basquear",
    "basqueareis": "basquear",
    "basquearemos": "basquear",
    "basquearen": "basquear",
    "basqueares": "basquear",
    "basquearon": "basquear",
    "basqueará": "basquear",
    "basquearán": "basquear",
    "basquearás": "basquear",
    "basquearé": "basquear",
    "basquearéis": "basquear",
    "basquearía": "basquear",
    "basquearíais": "basquear",
    "basquearíamos": "basquear",
    "basquearían": "basquear",
    "basquearías": "basquear",
    "basqueas": "basquear",
    "basquease": "basquear",
    "basqueaseis": "basquear",
    "basqueasen": "basquear",
    "basqueases": "basquear",
    "basqueaste": "basquear",
    "basqueasteis": "basquear",
    "basquee": "basquear",
    "basqueemos": "basquear",
    "basqueen": "basquear",
    "basquees": "basquear",
    "basqueo": "basquear",
    "basqueábamos": "basquear",
    "basqueáis": "basquear",
    "basqueáramos": "basquear",
    "basqueáremos": "basquear",
    "basqueásemos": "basquear",
    "basqueé": "basquear",
    "basqueéis": "basquear",
    "basqueó": "basquear",
    "basquillas": "basquilla",
    "basquiñas": "basquiña",
    "bassets": "basset",
    "basta": "basto",
    "bastaba": "bastar",
    "bastabais": "bastar",
    "bastaban": "bastar",
    "bastabas": "bastar",
    "bastad": "bastar",
    "bastada": "bastar",
    "bastadas": "bastar",
    "bastado": "bastar",
    "bastados": "bastar",
    "bastamos": "bastar",
    "bastan": "bastar",
    "bastando": "bastar",
    "bastantea": "bastantear",
    "bastanteaba": "bastantear",
    "bastanteabais": "bastantear",
    "bastanteaban": "bastantear",
    "bastanteabas": "bastantear",
    "bastantead": "bastantear",
    "bastanteada": "bastantear",
    "bastanteadas": "bastantear",
    "bastanteado": "bastantear",
    "bastanteados": "bastantear",
    "bastanteamos": "bastantear",
    "bastantean": "bastantear",
    "bastanteando": "bastantear",
    "bastanteara": "bastantear",
    "bastantearais": "bastantear",
    "bastantearan": "bastantear",
    "bastantearas": "bastantear",
    "bastanteare": "bastantear",
    "bastanteareis": "bastantear",
    "bastantearemos": "bastantear",
    "bastantearen": "bastantear",
    "bastanteares": "bastantear",
    "bastantearon": "bastantear",
    "bastanteará": "bastantear",
    "bastantearán": "bastantear",
    "bastantearás": "bastantear",
    "bastantearé": "bastantear",
    "bastantearéis": "bastantear",
    "bastantearía": "bastantear",
    "bastantearíais": "bastantear",
    "bastantearíamos": "bastantear",
    "bastantearían": "bastantear",
    "bastantearías": "bastantear",
    "bastanteas": "bastantear",
    "bastantease": "bastantear",
    "bastanteaseis": "bastantear",
    "bastanteasen": "bastantear",
    "bastanteases": "bastantear",
    "bastanteaste": "bastantear",
    "bastanteasteis": "bastantear",
    "bastantee": "bastantear",
    "bastanteemos": "bastantear",
    "bastanteen": "bastantear",
    "bastantees": "bastantear",
    "bastanteo": "bastantear",
    "bastanteos": "bastanteo",
    "bastantes": "bastante",
    "bastanteábamos": "bastantear",
    "bastanteáis": "bastantear",
    "bastanteáramos": "bastantear",
    "bastanteáremos": "bastantear",
    "bastanteásemos": "bastantear",
    "bastanteé": "bastantear",
    "bastanteéis": "bastantear",
    "bastanteó": "bastantear",
    "bastara": "bastar",
    "bastarais": "bastar",
    "bastaran": "bastar",
    "bastaras": "bastar",
    "bastarda": "bastardo",
    "bastardas": "bastardo",
    "bastardea": "bastardear",
    "bastardeaba": "bastardear",
    "bastardeabais": "bastardear",
    "bastardeaban": "bastardear",
    "bastardeabas": "bastardear",
    "bastardead": "bastardear",
    "bastardeada": "bastardear",
    "bastardeadas": "bastardear",
    "bastardeado": "bastardear",
    "bastardeados": "bastardear",
    "bastardeamos": "bastardear",
    "bastardean": "bastardear",
    "bastardeando": "bastardear",
    "bastardeara": "bastardear",
    "bastardearais": "bastardear",
    "bastardearan": "bastardear",
    "bastardearas": "bastardear",
    "bastardeare": "bastardear",
    "bastardeareis": "bastardear",
    "bastardearemos": "bastardear",
    "bastardearen": "bastardear",
    "bastardeares": "bastardear",
    "bastardearon": "bastardear",
    "bastardeará": "bastardear",
    "bastardearán": "bastardear",
    "bastardearás": "bastardear",
    "bastardearé": "bastardear",
    "bastardearéis": "bastardear",
    "bastardearía": "bastardear",
    "bastardearíais": "bastardear",
    "bastardearíamos": "bastardear",
    "bastardearían": "bastardear",
    "bastardearías": "bastardear",
    "bastardeas": "bastardear",
    "bastardease": "bastardear",
    "bastardeaseis": "bastardear",
    "bastardeasen": "bastardear",
    "bastardeases": "bastardear",
    "bastardeaste": "bastardear",
    "bastardeasteis": "bastardear",
    "bastardee": "bastardear",
    "bastardeemos": "bastardear",
    "bastardeen": "bastardear",
    "bastardees": "bastardear",
    "bastardeo": "bastardear",
    "bastardeábamos": "bastardear",
    "bastardeáis": "bastardear",
    "bastardeáramos": "bastardear",
    "bastardeáremos": "bastardear",
    "bastardeásemos": "bastardear",
    "bastardeé": "bastardear",
    "bastardeéis": "bastardear",
    "bastardeó": "bastardear",
    "bastardillas": "bastardilla",
    "bastardos": "bastardo",
    "bastardías": "bastardía",
    "bastare": "bastar",
    "bastareis": "bastar",
    "bastaremos": "bastar",
    "bastaren": "bastar",
    "bastares": "bastar",
    "bastaron": "bastar",
    "bastará": "bastar",
    "bastarán": "bastar",
    "bastarás": "bastar",
    "bastaré": "bastar",
    "bastaréis": "bastar",
    "bastaría": "bastar",
    "bastaríais": "bastar",
    "bastaríamos": "bastar",
    "bastarían": "bastar",
    "bastarías": "bastar",
    "bastas": "basto",
    "bastase": "bastar",
    "bastaseis": "bastar",
    "bastasen": "bastar",
    "bastases": "bastar",
    "bastaste": "bastar",
    "bastasteis": "bastar",
    "baste": "bastar",
    "bastea": "bastear",
    "basteaba": "bastear",
    "basteabais": "bastear",
    "basteaban": "bastear",
    "basteabas": "bastear",
    "bastead": "bastear",
    "basteada": "bastear",
    "basteadas": "bastear",
    "basteado": "bastear",
    "basteados": "bastear",
    "basteamos": "bastear",
    "bastean": "bastear",
    "basteando": "bastear",
    "basteara": "bastear",
    "bastearais": "bastear",
    "bastearan": "bastear",
    "bastearas": "bastear",
    "basteare": "bastear",
    "basteareis": "bastear",
    "bastearemos": "bastear",
    "bastearen": "bastear",
    "basteares": "bastear",
    "bastearon": "bastear",
    "basteará": "bastear",
    "bastearán": "bastear",
    "bastearás": "bastear",
    "bastearé": "bastear",
    "bastearéis": "bastear",
    "bastearía": "bastear",
    "bastearíais": "bastear",
    "bastearíamos": "bastear",
    "bastearían": "bastear",
    "bastearías": "bastear",
    "basteas": "bastear",
    "bastease": "bastear",
    "basteaseis": "bastear",
    "basteasen": "bastear",
    "basteases": "bastear",
    "basteaste": "bastear",
    "basteasteis": "bastear",
    "bastedads": "bastedad",
    "bastee": "bastear",
    "basteemos": "bastear",
    "basteen": "bastear",
    "bastees": "bastear",
    "bastemos": "bastar",
    "basten": "bastar",
    "basteo": "bastear",
    "bastera": "bastero",
    "basteras": "bastero",
    "basteros": "bastero",
    "bastes": "bastar",
    "bastetana": "bastetano",
    "bastetanas": "bastetano",
    "bastetanos": "bastetano",
    "bastezas": "basteza",
    "basteábamos": "bastear",
    "basteáis": "bastear",
    "basteáramos": "bastear",
    "basteáremos": "bastear",
    "basteásemos": "bastear",
    "basteé": "bastear",
    "basteéis": "bastear",
    "basteó": "bastear",
    "bastidas": "bastida",
    "bastidores": "bastidor",
    "bastillas": "bastilla",
    "bastimenta": "bastimentar",
    "bastimentaba": "bastimentar",
    "bastimentabais": "bastimentar",
    "bastimentaban": "bastimentar",
    "bastimentabas": "bastimentar",
    "bastimentad": "bastimentar",
    "bastimentada": "bastimentar",
    "bastimentadas": "bastimentar",
    "bastimentado": "bastimentar",
    "bastimentados": "bastimentar",
    "bastimentamos": "bastimentar",
    "bastimentan": "bastimentar",
    "bastimentando": "bastimentar",
    "bastimentara": "bastimentar",
    "bastimentarais": "bastimentar",
    "bastimentaran": "bastimentar",
    "bastimentaras": "bastimentar",
    "bastimentare": "bastimentar",
    "bastimentareis": "bastimentar",
    "bastimentaremos": "bastimentar",
    "bastimentaren": "bastimentar",
    "bastimentares": "bastimentar",
    "bastimentaron": "bastimentar",
    "bastimentará": "bastimentar",
    "bastimentarán": "bastimentar",
    "bastimentarás": "bastimentar",
    "bastimentaré": "bastimentar",
    "bastimentaréis": "bastimentar",
    "bastimentaría": "bastimentar",
    "bastimentaríais": "bastimentar",
    "bastimentaríamos": "bastimentar",
    "bastimentarían": "bastimentar",
    "bastimentarías": "bastimentar",
    "bastimentas": "bastimentar",
    "bastimentase": "bastimentar",
    "bastimentaseis": "bastimentar",
    "bastimentasen": "bastimentar",
    "bastimentases": "bastimentar",
    "bastimentaste": "bastimentar",
    "bastimentasteis": "bastimentar",
    "bastimente": "bastimentar",
    "bastimentemos": "bastimentar",
    "bastimenten": "bastimentar",
    "bastimentes": "bastimentar",
    "bastimento": "bastimentar",
    "bastimentos": "bastimento",
    "bastimentábamos": "bastimentar",
    "bastimentáis": "bastimentar",
    "bastimentáramos": "bastimentar",
    "bastimentáremos": "bastimentar",
    "bastimentásemos": "bastimentar",
    "bastimenté": "bastimentar",
    "bastimentéis": "bastimentar",
    "bastimentó": "bastimentar",
    "bastiones": "bastión",
    "bastnesitas": "bastnesita",
    "basto": "bastar",
    "bastonazos": "bastonazo",
    "bastoncillos": "bastoncillo",
    "bastonea": "bastonear",
    "bastoneaba": "bastonear",
    "bastoneabais": "bastonear",
    "bastoneaban": "bastonear",
    "bastoneabas": "bastonear",
    "bastonead": "bastonear",
    "bastoneada": "bastonear",
    "bastoneadas": "bastonear",
    "bastoneado": "bastonear",
    "bastoneados": "bastonear",
    "bastoneamos": "bastonear",
    "bastonean": "bastonear",
    "bastoneando": "bastonear",
    "bastoneara": "bastonear",
    "bastonearais": "bastonear",
    "bastonearan": "bastonear",
    "bastonearas": "bastonear",
    "bastoneare": "bastonear",
    "bastoneareis": "bastonear",
    "bastonearemos": "bastonear",
    "bastonearen": "bastonear",
    "bastoneares": "bastonear",
    "bastonearon": "bastonear",
    "bastoneará": "bastonear",
    "bastonearán": "bastonear",
    "bastonearás": "bastonear",
    "bastonearé": "bastonear",
    "bastonearéis": "bastonear",
    "bastonearía": "bastonear",
    "bastonearíais": "bastonear",
    "bastonearíamos": "bastonear",
    "bastonearían": "bastonear",
    "bastonearías": "bastonear",
    "bastoneas": "bastonear",
    "bastonease": "bastonear",
    "bastoneaseis": "bastonear",
    "bastoneasen": "bastonear",
    "bastoneases": "bastonear",
    "bastoneaste": "bastonear",
    "bastoneasteis": "bastonear",
    "bastonee": "bastonear",
    "bastoneemos": "bastonear",
    "bastoneen": "bastonear",
    "bastonees": "bastonear",
    "bastoneo": "bastonear",
    "bastoneos": "bastoneo",
    "bastonera": "bastonero",
    "bastoneras": "bastonero",
    "bastoneros": "bastonero",
    "bastones": "bastón",
    "bastoneábamos": "bastonear",
    "bastoneáis": "bastonear",
    "bastoneáramos": "bastonear",
    "bastoneáremos": "bastonear",
    "bastoneásemos": "bastonear",
    "bastoneé": "bastonear",
    "bastoneéis": "bastonear",
    "bastoneó": "bastonear",
    "bastos": "basto",
    "bastábamos": "bastar",
    "bastáis": "bastar",
    "bastáramos": "bastar",
    "bastáremos": "bastar",
    "bastásemos": "bastar",
    "basté": "bastar",
    "bastéis": "bastar",
    "bastó": "bastar",
    "basuras": "basura",
    "basurea": "basurear",
    "basureaba": "basurear",
    "basureabais": "basurear",
    "basureaban": "basurear",
    "basureabas": "basurear",
    "basuread": "basurear",
    "basureada": "basurear",
    "basureadas": "basurear",
    "basureado": "basurear",
    "basureados": "basurear",
    "basureamos": "basurear",
    "basurean": "basurear",
    "basureando": "basurear",
    "basureara": "basurear",
    "basurearais": "basurear",
    "basurearan": "basurear",
    "basurearas": "basurear",
    "basureare": "basurear",
    "basureareis": "basurear",
    "basurearemos": "basurear",
    "basurearen": "basurear",
    "basureares": "basurear",
    "basurearon": "basurear",
    "basureará": "basurear",
    "basurearán": "basurear",
    "basurearás": "basurear",
    "basurearé": "basurear",
    "basurearéis": "basurear",
    "basurearía": "basurear",
    "basurearíais": "basurear",
    "basurearíamos": "basurear",
    "basurearían": "basurear",
    "basurearías": "basurear",
    "basureas": "basurear",
    "basurease": "basurear",
    "basureaseis": "basurear",
    "basureasen": "basurear",
    "basureases": "basurear",
    "basureaste": "basurear",
    "basureasteis": "basurear",
    "basuree": "basurear",
    "basureemos": "basurear",
    "basureen": "basurear",
    "basurees": "basurear",
    "basureo": "basurear",
    "basurera": "basurero",
    "basureras": "basurero",
    "basureros": "basurero",
    "basureábamos": "basurear",
    "basureáis": "basurear",
    "basureáramos": "basurear",
    "basureáremos": "basurear",
    "basureásemos": "basurear",
    "basureé": "basurear",
    "basureéis": "basurear",
    "basureó": "basurear",
    "basábamos": "basar",
    "basáis": "basar",
    "basáltica": "basáltico",
    "basálticas": "basáltico",
    "basálticos": "basáltico",
    "basáramos": "basar",
    "basáremos": "basar",
    "basásemos": "basar",
    "basé": "basar",
    "baséis": "basar",
    "basílica": "basílico",
    "basílicas": "basílico",
    "basílicos": "basílico",
    "basó": "basar",
    "basófila": "basófilo",
    "basófilas": "basófilo",
    "basófilos": "basófilo",
    "bata": "batir",
    "batacazos": "batacazo",
    "bataholas": "batahola",
    "batakes": "batak",
    "bataks": "batak",
    "batalla": "batallar",
    "batallaba": "batallar",
    "batallabais": "batallar",
    "batallaban": "batallar",
    "batallabas": "batallar",
    "batallad": "batallar",
    "batallada": "batallar",
    "batalladas": "batallar",
    "batallado": "batallar",
    "batalladora": "batallador",
    "batalladoras": "batallador",
    "batalladores": "batallador",
    "batallados": "batallar",
    "batallamos": "batallar",
    "batallan": "batallar",
    "batallando": "batallar",
    "batallara": "batallar",
    "batallarais": "batallar",
    "batallaran": "batallar",
    "batallaras": "batallar",
    "batallare": "batallar",
    "batallareis": "batallar",
    "batallaremos": "batallar",
    "batallaren": "batallar",
    "batallares": "batallar",
    "batallaron": "batallar",
    "batallará": "batallar",
    "batallarán": "batallar",
    "batallarás": "batallar",
    "batallaré": "batallar",
    "batallaréis": "batallar",
    "batallaría": "batallar",
    "batallaríais": "batallar",
    "batallaríamos": "batallar",
    "batallarían": "batallar",
    "batallarías": "batallar",
    "batallas": "batallar",
    "batallase": "batallar",
    "batallaseis": "batallar",
    "batallasen": "batallar",
    "batallases": "batallar",
    "batallaste": "batallar",
    "batallasteis": "batallar",
    "batalle": "batallar",
    "batallemos": "batallar",
    "batallen": "batallar",
    "batalles": "batallar",
    "batallo": "batallar",
    "batallona": "batallón",
    "batallonas": "batallón",
    "batallones": "batallón",
    "batallábamos": "batallar",
    "batalláis": "batallar",
    "batalláramos": "batallar",
    "batalláremos": "batallar",
    "batallásemos": "batallar",
    "batallé": "batallar",
    "batalléis": "batallar",
    "batalló": "batallar",
    "batamos": "batir",
    "batan": "batir",
    "batana": "batanar",
    "batanaba": "batanar",
    "batanabais": "batanar",
    "batanaban": "batanar",
    "batanabas": "batanar",
    "batanad": "batanar",
    "batanada": "batanar",
    "batanadas": "batanar",
    "batanado": "batanar",
    "batanados": "batanar",
    "batanamos": "batanar",
    "batanan": "batanar",
    "batanando": "batanar",
    "batanara": "batanar",
    "batanarais": "batanar",
    "batanaran": "batanar",
    "batanaras": "batanar",
    "batanare": "batanar",
    "batanareis": "batanar",
    "batanaremos": "batanar",
    "batanaren": "batanar",
    "batanares": "batanar",
    "batanaron": "batanar",
    "batanará": "batanar",
    "batanarán": "batanar",
    "batanarás": "batanar",
    "batanaré": "batanar",
    "batanaréis": "batanar",
    "batanaría": "batanar",
    "batanaríais": "batanar",
    "batanaríamos": "batanar",
    "batanarían": "batanar",
    "batanarías": "batanar",
    "batanas": "batanar",
    "batanase": "batanar",
    "batanaseis": "batanar",
    "batanasen": "batanar",
    "batanases": "batanar",
    "batanaste": "batanar",
    "batanasteis": "batanar",
    "batane": "batanar",
    "batanea": "batanear",
    "bataneaba": "batanear",
    "bataneabais": "batanear",
    "bataneaban": "batanear",
    "bataneabas": "batanear",
    "batanead": "batanear",
    "bataneada": "batanear",
    "bataneadas": "batanear",
    "bataneado": "batanear",
    "bataneados": "batanear",
    "bataneamos": "batanear",
    "batanean": "batanear",
    "bataneando": "batanear",
    "bataneara": "batanear",
    "batanearais": "batanear",
    "batanearan": "batanear",
    "batanearas": "batanear",
    "bataneare": "batanear",
    "bataneareis": "batanear",
    "batanearemos": "batanear",
    "batanearen": "batanear",
    "bataneares": "batanear",
    "batanearon": "batanear",
    "bataneará": "batanear",
    "batanearán": "batanear",
    "batanearás": "batanear",
    "batanearé": "batanear",
    "batanearéis": "batanear",
    "batanearía": "batanear",
    "batanearíais": "batanear",
    "batanearíamos": "batanear",
    "batanearían": "batanear",
    "batanearías": "batanear",
    "bataneas": "batanear",
    "batanease": "batanear",
    "bataneaseis": "batanear",
    "bataneasen": "batanear",
    "bataneases": "batanear",
    "bataneaste": "batanear",
    "bataneasteis": "batanear",
    "batanee": "batanear",
    "bataneemos": "batanear",
    "bataneen": "batanear",
    "batanees": "batanear",
    "batanemos": "batanar",
    "batanen": "batanar",
    "bataneo": "batanear",
    "batanera": "batanero",
    "bataneras": "batanero",
    "bataneros": "batanero",
    "batanes": "batanar",
    "bataneábamos": "batanear",
    "bataneáis": "batanear",
    "bataneáramos": "batanear",
    "bataneáremos": "batanear",
    "bataneásemos": "batanear",
    "bataneé": "batanear",
    "bataneéis": "batanear",
    "bataneó": "batanear",
    "batangas": "batanga",
    "batano": "batanar",
    "batanábamos": "batanar",
    "batanáis": "batanar",
    "batanáramos": "batanar",
    "batanáremos": "batanar",
    "batanásemos": "batanar",
    "batané": "batanar",
    "batanéis": "batanar",
    "batanó": "batanar",
    "batas": "batir",
    "batatas": "batata",
    "batayolas": "batayola",
    "batazos": "batazo",
    "batches": "batch",
    "bate": "batir",
    "batea": "batear",
    "bateaba": "batear",
    "bateabais": "batear",
    "bateaban": "batear",
    "bateabas": "batear",
    "batead": "batear",
    "bateada": "batear",
    "bateadas": "batear",
    "bateado": "batear",
    "bateadora": "bateador",
    "bateadoras": "bateador",
    "bateadores": "bateador",
    "bateados": "batear",
    "bateamos": "batear",
    "batean": "batear",
    "bateando": "batear",
    "bateara": "batear",
    "batearais": "batear",
    "batearan": "batear",
    "batearas": "batear",
    "bateare": "batear",
    "bateareis": "batear",
    "batearemos": "batear",
    "batearen": "batear",
    "bateares": "batear",
    "batearon": "batear",
    "bateará": "batear",
    "batearán": "batear",
    "batearás": "batear",
    "batearé": "batear",
    "batearéis": "batear",
    "batearía": "batear",
    "batearíais": "batear",
    "batearíamos": "batear",
    "batearían": "batear",
    "batearías": "batear",
    "bateas": "batear",
    "batease": "batear",
    "bateaseis": "batear",
    "bateasen": "batear",
    "bateases": "batear",
    "bateaste": "batear",
    "bateasteis": "batear",
    "batee": "batear",
    "bateemos": "batear",
    "bateen": "batear",
    "batees": "batear",
    "batelera": "batelero",
    "bateleras": "batelero",
    "bateleros": "batelero",
    "bateles": "batel",
    "baten": "batir",
    "bateo": "batear",
    "bateos": "bateo",
    "batera": "batero",
    "bateras": "batero",
    "bateros": "batero",
    "baterías": "batería",
    "bates": "batir",
    "bateyes": "batey",
    "bateys": "batey",
    "bateábamos": "batear",
    "bateáis": "batear",
    "bateáramos": "batear",
    "bateáremos": "batear",
    "bateásemos": "batear",
    "bateé": "batear",
    "bateéis": "batear",
    "bateó": "batear",
    "batiales": "batial",
    "batiborrillos": "batiborrillo",
    "batiburrillos": "batiburrillo",
    "baticolas": "baticola",
    "baticulos": "baticulo",
    "batid": "batir",
    "batida": "batir",
    "batidas": "batir",
    "batideras": "batidera",
    "batideros": "batidero",
    "batido": "batir",
    "batidora": "batidor",
    "batidoras": "batidora",
    "batidores": "batidor",
    "batidos": "batir",
    "batiendo": "batir",
    "batientes": "batiente",
    "batiera": "batir",
    "batierais": "batir",
    "batieran": "batir",
    "batieras": "batir",
    "batiere": "batir",
    "batiereis": "batir",
    "batieren": "batir",
    "batieres": "batir",
    "batieron": "batir",
    "batiese": "batir",
    "batieseis": "batir",
    "batiesen": "batir",
    "batieses": "batir",
    "batihojas": "batihoja",
    "batiks": "batik",
    "batimanes": "batimán",
    "batimetrías": "batimetría",
    "batimientos": "batimiento",
    "batimos": "batir",
    "batimétrica": "batimétrico",
    "batimétricas": "batimétrico",
    "batimétricos": "batimétrico",
    "batines": "batín",
    "batintines": "batintín",
    "batipelágica": "batipelágico",
    "batipelágicas": "batipelágico",
    "batipelágicos": "batipelágico",
    "batiporta": "batiportar",
    "batiportaba": "batiportar",
    "batiportabais": "batiportar",
    "batiportaban": "batiportar",
    "batiportabas": "batiportar",
    "batiportad": "batiportar",
    "batiportada": "batiportar",
    "batiportadas": "batiportar",
    "batiportado": "batiportar",
    "batiportados": "batiportar",
    "batiportamos": "batiportar",
    "batiportan": "batiportar",
    "batiportando": "batiportar",
    "batiportara": "batiportar",
    "batiportarais": "batiportar",
    "batiportaran": "batiportar",
    "batiportaras": "batiportar",
    "batiportare": "batiportar",
    "batiportareis": "batiportar",
    "batiportaremos": "batiportar",
    "batiportaren": "batiportar",
    "batiportares": "batiportar",
    "batiportaron": "batiportar",
    "batiportará": "batiportar",
    "batiportarán": "batiportar",
    "batiportarás": "batiportar",
    "batiportaré": "batiportar",
    "batiportaréis": "batiportar",
    "batiportaría": "batiportar",
    "batiportaríais": "batiportar",
    "batiportaríamos": "batiportar",
    "batiportarían": "batiportar",
    "batiportarías": "batiportar",
    "batiportas": "batiportar",
    "batiportase": "batiportar",
    "batiportaseis": "batiportar",
    "batiportasen": "batiportar",
    "batiportases": "batiportar",
    "batiportaste": "batiportar",
    "batiportasteis": "batiportar",
    "batiporte": "batiportar",
    "batiportemos": "batiportar",
    "batiporten": "batiportar",
    "batiportes": "batiporte",
    "batiporto": "batiportar",
    "batiportábamos": "batiportar",
    "batiportáis": "batiportar",
    "batiportáramos": "batiportar",
    "batiportáremos": "batiportar",
    "batiportásemos": "batiportar",
    "batiporté": "batiportar",
    "batiportéis": "batiportar",
    "batiportó": "batiportar",
    "batiremos": "batir",
    "batirá": "batir",
    "batirán": "batir",
    "batirás": "batir",
    "batiré": "batir",
    "batiréis": "batir",
    "batiría": "batir",
    "batiríais": "batir",
    "batiríamos": "batir",
    "batirían": "batir",
    "batirías": "batir",
    "batiscafos": "batiscafo",
    "batisferas": "batisfera",
    "batisismos": "batisismo",
    "batistas": "batista",
    "batiste": "batir",
    "batisteis": "batir",
    "batitermógrafos": "batitermógrafo",
    "batiéramos": "batir",
    "batiéremos": "batir",
    "batiésemos": "batir",
    "batió": "batir",
    "batllismos": "batllismo",
    "bato": "batir",
    "batoja": "batojar",
    "batojaba": "batojar",
    "batojabais": "batojar",
    "batojaban": "batojar",
    "batojabas": "batojar",
    "batojad": "batojar",
    "batojada": "batojar",
    "batojadas": "batojar",
    "batojado": "batojar",
    "batojados": "batojar",
    "batojamos": "batojar",
    "batojan": "batojar",
    "batojando": "batojar",
    "batojara": "batojar",
    "batojarais": "batojar",
    "batojaran": "batojar",
    "batojaras": "batojar",
    "batojare": "batojar",
    "batojareis": "batojar",
    "batojaremos": "batojar",
    "batojaren": "batojar",
    "batojares": "batojar",
    "batojaron": "batojar",
    "batojará": "batojar",
    "batojarán": "batojar",
    "batojarás": "batojar",
    "batojaré": "batojar",
    "batojaréis": "batojar",
    "batojaría": "batojar",
    "batojaríais": "batojar",
    "batojaríamos": "batojar",
    "batojarían": "batojar",
    "batojarías": "batojar",
    "batojas": "batojar",
    "batojase": "batojar",
    "batojaseis": "batojar",
    "batojasen": "batojar",
    "batojases": "batojar",
    "batojaste": "batojar",
    "batojasteis": "batojar",
    "batoje": "batojar",
    "batojemos": "batojar",
    "batojen": "batojar",
    "batojes": "batojar",
    "batojo": "batojar",
    "batojábamos": "batojar",
    "batojáis": "batojar",
    "batojáramos": "batojar",
    "batojáremos": "batojar",
    "batojásemos": "batojar",
    "batojé": "batojar",
    "batojéis": "batojar",
    "batojó": "batojar",
    "batolitos": "batolito",
    "batologías": "batología",
    "batos": "bato",
    "batracia": "batracio",
    "batracias": "batracio",
    "batracios": "batracio",
    "batudas": "batuda",
    "batuquea": "batuquear",
    "batuqueaba": "batuquear",
    "batuqueabais": "batuquear",
    "batuqueaban": "batuquear",
    "batuqueabas": "batuquear",
    "batuquead": "batuquear",
    "batuqueada": "batuquear",
    "batuqueadas": "batuquear",
    "batuqueado": "batuquear",
    "batuqueados": "batuquear",
    "batuqueamos": "batuquear",
    "batuquean": "batuquear",
    "batuqueando": "batuquear",
    "batuqueara": "batuquear",
    "batuquearais": "batuquear",
    "batuquearan": "batuquear",
    "batuquearas": "batuquear",
    "batuqueare": "batuquear",
    "batuqueareis": "batuquear",
    "batuquearemos": "batuquear",
    "batuquearen": "batuquear",
    "batuqueares": "batuquear",
    "batuquearon": "batuquear",
    "batuqueará": "batuquear",
    "batuquearán": "batuquear",
    "batuquearás": "batuquear",
    "batuquearé": "batuquear",
    "batuquearéis": "batuquear",
    "batuquearía": "batuquear",
    "batuquearíais": "batuquear",
    "batuquearíamos": "batuquear",
    "batuquearían": "batuquear",
    "batuquearías": "batuquear",
    "batuqueas": "batuquear",
    "batuquease": "batuquear",
    "batuqueaseis": "batuquear",
    "batuqueasen": "batuquear",
    "batuqueases": "batuquear",
    "batuqueaste": "batuquear",
    "batuqueasteis": "batuquear",
    "batuquee": "batuquear",
    "batuqueemos": "batuquear",
    "batuqueen": "batuquear",
    "batuquees": "batuquear",
    "batuqueo": "batuquear",
    "batuqueábamos": "batuquear",
    "batuqueáis": "batuquear",
    "batuqueáramos": "batuquear",
    "batuqueáremos": "batuquear",
    "batuqueásemos": "batuquear",
    "batuqueé": "batuquear",
    "batuqueéis": "batuquear",
    "batuqueó": "batuquear",
    "baturra": "baturro",
    "baturradas": "baturrada",
    "baturras": "baturro",
    "baturrillos": "baturrillo",
    "baturros": "baturro",
    "batutas": "batuta",
    "batáis": "batir",
    "batí": "batir",
    "batía": "batir",
    "batíais": "batir",
    "batíamos": "batir",
    "batían": "batir",
    "batías": "batir",
    "batímetros": "batímetro",
    "batís": "batir",
    "batómetros": "batómetro",
    "baudes": "baud",
    "baudios": "baudio",
    "bauds": "baud",
    "baulera": "baulero",
    "bauleras": "baulero",
    "bauleros": "baulero",
    "baupreses": "bauprés",
    "bausana": "bausán",
    "bausanas": "bausán",
    "bausanes": "bausán",
    "bautice": "bautizar",
    "bauticemos": "bautizar",
    "bauticen": "bautizar",
    "bautices": "bautizar",
    "bauticé": "bautizar",
    "bauticéis": "bautizar",
    "bautismales": "bautismal",
    "bautismos": "bautismo",
    "bautistas": "bautista",
    "bautiza": "bautizar",
    "bautizaba": "bautizar",
    "bautizabais": "bautizar",
    "bautizaban": "bautizar",
    "bautizabas": "bautizar",
    "bautizad": "bautizar",
    "bautizada": "bautizar",
    "bautizadas": "bautizar",
    "bautizado": "bautizar",
    "bautizados": "bautizar",
    "bautizamos": "bautizar",
    "bautizan": "bautizar",
    "bautizando": "bautizar",
    "bautizara": "bautizar",
    "bautizarais": "bautizar",
    "bautizaran": "bautizar",
    "bautizaras": "bautizar",
    "bautizare": "bautizar",
    "bautizareis": "bautizar",
    "bautizaremos": "bautizar",
    "bautizaren": "bautizar",
    "bautizares": "bautizar",
    "bautizaron": "bautizar",
    "bautizará": "bautizar",
    "bautizarán": "bautizar",
    "bautizarás": "bautizar",
    "bautizaré": "bautizar",
    "bautizaréis": "bautizar",
    "bautizaría": "bautizar",
    "bautizaríais": "bautizar",
    "bautizaríamos": "bautizar",
    "bautizarían": "bautizar",
    "bautizarías": "bautizar",
    "bautizas": "bautizar",
    "bautizase": "bautizar",
    "bautizaseis": "bautizar",
    "bautizasen": "bautizar",
    "bautizases": "bautizar",
    "bautizaste": "bautizar",
    "bautizasteis": "bautizar",
    "bautizo": "bautizar",
    "bautizos": "bautizo",
    "bautizábamos": "bautizar",
    "bautizáis": "bautizar",
    "bautizáramos": "bautizar",
    "bautizáremos": "bautizar",
    "bautizásemos": "bautizar",
    "bautizó": "bautizar",
    "bauxitas": "bauxita",
    "bauzas": "bauza",
    "baya": "bayo",
    "bayaderas": "bayadera",
    "bayales": "bayal",
    "bayanismos": "bayanismo",
    "bayartes": "bayarte",
    "bayas": "bayo",
    "bayesiana": "bayesiano",
    "bayesianas": "bayesiano",
    "bayesianos": "bayesiano",
    "bayetas": "bayeta",
    "bayetones": "bayetón",
    "bayonetas": "bayoneta",
    "bayonetazos": "bayonetazo",
    "bayos": "bayo",
    "baza": "bazo",
    "bazares": "bazar",
    "bazas": "bazo",
    "bazofias": "bazofia",
    "bazookas": "bazooka",
    "bazos": "bazo",
    "bazuca": "bazucar",
    "bazucaba": "bazucar",
    "bazucabais": "bazucar",
    "bazucaban": "bazucar",
    "bazucabas": "bazucar",
    "bazucad": "bazucar",
    "bazucada": "bazucar",
    "bazucadas": "bazucar",
    "bazucado": "bazucar",
    "bazucados": "bazucar",
    "bazucamos": "bazucar",
    "bazucan": "bazucar",
    "bazucando": "bazucar",
    "bazucara": "bazucar",
    "bazucarais": "bazucar",
    "bazucaran": "bazucar",
    "bazucaras": "bazucar",
    "bazucare": "bazucar",
    "bazucareis": "bazucar",
    "bazucaremos": "bazucar",
    "bazucaren": "bazucar",
    "bazucares": "bazucar",
    "bazucaron": "bazucar",
    "bazucará": "bazucar",
    "bazucarán": "bazucar",
    "bazucarás": "bazucar",
    "bazucaré": "bazucar",
    "bazucaréis": "bazucar",
    "bazucaría": "bazucar",
    "bazucaríais": "bazucar",
    "bazucaríamos": "bazucar",
    "bazucarían": "bazucar",
    "bazucarías": "bazucar",
    "bazucas": "bazucar",
    "bazucase": "bazucar",
    "bazucaseis": "bazucar",
    "bazucasen": "bazucar",
    "bazucases": "bazucar",
    "bazucaste": "bazucar",
    "bazucasteis": "bazucar",
    "bazuco": "bazucar",
    "bazucábamos": "bazucar",
    "bazucáis": "bazucar",
    "bazucáramos": "bazucar",
    "bazucáremos": "bazucar",
    "bazucásemos": "bazucar",
    "bazucó": "bazucar",
    "bazuque": "bazucar",
    "bazuquea": "bazuquear",
    "bazuqueaba": "bazuquear",
    "bazuqueabais": "bazuquear",
    "bazuqueaban": "bazuquear",
    "bazuqueabas": "bazuquear",
    "bazuquead": "bazuquear",
    "bazuqueada": "bazuquear",
    "bazuqueadas": "bazuquear",
    "bazuqueado": "bazuquear",
    "bazuqueados": "bazuquear",
    "bazuqueamos": "bazuquear",
    "bazuquean": "bazuquear",
    "bazuqueando": "bazuquear",
    "bazuqueara": "bazuquear",
    "bazuquearais": "bazuquear",
    "bazuquearan": "bazuquear",
    "bazuquearas": "bazuquear",
    "bazuqueare": "bazuquear",
    "bazuqueareis": "bazuquear",
    "bazuquearemos": "bazuquear",
    "bazuquearen": "bazuquear",
    "bazuqueares": "bazuquear",
    "bazuquearon": "bazuquear",
    "bazuqueará": "bazuquear",
    "bazuquearán": "bazuquear",
    "bazuquearás": "bazuquear",
    "bazuquearé": "bazuquear",
    "bazuquearéis": "bazuquear",
    "bazuquearía": "bazuquear",
    "bazuquearíais": "bazuquear",
    "bazuquearíamos": "bazuquear",
    "bazuquearían": "bazuquear",
    "bazuquearías": "bazuquear",
    "bazuqueas": "bazuquear",
    "bazuquease": "bazuquear",
    "bazuqueaseis": "bazuquear",
    "bazuqueasen": "bazuquear",
    "bazuqueases": "bazuquear",
    "bazuqueaste": "bazuquear",
    "bazuqueasteis": "bazuquear",
    "bazuquee": "bazuquear",
    "bazuqueemos": "bazuquear",
    "bazuqueen": "bazuquear",
    "bazuquees": "bazuquear",
    "bazuquemos": "bazucar",
    "bazuquen": "bazucar",
    "bazuqueo": "bazuquear",
    "bazuques": "bazucar",
    "bazuqueábamos": "bazuquear",
    "bazuqueáis": "bazuquear",
    "bazuqueáramos": "bazuquear",
    "bazuqueáremos": "bazuquear",
    "bazuqueásemos": "bazuquear",
    "bazuqueé": "bazuquear",
    "bazuqueéis": "bazuquear",
    "bazuqueó": "bazuquear",
    "bazuqué": "bazucar",
    "bazuquéis": "bazucar",
    "baña": "bañar",
    "bañaba": "bañar",
    "bañabais": "bañar",
    "bañaban": "bañar",
    "bañabas": "bañar",
    "bañad": "bañar",
    "bañada": "bañar",
    "bañadas": "bañar",
    "bañaderos": "bañadero",
    "bañado": "bañar",
    "bañadora": "bañador",
    "bañadoras": "bañador",
    "bañadores": "bañador",
    "bañados": "bañar",
    "bañamos": "bañar",
    "bañan": "bañar",
    "bañando": "bañar",
    "bañara": "bañar",
    "bañarais": "bañar",
    "bañaran": "bañar",
    "bañaras": "bañar",
    "bañare": "bañar",
    "bañareis": "bañar",
    "bañaremos": "bañar",
    "bañaren": "bañar",
    "bañares": "bañar",
    "bañaron": "bañar",
    "bañará": "bañar",
    "bañarán": "bañar",
    "bañarás": "bañar",
    "bañaré": "bañar",
    "bañaréis": "bañar",
    "bañaría": "bañar",
    "bañaríais": "bañar",
    "bañaríamos": "bañar",
    "bañarían": "bañar",
    "bañarías": "bañar",
    "bañas": "bañar",
    "bañase": "bañar",
    "bañaseis": "bañar",
    "bañasen": "bañar",
    "bañases": "bañar",
    "bañaste": "bañar",
    "bañasteis": "bañar",
    "bañe": "bañar",
    "bañemos": "bañar",
    "bañen": "bañar",
    "bañeras": "bañera",
    "bañeros": "bañero",
    "bañes": "bañar",
    "bañistas": "bañista",
    "baño": "bañar",
    "baños": "baño",
    "bañábamos": "bañar",
    "bañáis": "bañar",
    "bañáramos": "bañar",
    "bañáremos": "bañar",
    "bañásemos": "bañar",
    "bañé": "bañar",
    "bañéis": "bañar",
    "bañó": "bañar",
    "baúles": "baúl",
    "beamontesa": "beamontés",
    "beamontesas": "beamontés",
    "beamonteses": "beamontés",
    "bearnesa": "bearneso",
    "bearnesas": "bearneso",
    "bearnesos": "bearneso",
    "beata": "beato",
    "beatas": "beato",
    "beaterios": "beaterio",
    "beatifica": "beatificar",
    "beatificaba": "beatificar",
    "beatificabais": "beatificar",
    "beatificaban": "beatificar",
    "beatificabas": "beatificar",
    "beatificaciones": "beatificación",
    "beatificad": "beatificar",
    "beatificada": "beatificar",
    "beatificadas": "beatificar",
    "beatificado": "beatificar",
    "beatificados": "beatificar",
    "beatificamos": "beatificar",
    "beatifican": "beatificar",
    "beatificando": "beatificar",
    "beatificara": "beatificar",
    "beatificarais": "beatificar",
    "beatificaran": "beatificar",
    "beatificaras": "beatificar",
    "beatificare": "beatificar",
    "beatificareis": "beatificar",
    "beatificaremos": "beatificar",
    "beatificaren": "beatificar",
    "beatificares": "beatificar",
    "beatificaron": "beatificar",
    "beatificará": "beatificar",
    "beatificarán": "beatificar",
    "beatificarás": "beatificar",
    "beatificaré": "beatificar",
    "beatificaréis": "beatificar",
    "beatificaría": "beatificar",
    "beatificaríais": "beatificar",
    "beatificaríamos": "beatificar",
    "beatificarían": "beatificar",
    "beatificarías": "beatificar",
    "beatificas": "beatificar",
    "beatificase": "beatificar",
    "beatificaseis": "beatificar",
    "beatificasen": "beatificar",
    "beatificases": "beatificar",
    "beatificaste": "beatificar",
    "beatificasteis": "beatificar",
    "beatifico": "beatificar",
    "beatificábamos": "beatificar",
    "beatificáis": "beatificar",
    "beatificáramos": "beatificar",
    "beatificáremos": "beatificar",
    "beatificásemos": "beatificar",
    "beatificó": "beatificar",
    "beatifique": "beatificar",
    "beatifiquemos": "beatificar",
    "beatifiquen": "beatificar",
    "beatifiques": "beatificar",
    "beatifiqué": "beatificar",
    "beatifiquéis": "beatificar",
    "beatitudes": "beatitud",
    "beatnikes": "beatnik",
    "beatniks": "beatnik",
    "beatos": "beato",
    "beatífica": "beatífico",
    "beatíficas": "beatífico",
    "beatíficos": "beatífico",
    "beaumontesa": "beaumontés",
    "beaumontesas": "beaumontés",
    "beaumonteses": "beaumontés",
    "beba": "beber",
    "bebamos": "beber",
    "beban": "beber",
    "bebas": "beber",
    "bebe": "beber",
    "bebed": "beber",
    "bebederos": "bebedero",
    "bebediza": "bebedizo",
    "bebedizas": "bebedizo",
    "bebedizos": "bebedizo",
    "bebedora": "bebedor",
    "bebedoras": "bebedor",
    "bebedores": "bebedor",
    "bebemos": "beber",
    "beben": "beber",
    "beberemos": "beber",
    "beberá": "beber",
    "beberán": "beber",
    "beberás": "beber",
    "beberé": "beber",
    "beberéis": "beber",
    "bebería": "beber",
    "beberíais": "beber",
    "beberíamos": "beber",
    "beberían": "beber",
    "beberías": "beber",
    "bebes": "beber",
    "bebestibles": "bebestible",
    "bebibles": "bebible",
    "bebida": "beber",
    "bebidas": "bebida",
    "bebido": "beber",
    "bebidos": "beber",
    "bebiendo": "beber",
    "bebiera": "beber",
    "bebierais": "beber",
    "bebieran": "beber",
    "bebieras": "beber",
    "bebiere": "beber",
    "bebiereis": "beber",
    "bebieren": "beber",
    "bebieres": "beber",
    "bebieron": "beber",
    "bebiese": "beber",
    "bebieseis": "beber",
    "bebiesen": "beber",
    "bebieses": "beber",
    "bebimos": "beber",
    "bebiste": "beber",
    "bebisteis": "beber",
    "bebistrajos": "bebistrajo",
    "bebiéramos": "beber",
    "bebiéremos": "beber",
    "bebiésemos": "beber",
    "bebió": "beber",
    "bebo": "beber",
    "beborrotea": "beborrotear",
    "beborroteaba": "beborrotear",
    "beborroteabais": "beborrotear",
    "beborroteaban": "beborrotear",
    "beborroteabas": "beborrotear",
    "beborrotead": "beborrotear",
    "beborroteada": "beborrotear",
    "beborroteadas": "beborrotear",
    "beborroteado": "beborrotear",
    "beborroteados": "beborrotear",
    "beborroteamos": "beborrotear",
    "beborrotean": "beborrotear",
    "beborroteando": "beborrotear",
    "beborroteara": "beborrotear",
    "beborrotearais": "beborrotear",
    "beborrotearan": "beborrotear",
    "beborrotearas": "beborrotear",
    "beborroteare": "beborrotear",
    "beborroteareis": "beborrotear",
    "beborrotearemos": "beborrotear",
    "beborrotearen": "beborrotear",
    "beborroteares": "beborrotear",
    "beborrotearon": "beborrotear",
    "beborroteará": "beborrotear",
    "beborrotearán": "beborrotear",
    "beborrotearás": "beborrotear",
    "beborrotearé": "beborrotear",
    "beborrotearéis": "beborrotear",
    "beborrotearía": "beborrotear",
    "beborrotearíais": "beborrotear",
    "beborrotearíamos": "beborrotear",
    "beborrotearían": "beborrotear",
    "beborrotearías": "beborrotear",
    "beborroteas": "beborrotear",
    "beborrotease": "beborrotear",
    "beborroteaseis": "beborrotear",
    "beborroteasen": "beborrotear",
    "beborroteases": "beborrotear",
    "beborroteaste": "beborrotear",
    "beborroteasteis": "beborrotear",
    "beborrotee": "beborrotear",
    "beborroteemos": "beborrotear",
    "beborroteen": "beborrotear",
    "beborrotees": "beborrotear",
    "beborroteo": "beborrotear",
    "beborroteábamos": "beborrotear",
    "beborroteáis": "beborrotear",
    "beborroteáramos": "beborrotear",
    "beborroteáremos": "beborrotear",
    "beborroteásemos": "beborrotear",
    "beborroteé": "beborrotear",
    "beborroteéis": "beborrotear",
    "beborroteó": "beborrotear",
    "bebáis": "beber",
    "bebéis": "beber",
    "bebés": "bebé",
    "bebí": "beber",
    "bebía": "beber",
    "bebíais": "beber",
    "bebíamos": "beber",
    "bebían": "beber",
    "bebías": "beber",
    "beca": "becar",
    "becaba": "becar",
    "becabais": "becar",
    "becaban": "becar",
    "becabas": "becar",
    "becad": "becar",
    "becada": "becar",
    "becadas": "becar",
    "becado": "becar",
    "becados": "becar",
    "becamos": "becar",
    "becan": "becar",
    "becando": "becar",
    "becara": "becar",
    "becarais": "becar",
    "becaran": "becar",
    "becaras": "becar",
    "becare": "becar",
    "becareis": "becar",
    "becaremos": "becar",
    "becaren": "becar",
    "becares": "becar",
    "becaria": "becario",
    "becarias": "becario",
    "becarios": "becario",
    "becaron": "becar",
    "becará": "becar",
    "becarán": "becar",
    "becarás": "becar",
    "becaré": "becar",
    "becaréis": "becar",
    "becaría": "becar",
    "becaríais": "becar",
    "becaríamos": "becar",
    "becarían": "becar",
    "becarías": "becar",
    "becas": "becar",
    "becase": "becar",
    "becaseis": "becar",
    "becasen": "becar",
    "becases": "becar",
    "becaste": "becar",
    "becasteis": "becar",
    "becerra": "becerro",
    "becerradas": "becerrada",
    "becerras": "becerro",
    "becerristas": "becerrista",
    "becerros": "becerro",
    "bechamelas": "bechamela",
    "bechameles": "bechamel",
    "beco": "becar",
    "becquereles": "becquerel",
    "becquerels": "becquerel",
    "becuadrados": "becuadrado",
    "becuadros": "becuadro",
    "becábamos": "becar",
    "becáis": "becar",
    "becáramos": "becar",
    "becáremos": "becar",
    "becásemos": "becar",
    "becó": "becar",
    "bedanos": "bedano",
    "bedeles": "bedel",
    "bedelías": "bedelía",
    "beduina": "beduino",
    "beduinas": "beduino",
    "beduinos": "beduino",
    "befa": "befo",
    "befaba": "befar",
    "befabais": "befar",
    "befaban": "befar",
    "befabas": "befar",
    "befabemíes": "befabemí",
    "befabemís": "befabemí",
    "befad": "befar",
    "befada": "befar",
    "befadas": "befar",
    "befado": "befar",
    "befados": "befar",
    "befamos": "befar",
    "befan": "befar",
    "befando": "befar",
    "befara": "befar",
    "befarais": "befar",
    "befaran": "befar",
    "befaras": "befar",
    "befare": "befar",
    "befareis": "befar",
    "befaremos": "befar",
    "befaren": "befar",
    "befares": "befar",
    "befaron": "befar",
    "befará": "befar",
    "befarán": "befar",
    "befarás": "befar",
    "befaré": "befar",
    "befaréis": "befar",
    "befaría": "befar",
    "befaríais": "befar",
    "befaríamos": "befar",
    "befarían": "befar",
    "befarías": "befar",
    "befas": "befo",
    "befase": "befar",
    "befaseis": "befar",
    "befasen": "befar",
    "befases": "befar",
    "befaste": "befar",
    "befasteis": "befar",
    "befe": "befar",
    "befemos": "befar",
    "befen": "befar",
    "befes": "befar",
    "beffrois": "beffroi",
    "befo": "befar",
    "befos": "befo",
    "befábamos": "befar",
    "befáis": "befar",
    "befáramos": "befar",
    "befáremos": "befar",
    "befásemos": "befar",
    "befé": "befar",
    "beféis": "befar",
    "befó": "befar",
    "begarda": "begardo",
    "begardas": "begardo",
    "begardos": "begardo",
    "begonias": "begonia",
    "begoniáceas": "begoniácea",
    "beguina": "beguino",
    "beguinajes": "beguinaje",
    "beguinas": "beguino",
    "beguinos": "beguino",
    "behaviorismos": "behaviorismo",
    "behavioristas": "behaviorista",
    "behaísmos": "behaísmo",
    "behenatos": "behenato",
    "behetrías": "behetría",
    "behénica": "behénico",
    "behénicas": "behénico",
    "behénicos": "behénico",
    "beiges": "beige",
    "beisboles": "beisbol",
    "beisbolistas": "beisbolista",
    "bejeles": "bejel",
    "bejucos": "bejuco",
    "bejuquea": "bejuquear",
    "bejuqueaba": "bejuquear",
    "bejuqueabais": "bejuquear",
    "bejuqueaban": "bejuquear",
    "bejuqueabas": "bejuquear",
    "bejuquead": "bejuquear",
    "bejuqueada": "bejuquear",
    "bejuqueadas": "bejuquear",
    "bejuqueado": "bejuquear",
    "bejuqueados": "bejuquear",
    "bejuqueamos": "bejuquear",
    "bejuquean": "bejuquear",
    "bejuqueando": "bejuquear",
    "bejuqueara": "bejuquear",
    "bejuquearais": "bejuquear",
    "bejuquearan": "bejuquear",
    "bejuquearas": "bejuquear",
    "bejuqueare": "bejuquear",
    "bejuqueareis": "bejuquear",
    "bejuquearemos": "bejuquear",
    "bejuquearen": "bejuquear",
    "bejuqueares": "bejuquear",
    "bejuquearon": "bejuquear",
    "bejuqueará": "bejuquear",
    "bejuquearán": "bejuquear",
    "bejuquearás": "bejuquear",
    "bejuquearé": "bejuquear",
    "bejuquearéis": "bejuquear",
    "bejuquearía": "bejuquear",
    "bejuquearíais": "bejuquear",
    "bejuquearíamos": "bejuquear",
    "bejuquearían": "bejuquear",
    "bejuquearías": "bejuquear",
    "bejuqueas": "bejuquear",
    "bejuquease": "bejuquear",
    "bejuqueaseis": "bejuquear",
    "bejuqueasen": "bejuquear",
    "bejuqueases": "bejuquear",
    "bejuqueaste": "bejuquear",
    "bejuqueasteis": "bejuquear",
    "bejuquee": "bejuquear",
    "bejuqueemos": "bejuquear",
    "bejuqueen": "bejuquear",
    "bejuquees": "bejuquear",
    "bejuqueo": "bejuquear",
    "bejuqueábamos": "bejuquear",
    "bejuqueáis": "bejuquear",
    "bejuqueáramos": "bejuquear",
    "bejuqueáremos": "bejuquear",
    "bejuqueásemos": "bejuquear",
    "bejuqueé": "bejuquear",
    "bejuqueéis": "bejuquear",
    "bejuqueó": "bejuquear",
    "belcantistas": "belcantista",
    "belcebúes": "belcebú",
    "belcebús": "belcebú",
    "beldaba": "beldar",
    "beldabais": "beldar",
    "beldaban": "beldar",
    "beldabas": "beldar",
    "beldad": "beldar",
    "beldada": "beldar",
    "beldadas": "beldar",
    "beldades": "beldad",
    "beldado": "beldar",
    "beldados": "beldar",
    "beldamos": "beldar",
    "beldando": "beldar",
    "beldara": "beldar",
    "beldarais": "beldar",
    "beldaran": "beldar",
    "beldaras": "beldar",
    "beldare": "beldar",
    "beldareis": "beldar",
    "beldaremos": "beldar",
    "beldaren": "beldar",
    "beldares": "beldar",
    "beldaron": "beldar",
    "beldará": "beldar",
    "beldarán": "beldar",
    "beldarás": "beldar",
    "beldaré": "beldar",
    "beldaréis": "beldar",
    "beldaría": "beldar",
    "beldaríais": "beldar",
    "beldaríamos": "beldar",
    "beldarían": "beldar",
    "beldarías": "beldar",
    "beldase": "beldar",
    "beldaseis": "beldar",
    "beldasen": "beldar",
    "beldases": "beldar",
    "beldaste": "beldar",
    "beldasteis": "beldar",
    "beldemos": "beldar",
    "beldábamos": "beldar",
    "beldáis": "beldar",
    "beldáramos": "beldar",
    "beldáremos": "beldar",
    "beldásemos": "beldar",
    "beldé": "beldar",
    "beldéis": "beldar",
    "beldó": "beldar",
    "beleces": "belez",
    "belemnitas": "belemnita",
    "belenes": "belén",
    "beleños": "beleño",
    "belfa": "belfo",
    "belfas": "belfo",
    "belfos": "belfo",
    "belgas": "belga",
    "belgoluxemburguesa": "belgoluxemburgués",
    "belgoluxemburguesas": "belgoluxemburgués",
    "belgoluxemburgueses": "belgoluxemburgués",
    "beliceña": "beliceño",
    "beliceñas": "beliceño",
    "beliceños": "beliceño",
    "belicismos": "belicismo",
    "belicistas": "belicista",
    "belicosa": "belicoso",
    "belicosas": "belicoso",
    "belicosidades": "belicosidad",
    "belicosos": "belicoso",
    "beligerancias": "beligerancia",
    "beligerantes": "beligerante",
    "belinógrafos": "belinógrafo",
    "belitres": "belitre",
    "bella": "bello",
    "bellaca": "bellaco",
    "bellacas": "bellaco",
    "bellacos": "bellaco",
    "belladonas": "belladona",
    "bellaquea": "bellaquear",
    "bellaqueaba": "bellaquear",
    "bellaqueabais": "bellaquear",
    "bellaqueaban": "bellaquear",
    "bellaqueabas": "bellaquear",
    "bellaquead": "bellaquear",
    "bellaqueada": "bellaquear",
    "bellaqueadas": "bellaquear",
    "bellaqueado": "bellaquear",
    "bellaqueados": "bellaquear",
    "bellaqueamos": "bellaquear",
    "bellaquean": "bellaquear",
    "bellaqueando": "bellaquear",
    "bellaqueara": "bellaquear",
    "bellaquearais": "bellaquear",
    "bellaquearan": "bellaquear",
    "bellaquearas": "bellaquear",
    "bellaqueare": "bellaquear",
    "bellaqueareis": "bellaquear",
    "bellaquearemos": "bellaquear",
    "bellaquearen": "bellaquear",
    "bellaqueares": "bellaquear",
    "bellaquearon": "bellaquear",
    "bellaqueará": "bellaquear",
    "bellaquearán": "bellaquear",
    "bellaquearás": "bellaquear",
    "bellaquearé": "bellaquear",
    "bellaquearéis": "bellaquear",
    "bellaquearía": "bellaquear",
    "bellaquearíais": "bellaquear",
    "bellaquearíamos": "bellaquear",
    "bellaquearían": "bellaquear",
    "bellaquearías": "bellaquear",
    "bellaqueas": "bellaquear",
    "bellaquease": "bellaquear",
    "bellaqueaseis": "bellaquear",
    "bellaqueasen": "bellaquear",
    "bellaqueases": "bellaquear",
    "bellaqueaste": "bellaquear",
    "bellaqueasteis": "bellaquear",
    "bellaquee": "bellaquear",
    "bellaqueemos": "bellaquear",
    "bellaqueen": "bellaquear",
    "bellaquees": "bellaquear",
    "bellaqueo": "bellaquear",
    "bellaquerías": "bellaquería",
    "bellaqueábamos": "bellaquear",
    "bellaqueáis": "bellaquear",
    "bellaqueáramos": "bellaquear",
    "bellaqueáremos": "bellaquear",
    "bellaqueásemos": "bellaquear",
    "bellaqueé": "bellaquear",
    "bellaqueéis": "bellaquear",
    "bellaqueó": "bellaquear",
    "bellas": "bello",
    "bellezas": "belleza",
    "bellos": "bello",
    "bellotas": "bellota",
    "bellotera": "bellotero",
    "belloteras": "bellotero",
    "belloteros": "bellotero",
    "bellotes": "bellote",
    "belortas": "belorta",
    "beluchis": "beluchi",
    "belísona": "belísono",
    "belísonas": "belísono",
    "belísonos": "belísono",
    "bemoles": "bemol",
    "bemolizaciones": "bemolización",
    "bemolizaciónes": "bemolización",
    "bemolizada": "bemolizado",
    "bemolizadas": "bemolizado",
    "bemolizados": "bemolizado",
    "bencenofosfónica": "bencenofosfónico",
    "bencenofosfónicas": "bencenofosfónico",
    "bencenofosfónicos": "bencenofosfónico",
    "bencenos": "benceno",
    "bencenosulfonbutilamidas": "bencenosulfonbutilamida",
    "bencenosulfónica": "bencenosulfónico",
    "bencenosulfónicas": "bencenosulfónico",
    "bencenosulfónicos": "bencenosulfónico",
    "bencidinas": "bencidina",
    "bencilanilinas": "bencilanilina",
    "bencinas": "bencina",
    "bencénica": "bencénico",
    "bencénicas": "bencénico",
    "bencénicos": "bencénico",
    "bendecid": "bendecir",
    "bendecimos": "bendecir",
    "bendeciremos": "bendecir",
    "bendecirá": "bendecir",
    "bendecirán": "bendecir",
    "bendecirás": "bendecir",
    "bendeciré": "bendecir",
    "bendeciréis": "bendecir",
    "bendeciría": "bendecir",
    "bendeciríais": "bendecir",
    "bendeciríamos": "bendecir",
    "bendecirían": "bendecir",
    "bendecirías": "bendecir",
    "bendecía": "bendecir",
    "bendecíais": "bendecir",
    "bendecíamos": "bendecir",
    "bendecían": "bendecir",
    "bendecías": "bendecir",
    "bendecís": "bendecir",
    "bendice": "bendecir",
    "bendicen": "bendecir",
    "bendices": "bendecir",
    "bendiciendo": "bendecir",
    "bendiciones": "bendición",
    "bendiga": "bendecir",
    "bendigamos": "bendecir",
    "bendigan": "bendecir",
    "bendigas": "bendecir",
    "bendigo": "bendecir",
    "bendigáis": "bendecir",
    "bendije": "bendecir",
    "bendijera": "bendecir",
    "bendijerais": "bendecir",
    "bendijeran": "bendecir",
    "bendijeras": "bendecir",
    "bendijere": "bendecir",
    "bendijereis": "bendecir",
    "bendijeren": "bendecir",
    "bendijeres": "bendecir",
    "bendijeron": "bendecir",
    "bendijese": "bendecir",
    "bendijeseis": "bendecir",
    "bendijesen": "bendecir",
    "bendijeses": "bendecir",
    "bendijimos": "bendecir",
    "bendijiste": "bendecir",
    "bendijisteis": "bendecir",
    "bendijo": "bendecir",
    "bendijéramos": "bendecir",
    "bendijéremos": "bendecir",
    "bendijésemos": "bendecir",
    "bendita": "bendecir",
    "benditas": "bendecir",
    "bendito": "bendecir",
    "benditos": "bendecir",
    "benedictina": "benedictino",
    "benedictinas": "benedictino",
    "benedictinos": "benedictino",
    "benedícites": "benedícite",
    "benefactora": "benefactor",
    "benefactoras": "benefactor",
    "benefactores": "benefactor",
    "beneficencias": "beneficencia",
    "beneficia": "beneficiar",
    "beneficiaba": "beneficiar",
    "beneficiabais": "beneficiar",
    "beneficiaban": "beneficiar",
    "beneficiabas": "beneficiar",
    "beneficiad": "beneficiar",
    "beneficiada": "beneficiar",
    "beneficiadas": "beneficiar",
    "beneficiado": "beneficiar",
    "beneficiadora": "beneficiador",
    "beneficiadoras": "beneficiador",
    "beneficiadores": "beneficiador",
    "beneficiados": "beneficiar",
    "beneficiamos": "beneficiar",
    "benefician": "beneficiar",
    "beneficiando": "beneficiar",
    "beneficiara": "beneficiar",
    "beneficiarais": "beneficiar",
    "beneficiaran": "beneficiar",
    "beneficiaras": "beneficiar",
    "beneficiare": "beneficiar",
    "beneficiareis": "beneficiar",
    "beneficiaremos": "beneficiar",
    "beneficiaren": "beneficiar",
    "beneficiares": "beneficiar",
    "beneficiaria": "beneficiario",
    "beneficiarias": "beneficiario",
    "beneficiarios": "beneficiario",
    "beneficiaron": "beneficiar",
    "beneficiará": "beneficiar",
    "beneficiarán": "beneficiar",
    "beneficiarás": "beneficiar",
    "beneficiaré": "beneficiar",
    "beneficiaréis": "beneficiar",
    "beneficiaría": "beneficiar",
    "beneficiaríais": "beneficiar",
    "beneficiaríamos": "beneficiar",
    "beneficiarían": "beneficiar",
    "beneficiarías": "beneficiar",
    "beneficias": "beneficiar",
    "beneficiase": "beneficiar",
    "beneficiaseis": "beneficiar",
    "beneficiasen": "beneficiar",
    "beneficiases": "beneficiar",
    "beneficiaste": "beneficiar",
    "beneficiasteis": "beneficiar",
    "beneficie": "beneficiar",
    "beneficiemos": "beneficiar",
    "beneficien": "beneficiar",
    "beneficies": "beneficiar",
    "beneficio": "beneficiar",
    "beneficios": "beneficio",
    "beneficiosa": "beneficioso",
    "beneficiosas": "beneficioso",
    "beneficiosos": "beneficioso",
    "beneficiábamos": "beneficiar",
    "beneficiáis": "beneficiar",
    "beneficiáramos": "beneficiar",
    "beneficiáremos": "beneficiar",
    "beneficiásemos": "beneficiar",
    "beneficié": "beneficiar",
    "beneficiéis": "beneficiar",
    "benefició": "beneficiar",
    "benemérita": "benemérito",
    "beneméritas": "benemérito",
    "beneméritos": "benemérito",
    "beneplácitos": "beneplácito",
    "benes": "ben",
    "benevolencias": "benevolencia",
    "benevolentes": "benevolente",
    "bengalas": "bengala",
    "bengalises": "bengalí",
    "bengalíes": "bengalí",
    "bengalís": "bengalí",
    "bengalíses": "bengalí",
    "benigna": "benigno",
    "benignas": "benigno",
    "benignidades": "benignidad",
    "benignos": "benigno",
    "benimerines": "benimerín",
    "beninesa": "beninés",
    "beninesas": "beninés",
    "benineses": "beninés",
    "benita": "benito",
    "benitas": "benito",
    "benitos": "benito",
    "benjamina": "benjamín",
    "benjaminas": "benjamín",
    "benjamines": "benjamín",
    "benjamitas": "benjamita",
    "benjuíes": "benjuí",
    "benjuís": "benjuí",
    "bentonitas": "bentonita",
    "bentónica": "bentónico",
    "bentónicas": "bentónico",
    "bentónicos": "bentónico",
    "benzalconios": "benzalconio",
    "benzaldehídos": "benzaldehído",
    "benzamidas": "benzamida",
    "benzaminas": "benzamina",
    "benzanilidas": "benzanilida",
    "benzedrinas": "benzedrina",
    "benzoacetatos": "benzoacetato",
    "benzoatos": "benzoato",
    "benzocaínas": "benzocaína",
    "benzodiacepinas": "benzodiacepina",
    "benzofenonas": "benzofenona",
    "benzoguaniminas": "benzoguanimina",
    "benzoica": "benzoico",
    "benzoicas": "benzoico",
    "benzoicos": "benzoico",
    "benzoilbenzoatos": "benzoilbenzoato",
    "benzoles": "benzol",
    "benzonitrilos": "benzonitrilo",
    "benzopurpurinas": "benzopurpurina",
    "benzotricloruros": "benzotricloruro",
    "benzotrifluoruros": "benzotrifluoruro",
    "benéfica": "benéfico",
    "benéficas": "benéfico",
    "benéficos": "benéfico",
    "benévola": "benévolo",
    "benévolas": "benévolo",
    "benévolos": "benévolo",
    "beocia": "beocio",
    "beocias": "beocio",
    "beocios": "beocio",
    "beoda": "beodo",
    "beodas": "beodo",
    "beodeces": "beodez",
    "beodos": "beodo",
    "beoríes": "beorí",
    "beorís": "beorí",
    "beque": "becar",
    "bequemos": "becar",
    "bequen": "becar",
    "beques": "beque",
    "bequé": "becar",
    "bequéis": "becar",
    "berberaminas": "berberamina",
    "berberechos": "berberecho",
    "berberisca": "berberisco",
    "berberiscas": "berberisco",
    "berberiscos": "berberisco",
    "berberíes": "berberí",
    "berberís": "berberí",
    "berbiquíes": "berbiquí",
    "berbiquís": "berbiquí",
    "berceuses": "berceuse",
    "bereberes": "bereber",
    "berenjenales": "berenjenal",
    "berenjenas": "berenjena",
    "bergamotas": "bergamota",
    "bergamotos": "bergamoto",
    "bergantes": "bergante",
    "bergantines": "bergantín",
    "beriberis": "beriberi",
    "berilios": "berilio",
    "berilonitas": "berilonita",
    "berilos": "berilo",
    "berkelios": "berkelio",
    "berlangas": "berlanga",
    "berlinas": "berlina",
    "berlinesa": "berlinés",
    "berlinesas": "berlinés",
    "berlineses": "berlinés",
    "berlingas": "berlinga",
    "bermas": "berma",
    "bermeja": "bermejo",
    "bermejales": "bermejal",
    "bermejas": "bermejo",
    "bermejiza": "bermejizo",
    "bermejizas": "bermejizo",
    "bermejizos": "bermejizo",
    "bermejos": "bermejo",
    "bermejuras": "bermejura",
    "bermellones": "bermellón",
    "bermellónes": "bermellón",
    "bermudianas": "bermudiana",
    "bermudinas": "bermudina",
    "bernarda": "bernardo",
    "bernardas": "bernardo",
    "bernardinas": "bernardina",
    "bernardos": "bernardo",
    "bernegales": "bernegal",
    "bernesa": "bernés",
    "bernesas": "bernés",
    "berneses": "bernés",
    "bernésa": "bernés",
    "bernésas": "bernés",
    "bernéses": "bernés",
    "berrea": "berrear",
    "berreaba": "berrear",
    "berreabais": "berrear",
    "berreaban": "berrear",
    "berreabas": "berrear",
    "berread": "berrear",
    "berreada": "berrear",
    "berreadas": "berrear",
    "berreado": "berrear",
    "berreados": "berrear",
    "berreamos": "berrear",
    "berrean": "berrear",
    "berreando": "berrear",
    "berreara": "berrear",
    "berrearais": "berrear",
    "berrearan": "berrear",
    "berrearas": "berrear",
    "berreare": "berrear",
    "berreareis": "berrear",
    "berrearemos": "berrear",
    "berrearen": "berrear",
    "berreares": "berrear",
    "berrearon": "berrear",
    "berreará": "berrear",
    "berrearán": "berrear",
    "berrearás": "berrear",
    "berrearé": "berrear",
    "berrearéis": "berrear",
    "berrearía": "berrear",
    "berrearíais": "berrear",
    "berrearíamos": "berrear",
    "berrearían": "berrear",
    "berrearías": "berrear",
    "berreas": "berrear",
    "berrease": "berrear",
    "berreaseis": "berrear",
    "berreasen": "berrear",
    "berreases": "berrear",
    "berreaste": "berrear",
    "berreasteis": "berrear",
    "berree": "berrear",
    "berreemos": "berrear",
    "berreen": "berrear",
    "berrees": "berrear",
    "berrenchines": "berrenchín",
    "berrenda": "berrendo",
    "berrendas": "berrendo",
    "berrendos": "berrendo",
    "berreo": "berrear",
    "berreona": "berreón",
    "berreonas": "berreón",
    "berreones": "berreón",
    "berreos": "berreo",
    "berreábamos": "berrear",
    "berreáis": "berrear",
    "berreáramos": "berrear",
    "berreáremos": "berrear",
    "berreásemos": "berrear",
    "berreé": "berrear",
    "berreéis": "berrear",
    "berreó": "berrear",
    "berridos": "berrido",
    "berrinches": "berrinche",
    "berrocales": "berrocal",
    "berros": "berro",
    "berruecos": "berrueco",
    "berzas": "berza",
    "beréberes": "beréber",
    "bes": "b",
    "besa": "besar",
    "besaba": "besar",
    "besabais": "besar",
    "besaban": "besar",
    "besabas": "besar",
    "besad": "besar",
    "besada": "besar",
    "besadas": "besar",
    "besado": "besar",
    "besados": "besar",
    "besalamanos": "besalamano",
    "besamelas": "besamela",
    "besameles": "besamel",
    "besamos": "besar",
    "besan": "besar",
    "besanas": "besana",
    "besando": "besar",
    "besantes": "besante",
    "besara": "besar",
    "besarais": "besar",
    "besaran": "besar",
    "besaras": "besar",
    "besare": "besar",
    "besareis": "besar",
    "besaremos": "besar",
    "besaren": "besar",
    "besares": "besar",
    "besaron": "besar",
    "besará": "besar",
    "besarán": "besar",
    "besarás": "besar",
    "besaré": "besar",
    "besaréis": "besar",
    "besaría": "besar",
    "besaríais": "besar",
    "besaríamos": "besar",
    "besarían": "besar",
    "besarías": "besar",
    "besas": "besar",
    "besase": "besar",
    "besaseis": "besar",
    "besasen": "besar",
    "besases": "besar",
    "besaste": "besar",
    "besasteis": "besar",
    "bese": "besar",
    "besemos": "besar",
    "besen": "besar",
    "beses": "besar",
    "beso": "besar",
    "besos": "beso",
    "best-selleres": "best-seller",
    "best-sellers": "best-seller",
    "bestiajes": "bestiaje",
    "bestiales": "bestial",
    "bestialidades": "bestialidad",
    "bestiarios": "bestiario",
    "bestias": "bestia",
    "bestiones": "bestión",
    "bestselleres": "bestseller",
    "bestsellers": "bestseller",
    "besuca": "besucar",
    "besucaba": "besucar",
    "besucabais": "besucar",
    "besucaban": "besucar",
    "besucabas": "besucar",
    "besucad": "besucar",
    "besucada": "besucar",
    "besucadas": "besucar",
    "besucado": "besucar",
    "besucados": "besucar",
    "besucamos": "besucar",
    "besucan": "besucar",
    "besucando": "besucar",
    "besucara": "besucar",
    "besucarais": "besucar",
    "besucaran": "besucar",
    "besucaras": "besucar",
    "besucare": "besucar",
    "besucareis": "besucar",
    "besucaremos": "besucar",
    "besucaren": "besucar",
    "besucares": "besucar",
    "besucaron": "besucar",
    "besucará": "besucar",
    "besucarán": "besucar",
    "besucarás": "besucar",
    "besucaré": "besucar",
    "besucaréis": "besucar",
    "besucaría": "besucar",
    "besucaríais": "besucar",
    "besucaríamos": "besucar",
    "besucarían": "besucar",
    "besucarías": "besucar",
    "besucas": "besucar",
    "besucase": "besucar",
    "besucaseis": "besucar",
    "besucasen": "besucar",
    "besucases": "besucar",
    "besucaste": "besucar",
    "besucasteis": "besucar",
    "besuco": "besucar",
    "besucábamos": "besucar",
    "besucáis": "besucar",
    "besucáramos": "besucar",
    "besucáremos": "besucar",
    "besucásemos": "besucar",
    "besucó": "besucar",
    "besugos": "besugo",
    "besugueras": "besuguera",
    "besuque": "besucar",
    "besuquea": "besuquear",
    "besuqueaba": "besuquear",
    "besuqueabais": "besuquear",
    "besuqueaban": "besuquear",
    "besuqueabas": "besuquear",
    "besuquead": "besuquear",
    "besuqueada": "besuquear",
    "besuqueadas": "besuquear",
    "besuqueado": "besuquear",
    "besuqueados": "besuquear",
    "besuqueamos": "besuquear",
    "besuquean": "besuquear",
    "besuqueando": "besuquear",
    "besuqueara": "besuquear",
    "besuquearais": "besuquear",
    "besuquearan": "besuquear",
    "besuquearas": "besuquear",
    "besuqueare": "besuquear",
    "besuqueareis": "besuquear",
    "besuquearemos": "besuquear",
    "besuquearen": "besuquear",
    "besuqueares": "besuquear",
    "besuquearon": "besuquear",
    "besuqueará": "besuquear",
    "besuquearán": "besuquear",
    "besuquearás": "besuquear",
    "besuquearé": "besuquear",
    "besuquearéis": "besuquear",
    "besuquearía": "besuquear",
    "besuquearíais": "besuquear",
    "besuquearíamos": "besuquear",
    "besuquearían": "besuquear",
    "besuquearías": "besuquear",
    "besuqueas": "besuquear",
    "besuquease": "besuquear",
    "besuqueaseis": "besuquear",
    "besuqueasen": "besuquear",
    "besuqueases": "besuquear",
    "besuqueaste": "besuquear",
    "besuqueasteis": "besuquear",
    "besuquee": "besuquear",
    "besuqueemos": "besuquear",
    "besuqueen": "besuquear",
    "besuquees": "besuquear",
    "besuquemos": "besucar",
    "besuquen": "besucar",
    "besuqueo": "besuquear",
    "besuqueos": "besuqueo",
    "besuques": "besucar",
    "besuqueábamos": "besuquear",
    "besuqueáis": "besuquear",
    "besuqueáramos": "besuquear",
    "besuqueáremos": "besuquear",
    "besuqueásemos": "besuquear",
    "besuqueé": "besuquear",
    "besuqueéis": "besuquear",
    "besuqueó": "besuquear",
    "besuqué": "besucar",
    "besuquéis": "besucar",
    "besábamos": "besar",
    "besáis": "besar",
    "besáramos": "besar",
    "besáremos": "besar",
    "besásemos": "besar",
    "besé": "besar",
    "beséis": "besar",
    "besó": "besar",
    "beta-oxidaciones": "beta-oxidación",
    "betabloqueantes": "betabloqueante",
    "betacarotenos": "betacaroteno",
    "betacismos": "betacismo",
    "betafitas": "betafita",
    "betaglobulinas": "betaglobulina",
    "betalipoproteinas": "betalipoproteina",
    "betarragas": "betarraga",
    "betas": "beta",
    "betatalasemias": "betatalasemia",
    "betatrones": "betatrón",
    "beteles": "betel",
    "betilos": "betilo",
    "betlemitas": "betlemita",
    "betuláceas": "betulácea",
    "betunes": "betún",
    "betónicas": "betónica",
    "beyes": "bey",
    "bezoares": "bezoar",
    "bezos": "bezo",
    "bezoárica": "bezoárico",
    "bezoáricas": "bezoárico",
    "bezoáricos": "bezoárico",
    "bezuda": "bezudo",
    "bezudas": "bezudo",
    "bezudos": "bezudo",
    "bhutanesa": "bhutanés",
    "bhutanesas": "bhutanés",
    "bhutaneses": "bhutanés",
    "biangulares": "biangular",
    "bianuales": "bianual",
    "biarcas": "biarca",
    "biarticulares": "biarticular",
    "biathlones": "biathlon",
    "biatlones": "biatlón",
    "biauditiva": "biauditivo",
    "biauditivas": "biauditivo",
    "biauditivos": "biauditivo",
    "biauriculares": "biauricular",
    "biaxiales": "biaxial",
    "bibelots": "bibelot",
    "biberones": "biberón",
    "biblias": "biblia",
    "bibliofilias": "bibliofilia",
    "bibliografías": "bibliografía",
    "bibliográfica": "bibliográfico",
    "bibliográficas": "bibliográfico",
    "bibliográficos": "bibliográfico",
    "bibliologías": "bibliología",
    "bibliomanías": "bibliomanía",
    "bibliometrías": "bibliometría",
    "bibliotecaria": "bibliotecario",
    "bibliotecarias": "bibliotecario",
    "bibliotecarios": "bibliotecario",
    "bibliotecas": "biblioteca",
    "bibliotecologías": "bibliotecología",
    "biblioteconomías": "biblioteconomía",
    "bibliófila": "bibliófilo",
    "bibliófilas": "bibliófilo",
    "bibliófilos": "bibliófilo",
    "bibliógrafa": "bibliógrafo",
    "bibliógrafas": "bibliógrafo",
    "bibliógrafos": "bibliógrafo",
    "biblióloga": "bibliólogo",
    "bibliólogas": "bibliólogo",
    "bibliólogos": "bibliólogo",
    "bibliómana": "bibliómano",
    "bibliómanas": "bibliómano",
    "bibliómanos": "bibliómano",
    "bibásica": "bibásico",
    "bibásicas": "bibásico",
    "bibásicos": "bibásico",
    "bicables": "bicable",
    "bicamerales": "bicameral",
    "bicameralismos": "bicameralismo",
    "bicampeona": "bicampeón",
    "bicampeonas": "bicampeón",
    "bicampeones": "bicampeón",
    "bicapas": "bicapa",
    "bicarbonatada": "bicarbonatado",
    "bicarbonatadas": "bicarbonatado",
    "bicarbonatados": "bicarbonatado",
    "bicarbonatos": "bicarbonato",
    "bicascos": "bicasco",
    "bicefalias": "bicefalia",
    "bicelulares": "bicelular",
    "bicentenarios": "bicentenario",
    "bicha": "bichar",
    "bichaba": "bichar",
    "bichabais": "bichar",
    "bichaban": "bichar",
    "bichabas": "bichar",
    "bichad": "bichar",
    "bichada": "bichar",
    "bichadas": "bichar",
    "bichado": "bichar",
    "bichados": "bichar",
    "bichamos": "bichar",
    "bichan": "bichar",
    "bichando": "bichar",
    "bichara": "bichar",
    "bicharais": "bichar",
    "bicharan": "bichar",
    "bicharas": "bichar",
    "bichare": "bichar",
    "bichareis": "bichar",
    "bicharemos": "bichar",
    "bicharen": "bichar",
    "bichares": "bichar",
    "bicharon": "bichar",
    "bicharracos": "bicharraco",
    "bichará": "bichar",
    "bicharán": "bichar",
    "bicharás": "bichar",
    "bicharé": "bichar",
    "bicharéis": "bichar",
    "bicharía": "bichar",
    "bicharíais": "bichar",
    "bicharíamos": "bichar",
    "bicharían": "bichar",
    "bicharías": "bichar",
    "bichas": "bichar",
    "bichase": "bichar",
    "bichaseis": "bichar",
    "bichasen": "bichar",
    "bichases": "bichar",
    "bichaste": "bichar",
    "bichasteis": "bichar",
    "biche": "bichar",
    "bichea": "bichear",
    "bicheaba": "bichear",
    "bicheabais": "bichear",
    "bicheaban": "bichear",
    "bicheabas": "bichear",
    "bichead": "bichear",
    "bicheada": "bichear",
    "bicheadas": "bichear",
    "bicheado": "bichear",
    "bicheados": "bichear",
    "bicheamos": "bichear",
    "bichean": "bichear",
    "bicheando": "bichear",
    "bicheara": "bichear",
    "bichearais": "bichear",
    "bichearan": "bichear",
    "bichearas": "bichear",
    "bicheare": "bichear",
    "bicheareis": "bichear",
    "bichearemos": "bichear",
    "bichearen": "bichear",
    "bicheares": "bichear",
    "bichearon": "bichear",
    "bicheará": "bichear",
    "bichearán": "bichear",
    "bichearás": "bichear",
    "bichearé": "bichear",
    "bichearéis": "bichear",
    "bichearía": "bichear",
    "bichearíais": "bichear",
    "bichearíamos": "bichear",
    "bichearían": "bichear",
    "bichearías": "bichear",
    "bicheas": "bichear",
    "bichease": "bichear",
    "bicheaseis": "bichear",
    "bicheasen": "bichear",
    "bicheases": "bichear",
    "bicheaste": "bichear",
    "bicheasteis": "bichear",
    "bichee": "bichear",
    "bicheemos": "bichear",
    "bicheen": "bichear",
    "bichees": "bichear",
    "bichemos": "bichar",
    "bichen": "bichar",
    "bicheo": "bichear",
    "bichera": "bichero",
    "bicheras": "bichero",
    "bicheros": "bichero",
    "biches": "bichar",
    "bicheábamos": "bichear",
    "bicheáis": "bichear",
    "bicheáramos": "bichear",
    "bicheáremos": "bichear",
    "bicheásemos": "bichear",
    "bicheé": "bichear",
    "bicheéis": "bichear",
    "bicheó": "bichear",
    "bichitos": "bichito",
    "bicho": "bichar",
    "bichos": "bicho",
    "bichoznos": "bichozno",
    "bichábamos": "bichar",
    "bicháis": "bichar",
    "bicháramos": "bichar",
    "bicháremos": "bichar",
    "bichásemos": "bichar",
    "biché": "bichar",
    "bichéis": "bichar",
    "bichó": "bichar",
    "bicicletas": "bicicleta",
    "biciclos": "biciclo",
    "bicigomática": "bicigomático",
    "bicigomáticas": "bicigomático",
    "bicigomáticos": "bicigomático",
    "bicipitales": "bicipital",
    "bicloruros": "bicloruro",
    "bicocas": "bicoca",
    "bicolores": "bicolor",
    "bicompuesta": "bicompuesto",
    "bicompuestas": "bicompuesto",
    "bicompuestos": "bicompuesto",
    "bicondílea": "bicondíleo",
    "bicondíleas": "bicondíleo",
    "bicondíleos": "bicondíleo",
    "biconvexa": "biconvexo",
    "biconvexas": "biconvexo",
    "biconvexos": "biconvexo",
    "bicoquetes": "bicoquete",
    "bicoquines": "bicoquín",
    "bicornes": "bicorne",
    "bicornios": "bicornio",
    "bicristales": "bicristal",
    "bicromatada": "bicromatado",
    "bicromatadas": "bicromatado",
    "bicromatados": "bicromatado",
    "bicromática": "bicromático",
    "bicromáticas": "bicromático",
    "bicromáticos": "bicromático",
    "bicromías": "bicromía",
    "bicuadrada": "bicuadrado",
    "bicuadradas": "bicuadrado",
    "bicuadrados": "bicuadrado",
    "bicuadráticas": "bicuadrática",
    "bicuspidada": "bicuspidado",
    "bicuspidadas": "bicuspidado",
    "bicuspidados": "bicuspidado",
    "bicéfala": "bicéfalo",
    "bicéfalas": "bicéfalo",
    "bicéfalos": "bicéfalo",
    "bicípites": "bicípite",
    "bicóncava": "bicóncavo",
    "bicóncavas": "bicóncavo",
    "bicóncavos": "bicóncavo",
    "bicónica": "bicónico",
    "bicónicas": "bicónico",
    "bicónicos": "bicónico",
    "bicúspides": "bicúspide",
    "bidelta": "bidelto",
    "bideltas": "bidelto",
    "bideltos": "bidelto",
    "bidets": "bidet",
    "bidimensionales": "bidimensional",
    "bidimensiones": "bidimensión",
    "bidireccionales": "bidireccional",
    "bidones": "bidón",
    "bidonvilles": "bidonville",
    "bidés": "bidé",
    "biedermeieres": "biedermeier",
    "biedermeiers": "biedermeier",
    "bielas": "biela",
    "bielda": "bieldar",
    "bieldaba": "bieldar",
    "bieldabais": "bieldar",
    "bieldaban": "bieldar",
    "bieldabas": "bieldar",
    "bieldad": "bieldar",
    "bieldada": "bieldar",
    "bieldadas": "bieldar",
    "bieldado": "bieldar",
    "bieldados": "bieldar",
    "bieldamos": "bieldar",
    "bieldan": "bieldar",
    "bieldando": "bieldar",
    "bieldara": "bieldar",
    "bieldarais": "bieldar",
    "bieldaran": "bieldar",
    "bieldaras": "bieldar",
    "bieldare": "bieldar",
    "bieldareis": "bieldar",
    "bieldaremos": "bieldar",
    "bieldaren": "bieldar",
    "bieldares": "bieldar",
    "bieldaron": "bieldar",
    "bieldará": "bieldar",
    "bieldarán": "bieldar",
    "bieldarás": "bieldar",
    "bieldaré": "bieldar",
    "bieldaréis": "bieldar",
    "bieldaría": "bieldar",
    "bieldaríais": "bieldar",
    "bieldaríamos": "bieldar",
    "bieldarían": "bieldar",
    "bieldarías": "bieldar",
    "bieldas": "bieldar",
    "bieldase": "bieldar",
    "bieldaseis": "bieldar",
    "bieldasen": "bieldar",
    "bieldases": "bieldar",
    "bieldaste": "bieldar",
    "bieldasteis": "bieldar",
    "bielde": "bieldar",
    "bieldemos": "bieldar",
    "bielden": "bieldar",
    "bieldes": "bieldar",
    "bieldo": "bieldar",
    "bieldos": "bieldo",
    "bieldábamos": "bieldar",
    "bieldáis": "bieldar",
    "bieldáramos": "bieldar",
    "bieldáremos": "bieldar",
    "bieldásemos": "bieldar",
    "bieldé": "bieldar",
    "bieldéis": "bieldar",
    "bieldó": "bieldar",
    "bielorrusa": "bielorruso",
    "bielorrusas": "bielorruso",
    "bielorrusos": "bielorruso",
    "bienales": "bienal",
    "bienandantes": "bienandante",
    "bienandanzas": "bienandanza",
    "bienaventurada": "bienaventurado",
    "bienaventuradas": "bienaventurado",
    "bienaventurados": "bienaventurado",
    "bienaventuranzas": "bienaventuranza",
    "bienes": "bien",
    "bienestares": "bienestar",
    "bienfortunada": "bienfortunado",
    "bienfortunadas": "bienfortunado",
    "bienfortunados": "bienfortunado",
    "biengranadas": "biengranada",
    "bienhablada": "bienhablado",
    "bienhabladas": "bienhablado",
    "bienhablados": "bienhablado",
    "bienhadada": "bienhadado",
    "bienhadadas": "bienhadado",
    "bienhadados": "bienhadado",
    "bienhechora": "bienhechor",
    "bienhechoras": "bienhechor",
    "bienhechores": "bienhechor",
    "bienintencionada": "bienintencionado",
    "bienintencionadas": "bienintencionado",
    "bienintencionados": "bienintencionado",
    "bienios": "bienio",
    "bienmandada": "bienmandado",
    "bienmandadas": "bienmandado",
    "bienmandados": "bienmandado",
    "bienmesabes": "bienmesabe",
    "bienolientes": "bienoliente",
    "bienqueramos": "bienquerer",
    "bienquered": "bienquerer",
    "bienqueremos": "bienquerer",
    "bienquerencias": "bienquerencia",
    "bienquereres": "bienquerer",
    "bienquerida": "bienquerer",
    "bienqueridas": "bienquerer",
    "bienquerido": "bienquerer",
    "bienqueridos": "bienquerer",
    "bienqueriendo": "bienquerer",
    "bienquerremos": "bienquerer",
    "bienquerrá": "bienquerer",
    "bienquerrán": "bienquerer",
    "bienquerrás": "bienquerer",
    "bienquerré": "bienquerer",
    "bienquerréis": "bienquerer",
    "bienquerría": "bienquerer",
    "bienquerríais": "bienquerer",
    "bienquerríamos": "bienquerer",
    "bienquerrían": "bienquerer",
    "bienquerrías": "bienquerer",
    "bienqueráis": "bienquerer",
    "bienqueréis": "bienquerer",
    "bienquería": "bienquerer",
    "bienqueríais": "bienquerer",
    "bienqueríamos": "bienquerer",
    "bienquerían": "bienquerer",
    "bienquerías": "bienquerer",
    "bienquiera": "bienquerer",
    "bienquieran": "bienquerer",
    "bienquieras": "bienquerer",
    "bienquiere": "bienquerer",
    "bienquieren": "bienquerer",
    "bienquieres": "bienquerer",
    "bienquiero": "bienquerer",
    "bienquise": "bienquerer",
    "bienquisiera": "bienquerer",
    "bienquisierais": "bienquerer",
    "bienquisieran": "bienquerer",
    "bienquisieras": "bienquerer",
    "bienquisiere": "bienquerer",
    "bienquisiereis": "bienquerer",
    "bienquisieren": "bienquerer",
    "bienquisieres": "bienquerer",
    "bienquisieron": "bienquerer",
    "bienquisiese": "bienquerer",
    "bienquisieseis": "bienquerer",
    "bienquisiesen": "bienquerer",
    "bienquisieses": "bienquerer",
    "bienquisimos": "bienquerer",
    "bienquisiste": "bienquerer",
    "bienquisisteis": "bienquerer",
    "bienquisiéramos": "bienquerer",
    "bienquisiéremos": "bienquerer",
    "bienquisiésemos": "bienquerer",
    "bienquiso": "bienquerer",
    "bienquista": "bienquisto",
    "bienquistaba": "bienquistar",
    "bienquistabais": "bienquistar",
    "bienquistaban": "bienquistar",
    "bienquistabas": "bienquistar",
    "bienquistad": "bienquistar",
    "bienquistada": "bienquistar",
    "bienquistadas": "bienquistar",
    "bienquistado": "bienquistar",
    "bienquistados": "bienquistar",
    "bienquistamos": "bienquistar",
    "bienquistan": "bienquistar",
    "bienquistando": "bienquistar",
    "bienquistara": "bienquistar",
    "bienquistarais": "bienquistar",
    "bienquistaran": "bienquistar",
    "bienquistaras": "bienquistar",
    "bienquistare": "bienquistar",
    "bienquistareis": "bienquistar",
    "bienquistaremos": "bienquistar",
    "bienquistaren": "bienquistar",
    "bienquistares": "bienquistar",
    "bienquistaron": "bienquistar",
    "bienquistará": "bienquistar",
    "bienquistarán": "bienquistar",
    "bienquistarás": "bienquistar",
    "bienquistaré": "bienquistar",
    "bienquistaréis": "bienquistar",
    "bienquistaría": "bienquistar",
    "bienquistaríais": "bienquistar",
    "bienquistaríamos": "bienquistar",
    "bienquistarían": "bienquistar",
    "bienquistarías": "bienquistar",
    "bienquistas": "bienquisto",
    "bienquistase": "bienquistar",
    "bienquistaseis": "bienquistar",
    "bienquistasen": "bienquistar",
    "bienquistases": "bienquistar",
    "bienquistaste": "bienquistar",
    "bienquistasteis": "bienquistar",
    "bienquiste": "bienquistar",
    "bienquistemos": "bienquistar",
    "bienquisten": "bienquistar",
    "bienquistes": "bienquistar",
    "bienquisto": "bienquistar",
    "bienquistos": "bienquisto",
    "bienquistábamos": "bienquistar",
    "bienquistáis": "bienquistar",
    "bienquistáramos": "bienquistar",
    "bienquistáremos": "bienquistar",
    "bienquistásemos": "bienquistar",
    "bienquisté": "bienquistar",
    "bienquistéis": "bienquistar",
    "bienquistó": "bienquistar",
    "bienvenida": "bienvenido",
    "bienvenidas": "bienvenido",
    "bienvenidos": "bienvenido",
    "bienvista": "bienvisto",
    "bienvistas": "bienvisto",
    "bienvistos": "bienvisto",
    "bieses": "bies",
    "biesfenoidales": "biesfenoidal",
    "biesfenoides": "biesfenoide",
    "biestables": "biestable",
    "bifaces": "bifaz",
    "bifaciales": "bifacial",
    "bifes": "bife",
    "bifidobacterias": "bifidobacteria",
    "bifilares": "bifilar",
    "biflora": "bifloro",
    "bifloras": "bifloro",
    "bifloros": "bifloro",
    "bifluoruros": "bifluoruro",
    "bifocales": "bifocal",
    "biformes": "biforme",
    "bifosfatos": "bifosfato",
    "bifrontes": "bifronte",
    "bifurca": "bifurcar",
    "bifurcaba": "bifurcar",
    "bifurcabais": "bifurcar",
    "bifurcaban": "bifurcar",
    "bifurcabas": "bifurcar",
    "bifurcaciones": "bifurcación",
    "bifurcaciónes": "bifurcación",
    "bifurcad": "bifurcar",
    "bifurcada": "bifurcar",
    "bifurcadas": "bifurcar",
    "bifurcado": "bifurcar",
    "bifurcados": "bifurcar",
    "bifurcamos": "bifurcar",
    "bifurcan": "bifurcar",
    "bifurcando": "bifurcar",
    "bifurcara": "bifurcar",
    "bifurcarais": "bifurcar",
    "bifurcaran": "bifurcar",
    "bifurcaras": "bifurcar",
    "bifurcare": "bifurcar",
    "bifurcareis": "bifurcar",
    "bifurcaremos": "bifurcar",
    "bifurcaren": "bifurcar",
    "bifurcares": "bifurcar",
    "bifurcaron": "bifurcar",
    "bifurcará": "bifurcar",
    "bifurcarán": "bifurcar",
    "bifurcarás": "bifurcar",
    "bifurcaré": "bifurcar",
    "bifurcaréis": "bifurcar",
    "bifurcaría": "bifurcar",
    "bifurcaríais": "bifurcar",
    "bifurcaríamos": "bifurcar",
    "bifurcarían": "bifurcar",
    "bifurcarías": "bifurcar",
    "bifurcas": "bifurcar",
    "bifurcase": "bifurcar",
    "bifurcaseis": "bifurcar",
    "bifurcasen": "bifurcar",
    "bifurcases": "bifurcar",
    "bifurcaste": "bifurcar",
    "bifurcasteis": "bifurcar",
    "bifurco": "bifurcar",
    "bifurcábamos": "bifurcar",
    "bifurcáis": "bifurcar",
    "bifurcáramos": "bifurcar",
    "bifurcáremos": "bifurcar",
    "bifurcásemos": "bifurcar",
    "bifurcó": "bifurcar",
    "bifurque": "bifurcar",
    "bifurquemos": "bifurcar",
    "bifurquen": "bifurcar",
    "bifurques": "bifurcar",
    "bifurqué": "bifurcar",
    "bifurquéis": "bifurcar",
    "bifásica": "bifásico",
    "bifásicas": "bifásico",
    "bifásicos": "bifásico",
    "bigamias": "bigamia",
    "bigarda": "bigardo",
    "bigardas": "bigardo",
    "bigardea": "bigardear",
    "bigardeaba": "bigardear",
    "bigardeabais": "bigardear",
    "bigardeaban": "bigardear",
    "bigardeabas": "bigardear",
    "bigardead": "bigardear",
    "bigardeada": "bigardear",
    "bigardeadas": "bigardear",
    "bigardeado": "bigardear",
    "bigardeados": "bigardear",
    "bigardeamos": "bigardear",
    "bigardean": "bigardear",
    "bigardeando": "bigardear",
    "bigardeara": "bigardear",
    "bigardearais": "bigardear",
    "bigardearan": "bigardear",
    "bigardearas": "bigardear",
    "bigardeare": "bigardear",
    "bigardeareis": "bigardear",
    "bigardearemos": "bigardear",
    "bigardearen": "bigardear",
    "bigardeares": "bigardear",
    "bigardearon": "bigardear",
    "bigardeará": "bigardear",
    "bigardearán": "bigardear",
    "bigardearás": "bigardear",
    "bigardearé": "bigardear",
    "bigardearéis": "bigardear",
    "bigardearía": "bigardear",
    "bigardearíais": "bigardear",
    "bigardearíamos": "bigardear",
    "bigardearían": "bigardear",
    "bigardearías": "bigardear",
    "bigardeas": "bigardear",
    "bigardease": "bigardear",
    "bigardeaseis": "bigardear",
    "bigardeasen": "bigardear",
    "bigardeases": "bigardear",
    "bigardeaste": "bigardear",
    "bigardeasteis": "bigardear",
    "bigardee": "bigardear",
    "bigardeemos": "bigardear",
    "bigardeen": "bigardear",
    "bigardees": "bigardear",
    "bigardeo": "bigardear",
    "bigardeábamos": "bigardear",
    "bigardeáis": "bigardear",
    "bigardeáramos": "bigardear",
    "bigardeáremos": "bigardear",
    "bigardeásemos": "bigardear",
    "bigardeé": "bigardear",
    "bigardeéis": "bigardear",
    "bigardeó": "bigardear",
    "bigardos": "bigardo",
    "bigardías": "bigardía",
    "bigas": "biga",
    "bigbangs": "bigbang",
    "bigeminada": "bigeminado",
    "bigeminadas": "bigeminado",
    "bigeminados": "bigeminado",
    "bigenérica": "bigenérico",
    "bigenéricas": "bigenérico",
    "bigenéricos": "bigenérico",
    "bignonias": "bignonia",
    "bignoniáceas": "bignoniácea",
    "bigornias": "bigornia",
    "bigoteras": "bigotera",
    "bigotes": "bigote",
    "bigotuda": "bigotudo",
    "bigotudas": "bigotudo",
    "bigotudos": "bigotudo",
    "bigudíes": "bigudí",
    "bigudís": "bigudí",
    "bigudíses": "bigudí",
    "bijas": "bija",
    "bikinis": "bikini",
    "bilabiada": "bilabiado",
    "bilabiadas": "bilabiado",
    "bilabiados": "bilabiado",
    "bilabiales": "bilabial",
    "bilabializaciones": "bilabialización",
    "bilabiovelares": "bilabiovelar",
    "bilaterales": "bilateral",
    "bilateralidades": "bilateralidad",
    "bilateralismos": "bilateralismo",
    "bilateralizaciones": "bilateralización",
    "bilbaína": "bilbaíno",
    "bilbaínas": "bilbaíno",
    "bilbaínos": "bilbaíno",
    "biliares": "biliar",
    "bilineales": "bilineal",
    "bilinealidades": "bilinealidad",
    "bilineares": "bilinear",
    "bilingües": "bilingüe",
    "bilingüismos": "bilingüismo",
    "bilirrubinas": "bilirrubina",
    "bilirrubinemias": "bilirrubinemia",
    "biliverdinas": "biliverdina",
    "billaldas": "billalda",
    "billardas": "billarda",
    "billares": "billar",
    "billaristas": "billarista",
    "billarística": "billarístico",
    "billarísticas": "billarístico",
    "billarísticos": "billarístico",
    "billas": "billa",
    "billetajes": "billetaje",
    "billetera": "billetero",
    "billeteras": "billetero",
    "billeteros": "billetero",
    "billetes": "billete",
    "billonaria": "billonario",
    "billonarias": "billonario",
    "billonarios": "billonario",
    "billones": "billón",
    "billonésima": "billonésimo",
    "billonésimas": "billonésimo",
    "billonésimo": "1000000000000",
    "billonésimos": "billonésimo",
    "bilobulada": "bilobulado",
    "bilobuladas": "bilobulado",
    "bilobulados": "bilobulado",
    "bilocales": "bilocal",
    "bilocalidades": "bilocalidad",
    "bilofodonta": "bilofodonto",
    "bilofodontas": "bilofodonto",
    "bilofodontos": "bilofodonto",
    "bilogías": "bilogía",
    "biláminas": "bilámina",
    "bilítera": "bilítero",
    "bilíteras": "bilítero",
    "bilíteros": "bilítero",
    "bimalares": "bimalar",
    "bimana": "bímano",
    "bimanas": "bímano",
    "bimanos": "bímano",
    "bimaxilares": "bimaxilar",
    "bimbas": "bimba",
    "bimembres": "bimembre",
    "bimensuales": "bimensual",
    "bimestrales": "bimestral",
    "bimestres": "bimestre",
    "bimetalismos": "bimetalismo",
    "bimetalistas": "bimetalista",
    "bimetálica": "bimetálico",
    "bimetálicas": "bimetálico",
    "bimetálicos": "bimetálico",
    "bimodales": "bimodal",
    "bimoleculares": "bimolecular",
    "bimotora": "bimotor",
    "bimotoras": "bimotor",
    "bimotores": "bimotor",
    "bimórfica": "bimórfico",
    "bimórficas": "bimórfico",
    "bimórficos": "bimórfico",
    "bina": "binar",
    "binaba": "binar",
    "binabais": "binar",
    "binaban": "binar",
    "binabas": "binar",
    "binaciones": "binación",
    "binad": "binar",
    "binada": "binar",
    "binadas": "binar",
    "binado": "binar",
    "binadora": "binador",
    "binadoras": "binador",
    "binadores": "binador",
    "binados": "binar",
    "binaduras": "binadura",
    "binamos": "binar",
    "binan": "binar",
    "binando": "binar",
    "binara": "binar",
    "binarais": "binar",
    "binaran": "binar",
    "binaras": "binar",
    "binare": "binar",
    "binareis": "binar",
    "binaremos": "binar",
    "binaren": "binar",
    "binares": "binar",
    "binaria": "binario",
    "binarias": "binario",
    "binarios": "binario",
    "binarismos": "binarismo",
    "binaron": "binar",
    "binará": "binar",
    "binarán": "binar",
    "binarás": "binar",
    "binaré": "binar",
    "binaréis": "binar",
    "binaría": "binar",
    "binaríais": "binar",
    "binaríamos": "binar",
    "binarían": "binar",
    "binarías": "binar",
    "binas": "binar",
    "binasales": "binasal",
    "binase": "binar",
    "binaseis": "binar",
    "binasen": "binar",
    "binases": "binar",
    "binaste": "binar",
    "binasteis": "binar",
    "binaurales": "binaural",
    "binauriculares": "binauricular",
    "binazones": "binazón",
    "bine": "binar",
    "binemos": "binar",
    "binen": "binar",
    "bines": "binar",
    "bingos": "bingo",
    "bino": "binar",
    "binoculares": "binocular",
    "binodos": "binodo",
    "binomiales": "binomial",
    "binomios": "binomio",
    "binzas": "binza",
    "binábamos": "binar",
    "bináis": "binar",
    "bináramos": "binar",
    "bináremos": "binar",
    "binásemos": "binar",
    "biné": "binar",
    "binéis": "binar",
    "binó": "binar",
    "binóculos": "binóculo",
    "binómica": "binómico",
    "binómicas": "binómico",
    "binómicos": "binómico",
    "bioacumulaciones": "bioacumulación",
    "bioalcoholes": "bioalcohol",
    "bioblastos": "bioblasto",
    "biocatalizadores": "biocatalizador",
    "biocidas": "biocida",
    "bioclimas": "bioclima",
    "bioclimatologías": "bioclimatología",
    "bioclástica": "bioclástico",
    "bioclásticas": "bioclástico",
    "bioclásticos": "bioclástico",
    "biocombustibles": "biocombustible",
    "biocompatibles": "biocompatible",
    "bioconstrucciones": "bioconstrucción",
    "biocoras": "biocora",
    "biocosméticas": "biocosmética",
    "biocronologías": "biocronología",
    "biodegradabilidades": "biodegradabilidad",
    "biodegradables": "biodegradable",
    "biodegradaciones": "biodegradación",
    "biodinámica": "biodinámico",
    "biodinámicas": "biodinámico",
    "biodinámicos": "biodinámico",
    "biodiversidades": "biodiversidad",
    "bioecus": "bioecu",
    "bioelementos": "bioelemento",
    "bioenergéticas": "bioenergética",
    "bioenergías": "bioenergía",
    "bioerosiones": "bioerosión",
    "bioestadística": "bioestadístico",
    "bioestadísticas": "bioestadístico",
    "bioestadísticos": "bioestadístico",
    "bioestratigrafías": "bioestratigrafía",
    "biofotómetros": "biofotómetro",
    "biofísica": "biofísico",
    "biofísicas": "biofísico",
    "biofísicos": "biofísico",
    "biogases": "biogás",
    "biogenética": "biogenético",
    "biogenéticas": "biogenético",
    "biogenéticos": "biogenético",
    "biogeografías": "biogeografía",
    "biogeográfica": "biogeográfico",
    "biogeográficas": "biogeográfico",
    "biogeográficos": "biogeográfico",
    "biogeoquímica": "biogeoquímico",
    "biogeoquímicas": "biogeoquímico",
    "biogeoquímicos": "biogeoquímico",
    "biografiaba": "biografiar",
    "biografiabais": "biografiar",
    "biografiaban": "biografiar",
    "biografiabas": "biografiar",
    "biografiad": "biografiar",
    "biografiada": "biografiar",
    "biografiadas": "biografiar",
    "biografiado": "biografiar",
    "biografiados": "biografiar",
    "biografiamos": "biografiar",
    "biografiando": "biografiar",
    "biografiara": "biografiar",
    "biografiarais": "biografiar",
    "biografiaran": "biografiar",
    "biografiaras": "biografiar",
    "biografiare": "biografiar",
    "biografiareis": "biografiar",
    "biografiaremos": "biografiar",
    "biografiaren": "biografiar",
    "biografiares": "biografiar",
    "biografiaron": "biografiar",
    "biografiará": "biografiar",
    "biografiarán": "biografiar",
    "biografiarás": "biografiar",
    "biografiaré": "biografiar",
    "biografiaréis": "biografiar",
    "biografiaría": "biografiar",
    "biografiaríais": "biografiar",
    "biografiaríamos": "biografiar",
    "biografiarían": "biografiar",
    "biografiarías": "biografiar",
    "biografiase": "biografiar",
    "biografiaseis": "biografiar",
    "biografiasen": "biografiar",
    "biografiases": "biografiar",
    "biografiaste": "biografiar",
    "biografiasteis": "biografiar",
    "biografiemos": "biografiar",
    "biografiábamos": "biografiar",
    "biografiáis": "biografiar",
    "biografiáramos": "biografiar",
    "biografiáremos": "biografiar",
    "biografiásemos": "biografiar",
    "biografié": "biografiar",
    "biografiéis": "biografiar",
    "biografió": "biografiar",
    "biografía": "biografiar",
    "biografían": "biografiar",
    "biografías": "biografiar",
    "biografíe": "biografiar",
    "biografíen": "biografiar",
    "biografíes": "biografiar",
    "biografío": "biografiar",
    "biográfica": "biográfico",
    "biográficas": "biográfico",
    "biográficos": "biográfico",
    "biogénica": "biogénico",
    "biogénicas": "biogénico",
    "biogénicos": "biogénico",
    "biohermes": "bioherme",
    "biohermos": "biohermo",
    "bioindicadores": "bioindicador",
    "bioingeniera": "bioingeniero",
    "bioingenieras": "bioingeniero",
    "bioingenieros": "bioingeniero",
    "bioingenierías": "bioingeniería",
    "biolixiviaciones": "biolixiviación",
    "biologías": "biología",
    "bioluminiscencias": "bioluminiscencia",
    "biolítica": "biolítico",
    "biolíticas": "biolítico",
    "biolíticos": "biolítico",
    "biológica": "biológico",
    "biológicas": "biológico",
    "biológicos": "biológico",
    "biomas": "bioma",
    "biomasas": "biomasa",
    "biomateriales": "biomaterial",
    "biombos": "biombo",
    "biomecánicas": "biomecánica",
    "biomedicinas": "biomedicina",
    "biometanizaciones": "biometanización",
    "biometanos": "biometano",
    "biometristas": "biometrista",
    "biometrías": "biometría",
    "biomicroscopias": "biomicroscopia",
    "biomédica": "biomédico",
    "biomédicas": "biomédico",
    "biomédicos": "biomédico",
    "biométrica": "biométrico",
    "biométricas": "biométrico",
    "biométricos": "biométrico",
    "biomórfica": "biomórfico",
    "biomórficas": "biomórfico",
    "biomórficos": "biomórfico",
    "bionomías": "bionomía",
    "bioplasmas": "bioplasma",
    "biopolímera": "biopolímero",
    "biopolímeras": "biopolímero",
    "biopolímeros": "biopolímero",
    "biopsias": "biopsia",
    "biopsicologías": "biopsicología",
    "biopulpectomías": "biopulpectomía",
    "biopulpotomías": "biopulpotomía",
    "bioquímica": "bioquímico",
    "bioquímicas": "bioquímico",
    "bioquímicos": "bioquímico",
    "biorreactores": "biorreactor",
    "biorrealimentaciones": "biorrealimentación",
    "biorretroalimentacoónes": "biorretroalimentación",
    "biorritmos": "biorritmo",
    "biosecuencias": "biosecuencia",
    "bioseguridades": "bioseguridad",
    "biosensores": "biosensor",
    "biosferas": "biosfera",
    "biosociologías": "biosociología",
    "biostasias": "biostasia",
    "biostratigrafías": "biostratigrafía",
    "biostromas": "biostroma",
    "biotas": "biota",
    "biotecnologías": "biotecnología",
    "biotecnológica": "biotecnológico",
    "biotecnológicas": "biotecnológico",
    "biotecnológicos": "biotecnológico",
    "biotes": "biot",
    "biotinas": "biotina",
    "biotipologías": "biotipología",
    "biotipológica": "biotipológico",
    "biotipológicas": "biotipológico",
    "biotipológicos": "biotipológico",
    "biotipos": "biotipo",
    "biotitas": "biotita",
    "biotopos": "biotopo",
    "bioturbaciones": "bioturbación",
    "biovaloraciones": "biovaloración",
    "bioxalatos": "bioxalato",
    "biozonas": "biozona",
    "bioéticas": "bioética",
    "bioórganos": "bioórgano",
    "biparticiones": "bipartición",
    "bipartida": "bipartido",
    "bipartidas": "bipartido",
    "bipartidismos": "bipartidismo",
    "bipartidos": "bipartido",
    "bipartita": "bipartito",
    "bipartitas": "bipartito",
    "bipartitos": "bipartito",
    "bipedalismos": "bipedalismo",
    "bipinnada": "bipinnado",
    "bipinnadas": "bipinnado",
    "bipinnados": "bipinnado",
    "bipiramidales": "bipiramidal",
    "bipirámides": "bipirámide",
    "biplanos": "biplano",
    "biplazas": "biplaza",
    "bipolares": "bipolar",
    "bipolaridades": "bipolaridad",
    "bipolarizaciones": "bipolarización",
    "bipolarones": "bipolarón",
    "bipolos": "bipolo",
    "bipolímeros": "bipolímero",
    "bipostes": "biposte",
    "biprocesadores": "biprocesador",
    "biprotrusiones": "biprotrusión",
    "bipuertas": "bipuerta",
    "bipátridas": "bipátrida",
    "biquinaria": "biquinario",
    "biquinarias": "biquinario",
    "biquinarios": "biquinario",
    "biquinis": "biquini",
    "birdies": "birdie",
    "biricúes": "biricú",
    "biricús": "biricú",
    "birimbaos": "birimbao",
    "birla": "birlar",
    "birlaba": "birlar",
    "birlabais": "birlar",
    "birlaban": "birlar",
    "birlabas": "birlar",
    "birlad": "birlar",
    "birlada": "birlar",
    "birladas": "birlar",
    "birlado": "birlar",
    "birladora": "birlador",
    "birladoras": "birlador",
    "birladores": "birlador",
    "birlados": "birlar",
    "birlamos": "birlar",
    "birlan": "birlar",
    "birlando": "birlar",
    "birlara": "birlar",
    "birlarais": "birlar",
    "birlaran": "birlar",
    "birlaras": "birlar",
    "birlare": "birlar",
    "birlareis": "birlar",
    "birlaremos": "birlar",
    "birlaren": "birlar",
    "birlares": "birlar",
    "birlaron": "birlar",
    "birlará": "birlar",
    "birlarán": "birlar",
    "birlarás": "birlar",
    "birlaré": "birlar",
    "birlaréis": "birlar",
    "birlaría": "birlar",
    "birlaríais": "birlar",
    "birlaríamos": "birlar",
    "birlarían": "birlar",
    "birlarías": "birlar",
    "birlas": "birlar",
    "birlase": "birlar",
    "birlaseis": "birlar",
    "birlasen": "birlar",
    "birlases": "birlar",
    "birlaste": "birlar",
    "birlasteis": "birlar",
    "birle": "birlar",
    "birlemos": "birlar",
    "birlen": "birlar",
    "birles": "birlar",
    "birlo": "birlar",
    "birlochas": "birlocha",
    "birlochos": "birlocho",
    "birlongas": "birlonga",
    "birlábamos": "birlar",
    "birláis": "birlar",
    "birláramos": "birlar",
    "birláremos": "birlar",
    "birlásemos": "birlar",
    "birlé": "birlar",
    "birléis": "birlar",
    "birlíes": "birlí",
    "birlís": "birlí",
    "birló": "birlar",
    "birmana": "birmano",
    "birmanas": "birmano",
    "birmanos": "birmano",
    "birreactores": "birreactor",
    "birrefringencias": "birrefringencia",
    "birrefringentes": "birrefringente",
    "birremes": "birreme",
    "birretas": "birreta",
    "birretes": "birrete",
    "birretinas": "birretina",
    "birrias": "birria",
    "birrs": "birr",
    "bisa": "bisar",
    "bisaba": "bisar",
    "bisabais": "bisar",
    "bisaban": "bisar",
    "bisabas": "bisar",
    "bisabuela": "bisabuelo",
    "bisabuelas": "bisabuelo",
    "bisabuelos": "bisabuelo",
    "bisad": "bisar",
    "bisada": "bisar",
    "bisadas": "bisar",
    "bisado": "bisar",
    "bisados": "bisar",
    "bisagras": "bisagra",
    "bisaltos": "bisalto",
    "bisamos": "bisar",
    "bisan": "bisar",
    "bisando": "bisar",
    "bisara": "bisar",
    "bisarais": "bisar",
    "bisaran": "bisar",
    "bisaras": "bisar",
    "bisare": "bisar",
    "bisareis": "bisar",
    "bisaremos": "bisar",
    "bisaren": "bisar",
    "bisares": "bisar",
    "bisaron": "bisar",
    "bisará": "bisar",
    "bisarán": "bisar",
    "bisarás": "bisar",
    "bisaré": "bisar",
    "bisaréis": "bisar",
    "bisaría": "bisar",
    "bisaríais": "bisar",
    "bisaríamos": "bisar",
    "bisarían": "bisar",
    "bisarías": "bisar",
    "bisas": "bisar",
    "bisase": "bisar",
    "bisaseis": "bisar",
    "bisasen": "bisar",
    "bisases": "bisar",
    "bisaste": "bisar",
    "bisasteis": "bisar",
    "bisbisa": "bisbisar",
    "bisbisaba": "bisbisar",
    "bisbisabais": "bisbisar",
    "bisbisaban": "bisbisar",
    "bisbisabas": "bisbisar",
    "bisbisad": "bisbisar",
    "bisbisada": "bisbisar",
    "bisbisadas": "bisbisar",
    "bisbisado": "bisbisar",
    "bisbisados": "bisbisar",
    "bisbisamos": "bisbisar",
    "bisbisan": "bisbisar",
    "bisbisando": "bisbisar",
    "bisbisara": "bisbisar",
    "bisbisarais": "bisbisar",
    "bisbisaran": "bisbisar",
    "bisbisaras": "bisbisar",
    "bisbisare": "bisbisar",
    "bisbisareis": "bisbisar",
    "bisbisaremos": "bisbisar",
    "bisbisaren": "bisbisar",
    "bisbisares": "bisbisar",
    "bisbisaron": "bisbisar",
    "bisbisará": "bisbisar",
    "bisbisarán": "bisbisar",
    "bisbisarás": "bisbisar",
    "bisbisaré": "bisbisar",
    "bisbisaréis": "bisbisar",
    "bisbisaría": "bisbisar",
    "bisbisaríais": "bisbisar",
    "bisbisaríamos": "bisbisar",
    "bisbisarían": "bisbisar",
    "bisbisarías": "bisbisar",
    "bisbisas": "bisbisar",
    "bisbisase": "bisbisar",
    "bisbisaseis": "bisbisar",
    "bisbisasen": "bisbisar",
    "bisbisases": "bisbisar",
    "bisbisaste": "bisbisar",
    "bisbisasteis": "bisbisar",
    "bisbise": "bisbisar",
    "bisbisea": "bisbisear",
    "bisbiseaba": "bisbisear",
    "bisbiseabais": "bisbisear",
    "bisbiseaban": "bisbisear",
    "bisbiseabas": "bisbisear",
    "bisbisead": "bisbisear",
    "bisbiseada": "bisbisear",
    "bisbiseadas": "bisbisear",
    "bisbiseado": "bisbisear",
    "bisbiseados": "bisbisear",
    "bisbiseamos": "bisbisear",
    "bisbisean": "bisbisear",
    "bisbiseando": "bisbisear",
    "bisbiseara": "bisbisear",
    "bisbisearais": "bisbisear",
    "bisbisearan": "bisbisear",
    "bisbisearas": "bisbisear",
    "bisbiseare": "bisbisear",
    "bisbiseareis": "bisbisear",
    "bisbisearemos": "bisbisear",
    "bisbisearen": "bisbisear",
    "bisbiseares": "bisbisear",
    "bisbisearon": "bisbisear",
    "bisbiseará": "bisbisear",
    "bisbisearán": "bisbisear",
    "bisbisearás": "bisbisear",
    "bisbisearé": "bisbisear",
    "bisbisearéis": "bisbisear",
    "bisbisearía": "bisbisear",
    "bisbisearíais": "bisbisear",
    "bisbisearíamos": "bisbisear",
    "bisbisearían": "bisbisear",
    "bisbisearías": "bisbisear",
    "bisbiseas": "bisbisear",
    "bisbisease": "bisbisear",
    "bisbiseaseis": "bisbisear",
    "bisbiseasen": "bisbisear",
    "bisbiseases": "bisbisear",
    "bisbiseaste": "bisbisear",
    "bisbiseasteis": "bisbisear",
    "bisbisee": "bisbisear",
    "bisbiseemos": "bisbisear",
    "bisbiseen": "bisbisear",
    "bisbisees": "bisbisear",
    "bisbisemos": "bisbisar",
    "bisbisen": "bisbisar",
    "bisbiseo": "bisbisear",
    "bisbiseos": "bisbiseo",
    "bisbises": "bisbisar",
    "bisbiseábamos": "bisbisear",
    "bisbiseáis": "bisbisear",
    "bisbiseáramos": "bisbisear",
    "bisbiseáremos": "bisbisear",
    "bisbiseásemos": "bisbisear",
    "bisbiseé": "bisbisear",
    "bisbiseéis": "bisbisear",
    "bisbiseó": "bisbisear",
    "bisbiso": "bisbisar",
    "bisbisábamos": "bisbisar",
    "bisbisáis": "bisbisar",
    "bisbisáramos": "bisbisar",
    "bisbisáremos": "bisbisar",
    "bisbisásemos": "bisbisar",
    "bisbisé": "bisbisar",
    "bisbiséis": "bisbisar",
    "bisbisó": "bisbisar",
    "bisbitas": "bisbita",
    "biscotes": "biscote",
    "biscuits": "biscuit",
    "bise": "bisar",
    "biseca": "bisecar",
    "bisecaba": "bisecar",
    "bisecabais": "bisecar",
    "bisecaban": "bisecar",
    "bisecabas": "bisecar",
    "bisecad": "bisecar",
    "bisecada": "bisecar",
    "bisecadas": "bisecar",
    "bisecado": "bisecar",
    "bisecados": "bisecar",
    "bisecamos": "bisecar",
    "bisecan": "bisecar",
    "bisecando": "bisecar",
    "bisecara": "bisecar",
    "bisecarais": "bisecar",
    "bisecaran": "bisecar",
    "bisecaras": "bisecar",
    "bisecare": "bisecar",
    "bisecareis": "bisecar",
    "bisecaremos": "bisecar",
    "bisecaren": "bisecar",
    "bisecares": "bisecar",
    "bisecaron": "bisecar",
    "bisecará": "bisecar",
    "bisecarán": "bisecar",
    "bisecarás": "bisecar",
    "bisecaré": "bisecar",
    "bisecaréis": "bisecar",
    "bisecaría": "bisecar",
    "bisecaríais": "bisecar",
    "bisecaríamos": "bisecar",
    "bisecarían": "bisecar",
    "bisecarías": "bisecar",
    "bisecas": "bisecar",
    "bisecase": "bisecar",
    "bisecaseis": "bisecar",
    "bisecasen": "bisecar",
    "bisecases": "bisecar",
    "bisecaste": "bisecar",
    "bisecasteis": "bisecar",
    "bisecciones": "bisección",
    "bisecciónes": "bisección",
    "biseco": "bisecar",
    "bisectrices": "bisectriz",
    "bisecábamos": "bisecar",
    "bisecáis": "bisecar",
    "bisecáramos": "bisecar",
    "bisecáremos": "bisecar",
    "bisecásemos": "bisecar",
    "bisecó": "bisecar",
    "bisela": "biselar",
    "biselaba": "biselar",
    "biselabais": "biselar",
    "biselaban": "biselar",
    "biselabas": "biselar",
    "biselad": "biselar",
    "biselada": "biselar",
    "biseladas": "biselar",
    "biselado": "biselar",
    "biseladora": "biselador",
    "biseladoras": "biselador",
    "biseladores": "biselador",
    "biselados": "biselar",
    "biselamos": "biselar",
    "biselan": "biselar",
    "biselando": "biselar",
    "biselara": "biselar",
    "biselarais": "biselar",
    "biselaran": "biselar",
    "biselaras": "biselar",
    "biselare": "biselar",
    "biselareis": "biselar",
    "biselaremos": "biselar",
    "biselaren": "biselar",
    "biselares": "biselar",
    "biselaron": "biselar",
    "biselará": "biselar",
    "biselarán": "biselar",
    "biselarás": "biselar",
    "biselaré": "biselar",
    "biselaréis": "biselar",
    "biselaría": "biselar",
    "biselaríais": "biselar",
    "biselaríamos": "biselar",
    "biselarían": "biselar",
    "biselarías": "biselar",
    "biselas": "biselar",
    "biselase": "biselar",
    "biselaseis": "biselar",
    "biselasen": "biselar",
    "biselases": "biselar",
    "biselaste": "biselar",
    "biselasteis": "biselar",
    "bisele": "biselar",
    "biselemos": "biselar",
    "biselen": "biselar",
    "biseles": "biselar",
    "biselo": "biselar",
    "biselábamos": "biselar",
    "biseláis": "biselar",
    "biseláramos": "biselar",
    "biseláremos": "biselar",
    "biselásemos": "biselar",
    "biselé": "biselar",
    "biseléis": "biselar",
    "biseló": "biselar",
    "bisemanales": "bisemanal",
    "bisemos": "bisar",
    "bisen": "bisar",
    "biseque": "bisecar",
    "bisequemos": "bisecar",
    "bisequen": "bisecar",
    "biseques": "bisecar",
    "bisequé": "bisecar",
    "bisequéis": "bisecar",
    "bises": "bisar",
    "bisexuales": "bisexual",
    "bisexualidades": "bisexualidad",
    "bisfenoles": "bisfenol",
    "bishidroxicumarinas": "bishidroxicumarina",
    "bisialitizaciones": "bisialitización",
    "bisiesta": "bisiesto",
    "bisiestas": "bisiesto",
    "bisiestos": "bisiesto",
    "bisilábica": "bisilábico",
    "bisilábicas": "bisilábico",
    "bisilábicos": "bisilábico",
    "bisistemas": "bisistema",
    "bismalvas": "bismalva",
    "bismanoles": "bismanol",
    "bismitas": "bismita",
    "bismutinas": "bismutina",
    "bismutinitas": "bismutinita",
    "bismutitas": "bismutita",
    "bismutos": "bismuto",
    "bisnieta": "bisnieto",
    "bisnietas": "bisnieto",
    "bisnietos": "bisnieto",
    "biso": "bisar",
    "bisoja": "bisojo",
    "bisojas": "bisojo",
    "bisojos": "bisojo",
    "bisonnna": "bisonnno",
    "bisonnnas": "bisonnno",
    "bisonnnos": "bisonnno",
    "bisontes": "bisonte",
    "bisoprololes": "bisoprolol",
    "bisos": "biso",
    "bisoña": "bisoño",
    "bisoñadas": "bisoñada",
    "bisoñas": "bisoño",
    "bisoñerías": "bisoñería",
    "bisoños": "bisoño",
    "bisoñés": "bisoñé",
    "bistecs": "bistec",
    "bistortas": "bistorta",
    "bistres": "bistre",
    "bisturíes": "bisturí",
    "bisturís": "bisturí",
    "bistés": "bisté",
    "bisulca": "bisulco",
    "bisulcas": "bisulco",
    "bisulcos": "bisulco",
    "bisulfatos": "bisulfato",
    "bisulfitos": "bisulfito",
    "bisulfuros": "bisulfuro",
    "bisurcos": "bisurco",
    "bisuterías": "bisutería",
    "bisábamos": "bisar",
    "bisáis": "bisar",
    "bisáramos": "bisar",
    "bisáremos": "bisar",
    "bisásemos": "bisar",
    "bisé": "bisar",
    "biséis": "bisar",
    "bisílaba": "bisílabo",
    "bisílabas": "bisílabo",
    "bisílabos": "bisílabo",
    "bisó": "bisar",
    "bitaduras": "bitadura",
    "bitangentes": "bitangente",
    "bitartratos": "bitartrato",
    "bitas": "bita",
    "bitemporales": "bitemporal",
    "bites": "bit",
    "bitionoles": "bitionol",
    "bitonales": "bitonal",
    "bitonga": "bitongo",
    "bitongas": "bitongo",
    "bitongos": "bitongo",
    "bitoques": "bitoque",
    "bits": "bit",
    "bituminada": "bituminado",
    "bituminadas": "bituminado",
    "bituminadoras": "bituminadora",
    "bituminados": "bituminado",
    "bituminosa": "bituminoso",
    "bituminosas": "bituminoso",
    "bituminosos": "bituminoso",
    "bitácoras": "bitácora",
    "bitúmenes": "bitumen",
    "biunívoca": "biunívoco",
    "biunívocas": "biunívoco",
    "biunívocos": "biunívoco",
    "biuretos": "biureto",
    "bivalencias": "bivalencia",
    "bivalentes": "bivalente",
    "bivalva": "bivalvo",
    "bivalvas": "bivalvo",
    "bivalvos": "bivalvo",
    "bivariables": "bivariable",
    "bivariantes": "bivariante",
    "bivios": "bivio",
    "biyecciones": "biyección",
    "biyecciónes": "biyección",
    "biyectiva": "biyectivo",
    "biyectivas": "biyectivo",
    "biyectivos": "biyectivo",
    "bizantina": "bizantino",
    "bizantinas": "bizantino",
    "bizantinismos": "bizantinismo",
    "bizantinos": "bizantino",
    "bizarra": "bizarro",
    "bizarras": "bizarro",
    "bizarrones": "bizarrón",
    "bizarros": "bizarro",
    "bizarrías": "bizarría",
    "bizazas": "bizaza",
    "bizca": "bizco",
    "bizcaba": "bizcar",
    "bizcabais": "bizcar",
    "bizcaban": "bizcar",
    "bizcabas": "bizcar",
    "bizcad": "bizcar",
    "bizcada": "bizcar",
    "bizcadas": "bizcar",
    "bizcado": "bizcar",
    "bizcados": "bizcar",
    "bizcamos": "bizcar",
    "bizcan": "bizcar",
    "bizcando": "bizcar",
    "bizcara": "bizcar",
    "bizcarais": "bizcar",
    "bizcaran": "bizcar",
    "bizcaras": "bizcar",
    "bizcare": "bizcar",
    "bizcareis": "bizcar",
    "bizcaremos": "bizcar",
    "bizcaren": "bizcar",
    "bizcares": "bizcar",
    "bizcaron": "bizcar",
    "bizcará": "bizcar",
    "bizcarán": "bizcar",
    "bizcarás": "bizcar",
    "bizcaré": "bizcar",
    "bizcaréis": "bizcar",
    "bizcaría": "bizcar",
    "bizcaríais": "bizcar",
    "bizcaríamos": "bizcar",
    "bizcarían": "bizcar",
    "bizcarías": "bizcar",
    "bizcas": "bizco",
    "bizcase": "bizcar",
    "bizcaseis": "bizcar",
    "bizcasen": "bizcar",
    "bizcases": "bizcar",
    "bizcaste": "bizcar",
    "bizcasteis": "bizcar",
    "bizco": "bizcar",
    "bizcocha": "bizcochar",
    "bizcochaba": "bizcochar",
    "bizcochabais": "bizcochar",
    "bizcochaban": "bizcochar",
    "bizcochabas": "bizcochar",
    "bizcochad": "bizcochar",
    "bizcochada": "bizcochar",
    "bizcochadas": "bizcochar",
    "bizcochado": "bizcochar",
    "bizcochados": "bizcochar",
    "bizcochamos": "bizcochar",
    "bizcochan": "bizcochar",
    "bizcochando": "bizcochar",
    "bizcochara": "bizcochar",
    "bizcocharais": "bizcochar",
    "bizcocharan": "bizcochar",
    "bizcocharas": "bizcochar",
    "bizcochare": "bizcochar",
    "bizcochareis": "bizcochar",
    "bizcocharemos": "bizcochar",
    "bizcocharen": "bizcochar",
    "bizcochares": "bizcochar",
    "bizcocharon": "bizcochar",
    "bizcochará": "bizcochar",
    "bizcocharán": "bizcochar",
    "bizcocharás": "bizcochar",
    "bizcocharé": "bizcochar",
    "bizcocharéis": "bizcochar",
    "bizcocharía": "bizcochar",
    "bizcocharíais": "bizcochar",
    "bizcocharíamos": "bizcochar",
    "bizcocharían": "bizcochar",
    "bizcocharías": "bizcochar",
    "bizcochas": "bizcochar",
    "bizcochase": "bizcochar",
    "bizcochaseis": "bizcochar",
    "bizcochasen": "bizcochar",
    "bizcochases": "bizcochar",
    "bizcochaste": "bizcochar",
    "bizcochasteis": "bizcochar",
    "bizcoche": "bizcochar",
    "bizcochemos": "bizcochar",
    "bizcochen": "bizcochar",
    "bizcoches": "bizcochar",
    "bizcocho": "bizcochar",
    "bizcochos": "bizcocho",
    "bizcochábamos": "bizcochar",
    "bizcocháis": "bizcochar",
    "bizcocháramos": "bizcochar",
    "bizcocháremos": "bizcochar",
    "bizcochásemos": "bizcochar",
    "bizcoché": "bizcochar",
    "bizcochéis": "bizcochar",
    "bizcochó": "bizcochar",
    "bizcornea": "bizcornear",
    "bizcorneaba": "bizcornear",
    "bizcorneabais": "bizcornear",
    "bizcorneaban": "bizcornear",
    "bizcorneabas": "bizcornear",
    "bizcornead": "bizcornear",
    "bizcorneada": "bizcornear",
    "bizcorneadas": "bizcornear",
    "bizcorneado": "bizcornear",
    "bizcorneados": "bizcornear",
    "bizcorneamos": "bizcornear",
    "bizcornean": "bizcornear",
    "bizcorneando": "bizcornear",
    "bizcorneara": "bizcornear",
    "bizcornearais": "bizcornear",
    "bizcornearan": "bizcornear",
    "bizcornearas": "bizcornear",
    "bizcorneare": "bizcornear",
    "bizcorneareis": "bizcornear",
    "bizcornearemos": "bizcornear",
    "bizcornearen": "bizcornear",
    "bizcorneares": "bizcornear",
    "bizcornearon": "bizcornear",
    "bizcorneará": "bizcornear",
    "bizcornearán": "bizcornear",
    "bizcornearás": "bizcornear",
    "bizcornearé": "bizcornear",
    "bizcornearéis": "bizcornear",
    "bizcornearía": "bizcornear",
    "bizcornearíais": "bizcornear",
    "bizcornearíamos": "bizcornear",
    "bizcornearían": "bizcornear",
    "bizcornearías": "bizcornear",
    "bizcorneas": "bizcornear",
    "bizcornease": "bizcornear",
    "bizcorneaseis": "bizcornear",
    "bizcorneasen": "bizcornear",
    "bizcorneases": "bizcornear",
    "bizcorneaste": "bizcornear",
    "bizcorneasteis": "bizcornear",
    "bizcornee": "bizcornear",
    "bizcorneemos": "bizcornear",
    "bizcorneen": "bizcornear",
    "bizcornees": "bizcornear",
    "bizcorneo": "bizcornear",
    "bizcorneábamos": "bizcornear",
    "bizcorneáis": "bizcornear",
    "bizcorneáramos": "bizcornear",
    "bizcorneáremos": "bizcornear",
    "bizcorneásemos": "bizcornear",
    "bizcorneé": "bizcornear",
    "bizcorneéis": "bizcornear",
    "bizcorneó": "bizcornear",
    "bizcos": "bizco",
    "bizcotelas": "bizcotela",
    "bizcábamos": "bizcar",
    "bizcáis": "bizcar",
    "bizcáramos": "bizcar",
    "bizcáremos": "bizcar",
    "bizcásemos": "bizcar",
    "bizcó": "bizcar",
    "bizmas": "bizma",
    "biznagas": "biznaga",
    "biznas": "bizna",
    "biznieta": "biznieto",
    "biznietas": "biznieto",
    "biznietos": "biznieto",
    "bizque": "bizcar",
    "bizquea": "bizquear",
    "bizqueaba": "bizquear",
    "bizqueabais": "bizquear",
    "bizqueaban": "bizquear",
    "bizqueabas": "bizquear",
    "bizquead": "bizquear",
    "bizqueada": "bizquear",
    "bizqueadas": "bizquear",
    "bizqueado": "bizquear",
    "bizqueados": "bizquear",
    "bizqueamos": "bizquear",
    "bizquean": "bizquear",
    "bizqueando": "bizquear",
    "bizqueara": "bizquear",
    "bizquearais": "bizquear",
    "bizquearan": "bizquear",
    "bizquearas": "bizquear",
    "bizqueare": "bizquear",
    "bizqueareis": "bizquear",
    "bizquearemos": "bizquear",
    "bizquearen": "bizquear",
    "bizqueares": "bizquear",
    "bizquearon": "bizquear",
    "bizqueará": "bizquear",
    "bizquearán": "bizquear",
    "bizquearás": "bizquear",
    "bizquearé": "bizquear",
    "bizquearéis": "bizquear",
    "bizquearía": "bizquear",
    "bizquearíais": "bizquear",
    "bizquearíamos": "bizquear",
    "bizquearían": "bizquear",
    "bizquearías": "bizquear",
    "bizqueas": "bizquear",
    "bizquease": "bizquear",
    "bizqueaseis": "bizquear",
    "bizqueasen": "bizquear",
    "bizqueases": "bizquear",
    "bizqueaste": "bizquear",
    "bizqueasteis": "bizquear",
    "bizquee": "bizquear",
    "bizqueemos": "bizquear",
    "bizqueen": "bizquear",
    "bizquees": "bizquear",
    "bizquemos": "bizcar",
    "bizquen": "bizcar",
    "bizqueo": "bizquear",
    "bizqueos": "bizqueo",
    "bizqueras": "bizquera",
    "bizques": "bizcar",
    "bizqueábamos": "bizquear",
    "bizqueáis": "bizquear",
    "bizqueáramos": "bizquear",
    "bizqueáremos": "bizquear",
    "bizqueásemos": "bizquear",
    "bizqueé": "bizquear",
    "bizqueéis": "bizquear",
    "bizqueó": "bizquear",
    "bizqué": "bizcar",
    "bizquéis": "bizcar",
    "biáxica": "biáxico",
    "biáxicas": "biáxico",
    "biáxicos": "biáxico",
    "biógena": "biógeno",
    "biógenas": "biógeno",
    "biógenos": "biógeno",
    "biógrafa": "biógrafo",
    "biógrafas": "biógrafo",
    "biógrafos": "biógrafo",
    "bióloga": "biólogo",
    "biólogas": "biólogo",
    "biólogos": "biólogo",
    "biónica": "biónico",
    "biónicas": "biónico",
    "biónicos": "biónico",
    "biótica": "biótico",
    "bióticas": "biótico",
    "bióticos": "biótico",
    "biótopos": "biótopo",
    "bióxidos": "bióxido",
    "blanca": "blanco",
    "blancas": "blanco",
    "blancores": "blancor",
    "blancos": "blanco",
    "blancuras": "blancura",
    "blancuzca": "blancuzco",
    "blancuzcas": "blancuzco",
    "blancuzcos": "blancuzco",
    "blanda": "blando",
    "blandamos": "blandir",
    "blandan": "blandir",
    "blandas": "blando",
    "blande": "blandir",
    "blandea": "blandear",
    "blandeaba": "blandear",
    "blandeabais": "blandear",
    "blandeaban": "blandear",
    "blandeabas": "blandear",
    "blandead": "blandear",
    "blandeada": "blandear",
    "blandeadas": "blandear",
    "blandeado": "blandear",
    "blandeados": "blandear",
    "blandeamos": "blandear",
    "blandean": "blandear",
    "blandeando": "blandear",
    "blandeara": "blandear",
    "blandearais": "blandear",
    "blandearan": "blandear",
    "blandearas": "blandear",
    "blandeare": "blandear",
    "blandeareis": "blandear",
    "blandearemos": "blandear",
    "blandearen": "blandear",
    "blandeares": "blandear",
    "blandearon": "blandear",
    "blandeará": "blandear",
    "blandearán": "blandear",
    "blandearás": "blandear",
    "blandearé": "blandear",
    "blandearéis": "blandear",
    "blandearía": "blandear",
    "blandearíais": "blandear",
    "blandearíamos": "blandear",
    "blandearían": "blandear",
    "blandearías": "blandear",
    "blandeas": "blandear",
    "blandease": "blandear",
    "blandeaseis": "blandear",
    "blandeasen": "blandear",
    "blandeases": "blandear",
    "blandeaste": "blandear",
    "blandeasteis": "blandear",
    "blandeces": "blandez",
    "blandee": "blandear",
    "blandeemos": "blandear",
    "blandeen": "blandear",
    "blandees": "blandear",
    "blanden": "blandir",
    "blandenguerías": "blandenguería",
    "blandengues": "blandengue",
    "blandeo": "blandear",
    "blandes": "blandir",
    "blandeábamos": "blandear",
    "blandeáis": "blandear",
    "blandeáramos": "blandear",
    "blandeáremos": "blandear",
    "blandeásemos": "blandear",
    "blandeé": "blandear",
    "blandeéis": "blandear",
    "blandeó": "blandear",
    "blandicias": "blandicia",
    "blandid": "blandir",
    "blandida": "blandir",
    "blandidas": "blandir",
    "blandido": "blandir",
    "blandidos": "blandir",
    "blandiendo": "blandir",
    "blandiera": "blandir",
    "blandierais": "blandir",
    "blandieran": "blandir",
    "blandieras": "blandir",
    "blandiere": "blandir",
    "blandiereis": "blandir",
    "blandieren": "blandir",
    "blandieres": "blandir",
    "blandieron": "blandir",
    "blandiese": "blandir",
    "blandieseis": "blandir",
    "blandiesen": "blandir",
    "blandieses": "blandir",
    "blandimos": "blandir",
    "blandiremos": "blandir",
    "blandirá": "blandir",
    "blandirán": "blandir",
    "blandirás": "blandir",
    "blandiré": "blandir",
    "blandiréis": "blandir",
    "blandiría": "blandir",
    "blandiríais": "blandir",
    "blandiríamos": "blandir",
    "blandirían": "blandir",
    "blandirías": "blandir",
    "blandiste": "blandir",
    "blandisteis": "blandir",
    "blandiéramos": "blandir",
    "blandiéremos": "blandir",
    "blandiésemos": "blandir",
    "blandió": "blandir",
    "blando": "blandir",
    "blandones": "blandón",
    "blandos": "blando",
    "blanducha": "blanducho",
    "blanduchas": "blanducho",
    "blanduchos": "blanducho",
    "blanduras": "blandura",
    "blanduzca": "blanduzco",
    "blanduzcas": "blanduzco",
    "blanduzcos": "blanduzco",
    "blandáis": "blandir",
    "blandí": "blandir",
    "blandía": "blandir",
    "blandíais": "blandir",
    "blandíamos": "blandir",
    "blandían": "blandir",
    "blandías": "blandir",
    "blandís": "blandir",
    "blanquea": "blanquear",
    "blanqueaba": "blanquear",
    "blanqueabais": "blanquear",
    "blanqueaban": "blanquear",
    "blanqueabas": "blanquear",
    "blanqueaciones": "blanqueación",
    "blanquead": "blanquear",
    "blanqueada": "blanquear",
    "blanqueadas": "blanquear",
    "blanqueado": "blanquear",
    "blanqueadores": "blanqueador",
    "blanqueados": "blanquear",
    "blanqueaduras": "blanqueadura",
    "blanqueamientos": "blanqueamiento",
    "blanqueamos": "blanquear",
    "blanquean": "blanquear",
    "blanqueando": "blanquear",
    "blanqueantes": "blanqueante",
    "blanqueara": "blanquear",
    "blanquearais": "blanquear",
    "blanquearan": "blanquear",
    "blanquearas": "blanquear",
    "blanqueare": "blanquear",
    "blanqueareis": "blanquear",
    "blanquearemos": "blanquear",
    "blanquearen": "blanquear",
    "blanqueares": "blanquear",
    "blanquearon": "blanquear",
    "blanqueará": "blanquear",
    "blanquearán": "blanquear",
    "blanquearás": "blanquear",
    "blanquearé": "blanquear",
    "blanquearéis": "blanquear",
    "blanquearía": "blanquear",
    "blanquearíais": "blanquear",
    "blanquearíamos": "blanquear",
    "blanquearían": "blanquear",
    "blanquearías": "blanquear",
    "blanqueas": "blanquear",
    "blanquease": "blanquear",
    "blanqueaseis": "blanquear",
    "blanqueasen": "blanquear",
    "blanqueases": "blanquear",
    "blanqueaste": "blanquear",
    "blanqueasteis": "blanquear",
    "blanquece": "blanquecer",
    "blanqueced": "blanquecer",
    "blanquecedora": "blanquecedor",
    "blanquecedoras": "blanquecedor",
    "blanquecedores": "blanquecedor",
    "blanquecemos": "blanquecer",
    "blanquecen": "blanquecer",
    "blanqueceremos": "blanquecer",
    "blanquecerá": "blanquecer",
    "blanquecerán": "blanquecer",
    "blanquecerás": "blanquecer",
    "blanqueceré": "blanquecer",
    "blanqueceréis": "blanquecer",
    "blanquecería": "blanquecer",
    "blanqueceríais": "blanquecer",
    "blanqueceríamos": "blanquecer",
    "blanquecerían": "blanquecer",
    "blanquecerías": "blanquecer",
    "blanqueces": "blanquecer",
    "blanquecida": "blanquecer",
    "blanquecidas": "blanquecer",
    "blanquecido": "blanquecer",
    "blanquecidos": "blanquecer",
    "blanqueciendo": "blanquecer",
    "blanqueciera": "blanquecer",
    "blanquecierais": "blanquecer",
    "blanquecieran": "blanquecer",
    "blanquecieras": "blanquecer",
    "blanqueciere": "blanquecer",
    "blanqueciereis": "blanquecer",
    "blanquecieren": "blanquecer",
    "blanquecieres": "blanquecer",
    "blanquecieron": "blanquecer",
    "blanqueciese": "blanquecer",
    "blanquecieseis": "blanquecer",
    "blanqueciesen": "blanquecer",
    "blanquecieses": "blanquecer",
    "blanquecimientos": "blanquecimiento",
    "blanquecimos": "blanquecer",
    "blanquecina": "blanquecino",
    "blanquecinas": "blanquecino",
    "blanquecinos": "blanquecino",
    "blanqueciste": "blanquecer",
    "blanquecisteis": "blanquecer",
    "blanqueciéramos": "blanquecer",
    "blanqueciéremos": "blanquecer",
    "blanqueciésemos": "blanquecer",
    "blanqueció": "blanquecer",
    "blanquecéis": "blanquecer",
    "blanquecí": "blanquecer",
    "blanquecía": "blanquecer",
    "blanquecíais": "blanquecer",
    "blanquecíamos": "blanquecer",
    "blanquecían": "blanquecer",
    "blanquecías": "blanquecer",
    "blanquee": "blanquear",
    "blanqueemos": "blanquear",
    "blanqueen": "blanquear",
    "blanquees": "blanquear",
    "blanqueo": "blanquear",
    "blanqueos": "blanqueo",
    "blanquetes": "blanquete",
    "blanquezca": "blanquecer",
    "blanquezcamos": "blanquecer",
    "blanquezcan": "blanquecer",
    "blanquezcas": "blanquecer",
    "blanquezco": "blanquecer",
    "blanquezcáis": "blanquecer",
    "blanqueábamos": "blanquear",
    "blanqueáis": "blanquear",
    "blanqueáramos": "blanquear",
    "blanqueáremos": "blanquear",
    "blanqueásemos": "blanquear",
    "blanqueé": "blanquear",
    "blanqueéis": "blanquear",
    "blanqueó": "blanquear",
    "blanquiazules": "blanquiazul",
    "blanquiciones": "blanquición",
    "blanquilla": "blanquillo",
    "blanquillas": "blanquillo",
    "blanquillos": "blanquillo",
    "blanquimentos": "blanquimento",
    "blanquimientos": "blanquimiento",
    "blanquinegra": "blanquinegro",
    "blanquinegras": "blanquinegro",
    "blanquinegros": "blanquinegro",
    "blanquinosa": "blanquinoso",
    "blanquinosas": "blanquinoso",
    "blanquinosos": "blanquinoso",
    "blanquizales": "blanquizal",
    "blanquizares": "blanquizar",
    "blasfema": "blasfemo",
    "blasfemaba": "blasfemar",
    "blasfemabais": "blasfemar",
    "blasfemaban": "blasfemar",
    "blasfemabas": "blasfemar",
    "blasfemad": "blasfemar",
    "blasfemada": "blasfemar",
    "blasfemadas": "blasfemar",
    "blasfemado": "blasfemar",
    "blasfemadora": "blasfemador",
    "blasfemadoras": "blasfemador",
    "blasfemadores": "blasfemador",
    "blasfemados": "blasfemar",
    "blasfemamos": "blasfemar",
    "blasfeman": "blasfemar",
    "blasfemando": "blasfemar",
    "blasfemara": "blasfemar",
    "blasfemarais": "blasfemar",
    "blasfemaran": "blasfemar",
    "blasfemaras": "blasfemar",
    "blasfemare": "blasfemar",
    "blasfemareis": "blasfemar",
    "blasfemaremos": "blasfemar",
    "blasfemaren": "blasfemar",
    "blasfemares": "blasfemar",
    "blasfemaron": "blasfemar",
    "blasfemará": "blasfemar",
    "blasfemarán": "blasfemar",
    "blasfemarás": "blasfemar",
    "blasfemaré": "blasfemar",
    "blasfemaréis": "blasfemar",
    "blasfemaría": "blasfemar",
    "blasfemaríais": "blasfemar",
    "blasfemaríamos": "blasfemar",
    "blasfemarían": "blasfemar",
    "blasfemarías": "blasfemar",
    "blasfemas": "blasfemo",
    "blasfemase": "blasfemar",
    "blasfemaseis": "blasfemar",
    "blasfemasen": "blasfemar",
    "blasfemases": "blasfemar",
    "blasfemaste": "blasfemar",
    "blasfemasteis": "blasfemar",
    "blasfematoria": "blasfematorio",
    "blasfematorias": "blasfematorio",
    "blasfematorios": "blasfematorio",
    "blasfeme": "blasfemar",
    "blasfememos": "blasfemar",
    "blasfemen": "blasfemar",
    "blasfemes": "blasfemar",
    "blasfemias": "blasfemia",
    "blasfemo": "blasfemar",
    "blasfemos": "blasfemo",
    "blasfemábamos": "blasfemar",
    "blasfemáis": "blasfemar",
    "blasfemáramos": "blasfemar",
    "blasfemáremos": "blasfemar",
    "blasfemásemos": "blasfemar",
    "blasfemé": "blasfemar",
    "blasfeméis": "blasfemar",
    "blasfemó": "blasfemar",
    "blasona": "blasonar",
    "blasonaba": "blasonar",
    "blasonabais": "blasonar",
    "blasonaban": "blasonar",
    "blasonabas": "blasonar",
    "blasonad": "blasonar",
    "blasonada": "blasonar",
    "blasonadas": "blasonar",
    "blasonado": "blasonar",
    "blasonadora": "blasonador",
    "blasonadoras": "blasonador",
    "blasonadores": "blasonador",
    "blasonados": "blasonar",
    "blasonamos": "blasonar",
    "blasonan": "blasonar",
    "blasonando": "blasonar",
    "blasonara": "blasonar",
    "blasonarais": "blasonar",
    "blasonaran": "blasonar",
    "blasonaras": "blasonar",
    "blasonare": "blasonar",
    "blasonareis": "blasonar",
    "blasonaremos": "blasonar",
    "blasonaren": "blasonar",
    "blasonares": "blasonar",
    "blasonaron": "blasonar",
    "blasonará": "blasonar",
    "blasonarán": "blasonar",
    "blasonarás": "blasonar",
    "blasonaré": "blasonar",
    "blasonaréis": "blasonar",
    "blasonaría": "blasonar",
    "blasonaríais": "blasonar",
    "blasonaríamos": "blasonar",
    "blasonarían": "blasonar",
    "blasonarías": "blasonar",
    "blasonas": "blasonar",
    "blasonase": "blasonar",
    "blasonaseis": "blasonar",
    "blasonasen": "blasonar",
    "blasonases": "blasonar",
    "blasonaste": "blasonar",
    "blasonasteis": "blasonar",
    "blasone": "blasonar",
    "blasonemos": "blasonar",
    "blasonen": "blasonar",
    "blasonerías": "blasonería",
    "blasones": "blasonar",
    "blasonistas": "blasonista",
    "blasono": "blasonar",
    "blasonábamos": "blasonar",
    "blasonáis": "blasonar",
    "blasonáramos": "blasonar",
    "blasonáremos": "blasonar",
    "blasonásemos": "blasonar",
    "blasoné": "blasonar",
    "blasonéis": "blasonar",
    "blasonó": "blasonar",
    "blastemas": "blastema",
    "blastoceles": "blastocele",
    "blastocelos": "blastocelo",
    "blastocistos": "blastocisto",
    "blastodermos": "blastodermo",
    "blastodérmica": "blastodérmico",
    "blastodérmicas": "blastodérmico",
    "blastodérmicos": "blastodérmico",
    "blastoides": "blastoide",
    "blastoporos": "blastoporo",
    "blastosferas": "blastosfera",
    "blastulaciones": "blastulación",
    "blastómeros": "blastómero",
    "blastóporos": "blastóporo",
    "blasónica": "blasónico",
    "blasónicas": "blasónico",
    "blasónicos": "blasónico",
    "blaugranas": "blaugrana",
    "bledos": "bledo",
    "blefarectomías": "blefarectomía",
    "blefarismos": "blefarismo",
    "blefarocalasias": "blefarocalasia",
    "blefaroclonos": "blefaroclono",
    "blefaroespasmos": "blefaroespasmo",
    "blefaroplastias": "blefaroplastia",
    "blefaroplastos": "blefaroplasto",
    "blefaroplejías": "blefaroplejía",
    "blefarorrafias": "blefarorrafia",
    "blefarosfinterectomías": "blefarosfinterectomía",
    "blefarotomías": "blefarotomía",
    "blefaróstatos": "blefaróstato",
    "blefárica": "blefárico",
    "blefáricas": "blefárico",
    "blefáricos": "blefárico",
    "blendas": "blenda",
    "blenoftalmías": "blenoftalmía",
    "blenorragias": "blenorragia",
    "blenorreas": "blenorrea",
    "blenorrágica": "blenorrágico",
    "blenorrágicas": "blenorrágico",
    "blenorrágicos": "blenorrágico",
    "blinda": "blindar",
    "blindaba": "blindar",
    "blindabais": "blindar",
    "blindaban": "blindar",
    "blindabas": "blindar",
    "blindad": "blindar",
    "blindada": "blindar",
    "blindadas": "blindar",
    "blindado": "blindar",
    "blindados": "blindar",
    "blindajes": "blindaje",
    "blindamos": "blindar",
    "blindan": "blindar",
    "blindando": "blindar",
    "blindara": "blindar",
    "blindarais": "blindar",
    "blindaran": "blindar",
    "blindaras": "blindar",
    "blindare": "blindar",
    "blindareis": "blindar",
    "blindaremos": "blindar",
    "blindaren": "blindar",
    "blindares": "blindar",
    "blindaron": "blindar",
    "blindará": "blindar",
    "blindarán": "blindar",
    "blindarás": "blindar",
    "blindaré": "blindar",
    "blindaréis": "blindar",
    "blindaría": "blindar",
    "blindaríais": "blindar",
    "blindaríamos": "blindar",
    "blindarían": "blindar",
    "blindarías": "blindar",
    "blindas": "blindar",
    "blindase": "blindar",
    "blindaseis": "blindar",
    "blindasen": "blindar",
    "blindases": "blindar",
    "blindaste": "blindar",
    "blindasteis": "blindar",
    "blinde": "blindar",
    "blindemos": "blindar",
    "blinden": "blindar",
    "blindes": "blindar",
    "blindo": "blindar",
    "blindábamos": "blindar",
    "blindáis": "blindar",
    "blindáramos": "blindar",
    "blindáremos": "blindar",
    "blindásemos": "blindar",
    "blindé": "blindar",
    "blindéis": "blindar",
    "blindó": "blindar",
    "bloca": "blocar",
    "blocaba": "blocar",
    "blocabais": "blocar",
    "blocaban": "blocar",
    "blocabas": "blocar",
    "blocad": "blocar",
    "blocada": "blocar",
    "blocadas": "blocar",
    "blocado": "blocar",
    "blocadora": "blocador",
    "blocadoras": "blocador",
    "blocadores": "blocador",
    "blocados": "blocar",
    "blocajes": "blocaje",
    "blocamos": "blocar",
    "blocan": "blocar",
    "blocando": "blocar",
    "blocaos": "blocao",
    "blocara": "blocar",
    "blocarais": "blocar",
    "blocaran": "blocar",
    "blocaras": "blocar",
    "blocare": "blocar",
    "blocareis": "blocar",
    "blocaremos": "blocar",
    "blocaren": "blocar",
    "blocares": "blocar",
    "blocaron": "blocar",
    "blocará": "blocar",
    "blocarán": "blocar",
    "blocarás": "blocar",
    "blocaré": "blocar",
    "blocaréis": "blocar",
    "blocaría": "blocar",
    "blocaríais": "blocar",
    "blocaríamos": "blocar",
    "blocarían": "blocar",
    "blocarías": "blocar",
    "blocas": "blocar",
    "blocase": "blocar",
    "blocaseis": "blocar",
    "blocasen": "blocar",
    "blocases": "blocar",
    "blocaste": "blocar",
    "blocasteis": "blocar",
    "bloco": "blocar",
    "blocs": "bloc",
    "blocábamos": "blocar",
    "blocáis": "blocar",
    "blocáramos": "blocar",
    "blocáremos": "blocar",
    "blocásemos": "blocar",
    "blocó": "blocar",
    "blondas": "blonda",
    "blondinas": "blondina",
    "blondines": "blondín",
    "bloque": "blocar",
    "bloquea": "bloquear",
    "bloqueaba": "bloquear",
    "bloqueabais": "bloquear",
    "bloqueaban": "bloquear",
    "bloqueabas": "bloquear",
    "bloqueables": "bloqueable",
    "bloquead": "bloquear",
    "bloqueada": "bloquear",
    "bloqueadas": "bloquear",
    "bloqueado": "bloquear",
    "bloqueadora": "bloqueador",
    "bloqueadoras": "bloqueador",
    "bloqueadores": "bloqueador",
    "bloqueados": "bloquear",
    "bloqueamos": "bloquear",
    "bloquean": "bloquear",
    "bloqueando": "bloquear",
    "bloqueantes": "bloqueante",
    "bloqueara": "bloquear",
    "bloquearais": "bloquear",
    "bloquearan": "bloquear",
    "bloquearas": "bloquear",
    "bloqueare": "bloquear",
    "bloqueareis": "bloquear",
    "bloquearemos": "bloquear",
    "bloquearen": "bloquear",
    "bloqueares": "bloquear",
    "bloquearon": "bloquear",
    "bloqueará": "bloquear",
    "bloquearán": "bloquear",
    "bloquearás": "bloquear",
    "bloquearé": "bloquear",
    "bloquearéis": "bloquear",
    "bloquearía": "bloquear",
    "bloquearíais": "bloquear",
    "bloquearíamos": "bloquear",
    "bloquearían": "bloquear",
    "bloquearías": "bloquear",
    "bloqueas": "bloquear",
    "bloquease": "bloquear",
    "bloqueaseis": "bloquear",
    "bloqueasen": "bloquear",
    "bloqueases": "bloquear",
    "bloqueaste": "bloquear",
    "bloqueasteis": "bloquear",
    "bloquee": "bloquear",
    "bloqueemos": "bloquear",
    "bloqueen": "bloquear",
    "bloquees": "bloquear",
    "bloquemos": "blocar",
    "bloquen": "blocar",
    "bloqueo": "bloquear",
    "bloqueos": "bloqueo",
    "bloques": "bloque",
    "bloqueábamos": "bloquear",
    "bloqueáis": "bloquear",
    "bloqueáramos": "bloquear",
    "bloqueáremos": "bloquear",
    "bloqueásemos": "bloquear",
    "bloqueé": "bloquear",
    "bloqueéis": "bloquear",
    "bloqueó": "bloquear",
    "bloqué": "blocar",
    "bloquéis": "blocar",
    "bluffs": "bluff",
    "blusas": "blusa",
    "blusones": "blusón",
    "blástica": "blástico",
    "blásticas": "blástico",
    "blásticos": "blástico",
    "blástulas": "blástula",
    "blátidos": "blátido",
    "blénidos": "blénido",
    "boalajes": "boalaje",
    "boalares": "boalar",
    "boardillas": "boardilla",
    "boas": "boa",
    "boatos": "boato",
    "boba": "bobo",
    "bobadas": "bobada",
    "bobalicona": "bobalicón",
    "bobaliconas": "bobalicón",
    "bobalicones": "bobalicón",
    "bobanguis": "bobangui",
    "bobas": "bobo",
    "bobateles": "bobatel",
    "bobea": "bobear",
    "bobeaba": "bobear",
    "bobeabais": "bobear",
    "bobeaban": "bobear",
    "bobeabas": "bobear",
    "bobead": "bobear",
    "bobeada": "bobear",
    "bobeadas": "bobear",
    "bobeado": "bobear",
    "bobeados": "bobear",
    "bobeamos": "bobear",
    "bobean": "bobear",
    "bobeando": "bobear",
    "bobeara": "bobear",
    "bobearais": "bobear",
    "bobearan": "bobear",
    "bobearas": "bobear",
    "bobeare": "bobear",
    "bobeareis": "bobear",
    "bobearemos": "bobear",
    "bobearen": "bobear",
    "bobeares": "bobear",
    "bobearon": "bobear",
    "bobeará": "bobear",
    "bobearán": "bobear",
    "bobearás": "bobear",
    "bobearé": "bobear",
    "bobearéis": "bobear",
    "bobearía": "bobear",
    "bobearíais": "bobear",
    "bobearíamos": "bobear",
    "bobearían": "bobear",
    "bobearías": "bobear",
    "bobeas": "bobear",
    "bobease": "bobear",
    "bobeaseis": "bobear",
    "bobeasen": "bobear",
    "bobeases": "bobear",
    "bobeaste": "bobear",
    "bobeasteis": "bobear",
    "bobee": "bobear",
    "bobeemos": "bobear",
    "bobeen": "bobear",
    "bobees": "bobear",
    "bobeo": "bobear",
    "boberías": "bobería",
    "bobeábamos": "bobear",
    "bobeáis": "bobear",
    "bobeáramos": "bobear",
    "bobeáremos": "bobear",
    "bobeásemos": "bobear",
    "bobeé": "bobear",
    "bobeéis": "bobear",
    "bobeó": "bobear",
    "bobillos": "bobillo",
    "bobina": "bobinar",
    "bobinaba": "bobinar",
    "bobinabais": "bobinar",
    "bobinaban": "bobinar",
    "bobinabas": "bobinar",
    "bobinad": "bobinar",
    "bobinada": "bobinar",
    "bobinadas": "bobinar",
    "bobinado": "bobinar",
    "bobinadoras": "bobinadora",
    "bobinados": "bobinar",
    "bobinamos": "bobinar",
    "bobinan": "bobinar",
    "bobinando": "bobinar",
    "bobinara": "bobinar",
    "bobinarais": "bobinar",
    "bobinaran": "bobinar",
    "bobinaras": "bobinar",
    "bobinare": "bobinar",
    "bobinareis": "bobinar",
    "bobinaremos": "bobinar",
    "bobinaren": "bobinar",
    "bobinares": "bobinar",
    "bobinaron": "bobinar",
    "bobinará": "bobinar",
    "bobinarán": "bobinar",
    "bobinarás": "bobinar",
    "bobinaré": "bobinar",
    "bobinaréis": "bobinar",
    "bobinaría": "bobinar",
    "bobinaríais": "bobinar",
    "bobinaríamos": "bobinar",
    "bobinarían": "bobinar",
    "bobinarías": "bobinar",
    "bobinas": "bobinar",
    "bobinase": "bobinar",
    "bobinaseis": "bobinar",
    "bobinasen": "bobinar",
    "bobinases": "bobinar",
    "bobinaste": "bobinar",
    "bobinasteis": "bobinar",
    "bobine": "bobinar",
    "bobinemos": "bobinar",
    "bobinen": "bobinar",
    "bobines": "bobinar",
    "bobino": "bobinar",
    "bobinábamos": "bobinar",
    "bobináis": "bobinar",
    "bobináramos": "bobinar",
    "bobináremos": "bobinar",
    "bobinásemos": "bobinar",
    "bobiné": "bobinar",
    "bobinéis": "bobinar",
    "bobinó": "bobinar",
    "bobos": "bobo",
    "bobs": "bob",
    "bobsleighs": "bobsleigh",
    "bobática": "bobático",
    "bobáticas": "bobático",
    "bobáticos": "bobático",
    "bocacaces": "bocacaz",
    "bocacalles": "bocacalle",
    "bocadea": "bocadear",
    "bocadeaba": "bocadear",
    "bocadeabais": "bocadear",
    "bocadeaban": "bocadear",
    "bocadeabas": "bocadear",
    "bocadead": "bocadear",
    "bocadeada": "bocadear",
    "bocadeadas": "bocadear",
    "bocadeado": "bocadear",
    "bocadeados": "bocadear",
    "bocadeamos": "bocadear",
    "bocadean": "bocadear",
    "bocadeando": "bocadear",
    "bocadeara": "bocadear",
    "bocadearais": "bocadear",
    "bocadearan": "bocadear",
    "bocadearas": "bocadear",
    "bocadeare": "bocadear",
    "bocadeareis": "bocadear",
    "bocadearemos": "bocadear",
    "bocadearen": "bocadear",
    "bocadeares": "bocadear",
    "bocadearon": "bocadear",
    "bocadeará": "bocadear",
    "bocadearán": "bocadear",
    "bocadearás": "bocadear",
    "bocadearé": "bocadear",
    "bocadearéis": "bocadear",
    "bocadearía": "bocadear",
    "bocadearíais": "bocadear",
    "bocadearíamos": "bocadear",
    "bocadearían": "bocadear",
    "bocadearías": "bocadear",
    "bocadeas": "bocadear",
    "bocadease": "bocadear",
    "bocadeaseis": "bocadear",
    "bocadeasen": "bocadear",
    "bocadeases": "bocadear",
    "bocadeaste": "bocadear",
    "bocadeasteis": "bocadear",
    "bocadee": "bocadear",
    "bocadeemos": "bocadear",
    "bocadeen": "bocadear",
    "bocadees": "bocadear",
    "bocadeo": "bocadear",
    "bocadeábamos": "bocadear",
    "bocadeáis": "bocadear",
    "bocadeáramos": "bocadear",
    "bocadeáremos": "bocadear",
    "bocadeásemos": "bocadear",
    "bocadeé": "bocadear",
    "bocadeéis": "bocadear",
    "bocadeó": "bocadear",
    "bocadillerías": "bocadillería",
    "bocadillos": "bocadillo",
    "bocados": "bocado",
    "bocages": "bocage",
    "bocales": "bocal",
    "bocallaves": "bocallave",
    "bocamangas": "bocamanga",
    "bocaminas": "bocamina",
    "bocanadas": "bocanada",
    "bocanas": "bocana",
    "bocanegras": "bocanegra",
    "bocartes": "bocarte",
    "bocas": "boca",
    "bocatas": "bocata",
    "bocatejas": "bocateja",
    "bocatijeras": "bocatijera",
    "bocatomas": "bocatoma",
    "bocazos": "bocazo",
    "bocea": "bocear",
    "boceaba": "bocear",
    "boceabais": "bocear",
    "boceaban": "bocear",
    "boceabas": "bocear",
    "bocead": "bocear",
    "boceada": "bocear",
    "boceadas": "bocear",
    "boceado": "bocear",
    "boceados": "bocear",
    "boceamos": "bocear",
    "bocean": "bocear",
    "boceando": "bocear",
    "boceara": "bocear",
    "bocearais": "bocear",
    "bocearan": "bocear",
    "bocearas": "bocear",
    "boceare": "bocear",
    "boceareis": "bocear",
    "bocearemos": "bocear",
    "bocearen": "bocear",
    "boceares": "bocear",
    "bocearon": "bocear",
    "boceará": "bocear",
    "bocearán": "bocear",
    "bocearás": "bocear",
    "bocearé": "bocear",
    "bocearéis": "bocear",
    "bocearía": "bocear",
    "bocearíais": "bocear",
    "bocearíamos": "bocear",
    "bocearían": "bocear",
    "bocearías": "bocear",
    "boceas": "bocear",
    "bocease": "bocear",
    "boceaseis": "bocear",
    "boceasen": "bocear",
    "boceases": "bocear",
    "boceaste": "bocear",
    "boceasteis": "bocear",
    "bocece": "bocezar",
    "bocecemos": "bocezar",
    "bocecen": "bocezar",
    "boceces": "bocezar",
    "bocecé": "bocezar",
    "bocecéis": "bocezar",
    "bocee": "bocear",
    "boceemos": "bocear",
    "boceen": "bocear",
    "bocees": "bocear",
    "bocela": "bocelar",
    "bocelaba": "bocelar",
    "bocelabais": "bocelar",
    "bocelaban": "bocelar",
    "bocelabas": "bocelar",
    "bocelad": "bocelar",
    "bocelada": "bocelar",
    "boceladas": "bocelar",
    "bocelado": "bocelar",
    "bocelados": "bocelar",
    "bocelamos": "bocelar",
    "bocelan": "bocelar",
    "bocelando": "bocelar",
    "bocelara": "bocelar",
    "bocelarais": "bocelar",
    "bocelaran": "bocelar",
    "bocelaras": "bocelar",
    "bocelare": "bocelar",
    "bocelareis": "bocelar",
    "bocelaremos": "bocelar",
    "bocelaren": "bocelar",
    "bocelares": "bocelar",
    "bocelaron": "bocelar",
    "bocelará": "bocelar",
    "bocelarán": "bocelar",
    "bocelarás": "bocelar",
    "bocelaré": "bocelar",
    "bocelaréis": "bocelar",
    "bocelaría": "bocelar",
    "bocelaríais": "bocelar",
    "bocelaríamos": "bocelar",
    "bocelarían": "bocelar",
    "bocelarías": "bocelar",
    "bocelas": "bocelar",
    "bocelase": "bocelar",
    "bocelaseis": "bocelar",
    "bocelasen": "bocelar",
    "bocelases": "bocelar",
    "bocelaste": "bocelar",
    "bocelasteis": "bocelar",
    "bocele": "bocelar",
    "bocelemos": "bocelar",
    "bocelen": "bocelar",
    "boceles": "bocelar",
    "bocelo": "bocelar",
    "bocelones": "bocelón",
    "bocelábamos": "bocelar",
    "boceláis": "bocelar",
    "boceláramos": "bocelar",
    "boceláremos": "bocelar",
    "bocelásemos": "bocelar",
    "bocelé": "bocelar",
    "boceléis": "bocelar",
    "boceló": "bocelar",
    "boceo": "bocear",
    "boceos": "boceo",
    "boceras": "bocera",
    "bocetistas": "bocetista",
    "bocetos": "boceto",
    "boceza": "bocezar",
    "bocezaba": "bocezar",
    "bocezabais": "bocezar",
    "bocezaban": "bocezar",
    "bocezabas": "bocezar",
    "bocezad": "bocezar",
    "bocezada": "bocezar",
    "bocezadas": "bocezar",
    "bocezado": "bocezar",
    "bocezados": "bocezar",
    "bocezamos": "bocezar",
    "bocezan": "bocezar",
    "bocezando": "bocezar",
    "bocezara": "bocezar",
    "bocezarais": "bocezar",
    "bocezaran": "bocezar",
    "bocezaras": "bocezar",
    "bocezare": "bocezar",
    "bocezareis": "bocezar",
    "bocezaremos": "bocezar",
    "bocezaren": "bocezar",
    "bocezares": "bocezar",
    "bocezaron": "bocezar",
    "bocezará": "bocezar",
    "bocezarán": "bocezar",
    "bocezarás": "bocezar",
    "bocezaré": "bocezar",
    "bocezaréis": "bocezar",
    "bocezaría": "bocezar",
    "bocezaríais": "bocezar",
    "bocezaríamos": "bocezar",
    "bocezarían": "bocezar",
    "bocezarías": "bocezar",
    "bocezas": "bocezar",
    "bocezase": "bocezar",
    "bocezaseis": "bocezar",
    "bocezasen": "bocezar",
    "bocezases": "bocezar",
    "bocezaste": "bocezar",
    "bocezasteis": "bocezar",
    "bocezo": "bocezar",
    "bocezábamos": "bocezar",
    "bocezáis": "bocezar",
    "bocezáramos": "bocezar",
    "bocezáremos": "bocezar",
    "bocezásemos": "bocezar",
    "bocezó": "bocezar",
    "boceábamos": "bocear",
    "boceáis": "bocear",
    "boceáramos": "bocear",
    "boceáremos": "bocear",
    "boceásemos": "bocear",
    "boceé": "bocear",
    "boceéis": "bocear",
    "boceó": "bocear",
    "bocha": "bochar",
    "bochaba": "bochar",
    "bochabais": "bochar",
    "bochaban": "bochar",
    "bochabas": "bochar",
    "bochad": "bochar",
    "bochada": "bochar",
    "bochadas": "bochar",
    "bochado": "bochar",
    "bochados": "bochar",
    "bochamos": "bochar",
    "bochan": "bochar",
    "bochando": "bochar",
    "bochara": "bochar",
    "bocharais": "bochar",
    "bocharan": "bochar",
    "bocharas": "bochar",
    "bochare": "bochar",
    "bochareis": "bochar",
    "bocharemos": "bochar",
    "bocharen": "bochar",
    "bochares": "bochar",
    "bocharon": "bochar",
    "bochará": "bochar",
    "bocharán": "bochar",
    "bocharás": "bochar",
    "bocharé": "bochar",
    "bocharéis": "bochar",
    "bocharía": "bochar",
    "bocharíais": "bochar",
    "bocharíamos": "bochar",
    "bocharían": "bochar",
    "bocharías": "bochar",
    "bochas": "bochar",
    "bochase": "bochar",
    "bochaseis": "bochar",
    "bochasen": "bochar",
    "bochases": "bochar",
    "bochaste": "bochar",
    "bochasteis": "bochar",
    "boche": "bochar",
    "bochemos": "bochar",
    "bochen": "bochar",
    "boches": "boche",
    "bochinchera": "bochinchero",
    "bochincheras": "bochinchero",
    "bochincheros": "bochinchero",
    "bochinches": "bochinche",
    "bochistas": "bochista",
    "bocho": "bochar",
    "bochornos": "bochorno",
    "bochornosa": "bochornoso",
    "bochornosas": "bochornoso",
    "bochornosos": "bochornoso",
    "bochábamos": "bochar",
    "bocháis": "bochar",
    "bocháramos": "bochar",
    "bocháremos": "bochar",
    "bochásemos": "bochar",
    "boché": "bochar",
    "bochéis": "bochar",
    "bochó": "bochar",
    "bocina": "bocinar",
    "bocinaba": "bocinar",
    "bocinabais": "bocinar",
    "bocinaban": "bocinar",
    "bocinabas": "bocinar",
    "bocinad": "bocinar",
    "bocinada": "bocinar",
    "bocinadas": "bocinar",
    "bocinado": "bocinar",
    "bocinados": "bocinar",
    "bocinamos": "bocinar",
    "bocinan": "bocinar",
    "bocinando": "bocinar",
    "bocinara": "bocinar",
    "bocinarais": "bocinar",
    "bocinaran": "bocinar",
    "bocinaras": "bocinar",
    "bocinare": "bocinar",
    "bocinareis": "bocinar",
    "bocinaremos": "bocinar",
    "bocinaren": "bocinar",
    "bocinares": "bocinar",
    "bocinaron": "bocinar",
    "bocinará": "bocinar",
    "bocinarán": "bocinar",
    "bocinarás": "bocinar",
    "bocinaré": "bocinar",
    "bocinaréis": "bocinar",
    "bocinaría": "bocinar",
    "bocinaríais": "bocinar",
    "bocinaríamos": "bocinar",
    "bocinarían": "bocinar",
    "bocinarías": "bocinar",
    "bocinas": "bocinar",
    "bocinase": "bocinar",
    "bocinaseis": "bocinar",
    "bocinasen": "bocinar",
    "bocinases": "bocinar",
    "bocinaste": "bocinar",
    "bocinasteis": "bocinar",
    "bocinazos": "bocinazo",
    "bocine": "bocinar",
    "bocinemos": "bocinar",
    "bocinen": "bocinar",
    "bocines": "bocinar",
    "bocino": "bocinar",
    "bocinábamos": "bocinar",
    "bocináis": "bocinar",
    "bocináramos": "bocinar",
    "bocináremos": "bocinar",
    "bocinásemos": "bocinar",
    "bociné": "bocinar",
    "bocinéis": "bocinar",
    "bocinó": "bocinar",
    "bocios": "bocio",
    "bociosa": "bocioso",
    "bociosas": "bocioso",
    "bociosos": "bocioso",
    "bocks": "bock",
    "bocona": "bocón",
    "boconas": "bocón",
    "bocones": "bocón",
    "bocuda": "bocudo",
    "bocudas": "bocudo",
    "bocudos": "bocudo",
    "bodas": "boda",
    "bodegas": "bodega",
    "bodegonera": "bodegonero",
    "bodegoneras": "bodegonero",
    "bodegoneros": "bodegonero",
    "bodegones": "bodegón",
    "bodeguera": "bodeguero",
    "bodegueras": "bodeguero",
    "bodegueros": "bodeguero",
    "bodeguillas": "bodeguilla",
    "bodes": "bode",
    "bodhis": "bodhi",
    "bodhisattvas": "bodhisattva",
    "bodigos": "bodigo",
    "bodijos": "bodijo",
    "bodocales": "bodocal",
    "bodocazos": "bodocazo",
    "bodones": "bodón",
    "bodoqueras": "bodoquera",
    "bodoques": "bodoque",
    "bodrios": "bodrio",
    "bodys": "body",
    "boezuelos": "boezuelo",
    "bofa": "bofo",
    "bofas": "bofo",
    "bofes": "bofe",
    "bofetadas": "bofetada",
    "bofetones": "bofetón",
    "bofos": "bofo",
    "boga": "bogar",
    "bogaba": "bogar",
    "bogabais": "bogar",
    "bogaban": "bogar",
    "bogabas": "bogar",
    "bogad": "bogar",
    "bogada": "bogar",
    "bogadas": "bogar",
    "bogado": "bogar",
    "bogadora": "bogador",
    "bogadoras": "bogador",
    "bogadores": "bogador",
    "bogados": "bogar",
    "bogamos": "bogar",
    "bogan": "bogar",
    "bogando": "bogar",
    "bogara": "bogar",
    "bogarais": "bogar",
    "bogaran": "bogar",
    "bogaras": "bogar",
    "bogare": "bogar",
    "bogareis": "bogar",
    "bogaremos": "bogar",
    "bogaren": "bogar",
    "bogares": "bogar",
    "bogaron": "bogar",
    "bogará": "bogar",
    "bogarán": "bogar",
    "bogarás": "bogar",
    "bogaré": "bogar",
    "bogaréis": "bogar",
    "bogaría": "bogar",
    "bogaríais": "bogar",
    "bogaríamos": "bogar",
    "bogarían": "bogar",
    "bogarías": "bogar",
    "bogas": "bogar",
    "bogase": "bogar",
    "bogaseis": "bogar",
    "bogasen": "bogar",
    "bogases": "bogar",
    "bogaste": "bogar",
    "bogasteis": "bogar",
    "bogavantes": "bogavante",
    "bogheads": "boghead",
    "bogies": "bogie",
    "bogo": "bogar",
    "bogomila": "bogomilo",
    "bogomilas": "bogomilo",
    "bogomilismos": "bogomilismo",
    "bogomilos": "bogomilo",
    "bogotana": "bogotano",
    "bogotanas": "bogotano",
    "bogotanos": "bogotano",
    "bogotazos": "bogotazo",
    "bogue": "bogar",
    "boguemos": "bogar",
    "boguen": "bogar",
    "bogues": "bogar",
    "bogué": "bogar",
    "boguéis": "bogar",
    "bogábamos": "bogar",
    "bogáis": "bogar",
    "bogáramos": "bogar",
    "bogáremos": "bogar",
    "bogásemos": "bogar",
    "bogó": "bogar",
    "bohema": "bohemo",
    "bohemas": "bohemo",
    "bohemia": "bohemio",
    "bohemias": "bohemio",
    "bohemios": "bohemio",
    "bohemos": "bohemo",
    "bohenas": "bohena",
    "bohordos": "bohordo",
    "bohémica": "bohémico",
    "bohémicas": "bohémico",
    "bohémicos": "bohémico",
    "bohíos": "bohío",
    "boicotea": "boicotear",
    "boicoteaba": "boicotear",
    "boicoteabais": "boicotear",
    "boicoteaban": "boicotear",
    "boicoteabas": "boicotear",
    "boicotead": "boicotear",
    "boicoteada": "boicotear",
    "boicoteadas": "boicotear",
    "boicoteado": "boicotear",
    "boicoteadora": "boicoteador",
    "boicoteadoras": "boicoteador",
    "boicoteadores": "boicoteador",
    "boicoteados": "boicotear",
    "boicoteamos": "boicotear",
    "boicotean": "boicotear",
    "boicoteando": "boicotear",
    "boicoteara": "boicotear",
    "boicotearais": "boicotear",
    "boicotearan": "boicotear",
    "boicotearas": "boicotear",
    "boicoteare": "boicotear",
    "boicoteareis": "boicotear",
    "boicotearemos": "boicotear",
    "boicotearen": "boicotear",
    "boicoteares": "boicotear",
    "boicotearon": "boicotear",
    "boicoteará": "boicotear",
    "boicotearán": "boicotear",
    "boicotearás": "boicotear",
    "boicotearé": "boicotear",
    "boicotearéis": "boicotear",
    "boicotearía": "boicotear",
    "boicotearíais": "boicotear",
    "boicotearíamos": "boicotear",
    "boicotearían": "boicotear",
    "boicotearías": "boicotear",
    "boicoteas": "boicotear",
    "boicotease": "boicotear",
    "boicoteaseis": "boicotear",
    "boicoteasen": "boicotear",
    "boicoteases": "boicotear",
    "boicoteaste": "boicotear",
    "boicoteasteis": "boicotear",
    "boicotee": "boicotear",
    "boicoteemos": "boicotear",
    "boicoteen": "boicotear",
    "boicotees": "boicotear",
    "boicoteo": "boicotear",
    "boicoteos": "boicoteo",
    "boicoteábamos": "boicotear",
    "boicoteáis": "boicotear",
    "boicoteáramos": "boicotear",
    "boicoteáremos": "boicotear",
    "boicoteásemos": "boicotear",
    "boicoteé": "boicotear",
    "boicoteéis": "boicotear",
    "boicoteó": "boicotear",
    "boicots": "boicot",
    "boinas": "boina",
    "boites": "boite",
    "boja": "bojar",
    "bojaba": "bojar",
    "bojabais": "bojar",
    "bojaban": "bojar",
    "bojabas": "bojar",
    "bojad": "bojar",
    "bojada": "bojar",
    "bojadas": "bojar",
    "bojado": "bojar",
    "bojados": "bojar",
    "bojamos": "bojar",
    "bojan": "bojar",
    "bojando": "bojar",
    "bojara": "bojar",
    "bojarais": "bojar",
    "bojaran": "bojar",
    "bojaras": "bojar",
    "bojare": "bojar",
    "bojareis": "bojar",
    "bojaremos": "bojar",
    "bojaren": "bojar",
    "bojares": "bojar",
    "bojaron": "bojar",
    "bojará": "bojar",
    "bojarán": "bojar",
    "bojarás": "bojar",
    "bojaré": "bojar",
    "bojaréis": "bojar",
    "bojaría": "bojar",
    "bojaríais": "bojar",
    "bojaríamos": "bojar",
    "bojarían": "bojar",
    "bojarías": "bojar",
    "bojas": "bojar",
    "bojase": "bojar",
    "bojaseis": "bojar",
    "bojasen": "bojar",
    "bojases": "bojar",
    "bojaste": "bojar",
    "bojasteis": "bojar",
    "boje": "bojar",
    "bojea": "bojear",
    "bojeaba": "bojear",
    "bojeabais": "bojear",
    "bojeaban": "bojear",
    "bojeabas": "bojear",
    "bojead": "bojear",
    "bojeada": "bojear",
    "bojeadas": "bojear",
    "bojeado": "bojear",
    "bojeados": "bojear",
    "bojeamos": "bojear",
    "bojean": "bojear",
    "bojeando": "bojear",
    "bojeara": "bojear",
    "bojearais": "bojear",
    "bojearan": "bojear",
    "bojearas": "bojear",
    "bojeare": "bojear",
    "bojeareis": "bojear",
    "bojearemos": "bojear",
    "bojearen": "bojear",
    "bojeares": "bojear",
    "bojearon": "bojear",
    "bojeará": "bojear",
    "bojearán": "bojear",
    "bojearás": "bojear",
    "bojearé": "bojear",
    "bojearéis": "bojear",
    "bojearía": "bojear",
    "bojearíais": "bojear",
    "bojearíamos": "bojear",
    "bojearían": "bojear",
    "bojearías": "bojear",
    "bojeas": "bojear",
    "bojease": "bojear",
    "bojeaseis": "bojear",
    "bojeasen": "bojear",
    "bojeases": "bojear",
    "bojeaste": "bojear",
    "bojeasteis": "bojear",
    "bojedales": "bojedal",
    "bojee": "bojear",
    "bojeemos": "bojear",
    "bojeen": "bojear",
    "bojees": "bojear",
    "bojemos": "bojar",
    "bojen": "bojar",
    "bojeo": "bojear",
    "bojes": "boje",
    "bojeábamos": "bojear",
    "bojeáis": "bojear",
    "bojeáramos": "bojear",
    "bojeáremos": "bojear",
    "bojeásemos": "bojear",
    "bojeé": "bojear",
    "bojeéis": "bojear",
    "bojeó": "bojear",
    "bojigangas": "bojiganga",
    "bojo": "bojar",
    "bojotes": "bojote",
    "bojábamos": "bojar",
    "bojáis": "bojar",
    "bojáramos": "bojar",
    "bojáremos": "bojar",
    "bojásemos": "bojar",
    "bojé": "bojar",
    "bojéis": "bojar",
    "bojó": "bojar",
    "boladas": "bolada",
    "bolados": "bolado",
    "bolandistas": "bolandista",
    "bolardos": "bolardo",
    "bolas": "bola",
    "bolcheviques": "bolchevique",
    "bolchevismos": "bolchevismo",
    "bolchevistas": "bolchevista",
    "boldos": "boldo",
    "bolea": "bolear",
    "boleaba": "bolear",
    "boleabais": "bolear",
    "boleaban": "bolear",
    "boleabas": "bolear",
    "bolead": "bolear",
    "boleada": "bolear",
    "boleadas": "bolear",
    "boleado": "bolear",
    "boleados": "bolear",
    "boleamos": "bolear",
    "bolean": "bolear",
    "boleando": "bolear",
    "boleara": "bolear",
    "bolearais": "bolear",
    "bolearan": "bolear",
    "bolearas": "bolear",
    "boleare": "bolear",
    "boleareis": "bolear",
    "bolearemos": "bolear",
    "bolearen": "bolear",
    "boleares": "bolear",
    "bolearon": "bolear",
    "boleará": "bolear",
    "bolearán": "bolear",
    "bolearás": "bolear",
    "bolearé": "bolear",
    "bolearéis": "bolear",
    "bolearía": "bolear",
    "bolearíais": "bolear",
    "bolearíamos": "bolear",
    "bolearían": "bolear",
    "bolearías": "bolear",
    "boleas": "bolear",
    "bolease": "bolear",
    "boleaseis": "bolear",
    "boleasen": "bolear",
    "boleases": "bolear",
    "boleaste": "bolear",
    "boleasteis": "bolear",
    "bolee": "bolear",
    "boleemos": "bolear",
    "boleen": "bolear",
    "bolees": "bolear",
    "boleo": "bolear",
    "boleos": "boleo",
    "bolera": "bolero",
    "boleras": "bolero",
    "boleristas": "bolerista",
    "boleros": "bolero",
    "boles": "bol",
    "boletas": "boleta",
    "boleterías": "boletería",
    "boletines": "boletín",
    "boletos": "boleto",
    "boleábamos": "bolear",
    "boleáis": "bolear",
    "boleáramos": "bolear",
    "boleáremos": "bolear",
    "boleásemos": "bolear",
    "boleé": "bolear",
    "boleéis": "bolear",
    "boleó": "bolear",
    "bolichadas": "bolichada",
    "bolichera": "bolichero",
    "bolicheras": "bolichero",
    "bolicheros": "bolichero",
    "boliches": "boliche",
    "bolillos": "bolillo",
    "bolinas": "bolina",
    "bolinches": "bolinche",
    "bolineadora": "bolineador",
    "bolineadoras": "bolineador",
    "bolineadores": "bolineador",
    "bolinera": "bolinero",
    "bolineras": "bolinero",
    "bolineros": "bolinero",
    "bolines": "bolín",
    "bolis": "boli",
    "bolitas": "bolita",
    "bolivarenses": "bolivarense",
    "bolivariana": "bolivariano",
    "bolivarianas": "bolivariano",
    "bolivarianos": "bolivariano",
    "boliviana": "boliviano",
    "bolivianas": "boliviano",
    "bolivianismos": "bolivianismo",
    "bolivianos": "boliviano",
    "bolla": "bollar",
    "bollaba": "bollar",
    "bollabais": "bollar",
    "bollaban": "bollar",
    "bollabas": "bollar",
    "bollad": "bollar",
    "bollada": "bollar",
    "bolladas": "bollar",
    "bollado": "bollar",
    "bollados": "bollar",
    "bolladuras": "bolladura",
    "bollamos": "bollar",
    "bollan": "bollar",
    "bollando": "bollar",
    "bollara": "bollar",
    "bollarais": "bollar",
    "bollaran": "bollar",
    "bollaras": "bollar",
    "bollare": "bollar",
    "bollareis": "bollar",
    "bollaremos": "bollar",
    "bollaren": "bollar",
    "bollares": "bollar",
    "bollaron": "bollar",
    "bollará": "bollar",
    "bollarán": "bollar",
    "bollarás": "bollar",
    "bollaré": "bollar",
    "bollaréis": "bollar",
    "bollaría": "bollar",
    "bollaríais": "bollar",
    "bollaríamos": "bollar",
    "bollarían": "bollar",
    "bollarías": "bollar",
    "bollas": "bollar",
    "bollase": "bollar",
    "bollaseis": "bollar",
    "bollasen": "bollar",
    "bollases": "bollar",
    "bollaste": "bollar",
    "bollasteis": "bollar",
    "bolle": "bollar",
    "bollemos": "bollar",
    "bollen": "bollar",
    "bollera": "bollero",
    "bolleras": "bollero",
    "bolleros": "bollero",
    "bollerías": "bollería",
    "bolles": "bollar",
    "bollo": "bollar",
    "bollonada": "bollonado",
    "bollonadas": "bollonado",
    "bollonados": "bollonado",
    "bollones": "bollón",
    "bollos": "bollo",
    "bollábamos": "bollar",
    "bolláis": "bollar",
    "bolláramos": "bollar",
    "bolláremos": "bollar",
    "bollásemos": "bollar",
    "bollé": "bollar",
    "bolléis": "bollar",
    "bolló": "bollar",
    "bolometrías": "bolometría",
    "bolos": "bolo",
    "boloñesa": "boloñés",
    "boloñesas": "boloñés",
    "boloñeses": "boloñés",
    "bolsadas": "bolsada",
    "bolsas": "bolsa",
    "bolsea": "bolsear",
    "bolseaba": "bolsear",
    "bolseabais": "bolsear",
    "bolseaban": "bolsear",
    "bolseabas": "bolsear",
    "bolsead": "bolsear",
    "bolseada": "bolsear",
    "bolseadas": "bolsear",
    "bolseado": "bolsear",
    "bolseados": "bolsear",
    "bolseamos": "bolsear",
    "bolsean": "bolsear",
    "bolseando": "bolsear",
    "bolseara": "bolsear",
    "bolsearais": "bolsear",
    "bolsearan": "bolsear",
    "bolsearas": "bolsear",
    "bolseare": "bolsear",
    "bolseareis": "bolsear",
    "bolsearemos": "bolsear",
    "bolsearen": "bolsear",
    "bolseares": "bolsear",
    "bolsearon": "bolsear",
    "bolseará": "bolsear",
    "bolsearán": "bolsear",
    "bolsearás": "bolsear",
    "bolsearé": "bolsear",
    "bolsearéis": "bolsear",
    "bolsearía": "bolsear",
    "bolsearíais": "bolsear",
    "bolsearíamos": "bolsear",
    "bolsearían": "bolsear",
    "bolsearías": "bolsear",
    "bolseas": "bolsear",
    "bolsease": "bolsear",
    "bolseaseis": "bolsear",
    "bolseasen": "bolsear",
    "bolseases": "bolsear",
    "bolseaste": "bolsear",
    "bolseasteis": "bolsear",
    "bolsee": "bolsear",
    "bolseemos": "bolsear",
    "bolseen": "bolsear",
    "bolsees": "bolsear",
    "bolseo": "bolsear",
    "bolseras": "bolsera",
    "bolseábamos": "bolsear",
    "bolseáis": "bolsear",
    "bolseáramos": "bolsear",
    "bolseáremos": "bolsear",
    "bolseásemos": "bolsear",
    "bolseé": "bolsear",
    "bolseéis": "bolsear",
    "bolseó": "bolsear",
    "bolsillos": "bolsillo",
    "bolsines": "bolsín",
    "bolsiquea": "bolsiquear",
    "bolsiqueaba": "bolsiquear",
    "bolsiqueabais": "bolsiquear",
    "bolsiqueaban": "bolsiquear",
    "bolsiqueabas": "bolsiquear",
    "bolsiquead": "bolsiquear",
    "bolsiqueada": "bolsiquear",
    "bolsiqueadas": "bolsiquear",
    "bolsiqueado": "bolsiquear",
    "bolsiqueados": "bolsiquear",
    "bolsiqueamos": "bolsiquear",
    "bolsiquean": "bolsiquear",
    "bolsiqueando": "bolsiquear",
    "bolsiqueara": "bolsiquear",
    "bolsiquearais": "bolsiquear",
    "bolsiquearan": "bolsiquear",
    "bolsiquearas": "bolsiquear",
    "bolsiqueare": "bolsiquear",
    "bolsiqueareis": "bolsiquear",
    "bolsiquearemos": "bolsiquear",
    "bolsiquearen": "bolsiquear",
    "bolsiqueares": "bolsiquear",
    "bolsiquearon": "bolsiquear",
    "bolsiqueará": "bolsiquear",
    "bolsiquearán": "bolsiquear",
    "bolsiquearás": "bolsiquear",
    "bolsiquearé": "bolsiquear",
    "bolsiquearéis": "bolsiquear",
    "bolsiquearía": "bolsiquear",
    "bolsiquearíais": "bolsiquear",
    "bolsiquearíamos": "bolsiquear",
    "bolsiquearían": "bolsiquear",
    "bolsiquearías": "bolsiquear",
    "bolsiqueas": "bolsiquear",
    "bolsiquease": "bolsiquear",
    "bolsiqueaseis": "bolsiquear",
    "bolsiqueasen": "bolsiquear",
    "bolsiqueases": "bolsiquear",
    "bolsiqueaste": "bolsiquear",
    "bolsiqueasteis": "bolsiquear",
    "bolsiquee": "bolsiquear",
    "bolsiqueemos": "bolsiquear",
    "bolsiqueen": "bolsiquear",
    "bolsiquees": "bolsiquear",
    "bolsiqueo": "bolsiquear",
    "bolsiqueábamos": "bolsiquear",
    "bolsiqueáis": "bolsiquear",
    "bolsiqueáramos": "bolsiquear",
    "bolsiqueáremos": "bolsiquear",
    "bolsiqueásemos": "bolsiquear",
    "bolsiqueé": "bolsiquear",
    "bolsiqueéis": "bolsiquear",
    "bolsiqueó": "bolsiquear",
    "bolsistas": "bolsista",
    "bolsones": "bolsón",
    "bolsos": "bolso",
    "bolígrafos": "bolígrafo",
    "bolívares": "bolívar",
    "bolómetros": "bolómetro",
    "bomba": "bombo",
    "bombacha": "bombacho",
    "bombachas": "bombacho",
    "bombachos": "bombacho",
    "bombardas": "bombarda",
    "bombardea": "bombardear",
    "bombardeaba": "bombardear",
    "bombardeabais": "bombardear",
    "bombardeaban": "bombardear",
    "bombardeabas": "bombardear",
    "bombardead": "bombardear",
    "bombardeada": "bombardear",
    "bombardeadas": "bombardear",
    "bombardeado": "bombardear",
    "bombardeados": "bombardear",
    "bombardeamos": "bombardear",
    "bombardean": "bombardear",
    "bombardeando": "bombardear",
    "bombardeara": "bombardear",
    "bombardearais": "bombardear",
    "bombardearan": "bombardear",
    "bombardearas": "bombardear",
    "bombardeare": "bombardear",
    "bombardeareis": "bombardear",
    "bombardearemos": "bombardear",
    "bombardearen": "bombardear",
    "bombardeares": "bombardear",
    "bombardearon": "bombardear",
    "bombardeará": "bombardear",
    "bombardearán": "bombardear",
    "bombardearás": "bombardear",
    "bombardearé": "bombardear",
    "bombardearéis": "bombardear",
    "bombardearía": "bombardear",
    "bombardearíais": "bombardear",
    "bombardearíamos": "bombardear",
    "bombardearían": "bombardear",
    "bombardearías": "bombardear",
    "bombardeas": "bombardear",
    "bombardease": "bombardear",
    "bombardeaseis": "bombardear",
    "bombardeasen": "bombardear",
    "bombardeases": "bombardear",
    "bombardeaste": "bombardear",
    "bombardeasteis": "bombardear",
    "bombardee": "bombardear",
    "bombardeemos": "bombardear",
    "bombardeen": "bombardear",
    "bombardees": "bombardear",
    "bombardeo": "bombardear",
    "bombardeos": "bombardeo",
    "bombarderos": "bombardero",
    "bombardeábamos": "bombardear",
    "bombardeáis": "bombardear",
    "bombardeáramos": "bombardear",
    "bombardeáremos": "bombardear",
    "bombardeásemos": "bombardear",
    "bombardeé": "bombardear",
    "bombardeéis": "bombardear",
    "bombardeó": "bombardear",
    "bombardinos": "bombardino",
    "bombardones": "bombardón",
    "bombas": "bombo",
    "bombasíes": "bombasí",
    "bombasís": "bombasí",
    "bombazos": "bombazo",
    "bombea": "bombear",
    "bombeaba": "bombear",
    "bombeabais": "bombear",
    "bombeaban": "bombear",
    "bombeabas": "bombear",
    "bombead": "bombear",
    "bombeada": "bombear",
    "bombeadas": "bombear",
    "bombeado": "bombear",
    "bombeados": "bombear",
    "bombeamos": "bombear",
    "bombean": "bombear",
    "bombeando": "bombear",
    "bombeara": "bombear",
    "bombearais": "bombear",
    "bombearan": "bombear",
    "bombearas": "bombear",
    "bombeare": "bombear",
    "bombeareis": "bombear",
    "bombearemos": "bombear",
    "bombearen": "bombear",
    "bombeares": "bombear",
    "bombearon": "bombear",
    "bombeará": "bombear",
    "bombearán": "bombear",
    "bombearás": "bombear",
    "bombearé": "bombear",
    "bombearéis": "bombear",
    "bombearía": "bombear",
    "bombearíais": "bombear",
    "bombearíamos": "bombear",
    "bombearían": "bombear",
    "bombearías": "bombear",
    "bombeas": "bombear",
    "bombease": "bombear",
    "bombeaseis": "bombear",
    "bombeasen": "bombear",
    "bombeases": "bombear",
    "bombeaste": "bombear",
    "bombeasteis": "bombear",
    "bombee": "bombear",
    "bombeemos": "bombear",
    "bombeen": "bombear",
    "bombees": "bombear",
    "bombeo": "bombear",
    "bombeos": "bombeo",
    "bombera": "bombero",
    "bomberas": "bombero",
    "bomberos": "bombero",
    "bombeábamos": "bombear",
    "bombeáis": "bombear",
    "bombeáramos": "bombear",
    "bombeáremos": "bombear",
    "bombeásemos": "bombear",
    "bombeé": "bombear",
    "bombeéis": "bombear",
    "bombeó": "bombear",
    "bombillas": "bombilla",
    "bombillos": "bombillo",
    "bombines": "bombín",
    "bombonas": "bombona",
    "bomboneras": "bombonera",
    "bombonerías": "bombonería",
    "bombones": "bombón",
    "bombos": "bombo",
    "bombáceas": "bombácea",
    "bombástica": "bombástico",
    "bombásticas": "bombástico",
    "bombásticos": "bombástico",
    "bombés": "bombé",
    "bonachona": "bonachón",
    "bonachonas": "bonachón",
    "bonachonerías": "bonachonería",
    "bonachones": "bonachón",
    "bonaerenses": "bonaerense",
    "bonancibles": "bonancible",
    "bonanzas": "bonanza",
    "bonanzosa": "bonanzoso",
    "bonanzosas": "bonanzoso",
    "bonanzosos": "bonanzoso",
    "bonapartismos": "bonapartismo",
    "bonapartistas": "bonapartista",
    "bondades": "bondad",
    "bondadosa": "bondadoso",
    "bondadosas": "bondadoso",
    "bondadosos": "bondadoso",
    "bonetas": "boneta",
    "bonetera": "bonetero",
    "boneteras": "bonetero",
    "boneteros": "bonetero",
    "boneterías": "bonetería",
    "bonetes": "bonete",
    "bongos": "bongo",
    "boniatos": "boniato",
    "bonifica": "bonificar",
    "bonificaba": "bonificar",
    "bonificabais": "bonificar",
    "bonificaban": "bonificar",
    "bonificabas": "bonificar",
    "bonificaciones": "bonificación",
    "bonificad": "bonificar",
    "bonificada": "bonificar",
    "bonificadas": "bonificar",
    "bonificado": "bonificar",
    "bonificados": "bonificar",
    "bonificamos": "bonificar",
    "bonifican": "bonificar",
    "bonificando": "bonificar",
    "bonificara": "bonificar",
    "bonificarais": "bonificar",
    "bonificaran": "bonificar",
    "bonificaras": "bonificar",
    "bonificare": "bonificar",
    "bonificareis": "bonificar",
    "bonificaremos": "bonificar",
    "bonificaren": "bonificar",
    "bonificares": "bonificar",
    "bonificaron": "bonificar",
    "bonificará": "bonificar",
    "bonificarán": "bonificar",
    "bonificarás": "bonificar",
    "bonificaré": "bonificar",
    "bonificaréis": "bonificar",
    "bonificaría": "bonificar",
    "bonificaríais": "bonificar",
    "bonificaríamos": "bonificar",
    "bonificarían": "bonificar",
    "bonificarías": "bonificar",
    "bonificas": "bonificar",
    "bonificase": "bonificar",
    "bonificaseis": "bonificar",
    "bonificasen": "bonificar",
    "bonificases": "bonificar",
    "bonificaste": "bonificar",
    "bonificasteis": "bonificar",
    "bonifico": "bonificar",
    "bonificábamos": "bonificar",
    "bonificáis": "bonificar",
    "bonificáramos": "bonificar",
    "bonificáremos": "bonificar",
    "bonificásemos": "bonificar",
    "bonificó": "bonificar",
    "bonifique": "bonificar",
    "bonifiquemos": "bonificar",
    "bonifiquen": "bonificar",
    "bonifiques": "bonificar",
    "bonifiqué": "bonificar",
    "bonifiquéis": "bonificar",
    "bonita": "bonito",
    "bonitas": "bonito",
    "boniteras": "bonitera",
    "bonitos": "bonito",
    "bonobos": "bonobo",
    "bonos": "bono",
    "bonotes": "bonote",
    "bonsais": "bonsai",
    "bonzos": "bonzo",
    "booleana": "booleano",
    "booleanas": "booleano",
    "booleanos": "booleano",
    "boolings": "booling",
    "boomerangs": "boomerang",
    "booms": "boom",
    "boosts": "boost",
    "boquea": "boquear",
    "boqueaba": "boquear",
    "boqueabais": "boquear",
    "boqueaban": "boquear",
    "boqueabas": "boquear",
    "boquead": "boquear",
    "boqueada": "boquear",
    "boqueadas": "boquear",
    "boqueado": "boquear",
    "boqueados": "boquear",
    "boqueamos": "boquear",
    "boquean": "boquear",
    "boqueando": "boquear",
    "boqueara": "boquear",
    "boquearais": "boquear",
    "boquearan": "boquear",
    "boquearas": "boquear",
    "boqueare": "boquear",
    "boqueareis": "boquear",
    "boquearemos": "boquear",
    "boquearen": "boquear",
    "boqueares": "boquear",
    "boquearon": "boquear",
    "boqueará": "boquear",
    "boquearán": "boquear",
    "boquearás": "boquear",
    "boquearé": "boquear",
    "boquearéis": "boquear",
    "boquearía": "boquear",
    "boquearíais": "boquear",
    "boquearíamos": "boquear",
    "boquearían": "boquear",
    "boquearías": "boquear",
    "boqueas": "boquear",
    "boquease": "boquear",
    "boqueaseis": "boquear",
    "boqueasen": "boquear",
    "boqueases": "boquear",
    "boqueaste": "boquear",
    "boqueasteis": "boquear",
    "boquee": "boquear",
    "boqueemos": "boquear",
    "boqueen": "boquear",
    "boquees": "boquear",
    "boqueo": "boquear",
    "boqueras": "boquera",
    "boquerienta": "boqueriento",
    "boquerientas": "boqueriento",
    "boquerientos": "boqueriento",
    "boqueroneras": "boqueronera",
    "boquerones": "boquerón",
    "boquetes": "boquete",
    "boqueábamos": "boquear",
    "boqueáis": "boquear",
    "boqueáramos": "boquear",
    "boqueáremos": "boquear",
    "boqueásemos": "boquear",
    "boqueé": "boquear",
    "boqueéis": "boquear",
    "boqueó": "boquear",
    "boquiabierta": "boquiabierto",
    "boquiabiertas": "boquiabierto",
    "boquiabiertos": "boquiabierto",
    "boquiconejuna": "boquiconejuno",
    "boquiconejunas": "boquiconejuno",
    "boquiconejunos": "boquiconejuno",
    "boquifresca": "boquifresco",
    "boquifrescas": "boquifresco",
    "boquifrescos": "boquifresco",
    "boquifruncida": "boquifruncido",
    "boquifruncidas": "boquifruncido",
    "boquifruncidos": "boquifruncido",
    "boquihendida": "boquihendido",
    "boquihendidas": "boquihendido",
    "boquihendidos": "boquihendido",
    "boquihundida": "boquihundido",
    "boquihundidas": "boquihundido",
    "boquihundidos": "boquihundido",
    "boquillas": "boquilla",
    "boquimuelles": "boquimuelle",
    "boquinegra": "boquinegro",
    "boquinegras": "boquinegro",
    "boquinegros": "boquinegro",
    "boquines": "boquín",
    "boquirrota": "boquirroto",
    "boquirrotas": "boquirroto",
    "boquirrotos": "boquirroto",
    "boquirrubia": "boquirrubio",
    "boquirrubias": "boquirrubio",
    "boquirrubios": "boquirrubio",
    "boquiseca": "boquiseco",
    "boquisecas": "boquiseco",
    "boquisecos": "boquiseco",
    "boquitas": "boquita",
    "boracitas": "boracita",
    "boranos": "borano",
    "boras": "bora",
    "boratos": "borato",
    "borbolea": "borbolear",
    "borboleaba": "borbolear",
    "borboleabais": "borbolear",
    "borboleaban": "borbolear",
    "borboleabas": "borbolear",
    "borbolead": "borbolear",
    "borboleada": "borbolear",
    "borboleadas": "borbolear",
    "borboleado": "borbolear",
    "borboleados": "borbolear",
    "borboleamos": "borbolear",
    "borbolean": "borbolear",
    "borboleando": "borbolear",
    "borboleara": "borbolear",
    "borbolearais": "borbolear",
    "borbolearan": "borbolear",
    "borbolearas": "borbolear",
    "borboleare": "borbolear",
    "borboleareis": "borbolear",
    "borbolearemos": "borbolear",
    "borbolearen": "borbolear",
    "borboleares": "borbolear",
    "borbolearon": "borbolear",
    "borboleará": "borbolear",
    "borbolearán": "borbolear",
    "borbolearás": "borbolear",
    "borbolearé": "borbolear",
    "borbolearéis": "borbolear",
    "borbolearía": "borbolear",
    "borbolearíais": "borbolear",
    "borbolearíamos": "borbolear",
    "borbolearían": "borbolear",
    "borbolearías": "borbolear",
    "borboleas": "borbolear",
    "borbolease": "borbolear",
    "borboleaseis": "borbolear",
    "borboleasen": "borbolear",
    "borboleases": "borbolear",
    "borboleaste": "borbolear",
    "borboleasteis": "borbolear",
    "borbolee": "borbolear",
    "borboleemos": "borbolear",
    "borboleen": "borbolear",
    "borbolees": "borbolear",
    "borboleo": "borbolear",
    "borboleábamos": "borbolear",
    "borboleáis": "borbolear",
    "borboleáramos": "borbolear",
    "borboleáremos": "borbolear",
    "borboleásemos": "borbolear",
    "borboleé": "borbolear",
    "borboleéis": "borbolear",
    "borboleó": "borbolear",
    "borbolla": "borbollar",
    "borbollaba": "borbollar",
    "borbollabais": "borbollar",
    "borbollaban": "borbollar",
    "borbollabas": "borbollar",
    "borbollad": "borbollar",
    "borbollada": "borbollar",
    "borbolladas": "borbollar",
    "borbollado": "borbollar",
    "borbollados": "borbollar",
    "borbollamos": "borbollar",
    "borbollan": "borbollar",
    "borbollando": "borbollar",
    "borbollara": "borbollar",
    "borbollarais": "borbollar",
    "borbollaran": "borbollar",
    "borbollaras": "borbollar",
    "borbollare": "borbollar",
    "borbollareis": "borbollar",
    "borbollaremos": "borbollar",
    "borbollaren": "borbollar",
    "borbollares": "borbollar",
    "borbollaron": "borbollar",
    "borbollará": "borbollar",
    "borbollarán": "borbollar",
    "borbollarás": "borbollar",
    "borbollaré": "borbollar",
    "borbollaréis": "borbollar",
    "borbollaría": "borbollar",
    "borbollaríais": "borbollar",
    "borbollaríamos": "borbollar",
    "borbollarían": "borbollar",
    "borbollarías": "borbollar",
    "borbollas": "borbollar",
    "borbollase": "borbollar",
    "borbollaseis": "borbollar",
    "borbollasen": "borbollar",
    "borbollases": "borbollar",
    "borbollaste": "borbollar",
    "borbollasteis": "borbollar",
    "borbolle": "borbollar",
    "borbollemos": "borbollar",
    "borbollen": "borbollar",
    "borbolleos": "borbolleo",
    "borbolles": "borbollar",
    "borbollo": "borbollar",
    "borbollonea": "borbollonear",
    "borbolloneaba": "borbollonear",
    "borbolloneabais": "borbollonear",
    "borbolloneaban": "borbollonear",
    "borbolloneabas": "borbollonear",
    "borbollonead": "borbollonear",
    "borbolloneada": "borbollonear",
    "borbolloneadas": "borbollonear",
    "borbolloneado": "borbollonear",
    "borbolloneados": "borbollonear",
    "borbolloneamos": "borbollonear",
    "borbollonean": "borbollonear",
    "borbolloneando": "borbollonear",
    "borbolloneara": "borbollonear",
    "borbollonearais": "borbollonear",
    "borbollonearan": "borbollonear",
    "borbollonearas": "borbollonear",
    "borbolloneare": "borbollonear",
    "borbolloneareis": "borbollonear",
    "borbollonearemos": "borbollonear",
    "borbollonearen": "borbollonear",
    "borbolloneares": "borbollonear",
    "borbollonearon": "borbollonear",
    "borbolloneará": "borbollonear",
    "borbollonearán": "borbollonear",
    "borbollonearás": "borbollonear",
    "borbollonearé": "borbollonear",
    "borbollonearéis": "borbollonear",
    "borbollonearía": "borbollonear",
    "borbollonearíais": "borbollonear",
    "borbollonearíamos": "borbollonear",
    "borbollonearían": "borbollonear",
    "borbollonearías": "borbollonear",
    "borbolloneas": "borbollonear",
    "borbollonease": "borbollonear",
    "borbolloneaseis": "borbollonear",
    "borbolloneasen": "borbollonear",
    "borbolloneases": "borbollonear",
    "borbolloneaste": "borbollonear",
    "borbolloneasteis": "borbollonear",
    "borbollonee": "borbollonear",
    "borbolloneemos": "borbollonear",
    "borbolloneen": "borbollonear",
    "borbollonees": "borbollonear",
    "borbolloneo": "borbollonear",
    "borbollones": "borbollón",
    "borbolloneábamos": "borbollonear",
    "borbolloneáis": "borbollonear",
    "borbolloneáramos": "borbollonear",
    "borbolloneáremos": "borbollonear",
    "borbolloneásemos": "borbollonear",
    "borbolloneé": "borbollonear",
    "borbolloneéis": "borbollonear",
    "borbolloneó": "borbollonear",
    "borbollábamos": "borbollar",
    "borbolláis": "borbollar",
    "borbolláramos": "borbollar",
    "borbolláremos": "borbollar",
    "borbollásemos": "borbollar",
    "borbollé": "borbollar",
    "borbolléis": "borbollar",
    "borbolló": "borbollar",
    "borborigmos": "borborigmo",
    "borborita": "borboritar",
    "borboritaba": "borboritar",
    "borboritabais": "borboritar",
    "borboritaban": "borboritar",
    "borboritabas": "borboritar",
    "borboritad": "borboritar",
    "borboritada": "borboritar",
    "borboritadas": "borboritar",
    "borboritado": "borboritar",
    "borboritados": "borboritar",
    "borboritamos": "borboritar",
    "borboritan": "borboritar",
    "borboritando": "borboritar",
    "borboritara": "borboritar",
    "borboritarais": "borboritar",
    "borboritaran": "borboritar",
    "borboritaras": "borboritar",
    "borboritare": "borboritar",
    "borboritareis": "borboritar",
    "borboritaremos": "borboritar",
    "borboritaren": "borboritar",
    "borboritares": "borboritar",
    "borboritaron": "borboritar",
    "borboritará": "borboritar",
    "borboritarán": "borboritar",
    "borboritarás": "borboritar",
    "borboritaré": "borboritar",
    "borboritaréis": "borboritar",
    "borboritaría": "borboritar",
    "borboritaríais": "borboritar",
    "borboritaríamos": "borboritar",
    "borboritarían": "borboritar",
    "borboritarías": "borboritar",
    "borboritas": "borboritar",
    "borboritase": "borboritar",
    "borboritaseis": "borboritar",
    "borboritasen": "borboritar",
    "borboritases": "borboritar",
    "borboritaste": "borboritar",
    "borboritasteis": "borboritar",
    "borborite": "borboritar",
    "borboritemos": "borboritar",
    "borboriten": "borboritar",
    "borborites": "borboritar",
    "borborito": "borboritar",
    "borboritábamos": "borboritar",
    "borboritáis": "borboritar",
    "borboritáramos": "borboritar",
    "borboritáremos": "borboritar",
    "borboritásemos": "borboritar",
    "borborité": "borboritar",
    "borboritéis": "borboritar",
    "borboritó": "borboritar",
    "borbota": "borbotar",
    "borbotaba": "borbotar",
    "borbotabais": "borbotar",
    "borbotaban": "borbotar",
    "borbotabas": "borbotar",
    "borbotad": "borbotar",
    "borbotada": "borbotar",
    "borbotadas": "borbotar",
    "borbotado": "borbotar",
    "borbotados": "borbotar",
    "borbotamos": "borbotar",
    "borbotan": "borbotar",
    "borbotando": "borbotar",
    "borbotara": "borbotar",
    "borbotarais": "borbotar",
    "borbotaran": "borbotar",
    "borbotaras": "borbotar",
    "borbotare": "borbotar",
    "borbotareis": "borbotar",
    "borbotaremos": "borbotar",
    "borbotaren": "borbotar",
    "borbotares": "borbotar",
    "borbotaron": "borbotar",
    "borbotará": "borbotar",
    "borbotarán": "borbotar",
    "borbotarás": "borbotar",
    "borbotaré": "borbotar",
    "borbotaréis": "borbotar",
    "borbotaría": "borbotar",
    "borbotaríais": "borbotar",
    "borbotaríamos": "borbotar",
    "borbotarían": "borbotar",
    "borbotarías": "borbotar",
    "borbotas": "borbotar",
    "borbotase": "borbotar",
    "borbotaseis": "borbotar",
    "borbotasen": "borbotar",
    "borbotases": "borbotar",
    "borbotaste": "borbotar",
    "borbotasteis": "borbotar",
    "borbote": "borbotar",
    "borbotea": "borbotear",
    "borboteaba": "borbotear",
    "borboteabais": "borbotear",
    "borboteaban": "borbotear",
    "borboteabas": "borbotear",
    "borbotead": "borbotear",
    "borboteada": "borbotear",
    "borboteadas": "borbotear",
    "borboteado": "borbotear",
    "borboteados": "borbotear",
    "borboteamos": "borbotear",
    "borbotean": "borbotear",
    "borboteando": "borbotear",
    "borboteara": "borbotear",
    "borbotearais": "borbotear",
    "borbotearan": "borbotear",
    "borbotearas": "borbotear",
    "borboteare": "borbotear",
    "borboteareis": "borbotear",
    "borbotearemos": "borbotear",
    "borbotearen": "borbotear",
    "borboteares": "borbotear",
    "borbotearon": "borbotear",
    "borboteará": "borbotear",
    "borbotearán": "borbotear",
    "borbotearás": "borbotear",
    "borbotearé": "borbotear",
    "borbotearéis": "borbotear",
    "borbotearía": "borbotear",
    "borbotearíais": "borbotear",
    "borbotearíamos": "borbotear",
    "borbotearían": "borbotear",
    "borbotearías": "borbotear",
    "borboteas": "borbotear",
    "borbotease": "borbotear",
    "borboteaseis": "borbotear",
    "borboteasen": "borbotear",
    "borboteases": "borbotear",
    "borboteaste": "borbotear",
    "borboteasteis": "borbotear",
    "borbotee": "borbotear",
    "borboteemos": "borbotear",
    "borboteen": "borbotear",
    "borbotees": "borbotear",
    "borbotemos": "borbotar",
    "borboten": "borbotar",
    "borboteo": "borbotear",
    "borboteos": "borboteo",
    "borbotes": "borbotar",
    "borboteábamos": "borbotear",
    "borboteáis": "borbotear",
    "borboteáramos": "borbotear",
    "borboteáremos": "borbotear",
    "borboteásemos": "borbotear",
    "borboteé": "borbotear",
    "borboteéis": "borbotear",
    "borboteó": "borbotear",
    "borboto": "borbotar",
    "borbotones": "borbotón",
    "borbotábamos": "borbotar",
    "borbotáis": "borbotar",
    "borbotáramos": "borbotar",
    "borbotáremos": "borbotar",
    "borbotásemos": "borbotar",
    "borboté": "borbotar",
    "borbotéis": "borbotar",
    "borbotó": "borbotar",
    "borbónica": "borbónico",
    "borbónicas": "borbónico",
    "borbónicos": "borbónico",
    "borceguinera": "borceguinero",
    "borceguineras": "borceguinero",
    "borceguineros": "borceguinero",
    "borceguinerías": "borceguinería",
    "borceguíes": "borceguí",
    "borceguís": "borceguí",
    "borcellares": "borcellar",
    "borda": "bordar",
    "bordaba": "bordar",
    "bordabais": "bordar",
    "bordaban": "bordar",
    "bordabas": "bordar",
    "bordad": "bordar",
    "bordada": "bordar",
    "bordadas": "bordar",
    "bordado": "bordar",
    "bordadora": "bordador",
    "bordadoras": "bordador",
    "bordadores": "bordador",
    "bordados": "bordar",
    "bordaduras": "bordadura",
    "bordamos": "bordar",
    "bordan": "bordar",
    "bordando": "bordar",
    "bordara": "bordar",
    "bordarais": "bordar",
    "bordaran": "bordar",
    "bordaras": "bordar",
    "bordare": "bordar",
    "bordareis": "bordar",
    "bordaremos": "bordar",
    "bordaren": "bordar",
    "bordares": "bordar",
    "bordaron": "bordar",
    "bordará": "bordar",
    "bordarán": "bordar",
    "bordarás": "bordar",
    "bordaré": "bordar",
    "bordaréis": "bordar",
    "bordaría": "bordar",
    "bordaríais": "bordar",
    "bordaríamos": "bordar",
    "bordarían": "bordar",
    "bordarías": "bordar",
    "bordas": "bordar",
    "bordase": "bordar",
    "bordaseis": "bordar",
    "bordasen": "bordar",
    "bordases": "bordar",
    "bordaste": "bordar",
    "bordasteis": "bordar",
    "borde": "bordar",
    "bordea": "bordear",
    "bordeaba": "bordear",
    "bordeabais": "bordear",
    "bordeaban": "bordear",
    "bordeabas": "bordear",
    "bordead": "bordear",
    "bordeada": "bordear",
    "bordeadas": "bordear",
    "bordeado": "bordear",
    "bordeados": "bordear",
    "bordeamos": "bordear",
    "bordean": "bordear",
    "bordeando": "bordear",
    "bordeara": "bordear",
    "bordearais": "bordear",
    "bordearan": "bordear",
    "bordearas": "bordear",
    "bordeare": "bordear",
    "bordeareis": "bordear",
    "bordearemos": "bordear",
    "bordearen": "bordear",
    "bordeares": "bordear",
    "bordearon": "bordear",
    "bordeará": "bordear",
    "bordearán": "bordear",
    "bordearás": "bordear",
    "bordearé": "bordear",
    "bordearéis": "bordear",
    "bordearía": "bordear",
    "bordearíais": "bordear",
    "bordearíamos": "bordear",
    "bordearían": "bordear",
    "bordearías": "bordear",
    "bordeas": "bordear",
    "bordease": "bordear",
    "bordeaseis": "bordear",
    "bordeasen": "bordear",
    "bordeases": "bordear",
    "bordeaste": "bordear",
    "bordeasteis": "bordear",
    "bordee": "bordear",
    "bordeemos": "bordear",
    "bordeen": "bordear",
    "bordees": "bordear",
    "bordelesa": "bordelés",
    "bordelesas": "bordelés",
    "bordeleses": "bordelés",
    "bordemos": "bordar",
    "borden": "bordar",
    "bordeo": "bordear",
    "borderós": "borderó",
    "bordes": "borde",
    "bordeábamos": "bordear",
    "bordeáis": "bordear",
    "bordeáramos": "bordear",
    "bordeáremos": "bordear",
    "bordeásemos": "bordear",
    "bordeé": "bordear",
    "bordeéis": "bordear",
    "bordeó": "bordear",
    "bordillos": "bordillo",
    "bordo": "bordar",
    "bordonadoras": "bordonadora",
    "bordoncillos": "bordoncillo",
    "bordonea": "bordonear",
    "bordoneaba": "bordonear",
    "bordoneabais": "bordonear",
    "bordoneaban": "bordonear",
    "bordoneabas": "bordonear",
    "bordonead": "bordonear",
    "bordoneada": "bordonear",
    "bordoneadas": "bordonear",
    "bordoneado": "bordonear",
    "bordoneados": "bordonear",
    "bordoneamos": "bordonear",
    "bordonean": "bordonear",
    "bordoneando": "bordonear",
    "bordoneara": "bordonear",
    "bordonearais": "bordonear",
    "bordonearan": "bordonear",
    "bordonearas": "bordonear",
    "bordoneare": "bordonear",
    "bordoneareis": "bordonear",
    "bordonearemos": "bordonear",
    "bordonearen": "bordonear",
    "bordoneares": "bordonear",
    "bordonearon": "bordonear",
    "bordoneará": "bordonear",
    "bordonearán": "bordonear",
    "bordonearás": "bordonear",
    "bordonearé": "bordonear",
    "bordonearéis": "bordonear",
    "bordonearía": "bordonear",
    "bordonearíais": "bordonear",
    "bordonearíamos": "bordonear",
    "bordonearían": "bordonear",
    "bordonearías": "bordonear",
    "bordoneas": "bordonear",
    "bordonease": "bordonear",
    "bordoneaseis": "bordonear",
    "bordoneasen": "bordonear",
    "bordoneases": "bordonear",
    "bordoneaste": "bordonear",
    "bordoneasteis": "bordonear",
    "bordonee": "bordonear",
    "bordoneemos": "bordonear",
    "bordoneen": "bordonear",
    "bordonees": "bordonear",
    "bordoneo": "bordonear",
    "bordoneos": "bordoneo",
    "bordonera": "bordonero",
    "bordoneras": "bordonero",
    "bordoneros": "bordonero",
    "bordones": "bordón",
    "bordoneábamos": "bordonear",
    "bordoneáis": "bordonear",
    "bordoneáramos": "bordonear",
    "bordoneáremos": "bordonear",
    "bordoneásemos": "bordonear",
    "bordoneé": "bordonear",
    "bordoneéis": "bordonear",
    "bordoneó": "bordonear",
    "bordos": "bordo",
    "borduras": "bordura",
    "bordábamos": "bordar",
    "bordáis": "bordar",
    "bordáramos": "bordar",
    "bordáremos": "bordar",
    "bordásemos": "bordar",
    "bordé": "bordar",
    "bordéis": "bordar",
    "bordó": "bordar",
    "boreales": "boreal",
    "boreliana": "boreliano",
    "borelianas": "boreliano",
    "borelianos": "boreliano",
    "bores": "bore",
    "borgoñas": "borgoña",
    "borgoñona": "borgoñón",
    "borgoñonas": "borgoñón",
    "borgoñones": "borgoñón",
    "boriboratos": "boriborato",
    "boricua": "boricuo",
    "boricuas": "boricuo",
    "boricuos": "boricuo",
    "borlas": "borla",
    "bornea": "bornear",
    "borneaba": "bornear",
    "borneabais": "bornear",
    "borneaban": "bornear",
    "borneabas": "bornear",
    "bornead": "bornear",
    "borneada": "bornear",
    "borneadas": "bornear",
    "borneado": "bornear",
    "borneados": "bornear",
    "borneaduras": "borneadura",
    "borneamos": "bornear",
    "bornean": "bornear",
    "borneando": "bornear",
    "borneara": "bornear",
    "bornearais": "bornear",
    "bornearan": "bornear",
    "bornearas": "bornear",
    "borneare": "bornear",
    "borneareis": "bornear",
    "bornearemos": "bornear",
    "bornearen": "bornear",
    "borneares": "bornear",
    "bornearon": "bornear",
    "borneará": "bornear",
    "bornearán": "bornear",
    "bornearás": "bornear",
    "bornearé": "bornear",
    "bornearéis": "bornear",
    "bornearía": "bornear",
    "bornearíais": "bornear",
    "bornearíamos": "bornear",
    "bornearían": "bornear",
    "bornearías": "bornear",
    "borneas": "bornear",
    "bornease": "bornear",
    "borneaseis": "bornear",
    "borneasen": "bornear",
    "borneases": "bornear",
    "borneaste": "bornear",
    "borneasteis": "bornear",
    "bornee": "bornear",
    "borneemos": "bornear",
    "borneen": "bornear",
    "bornees": "bornear",
    "borneo": "bornear",
    "borneoles": "borneol",
    "borneos": "borneo",
    "bornes": "borne",
    "borneábamos": "bornear",
    "borneáis": "bornear",
    "borneáramos": "bornear",
    "borneáremos": "bornear",
    "borneásemos": "bornear",
    "borneé": "bornear",
    "borneéis": "bornear",
    "borneó": "bornear",
    "bornitas": "bornita",
    "bornizos": "bornizo",
    "borohidruros": "borohidruro",
    "boronas": "borona",
    "boronías": "boronía",
    "bororós": "bororó",
    "boros": "boro",
    "borosilicatos": "borosilicato",
    "borra": "borrar",
    "borraba": "borrar",
    "borrabais": "borrar",
    "borraban": "borrar",
    "borrabas": "borrar",
    "borrables": "borrable",
    "borracha": "borracho",
    "borrachas": "borracho",
    "borrachea": "borrachear",
    "borracheaba": "borrachear",
    "borracheabais": "borrachear",
    "borracheaban": "borrachear",
    "borracheabas": "borrachear",
    "borrachead": "borrachear",
    "borracheada": "borrachear",
    "borracheadas": "borrachear",
    "borracheado": "borrachear",
    "borracheados": "borrachear",
    "borracheamos": "borrachear",
    "borrachean": "borrachear",
    "borracheando": "borrachear",
    "borracheara": "borrachear",
    "borrachearais": "borrachear",
    "borrachearan": "borrachear",
    "borrachearas": "borrachear",
    "borracheare": "borrachear",
    "borracheareis": "borrachear",
    "borrachearemos": "borrachear",
    "borrachearen": "borrachear",
    "borracheares": "borrachear",
    "borrachearon": "borrachear",
    "borracheará": "borrachear",
    "borrachearán": "borrachear",
    "borrachearás": "borrachear",
    "borrachearé": "borrachear",
    "borrachearéis": "borrachear",
    "borrachearía": "borrachear",
    "borrachearíais": "borrachear",
    "borrachearíamos": "borrachear",
    "borrachearían": "borrachear",
    "borrachearías": "borrachear",
    "borracheas": "borrachear",
    "borrachease": "borrachear",
    "borracheaseis": "borrachear",
    "borracheasen": "borrachear",
    "borracheases": "borrachear",
    "borracheaste": "borrachear",
    "borracheasteis": "borrachear",
    "borrachee": "borrachear",
    "borracheemos": "borrachear",
    "borracheen": "borrachear",
    "borrachees": "borrachear",
    "borracheo": "borrachear",
    "borracheras": "borrachera",
    "borracheros": "borrachero",
    "borracheábamos": "borrachear",
    "borracheáis": "borrachear",
    "borracheáramos": "borrachear",
    "borracheáremos": "borrachear",
    "borracheásemos": "borrachear",
    "borracheé": "borrachear",
    "borracheéis": "borrachear",
    "borracheó": "borrachear",
    "borrachina": "borrachín",
    "borrachinas": "borrachín",
    "borrachines": "borrachín",
    "borrachos": "borracho",
    "borrad": "borrar",
    "borrada": "borrar",
    "borradas": "borrar",
    "borrado": "borrar",
    "borradora": "borrador",
    "borradoras": "borrador",
    "borradores": "borrador",
    "borrados": "borrar",
    "borraduras": "borradura",
    "borrajas": "borraja",
    "borrajea": "borrajear",
    "borrajeaba": "borrajear",
    "borrajeabais": "borrajear",
    "borrajeaban": "borrajear",
    "borrajeabas": "borrajear",
    "borrajead": "borrajear",
    "borrajeada": "borrajear",
    "borrajeadas": "borrajear",
    "borrajeado": "borrajear",
    "borrajeados": "borrajear",
    "borrajeamos": "borrajear",
    "borrajean": "borrajear",
    "borrajeando": "borrajear",
    "borrajeara": "borrajear",
    "borrajearais": "borrajear",
    "borrajearan": "borrajear",
    "borrajearas": "borrajear",
    "borrajeare": "borrajear",
    "borrajeareis": "borrajear",
    "borrajearemos": "borrajear",
    "borrajearen": "borrajear",
    "borrajeares": "borrajear",
    "borrajearon": "borrajear",
    "borrajeará": "borrajear",
    "borrajearán": "borrajear",
    "borrajearás": "borrajear",
    "borrajearé": "borrajear",
    "borrajearéis": "borrajear",
    "borrajearía": "borrajear",
    "borrajearíais": "borrajear",
    "borrajearíamos": "borrajear",
    "borrajearían": "borrajear",
    "borrajearías": "borrajear",
    "borrajeas": "borrajear",
    "borrajease": "borrajear",
    "borrajeaseis": "borrajear",
    "borrajeasen": "borrajear",
    "borrajeases": "borrajear",
    "borrajeaste": "borrajear",
    "borrajeasteis": "borrajear",
    "borrajee": "borrajear",
    "borrajeemos": "borrajear",
    "borrajeen": "borrajear",
    "borrajees": "borrajear",
    "borrajeo": "borrajear",
    "borrajeábamos": "borrajear",
    "borrajeáis": "borrajear",
    "borrajeáramos": "borrajear",
    "borrajeáremos": "borrajear",
    "borrajeásemos": "borrajear",
    "borrajeé": "borrajear",
    "borrajeéis": "borrajear",
    "borrajeó": "borrajear",
    "borrajos": "borrajo",
    "borramos": "borrar",
    "borran": "borrar",
    "borrando": "borrar",
    "borrara": "borrar",
    "borrarais": "borrar",
    "borraran": "borrar",
    "borraras": "borrar",
    "borrare": "borrar",
    "borrareis": "borrar",
    "borraremos": "borrar",
    "borraren": "borrar",
    "borrares": "borrar",
    "borraron": "borrar",
    "borrará": "borrar",
    "borrarán": "borrar",
    "borrarás": "borrar",
    "borraré": "borrar",
    "borraréis": "borrar",
    "borraría": "borrar",
    "borraríais": "borrar",
    "borraríamos": "borrar",
    "borrarían": "borrar",
    "borrarías": "borrar",
    "borras": "borrar",
    "borrascas": "borrasca",
    "borrascosa": "borrascoso",
    "borrascosas": "borrascoso",
    "borrascosos": "borrascoso",
    "borrase": "borrar",
    "borraseis": "borrar",
    "borrasen": "borrar",
    "borrases": "borrar",
    "borrasquera": "borrasquero",
    "borrasqueras": "borrasquero",
    "borrasqueros": "borrasquero",
    "borraste": "borrar",
    "borrasteis": "borrar",
    "borre": "borrar",
    "borrega": "borrego",
    "borregadas": "borregada",
    "borregas": "borrego",
    "borregos": "borrego",
    "borreguera": "borreguero",
    "borregueras": "borreguero",
    "borregueros": "borreguero",
    "borreguiles": "borreguil",
    "borremos": "borrar",
    "borren": "borrar",
    "borrenes": "borrén",
    "borres": "borrar",
    "borrica": "borrico",
    "borricadas": "borricada",
    "borricales": "borrical",
    "borricas": "borrico",
    "borricones": "borricón",
    "borricos": "borrico",
    "borricotes": "borricote",
    "borriquera": "borriquero",
    "borriqueras": "borriquero",
    "borriqueros": "borriquero",
    "borriquetes": "borriquete",
    "borriqueña": "borriqueño",
    "borriqueñas": "borriqueño",
    "borriqueños": "borriqueño",
    "borro": "borrar",
    "borrominesca": "borrominesco",
    "borrominescas": "borrominesco",
    "borrominescos": "borrominesco",
    "borronea": "borronear",
    "borroneaba": "borronear",
    "borroneabais": "borronear",
    "borroneaban": "borronear",
    "borroneabas": "borronear",
    "borronead": "borronear",
    "borroneada": "borronear",
    "borroneadas": "borronear",
    "borroneado": "borronear",
    "borroneados": "borronear",
    "borroneamos": "borronear",
    "borronean": "borronear",
    "borroneando": "borronear",
    "borroneara": "borronear",
    "borronearais": "borronear",
    "borronearan": "borronear",
    "borronearas": "borronear",
    "borroneare": "borronear",
    "borroneareis": "borronear",
    "borronearemos": "borronear",
    "borronearen": "borronear",
    "borroneares": "borronear",
    "borronearon": "borronear",
    "borroneará": "borronear",
    "borronearán": "borronear",
    "borronearás": "borronear",
    "borronearé": "borronear",
    "borronearéis": "borronear",
    "borronearía": "borronear",
    "borronearíais": "borronear",
    "borronearíamos": "borronear",
    "borronearían": "borronear",
    "borronearías": "borronear",
    "borroneas": "borronear",
    "borronease": "borronear",
    "borroneaseis": "borronear",
    "borroneasen": "borronear",
    "borroneases": "borronear",
    "borroneaste": "borronear",
    "borroneasteis": "borronear",
    "borronee": "borronear",
    "borroneemos": "borronear",
    "borroneen": "borronear",
    "borronees": "borronear",
    "borroneo": "borronear",
    "borrones": "borrón",
    "borroneábamos": "borronear",
    "borroneáis": "borronear",
    "borroneáramos": "borronear",
    "borroneáremos": "borronear",
    "borroneásemos": "borronear",
    "borroneé": "borronear",
    "borroneéis": "borronear",
    "borroneó": "borronear",
    "borros": "borro",
    "borrosa": "borroso",
    "borrosas": "borroso",
    "borrosidades": "borrosidad",
    "borrosos": "borroso",
    "borrumbadas": "borrumbada",
    "borrábamos": "borrar",
    "borráis": "borrar",
    "borráramos": "borrar",
    "borráremos": "borrar",
    "borrásemos": "borrar",
    "borré": "borrar",
    "borréis": "borrar",
    "borró": "borrar",
    "bortes": "bort",
    "borujos": "borujo",
    "boruraciones": "boruración",
    "boruros": "boruro",
    "bosatsus": "bosatsu",
    "boscajes": "boscaje",
    "boscosa": "boscoso",
    "boscosas": "boscoso",
    "boscosos": "boscoso",
    "bosnia": "bosnio",
    "bosnias": "bosnio",
    "bosniocroatas": "bosniocroata",
    "bosnios": "bosnio",
    "bosones": "bosón",
    "bosqueja": "bosquejar",
    "bosquejaba": "bosquejar",
    "bosquejabais": "bosquejar",
    "bosquejaban": "bosquejar",
    "bosquejabas": "bosquejar",
    "bosquejad": "bosquejar",
    "bosquejada": "bosquejar",
    "bosquejadas": "bosquejar",
    "bosquejado": "bosquejar",
    "bosquejados": "bosquejar",
    "bosquejamos": "bosquejar",
    "bosquejan": "bosquejar",
    "bosquejando": "bosquejar",
    "bosquejara": "bosquejar",
    "bosquejarais": "bosquejar",
    "bosquejaran": "bosquejar",
    "bosquejaras": "bosquejar",
    "bosquejare": "bosquejar",
    "bosquejareis": "bosquejar",
    "bosquejaremos": "bosquejar",
    "bosquejaren": "bosquejar",
    "bosquejares": "bosquejar",
    "bosquejaron": "bosquejar",
    "bosquejará": "bosquejar",
    "bosquejarán": "bosquejar",
    "bosquejarás": "bosquejar",
    "bosquejaré": "bosquejar",
    "bosquejaréis": "bosquejar",
    "bosquejaría": "bosquejar",
    "bosquejaríais": "bosquejar",
    "bosquejaríamos": "bosquejar",
    "bosquejarían": "bosquejar",
    "bosquejarías": "bosquejar",
    "bosquejas": "bosquejar",
    "bosquejase": "bosquejar",
    "bosquejaseis": "bosquejar",
    "bosquejasen": "bosquejar",
    "bosquejases": "bosquejar",
    "bosquejaste": "bosquejar",
    "bosquejasteis": "bosquejar",
    "bosqueje": "bosquejar",
    "bosquejemos": "bosquejar",
    "bosquejen": "bosquejar",
    "bosquejes": "bosquejar",
    "bosquejo": "bosquejar",
    "bosquejos": "bosquejo",
    "bosquejábamos": "bosquejar",
    "bosquejáis": "bosquejar",
    "bosquejáramos": "bosquejar",
    "bosquejáremos": "bosquejar",
    "bosquejásemos": "bosquejar",
    "bosquejé": "bosquejar",
    "bosquejéis": "bosquejar",
    "bosquejó": "bosquejar",
    "bosques": "bosque",
    "bosquetes": "bosquete",
    "bosquimana": "bosquimano",
    "bosquimanas": "bosquimano",
    "bosquimanos": "bosquimano",
    "bostas": "bosta",
    "bostece": "bostezar",
    "bostecemos": "bostezar",
    "bostecen": "bostezar",
    "bosteces": "bostezar",
    "bostecé": "bostezar",
    "bostecéis": "bostezar",
    "bosteza": "bostezar",
    "bostezaba": "bostezar",
    "bostezabais": "bostezar",
    "bostezaban": "bostezar",
    "bostezabas": "bostezar",
    "bostezad": "bostezar",
    "bostezada": "bostezar",
    "bostezadas": "bostezar",
    "bostezado": "bostezar",
    "bostezadora": "bostezador",
    "bostezadoras": "bostezador",
    "bostezadores": "bostezador",
    "bostezados": "bostezar",
    "bostezamos": "bostezar",
    "bostezan": "bostezar",
    "bostezando": "bostezar",
    "bostezara": "bostezar",
    "bostezarais": "bostezar",
    "bostezaran": "bostezar",
    "bostezaras": "bostezar",
    "bostezare": "bostezar",
    "bostezareis": "bostezar",
    "bostezaremos": "bostezar",
    "bostezaren": "bostezar",
    "bostezares": "bostezar",
    "bostezaron": "bostezar",
    "bostezará": "bostezar",
    "bostezarán": "bostezar",
    "bostezarás": "bostezar",
    "bostezaré": "bostezar",
    "bostezaréis": "bostezar",
    "bostezaría": "bostezar",
    "bostezaríais": "bostezar",
    "bostezaríamos": "bostezar",
    "bostezarían": "bostezar",
    "bostezarías": "bostezar",
    "bostezas": "bostezar",
    "bostezase": "bostezar",
    "bostezaseis": "bostezar",
    "bostezasen": "bostezar",
    "bostezases": "bostezar",
    "bostezaste": "bostezar",
    "bostezasteis": "bostezar",
    "bostezo": "bostezar",
    "bostezos": "bostezo",
    "bostezábamos": "bostezar",
    "bostezáis": "bostezar",
    "bostezáramos": "bostezar",
    "bostezáremos": "bostezar",
    "bostezásemos": "bostezar",
    "bostezó": "bostezar",
    "bostoniana": "bostoniano",
    "bostonianas": "bostoniano",
    "bostonianos": "bostoniano",
    "bota": "boto",
    "botaba": "botar",
    "botabais": "botar",
    "botaban": "botar",
    "botabas": "botar",
    "botad": "botar",
    "botada": "botar",
    "botadas": "botar",
    "botado": "botar",
    "botadora": "botador",
    "botadoras": "botador",
    "botadores": "botador",
    "botados": "botar",
    "botaduras": "botadura",
    "botafuegos": "botafuego",
    "botafumeiros": "botafumeiro",
    "botagueñas": "botagueña",
    "botalones": "botalón",
    "botamos": "botar",
    "botan": "botar",
    "botanas": "botana",
    "botando": "botar",
    "botara": "botar",
    "botarais": "botar",
    "botaran": "botar",
    "botaras": "botar",
    "botaratadas": "botaratada",
    "botarates": "botarate",
    "botare": "botar",
    "botareis": "botar",
    "botaremos": "botar",
    "botaren": "botar",
    "botares": "botar",
    "botargas": "botarga",
    "botaron": "botar",
    "botará": "botar",
    "botarán": "botar",
    "botarás": "botar",
    "botaré": "botar",
    "botaréis": "botar",
    "botaría": "botar",
    "botaríais": "botar",
    "botaríamos": "botar",
    "botarían": "botar",
    "botarías": "botar",
    "botas": "boto",
    "botase": "botar",
    "botaseis": "botar",
    "botasen": "botar",
    "botases": "botar",
    "botasillas": "botasilla",
    "botaste": "botar",
    "botasteis": "botar",
    "botavaras": "botavara",
    "bote": "botar",
    "botellas": "botella",
    "botellazos": "botellazo",
    "botellera": "botellero",
    "botelleras": "botellero",
    "botelleros": "botellero",
    "botellerías": "botellería",
    "botellines": "botellín",
    "botemos": "botar",
    "boten": "botar",
    "boteprontos": "botepronto",
    "botera": "botero",
    "boteras": "botero",
    "boteros": "botero",
    "boterías": "botería",
    "botes": "bote",
    "boticaria": "boticario",
    "boticarias": "boticario",
    "boticarios": "boticario",
    "boticas": "botica",
    "botifarradas": "botifarrada",
    "botijas": "botija",
    "botijera": "botijero",
    "botijeras": "botijero",
    "botijeros": "botijero",
    "botijos": "botijo",
    "botillas": "botilla",
    "botillera": "botillero",
    "botilleras": "botillero",
    "botilleros": "botillero",
    "botillerías": "botillería",
    "botillos": "botillo",
    "botinas": "botina",
    "botinera": "botinero",
    "botineras": "botinero",
    "botineros": "botinero",
    "botinerías": "botinería",
    "botines": "botín",
    "botionda": "botiondo",
    "botiondas": "botiondo",
    "botiondos": "botiondo",
    "botiquines": "botiquín",
    "botivoleos": "botivoleo",
    "boto": "botar",
    "botocuda": "botocudo",
    "botocudas": "botocudo",
    "botocudos": "botocudo",
    "botona": "botonar",
    "botonaba": "botonar",
    "botonabais": "botonar",
    "botonaban": "botonar",
    "botonabas": "botonar",
    "botonad": "botonar",
    "botonada": "botonar",
    "botonadas": "botonar",
    "botonado": "botonar",
    "botonados": "botonar",
    "botonaduras": "botonadura",
    "botonamos": "botonar",
    "botonan": "botonar",
    "botonando": "botonar",
    "botonara": "botonar",
    "botonarais": "botonar",
    "botonaran": "botonar",
    "botonaras": "botonar",
    "botonare": "botonar",
    "botonareis": "botonar",
    "botonaremos": "botonar",
    "botonaren": "botonar",
    "botonares": "botonar",
    "botonaron": "botonar",
    "botonará": "botonar",
    "botonarán": "botonar",
    "botonarás": "botonar",
    "botonaré": "botonar",
    "botonaréis": "botonar",
    "botonaría": "botonar",
    "botonaríais": "botonar",
    "botonaríamos": "botonar",
    "botonarían": "botonar",
    "botonarías": "botonar",
    "botonas": "botonar",
    "botonase": "botonar",
    "botonaseis": "botonar",
    "botonasen": "botonar",
    "botonases": "botonar",
    "botonaste": "botonar",
    "botonasteis": "botonar",
    "botonazos": "botonazo",
    "botoncitos": "botoncito",
    "botone": "botonar",
    "botonemos": "botonar",
    "botonen": "botonar",
    "botoneras": "botonera",
    "botoneros": "botonero",
    "botonerías": "botonería",
    "botones": "botonar",
    "botono": "botonar",
    "botonábamos": "botonar",
    "botonáis": "botonar",
    "botonáramos": "botonar",
    "botonáremos": "botonar",
    "botonásemos": "botonar",
    "botoné": "botonar",
    "botonéis": "botonar",
    "botonó": "botonar",
    "botos": "boto",
    "botswana": "botswano",
    "botswanas": "botswano",
    "botswanos": "botswano",
    "botulismos": "botulismo",
    "botutos": "botuto",
    "botábamos": "botar",
    "botáis": "botar",
    "botámenes": "botamen",
    "botánica": "botánico",
    "botánicas": "botánico",
    "botánicos": "botánico",
    "botáramos": "botar",
    "botáremos": "botar",
    "botásemos": "botar",
    "boté": "botar",
    "botéis": "botar",
    "botó": "botar",
    "boudoires": "boudoir",
    "boudoirs": "boudoir",
    "bouquets": "bouquet",
    "bournonitas": "bournonita",
    "boutiques": "boutique",
    "bovajes": "bovaje",
    "bovedillas": "bovedilla",
    "bovina": "bovino",
    "bovinas": "bovino",
    "bovinos": "bovino",
    "bovinotecnias": "bovinotecnia",
    "bowlings": "bowling",
    "boxea": "boxear",
    "boxeaba": "boxear",
    "boxeabais": "boxear",
    "boxeaban": "boxear",
    "boxeabas": "boxear",
    "boxead": "boxear",
    "boxeada": "boxear",
    "boxeadas": "boxear",
    "boxeado": "boxear",
    "boxeadora": "boxeador",
    "boxeadoras": "boxeador",
    "boxeadores": "boxeador",
    "boxeados": "boxear",
    "boxeamos": "boxear",
    "boxean": "boxear",
    "boxeando": "boxear",
    "boxeara": "boxear",
    "boxearais": "boxear",
    "boxearan": "boxear",
    "boxearas": "boxear",
    "boxeare": "boxear",
    "boxeareis": "boxear",
    "boxearemos": "boxear",
    "boxearen": "boxear",
    "boxeares": "boxear",
    "boxearon": "boxear",
    "boxeará": "boxear",
    "boxearán": "boxear",
    "boxearás": "boxear",
    "boxearé": "boxear",
    "boxearéis": "boxear",
    "boxearía": "boxear",
    "boxearíais": "boxear",
    "boxearíamos": "boxear",
    "boxearían": "boxear",
    "boxearías": "boxear",
    "boxeas": "boxear",
    "boxease": "boxear",
    "boxeaseis": "boxear",
    "boxeasen": "boxear",
    "boxeases": "boxear",
    "boxeaste": "boxear",
    "boxeasteis": "boxear",
    "boxee": "boxear",
    "boxeemos": "boxear",
    "boxeen": "boxear",
    "boxees": "boxear",
    "boxeo": "boxear",
    "boxeos": "boxeo",
    "boxes": "box",
    "boxeábamos": "boxear",
    "boxeáis": "boxear",
    "boxeáramos": "boxear",
    "boxeáremos": "boxear",
    "boxeásemos": "boxear",
    "boxeé": "boxear",
    "boxeéis": "boxear",
    "boxeó": "boxear",
    "boya": "boyar",
    "boyaba": "boyar",
    "boyabais": "boyar",
    "boyaban": "boyar",
    "boyabas": "boyar",
    "boyad": "boyar",
    "boyada": "boyar",
    "boyadas": "boyar",
    "boyado": "boyar",
    "boyados": "boyar",
    "boyales": "boyal",
    "boyamos": "boyar",
    "boyan": "boyar",
    "boyando": "boyar",
    "boyantes": "boyante",
    "boyara": "boyar",
    "boyarais": "boyar",
    "boyaran": "boyar",
    "boyaras": "boyar",
    "boyardos": "boyardo",
    "boyare": "boyar",
    "boyareis": "boyar",
    "boyaremos": "boyar",
    "boyaren": "boyar",
    "boyares": "boyar",
    "boyarines": "boyarín",
    "boyaron": "boyar",
    "boyará": "boyar",
    "boyarán": "boyar",
    "boyarás": "boyar",
    "boyaré": "boyar",
    "boyaréis": "boyar",
    "boyaría": "boyar",
    "boyaríais": "boyar",
    "boyaríamos": "boyar",
    "boyarían": "boyar",
    "boyarías": "boyar",
    "boyas": "boyar",
    "boyase": "boyar",
    "boyaseis": "boyar",
    "boyasen": "boyar",
    "boyases": "boyar",
    "boyaste": "boyar",
    "boyasteis": "boyar",
    "boye": "boyar",
    "boyemos": "boyar",
    "boyen": "boyar",
    "boyera": "boyero",
    "boyeras": "boyero",
    "boyeriza": "boyerizo",
    "boyerizas": "boyerizo",
    "boyerizos": "boyerizo",
    "boyeros": "boyero",
    "boyes": "boyar",
    "boyo": "boyar",
    "boyscouts": "boyscout",
    "boyuna": "boyuno",
    "boyunas": "boyuno",
    "boyunos": "boyuno",
    "boyábamos": "boyar",
    "boyáis": "boyar",
    "boyáramos": "boyar",
    "boyáremos": "boyar",
    "boyásemos": "boyar",
    "boyé": "boyar",
    "boyéis": "boyar",
    "boyó": "boyar",
    "bozales": "bozal",
    "bozas": "boza",
    "bozos": "bozo",
    "boñigas": "boñiga",
    "boñigos": "boñigo",
    "braca": "braco",
    "bracas": "braco",
    "bracea": "bracear",
    "braceaba": "bracear",
    "braceabais": "bracear",
    "braceaban": "bracear",
    "braceabas": "bracear",
    "bracead": "bracear",
    "braceada": "bracear",
    "braceadas": "bracear",
    "braceado": "bracear",
    "braceadora": "braceador",
    "braceadoras": "braceador",
    "braceadores": "braceador",
    "braceados": "bracear",
    "braceajes": "braceaje",
    "braceamos": "bracear",
    "bracean": "bracear",
    "braceando": "bracear",
    "braceara": "bracear",
    "bracearais": "bracear",
    "bracearan": "bracear",
    "bracearas": "bracear",
    "braceare": "bracear",
    "braceareis": "bracear",
    "bracearemos": "bracear",
    "bracearen": "bracear",
    "braceares": "bracear",
    "bracearon": "bracear",
    "braceará": "bracear",
    "bracearán": "bracear",
    "bracearás": "bracear",
    "bracearé": "bracear",
    "bracearéis": "bracear",
    "bracearía": "bracear",
    "bracearíais": "bracear",
    "bracearíamos": "bracear",
    "bracearían": "bracear",
    "bracearías": "bracear",
    "braceas": "bracear",
    "bracease": "bracear",
    "braceaseis": "bracear",
    "braceasen": "bracear",
    "braceases": "bracear",
    "braceaste": "bracear",
    "braceasteis": "bracear",
    "bracee": "bracear",
    "braceemos": "bracear",
    "braceen": "bracear",
    "bracees": "bracear",
    "braceo": "bracear",
    "braceos": "braceo",
    "bracera": "bracero",
    "braceras": "bracero",
    "braceros": "bracero",
    "braceábamos": "bracear",
    "braceáis": "bracear",
    "braceáramos": "bracear",
    "braceáremos": "bracear",
    "braceásemos": "bracear",
    "braceé": "bracear",
    "braceéis": "bracear",
    "braceó": "bracear",
    "bracillos": "bracillo",
    "bracistas": "bracista",
    "bracmanes": "bracmán",
    "bracos": "braco",
    "bradiartrias": "bradiartria",
    "bradicardias": "bradicardia",
    "bradicininas": "bradicinina",
    "bradifagias": "bradifagia",
    "bradiglosias": "bradiglosia",
    "bradilalias": "bradilalia",
    "bradilexias": "bradilexia",
    "bradipódidos": "bradipódido",
    "bradiquininas": "bradiquinina",
    "brafoneras": "brafonera",
    "bragada": "bragado",
    "bragadas": "bragado",
    "bragados": "bragado",
    "bragaduras": "bragadura",
    "bragas": "braga",
    "bragueros": "braguero",
    "braguetas": "bragueta",
    "braguetazos": "braguetazo",
    "braguetera": "braguetero",
    "bragueteras": "braguetero",
    "bragueteros": "braguetero",
    "braguetona": "braguetón",
    "braguetonas": "braguetón",
    "braguetones": "braguetón",
    "braguitas": "braguita",
    "brahmanes": "brahmán",
    "brahmanismos": "brahmanismo",
    "brahmánica": "brahmánico",
    "brahmánicas": "brahmánico",
    "brahmánicos": "brahmánico",
    "brama": "bramar",
    "bramaba": "bramar",
    "bramabais": "bramar",
    "bramaban": "bramar",
    "bramabas": "bramar",
    "bramad": "bramar",
    "bramada": "bramar",
    "bramadas": "bramar",
    "bramaderas": "bramadera",
    "bramaderos": "bramadero",
    "bramado": "bramar",
    "bramadora": "bramador",
    "bramadoras": "bramador",
    "bramadores": "bramador",
    "bramados": "bramar",
    "bramamos": "bramar",
    "braman": "bramar",
    "bramando": "bramar",
    "bramantes": "bramante",
    "bramara": "bramar",
    "bramarais": "bramar",
    "bramaran": "bramar",
    "bramaras": "bramar",
    "bramare": "bramar",
    "bramareis": "bramar",
    "bramaremos": "bramar",
    "bramaren": "bramar",
    "bramares": "bramar",
    "bramaron": "bramar",
    "bramará": "bramar",
    "bramarán": "bramar",
    "bramarás": "bramar",
    "bramaré": "bramar",
    "bramaréis": "bramar",
    "bramaría": "bramar",
    "bramaríais": "bramar",
    "bramaríamos": "bramar",
    "bramarían": "bramar",
    "bramarías": "bramar",
    "bramas": "bramar",
    "bramase": "bramar",
    "bramaseis": "bramar",
    "bramasen": "bramar",
    "bramases": "bramar",
    "bramaste": "bramar",
    "bramasteis": "bramar",
    "brame": "bramar",
    "bramemos": "bramar",
    "bramen": "bramar",
    "brames": "bramar",
    "bramidos": "bramido",
    "bramo": "bramar",
    "bramores": "bramor",
    "bramábamos": "bramar",
    "bramáis": "bramar",
    "bramáramos": "bramar",
    "bramáremos": "bramar",
    "bramásemos": "bramar",
    "bramé": "bramar",
    "braméis": "bramar",
    "bramó": "bramar",
    "brancadas": "brancada",
    "brancales": "brancal",
    "branda": "brandar",
    "brandaba": "brandar",
    "brandabais": "brandar",
    "brandaban": "brandar",
    "brandabas": "brandar",
    "brandad": "brandar",
    "brandada": "brandar",
    "brandadas": "brandar",
    "brandado": "brandar",
    "brandados": "brandar",
    "brandales": "brandal",
    "brandamos": "brandar",
    "brandan": "brandar",
    "brandando": "brandar",
    "brandara": "brandar",
    "brandarais": "brandar",
    "brandaran": "brandar",
    "brandaras": "brandar",
    "brandare": "brandar",
    "brandareis": "brandar",
    "brandaremos": "brandar",
    "brandaren": "brandar",
    "brandares": "brandar",
    "brandaron": "brandar",
    "brandará": "brandar",
    "brandarán": "brandar",
    "brandarás": "brandar",
    "brandaré": "brandar",
    "brandaréis": "brandar",
    "brandaría": "brandar",
    "brandaríais": "brandar",
    "brandaríamos": "brandar",
    "brandarían": "brandar",
    "brandarías": "brandar",
    "brandas": "brandar",
    "brandase": "brandar",
    "brandaseis": "brandar",
    "brandasen": "brandar",
    "brandases": "brandar",
    "brandaste": "brandar",
    "brandasteis": "brandar",
    "brande": "brandar",
    "brandemos": "brandar",
    "branden": "brandar",
    "brandes": "brandar",
    "brandises": "brandís",
    "brando": "brandar",
    "brandys": "brandy",
    "brandábamos": "brandar",
    "brandáis": "brandar",
    "brandáramos": "brandar",
    "brandáremos": "brandar",
    "brandásemos": "brandar",
    "brandé": "brandar",
    "brandéis": "brandar",
    "brandó": "brandar",
    "branquiales": "branquial",
    "branquias": "branquia",
    "branquiomas": "branquioma",
    "branquífera": "branquífero",
    "branquíferas": "branquífero",
    "branquíferos": "branquífero",
    "braquiales": "braquial",
    "braquicefalias": "braquicefalia",
    "braquicefalismos": "braquicefalismo",
    "braquicéfala": "braquicéfalo",
    "braquicéfalas": "braquicéfalo",
    "braquicéfalos": "braquicéfalo",
    "braquiestafilina": "braquiestafilino",
    "braquiestafilinas": "braquiestafilino",
    "braquiestafilinos": "braquiestafilino",
    "braquifaciales": "braquifacial",
    "braquignatias": "braquignatia",
    "braquigrafías": "braquigrafía",
    "braquilogías": "braquilogía",
    "braquiocefálica": "braquiocefálico",
    "braquiocefálicas": "braquiocefálico",
    "braquiocefálicos": "braquiocefálico",
    "braquiodonta": "braquiodonto",
    "braquiodontas": "braquiodonto",
    "braquiodontos": "braquiodonto",
    "braquiorradiales": "braquiorradial",
    "braquiqueilias": "braquiqueilia",
    "braquistafilina": "braquistafilino",
    "braquistafilinas": "braquistafilino",
    "braquistafilinos": "braquistafilino",
    "braquistocronas": "braquistocrona",
    "braquiuránica": "braquiuránico",
    "braquiuránicas": "braquiuránico",
    "braquiuránicos": "braquiuránico",
    "braquiópodos": "braquiópodo",
    "brasas": "brasa",
    "brascas": "brasca",
    "braserillos": "braserillo",
    "braseros": "brasero",
    "brasilera": "brasilero",
    "brasileras": "brasilero",
    "brasileros": "brasilero",
    "brasiles": "brasil",
    "brasileña": "brasileño",
    "brasileñas": "brasileño",
    "brasileños": "brasileño",
    "brasilianitas": "brasilianita",
    "brasmologías": "brasmología",
    "bratislava": "bratislavo",
    "bratislavas": "bratislavo",
    "bratislavos": "bratislavo",
    "braunitas": "braunita",
    "brava": "bravo",
    "bravas": "bravo",
    "bravatas": "bravata",
    "bravea": "bravear",
    "braveaba": "bravear",
    "braveabais": "bravear",
    "braveaban": "bravear",
    "braveabas": "bravear",
    "bravead": "bravear",
    "braveada": "bravear",
    "braveadas": "bravear",
    "braveado": "bravear",
    "braveados": "bravear",
    "braveamos": "bravear",
    "bravean": "bravear",
    "braveando": "bravear",
    "braveara": "bravear",
    "bravearais": "bravear",
    "bravearan": "bravear",
    "bravearas": "bravear",
    "braveare": "bravear",
    "braveareis": "bravear",
    "bravearemos": "bravear",
    "bravearen": "bravear",
    "braveares": "bravear",
    "bravearon": "bravear",
    "braveará": "bravear",
    "bravearán": "bravear",
    "bravearás": "bravear",
    "bravearé": "bravear",
    "bravearéis": "bravear",
    "bravearía": "bravear",
    "bravearíais": "bravear",
    "bravearíamos": "bravear",
    "bravearían": "bravear",
    "bravearías": "bravear",
    "braveas": "bravear",
    "bravease": "bravear",
    "braveaseis": "bravear",
    "braveasen": "bravear",
    "braveases": "bravear",
    "braveaste": "bravear",
    "braveasteis": "bravear",
    "bravee": "bravear",
    "braveemos": "bravear",
    "braveen": "bravear",
    "bravees": "bravear",
    "braveo": "bravear",
    "braveras": "bravera",
    "bravezas": "braveza",
    "braveábamos": "bravear",
    "braveáis": "bravear",
    "braveáramos": "bravear",
    "braveáremos": "bravear",
    "braveásemos": "bravear",
    "braveé": "bravear",
    "braveéis": "bravear",
    "braveó": "bravear",
    "bravos": "bravo",
    "bravucona": "bravucón",
    "bravuconadas": "bravuconada",
    "bravuconas": "bravucón",
    "bravuconea": "bravuconear",
    "bravuconeaba": "bravuconear",
    "bravuconeabais": "bravuconear",
    "bravuconeaban": "bravuconear",
    "bravuconeabas": "bravuconear",
    "bravuconead": "bravuconear",
    "bravuconeada": "bravuconear",
    "bravuconeadas": "bravuconear",
    "bravuconeado": "bravuconear",
    "bravuconeados": "bravuconear",
    "bravuconeamos": "bravuconear",
    "bravuconean": "bravuconear",
    "bravuconeando": "bravuconear",
    "bravuconeara": "bravuconear",
    "bravuconearais": "bravuconear",
    "bravuconearan": "bravuconear",
    "bravuconearas": "bravuconear",
    "bravuconeare": "bravuconear",
    "bravuconeareis": "bravuconear",
    "bravuconearemos": "bravuconear",
    "bravuconearen": "bravuconear",
    "bravuconeares": "bravuconear",
    "bravuconearon": "bravuconear",
    "bravuconeará": "bravuconear",
    "bravuconearán": "bravuconear",
    "bravuconearás": "bravuconear",
    "bravuconearé": "bravuconear",
    "bravuconearéis": "bravuconear",
    "bravuconearía": "bravuconear",
    "bravuconearíais": "bravuconear",
    "bravuconearíamos": "bravuconear",
    "bravuconearían": "bravuconear",
    "bravuconearías": "bravuconear",
    "bravuconeas": "bravuconear",
    "bravuconease": "bravuconear",
    "bravuconeaseis": "bravuconear",
    "bravuconeasen": "bravuconear",
    "bravuconeases": "bravuconear",
    "bravuconeaste": "bravuconear",
    "bravuconeasteis": "bravuconear",
    "bravuconee": "bravuconear",
    "bravuconeemos": "bravuconear",
    "bravuconeen": "bravuconear",
    "bravuconees": "bravuconear",
    "bravuconeo": "bravuconear",
    "bravuconerías": "bravuconería",
    "bravucones": "bravucón",
    "bravuconeábamos": "bravuconear",
    "bravuconeáis": "bravuconear",
    "bravuconeáramos": "bravuconear",
    "bravuconeáremos": "bravuconear",
    "bravuconeásemos": "bravuconear",
    "bravuconeé": "bravuconear",
    "bravuconeéis": "bravuconear",
    "bravuconeó": "bravuconear",
    "bravuras": "bravura",
    "bravía": "bravío",
    "bravías": "bravío",
    "bravíos": "bravío",
    "brazadas": "brazada",
    "brazados": "brazado",
    "brazajes": "brazaje",
    "brazaleras": "brazalera",
    "brazales": "brazal",
    "brazaletes": "brazalete",
    "brazas": "braza",
    "brazolas": "brazola",
    "brazos": "brazo",
    "brazuelos": "brazuelo",
    "brea": "brear",
    "breaba": "brear",
    "breabais": "brear",
    "breaban": "brear",
    "breabas": "brear",
    "bread": "brear",
    "breada": "brear",
    "breadas": "brear",
    "breado": "brear",
    "breados": "brear",
    "breakpoints": "breakpoint",
    "breaks": "break",
    "breamos": "brear",
    "brean": "brear",
    "breando": "brear",
    "breara": "brear",
    "brearais": "brear",
    "brearan": "brear",
    "brearas": "brear",
    "breare": "brear",
    "breareis": "brear",
    "brearemos": "brear",
    "brearen": "brear",
    "breares": "brear",
    "brearon": "brear",
    "breará": "brear",
    "brearán": "brear",
    "brearás": "brear",
    "brearé": "brear",
    "brearéis": "brear",
    "brearía": "brear",
    "brearíais": "brear",
    "brearíamos": "brear",
    "brearían": "brear",
    "brearías": "brear",
    "breas": "brear",
    "brease": "brear",
    "breaseis": "brear",
    "breasen": "brear",
    "breases": "brear",
    "breaste": "brear",
    "breasteis": "brear",
    "brebajes": "brebaje",
    "brecas": "breca",
    "brechas": "brecha",
    "bree": "brear",
    "breemos": "brear",
    "breen": "brear",
    "brees": "brear",
    "brega": "bregar",
    "bregaba": "bregar",
    "bregabais": "bregar",
    "bregaban": "bregar",
    "bregabas": "bregar",
    "bregad": "bregar",
    "bregada": "bregar",
    "bregadas": "bregar",
    "bregado": "bregar",
    "bregados": "bregar",
    "bregamos": "bregar",
    "bregan": "bregar",
    "bregando": "bregar",
    "bregara": "bregar",
    "bregarais": "bregar",
    "bregaran": "bregar",
    "bregaras": "bregar",
    "bregare": "bregar",
    "bregareis": "bregar",
    "bregaremos": "bregar",
    "bregaren": "bregar",
    "bregares": "bregar",
    "bregaron": "bregar",
    "bregará": "bregar",
    "bregarán": "bregar",
    "bregarás": "bregar",
    "bregaré": "bregar",
    "bregaréis": "bregar",
    "bregaría": "bregar",
    "bregaríais": "bregar",
    "bregaríamos": "bregar",
    "bregarían": "bregar",
    "bregarías": "bregar",
    "bregas": "bregar",
    "bregase": "bregar",
    "bregaseis": "bregar",
    "bregasen": "bregar",
    "bregases": "bregar",
    "bregaste": "bregar",
    "bregasteis": "bregar",
    "bregmas": "bregma",
    "bregmática": "bregmático",
    "bregmáticas": "bregmático",
    "bregmáticos": "bregmático",
    "brego": "bregar",
    "bregue": "bregar",
    "breguemos": "bregar",
    "breguen": "bregar",
    "bregues": "bregar",
    "bregué": "bregar",
    "breguéis": "bregar",
    "bregábamos": "bregar",
    "bregáis": "bregar",
    "bregáramos": "bregar",
    "bregáremos": "bregar",
    "bregásemos": "bregar",
    "bregó": "bregar",
    "brencas": "brenca",
    "brennnosa": "brennnoso",
    "brennnosas": "brennnoso",
    "brennnosos": "brennnoso",
    "breo": "brear",
    "bresca": "brescar",
    "brescaba": "brescar",
    "brescabais": "brescar",
    "brescaban": "brescar",
    "brescabas": "brescar",
    "brescad": "brescar",
    "brescada": "brescar",
    "brescadas": "brescar",
    "brescado": "brescar",
    "brescados": "brescar",
    "brescamos": "brescar",
    "brescan": "brescar",
    "brescando": "brescar",
    "brescara": "brescar",
    "brescarais": "brescar",
    "brescaran": "brescar",
    "brescaras": "brescar",
    "brescare": "brescar",
    "brescareis": "brescar",
    "brescaremos": "brescar",
    "brescaren": "brescar",
    "brescares": "brescar",
    "brescaron": "brescar",
    "brescará": "brescar",
    "brescarán": "brescar",
    "brescarás": "brescar",
    "brescaré": "brescar",
    "brescaréis": "brescar",
    "brescaría": "brescar",
    "brescaríais": "brescar",
    "brescaríamos": "brescar",
    "brescarían": "brescar",
    "brescarías": "brescar",
    "brescas": "brescar",
    "brescase": "brescar",
    "brescaseis": "brescar",
    "brescasen": "brescar",
    "brescases": "brescar",
    "brescaste": "brescar",
    "brescasteis": "brescar",
    "bresco": "brescar",
    "brescábamos": "brescar",
    "brescáis": "brescar",
    "brescáramos": "brescar",
    "brescáremos": "brescar",
    "brescásemos": "brescar",
    "brescó": "brescar",
    "bresque": "brescar",
    "bresquemos": "brescar",
    "bresquen": "brescar",
    "bresques": "brescar",
    "bresqué": "brescar",
    "bresquéis": "brescar",
    "bretes": "brete",
    "bretona": "bretón",
    "bretonas": "bretón",
    "bretones": "bretón",
    "brevas": "breva",
    "brevedades": "brevedad",
    "breves": "breve",
    "breviarios": "breviario",
    "brezales": "brezal",
    "brezos": "brezo",
    "breábamos": "brear",
    "breáis": "brear",
    "breáramos": "brear",
    "breáremos": "brear",
    "breásemos": "brear",
    "breé": "brear",
    "breéis": "brear",
    "breñales": "breñal",
    "breñares": "breñar",
    "breñas": "breña",
    "breñosa": "breñoso",
    "breñosas": "breñoso",
    "breñosos": "breñoso",
    "breó": "brear",
    "briagas": "briaga",
    "briales": "brial",
    "briba": "bribar",
    "bribaba": "bribar",
    "bribabais": "bribar",
    "bribaban": "bribar",
    "bribabas": "bribar",
    "bribad": "bribar",
    "bribada": "bribar",
    "bribadas": "bribar",
    "bribado": "bribar",
    "bribados": "bribar",
    "bribamos": "bribar",
    "briban": "bribar",
    "bribando": "bribar",
    "bribara": "bribar",
    "bribarais": "bribar",
    "bribaran": "bribar",
    "bribaras": "bribar",
    "bribare": "bribar",
    "bribareis": "bribar",
    "bribaremos": "bribar",
    "bribaren": "bribar",
    "bribares": "bribar",
    "bribaron": "bribar",
    "bribará": "bribar",
    "bribarán": "bribar",
    "bribarás": "bribar",
    "bribaré": "bribar",
    "bribaréis": "bribar",
    "bribaría": "bribar",
    "bribaríais": "bribar",
    "bribaríamos": "bribar",
    "bribarían": "bribar",
    "bribarías": "bribar",
    "bribas": "bribar",
    "bribase": "bribar",
    "bribaseis": "bribar",
    "bribasen": "bribar",
    "bribases": "bribar",
    "bribaste": "bribar",
    "bribasteis": "bribar",
    "bribe": "bribar",
    "bribemos": "bribar",
    "briben": "bribar",
    "bribes": "bribar",
    "bribia": "bribiar",
    "bribiaba": "bribiar",
    "bribiabais": "bribiar",
    "bribiaban": "bribiar",
    "bribiabas": "bribiar",
    "bribiad": "bribiar",
    "bribiada": "bribiar",
    "bribiadas": "bribiar",
    "bribiado": "bribiar",
    "bribiados": "bribiar",
    "bribiamos": "bribiar",
    "bribian": "bribiar",
    "bribiando": "bribiar",
    "bribiara": "bribiar",
    "bribiarais": "bribiar",
    "bribiaran": "bribiar",
    "bribiaras": "bribiar",
    "bribiare": "bribiar",
    "bribiareis": "bribiar",
    "bribiaremos": "bribiar",
    "bribiaren": "bribiar",
    "bribiares": "bribiar",
    "bribiaron": "bribiar",
    "bribiará": "bribiar",
    "bribiarán": "bribiar",
    "bribiarás": "bribiar",
    "bribiaré": "bribiar",
    "bribiaréis": "bribiar",
    "bribiaría": "bribiar",
    "bribiaríais": "bribiar",
    "bribiaríamos": "bribiar",
    "bribiarían": "bribiar",
    "bribiarías": "bribiar",
    "bribias": "bribiar",
    "bribiase": "bribiar",
    "bribiaseis": "bribiar",
    "bribiasen": "bribiar",
    "bribiases": "bribiar",
    "bribiaste": "bribiar",
    "bribiasteis": "bribiar",
    "bribie": "bribiar",
    "bribiemos": "bribiar",
    "bribien": "bribiar",
    "bribies": "bribiar",
    "bribio": "bribiar",
    "bribiábamos": "bribiar",
    "bribiáis": "bribiar",
    "bribiáramos": "bribiar",
    "bribiáremos": "bribiar",
    "bribiásemos": "bribiar",
    "bribié": "bribiar",
    "bribiéis": "bribiar",
    "bribió": "bribiar",
    "bribo": "bribar",
    "bribona": "bribón",
    "bribonadas": "bribonada",
    "bribonas": "bribón",
    "bribonea": "bribonear",
    "briboneaba": "bribonear",
    "briboneabais": "bribonear",
    "briboneaban": "bribonear",
    "briboneabas": "bribonear",
    "bribonead": "bribonear",
    "briboneada": "bribonear",
    "briboneadas": "bribonear",
    "briboneado": "bribonear",
    "briboneados": "bribonear",
    "briboneamos": "bribonear",
    "bribonean": "bribonear",
    "briboneando": "bribonear",
    "briboneara": "bribonear",
    "bribonearais": "bribonear",
    "bribonearan": "bribonear",
    "bribonearas": "bribonear",
    "briboneare": "bribonear",
    "briboneareis": "bribonear",
    "bribonearemos": "bribonear",
    "bribonearen": "bribonear",
    "briboneares": "bribonear",
    "bribonearon": "bribonear",
    "briboneará": "bribonear",
    "bribonearán": "bribonear",
    "bribonearás": "bribonear",
    "bribonearé": "bribonear",
    "bribonearéis": "bribonear",
    "bribonearía": "bribonear",
    "bribonearíais": "bribonear",
    "bribonearíamos": "bribonear",
    "bribonearían": "bribonear",
    "bribonearías": "bribonear",
    "briboneas": "bribonear",
    "bribonease": "bribonear",
    "briboneaseis": "bribonear",
    "briboneasen": "bribonear",
    "briboneases": "bribonear",
    "briboneaste": "bribonear",
    "briboneasteis": "bribonear",
    "bribonee": "bribonear",
    "briboneemos": "bribonear",
    "briboneen": "bribonear",
    "bribonees": "bribonear",
    "briboneo": "bribonear",
    "bribonerías": "bribonería",
    "bribones": "bribón",
    "bribonesca": "bribonesco",
    "bribonescas": "bribonesco",
    "bribonescos": "bribonesco",
    "briboneábamos": "bribonear",
    "briboneáis": "bribonear",
    "briboneáramos": "bribonear",
    "briboneáremos": "bribonear",
    "briboneásemos": "bribonear",
    "briboneé": "bribonear",
    "briboneéis": "bribonear",
    "briboneó": "bribonear",
    "bribábamos": "bribar",
    "bribáis": "bribar",
    "bribáramos": "bribar",
    "bribáremos": "bribar",
    "bribásemos": "bribar",
    "bribé": "bribar",
    "bribéis": "bribar",
    "bribó": "bribar",
    "brichos": "bricho",
    "bricks": "brick",
    "bricolages": "bricolage",
    "bricolajes": "bricolaje",
    "bricomanías": "bricomanía",
    "brida": "bridar",
    "bridaba": "bridar",
    "bridabais": "bridar",
    "bridaban": "bridar",
    "bridabas": "bridar",
    "bridad": "bridar",
    "bridada": "bridar",
    "bridadas": "bridar",
    "bridado": "bridar",
    "bridados": "bridar",
    "bridamos": "bridar",
    "bridan": "bridar",
    "bridando": "bridar",
    "bridara": "bridar",
    "bridarais": "bridar",
    "bridaran": "bridar",
    "bridaras": "bridar",
    "bridare": "bridar",
    "bridareis": "bridar",
    "bridaremos": "bridar",
    "bridaren": "bridar",
    "bridares": "bridar",
    "bridaron": "bridar",
    "bridará": "bridar",
    "bridarán": "bridar",
    "bridarás": "bridar",
    "bridaré": "bridar",
    "bridaréis": "bridar",
    "bridaría": "bridar",
    "bridaríais": "bridar",
    "bridaríamos": "bridar",
    "bridarían": "bridar",
    "bridarías": "bridar",
    "bridas": "bridar",
    "bridase": "bridar",
    "bridaseis": "bridar",
    "bridasen": "bridar",
    "bridases": "bridar",
    "bridaste": "bridar",
    "bridasteis": "bridar",
    "bride": "bridar",
    "bridemos": "bridar",
    "briden": "bridar",
    "brides": "bridar",
    "bridges": "bridge",
    "brido": "bridar",
    "bridones": "bridón",
    "bridábamos": "bridar",
    "bridáis": "bridar",
    "bridáramos": "bridar",
    "bridáremos": "bridar",
    "bridásemos": "bridar",
    "bridé": "bridar",
    "bridéis": "bridar",
    "bridó": "bridar",
    "brigadas": "brigada",
    "brigadieres": "brigadier",
    "brigadistas": "brigadista",
    "brigantina": "brigantino",
    "brigantinas": "brigantino",
    "brigantinos": "brigantino",
    "brigolas": "brigola",
    "brilla": "brillar",
    "brillaba": "brillar",
    "brillabais": "brillar",
    "brillaban": "brillar",
    "brillabas": "brillar",
    "brillad": "brillar",
    "brillada": "brillar",
    "brilladas": "brillar",
    "brillado": "brillar",
    "brilladora": "brillador",
    "brilladoras": "brillador",
    "brilladores": "brillador",
    "brillados": "brillar",
    "brillamos": "brillar",
    "brillan": "brillar",
    "brillancímetros": "brillancímetro",
    "brillando": "brillar",
    "brillanteces": "brillantez",
    "brillantes": "brillante",
    "brillantinas": "brillantina",
    "brillara": "brillar",
    "brillarais": "brillar",
    "brillaran": "brillar",
    "brillaras": "brillar",
    "brillare": "brillar",
    "brillareis": "brillar",
    "brillaremos": "brillar",
    "brillaren": "brillar",
    "brillares": "brillar",
    "brillaron": "brillar",
    "brillará": "brillar",
    "brillarán": "brillar",
    "brillarás": "brillar",
    "brillaré": "brillar",
    "brillaréis": "brillar",
    "brillaría": "brillar",
    "brillaríais": "brillar",
    "brillaríamos": "brillar",
    "brillarían": "brillar",
    "brillarías": "brillar",
    "brillas": "brillar",
    "brillase": "brillar",
    "brillaseis": "brillar",
    "brillasen": "brillar",
    "brillases": "brillar",
    "brillaste": "brillar",
    "brillasteis": "brillar",
    "brille": "brillar",
    "brillemos": "brillar",
    "brillen": "brillar",
    "brilles": "brillar",
    "brillo": "brillar",
    "brillos": "brillo",
    "brillábamos": "brillar",
    "brilláis": "brillar",
    "brilláramos": "brillar",
    "brilláremos": "brillar",
    "brillásemos": "brillar",
    "brillé": "brillar",
    "brilléis": "brillar",
    "brilló": "brillar",
    "brillómetros": "brillómetro",
    "brinca": "brincar",
    "brincaba": "brincar",
    "brincabais": "brincar",
    "brincaban": "brincar",
    "brincabas": "brincar",
    "brincad": "brincar",
    "brincada": "brincar",
    "brincadas": "brincar",
    "brincado": "brincar",
    "brincados": "brincar",
    "brincamos": "brincar",
    "brincan": "brincar",
    "brincando": "brincar",
    "brincara": "brincar",
    "brincarais": "brincar",
    "brincaran": "brincar",
    "brincaras": "brincar",
    "brincare": "brincar",
    "brincareis": "brincar",
    "brincaremos": "brincar",
    "brincaren": "brincar",
    "brincares": "brincar",
    "brincaron": "brincar",
    "brincará": "brincar",
    "brincarán": "brincar",
    "brincarás": "brincar",
    "brincaré": "brincar",
    "brincaréis": "brincar",
    "brincaría": "brincar",
    "brincaríais": "brincar",
    "brincaríamos": "brincar",
    "brincarían": "brincar",
    "brincarías": "brincar",
    "brincas": "brincar",
    "brincase": "brincar",
    "brincaseis": "brincar",
    "brincasen": "brincar",
    "brincases": "brincar",
    "brincaste": "brincar",
    "brincasteis": "brincar",
    "brinchos": "brincho",
    "brinco": "brincar",
    "brincos": "brinco",
    "brincábamos": "brincar",
    "brincáis": "brincar",
    "brincáramos": "brincar",
    "brincáremos": "brincar",
    "brincásemos": "brincar",
    "brincó": "brincar",
    "brinda": "brindar",
    "brindaba": "brindar",
    "brindabais": "brindar",
    "brindaban": "brindar",
    "brindabas": "brindar",
    "brindad": "brindar",
    "brindada": "brindar",
    "brindadas": "brindar",
    "brindado": "brindar",
    "brindados": "brindar",
    "brindamos": "brindar",
    "brindan": "brindar",
    "brindando": "brindar",
    "brindara": "brindar",
    "brindarais": "brindar",
    "brindaran": "brindar",
    "brindaras": "brindar",
    "brindare": "brindar",
    "brindareis": "brindar",
    "brindaremos": "brindar",
    "brindaren": "brindar",
    "brindares": "brindar",
    "brindaron": "brindar",
    "brindará": "brindar",
    "brindarán": "brindar",
    "brindarás": "brindar",
    "brindaré": "brindar",
    "brindaréis": "brindar",
    "brindaría": "brindar",
    "brindaríais": "brindar",
    "brindaríamos": "brindar",
    "brindarían": "brindar",
    "brindarías": "brindar",
    "brindas": "brindar",
    "brindase": "brindar",
    "brindaseis": "brindar",
    "brindasen": "brindar",
    "brindases": "brindar",
    "brindaste": "brindar",
    "brindasteis": "brindar",
    "brinde": "brindar",
    "brindemos": "brindar",
    "brinden": "brindar",
    "brindes": "brindar",
    "brindo": "brindar",
    "brindábamos": "brindar",
    "brindáis": "brindar",
    "brindáramos": "brindar",
    "brindáremos": "brindar",
    "brindásemos": "brindar",
    "brindé": "brindar",
    "brindéis": "brindar",
    "brindó": "brindar",
    "brines": "brin",
    "brinque": "brincar",
    "brinquemos": "brincar",
    "brinquen": "brincar",
    "brinques": "brincar",
    "brinqué": "brincar",
    "brinquéis": "brincar",
    "brioches": "brioche",
    "brioles": "briol",
    "briologías": "briología",
    "briológica": "briológico",
    "briológicas": "briológico",
    "briológicos": "briológico",
    "briosa": "brioso",
    "briosas": "brioso",
    "briosos": "brioso",
    "briozoos": "briozoo",
    "briquetas": "briqueta",
    "brisas": "brisa",
    "briscada": "briscado",
    "briscadas": "briscado",
    "briscados": "briscado",
    "briscas": "brisca",
    "brisuras": "brisura",
    "britana": "britano",
    "britanas": "britano",
    "britanos": "britano",
    "británica": "británico",
    "británicas": "británico",
    "británicos": "británico",
    "britónica": "britónico",
    "britónicas": "britónico",
    "britónicos": "britónico",
    "briznas": "brizna",
    "brizos": "brizo",
    "briófitos": "briófito",
    "broas": "broa",
    "brocadillos": "brocadillo",
    "brocados": "brocado",
    "brocales": "brocal",
    "brocamantones": "brocamantón",
    "brocantitas": "brocantita",
    "brocas": "broca",
    "brocateles": "brocatel",
    "brocea": "brocear",
    "broceaba": "brocear",
    "broceabais": "brocear",
    "broceaban": "brocear",
    "broceabas": "brocear",
    "brocead": "brocear",
    "broceada": "brocear",
    "broceadas": "brocear",
    "broceado": "brocear",
    "broceados": "brocear",
    "broceamos": "brocear",
    "brocean": "brocear",
    "broceando": "brocear",
    "broceara": "brocear",
    "brocearais": "brocear",
    "brocearan": "brocear",
    "brocearas": "brocear",
    "broceare": "brocear",
    "broceareis": "brocear",
    "brocearemos": "brocear",
    "brocearen": "brocear",
    "broceares": "brocear",
    "brocearon": "brocear",
    "broceará": "brocear",
    "brocearán": "brocear",
    "brocearás": "brocear",
    "brocearé": "brocear",
    "brocearéis": "brocear",
    "brocearía": "brocear",
    "brocearíais": "brocear",
    "brocearíamos": "brocear",
    "brocearían": "brocear",
    "brocearías": "brocear",
    "broceas": "brocear",
    "brocease": "brocear",
    "broceaseis": "brocear",
    "broceasen": "brocear",
    "broceases": "brocear",
    "broceaste": "brocear",
    "broceasteis": "brocear",
    "brocee": "brocear",
    "broceemos": "brocear",
    "broceen": "brocear",
    "brocees": "brocear",
    "broceo": "brocear",
    "broceábamos": "brocear",
    "broceáis": "brocear",
    "broceáramos": "brocear",
    "broceáremos": "brocear",
    "broceásemos": "brocear",
    "broceé": "brocear",
    "broceéis": "brocear",
    "broceó": "brocear",
    "brochada": "brochado",
    "brochadas": "brochado",
    "brochadoras": "brochadora",
    "brochados": "brochado",
    "brochales": "brochal",
    "brochas": "brocha",
    "brochazos": "brochazo",
    "broches": "broche",
    "brochetas": "brocheta",
    "brochones": "brochón",
    "brocárdicos": "brocárdico",
    "brokeres": "broker",
    "brokers": "broker",
    "broma": "bromar",
    "bromaba": "bromar",
    "bromabais": "bromar",
    "bromaban": "bromar",
    "bromabas": "bromar",
    "bromaciones": "bromación",
    "bromad": "bromar",
    "bromada": "bromar",
    "bromadas": "bromar",
    "bromado": "bromar",
    "bromados": "bromar",
    "bromamos": "bromar",
    "broman": "bromar",
    "bromando": "bromar",
    "bromara": "bromar",
    "bromarais": "bromar",
    "bromaran": "bromar",
    "bromaras": "bromar",
    "bromare": "bromar",
    "bromareis": "bromar",
    "bromaremos": "bromar",
    "bromaren": "bromar",
    "bromares": "bromar",
    "bromaron": "bromar",
    "bromará": "bromar",
    "bromarán": "bromar",
    "bromarás": "bromar",
    "bromaré": "bromar",
    "bromaréis": "bromar",
    "bromaría": "bromar",
    "bromaríais": "bromar",
    "bromaríamos": "bromar",
    "bromarían": "bromar",
    "bromarías": "bromar",
    "bromas": "bromar",
    "bromase": "bromar",
    "bromaseis": "bromar",
    "bromasen": "bromar",
    "bromases": "bromar",
    "bromaste": "bromar",
    "bromasteis": "bromar",
    "bromatologías": "bromatología",
    "bromatos": "bromato",
    "bromatóloga": "bromatólogo",
    "bromatólogas": "bromatólogo",
    "bromatólogos": "bromatólogo",
    "brome": "bromar",
    "bromea": "bromear",
    "bromeaba": "bromear",
    "bromeabais": "bromear",
    "bromeaban": "bromear",
    "bromeabas": "bromear",
    "bromead": "bromear",
    "bromeada": "bromear",
    "bromeadas": "bromear",
    "bromeado": "bromear",
    "bromeados": "bromear",
    "bromeamos": "bromear",
    "bromean": "bromear",
    "bromeando": "bromear",
    "bromeara": "bromear",
    "bromearais": "bromear",
    "bromearan": "bromear",
    "bromearas": "bromear",
    "bromeare": "bromear",
    "bromeareis": "bromear",
    "bromearemos": "bromear",
    "bromearen": "bromear",
    "bromeares": "bromear",
    "bromearon": "bromear",
    "bromeará": "bromear",
    "bromearán": "bromear",
    "bromearás": "bromear",
    "bromearé": "bromear",
    "bromearéis": "bromear",
    "bromearía": "bromear",
    "bromearíais": "bromear",
    "bromearíamos": "bromear",
    "bromearían": "bromear",
    "bromearías": "bromear",
    "bromeas": "bromear",
    "bromease": "bromear",
    "bromeaseis": "bromear",
    "bromeasen": "bromear",
    "bromeases": "bromear",
    "bromeaste": "bromear",
    "bromeasteis": "bromear",
    "bromee": "bromear",
    "bromeemos": "bromear",
    "bromeen": "bromear",
    "bromees": "bromear",
    "bromeliáceas": "bromeliácea",
    "bromemos": "bromar",
    "bromen": "bromar",
    "bromeo": "bromear",
    "bromes": "bromar",
    "bromeábamos": "bromear",
    "bromeáis": "bromear",
    "bromeáramos": "bromear",
    "bromeáremos": "bromear",
    "bromeásemos": "bromear",
    "bromeé": "bromear",
    "bromeéis": "bromear",
    "bromeó": "bromear",
    "bromhídrica": "bromhídrico",
    "bromhídricas": "bromhídrico",
    "bromhídricos": "bromhídrico",
    "bromiritas": "bromirita",
    "bromismos": "bromismo",
    "bromistas": "bromista",
    "bromo": "bromar",
    "bromoacetatos": "bromoacetato",
    "bromoacetonas": "bromoacetona",
    "bromobencenos": "bromobenceno",
    "bromoclorodimetilhidantoínas": "bromoclorodimetilhidantoína",
    "bromocloroetanos": "bromocloroetano",
    "bromodietilacetilureas": "bromodietilacetilurea",
    "bromoestiroles": "bromoestirol",
    "bromoformos": "bromoformo",
    "bromofosgenos": "bromofosgeno",
    "bromos": "bromo",
    "bromotrifluorometanos": "bromotrifluorometano",
    "bromuros": "bromuro",
    "bromábamos": "bromar",
    "bromáis": "bromar",
    "bromáramos": "bromar",
    "bromáremos": "bromar",
    "bromásemos": "bromar",
    "bromé": "bromar",
    "broméis": "bromar",
    "bromó": "bromar",
    "bronca": "bronco",
    "broncas": "bronco",
    "broncea": "broncear",
    "bronceaba": "broncear",
    "bronceabais": "broncear",
    "bronceaban": "broncear",
    "bronceabas": "broncear",
    "broncead": "broncear",
    "bronceada": "broncear",
    "bronceadas": "broncear",
    "bronceado": "broncear",
    "bronceadora": "bronceador",
    "bronceadoras": "bronceador",
    "bronceadores": "bronceador",
    "bronceados": "broncear",
    "bronceamos": "broncear",
    "broncean": "broncear",
    "bronceando": "broncear",
    "bronceara": "broncear",
    "broncearais": "broncear",
    "broncearan": "broncear",
    "broncearas": "broncear",
    "bronceare": "broncear",
    "bronceareis": "broncear",
    "broncearemos": "broncear",
    "broncearen": "broncear",
    "bronceares": "broncear",
    "broncearon": "broncear",
    "bronceará": "broncear",
    "broncearán": "broncear",
    "broncearás": "broncear",
    "broncearé": "broncear",
    "broncearéis": "broncear",
    "broncearía": "broncear",
    "broncearíais": "broncear",
    "broncearíamos": "broncear",
    "broncearían": "broncear",
    "broncearías": "broncear",
    "bronceas": "broncear",
    "broncease": "broncear",
    "bronceaseis": "broncear",
    "bronceasen": "broncear",
    "bronceases": "broncear",
    "bronceaste": "broncear",
    "bronceasteis": "broncear",
    "broncee": "broncear",
    "bronceemos": "broncear",
    "bronceen": "broncear",
    "broncees": "broncear",
    "bronceo": "broncear",
    "broncerías": "broncería",
    "bronces": "bronce",
    "bronceábamos": "broncear",
    "bronceáis": "broncear",
    "bronceáramos": "broncear",
    "bronceáremos": "broncear",
    "bronceásemos": "broncear",
    "bronceé": "broncear",
    "bronceéis": "broncear",
    "bronceó": "broncear",
    "broncistas": "broncista",
    "broncitas": "broncita",
    "broncoesofágica": "broncoesofágico",
    "broncoesofágicas": "broncoesofágico",
    "broncoesofágicos": "broncoesofágico",
    "broncoespasmos": "broncoespasmo",
    "bronconeumonías": "bronconeumonía",
    "broncopatías": "broncopatía",
    "broncopulmonares": "broncopulmonar",
    "broncos": "bronco",
    "broncínea": "broncíneo",
    "broncíneas": "broncíneo",
    "broncíneos": "broncíneo",
    "bronquedades": "bronquedad",
    "bronquiales": "bronquial",
    "bronquiectasias": "bronquiectasia",
    "bronquinas": "bronquina",
    "bronquiolos": "bronquiolo",
    "bronquios": "bronquio",
    "bronquíolos": "bronquíolo",
    "brontosauros": "brontosauro",
    "brookitas": "brookita",
    "broquela": "broquelar",
    "broquelaba": "broquelar",
    "broquelabais": "broquelar",
    "broquelaban": "broquelar",
    "broquelabas": "broquelar",
    "broquelad": "broquelar",
    "broquelada": "broquelar",
    "broqueladas": "broquelar",
    "broquelado": "broquelar",
    "broquelados": "broquelar",
    "broquelamos": "broquelar",
    "broquelan": "broquelar",
    "broquelando": "broquelar",
    "broquelara": "broquelar",
    "broquelarais": "broquelar",
    "broquelaran": "broquelar",
    "broquelaras": "broquelar",
    "broquelare": "broquelar",
    "broquelareis": "broquelar",
    "broquelaremos": "broquelar",
    "broquelaren": "broquelar",
    "broquelares": "broquelar",
    "broquelaron": "broquelar",
    "broquelará": "broquelar",
    "broquelarán": "broquelar",
    "broquelarás": "broquelar",
    "broquelaré": "broquelar",
    "broquelaréis": "broquelar",
    "broquelaría": "broquelar",
    "broquelaríais": "broquelar",
    "broquelaríamos": "broquelar",
    "broquelarían": "broquelar",
    "broquelarías": "broquelar",
    "broquelas": "broquelar",
    "broquelase": "broquelar",
    "broquelaseis": "broquelar",
    "broquelasen": "broquelar",
    "broquelases": "broquelar",
    "broquelaste": "broquelar",
    "broquelasteis": "broquelar",
    "broquele": "broquelar",
    "broquelemos": "broquelar",
    "broquelen": "broquelar",
    "broqueles": "broquelar",
    "broquelillos": "broquelillo",
    "broquelo": "broquelar",
    "broquelábamos": "broquelar",
    "broqueláis": "broquelar",
    "broqueláramos": "broquelar",
    "broqueláremos": "broquelar",
    "broquelásemos": "broquelar",
    "broquelé": "broquelar",
    "broqueléis": "broquelar",
    "broqueló": "broquelar",
    "broquetas": "broqueta",
    "brota": "brotar",
    "brotaba": "brotar",
    "brotabais": "brotar",
    "brotaban": "brotar",
    "brotabas": "brotar",
    "brotad": "brotar",
    "brotada": "brotar",
    "brotadas": "brotar",
    "brotado": "brotar",
    "brotados": "brotar",
    "brotaduras": "brotadura",
    "brotamos": "brotar",
    "brotan": "brotar",
    "brotando": "brotar",
    "brotara": "brotar",
    "brotarais": "brotar",
    "brotaran": "brotar",
    "brotaras": "brotar",
    "brotare": "brotar",
    "brotareis": "brotar",
    "brotaremos": "brotar",
    "brotaren": "brotar",
    "brotares": "brotar",
    "brotaron": "brotar",
    "brotará": "brotar",
    "brotarán": "brotar",
    "brotarás": "brotar",
    "brotaré": "brotar",
    "brotaréis": "brotar",
    "brotaría": "brotar",
    "brotaríais": "brotar",
    "brotaríamos": "brotar",
    "brotarían": "brotar",
    "brotarías": "brotar",
    "brotas": "brotar",
    "brotase": "brotar",
    "brotaseis": "brotar",
    "brotasen": "brotar",
    "brotases": "brotar",
    "brotaste": "brotar",
    "brotasteis": "brotar",
    "brote": "brotar",
    "brotemos": "brotar",
    "broten": "brotar",
    "brotes": "brote",
    "broto": "brotar",
    "brotábamos": "brotar",
    "brotáis": "brotar",
    "brotáramos": "brotar",
    "brotáremos": "brotar",
    "brotásemos": "brotar",
    "broté": "brotar",
    "brotéis": "brotar",
    "brotó": "brotar",
    "browniana": "browniano",
    "brownianas": "browniano",
    "brownianos": "browniano",
    "brozadora": "brozador",
    "brozadoras": "brozador",
    "brozadores": "brozador",
    "brozas": "broza",
    "brozosa": "brozoso",
    "brozosas": "brozoso",
    "brozosos": "brozoso",
    "bruce": "bruzar",
    "brucemos": "bruzar",
    "brucen": "bruzar",
    "brucera": "brucero",
    "bruceras": "brucero",
    "bruceros": "brucero",
    "bruces": "bruzar",
    "brucinas": "brucina",
    "brucitas": "brucita",
    "brucé": "bruzar",
    "brucéis": "bruzar",
    "bruja": "brujo",
    "brujamos": "brujir",
    "brujan": "brujir",
    "brujas": "brujo",
    "bruje": "brujir",
    "brujea": "brujear",
    "brujeaba": "brujear",
    "brujeabais": "brujear",
    "brujeaban": "brujear",
    "brujeabas": "brujear",
    "brujead": "brujear",
    "brujeada": "brujear",
    "brujeadas": "brujear",
    "brujeado": "brujear",
    "brujeados": "brujear",
    "brujeamos": "brujear",
    "brujean": "brujear",
    "brujeando": "brujear",
    "brujeara": "brujear",
    "brujearais": "brujear",
    "brujearan": "brujear",
    "brujearas": "brujear",
    "brujeare": "brujear",
    "brujeareis": "brujear",
    "brujearemos": "brujear",
    "brujearen": "brujear",
    "brujeares": "brujear",
    "brujearon": "brujear",
    "brujeará": "brujear",
    "brujearán": "brujear",
    "brujearás": "brujear",
    "brujearé": "brujear",
    "brujearéis": "brujear",
    "brujearía": "brujear",
    "brujearíais": "brujear",
    "brujearíamos": "brujear",
    "brujearían": "brujear",
    "brujearías": "brujear",
    "brujeas": "brujear",
    "brujease": "brujear",
    "brujeaseis": "brujear",
    "brujeasen": "brujear",
    "brujeases": "brujear",
    "brujeaste": "brujear",
    "brujeasteis": "brujear",
    "brujee": "brujear",
    "brujeemos": "brujear",
    "brujeen": "brujear",
    "brujees": "brujear",
    "brujen": "brujir",
    "brujeo": "brujear",
    "brujerías": "brujería",
    "brujes": "brujir",
    "brujesca": "brujesco",
    "brujescas": "brujesco",
    "brujescos": "brujesco",
    "brujeábamos": "brujear",
    "brujeáis": "brujear",
    "brujeáramos": "brujear",
    "brujeáremos": "brujear",
    "brujeásemos": "brujear",
    "brujeé": "brujear",
    "brujeéis": "brujear",
    "brujeó": "brujear",
    "brujid": "brujir",
    "brujida": "brujir",
    "brujidas": "brujir",
    "brujido": "brujir",
    "brujidos": "brujir",
    "brujiendo": "brujir",
    "brujiera": "brujir",
    "brujierais": "brujir",
    "brujieran": "brujir",
    "brujieras": "brujir",
    "brujiere": "brujir",
    "brujiereis": "brujir",
    "brujieren": "brujir",
    "brujieres": "brujir",
    "brujieron": "brujir",
    "brujiese": "brujir",
    "brujieseis": "brujir",
    "brujiesen": "brujir",
    "brujieses": "brujir",
    "brujimos": "brujir",
    "brujiremos": "brujir",
    "brujirá": "brujir",
    "brujirán": "brujir",
    "brujirás": "brujir",
    "brujiré": "brujir",
    "brujiréis": "brujir",
    "brujiría": "brujir",
    "brujiríais": "brujir",
    "brujiríamos": "brujir",
    "brujirían": "brujir",
    "brujirías": "brujir",
    "brujiste": "brujir",
    "brujisteis": "brujir",
    "brujiéramos": "brujir",
    "brujiéremos": "brujir",
    "brujiésemos": "brujir",
    "brujió": "brujir",
    "brujo": "brujir",
    "brujos": "brujo",
    "brujulea": "brujulear",
    "brujuleaba": "brujulear",
    "brujuleabais": "brujulear",
    "brujuleaban": "brujulear",
    "brujuleabas": "brujulear",
    "brujulead": "brujulear",
    "brujuleada": "brujulear",
    "brujuleadas": "brujulear",
    "brujuleado": "brujulear",
    "brujuleados": "brujulear",
    "brujuleamos": "brujulear",
    "brujulean": "brujulear",
    "brujuleando": "brujulear",
    "brujuleara": "brujulear",
    "brujulearais": "brujulear",
    "brujulearan": "brujulear",
    "brujulearas": "brujulear",
    "brujuleare": "brujulear",
    "brujuleareis": "brujulear",
    "brujulearemos": "brujulear",
    "brujulearen": "brujulear",
    "brujuleares": "brujulear",
    "brujulearon": "brujulear",
    "brujuleará": "brujulear",
    "brujulearán": "brujulear",
    "brujulearás": "brujulear",
    "brujulearé": "brujulear",
    "brujulearéis": "brujulear",
    "brujulearía": "brujulear",
    "brujulearíais": "brujulear",
    "brujulearíamos": "brujulear",
    "brujulearían": "brujulear",
    "brujulearías": "brujulear",
    "brujuleas": "brujulear",
    "brujulease": "brujulear",
    "brujuleaseis": "brujulear",
    "brujuleasen": "brujulear",
    "brujuleases": "brujulear",
    "brujuleaste": "brujulear",
    "brujuleasteis": "brujulear",
    "brujulee": "brujulear",
    "brujuleemos": "brujulear",
    "brujuleen": "brujulear",
    "brujulees": "brujulear",
    "brujuleo": "brujulear",
    "brujuleos": "brujuleo",
    "brujuleábamos": "brujulear",
    "brujuleáis": "brujulear",
    "brujuleáramos": "brujulear",
    "brujuleáremos": "brujulear",
    "brujuleásemos": "brujulear",
    "brujuleé": "brujulear",
    "brujuleéis": "brujulear",
    "brujuleó": "brujulear",
    "brujáis": "brujir",
    "brují": "brujir",
    "brujía": "brujir",
    "brujíais": "brujir",
    "brujíamos": "brujir",
    "brujían": "brujir",
    "brujías": "brujir",
    "brujís": "brujir",
    "brulotes": "brulote",
    "bruma": "brumar",
    "brumaba": "brumar",
    "brumabais": "brumar",
    "brumaban": "brumar",
    "brumabas": "brumar",
    "brumad": "brumar",
    "brumada": "brumar",
    "brumadas": "brumar",
    "brumado": "brumar",
    "brumados": "brumar",
    "brumales": "brumal",
    "brumamos": "brumar",
    "bruman": "brumar",
    "brumando": "brumar",
    "brumara": "brumar",
    "brumarais": "brumar",
    "brumaran": "brumar",
    "brumaras": "brumar",
    "brumare": "brumar",
    "brumareis": "brumar",
    "brumaremos": "brumar",
    "brumaren": "brumar",
    "brumares": "brumar",
    "brumarios": "brumario",
    "brumaron": "brumar",
    "brumará": "brumar",
    "brumarán": "brumar",
    "brumarás": "brumar",
    "brumaré": "brumar",
    "brumaréis": "brumar",
    "brumaría": "brumar",
    "brumaríais": "brumar",
    "brumaríamos": "brumar",
    "brumarían": "brumar",
    "brumarías": "brumar",
    "brumas": "brumar",
    "brumase": "brumar",
    "brumaseis": "brumar",
    "brumasen": "brumar",
    "brumases": "brumar",
    "brumaste": "brumar",
    "brumasteis": "brumar",
    "brume": "brumar",
    "brumemos": "brumar",
    "brumen": "brumar",
    "brumes": "brumar",
    "brumo": "brumar",
    "brumos": "brumo",
    "brumosa": "brumoso",
    "brumosas": "brumoso",
    "brumosos": "brumoso",
    "brumábamos": "brumar",
    "brumáis": "brumar",
    "brumáramos": "brumar",
    "brumáremos": "brumar",
    "brumásemos": "brumar",
    "brumé": "brumar",
    "bruméis": "brumar",
    "brumó": "brumar",
    "bruna": "bruno",
    "brunas": "bruno",
    "brunos": "bruno",
    "brusca": "brusco",
    "bruscaderas": "bruscadera",
    "bruscas": "brusco",
    "bruscos": "brusco",
    "bruselas": "brusela",
    "bruselenses": "bruselense",
    "brusquedades": "brusquedad",
    "bruta": "bruto",
    "brutales": "brutal",
    "brutalice": "brutalizar",
    "brutalicemos": "brutalizar",
    "brutalicen": "brutalizar",
    "brutalices": "brutalizar",
    "brutalicé": "brutalizar",
    "brutalicéis": "brutalizar",
    "brutalidades": "brutalidad",
    "brutalismos": "brutalismo",
    "brutaliza": "brutalizar",
    "brutalizaba": "brutalizar",
    "brutalizabais": "brutalizar",
    "brutalizaban": "brutalizar",
    "brutalizabas": "brutalizar",
    "brutalizad": "brutalizar",
    "brutalizada": "brutalizar",
    "brutalizadas": "brutalizar",
    "brutalizado": "brutalizar",
    "brutalizados": "brutalizar",
    "brutalizamos": "brutalizar",
    "brutalizan": "brutalizar",
    "brutalizando": "brutalizar",
    "brutalizara": "brutalizar",
    "brutalizarais": "brutalizar",
    "brutalizaran": "brutalizar",
    "brutalizaras": "brutalizar",
    "brutalizare": "brutalizar",
    "brutalizareis": "brutalizar",
    "brutalizaremos": "brutalizar",
    "brutalizaren": "brutalizar",
    "brutalizares": "brutalizar",
    "brutalizaron": "brutalizar",
    "brutalizará": "brutalizar",
    "brutalizarán": "brutalizar",
    "brutalizarás": "brutalizar",
    "brutalizaré": "brutalizar",
    "brutalizaréis": "brutalizar",
    "brutalizaría": "brutalizar",
    "brutalizaríais": "brutalizar",
    "brutalizaríamos": "brutalizar",
    "brutalizarían": "brutalizar",
    "brutalizarías": "brutalizar",
    "brutalizas": "brutalizar",
    "brutalizase": "brutalizar",
    "brutalizaseis": "brutalizar",
    "brutalizasen": "brutalizar",
    "brutalizases": "brutalizar",
    "brutalizaste": "brutalizar",
    "brutalizasteis": "brutalizar",
    "brutalizo": "brutalizar",
    "brutalizábamos": "brutalizar",
    "brutalizáis": "brutalizar",
    "brutalizáramos": "brutalizar",
    "brutalizáremos": "brutalizar",
    "brutalizásemos": "brutalizar",
    "brutalizó": "brutalizar",
    "brutas": "bruto",
    "brutezas": "bruteza",
    "brutos": "bruto",
    "bruxismos": "bruxismo",
    "bruxomanías": "bruxomanía",
    "bruza": "bruzar",
    "bruzaba": "bruzar",
    "bruzabais": "bruzar",
    "bruzaban": "bruzar",
    "bruzabas": "bruzar",
    "bruzad": "bruzar",
    "bruzada": "bruzar",
    "bruzadas": "bruzar",
    "bruzado": "bruzar",
    "bruzadora": "bruzador",
    "bruzadoras": "bruzador",
    "bruzadores": "bruzador",
    "bruzados": "bruzar",
    "bruzamos": "bruzar",
    "bruzan": "bruzar",
    "bruzando": "bruzar",
    "bruzara": "bruzar",
    "bruzarais": "bruzar",
    "bruzaran": "bruzar",
    "bruzaras": "bruzar",
    "bruzare": "bruzar",
    "bruzareis": "bruzar",
    "bruzaremos": "bruzar",
    "bruzaren": "bruzar",
    "bruzares": "bruzar",
    "bruzaron": "bruzar",
    "bruzará": "bruzar",
    "bruzarán": "bruzar",
    "bruzarás": "bruzar",
    "bruzaré": "bruzar",
    "bruzaréis": "bruzar",
    "bruzaría": "bruzar",
    "bruzaríais": "bruzar",
    "bruzaríamos": "bruzar",
    "bruzarían": "bruzar",
    "bruzarías": "bruzar",
    "bruzas": "bruzar",
    "bruzase": "bruzar",
    "bruzaseis": "bruzar",
    "bruzasen": "bruzar",
    "bruzases": "bruzar",
    "bruzaste": "bruzar",
    "bruzasteis": "bruzar",
    "bruzo": "bruzar",
    "bruzábamos": "bruzar",
    "bruzáis": "bruzar",
    "bruzáramos": "bruzar",
    "bruzáremos": "bruzar",
    "bruzásemos": "bruzar",
    "bruzó": "bruzar",
    "bruña": "bruñir",
    "bruñamos": "bruñir",
    "bruñan": "bruñir",
    "bruñas": "bruñir",
    "bruñe": "bruñir",
    "bruñen": "bruñir",
    "bruñendo": "bruñir",
    "bruñera": "bruñir",
    "bruñerais": "bruñir",
    "bruñeran": "bruñir",
    "bruñeras": "bruñir",
    "bruñere": "bruñir",
    "bruñereis": "bruñir",
    "bruñeren": "bruñir",
    "bruñeres": "bruñir",
    "bruñeron": "bruñir",
    "bruñes": "bruñir",
    "bruñese": "bruñir",
    "bruñeseis": "bruñir",
    "bruñesen": "bruñir",
    "bruñeses": "bruñir",
    "bruñid": "bruñir",
    "bruñida": "bruñir",
    "bruñidas": "bruñir",
    "bruñido": "bruñir",
    "bruñidora": "bruñidor",
    "bruñidoras": "bruñidora",
    "bruñidores": "bruñidor",
    "bruñidos": "bruñir",
    "bruñimientos": "bruñimiento",
    "bruñimos": "bruñir",
    "bruñiremos": "bruñir",
    "bruñirá": "bruñir",
    "bruñirán": "bruñir",
    "bruñirás": "bruñir",
    "bruñiré": "bruñir",
    "bruñiréis": "bruñir",
    "bruñiría": "bruñir",
    "bruñiríais": "bruñir",
    "bruñiríamos": "bruñir",
    "bruñirían": "bruñir",
    "bruñirías": "bruñir",
    "bruñiste": "bruñir",
    "bruñisteis": "bruñir",
    "bruño": "bruñir",
    "bruñáis": "bruñir",
    "bruñéramos": "bruñir",
    "bruñéremos": "bruñir",
    "bruñésemos": "bruñir",
    "bruñí": "bruñir",
    "bruñía": "bruñir",
    "bruñíais": "bruñir",
    "bruñíamos": "bruñir",
    "bruñían": "bruñir",
    "bruñías": "bruñir",
    "bruñís": "bruñir",
    "bruñó": "bruñir",
    "brácteas": "bráctea",
    "brécoles": "brécol",
    "bríos": "brío",
    "bróculis": "bróculi",
    "brújulas": "brújula",
    "bubas": "buba",
    "bubis": "bubi",
    "bubones": "bubón",
    "bubosa": "buboso",
    "bubosas": "buboso",
    "bubosos": "buboso",
    "bubática": "bubático",
    "bubáticas": "bubático",
    "bubáticos": "bubático",
    "bubónica": "bubónico",
    "bubónicas": "bubónico",
    "bubónicos": "bubónico",
    "bucales": "bucal",
    "bucanera": "bucanero",
    "bucaneras": "bucanero",
    "bucaneros": "bucanero",
    "bucardos": "bucardo",
    "buccheros": "bucchero",
    "buccinadora": "buccinador",
    "buccinadoras": "buccinador",
    "buccinadores": "buccinador",
    "buccinatofaríngea": "buccinatofaríngeo",
    "buccinatofaríngeas": "buccinatofaríngeo",
    "buccinatofaríngeos": "buccinatofaríngeo",
    "buccinatoria": "buccinatorio",
    "buccinatorias": "buccinatorio",
    "buccinatorios": "buccinatorio",
    "buce": "buzar",
    "bucea": "bucear",
    "buceaba": "bucear",
    "buceabais": "bucear",
    "buceaban": "bucear",
    "buceabas": "bucear",
    "bucead": "bucear",
    "buceada": "bucear",
    "buceadas": "bucear",
    "buceado": "bucear",
    "buceadora": "buceador",
    "buceadoras": "buceador",
    "buceadores": "buceador",
    "buceados": "bucear",
    "buceamientos": "buceamiento",
    "buceamos": "bucear",
    "bucean": "bucear",
    "buceando": "bucear",
    "buceara": "bucear",
    "bucearais": "bucear",
    "bucearan": "bucear",
    "bucearas": "bucear",
    "buceare": "bucear",
    "buceareis": "bucear",
    "bucearemos": "bucear",
    "bucearen": "bucear",
    "buceares": "bucear",
    "bucearon": "bucear",
    "buceará": "bucear",
    "bucearán": "bucear",
    "bucearás": "bucear",
    "bucearé": "bucear",
    "bucearéis": "bucear",
    "bucearía": "bucear",
    "bucearíais": "bucear",
    "bucearíamos": "bucear",
    "bucearían": "bucear",
    "bucearías": "bucear",
    "buceas": "bucear",
    "bucease": "bucear",
    "buceaseis": "bucear",
    "buceasen": "bucear",
    "buceases": "bucear",
    "buceaste": "bucear",
    "buceasteis": "bucear",
    "bucee": "bucear",
    "buceemos": "bucear",
    "buceen": "bucear",
    "bucees": "bucear",
    "bucelarios": "bucelario",
    "bucemos": "buzar",
    "bucen": "buzar",
    "bucentauros": "bucentauro",
    "buceo": "bucear",
    "buceos": "buceo",
    "buces": "buzar",
    "buceábamos": "bucear",
    "buceáis": "bucear",
    "buceáramos": "bucear",
    "buceáremos": "bucear",
    "buceásemos": "bucear",
    "buceé": "bucear",
    "buceéis": "bucear",
    "buceó": "bucear",
    "buchacas": "buchaca",
    "buches": "buche",
    "buchetes": "buchete",
    "buchinches": "buchinche",
    "buchona": "buchón",
    "buchonas": "buchón",
    "buchones": "buchón",
    "bucleos": "bucleo",
    "bucles": "bucle",
    "bucocervicales": "bucocervical",
    "bucofaringes": "bucofaringe",
    "bucofaríngea": "bucofaríngeo",
    "bucofaríngeas": "bucofaríngeo",
    "bucofaríngeos": "bucofaríngeo",
    "bucolabiales": "bucolabial",
    "bucolinguales": "bucolingual",
    "bucolismos": "bucolismo",
    "bucos": "buco",
    "bucé": "buzar",
    "bucéfalos": "bucéfalo",
    "bucéis": "buzar",
    "bucólica": "bucólico",
    "bucólicas": "bucólico",
    "bucólicos": "bucólico",
    "bucónidos": "bucónido",
    "budinadoras": "budinadora",
    "budines": "budín",
    "budiones": "budión",
    "budismos": "budismo",
    "budistas": "budista",
    "budrieres": "budrier",
    "buen": "bueno",
    "buena": "bueno",
    "buenandanzas": "buenandanza",
    "buenas": "bueno",
    "buenaventuras": "buenaventura",
    "buenos": "bueno",
    "bueyes": "buey",
    "bufa": "bufo",
    "bufaba": "bufar",
    "bufabais": "bufar",
    "bufaban": "bufar",
    "bufabas": "bufar",
    "bufad": "bufar",
    "bufada": "bufar",
    "bufadas": "bufar",
    "bufaderos": "bufadero",
    "bufado": "bufar",
    "bufadores": "bufador",
    "bufados": "bufar",
    "bufamos": "bufar",
    "bufan": "bufar",
    "bufandas": "bufanda",
    "bufando": "bufar",
    "bufara": "bufar",
    "bufarais": "bufar",
    "bufaran": "bufar",
    "bufaras": "bufar",
    "bufare": "bufar",
    "bufareis": "bufar",
    "bufaremos": "bufar",
    "bufaren": "bufar",
    "bufares": "bufar",
    "bufaron": "bufar",
    "bufará": "bufar",
    "bufarán": "bufar",
    "bufarás": "bufar",
    "bufaré": "bufar",
    "bufaréis": "bufar",
    "bufaría": "bufar",
    "bufaríais": "bufar",
    "bufaríamos": "bufar",
    "bufarían": "bufar",
    "bufarías": "bufar",
    "bufas": "bufo",
    "bufase": "bufar",
    "bufaseis": "bufar",
    "bufasen": "bufar",
    "bufases": "bufar",
    "bufaste": "bufar",
    "bufasteis": "bufar",
    "bufe": "bufar",
    "bufemos": "bufar",
    "bufen": "bufar",
    "bufes": "bufar",
    "bufetes": "bufete",
    "buffers": "buffer",
    "buffets": "buffet",
    "bufidos": "bufido",
    "bufo": "bufar",
    "bufona": "bufón",
    "bufonadas": "bufonada",
    "bufonas": "bufón",
    "bufonea": "bufonear",
    "bufoneaba": "bufonear",
    "bufoneabais": "bufonear",
    "bufoneaban": "bufonear",
    "bufoneabas": "bufonear",
    "bufonead": "bufonear",
    "bufoneada": "bufonear",
    "bufoneadas": "bufonear",
    "bufoneado": "bufonear",
    "bufoneados": "bufonear",
    "bufoneamos": "bufonear",
    "bufonean": "bufonear",
    "bufoneando": "bufonear",
    "bufoneara": "bufonear",
    "bufonearais": "bufonear",
    "bufonearan": "bufonear",
    "bufonearas": "bufonear",
    "bufoneare": "bufonear",
    "bufoneareis": "bufonear",
    "bufonearemos": "bufonear",
    "bufonearen": "bufonear",
    "bufoneares": "bufonear",
    "bufonearon": "bufonear",
    "bufoneará": "bufonear",
    "bufonearán": "bufonear",
    "bufonearás": "bufonear",
    "bufonearé": "bufonear",
    "bufonearéis": "bufonear",
    "bufonearía": "bufonear",
    "bufonearíais": "bufonear",
    "bufonearíamos": "bufonear",
    "bufonearían": "bufonear",
    "bufonearías": "bufonear",
    "bufoneas": "bufonear",
    "bufonease": "bufonear",
    "bufoneaseis": "bufonear",
    "bufoneasen": "bufonear",
    "bufoneases": "bufonear",
    "bufoneaste": "bufonear",
    "bufoneasteis": "bufonear",
    "bufonee": "bufonear",
    "bufoneemos": "bufonear",
    "bufoneen": "bufonear",
    "bufonees": "bufonear",
    "bufoneo": "bufonear",
    "bufonerías": "bufonería",
    "bufones": "bufón",
    "bufonesca": "bufonesco",
    "bufonescas": "bufonesco",
    "bufonescos": "bufonesco",
    "bufoneábamos": "bufonear",
    "bufoneáis": "bufonear",
    "bufoneáramos": "bufonear",
    "bufoneáremos": "bufonear",
    "bufoneásemos": "bufonear",
    "bufoneé": "bufonear",
    "bufoneéis": "bufonear",
    "bufoneó": "bufonear",
    "bufos": "bufo",
    "buftalmías": "buftalmía",
    "bufábamos": "bufar",
    "bufáis": "bufar",
    "bufáramos": "bufar",
    "bufáremos": "bufar",
    "bufásemos": "bufar",
    "bufé": "bufar",
    "buféis": "bufar",
    "bufés": "bufé",
    "bufó": "bufar",
    "bugallas": "bugalla",
    "buganvilias": "buganvilia",
    "buganvillas": "buganvilla",
    "buge": "bugir",
    "bugen": "bugir",
    "buges": "bugir",
    "bugid": "bugir",
    "bugida": "bugir",
    "bugidas": "bugir",
    "bugido": "bugir",
    "bugidos": "bugir",
    "bugiendo": "bugir",
    "bugiera": "bugir",
    "bugierais": "bugir",
    "bugieran": "bugir",
    "bugieras": "bugir",
    "bugiere": "bugir",
    "bugiereis": "bugir",
    "bugieren": "bugir",
    "bugieres": "bugir",
    "bugieron": "bugir",
    "bugiese": "bugir",
    "bugieseis": "bugir",
    "bugiesen": "bugir",
    "bugieses": "bugir",
    "bugimos": "bugir",
    "bugiremos": "bugir",
    "bugirá": "bugir",
    "bugirán": "bugir",
    "bugirás": "bugir",
    "bugiré": "bugir",
    "bugiréis": "bugir",
    "bugiría": "bugir",
    "bugiríais": "bugir",
    "bugiríamos": "bugir",
    "bugirían": "bugir",
    "bugirías": "bugir",
    "bugiste": "bugir",
    "bugisteis": "bugir",
    "bugiéramos": "bugir",
    "bugiéremos": "bugir",
    "bugiésemos": "bugir",
    "bugió": "bugir",
    "bugles": "bugle",
    "bugí": "bugir",
    "bugía": "bugir",
    "bugíais": "bugir",
    "bugíamos": "bugir",
    "bugían": "bugir",
    "bugías": "bugir",
    "bugís": "bugir",
    "buhardas": "buharda",
    "buhardillas": "buhardilla",
    "buhederas": "buhedera",
    "buhonera": "buhonero",
    "buhoneras": "buhonero",
    "buhoneros": "buhonero",
    "buhonerías": "buhonería",
    "buitrea": "buitrear",
    "buitreaba": "buitrear",
    "buitreabais": "buitrear",
    "buitreaban": "buitrear",
    "buitreabas": "buitrear",
    "buitread": "buitrear",
    "buitreada": "buitrear",
    "buitreadas": "buitrear",
    "buitreado": "buitrear",
    "buitreados": "buitrear",
    "buitreamos": "buitrear",
    "buitrean": "buitrear",
    "buitreando": "buitrear",
    "buitreara": "buitrear",
    "buitrearais": "buitrear",
    "buitrearan": "buitrear",
    "buitrearas": "buitrear",
    "buitreare": "buitrear",
    "buitreareis": "buitrear",
    "buitrearemos": "buitrear",
    "buitrearen": "buitrear",
    "buitreares": "buitrear",
    "buitrearon": "buitrear",
    "buitreará": "buitrear",
    "buitrearán": "buitrear",
    "buitrearás": "buitrear",
    "buitrearé": "buitrear",
    "buitrearéis": "buitrear",
    "buitrearía": "buitrear",
    "buitrearíais": "buitrear",
    "buitrearíamos": "buitrear",
    "buitrearían": "buitrear",
    "buitrearías": "buitrear",
    "buitreas": "buitrear",
    "buitrease": "buitrear",
    "buitreaseis": "buitrear",
    "buitreasen": "buitrear",
    "buitreases": "buitrear",
    "buitreaste": "buitrear",
    "buitreasteis": "buitrear",
    "buitree": "buitrear",
    "buitreemos": "buitrear",
    "buitreen": "buitrear",
    "buitrees": "buitrear",
    "buitreo": "buitrear",
    "buitrera": "buitrero",
    "buitreras": "buitrero",
    "buitreros": "buitrero",
    "buitres": "buitre",
    "buitreábamos": "buitrear",
    "buitreáis": "buitrear",
    "buitreáramos": "buitrear",
    "buitreáremos": "buitrear",
    "buitreásemos": "buitrear",
    "buitreé": "buitrear",
    "buitreéis": "buitrear",
    "buitreó": "buitrear",
    "buitrones": "buitrón",
    "buja": "bugir",
    "bujamos": "bugir",
    "bujan": "bugir",
    "bujardados": "bujardado",
    "bujardas": "bujarda",
    "bujarrones": "bujarrón",
    "bujas": "bugir",
    "bujedales": "bujedal",
    "bujedas": "bujeda",
    "bujedos": "bujedo",
    "bujerías": "bujería",
    "bujes": "buje",
    "bujetas": "bujeta",
    "bujieres": "bujier",
    "bujierías": "bujiería",
    "bujo": "bugir",
    "bujáis": "bugir",
    "bujías": "bujía",
    "bularios": "bulario",
    "bulas": "bula",
    "bulbares": "bulbar",
    "bulbiformes": "bulbiforme",
    "bulbillos": "bulbillo",
    "bulbocapninas": "bulbocapnina",
    "bulbocavernosa": "bulbocavernoso",
    "bulbocavernosas": "bulbocavernoso",
    "bulbocavernosos": "bulbocavernoso",
    "bulbocerebelosa": "bulbocerebeloso",
    "bulbocerebelosas": "bulbocerebeloso",
    "bulbocerebelosos": "bulbocerebeloso",
    "bulbos": "bulbo",
    "bulbosa": "bulboso",
    "bulbosas": "bulboso",
    "bulbosos": "bulboso",
    "bulbouretrales": "bulbouretral",
    "buldogs": "buldog",
    "bulerías": "bulería",
    "bulevares": "bulevar",
    "bulimias": "bulimia",
    "bulla": "bullir",
    "bullabesas": "bullabesa",
    "bullajes": "bullaje",
    "bullamos": "bullir",
    "bullan": "bullir",
    "bullangas": "bullanga",
    "bullangera": "bullangero",
    "bullangeras": "bullangero",
    "bullangeros": "bullangero",
    "bullanguera": "bullanguero",
    "bullangueras": "bullanguero",
    "bullangueros": "bullanguero",
    "bullas": "bullir",
    "bulldogs": "bulldog",
    "bulldozeres": "bulldozer",
    "bulldozers": "bulldozer",
    "bulle": "bullir",
    "bullebulles": "bullebulle",
    "bullen": "bullir",
    "bullendo": "bullir",
    "bullera": "bullir",
    "bullerais": "bullir",
    "bulleran": "bullir",
    "bulleras": "bullir",
    "bullere": "bullir",
    "bullereis": "bullir",
    "bulleren": "bullir",
    "bulleres": "bullir",
    "bulleron": "bullir",
    "bulles": "bullir",
    "bullese": "bullir",
    "bulleseis": "bullir",
    "bullesen": "bullir",
    "bulleses": "bullir",
    "bullicios": "bullicio",
    "bulliciosa": "bullicioso",
    "bulliciosas": "bullicioso",
    "bulliciosos": "bullicioso",
    "bullid": "bullir",
    "bullida": "bullir",
    "bullidas": "bullir",
    "bullido": "bullir",
    "bullidora": "bullidor",
    "bullidoras": "bullidor",
    "bullidores": "bullidor",
    "bullidos": "bullir",
    "bullimos": "bullir",
    "bulliones": "bullion",
    "bullionismos": "bullionismo",
    "bullions": "bullion",
    "bulliremos": "bullir",
    "bullirá": "bullir",
    "bullirán": "bullir",
    "bullirás": "bullir",
    "bulliré": "bullir",
    "bulliréis": "bullir",
    "bulliría": "bullir",
    "bulliríais": "bullir",
    "bulliríamos": "bullir",
    "bullirían": "bullir",
    "bullirías": "bullir",
    "bulliste": "bullir",
    "bullisteis": "bullir",
    "bullo": "bullir",
    "bullones": "bullón",
    "bullys": "bully",
    "bulláis": "bullir",
    "bulléramos": "bullir",
    "bulléremos": "bullir",
    "bullésemos": "bullir",
    "bullí": "bullir",
    "bullía": "bullir",
    "bullíais": "bullir",
    "bullíamos": "bullir",
    "bullían": "bullir",
    "bullías": "bullir",
    "bullís": "bullir",
    "bulló": "bullir",
    "bulones": "bulón",
    "bulos": "bulo",
    "bultos": "bulto",
    "bululúes": "bululú",
    "bululús": "bululú",
    "bulés": "bulé",
    "bulímica": "bulímico",
    "bulímicas": "bulímico",
    "bulímicos": "bulímico",
    "bumeranes": "bumerán",
    "bumerangs": "bumerang",
    "bunas": "buna",
    "bundeslands": "bundesland",
    "bundesrats": "bundesrat",
    "bundestags": "bundestag",
    "bungalowes": "bungalow",
    "bungalows": "bungalow",
    "bunios": "bunio",
    "bunkers": "búnker",
    "bunkerses": "bunker",
    "bunodonta": "bunodonto",
    "bunodontas": "bunodonto",
    "bunodontos": "bunodonto",
    "bunolofodonta": "bunolofodonto",
    "bunolofodontas": "bunolofodonto",
    "bunolofodontos": "bunolofodonto",
    "bunoselenodonta": "bunoselenodonto",
    "bunoselenodontas": "bunoselenodonto",
    "bunoselenodontos": "bunoselenodonto",
    "bunsenitas": "bunsenita",
    "buques": "buque",
    "buratos": "burato",
    "burbujas": "burbuja",
    "burbujea": "burbujear",
    "burbujeaba": "burbujear",
    "burbujeabais": "burbujear",
    "burbujeaban": "burbujear",
    "burbujeabas": "burbujear",
    "burbujead": "burbujear",
    "burbujeada": "burbujear",
    "burbujeadas": "burbujear",
    "burbujeado": "burbujear",
    "burbujeados": "burbujear",
    "burbujeamos": "burbujear",
    "burbujean": "burbujear",
    "burbujeando": "burbujear",
    "burbujeantes": "burbujeante",
    "burbujeara": "burbujear",
    "burbujearais": "burbujear",
    "burbujearan": "burbujear",
    "burbujearas": "burbujear",
    "burbujeare": "burbujear",
    "burbujeareis": "burbujear",
    "burbujearemos": "burbujear",
    "burbujearen": "burbujear",
    "burbujeares": "burbujear",
    "burbujearon": "burbujear",
    "burbujeará": "burbujear",
    "burbujearán": "burbujear",
    "burbujearás": "burbujear",
    "burbujearé": "burbujear",
    "burbujearéis": "burbujear",
    "burbujearía": "burbujear",
    "burbujearíais": "burbujear",
    "burbujearíamos": "burbujear",
    "burbujearían": "burbujear",
    "burbujearías": "burbujear",
    "burbujeas": "burbujear",
    "burbujease": "burbujear",
    "burbujeaseis": "burbujear",
    "burbujeasen": "burbujear",
    "burbujeases": "burbujear",
    "burbujeaste": "burbujear",
    "burbujeasteis": "burbujear",
    "burbujee": "burbujear",
    "burbujeemos": "burbujear",
    "burbujeen": "burbujear",
    "burbujees": "burbujear",
    "burbujeo": "burbujear",
    "burbujeos": "burbujeo",
    "burbujeábamos": "burbujear",
    "burbujeáis": "burbujear",
    "burbujeáramos": "burbujear",
    "burbujeáremos": "burbujear",
    "burbujeásemos": "burbujear",
    "burbujeé": "burbujear",
    "burbujeéis": "burbujear",
    "burbujeó": "burbujear",
    "burches": "burche",
    "burda": "burdo",
    "burdas": "burdo",
    "burdeles": "burdel",
    "burdos": "burdo",
    "burdéganos": "burdégano",
    "burelada": "burelado",
    "bureladas": "burelado",
    "burelados": "burelado",
    "bureles": "burel",
    "bureletes": "burelete",
    "bureos": "bureo",
    "buretas": "bureta",
    "burgados": "burgado",
    "burgalesa": "burgalés",
    "burgalesas": "burgalés",
    "burgaleses": "burgalés",
    "burgomaestres": "burgomaestre",
    "burgos": "burgo",
    "burgraves": "burgrave",
    "burgraviatos": "burgraviato",
    "burguesa": "burgués",
    "burguesas": "burgués",
    "burgueses": "burgués",
    "burguesías": "burguesía",
    "burgundios": "burgundio",
    "burieles": "buriel",
    "burila": "burilar",
    "burilaba": "burilar",
    "burilabais": "burilar",
    "burilaban": "burilar",
    "burilabas": "burilar",
    "burilad": "burilar",
    "burilada": "burilar",
    "buriladas": "burilar",
    "burilado": "burilar",
    "burilados": "burilar",
    "buriladuras": "buriladura",
    "burilamos": "burilar",
    "burilan": "burilar",
    "burilando": "burilar",
    "burilara": "burilar",
    "burilarais": "burilar",
    "burilaran": "burilar",
    "burilaras": "burilar",
    "burilare": "burilar",
    "burilareis": "burilar",
    "burilaremos": "burilar",
    "burilaren": "burilar",
    "burilares": "burilar",
    "burilaron": "burilar",
    "burilará": "burilar",
    "burilarán": "burilar",
    "burilarás": "burilar",
    "burilaré": "burilar",
    "burilaréis": "burilar",
    "burilaría": "burilar",
    "burilaríais": "burilar",
    "burilaríamos": "burilar",
    "burilarían": "burilar",
    "burilarías": "burilar",
    "burilas": "burilar",
    "burilase": "burilar",
    "burilaseis": "burilar",
    "burilasen": "burilar",
    "burilases": "burilar",
    "burilaste": "burilar",
    "burilasteis": "burilar",
    "burile": "burilar",
    "burilemos": "burilar",
    "burilen": "burilar",
    "buriles": "burilar",
    "burilo": "burilar",
    "burilábamos": "burilar",
    "buriláis": "burilar",
    "buriláramos": "burilar",
    "buriláremos": "burilar",
    "burilásemos": "burilar",
    "burilé": "burilar",
    "buriléis": "burilar",
    "buriló": "burilar",
    "burla": "burlar",
    "burlaba": "burlar",
    "burlabais": "burlar",
    "burlaban": "burlar",
    "burlabas": "burlar",
    "burlad": "burlar",
    "burlada": "burlar",
    "burladas": "burlar",
    "burladeros": "burladero",
    "burlado": "burlar",
    "burladora": "burlador",
    "burladoras": "burladora",
    "burladores": "burlador",
    "burlados": "burlar",
    "burlamos": "burlar",
    "burlan": "burlar",
    "burlando": "burlar",
    "burlara": "burlar",
    "burlarais": "burlar",
    "burlaran": "burlar",
    "burlaras": "burlar",
    "burlare": "burlar",
    "burlareis": "burlar",
    "burlaremos": "burlar",
    "burlaren": "burlar",
    "burlares": "burlar",
    "burlaron": "burlar",
    "burlará": "burlar",
    "burlarán": "burlar",
    "burlarás": "burlar",
    "burlaré": "burlar",
    "burlaréis": "burlar",
    "burlaría": "burlar",
    "burlaríais": "burlar",
    "burlaríamos": "burlar",
    "burlarían": "burlar",
    "burlarías": "burlar",
    "burlas": "burlar",
    "burlase": "burlar",
    "burlaseis": "burlar",
    "burlasen": "burlar",
    "burlases": "burlar",
    "burlaste": "burlar",
    "burlasteis": "burlar",
    "burle": "burlar",
    "burlemos": "burlar",
    "burlen": "burlar",
    "burlerías": "burlería",
    "burles": "burlar",
    "burlesca": "burlesco",
    "burlescas": "burlesco",
    "burlescos": "burlesco",
    "burletes": "burlete",
    "burlo": "burlar",
    "burlona": "burlón",
    "burlonas": "burlón",
    "burlones": "burlón",
    "burlotes": "burlote",
    "burlábamos": "burlar",
    "burláis": "burlar",
    "burláramos": "burlar",
    "burláremos": "burlar",
    "burlásemos": "burlar",
    "burlé": "burlar",
    "burléis": "burlar",
    "burló": "burlar",
    "burmitas": "burmita",
    "burocracias": "burocracia",
    "burocratismos": "burocratismo",
    "burocratizaciones": "burocratización",
    "burocratizada": "burocratizado",
    "burocratizadas": "burocratizado",
    "burocratizados": "burocratizado",
    "burocrática": "burocrático",
    "burocráticas": "burocrático",
    "burocráticos": "burocrático",
    "burra": "burro",
    "burradas": "burrada",
    "burrajos": "burrajo",
    "burras": "burro",
    "burreños": "burreño",
    "burros": "burro",
    "bursatilidades": "bursatilidad",
    "bursátiles": "bursátil",
    "burujos": "burujo",
    "burócratas": "burócrata",
    "burós": "buró",
    "buróticas": "burótica",
    "bus-bóbs": "bus-bob",
    "busardas": "busarda",
    "busca": "buscar",
    "buscaba": "buscar",
    "buscabais": "buscar",
    "buscaban": "buscar",
    "buscabas": "buscar",
    "buscad": "buscar",
    "buscada": "buscar",
    "buscadas": "buscar",
    "buscado": "buscar",
    "buscadora": "buscador",
    "buscadoras": "buscador",
    "buscadores": "buscador",
    "buscados": "buscar",
    "buscamos": "buscar",
    "buscan": "buscar",
    "buscando": "buscar",
    "buscapiés": "buscapié",
    "buscara": "buscar",
    "buscarais": "buscar",
    "buscaran": "buscar",
    "buscaras": "buscar",
    "buscare": "buscar",
    "buscareis": "buscar",
    "buscaremos": "buscar",
    "buscaren": "buscar",
    "buscares": "buscar",
    "buscarlas": "buscarla",
    "buscaron": "buscar",
    "buscará": "buscar",
    "buscarán": "buscar",
    "buscarás": "buscar",
    "buscaré": "buscar",
    "buscaréis": "buscar",
    "buscaría": "buscar",
    "buscaríais": "buscar",
    "buscaríamos": "buscar",
    "buscarían": "buscar",
    "buscarías": "buscar",
    "buscas": "buscar",
    "buscase": "buscar",
    "buscaseis": "buscar",
    "buscasen": "buscar",
    "buscases": "buscar",
    "buscaste": "buscar",
    "buscasteis": "buscar",
    "busco": "buscar",
    "buscona": "buscón",
    "busconas": "buscón",
    "buscones": "buscón",
    "buscos": "busco",
    "buscábamos": "buscar",
    "buscáis": "buscar",
    "buscáramos": "buscar",
    "buscáremos": "buscar",
    "buscásemos": "buscar",
    "buscó": "buscar",
    "buses": "bus",
    "bushels": "bushel",
    "busque": "buscar",
    "busquemos": "buscar",
    "busquen": "buscar",
    "busques": "buscar",
    "busqué": "buscar",
    "busquéis": "buscar",
    "bustos": "busto",
    "bustrofedones": "bustrofedon",
    "bustrófedones": "bustrófedon",
    "busólfanes": "busolfan",
    "butacas": "butaca",
    "butacones": "butacón",
    "butanera": "butanero",
    "butaneras": "butanero",
    "butaneros": "butanero",
    "butanos": "butano",
    "buterolas": "buterola",
    "buthanesa": "buthanés",
    "buthanesas": "buthanés",
    "buthaneses": "buthanés",
    "butifarras": "butifarra",
    "butifarrera": "butifarrero",
    "butifarreras": "butifarrero",
    "butifarreros": "butifarrero",
    "butilfosfórica": "butilfosfórico",
    "butilfosfóricas": "butilfosfórico",
    "butilfosfóricos": "butilfosfórico",
    "butilos": "butilo",
    "butinodioles": "butinodiol",
    "butinoles": "butinol",
    "butirales": "butiral",
    "butiratos": "butirato",
    "butirolactonas": "butirolactona",
    "butironitrilos": "butironitrilo",
    "butiros": "butiro",
    "butirómetros": "butirómetro",
    "butomáceas": "butomácea",
    "butonatos": "butonato",
    "butopironoxilos": "butopironoxilo",
    "butrinos": "butrino",
    "butronera": "butronero",
    "butroneras": "butronero",
    "butroneros": "butronero",
    "butrones": "butrón",
    "butírica": "butírico",
    "butíricas": "butírico",
    "butíricos": "butírico",
    "buxáceas": "buxácea",
    "buyos": "buyo",
    "buza": "buzar",
    "buzaba": "buzar",
    "buzabais": "buzar",
    "buzaban": "buzar",
    "buzabas": "buzar",
    "buzad": "buzar",
    "buzada": "buzar",
    "buzadas": "buzar",
    "buzado": "buzar",
    "buzados": "buzar",
    "buzamientos": "buzamiento",
    "buzamos": "buzar",
    "buzan": "buzar",
    "buzando": "buzar",
    "buzaques": "buzaque",
    "buzara": "buzar",
    "buzarais": "buzar",
    "buzaran": "buzar",
    "buzaras": "buzar",
    "buzardas": "buzarda",
    "buzare": "buzar",
    "buzareis": "buzar",
    "buzaremos": "buzar",
    "buzaren": "buzar",
    "buzares": "buzar",
    "buzaron": "buzar",
    "buzará": "buzar",
    "buzarán": "buzar",
    "buzarás": "buzar",
    "buzaré": "buzar",
    "buzaréis": "buzar",
    "buzaría": "buzar",
    "buzaríais": "buzar",
    "buzaríamos": "buzar",
    "buzarían": "buzar",
    "buzarías": "buzar",
    "buzas": "buzar",
    "buzase": "buzar",
    "buzaseis": "buzar",
    "buzasen": "buzar",
    "buzases": "buzar",
    "buzaste": "buzar",
    "buzasteis": "buzar",
    "buzo": "buzar",
    "buzoneos": "buzoneo",
    "buzones": "buzón",
    "buzos": "buzo",
    "buzábamos": "buzar",
    "buzáis": "buzar",
    "buzáramos": "buzar",
    "buzáremos": "buzar",
    "buzásemos": "buzar",
    "buzó": "buzar",
    "buñolera": "buñolero",
    "buñoleras": "buñolero",
    "buñoleros": "buñolero",
    "buñolerías": "buñolería",
    "buñuelos": "buñuelo",
    "bylinas": "bylina",
    "bypasses": "bypass",
    "bytes": "byte",
    "bytownitas": "bytownita",
    "báckspins": "backspin",
    "báculos": "báculo",
    "bálagos": "bálago",
    "bálanos": "bálano",
    "bálsamos": "bálsamo",
    "báltica": "báltico",
    "bálticas": "báltico",
    "bálticos": "báltico",
    "báquica": "báquico",
    "báquicas": "báquico",
    "báquicos": "báquico",
    "báratros": "báratro",
    "bárbara": "bárbaro",
    "bárbaras": "bárbaro",
    "bárbaros": "bárbaro",
    "bárica": "bárico",
    "báricas": "bárico",
    "báricos": "bárico",
    "bártulos": "bártulo",
    "básculas": "báscula",
    "básica": "básico",
    "básicas": "básico",
    "básicos": "básico",
    "básquetes": "básquet",
    "básquets": "básquet",
    "bátava": "bátavo",
    "bátavas": "bátavo",
    "bátavos": "bátavo",
    "bávara": "bávaro",
    "bávaras": "bávaro",
    "bávaros": "bávaro",
    "béisboles": "béisbol",
    "bélica": "bélico",
    "bélicas": "bélico",
    "bélicos": "bélico",
    "bética": "bético",
    "béticas": "bético",
    "béticos": "bético",
    "bíblica": "bíblico",
    "bíblicas": "bíblico",
    "bíblicos": "bíblico",
    "bífera": "bífero",
    "bíferas": "bífero",
    "bíferos": "bífero",
    "bífida": "bífido",
    "bífidas": "bífido",
    "bífidos": "bífido",
    "bígama": "bígamo",
    "bígamas": "bígamo",
    "bígamos": "bígamo",
    "bígaros": "bígaro",
    "bímana": "bímano",
    "bímanas": "bímano",
    "bímanos": "bímano",
    "bínuba": "bínubo",
    "bínubas": "bínubo",
    "bínubos": "bínubo",
    "bípara": "bíparo",
    "bíparas": "bíparo",
    "bíparos": "bíparo",
    "bípeda": "bípedo",
    "bípedas": "bípedo",
    "bípedos": "bípedo",
    "bípodes": "bípode",
    "bóers": "bóer",
    "bólidos": "bólido",
    "bóraxes": "bórax",
    "bórica": "bórico",
    "bóricas": "bórico",
    "bóricos": "bórico",
    "bóvedas": "bóveda",
    "bóvida": "bóvido",
    "bóvidas": "bóvido",
    "bóvidos": "bóvido",
    "bóxeres": "bóxer",
    "búas": "búa",
    "búcaros": "búcaro",
    "búes": "bu",
    "búfalos": "búfalo",
    "búfers": "búfer",
    "búhos": "búho",
    "búlgara": "búlgaro",
    "búlgaras": "búlgaro",
    "búlgaros": "búlgaro",
    "búnkeres": "búnker",
    "búnkerses": "búnker",
    "búsquedas": "búsqueda",
    "búteos": "búteo",
    "cabales": "cabal",
    "cabalga": "cabalgar",
    "cabalgaba": "cabalgar",
    "cabalgabais": "cabalgar",
    "cabalgaban": "cabalgar",
    "cabalgabas": "cabalgar",
    "cabalgad": "cabalgar",
    "cabalgada": "cabalgar",
    "cabalgadas": "cabalgar",
    "cabalgado": "cabalgar",
    "cabalgados": "cabalgar",
    "cabalgaduras": "cabalgadura",
    "cabalgamientos": "cabalgamiento",
    "cabalgamos": "cabalgar",
    "cabalgan": "cabalgar",
    "cabalgando": "cabalgar",
    "cabalgara": "cabalgar",
    "cabalgarais": "cabalgar",
    "cabalgaran": "cabalgar",
    "cabalgaras": "cabalgar",
    "cabalgare": "cabalgar",
    "cabalgareis": "cabalgar",
    "cabalgaremos": "cabalgar",
    "cabalgaren": "cabalgar",
    "cabalgares": "cabalgar",
    "cabalgaron": "cabalgar",
    "cabalgará": "cabalgar",
    "cabalgarán": "cabalgar",
    "cabalgarás": "cabalgar",
    "cabalgaré": "cabalgar",
    "cabalgaréis": "cabalgar",
    "cabalgaría": "cabalgar",
    "cabalgaríais": "cabalgar",
    "cabalgaríamos": "cabalgar",
    "cabalgarían": "cabalgar",
    "cabalgarías": "cabalgar",
    "cabalgas": "cabalgar",
    "cabalgase": "cabalgar",
    "cabalgaseis": "cabalgar",
    "cabalgasen": "cabalgar",
    "cabalgases": "cabalgar",
    "cabalgaste": "cabalgar",
    "cabalgasteis": "cabalgar",
    "cabalgatas": "cabalgata",
    "cabalgazones": "cabalgazón",
    "cabalgo": "cabalgar",
    "cabalgue": "cabalgar",
    "cabalguemos": "cabalgar",
    "cabalguen": "cabalgar",
    "cabalgues": "cabalgar",
    "cabalgué": "cabalgar",
    "cabalguéis": "cabalgar",
    "cabalgábamos": "cabalgar",
    "cabalgáis": "cabalgar",
    "cabalgáramos": "cabalgar",
    "cabalgáremos": "cabalgar",
    "cabalgásemos": "cabalgar",
    "cabalgó": "cabalgar",
    "cabalistas": "cabalista",
    "caballadas": "caballada",
    "caballajes": "caballaje",
    "caballares": "caballar",
    "caballas": "caballa",
    "caballea": "caballear",
    "caballeaba": "caballear",
    "caballeabais": "caballear",
    "caballeaban": "caballear",
    "caballeabas": "caballear",
    "caballead": "caballear",
    "caballeada": "caballear",
    "caballeadas": "caballear",
    "caballeado": "caballear",
    "caballeados": "caballear",
    "caballeamos": "caballear",
    "caballean": "caballear",
    "caballeando": "caballear",
    "caballeara": "caballear",
    "caballearais": "caballear",
    "caballearan": "caballear",
    "caballearas": "caballear",
    "caballeare": "caballear",
    "caballeareis": "caballear",
    "caballearemos": "caballear",
    "caballearen": "caballear",
    "caballeares": "caballear",
    "caballearon": "caballear",
    "caballeará": "caballear",
    "caballearán": "caballear",
    "caballearás": "caballear",
    "caballearé": "caballear",
    "caballearéis": "caballear",
    "caballearía": "caballear",
    "caballearíais": "caballear",
    "caballearíamos": "caballear",
    "caballearían": "caballear",
    "caballearías": "caballear",
    "caballeas": "caballear",
    "caballease": "caballear",
    "caballeaseis": "caballear",
    "caballeasen": "caballear",
    "caballeases": "caballear",
    "caballeaste": "caballear",
    "caballeasteis": "caballear",
    "caballee": "caballear",
    "caballeemos": "caballear",
    "caballeen": "caballear",
    "caballees": "caballear",
    "caballeo": "caballear",
    "caballera": "caballero",
    "caballeras": "caballero",
    "caballeratos": "caballerato",
    "caballerea": "caballerear",
    "caballereaba": "caballerear",
    "caballereabais": "caballerear",
    "caballereaban": "caballerear",
    "caballereabas": "caballerear",
    "caballeread": "caballerear",
    "caballereada": "caballerear",
    "caballereadas": "caballerear",
    "caballereado": "caballerear",
    "caballereados": "caballerear",
    "caballereamos": "caballerear",
    "caballerean": "caballerear",
    "caballereando": "caballerear",
    "caballereara": "caballerear",
    "caballerearais": "caballerear",
    "caballerearan": "caballerear",
    "caballerearas": "caballerear",
    "caballereare": "caballerear",
    "caballereareis": "caballerear",
    "caballerearemos": "caballerear",
    "caballerearen": "caballerear",
    "caballereares": "caballerear",
    "caballerearon": "caballerear",
    "caballereará": "caballerear",
    "caballerearán": "caballerear",
    "caballerearás": "caballerear",
    "caballerearé": "caballerear",
    "caballerearéis": "caballerear",
    "caballerearía": "caballerear",
    "caballerearíais": "caballerear",
    "caballerearíamos": "caballerear",
    "caballerearían": "caballerear",
    "caballerearías": "caballerear",
    "caballereas": "caballerear",
    "caballerease": "caballerear",
    "caballereaseis": "caballerear",
    "caballereasen": "caballerear",
    "caballereases": "caballerear",
    "caballereaste": "caballerear",
    "caballereasteis": "caballerear",
    "caballeree": "caballerear",
    "caballereemos": "caballerear",
    "caballereen": "caballerear",
    "caballerees": "caballerear",
    "caballereo": "caballerear",
    "caballeresca": "caballeresco",
    "caballerescas": "caballeresco",
    "caballerescos": "caballeresco",
    "caballeretes": "caballerete",
    "caballereábamos": "caballerear",
    "caballereáis": "caballerear",
    "caballereáramos": "caballerear",
    "caballereáremos": "caballerear",
    "caballereásemos": "caballerear",
    "caballereé": "caballerear",
    "caballereéis": "caballerear",
    "caballereó": "caballerear",
    "caballerizas": "caballeriza",
    "caballerizos": "caballerizo",
    "caballeros": "caballero",
    "caballerosa": "caballeroso",
    "caballerosas": "caballeroso",
    "caballerosidades": "caballerosidad",
    "caballerosos": "caballeroso",
    "caballerías": "caballería",
    "caballetes": "caballete",
    "caballeábamos": "caballear",
    "caballeáis": "caballear",
    "caballeáramos": "caballear",
    "caballeáremos": "caballear",
    "caballeásemos": "caballear",
    "caballeé": "caballear",
    "caballeéis": "caballear",
    "caballeó": "caballear",
    "caballistas": "caballista",
    "caballitos": "caballito",
    "caballones": "caballón",
    "caballos": "caballo",
    "caballuna": "caballuno",
    "caballunas": "caballuno",
    "caballunos": "caballuno",
    "cabalística": "cabalístico",
    "cabalísticas": "cabalístico",
    "cabalísticos": "cabalístico",
    "cabareteras": "cabaretera",
    "cabarets": "cabaret",
    "cabases": "cabás",
    "cabañales": "cabañal",
    "cabañas": "cabaña",
    "cabañera": "cabañero",
    "cabañeras": "cabañero",
    "cabañeros": "cabañero",
    "cabañerías": "cabañería",
    "cabañiles": "cabañil",
    "cabañuelas": "cabañuela",
    "cabe": "caber",
    "cabecea": "cabecear",
    "cabeceaba": "cabecear",
    "cabeceabais": "cabecear",
    "cabeceaban": "cabecear",
    "cabeceabas": "cabecear",
    "cabecead": "cabecear",
    "cabeceada": "cabecear",
    "cabeceadas": "cabecear",
    "cabeceado": "cabecear",
    "cabeceados": "cabecear",
    "cabeceamientos": "cabeceamiento",
    "cabeceamos": "cabecear",
    "cabecean": "cabecear",
    "cabeceando": "cabecear",
    "cabeceara": "cabecear",
    "cabecearais": "cabecear",
    "cabecearan": "cabecear",
    "cabecearas": "cabecear",
    "cabeceare": "cabecear",
    "cabeceareis": "cabecear",
    "cabecearemos": "cabecear",
    "cabecearen": "cabecear",
    "cabeceares": "cabecear",
    "cabecearon": "cabecear",
    "cabeceará": "cabecear",
    "cabecearán": "cabecear",
    "cabecearás": "cabecear",
    "cabecearé": "cabecear",
    "cabecearéis": "cabecear",
    "cabecearía": "cabecear",
    "cabecearíais": "cabecear",
    "cabecearíamos": "cabecear",
    "cabecearían": "cabecear",
    "cabecearías": "cabecear",
    "cabeceas": "cabecear",
    "cabecease": "cabecear",
    "cabeceaseis": "cabecear",
    "cabeceasen": "cabecear",
    "cabeceases": "cabecear",
    "cabeceaste": "cabecear",
    "cabeceasteis": "cabecear",
    "cabecee": "cabecear",
    "cabeceemos": "cabecear",
    "cabeceen": "cabecear",
    "cabecees": "cabecear",
    "cabeceo": "cabecear",
    "cabeceos": "cabeceo",
    "cabeceras": "cabecera",
    "cabeceros": "cabecero",
    "cabeceábamos": "cabecear",
    "cabeceáis": "cabecear",
    "cabeceáramos": "cabecear",
    "cabeceáremos": "cabecear",
    "cabeceásemos": "cabecear",
    "cabeceé": "cabecear",
    "cabeceéis": "cabecear",
    "cabeceó": "cabecear",
    "cabecidura": "cabeciduro",
    "cabeciduras": "cabeciduro",
    "cabeciduros": "cabeciduro",
    "cabecillas": "cabecilla",
    "cabed": "caber",
    "cabella": "cabellar",
    "cabellaba": "cabellar",
    "cabellabais": "cabellar",
    "cabellaban": "cabellar",
    "cabellabas": "cabellar",
    "cabellad": "cabellar",
    "cabellada": "cabellar",
    "cabelladas": "cabellar",
    "cabellado": "cabellar",
    "cabellados": "cabellar",
    "cabellamos": "cabellar",
    "cabellan": "cabellar",
    "cabellando": "cabellar",
    "cabellara": "cabellar",
    "cabellarais": "cabellar",
    "cabellaran": "cabellar",
    "cabellaras": "cabellar",
    "cabellare": "cabellar",
    "cabellareis": "cabellar",
    "cabellaremos": "cabellar",
    "cabellaren": "cabellar",
    "cabellares": "cabellar",
    "cabellaron": "cabellar",
    "cabellará": "cabellar",
    "cabellarán": "cabellar",
    "cabellarás": "cabellar",
    "cabellaré": "cabellar",
    "cabellaréis": "cabellar",
    "cabellaría": "cabellar",
    "cabellaríais": "cabellar",
    "cabellaríamos": "cabellar",
    "cabellarían": "cabellar",
    "cabellarías": "cabellar",
    "cabellas": "cabellar",
    "cabellase": "cabellar",
    "cabellaseis": "cabellar",
    "cabellasen": "cabellar",
    "cabellases": "cabellar",
    "cabellaste": "cabellar",
    "cabellasteis": "cabellar",
    "cabelle": "cabellar",
    "cabellemos": "cabellar",
    "cabellen": "cabellar",
    "cabelleras": "cabellera",
    "cabelles": "cabellar",
    "cabello": "cabellar",
    "cabellos": "cabello",
    "cabelluda": "cabelludo",
    "cabelludas": "cabelludo",
    "cabelludos": "cabelludo",
    "cabellábamos": "cabellar",
    "cabelláis": "cabellar",
    "cabelláramos": "cabellar",
    "cabelláremos": "cabellar",
    "cabellásemos": "cabellar",
    "cabellé": "cabellar",
    "cabelléis": "cabellar",
    "cabelló": "cabellar",
    "cabemos": "caber",
    "caben": "caber",
    "cabes": "caber",
    "cabestra": "cabestrar",
    "cabestraba": "cabestrar",
    "cabestrabais": "cabestrar",
    "cabestraban": "cabestrar",
    "cabestrabas": "cabestrar",
    "cabestrad": "cabestrar",
    "cabestrada": "cabestrar",
    "cabestradas": "cabestrar",
    "cabestrado": "cabestrar",
    "cabestrados": "cabestrar",
    "cabestrajes": "cabestraje",
    "cabestramos": "cabestrar",
    "cabestran": "cabestrar",
    "cabestrando": "cabestrar",
    "cabestrantes": "cabestrante",
    "cabestrara": "cabestrar",
    "cabestrarais": "cabestrar",
    "cabestraran": "cabestrar",
    "cabestraras": "cabestrar",
    "cabestrare": "cabestrar",
    "cabestrareis": "cabestrar",
    "cabestraremos": "cabestrar",
    "cabestraren": "cabestrar",
    "cabestrares": "cabestrar",
    "cabestraron": "cabestrar",
    "cabestrará": "cabestrar",
    "cabestrarán": "cabestrar",
    "cabestrarás": "cabestrar",
    "cabestraré": "cabestrar",
    "cabestraréis": "cabestrar",
    "cabestraría": "cabestrar",
    "cabestraríais": "cabestrar",
    "cabestraríamos": "cabestrar",
    "cabestrarían": "cabestrar",
    "cabestrarías": "cabestrar",
    "cabestras": "cabestrar",
    "cabestrase": "cabestrar",
    "cabestraseis": "cabestrar",
    "cabestrasen": "cabestrar",
    "cabestrases": "cabestrar",
    "cabestraste": "cabestrar",
    "cabestrasteis": "cabestrar",
    "cabestre": "cabestrar",
    "cabestrea": "cabestrear",
    "cabestreaba": "cabestrear",
    "cabestreabais": "cabestrear",
    "cabestreaban": "cabestrear",
    "cabestreabas": "cabestrear",
    "cabestread": "cabestrear",
    "cabestreada": "cabestrear",
    "cabestreadas": "cabestrear",
    "cabestreado": "cabestrear",
    "cabestreados": "cabestrear",
    "cabestreamos": "cabestrear",
    "cabestrean": "cabestrear",
    "cabestreando": "cabestrear",
    "cabestreara": "cabestrear",
    "cabestrearais": "cabestrear",
    "cabestrearan": "cabestrear",
    "cabestrearas": "cabestrear",
    "cabestreare": "cabestrear",
    "cabestreareis": "cabestrear",
    "cabestrearemos": "cabestrear",
    "cabestrearen": "cabestrear",
    "cabestreares": "cabestrear",
    "cabestrearon": "cabestrear",
    "cabestreará": "cabestrear",
    "cabestrearán": "cabestrear",
    "cabestrearás": "cabestrear",
    "cabestrearé": "cabestrear",
    "cabestrearéis": "cabestrear",
    "cabestrearía": "cabestrear",
    "cabestrearíais": "cabestrear",
    "cabestrearíamos": "cabestrear",
    "cabestrearían": "cabestrear",
    "cabestrearías": "cabestrear",
    "cabestreas": "cabestrear",
    "cabestrease": "cabestrear",
    "cabestreaseis": "cabestrear",
    "cabestreasen": "cabestrear",
    "cabestreases": "cabestrear",
    "cabestreaste": "cabestrear",
    "cabestreasteis": "cabestrear",
    "cabestree": "cabestrear",
    "cabestreemos": "cabestrear",
    "cabestreen": "cabestrear",
    "cabestrees": "cabestrear",
    "cabestremos": "cabestrar",
    "cabestren": "cabestrar",
    "cabestreo": "cabestrear",
    "cabestreros": "cabestrero",
    "cabestrerías": "cabestrería",
    "cabestres": "cabestrar",
    "cabestreábamos": "cabestrear",
    "cabestreáis": "cabestrear",
    "cabestreáramos": "cabestrear",
    "cabestreáremos": "cabestrear",
    "cabestreásemos": "cabestrear",
    "cabestreé": "cabestrear",
    "cabestreéis": "cabestrear",
    "cabestreó": "cabestrear",
    "cabestrillos": "cabestrillo",
    "cabestro": "cabestrar",
    "cabestros": "cabestro",
    "cabestrábamos": "cabestrar",
    "cabestráis": "cabestrar",
    "cabestráramos": "cabestrar",
    "cabestráremos": "cabestrar",
    "cabestrásemos": "cabestrar",
    "cabestré": "cabestrar",
    "cabestréis": "cabestrar",
    "cabestró": "cabestrar",
    "cabetes": "cabete",
    "cabezadas": "cabezada",
    "cabezales": "cabezal",
    "cabezas": "cabeza",
    "cabezazos": "cabezazo",
    "cabezona": "cabezón",
    "cabezonadas": "cabezonada",
    "cabezonas": "cabezón",
    "cabezones": "cabezón",
    "cabezorros": "cabezorro",
    "cabezos": "cabezo",
    "cabezotas": "cabezota",
    "cabezuda": "cabezudo",
    "cabezudas": "cabezudo",
    "cabezudos": "cabezudo",
    "cabezuelas": "cabezuela",
    "cabida": "caber",
    "cabidas": "cabida",
    "cabido": "caber",
    "cabidos": "caber",
    "cabiendo": "caber",
    "cabilas": "cabila",
    "cabildadas": "cabildada",
    "cabildea": "cabildear",
    "cabildeaba": "cabildear",
    "cabildeabais": "cabildear",
    "cabildeaban": "cabildear",
    "cabildeabas": "cabildear",
    "cabildead": "cabildear",
    "cabildeada": "cabildear",
    "cabildeadas": "cabildear",
    "cabildeado": "cabildear",
    "cabildeados": "cabildear",
    "cabildeamos": "cabildear",
    "cabildean": "cabildear",
    "cabildeando": "cabildear",
    "cabildeara": "cabildear",
    "cabildearais": "cabildear",
    "cabildearan": "cabildear",
    "cabildearas": "cabildear",
    "cabildeare": "cabildear",
    "cabildeareis": "cabildear",
    "cabildearemos": "cabildear",
    "cabildearen": "cabildear",
    "cabildeares": "cabildear",
    "cabildearon": "cabildear",
    "cabildeará": "cabildear",
    "cabildearán": "cabildear",
    "cabildearás": "cabildear",
    "cabildearé": "cabildear",
    "cabildearéis": "cabildear",
    "cabildearía": "cabildear",
    "cabildearíais": "cabildear",
    "cabildearíamos": "cabildear",
    "cabildearían": "cabildear",
    "cabildearías": "cabildear",
    "cabildeas": "cabildear",
    "cabildease": "cabildear",
    "cabildeaseis": "cabildear",
    "cabildeasen": "cabildear",
    "cabildeases": "cabildear",
    "cabildeaste": "cabildear",
    "cabildeasteis": "cabildear",
    "cabildee": "cabildear",
    "cabildeemos": "cabildear",
    "cabildeen": "cabildear",
    "cabildees": "cabildear",
    "cabildeo": "cabildear",
    "cabildeos": "cabildeo",
    "cabildera": "cabildero",
    "cabilderas": "cabildero",
    "cabilderos": "cabildero",
    "cabildeábamos": "cabildear",
    "cabildeáis": "cabildear",
    "cabildeáramos": "cabildear",
    "cabildeáremos": "cabildear",
    "cabildeásemos": "cabildear",
    "cabildeé": "cabildear",
    "cabildeéis": "cabildear",
    "cabildeó": "cabildear",
    "cabildos": "cabildo",
    "cabileña": "cabileño",
    "cabileñas": "cabileño",
    "cabileños": "cabileño",
    "cabillas": "cabilla",
    "cabilleros": "cabillero",
    "cabimientos": "cabimiento",
    "cabinas": "cabina",
    "cabinetes": "cabinete",
    "cabios": "cabio",
    "cabiros": "cabiro",
    "cabizbaja": "cabizbajo",
    "cabizbajas": "cabizbajo",
    "cabizbajos": "cabizbajo",
    "cabizmordida": "cabizmordido",
    "cabizmordidas": "cabizmordido",
    "cabizmordidos": "cabizmordido",
    "cablea": "cablear",
    "cableaba": "cablear",
    "cableabais": "cablear",
    "cableaban": "cablear",
    "cableabas": "cablear",
    "cablead": "cablear",
    "cableada": "cablear",
    "cableadas": "cablear",
    "cableado": "cablear",
    "cableadoras": "cableadora",
    "cableados": "cablear",
    "cableamos": "cablear",
    "cablean": "cablear",
    "cableando": "cablear",
    "cableara": "cablear",
    "cablearais": "cablear",
    "cablearan": "cablear",
    "cablearas": "cablear",
    "cableare": "cablear",
    "cableareis": "cablear",
    "cablearemos": "cablear",
    "cablearen": "cablear",
    "cableares": "cablear",
    "cablearon": "cablear",
    "cableará": "cablear",
    "cablearán": "cablear",
    "cablearás": "cablear",
    "cablearé": "cablear",
    "cablearéis": "cablear",
    "cablearía": "cablear",
    "cablearíais": "cablear",
    "cablearíamos": "cablear",
    "cablearían": "cablear",
    "cablearías": "cablear",
    "cableas": "cablear",
    "cablease": "cablear",
    "cableaseis": "cablear",
    "cableasen": "cablear",
    "cableases": "cablear",
    "cableaste": "cablear",
    "cableasteis": "cablear",
    "cablee": "cablear",
    "cableemos": "cablear",
    "cableen": "cablear",
    "cablees": "cablear",
    "cablegrafiaba": "cablegrafiar",
    "cablegrafiabais": "cablegrafiar",
    "cablegrafiaban": "cablegrafiar",
    "cablegrafiabas": "cablegrafiar",
    "cablegrafiad": "cablegrafiar",
    "cablegrafiada": "cablegrafiar",
    "cablegrafiadas": "cablegrafiar",
    "cablegrafiado": "cablegrafiar",
    "cablegrafiados": "cablegrafiar",
    "cablegrafiamos": "cablegrafiar",
    "cablegrafiando": "cablegrafiar",
    "cablegrafiara": "cablegrafiar",
    "cablegrafiarais": "cablegrafiar",
    "cablegrafiaran": "cablegrafiar",
    "cablegrafiaras": "cablegrafiar",
    "cablegrafiare": "cablegrafiar",
    "cablegrafiareis": "cablegrafiar",
    "cablegrafiaremos": "cablegrafiar",
    "cablegrafiaren": "cablegrafiar",
    "cablegrafiares": "cablegrafiar",
    "cablegrafiaron": "cablegrafiar",
    "cablegrafiará": "cablegrafiar",
    "cablegrafiarán": "cablegrafiar",
    "cablegrafiarás": "cablegrafiar",
    "cablegrafiaré": "cablegrafiar",
    "cablegrafiaréis": "cablegrafiar",
    "cablegrafiaría": "cablegrafiar",
    "cablegrafiaríais": "cablegrafiar",
    "cablegrafiaríamos": "cablegrafiar",
    "cablegrafiarían": "cablegrafiar",
    "cablegrafiarías": "cablegrafiar",
    "cablegrafiase": "cablegrafiar",
    "cablegrafiaseis": "cablegrafiar",
    "cablegrafiasen": "cablegrafiar",
    "cablegrafiases": "cablegrafiar",
    "cablegrafiaste": "cablegrafiar",
    "cablegrafiasteis": "cablegrafiar",
    "cablegrafiemos": "cablegrafiar",
    "cablegrafiábamos": "cablegrafiar",
    "cablegrafiáis": "cablegrafiar",
    "cablegrafiáramos": "cablegrafiar",
    "cablegrafiáremos": "cablegrafiar",
    "cablegrafiásemos": "cablegrafiar",
    "cablegrafié": "cablegrafiar",
    "cablegrafiéis": "cablegrafiar",
    "cablegrafió": "cablegrafiar",
    "cablegrafía": "cablegrafiar",
    "cablegrafían": "cablegrafiar",
    "cablegrafías": "cablegrafiar",
    "cablegrafíe": "cablegrafiar",
    "cablegrafíen": "cablegrafiar",
    "cablegrafíes": "cablegrafiar",
    "cablegrafío": "cablegrafiar",
    "cablegramas": "cablegrama",
    "cablegráfica": "cablegráfico",
    "cablegráficas": "cablegráfico",
    "cablegráficos": "cablegráfico",
    "cableo": "cablear",
    "cablera": "cablero",
    "cableras": "cablero",
    "cableros": "cablero",
    "cables": "cable",
    "cableábamos": "cablear",
    "cableáis": "cablear",
    "cableáramos": "cablear",
    "cableáremos": "cablear",
    "cableásemos": "cablear",
    "cableé": "cablear",
    "cableéis": "cablear",
    "cableó": "cablear",
    "cabos": "cabo",
    "cabotajes": "cabotaje",
    "cabrachos": "cabracho",
    "cabrahigaba": "cabrahigar",
    "cabrahigabais": "cabrahigar",
    "cabrahigaban": "cabrahigar",
    "cabrahigabas": "cabrahigar",
    "cabrahigad": "cabrahigar",
    "cabrahigada": "cabrahigar",
    "cabrahigadas": "cabrahigar",
    "cabrahigado": "cabrahigar",
    "cabrahigados": "cabrahigar",
    "cabrahigamos": "cabrahigar",
    "cabrahigando": "cabrahigar",
    "cabrahigara": "cabrahigar",
    "cabrahigarais": "cabrahigar",
    "cabrahigaran": "cabrahigar",
    "cabrahigaras": "cabrahigar",
    "cabrahigare": "cabrahigar",
    "cabrahigareis": "cabrahigar",
    "cabrahigaremos": "cabrahigar",
    "cabrahigaren": "cabrahigar",
    "cabrahigares": "cabrahigar",
    "cabrahigaron": "cabrahigar",
    "cabrahigará": "cabrahigar",
    "cabrahigarán": "cabrahigar",
    "cabrahigarás": "cabrahigar",
    "cabrahigaré": "cabrahigar",
    "cabrahigaréis": "cabrahigar",
    "cabrahigaría": "cabrahigar",
    "cabrahigaríais": "cabrahigar",
    "cabrahigaríamos": "cabrahigar",
    "cabrahigarían": "cabrahigar",
    "cabrahigarías": "cabrahigar",
    "cabrahigase": "cabrahigar",
    "cabrahigaseis": "cabrahigar",
    "cabrahigasen": "cabrahigar",
    "cabrahigases": "cabrahigar",
    "cabrahigaste": "cabrahigar",
    "cabrahigasteis": "cabrahigar",
    "cabrahiguemos": "cabrahigar",
    "cabrahigué": "cabrahigar",
    "cabrahiguéis": "cabrahigar",
    "cabrahigábamos": "cabrahigar",
    "cabrahigáis": "cabrahigar",
    "cabrahigáramos": "cabrahigar",
    "cabrahigáremos": "cabrahigar",
    "cabrahigásemos": "cabrahigar",
    "cabrahigó": "cabrahigar",
    "cabrahíga": "cabrahigar",
    "cabrahígan": "cabrahigar",
    "cabrahígas": "cabrahigar",
    "cabrahígo": "cabrahigar",
    "cabrahígos": "cabrahígo",
    "cabrahígue": "cabrahigar",
    "cabrahíguen": "cabrahigar",
    "cabrahígues": "cabrahigar",
    "cabras": "cabra",
    "cabrea": "cabrear",
    "cabreaba": "cabrear",
    "cabreabais": "cabrear",
    "cabreaban": "cabrear",
    "cabreabas": "cabrear",
    "cabread": "cabrear",
    "cabreada": "cabrear",
    "cabreadas": "cabrear",
    "cabreado": "cabrear",
    "cabreados": "cabrear",
    "cabreamos": "cabrear",
    "cabrean": "cabrear",
    "cabreando": "cabrear",
    "cabreantes": "cabreante",
    "cabreara": "cabrear",
    "cabrearais": "cabrear",
    "cabrearan": "cabrear",
    "cabrearas": "cabrear",
    "cabreare": "cabrear",
    "cabreareis": "cabrear",
    "cabrearemos": "cabrear",
    "cabrearen": "cabrear",
    "cabreares": "cabrear",
    "cabrearon": "cabrear",
    "cabreará": "cabrear",
    "cabrearán": "cabrear",
    "cabrearás": "cabrear",
    "cabrearé": "cabrear",
    "cabrearéis": "cabrear",
    "cabrearía": "cabrear",
    "cabrearíais": "cabrear",
    "cabrearíamos": "cabrear",
    "cabrearían": "cabrear",
    "cabrearías": "cabrear",
    "cabreas": "cabrear",
    "cabrease": "cabrear",
    "cabreaseis": "cabrear",
    "cabreasen": "cabrear",
    "cabreases": "cabrear",
    "cabreaste": "cabrear",
    "cabreasteis": "cabrear",
    "cabree": "cabrear",
    "cabreemos": "cabrear",
    "cabreen": "cabrear",
    "cabrees": "cabrear",
    "cabremos": "caber",
    "cabreo": "cabrear",
    "cabreos": "cabreo",
    "cabrera": "cabrero",
    "cabreras": "cabrero",
    "cabreriza": "cabrerizo",
    "cabrerizas": "cabrerizo",
    "cabrerizos": "cabrerizo",
    "cabreros": "cabrero",
    "cabrerías": "cabrería",
    "cabrestantes": "cabrestante",
    "cabreábamos": "cabrear",
    "cabreáis": "cabrear",
    "cabreáramos": "cabrear",
    "cabreáremos": "cabrear",
    "cabreásemos": "cabrear",
    "cabreé": "cabrear",
    "cabreéis": "cabrear",
    "cabreó": "cabrear",
    "cabrias": "cabria",
    "cabrillas": "cabrilla",
    "cabrillea": "cabrillear",
    "cabrilleaba": "cabrillear",
    "cabrilleabais": "cabrillear",
    "cabrilleaban": "cabrillear",
    "cabrilleabas": "cabrillear",
    "cabrillead": "cabrillear",
    "cabrilleada": "cabrillear",
    "cabrilleadas": "cabrillear",
    "cabrilleado": "cabrillear",
    "cabrilleados": "cabrillear",
    "cabrilleamos": "cabrillear",
    "cabrillean": "cabrillear",
    "cabrilleando": "cabrillear",
    "cabrilleara": "cabrillear",
    "cabrillearais": "cabrillear",
    "cabrillearan": "cabrillear",
    "cabrillearas": "cabrillear",
    "cabrilleare": "cabrillear",
    "cabrilleareis": "cabrillear",
    "cabrillearemos": "cabrillear",
    "cabrillearen": "cabrillear",
    "cabrilleares": "cabrillear",
    "cabrillearon": "cabrillear",
    "cabrilleará": "cabrillear",
    "cabrillearán": "cabrillear",
    "cabrillearás": "cabrillear",
    "cabrillearé": "cabrillear",
    "cabrillearéis": "cabrillear",
    "cabrillearía": "cabrillear",
    "cabrillearíais": "cabrillear",
    "cabrillearíamos": "cabrillear",
    "cabrillearían": "cabrillear",
    "cabrillearías": "cabrillear",
    "cabrilleas": "cabrillear",
    "cabrillease": "cabrillear",
    "cabrilleaseis": "cabrillear",
    "cabrilleasen": "cabrillear",
    "cabrilleases": "cabrillear",
    "cabrilleaste": "cabrillear",
    "cabrilleasteis": "cabrillear",
    "cabrillee": "cabrillear",
    "cabrilleemos": "cabrillear",
    "cabrilleen": "cabrillear",
    "cabrillees": "cabrillear",
    "cabrilleo": "cabrillear",
    "cabrilleos": "cabrilleo",
    "cabrilleábamos": "cabrillear",
    "cabrilleáis": "cabrillear",
    "cabrilleáramos": "cabrillear",
    "cabrilleáremos": "cabrillear",
    "cabrilleásemos": "cabrillear",
    "cabrilleé": "cabrillear",
    "cabrilleéis": "cabrillear",
    "cabrilleó": "cabrillear",
    "cabriola": "cabriolar",
    "cabriolaba": "cabriolar",
    "cabriolabais": "cabriolar",
    "cabriolaban": "cabriolar",
    "cabriolabas": "cabriolar",
    "cabriolad": "cabriolar",
    "cabriolada": "cabriolar",
    "cabrioladas": "cabriolar",
    "cabriolado": "cabriolar",
    "cabriolados": "cabriolar",
    "cabriolamos": "cabriolar",
    "cabriolan": "cabriolar",
    "cabriolando": "cabriolar",
    "cabriolara": "cabriolar",
    "cabriolarais": "cabriolar",
    "cabriolaran": "cabriolar",
    "cabriolaras": "cabriolar",
    "cabriolare": "cabriolar",
    "cabriolareis": "cabriolar",
    "cabriolaremos": "cabriolar",
    "cabriolaren": "cabriolar",
    "cabriolares": "cabriolar",
    "cabriolaron": "cabriolar",
    "cabriolará": "cabriolar",
    "cabriolarán": "cabriolar",
    "cabriolarás": "cabriolar",
    "cabriolaré": "cabriolar",
    "cabriolaréis": "cabriolar",
    "cabriolaría": "cabriolar",
    "cabriolaríais": "cabriolar",
    "cabriolaríamos": "cabriolar",
    "cabriolarían": "cabriolar",
    "cabriolarías": "cabriolar",
    "cabriolas": "cabriolar",
    "cabriolase": "cabriolar",
    "cabriolaseis": "cabriolar",
    "cabriolasen": "cabriolar",
    "cabriolases": "cabriolar",
    "cabriolaste": "cabriolar",
    "cabriolasteis": "cabriolar",
    "cabriole": "cabriolar",
    "cabriolea": "cabriolear",
    "cabrioleaba": "cabriolear",
    "cabrioleabais": "cabriolear",
    "cabrioleaban": "cabriolear",
    "cabrioleabas": "cabriolear",
    "cabriolead": "cabriolear",
    "cabrioleada": "cabriolear",
    "cabrioleadas": "cabriolear",
    "cabrioleado": "cabriolear",
    "cabrioleados": "cabriolear",
    "cabrioleamos": "cabriolear",
    "cabriolean": "cabriolear",
    "cabrioleando": "cabriolear",
    "cabrioleara": "cabriolear",
    "cabriolearais": "cabriolear",
    "cabriolearan": "cabriolear",
    "cabriolearas": "cabriolear",
    "cabrioleare": "cabriolear",
    "cabrioleareis": "cabriolear",
    "cabriolearemos": "cabriolear",
    "cabriolearen": "cabriolear",
    "cabrioleares": "cabriolear",
    "cabriolearon": "cabriolear",
    "cabrioleará": "cabriolear",
    "cabriolearán": "cabriolear",
    "cabriolearás": "cabriolear",
    "cabriolearé": "cabriolear",
    "cabriolearéis": "cabriolear",
    "cabriolearía": "cabriolear",
    "cabriolearíais": "cabriolear",
    "cabriolearíamos": "cabriolear",
    "cabriolearían": "cabriolear",
    "cabriolearías": "cabriolear",
    "cabrioleas": "cabriolear",
    "cabriolease": "cabriolear",
    "cabrioleaseis": "cabriolear",
    "cabrioleasen": "cabriolear",
    "cabrioleases": "cabriolear",
    "cabrioleaste": "cabriolear",
    "cabrioleasteis": "cabriolear",
    "cabriolee": "cabriolear",
    "cabrioleemos": "cabriolear",
    "cabrioleen": "cabriolear",
    "cabriolees": "cabriolear",
    "cabriolemos": "cabriolar",
    "cabriolen": "cabriolar",
    "cabrioleo": "cabriolear",
    "cabrioles": "cabriolar",
    "cabriolets": "cabriolet",
    "cabrioleábamos": "cabriolear",
    "cabrioleáis": "cabriolear",
    "cabrioleáramos": "cabriolear",
    "cabrioleáremos": "cabriolear",
    "cabrioleásemos": "cabriolear",
    "cabrioleé": "cabriolear",
    "cabrioleéis": "cabriolear",
    "cabrioleó": "cabriolear",
    "cabriolo": "cabriolar",
    "cabriolábamos": "cabriolar",
    "cabrioláis": "cabriolar",
    "cabrioláramos": "cabriolar",
    "cabrioláremos": "cabriolar",
    "cabriolásemos": "cabriolar",
    "cabriolé": "cabriolar",
    "cabrioléis": "cabriolar",
    "cabriolés": "cabriolé",
    "cabrioló": "cabriolar",
    "cabriones": "cabrión",
    "cabrios": "cabrio",
    "cabrita": "cabrito",
    "cabritas": "cabrito",
    "cabritillas": "cabritilla",
    "cabritillo": "cabritilla",
    "cabritillos": "cabritilla",
    "cabritos": "cabrito",
    "cabrituna": "cabrituno",
    "cabritunas": "cabrituno",
    "cabritunos": "cabrituno",
    "cabrona": "cabrón",
    "cabronadas": "cabronada",
    "cabronas": "cabrón",
    "cabrones": "cabrón",
    "cabruna": "cabruno",
    "cabrunas": "cabruno",
    "cabrunos": "cabruno",
    "cabrá": "caber",
    "cabrán": "caber",
    "cabrás": "caber",
    "cabré": "caber",
    "cabréis": "caber",
    "cabría": "cabrío",
    "cabríais": "caber",
    "cabríamos": "caber",
    "cabrían": "caber",
    "cabrías": "cabrío",
    "cabríos": "cabrío",
    "cabujones": "cabujón",
    "cabuyas": "cabuya",
    "cabuyeras": "cabuyera",
    "cabuyerías": "cabuyería",
    "cabéis": "caber",
    "cabía": "caber",
    "cabíais": "caber",
    "cabíamos": "caber",
    "cabían": "caber",
    "cabías": "caber",
    "cacahuales": "cacahual",
    "cacahuates": "cacahuate",
    "cacahuera": "cacahuero",
    "cacahueras": "cacahuero",
    "cacahueros": "cacahuero",
    "cacahuetes": "cacahuete",
    "cacahués": "cacahué",
    "cacaos": "cacao",
    "cacaotales": "cacaotal",
    "cacaraña": "cacarañar",
    "cacarañaba": "cacarañar",
    "cacarañabais": "cacarañar",
    "cacarañaban": "cacarañar",
    "cacarañabas": "cacarañar",
    "cacarañad": "cacarañar",
    "cacarañada": "cacarañar",
    "cacarañadas": "cacarañar",
    "cacarañado": "cacarañar",
    "cacarañados": "cacarañar",
    "cacarañamos": "cacarañar",
    "cacarañan": "cacarañar",
    "cacarañando": "cacarañar",
    "cacarañara": "cacarañar",
    "cacarañarais": "cacarañar",
    "cacarañaran": "cacarañar",
    "cacarañaras": "cacarañar",
    "cacarañare": "cacarañar",
    "cacarañareis": "cacarañar",
    "cacarañaremos": "cacarañar",
    "cacarañaren": "cacarañar",
    "cacarañares": "cacarañar",
    "cacarañaron": "cacarañar",
    "cacarañará": "cacarañar",
    "cacarañarán": "cacarañar",
    "cacarañarás": "cacarañar",
    "cacarañaré": "cacarañar",
    "cacarañaréis": "cacarañar",
    "cacarañaría": "cacarañar",
    "cacarañaríais": "cacarañar",
    "cacarañaríamos": "cacarañar",
    "cacarañarían": "cacarañar",
    "cacarañarías": "cacarañar",
    "cacarañas": "cacarañar",
    "cacarañase": "cacarañar",
    "cacarañaseis": "cacarañar",
    "cacarañasen": "cacarañar",
    "cacarañases": "cacarañar",
    "cacarañaste": "cacarañar",
    "cacarañasteis": "cacarañar",
    "cacarañe": "cacarañar",
    "cacarañemos": "cacarañar",
    "cacarañen": "cacarañar",
    "cacarañes": "cacarañar",
    "cacaraño": "cacarañar",
    "cacarañábamos": "cacarañar",
    "cacarañáis": "cacarañar",
    "cacarañáramos": "cacarañar",
    "cacarañáremos": "cacarañar",
    "cacarañásemos": "cacarañar",
    "cacarañé": "cacarañar",
    "cacarañéis": "cacarañar",
    "cacarañó": "cacarañar",
    "cacarea": "cacarear",
    "cacareaba": "cacarear",
    "cacareabais": "cacarear",
    "cacareaban": "cacarear",
    "cacareabas": "cacarear",
    "cacaread": "cacarear",
    "cacareada": "cacarear",
    "cacareadas": "cacarear",
    "cacareado": "cacarear",
    "cacareados": "cacarear",
    "cacareamos": "cacarear",
    "cacarean": "cacarear",
    "cacareando": "cacarear",
    "cacareara": "cacarear",
    "cacarearais": "cacarear",
    "cacarearan": "cacarear",
    "cacarearas": "cacarear",
    "cacareare": "cacarear",
    "cacareareis": "cacarear",
    "cacarearemos": "cacarear",
    "cacarearen": "cacarear",
    "cacareares": "cacarear",
    "cacarearon": "cacarear",
    "cacareará": "cacarear",
    "cacarearán": "cacarear",
    "cacarearás": "cacarear",
    "cacarearé": "cacarear",
    "cacarearéis": "cacarear",
    "cacarearía": "cacarear",
    "cacarearíais": "cacarear",
    "cacarearíamos": "cacarear",
    "cacarearían": "cacarear",
    "cacarearías": "cacarear",
    "cacareas": "cacarear",
    "cacarease": "cacarear",
    "cacareaseis": "cacarear",
    "cacareasen": "cacarear",
    "cacareases": "cacarear",
    "cacareaste": "cacarear",
    "cacareasteis": "cacarear",
    "cacaree": "cacarear",
    "cacareemos": "cacarear",
    "cacareen": "cacarear",
    "cacarees": "cacarear",
    "cacareo": "cacarear",
    "cacareos": "cacareo",
    "cacareábamos": "cacarear",
    "cacareáis": "cacarear",
    "cacareáramos": "cacarear",
    "cacareáremos": "cacarear",
    "cacareásemos": "cacarear",
    "cacareé": "cacarear",
    "cacareéis": "cacarear",
    "cacareó": "cacarear",
    "cacas": "caca",
    "cacatúas": "cacatúa",
    "caccias": "caccia",
    "cace": "cazar",
    "cacea": "cacear",
    "caceaba": "cacear",
    "caceabais": "cacear",
    "caceaban": "cacear",
    "caceabas": "cacear",
    "cacead": "cacear",
    "caceada": "cacear",
    "caceadas": "cacear",
    "caceado": "cacear",
    "caceados": "cacear",
    "caceamos": "cacear",
    "cacean": "cacear",
    "caceando": "cacear",
    "caceara": "cacear",
    "cacearais": "cacear",
    "cacearan": "cacear",
    "cacearas": "cacear",
    "caceare": "cacear",
    "caceareis": "cacear",
    "cacearemos": "cacear",
    "cacearen": "cacear",
    "caceares": "cacear",
    "cacearon": "cacear",
    "caceará": "cacear",
    "cacearán": "cacear",
    "cacearás": "cacear",
    "cacearé": "cacear",
    "cacearéis": "cacear",
    "cacearía": "cacear",
    "cacearíais": "cacear",
    "cacearíamos": "cacear",
    "cacearían": "cacear",
    "cacearías": "cacear",
    "caceas": "cacear",
    "cacease": "cacear",
    "caceaseis": "cacear",
    "caceasen": "cacear",
    "caceases": "cacear",
    "caceaste": "cacear",
    "caceasteis": "cacear",
    "cacee": "cacear",
    "caceemos": "cacear",
    "caceen": "cacear",
    "cacees": "cacear",
    "cacemos": "cazar",
    "cacen": "cazar",
    "caceo": "cacear",
    "caceras": "cacera",
    "cacereña": "cacereño",
    "cacereñas": "cacereño",
    "cacereños": "cacereño",
    "cacerinas": "cacerina",
    "cacerolas": "cacerola",
    "cacerías": "cacería",
    "caces": "cazar",
    "cacetas": "caceta",
    "caceábamos": "cacear",
    "caceáis": "cacear",
    "caceáramos": "cacear",
    "caceáremos": "cacear",
    "caceásemos": "cacear",
    "caceé": "cacear",
    "caceéis": "cacear",
    "caceó": "cacear",
    "cacha": "cacho",
    "cachaba": "cachar",
    "cachabais": "cachar",
    "cachaban": "cachar",
    "cachabas": "cachar",
    "cachacos": "cachaco",
    "cachad": "cachar",
    "cachada": "cachar",
    "cachadas": "cachar",
    "cachado": "cachar",
    "cachados": "cachar",
    "cachalotes": "cachalote",
    "cachamos": "cachar",
    "cachan": "cachar",
    "cachando": "cachar",
    "cachanos": "cachano",
    "cachapas": "cachapa",
    "cachara": "cachar",
    "cacharais": "cachar",
    "cacharan": "cachar",
    "cacharas": "cachar",
    "cachare": "cachar",
    "cachareis": "cachar",
    "cacharemos": "cachar",
    "cacharen": "cachar",
    "cachares": "cachar",
    "cacharon": "cachar",
    "cacharrazos": "cacharrazo",
    "cacharrera": "cacharrero",
    "cacharreras": "cacharrero",
    "cacharreros": "cacharrero",
    "cacharrerías": "cacharrería",
    "cacharros": "cacharro",
    "cachará": "cachar",
    "cacharán": "cachar",
    "cacharás": "cachar",
    "cacharé": "cachar",
    "cacharéis": "cachar",
    "cacharía": "cachar",
    "cacharíais": "cachar",
    "cacharíamos": "cachar",
    "cacharían": "cachar",
    "cacharías": "cachar",
    "cachas": "cacho",
    "cachase": "cachar",
    "cachaseis": "cachar",
    "cachasen": "cachar",
    "cachases": "cachar",
    "cachaste": "cachar",
    "cachasteis": "cachar",
    "cachavas": "cachava",
    "cachazas": "cachaza",
    "cache": "cachar",
    "cachea": "cachear",
    "cacheaba": "cachear",
    "cacheabais": "cachear",
    "cacheaban": "cachear",
    "cacheabas": "cachear",
    "cachead": "cachear",
    "cacheada": "cachear",
    "cacheadas": "cachear",
    "cacheado": "cachear",
    "cacheados": "cachear",
    "cacheamos": "cachear",
    "cachean": "cachear",
    "cacheando": "cachear",
    "cacheara": "cachear",
    "cachearais": "cachear",
    "cachearan": "cachear",
    "cachearas": "cachear",
    "cacheare": "cachear",
    "cacheareis": "cachear",
    "cachearemos": "cachear",
    "cachearen": "cachear",
    "cacheares": "cachear",
    "cachearon": "cachear",
    "cacheará": "cachear",
    "cachearán": "cachear",
    "cachearás": "cachear",
    "cachearé": "cachear",
    "cachearéis": "cachear",
    "cachearía": "cachear",
    "cachearíais": "cachear",
    "cachearíamos": "cachear",
    "cachearían": "cachear",
    "cachearías": "cachear",
    "cacheas": "cachear",
    "cachease": "cachear",
    "cacheaseis": "cachear",
    "cacheasen": "cachear",
    "cacheases": "cachear",
    "cacheaste": "cachear",
    "cacheasteis": "cachear",
    "cachee": "cachear",
    "cacheemos": "cachear",
    "cacheen": "cachear",
    "cachees": "cachear",
    "cachemiras": "cachemira",
    "cachemires": "cachemir",
    "cachemos": "cachar",
    "cachen": "cachar",
    "cacheo": "cachear",
    "cacheos": "cacheo",
    "cacheras": "cachera",
    "caches": "cache",
    "cachetadas": "cachetada",
    "cachetea": "cachetear",
    "cacheteaba": "cachetear",
    "cacheteabais": "cachetear",
    "cacheteaban": "cachetear",
    "cacheteabas": "cachetear",
    "cachetead": "cachetear",
    "cacheteada": "cachetear",
    "cacheteadas": "cachetear",
    "cacheteado": "cachetear",
    "cacheteados": "cachetear",
    "cacheteamos": "cachetear",
    "cachetean": "cachetear",
    "cacheteando": "cachetear",
    "cacheteara": "cachetear",
    "cachetearais": "cachetear",
    "cachetearan": "cachetear",
    "cachetearas": "cachetear",
    "cacheteare": "cachetear",
    "cacheteareis": "cachetear",
    "cachetearemos": "cachetear",
    "cachetearen": "cachetear",
    "cacheteares": "cachetear",
    "cachetearon": "cachetear",
    "cacheteará": "cachetear",
    "cachetearán": "cachetear",
    "cachetearás": "cachetear",
    "cachetearé": "cachetear",
    "cachetearéis": "cachetear",
    "cachetearía": "cachetear",
    "cachetearíais": "cachetear",
    "cachetearíamos": "cachetear",
    "cachetearían": "cachetear",
    "cachetearías": "cachetear",
    "cacheteas": "cachetear",
    "cachetease": "cachetear",
    "cacheteaseis": "cachetear",
    "cacheteasen": "cachetear",
    "cacheteases": "cachetear",
    "cacheteaste": "cachetear",
    "cacheteasteis": "cachetear",
    "cachetee": "cachetear",
    "cacheteemos": "cachetear",
    "cacheteen": "cachetear",
    "cachetees": "cachetear",
    "cacheteo": "cachetear",
    "cacheteros": "cachetero",
    "cachetes": "cachete",
    "cacheteábamos": "cachetear",
    "cacheteáis": "cachetear",
    "cacheteáramos": "cachetear",
    "cacheteáremos": "cachetear",
    "cacheteásemos": "cachetear",
    "cacheteé": "cachetear",
    "cacheteéis": "cachetear",
    "cacheteó": "cachetear",
    "cachetinas": "cachetina",
    "cachetona": "cachetón",
    "cachetonas": "cachetón",
    "cachetones": "cachetón",
    "cachets": "cachet",
    "cachetuda": "cachetudo",
    "cachetudas": "cachetudo",
    "cachetudos": "cachetudo",
    "cacheábamos": "cachear",
    "cacheáis": "cachear",
    "cacheáramos": "cachear",
    "cacheáremos": "cachear",
    "cacheásemos": "cachear",
    "cacheé": "cachear",
    "cacheéis": "cachear",
    "cacheó": "cachear",
    "cachicuerna": "cachicuerno",
    "cachicuernas": "cachicuerno",
    "cachicuernos": "cachicuerno",
    "cachidiablos": "cachidiablo",
    "cachifolla": "cachifollar",
    "cachifollaba": "cachifollar",
    "cachifollabais": "cachifollar",
    "cachifollaban": "cachifollar",
    "cachifollabas": "cachifollar",
    "cachifollad": "cachifollar",
    "cachifollada": "cachifollar",
    "cachifolladas": "cachifollar",
    "cachifollado": "cachifollar",
    "cachifollados": "cachifollar",
    "cachifollamos": "cachifollar",
    "cachifollan": "cachifollar",
    "cachifollando": "cachifollar",
    "cachifollara": "cachifollar",
    "cachifollarais": "cachifollar",
    "cachifollaran": "cachifollar",
    "cachifollaras": "cachifollar",
    "cachifollare": "cachifollar",
    "cachifollareis": "cachifollar",
    "cachifollaremos": "cachifollar",
    "cachifollaren": "cachifollar",
    "cachifollares": "cachifollar",
    "cachifollaron": "cachifollar",
    "cachifollará": "cachifollar",
    "cachifollarán": "cachifollar",
    "cachifollarás": "cachifollar",
    "cachifollaré": "cachifollar",
    "cachifollaréis": "cachifollar",
    "cachifollaría": "cachifollar",
    "cachifollaríais": "cachifollar",
    "cachifollaríamos": "cachifollar",
    "cachifollarían": "cachifollar",
    "cachifollarías": "cachifollar",
    "cachifollas": "cachifollar",
    "cachifollase": "cachifollar",
    "cachifollaseis": "cachifollar",
    "cachifollasen": "cachifollar",
    "cachifollases": "cachifollar",
    "cachifollaste": "cachifollar",
    "cachifollasteis": "cachifollar",
    "cachifolle": "cachifollar",
    "cachifollemos": "cachifollar",
    "cachifollen": "cachifollar",
    "cachifolles": "cachifollar",
    "cachifollo": "cachifollar",
    "cachifollábamos": "cachifollar",
    "cachifolláis": "cachifollar",
    "cachifolláramos": "cachifollar",
    "cachifolláremos": "cachifollar",
    "cachifollásemos": "cachifollar",
    "cachifollé": "cachifollar",
    "cachifolléis": "cachifollar",
    "cachifolló": "cachifollar",
    "cachimbas": "cachimba",
    "cachimbos": "cachimbo",
    "cachipoda": "cachipodar",
    "cachipodaba": "cachipodar",
    "cachipodabais": "cachipodar",
    "cachipodaban": "cachipodar",
    "cachipodabas": "cachipodar",
    "cachipodad": "cachipodar",
    "cachipodada": "cachipodar",
    "cachipodadas": "cachipodar",
    "cachipodado": "cachipodar",
    "cachipodados": "cachipodar",
    "cachipodamos": "cachipodar",
    "cachipodan": "cachipodar",
    "cachipodando": "cachipodar",
    "cachipodara": "cachipodar",
    "cachipodarais": "cachipodar",
    "cachipodaran": "cachipodar",
    "cachipodaras": "cachipodar",
    "cachipodare": "cachipodar",
    "cachipodareis": "cachipodar",
    "cachipodaremos": "cachipodar",
    "cachipodaren": "cachipodar",
    "cachipodares": "cachipodar",
    "cachipodaron": "cachipodar",
    "cachipodará": "cachipodar",
    "cachipodarán": "cachipodar",
    "cachipodarás": "cachipodar",
    "cachipodaré": "cachipodar",
    "cachipodaréis": "cachipodar",
    "cachipodaría": "cachipodar",
    "cachipodaríais": "cachipodar",
    "cachipodaríamos": "cachipodar",
    "cachipodarían": "cachipodar",
    "cachipodarías": "cachipodar",
    "cachipodas": "cachipodar",
    "cachipodase": "cachipodar",
    "cachipodaseis": "cachipodar",
    "cachipodasen": "cachipodar",
    "cachipodases": "cachipodar",
    "cachipodaste": "cachipodar",
    "cachipodasteis": "cachipodar",
    "cachipode": "cachipodar",
    "cachipodemos": "cachipodar",
    "cachipoden": "cachipodar",
    "cachipodes": "cachipodar",
    "cachipodo": "cachipodar",
    "cachipodábamos": "cachipodar",
    "cachipodáis": "cachipodar",
    "cachipodáramos": "cachipodar",
    "cachipodáremos": "cachipodar",
    "cachipodásemos": "cachipodar",
    "cachipodé": "cachipodar",
    "cachipodéis": "cachipodar",
    "cachipodó": "cachipodar",
    "cachiporras": "cachiporra",
    "cachiporrazos": "cachiporrazo",
    "cachirulos": "cachirulo",
    "cachivaches": "cachivache",
    "cacho": "cachar",
    "cachonda": "cachondo",
    "cachondas": "cachondo",
    "cachondea": "cachondear",
    "cachondeaba": "cachondear",
    "cachondeabais": "cachondear",
    "cachondeaban": "cachondear",
    "cachondeabas": "cachondear",
    "cachondead": "cachondear",
    "cachondeada": "cachondear",
    "cachondeadas": "cachondear",
    "cachondeado": "cachondear",
    "cachondeados": "cachondear",
    "cachondeamos": "cachondear",
    "cachondean": "cachondear",
    "cachondeando": "cachondear",
    "cachondeara": "cachondear",
    "cachondearais": "cachondear",
    "cachondearan": "cachondear",
    "cachondearas": "cachondear",
    "cachondeare": "cachondear",
    "cachondeareis": "cachondear",
    "cachondearemos": "cachondear",
    "cachondearen": "cachondear",
    "cachondeares": "cachondear",
    "cachondearon": "cachondear",
    "cachondeará": "cachondear",
    "cachondearán": "cachondear",
    "cachondearás": "cachondear",
    "cachondearé": "cachondear",
    "cachondearéis": "cachondear",
    "cachondearía": "cachondear",
    "cachondearíais": "cachondear",
    "cachondearíamos": "cachondear",
    "cachondearían": "cachondear",
    "cachondearías": "cachondear",
    "cachondeas": "cachondear",
    "cachondease": "cachondear",
    "cachondeaseis": "cachondear",
    "cachondeasen": "cachondear",
    "cachondeases": "cachondear",
    "cachondeaste": "cachondear",
    "cachondeasteis": "cachondear",
    "cachondeces": "cachondez",
    "cachondee": "cachondear",
    "cachondeemos": "cachondear",
    "cachondeen": "cachondear",
    "cachondees": "cachondear",
    "cachondeo": "cachondear",
    "cachondeos": "cachondeo",
    "cachondeábamos": "cachondear",
    "cachondeáis": "cachondear",
    "cachondeáramos": "cachondear",
    "cachondeáremos": "cachondear",
    "cachondeásemos": "cachondear",
    "cachondeé": "cachondear",
    "cachondeéis": "cachondear",
    "cachondeó": "cachondear",
    "cachondos": "cachondo",
    "cachones": "cachón",
    "cachorra": "cachorro",
    "cachorras": "cachorro",
    "cachorrilla": "cachorrillo",
    "cachorrillas": "cachorrillo",
    "cachorrillos": "cachorrillo",
    "cachorros": "cachorro",
    "cachos": "cacho",
    "cachuchas": "cachucha",
    "cachuchos": "cachucho",
    "cachuda": "cachudo",
    "cachudas": "cachudo",
    "cachudos": "cachudo",
    "cachuelas": "cachuela",
    "cachuelos": "cachuelo",
    "cachupina": "cachupín",
    "cachupinadas": "cachupinada",
    "cachupinas": "cachupín",
    "cachupines": "cachupín",
    "cachábamos": "cachar",
    "cacháis": "cachar",
    "cacháramos": "cachar",
    "cacháremos": "cachar",
    "cachásemos": "cachar",
    "caché": "cachar",
    "cachéis": "cachar",
    "cachés": "caché",
    "cachó": "cachar",
    "cachúas": "cachúa",
    "cacicas": "cacica",
    "cacicatos": "cacicato",
    "cacicazgos": "cacicazgo",
    "cacimbas": "cacimba",
    "caciques": "cacique",
    "caciquiles": "caciquil",
    "caciquismos": "caciquismo",
    "cacles": "cacle",
    "cacodilatos": "cacodilato",
    "cacodílica": "cacodílico",
    "cacodílicas": "cacodílico",
    "cacodílicos": "cacodílico",
    "cacofagias": "cacofagia",
    "cacofonías": "cacofonía",
    "cacofónica": "cacofónico",
    "cacofónicas": "cacofónico",
    "cacofónicos": "cacofónico",
    "cacografías": "cacografía",
    "cacologías": "cacología",
    "cacoquimias": "cacoquimia",
    "cacos": "caco",
    "cacosmias": "cacosmia",
    "cactos": "cacto",
    "cactáceas": "cactácea",
    "cacuminales": "cacuminal",
    "cacé": "cazar",
    "cacéis": "cazar",
    "cacúmenes": "cacumen",
    "cadalsos": "cadalso",
    "cadarzos": "cadarzo",
    "cadavérica": "cadavérico",
    "cadavéricas": "cadavérico",
    "cadavéricos": "cadavérico",
    "cadañera": "cadañero",
    "cadañeras": "cadañero",
    "cadañeros": "cadañero",
    "caddies": "caddie",
    "cadejos": "cadejo",
    "cadenas": "cadena",
    "cadencias": "cadencia",
    "cadenciosa": "cadencioso",
    "cadenciosas": "cadencioso",
    "cadenciosos": "cadencioso",
    "cadenciómetros": "cadenciómetro",
    "cadenetas": "cadeneta",
    "cadenillas": "cadenilla",
    "cadentes": "cadente",
    "caderamenes": "caderamen",
    "caderas": "cadera",
    "cadetes": "cadete",
    "cadillos": "cadillo",
    "cadmia": "cadmiar",
    "cadmiaba": "cadmiar",
    "cadmiabais": "cadmiar",
    "cadmiaban": "cadmiar",
    "cadmiabas": "cadmiar",
    "cadmiad": "cadmiar",
    "cadmiada": "cadmiar",
    "cadmiadas": "cadmiar",
    "cadmiado": "cadmiar",
    "cadmiados": "cadmiar",
    "cadmiamos": "cadmiar",
    "cadmian": "cadmiar",
    "cadmiando": "cadmiar",
    "cadmiara": "cadmiar",
    "cadmiarais": "cadmiar",
    "cadmiaran": "cadmiar",
    "cadmiaras": "cadmiar",
    "cadmiare": "cadmiar",
    "cadmiareis": "cadmiar",
    "cadmiaremos": "cadmiar",
    "cadmiaren": "cadmiar",
    "cadmiares": "cadmiar",
    "cadmiaron": "cadmiar",
    "cadmiará": "cadmiar",
    "cadmiarán": "cadmiar",
    "cadmiarás": "cadmiar",
    "cadmiaré": "cadmiar",
    "cadmiaréis": "cadmiar",
    "cadmiaría": "cadmiar",
    "cadmiaríais": "cadmiar",
    "cadmiaríamos": "cadmiar",
    "cadmiarían": "cadmiar",
    "cadmiarías": "cadmiar",
    "cadmias": "cadmiar",
    "cadmiase": "cadmiar",
    "cadmiaseis": "cadmiar",
    "cadmiasen": "cadmiar",
    "cadmiases": "cadmiar",
    "cadmiaste": "cadmiar",
    "cadmiasteis": "cadmiar",
    "cadmie": "cadmiar",
    "cadmiemos": "cadmiar",
    "cadmien": "cadmiar",
    "cadmies": "cadmiar",
    "cadmio": "cadmiar",
    "cadmios": "cadmio",
    "cadmiábamos": "cadmiar",
    "cadmiáis": "cadmiar",
    "cadmiáramos": "cadmiar",
    "cadmiáremos": "cadmiar",
    "cadmiásemos": "cadmiar",
    "cadmié": "cadmiar",
    "cadmiéis": "cadmiar",
    "cadmió": "cadmiar",
    "caduca": "caduco",
    "caducaba": "caducar",
    "caducabais": "caducar",
    "caducaban": "caducar",
    "caducabas": "caducar",
    "caducad": "caducar",
    "caducada": "caducar",
    "caducadas": "caducar",
    "caducado": "caducar",
    "caducados": "caducar",
    "caducamos": "caducar",
    "caducan": "caducar",
    "caducando": "caducar",
    "caducara": "caducar",
    "caducarais": "caducar",
    "caducaran": "caducar",
    "caducaras": "caducar",
    "caducare": "caducar",
    "caducareis": "caducar",
    "caducaremos": "caducar",
    "caducaren": "caducar",
    "caducares": "caducar",
    "caducaron": "caducar",
    "caducará": "caducar",
    "caducarán": "caducar",
    "caducarás": "caducar",
    "caducaré": "caducar",
    "caducaréis": "caducar",
    "caducaría": "caducar",
    "caducaríais": "caducar",
    "caducaríamos": "caducar",
    "caducarían": "caducar",
    "caducarías": "caducar",
    "caducas": "caduco",
    "caducase": "caducar",
    "caducaseis": "caducar",
    "caducasen": "caducar",
    "caducases": "caducar",
    "caducaste": "caducar",
    "caducasteis": "caducar",
    "caduceos": "caduceo",
    "caducidades": "caducidad",
    "caducifolia": "caducifolio",
    "caducifolias": "caducifolio",
    "caducifolios": "caducifolio",
    "caduco": "caducar",
    "caducos": "caduco",
    "caducábamos": "caducar",
    "caducáis": "caducar",
    "caducáramos": "caducar",
    "caducáremos": "caducar",
    "caducásemos": "caducar",
    "caducó": "caducar",
    "caduque": "caducar",
    "caduquemos": "caducar",
    "caduquen": "caducar",
    "caduques": "caducar",
    "caduqué": "caducar",
    "caduquéis": "caducar",
    "cadáveres": "cadáver",
    "cadíes": "cadí",
    "cadís": "cadí",
    "cae": "caer",
    "caed": "caer",
    "caediza": "caedizo",
    "caedizas": "caedizo",
    "caedizos": "caedizo",
    "caeduras": "caedura",
    "caemos": "caer",
    "caen": "caer",
    "caeremos": "caer",
    "caerá": "caer",
    "caerán": "caer",
    "caerás": "caer",
    "caeré": "caer",
    "caeréis": "caer",
    "caería": "caer",
    "caeríais": "caer",
    "caeríamos": "caer",
    "caerían": "caer",
    "caerías": "caer",
    "caes": "caer",
    "cafetales": "cafetal",
    "cafetera": "cafetero",
    "cafeteras": "cafetero",
    "cafeteros": "cafetero",
    "cafeterías": "cafetería",
    "cafetines": "cafetín",
    "cafetos": "cafeto",
    "cafetuchos": "cafetucho",
    "cafeínas": "cafeína",
    "cafeísmos": "cafeísmo",
    "caficultora": "caficultor",
    "caficultoras": "caficultor",
    "caficultores": "caficultor",
    "cafres": "cafre",
    "caftanes": "caftán",
    "cafés": "café",
    "caga": "cagar",
    "cagaba": "cagar",
    "cagabais": "cagar",
    "cagaban": "cagar",
    "cagabas": "cagar",
    "cagad": "cagar",
    "cagada": "cagar",
    "cagadas": "cagar",
    "cagaderos": "cagadero",
    "cagado": "cagar",
    "cagados": "cagar",
    "cagafierros": "cagafierro",
    "cagajones": "cagajón",
    "cagaleras": "cagalera",
    "cagaletas": "cagaleta",
    "cagamos": "cagar",
    "cagan": "cagar",
    "cagando": "cagar",
    "caganidos": "caganido",
    "cagara": "cagar",
    "cagarais": "cagar",
    "cagaran": "cagar",
    "cagaras": "cagar",
    "cagare": "cagar",
    "cagareis": "cagar",
    "cagaremos": "cagar",
    "cagaren": "cagar",
    "cagares": "cagar",
    "cagaron": "cagar",
    "cagarrutas": "cagarruta",
    "cagará": "cagar",
    "cagarán": "cagar",
    "cagarás": "cagar",
    "cagaré": "cagar",
    "cagaréis": "cagar",
    "cagaría": "cagar",
    "cagaríais": "cagar",
    "cagaríamos": "cagar",
    "cagarían": "cagar",
    "cagarías": "cagar",
    "cagas": "cagar",
    "cagase": "cagar",
    "cagaseis": "cagar",
    "cagasen": "cagar",
    "cagases": "cagar",
    "cagaste": "cagar",
    "cagasteis": "cagar",
    "cagatorios": "cagatorio",
    "cago": "cagar",
    "cagona": "cagón",
    "cagonas": "cagón",
    "cagones": "cagón",
    "caguamas": "caguama",
    "cague": "cagar",
    "caguemos": "cagar",
    "caguen": "cagar",
    "cagues": "cagar",
    "caguetas": "cagueta",
    "cagué": "cagar",
    "caguéis": "cagar",
    "cagábamos": "cagar",
    "cagáis": "cagar",
    "cagáramos": "cagar",
    "cagáremos": "cagar",
    "cagásemos": "cagar",
    "cagó": "cagar",
    "cahizadas": "cahizada",
    "cahíces": "cahíz",
    "caiga": "caer",
    "caigamos": "caer",
    "caigan": "caer",
    "caigas": "caer",
    "caigo": "caer",
    "caigáis": "caer",
    "cailones": "cailón",
    "caimacanes": "caimacán",
    "caimanes": "caimán",
    "caimitos": "caimito",
    "caimánes": "caimán",
    "cainitas": "cainita",
    "caireles": "cairel",
    "cairotas": "cairota",
    "cajas": "caja",
    "cajea": "cajear",
    "cajeaba": "cajear",
    "cajeabais": "cajear",
    "cajeaban": "cajear",
    "cajeabas": "cajear",
    "cajead": "cajear",
    "cajeada": "cajear",
    "cajeadas": "cajear",
    "cajeado": "cajear",
    "cajeados": "cajear",
    "cajeamos": "cajear",
    "cajean": "cajear",
    "cajeando": "cajear",
    "cajeara": "cajear",
    "cajearais": "cajear",
    "cajearan": "cajear",
    "cajearas": "cajear",
    "cajeare": "cajear",
    "cajeareis": "cajear",
    "cajearemos": "cajear",
    "cajearen": "cajear",
    "cajeares": "cajear",
    "cajearon": "cajear",
    "cajeará": "cajear",
    "cajearán": "cajear",
    "cajearás": "cajear",
    "cajearé": "cajear",
    "cajearéis": "cajear",
    "cajearía": "cajear",
    "cajearíais": "cajear",
    "cajearíamos": "cajear",
    "cajearían": "cajear",
    "cajearías": "cajear",
    "cajeas": "cajear",
    "cajease": "cajear",
    "cajeaseis": "cajear",
    "cajeasen": "cajear",
    "cajeases": "cajear",
    "cajeaste": "cajear",
    "cajeasteis": "cajear",
    "cajee": "cajear",
    "cajeemos": "cajear",
    "cajeen": "cajear",
    "cajees": "cajear",
    "cajeo": "cajear",
    "cajera": "cajero",
    "cajeras": "cajero",
    "cajeros": "cajero",
    "cajerías": "cajería",
    "cajetillas": "cajetilla",
    "cajetines": "cajetín",
    "cajeábamos": "cajear",
    "cajeáis": "cajear",
    "cajeáramos": "cajear",
    "cajeáremos": "cajear",
    "cajeásemos": "cajear",
    "cajeé": "cajear",
    "cajeéis": "cajear",
    "cajeó": "cajear",
    "cajistas": "cajista",
    "cajitas": "cajita",
    "cajonadas": "cajonada",
    "cajoneras": "cajonera",
    "cajonerías": "cajonería",
    "cajones": "cajón",
    "cajos": "cajo",
    "cal": "caloría",
    "cala": "calar",
    "calaba": "calar",
    "calabacea": "calabacear",
    "calabaceaba": "calabacear",
    "calabaceabais": "calabacear",
    "calabaceaban": "calabacear",
    "calabaceabas": "calabacear",
    "calabacead": "calabacear",
    "calabaceada": "calabacear",
    "calabaceadas": "calabacear",
    "calabaceado": "calabacear",
    "calabaceados": "calabacear",
    "calabaceamos": "calabacear",
    "calabacean": "calabacear",
    "calabaceando": "calabacear",
    "calabaceara": "calabacear",
    "calabacearais": "calabacear",
    "calabacearan": "calabacear",
    "calabacearas": "calabacear",
    "calabaceare": "calabacear",
    "calabaceareis": "calabacear",
    "calabacearemos": "calabacear",
    "calabacearen": "calabacear",
    "calabaceares": "calabacear",
    "calabacearon": "calabacear",
    "calabaceará": "calabacear",
    "calabacearán": "calabacear",
    "calabacearás": "calabacear",
    "calabacearé": "calabacear",
    "calabacearéis": "calabacear",
    "calabacearía": "calabacear",
    "calabacearíais": "calabacear",
    "calabacearíamos": "calabacear",
    "calabacearían": "calabacear",
    "calabacearías": "calabacear",
    "calabaceas": "calabacear",
    "calabacease": "calabacear",
    "calabaceaseis": "calabacear",
    "calabaceasen": "calabacear",
    "calabaceases": "calabacear",
    "calabaceaste": "calabacear",
    "calabaceasteis": "calabacear",
    "calabacee": "calabacear",
    "calabaceemos": "calabacear",
    "calabaceen": "calabacear",
    "calabacees": "calabacear",
    "calabaceo": "calabacear",
    "calabacera": "calabacero",
    "calabaceras": "calabacero",
    "calabaceros": "calabacero",
    "calabaceábamos": "calabacear",
    "calabaceáis": "calabacear",
    "calabaceáramos": "calabacear",
    "calabaceáremos": "calabacear",
    "calabaceásemos": "calabacear",
    "calabaceé": "calabacear",
    "calabaceéis": "calabacear",
    "calabaceó": "calabacear",
    "calabacines": "calabacín",
    "calabacinos": "calabacino",
    "calabais": "calar",
    "calaban": "calar",
    "calabas": "calar",
    "calabazadas": "calabazada",
    "calabazares": "calabazar",
    "calabazas": "calabaza",
    "calabazates": "calabazate",
    "calabazazos": "calabazazo",
    "calabozos": "calabozo",
    "calabresa": "calabrés",
    "calabresas": "calabrés",
    "calabreses": "calabrés",
    "calabriadas": "calabriada",
    "calabrotes": "calabrote",
    "calacuerdas": "calacuerda",
    "calad": "calar",
    "calada": "calar",
    "caladas": "calar",
    "caladeros": "caladero",
    "calado": "calar",
    "caladora": "calador",
    "caladoras": "calador",
    "caladores": "calador",
    "calados": "calar",
    "caladuras": "caladura",
    "calafatea": "calafatear",
    "calafateaba": "calafatear",
    "calafateabais": "calafatear",
    "calafateaban": "calafatear",
    "calafateabas": "calafatear",
    "calafatead": "calafatear",
    "calafateada": "calafatear",
    "calafateadas": "calafatear",
    "calafateado": "calafatear",
    "calafateadora": "calafateador",
    "calafateadoras": "calafateador",
    "calafateadores": "calafateador",
    "calafateados": "calafatear",
    "calafateaduras": "calafateadura",
    "calafateamos": "calafatear",
    "calafatean": "calafatear",
    "calafateando": "calafatear",
    "calafateara": "calafatear",
    "calafatearais": "calafatear",
    "calafatearan": "calafatear",
    "calafatearas": "calafatear",
    "calafateare": "calafatear",
    "calafateareis": "calafatear",
    "calafatearemos": "calafatear",
    "calafatearen": "calafatear",
    "calafateares": "calafatear",
    "calafatearon": "calafatear",
    "calafateará": "calafatear",
    "calafatearán": "calafatear",
    "calafatearás": "calafatear",
    "calafatearé": "calafatear",
    "calafatearéis": "calafatear",
    "calafatearía": "calafatear",
    "calafatearíais": "calafatear",
    "calafatearíamos": "calafatear",
    "calafatearían": "calafatear",
    "calafatearías": "calafatear",
    "calafateas": "calafatear",
    "calafatease": "calafatear",
    "calafateaseis": "calafatear",
    "calafateasen": "calafatear",
    "calafateases": "calafatear",
    "calafateaste": "calafatear",
    "calafateasteis": "calafatear",
    "calafatee": "calafatear",
    "calafateemos": "calafatear",
    "calafateen": "calafatear",
    "calafatees": "calafatear",
    "calafateo": "calafatear",
    "calafateos": "calafateo",
    "calafaterías": "calafatería",
    "calafates": "calafate",
    "calafateábamos": "calafatear",
    "calafateáis": "calafatear",
    "calafateáramos": "calafatear",
    "calafateáremos": "calafatear",
    "calafateásemos": "calafatear",
    "calafateé": "calafatear",
    "calafateéis": "calafatear",
    "calafateó": "calafatear",
    "calafatines": "calafatín",
    "calagrañas": "calagraña",
    "calagurritana": "calagurritano",
    "calagurritanas": "calagurritano",
    "calagurritanos": "calagurritano",
    "calaitas": "calaita",
    "calamares": "calamar",
    "calambres": "calambre",
    "calambreñas": "calambreña",
    "calambures": "calambur",
    "calamentos": "calamento",
    "calamidades": "calamidad",
    "calamiformes": "calamiforme",
    "calamilleras": "calamillera",
    "calaminas": "calamina",
    "calamintas": "calaminta",
    "calamitas": "calamita",
    "calamites": "calamite",
    "calamitosa": "calamitoso",
    "calamitosas": "calamitoso",
    "calamitosos": "calamitoso",
    "calamocana": "calamocano",
    "calamocanas": "calamocano",
    "calamocanos": "calamocano",
    "calamones": "calamón",
    "calamorradas": "calamorrada",
    "calamorras": "calamorra",
    "calamorrazos": "calamorrazo",
    "calamos": "calar",
    "calan": "calar",
    "calando": "calar",
    "calandra": "calandrar",
    "calandraba": "calandrar",
    "calandrabais": "calandrar",
    "calandraban": "calandrar",
    "calandrabas": "calandrar",
    "calandrad": "calandrar",
    "calandrada": "calandrar",
    "calandradas": "calandrar",
    "calandrado": "calandrar",
    "calandrados": "calandrar",
    "calandrajos": "calandrajo",
    "calandramos": "calandrar",
    "calandran": "calandrar",
    "calandrando": "calandrar",
    "calandrara": "calandrar",
    "calandrarais": "calandrar",
    "calandraran": "calandrar",
    "calandraras": "calandrar",
    "calandrare": "calandrar",
    "calandrareis": "calandrar",
    "calandraremos": "calandrar",
    "calandraren": "calandrar",
    "calandrares": "calandrar",
    "calandraron": "calandrar",
    "calandrará": "calandrar",
    "calandrarán": "calandrar",
    "calandrarás": "calandrar",
    "calandraré": "calandrar",
    "calandraréis": "calandrar",
    "calandraría": "calandrar",
    "calandraríais": "calandrar",
    "calandraríamos": "calandrar",
    "calandrarían": "calandrar",
    "calandrarías": "calandrar",
    "calandras": "calandrar",
    "calandrase": "calandrar",
    "calandraseis": "calandrar",
    "calandrasen": "calandrar",
    "calandrases": "calandrar",
    "calandraste": "calandrar",
    "calandrasteis": "calandrar",
    "calandre": "calandrar",
    "calandremos": "calandrar",
    "calandren": "calandrar",
    "calandres": "calandrar",
    "calandrias": "calandria",
    "calandro": "calandrar",
    "calandrábamos": "calandrar",
    "calandráis": "calandrar",
    "calandráramos": "calandrar",
    "calandráremos": "calandrar",
    "calandrásemos": "calandrar",
    "calandré": "calandrar",
    "calandréis": "calandrar",
    "calandró": "calandrar",
    "calapiés": "calapié",
    "calara": "calar",
    "calarais": "calar",
    "calaran": "calar",
    "calaras": "calar",
    "calare": "calar",
    "calareis": "calar",
    "calaremos": "calar",
    "calaren": "calar",
    "calares": "calar",
    "calaron": "calar",
    "calará": "calar",
    "calarán": "calar",
    "calarás": "calar",
    "calaré": "calar",
    "calaréis": "calar",
    "calaría": "calar",
    "calaríais": "calar",
    "calaríamos": "calar",
    "calarían": "calar",
    "calarías": "calar",
    "calas": "calar",
    "calasancia": "calasancio",
    "calasancias": "calasancio",
    "calasancios": "calasancio",
    "calase": "calar",
    "calaseis": "calar",
    "calasen": "calar",
    "calases": "calar",
    "calaste": "calar",
    "calasteis": "calar",
    "calata": "cálato",
    "calatas": "cálato",
    "calatos": "cálato",
    "calatrava": "calatravo",
    "calatravas": "calatravo",
    "calatravos": "calatravo",
    "calaveradas": "calaverada",
    "calaveras": "calavera",
    "calañas": "calaña",
    "calca": "calcar",
    "calcaba": "calcar",
    "calcabais": "calcar",
    "calcaban": "calcar",
    "calcabas": "calcar",
    "calcad": "calcar",
    "calcada": "calcar",
    "calcadas": "calcar",
    "calcado": "calcar",
    "calcadora": "calcador",
    "calcadoras": "calcador",
    "calcadores": "calcador",
    "calcados": "calcar",
    "calcamos": "calcar",
    "calcan": "calcar",
    "calcando": "calcar",
    "calcaneocuboidea": "calcaneocuboideo",
    "calcaneocuboideas": "calcaneocuboideo",
    "calcaneocuboideos": "calcaneocuboideo",
    "calcaneonaviculares": "calcaneonavicular",
    "calcantitas": "calcantita",
    "calcara": "calcar",
    "calcarais": "calcar",
    "calcaran": "calcar",
    "calcaras": "calcar",
    "calcare": "calcar",
    "calcareis": "calcar",
    "calcaremos": "calcar",
    "calcaren": "calcar",
    "calcares": "calcar",
    "calcarina": "calcarino",
    "calcarinas": "calcarino",
    "calcarinos": "calcarino",
    "calcaron": "calcar",
    "calcará": "calcar",
    "calcarán": "calcar",
    "calcarás": "calcar",
    "calcaré": "calcar",
    "calcaréis": "calcar",
    "calcaría": "calcar",
    "calcaríais": "calcar",
    "calcaríamos": "calcar",
    "calcarían": "calcar",
    "calcarías": "calcar",
    "calcas": "calcar",
    "calcase": "calcar",
    "calcaseis": "calcar",
    "calcasen": "calcar",
    "calcases": "calcar",
    "calcaste": "calcar",
    "calcasteis": "calcar",
    "calcañales": "calcañal",
    "calcañares": "calcañar",
    "calce": "calzar",
    "calcedonias": "calcedonia",
    "calcemos": "calzar",
    "calcen": "calzar",
    "calceolarias": "calceolaria",
    "calces": "calzar",
    "calceses": "calcés",
    "calceta": "calcetar",
    "calcetaba": "calcetar",
    "calcetabais": "calcetar",
    "calcetaban": "calcetar",
    "calcetabas": "calcetar",
    "calcetad": "calcetar",
    "calcetada": "calcetar",
    "calcetadas": "calcetar",
    "calcetado": "calcetar",
    "calcetados": "calcetar",
    "calcetamos": "calcetar",
    "calcetan": "calcetar",
    "calcetando": "calcetar",
    "calcetara": "calcetar",
    "calcetarais": "calcetar",
    "calcetaran": "calcetar",
    "calcetaras": "calcetar",
    "calcetare": "calcetar",
    "calcetareis": "calcetar",
    "calcetaremos": "calcetar",
    "calcetaren": "calcetar",
    "calcetares": "calcetar",
    "calcetaron": "calcetar",
    "calcetará": "calcetar",
    "calcetarán": "calcetar",
    "calcetarás": "calcetar",
    "calcetaré": "calcetar",
    "calcetaréis": "calcetar",
    "calcetaría": "calcetar",
    "calcetaríais": "calcetar",
    "calcetaríamos": "calcetar",
    "calcetarían": "calcetar",
    "calcetarías": "calcetar",
    "calcetas": "calcetar",
    "calcetase": "calcetar",
    "calcetaseis": "calcetar",
    "calcetasen": "calcetar",
    "calcetases": "calcetar",
    "calcetaste": "calcetar",
    "calcetasteis": "calcetar",
    "calcete": "calcetar",
    "calcetemos": "calcetar",
    "calceten": "calcetar",
    "calceterías": "calcetería",
    "calcetes": "calcetar",
    "calcetines": "calcetín",
    "calceto": "calcetar",
    "calcetábamos": "calcetar",
    "calcetáis": "calcetar",
    "calcetáramos": "calcetar",
    "calcetáremos": "calcetar",
    "calcetásemos": "calcetar",
    "calceté": "calcetar",
    "calcetéis": "calcetar",
    "calcetó": "calcetar",
    "calchaquíes": "calchaquí",
    "calchaquís": "calchaquí",
    "calcifica": "calcificar",
    "calcificaba": "calcificar",
    "calcificabais": "calcificar",
    "calcificaban": "calcificar",
    "calcificabas": "calcificar",
    "calcificaciones": "calcificación",
    "calcificad": "calcificar",
    "calcificada": "calcificar",
    "calcificadas": "calcificar",
    "calcificado": "calcificar",
    "calcificados": "calcificar",
    "calcificamos": "calcificar",
    "calcifican": "calcificar",
    "calcificando": "calcificar",
    "calcificara": "calcificar",
    "calcificarais": "calcificar",
    "calcificaran": "calcificar",
    "calcificaras": "calcificar",
    "calcificare": "calcificar",
    "calcificareis": "calcificar",
    "calcificaremos": "calcificar",
    "calcificaren": "calcificar",
    "calcificares": "calcificar",
    "calcificaron": "calcificar",
    "calcificará": "calcificar",
    "calcificarán": "calcificar",
    "calcificarás": "calcificar",
    "calcificaré": "calcificar",
    "calcificaréis": "calcificar",
    "calcificaría": "calcificar",
    "calcificaríais": "calcificar",
    "calcificaríamos": "calcificar",
    "calcificarían": "calcificar",
    "calcificarías": "calcificar",
    "calcificas": "calcificar",
    "calcificase": "calcificar",
    "calcificaseis": "calcificar",
    "calcificasen": "calcificar",
    "calcificases": "calcificar",
    "calcificaste": "calcificar",
    "calcificasteis": "calcificar",
    "calcifico": "calcificar",
    "calcificábamos": "calcificar",
    "calcificáis": "calcificar",
    "calcificáramos": "calcificar",
    "calcificáremos": "calcificar",
    "calcificásemos": "calcificar",
    "calcificó": "calcificar",
    "calcifique": "calcificar",
    "calcifiquemos": "calcificar",
    "calcifiquen": "calcificar",
    "calcifiques": "calcificar",
    "calcifiqué": "calcificar",
    "calcifiquéis": "calcificar",
    "calcimagnésica": "calcimagnésico",
    "calcimagnésicas": "calcimagnésico",
    "calcimagnésicos": "calcimagnésico",
    "calcina": "calcinar",
    "calcinaba": "calcinar",
    "calcinabais": "calcinar",
    "calcinaban": "calcinar",
    "calcinabas": "calcinar",
    "calcinaciones": "calcinación",
    "calcinad": "calcinar",
    "calcinada": "calcinar",
    "calcinadas": "calcinar",
    "calcinado": "calcinar",
    "calcinadora": "calcinador",
    "calcinadoras": "calcinador",
    "calcinadores": "calcinador",
    "calcinados": "calcinar",
    "calcinamientos": "calcinamiento",
    "calcinamos": "calcinar",
    "calcinan": "calcinar",
    "calcinando": "calcinar",
    "calcinara": "calcinar",
    "calcinarais": "calcinar",
    "calcinaran": "calcinar",
    "calcinaras": "calcinar",
    "calcinare": "calcinar",
    "calcinareis": "calcinar",
    "calcinaremos": "calcinar",
    "calcinaren": "calcinar",
    "calcinares": "calcinar",
    "calcinaron": "calcinar",
    "calcinará": "calcinar",
    "calcinarán": "calcinar",
    "calcinarás": "calcinar",
    "calcinaré": "calcinar",
    "calcinaréis": "calcinar",
    "calcinaría": "calcinar",
    "calcinaríais": "calcinar",
    "calcinaríamos": "calcinar",
    "calcinarían": "calcinar",
    "calcinarías": "calcinar",
    "calcinas": "calcinar",
    "calcinase": "calcinar",
    "calcinaseis": "calcinar",
    "calcinasen": "calcinar",
    "calcinases": "calcinar",
    "calcinaste": "calcinar",
    "calcinasteis": "calcinar",
    "calcinatorios": "calcinatorio",
    "calcine": "calcinar",
    "calcinemos": "calcinar",
    "calcinen": "calcinar",
    "calcines": "calcinar",
    "calcino": "calcinar",
    "calcinábamos": "calcinar",
    "calcináis": "calcinar",
    "calcináramos": "calcinar",
    "calcináremos": "calcinar",
    "calcinásemos": "calcinar",
    "calciné": "calcinar",
    "calcinéis": "calcinar",
    "calcinó": "calcinar",
    "calcios": "calcio",
    "calcitas": "calcita",
    "calco": "calcar",
    "calcocitas": "calcocita",
    "calcogenuros": "calcogenuro",
    "calcografías": "calcografía",
    "calcográfica": "calcográfico",
    "calcográficas": "calcográfico",
    "calcográficos": "calcográfico",
    "calcolíticos": "calcolítico",
    "calcomanías": "calcomanía",
    "calcopiritas": "calcopirita",
    "calcos": "calco",
    "calcosinas": "calcosina",
    "calcosquistos": "calcosquisto",
    "calcostibitas": "calcostibita",
    "calcotipias": "calcotipia",
    "calcotriquitas": "calcotriquita",
    "calcula": "calcular",
    "calculaba": "calcular",
    "calculabais": "calcular",
    "calculaban": "calcular",
    "calculabas": "calcular",
    "calculables": "calculable",
    "calculad": "calcular",
    "calculada": "calcular",
    "calculadas": "calcular",
    "calculado": "calcular",
    "calculadora": "calculador",
    "calculadoras": "calculadora",
    "calculadores": "calculador",
    "calculados": "calcular",
    "calculamos": "calcular",
    "calculan": "calcular",
    "calculando": "calcular",
    "calculara": "calcular",
    "calcularais": "calcular",
    "calcularan": "calcular",
    "calcularas": "calcular",
    "calculare": "calcular",
    "calculareis": "calcular",
    "calcularemos": "calcular",
    "calcularen": "calcular",
    "calculares": "calcular",
    "calcularon": "calcular",
    "calculará": "calcular",
    "calcularán": "calcular",
    "calcularás": "calcular",
    "calcularé": "calcular",
    "calcularéis": "calcular",
    "calcularía": "calcular",
    "calcularíais": "calcular",
    "calcularíamos": "calcular",
    "calcularían": "calcular",
    "calcularías": "calcular",
    "calculas": "calcular",
    "calculase": "calcular",
    "calculaseis": "calcular",
    "calculasen": "calcular",
    "calculases": "calcular",
    "calculaste": "calcular",
    "calculasteis": "calcular",
    "calcule": "calcular",
    "calculemos": "calcular",
    "calculen": "calcular",
    "calcules": "calcular",
    "calculistas": "calculista",
    "calculo": "calcular",
    "calculosa": "calculoso",
    "calculosas": "calculoso",
    "calculosos": "calculoso",
    "calculábamos": "calcular",
    "calculáis": "calcular",
    "calculáramos": "calcular",
    "calculáremos": "calcular",
    "calculásemos": "calcular",
    "calculé": "calcular",
    "calculéis": "calcular",
    "calculó": "calcular",
    "calcábamos": "calcar",
    "calcáis": "calcar",
    "calcánea": "calcáneo",
    "calcáneas": "calcáneo",
    "calcáneos": "calcáneo",
    "calcáramos": "calcar",
    "calcárea": "calcáreo",
    "calcáreas": "calcáreo",
    "calcáremos": "calcar",
    "calcáreos": "calcáreo",
    "calcásemos": "calcar",
    "calcé": "calzar",
    "calcéis": "calzar",
    "calcícolas": "calcícola",
    "calcídicos": "calcídico",
    "calcífuga": "calcífugo",
    "calcífugas": "calcífugo",
    "calcífugos": "calcífugo",
    "calcímetros": "calcímetro",
    "calcó": "calcar",
    "calcófila": "calcófilo",
    "calcófilas": "calcófilo",
    "calcófilos": "calcófilo",
    "calcógrafa": "calcógrafo",
    "calcógrafas": "calcógrafo",
    "calcógrafos": "calcógrafo",
    "caldarios": "caldario",
    "caldas": "calda",
    "caldea": "caldear",
    "caldeaba": "caldear",
    "caldeabais": "caldear",
    "caldeaban": "caldear",
    "caldeabas": "caldear",
    "caldead": "caldear",
    "caldeada": "caldear",
    "caldeadas": "caldear",
    "caldeado": "caldear",
    "caldeados": "caldear",
    "caldeamientos": "caldeamiento",
    "caldeamos": "caldear",
    "caldean": "caldear",
    "caldeando": "caldear",
    "caldeara": "caldear",
    "caldearais": "caldear",
    "caldearan": "caldear",
    "caldearas": "caldear",
    "caldeare": "caldear",
    "caldeareis": "caldear",
    "caldearemos": "caldear",
    "caldearen": "caldear",
    "caldeares": "caldear",
    "caldearon": "caldear",
    "caldeará": "caldear",
    "caldearán": "caldear",
    "caldearás": "caldear",
    "caldearé": "caldear",
    "caldearéis": "caldear",
    "caldearía": "caldear",
    "caldearíais": "caldear",
    "caldearíamos": "caldear",
    "caldearían": "caldear",
    "caldearías": "caldear",
    "caldeas": "caldear",
    "caldease": "caldear",
    "caldeaseis": "caldear",
    "caldeasen": "caldear",
    "caldeases": "caldear",
    "caldeaste": "caldear",
    "caldeasteis": "caldear",
    "caldee": "caldear",
    "caldeemos": "caldear",
    "caldeen": "caldear",
    "caldees": "caldear",
    "caldeiradas": "caldeirada",
    "caldeo": "caldear",
    "caldeos": "caldeo",
    "calderadas": "calderada",
    "calderas": "caldera",
    "calderera": "calderero",
    "caldereras": "calderero",
    "caldereros": "calderero",
    "caldererías": "calderería",
    "calderetas": "caldereta",
    "calderillas": "calderilla",
    "calderones": "calderón",
    "calderos": "caldero",
    "caldeábamos": "caldear",
    "caldeáis": "caldear",
    "caldeáramos": "caldear",
    "caldeáremos": "caldear",
    "caldeásemos": "caldear",
    "caldeé": "caldear",
    "caldeéis": "caldear",
    "caldeó": "caldear",
    "calditos": "caldito",
    "caldos": "caldo",
    "caldosa": "caldoso",
    "caldosas": "caldoso",
    "caldosos": "caldoso",
    "calduchos": "calducho",
    "cale": "calar",
    "calece": "calecer",
    "caleced": "calecer",
    "calecemos": "calecer",
    "calecen": "calecer",
    "caleceremos": "calecer",
    "calecerá": "calecer",
    "calecerán": "calecer",
    "calecerás": "calecer",
    "caleceré": "calecer",
    "caleceréis": "calecer",
    "calecería": "calecer",
    "caleceríais": "calecer",
    "caleceríamos": "calecer",
    "calecerían": "calecer",
    "calecerías": "calecer",
    "caleces": "calecer",
    "calecida": "calecer",
    "calecidas": "calecer",
    "calecido": "calecer",
    "calecidos": "calecer",
    "caleciendo": "calecer",
    "caleciera": "calecer",
    "calecierais": "calecer",
    "calecieran": "calecer",
    "calecieras": "calecer",
    "caleciere": "calecer",
    "caleciereis": "calecer",
    "calecieren": "calecer",
    "calecieres": "calecer",
    "calecieron": "calecer",
    "caleciese": "calecer",
    "calecieseis": "calecer",
    "caleciesen": "calecer",
    "calecieses": "calecer",
    "calecimos": "calecer",
    "caleciste": "calecer",
    "calecisteis": "calecer",
    "caleciéramos": "calecer",
    "caleciéremos": "calecer",
    "caleciésemos": "calecer",
    "caleció": "calecer",
    "calecéis": "calecer",
    "calecí": "calecer",
    "calecía": "calecer",
    "calecíais": "calecer",
    "calecíamos": "calecer",
    "calecían": "calecer",
    "calecías": "calecer",
    "calefacciona": "calefaccionar",
    "calefaccionaba": "calefaccionar",
    "calefaccionabais": "calefaccionar",
    "calefaccionaban": "calefaccionar",
    "calefaccionabas": "calefaccionar",
    "calefaccionad": "calefaccionar",
    "calefaccionada": "calefaccionar",
    "calefaccionadas": "calefaccionar",
    "calefaccionado": "calefaccionar",
    "calefaccionados": "calefaccionar",
    "calefaccionamos": "calefaccionar",
    "calefaccionan": "calefaccionar",
    "calefaccionando": "calefaccionar",
    "calefaccionara": "calefaccionar",
    "calefaccionarais": "calefaccionar",
    "calefaccionaran": "calefaccionar",
    "calefaccionaras": "calefaccionar",
    "calefaccionare": "calefaccionar",
    "calefaccionareis": "calefaccionar",
    "calefaccionaremos": "calefaccionar",
    "calefaccionaren": "calefaccionar",
    "calefaccionares": "calefaccionar",
    "calefaccionaron": "calefaccionar",
    "calefaccionará": "calefaccionar",
    "calefaccionarán": "calefaccionar",
    "calefaccionarás": "calefaccionar",
    "calefaccionaré": "calefaccionar",
    "calefaccionaréis": "calefaccionar",
    "calefaccionaría": "calefaccionar",
    "calefaccionaríais": "calefaccionar",
    "calefaccionaríamos": "calefaccionar",
    "calefaccionarían": "calefaccionar",
    "calefaccionarías": "calefaccionar",
    "calefaccionas": "calefaccionar",
    "calefaccionase": "calefaccionar",
    "calefaccionaseis": "calefaccionar",
    "calefaccionasen": "calefaccionar",
    "calefaccionases": "calefaccionar",
    "calefaccionaste": "calefaccionar",
    "calefaccionasteis": "calefaccionar",
    "calefaccione": "calefaccionar",
    "calefaccionemos": "calefaccionar",
    "calefaccionen": "calefaccionar",
    "calefacciones": "calefaccionar",
    "calefacciono": "calefaccionar",
    "calefaccionábamos": "calefaccionar",
    "calefaccionáis": "calefaccionar",
    "calefaccionáramos": "calefaccionar",
    "calefaccionáremos": "calefaccionar",
    "calefaccionásemos": "calefaccionar",
    "calefaccioné": "calefaccionar",
    "calefaccionéis": "calefaccionar",
    "calefaccionó": "calefaccionar",
    "calefactora": "calefactor",
    "calefactoras": "calefactor",
    "calefactores": "calefactor",
    "calefactorios": "calefactorio",
    "calefones": "calefón",
    "caleidoscopios": "caleidoscopio",
    "caleidoscópica": "caleidoscópico",
    "caleidoscópicas": "caleidoscópico",
    "caleidoscópicos": "caleidoscópico",
    "calemos": "calar",
    "calen": "calar",
    "calenda": "calendar",
    "calendaba": "calendar",
    "calendabais": "calendar",
    "calendaban": "calendar",
    "calendabas": "calendar",
    "calendad": "calendar",
    "calendada": "calendar",
    "calendadas": "calendar",
    "calendado": "calendar",
    "calendados": "calendar",
    "calendamos": "calendar",
    "calendan": "calendar",
    "calendando": "calendar",
    "calendara": "calendar",
    "calendarais": "calendar",
    "calendaran": "calendar",
    "calendaras": "calendar",
    "calendare": "calendar",
    "calendareis": "calendar",
    "calendaremos": "calendar",
    "calendaren": "calendar",
    "calendares": "calendar",
    "calendarios": "calendario",
    "calendaron": "calendar",
    "calendará": "calendar",
    "calendarán": "calendar",
    "calendarás": "calendar",
    "calendaré": "calendar",
    "calendaréis": "calendar",
    "calendaría": "calendar",
    "calendaríais": "calendar",
    "calendaríamos": "calendar",
    "calendarían": "calendar",
    "calendarías": "calendar",
    "calendas": "calendar",
    "calendase": "calendar",
    "calendaseis": "calendar",
    "calendasen": "calendar",
    "calendases": "calendar",
    "calendaste": "calendar",
    "calendasteis": "calendar",
    "calende": "calendar",
    "calendemos": "calendar",
    "calenden": "calendar",
    "calendes": "calendar",
    "calendo": "calendar",
    "calendábamos": "calendar",
    "calendáis": "calendar",
    "calendáramos": "calendar",
    "calendáremos": "calendar",
    "calendásemos": "calendar",
    "calendé": "calendar",
    "calendéis": "calendar",
    "calendó": "calendar",
    "calentaba": "calentar",
    "calentabais": "calentar",
    "calentaban": "calentar",
    "calentabas": "calentar",
    "calentad": "calentar",
    "calentada": "calentar",
    "calentadas": "calentar",
    "calentado": "calentar",
    "calentadora": "calentador",
    "calentadoras": "calentador",
    "calentadores": "calentador",
    "calentados": "calentar",
    "calentamientos": "calentamiento",
    "calentamos": "calentar",
    "calentando": "calentar",
    "calentara": "calentar",
    "calentarais": "calentar",
    "calentaran": "calentar",
    "calentaras": "calentar",
    "calentare": "calentar",
    "calentareis": "calentar",
    "calentaremos": "calentar",
    "calentaren": "calentar",
    "calentares": "calentar",
    "calentaron": "calentar",
    "calentará": "calentar",
    "calentarán": "calentar",
    "calentarás": "calentar",
    "calentaré": "calentar",
    "calentaréis": "calentar",
    "calentaría": "calentar",
    "calentaríais": "calentar",
    "calentaríamos": "calentar",
    "calentarían": "calentar",
    "calentarías": "calentar",
    "calentase": "calentar",
    "calentaseis": "calentar",
    "calentasen": "calentar",
    "calentases": "calentar",
    "calentaste": "calentar",
    "calentasteis": "calentar",
    "calentemos": "calentar",
    "calentita": "calentito",
    "calentitas": "calentito",
    "calentitos": "calentito",
    "calentona": "calentón",
    "calentonas": "calentón",
    "calentones": "calentón",
    "calenturas": "calentura",
    "calenturienta": "calenturiento",
    "calenturientas": "calenturiento",
    "calenturientos": "calenturiento",
    "calenturosa": "calenturoso",
    "calenturosas": "calenturoso",
    "calenturosos": "calenturoso",
    "calentábamos": "calentar",
    "calentáis": "calentar",
    "calentáramos": "calentar",
    "calentáremos": "calentar",
    "calentásemos": "calentar",
    "calenté": "calentar",
    "calentéis": "calentar",
    "calentó": "calentar",
    "calera": "calero",
    "caleras": "calero",
    "caleros": "calero",
    "cales": "calar",
    "calesas": "calesa",
    "calesera": "calesero",
    "caleseras": "calesero",
    "caleseros": "calesero",
    "caleses": "calés",
    "calesinera": "calesinero",
    "calesineras": "calesinero",
    "calesineros": "calesinero",
    "calesines": "calesín",
    "calesitas": "calesita",
    "caletas": "caleta",
    "caletres": "caletre",
    "calezca": "calecer",
    "calezcamos": "calecer",
    "calezcan": "calecer",
    "calezcas": "calecer",
    "calezco": "calecer",
    "calezcáis": "calecer",
    "caleña": "caleño",
    "caleñas": "caleño",
    "caleños": "caleño",
    "calibra": "calibrar",
    "calibraba": "calibrar",
    "calibrabais": "calibrar",
    "calibraban": "calibrar",
    "calibrabas": "calibrar",
    "calibrables": "calibrable",
    "calibraciones": "calibración",
    "calibrad": "calibrar",
    "calibrada": "calibrar",
    "calibradas": "calibrar",
    "calibrado": "calibrar",
    "calibradora": "calibrador",
    "calibradoras": "calibrador",
    "calibradores": "calibrador",
    "calibrados": "calibrar",
    "calibramos": "calibrar",
    "calibran": "calibrar",
    "calibrando": "calibrar",
    "calibrara": "calibrar",
    "calibrarais": "calibrar",
    "calibraran": "calibrar",
    "calibraras": "calibrar",
    "calibrare": "calibrar",
    "calibrareis": "calibrar",
    "calibraremos": "calibrar",
    "calibraren": "calibrar",
    "calibrares": "calibrar",
    "calibraron": "calibrar",
    "calibrará": "calibrar",
    "calibrarán": "calibrar",
    "calibrarás": "calibrar",
    "calibraré": "calibrar",
    "calibraréis": "calibrar",
    "calibraría": "calibrar",
    "calibraríais": "calibrar",
    "calibraríamos": "calibrar",
    "calibrarían": "calibrar",
    "calibrarías": "calibrar",
    "calibras": "calibrar",
    "calibrase": "calibrar",
    "calibraseis": "calibrar",
    "calibrasen": "calibrar",
    "calibrases": "calibrar",
    "calibraste": "calibrar",
    "calibrasteis": "calibrar",
    "calibre": "calibrar",
    "calibremos": "calibrar",
    "calibren": "calibrar",
    "calibres": "calibre",
    "calibro": "calibrar",
    "calibrábamos": "calibrar",
    "calibráis": "calibrar",
    "calibráramos": "calibrar",
    "calibráremos": "calibrar",
    "calibrásemos": "calibrar",
    "calibré": "calibrar",
    "calibréis": "calibrar",
    "calibró": "calibrar",
    "calicantos": "calicanto",
    "calicantáceas": "calicantácea",
    "calicatas": "calicata",
    "calicheras": "calichera",
    "caliches": "caliche",
    "calicifloras": "caliciflora",
    "caliciformes": "caliciforme",
    "calicreínas": "calicreína",
    "caliculares": "calicular",
    "calidades": "calidad",
    "calideces": "calidez",
    "calidinas": "calidina",
    "calidoscopios": "calidoscopio",
    "calidoscópica": "calidoscópico",
    "calidoscópicas": "calidoscópico",
    "calidoscópicos": "calidoscópico",
    "calienta": "calentar",
    "calientan": "calentar",
    "calientas": "calentar",
    "caliente": "calentar",
    "calienten": "calentar",
    "calientes": "caliente",
    "caliento": "calentar",
    "califales": "califal",
    "califas": "califa",
    "califatos": "califato",
    "califica": "calificar",
    "calificaba": "calificar",
    "calificabais": "calificar",
    "calificaban": "calificar",
    "calificabas": "calificar",
    "calificables": "calificable",
    "calificaciones": "calificación",
    "calificad": "calificar",
    "calificada": "calificar",
    "calificadas": "calificar",
    "calificado": "calificar",
    "calificadora": "calificador",
    "calificadoras": "calificador",
    "calificadores": "calificador",
    "calificados": "calificar",
    "calificamos": "calificar",
    "califican": "calificar",
    "calificando": "calificar",
    "calificara": "calificar",
    "calificarais": "calificar",
    "calificaran": "calificar",
    "calificaras": "calificar",
    "calificare": "calificar",
    "calificareis": "calificar",
    "calificaremos": "calificar",
    "calificaren": "calificar",
    "calificares": "calificar",
    "calificaron": "calificar",
    "calificará": "calificar",
    "calificarán": "calificar",
    "calificarás": "calificar",
    "calificaré": "calificar",
    "calificaréis": "calificar",
    "calificaría": "calificar",
    "calificaríais": "calificar",
    "calificaríamos": "calificar",
    "calificarían": "calificar",
    "calificarías": "calificar",
    "calificas": "calificar",
    "calificase": "calificar",
    "calificaseis": "calificar",
    "calificasen": "calificar",
    "calificases": "calificar",
    "calificaste": "calificar",
    "calificasteis": "calificar",
    "calificativa": "calificativo",
    "calificativas": "calificativo",
    "calificativos": "calificativo",
    "califico": "calificar",
    "calificábamos": "calificar",
    "calificáis": "calificar",
    "calificáramos": "calificar",
    "calificáremos": "calificar",
    "calificásemos": "calificar",
    "calificó": "calificar",
    "califique": "calificar",
    "califiquemos": "calificar",
    "califiquen": "calificar",
    "califiques": "calificar",
    "califiqué": "calificar",
    "califiquéis": "calificar",
    "californiana": "californiano",
    "californianas": "californiano",
    "californianos": "californiano",
    "californios": "californio",
    "californitas": "californita",
    "caliginosa": "caliginoso",
    "caliginosas": "caliginoso",
    "caliginosos": "caliginoso",
    "caligrafiaba": "caligrafiar",
    "caligrafiabais": "caligrafiar",
    "caligrafiaban": "caligrafiar",
    "caligrafiabas": "caligrafiar",
    "caligrafiad": "caligrafiar",
    "caligrafiada": "caligrafiar",
    "caligrafiadas": "caligrafiar",
    "caligrafiado": "caligrafiar",
    "caligrafiados": "caligrafiar",
    "caligrafiamos": "caligrafiar",
    "caligrafiando": "caligrafiar",
    "caligrafiara": "caligrafiar",
    "caligrafiarais": "caligrafiar",
    "caligrafiaran": "caligrafiar",
    "caligrafiaras": "caligrafiar",
    "caligrafiare": "caligrafiar",
    "caligrafiareis": "caligrafiar",
    "caligrafiaremos": "caligrafiar",
    "caligrafiaren": "caligrafiar",
    "caligrafiares": "caligrafiar",
    "caligrafiaron": "caligrafiar",
    "caligrafiará": "caligrafiar",
    "caligrafiarán": "caligrafiar",
    "caligrafiarás": "caligrafiar",
    "caligrafiaré": "caligrafiar",
    "caligrafiaréis": "caligrafiar",
    "caligrafiaría": "caligrafiar",
    "caligrafiaríais": "caligrafiar",
    "caligrafiaríamos": "caligrafiar",
    "caligrafiarían": "caligrafiar",
    "caligrafiarías": "caligrafiar",
    "caligrafiase": "caligrafiar",
    "caligrafiaseis": "caligrafiar",
    "caligrafiasen": "caligrafiar",
    "caligrafiases": "caligrafiar",
    "caligrafiaste": "caligrafiar",
    "caligrafiasteis": "caligrafiar",
    "caligrafiemos": "caligrafiar",
    "caligrafiábamos": "caligrafiar",
    "caligrafiáis": "caligrafiar",
    "caligrafiáramos": "caligrafiar",
    "caligrafiáremos": "caligrafiar",
    "caligrafiásemos": "caligrafiar",
    "caligrafié": "caligrafiar",
    "caligrafiéis": "caligrafiar",
    "caligrafió": "caligrafiar",
    "caligrafía": "caligrafiar",
    "caligrafían": "caligrafiar",
    "caligrafías": "caligrafiar",
    "caligrafíe": "caligrafiar",
    "caligrafíen": "caligrafiar",
    "caligrafíes": "caligrafiar",
    "caligrafío": "caligrafiar",
    "caligramas": "caligrama",
    "caligráfica": "caligráfico",
    "caligráficas": "caligráfico",
    "caligráficos": "caligráfico",
    "calimas": "calima",
    "calimba": "calimbar",
    "calimbaba": "calimbar",
    "calimbabais": "calimbar",
    "calimbaban": "calimbar",
    "calimbabas": "calimbar",
    "calimbad": "calimbar",
    "calimbada": "calimbar",
    "calimbadas": "calimbar",
    "calimbado": "calimbar",
    "calimbados": "calimbar",
    "calimbamos": "calimbar",
    "calimban": "calimbar",
    "calimbando": "calimbar",
    "calimbara": "calimbar",
    "calimbarais": "calimbar",
    "calimbaran": "calimbar",
    "calimbaras": "calimbar",
    "calimbare": "calimbar",
    "calimbareis": "calimbar",
    "calimbaremos": "calimbar",
    "calimbaren": "calimbar",
    "calimbares": "calimbar",
    "calimbaron": "calimbar",
    "calimbará": "calimbar",
    "calimbarán": "calimbar",
    "calimbarás": "calimbar",
    "calimbaré": "calimbar",
    "calimbaréis": "calimbar",
    "calimbaría": "calimbar",
    "calimbaríais": "calimbar",
    "calimbaríamos": "calimbar",
    "calimbarían": "calimbar",
    "calimbarías": "calimbar",
    "calimbas": "calimbar",
    "calimbase": "calimbar",
    "calimbaseis": "calimbar",
    "calimbasen": "calimbar",
    "calimbases": "calimbar",
    "calimbaste": "calimbar",
    "calimbasteis": "calimbar",
    "calimbe": "calimbar",
    "calimbemos": "calimbar",
    "calimben": "calimbar",
    "calimbes": "calimbar",
    "calimbo": "calimbar",
    "calimbábamos": "calimbar",
    "calimbáis": "calimbar",
    "calimbáramos": "calimbar",
    "calimbáremos": "calimbar",
    "calimbásemos": "calimbar",
    "calimbé": "calimbar",
    "calimbéis": "calimbar",
    "calimbó": "calimbar",
    "calimicos": "calimico",
    "calinas": "calina",
    "calinosa": "calinoso",
    "calinosas": "calinoso",
    "calinosos": "calinoso",
    "calistenias": "calistenia",
    "calitas": "calita",
    "calitipias": "calitipia",
    "caliza": "calizo",
    "calizas": "calizo",
    "calizos": "calizo",
    "calla": "callar",
    "callaba": "callar",
    "callabais": "callar",
    "callaban": "callar",
    "callabas": "callar",
    "callad": "callar",
    "callada": "callar",
    "calladas": "callar",
    "callado": "callar",
    "callados": "callar",
    "callamos": "callar",
    "callan": "callar",
    "callanas": "callana",
    "callando": "callar",
    "callanta": "callantar",
    "callantaba": "callantar",
    "callantabais": "callantar",
    "callantaban": "callantar",
    "callantabas": "callantar",
    "callantad": "callantar",
    "callantada": "callantar",
    "callantadas": "callantar",
    "callantado": "callantar",
    "callantados": "callantar",
    "callantamos": "callantar",
    "callantan": "callantar",
    "callantando": "callantar",
    "callantara": "callantar",
    "callantarais": "callantar",
    "callantaran": "callantar",
    "callantaras": "callantar",
    "callantare": "callantar",
    "callantareis": "callantar",
    "callantaremos": "callantar",
    "callantaren": "callantar",
    "callantares": "callantar",
    "callantaron": "callantar",
    "callantará": "callantar",
    "callantarán": "callantar",
    "callantarás": "callantar",
    "callantaré": "callantar",
    "callantaréis": "callantar",
    "callantaría": "callantar",
    "callantaríais": "callantar",
    "callantaríamos": "callantar",
    "callantarían": "callantar",
    "callantarías": "callantar",
    "callantas": "callantar",
    "callantase": "callantar",
    "callantaseis": "callantar",
    "callantasen": "callantar",
    "callantases": "callantar",
    "callantaste": "callantar",
    "callantasteis": "callantar",
    "callante": "callantar",
    "callantemos": "callantar",
    "callanten": "callantar",
    "callantes": "callantar",
    "callanto": "callantar",
    "callantábamos": "callantar",
    "callantáis": "callantar",
    "callantáramos": "callantar",
    "callantáremos": "callantar",
    "callantásemos": "callantar",
    "callanté": "callantar",
    "callantéis": "callantar",
    "callantó": "callantar",
    "callaos": "callao",
    "callara": "callar",
    "callarais": "callar",
    "callaran": "callar",
    "callaras": "callar",
    "callare": "callar",
    "callareis": "callar",
    "callaremos": "callar",
    "callaren": "callar",
    "callares": "callar",
    "callaron": "callar",
    "callará": "callar",
    "callarán": "callar",
    "callarás": "callar",
    "callaré": "callar",
    "callaréis": "callar",
    "callaría": "callar",
    "callaríais": "callar",
    "callaríamos": "callar",
    "callarían": "callar",
    "callarías": "callar",
    "callas": "callar",
    "callase": "callar",
    "callaseis": "callar",
    "callasen": "callar",
    "callases": "callar",
    "callaste": "callar",
    "callasteis": "callar",
    "calle": "callar",
    "callea": "callear",
    "calleaba": "callear",
    "calleabais": "callear",
    "calleaban": "callear",
    "calleabas": "callear",
    "callead": "callear",
    "calleada": "callear",
    "calleadas": "callear",
    "calleado": "callear",
    "calleados": "callear",
    "calleamos": "callear",
    "callean": "callear",
    "calleando": "callear",
    "calleara": "callear",
    "callearais": "callear",
    "callearan": "callear",
    "callearas": "callear",
    "calleare": "callear",
    "calleareis": "callear",
    "callearemos": "callear",
    "callearen": "callear",
    "calleares": "callear",
    "callearon": "callear",
    "calleará": "callear",
    "callearán": "callear",
    "callearás": "callear",
    "callearé": "callear",
    "callearéis": "callear",
    "callearía": "callear",
    "callearíais": "callear",
    "callearíamos": "callear",
    "callearían": "callear",
    "callearías": "callear",
    "calleas": "callear",
    "callease": "callear",
    "calleaseis": "callear",
    "calleasen": "callear",
    "calleases": "callear",
    "calleaste": "callear",
    "calleasteis": "callear",
    "callee": "callear",
    "calleemos": "callear",
    "calleen": "callear",
    "callees": "callear",
    "callejea": "callejear",
    "callejeaba": "callejear",
    "callejeabais": "callejear",
    "callejeaban": "callejear",
    "callejeabas": "callejear",
    "callejead": "callejear",
    "callejeada": "callejear",
    "callejeadas": "callejear",
    "callejeado": "callejear",
    "callejeados": "callejear",
    "callejeamos": "callejear",
    "callejean": "callejear",
    "callejeando": "callejear",
    "callejeara": "callejear",
    "callejearais": "callejear",
    "callejearan": "callejear",
    "callejearas": "callejear",
    "callejeare": "callejear",
    "callejeareis": "callejear",
    "callejearemos": "callejear",
    "callejearen": "callejear",
    "callejeares": "callejear",
    "callejearon": "callejear",
    "callejeará": "callejear",
    "callejearán": "callejear",
    "callejearás": "callejear",
    "callejearé": "callejear",
    "callejearéis": "callejear",
    "callejearía": "callejear",
    "callejearíais": "callejear",
    "callejearíamos": "callejear",
    "callejearían": "callejear",
    "callejearías": "callejear",
    "callejeas": "callejear",
    "callejease": "callejear",
    "callejeaseis": "callejear",
    "callejeasen": "callejear",
    "callejeases": "callejear",
    "callejeaste": "callejear",
    "callejeasteis": "callejear",
    "callejee": "callejear",
    "callejeemos": "callejear",
    "callejeen": "callejear",
    "callejees": "callejear",
    "callejeo": "callejear",
    "callejeos": "callejeo",
    "callejera": "callejero",
    "callejeras": "callejero",
    "callejeros": "callejero",
    "callejeábamos": "callejear",
    "callejeáis": "callejear",
    "callejeáramos": "callejear",
    "callejeáremos": "callejear",
    "callejeásemos": "callejear",
    "callejeé": "callejear",
    "callejeéis": "callejear",
    "callejeó": "callejear",
    "callejones": "callejón",
    "callejuelas": "callejuela",
    "callemos": "callar",
    "callen": "callar",
    "calleo": "callear",
    "calleras": "callera",
    "calles": "calle",
    "calleábamos": "callear",
    "calleáis": "callear",
    "calleáramos": "callear",
    "calleáremos": "callear",
    "calleásemos": "callear",
    "calleé": "callear",
    "calleéis": "callear",
    "calleó": "callear",
    "callicidas": "callicida",
    "callistas": "callista",
    "callo": "callar",
    "callos": "callo",
    "callosa": "calloso",
    "callosas": "calloso",
    "callosidades": "callosidad",
    "callosos": "calloso",
    "callábamos": "callar",
    "calláis": "callar",
    "calláramos": "callar",
    "calláremos": "callar",
    "callásemos": "callar",
    "callé": "callar",
    "calléis": "callar",
    "calló": "callar",
    "calma": "calmo",
    "calmaba": "calmar",
    "calmabais": "calmar",
    "calmaban": "calmar",
    "calmabas": "calmar",
    "calmad": "calmar",
    "calmada": "calmar",
    "calmadas": "calmar",
    "calmado": "calmar",
    "calmados": "calmar",
    "calmamos": "calmar",
    "calman": "calmar",
    "calmando": "calmar",
    "calmantes": "calmante",
    "calmara": "calmar",
    "calmarais": "calmar",
    "calmaran": "calmar",
    "calmaras": "calmar",
    "calmare": "calmar",
    "calmareis": "calmar",
    "calmaremos": "calmar",
    "calmaren": "calmar",
    "calmares": "calmar",
    "calmaron": "calmar",
    "calmará": "calmar",
    "calmarán": "calmar",
    "calmarás": "calmar",
    "calmaré": "calmar",
    "calmaréis": "calmar",
    "calmaría": "calmar",
    "calmaríais": "calmar",
    "calmaríamos": "calmar",
    "calmarían": "calmar",
    "calmarías": "calmar",
    "calmas": "calmo",
    "calmase": "calmar",
    "calmaseis": "calmar",
    "calmasen": "calmar",
    "calmases": "calmar",
    "calmaste": "calmar",
    "calmasteis": "calmar",
    "calme": "calmar",
    "calmemos": "calmar",
    "calmen": "calmar",
    "calmerías": "calmería",
    "calmes": "calmar",
    "calmo": "calmar",
    "calmodulinas": "calmodulina",
    "calmos": "calmo",
    "calmosa": "calmoso",
    "calmosas": "calmoso",
    "calmosos": "calmoso",
    "calmuda": "calmudo",
    "calmudas": "calmudo",
    "calmudos": "calmudo",
    "calmábamos": "calmar",
    "calmáis": "calmar",
    "calmáramos": "calmar",
    "calmáremos": "calmar",
    "calmásemos": "calmar",
    "calmé": "calmar",
    "calméis": "calmar",
    "calmó": "calmar",
    "calo": "calar",
    "calofriaba": "calofriar",
    "calofriabais": "calofriar",
    "calofriaban": "calofriar",
    "calofriabas": "calofriar",
    "calofriad": "calofriar",
    "calofriada": "calofriar",
    "calofriadas": "calofriar",
    "calofriado": "calofriar",
    "calofriados": "calofriar",
    "calofriamos": "calofriar",
    "calofriando": "calofriar",
    "calofriara": "calofriar",
    "calofriarais": "calofriar",
    "calofriaran": "calofriar",
    "calofriaras": "calofriar",
    "calofriare": "calofriar",
    "calofriareis": "calofriar",
    "calofriaremos": "calofriar",
    "calofriaren": "calofriar",
    "calofriares": "calofriar",
    "calofriaron": "calofriar",
    "calofriará": "calofriar",
    "calofriarán": "calofriar",
    "calofriarás": "calofriar",
    "calofriaré": "calofriar",
    "calofriaréis": "calofriar",
    "calofriaría": "calofriar",
    "calofriaríais": "calofriar",
    "calofriaríamos": "calofriar",
    "calofriarían": "calofriar",
    "calofriarías": "calofriar",
    "calofriase": "calofriar",
    "calofriaseis": "calofriar",
    "calofriasen": "calofriar",
    "calofriases": "calofriar",
    "calofriaste": "calofriar",
    "calofriasteis": "calofriar",
    "calofriemos": "calofriar",
    "calofriábamos": "calofriar",
    "calofriáis": "calofriar",
    "calofriáramos": "calofriar",
    "calofriáremos": "calofriar",
    "calofriásemos": "calofriar",
    "calofrié": "calofriar",
    "calofriéis": "calofriar",
    "calofrió": "calofriar",
    "calofría": "calofriar",
    "calofrían": "calofriar",
    "calofrías": "calofriar",
    "calofríe": "calofriar",
    "calofríen": "calofriar",
    "calofríes": "calofriar",
    "calofrío": "calofriar",
    "calofríos": "calofrío",
    "calomelano": "calomelanos",
    "calones": "calón",
    "calorelectricidades": "calorelectricidad",
    "calores": "calor",
    "calorice": "calorizar",
    "caloricemos": "calorizar",
    "caloricen": "calorizar",
    "calorices": "calorizar",
    "caloricé": "calorizar",
    "caloricéis": "calorizar",
    "calorifugaciones": "calorifugación",
    "calorifugada": "calorifugado",
    "calorifugadas": "calorifugado",
    "calorifugados": "calorifugado",
    "calorimetrías": "calorimetría",
    "calorimétrica": "calorimétrico",
    "calorimétricas": "calorimétrico",
    "calorimétricos": "calorimétrico",
    "caloriza": "calorizar",
    "calorizaba": "calorizar",
    "calorizabais": "calorizar",
    "calorizaban": "calorizar",
    "calorizabas": "calorizar",
    "calorizaciones": "calorización",
    "calorizad": "calorizar",
    "calorizada": "calorizar",
    "calorizadas": "calorizar",
    "calorizado": "calorizar",
    "calorizados": "calorizar",
    "calorizamos": "calorizar",
    "calorizan": "calorizar",
    "calorizando": "calorizar",
    "calorizara": "calorizar",
    "calorizarais": "calorizar",
    "calorizaran": "calorizar",
    "calorizaras": "calorizar",
    "calorizare": "calorizar",
    "calorizareis": "calorizar",
    "calorizaremos": "calorizar",
    "calorizaren": "calorizar",
    "calorizares": "calorizar",
    "calorizaron": "calorizar",
    "calorizará": "calorizar",
    "calorizarán": "calorizar",
    "calorizarás": "calorizar",
    "calorizaré": "calorizar",
    "calorizaréis": "calorizar",
    "calorizaría": "calorizar",
    "calorizaríais": "calorizar",
    "calorizaríamos": "calorizar",
    "calorizarían": "calorizar",
    "calorizarías": "calorizar",
    "calorizas": "calorizar",
    "calorizase": "calorizar",
    "calorizaseis": "calorizar",
    "calorizasen": "calorizar",
    "calorizases": "calorizar",
    "calorizaste": "calorizar",
    "calorizasteis": "calorizar",
    "calorizo": "calorizar",
    "calorizábamos": "calorizar",
    "calorizáis": "calorizar",
    "calorizáramos": "calorizar",
    "calorizáremos": "calorizar",
    "calorizásemos": "calorizar",
    "calorizó": "calorizar",
    "calorías": "caloría",
    "calorífera": "calorífero",
    "caloríferas": "calorífero",
    "caloríferos": "calorífero",
    "calorífica": "calorífico",
    "caloríficas": "calorífico",
    "caloríficos": "calorífico",
    "calorífuga": "calorífugo",
    "calorífugas": "calorífugo",
    "calorífugos": "calorífugo",
    "calorímetros": "calorímetro",
    "calostros": "calostro",
    "calotos": "caloto",
    "caloyos": "caloyo",
    "calpullis": "calpulli",
    "calque": "calcar",
    "calquemos": "calcar",
    "calquen": "calcar",
    "calques": "calcar",
    "calqué": "calcar",
    "calquéis": "calcar",
    "calumas": "caluma",
    "calumnia": "calumniar",
    "calumniaba": "calumniar",
    "calumniabais": "calumniar",
    "calumniaban": "calumniar",
    "calumniabas": "calumniar",
    "calumniad": "calumniar",
    "calumniada": "calumniar",
    "calumniadas": "calumniar",
    "calumniado": "calumniar",
    "calumniadora": "calumniador",
    "calumniadoras": "calumniador",
    "calumniadores": "calumniador",
    "calumniados": "calumniar",
    "calumniamos": "calumniar",
    "calumnian": "calumniar",
    "calumniando": "calumniar",
    "calumniara": "calumniar",
    "calumniarais": "calumniar",
    "calumniaran": "calumniar",
    "calumniaras": "calumniar",
    "calumniare": "calumniar",
    "calumniareis": "calumniar",
    "calumniaremos": "calumniar",
    "calumniaren": "calumniar",
    "calumniares": "calumniar",
    "calumniaron": "calumniar",
    "calumniará": "calumniar",
    "calumniarán": "calumniar",
    "calumniarás": "calumniar",
    "calumniaré": "calumniar",
    "calumniaréis": "calumniar",
    "calumniaría": "calumniar",
    "calumniaríais": "calumniar",
    "calumniaríamos": "calumniar",
    "calumniarían": "calumniar",
    "calumniarías": "calumniar",
    "calumnias": "calumniar",
    "calumniase": "calumniar",
    "calumniaseis": "calumniar",
    "calumniasen": "calumniar",
    "calumniases": "calumniar",
    "calumniaste": "calumniar",
    "calumniasteis": "calumniar",
    "calumnie": "calumniar",
    "calumniemos": "calumniar",
    "calumnien": "calumniar",
    "calumnies": "calumniar",
    "calumnio": "calumniar",
    "calumniosa": "calumnioso",
    "calumniosas": "calumnioso",
    "calumniosos": "calumnioso",
    "calumniábamos": "calumniar",
    "calumniáis": "calumniar",
    "calumniáramos": "calumniar",
    "calumniáremos": "calumniar",
    "calumniásemos": "calumniar",
    "calumnié": "calumniar",
    "calumniéis": "calumniar",
    "calumnió": "calumniar",
    "calurosa": "caluroso",
    "calurosas": "caluroso",
    "calurosos": "caluroso",
    "calva": "calvo",
    "calvarios": "calvario",
    "calvas": "calvo",
    "calveros": "calvero",
    "calvicies": "calvicie",
    "calvinismos": "calvinismo",
    "calvinistas": "calvinista",
    "calvos": "calvo",
    "calza": "calzar",
    "calzaba": "calzar",
    "calzabais": "calzar",
    "calzaban": "calzar",
    "calzabas": "calzar",
    "calzad": "calzar",
    "calzada": "calzar",
    "calzadas": "calzar",
    "calzaderas": "calzadera",
    "calzado": "calzar",
    "calzadores": "calzador",
    "calzados": "calzar",
    "calzaduras": "calzadura",
    "calzamos": "calzar",
    "calzan": "calzar",
    "calzando": "calzar",
    "calzara": "calzar",
    "calzarais": "calzar",
    "calzaran": "calzar",
    "calzaras": "calzar",
    "calzare": "calzar",
    "calzareis": "calzar",
    "calzaremos": "calzar",
    "calzaren": "calzar",
    "calzares": "calzar",
    "calzaron": "calzar",
    "calzará": "calzar",
    "calzarán": "calzar",
    "calzarás": "calzar",
    "calzaré": "calzar",
    "calzaréis": "calzar",
    "calzaría": "calzar",
    "calzaríais": "calzar",
    "calzaríamos": "calzar",
    "calzarían": "calzar",
    "calzarías": "calzar",
    "calzas": "calzar",
    "calzase": "calzar",
    "calzaseis": "calzar",
    "calzasen": "calzar",
    "calzases": "calzar",
    "calzaste": "calzar",
    "calzasteis": "calzar",
    "calzo": "calzar",
    "calzonas": "calzona",
    "calzoncillos": "calzoncillo",
    "calzones": "calzón",
    "calzos": "calzo",
    "calzábamos": "calzar",
    "calzáis": "calzar",
    "calzáramos": "calzar",
    "calzáremos": "calzar",
    "calzásemos": "calzar",
    "calzó": "calzar",
    "calábamos": "calar",
    "caláis": "calar",
    "caláramos": "calar",
    "caláremos": "calar",
    "calásemos": "calar",
    "calé": "calar",
    "caléis": "calar",
    "caléndulas": "caléndula",
    "calés": "calé",
    "calículos": "calículo",
    "calígines": "calígine",
    "calígrafa": "calígrafo",
    "calígrafas": "calígrafo",
    "calígrafos": "calígrafo",
    "caló": "calar",
    "calógera": "calógero",
    "calógeras": "calógero",
    "calógeros": "calógero",
    "calórica": "calórico",
    "calóricas": "calórico",
    "calóricos": "calórico",
    "calós": "caló",
    "camachuelos": "camachuelo",
    "camadas": "camada",
    "camafeos": "camafeo",
    "camaguas": "camagua",
    "camaldulenses": "camaldulense",
    "camaleona": "camaleón",
    "camaleonas": "camaleón",
    "camaleones": "camaleón",
    "camales": "camal",
    "camamilas": "camamila",
    "camandulea": "camandulear",
    "camanduleaba": "camandulear",
    "camanduleabais": "camandulear",
    "camanduleaban": "camandulear",
    "camanduleabas": "camandulear",
    "camandulead": "camandulear",
    "camanduleada": "camandulear",
    "camanduleadas": "camandulear",
    "camanduleado": "camandulear",
    "camanduleados": "camandulear",
    "camanduleamos": "camandulear",
    "camandulean": "camandulear",
    "camanduleando": "camandulear",
    "camanduleara": "camandulear",
    "camandulearais": "camandulear",
    "camandulearan": "camandulear",
    "camandulearas": "camandulear",
    "camanduleare": "camandulear",
    "camanduleareis": "camandulear",
    "camandulearemos": "camandulear",
    "camandulearen": "camandulear",
    "camanduleares": "camandulear",
    "camandulearon": "camandulear",
    "camanduleará": "camandulear",
    "camandulearán": "camandulear",
    "camandulearás": "camandulear",
    "camandulearé": "camandulear",
    "camandulearéis": "camandulear",
    "camandulearía": "camandulear",
    "camandulearíais": "camandulear",
    "camandulearíamos": "camandulear",
    "camandulearían": "camandulear",
    "camandulearías": "camandulear",
    "camanduleas": "camandulear",
    "camandulease": "camandulear",
    "camanduleaseis": "camandulear",
    "camanduleasen": "camandulear",
    "camanduleases": "camandulear",
    "camanduleaste": "camandulear",
    "camanduleasteis": "camandulear",
    "camandulee": "camandulear",
    "camanduleemos": "camandulear",
    "camanduleen": "camandulear",
    "camandulees": "camandulear",
    "camandulenses": "camandulense",
    "camanduleo": "camandulear",
    "camandulera": "camandulero",
    "camanduleras": "camandulero",
    "camanduleros": "camandulero",
    "camandulerías": "camandulería",
    "camanduleábamos": "camandulear",
    "camanduleáis": "camandulear",
    "camanduleáramos": "camandulear",
    "camanduleáremos": "camandulear",
    "camanduleásemos": "camandulear",
    "camanduleé": "camandulear",
    "camanduleéis": "camandulear",
    "camanduleó": "camandulear",
    "camaradas": "camarada",
    "camaraderías": "camaradería",
    "camaranchones": "camaranchón",
    "camarera": "camarero",
    "camareras": "camarero",
    "camareros": "camarero",
    "camaretas": "camareta",
    "camarillas": "camarilla",
    "camarillesca": "camarillesco",
    "camarillescas": "camarillesco",
    "camarillescos": "camarillesco",
    "camarines": "camarín",
    "camaristas": "camarista",
    "camarlengos": "camarlengo",
    "camaronera": "camaronero",
    "camaroneras": "camaronero",
    "camaroneros": "camaronero",
    "camarones": "camarón",
    "camarotes": "camarote",
    "camas": "cama",
    "camastrona": "camastrón",
    "camastronas": "camastrón",
    "camastronerías": "camastronería",
    "camastrones": "camastrón",
    "camastros": "camastro",
    "camauros": "camauro",
    "cambalachea": "cambalachear",
    "cambalacheaba": "cambalachear",
    "cambalacheabais": "cambalachear",
    "cambalacheaban": "cambalachear",
    "cambalacheabas": "cambalachear",
    "cambalachead": "cambalachear",
    "cambalacheada": "cambalachear",
    "cambalacheadas": "cambalachear",
    "cambalacheado": "cambalachear",
    "cambalacheados": "cambalachear",
    "cambalacheamos": "cambalachear",
    "cambalachean": "cambalachear",
    "cambalacheando": "cambalachear",
    "cambalacheara": "cambalachear",
    "cambalachearais": "cambalachear",
    "cambalachearan": "cambalachear",
    "cambalachearas": "cambalachear",
    "cambalacheare": "cambalachear",
    "cambalacheareis": "cambalachear",
    "cambalachearemos": "cambalachear",
    "cambalachearen": "cambalachear",
    "cambalacheares": "cambalachear",
    "cambalachearon": "cambalachear",
    "cambalacheará": "cambalachear",
    "cambalachearán": "cambalachear",
    "cambalachearás": "cambalachear",
    "cambalachearé": "cambalachear",
    "cambalachearéis": "cambalachear",
    "cambalachearía": "cambalachear",
    "cambalachearíais": "cambalachear",
    "cambalachearíamos": "cambalachear",
    "cambalachearían": "cambalachear",
    "cambalachearías": "cambalachear",
    "cambalacheas": "cambalachear",
    "cambalachease": "cambalachear",
    "cambalacheaseis": "cambalachear",
    "cambalacheasen": "cambalachear",
    "cambalacheases": "cambalachear",
    "cambalacheaste": "cambalachear",
    "cambalacheasteis": "cambalachear",
    "cambalachee": "cambalachear",
    "cambalacheemos": "cambalachear",
    "cambalacheen": "cambalachear",
    "cambalachees": "cambalachear",
    "cambalacheo": "cambalachear",
    "cambalachera": "cambalachero",
    "cambalacheras": "cambalachero",
    "cambalacheros": "cambalachero",
    "cambalaches": "cambalache",
    "cambalacheábamos": "cambalachear",
    "cambalacheáis": "cambalachear",
    "cambalacheáramos": "cambalachear",
    "cambalacheáremos": "cambalachear",
    "cambalacheásemos": "cambalachear",
    "cambalacheé": "cambalachear",
    "cambalacheéis": "cambalachear",
    "cambalacheó": "cambalachear",
    "cambia": "cambiar",
    "cambiaba": "cambiar",
    "cambiabais": "cambiar",
    "cambiaban": "cambiar",
    "cambiabas": "cambiar",
    "cambiables": "cambiable",
    "cambiad": "cambiar",
    "cambiada": "cambiar",
    "cambiadas": "cambiar",
    "cambiado": "cambiar",
    "cambiadores": "cambiador",
    "cambiados": "cambiar",
    "cambiales": "cambial",
    "cambiamos": "cambiar",
    "cambian": "cambiar",
    "cambiando": "cambiar",
    "cambiantes": "cambiante",
    "cambiara": "cambiar",
    "cambiarais": "cambiar",
    "cambiaran": "cambiar",
    "cambiaras": "cambiar",
    "cambiare": "cambiar",
    "cambiareis": "cambiar",
    "cambiaremos": "cambiar",
    "cambiaren": "cambiar",
    "cambiares": "cambiar",
    "cambiaria": "cambiario",
    "cambiarias": "cambiario",
    "cambiarios": "cambiario",
    "cambiaron": "cambiar",
    "cambiará": "cambiar",
    "cambiarán": "cambiar",
    "cambiarás": "cambiar",
    "cambiaré": "cambiar",
    "cambiaréis": "cambiar",
    "cambiaría": "cambiar",
    "cambiaríais": "cambiar",
    "cambiaríamos": "cambiar",
    "cambiarían": "cambiar",
    "cambiarías": "cambiar",
    "cambias": "cambiar",
    "cambiase": "cambiar",
    "cambiaseis": "cambiar",
    "cambiasen": "cambiar",
    "cambiases": "cambiar",
    "cambiaste": "cambiar",
    "cambiasteis": "cambiar",
    "cambiazos": "cambiazo",
    "cambie": "cambiar",
    "cambiemos": "cambiar",
    "cambien": "cambiar",
    "cambies": "cambiar",
    "cambio": "cambiar",
    "cambios": "cambio",
    "cambistas": "cambista",
    "cambiumes": "cambium",
    "cambiums": "cambium",
    "cambiábamos": "cambiar",
    "cambiáis": "cambiar",
    "cambiáramos": "cambiar",
    "cambiáremos": "cambiar",
    "cambiásemos": "cambiar",
    "cambié": "cambiar",
    "cambiéis": "cambiar",
    "cambió": "cambiar",
    "camboyana": "camboyano",
    "camboyanas": "camboyano",
    "camboyanos": "camboyano",
    "cambrayes": "cambray",
    "cambriana": "cambriano",
    "cambrianas": "cambriano",
    "cambrianos": "cambriano",
    "cambrillones": "cambrillón",
    "cambrones": "cambrón",
    "cambroños": "cambroño",
    "cambuchos": "cambucho",
    "cambuja": "cambujo",
    "cambujas": "cambujo",
    "cambujes": "cambuj",
    "cambujos": "cambujo",
    "cambullones": "cambullón",
    "cambures": "cambur",
    "cambuta": "cambuto",
    "cambutas": "cambuto",
    "cambutes": "cambute",
    "cambutos": "cambuto",
    "camedrios": "camedrio",
    "camela": "camelar",
    "camelaba": "camelar",
    "camelabais": "camelar",
    "camelaban": "camelar",
    "camelabas": "camelar",
    "camelad": "camelar",
    "camelada": "camelar",
    "cameladas": "camelar",
    "camelado": "camelar",
    "camelados": "camelar",
    "camelamos": "camelar",
    "camelan": "camelar",
    "camelando": "camelar",
    "camelara": "camelar",
    "camelarais": "camelar",
    "camelaran": "camelar",
    "camelaras": "camelar",
    "camelare": "camelar",
    "camelareis": "camelar",
    "camelaremos": "camelar",
    "camelaren": "camelar",
    "camelares": "camelar",
    "camelaron": "camelar",
    "camelará": "camelar",
    "camelarán": "camelar",
    "camelarás": "camelar",
    "camelaré": "camelar",
    "camelaréis": "camelar",
    "camelaría": "camelar",
    "camelaríais": "camelar",
    "camelaríamos": "camelar",
    "camelarían": "camelar",
    "camelarías": "camelar",
    "camelas": "camelar",
    "camelase": "camelar",
    "camelaseis": "camelar",
    "camelasen": "camelar",
    "camelases": "camelar",
    "camelaste": "camelar",
    "camelasteis": "camelar",
    "camele": "camelar",
    "camelemos": "camelar",
    "camelen": "camelar",
    "cameles": "camelar",
    "cameletes": "camelete",
    "camelias": "camelia",
    "camelinas": "camelina",
    "camella": "camello",
    "camellas": "camello",
    "camellera": "camellero",
    "camelleras": "camellero",
    "camelleros": "camellero",
    "camellones": "camellón",
    "camellos": "camello",
    "camelo": "camelar",
    "camelos": "camelo",
    "camelábamos": "camelar",
    "cameláis": "camelar",
    "cameláramos": "camelar",
    "cameláremos": "camelar",
    "camelásemos": "camelar",
    "camelé": "camelar",
    "cameléis": "camelar",
    "camelística": "camelístico",
    "camelísticas": "camelístico",
    "camelísticos": "camelístico",
    "cameló": "camelar",
    "camemberts": "camembert",
    "camenales": "camenal",
    "camenas": "camena",
    "camera": "camero",
    "cameralismos": "cameralismo",
    "cameralistas": "cameralista",
    "cameras": "camero",
    "camerinos": "camerino",
    "cameros": "camero",
    "camerunenses": "camerunense",
    "camerunesa": "camerunés",
    "camerunesas": "camerunés",
    "cameruneses": "camerunés",
    "camerámanes": "cameraman",
    "camila": "camilo",
    "camilas": "camilo",
    "camillas": "camilla",
    "camillera": "camillero",
    "camilleras": "camillero",
    "camilleros": "camillero",
    "camilos": "camilo",
    "camina": "caminar",
    "caminaba": "caminar",
    "caminabais": "caminar",
    "caminaban": "caminar",
    "caminabas": "caminar",
    "caminad": "caminar",
    "caminada": "caminar",
    "caminadas": "caminar",
    "caminado": "caminar",
    "caminadora": "caminador",
    "caminadoras": "caminador",
    "caminadores": "caminador",
    "caminados": "caminar",
    "caminamos": "caminar",
    "caminan": "caminar",
    "caminando": "caminar",
    "caminantes": "caminante",
    "caminara": "caminar",
    "caminarais": "caminar",
    "caminaran": "caminar",
    "caminaras": "caminar",
    "caminare": "caminar",
    "caminareis": "caminar",
    "caminaremos": "caminar",
    "caminaren": "caminar",
    "caminares": "caminar",
    "caminaron": "caminar",
    "caminará": "caminar",
    "caminarán": "caminar",
    "caminarás": "caminar",
    "caminaré": "caminar",
    "caminaréis": "caminar",
    "caminaría": "caminar",
    "caminaríais": "caminar",
    "caminaríamos": "caminar",
    "caminarían": "caminar",
    "caminarías": "caminar",
    "caminas": "caminar",
    "caminase": "caminar",
    "caminaseis": "caminar",
    "caminasen": "caminar",
    "caminases": "caminar",
    "caminaste": "caminar",
    "caminasteis": "caminar",
    "caminatas": "caminata",
    "camine": "caminar",
    "caminemos": "caminar",
    "caminen": "caminar",
    "caminera": "caminero",
    "camineras": "caminero",
    "camineros": "caminero",
    "camines": "caminar",
    "camino": "caminar",
    "caminos": "camino",
    "caminábamos": "caminar",
    "camináis": "caminar",
    "camináramos": "caminar",
    "camináremos": "caminar",
    "caminásemos": "caminar",
    "caminé": "caminar",
    "caminéis": "caminar",
    "caminíes": "caminí",
    "caminís": "caminí",
    "caminó": "caminar",
    "camionajes": "camionaje",
    "camionera": "camionero",
    "camioneras": "camionero",
    "camioneros": "camionero",
    "camiones": "camión",
    "camionetas": "camioneta",
    "camisards": "camisard",
    "camisas": "camisa",
    "camisera": "camisero",
    "camiseras": "camisero",
    "camiseros": "camisero",
    "camiserías": "camisería",
    "camisetas": "camiseta",
    "camisolas": "camisola",
    "camisolines": "camisolín",
    "camisones": "camisón",
    "camitas": "camita",
    "camitosemítica": "camitosemítico",
    "camitosemíticas": "camitosemítico",
    "camitosemíticos": "camitosemítico",
    "camocha": "camochar",
    "camochaba": "camochar",
    "camochabais": "camochar",
    "camochaban": "camochar",
    "camochabas": "camochar",
    "camochad": "camochar",
    "camochada": "camochar",
    "camochadas": "camochar",
    "camochado": "camochar",
    "camochados": "camochar",
    "camochamos": "camochar",
    "camochan": "camochar",
    "camochando": "camochar",
    "camochara": "camochar",
    "camocharais": "camochar",
    "camocharan": "camochar",
    "camocharas": "camochar",
    "camochare": "camochar",
    "camochareis": "camochar",
    "camocharemos": "camochar",
    "camocharen": "camochar",
    "camochares": "camochar",
    "camocharon": "camochar",
    "camochará": "camochar",
    "camocharán": "camochar",
    "camocharás": "camochar",
    "camocharé": "camochar",
    "camocharéis": "camochar",
    "camocharía": "camochar",
    "camocharíais": "camochar",
    "camocharíamos": "camochar",
    "camocharían": "camochar",
    "camocharías": "camochar",
    "camochas": "camochar",
    "camochase": "camochar",
    "camochaseis": "camochar",
    "camochasen": "camochar",
    "camochases": "camochar",
    "camochaste": "camochar",
    "camochasteis": "camochar",
    "camoche": "camochar",
    "camochemos": "camochar",
    "camochen": "camochar",
    "camoches": "camochar",
    "camocho": "camochar",
    "camochábamos": "camochar",
    "camocháis": "camochar",
    "camocháramos": "camochar",
    "camocháremos": "camochar",
    "camochásemos": "camochar",
    "camoché": "camochar",
    "camochéis": "camochar",
    "camochó": "camochar",
    "camomilas": "camomila",
    "camoncillos": "camoncillo",
    "camones": "camón",
    "camorras": "camorra",
    "camorrera": "camorrero",
    "camorreras": "camorrero",
    "camorreros": "camorrero",
    "camorristas": "camorrista",
    "camotes": "camote",
    "camotillos": "camotillo",
    "campa": "campar",
    "campaba": "campar",
    "campabais": "campar",
    "campaban": "campar",
    "campabas": "campar",
    "campad": "campar",
    "campada": "campar",
    "campadas": "campar",
    "campado": "campar",
    "campados": "campar",
    "campales": "campal",
    "campamentos": "campamento",
    "campamos": "campar",
    "campan": "campar",
    "campanadas": "campanada",
    "campanarios": "campanario",
    "campanas": "campana",
    "campando": "campar",
    "campanelas": "campanela",
    "campaneos": "campaneo",
    "campanera": "campanero",
    "campaneras": "campanero",
    "campaneros": "campanero",
    "campanienses": "campaniense",
    "campaniformes": "campaniforme",
    "campaniles": "campanil",
    "campanillas": "campanilla",
    "campanillazos": "campanillazo",
    "campanillea": "campanillear",
    "campanilleaba": "campanillear",
    "campanilleabais": "campanillear",
    "campanilleaban": "campanillear",
    "campanilleabas": "campanillear",
    "campanillead": "campanillear",
    "campanilleada": "campanillear",
    "campanilleadas": "campanillear",
    "campanilleado": "campanillear",
    "campanilleados": "campanillear",
    "campanilleamos": "campanillear",
    "campanillean": "campanillear",
    "campanilleando": "campanillear",
    "campanilleara": "campanillear",
    "campanillearais": "campanillear",
    "campanillearan": "campanillear",
    "campanillearas": "campanillear",
    "campanilleare": "campanillear",
    "campanilleareis": "campanillear",
    "campanillearemos": "campanillear",
    "campanillearen": "campanillear",
    "campanilleares": "campanillear",
    "campanillearon": "campanillear",
    "campanilleará": "campanillear",
    "campanillearán": "campanillear",
    "campanillearás": "campanillear",
    "campanillearé": "campanillear",
    "campanillearéis": "campanillear",
    "campanillearía": "campanillear",
    "campanillearíais": "campanillear",
    "campanillearíamos": "campanillear",
    "campanillearían": "campanillear",
    "campanillearías": "campanillear",
    "campanilleas": "campanillear",
    "campanillease": "campanillear",
    "campanilleaseis": "campanillear",
    "campanilleasen": "campanillear",
    "campanilleases": "campanillear",
    "campanilleaste": "campanillear",
    "campanilleasteis": "campanillear",
    "campanillee": "campanillear",
    "campanilleemos": "campanillear",
    "campanilleen": "campanillear",
    "campanillees": "campanillear",
    "campanilleo": "campanillear",
    "campanilleos": "campanilleo",
    "campanilleábamos": "campanillear",
    "campanilleáis": "campanillear",
    "campanilleáramos": "campanillear",
    "campanilleáremos": "campanillear",
    "campanilleásemos": "campanillear",
    "campanilleé": "campanillear",
    "campanilleéis": "campanillear",
    "campanilleó": "campanillear",
    "campanologías": "campanología",
    "campanos": "campano",
    "campantes": "campante",
    "campanuda": "campanudo",
    "campanudas": "campanudo",
    "campanudos": "campanudo",
    "campanuláceas": "campanulácea",
    "campanóloga": "campanólogo",
    "campanólogas": "campanólogo",
    "campanólogos": "campanólogo",
    "campara": "campar",
    "camparais": "campar",
    "camparan": "campar",
    "camparas": "campar",
    "campare": "campar",
    "campareis": "campar",
    "camparemos": "campar",
    "camparen": "campar",
    "campares": "campar",
    "camparon": "campar",
    "campará": "campar",
    "camparán": "campar",
    "camparás": "campar",
    "camparé": "campar",
    "camparéis": "campar",
    "camparía": "campar",
    "camparíais": "campar",
    "camparíamos": "campar",
    "camparían": "campar",
    "camparías": "campar",
    "campas": "campar",
    "campase": "campar",
    "campaseis": "campar",
    "campasen": "campar",
    "campases": "campar",
    "campaste": "campar",
    "campasteis": "campar",
    "campañas": "campaña",
    "campe": "campar",
    "campea": "campear",
    "campeaba": "campear",
    "campeabais": "campear",
    "campeaban": "campear",
    "campeabas": "campear",
    "campead": "campear",
    "campeada": "campear",
    "campeadas": "campear",
    "campeado": "campear",
    "campeadora": "campeador",
    "campeadoras": "campeador",
    "campeadores": "campeador",
    "campeados": "campear",
    "campeamos": "campear",
    "campean": "campear",
    "campeando": "campear",
    "campeara": "campear",
    "campearais": "campear",
    "campearan": "campear",
    "campearas": "campear",
    "campeare": "campear",
    "campeareis": "campear",
    "campearemos": "campear",
    "campearen": "campear",
    "campeares": "campear",
    "campearon": "campear",
    "campeará": "campear",
    "campearán": "campear",
    "campearás": "campear",
    "campearé": "campear",
    "campearéis": "campear",
    "campearía": "campear",
    "campearíais": "campear",
    "campearíamos": "campear",
    "campearían": "campear",
    "campearías": "campear",
    "campeas": "campear",
    "campease": "campear",
    "campeaseis": "campear",
    "campeasen": "campear",
    "campeases": "campear",
    "campeaste": "campear",
    "campeasteis": "campear",
    "campechana": "campechano",
    "campechanas": "campechano",
    "campechanos": "campechano",
    "campechanías": "campechanía",
    "campeches": "campeche",
    "campee": "campear",
    "campeemos": "campear",
    "campeen": "campear",
    "campees": "campear",
    "campemos": "campar",
    "campen": "campar",
    "campeo": "campear",
    "campeona": "campeón",
    "campeonas": "campeón",
    "campeonatos": "campeonato",
    "campeones": "campeón",
    "campera": "campero",
    "camperas": "campero",
    "camperos": "campero",
    "campes": "campar",
    "campesina": "campesino",
    "campesinados": "campesinado",
    "campesinas": "campesino",
    "campesinos": "campesino",
    "campestres": "campestre",
    "campeábamos": "campear",
    "campeáis": "campear",
    "campeáramos": "campear",
    "campeáremos": "campear",
    "campeásemos": "campear",
    "campeé": "campear",
    "campeéis": "campear",
    "campeó": "campear",
    "campidaneses": "campidanés",
    "campimetrías": "campimetría",
    "campings": "camping",
    "campistas": "campista",
    "campizales": "campizal",
    "campiñas": "campiña",
    "campo": "campar",
    "campogibraltareña": "campogibraltareño",
    "campogibraltareñas": "campogibraltareño",
    "campogibraltareños": "campogibraltareño",
    "campos": "campo",
    "camposantos": "camposanto",
    "campábamos": "campar",
    "campáis": "campar",
    "campánulas": "campánula",
    "campáramos": "campar",
    "campáremos": "campar",
    "campásemos": "campar",
    "campé": "campar",
    "campéis": "campar",
    "campímetros": "campímetro",
    "campó": "campar",
    "camuesa": "camueso",
    "camuesas": "camueso",
    "camuesos": "camueso",
    "camufla": "camuflar",
    "camuflaba": "camuflar",
    "camuflabais": "camuflar",
    "camuflaban": "camuflar",
    "camuflabas": "camuflar",
    "camuflad": "camuflar",
    "camuflada": "camuflar",
    "camufladas": "camuflar",
    "camuflado": "camuflar",
    "camuflados": "camuflar",
    "camuflajes": "camuflaje",
    "camuflamientos": "camuflamiento",
    "camuflamos": "camuflar",
    "camuflan": "camuflar",
    "camuflando": "camuflar",
    "camuflara": "camuflar",
    "camuflarais": "camuflar",
    "camuflaran": "camuflar",
    "camuflaras": "camuflar",
    "camuflare": "camuflar",
    "camuflareis": "camuflar",
    "camuflaremos": "camuflar",
    "camuflaren": "camuflar",
    "camuflares": "camuflar",
    "camuflaron": "camuflar",
    "camuflará": "camuflar",
    "camuflarán": "camuflar",
    "camuflarás": "camuflar",
    "camuflaré": "camuflar",
    "camuflaréis": "camuflar",
    "camuflaría": "camuflar",
    "camuflaríais": "camuflar",
    "camuflaríamos": "camuflar",
    "camuflarían": "camuflar",
    "camuflarías": "camuflar",
    "camuflas": "camuflar",
    "camuflase": "camuflar",
    "camuflaseis": "camuflar",
    "camuflasen": "camuflar",
    "camuflases": "camuflar",
    "camuflaste": "camuflar",
    "camuflasteis": "camuflar",
    "camufle": "camuflar",
    "camuflemos": "camuflar",
    "camuflen": "camuflar",
    "camufles": "camuflar",
    "camuflo": "camuflar",
    "camuflábamos": "camuflar",
    "camufláis": "camuflar",
    "camufláramos": "camuflar",
    "camufláremos": "camuflar",
    "camuflásemos": "camuflar",
    "camuflé": "camuflar",
    "camufléis": "camuflar",
    "camufló": "camuflar",
    "camus": "camu",
    "camáldulas": "camáldula",
    "camándulas": "camándula",
    "caméfitos": "caméfito",
    "camélidos": "camélido",
    "camítica": "camítico",
    "camíticas": "camítico",
    "camíticos": "camítico",
    "cana": "caño",
    "canaca": "canaco",
    "canacas": "canaco",
    "canacos": "canaco",
    "canadienses": "canadiense",
    "canaladuras": "canaladura",
    "canalejas": "canaleja",
    "canales": "canal",
    "canaletas": "canaleta",
    "canaletes": "canalete",
    "canaletos": "canaleto",
    "canalice": "canalizar",
    "canalicemos": "canalizar",
    "canalicen": "canalizar",
    "canalices": "canalizar",
    "canalicé": "canalizar",
    "canalicéis": "canalizar",
    "canaliza": "canalizar",
    "canalizaba": "canalizar",
    "canalizabais": "canalizar",
    "canalizaban": "canalizar",
    "canalizabas": "canalizar",
    "canalizables": "canalizable",
    "canalizaciones": "canalización",
    "canalizad": "canalizar",
    "canalizada": "canalizar",
    "canalizadas": "canalizar",
    "canalizado": "canalizar",
    "canalizadora": "canalizador",
    "canalizadoras": "canalizador",
    "canalizadores": "canalizador",
    "canalizados": "canalizar",
    "canalizamos": "canalizar",
    "canalizan": "canalizar",
    "canalizando": "canalizar",
    "canalizara": "canalizar",
    "canalizarais": "canalizar",
    "canalizaran": "canalizar",
    "canalizaras": "canalizar",
    "canalizare": "canalizar",
    "canalizareis": "canalizar",
    "canalizaremos": "canalizar",
    "canalizaren": "canalizar",
    "canalizares": "canalizar",
    "canalizaron": "canalizar",
    "canalizará": "canalizar",
    "canalizarán": "canalizar",
    "canalizarás": "canalizar",
    "canalizaré": "canalizar",
    "canalizaréis": "canalizar",
    "canalizaría": "canalizar",
    "canalizaríais": "canalizar",
    "canalizaríamos": "canalizar",
    "canalizarían": "canalizar",
    "canalizarías": "canalizar",
    "canalizas": "canalizar",
    "canalizase": "canalizar",
    "canalizaseis": "canalizar",
    "canalizasen": "canalizar",
    "canalizases": "canalizar",
    "canalizaste": "canalizar",
    "canalizasteis": "canalizar",
    "canalizo": "canalizar",
    "canalizos": "canalizo",
    "canalizábamos": "canalizar",
    "canalizáis": "canalizar",
    "canalizáramos": "canalizar",
    "canalizáremos": "canalizar",
    "canalizásemos": "canalizar",
    "canalizó": "canalizar",
    "canalladas": "canallada",
    "canallas": "canalla",
    "canallesca": "canallesco",
    "canallescas": "canallesco",
    "canallescos": "canallesco",
    "canalones": "canalón",
    "canalículos": "canalículo",
    "cananas": "canana",
    "cananea": "cananeo",
    "cananeas": "cananeo",
    "cananeos": "cananeo",
    "canapés": "canapé",
    "canaria": "canario",
    "canarias": "canario",
    "canarios": "canario",
    "canas": "caño",
    "canastadas": "canastada",
    "canastas": "canasta",
    "canastera": "canastero",
    "canasteras": "canastero",
    "canasteros": "canastero",
    "canastillas": "canastilla",
    "canastillos": "canastillo",
    "canastos": "canasto",
    "cancaguas": "cancagua",
    "cancamusas": "cancamusa",
    "cancanea": "cancanear",
    "cancaneaba": "cancanear",
    "cancaneabais": "cancanear",
    "cancaneaban": "cancanear",
    "cancaneabas": "cancanear",
    "cancanead": "cancanear",
    "cancaneada": "cancanear",
    "cancaneadas": "cancanear",
    "cancaneado": "cancanear",
    "cancaneados": "cancanear",
    "cancaneamos": "cancanear",
    "cancanean": "cancanear",
    "cancaneando": "cancanear",
    "cancaneara": "cancanear",
    "cancanearais": "cancanear",
    "cancanearan": "cancanear",
    "cancanearas": "cancanear",
    "cancaneare": "cancanear",
    "cancaneareis": "cancanear",
    "cancanearemos": "cancanear",
    "cancanearen": "cancanear",
    "cancaneares": "cancanear",
    "cancanearon": "cancanear",
    "cancaneará": "cancanear",
    "cancanearán": "cancanear",
    "cancanearás": "cancanear",
    "cancanearé": "cancanear",
    "cancanearéis": "cancanear",
    "cancanearía": "cancanear",
    "cancanearíais": "cancanear",
    "cancanearíamos": "cancanear",
    "cancanearían": "cancanear",
    "cancanearías": "cancanear",
    "cancaneas": "cancanear",
    "cancanease": "cancanear",
    "cancaneaseis": "cancanear",
    "cancaneasen": "cancanear",
    "cancaneases": "cancanear",
    "cancaneaste": "cancanear",
    "cancaneasteis": "cancanear",
    "cancanee": "cancanear",
    "cancaneemos": "cancanear",
    "cancaneen": "cancanear",
    "cancanees": "cancanear",
    "cancaneo": "cancanear",
    "cancanes": "cancán",
    "cancaneábamos": "cancanear",
    "cancaneáis": "cancanear",
    "cancaneáramos": "cancanear",
    "cancaneáremos": "cancanear",
    "cancaneásemos": "cancanear",
    "cancaneé": "cancanear",
    "cancaneéis": "cancanear",
    "cancaneó": "cancanear",
    "cancela": "cancelar",
    "cancelaba": "cancelar",
    "cancelabais": "cancelar",
    "cancelaban": "cancelar",
    "cancelabas": "cancelar",
    "cancelaciones": "cancelación",
    "cancelad": "cancelar",
    "cancelada": "cancelar",
    "canceladas": "cancelar",
    "cancelado": "cancelar",
    "canceladoras": "canceladora",
    "cancelados": "cancelar",
    "cancelamos": "cancelar",
    "cancelan": "cancelar",
    "cancelando": "cancelar",
    "cancelara": "cancelar",
    "cancelarais": "cancelar",
    "cancelaran": "cancelar",
    "cancelaras": "cancelar",
    "cancelare": "cancelar",
    "cancelareis": "cancelar",
    "cancelaremos": "cancelar",
    "cancelaren": "cancelar",
    "cancelares": "cancelar",
    "cancelarios": "cancelario",
    "cancelaron": "cancelar",
    "cancelará": "cancelar",
    "cancelarán": "cancelar",
    "cancelarás": "cancelar",
    "cancelaré": "cancelar",
    "cancelaréis": "cancelar",
    "cancelaría": "cancelar",
    "cancelaríais": "cancelar",
    "cancelaríamos": "cancelar",
    "cancelarían": "cancelar",
    "cancelarías": "cancelaría",
    "cancelas": "cancelar",
    "cancelase": "cancelar",
    "cancelaseis": "cancelar",
    "cancelasen": "cancelar",
    "cancelases": "cancelar",
    "cancelaste": "cancelar",
    "cancelasteis": "cancelar",
    "cancele": "cancelar",
    "cancelemos": "cancelar",
    "cancelen": "cancelar",
    "cancelerías": "cancelería",
    "canceles": "cancelar",
    "cancellerías": "cancellería",
    "cancelo": "cancelar",
    "cancelábamos": "cancelar",
    "canceláis": "cancelar",
    "canceláramos": "cancelar",
    "canceláremos": "cancelar",
    "cancelásemos": "cancelar",
    "cancelé": "cancelar",
    "canceléis": "cancelar",
    "canceló": "cancelar",
    "cancera": "cancerar",
    "canceraba": "cancerar",
    "cancerabais": "cancerar",
    "canceraban": "cancerar",
    "cancerabas": "cancerar",
    "cancerad": "cancerar",
    "cancerada": "cancerar",
    "canceradas": "cancerar",
    "cancerado": "cancerar",
    "cancerados": "cancerar",
    "canceramos": "cancerar",
    "canceran": "cancerar",
    "cancerando": "cancerar",
    "cancerara": "cancerar",
    "cancerarais": "cancerar",
    "canceraran": "cancerar",
    "canceraras": "cancerar",
    "cancerare": "cancerar",
    "cancerareis": "cancerar",
    "canceraremos": "cancerar",
    "canceraren": "cancerar",
    "cancerares": "cancerar",
    "canceraron": "cancerar",
    "cancerará": "cancerar",
    "cancerarán": "cancerar",
    "cancerarás": "cancerar",
    "canceraré": "cancerar",
    "canceraréis": "cancerar",
    "canceraría": "cancerar",
    "canceraríais": "cancerar",
    "canceraríamos": "cancerar",
    "cancerarían": "cancerar",
    "cancerarías": "cancerar",
    "canceras": "cancerar",
    "cancerase": "cancerar",
    "canceraseis": "cancerar",
    "cancerasen": "cancerar",
    "cancerases": "cancerar",
    "canceraste": "cancerar",
    "cancerasteis": "cancerar",
    "cancerbera": "cancerbero",
    "cancerberas": "cancerbero",
    "cancerberos": "cancerbero",
    "cancere": "cancerar",
    "canceremos": "cancerar",
    "canceren": "cancerar",
    "canceres": "cancerar",
    "canceriformes": "canceriforme",
    "cancero": "cancerar",
    "cancerosa": "canceroso",
    "cancerosas": "canceroso",
    "cancerosos": "canceroso",
    "cancerábamos": "cancerar",
    "canceráis": "cancerar",
    "canceráramos": "cancerar",
    "canceráremos": "cancerar",
    "cancerásemos": "cancerar",
    "canceré": "cancerar",
    "canceréis": "cancerar",
    "cancerígena": "cancerígeno",
    "cancerígenas": "cancerígeno",
    "cancerígenos": "cancerígeno",
    "cancerínica": "cancerínico",
    "cancerínicas": "cancerínico",
    "cancerínicos": "cancerínico",
    "canceró": "cancerar",
    "canchales": "canchal",
    "canchas": "cancha",
    "canchea": "canchear",
    "cancheaba": "canchear",
    "cancheabais": "canchear",
    "cancheaban": "canchear",
    "cancheabas": "canchear",
    "canchead": "canchear",
    "cancheada": "canchear",
    "cancheadas": "canchear",
    "cancheado": "canchear",
    "cancheados": "canchear",
    "cancheamos": "canchear",
    "canchean": "canchear",
    "cancheando": "canchear",
    "cancheara": "canchear",
    "canchearais": "canchear",
    "canchearan": "canchear",
    "canchearas": "canchear",
    "cancheare": "canchear",
    "cancheareis": "canchear",
    "canchearemos": "canchear",
    "canchearen": "canchear",
    "cancheares": "canchear",
    "canchearon": "canchear",
    "cancheará": "canchear",
    "canchearán": "canchear",
    "canchearás": "canchear",
    "canchearé": "canchear",
    "canchearéis": "canchear",
    "canchearía": "canchear",
    "canchearíais": "canchear",
    "canchearíamos": "canchear",
    "canchearían": "canchear",
    "canchearías": "canchear",
    "cancheas": "canchear",
    "canchease": "canchear",
    "cancheaseis": "canchear",
    "cancheasen": "canchear",
    "cancheases": "canchear",
    "cancheaste": "canchear",
    "cancheasteis": "canchear",
    "canchee": "canchear",
    "cancheemos": "canchear",
    "cancheen": "canchear",
    "canchees": "canchear",
    "cancheo": "canchear",
    "cancheos": "cancheo",
    "canchera": "canchero",
    "cancheras": "canchero",
    "cancheros": "canchero",
    "cancheábamos": "canchear",
    "cancheáis": "canchear",
    "cancheáramos": "canchear",
    "cancheáremos": "canchear",
    "cancheásemos": "canchear",
    "cancheé": "canchear",
    "cancheéis": "canchear",
    "cancheó": "canchear",
    "canchos": "cancho",
    "cancillas": "cancilla",
    "cancilleres": "canciller",
    "cancilleresca": "cancilleresco",
    "cancillerescas": "cancilleresco",
    "cancillerescos": "cancilleresco",
    "cancillerías": "cancillería",
    "cancina": "cancín",
    "cancinas": "cancín",
    "cancines": "cancín",
    "cancioncillas": "cancioncilla",
    "cancioneriles": "cancioneril",
    "cancioneros": "cancionero",
    "canciones": "canción",
    "cancionistas": "cancionista",
    "cancones": "cancón",
    "cancrinitas": "cancrinita",
    "cancroides": "cancroide",
    "cancros": "cancro",
    "canda": "candar",
    "candaba": "candar",
    "candabais": "candar",
    "candaban": "candar",
    "candabas": "candar",
    "candad": "candar",
    "candada": "candar",
    "candadas": "candar",
    "candado": "candar",
    "candados": "candar",
    "candaletas": "candaleta",
    "candalizas": "candaliza",
    "candamos": "candar",
    "candan": "candar",
    "candando": "candar",
    "candara": "candar",
    "candarais": "candar",
    "candaran": "candar",
    "candaras": "candar",
    "candare": "candar",
    "candareis": "candar",
    "candaremos": "candar",
    "candaren": "candar",
    "candares": "candar",
    "candaron": "candar",
    "candará": "candar",
    "candarán": "candar",
    "candarás": "candar",
    "candaré": "candar",
    "candaréis": "candar",
    "candaría": "candar",
    "candaríais": "candar",
    "candaríamos": "candar",
    "candarían": "candar",
    "candarías": "candar",
    "candas": "candar",
    "candase": "candar",
    "candaseis": "candar",
    "candasen": "candar",
    "candases": "candar",
    "candaste": "candar",
    "candasteis": "candar",
    "cande": "candar",
    "candeales": "candeal",
    "candelabros": "candelabro",
    "candelas": "candela",
    "candelechos": "candelecho",
    "candeleros": "candelero",
    "candelillas": "candelilla",
    "candelizos": "candelizo",
    "candemos": "candar",
    "canden": "candar",
    "candencias": "candencia",
    "candentes": "candente",
    "candes": "cande",
    "candidaciones": "candidación",
    "candidata": "candidato",
    "candidatas": "candidato",
    "candidatos": "candidato",
    "candidaturas": "candidatura",
    "candideces": "candidez",
    "candieles": "candiel",
    "candies": "candy",
    "candiladas": "candilada",
    "candilejas": "candileja",
    "candiles": "candil",
    "candiotas": "candiota",
    "candioteras": "candiotera",
    "candis": "candi",
    "cando": "candar",
    "candombes": "candombe",
    "candonga": "candongo",
    "candongas": "candongo",
    "candongos": "candongo",
    "candonguea": "candonguear",
    "candongueaba": "candonguear",
    "candongueabais": "candonguear",
    "candongueaban": "candonguear",
    "candongueabas": "candonguear",
    "candonguead": "candonguear",
    "candongueada": "candonguear",
    "candongueadas": "candonguear",
    "candongueado": "candonguear",
    "candongueados": "candonguear",
    "candongueamos": "candonguear",
    "candonguean": "candonguear",
    "candongueando": "candonguear",
    "candongueara": "candonguear",
    "candonguearais": "candonguear",
    "candonguearan": "candonguear",
    "candonguearas": "candonguear",
    "candongueare": "candonguear",
    "candongueareis": "candonguear",
    "candonguearemos": "candonguear",
    "candonguearen": "candonguear",
    "candongueares": "candonguear",
    "candonguearon": "candonguear",
    "candongueará": "candonguear",
    "candonguearán": "candonguear",
    "candonguearás": "candonguear",
    "candonguearé": "candonguear",
    "candonguearéis": "candonguear",
    "candonguearía": "candonguear",
    "candonguearíais": "candonguear",
    "candonguearíamos": "candonguear",
    "candonguearían": "candonguear",
    "candonguearías": "candonguear",
    "candongueas": "candonguear",
    "candonguease": "candonguear",
    "candongueaseis": "candonguear",
    "candongueasen": "candonguear",
    "candongueases": "candonguear",
    "candongueaste": "candonguear",
    "candongueasteis": "candonguear",
    "candonguee": "candonguear",
    "candongueemos": "candonguear",
    "candongueen": "candonguear",
    "candonguees": "candonguear",
    "candongueo": "candonguear",
    "candongueábamos": "candonguear",
    "candongueáis": "candonguear",
    "candongueáramos": "candonguear",
    "candongueáremos": "candonguear",
    "candongueásemos": "candonguear",
    "candongueé": "candonguear",
    "candongueéis": "candonguear",
    "candongueó": "candonguear",
    "candores": "candor",
    "candorosa": "candoroso",
    "candorosas": "candoroso",
    "candorosos": "candoroso",
    "candrayes": "candray",
    "candábamos": "candar",
    "candáis": "candar",
    "candáramos": "candar",
    "candáremos": "candar",
    "candásemos": "candar",
    "candé": "candar",
    "candéis": "candar",
    "candó": "candar",
    "canecas": "caneca",
    "canecos": "caneco",
    "canela": "canelo",
    "canelas": "canelo",
    "canelones": "canelón",
    "canelos": "canelo",
    "canes": "can",
    "canesús": "canesú",
    "canesúses": "canesú",
    "canfanos": "canfano",
    "canfenos": "canfeno",
    "canfosulfonatos": "canfosulfonato",
    "canfórica": "canfórico",
    "canfóricas": "canfórico",
    "canfóricos": "canfórico",
    "cangallas": "cangalla",
    "cangas": "canga",
    "cangilones": "cangilón",
    "cangrejas": "cangreja",
    "cangrejos": "cangrejo",
    "cangrena": "cangrenar",
    "cangrenaba": "cangrenar",
    "cangrenabais": "cangrenar",
    "cangrenaban": "cangrenar",
    "cangrenabas": "cangrenar",
    "cangrenad": "cangrenar",
    "cangrenada": "cangrenar",
    "cangrenadas": "cangrenar",
    "cangrenado": "cangrenar",
    "cangrenados": "cangrenar",
    "cangrenamos": "cangrenar",
    "cangrenan": "cangrenar",
    "cangrenando": "cangrenar",
    "cangrenara": "cangrenar",
    "cangrenarais": "cangrenar",
    "cangrenaran": "cangrenar",
    "cangrenaras": "cangrenar",
    "cangrenare": "cangrenar",
    "cangrenareis": "cangrenar",
    "cangrenaremos": "cangrenar",
    "cangrenaren": "cangrenar",
    "cangrenares": "cangrenar",
    "cangrenaron": "cangrenar",
    "cangrenará": "cangrenar",
    "cangrenarán": "cangrenar",
    "cangrenarás": "cangrenar",
    "cangrenaré": "cangrenar",
    "cangrenaréis": "cangrenar",
    "cangrenaría": "cangrenar",
    "cangrenaríais": "cangrenar",
    "cangrenaríamos": "cangrenar",
    "cangrenarían": "cangrenar",
    "cangrenarías": "cangrenar",
    "cangrenas": "cangrenar",
    "cangrenase": "cangrenar",
    "cangrenaseis": "cangrenar",
    "cangrenasen": "cangrenar",
    "cangrenases": "cangrenar",
    "cangrenaste": "cangrenar",
    "cangrenasteis": "cangrenar",
    "cangrene": "cangrenar",
    "cangrenemos": "cangrenar",
    "cangrenen": "cangrenar",
    "cangrenes": "cangrenar",
    "cangreno": "cangrenar",
    "cangrenábamos": "cangrenar",
    "cangrenáis": "cangrenar",
    "cangrenáramos": "cangrenar",
    "cangrenáremos": "cangrenar",
    "cangrenásemos": "cangrenar",
    "cangrené": "cangrenar",
    "cangrenéis": "cangrenar",
    "cangrenó": "cangrenar",
    "canguelos": "canguelo",
    "canguros": "canguro",
    "canibalismos": "canibalismo",
    "canicas": "canica",
    "caniches": "caniche",
    "canicies": "canicie",
    "caniculares": "canicular",
    "canija": "canijo",
    "canijas": "canijo",
    "canijos": "canijo",
    "canillas": "canilla",
    "canillera": "canillero",
    "canilleras": "canillero",
    "canilleros": "canillero",
    "canina": "canino",
    "caninas": "canino",
    "canineces": "caninez",
    "caniniformes": "caniniforme",
    "caninos": "canino",
    "caniza": "canizo",
    "canizas": "canizo",
    "canizos": "canizo",
    "canjea": "canjear",
    "canjeaba": "canjear",
    "canjeabais": "canjear",
    "canjeaban": "canjear",
    "canjeabas": "canjear",
    "canjeables": "canjeable",
    "canjead": "canjear",
    "canjeada": "canjear",
    "canjeadas": "canjear",
    "canjeado": "canjear",
    "canjeados": "canjear",
    "canjeamos": "canjear",
    "canjean": "canjear",
    "canjeando": "canjear",
    "canjeara": "canjear",
    "canjearais": "canjear",
    "canjearan": "canjear",
    "canjearas": "canjear",
    "canjeare": "canjear",
    "canjeareis": "canjear",
    "canjearemos": "canjear",
    "canjearen": "canjear",
    "canjeares": "canjear",
    "canjearon": "canjear",
    "canjeará": "canjear",
    "canjearán": "canjear",
    "canjearás": "canjear",
    "canjearé": "canjear",
    "canjearéis": "canjear",
    "canjearía": "canjear",
    "canjearíais": "canjear",
    "canjearíamos": "canjear",
    "canjearían": "canjear",
    "canjearías": "canjear",
    "canjeas": "canjear",
    "canjease": "canjear",
    "canjeaseis": "canjear",
    "canjeasen": "canjear",
    "canjeases": "canjear",
    "canjeaste": "canjear",
    "canjeasteis": "canjear",
    "canjee": "canjear",
    "canjeemos": "canjear",
    "canjeen": "canjear",
    "canjees": "canjear",
    "canjeo": "canjear",
    "canjes": "canje",
    "canjeábamos": "canjear",
    "canjeáis": "canjear",
    "canjeáramos": "canjear",
    "canjeáremos": "canjear",
    "canjeásemos": "canjear",
    "canjeé": "canjear",
    "canjeéis": "canjear",
    "canjeó": "canjear",
    "cannabáceas": "cannabácea",
    "cannáceas": "cannácea",
    "canoas": "canoa",
    "canoera": "canoero",
    "canoeras": "canoero",
    "canoeros": "canoero",
    "canonesas": "canonesa",
    "canonicales": "canonical",
    "canonicatos": "canonicato",
    "canonice": "canonizar",
    "canonicemos": "canonizar",
    "canonicen": "canonizar",
    "canonices": "canonizar",
    "canonicé": "canonizar",
    "canonicéis": "canonizar",
    "canonistas": "canonista",
    "canoniza": "canonizar",
    "canonizaba": "canonizar",
    "canonizabais": "canonizar",
    "canonizaban": "canonizar",
    "canonizabas": "canonizar",
    "canonizaciones": "canonización",
    "canonizad": "canonizar",
    "canonizada": "canonizar",
    "canonizadas": "canonizar",
    "canonizado": "canonizar",
    "canonizados": "canonizar",
    "canonizamos": "canonizar",
    "canonizan": "canonizar",
    "canonizando": "canonizar",
    "canonizara": "canonizar",
    "canonizarais": "canonizar",
    "canonizaran": "canonizar",
    "canonizaras": "canonizar",
    "canonizare": "canonizar",
    "canonizareis": "canonizar",
    "canonizaremos": "canonizar",
    "canonizaren": "canonizar",
    "canonizares": "canonizar",
    "canonizaron": "canonizar",
    "canonizará": "canonizar",
    "canonizarán": "canonizar",
    "canonizarás": "canonizar",
    "canonizaré": "canonizar",
    "canonizaréis": "canonizar",
    "canonizaría": "canonizar",
    "canonizaríais": "canonizar",
    "canonizaríamos": "canonizar",
    "canonizarían": "canonizar",
    "canonizarías": "canonizar",
    "canonizas": "canonizar",
    "canonizase": "canonizar",
    "canonizaseis": "canonizar",
    "canonizasen": "canonizar",
    "canonizases": "canonizar",
    "canonizaste": "canonizar",
    "canonizasteis": "canonizar",
    "canonizo": "canonizar",
    "canonizábamos": "canonizar",
    "canonizáis": "canonizar",
    "canonizáramos": "canonizar",
    "canonizáremos": "canonizar",
    "canonizásemos": "canonizar",
    "canonizó": "canonizar",
    "canonjías": "canonjía",
    "canopes": "canope",
    "canora": "canoro",
    "canoras": "canoro",
    "canoros": "canoro",
    "canos": "caño",
    "canosa": "canoso",
    "canosas": "canoso",
    "canosos": "canoso",
    "canotajes": "canotaje",
    "canoístas": "canoísta",
    "cansa": "cansar",
    "cansaba": "cansar",
    "cansabais": "cansar",
    "cansaban": "cansar",
    "cansabas": "cansar",
    "cansad": "cansar",
    "cansada": "cansar",
    "cansadas": "cansar",
    "cansado": "cansar",
    "cansadora": "cansador",
    "cansadoras": "cansador",
    "cansadores": "cansador",
    "cansados": "cansar",
    "cansamos": "cansar",
    "cansan": "cansar",
    "cansancioos": "cansancio",
    "cansancios": "cansancio",
    "cansando": "cansar",
    "cansara": "cansar",
    "cansarais": "cansar",
    "cansaran": "cansar",
    "cansaras": "cansar",
    "cansare": "cansar",
    "cansareis": "cansar",
    "cansaremos": "cansar",
    "cansaren": "cansar",
    "cansares": "cansar",
    "cansaron": "cansar",
    "cansará": "cansar",
    "cansarán": "cansar",
    "cansarás": "cansar",
    "cansaré": "cansar",
    "cansaréis": "cansar",
    "cansaría": "cansar",
    "cansaríais": "cansar",
    "cansaríamos": "cansar",
    "cansarían": "cansar",
    "cansarías": "cansar",
    "cansas": "cansar",
    "cansase": "cansar",
    "cansaseis": "cansar",
    "cansasen": "cansar",
    "cansases": "cansar",
    "cansaste": "cansar",
    "cansasteis": "cansar",
    "canse": "cansar",
    "cansemos": "cansar",
    "cansen": "cansar",
    "canses": "cansar",
    "cansina": "cansino",
    "cansinas": "cansino",
    "cansinos": "cansino",
    "canso": "cansar",
    "cansábamos": "cansar",
    "cansáis": "cansar",
    "cansáramos": "cansar",
    "cansáremos": "cansar",
    "cansásemos": "cansar",
    "cansé": "cansar",
    "canséis": "cansar",
    "cansó": "cansar",
    "canta": "cantar",
    "cantaba": "cantar",
    "cantabais": "cantar",
    "cantaban": "cantar",
    "cantabas": "cantar",
    "cantables": "cantable",
    "cantad": "cantar",
    "cantada": "cantar",
    "cantadas": "cantar",
    "cantado": "cantar",
    "cantadora": "cantador",
    "cantadoras": "cantador",
    "cantadores": "cantador",
    "cantados": "cantar",
    "cantales": "cantal",
    "cantaletas": "cantaleta",
    "cantaletea": "cantaletear",
    "cantaleteaba": "cantaletear",
    "cantaleteabais": "cantaletear",
    "cantaleteaban": "cantaletear",
    "cantaleteabas": "cantaletear",
    "cantaletead": "cantaletear",
    "cantaleteada": "cantaletear",
    "cantaleteadas": "cantaletear",
    "cantaleteado": "cantaletear",
    "cantaleteados": "cantaletear",
    "cantaleteamos": "cantaletear",
    "cantaletean": "cantaletear",
    "cantaleteando": "cantaletear",
    "cantaleteara": "cantaletear",
    "cantaletearais": "cantaletear",
    "cantaletearan": "cantaletear",
    "cantaletearas": "cantaletear",
    "cantaleteare": "cantaletear",
    "cantaleteareis": "cantaletear",
    "cantaletearemos": "cantaletear",
    "cantaletearen": "cantaletear",
    "cantaleteares": "cantaletear",
    "cantaletearon": "cantaletear",
    "cantaleteará": "cantaletear",
    "cantaletearán": "cantaletear",
    "cantaletearás": "cantaletear",
    "cantaletearé": "cantaletear",
    "cantaletearéis": "cantaletear",
    "cantaletearía": "cantaletear",
    "cantaletearíais": "cantaletear",
    "cantaletearíamos": "cantaletear",
    "cantaletearían": "cantaletear",
    "cantaletearías": "cantaletear",
    "cantaleteas": "cantaletear",
    "cantaletease": "cantaletear",
    "cantaleteaseis": "cantaletear",
    "cantaleteasen": "cantaletear",
    "cantaleteases": "cantaletear",
    "cantaleteaste": "cantaletear",
    "cantaleteasteis": "cantaletear",
    "cantaletee": "cantaletear",
    "cantaleteemos": "cantaletear",
    "cantaleteen": "cantaletear",
    "cantaletees": "cantaletear",
    "cantaleteo": "cantaletear",
    "cantaleteábamos": "cantaletear",
    "cantaleteáis": "cantaletear",
    "cantaleteáramos": "cantaletear",
    "cantaleteáremos": "cantaletear",
    "cantaleteásemos": "cantaletear",
    "cantaleteé": "cantaletear",
    "cantaleteéis": "cantaletear",
    "cantaleteó": "cantaletear",
    "cantamos": "cantar",
    "cantan": "cantar",
    "cantando": "cantar",
    "cantantes": "cantante",
    "cantaora": "cantaor",
    "cantaoras": "cantaor",
    "cantaores": "cantaor",
    "cantara": "cantar",
    "cantarais": "cantar",
    "cantaran": "cantar",
    "cantaras": "cantar",
    "cantare": "cantar",
    "cantareis": "cantar",
    "cantarelas": "cantarela",
    "cantaremos": "cantar",
    "cantaren": "cantar",
    "cantareras": "cantarera",
    "cantares": "cantar",
    "cantarillas": "cantarilla",
    "cantarina": "cantarín",
    "cantarinas": "cantarín",
    "cantarines": "cantarín",
    "cantaron": "cantar",
    "cantará": "cantar",
    "cantarán": "cantar",
    "cantarás": "cantar",
    "cantaré": "cantar",
    "cantaréis": "cantar",
    "cantaría": "cantar",
    "cantaríais": "cantar",
    "cantaríamos": "cantar",
    "cantarían": "cantar",
    "cantarías": "cantar",
    "cantas": "cantar",
    "cantase": "cantar",
    "cantaseis": "cantar",
    "cantasen": "cantar",
    "cantases": "cantar",
    "cantaste": "cantar",
    "cantasteis": "cantar",
    "cantatas": "cantata",
    "cantatrices": "cantatriz",
    "cantautora": "cantautor",
    "cantautoras": "cantautor",
    "cantautores": "cantautor",
    "cantazos": "cantazo",
    "cante": "cantar",
    "cantea": "cantear",
    "canteaba": "cantear",
    "canteabais": "cantear",
    "canteaban": "cantear",
    "canteabas": "cantear",
    "cantead": "cantear",
    "canteada": "cantear",
    "canteadas": "cantear",
    "canteado": "cantear",
    "canteados": "cantear",
    "canteamos": "cantear",
    "cantean": "cantear",
    "canteando": "cantear",
    "canteara": "cantear",
    "cantearais": "cantear",
    "cantearan": "cantear",
    "cantearas": "cantear",
    "canteare": "cantear",
    "canteareis": "cantear",
    "cantearemos": "cantear",
    "cantearen": "cantear",
    "canteares": "cantear",
    "cantearon": "cantear",
    "canteará": "cantear",
    "cantearán": "cantear",
    "cantearás": "cantear",
    "cantearé": "cantear",
    "cantearéis": "cantear",
    "cantearía": "cantear",
    "cantearíais": "cantear",
    "cantearíamos": "cantear",
    "cantearían": "cantear",
    "cantearías": "cantear",
    "canteas": "cantear",
    "cantease": "cantear",
    "canteaseis": "cantear",
    "canteasen": "cantear",
    "canteases": "cantear",
    "canteaste": "cantear",
    "canteasteis": "cantear",
    "cantee": "cantear",
    "canteemos": "cantear",
    "canteen": "cantear",
    "cantees": "cantear",
    "canteles": "cantel",
    "cantemos": "cantar",
    "canten": "cantar",
    "canteo": "cantear",
    "canterana": "canterano",
    "canteranas": "canterano",
    "canteranos": "canterano",
    "canteras": "cantera",
    "canterios": "canterio",
    "canteritos": "canterito",
    "canteros": "cantero",
    "canterías": "cantería",
    "cantes": "cante",
    "canteábamos": "cantear",
    "canteáis": "cantear",
    "canteáramos": "cantear",
    "canteáremos": "cantear",
    "canteásemos": "cantear",
    "canteé": "cantear",
    "canteéis": "cantear",
    "canteó": "cantear",
    "canticios": "canticio",
    "cantidades": "cantidad",
    "cantigas": "cantiga",
    "cantilenas": "cantilena",
    "cantiles": "cantil",
    "cantimploras": "cantimplora",
    "cantinas": "cantina",
    "cantinelas": "cantinela",
    "cantinera": "cantinero",
    "cantineras": "cantinero",
    "cantineros": "cantinero",
    "cantizales": "cantizal",
    "canto": "cantar",
    "cantogramas": "cantograma",
    "cantollanistas": "cantollanista",
    "cantona": "cantonar",
    "cantonaba": "cantonar",
    "cantonabais": "cantonar",
    "cantonaban": "cantonar",
    "cantonabas": "cantonar",
    "cantonad": "cantonar",
    "cantonada": "cantonar",
    "cantonadas": "cantonar",
    "cantonado": "cantonar",
    "cantonados": "cantonar",
    "cantonales": "cantonal",
    "cantonalismos": "cantonalismo",
    "cantonalistas": "cantonalista",
    "cantonamos": "cantonar",
    "cantonan": "cantonar",
    "cantonando": "cantonar",
    "cantonara": "cantonar",
    "cantonarais": "cantonar",
    "cantonaran": "cantonar",
    "cantonaras": "cantonar",
    "cantonare": "cantonar",
    "cantonareis": "cantonar",
    "cantonaremos": "cantonar",
    "cantonaren": "cantonar",
    "cantonares": "cantonar",
    "cantonaron": "cantonar",
    "cantonará": "cantonar",
    "cantonarán": "cantonar",
    "cantonarás": "cantonar",
    "cantonaré": "cantonar",
    "cantonaréis": "cantonar",
    "cantonaría": "cantonar",
    "cantonaríais": "cantonar",
    "cantonaríamos": "cantonar",
    "cantonarían": "cantonar",
    "cantonarías": "cantonar",
    "cantonas": "cantonar",
    "cantonase": "cantonar",
    "cantonaseis": "cantonar",
    "cantonasen": "cantonar",
    "cantonases": "cantonar",
    "cantonaste": "cantonar",
    "cantonasteis": "cantonar",
    "cantone": "cantonar",
    "cantonea": "cantonear",
    "cantoneaba": "cantonear",
    "cantoneabais": "cantonear",
    "cantoneaban": "cantonear",
    "cantoneabas": "cantonear",
    "cantonead": "cantonear",
    "cantoneada": "cantonear",
    "cantoneadas": "cantonear",
    "cantoneado": "cantonear",
    "cantoneados": "cantonear",
    "cantoneamos": "cantonear",
    "cantonean": "cantonear",
    "cantoneando": "cantonear",
    "cantoneara": "cantonear",
    "cantonearais": "cantonear",
    "cantonearan": "cantonear",
    "cantonearas": "cantonear",
    "cantoneare": "cantonear",
    "cantoneareis": "cantonear",
    "cantonearemos": "cantonear",
    "cantonearen": "cantonear",
    "cantoneares": "cantonear",
    "cantonearon": "cantonear",
    "cantoneará": "cantonear",
    "cantonearán": "cantonear",
    "cantonearás": "cantonear",
    "cantonearé": "cantonear",
    "cantonearéis": "cantonear",
    "cantonearía": "cantonear",
    "cantonearíais": "cantonear",
    "cantonearíamos": "cantonear",
    "cantonearían": "cantonear",
    "cantonearías": "cantonear",
    "cantoneas": "cantonear",
    "cantonease": "cantonear",
    "cantoneaseis": "cantonear",
    "cantoneasen": "cantonear",
    "cantoneases": "cantonear",
    "cantoneaste": "cantonear",
    "cantoneasteis": "cantonear",
    "cantonee": "cantonear",
    "cantoneemos": "cantonear",
    "cantoneen": "cantonear",
    "cantonees": "cantonear",
    "cantonemos": "cantonar",
    "cantonen": "cantonar",
    "cantoneo": "cantonear",
    "cantoneos": "cantoneo",
    "cantonera": "cantonero",
    "cantoneras": "cantonero",
    "cantoneros": "cantonero",
    "cantones": "cantonar",
    "cantonesa": "cantonés",
    "cantonesas": "cantonés",
    "cantoneses": "cantonés",
    "cantoneábamos": "cantonear",
    "cantoneáis": "cantonear",
    "cantoneáramos": "cantonear",
    "cantoneáremos": "cantonear",
    "cantoneásemos": "cantonear",
    "cantoneé": "cantonear",
    "cantoneéis": "cantonear",
    "cantoneó": "cantonear",
    "cantono": "cantonar",
    "cantonábamos": "cantonar",
    "cantonáis": "cantonar",
    "cantonáramos": "cantonar",
    "cantonáremos": "cantonar",
    "cantonásemos": "cantonar",
    "cantoné": "cantonar",
    "cantonéis": "cantonar",
    "cantonó": "cantonar",
    "cantora": "cantor",
    "cantorales": "cantoral",
    "cantoras": "cantor",
    "cantores": "cantor",
    "cantorrales": "cantorral",
    "cantorías": "cantoría",
    "cantos": "canto",
    "cantuesos": "cantueso",
    "canturrales": "canturral",
    "canturrea": "canturrear",
    "canturreaba": "canturrear",
    "canturreabais": "canturrear",
    "canturreaban": "canturrear",
    "canturreabas": "canturrear",
    "canturread": "canturrear",
    "canturreada": "canturrear",
    "canturreadas": "canturrear",
    "canturreado": "canturrear",
    "canturreados": "canturrear",
    "canturreamos": "canturrear",
    "canturrean": "canturrear",
    "canturreando": "canturrear",
    "canturreara": "canturrear",
    "canturrearais": "canturrear",
    "canturrearan": "canturrear",
    "canturrearas": "canturrear",
    "canturreare": "canturrear",
    "canturreareis": "canturrear",
    "canturrearemos": "canturrear",
    "canturrearen": "canturrear",
    "canturreares": "canturrear",
    "canturrearon": "canturrear",
    "canturreará": "canturrear",
    "canturrearán": "canturrear",
    "canturrearás": "canturrear",
    "canturrearé": "canturrear",
    "canturrearéis": "canturrear",
    "canturrearía": "canturrear",
    "canturrearíais": "canturrear",
    "canturrearíamos": "canturrear",
    "canturrearían": "canturrear",
    "canturrearías": "canturrear",
    "canturreas": "canturrear",
    "canturrease": "canturrear",
    "canturreaseis": "canturrear",
    "canturreasen": "canturrear",
    "canturreases": "canturrear",
    "canturreaste": "canturrear",
    "canturreasteis": "canturrear",
    "canturree": "canturrear",
    "canturreemos": "canturrear",
    "canturreen": "canturrear",
    "canturrees": "canturrear",
    "canturreo": "canturrear",
    "canturreos": "canturreo",
    "canturreábamos": "canturrear",
    "canturreáis": "canturrear",
    "canturreáramos": "canturrear",
    "canturreáremos": "canturrear",
    "canturreásemos": "canturrear",
    "canturreé": "canturrear",
    "canturreéis": "canturrear",
    "canturreó": "canturrear",
    "canturria": "canturriar",
    "canturriaba": "canturriar",
    "canturriabais": "canturriar",
    "canturriaban": "canturriar",
    "canturriabas": "canturriar",
    "canturriad": "canturriar",
    "canturriada": "canturriar",
    "canturriadas": "canturriar",
    "canturriado": "canturriar",
    "canturriados": "canturriar",
    "canturriamos": "canturriar",
    "canturrian": "canturriar",
    "canturriando": "canturriar",
    "canturriara": "canturriar",
    "canturriarais": "canturriar",
    "canturriaran": "canturriar",
    "canturriaras": "canturriar",
    "canturriare": "canturriar",
    "canturriareis": "canturriar",
    "canturriaremos": "canturriar",
    "canturriaren": "canturriar",
    "canturriares": "canturriar",
    "canturriaron": "canturriar",
    "canturriará": "canturriar",
    "canturriarán": "canturriar",
    "canturriarás": "canturriar",
    "canturriaré": "canturriar",
    "canturriaréis": "canturriar",
    "canturriaría": "canturriar",
    "canturriaríais": "canturriar",
    "canturriaríamos": "canturriar",
    "canturriarían": "canturriar",
    "canturriarías": "canturriar",
    "canturrias": "canturriar",
    "canturriase": "canturriar",
    "canturriaseis": "canturriar",
    "canturriasen": "canturriar",
    "canturriases": "canturriar",
    "canturriaste": "canturriar",
    "canturriasteis": "canturriar",
    "canturrie": "canturriar",
    "canturriemos": "canturriar",
    "canturrien": "canturriar",
    "canturries": "canturriar",
    "canturrio": "canturriar",
    "canturriábamos": "canturriar",
    "canturriáis": "canturriar",
    "canturriáramos": "canturriar",
    "canturriáremos": "canturriar",
    "canturriásemos": "canturriar",
    "canturrié": "canturriar",
    "canturriéis": "canturriar",
    "canturrió": "canturriar",
    "canturías": "canturía",
    "cantábamos": "cantar",
    "cantábrica": "cantábrico",
    "cantábricas": "cantábrico",
    "cantábricos": "cantábrico",
    "cantáis": "cantar",
    "cantáramos": "cantar",
    "cantáremos": "cantar",
    "cantáridas": "cantárida",
    "cantásemos": "cantar",
    "canté": "cantar",
    "cantéis": "cantar",
    "cantó": "cantar",
    "canulares": "canular",
    "canutillos": "canutillo",
    "canutos": "canuto",
    "canéforas": "canéfora",
    "canés": "cané",
    "caníbales": "caníbal",
    "canículas": "canícula",
    "canódromos": "canódromo",
    "canónica": "canónico",
    "canónicas": "canónico",
    "canónicos": "canónico",
    "canónigas": "canóniga",
    "canónigos": "canónigo",
    "caobanas": "caobana",
    "caobas": "caoba",
    "caobos": "caobo",
    "caolines": "caolín",
    "caolinitas": "caolinita",
    "caolinizaciones": "caolinización",
    "cap": "capítulo",
    "capa": "capar",
    "capaba": "capar",
    "capabais": "capar",
    "capaban": "capar",
    "capabas": "capar",
    "capaces": "capaz",
    "capacetes": "capacete",
    "capachas": "capacha",
    "capachos": "capacho",
    "capacidades": "capacidad",
    "capacita": "capacitar",
    "capacitaba": "capacitar",
    "capacitabais": "capacitar",
    "capacitaban": "capacitar",
    "capacitabas": "capacitar",
    "capacitaciones": "capacitación",
    "capacitad": "capacitar",
    "capacitada": "capacitar",
    "capacitadas": "capacitar",
    "capacitado": "capacitar",
    "capacitados": "capacitar",
    "capacitamos": "capacitar",
    "capacitan": "capacitar",
    "capacitancias": "capacitancia",
    "capacitando": "capacitar",
    "capacitara": "capacitar",
    "capacitarais": "capacitar",
    "capacitaran": "capacitar",
    "capacitaras": "capacitar",
    "capacitare": "capacitar",
    "capacitareis": "capacitar",
    "capacitaremos": "capacitar",
    "capacitaren": "capacitar",
    "capacitares": "capacitar",
    "capacitaron": "capacitar",
    "capacitará": "capacitar",
    "capacitarán": "capacitar",
    "capacitarás": "capacitar",
    "capacitaré": "capacitar",
    "capacitaréis": "capacitar",
    "capacitaría": "capacitar",
    "capacitaríais": "capacitar",
    "capacitaríamos": "capacitar",
    "capacitarían": "capacitar",
    "capacitarías": "capacitar",
    "capacitas": "capacitar",
    "capacitase": "capacitar",
    "capacitaseis": "capacitar",
    "capacitasen": "capacitar",
    "capacitases": "capacitar",
    "capacitaste": "capacitar",
    "capacitasteis": "capacitar",
    "capacitativa": "capacitativo",
    "capacitativas": "capacitativo",
    "capacitativos": "capacitativo",
    "capacite": "capacitar",
    "capacitemos": "capacitar",
    "capaciten": "capacitar",
    "capacites": "capacitar",
    "capacitiva": "capacitivo",
    "capacitivas": "capacitivo",
    "capacitivos": "capacitivo",
    "capacito": "capacitar",
    "capacitábamos": "capacitar",
    "capacitáis": "capacitar",
    "capacitáramos": "capacitar",
    "capacitáremos": "capacitar",
    "capacitásemos": "capacitar",
    "capacité": "capacitar",
    "capacitéis": "capacitar",
    "capacitó": "capacitar",
    "capacitómetros": "capacitómetro",
    "capacímetros": "capacímetro",
    "capad": "capar",
    "capada": "capar",
    "capadas": "capar",
    "capado": "capar",
    "capadora": "capador",
    "capadoras": "capador",
    "capadores": "capador",
    "capados": "capar",
    "capaduras": "capadura",
    "capamos": "capar",
    "capan": "capar",
    "capando": "capar",
    "capara": "capar",
    "caparais": "capar",
    "caparan": "capar",
    "caparas": "capar",
    "caparazones": "caparazón",
    "caparazónes": "caparazón",
    "capare": "capar",
    "capareis": "capar",
    "caparemos": "capar",
    "caparen": "capar",
    "capares": "capar",
    "caparidáceas": "caparidácea",
    "caparon": "capar",
    "caparras": "caparra",
    "caparrones": "caparrón",
    "caparros": "caparro",
    "caparrosas": "caparrosa",
    "capará": "capar",
    "caparán": "capar",
    "caparás": "capar",
    "caparé": "capar",
    "caparéis": "capar",
    "caparía": "capar",
    "caparíais": "capar",
    "caparíamos": "capar",
    "caparían": "capar",
    "caparías": "capar",
    "capas": "capar",
    "capase": "capar",
    "capaseis": "capar",
    "capasen": "capar",
    "capases": "capar",
    "capaste": "capar",
    "capasteis": "capar",
    "capataces": "capataz",
    "capazes": "capaz",
    "capazones": "capazón",
    "capazos": "capazo",
    "capciona": "capcionar",
    "capcionaba": "capcionar",
    "capcionabais": "capcionar",
    "capcionaban": "capcionar",
    "capcionabas": "capcionar",
    "capcionad": "capcionar",
    "capcionada": "capcionar",
    "capcionadas": "capcionar",
    "capcionado": "capcionar",
    "capcionados": "capcionar",
    "capcionamos": "capcionar",
    "capcionan": "capcionar",
    "capcionando": "capcionar",
    "capcionara": "capcionar",
    "capcionarais": "capcionar",
    "capcionaran": "capcionar",
    "capcionaras": "capcionar",
    "capcionare": "capcionar",
    "capcionareis": "capcionar",
    "capcionaremos": "capcionar",
    "capcionaren": "capcionar",
    "capcionares": "capcionar",
    "capcionaron": "capcionar",
    "capcionará": "capcionar",
    "capcionarán": "capcionar",
    "capcionarás": "capcionar",
    "capcionaré": "capcionar",
    "capcionaréis": "capcionar",
    "capcionaría": "capcionar",
    "capcionaríais": "capcionar",
    "capcionaríamos": "capcionar",
    "capcionarían": "capcionar",
    "capcionarías": "capcionar",
    "capcionas": "capcionar",
    "capcionase": "capcionar",
    "capcionaseis": "capcionar",
    "capcionasen": "capcionar",
    "capcionases": "capcionar",
    "capcionaste": "capcionar",
    "capcionasteis": "capcionar",
    "capcione": "capcionar",
    "capcionemos": "capcionar",
    "capcionen": "capcionar",
    "capciones": "capcionar",
    "capciono": "capcionar",
    "capcionábamos": "capcionar",
    "capcionáis": "capcionar",
    "capcionáramos": "capcionar",
    "capcionáremos": "capcionar",
    "capcionásemos": "capcionar",
    "capcioné": "capcionar",
    "capcionéis": "capcionar",
    "capcionó": "capcionar",
    "capciosa": "capcioso",
    "capciosas": "capcioso",
    "capciosidades": "capciosidad",
    "capciosos": "capcioso",
    "cape": "capar",
    "capea": "capear",
    "capeaba": "capear",
    "capeabais": "capear",
    "capeaban": "capear",
    "capeabas": "capear",
    "capead": "capear",
    "capeada": "capear",
    "capeadas": "capear",
    "capeado": "capear",
    "capeados": "capear",
    "capeamos": "capear",
    "capean": "capear",
    "capeando": "capear",
    "capeara": "capear",
    "capearais": "capear",
    "capearan": "capear",
    "capearas": "capear",
    "capeare": "capear",
    "capeareis": "capear",
    "capearemos": "capear",
    "capearen": "capear",
    "capeares": "capear",
    "capearon": "capear",
    "capeará": "capear",
    "capearán": "capear",
    "capearás": "capear",
    "capearé": "capear",
    "capearéis": "capear",
    "capearía": "capear",
    "capearíais": "capear",
    "capearíamos": "capear",
    "capearían": "capear",
    "capearías": "capear",
    "capeas": "capear",
    "capease": "capear",
    "capeaseis": "capear",
    "capeasen": "capear",
    "capeases": "capear",
    "capeaste": "capear",
    "capeasteis": "capear",
    "capee": "capear",
    "capeemos": "capear",
    "capeen": "capear",
    "capees": "capear",
    "capelinas": "capelina",
    "capelladas": "capellada",
    "capellanes": "capellán",
    "capellanías": "capellanía",
    "capellinas": "capellina",
    "capelos": "capelo",
    "capemos": "capar",
    "capen": "capar",
    "capeo": "capear",
    "capeones": "capeón",
    "capeos": "capeo",
    "caperoles": "caperol",
    "caperos": "capero",
    "caperuzas": "caperuza",
    "capes": "capar",
    "capetas": "capeta",
    "capetonadas": "capetonada",
    "capeábamos": "capear",
    "capeáis": "capear",
    "capeáramos": "capear",
    "capeáremos": "capear",
    "capeásemos": "capear",
    "capeé": "capear",
    "capeéis": "capear",
    "capeó": "capear",
    "capialce": "capialzar",
    "capialcemos": "capialzar",
    "capialcen": "capialzar",
    "capialces": "capialzar",
    "capialcé": "capialzar",
    "capialcéis": "capialzar",
    "capialza": "capialzar",
    "capialzaba": "capialzar",
    "capialzabais": "capialzar",
    "capialzaban": "capialzar",
    "capialzabas": "capialzar",
    "capialzad": "capialzar",
    "capialzada": "capialzar",
    "capialzadas": "capialzar",
    "capialzado": "capialzar",
    "capialzados": "capialzar",
    "capialzamos": "capialzar",
    "capialzan": "capialzar",
    "capialzando": "capialzar",
    "capialzara": "capialzar",
    "capialzarais": "capialzar",
    "capialzaran": "capialzar",
    "capialzaras": "capialzar",
    "capialzare": "capialzar",
    "capialzareis": "capialzar",
    "capialzaremos": "capialzar",
    "capialzaren": "capialzar",
    "capialzares": "capialzar",
    "capialzaron": "capialzar",
    "capialzará": "capialzar",
    "capialzarán": "capialzar",
    "capialzarás": "capialzar",
    "capialzaré": "capialzar",
    "capialzaréis": "capialzar",
    "capialzaría": "capialzar",
    "capialzaríais": "capialzar",
    "capialzaríamos": "capialzar",
    "capialzarían": "capialzar",
    "capialzarías": "capialzar",
    "capialzas": "capialzar",
    "capialzase": "capialzar",
    "capialzaseis": "capialzar",
    "capialzasen": "capialzar",
    "capialzases": "capialzar",
    "capialzaste": "capialzar",
    "capialzasteis": "capialzar",
    "capialzo": "capialzar",
    "capialzos": "capialzo",
    "capialzábamos": "capialzar",
    "capialzáis": "capialzar",
    "capialzáramos": "capialzar",
    "capialzáremos": "capialzar",
    "capialzásemos": "capialzar",
    "capialzó": "capialzar",
    "capias": "capia",
    "capicúas": "capicúa",
    "capigorras": "capigorra",
    "capigorristas": "capigorrista",
    "capigorrona": "capigorrón",
    "capigorronas": "capigorrón",
    "capigorrones": "capigorrón",
    "capilares": "capilar",
    "capilaridades": "capilaridad",
    "capilarímetros": "capilarímetro",
    "capillas": "capilla",
    "capillos": "capillo",
    "capipardos": "capipardo",
    "capirotadas": "capirotada",
    "capirotazos": "capirotazo",
    "capirotes": "capirote",
    "capiruchos": "capirucho",
    "capisayos": "capisayo",
    "capitaciones": "capitación",
    "capitales": "capital",
    "capitalice": "capitalizar",
    "capitalicemos": "capitalizar",
    "capitalicen": "capitalizar",
    "capitalices": "capitalizar",
    "capitalicé": "capitalizar",
    "capitalicéis": "capitalizar",
    "capitalidades": "capitalidad",
    "capitalina": "capitalino",
    "capitalinas": "capitalino",
    "capitalinos": "capitalino",
    "capitalismos": "capitalismo",
    "capitalistas": "capitalista",
    "capitaliza": "capitalizar",
    "capitalizaba": "capitalizar",
    "capitalizabais": "capitalizar",
    "capitalizaban": "capitalizar",
    "capitalizabas": "capitalizar",
    "capitalizaciones": "capitalización",
    "capitalizad": "capitalizar",
    "capitalizada": "capitalizar",
    "capitalizadas": "capitalizar",
    "capitalizado": "capitalizar",
    "capitalizadora": "capitalizador",
    "capitalizadoras": "capitalizador",
    "capitalizadores": "capitalizador",
    "capitalizados": "capitalizar",
    "capitalizamos": "capitalizar",
    "capitalizan": "capitalizar",
    "capitalizando": "capitalizar",
    "capitalizara": "capitalizar",
    "capitalizarais": "capitalizar",
    "capitalizaran": "capitalizar",
    "capitalizaras": "capitalizar",
    "capitalizare": "capitalizar",
    "capitalizareis": "capitalizar",
    "capitalizaremos": "capitalizar",
    "capitalizaren": "capitalizar",
    "capitalizares": "capitalizar",
    "capitalizaron": "capitalizar",
    "capitalizará": "capitalizar",
    "capitalizarán": "capitalizar",
    "capitalizarás": "capitalizar",
    "capitalizaré": "capitalizar",
    "capitalizaréis": "capitalizar",
    "capitalizaría": "capitalizar",
    "capitalizaríais": "capitalizar",
    "capitalizaríamos": "capitalizar",
    "capitalizarían": "capitalizar",
    "capitalizarías": "capitalizar",
    "capitalizas": "capitalizar",
    "capitalizase": "capitalizar",
    "capitalizaseis": "capitalizar",
    "capitalizasen": "capitalizar",
    "capitalizases": "capitalizar",
    "capitalizaste": "capitalizar",
    "capitalizasteis": "capitalizar",
    "capitalizo": "capitalizar",
    "capitalizábamos": "capitalizar",
    "capitalizáis": "capitalizar",
    "capitalizáramos": "capitalizar",
    "capitalizáremos": "capitalizar",
    "capitalizásemos": "capitalizar",
    "capitalizó": "capitalizar",
    "capitana": "capitán",
    "capitanas": "capitana",
    "capitanea": "capitanear",
    "capitaneaba": "capitanear",
    "capitaneabais": "capitanear",
    "capitaneaban": "capitanear",
    "capitaneabas": "capitanear",
    "capitanead": "capitanear",
    "capitaneada": "capitanear",
    "capitaneadas": "capitanear",
    "capitaneado": "capitanear",
    "capitaneados": "capitanear",
    "capitaneamos": "capitanear",
    "capitanean": "capitanear",
    "capitaneando": "capitanear",
    "capitaneara": "capitanear",
    "capitanearais": "capitanear",
    "capitanearan": "capitanear",
    "capitanearas": "capitanear",
    "capitaneare": "capitanear",
    "capitaneareis": "capitanear",
    "capitanearemos": "capitanear",
    "capitanearen": "capitanear",
    "capitaneares": "capitanear",
    "capitanearon": "capitanear",
    "capitaneará": "capitanear",
    "capitanearán": "capitanear",
    "capitanearás": "capitanear",
    "capitanearé": "capitanear",
    "capitanearéis": "capitanear",
    "capitanearía": "capitanear",
    "capitanearíais": "capitanear",
    "capitanearíamos": "capitanear",
    "capitanearían": "capitanear",
    "capitanearías": "capitanear",
    "capitaneas": "capitanear",
    "capitanease": "capitanear",
    "capitaneaseis": "capitanear",
    "capitaneasen": "capitanear",
    "capitaneases": "capitanear",
    "capitaneaste": "capitanear",
    "capitaneasteis": "capitanear",
    "capitanee": "capitanear",
    "capitaneemos": "capitanear",
    "capitaneen": "capitanear",
    "capitanees": "capitanear",
    "capitaneo": "capitanear",
    "capitanes": "capitán",
    "capitaneábamos": "capitanear",
    "capitaneáis": "capitanear",
    "capitaneáramos": "capitanear",
    "capitaneáremos": "capitanear",
    "capitaneásemos": "capitanear",
    "capitaneé": "capitanear",
    "capitaneéis": "capitanear",
    "capitaneó": "capitanear",
    "capitanías": "capitanía",
    "capiteles": "capitel",
    "capitolina": "capitolino",
    "capitolinas": "capitolino",
    "capitolinos": "capitolino",
    "capitolios": "capitolio",
    "capitonés": "capitoné",
    "capitostes": "capitoste",
    "capitula": "capitular",
    "capitulaba": "capitular",
    "capitulabais": "capitular",
    "capitulaban": "capitular",
    "capitulabas": "capitular",
    "capitulaciones": "capitulación",
    "capitulad": "capitular",
    "capitulada": "capitular",
    "capituladas": "capitular",
    "capitulado": "capitular",
    "capitulados": "capitular",
    "capitulamos": "capitular",
    "capitulan": "capitular",
    "capitulando": "capitular",
    "capitulara": "capitular",
    "capitularais": "capitular",
    "capitularan": "capitular",
    "capitularas": "capitular",
    "capitulare": "capitular",
    "capitulareis": "capitular",
    "capitularemos": "capitular",
    "capitularen": "capitular",
    "capitulares": "capitular",
    "capitularios": "capitulario",
    "capitularon": "capitular",
    "capitulará": "capitular",
    "capitularán": "capitular",
    "capitularás": "capitular",
    "capitularé": "capitular",
    "capitularéis": "capitular",
    "capitularía": "capitular",
    "capitularíais": "capitular",
    "capitularíamos": "capitular",
    "capitularían": "capitular",
    "capitularías": "capitular",
    "capitulas": "capitular",
    "capitulase": "capitular",
    "capitulaseis": "capitular",
    "capitulasen": "capitular",
    "capitulases": "capitular",
    "capitulaste": "capitular",
    "capitulasteis": "capitular",
    "capitule": "capitular",
    "capitulemos": "capitular",
    "capitulen": "capitular",
    "capitules": "capitular",
    "capitulo": "capitular",
    "capitulábamos": "capitular",
    "capituláis": "capitular",
    "capituláramos": "capitular",
    "capituláremos": "capitular",
    "capitulásemos": "capitular",
    "capitulé": "capitular",
    "capituléis": "capitular",
    "capituló": "capitular",
    "capizanas": "capizana",
    "capnomancias": "capnomancia",
    "capnomancías": "capnomancía",
    "capo": "capar",
    "capola": "capolar",
    "capolaba": "capolar",
    "capolabais": "capolar",
    "capolaban": "capolar",
    "capolabas": "capolar",
    "capolad": "capolar",
    "capolada": "capolar",
    "capoladas": "capolar",
    "capolado": "capolar",
    "capolados": "capolar",
    "capolamos": "capolar",
    "capolan": "capolar",
    "capolando": "capolar",
    "capolara": "capolar",
    "capolarais": "capolar",
    "capolaran": "capolar",
    "capolaras": "capolar",
    "capolare": "capolar",
    "capolareis": "capolar",
    "capolaremos": "capolar",
    "capolaren": "capolar",
    "capolares": "capolar",
    "capolaron": "capolar",
    "capolará": "capolar",
    "capolarán": "capolar",
    "capolarás": "capolar",
    "capolaré": "capolar",
    "capolaréis": "capolar",
    "capolaría": "capolar",
    "capolaríais": "capolar",
    "capolaríamos": "capolar",
    "capolarían": "capolar",
    "capolarías": "capolar",
    "capolas": "capolar",
    "capolase": "capolar",
    "capolaseis": "capolar",
    "capolasen": "capolar",
    "capolases": "capolar",
    "capolaste": "capolar",
    "capolasteis": "capolar",
    "capole": "capolar",
    "capolemos": "capolar",
    "capolen": "capolar",
    "capoles": "capolar",
    "capolo": "capolar",
    "capolábamos": "capolar",
    "capoláis": "capolar",
    "capoláramos": "capolar",
    "capoláremos": "capolar",
    "capolásemos": "capolar",
    "capolé": "capolar",
    "capoléis": "capolar",
    "capoló": "capolar",
    "capona": "caponar",
    "caponaba": "caponar",
    "caponabais": "caponar",
    "caponaban": "caponar",
    "caponabas": "caponar",
    "caponad": "caponar",
    "caponada": "caponar",
    "caponadas": "caponar",
    "caponado": "caponar",
    "caponados": "caponar",
    "caponamos": "caponar",
    "caponan": "caponar",
    "caponando": "caponar",
    "caponara": "caponar",
    "caponarais": "caponar",
    "caponaran": "caponar",
    "caponaras": "caponar",
    "caponare": "caponar",
    "caponareis": "caponar",
    "caponaremos": "caponar",
    "caponaren": "caponar",
    "caponares": "caponar",
    "caponaron": "caponar",
    "caponará": "caponar",
    "caponarán": "caponar",
    "caponarás": "caponar",
    "caponaré": "caponar",
    "caponaréis": "caponar",
    "caponaría": "caponar",
    "caponaríais": "caponar",
    "caponaríamos": "caponar",
    "caponarían": "caponar",
    "caponarías": "caponar",
    "caponas": "caponar",
    "caponase": "caponar",
    "caponaseis": "caponar",
    "caponasen": "caponar",
    "caponases": "caponar",
    "caponaste": "caponar",
    "caponasteis": "caponar",
    "capone": "caponar",
    "caponemos": "caponar",
    "caponen": "caponar",
    "caponeras": "caponera",
    "capones": "caponar",
    "capono": "caponar",
    "caponábamos": "caponar",
    "caponáis": "caponar",
    "caponáramos": "caponar",
    "caponáremos": "caponar",
    "caponásemos": "caponar",
    "caponé": "caponar",
    "caponéis": "caponar",
    "caponó": "caponar",
    "caporales": "caporal",
    "capota": "capotar",
    "capotaba": "capotar",
    "capotabais": "capotar",
    "capotaban": "capotar",
    "capotabas": "capotar",
    "capotad": "capotar",
    "capotada": "capotar",
    "capotadas": "capotar",
    "capotado": "capotar",
    "capotados": "capotar",
    "capotajes": "capotaje",
    "capotamos": "capotar",
    "capotan": "capotar",
    "capotando": "capotar",
    "capotara": "capotar",
    "capotarais": "capotar",
    "capotaran": "capotar",
    "capotaras": "capotar",
    "capotare": "capotar",
    "capotareis": "capotar",
    "capotaremos": "capotar",
    "capotaren": "capotar",
    "capotares": "capotar",
    "capotaron": "capotar",
    "capotará": "capotar",
    "capotarán": "capotar",
    "capotarás": "capotar",
    "capotaré": "capotar",
    "capotaréis": "capotar",
    "capotaría": "capotar",
    "capotaríais": "capotar",
    "capotaríamos": "capotar",
    "capotarían": "capotar",
    "capotarías": "capotar",
    "capotas": "capotar",
    "capotase": "capotar",
    "capotaseis": "capotar",
    "capotasen": "capotar",
    "capotases": "capotar",
    "capotaste": "capotar",
    "capotasteis": "capotar",
    "capotazos": "capotazo",
    "capote": "capotar",
    "capotea": "capotear",
    "capoteaba": "capotear",
    "capoteabais": "capotear",
    "capoteaban": "capotear",
    "capoteabas": "capotear",
    "capotead": "capotear",
    "capoteada": "capotear",
    "capoteadas": "capotear",
    "capoteado": "capotear",
    "capoteados": "capotear",
    "capoteamos": "capotear",
    "capotean": "capotear",
    "capoteando": "capotear",
    "capoteara": "capotear",
    "capotearais": "capotear",
    "capotearan": "capotear",
    "capotearas": "capotear",
    "capoteare": "capotear",
    "capoteareis": "capotear",
    "capotearemos": "capotear",
    "capotearen": "capotear",
    "capoteares": "capotear",
    "capotearon": "capotear",
    "capoteará": "capotear",
    "capotearán": "capotear",
    "capotearás": "capotear",
    "capotearé": "capotear",
    "capotearéis": "capotear",
    "capotearía": "capotear",
    "capotearíais": "capotear",
    "capotearíamos": "capotear",
    "capotearían": "capotear",
    "capotearías": "capotear",
    "capoteas": "capotear",
    "capotease": "capotear",
    "capoteaseis": "capotear",
    "capoteasen": "capotear",
    "capoteases": "capotear",
    "capoteaste": "capotear",
    "capoteasteis": "capotear",
    "capotee": "capotear",
    "capoteemos": "capotear",
    "capoteen": "capotear",
    "capotees": "capotear",
    "capotemos": "capotar",
    "capoten": "capotar",
    "capoteo": "capotear",
    "capoteos": "capoteo",
    "capotes": "capote",
    "capoteábamos": "capotear",
    "capoteáis": "capotear",
    "capoteáramos": "capotear",
    "capoteáremos": "capotear",
    "capoteásemos": "capotear",
    "capoteé": "capotear",
    "capoteéis": "capotear",
    "capoteó": "capotear",
    "capotillos": "capotillo",
    "capoto": "capotar",
    "capots": "capot",
    "capotábamos": "capotar",
    "capotáis": "capotar",
    "capotáramos": "capotar",
    "capotáremos": "capotar",
    "capotásemos": "capotar",
    "capoté": "capotar",
    "capotéis": "capotar",
    "capotó": "capotar",
    "cappas": "cappa",
    "capraria": "caprario",
    "caprarias": "caprario",
    "caprarios": "caprario",
    "capratos": "caprato",
    "caprichos": "capricho",
    "caprichosa": "caprichoso",
    "caprichosas": "caprichoso",
    "caprichosos": "caprichoso",
    "caprichuda": "caprichudo",
    "caprichudas": "caprichudo",
    "caprichudos": "caprichudo",
    "capricornios": "capricornio",
    "caprifoliáceas": "caprifoliácea",
    "caprilatos": "caprilato",
    "caprina": "caprino",
    "caprinas": "caprino",
    "caprinos": "caprino",
    "caproatos": "caproato",
    "caproica": "caproico",
    "caproicas": "caproico",
    "caproicos": "caproico",
    "caprolactamas": "caprolactama",
    "caprílica": "caprílico",
    "caprílicas": "caprílico",
    "caprílicos": "caprílico",
    "caps": "capítulo",
    "capsienses": "capsiense",
    "capsula": "capsular",
    "capsulaba": "capsular",
    "capsulabais": "capsular",
    "capsulaban": "capsular",
    "capsulabas": "capsular",
    "capsulaciones": "capsulación",
    "capsulad": "capsular",
    "capsulada": "capsular",
    "capsuladas": "capsular",
    "capsulado": "capsular",
    "capsulados": "capsular",
    "capsulamos": "capsular",
    "capsulan": "capsular",
    "capsulando": "capsular",
    "capsulara": "capsular",
    "capsularais": "capsular",
    "capsularan": "capsular",
    "capsularas": "capsular",
    "capsulare": "capsular",
    "capsulareis": "capsular",
    "capsularemos": "capsular",
    "capsularen": "capsular",
    "capsulares": "capsular",
    "capsularon": "capsular",
    "capsulará": "capsular",
    "capsularán": "capsular",
    "capsularás": "capsular",
    "capsularé": "capsular",
    "capsularéis": "capsular",
    "capsularía": "capsular",
    "capsularíais": "capsular",
    "capsularíamos": "capsular",
    "capsularían": "capsular",
    "capsularías": "capsular",
    "capsulas": "capsular",
    "capsulase": "capsular",
    "capsulaseis": "capsular",
    "capsulasen": "capsular",
    "capsulases": "capsular",
    "capsulaste": "capsular",
    "capsulasteis": "capsular",
    "capsule": "capsular",
    "capsulectomías": "capsulectomía",
    "capsulemos": "capsular",
    "capsulen": "capsular",
    "capsules": "capsular",
    "capsulo": "capsular",
    "capsulotomías": "capsulotomía",
    "capsulábamos": "capsular",
    "capsuláis": "capsular",
    "capsuláramos": "capsular",
    "capsuláremos": "capsular",
    "capsulásemos": "capsular",
    "capsulé": "capsular",
    "capsuléis": "capsular",
    "capsuló": "capsular",
    "capsómeros": "capsómero",
    "capta": "captar",
    "captaba": "captar",
    "captabais": "captar",
    "captaban": "captar",
    "captabas": "captar",
    "captaciones": "captación",
    "captad": "captar",
    "captada": "captar",
    "captadas": "captar",
    "captado": "captar",
    "captadora": "captador",
    "captadoras": "captador",
    "captadores": "captador",
    "captados": "captar",
    "captamos": "captar",
    "captan": "captar",
    "captando": "captar",
    "captanos": "captano",
    "captara": "captar",
    "captarais": "captar",
    "captaran": "captar",
    "captaras": "captar",
    "captare": "captar",
    "captareis": "captar",
    "captaremos": "captar",
    "captaren": "captar",
    "captares": "captar",
    "captaron": "captar",
    "captará": "captar",
    "captarán": "captar",
    "captarás": "captar",
    "captaré": "captar",
    "captaréis": "captar",
    "captaría": "captar",
    "captaríais": "captar",
    "captaríamos": "captar",
    "captarían": "captar",
    "captarías": "captar",
    "captas": "captar",
    "captase": "captar",
    "captaseis": "captar",
    "captasen": "captar",
    "captases": "captar",
    "captaste": "captar",
    "captasteis": "captar",
    "captatoria": "captatorio",
    "captatorias": "captatorio",
    "captatorios": "captatorio",
    "capte": "captar",
    "captemos": "captar",
    "capten": "captar",
    "captes": "captar",
    "captividades": "captividad",
    "capto": "captar",
    "captura": "capturar",
    "capturaba": "capturar",
    "capturabais": "capturar",
    "capturaban": "capturar",
    "capturabas": "capturar",
    "capturad": "capturar",
    "capturada": "capturar",
    "capturadas": "capturar",
    "capturado": "capturar",
    "capturadora": "capturador",
    "capturadoras": "capturador",
    "capturadores": "capturador",
    "capturados": "capturar",
    "capturamos": "capturar",
    "capturan": "capturar",
    "capturando": "capturar",
    "capturara": "capturar",
    "capturarais": "capturar",
    "capturaran": "capturar",
    "capturaras": "capturar",
    "capturare": "capturar",
    "capturareis": "capturar",
    "capturaremos": "capturar",
    "capturaren": "capturar",
    "capturares": "capturar",
    "capturaron": "capturar",
    "capturará": "capturar",
    "capturarán": "capturar",
    "capturarás": "capturar",
    "capturaré": "capturar",
    "capturaréis": "capturar",
    "capturaría": "capturar",
    "capturaríais": "capturar",
    "capturaríamos": "capturar",
    "capturarían": "capturar",
    "capturarías": "capturar",
    "capturas": "capturar",
    "capturase": "capturar",
    "capturaseis": "capturar",
    "capturasen": "capturar",
    "capturases": "capturar",
    "capturaste": "capturar",
    "capturasteis": "capturar",
    "capture": "capturar",
    "capturemos": "capturar",
    "capturen": "capturar",
    "captures": "capturar",
    "capturo": "capturar",
    "capturábamos": "capturar",
    "capturáis": "capturar",
    "capturáramos": "capturar",
    "capturáremos": "capturar",
    "capturásemos": "capturar",
    "capturé": "capturar",
    "capturéis": "capturar",
    "capturó": "capturar",
    "captábamos": "captar",
    "captáis": "captar",
    "captáramos": "captar",
    "captáremos": "captar",
    "captásemos": "captar",
    "capté": "captar",
    "captéis": "captar",
    "captó": "captar",
    "capuanas": "capuana",
    "capuce": "capuzar",
    "capucemos": "capuzar",
    "capucen": "capuzar",
    "capuces": "capuzar",
    "capuchas": "capucha",
    "capuchina": "capuchino",
    "capuchinas": "capuchino",
    "capuchinos": "capuchino",
    "capuchones": "capuchón",
    "capuchos": "capucho",
    "capucé": "capuzar",
    "capucéis": "capuzar",
    "capuletos": "capuleto",
    "capullos": "capullo",
    "capuza": "capuzar",
    "capuzaba": "capuzar",
    "capuzabais": "capuzar",
    "capuzaban": "capuzar",
    "capuzabas": "capuzar",
    "capuzad": "capuzar",
    "capuzada": "capuzar",
    "capuzadas": "capuzar",
    "capuzado": "capuzar",
    "capuzados": "capuzar",
    "capuzamos": "capuzar",
    "capuzan": "capuzar",
    "capuzando": "capuzar",
    "capuzara": "capuzar",
    "capuzarais": "capuzar",
    "capuzaran": "capuzar",
    "capuzaras": "capuzar",
    "capuzare": "capuzar",
    "capuzareis": "capuzar",
    "capuzaremos": "capuzar",
    "capuzaren": "capuzar",
    "capuzares": "capuzar",
    "capuzaron": "capuzar",
    "capuzará": "capuzar",
    "capuzarán": "capuzar",
    "capuzarás": "capuzar",
    "capuzaré": "capuzar",
    "capuzaréis": "capuzar",
    "capuzaría": "capuzar",
    "capuzaríais": "capuzar",
    "capuzaríamos": "capuzar",
    "capuzarían": "capuzar",
    "capuzarías": "capuzar",
    "capuzas": "capuzar",
    "capuzase": "capuzar",
    "capuzaseis": "capuzar",
    "capuzasen": "capuzar",
    "capuzases": "capuzar",
    "capuzaste": "capuzar",
    "capuzasteis": "capuzar",
    "capuzo": "capuzar",
    "capuzones": "capuzón",
    "capuzábamos": "capuzar",
    "capuzáis": "capuzar",
    "capuzáramos": "capuzar",
    "capuzáremos": "capuzar",
    "capuzásemos": "capuzar",
    "capuzó": "capuzar",
    "capábamos": "capar",
    "capáis": "capar",
    "capáramos": "capar",
    "capáremos": "capar",
    "capásemos": "capar",
    "capé": "capar",
    "capéis": "capar",
    "capítulas": "capítula",
    "capítulos": "capítulo",
    "capó": "capar",
    "capós": "capó",
    "caquexias": "caquexia",
    "caquis": "caqui",
    "caquéctica": "caquéctico",
    "caquécticas": "caquéctico",
    "caquécticos": "caquéctico",
    "cara": "caro",
    "carabas": "caraba",
    "carabelas": "carabela",
    "carabinas": "carabina",
    "carabinazos": "carabinazo",
    "carabinera": "carabinero",
    "carabineras": "carabinero",
    "carabineros": "carabinero",
    "caracales": "caracal",
    "caracarás": "caracará",
    "carachas": "caracha",
    "caracoladas": "caracolada",
    "caracolas": "caracola",
    "caracolea": "caracolear",
    "caracoleaba": "caracolear",
    "caracoleabais": "caracolear",
    "caracoleaban": "caracolear",
    "caracoleabas": "caracolear",
    "caracolead": "caracolear",
    "caracoleada": "caracolear",
    "caracoleadas": "caracolear",
    "caracoleado": "caracolear",
    "caracoleados": "caracolear",
    "caracoleamos": "caracolear",
    "caracolean": "caracolear",
    "caracoleando": "caracolear",
    "caracoleara": "caracolear",
    "caracolearais": "caracolear",
    "caracolearan": "caracolear",
    "caracolearas": "caracolear",
    "caracoleare": "caracolear",
    "caracoleareis": "caracolear",
    "caracolearemos": "caracolear",
    "caracolearen": "caracolear",
    "caracoleares": "caracolear",
    "caracolearon": "caracolear",
    "caracoleará": "caracolear",
    "caracolearán": "caracolear",
    "caracolearás": "caracolear",
    "caracolearé": "caracolear",
    "caracolearéis": "caracolear",
    "caracolearía": "caracolear",
    "caracolearíais": "caracolear",
    "caracolearíamos": "caracolear",
    "caracolearían": "caracolear",
    "caracolearías": "caracolear",
    "caracoleas": "caracolear",
    "caracolease": "caracolear",
    "caracoleaseis": "caracolear",
    "caracoleasen": "caracolear",
    "caracoleases": "caracolear",
    "caracoleaste": "caracolear",
    "caracoleasteis": "caracolear",
    "caracolee": "caracolear",
    "caracoleemos": "caracolear",
    "caracoleen": "caracolear",
    "caracolees": "caracolear",
    "caracoleo": "caracolear",
    "caracoleos": "caracoleo",
    "caracoles": "caracol",
    "caracoleábamos": "caracolear",
    "caracoleáis": "caracolear",
    "caracoleáramos": "caracolear",
    "caracoleáremos": "caracolear",
    "caracoleásemos": "caracolear",
    "caracoleé": "caracolear",
    "caracoleéis": "caracolear",
    "caracoleó": "caracolear",
    "caracolillos": "caracolillo",
    "caracteres": "carácter",
    "caracteriales": "caracterial",
    "caracterice": "caracterizar",
    "caractericemos": "caracterizar",
    "caractericen": "caracterizar",
    "caracterices": "caracterizar",
    "caractericé": "caracterizar",
    "caractericéis": "caracterizar",
    "caracteriza": "caracterizar",
    "caracterizaba": "caracterizar",
    "caracterizabais": "caracterizar",
    "caracterizaban": "caracterizar",
    "caracterizabas": "caracterizar",
    "caracterizaciones": "caracterización",
    "caracterizad": "caracterizar",
    "caracterizada": "caracterizar",
    "caracterizadas": "caracterizar",
    "caracterizado": "caracterizar",
    "caracterizados": "caracterizar",
    "caracterizamos": "caracterizar",
    "caracterizan": "caracterizar",
    "caracterizando": "caracterizar",
    "caracterizara": "caracterizar",
    "caracterizarais": "caracterizar",
    "caracterizaran": "caracterizar",
    "caracterizaras": "caracterizar",
    "caracterizare": "caracterizar",
    "caracterizareis": "caracterizar",
    "caracterizaremos": "caracterizar",
    "caracterizaren": "caracterizar",
    "caracterizares": "caracterizar",
    "caracterizaron": "caracterizar",
    "caracterizará": "caracterizar",
    "caracterizarán": "caracterizar",
    "caracterizarás": "caracterizar",
    "caracterizaré": "caracterizar",
    "caracterizaréis": "caracterizar",
    "caracterizaría": "caracterizar",
    "caracterizaríais": "caracterizar",
    "caracterizaríamos": "caracterizar",
    "caracterizarían": "caracterizar",
    "caracterizarías": "caracterizar",
    "caracterizas": "caracterizar",
    "caracterizase": "caracterizar",
    "caracterizaseis": "caracterizar",
    "caracterizasen": "caracterizar",
    "caracterizases": "caracterizar",
    "caracterizaste": "caracterizar",
    "caracterizasteis": "caracterizar",
    "caracterizo": "caracterizar",
    "caracterizábamos": "caracterizar",
    "caracterizáis": "caracterizar",
    "caracterizáramos": "caracterizar",
    "caracterizáremos": "caracterizar",
    "caracterizásemos": "caracterizar",
    "caracterizó": "caracterizar",
    "caracterologías": "caracterología",
    "caracterológica": "caracterológico",
    "caracterológicas": "caracterológico",
    "caracterológicos": "caracterológico",
    "característica": "característico",
    "características": "característico",
    "característicos": "característico",
    "caracules": "caracul",
    "carada": "carado",
    "caradas": "carado",
    "carados": "carado",
    "caradriformes": "caradriforme",
    "caraduras": "caradura",
    "carajillos": "carajillo",
    "carajos": "carajo",
    "carajás": "carajá",
    "caramancheles": "caramanchel",
    "carambas": "caramba",
    "carambolas": "carambola",
    "carambolistas": "carambolista",
    "carambolos": "carambolo",
    "caramelea": "caramelear",
    "carameleaba": "caramelear",
    "carameleabais": "caramelear",
    "carameleaban": "caramelear",
    "carameleabas": "caramelear",
    "caramelead": "caramelear",
    "carameleada": "caramelear",
    "carameleadas": "caramelear",
    "carameleado": "caramelear",
    "carameleados": "caramelear",
    "carameleamos": "caramelear",
    "caramelean": "caramelear",
    "carameleando": "caramelear",
    "carameleara": "caramelear",
    "caramelearais": "caramelear",
    "caramelearan": "caramelear",
    "caramelearas": "caramelear",
    "carameleare": "caramelear",
    "carameleareis": "caramelear",
    "caramelearemos": "caramelear",
    "caramelearen": "caramelear",
    "carameleares": "caramelear",
    "caramelearon": "caramelear",
    "carameleará": "caramelear",
    "caramelearán": "caramelear",
    "caramelearás": "caramelear",
    "caramelearé": "caramelear",
    "caramelearéis": "caramelear",
    "caramelearía": "caramelear",
    "caramelearíais": "caramelear",
    "caramelearíamos": "caramelear",
    "caramelearían": "caramelear",
    "caramelearías": "caramelear",
    "carameleas": "caramelear",
    "caramelease": "caramelear",
    "carameleaseis": "caramelear",
    "carameleasen": "caramelear",
    "carameleases": "caramelear",
    "carameleaste": "caramelear",
    "carameleasteis": "caramelear",
    "caramelee": "caramelear",
    "carameleemos": "caramelear",
    "carameleen": "caramelear",
    "caramelees": "caramelear",
    "carameleo": "caramelear",
    "carameleábamos": "caramelear",
    "carameleáis": "caramelear",
    "carameleáramos": "caramelear",
    "carameleáremos": "caramelear",
    "carameleásemos": "caramelear",
    "carameleé": "caramelear",
    "carameleéis": "caramelear",
    "carameleó": "caramelear",
    "caramelice": "caramelizar",
    "caramelicemos": "caramelizar",
    "caramelicen": "caramelizar",
    "caramelices": "caramelizar",
    "caramelicé": "caramelizar",
    "caramelicéis": "caramelizar",
    "carameliza": "caramelizar",
    "caramelizaba": "caramelizar",
    "caramelizabais": "caramelizar",
    "caramelizaban": "caramelizar",
    "caramelizabas": "caramelizar",
    "caramelizaciones": "caramelización",
    "caramelizad": "caramelizar",
    "caramelizada": "caramelizar",
    "caramelizadas": "caramelizar",
    "caramelizado": "caramelizar",
    "caramelizados": "caramelizar",
    "caramelizamos": "caramelizar",
    "caramelizan": "caramelizar",
    "caramelizando": "caramelizar",
    "caramelizara": "caramelizar",
    "caramelizarais": "caramelizar",
    "caramelizaran": "caramelizar",
    "caramelizaras": "caramelizar",
    "caramelizare": "caramelizar",
    "caramelizareis": "caramelizar",
    "caramelizaremos": "caramelizar",
    "caramelizaren": "caramelizar",
    "caramelizares": "caramelizar",
    "caramelizaron": "caramelizar",
    "caramelizará": "caramelizar",
    "caramelizarán": "caramelizar",
    "caramelizarás": "caramelizar",
    "caramelizaré": "caramelizar",
    "caramelizaréis": "caramelizar",
    "caramelizaría": "caramelizar",
    "caramelizaríais": "caramelizar",
    "caramelizaríamos": "caramelizar",
    "caramelizarían": "caramelizar",
    "caramelizarías": "caramelizar",
    "caramelizas": "caramelizar",
    "caramelizase": "caramelizar",
    "caramelizaseis": "caramelizar",
    "caramelizasen": "caramelizar",
    "caramelizases": "caramelizar",
    "caramelizaste": "caramelizar",
    "caramelizasteis": "caramelizar",
    "caramelizo": "caramelizar",
    "caramelizábamos": "caramelizar",
    "caramelizáis": "caramelizar",
    "caramelizáramos": "caramelizar",
    "caramelizáremos": "caramelizar",
    "caramelizásemos": "caramelizar",
    "caramelizó": "caramelizar",
    "caramelos": "caramelo",
    "caramillos": "caramillo",
    "caramillosa": "caramilloso",
    "caramillosas": "caramilloso",
    "caramillosos": "caramilloso",
    "caramujos": "caramujo",
    "caramuzales": "caramuzal",
    "caranchos": "carancho",
    "carantamaulas": "carantamaula",
    "carantoñas": "carantoña",
    "carantoñera": "carantoñero",
    "carantoñeras": "carantoñero",
    "carantoñeros": "carantoñero",
    "carapachos": "carapacho",
    "carapas": "carapa",
    "carapatos": "carapato",
    "caraqueña": "caraqueño",
    "caraqueñas": "caraqueño",
    "caraqueños": "caraqueño",
    "caras": "caro",
    "carasoles": "carasol",
    "carateas": "caratea",
    "carates": "carate",
    "caravagismos": "caravagismo",
    "caravanas": "caravana",
    "caravanera": "caravanero",
    "caravaneras": "caravanero",
    "caravaneros": "caravanero",
    "caravasares": "caravasar",
    "caraísmos": "caraísmo",
    "caraítas": "caraíta",
    "carbamatos": "carbamato",
    "carbamidas": "carbamida",
    "carbanilidas": "carbanilida",
    "carbaniones": "carbanión",
    "carbasonas": "carbasona",
    "carbatos": "carbato",
    "carbazoles": "carbazol",
    "carbenos": "carbeno",
    "carbinoles": "carbinol",
    "carbocationes": "carbocatión",
    "carbocloraciones": "carbocloración",
    "carbocíclica": "carbocíclico",
    "carbocíclicas": "carbocíclico",
    "carbocíclicos": "carbocíclico",
    "carbohidrasas": "carbohidrasa",
    "carbohidratos": "carbohidrato",
    "carbolíneos": "carbolíneo",
    "carbomicinas": "carbomicina",
    "carbona": "carbonar",
    "carbonaba": "carbonar",
    "carbonabais": "carbonar",
    "carbonaban": "carbonar",
    "carbonabas": "carbonar",
    "carbonad": "carbonar",
    "carbonada": "carbonar",
    "carbonadas": "carbonar",
    "carbonado": "carbonar",
    "carbonados": "carbonar",
    "carbonallas": "carbonalla",
    "carbonamos": "carbonar",
    "carbonan": "carbonar",
    "carbonando": "carbonar",
    "carbonara": "carbonar",
    "carbonarais": "carbonar",
    "carbonaran": "carbonar",
    "carbonaras": "carbonar",
    "carbonare": "carbonar",
    "carbonareis": "carbonar",
    "carbonaremos": "carbonar",
    "carbonaren": "carbonar",
    "carbonares": "carbonar",
    "carbonaria": "carbonario",
    "carbonarias": "carbonario",
    "carbonarios": "carbonario",
    "carbonarismos": "carbonarismo",
    "carbonaron": "carbonar",
    "carbonará": "carbonar",
    "carbonarán": "carbonar",
    "carbonarás": "carbonar",
    "carbonaré": "carbonar",
    "carbonaréis": "carbonar",
    "carbonaría": "carbonar",
    "carbonaríais": "carbonar",
    "carbonaríamos": "carbonar",
    "carbonarían": "carbonar",
    "carbonarías": "carbonar",
    "carbonas": "carbonar",
    "carbonase": "carbonar",
    "carbonaseis": "carbonar",
    "carbonasen": "carbonar",
    "carbonases": "carbonar",
    "carbonaste": "carbonar",
    "carbonasteis": "carbonar",
    "carbonata": "carbonatar",
    "carbonataba": "carbonatar",
    "carbonatabais": "carbonatar",
    "carbonataban": "carbonatar",
    "carbonatabas": "carbonatar",
    "carbonataciones": "carbonatación",
    "carbonatad": "carbonatar",
    "carbonatada": "carbonatar",
    "carbonatadas": "carbonatar",
    "carbonatado": "carbonatar",
    "carbonatadores": "carbonatador",
    "carbonatados": "carbonatar",
    "carbonatamos": "carbonatar",
    "carbonatan": "carbonatar",
    "carbonatando": "carbonatar",
    "carbonatara": "carbonatar",
    "carbonatarais": "carbonatar",
    "carbonataran": "carbonatar",
    "carbonataras": "carbonatar",
    "carbonatare": "carbonatar",
    "carbonatareis": "carbonatar",
    "carbonataremos": "carbonatar",
    "carbonataren": "carbonatar",
    "carbonatares": "carbonatar",
    "carbonataron": "carbonatar",
    "carbonatará": "carbonatar",
    "carbonatarán": "carbonatar",
    "carbonatarás": "carbonatar",
    "carbonataré": "carbonatar",
    "carbonataréis": "carbonatar",
    "carbonataría": "carbonatar",
    "carbonataríais": "carbonatar",
    "carbonataríamos": "carbonatar",
    "carbonatarían": "carbonatar",
    "carbonatarías": "carbonatar",
    "carbonatas": "carbonatar",
    "carbonatase": "carbonatar",
    "carbonataseis": "carbonatar",
    "carbonatasen": "carbonatar",
    "carbonatases": "carbonatar",
    "carbonataste": "carbonatar",
    "carbonatasteis": "carbonatar",
    "carbonate": "carbonatar",
    "carbonatemos": "carbonatar",
    "carbonaten": "carbonatar",
    "carbonates": "carbonatar",
    "carbonatitas": "carbonatita",
    "carbonato": "carbonatar",
    "carbonatos": "carbonato",
    "carbonatábamos": "carbonatar",
    "carbonatáis": "carbonatar",
    "carbonatáramos": "carbonatar",
    "carbonatáremos": "carbonatar",
    "carbonatásemos": "carbonatar",
    "carbonaté": "carbonatar",
    "carbonatéis": "carbonatar",
    "carbonató": "carbonatar",
    "carboncillos": "carboncillo",
    "carbone": "carbonar",
    "carbonea": "carbonear",
    "carboneaba": "carbonear",
    "carboneabais": "carbonear",
    "carboneaban": "carbonear",
    "carboneabas": "carbonear",
    "carbonead": "carbonear",
    "carboneada": "carbonear",
    "carboneadas": "carbonear",
    "carboneado": "carbonear",
    "carboneados": "carbonear",
    "carboneamos": "carbonear",
    "carbonean": "carbonear",
    "carboneando": "carbonear",
    "carboneara": "carbonear",
    "carbonearais": "carbonear",
    "carbonearan": "carbonear",
    "carbonearas": "carbonear",
    "carboneare": "carbonear",
    "carboneareis": "carbonear",
    "carbonearemos": "carbonear",
    "carbonearen": "carbonear",
    "carboneares": "carbonear",
    "carbonearon": "carbonear",
    "carboneará": "carbonear",
    "carbonearán": "carbonear",
    "carbonearás": "carbonear",
    "carbonearé": "carbonear",
    "carbonearéis": "carbonear",
    "carbonearía": "carbonear",
    "carbonearíais": "carbonear",
    "carbonearíamos": "carbonear",
    "carbonearían": "carbonear",
    "carbonearías": "carbonear",
    "carboneas": "carbonear",
    "carbonease": "carbonear",
    "carboneaseis": "carbonear",
    "carboneasen": "carbonear",
    "carboneases": "carbonear",
    "carboneaste": "carbonear",
    "carboneasteis": "carbonear",
    "carbonee": "carbonear",
    "carboneemos": "carbonear",
    "carboneen": "carbonear",
    "carbonees": "carbonear",
    "carbonemos": "carbonar",
    "carbonen": "carbonar",
    "carboneo": "carbonear",
    "carboneos": "carboneo",
    "carbonera": "carbonero",
    "carboneras": "carbonero",
    "carboneros": "carbonero",
    "carbonerías": "carbonería",
    "carbones": "carbonar",
    "carboneábamos": "carbonear",
    "carboneáis": "carbonear",
    "carboneáramos": "carbonear",
    "carboneáremos": "carbonear",
    "carboneásemos": "carbonear",
    "carboneé": "carbonear",
    "carboneéis": "carbonear",
    "carboneó": "carbonear",
    "carbonice": "carbonizar",
    "carbonicemos": "carbonizar",
    "carbonicen": "carbonizar",
    "carbonices": "carbonizar",
    "carbonicé": "carbonizar",
    "carbonicéis": "carbonizar",
    "carbonificaciones": "carbonificación",
    "carbonilaciones": "carbonilación",
    "carbonillas": "carbonilla",
    "carbonilos": "carbonilo",
    "carbonitruraciones": "carbonitruración",
    "carboniza": "carbonizar",
    "carbonizaba": "carbonizar",
    "carbonizabais": "carbonizar",
    "carbonizaban": "carbonizar",
    "carbonizabas": "carbonizar",
    "carbonizaciones": "carbonización",
    "carbonizad": "carbonizar",
    "carbonizada": "carbonizar",
    "carbonizadas": "carbonizar",
    "carbonizado": "carbonizar",
    "carbonizados": "carbonizar",
    "carbonizamientos": "carbonizamiento",
    "carbonizamos": "carbonizar",
    "carbonizan": "carbonizar",
    "carbonizando": "carbonizar",
    "carbonizara": "carbonizar",
    "carbonizarais": "carbonizar",
    "carbonizaran": "carbonizar",
    "carbonizaras": "carbonizar",
    "carbonizare": "carbonizar",
    "carbonizareis": "carbonizar",
    "carbonizaremos": "carbonizar",
    "carbonizaren": "carbonizar",
    "carbonizares": "carbonizar",
    "carbonizaron": "carbonizar",
    "carbonizará": "carbonizar",
    "carbonizarán": "carbonizar",
    "carbonizarás": "carbonizar",
    "carbonizaré": "carbonizar",
    "carbonizaréis": "carbonizar",
    "carbonizaría": "carbonizar",
    "carbonizaríais": "carbonizar",
    "carbonizaríamos": "carbonizar",
    "carbonizarían": "carbonizar",
    "carbonizarías": "carbonizar",
    "carbonizas": "carbonizar",
    "carbonizase": "carbonizar",
    "carbonizaseis": "carbonizar",
    "carbonizasen": "carbonizar",
    "carbonizases": "carbonizar",
    "carbonizaste": "carbonizar",
    "carbonizasteis": "carbonizar",
    "carbonizo": "carbonizar",
    "carbonizábamos": "carbonizar",
    "carbonizáis": "carbonizar",
    "carbonizáramos": "carbonizar",
    "carbonizáremos": "carbonizar",
    "carbonizásemos": "carbonizar",
    "carbonizó": "carbonizar",
    "carbono": "carbonar",
    "carbonoidea": "carbonoideo",
    "carbonoideas": "carbonoideo",
    "carbonoideos": "carbonoideo",
    "carbonos": "carbono",
    "carbonábamos": "carbonar",
    "carbonáis": "carbonar",
    "carbonáramos": "carbonar",
    "carbonáremos": "carbonar",
    "carbonásemos": "carbonar",
    "carboné": "carbonar",
    "carbonéis": "carbonar",
    "carbonífera": "carbonífero",
    "carboníferas": "carbonífero",
    "carboníferos": "carbonífero",
    "carbonó": "carbonar",
    "carborundos": "carborundo",
    "carboxihemoglobinas": "carboxihemoglobina",
    "carboxilaciones": "carboxilación",
    "carboxilos": "carboxilo",
    "carboximetilcelulosas": "carboximetilcelulosa",
    "carboxílica": "carboxílico",
    "carboxílicas": "carboxílico",
    "carboxílicos": "carboxílico",
    "carbuncales": "carbuncal",
    "carbunclos": "carbunclo",
    "carbuncos": "carbunco",
    "carbuncosa": "carbuncoso",
    "carbuncosas": "carbuncoso",
    "carbuncosos": "carbuncoso",
    "carbura": "carburar",
    "carburaba": "carburar",
    "carburabais": "carburar",
    "carburaban": "carburar",
    "carburabas": "carburar",
    "carburaciones": "carburación",
    "carburad": "carburar",
    "carburada": "carburar",
    "carburadas": "carburar",
    "carburado": "carburar",
    "carburadores": "carburador",
    "carburados": "carburar",
    "carburamos": "carburar",
    "carburan": "carburar",
    "carburando": "carburar",
    "carburantes": "carburante",
    "carburara": "carburar",
    "carburarais": "carburar",
    "carburaran": "carburar",
    "carburaras": "carburar",
    "carburare": "carburar",
    "carburareis": "carburar",
    "carburaremos": "carburar",
    "carburaren": "carburar",
    "carburares": "carburar",
    "carburaron": "carburar",
    "carburará": "carburar",
    "carburarán": "carburar",
    "carburarás": "carburar",
    "carburaré": "carburar",
    "carburaréis": "carburar",
    "carburaría": "carburar",
    "carburaríais": "carburar",
    "carburaríamos": "carburar",
    "carburarían": "carburar",
    "carburarías": "carburar",
    "carburas": "carburar",
    "carburase": "carburar",
    "carburaseis": "carburar",
    "carburasen": "carburar",
    "carburases": "carburar",
    "carburaste": "carburar",
    "carburasteis": "carburar",
    "carbure": "carburar",
    "carburemos": "carburar",
    "carburen": "carburar",
    "carbures": "carburar",
    "carburo": "carburar",
    "carburos": "carburo",
    "carburábamos": "carburar",
    "carburáis": "carburar",
    "carburáramos": "carburar",
    "carburáremos": "carburar",
    "carburásemos": "carburar",
    "carburé": "carburar",
    "carburéis": "carburar",
    "carburó": "carburar",
    "carbámica": "carbámico",
    "carbámicas": "carbámico",
    "carbámicos": "carbámico",
    "carbógenos": "carbógeno",
    "carbónica": "carbónico",
    "carbónicas": "carbónico",
    "carbónicos": "carbónico",
    "carbúnculos": "carbúnculo",
    "carcajadas": "carcajada",
    "carcajea": "carcajear",
    "carcajeaba": "carcajear",
    "carcajeabais": "carcajear",
    "carcajeaban": "carcajear",
    "carcajeabas": "carcajear",
    "carcajead": "carcajear",
    "carcajeada": "carcajear",
    "carcajeadas": "carcajear",
    "carcajeado": "carcajear",
    "carcajeados": "carcajear",
    "carcajeamos": "carcajear",
    "carcajean": "carcajear",
    "carcajeando": "carcajear",
    "carcajeara": "carcajear",
    "carcajearais": "carcajear",
    "carcajearan": "carcajear",
    "carcajearas": "carcajear",
    "carcajeare": "carcajear",
    "carcajeareis": "carcajear",
    "carcajearemos": "carcajear",
    "carcajearen": "carcajear",
    "carcajeares": "carcajear",
    "carcajearon": "carcajear",
    "carcajeará": "carcajear",
    "carcajearán": "carcajear",
    "carcajearás": "carcajear",
    "carcajearé": "carcajear",
    "carcajearéis": "carcajear",
    "carcajearía": "carcajear",
    "carcajearíais": "carcajear",
    "carcajearíamos": "carcajear",
    "carcajearían": "carcajear",
    "carcajearías": "carcajear",
    "carcajeas": "carcajear",
    "carcajease": "carcajear",
    "carcajeaseis": "carcajear",
    "carcajeasen": "carcajear",
    "carcajeases": "carcajear",
    "carcajeaste": "carcajear",
    "carcajeasteis": "carcajear",
    "carcajee": "carcajear",
    "carcajeemos": "carcajear",
    "carcajeen": "carcajear",
    "carcajees": "carcajear",
    "carcajeo": "carcajear",
    "carcajes": "carcaj",
    "carcajeábamos": "carcajear",
    "carcajeáis": "carcajear",
    "carcajeáramos": "carcajear",
    "carcajeáremos": "carcajear",
    "carcajeásemos": "carcajear",
    "carcajeé": "carcajear",
    "carcajeéis": "carcajear",
    "carcajeó": "carcajear",
    "carcamales": "carcamal",
    "carcamanes": "carcamán",
    "carcas": "carca",
    "carcasas": "carcasa",
    "carcavones": "carcavón",
    "carcavuezos": "carcavuezo",
    "carcelaria": "carcelario",
    "carcelarias": "carcelario",
    "carcelarios": "carcelario",
    "carcelera": "carcelero",
    "carceleras": "carcelero",
    "carceleros": "carcelero",
    "carcinoembrionaria": "carcinoembrionario",
    "carcinoembrionarias": "carcinoembrionario",
    "carcinoembrionarios": "carcinoembrionario",
    "carcinogénica": "carcinogénico",
    "carcinogénicas": "carcinogénico",
    "carcinogénicos": "carcinogénico",
    "carcinoides": "carcinoide",
    "carcinologías": "carcinología",
    "carcinológica": "carcinológico",
    "carcinológicas": "carcinológico",
    "carcinológicos": "carcinológico",
    "carcinomas": "carcinoma",
    "carcinomatosa": "carcinomatoso",
    "carcinomatosas": "carcinomatoso",
    "carcinomatosos": "carcinomatoso",
    "carcinógena": "carcinógeno",
    "carcinógenas": "carcinógeno",
    "carcinógenos": "carcinógeno",
    "carcinóloga": "carcinólogo",
    "carcinólogas": "carcinólogo",
    "carcinólogos": "carcinólogo",
    "carcoma": "carcomer",
    "carcomamos": "carcomer",
    "carcoman": "carcomer",
    "carcomas": "carcomer",
    "carcome": "carcomer",
    "carcomed": "carcomer",
    "carcomemos": "carcomer",
    "carcomen": "carcomer",
    "carcomeremos": "carcomer",
    "carcomerá": "carcomer",
    "carcomerán": "carcomer",
    "carcomerás": "carcomer",
    "carcomeré": "carcomer",
    "carcomeréis": "carcomer",
    "carcomería": "carcomer",
    "carcomeríais": "carcomer",
    "carcomeríamos": "carcomer",
    "carcomerían": "carcomer",
    "carcomerías": "carcomer",
    "carcomes": "carcomer",
    "carcomida": "carcomer",
    "carcomidas": "carcomer",
    "carcomido": "carcomer",
    "carcomidos": "carcomer",
    "carcomiendo": "carcomer",
    "carcomiera": "carcomer",
    "carcomierais": "carcomer",
    "carcomieran": "carcomer",
    "carcomieras": "carcomer",
    "carcomiere": "carcomer",
    "carcomiereis": "carcomer",
    "carcomieren": "carcomer",
    "carcomieres": "carcomer",
    "carcomieron": "carcomer",
    "carcomiese": "carcomer",
    "carcomieseis": "carcomer",
    "carcomiesen": "carcomer",
    "carcomieses": "carcomer",
    "carcomimos": "carcomer",
    "carcomiste": "carcomer",
    "carcomisteis": "carcomer",
    "carcomiéramos": "carcomer",
    "carcomiéremos": "carcomer",
    "carcomiésemos": "carcomer",
    "carcomió": "carcomer",
    "carcomo": "carcomer",
    "carcomáis": "carcomer",
    "carcoméis": "carcomer",
    "carcomí": "carcomer",
    "carcomía": "carcomer",
    "carcomíais": "carcomer",
    "carcomíamos": "carcomer",
    "carcomían": "carcomer",
    "carcomías": "carcomer",
    "carcones": "carcón",
    "carcundas": "carcunda",
    "carda": "cardar",
    "cardaba": "cardar",
    "cardabais": "cardar",
    "cardaban": "cardar",
    "cardabas": "cardar",
    "cardad": "cardar",
    "cardada": "cardar",
    "cardadas": "cardar",
    "cardado": "cardar",
    "cardadora": "cardador",
    "cardadoras": "cardador",
    "cardadores": "cardador",
    "cardados": "cardar",
    "cardaduras": "cardadura",
    "cardales": "cardal",
    "cardamomos": "cardamomo",
    "cardamos": "cardar",
    "cardan": "cardar",
    "cardando": "cardar",
    "cardanes": "cardán",
    "cardara": "cardar",
    "cardarais": "cardar",
    "cardaran": "cardar",
    "cardaras": "cardar",
    "cardare": "cardar",
    "cardareis": "cardar",
    "cardaremos": "cardar",
    "cardaren": "cardar",
    "cardares": "cardar",
    "cardaron": "cardar",
    "cardará": "cardar",
    "cardarán": "cardar",
    "cardarás": "cardar",
    "cardaré": "cardar",
    "cardaréis": "cardar",
    "cardaría": "cardar",
    "cardaríais": "cardar",
    "cardaríamos": "cardar",
    "cardarían": "cardar",
    "cardarías": "cardar",
    "cardas": "cardar",
    "cardase": "cardar",
    "cardaseis": "cardar",
    "cardasen": "cardar",
    "cardases": "cardar",
    "cardaste": "cardar",
    "cardasteis": "cardar",
    "carde": "cardar",
    "cardemos": "cardar",
    "carden": "cardar",
    "cardenalatos": "cardenalato",
    "cardenales": "cardenal",
    "cardenalicia": "cardenalicio",
    "cardenalicias": "cardenalicio",
    "cardenalicios": "cardenalicio",
    "cardenchales": "cardenchal",
    "cardenchas": "cardencha",
    "cardenillos": "cardenillo",
    "carderías": "cardería",
    "cardes": "cardar",
    "cardiaca": "cardiaco",
    "cardiacas": "cardiaco",
    "cardiacos": "cardiaco",
    "cardiales": "cardial",
    "cardialgias": "cardialgia",
    "cardillos": "cardillo",
    "cardinales": "cardinal",
    "cardinalidades": "cardinalidad",
    "cardio": "cardiología",
    "cardiodepresiones": "cardiodepresión",
    "cardiografías": "cardiografía",
    "cardiogramas": "cardiograma",
    "cardioides": "cardioide",
    "cardiologías": "cardiología",
    "cardiológica": "cardiológico",
    "cardiológicas": "cardiológico",
    "cardiológicos": "cardiológico",
    "cardiopatías": "cardiopatía",
    "cardioprotectora": "cardioprotector",
    "cardioprotectoras": "cardioprotector",
    "cardioprotectores": "cardioprotector",
    "cardioscopios": "cardioscopio",
    "cardiotoxicidades": "cardiotoxicidad",
    "cardiotónica": "cardiotónico",
    "cardiotónicas": "cardiotónico",
    "cardiotónicos": "cardiotónico",
    "cardiotóxica": "cardiotóxico",
    "cardiotóxicas": "cardiotóxico",
    "cardiotóxicos": "cardiotóxico",
    "cardiovasculares": "cardiovascular",
    "cardizales": "cardizal",
    "cardiálgica": "cardiálgico",
    "cardiálgicas": "cardiálgico",
    "cardiálgicos": "cardiálgico",
    "cardiógrafos": "cardiógrafo",
    "cardióloga": "cardiólogo",
    "cardiólogas": "cardiólogo",
    "cardiólogos": "cardiólogo",
    "cardiópatas": "cardiópata",
    "cardo": "cardar",
    "cardones": "cardón",
    "cardos": "cardo",
    "carduce": "carduzar",
    "carducemos": "carduzar",
    "carducen": "carduzar",
    "carduces": "carduzar",
    "carduchas": "carducha",
    "carducé": "carduzar",
    "carducéis": "carduzar",
    "carduza": "carduzar",
    "carduzaba": "carduzar",
    "carduzabais": "carduzar",
    "carduzaban": "carduzar",
    "carduzabas": "carduzar",
    "carduzad": "carduzar",
    "carduzada": "carduzar",
    "carduzadas": "carduzar",
    "carduzado": "carduzar",
    "carduzadora": "carduzador",
    "carduzadoras": "carduzador",
    "carduzadores": "carduzador",
    "carduzados": "carduzar",
    "carduzamos": "carduzar",
    "carduzan": "carduzar",
    "carduzando": "carduzar",
    "carduzara": "carduzar",
    "carduzarais": "carduzar",
    "carduzaran": "carduzar",
    "carduzaras": "carduzar",
    "carduzare": "carduzar",
    "carduzareis": "carduzar",
    "carduzaremos": "carduzar",
    "carduzaren": "carduzar",
    "carduzares": "carduzar",
    "carduzaron": "carduzar",
    "carduzará": "carduzar",
    "carduzarán": "carduzar",
    "carduzarás": "carduzar",
    "carduzaré": "carduzar",
    "carduzaréis": "carduzar",
    "carduzaría": "carduzar",
    "carduzaríais": "carduzar",
    "carduzaríamos": "carduzar",
    "carduzarían": "carduzar",
    "carduzarías": "carduzar",
    "carduzas": "carduzar",
    "carduzase": "carduzar",
    "carduzaseis": "carduzar",
    "carduzasen": "carduzar",
    "carduzases": "carduzar",
    "carduzaste": "carduzar",
    "carduzasteis": "carduzar",
    "carduzo": "carduzar",
    "carduzábamos": "carduzar",
    "carduzáis": "carduzar",
    "carduzáramos": "carduzar",
    "carduzáremos": "carduzar",
    "carduzásemos": "carduzar",
    "carduzó": "carduzar",
    "cardábamos": "cardar",
    "cardáis": "cardar",
    "cardánica": "cardánico",
    "cardánicas": "cardánico",
    "cardánicos": "cardánico",
    "cardáramos": "cardar",
    "cardáremos": "cardar",
    "cardásemos": "cardar",
    "cardé": "cardar",
    "cardéis": "cardar",
    "cardíaca": "cardíaco",
    "cardíacas": "cardíaco",
    "cardíacos": "cardíaco",
    "cardó": "cardar",
    "cardúmenes": "cardumen",
    "carea": "carear",
    "careaba": "carear",
    "careabais": "carear",
    "careaban": "carear",
    "careabas": "carear",
    "caread": "carear",
    "careada": "carear",
    "careadas": "carear",
    "careado": "carear",
    "careados": "carear",
    "careamos": "carear",
    "carean": "carear",
    "careando": "carear",
    "careara": "carear",
    "carearais": "carear",
    "carearan": "carear",
    "carearas": "carear",
    "careare": "carear",
    "careareis": "carear",
    "carearemos": "carear",
    "carearen": "carear",
    "careares": "carear",
    "carearon": "carear",
    "careará": "carear",
    "carearán": "carear",
    "carearás": "carear",
    "carearé": "carear",
    "carearéis": "carear",
    "carearía": "carear",
    "carearíais": "carear",
    "carearíamos": "carear",
    "carearían": "carear",
    "carearías": "carear",
    "careas": "carear",
    "carease": "carear",
    "careaseis": "carear",
    "careasen": "carear",
    "careases": "carear",
    "careaste": "carear",
    "careasteis": "carear",
    "carece": "carecer",
    "careced": "carecer",
    "carecemos": "carecer",
    "carecen": "carecer",
    "careceremos": "carecer",
    "carecerá": "carecer",
    "carecerán": "carecer",
    "carecerás": "carecer",
    "careceré": "carecer",
    "careceréis": "carecer",
    "carecería": "carecer",
    "careceríais": "carecer",
    "careceríamos": "carecer",
    "carecerían": "carecer",
    "carecerías": "carecer",
    "careces": "carecer",
    "carecida": "carecer",
    "carecidas": "carecer",
    "carecido": "carecer",
    "carecidos": "carecer",
    "careciendo": "carecer",
    "careciera": "carecer",
    "carecierais": "carecer",
    "carecieran": "carecer",
    "carecieras": "carecer",
    "careciere": "carecer",
    "careciereis": "carecer",
    "carecieren": "carecer",
    "carecieres": "carecer",
    "carecieron": "carecer",
    "careciese": "carecer",
    "carecieseis": "carecer",
    "careciesen": "carecer",
    "carecieses": "carecer",
    "carecimientos": "carecimiento",
    "carecimos": "carecer",
    "careciste": "carecer",
    "carecisteis": "carecer",
    "careciéramos": "carecer",
    "careciéremos": "carecer",
    "careciésemos": "carecer",
    "careció": "carecer",
    "carecéis": "carecer",
    "carecí": "carecer",
    "carecía": "carecer",
    "carecíais": "carecer",
    "carecíamos": "carecer",
    "carecían": "carecer",
    "carecías": "carecer",
    "caree": "carear",
    "careemos": "carear",
    "careen": "carear",
    "carees": "carear",
    "careles": "carel",
    "carena": "carenar",
    "carenaba": "carenar",
    "carenabais": "carenar",
    "carenaban": "carenar",
    "carenabas": "carenar",
    "carenad": "carenar",
    "carenada": "carenar",
    "carenadas": "carenar",
    "carenado": "carenar",
    "carenados": "carenar",
    "carenaduras": "carenadura",
    "carenajes": "carenaje",
    "carenamos": "carenar",
    "carenan": "carenar",
    "carenando": "carenar",
    "carenara": "carenar",
    "carenarais": "carenar",
    "carenaran": "carenar",
    "carenaras": "carenar",
    "carenare": "carenar",
    "carenareis": "carenar",
    "carenaremos": "carenar",
    "carenaren": "carenar",
    "carenares": "carenar",
    "carenaron": "carenar",
    "carenará": "carenar",
    "carenarán": "carenar",
    "carenarás": "carenar",
    "carenaré": "carenar",
    "carenaréis": "carenar",
    "carenaría": "carenar",
    "carenaríais": "carenar",
    "carenaríamos": "carenar",
    "carenarían": "carenar",
    "carenarías": "carenar",
    "carenas": "carenar",
    "carenase": "carenar",
    "carenaseis": "carenar",
    "carenasen": "carenar",
    "carenases": "carenar",
    "carenaste": "carenar",
    "carenasteis": "carenar",
    "carenciales": "carencial",
    "carencias": "carencia",
    "carene": "carenar",
    "carenemos": "carenar",
    "carenen": "carenar",
    "careneros": "carenero",
    "carenes": "carenar",
    "careno": "carenar",
    "carenotes": "carenote",
    "carentes": "carente",
    "carenábamos": "carenar",
    "carenáis": "carenar",
    "carenáramos": "carenar",
    "carenáremos": "carenar",
    "carenásemos": "carenar",
    "carené": "carenar",
    "carenéis": "carenar",
    "carenó": "carenar",
    "careo": "carear",
    "careos": "careo",
    "carera": "carero",
    "careras": "carero",
    "careros": "carero",
    "cares": "car",
    "carestías": "carestía",
    "caretas": "careta",
    "caretos": "careto",
    "careyes": "carey",
    "carezca": "carecer",
    "carezcamos": "carecer",
    "carezcan": "carecer",
    "carezcas": "carecer",
    "carezco": "carecer",
    "carezcáis": "carecer",
    "careábamos": "carear",
    "careáis": "carear",
    "careáramos": "carear",
    "careáremos": "carear",
    "careásemos": "carear",
    "careé": "carear",
    "careéis": "carear",
    "careó": "carear",
    "carga": "cargar",
    "cargaba": "cargar",
    "cargabais": "cargar",
    "cargaban": "cargar",
    "cargabas": "cargar",
    "cargabilidades": "cargabilidad",
    "cargables": "cargable",
    "cargad": "cargar",
    "cargada": "cargar",
    "cargadas": "cargar",
    "cargaderas": "cargadera",
    "cargaderos": "cargadero",
    "cargadillas": "cargadilla",
    "cargado": "cargar",
    "cargadora": "cargador",
    "cargadoras": "cargadora",
    "cargadores": "cargador",
    "cargados": "cargar",
    "cargamentos": "cargamento",
    "cargamos": "cargar",
    "cargan": "cargar",
    "cargando": "cargar",
    "cargantes": "cargante",
    "cargara": "cargar",
    "cargarais": "cargar",
    "cargaran": "cargar",
    "cargaras": "cargar",
    "cargare": "cargar",
    "cargareis": "cargar",
    "cargaremes": "cargareme",
    "cargaremos": "cargar",
    "cargaren": "cargar",
    "cargares": "cargar",
    "cargaron": "cargar",
    "cargará": "cargar",
    "cargarán": "cargar",
    "cargarás": "cargar",
    "cargaré": "cargar",
    "cargaréis": "cargar",
    "cargaría": "cargar",
    "cargaríais": "cargar",
    "cargaríamos": "cargar",
    "cargarían": "cargar",
    "cargarías": "cargar",
    "cargas": "cargar",
    "cargase": "cargar",
    "cargaseis": "cargar",
    "cargasen": "cargar",
    "cargases": "cargar",
    "cargaste": "cargar",
    "cargasteis": "cargar",
    "cargazones": "cargazón",
    "cargo": "cargar",
    "cargos": "cargo",
    "cargosa": "cargoso",
    "cargosas": "cargoso",
    "cargosea": "cargosear",
    "cargoseaba": "cargosear",
    "cargoseabais": "cargosear",
    "cargoseaban": "cargosear",
    "cargoseabas": "cargosear",
    "cargosead": "cargosear",
    "cargoseada": "cargosear",
    "cargoseadas": "cargosear",
    "cargoseado": "cargosear",
    "cargoseados": "cargosear",
    "cargoseamos": "cargosear",
    "cargosean": "cargosear",
    "cargoseando": "cargosear",
    "cargoseara": "cargosear",
    "cargosearais": "cargosear",
    "cargosearan": "cargosear",
    "cargosearas": "cargosear",
    "cargoseare": "cargosear",
    "cargoseareis": "cargosear",
    "cargosearemos": "cargosear",
    "cargosearen": "cargosear",
    "cargoseares": "cargosear",
    "cargosearon": "cargosear",
    "cargoseará": "cargosear",
    "cargosearán": "cargosear",
    "cargosearás": "cargosear",
    "cargosearé": "cargosear",
    "cargosearéis": "cargosear",
    "cargosearía": "cargosear",
    "cargosearíais": "cargosear",
    "cargosearíamos": "cargosear",
    "cargosearían": "cargosear",
    "cargosearías": "cargosear",
    "cargoseas": "cargosear",
    "cargosease": "cargosear",
    "cargoseaseis": "cargosear",
    "cargoseasen": "cargosear",
    "cargoseases": "cargosear",
    "cargoseaste": "cargosear",
    "cargoseasteis": "cargosear",
    "cargosee": "cargosear",
    "cargoseemos": "cargosear",
    "cargoseen": "cargosear",
    "cargosees": "cargosear",
    "cargoseo": "cargosear",
    "cargoseábamos": "cargosear",
    "cargoseáis": "cargosear",
    "cargoseáramos": "cargosear",
    "cargoseáremos": "cargosear",
    "cargoseásemos": "cargosear",
    "cargoseé": "cargosear",
    "cargoseéis": "cargosear",
    "cargoseó": "cargosear",
    "cargosos": "cargoso",
    "cargue": "cargar",
    "carguemos": "cargar",
    "carguen": "cargar",
    "cargueros": "carguero",
    "cargues": "cargar",
    "cargué": "cargar",
    "carguéis": "cargar",
    "carguíos": "carguío",
    "cargábamos": "cargar",
    "cargáis": "cargar",
    "cargáramos": "cargar",
    "cargáremos": "cargar",
    "cargásemos": "cargar",
    "cargó": "cargar",
    "caria": "cariar",
    "cariaba": "cariar",
    "cariabais": "cariar",
    "cariaban": "cariar",
    "cariabas": "cariar",
    "cariaceda": "cariacedo",
    "cariacedas": "cariacedo",
    "cariacedos": "cariacedo",
    "cariacontecida": "cariacontecido",
    "cariacontecidas": "cariacontecido",
    "cariacontecidos": "cariacontecido",
    "cariad": "cariar",
    "cariada": "cariar",
    "cariadas": "cariar",
    "cariado": "cariar",
    "cariados": "cariar",
    "cariaduras": "cariadura",
    "cariamos": "cariar",
    "carian": "cariar",
    "cariando": "cariar",
    "cariara": "cariar",
    "cariarais": "cariar",
    "cariaran": "cariar",
    "cariaras": "cariar",
    "cariare": "cariar",
    "cariareis": "cariar",
    "cariaremos": "cariar",
    "cariaren": "cariar",
    "cariares": "cariar",
    "cariaron": "cariar",
    "cariará": "cariar",
    "cariarán": "cariar",
    "cariarás": "cariar",
    "cariaré": "cariar",
    "cariaréis": "cariar",
    "cariaría": "cariar",
    "cariaríais": "cariar",
    "cariaríamos": "cariar",
    "cariarían": "cariar",
    "cariarías": "cariar",
    "carias": "cariar",
    "cariase": "cariar",
    "cariaseis": "cariar",
    "cariasen": "cariar",
    "cariases": "cariar",
    "cariaste": "cariar",
    "cariasteis": "cariar",
    "caribeña": "caribeño",
    "caribeñas": "caribeño",
    "caribeños": "caribeño",
    "caribúes": "caribú",
    "caribús": "caribú",
    "caricatos": "caricato",
    "caricatura": "caricaturar",
    "caricaturaba": "caricaturar",
    "caricaturabais": "caricaturar",
    "caricaturaban": "caricaturar",
    "caricaturabas": "caricaturar",
    "caricaturad": "caricaturar",
    "caricaturada": "caricaturar",
    "caricaturadas": "caricaturar",
    "caricaturado": "caricaturar",
    "caricaturados": "caricaturar",
    "caricaturamos": "caricaturar",
    "caricaturan": "caricaturar",
    "caricaturando": "caricaturar",
    "caricaturara": "caricaturar",
    "caricaturarais": "caricaturar",
    "caricaturaran": "caricaturar",
    "caricaturaras": "caricaturar",
    "caricaturare": "caricaturar",
    "caricaturareis": "caricaturar",
    "caricaturaremos": "caricaturar",
    "caricaturaren": "caricaturar",
    "caricaturares": "caricaturar",
    "caricaturaron": "caricaturar",
    "caricaturará": "caricaturar",
    "caricaturarán": "caricaturar",
    "caricaturarás": "caricaturar",
    "caricaturaré": "caricaturar",
    "caricaturaréis": "caricaturar",
    "caricaturaría": "caricaturar",
    "caricaturaríais": "caricaturar",
    "caricaturaríamos": "caricaturar",
    "caricaturarían": "caricaturar",
    "caricaturarías": "caricaturar",
    "caricaturas": "caricaturar",
    "caricaturase": "caricaturar",
    "caricaturaseis": "caricaturar",
    "caricaturasen": "caricaturar",
    "caricaturases": "caricaturar",
    "caricaturaste": "caricaturar",
    "caricaturasteis": "caricaturar",
    "caricature": "caricaturar",
    "caricaturemos": "caricaturar",
    "caricaturen": "caricaturar",
    "caricatures": "caricaturar",
    "caricaturesca": "caricaturesco",
    "caricaturescas": "caricaturesco",
    "caricaturescos": "caricaturesco",
    "caricaturice": "caricaturizar",
    "caricaturicemos": "caricaturizar",
    "caricaturicen": "caricaturizar",
    "caricaturices": "caricaturizar",
    "caricaturicé": "caricaturizar",
    "caricaturicéis": "caricaturizar",
    "caricaturistas": "caricaturista",
    "caricaturiza": "caricaturizar",
    "caricaturizaba": "caricaturizar",
    "caricaturizabais": "caricaturizar",
    "caricaturizaban": "caricaturizar",
    "caricaturizabas": "caricaturizar",
    "caricaturizad": "caricaturizar",
    "caricaturizada": "caricaturizar",
    "caricaturizadas": "caricaturizar",
    "caricaturizado": "caricaturizar",
    "caricaturizados": "caricaturizar",
    "caricaturizamos": "caricaturizar",
    "caricaturizan": "caricaturizar",
    "caricaturizando": "caricaturizar",
    "caricaturizara": "caricaturizar",
    "caricaturizarais": "caricaturizar",
    "caricaturizaran": "caricaturizar",
    "caricaturizaras": "caricaturizar",
    "caricaturizare": "caricaturizar",
    "caricaturizareis": "caricaturizar",
    "caricaturizaremos": "caricaturizar",
    "caricaturizaren": "caricaturizar",
    "caricaturizares": "caricaturizar",
    "caricaturizaron": "caricaturizar",
    "caricaturizará": "caricaturizar",
    "caricaturizarán": "caricaturizar",
    "caricaturizarás": "caricaturizar",
    "caricaturizaré": "caricaturizar",
    "caricaturizaréis": "caricaturizar",
    "caricaturizaría": "caricaturizar",
    "caricaturizaríais": "caricaturizar",
    "caricaturizaríamos": "caricaturizar",
    "caricaturizarían": "caricaturizar",
    "caricaturizarías": "caricaturizar",
    "caricaturizas": "caricaturizar",
    "caricaturizase": "caricaturizar",
    "caricaturizaseis": "caricaturizar",
    "caricaturizasen": "caricaturizar",
    "caricaturizases": "caricaturizar",
    "caricaturizaste": "caricaturizar",
    "caricaturizasteis": "caricaturizar",
    "caricaturizo": "caricaturizar",
    "caricaturizábamos": "caricaturizar",
    "caricaturizáis": "caricaturizar",
    "caricaturizáramos": "caricaturizar",
    "caricaturizáremos": "caricaturizar",
    "caricaturizásemos": "caricaturizar",
    "caricaturizó": "caricaturizar",
    "caricaturo": "caricaturar",
    "caricaturábamos": "caricaturar",
    "caricaturáis": "caricaturar",
    "caricaturáramos": "caricaturar",
    "caricaturáremos": "caricaturar",
    "caricaturásemos": "caricaturar",
    "caricaturé": "caricaturar",
    "caricaturéis": "caricaturar",
    "caricaturó": "caricaturar",
    "carices": "cariz",
    "caricias": "caricia",
    "caricáceas": "caricácea",
    "caridades": "caridad",
    "carie": "cariar",
    "cariemos": "cariar",
    "carien": "cariar",
    "carientismos": "carientismo",
    "caries": "cariar",
    "carilarga": "carilargo",
    "carilargas": "carilargo",
    "carilargos": "carilargo",
    "carilla": "carillo",
    "carillas": "carillo",
    "carillones": "carillón",
    "carillos": "carillo",
    "carinas": "carina",
    "cario": "cariar",
    "cariocas": "carioca",
    "cariocinética": "cariocinético",
    "cariocinéticas": "cariocinético",
    "cariocinéticos": "cariocinético",
    "cariofilenos": "cariofileno",
    "cariofiláceas": "cariofilácea",
    "cariogamias": "cariogamia",
    "cariogénica": "cariogénico",
    "cariogénicas": "cariogénico",
    "cariogénicos": "cariogénico",
    "cariologías": "cariología",
    "cariopareja": "carioparejo",
    "carioparejas": "carioparejo",
    "carioparejos": "carioparejo",
    "cariosomas": "cariosoma",
    "cariostática": "cariostático",
    "cariostáticas": "cariostático",
    "cariostáticos": "cariostático",
    "cariotinas": "cariotina",
    "cariotipos": "cariotipo",
    "carirraída": "carirraído",
    "carirraídas": "carirraído",
    "carirraídos": "carirraído",
    "carirredonda": "carirredondo",
    "carirredondas": "carirredondo",
    "carirredondos": "carirredondo",
    "carismas": "carisma",
    "carismática": "carismático",
    "carismáticas": "carismático",
    "carismáticos": "carismático",
    "caritas": "carita",
    "caritativa": "caritativo",
    "caritativas": "caritativo",
    "caritativos": "caritativo",
    "cariábamos": "cariar",
    "cariáis": "cariar",
    "cariáramos": "cariar",
    "cariáremos": "cariar",
    "cariásemos": "cariar",
    "cariátides": "cariátide",
    "carié": "cariar",
    "cariéis": "cariar",
    "cariñanas": "cariñana",
    "cariñenas": "cariñena",
    "cariños": "cariño",
    "cariñosa": "cariñoso",
    "cariñosas": "cariñoso",
    "cariñosos": "cariñoso",
    "carió": "cariar",
    "cariógena": "cariógeno",
    "cariógenas": "cariógeno",
    "cariógenos": "cariógeno",
    "cariópsides": "cariópside",
    "carlancas": "carlanca",
    "carlancona": "carlancón",
    "carlanconas": "carlancón",
    "carlancones": "carlancón",
    "carlas": "carla",
    "carlea": "carlear",
    "carleaba": "carlear",
    "carleabais": "carlear",
    "carleaban": "carlear",
    "carleabas": "carlear",
    "carlead": "carlear",
    "carleada": "carlear",
    "carleadas": "carlear",
    "carleado": "carlear",
    "carleados": "carlear",
    "carleamos": "carlear",
    "carlean": "carlear",
    "carleando": "carlear",
    "carleara": "carlear",
    "carlearais": "carlear",
    "carlearan": "carlear",
    "carlearas": "carlear",
    "carleare": "carlear",
    "carleareis": "carlear",
    "carlearemos": "carlear",
    "carlearen": "carlear",
    "carleares": "carlear",
    "carlearon": "carlear",
    "carleará": "carlear",
    "carlearán": "carlear",
    "carlearás": "carlear",
    "carlearé": "carlear",
    "carlearéis": "carlear",
    "carlearía": "carlear",
    "carlearíais": "carlear",
    "carlearíamos": "carlear",
    "carlearían": "carlear",
    "carlearías": "carlear",
    "carleas": "carlear",
    "carlease": "carlear",
    "carleaseis": "carlear",
    "carleasen": "carlear",
    "carleases": "carlear",
    "carleaste": "carlear",
    "carleasteis": "carlear",
    "carlee": "carlear",
    "carleemos": "carlear",
    "carleen": "carlear",
    "carlees": "carlear",
    "carleo": "carlear",
    "carletas": "carleta",
    "carleábamos": "carlear",
    "carleáis": "carlear",
    "carleáramos": "carlear",
    "carleáremos": "carlear",
    "carleásemos": "carlear",
    "carleé": "carlear",
    "carleéis": "carlear",
    "carleó": "carlear",
    "carlinas": "carlina",
    "carlingas": "carlinga",
    "carlismos": "carlismo",
    "carlistas": "carlista",
    "carlitas": "carlita",
    "carlovingia": "carlovingio",
    "carlovingias": "carlovingio",
    "carlovingios": "carlovingio",
    "carmañolas": "carmañola",
    "carmelinas": "carmelina",
    "carmelitas": "carmelita",
    "carmena": "carmenar",
    "carmenaba": "carmenar",
    "carmenabais": "carmenar",
    "carmenaban": "carmenar",
    "carmenabas": "carmenar",
    "carmenad": "carmenar",
    "carmenada": "carmenar",
    "carmenadas": "carmenar",
    "carmenado": "carmenar",
    "carmenadora": "carmenador",
    "carmenadoras": "carmenador",
    "carmenadores": "carmenador",
    "carmenados": "carmenar",
    "carmenaduras": "carmenadura",
    "carmenamos": "carmenar",
    "carmenan": "carmenar",
    "carmenando": "carmenar",
    "carmenara": "carmenar",
    "carmenarais": "carmenar",
    "carmenaran": "carmenar",
    "carmenaras": "carmenar",
    "carmenare": "carmenar",
    "carmenareis": "carmenar",
    "carmenaremos": "carmenar",
    "carmenaren": "carmenar",
    "carmenares": "carmenar",
    "carmenaron": "carmenar",
    "carmenará": "carmenar",
    "carmenarán": "carmenar",
    "carmenarás": "carmenar",
    "carmenaré": "carmenar",
    "carmenaréis": "carmenar",
    "carmenaría": "carmenar",
    "carmenaríais": "carmenar",
    "carmenaríamos": "carmenar",
    "carmenarían": "carmenar",
    "carmenarías": "carmenar",
    "carmenas": "carmenar",
    "carmenase": "carmenar",
    "carmenaseis": "carmenar",
    "carmenasen": "carmenar",
    "carmenases": "carmenar",
    "carmenaste": "carmenar",
    "carmenasteis": "carmenar",
    "carmene": "carmenar",
    "carmenemos": "carmenar",
    "carmenen": "carmenar",
    "carmenes": "carmenar",
    "carmeno": "carmenar",
    "carmenábamos": "carmenar",
    "carmenáis": "carmenar",
    "carmenáramos": "carmenar",
    "carmenáremos": "carmenar",
    "carmenásemos": "carmenar",
    "carmené": "carmenar",
    "carmenéis": "carmenar",
    "carmenó": "carmenar",
    "carmesíes": "carmesí",
    "carmesís": "carmesí",
    "carmesíses": "carmesí",
    "carminativa": "carminativo",
    "carminativas": "carminativo",
    "carminativos": "carminativo",
    "carmines": "carmín",
    "carnaciones": "carnación",
    "carnadas": "carnada",
    "carnaduras": "carnadura",
    "carnajes": "carnaje",
    "carnales": "carnal",
    "carnalidades": "carnalidad",
    "carnalitas": "carnalita",
    "carnaubas": "carnauba",
    "carnavaladas": "carnavalada",
    "carnavales": "carnaval",
    "carnavalesca": "carnavalesco",
    "carnavalescas": "carnavalesco",
    "carnavalescos": "carnavalesco",
    "carnazas": "carnaza",
    "carnea": "carnear",
    "carneaba": "carnear",
    "carneabais": "carnear",
    "carneaban": "carnear",
    "carneabas": "carnear",
    "carnead": "carnear",
    "carneada": "carnear",
    "carneadas": "carnear",
    "carneado": "carnear",
    "carneados": "carnear",
    "carneamos": "carnear",
    "carnean": "carnear",
    "carneando": "carnear",
    "carneara": "carnear",
    "carnearais": "carnear",
    "carnearan": "carnear",
    "carnearas": "carnear",
    "carneare": "carnear",
    "carneareis": "carnear",
    "carnearemos": "carnear",
    "carnearen": "carnear",
    "carneares": "carnear",
    "carnearon": "carnear",
    "carneará": "carnear",
    "carnearán": "carnear",
    "carnearás": "carnear",
    "carnearé": "carnear",
    "carnearéis": "carnear",
    "carnearía": "carnear",
    "carnearíais": "carnear",
    "carnearíamos": "carnear",
    "carnearían": "carnear",
    "carnearías": "carnear",
    "carneas": "carnear",
    "carnease": "carnear",
    "carneaseis": "carnear",
    "carneasen": "carnear",
    "carneases": "carnear",
    "carneaste": "carnear",
    "carneasteis": "carnear",
    "carnecillas": "carnecilla",
    "carnee": "carnear",
    "carneemos": "carnear",
    "carneen": "carnear",
    "carnees": "carnear",
    "carneo": "carnear",
    "carneradas": "carnerada",
    "carnerea": "carnerear",
    "carnereaba": "carnerear",
    "carnereabais": "carnerear",
    "carnereaban": "carnerear",
    "carnereabas": "carnerear",
    "carneread": "carnerear",
    "carnereada": "carnerear",
    "carnereadas": "carnerear",
    "carnereado": "carnerear",
    "carnereados": "carnerear",
    "carnereamientos": "carnereamiento",
    "carnereamos": "carnerear",
    "carnerean": "carnerear",
    "carnereando": "carnerear",
    "carnereara": "carnerear",
    "carnerearais": "carnerear",
    "carnerearan": "carnerear",
    "carnerearas": "carnerear",
    "carnereare": "carnerear",
    "carnereareis": "carnerear",
    "carnerearemos": "carnerear",
    "carnerearen": "carnerear",
    "carnereares": "carnerear",
    "carnerearon": "carnerear",
    "carnereará": "carnerear",
    "carnerearán": "carnerear",
    "carnerearás": "carnerear",
    "carnerearé": "carnerear",
    "carnerearéis": "carnerear",
    "carnerearía": "carnerear",
    "carnerearíais": "carnerear",
    "carnerearíamos": "carnerear",
    "carnerearían": "carnerear",
    "carnerearías": "carnerear",
    "carnereas": "carnerear",
    "carnerease": "carnerear",
    "carnereaseis": "carnerear",
    "carnereasen": "carnerear",
    "carnereases": "carnerear",
    "carnereaste": "carnerear",
    "carnereasteis": "carnerear",
    "carneree": "carnerear",
    "carnereemos": "carnerear",
    "carnereen": "carnerear",
    "carnerees": "carnerear",
    "carnereo": "carnerear",
    "carnerera": "carnerero",
    "carnereras": "carnerero",
    "carnereros": "carnerero",
    "carnereábamos": "carnerear",
    "carnereáis": "carnerear",
    "carnereáramos": "carnerear",
    "carnereáremos": "carnerear",
    "carnereásemos": "carnerear",
    "carnereé": "carnerear",
    "carnereéis": "carnerear",
    "carnereó": "carnerear",
    "carneros": "carnero",
    "carneruna": "carneruno",
    "carnerunas": "carneruno",
    "carnerunos": "carneruno",
    "carnes": "carne",
    "carnets": "carnet",
    "carneábamos": "carnear",
    "carneáis": "carnear",
    "carneáramos": "carnear",
    "carneáremos": "carnear",
    "carneásemos": "carnear",
    "carneé": "carnear",
    "carneéis": "carnear",
    "carneó": "carnear",
    "carnicera": "carnicero",
    "carniceras": "carnicero",
    "carniceros": "carnicero",
    "carnicerías": "carnicería",
    "carnicoles": "carnicol",
    "carnifica": "carnificar",
    "carnificaba": "carnificar",
    "carnificabais": "carnificar",
    "carnificaban": "carnificar",
    "carnificabas": "carnificar",
    "carnificaciones": "carnificación",
    "carnificad": "carnificar",
    "carnificada": "carnificar",
    "carnificadas": "carnificar",
    "carnificado": "carnificar",
    "carnificados": "carnificar",
    "carnificamos": "carnificar",
    "carnifican": "carnificar",
    "carnificando": "carnificar",
    "carnificara": "carnificar",
    "carnificarais": "carnificar",
    "carnificaran": "carnificar",
    "carnificaras": "carnificar",
    "carnificare": "carnificar",
    "carnificareis": "carnificar",
    "carnificaremos": "carnificar",
    "carnificaren": "carnificar",
    "carnificares": "carnificar",
    "carnificaron": "carnificar",
    "carnificará": "carnificar",
    "carnificarán": "carnificar",
    "carnificarás": "carnificar",
    "carnificaré": "carnificar",
    "carnificaréis": "carnificar",
    "carnificaría": "carnificar",
    "carnificaríais": "carnificar",
    "carnificaríamos": "carnificar",
    "carnificarían": "carnificar",
    "carnificarías": "carnificar",
    "carnificas": "carnificar",
    "carnificase": "carnificar",
    "carnificaseis": "carnificar",
    "carnificasen": "carnificar",
    "carnificases": "carnificar",
    "carnificaste": "carnificar",
    "carnificasteis": "carnificar",
    "carnifico": "carnificar",
    "carnificábamos": "carnificar",
    "carnificáis": "carnificar",
    "carnificáramos": "carnificar",
    "carnificáremos": "carnificar",
    "carnificásemos": "carnificar",
    "carnificó": "carnificar",
    "carnifique": "carnificar",
    "carnifiquemos": "carnificar",
    "carnifiquen": "carnificar",
    "carnifiques": "carnificar",
    "carnifiqué": "carnificar",
    "carnifiquéis": "carnificar",
    "carnizas": "carniza",
    "carnosa": "carnoso",
    "carnosas": "carnoso",
    "carnosidades": "carnosidad",
    "carnosos": "carnoso",
    "carnotitas": "carnotita",
    "carnés": "carné",
    "carnívora": "carnívoro",
    "carnívoras": "carnívoro",
    "carnívoros": "carnívoro",
    "carocas": "caroca",
    "carochas": "carocha",
    "carofíceas": "carofícea",
    "carolas": "carola",
    "carolingia": "carolingio",
    "carolingias": "carolingio",
    "carolingios": "carolingio",
    "caromomias": "caromomia",
    "caronas": "carona",
    "caros": "caro",
    "carotas": "carota",
    "carotenoides": "carotenoide",
    "carotenos": "caroteno",
    "caroticotimpánica": "caroticotimpánico",
    "caroticotimpánicas": "caroticotimpánico",
    "caroticotimpánicos": "caroticotimpánico",
    "carotidotimpánica": "carotidotimpánico",
    "carotidotimpánicas": "carotidotimpánico",
    "carotidotimpánicos": "carotidotimpánico",
    "carotinas": "carotina",
    "carotídea": "carotídeo",
    "carotídeas": "carotídeo",
    "carotídeos": "carotídeo",
    "carozos": "carozo",
    "caroñosa": "caroñoso",
    "caroñosas": "caroñoso",
    "caroñosos": "caroñoso",
    "carpa": "carpir",
    "carpada": "carpado",
    "carpadas": "carpado",
    "carpados": "carpado",
    "carpamos": "carpir",
    "carpan": "carpir",
    "carpaneles": "carpanel",
    "carpantas": "carpanta",
    "carpas": "carpir",
    "carpe": "carpir",
    "carpelos": "carpelo",
    "carpen": "carpir",
    "carpes": "carpir",
    "carpetanos": "carpetano",
    "carpetas": "carpeta",
    "carpetazos": "carpetazo",
    "carpetovetónica": "carpetovetónico",
    "carpetovetónicas": "carpetovetónico",
    "carpetovetónicos": "carpetovetónico",
    "carpiana": "carpiano",
    "carpianas": "carpiano",
    "carpianos": "carpiano",
    "carpid": "carpir",
    "carpida": "carpir",
    "carpidas": "carpir",
    "carpido": "carpir",
    "carpidos": "carpir",
    "carpiendo": "carpir",
    "carpiera": "carpir",
    "carpierais": "carpir",
    "carpieran": "carpir",
    "carpieras": "carpir",
    "carpiere": "carpir",
    "carpiereis": "carpir",
    "carpieren": "carpir",
    "carpieres": "carpir",
    "carpieron": "carpir",
    "carpiese": "carpir",
    "carpieseis": "carpir",
    "carpiesen": "carpir",
    "carpieses": "carpir",
    "carpimos": "carpir",
    "carpintea": "carpintear",
    "carpinteaba": "carpintear",
    "carpinteabais": "carpintear",
    "carpinteaban": "carpintear",
    "carpinteabas": "carpintear",
    "carpintead": "carpintear",
    "carpinteada": "carpintear",
    "carpinteadas": "carpintear",
    "carpinteado": "carpintear",
    "carpinteados": "carpintear",
    "carpinteamos": "carpintear",
    "carpintean": "carpintear",
    "carpinteando": "carpintear",
    "carpinteara": "carpintear",
    "carpintearais": "carpintear",
    "carpintearan": "carpintear",
    "carpintearas": "carpintear",
    "carpinteare": "carpintear",
    "carpinteareis": "carpintear",
    "carpintearemos": "carpintear",
    "carpintearen": "carpintear",
    "carpinteares": "carpintear",
    "carpintearon": "carpintear",
    "carpinteará": "carpintear",
    "carpintearán": "carpintear",
    "carpintearás": "carpintear",
    "carpintearé": "carpintear",
    "carpintearéis": "carpintear",
    "carpintearía": "carpintear",
    "carpintearíais": "carpintear",
    "carpintearíamos": "carpintear",
    "carpintearían": "carpintear",
    "carpintearías": "carpintear",
    "carpinteas": "carpintear",
    "carpintease": "carpintear",
    "carpinteaseis": "carpintear",
    "carpinteasen": "carpintear",
    "carpinteases": "carpintear",
    "carpinteaste": "carpintear",
    "carpinteasteis": "carpintear",
    "carpintee": "carpintear",
    "carpinteemos": "carpintear",
    "carpinteen": "carpintear",
    "carpintees": "carpintear",
    "carpinteo": "carpintear",
    "carpintera": "carpintero",
    "carpinteras": "carpintero",
    "carpinteriles": "carpinteril",
    "carpinteros": "carpintero",
    "carpinterías": "carpintería",
    "carpinteábamos": "carpintear",
    "carpinteáis": "carpintear",
    "carpinteáramos": "carpintear",
    "carpinteáremos": "carpintear",
    "carpinteásemos": "carpintear",
    "carpinteé": "carpintear",
    "carpinteéis": "carpintear",
    "carpinteó": "carpintear",
    "carpiremos": "carpir",
    "carpirá": "carpir",
    "carpirán": "carpir",
    "carpirás": "carpir",
    "carpiré": "carpir",
    "carpiréis": "carpir",
    "carpiría": "carpir",
    "carpiríais": "carpir",
    "carpiríamos": "carpir",
    "carpirían": "carpir",
    "carpirías": "carpir",
    "carpiste": "carpir",
    "carpisteis": "carpir",
    "carpiéramos": "carpir",
    "carpiéremos": "carpir",
    "carpiésemos": "carpir",
    "carpió": "carpir",
    "carpo": "carpir",
    "carpologías": "carpología",
    "carpometacarpiana": "carpometacarpiano",
    "carpometacarpianas": "carpometacarpiano",
    "carpometacarpianos": "carpometacarpiano",
    "carpos": "carpo",
    "carpáis": "carpir",
    "carpí": "carpir",
    "carpía": "carpir",
    "carpíais": "carpir",
    "carpíamos": "carpir",
    "carpían": "carpir",
    "carpías": "carpir",
    "carpís": "carpir",
    "carpófaga": "carpófago",
    "carpófagas": "carpófago",
    "carpófagos": "carpófago",
    "carqueroles": "carquerol",
    "carquesas": "carquesa",
    "carquiñoles": "carquiñol",
    "carraca": "carraco",
    "carracas": "carraco",
    "carracos": "carraco",
    "carralejas": "carraleja",
    "carrales": "carral",
    "carrancas": "carranca",
    "carranzas": "carranza",
    "carraones": "carraón",
    "carras": "carra",
    "carrascales": "carrascal",
    "carrascas": "carrasca",
    "carrascos": "carrasco",
    "carraspadas": "carraspada",
    "carraspea": "carraspear",
    "carraspeaba": "carraspear",
    "carraspeabais": "carraspear",
    "carraspeaban": "carraspear",
    "carraspeabas": "carraspear",
    "carraspead": "carraspear",
    "carraspeada": "carraspear",
    "carraspeadas": "carraspear",
    "carraspeado": "carraspear",
    "carraspeados": "carraspear",
    "carraspeamos": "carraspear",
    "carraspean": "carraspear",
    "carraspeando": "carraspear",
    "carraspeara": "carraspear",
    "carraspearais": "carraspear",
    "carraspearan": "carraspear",
    "carraspearas": "carraspear",
    "carraspeare": "carraspear",
    "carraspeareis": "carraspear",
    "carraspearemos": "carraspear",
    "carraspearen": "carraspear",
    "carraspeares": "carraspear",
    "carraspearon": "carraspear",
    "carraspeará": "carraspear",
    "carraspearán": "carraspear",
    "carraspearás": "carraspear",
    "carraspearé": "carraspear",
    "carraspearéis": "carraspear",
    "carraspearía": "carraspear",
    "carraspearíais": "carraspear",
    "carraspearíamos": "carraspear",
    "carraspearían": "carraspear",
    "carraspearías": "carraspear",
    "carraspeas": "carraspear",
    "carraspease": "carraspear",
    "carraspeaseis": "carraspear",
    "carraspeasen": "carraspear",
    "carraspeases": "carraspear",
    "carraspeaste": "carraspear",
    "carraspeasteis": "carraspear",
    "carraspee": "carraspear",
    "carraspeemos": "carraspear",
    "carraspeen": "carraspear",
    "carraspees": "carraspear",
    "carraspeo": "carraspear",
    "carraspeos": "carraspeo",
    "carrasperas": "carraspera",
    "carraspeábamos": "carraspear",
    "carraspeáis": "carraspear",
    "carraspeáramos": "carraspear",
    "carraspeáremos": "carraspear",
    "carraspeásemos": "carraspear",
    "carraspeé": "carraspear",
    "carraspeéis": "carraspear",
    "carraspeó": "carraspear",
    "carrasposa": "carrasposo",
    "carrasposas": "carrasposo",
    "carrasposos": "carrasposo",
    "carrasqueras": "carrasquera",
    "carrasqueña": "carrasqueño",
    "carrasqueñas": "carrasqueño",
    "carrasqueños": "carrasqueño",
    "carreras": "carrera",
    "carrerillas": "carrerilla",
    "carreristas": "carrerista",
    "carreros": "carrero",
    "carretadas": "carretada",
    "carretajes": "carretaje",
    "carretales": "carretal",
    "carretas": "carreta",
    "carretea": "carretear",
    "carreteaba": "carretear",
    "carreteabais": "carretear",
    "carreteaban": "carretear",
    "carreteabas": "carretear",
    "carretead": "carretear",
    "carreteada": "carretear",
    "carreteadas": "carretear",
    "carreteado": "carretear",
    "carreteados": "carretear",
    "carreteamos": "carretear",
    "carretean": "carretear",
    "carreteando": "carretear",
    "carreteara": "carretear",
    "carretearais": "carretear",
    "carretearan": "carretear",
    "carretearas": "carretear",
    "carreteare": "carretear",
    "carreteareis": "carretear",
    "carretearemos": "carretear",
    "carretearen": "carretear",
    "carreteares": "carretear",
    "carretearon": "carretear",
    "carreteará": "carretear",
    "carretearán": "carretear",
    "carretearás": "carretear",
    "carretearé": "carretear",
    "carretearéis": "carretear",
    "carretearía": "carretear",
    "carretearíais": "carretear",
    "carretearíamos": "carretear",
    "carretearían": "carretear",
    "carretearías": "carretear",
    "carreteas": "carretear",
    "carretease": "carretear",
    "carreteaseis": "carretear",
    "carreteasen": "carretear",
    "carreteases": "carretear",
    "carreteaste": "carretear",
    "carreteasteis": "carretear",
    "carretee": "carretear",
    "carreteemos": "carretear",
    "carreteen": "carretear",
    "carretees": "carretear",
    "carretelas": "carretela",
    "carreteles": "carretel",
    "carreteo": "carretear",
    "carreteos": "carreteo",
    "carretera": "carretero",
    "carreteras": "carretero",
    "carreteriles": "carreteril",
    "carreteros": "carretero",
    "carreterías": "carretería",
    "carretes": "carrete",
    "carreteábamos": "carretear",
    "carreteáis": "carretear",
    "carreteáramos": "carretear",
    "carreteáremos": "carretear",
    "carreteásemos": "carretear",
    "carreteé": "carretear",
    "carreteéis": "carretear",
    "carreteó": "carretear",
    "carretilladas": "carretillada",
    "carretillas": "carretilla",
    "carretonadas": "carretonada",
    "carretonera": "carretonero",
    "carretoneras": "carretonero",
    "carretoneros": "carretonero",
    "carretones": "carretón",
    "carriceras": "carricera",
    "carriceros": "carricero",
    "carricoches": "carricoche",
    "carricubas": "carricuba",
    "carriegos": "carriego",
    "carrilera": "carrilero",
    "carrileras": "carrilero",
    "carrileros": "carrilero",
    "carriles": "carril",
    "carrilladas": "carrillada",
    "carrilleras": "carrillera",
    "carrillones": "carrillón",
    "carrillos": "carrillo",
    "carrilluda": "carrilludo",
    "carrilludas": "carrilludo",
    "carrilludos": "carrilludo",
    "carriquíes": "carriquí",
    "carriquís": "carriquí",
    "carritos": "carrito",
    "carrizadas": "carrizada",
    "carrizales": "carrizal",
    "carrizos": "carrizo",
    "carrocera": "carrocero",
    "carroceras": "carrocero",
    "carroceros": "carrocero",
    "carrocerías": "carrocería",
    "carrocha": "carrochar",
    "carrochaba": "carrochar",
    "carrochabais": "carrochar",
    "carrochaban": "carrochar",
    "carrochabas": "carrochar",
    "carrochad": "carrochar",
    "carrochada": "carrochar",
    "carrochadas": "carrochar",
    "carrochado": "carrochar",
    "carrochados": "carrochar",
    "carrochamos": "carrochar",
    "carrochan": "carrochar",
    "carrochando": "carrochar",
    "carrochara": "carrochar",
    "carrocharais": "carrochar",
    "carrocharan": "carrochar",
    "carrocharas": "carrochar",
    "carrochare": "carrochar",
    "carrochareis": "carrochar",
    "carrocharemos": "carrochar",
    "carrocharen": "carrochar",
    "carrochares": "carrochar",
    "carrocharon": "carrochar",
    "carrochará": "carrochar",
    "carrocharán": "carrochar",
    "carrocharás": "carrochar",
    "carrocharé": "carrochar",
    "carrocharéis": "carrochar",
    "carrocharía": "carrochar",
    "carrocharíais": "carrochar",
    "carrocharíamos": "carrochar",
    "carrocharían": "carrochar",
    "carrocharías": "carrochar",
    "carrochas": "carrochar",
    "carrochase": "carrochar",
    "carrochaseis": "carrochar",
    "carrochasen": "carrochar",
    "carrochases": "carrochar",
    "carrochaste": "carrochar",
    "carrochasteis": "carrochar",
    "carroche": "carrochar",
    "carrochemos": "carrochar",
    "carrochen": "carrochar",
    "carroches": "carrochar",
    "carrocho": "carrochar",
    "carrochábamos": "carrochar",
    "carrocháis": "carrochar",
    "carrocháramos": "carrochar",
    "carrocháremos": "carrochar",
    "carrochásemos": "carrochar",
    "carroché": "carrochar",
    "carrochéis": "carrochar",
    "carrochó": "carrochar",
    "carromatos": "carromato",
    "carrones": "carrón",
    "carros": "carro",
    "carrozas": "carroza",
    "carroñas": "carroña",
    "carroñera": "carroñero",
    "carroñeras": "carroñero",
    "carroñeros": "carroñero",
    "carruajera": "carruajero",
    "carruajeras": "carruajero",
    "carruajeros": "carruajero",
    "carruajes": "carruaje",
    "carrucas": "carruca",
    "carruchas": "carrucha",
    "carrucos": "carruco",
    "carrujos": "carrujo",
    "carruseles": "carrusel",
    "carstificaciones": "carstificación",
    "carta-órdenes": "carta-orden",
    "cartabones": "cartabón",
    "cartagenera": "cartagenero",
    "cartageneras": "cartagenero",
    "cartageneros": "cartagenero",
    "cartaginenses": "cartaginense",
    "cartaginesa": "cartaginés",
    "cartaginesas": "cartaginés",
    "cartagineses": "cartaginés",
    "cartapacios": "cartapacio",
    "cartapeles": "cartapel",
    "cartas": "carta",
    "cartea": "cartear",
    "carteaba": "cartear",
    "carteabais": "cartear",
    "carteaban": "cartear",
    "carteabas": "cartear",
    "cartead": "cartear",
    "carteada": "cartear",
    "carteadas": "cartear",
    "carteado": "cartear",
    "carteados": "cartear",
    "carteamos": "cartear",
    "cartean": "cartear",
    "carteando": "cartear",
    "carteara": "cartear",
    "cartearais": "cartear",
    "cartearan": "cartear",
    "cartearas": "cartear",
    "carteare": "cartear",
    "carteareis": "cartear",
    "cartearemos": "cartear",
    "cartearen": "cartear",
    "carteares": "cartear",
    "cartearon": "cartear",
    "carteará": "cartear",
    "cartearán": "cartear",
    "cartearás": "cartear",
    "cartearé": "cartear",
    "cartearéis": "cartear",
    "cartearía": "cartear",
    "cartearíais": "cartear",
    "cartearíamos": "cartear",
    "cartearían": "cartear",
    "cartearías": "cartear",
    "carteas": "cartear",
    "cartease": "cartear",
    "carteaseis": "cartear",
    "carteasen": "cartear",
    "carteases": "cartear",
    "carteaste": "cartear",
    "carteasteis": "cartear",
    "cartee": "cartear",
    "carteemos": "cartear",
    "carteen": "cartear",
    "cartees": "cartear",
    "cartelas": "cartela",
    "carteleras": "cartelera",
    "carteleros": "cartelero",
    "carteles": "cartel",
    "cartelitos": "cartelito",
    "cartelizaciones": "cartelización",
    "cartelones": "cartelón",
    "carteo": "cartear",
    "carteos": "carteo",
    "cartera": "cartero",
    "carteras": "cartero",
    "carteristas": "carterista",
    "carteros": "cartero",
    "carterías": "cartería",
    "cartesiana": "cartesiano",
    "cartesianas": "cartesiano",
    "cartesianismos": "cartesianismo",
    "cartesianos": "cartesiano",
    "carteábamos": "cartear",
    "carteáis": "cartear",
    "carteáramos": "cartear",
    "carteáremos": "cartear",
    "carteásemos": "cartear",
    "carteé": "cartear",
    "carteéis": "cartear",
    "carteó": "cartear",
    "cartilaginosa": "cartilaginoso",
    "cartilaginosas": "cartilaginoso",
    "cartilaginosos": "cartilaginoso",
    "cartillas": "cartilla",
    "cartillera": "cartillero",
    "cartilleras": "cartillero",
    "cartilleros": "cartillero",
    "cartismos": "cartismo",
    "cartistas": "cartista",
    "cartivanas": "cartivana",
    "cartodiagramas": "cartodiagrama",
    "cartografiaba": "cartografiar",
    "cartografiabais": "cartografiar",
    "cartografiaban": "cartografiar",
    "cartografiabas": "cartografiar",
    "cartografiad": "cartografiar",
    "cartografiada": "cartografiar",
    "cartografiadas": "cartografiar",
    "cartografiado": "cartografiar",
    "cartografiados": "cartografiar",
    "cartografiamos": "cartografiar",
    "cartografiando": "cartografiar",
    "cartografiara": "cartografiar",
    "cartografiarais": "cartografiar",
    "cartografiaran": "cartografiar",
    "cartografiaras": "cartografiar",
    "cartografiare": "cartografiar",
    "cartografiareis": "cartografiar",
    "cartografiaremos": "cartografiar",
    "cartografiaren": "cartografiar",
    "cartografiares": "cartografiar",
    "cartografiaron": "cartografiar",
    "cartografiará": "cartografiar",
    "cartografiarán": "cartografiar",
    "cartografiarás": "cartografiar",
    "cartografiaré": "cartografiar",
    "cartografiaréis": "cartografiar",
    "cartografiaría": "cartografiar",
    "cartografiaríais": "cartografiar",
    "cartografiaríamos": "cartografiar",
    "cartografiarían": "cartografiar",
    "cartografiarías": "cartografiar",
    "cartografiase": "cartografiar",
    "cartografiaseis": "cartografiar",
    "cartografiasen": "cartografiar",
    "cartografiases": "cartografiar",
    "cartografiaste": "cartografiar",
    "cartografiasteis": "cartografiar",
    "cartografiemos": "cartografiar",
    "cartografiábamos": "cartografiar",
    "cartografiáis": "cartografiar",
    "cartografiáramos": "cartografiar",
    "cartografiáremos": "cartografiar",
    "cartografiásemos": "cartografiar",
    "cartografié": "cartografiar",
    "cartografiéis": "cartografiar",
    "cartografió": "cartografiar",
    "cartografía": "cartografiar",
    "cartografían": "cartografiar",
    "cartografías": "cartografiar",
    "cartografíe": "cartografiar",
    "cartografíen": "cartografiar",
    "cartografíes": "cartografiar",
    "cartografío": "cartografiar",
    "cartogramas": "cartograma",
    "cartográfica": "cartográfico",
    "cartográficas": "cartográfico",
    "cartográficos": "cartográfico",
    "cartologías": "cartología",
    "cartomancias": "cartomancia",
    "cartomancías": "cartomancía",
    "cartometrías": "cartometría",
    "cartomántica": "cartomántico",
    "cartománticas": "cartomántico",
    "cartománticos": "cartomántico",
    "cartométrica": "cartométrico",
    "cartométricas": "cartométrico",
    "cartométricos": "cartométrico",
    "cartonajes": "cartonaje",
    "cartonera": "cartonero",
    "cartoneras": "cartonero",
    "cartoneros": "cartonero",
    "cartonerías": "cartonería",
    "cartones": "cartón",
    "cartonés": "cartoné",
    "cartotecas": "cartoteca",
    "cartucheras": "cartuchera",
    "cartuchos": "cartucho",
    "cartuja": "cartujo",
    "cartujana": "cartujano",
    "cartujanas": "cartujano",
    "cartujanos": "cartujano",
    "cartujas": "cartujo",
    "cartujos": "cartujo",
    "cartularia": "cartulario",
    "cartularias": "cartulario",
    "cartularios": "cartulario",
    "cartulinas": "cartulina",
    "cartusanas": "cartusana",
    "cartílagos": "cartílago",
    "cartógrafa": "cartógrafo",
    "cartógrafas": "cartógrafo",
    "cartógrafos": "cartógrafo",
    "cartómetros": "cartómetro",
    "carvacroles": "carvacrol",
    "carvakas": "carvaka",
    "carvallares": "carvallar",
    "carvalledos": "carvalledo",
    "carvallos": "carvallo",
    "carvediloles": "carvedilol",
    "carvonas": "carvona",
    "carácteres": "carácter",
    "carámbanos": "carámbano",
    "caránganos": "carángano",
    "carátulas": "carátula",
    "carótida": "carótido",
    "carótidas": "carótido",
    "carótidos": "carótido",
    "carúnculas": "carúncula",
    "cas": "ca",
    "casa": "casar",
    "casaba": "casar",
    "casabais": "casar",
    "casaban": "casar",
    "casabas": "casar",
    "casacas": "casaca",
    "casaciones": "casación",
    "casad": "casar",
    "casada": "casar",
    "casadas": "casar",
    "casadera": "casadero",
    "casaderas": "casadero",
    "casaderos": "casadero",
    "casado": "casar",
    "casados": "casar",
    "casales": "casal",
    "casalicios": "casalicio",
    "casamatas": "casamata",
    "casamentera": "casamentero",
    "casamenteras": "casamentero",
    "casamenteros": "casamentero",
    "casamientos": "casamiento",
    "casamos": "casar",
    "casamuros": "casamuro",
    "casan": "casar",
    "casando": "casar",
    "casapuertas": "casapuerta",
    "casaquines": "casaquín",
    "casara": "casar",
    "casarais": "casar",
    "casaran": "casar",
    "casaras": "casar",
    "casare": "casar",
    "casareis": "casar",
    "casaremos": "casar",
    "casaren": "casar",
    "casares": "casar",
    "casaron": "casar",
    "casará": "casar",
    "casarán": "casar",
    "casarás": "casar",
    "casaré": "casar",
    "casaréis": "casar",
    "casaría": "casar",
    "casaríais": "casar",
    "casaríamos": "casar",
    "casarían": "casar",
    "casarías": "casar",
    "casas": "casar",
    "casas-museo": "casa-museo",
    "casase": "casar",
    "casaseis": "casar",
    "casasen": "casar",
    "casases": "casar",
    "casaste": "casar",
    "casasteis": "casar",
    "casca": "cascar",
    "cascaba": "cascar",
    "cascabais": "cascar",
    "cascaban": "cascar",
    "cascabas": "cascar",
    "cascabeladas": "cascabelada",
    "cascabelea": "cascabelear",
    "cascabeleaba": "cascabelear",
    "cascabeleabais": "cascabelear",
    "cascabeleaban": "cascabelear",
    "cascabeleabas": "cascabelear",
    "cascabelead": "cascabelear",
    "cascabeleada": "cascabelear",
    "cascabeleadas": "cascabelear",
    "cascabeleado": "cascabelear",
    "cascabeleados": "cascabelear",
    "cascabeleamos": "cascabelear",
    "cascabelean": "cascabelear",
    "cascabeleando": "cascabelear",
    "cascabeleara": "cascabelear",
    "cascabelearais": "cascabelear",
    "cascabelearan": "cascabelear",
    "cascabelearas": "cascabelear",
    "cascabeleare": "cascabelear",
    "cascabeleareis": "cascabelear",
    "cascabelearemos": "cascabelear",
    "cascabelearen": "cascabelear",
    "cascabeleares": "cascabelear",
    "cascabelearon": "cascabelear",
    "cascabeleará": "cascabelear",
    "cascabelearán": "cascabelear",
    "cascabelearás": "cascabelear",
    "cascabelearé": "cascabelear",
    "cascabelearéis": "cascabelear",
    "cascabelearía": "cascabelear",
    "cascabelearíais": "cascabelear",
    "cascabelearíamos": "cascabelear",
    "cascabelearían": "cascabelear",
    "cascabelearías": "cascabelear",
    "cascabeleas": "cascabelear",
    "cascabelease": "cascabelear",
    "cascabeleaseis": "cascabelear",
    "cascabeleasen": "cascabelear",
    "cascabeleases": "cascabelear",
    "cascabeleaste": "cascabelear",
    "cascabeleasteis": "cascabelear",
    "cascabelee": "cascabelear",
    "cascabeleemos": "cascabelear",
    "cascabeleen": "cascabelear",
    "cascabelees": "cascabelear",
    "cascabeleo": "cascabelear",
    "cascabeleos": "cascabeleo",
    "cascabelera": "cascabelero",
    "cascabeleras": "cascabelero",
    "cascabeleros": "cascabelero",
    "cascabeles": "cascabel",
    "cascabeleábamos": "cascabelear",
    "cascabeleáis": "cascabelear",
    "cascabeleáramos": "cascabelear",
    "cascabeleáremos": "cascabelear",
    "cascabeleásemos": "cascabelear",
    "cascabeleé": "cascabelear",
    "cascabeleéis": "cascabelear",
    "cascabeleó": "cascabelear",
    "cascabelillos": "cascabelillo",
    "cascabillos": "cascabillo",
    "cascad": "cascar",
    "cascada": "cascar",
    "cascadas": "cascar",
    "cascado": "cascar",
    "cascados": "cascar",
    "cascaduras": "cascadura",
    "cascajales": "cascajal",
    "cascajares": "cascajar",
    "cascajos": "cascajo",
    "cascajosa": "cascajoso",
    "cascajosas": "cascajoso",
    "cascajosos": "cascajoso",
    "cascamaja": "cascamajar",
    "cascamajaba": "cascamajar",
    "cascamajabais": "cascamajar",
    "cascamajaban": "cascamajar",
    "cascamajabas": "cascamajar",
    "cascamajad": "cascamajar",
    "cascamajada": "cascamajar",
    "cascamajadas": "cascamajar",
    "cascamajado": "cascamajar",
    "cascamajados": "cascamajar",
    "cascamajamos": "cascamajar",
    "cascamajan": "cascamajar",
    "cascamajando": "cascamajar",
    "cascamajara": "cascamajar",
    "cascamajarais": "cascamajar",
    "cascamajaran": "cascamajar",
    "cascamajaras": "cascamajar",
    "cascamajare": "cascamajar",
    "cascamajareis": "cascamajar",
    "cascamajaremos": "cascamajar",
    "cascamajaren": "cascamajar",
    "cascamajares": "cascamajar",
    "cascamajaron": "cascamajar",
    "cascamajará": "cascamajar",
    "cascamajarán": "cascamajar",
    "cascamajarás": "cascamajar",
    "cascamajaré": "cascamajar",
    "cascamajaréis": "cascamajar",
    "cascamajaría": "cascamajar",
    "cascamajaríais": "cascamajar",
    "cascamajaríamos": "cascamajar",
    "cascamajarían": "cascamajar",
    "cascamajarías": "cascamajar",
    "cascamajas": "cascamajar",
    "cascamajase": "cascamajar",
    "cascamajaseis": "cascamajar",
    "cascamajasen": "cascamajar",
    "cascamajases": "cascamajar",
    "cascamajaste": "cascamajar",
    "cascamajasteis": "cascamajar",
    "cascamaje": "cascamajar",
    "cascamajemos": "cascamajar",
    "cascamajen": "cascamajar",
    "cascamajes": "cascamajar",
    "cascamajo": "cascamajar",
    "cascamajábamos": "cascamajar",
    "cascamajáis": "cascamajar",
    "cascamajáramos": "cascamajar",
    "cascamajáremos": "cascamajar",
    "cascamajásemos": "cascamajar",
    "cascamajé": "cascamajar",
    "cascamajéis": "cascamajar",
    "cascamajó": "cascamajar",
    "cascamientos": "cascamiento",
    "cascamos": "cascar",
    "cascan": "cascar",
    "cascando": "cascar",
    "cascara": "cascar",
    "cascarais": "cascar",
    "cascaran": "cascar",
    "cascaras": "cascar",
    "cascare": "cascar",
    "cascareis": "cascar",
    "cascaremos": "cascar",
    "cascaren": "cascar",
    "cascares": "cascar",
    "cascarillas": "cascarilla",
    "cascaron": "cascar",
    "cascarones": "cascarón",
    "cascarrias": "cascarria",
    "cascarrona": "cascarrón",
    "cascarronas": "cascarrón",
    "cascarrones": "cascarrón",
    "cascarroyas": "cascarroya",
    "cascaruletas": "cascaruleta",
    "cascará": "cascar",
    "cascarán": "cascar",
    "cascarás": "cascar",
    "cascaré": "cascar",
    "cascaréis": "cascar",
    "cascaría": "cascar",
    "cascaríais": "cascar",
    "cascaríamos": "cascar",
    "cascarían": "cascar",
    "cascarías": "cascar",
    "cascas": "cascar",
    "cascase": "cascar",
    "cascaseis": "cascar",
    "cascasen": "cascar",
    "cascases": "cascar",
    "cascaste": "cascar",
    "cascasteis": "cascar",
    "casco": "cascar",
    "cascos": "casco",
    "cascotes": "cascote",
    "cascuda": "cascudo",
    "cascudas": "cascudo",
    "cascudos": "cascudo",
    "cascábamos": "cascar",
    "cascáis": "cascar",
    "cascáramos": "cascar",
    "cascáremos": "cascar",
    "cascásemos": "cascar",
    "cascó": "cascar",
    "case": "casar",
    "caseaciones": "caseación",
    "caseifica": "caseificar",
    "caseificaba": "caseificar",
    "caseificabais": "caseificar",
    "caseificaban": "caseificar",
    "caseificabas": "caseificar",
    "caseificaciones": "caseificación",
    "caseificad": "caseificar",
    "caseificada": "caseificar",
    "caseificadas": "caseificar",
    "caseificado": "caseificar",
    "caseificados": "caseificar",
    "caseificamos": "caseificar",
    "caseifican": "caseificar",
    "caseificando": "caseificar",
    "caseificara": "caseificar",
    "caseificarais": "caseificar",
    "caseificaran": "caseificar",
    "caseificaras": "caseificar",
    "caseificare": "caseificar",
    "caseificareis": "caseificar",
    "caseificaremos": "caseificar",
    "caseificaren": "caseificar",
    "caseificares": "caseificar",
    "caseificaron": "caseificar",
    "caseificará": "caseificar",
    "caseificarán": "caseificar",
    "caseificarás": "caseificar",
    "caseificaré": "caseificar",
    "caseificaréis": "caseificar",
    "caseificaría": "caseificar",
    "caseificaríais": "caseificar",
    "caseificaríamos": "caseificar",
    "caseificarían": "caseificar",
    "caseificarías": "caseificar",
    "caseificas": "caseificar",
    "caseificase": "caseificar",
    "caseificaseis": "caseificar",
    "caseificasen": "caseificar",
    "caseificases": "caseificar",
    "caseificaste": "caseificar",
    "caseificasteis": "caseificar",
    "caseifico": "caseificar",
    "caseificábamos": "caseificar",
    "caseificáis": "caseificar",
    "caseificáramos": "caseificar",
    "caseificáremos": "caseificar",
    "caseificásemos": "caseificar",
    "caseificó": "caseificar",
    "caseifique": "caseificar",
    "caseifiquemos": "caseificar",
    "caseifiquen": "caseificar",
    "caseifiques": "caseificar",
    "caseifiqué": "caseificar",
    "caseifiquéis": "caseificar",
    "caseinógenos": "caseinógeno",
    "casemos": "casar",
    "casen": "casar",
    "caseosa": "caseoso",
    "caseosas": "caseoso",
    "caseosos": "caseoso",
    "casera": "casero",
    "caseras": "casero",
    "casernas": "caserna",
    "caserones": "caserón",
    "caseros": "casero",
    "caserías": "casería",
    "caseríos": "caserío",
    "cases": "casar",
    "casetas": "caseta",
    "casetes": "casete",
    "casetones": "casetón",
    "caseínas": "caseína",
    "casidas": "casida",
    "casidomicilios": "casidomicilio",
    "casillas": "casilla",
    "casilleros": "casillero",
    "casimiras": "casimira",
    "casimires": "casimir",
    "casinas": "casina",
    "casinos": "casino",
    "casitas": "casita",
    "casiteritas": "casiterita",
    "casitéridos": "casitérido",
    "caso": "casar",
    "casonas": "casona",
    "casorios": "casorio",
    "casos": "caso",
    "caspas": "caspa",
    "casperas": "caspera",
    "casposa": "casposo",
    "casposas": "casposo",
    "casposos": "casposo",
    "casque": "cascar",
    "casquemos": "cascar",
    "casquen": "cascar",
    "casquera": "casquero",
    "casqueras": "casquero",
    "casqueros": "casquero",
    "casquerías": "casquería",
    "casques": "cascar",
    "casquetes": "casquete",
    "casquijos": "casquijo",
    "casquillas": "casquilla",
    "casquillos": "casquillo",
    "casquivana": "casquivano",
    "casquivanas": "casquivano",
    "casquivanos": "casquivano",
    "casqué": "cascar",
    "casquéis": "cascar",
    "cassettes": "cassette",
    "casta": "casto",
    "castas": "casto",
    "castaña": "castaño",
    "castañadas": "castañada",
    "castañares": "castañar",
    "castañas": "castaño",
    "castañeras": "castañera",
    "castañetadas": "castañetada",
    "castañetas": "castañeta",
    "castañetazos": "castañetazo",
    "castañetea": "castañetear",
    "castañeteaba": "castañetear",
    "castañeteabais": "castañetear",
    "castañeteaban": "castañetear",
    "castañeteabas": "castañetear",
    "castañetead": "castañetear",
    "castañeteada": "castañetear",
    "castañeteadas": "castañetear",
    "castañeteado": "castañetear",
    "castañeteados": "castañetear",
    "castañeteamos": "castañetear",
    "castañetean": "castañetear",
    "castañeteando": "castañetear",
    "castañeteara": "castañetear",
    "castañetearais": "castañetear",
    "castañetearan": "castañetear",
    "castañetearas": "castañetear",
    "castañeteare": "castañetear",
    "castañeteareis": "castañetear",
    "castañetearemos": "castañetear",
    "castañetearen": "castañetear",
    "castañeteares": "castañetear",
    "castañetearon": "castañetear",
    "castañeteará": "castañetear",
    "castañetearán": "castañetear",
    "castañetearás": "castañetear",
    "castañetearé": "castañetear",
    "castañetearéis": "castañetear",
    "castañetearía": "castañetear",
    "castañetearíais": "castañetear",
    "castañetearíamos": "castañetear",
    "castañetearían": "castañetear",
    "castañetearías": "castañetear",
    "castañeteas": "castañetear",
    "castañetease": "castañetear",
    "castañeteaseis": "castañetear",
    "castañeteasen": "castañetear",
    "castañeteases": "castañetear",
    "castañeteaste": "castañetear",
    "castañeteasteis": "castañetear",
    "castañetee": "castañetear",
    "castañeteemos": "castañetear",
    "castañeteen": "castañetear",
    "castañetees": "castañetear",
    "castañeteo": "castañetear",
    "castañeteos": "castañeteo",
    "castañeteábamos": "castañetear",
    "castañeteáis": "castañetear",
    "castañeteáramos": "castañetear",
    "castañeteáremos": "castañetear",
    "castañeteásemos": "castañetear",
    "castañeteé": "castañetear",
    "castañeteéis": "castañetear",
    "castañeteó": "castañetear",
    "castañolas": "castañola",
    "castaños": "castaño",
    "castañuela": "castañuelo",
    "castañuelas": "castañuelo",
    "castañuelos": "castañuelo",
    "castellana": "castellano",
    "castellanas": "castellano",
    "castellanice": "castellanizar",
    "castellanicemos": "castellanizar",
    "castellanicen": "castellanizar",
    "castellanices": "castellanizar",
    "castellanicé": "castellanizar",
    "castellanicéis": "castellanizar",
    "castellanismos": "castellanismo",
    "castellaniza": "castellanizar",
    "castellanizaba": "castellanizar",
    "castellanizabais": "castellanizar",
    "castellanizaban": "castellanizar",
    "castellanizabas": "castellanizar",
    "castellanizaciones": "castellanización",
    "castellanizad": "castellanizar",
    "castellanizada": "castellanizar",
    "castellanizadas": "castellanizar",
    "castellanizado": "castellanizar",
    "castellanizados": "castellanizar",
    "castellanizamos": "castellanizar",
    "castellanizan": "castellanizar",
    "castellanizando": "castellanizar",
    "castellanizara": "castellanizar",
    "castellanizarais": "castellanizar",
    "castellanizaran": "castellanizar",
    "castellanizaras": "castellanizar",
    "castellanizare": "castellanizar",
    "castellanizareis": "castellanizar",
    "castellanizaremos": "castellanizar",
    "castellanizaren": "castellanizar",
    "castellanizares": "castellanizar",
    "castellanizaron": "castellanizar",
    "castellanizará": "castellanizar",
    "castellanizarán": "castellanizar",
    "castellanizarás": "castellanizar",
    "castellanizaré": "castellanizar",
    "castellanizaréis": "castellanizar",
    "castellanizaría": "castellanizar",
    "castellanizaríais": "castellanizar",
    "castellanizaríamos": "castellanizar",
    "castellanizarían": "castellanizar",
    "castellanizarías": "castellanizar",
    "castellanizas": "castellanizar",
    "castellanizase": "castellanizar",
    "castellanizaseis": "castellanizar",
    "castellanizasen": "castellanizar",
    "castellanizases": "castellanizar",
    "castellanizaste": "castellanizar",
    "castellanizasteis": "castellanizar",
    "castellanizo": "castellanizar",
    "castellanizábamos": "castellanizar",
    "castellanizáis": "castellanizar",
    "castellanizáramos": "castellanizar",
    "castellanizáremos": "castellanizar",
    "castellanizásemos": "castellanizar",
    "castellanizó": "castellanizar",
    "castellanohablantes": "castellanohablante",
    "castellanoparlantes": "castellanoparlante",
    "castellanos": "castellano",
    "castellanías": "castellanía",
    "castellera": "castellero",
    "castelleras": "castellero",
    "castelleros": "castellero",
    "castellonenses": "castellonense",
    "casticidades": "casticidad",
    "casticismos": "casticismo",
    "casticistas": "casticista",
    "castidades": "castidad",
    "castiga": "castigar",
    "castigaba": "castigar",
    "castigabais": "castigar",
    "castigaban": "castigar",
    "castigabas": "castigar",
    "castigad": "castigar",
    "castigada": "castigar",
    "castigadas": "castigar",
    "castigado": "castigar",
    "castigadora": "castigador",
    "castigadoras": "castigador",
    "castigadores": "castigador",
    "castigados": "castigar",
    "castigamos": "castigar",
    "castigan": "castigar",
    "castigando": "castigar",
    "castigara": "castigar",
    "castigarais": "castigar",
    "castigaran": "castigar",
    "castigaras": "castigar",
    "castigare": "castigar",
    "castigareis": "castigar",
    "castigaremos": "castigar",
    "castigaren": "castigar",
    "castigares": "castigar",
    "castigaron": "castigar",
    "castigará": "castigar",
    "castigarán": "castigar",
    "castigarás": "castigar",
    "castigaré": "castigar",
    "castigaréis": "castigar",
    "castigaría": "castigar",
    "castigaríais": "castigar",
    "castigaríamos": "castigar",
    "castigarían": "castigar",
    "castigarías": "castigar",
    "castigas": "castigar",
    "castigase": "castigar",
    "castigaseis": "castigar",
    "castigasen": "castigar",
    "castigases": "castigar",
    "castigaste": "castigar",
    "castigasteis": "castigar",
    "castigo": "castigar",
    "castigos": "castigo",
    "castigue": "castigar",
    "castiguemos": "castigar",
    "castiguen": "castigar",
    "castigues": "castigar",
    "castigué": "castigar",
    "castiguéis": "castigar",
    "castigábamos": "castigar",
    "castigáis": "castigar",
    "castigáramos": "castigar",
    "castigáremos": "castigar",
    "castigásemos": "castigar",
    "castigó": "castigar",
    "castillajes": "castillaje",
    "castillas": "castilla",
    "castillejos": "castillejo",
    "castillerías": "castillería",
    "castilletes": "castillete",
    "castillos": "castillo",
    "castings": "casting",
    "castiza": "castizo",
    "castizas": "castizo",
    "castizos": "castizo",
    "castores": "castor",
    "castoreña": "castoreño",
    "castoreñas": "castoreño",
    "castoreños": "castoreño",
    "castos": "casto",
    "castra": "castrar",
    "castraba": "castrar",
    "castrabais": "castrar",
    "castraban": "castrar",
    "castrabas": "castrar",
    "castraciones": "castración",
    "castrad": "castrar",
    "castrada": "castrar",
    "castradas": "castrar",
    "castrado": "castrar",
    "castrados": "castrar",
    "castraduras": "castradura",
    "castrales": "castral",
    "castrametaciones": "castrametación",
    "castramos": "castrar",
    "castran": "castrar",
    "castrando": "castrar",
    "castrara": "castrar",
    "castrarais": "castrar",
    "castraran": "castrar",
    "castraras": "castrar",
    "castrare": "castrar",
    "castrareis": "castrar",
    "castraremos": "castrar",
    "castraren": "castrar",
    "castrares": "castrar",
    "castraron": "castrar",
    "castrará": "castrar",
    "castrarán": "castrar",
    "castrarás": "castrar",
    "castraré": "castrar",
    "castraréis": "castrar",
    "castraría": "castrar",
    "castraríais": "castrar",
    "castraríamos": "castrar",
    "castrarían": "castrar",
    "castrarías": "castrar",
    "castras": "castrar",
    "castrase": "castrar",
    "castraseis": "castrar",
    "castrasen": "castrar",
    "castrases": "castrar",
    "castraste": "castrar",
    "castrasteis": "castrar",
    "castre": "castrar",
    "castremos": "castrar",
    "castren": "castrar",
    "castrenses": "castrense",
    "castres": "castrar",
    "castrismos": "castrismo",
    "castristas": "castrista",
    "castro": "castrar",
    "castrones": "castrón",
    "castros": "castro",
    "castrábamos": "castrar",
    "castráis": "castrar",
    "castráramos": "castrar",
    "castráremos": "castrar",
    "castrásemos": "castrar",
    "castré": "castrar",
    "castréis": "castrar",
    "castró": "castrar",
    "castóreos": "castóreo",
    "casuales": "casual",
    "casualidades": "casualidad",
    "casualismos": "casualismo",
    "casualistas": "casualista",
    "casuarináceas": "casuarinácea",
    "casuarios": "casuario",
    "casucas": "casuca",
    "casuchas": "casucha",
    "casuchos": "casucho",
    "casuismos": "casuismo",
    "casuistas": "casuista",
    "casullas": "casulla",
    "casuáridos": "casuárido",
    "casuística": "casuístico",
    "casuísticas": "casuístico",
    "casuísticos": "casuístico",
    "casábamos": "casar",
    "casáis": "casar",
    "casáramos": "casar",
    "casáremos": "casar",
    "casásemos": "casar",
    "casé": "casar",
    "caséis": "casar",
    "casó": "casar",
    "cata": "catar",
    "cataba": "catar",
    "catabais": "catar",
    "cataban": "catar",
    "catabas": "catar",
    "catabolismos": "catabolismo",
    "catabática": "catabático",
    "catabáticas": "catabático",
    "catabáticos": "catabático",
    "cataclismos": "cataclismo",
    "catacumbas": "catacumba",
    "catacáusticas": "catacáustica",
    "catad": "catar",
    "catada": "catar",
    "catadas": "catar",
    "catadióptrica": "catadióptrico",
    "catadióptricas": "catadióptrico",
    "catadióptricos": "catadióptrico",
    "catado": "catar",
    "catadora": "catador",
    "catadoras": "catador",
    "catadores": "catador",
    "catados": "catar",
    "cataduras": "catadura",
    "catafalcos": "catafalco",
    "catafórica": "catafórico",
    "catafóricas": "catafórico",
    "catafóricos": "catafórico",
    "catalana": "catalán",
    "catalanas": "catalán",
    "catalanes": "catalán",
    "catalanidades": "catalanidad",
    "catalanismos": "catalanismo",
    "catalanistas": "catalanista",
    "catalanizaciones": "catalanización",
    "catalano-hablantes": "catalano-hablante",
    "catalanohablantes": "catalanohablante",
    "catalanoparlantes": "catalanoparlante",
    "catalarces": "catalarc",
    "catalasas": "catalasa",
    "cataldos": "cataldo",
    "catalejos": "catalejo",
    "catalepsias": "catalepsia",
    "catalice": "catalizar",
    "catalicemos": "catalizar",
    "catalicen": "catalizar",
    "catalices": "catalizar",
    "catalicé": "catalizar",
    "catalicéis": "catalizar",
    "catalinas": "catalina",
    "catalinos": "catalino",
    "cataliza": "catalizar",
    "catalizaba": "catalizar",
    "catalizabais": "catalizar",
    "catalizaban": "catalizar",
    "catalizabas": "catalizar",
    "catalizad": "catalizar",
    "catalizada": "catalizar",
    "catalizadas": "catalizar",
    "catalizado": "catalizar",
    "catalizadora": "catalizador",
    "catalizadoras": "catalizador",
    "catalizadores": "catalizador",
    "catalizados": "catalizar",
    "catalizamos": "catalizar",
    "catalizan": "catalizar",
    "catalizando": "catalizar",
    "catalizara": "catalizar",
    "catalizarais": "catalizar",
    "catalizaran": "catalizar",
    "catalizaras": "catalizar",
    "catalizare": "catalizar",
    "catalizareis": "catalizar",
    "catalizaremos": "catalizar",
    "catalizaren": "catalizar",
    "catalizares": "catalizar",
    "catalizaron": "catalizar",
    "catalizará": "catalizar",
    "catalizarán": "catalizar",
    "catalizarás": "catalizar",
    "catalizaré": "catalizar",
    "catalizaréis": "catalizar",
    "catalizaría": "catalizar",
    "catalizaríais": "catalizar",
    "catalizaríamos": "catalizar",
    "catalizarían": "catalizar",
    "catalizarías": "catalizar",
    "catalizas": "catalizar",
    "catalizase": "catalizar",
    "catalizaseis": "catalizar",
    "catalizasen": "catalizar",
    "catalizases": "catalizar",
    "catalizaste": "catalizar",
    "catalizasteis": "catalizar",
    "catalizo": "catalizar",
    "catalizábamos": "catalizar",
    "catalizáis": "catalizar",
    "catalizáramos": "catalizar",
    "catalizáremos": "catalizar",
    "catalizásemos": "catalizar",
    "catalizó": "catalizar",
    "cataloga": "catalogar",
    "catalogaba": "catalogar",
    "catalogabais": "catalogar",
    "catalogaban": "catalogar",
    "catalogabas": "catalogar",
    "catalogables": "catalogable",
    "catalogaciones": "catalogación",
    "catalogad": "catalogar",
    "catalogada": "catalogar",
    "catalogadas": "catalogar",
    "catalogado": "catalogar",
    "catalogadora": "catalogador",
    "catalogadoras": "catalogador",
    "catalogadores": "catalogador",
    "catalogados": "catalogar",
    "catalogamos": "catalogar",
    "catalogan": "catalogar",
    "catalogando": "catalogar",
    "catalogara": "catalogar",
    "catalogarais": "catalogar",
    "catalogaran": "catalogar",
    "catalogaras": "catalogar",
    "catalogare": "catalogar",
    "catalogareis": "catalogar",
    "catalogaremos": "catalogar",
    "catalogaren": "catalogar",
    "catalogares": "catalogar",
    "catalogaron": "catalogar",
    "catalogará": "catalogar",
    "catalogarán": "catalogar",
    "catalogarás": "catalogar",
    "catalogaré": "catalogar",
    "catalogaréis": "catalogar",
    "catalogaría": "catalogar",
    "catalogaríais": "catalogar",
    "catalogaríamos": "catalogar",
    "catalogarían": "catalogar",
    "catalogarías": "catalogar",
    "catalogas": "catalogar",
    "catalogase": "catalogar",
    "catalogaseis": "catalogar",
    "catalogasen": "catalogar",
    "catalogases": "catalogar",
    "catalogaste": "catalogar",
    "catalogasteis": "catalogar",
    "catalogo": "catalogar",
    "catalográfica": "catalográfico",
    "catalográficas": "catalográfico",
    "catalográficos": "catalográfico",
    "catalogue": "catalogar",
    "cataloguemos": "catalogar",
    "cataloguen": "catalogar",
    "catalogues": "catalogar",
    "catalogué": "catalogar",
    "cataloguéis": "catalogar",
    "catalogábamos": "catalogar",
    "catalogáis": "catalogar",
    "catalogáramos": "catalogar",
    "catalogáremos": "catalogar",
    "catalogásemos": "catalogar",
    "catalogó": "catalogar",
    "catalufas": "catalufa",
    "cataléctica": "cataléctico",
    "catalécticas": "cataléctico",
    "catalécticos": "cataléctico",
    "cataléptica": "cataléptico",
    "catalépticas": "cataléptico",
    "catalépticos": "cataléptico",
    "catalítica": "catalítico",
    "catalíticas": "catalítico",
    "catalíticos": "catalítico",
    "catamaranes": "catamarán",
    "catamarqueña": "catamarqueño",
    "catamarqueñas": "catamarqueño",
    "catamarqueños": "catamarqueño",
    "catamaránes": "catamarán",
    "catameniales": "catamenial",
    "catamos": "catar",
    "catan": "catar",
    "catanas": "catana",
    "catando": "catar",
    "catanes": "catán",
    "cataplasmas": "cataplasma",
    "cataplexias": "cataplexia",
    "catapulta": "catapultar",
    "catapultaba": "catapultar",
    "catapultabais": "catapultar",
    "catapultaban": "catapultar",
    "catapultabas": "catapultar",
    "catapultad": "catapultar",
    "catapultada": "catapultar",
    "catapultadas": "catapultar",
    "catapultado": "catapultar",
    "catapultados": "catapultar",
    "catapultamos": "catapultar",
    "catapultan": "catapultar",
    "catapultando": "catapultar",
    "catapultara": "catapultar",
    "catapultarais": "catapultar",
    "catapultaran": "catapultar",
    "catapultaras": "catapultar",
    "catapultare": "catapultar",
    "catapultareis": "catapultar",
    "catapultaremos": "catapultar",
    "catapultaren": "catapultar",
    "catapultares": "catapultar",
    "catapultaron": "catapultar",
    "catapultará": "catapultar",
    "catapultarán": "catapultar",
    "catapultarás": "catapultar",
    "catapultaré": "catapultar",
    "catapultaréis": "catapultar",
    "catapultaría": "catapultar",
    "catapultaríais": "catapultar",
    "catapultaríamos": "catapultar",
    "catapultarían": "catapultar",
    "catapultarías": "catapultar",
    "catapultas": "catapultar",
    "catapultase": "catapultar",
    "catapultaseis": "catapultar",
    "catapultasen": "catapultar",
    "catapultases": "catapultar",
    "catapultaste": "catapultar",
    "catapultasteis": "catapultar",
    "catapulte": "catapultar",
    "catapultemos": "catapultar",
    "catapulten": "catapultar",
    "catapultes": "catapultar",
    "catapulto": "catapultar",
    "catapultábamos": "catapultar",
    "catapultáis": "catapultar",
    "catapultáramos": "catapultar",
    "catapultáremos": "catapultar",
    "catapultásemos": "catapultar",
    "catapulté": "catapultar",
    "catapultéis": "catapultar",
    "catapultó": "catapultar",
    "catara": "catar",
    "catarais": "catar",
    "cataran": "catar",
    "cataras": "catar",
    "cataratas": "catarata",
    "catare": "catar",
    "catareis": "catar",
    "cataremos": "catar",
    "cataren": "catar",
    "catares": "catar",
    "cataron": "catar",
    "catarrales": "catarral",
    "catarrinos": "catarrino",
    "catarros": "catarro",
    "catarrosa": "catarroso",
    "catarrosas": "catarroso",
    "catarrosos": "catarroso",
    "catará": "catar",
    "catarán": "catar",
    "catarás": "catar",
    "cataré": "catar",
    "cataréis": "catar",
    "cataría": "catar",
    "cataríais": "catar",
    "cataríamos": "catar",
    "catarían": "catar",
    "catarías": "catar",
    "catas": "catar",
    "catase": "catar",
    "cataseis": "catar",
    "catasen": "catar",
    "catases": "catar",
    "cataste": "catar",
    "catasteis": "catar",
    "catastra": "catastrar",
    "catastraba": "catastrar",
    "catastrabais": "catastrar",
    "catastraban": "catastrar",
    "catastrabas": "catastrar",
    "catastrad": "catastrar",
    "catastrada": "catastrar",
    "catastradas": "catastrar",
    "catastrado": "catastrar",
    "catastrados": "catastrar",
    "catastrales": "catastral",
    "catastramos": "catastrar",
    "catastran": "catastrar",
    "catastrando": "catastrar",
    "catastrara": "catastrar",
    "catastrarais": "catastrar",
    "catastraran": "catastrar",
    "catastraras": "catastrar",
    "catastrare": "catastrar",
    "catastrareis": "catastrar",
    "catastraremos": "catastrar",
    "catastraren": "catastrar",
    "catastrares": "catastrar",
    "catastraron": "catastrar",
    "catastrará": "catastrar",
    "catastrarán": "catastrar",
    "catastrarás": "catastrar",
    "catastraré": "catastrar",
    "catastraréis": "catastrar",
    "catastraría": "catastrar",
    "catastraríais": "catastrar",
    "catastraríamos": "catastrar",
    "catastrarían": "catastrar",
    "catastrarías": "catastrar",
    "catastras": "catastrar",
    "catastrase": "catastrar",
    "catastraseis": "catastrar",
    "catastrasen": "catastrar",
    "catastrases": "catastrar",
    "catastraste": "catastrar",
    "catastrasteis": "catastrar",
    "catastre": "catastrar",
    "catastremos": "catastrar",
    "catastren": "catastrar",
    "catastres": "catastrar",
    "catastro": "catastrar",
    "catastrofismos": "catastrofismo",
    "catastrofistas": "catastrofista",
    "catastros": "catastro",
    "catastrábamos": "catastrar",
    "catastráis": "catastrar",
    "catastráramos": "catastrar",
    "catastráremos": "catastrar",
    "catastrásemos": "catastrar",
    "catastré": "catastrar",
    "catastréis": "catastrar",
    "catastró": "catastrar",
    "catastrófica": "catastrófico",
    "catastróficas": "catastrófico",
    "catastróficos": "catastrófico",
    "catata": "catatar",
    "catataba": "catatar",
    "catatabais": "catatar",
    "catataban": "catatar",
    "catatabas": "catatar",
    "catatad": "catatar",
    "catatada": "catatar",
    "catatadas": "catatar",
    "catatado": "catatar",
    "catatados": "catatar",
    "catatamos": "catatar",
    "catatan": "catatar",
    "catatando": "catatar",
    "catatara": "catatar",
    "catatarais": "catatar",
    "catataran": "catatar",
    "catataras": "catatar",
    "catatare": "catatar",
    "catatareis": "catatar",
    "catataremos": "catatar",
    "catataren": "catatar",
    "catatares": "catatar",
    "catataron": "catatar",
    "catatará": "catatar",
    "catatarán": "catatar",
    "catatarás": "catatar",
    "catataré": "catatar",
    "catataréis": "catatar",
    "catataría": "catatar",
    "catataríais": "catatar",
    "catataríamos": "catatar",
    "catatarían": "catatar",
    "catatarías": "catatar",
    "catatas": "catatar",
    "catatase": "catatar",
    "catataseis": "catatar",
    "catatasen": "catatar",
    "catatases": "catatar",
    "catataste": "catatar",
    "catatasteis": "catatar",
    "catate": "catatar",
    "catatemos": "catatar",
    "cataten": "catatar",
    "catates": "catatar",
    "catatipias": "catatipia",
    "catato": "catatar",
    "catatonías": "catatonía",
    "catatábamos": "catatar",
    "catatáis": "catatar",
    "catatáramos": "catatar",
    "catatáremos": "catatar",
    "catatásemos": "catatar",
    "cataté": "catatar",
    "catatéis": "catatar",
    "catató": "catatar",
    "catatónica": "catatónico",
    "catatónicas": "catatónico",
    "catatónicos": "catatónico",
    "cataubas": "catauba",
    "catavientos": "cataviento",
    "catavinos": "catavino",
    "catazonas": "catazona",
    "catcheres": "catcher",
    "catchers": "catcher",
    "cate": "catar",
    "catea": "catear",
    "cateaba": "catear",
    "cateabais": "catear",
    "cateaban": "catear",
    "cateabas": "catear",
    "catead": "catear",
    "cateada": "catear",
    "cateadas": "catear",
    "cateado": "catear",
    "cateadora": "cateador",
    "cateadoras": "cateador",
    "cateadores": "cateador",
    "cateados": "catear",
    "cateamos": "catear",
    "catean": "catear",
    "cateando": "catear",
    "cateara": "catear",
    "catearais": "catear",
    "catearan": "catear",
    "catearas": "catear",
    "cateare": "catear",
    "cateareis": "catear",
    "catearemos": "catear",
    "catearen": "catear",
    "cateares": "catear",
    "catearon": "catear",
    "cateará": "catear",
    "catearán": "catear",
    "catearás": "catear",
    "catearé": "catear",
    "catearéis": "catear",
    "catearía": "catear",
    "catearíais": "catear",
    "catearíamos": "catear",
    "catearían": "catear",
    "catearías": "catear",
    "cateas": "catear",
    "catease": "catear",
    "cateaseis": "catear",
    "cateasen": "catear",
    "cateases": "catear",
    "cateaste": "catear",
    "cateasteis": "catear",
    "catecismos": "catecismo",
    "catecumenados": "catecumenado",
    "catecúmena": "catecúmeno",
    "catecúmenas": "catecúmeno",
    "catecúmenos": "catecúmeno",
    "catedrales": "catedral",
    "catedralicia": "catedralicio",
    "catedralicias": "catedralicio",
    "catedralicios": "catedralicio",
    "catedralidades": "catedralidad",
    "catedrática": "catedrático",
    "catedráticas": "catedrático",
    "catedráticos": "catedrático",
    "catee": "catear",
    "cateemos": "catear",
    "cateen": "catear",
    "catees": "catear",
    "categoremas": "categorema",
    "categoremática": "categoremático",
    "categoremáticas": "categoremático",
    "categoremáticos": "categoremático",
    "categoriales": "categorial",
    "categorice": "categorizar",
    "categoricemos": "categorizar",
    "categoricen": "categorizar",
    "categorices": "categorizar",
    "categoricé": "categorizar",
    "categoricéis": "categorizar",
    "categorismos": "categorismo",
    "categoriza": "categorizar",
    "categorizaba": "categorizar",
    "categorizabais": "categorizar",
    "categorizaban": "categorizar",
    "categorizabas": "categorizar",
    "categorizaciones": "categorización",
    "categorizad": "categorizar",
    "categorizada": "categorizar",
    "categorizadas": "categorizar",
    "categorizado": "categorizar",
    "categorizados": "categorizar",
    "categorizamos": "categorizar",
    "categorizan": "categorizar",
    "categorizando": "categorizar",
    "categorizara": "categorizar",
    "categorizarais": "categorizar",
    "categorizaran": "categorizar",
    "categorizaras": "categorizar",
    "categorizare": "categorizar",
    "categorizareis": "categorizar",
    "categorizaremos": "categorizar",
    "categorizaren": "categorizar",
    "categorizares": "categorizar",
    "categorizaron": "categorizar",
    "categorizará": "categorizar",
    "categorizarán": "categorizar",
    "categorizarás": "categorizar",
    "categorizaré": "categorizar",
    "categorizaréis": "categorizar",
    "categorizaría": "categorizar",
    "categorizaríais": "categorizar",
    "categorizaríamos": "categorizar",
    "categorizarían": "categorizar",
    "categorizarías": "categorizar",
    "categorizas": "categorizar",
    "categorizase": "categorizar",
    "categorizaseis": "categorizar",
    "categorizasen": "categorizar",
    "categorizases": "categorizar",
    "categorizaste": "categorizar",
    "categorizasteis": "categorizar",
    "categorizo": "categorizar",
    "categorizábamos": "categorizar",
    "categorizáis": "categorizar",
    "categorizáramos": "categorizar",
    "categorizáremos": "categorizar",
    "categorizásemos": "categorizar",
    "categorizó": "categorizar",
    "categorías": "categoría",
    "categórica": "categórico",
    "categóricas": "categórico",
    "categóricos": "categórico",
    "catelas": "catela",
    "catemos": "catar",
    "caten": "catar",
    "catenaciones": "catenación",
    "catenanos": "catenano",
    "catenarias": "catenaria",
    "catenas": "catena",
    "catenulares": "catenular",
    "cateo": "catear",
    "cateos": "cateo",
    "catepsinas": "catepsina",
    "catequice": "catequizar",
    "catequicemos": "catequizar",
    "catequicen": "catequizar",
    "catequices": "catequizar",
    "catequicé": "catequizar",
    "catequicéis": "catequizar",
    "catequismos": "catequismo",
    "catequistas": "catequista",
    "catequiza": "catequizar",
    "catequizaba": "catequizar",
    "catequizabais": "catequizar",
    "catequizaban": "catequizar",
    "catequizabas": "catequizar",
    "catequizaciones": "catequización",
    "catequizad": "catequizar",
    "catequizada": "catequizar",
    "catequizadas": "catequizar",
    "catequizado": "catequizar",
    "catequizadora": "catequizador",
    "catequizadoras": "catequizador",
    "catequizadores": "catequizador",
    "catequizados": "catequizar",
    "catequizamos": "catequizar",
    "catequizan": "catequizar",
    "catequizando": "catequizar",
    "catequizara": "catequizar",
    "catequizarais": "catequizar",
    "catequizaran": "catequizar",
    "catequizaras": "catequizar",
    "catequizare": "catequizar",
    "catequizareis": "catequizar",
    "catequizaremos": "catequizar",
    "catequizaren": "catequizar",
    "catequizares": "catequizar",
    "catequizaron": "catequizar",
    "catequizará": "catequizar",
    "catequizarán": "catequizar",
    "catequizarás": "catequizar",
    "catequizaré": "catequizar",
    "catequizaréis": "catequizar",
    "catequizaría": "catequizar",
    "catequizaríais": "catequizar",
    "catequizaríamos": "catequizar",
    "catequizarían": "catequizar",
    "catequizarías": "catequizar",
    "catequizas": "catequizar",
    "catequizase": "catequizar",
    "catequizaseis": "catequizar",
    "catequizasen": "catequizar",
    "catequizases": "catequizar",
    "catequizaste": "catequizar",
    "catequizasteis": "catequizar",
    "catequizo": "catequizar",
    "catequizábamos": "catequizar",
    "catequizáis": "catequizar",
    "catequizáramos": "catequizar",
    "catequizáremos": "catequizar",
    "catequizásemos": "catequizar",
    "catequizó": "catequizar",
    "catequéticas": "catequética",
    "catequística": "catequístico",
    "catequísticas": "catequístico",
    "catequísticos": "catequístico",
    "caterings": "catering",
    "catervarios": "catervario",
    "catervas": "caterva",
    "cates": "cate",
    "cateta": "cateto",
    "catetadas": "catetada",
    "catetas": "cateto",
    "cateterismos": "cateterismo",
    "catetos": "cateto",
    "catetómetros": "catetómetro",
    "cateyas": "cateya",
    "cateábamos": "catear",
    "cateáis": "catear",
    "cateáramos": "catear",
    "cateáremos": "catear",
    "cateásemos": "catear",
    "cateé": "catear",
    "cateéis": "catear",
    "cateó": "catear",
    "catguts": "catgut",
    "catilinarias": "catilinaria",
    "catingas": "catinga",
    "cationes": "catión",
    "catira": "catiro",
    "catiras": "catiro",
    "catires": "catire",
    "catiros": "catiro",
    "catitea": "catitear",
    "catiteaba": "catitear",
    "catiteabais": "catitear",
    "catiteaban": "catitear",
    "catiteabas": "catitear",
    "catitead": "catitear",
    "catiteada": "catitear",
    "catiteadas": "catitear",
    "catiteado": "catitear",
    "catiteados": "catitear",
    "catiteamos": "catitear",
    "catitean": "catitear",
    "catiteando": "catitear",
    "catiteara": "catitear",
    "catitearais": "catitear",
    "catitearan": "catitear",
    "catitearas": "catitear",
    "catiteare": "catitear",
    "catiteareis": "catitear",
    "catitearemos": "catitear",
    "catitearen": "catitear",
    "catiteares": "catitear",
    "catitearon": "catitear",
    "catiteará": "catitear",
    "catitearán": "catitear",
    "catitearás": "catitear",
    "catitearé": "catitear",
    "catitearéis": "catitear",
    "catitearía": "catitear",
    "catitearíais": "catitear",
    "catitearíamos": "catitear",
    "catitearían": "catitear",
    "catitearías": "catitear",
    "catiteas": "catitear",
    "catitease": "catitear",
    "catiteaseis": "catitear",
    "catiteasen": "catitear",
    "catiteases": "catitear",
    "catiteaste": "catitear",
    "catiteasteis": "catitear",
    "catitee": "catitear",
    "catiteemos": "catitear",
    "catiteen": "catitear",
    "catitees": "catitear",
    "catiteo": "catitear",
    "catites": "catite",
    "catiteábamos": "catitear",
    "catiteáis": "catitear",
    "catiteáramos": "catitear",
    "catiteáremos": "catitear",
    "catiteásemos": "catitear",
    "catiteé": "catitear",
    "catiteéis": "catitear",
    "catiteó": "catitear",
    "catiónica": "catiónico",
    "catiónicas": "catiónico",
    "catiónicos": "catiónico",
    "catleyas": "catleya",
    "cato": "catar",
    "catodoluminiscencias": "catodoluminiscencia",
    "catolicidades": "catolicidad",
    "catolicismos": "catolicismo",
    "catolitos": "catolito",
    "catones": "catón",
    "catoniana": "catoniano",
    "catonianas": "catoniano",
    "catonianos": "catoniano",
    "catonice": "catonizar",
    "catonicemos": "catonizar",
    "catonicen": "catonizar",
    "catonices": "catonizar",
    "catonicé": "catonizar",
    "catonicéis": "catonizar",
    "catonismos": "catonismo",
    "catoniza": "catonizar",
    "catonizaba": "catonizar",
    "catonizabais": "catonizar",
    "catonizaban": "catonizar",
    "catonizabas": "catonizar",
    "catonizad": "catonizar",
    "catonizada": "catonizar",
    "catonizadas": "catonizar",
    "catonizado": "catonizar",
    "catonizados": "catonizar",
    "catonizamos": "catonizar",
    "catonizan": "catonizar",
    "catonizando": "catonizar",
    "catonizara": "catonizar",
    "catonizarais": "catonizar",
    "catonizaran": "catonizar",
    "catonizaras": "catonizar",
    "catonizare": "catonizar",
    "catonizareis": "catonizar",
    "catonizaremos": "catonizar",
    "catonizaren": "catonizar",
    "catonizares": "catonizar",
    "catonizaron": "catonizar",
    "catonizará": "catonizar",
    "catonizarán": "catonizar",
    "catonizarás": "catonizar",
    "catonizaré": "catonizar",
    "catonizaréis": "catonizar",
    "catonizaría": "catonizar",
    "catonizaríais": "catonizar",
    "catonizaríamos": "catonizar",
    "catonizarían": "catonizar",
    "catonizarías": "catonizar",
    "catonizas": "catonizar",
    "catonizase": "catonizar",
    "catonizaseis": "catonizar",
    "catonizasen": "catonizar",
    "catonizases": "catonizar",
    "catonizaste": "catonizar",
    "catonizasteis": "catonizar",
    "catonizo": "catonizar",
    "catonizábamos": "catonizar",
    "catonizáis": "catonizar",
    "catonizáramos": "catonizar",
    "catonizáremos": "catonizar",
    "catonizásemos": "catonizar",
    "catonizó": "catonizar",
    "catoptromancias": "catoptromancia",
    "catoptromancías": "catoptromancía",
    "catorceava": "catorceavo",
    "catorceavas": "catorceavo",
    "catorceavo": "14",
    "catorceavos": "catorceavo",
    "catos": "cato",
    "catrecillos": "catrecillo",
    "catres": "catre",
    "catábamos": "catar",
    "catáforas": "catáfora",
    "catáis": "catar",
    "catálogos": "catálogo",
    "catáramos": "catar",
    "catáremos": "catar",
    "catártica": "catártico",
    "catárticas": "catártico",
    "catárticos": "catártico",
    "catártidos": "catártido",
    "catásemos": "catar",
    "catástrofes": "catástrofe",
    "caté": "catar",
    "catéis": "catar",
    "catéteres": "catéter",
    "cató": "catar",
    "catódica": "catódico",
    "catódicas": "catódico",
    "catódicos": "catódico",
    "católica": "católico",
    "católicas": "católico",
    "católicos": "católico",
    "catóptrica": "catóptrico",
    "catóptricas": "catóptrico",
    "catóptricos": "catóptrico",
    "caucasiana": "caucasiano",
    "caucasianas": "caucasiano",
    "caucasianos": "caucasiano",
    "caucasoides": "caucasoide",
    "cauces": "cauce",
    "cauchales": "cauchal",
    "cauchera": "cauchero",
    "caucheras": "cauchero",
    "caucheros": "cauchero",
    "cauchos": "caucho",
    "cauchotada": "cauchotado",
    "cauchotadas": "cauchotado",
    "cauchotados": "cauchotado",
    "cauchutada": "cauchutado",
    "cauchutadas": "cauchutado",
    "cauchutados": "cauchutado",
    "cauciona": "caucionar",
    "caucionaba": "caucionar",
    "caucionabais": "caucionar",
    "caucionaban": "caucionar",
    "caucionabas": "caucionar",
    "caucionables": "caucionable",
    "caucionad": "caucionar",
    "caucionada": "caucionar",
    "caucionadas": "caucionar",
    "caucionado": "caucionar",
    "caucionados": "caucionar",
    "caucionamos": "caucionar",
    "caucionan": "caucionar",
    "caucionando": "caucionar",
    "caucionara": "caucionar",
    "caucionarais": "caucionar",
    "caucionaran": "caucionar",
    "caucionaras": "caucionar",
    "caucionare": "caucionar",
    "caucionareis": "caucionar",
    "caucionaremos": "caucionar",
    "caucionaren": "caucionar",
    "caucionares": "caucionar",
    "caucionaron": "caucionar",
    "caucionará": "caucionar",
    "caucionarán": "caucionar",
    "caucionarás": "caucionar",
    "caucionaré": "caucionar",
    "caucionaréis": "caucionar",
    "caucionaría": "caucionar",
    "caucionaríais": "caucionar",
    "caucionaríamos": "caucionar",
    "caucionarían": "caucionar",
    "caucionarías": "caucionar",
    "caucionas": "caucionar",
    "caucionase": "caucionar",
    "caucionaseis": "caucionar",
    "caucionasen": "caucionar",
    "caucionases": "caucionar",
    "caucionaste": "caucionar",
    "caucionasteis": "caucionar",
    "caucione": "caucionar",
    "caucionemos": "caucionar",
    "caucionen": "caucionar",
    "cauciones": "caucionar",
    "cauciono": "caucionar",
    "caucionábamos": "caucionar",
    "caucionáis": "caucionar",
    "caucionáramos": "caucionar",
    "caucionáremos": "caucionar",
    "caucionásemos": "caucionar",
    "caucioné": "caucionar",
    "caucionéis": "caucionar",
    "caucionó": "caucionar",
    "caucos": "cauco",
    "caucásea": "caucáseo",
    "caucáseas": "caucáseo",
    "caucáseos": "caucáseo",
    "caucásica": "caucásico",
    "caucásicas": "caucásico",
    "caucásicos": "caucásico",
    "caudada": "caudado",
    "caudadas": "caudado",
    "caudados": "caudado",
    "caudales": "caudal",
    "caudalosa": "caudaloso",
    "caudalosas": "caudaloso",
    "caudalosos": "caudaloso",
    "caudalómetros": "caudalómetro",
    "caudas": "cauda",
    "caudataria": "caudatario",
    "caudatarias": "caudatario",
    "caudatarios": "caudatario",
    "caudillajes": "caudillaje",
    "caudillismos": "caudillismo",
    "caudillos": "caudillo",
    "caudimana": "caudímano",
    "caudimanas": "caudímano",
    "caudimanos": "caudímano",
    "caudímana": "caudímano",
    "caudímanas": "caudímano",
    "caudímanos": "caudímano",
    "caulescentes": "caulescente",
    "caulícolos": "caulícolo",
    "caulículos": "caulículo",
    "caulífera": "caulífero",
    "caulíferas": "caulífero",
    "caulíferos": "caulífero",
    "caulógena": "caulógeno",
    "caulógenas": "caulógeno",
    "caulógenos": "caulógeno",
    "cauros": "cauro",
    "causa": "causar",
    "causaba": "causar",
    "causabais": "causar",
    "causaban": "causar",
    "causabas": "causar",
    "causaciones": "causación",
    "causad": "causar",
    "causada": "causar",
    "causadas": "causar",
    "causado": "causar",
    "causados": "causar",
    "causahabientes": "causahabiente",
    "causales": "causal",
    "causalidades": "causalidad",
    "causalismos": "causalismo",
    "causamos": "causar",
    "causan": "causar",
    "causando": "causar",
    "causantes": "causante",
    "causara": "causar",
    "causarais": "causar",
    "causaran": "causar",
    "causaras": "causar",
    "causare": "causar",
    "causareis": "causar",
    "causaremos": "causar",
    "causaren": "causar",
    "causares": "causar",
    "causaron": "causar",
    "causará": "causar",
    "causarán": "causar",
    "causarás": "causar",
    "causaré": "causar",
    "causaréis": "causar",
    "causaría": "causar",
    "causaríais": "causar",
    "causaríamos": "causar",
    "causarían": "causar",
    "causarías": "causar",
    "causas": "causar",
    "causase": "causar",
    "causaseis": "causar",
    "causasen": "causar",
    "causases": "causar",
    "causaste": "causar",
    "causasteis": "causar",
    "causativa": "causativo",
    "causativas": "causativo",
    "causativos": "causativo",
    "cause": "causar",
    "causea": "causear",
    "causeaba": "causear",
    "causeabais": "causear",
    "causeaban": "causear",
    "causeabas": "causear",
    "causead": "causear",
    "causeada": "causear",
    "causeadas": "causear",
    "causeado": "causear",
    "causeados": "causear",
    "causeamos": "causear",
    "causean": "causear",
    "causeando": "causear",
    "causeara": "causear",
    "causearais": "causear",
    "causearan": "causear",
    "causearas": "causear",
    "causeare": "causear",
    "causeareis": "causear",
    "causearemos": "causear",
    "causearen": "causear",
    "causeares": "causear",
    "causearon": "causear",
    "causeará": "causear",
    "causearán": "causear",
    "causearás": "causear",
    "causearé": "causear",
    "causearéis": "causear",
    "causearía": "causear",
    "causearíais": "causear",
    "causearíamos": "causear",
    "causearían": "causear",
    "causearías": "causear",
    "causeas": "causear",
    "causease": "causear",
    "causeaseis": "causear",
    "causeasen": "causear",
    "causeases": "causear",
    "causeaste": "causear",
    "causeasteis": "causear",
    "causee": "causear",
    "causeemos": "causear",
    "causeen": "causear",
    "causees": "causear",
    "causemos": "causar",
    "causen": "causar",
    "causeo": "causear",
    "causes": "causar",
    "causeábamos": "causear",
    "causeáis": "causear",
    "causeáramos": "causear",
    "causeáremos": "causear",
    "causeásemos": "causear",
    "causeé": "causear",
    "causeéis": "causear",
    "causeó": "causear",
    "causo": "causar",
    "causones": "causón",
    "causticidades": "causticidad",
    "causábamos": "causar",
    "causáis": "causar",
    "causáramos": "causar",
    "causáremos": "causar",
    "causásemos": "causar",
    "causé": "causar",
    "causéis": "causar",
    "causídica": "causídico",
    "causídicas": "causídico",
    "causídicos": "causídico",
    "causó": "causar",
    "cauta": "cauto",
    "cautas": "cauto",
    "cautela": "cautelar",
    "cautelaba": "cautelar",
    "cautelabais": "cautelar",
    "cautelaban": "cautelar",
    "cautelabas": "cautelar",
    "cautelad": "cautelar",
    "cautelada": "cautelar",
    "cauteladas": "cautelar",
    "cautelado": "cautelar",
    "cautelados": "cautelar",
    "cautelamos": "cautelar",
    "cautelan": "cautelar",
    "cautelando": "cautelar",
    "cautelara": "cautelar",
    "cautelarais": "cautelar",
    "cautelaran": "cautelar",
    "cautelaras": "cautelar",
    "cautelare": "cautelar",
    "cautelareis": "cautelar",
    "cautelaremos": "cautelar",
    "cautelaren": "cautelar",
    "cautelares": "cautelar",
    "cautelaron": "cautelar",
    "cautelará": "cautelar",
    "cautelarán": "cautelar",
    "cautelarás": "cautelar",
    "cautelaré": "cautelar",
    "cautelaréis": "cautelar",
    "cautelaría": "cautelar",
    "cautelaríais": "cautelar",
    "cautelaríamos": "cautelar",
    "cautelarían": "cautelar",
    "cautelarías": "cautelar",
    "cautelas": "cautelar",
    "cautelase": "cautelar",
    "cautelaseis": "cautelar",
    "cautelasen": "cautelar",
    "cautelases": "cautelar",
    "cautelaste": "cautelar",
    "cautelasteis": "cautelar",
    "cautele": "cautelar",
    "cautelemos": "cautelar",
    "cautelen": "cautelar",
    "cauteles": "cautelar",
    "cautelo": "cautelar",
    "cautelosa": "cauteloso",
    "cautelosas": "cauteloso",
    "cautelosos": "cauteloso",
    "cautelábamos": "cautelar",
    "cauteláis": "cautelar",
    "cauteláramos": "cautelar",
    "cauteláremos": "cautelar",
    "cautelásemos": "cautelar",
    "cautelé": "cautelar",
    "cauteléis": "cautelar",
    "cauteló": "cautelar",
    "cauterice": "cauterizar",
    "cautericemos": "cauterizar",
    "cautericen": "cauterizar",
    "cauterices": "cauterizar",
    "cautericé": "cauterizar",
    "cautericéis": "cauterizar",
    "cauterios": "cauterio",
    "cauteriza": "cauterizar",
    "cauterizaba": "cauterizar",
    "cauterizabais": "cauterizar",
    "cauterizaban": "cauterizar",
    "cauterizabas": "cauterizar",
    "cauterizaciones": "cauterización",
    "cauterizad": "cauterizar",
    "cauterizada": "cauterizar",
    "cauterizadas": "cauterizar",
    "cauterizado": "cauterizar",
    "cauterizadora": "cauterizador",
    "cauterizadoras": "cauterizador",
    "cauterizadores": "cauterizador",
    "cauterizados": "cauterizar",
    "cauterizamientos": "cauterizamiento",
    "cauterizamos": "cauterizar",
    "cauterizan": "cauterizar",
    "cauterizando": "cauterizar",
    "cauterizara": "cauterizar",
    "cauterizarais": "cauterizar",
    "cauterizaran": "cauterizar",
    "cauterizaras": "cauterizar",
    "cauterizare": "cauterizar",
    "cauterizareis": "cauterizar",
    "cauterizaremos": "cauterizar",
    "cauterizaren": "cauterizar",
    "cauterizares": "cauterizar",
    "cauterizaron": "cauterizar",
    "cauterizará": "cauterizar",
    "cauterizarán": "cauterizar",
    "cauterizarás": "cauterizar",
    "cauterizaré": "cauterizar",
    "cauterizaréis": "cauterizar",
    "cauterizaría": "cauterizar",
    "cauterizaríais": "cauterizar",
    "cauterizaríamos": "cauterizar",
    "cauterizarían": "cauterizar",
    "cauterizarías": "cauterizar",
    "cauterizas": "cauterizar",
    "cauterizase": "cauterizar",
    "cauterizaseis": "cauterizar",
    "cauterizasen": "cauterizar",
    "cauterizases": "cauterizar",
    "cauterizaste": "cauterizar",
    "cauterizasteis": "cauterizar",
    "cauterizo": "cauterizar",
    "cauterizábamos": "cauterizar",
    "cauterizáis": "cauterizar",
    "cauterizáramos": "cauterizar",
    "cauterizáremos": "cauterizar",
    "cauterizásemos": "cauterizar",
    "cauterizó": "cauterizar",
    "cautines": "cautín",
    "cautiva": "cautivo",
    "cautivaba": "cautivar",
    "cautivabais": "cautivar",
    "cautivaban": "cautivar",
    "cautivabas": "cautivar",
    "cautivad": "cautivar",
    "cautivada": "cautivar",
    "cautivadas": "cautivar",
    "cautivado": "cautivar",
    "cautivadora": "cautivador",
    "cautivadoras": "cautivador",
    "cautivadores": "cautivador",
    "cautivados": "cautivar",
    "cautivamos": "cautivar",
    "cautivan": "cautivar",
    "cautivando": "cautivar",
    "cautivara": "cautivar",
    "cautivarais": "cautivar",
    "cautivaran": "cautivar",
    "cautivaras": "cautivar",
    "cautivare": "cautivar",
    "cautivareis": "cautivar",
    "cautivaremos": "cautivar",
    "cautivaren": "cautivar",
    "cautivares": "cautivar",
    "cautivaron": "cautivar",
    "cautivará": "cautivar",
    "cautivarán": "cautivar",
    "cautivarás": "cautivar",
    "cautivaré": "cautivar",
    "cautivaréis": "cautivar",
    "cautivaría": "cautivar",
    "cautivaríais": "cautivar",
    "cautivaríamos": "cautivar",
    "cautivarían": "cautivar",
    "cautivarías": "cautivar",
    "cautivas": "cautivo",
    "cautivase": "cautivar",
    "cautivaseis": "cautivar",
    "cautivasen": "cautivar",
    "cautivases": "cautivar",
    "cautivaste": "cautivar",
    "cautivasteis": "cautivar",
    "cautive": "cautivar",
    "cautivemos": "cautivar",
    "cautiven": "cautivar",
    "cautiverios": "cautiverio",
    "cautives": "cautivar",
    "cautividades": "cautividad",
    "cautivo": "cautivar",
    "cautivos": "cautivo",
    "cautivábamos": "cautivar",
    "cautiváis": "cautivar",
    "cautiváramos": "cautivar",
    "cautiváremos": "cautivar",
    "cautivásemos": "cautivar",
    "cautivé": "cautivar",
    "cautivéis": "cautivar",
    "cautivó": "cautivar",
    "cautos": "cauto",
    "cava": "cavo",
    "cavaba": "cavar",
    "cavabais": "cavar",
    "cavaban": "cavar",
    "cavabas": "cavar",
    "cavacotes": "cavacote",
    "cavad": "cavar",
    "cavada": "cavar",
    "cavadas": "cavar",
    "cavado": "cavar",
    "cavadora": "cavador",
    "cavadoras": "cavador",
    "cavadores": "cavador",
    "cavados": "cavar",
    "cavaduras": "cavadura",
    "cavalillos": "cavalillo",
    "cavamos": "cavar",
    "cavan": "cavar",
    "cavando": "cavar",
    "cavara": "cavar",
    "cavarais": "cavar",
    "cavaran": "cavar",
    "cavaras": "cavar",
    "cavare": "cavar",
    "cavareis": "cavar",
    "cavaremos": "cavar",
    "cavaren": "cavar",
    "cavares": "cavar",
    "cavaron": "cavar",
    "cavará": "cavar",
    "cavarán": "cavar",
    "cavarás": "cavar",
    "cavaré": "cavar",
    "cavaréis": "cavar",
    "cavaría": "cavar",
    "cavaríais": "cavar",
    "cavaríamos": "cavar",
    "cavarían": "cavar",
    "cavarías": "cavar",
    "cavas": "cavo",
    "cavase": "cavar",
    "cavaseis": "cavar",
    "cavasen": "cavar",
    "cavases": "cavar",
    "cavaste": "cavar",
    "cavasteis": "cavar",
    "cavatinas": "cavatina",
    "cavazones": "cavazón",
    "cave": "cavar",
    "cavedios": "cavedio",
    "cavemos": "cavar",
    "caven": "cavar",
    "cavernaria": "cavernario",
    "cavernarias": "cavernario",
    "cavernarios": "cavernario",
    "cavernas": "caverna",
    "cavernosa": "cavernoso",
    "cavernosas": "cavernoso",
    "cavernosidades": "cavernosidad",
    "cavernosos": "cavernoso",
    "cavernícolas": "cavernícola",
    "caves": "cavar",
    "cavetos": "caveto",
    "caviares": "caviar",
    "cavias": "cavia",
    "cavidades": "cavidad",
    "cavila": "cavilar",
    "cavilaba": "cavilar",
    "cavilabais": "cavilar",
    "cavilaban": "cavilar",
    "cavilabas": "cavilar",
    "cavilaciones": "cavilación",
    "cavilad": "cavilar",
    "cavilada": "cavilar",
    "caviladas": "cavilar",
    "cavilado": "cavilar",
    "cavilados": "cavilar",
    "cavilamos": "cavilar",
    "cavilan": "cavilar",
    "cavilando": "cavilar",
    "cavilara": "cavilar",
    "cavilarais": "cavilar",
    "cavilaran": "cavilar",
    "cavilaras": "cavilar",
    "cavilare": "cavilar",
    "cavilareis": "cavilar",
    "cavilaremos": "cavilar",
    "cavilaren": "cavilar",
    "cavilares": "cavilar",
    "cavilaron": "cavilar",
    "cavilará": "cavilar",
    "cavilarán": "cavilar",
    "cavilarás": "cavilar",
    "cavilaré": "cavilar",
    "cavilaréis": "cavilar",
    "cavilaría": "cavilar",
    "cavilaríais": "cavilar",
    "cavilaríamos": "cavilar",
    "cavilarían": "cavilar",
    "cavilarías": "cavilar",
    "cavilas": "cavilar",
    "cavilase": "cavilar",
    "cavilaseis": "cavilar",
    "cavilasen": "cavilar",
    "cavilases": "cavilar",
    "cavilaste": "cavilar",
    "cavilasteis": "cavilar",
    "cavile": "cavilar",
    "cavilemos": "cavilar",
    "cavilen": "cavilar",
    "caviles": "cavilar",
    "cavilo": "cavilar",
    "cavilosa": "caviloso",
    "cavilosas": "caviloso",
    "cavilosidades": "cavilosidad",
    "cavilosos": "caviloso",
    "cavilábamos": "cavilar",
    "caviláis": "cavilar",
    "caviláramos": "cavilar",
    "caviláremos": "cavilar",
    "cavilásemos": "cavilar",
    "cavilé": "cavilar",
    "caviléis": "cavilar",
    "caviló": "cavilar",
    "cavitaciones": "cavitación",
    "cavo": "cavar",
    "cavos": "cavo",
    "cavábamos": "cavar",
    "caváis": "cavar",
    "caváramos": "cavar",
    "caváremos": "cavar",
    "cavásemos": "cavar",
    "cavé": "cavar",
    "cavéis": "cavar",
    "cavó": "cavar",
    "cayadillas": "cayadilla",
    "cayadoes": "cayado",
    "cayados": "cayado",
    "cayapea": "cayapear",
    "cayapeaba": "cayapear",
    "cayapeabais": "cayapear",
    "cayapeaban": "cayapear",
    "cayapeabas": "cayapear",
    "cayapead": "cayapear",
    "cayapeada": "cayapear",
    "cayapeadas": "cayapear",
    "cayapeado": "cayapear",
    "cayapeados": "cayapear",
    "cayapeamos": "cayapear",
    "cayapean": "cayapear",
    "cayapeando": "cayapear",
    "cayapeara": "cayapear",
    "cayapearais": "cayapear",
    "cayapearan": "cayapear",
    "cayapearas": "cayapear",
    "cayapeare": "cayapear",
    "cayapeareis": "cayapear",
    "cayapearemos": "cayapear",
    "cayapearen": "cayapear",
    "cayapeares": "cayapear",
    "cayapearon": "cayapear",
    "cayapeará": "cayapear",
    "cayapearán": "cayapear",
    "cayapearás": "cayapear",
    "cayapearé": "cayapear",
    "cayapearéis": "cayapear",
    "cayapearía": "cayapear",
    "cayapearíais": "cayapear",
    "cayapearíamos": "cayapear",
    "cayapearían": "cayapear",
    "cayapearías": "cayapear",
    "cayapeas": "cayapear",
    "cayapease": "cayapear",
    "cayapeaseis": "cayapear",
    "cayapeasen": "cayapear",
    "cayapeases": "cayapear",
    "cayapeaste": "cayapear",
    "cayapeasteis": "cayapear",
    "cayapee": "cayapear",
    "cayapeemos": "cayapear",
    "cayapeen": "cayapear",
    "cayapees": "cayapear",
    "cayapeo": "cayapear",
    "cayapeábamos": "cayapear",
    "cayapeáis": "cayapear",
    "cayapeáramos": "cayapear",
    "cayapeáremos": "cayapear",
    "cayapeásemos": "cayapear",
    "cayapeé": "cayapear",
    "cayapeéis": "cayapear",
    "cayapeó": "cayapear",
    "cayenas": "cayena",
    "cayendo": "caer",
    "cayeputis": "cayeputi",
    "cayera": "caer",
    "cayerais": "caer",
    "cayeran": "caer",
    "cayeras": "caer",
    "cayere": "caer",
    "cayereis": "caer",
    "cayeren": "caer",
    "cayeres": "caer",
    "cayeron": "caer",
    "cayese": "caer",
    "cayeseis": "caer",
    "cayesen": "caer",
    "cayeses": "caer",
    "cayos": "cayo",
    "cayotes": "cayote",
    "cayás": "cayá",
    "cayéramos": "caer",
    "cayéremos": "caer",
    "cayésemos": "caer",
    "cayó": "caer",
    "caza": "cazar",
    "cazaba": "cazar",
    "cazabais": "cazar",
    "cazaban": "cazar",
    "cazabas": "cazar",
    "cazabes": "cazabe",
    "cazabombardera": "cazabombardero",
    "cazabombarderas": "cazabombardero",
    "cazabombarderos": "cazabombardero",
    "cazad": "cazar",
    "cazada": "cazar",
    "cazadas": "cazar",
    "cazadera": "cazadero",
    "cazaderas": "cazadero",
    "cazaderos": "cazadero",
    "cazado": "cazar",
    "cazadora": "cazador",
    "cazadoras": "cazadora",
    "cazadores": "cazador",
    "cazados": "cazar",
    "cazallas": "cazalla",
    "cazamos": "cazar",
    "cazan": "cazar",
    "cazando": "cazar",
    "cazara": "cazar",
    "cazarais": "cazar",
    "cazaran": "cazar",
    "cazaras": "cazar",
    "cazare": "cazar",
    "cazareis": "cazar",
    "cazaremos": "cazar",
    "cazaren": "cazar",
    "cazares": "cazar",
    "cazaretes": "cazarete",
    "cazaron": "cazar",
    "cazará": "cazar",
    "cazarán": "cazar",
    "cazarás": "cazar",
    "cazaré": "cazar",
    "cazaréis": "cazar",
    "cazaría": "cazar",
    "cazaríais": "cazar",
    "cazaríamos": "cazar",
    "cazarían": "cazar",
    "cazarías": "cazar",
    "cazas": "cazar",
    "cazase": "cazar",
    "cazaseis": "cazar",
    "cazasen": "cazar",
    "cazases": "cazar",
    "cazaste": "cazar",
    "cazasteis": "cazar",
    "cazasubmarinos": "cazasubmarino",
    "cazatorpederos": "cazatorpedero",
    "cazcalea": "cazcalear",
    "cazcaleaba": "cazcalear",
    "cazcaleabais": "cazcalear",
    "cazcaleaban": "cazcalear",
    "cazcaleabas": "cazcalear",
    "cazcalead": "cazcalear",
    "cazcaleada": "cazcalear",
    "cazcaleadas": "cazcalear",
    "cazcaleado": "cazcalear",
    "cazcaleados": "cazcalear",
    "cazcaleamos": "cazcalear",
    "cazcalean": "cazcalear",
    "cazcaleando": "cazcalear",
    "cazcaleara": "cazcalear",
    "cazcalearais": "cazcalear",
    "cazcalearan": "cazcalear",
    "cazcalearas": "cazcalear",
    "cazcaleare": "cazcalear",
    "cazcaleareis": "cazcalear",
    "cazcalearemos": "cazcalear",
    "cazcalearen": "cazcalear",
    "cazcaleares": "cazcalear",
    "cazcalearon": "cazcalear",
    "cazcaleará": "cazcalear",
    "cazcalearán": "cazcalear",
    "cazcalearás": "cazcalear",
    "cazcalearé": "cazcalear",
    "cazcalearéis": "cazcalear",
    "cazcalearía": "cazcalear",
    "cazcalearíais": "cazcalear",
    "cazcalearíamos": "cazcalear",
    "cazcalearían": "cazcalear",
    "cazcalearías": "cazcalear",
    "cazcaleas": "cazcalear",
    "cazcalease": "cazcalear",
    "cazcaleaseis": "cazcalear",
    "cazcaleasen": "cazcalear",
    "cazcaleases": "cazcalear",
    "cazcaleaste": "cazcalear",
    "cazcaleasteis": "cazcalear",
    "cazcalee": "cazcalear",
    "cazcaleemos": "cazcalear",
    "cazcaleen": "cazcalear",
    "cazcalees": "cazcalear",
    "cazcaleo": "cazcalear",
    "cazcaleábamos": "cazcalear",
    "cazcaleáis": "cazcalear",
    "cazcaleáramos": "cazcalear",
    "cazcaleáremos": "cazcalear",
    "cazcaleásemos": "cazcalear",
    "cazcaleé": "cazcalear",
    "cazcaleéis": "cazcalear",
    "cazcaleó": "cazcalear",
    "cazcarrias": "cazcarria",
    "cazcarrienta": "cazcarriento",
    "cazcarrientas": "cazcarriento",
    "cazcarrientos": "cazcarriento",
    "cazcorva": "cazcorvo",
    "cazcorvas": "cazcorvo",
    "cazcorvos": "cazcorvo",
    "cazo": "cazar",
    "cazoladas": "cazolada",
    "cazoletas": "cazoleta",
    "cazoletea": "cazoletear",
    "cazoleteaba": "cazoletear",
    "cazoleteabais": "cazoletear",
    "cazoleteaban": "cazoletear",
    "cazoleteabas": "cazoletear",
    "cazoletead": "cazoletear",
    "cazoleteada": "cazoletear",
    "cazoleteadas": "cazoletear",
    "cazoleteado": "cazoletear",
    "cazoleteados": "cazoletear",
    "cazoleteamos": "cazoletear",
    "cazoletean": "cazoletear",
    "cazoleteando": "cazoletear",
    "cazoleteara": "cazoletear",
    "cazoletearais": "cazoletear",
    "cazoletearan": "cazoletear",
    "cazoletearas": "cazoletear",
    "cazoleteare": "cazoletear",
    "cazoleteareis": "cazoletear",
    "cazoletearemos": "cazoletear",
    "cazoletearen": "cazoletear",
    "cazoleteares": "cazoletear",
    "cazoletearon": "cazoletear",
    "cazoleteará": "cazoletear",
    "cazoletearán": "cazoletear",
    "cazoletearás": "cazoletear",
    "cazoletearé": "cazoletear",
    "cazoletearéis": "cazoletear",
    "cazoletearía": "cazoletear",
    "cazoletearíais": "cazoletear",
    "cazoletearíamos": "cazoletear",
    "cazoletearían": "cazoletear",
    "cazoletearías": "cazoletear",
    "cazoleteas": "cazoletear",
    "cazoletease": "cazoletear",
    "cazoleteaseis": "cazoletear",
    "cazoleteasen": "cazoletear",
    "cazoleteases": "cazoletear",
    "cazoleteaste": "cazoletear",
    "cazoleteasteis": "cazoletear",
    "cazoletee": "cazoletear",
    "cazoleteemos": "cazoletear",
    "cazoleteen": "cazoletear",
    "cazoletees": "cazoletear",
    "cazoleteo": "cazoletear",
    "cazoletera": "cazoletero",
    "cazoleteras": "cazoletero",
    "cazoleteros": "cazoletero",
    "cazoleteábamos": "cazoletear",
    "cazoleteáis": "cazoletear",
    "cazoleteáramos": "cazoletear",
    "cazoleteáremos": "cazoletear",
    "cazoleteásemos": "cazoletear",
    "cazoleteé": "cazoletear",
    "cazoleteéis": "cazoletear",
    "cazoleteó": "cazoletear",
    "cazonales": "cazonal",
    "cazones": "cazón",
    "cazos": "cazo",
    "cazuda": "cazudo",
    "cazudas": "cazudo",
    "cazudos": "cazudo",
    "cazuelas": "cazuela",
    "cazumbra": "cazumbrar",
    "cazumbraba": "cazumbrar",
    "cazumbrabais": "cazumbrar",
    "cazumbraban": "cazumbrar",
    "cazumbrabas": "cazumbrar",
    "cazumbrad": "cazumbrar",
    "cazumbrada": "cazumbrar",
    "cazumbradas": "cazumbrar",
    "cazumbrado": "cazumbrar",
    "cazumbrados": "cazumbrar",
    "cazumbramos": "cazumbrar",
    "cazumbran": "cazumbrar",
    "cazumbrando": "cazumbrar",
    "cazumbrara": "cazumbrar",
    "cazumbrarais": "cazumbrar",
    "cazumbraran": "cazumbrar",
    "cazumbraras": "cazumbrar",
    "cazumbrare": "cazumbrar",
    "cazumbrareis": "cazumbrar",
    "cazumbraremos": "cazumbrar",
    "cazumbraren": "cazumbrar",
    "cazumbrares": "cazumbrar",
    "cazumbraron": "cazumbrar",
    "cazumbrará": "cazumbrar",
    "cazumbrarán": "cazumbrar",
    "cazumbrarás": "cazumbrar",
    "cazumbraré": "cazumbrar",
    "cazumbraréis": "cazumbrar",
    "cazumbraría": "cazumbrar",
    "cazumbraríais": "cazumbrar",
    "cazumbraríamos": "cazumbrar",
    "cazumbrarían": "cazumbrar",
    "cazumbrarías": "cazumbrar",
    "cazumbras": "cazumbrar",
    "cazumbrase": "cazumbrar",
    "cazumbraseis": "cazumbrar",
    "cazumbrasen": "cazumbrar",
    "cazumbrases": "cazumbrar",
    "cazumbraste": "cazumbrar",
    "cazumbrasteis": "cazumbrar",
    "cazumbre": "cazumbrar",
    "cazumbremos": "cazumbrar",
    "cazumbren": "cazumbrar",
    "cazumbres": "cazumbre",
    "cazumbro": "cazumbrar",
    "cazumbrábamos": "cazumbrar",
    "cazumbráis": "cazumbrar",
    "cazumbráramos": "cazumbrar",
    "cazumbráremos": "cazumbrar",
    "cazumbrásemos": "cazumbrar",
    "cazumbré": "cazumbrar",
    "cazumbréis": "cazumbrar",
    "cazumbró": "cazumbrar",
    "cazurra": "cazurro",
    "cazurras": "cazurro",
    "cazurrerías": "cazurrería",
    "cazurros": "cazurro",
    "cazábamos": "cazar",
    "cazáis": "cazar",
    "cazáramos": "cazar",
    "cazáremos": "cazar",
    "cazásemos": "cazar",
    "cazó": "cazar",
    "caéis": "caer",
    "caí": "caer",
    "caía": "caer",
    "caíais": "caer",
    "caíamos": "caer",
    "caían": "caer",
    "caías": "caer",
    "caída": "caer",
    "caídas": "caída",
    "caídes": "caíd",
    "caído": "caer",
    "caídos": "caído",
    "caíes": "caí",
    "caímos": "caer",
    "caís": "caí",
    "caíste": "caer",
    "caísteis": "caer",
    "cañabrejas": "cañabreja",
    "cañacoros": "cañacoro",
    "cañadas": "cañada",
    "cañados": "cañado",
    "cañafístulas": "cañafístula",
    "cañahejas": "cañaheja",
    "cañaliegas": "cañaliega",
    "cañamazos": "cañamazo",
    "cañamelares": "cañamelar",
    "cañamera": "cañamero",
    "cañameras": "cañamero",
    "cañameros": "cañamero",
    "cañameña": "cañameño",
    "cañameñas": "cañameño",
    "cañameños": "cañameño",
    "cañamieles": "cañamiel",
    "cañamoncillos": "cañamoncillo",
    "cañamones": "cañamón",
    "cañares": "cañar",
    "cañariega": "cañariego",
    "cañariegas": "cañariego",
    "cañariegos": "cañariego",
    "cañas": "caña",
    "cañaverales": "cañaveral",
    "cañazos": "cañazo",
    "cañedos": "cañedo",
    "cañerías": "cañería",
    "cañizales": "cañizal",
    "cañizares": "cañizar",
    "cañizos": "cañizo",
    "cañocales": "cañocal",
    "cañonazos": "cañonazo",
    "cañonea": "cañonear",
    "cañoneaba": "cañonear",
    "cañoneabais": "cañonear",
    "cañoneaban": "cañonear",
    "cañoneabas": "cañonear",
    "cañonead": "cañonear",
    "cañoneada": "cañonear",
    "cañoneadas": "cañonear",
    "cañoneado": "cañonear",
    "cañoneados": "cañonear",
    "cañoneamos": "cañonear",
    "cañonean": "cañonear",
    "cañoneando": "cañonear",
    "cañoneara": "cañonear",
    "cañonearais": "cañonear",
    "cañonearan": "cañonear",
    "cañonearas": "cañonear",
    "cañoneare": "cañonear",
    "cañoneareis": "cañonear",
    "cañonearemos": "cañonear",
    "cañonearen": "cañonear",
    "cañoneares": "cañonear",
    "cañonearon": "cañonear",
    "cañoneará": "cañonear",
    "cañonearán": "cañonear",
    "cañonearás": "cañonear",
    "cañonearé": "cañonear",
    "cañonearéis": "cañonear",
    "cañonearía": "cañonear",
    "cañonearíais": "cañonear",
    "cañonearíamos": "cañonear",
    "cañonearían": "cañonear",
    "cañonearías": "cañonear",
    "cañoneas": "cañonear",
    "cañonease": "cañonear",
    "cañoneaseis": "cañonear",
    "cañoneasen": "cañonear",
    "cañoneases": "cañonear",
    "cañoneaste": "cañonear",
    "cañoneasteis": "cañonear",
    "cañonee": "cañonear",
    "cañoneemos": "cañonear",
    "cañoneen": "cañonear",
    "cañonees": "cañonear",
    "cañoneo": "cañonear",
    "cañoneos": "cañoneo",
    "cañonera": "cañoner",
    "cañoneras": "cañonera",
    "cañoneres": "cañoner",
    "cañoneros": "cañonero",
    "cañonerías": "cañonería",
    "cañones": "cañón",
    "cañoneábamos": "cañonear",
    "cañoneáis": "cañonear",
    "cañoneáramos": "cañonear",
    "cañoneáremos": "cañonear",
    "cañoneásemos": "cañonear",
    "cañoneé": "cañonear",
    "cañoneéis": "cañonear",
    "cañoneó": "cañonear",
    "caños": "caño",
    "cañutazos": "cañutazo",
    "cañuteros": "cañutero",
    "cañuterías": "cañutería",
    "cañutillos": "cañutillo",
    "cañutos": "cañuto",
    "cañís": "cañí",
    "caótica": "caótico",
    "caóticas": "caótico",
    "caóticos": "caótico",
    "caústicos": "caústico",
    "cd-roms": "cd-rom",
    "cdrom": "cd-rom",
    "cdroms": "cd-rom",
    "ceba": "cebar",
    "cebaba": "cebar",
    "cebabais": "cebar",
    "cebaban": "cebar",
    "cebabas": "cebar",
    "cebad": "cebar",
    "cebada": "cebar",
    "cebadaba": "cebadar",
    "cebadabais": "cebadar",
    "cebadaban": "cebadar",
    "cebadabas": "cebadar",
    "cebadad": "cebadar",
    "cebadada": "cebadar",
    "cebadadas": "cebadar",
    "cebadado": "cebadar",
    "cebadados": "cebadar",
    "cebadales": "cebadal",
    "cebadamos": "cebadar",
    "cebadan": "cebadar",
    "cebadando": "cebadar",
    "cebadara": "cebadar",
    "cebadarais": "cebadar",
    "cebadaran": "cebadar",
    "cebadaras": "cebadar",
    "cebadare": "cebadar",
    "cebadareis": "cebadar",
    "cebadaremos": "cebadar",
    "cebadaren": "cebadar",
    "cebadares": "cebadar",
    "cebadaron": "cebadar",
    "cebadará": "cebadar",
    "cebadarán": "cebadar",
    "cebadarás": "cebadar",
    "cebadaré": "cebadar",
    "cebadaréis": "cebadar",
    "cebadaría": "cebadar",
    "cebadaríais": "cebadar",
    "cebadaríamos": "cebadar",
    "cebadarían": "cebadar",
    "cebadarías": "cebadar",
    "cebadas": "cebar",
    "cebadase": "cebadar",
    "cebadaseis": "cebadar",
    "cebadasen": "cebadar",
    "cebadases": "cebadar",
    "cebadaste": "cebadar",
    "cebadasteis": "cebadar",
    "cebade": "cebadar",
    "cebademos": "cebadar",
    "cebaden": "cebadar",
    "cebaderas": "cebadera",
    "cebaderos": "cebadero",
    "cebades": "cebadar",
    "cebadillas": "cebadilla",
    "cebado": "cebar",
    "cebadora": "cebador",
    "cebadoras": "cebador",
    "cebadores": "cebador",
    "cebados": "cebar",
    "cebaduras": "cebadura",
    "cebadábamos": "cebadar",
    "cebadáis": "cebadar",
    "cebadáramos": "cebadar",
    "cebadáremos": "cebadar",
    "cebadásemos": "cebadar",
    "cebadé": "cebadar",
    "cebadéis": "cebadar",
    "cebadó": "cebadar",
    "cebamos": "cebar",
    "ceban": "cebar",
    "cebando": "cebar",
    "cebara": "cebar",
    "cebarais": "cebar",
    "cebaran": "cebar",
    "cebaras": "cebar",
    "cebare": "cebar",
    "cebareis": "cebar",
    "cebaremos": "cebar",
    "cebaren": "cebar",
    "cebares": "cebar",
    "cebaron": "cebar",
    "cebará": "cebar",
    "cebarán": "cebar",
    "cebarás": "cebar",
    "cebaré": "cebar",
    "cebaréis": "cebar",
    "cebaría": "cebar",
    "cebaríais": "cebar",
    "cebaríamos": "cebar",
    "cebarían": "cebar",
    "cebarías": "cebar",
    "cebas": "cebar",
    "cebase": "cebar",
    "cebaseis": "cebar",
    "cebasen": "cebar",
    "cebases": "cebar",
    "cebaste": "cebar",
    "cebasteis": "cebar",
    "cebe": "cebar",
    "cebellina": "cebellino",
    "cebellinas": "cebellino",
    "cebellinos": "cebellino",
    "cebemos": "cebar",
    "ceben": "cebar",
    "cebes": "cebar",
    "cebo": "cebar",
    "cebollanas": "cebollana",
    "cebollares": "cebollar",
    "cebollas": "cebolla",
    "cebollera": "cebollero",
    "cebolleras": "cebollero",
    "cebolleros": "cebollero",
    "cebolletas": "cebolleta",
    "cebollinos": "cebollino",
    "cebolluda": "cebolludo",
    "cebolludas": "cebolludo",
    "cebolludos": "cebolludo",
    "cebona": "cebón",
    "cebonas": "cebón",
    "cebones": "cebón",
    "cebos": "cebo",
    "cebrada": "cebrado",
    "cebradas": "cebrado",
    "cebrados": "cebrado",
    "cebras": "cebra",
    "cebreada": "cebreado",
    "cebreadas": "cebreado",
    "cebreados": "cebreado",
    "cebruna": "cebruno",
    "cebrunas": "cebruno",
    "cebrunos": "cebruno",
    "ceburra": "ceburro",
    "ceburras": "ceburro",
    "ceburros": "ceburro",
    "cebábamos": "cebar",
    "cebáis": "cebar",
    "cebáramos": "cebar",
    "cebáremos": "cebar",
    "cebásemos": "cebar",
    "cebé": "cebar",
    "cebéis": "cebar",
    "cebó": "cebar",
    "cebúes": "cebú",
    "cebús": "cebú",
    "cecales": "cecal",
    "cecas": "ceca",
    "cecea": "cecear",
    "ceceaba": "cecear",
    "ceceabais": "cecear",
    "ceceaban": "cecear",
    "ceceabas": "cecear",
    "cecead": "cecear",
    "ceceada": "cecear",
    "ceceadas": "cecear",
    "ceceado": "cecear",
    "ceceados": "cecear",
    "ceceamos": "cecear",
    "cecean": "cecear",
    "ceceando": "cecear",
    "ceceara": "cecear",
    "cecearais": "cecear",
    "cecearan": "cecear",
    "cecearas": "cecear",
    "ceceare": "cecear",
    "ceceareis": "cecear",
    "cecearemos": "cecear",
    "cecearen": "cecear",
    "ceceares": "cecear",
    "cecearon": "cecear",
    "ceceará": "cecear",
    "cecearán": "cecear",
    "cecearás": "cecear",
    "cecearé": "cecear",
    "cecearéis": "cecear",
    "cecearía": "cecear",
    "cecearíais": "cecear",
    "cecearíamos": "cecear",
    "cecearían": "cecear",
    "cecearías": "cecear",
    "ceceas": "cecear",
    "cecease": "cecear",
    "ceceaseis": "cecear",
    "ceceasen": "cecear",
    "ceceases": "cecear",
    "ceceaste": "cecear",
    "ceceasteis": "cecear",
    "cecee": "cecear",
    "ceceemos": "cecear",
    "ceceen": "cecear",
    "cecees": "cecear",
    "ceceo": "cecear",
    "ceceos": "ceceo",
    "ceceosa": "ceceoso",
    "ceceosas": "ceceoso",
    "ceceosos": "ceceoso",
    "ceceábamos": "cecear",
    "ceceáis": "cecear",
    "ceceáramos": "cecear",
    "ceceáremos": "cecear",
    "ceceásemos": "cecear",
    "ceceé": "cecear",
    "ceceéis": "cecear",
    "ceceó": "cecear",
    "ceciales": "cecial",
    "cecidias": "cecidia",
    "cecidios": "cecidio",
    "cecinas": "cecina",
    "cecografías": "cecografía",
    "cecógrafos": "cecógrafo",
    "ceda": "ceder",
    "cedacea": "cedacear",
    "cedaceaba": "cedacear",
    "cedaceabais": "cedacear",
    "cedaceaban": "cedacear",
    "cedaceabas": "cedacear",
    "cedacead": "cedacear",
    "cedaceada": "cedacear",
    "cedaceadas": "cedacear",
    "cedaceado": "cedacear",
    "cedaceados": "cedacear",
    "cedaceamos": "cedacear",
    "cedacean": "cedacear",
    "cedaceando": "cedacear",
    "cedaceara": "cedacear",
    "cedacearais": "cedacear",
    "cedacearan": "cedacear",
    "cedacearas": "cedacear",
    "cedaceare": "cedacear",
    "cedaceareis": "cedacear",
    "cedacearemos": "cedacear",
    "cedacearen": "cedacear",
    "cedaceares": "cedacear",
    "cedacearon": "cedacear",
    "cedaceará": "cedacear",
    "cedacearán": "cedacear",
    "cedacearás": "cedacear",
    "cedacearé": "cedacear",
    "cedacearéis": "cedacear",
    "cedacearía": "cedacear",
    "cedacearíais": "cedacear",
    "cedacearíamos": "cedacear",
    "cedacearían": "cedacear",
    "cedacearías": "cedacear",
    "cedaceas": "cedacear",
    "cedacease": "cedacear",
    "cedaceaseis": "cedacear",
    "cedaceasen": "cedacear",
    "cedaceases": "cedacear",
    "cedaceaste": "cedacear",
    "cedaceasteis": "cedacear",
    "cedacee": "cedacear",
    "cedaceemos": "cedacear",
    "cedaceen": "cedacear",
    "cedacees": "cedacear",
    "cedaceo": "cedacear",
    "cedacera": "cedacero",
    "cedaceras": "cedacero",
    "cedaceros": "cedacero",
    "cedacerías": "cedacería",
    "cedaceábamos": "cedacear",
    "cedaceáis": "cedacear",
    "cedaceáramos": "cedacear",
    "cedaceáremos": "cedacear",
    "cedaceásemos": "cedacear",
    "cedaceé": "cedacear",
    "cedaceéis": "cedacear",
    "cedaceó": "cedacear",
    "cedamos": "ceder",
    "cedan": "ceder",
    "cedas": "ceder",
    "cedazos": "cedazo",
    "cede": "ceder",
    "ceded": "ceder",
    "cedemos": "ceder",
    "ceden": "ceder",
    "cedentes": "cedente",
    "cederemos": "ceder",
    "cederá": "ceder",
    "cederán": "ceder",
    "cederás": "ceder",
    "cederé": "ceder",
    "cederéis": "ceder",
    "cedería": "ceder",
    "cederíais": "ceder",
    "cederíamos": "ceder",
    "cederían": "ceder",
    "cederías": "ceder",
    "cedes": "ceder",
    "cedida": "ceder",
    "cedidas": "ceder",
    "cedido": "ceder",
    "cedidos": "ceder",
    "cediendo": "ceder",
    "cediera": "ceder",
    "cedierais": "ceder",
    "cedieran": "ceder",
    "cedieras": "ceder",
    "cediere": "ceder",
    "cediereis": "ceder",
    "cedieren": "ceder",
    "cedieres": "ceder",
    "cedieron": "ceder",
    "cediese": "ceder",
    "cedieseis": "ceder",
    "cediesen": "ceder",
    "cedieses": "ceder",
    "cedillas": "cedilla",
    "cedimos": "ceder",
    "cedis": "cedi",
    "cediste": "ceder",
    "cedisteis": "ceder",
    "cediza": "cedizo",
    "cedizas": "cedizo",
    "cedizos": "cedizo",
    "cediéramos": "ceder",
    "cediéremos": "ceder",
    "cediésemos": "ceder",
    "cedió": "ceder",
    "cedo": "ceder",
    "cedoarias": "cedoaria",
    "cedrenos": "cedreno",
    "cedritos": "cedrito",
    "cedrones": "cedrón",
    "cedros": "cedro",
    "cedularios": "cedulario",
    "cedulones": "cedulón",
    "cedáis": "ceder",
    "cedéis": "ceder",
    "cedés": "cedé",
    "cedí": "ceder",
    "cedía": "ceder",
    "cedíais": "ceder",
    "cedíamos": "ceder",
    "cedían": "ceder",
    "cedías": "ceder",
    "cefalalgias": "cefalalgia",
    "cefaleas": "cefalea",
    "cefalinas": "cefalina",
    "cefalocordados": "cefalocordado",
    "cefalogramas": "cefalograma",
    "cefalometrías": "cefalometría",
    "cefalométrica": "cefalométrico",
    "cefalométricas": "cefalométrico",
    "cefalométricos": "cefalométrico",
    "cefalorraquídea": "cefalorraquídeo",
    "cefalorraquídeas": "cefalorraquídeo",
    "cefalorraquídeos": "cefalorraquídeo",
    "cefalostatos": "cefalostato",
    "cefalotóraxes": "cefalotórax",
    "cefalómetros": "cefalómetro",
    "cefalópoda": "cefalópodo",
    "cefalópodas": "cefalópodo",
    "cefalópodos": "cefalópodo",
    "cefeidas": "cefeida",
    "cefálica": "cefálico",
    "cefálicas": "cefálico",
    "cefálicos": "cefálico",
    "cegaba": "cegar",
    "cegabais": "cegar",
    "cegaban": "cegar",
    "cegabas": "cegar",
    "cegad": "cegar",
    "cegada": "cegar",
    "cegadas": "cegar",
    "cegado": "cegar",
    "cegadora": "cegador",
    "cegadoras": "cegador",
    "cegadores": "cegador",
    "cegados": "cegar",
    "cegaja": "cegajo",
    "cegajas": "cegajo",
    "cegajos": "cegajo",
    "cegajosa": "cegajoso",
    "cegajosas": "cegajoso",
    "cegajosos": "cegajoso",
    "cegamos": "cegar",
    "cegando": "cegar",
    "cegara": "cegar",
    "cegarais": "cegar",
    "cegaran": "cegar",
    "cegaras": "cegar",
    "cegare": "cegar",
    "cegareis": "cegar",
    "cegaremos": "cegar",
    "cegaren": "cegar",
    "cegares": "cegar",
    "cegaron": "cegar",
    "cegarras": "cegarra",
    "cegará": "cegar",
    "cegarán": "cegar",
    "cegarás": "cegar",
    "cegaré": "cegar",
    "cegaréis": "cegar",
    "cegaría": "cegar",
    "cegaríais": "cegar",
    "cegaríamos": "cegar",
    "cegarían": "cegar",
    "cegarías": "cegar",
    "cegase": "cegar",
    "cegaseis": "cegar",
    "cegasen": "cegar",
    "cegases": "cegar",
    "cegaste": "cegar",
    "cegasteis": "cegar",
    "cegata": "cegato",
    "cegatas": "cegato",
    "cegatos": "cegato",
    "cegesimales": "cegesimal",
    "ceguedades": "ceguedad",
    "ceguemos": "cegar",
    "cegueras": "ceguera",
    "cegué": "cegar",
    "ceguéis": "cegar",
    "cegábamos": "cegar",
    "cegáis": "cegar",
    "cegáramos": "cegar",
    "cegáremos": "cegar",
    "cegásemos": "cegar",
    "cegó": "cegar",
    "ceibas": "ceiba",
    "ceilanesa": "ceilanés",
    "ceilanesas": "ceilanés",
    "ceilaneses": "ceilanés",
    "ceilanitas": "ceilanita",
    "ceja": "cejar",
    "cejaba": "cejar",
    "cejabais": "cejar",
    "cejaban": "cejar",
    "cejabas": "cejar",
    "cejad": "cejar",
    "cejada": "cejar",
    "cejadas": "cejar",
    "cejaderos": "cejadero",
    "cejado": "cejar",
    "cejadores": "cejador",
    "cejados": "cejar",
    "cejamos": "cejar",
    "cejan": "cejar",
    "cejando": "cejar",
    "cejara": "cejar",
    "cejarais": "cejar",
    "cejaran": "cejar",
    "cejaras": "cejar",
    "cejare": "cejar",
    "cejareis": "cejar",
    "cejaremos": "cejar",
    "cejaren": "cejar",
    "cejares": "cejar",
    "cejaron": "cejar",
    "cejará": "cejar",
    "cejarán": "cejar",
    "cejarás": "cejar",
    "cejaré": "cejar",
    "cejaréis": "cejar",
    "cejaría": "cejar",
    "cejaríais": "cejar",
    "cejaríamos": "cejar",
    "cejarían": "cejar",
    "cejarías": "cejar",
    "cejas": "cejar",
    "cejase": "cejar",
    "cejaseis": "cejar",
    "cejasen": "cejar",
    "cejases": "cejar",
    "cejaste": "cejar",
    "cejasteis": "cejar",
    "ceje": "cejar",
    "cejemos": "cejar",
    "cejen": "cejar",
    "cejes": "cejar",
    "cejijunta": "cejijunto",
    "cejijuntas": "cejijunto",
    "cejijuntos": "cejijunto",
    "cejillas": "cejilla",
    "cejo": "cejar",
    "cejos": "cejo",
    "cejuda": "cejudo",
    "cejudas": "cejudo",
    "cejudos": "cejudo",
    "cejuelas": "cejuela",
    "cejunta": "cejunto",
    "cejuntas": "cejunto",
    "cejuntos": "cejunto",
    "cejábamos": "cejar",
    "cejáis": "cejar",
    "cejáramos": "cejar",
    "cejáremos": "cejar",
    "cejásemos": "cejar",
    "cejé": "cejar",
    "cejéis": "cejar",
    "cejó": "cejar",
    "cela": "celar",
    "celaba": "celar",
    "celabais": "celar",
    "celaban": "celar",
    "celabas": "celar",
    "celad": "celar",
    "celada": "celar",
    "celadas": "celar",
    "celado": "celar",
    "celadora": "celador",
    "celadoras": "celador",
    "celadores": "celador",
    "celados": "celar",
    "celadurías": "celaduría",
    "celajerías": "celajería",
    "celajes": "celaje",
    "celamos": "celar",
    "celan": "celar",
    "celandesa": "celandés",
    "celandesas": "celandés",
    "celandeses": "celandés",
    "celando": "celar",
    "celantes": "celante",
    "celara": "celar",
    "celarais": "celar",
    "celaran": "celar",
    "celaras": "celar",
    "celare": "celar",
    "celareis": "celar",
    "celaremos": "celar",
    "celaren": "celar",
    "celares": "celar",
    "celaron": "celar",
    "celará": "celar",
    "celarán": "celar",
    "celarás": "celar",
    "celaré": "celar",
    "celaréis": "celar",
    "celaría": "celar",
    "celaríais": "celar",
    "celaríamos": "celar",
    "celarían": "celar",
    "celarías": "celar",
    "celas": "celar",
    "celase": "celar",
    "celaseis": "celar",
    "celasen": "celar",
    "celases": "celar",
    "celaste": "celar",
    "celasteis": "celar",
    "celastráceas": "celastrácea",
    "celdas": "celda",
    "celdillas": "celdilla",
    "cele": "celar",
    "celebra": "celebrar",
    "celebraba": "celebrar",
    "celebrabais": "celebrar",
    "celebraban": "celebrar",
    "celebrabas": "celebrar",
    "celebraciones": "celebración",
    "celebrad": "celebrar",
    "celebrada": "celebrar",
    "celebradas": "celebrar",
    "celebrado": "celebrar",
    "celebrados": "celebrar",
    "celebramos": "celebrar",
    "celebran": "celebrar",
    "celebrando": "celebrar",
    "celebrantes": "celebrante",
    "celebrara": "celebrar",
    "celebrarais": "celebrar",
    "celebraran": "celebrar",
    "celebraras": "celebrar",
    "celebrare": "celebrar",
    "celebrareis": "celebrar",
    "celebraremos": "celebrar",
    "celebraren": "celebrar",
    "celebrares": "celebrar",
    "celebraron": "celebrar",
    "celebrará": "celebrar",
    "celebrarán": "celebrar",
    "celebrarás": "celebrar",
    "celebraré": "celebrar",
    "celebraréis": "celebrar",
    "celebraría": "celebrar",
    "celebraríais": "celebrar",
    "celebraríamos": "celebrar",
    "celebrarían": "celebrar",
    "celebrarías": "celebrar",
    "celebras": "celebrar",
    "celebrase": "celebrar",
    "celebraseis": "celebrar",
    "celebrasen": "celebrar",
    "celebrases": "celebrar",
    "celebraste": "celebrar",
    "celebrasteis": "celebrar",
    "celebre": "celebrar",
    "celebremos": "celebrar",
    "celebren": "celebrar",
    "celebres": "celebrar",
    "celebridades": "celebridad",
    "celebro": "celebrar",
    "celebrábamos": "celebrar",
    "celebráis": "celebrar",
    "celebráramos": "celebrar",
    "celebráremos": "celebrar",
    "celebrásemos": "celebrar",
    "celebré": "celebrar",
    "celebréis": "celebrar",
    "celebró": "celebrar",
    "celeminadas": "celeminada",
    "celemines": "celemín",
    "celemos": "celar",
    "celen": "celar",
    "celentéreos": "celentéreo",
    "celeras": "celera",
    "celeridades": "celeridad",
    "celes": "celar",
    "celestas": "celesta",
    "celestes": "celeste",
    "celestiales": "celestial",
    "celestina": "celestino",
    "celestinas": "celestino",
    "celestinesca": "celestinesco",
    "celestinescas": "celestinesco",
    "celestinescos": "celestinesco",
    "celestinos": "celestino",
    "celibatos": "celibato",
    "celidonias": "celidonia",
    "celiprololes": "celiprolol",
    "cellas": "cella",
    "cellenca": "cellenco",
    "cellencas": "cellenco",
    "cellencos": "cellenco",
    "celliscas": "cellisca",
    "cellisquea": "cellisquear",
    "cellisqueaba": "cellisquear",
    "cellisqueabais": "cellisquear",
    "cellisqueaban": "cellisquear",
    "cellisqueabas": "cellisquear",
    "cellisquead": "cellisquear",
    "cellisqueada": "cellisquear",
    "cellisqueadas": "cellisquear",
    "cellisqueado": "cellisquear",
    "cellisqueados": "cellisquear",
    "cellisqueamos": "cellisquear",
    "cellisquean": "cellisquear",
    "cellisqueando": "cellisquear",
    "cellisqueara": "cellisquear",
    "cellisquearais": "cellisquear",
    "cellisquearan": "cellisquear",
    "cellisquearas": "cellisquear",
    "cellisqueare": "cellisquear",
    "cellisqueareis": "cellisquear",
    "cellisquearemos": "cellisquear",
    "cellisquearen": "cellisquear",
    "cellisqueares": "cellisquear",
    "cellisquearon": "cellisquear",
    "cellisqueará": "cellisquear",
    "cellisquearán": "cellisquear",
    "cellisquearás": "cellisquear",
    "cellisquearé": "cellisquear",
    "cellisquearéis": "cellisquear",
    "cellisquearía": "cellisquear",
    "cellisquearíais": "cellisquear",
    "cellisquearíamos": "cellisquear",
    "cellisquearían": "cellisquear",
    "cellisquearías": "cellisquear",
    "cellisqueas": "cellisquear",
    "cellisquease": "cellisquear",
    "cellisqueaseis": "cellisquear",
    "cellisqueasen": "cellisquear",
    "cellisqueases": "cellisquear",
    "cellisqueaste": "cellisquear",
    "cellisqueasteis": "cellisquear",
    "cellisquee": "cellisquear",
    "cellisqueemos": "cellisquear",
    "cellisqueen": "cellisquear",
    "cellisquees": "cellisquear",
    "cellisqueo": "cellisquear",
    "cellisqueábamos": "cellisquear",
    "cellisqueáis": "cellisquear",
    "cellisqueáramos": "cellisquear",
    "cellisqueáremos": "cellisquear",
    "cellisqueásemos": "cellisquear",
    "cellisqueé": "cellisquear",
    "cellisqueéis": "cellisquear",
    "cellisqueó": "cellisquear",
    "cellos": "cello",
    "celo": "celar",
    "celofanes": "celofán",
    "celoidinas": "celoidina",
    "celomas": "celoma",
    "celos": "celo",
    "celosa": "celoso",
    "celosas": "celoso",
    "celosos": "celoso",
    "celosías": "celosía",
    "celotes": "celote",
    "celotipias": "celotipia",
    "celsios": "celsio",
    "celsitudes": "celsitud",
    "celtas": "celta",
    "celtibera": "celtíbero",
    "celtiberas": "celtíbero",
    "celtiberos": "celtíbero",
    "celtibérica": "celtibérico",
    "celtibéricas": "celtibérico",
    "celtibéricos": "celtibérico",
    "celtismos": "celtismo",
    "celtistas": "celtista",
    "celtohispánica": "celtohispánico",
    "celtohispánicas": "celtohispánico",
    "celtohispánicos": "celtohispánico",
    "celtolatina": "celtolatino",
    "celtolatinas": "celtolatino",
    "celtolatinos": "celtolatino",
    "celtíbera": "celtíbero",
    "celtíberas": "celtíbero",
    "celtíberos": "celtíbero",
    "celulares": "celular",
    "celulasas": "celulasa",
    "celuloides": "celuloide",
    "celulosas": "celulosa",
    "celulósica": "celulósico",
    "celulósicas": "celulósico",
    "celulósicos": "celulósico",
    "celábamos": "celar",
    "celáis": "celar",
    "celáramos": "celar",
    "celáremos": "celar",
    "celásemos": "celar",
    "celé": "celar",
    "celéis": "celar",
    "celíaca": "celíaco",
    "celíacas": "celíaco",
    "celíacos": "celíaco",
    "celó": "celar",
    "cementa": "cementar",
    "cementaba": "cementar",
    "cementabais": "cementar",
    "cementaban": "cementar",
    "cementabas": "cementar",
    "cementaciones": "cementación",
    "cementad": "cementar",
    "cementada": "cementar",
    "cementadas": "cementar",
    "cementado": "cementar",
    "cementados": "cementar",
    "cementamos": "cementar",
    "cementan": "cementar",
    "cementando": "cementar",
    "cementara": "cementar",
    "cementarais": "cementar",
    "cementaran": "cementar",
    "cementaras": "cementar",
    "cementare": "cementar",
    "cementareis": "cementar",
    "cementaremos": "cementar",
    "cementaren": "cementar",
    "cementares": "cementar",
    "cementaron": "cementar",
    "cementará": "cementar",
    "cementarán": "cementar",
    "cementarás": "cementar",
    "cementaré": "cementar",
    "cementaréis": "cementar",
    "cementaría": "cementar",
    "cementaríais": "cementar",
    "cementaríamos": "cementar",
    "cementarían": "cementar",
    "cementarías": "cementar",
    "cementas": "cementar",
    "cementase": "cementar",
    "cementaseis": "cementar",
    "cementasen": "cementar",
    "cementases": "cementar",
    "cementaste": "cementar",
    "cementasteis": "cementar",
    "cemente": "cementar",
    "cementemos": "cementar",
    "cementen": "cementar",
    "cementeras": "cementera",
    "cementeriales": "cementerial",
    "cementerios": "cementerio",
    "cementes": "cementar",
    "cementicles": "cementicle",
    "cementitas": "cementita",
    "cemento": "cementar",
    "cementoadamantina": "cementoadamantino",
    "cementoadamantinas": "cementoadamantino",
    "cementoadamantinos": "cementoadamantino",
    "cementoblastomas": "cementoblastoma",
    "cementoblastos": "cementoblasto",
    "cementoclasias": "cementoclasia",
    "cementoclastos": "cementoclasto",
    "cementodentinales": "cementodentinal",
    "cementoides": "cementoide",
    "cementomas": "cementoma",
    "cementopatías": "cementopatía",
    "cementos": "cemento",
    "cementosa": "cementoso",
    "cementosas": "cementoso",
    "cementosos": "cementoso",
    "cementábamos": "cementar",
    "cementáis": "cementar",
    "cementáramos": "cementar",
    "cementáremos": "cementar",
    "cementásemos": "cementar",
    "cementé": "cementar",
    "cementéis": "cementar",
    "cementículos": "cementículo",
    "cementó": "cementar",
    "cementógena": "cementógeno",
    "cementógenas": "cementógeno",
    "cementógenos": "cementógeno",
    "cena": "cenar",
    "cenaba": "cenar",
    "cenabais": "cenar",
    "cenaban": "cenar",
    "cenabas": "cenar",
    "cenachos": "cenacho",
    "cenad": "cenar",
    "cenada": "cenar",
    "cenadas": "cenar",
    "cenado": "cenar",
    "cenadora": "cenador",
    "cenadoras": "cenador",
    "cenadores": "cenador",
    "cenados": "cenar",
    "cenagales": "cenagal",
    "cenagosa": "cenagoso",
    "cenagosas": "cenagoso",
    "cenagosos": "cenagoso",
    "cenamos": "cenar",
    "cenan": "cenar",
    "cenando": "cenar",
    "cenara": "cenar",
    "cenarais": "cenar",
    "cenaran": "cenar",
    "cenaras": "cenar",
    "cenare": "cenar",
    "cenareis": "cenar",
    "cenaremos": "cenar",
    "cenaren": "cenar",
    "cenares": "cenar",
    "cenaron": "cenar",
    "cenará": "cenar",
    "cenarán": "cenar",
    "cenarás": "cenar",
    "cenaré": "cenar",
    "cenaréis": "cenar",
    "cenaría": "cenar",
    "cenaríais": "cenar",
    "cenaríamos": "cenar",
    "cenarían": "cenar",
    "cenarías": "cenar",
    "cenas": "cenar",
    "cenase": "cenar",
    "cenaseis": "cenar",
    "cenasen": "cenar",
    "cenases": "cenar",
    "cenaste": "cenar",
    "cenasteis": "cenar",
    "cencelladas": "cencellada",
    "cencerradas": "cencerrada",
    "cencerras": "cencerra",
    "cencerrea": "cencerrear",
    "cencerreaba": "cencerrear",
    "cencerreabais": "cencerrear",
    "cencerreaban": "cencerrear",
    "cencerreabas": "cencerrear",
    "cencerread": "cencerrear",
    "cencerreada": "cencerrear",
    "cencerreadas": "cencerrear",
    "cencerreado": "cencerrear",
    "cencerreados": "cencerrear",
    "cencerreamos": "cencerrear",
    "cencerrean": "cencerrear",
    "cencerreando": "cencerrear",
    "cencerreara": "cencerrear",
    "cencerrearais": "cencerrear",
    "cencerrearan": "cencerrear",
    "cencerrearas": "cencerrear",
    "cencerreare": "cencerrear",
    "cencerreareis": "cencerrear",
    "cencerrearemos": "cencerrear",
    "cencerrearen": "cencerrear",
    "cencerreares": "cencerrear",
    "cencerrearon": "cencerrear",
    "cencerreará": "cencerrear",
    "cencerrearán": "cencerrear",
    "cencerrearás": "cencerrear",
    "cencerrearé": "cencerrear",
    "cencerrearéis": "cencerrear",
    "cencerrearía": "cencerrear",
    "cencerrearíais": "cencerrear",
    "cencerrearíamos": "cencerrear",
    "cencerrearían": "cencerrear",
    "cencerrearías": "cencerrear",
    "cencerreas": "cencerrear",
    "cencerrease": "cencerrear",
    "cencerreaseis": "cencerrear",
    "cencerreasen": "cencerrear",
    "cencerreases": "cencerrear",
    "cencerreaste": "cencerrear",
    "cencerreasteis": "cencerrear",
    "cencerree": "cencerrear",
    "cencerreemos": "cencerrear",
    "cencerreen": "cencerrear",
    "cencerrees": "cencerrear",
    "cencerreo": "cencerrear",
    "cencerreos": "cencerreo",
    "cencerreábamos": "cencerrear",
    "cencerreáis": "cencerrear",
    "cencerreáramos": "cencerrear",
    "cencerreáremos": "cencerrear",
    "cencerreásemos": "cencerrear",
    "cencerreé": "cencerrear",
    "cencerreéis": "cencerrear",
    "cencerreó": "cencerrear",
    "cencerrones": "cencerrón",
    "cencerros": "cencerro",
    "cenceña": "cenceño",
    "cenceñas": "cenceño",
    "cenceños": "cenceño",
    "cenchas": "cencha",
    "cencida": "cencido",
    "cencidas": "cencido",
    "cencidos": "cencido",
    "cencoces": "cencoz",
    "cendales": "cendal",
    "cendalíes": "cendalí",
    "cendalís": "cendalí",
    "cendra": "cendrar",
    "cendraba": "cendrar",
    "cendrabais": "cendrar",
    "cendraban": "cendrar",
    "cendrabas": "cendrar",
    "cendrad": "cendrar",
    "cendrada": "cendrar",
    "cendradas": "cendrar",
    "cendradillas": "cendradilla",
    "cendrado": "cendrar",
    "cendrados": "cendrar",
    "cendramos": "cendrar",
    "cendran": "cendrar",
    "cendrando": "cendrar",
    "cendrara": "cendrar",
    "cendrarais": "cendrar",
    "cendraran": "cendrar",
    "cendraras": "cendrar",
    "cendrare": "cendrar",
    "cendrareis": "cendrar",
    "cendraremos": "cendrar",
    "cendraren": "cendrar",
    "cendrares": "cendrar",
    "cendraron": "cendrar",
    "cendrará": "cendrar",
    "cendrarán": "cendrar",
    "cendrarás": "cendrar",
    "cendraré": "cendrar",
    "cendraréis": "cendrar",
    "cendraría": "cendrar",
    "cendraríais": "cendrar",
    "cendraríamos": "cendrar",
    "cendrarían": "cendrar",
    "cendrarías": "cendrar",
    "cendras": "cendrar",
    "cendrase": "cendrar",
    "cendraseis": "cendrar",
    "cendrasen": "cendrar",
    "cendrases": "cendrar",
    "cendraste": "cendrar",
    "cendrasteis": "cendrar",
    "cendrazos": "cendrazo",
    "cendre": "cendrar",
    "cendremos": "cendrar",
    "cendren": "cendrar",
    "cendres": "cendrar",
    "cendro": "cendrar",
    "cendrábamos": "cendrar",
    "cendráis": "cendrar",
    "cendráramos": "cendrar",
    "cendráremos": "cendrar",
    "cendrásemos": "cendrar",
    "cendré": "cendrar",
    "cendréis": "cendrar",
    "cendró": "cendrar",
    "cene": "cenar",
    "cenefas": "cenefa",
    "cenemas": "cenema",
    "cenematemas": "cenematema",
    "cenemos": "cenar",
    "cenemáticas": "cenemática",
    "cenen": "cenar",
    "cenerentolas": "cenerentola",
    "cenes": "cenar",
    "cenestesias": "cenestesia",
    "cenestopatías": "cenestopatía",
    "cenestésica": "cenestésico",
    "cenestésicas": "cenestésico",
    "cenestésicos": "cenestésico",
    "ceniceros": "cenicero",
    "cenicienta": "ceniciento",
    "cenicientas": "ceniciento",
    "cenicientos": "ceniciento",
    "cenicillas": "cenicilla",
    "cenitales": "cenital",
    "cenits": "cenit",
    "ceniza": "cenizo",
    "cenizas": "cenizo",
    "cenizos": "cenizo",
    "cenizosa": "cenizoso",
    "cenizosas": "cenizoso",
    "cenizosos": "cenizoso",
    "ceno": "cenar",
    "cenobiales": "cenobial",
    "cenobios": "cenobio",
    "cenobitas": "cenobita",
    "cenobitismos": "cenobitismo",
    "cenobítica": "cenobítico",
    "cenobíticas": "cenobítico",
    "cenobíticos": "cenobítico",
    "cenocarpos": "cenocarpo",
    "cenocitos": "cenocito",
    "cenocárpica": "cenocárpico",
    "cenocárpicas": "cenocárpico",
    "cenocárpicos": "cenocárpico",
    "cenogametos": "cenogameto",
    "cenojiles": "cenojil",
    "cenologías": "cenología",
    "cenotafios": "cenotafio",
    "cenozoicos": "cenozoico",
    "censa": "censar",
    "censaba": "censar",
    "censabais": "censar",
    "censaban": "censar",
    "censabas": "censar",
    "censad": "censar",
    "censada": "censar",
    "censadas": "censar",
    "censado": "censar",
    "censados": "censar",
    "censales": "censal",
    "censalistas": "censalista",
    "censamos": "censar",
    "censan": "censar",
    "censando": "censar",
    "censara": "censar",
    "censarais": "censar",
    "censaran": "censar",
    "censaras": "censar",
    "censare": "censar",
    "censareis": "censar",
    "censaremos": "censar",
    "censaren": "censar",
    "censares": "censar",
    "censaron": "censar",
    "censará": "censar",
    "censarán": "censar",
    "censarás": "censar",
    "censaré": "censar",
    "censaréis": "censar",
    "censaría": "censar",
    "censaríais": "censar",
    "censaríamos": "censar",
    "censarían": "censar",
    "censarías": "censar",
    "censas": "censar",
    "censase": "censar",
    "censaseis": "censar",
    "censasen": "censar",
    "censases": "censar",
    "censaste": "censar",
    "censasteis": "censar",
    "censataria": "censatario",
    "censatarias": "censatario",
    "censatarios": "censatario",
    "cense": "censar",
    "censemos": "censar",
    "censen": "censar",
    "censes": "censar",
    "censistas": "censista",
    "censo": "censar",
    "censora": "censor",
    "censoras": "censor",
    "censores": "censor",
    "censoria": "censorio",
    "censorias": "censorio",
    "censorios": "censorio",
    "censos": "censo",
    "censuales": "censual",
    "censualistas": "censualista",
    "censura": "censurar",
    "censuraba": "censurar",
    "censurabais": "censurar",
    "censuraban": "censurar",
    "censurabas": "censurar",
    "censurables": "censurable",
    "censurad": "censurar",
    "censurada": "censurar",
    "censuradas": "censurar",
    "censurado": "censurar",
    "censuradora": "censurador",
    "censuradoras": "censurador",
    "censuradores": "censurador",
    "censurados": "censurar",
    "censuramos": "censurar",
    "censuran": "censurar",
    "censurando": "censurar",
    "censurara": "censurar",
    "censurarais": "censurar",
    "censuraran": "censurar",
    "censuraras": "censurar",
    "censurare": "censurar",
    "censurareis": "censurar",
    "censuraremos": "censurar",
    "censuraren": "censurar",
    "censurares": "censurar",
    "censuraron": "censurar",
    "censurará": "censurar",
    "censurarán": "censurar",
    "censurarás": "censurar",
    "censuraré": "censurar",
    "censuraréis": "censurar",
    "censuraría": "censurar",
    "censuraríais": "censurar",
    "censuraríamos": "censurar",
    "censurarían": "censurar",
    "censurarías": "censurar",
    "censuras": "censurar",
    "censurase": "censurar",
    "censuraseis": "censurar",
    "censurasen": "censurar",
    "censurases": "censurar",
    "censuraste": "censurar",
    "censurasteis": "censurar",
    "censure": "censurar",
    "censuremos": "censurar",
    "censuren": "censurar",
    "censures": "censurar",
    "censuristaa": "censurista",
    "censuristaas": "censurista",
    "censuristas": "censurista",
    "censuro": "censurar",
    "censurábamos": "censurar",
    "censuráis": "censurar",
    "censuráramos": "censurar",
    "censuráremos": "censurar",
    "censurásemos": "censurar",
    "censuré": "censurar",
    "censuréis": "censurar",
    "censuró": "censurar",
    "censábamos": "censar",
    "censáis": "censar",
    "censáramos": "censar",
    "censáremos": "censar",
    "censásemos": "censar",
    "censé": "censar",
    "censéis": "censar",
    "censó": "censar",
    "centauras": "centaura",
    "centauromaquias": "centauromaquia",
    "centauros": "centauro",
    "centava": "centavo",
    "centavas": "centavo",
    "centavos": "centavo",
    "centella": "centellar",
    "centellaba": "centellar",
    "centellabais": "centellar",
    "centellaban": "centellar",
    "centellabas": "centellar",
    "centellad": "centellar",
    "centellada": "centellar",
    "centelladas": "centellar",
    "centellado": "centellar",
    "centelladora": "centellador",
    "centelladoras": "centellador",
    "centelladores": "centellador",
    "centellados": "centellar",
    "centellamos": "centellar",
    "centellan": "centellar",
    "centellando": "centellar",
    "centellara": "centellar",
    "centellarais": "centellar",
    "centellaran": "centellar",
    "centellaras": "centellar",
    "centellare": "centellar",
    "centellareis": "centellar",
    "centellaremos": "centellar",
    "centellaren": "centellar",
    "centellares": "centellar",
    "centellaron": "centellar",
    "centellará": "centellar",
    "centellarán": "centellar",
    "centellarás": "centellar",
    "centellaré": "centellar",
    "centellaréis": "centellar",
    "centellaría": "centellar",
    "centellaríais": "centellar",
    "centellaríamos": "centellar",
    "centellarían": "centellar",
    "centellarías": "centellar",
    "centellas": "centellar",
    "centellase": "centellar",
    "centellaseis": "centellar",
    "centellasen": "centellar",
    "centellases": "centellar",
    "centellaste": "centellar",
    "centellasteis": "centellar",
    "centelle": "centellar",
    "centellea": "centellear",
    "centelleaba": "centellear",
    "centelleabais": "centellear",
    "centelleaban": "centellear",
    "centelleabas": "centellear",
    "centellead": "centellear",
    "centelleada": "centellear",
    "centelleadas": "centellear",
    "centelleado": "centellear",
    "centelleados": "centellear",
    "centelleamos": "centellear",
    "centellean": "centellear",
    "centelleando": "centellear",
    "centelleantes": "centelleante",
    "centelleara": "centellear",
    "centellearais": "centellear",
    "centellearan": "centellear",
    "centellearas": "centellear",
    "centelleare": "centellear",
    "centelleareis": "centellear",
    "centellearemos": "centellear",
    "centellearen": "centellear",
    "centelleares": "centellear",
    "centellearon": "centellear",
    "centelleará": "centellear",
    "centellearán": "centellear",
    "centellearás": "centellear",
    "centellearé": "centellear",
    "centellearéis": "centellear",
    "centellearía": "centellear",
    "centellearíais": "centellear",
    "centellearíamos": "centellear",
    "centellearían": "centellear",
    "centellearías": "centellear",
    "centelleas": "centellear",
    "centellease": "centellear",
    "centelleaseis": "centellear",
    "centelleasen": "centellear",
    "centelleases": "centellear",
    "centelleaste": "centellear",
    "centelleasteis": "centellear",
    "centellee": "centellear",
    "centelleemos": "centellear",
    "centelleen": "centellear",
    "centellees": "centellear",
    "centellemos": "centellar",
    "centellen": "centellar",
    "centelleo": "centellear",
    "centelleos": "centelleo",
    "centelles": "centellar",
    "centelleábamos": "centellear",
    "centelleáis": "centellear",
    "centelleáramos": "centellear",
    "centelleáremos": "centellear",
    "centelleásemos": "centellear",
    "centelleé": "centellear",
    "centelleéis": "centellear",
    "centelleó": "centellear",
    "centello": "centellar",
    "centellábamos": "centellar",
    "centelláis": "centellar",
    "centelláramos": "centellar",
    "centelláremos": "centellar",
    "centellásemos": "centellar",
    "centellé": "centellar",
    "centelléis": "centellar",
    "centelló": "centellar",
    "centenadas": "centenada",
    "centenales": "centenal",
    "centenares": "centenar",
    "centenaria": "centenario",
    "centenarias": "centenario",
    "centenarios": "centenario",
    "centenas": "centena",
    "centenera": "centenero",
    "centeneras": "centenero",
    "centeneros": "centenero",
    "centenos": "centeno",
    "centenosa": "centenoso",
    "centenosas": "centenoso",
    "centenosos": "centenoso",
    "centesimales": "centesimal",
    "centigramos": "centigramo",
    "centilas": "centila",
    "centilitros": "centilitro",
    "centilleros": "centillero",
    "centiloquios": "centiloquio",
    "centimana": "centímano",
    "centimanas": "centímano",
    "centimanos": "centímano",
    "centinelas": "centinela",
    "centinodias": "centinodia",
    "centiplicada": "centiplicado",
    "centiplicadas": "centiplicado",
    "centiplicados": "centiplicado",
    "centipondios": "centipondio",
    "centiáreas": "centiárea",
    "centollas": "centolla",
    "centollos": "centollo",
    "centona": "centonar",
    "centonaba": "centonar",
    "centonabais": "centonar",
    "centonaban": "centonar",
    "centonabas": "centonar",
    "centonad": "centonar",
    "centonada": "centonar",
    "centonadas": "centonar",
    "centonado": "centonar",
    "centonados": "centonar",
    "centonamos": "centonar",
    "centonan": "centonar",
    "centonando": "centonar",
    "centonara": "centonar",
    "centonarais": "centonar",
    "centonaran": "centonar",
    "centonaras": "centonar",
    "centonare": "centonar",
    "centonareis": "centonar",
    "centonaremos": "centonar",
    "centonaren": "centonar",
    "centonares": "centonar",
    "centonaron": "centonar",
    "centonará": "centonar",
    "centonarán": "centonar",
    "centonarás": "centonar",
    "centonaré": "centonar",
    "centonaréis": "centonar",
    "centonaría": "centonar",
    "centonaríais": "centonar",
    "centonaríamos": "centonar",
    "centonarían": "centonar",
    "centonarías": "centonar",
    "centonas": "centonar",
    "centonase": "centonar",
    "centonaseis": "centonar",
    "centonasen": "centonar",
    "centonases": "centonar",
    "centonaste": "centonar",
    "centonasteis": "centonar",
    "centone": "centonar",
    "centonemos": "centonar",
    "centonen": "centonar",
    "centones": "centonar",
    "centono": "centonar",
    "centonábamos": "centonar",
    "centonáis": "centonar",
    "centonáramos": "centonar",
    "centonáremos": "centonar",
    "centonásemos": "centonar",
    "centoné": "centonar",
    "centonéis": "centonar",
    "centonó": "centonar",
    "centra": "centrar",
    "centraba": "centrar",
    "centrabais": "centrar",
    "centraban": "centrar",
    "centrabas": "centrar",
    "centrad": "centrar",
    "centrada": "centrar",
    "centradas": "centrar",
    "centrado": "centrar",
    "centradores": "centrador",
    "centrados": "centrar",
    "centrajes": "centraje",
    "centrales": "central",
    "centralice": "centralizar",
    "centralicemos": "centralizar",
    "centralicen": "centralizar",
    "centralices": "centralizar",
    "centralicé": "centralizar",
    "centralicéis": "centralizar",
    "centralidades": "centralidad",
    "centralismos": "centralismo",
    "centralistas": "centralista",
    "centralitas": "centralita",
    "centraliza": "centralizar",
    "centralizaba": "centralizar",
    "centralizabais": "centralizar",
    "centralizaban": "centralizar",
    "centralizabas": "centralizar",
    "centralizaciones": "centralización",
    "centralizad": "centralizar",
    "centralizada": "centralizar",
    "centralizadas": "centralizar",
    "centralizado": "centralizar",
    "centralizadora": "centralizador",
    "centralizadoras": "centralizador",
    "centralizadores": "centralizador",
    "centralizados": "centralizar",
    "centralizamos": "centralizar",
    "centralizan": "centralizar",
    "centralizando": "centralizar",
    "centralizara": "centralizar",
    "centralizarais": "centralizar",
    "centralizaran": "centralizar",
    "centralizaras": "centralizar",
    "centralizare": "centralizar",
    "centralizareis": "centralizar",
    "centralizaremos": "centralizar",
    "centralizaren": "centralizar",
    "centralizares": "centralizar",
    "centralizaron": "centralizar",
    "centralizará": "centralizar",
    "centralizarán": "centralizar",
    "centralizarás": "centralizar",
    "centralizaré": "centralizar",
    "centralizaréis": "centralizar",
    "centralizaría": "centralizar",
    "centralizaríais": "centralizar",
    "centralizaríamos": "centralizar",
    "centralizarían": "centralizar",
    "centralizarías": "centralizar",
    "centralizas": "centralizar",
    "centralizase": "centralizar",
    "centralizaseis": "centralizar",
    "centralizasen": "centralizar",
    "centralizases": "centralizar",
    "centralizaste": "centralizar",
    "centralizasteis": "centralizar",
    "centralizo": "centralizar",
    "centralizábamos": "centralizar",
    "centralizáis": "centralizar",
    "centralizáramos": "centralizar",
    "centralizáremos": "centralizar",
    "centralizásemos": "centralizar",
    "centralizó": "centralizar",
    "centramos": "centrar",
    "centran": "centrar",
    "centrando": "centrar",
    "centrara": "centrar",
    "centrarais": "centrar",
    "centraran": "centrar",
    "centraras": "centrar",
    "centrare": "centrar",
    "centrareis": "centrar",
    "centraremos": "centrar",
    "centraren": "centrar",
    "centrares": "centrar",
    "centraron": "centrar",
    "centrará": "centrar",
    "centrarán": "centrar",
    "centrarás": "centrar",
    "centraré": "centrar",
    "centraréis": "centrar",
    "centraría": "centrar",
    "centraríais": "centrar",
    "centraríamos": "centrar",
    "centrarían": "centrar",
    "centrarías": "centrar",
    "centras": "centrar",
    "centrase": "centrar",
    "centraseis": "centrar",
    "centrasen": "centrar",
    "centrases": "centrar",
    "centraste": "centrar",
    "centrasteis": "centrar",
    "centre": "centrar",
    "centremos": "centrar",
    "centren": "centrar",
    "centres": "centrar",
    "centrifuga": "centrifugar",
    "centrifugaba": "centrifugar",
    "centrifugabais": "centrifugar",
    "centrifugaban": "centrifugar",
    "centrifugabas": "centrifugar",
    "centrifugaciones": "centrifugación",
    "centrifugad": "centrifugar",
    "centrifugada": "centrifugar",
    "centrifugadas": "centrifugar",
    "centrifugado": "centrifugar",
    "centrifugadora": "centrifugador",
    "centrifugadoras": "centrifugadora",
    "centrifugadores": "centrifugador",
    "centrifugados": "centrifugar",
    "centrifugamos": "centrifugar",
    "centrifugan": "centrifugar",
    "centrifugando": "centrifugar",
    "centrifugara": "centrifugar",
    "centrifugarais": "centrifugar",
    "centrifugaran": "centrifugar",
    "centrifugaras": "centrifugar",
    "centrifugare": "centrifugar",
    "centrifugareis": "centrifugar",
    "centrifugaremos": "centrifugar",
    "centrifugaren": "centrifugar",
    "centrifugares": "centrifugar",
    "centrifugaron": "centrifugar",
    "centrifugará": "centrifugar",
    "centrifugarán": "centrifugar",
    "centrifugarás": "centrifugar",
    "centrifugaré": "centrifugar",
    "centrifugaréis": "centrifugar",
    "centrifugaría": "centrifugar",
    "centrifugaríais": "centrifugar",
    "centrifugaríamos": "centrifugar",
    "centrifugarían": "centrifugar",
    "centrifugarías": "centrifugar",
    "centrifugas": "centrifugar",
    "centrifugase": "centrifugar",
    "centrifugaseis": "centrifugar",
    "centrifugasen": "centrifugar",
    "centrifugases": "centrifugar",
    "centrifugaste": "centrifugar",
    "centrifugasteis": "centrifugar",
    "centrifugo": "centrifugar",
    "centrifugue": "centrifugar",
    "centrifuguemos": "centrifugar",
    "centrifuguen": "centrifugar",
    "centrifugues": "centrifugar",
    "centrifugué": "centrifugar",
    "centrifuguéis": "centrifugar",
    "centrifugábamos": "centrifugar",
    "centrifugáis": "centrifugar",
    "centrifugáramos": "centrifugar",
    "centrifugáremos": "centrifugar",
    "centrifugásemos": "centrifugar",
    "centrifugó": "centrifugar",
    "centriolos": "centriolo",
    "centristas": "centrista",
    "centro": "centrar",
    "centroafricana": "centroafricano",
    "centroafricanas": "centroafricano",
    "centroafricanos": "centroafricano",
    "centroamericana": "centroamericano",
    "centroamericanas": "centroamericano",
    "centroamericanos": "centroamericano",
    "centroasiática": "centroasiático",
    "centroasiáticas": "centroasiático",
    "centroasiáticos": "centroasiático",
    "centroblastos": "centroblasto",
    "centrobárica": "centrobárico",
    "centrobáricas": "centrobárico",
    "centrobáricos": "centrobárico",
    "centrocampistas": "centrocampista",
    "centrocitos": "centrocito",
    "centrocromatinas": "centrocromatina",
    "centroeuropea": "centroeuropeo",
    "centroeuropeas": "centroeuropeo",
    "centroeuropeos": "centroeuropeo",
    "centroidales": "centroidal",
    "centroides": "centroide",
    "centroizquierdas": "centroizquierda",
    "centrooceánica": "centrooceánico",
    "centrooceánicas": "centrooceánico",
    "centrooceánicos": "centrooceánico",
    "centros": "centro",
    "centrosferas": "centrosfera",
    "centrosomas": "centrosoma",
    "centrospermas": "centrosperma",
    "centrábamos": "centrar",
    "centráis": "centrar",
    "centráramos": "centrar",
    "centráremos": "centrar",
    "centrásemos": "centrar",
    "centré": "centrar",
    "centréis": "centrar",
    "centrífuga": "centrífugo",
    "centrífugas": "centrífugo",
    "centrífugos": "centrífugo",
    "centrípeta": "centrípeto",
    "centrípetas": "centrípeto",
    "centrípetos": "centrípeto",
    "centró": "centrar",
    "centrómeros": "centrómero",
    "centunviratos": "centunvirato",
    "centunviros": "centunviro",
    "centuplica": "centuplicar",
    "centuplicaba": "centuplicar",
    "centuplicabais": "centuplicar",
    "centuplicaban": "centuplicar",
    "centuplicabas": "centuplicar",
    "centuplicad": "centuplicar",
    "centuplicada": "centuplicar",
    "centuplicadas": "centuplicar",
    "centuplicado": "centuplicar",
    "centuplicados": "centuplicar",
    "centuplicamos": "centuplicar",
    "centuplican": "centuplicar",
    "centuplicando": "centuplicar",
    "centuplicara": "centuplicar",
    "centuplicarais": "centuplicar",
    "centuplicaran": "centuplicar",
    "centuplicaras": "centuplicar",
    "centuplicare": "centuplicar",
    "centuplicareis": "centuplicar",
    "centuplicaremos": "centuplicar",
    "centuplicaren": "centuplicar",
    "centuplicares": "centuplicar",
    "centuplicaron": "centuplicar",
    "centuplicará": "centuplicar",
    "centuplicarán": "centuplicar",
    "centuplicarás": "centuplicar",
    "centuplicaré": "centuplicar",
    "centuplicaréis": "centuplicar",
    "centuplicaría": "centuplicar",
    "centuplicaríais": "centuplicar",
    "centuplicaríamos": "centuplicar",
    "centuplicarían": "centuplicar",
    "centuplicarías": "centuplicar",
    "centuplicas": "centuplicar",
    "centuplicase": "centuplicar",
    "centuplicaseis": "centuplicar",
    "centuplicasen": "centuplicar",
    "centuplicases": "centuplicar",
    "centuplicaste": "centuplicar",
    "centuplicasteis": "centuplicar",
    "centuplico": "centuplicar",
    "centuplicábamos": "centuplicar",
    "centuplicáis": "centuplicar",
    "centuplicáramos": "centuplicar",
    "centuplicáremos": "centuplicar",
    "centuplicásemos": "centuplicar",
    "centuplicó": "centuplicar",
    "centuplique": "centuplicar",
    "centupliquemos": "centuplicar",
    "centupliquen": "centuplicar",
    "centupliques": "centuplicar",
    "centupliqué": "centuplicar",
    "centupliquéis": "centuplicar",
    "centurias": "centuria",
    "centuriona": "centurión",
    "centurionas": "centurión",
    "centuriones": "centurión",
    "centésima": "100",
    "centésimas": "centésima",
    "centésimo": "100",
    "centésimos": "centésimo",
    "centígrada": "centígrado",
    "centígradas": "centígrado",
    "centígrados": "centígrado",
    "centígramos": "centígramo",
    "centímana": "centímano",
    "centímanas": "centímano",
    "centímanos": "centímano",
    "centímetros": "centímetro",
    "cenábamos": "cenar",
    "cenáculos": "cenáculo",
    "cenáis": "cenar",
    "cenáramos": "cenar",
    "cenáremos": "cenar",
    "cenásemos": "cenar",
    "cené": "cenar",
    "cenéis": "cenar",
    "ceníes": "cení",
    "cenís": "cení",
    "cenízaros": "cenízaro",
    "cenó": "cenar",
    "cepas": "cepa",
    "cepedas": "cepeda",
    "cepellones": "cepellón",
    "cepilla": "cepillar",
    "cepillaba": "cepillar",
    "cepillabais": "cepillar",
    "cepillaban": "cepillar",
    "cepillabas": "cepillar",
    "cepillad": "cepillar",
    "cepillada": "cepillar",
    "cepilladas": "cepillar",
    "cepillado": "cepillar",
    "cepilladoras": "cepilladora",
    "cepillados": "cepillar",
    "cepillamos": "cepillar",
    "cepillan": "cepillar",
    "cepillando": "cepillar",
    "cepillara": "cepillar",
    "cepillarais": "cepillar",
    "cepillaran": "cepillar",
    "cepillaras": "cepillar",
    "cepillare": "cepillar",
    "cepillareis": "cepillar",
    "cepillaremos": "cepillar",
    "cepillaren": "cepillar",
    "cepillares": "cepillar",
    "cepillaron": "cepillar",
    "cepillará": "cepillar",
    "cepillarán": "cepillar",
    "cepillarás": "cepillar",
    "cepillaré": "cepillar",
    "cepillaréis": "cepillar",
    "cepillaría": "cepillar",
    "cepillaríais": "cepillar",
    "cepillaríamos": "cepillar",
    "cepillarían": "cepillar",
    "cepillarías": "cepillar",
    "cepillas": "cepillar",
    "cepillase": "cepillar",
    "cepillaseis": "cepillar",
    "cepillasen": "cepillar",
    "cepillases": "cepillar",
    "cepillaste": "cepillar",
    "cepillasteis": "cepillar",
    "cepille": "cepillar",
    "cepillemos": "cepillar",
    "cepillen": "cepillar",
    "cepilles": "cepillar",
    "cepillo": "cepillar",
    "cepillos": "cepillo",
    "cepillábamos": "cepillar",
    "cepilláis": "cepillar",
    "cepilláramos": "cepillar",
    "cepilláremos": "cepillar",
    "cepillásemos": "cepillar",
    "cepillé": "cepillar",
    "cepilléis": "cepillar",
    "cepilló": "cepillar",
    "ceporra": "ceporro",
    "ceporras": "ceporro",
    "ceporros": "ceporro",
    "cepos": "cepo",
    "cequias": "cequia",
    "cerametales": "cerametal",
    "ceramidas": "ceramida",
    "ceramistas": "ceramista",
    "ceramizada": "ceramizado",
    "ceramizadas": "ceramizado",
    "ceramizados": "ceramizado",
    "cerapeces": "cerapez",
    "ceras": "cera",
    "cerasiotes": "cerasiote",
    "ceratinas": "ceratina",
    "ceratocricoidea": "ceratocricoideo",
    "ceratocricoideas": "ceratocricoideo",
    "ceratocricoideos": "ceratocricoideo",
    "ceratos": "cerato",
    "ceraunomancias": "ceraunomancia",
    "ceraunomancías": "ceraunomancía",
    "ceraunómetros": "ceraunómetro",
    "cerbatanas": "cerbatana",
    "cerberos": "cerbero",
    "cerca": "cercar",
    "cercaba": "cercar",
    "cercabais": "cercar",
    "cercaban": "cercar",
    "cercabas": "cercar",
    "cercad": "cercar",
    "cercada": "cercar",
    "cercadas": "cercar",
    "cercado": "cercar",
    "cercadora": "cercador",
    "cercadoras": "cercador",
    "cercadores": "cercador",
    "cercados": "cercar",
    "cercamos": "cercar",
    "cercan": "cercar",
    "cercana": "cercano",
    "cercanas": "cercano",
    "cercando": "cercar",
    "cercanos": "cercano",
    "cercanías": "cercanía",
    "cercara": "cercar",
    "cercarais": "cercar",
    "cercaran": "cercar",
    "cercaras": "cercar",
    "cercare": "cercar",
    "cercareis": "cercar",
    "cercaremos": "cercar",
    "cercaren": "cercar",
    "cercares": "cercar",
    "cercaron": "cercar",
    "cercará": "cercar",
    "cercarán": "cercar",
    "cercarás": "cercar",
    "cercaré": "cercar",
    "cercaréis": "cercar",
    "cercaría": "cercar",
    "cercaríais": "cercar",
    "cercaríamos": "cercar",
    "cercarían": "cercar",
    "cercarías": "cercar",
    "cercas": "cercar",
    "cercase": "cercar",
    "cercaseis": "cercar",
    "cercasen": "cercar",
    "cercases": "cercar",
    "cercaste": "cercar",
    "cercasteis": "cercar",
    "cercea": "cercear",
    "cerceaba": "cercear",
    "cerceabais": "cercear",
    "cerceaban": "cercear",
    "cerceabas": "cercear",
    "cercead": "cercear",
    "cerceada": "cercear",
    "cerceadas": "cercear",
    "cerceado": "cercear",
    "cerceados": "cercear",
    "cerceamos": "cercear",
    "cercean": "cercear",
    "cerceando": "cercear",
    "cerceara": "cercear",
    "cercearais": "cercear",
    "cercearan": "cercear",
    "cercearas": "cercear",
    "cerceare": "cercear",
    "cerceareis": "cercear",
    "cercearemos": "cercear",
    "cercearen": "cercear",
    "cerceares": "cercear",
    "cercearon": "cercear",
    "cerceará": "cercear",
    "cercearán": "cercear",
    "cercearás": "cercear",
    "cercearé": "cercear",
    "cercearéis": "cercear",
    "cercearía": "cercear",
    "cercearíais": "cercear",
    "cercearíamos": "cercear",
    "cercearían": "cercear",
    "cercearías": "cercear",
    "cerceas": "cercear",
    "cercease": "cercear",
    "cerceaseis": "cercear",
    "cerceasen": "cercear",
    "cerceases": "cercear",
    "cerceaste": "cercear",
    "cerceasteis": "cercear",
    "cercee": "cercear",
    "cerceemos": "cercear",
    "cerceen": "cercear",
    "cercees": "cercear",
    "cercena": "cercenar",
    "cercenaba": "cercenar",
    "cercenabais": "cercenar",
    "cercenaban": "cercenar",
    "cercenabas": "cercenar",
    "cercenad": "cercenar",
    "cercenada": "cercenar",
    "cercenadas": "cercenar",
    "cercenado": "cercenar",
    "cercenadora": "cercenador",
    "cercenadoras": "cercenador",
    "cercenadores": "cercenador",
    "cercenados": "cercenar",
    "cercenaduras": "cercenadura",
    "cercenamientos": "cercenamiento",
    "cercenamos": "cercenar",
    "cercenan": "cercenar",
    "cercenando": "cercenar",
    "cercenara": "cercenar",
    "cercenarais": "cercenar",
    "cercenaran": "cercenar",
    "cercenaras": "cercenar",
    "cercenare": "cercenar",
    "cercenareis": "cercenar",
    "cercenaremos": "cercenar",
    "cercenaren": "cercenar",
    "cercenares": "cercenar",
    "cercenaron": "cercenar",
    "cercenará": "cercenar",
    "cercenarán": "cercenar",
    "cercenarás": "cercenar",
    "cercenaré": "cercenar",
    "cercenaréis": "cercenar",
    "cercenaría": "cercenar",
    "cercenaríais": "cercenar",
    "cercenaríamos": "cercenar",
    "cercenarían": "cercenar",
    "cercenarías": "cercenar",
    "cercenas": "cercenar",
    "cercenase": "cercenar",
    "cercenaseis": "cercenar",
    "cercenasen": "cercenar",
    "cercenases": "cercenar",
    "cercenaste": "cercenar",
    "cercenasteis": "cercenar",
    "cercene": "cercenar",
    "cercenemos": "cercenar",
    "cercenen": "cercenar",
    "cercenes": "cercenar",
    "cerceno": "cercenar",
    "cercenábamos": "cercenar",
    "cercenáis": "cercenar",
    "cercenáramos": "cercenar",
    "cercenáremos": "cercenar",
    "cercenásemos": "cercenar",
    "cercené": "cercenar",
    "cercenéis": "cercenar",
    "cercenó": "cercenar",
    "cerceo": "cercear",
    "cercetas": "cerceta",
    "cerceábamos": "cercear",
    "cerceáis": "cercear",
    "cerceáramos": "cercear",
    "cerceáremos": "cercear",
    "cerceásemos": "cercear",
    "cerceé": "cercear",
    "cerceéis": "cercear",
    "cerceó": "cercear",
    "cercha": "cerchar",
    "cerchaba": "cerchar",
    "cerchabais": "cerchar",
    "cerchaban": "cerchar",
    "cerchabas": "cerchar",
    "cerchad": "cerchar",
    "cerchada": "cerchar",
    "cerchadas": "cerchar",
    "cerchado": "cerchar",
    "cerchados": "cerchar",
    "cerchamos": "cerchar",
    "cerchan": "cerchar",
    "cerchando": "cerchar",
    "cerchara": "cerchar",
    "cercharais": "cerchar",
    "cercharan": "cerchar",
    "cercharas": "cerchar",
    "cerchare": "cerchar",
    "cerchareis": "cerchar",
    "cercharemos": "cerchar",
    "cercharen": "cerchar",
    "cerchares": "cerchar",
    "cercharon": "cerchar",
    "cerchará": "cerchar",
    "cercharán": "cerchar",
    "cercharás": "cerchar",
    "cercharé": "cerchar",
    "cercharéis": "cerchar",
    "cercharía": "cerchar",
    "cercharíais": "cerchar",
    "cercharíamos": "cerchar",
    "cercharían": "cerchar",
    "cercharías": "cerchar",
    "cerchas": "cerchar",
    "cerchase": "cerchar",
    "cerchaseis": "cerchar",
    "cerchasen": "cerchar",
    "cerchases": "cerchar",
    "cerchaste": "cerchar",
    "cerchasteis": "cerchar",
    "cerche": "cerchar",
    "cerchea": "cerchear",
    "cercheaba": "cerchear",
    "cercheabais": "cerchear",
    "cercheaban": "cerchear",
    "cercheabas": "cerchear",
    "cerchead": "cerchear",
    "cercheada": "cerchear",
    "cercheadas": "cerchear",
    "cercheado": "cerchear",
    "cercheados": "cerchear",
    "cercheamos": "cerchear",
    "cerchean": "cerchear",
    "cercheando": "cerchear",
    "cercheara": "cerchear",
    "cerchearais": "cerchear",
    "cerchearan": "cerchear",
    "cerchearas": "cerchear",
    "cercheare": "cerchear",
    "cercheareis": "cerchear",
    "cerchearemos": "cerchear",
    "cerchearen": "cerchear",
    "cercheares": "cerchear",
    "cerchearon": "cerchear",
    "cercheará": "cerchear",
    "cerchearán": "cerchear",
    "cerchearás": "cerchear",
    "cerchearé": "cerchear",
    "cerchearéis": "cerchear",
    "cerchearía": "cerchear",
    "cerchearíais": "cerchear",
    "cerchearíamos": "cerchear",
    "cerchearían": "cerchear",
    "cerchearías": "cerchear",
    "cercheas": "cerchear",
    "cerchease": "cerchear",
    "cercheaseis": "cerchear",
    "cercheasen": "cerchear",
    "cercheases": "cerchear",
    "cercheaste": "cerchear",
    "cercheasteis": "cerchear",
    "cerchee": "cerchear",
    "cercheemos": "cerchear",
    "cercheen": "cerchear",
    "cerchees": "cerchear",
    "cerchemos": "cerchar",
    "cerchen": "cerchar",
    "cercheo": "cerchear",
    "cerches": "cerchar",
    "cercheábamos": "cerchear",
    "cercheáis": "cerchear",
    "cercheáramos": "cerchear",
    "cercheáremos": "cerchear",
    "cercheásemos": "cerchear",
    "cercheé": "cerchear",
    "cercheéis": "cerchear",
    "cercheó": "cerchear",
    "cercho": "cerchar",
    "cerchones": "cerchón",
    "cerchábamos": "cerchar",
    "cercháis": "cerchar",
    "cercháramos": "cerchar",
    "cercháremos": "cerchar",
    "cerchásemos": "cerchar",
    "cerché": "cerchar",
    "cerchéis": "cerchar",
    "cerchó": "cerchar",
    "cerciora": "cerciorar",
    "cercioraba": "cerciorar",
    "cerciorabais": "cerciorar",
    "cercioraban": "cerciorar",
    "cerciorabas": "cerciorar",
    "cerciorad": "cerciorar",
    "cerciorada": "cerciorar",
    "cercioradas": "cerciorar",
    "cerciorado": "cerciorar",
    "cerciorados": "cerciorar",
    "cercioramientos": "cercioramiento",
    "cercioramos": "cerciorar",
    "cercioran": "cerciorar",
    "cerciorando": "cerciorar",
    "cerciorara": "cerciorar",
    "cerciorarais": "cerciorar",
    "cercioraran": "cerciorar",
    "cercioraras": "cerciorar",
    "cerciorare": "cerciorar",
    "cerciorareis": "cerciorar",
    "cercioraremos": "cerciorar",
    "cercioraren": "cerciorar",
    "cerciorares": "cerciorar",
    "cercioraron": "cerciorar",
    "cerciorará": "cerciorar",
    "cerciorarán": "cerciorar",
    "cerciorarás": "cerciorar",
    "cercioraré": "cerciorar",
    "cercioraréis": "cerciorar",
    "cercioraría": "cerciorar",
    "cercioraríais": "cerciorar",
    "cercioraríamos": "cerciorar",
    "cerciorarían": "cerciorar",
    "cerciorarías": "cerciorar",
    "cercioras": "cerciorar",
    "cerciorase": "cerciorar",
    "cercioraseis": "cerciorar",
    "cerciorasen": "cerciorar",
    "cerciorases": "cerciorar",
    "cercioraste": "cerciorar",
    "cerciorasteis": "cerciorar",
    "cerciore": "cerciorar",
    "cercioremos": "cerciorar",
    "cercioren": "cerciorar",
    "cerciores": "cerciorar",
    "cercioro": "cerciorar",
    "cerciorábamos": "cerciorar",
    "cercioráis": "cerciorar",
    "cercioráramos": "cerciorar",
    "cercioráremos": "cerciorar",
    "cerciorásemos": "cerciorar",
    "cercioré": "cerciorar",
    "cercioréis": "cerciorar",
    "cercioró": "cerciorar",
    "cerco": "cercar",
    "cercopitecos": "cercopiteco",
    "cercopitécidos": "cercopitécido",
    "cercos": "cerco",
    "cercotes": "cercote",
    "cercábamos": "cercar",
    "cercáis": "cercar",
    "cercáramos": "cercar",
    "cercáremos": "cercar",
    "cercásemos": "cercar",
    "cercó": "cercar",
    "cerda": "cerdo",
    "cerdadas": "cerdada",
    "cerdas": "cerdo",
    "cerdea": "cerdear",
    "cerdeaba": "cerdear",
    "cerdeabais": "cerdear",
    "cerdeaban": "cerdear",
    "cerdeabas": "cerdear",
    "cerdead": "cerdear",
    "cerdeada": "cerdear",
    "cerdeadas": "cerdear",
    "cerdeado": "cerdear",
    "cerdeados": "cerdear",
    "cerdeamos": "cerdear",
    "cerdean": "cerdear",
    "cerdeando": "cerdear",
    "cerdeara": "cerdear",
    "cerdearais": "cerdear",
    "cerdearan": "cerdear",
    "cerdearas": "cerdear",
    "cerdeare": "cerdear",
    "cerdeareis": "cerdear",
    "cerdearemos": "cerdear",
    "cerdearen": "cerdear",
    "cerdeares": "cerdear",
    "cerdearon": "cerdear",
    "cerdeará": "cerdear",
    "cerdearán": "cerdear",
    "cerdearás": "cerdear",
    "cerdearé": "cerdear",
    "cerdearéis": "cerdear",
    "cerdearía": "cerdear",
    "cerdearíais": "cerdear",
    "cerdearíamos": "cerdear",
    "cerdearían": "cerdear",
    "cerdearías": "cerdear",
    "cerdeas": "cerdear",
    "cerdease": "cerdear",
    "cerdeaseis": "cerdear",
    "cerdeasen": "cerdear",
    "cerdeases": "cerdear",
    "cerdeaste": "cerdear",
    "cerdeasteis": "cerdear",
    "cerdee": "cerdear",
    "cerdeemos": "cerdear",
    "cerdeen": "cerdear",
    "cerdees": "cerdear",
    "cerdeo": "cerdear",
    "cerdeábamos": "cerdear",
    "cerdeáis": "cerdear",
    "cerdeáramos": "cerdear",
    "cerdeáremos": "cerdear",
    "cerdeásemos": "cerdear",
    "cerdeé": "cerdear",
    "cerdeéis": "cerdear",
    "cerdeó": "cerdear",
    "cerdos": "cerdo",
    "cerdosa": "cerdoso",
    "cerdosas": "cerdoso",
    "cerdosos": "cerdoso",
    "cerduda": "cerdudo",
    "cerdudas": "cerdudo",
    "cerdudos": "cerdudo",
    "cerdámenes": "cerdamen",
    "cerealera": "cerealero",
    "cerealeras": "cerealero",
    "cerealeros": "cerealero",
    "cereales": "cereal",
    "cerealiculturas": "cerealicultura",
    "cerealinas": "cerealina",
    "cerealistas": "cerealista",
    "cerealícolas": "cerealícola",
    "cerebelos": "cerebelo",
    "cerebelosa": "cerebeloso",
    "cerebelosas": "cerebeloso",
    "cerebelosos": "cerebeloso",
    "cerebrales": "cerebral",
    "cerebroespinales": "cerebroespinal",
    "cerebros": "cerebro",
    "cerebrospinales": "cerebrospinal",
    "cerebrovasculares": "cerebrovascular",
    "cerebrósidos": "cerebrósido",
    "cereceros": "cerecero",
    "cerecillas": "cerecilla",
    "ceremoniales": "ceremonial",
    "ceremonias": "ceremonia",
    "ceremoniosa": "ceremonioso",
    "ceremoniosas": "ceremonioso",
    "ceremoniosos": "ceremonioso",
    "cerera": "cerero",
    "cereras": "cerero",
    "cereros": "cerero",
    "cererías": "cerería",
    "ceretanos": "ceretano",
    "cerevisinas": "cerevisina",
    "cerezas": "cereza",
    "cerezos": "cerezo",
    "ceriantarios": "ceriantario",
    "ceriflores": "ceriflor",
    "cerillas": "cerilla",
    "cerillera": "cerillero",
    "cerilleras": "cerillero",
    "cerilleros": "cerillero",
    "cerillos": "cerillo",
    "cerimanes": "cerimán",
    "cerinas": "cerina",
    "cerios": "cerio",
    "ceritas": "cerita",
    "cermeña": "cermeño",
    "cermeñas": "cermeño",
    "cermeños": "cermeño",
    "cernadas": "cernada",
    "cernaderos": "cernadero",
    "cernamos": "cernir",
    "cerned": "cerner",
    "cernederas": "cernedera",
    "cernederos": "cernedero",
    "cernedora": "cernedor",
    "cernedoras": "cernedor",
    "cernedores": "cernedor",
    "cernejas": "cerneja",
    "cernejuda": "cernejudo",
    "cernejudas": "cernejudo",
    "cernejudos": "cernejudo",
    "cernemos": "cerner",
    "cerneremos": "cerner",
    "cernerá": "cerner",
    "cernerán": "cerner",
    "cernerás": "cerner",
    "cerneré": "cerner",
    "cerneréis": "cerner",
    "cernería": "cerner",
    "cerneríais": "cerner",
    "cerneríamos": "cerner",
    "cernerían": "cerner",
    "cernerías": "cerner",
    "cernes": "cerne",
    "cernid": "cernir",
    "cernida": "cernir",
    "cernidas": "cernir",
    "cernidillos": "cernidillo",
    "cernido": "cernir",
    "cernidos": "cernir",
    "cerniduras": "cernidura",
    "cerniendo": "cernir",
    "cerniera": "cernir",
    "cernierais": "cernir",
    "cernieran": "cernir",
    "cernieras": "cernir",
    "cerniere": "cernir",
    "cerniereis": "cernir",
    "cernieren": "cernir",
    "cernieres": "cernir",
    "cernieron": "cernir",
    "cerniese": "cernir",
    "cernieseis": "cernir",
    "cerniesen": "cernir",
    "cernieses": "cernir",
    "cernimos": "cernir",
    "cerniremos": "cernir",
    "cernirá": "cernir",
    "cernirán": "cernir",
    "cernirás": "cernir",
    "cerniré": "cernir",
    "cerniréis": "cernir",
    "cerniría": "cernir",
    "cerniríais": "cernir",
    "cerniríamos": "cernir",
    "cernirían": "cernir",
    "cernirías": "cernir",
    "cerniste": "cernir",
    "cernisteis": "cernir",
    "cerniéramos": "cernir",
    "cerniéremos": "cernir",
    "cerniésemos": "cernir",
    "cernió": "cernir",
    "cernos": "cerno",
    "cernudiana": "cernudiano",
    "cernudianas": "cernudiano",
    "cernudianos": "cernudiano",
    "cernáis": "cernir",
    "cernéis": "cerner",
    "cerní": "cernir",
    "cernía": "cernir",
    "cerníais": "cernir",
    "cerníamos": "cernir",
    "cernían": "cernir",
    "cernías": "cernir",
    "cernícalos": "cernícalo",
    "cernís": "cernir",
    "ceroferarios": "ceroferario",
    "ceroleínas": "ceroleína",
    "cerolla": "cerollo",
    "cerollas": "cerollo",
    "cerollos": "cerollo",
    "ceromancias": "ceromancia",
    "ceromancías": "ceromancía",
    "ceromas": "ceroma",
    "ceromática": "ceromático",
    "ceromáticas": "ceromático",
    "ceromáticos": "ceromático",
    "ceronera": "ceronero",
    "ceroneras": "ceronero",
    "ceroneros": "ceronero",
    "cerones": "cerón",
    "ceroplásticas": "ceroplástica",
    "ceros": "cero",
    "cerosa": "ceroso",
    "cerosas": "ceroso",
    "cerosos": "ceroso",
    "cerotatos": "cerotato",
    "cerotea": "cerotear",
    "ceroteaba": "cerotear",
    "ceroteabais": "cerotear",
    "ceroteaban": "cerotear",
    "ceroteabas": "cerotear",
    "cerotead": "cerotear",
    "ceroteada": "cerotear",
    "ceroteadas": "cerotear",
    "ceroteado": "cerotear",
    "ceroteados": "cerotear",
    "ceroteamos": "cerotear",
    "cerotean": "cerotear",
    "ceroteando": "cerotear",
    "ceroteara": "cerotear",
    "cerotearais": "cerotear",
    "cerotearan": "cerotear",
    "cerotearas": "cerotear",
    "ceroteare": "cerotear",
    "ceroteareis": "cerotear",
    "cerotearemos": "cerotear",
    "cerotearen": "cerotear",
    "ceroteares": "cerotear",
    "cerotearon": "cerotear",
    "ceroteará": "cerotear",
    "cerotearán": "cerotear",
    "cerotearás": "cerotear",
    "cerotearé": "cerotear",
    "cerotearéis": "cerotear",
    "cerotearía": "cerotear",
    "cerotearíais": "cerotear",
    "cerotearíamos": "cerotear",
    "cerotearían": "cerotear",
    "cerotearías": "cerotear",
    "ceroteas": "cerotear",
    "cerotease": "cerotear",
    "ceroteaseis": "cerotear",
    "ceroteasen": "cerotear",
    "ceroteases": "cerotear",
    "ceroteaste": "cerotear",
    "ceroteasteis": "cerotear",
    "cerotee": "cerotear",
    "ceroteemos": "cerotear",
    "ceroteen": "cerotear",
    "cerotees": "cerotear",
    "ceroteo": "cerotear",
    "ceroteros": "cerotero",
    "cerotes": "cerote",
    "ceroteábamos": "cerotear",
    "ceroteáis": "cerotear",
    "ceroteáramos": "cerotear",
    "ceroteáremos": "cerotear",
    "ceroteásemos": "cerotear",
    "ceroteé": "cerotear",
    "ceroteéis": "cerotear",
    "ceroteó": "cerotear",
    "cerque": "cercar",
    "cerquemos": "cercar",
    "cerquen": "cercar",
    "cerquera": "cerquero",
    "cerqueras": "cerquero",
    "cerqueros": "cerquero",
    "cerques": "cercar",
    "cerquillos": "cerquillo",
    "cerqué": "cercar",
    "cerquéis": "cercar",
    "cerraba": "cerrar",
    "cerrabais": "cerrar",
    "cerraban": "cerrar",
    "cerrabas": "cerrar",
    "cerrad": "cerrar",
    "cerrada": "cerrar",
    "cerradas": "cerrar",
    "cerradera": "cerradero",
    "cerraderas": "cerradero",
    "cerraderos": "cerradero",
    "cerrado": "cerrar",
    "cerradora": "cerrador",
    "cerradoras": "cerrador",
    "cerradores": "cerrador",
    "cerrados": "cerrar",
    "cerraduras": "cerradura",
    "cerrajas": "cerraja",
    "cerrajea": "cerrajear",
    "cerrajeaba": "cerrajear",
    "cerrajeabais": "cerrajear",
    "cerrajeaban": "cerrajear",
    "cerrajeabas": "cerrajear",
    "cerrajead": "cerrajear",
    "cerrajeada": "cerrajear",
    "cerrajeadas": "cerrajear",
    "cerrajeado": "cerrajear",
    "cerrajeados": "cerrajear",
    "cerrajeamos": "cerrajear",
    "cerrajean": "cerrajear",
    "cerrajeando": "cerrajear",
    "cerrajeara": "cerrajear",
    "cerrajearais": "cerrajear",
    "cerrajearan": "cerrajear",
    "cerrajearas": "cerrajear",
    "cerrajeare": "cerrajear",
    "cerrajeareis": "cerrajear",
    "cerrajearemos": "cerrajear",
    "cerrajearen": "cerrajear",
    "cerrajeares": "cerrajear",
    "cerrajearon": "cerrajear",
    "cerrajeará": "cerrajear",
    "cerrajearán": "cerrajear",
    "cerrajearás": "cerrajear",
    "cerrajearé": "cerrajear",
    "cerrajearéis": "cerrajear",
    "cerrajearía": "cerrajear",
    "cerrajearíais": "cerrajear",
    "cerrajearíamos": "cerrajear",
    "cerrajearían": "cerrajear",
    "cerrajearías": "cerrajear",
    "cerrajeas": "cerrajear",
    "cerrajease": "cerrajear",
    "cerrajeaseis": "cerrajear",
    "cerrajeasen": "cerrajear",
    "cerrajeases": "cerrajear",
    "cerrajeaste": "cerrajear",
    "cerrajeasteis": "cerrajear",
    "cerrajee": "cerrajear",
    "cerrajeemos": "cerrajear",
    "cerrajeen": "cerrajear",
    "cerrajees": "cerrajear",
    "cerrajeo": "cerrajear",
    "cerrajera": "cerrajero",
    "cerrajeras": "cerrajero",
    "cerrajeros": "cerrajero",
    "cerrajerías": "cerrajería",
    "cerrajeábamos": "cerrajear",
    "cerrajeáis": "cerrajear",
    "cerrajeáramos": "cerrajear",
    "cerrajeáremos": "cerrajear",
    "cerrajeásemos": "cerrajear",
    "cerrajeé": "cerrajear",
    "cerrajeéis": "cerrajear",
    "cerrajeó": "cerrajear",
    "cerrajones": "cerrajón",
    "cerramientos": "cerramiento",
    "cerramos": "cerrar",
    "cerrando": "cerrar",
    "cerrara": "cerrar",
    "cerrarais": "cerrar",
    "cerraran": "cerrar",
    "cerraras": "cerrar",
    "cerrare": "cerrar",
    "cerrareis": "cerrar",
    "cerraremos": "cerrar",
    "cerraren": "cerrar",
    "cerrares": "cerrar",
    "cerraron": "cerrar",
    "cerrará": "cerrar",
    "cerrarán": "cerrar",
    "cerrarás": "cerrar",
    "cerraré": "cerrar",
    "cerraréis": "cerrar",
    "cerraría": "cerrar",
    "cerraríais": "cerrar",
    "cerraríamos": "cerrar",
    "cerrarían": "cerrar",
    "cerrarías": "cerrar",
    "cerrase": "cerrar",
    "cerraseis": "cerrar",
    "cerrasen": "cerrar",
    "cerrases": "cerrar",
    "cerraste": "cerrar",
    "cerrasteis": "cerrar",
    "cerratina": "cerratín",
    "cerratinas": "cerratín",
    "cerratines": "cerratín",
    "cerrazones": "cerrazón",
    "cerrejones": "cerrejón",
    "cerremos": "cerrar",
    "cerrera": "cerrero",
    "cerreras": "cerrero",
    "cerreros": "cerrero",
    "cerrerías": "cerrería",
    "cerriles": "cerril",
    "cerrilidades": "cerrilidad",
    "cerrilismos": "cerrilismo",
    "cerrillas": "cerrilla",
    "cerrojazos": "cerrojazo",
    "cerrojos": "cerrojo",
    "cerros": "cerro",
    "cerrumas": "cerruma",
    "cerrábamos": "cerrar",
    "cerráis": "cerrar",
    "cerráramos": "cerrar",
    "cerráremos": "cerrar",
    "cerrásemos": "cerrar",
    "cerré": "cerrar",
    "cerréis": "cerrar",
    "cerró": "cerrar",
    "certera": "certero",
    "certeras": "certero",
    "certeros": "certero",
    "certezas": "certeza",
    "certidumbres": "certidumbre",
    "certifica": "certificar",
    "certificaba": "certificar",
    "certificabais": "certificar",
    "certificaban": "certificar",
    "certificabas": "certificar",
    "certificables": "certificable",
    "certificaciones": "certificación",
    "certificad": "certificar",
    "certificada": "certificar",
    "certificadas": "certificar",
    "certificado": "certificar",
    "certificadora": "certificador",
    "certificadoras": "certificador",
    "certificadores": "certificador",
    "certificados": "certificar",
    "certificamos": "certificar",
    "certifican": "certificar",
    "certificando": "certificar",
    "certificantes": "certificante",
    "certificara": "certificar",
    "certificarais": "certificar",
    "certificaran": "certificar",
    "certificaras": "certificar",
    "certificare": "certificar",
    "certificareis": "certificar",
    "certificaremos": "certificar",
    "certificaren": "certificar",
    "certificares": "certificar",
    "certificaron": "certificar",
    "certificará": "certificar",
    "certificarán": "certificar",
    "certificarás": "certificar",
    "certificaré": "certificar",
    "certificaréis": "certificar",
    "certificaría": "certificar",
    "certificaríais": "certificar",
    "certificaríamos": "certificar",
    "certificarían": "certificar",
    "certificarías": "certificar",
    "certificas": "certificar",
    "certificase": "certificar",
    "certificaseis": "certificar",
    "certificasen": "certificar",
    "certificases": "certificar",
    "certificaste": "certificar",
    "certificasteis": "certificar",
    "certificatoria": "certificatorio",
    "certificatorias": "certificatorio",
    "certificatorios": "certificatorio",
    "certifico": "certificar",
    "certificábamos": "certificar",
    "certificáis": "certificar",
    "certificáramos": "certificar",
    "certificáremos": "certificar",
    "certificásemos": "certificar",
    "certificó": "certificar",
    "certifique": "certificar",
    "certifiquemos": "certificar",
    "certifiquen": "certificar",
    "certifiques": "certificar",
    "certifiqué": "certificar",
    "certifiquéis": "certificar",
    "certitudes": "certitud",
    "certámenes": "certamen",
    "ceruminomas": "ceruminoma",
    "ceruminosa": "ceruminoso",
    "ceruminosas": "ceruminoso",
    "ceruminosos": "ceruminoso",
    "cerusitas": "cerusita",
    "cervales": "cerval",
    "cervantesca": "cervantesco",
    "cervantescas": "cervantesco",
    "cervantescos": "cervantesco",
    "cervantina": "cervantino",
    "cervantinas": "cervantino",
    "cervantinos": "cervantino",
    "cervantismos": "cervantismo",
    "cervantistas": "cervantista",
    "cervantófila": "cervantófilo",
    "cervantófilas": "cervantófilo",
    "cervantófilos": "cervantófilo",
    "cervaria": "cervario",
    "cervarias": "cervario",
    "cervarios": "cervario",
    "cervatos": "cervato",
    "cerveceos": "cerveceo",
    "cervecera": "cervecero",
    "cerveceras": "cervecero",
    "cerveceros": "cervecero",
    "cervecerías": "cervecería",
    "cervezas": "cerveza",
    "cervicales": "cervical",
    "cervicapras": "cervicapra",
    "cervices": "cerviz",
    "cervicotorácica": "cervicotorácico",
    "cervicotorácicas": "cervicotorácico",
    "cervicotorácicos": "cervicotorácico",
    "cervigones": "cervigón",
    "cerviguda": "cervigudo",
    "cervigudas": "cervigudo",
    "cervigudos": "cervigudo",
    "cerviguillos": "cerviguillo",
    "cervina": "cervino",
    "cervinas": "cervino",
    "cervinos": "cervino",
    "cervuna": "cervuno",
    "cervunas": "cervuno",
    "cervunos": "cervuno",
    "cervántica": "cervántico",
    "cervánticas": "cervántico",
    "cervánticos": "cervántico",
    "cerámica": "cerámico",
    "cerámicas": "cerámico",
    "cerámicos": "cerámico",
    "cerífera": "cerífero",
    "ceríferas": "cerífero",
    "ceríferos": "cerífero",
    "cerógrafos": "cerógrafo",
    "cerúlea": "cerúleo",
    "cerúleas": "cerúleo",
    "cerúleos": "cerúleo",
    "cerúmenes": "cerumen",
    "ces": "c",
    "cesa": "cesar",
    "cesaba": "cesar",
    "cesabais": "cesar",
    "cesaban": "cesar",
    "cesabas": "cesar",
    "cesaciones": "cesación",
    "cesad": "cesar",
    "cesada": "cesar",
    "cesadas": "cesar",
    "cesado": "cesar",
    "cesados": "cesar",
    "cesalpiniáceas": "cesalpiniácea",
    "cesamientos": "cesamiento",
    "cesamos": "cesar",
    "cesan": "cesar",
    "cesando": "cesar",
    "cesantes": "cesante",
    "cesantías": "cesantía",
    "cesara": "cesar",
    "cesarais": "cesar",
    "cesaran": "cesar",
    "cesaras": "cesar",
    "cesaraugustana": "cesaraugustano",
    "cesaraugustanas": "cesaraugustano",
    "cesaraugustanos": "cesaraugustano",
    "cesare": "cesar",
    "cesareis": "cesar",
    "cesaremos": "cesar",
    "cesaren": "cesar",
    "cesares": "cesar",
    "cesariana": "cesariano",
    "cesarianas": "cesariano",
    "cesarianos": "cesariano",
    "cesarismos": "cesarismo",
    "cesaristas": "cesarista",
    "cesaron": "cesar",
    "cesará": "cesar",
    "cesarán": "cesar",
    "cesarás": "cesar",
    "cesaré": "cesar",
    "cesaréis": "cesar",
    "cesaría": "cesar",
    "cesaríais": "cesar",
    "cesaríamos": "cesar",
    "cesarían": "cesar",
    "cesarías": "cesar",
    "cesas": "cesar",
    "cesase": "cesar",
    "cesaseis": "cesar",
    "cesasen": "cesar",
    "cesases": "cesar",
    "cesaste": "cesar",
    "cesasteis": "cesar",
    "cese": "cesar",
    "cesemos": "cesar",
    "cesen": "cesar",
    "ceses": "cese",
    "cesibles": "cesible",
    "cesionables": "cesionable",
    "cesionaria": "cesionario",
    "cesionarias": "cesionario",
    "cesionarios": "cesionario",
    "cesiones": "cesión",
    "cesionistas": "cesionista",
    "cesios": "cesio",
    "ceso": "cesar",
    "cespita": "cespitar",
    "cespitaba": "cespitar",
    "cespitabais": "cespitar",
    "cespitaban": "cespitar",
    "cespitabas": "cespitar",
    "cespitad": "cespitar",
    "cespitada": "cespitar",
    "cespitadas": "cespitar",
    "cespitado": "cespitar",
    "cespitados": "cespitar",
    "cespitamos": "cespitar",
    "cespitan": "cespitar",
    "cespitando": "cespitar",
    "cespitara": "cespitar",
    "cespitarais": "cespitar",
    "cespitaran": "cespitar",
    "cespitaras": "cespitar",
    "cespitare": "cespitar",
    "cespitareis": "cespitar",
    "cespitaremos": "cespitar",
    "cespitaren": "cespitar",
    "cespitares": "cespitar",
    "cespitaron": "cespitar",
    "cespitará": "cespitar",
    "cespitarán": "cespitar",
    "cespitarás": "cespitar",
    "cespitaré": "cespitar",
    "cespitaréis": "cespitar",
    "cespitaría": "cespitar",
    "cespitaríais": "cespitar",
    "cespitaríamos": "cespitar",
    "cespitarían": "cespitar",
    "cespitarías": "cespitar",
    "cespitas": "cespitar",
    "cespitase": "cespitar",
    "cespitaseis": "cespitar",
    "cespitasen": "cespitar",
    "cespitases": "cespitar",
    "cespitaste": "cespitar",
    "cespitasteis": "cespitar",
    "cespite": "cespitar",
    "cespitemos": "cespitar",
    "cespiten": "cespitar",
    "cespites": "cespitar",
    "cespito": "cespitar",
    "cespitosa": "cespitoso",
    "cespitosas": "cespitoso",
    "cespitosos": "cespitoso",
    "cespitábamos": "cespitar",
    "cespitáis": "cespitar",
    "cespitáramos": "cespitar",
    "cespitáremos": "cespitar",
    "cespitásemos": "cespitar",
    "cespité": "cespitar",
    "cespitéis": "cespitar",
    "cespitó": "cespitar",
    "cestadas": "cestada",
    "cestas": "cesta",
    "cestera": "cestero",
    "cesteras": "cestero",
    "cesteros": "cestero",
    "cesterías": "cestería",
    "cestiaria": "cestiario",
    "cestiarias": "cestiario",
    "cestiarios": "cestiario",
    "cestodos": "cestodo",
    "cestones": "cestón",
    "cestos": "cesto",
    "cesuras": "cesura",
    "cesábamos": "cesar",
    "cesáis": "cesar",
    "cesáramos": "cesar",
    "cesárea": "cesáreo",
    "cesáreas": "cesáreo",
    "cesáremos": "cesar",
    "cesáreos": "cesáreo",
    "cesásemos": "cesar",
    "cesé": "cesar",
    "ceséis": "cesar",
    "cesó": "cesar",
    "cetacismos": "cetacismo",
    "cetales": "cetal",
    "cetanos": "cetano",
    "cetarias": "cetaria",
    "cetenas": "cetena",
    "cetinas": "cetina",
    "cetobenzotriazinas": "cetobenzotriazina",
    "cetoles": "cetol",
    "cetonas": "cetona",
    "cetonemias": "cetonemia",
    "cetonias": "cetonia",
    "cetonurias": "cetonuria",
    "cetosas": "cetosa",
    "cetoximas": "cetoxima",
    "cetoácidos": "cetoácido",
    "cetrera": "cetrero",
    "cetreras": "cetrero",
    "cetreros": "cetrero",
    "cetrerías": "cetrería",
    "cetrina": "cetrino",
    "cetrinas": "cetrino",
    "cetrinos": "cetrino",
    "cetros": "cetro",
    "cetáceos": "cetáceo",
    "cetáreas": "cetárea",
    "cetílica": "cetílico",
    "cetílicas": "cetílico",
    "cetílicos": "cetílico",
    "cetónica": "cetónico",
    "cetónicas": "cetónico",
    "cetónicos": "cetónico",
    "ceugmas": "ceugma",
    "ceutíes": "ceutí",
    "ceutís": "ceutí",
    "cevitámica": "cevitámico",
    "cevitámicas": "cevitámico",
    "cevitámicos": "cevitámico",
    "ceñid": "ceñir",
    "ceñida": "ceñir",
    "ceñidas": "ceñir",
    "ceñido": "ceñir",
    "ceñidora": "ceñidor",
    "ceñidoras": "ceñidor",
    "ceñidores": "ceñidor",
    "ceñidos": "ceñir",
    "ceñimientos": "ceñimiento",
    "ceñimos": "ceñir",
    "ceñiremos": "ceñir",
    "ceñirá": "ceñir",
    "ceñirán": "ceñir",
    "ceñirás": "ceñir",
    "ceñiré": "ceñir",
    "ceñiréis": "ceñir",
    "ceñiría": "ceñir",
    "ceñiríais": "ceñir",
    "ceñiríamos": "ceñir",
    "ceñirían": "ceñir",
    "ceñirías": "ceñir",
    "ceñiste": "ceñir",
    "ceñisteis": "ceñir",
    "ceños": "ceño",
    "ceñosa": "ceñoso",
    "ceñosas": "ceñoso",
    "ceñosos": "ceñoso",
    "ceñuda": "ceñudo",
    "ceñudas": "ceñudo",
    "ceñudos": "ceñudo",
    "ceñí": "ceñir",
    "ceñía": "ceñir",
    "ceñíais": "ceñir",
    "ceñíamos": "ceñir",
    "ceñían": "ceñir",
    "ceñías": "ceñir",
    "ceñís": "ceñir",
    "cha-cha-chá": "chachachá",
    "cha-cha-chás": "chachachá",
    "chabacana": "chabacano",
    "chabacanas": "chabacano",
    "chabacanerías": "chabacanería",
    "chabacanos": "chabacano",
    "chabolas": "chabola",
    "chabolismos": "chabolismo",
    "chacales": "chacal",
    "chacanea": "chacanear",
    "chacaneaba": "chacanear",
    "chacaneabais": "chacanear",
    "chacaneaban": "chacanear",
    "chacaneabas": "chacanear",
    "chacanead": "chacanear",
    "chacaneada": "chacanear",
    "chacaneadas": "chacanear",
    "chacaneado": "chacanear",
    "chacaneados": "chacanear",
    "chacaneamos": "chacanear",
    "chacanean": "chacanear",
    "chacaneando": "chacanear",
    "chacaneara": "chacanear",
    "chacanearais": "chacanear",
    "chacanearan": "chacanear",
    "chacanearas": "chacanear",
    "chacaneare": "chacanear",
    "chacaneareis": "chacanear",
    "chacanearemos": "chacanear",
    "chacanearen": "chacanear",
    "chacaneares": "chacanear",
    "chacanearon": "chacanear",
    "chacaneará": "chacanear",
    "chacanearán": "chacanear",
    "chacanearás": "chacanear",
    "chacanearé": "chacanear",
    "chacanearéis": "chacanear",
    "chacanearía": "chacanear",
    "chacanearíais": "chacanear",
    "chacanearíamos": "chacanear",
    "chacanearían": "chacanear",
    "chacanearías": "chacanear",
    "chacaneas": "chacanear",
    "chacanease": "chacanear",
    "chacaneaseis": "chacanear",
    "chacaneasen": "chacanear",
    "chacaneases": "chacanear",
    "chacaneaste": "chacanear",
    "chacaneasteis": "chacanear",
    "chacanee": "chacanear",
    "chacaneemos": "chacanear",
    "chacaneen": "chacanear",
    "chacanees": "chacanear",
    "chacaneo": "chacanear",
    "chacaneábamos": "chacanear",
    "chacaneáis": "chacanear",
    "chacaneáramos": "chacanear",
    "chacaneáremos": "chacanear",
    "chacaneásemos": "chacanear",
    "chacaneé": "chacanear",
    "chacaneéis": "chacanear",
    "chacaneó": "chacanear",
    "chacarera": "chacarero",
    "chacareras": "chacarero",
    "chacareros": "chacarero",
    "chacarrachacas": "chacarrachaca",
    "chace": "chazar",
    "chacemos": "chazar",
    "chacen": "chazar",
    "chaces": "chazar",
    "chachachás": "chachachá",
    "chachalacas": "chachalaca",
    "chacharea": "chacharear",
    "chachareaba": "chacharear",
    "chachareabais": "chacharear",
    "chachareaban": "chacharear",
    "chachareabas": "chacharear",
    "chacharead": "chacharear",
    "chachareada": "chacharear",
    "chachareadas": "chacharear",
    "chachareado": "chacharear",
    "chachareados": "chacharear",
    "chachareamos": "chacharear",
    "chacharean": "chacharear",
    "chachareando": "chacharear",
    "chachareara": "chacharear",
    "chacharearais": "chacharear",
    "chacharearan": "chacharear",
    "chacharearas": "chacharear",
    "chachareare": "chacharear",
    "chachareareis": "chacharear",
    "chacharearemos": "chacharear",
    "chacharearen": "chacharear",
    "chachareares": "chacharear",
    "chacharearon": "chacharear",
    "chachareará": "chacharear",
    "chacharearán": "chacharear",
    "chacharearás": "chacharear",
    "chacharearé": "chacharear",
    "chacharearéis": "chacharear",
    "chacharearía": "chacharear",
    "chacharearíais": "chacharear",
    "chacharearíamos": "chacharear",
    "chacharearían": "chacharear",
    "chacharearías": "chacharear",
    "chachareas": "chacharear",
    "chacharease": "chacharear",
    "chachareaseis": "chacharear",
    "chachareasen": "chacharear",
    "chachareases": "chacharear",
    "chachareaste": "chacharear",
    "chachareasteis": "chacharear",
    "chacharee": "chacharear",
    "chachareemos": "chacharear",
    "chachareen": "chacharear",
    "chacharees": "chacharear",
    "chachareo": "chacharear",
    "chacharera": "chacharero",
    "chachareras": "chacharero",
    "chachareros": "chacharero",
    "chachareábamos": "chacharear",
    "chachareáis": "chacharear",
    "chachareáramos": "chacharear",
    "chachareáremos": "chacharear",
    "chachareásemos": "chacharear",
    "chachareé": "chacharear",
    "chachareéis": "chacharear",
    "chachareó": "chacharear",
    "chachas": "chacha",
    "chachis": "chachi",
    "chachos": "chacho",
    "chacinas": "chacina",
    "chacinera": "chacinero",
    "chacineras": "chacinero",
    "chacineros": "chacinero",
    "chacinerías": "chacinería",
    "chacolotea": "chacolotear",
    "chacoloteaba": "chacolotear",
    "chacoloteabais": "chacolotear",
    "chacoloteaban": "chacolotear",
    "chacoloteabas": "chacolotear",
    "chacolotead": "chacolotear",
    "chacoloteada": "chacolotear",
    "chacoloteadas": "chacolotear",
    "chacoloteado": "chacolotear",
    "chacoloteados": "chacolotear",
    "chacoloteamos": "chacolotear",
    "chacolotean": "chacolotear",
    "chacoloteando": "chacolotear",
    "chacoloteara": "chacolotear",
    "chacolotearais": "chacolotear",
    "chacolotearan": "chacolotear",
    "chacolotearas": "chacolotear",
    "chacoloteare": "chacolotear",
    "chacoloteareis": "chacolotear",
    "chacolotearemos": "chacolotear",
    "chacolotearen": "chacolotear",
    "chacoloteares": "chacolotear",
    "chacolotearon": "chacolotear",
    "chacoloteará": "chacolotear",
    "chacolotearán": "chacolotear",
    "chacolotearás": "chacolotear",
    "chacolotearé": "chacolotear",
    "chacolotearéis": "chacolotear",
    "chacolotearía": "chacolotear",
    "chacolotearíais": "chacolotear",
    "chacolotearíamos": "chacolotear",
    "chacolotearían": "chacolotear",
    "chacolotearías": "chacolotear",
    "chacoloteas": "chacolotear",
    "chacolotease": "chacolotear",
    "chacoloteaseis": "chacolotear",
    "chacoloteasen": "chacolotear",
    "chacoloteases": "chacolotear",
    "chacoloteaste": "chacolotear",
    "chacoloteasteis": "chacolotear",
    "chacolotee": "chacolotear",
    "chacoloteemos": "chacolotear",
    "chacoloteen": "chacolotear",
    "chacolotees": "chacolotear",
    "chacoloteo": "chacolotear",
    "chacoloteos": "chacoloteo",
    "chacoloteábamos": "chacolotear",
    "chacoloteáis": "chacolotear",
    "chacoloteáramos": "chacolotear",
    "chacoloteáremos": "chacolotear",
    "chacoloteásemos": "chacolotear",
    "chacoloteé": "chacolotear",
    "chacoloteéis": "chacolotear",
    "chacoloteó": "chacolotear",
    "chacolíes": "chacolí",
    "chacolís": "chacolí",
    "chaconas": "chacona",
    "chacones": "chacón",
    "chacos": "chaco",
    "chacotas": "chacota",
    "chacotea": "chacotear",
    "chacoteaba": "chacotear",
    "chacoteabais": "chacotear",
    "chacoteaban": "chacotear",
    "chacoteabas": "chacotear",
    "chacotead": "chacotear",
    "chacoteada": "chacotear",
    "chacoteadas": "chacotear",
    "chacoteado": "chacotear",
    "chacoteados": "chacotear",
    "chacoteamos": "chacotear",
    "chacotean": "chacotear",
    "chacoteando": "chacotear",
    "chacoteara": "chacotear",
    "chacotearais": "chacotear",
    "chacotearan": "chacotear",
    "chacotearas": "chacotear",
    "chacoteare": "chacotear",
    "chacoteareis": "chacotear",
    "chacotearemos": "chacotear",
    "chacotearen": "chacotear",
    "chacoteares": "chacotear",
    "chacotearon": "chacotear",
    "chacoteará": "chacotear",
    "chacotearán": "chacotear",
    "chacotearás": "chacotear",
    "chacotearé": "chacotear",
    "chacotearéis": "chacotear",
    "chacotearía": "chacotear",
    "chacotearíais": "chacotear",
    "chacotearíamos": "chacotear",
    "chacotearían": "chacotear",
    "chacotearías": "chacotear",
    "chacoteas": "chacotear",
    "chacotease": "chacotear",
    "chacoteaseis": "chacotear",
    "chacoteasen": "chacotear",
    "chacoteases": "chacotear",
    "chacoteaste": "chacotear",
    "chacoteasteis": "chacotear",
    "chacotee": "chacotear",
    "chacoteemos": "chacotear",
    "chacoteen": "chacotear",
    "chacotees": "chacotear",
    "chacoteo": "chacotear",
    "chacoteábamos": "chacotear",
    "chacoteáis": "chacotear",
    "chacoteáramos": "chacotear",
    "chacoteáremos": "chacotear",
    "chacoteásemos": "chacotear",
    "chacoteé": "chacotear",
    "chacoteéis": "chacotear",
    "chacoteó": "chacotear",
    "chacras": "chacra",
    "chacuacos": "chacuaco",
    "chacé": "chazar",
    "chacéis": "chazar",
    "chacós": "chacó",
    "chadiana": "chadiano",
    "chadianas": "chadiano",
    "chadianos": "chadiano",
    "chadores": "chador",
    "chafa": "chafar",
    "chafaba": "chafar",
    "chafabais": "chafar",
    "chafaban": "chafar",
    "chafabas": "chafar",
    "chafad": "chafar",
    "chafada": "chafar",
    "chafadas": "chafar",
    "chafado": "chafar",
    "chafados": "chafar",
    "chafaldetes": "chafaldete",
    "chafalditas": "chafaldita",
    "chafalditera": "chafalditero",
    "chafalditeras": "chafalditero",
    "chafalditeros": "chafalditero",
    "chafalla": "chafallar",
    "chafallaba": "chafallar",
    "chafallabais": "chafallar",
    "chafallaban": "chafallar",
    "chafallabas": "chafallar",
    "chafallad": "chafallar",
    "chafallada": "chafallar",
    "chafalladas": "chafallar",
    "chafallado": "chafallar",
    "chafallados": "chafallar",
    "chafallamos": "chafallar",
    "chafallan": "chafallar",
    "chafallando": "chafallar",
    "chafallara": "chafallar",
    "chafallarais": "chafallar",
    "chafallaran": "chafallar",
    "chafallaras": "chafallar",
    "chafallare": "chafallar",
    "chafallareis": "chafallar",
    "chafallaremos": "chafallar",
    "chafallaren": "chafallar",
    "chafallares": "chafallar",
    "chafallaron": "chafallar",
    "chafallará": "chafallar",
    "chafallarán": "chafallar",
    "chafallarás": "chafallar",
    "chafallaré": "chafallar",
    "chafallaréis": "chafallar",
    "chafallaría": "chafallar",
    "chafallaríais": "chafallar",
    "chafallaríamos": "chafallar",
    "chafallarían": "chafallar",
    "chafallarías": "chafallar",
    "chafallas": "chafallar",
    "chafallase": "chafallar",
    "chafallaseis": "chafallar",
    "chafallasen": "chafallar",
    "chafallases": "chafallar",
    "chafallaste": "chafallar",
    "chafallasteis": "chafallar",
    "chafalle": "chafallar",
    "chafallemos": "chafallar",
    "chafallen": "chafallar",
    "chafalles": "chafallar",
    "chafallo": "chafallar",
    "chafallona": "chafallón",
    "chafallonas": "chafallón",
    "chafallones": "chafallón",
    "chafallos": "chafallo",
    "chafallábamos": "chafallar",
    "chafalláis": "chafallar",
    "chafalláramos": "chafallar",
    "chafalláremos": "chafallar",
    "chafallásemos": "chafallar",
    "chafallé": "chafallar",
    "chafalléis": "chafallar",
    "chafalló": "chafallar",
    "chafalonías": "chafalonía",
    "chafamientos": "chafamiento",
    "chafamos": "chafar",
    "chafan": "chafar",
    "chafandines": "chafandín",
    "chafando": "chafar",
    "chafara": "chafar",
    "chafarais": "chafar",
    "chafaran": "chafar",
    "chafaras": "chafar",
    "chafare": "chafar",
    "chafareis": "chafar",
    "chafaremos": "chafar",
    "chafaren": "chafar",
    "chafares": "chafar",
    "chafarices": "chafariz",
    "chafaron": "chafar",
    "chafarotes": "chafarote",
    "chafarrina": "chafarrinar",
    "chafarrinaba": "chafarrinar",
    "chafarrinabais": "chafarrinar",
    "chafarrinaban": "chafarrinar",
    "chafarrinabas": "chafarrinar",
    "chafarrinad": "chafarrinar",
    "chafarrinada": "chafarrinar",
    "chafarrinadas": "chafarrinar",
    "chafarrinado": "chafarrinar",
    "chafarrinados": "chafarrinar",
    "chafarrinamos": "chafarrinar",
    "chafarrinan": "chafarrinar",
    "chafarrinando": "chafarrinar",
    "chafarrinara": "chafarrinar",
    "chafarrinarais": "chafarrinar",
    "chafarrinaran": "chafarrinar",
    "chafarrinaras": "chafarrinar",
    "chafarrinare": "chafarrinar",
    "chafarrinareis": "chafarrinar",
    "chafarrinaremos": "chafarrinar",
    "chafarrinaren": "chafarrinar",
    "chafarrinares": "chafarrinar",
    "chafarrinaron": "chafarrinar",
    "chafarrinará": "chafarrinar",
    "chafarrinarán": "chafarrinar",
    "chafarrinarás": "chafarrinar",
    "chafarrinaré": "chafarrinar",
    "chafarrinaréis": "chafarrinar",
    "chafarrinaría": "chafarrinar",
    "chafarrinaríais": "chafarrinar",
    "chafarrinaríamos": "chafarrinar",
    "chafarrinarían": "chafarrinar",
    "chafarrinarías": "chafarrinar",
    "chafarrinas": "chafarrinar",
    "chafarrinase": "chafarrinar",
    "chafarrinaseis": "chafarrinar",
    "chafarrinasen": "chafarrinar",
    "chafarrinases": "chafarrinar",
    "chafarrinaste": "chafarrinar",
    "chafarrinasteis": "chafarrinar",
    "chafarrine": "chafarrinar",
    "chafarrinemos": "chafarrinar",
    "chafarrinen": "chafarrinar",
    "chafarrines": "chafarrinar",
    "chafarrino": "chafarrinar",
    "chafarrinones": "chafarrinón",
    "chafarrinábamos": "chafarrinar",
    "chafarrináis": "chafarrinar",
    "chafarrináramos": "chafarrinar",
    "chafarrináremos": "chafarrinar",
    "chafarrinásemos": "chafarrinar",
    "chafarriné": "chafarrinar",
    "chafarrinéis": "chafarrinar",
    "chafarrinó": "chafarrinar",
    "chafará": "chafar",
    "chafarán": "chafar",
    "chafarás": "chafar",
    "chafaré": "chafar",
    "chafaréis": "chafar",
    "chafaría": "chafar",
    "chafaríais": "chafar",
    "chafaríamos": "chafar",
    "chafarían": "chafar",
    "chafarías": "chafar",
    "chafas": "chafar",
    "chafase": "chafar",
    "chafaseis": "chafar",
    "chafasen": "chafar",
    "chafases": "chafar",
    "chafaste": "chafar",
    "chafasteis": "chafar",
    "chafe": "chafar",
    "chafemos": "chafar",
    "chafen": "chafar",
    "chafes": "chafar",
    "chaflana": "chaflanar",
    "chaflanaba": "chaflanar",
    "chaflanabais": "chaflanar",
    "chaflanaban": "chaflanar",
    "chaflanabas": "chaflanar",
    "chaflanad": "chaflanar",
    "chaflanada": "chaflanar",
    "chaflanadas": "chaflanar",
    "chaflanado": "chaflanar",
    "chaflanados": "chaflanar",
    "chaflanamos": "chaflanar",
    "chaflanan": "chaflanar",
    "chaflanando": "chaflanar",
    "chaflanara": "chaflanar",
    "chaflanarais": "chaflanar",
    "chaflanaran": "chaflanar",
    "chaflanaras": "chaflanar",
    "chaflanare": "chaflanar",
    "chaflanareis": "chaflanar",
    "chaflanaremos": "chaflanar",
    "chaflanaren": "chaflanar",
    "chaflanares": "chaflanar",
    "chaflanaron": "chaflanar",
    "chaflanará": "chaflanar",
    "chaflanarán": "chaflanar",
    "chaflanarás": "chaflanar",
    "chaflanaré": "chaflanar",
    "chaflanaréis": "chaflanar",
    "chaflanaría": "chaflanar",
    "chaflanaríais": "chaflanar",
    "chaflanaríamos": "chaflanar",
    "chaflanarían": "chaflanar",
    "chaflanarías": "chaflanar",
    "chaflanas": "chaflanar",
    "chaflanase": "chaflanar",
    "chaflanaseis": "chaflanar",
    "chaflanasen": "chaflanar",
    "chaflanases": "chaflanar",
    "chaflanaste": "chaflanar",
    "chaflanasteis": "chaflanar",
    "chaflane": "chaflanar",
    "chaflanemos": "chaflanar",
    "chaflanen": "chaflanar",
    "chaflanes": "chaflanar",
    "chaflano": "chaflanar",
    "chaflanábamos": "chaflanar",
    "chaflanáis": "chaflanar",
    "chaflanáramos": "chaflanar",
    "chaflanáremos": "chaflanar",
    "chaflanásemos": "chaflanar",
    "chaflané": "chaflanar",
    "chaflanéis": "chaflanar",
    "chaflanó": "chaflanar",
    "chafo": "chafar",
    "chafábamos": "chafar",
    "chafáis": "chafar",
    "chafáramos": "chafar",
    "chafáremos": "chafar",
    "chafásemos": "chafar",
    "chafé": "chafar",
    "chaféis": "chafar",
    "chafó": "chafar",
    "chagras": "chagra",
    "chagualas": "chaguala",
    "chagualones": "chagualón",
    "chaimas": "chaima",
    "chairas": "chaira",
    "chajás": "chajá",
    "chala": "chalar",
    "chalaba": "chalar",
    "chalabais": "chalar",
    "chalaban": "chalar",
    "chalabas": "chalar",
    "chalad": "chalar",
    "chalada": "chalar",
    "chaladas": "chalar",
    "chalado": "chalar",
    "chalados": "chalar",
    "chaladuras": "chaladura",
    "chalamientos": "chalamiento",
    "chalamos": "chalar",
    "chalan": "chalar",
    "chalana": "chalán",
    "chalanas": "chalana",
    "chalando": "chalar",
    "chalanea": "chalanear",
    "chalaneaba": "chalanear",
    "chalaneabais": "chalanear",
    "chalaneaban": "chalanear",
    "chalaneabas": "chalanear",
    "chalanead": "chalanear",
    "chalaneada": "chalanear",
    "chalaneadas": "chalanear",
    "chalaneado": "chalanear",
    "chalaneados": "chalanear",
    "chalaneamos": "chalanear",
    "chalanean": "chalanear",
    "chalaneando": "chalanear",
    "chalaneara": "chalanear",
    "chalanearais": "chalanear",
    "chalanearan": "chalanear",
    "chalanearas": "chalanear",
    "chalaneare": "chalanear",
    "chalaneareis": "chalanear",
    "chalanearemos": "chalanear",
    "chalanearen": "chalanear",
    "chalaneares": "chalanear",
    "chalanearon": "chalanear",
    "chalaneará": "chalanear",
    "chalanearán": "chalanear",
    "chalanearás": "chalanear",
    "chalanearé": "chalanear",
    "chalanearéis": "chalanear",
    "chalanearía": "chalanear",
    "chalanearíais": "chalanear",
    "chalanearíamos": "chalanear",
    "chalanearían": "chalanear",
    "chalanearías": "chalanear",
    "chalaneas": "chalanear",
    "chalanease": "chalanear",
    "chalaneaseis": "chalanear",
    "chalaneasen": "chalanear",
    "chalaneases": "chalanear",
    "chalaneaste": "chalanear",
    "chalaneasteis": "chalanear",
    "chalanee": "chalanear",
    "chalaneemos": "chalanear",
    "chalaneen": "chalanear",
    "chalanees": "chalanear",
    "chalaneo": "chalanear",
    "chalaneos": "chalaneo",
    "chalanerías": "chalanería",
    "chalanes": "chalán",
    "chalanesca": "chalanesco",
    "chalanescas": "chalanesco",
    "chalanescos": "chalanesco",
    "chalaneábamos": "chalanear",
    "chalaneáis": "chalanear",
    "chalaneáramos": "chalanear",
    "chalaneáremos": "chalanear",
    "chalaneásemos": "chalanear",
    "chalaneé": "chalanear",
    "chalaneéis": "chalanear",
    "chalaneó": "chalanear",
    "chalara": "chalar",
    "chalarais": "chalar",
    "chalaran": "chalar",
    "chalaras": "chalar",
    "chalare": "chalar",
    "chalareis": "chalar",
    "chalaremos": "chalar",
    "chalaren": "chalar",
    "chalares": "chalar",
    "chalaron": "chalar",
    "chalará": "chalar",
    "chalarán": "chalar",
    "chalarás": "chalar",
    "chalaré": "chalar",
    "chalaréis": "chalar",
    "chalaría": "chalar",
    "chalaríais": "chalar",
    "chalaríamos": "chalar",
    "chalarían": "chalar",
    "chalarías": "chalar",
    "chalas": "chalar",
    "chalase": "chalar",
    "chalaseis": "chalar",
    "chalasen": "chalar",
    "chalases": "chalar",
    "chalaste": "chalar",
    "chalasteis": "chalar",
    "chalazas": "chalaza",
    "chale": "chalar",
    "chalecos": "chaleco",
    "chalemos": "chalar",
    "chalen": "chalar",
    "chalequera": "chalequero",
    "chalequeras": "chalequero",
    "chalequeros": "chalequero",
    "chales": "chalar",
    "chalets": "chalet",
    "chalinas": "chalina",
    "challenges": "challenge",
    "chalo": "chalar",
    "chalotes": "chalote",
    "chalupas": "chalupa",
    "chalábamos": "chalar",
    "chaláis": "chalar",
    "chaláramos": "chalar",
    "chaláremos": "chalar",
    "chalásemos": "chalar",
    "chalé": "chalar",
    "chaléis": "chalar",
    "chalés": "chalé",
    "chaló": "chalar",
    "chamaca": "chamaco",
    "chamacas": "chamaco",
    "chamacos": "chamaco",
    "chamadas": "chamada",
    "chamales": "chamal",
    "chamanes": "chamán",
    "chamanismos": "chamanismo",
    "chamanistas": "chamanista",
    "chamantos": "chamanto",
    "chamarascas": "chamarasca",
    "chamarices": "chamariz",
    "chamarilea": "chamarilear",
    "chamarileaba": "chamarilear",
    "chamarileabais": "chamarilear",
    "chamarileaban": "chamarilear",
    "chamarileabas": "chamarilear",
    "chamarilead": "chamarilear",
    "chamarileada": "chamarilear",
    "chamarileadas": "chamarilear",
    "chamarileado": "chamarilear",
    "chamarileados": "chamarilear",
    "chamarileamos": "chamarilear",
    "chamarilean": "chamarilear",
    "chamarileando": "chamarilear",
    "chamarileara": "chamarilear",
    "chamarilearais": "chamarilear",
    "chamarilearan": "chamarilear",
    "chamarilearas": "chamarilear",
    "chamarileare": "chamarilear",
    "chamarileareis": "chamarilear",
    "chamarilearemos": "chamarilear",
    "chamarilearen": "chamarilear",
    "chamarileares": "chamarilear",
    "chamarilearon": "chamarilear",
    "chamarileará": "chamarilear",
    "chamarilearán": "chamarilear",
    "chamarilearás": "chamarilear",
    "chamarilearé": "chamarilear",
    "chamarilearéis": "chamarilear",
    "chamarilearía": "chamarilear",
    "chamarilearíais": "chamarilear",
    "chamarilearíamos": "chamarilear",
    "chamarilearían": "chamarilear",
    "chamarilearías": "chamarilear",
    "chamarileas": "chamarilear",
    "chamarilease": "chamarilear",
    "chamarileaseis": "chamarilear",
    "chamarileasen": "chamarilear",
    "chamarileases": "chamarilear",
    "chamarileaste": "chamarilear",
    "chamarileasteis": "chamarilear",
    "chamarilee": "chamarilear",
    "chamarileemos": "chamarilear",
    "chamarileen": "chamarilear",
    "chamarilees": "chamarilear",
    "chamarileo": "chamarilear",
    "chamarileos": "chamarileo",
    "chamarilera": "chamarilero",
    "chamarileras": "chamarilero",
    "chamarileros": "chamarilero",
    "chamarileábamos": "chamarilear",
    "chamarileáis": "chamarilear",
    "chamarileáramos": "chamarilear",
    "chamarileáremos": "chamarilear",
    "chamarileásemos": "chamarilear",
    "chamarileé": "chamarilear",
    "chamarileéis": "chamarilear",
    "chamarileó": "chamarilear",
    "chamarones": "chamarón",
    "chamarras": "chamarra",
    "chamarretas": "chamarreta",
    "chamas": "chama",
    "chambas": "chamba",
    "chambelanes": "chambelán",
    "chamberga": "chambergo",
    "chambergas": "chambergo",
    "chambergos": "chambergo",
    "chambillas": "chambilla",
    "chambona": "chambón",
    "chambonadas": "chambonada",
    "chambonas": "chambón",
    "chambones": "chambón",
    "chambranas": "chambrana",
    "chambras": "chambra",
    "chamburos": "chamburo",
    "chamelos": "chamelo",
    "chamiceras": "chamicera",
    "chamizas": "chamiza",
    "chamizos": "chamizo",
    "chamorra": "chamorro",
    "chamorras": "chamorro",
    "chamorros": "chamorro",
    "chamositas": "chamosita",
    "chamotas": "chamota",
    "champanes": "champán",
    "champañas": "champaña",
    "champañeras": "champañera",
    "champañerías": "champañería",
    "champiñones": "champiñón",
    "champurra": "champurrar",
    "champurraba": "champurrar",
    "champurrabais": "champurrar",
    "champurraban": "champurrar",
    "champurrabas": "champurrar",
    "champurrad": "champurrar",
    "champurrada": "champurrar",
    "champurradas": "champurrar",
    "champurrado": "champurrar",
    "champurrados": "champurrar",
    "champurramos": "champurrar",
    "champurran": "champurrar",
    "champurrando": "champurrar",
    "champurrara": "champurrar",
    "champurrarais": "champurrar",
    "champurraran": "champurrar",
    "champurraras": "champurrar",
    "champurrare": "champurrar",
    "champurrareis": "champurrar",
    "champurraremos": "champurrar",
    "champurraren": "champurrar",
    "champurrares": "champurrar",
    "champurraron": "champurrar",
    "champurrará": "champurrar",
    "champurrarán": "champurrar",
    "champurrarás": "champurrar",
    "champurraré": "champurrar",
    "champurraréis": "champurrar",
    "champurraría": "champurrar",
    "champurraríais": "champurrar",
    "champurraríamos": "champurrar",
    "champurrarían": "champurrar",
    "champurrarías": "champurrar",
    "champurras": "champurrar",
    "champurrase": "champurrar",
    "champurraseis": "champurrar",
    "champurrasen": "champurrar",
    "champurrases": "champurrar",
    "champurraste": "champurrar",
    "champurrasteis": "champurrar",
    "champurre": "champurrar",
    "champurremos": "champurrar",
    "champurren": "champurrar",
    "champurres": "champurrar",
    "champurro": "champurrar",
    "champurrábamos": "champurrar",
    "champurráis": "champurrar",
    "champurráramos": "champurrar",
    "champurráremos": "champurrar",
    "champurrásemos": "champurrar",
    "champurré": "champurrar",
    "champurréis": "champurrar",
    "champurró": "champurrar",
    "champúes": "champú",
    "champús": "champú",
    "champúses": "champú",
    "chamuchinas": "chamuchina",
    "chamusca": "chamuscar",
    "chamuscaba": "chamuscar",
    "chamuscabais": "chamuscar",
    "chamuscaban": "chamuscar",
    "chamuscabas": "chamuscar",
    "chamuscad": "chamuscar",
    "chamuscada": "chamuscar",
    "chamuscadas": "chamuscar",
    "chamuscado": "chamuscar",
    "chamuscados": "chamuscar",
    "chamuscamos": "chamuscar",
    "chamuscan": "chamuscar",
    "chamuscando": "chamuscar",
    "chamuscara": "chamuscar",
    "chamuscarais": "chamuscar",
    "chamuscaran": "chamuscar",
    "chamuscaras": "chamuscar",
    "chamuscare": "chamuscar",
    "chamuscareis": "chamuscar",
    "chamuscaremos": "chamuscar",
    "chamuscaren": "chamuscar",
    "chamuscares": "chamuscar",
    "chamuscaron": "chamuscar",
    "chamuscará": "chamuscar",
    "chamuscarán": "chamuscar",
    "chamuscarás": "chamuscar",
    "chamuscaré": "chamuscar",
    "chamuscaréis": "chamuscar",
    "chamuscaría": "chamuscar",
    "chamuscaríais": "chamuscar",
    "chamuscaríamos": "chamuscar",
    "chamuscarían": "chamuscar",
    "chamuscarías": "chamuscar",
    "chamuscas": "chamuscar",
    "chamuscase": "chamuscar",
    "chamuscaseis": "chamuscar",
    "chamuscasen": "chamuscar",
    "chamuscases": "chamuscar",
    "chamuscaste": "chamuscar",
    "chamuscasteis": "chamuscar",
    "chamusco": "chamuscar",
    "chamuscos": "chamusco",
    "chamuscábamos": "chamuscar",
    "chamuscáis": "chamuscar",
    "chamuscáramos": "chamuscar",
    "chamuscáremos": "chamuscar",
    "chamuscásemos": "chamuscar",
    "chamuscó": "chamuscar",
    "chamusque": "chamuscar",
    "chamusquemos": "chamuscar",
    "chamusquen": "chamuscar",
    "chamusques": "chamuscar",
    "chamusquinas": "chamusquina",
    "chamusqué": "chamuscar",
    "chamusquéis": "chamuscar",
    "chanadas": "chanada",
    "chanca": "chancar",
    "chancaba": "chancar",
    "chancabais": "chancar",
    "chancaban": "chancar",
    "chancabas": "chancar",
    "chancacas": "chancaca",
    "chancad": "chancar",
    "chancada": "chancar",
    "chancadas": "chancar",
    "chancado": "chancar",
    "chancados": "chancar",
    "chancamos": "chancar",
    "chancan": "chancar",
    "chancando": "chancar",
    "chancaquitas": "chancaquita",
    "chancara": "chancar",
    "chancarais": "chancar",
    "chancaran": "chancar",
    "chancaras": "chancar",
    "chancare": "chancar",
    "chancareis": "chancar",
    "chancaremos": "chancar",
    "chancaren": "chancar",
    "chancares": "chancar",
    "chancaron": "chancar",
    "chancará": "chancar",
    "chancarán": "chancar",
    "chancarás": "chancar",
    "chancaré": "chancar",
    "chancaréis": "chancar",
    "chancaría": "chancar",
    "chancaríais": "chancar",
    "chancaríamos": "chancar",
    "chancarían": "chancar",
    "chancarías": "chancar",
    "chancas": "chancar",
    "chancase": "chancar",
    "chancaseis": "chancar",
    "chancasen": "chancar",
    "chancases": "chancar",
    "chancaste": "chancar",
    "chancasteis": "chancar",
    "chancea": "chancear",
    "chanceaba": "chancear",
    "chanceabais": "chancear",
    "chanceaban": "chancear",
    "chanceabas": "chancear",
    "chancead": "chancear",
    "chanceada": "chancear",
    "chanceadas": "chancear",
    "chanceado": "chancear",
    "chanceados": "chancear",
    "chanceamos": "chancear",
    "chancean": "chancear",
    "chanceando": "chancear",
    "chanceara": "chancear",
    "chancearais": "chancear",
    "chancearan": "chancear",
    "chancearas": "chancear",
    "chanceare": "chancear",
    "chanceareis": "chancear",
    "chancearemos": "chancear",
    "chancearen": "chancear",
    "chanceares": "chancear",
    "chancearon": "chancear",
    "chanceará": "chancear",
    "chancearán": "chancear",
    "chancearás": "chancear",
    "chancearé": "chancear",
    "chancearéis": "chancear",
    "chancearía": "chancear",
    "chancearíais": "chancear",
    "chancearíamos": "chancear",
    "chancearían": "chancear",
    "chancearías": "chancear",
    "chanceas": "chancear",
    "chancease": "chancear",
    "chanceaseis": "chancear",
    "chanceasen": "chancear",
    "chanceases": "chancear",
    "chanceaste": "chancear",
    "chanceasteis": "chancear",
    "chancee": "chancear",
    "chanceemos": "chancear",
    "chanceen": "chancear",
    "chancees": "chancear",
    "chanceo": "chancear",
    "chancera": "chancero",
    "chanceras": "chancero",
    "chanceros": "chancero",
    "chanceábamos": "chancear",
    "chanceáis": "chancear",
    "chanceáramos": "chancear",
    "chanceáremos": "chancear",
    "chanceásemos": "chancear",
    "chanceé": "chancear",
    "chanceéis": "chancear",
    "chanceó": "chancear",
    "chancha": "chancho",
    "chanchas": "chancho",
    "chancherías": "chanchería",
    "chanchis": "chanchi",
    "chanchos": "chancho",
    "chanchullos": "chanchullo",
    "chancillerías": "chancillería",
    "chanclas": "chancla",
    "chancletas": "chancleta",
    "chancletea": "chancletear",
    "chancleteaba": "chancletear",
    "chancleteabais": "chancletear",
    "chancleteaban": "chancletear",
    "chancleteabas": "chancletear",
    "chancletead": "chancletear",
    "chancleteada": "chancletear",
    "chancleteadas": "chancletear",
    "chancleteado": "chancletear",
    "chancleteados": "chancletear",
    "chancleteamos": "chancletear",
    "chancletean": "chancletear",
    "chancleteando": "chancletear",
    "chancleteara": "chancletear",
    "chancletearais": "chancletear",
    "chancletearan": "chancletear",
    "chancletearas": "chancletear",
    "chancleteare": "chancletear",
    "chancleteareis": "chancletear",
    "chancletearemos": "chancletear",
    "chancletearen": "chancletear",
    "chancleteares": "chancletear",
    "chancletearon": "chancletear",
    "chancleteará": "chancletear",
    "chancletearán": "chancletear",
    "chancletearás": "chancletear",
    "chancletearé": "chancletear",
    "chancletearéis": "chancletear",
    "chancletearía": "chancletear",
    "chancletearíais": "chancletear",
    "chancletearíamos": "chancletear",
    "chancletearían": "chancletear",
    "chancletearías": "chancletear",
    "chancleteas": "chancletear",
    "chancletease": "chancletear",
    "chancleteaseis": "chancletear",
    "chancleteasen": "chancletear",
    "chancleteases": "chancletear",
    "chancleteaste": "chancletear",
    "chancleteasteis": "chancletear",
    "chancletee": "chancletear",
    "chancleteemos": "chancletear",
    "chancleteen": "chancletear",
    "chancletees": "chancletear",
    "chancleteo": "chancletear",
    "chancleteos": "chancleteo",
    "chancleteábamos": "chancletear",
    "chancleteáis": "chancletear",
    "chancleteáramos": "chancletear",
    "chancleteáremos": "chancletear",
    "chancleteásemos": "chancletear",
    "chancleteé": "chancletear",
    "chancleteéis": "chancletear",
    "chancleteó": "chancletear",
    "chanclos": "chanclo",
    "chanco": "chancar",
    "chancros": "chancro",
    "chancábamos": "chancar",
    "chancáis": "chancar",
    "chancáramos": "chancar",
    "chancáremos": "chancar",
    "chancásemos": "chancar",
    "chancó": "chancar",
    "chanfainas": "chanfaina",
    "chanflona": "chanflón",
    "chanflonas": "chanflón",
    "chanflones": "chanflón",
    "changa": "changar",
    "changaba": "changar",
    "changabais": "changar",
    "changaban": "changar",
    "changabas": "changar",
    "changad": "changar",
    "changada": "changar",
    "changadas": "changar",
    "changado": "changar",
    "changados": "changar",
    "changamos": "changar",
    "changan": "changar",
    "changando": "changar",
    "changara": "changar",
    "changarais": "changar",
    "changaran": "changar",
    "changaras": "changar",
    "changare": "changar",
    "changareis": "changar",
    "changaremos": "changar",
    "changaren": "changar",
    "changares": "changar",
    "changaron": "changar",
    "changarros": "changarro",
    "changará": "changar",
    "changarán": "changar",
    "changarás": "changar",
    "changaré": "changar",
    "changaréis": "changar",
    "changaría": "changar",
    "changaríais": "changar",
    "changaríamos": "changar",
    "changarían": "changar",
    "changarías": "changar",
    "changas": "changar",
    "changase": "changar",
    "changaseis": "changar",
    "changasen": "changar",
    "changases": "changar",
    "changaste": "changar",
    "changasteis": "changar",
    "chango": "changar",
    "changue": "changar",
    "changuemos": "changar",
    "changuen": "changar",
    "changues": "changar",
    "changué": "changar",
    "changuéis": "changar",
    "changábamos": "changar",
    "changáis": "changar",
    "changáramos": "changar",
    "changáremos": "changar",
    "changásemos": "changar",
    "changó": "changar",
    "chanos": "chano",
    "chanque": "chancar",
    "chanquemos": "chancar",
    "chanquen": "chancar",
    "chanques": "chancar",
    "chanquetes": "chanquete",
    "chanqué": "chancar",
    "chanquéis": "chancar",
    "chansonnieres": "chansonnier",
    "chantajea": "chantajear",
    "chantajeaba": "chantajear",
    "chantajeabais": "chantajear",
    "chantajeaban": "chantajear",
    "chantajeabas": "chantajear",
    "chantajead": "chantajear",
    "chantajeada": "chantajear",
    "chantajeadas": "chantajear",
    "chantajeado": "chantajear",
    "chantajeadora": "chantajeador",
    "chantajeadoras": "chantajeador",
    "chantajeadores": "chantajeador",
    "chantajeados": "chantajear",
    "chantajeamos": "chantajear",
    "chantajean": "chantajear",
    "chantajeando": "chantajear",
    "chantajeara": "chantajear",
    "chantajearais": "chantajear",
    "chantajearan": "chantajear",
    "chantajearas": "chantajear",
    "chantajeare": "chantajear",
    "chantajeareis": "chantajear",
    "chantajearemos": "chantajear",
    "chantajearen": "chantajear",
    "chantajeares": "chantajear",
    "chantajearon": "chantajear",
    "chantajeará": "chantajear",
    "chantajearán": "chantajear",
    "chantajearás": "chantajear",
    "chantajearé": "chantajear",
    "chantajearéis": "chantajear",
    "chantajearía": "chantajear",
    "chantajearíais": "chantajear",
    "chantajearíamos": "chantajear",
    "chantajearían": "chantajear",
    "chantajearías": "chantajear",
    "chantajeas": "chantajear",
    "chantajease": "chantajear",
    "chantajeaseis": "chantajear",
    "chantajeasen": "chantajear",
    "chantajeases": "chantajear",
    "chantajeaste": "chantajear",
    "chantajeasteis": "chantajear",
    "chantajee": "chantajear",
    "chantajeemos": "chantajear",
    "chantajeen": "chantajear",
    "chantajees": "chantajear",
    "chantajeo": "chantajear",
    "chantajes": "chantaje",
    "chantajeábamos": "chantajear",
    "chantajeáis": "chantajear",
    "chantajeáramos": "chantajear",
    "chantajeáremos": "chantajear",
    "chantajeásemos": "chantajear",
    "chantajeé": "chantajear",
    "chantajeéis": "chantajear",
    "chantajeó": "chantajear",
    "chantajistas": "chantajista",
    "chantillones": "chantillón",
    "chantillíes": "chantillí",
    "chantillís": "chantillí",
    "chantres": "chantre",
    "chantrías": "chantría",
    "chantúes": "chantú",
    "chantús": "chantú",
    "chanzas": "chanza",
    "chanás": "chaná",
    "chapa": "chapar",
    "chapaba": "chapar",
    "chapabais": "chapar",
    "chapaban": "chapar",
    "chapabas": "chapar",
    "chapad": "chapar",
    "chapada": "chapar",
    "chapadas": "chapar",
    "chapado": "chapar",
    "chapados": "chapar",
    "chapalea": "chapalear",
    "chapaleaba": "chapalear",
    "chapaleabais": "chapalear",
    "chapaleaban": "chapalear",
    "chapaleabas": "chapalear",
    "chapalead": "chapalear",
    "chapaleada": "chapalear",
    "chapaleadas": "chapalear",
    "chapaleado": "chapalear",
    "chapaleados": "chapalear",
    "chapaleamos": "chapalear",
    "chapalean": "chapalear",
    "chapaleando": "chapalear",
    "chapaleara": "chapalear",
    "chapalearais": "chapalear",
    "chapalearan": "chapalear",
    "chapalearas": "chapalear",
    "chapaleare": "chapalear",
    "chapaleareis": "chapalear",
    "chapalearemos": "chapalear",
    "chapalearen": "chapalear",
    "chapaleares": "chapalear",
    "chapalearon": "chapalear",
    "chapaleará": "chapalear",
    "chapalearán": "chapalear",
    "chapalearás": "chapalear",
    "chapalearé": "chapalear",
    "chapalearéis": "chapalear",
    "chapalearía": "chapalear",
    "chapalearíais": "chapalear",
    "chapalearíamos": "chapalear",
    "chapalearían": "chapalear",
    "chapalearías": "chapalear",
    "chapaleas": "chapalear",
    "chapalease": "chapalear",
    "chapaleaseis": "chapalear",
    "chapaleasen": "chapalear",
    "chapaleases": "chapalear",
    "chapaleaste": "chapalear",
    "chapaleasteis": "chapalear",
    "chapalee": "chapalear",
    "chapaleemos": "chapalear",
    "chapaleen": "chapalear",
    "chapalees": "chapalear",
    "chapaleo": "chapalear",
    "chapaleos": "chapaleo",
    "chapaletas": "chapaleta",
    "chapaleábamos": "chapalear",
    "chapaleáis": "chapalear",
    "chapaleáramos": "chapalear",
    "chapaleáremos": "chapalear",
    "chapaleásemos": "chapalear",
    "chapaleé": "chapalear",
    "chapaleéis": "chapalear",
    "chapaleó": "chapalear",
    "chapamos": "chapar",
    "chapan": "chapar",
    "chapando": "chapar",
    "chapapotes": "chapapote",
    "chapara": "chapar",
    "chaparais": "chapar",
    "chaparan": "chapar",
    "chaparas": "chapar",
    "chapare": "chapar",
    "chapareis": "chapar",
    "chaparemos": "chapar",
    "chaparen": "chapar",
    "chapares": "chapar",
    "chaparon": "chapar",
    "chaparra": "chaparro",
    "chaparrales": "chaparral",
    "chaparras": "chaparro",
    "chaparrazos": "chaparrazo",
    "chaparrea": "chaparrear",
    "chaparreaba": "chaparrear",
    "chaparreabais": "chaparrear",
    "chaparreaban": "chaparrear",
    "chaparreabas": "chaparrear",
    "chaparread": "chaparrear",
    "chaparreada": "chaparrear",
    "chaparreadas": "chaparrear",
    "chaparreado": "chaparrear",
    "chaparreados": "chaparrear",
    "chaparreamos": "chaparrear",
    "chaparrean": "chaparrear",
    "chaparreando": "chaparrear",
    "chaparreara": "chaparrear",
    "chaparrearais": "chaparrear",
    "chaparrearan": "chaparrear",
    "chaparrearas": "chaparrear",
    "chaparreare": "chaparrear",
    "chaparreareis": "chaparrear",
    "chaparrearemos": "chaparrear",
    "chaparrearen": "chaparrear",
    "chaparreares": "chaparrear",
    "chaparrearon": "chaparrear",
    "chaparreará": "chaparrear",
    "chaparrearán": "chaparrear",
    "chaparrearás": "chaparrear",
    "chaparrearé": "chaparrear",
    "chaparrearéis": "chaparrear",
    "chaparrearía": "chaparrear",
    "chaparrearíais": "chaparrear",
    "chaparrearíamos": "chaparrear",
    "chaparrearían": "chaparrear",
    "chaparrearías": "chaparrear",
    "chaparreas": "chaparrear",
    "chaparrease": "chaparrear",
    "chaparreaseis": "chaparrear",
    "chaparreasen": "chaparrear",
    "chaparreases": "chaparrear",
    "chaparreaste": "chaparrear",
    "chaparreasteis": "chaparrear",
    "chaparree": "chaparrear",
    "chaparreemos": "chaparrear",
    "chaparreen": "chaparrear",
    "chaparrees": "chaparrear",
    "chaparreo": "chaparrear",
    "chaparreábamos": "chaparrear",
    "chaparreáis": "chaparrear",
    "chaparreáramos": "chaparrear",
    "chaparreáremos": "chaparrear",
    "chaparreásemos": "chaparrear",
    "chaparreé": "chaparrear",
    "chaparreéis": "chaparrear",
    "chaparreó": "chaparrear",
    "chaparrillos": "chaparrillo",
    "chaparrones": "chaparrón",
    "chaparros": "chaparro",
    "chapará": "chapar",
    "chaparán": "chapar",
    "chaparás": "chapar",
    "chaparé": "chapar",
    "chaparéis": "chapar",
    "chaparía": "chapar",
    "chaparíais": "chapar",
    "chaparíamos": "chapar",
    "chaparían": "chapar",
    "chaparías": "chapar",
    "chapas": "chapar",
    "chapase": "chapar",
    "chapaseis": "chapar",
    "chapasen": "chapar",
    "chapases": "chapar",
    "chapaste": "chapar",
    "chapasteis": "chapar",
    "chapatales": "chapatal",
    "chape": "chapar",
    "chapea": "chapear",
    "chapeaba": "chapear",
    "chapeabais": "chapear",
    "chapeaban": "chapear",
    "chapeabas": "chapear",
    "chapead": "chapear",
    "chapeada": "chapear",
    "chapeadas": "chapear",
    "chapeado": "chapear",
    "chapeados": "chapear",
    "chapeamos": "chapear",
    "chapean": "chapear",
    "chapeando": "chapear",
    "chapeara": "chapear",
    "chapearais": "chapear",
    "chapearan": "chapear",
    "chapearas": "chapear",
    "chapeare": "chapear",
    "chapeareis": "chapear",
    "chapearemos": "chapear",
    "chapearen": "chapear",
    "chapeares": "chapear",
    "chapearon": "chapear",
    "chapeará": "chapear",
    "chapearán": "chapear",
    "chapearás": "chapear",
    "chapearé": "chapear",
    "chapearéis": "chapear",
    "chapearía": "chapear",
    "chapearíais": "chapear",
    "chapearíamos": "chapear",
    "chapearían": "chapear",
    "chapearías": "chapear",
    "chapeas": "chapear",
    "chapease": "chapear",
    "chapeaseis": "chapear",
    "chapeasen": "chapear",
    "chapeases": "chapear",
    "chapeaste": "chapear",
    "chapeasteis": "chapear",
    "chapeca": "chapecar",
    "chapecaba": "chapecar",
    "chapecabais": "chapecar",
    "chapecaban": "chapecar",
    "chapecabas": "chapecar",
    "chapecad": "chapecar",
    "chapecada": "chapecar",
    "chapecadas": "chapecar",
    "chapecado": "chapecar",
    "chapecados": "chapecar",
    "chapecamos": "chapecar",
    "chapecan": "chapecar",
    "chapecando": "chapecar",
    "chapecara": "chapecar",
    "chapecarais": "chapecar",
    "chapecaran": "chapecar",
    "chapecaras": "chapecar",
    "chapecare": "chapecar",
    "chapecareis": "chapecar",
    "chapecaremos": "chapecar",
    "chapecaren": "chapecar",
    "chapecares": "chapecar",
    "chapecaron": "chapecar",
    "chapecará": "chapecar",
    "chapecarán": "chapecar",
    "chapecarás": "chapecar",
    "chapecaré": "chapecar",
    "chapecaréis": "chapecar",
    "chapecaría": "chapecar",
    "chapecaríais": "chapecar",
    "chapecaríamos": "chapecar",
    "chapecarían": "chapecar",
    "chapecarías": "chapecar",
    "chapecas": "chapecar",
    "chapecase": "chapecar",
    "chapecaseis": "chapecar",
    "chapecasen": "chapecar",
    "chapecases": "chapecar",
    "chapecaste": "chapecar",
    "chapecasteis": "chapecar",
    "chapeco": "chapecar",
    "chapecábamos": "chapecar",
    "chapecáis": "chapecar",
    "chapecáramos": "chapecar",
    "chapecáremos": "chapecar",
    "chapecásemos": "chapecar",
    "chapecó": "chapecar",
    "chapee": "chapear",
    "chapeemos": "chapear",
    "chapeen": "chapear",
    "chapees": "chapear",
    "chapelas": "chapela",
    "chapemos": "chapar",
    "chapen": "chapar",
    "chapeo": "chapear",
    "chapeos": "chapeo",
    "chapeque": "chapecar",
    "chapequemos": "chapecar",
    "chapequen": "chapecar",
    "chapeques": "chapecar",
    "chapequé": "chapecar",
    "chapequéis": "chapecar",
    "chaperas": "chapera",
    "chaperones": "chaperón",
    "chaperías": "chapería",
    "chapes": "chapar",
    "chapetas": "chapeta",
    "chapetona": "chapetón",
    "chapetonas": "chapetón",
    "chapetones": "chapetón",
    "chapeábamos": "chapear",
    "chapeáis": "chapear",
    "chapeáramos": "chapear",
    "chapeáremos": "chapear",
    "chapeásemos": "chapear",
    "chapeé": "chapear",
    "chapeéis": "chapear",
    "chapeó": "chapear",
    "chapinera": "chapinero",
    "chapineras": "chapinero",
    "chapineros": "chapinero",
    "chapinerías": "chapinería",
    "chapines": "chapín",
    "chapistas": "chapista",
    "chapisterías": "chapistería",
    "chapiteles": "chapitel",
    "chapo": "chapar",
    "chapoda": "chapodar",
    "chapodaba": "chapodar",
    "chapodabais": "chapodar",
    "chapodaban": "chapodar",
    "chapodabas": "chapodar",
    "chapodad": "chapodar",
    "chapodada": "chapodar",
    "chapodadas": "chapodar",
    "chapodado": "chapodar",
    "chapodados": "chapodar",
    "chapodamos": "chapodar",
    "chapodan": "chapodar",
    "chapodando": "chapodar",
    "chapodara": "chapodar",
    "chapodarais": "chapodar",
    "chapodaran": "chapodar",
    "chapodaras": "chapodar",
    "chapodare": "chapodar",
    "chapodareis": "chapodar",
    "chapodaremos": "chapodar",
    "chapodaren": "chapodar",
    "chapodares": "chapodar",
    "chapodaron": "chapodar",
    "chapodará": "chapodar",
    "chapodarán": "chapodar",
    "chapodarás": "chapodar",
    "chapodaré": "chapodar",
    "chapodaréis": "chapodar",
    "chapodaría": "chapodar",
    "chapodaríais": "chapodar",
    "chapodaríamos": "chapodar",
    "chapodarían": "chapodar",
    "chapodarías": "chapodar",
    "chapodas": "chapodar",
    "chapodase": "chapodar",
    "chapodaseis": "chapodar",
    "chapodasen": "chapodar",
    "chapodases": "chapodar",
    "chapodaste": "chapodar",
    "chapodasteis": "chapodar",
    "chapode": "chapodar",
    "chapodemos": "chapodar",
    "chapoden": "chapodar",
    "chapodes": "chapodar",
    "chapodo": "chapodar",
    "chapodos": "chapodo",
    "chapodábamos": "chapodar",
    "chapodáis": "chapodar",
    "chapodáramos": "chapodar",
    "chapodáremos": "chapodar",
    "chapodásemos": "chapodar",
    "chapodé": "chapodar",
    "chapodéis": "chapodar",
    "chapodó": "chapodar",
    "chapones": "chapón",
    "chapotea": "chapotear",
    "chapoteaba": "chapotear",
    "chapoteabais": "chapotear",
    "chapoteaban": "chapotear",
    "chapoteabas": "chapotear",
    "chapotead": "chapotear",
    "chapoteada": "chapotear",
    "chapoteadas": "chapotear",
    "chapoteado": "chapotear",
    "chapoteados": "chapotear",
    "chapoteamos": "chapotear",
    "chapotean": "chapotear",
    "chapoteando": "chapotear",
    "chapoteara": "chapotear",
    "chapotearais": "chapotear",
    "chapotearan": "chapotear",
    "chapotearas": "chapotear",
    "chapoteare": "chapotear",
    "chapoteareis": "chapotear",
    "chapotearemos": "chapotear",
    "chapotearen": "chapotear",
    "chapoteares": "chapotear",
    "chapotearon": "chapotear",
    "chapoteará": "chapotear",
    "chapotearán": "chapotear",
    "chapotearás": "chapotear",
    "chapotearé": "chapotear",
    "chapotearéis": "chapotear",
    "chapotearía": "chapotear",
    "chapotearíais": "chapotear",
    "chapotearíamos": "chapotear",
    "chapotearían": "chapotear",
    "chapotearías": "chapotear",
    "chapoteas": "chapotear",
    "chapotease": "chapotear",
    "chapoteaseis": "chapotear",
    "chapoteasen": "chapotear",
    "chapoteases": "chapotear",
    "chapoteaste": "chapotear",
    "chapoteasteis": "chapotear",
    "chapotee": "chapotear",
    "chapoteemos": "chapotear",
    "chapoteen": "chapotear",
    "chapotees": "chapotear",
    "chapoteo": "chapotear",
    "chapoteos": "chapoteo",
    "chapoteábamos": "chapotear",
    "chapoteáis": "chapotear",
    "chapoteáramos": "chapotear",
    "chapoteáremos": "chapotear",
    "chapoteásemos": "chapotear",
    "chapoteé": "chapotear",
    "chapoteéis": "chapotear",
    "chapoteó": "chapotear",
    "chapuce": "chapuzar",
    "chapucea": "chapucear",
    "chapuceaba": "chapucear",
    "chapuceabais": "chapucear",
    "chapuceaban": "chapucear",
    "chapuceabas": "chapucear",
    "chapucead": "chapucear",
    "chapuceada": "chapucear",
    "chapuceadas": "chapucear",
    "chapuceado": "chapucear",
    "chapuceados": "chapucear",
    "chapuceamos": "chapucear",
    "chapucean": "chapucear",
    "chapuceando": "chapucear",
    "chapuceara": "chapucear",
    "chapucearais": "chapucear",
    "chapucearan": "chapucear",
    "chapucearas": "chapucear",
    "chapuceare": "chapucear",
    "chapuceareis": "chapucear",
    "chapucearemos": "chapucear",
    "chapucearen": "chapucear",
    "chapuceares": "chapucear",
    "chapucearon": "chapucear",
    "chapuceará": "chapucear",
    "chapucearán": "chapucear",
    "chapucearás": "chapucear",
    "chapucearé": "chapucear",
    "chapucearéis": "chapucear",
    "chapucearía": "chapucear",
    "chapucearíais": "chapucear",
    "chapucearíamos": "chapucear",
    "chapucearían": "chapucear",
    "chapucearías": "chapucear",
    "chapuceas": "chapucear",
    "chapucease": "chapucear",
    "chapuceaseis": "chapucear",
    "chapuceasen": "chapucear",
    "chapuceases": "chapucear",
    "chapuceaste": "chapucear",
    "chapuceasteis": "chapucear",
    "chapucee": "chapucear",
    "chapuceemos": "chapucear",
    "chapuceen": "chapucear",
    "chapucees": "chapucear",
    "chapucemos": "chapuzar",
    "chapucen": "chapuzar",
    "chapuceo": "chapucear",
    "chapucera": "chapucero",
    "chapuceras": "chapucero",
    "chapuceros": "chapucero",
    "chapucerías": "chapucería",
    "chapuces": "chapuzar",
    "chapuceábamos": "chapucear",
    "chapuceáis": "chapucear",
    "chapuceáramos": "chapucear",
    "chapuceáremos": "chapucear",
    "chapuceásemos": "chapucear",
    "chapuceé": "chapucear",
    "chapuceéis": "chapucear",
    "chapuceó": "chapucear",
    "chapucé": "chapuzar",
    "chapucéis": "chapuzar",
    "chapulines": "chapulín",
    "chapurra": "chapurrar",
    "chapurraba": "chapurrar",
    "chapurrabais": "chapurrar",
    "chapurraban": "chapurrar",
    "chapurrabas": "chapurrar",
    "chapurrad": "chapurrar",
    "chapurrada": "chapurrar",
    "chapurradas": "chapurrar",
    "chapurrado": "chapurrar",
    "chapurrados": "chapurrar",
    "chapurramos": "chapurrar",
    "chapurran": "chapurrar",
    "chapurrando": "chapurrar",
    "chapurrara": "chapurrar",
    "chapurrarais": "chapurrar",
    "chapurraran": "chapurrar",
    "chapurraras": "chapurrar",
    "chapurrare": "chapurrar",
    "chapurrareis": "chapurrar",
    "chapurraremos": "chapurrar",
    "chapurraren": "chapurrar",
    "chapurrares": "chapurrar",
    "chapurraron": "chapurrar",
    "chapurrará": "chapurrar",
    "chapurrarán": "chapurrar",
    "chapurrarás": "chapurrar",
    "chapurraré": "chapurrar",
    "chapurraréis": "chapurrar",
    "chapurraría": "chapurrar",
    "chapurraríais": "chapurrar",
    "chapurraríamos": "chapurrar",
    "chapurrarían": "chapurrar",
    "chapurrarías": "chapurrar",
    "chapurras": "chapurrar",
    "chapurrase": "chapurrar",
    "chapurraseis": "chapurrar",
    "chapurrasen": "chapurrar",
    "chapurrases": "chapurrar",
    "chapurraste": "chapurrar",
    "chapurrasteis": "chapurrar",
    "chapurre": "chapurrar",
    "chapurrea": "chapurrear",
    "chapurreaba": "chapurrear",
    "chapurreabais": "chapurrear",
    "chapurreaban": "chapurrear",
    "chapurreabas": "chapurrear",
    "chapurread": "chapurrear",
    "chapurreada": "chapurrear",
    "chapurreadas": "chapurrear",
    "chapurreado": "chapurrear",
    "chapurreados": "chapurrear",
    "chapurreamos": "chapurrear",
    "chapurrean": "chapurrear",
    "chapurreando": "chapurrear",
    "chapurreara": "chapurrear",
    "chapurrearais": "chapurrear",
    "chapurrearan": "chapurrear",
    "chapurrearas": "chapurrear",
    "chapurreare": "chapurrear",
    "chapurreareis": "chapurrear",
    "chapurrearemos": "chapurrear",
    "chapurrearen": "chapurrear",
    "chapurreares": "chapurrear",
    "chapurrearon": "chapurrear",
    "chapurreará": "chapurrear",
    "chapurrearán": "chapurrear",
    "chapurrearás": "chapurrear",
    "chapurrearé": "chapurrear",
    "chapurrearéis": "chapurrear",
    "chapurrearía": "chapurrear",
    "chapurrearíais": "chapurrear",
    "chapurrearíamos": "chapurrear",
    "chapurrearían": "chapurrear",
    "chapurrearías": "chapurrear",
    "chapurreas": "chapurrear",
    "chapurrease": "chapurrear",
    "chapurreaseis": "chapurrear",
    "chapurreasen": "chapurrear",
    "chapurreases": "chapurrear",
    "chapurreaste": "chapurrear",
    "chapurreasteis": "chapurrear",
    "chapurree": "chapurrear",
    "chapurreemos": "chapurrear",
    "chapurreen": "chapurrear",
    "chapurrees": "chapurrear",
    "chapurremos": "chapurrar",
    "chapurren": "chapurrar",
    "chapurreo": "chapurrear",
    "chapurreos": "chapurreo",
    "chapurres": "chapurrar",
    "chapurreábamos": "chapurrear",
    "chapurreáis": "chapurrear",
    "chapurreáramos": "chapurrear",
    "chapurreáremos": "chapurrear",
    "chapurreásemos": "chapurrear",
    "chapurreé": "chapurrear",
    "chapurreéis": "chapurrear",
    "chapurreó": "chapurrear",
    "chapurro": "chapurrar",
    "chapurrábamos": "chapurrar",
    "chapurráis": "chapurrar",
    "chapurráramos": "chapurrar",
    "chapurráremos": "chapurrar",
    "chapurrásemos": "chapurrar",
    "chapurré": "chapurrar",
    "chapurréis": "chapurrar",
    "chapurró": "chapurrar",
    "chapuza": "chapuzar",
    "chapuzaba": "chapuzar",
    "chapuzabais": "chapuzar",
    "chapuzaban": "chapuzar",
    "chapuzabas": "chapuzar",
    "chapuzad": "chapuzar",
    "chapuzada": "chapuzar",
    "chapuzadas": "chapuzar",
    "chapuzado": "chapuzar",
    "chapuzados": "chapuzar",
    "chapuzamos": "chapuzar",
    "chapuzan": "chapuzar",
    "chapuzando": "chapuzar",
    "chapuzara": "chapuzar",
    "chapuzarais": "chapuzar",
    "chapuzaran": "chapuzar",
    "chapuzaras": "chapuzar",
    "chapuzare": "chapuzar",
    "chapuzareis": "chapuzar",
    "chapuzaremos": "chapuzar",
    "chapuzaren": "chapuzar",
    "chapuzares": "chapuzar",
    "chapuzaron": "chapuzar",
    "chapuzará": "chapuzar",
    "chapuzarán": "chapuzar",
    "chapuzarás": "chapuzar",
    "chapuzaré": "chapuzar",
    "chapuzaréis": "chapuzar",
    "chapuzaría": "chapuzar",
    "chapuzaríais": "chapuzar",
    "chapuzaríamos": "chapuzar",
    "chapuzarían": "chapuzar",
    "chapuzarías": "chapuzar",
    "chapuzas": "chapuzar",
    "chapuzase": "chapuzar",
    "chapuzaseis": "chapuzar",
    "chapuzasen": "chapuzar",
    "chapuzases": "chapuzar",
    "chapuzaste": "chapuzar",
    "chapuzasteis": "chapuzar",
    "chapuzo": "chapuzar",
    "chapuzones": "chapuzón",
    "chapuzábamos": "chapuzar",
    "chapuzáis": "chapuzar",
    "chapuzáramos": "chapuzar",
    "chapuzáremos": "chapuzar",
    "chapuzásemos": "chapuzar",
    "chapuzó": "chapuzar",
    "chapábamos": "chapar",
    "chapáis": "chapar",
    "chapáramos": "chapar",
    "chapáremos": "chapar",
    "chapásemos": "chapar",
    "chapé": "chapar",
    "chapéis": "chapar",
    "chapó": "chapar",
    "chapós": "chapó",
    "chaquetas": "chaqueta",
    "chaquetea": "chaquetear",
    "chaqueteaba": "chaquetear",
    "chaqueteabais": "chaquetear",
    "chaqueteaban": "chaquetear",
    "chaqueteabas": "chaquetear",
    "chaquetead": "chaquetear",
    "chaqueteada": "chaquetear",
    "chaqueteadas": "chaquetear",
    "chaqueteado": "chaquetear",
    "chaqueteados": "chaquetear",
    "chaqueteamos": "chaquetear",
    "chaquetean": "chaquetear",
    "chaqueteando": "chaquetear",
    "chaqueteara": "chaquetear",
    "chaquetearais": "chaquetear",
    "chaquetearan": "chaquetear",
    "chaquetearas": "chaquetear",
    "chaqueteare": "chaquetear",
    "chaqueteareis": "chaquetear",
    "chaquetearemos": "chaquetear",
    "chaquetearen": "chaquetear",
    "chaqueteares": "chaquetear",
    "chaquetearon": "chaquetear",
    "chaqueteará": "chaquetear",
    "chaquetearán": "chaquetear",
    "chaquetearás": "chaquetear",
    "chaquetearé": "chaquetear",
    "chaquetearéis": "chaquetear",
    "chaquetearía": "chaquetear",
    "chaquetearíais": "chaquetear",
    "chaquetearíamos": "chaquetear",
    "chaquetearían": "chaquetear",
    "chaquetearías": "chaquetear",
    "chaqueteas": "chaquetear",
    "chaquetease": "chaquetear",
    "chaqueteaseis": "chaquetear",
    "chaqueteasen": "chaquetear",
    "chaqueteases": "chaquetear",
    "chaqueteaste": "chaquetear",
    "chaqueteasteis": "chaquetear",
    "chaquetee": "chaquetear",
    "chaqueteemos": "chaquetear",
    "chaqueteen": "chaquetear",
    "chaquetees": "chaquetear",
    "chaqueteo": "chaquetear",
    "chaqueteos": "chaqueteo",
    "chaquetera": "chaquetero",
    "chaqueteras": "chaquetero",
    "chaqueteros": "chaquetero",
    "chaquetes": "chaquete",
    "chaqueteábamos": "chaquetear",
    "chaqueteáis": "chaquetear",
    "chaqueteáramos": "chaquetear",
    "chaqueteáremos": "chaquetear",
    "chaqueteásemos": "chaquetear",
    "chaqueteé": "chaquetear",
    "chaqueteéis": "chaquetear",
    "chaqueteó": "chaquetear",
    "chaquetillas": "chaquetilla",
    "chaquetones": "chaquetón",
    "chaquiras": "chaquira",
    "chaqués": "chaqué",
    "charabanes": "charabán",
    "charadas": "charada",
    "charangas": "charanga",
    "charangos": "charango",
    "charanguera": "charanguero",
    "charangueras": "charanguero",
    "charangueros": "charanguero",
    "charatas": "charata",
    "charcales": "charcal",
    "charcas": "charca",
    "charcos": "charco",
    "charcutera": "charcutero",
    "charcuteras": "charcutero",
    "charcuteros": "charcutero",
    "charcuterías": "charcutería",
    "charla": "charlar",
    "charlaba": "charlar",
    "charlabais": "charlar",
    "charlaban": "charlar",
    "charlabas": "charlar",
    "charlad": "charlar",
    "charlada": "charlar",
    "charladas": "charlar",
    "charlado": "charlar",
    "charladora": "charlador",
    "charladoras": "charlador",
    "charladores": "charlador",
    "charlados": "charlar",
    "charladurías": "charladuría",
    "charlamos": "charlar",
    "charlan": "charlar",
    "charlando": "charlar",
    "charlara": "charlar",
    "charlarais": "charlar",
    "charlaran": "charlar",
    "charlaras": "charlar",
    "charlare": "charlar",
    "charlareis": "charlar",
    "charlaremos": "charlar",
    "charlaren": "charlar",
    "charlares": "charlar",
    "charlaron": "charlar",
    "charlará": "charlar",
    "charlarán": "charlar",
    "charlarás": "charlar",
    "charlaré": "charlar",
    "charlaréis": "charlar",
    "charlaría": "charlar",
    "charlaríais": "charlar",
    "charlaríamos": "charlar",
    "charlarían": "charlar",
    "charlarías": "charlar",
    "charlas": "charlar",
    "charlase": "charlar",
    "charlaseis": "charlar",
    "charlasen": "charlar",
    "charlases": "charlar",
    "charlaste": "charlar",
    "charlasteis": "charlar",
    "charlatana": "charlatán",
    "charlatanas": "charlatán",
    "charlatanea": "charlatanear",
    "charlataneaba": "charlatanear",
    "charlataneabais": "charlatanear",
    "charlataneaban": "charlatanear",
    "charlataneabas": "charlatanear",
    "charlatanead": "charlatanear",
    "charlataneada": "charlatanear",
    "charlataneadas": "charlatanear",
    "charlataneado": "charlatanear",
    "charlataneados": "charlatanear",
    "charlataneamos": "charlatanear",
    "charlatanean": "charlatanear",
    "charlataneando": "charlatanear",
    "charlataneara": "charlatanear",
    "charlatanearais": "charlatanear",
    "charlatanearan": "charlatanear",
    "charlatanearas": "charlatanear",
    "charlataneare": "charlatanear",
    "charlataneareis": "charlatanear",
    "charlatanearemos": "charlatanear",
    "charlatanearen": "charlatanear",
    "charlataneares": "charlatanear",
    "charlatanearon": "charlatanear",
    "charlataneará": "charlatanear",
    "charlatanearán": "charlatanear",
    "charlatanearás": "charlatanear",
    "charlatanearé": "charlatanear",
    "charlatanearéis": "charlatanear",
    "charlatanearía": "charlatanear",
    "charlatanearíais": "charlatanear",
    "charlatanearíamos": "charlatanear",
    "charlatanearían": "charlatanear",
    "charlatanearías": "charlatanear",
    "charlataneas": "charlatanear",
    "charlatanease": "charlatanear",
    "charlataneaseis": "charlatanear",
    "charlataneasen": "charlatanear",
    "charlataneases": "charlatanear",
    "charlataneaste": "charlatanear",
    "charlataneasteis": "charlatanear",
    "charlatanee": "charlatanear",
    "charlataneemos": "charlatanear",
    "charlataneen": "charlatanear",
    "charlatanees": "charlatanear",
    "charlataneo": "charlatanear",
    "charlatanerías": "charlatanería",
    "charlatanes": "charlatán",
    "charlataneábamos": "charlatanear",
    "charlataneáis": "charlatanear",
    "charlataneáramos": "charlatanear",
    "charlataneáremos": "charlatanear",
    "charlataneásemos": "charlatanear",
    "charlataneé": "charlatanear",
    "charlataneéis": "charlatanear",
    "charlataneó": "charlatanear",
    "charle": "charlar",
    "charlemos": "charlar",
    "charlen": "charlar",
    "charles": "charlar",
    "charlo": "charlar",
    "charlotadas": "charlotada",
    "charlotea": "charlotear",
    "charloteaba": "charlotear",
    "charloteabais": "charlotear",
    "charloteaban": "charlotear",
    "charloteabas": "charlotear",
    "charlotead": "charlotear",
    "charloteada": "charlotear",
    "charloteadas": "charlotear",
    "charloteado": "charlotear",
    "charloteados": "charlotear",
    "charloteamos": "charlotear",
    "charlotean": "charlotear",
    "charloteando": "charlotear",
    "charloteara": "charlotear",
    "charlotearais": "charlotear",
    "charlotearan": "charlotear",
    "charlotearas": "charlotear",
    "charloteare": "charlotear",
    "charloteareis": "charlotear",
    "charlotearemos": "charlotear",
    "charlotearen": "charlotear",
    "charloteares": "charlotear",
    "charlotearon": "charlotear",
    "charloteará": "charlotear",
    "charlotearán": "charlotear",
    "charlotearás": "charlotear",
    "charlotearé": "charlotear",
    "charlotearéis": "charlotear",
    "charlotearía": "charlotear",
    "charlotearíais": "charlotear",
    "charlotearíamos": "charlotear",
    "charlotearían": "charlotear",
    "charlotearías": "charlotear",
    "charloteas": "charlotear",
    "charlotease": "charlotear",
    "charloteaseis": "charlotear",
    "charloteasen": "charlotear",
    "charloteases": "charlotear",
    "charloteaste": "charlotear",
    "charloteasteis": "charlotear",
    "charlotee": "charlotear",
    "charloteemos": "charlotear",
    "charloteen": "charlotear",
    "charlotees": "charlotear",
    "charloteo": "charlotear",
    "charloteos": "charloteo",
    "charloteábamos": "charlotear",
    "charloteáis": "charlotear",
    "charloteáramos": "charlotear",
    "charloteáremos": "charlotear",
    "charloteásemos": "charlotear",
    "charloteé": "charlotear",
    "charloteéis": "charlotear",
    "charloteó": "charlotear",
    "charlábamos": "charlar",
    "charláis": "charlar",
    "charláramos": "charlar",
    "charláremos": "charlar",
    "charlásemos": "charlar",
    "charlé": "charlar",
    "charléis": "charlar",
    "charló": "charlar",
    "charnega": "charnego",
    "charnegas": "charnego",
    "charnegos": "charnego",
    "charnelas": "charnela",
    "charola": "charolar",
    "charolaba": "charolar",
    "charolabais": "charolar",
    "charolaban": "charolar",
    "charolabas": "charolar",
    "charolad": "charolar",
    "charolada": "charolar",
    "charoladas": "charolar",
    "charolado": "charolar",
    "charoladora": "charolador",
    "charoladoras": "charolador",
    "charoladores": "charolador",
    "charolados": "charolar",
    "charolamos": "charolar",
    "charolan": "charolar",
    "charolando": "charolar",
    "charolara": "charolar",
    "charolarais": "charolar",
    "charolaran": "charolar",
    "charolaras": "charolar",
    "charolare": "charolar",
    "charolareis": "charolar",
    "charolaremos": "charolar",
    "charolaren": "charolar",
    "charolares": "charolar",
    "charolaron": "charolar",
    "charolará": "charolar",
    "charolarán": "charolar",
    "charolarás": "charolar",
    "charolaré": "charolar",
    "charolaréis": "charolar",
    "charolaría": "charolar",
    "charolaríais": "charolar",
    "charolaríamos": "charolar",
    "charolarían": "charolar",
    "charolarías": "charolar",
    "charolas": "charolar",
    "charolase": "charolar",
    "charolaseis": "charolar",
    "charolasen": "charolar",
    "charolases": "charolar",
    "charolaste": "charolar",
    "charolasteis": "charolar",
    "charole": "charolar",
    "charolemos": "charolar",
    "charolen": "charolar",
    "charoles": "charolar",
    "charolistas": "charolista",
    "charolo": "charolar",
    "charolábamos": "charolar",
    "charoláis": "charolar",
    "charoláramos": "charolar",
    "charoláremos": "charolar",
    "charolásemos": "charolar",
    "charolé": "charolar",
    "charoléis": "charolar",
    "charoló": "charolar",
    "charpas": "charpa",
    "charquea": "charquear",
    "charqueaba": "charquear",
    "charqueabais": "charquear",
    "charqueaban": "charquear",
    "charqueabas": "charquear",
    "charquead": "charquear",
    "charqueada": "charquear",
    "charqueadas": "charquear",
    "charqueado": "charquear",
    "charqueados": "charquear",
    "charqueamos": "charquear",
    "charquean": "charquear",
    "charqueando": "charquear",
    "charqueara": "charquear",
    "charquearais": "charquear",
    "charquearan": "charquear",
    "charquearas": "charquear",
    "charqueare": "charquear",
    "charqueareis": "charquear",
    "charquearemos": "charquear",
    "charquearen": "charquear",
    "charqueares": "charquear",
    "charquearon": "charquear",
    "charqueará": "charquear",
    "charquearán": "charquear",
    "charquearás": "charquear",
    "charquearé": "charquear",
    "charquearéis": "charquear",
    "charquearía": "charquear",
    "charquearíais": "charquear",
    "charquearíamos": "charquear",
    "charquearían": "charquear",
    "charquearías": "charquear",
    "charqueas": "charquear",
    "charquease": "charquear",
    "charqueaseis": "charquear",
    "charqueasen": "charquear",
    "charqueases": "charquear",
    "charqueaste": "charquear",
    "charqueasteis": "charquear",
    "charquee": "charquear",
    "charqueemos": "charquear",
    "charqueen": "charquear",
    "charquees": "charquear",
    "charqueo": "charquear",
    "charques": "charque",
    "charqueábamos": "charquear",
    "charqueáis": "charquear",
    "charqueáramos": "charquear",
    "charqueáremos": "charquear",
    "charqueásemos": "charquear",
    "charqueé": "charquear",
    "charqueéis": "charquear",
    "charqueó": "charquear",
    "charquicanes": "charquicán",
    "charquis": "charqui",
    "charra": "charro",
    "charradas": "charrada",
    "charrales": "charral",
    "charrana": "charrán",
    "charranadas": "charranada",
    "charranas": "charrán",
    "charrancitos": "charrancito",
    "charranea": "charranear",
    "charraneaba": "charranear",
    "charraneabais": "charranear",
    "charraneaban": "charranear",
    "charraneabas": "charranear",
    "charranead": "charranear",
    "charraneada": "charranear",
    "charraneadas": "charranear",
    "charraneado": "charranear",
    "charraneados": "charranear",
    "charraneamos": "charranear",
    "charranean": "charranear",
    "charraneando": "charranear",
    "charraneara": "charranear",
    "charranearais": "charranear",
    "charranearan": "charranear",
    "charranearas": "charranear",
    "charraneare": "charranear",
    "charraneareis": "charranear",
    "charranearemos": "charranear",
    "charranearen": "charranear",
    "charraneares": "charranear",
    "charranearon": "charranear",
    "charraneará": "charranear",
    "charranearán": "charranear",
    "charranearás": "charranear",
    "charranearé": "charranear",
    "charranearéis": "charranear",
    "charranearía": "charranear",
    "charranearíais": "charranear",
    "charranearíamos": "charranear",
    "charranearían": "charranear",
    "charranearías": "charranear",
    "charraneas": "charranear",
    "charranease": "charranear",
    "charraneaseis": "charranear",
    "charraneasen": "charranear",
    "charraneases": "charranear",
    "charraneaste": "charranear",
    "charraneasteis": "charranear",
    "charranee": "charranear",
    "charraneemos": "charranear",
    "charraneen": "charranear",
    "charranees": "charranear",
    "charraneo": "charranear",
    "charranerías": "charranería",
    "charranes": "charrán",
    "charraneábamos": "charranear",
    "charraneáis": "charranear",
    "charraneáramos": "charranear",
    "charraneáremos": "charranear",
    "charraneásemos": "charranear",
    "charraneé": "charranear",
    "charraneéis": "charranear",
    "charraneó": "charranear",
    "charras": "charro",
    "charrascas": "charrasca",
    "charrascos": "charrasco",
    "charrerías": "charrería",
    "charreteras": "charretera",
    "charretes": "charrete",
    "charros": "charro",
    "charrúas": "charrúa",
    "charterses": "charter",
    "chartreuses": "chartreuse",
    "chas": "cha",
    "chasca": "chascar",
    "chascaba": "chascar",
    "chascabais": "chascar",
    "chascaban": "chascar",
    "chascabas": "chascar",
    "chascad": "chascar",
    "chascada": "chascar",
    "chascadas": "chascar",
    "chascado": "chascar",
    "chascados": "chascar",
    "chascamos": "chascar",
    "chascan": "chascar",
    "chascando": "chascar",
    "chascara": "chascar",
    "chascarais": "chascar",
    "chascaran": "chascar",
    "chascaras": "chascar",
    "chascare": "chascar",
    "chascareis": "chascar",
    "chascaremos": "chascar",
    "chascaren": "chascar",
    "chascares": "chascar",
    "chascaron": "chascar",
    "chascarrillos": "chascarrillo",
    "chascarros": "chascarro",
    "chascará": "chascar",
    "chascarán": "chascar",
    "chascarás": "chascar",
    "chascaré": "chascar",
    "chascaréis": "chascar",
    "chascaría": "chascar",
    "chascaríais": "chascar",
    "chascaríamos": "chascar",
    "chascarían": "chascar",
    "chascarías": "chascar",
    "chascas": "chascar",
    "chascase": "chascar",
    "chascaseis": "chascar",
    "chascasen": "chascar",
    "chascases": "chascar",
    "chascaste": "chascar",
    "chascasteis": "chascar",
    "chasco": "chascar",
    "chascos": "chasco",
    "chascábamos": "chascar",
    "chascáis": "chascar",
    "chascáramos": "chascar",
    "chascáremos": "chascar",
    "chascásemos": "chascar",
    "chascó": "chascar",
    "chaspes": "chaspe",
    "chasponazos": "chasponazo",
    "chasque": "chascar",
    "chasquea": "chasquear",
    "chasqueaba": "chasquear",
    "chasqueabais": "chasquear",
    "chasqueaban": "chasquear",
    "chasqueabas": "chasquear",
    "chasquead": "chasquear",
    "chasqueada": "chasquear",
    "chasqueadas": "chasquear",
    "chasqueado": "chasquear",
    "chasqueados": "chasquear",
    "chasqueamos": "chasquear",
    "chasquean": "chasquear",
    "chasqueando": "chasquear",
    "chasqueara": "chasquear",
    "chasquearais": "chasquear",
    "chasquearan": "chasquear",
    "chasquearas": "chasquear",
    "chasqueare": "chasquear",
    "chasqueareis": "chasquear",
    "chasquearemos": "chasquear",
    "chasquearen": "chasquear",
    "chasqueares": "chasquear",
    "chasquearon": "chasquear",
    "chasqueará": "chasquear",
    "chasquearán": "chasquear",
    "chasquearás": "chasquear",
    "chasquearé": "chasquear",
    "chasquearéis": "chasquear",
    "chasquearía": "chasquear",
    "chasquearíais": "chasquear",
    "chasquearíamos": "chasquear",
    "chasquearían": "chasquear",
    "chasquearías": "chasquear",
    "chasqueas": "chasquear",
    "chasquease": "chasquear",
    "chasqueaseis": "chasquear",
    "chasqueasen": "chasquear",
    "chasqueases": "chasquear",
    "chasqueaste": "chasquear",
    "chasqueasteis": "chasquear",
    "chasquee": "chasquear",
    "chasqueemos": "chasquear",
    "chasqueen": "chasquear",
    "chasquees": "chasquear",
    "chasquemos": "chascar",
    "chasquen": "chascar",
    "chasqueo": "chasquear",
    "chasqueos": "chasqueo",
    "chasques": "chascar",
    "chasqueábamos": "chasquear",
    "chasqueáis": "chasquear",
    "chasqueáramos": "chasquear",
    "chasqueáremos": "chasquear",
    "chasqueásemos": "chasquear",
    "chasqueé": "chasquear",
    "chasqueéis": "chasquear",
    "chasqueó": "chasquear",
    "chasquidos": "chasquido",
    "chasqué": "chascar",
    "chasquéis": "chascar",
    "chasses": "chassé",
    "chata": "chato",
    "chatarras": "chatarra",
    "chatarrera": "chatarrero",
    "chatarreras": "chatarrero",
    "chatarreros": "chatarrero",
    "chatarrerías": "chatarrería",
    "chatas": "chato",
    "chatedades": "chatedad",
    "chateos": "chateo",
    "chatona": "chatón",
    "chatonas": "chatón",
    "chatones": "chatón",
    "chatos": "chato",
    "chatunga": "chatungo",
    "chatungas": "chatungo",
    "chatungos": "chatungo",
    "chauvinismos": "chauvinismo",
    "chauvinistas": "chauvinista",
    "chavala": "chaval",
    "chavalas": "chaval",
    "chavales": "chaval",
    "chavalines": "chavalín",
    "chaveas": "chavea",
    "chavetas": "chaveta",
    "chaveteros": "chavetero",
    "chavolas": "chavola",
    "chavos": "chavo",
    "chayoteras": "chayotera",
    "chayotes": "chayote",
    "chaza": "chazar",
    "chazaba": "chazar",
    "chazabais": "chazar",
    "chazaban": "chazar",
    "chazabas": "chazar",
    "chazad": "chazar",
    "chazada": "chazar",
    "chazadas": "chazar",
    "chazado": "chazar",
    "chazadora": "chazador",
    "chazadoras": "chazador",
    "chazadores": "chazador",
    "chazados": "chazar",
    "chazamos": "chazar",
    "chazan": "chazar",
    "chazando": "chazar",
    "chazara": "chazar",
    "chazarais": "chazar",
    "chazaran": "chazar",
    "chazaras": "chazar",
    "chazare": "chazar",
    "chazareis": "chazar",
    "chazaremos": "chazar",
    "chazaren": "chazar",
    "chazares": "chazar",
    "chazaron": "chazar",
    "chazará": "chazar",
    "chazarán": "chazar",
    "chazarás": "chazar",
    "chazaré": "chazar",
    "chazaréis": "chazar",
    "chazaría": "chazar",
    "chazaríais": "chazar",
    "chazaríamos": "chazar",
    "chazarían": "chazar",
    "chazarías": "chazar",
    "chazas": "chazar",
    "chazase": "chazar",
    "chazaseis": "chazar",
    "chazasen": "chazar",
    "chazases": "chazar",
    "chazaste": "chazar",
    "chazasteis": "chazar",
    "chazo": "chazar",
    "chazábamos": "chazar",
    "chazáis": "chazar",
    "chazáramos": "chazar",
    "chazáremos": "chazar",
    "chazásemos": "chazar",
    "chazó": "chazar",
    "chañares": "chañar",
    "checa": "checo",
    "checas": "checo",
    "chechena": "checheno",
    "chechenas": "checheno",
    "chechenia": "chechenio",
    "chechenias": "chechenio",
    "chechenios": "chechenio",
    "chechenos": "checheno",
    "checos": "checo",
    "checoslovaca": "checoslovaco",
    "checoslovacas": "checoslovaco",
    "checoslovacos": "checoslovaco",
    "cheerleaders": "cheerleader",
    "cheiras": "cheira",
    "chelines": "chelín",
    "chellines": "chellín",
    "chelos": "chelo",
    "chenillas": "chenilla",
    "chepas": "chepa",
    "cheposa": "cheposo",
    "cheposas": "cheposo",
    "cheposos": "cheposo",
    "chequeos": "chequeo",
    "cheques": "cheque",
    "chercanes": "chercán",
    "chercha": "cherchar",
    "cherchaba": "cherchar",
    "cherchabais": "cherchar",
    "cherchaban": "cherchar",
    "cherchabas": "cherchar",
    "cherchad": "cherchar",
    "cherchada": "cherchar",
    "cherchadas": "cherchar",
    "cherchado": "cherchar",
    "cherchados": "cherchar",
    "cherchamos": "cherchar",
    "cherchan": "cherchar",
    "cherchando": "cherchar",
    "cherchara": "cherchar",
    "chercharais": "cherchar",
    "chercharan": "cherchar",
    "chercharas": "cherchar",
    "cherchare": "cherchar",
    "cherchareis": "cherchar",
    "chercharemos": "cherchar",
    "chercharen": "cherchar",
    "cherchares": "cherchar",
    "chercharon": "cherchar",
    "cherchará": "cherchar",
    "chercharán": "cherchar",
    "chercharás": "cherchar",
    "chercharé": "cherchar",
    "chercharéis": "cherchar",
    "chercharía": "cherchar",
    "chercharíais": "cherchar",
    "chercharíamos": "cherchar",
    "chercharían": "cherchar",
    "chercharías": "cherchar",
    "cherchas": "cherchar",
    "cherchase": "cherchar",
    "cherchaseis": "cherchar",
    "cherchasen": "cherchar",
    "cherchases": "cherchar",
    "cherchaste": "cherchar",
    "cherchasteis": "cherchar",
    "cherche": "cherchar",
    "cherchemos": "cherchar",
    "cherchen": "cherchar",
    "cherches": "cherchar",
    "chercho": "cherchar",
    "cherchábamos": "cherchar",
    "chercháis": "cherchar",
    "chercháramos": "cherchar",
    "chercháremos": "cherchar",
    "cherchásemos": "cherchar",
    "cherché": "cherchar",
    "cherchéis": "cherchar",
    "cherchó": "cherchar",
    "cherkés": "cherké",
    "chernas": "cherna",
    "chernocienes": "chernocién",
    "cheroquis": "cheroqui",
    "ches": "ch",
    "cheviotes": "cheviot",
    "cheviots": "cheviot",
    "cheyenes": "cheyene",
    "chiantis": "chianti",
    "chibaletes": "chibalete",
    "chibchas": "chibcha",
    "chibola": "chibolo",
    "chibolas": "chibolo",
    "chibolos": "chibolo",
    "chibuquíes": "chibuquí",
    "chibuquís": "chibuquí",
    "chica": "chico",
    "chicadas": "chicada",
    "chicana": "chicano",
    "chicanas": "chicano",
    "chicanea": "chicanear",
    "chicaneaba": "chicanear",
    "chicaneabais": "chicanear",
    "chicaneaban": "chicanear",
    "chicaneabas": "chicanear",
    "chicanead": "chicanear",
    "chicaneada": "chicanear",
    "chicaneadas": "chicanear",
    "chicaneado": "chicanear",
    "chicaneados": "chicanear",
    "chicaneamos": "chicanear",
    "chicanean": "chicanear",
    "chicaneando": "chicanear",
    "chicaneara": "chicanear",
    "chicanearais": "chicanear",
    "chicanearan": "chicanear",
    "chicanearas": "chicanear",
    "chicaneare": "chicanear",
    "chicaneareis": "chicanear",
    "chicanearemos": "chicanear",
    "chicanearen": "chicanear",
    "chicaneares": "chicanear",
    "chicanearon": "chicanear",
    "chicaneará": "chicanear",
    "chicanearán": "chicanear",
    "chicanearás": "chicanear",
    "chicanearé": "chicanear",
    "chicanearéis": "chicanear",
    "chicanearía": "chicanear",
    "chicanearíais": "chicanear",
    "chicanearíamos": "chicanear",
    "chicanearían": "chicanear",
    "chicanearías": "chicanear",
    "chicaneas": "chicanear",
    "chicanease": "chicanear",
    "chicaneaseis": "chicanear",
    "chicaneasen": "chicanear",
    "chicaneases": "chicanear",
    "chicaneaste": "chicanear",
    "chicaneasteis": "chicanear",
    "chicanee": "chicanear",
    "chicaneemos": "chicanear",
    "chicaneen": "chicanear",
    "chicanees": "chicanear",
    "chicaneo": "chicanear",
    "chicaneábamos": "chicanear",
    "chicaneáis": "chicanear",
    "chicaneáramos": "chicanear",
    "chicaneáremos": "chicanear",
    "chicaneásemos": "chicanear",
    "chicaneé": "chicanear",
    "chicaneéis": "chicanear",
    "chicaneó": "chicanear",
    "chicanos": "chicano",
    "chicarrona": "chicarrón",
    "chicarronas": "chicarrón",
    "chicarrones": "chicarrón",
    "chicas": "chico",
    "chicharras": "chicharra",
    "chicharrera": "chicharrero",
    "chicharreras": "chicharrero",
    "chicharreros": "chicharrero",
    "chicharrones": "chicharrón",
    "chicharros": "chicharro",
    "chichas": "chicha",
    "chichea": "chichear",
    "chicheaba": "chichear",
    "chicheabais": "chichear",
    "chicheaban": "chichear",
    "chicheabas": "chichear",
    "chichead": "chichear",
    "chicheada": "chichear",
    "chicheadas": "chichear",
    "chicheado": "chichear",
    "chicheados": "chichear",
    "chicheamos": "chichear",
    "chichean": "chichear",
    "chicheando": "chichear",
    "chicheara": "chichear",
    "chichearais": "chichear",
    "chichearan": "chichear",
    "chichearas": "chichear",
    "chicheare": "chichear",
    "chicheareis": "chichear",
    "chichearemos": "chichear",
    "chichearen": "chichear",
    "chicheares": "chichear",
    "chichearon": "chichear",
    "chicheará": "chichear",
    "chichearán": "chichear",
    "chichearás": "chichear",
    "chichearé": "chichear",
    "chichearéis": "chichear",
    "chichearía": "chichear",
    "chichearíais": "chichear",
    "chichearíamos": "chichear",
    "chichearían": "chichear",
    "chichearías": "chichear",
    "chicheas": "chichear",
    "chichease": "chichear",
    "chicheaseis": "chichear",
    "chicheasen": "chichear",
    "chicheases": "chichear",
    "chicheaste": "chichear",
    "chicheasteis": "chichear",
    "chichee": "chichear",
    "chicheemos": "chichear",
    "chicheen": "chichear",
    "chichees": "chichear",
    "chicheo": "chichear",
    "chicheos": "chicheo",
    "chichera": "chichero",
    "chicheras": "chichero",
    "chicheros": "chichero",
    "chicherías": "chichería",
    "chicheábamos": "chichear",
    "chicheáis": "chichear",
    "chicheáramos": "chichear",
    "chicheáremos": "chichear",
    "chicheásemos": "chichear",
    "chicheé": "chichear",
    "chicheéis": "chichear",
    "chicheó": "chichear",
    "chichicastes": "chichicaste",
    "chichimecas": "chichimeca",
    "chichisbeos": "chichisbeo",
    "chichoneras": "chichonera",
    "chichones": "chichón",
    "chichos": "chicho",
    "chichotas": "chichota",
    "chichurros": "chichurro",
    "chiclea": "chiclear",
    "chicleaba": "chiclear",
    "chicleabais": "chiclear",
    "chicleaban": "chiclear",
    "chicleabas": "chiclear",
    "chiclead": "chiclear",
    "chicleada": "chiclear",
    "chicleadas": "chiclear",
    "chicleado": "chiclear",
    "chicleados": "chiclear",
    "chicleamos": "chiclear",
    "chiclean": "chiclear",
    "chicleando": "chiclear",
    "chicleara": "chiclear",
    "chiclearais": "chiclear",
    "chiclearan": "chiclear",
    "chiclearas": "chiclear",
    "chicleare": "chiclear",
    "chicleareis": "chiclear",
    "chiclearemos": "chiclear",
    "chiclearen": "chiclear",
    "chicleares": "chiclear",
    "chiclearon": "chiclear",
    "chicleará": "chiclear",
    "chiclearán": "chiclear",
    "chiclearás": "chiclear",
    "chiclearé": "chiclear",
    "chiclearéis": "chiclear",
    "chiclearía": "chiclear",
    "chiclearíais": "chiclear",
    "chiclearíamos": "chiclear",
    "chiclearían": "chiclear",
    "chiclearías": "chiclear",
    "chicleas": "chiclear",
    "chiclease": "chiclear",
    "chicleaseis": "chiclear",
    "chicleasen": "chiclear",
    "chicleases": "chiclear",
    "chicleaste": "chiclear",
    "chicleasteis": "chiclear",
    "chiclee": "chiclear",
    "chicleemos": "chiclear",
    "chicleen": "chiclear",
    "chiclees": "chiclear",
    "chicleo": "chiclear",
    "chicles": "chicle",
    "chicleábamos": "chiclear",
    "chicleáis": "chiclear",
    "chicleáramos": "chiclear",
    "chicleáremos": "chiclear",
    "chicleásemos": "chiclear",
    "chicleé": "chiclear",
    "chicleéis": "chiclear",
    "chicleó": "chiclear",
    "chiclés": "chiclé",
    "chicolea": "chicolear",
    "chicoleaba": "chicolear",
    "chicoleabais": "chicolear",
    "chicoleaban": "chicolear",
    "chicoleabas": "chicolear",
    "chicolead": "chicolear",
    "chicoleada": "chicolear",
    "chicoleadas": "chicolear",
    "chicoleado": "chicolear",
    "chicoleados": "chicolear",
    "chicoleamos": "chicolear",
    "chicolean": "chicolear",
    "chicoleando": "chicolear",
    "chicoleara": "chicolear",
    "chicolearais": "chicolear",
    "chicolearan": "chicolear",
    "chicolearas": "chicolear",
    "chicoleare": "chicolear",
    "chicoleareis": "chicolear",
    "chicolearemos": "chicolear",
    "chicolearen": "chicolear",
    "chicoleares": "chicolear",
    "chicolearon": "chicolear",
    "chicoleará": "chicolear",
    "chicolearán": "chicolear",
    "chicolearás": "chicolear",
    "chicolearé": "chicolear",
    "chicolearéis": "chicolear",
    "chicolearía": "chicolear",
    "chicolearíais": "chicolear",
    "chicolearíamos": "chicolear",
    "chicolearían": "chicolear",
    "chicolearías": "chicolear",
    "chicoleas": "chicolear",
    "chicolease": "chicolear",
    "chicoleaseis": "chicolear",
    "chicoleasen": "chicolear",
    "chicoleases": "chicolear",
    "chicoleaste": "chicolear",
    "chicoleasteis": "chicolear",
    "chicolee": "chicolear",
    "chicoleemos": "chicolear",
    "chicoleen": "chicolear",
    "chicolees": "chicolear",
    "chicoleo": "chicolear",
    "chicoleos": "chicoleo",
    "chicoleábamos": "chicolear",
    "chicoleáis": "chicolear",
    "chicoleáramos": "chicolear",
    "chicoleáremos": "chicolear",
    "chicoleásemos": "chicolear",
    "chicoleé": "chicolear",
    "chicoleéis": "chicolear",
    "chicoleó": "chicolear",
    "chicorias": "chicoria",
    "chicorra": "chicorro",
    "chicorras": "chicorro",
    "chicorros": "chicorro",
    "chicos": "chico",
    "chicota": "chicote",
    "chicotas": "chicote",
    "chicotazos": "chicotazo",
    "chicotea": "chicotear",
    "chicoteaba": "chicotear",
    "chicoteabais": "chicotear",
    "chicoteaban": "chicotear",
    "chicoteabas": "chicotear",
    "chicotead": "chicotear",
    "chicoteada": "chicotear",
    "chicoteadas": "chicotear",
    "chicoteado": "chicotear",
    "chicoteados": "chicotear",
    "chicoteamos": "chicotear",
    "chicotean": "chicotear",
    "chicoteando": "chicotear",
    "chicoteara": "chicotear",
    "chicotearais": "chicotear",
    "chicotearan": "chicotear",
    "chicotearas": "chicotear",
    "chicoteare": "chicotear",
    "chicoteareis": "chicotear",
    "chicotearemos": "chicotear",
    "chicotearen": "chicotear",
    "chicoteares": "chicotear",
    "chicotearon": "chicotear",
    "chicoteará": "chicotear",
    "chicotearán": "chicotear",
    "chicotearás": "chicotear",
    "chicotearé": "chicotear",
    "chicotearéis": "chicotear",
    "chicotearía": "chicotear",
    "chicotearíais": "chicotear",
    "chicotearíamos": "chicotear",
    "chicotearían": "chicotear",
    "chicotearías": "chicotear",
    "chicoteas": "chicotear",
    "chicotease": "chicotear",
    "chicoteaseis": "chicotear",
    "chicoteasen": "chicotear",
    "chicoteases": "chicotear",
    "chicoteaste": "chicotear",
    "chicoteasteis": "chicotear",
    "chicotee": "chicotear",
    "chicoteemos": "chicotear",
    "chicoteen": "chicotear",
    "chicotees": "chicotear",
    "chicoteo": "chicotear",
    "chicotes": "chicote",
    "chicoteábamos": "chicotear",
    "chicoteáis": "chicotear",
    "chicoteáramos": "chicotear",
    "chicoteáremos": "chicotear",
    "chicoteásemos": "chicotear",
    "chicoteé": "chicotear",
    "chicoteéis": "chicotear",
    "chicoteó": "chicotear",
    "chicuelinas": "chicuelina",
    "chiffonnieres": "chiffonnier",
    "chifla": "chiflar",
    "chiflaba": "chiflar",
    "chiflabais": "chiflar",
    "chiflaban": "chiflar",
    "chiflabas": "chiflar",
    "chiflad": "chiflar",
    "chiflada": "chiflar",
    "chifladas": "chiflar",
    "chifladeras": "chifladera",
    "chiflado": "chiflar",
    "chiflados": "chiflar",
    "chifladuras": "chifladura",
    "chiflamos": "chiflar",
    "chiflan": "chiflar",
    "chiflando": "chiflar",
    "chiflara": "chiflar",
    "chiflarais": "chiflar",
    "chiflaran": "chiflar",
    "chiflaras": "chiflar",
    "chiflare": "chiflar",
    "chiflareis": "chiflar",
    "chiflaremos": "chiflar",
    "chiflaren": "chiflar",
    "chiflares": "chiflar",
    "chiflaron": "chiflar",
    "chiflará": "chiflar",
    "chiflarán": "chiflar",
    "chiflarás": "chiflar",
    "chiflaré": "chiflar",
    "chiflaréis": "chiflar",
    "chiflaría": "chiflar",
    "chiflaríais": "chiflar",
    "chiflaríamos": "chiflar",
    "chiflarían": "chiflar",
    "chiflarías": "chiflar",
    "chiflas": "chiflar",
    "chiflase": "chiflar",
    "chiflaseis": "chiflar",
    "chiflasen": "chiflar",
    "chiflases": "chiflar",
    "chiflaste": "chiflar",
    "chiflasteis": "chiflar",
    "chiflatos": "chiflato",
    "chifle": "chiflar",
    "chiflemos": "chiflar",
    "chiflen": "chiflar",
    "chifles": "chifle",
    "chiflidos": "chiflido",
    "chiflo": "chiflar",
    "chiflones": "chiflón",
    "chiflos": "chiflo",
    "chiflábamos": "chiflar",
    "chifláis": "chiflar",
    "chifláramos": "chiflar",
    "chifláremos": "chiflar",
    "chiflásemos": "chiflar",
    "chiflé": "chiflar",
    "chifléis": "chiflar",
    "chifló": "chiflar",
    "chigres": "chigre",
    "chihuahuas": "chihuahua",
    "chiitas": "chiíta",
    "chilabas": "chilaba",
    "chilares": "chilar",
    "chilates": "chilate",
    "chilcos": "chilco",
    "chilena": "chileno",
    "chilenas": "chileno",
    "chilenismos": "chilenismo",
    "chilenos": "chileno",
    "chiles": "chile",
    "chilindrinas": "chilindrina",
    "chilindrones": "chilindrón",
    "chilla": "chillar",
    "chillaba": "chillar",
    "chillabais": "chillar",
    "chillaban": "chillar",
    "chillabas": "chillar",
    "chillad": "chillar",
    "chillada": "chillar",
    "chilladas": "chillar",
    "chillado": "chillar",
    "chilladora": "chillador",
    "chilladoras": "chillador",
    "chilladores": "chillador",
    "chillados": "chillar",
    "chillamos": "chillar",
    "chillan": "chillar",
    "chillando": "chillar",
    "chillara": "chillar",
    "chillarais": "chillar",
    "chillaran": "chillar",
    "chillaras": "chillar",
    "chillare": "chillar",
    "chillareis": "chillar",
    "chillaremos": "chillar",
    "chillaren": "chillar",
    "chillares": "chillar",
    "chillaron": "chillar",
    "chillará": "chillar",
    "chillarán": "chillar",
    "chillarás": "chillar",
    "chillaré": "chillar",
    "chillaréis": "chillar",
    "chillaría": "chillar",
    "chillaríais": "chillar",
    "chillaríamos": "chillar",
    "chillarían": "chillar",
    "chillarías": "chillar",
    "chillas": "chillar",
    "chillase": "chillar",
    "chillaseis": "chillar",
    "chillasen": "chillar",
    "chillases": "chillar",
    "chillaste": "chillar",
    "chillasteis": "chillar",
    "chille": "chillar",
    "chillemos": "chillar",
    "chillen": "chillar",
    "chillerías": "chillería",
    "chilles": "chillar",
    "chillidos": "chillido",
    "chillo": "chillar",
    "chillona": "chillón",
    "chillonas": "chillón",
    "chillones": "chillón",
    "chillos": "chillo",
    "chillábamos": "chillar",
    "chilláis": "chillar",
    "chilláramos": "chillar",
    "chilláremos": "chillar",
    "chillásemos": "chillar",
    "chillé": "chillar",
    "chilléis": "chillar",
    "chilló": "chillar",
    "chilotes": "chilote",
    "chimba": "chimbo",
    "chimbas": "chimbo",
    "chimbos": "chimbo",
    "chimeneas": "chimenea",
    "chimpancés": "chimpancé",
    "china": "chino",
    "chinampas": "chinampa",
    "chinampera": "chinampero",
    "chinamperas": "chinampero",
    "chinamperos": "chinampero",
    "chinarros": "chinarro",
    "chinas": "chino",
    "chincha": "chinchar",
    "chinchaba": "chinchar",
    "chinchabais": "chinchar",
    "chinchaban": "chinchar",
    "chinchabas": "chinchar",
    "chinchad": "chinchar",
    "chinchada": "chinchar",
    "chinchadas": "chinchar",
    "chinchado": "chinchar",
    "chinchados": "chinchar",
    "chinchamos": "chinchar",
    "chinchan": "chinchar",
    "chinchando": "chinchar",
    "chinchara": "chinchar",
    "chincharais": "chinchar",
    "chincharan": "chinchar",
    "chincharas": "chinchar",
    "chinchare": "chinchar",
    "chinchareis": "chinchar",
    "chincharemos": "chinchar",
    "chincharen": "chinchar",
    "chinchares": "chinchar",
    "chincharon": "chinchar",
    "chincharreros": "chincharrero",
    "chinchará": "chinchar",
    "chincharán": "chinchar",
    "chincharás": "chinchar",
    "chincharé": "chinchar",
    "chincharéis": "chinchar",
    "chincharía": "chinchar",
    "chincharíais": "chinchar",
    "chincharíamos": "chinchar",
    "chincharían": "chinchar",
    "chincharías": "chinchar",
    "chinchas": "chinchar",
    "chinchase": "chinchar",
    "chinchaseis": "chinchar",
    "chinchasen": "chinchar",
    "chinchases": "chinchar",
    "chinchaste": "chinchar",
    "chinchasteis": "chinchar",
    "chinche": "chinchar",
    "chinchemos": "chinchar",
    "chinchen": "chinchar",
    "chinches": "chinche",
    "chinchetas": "chincheta",
    "chinchillas": "chinchilla",
    "chinchillones": "chinchillón",
    "chinchimenes": "chinchimén",
    "chinchines": "chinchín",
    "chincho": "chinchar",
    "chinchorrera": "chinchorrero",
    "chinchorreras": "chinchorrero",
    "chinchorreros": "chinchorrero",
    "chinchorrerías": "chinchorrería",
    "chinchosa": "chinchoso",
    "chinchosas": "chinchoso",
    "chinchosos": "chinchoso",
    "chinchábamos": "chinchar",
    "chincháis": "chinchar",
    "chincháramos": "chinchar",
    "chincháremos": "chinchar",
    "chinchásemos": "chinchar",
    "chinché": "chinchar",
    "chinchéis": "chinchar",
    "chinchíllidos": "chinchíllido",
    "chinchó": "chinchar",
    "chincoles": "chincol",
    "chindas": "chinda",
    "chinea": "chinear",
    "chineaba": "chinear",
    "chineabais": "chinear",
    "chineaban": "chinear",
    "chineabas": "chinear",
    "chinead": "chinear",
    "chineada": "chinear",
    "chineadas": "chinear",
    "chineado": "chinear",
    "chineados": "chinear",
    "chineamos": "chinear",
    "chinean": "chinear",
    "chineando": "chinear",
    "chineara": "chinear",
    "chinearais": "chinear",
    "chinearan": "chinear",
    "chinearas": "chinear",
    "chineare": "chinear",
    "chineareis": "chinear",
    "chinearemos": "chinear",
    "chinearen": "chinear",
    "chineares": "chinear",
    "chinearon": "chinear",
    "chineará": "chinear",
    "chinearán": "chinear",
    "chinearás": "chinear",
    "chinearé": "chinear",
    "chinearéis": "chinear",
    "chinearía": "chinear",
    "chinearíais": "chinear",
    "chinearíamos": "chinear",
    "chinearían": "chinear",
    "chinearías": "chinear",
    "chineas": "chinear",
    "chinease": "chinear",
    "chineaseis": "chinear",
    "chineasen": "chinear",
    "chineases": "chinear",
    "chineaste": "chinear",
    "chineasteis": "chinear",
    "chinee": "chinear",
    "chineemos": "chinear",
    "chineen": "chinear",
    "chinees": "chinear",
    "chinelas": "chinela",
    "chineo": "chinear",
    "chineros": "chinero",
    "chinesca": "chinesco",
    "chinescas": "chinesco",
    "chinescos": "chinesco",
    "chineábamos": "chinear",
    "chineáis": "chinear",
    "chineáramos": "chinear",
    "chineáremos": "chinear",
    "chineásemos": "chinear",
    "chineé": "chinear",
    "chineéis": "chinear",
    "chineó": "chinear",
    "chinga": "chingo",
    "chingaba": "chingar",
    "chingabais": "chingar",
    "chingaban": "chingar",
    "chingabas": "chingar",
    "chingad": "chingar",
    "chingada": "chingar",
    "chingadas": "chingar",
    "chingado": "chingar",
    "chingados": "chingar",
    "chingamos": "chingar",
    "chingan": "chingar",
    "chinganas": "chingana",
    "chingando": "chingar",
    "chingara": "chingar",
    "chingarais": "chingar",
    "chingaran": "chingar",
    "chingaras": "chingar",
    "chingare": "chingar",
    "chingareis": "chingar",
    "chingaremos": "chingar",
    "chingaren": "chingar",
    "chingares": "chingar",
    "chingaron": "chingar",
    "chingará": "chingar",
    "chingarán": "chingar",
    "chingarás": "chingar",
    "chingaré": "chingar",
    "chingaréis": "chingar",
    "chingaría": "chingar",
    "chingaríais": "chingar",
    "chingaríamos": "chingar",
    "chingarían": "chingar",
    "chingarías": "chingar",
    "chingas": "chingo",
    "chingase": "chingar",
    "chingaseis": "chingar",
    "chingasen": "chingar",
    "chingases": "chingar",
    "chingaste": "chingar",
    "chingasteis": "chingar",
    "chingo": "chingar",
    "chingos": "chingo",
    "chingue": "chingar",
    "chinguemos": "chingar",
    "chinguen": "chingar",
    "chingues": "chingar",
    "chingué": "chingar",
    "chinguéis": "chingar",
    "chingábamos": "chingar",
    "chingáis": "chingar",
    "chingáramos": "chingar",
    "chingáremos": "chingar",
    "chingásemos": "chingar",
    "chingó": "chingar",
    "chinos": "chino",
    "chinés": "chiné",
    "chipichapes": "chipichape",
    "chipirones": "chipirón",
    "chipirónes": "chipirón",
    "chipriotas": "chipriota",
    "chipriotes": "chipriote",
    "chips": "chip",
    "chipés": "chipé",
    "chiquea": "chiquear",
    "chiqueaba": "chiquear",
    "chiqueabais": "chiquear",
    "chiqueaban": "chiquear",
    "chiqueabas": "chiquear",
    "chiquead": "chiquear",
    "chiqueada": "chiquear",
    "chiqueadas": "chiquear",
    "chiqueado": "chiquear",
    "chiqueados": "chiquear",
    "chiqueamos": "chiquear",
    "chiquean": "chiquear",
    "chiqueando": "chiquear",
    "chiqueara": "chiquear",
    "chiquearais": "chiquear",
    "chiquearan": "chiquear",
    "chiquearas": "chiquear",
    "chiqueare": "chiquear",
    "chiqueareis": "chiquear",
    "chiquearemos": "chiquear",
    "chiquearen": "chiquear",
    "chiqueares": "chiquear",
    "chiquearon": "chiquear",
    "chiqueará": "chiquear",
    "chiquearán": "chiquear",
    "chiquearás": "chiquear",
    "chiquearé": "chiquear",
    "chiquearéis": "chiquear",
    "chiquearía": "chiquear",
    "chiquearíais": "chiquear",
    "chiquearíamos": "chiquear",
    "chiquearían": "chiquear",
    "chiquearías": "chiquear",
    "chiqueas": "chiquear",
    "chiquease": "chiquear",
    "chiqueaseis": "chiquear",
    "chiqueasen": "chiquear",
    "chiqueases": "chiquear",
    "chiqueaste": "chiquear",
    "chiqueasteis": "chiquear",
    "chiquee": "chiquear",
    "chiqueemos": "chiquear",
    "chiqueen": "chiquear",
    "chiquees": "chiquear",
    "chiqueo": "chiquear",
    "chiqueros": "chiquero",
    "chiqueábamos": "chiquear",
    "chiqueáis": "chiquear",
    "chiqueáramos": "chiquear",
    "chiqueáremos": "chiquear",
    "chiqueásemos": "chiquear",
    "chiqueé": "chiquear",
    "chiqueéis": "chiquear",
    "chiqueó": "chiquear",
    "chiquilina": "chiquilín",
    "chiquilinas": "chiquilín",
    "chiquilines": "chiquilín",
    "chiquilla": "chiquillo",
    "chiquilladas": "chiquillada",
    "chiquillas": "chiquillo",
    "chiquillerías": "chiquillería",
    "chiquillos": "chiquillo",
    "chiquirritina": "chiquirritín",
    "chiquirritinas": "chiquirritín",
    "chiquirritines": "chiquirritín",
    "chiquita": "chiquito",
    "chiquitas": "chiquito",
    "chiquitina": "chiquitín",
    "chiquitinas": "chiquitín",
    "chiquitines": "chiquitín",
    "chiquitita": "chiquitito",
    "chiquititas": "chiquitito",
    "chiquititos": "chiquitito",
    "chiquitos": "chiquito",
    "chiribitas": "chiribita",
    "chiribitiles": "chiribitil",
    "chirigotas": "chirigota",
    "chirigotera": "chirigotero",
    "chirigoteras": "chirigotero",
    "chirigoteros": "chirigotero",
    "chirigües": "chirigüe",
    "chirimbolos": "chirimbolo",
    "chirimoyas": "chirimoya",
    "chirimoyos": "chirimoyo",
    "chirimías": "chirimía",
    "chiringuitos": "chiringuito",
    "chirinolas": "chirinola",
    "chiripas": "chiripa",
    "chiripea": "chiripear",
    "chiripeaba": "chiripear",
    "chiripeabais": "chiripear",
    "chiripeaban": "chiripear",
    "chiripeabas": "chiripear",
    "chiripead": "chiripear",
    "chiripeada": "chiripear",
    "chiripeadas": "chiripear",
    "chiripeado": "chiripear",
    "chiripeados": "chiripear",
    "chiripeamos": "chiripear",
    "chiripean": "chiripear",
    "chiripeando": "chiripear",
    "chiripeara": "chiripear",
    "chiripearais": "chiripear",
    "chiripearan": "chiripear",
    "chiripearas": "chiripear",
    "chiripeare": "chiripear",
    "chiripeareis": "chiripear",
    "chiripearemos": "chiripear",
    "chiripearen": "chiripear",
    "chiripeares": "chiripear",
    "chiripearon": "chiripear",
    "chiripeará": "chiripear",
    "chiripearán": "chiripear",
    "chiripearás": "chiripear",
    "chiripearé": "chiripear",
    "chiripearéis": "chiripear",
    "chiripearía": "chiripear",
    "chiripearíais": "chiripear",
    "chiripearíamos": "chiripear",
    "chiripearían": "chiripear",
    "chiripearías": "chiripear",
    "chiripeas": "chiripear",
    "chiripease": "chiripear",
    "chiripeaseis": "chiripear",
    "chiripeasen": "chiripear",
    "chiripeases": "chiripear",
    "chiripeaste": "chiripear",
    "chiripeasteis": "chiripear",
    "chiripee": "chiripear",
    "chiripeemos": "chiripear",
    "chiripeen": "chiripear",
    "chiripees": "chiripear",
    "chiripeo": "chiripear",
    "chiripera": "chiripero",
    "chiriperas": "chiripero",
    "chiriperos": "chiripero",
    "chiripeábamos": "chiripear",
    "chiripeáis": "chiripear",
    "chiripeáramos": "chiripear",
    "chiripeáremos": "chiripear",
    "chiripeásemos": "chiripear",
    "chiripeé": "chiripear",
    "chiripeéis": "chiripear",
    "chiripeó": "chiripear",
    "chirivías": "chirivía",
    "chirla": "chirlar",
    "chirlaba": "chirlar",
    "chirlabais": "chirlar",
    "chirlaban": "chirlar",
    "chirlabas": "chirlar",
    "chirlad": "chirlar",
    "chirlada": "chirlar",
    "chirladas": "chirlar",
    "chirlado": "chirlar",
    "chirlados": "chirlar",
    "chirlamos": "chirlar",
    "chirlan": "chirlar",
    "chirlando": "chirlar",
    "chirlara": "chirlar",
    "chirlarais": "chirlar",
    "chirlaran": "chirlar",
    "chirlaras": "chirlar",
    "chirlare": "chirlar",
    "chirlareis": "chirlar",
    "chirlaremos": "chirlar",
    "chirlaren": "chirlar",
    "chirlares": "chirlar",
    "chirlaron": "chirlar",
    "chirlará": "chirlar",
    "chirlarán": "chirlar",
    "chirlarás": "chirlar",
    "chirlaré": "chirlar",
    "chirlaréis": "chirlar",
    "chirlaría": "chirlar",
    "chirlaríais": "chirlar",
    "chirlaríamos": "chirlar",
    "chirlarían": "chirlar",
    "chirlarías": "chirlar",
    "chirlas": "chirlar",
    "chirlase": "chirlar",
    "chirlaseis": "chirlar",
    "chirlasen": "chirlar",
    "chirlases": "chirlar",
    "chirlaste": "chirlar",
    "chirlasteis": "chirlar",
    "chirlatas": "chirlata",
    "chirle": "chirlar",
    "chirlemos": "chirlar",
    "chirlen": "chirlar",
    "chirles": "chirle",
    "chirlo": "chirlar",
    "chirlomirlos": "chirlomirlo",
    "chirlos": "chirlo",
    "chirlábamos": "chirlar",
    "chirláis": "chirlar",
    "chirláramos": "chirlar",
    "chirláremos": "chirlar",
    "chirlásemos": "chirlar",
    "chirlé": "chirlar",
    "chirléis": "chirlar",
    "chirló": "chirlar",
    "chironas": "chirona",
    "chirriaba": "chirriar",
    "chirriabais": "chirriar",
    "chirriaban": "chirriar",
    "chirriabas": "chirriar",
    "chirriad": "chirriar",
    "chirriada": "chirriar",
    "chirriadas": "chirriar",
    "chirriado": "chirriar",
    "chirriados": "chirriar",
    "chirriamos": "chirriar",
    "chirriando": "chirriar",
    "chirriantes": "chirriante",
    "chirriara": "chirriar",
    "chirriarais": "chirriar",
    "chirriaran": "chirriar",
    "chirriaras": "chirriar",
    "chirriare": "chirriar",
    "chirriareis": "chirriar",
    "chirriaremos": "chirriar",
    "chirriaren": "chirriar",
    "chirriares": "chirriar",
    "chirriaron": "chirriar",
    "chirriará": "chirriar",
    "chirriarán": "chirriar",
    "chirriarás": "chirriar",
    "chirriaré": "chirriar",
    "chirriaréis": "chirriar",
    "chirriaría": "chirriar",
    "chirriaríais": "chirriar",
    "chirriaríamos": "chirriar",
    "chirriarían": "chirriar",
    "chirriarías": "chirriar",
    "chirriase": "chirriar",
    "chirriaseis": "chirriar",
    "chirriasen": "chirriar",
    "chirriases": "chirriar",
    "chirriaste": "chirriar",
    "chirriasteis": "chirriar",
    "chirridos": "chirrido",
    "chirriemos": "chirriar",
    "chirrionera": "chirrionero",
    "chirrioneras": "chirrionero",
    "chirrioneros": "chirrionero",
    "chirriones": "chirrión",
    "chirriábamos": "chirriar",
    "chirriáis": "chirriar",
    "chirriáramos": "chirriar",
    "chirriáremos": "chirriar",
    "chirriásemos": "chirriar",
    "chirrié": "chirriar",
    "chirriéis": "chirriar",
    "chirrió": "chirriar",
    "chirría": "chirriar",
    "chirrían": "chirriar",
    "chirrías": "chirriar",
    "chirríe": "chirriar",
    "chirríen": "chirriar",
    "chirríes": "chirriar",
    "chirrío": "chirriar",
    "chirusas": "chirusa",
    "chiruzas": "chiruza",
    "chirúmenes": "chirumen",
    "chiscarras": "chiscarra",
    "chisguetea": "chisguetear",
    "chisgueteaba": "chisguetear",
    "chisgueteabais": "chisguetear",
    "chisgueteaban": "chisguetear",
    "chisgueteabas": "chisguetear",
    "chisguetead": "chisguetear",
    "chisgueteada": "chisguetear",
    "chisgueteadas": "chisguetear",
    "chisgueteado": "chisguetear",
    "chisgueteados": "chisguetear",
    "chisgueteamos": "chisguetear",
    "chisguetean": "chisguetear",
    "chisgueteando": "chisguetear",
    "chisgueteara": "chisguetear",
    "chisguetearais": "chisguetear",
    "chisguetearan": "chisguetear",
    "chisguetearas": "chisguetear",
    "chisgueteare": "chisguetear",
    "chisgueteareis": "chisguetear",
    "chisguetearemos": "chisguetear",
    "chisguetearen": "chisguetear",
    "chisgueteares": "chisguetear",
    "chisguetearon": "chisguetear",
    "chisgueteará": "chisguetear",
    "chisguetearán": "chisguetear",
    "chisguetearás": "chisguetear",
    "chisguetearé": "chisguetear",
    "chisguetearéis": "chisguetear",
    "chisguetearía": "chisguetear",
    "chisguetearíais": "chisguetear",
    "chisguetearíamos": "chisguetear",
    "chisguetearían": "chisguetear",
    "chisguetearías": "chisguetear",
    "chisgueteas": "chisguetear",
    "chisguetease": "chisguetear",
    "chisgueteaseis": "chisguetear",
    "chisgueteasen": "chisguetear",
    "chisgueteases": "chisguetear",
    "chisgueteaste": "chisguetear",
    "chisgueteasteis": "chisguetear",
    "chisguetee": "chisguetear",
    "chisgueteemos": "chisguetear",
    "chisgueteen": "chisguetear",
    "chisguetees": "chisguetear",
    "chisgueteo": "chisguetear",
    "chisguetes": "chisguete",
    "chisgueteábamos": "chisguetear",
    "chisgueteáis": "chisguetear",
    "chisgueteáramos": "chisguetear",
    "chisgueteáremos": "chisguetear",
    "chisgueteásemos": "chisguetear",
    "chisgueteé": "chisguetear",
    "chisgueteéis": "chisguetear",
    "chisgueteó": "chisguetear",
    "chismea": "chismear",
    "chismeaba": "chismear",
    "chismeabais": "chismear",
    "chismeaban": "chismear",
    "chismeabas": "chismear",
    "chismead": "chismear",
    "chismeada": "chismear",
    "chismeadas": "chismear",
    "chismeado": "chismear",
    "chismeados": "chismear",
    "chismeamos": "chismear",
    "chismean": "chismear",
    "chismeando": "chismear",
    "chismeara": "chismear",
    "chismearais": "chismear",
    "chismearan": "chismear",
    "chismearas": "chismear",
    "chismeare": "chismear",
    "chismeareis": "chismear",
    "chismearemos": "chismear",
    "chismearen": "chismear",
    "chismeares": "chismear",
    "chismearon": "chismear",
    "chismeará": "chismear",
    "chismearán": "chismear",
    "chismearás": "chismear",
    "chismearé": "chismear",
    "chismearéis": "chismear",
    "chismearía": "chismear",
    "chismearíais": "chismear",
    "chismearíamos": "chismear",
    "chismearían": "chismear",
    "chismearías": "chismear",
    "chismeas": "chismear",
    "chismease": "chismear",
    "chismeaseis": "chismear",
    "chismeasen": "chismear",
    "chismeases": "chismear",
    "chismeaste": "chismear",
    "chismeasteis": "chismear",
    "chismee": "chismear",
    "chismeemos": "chismear",
    "chismeen": "chismear",
    "chismees": "chismear",
    "chismeo": "chismear",
    "chismera": "chismero",
    "chismeras": "chismero",
    "chismeros": "chismero",
    "chismes": "chisme",
    "chismeábamos": "chismear",
    "chismeáis": "chismear",
    "chismeáramos": "chismear",
    "chismeáremos": "chismear",
    "chismeásemos": "chismear",
    "chismeé": "chismear",
    "chismeéis": "chismear",
    "chismeó": "chismear",
    "chismografías": "chismografía",
    "chismorrea": "chismorrear",
    "chismorreaba": "chismorrear",
    "chismorreabais": "chismorrear",
    "chismorreaban": "chismorrear",
    "chismorreabas": "chismorrear",
    "chismorread": "chismorrear",
    "chismorreada": "chismorrear",
    "chismorreadas": "chismorrear",
    "chismorreado": "chismorrear",
    "chismorreados": "chismorrear",
    "chismorreamos": "chismorrear",
    "chismorrean": "chismorrear",
    "chismorreando": "chismorrear",
    "chismorreara": "chismorrear",
    "chismorrearais": "chismorrear",
    "chismorrearan": "chismorrear",
    "chismorrearas": "chismorrear",
    "chismorreare": "chismorrear",
    "chismorreareis": "chismorrear",
    "chismorrearemos": "chismorrear",
    "chismorrearen": "chismorrear",
    "chismorreares": "chismorrear",
    "chismorrearon": "chismorrear",
    "chismorreará": "chismorrear",
    "chismorrearán": "chismorrear",
    "chismorrearás": "chismorrear",
    "chismorrearé": "chismorrear",
    "chismorrearéis": "chismorrear",
    "chismorrearía": "chismorrear",
    "chismorrearíais": "chismorrear",
    "chismorrearíamos": "chismorrear",
    "chismorrearían": "chismorrear",
    "chismorrearías": "chismorrear",
    "chismorreas": "chismorrear",
    "chismorrease": "chismorrear",
    "chismorreaseis": "chismorrear",
    "chismorreasen": "chismorrear",
    "chismorreases": "chismorrear",
    "chismorreaste": "chismorrear",
    "chismorreasteis": "chismorrear",
    "chismorree": "chismorrear",
    "chismorreemos": "chismorrear",
    "chismorreen": "chismorrear",
    "chismorrees": "chismorrear",
    "chismorreo": "chismorrear",
    "chismorreos": "chismorreo",
    "chismorreábamos": "chismorrear",
    "chismorreáis": "chismorrear",
    "chismorreáramos": "chismorrear",
    "chismorreáremos": "chismorrear",
    "chismorreásemos": "chismorrear",
    "chismorreé": "chismorrear",
    "chismorreéis": "chismorrear",
    "chismorreó": "chismorrear",
    "chismosa": "chismoso",
    "chismosas": "chismoso",
    "chismosos": "chismoso",
    "chismoteos": "chismoteo",
    "chispa": "chispo",
    "chispas": "chispo",
    "chispazos": "chispazo",
    "chispea": "chispear",
    "chispeaba": "chispear",
    "chispeabais": "chispear",
    "chispeaban": "chispear",
    "chispeabas": "chispear",
    "chispead": "chispear",
    "chispeada": "chispear",
    "chispeadas": "chispear",
    "chispeado": "chispear",
    "chispeados": "chispear",
    "chispeamos": "chispear",
    "chispean": "chispear",
    "chispeando": "chispear",
    "chispeantes": "chispeante",
    "chispeara": "chispear",
    "chispearais": "chispear",
    "chispearan": "chispear",
    "chispearas": "chispear",
    "chispeare": "chispear",
    "chispeareis": "chispear",
    "chispearemos": "chispear",
    "chispearen": "chispear",
    "chispeares": "chispear",
    "chispearon": "chispear",
    "chispeará": "chispear",
    "chispearán": "chispear",
    "chispearás": "chispear",
    "chispearé": "chispear",
    "chispearéis": "chispear",
    "chispearía": "chispear",
    "chispearíais": "chispear",
    "chispearíamos": "chispear",
    "chispearían": "chispear",
    "chispearías": "chispear",
    "chispeas": "chispear",
    "chispease": "chispear",
    "chispeaseis": "chispear",
    "chispeasen": "chispear",
    "chispeases": "chispear",
    "chispeaste": "chispear",
    "chispeasteis": "chispear",
    "chispee": "chispear",
    "chispeemos": "chispear",
    "chispeen": "chispear",
    "chispees": "chispear",
    "chispeo": "chispear",
    "chispeos": "chispeo",
    "chispeábamos": "chispear",
    "chispeáis": "chispear",
    "chispeáramos": "chispear",
    "chispeáremos": "chispear",
    "chispeásemos": "chispear",
    "chispeé": "chispear",
    "chispeéis": "chispear",
    "chispeó": "chispear",
    "chispoleta": "chispoleto",
    "chispoletas": "chispoleto",
    "chispoletos": "chispoleto",
    "chisporrotea": "chisporrotear",
    "chisporroteaba": "chisporrotear",
    "chisporroteabais": "chisporrotear",
    "chisporroteaban": "chisporrotear",
    "chisporroteabas": "chisporrotear",
    "chisporrotead": "chisporrotear",
    "chisporroteada": "chisporrotear",
    "chisporroteadas": "chisporrotear",
    "chisporroteado": "chisporrotear",
    "chisporroteados": "chisporrotear",
    "chisporroteamos": "chisporrotear",
    "chisporrotean": "chisporrotear",
    "chisporroteando": "chisporrotear",
    "chisporroteara": "chisporrotear",
    "chisporrotearais": "chisporrotear",
    "chisporrotearan": "chisporrotear",
    "chisporrotearas": "chisporrotear",
    "chisporroteare": "chisporrotear",
    "chisporroteareis": "chisporrotear",
    "chisporrotearemos": "chisporrotear",
    "chisporrotearen": "chisporrotear",
    "chisporroteares": "chisporrotear",
    "chisporrotearon": "chisporrotear",
    "chisporroteará": "chisporrotear",
    "chisporrotearán": "chisporrotear",
    "chisporrotearás": "chisporrotear",
    "chisporrotearé": "chisporrotear",
    "chisporrotearéis": "chisporrotear",
    "chisporrotearía": "chisporrotear",
    "chisporrotearíais": "chisporrotear",
    "chisporrotearíamos": "chisporrotear",
    "chisporrotearían": "chisporrotear",
    "chisporrotearías": "chisporrotear",
    "chisporroteas": "chisporrotear",
    "chisporrotease": "chisporrotear",
    "chisporroteaseis": "chisporrotear",
    "chisporroteasen": "chisporrotear",
    "chisporroteases": "chisporrotear",
    "chisporroteaste": "chisporrotear",
    "chisporroteasteis": "chisporrotear",
    "chisporrotee": "chisporrotear",
    "chisporroteemos": "chisporrotear",
    "chisporroteen": "chisporrotear",
    "chisporrotees": "chisporrotear",
    "chisporroteo": "chisporrotear",
    "chisporroteos": "chisporroteo",
    "chisporroteábamos": "chisporrotear",
    "chisporroteáis": "chisporrotear",
    "chisporroteáramos": "chisporrotear",
    "chisporroteáremos": "chisporrotear",
    "chisporroteásemos": "chisporrotear",
    "chisporroteé": "chisporrotear",
    "chisporroteéis": "chisporrotear",
    "chisporroteó": "chisporrotear",
    "chispos": "chispo",
    "chisposa": "chisposo",
    "chisposas": "chisposo",
    "chisposos": "chisposo",
    "chispómetros": "chispómetro",
    "chisqueros": "chisquero",
    "chista": "chistar",
    "chistaba": "chistar",
    "chistabais": "chistar",
    "chistaban": "chistar",
    "chistabas": "chistar",
    "chistad": "chistar",
    "chistada": "chistar",
    "chistadas": "chistar",
    "chistado": "chistar",
    "chistados": "chistar",
    "chistamos": "chistar",
    "chistan": "chistar",
    "chistando": "chistar",
    "chistara": "chistar",
    "chistarais": "chistar",
    "chistaran": "chistar",
    "chistaras": "chistar",
    "chistare": "chistar",
    "chistareis": "chistar",
    "chistaremos": "chistar",
    "chistaren": "chistar",
    "chistares": "chistar",
    "chistaron": "chistar",
    "chistará": "chistar",
    "chistarán": "chistar",
    "chistarás": "chistar",
    "chistaré": "chistar",
    "chistaréis": "chistar",
    "chistaría": "chistar",
    "chistaríais": "chistar",
    "chistaríamos": "chistar",
    "chistarían": "chistar",
    "chistarías": "chistar",
    "chistas": "chistar",
    "chistase": "chistar",
    "chistaseis": "chistar",
    "chistasen": "chistar",
    "chistases": "chistar",
    "chistaste": "chistar",
    "chistasteis": "chistar",
    "chiste": "chistar",
    "chistemos": "chistar",
    "chisten": "chistar",
    "chisteras": "chistera",
    "chistes": "chiste",
    "chisto": "chistar",
    "chistorras": "chistorra",
    "chistosa": "chistoso",
    "chistosas": "chistoso",
    "chistosos": "chistoso",
    "chistularis": "chistulari",
    "chistus": "chistu",
    "chistábamos": "chistar",
    "chistáis": "chistar",
    "chistáramos": "chistar",
    "chistáremos": "chistar",
    "chistásemos": "chistar",
    "chisté": "chistar",
    "chistéis": "chistar",
    "chistó": "chistar",
    "chitales": "chital",
    "chitas": "chita",
    "chites": "chite",
    "chiticallas": "chiticalla",
    "chitos": "chito",
    "chiva": "chivo",
    "chivaba": "chivar",
    "chivabais": "chivar",
    "chivaban": "chivar",
    "chivabas": "chivar",
    "chivad": "chivar",
    "chivada": "chivar",
    "chivadas": "chivar",
    "chivado": "chivar",
    "chivados": "chivar",
    "chivamos": "chivar",
    "chivan": "chivar",
    "chivando": "chivar",
    "chivara": "chivar",
    "chivarais": "chivar",
    "chivaran": "chivar",
    "chivaras": "chivar",
    "chivare": "chivar",
    "chivareis": "chivar",
    "chivaremos": "chivar",
    "chivaren": "chivar",
    "chivares": "chivar",
    "chivaron": "chivar",
    "chivará": "chivar",
    "chivarán": "chivar",
    "chivarás": "chivar",
    "chivaré": "chivar",
    "chivaréis": "chivar",
    "chivaría": "chivar",
    "chivaríais": "chivar",
    "chivaríamos": "chivar",
    "chivarían": "chivar",
    "chivarías": "chivar",
    "chivas": "chivo",
    "chivase": "chivar",
    "chivaseis": "chivar",
    "chivasen": "chivar",
    "chivases": "chivar",
    "chivaste": "chivar",
    "chivasteis": "chivar",
    "chivata": "chivato",
    "chivatadas": "chivatada",
    "chivatas": "chivato",
    "chivatea": "chivatear",
    "chivateaba": "chivatear",
    "chivateabais": "chivatear",
    "chivateaban": "chivatear",
    "chivateabas": "chivatear",
    "chivatead": "chivatear",
    "chivateada": "chivatear",
    "chivateadas": "chivatear",
    "chivateado": "chivatear",
    "chivateados": "chivatear",
    "chivateamos": "chivatear",
    "chivatean": "chivatear",
    "chivateando": "chivatear",
    "chivateara": "chivatear",
    "chivatearais": "chivatear",
    "chivatearan": "chivatear",
    "chivatearas": "chivatear",
    "chivateare": "chivatear",
    "chivateareis": "chivatear",
    "chivatearemos": "chivatear",
    "chivatearen": "chivatear",
    "chivateares": "chivatear",
    "chivatearon": "chivatear",
    "chivateará": "chivatear",
    "chivatearán": "chivatear",
    "chivatearás": "chivatear",
    "chivatearé": "chivatear",
    "chivatearéis": "chivatear",
    "chivatearía": "chivatear",
    "chivatearíais": "chivatear",
    "chivatearíamos": "chivatear",
    "chivatearían": "chivatear",
    "chivatearías": "chivatear",
    "chivateas": "chivatear",
    "chivatease": "chivatear",
    "chivateaseis": "chivatear",
    "chivateasen": "chivatear",
    "chivateases": "chivatear",
    "chivateaste": "chivatear",
    "chivateasteis": "chivatear",
    "chivatee": "chivatear",
    "chivateemos": "chivatear",
    "chivateen": "chivatear",
    "chivatees": "chivatear",
    "chivateo": "chivatear",
    "chivateábamos": "chivatear",
    "chivateáis": "chivatear",
    "chivateáramos": "chivatear",
    "chivateáremos": "chivatear",
    "chivateásemos": "chivatear",
    "chivateé": "chivatear",
    "chivateéis": "chivatear",
    "chivateó": "chivatear",
    "chivatos": "chivato",
    "chive": "chivar",
    "chivemos": "chivar",
    "chiven": "chivar",
    "chives": "chivar",
    "chiveteros": "chivetero",
    "chivicoyos": "chivicoyo",
    "chivo": "chivar",
    "chivos": "chivo",
    "chivábamos": "chivar",
    "chiváis": "chivar",
    "chiváramos": "chivar",
    "chiváremos": "chivar",
    "chivásemos": "chivar",
    "chivé": "chivar",
    "chivéis": "chivar",
    "chivó": "chivar",
    "chiítas": "chiíta",
    "choca": "chocar",
    "chocaba": "chocar",
    "chocabais": "chocar",
    "chocaban": "chocar",
    "chocabas": "chocar",
    "chocad": "chocar",
    "chocada": "chocar",
    "chocadas": "chocar",
    "chocado": "chocar",
    "chocados": "chocar",
    "chocamos": "chocar",
    "chocan": "chocar",
    "chocando": "chocar",
    "chocantes": "chocante",
    "chocara": "chocar",
    "chocarais": "chocar",
    "chocaran": "chocar",
    "chocaras": "chocar",
    "chocare": "chocar",
    "chocareis": "chocar",
    "chocaremos": "chocar",
    "chocaren": "chocar",
    "chocares": "chocar",
    "chocaron": "chocar",
    "chocarrea": "chocarrear",
    "chocarreaba": "chocarrear",
    "chocarreabais": "chocarrear",
    "chocarreaban": "chocarrear",
    "chocarreabas": "chocarrear",
    "chocarread": "chocarrear",
    "chocarreada": "chocarrear",
    "chocarreadas": "chocarrear",
    "chocarreado": "chocarrear",
    "chocarreados": "chocarrear",
    "chocarreamos": "chocarrear",
    "chocarrean": "chocarrear",
    "chocarreando": "chocarrear",
    "chocarreara": "chocarrear",
    "chocarrearais": "chocarrear",
    "chocarrearan": "chocarrear",
    "chocarrearas": "chocarrear",
    "chocarreare": "chocarrear",
    "chocarreareis": "chocarrear",
    "chocarrearemos": "chocarrear",
    "chocarrearen": "chocarrear",
    "chocarreares": "chocarrear",
    "chocarrearon": "chocarrear",
    "chocarreará": "chocarrear",
    "chocarrearán": "chocarrear",
    "chocarrearás": "chocarrear",
    "chocarrearé": "chocarrear",
    "chocarrearéis": "chocarrear",
    "chocarrearía": "chocarrear",
    "chocarrearíais": "chocarrear",
    "chocarrearíamos": "chocarrear",
    "chocarrearían": "chocarrear",
    "chocarrearías": "chocarrear",
    "chocarreas": "chocarrear",
    "chocarrease": "chocarrear",
    "chocarreaseis": "chocarrear",
    "chocarreasen": "chocarrear",
    "chocarreases": "chocarrear",
    "chocarreaste": "chocarrear",
    "chocarreasteis": "chocarrear",
    "chocarree": "chocarrear",
    "chocarreemos": "chocarrear",
    "chocarreen": "chocarrear",
    "chocarrees": "chocarrear",
    "chocarreo": "chocarrear",
    "chocarrera": "chocarrero",
    "chocarreras": "chocarrero",
    "chocarreros": "chocarrero",
    "chocarrerías": "chocarrería",
    "chocarreábamos": "chocarrear",
    "chocarreáis": "chocarrear",
    "chocarreáramos": "chocarrear",
    "chocarreáremos": "chocarrear",
    "chocarreásemos": "chocarrear",
    "chocarreé": "chocarrear",
    "chocarreéis": "chocarrear",
    "chocarreó": "chocarrear",
    "chocará": "chocar",
    "chocarán": "chocar",
    "chocarás": "chocar",
    "chocaré": "chocar",
    "chocaréis": "chocar",
    "chocaría": "chocar",
    "chocaríais": "chocar",
    "chocaríamos": "chocar",
    "chocarían": "chocar",
    "chocarías": "chocar",
    "chocas": "chocar",
    "chocase": "chocar",
    "chocaseis": "chocar",
    "chocasen": "chocar",
    "chocases": "chocar",
    "chocaste": "chocar",
    "chocasteis": "chocar",
    "chocha": "chocho",
    "chochaperdices": "chochaperdiz",
    "chochas": "chocho",
    "chochea": "chochear",
    "chocheaba": "chochear",
    "chocheabais": "chochear",
    "chocheaban": "chochear",
    "chocheabas": "chochear",
    "chochead": "chochear",
    "chocheada": "chochear",
    "chocheadas": "chochear",
    "chocheado": "chochear",
    "chocheados": "chochear",
    "chocheamos": "chochear",
    "chochean": "chochear",
    "chocheando": "chochear",
    "chocheara": "chochear",
    "chochearais": "chochear",
    "chochearan": "chochear",
    "chochearas": "chochear",
    "chocheare": "chochear",
    "chocheareis": "chochear",
    "chochearemos": "chochear",
    "chochearen": "chochear",
    "chocheares": "chochear",
    "chochearon": "chochear",
    "chocheará": "chochear",
    "chochearán": "chochear",
    "chochearás": "chochear",
    "chochearé": "chochear",
    "chochearéis": "chochear",
    "chochearía": "chochear",
    "chochearíais": "chochear",
    "chochearíamos": "chochear",
    "chochearían": "chochear",
    "chochearías": "chochear",
    "chocheas": "chochear",
    "chochease": "chochear",
    "chocheaseis": "chochear",
    "chocheasen": "chochear",
    "chocheases": "chochear",
    "chocheaste": "chochear",
    "chocheasteis": "chochear",
    "chocheces": "chochez",
    "chochee": "chochear",
    "chocheemos": "chochear",
    "chocheen": "chochear",
    "chochees": "chochear",
    "chocheo": "chochear",
    "chocheos": "chocheo",
    "chocheábamos": "chochear",
    "chocheáis": "chochear",
    "chocheáramos": "chochear",
    "chocheáremos": "chochear",
    "chocheásemos": "chochear",
    "chocheé": "chochear",
    "chocheéis": "chochear",
    "chocheó": "chochear",
    "chochines": "chochín",
    "chochos": "chocho",
    "choclona": "choclón",
    "choclonas": "choclón",
    "choclones": "choclón",
    "choclos": "choclo",
    "choco": "chocar",
    "chocolatera": "chocolatero",
    "chocolateras": "chocolatero",
    "chocolateros": "chocolatero",
    "chocolaterías": "chocolatería",
    "chocolates": "chocolate",
    "chocolatinas": "chocolatina",
    "chocolatines": "chocolatín",
    "chocábamos": "chocar",
    "chocáis": "chocar",
    "chocáramos": "chocar",
    "chocáremos": "chocar",
    "chocásemos": "chocar",
    "chocó": "chocar",
    "choferesas": "choferesa",
    "chola": "cholo",
    "cholas": "cholo",
    "chollas": "cholla",
    "chollos": "chollo",
    "cholos": "cholo",
    "chonguea": "chonguear",
    "chongueaba": "chonguear",
    "chongueabais": "chonguear",
    "chongueaban": "chonguear",
    "chongueabas": "chonguear",
    "chonguead": "chonguear",
    "chongueada": "chonguear",
    "chongueadas": "chonguear",
    "chongueado": "chonguear",
    "chongueados": "chonguear",
    "chongueamos": "chonguear",
    "chonguean": "chonguear",
    "chongueando": "chonguear",
    "chongueara": "chonguear",
    "chonguearais": "chonguear",
    "chonguearan": "chonguear",
    "chonguearas": "chonguear",
    "chongueare": "chonguear",
    "chongueareis": "chonguear",
    "chonguearemos": "chonguear",
    "chonguearen": "chonguear",
    "chongueares": "chonguear",
    "chonguearon": "chonguear",
    "chongueará": "chonguear",
    "chonguearán": "chonguear",
    "chonguearás": "chonguear",
    "chonguearé": "chonguear",
    "chonguearéis": "chonguear",
    "chonguearía": "chonguear",
    "chonguearíais": "chonguear",
    "chonguearíamos": "chonguear",
    "chonguearían": "chonguear",
    "chonguearías": "chonguear",
    "chongueas": "chonguear",
    "chonguease": "chonguear",
    "chongueaseis": "chonguear",
    "chongueasen": "chonguear",
    "chongueases": "chonguear",
    "chongueaste": "chonguear",
    "chongueasteis": "chonguear",
    "chonguee": "chonguear",
    "chongueemos": "chonguear",
    "chongueen": "chonguear",
    "chonguees": "chonguear",
    "chongueo": "chonguear",
    "chongueábamos": "chonguear",
    "chongueáis": "chonguear",
    "chongueáramos": "chonguear",
    "chongueáremos": "chonguear",
    "chongueásemos": "chonguear",
    "chongueé": "chonguear",
    "chongueéis": "chonguear",
    "chongueó": "chonguear",
    "chontales": "chontal",
    "chopas": "chopa",
    "choperas": "chopera",
    "chopos": "chopo",
    "choque": "chocar",
    "choquemos": "chocar",
    "choquen": "chocar",
    "choques": "choque",
    "choquezuelas": "choquezuela",
    "choqué": "chocar",
    "choquéis": "chocar",
    "chora": "choro",
    "choras": "choro",
    "choricera": "choricero",
    "choriceras": "choricero",
    "choriceros": "choricero",
    "choricerías": "choricería",
    "chorizos": "chorizo",
    "chorlitas": "chorlita",
    "chorlitejos": "chorlitejo",
    "chorlitos": "chorlito",
    "chorlos": "chorlo",
    "choros": "choro",
    "chorradas": "chorrada",
    "chorrea": "chorrear",
    "chorreaba": "chorrear",
    "chorreabais": "chorrear",
    "chorreaban": "chorrear",
    "chorreabas": "chorrear",
    "chorread": "chorrear",
    "chorreada": "chorrear",
    "chorreadas": "chorrear",
    "chorreado": "chorrear",
    "chorreados": "chorrear",
    "chorreaduras": "chorreadura",
    "chorreamos": "chorrear",
    "chorrean": "chorrear",
    "chorreando": "chorrear",
    "chorreara": "chorrear",
    "chorrearais": "chorrear",
    "chorrearan": "chorrear",
    "chorrearas": "chorrear",
    "chorreare": "chorrear",
    "chorreareis": "chorrear",
    "chorrearemos": "chorrear",
    "chorrearen": "chorrear",
    "chorreares": "chorrear",
    "chorrearon": "chorrear",
    "chorreará": "chorrear",
    "chorrearán": "chorrear",
    "chorrearás": "chorrear",
    "chorrearé": "chorrear",
    "chorrearéis": "chorrear",
    "chorrearía": "chorrear",
    "chorrearíais": "chorrear",
    "chorrearíamos": "chorrear",
    "chorrearían": "chorrear",
    "chorrearías": "chorrear",
    "chorreas": "chorrear",
    "chorrease": "chorrear",
    "chorreaseis": "chorrear",
    "chorreasen": "chorrear",
    "chorreases": "chorrear",
    "chorreaste": "chorrear",
    "chorreasteis": "chorrear",
    "chorree": "chorrear",
    "chorreemos": "chorrear",
    "chorreen": "chorrear",
    "chorrees": "chorrear",
    "chorreo": "chorrear",
    "chorreos": "chorreo",
    "chorreras": "chorrera",
    "chorretadas": "chorretada",
    "chorreábamos": "chorrear",
    "chorreáis": "chorrear",
    "chorreáramos": "chorrear",
    "chorreáremos": "chorrear",
    "chorreásemos": "chorrear",
    "chorreé": "chorrear",
    "chorreéis": "chorrear",
    "chorreó": "chorrear",
    "chorrillos": "chorrillo",
    "chorritos": "chorrito",
    "chorrones": "chorrón",
    "chorros": "chorro",
    "chortales": "chortal",
    "chota": "choto",
    "chotas": "choto",
    "chotea": "chotear",
    "choteaba": "chotear",
    "choteabais": "chotear",
    "choteaban": "chotear",
    "choteabas": "chotear",
    "chotead": "chotear",
    "choteada": "chotear",
    "choteadas": "chotear",
    "choteado": "chotear",
    "choteados": "chotear",
    "choteamos": "chotear",
    "chotean": "chotear",
    "choteando": "chotear",
    "choteara": "chotear",
    "chotearais": "chotear",
    "chotearan": "chotear",
    "chotearas": "chotear",
    "choteare": "chotear",
    "choteareis": "chotear",
    "chotearemos": "chotear",
    "chotearen": "chotear",
    "choteares": "chotear",
    "chotearon": "chotear",
    "choteará": "chotear",
    "chotearán": "chotear",
    "chotearás": "chotear",
    "chotearé": "chotear",
    "chotearéis": "chotear",
    "chotearía": "chotear",
    "chotearíais": "chotear",
    "chotearíamos": "chotear",
    "chotearían": "chotear",
    "chotearías": "chotear",
    "choteas": "chotear",
    "chotease": "chotear",
    "choteaseis": "chotear",
    "choteasen": "chotear",
    "choteases": "chotear",
    "choteaste": "chotear",
    "choteasteis": "chotear",
    "chotee": "chotear",
    "choteemos": "chotear",
    "choteen": "chotear",
    "chotees": "chotear",
    "choteo": "chotear",
    "choteos": "choteo",
    "choteábamos": "chotear",
    "choteáis": "chotear",
    "choteáramos": "chotear",
    "choteáremos": "chotear",
    "choteásemos": "chotear",
    "choteé": "chotear",
    "choteéis": "chotear",
    "choteó": "chotear",
    "chotos": "choto",
    "chotuna": "chotuno",
    "chotunas": "chotuno",
    "chotunos": "chotuno",
    "choucroutes": "choucroute",
    "chovas": "chova",
    "chovinismos": "chovinismo",
    "chovinistas": "chovinista",
    "chowchows": "chowchow",
    "chozas": "choza",
    "chozos": "chozo",
    "chozpa": "chozpar",
    "chozpaba": "chozpar",
    "chozpabais": "chozpar",
    "chozpaban": "chozpar",
    "chozpabas": "chozpar",
    "chozpad": "chozpar",
    "chozpada": "chozpar",
    "chozpadas": "chozpar",
    "chozpado": "chozpar",
    "chozpados": "chozpar",
    "chozpamos": "chozpar",
    "chozpan": "chozpar",
    "chozpando": "chozpar",
    "chozpara": "chozpar",
    "chozparais": "chozpar",
    "chozparan": "chozpar",
    "chozparas": "chozpar",
    "chozpare": "chozpar",
    "chozpareis": "chozpar",
    "chozparemos": "chozpar",
    "chozparen": "chozpar",
    "chozpares": "chozpar",
    "chozparon": "chozpar",
    "chozpará": "chozpar",
    "chozparán": "chozpar",
    "chozparás": "chozpar",
    "chozparé": "chozpar",
    "chozparéis": "chozpar",
    "chozparía": "chozpar",
    "chozparíais": "chozpar",
    "chozparíamos": "chozpar",
    "chozparían": "chozpar",
    "chozparías": "chozpar",
    "chozpas": "chozpar",
    "chozpase": "chozpar",
    "chozpaseis": "chozpar",
    "chozpasen": "chozpar",
    "chozpases": "chozpar",
    "chozpaste": "chozpar",
    "chozpasteis": "chozpar",
    "chozpe": "chozpar",
    "chozpemos": "chozpar",
    "chozpen": "chozpar",
    "chozpes": "chozpar",
    "chozpo": "chozpar",
    "chozpos": "chozpo",
    "chozpábamos": "chozpar",
    "chozpáis": "chozpar",
    "chozpáramos": "chozpar",
    "chozpáremos": "chozpar",
    "chozpásemos": "chozpar",
    "chozpé": "chozpar",
    "chozpéis": "chozpar",
    "chozpó": "chozpar",
    "chrisóbulas": "chrisóbula",
    "chubascos": "chubasco",
    "chubasqueros": "chubasquero",
    "chucas": "chuca",
    "chuceros": "chucero",
    "chuchea": "chuchear",
    "chucheaba": "chuchear",
    "chucheabais": "chuchear",
    "chucheaban": "chuchear",
    "chucheabas": "chuchear",
    "chuchead": "chuchear",
    "chucheada": "chuchear",
    "chucheadas": "chuchear",
    "chucheado": "chuchear",
    "chucheados": "chuchear",
    "chucheamos": "chuchear",
    "chuchean": "chuchear",
    "chucheando": "chuchear",
    "chucheara": "chuchear",
    "chuchearais": "chuchear",
    "chuchearan": "chuchear",
    "chuchearas": "chuchear",
    "chucheare": "chuchear",
    "chucheareis": "chuchear",
    "chuchearemos": "chuchear",
    "chuchearen": "chuchear",
    "chucheares": "chuchear",
    "chuchearon": "chuchear",
    "chucheará": "chuchear",
    "chuchearán": "chuchear",
    "chuchearás": "chuchear",
    "chuchearé": "chuchear",
    "chuchearéis": "chuchear",
    "chuchearía": "chuchear",
    "chuchearíais": "chuchear",
    "chuchearíamos": "chuchear",
    "chuchearían": "chuchear",
    "chuchearías": "chuchear",
    "chucheas": "chuchear",
    "chuchease": "chuchear",
    "chucheaseis": "chuchear",
    "chucheasen": "chuchear",
    "chucheases": "chuchear",
    "chucheaste": "chuchear",
    "chucheasteis": "chuchear",
    "chuchee": "chuchear",
    "chucheemos": "chuchear",
    "chucheen": "chuchear",
    "chuchees": "chuchear",
    "chucheo": "chuchear",
    "chuchera": "chuchero",
    "chucheras": "chuchero",
    "chucheros": "chuchero",
    "chucherías": "chuchería",
    "chucheábamos": "chuchear",
    "chucheáis": "chuchear",
    "chucheáramos": "chuchear",
    "chucheáremos": "chuchear",
    "chucheásemos": "chuchear",
    "chucheé": "chuchear",
    "chucheéis": "chuchear",
    "chucheó": "chuchear",
    "chuchocas": "chuchoca",
    "chuchos": "chucho",
    "chuchumecos": "chuchumeco",
    "chuclas": "chucla",
    "chuecas": "chueca",
    "chuetas": "chueta",
    "chufas": "chufa",
    "chufletas": "chufleta",
    "chufletea": "chufletear",
    "chufleteaba": "chufletear",
    "chufleteabais": "chufletear",
    "chufleteaban": "chufletear",
    "chufleteabas": "chufletear",
    "chufletead": "chufletear",
    "chufleteada": "chufletear",
    "chufleteadas": "chufletear",
    "chufleteado": "chufletear",
    "chufleteados": "chufletear",
    "chufleteamos": "chufletear",
    "chufletean": "chufletear",
    "chufleteando": "chufletear",
    "chufleteara": "chufletear",
    "chufletearais": "chufletear",
    "chufletearan": "chufletear",
    "chufletearas": "chufletear",
    "chufleteare": "chufletear",
    "chufleteareis": "chufletear",
    "chufletearemos": "chufletear",
    "chufletearen": "chufletear",
    "chufleteares": "chufletear",
    "chufletearon": "chufletear",
    "chufleteará": "chufletear",
    "chufletearán": "chufletear",
    "chufletearás": "chufletear",
    "chufletearé": "chufletear",
    "chufletearéis": "chufletear",
    "chufletearía": "chufletear",
    "chufletearíais": "chufletear",
    "chufletearíamos": "chufletear",
    "chufletearían": "chufletear",
    "chufletearías": "chufletear",
    "chufleteas": "chufletear",
    "chufletease": "chufletear",
    "chufleteaseis": "chufletear",
    "chufleteasen": "chufletear",
    "chufleteases": "chufletear",
    "chufleteaste": "chufletear",
    "chufleteasteis": "chufletear",
    "chufletee": "chufletear",
    "chufleteemos": "chufletear",
    "chufleteen": "chufletear",
    "chufletees": "chufletear",
    "chufleteo": "chufletear",
    "chufleteábamos": "chufletear",
    "chufleteáis": "chufletear",
    "chufleteáramos": "chufletear",
    "chufleteáremos": "chufletear",
    "chufleteásemos": "chufletear",
    "chufleteé": "chufletear",
    "chufleteéis": "chufletear",
    "chufleteó": "chufletear",
    "chula": "chulo",
    "chuladas": "chulada",
    "chulapa": "chulapo",
    "chulapas": "chulapo",
    "chulapona": "chulapón",
    "chulaponas": "chulapón",
    "chulapones": "chulapón",
    "chulapos": "chulapo",
    "chulas": "chulo",
    "chulea": "chulear",
    "chuleaba": "chulear",
    "chuleabais": "chulear",
    "chuleaban": "chulear",
    "chuleabas": "chulear",
    "chulead": "chulear",
    "chuleada": "chulear",
    "chuleadas": "chulear",
    "chuleado": "chulear",
    "chuleados": "chulear",
    "chuleamos": "chulear",
    "chulean": "chulear",
    "chuleando": "chulear",
    "chuleara": "chulear",
    "chulearais": "chulear",
    "chulearan": "chulear",
    "chulearas": "chulear",
    "chuleare": "chulear",
    "chuleareis": "chulear",
    "chulearemos": "chulear",
    "chulearen": "chulear",
    "chuleares": "chulear",
    "chulearon": "chulear",
    "chuleará": "chulear",
    "chulearán": "chulear",
    "chulearás": "chulear",
    "chulearé": "chulear",
    "chulearéis": "chulear",
    "chulearía": "chulear",
    "chulearíais": "chulear",
    "chulearíamos": "chulear",
    "chulearían": "chulear",
    "chulearías": "chulear",
    "chuleas": "chulear",
    "chulease": "chulear",
    "chuleaseis": "chulear",
    "chuleasen": "chulear",
    "chuleases": "chulear",
    "chuleaste": "chulear",
    "chuleasteis": "chulear",
    "chulee": "chulear",
    "chuleemos": "chulear",
    "chuleen": "chulear",
    "chulees": "chulear",
    "chuleo": "chulear",
    "chuleos": "chuleo",
    "chulerías": "chulería",
    "chulesca": "chulesco",
    "chulescas": "chulesco",
    "chulescos": "chulesco",
    "chuletas": "chuleta",
    "chuletera": "chuletero",
    "chuleteras": "chuletero",
    "chuleteros": "chuletero",
    "chuletones": "chuletón",
    "chuleábamos": "chulear",
    "chuleáis": "chulear",
    "chuleáramos": "chulear",
    "chuleáremos": "chulear",
    "chuleásemos": "chulear",
    "chuleé": "chulear",
    "chuleéis": "chulear",
    "chuleó": "chulear",
    "chulos": "chulo",
    "chuma": "chumar",
    "chumaba": "chumar",
    "chumabais": "chumar",
    "chumaban": "chumar",
    "chumabas": "chumar",
    "chumaceras": "chumacera",
    "chumad": "chumar",
    "chumada": "chumar",
    "chumadas": "chumar",
    "chumado": "chumar",
    "chumados": "chumar",
    "chumamos": "chumar",
    "chuman": "chumar",
    "chumando": "chumar",
    "chumara": "chumar",
    "chumarais": "chumar",
    "chumaran": "chumar",
    "chumaras": "chumar",
    "chumare": "chumar",
    "chumareis": "chumar",
    "chumaremos": "chumar",
    "chumaren": "chumar",
    "chumares": "chumar",
    "chumaron": "chumar",
    "chumará": "chumar",
    "chumarán": "chumar",
    "chumarás": "chumar",
    "chumaré": "chumar",
    "chumaréis": "chumar",
    "chumaría": "chumar",
    "chumaríais": "chumar",
    "chumaríamos": "chumar",
    "chumarían": "chumar",
    "chumarías": "chumar",
    "chumas": "chumar",
    "chumase": "chumar",
    "chumaseis": "chumar",
    "chumasen": "chumar",
    "chumases": "chumar",
    "chumaste": "chumar",
    "chumasteis": "chumar",
    "chumba": "chumbo",
    "chumbas": "chumbo",
    "chumberas": "chumbera",
    "chumbos": "chumbo",
    "chume": "chumar",
    "chumemos": "chumar",
    "chumen": "chumar",
    "chumes": "chumar",
    "chumo": "chumar",
    "chumábamos": "chumar",
    "chumáis": "chumar",
    "chumáramos": "chumar",
    "chumáremos": "chumar",
    "chumásemos": "chumar",
    "chumé": "chumar",
    "chuméis": "chumar",
    "chumó": "chumar",
    "chungas": "chunga",
    "chunguea": "chunguear",
    "chungueaba": "chunguear",
    "chungueabais": "chunguear",
    "chungueaban": "chunguear",
    "chungueabas": "chunguear",
    "chunguead": "chunguear",
    "chungueada": "chunguear",
    "chungueadas": "chunguear",
    "chungueado": "chunguear",
    "chungueados": "chunguear",
    "chungueamos": "chunguear",
    "chunguean": "chunguear",
    "chungueando": "chunguear",
    "chungueara": "chunguear",
    "chunguearais": "chunguear",
    "chunguearan": "chunguear",
    "chunguearas": "chunguear",
    "chungueare": "chunguear",
    "chungueareis": "chunguear",
    "chunguearemos": "chunguear",
    "chunguearen": "chunguear",
    "chungueares": "chunguear",
    "chunguearon": "chunguear",
    "chungueará": "chunguear",
    "chunguearán": "chunguear",
    "chunguearás": "chunguear",
    "chunguearé": "chunguear",
    "chunguearéis": "chunguear",
    "chunguearía": "chunguear",
    "chunguearíais": "chunguear",
    "chunguearíamos": "chunguear",
    "chunguearían": "chunguear",
    "chunguearías": "chunguear",
    "chungueas": "chunguear",
    "chunguease": "chunguear",
    "chungueaseis": "chunguear",
    "chungueasen": "chunguear",
    "chungueases": "chunguear",
    "chungueaste": "chunguear",
    "chungueasteis": "chunguear",
    "chunguee": "chunguear",
    "chungueemos": "chunguear",
    "chungueen": "chunguear",
    "chunguees": "chunguear",
    "chungueo": "chunguear",
    "chungueábamos": "chunguear",
    "chungueáis": "chunguear",
    "chungueáramos": "chunguear",
    "chungueáremos": "chunguear",
    "chungueásemos": "chunguear",
    "chungueé": "chunguear",
    "chungueéis": "chunguear",
    "chungueó": "chunguear",
    "chupa": "chupar",
    "chupaba": "chupar",
    "chupabais": "chupar",
    "chupaban": "chupar",
    "chupabas": "chupar",
    "chupad": "chupar",
    "chupada": "chupar",
    "chupadas": "chupar",
    "chupadera": "chupadero",
    "chupaderas": "chupadero",
    "chupaderos": "chupadero",
    "chupado": "chupar",
    "chupadora": "chupador",
    "chupadoras": "chupador",
    "chupadores": "chupador",
    "chupados": "chupar",
    "chupaduras": "chupadura",
    "chupamos": "chupar",
    "chupan": "chupar",
    "chupando": "chupar",
    "chupara": "chupar",
    "chuparais": "chupar",
    "chuparan": "chupar",
    "chuparas": "chupar",
    "chupare": "chupar",
    "chupareis": "chupar",
    "chuparemos": "chupar",
    "chuparen": "chupar",
    "chupares": "chupar",
    "chuparon": "chupar",
    "chupará": "chupar",
    "chuparán": "chupar",
    "chuparás": "chupar",
    "chuparé": "chupar",
    "chuparéis": "chupar",
    "chuparía": "chupar",
    "chuparíais": "chupar",
    "chuparíamos": "chupar",
    "chuparían": "chupar",
    "chuparías": "chupar",
    "chupas": "chupar",
    "chupase": "chupar",
    "chupaseis": "chupar",
    "chupasen": "chupar",
    "chupases": "chupar",
    "chupaste": "chupar",
    "chupasteis": "chupar",
    "chupe": "chupar",
    "chupemos": "chupar",
    "chupen": "chupar",
    "chuperreteos": "chuperreteo",
    "chupes": "chupe",
    "chupetas": "chupeta",
    "chupetea": "chupetear",
    "chupeteaba": "chupetear",
    "chupeteabais": "chupetear",
    "chupeteaban": "chupetear",
    "chupeteabas": "chupetear",
    "chupetead": "chupetear",
    "chupeteada": "chupetear",
    "chupeteadas": "chupetear",
    "chupeteado": "chupetear",
    "chupeteados": "chupetear",
    "chupeteamos": "chupetear",
    "chupetean": "chupetear",
    "chupeteando": "chupetear",
    "chupeteara": "chupetear",
    "chupetearais": "chupetear",
    "chupetearan": "chupetear",
    "chupetearas": "chupetear",
    "chupeteare": "chupetear",
    "chupeteareis": "chupetear",
    "chupetearemos": "chupetear",
    "chupetearen": "chupetear",
    "chupeteares": "chupetear",
    "chupetearon": "chupetear",
    "chupeteará": "chupetear",
    "chupetearán": "chupetear",
    "chupetearás": "chupetear",
    "chupetearé": "chupetear",
    "chupetearéis": "chupetear",
    "chupetearía": "chupetear",
    "chupetearíais": "chupetear",
    "chupetearíamos": "chupetear",
    "chupetearían": "chupetear",
    "chupetearías": "chupetear",
    "chupeteas": "chupetear",
    "chupetease": "chupetear",
    "chupeteaseis": "chupetear",
    "chupeteasen": "chupetear",
    "chupeteases": "chupetear",
    "chupeteaste": "chupetear",
    "chupeteasteis": "chupetear",
    "chupetee": "chupetear",
    "chupeteemos": "chupetear",
    "chupeteen": "chupetear",
    "chupetees": "chupetear",
    "chupeteo": "chupetear",
    "chupeteos": "chupeteo",
    "chupetes": "chupete",
    "chupeteábamos": "chupetear",
    "chupeteáis": "chupetear",
    "chupeteáramos": "chupetear",
    "chupeteáremos": "chupetear",
    "chupeteásemos": "chupetear",
    "chupeteé": "chupetear",
    "chupeteéis": "chupetear",
    "chupeteó": "chupetear",
    "chupetillas": "chupetilla",
    "chupillas": "chupilla",
    "chupinazos": "chupinazo",
    "chupo": "chupar",
    "chupona": "chupón",
    "chuponas": "chupón",
    "chupones": "chupón",
    "chupábamos": "chupar",
    "chupáis": "chupar",
    "chupáramos": "chupar",
    "chupáremos": "chupar",
    "chupásemos": "chupar",
    "chupé": "chupar",
    "chupéis": "chupar",
    "chupó": "chupar",
    "chupópteros": "chupóptero",
    "chuquiraguas": "chuquiragua",
    "churdones": "churdón",
    "churlas": "churla",
    "churlos": "churlo",
    "churquis": "churqui",
    "churra": "churro",
    "churras": "churro",
    "churrascos": "churrasco",
    "churrera": "churrero",
    "churreras": "churrero",
    "churreros": "churrero",
    "churrerías": "churrería",
    "churres": "churre",
    "churretadas": "churretada",
    "churretes": "churrete",
    "churretosa": "churretoso",
    "churretosas": "churretoso",
    "churretosos": "churretoso",
    "churrienta": "churriento",
    "churrientas": "churriento",
    "churrientos": "churriento",
    "churrigueresca": "churrigueresco",
    "churriguerescas": "churrigueresco",
    "churriguerescos": "churrigueresco",
    "churriguerismos": "churriguerismo",
    "churrigueristas": "churriguerista",
    "churros": "churro",
    "churrullera": "churrullero",
    "churrulleras": "churrullero",
    "churrulleros": "churrullero",
    "churrusca": "churruscar",
    "churruscaba": "churruscar",
    "churruscabais": "churruscar",
    "churruscaban": "churruscar",
    "churruscabas": "churruscar",
    "churruscad": "churruscar",
    "churruscada": "churruscar",
    "churruscadas": "churruscar",
    "churruscado": "churruscar",
    "churruscados": "churruscar",
    "churruscamos": "churruscar",
    "churruscan": "churruscar",
    "churruscando": "churruscar",
    "churruscara": "churruscar",
    "churruscarais": "churruscar",
    "churruscaran": "churruscar",
    "churruscaras": "churruscar",
    "churruscare": "churruscar",
    "churruscareis": "churruscar",
    "churruscaremos": "churruscar",
    "churruscaren": "churruscar",
    "churruscares": "churruscar",
    "churruscaron": "churruscar",
    "churruscará": "churruscar",
    "churruscarán": "churruscar",
    "churruscarás": "churruscar",
    "churruscaré": "churruscar",
    "churruscaréis": "churruscar",
    "churruscaría": "churruscar",
    "churruscaríais": "churruscar",
    "churruscaríamos": "churruscar",
    "churruscarían": "churruscar",
    "churruscarías": "churruscar",
    "churruscas": "churruscar",
    "churruscase": "churruscar",
    "churruscaseis": "churruscar",
    "churruscasen": "churruscar",
    "churruscases": "churruscar",
    "churruscaste": "churruscar",
    "churruscasteis": "churruscar",
    "churrusco": "churruscar",
    "churruscos": "churrusco",
    "churruscábamos": "churruscar",
    "churruscáis": "churruscar",
    "churruscáramos": "churruscar",
    "churruscáremos": "churruscar",
    "churruscásemos": "churruscar",
    "churruscó": "churruscar",
    "churrusque": "churruscar",
    "churrusquemos": "churruscar",
    "churrusquen": "churruscar",
    "churrusques": "churruscar",
    "churrusqué": "churruscar",
    "churrusquéis": "churruscar",
    "churumbelas": "churumbela",
    "churumos": "churumo",
    "chusca": "chusco",
    "chuscadas": "chuscada",
    "chuscas": "chusco",
    "chuscos": "chusco",
    "chusmas": "chusma",
    "chuspas": "chuspa",
    "chuta": "chutar",
    "chutaba": "chutar",
    "chutabais": "chutar",
    "chutaban": "chutar",
    "chutabas": "chutar",
    "chutad": "chutar",
    "chutada": "chutar",
    "chutadas": "chutar",
    "chutado": "chutar",
    "chutadora": "chutador",
    "chutadoras": "chutador",
    "chutadores": "chutador",
    "chutados": "chutar",
    "chutamos": "chutar",
    "chutan": "chutar",
    "chutando": "chutar",
    "chutara": "chutar",
    "chutarais": "chutar",
    "chutaran": "chutar",
    "chutaras": "chutar",
    "chutare": "chutar",
    "chutareis": "chutar",
    "chutaremos": "chutar",
    "chutaren": "chutar",
    "chutares": "chutar",
    "chutaron": "chutar",
    "chutará": "chutar",
    "chutarán": "chutar",
    "chutarás": "chutar",
    "chutaré": "chutar",
    "chutaréis": "chutar",
    "chutaría": "chutar",
    "chutaríais": "chutar",
    "chutaríamos": "chutar",
    "chutarían": "chutar",
    "chutarías": "chutar",
    "chutas": "chutar",
    "chutase": "chutar",
    "chutaseis": "chutar",
    "chutasen": "chutar",
    "chutases": "chutar",
    "chutaste": "chutar",
    "chutasteis": "chutar",
    "chute": "chutar",
    "chutemos": "chutar",
    "chuten": "chutar",
    "chutes": "chute",
    "chuto": "chutar",
    "chuts": "chut",
    "chutábamos": "chutar",
    "chutáis": "chutar",
    "chutáramos": "chutar",
    "chutáremos": "chutar",
    "chutásemos": "chutar",
    "chuté": "chutar",
    "chutéis": "chutar",
    "chutó": "chutar",
    "chuzona": "chuzón",
    "chuzonadas": "chuzonada",
    "chuzonas": "chuzón",
    "chuzones": "chuzón",
    "chuzos": "chuzo",
    "chuños": "chuño",
    "chácaras": "chácara",
    "chácharas": "cháchara",
    "chámaras": "chámara",
    "chándales": "chándal",
    "chárteres": "chárter",
    "chésteres": "chéster",
    "chías": "chía",
    "chóferes": "chófer",
    "chúcara": "chúcaro",
    "chúcaras": "chúcaro",
    "chúcaros": "chúcaro",
    "ciaba": "ciar",
    "ciabais": "ciar",
    "ciaban": "ciar",
    "ciabas": "ciar",
    "ciabogas": "ciaboga",
    "ciad": "ciar",
    "ciada": "ciar",
    "ciadas": "ciar",
    "ciado": "ciar",
    "ciados": "ciar",
    "ciamos": "ciar",
    "cianacética": "cianacético",
    "cianacéticas": "cianacético",
    "cianacéticos": "cianacético",
    "cianamidas": "cianamida",
    "cianatos": "cianato",
    "ciando": "ciar",
    "cianhemoglobinas": "cianhemoglobina",
    "cianhídrica": "cianhídrico",
    "cianhídricas": "cianhídrico",
    "cianhídricos": "cianhídrico",
    "cianinas": "cianina",
    "cianitas": "cianita",
    "cianmetahemoglobinas": "cianmetahemoglobina",
    "cianoacetamidas": "cianoacetamida",
    "cianoacetatos": "cianoacetato",
    "cianocetilenos": "cianocetileno",
    "cianoetilaciones": "cianoetilación",
    "cianoetilada": "cianoetilado",
    "cianoetiladas": "cianoetilado",
    "cianoetilados": "cianoetilado",
    "cianoformiatos": "cianoformiato",
    "cianofíceas": "cianofícea",
    "cianohidrinas": "cianohidrina",
    "cianopsias": "cianopsia",
    "cianotipias": "cianotipia",
    "cianuraciones": "cianuración",
    "cianuratos": "cianurato",
    "cianuros": "cianuro",
    "cianófitos": "cianófito",
    "cianógenos": "cianógeno",
    "cianótica": "cianótico",
    "cianóticas": "cianótico",
    "cianóticos": "cianótico",
    "cianúrica": "cianúrico",
    "cianúricas": "cianúrico",
    "cianúricos": "cianúrico",
    "ciara": "ciar",
    "ciarais": "ciar",
    "ciaran": "ciar",
    "ciaras": "ciar",
    "ciare": "ciar",
    "ciareis": "ciar",
    "ciaremos": "ciar",
    "ciaren": "ciar",
    "ciares": "ciar",
    "ciaron": "ciar",
    "ciará": "ciar",
    "ciarán": "ciar",
    "ciarás": "ciar",
    "ciaré": "ciar",
    "ciaréis": "ciar",
    "ciaría": "ciar",
    "ciaríais": "ciar",
    "ciaríamos": "ciar",
    "ciarían": "ciar",
    "ciarías": "ciar",
    "ciase": "ciar",
    "ciaseis": "ciar",
    "ciasen": "ciar",
    "ciases": "ciar",
    "ciaste": "ciar",
    "ciasteis": "ciar",
    "ciateales": "ciateal",
    "ciatos": "ciato",
    "cibarca": "cibarco",
    "cibarcas": "cibarco",
    "cibarcos": "cibarco",
    "ciberarcadi": "ciberarcadia",
    "ciberas": "cibera",
    "ciberdemocracias": "ciberdemocracia",
    "ciberespacios": "ciberespacio",
    "cibernética": "cibernético",
    "cibernéticas": "cibernético",
    "cibernéticos": "cibernético",
    "cibicas": "cibica",
    "ciborios": "ciborio",
    "cicadales": "cicadal",
    "cicadópsidas": "cicadópsida",
    "cicas": "cica",
    "cicatea": "cicatear",
    "cicateaba": "cicatear",
    "cicateabais": "cicatear",
    "cicateaban": "cicatear",
    "cicateabas": "cicatear",
    "cicatead": "cicatear",
    "cicateada": "cicatear",
    "cicateadas": "cicatear",
    "cicateado": "cicatear",
    "cicateados": "cicatear",
    "cicateamos": "cicatear",
    "cicatean": "cicatear",
    "cicateando": "cicatear",
    "cicateara": "cicatear",
    "cicatearais": "cicatear",
    "cicatearan": "cicatear",
    "cicatearas": "cicatear",
    "cicateare": "cicatear",
    "cicateareis": "cicatear",
    "cicatearemos": "cicatear",
    "cicatearen": "cicatear",
    "cicateares": "cicatear",
    "cicatearon": "cicatear",
    "cicateará": "cicatear",
    "cicatearán": "cicatear",
    "cicatearás": "cicatear",
    "cicatearé": "cicatear",
    "cicatearéis": "cicatear",
    "cicatearía": "cicatear",
    "cicatearíais": "cicatear",
    "cicatearíamos": "cicatear",
    "cicatearían": "cicatear",
    "cicatearías": "cicatear",
    "cicateas": "cicatear",
    "cicatease": "cicatear",
    "cicateaseis": "cicatear",
    "cicateasen": "cicatear",
    "cicateases": "cicatear",
    "cicateaste": "cicatear",
    "cicateasteis": "cicatear",
    "cicatee": "cicatear",
    "cicateemos": "cicatear",
    "cicateen": "cicatear",
    "cicatees": "cicatear",
    "cicateo": "cicatear",
    "cicatera": "cicatero",
    "cicateras": "cicatero",
    "cicateros": "cicatero",
    "cicaterías": "cicatería",
    "cicateábamos": "cicatear",
    "cicateáis": "cicatear",
    "cicateáramos": "cicatear",
    "cicateáremos": "cicatear",
    "cicateásemos": "cicatear",
    "cicateé": "cicatear",
    "cicateéis": "cicatear",
    "cicateó": "cicatear",
    "cicatrice": "cicatrizar",
    "cicatricemos": "cicatrizar",
    "cicatricen": "cicatrizar",
    "cicatrices": "cicatrizar",
    "cicatricé": "cicatrizar",
    "cicatricéis": "cicatrizar",
    "cicatriza": "cicatrizar",
    "cicatrizaba": "cicatrizar",
    "cicatrizabais": "cicatrizar",
    "cicatrizaban": "cicatrizar",
    "cicatrizabas": "cicatrizar",
    "cicatrizaciones": "cicatrización",
    "cicatrizad": "cicatrizar",
    "cicatrizada": "cicatrizar",
    "cicatrizadas": "cicatrizar",
    "cicatrizado": "cicatrizar",
    "cicatrizados": "cicatrizar",
    "cicatrizales": "cicatrizal",
    "cicatrizamos": "cicatrizar",
    "cicatrizan": "cicatrizar",
    "cicatrizando": "cicatrizar",
    "cicatrizantes": "cicatrizante",
    "cicatrizara": "cicatrizar",
    "cicatrizarais": "cicatrizar",
    "cicatrizaran": "cicatrizar",
    "cicatrizaras": "cicatrizar",
    "cicatrizare": "cicatrizar",
    "cicatrizareis": "cicatrizar",
    "cicatrizaremos": "cicatrizar",
    "cicatrizaren": "cicatrizar",
    "cicatrizares": "cicatrizar",
    "cicatrizaron": "cicatrizar",
    "cicatrizará": "cicatrizar",
    "cicatrizarán": "cicatrizar",
    "cicatrizarás": "cicatrizar",
    "cicatrizaré": "cicatrizar",
    "cicatrizaréis": "cicatrizar",
    "cicatrizaría": "cicatrizar",
    "cicatrizaríais": "cicatrizar",
    "cicatrizaríamos": "cicatrizar",
    "cicatrizarían": "cicatrizar",
    "cicatrizarías": "cicatrizar",
    "cicatrizas": "cicatrizar",
    "cicatrizase": "cicatrizar",
    "cicatrizaseis": "cicatrizar",
    "cicatrizasen": "cicatrizar",
    "cicatrizases": "cicatrizar",
    "cicatrizaste": "cicatrizar",
    "cicatrizasteis": "cicatrizar",
    "cicatrizativa": "cicatrizativo",
    "cicatrizativas": "cicatrizativo",
    "cicatrizativos": "cicatrizativo",
    "cicatrizo": "cicatrizar",
    "cicatrizábamos": "cicatrizar",
    "cicatrizáis": "cicatrizar",
    "cicatrizáramos": "cicatrizar",
    "cicatrizáremos": "cicatrizar",
    "cicatrizásemos": "cicatrizar",
    "cicatrizó": "cicatrizar",
    "cicerones": "cicerone",
    "ciceroniana": "ciceroniano",
    "ciceronianas": "ciceroniano",
    "ciceronianos": "ciceroniano",
    "cicindelas": "cicindela",
    "cicla": "ciclar",
    "ciclaba": "ciclar",
    "ciclabais": "ciclar",
    "ciclaban": "ciclar",
    "ciclabas": "ciclar",
    "ciclad": "ciclar",
    "ciclada": "ciclar",
    "cicladas": "ciclar",
    "ciclado": "ciclar",
    "ciclados": "ciclar",
    "ciclamatos": "ciclamato",
    "ciclamores": "ciclamor",
    "ciclamos": "ciclar",
    "ciclan": "ciclar",
    "ciclando": "ciclar",
    "ciclanes": "ciclán",
    "ciclanos": "ciclano",
    "ciclantáceas": "ciclantácea",
    "ciclara": "ciclar",
    "ciclarais": "ciclar",
    "ciclaran": "ciclar",
    "ciclaras": "ciclar",
    "ciclare": "ciclar",
    "ciclareis": "ciclar",
    "ciclaremos": "ciclar",
    "ciclaren": "ciclar",
    "ciclares": "ciclar",
    "ciclaron": "ciclar",
    "ciclará": "ciclar",
    "ciclarán": "ciclar",
    "ciclarás": "ciclar",
    "ciclaré": "ciclar",
    "ciclaréis": "ciclar",
    "ciclaría": "ciclar",
    "ciclaríais": "ciclar",
    "ciclaríamos": "ciclar",
    "ciclarían": "ciclar",
    "ciclarías": "ciclar",
    "ciclas": "ciclar",
    "ciclase": "ciclar",
    "ciclaseis": "ciclar",
    "ciclasen": "ciclar",
    "ciclases": "ciclar",
    "ciclaste": "ciclar",
    "ciclasteis": "ciclar",
    "ciclatones": "ciclatón",
    "cicle": "ciclar",
    "ciclectomías": "ciclectomía",
    "ciclemos": "ciclar",
    "ciclen": "ciclar",
    "cicles": "ciclar",
    "cicletrinas": "cicletrina",
    "ciclicidades": "ciclicidad",
    "ciclismos": "ciclismo",
    "ciclistas": "ciclista",
    "ciclo": "ciclar",
    "ciclo-crosses": "ciclo-cross",
    "cicloadiciones": "cicloadición",
    "ciclobarbitales": "ciclobarbital",
    "ciclocefalias": "ciclocefalia",
    "cicloconvertidores": "cicloconvertidor",
    "ciclocumaroles": "ciclocumarol",
    "ciclodiatermias": "ciclodiatermia",
    "cicloducciones": "cicloducción",
    "ciclofilinas": "ciclofilina",
    "cicloforias": "cicloforia",
    "cicloforómetros": "cicloforómetro",
    "ciclofosfamidas": "ciclofosfamida",
    "ciclohexandimetanoles": "ciclohexandimetanol",
    "ciclohexanoles": "ciclohexanol",
    "ciclohexanonas": "ciclohexanona",
    "ciclohexanos": "ciclohexano",
    "ciclohexenos": "ciclohexeno",
    "ciclohexilaminas": "ciclohexilamina",
    "cicloidales": "cicloidal",
    "cicloidea": "cicloideo",
    "cicloideas": "cicloideo",
    "cicloideos": "cicloideo",
    "cicloides": "cicloide",
    "ciclomotora": "ciclomotor",
    "ciclomotoras": "ciclomotor",
    "ciclomotores": "ciclomotor",
    "ciclomotoristas": "ciclomotorista",
    "ciclonales": "ciclonal",
    "ciclones": "ciclón",
    "ciclooxigenasas": "ciclooxigenasa",
    "cicloparafinas": "cicloparafina",
    "ciclopentanoles": "ciclopentanol",
    "ciclopentanos": "ciclopentano",
    "ciclopentenos": "ciclopenteno",
    "ciclopentilpropionatos": "ciclopentilpropionato",
    "cicloplejías": "cicloplejía",
    "ciclopléjica": "ciclopléjico",
    "ciclopléjicas": "ciclopléjico",
    "ciclopléjicos": "ciclopléjico",
    "ciclopropanos": "ciclopropano",
    "ciclopías": "ciclopía",
    "cicloramas": "ciclorama",
    "ciclos": "ciclo",
    "ciclosilicatos": "ciclosilicato",
    "ciclosiloxanos": "ciclosiloxano",
    "ciclosporinas": "ciclosporina",
    "ciclostiles": "ciclostil",
    "ciclostilos": "ciclostilo",
    "ciclotemas": "ciclotema",
    "ciclotimias": "ciclotimia",
    "ciclotomías": "ciclotomía",
    "ciclotrones": "ciclotrón",
    "ciclotropías": "ciclotropía",
    "ciclotrónica": "ciclotrónico",
    "ciclotrónicas": "ciclotrónico",
    "ciclotrónicos": "ciclotrónico",
    "cicloturistas": "cicloturista",
    "ciclotímica": "ciclotímico",
    "ciclotímicas": "ciclotímico",
    "ciclotímicos": "ciclotímico",
    "ciclotómica": "ciclotómico",
    "ciclotómicas": "ciclotómico",
    "ciclotómicos": "ciclotómico",
    "ciclábamos": "ciclar",
    "cicláis": "ciclar",
    "ciclámenes": "ciclamen",
    "cicláramos": "ciclar",
    "cicláremos": "ciclar",
    "ciclásemos": "ciclar",
    "ciclé": "ciclar",
    "cicléis": "ciclar",
    "cicló": "ciclar",
    "ciclónica": "ciclónico",
    "ciclónicas": "ciclónico",
    "ciclónicos": "ciclónico",
    "ciclópea": "ciclópeo",
    "ciclópeas": "ciclópeo",
    "ciclópeos": "ciclópeo",
    "ciclótoma": "ciclótomo",
    "ciclótomas": "ciclótomo",
    "ciclótomos": "ciclótomo",
    "ciconiformes": "ciconiforme",
    "cicutas": "cicuta",
    "cicutinas": "cicutina",
    "cicónidos": "cicónido",
    "cidiana": "cidiano",
    "cidianas": "cidiano",
    "cidianos": "cidiano",
    "cidos": "ácido",
    "cidradas": "cidrada",
    "cidrales": "cidral",
    "cidras": "cidra",
    "cidros": "cidro",
    "ciega": "ciego",
    "ciegan": "cegar",
    "ciegas": "ciego",
    "ciego": "cegar",
    "ciegos": "ciego",
    "ciegue": "cegar",
    "cieguen": "cegar",
    "ciegues": "cegar",
    "cielos": "cielo",
    "ciemos": "ciar",
    "ciencias": "ciencia",
    "cienmillonésima": "cienmillonésimo",
    "cienmillonésimas": "cienmillonésimo",
    "cienmillonésimo": "100000000",
    "cienmillonésimos": "cienmillonésimo",
    "cienmilmillonésima": "cienmilmillonésimo",
    "cienmilmillonésimas": "cienmilmillonésimo",
    "cienmilmillonésimo": "100000000000",
    "cienmilmillonésimos": "cienmilmillonésimo",
    "cienmilésima": "cienmilésimo",
    "cienmilésimas": "cienmilésimo",
    "cienmilésimo": "100000",
    "cienmilésimos": "cienmilésimo",
    "cienmilímetros": "cienmilímetro",
    "cienos": "cieno",
    "cientificismos": "cientificismo",
    "cientificistas": "cientificista",
    "cientismos": "cientismo",
    "cientistas": "cientista",
    "cientos": "ciento",
    "científica": "científico",
    "científicas": "científico",
    "científico-técnica": "científico-técnico",
    "científico-técnicas": "científico-técnico",
    "científico-técnicos": "científico-técnico",
    "científicos": "científico",
    "cierna": "cernir",
    "ciernan": "cernir",
    "ciernas": "cierna",
    "cierne": "cernir",
    "ciernen": "cernir",
    "ciernes": "cierne",
    "cierno": "cernir",
    "cierra": "cerrar",
    "cierran": "cerrar",
    "cierras": "cerrar",
    "cierre": "cerrar",
    "cierren": "cerrar",
    "cierres": "cierre",
    "cierro": "cerrar",
    "cierros": "cierro",
    "cierta": "cierto",
    "ciertas": "cierto",
    "ciertos": "cierto",
    "cierva": "ciervo",
    "ciervas": "ciervo",
    "ciervos": "ciervo",
    "cierzos": "cierzo",
    "cifra": "cifrar",
    "cifraba": "cifrar",
    "cifrabais": "cifrar",
    "cifraban": "cifrar",
    "cifrabas": "cifrar",
    "cifrad": "cifrar",
    "cifrada": "cifrar",
    "cifradas": "cifrar",
    "cifrado": "cifrar",
    "cifrados": "cifrar",
    "ciframos": "cifrar",
    "cifran": "cifrar",
    "cifrando": "cifrar",
    "cifrara": "cifrar",
    "cifrarais": "cifrar",
    "cifraran": "cifrar",
    "cifraras": "cifrar",
    "cifrare": "cifrar",
    "cifrareis": "cifrar",
    "cifraremos": "cifrar",
    "cifraren": "cifrar",
    "cifrares": "cifrar",
    "cifraron": "cifrar",
    "cifrará": "cifrar",
    "cifrarán": "cifrar",
    "cifrarás": "cifrar",
    "cifraré": "cifrar",
    "cifraréis": "cifrar",
    "cifraría": "cifrar",
    "cifraríais": "cifrar",
    "cifraríamos": "cifrar",
    "cifrarían": "cifrar",
    "cifrarías": "cifrar",
    "cifras": "cifrar",
    "cifrase": "cifrar",
    "cifraseis": "cifrar",
    "cifrasen": "cifrar",
    "cifrases": "cifrar",
    "cifraste": "cifrar",
    "cifrasteis": "cifrar",
    "cifre": "cifrar",
    "cifremos": "cifrar",
    "cifren": "cifrar",
    "cifres": "cifrar",
    "cifro": "cifrar",
    "cifrábamos": "cifrar",
    "cifráis": "cifrar",
    "cifráramos": "cifrar",
    "cifráremos": "cifrar",
    "cifrásemos": "cifrar",
    "cifré": "cifrar",
    "cifréis": "cifrar",
    "cifró": "cifrar",
    "cigalas": "cigala",
    "cigallos": "cigallo",
    "cigarrales": "cigarral",
    "cigarras": "cigarra",
    "cigarreras": "cigarrera",
    "cigarreros": "cigarrero",
    "cigarrerías": "cigarrería",
    "cigarrillos": "cigarrillo",
    "cigarrones": "cigarrón",
    "cigarros": "cigarro",
    "cigomas": "cigoma",
    "cigomaticofaciales": "cigomaticofacial",
    "cigomaticofrontales": "cigomaticofrontal",
    "cigomaticomaxilares": "cigomaticomaxilar",
    "cigomaticoorbitaria": "cigomaticoorbitario",
    "cigomaticoorbitarias": "cigomaticoorbitario",
    "cigomaticoorbitarios": "cigomaticoorbitario",
    "cigomaticotemporales": "cigomaticotemporal",
    "cigomaxilares": "cigomaxilar",
    "cigomática": "cigomático",
    "cigomáticas": "cigomático",
    "cigomáticos": "cigomático",
    "cigosomas": "cigosoma",
    "cigotenas": "cigotena",
    "cigotoblastos": "cigotoblasto",
    "cigotonúcleos": "cigotonúcleo",
    "cigotos": "cigoto",
    "cigoñales": "cigoñal",
    "cigoñinos": "cigoñino",
    "ciguata": "ciguato",
    "ciguataba": "ciguatar",
    "ciguatabais": "ciguatar",
    "ciguataban": "ciguatar",
    "ciguatabas": "ciguatar",
    "ciguatad": "ciguatar",
    "ciguatada": "ciguatar",
    "ciguatadas": "ciguatar",
    "ciguatado": "ciguatar",
    "ciguatados": "ciguatar",
    "ciguatamos": "ciguatar",
    "ciguatan": "ciguatar",
    "ciguatando": "ciguatar",
    "ciguatara": "ciguatar",
    "ciguatarais": "ciguatar",
    "ciguataran": "ciguatar",
    "ciguataras": "ciguatar",
    "ciguatare": "ciguatar",
    "ciguatareis": "ciguatar",
    "ciguataremos": "ciguatar",
    "ciguataren": "ciguatar",
    "ciguatares": "ciguatar",
    "ciguataron": "ciguatar",
    "ciguatará": "ciguatar",
    "ciguatarán": "ciguatar",
    "ciguatarás": "ciguatar",
    "ciguataré": "ciguatar",
    "ciguataréis": "ciguatar",
    "ciguataría": "ciguatar",
    "ciguataríais": "ciguatar",
    "ciguataríamos": "ciguatar",
    "ciguatarían": "ciguatar",
    "ciguatarías": "ciguatar",
    "ciguatas": "ciguato",
    "ciguatase": "ciguatar",
    "ciguataseis": "ciguatar",
    "ciguatasen": "ciguatar",
    "ciguatases": "ciguatar",
    "ciguataste": "ciguatar",
    "ciguatasteis": "ciguatar",
    "ciguate": "ciguatar",
    "ciguatemos": "ciguatar",
    "ciguaten": "ciguatar",
    "ciguateras": "ciguatera",
    "ciguates": "ciguatar",
    "ciguato": "ciguatar",
    "ciguatos": "ciguato",
    "ciguatoxinas": "ciguatoxina",
    "ciguatábamos": "ciguatar",
    "ciguatáis": "ciguatar",
    "ciguatáramos": "ciguatar",
    "ciguatáremos": "ciguatar",
    "ciguatásemos": "ciguatar",
    "ciguaté": "ciguatar",
    "ciguatéis": "ciguatar",
    "ciguató": "ciguatar",
    "ciguaya": "ciguayo",
    "ciguayas": "ciguayo",
    "ciguayos": "ciguayo",
    "cigüeñales": "cigüeñal",
    "cigüeñas": "cigüeña",
    "cigüeñatos": "cigüeñato",
    "cigüeñuelas": "cigüeñuela",
    "cihuacóatles": "cihuacóatl",
    "cihuateteos": "cihuateteo",
    "cijas": "cija",
    "cilancos": "cilanco",
    "cilantros": "cilantro",
    "ciliada": "ciliado",
    "ciliadas": "ciliado",
    "ciliados": "ciliado",
    "ciliares": "ciliar",
    "ciliaroscopios": "ciliaroscopio",
    "ciliarotomías": "ciliarotomía",
    "cilicios": "cilicio",
    "cilicotomías": "cilicotomía",
    "ciliectomías": "ciliectomía",
    "cilindra": "cilindrar",
    "cilindraba": "cilindrar",
    "cilindrabais": "cilindrar",
    "cilindraban": "cilindrar",
    "cilindrabas": "cilindrar",
    "cilindrad": "cilindrar",
    "cilindrada": "cilindrar",
    "cilindradas": "cilindrar",
    "cilindrado": "cilindrar",
    "cilindrados": "cilindrar",
    "cilindramos": "cilindrar",
    "cilindran": "cilindrar",
    "cilindrando": "cilindrar",
    "cilindrara": "cilindrar",
    "cilindrarais": "cilindrar",
    "cilindraran": "cilindrar",
    "cilindraras": "cilindrar",
    "cilindrare": "cilindrar",
    "cilindrareis": "cilindrar",
    "cilindraremos": "cilindrar",
    "cilindraren": "cilindrar",
    "cilindrares": "cilindrar",
    "cilindraron": "cilindrar",
    "cilindrará": "cilindrar",
    "cilindrarán": "cilindrar",
    "cilindrarás": "cilindrar",
    "cilindraré": "cilindrar",
    "cilindraréis": "cilindrar",
    "cilindraría": "cilindrar",
    "cilindraríais": "cilindrar",
    "cilindraríamos": "cilindrar",
    "cilindrarían": "cilindrar",
    "cilindrarías": "cilindrar",
    "cilindras": "cilindrar",
    "cilindrase": "cilindrar",
    "cilindraseis": "cilindrar",
    "cilindrasen": "cilindrar",
    "cilindrases": "cilindrar",
    "cilindraste": "cilindrar",
    "cilindrasteis": "cilindrar",
    "cilindre": "cilindrar",
    "cilindremos": "cilindrar",
    "cilindren": "cilindrar",
    "cilindres": "cilindrar",
    "cilindricidades": "cilindricidad",
    "cilindro": "cilindrar",
    "cilindroides": "cilindroide",
    "cilindros": "cilindro",
    "cilindrábamos": "cilindrar",
    "cilindráis": "cilindrar",
    "cilindráramos": "cilindrar",
    "cilindráremos": "cilindrar",
    "cilindrásemos": "cilindrar",
    "cilindré": "cilindrar",
    "cilindréis": "cilindrar",
    "cilindró": "cilindrar",
    "cilios": "cilio",
    "cillas": "cilla",
    "cillazgos": "cillazgo",
    "cillereros": "cillerero",
    "cillerizos": "cillerizo",
    "cilleros": "cillero",
    "cillerías": "cillería",
    "cilíndrica": "cilíndrico",
    "cilíndricas": "cilíndrico",
    "cilíndricos": "cilíndrico",
    "cimacios": "cimacio",
    "cimarrona": "cimarrón",
    "cimarronadas": "cimarronada",
    "cimarronas": "cimarrón",
    "cimarrones": "cimarrón",
    "cimas": "cima",
    "cimbalarias": "cimbalaria",
    "cimbalera": "cimbalero",
    "cimbaleras": "cimbalero",
    "cimbaleros": "cimbalero",
    "cimbalillos": "cimbalillo",
    "cimbalistas": "cimbalista",
    "cimbanillos": "cimbanillo",
    "cimbas": "cimba",
    "cimbeles": "cimbel",
    "cimborios": "cimborio",
    "cimborrios": "cimborrio",
    "cimbra": "cimbrar",
    "cimbraba": "cimbrar",
    "cimbrabais": "cimbrar",
    "cimbraban": "cimbrar",
    "cimbrabas": "cimbrar",
    "cimbrad": "cimbrar",
    "cimbrada": "cimbrar",
    "cimbradas": "cimbrar",
    "cimbrado": "cimbrar",
    "cimbrados": "cimbrar",
    "cimbramos": "cimbrar",
    "cimbran": "cimbrar",
    "cimbrando": "cimbrar",
    "cimbrara": "cimbrar",
    "cimbrarais": "cimbrar",
    "cimbraran": "cimbrar",
    "cimbraras": "cimbrar",
    "cimbrare": "cimbrar",
    "cimbrareis": "cimbrar",
    "cimbraremos": "cimbrar",
    "cimbraren": "cimbrar",
    "cimbrares": "cimbrar",
    "cimbraron": "cimbrar",
    "cimbrará": "cimbrar",
    "cimbrarán": "cimbrar",
    "cimbrarás": "cimbrar",
    "cimbraré": "cimbrar",
    "cimbraréis": "cimbrar",
    "cimbraría": "cimbrar",
    "cimbraríais": "cimbrar",
    "cimbraríamos": "cimbrar",
    "cimbrarían": "cimbrar",
    "cimbrarías": "cimbrar",
    "cimbras": "cimbrar",
    "cimbrase": "cimbrar",
    "cimbraseis": "cimbrar",
    "cimbrasen": "cimbrar",
    "cimbrases": "cimbrar",
    "cimbraste": "cimbrar",
    "cimbrasteis": "cimbrar",
    "cimbre": "cimbrar",
    "cimbrea": "cimbrear",
    "cimbreaba": "cimbrear",
    "cimbreabais": "cimbrear",
    "cimbreaban": "cimbrear",
    "cimbreabas": "cimbrear",
    "cimbread": "cimbrear",
    "cimbreada": "cimbrear",
    "cimbreadas": "cimbrear",
    "cimbreado": "cimbrear",
    "cimbreados": "cimbrear",
    "cimbreamos": "cimbrear",
    "cimbrean": "cimbrear",
    "cimbreando": "cimbrear",
    "cimbreara": "cimbrear",
    "cimbrearais": "cimbrear",
    "cimbrearan": "cimbrear",
    "cimbrearas": "cimbrear",
    "cimbreare": "cimbrear",
    "cimbreareis": "cimbrear",
    "cimbrearemos": "cimbrear",
    "cimbrearen": "cimbrear",
    "cimbreares": "cimbrear",
    "cimbrearon": "cimbrear",
    "cimbreará": "cimbrear",
    "cimbrearán": "cimbrear",
    "cimbrearás": "cimbrear",
    "cimbrearé": "cimbrear",
    "cimbrearéis": "cimbrear",
    "cimbrearía": "cimbrear",
    "cimbrearíais": "cimbrear",
    "cimbrearíamos": "cimbrear",
    "cimbrearían": "cimbrear",
    "cimbrearías": "cimbrear",
    "cimbreas": "cimbrear",
    "cimbrease": "cimbrear",
    "cimbreaseis": "cimbrear",
    "cimbreasen": "cimbrear",
    "cimbreases": "cimbrear",
    "cimbreaste": "cimbrear",
    "cimbreasteis": "cimbrear",
    "cimbree": "cimbrear",
    "cimbreemos": "cimbrear",
    "cimbreen": "cimbrear",
    "cimbrees": "cimbrear",
    "cimbremos": "cimbrar",
    "cimbren": "cimbrar",
    "cimbreo": "cimbrear",
    "cimbreos": "cimbreo",
    "cimbres": "cimbre",
    "cimbreábamos": "cimbrear",
    "cimbreáis": "cimbrear",
    "cimbreáramos": "cimbrear",
    "cimbreáremos": "cimbrear",
    "cimbreásemos": "cimbrear",
    "cimbreé": "cimbrear",
    "cimbreéis": "cimbrear",
    "cimbreña": "cimbreño",
    "cimbreñas": "cimbreño",
    "cimbreños": "cimbreño",
    "cimbreó": "cimbrear",
    "cimbrios": "cimbrio",
    "cimbro": "cimbrar",
    "cimbrábamos": "cimbrar",
    "cimbráis": "cimbrar",
    "cimbráramos": "cimbrar",
    "cimbráremos": "cimbrar",
    "cimbrásemos": "cimbrar",
    "cimbré": "cimbrar",
    "cimbréis": "cimbrar",
    "cimbró": "cimbrar",
    "cimenos": "cimeno",
    "cimentaba": "cimentar",
    "cimentabais": "cimentar",
    "cimentaban": "cimentar",
    "cimentabas": "cimentar",
    "cimentaciones": "cimentación",
    "cimentad": "cimentar",
    "cimentada": "cimentar",
    "cimentadas": "cimentar",
    "cimentado": "cimentar",
    "cimentadora": "cimentador",
    "cimentadoras": "cimentador",
    "cimentadores": "cimentador",
    "cimentados": "cimentar",
    "cimentamos": "cimentar",
    "cimentando": "cimentar",
    "cimentara": "cimentar",
    "cimentarais": "cimentar",
    "cimentaran": "cimentar",
    "cimentaras": "cimentar",
    "cimentare": "cimentar",
    "cimentareis": "cimentar",
    "cimentaremos": "cimentar",
    "cimentaren": "cimentar",
    "cimentares": "cimentar",
    "cimentaron": "cimentar",
    "cimentará": "cimentar",
    "cimentarán": "cimentar",
    "cimentarás": "cimentar",
    "cimentaré": "cimentar",
    "cimentaréis": "cimentar",
    "cimentaría": "cimentar",
    "cimentaríais": "cimentar",
    "cimentaríamos": "cimentar",
    "cimentarían": "cimentar",
    "cimentarías": "cimentar",
    "cimentase": "cimentar",
    "cimentaseis": "cimentar",
    "cimentasen": "cimentar",
    "cimentases": "cimentar",
    "cimentaste": "cimentar",
    "cimentasteis": "cimentar",
    "cimentemos": "cimentar",
    "cimentábamos": "cimentar",
    "cimentáis": "cimentar",
    "cimentáramos": "cimentar",
    "cimentáremos": "cimentar",
    "cimentásemos": "cimentar",
    "cimenté": "cimentar",
    "cimentéis": "cimentar",
    "cimentó": "cimentar",
    "cimera": "cimero",
    "cimeras": "cimero",
    "cimerios": "cimerio",
    "cimeros": "cimero",
    "cimetidinas": "cimetidina",
    "cimienta": "cimentar",
    "cimientan": "cimentar",
    "cimientas": "cimentar",
    "cimiente": "cimentar",
    "cimienten": "cimentar",
    "cimientes": "cimentar",
    "cimiento": "cimentar",
    "cimientos": "cimiento",
    "cimillos": "cimillo",
    "ciminogenasas": "ciminogenasa",
    "cimitarras": "cimitarra",
    "cimosa": "cimoso",
    "cimosas": "cimoso",
    "cimosos": "cimoso",
    "cimófanos": "cimófano",
    "cinabrios": "cinabrio",
    "cinacinas": "cinacina",
    "cinamatos": "cinamato",
    "cinamomos": "cinamomo",
    "cinca": "cincar",
    "cincaba": "cincar",
    "cincabais": "cincar",
    "cincaban": "cincar",
    "cincabas": "cincar",
    "cincad": "cincar",
    "cincada": "cincar",
    "cincadas": "cincar",
    "cincado": "cincar",
    "cincados": "cincar",
    "cincamos": "cincar",
    "cincan": "cincar",
    "cincando": "cincar",
    "cincara": "cincar",
    "cincarais": "cincar",
    "cincaran": "cincar",
    "cincaras": "cincar",
    "cincare": "cincar",
    "cincareis": "cincar",
    "cincaremos": "cincar",
    "cincaren": "cincar",
    "cincares": "cincar",
    "cincaron": "cincar",
    "cincará": "cincar",
    "cincarán": "cincar",
    "cincarás": "cincar",
    "cincaré": "cincar",
    "cincaréis": "cincar",
    "cincaría": "cincar",
    "cincaríais": "cincar",
    "cincaríamos": "cincar",
    "cincarían": "cincar",
    "cincarías": "cincar",
    "cincas": "cincar",
    "cincase": "cincar",
    "cincaseis": "cincar",
    "cincasen": "cincar",
    "cincases": "cincar",
    "cincaste": "cincar",
    "cincasteis": "cincar",
    "cincela": "cincelar",
    "cincelaba": "cincelar",
    "cincelabais": "cincelar",
    "cincelaban": "cincelar",
    "cincelabas": "cincelar",
    "cincelad": "cincelar",
    "cincelada": "cincelar",
    "cinceladas": "cincelar",
    "cincelado": "cincelar",
    "cinceladora": "cincelador",
    "cinceladoras": "cincelador",
    "cinceladores": "cincelador",
    "cincelados": "cincelar",
    "cinceladuras": "cinceladura",
    "cincelamos": "cincelar",
    "cincelan": "cincelar",
    "cincelando": "cincelar",
    "cincelara": "cincelar",
    "cincelarais": "cincelar",
    "cincelaran": "cincelar",
    "cincelaras": "cincelar",
    "cincelare": "cincelar",
    "cincelareis": "cincelar",
    "cincelaremos": "cincelar",
    "cincelaren": "cincelar",
    "cincelares": "cincelar",
    "cincelaron": "cincelar",
    "cincelará": "cincelar",
    "cincelarán": "cincelar",
    "cincelarás": "cincelar",
    "cincelaré": "cincelar",
    "cincelaréis": "cincelar",
    "cincelaría": "cincelar",
    "cincelaríais": "cincelar",
    "cincelaríamos": "cincelar",
    "cincelarían": "cincelar",
    "cincelarías": "cincelar",
    "cincelas": "cincelar",
    "cincelase": "cincelar",
    "cincelaseis": "cincelar",
    "cincelasen": "cincelar",
    "cincelases": "cincelar",
    "cincelaste": "cincelar",
    "cincelasteis": "cincelar",
    "cincele": "cincelar",
    "cincelemos": "cincelar",
    "cincelen": "cincelar",
    "cinceles": "cincelar",
    "cincelo": "cincelar",
    "cincelábamos": "cincelar",
    "cinceláis": "cincelar",
    "cinceláramos": "cincelar",
    "cinceláremos": "cincelar",
    "cincelásemos": "cincelar",
    "cincelé": "cincelar",
    "cinceléis": "cincelar",
    "cinceló": "cincelar",
    "cincha": "cinchar",
    "cinchaba": "cinchar",
    "cinchabais": "cinchar",
    "cinchaban": "cinchar",
    "cinchabas": "cinchar",
    "cinchad": "cinchar",
    "cinchada": "cinchar",
    "cinchadas": "cinchar",
    "cinchado": "cinchar",
    "cinchados": "cinchar",
    "cinchaduras": "cinchadura",
    "cinchamos": "cinchar",
    "cinchan": "cinchar",
    "cinchando": "cinchar",
    "cinchara": "cinchar",
    "cincharais": "cinchar",
    "cincharan": "cinchar",
    "cincharas": "cinchar",
    "cinchare": "cinchar",
    "cinchareis": "cinchar",
    "cincharemos": "cinchar",
    "cincharen": "cinchar",
    "cinchares": "cinchar",
    "cincharon": "cinchar",
    "cinchará": "cinchar",
    "cincharán": "cinchar",
    "cincharás": "cinchar",
    "cincharé": "cinchar",
    "cincharéis": "cinchar",
    "cincharía": "cinchar",
    "cincharíais": "cinchar",
    "cincharíamos": "cinchar",
    "cincharían": "cinchar",
    "cincharías": "cinchar",
    "cinchas": "cinchar",
    "cinchase": "cinchar",
    "cinchaseis": "cinchar",
    "cinchasen": "cinchar",
    "cinchases": "cinchar",
    "cinchaste": "cinchar",
    "cinchasteis": "cinchar",
    "cinche": "cinchar",
    "cinchemos": "cinchar",
    "cinchen": "cinchar",
    "cincheras": "cinchera",
    "cinches": "cinchar",
    "cincho": "cinchar",
    "cinchos": "cincho",
    "cinchuelos": "cinchuelo",
    "cinchábamos": "cinchar",
    "cincháis": "cinchar",
    "cincháramos": "cinchar",
    "cincháremos": "cinchar",
    "cinchásemos": "cinchar",
    "cinché": "cinchar",
    "cinchéis": "cinchar",
    "cinchó": "cinchar",
    "cincinos": "cincino",
    "cincitas": "cincita",
    "cinco": "cincar",
    "cincoenramas": "cincoenrama",
    "cincograbados": "cincograbado",
    "cincografías": "cincografía",
    "cincomesina": "cincomesino",
    "cincomesinas": "cincomesino",
    "cincomesinos": "cincomesino",
    "cinconidinas": "cinconidina",
    "cincuentaicincoava": "cincuentaicincoavo",
    "cincuentaicincoavas": "cincuentaicincoavo",
    "cincuentaicincoavo": "55",
    "cincuentaicincoavos": "cincuentaicincoavo",
    "cincuentaicuatroava": "cincuentaicuatroavo",
    "cincuentaicuatroavas": "cincuentaicuatroavo",
    "cincuentaicuatroavo": "54",
    "cincuentaicuatroavos": "cincuentaicuatroavo",
    "cincuentaidosava": "cincuentaidosavo",
    "cincuentaidosavas": "cincuentaidosavo",
    "cincuentaidosavo": "52",
    "cincuentaidosavos": "cincuentaidosavo",
    "cincuentainueveava": "cincuentainueveavo",
    "cincuentainueveavas": "cincuentainueveavo",
    "cincuentainueveavo": "59",
    "cincuentainueveavos": "cincuentainueveavo",
    "cincuentaiochava": "cincuentaiochoavo",
    "cincuentaiochavas": "cincuentaiochoavo",
    "cincuentaiochavo": "cincuentaiochoavo",
    "cincuentaiochavos": "cincuentaiochoavo",
    "cincuentaiochoava": "cincuentaiochoavo",
    "cincuentaiochoavas": "cincuentaiochoavo",
    "cincuentaiochoavo": "58",
    "cincuentaiochoavos": "cincuentaiochoavo",
    "cincuentaiseisava": "cincuentaiseisavo",
    "cincuentaiseisavas": "cincuentaiseisavo",
    "cincuentaiseisavo": "56",
    "cincuentaiseisavos": "cincuentaiseisavo",
    "cincuentaisieteava": "cincuentaisieteavo",
    "cincuentaisieteavas": "cincuentaisieteavo",
    "cincuentaisieteavo": "57",
    "cincuentaisieteavos": "cincuentaisieteavo",
    "cincuentaitresava": "cincuentaitresavo",
    "cincuentaitresavas": "cincuentaitresavo",
    "cincuentaitresavo": "53",
    "cincuentaitresavos": "cincuentaitresavo",
    "cincuentaiunava": "cincuentaiunavo",
    "cincuentaiunavas": "cincuentaiunavo",
    "cincuentaiunavo": "51",
    "cincuentaiunavos": "cincuentaiunavo",
    "cincuentava": "cincuentavo",
    "cincuentavas": "cincuentavo",
    "cincuentavo": "50",
    "cincuentavos": "cincuentavo",
    "cincuentenarios": "cincuentenario",
    "cincuentenas": "cincuentena",
    "cincuentona": "cincuentón",
    "cincuentonas": "cincuentón",
    "cincuentones": "cincuentón",
    "cincuentésima": "50",
    "cincuentésimas": "50",
    "cincuentésimo": "50",
    "cincuentésimos": "50",
    "cincábamos": "cincar",
    "cincáis": "cincar",
    "cincáramos": "cincar",
    "cincáremos": "cincar",
    "cincásemos": "cincar",
    "cincó": "cincar",
    "cincófenos": "cincófeno",
    "cincógrafa": "cincógrafo",
    "cincógrafas": "cincógrafo",
    "cincógrafos": "cincógrafo",
    "cineastas": "cineasta",
    "cineclubes": "cineclub",
    "cineclubs": "cineclub",
    "cinecámaras": "cinecámara",
    "cinefórums": "cinefórum",
    "cinegética": "cinegético",
    "cinegéticas": "cinegético",
    "cinegéticos": "cinegético",
    "cinemas": "cinema",
    "cinemascopes": "cinemascope",
    "cinematografías": "cinematografía",
    "cinematográfica": "cinematográfico",
    "cinematográficas": "cinematográfico",
    "cinematográficos": "cinematográfico",
    "cinematógrafos": "cinematógrafo",
    "cinemática": "cinemático",
    "cinemáticas": "cinemático",
    "cinemáticos": "cinemático",
    "cinemómetros": "cinemómetro",
    "cineramas": "cinerama",
    "cineraria": "cinerario",
    "cinerarias": "cinerario",
    "cinerarios": "cinerario",
    "cinerinas": "cinerina",
    "cineritas": "cinerita",
    "cines": "cine",
    "cinescopios": "cinescopio",
    "cinesiologías": "cinesiología",
    "cinesiterapeutas": "cinesiterapeuta",
    "cinesiterapias": "cinesiterapia",
    "cinestesias": "cinestesia",
    "cinetoblastos": "cinetoblasto",
    "cinetocoros": "cinetocoro",
    "cinetocórica": "cinetocórico",
    "cinetocóricas": "cinetocórico",
    "cinetocóricos": "cinetocórico",
    "cinetoplasmas": "cinetoplasma",
    "cinetosomas": "cinetosoma",
    "cingalesa": "cingalés",
    "cingalesas": "cingalés",
    "cingaleses": "cingalés",
    "cingla": "cinglar",
    "cinglaba": "cinglar",
    "cinglabais": "cinglar",
    "cinglaban": "cinglar",
    "cinglabas": "cinglar",
    "cinglad": "cinglar",
    "cinglada": "cinglar",
    "cingladas": "cinglar",
    "cinglado": "cinglar",
    "cinglados": "cinglar",
    "cinglamos": "cinglar",
    "cinglan": "cinglar",
    "cinglando": "cinglar",
    "cinglara": "cinglar",
    "cinglarais": "cinglar",
    "cinglaran": "cinglar",
    "cinglaras": "cinglar",
    "cinglare": "cinglar",
    "cinglareis": "cinglar",
    "cinglaremos": "cinglar",
    "cinglaren": "cinglar",
    "cinglares": "cinglar",
    "cinglaron": "cinglar",
    "cinglará": "cinglar",
    "cinglarán": "cinglar",
    "cinglarás": "cinglar",
    "cinglaré": "cinglar",
    "cinglaréis": "cinglar",
    "cinglaría": "cinglar",
    "cinglaríais": "cinglar",
    "cinglaríamos": "cinglar",
    "cinglarían": "cinglar",
    "cinglarías": "cinglar",
    "cinglas": "cinglar",
    "cinglase": "cinglar",
    "cinglaseis": "cinglar",
    "cinglasen": "cinglar",
    "cinglases": "cinglar",
    "cinglaste": "cinglar",
    "cinglasteis": "cinglar",
    "cingle": "cinglar",
    "cinglemos": "cinglar",
    "cinglen": "cinglar",
    "cingles": "cinglar",
    "cingletas": "cingleta",
    "cinglo": "cinglar",
    "cinglábamos": "cinglar",
    "cingláis": "cinglar",
    "cingláramos": "cinglar",
    "cingláremos": "cinglar",
    "cinglásemos": "cinglar",
    "cinglé": "cinglar",
    "cingléis": "cinglar",
    "cingló": "cinglar",
    "cininas": "cinina",
    "cininógenos": "cininógeno",
    "cinismos": "cinismo",
    "cinocéfala": "cinocéfalo",
    "cinocéfalas": "cinocéfalo",
    "cinocéfalos": "cinocéfalo",
    "cinodontismos": "cinodontismo",
    "cinodontos": "cinodonto",
    "cinoplasmas": "cinoplasma",
    "cinosomas": "cinosoma",
    "cinque": "cincar",
    "cinquecentos": "cinquecento",
    "cinquemos": "cincar",
    "cinquen": "cincar",
    "cinques": "cincar",
    "cinqué": "cincar",
    "cinquéis": "cincar",
    "cinta": "cintar",
    "cintaba": "cintar",
    "cintabais": "cintar",
    "cintaban": "cintar",
    "cintabas": "cintar",
    "cintad": "cintar",
    "cintada": "cintar",
    "cintadas": "cintar",
    "cintado": "cintar",
    "cintados": "cintar",
    "cintagordas": "cintagorda",
    "cintamos": "cintar",
    "cintan": "cintar",
    "cintando": "cintar",
    "cintara": "cintar",
    "cintarais": "cintar",
    "cintaran": "cintar",
    "cintaras": "cintar",
    "cintarazos": "cintarazo",
    "cintare": "cintar",
    "cintarea": "cintarear",
    "cintareaba": "cintarear",
    "cintareabais": "cintarear",
    "cintareaban": "cintarear",
    "cintareabas": "cintarear",
    "cintaread": "cintarear",
    "cintareada": "cintarear",
    "cintareadas": "cintarear",
    "cintareado": "cintarear",
    "cintareados": "cintarear",
    "cintareamos": "cintarear",
    "cintarean": "cintarear",
    "cintareando": "cintarear",
    "cintareara": "cintarear",
    "cintarearais": "cintarear",
    "cintarearan": "cintarear",
    "cintarearas": "cintarear",
    "cintareare": "cintarear",
    "cintareareis": "cintarear",
    "cintarearemos": "cintarear",
    "cintarearen": "cintarear",
    "cintareares": "cintarear",
    "cintarearon": "cintarear",
    "cintareará": "cintarear",
    "cintarearán": "cintarear",
    "cintarearás": "cintarear",
    "cintarearé": "cintarear",
    "cintarearéis": "cintarear",
    "cintarearía": "cintarear",
    "cintarearíais": "cintarear",
    "cintarearíamos": "cintarear",
    "cintarearían": "cintarear",
    "cintarearías": "cintarear",
    "cintareas": "cintarear",
    "cintarease": "cintarear",
    "cintareaseis": "cintarear",
    "cintareasen": "cintarear",
    "cintareases": "cintarear",
    "cintareaste": "cintarear",
    "cintareasteis": "cintarear",
    "cintaree": "cintarear",
    "cintareemos": "cintarear",
    "cintareen": "cintarear",
    "cintarees": "cintarear",
    "cintareis": "cintar",
    "cintaremos": "cintar",
    "cintaren": "cintar",
    "cintareo": "cintarear",
    "cintares": "cintar",
    "cintareábamos": "cintarear",
    "cintareáis": "cintarear",
    "cintareáramos": "cintarear",
    "cintareáremos": "cintarear",
    "cintareásemos": "cintarear",
    "cintareé": "cintarear",
    "cintareéis": "cintarear",
    "cintareó": "cintarear",
    "cintaron": "cintar",
    "cintará": "cintar",
    "cintarán": "cintar",
    "cintarás": "cintar",
    "cintaré": "cintar",
    "cintaréis": "cintar",
    "cintaría": "cintar",
    "cintaríais": "cintar",
    "cintaríamos": "cintar",
    "cintarían": "cintar",
    "cintarías": "cintar",
    "cintas": "cintar",
    "cintase": "cintar",
    "cintaseis": "cintar",
    "cintasen": "cintar",
    "cintases": "cintar",
    "cintaste": "cintar",
    "cintasteis": "cintar",
    "cinte": "cintar",
    "cinteada": "cinteado",
    "cinteadas": "cinteado",
    "cinteados": "cinteado",
    "cintemos": "cintar",
    "cinten": "cintar",
    "cintera": "cintero",
    "cinteras": "cintero",
    "cinteros": "cintero",
    "cinterías": "cintería",
    "cintes": "cintar",
    "cintetas": "cinteta",
    "cintila": "cintilar",
    "cintilaba": "cintilar",
    "cintilabais": "cintilar",
    "cintilaban": "cintilar",
    "cintilabas": "cintilar",
    "cintilad": "cintilar",
    "cintilada": "cintilar",
    "cintiladas": "cintilar",
    "cintilado": "cintilar",
    "cintilados": "cintilar",
    "cintilamos": "cintilar",
    "cintilan": "cintilar",
    "cintilando": "cintilar",
    "cintilara": "cintilar",
    "cintilarais": "cintilar",
    "cintilaran": "cintilar",
    "cintilaras": "cintilar",
    "cintilare": "cintilar",
    "cintilareis": "cintilar",
    "cintilaremos": "cintilar",
    "cintilaren": "cintilar",
    "cintilares": "cintilar",
    "cintilaron": "cintilar",
    "cintilará": "cintilar",
    "cintilarán": "cintilar",
    "cintilarás": "cintilar",
    "cintilaré": "cintilar",
    "cintilaréis": "cintilar",
    "cintilaría": "cintilar",
    "cintilaríais": "cintilar",
    "cintilaríamos": "cintilar",
    "cintilarían": "cintilar",
    "cintilarías": "cintilar",
    "cintilas": "cintilar",
    "cintilase": "cintilar",
    "cintilaseis": "cintilar",
    "cintilasen": "cintilar",
    "cintilases": "cintilar",
    "cintilaste": "cintilar",
    "cintilasteis": "cintilar",
    "cintile": "cintilar",
    "cintilemos": "cintilar",
    "cintilen": "cintilar",
    "cintiles": "cintilar",
    "cintillas": "cintilla",
    "cintillos": "cintillo",
    "cintilo": "cintilar",
    "cintilábamos": "cintilar",
    "cintiláis": "cintilar",
    "cintiláramos": "cintilar",
    "cintiláremos": "cintilar",
    "cintilásemos": "cintilar",
    "cintilé": "cintilar",
    "cintiléis": "cintilar",
    "cintiló": "cintilar",
    "cinto": "cintar",
    "cintones": "cintón",
    "cintos": "cinto",
    "cintrada": "cintrado",
    "cintradas": "cintrado",
    "cintrados": "cintrado",
    "cintras": "cintra",
    "cintreles": "cintrel",
    "cinturas": "cintura",
    "cinturillas": "cinturilla",
    "cinturones": "cinturón",
    "cintábamos": "cintar",
    "cintáis": "cintar",
    "cintáramos": "cintar",
    "cintáremos": "cintar",
    "cintásemos": "cintar",
    "cinté": "cintar",
    "cintéis": "cintar",
    "cintó": "cintar",
    "cinzitas": "cinzita",
    "cinzolina": "cinzolín",
    "cinzolinas": "cinzolín",
    "cinzolines": "cinzolín",
    "cinámica": "cinámico",
    "cinámicas": "cinámico",
    "cinámicos": "cinámico",
    "cinéfila": "cinéfilo",
    "cinéfilas": "cinéfilo",
    "cinéfilos": "cinéfilo",
    "cinérea": "cinéreo",
    "cinéreas": "cinéreo",
    "cinéreos": "cinéreo",
    "cinética": "cinético",
    "cinéticas": "cinético",
    "cinéticos": "cinético",
    "cinípedos": "cinípedo",
    "cipayos": "cipayo",
    "cipeitas": "cipeita",
    "ciperas": "cipera",
    "cipolina": "cipolino",
    "cipolinas": "cipolino",
    "cipolinos": "cipolino",
    "cipos": "cipo",
    "cipotes": "cipote",
    "cipresales": "cipresal",
    "cipreses": "ciprés",
    "cipria": "ciprio",
    "ciprias": "ciprio",
    "ciprina": "ciprino",
    "ciprinas": "ciprino",
    "ciprinos": "ciprino",
    "ciprios": "ciprio",
    "cipriotas": "cipriota",
    "ciprínidos": "ciprínido",
    "cirates": "cirate",
    "circasiana": "circasiano",
    "circasianas": "circasiano",
    "circasianos": "circasiano",
    "circenses": "circense",
    "circonas": "circona",
    "circones": "circón",
    "circonios": "circonio",
    "circos": "circo",
    "circuiciones": "circuición",
    "circuitos": "circuito",
    "circula": "circular",
    "circulaba": "circular",
    "circulabais": "circular",
    "circulaban": "circular",
    "circulabas": "circular",
    "circulaciones": "circulación",
    "circulad": "circular",
    "circulada": "circular",
    "circuladas": "circular",
    "circulado": "circular",
    "circulados": "circular",
    "circulamos": "circular",
    "circulan": "circular",
    "circulando": "circular",
    "circulantes": "circulante",
    "circulara": "circular",
    "circularais": "circular",
    "circularan": "circular",
    "circularas": "circular",
    "circulare": "circular",
    "circulareis": "circular",
    "circularemos": "circular",
    "circularen": "circular",
    "circulares": "circular",
    "circularidades": "circularidad",
    "circularon": "circular",
    "circulará": "circular",
    "circularán": "circular",
    "circularás": "circular",
    "circularé": "circular",
    "circularéis": "circular",
    "circularía": "circular",
    "circularíais": "circular",
    "circularíamos": "circular",
    "circularían": "circular",
    "circularías": "circular",
    "circulas": "circular",
    "circulase": "circular",
    "circulaseis": "circular",
    "circulasen": "circular",
    "circulases": "circular",
    "circulaste": "circular",
    "circulasteis": "circular",
    "circulatoria": "circulatorio",
    "circulatorias": "circulatorio",
    "circulatorios": "circulatorio",
    "circule": "circular",
    "circulemos": "circular",
    "circulen": "circular",
    "circules": "circular",
    "circulo": "circular",
    "circulábamos": "circular",
    "circuláis": "circular",
    "circuláramos": "circular",
    "circuláremos": "circular",
    "circulásemos": "circular",
    "circulé": "circular",
    "circuléis": "circular",
    "circuló": "circular",
    "circumanales": "circumanal",
    "circumpolares": "circumpolar",
    "circuncentros": "circuncentro",
    "circuncida": "circuncidar",
    "circuncidaba": "circuncidar",
    "circuncidabais": "circuncidar",
    "circuncidaban": "circuncidar",
    "circuncidabas": "circuncidar",
    "circuncidad": "circuncidar",
    "circuncidada": "circuncidar",
    "circuncidadas": "circuncidar",
    "circuncidado": "circuncidar",
    "circuncidados": "circuncidar",
    "circuncidamos": "circuncidar",
    "circuncidan": "circuncidar",
    "circuncidando": "circuncidar",
    "circuncidara": "circuncidar",
    "circuncidarais": "circuncidar",
    "circuncidaran": "circuncidar",
    "circuncidaras": "circuncidar",
    "circuncidare": "circuncidar",
    "circuncidareis": "circuncidar",
    "circuncidaremos": "circuncidar",
    "circuncidaren": "circuncidar",
    "circuncidares": "circuncidar",
    "circuncidaron": "circuncidar",
    "circuncidará": "circuncidar",
    "circuncidarán": "circuncidar",
    "circuncidarás": "circuncidar",
    "circuncidaré": "circuncidar",
    "circuncidaréis": "circuncidar",
    "circuncidaría": "circuncidar",
    "circuncidaríais": "circuncidar",
    "circuncidaríamos": "circuncidar",
    "circuncidarían": "circuncidar",
    "circuncidarías": "circuncidar",
    "circuncidas": "circuncidar",
    "circuncidase": "circuncidar",
    "circuncidaseis": "circuncidar",
    "circuncidasen": "circuncidar",
    "circuncidases": "circuncidar",
    "circuncidaste": "circuncidar",
    "circuncidasteis": "circuncidar",
    "circuncide": "circuncidar",
    "circuncidemos": "circuncidar",
    "circunciden": "circuncidar",
    "circuncides": "circuncidar",
    "circuncido": "circuncidar",
    "circuncidábamos": "circuncidar",
    "circuncidáis": "circuncidar",
    "circuncidáramos": "circuncidar",
    "circuncidáremos": "circuncidar",
    "circuncidásemos": "circuncidar",
    "circuncidé": "circuncidar",
    "circuncidéis": "circuncidar",
    "circuncidó": "circuncidar",
    "circuncisa": "circunciso",
    "circuncisas": "circunciso",
    "circuncisiones": "circuncisión",
    "circuncisos": "circunciso",
    "circunda": "circundar",
    "circundaba": "circundar",
    "circundabais": "circundar",
    "circundaban": "circundar",
    "circundabas": "circundar",
    "circundad": "circundar",
    "circundada": "circundar",
    "circundadas": "circundar",
    "circundado": "circundar",
    "circundados": "circundar",
    "circundamos": "circundar",
    "circundan": "circundar",
    "circundando": "circundar",
    "circundantes": "circundante",
    "circundara": "circundar",
    "circundarais": "circundar",
    "circundaran": "circundar",
    "circundaras": "circundar",
    "circundare": "circundar",
    "circundareis": "circundar",
    "circundaremos": "circundar",
    "circundaren": "circundar",
    "circundares": "circundar",
    "circundaron": "circundar",
    "circundará": "circundar",
    "circundarán": "circundar",
    "circundarás": "circundar",
    "circundaré": "circundar",
    "circundaréis": "circundar",
    "circundaría": "circundar",
    "circundaríais": "circundar",
    "circundaríamos": "circundar",
    "circundarían": "circundar",
    "circundarías": "circundar",
    "circundas": "circundar",
    "circundase": "circundar",
    "circundaseis": "circundar",
    "circundasen": "circundar",
    "circundases": "circundar",
    "circundaste": "circundar",
    "circundasteis": "circundar",
    "circunde": "circundar",
    "circundemos": "circundar",
    "circunden": "circundar",
    "circundes": "circundar",
    "circundo": "circundar",
    "circunducciones": "circunducción",
    "circundábamos": "circundar",
    "circundáis": "circundar",
    "circundáramos": "circundar",
    "circundáremos": "circundar",
    "circundásemos": "circundar",
    "circundé": "circundar",
    "circundéis": "circundar",
    "circundó": "circundar",
    "circunferenciales": "circunferencial",
    "circunferencias": "circunferencia",
    "circunferentes": "circunferente",
    "circunferid": "circunferir",
    "circunferida": "circunferir",
    "circunferidas": "circunferir",
    "circunferido": "circunferir",
    "circunferidos": "circunferir",
    "circunferimos": "circunferir",
    "circunferiremos": "circunferir",
    "circunferirá": "circunferir",
    "circunferirán": "circunferir",
    "circunferirás": "circunferir",
    "circunferiré": "circunferir",
    "circunferiréis": "circunferir",
    "circunferiría": "circunferir",
    "circunferiríais": "circunferir",
    "circunferiríamos": "circunferir",
    "circunferirían": "circunferir",
    "circunferirías": "circunferir",
    "circunferiste": "circunferir",
    "circunferisteis": "circunferir",
    "circunferí": "circunferir",
    "circunfería": "circunferir",
    "circunferíais": "circunferir",
    "circunferíamos": "circunferir",
    "circunferían": "circunferir",
    "circunferías": "circunferir",
    "circunferís": "circunferir",
    "circunfiera": "circunferir",
    "circunfieran": "circunferir",
    "circunfieras": "circunferir",
    "circunfiere": "circunferir",
    "circunfieren": "circunferir",
    "circunfieres": "circunferir",
    "circunfiero": "circunferir",
    "circunfiramos": "circunferir",
    "circunfiriendo": "circunferir",
    "circunfiriera": "circunferir",
    "circunfirierais": "circunferir",
    "circunfirieran": "circunferir",
    "circunfirieras": "circunferir",
    "circunfiriere": "circunferir",
    "circunfiriereis": "circunferir",
    "circunfirieren": "circunferir",
    "circunfirieres": "circunferir",
    "circunfirieron": "circunferir",
    "circunfiriese": "circunferir",
    "circunfirieseis": "circunferir",
    "circunfiriesen": "circunferir",
    "circunfirieses": "circunferir",
    "circunfiriéramos": "circunferir",
    "circunfiriéremos": "circunferir",
    "circunfiriésemos": "circunferir",
    "circunfirió": "circunferir",
    "circunfiráis": "circunferir",
    "circunfleja": "circunflejo",
    "circunflejas": "circunflejo",
    "circunflejos": "circunflejo",
    "circunfusa": "circunfuso",
    "circunfusas": "circunfuso",
    "circunfusos": "circunfuso",
    "circunlocuciones": "circunlocución",
    "circunloquios": "circunloquio",
    "circunnavega": "circunnavegar",
    "circunnavegaba": "circunnavegar",
    "circunnavegabais": "circunnavegar",
    "circunnavegaban": "circunnavegar",
    "circunnavegabas": "circunnavegar",
    "circunnavegaciones": "circunnavegación",
    "circunnavegad": "circunnavegar",
    "circunnavegada": "circunnavegar",
    "circunnavegadas": "circunnavegar",
    "circunnavegado": "circunnavegar",
    "circunnavegados": "circunnavegar",
    "circunnavegamos": "circunnavegar",
    "circunnavegan": "circunnavegar",
    "circunnavegando": "circunnavegar",
    "circunnavegara": "circunnavegar",
    "circunnavegarais": "circunnavegar",
    "circunnavegaran": "circunnavegar",
    "circunnavegaras": "circunnavegar",
    "circunnavegare": "circunnavegar",
    "circunnavegareis": "circunnavegar",
    "circunnavegaremos": "circunnavegar",
    "circunnavegaren": "circunnavegar",
    "circunnavegares": "circunnavegar",
    "circunnavegaron": "circunnavegar",
    "circunnavegará": "circunnavegar",
    "circunnavegarán": "circunnavegar",
    "circunnavegarás": "circunnavegar",
    "circunnavegaré": "circunnavegar",
    "circunnavegaréis": "circunnavegar",
    "circunnavegaría": "circunnavegar",
    "circunnavegaríais": "circunnavegar",
    "circunnavegaríamos": "circunnavegar",
    "circunnavegarían": "circunnavegar",
    "circunnavegarías": "circunnavegar",
    "circunnavegas": "circunnavegar",
    "circunnavegase": "circunnavegar",
    "circunnavegaseis": "circunnavegar",
    "circunnavegasen": "circunnavegar",
    "circunnavegases": "circunnavegar",
    "circunnavegaste": "circunnavegar",
    "circunnavegasteis": "circunnavegar",
    "circunnavego": "circunnavegar",
    "circunnavegue": "circunnavegar",
    "circunnaveguemos": "circunnavegar",
    "circunnaveguen": "circunnavegar",
    "circunnavegues": "circunnavegar",
    "circunnavegué": "circunnavegar",
    "circunnaveguéis": "circunnavegar",
    "circunnavegábamos": "circunnavegar",
    "circunnavegáis": "circunnavegar",
    "circunnavegáramos": "circunnavegar",
    "circunnavegáremos": "circunnavegar",
    "circunnavegásemos": "circunnavegar",
    "circunnavegó": "circunnavegar",
    "circunscriba": "circunscribir",
    "circunscribamos": "circunscribir",
    "circunscriban": "circunscribir",
    "circunscribas": "circunscribir",
    "circunscribe": "circunscribir",
    "circunscriben": "circunscribir",
    "circunscribes": "circunscribir",
    "circunscribid": "circunscribir",
    "circunscribiendo": "circunscribir",
    "circunscribiera": "circunscribir",
    "circunscribierais": "circunscribir",
    "circunscribieran": "circunscribir",
    "circunscribieras": "circunscribir",
    "circunscribiere": "circunscribir",
    "circunscribiereis": "circunscribir",
    "circunscribieren": "circunscribir",
    "circunscribieres": "circunscribir",
    "circunscribieron": "circunscribir",
    "circunscribiese": "circunscribir",
    "circunscribieseis": "circunscribir",
    "circunscribiesen": "circunscribir",
    "circunscribieses": "circunscribir",
    "circunscribimos": "circunscribir",
    "circunscribiremos": "circunscribir",
    "circunscribirá": "circunscribir",
    "circunscribirán": "circunscribir",
    "circunscribirás": "circunscribir",
    "circunscribiré": "circunscribir",
    "circunscribiréis": "circunscribir",
    "circunscribiría": "circunscribir",
    "circunscribiríais": "circunscribir",
    "circunscribiríamos": "circunscribir",
    "circunscribirían": "circunscribir",
    "circunscribirías": "circunscribir",
    "circunscribiste": "circunscribir",
    "circunscribisteis": "circunscribir",
    "circunscribiéramos": "circunscribir",
    "circunscribiéremos": "circunscribir",
    "circunscribiésemos": "circunscribir",
    "circunscribió": "circunscribir",
    "circunscribo": "circunscribir",
    "circunscribáis": "circunscribir",
    "circunscribí": "circunscribir",
    "circunscribía": "circunscribir",
    "circunscribíais": "circunscribir",
    "circunscribíamos": "circunscribir",
    "circunscribían": "circunscribir",
    "circunscribías": "circunscribir",
    "circunscribís": "circunscribir",
    "circunscripciones": "circunscripción",
    "circunscrita": "circunscribir",
    "circunscritas": "circunscribir",
    "circunscrito": "circunscribir",
    "circunscritos": "circunscribir",
    "circunsolares": "circunsolar",
    "circunspecciones": "circunspección",
    "circunspecta": "circunspecto",
    "circunspectas": "circunspecto",
    "circunspectos": "circunspecto",
    "circunstanciada": "circunstanciado",
    "circunstanciadas": "circunstanciado",
    "circunstanciados": "circunstanciado",
    "circunstanciales": "circunstancial",
    "circunstancias": "circunstancia",
    "circunstantes": "circunstante",
    "circunvala": "circunvalar",
    "circunvalaba": "circunvalar",
    "circunvalabais": "circunvalar",
    "circunvalaban": "circunvalar",
    "circunvalabas": "circunvalar",
    "circunvalaciones": "circunvalación",
    "circunvalad": "circunvalar",
    "circunvalada": "circunvalar",
    "circunvaladas": "circunvalar",
    "circunvalado": "circunvalar",
    "circunvalados": "circunvalar",
    "circunvalamos": "circunvalar",
    "circunvalan": "circunvalar",
    "circunvalando": "circunvalar",
    "circunvalara": "circunvalar",
    "circunvalarais": "circunvalar",
    "circunvalaran": "circunvalar",
    "circunvalaras": "circunvalar",
    "circunvalare": "circunvalar",
    "circunvalareis": "circunvalar",
    "circunvalaremos": "circunvalar",
    "circunvalaren": "circunvalar",
    "circunvalares": "circunvalar",
    "circunvalaron": "circunvalar",
    "circunvalará": "circunvalar",
    "circunvalarán": "circunvalar",
    "circunvalarás": "circunvalar",
    "circunvalaré": "circunvalar",
    "circunvalaréis": "circunvalar",
    "circunvalaría": "circunvalar",
    "circunvalaríais": "circunvalar",
    "circunvalaríamos": "circunvalar",
    "circunvalarían": "circunvalar",
    "circunvalarías": "circunvalar",
    "circunvalas": "circunvalar",
    "circunvalase": "circunvalar",
    "circunvalaseis": "circunvalar",
    "circunvalasen": "circunvalar",
    "circunvalases": "circunvalar",
    "circunvalaste": "circunvalar",
    "circunvalasteis": "circunvalar",
    "circunvale": "circunvalar",
    "circunvalemos": "circunvalar",
    "circunvalen": "circunvalar",
    "circunvales": "circunvalar",
    "circunvalo": "circunvalar",
    "circunvalábamos": "circunvalar",
    "circunvaláis": "circunvalar",
    "circunvaláramos": "circunvalar",
    "circunvaláremos": "circunvalar",
    "circunvalásemos": "circunvalar",
    "circunvalé": "circunvalar",
    "circunvaléis": "circunvalar",
    "circunvaló": "circunvalar",
    "circunvecina": "circunvecino",
    "circunvecinas": "circunvecino",
    "circunvecinos": "circunvecino",
    "circunvolaba": "circunvolar",
    "circunvolabais": "circunvolar",
    "circunvolaban": "circunvolar",
    "circunvolabas": "circunvolar",
    "circunvolad": "circunvolar",
    "circunvolada": "circunvolar",
    "circunvoladas": "circunvolar",
    "circunvolado": "circunvolar",
    "circunvolados": "circunvolar",
    "circunvolamos": "circunvolar",
    "circunvolando": "circunvolar",
    "circunvolara": "circunvolar",
    "circunvolarais": "circunvolar",
    "circunvolaran": "circunvolar",
    "circunvolaras": "circunvolar",
    "circunvolare": "circunvolar",
    "circunvolareis": "circunvolar",
    "circunvolaremos": "circunvolar",
    "circunvolaren": "circunvolar",
    "circunvolares": "circunvolar",
    "circunvolaron": "circunvolar",
    "circunvolará": "circunvolar",
    "circunvolarán": "circunvolar",
    "circunvolarás": "circunvolar",
    "circunvolaré": "circunvolar",
    "circunvolaréis": "circunvolar",
    "circunvolaría": "circunvolar",
    "circunvolaríais": "circunvolar",
    "circunvolaríamos": "circunvolar",
    "circunvolarían": "circunvolar",
    "circunvolarías": "circunvolar",
    "circunvolase": "circunvolar",
    "circunvolaseis": "circunvolar",
    "circunvolasen": "circunvolar",
    "circunvolases": "circunvolar",
    "circunvolaste": "circunvolar",
    "circunvolasteis": "circunvolar",
    "circunvolemos": "circunvolar",
    "circunvoluciones": "circunvolución",
    "circunvolábamos": "circunvolar",
    "circunvoláis": "circunvolar",
    "circunvoláramos": "circunvolar",
    "circunvoláremos": "circunvolar",
    "circunvolásemos": "circunvolar",
    "circunvolé": "circunvolar",
    "circunvoléis": "circunvolar",
    "circunvoló": "circunvolar",
    "circunvuela": "circunvolar",
    "circunvuelan": "circunvolar",
    "circunvuelas": "circunvolar",
    "circunvuele": "circunvolar",
    "circunvuelen": "circunvolar",
    "circunvueles": "circunvolar",
    "circunvuelo": "circunvolar",
    "cirenaica": "cirenaico",
    "cirenaicas": "cirenaico",
    "cirenaicos": "cirenaico",
    "ciriales": "cirial",
    "cirinea": "cirineo",
    "cirineas": "cirineo",
    "cirineos": "cirineo",
    "cirios": "cirio",
    "cirrocúmulos": "cirrocúmulo",
    "cirroestratos": "cirroestrato",
    "cirros": "cirro",
    "cirrosa": "cirroso",
    "cirrosas": "cirroso",
    "cirrosos": "cirroso",
    "cirrostratos": "cirrostrato",
    "cirrípedos": "cirrípedo",
    "cirrógena": "cirrógeno",
    "cirrógenas": "cirrógeno",
    "cirrógenos": "cirrógeno",
    "cirrótica": "cirrótico",
    "cirróticas": "cirrótico",
    "cirróticos": "cirrótico",
    "ciruelas": "ciruela",
    "ciruelos": "ciruelo",
    "cirugías": "cirugía",
    "cirujana": "cirujano",
    "cirujanas": "cirujano",
    "cirujanos": "cirujano",
    "cirílica": "cirílico",
    "cirílicas": "cirílico",
    "cirílicos": "cirílico",
    "cisalpina": "cisalpino",
    "cisalpinas": "cisalpino",
    "cisalpinos": "cisalpino",
    "cisandina": "cisandino",
    "cisandinas": "cisandino",
    "cisandinos": "cisandino",
    "cisca": "ciscar",
    "ciscaba": "ciscar",
    "ciscabais": "ciscar",
    "ciscaban": "ciscar",
    "ciscabas": "ciscar",
    "ciscad": "ciscar",
    "ciscada": "ciscar",
    "ciscadas": "ciscar",
    "ciscado": "ciscar",
    "ciscados": "ciscar",
    "ciscamos": "ciscar",
    "ciscan": "ciscar",
    "ciscando": "ciscar",
    "ciscara": "ciscar",
    "ciscarais": "ciscar",
    "ciscaran": "ciscar",
    "ciscaras": "ciscar",
    "ciscare": "ciscar",
    "ciscareis": "ciscar",
    "ciscaremos": "ciscar",
    "ciscaren": "ciscar",
    "ciscares": "ciscar",
    "ciscaron": "ciscar",
    "ciscará": "ciscar",
    "ciscarán": "ciscar",
    "ciscarás": "ciscar",
    "ciscaré": "ciscar",
    "ciscaréis": "ciscar",
    "ciscaría": "ciscar",
    "ciscaríais": "ciscar",
    "ciscaríamos": "ciscar",
    "ciscarían": "ciscar",
    "ciscarías": "ciscar",
    "ciscas": "ciscar",
    "ciscase": "ciscar",
    "ciscaseis": "ciscar",
    "ciscasen": "ciscar",
    "ciscases": "ciscar",
    "ciscaste": "ciscar",
    "ciscasteis": "ciscar",
    "cisco": "ciscar",
    "ciscona": "ciscón",
    "cisconas": "ciscón",
    "ciscones": "ciscón",
    "ciscos": "cisco",
    "ciscábamos": "ciscar",
    "ciscáis": "ciscar",
    "ciscáramos": "ciscar",
    "ciscáremos": "ciscar",
    "ciscásemos": "ciscar",
    "ciscó": "ciscar",
    "cisiones": "cisión",
    "cisjordana": "cisjordano",
    "cisjordanas": "cisjordano",
    "cisjordanos": "cisjordano",
    "cismas": "cisma",
    "cismontana": "cismontano",
    "cismontanas": "cismontano",
    "cismontanos": "cismontano",
    "cismática": "cismático",
    "cismáticas": "cismático",
    "cismáticos": "cismático",
    "cisnes": "cisne",
    "cisoides": "cisoide",
    "cisorias": "cisoria",
    "cisque": "ciscar",
    "cisquemos": "ciscar",
    "cisquen": "ciscar",
    "cisqueras": "cisquera",
    "cisques": "ciscar",
    "cisqué": "ciscar",
    "cisquéis": "ciscar",
    "cistacinas": "cistacina",
    "cistalgias": "cistalgia",
    "cistas": "cista",
    "cistectomías": "cistectomía",
    "cistercienses": "cisterciense",
    "cisternas": "cisterna",
    "cisteínas": "cisteína",
    "cisticercos": "cisticerco",
    "cistinas": "cistina",
    "cistolitos": "cistolito",
    "cistopatías": "cistopatía",
    "cistoscopias": "cistoscopia",
    "cistoscopios": "cistoscopio",
    "cistotomías": "cistotomía",
    "cistáceas": "cistácea",
    "cisuras": "cisura",
    "cisípeda": "cisípedo",
    "cisípedas": "cisípedo",
    "cisípedos": "cisípedo",
    "cita": "citar",
    "citaba": "citar",
    "citabais": "citar",
    "citaban": "citar",
    "citabas": "citar",
    "citaciones": "citación",
    "citad": "citar",
    "citada": "citar",
    "citadas": "citar",
    "citado": "citar",
    "citadora": "citador",
    "citadoras": "citador",
    "citadores": "citador",
    "citados": "citar",
    "citamos": "citar",
    "citan": "citar",
    "citando": "citar",
    "citanias": "citania",
    "citara": "citar",
    "citarais": "citar",
    "citaran": "citar",
    "citaras": "citara",
    "citare": "citar",
    "citareis": "citar",
    "citaremos": "citar",
    "citaren": "citar",
    "citares": "citar",
    "citaristas": "citarista",
    "citaron": "citar",
    "citarones": "citarón",
    "citará": "citar",
    "citarán": "citar",
    "citarás": "citar",
    "citaré": "citar",
    "citaréis": "citar",
    "citaría": "citar",
    "citaríais": "citar",
    "citaríamos": "citar",
    "citarían": "citar",
    "citarías": "citar",
    "citas": "citar",
    "citase": "citar",
    "citaseis": "citar",
    "citasen": "citar",
    "citases": "citar",
    "citaste": "citar",
    "citasteis": "citar",
    "citatoria": "citatorio",
    "citatorias": "citatorio",
    "citatorios": "citatorio",
    "cite": "citar",
    "citelos": "citelo",
    "citemos": "citar",
    "citen": "citar",
    "citeriores": "citerior",
    "cites": "citar",
    "cities": "city",
    "cito": "citar",
    "citoblastos": "citoblasto",
    "citocalasinas": "citocalasina",
    "citocentros": "citocentro",
    "citocromos": "citocromo",
    "citoesqueletos": "citoesqueleto",
    "citofílica": "citofílico",
    "citofílicas": "citofílico",
    "citofílicos": "citofílico",
    "citogenes": "citogén",
    "citogenéticas": "citogenética",
    "citohistologías": "citohistología",
    "citolisinas": "citolisina",
    "citologías": "citología",
    "citolítica": "citolítico",
    "citolíticas": "citolítico",
    "citolíticos": "citolítico",
    "citológica": "citológico",
    "citológicas": "citológico",
    "citológicos": "citológico",
    "citomas": "citoma",
    "citomegalovíruses": "citomegalovirus",
    "citopenias": "citopenia",
    "citoplasmas": "citoplasma",
    "citoplasmática": "citoplasmático",
    "citoplasmáticas": "citoplasmático",
    "citoplasmáticos": "citoplasmático",
    "citoplásmica": "citoplásmico",
    "citoplásmicas": "citoplásmico",
    "citoplásmicos": "citoplásmico",
    "citoqueratinas": "citoqueratina",
    "citoquinas": "citoquina",
    "citosinas": "citosina",
    "citosoles": "citosol",
    "citosomas": "citosoma",
    "citostomas": "citostoma",
    "citostática": "citostático",
    "citostáticas": "citostático",
    "citostáticos": "citostático",
    "citotoxicidades": "citotoxicidad",
    "citotrópica": "citotrópico",
    "citotrópicas": "citotrópico",
    "citotrópicos": "citotrópico",
    "citotóxica": "citotóxico",
    "citotóxicas": "citotóxico",
    "citotóxicos": "citotóxico",
    "citrales": "citral",
    "citratos": "citrato",
    "citreoviridinas": "citreoviridina",
    "citriculturas": "citricultura",
    "citrinas": "citrina",
    "citrininas": "citrinina",
    "citrobácteres": "citrobácter",
    "citronelales": "citronelal",
    "citrícolas": "citrícola",
    "citábamos": "citar",
    "citáis": "citar",
    "citáramos": "citar",
    "citáremos": "citar",
    "citásemos": "citar",
    "citásteres": "citáster",
    "cité": "citar",
    "citéis": "citar",
    "citó": "citar",
    "citógena": "citógeno",
    "citógenas": "citógeno",
    "citógenos": "citógeno",
    "citóloga": "citólogo",
    "citólogas": "citólogo",
    "citólogos": "citólogo",
    "citómetros": "citómetro",
    "ciudadana": "ciudadano",
    "ciudadanas": "ciudadano",
    "ciudadanos": "ciudadano",
    "ciudadanías": "ciudadanía",
    "ciudadelas": "ciudadela",
    "ciudades": "ciudad",
    "ciudadrealeña": "ciudadrealeño",
    "ciudadrealeñas": "ciudadrealeño",
    "ciudadrealeños": "ciudadrealeño",
    "civetas": "civeta",
    "civetos": "civeto",
    "civiles": "civil",
    "civilice": "civilizar",
    "civilicemos": "civilizar",
    "civilicen": "civilizar",
    "civilices": "civilizar",
    "civilicé": "civilizar",
    "civilicéis": "civilizar",
    "civilistas": "civilista",
    "civiliza": "civilizar",
    "civilizaba": "civilizar",
    "civilizabais": "civilizar",
    "civilizaban": "civilizar",
    "civilizabas": "civilizar",
    "civilizables": "civilizable",
    "civilizacionales": "civilizacional",
    "civilizaciones": "civilización",
    "civilizad": "civilizar",
    "civilizada": "civilizar",
    "civilizadas": "civilizar",
    "civilizado": "civilizar",
    "civilizadora": "civilizador",
    "civilizadoras": "civilizador",
    "civilizadores": "civilizador",
    "civilizados": "civilizar",
    "civilizamos": "civilizar",
    "civilizan": "civilizar",
    "civilizando": "civilizar",
    "civilizara": "civilizar",
    "civilizarais": "civilizar",
    "civilizaran": "civilizar",
    "civilizaras": "civilizar",
    "civilizare": "civilizar",
    "civilizareis": "civilizar",
    "civilizaremos": "civilizar",
    "civilizaren": "civilizar",
    "civilizares": "civilizar",
    "civilizaron": "civilizar",
    "civilizará": "civilizar",
    "civilizarán": "civilizar",
    "civilizarás": "civilizar",
    "civilizaré": "civilizar",
    "civilizaréis": "civilizar",
    "civilizaría": "civilizar",
    "civilizaríais": "civilizar",
    "civilizaríamos": "civilizar",
    "civilizarían": "civilizar",
    "civilizarías": "civilizar",
    "civilizas": "civilizar",
    "civilizase": "civilizar",
    "civilizaseis": "civilizar",
    "civilizasen": "civilizar",
    "civilizases": "civilizar",
    "civilizaste": "civilizar",
    "civilizasteis": "civilizar",
    "civilizatoria": "civilizatorio",
    "civilizatorias": "civilizatorio",
    "civilizatorios": "civilizatorio",
    "civilizo": "civilizar",
    "civilizábamos": "civilizar",
    "civilizáis": "civilizar",
    "civilizáramos": "civilizar",
    "civilizáremos": "civilizar",
    "civilizásemos": "civilizar",
    "civilizó": "civilizar",
    "civismos": "civismo",
    "cizalla": "cizallar",
    "cizallaba": "cizallar",
    "cizallabais": "cizallar",
    "cizallaban": "cizallar",
    "cizallabas": "cizallar",
    "cizallad": "cizallar",
    "cizallada": "cizallar",
    "cizalladas": "cizallar",
    "cizallado": "cizallar",
    "cizalladora": "cizallador",
    "cizalladoras": "cizallador",
    "cizalladores": "cizallador",
    "cizallados": "cizallar",
    "cizalladuras": "cizalladura",
    "cizallamientos": "cizallamiento",
    "cizallamos": "cizallar",
    "cizallan": "cizallar",
    "cizallando": "cizallar",
    "cizallara": "cizallar",
    "cizallarais": "cizallar",
    "cizallaran": "cizallar",
    "cizallaras": "cizallar",
    "cizallare": "cizallar",
    "cizallareis": "cizallar",
    "cizallaremos": "cizallar",
    "cizallaren": "cizallar",
    "cizallares": "cizallar",
    "cizallaron": "cizallar",
    "cizallará": "cizallar",
    "cizallarán": "cizallar",
    "cizallarás": "cizallar",
    "cizallaré": "cizallar",
    "cizallaréis": "cizallar",
    "cizallaría": "cizallar",
    "cizallaríais": "cizallar",
    "cizallaríamos": "cizallar",
    "cizallarían": "cizallar",
    "cizallarías": "cizallar",
    "cizallas": "cizallar",
    "cizallase": "cizallar",
    "cizallaseis": "cizallar",
    "cizallasen": "cizallar",
    "cizallases": "cizallar",
    "cizallaste": "cizallar",
    "cizallasteis": "cizallar",
    "cizalle": "cizallar",
    "cizallemos": "cizallar",
    "cizallen": "cizallar",
    "cizalles": "cizallar",
    "cizallo": "cizallar",
    "cizallábamos": "cizallar",
    "cizalláis": "cizallar",
    "cizalláramos": "cizallar",
    "cizalláremos": "cizallar",
    "cizallásemos": "cizallar",
    "cizallé": "cizallar",
    "cizalléis": "cizallar",
    "cizalló": "cizallar",
    "cizaña": "cizañar",
    "cizañaba": "cizañar",
    "cizañabais": "cizañar",
    "cizañaban": "cizañar",
    "cizañabas": "cizañar",
    "cizañad": "cizañar",
    "cizañada": "cizañar",
    "cizañadas": "cizañar",
    "cizañado": "cizañar",
    "cizañadora": "cizañador",
    "cizañadoras": "cizañador",
    "cizañadores": "cizañador",
    "cizañados": "cizañar",
    "cizañamos": "cizañar",
    "cizañan": "cizañar",
    "cizañando": "cizañar",
    "cizañara": "cizañar",
    "cizañarais": "cizañar",
    "cizañaran": "cizañar",
    "cizañaras": "cizañar",
    "cizañare": "cizañar",
    "cizañareis": "cizañar",
    "cizañaremos": "cizañar",
    "cizañaren": "cizañar",
    "cizañares": "cizañar",
    "cizañaron": "cizañar",
    "cizañará": "cizañar",
    "cizañarán": "cizañar",
    "cizañarás": "cizañar",
    "cizañaré": "cizañar",
    "cizañaréis": "cizañar",
    "cizañaría": "cizañar",
    "cizañaríais": "cizañar",
    "cizañaríamos": "cizañar",
    "cizañarían": "cizañar",
    "cizañarías": "cizañar",
    "cizañas": "cizañar",
    "cizañase": "cizañar",
    "cizañaseis": "cizañar",
    "cizañasen": "cizañar",
    "cizañases": "cizañar",
    "cizañaste": "cizañar",
    "cizañasteis": "cizañar",
    "cizañe": "cizañar",
    "cizañea": "cizañear",
    "cizañeaba": "cizañear",
    "cizañeabais": "cizañear",
    "cizañeaban": "cizañear",
    "cizañeabas": "cizañear",
    "cizañead": "cizañear",
    "cizañeada": "cizañear",
    "cizañeadas": "cizañear",
    "cizañeado": "cizañear",
    "cizañeados": "cizañear",
    "cizañeamos": "cizañear",
    "cizañean": "cizañear",
    "cizañeando": "cizañear",
    "cizañeara": "cizañear",
    "cizañearais": "cizañear",
    "cizañearan": "cizañear",
    "cizañearas": "cizañear",
    "cizañeare": "cizañear",
    "cizañeareis": "cizañear",
    "cizañearemos": "cizañear",
    "cizañearen": "cizañear",
    "cizañeares": "cizañear",
    "cizañearon": "cizañear",
    "cizañeará": "cizañear",
    "cizañearán": "cizañear",
    "cizañearás": "cizañear",
    "cizañearé": "cizañear",
    "cizañearéis": "cizañear",
    "cizañearía": "cizañear",
    "cizañearíais": "cizañear",
    "cizañearíamos": "cizañear",
    "cizañearían": "cizañear",
    "cizañearías": "cizañear",
    "cizañeas": "cizañear",
    "cizañease": "cizañear",
    "cizañeaseis": "cizañear",
    "cizañeasen": "cizañear",
    "cizañeases": "cizañear",
    "cizañeaste": "cizañear",
    "cizañeasteis": "cizañear",
    "cizañee": "cizañear",
    "cizañeemos": "cizañear",
    "cizañeen": "cizañear",
    "cizañees": "cizañear",
    "cizañemos": "cizañar",
    "cizañen": "cizañar",
    "cizañeo": "cizañear",
    "cizañera": "cizañero",
    "cizañeras": "cizañero",
    "cizañeros": "cizañero",
    "cizañes": "cizañar",
    "cizañeábamos": "cizañear",
    "cizañeáis": "cizañear",
    "cizañeáramos": "cizañear",
    "cizañeáremos": "cizañear",
    "cizañeásemos": "cizañear",
    "cizañeé": "cizañear",
    "cizañeéis": "cizañear",
    "cizañeó": "cizañear",
    "cizaño": "cizañar",
    "cizañábamos": "cizañar",
    "cizañáis": "cizañar",
    "cizañáramos": "cizañar",
    "cizañáremos": "cizañar",
    "cizañásemos": "cizañar",
    "cizañé": "cizañar",
    "cizañéis": "cizañar",
    "cizañó": "cizañar",
    "ciábamos": "ciar",
    "ciáis": "ciar",
    "ciáramos": "ciar",
    "ciáremos": "ciar",
    "ciásemos": "ciar",
    "ciática": "ciático",
    "ciáticas": "ciático",
    "ciáticos": "ciático",
    "cié": "ciar",
    "ciéis": "ciar",
    "ciénagas": "ciénaga",
    "ciénegas": "ciénega",
    "ciénegos": "ciénego",
    "ciña": "ceñir",
    "ciñamos": "ceñir",
    "ciñan": "ceñir",
    "ciñas": "ceñir",
    "ciñe": "ceñir",
    "ciñen": "ceñir",
    "ciñendo": "ceñir",
    "ciñera": "ceñir",
    "ciñerais": "ceñir",
    "ciñeran": "ceñir",
    "ciñeras": "ceñir",
    "ciñere": "ceñir",
    "ciñereis": "ceñir",
    "ciñeren": "ceñir",
    "ciñeres": "ceñir",
    "ciñeron": "ceñir",
    "ciñes": "ceñir",
    "ciñese": "ceñir",
    "ciñeseis": "ceñir",
    "ciñesen": "ceñir",
    "ciñeses": "ceñir",
    "ciño": "ceñir",
    "ciñáis": "ceñir",
    "ciñéramos": "ceñir",
    "ciñéremos": "ceñir",
    "ciñésemos": "ceñir",
    "ciñó": "ceñir",
    "ció": "ciar",
    "cl": "centilitro",
    "clacas": "claca",
    "clacs": "clac",
    "cladodios": "cladodio",
    "cladogramas": "cladograma",
    "clados": "clado",
    "cladóceros": "cladócero",
    "clama": "clamar",
    "clamaba": "clamar",
    "clamabais": "clamar",
    "clamaban": "clamar",
    "clamabas": "clamar",
    "clamad": "clamar",
    "clamada": "clamar",
    "clamadas": "clamar",
    "clamado": "clamar",
    "clamados": "clamar",
    "clamamos": "clamar",
    "claman": "clamar",
    "clamando": "clamar",
    "clamara": "clamar",
    "clamarais": "clamar",
    "clamaran": "clamar",
    "clamaras": "clamar",
    "clamare": "clamar",
    "clamareis": "clamar",
    "clamaremos": "clamar",
    "clamaren": "clamar",
    "clamares": "clamar",
    "clamaron": "clamar",
    "clamará": "clamar",
    "clamarán": "clamar",
    "clamarás": "clamar",
    "clamaré": "clamar",
    "clamaréis": "clamar",
    "clamaría": "clamar",
    "clamaríais": "clamar",
    "clamaríamos": "clamar",
    "clamarían": "clamar",
    "clamarías": "clamar",
    "clamas": "clamar",
    "clamase": "clamar",
    "clamaseis": "clamar",
    "clamasen": "clamar",
    "clamases": "clamar",
    "clamaste": "clamar",
    "clamasteis": "clamar",
    "clame": "clamar",
    "clamemos": "clamar",
    "clamen": "clamar",
    "clames": "clamar",
    "clamo": "clamar",
    "clamorea": "clamorear",
    "clamoreaba": "clamorear",
    "clamoreabais": "clamorear",
    "clamoreaban": "clamorear",
    "clamoreabas": "clamorear",
    "clamoread": "clamorear",
    "clamoreada": "clamorear",
    "clamoreadas": "clamorear",
    "clamoreado": "clamorear",
    "clamoreados": "clamorear",
    "clamoreamos": "clamorear",
    "clamorean": "clamorear",
    "clamoreando": "clamorear",
    "clamoreara": "clamorear",
    "clamorearais": "clamorear",
    "clamorearan": "clamorear",
    "clamorearas": "clamorear",
    "clamoreare": "clamorear",
    "clamoreareis": "clamorear",
    "clamorearemos": "clamorear",
    "clamorearen": "clamorear",
    "clamoreares": "clamorear",
    "clamorearon": "clamorear",
    "clamoreará": "clamorear",
    "clamorearán": "clamorear",
    "clamorearás": "clamorear",
    "clamorearé": "clamorear",
    "clamorearéis": "clamorear",
    "clamorearía": "clamorear",
    "clamorearíais": "clamorear",
    "clamorearíamos": "clamorear",
    "clamorearían": "clamorear",
    "clamorearías": "clamorear",
    "clamoreas": "clamorear",
    "clamorease": "clamorear",
    "clamoreaseis": "clamorear",
    "clamoreasen": "clamorear",
    "clamoreases": "clamorear",
    "clamoreaste": "clamorear",
    "clamoreasteis": "clamorear",
    "clamoree": "clamorear",
    "clamoreemos": "clamorear",
    "clamoreen": "clamorear",
    "clamorees": "clamorear",
    "clamoreo": "clamorear",
    "clamoreos": "clamoreo",
    "clamores": "clamor",
    "clamoreábamos": "clamorear",
    "clamoreáis": "clamorear",
    "clamoreáramos": "clamorear",
    "clamoreáremos": "clamorear",
    "clamoreásemos": "clamorear",
    "clamoreé": "clamorear",
    "clamoreéis": "clamorear",
    "clamoreó": "clamorear",
    "clamorosa": "clamoroso",
    "clamorosas": "clamoroso",
    "clamorosos": "clamoroso",
    "clamábamos": "clamar",
    "clamáis": "clamar",
    "clamáramos": "clamar",
    "clamáremos": "clamar",
    "clamásemos": "clamar",
    "clamé": "clamar",
    "claméis": "clamar",
    "clamó": "clamar",
    "clandestina": "clandestino",
    "clandestinas": "clandestino",
    "clandestinidades": "clandestinidad",
    "clandestinos": "clandestino",
    "clanes": "clan",
    "clangores": "clangor",
    "clapas": "clapa",
    "claques": "claque",
    "claquetas": "claqueta",
    "claqués": "claqué",
    "clara": "claro",
    "claraboyas": "claraboya",
    "claras": "claro",
    "clarea": "clarear",
    "clareaba": "clarear",
    "clareabais": "clarear",
    "clareaban": "clarear",
    "clareabas": "clarear",
    "claread": "clarear",
    "clareada": "clarear",
    "clareadas": "clarear",
    "clareado": "clarear",
    "clareados": "clarear",
    "clareamos": "clarear",
    "clarean": "clarear",
    "clareando": "clarear",
    "clareara": "clarear",
    "clarearais": "clarear",
    "clarearan": "clarear",
    "clarearas": "clarear",
    "clareare": "clarear",
    "clareareis": "clarear",
    "clarearemos": "clarear",
    "clarearen": "clarear",
    "clareares": "clarear",
    "clarearon": "clarear",
    "clareará": "clarear",
    "clarearán": "clarear",
    "clarearás": "clarear",
    "clarearé": "clarear",
    "clarearéis": "clarear",
    "clarearía": "clarear",
    "clarearíais": "clarear",
    "clarearíamos": "clarear",
    "clarearían": "clarear",
    "clarearías": "clarear",
    "clareas": "clarear",
    "clarease": "clarear",
    "clareaseis": "clarear",
    "clareasen": "clarear",
    "clareases": "clarear",
    "clareaste": "clarear",
    "clareasteis": "clarear",
    "clarece": "clarecer",
    "clareced": "clarecer",
    "clarecemos": "clarecer",
    "clarecen": "clarecer",
    "clareceremos": "clarecer",
    "clarecerá": "clarecer",
    "clarecerán": "clarecer",
    "clarecerás": "clarecer",
    "clareceré": "clarecer",
    "clareceréis": "clarecer",
    "clarecería": "clarecer",
    "clareceríais": "clarecer",
    "clareceríamos": "clarecer",
    "clarecerían": "clarecer",
    "clarecerías": "clarecer",
    "clareces": "clarecer",
    "clarecida": "clarecer",
    "clarecidas": "clarecer",
    "clarecido": "clarecer",
    "clarecidos": "clarecer",
    "clareciendo": "clarecer",
    "clareciera": "clarecer",
    "clarecierais": "clarecer",
    "clarecieran": "clarecer",
    "clarecieras": "clarecer",
    "clareciere": "clarecer",
    "clareciereis": "clarecer",
    "clarecieren": "clarecer",
    "clarecieres": "clarecer",
    "clarecieron": "clarecer",
    "clareciese": "clarecer",
    "clarecieseis": "clarecer",
    "clareciesen": "clarecer",
    "clarecieses": "clarecer",
    "clarecimos": "clarecer",
    "clareciste": "clarecer",
    "clarecisteis": "clarecer",
    "clareciéramos": "clarecer",
    "clareciéremos": "clarecer",
    "clareciésemos": "clarecer",
    "clareció": "clarecer",
    "clarecéis": "clarecer",
    "clarecí": "clarecer",
    "clarecía": "clarecer",
    "clarecíais": "clarecer",
    "clarecíamos": "clarecer",
    "clarecían": "clarecer",
    "clarecías": "clarecer",
    "claree": "clarear",
    "clareemos": "clarear",
    "clareen": "clarear",
    "clarees": "clarear",
    "clareo": "clarear",
    "clareos": "clareo",
    "clareta": "clarete",
    "claretas": "clarete",
    "claretes": "clarete",
    "claretiana": "claretiano",
    "claretianas": "claretiano",
    "claretianos": "claretiano",
    "clarezas": "clareza",
    "clarezca": "clarecer",
    "clarezcamos": "clarecer",
    "clarezcan": "clarecer",
    "clarezcas": "clarecer",
    "clarezco": "clarecer",
    "clarezcáis": "clarecer",
    "clareábamos": "clarear",
    "clareáis": "clarear",
    "clareáramos": "clarear",
    "clareáremos": "clarear",
    "clareásemos": "clarear",
    "clareé": "clarear",
    "clareéis": "clarear",
    "clareó": "clarear",
    "claridades": "claridad",
    "clarifica": "clarificar",
    "clarificaba": "clarificar",
    "clarificabais": "clarificar",
    "clarificaban": "clarificar",
    "clarificabas": "clarificar",
    "clarificaciones": "clarificación",
    "clarificad": "clarificar",
    "clarificada": "clarificar",
    "clarificadas": "clarificar",
    "clarificado": "clarificar",
    "clarificadora": "clarificador",
    "clarificadoras": "clarificador",
    "clarificadores": "clarificador",
    "clarificados": "clarificar",
    "clarificamos": "clarificar",
    "clarifican": "clarificar",
    "clarificando": "clarificar",
    "clarificara": "clarificar",
    "clarificarais": "clarificar",
    "clarificaran": "clarificar",
    "clarificaras": "clarificar",
    "clarificare": "clarificar",
    "clarificareis": "clarificar",
    "clarificaremos": "clarificar",
    "clarificaren": "clarificar",
    "clarificares": "clarificar",
    "clarificaron": "clarificar",
    "clarificará": "clarificar",
    "clarificarán": "clarificar",
    "clarificarás": "clarificar",
    "clarificaré": "clarificar",
    "clarificaréis": "clarificar",
    "clarificaría": "clarificar",
    "clarificaríais": "clarificar",
    "clarificaríamos": "clarificar",
    "clarificarían": "clarificar",
    "clarificarías": "clarificar",
    "clarificas": "clarificar",
    "clarificase": "clarificar",
    "clarificaseis": "clarificar",
    "clarificasen": "clarificar",
    "clarificases": "clarificar",
    "clarificaste": "clarificar",
    "clarificasteis": "clarificar",
    "clarificativa": "clarificativo",
    "clarificativas": "clarificativo",
    "clarificativos": "clarificativo",
    "clarifico": "clarificar",
    "clarificábamos": "clarificar",
    "clarificáis": "clarificar",
    "clarificáramos": "clarificar",
    "clarificáremos": "clarificar",
    "clarificásemos": "clarificar",
    "clarificó": "clarificar",
    "clarifique": "clarificar",
    "clarifiquemos": "clarificar",
    "clarifiquen": "clarificar",
    "clarifiques": "clarificar",
    "clarifiqué": "clarificar",
    "clarifiquéis": "clarificar",
    "clarimentos": "clarimento",
    "clarinada": "clarinado",
    "clarinadas": "clarinado",
    "clarinados": "clarinado",
    "clarinera": "clarinero",
    "clarineras": "clarinero",
    "clarineros": "clarinero",
    "clarines": "clarín",
    "clarinetes": "clarinete",
    "clarinetistas": "clarinetista",
    "clarioncillos": "clarioncillo",
    "clariones": "clarión",
    "clarisas": "clarisa",
    "clarividencias": "clarividencia",
    "clarividentes": "clarividente",
    "clarores": "claror",
    "claros": "claro",
    "claroscuros": "claroscuro",
    "clarucha": "clarucho",
    "claruchas": "clarucho",
    "claruchos": "clarucho",
    "clases": "clase",
    "clasicismos": "clasicismo",
    "clasicistas": "clasicista",
    "clasicómanos": "clasicómano",
    "clasifica": "clasificar",
    "clasificaba": "clasificar",
    "clasificabais": "clasificar",
    "clasificaban": "clasificar",
    "clasificabas": "clasificar",
    "clasificables": "clasificable",
    "clasificaciones": "clasificación",
    "clasificad": "clasificar",
    "clasificada": "clasificar",
    "clasificadas": "clasificar",
    "clasificado": "clasificar",
    "clasificadora": "clasificador",
    "clasificadoras": "clasificadora",
    "clasificadores": "clasificador",
    "clasificados": "clasificar",
    "clasificamos": "clasificar",
    "clasifican": "clasificar",
    "clasificando": "clasificar",
    "clasificara": "clasificar",
    "clasificarais": "clasificar",
    "clasificaran": "clasificar",
    "clasificaras": "clasificar",
    "clasificare": "clasificar",
    "clasificareis": "clasificar",
    "clasificaremos": "clasificar",
    "clasificaren": "clasificar",
    "clasificares": "clasificar",
    "clasificaron": "clasificar",
    "clasificará": "clasificar",
    "clasificarán": "clasificar",
    "clasificarás": "clasificar",
    "clasificaré": "clasificar",
    "clasificaréis": "clasificar",
    "clasificaría": "clasificar",
    "clasificaríais": "clasificar",
    "clasificaríamos": "clasificar",
    "clasificarían": "clasificar",
    "clasificarías": "clasificar",
    "clasificas": "clasificar",
    "clasificase": "clasificar",
    "clasificaseis": "clasificar",
    "clasificasen": "clasificar",
    "clasificases": "clasificar",
    "clasificaste": "clasificar",
    "clasificasteis": "clasificar",
    "clasificatoria": "clasificatorio",
    "clasificatorias": "clasificatorio",
    "clasificatorios": "clasificatorio",
    "clasifico": "clasificar",
    "clasificábamos": "clasificar",
    "clasificáis": "clasificar",
    "clasificáramos": "clasificar",
    "clasificáremos": "clasificar",
    "clasificásemos": "clasificar",
    "clasificó": "clasificar",
    "clasifique": "clasificar",
    "clasifiquemos": "clasificar",
    "clasifiquen": "clasificar",
    "clasifiques": "clasificar",
    "clasifiqué": "clasificar",
    "clasifiquéis": "clasificar",
    "clasismos": "clasismo",
    "clasistas": "clasista",
    "clatratos": "clatrato",
    "clatrinas": "clatrina",
    "claudica": "claudicar",
    "claudicaba": "claudicar",
    "claudicabais": "claudicar",
    "claudicaban": "claudicar",
    "claudicabas": "claudicar",
    "claudicaciones": "claudicación",
    "claudicad": "claudicar",
    "claudicada": "claudicar",
    "claudicadas": "claudicar",
    "claudicado": "claudicar",
    "claudicados": "claudicar",
    "claudicamos": "claudicar",
    "claudican": "claudicar",
    "claudicando": "claudicar",
    "claudicantes": "claudicante",
    "claudicara": "claudicar",
    "claudicarais": "claudicar",
    "claudicaran": "claudicar",
    "claudicaras": "claudicar",
    "claudicare": "claudicar",
    "claudicareis": "claudicar",
    "claudicaremos": "claudicar",
    "claudicaren": "claudicar",
    "claudicares": "claudicar",
    "claudicaron": "claudicar",
    "claudicará": "claudicar",
    "claudicarán": "claudicar",
    "claudicarás": "claudicar",
    "claudicaré": "claudicar",
    "claudicaréis": "claudicar",
    "claudicaría": "claudicar",
    "claudicaríais": "claudicar",
    "claudicaríamos": "claudicar",
    "claudicarían": "claudicar",
    "claudicarías": "claudicar",
    "claudicas": "claudicar",
    "claudicase": "claudicar",
    "claudicaseis": "claudicar",
    "claudicasen": "claudicar",
    "claudicases": "claudicar",
    "claudicaste": "claudicar",
    "claudicasteis": "claudicar",
    "claudico": "claudicar",
    "claudicábamos": "claudicar",
    "claudicáis": "claudicar",
    "claudicáramos": "claudicar",
    "claudicáremos": "claudicar",
    "claudicásemos": "claudicar",
    "claudicó": "claudicar",
    "claudique": "claudicar",
    "claudiquemos": "claudicar",
    "claudiquen": "claudicar",
    "claudiques": "claudicar",
    "claudiqué": "claudicar",
    "claudiquéis": "claudicar",
    "clausales": "clausal",
    "claustrales": "claustral",
    "claustrofobias": "claustrofobia",
    "claustrofóbica": "claustrofóbico",
    "claustrofóbicas": "claustrofóbico",
    "claustrofóbicos": "claustrofóbico",
    "claustros": "claustro",
    "clausula": "clausular",
    "clausulaba": "clausular",
    "clausulabais": "clausular",
    "clausulaban": "clausular",
    "clausulabas": "clausular",
    "clausulad": "clausular",
    "clausulada": "clausular",
    "clausuladas": "clausular",
    "clausulado": "clausular",
    "clausulados": "clausular",
    "clausulamos": "clausular",
    "clausulan": "clausular",
    "clausulando": "clausular",
    "clausulara": "clausular",
    "clausularais": "clausular",
    "clausularan": "clausular",
    "clausularas": "clausular",
    "clausulare": "clausular",
    "clausulareis": "clausular",
    "clausularemos": "clausular",
    "clausularen": "clausular",
    "clausulares": "clausular",
    "clausularon": "clausular",
    "clausulará": "clausular",
    "clausularán": "clausular",
    "clausularás": "clausular",
    "clausularé": "clausular",
    "clausularéis": "clausular",
    "clausularía": "clausular",
    "clausularíais": "clausular",
    "clausularíamos": "clausular",
    "clausularían": "clausular",
    "clausularías": "clausular",
    "clausulas": "clausular",
    "clausulase": "clausular",
    "clausulaseis": "clausular",
    "clausulasen": "clausular",
    "clausulases": "clausular",
    "clausulaste": "clausular",
    "clausulasteis": "clausular",
    "clausule": "clausular",
    "clausulemos": "clausular",
    "clausulen": "clausular",
    "clausules": "clausular",
    "clausulo": "clausular",
    "clausulábamos": "clausular",
    "clausuláis": "clausular",
    "clausuláramos": "clausular",
    "clausuláremos": "clausular",
    "clausulásemos": "clausular",
    "clausulé": "clausular",
    "clausuléis": "clausular",
    "clausuló": "clausular",
    "clausura": "clausurar",
    "clausuraba": "clausurar",
    "clausurabais": "clausurar",
    "clausuraban": "clausurar",
    "clausurabas": "clausurar",
    "clausurad": "clausurar",
    "clausurada": "clausurar",
    "clausuradas": "clausurar",
    "clausurado": "clausurar",
    "clausurados": "clausurar",
    "clausuramos": "clausurar",
    "clausuran": "clausurar",
    "clausurando": "clausurar",
    "clausurara": "clausurar",
    "clausurarais": "clausurar",
    "clausuraran": "clausurar",
    "clausuraras": "clausurar",
    "clausurare": "clausurar",
    "clausurareis": "clausurar",
    "clausuraremos": "clausurar",
    "clausuraren": "clausurar",
    "clausurares": "clausurar",
    "clausuraron": "clausurar",
    "clausurará": "clausurar",
    "clausurarán": "clausurar",
    "clausurarás": "clausurar",
    "clausuraré": "clausurar",
    "clausuraréis": "clausurar",
    "clausuraría": "clausurar",
    "clausuraríais": "clausurar",
    "clausuraríamos": "clausurar",
    "clausurarían": "clausurar",
    "clausurarías": "clausurar",
    "clausuras": "clausurar",
    "clausurase": "clausurar",
    "clausuraseis": "clausurar",
    "clausurasen": "clausurar",
    "clausurases": "clausurar",
    "clausuraste": "clausurar",
    "clausurasteis": "clausurar",
    "clausure": "clausurar",
    "clausuremos": "clausurar",
    "clausuren": "clausurar",
    "clausures": "clausurar",
    "clausuro": "clausurar",
    "clausurábamos": "clausurar",
    "clausuráis": "clausurar",
    "clausuráramos": "clausurar",
    "clausuráremos": "clausurar",
    "clausurásemos": "clausurar",
    "clausuré": "clausurar",
    "clausuréis": "clausurar",
    "clausuró": "clausurar",
    "clava": "clavar",
    "clavaba": "clavar",
    "clavabais": "clavar",
    "clavaban": "clavar",
    "clavabas": "clavar",
    "clavad": "clavar",
    "clavada": "clavar",
    "clavadas": "clavar",
    "clavadiza": "clavadizo",
    "clavadizas": "clavadizo",
    "clavadizos": "clavadizo",
    "clavado": "clavar",
    "clavadoras": "clavadora",
    "clavados": "clavar",
    "clavaduras": "clavadura",
    "clavamos": "clavar",
    "clavan": "clavar",
    "clavando": "clavar",
    "clavara": "clavar",
    "clavarais": "clavar",
    "clavaran": "clavar",
    "clavaras": "clavar",
    "clavare": "clavar",
    "clavareis": "clavar",
    "clavaremos": "clavar",
    "clavaren": "clavar",
    "clavares": "clavar",
    "clavaron": "clavar",
    "clavará": "clavar",
    "clavarán": "clavar",
    "clavarás": "clavar",
    "clavaré": "clavar",
    "clavaréis": "clavar",
    "clavaría": "clavar",
    "clavaríais": "clavar",
    "clavaríamos": "clavar",
    "clavarían": "clavar",
    "clavarías": "clavar",
    "clavas": "clavar",
    "clavase": "clavar",
    "clavaseis": "clavar",
    "clavasen": "clavar",
    "clavases": "clavar",
    "clavaste": "clavar",
    "clavasteis": "clavar",
    "clavazones": "clavazón",
    "clave": "clavar",
    "clavecines": "clavecín",
    "claveles": "clavel",
    "clavelitos": "clavelito",
    "clavellinas": "clavellina",
    "clavelones": "clavelón",
    "clavemos": "clavar",
    "claven": "clavar",
    "claveques": "claveque",
    "claveras": "clavera",
    "claveros": "clavero",
    "claverías": "clavería",
    "claves": "clave",
    "clavetas": "claveta",
    "clavetea": "clavetear",
    "claveteaba": "clavetear",
    "claveteabais": "clavetear",
    "claveteaban": "clavetear",
    "claveteabas": "clavetear",
    "clavetead": "clavetear",
    "claveteada": "clavetear",
    "claveteadas": "clavetear",
    "claveteado": "clavetear",
    "claveteados": "clavetear",
    "claveteamos": "clavetear",
    "clavetean": "clavetear",
    "claveteando": "clavetear",
    "claveteara": "clavetear",
    "clavetearais": "clavetear",
    "clavetearan": "clavetear",
    "clavetearas": "clavetear",
    "claveteare": "clavetear",
    "claveteareis": "clavetear",
    "clavetearemos": "clavetear",
    "clavetearen": "clavetear",
    "claveteares": "clavetear",
    "clavetearon": "clavetear",
    "claveteará": "clavetear",
    "clavetearán": "clavetear",
    "clavetearás": "clavetear",
    "clavetearé": "clavetear",
    "clavetearéis": "clavetear",
    "clavetearía": "clavetear",
    "clavetearíais": "clavetear",
    "clavetearíamos": "clavetear",
    "clavetearían": "clavetear",
    "clavetearías": "clavetear",
    "claveteas": "clavetear",
    "clavetease": "clavetear",
    "claveteaseis": "clavetear",
    "claveteasen": "clavetear",
    "claveteases": "clavetear",
    "claveteaste": "clavetear",
    "claveteasteis": "clavetear",
    "clavetee": "clavetear",
    "claveteemos": "clavetear",
    "claveteen": "clavetear",
    "clavetees": "clavetear",
    "claveteo": "clavetear",
    "claveteos": "claveteo",
    "clavetes": "clavete",
    "claveteábamos": "clavetear",
    "claveteáis": "clavetear",
    "claveteáramos": "clavetear",
    "claveteáremos": "clavetear",
    "claveteásemos": "clavetear",
    "claveteé": "clavetear",
    "claveteéis": "clavetear",
    "claveteó": "clavetear",
    "clavicembalistas": "clavicembalista",
    "clavicordios": "clavicordio",
    "claviculares": "clavicular",
    "clavicémbalos": "clavicémbalo",
    "claviformes": "claviforme",
    "clavijas": "clavija",
    "clavijeros": "clavijero",
    "clavillos": "clavillo",
    "clavipectorales": "clavipectoral",
    "clavitos": "clavito",
    "claviórganos": "claviórgano",
    "clavo": "clavar",
    "clavos": "clavo",
    "clavábamos": "clavar",
    "claváis": "clavar",
    "claváramos": "clavar",
    "claváremos": "clavar",
    "clavásemos": "clavar",
    "clavé": "clavar",
    "clavéis": "clavar",
    "clavículas": "clavícula",
    "clavó": "clavar",
    "cleidocraneales": "cleidocraneal",
    "clemencias": "clemencia",
    "clementes": "clemente",
    "clementinas": "clementina",
    "clemátides": "clemátide",
    "clepsidras": "clepsidra",
    "cleptomaníaca": "cleptomaníaco",
    "cleptomaníacas": "cleptomaníaco",
    "cleptomaníacos": "cleptomaníaco",
    "cleptomanías": "cleptomanía",
    "cleptómana": "cleptómano",
    "cleptómanas": "cleptómano",
    "cleptómanos": "cleptómano",
    "clerecías": "clerecía",
    "clericales": "clerical",
    "clericalismos": "clericalismo",
    "clericatos": "clericato",
    "clericaturas": "clericatura",
    "clerizontes": "clerizonte",
    "clerofobias": "clerofobia",
    "cleros": "clero",
    "clerófoba": "clerófobo",
    "clerófobas": "clerófobo",
    "clerófobos": "clerófobo",
    "clicas": "clica",
    "cliches": "cliché",
    "clichés": "cliché",
    "clics": "clic",
    "clienta": "cliente",
    "clientas": "cliente",
    "clientelas": "clientela",
    "clientelismos": "clientelismo",
    "clientes": "cliente",
    "climas": "clima",
    "climaterios": "climaterio",
    "climatice": "climatizar",
    "climaticemos": "climatizar",
    "climaticen": "climatizar",
    "climatices": "climatizar",
    "climaticé": "climatizar",
    "climaticéis": "climatizar",
    "climatiza": "climatizar",
    "climatizaba": "climatizar",
    "climatizabais": "climatizar",
    "climatizaban": "climatizar",
    "climatizabas": "climatizar",
    "climatizaciones": "climatización",
    "climatizad": "climatizar",
    "climatizada": "climatizar",
    "climatizadas": "climatizar",
    "climatizado": "climatizar",
    "climatizadora": "climatizador",
    "climatizadoras": "climatizador",
    "climatizadores": "climatizador",
    "climatizados": "climatizar",
    "climatizamos": "climatizar",
    "climatizan": "climatizar",
    "climatizando": "climatizar",
    "climatizara": "climatizar",
    "climatizarais": "climatizar",
    "climatizaran": "climatizar",
    "climatizaras": "climatizar",
    "climatizare": "climatizar",
    "climatizareis": "climatizar",
    "climatizaremos": "climatizar",
    "climatizaren": "climatizar",
    "climatizares": "climatizar",
    "climatizaron": "climatizar",
    "climatizará": "climatizar",
    "climatizarán": "climatizar",
    "climatizarás": "climatizar",
    "climatizaré": "climatizar",
    "climatizaréis": "climatizar",
    "climatizaría": "climatizar",
    "climatizaríais": "climatizar",
    "climatizaríamos": "climatizar",
    "climatizarían": "climatizar",
    "climatizarías": "climatizar",
    "climatizas": "climatizar",
    "climatizase": "climatizar",
    "climatizaseis": "climatizar",
    "climatizasen": "climatizar",
    "climatizases": "climatizar",
    "climatizaste": "climatizar",
    "climatizasteis": "climatizar",
    "climatizo": "climatizar",
    "climatizábamos": "climatizar",
    "climatizáis": "climatizar",
    "climatizáramos": "climatizar",
    "climatizáremos": "climatizar",
    "climatizásemos": "climatizar",
    "climatizó": "climatizar",
    "climatologías": "climatología",
    "climatológica": "climatológico",
    "climatológicas": "climatológico",
    "climatológicos": "climatológico",
    "climatopatologías": "climatopatología",
    "climatérica": "climatérico",
    "climatéricas": "climatérico",
    "climatéricos": "climatérico",
    "climogramas": "climograma",
    "climosecuencias": "climosecuencia",
    "climácica": "climácico",
    "climácicas": "climácico",
    "climácicos": "climácico",
    "climática": "climático",
    "climáticas": "climático",
    "climáticos": "climático",
    "clinas": "clina",
    "clincas": "clinca",
    "clines": "clin",
    "clinkers": "clinker",
    "clinopodios": "clinopodio",
    "clinosecuencias": "clinosecuencia",
    "clinozoisitas": "clinozoisita",
    "clintonitas": "clintonita",
    "clinómetros": "clinómetro",
    "clipes": "clipe",
    "clipperes": "clipper",
    "clips": "clip",
    "clisa": "clisar",
    "clisaba": "clisar",
    "clisabais": "clisar",
    "clisaban": "clisar",
    "clisabas": "clisar",
    "clisad": "clisar",
    "clisada": "clisar",
    "clisadas": "clisar",
    "clisado": "clisar",
    "clisados": "clisar",
    "clisamos": "clisar",
    "clisan": "clisar",
    "clisando": "clisar",
    "clisara": "clisar",
    "clisarais": "clisar",
    "clisaran": "clisar",
    "clisaras": "clisar",
    "clisare": "clisar",
    "clisareis": "clisar",
    "clisaremos": "clisar",
    "clisaren": "clisar",
    "clisares": "clisar",
    "clisaron": "clisar",
    "clisará": "clisar",
    "clisarán": "clisar",
    "clisarás": "clisar",
    "clisaré": "clisar",
    "clisaréis": "clisar",
    "clisaría": "clisar",
    "clisaríais": "clisar",
    "clisaríamos": "clisar",
    "clisarían": "clisar",
    "clisarías": "clisar",
    "clisas": "clisar",
    "clisase": "clisar",
    "clisaseis": "clisar",
    "clisasen": "clisar",
    "clisases": "clisar",
    "clisaste": "clisar",
    "clisasteis": "clisar",
    "clise": "clisar",
    "clisemos": "clisar",
    "clisen": "clisar",
    "clises": "clisar",
    "cliso": "clisar",
    "clisteles": "clistel",
    "clisteres": "clister",
    "clisábamos": "clisar",
    "clisáis": "clisar",
    "clisáramos": "clisar",
    "clisáremos": "clisar",
    "clisásemos": "clisar",
    "clisé": "clisar",
    "cliséis": "clisar",
    "clisés": "clisé",
    "clisímetros": "clisímetro",
    "clisó": "clisar",
    "clitelos": "clitelo",
    "clitoridectomías": "clitoridectomía",
    "clitorismos": "clitorismo",
    "cloacas": "cloaca",
    "cloasmas": "cloasma",
    "clocaba": "clocar",
    "clocabais": "clocar",
    "clocaban": "clocar",
    "clocabas": "clocar",
    "clocad": "clocar",
    "clocada": "clocar",
    "clocadas": "clocar",
    "clocado": "clocar",
    "clocados": "clocar",
    "clocamos": "clocar",
    "clocando": "clocar",
    "clocara": "clocar",
    "clocarais": "clocar",
    "clocaran": "clocar",
    "clocaras": "clocar",
    "clocare": "clocar",
    "clocareis": "clocar",
    "clocaremos": "clocar",
    "clocaren": "clocar",
    "clocares": "clocar",
    "clocaron": "clocar",
    "clocará": "clocar",
    "clocarán": "clocar",
    "clocarás": "clocar",
    "clocaré": "clocar",
    "clocaréis": "clocar",
    "clocaría": "clocar",
    "clocaríais": "clocar",
    "clocaríamos": "clocar",
    "clocarían": "clocar",
    "clocarías": "clocar",
    "clocase": "clocar",
    "clocaseis": "clocar",
    "clocasen": "clocar",
    "clocases": "clocar",
    "clocaste": "clocar",
    "clocasteis": "clocar",
    "clocábamos": "clocar",
    "clocáis": "clocar",
    "clocáramos": "clocar",
    "clocáremos": "clocar",
    "clocásemos": "clocar",
    "clocó": "clocar",
    "clofenos": "clofeno",
    "cloisonnés": "cloisonné",
    "clona": "clonar",
    "clonaba": "clonar",
    "clonabais": "clonar",
    "clonaban": "clonar",
    "clonabas": "clonar",
    "clonaciones": "clonación",
    "clonad": "clonar",
    "clonada": "clonar",
    "clonadas": "clonar",
    "clonado": "clonar",
    "clonados": "clonar",
    "clonales": "clonal",
    "clonamos": "clonar",
    "clonan": "clonar",
    "clonando": "clonar",
    "clonara": "clonar",
    "clonarais": "clonar",
    "clonaran": "clonar",
    "clonaras": "clonar",
    "clonare": "clonar",
    "clonareis": "clonar",
    "clonaremos": "clonar",
    "clonaren": "clonar",
    "clonares": "clonar",
    "clonaron": "clonar",
    "clonará": "clonar",
    "clonarán": "clonar",
    "clonarás": "clonar",
    "clonaré": "clonar",
    "clonaréis": "clonar",
    "clonaría": "clonar",
    "clonaríais": "clonar",
    "clonaríamos": "clonar",
    "clonarían": "clonar",
    "clonarías": "clonar",
    "clonas": "clonar",
    "clonase": "clonar",
    "clonaseis": "clonar",
    "clonasen": "clonar",
    "clonases": "clonar",
    "clonaste": "clonar",
    "clonasteis": "clonar",
    "clone": "clonar",
    "clonemos": "clonar",
    "clonen": "clonar",
    "clones": "clonar",
    "clonicotónica": "clonicotónico",
    "clonicotónicas": "clonicotónico",
    "clonicotónicos": "clonicotónico",
    "clono": "clonar",
    "clonotípica": "clonotípico",
    "clonotípicas": "clonotípico",
    "clonotípicos": "clonotípico",
    "clonábamos": "clonar",
    "clonáis": "clonar",
    "clonáramos": "clonar",
    "clonáremos": "clonar",
    "clonásemos": "clonar",
    "cloné": "clonar",
    "clonéis": "clonar",
    "clonó": "clonar",
    "cloquea": "cloquear",
    "cloqueaba": "cloquear",
    "cloqueabais": "cloquear",
    "cloqueaban": "cloquear",
    "cloqueabas": "cloquear",
    "cloquead": "cloquear",
    "cloqueada": "cloquear",
    "cloqueadas": "cloquear",
    "cloqueado": "cloquear",
    "cloqueados": "cloquear",
    "cloqueamos": "cloquear",
    "cloquean": "cloquear",
    "cloqueando": "cloquear",
    "cloqueara": "cloquear",
    "cloquearais": "cloquear",
    "cloquearan": "cloquear",
    "cloquearas": "cloquear",
    "cloqueare": "cloquear",
    "cloqueareis": "cloquear",
    "cloquearemos": "cloquear",
    "cloquearen": "cloquear",
    "cloqueares": "cloquear",
    "cloquearon": "cloquear",
    "cloqueará": "cloquear",
    "cloquearán": "cloquear",
    "cloquearás": "cloquear",
    "cloquearé": "cloquear",
    "cloquearéis": "cloquear",
    "cloquearía": "cloquear",
    "cloquearíais": "cloquear",
    "cloquearíamos": "cloquear",
    "cloquearían": "cloquear",
    "cloquearías": "cloquear",
    "cloqueas": "cloquear",
    "cloquease": "cloquear",
    "cloqueaseis": "cloquear",
    "cloqueasen": "cloquear",
    "cloqueases": "cloquear",
    "cloqueaste": "cloquear",
    "cloqueasteis": "cloquear",
    "cloquee": "cloquear",
    "cloqueemos": "cloquear",
    "cloqueen": "cloquear",
    "cloquees": "cloquear",
    "cloquemos": "clocar",
    "cloqueo": "cloquear",
    "cloqueos": "cloqueo",
    "cloquera": "cloquero",
    "cloqueras": "cloquero",
    "cloqueros": "cloquero",
    "cloques": "cloque",
    "cloqueábamos": "cloquear",
    "cloqueáis": "cloquear",
    "cloqueáramos": "cloquear",
    "cloqueáremos": "cloquear",
    "cloqueásemos": "cloquear",
    "cloqueé": "cloquear",
    "cloqueéis": "cloquear",
    "cloqueó": "cloquear",
    "cloqué": "clocar",
    "cloquéis": "clocar",
    "cloracetaldehídos": "cloracetaldehído",
    "cloraciones": "cloración",
    "clorada": "clorado",
    "cloradas": "clorado",
    "cloradores": "clorador",
    "clorados": "clorado",
    "clorales": "cloral",
    "clorambuciles": "clorambucil",
    "cloramfenicoles": "cloramfenicol",
    "cloraminas": "cloramina",
    "cloranilos": "cloranilo",
    "cloratada": "cloratado",
    "cloratadas": "cloratado",
    "cloratados": "cloratado",
    "cloratos": "clorato",
    "clorbensidas": "clorbensida",
    "clorbutanoles": "clorbutanol",
    "clorhexidinas": "clorhexidina",
    "clorhidratos": "clorhidrato",
    "clorhidrinas": "clorhidrina",
    "clorhídrica": "clorhídrico",
    "clorhídricas": "clorhídrico",
    "clorhídricos": "clorhídrico",
    "clorinidades": "clorinidad",
    "cloritas": "clorita",
    "cloritos": "clorito",
    "clormerodrinas": "clormerodrina",
    "cloroacetatos": "cloroacetato",
    "cloroacetonas": "cloroacetona",
    "cloroacética": "cloroacético",
    "cloroacéticas": "cloroacético",
    "cloroacéticos": "cloroacético",
    "cloroazodinas": "cloroazodina",
    "clorobencenos": "clorobenceno",
    "clorobutanoles": "clorobutanol",
    "clorocarbonatos": "clorocarbonato",
    "clorocromatos": "clorocromato",
    "clorodibromuros": "clorodibromuro",
    "clorodifluometanos": "clorodifluometano",
    "clorodifluoroacética": "clorodifluoroacético",
    "clorodifluoroacéticas": "clorodifluoroacético",
    "clorodifluoroacéticos": "clorodifluoroacético",
    "clorofenoles": "clorofenol",
    "clorofibras": "clorofibra",
    "clorofila": "clorofilo",
    "clorofilas": "clorofilo",
    "clorofilos": "clorofilo",
    "clorofluorcarbonada": "clorofluorcarbonado",
    "clorofluorcarbonadas": "clorofluorcarbonado",
    "clorofluorcarbonados": "clorofluorcarbonado",
    "clorofluorocarbonos": "clorofluorocarbono",
    "clorofluorocarburos": "clorofluorocarburo",
    "cloroformiatos": "cloroformiato",
    "cloroformice": "cloroformizar",
    "cloroformicemos": "cloroformizar",
    "cloroformicen": "cloroformizar",
    "cloroformices": "cloroformizar",
    "cloroformicé": "cloroformizar",
    "cloroformicéis": "cloroformizar",
    "cloroformiza": "cloroformizar",
    "cloroformizaba": "cloroformizar",
    "cloroformizabais": "cloroformizar",
    "cloroformizaban": "cloroformizar",
    "cloroformizabas": "cloroformizar",
    "cloroformizaciones": "cloroformización",
    "cloroformizad": "cloroformizar",
    "cloroformizada": "cloroformizar",
    "cloroformizadas": "cloroformizar",
    "cloroformizado": "cloroformizar",
    "cloroformizados": "cloroformizar",
    "cloroformizamos": "cloroformizar",
    "cloroformizan": "cloroformizar",
    "cloroformizando": "cloroformizar",
    "cloroformizara": "cloroformizar",
    "cloroformizarais": "cloroformizar",
    "cloroformizaran": "cloroformizar",
    "cloroformizaras": "cloroformizar",
    "cloroformizare": "cloroformizar",
    "cloroformizareis": "cloroformizar",
    "cloroformizaremos": "cloroformizar",
    "cloroformizaren": "cloroformizar",
    "cloroformizares": "cloroformizar",
    "cloroformizaron": "cloroformizar",
    "cloroformizará": "cloroformizar",
    "cloroformizarán": "cloroformizar",
    "cloroformizarás": "cloroformizar",
    "cloroformizaré": "cloroformizar",
    "cloroformizaréis": "cloroformizar",
    "cloroformizaría": "cloroformizar",
    "cloroformizaríais": "cloroformizar",
    "cloroformizaríamos": "cloroformizar",
    "cloroformizarían": "cloroformizar",
    "cloroformizarías": "cloroformizar",
    "cloroformizas": "cloroformizar",
    "cloroformizase": "cloroformizar",
    "cloroformizaseis": "cloroformizar",
    "cloroformizasen": "cloroformizar",
    "cloroformizases": "cloroformizar",
    "cloroformizaste": "cloroformizar",
    "cloroformizasteis": "cloroformizar",
    "cloroformizo": "cloroformizar",
    "cloroformizábamos": "cloroformizar",
    "cloroformizáis": "cloroformizar",
    "cloroformizáramos": "cloroformizar",
    "cloroformizáremos": "cloroformizar",
    "cloroformizásemos": "cloroformizar",
    "cloroformizó": "cloroformizar",
    "cloroformos": "cloroformo",
    "clorofíceas": "clorofícea",
    "clorofílica": "clorofílico",
    "clorofílicas": "clorofílico",
    "clorofílicos": "clorofílico",
    "clorofórmica": "clorofórmico",
    "clorofórmicas": "clorofórmico",
    "clorofórmicos": "clorofórmico",
    "clorohidroquinonas": "clorohidroquinona",
    "cloromaleica": "cloromaleico",
    "cloromaleicas": "cloromaleico",
    "cloromaleicos": "cloromaleico",
    "cloromelanitas": "cloromelanita",
    "clorometilfosfónica": "clorometilfosfónico",
    "clorometilfosfónicas": "clorometilfosfónico",
    "clorometilfosfónicos": "clorometilfosfónico",
    "cloromicetinas": "cloromicetina",
    "cloronitroacetofenonas": "cloronitroacetofenona",
    "cloropicrinas": "cloropicrina",
    "cloroplastidios": "cloroplastidio",
    "cloroplastinas": "cloroplastina",
    "cloroplastos": "cloroplasto",
    "cloroplatinatos": "cloroplatinato",
    "cloroprenos": "cloropreno",
    "cloropromacinas": "cloropromacina",
    "cloropsias": "cloropsia",
    "cloroquinas": "cloroquina",
    "cloros": "cloro",
    "clorosa": "cloroso",
    "clorosas": "cloroso",
    "clorosos": "cloroso",
    "clorosulfonatos": "clorosulfonato",
    "clorosulfónica": "clorosulfónico",
    "clorosulfónicas": "clorosulfónico",
    "clorosulfónicos": "clorosulfónico",
    "clorotetraciclinas": "clorotetraciclina",
    "clorotimoles": "clorotimol",
    "clorotiofenoles": "clorotiofenol",
    "clorotoluenos": "clorotolueno",
    "clorotrifluoetilenos": "clorotrifluoetileno",
    "clorotrifluometanos": "clorotrifluometano",
    "cloroyoduros": "cloroyoduro",
    "clorquinadoles": "clorquinadol",
    "clorura": "clorurar",
    "cloruraba": "clorurar",
    "clorurabais": "clorurar",
    "cloruraban": "clorurar",
    "clorurabas": "clorurar",
    "cloruraciones": "cloruración",
    "clorurad": "clorurar",
    "clorurada": "clorurar",
    "cloruradas": "clorurar",
    "clorurado": "clorurar",
    "clorurados": "clorurar",
    "cloruramos": "clorurar",
    "cloruran": "clorurar",
    "clorurando": "clorurar",
    "clorurara": "clorurar",
    "clorurarais": "clorurar",
    "cloruraran": "clorurar",
    "cloruraras": "clorurar",
    "clorurare": "clorurar",
    "clorurareis": "clorurar",
    "cloruraremos": "clorurar",
    "cloruraren": "clorurar",
    "clorurares": "clorurar",
    "cloruraron": "clorurar",
    "clorurará": "clorurar",
    "clorurarán": "clorurar",
    "clorurarás": "clorurar",
    "cloruraré": "clorurar",
    "cloruraréis": "clorurar",
    "cloruraría": "clorurar",
    "cloruraríais": "clorurar",
    "cloruraríamos": "clorurar",
    "clorurarían": "clorurar",
    "clorurarías": "clorurar",
    "cloruras": "clorurar",
    "clorurase": "clorurar",
    "cloruraseis": "clorurar",
    "clorurasen": "clorurar",
    "clorurases": "clorurar",
    "cloruraste": "clorurar",
    "clorurasteis": "clorurar",
    "clorure": "clorurar",
    "cloruremos": "clorurar",
    "cloruren": "clorurar",
    "clorures": "clorurar",
    "cloruro": "clorurar",
    "cloruros": "cloruro",
    "clorurábamos": "clorurar",
    "cloruráis": "clorurar",
    "cloruráramos": "clorurar",
    "cloruráremos": "clorurar",
    "clorurásemos": "clorurar",
    "cloruré": "clorurar",
    "cloruréis": "clorurar",
    "cloruró": "clorurar",
    "clorénquimas": "clorénquima",
    "clorófitos": "clorófito",
    "clorómetros": "clorómetro",
    "clorótica": "clorótico",
    "cloróticas": "clorótico",
    "cloróticos": "clorótico",
    "closets": "closet",
    "clowns": "clown",
    "clube": "club",
    "clubes": "club",
    "clubistas": "clubista",
    "clubs": "club",
    "clueca": "clueco",
    "cluecan": "clocar",
    "cluecas": "clueco",
    "clueco": "clocar",
    "cluecos": "clueco",
    "clueque": "clocar",
    "cluequen": "clocar",
    "clueques": "clocar",
    "cluniacenses": "cluniacense",
    "clupeidos": "clupeido",
    "clusters": "cluster",
    "clámides": "clámide",
    "clásica": "clásico",
    "clásicas": "clásico",
    "clásicos": "clásico",
    "clástica": "clástico",
    "clásticas": "clástico",
    "clásticos": "clástico",
    "cláusulas": "cláusula",
    "cláxones": "claxon",
    "clérigos": "clérigo",
    "clímaxes": "clímax",
    "clínica": "clínico",
    "clínicas": "clínico",
    "clínicos": "clínico",
    "clíperes": "clíper",
    "clípers": "clíper",
    "clónica": "clónico",
    "clónicas": "clónico",
    "clónicos": "clónico",
    "clórica": "clórico",
    "clóricas": "clórico",
    "clóricos": "clórico",
    "cm": "centímetro",
    "cnidarios": "cnidario",
    "cnoidales": "cnoidal",
    "coacciona": "coaccionar",
    "coaccionaba": "coaccionar",
    "coaccionabais": "coaccionar",
    "coaccionaban": "coaccionar",
    "coaccionabas": "coaccionar",
    "coaccionad": "coaccionar",
    "coaccionada": "coaccionar",
    "coaccionadas": "coaccionar",
    "coaccionado": "coaccionar",
    "coaccionados": "coaccionar",
    "coaccionamos": "coaccionar",
    "coaccionan": "coaccionar",
    "coaccionando": "coaccionar",
    "coaccionara": "coaccionar",
    "coaccionarais": "coaccionar",
    "coaccionaran": "coaccionar",
    "coaccionaras": "coaccionar",
    "coaccionare": "coaccionar",
    "coaccionareis": "coaccionar",
    "coaccionaremos": "coaccionar",
    "coaccionaren": "coaccionar",
    "coaccionares": "coaccionar",
    "coaccionaron": "coaccionar",
    "coaccionará": "coaccionar",
    "coaccionarán": "coaccionar",
    "coaccionarás": "coaccionar",
    "coaccionaré": "coaccionar",
    "coaccionaréis": "coaccionar",
    "coaccionaría": "coaccionar",
    "coaccionaríais": "coaccionar",
    "coaccionaríamos": "coaccionar",
    "coaccionarían": "coaccionar",
    "coaccionarías": "coaccionar",
    "coaccionas": "coaccionar",
    "coaccionase": "coaccionar",
    "coaccionaseis": "coaccionar",
    "coaccionasen": "coaccionar",
    "coaccionases": "coaccionar",
    "coaccionaste": "coaccionar",
    "coaccionasteis": "coaccionar",
    "coaccione": "coaccionar",
    "coaccionemos": "coaccionar",
    "coaccionen": "coaccionar",
    "coacciones": "coaccionar",
    "coacciono": "coaccionar",
    "coaccionábamos": "coaccionar",
    "coaccionáis": "coaccionar",
    "coaccionáramos": "coaccionar",
    "coaccionáremos": "coaccionar",
    "coaccionásemos": "coaccionar",
    "coaccioné": "coaccionar",
    "coaccionéis": "coaccionar",
    "coaccionó": "coaccionar",
    "coacerva": "coacervar",
    "coacervaba": "coacervar",
    "coacervabais": "coacervar",
    "coacervaban": "coacervar",
    "coacervabas": "coacervar",
    "coacervaciones": "coacervación",
    "coacervad": "coacervar",
    "coacervada": "coacervar",
    "coacervadas": "coacervar",
    "coacervado": "coacervar",
    "coacervados": "coacervar",
    "coacervamos": "coacervar",
    "coacervan": "coacervar",
    "coacervando": "coacervar",
    "coacervara": "coacervar",
    "coacervarais": "coacervar",
    "coacervaran": "coacervar",
    "coacervaras": "coacervar",
    "coacervare": "coacervar",
    "coacervareis": "coacervar",
    "coacervaremos": "coacervar",
    "coacervaren": "coacervar",
    "coacervares": "coacervar",
    "coacervaron": "coacervar",
    "coacervará": "coacervar",
    "coacervarán": "coacervar",
    "coacervarás": "coacervar",
    "coacervaré": "coacervar",
    "coacervaréis": "coacervar",
    "coacervaría": "coacervar",
    "coacervaríais": "coacervar",
    "coacervaríamos": "coacervar",
    "coacervarían": "coacervar",
    "coacervarías": "coacervar",
    "coacervas": "coacervar",
    "coacervase": "coacervar",
    "coacervaseis": "coacervar",
    "coacervasen": "coacervar",
    "coacervases": "coacervar",
    "coacervaste": "coacervar",
    "coacervasteis": "coacervar",
    "coacerve": "coacervar",
    "coacervemos": "coacervar",
    "coacerven": "coacervar",
    "coacerves": "coacervar",
    "coacervo": "coacervar",
    "coacervábamos": "coacervar",
    "coacerváis": "coacervar",
    "coacerváramos": "coacervar",
    "coacerváremos": "coacervar",
    "coacervásemos": "coacervar",
    "coacervé": "coacervar",
    "coacervéis": "coacervar",
    "coacervó": "coacervar",
    "coaches": "coach",
    "coacreedora": "coacreedor",
    "coacreedoras": "coacreedor",
    "coacreedores": "coacreedor",
    "coactiva": "coactivo",
    "coactivas": "coactivo",
    "coactivos": "coactivo",
    "coacusada": "coacusado",
    "coacusadas": "coacusado",
    "coacusados": "coacusado",
    "coadjutora": "coadjutor",
    "coadjutoras": "coadjutor",
    "coadjutores": "coadjutor",
    "coadjutorías": "coadjutoría",
    "coadministradora": "coadministrador",
    "coadministradoras": "coadministrador",
    "coadministradores": "coadministrador",
    "coadquisiciones": "coadquisición",
    "coaduna": "coadunar",
    "coadunaba": "coadunar",
    "coadunabais": "coadunar",
    "coadunaban": "coadunar",
    "coadunabas": "coadunar",
    "coadunaciones": "coadunación",
    "coadunad": "coadunar",
    "coadunada": "coadunar",
    "coadunadas": "coadunar",
    "coadunado": "coadunar",
    "coadunados": "coadunar",
    "coadunamientos": "coadunamiento",
    "coadunamos": "coadunar",
    "coadunan": "coadunar",
    "coadunando": "coadunar",
    "coadunara": "coadunar",
    "coadunarais": "coadunar",
    "coadunaran": "coadunar",
    "coadunaras": "coadunar",
    "coadunare": "coadunar",
    "coadunareis": "coadunar",
    "coadunaremos": "coadunar",
    "coadunaren": "coadunar",
    "coadunares": "coadunar",
    "coadunaron": "coadunar",
    "coadunará": "coadunar",
    "coadunarán": "coadunar",
    "coadunarás": "coadunar",
    "coadunaré": "coadunar",
    "coadunaréis": "coadunar",
    "coadunaría": "coadunar",
    "coadunaríais": "coadunar",
    "coadunaríamos": "coadunar",
    "coadunarían": "coadunar",
    "coadunarías": "coadunar",
    "coadunas": "coadunar",
    "coadunase": "coadunar",
    "coadunaseis": "coadunar",
    "coadunasen": "coadunar",
    "coadunases": "coadunar",
    "coadunaste": "coadunar",
    "coadunasteis": "coadunar",
    "coadune": "coadunar",
    "coadunemos": "coadunar",
    "coadunen": "coadunar",
    "coadunes": "coadunar",
    "coaduno": "coadunar",
    "coadunábamos": "coadunar",
    "coadunáis": "coadunar",
    "coadunáramos": "coadunar",
    "coadunáremos": "coadunar",
    "coadunásemos": "coadunar",
    "coaduné": "coadunar",
    "coadunéis": "coadunar",
    "coadunó": "coadunar",
    "coadyuva": "coadyuvar",
    "coadyuvaba": "coadyuvar",
    "coadyuvabais": "coadyuvar",
    "coadyuvaban": "coadyuvar",
    "coadyuvabas": "coadyuvar",
    "coadyuvad": "coadyuvar",
    "coadyuvada": "coadyuvar",
    "coadyuvadas": "coadyuvar",
    "coadyuvado": "coadyuvar",
    "coadyuvados": "coadyuvar",
    "coadyuvamos": "coadyuvar",
    "coadyuvan": "coadyuvar",
    "coadyuvando": "coadyuvar",
    "coadyuvantes": "coadyuvante",
    "coadyuvara": "coadyuvar",
    "coadyuvarais": "coadyuvar",
    "coadyuvaran": "coadyuvar",
    "coadyuvaras": "coadyuvar",
    "coadyuvare": "coadyuvar",
    "coadyuvareis": "coadyuvar",
    "coadyuvaremos": "coadyuvar",
    "coadyuvaren": "coadyuvar",
    "coadyuvares": "coadyuvar",
    "coadyuvaron": "coadyuvar",
    "coadyuvará": "coadyuvar",
    "coadyuvarán": "coadyuvar",
    "coadyuvarás": "coadyuvar",
    "coadyuvaré": "coadyuvar",
    "coadyuvaréis": "coadyuvar",
    "coadyuvaría": "coadyuvar",
    "coadyuvaríais": "coadyuvar",
    "coadyuvaríamos": "coadyuvar",
    "coadyuvarían": "coadyuvar",
    "coadyuvarías": "coadyuvar",
    "coadyuvas": "coadyuvar",
    "coadyuvase": "coadyuvar",
    "coadyuvaseis": "coadyuvar",
    "coadyuvasen": "coadyuvar",
    "coadyuvases": "coadyuvar",
    "coadyuvaste": "coadyuvar",
    "coadyuvasteis": "coadyuvar",
    "coadyuve": "coadyuvar",
    "coadyuvemos": "coadyuvar",
    "coadyuven": "coadyuvar",
    "coadyuves": "coadyuvar",
    "coadyuvo": "coadyuvar",
    "coadyuvábamos": "coadyuvar",
    "coadyuváis": "coadyuvar",
    "coadyuváramos": "coadyuvar",
    "coadyuváremos": "coadyuvar",
    "coadyuvásemos": "coadyuvar",
    "coadyuvé": "coadyuvar",
    "coadyuvéis": "coadyuvar",
    "coadyuvó": "coadyuvar",
    "coafianzamientos": "coafianzamiento",
    "coagentes": "coagente",
    "coagula": "coagular",
    "coagulaba": "coagular",
    "coagulabais": "coagular",
    "coagulaban": "coagular",
    "coagulabas": "coagular",
    "coagulabilidades": "coagulabilidad",
    "coagulables": "coagulable",
    "coagulaciones": "coagulación",
    "coagulad": "coagular",
    "coagulada": "coagular",
    "coaguladas": "coagular",
    "coagulado": "coagular",
    "coaguladora": "coagulador",
    "coaguladoras": "coagulador",
    "coaguladores": "coagulador",
    "coagulados": "coagular",
    "coagulamos": "coagular",
    "coagulan": "coagular",
    "coagulando": "coagular",
    "coagulantes": "coagulante",
    "coagulara": "coagular",
    "coagularais": "coagular",
    "coagularan": "coagular",
    "coagularas": "coagular",
    "coagulare": "coagular",
    "coagulareis": "coagular",
    "coagularemos": "coagular",
    "coagularen": "coagular",
    "coagulares": "coagular",
    "coagularon": "coagular",
    "coagulará": "coagular",
    "coagularán": "coagular",
    "coagularás": "coagular",
    "coagularé": "coagular",
    "coagularéis": "coagular",
    "coagularía": "coagular",
    "coagularíais": "coagular",
    "coagularíamos": "coagular",
    "coagularían": "coagular",
    "coagularías": "coagular",
    "coagulas": "coagular",
    "coagulasas": "coagulasa",
    "coagulase": "coagular",
    "coagulaseis": "coagular",
    "coagulasen": "coagular",
    "coagulases": "coagular",
    "coagulaste": "coagular",
    "coagulasteis": "coagular",
    "coagule": "coagular",
    "coagulemos": "coagular",
    "coagulen": "coagular",
    "coagules": "coagular",
    "coagulo": "coagular",
    "coagulocitos": "coagulocito",
    "coagulopatías": "coagulopatía",
    "coagulosa": "coaguloso",
    "coagulosas": "coaguloso",
    "coagulosos": "coaguloso",
    "coagulábamos": "coagular",
    "coaguláis": "coagular",
    "coaguláramos": "coagular",
    "coaguláremos": "coagular",
    "coagulásemos": "coagular",
    "coagulé": "coagular",
    "coaguléis": "coagular",
    "coaguló": "coagular",
    "coalas": "coala",
    "coalcares": "coalcar",
    "coalescencias": "coalescencia",
    "coalescentes": "coalescente",
    "coaliciones": "coalición",
    "coalicionistas": "coalicionista",
    "coanas": "coana",
    "coaptaciones": "coaptación",
    "coarchos": "coarcho",
    "coarrendadora": "coarrendador",
    "coarrendadoras": "coarrendador",
    "coarrendadores": "coarrendador",
    "coarrendatarios": "coarrendatario",
    "coarta": "coartar",
    "coartaba": "coartar",
    "coartabais": "coartar",
    "coartaban": "coartar",
    "coartabas": "coartar",
    "coartaciones": "coartación",
    "coartad": "coartar",
    "coartada": "coartar",
    "coartadas": "coartar",
    "coartado": "coartar",
    "coartadora": "coartador",
    "coartadoras": "coartador",
    "coartadores": "coartador",
    "coartados": "coartar",
    "coartamos": "coartar",
    "coartan": "coartar",
    "coartando": "coartar",
    "coartara": "coartar",
    "coartarais": "coartar",
    "coartaran": "coartar",
    "coartaras": "coartar",
    "coartare": "coartar",
    "coartareis": "coartar",
    "coartaremos": "coartar",
    "coartaren": "coartar",
    "coartares": "coartar",
    "coartaron": "coartar",
    "coartará": "coartar",
    "coartarán": "coartar",
    "coartarás": "coartar",
    "coartaré": "coartar",
    "coartaréis": "coartar",
    "coartaría": "coartar",
    "coartaríais": "coartar",
    "coartaríamos": "coartar",
    "coartarían": "coartar",
    "coartarías": "coartar",
    "coartas": "coartar",
    "coartase": "coartar",
    "coartaseis": "coartar",
    "coartasen": "coartar",
    "coartases": "coartar",
    "coartaste": "coartar",
    "coartasteis": "coartar",
    "coarte": "coartar",
    "coartemos": "coartar",
    "coarten": "coartar",
    "coartes": "coartar",
    "coarticulaciones": "coarticulación",
    "coarticularia": "coarticulario",
    "coarticularias": "coarticulario",
    "coarticularios": "coarticulario",
    "coarto": "coartar",
    "coartábamos": "coartar",
    "coartáis": "coartar",
    "coartáramos": "coartar",
    "coartáremos": "coartar",
    "coartásemos": "coartar",
    "coarté": "coartar",
    "coartéis": "coartar",
    "coartó": "coartar",
    "coas": "coa",
    "coaseguradora": "coasegurador",
    "coaseguradoras": "coasegurador",
    "coaseguradores": "coasegurador",
    "coaseguros": "coaseguro",
    "coatises": "coatí",
    "coatás": "coatá",
    "coatíes": "coatí",
    "coatís": "coatí",
    "coautora": "coautor",
    "coautoras": "coautor",
    "coautores": "coautor",
    "coaxiales": "coaxial",
    "cobalta": "cobaltar",
    "cobaltaba": "cobaltar",
    "cobaltabais": "cobaltar",
    "cobaltaban": "cobaltar",
    "cobaltabas": "cobaltar",
    "cobaltad": "cobaltar",
    "cobaltada": "cobaltar",
    "cobaltadas": "cobaltar",
    "cobaltado": "cobaltar",
    "cobaltados": "cobaltar",
    "cobaltamos": "cobaltar",
    "cobaltan": "cobaltar",
    "cobaltando": "cobaltar",
    "cobaltara": "cobaltar",
    "cobaltarais": "cobaltar",
    "cobaltaran": "cobaltar",
    "cobaltaras": "cobaltar",
    "cobaltare": "cobaltar",
    "cobaltareis": "cobaltar",
    "cobaltaremos": "cobaltar",
    "cobaltaren": "cobaltar",
    "cobaltares": "cobaltar",
    "cobaltaron": "cobaltar",
    "cobaltará": "cobaltar",
    "cobaltarán": "cobaltar",
    "cobaltarás": "cobaltar",
    "cobaltaré": "cobaltar",
    "cobaltaréis": "cobaltar",
    "cobaltaría": "cobaltar",
    "cobaltaríais": "cobaltar",
    "cobaltaríamos": "cobaltar",
    "cobaltarían": "cobaltar",
    "cobaltarías": "cobaltar",
    "cobaltas": "cobaltar",
    "cobaltase": "cobaltar",
    "cobaltaseis": "cobaltar",
    "cobaltasen": "cobaltar",
    "cobaltases": "cobaltar",
    "cobaltaste": "cobaltar",
    "cobaltasteis": "cobaltar",
    "cobalte": "cobaltar",
    "cobaltemos": "cobaltar",
    "cobalten": "cobaltar",
    "cobaltes": "cobaltar",
    "cobaltinas": "cobaltina",
    "cobaltitas": "cobaltita",
    "cobalto": "cobaltar",
    "cobaltos": "cobalto",
    "cobaltosa": "cobaltoso",
    "cobaltosas": "cobaltoso",
    "cobaltosos": "cobaltoso",
    "cobaltábamos": "cobaltar",
    "cobaltáis": "cobaltar",
    "cobaltáramos": "cobaltar",
    "cobaltáremos": "cobaltar",
    "cobaltásemos": "cobaltar",
    "cobalté": "cobaltar",
    "cobaltéis": "cobaltar",
    "cobaltó": "cobaltar",
    "cobarchos": "cobarcho",
    "cobardes": "cobarde",
    "cobardías": "cobardía",
    "cobas": "coba",
    "cobayas": "cobaya",
    "cobertera": "cobertero",
    "coberteras": "cobertero",
    "coberteros": "cobertero",
    "cobertizos": "cobertizo",
    "cobertores": "cobertor",
    "coberturas": "cobertura",
    "cobija": "cobijar",
    "cobijaba": "cobijar",
    "cobijabais": "cobijar",
    "cobijaban": "cobijar",
    "cobijabas": "cobijar",
    "cobijad": "cobijar",
    "cobijada": "cobijar",
    "cobijadas": "cobijar",
    "cobijado": "cobijar",
    "cobijadora": "cobijador",
    "cobijadoras": "cobijador",
    "cobijadores": "cobijador",
    "cobijados": "cobijar",
    "cobijamientos": "cobijamiento",
    "cobijamos": "cobijar",
    "cobijan": "cobijar",
    "cobijando": "cobijar",
    "cobijara": "cobijar",
    "cobijarais": "cobijar",
    "cobijaran": "cobijar",
    "cobijaras": "cobijar",
    "cobijare": "cobijar",
    "cobijareis": "cobijar",
    "cobijaremos": "cobijar",
    "cobijaren": "cobijar",
    "cobijares": "cobijar",
    "cobijaron": "cobijar",
    "cobijará": "cobijar",
    "cobijarán": "cobijar",
    "cobijarás": "cobijar",
    "cobijaré": "cobijar",
    "cobijaréis": "cobijar",
    "cobijaría": "cobijar",
    "cobijaríais": "cobijar",
    "cobijaríamos": "cobijar",
    "cobijarían": "cobijar",
    "cobijarías": "cobijar",
    "cobijas": "cobijar",
    "cobijase": "cobijar",
    "cobijaseis": "cobijar",
    "cobijasen": "cobijar",
    "cobijases": "cobijar",
    "cobijaste": "cobijar",
    "cobijasteis": "cobijar",
    "cobije": "cobijar",
    "cobijemos": "cobijar",
    "cobijen": "cobijar",
    "cobijes": "cobijar",
    "cobijo": "cobijar",
    "cobijos": "cobijo",
    "cobijábamos": "cobijar",
    "cobijáis": "cobijar",
    "cobijáramos": "cobijar",
    "cobijáremos": "cobijar",
    "cobijásemos": "cobijar",
    "cobijé": "cobijar",
    "cobijéis": "cobijar",
    "cobijó": "cobijar",
    "cobistas": "cobista",
    "coblas": "cobla",
    "coboles": "cobol",
    "cobra": "cobrar",
    "cobraba": "cobrar",
    "cobrabais": "cobrar",
    "cobraban": "cobrar",
    "cobrabas": "cobrar",
    "cobrables": "cobrable",
    "cobrad": "cobrar",
    "cobrada": "cobrar",
    "cobradas": "cobrar",
    "cobradera": "cobradero",
    "cobraderas": "cobradero",
    "cobraderos": "cobradero",
    "cobrado": "cobrar",
    "cobradora": "cobrador",
    "cobradoras": "cobrador",
    "cobradores": "cobrador",
    "cobrados": "cobrar",
    "cobramos": "cobrar",
    "cobran": "cobrar",
    "cobrando": "cobrar",
    "cobranzas": "cobranza",
    "cobrara": "cobrar",
    "cobrarais": "cobrar",
    "cobraran": "cobrar",
    "cobraras": "cobrar",
    "cobrare": "cobrar",
    "cobrareis": "cobrar",
    "cobraremos": "cobrar",
    "cobraren": "cobrar",
    "cobrares": "cobrar",
    "cobraron": "cobrar",
    "cobrará": "cobrar",
    "cobrarán": "cobrar",
    "cobrarás": "cobrar",
    "cobraré": "cobrar",
    "cobraréis": "cobrar",
    "cobraría": "cobrar",
    "cobraríais": "cobrar",
    "cobraríamos": "cobrar",
    "cobrarían": "cobrar",
    "cobrarías": "cobrar",
    "cobras": "cobrar",
    "cobrase": "cobrar",
    "cobraseis": "cobrar",
    "cobrasen": "cobrar",
    "cobrases": "cobrar",
    "cobraste": "cobrar",
    "cobrasteis": "cobrar",
    "cobratoria": "cobratorio",
    "cobratorias": "cobratorio",
    "cobratorios": "cobratorio",
    "cobre": "cobrar",
    "cobrea": "cobrear",
    "cobreaba": "cobrear",
    "cobreabais": "cobrear",
    "cobreaban": "cobrear",
    "cobreabas": "cobrear",
    "cobread": "cobrear",
    "cobreada": "cobrear",
    "cobreadas": "cobrear",
    "cobreado": "cobrear",
    "cobreados": "cobrear",
    "cobreamos": "cobrear",
    "cobrean": "cobrear",
    "cobreando": "cobrear",
    "cobreara": "cobrear",
    "cobrearais": "cobrear",
    "cobrearan": "cobrear",
    "cobrearas": "cobrear",
    "cobreare": "cobrear",
    "cobreareis": "cobrear",
    "cobrearemos": "cobrear",
    "cobrearen": "cobrear",
    "cobreares": "cobrear",
    "cobrearon": "cobrear",
    "cobreará": "cobrear",
    "cobrearán": "cobrear",
    "cobrearás": "cobrear",
    "cobrearé": "cobrear",
    "cobrearéis": "cobrear",
    "cobrearía": "cobrear",
    "cobrearíais": "cobrear",
    "cobrearíamos": "cobrear",
    "cobrearían": "cobrear",
    "cobrearías": "cobrear",
    "cobreas": "cobrear",
    "cobrease": "cobrear",
    "cobreaseis": "cobrear",
    "cobreasen": "cobrear",
    "cobreases": "cobrear",
    "cobreaste": "cobrear",
    "cobreasteis": "cobrear",
    "cobree": "cobrear",
    "cobreemos": "cobrear",
    "cobreen": "cobrear",
    "cobrees": "cobrear",
    "cobremos": "cobrar",
    "cobren": "cobrar",
    "cobreo": "cobrear",
    "cobres": "cobre",
    "cobreábamos": "cobrear",
    "cobreáis": "cobrear",
    "cobreáramos": "cobrear",
    "cobreáremos": "cobrear",
    "cobreásemos": "cobrear",
    "cobreé": "cobrear",
    "cobreéis": "cobrear",
    "cobreña": "cobreño",
    "cobreñas": "cobreño",
    "cobreños": "cobreño",
    "cobreó": "cobrear",
    "cobriza": "cobrizo",
    "cobrizas": "cobrizo",
    "cobrizos": "cobrizo",
    "cobro": "cobrar",
    "cobros": "cobro",
    "cobrábamos": "cobrar",
    "cobráis": "cobrar",
    "cobráramos": "cobrar",
    "cobráremos": "cobrar",
    "cobrásemos": "cobrar",
    "cobré": "cobrar",
    "cobréis": "cobrar",
    "cobró": "cobrar",
    "cobáltica": "cobáltico",
    "cobálticas": "cobáltico",
    "cobálticos": "cobáltico",
    "coca": "cocar",
    "cocaba": "cocar",
    "cocabais": "cocar",
    "cocaban": "cocar",
    "cocabas": "cocar",
    "cocad": "cocar",
    "cocada": "cocar",
    "cocadas": "cocar",
    "cocado": "cocar",
    "cocadora": "cocador",
    "cocadoras": "cocador",
    "cocadores": "cocador",
    "cocados": "cocar",
    "cocainismos": "cocainismo",
    "cocainomanías": "cocainomanía",
    "cocainómana": "cocainómano",
    "cocainómanas": "cocainómano",
    "cocainómanos": "cocainómano",
    "cocales": "cocal",
    "cocamas": "cocama",
    "cocamos": "cocar",
    "cocan": "cocar",
    "cocando": "cocar",
    "cocara": "cocarar",
    "cocaraba": "cocarar",
    "cocarabais": "cocarar",
    "cocaraban": "cocarar",
    "cocarabas": "cocarar",
    "cocarad": "cocarar",
    "cocarada": "cocarar",
    "cocaradas": "cocarar",
    "cocarado": "cocarar",
    "cocarados": "cocarar",
    "cocarais": "cocar",
    "cocaramos": "cocarar",
    "cocaran": "cocarar",
    "cocarando": "cocarar",
    "cocarara": "cocarar",
    "cocararais": "cocarar",
    "cocararan": "cocarar",
    "cocararas": "cocarar",
    "cocarare": "cocarar",
    "cocarareis": "cocarar",
    "cocararemos": "cocarar",
    "cocararen": "cocarar",
    "cocarares": "cocarar",
    "cocararon": "cocarar",
    "cocarará": "cocarar",
    "cocararán": "cocarar",
    "cocararás": "cocarar",
    "cocararé": "cocarar",
    "cocararéis": "cocarar",
    "cocararía": "cocarar",
    "cocararíais": "cocarar",
    "cocararíamos": "cocarar",
    "cocararían": "cocarar",
    "cocararías": "cocarar",
    "cocaras": "cocarar",
    "cocarase": "cocarar",
    "cocaraseis": "cocarar",
    "cocarasen": "cocarar",
    "cocarases": "cocarar",
    "cocaraste": "cocarar",
    "cocarasteis": "cocarar",
    "cocare": "cocarar",
    "cocareis": "cocar",
    "cocaremos": "cocarar",
    "cocaren": "cocarar",
    "cocares": "cocarar",
    "cocaro": "cocarar",
    "cocaron": "cocar",
    "cocará": "cocar",
    "cocarábamos": "cocarar",
    "cocaráis": "cocarar",
    "cocarán": "cocar",
    "cocaráramos": "cocarar",
    "cocaráremos": "cocarar",
    "cocarás": "cocar",
    "cocarásemos": "cocarar",
    "cocaré": "cocarar",
    "cocaréis": "cocarar",
    "cocaría": "cocar",
    "cocaríais": "cocar",
    "cocaríamos": "cocar",
    "cocarían": "cocar",
    "cocarías": "cocar",
    "cocaró": "cocarar",
    "cocas": "cocar",
    "cocase": "cocar",
    "cocaseis": "cocar",
    "cocasen": "cocar",
    "cocases": "cocar",
    "cocaste": "cocar",
    "cocasteis": "cocar",
    "cocaínas": "cocaína",
    "coccidioidinas": "coccidioidina",
    "coccidios": "coccidio",
    "coccinélidos": "coccinélido",
    "cocciones": "cocción",
    "coccígea": "coccígeo",
    "coccígeas": "coccígeo",
    "coccígeos": "coccígeo",
    "coccínea": "coccíneo",
    "coccíneas": "coccíneo",
    "coccíneos": "coccíneo",
    "cocea": "cocear",
    "coceaba": "cocear",
    "coceabais": "cocear",
    "coceaban": "cocear",
    "coceabas": "cocear",
    "cocead": "cocear",
    "coceada": "cocear",
    "coceadas": "cocear",
    "coceado": "cocear",
    "coceadora": "coceador",
    "coceadoras": "coceador",
    "coceadores": "coceador",
    "coceados": "cocear",
    "coceaduras": "coceadura",
    "coceamientos": "coceamiento",
    "coceamos": "cocear",
    "cocean": "cocear",
    "coceando": "cocear",
    "coceara": "cocear",
    "cocearais": "cocear",
    "cocearan": "cocear",
    "cocearas": "cocear",
    "coceare": "cocear",
    "coceareis": "cocear",
    "cocearemos": "cocear",
    "cocearen": "cocear",
    "coceares": "cocear",
    "cocearon": "cocear",
    "coceará": "cocear",
    "cocearán": "cocear",
    "cocearás": "cocear",
    "cocearé": "cocear",
    "cocearéis": "cocear",
    "cocearía": "cocear",
    "cocearíais": "cocear",
    "cocearíamos": "cocear",
    "cocearían": "cocear",
    "cocearías": "cocear",
    "coceas": "cocear",
    "cocease": "cocear",
    "coceaseis": "cocear",
    "coceasen": "cocear",
    "coceases": "cocear",
    "coceaste": "cocear",
    "coceasteis": "cocear",
    "coced": "cocer",
    "cocedera": "cocedero",
    "cocederas": "cocedero",
    "cocederos": "cocedero",
    "cocediza": "cocedizo",
    "cocedizas": "cocedizo",
    "cocedizos": "cocedizo",
    "coceduras": "cocedura",
    "cocee": "cocear",
    "coceemos": "cocear",
    "coceen": "cocear",
    "cocees": "cocear",
    "cocemos": "cocer",
    "coceo": "cocear",
    "coceos": "coceo",
    "coceremos": "cocer",
    "cocerá": "cocer",
    "cocerán": "cocer",
    "cocerás": "cocer",
    "coceré": "cocer",
    "coceréis": "cocer",
    "cocería": "cocer",
    "coceríais": "cocer",
    "coceríamos": "cocer",
    "cocerían": "cocer",
    "cocerías": "cocer",
    "coces": "coz",
    "coceábamos": "cocear",
    "coceáis": "cocear",
    "coceáramos": "cocear",
    "coceáremos": "cocear",
    "coceásemos": "cocear",
    "coceé": "cocear",
    "coceéis": "cocear",
    "coceó": "cocear",
    "cochambrera": "cochambrero",
    "cochambreras": "cochambrero",
    "cochambreros": "cochambrero",
    "cochambrerías": "cochambrería",
    "cochambres": "cochambre",
    "cochambrosa": "cochambroso",
    "cochambrosas": "cochambroso",
    "cochambrosos": "cochambroso",
    "cocharros": "cocharro",
    "cochas": "cocha",
    "cochastros": "cochastro",
    "cochazos": "cochazo",
    "cochea": "cochear",
    "cocheaba": "cochear",
    "cocheabais": "cochear",
    "cocheaban": "cochear",
    "cocheabas": "cochear",
    "cochead": "cochear",
    "cocheada": "cochear",
    "cocheadas": "cochear",
    "cocheado": "cochear",
    "cocheados": "cochear",
    "cocheamos": "cochear",
    "cochean": "cochear",
    "cocheando": "cochear",
    "cocheara": "cochear",
    "cochearais": "cochear",
    "cochearan": "cochear",
    "cochearas": "cochear",
    "cocheare": "cochear",
    "cocheareis": "cochear",
    "cochearemos": "cochear",
    "cochearen": "cochear",
    "cocheares": "cochear",
    "cochearon": "cochear",
    "cocheará": "cochear",
    "cochearán": "cochear",
    "cochearás": "cochear",
    "cochearé": "cochear",
    "cochearéis": "cochear",
    "cochearía": "cochear",
    "cochearíais": "cochear",
    "cochearíamos": "cochear",
    "cochearían": "cochear",
    "cochearías": "cochear",
    "cocheas": "cochear",
    "cochease": "cochear",
    "cocheaseis": "cochear",
    "cocheasen": "cochear",
    "cocheases": "cochear",
    "cocheaste": "cochear",
    "cocheasteis": "cochear",
    "cochecitos": "cochecito",
    "cochee": "cochear",
    "cocheemos": "cochear",
    "cocheen": "cochear",
    "cochees": "cochear",
    "cocheo": "cochear",
    "cocheras": "cochera",
    "cocheros": "cochero",
    "coches": "coche",
    "cocheábamos": "cochear",
    "cocheáis": "cochear",
    "cocheáramos": "cochear",
    "cocheáremos": "cochear",
    "cocheásemos": "cochear",
    "cocheé": "cochear",
    "cocheéis": "cochear",
    "cocheó": "cochear",
    "cochifritos": "cochifrito",
    "cochina": "cochino",
    "cochinadas": "cochinada",
    "cochinas": "cochino",
    "cochinatas": "cochinata",
    "cochinerías": "cochinería",
    "cochinillas": "cochinilla",
    "cochinillos": "cochinillo",
    "cochinos": "cochino",
    "cochiqueras": "cochiquera",
    "cochitriles": "cochitril",
    "cochizos": "cochizo",
    "cochuras": "cochura",
    "cochurera": "cochurero",
    "cochureras": "cochurero",
    "cochureros": "cochurero",
    "cocida": "cocer",
    "cocidas": "cocer",
    "cocido": "cocer",
    "cocidos": "cocido",
    "cociendo": "cocer",
    "cocientes": "cociente",
    "cociera": "cocer",
    "cocierais": "cocer",
    "cocieran": "cocer",
    "cocieras": "cocer",
    "cociere": "cocer",
    "cociereis": "cocer",
    "cocieren": "cocer",
    "cocieres": "cocer",
    "cocieron": "cocer",
    "cociese": "cocer",
    "cocieseis": "cocer",
    "cociesen": "cocer",
    "cocieses": "cocer",
    "cocimientos": "cocimiento",
    "cocimos": "cocer",
    "cocina": "cocinar",
    "cocinaba": "cocinar",
    "cocinabais": "cocinar",
    "cocinaban": "cocinar",
    "cocinabas": "cocinar",
    "cocinad": "cocinar",
    "cocinada": "cocinar",
    "cocinadas": "cocinar",
    "cocinado": "cocinar",
    "cocinados": "cocinar",
    "cocinamos": "cocinar",
    "cocinan": "cocinar",
    "cocinando": "cocinar",
    "cocinara": "cocinar",
    "cocinarais": "cocinar",
    "cocinaran": "cocinar",
    "cocinaras": "cocinar",
    "cocinare": "cocinar",
    "cocinareis": "cocinar",
    "cocinaremos": "cocinar",
    "cocinaren": "cocinar",
    "cocinares": "cocinar",
    "cocinaron": "cocinar",
    "cocinará": "cocinar",
    "cocinarán": "cocinar",
    "cocinarás": "cocinar",
    "cocinaré": "cocinar",
    "cocinaréis": "cocinar",
    "cocinaría": "cocinar",
    "cocinaríais": "cocinar",
    "cocinaríamos": "cocinar",
    "cocinarían": "cocinar",
    "cocinarías": "cocinar",
    "cocinas": "cocinar",
    "cocinase": "cocinar",
    "cocinaseis": "cocinar",
    "cocinasen": "cocinar",
    "cocinases": "cocinar",
    "cocinaste": "cocinar",
    "cocinasteis": "cocinar",
    "cocine": "cocinar",
    "cocinemos": "cocinar",
    "cocinen": "cocinar",
    "cocinera": "cocinero",
    "cocineras": "cocinero",
    "cocineros": "cocinero",
    "cocines": "cocinar",
    "cocinillas": "cocinilla",
    "cocino": "cocinar",
    "cocinábamos": "cocinar",
    "cocináis": "cocinar",
    "cocináramos": "cocinar",
    "cocináremos": "cocinar",
    "cocinásemos": "cocinar",
    "cociné": "cocinar",
    "cocinéis": "cocinar",
    "cocinó": "cocinar",
    "cociste": "cocer",
    "cocisteis": "cocer",
    "cociéramos": "cocer",
    "cociéremos": "cocer",
    "cociésemos": "cocer",
    "coció": "cocer",
    "cockers": "cocker",
    "cockites": "cockit",
    "cockpits": "cockpit",
    "cocktailes": "cocktail",
    "cocktails": "cocktail",
    "cocleares": "coclear",
    "coclearias": "coclearia",
    "cocleariformes": "cocleariforme",
    "cocleoestapedia": "cocleoestapedio",
    "cocleoestapedias": "cocleoestapedio",
    "cocleoestapedios": "cocleoestapedio",
    "coco": "cocar",
    "cocochas": "cococha",
    "cocodrilos": "cocodrilo",
    "cocoinatos": "cocoinato",
    "cocoliches": "cocoliche",
    "cocolitoforáceas": "cocolitoforácea",
    "cocolitos": "cocolito",
    "cocos": "coco",
    "cocosa": "cocoso",
    "cocosas": "cocoso",
    "cocosos": "cocoso",
    "cocotales": "cocotal",
    "cocotas": "cocota",
    "cocoteras": "cocotera",
    "cocoteros": "cocotero",
    "cocottes": "cocotte",
    "cocteleras": "coctelera",
    "cocumas": "cocuma",
    "cocuyos": "cocuyo",
    "cocábamos": "cocar",
    "cocáis": "cocar",
    "cocáramos": "cocar",
    "cocáremos": "cocar",
    "cocásemos": "cocar",
    "cocéis": "cocer",
    "cocí": "cocer",
    "cocía": "cocer",
    "cocíais": "cocer",
    "cocíamos": "cocer",
    "cocían": "cocer",
    "cocías": "cocer",
    "cocó": "cocar",
    "codaduras": "codadura",
    "codales": "codal",
    "codas": "coda",
    "codastes": "codaste",
    "codazos": "codazo",
    "codea": "codear",
    "codeaba": "codear",
    "codeabais": "codear",
    "codeaban": "codear",
    "codeabas": "codear",
    "codead": "codear",
    "codeada": "codear",
    "codeadas": "codear",
    "codeado": "codear",
    "codeados": "codear",
    "codeamos": "codear",
    "codean": "codear",
    "codeando": "codear",
    "codeara": "codear",
    "codearais": "codear",
    "codearan": "codear",
    "codearas": "codear",
    "codeare": "codear",
    "codeareis": "codear",
    "codearemos": "codear",
    "codearen": "codear",
    "codeares": "codear",
    "codearon": "codear",
    "codeará": "codear",
    "codearán": "codear",
    "codearás": "codear",
    "codearé": "codear",
    "codearéis": "codear",
    "codearía": "codear",
    "codearíais": "codear",
    "codearíamos": "codear",
    "codearían": "codear",
    "codearías": "codear",
    "codeas": "codear",
    "codease": "codear",
    "codeaseis": "codear",
    "codeasen": "codear",
    "codeases": "codear",
    "codeaste": "codear",
    "codeasteis": "codear",
    "codecs": "codec",
    "codee": "codear",
    "codeemos": "codear",
    "codeen": "codear",
    "codees": "codear",
    "codelincuencias": "codelincuencia",
    "codeo": "codear",
    "codeos": "codeo",
    "codeposiciones": "codeposición",
    "codera": "codero",
    "coderas": "codero",
    "coderos": "codero",
    "codesos": "codeso",
    "codeudora": "codeudor",
    "codeudoras": "codeudor",
    "codeudores": "codeudor",
    "codeábamos": "codear",
    "codeáis": "codear",
    "codeáramos": "codear",
    "codeáremos": "codear",
    "codeásemos": "codear",
    "codeé": "codear",
    "codeéis": "codear",
    "codeínas": "codeína",
    "codeó": "codear",
    "codicia": "codiciar",
    "codiciaba": "codiciar",
    "codiciabais": "codiciar",
    "codiciaban": "codiciar",
    "codiciabas": "codiciar",
    "codiciables": "codiciable",
    "codiciad": "codiciar",
    "codiciada": "codiciar",
    "codiciadas": "codiciar",
    "codiciado": "codiciar",
    "codiciados": "codiciar",
    "codiciamos": "codiciar",
    "codician": "codiciar",
    "codiciando": "codiciar",
    "codiciara": "codiciar",
    "codiciarais": "codiciar",
    "codiciaran": "codiciar",
    "codiciaras": "codiciar",
    "codiciare": "codiciar",
    "codiciareis": "codiciar",
    "codiciaremos": "codiciar",
    "codiciaren": "codiciar",
    "codiciares": "codiciar",
    "codiciaron": "codiciar",
    "codiciará": "codiciar",
    "codiciarán": "codiciar",
    "codiciarás": "codiciar",
    "codiciaré": "codiciar",
    "codiciaréis": "codiciar",
    "codiciaría": "codiciar",
    "codiciaríais": "codiciar",
    "codiciaríamos": "codiciar",
    "codiciarían": "codiciar",
    "codiciarías": "codiciar",
    "codicias": "codiciar",
    "codiciase": "codiciar",
    "codiciaseis": "codiciar",
    "codiciasen": "codiciar",
    "codiciases": "codiciar",
    "codiciaste": "codiciar",
    "codiciasteis": "codiciar",
    "codicie": "codiciar",
    "codiciemos": "codiciar",
    "codicien": "codiciar",
    "codicies": "codiciar",
    "codicilares": "codicilar",
    "codicilios": "codicilio",
    "codicilos": "codicilo",
    "codicio": "codiciar",
    "codiciosa": "codicioso",
    "codiciosas": "codicioso",
    "codiciosos": "codicioso",
    "codiciábamos": "codiciar",
    "codiciáis": "codiciar",
    "codiciáramos": "codiciar",
    "codiciáremos": "codiciar",
    "codiciásemos": "codiciar",
    "codicié": "codiciar",
    "codiciéis": "codiciar",
    "codició": "codiciar",
    "codifica": "codificar",
    "codificaba": "codificar",
    "codificabais": "codificar",
    "codificaban": "codificar",
    "codificabas": "codificar",
    "codificaciones": "codificación",
    "codificad": "codificar",
    "codificada": "codificar",
    "codificadas": "codificar",
    "codificado": "codificar",
    "codificadora": "codificador",
    "codificadoras": "codificador",
    "codificadores": "codificador",
    "codificados": "codificar",
    "codificamos": "codificar",
    "codifican": "codificar",
    "codificando": "codificar",
    "codificara": "codificar",
    "codificarais": "codificar",
    "codificaran": "codificar",
    "codificaras": "codificar",
    "codificare": "codificar",
    "codificareis": "codificar",
    "codificaremos": "codificar",
    "codificaren": "codificar",
    "codificares": "codificar",
    "codificaron": "codificar",
    "codificará": "codificar",
    "codificarán": "codificar",
    "codificarás": "codificar",
    "codificaré": "codificar",
    "codificaréis": "codificar",
    "codificaría": "codificar",
    "codificaríais": "codificar",
    "codificaríamos": "codificar",
    "codificarían": "codificar",
    "codificarías": "codificar",
    "codificas": "codificar",
    "codificase": "codificar",
    "codificaseis": "codificar",
    "codificasen": "codificar",
    "codificases": "codificar",
    "codificaste": "codificar",
    "codificasteis": "codificar",
    "codifico": "codificar",
    "codificábamos": "codificar",
    "codificáis": "codificar",
    "codificáramos": "codificar",
    "codificáremos": "codificar",
    "codificásemos": "codificar",
    "codificó": "codificar",
    "codifique": "codificar",
    "codifiquemos": "codificar",
    "codifiquen": "codificar",
    "codifiques": "codificar",
    "codifiqué": "codificar",
    "codifiquéis": "codificar",
    "codilleras": "codillera",
    "codillos": "codillo",
    "codimensiones": "codimensión",
    "codirectora": "codirector",
    "codirectoras": "codirector",
    "codirectores": "codirector",
    "codirige": "codirigir",
    "codirigen": "codirigir",
    "codiriges": "codirigir",
    "codirigid": "codirigir",
    "codirigida": "codirigir",
    "codirigidas": "codirigir",
    "codirigido": "codirigir",
    "codirigidos": "codirigir",
    "codirigiendo": "codirigir",
    "codirigiera": "codirigir",
    "codirigierais": "codirigir",
    "codirigieran": "codirigir",
    "codirigieras": "codirigir",
    "codirigiere": "codirigir",
    "codirigiereis": "codirigir",
    "codirigieren": "codirigir",
    "codirigieres": "codirigir",
    "codirigieron": "codirigir",
    "codirigiese": "codirigir",
    "codirigieseis": "codirigir",
    "codirigiesen": "codirigir",
    "codirigieses": "codirigir",
    "codirigimos": "codirigir",
    "codirigiremos": "codirigir",
    "codirigirá": "codirigir",
    "codirigirán": "codirigir",
    "codirigirás": "codirigir",
    "codirigiré": "codirigir",
    "codirigiréis": "codirigir",
    "codirigiría": "codirigir",
    "codirigiríais": "codirigir",
    "codirigiríamos": "codirigir",
    "codirigirían": "codirigir",
    "codirigirías": "codirigir",
    "codirigiste": "codirigir",
    "codirigisteis": "codirigir",
    "codirigiéramos": "codirigir",
    "codirigiéremos": "codirigir",
    "codirigiésemos": "codirigir",
    "codirigió": "codirigir",
    "codirigí": "codirigir",
    "codirigía": "codirigir",
    "codirigíais": "codirigir",
    "codirigíamos": "codirigir",
    "codirigían": "codirigir",
    "codirigías": "codirigir",
    "codirigís": "codirigir",
    "codirija": "codirigir",
    "codirijamos": "codirigir",
    "codirijan": "codirigir",
    "codirijas": "codirigir",
    "codirijo": "codirigir",
    "codirijáis": "codirigir",
    "codones": "codón",
    "codornices": "codorniz",
    "codos": "codo",
    "codoñates": "codoñate",
    "codónes": "codón",
    "coediciones": "coedición",
    "coeducacionales": "coeducacional",
    "coeducaciones": "coeducación",
    "coeficiencias": "coeficiencia",
    "coeficientes": "coeficiente",
    "coenzimas": "coenzima",
    "coerce": "coercer",
    "coerced": "coercer",
    "coercemos": "coercer",
    "coercen": "coercer",
    "coerceremos": "coercer",
    "coercerá": "coercer",
    "coercerán": "coercer",
    "coercerás": "coercer",
    "coerceré": "coercer",
    "coerceréis": "coercer",
    "coercería": "coercer",
    "coerceríais": "coercer",
    "coerceríamos": "coercer",
    "coercerían": "coercer",
    "coercerías": "coercer",
    "coerces": "coercer",
    "coercibilidades": "coercibilidad",
    "coercibles": "coercible",
    "coercida": "coercer",
    "coercidas": "coercer",
    "coercido": "coercer",
    "coercidos": "coercer",
    "coerciendo": "coercer",
    "coerciera": "coercer",
    "coercierais": "coercer",
    "coercieran": "coercer",
    "coercieras": "coercer",
    "coerciere": "coercer",
    "coerciereis": "coercer",
    "coercieren": "coercer",
    "coercieres": "coercer",
    "coercieron": "coercer",
    "coerciese": "coercer",
    "coercieseis": "coercer",
    "coerciesen": "coercer",
    "coercieses": "coercer",
    "coercimientos": "coercimiento",
    "coercimos": "coercer",
    "coerciones": "coerción",
    "coerciste": "coercer",
    "coercisteis": "coercer",
    "coercitiva": "coercitivo",
    "coercitivas": "coercitivo",
    "coercitividades": "coercitividad",
    "coercitivos": "coercitivo",
    "coerciéramos": "coercer",
    "coerciéremos": "coercer",
    "coerciésemos": "coercer",
    "coerció": "coercer",
    "coercéis": "coercer",
    "coercí": "coercer",
    "coercía": "coercer",
    "coercíais": "coercer",
    "coercíamos": "coercer",
    "coercían": "coercer",
    "coercías": "coercer",
    "coerza": "coercer",
    "coerzamos": "coercer",
    "coerzan": "coercer",
    "coerzas": "coercer",
    "coerzo": "coercer",
    "coerzáis": "coercer",
    "coesposas": "coesposa",
    "coestimulaciones": "coestimulación",
    "coeterna": "coeterno",
    "coeternas": "coeterno",
    "coeternidades": "coeternidad",
    "coeternos": "coeterno",
    "coetánea": "coetáneo",
    "coetáneas": "coetáneo",
    "coetáneos": "coetáneo",
    "coeva": "coevo",
    "coevas": "coevo",
    "coevos": "coevo",
    "coexista": "coexistir",
    "coexistamos": "coexistir",
    "coexistan": "coexistir",
    "coexistas": "coexistir",
    "coexiste": "coexistir",
    "coexisten": "coexistir",
    "coexistencias": "coexistencia",
    "coexistes": "coexistir",
    "coexistid": "coexistir",
    "coexistida": "coexistir",
    "coexistidas": "coexistir",
    "coexistido": "coexistir",
    "coexistidos": "coexistir",
    "coexistiendo": "coexistir",
    "coexistiera": "coexistir",
    "coexistierais": "coexistir",
    "coexistieran": "coexistir",
    "coexistieras": "coexistir",
    "coexistiere": "coexistir",
    "coexistiereis": "coexistir",
    "coexistieren": "coexistir",
    "coexistieres": "coexistir",
    "coexistieron": "coexistir",
    "coexistiese": "coexistir",
    "coexistieseis": "coexistir",
    "coexistiesen": "coexistir",
    "coexistieses": "coexistir",
    "coexistimos": "coexistir",
    "coexistiremos": "coexistir",
    "coexistirá": "coexistir",
    "coexistirán": "coexistir",
    "coexistirás": "coexistir",
    "coexistiré": "coexistir",
    "coexistiréis": "coexistir",
    "coexistiría": "coexistir",
    "coexistiríais": "coexistir",
    "coexistiríamos": "coexistir",
    "coexistirían": "coexistir",
    "coexistirías": "coexistir",
    "coexististe": "coexistir",
    "coexististeis": "coexistir",
    "coexistiéramos": "coexistir",
    "coexistiéremos": "coexistir",
    "coexistiésemos": "coexistir",
    "coexistió": "coexistir",
    "coexisto": "coexistir",
    "coexistáis": "coexistir",
    "coexistí": "coexistir",
    "coexistía": "coexistir",
    "coexistíais": "coexistir",
    "coexistíamos": "coexistir",
    "coexistían": "coexistir",
    "coexistías": "coexistir",
    "coexistís": "coexistir",
    "coextensiones": "coextensión",
    "coextrusiones": "coextrusión",
    "cofactores": "cofactor",
    "cofas": "cofa",
    "cofiadora": "cofiador",
    "cofiadoras": "cofiador",
    "cofiadores": "cofiador",
    "cofias": "cofia",
    "cofiduciarios": "cofiduciario",
    "cofinanciaciones": "cofinanciación",
    "cofinanciamientos": "cofinanciamiento",
    "cofinanzamientos": "cofinanzamiento",
    "cofines": "cofín",
    "cofinitas": "cofinita",
    "cofirmantes": "cofirmante",
    "cofrades": "cofrade",
    "cofradías": "cofradía",
    "cofrecillos": "cofrecillo",
    "cofres": "cofre",
    "cofronteras": "cofrontera",
    "cofunciones": "cofunción",
    "cofásica": "cofásico",
    "cofásicas": "cofásico",
    "cofásicos": "cofásico",
    "cogarantes": "cogarante",
    "coge": "coger",
    "coged": "coger",
    "cogedera": "cogedero",
    "cogederas": "cogedero",
    "cogederos": "cogedero",
    "cogediza": "cogedizo",
    "cogedizas": "cogedizo",
    "cogedizos": "cogedizo",
    "cogedora": "cogedor",
    "cogedoras": "cogedor",
    "cogedores": "cogedor",
    "cogemos": "coger",
    "cogen": "coger",
    "cogeneraciones": "cogeneración",
    "cogeremos": "coger",
    "cogerá": "coger",
    "cogerán": "coger",
    "cogerás": "coger",
    "cogeré": "coger",
    "cogeréis": "coger",
    "cogería": "coger",
    "cogeríais": "coger",
    "cogeríamos": "coger",
    "cogerían": "coger",
    "cogerías": "coger",
    "coges": "coger",
    "cogestiones": "cogestión",
    "cogida": "coger",
    "cogidas": "cogida",
    "cogido": "coger",
    "cogidos": "cogido",
    "cogiendo": "coger",
    "cogiera": "coger",
    "cogierais": "coger",
    "cogieran": "coger",
    "cogieras": "coger",
    "cogiere": "coger",
    "cogiereis": "coger",
    "cogieren": "coger",
    "cogieres": "coger",
    "cogieron": "coger",
    "cogiese": "coger",
    "cogieseis": "coger",
    "cogiesen": "coger",
    "cogieses": "coger",
    "cogimos": "coger",
    "cogiradora": "cogirador",
    "cogiradoras": "cogirador",
    "cogiradores": "cogirador",
    "cogiste": "coger",
    "cogisteis": "coger",
    "cogita": "cogitar",
    "cogitaba": "cogitar",
    "cogitabais": "cogitar",
    "cogitaban": "cogitar",
    "cogitabas": "cogitar",
    "cogitabunda": "cogitabundo",
    "cogitabundas": "cogitabundo",
    "cogitabundos": "cogitabundo",
    "cogitad": "cogitar",
    "cogitada": "cogitar",
    "cogitadas": "cogitar",
    "cogitado": "cogitar",
    "cogitados": "cogitar",
    "cogitamos": "cogitar",
    "cogitan": "cogitar",
    "cogitando": "cogitar",
    "cogitara": "cogitar",
    "cogitarais": "cogitar",
    "cogitaran": "cogitar",
    "cogitaras": "cogitar",
    "cogitare": "cogitar",
    "cogitareis": "cogitar",
    "cogitaremos": "cogitar",
    "cogitaren": "cogitar",
    "cogitares": "cogitar",
    "cogitaron": "cogitar",
    "cogitará": "cogitar",
    "cogitarán": "cogitar",
    "cogitarás": "cogitar",
    "cogitaré": "cogitar",
    "cogitaréis": "cogitar",
    "cogitaría": "cogitar",
    "cogitaríais": "cogitar",
    "cogitaríamos": "cogitar",
    "cogitarían": "cogitar",
    "cogitarías": "cogitar",
    "cogitas": "cogitar",
    "cogitase": "cogitar",
    "cogitaseis": "cogitar",
    "cogitasen": "cogitar",
    "cogitases": "cogitar",
    "cogitaste": "cogitar",
    "cogitasteis": "cogitar",
    "cogitativa": "cogitativo",
    "cogitativas": "cogitativo",
    "cogitativos": "cogitativo",
    "cogite": "cogitar",
    "cogitemos": "cogitar",
    "cogiten": "cogitar",
    "cogites": "cogitar",
    "cogito": "cogitar",
    "cogitos": "cogito",
    "cogitábamos": "cogitar",
    "cogitáis": "cogitar",
    "cogitáramos": "cogitar",
    "cogitáremos": "cogitar",
    "cogitásemos": "cogitar",
    "cogité": "cogitar",
    "cogitéis": "cogitar",
    "cogitó": "cogitar",
    "cogiéramos": "coger",
    "cogiéremos": "coger",
    "cogiésemos": "coger",
    "cogió": "coger",
    "cognaciones": "cognación",
    "cognada": "cognado",
    "cognadas": "cognado",
    "cognados": "cognado",
    "cognata": "cognato",
    "cognatas": "cognato",
    "cognaticia": "cognaticio",
    "cognaticias": "cognaticio",
    "cognaticios": "cognaticio",
    "cognatos": "cognato",
    "cogniciones": "cognición",
    "cognitiva": "cognitivo",
    "cognitivas": "cognitivo",
    "cognitivos": "cognitivo",
    "cognomentos": "cognomento",
    "cognoscibilidades": "cognoscibilidad",
    "cognoscibles": "cognoscible",
    "cognoscitiva": "cognoscitivo",
    "cognoscitivas": "cognoscitivo",
    "cognoscitivos": "cognoscitivo",
    "cogoberna": "cogobernar",
    "cogobernaba": "cogobernar",
    "cogobernabais": "cogobernar",
    "cogobernaban": "cogobernar",
    "cogobernabas": "cogobernar",
    "cogobernad": "cogobernar",
    "cogobernada": "cogobernar",
    "cogobernadas": "cogobernar",
    "cogobernado": "cogobernar",
    "cogobernados": "cogobernar",
    "cogobernamos": "cogobernar",
    "cogobernan": "cogobernar",
    "cogobernando": "cogobernar",
    "cogobernara": "cogobernar",
    "cogobernarais": "cogobernar",
    "cogobernaran": "cogobernar",
    "cogobernaras": "cogobernar",
    "cogobernare": "cogobernar",
    "cogobernareis": "cogobernar",
    "cogobernaremos": "cogobernar",
    "cogobernaren": "cogobernar",
    "cogobernares": "cogobernar",
    "cogobernaron": "cogobernar",
    "cogobernará": "cogobernar",
    "cogobernarán": "cogobernar",
    "cogobernarás": "cogobernar",
    "cogobernaré": "cogobernar",
    "cogobernaréis": "cogobernar",
    "cogobernaría": "cogobernar",
    "cogobernaríais": "cogobernar",
    "cogobernaríamos": "cogobernar",
    "cogobernarían": "cogobernar",
    "cogobernarías": "cogobernar",
    "cogobernas": "cogobernar",
    "cogobernase": "cogobernar",
    "cogobernaseis": "cogobernar",
    "cogobernasen": "cogobernar",
    "cogobernases": "cogobernar",
    "cogobernaste": "cogobernar",
    "cogobernasteis": "cogobernar",
    "cogoberne": "cogobernar",
    "cogobernemos": "cogobernar",
    "cogobernen": "cogobernar",
    "cogobernes": "cogobernar",
    "cogoberno": "cogobernar",
    "cogobernábamos": "cogobernar",
    "cogobernáis": "cogobernar",
    "cogobernáramos": "cogobernar",
    "cogobernáremos": "cogobernar",
    "cogobernásemos": "cogobernar",
    "cogoberné": "cogobernar",
    "cogobernéis": "cogobernar",
    "cogobernó": "cogobernar",
    "cogollos": "cogollo",
    "cogombros": "cogombro",
    "cogorzas": "cogorza",
    "cogotazos": "cogotazo",
    "cogoteras": "cogotera",
    "cogotes": "cogote",
    "cogotillos": "cogotillo",
    "cogotuda": "cogotudo",
    "cogotudas": "cogotudo",
    "cogotudos": "cogotudo",
    "coguchos": "cogucho",
    "cogujadas": "cogujada",
    "cogujones": "cogujón",
    "cogulladas": "cogullada",
    "cogullas": "cogulla",
    "cogéis": "coger",
    "cogí": "coger",
    "cogía": "coger",
    "cogíais": "coger",
    "cogíamos": "coger",
    "cogían": "coger",
    "cogías": "coger",
    "cohabita": "cohabitar",
    "cohabitaba": "cohabitar",
    "cohabitabais": "cohabitar",
    "cohabitaban": "cohabitar",
    "cohabitabas": "cohabitar",
    "cohabitaciones": "cohabitación",
    "cohabitad": "cohabitar",
    "cohabitada": "cohabitar",
    "cohabitadas": "cohabitar",
    "cohabitado": "cohabitar",
    "cohabitados": "cohabitar",
    "cohabitamos": "cohabitar",
    "cohabitan": "cohabitar",
    "cohabitando": "cohabitar",
    "cohabitara": "cohabitar",
    "cohabitarais": "cohabitar",
    "cohabitaran": "cohabitar",
    "cohabitaras": "cohabitar",
    "cohabitare": "cohabitar",
    "cohabitareis": "cohabitar",
    "cohabitaremos": "cohabitar",
    "cohabitaren": "cohabitar",
    "cohabitares": "cohabitar",
    "cohabitaron": "cohabitar",
    "cohabitará": "cohabitar",
    "cohabitarán": "cohabitar",
    "cohabitarás": "cohabitar",
    "cohabitaré": "cohabitar",
    "cohabitaréis": "cohabitar",
    "cohabitaría": "cohabitar",
    "cohabitaríais": "cohabitar",
    "cohabitaríamos": "cohabitar",
    "cohabitarían": "cohabitar",
    "cohabitarías": "cohabitar",
    "cohabitas": "cohabitar",
    "cohabitase": "cohabitar",
    "cohabitaseis": "cohabitar",
    "cohabitasen": "cohabitar",
    "cohabitases": "cohabitar",
    "cohabitaste": "cohabitar",
    "cohabitasteis": "cohabitar",
    "cohabite": "cohabitar",
    "cohabitemos": "cohabitar",
    "cohabiten": "cohabitar",
    "cohabites": "cohabitar",
    "cohabito": "cohabitar",
    "cohabitábamos": "cohabitar",
    "cohabitáis": "cohabitar",
    "cohabitáramos": "cohabitar",
    "cohabitáremos": "cohabitar",
    "cohabitásemos": "cohabitar",
    "cohabité": "cohabitar",
    "cohabitéis": "cohabitar",
    "cohabitó": "cohabitar",
    "cohecha": "cohechar",
    "cohechaba": "cohechar",
    "cohechabais": "cohechar",
    "cohechaban": "cohechar",
    "cohechabas": "cohechar",
    "cohechad": "cohechar",
    "cohechada": "cohechar",
    "cohechadas": "cohechar",
    "cohechado": "cohechar",
    "cohechados": "cohechar",
    "cohechamos": "cohechar",
    "cohechan": "cohechar",
    "cohechando": "cohechar",
    "cohechara": "cohechar",
    "cohecharais": "cohechar",
    "cohecharan": "cohechar",
    "cohecharas": "cohechar",
    "cohechare": "cohechar",
    "cohechareis": "cohechar",
    "cohecharemos": "cohechar",
    "cohecharen": "cohechar",
    "cohechares": "cohechar",
    "cohecharon": "cohechar",
    "cohechará": "cohechar",
    "cohecharán": "cohechar",
    "cohecharás": "cohechar",
    "cohecharé": "cohechar",
    "cohecharéis": "cohechar",
    "cohecharía": "cohechar",
    "cohecharíais": "cohechar",
    "cohecharíamos": "cohechar",
    "cohecharían": "cohechar",
    "cohecharías": "cohechar",
    "cohechas": "cohechar",
    "cohechase": "cohechar",
    "cohechaseis": "cohechar",
    "cohechasen": "cohechar",
    "cohechases": "cohechar",
    "cohechaste": "cohechar",
    "cohechasteis": "cohechar",
    "coheche": "cohechar",
    "cohechemos": "cohechar",
    "cohechen": "cohechar",
    "coheches": "cohechar",
    "cohecho": "cohechar",
    "cohechos": "cohecho",
    "cohechábamos": "cohechar",
    "cohecháis": "cohechar",
    "cohecháramos": "cohechar",
    "cohecháremos": "cohechar",
    "cohechásemos": "cohechar",
    "coheché": "cohechar",
    "cohechéis": "cohechar",
    "cohechó": "cohechar",
    "coheredera": "coheredero",
    "coherederas": "coheredero",
    "coherederos": "coheredero",
    "coherencias": "coherencia",
    "coherentes": "coherente",
    "cohesiona": "cohesionar",
    "cohesionaba": "cohesionar",
    "cohesionabais": "cohesionar",
    "cohesionaban": "cohesionar",
    "cohesionabas": "cohesionar",
    "cohesionad": "cohesionar",
    "cohesionada": "cohesionar",
    "cohesionadas": "cohesionar",
    "cohesionado": "cohesionar",
    "cohesionados": "cohesionar",
    "cohesionamos": "cohesionar",
    "cohesionan": "cohesionar",
    "cohesionando": "cohesionar",
    "cohesionara": "cohesionar",
    "cohesionarais": "cohesionar",
    "cohesionaran": "cohesionar",
    "cohesionaras": "cohesionar",
    "cohesionare": "cohesionar",
    "cohesionareis": "cohesionar",
    "cohesionaremos": "cohesionar",
    "cohesionaren": "cohesionar",
    "cohesionares": "cohesionar",
    "cohesionaron": "cohesionar",
    "cohesionará": "cohesionar",
    "cohesionarán": "cohesionar",
    "cohesionarás": "cohesionar",
    "cohesionaré": "cohesionar",
    "cohesionaréis": "cohesionar",
    "cohesionaría": "cohesionar",
    "cohesionaríais": "cohesionar",
    "cohesionaríamos": "cohesionar",
    "cohesionarían": "cohesionar",
    "cohesionarías": "cohesionar",
    "cohesionas": "cohesionar",
    "cohesionase": "cohesionar",
    "cohesionaseis": "cohesionar",
    "cohesionasen": "cohesionar",
    "cohesionases": "cohesionar",
    "cohesionaste": "cohesionar",
    "cohesionasteis": "cohesionar",
    "cohesione": "cohesionar",
    "cohesionemos": "cohesionar",
    "cohesionen": "cohesionar",
    "cohesiones": "cohesionar",
    "cohesiono": "cohesionar",
    "cohesionábamos": "cohesionar",
    "cohesionáis": "cohesionar",
    "cohesionáramos": "cohesionar",
    "cohesionáremos": "cohesionar",
    "cohesionásemos": "cohesionar",
    "cohesioné": "cohesionar",
    "cohesionéis": "cohesionar",
    "cohesionó": "cohesionar",
    "cohesiva": "cohesivo",
    "cohesivas": "cohesivo",
    "cohesivos": "cohesivo",
    "cohesora": "cohesor",
    "cohesoras": "cohesor",
    "cohesores": "cohesor",
    "cohete-splits": "cohete-split",
    "coheterías": "cohetería",
    "cohetes": "cohete",
    "cohibamos": "cohibir",
    "cohibiciones": "cohibición",
    "cohibid": "cohibir",
    "cohibida": "cohibir",
    "cohibidas": "cohibir",
    "cohibido": "cohibir",
    "cohibidos": "cohibir",
    "cohibiendo": "cohibir",
    "cohibiera": "cohibir",
    "cohibierais": "cohibir",
    "cohibieran": "cohibir",
    "cohibieras": "cohibir",
    "cohibiere": "cohibir",
    "cohibiereis": "cohibir",
    "cohibieren": "cohibir",
    "cohibieres": "cohibir",
    "cohibieron": "cohibir",
    "cohibiese": "cohibir",
    "cohibieseis": "cohibir",
    "cohibiesen": "cohibir",
    "cohibieses": "cohibir",
    "cohibimientos": "cohibimiento",
    "cohibimos": "cohibir",
    "cohibiremos": "cohibir",
    "cohibirá": "cohibir",
    "cohibirán": "cohibir",
    "cohibirás": "cohibir",
    "cohibiré": "cohibir",
    "cohibiréis": "cohibir",
    "cohibiría": "cohibir",
    "cohibiríais": "cohibir",
    "cohibiríamos": "cohibir",
    "cohibirían": "cohibir",
    "cohibirías": "cohibir",
    "cohibiste": "cohibir",
    "cohibisteis": "cohibir",
    "cohibiéramos": "cohibir",
    "cohibiéremos": "cohibir",
    "cohibiésemos": "cohibir",
    "cohibió": "cohibir",
    "cohibáis": "cohibir",
    "cohibí": "cohibir",
    "cohibía": "cohibir",
    "cohibíais": "cohibir",
    "cohibíamos": "cohibir",
    "cohibían": "cohibir",
    "cohibías": "cohibir",
    "cohibís": "cohibir",
    "cohiponimias": "cohiponimia",
    "cohipónimos": "cohipónimo",
    "cohoba": "cohobar",
    "cohobaba": "cohobar",
    "cohobabais": "cohobar",
    "cohobaban": "cohobar",
    "cohobabas": "cohobar",
    "cohobaciones": "cohobación",
    "cohobad": "cohobar",
    "cohobada": "cohobar",
    "cohobadas": "cohobar",
    "cohobado": "cohobar",
    "cohobados": "cohobar",
    "cohobamos": "cohobar",
    "cohoban": "cohobar",
    "cohobando": "cohobar",
    "cohobara": "cohobar",
    "cohobarais": "cohobar",
    "cohobaran": "cohobar",
    "cohobaras": "cohobar",
    "cohobare": "cohobar",
    "cohobareis": "cohobar",
    "cohobaremos": "cohobar",
    "cohobaren": "cohobar",
    "cohobares": "cohobar",
    "cohobaron": "cohobar",
    "cohobará": "cohobar",
    "cohobarán": "cohobar",
    "cohobarás": "cohobar",
    "cohobaré": "cohobar",
    "cohobaréis": "cohobar",
    "cohobaría": "cohobar",
    "cohobaríais": "cohobar",
    "cohobaríamos": "cohobar",
    "cohobarían": "cohobar",
    "cohobarías": "cohobar",
    "cohobas": "cohobar",
    "cohobase": "cohobar",
    "cohobaseis": "cohobar",
    "cohobasen": "cohobar",
    "cohobases": "cohobar",
    "cohobaste": "cohobar",
    "cohobasteis": "cohobar",
    "cohobe": "cohobar",
    "cohobemos": "cohobar",
    "cohoben": "cohobar",
    "cohobes": "cohobar",
    "cohobo": "cohobar",
    "cohobábamos": "cohobar",
    "cohobáis": "cohobar",
    "cohobáramos": "cohobar",
    "cohobáremos": "cohobar",
    "cohobásemos": "cohobar",
    "cohobé": "cohobar",
    "cohobéis": "cohobar",
    "cohobó": "cohobar",
    "cohombrillos": "cohombrillo",
    "cohombros": "cohombro",
    "cohomologías": "cohomología",
    "cohonesta": "cohonestar",
    "cohonestaba": "cohonestar",
    "cohonestabais": "cohonestar",
    "cohonestaban": "cohonestar",
    "cohonestabas": "cohonestar",
    "cohonestad": "cohonestar",
    "cohonestada": "cohonestar",
    "cohonestadas": "cohonestar",
    "cohonestado": "cohonestar",
    "cohonestados": "cohonestar",
    "cohonestamos": "cohonestar",
    "cohonestan": "cohonestar",
    "cohonestando": "cohonestar",
    "cohonestara": "cohonestar",
    "cohonestarais": "cohonestar",
    "cohonestaran": "cohonestar",
    "cohonestaras": "cohonestar",
    "cohonestare": "cohonestar",
    "cohonestareis": "cohonestar",
    "cohonestaremos": "cohonestar",
    "cohonestaren": "cohonestar",
    "cohonestares": "cohonestar",
    "cohonestaron": "cohonestar",
    "cohonestará": "cohonestar",
    "cohonestarán": "cohonestar",
    "cohonestarás": "cohonestar",
    "cohonestaré": "cohonestar",
    "cohonestaréis": "cohonestar",
    "cohonestaría": "cohonestar",
    "cohonestaríais": "cohonestar",
    "cohonestaríamos": "cohonestar",
    "cohonestarían": "cohonestar",
    "cohonestarías": "cohonestar",
    "cohonestas": "cohonestar",
    "cohonestase": "cohonestar",
    "cohonestaseis": "cohonestar",
    "cohonestasen": "cohonestar",
    "cohonestases": "cohonestar",
    "cohonestaste": "cohonestar",
    "cohonestasteis": "cohonestar",
    "cohoneste": "cohonestar",
    "cohonestemos": "cohonestar",
    "cohonesten": "cohonestar",
    "cohonestes": "cohonestar",
    "cohonesto": "cohonestar",
    "cohonestábamos": "cohonestar",
    "cohonestáis": "cohonestar",
    "cohonestáramos": "cohonestar",
    "cohonestáremos": "cohonestar",
    "cohonestásemos": "cohonestar",
    "cohonesté": "cohonestar",
    "cohonestéis": "cohonestar",
    "cohonestó": "cohonestar",
    "cohortes": "cohorte",
    "cohíba": "cohibir",
    "cohíban": "cohibir",
    "cohíbas": "cohibir",
    "cohíbe": "cohibir",
    "cohíben": "cohibir",
    "cohíbes": "cohibir",
    "cohíbo": "cohibir",
    "coimagenes": "coimagen",
    "coimas": "coima",
    "coimera": "coimero",
    "coimeras": "coimero",
    "coimeros": "coimero",
    "coimes": "coime",
    "coimágenes": "coimagen",
    "coincida": "coincidir",
    "coincidamos": "coincidir",
    "coincidan": "coincidir",
    "coincidas": "coincidir",
    "coincide": "coincidir",
    "coinciden": "coincidir",
    "coincidencias": "coincidencia",
    "coincidentes": "coincidente",
    "coincides": "coincidir",
    "coincidid": "coincidir",
    "coincidida": "coincidir",
    "coincididas": "coincidir",
    "coincidido": "coincidir",
    "coincididos": "coincidir",
    "coincidiendo": "coincidir",
    "coincidiera": "coincidir",
    "coincidierais": "coincidir",
    "coincidieran": "coincidir",
    "coincidieras": "coincidir",
    "coincidiere": "coincidir",
    "coincidiereis": "coincidir",
    "coincidieren": "coincidir",
    "coincidieres": "coincidir",
    "coincidieron": "coincidir",
    "coincidiese": "coincidir",
    "coincidieseis": "coincidir",
    "coincidiesen": "coincidir",
    "coincidieses": "coincidir",
    "coincidimos": "coincidir",
    "coincidiremos": "coincidir",
    "coincidirá": "coincidir",
    "coincidirán": "coincidir",
    "coincidirás": "coincidir",
    "coincidiré": "coincidir",
    "coincidiréis": "coincidir",
    "coincidiría": "coincidir",
    "coincidiríais": "coincidir",
    "coincidiríamos": "coincidir",
    "coincidirían": "coincidir",
    "coincidirías": "coincidir",
    "coincidiste": "coincidir",
    "coincidisteis": "coincidir",
    "coincidiéramos": "coincidir",
    "coincidiéremos": "coincidir",
    "coincidiésemos": "coincidir",
    "coincidió": "coincidir",
    "coincido": "coincidir",
    "coincidáis": "coincidir",
    "coincidí": "coincidir",
    "coincidía": "coincidir",
    "coincidíais": "coincidir",
    "coincidíamos": "coincidir",
    "coincidían": "coincidir",
    "coincidías": "coincidir",
    "coincidís": "coincidir",
    "coinculpada": "coinculpado",
    "coinculpadas": "coinculpado",
    "coinculpados": "coinculpado",
    "coinquilinos": "coinquilino",
    "coinyecciones": "coinyección",
    "coinés": "coiné",
    "coipos": "coipo",
    "coisogénica": "coisogénico",
    "coisogénicas": "coisogénico",
    "coisogénicos": "coisogénico",
    "coitos": "coito",
    "coja": "cojo",
    "cojamos": "coger",
    "cojan": "coger",
    "cojas": "cojo",
    "cojea": "cojear",
    "cojeaba": "cojear",
    "cojeabais": "cojear",
    "cojeaban": "cojear",
    "cojeabas": "cojear",
    "cojead": "cojear",
    "cojeada": "cojear",
    "cojeadas": "cojear",
    "cojeado": "cojear",
    "cojeados": "cojear",
    "cojeamos": "cojear",
    "cojean": "cojear",
    "cojeando": "cojear",
    "cojeara": "cojear",
    "cojearais": "cojear",
    "cojearan": "cojear",
    "cojearas": "cojear",
    "cojeare": "cojear",
    "cojeareis": "cojear",
    "cojearemos": "cojear",
    "cojearen": "cojear",
    "cojeares": "cojear",
    "cojearon": "cojear",
    "cojeará": "cojear",
    "cojearán": "cojear",
    "cojearás": "cojear",
    "cojearé": "cojear",
    "cojearéis": "cojear",
    "cojearía": "cojear",
    "cojearíais": "cojear",
    "cojearíamos": "cojear",
    "cojearían": "cojear",
    "cojearías": "cojear",
    "cojeas": "cojear",
    "cojease": "cojear",
    "cojeaseis": "cojear",
    "cojeasen": "cojear",
    "cojeases": "cojear",
    "cojeaste": "cojear",
    "cojeasteis": "cojear",
    "cojee": "cojear",
    "cojeemos": "cojear",
    "cojeen": "cojear",
    "cojees": "cojear",
    "cojeo": "cojear",
    "cojeras": "cojera",
    "cojeábamos": "cojear",
    "cojeáis": "cojear",
    "cojeáramos": "cojear",
    "cojeáremos": "cojear",
    "cojeásemos": "cojear",
    "cojeé": "cojear",
    "cojeéis": "cojear",
    "cojeó": "cojear",
    "cojijos": "cojijo",
    "cojijosa": "cojijoso",
    "cojijosas": "cojijoso",
    "cojijosos": "cojijoso",
    "cojines": "cojín",
    "cojinetes": "cojinete",
    "cojitranca": "cojitranco",
    "cojitrancas": "cojitranco",
    "cojitrancos": "cojitranco",
    "cojo": "coger",
    "cojobas": "cojoba",
    "cojones": "cojón",
    "cojonuda": "cojonudo",
    "cojonudas": "cojonudo",
    "cojonudos": "cojonudo",
    "cojos": "cojo",
    "cojuda": "cojudo",
    "cojudas": "cojudo",
    "cojudos": "cojudo",
    "cojáis": "coger",
    "colaba": "colar",
    "colabais": "colar",
    "colaban": "colar",
    "colabas": "colar",
    "colabilidades": "colabilidad",
    "colabora": "colaborar",
    "colaboraba": "colaborar",
    "colaborabais": "colaborar",
    "colaboraban": "colaborar",
    "colaborabas": "colaborar",
    "colaboraciones": "colaboración",
    "colaboracionismos": "colaboracionismo",
    "colaboracionistas": "colaboracionista",
    "colaborad": "colaborar",
    "colaborada": "colaborar",
    "colaboradas": "colaborar",
    "colaborado": "colaborar",
    "colaboradora": "colaborador",
    "colaboradoras": "colaborador",
    "colaboradores": "colaborador",
    "colaborados": "colaborar",
    "colaboramos": "colaborar",
    "colaboran": "colaborar",
    "colaborando": "colaborar",
    "colaborantes": "colaborante",
    "colaborara": "colaborar",
    "colaborarais": "colaborar",
    "colaboraran": "colaborar",
    "colaboraras": "colaborar",
    "colaborare": "colaborar",
    "colaborareis": "colaborar",
    "colaboraremos": "colaborar",
    "colaboraren": "colaborar",
    "colaborares": "colaborar",
    "colaboraron": "colaborar",
    "colaborará": "colaborar",
    "colaborarán": "colaborar",
    "colaborarás": "colaborar",
    "colaboraré": "colaborar",
    "colaboraréis": "colaborar",
    "colaboraría": "colaborar",
    "colaboraríais": "colaborar",
    "colaboraríamos": "colaborar",
    "colaborarían": "colaborar",
    "colaborarías": "colaborar",
    "colaboras": "colaborar",
    "colaborase": "colaborar",
    "colaboraseis": "colaborar",
    "colaborasen": "colaborar",
    "colaborases": "colaborar",
    "colaboraste": "colaborar",
    "colaborasteis": "colaborar",
    "colabore": "colaborar",
    "colaboremos": "colaborar",
    "colaboren": "colaborar",
    "colabores": "colaborar",
    "colaboro": "colaborar",
    "colaborábamos": "colaborar",
    "colaboráis": "colaborar",
    "colaboráramos": "colaborar",
    "colaboráremos": "colaborar",
    "colaborásemos": "colaborar",
    "colaboré": "colaborar",
    "colaboréis": "colaborar",
    "colaboró": "colaborar",
    "colaciona": "colacionar",
    "colacionaba": "colacionar",
    "colacionabais": "colacionar",
    "colacionaban": "colacionar",
    "colacionabas": "colacionar",
    "colacionad": "colacionar",
    "colacionada": "colacionar",
    "colacionadas": "colacionar",
    "colacionado": "colacionar",
    "colacionados": "colacionar",
    "colacionamos": "colacionar",
    "colacionan": "colacionar",
    "colacionando": "colacionar",
    "colacionara": "colacionar",
    "colacionarais": "colacionar",
    "colacionaran": "colacionar",
    "colacionaras": "colacionar",
    "colacionare": "colacionar",
    "colacionareis": "colacionar",
    "colacionaremos": "colacionar",
    "colacionaren": "colacionar",
    "colacionares": "colacionar",
    "colacionaron": "colacionar",
    "colacionará": "colacionar",
    "colacionarán": "colacionar",
    "colacionarás": "colacionar",
    "colacionaré": "colacionar",
    "colacionaréis": "colacionar",
    "colacionaría": "colacionar",
    "colacionaríais": "colacionar",
    "colacionaríamos": "colacionar",
    "colacionarían": "colacionar",
    "colacionarías": "colacionar",
    "colacionas": "colacionar",
    "colacionase": "colacionar",
    "colacionaseis": "colacionar",
    "colacionasen": "colacionar",
    "colacionases": "colacionar",
    "colacionaste": "colacionar",
    "colacionasteis": "colacionar",
    "colacione": "colacionar",
    "colacionemos": "colacionar",
    "colacionen": "colacionar",
    "colaciones": "colacionar",
    "colaciono": "colacionar",
    "colacionábamos": "colacionar",
    "colacionáis": "colacionar",
    "colacionáramos": "colacionar",
    "colacionáremos": "colacionar",
    "colacionásemos": "colacionar",
    "colacioné": "colacionar",
    "colacionéis": "colacionar",
    "colacionó": "colacionar",
    "colactánea": "colactáneo",
    "colactáneas": "colactáneo",
    "colactáneos": "colactáneo",
    "colad": "colar",
    "colada": "colar",
    "coladas": "colar",
    "coladeras": "coladera",
    "coladeros": "coladero",
    "coladiza": "coladizo",
    "coladizas": "coladizo",
    "coladizos": "coladizo",
    "colado": "colar",
    "coladores": "colador",
    "colados": "colar",
    "coladuras": "coladura",
    "colagoga": "colagogo",
    "colagogas": "colagogo",
    "colagogos": "colagogo",
    "colamos": "colar",
    "colando": "colar",
    "colapeces": "colapez",
    "colapsa": "colapsar",
    "colapsaba": "colapsar",
    "colapsabais": "colapsar",
    "colapsaban": "colapsar",
    "colapsabas": "colapsar",
    "colapsables": "colapsable",
    "colapsad": "colapsar",
    "colapsada": "colapsar",
    "colapsadas": "colapsar",
    "colapsado": "colapsar",
    "colapsados": "colapsar",
    "colapsamos": "colapsar",
    "colapsan": "colapsar",
    "colapsando": "colapsar",
    "colapsara": "colapsar",
    "colapsarais": "colapsar",
    "colapsaran": "colapsar",
    "colapsaras": "colapsar",
    "colapsare": "colapsar",
    "colapsareis": "colapsar",
    "colapsaremos": "colapsar",
    "colapsaren": "colapsar",
    "colapsares": "colapsar",
    "colapsaron": "colapsar",
    "colapsará": "colapsar",
    "colapsarán": "colapsar",
    "colapsarás": "colapsar",
    "colapsaré": "colapsar",
    "colapsaréis": "colapsar",
    "colapsaría": "colapsar",
    "colapsaríais": "colapsar",
    "colapsaríamos": "colapsar",
    "colapsarían": "colapsar",
    "colapsarías": "colapsar",
    "colapsas": "colapsar",
    "colapsase": "colapsar",
    "colapsaseis": "colapsar",
    "colapsasen": "colapsar",
    "colapsases": "colapsar",
    "colapsaste": "colapsar",
    "colapsasteis": "colapsar",
    "colapse": "colapsar",
    "colapsemos": "colapsar",
    "colapsen": "colapsar",
    "colapses": "colapsar",
    "colapso": "colapsar",
    "colapsos": "colapso",
    "colapsábamos": "colapsar",
    "colapsáis": "colapsar",
    "colapsáramos": "colapsar",
    "colapsáremos": "colapsar",
    "colapsásemos": "colapsar",
    "colapsé": "colapsar",
    "colapséis": "colapsar",
    "colapsó": "colapsar",
    "colara": "colar",
    "colarais": "colar",
    "colaran": "colar",
    "colaras": "colar",
    "colare": "colar",
    "colareis": "colar",
    "colaremos": "colar",
    "colaren": "colar",
    "colares": "colar",
    "colaron": "colar",
    "colará": "colar",
    "colarán": "colar",
    "colarás": "colar",
    "colaré": "colar",
    "colaréis": "colar",
    "colaría": "colar",
    "colaríais": "colar",
    "colaríamos": "colar",
    "colarían": "colar",
    "colarías": "colar",
    "colas": "cola",
    "colase": "colar",
    "colaseis": "colar",
    "colasen": "colar",
    "colases": "colar",
    "colaste": "colar",
    "colasteis": "colar",
    "colaterales": "colateral",
    "colateralidades": "colateralidad",
    "colatitudes": "colatitud",
    "colativa": "colativo",
    "colativas": "colativo",
    "colativos": "colativo",
    "colañas": "colaña",
    "colcha": "colchar",
    "colchaba": "colchar",
    "colchabais": "colchar",
    "colchaban": "colchar",
    "colchabas": "colchar",
    "colchad": "colchar",
    "colchada": "colchar",
    "colchadas": "colchar",
    "colchado": "colchar",
    "colchados": "colchar",
    "colchaduras": "colchadura",
    "colchamos": "colchar",
    "colchan": "colchar",
    "colchando": "colchar",
    "colchara": "colchar",
    "colcharais": "colchar",
    "colcharan": "colchar",
    "colcharas": "colchar",
    "colchare": "colchar",
    "colchareis": "colchar",
    "colcharemos": "colchar",
    "colcharen": "colchar",
    "colchares": "colchar",
    "colcharon": "colchar",
    "colchará": "colchar",
    "colcharán": "colchar",
    "colcharás": "colchar",
    "colcharé": "colchar",
    "colcharéis": "colchar",
    "colcharía": "colchar",
    "colcharíais": "colchar",
    "colcharíamos": "colchar",
    "colcharían": "colchar",
    "colcharías": "colchar",
    "colchas": "colchar",
    "colchase": "colchar",
    "colchaseis": "colchar",
    "colchasen": "colchar",
    "colchases": "colchar",
    "colchaste": "colchar",
    "colchasteis": "colchar",
    "colche": "colchar",
    "colchemos": "colchar",
    "colchen": "colchar",
    "colches": "colchar",
    "colcho": "colchar",
    "colchonera": "colchonero",
    "colchoneras": "colchonero",
    "colchoneros": "colchonero",
    "colchonerías": "colchonería",
    "colchones": "colchón",
    "colchonetas": "colchoneta",
    "colchábamos": "colchar",
    "colcháis": "colchar",
    "colcháramos": "colchar",
    "colcháremos": "colchar",
    "colchásemos": "colchar",
    "colché": "colchar",
    "colchéis": "colchar",
    "colchó": "colchar",
    "colcótares": "colcótar",
    "colea": "colear",
    "coleaba": "colear",
    "coleabais": "colear",
    "coleaban": "colear",
    "coleabas": "colear",
    "colead": "colear",
    "coleada": "colear",
    "coleadas": "colear",
    "coleado": "colear",
    "coleados": "colear",
    "coleaduras": "coleadura",
    "coleamos": "colear",
    "colean": "colear",
    "coleando": "colear",
    "coleara": "colear",
    "colearais": "colear",
    "colearan": "colear",
    "colearas": "colear",
    "coleare": "colear",
    "coleareis": "colear",
    "colearemos": "colear",
    "colearen": "colear",
    "coleares": "colear",
    "colearon": "colear",
    "coleará": "colear",
    "colearán": "colear",
    "colearás": "colear",
    "colearé": "colear",
    "colearéis": "colear",
    "colearía": "colear",
    "colearíais": "colear",
    "colearíamos": "colear",
    "colearían": "colear",
    "colearías": "colear",
    "coleas": "colear",
    "colease": "colear",
    "coleaseis": "colear",
    "coleasen": "colear",
    "coleases": "colear",
    "coleaste": "colear",
    "coleasteis": "colear",
    "colecciona": "coleccionar",
    "coleccionaba": "coleccionar",
    "coleccionabais": "coleccionar",
    "coleccionaban": "coleccionar",
    "coleccionabas": "coleccionar",
    "coleccionad": "coleccionar",
    "coleccionada": "coleccionar",
    "coleccionadas": "coleccionar",
    "coleccionado": "coleccionar",
    "coleccionados": "coleccionar",
    "coleccionamos": "coleccionar",
    "coleccionan": "coleccionar",
    "coleccionando": "coleccionar",
    "coleccionara": "coleccionar",
    "coleccionarais": "coleccionar",
    "coleccionaran": "coleccionar",
    "coleccionaras": "coleccionar",
    "coleccionare": "coleccionar",
    "coleccionareis": "coleccionar",
    "coleccionaremos": "coleccionar",
    "coleccionaren": "coleccionar",
    "coleccionares": "coleccionar",
    "coleccionaron": "coleccionar",
    "coleccionará": "coleccionar",
    "coleccionarán": "coleccionar",
    "coleccionarás": "coleccionar",
    "coleccionaré": "coleccionar",
    "coleccionaréis": "coleccionar",
    "coleccionaría": "coleccionar",
    "coleccionaríais": "coleccionar",
    "coleccionaríamos": "coleccionar",
    "coleccionarían": "coleccionar",
    "coleccionarías": "coleccionar",
    "coleccionas": "coleccionar",
    "coleccionase": "coleccionar",
    "coleccionaseis": "coleccionar",
    "coleccionasen": "coleccionar",
    "coleccionases": "coleccionar",
    "coleccionaste": "coleccionar",
    "coleccionasteis": "coleccionar",
    "coleccione": "coleccionar",
    "coleccionemos": "coleccionar",
    "coleccionen": "coleccionar",
    "colecciones": "coleccionar",
    "coleccionismos": "coleccionismo",
    "coleccionistas": "coleccionista",
    "colecciono": "coleccionar",
    "coleccionábamos": "coleccionar",
    "coleccionáis": "coleccionar",
    "coleccionáramos": "coleccionar",
    "coleccionáremos": "coleccionar",
    "coleccionásemos": "coleccionar",
    "coleccioné": "coleccionar",
    "coleccionéis": "coleccionar",
    "coleccionó": "coleccionar",
    "colecistos": "colecisto",
    "colecta": "colectar",
    "colectaba": "colectar",
    "colectabais": "colectar",
    "colectaban": "colectar",
    "colectabas": "colectar",
    "colectaciones": "colectación",
    "colectad": "colectar",
    "colectada": "colectar",
    "colectadas": "colectar",
    "colectado": "colectar",
    "colectados": "colectar",
    "colectamos": "colectar",
    "colectan": "colectar",
    "colectando": "colectar",
    "colectara": "colectar",
    "colectarais": "colectar",
    "colectaran": "colectar",
    "colectaras": "colectar",
    "colectare": "colectar",
    "colectareis": "colectar",
    "colectaremos": "colectar",
    "colectaren": "colectar",
    "colectares": "colectar",
    "colectaron": "colectar",
    "colectará": "colectar",
    "colectarán": "colectar",
    "colectarás": "colectar",
    "colectaré": "colectar",
    "colectaréis": "colectar",
    "colectaría": "colectar",
    "colectaríais": "colectar",
    "colectaríamos": "colectar",
    "colectarían": "colectar",
    "colectarías": "colectar",
    "colectas": "colectar",
    "colectase": "colectar",
    "colectaseis": "colectar",
    "colectasen": "colectar",
    "colectases": "colectar",
    "colectaste": "colectar",
    "colectasteis": "colectar",
    "colecte": "colectar",
    "colectemos": "colectar",
    "colecten": "colectar",
    "colectes": "colectar",
    "colecticia": "colecticio",
    "colecticias": "colecticio",
    "colecticios": "colecticio",
    "colectiva": "colectivo",
    "colectivas": "colectivo",
    "colectivice": "colectivizar",
    "colectivicemos": "colectivizar",
    "colectivicen": "colectivizar",
    "colectivices": "colectivizar",
    "colectivicé": "colectivizar",
    "colectivicéis": "colectivizar",
    "colectividades": "colectividad",
    "colectivismos": "colectivismo",
    "colectivistas": "colectivista",
    "colectiviza": "colectivizar",
    "colectivizaba": "colectivizar",
    "colectivizabais": "colectivizar",
    "colectivizaban": "colectivizar",
    "colectivizabas": "colectivizar",
    "colectivizaciones": "colectivización",
    "colectivizad": "colectivizar",
    "colectivizada": "colectivizar",
    "colectivizadas": "colectivizar",
    "colectivizado": "colectivizar",
    "colectivizados": "colectivizar",
    "colectivizamos": "colectivizar",
    "colectivizan": "colectivizar",
    "colectivizando": "colectivizar",
    "colectivizara": "colectivizar",
    "colectivizarais": "colectivizar",
    "colectivizaran": "colectivizar",
    "colectivizaras": "colectivizar",
    "colectivizare": "colectivizar",
    "colectivizareis": "colectivizar",
    "colectivizaremos": "colectivizar",
    "colectivizaren": "colectivizar",
    "colectivizares": "colectivizar",
    "colectivizaron": "colectivizar",
    "colectivizará": "colectivizar",
    "colectivizarán": "colectivizar",
    "colectivizarás": "colectivizar",
    "colectivizaré": "colectivizar",
    "colectivizaréis": "colectivizar",
    "colectivizaría": "colectivizar",
    "colectivizaríais": "colectivizar",
    "colectivizaríamos": "colectivizar",
    "colectivizarían": "colectivizar",
    "colectivizarías": "colectivizar",
    "colectivizas": "colectivizar",
    "colectivizase": "colectivizar",
    "colectivizaseis": "colectivizar",
    "colectivizasen": "colectivizar",
    "colectivizases": "colectivizar",
    "colectivizaste": "colectivizar",
    "colectivizasteis": "colectivizar",
    "colectivizo": "colectivizar",
    "colectivizábamos": "colectivizar",
    "colectivizáis": "colectivizar",
    "colectivizáramos": "colectivizar",
    "colectivizáremos": "colectivizar",
    "colectivizásemos": "colectivizar",
    "colectivizó": "colectivizar",
    "colectivos": "colectivo",
    "colecto": "colectar",
    "colectora": "colector",
    "colectoras": "colector",
    "colectores": "colector",
    "colecturías": "colecturía",
    "colectábamos": "colectar",
    "colectáis": "colectar",
    "colectáramos": "colectar",
    "colectáremos": "colectar",
    "colectásemos": "colectar",
    "colecté": "colectar",
    "colectéis": "colectar",
    "colectó": "colectar",
    "colee": "colear",
    "coleemos": "colear",
    "coleen": "colear",
    "colees": "colear",
    "colegas": "colega",
    "colegatarios": "colegatario",
    "colegia": "colegiar",
    "colegiaba": "colegiar",
    "colegiabais": "colegiar",
    "colegiaban": "colegiar",
    "colegiabas": "colegiar",
    "colegiaciones": "colegiación",
    "colegiad": "colegiar",
    "colegiada": "colegiar",
    "colegiadas": "colegiar",
    "colegiado": "colegiar",
    "colegiados": "colegiar",
    "colegiala": "colegial",
    "colegialas": "colegial",
    "colegiales": "colegial",
    "colegialidades": "colegialidad",
    "colegiamos": "colegiar",
    "colegian": "colegiar",
    "colegiando": "colegiar",
    "colegiara": "colegiar",
    "colegiarais": "colegiar",
    "colegiaran": "colegiar",
    "colegiaras": "colegiar",
    "colegiare": "colegiar",
    "colegiareis": "colegiar",
    "colegiaremos": "colegiar",
    "colegiaren": "colegiar",
    "colegiares": "colegiar",
    "colegiaron": "colegiar",
    "colegiará": "colegiar",
    "colegiarán": "colegiar",
    "colegiarás": "colegiar",
    "colegiaré": "colegiar",
    "colegiaréis": "colegiar",
    "colegiaría": "colegiar",
    "colegiaríais": "colegiar",
    "colegiaríamos": "colegiar",
    "colegiarían": "colegiar",
    "colegiarías": "colegiar",
    "colegias": "colegiar",
    "colegiase": "colegiar",
    "colegiaseis": "colegiar",
    "colegiasen": "colegiar",
    "colegiases": "colegiar",
    "colegiaste": "colegiar",
    "colegiasteis": "colegiar",
    "colegiatas": "colegiata",
    "colegiaturas": "colegiatura",
    "colegid": "colegir",
    "colegida": "colegir",
    "colegidas": "colegir",
    "colegido": "colegir",
    "colegidos": "colegir",
    "colegie": "colegiar",
    "colegiemos": "colegiar",
    "colegien": "colegiar",
    "colegies": "colegiar",
    "colegimos": "colegir",
    "colegio": "colegiar",
    "colegios": "colegio",
    "colegiremos": "colegir",
    "colegirá": "colegir",
    "colegirán": "colegir",
    "colegirás": "colegir",
    "colegiré": "colegir",
    "colegiréis": "colegir",
    "colegiría": "colegir",
    "colegiríais": "colegir",
    "colegiríamos": "colegir",
    "colegirían": "colegir",
    "colegirías": "colegir",
    "colegisladora": "colegislador",
    "colegisladoras": "colegislador",
    "colegisladores": "colegislador",
    "colegiste": "colegir",
    "colegisteis": "colegir",
    "colegiábamos": "colegiar",
    "colegiáis": "colegiar",
    "colegiáramos": "colegiar",
    "colegiáremos": "colegiar",
    "colegiásemos": "colegiar",
    "colegié": "colegiar",
    "colegiéis": "colegiar",
    "colegió": "colegiar",
    "colegí": "colegir",
    "colegía": "colegir",
    "colegíais": "colegir",
    "colegíamos": "colegir",
    "colegían": "colegir",
    "colegías": "colegir",
    "colegís": "colegir",
    "colemanitas": "colemanita",
    "colemias": "colemia",
    "colemos": "colar",
    "coleo": "colear",
    "colerinas": "colerina",
    "colerética": "colerético",
    "coleréticas": "colerético",
    "coleréticos": "colerético",
    "coles": "col",
    "colesteatomas": "colesteatoma",
    "colesteatomatosa": "colesteatomatoso",
    "colesteatomatosas": "colesteatomatoso",
    "colesteatomatosos": "colesteatomatoso",
    "colesterasas": "colesterasa",
    "colesteremias": "colesteremia",
    "colesterinas": "colesterina",
    "colesterolemias": "colesterolemia",
    "colesteroles": "colesterol",
    "coletas": "coleta",
    "coletazos": "coletazo",
    "coletillas": "coletilla",
    "coletos": "coleto",
    "coletuda": "coletudo",
    "coletudas": "coletudo",
    "coletudos": "coletudo",
    "coleábamos": "colear",
    "coleáis": "colear",
    "coleáramos": "colear",
    "coleáremos": "colear",
    "coleásemos": "colear",
    "coleé": "colear",
    "coleéis": "colear",
    "coleó": "colear",
    "coleópteros": "coleóptero",
    "coleópteross": "coleópteros",
    "colgaba": "colgar",
    "colgabais": "colgar",
    "colgaban": "colgar",
    "colgabas": "colgar",
    "colgad": "colgar",
    "colgada": "colgar",
    "colgadas": "colgar",
    "colgadera": "colgadero",
    "colgaderas": "colgadero",
    "colgaderos": "colgadero",
    "colgadiza": "colgadizo",
    "colgadizas": "colgadizo",
    "colgadizos": "colgadizo",
    "colgado": "colgar",
    "colgadores": "colgador",
    "colgados": "colgar",
    "colgaduras": "colgadura",
    "colgajos": "colgajo",
    "colgamos": "colgar",
    "colgando": "colgar",
    "colgantes": "colgante",
    "colgara": "colgar",
    "colgarais": "colgar",
    "colgaran": "colgar",
    "colgaras": "colgar",
    "colgare": "colgar",
    "colgareis": "colgar",
    "colgaremos": "colgar",
    "colgaren": "colgar",
    "colgares": "colgar",
    "colgaron": "colgar",
    "colgará": "colgar",
    "colgarán": "colgar",
    "colgarás": "colgar",
    "colgaré": "colgar",
    "colgaréis": "colgar",
    "colgaría": "colgar",
    "colgaríais": "colgar",
    "colgaríamos": "colgar",
    "colgarían": "colgar",
    "colgarías": "colgar",
    "colgase": "colgar",
    "colgaseis": "colgar",
    "colgasen": "colgar",
    "colgases": "colgar",
    "colgaste": "colgar",
    "colgasteis": "colgar",
    "colguemos": "colgar",
    "colgué": "colgar",
    "colguéis": "colgar",
    "colgábamos": "colgar",
    "colgáis": "colgar",
    "colgáramos": "colgar",
    "colgáremos": "colgar",
    "colgásemos": "colgar",
    "colgó": "colgar",
    "colhuas": "colhua",
    "colibacilos": "colibacilo",
    "colibrís": "colibrí",
    "colibríses": "colibrí",
    "coliches": "coliche",
    "colicinas": "colicina",
    "colicuaciones": "colicuación",
    "colicuativa": "colicuativo",
    "colicuativas": "colicuativo",
    "colicuativos": "colicuativo",
    "coliflores": "coliflor",
    "coliformes": "coliforme",
    "coliga": "coligar",
    "coligaba": "coligar",
    "coligabais": "coligar",
    "coligaban": "coligar",
    "coligabas": "coligar",
    "coligaciones": "coligación",
    "coligad": "coligar",
    "coligada": "coligar",
    "coligadas": "coligar",
    "coligado": "coligar",
    "coligados": "coligar",
    "coligaduras": "coligadura",
    "coligamientos": "coligamiento",
    "coligamos": "coligar",
    "coligan": "coligar",
    "coligando": "coligar",
    "coligara": "coligar",
    "coligarais": "coligar",
    "coligaran": "coligar",
    "coligaras": "coligar",
    "coligare": "coligar",
    "coligareis": "coligar",
    "coligaremos": "coligar",
    "coligaren": "coligar",
    "coligares": "coligar",
    "coligaron": "coligar",
    "coligará": "coligar",
    "coligarán": "coligar",
    "coligarás": "coligar",
    "coligaré": "coligar",
    "coligaréis": "coligar",
    "coligaría": "coligar",
    "coligaríais": "coligar",
    "coligaríamos": "coligar",
    "coligarían": "coligar",
    "coligarías": "coligar",
    "coligas": "coligar",
    "coligase": "coligar",
    "coligaseis": "coligar",
    "coligasen": "coligar",
    "coligases": "coligar",
    "coligaste": "coligar",
    "coligasteis": "coligar",
    "coligativa": "coligativo",
    "coligativas": "coligativo",
    "coligativos": "coligativo",
    "colige": "colegir",
    "coligen": "colegir",
    "coliges": "colegir",
    "coligiendo": "colegir",
    "coligiera": "colegir",
    "coligierais": "colegir",
    "coligieran": "colegir",
    "coligieras": "colegir",
    "coligiere": "colegir",
    "coligiereis": "colegir",
    "coligieren": "colegir",
    "coligieres": "colegir",
    "coligieron": "colegir",
    "coligiese": "colegir",
    "coligieseis": "colegir",
    "coligiesen": "colegir",
    "coligieses": "colegir",
    "coligiéramos": "colegir",
    "coligiéremos": "colegir",
    "coligiésemos": "colegir",
    "coligió": "colegir",
    "coligo": "coligar",
    "coligruesa": "coligrueso",
    "coligruesas": "coligrueso",
    "coligruesos": "coligrueso",
    "coligue": "coligar",
    "coliguemos": "coligar",
    "coliguen": "coligar",
    "coligues": "coligar",
    "coligué": "coligar",
    "coliguéis": "coligar",
    "coligábamos": "coligar",
    "coligáis": "coligar",
    "coligáramos": "coligar",
    "coligáremos": "coligar",
    "coligásemos": "coligar",
    "coligó": "coligar",
    "colija": "colegir",
    "colijamos": "colegir",
    "colijan": "colegir",
    "colijas": "colegir",
    "colijo": "colegir",
    "colijáis": "colegir",
    "colilargas": "colilarga",
    "colillas": "colilla",
    "colillera": "colillero",
    "colilleras": "colillero",
    "colilleros": "colillero",
    "colima": "colimar",
    "colimaba": "colimar",
    "colimabais": "colimar",
    "colimaban": "colimar",
    "colimabas": "colimar",
    "colimaciones": "colimación",
    "colimad": "colimar",
    "colimada": "colimar",
    "colimadas": "colimar",
    "colimado": "colimar",
    "colimadores": "colimador",
    "colimados": "colimar",
    "colimamos": "colimar",
    "coliman": "colimar",
    "colimando": "colimar",
    "colimara": "colimar",
    "colimarais": "colimar",
    "colimaran": "colimar",
    "colimaras": "colimar",
    "colimare": "colimar",
    "colimareis": "colimar",
    "colimaremos": "colimar",
    "colimaren": "colimar",
    "colimares": "colimar",
    "colimaron": "colimar",
    "colimará": "colimar",
    "colimarán": "colimar",
    "colimarás": "colimar",
    "colimaré": "colimar",
    "colimaréis": "colimar",
    "colimaría": "colimar",
    "colimaríais": "colimar",
    "colimaríamos": "colimar",
    "colimarían": "colimar",
    "colimarías": "colimar",
    "colimas": "colimar",
    "colimase": "colimar",
    "colimaseis": "colimar",
    "colimasen": "colimar",
    "colimases": "colimar",
    "colimaste": "colimar",
    "colimasteis": "colimar",
    "colimbos": "colimbo",
    "colime": "colimar",
    "colimemos": "colimar",
    "colimen": "colimar",
    "colimes": "colimar",
    "colimo": "colimar",
    "colimábamos": "colimar",
    "colimáis": "colimar",
    "colimáramos": "colimar",
    "colimáremos": "colimar",
    "colimásemos": "colimar",
    "colimé": "colimar",
    "coliméis": "colimar",
    "colimó": "colimar",
    "colina": "colín",
    "colinabos": "colinabo",
    "colinas": "colina",
    "colinda": "colindar",
    "colindaba": "colindar",
    "colindabais": "colindar",
    "colindaban": "colindar",
    "colindabas": "colindar",
    "colindad": "colindar",
    "colindada": "colindar",
    "colindadas": "colindar",
    "colindado": "colindar",
    "colindados": "colindar",
    "colindamos": "colindar",
    "colindan": "colindar",
    "colindando": "colindar",
    "colindantes": "colindante",
    "colindara": "colindar",
    "colindarais": "colindar",
    "colindaran": "colindar",
    "colindaras": "colindar",
    "colindare": "colindar",
    "colindareis": "colindar",
    "colindaremos": "colindar",
    "colindaren": "colindar",
    "colindares": "colindar",
    "colindaron": "colindar",
    "colindará": "colindar",
    "colindarán": "colindar",
    "colindarás": "colindar",
    "colindaré": "colindar",
    "colindaréis": "colindar",
    "colindaría": "colindar",
    "colindaríais": "colindar",
    "colindaríamos": "colindar",
    "colindarían": "colindar",
    "colindarías": "colindar",
    "colindas": "colindar",
    "colindase": "colindar",
    "colindaseis": "colindar",
    "colindasen": "colindar",
    "colindases": "colindar",
    "colindaste": "colindar",
    "colindasteis": "colindar",
    "colinde": "colindar",
    "colindemos": "colindar",
    "colinden": "colindar",
    "colindes": "colindar",
    "colindo": "colindar",
    "colindábamos": "colindar",
    "colindáis": "colindar",
    "colindáramos": "colindar",
    "colindáremos": "colindar",
    "colindásemos": "colindar",
    "colindé": "colindar",
    "colindéis": "colindar",
    "colindó": "colindar",
    "colineaciones": "colineación",
    "colineales": "colineal",
    "colines": "colín",
    "colinesterasas": "colinesterasa",
    "colinetas": "colineta",
    "colinos": "colino",
    "colinérgica": "colinérgico",
    "colinérgicas": "colinérgico",
    "colinérgicos": "colinérgico",
    "colipava": "colipavo",
    "colipavas": "colipavo",
    "colipavos": "colipavo",
    "colirios": "colirio",
    "colirrojos": "colirrojo",
    "colirrábanos": "colirrábano",
    "colisas": "colisa",
    "coliseos": "coliseo",
    "colisiona": "colisionar",
    "colisionaba": "colisionar",
    "colisionabais": "colisionar",
    "colisionaban": "colisionar",
    "colisionabas": "colisionar",
    "colisionad": "colisionar",
    "colisionada": "colisionar",
    "colisionadas": "colisionar",
    "colisionado": "colisionar",
    "colisionadores": "colisionador",
    "colisionados": "colisionar",
    "colisionamos": "colisionar",
    "colisionan": "colisionar",
    "colisionando": "colisionar",
    "colisionara": "colisionar",
    "colisionarais": "colisionar",
    "colisionaran": "colisionar",
    "colisionaras": "colisionar",
    "colisionare": "colisionar",
    "colisionareis": "colisionar",
    "colisionaremos": "colisionar",
    "colisionaren": "colisionar",
    "colisionares": "colisionar",
    "colisionaron": "colisionar",
    "colisionará": "colisionar",
    "colisionarán": "colisionar",
    "colisionarás": "colisionar",
    "colisionaré": "colisionar",
    "colisionaréis": "colisionar",
    "colisionaría": "colisionar",
    "colisionaríais": "colisionar",
    "colisionaríamos": "colisionar",
    "colisionarían": "colisionar",
    "colisionarías": "colisionar",
    "colisionas": "colisionar",
    "colisionase": "colisionar",
    "colisionaseis": "colisionar",
    "colisionasen": "colisionar",
    "colisionases": "colisionar",
    "colisionaste": "colisionar",
    "colisionasteis": "colisionar",
    "colisione": "colisionar",
    "colisionemos": "colisionar",
    "colisionen": "colisionar",
    "colisiones": "colisionar",
    "colisiono": "colisionar",
    "colisionábamos": "colisionar",
    "colisionáis": "colisionar",
    "colisionáramos": "colisionar",
    "colisionáremos": "colisionar",
    "colisionásemos": "colisionar",
    "colisioné": "colisionar",
    "colisionéis": "colisionar",
    "colisionó": "colisionar",
    "colisos": "coliso",
    "colistas": "colista",
    "colitigantes": "colitigante",
    "colizas": "coliza",
    "colladas": "collada",
    "collados": "collado",
    "collages": "collage",
    "collalbas": "collalba",
    "collarejas": "collareja",
    "collares": "collar",
    "collarines": "collarín",
    "collarinos": "collarino",
    "collas": "colla",
    "collaza": "collazo",
    "collazas": "collazo",
    "collazos": "collazo",
    "collejas": "colleja",
    "colleras": "collera",
    "collerones": "collerón",
    "collies": "collie",
    "collona": "collón",
    "collonadas": "collonada",
    "collonas": "collón",
    "collonerías": "collonería",
    "collones": "collón",
    "colma": "colmo",
    "colmaba": "colmar",
    "colmabais": "colmar",
    "colmaban": "colmar",
    "colmabas": "colmar",
    "colmad": "colmar",
    "colmada": "colmar",
    "colmadas": "colmar",
    "colmado": "colmar",
    "colmados": "colmar",
    "colmamos": "colmar",
    "colman": "colmar",
    "colmando": "colmar",
    "colmara": "colmar",
    "colmarais": "colmar",
    "colmaran": "colmar",
    "colmaras": "colmar",
    "colmare": "colmar",
    "colmareis": "colmar",
    "colmaremos": "colmar",
    "colmaren": "colmar",
    "colmares": "colmar",
    "colmaron": "colmar",
    "colmará": "colmar",
    "colmarán": "colmar",
    "colmarás": "colmar",
    "colmaré": "colmar",
    "colmaréis": "colmar",
    "colmaría": "colmar",
    "colmaríais": "colmar",
    "colmaríamos": "colmar",
    "colmarían": "colmar",
    "colmarías": "colmar",
    "colmas": "colmo",
    "colmase": "colmar",
    "colmaseis": "colmar",
    "colmasen": "colmar",
    "colmases": "colmar",
    "colmaste": "colmar",
    "colmasteis": "colmar",
    "colmataciones": "colmatación",
    "colmatajes": "colmataje",
    "colme": "colmar",
    "colmemos": "colmar",
    "colmen": "colmar",
    "colmenares": "colmenar",
    "colmenas": "colmena",
    "colmenera": "colmenero",
    "colmeneras": "colmenero",
    "colmeneros": "colmenero",
    "colmenillas": "colmenilla",
    "colmes": "colmar",
    "colmilladas": "colmillada",
    "colmillares": "colmillar",
    "colmillazos": "colmillazo",
    "colmillos": "colmillo",
    "colmilluda": "colmilludo",
    "colmilludas": "colmilludo",
    "colmilludos": "colmilludo",
    "colmo": "colmar",
    "colmos": "colmo",
    "colmábamos": "colmar",
    "colmáis": "colmar",
    "colmáramos": "colmar",
    "colmáremos": "colmar",
    "colmásemos": "colmar",
    "colmé": "colmar",
    "colméis": "colmar",
    "colmó": "colmar",
    "colobomas": "coloboma",
    "coloca": "colocar",
    "colocaba": "colocar",
    "colocabais": "colocar",
    "colocaban": "colocar",
    "colocabas": "colocar",
    "colocaciones": "colocación",
    "colocad": "colocar",
    "colocada": "colocar",
    "colocadas": "colocar",
    "colocado": "colocar",
    "colocadores": "colocador",
    "colocados": "colocar",
    "colocamos": "colocar",
    "colocan": "colocar",
    "colocando": "colocar",
    "colocara": "colocar",
    "colocarais": "colocar",
    "colocaran": "colocar",
    "colocaras": "colocar",
    "colocare": "colocar",
    "colocareis": "colocar",
    "colocaremos": "colocar",
    "colocaren": "colocar",
    "colocares": "colocar",
    "colocaron": "colocar",
    "colocará": "colocar",
    "colocarán": "colocar",
    "colocarás": "colocar",
    "colocaré": "colocar",
    "colocaréis": "colocar",
    "colocaría": "colocar",
    "colocaríais": "colocar",
    "colocaríamos": "colocar",
    "colocarían": "colocar",
    "colocarías": "colocar",
    "colocas": "colocar",
    "colocase": "colocar",
    "colocaseis": "colocar",
    "colocasen": "colocar",
    "colocases": "colocar",
    "colocasias": "colocasia",
    "colocaste": "colocar",
    "colocasteis": "colocar",
    "coloco": "colocar",
    "colocutora": "colocutor",
    "colocutoras": "colocutor",
    "colocutores": "colocutor",
    "colocábamos": "colocar",
    "colocáis": "colocar",
    "colocáramos": "colocar",
    "colocáremos": "colocar",
    "colocásemos": "colocar",
    "colocó": "colocar",
    "colodiones": "colodión",
    "colodras": "colodra",
    "colodrillos": "colodrillo",
    "colofones": "colofón",
    "colofonias": "colofonia",
    "colofonitas": "colofonita",
    "cologaritmos": "cologaritmo",
    "coloidales": "coloidal",
    "coloides": "coloide",
    "coloidoquímicas": "coloidoquímica",
    "colombiana": "colombiano",
    "colombianas": "colombiano",
    "colombianismos": "colombianismo",
    "colombianos": "colombiano",
    "colombiculturas": "colombicultura",
    "colombina": "colombino",
    "colombinas": "colombino",
    "colombinos": "colombino",
    "colombofilias": "colombofilia",
    "colombos": "colombo",
    "colombófila": "colombófilo",
    "colombófilas": "colombófilo",
    "colombófilos": "colombófilo",
    "colonatos": "colonato",
    "colones": "colón",
    "coloniajes": "coloniaje",
    "coloniales": "colonial",
    "colonialismos": "colonialismo",
    "colonialistas": "colonialista",
    "colonias": "colonia",
    "colonice": "colonizar",
    "colonicemos": "colonizar",
    "colonicen": "colonizar",
    "colonices": "colonizar",
    "colonicé": "colonizar",
    "colonicéis": "colonizar",
    "coloniza": "colonizar",
    "colonizaba": "colonizar",
    "colonizabais": "colonizar",
    "colonizaban": "colonizar",
    "colonizabas": "colonizar",
    "colonizaciones": "colonización",
    "colonizad": "colonizar",
    "colonizada": "colonizar",
    "colonizadas": "colonizar",
    "colonizado": "colonizar",
    "colonizadora": "colonizador",
    "colonizadoras": "colonizador",
    "colonizadores": "colonizador",
    "colonizados": "colonizar",
    "colonizamos": "colonizar",
    "colonizan": "colonizar",
    "colonizando": "colonizar",
    "colonizara": "colonizar",
    "colonizarais": "colonizar",
    "colonizaran": "colonizar",
    "colonizaras": "colonizar",
    "colonizare": "colonizar",
    "colonizareis": "colonizar",
    "colonizaremos": "colonizar",
    "colonizaren": "colonizar",
    "colonizares": "colonizar",
    "colonizaron": "colonizar",
    "colonizará": "colonizar",
    "colonizarán": "colonizar",
    "colonizarás": "colonizar",
    "colonizaré": "colonizar",
    "colonizaréis": "colonizar",
    "colonizaría": "colonizar",
    "colonizaríais": "colonizar",
    "colonizaríamos": "colonizar",
    "colonizarían": "colonizar",
    "colonizarías": "colonizar",
    "colonizas": "colonizar",
    "colonizase": "colonizar",
    "colonizaseis": "colonizar",
    "colonizasen": "colonizar",
    "colonizases": "colonizar",
    "colonizaste": "colonizar",
    "colonizasteis": "colonizar",
    "colonizo": "colonizar",
    "colonizábamos": "colonizar",
    "colonizáis": "colonizar",
    "colonizáramos": "colonizar",
    "colonizáremos": "colonizar",
    "colonizásemos": "colonizar",
    "colonizó": "colonizar",
    "colonos": "colono",
    "coloque": "colocar",
    "coloquemos": "colocar",
    "coloquen": "colocar",
    "coloques": "colocar",
    "coloquiales": "coloquial",
    "coloquialismos": "coloquialismo",
    "coloquios": "coloquio",
    "coloqué": "colocar",
    "coloquéis": "colocar",
    "colora": "colorir",
    "coloraba": "colorar",
    "colorabais": "colorar",
    "coloraban": "colorar",
    "colorabas": "colorar",
    "coloraciones": "coloración",
    "colorad": "colorar",
    "colorada": "colorar",
    "coloradas": "colorar",
    "colorado": "colorar",
    "colorados": "colorar",
    "coloramos": "colorir",
    "coloran": "colorir",
    "colorando": "colorar",
    "colorantes": "colorante",
    "colorara": "colorar",
    "colorarais": "colorar",
    "coloraran": "colorar",
    "coloraras": "colorar",
    "colorare": "colorar",
    "colorareis": "colorar",
    "coloraremos": "colorar",
    "coloraren": "colorar",
    "colorares": "colorar",
    "coloraron": "colorar",
    "colorará": "colorar",
    "colorarán": "colorar",
    "colorarás": "colorar",
    "coloraré": "colorar",
    "coloraréis": "colorar",
    "coloraría": "colorar",
    "coloraríais": "colorar",
    "coloraríamos": "colorar",
    "colorarían": "colorar",
    "colorarías": "colorar",
    "coloras": "colorir",
    "colorase": "colorar",
    "coloraseis": "colorar",
    "colorasen": "colorar",
    "colorases": "colorar",
    "coloraste": "colorar",
    "colorasteis": "colorar",
    "colorativa": "colorativo",
    "colorativas": "colorativo",
    "colorativos": "colorativo",
    "colore": "colorir",
    "colorea": "colorear",
    "coloreaba": "colorear",
    "coloreabais": "colorear",
    "coloreaban": "colorear",
    "coloreabas": "colorear",
    "coloread": "colorear",
    "coloreada": "colorear",
    "coloreadas": "colorear",
    "coloreado": "colorear",
    "coloreados": "colorear",
    "coloreamos": "colorear",
    "colorean": "colorear",
    "coloreando": "colorear",
    "coloreara": "colorear",
    "colorearais": "colorear",
    "colorearan": "colorear",
    "colorearas": "colorear",
    "coloreare": "colorear",
    "coloreareis": "colorear",
    "colorearemos": "colorear",
    "colorearen": "colorear",
    "coloreares": "colorear",
    "colorearon": "colorear",
    "coloreará": "colorear",
    "colorearán": "colorear",
    "colorearás": "colorear",
    "colorearé": "colorear",
    "colorearéis": "colorear",
    "colorearía": "colorear",
    "colorearíais": "colorear",
    "colorearíamos": "colorear",
    "colorearían": "colorear",
    "colorearías": "colorear",
    "coloreas": "colorear",
    "colorease": "colorear",
    "coloreaseis": "colorear",
    "coloreasen": "colorear",
    "coloreases": "colorear",
    "coloreaste": "colorear",
    "coloreasteis": "colorear",
    "coloree": "colorear",
    "coloreemos": "colorear",
    "coloreen": "colorear",
    "colorees": "colorear",
    "coloremos": "colorar",
    "coloren": "colorir",
    "coloreo": "colorear",
    "coloreos": "coloreo",
    "colores": "colorir",
    "coloretes": "colorete",
    "coloreábamos": "colorear",
    "coloreáis": "colorear",
    "coloreáramos": "colorear",
    "coloreáremos": "colorear",
    "coloreásemos": "colorear",
    "coloreé": "colorear",
    "coloreéis": "colorear",
    "coloreó": "colorear",
    "colorid": "colorir",
    "colorida": "colorir",
    "coloridas": "colorir",
    "colorido": "colorir",
    "coloridos": "colorir",
    "coloriendo": "colorir",
    "coloriera": "colorir",
    "colorierais": "colorir",
    "colorieran": "colorir",
    "colorieras": "colorir",
    "coloriere": "colorir",
    "coloriereis": "colorir",
    "colorieren": "colorir",
    "colorieres": "colorir",
    "colorieron": "colorir",
    "coloriese": "colorir",
    "colorieseis": "colorir",
    "coloriesen": "colorir",
    "colorieses": "colorir",
    "colorimetrías": "colorimetría",
    "colorimos": "colorir",
    "colorimétrica": "colorimétrico",
    "colorimétricas": "colorimétrico",
    "colorimétricos": "colorimétrico",
    "colorines": "colorín",
    "coloriremos": "colorir",
    "colorirá": "colorir",
    "colorirán": "colorir",
    "colorirás": "colorir",
    "coloriré": "colorir",
    "coloriréis": "colorir",
    "coloriría": "colorir",
    "coloriríais": "colorir",
    "coloriríamos": "colorir",
    "colorirían": "colorir",
    "colorirías": "colorir",
    "coloristas": "colorista",
    "coloriste": "colorir",
    "coloristeis": "colorir",
    "coloriéramos": "colorir",
    "coloriéremos": "colorir",
    "coloriésemos": "colorir",
    "colorió": "colorir",
    "coloro": "colorir",
    "colorábamos": "colorar",
    "coloráis": "colorir",
    "coloráramos": "colorar",
    "coloráremos": "colorar",
    "colorásemos": "colorar",
    "coloré": "colorar",
    "coloréis": "colorar",
    "colorí": "colorir",
    "coloría": "colorir",
    "coloríais": "colorir",
    "coloríamos": "colorir",
    "colorían": "colorir",
    "colorías": "colorir",
    "colorímetros": "colorímetro",
    "colorís": "colorir",
    "coloró": "colorar",
    "colosales": "colosal",
    "colosalismos": "colosalismo",
    "colosos": "coloso",
    "colquicinas": "colquicina",
    "coluda": "coludir",
    "coludamos": "coludir",
    "coludan": "coludir",
    "coludas": "coludir",
    "colude": "coludir",
    "coluden": "coludir",
    "coludes": "coludir",
    "coludid": "coludir",
    "coludida": "coludir",
    "coludidas": "coludir",
    "coludido": "coludir",
    "coludidos": "coludir",
    "coludiendo": "coludir",
    "coludiera": "coludir",
    "coludierais": "coludir",
    "coludieran": "coludir",
    "coludieras": "coludir",
    "coludiere": "coludir",
    "coludiereis": "coludir",
    "coludieren": "coludir",
    "coludieres": "coludir",
    "coludieron": "coludir",
    "coludiese": "coludir",
    "coludieseis": "coludir",
    "coludiesen": "coludir",
    "coludieses": "coludir",
    "coludimos": "coludir",
    "coludiremos": "coludir",
    "coludirá": "coludir",
    "coludirán": "coludir",
    "coludirás": "coludir",
    "coludiré": "coludir",
    "coludiréis": "coludir",
    "coludiría": "coludir",
    "coludiríais": "coludir",
    "coludiríamos": "coludir",
    "coludirían": "coludir",
    "coludirías": "coludir",
    "coludiste": "coludir",
    "coludisteis": "coludir",
    "coludiéramos": "coludir",
    "coludiéremos": "coludir",
    "coludiésemos": "coludir",
    "coludió": "coludir",
    "coludo": "coludir",
    "coludáis": "coludir",
    "coludí": "coludir",
    "coludía": "coludir",
    "coludíais": "coludir",
    "coludíamos": "coludir",
    "coludían": "coludir",
    "coludías": "coludir",
    "coludís": "coludir",
    "columbarios": "columbario",
    "columbetas": "columbeta",
    "columbina": "columbino",
    "columbinas": "columbino",
    "columbinos": "columbino",
    "columbio": "coulomb",
    "columbios": "coulomb",
    "columbitas": "columbita",
    "columbra": "columbrar",
    "columbraba": "columbrar",
    "columbrabais": "columbrar",
    "columbraban": "columbrar",
    "columbrabas": "columbrar",
    "columbrad": "columbrar",
    "columbrada": "columbrar",
    "columbradas": "columbrar",
    "columbrado": "columbrar",
    "columbrados": "columbrar",
    "columbramos": "columbrar",
    "columbran": "columbrar",
    "columbrando": "columbrar",
    "columbrara": "columbrar",
    "columbrarais": "columbrar",
    "columbraran": "columbrar",
    "columbraras": "columbrar",
    "columbrare": "columbrar",
    "columbrareis": "columbrar",
    "columbraremos": "columbrar",
    "columbraren": "columbrar",
    "columbrares": "columbrar",
    "columbraron": "columbrar",
    "columbrará": "columbrar",
    "columbrarán": "columbrar",
    "columbrarás": "columbrar",
    "columbraré": "columbrar",
    "columbraréis": "columbrar",
    "columbraría": "columbrar",
    "columbraríais": "columbrar",
    "columbraríamos": "columbrar",
    "columbrarían": "columbrar",
    "columbrarías": "columbrar",
    "columbras": "columbrar",
    "columbrase": "columbrar",
    "columbraseis": "columbrar",
    "columbrasen": "columbrar",
    "columbrases": "columbrar",
    "columbraste": "columbrar",
    "columbrasteis": "columbrar",
    "columbre": "columbrar",
    "columbremos": "columbrar",
    "columbren": "columbrar",
    "columbres": "columbrar",
    "columbretes": "columbrete",
    "columbro": "columbrar",
    "columbrábamos": "columbrar",
    "columbráis": "columbrar",
    "columbráramos": "columbrar",
    "columbráremos": "columbrar",
    "columbrásemos": "columbrar",
    "columbré": "columbrar",
    "columbréis": "columbrar",
    "columbró": "columbrar",
    "columbófila": "columbófilo",
    "columbófilas": "columbófilo",
    "columbófilos": "columbófilo",
    "columelas": "columela",
    "columnares": "columnar",
    "columnaria": "columnario",
    "columnarias": "columnario",
    "columnarios": "columnario",
    "columnas": "columna",
    "columnatas": "columnata",
    "columnistas": "columnista",
    "columpia": "columpiar",
    "columpiaba": "columpiar",
    "columpiabais": "columpiar",
    "columpiaban": "columpiar",
    "columpiabas": "columpiar",
    "columpiad": "columpiar",
    "columpiada": "columpiar",
    "columpiadas": "columpiar",
    "columpiado": "columpiar",
    "columpiados": "columpiar",
    "columpiamos": "columpiar",
    "columpian": "columpiar",
    "columpiando": "columpiar",
    "columpiara": "columpiar",
    "columpiarais": "columpiar",
    "columpiaran": "columpiar",
    "columpiaras": "columpiar",
    "columpiare": "columpiar",
    "columpiareis": "columpiar",
    "columpiaremos": "columpiar",
    "columpiaren": "columpiar",
    "columpiares": "columpiar",
    "columpiaron": "columpiar",
    "columpiará": "columpiar",
    "columpiarán": "columpiar",
    "columpiarás": "columpiar",
    "columpiaré": "columpiar",
    "columpiaréis": "columpiar",
    "columpiaría": "columpiar",
    "columpiaríais": "columpiar",
    "columpiaríamos": "columpiar",
    "columpiarían": "columpiar",
    "columpiarías": "columpiar",
    "columpias": "columpiar",
    "columpiase": "columpiar",
    "columpiaseis": "columpiar",
    "columpiasen": "columpiar",
    "columpiases": "columpiar",
    "columpiaste": "columpiar",
    "columpiasteis": "columpiar",
    "columpie": "columpiar",
    "columpiemos": "columpiar",
    "columpien": "columpiar",
    "columpies": "columpiar",
    "columpio": "columpiar",
    "columpios": "columpio",
    "columpiábamos": "columpiar",
    "columpiáis": "columpiar",
    "columpiáramos": "columpiar",
    "columpiáremos": "columpiar",
    "columpiásemos": "columpiar",
    "columpié": "columpiar",
    "columpiéis": "columpiar",
    "columpió": "columpiar",
    "coluros": "coluro",
    "colusiones": "colusión",
    "colusores": "colusor",
    "colusoria": "colusorio",
    "colusorias": "colusorio",
    "colusorios": "colusorio",
    "colutorios": "colutorio",
    "coluviales": "coluvial",
    "coluvies": "coluvie",
    "coluviones": "coluvión",
    "coluvios": "coluvio",
    "colzas": "colza",
    "colábamos": "colar",
    "colágenas": "colágena",
    "colágenos": "colágeno",
    "coláis": "colar",
    "coláramos": "colar",
    "coláremos": "colar",
    "colásemos": "colar",
    "colé": "colar",
    "colédoca": "colédoco",
    "colédocas": "colédoco",
    "colédocos": "colédoco",
    "coléis": "colar",
    "colémbolos": "colémbolo",
    "colénquimas": "colénquima",
    "colérica": "colérico",
    "coléricas": "colérico",
    "coléricos": "colérico",
    "colículos": "colículo",
    "colímites": "colímite",
    "coló": "colar",
    "colúmbidos": "colúmbido",
    "coma": "comer",
    "comadrazgos": "comadrazgo",
    "comadrea": "comadrear",
    "comadreaba": "comadrear",
    "comadreabais": "comadrear",
    "comadreaban": "comadrear",
    "comadreabas": "comadrear",
    "comadread": "comadrear",
    "comadreada": "comadrear",
    "comadreadas": "comadrear",
    "comadreado": "comadrear",
    "comadreados": "comadrear",
    "comadreamos": "comadrear",
    "comadrean": "comadrear",
    "comadreando": "comadrear",
    "comadreara": "comadrear",
    "comadrearais": "comadrear",
    "comadrearan": "comadrear",
    "comadrearas": "comadrear",
    "comadreare": "comadrear",
    "comadreareis": "comadrear",
    "comadrearemos": "comadrear",
    "comadrearen": "comadrear",
    "comadreares": "comadrear",
    "comadrearon": "comadrear",
    "comadreará": "comadrear",
    "comadrearán": "comadrear",
    "comadrearás": "comadrear",
    "comadrearé": "comadrear",
    "comadrearéis": "comadrear",
    "comadrearía": "comadrear",
    "comadrearíais": "comadrear",
    "comadrearíamos": "comadrear",
    "comadrearían": "comadrear",
    "comadrearías": "comadrear",
    "comadreas": "comadrear",
    "comadrease": "comadrear",
    "comadreaseis": "comadrear",
    "comadreasen": "comadrear",
    "comadreases": "comadrear",
    "comadreaste": "comadrear",
    "comadreasteis": "comadrear",
    "comadree": "comadrear",
    "comadreemos": "comadrear",
    "comadreen": "comadrear",
    "comadrees": "comadrear",
    "comadrejas": "comadreja",
    "comadreo": "comadrear",
    "comadreos": "comadreo",
    "comadrera": "comadrero",
    "comadreras": "comadrero",
    "comadreros": "comadrero",
    "comadrerías": "comadrería",
    "comadres": "comadre",
    "comadreábamos": "comadrear",
    "comadreáis": "comadrear",
    "comadreáramos": "comadrear",
    "comadreáremos": "comadrear",
    "comadreásemos": "comadrear",
    "comadreé": "comadrear",
    "comadreéis": "comadrear",
    "comadreó": "comadrear",
    "comadronas": "comadrona",
    "comadrones": "comadrón",
    "comales": "comal",
    "comalias": "comalia",
    "comamos": "comer",
    "coman": "comer",
    "comanches": "comanche",
    "comanda": "comandar",
    "comandaba": "comandar",
    "comandabais": "comandar",
    "comandaban": "comandar",
    "comandabas": "comandar",
    "comandad": "comandar",
    "comandada": "comandar",
    "comandadas": "comandar",
    "comandado": "comandar",
    "comandados": "comandar",
    "comandamos": "comandar",
    "comandan": "comandar",
    "comandancias": "comandancia",
    "comandando": "comandar",
    "comandantas": "comandanta",
    "comandantes": "comandante",
    "comandara": "comandar",
    "comandarais": "comandar",
    "comandaran": "comandar",
    "comandaras": "comandar",
    "comandare": "comandar",
    "comandareis": "comandar",
    "comandaremos": "comandar",
    "comandaren": "comandar",
    "comandares": "comandar",
    "comandaron": "comandar",
    "comandará": "comandar",
    "comandarán": "comandar",
    "comandarás": "comandar",
    "comandaré": "comandar",
    "comandaréis": "comandar",
    "comandaría": "comandar",
    "comandaríais": "comandar",
    "comandaríamos": "comandar",
    "comandarían": "comandar",
    "comandarías": "comandar",
    "comandas": "comandar",
    "comandase": "comandar",
    "comandaseis": "comandar",
    "comandasen": "comandar",
    "comandases": "comandar",
    "comandaste": "comandar",
    "comandasteis": "comandar",
    "comande": "comandar",
    "comandemos": "comandar",
    "comanden": "comandar",
    "comandes": "comandar",
    "comandita": "comanditar",
    "comanditaba": "comanditar",
    "comanditabais": "comanditar",
    "comanditaban": "comanditar",
    "comanditabas": "comanditar",
    "comanditad": "comanditar",
    "comanditada": "comanditar",
    "comanditadas": "comanditar",
    "comanditado": "comanditar",
    "comanditados": "comanditar",
    "comanditamos": "comanditar",
    "comanditan": "comanditar",
    "comanditando": "comanditar",
    "comanditara": "comanditar",
    "comanditarais": "comanditar",
    "comanditaran": "comanditar",
    "comanditaras": "comanditar",
    "comanditare": "comanditar",
    "comanditareis": "comanditar",
    "comanditaremos": "comanditar",
    "comanditaren": "comanditar",
    "comanditares": "comanditar",
    "comanditaria": "comanditario",
    "comanditarias": "comanditario",
    "comanditarios": "comanditario",
    "comanditaron": "comanditar",
    "comanditará": "comanditar",
    "comanditarán": "comanditar",
    "comanditarás": "comanditar",
    "comanditaré": "comanditar",
    "comanditaréis": "comanditar",
    "comanditaría": "comanditar",
    "comanditaríais": "comanditar",
    "comanditaríamos": "comanditar",
    "comanditarían": "comanditar",
    "comanditarías": "comanditar",
    "comanditas": "comanditar",
    "comanditase": "comanditar",
    "comanditaseis": "comanditar",
    "comanditasen": "comanditar",
    "comanditases": "comanditar",
    "comanditaste": "comanditar",
    "comanditasteis": "comanditar",
    "comandite": "comanditar",
    "comanditemos": "comanditar",
    "comanditen": "comanditar",
    "comandites": "comanditar",
    "comandito": "comanditar",
    "comanditábamos": "comanditar",
    "comanditáis": "comanditar",
    "comanditáramos": "comanditar",
    "comanditáremos": "comanditar",
    "comanditásemos": "comanditar",
    "comandité": "comanditar",
    "comanditéis": "comanditar",
    "comanditó": "comanditar",
    "comando": "comandar",
    "comandos": "comando",
    "comandábamos": "comandar",
    "comandáis": "comandar",
    "comandáramos": "comandar",
    "comandáremos": "comandar",
    "comandásemos": "comandar",
    "comandé": "comandar",
    "comandéis": "comandar",
    "comandó": "comandar",
    "comarca": "comarcar",
    "comarcaba": "comarcar",
    "comarcabais": "comarcar",
    "comarcaban": "comarcar",
    "comarcabas": "comarcar",
    "comarcad": "comarcar",
    "comarcada": "comarcar",
    "comarcadas": "comarcar",
    "comarcado": "comarcar",
    "comarcados": "comarcar",
    "comarcales": "comarcal",
    "comarcamos": "comarcar",
    "comarcan": "comarcar",
    "comarcana": "comarcano",
    "comarcanas": "comarcano",
    "comarcando": "comarcar",
    "comarcanos": "comarcano",
    "comarcara": "comarcar",
    "comarcarais": "comarcar",
    "comarcaran": "comarcar",
    "comarcaras": "comarcar",
    "comarcare": "comarcar",
    "comarcareis": "comarcar",
    "comarcaremos": "comarcar",
    "comarcaren": "comarcar",
    "comarcares": "comarcar",
    "comarcaron": "comarcar",
    "comarcará": "comarcar",
    "comarcarán": "comarcar",
    "comarcarás": "comarcar",
    "comarcaré": "comarcar",
    "comarcaréis": "comarcar",
    "comarcaría": "comarcar",
    "comarcaríais": "comarcar",
    "comarcaríamos": "comarcar",
    "comarcarían": "comarcar",
    "comarcarías": "comarcar",
    "comarcas": "comarcar",
    "comarcase": "comarcar",
    "comarcaseis": "comarcar",
    "comarcasen": "comarcar",
    "comarcases": "comarcar",
    "comarcaste": "comarcar",
    "comarcasteis": "comarcar",
    "comarco": "comarcar",
    "comarcábamos": "comarcar",
    "comarcáis": "comarcar",
    "comarcáramos": "comarcar",
    "comarcáremos": "comarcar",
    "comarcásemos": "comarcar",
    "comarcó": "comarcar",
    "comarque": "comarcar",
    "comarquemos": "comarcar",
    "comarquen": "comarcar",
    "comarques": "comarcar",
    "comarqué": "comarcar",
    "comarquéis": "comarcar",
    "comas": "comer",
    "comatosa": "comatoso",
    "comatosas": "comatoso",
    "comatosos": "comatoso",
    "comba": "combo",
    "combaba": "combar",
    "combabais": "combar",
    "combaban": "combar",
    "combabas": "combar",
    "combad": "combar",
    "combada": "combar",
    "combadas": "combar",
    "combado": "combar",
    "combados": "combar",
    "combaduras": "combadura",
    "combamos": "combar",
    "comban": "combar",
    "combando": "combar",
    "combara": "combar",
    "combarais": "combar",
    "combaran": "combar",
    "combaras": "combar",
    "combare": "combar",
    "combareis": "combar",
    "combaremos": "combar",
    "combaren": "combar",
    "combares": "combar",
    "combaron": "combar",
    "combará": "combar",
    "combarán": "combar",
    "combarás": "combar",
    "combaré": "combar",
    "combaréis": "combar",
    "combaría": "combar",
    "combaríais": "combar",
    "combaríamos": "combar",
    "combarían": "combar",
    "combarías": "combar",
    "combas": "combo",
    "combase": "combar",
    "combaseis": "combar",
    "combasen": "combar",
    "combases": "combar",
    "combaste": "combar",
    "combasteis": "combar",
    "combata": "combatir",
    "combatamos": "combatir",
    "combatan": "combatir",
    "combatas": "combatir",
    "combate": "combatir",
    "combaten": "combatir",
    "combates": "combatir",
    "combatibles": "combatible",
    "combatid": "combatir",
    "combatida": "combatir",
    "combatidas": "combatir",
    "combatido": "combatir",
    "combatidos": "combatir",
    "combatiendo": "combatir",
    "combatientes": "combatiente",
    "combatiera": "combatir",
    "combatierais": "combatir",
    "combatieran": "combatir",
    "combatieras": "combatir",
    "combatiere": "combatir",
    "combatiereis": "combatir",
    "combatieren": "combatir",
    "combatieres": "combatir",
    "combatieron": "combatir",
    "combatiese": "combatir",
    "combatieseis": "combatir",
    "combatiesen": "combatir",
    "combatieses": "combatir",
    "combatimos": "combatir",
    "combatiremos": "combatir",
    "combatirá": "combatir",
    "combatirán": "combatir",
    "combatirás": "combatir",
    "combatiré": "combatir",
    "combatiréis": "combatir",
    "combatiría": "combatir",
    "combatiríais": "combatir",
    "combatiríamos": "combatir",
    "combatirían": "combatir",
    "combatirías": "combatir",
    "combatiste": "combatir",
    "combatisteis": "combatir",
    "combativa": "combativo",
    "combativas": "combativo",
    "combatividades": "combatividad",
    "combativos": "combativo",
    "combatiéramos": "combatir",
    "combatiéremos": "combatir",
    "combatiésemos": "combatir",
    "combatió": "combatir",
    "combato": "combatir",
    "combatáis": "combatir",
    "combatí": "combatir",
    "combatía": "combatir",
    "combatíais": "combatir",
    "combatíamos": "combatir",
    "combatían": "combatir",
    "combatías": "combatir",
    "combatís": "combatir",
    "combe": "combar",
    "combemos": "combar",
    "comben": "combar",
    "combes": "combe",
    "combeses": "combés",
    "combina": "combinar",
    "combinaba": "combinar",
    "combinabais": "combinar",
    "combinaban": "combinar",
    "combinabas": "combinar",
    "combinables": "combinable",
    "combinaciones": "combinación",
    "combinad": "combinar",
    "combinada": "combinar",
    "combinadas": "combinar",
    "combinado": "combinar",
    "combinadora": "combinador",
    "combinadoras": "combinador",
    "combinadores": "combinador",
    "combinados": "combinar",
    "combinamos": "combinar",
    "combinan": "combinar",
    "combinando": "combinar",
    "combinara": "combinar",
    "combinarais": "combinar",
    "combinaran": "combinar",
    "combinaras": "combinar",
    "combinare": "combinar",
    "combinareis": "combinar",
    "combinaremos": "combinar",
    "combinaren": "combinar",
    "combinares": "combinar",
    "combinaron": "combinar",
    "combinará": "combinar",
    "combinarán": "combinar",
    "combinarás": "combinar",
    "combinaré": "combinar",
    "combinaréis": "combinar",
    "combinaría": "combinar",
    "combinaríais": "combinar",
    "combinaríamos": "combinar",
    "combinarían": "combinar",
    "combinarías": "combinar",
    "combinas": "combinar",
    "combinase": "combinar",
    "combinaseis": "combinar",
    "combinasen": "combinar",
    "combinases": "combinar",
    "combinaste": "combinar",
    "combinasteis": "combinar",
    "combinatoria": "combinatorio",
    "combinatorias": "combinatorio",
    "combinatorios": "combinatorio",
    "combine": "combinar",
    "combinemos": "combinar",
    "combinen": "combinar",
    "combines": "combinar",
    "combino": "combinar",
    "combinábamos": "combinar",
    "combináis": "combinar",
    "combináramos": "combinar",
    "combináremos": "combinar",
    "combinásemos": "combinar",
    "combiné": "combinar",
    "combinéis": "combinar",
    "combinó": "combinar",
    "combo": "combar",
    "combos": "combo",
    "combosa": "comboso",
    "combosas": "comboso",
    "combosos": "comboso",
    "comburentes": "comburente",
    "combustibilidades": "combustibilidad",
    "combustibles": "combustible",
    "combustiones": "combustión",
    "combábamos": "combar",
    "combáis": "combar",
    "combáramos": "combar",
    "combáremos": "combar",
    "combásemos": "combar",
    "combé": "combar",
    "combéis": "combar",
    "combó": "combar",
    "come": "comer",
    "comed": "comer",
    "comederos": "comedero",
    "comedia": "comediar",
    "comediaba": "comediar",
    "comediabais": "comediar",
    "comediaban": "comediar",
    "comediabas": "comediar",
    "comediad": "comediar",
    "comediada": "comediar",
    "comediadas": "comediar",
    "comediado": "comediar",
    "comediados": "comediar",
    "comediamos": "comediar",
    "comedian": "comediar",
    "comediando": "comediar",
    "comedianta": "comediante",
    "comediantas": "comediante",
    "comediantes": "comediante",
    "comediara": "comediar",
    "comediarais": "comediar",
    "comediaran": "comediar",
    "comediaras": "comediar",
    "comediare": "comediar",
    "comediareis": "comediar",
    "comediaremos": "comediar",
    "comediaren": "comediar",
    "comediares": "comediar",
    "comediaron": "comediar",
    "comediará": "comediar",
    "comediarán": "comediar",
    "comediarás": "comediar",
    "comediaré": "comediar",
    "comediaréis": "comediar",
    "comediaría": "comediar",
    "comediaríais": "comediar",
    "comediaríamos": "comediar",
    "comediarían": "comediar",
    "comediarías": "comediar",
    "comedias": "comediar",
    "comediase": "comediar",
    "comediaseis": "comediar",
    "comediasen": "comediar",
    "comediases": "comediar",
    "comediaste": "comediar",
    "comediasteis": "comediar",
    "comedid": "comedir",
    "comedida": "comedir",
    "comedidas": "comedir",
    "comedido": "comedir",
    "comedidos": "comedir",
    "comedie": "comediar",
    "comediemos": "comediar",
    "comedien": "comediar",
    "comedies": "comediar",
    "comedimientos": "comedimiento",
    "comedimos": "comedir",
    "comedio": "comediar",
    "comedios": "comedio",
    "comediremos": "comedir",
    "comedirá": "comedir",
    "comedirán": "comedir",
    "comedirás": "comedir",
    "comediré": "comedir",
    "comediréis": "comedir",
    "comediría": "comedir",
    "comediríais": "comedir",
    "comediríamos": "comedir",
    "comedirían": "comedir",
    "comedirías": "comedir",
    "comediste": "comedir",
    "comedisteis": "comedir",
    "comediábamos": "comediar",
    "comediáis": "comediar",
    "comediáramos": "comediar",
    "comediáremos": "comediar",
    "comediásemos": "comediar",
    "comedié": "comediar",
    "comediéis": "comediar",
    "comedió": "comediar",
    "comediógrafa": "comediógrafo",
    "comediógrafas": "comediógrafo",
    "comediógrafos": "comediógrafo",
    "comedones": "comedón",
    "comedora": "comedor",
    "comedoras": "comedor",
    "comedores": "comedor",
    "comedí": "comedir",
    "comedía": "comedir",
    "comedíais": "comedir",
    "comedíamos": "comedir",
    "comedían": "comedir",
    "comedías": "comedir",
    "comedís": "comedir",
    "comejeneras": "comejenera",
    "comejenes": "comején",
    "comemos": "comer",
    "comen": "comer",
    "comencemos": "comenzar",
    "comencé": "comenzar",
    "comencéis": "comenzar",
    "comendadora": "comendador",
    "comendadoras": "comendador",
    "comendadores": "comendador",
    "comendatarios": "comendatario",
    "comendaticia": "comendaticio",
    "comendaticias": "comendaticio",
    "comendaticios": "comendaticio",
    "comendatoria": "comendatorio",
    "comendatorias": "comendatorio",
    "comendatorios": "comendatorio",
    "comenderos": "comendero",
    "comensales": "comensal",
    "comensalismos": "comensalismo",
    "comensalías": "comensalía",
    "comenta": "comentar",
    "comentaba": "comentar",
    "comentabais": "comentar",
    "comentaban": "comentar",
    "comentabas": "comentar",
    "comentad": "comentar",
    "comentada": "comentar",
    "comentadas": "comentar",
    "comentado": "comentar",
    "comentadora": "comentador",
    "comentadoras": "comentador",
    "comentadores": "comentador",
    "comentados": "comentar",
    "comentamos": "comentar",
    "comentan": "comentar",
    "comentando": "comentar",
    "comentara": "comentar",
    "comentarais": "comentar",
    "comentaran": "comentar",
    "comentaras": "comentar",
    "comentare": "comentar",
    "comentareis": "comentar",
    "comentaremos": "comentar",
    "comentaren": "comentar",
    "comentares": "comentar",
    "comentarios": "comentario",
    "comentaristas": "comentarista",
    "comentaron": "comentar",
    "comentará": "comentar",
    "comentarán": "comentar",
    "comentarás": "comentar",
    "comentaré": "comentar",
    "comentaréis": "comentar",
    "comentaría": "comentar",
    "comentaríais": "comentar",
    "comentaríamos": "comentar",
    "comentarían": "comentar",
    "comentarías": "comentar",
    "comentas": "comentar",
    "comentase": "comentar",
    "comentaseis": "comentar",
    "comentasen": "comentar",
    "comentases": "comentar",
    "comentaste": "comentar",
    "comentasteis": "comentar",
    "comente": "comentar",
    "comentemos": "comentar",
    "comenten": "comentar",
    "comentes": "comentar",
    "comento": "comentar",
    "comentos": "comento",
    "comentábamos": "comentar",
    "comentáis": "comentar",
    "comentáramos": "comentar",
    "comentáremos": "comentar",
    "comentásemos": "comentar",
    "comenté": "comentar",
    "comentéis": "comentar",
    "comentó": "comentar",
    "comenzaba": "comenzar",
    "comenzabais": "comenzar",
    "comenzaban": "comenzar",
    "comenzabas": "comenzar",
    "comenzad": "comenzar",
    "comenzada": "comenzar",
    "comenzadas": "comenzar",
    "comenzado": "comenzar",
    "comenzados": "comenzar",
    "comenzamos": "comenzar",
    "comenzando": "comenzar",
    "comenzara": "comenzar",
    "comenzarais": "comenzar",
    "comenzaran": "comenzar",
    "comenzaras": "comenzar",
    "comenzare": "comenzar",
    "comenzareis": "comenzar",
    "comenzaremos": "comenzar",
    "comenzaren": "comenzar",
    "comenzares": "comenzar",
    "comenzaron": "comenzar",
    "comenzará": "comenzar",
    "comenzarán": "comenzar",
    "comenzarás": "comenzar",
    "comenzaré": "comenzar",
    "comenzaréis": "comenzar",
    "comenzaría": "comenzar",
    "comenzaríais": "comenzar",
    "comenzaríamos": "comenzar",
    "comenzarían": "comenzar",
    "comenzarías": "comenzar",
    "comenzase": "comenzar",
    "comenzaseis": "comenzar",
    "comenzasen": "comenzar",
    "comenzases": "comenzar",
    "comenzaste": "comenzar",
    "comenzasteis": "comenzar",
    "comenzábamos": "comenzar",
    "comenzáis": "comenzar",
    "comenzáramos": "comenzar",
    "comenzáremos": "comenzar",
    "comenzásemos": "comenzar",
    "comenzó": "comenzar",
    "comercia": "comerciar",
    "comerciaba": "comerciar",
    "comerciabais": "comerciar",
    "comerciaban": "comerciar",
    "comerciabas": "comerciar",
    "comerciabilidades": "comerciabilidad",
    "comerciables": "comerciable",
    "comerciad": "comerciar",
    "comerciada": "comerciar",
    "comerciadas": "comerciar",
    "comerciado": "comerciar",
    "comerciados": "comerciar",
    "comerciales": "comercial",
    "comercialice": "comercializar",
    "comercialicemos": "comercializar",
    "comercialicen": "comercializar",
    "comercialices": "comercializar",
    "comercialicé": "comercializar",
    "comercialicéis": "comercializar",
    "comercializa": "comercializar",
    "comercializaba": "comercializar",
    "comercializabais": "comercializar",
    "comercializaban": "comercializar",
    "comercializabas": "comercializar",
    "comercializables": "comercializable",
    "comercializaciones": "comercialización",
    "comercializad": "comercializar",
    "comercializada": "comercializar",
    "comercializadas": "comercializar",
    "comercializado": "comercializar",
    "comercializadora": "comercializador",
    "comercializadoras": "comercializador",
    "comercializadores": "comercializador",
    "comercializados": "comercializar",
    "comercializamos": "comercializar",
    "comercializan": "comercializar",
    "comercializando": "comercializar",
    "comercializara": "comercializar",
    "comercializarais": "comercializar",
    "comercializaran": "comercializar",
    "comercializaras": "comercializar",
    "comercializare": "comercializar",
    "comercializareis": "comercializar",
    "comercializaremos": "comercializar",
    "comercializaren": "comercializar",
    "comercializares": "comercializar",
    "comercializaron": "comercializar",
    "comercializará": "comercializar",
    "comercializarán": "comercializar",
    "comercializarás": "comercializar",
    "comercializaré": "comercializar",
    "comercializaréis": "comercializar",
    "comercializaría": "comercializar",
    "comercializaríais": "comercializar",
    "comercializaríamos": "comercializar",
    "comercializarían": "comercializar",
    "comercializarías": "comercializar",
    "comercializas": "comercializar",
    "comercializase": "comercializar",
    "comercializaseis": "comercializar",
    "comercializasen": "comercializar",
    "comercializases": "comercializar",
    "comercializaste": "comercializar",
    "comercializasteis": "comercializar",
    "comercializo": "comercializar",
    "comercializábamos": "comercializar",
    "comercializáis": "comercializar",
    "comercializáramos": "comercializar",
    "comercializáremos": "comercializar",
    "comercializásemos": "comercializar",
    "comercializó": "comercializar",
    "comerciamos": "comerciar",
    "comercian": "comerciar",
    "comerciando": "comerciar",
    "comerciantes": "comerciante",
    "comerciara": "comerciar",
    "comerciarais": "comerciar",
    "comerciaran": "comerciar",
    "comerciaras": "comerciar",
    "comerciare": "comerciar",
    "comerciareis": "comerciar",
    "comerciaremos": "comerciar",
    "comerciaren": "comerciar",
    "comerciares": "comerciar",
    "comerciaron": "comerciar",
    "comerciará": "comerciar",
    "comerciarán": "comerciar",
    "comerciarás": "comerciar",
    "comerciaré": "comerciar",
    "comerciaréis": "comerciar",
    "comerciaría": "comerciar",
    "comerciaríais": "comerciar",
    "comerciaríamos": "comerciar",
    "comerciarían": "comerciar",
    "comerciarías": "comerciar",
    "comercias": "comerciar",
    "comerciase": "comerciar",
    "comerciaseis": "comerciar",
    "comerciasen": "comerciar",
    "comerciases": "comerciar",
    "comerciaste": "comerciar",
    "comerciasteis": "comerciar",
    "comercie": "comerciar",
    "comerciemos": "comerciar",
    "comercien": "comerciar",
    "comercies": "comerciar",
    "comercio": "comerciar",
    "comercios": "comercio",
    "comerciábamos": "comerciar",
    "comerciáis": "comerciar",
    "comerciáramos": "comerciar",
    "comerciáremos": "comerciar",
    "comerciásemos": "comerciar",
    "comercié": "comerciar",
    "comerciéis": "comerciar",
    "comerció": "comerciar",
    "comeremos": "comer",
    "comeres": "comer",
    "comerá": "comer",
    "comerán": "comer",
    "comerás": "comer",
    "comeré": "comer",
    "comeréis": "comer",
    "comería": "comer",
    "comeríais": "comer",
    "comeríamos": "comer",
    "comerían": "comer",
    "comerías": "comer",
    "comes": "comer",
    "comestibles": "comestible",
    "cometa": "cometer",
    "cometamos": "cometer",
    "cometan": "cometer",
    "cometaria": "cometario",
    "cometarias": "cometario",
    "cometarios": "cometario",
    "cometas": "cometer",
    "comete": "cometer",
    "cometed": "cometer",
    "cometemos": "cometer",
    "cometen": "cometer",
    "cometeremos": "cometer",
    "cometerá": "cometer",
    "cometerán": "cometer",
    "cometerás": "cometer",
    "cometeré": "cometer",
    "cometeréis": "cometer",
    "cometería": "cometer",
    "cometeríais": "cometer",
    "cometeríamos": "cometer",
    "cometerían": "cometer",
    "cometerías": "cometer",
    "cometes": "cometer",
    "cometida": "cometer",
    "cometidas": "cometer",
    "cometido": "cometer",
    "cometidos": "cometido",
    "cometiendo": "cometer",
    "cometiera": "cometer",
    "cometierais": "cometer",
    "cometieran": "cometer",
    "cometieras": "cometer",
    "cometiere": "cometer",
    "cometiereis": "cometer",
    "cometieren": "cometer",
    "cometieres": "cometer",
    "cometieron": "cometer",
    "cometiese": "cometer",
    "cometieseis": "cometer",
    "cometiesen": "cometer",
    "cometieses": "cometer",
    "cometimos": "cometer",
    "cometiste": "cometer",
    "cometisteis": "cometer",
    "cometiéramos": "cometer",
    "cometiéremos": "cometer",
    "cometiésemos": "cometer",
    "cometió": "cometer",
    "cometo": "cometer",
    "cometáis": "cometer",
    "cometéis": "cometer",
    "cometí": "cometer",
    "cometía": "cometer",
    "cometíais": "cometer",
    "cometíamos": "cometer",
    "cometían": "cometer",
    "cometías": "cometer",
    "comezones": "comezón",
    "comibles": "comible",
    "comicastros": "comicastro",
    "comiciales": "comicial",
    "comicidades": "comicidad",
    "comicios": "comicio",
    "comics": "comic",
    "comida": "comer",
    "comidamos": "comedir",
    "comidan": "comedir",
    "comidas": "comida",
    "comide": "comedir",
    "comiden": "comedir",
    "comides": "comedir",
    "comidiendo": "comedir",
    "comidiera": "comedir",
    "comidierais": "comedir",
    "comidieran": "comedir",
    "comidieras": "comedir",
    "comidiere": "comedir",
    "comidiereis": "comedir",
    "comidieren": "comedir",
    "comidieres": "comedir",
    "comidieron": "comedir",
    "comidiese": "comedir",
    "comidieseis": "comedir",
    "comidiesen": "comedir",
    "comidieses": "comedir",
    "comidillas": "comidilla",
    "comidiéramos": "comedir",
    "comidiéremos": "comedir",
    "comidiésemos": "comedir",
    "comidió": "comedir",
    "comido": "comer",
    "comidos": "comer",
    "comidáis": "comedir",
    "comience": "comenzar",
    "comiencen": "comenzar",
    "comiences": "comenzar",
    "comiendo": "comer",
    "comienza": "comenzar",
    "comienzan": "comenzar",
    "comienzas": "comenzar",
    "comienzo": "comenzar",
    "comienzos": "comienzo",
    "comiera": "comer",
    "comierais": "comer",
    "comieran": "comer",
    "comieras": "comer",
    "comiere": "comer",
    "comiereis": "comer",
    "comieren": "comer",
    "comieres": "comer",
    "comieron": "comer",
    "comiese": "comer",
    "comieseis": "comer",
    "comiesen": "comer",
    "comieses": "comer",
    "comilitonas": "comilitona",
    "comilona": "comilón",
    "comilonas": "comilona",
    "comilones": "comilón",
    "comimos": "comer",
    "cominea": "cominear",
    "comineaba": "cominear",
    "comineabais": "cominear",
    "comineaban": "cominear",
    "comineabas": "cominear",
    "cominead": "cominear",
    "comineada": "cominear",
    "comineadas": "cominear",
    "comineado": "cominear",
    "comineados": "cominear",
    "comineamos": "cominear",
    "cominean": "cominear",
    "comineando": "cominear",
    "comineara": "cominear",
    "cominearais": "cominear",
    "cominearan": "cominear",
    "cominearas": "cominear",
    "comineare": "cominear",
    "comineareis": "cominear",
    "cominearemos": "cominear",
    "cominearen": "cominear",
    "comineares": "cominear",
    "cominearon": "cominear",
    "comineará": "cominear",
    "cominearán": "cominear",
    "cominearás": "cominear",
    "cominearé": "cominear",
    "cominearéis": "cominear",
    "cominearía": "cominear",
    "cominearíais": "cominear",
    "cominearíamos": "cominear",
    "cominearían": "cominear",
    "cominearías": "cominear",
    "comineas": "cominear",
    "cominease": "cominear",
    "comineaseis": "cominear",
    "comineasen": "cominear",
    "comineases": "cominear",
    "comineaste": "cominear",
    "comineasteis": "cominear",
    "cominee": "cominear",
    "comineemos": "cominear",
    "comineen": "cominear",
    "cominees": "cominear",
    "comineo": "cominear",
    "cominera": "cominero",
    "comineras": "cominero",
    "comineros": "cominero",
    "cominerías": "cominería",
    "comineábamos": "cominear",
    "comineáis": "cominear",
    "comineáramos": "cominear",
    "comineáremos": "cominear",
    "comineásemos": "cominear",
    "comineé": "cominear",
    "comineéis": "cominear",
    "comineó": "cominear",
    "cominos": "comino",
    "comiquerías": "comiquería",
    "comisa": "comisar",
    "comisaba": "comisar",
    "comisabais": "comisar",
    "comisaban": "comisar",
    "comisabas": "comisar",
    "comisad": "comisar",
    "comisada": "comisar",
    "comisadas": "comisar",
    "comisado": "comisar",
    "comisados": "comisar",
    "comisamos": "comisar",
    "comisan": "comisar",
    "comisando": "comisar",
    "comisara": "comisar",
    "comisarais": "comisar",
    "comisaran": "comisar",
    "comisaras": "comisar",
    "comisare": "comisar",
    "comisareis": "comisar",
    "comisaremos": "comisar",
    "comisaren": "comisar",
    "comisares": "comisar",
    "comisariados": "comisariado",
    "comisarias": "comisaria",
    "comisarios": "comisario",
    "comisaron": "comisar",
    "comisará": "comisar",
    "comisarán": "comisar",
    "comisarás": "comisar",
    "comisaré": "comisar",
    "comisaréis": "comisar",
    "comisaría": "comisar",
    "comisaríais": "comisar",
    "comisaríamos": "comisar",
    "comisarían": "comisar",
    "comisarías": "comisaría",
    "comisas": "comisar",
    "comisase": "comisar",
    "comisaseis": "comisar",
    "comisasen": "comisar",
    "comisases": "comisar",
    "comisaste": "comisar",
    "comisasteis": "comisar",
    "comisca": "comiscar",
    "comiscaba": "comiscar",
    "comiscabais": "comiscar",
    "comiscaban": "comiscar",
    "comiscabas": "comiscar",
    "comiscad": "comiscar",
    "comiscada": "comiscar",
    "comiscadas": "comiscar",
    "comiscado": "comiscar",
    "comiscados": "comiscar",
    "comiscamos": "comiscar",
    "comiscan": "comiscar",
    "comiscando": "comiscar",
    "comiscara": "comiscar",
    "comiscarais": "comiscar",
    "comiscaran": "comiscar",
    "comiscaras": "comiscar",
    "comiscare": "comiscar",
    "comiscareis": "comiscar",
    "comiscaremos": "comiscar",
    "comiscaren": "comiscar",
    "comiscares": "comiscar",
    "comiscaron": "comiscar",
    "comiscará": "comiscar",
    "comiscarán": "comiscar",
    "comiscarás": "comiscar",
    "comiscaré": "comiscar",
    "comiscaréis": "comiscar",
    "comiscaría": "comiscar",
    "comiscaríais": "comiscar",
    "comiscaríamos": "comiscar",
    "comiscarían": "comiscar",
    "comiscarías": "comiscar",
    "comiscas": "comiscar",
    "comiscase": "comiscar",
    "comiscaseis": "comiscar",
    "comiscasen": "comiscar",
    "comiscases": "comiscar",
    "comiscaste": "comiscar",
    "comiscasteis": "comiscar",
    "comisco": "comiscar",
    "comiscábamos": "comiscar",
    "comiscáis": "comiscar",
    "comiscáramos": "comiscar",
    "comiscáremos": "comiscar",
    "comiscásemos": "comiscar",
    "comiscó": "comiscar",
    "comise": "comisar",
    "comisemos": "comisar",
    "comisen": "comisar",
    "comises": "comisar",
    "comisiona": "comisionar",
    "comisionaba": "comisionar",
    "comisionabais": "comisionar",
    "comisionaban": "comisionar",
    "comisionabas": "comisionar",
    "comisionad": "comisionar",
    "comisionada": "comisionar",
    "comisionadas": "comisionar",
    "comisionado": "comisionar",
    "comisionados": "comisionar",
    "comisionamos": "comisionar",
    "comisionan": "comisionar",
    "comisionando": "comisionar",
    "comisionara": "comisionar",
    "comisionarais": "comisionar",
    "comisionaran": "comisionar",
    "comisionaras": "comisionar",
    "comisionare": "comisionar",
    "comisionareis": "comisionar",
    "comisionaremos": "comisionar",
    "comisionaren": "comisionar",
    "comisionares": "comisionar",
    "comisionaron": "comisionar",
    "comisionará": "comisionar",
    "comisionarán": "comisionar",
    "comisionarás": "comisionar",
    "comisionaré": "comisionar",
    "comisionaréis": "comisionar",
    "comisionaría": "comisionar",
    "comisionaríais": "comisionar",
    "comisionaríamos": "comisionar",
    "comisionarían": "comisionar",
    "comisionarías": "comisionar",
    "comisionas": "comisionar",
    "comisionase": "comisionar",
    "comisionaseis": "comisionar",
    "comisionasen": "comisionar",
    "comisionases": "comisionar",
    "comisionaste": "comisionar",
    "comisionasteis": "comisionar",
    "comisione": "comisionar",
    "comisionemos": "comisionar",
    "comisionen": "comisionar",
    "comisiones": "comisionar",
    "comisionistas": "comisionista",
    "comisiono": "comisionar",
    "comisionábamos": "comisionar",
    "comisionáis": "comisionar",
    "comisionáramos": "comisionar",
    "comisionáremos": "comisionar",
    "comisionásemos": "comisionar",
    "comisioné": "comisionar",
    "comisionéis": "comisionar",
    "comisionó": "comisionar",
    "comiso": "comisar",
    "comisoria": "comisorio",
    "comisorias": "comisorio",
    "comisorios": "comisorio",
    "comisos": "comiso",
    "comisque": "comiscar",
    "comisquea": "comisquear",
    "comisqueaba": "comisquear",
    "comisqueabais": "comisquear",
    "comisqueaban": "comisquear",
    "comisqueabas": "comisquear",
    "comisquead": "comisquear",
    "comisqueada": "comisquear",
    "comisqueadas": "comisquear",
    "comisqueado": "comisquear",
    "comisqueados": "comisquear",
    "comisqueamos": "comisquear",
    "comisquean": "comisquear",
    "comisqueando": "comisquear",
    "comisqueara": "comisquear",
    "comisquearais": "comisquear",
    "comisquearan": "comisquear",
    "comisquearas": "comisquear",
    "comisqueare": "comisquear",
    "comisqueareis": "comisquear",
    "comisquearemos": "comisquear",
    "comisquearen": "comisquear",
    "comisqueares": "comisquear",
    "comisquearon": "comisquear",
    "comisqueará": "comisquear",
    "comisquearán": "comisquear",
    "comisquearás": "comisquear",
    "comisquearé": "comisquear",
    "comisquearéis": "comisquear",
    "comisquearía": "comisquear",
    "comisquearíais": "comisquear",
    "comisquearíamos": "comisquear",
    "comisquearían": "comisquear",
    "comisquearías": "comisquear",
    "comisqueas": "comisquear",
    "comisquease": "comisquear",
    "comisqueaseis": "comisquear",
    "comisqueasen": "comisquear",
    "comisqueases": "comisquear",
    "comisqueaste": "comisquear",
    "comisqueasteis": "comisquear",
    "comisquee": "comisquear",
    "comisqueemos": "comisquear",
    "comisqueen": "comisquear",
    "comisquees": "comisquear",
    "comisquemos": "comiscar",
    "comisquen": "comiscar",
    "comisqueo": "comisquear",
    "comisques": "comiscar",
    "comisqueábamos": "comisquear",
    "comisqueáis": "comisquear",
    "comisqueáramos": "comisquear",
    "comisqueáremos": "comisquear",
    "comisqueásemos": "comisquear",
    "comisqueé": "comisquear",
    "comisqueéis": "comisquear",
    "comisqueó": "comisquear",
    "comisqué": "comiscar",
    "comisquéis": "comiscar",
    "comiste": "comer",
    "comisteis": "comer",
    "comistrajos": "comistrajo",
    "comisurales": "comisural",
    "comisuras": "comisura",
    "comisábamos": "comisar",
    "comisáis": "comisar",
    "comisáramos": "comisar",
    "comisáremos": "comisar",
    "comisásemos": "comisar",
    "comisé": "comisar",
    "comiséis": "comisar",
    "comisó": "comisar",
    "comitados": "comitado",
    "comitativa": "comitativo",
    "comitativas": "comitativo",
    "comitativos": "comitativo",
    "comitentes": "comitente",
    "comitivas": "comitiva",
    "comités": "comité",
    "comiéramos": "comer",
    "comiéremos": "comer",
    "comiésemos": "comer",
    "comió": "comer",
    "commonlaws": "commonlaw",
    "como": "comer",
    "comodantes": "comodante",
    "comodatarios": "comodatario",
    "comodatoria": "comodatorio",
    "comodatorias": "comodatorio",
    "comodatorios": "comodatorio",
    "comodatos": "comodato",
    "comodidades": "comodidad",
    "comodines": "comodín",
    "comodona": "comodón",
    "comodonas": "comodón",
    "comodonerías": "comodonería",
    "comodones": "comodón",
    "comodoros": "comodoro",
    "compacidades": "compacidad",
    "compacta": "compacto",
    "compactaba": "compactar",
    "compactabais": "compactar",
    "compactaban": "compactar",
    "compactabas": "compactar",
    "compactaciones": "compactación",
    "compactad": "compactar",
    "compactada": "compactar",
    "compactadas": "compactar",
    "compactado": "compactar",
    "compactadora": "compactador",
    "compactadoras": "compactador",
    "compactadores": "compactador",
    "compactados": "compactar",
    "compactamos": "compactar",
    "compactan": "compactar",
    "compactando": "compactar",
    "compactara": "compactar",
    "compactarais": "compactar",
    "compactaran": "compactar",
    "compactaras": "compactar",
    "compactare": "compactar",
    "compactareis": "compactar",
    "compactaremos": "compactar",
    "compactaren": "compactar",
    "compactares": "compactar",
    "compactaron": "compactar",
    "compactará": "compactar",
    "compactarán": "compactar",
    "compactarás": "compactar",
    "compactaré": "compactar",
    "compactaréis": "compactar",
    "compactaría": "compactar",
    "compactaríais": "compactar",
    "compactaríamos": "compactar",
    "compactarían": "compactar",
    "compactarías": "compactar",
    "compactas": "compacto",
    "compactase": "compactar",
    "compactaseis": "compactar",
    "compactasen": "compactar",
    "compactases": "compactar",
    "compactaste": "compactar",
    "compactasteis": "compactar",
    "compacte": "compactar",
    "compactemos": "compactar",
    "compacten": "compactar",
    "compactes": "compactar",
    "compacticidades": "compacticidad",
    "compactificaciones": "compactificación",
    "compacto": "compactar",
    "compactos": "compacto",
    "compactábamos": "compactar",
    "compactáis": "compactar",
    "compactáramos": "compactar",
    "compactáremos": "compactar",
    "compactásemos": "compactar",
    "compacté": "compactar",
    "compactéis": "compactar",
    "compactó": "compactar",
    "compactómetros": "compactómetro",
    "compadece": "compadecer",
    "compadeced": "compadecer",
    "compadecemos": "compadecer",
    "compadecen": "compadecer",
    "compadeceremos": "compadecer",
    "compadecerá": "compadecer",
    "compadecerán": "compadecer",
    "compadecerás": "compadecer",
    "compadeceré": "compadecer",
    "compadeceréis": "compadecer",
    "compadecería": "compadecer",
    "compadeceríais": "compadecer",
    "compadeceríamos": "compadecer",
    "compadecerían": "compadecer",
    "compadecerías": "compadecer",
    "compadeces": "compadecer",
    "compadecida": "compadecer",
    "compadecidas": "compadecer",
    "compadecido": "compadecer",
    "compadecidos": "compadecer",
    "compadeciendo": "compadecer",
    "compadeciera": "compadecer",
    "compadecierais": "compadecer",
    "compadecieran": "compadecer",
    "compadecieras": "compadecer",
    "compadeciere": "compadecer",
    "compadeciereis": "compadecer",
    "compadecieren": "compadecer",
    "compadecieres": "compadecer",
    "compadecieron": "compadecer",
    "compadeciese": "compadecer",
    "compadecieseis": "compadecer",
    "compadeciesen": "compadecer",
    "compadecieses": "compadecer",
    "compadecimientos": "compadecimiento",
    "compadecimos": "compadecer",
    "compadeciste": "compadecer",
    "compadecisteis": "compadecer",
    "compadeciéramos": "compadecer",
    "compadeciéremos": "compadecer",
    "compadeciésemos": "compadecer",
    "compadeció": "compadecer",
    "compadecéis": "compadecer",
    "compadecí": "compadecer",
    "compadecía": "compadecer",
    "compadecíais": "compadecer",
    "compadecíamos": "compadecer",
    "compadecían": "compadecer",
    "compadecías": "compadecer",
    "compadezca": "compadecer",
    "compadezcamos": "compadecer",
    "compadezcan": "compadecer",
    "compadezcas": "compadecer",
    "compadezco": "compadecer",
    "compadezcáis": "compadecer",
    "compadra": "compadrar",
    "compadraba": "compadrar",
    "compadrabais": "compadrar",
    "compadraban": "compadrar",
    "compadrabas": "compadrar",
    "compadrad": "compadrar",
    "compadrada": "compadrar",
    "compadradas": "compadrar",
    "compadrado": "compadrar",
    "compadrados": "compadrar",
    "compadrajes": "compadraje",
    "compadramos": "compadrar",
    "compadran": "compadrar",
    "compadrando": "compadrar",
    "compadrara": "compadrar",
    "compadrarais": "compadrar",
    "compadraran": "compadrar",
    "compadraras": "compadrar",
    "compadrare": "compadrar",
    "compadrareis": "compadrar",
    "compadraremos": "compadrar",
    "compadraren": "compadrar",
    "compadrares": "compadrar",
    "compadraron": "compadrar",
    "compadrará": "compadrar",
    "compadrarán": "compadrar",
    "compadrarás": "compadrar",
    "compadraré": "compadrar",
    "compadraréis": "compadrar",
    "compadraría": "compadrar",
    "compadraríais": "compadrar",
    "compadraríamos": "compadrar",
    "compadrarían": "compadrar",
    "compadrarías": "compadrar",
    "compadras": "compadrar",
    "compadrase": "compadrar",
    "compadraseis": "compadrar",
    "compadrasen": "compadrar",
    "compadrases": "compadrar",
    "compadraste": "compadrar",
    "compadrasteis": "compadrar",
    "compadrazgos": "compadrazgo",
    "compadre": "compadrar",
    "compadrea": "compadrear",
    "compadreaba": "compadrear",
    "compadreabais": "compadrear",
    "compadreaban": "compadrear",
    "compadreabas": "compadrear",
    "compadread": "compadrear",
    "compadreada": "compadrear",
    "compadreadas": "compadrear",
    "compadreado": "compadrear",
    "compadreados": "compadrear",
    "compadreamos": "compadrear",
    "compadrean": "compadrear",
    "compadreando": "compadrear",
    "compadreara": "compadrear",
    "compadrearais": "compadrear",
    "compadrearan": "compadrear",
    "compadrearas": "compadrear",
    "compadreare": "compadrear",
    "compadreareis": "compadrear",
    "compadrearemos": "compadrear",
    "compadrearen": "compadrear",
    "compadreares": "compadrear",
    "compadrearon": "compadrear",
    "compadreará": "compadrear",
    "compadrearán": "compadrear",
    "compadrearás": "compadrear",
    "compadrearé": "compadrear",
    "compadrearéis": "compadrear",
    "compadrearía": "compadrear",
    "compadrearíais": "compadrear",
    "compadrearíamos": "compadrear",
    "compadrearían": "compadrear",
    "compadrearías": "compadrear",
    "compadreas": "compadrear",
    "compadrease": "compadrear",
    "compadreaseis": "compadrear",
    "compadreasen": "compadrear",
    "compadreases": "compadrear",
    "compadreaste": "compadrear",
    "compadreasteis": "compadrear",
    "compadree": "compadrear",
    "compadreemos": "compadrear",
    "compadreen": "compadrear",
    "compadrees": "compadrear",
    "compadremos": "compadrar",
    "compadren": "compadrar",
    "compadreo": "compadrear",
    "compadreos": "compadreo",
    "compadrerías": "compadrería",
    "compadres": "compadre",
    "compadreábamos": "compadrear",
    "compadreáis": "compadrear",
    "compadreáramos": "compadrear",
    "compadreáremos": "compadrear",
    "compadreásemos": "compadrear",
    "compadreé": "compadrear",
    "compadreéis": "compadrear",
    "compadreó": "compadrear",
    "compadro": "compadrar",
    "compadrábamos": "compadrar",
    "compadráis": "compadrar",
    "compadráramos": "compadrar",
    "compadráremos": "compadrar",
    "compadrásemos": "compadrar",
    "compadré": "compadrar",
    "compadréis": "compadrar",
    "compadró": "compadrar",
    "compagina": "compaginar",
    "compaginaba": "compaginar",
    "compaginabais": "compaginar",
    "compaginaban": "compaginar",
    "compaginabas": "compaginar",
    "compaginaciones": "compaginación",
    "compaginad": "compaginar",
    "compaginada": "compaginar",
    "compaginadas": "compaginar",
    "compaginado": "compaginar",
    "compaginadora": "compaginador",
    "compaginadoras": "compaginador",
    "compaginadores": "compaginador",
    "compaginados": "compaginar",
    "compaginamos": "compaginar",
    "compaginan": "compaginar",
    "compaginando": "compaginar",
    "compaginara": "compaginar",
    "compaginarais": "compaginar",
    "compaginaran": "compaginar",
    "compaginaras": "compaginar",
    "compaginare": "compaginar",
    "compaginareis": "compaginar",
    "compaginaremos": "compaginar",
    "compaginaren": "compaginar",
    "compaginares": "compaginar",
    "compaginaron": "compaginar",
    "compaginará": "compaginar",
    "compaginarán": "compaginar",
    "compaginarás": "compaginar",
    "compaginaré": "compaginar",
    "compaginaréis": "compaginar",
    "compaginaría": "compaginar",
    "compaginaríais": "compaginar",
    "compaginaríamos": "compaginar",
    "compaginarían": "compaginar",
    "compaginarías": "compaginar",
    "compaginas": "compaginar",
    "compaginase": "compaginar",
    "compaginaseis": "compaginar",
    "compaginasen": "compaginar",
    "compaginases": "compaginar",
    "compaginaste": "compaginar",
    "compaginasteis": "compaginar",
    "compagine": "compaginar",
    "compaginemos": "compaginar",
    "compaginen": "compaginar",
    "compagines": "compaginar",
    "compagino": "compaginar",
    "compaginábamos": "compaginar",
    "compagináis": "compaginar",
    "compagináramos": "compaginar",
    "compagináremos": "compaginar",
    "compaginásemos": "compaginar",
    "compaginé": "compaginar",
    "compaginéis": "compaginar",
    "compaginó": "compaginar",
    "compara": "comparar",
    "comparaba": "comparar",
    "comparabais": "comparar",
    "comparaban": "comparar",
    "comparabas": "comparar",
    "comparabilidades": "comparabilidad",
    "comparables": "comparable",
    "comparaciones": "comparación",
    "comparad": "comparar",
    "comparada": "comparar",
    "comparadas": "comparar",
    "comparado": "comparar",
    "comparadores": "comparador",
    "comparados": "comparar",
    "comparamos": "comparar",
    "comparan": "comparar",
    "comparando": "comparar",
    "comparara": "comparar",
    "compararais": "comparar",
    "compararan": "comparar",
    "compararas": "comparar",
    "comparare": "comparar",
    "comparareis": "comparar",
    "compararemos": "comparar",
    "compararen": "comparar",
    "comparares": "comparar",
    "compararon": "comparar",
    "comparará": "comparar",
    "compararán": "comparar",
    "compararás": "comparar",
    "compararé": "comparar",
    "compararéis": "comparar",
    "compararía": "comparar",
    "compararíais": "comparar",
    "compararíamos": "comparar",
    "compararían": "comparar",
    "compararías": "comparar",
    "comparas": "comparar",
    "comparase": "comparar",
    "comparaseis": "comparar",
    "comparasen": "comparar",
    "comparases": "comparar",
    "comparaste": "comparar",
    "comparasteis": "comparar",
    "comparatismos": "comparatismo",
    "comparatistas": "comparatista",
    "comparativa": "comparativo",
    "comparativas": "comparativo",
    "comparativos": "comparativo",
    "compare": "comparar",
    "comparece": "comparecer",
    "compareced": "comparecer",
    "comparecemos": "comparecer",
    "comparecen": "comparecer",
    "comparecencias": "comparecencia",
    "compareceremos": "comparecer",
    "comparecerá": "comparecer",
    "comparecerán": "comparecer",
    "comparecerás": "comparecer",
    "compareceré": "comparecer",
    "compareceréis": "comparecer",
    "comparecería": "comparecer",
    "compareceríais": "comparecer",
    "compareceríamos": "comparecer",
    "comparecerían": "comparecer",
    "comparecerías": "comparecer",
    "compareces": "comparecer",
    "comparecida": "comparecido",
    "comparecidas": "comparecido",
    "comparecido": "comparecer",
    "comparecidos": "comparecido",
    "compareciendo": "comparecer",
    "comparecientes": "compareciente",
    "compareciera": "comparecer",
    "comparecierais": "comparecer",
    "comparecieran": "comparecer",
    "comparecieras": "comparecer",
    "compareciere": "comparecer",
    "compareciereis": "comparecer",
    "comparecieren": "comparecer",
    "comparecieres": "comparecer",
    "comparecieron": "comparecer",
    "compareciese": "comparecer",
    "comparecieseis": "comparecer",
    "compareciesen": "comparecer",
    "comparecieses": "comparecer",
    "comparecimos": "comparecer",
    "compareciste": "comparecer",
    "comparecisteis": "comparecer",
    "compareciéramos": "comparecer",
    "compareciéremos": "comparecer",
    "compareciésemos": "comparecer",
    "compareció": "comparecer",
    "comparecéis": "comparecer",
    "comparecí": "comparecer",
    "comparecía": "comparecer",
    "comparecíais": "comparecer",
    "comparecíamos": "comparecer",
    "comparecían": "comparecer",
    "comparecías": "comparecer",
    "comparemos": "comparar",
    "comparen": "comparar",
    "compares": "comparar",
    "comparezca": "comparecer",
    "comparezcamos": "comparecer",
    "comparezcan": "comparecer",
    "comparezcas": "comparecer",
    "comparezco": "comparecer",
    "comparezcáis": "comparecer",
    "comparo": "comparar",
    "comparsas": "comparsa",
    "comparta": "compartir",
    "compartamos": "compartir",
    "compartan": "compartir",
    "compartas": "compartir",
    "comparte": "compartir",
    "comparten": "compartir",
    "compartes": "compartir",
    "compartibles": "compartible",
    "comparticiones": "compartición",
    "compartid": "compartir",
    "compartida": "compartir",
    "compartidas": "compartir",
    "compartido": "compartir",
    "compartidos": "compartir",
    "compartiendo": "compartir",
    "compartiera": "compartir",
    "compartierais": "compartir",
    "compartieran": "compartir",
    "compartieras": "compartir",
    "compartiere": "compartir",
    "compartiereis": "compartir",
    "compartieren": "compartir",
    "compartieres": "compartir",
    "compartieron": "compartir",
    "compartiese": "compartir",
    "compartieseis": "compartir",
    "compartiesen": "compartir",
    "compartieses": "compartir",
    "compartimenta": "compartimentar",
    "compartimentaba": "compartimentar",
    "compartimentabais": "compartimentar",
    "compartimentaban": "compartimentar",
    "compartimentabas": "compartimentar",
    "compartimentaciones": "compartimentación",
    "compartimentad": "compartimentar",
    "compartimentada": "compartimentar",
    "compartimentadas": "compartimentar",
    "compartimentado": "compartimentar",
    "compartimentados": "compartimentar",
    "compartimentamos": "compartimentar",
    "compartimentan": "compartimentar",
    "compartimentando": "compartimentar",
    "compartimentara": "compartimentar",
    "compartimentarais": "compartimentar",
    "compartimentaran": "compartimentar",
    "compartimentaras": "compartimentar",
    "compartimentare": "compartimentar",
    "compartimentareis": "compartimentar",
    "compartimentaremos": "compartimentar",
    "compartimentaren": "compartimentar",
    "compartimentares": "compartimentar",
    "compartimentaron": "compartimentar",
    "compartimentará": "compartimentar",
    "compartimentarán": "compartimentar",
    "compartimentarás": "compartimentar",
    "compartimentaré": "compartimentar",
    "compartimentaréis": "compartimentar",
    "compartimentaría": "compartimentar",
    "compartimentaríais": "compartimentar",
    "compartimentaríamos": "compartimentar",
    "compartimentarían": "compartimentar",
    "compartimentarías": "compartimentar",
    "compartimentas": "compartimentar",
    "compartimentase": "compartimentar",
    "compartimentaseis": "compartimentar",
    "compartimentasen": "compartimentar",
    "compartimentases": "compartimentar",
    "compartimentaste": "compartimentar",
    "compartimentasteis": "compartimentar",
    "compartimente": "compartimentar",
    "compartimentemos": "compartimentar",
    "compartimenten": "compartimentar",
    "compartimentes": "compartimentar",
    "compartimento": "compartimentar",
    "compartimentos": "compartimento",
    "compartimentábamos": "compartimentar",
    "compartimentáis": "compartimentar",
    "compartimentáramos": "compartimentar",
    "compartimentáremos": "compartimentar",
    "compartimentásemos": "compartimentar",
    "compartimenté": "compartimentar",
    "compartimentéis": "compartimentar",
    "compartimentó": "compartimentar",
    "compartimientos": "compartimiento",
    "compartimos": "compartir",
    "compartiremos": "compartir",
    "compartirá": "compartir",
    "compartirán": "compartir",
    "compartirás": "compartir",
    "compartiré": "compartir",
    "compartiréis": "compartir",
    "compartiría": "compartir",
    "compartiríais": "compartir",
    "compartiríamos": "compartir",
    "compartirían": "compartir",
    "compartirías": "compartir",
    "compartiste": "compartir",
    "compartisteis": "compartir",
    "compartiéramos": "compartir",
    "compartiéremos": "compartir",
    "compartiésemos": "compartir",
    "compartió": "compartir",
    "comparto": "compartir",
    "compartáis": "compartir",
    "compartí": "compartir",
    "compartía": "compartir",
    "compartíais": "compartir",
    "compartíamos": "compartir",
    "compartían": "compartir",
    "compartías": "compartir",
    "compartís": "compartir",
    "comparábamos": "comparar",
    "comparáis": "comparar",
    "comparáramos": "comparar",
    "comparáremos": "comparar",
    "comparásemos": "comparar",
    "comparé": "comparar",
    "comparéis": "comparar",
    "comparó": "comparar",
    "compasa": "compasar",
    "compasaba": "compasar",
    "compasabais": "compasar",
    "compasaban": "compasar",
    "compasabas": "compasar",
    "compasad": "compasar",
    "compasada": "compasar",
    "compasadas": "compasar",
    "compasado": "compasar",
    "compasados": "compasar",
    "compasamos": "compasar",
    "compasan": "compasar",
    "compasando": "compasar",
    "compasara": "compasar",
    "compasarais": "compasar",
    "compasaran": "compasar",
    "compasaras": "compasar",
    "compasare": "compasar",
    "compasareis": "compasar",
    "compasaremos": "compasar",
    "compasaren": "compasar",
    "compasares": "compasar",
    "compasaron": "compasar",
    "compasará": "compasar",
    "compasarán": "compasar",
    "compasarás": "compasar",
    "compasaré": "compasar",
    "compasaréis": "compasar",
    "compasaría": "compasar",
    "compasaríais": "compasar",
    "compasaríamos": "compasar",
    "compasarían": "compasar",
    "compasarías": "compasar",
    "compasas": "compasar",
    "compasase": "compasar",
    "compasaseis": "compasar",
    "compasasen": "compasar",
    "compasases": "compasar",
    "compasaste": "compasar",
    "compasasteis": "compasar",
    "compase": "compasar",
    "compasemos": "compasar",
    "compasen": "compasar",
    "compases": "compasar",
    "compasibles": "compasible",
    "compasillos": "compasillo",
    "compasiones": "compasión",
    "compasiva": "compasivo",
    "compasivas": "compasivo",
    "compasivos": "compasivo",
    "compaso": "compasar",
    "compasábamos": "compasar",
    "compasáis": "compasar",
    "compasáramos": "compasar",
    "compasáremos": "compasar",
    "compasásemos": "compasar",
    "compasé": "compasar",
    "compaséis": "compasar",
    "compasó": "compasar",
    "compaternidades": "compaternidad",
    "compatibilice": "compatibilizar",
    "compatibilicemos": "compatibilizar",
    "compatibilicen": "compatibilizar",
    "compatibilices": "compatibilizar",
    "compatibilicé": "compatibilizar",
    "compatibilicéis": "compatibilizar",
    "compatibilidades": "compatibilidad",
    "compatibiliza": "compatibilizar",
    "compatibilizaba": "compatibilizar",
    "compatibilizabais": "compatibilizar",
    "compatibilizaban": "compatibilizar",
    "compatibilizabas": "compatibilizar",
    "compatibilizad": "compatibilizar",
    "compatibilizada": "compatibilizar",
    "compatibilizadas": "compatibilizar",
    "compatibilizado": "compatibilizar",
    "compatibilizadora": "compatibilizador",
    "compatibilizadoras": "compatibilizador",
    "compatibilizadores": "compatibilizador",
    "compatibilizados": "compatibilizar",
    "compatibilizamos": "compatibilizar",
    "compatibilizan": "compatibilizar",
    "compatibilizando": "compatibilizar",
    "compatibilizara": "compatibilizar",
    "compatibilizarais": "compatibilizar",
    "compatibilizaran": "compatibilizar",
    "compatibilizaras": "compatibilizar",
    "compatibilizare": "compatibilizar",
    "compatibilizareis": "compatibilizar",
    "compatibilizaremos": "compatibilizar",
    "compatibilizaren": "compatibilizar",
    "compatibilizares": "compatibilizar",
    "compatibilizaron": "compatibilizar",
    "compatibilizará": "compatibilizar",
    "compatibilizarán": "compatibilizar",
    "compatibilizarás": "compatibilizar",
    "compatibilizaré": "compatibilizar",
    "compatibilizaréis": "compatibilizar",
    "compatibilizaría": "compatibilizar",
    "compatibilizaríais": "compatibilizar",
    "compatibilizaríamos": "compatibilizar",
    "compatibilizarían": "compatibilizar",
    "compatibilizarías": "compatibilizar",
    "compatibilizas": "compatibilizar",
    "compatibilizase": "compatibilizar",
    "compatibilizaseis": "compatibilizar",
    "compatibilizasen": "compatibilizar",
    "compatibilizases": "compatibilizar",
    "compatibilizaste": "compatibilizar",
    "compatibilizasteis": "compatibilizar",
    "compatibilizo": "compatibilizar",
    "compatibilizábamos": "compatibilizar",
    "compatibilizáis": "compatibilizar",
    "compatibilizáramos": "compatibilizar",
    "compatibilizáremos": "compatibilizar",
    "compatibilizásemos": "compatibilizar",
    "compatibilizó": "compatibilizar",
    "compatibles": "compatible",
    "compatricia": "compatricio",
    "compatricias": "compatricio",
    "compatricios": "compatricio",
    "compatriotas": "compatriota",
    "compañas": "compaña",
    "compañera": "compañero",
    "compañeras": "compañero",
    "compañerismos": "compañerismo",
    "compañeros": "compañero",
    "compañones": "compañón",
    "compañías": "compañía",
    "compela": "compeler",
    "compelamos": "compeler",
    "compelan": "compeler",
    "compelas": "compeler",
    "compele": "compeler",
    "compeled": "compeler",
    "compelemos": "compeler",
    "compelen": "compeler",
    "compeleremos": "compeler",
    "compelerá": "compeler",
    "compelerán": "compeler",
    "compelerás": "compeler",
    "compeleré": "compeler",
    "compeleréis": "compeler",
    "compelería": "compeler",
    "compeleríais": "compeler",
    "compeleríamos": "compeler",
    "compelerían": "compeler",
    "compelerías": "compeler",
    "compeles": "compeler",
    "compelida": "compeler",
    "compelidas": "compeler",
    "compelido": "compeler",
    "compelidos": "compeler",
    "compeliendo": "compeler",
    "compeliera": "compeler",
    "compelierais": "compeler",
    "compelieran": "compeler",
    "compelieras": "compeler",
    "compeliere": "compeler",
    "compeliereis": "compeler",
    "compelieren": "compeler",
    "compelieres": "compeler",
    "compelieron": "compeler",
    "compeliese": "compeler",
    "compelieseis": "compeler",
    "compeliesen": "compeler",
    "compelieses": "compeler",
    "compelimos": "compeler",
    "compeliste": "compeler",
    "compelisteis": "compeler",
    "compeliéramos": "compeler",
    "compeliéremos": "compeler",
    "compeliésemos": "compeler",
    "compelió": "compeler",
    "compelo": "compeler",
    "compeláis": "compeler",
    "compeléis": "compeler",
    "compelí": "compeler",
    "compelía": "compeler",
    "compelíais": "compeler",
    "compelíamos": "compeler",
    "compelían": "compeler",
    "compelías": "compeler",
    "compendia": "compendiar",
    "compendiaba": "compendiar",
    "compendiabais": "compendiar",
    "compendiaban": "compendiar",
    "compendiabas": "compendiar",
    "compendiad": "compendiar",
    "compendiada": "compendiar",
    "compendiadas": "compendiar",
    "compendiado": "compendiar",
    "compendiadora": "compendiador",
    "compendiadoras": "compendiador",
    "compendiadores": "compendiador",
    "compendiados": "compendiar",
    "compendiamos": "compendiar",
    "compendian": "compendiar",
    "compendiando": "compendiar",
    "compendiara": "compendiar",
    "compendiarais": "compendiar",
    "compendiaran": "compendiar",
    "compendiaras": "compendiar",
    "compendiare": "compendiar",
    "compendiareis": "compendiar",
    "compendiaremos": "compendiar",
    "compendiaren": "compendiar",
    "compendiares": "compendiar",
    "compendiaron": "compendiar",
    "compendiará": "compendiar",
    "compendiarán": "compendiar",
    "compendiarás": "compendiar",
    "compendiaré": "compendiar",
    "compendiaréis": "compendiar",
    "compendiaría": "compendiar",
    "compendiaríais": "compendiar",
    "compendiaríamos": "compendiar",
    "compendiarían": "compendiar",
    "compendiarías": "compendiar",
    "compendias": "compendiar",
    "compendiase": "compendiar",
    "compendiaseis": "compendiar",
    "compendiasen": "compendiar",
    "compendiases": "compendiar",
    "compendiaste": "compendiar",
    "compendiasteis": "compendiar",
    "compendice": "compendizar",
    "compendicemos": "compendizar",
    "compendicen": "compendizar",
    "compendices": "compendizar",
    "compendicé": "compendizar",
    "compendicéis": "compendizar",
    "compendie": "compendiar",
    "compendiemos": "compendiar",
    "compendien": "compendiar",
    "compendies": "compendiar",
    "compendio": "compendiar",
    "compendios": "compendio",
    "compendiosa": "compendioso",
    "compendiosas": "compendioso",
    "compendiosos": "compendioso",
    "compendistas": "compendista",
    "compendiza": "compendizar",
    "compendizaba": "compendizar",
    "compendizabais": "compendizar",
    "compendizaban": "compendizar",
    "compendizabas": "compendizar",
    "compendizad": "compendizar",
    "compendizada": "compendizar",
    "compendizadas": "compendizar",
    "compendizado": "compendizar",
    "compendizados": "compendizar",
    "compendizamos": "compendizar",
    "compendizan": "compendizar",
    "compendizando": "compendizar",
    "compendizara": "compendizar",
    "compendizarais": "compendizar",
    "compendizaran": "compendizar",
    "compendizaras": "compendizar",
    "compendizare": "compendizar",
    "compendizareis": "compendizar",
    "compendizaremos": "compendizar",
    "compendizaren": "compendizar",
    "compendizares": "compendizar",
    "compendizaron": "compendizar",
    "compendizará": "compendizar",
    "compendizarán": "compendizar",
    "compendizarás": "compendizar",
    "compendizaré": "compendizar",
    "compendizaréis": "compendizar",
    "compendizaría": "compendizar",
    "compendizaríais": "compendizar",
    "compendizaríamos": "compendizar",
    "compendizarían": "compendizar",
    "compendizarías": "compendizar",
    "compendizas": "compendizar",
    "compendizase": "compendizar",
    "compendizaseis": "compendizar",
    "compendizasen": "compendizar",
    "compendizases": "compendizar",
    "compendizaste": "compendizar",
    "compendizasteis": "compendizar",
    "compendizo": "compendizar",
    "compendizábamos": "compendizar",
    "compendizáis": "compendizar",
    "compendizáramos": "compendizar",
    "compendizáremos": "compendizar",
    "compendizásemos": "compendizar",
    "compendizó": "compendizar",
    "compendiábamos": "compendiar",
    "compendiáis": "compendiar",
    "compendiáramos": "compendiar",
    "compendiáremos": "compendiar",
    "compendiásemos": "compendiar",
    "compendié": "compendiar",
    "compendiéis": "compendiar",
    "compendió": "compendiar",
    "compenetra": "compenetrar",
    "compenetraba": "compenetrar",
    "compenetrabais": "compenetrar",
    "compenetraban": "compenetrar",
    "compenetrabas": "compenetrar",
    "compenetraciones": "compenetración",
    "compenetrad": "compenetrar",
    "compenetrada": "compenetrar",
    "compenetradas": "compenetrar",
    "compenetrado": "compenetrar",
    "compenetrados": "compenetrar",
    "compenetramos": "compenetrar",
    "compenetran": "compenetrar",
    "compenetrando": "compenetrar",
    "compenetrara": "compenetrar",
    "compenetrarais": "compenetrar",
    "compenetraran": "compenetrar",
    "compenetraras": "compenetrar",
    "compenetrare": "compenetrar",
    "compenetrareis": "compenetrar",
    "compenetraremos": "compenetrar",
    "compenetraren": "compenetrar",
    "compenetrares": "compenetrar",
    "compenetraron": "compenetrar",
    "compenetrará": "compenetrar",
    "compenetrarán": "compenetrar",
    "compenetrarás": "compenetrar",
    "compenetraré": "compenetrar",
    "compenetraréis": "compenetrar",
    "compenetraría": "compenetrar",
    "compenetraríais": "compenetrar",
    "compenetraríamos": "compenetrar",
    "compenetrarían": "compenetrar",
    "compenetrarías": "compenetrar",
    "compenetras": "compenetrar",
    "compenetrase": "compenetrar",
    "compenetraseis": "compenetrar",
    "compenetrasen": "compenetrar",
    "compenetrases": "compenetrar",
    "compenetraste": "compenetrar",
    "compenetrasteis": "compenetrar",
    "compenetre": "compenetrar",
    "compenetremos": "compenetrar",
    "compenetren": "compenetrar",
    "compenetres": "compenetrar",
    "compenetro": "compenetrar",
    "compenetrábamos": "compenetrar",
    "compenetráis": "compenetrar",
    "compenetráramos": "compenetrar",
    "compenetráremos": "compenetrar",
    "compenetrásemos": "compenetrar",
    "compenetré": "compenetrar",
    "compenetréis": "compenetrar",
    "compenetró": "compenetrar",
    "compensa": "compensar",
    "compensaba": "compensar",
    "compensabais": "compensar",
    "compensaban": "compensar",
    "compensabas": "compensar",
    "compensables": "compensable",
    "compensaciones": "compensación",
    "compensad": "compensar",
    "compensada": "compensar",
    "compensadas": "compensar",
    "compensado": "compensar",
    "compensadora": "compensador",
    "compensadoras": "compensador",
    "compensadores": "compensador",
    "compensados": "compensar",
    "compensamos": "compensar",
    "compensan": "compensar",
    "compensando": "compensar",
    "compensara": "compensar",
    "compensarais": "compensar",
    "compensaran": "compensar",
    "compensaras": "compensar",
    "compensare": "compensar",
    "compensareis": "compensar",
    "compensaremos": "compensar",
    "compensaren": "compensar",
    "compensares": "compensar",
    "compensaron": "compensar",
    "compensará": "compensar",
    "compensarán": "compensar",
    "compensarás": "compensar",
    "compensaré": "compensar",
    "compensaréis": "compensar",
    "compensaría": "compensar",
    "compensaríais": "compensar",
    "compensaríamos": "compensar",
    "compensarían": "compensar",
    "compensarías": "compensar",
    "compensas": "compensar",
    "compensase": "compensar",
    "compensaseis": "compensar",
    "compensasen": "compensar",
    "compensases": "compensar",
    "compensaste": "compensar",
    "compensasteis": "compensar",
    "compensativa": "compensativo",
    "compensativas": "compensativo",
    "compensativos": "compensativo",
    "compensatoria": "compensatorio",
    "compensatorias": "compensatorio",
    "compensatorios": "compensatorio",
    "compense": "compensar",
    "compensemos": "compensar",
    "compensen": "compensar",
    "compenses": "compensar",
    "compenso": "compensar",
    "compensábamos": "compensar",
    "compensáis": "compensar",
    "compensáramos": "compensar",
    "compensáremos": "compensar",
    "compensásemos": "compensar",
    "compensé": "compensar",
    "compenséis": "compensar",
    "compensó": "compensar",
    "competa": "competer",
    "competamos": "competer",
    "competan": "competer",
    "competas": "competer",
    "compete": "competer",
    "competed": "competer",
    "competemos": "competer",
    "competen": "competer",
    "competenciales": "competencial",
    "competencias": "competencia",
    "competentes": "competente",
    "competeremos": "competer",
    "competerá": "competer",
    "competerán": "competer",
    "competerás": "competer",
    "competeré": "competer",
    "competeréis": "competer",
    "competería": "competer",
    "competeríais": "competer",
    "competeríamos": "competer",
    "competerían": "competer",
    "competerías": "competer",
    "competes": "competer",
    "competiciones": "competición",
    "competid": "competir",
    "competida": "competir",
    "competidas": "competir",
    "competido": "competir",
    "competidora": "competidor",
    "competidoras": "competidor",
    "competidores": "competidor",
    "competidos": "competir",
    "competiendo": "competer",
    "competiera": "competer",
    "competierais": "competer",
    "competieran": "competer",
    "competieras": "competer",
    "competiere": "competer",
    "competiereis": "competer",
    "competieren": "competer",
    "competieres": "competer",
    "competieron": "competer",
    "competiese": "competer",
    "competieseis": "competer",
    "competiesen": "competer",
    "competieses": "competer",
    "competimos": "competir",
    "competiremos": "competir",
    "competirá": "competir",
    "competirán": "competir",
    "competirás": "competir",
    "competiré": "competir",
    "competiréis": "competir",
    "competiría": "competir",
    "competiríais": "competir",
    "competiríamos": "competir",
    "competirían": "competir",
    "competirías": "competir",
    "competiste": "competir",
    "competisteis": "competir",
    "competitiva": "competitivo",
    "competitivas": "competitivo",
    "competitividades": "competitividad",
    "competitivos": "competitivo",
    "competiéramos": "competer",
    "competiéremos": "competer",
    "competiésemos": "competer",
    "competió": "competer",
    "competo": "competer",
    "competáis": "competer",
    "competéis": "competer",
    "competí": "competir",
    "competía": "competir",
    "competíais": "competir",
    "competíamos": "competir",
    "competían": "competir",
    "competías": "competir",
    "competís": "competir",
    "compila": "compilar",
    "compilaba": "compilar",
    "compilabais": "compilar",
    "compilaban": "compilar",
    "compilabas": "compilar",
    "compilabilidades": "compilabilidad",
    "compilables": "compilable",
    "compilaciones": "compilación",
    "compilad": "compilar",
    "compilada": "compilar",
    "compiladas": "compilar",
    "compilado": "compilar",
    "compiladora": "compilador",
    "compiladoras": "compilador",
    "compiladores": "compilador",
    "compilados": "compilar",
    "compilamos": "compilar",
    "compilan": "compilar",
    "compilando": "compilar",
    "compilara": "compilar",
    "compilarais": "compilar",
    "compilaran": "compilar",
    "compilaras": "compilar",
    "compilare": "compilar",
    "compilareis": "compilar",
    "compilaremos": "compilar",
    "compilaren": "compilar",
    "compilares": "compilar",
    "compilaron": "compilar",
    "compilará": "compilar",
    "compilarán": "compilar",
    "compilarás": "compilar",
    "compilaré": "compilar",
    "compilaréis": "compilar",
    "compilaría": "compilar",
    "compilaríais": "compilar",
    "compilaríamos": "compilar",
    "compilarían": "compilar",
    "compilarías": "compilar",
    "compilas": "compilar",
    "compilase": "compilar",
    "compilaseis": "compilar",
    "compilasen": "compilar",
    "compilases": "compilar",
    "compilaste": "compilar",
    "compilasteis": "compilar",
    "compilatoria": "compilatorio",
    "compilatorias": "compilatorio",
    "compilatorios": "compilatorio",
    "compile": "compilar",
    "compilemos": "compilar",
    "compilen": "compilar",
    "compiles": "compilar",
    "compilo": "compilar",
    "compilábamos": "compilar",
    "compiláis": "compilar",
    "compiláramos": "compilar",
    "compiláremos": "compilar",
    "compilásemos": "compilar",
    "compilé": "compilar",
    "compiléis": "compilar",
    "compiló": "compilar",
    "compinches": "compinche",
    "compita": "competir",
    "compitamos": "competir",
    "compitan": "competir",
    "compitas": "competir",
    "compite": "competir",
    "compiten": "competir",
    "compites": "competir",
    "compitiendo": "competir",
    "compitiera": "competir",
    "compitierais": "competir",
    "compitieran": "competir",
    "compitieras": "competir",
    "compitiere": "competir",
    "compitiereis": "competir",
    "compitieren": "competir",
    "compitieres": "competir",
    "compitieron": "competir",
    "compitiese": "competir",
    "compitieseis": "competir",
    "compitiesen": "competir",
    "compitieses": "competir",
    "compitiéramos": "competir",
    "compitiéremos": "competir",
    "compitiésemos": "competir",
    "compitió": "competir",
    "compito": "competir",
    "compitáis": "competir",
    "complace": "complacer",
    "complaced": "complacer",
    "complacemos": "complacer",
    "complacen": "complacer",
    "complacencias": "complacencia",
    "complaceremos": "complacer",
    "complacerá": "complacer",
    "complacerán": "complacer",
    "complacerás": "complacer",
    "complaceré": "complacer",
    "complaceréis": "complacer",
    "complacería": "complacer",
    "complaceríais": "complacer",
    "complaceríamos": "complacer",
    "complacerían": "complacer",
    "complacerías": "complacer",
    "complaces": "complacer",
    "complacida": "complacer",
    "complacidas": "complacer",
    "complacido": "complacer",
    "complacidos": "complacer",
    "complaciendo": "complacer",
    "complacientes": "complaciente",
    "complaciera": "complacer",
    "complacierais": "complacer",
    "complacieran": "complacer",
    "complacieras": "complacer",
    "complaciere": "complacer",
    "complaciereis": "complacer",
    "complacieren": "complacer",
    "complacieres": "complacer",
    "complacieron": "complacer",
    "complaciese": "complacer",
    "complacieseis": "complacer",
    "complaciesen": "complacer",
    "complacieses": "complacer",
    "complacimientos": "complacimiento",
    "complacimos": "complacer",
    "complaciste": "complacer",
    "complacisteis": "complacer",
    "complaciéramos": "complacer",
    "complaciéremos": "complacer",
    "complaciésemos": "complacer",
    "complació": "complacer",
    "complacéis": "complacer",
    "complací": "complacer",
    "complacía": "complacer",
    "complacíais": "complacer",
    "complacíamos": "complacer",
    "complacían": "complacer",
    "complacías": "complacer",
    "complazca": "complacer",
    "complazcamos": "complacer",
    "complazcan": "complacer",
    "complazcas": "complacer",
    "complazco": "complacer",
    "complazcáis": "complacer",
    "compleciones": "compleción",
    "compleja": "complejo",
    "complejaciones": "complejación",
    "complejas": "complejo",
    "complejidades": "complejidad",
    "complejismos": "complejismo",
    "complejos": "complejo",
    "complementa": "complementar",
    "complementaba": "complementar",
    "complementabais": "complementar",
    "complementaban": "complementar",
    "complementabas": "complementar",
    "complementaciones": "complementación",
    "complementad": "complementar",
    "complementada": "complementar",
    "complementadas": "complementar",
    "complementado": "complementar",
    "complementados": "complementar",
    "complementamos": "complementar",
    "complementan": "complementar",
    "complementando": "complementar",
    "complementara": "complementar",
    "complementarais": "complementar",
    "complementaran": "complementar",
    "complementaras": "complementar",
    "complementare": "complementar",
    "complementareis": "complementar",
    "complementaremos": "complementar",
    "complementaren": "complementar",
    "complementares": "complementar",
    "complementaria": "complementario",
    "complementarias": "complementario",
    "complementariedades": "complementariedad",
    "complementarios": "complementario",
    "complementaron": "complementar",
    "complementará": "complementar",
    "complementarán": "complementar",
    "complementarás": "complementar",
    "complementaré": "complementar",
    "complementaréis": "complementar",
    "complementaría": "complementar",
    "complementaríais": "complementar",
    "complementaríamos": "complementar",
    "complementarían": "complementar",
    "complementarías": "complementar",
    "complementas": "complementar",
    "complementase": "complementar",
    "complementaseis": "complementar",
    "complementasen": "complementar",
    "complementases": "complementar",
    "complementaste": "complementar",
    "complementasteis": "complementar",
    "complemente": "complementar",
    "complementemos": "complementar",
    "complementen": "complementar",
    "complementes": "complementar",
    "complemento": "complementar",
    "complementos": "complemento",
    "complementábamos": "complementar",
    "complementáis": "complementar",
    "complementáramos": "complementar",
    "complementáremos": "complementar",
    "complementásemos": "complementar",
    "complementé": "complementar",
    "complementéis": "complementar",
    "complementó": "complementar",
    "completa": "completo",
    "completaba": "completar",
    "completabais": "completar",
    "completaban": "completar",
    "completabas": "completar",
    "completad": "completar",
    "completada": "completar",
    "completadas": "completar",
    "completado": "completar",
    "completados": "completar",
    "completamos": "completar",
    "completan": "completar",
    "completando": "completar",
    "completara": "completar",
    "completarais": "completar",
    "completaran": "completar",
    "completaras": "completar",
    "completare": "completar",
    "completareis": "completar",
    "completaremos": "completar",
    "completaren": "completar",
    "completares": "completar",
    "completaron": "completar",
    "completará": "completar",
    "completarán": "completar",
    "completarás": "completar",
    "completaré": "completar",
    "completaréis": "completar",
    "completaría": "completar",
    "completaríais": "completar",
    "completaríamos": "completar",
    "completarían": "completar",
    "completarías": "completar",
    "completas": "completo",
    "completase": "completar",
    "completaseis": "completar",
    "completasen": "completar",
    "completases": "completar",
    "completaste": "completar",
    "completasteis": "completar",
    "complete": "completar",
    "completemos": "completar",
    "completen": "completar",
    "completes": "completar",
    "completiva": "completivo",
    "completivas": "completivo",
    "completivos": "completivo",
    "completo": "completar",
    "completos": "completo",
    "completábamos": "completar",
    "completáis": "completar",
    "completáramos": "completar",
    "completáremos": "completar",
    "completásemos": "completar",
    "completé": "completar",
    "completéis": "completar",
    "completó": "completar",
    "complexa": "complexo",
    "complexas": "complexo",
    "complexidades": "complexidad",
    "complexionada": "complexionado",
    "complexionadas": "complexionado",
    "complexionados": "complexionado",
    "complexionales": "complexional",
    "complexiones": "complexión",
    "complexos": "complexo",
    "compliancias": "compliancia",
    "complica": "complicar",
    "complicaba": "complicar",
    "complicabais": "complicar",
    "complicaban": "complicar",
    "complicabas": "complicar",
    "complicaciones": "complicación",
    "complicad": "complicar",
    "complicada": "complicar",
    "complicadas": "complicar",
    "complicado": "complicar",
    "complicados": "complicar",
    "complicamos": "complicar",
    "complican": "complicar",
    "complicando": "complicar",
    "complicara": "complicar",
    "complicarais": "complicar",
    "complicaran": "complicar",
    "complicaras": "complicar",
    "complicare": "complicar",
    "complicareis": "complicar",
    "complicaremos": "complicar",
    "complicaren": "complicar",
    "complicares": "complicar",
    "complicaron": "complicar",
    "complicará": "complicar",
    "complicarán": "complicar",
    "complicarás": "complicar",
    "complicaré": "complicar",
    "complicaréis": "complicar",
    "complicaría": "complicar",
    "complicaríais": "complicar",
    "complicaríamos": "complicar",
    "complicarían": "complicar",
    "complicarías": "complicar",
    "complicas": "complicar",
    "complicase": "complicar",
    "complicaseis": "complicar",
    "complicasen": "complicar",
    "complicases": "complicar",
    "complicaste": "complicar",
    "complicasteis": "complicar",
    "complicidades": "complicidad",
    "complico": "complicar",
    "complicábamos": "complicar",
    "complicáis": "complicar",
    "complicáramos": "complicar",
    "complicáremos": "complicar",
    "complicásemos": "complicar",
    "complicó": "complicar",
    "complique": "complicar",
    "compliquemos": "complicar",
    "compliquen": "complicar",
    "compliques": "complicar",
    "compliqué": "complicar",
    "compliquéis": "complicar",
    "complota": "complotar",
    "complotaba": "complotar",
    "complotabais": "complotar",
    "complotaban": "complotar",
    "complotabas": "complotar",
    "complotad": "complotar",
    "complotada": "complotar",
    "complotadas": "complotar",
    "complotado": "complotar",
    "complotados": "complotar",
    "complotamos": "complotar",
    "complotan": "complotar",
    "complotando": "complotar",
    "complotara": "complotar",
    "complotarais": "complotar",
    "complotaran": "complotar",
    "complotaras": "complotar",
    "complotare": "complotar",
    "complotareis": "complotar",
    "complotaremos": "complotar",
    "complotaren": "complotar",
    "complotares": "complotar",
    "complotaron": "complotar",
    "complotará": "complotar",
    "complotarán": "complotar",
    "complotarás": "complotar",
    "complotaré": "complotar",
    "complotaréis": "complotar",
    "complotaría": "complotar",
    "complotaríais": "complotar",
    "complotaríamos": "complotar",
    "complotarían": "complotar",
    "complotarías": "complotar",
    "complotas": "complotar",
    "complotase": "complotar",
    "complotaseis": "complotar",
    "complotasen": "complotar",
    "complotases": "complotar",
    "complotaste": "complotar",
    "complotasteis": "complotar",
    "complote": "complotar",
    "complotemos": "complotar",
    "comploten": "complotar",
    "complotes": "complotar",
    "complotipos": "complotipo",
    "comploto": "complotar",
    "complots": "complot",
    "complotábamos": "complotar",
    "complotáis": "complotar",
    "complotáramos": "complotar",
    "complotáremos": "complotar",
    "complotásemos": "complotar",
    "comploté": "complotar",
    "complotéis": "complotar",
    "complotó": "complotar",
    "complutenses": "complutense",
    "compluvios": "compluvio",
    "compluviums": "compluvium",
    "compondremos": "componer",
    "compondrá": "componer",
    "compondrán": "componer",
    "compondrás": "componer",
    "compondré": "componer",
    "compondréis": "componer",
    "compondría": "componer",
    "compondríais": "componer",
    "compondríamos": "componer",
    "compondrían": "componer",
    "compondrías": "componer",
    "compone": "componer",
    "componed": "componer",
    "componedoras": "componedora",
    "componedores": "componedor",
    "componemos": "componer",
    "componen": "componer",
    "componenciales": "componencial",
    "componendas": "componenda",
    "componentes": "componente",
    "compones": "componer",
    "componga": "componer",
    "compongamos": "componer",
    "compongan": "componer",
    "compongas": "componer",
    "compongo": "componer",
    "compongáis": "componer",
    "componibilidades": "componibilidad",
    "componibles": "componible",
    "componiendo": "componer",
    "componéis": "componer",
    "componía": "componer",
    "componíais": "componer",
    "componíamos": "componer",
    "componían": "componer",
    "componías": "componer",
    "comporta": "comportar",
    "comportaba": "comportar",
    "comportabais": "comportar",
    "comportaban": "comportar",
    "comportabas": "comportar",
    "comportables": "comportable",
    "comportad": "comportar",
    "comportada": "comportar",
    "comportadas": "comportar",
    "comportado": "comportar",
    "comportados": "comportar",
    "comportamientos": "comportamiento",
    "comportamos": "comportar",
    "comportan": "comportar",
    "comportando": "comportar",
    "comportara": "comportar",
    "comportarais": "comportar",
    "comportaran": "comportar",
    "comportaras": "comportar",
    "comportare": "comportar",
    "comportareis": "comportar",
    "comportaremos": "comportar",
    "comportaren": "comportar",
    "comportares": "comportar",
    "comportaron": "comportar",
    "comportará": "comportar",
    "comportarán": "comportar",
    "comportarás": "comportar",
    "comportaré": "comportar",
    "comportaréis": "comportar",
    "comportaría": "comportar",
    "comportaríais": "comportar",
    "comportaríamos": "comportar",
    "comportarían": "comportar",
    "comportarías": "comportar",
    "comportas": "comportar",
    "comportase": "comportar",
    "comportaseis": "comportar",
    "comportasen": "comportar",
    "comportases": "comportar",
    "comportaste": "comportar",
    "comportasteis": "comportar",
    "comporte": "comportar",
    "comportemos": "comportar",
    "comporten": "comportar",
    "comportera": "comportero",
    "comporteras": "comportero",
    "comporteros": "comportero",
    "comporterías": "comportería",
    "comportes": "comportar",
    "comporto": "comportar",
    "comportábamos": "comportar",
    "comportáis": "comportar",
    "comportáramos": "comportar",
    "comportáremos": "comportar",
    "comportásemos": "comportar",
    "comporté": "comportar",
    "comportéis": "comportar",
    "comportó": "comportar",
    "composibilidades": "composibilidad",
    "composiciones": "composición",
    "composites": "composite",
    "compositiva": "compositivo",
    "compositivas": "compositivo",
    "compositivos": "compositivo",
    "compositora": "compositor",
    "compositoras": "compositor",
    "compositores": "compositor",
    "compositos": "composito",
    "compostabilidades": "compostabilidad",
    "compostables": "compostable",
    "compostajes": "compostaje",
    "compostelana": "compostelano",
    "compostelanas": "compostelano",
    "compostelanos": "compostelano",
    "composturas": "compostura",
    "compotas": "compota",
    "compoteras": "compotera",
    "compra": "comprar",
    "compra-venta": "compraventa",
    "compra-ventas": "compraventa",
    "compraba": "comprar",
    "comprabais": "comprar",
    "compraban": "comprar",
    "comprabas": "comprar",
    "comprables": "comprable",
    "comprad": "comprar",
    "comprada": "comprar",
    "compradas": "comprar",
    "comprado": "comprar",
    "compradora": "comprador",
    "compradoras": "comprador",
    "compradores": "comprador",
    "comprados": "comprar",
    "compramos": "comprar",
    "compran": "comprar",
    "comprando": "comprar",
    "comprara": "comprar",
    "comprarais": "comprar",
    "compraran": "comprar",
    "compraras": "comprar",
    "comprare": "comprar",
    "comprareis": "comprar",
    "compraremos": "comprar",
    "compraren": "comprar",
    "comprares": "comprar",
    "compraron": "comprar",
    "comprará": "comprar",
    "comprarán": "comprar",
    "comprarás": "comprar",
    "compraré": "comprar",
    "compraréis": "comprar",
    "compraría": "comprar",
    "compraríais": "comprar",
    "compraríamos": "comprar",
    "comprarían": "comprar",
    "comprarías": "comprar",
    "compras": "comprar",
    "comprase": "comprar",
    "compraseis": "comprar",
    "comprasen": "comprar",
    "comprases": "comprar",
    "compraste": "comprar",
    "comprasteis": "comprar",
    "compraventas": "compraventa",
    "compre": "comprar",
    "comprehensividades": "comprehensividad",
    "compremos": "comprar",
    "compren": "comprar",
    "comprenda": "comprender",
    "comprendamos": "comprender",
    "comprendan": "comprender",
    "comprendas": "comprender",
    "comprende": "comprender",
    "comprended": "comprender",
    "comprendemos": "comprender",
    "comprenden": "comprender",
    "comprenderemos": "comprender",
    "comprenderá": "comprender",
    "comprenderán": "comprender",
    "comprenderás": "comprender",
    "comprenderé": "comprender",
    "comprenderéis": "comprender",
    "comprendería": "comprender",
    "comprenderíais": "comprender",
    "comprenderíamos": "comprender",
    "comprenderían": "comprender",
    "comprenderías": "comprender",
    "comprendes": "comprender",
    "comprendida": "comprender",
    "comprendidas": "comprender",
    "comprendido": "comprender",
    "comprendidos": "comprender",
    "comprendiendo": "comprender",
    "comprendiera": "comprender",
    "comprendierais": "comprender",
    "comprendieran": "comprender",
    "comprendieras": "comprender",
    "comprendiere": "comprender",
    "comprendiereis": "comprender",
    "comprendieren": "comprender",
    "comprendieres": "comprender",
    "comprendieron": "comprender",
    "comprendiese": "comprender",
    "comprendieseis": "comprender",
    "comprendiesen": "comprender",
    "comprendieses": "comprender",
    "comprendimos": "comprender",
    "comprendiste": "comprender",
    "comprendisteis": "comprender",
    "comprendiéramos": "comprender",
    "comprendiéremos": "comprender",
    "comprendiésemos": "comprender",
    "comprendió": "comprender",
    "comprendo": "comprender",
    "comprendáis": "comprender",
    "comprendéis": "comprender",
    "comprendí": "comprender",
    "comprendía": "comprender",
    "comprendíais": "comprender",
    "comprendíamos": "comprender",
    "comprendían": "comprender",
    "comprendías": "comprender",
    "comprensibilidades": "comprensibilidad",
    "comprensibles": "comprensible",
    "comprensiones": "comprensión",
    "comprensiva": "comprensivo",
    "comprensivas": "comprensivo",
    "comprensivos": "comprensivo",
    "comprensora": "comprensor",
    "comprensoras": "comprensor",
    "comprensores": "comprensor",
    "compres": "comprar",
    "compresas": "compresa",
    "compresibilidades": "compresibilidad",
    "compresibles": "compresible",
    "compresiones": "compresión",
    "compresiva": "compresivo",
    "compresivas": "compresivo",
    "compresivos": "compresivo",
    "compresora": "compresor",
    "compresoras": "compresor",
    "compresores": "compresor",
    "comprima": "comprimir",
    "comprimamos": "comprimir",
    "compriman": "comprimir",
    "comprimaria": "comprimario",
    "comprimarias": "comprimario",
    "comprimarios": "comprimario",
    "comprimas": "comprimir",
    "comprime": "comprimir",
    "comprimen": "comprimir",
    "comprimes": "comprimir",
    "comprimibles": "comprimible",
    "comprimid": "comprimir",
    "comprimida": "comprimir",
    "comprimidas": "comprimir",
    "comprimido": "comprimir",
    "comprimidos": "comprimir",
    "comprimiendo": "comprimir",
    "comprimiera": "comprimir",
    "comprimierais": "comprimir",
    "comprimieran": "comprimir",
    "comprimieras": "comprimir",
    "comprimiere": "comprimir",
    "comprimiereis": "comprimir",
    "comprimieren": "comprimir",
    "comprimieres": "comprimir",
    "comprimieron": "comprimir",
    "comprimiese": "comprimir",
    "comprimieseis": "comprimir",
    "comprimiesen": "comprimir",
    "comprimieses": "comprimir",
    "comprimimos": "comprimir",
    "comprimiremos": "comprimir",
    "comprimirá": "comprimir",
    "comprimirán": "comprimir",
    "comprimirás": "comprimir",
    "comprimiré": "comprimir",
    "comprimiréis": "comprimir",
    "comprimiría": "comprimir",
    "comprimiríais": "comprimir",
    "comprimiríamos": "comprimir",
    "comprimirían": "comprimir",
    "comprimirías": "comprimir",
    "comprimiste": "comprimir",
    "comprimisteis": "comprimir",
    "comprimiéramos": "comprimir",
    "comprimiéremos": "comprimir",
    "comprimiésemos": "comprimir",
    "comprimió": "comprimir",
    "comprimo": "comprimir",
    "comprimáis": "comprimir",
    "comprimí": "comprimir",
    "comprimía": "comprimir",
    "comprimíais": "comprimir",
    "comprimíamos": "comprimir",
    "comprimían": "comprimir",
    "comprimías": "comprimir",
    "comprimís": "comprimir",
    "compro": "comprar",
    "comprobaba": "comprobar",
    "comprobabais": "comprobar",
    "comprobaban": "comprobar",
    "comprobabas": "comprobar",
    "comprobables": "comprobable",
    "comprobaciones": "comprobación",
    "comprobad": "comprobar",
    "comprobada": "comprobar",
    "comprobadas": "comprobar",
    "comprobado": "comprobar",
    "comprobadores": "comprobador",
    "comprobados": "comprobar",
    "comprobamos": "comprobar",
    "comprobando": "comprobar",
    "comprobantes": "comprobante",
    "comprobara": "comprobar",
    "comprobarais": "comprobar",
    "comprobaran": "comprobar",
    "comprobaras": "comprobar",
    "comprobare": "comprobar",
    "comprobareis": "comprobar",
    "comprobaremos": "comprobar",
    "comprobaren": "comprobar",
    "comprobares": "comprobar",
    "comprobaron": "comprobar",
    "comprobará": "comprobar",
    "comprobarán": "comprobar",
    "comprobarás": "comprobar",
    "comprobaré": "comprobar",
    "comprobaréis": "comprobar",
    "comprobaría": "comprobar",
    "comprobaríais": "comprobar",
    "comprobaríamos": "comprobar",
    "comprobarían": "comprobar",
    "comprobarías": "comprobar",
    "comprobase": "comprobar",
    "comprobaseis": "comprobar",
    "comprobasen": "comprobar",
    "comprobases": "comprobar",
    "comprobaste": "comprobar",
    "comprobasteis": "comprobar",
    "comprobatoria": "comprobatorio",
    "comprobatorias": "comprobatorio",
    "comprobatorios": "comprobatorio",
    "comprobemos": "comprobar",
    "comprobábamos": "comprobar",
    "comprobáis": "comprobar",
    "comprobáramos": "comprobar",
    "comprobáremos": "comprobar",
    "comprobásemos": "comprobar",
    "comprobé": "comprobar",
    "comprobéis": "comprobar",
    "comprobó": "comprobar",
    "comprometa": "comprometer",
    "comprometamos": "comprometer",
    "comprometan": "comprometer",
    "comprometas": "comprometer",
    "compromete": "comprometer",
    "comprometed": "comprometer",
    "comprometedora": "comprometedor",
    "comprometedoras": "comprometedor",
    "comprometedores": "comprometedor",
    "comprometemos": "comprometer",
    "comprometen": "comprometer",
    "comprometeremos": "comprometer",
    "comprometerá": "comprometer",
    "comprometerán": "comprometer",
    "comprometerás": "comprometer",
    "comprometeré": "comprometer",
    "comprometeréis": "comprometer",
    "comprometería": "comprometer",
    "comprometeríais": "comprometer",
    "comprometeríamos": "comprometer",
    "comprometerían": "comprometer",
    "comprometerías": "comprometer",
    "comprometes": "comprometer",
    "comprometida": "comprometer",
    "comprometidas": "comprometer",
    "comprometido": "comprometer",
    "comprometidos": "comprometer",
    "comprometiendo": "comprometer",
    "comprometiera": "comprometer",
    "comprometierais": "comprometer",
    "comprometieran": "comprometer",
    "comprometieras": "comprometer",
    "comprometiere": "comprometer",
    "comprometiereis": "comprometer",
    "comprometieren": "comprometer",
    "comprometieres": "comprometer",
    "comprometieron": "comprometer",
    "comprometiese": "comprometer",
    "comprometieseis": "comprometer",
    "comprometiesen": "comprometer",
    "comprometieses": "comprometer",
    "comprometimientos": "comprometimiento",
    "comprometimos": "comprometer",
    "comprometiste": "comprometer",
    "comprometisteis": "comprometer",
    "comprometiéramos": "comprometer",
    "comprometiéremos": "comprometer",
    "comprometiésemos": "comprometer",
    "comprometió": "comprometer",
    "comprometo": "comprometer",
    "comprometáis": "comprometer",
    "comprometéis": "comprometer",
    "comprometí": "comprometer",
    "comprometía": "comprometer",
    "comprometíais": "comprometer",
    "comprometíamos": "comprometer",
    "comprometían": "comprometer",
    "comprometías": "comprometer",
    "compromisaria": "compromisario",
    "compromisarias": "compromisario",
    "compromisarios": "compromisario",
    "compromisoria": "compromisorio",
    "compromisorias": "compromisorio",
    "compromisorios": "compromisorio",
    "compromisos": "compromiso",
    "comprueba": "comprobar",
    "comprueban": "comprobar",
    "compruebas": "comprueba",
    "compruebe": "comprobar",
    "comprueben": "comprobar",
    "compruebes": "comprobar",
    "compruebo": "comprobar",
    "comprábamos": "comprar",
    "compráis": "comprar",
    "compráramos": "comprar",
    "compráremos": "comprar",
    "comprásemos": "comprar",
    "compré": "comprar",
    "compréis": "comprar",
    "compró": "comprar",
    "compuertas": "compuerta",
    "compuesta": "componer",
    "compuestas": "componer",
    "compuesto": "componer",
    "compuestos": "compuesto",
    "compulsa": "compulsar",
    "compulsaba": "compulsar",
    "compulsabais": "compulsar",
    "compulsaban": "compulsar",
    "compulsabas": "compulsar",
    "compulsaciones": "compulsación",
    "compulsad": "compulsar",
    "compulsada": "compulsar",
    "compulsadas": "compulsar",
    "compulsado": "compulsar",
    "compulsados": "compulsar",
    "compulsamos": "compulsar",
    "compulsan": "compulsar",
    "compulsando": "compulsar",
    "compulsara": "compulsar",
    "compulsarais": "compulsar",
    "compulsaran": "compulsar",
    "compulsaras": "compulsar",
    "compulsare": "compulsar",
    "compulsareis": "compulsar",
    "compulsaremos": "compulsar",
    "compulsaren": "compulsar",
    "compulsares": "compulsar",
    "compulsaron": "compulsar",
    "compulsará": "compulsar",
    "compulsarán": "compulsar",
    "compulsarás": "compulsar",
    "compulsaré": "compulsar",
    "compulsaréis": "compulsar",
    "compulsaría": "compulsar",
    "compulsaríais": "compulsar",
    "compulsaríamos": "compulsar",
    "compulsarían": "compulsar",
    "compulsarías": "compulsar",
    "compulsas": "compulsar",
    "compulsase": "compulsar",
    "compulsaseis": "compulsar",
    "compulsasen": "compulsar",
    "compulsases": "compulsar",
    "compulsaste": "compulsar",
    "compulsasteis": "compulsar",
    "compulse": "compulsar",
    "compulsemos": "compulsar",
    "compulsen": "compulsar",
    "compulses": "compulsar",
    "compulsiones": "compulsión",
    "compulsiva": "compulsivo",
    "compulsivas": "compulsivo",
    "compulsivos": "compulsivo",
    "compulso": "compulsar",
    "compulsora": "compulsor",
    "compulsoras": "compulsor",
    "compulsores": "compulsor",
    "compulsoria": "compulsorio",
    "compulsorias": "compulsorio",
    "compulsorios": "compulsorio",
    "compulsábamos": "compulsar",
    "compulsáis": "compulsar",
    "compulsáramos": "compulsar",
    "compulsáremos": "compulsar",
    "compulsásemos": "compulsar",
    "compulsé": "compulsar",
    "compulséis": "compulsar",
    "compulsó": "compulsar",
    "compunciones": "compunción",
    "compunge": "compungir",
    "compungen": "compungir",
    "compunges": "compungir",
    "compungid": "compungir",
    "compungida": "compungir",
    "compungidas": "compungir",
    "compungido": "compungir",
    "compungidos": "compungir",
    "compungiendo": "compungir",
    "compungiera": "compungir",
    "compungierais": "compungir",
    "compungieran": "compungir",
    "compungieras": "compungir",
    "compungiere": "compungir",
    "compungiereis": "compungir",
    "compungieren": "compungir",
    "compungieres": "compungir",
    "compungieron": "compungir",
    "compungiese": "compungir",
    "compungieseis": "compungir",
    "compungiesen": "compungir",
    "compungieses": "compungir",
    "compungimientos": "compungimiento",
    "compungimos": "compungir",
    "compungiremos": "compungir",
    "compungirá": "compungir",
    "compungirán": "compungir",
    "compungirás": "compungir",
    "compungiré": "compungir",
    "compungiréis": "compungir",
    "compungiría": "compungir",
    "compungiríais": "compungir",
    "compungiríamos": "compungir",
    "compungirían": "compungir",
    "compungirías": "compungir",
    "compungiste": "compungir",
    "compungisteis": "compungir",
    "compungiva": "compungivo",
    "compungivas": "compungivo",
    "compungivos": "compungivo",
    "compungiéramos": "compungir",
    "compungiéremos": "compungir",
    "compungiésemos": "compungir",
    "compungió": "compungir",
    "compungí": "compungir",
    "compungía": "compungir",
    "compungíais": "compungir",
    "compungíamos": "compungir",
    "compungían": "compungir",
    "compungías": "compungir",
    "compungís": "compungir",
    "compunja": "compungir",
    "compunjamos": "compungir",
    "compunjan": "compungir",
    "compunjas": "compungir",
    "compunjo": "compungir",
    "compunjáis": "compungir",
    "compurgaciones": "compurgación",
    "compurgadora": "compurgador",
    "compurgadoras": "compurgador",
    "compurgadores": "compurgador",
    "compuse": "componer",
    "compusiera": "componer",
    "compusierais": "componer",
    "compusieran": "componer",
    "compusieras": "componer",
    "compusiere": "componer",
    "compusiereis": "componer",
    "compusieren": "componer",
    "compusieres": "componer",
    "compusieron": "componer",
    "compusiese": "componer",
    "compusieseis": "componer",
    "compusiesen": "componer",
    "compusieses": "componer",
    "compusimos": "componer",
    "compusiste": "componer",
    "compusisteis": "componer",
    "compusiéramos": "componer",
    "compusiéremos": "componer",
    "compusiésemos": "componer",
    "compuso": "componer",
    "computa": "computar",
    "computaba": "computar",
    "computabais": "computar",
    "computaban": "computar",
    "computabas": "computar",
    "computabilidades": "computabilidad",
    "computables": "computable",
    "computacionales": "computacional",
    "computaciones": "computación",
    "computad": "computar",
    "computada": "computar",
    "computadas": "computar",
    "computado": "computar",
    "computadora": "computador",
    "computadoras": "computador",
    "computadores": "computador",
    "computados": "computar",
    "computamos": "computar",
    "computan": "computar",
    "computando": "computar",
    "computara": "computar",
    "computarais": "computar",
    "computaran": "computar",
    "computaras": "computar",
    "computare": "computar",
    "computareis": "computar",
    "computaremos": "computar",
    "computaren": "computar",
    "computares": "computar",
    "computaron": "computar",
    "computará": "computar",
    "computarán": "computar",
    "computarás": "computar",
    "computaré": "computar",
    "computaréis": "computar",
    "computaría": "computar",
    "computaríais": "computar",
    "computaríamos": "computar",
    "computarían": "computar",
    "computarías": "computar",
    "computas": "computar",
    "computase": "computar",
    "computaseis": "computar",
    "computasen": "computar",
    "computases": "computar",
    "computaste": "computar",
    "computasteis": "computar",
    "compute": "computar",
    "computemos": "computar",
    "computen": "computar",
    "computes": "computar",
    "computistas": "computista",
    "computo": "computar",
    "computábamos": "computar",
    "computáis": "computar",
    "computáramos": "computar",
    "computáremos": "computar",
    "computásemos": "computar",
    "computé": "computar",
    "computéis": "computar",
    "computó": "computar",
    "compón": "componer",
    "comulga": "comulgar",
    "comulgaba": "comulgar",
    "comulgabais": "comulgar",
    "comulgaban": "comulgar",
    "comulgabas": "comulgar",
    "comulgad": "comulgar",
    "comulgada": "comulgar",
    "comulgadas": "comulgar",
    "comulgado": "comulgar",
    "comulgados": "comulgar",
    "comulgamos": "comulgar",
    "comulgan": "comulgar",
    "comulgando": "comulgar",
    "comulgara": "comulgar",
    "comulgarais": "comulgar",
    "comulgaran": "comulgar",
    "comulgaras": "comulgar",
    "comulgare": "comulgar",
    "comulgareis": "comulgar",
    "comulgaremos": "comulgar",
    "comulgaren": "comulgar",
    "comulgares": "comulgar",
    "comulgaron": "comulgar",
    "comulgará": "comulgar",
    "comulgarán": "comulgar",
    "comulgarás": "comulgar",
    "comulgaré": "comulgar",
    "comulgaréis": "comulgar",
    "comulgaría": "comulgar",
    "comulgaríais": "comulgar",
    "comulgaríamos": "comulgar",
    "comulgarían": "comulgar",
    "comulgarías": "comulgar",
    "comulgas": "comulgar",
    "comulgase": "comulgar",
    "comulgaseis": "comulgar",
    "comulgasen": "comulgar",
    "comulgases": "comulgar",
    "comulgaste": "comulgar",
    "comulgasteis": "comulgar",
    "comulgatorios": "comulgatorio",
    "comulgo": "comulgar",
    "comulgue": "comulgar",
    "comulguemos": "comulgar",
    "comulguen": "comulgar",
    "comulgues": "comulgar",
    "comulgué": "comulgar",
    "comulguéis": "comulgar",
    "comulgábamos": "comulgar",
    "comulgáis": "comulgar",
    "comulgáramos": "comulgar",
    "comulgáremos": "comulgar",
    "comulgásemos": "comulgar",
    "comulgó": "comulgar",
    "comunales": "comunal",
    "comunas": "comuna",
    "comunera": "comunero",
    "comuneras": "comunero",
    "comuneros": "comunero",
    "comunes": "común",
    "comunica": "comunicar",
    "comunicaba": "comunicar",
    "comunicabais": "comunicar",
    "comunicaban": "comunicar",
    "comunicabas": "comunicar",
    "comunicabilidades": "comunicabilidad",
    "comunicables": "comunicable",
    "comunicaciones": "comunicación",
    "comunicad": "comunicar",
    "comunicada": "comunicar",
    "comunicadas": "comunicar",
    "comunicado": "comunicar",
    "comunicadores": "comunicador",
    "comunicados": "comunicar",
    "comunicamos": "comunicar",
    "comunican": "comunicar",
    "comunicando": "comunicar",
    "comunicantes": "comunicante",
    "comunicara": "comunicar",
    "comunicarais": "comunicar",
    "comunicaran": "comunicar",
    "comunicaras": "comunicar",
    "comunicare": "comunicar",
    "comunicareis": "comunicar",
    "comunicaremos": "comunicar",
    "comunicaren": "comunicar",
    "comunicares": "comunicar",
    "comunicaron": "comunicar",
    "comunicará": "comunicar",
    "comunicarán": "comunicar",
    "comunicarás": "comunicar",
    "comunicaré": "comunicar",
    "comunicaréis": "comunicar",
    "comunicaría": "comunicar",
    "comunicaríais": "comunicar",
    "comunicaríamos": "comunicar",
    "comunicarían": "comunicar",
    "comunicarías": "comunicar",
    "comunicas": "comunicar",
    "comunicase": "comunicar",
    "comunicaseis": "comunicar",
    "comunicasen": "comunicar",
    "comunicases": "comunicar",
    "comunicaste": "comunicar",
    "comunicasteis": "comunicar",
    "comunicativa": "comunicativo",
    "comunicativas": "comunicativo",
    "comunicatividades": "comunicatividad",
    "comunicativos": "comunicativo",
    "comunico": "comunicar",
    "comunicologías": "comunicología",
    "comunicábamos": "comunicar",
    "comunicáis": "comunicar",
    "comunicáramos": "comunicar",
    "comunicáremos": "comunicar",
    "comunicásemos": "comunicar",
    "comunicó": "comunicar",
    "comunidades": "comunidad",
    "comuniones": "comunión",
    "comunique": "comunicar",
    "comuniquemos": "comunicar",
    "comuniquen": "comunicar",
    "comuniques": "comunicar",
    "comuniqué": "comunicar",
    "comuniquéis": "comunicar",
    "comunismos": "comunismo",
    "comunistas": "comunista",
    "comunitaria": "comunitario",
    "comunitarias": "comunitario",
    "comunitarios": "comunitario",
    "comunitarismos": "comunitarismo",
    "comunolocales": "comunolocal",
    "comáis": "comer",
    "coméis": "comer",
    "comí": "comer",
    "comía": "comer",
    "comíais": "comer",
    "comíamos": "comer",
    "comían": "comer",
    "comías": "comer",
    "conativa": "conativo",
    "conativas": "conativo",
    "conativos": "conativo",
    "conatos": "conato",
    "concadena": "concadenar",
    "concadenaba": "concadenar",
    "concadenabais": "concadenar",
    "concadenaban": "concadenar",
    "concadenabas": "concadenar",
    "concadenad": "concadenar",
    "concadenada": "concadenar",
    "concadenadas": "concadenar",
    "concadenado": "concadenar",
    "concadenados": "concadenar",
    "concadenamos": "concadenar",
    "concadenan": "concadenar",
    "concadenando": "concadenar",
    "concadenara": "concadenar",
    "concadenarais": "concadenar",
    "concadenaran": "concadenar",
    "concadenaras": "concadenar",
    "concadenare": "concadenar",
    "concadenareis": "concadenar",
    "concadenaremos": "concadenar",
    "concadenaren": "concadenar",
    "concadenares": "concadenar",
    "concadenaron": "concadenar",
    "concadenará": "concadenar",
    "concadenarán": "concadenar",
    "concadenarás": "concadenar",
    "concadenaré": "concadenar",
    "concadenaréis": "concadenar",
    "concadenaría": "concadenar",
    "concadenaríais": "concadenar",
    "concadenaríamos": "concadenar",
    "concadenarían": "concadenar",
    "concadenarías": "concadenar",
    "concadenas": "concadenar",
    "concadenase": "concadenar",
    "concadenaseis": "concadenar",
    "concadenasen": "concadenar",
    "concadenases": "concadenar",
    "concadenaste": "concadenar",
    "concadenasteis": "concadenar",
    "concadene": "concadenar",
    "concadenemos": "concadenar",
    "concadenen": "concadenar",
    "concadenes": "concadenar",
    "concadeno": "concadenar",
    "concadenábamos": "concadenar",
    "concadenáis": "concadenar",
    "concadenáramos": "concadenar",
    "concadenáremos": "concadenar",
    "concadenásemos": "concadenar",
    "concadené": "concadenar",
    "concadenéis": "concadenar",
    "concadenó": "concadenar",
    "concambios": "concambio",
    "concas": "conca",
    "concatedrales": "concatedral",
    "concatedralidades": "concatedralidad",
    "concatena": "concatenar",
    "concatenaba": "concatenar",
    "concatenabais": "concatenar",
    "concatenaban": "concatenar",
    "concatenabas": "concatenar",
    "concatenaciones": "concatenación",
    "concatenad": "concatenar",
    "concatenada": "concatenar",
    "concatenadas": "concatenar",
    "concatenado": "concatenar",
    "concatenados": "concatenar",
    "concatenamos": "concatenar",
    "concatenan": "concatenar",
    "concatenando": "concatenar",
    "concatenara": "concatenar",
    "concatenarais": "concatenar",
    "concatenaran": "concatenar",
    "concatenaras": "concatenar",
    "concatenare": "concatenar",
    "concatenareis": "concatenar",
    "concatenaremos": "concatenar",
    "concatenaren": "concatenar",
    "concatenares": "concatenar",
    "concatenaron": "concatenar",
    "concatenará": "concatenar",
    "concatenarán": "concatenar",
    "concatenarás": "concatenar",
    "concatenaré": "concatenar",
    "concatenaréis": "concatenar",
    "concatenaría": "concatenar",
    "concatenaríais": "concatenar",
    "concatenaríamos": "concatenar",
    "concatenarían": "concatenar",
    "concatenarías": "concatenar",
    "concatenas": "concatenar",
    "concatenase": "concatenar",
    "concatenaseis": "concatenar",
    "concatenasen": "concatenar",
    "concatenases": "concatenar",
    "concatenaste": "concatenar",
    "concatenasteis": "concatenar",
    "concatene": "concatenar",
    "concatenemos": "concatenar",
    "concatenen": "concatenar",
    "concatenes": "concatenar",
    "concateno": "concatenar",
    "concatenábamos": "concatenar",
    "concatenáis": "concatenar",
    "concatenáramos": "concatenar",
    "concatenáremos": "concatenar",
    "concatenásemos": "concatenar",
    "concatené": "concatenar",
    "concatenéis": "concatenar",
    "concatenó": "concatenar",
    "concatémera": "concatémero",
    "concatémeras": "concatémero",
    "concatémeros": "concatémero",
    "concausas": "concausa",
    "concavidades": "concavidad",
    "concebibles": "concebible",
    "concebid": "concebir",
    "concebida": "concebir",
    "concebidas": "concebir",
    "concebido": "concebir",
    "concebidos": "concebir",
    "concebimos": "concebir",
    "concebiremos": "concebir",
    "concebirá": "concebir",
    "concebirán": "concebir",
    "concebirás": "concebir",
    "concebiré": "concebir",
    "concebiréis": "concebir",
    "concebiría": "concebir",
    "concebiríais": "concebir",
    "concebiríamos": "concebir",
    "concebirían": "concebir",
    "concebirías": "concebir",
    "concebiste": "concebir",
    "concebisteis": "concebir",
    "concebí": "concebir",
    "concebía": "concebir",
    "concebíais": "concebir",
    "concebíamos": "concebir",
    "concebían": "concebir",
    "concebías": "concebir",
    "concebís": "concebir",
    "conceda": "conceder",
    "concedamos": "conceder",
    "concedan": "conceder",
    "concedas": "conceder",
    "concede": "conceder",
    "conceded": "conceder",
    "concedemos": "conceder",
    "conceden": "conceder",
    "concedentes": "concedente",
    "concederemos": "conceder",
    "concederá": "conceder",
    "concederán": "conceder",
    "concederás": "conceder",
    "concederé": "conceder",
    "concederéis": "conceder",
    "concedería": "conceder",
    "concederíais": "conceder",
    "concederíamos": "conceder",
    "concederían": "conceder",
    "concederías": "conceder",
    "concedes": "conceder",
    "concedida": "conceder",
    "concedidas": "conceder",
    "concedido": "conceder",
    "concedidos": "conceder",
    "concediendo": "conceder",
    "concediera": "conceder",
    "concedierais": "conceder",
    "concedieran": "conceder",
    "concedieras": "conceder",
    "concediere": "conceder",
    "concediereis": "conceder",
    "concedieren": "conceder",
    "concedieres": "conceder",
    "concedieron": "conceder",
    "concediese": "conceder",
    "concedieseis": "conceder",
    "concediesen": "conceder",
    "concedieses": "conceder",
    "concedimos": "conceder",
    "concediste": "conceder",
    "concedisteis": "conceder",
    "concediéramos": "conceder",
    "concediéremos": "conceder",
    "concediésemos": "conceder",
    "concedió": "conceder",
    "concedo": "conceder",
    "concedáis": "conceder",
    "concedéis": "conceder",
    "concedí": "conceder",
    "concedía": "conceder",
    "concedíais": "conceder",
    "concedíamos": "conceder",
    "concedían": "conceder",
    "concedías": "conceder",
    "concejala": "concejal",
    "concejalas": "concejal",
    "concejales": "concejal",
    "concejalías": "concejalía",
    "concejiles": "concejil",
    "concejos": "concejo",
    "concelebra": "concelebrar",
    "concelebraba": "concelebrar",
    "concelebrabais": "concelebrar",
    "concelebraban": "concelebrar",
    "concelebrabas": "concelebrar",
    "concelebraciones": "concelebración",
    "concelebrad": "concelebrar",
    "concelebrada": "concelebrar",
    "concelebradas": "concelebrar",
    "concelebrado": "concelebrar",
    "concelebrados": "concelebrar",
    "concelebramos": "concelebrar",
    "concelebran": "concelebrar",
    "concelebrando": "concelebrar",
    "concelebrara": "concelebrar",
    "concelebrarais": "concelebrar",
    "concelebraran": "concelebrar",
    "concelebraras": "concelebrar",
    "concelebrare": "concelebrar",
    "concelebrareis": "concelebrar",
    "concelebraremos": "concelebrar",
    "concelebraren": "concelebrar",
    "concelebrares": "concelebrar",
    "concelebraron": "concelebrar",
    "concelebrará": "concelebrar",
    "concelebrarán": "concelebrar",
    "concelebrarás": "concelebrar",
    "concelebraré": "concelebrar",
    "concelebraréis": "concelebrar",
    "concelebraría": "concelebrar",
    "concelebraríais": "concelebrar",
    "concelebraríamos": "concelebrar",
    "concelebrarían": "concelebrar",
    "concelebrarías": "concelebrar",
    "concelebras": "concelebrar",
    "concelebrase": "concelebrar",
    "concelebraseis": "concelebrar",
    "concelebrasen": "concelebrar",
    "concelebrases": "concelebrar",
    "concelebraste": "concelebrar",
    "concelebrasteis": "concelebrar",
    "concelebre": "concelebrar",
    "concelebremos": "concelebrar",
    "concelebren": "concelebrar",
    "concelebres": "concelebrar",
    "concelebro": "concelebrar",
    "concelebrábamos": "concelebrar",
    "concelebráis": "concelebrar",
    "concelebráramos": "concelebrar",
    "concelebráremos": "concelebrar",
    "concelebrásemos": "concelebrar",
    "concelebré": "concelebrar",
    "concelebréis": "concelebrar",
    "concelebró": "concelebrar",
    "concelleres": "conceller",
    "concentos": "concento",
    "concentra": "concentrar",
    "concentraba": "concentrar",
    "concentrabais": "concentrar",
    "concentraban": "concentrar",
    "concentrabas": "concentrar",
    "concentraciones": "concentración",
    "concentrad": "concentrar",
    "concentrada": "concentrar",
    "concentradas": "concentrar",
    "concentrado": "concentrar",
    "concentradora": "concentrador",
    "concentradoras": "concentrador",
    "concentradores": "concentrador",
    "concentrados": "concentrar",
    "concentramos": "concentrar",
    "concentran": "concentrar",
    "concentrando": "concentrar",
    "concentrara": "concentrar",
    "concentrarais": "concentrar",
    "concentraran": "concentrar",
    "concentraras": "concentrar",
    "concentrare": "concentrar",
    "concentrareis": "concentrar",
    "concentraremos": "concentrar",
    "concentraren": "concentrar",
    "concentrares": "concentrar",
    "concentraron": "concentrar",
    "concentrará": "concentrar",
    "concentrarán": "concentrar",
    "concentrarás": "concentrar",
    "concentraré": "concentrar",
    "concentraréis": "concentrar",
    "concentraría": "concentrar",
    "concentraríais": "concentrar",
    "concentraríamos": "concentrar",
    "concentrarían": "concentrar",
    "concentrarías": "concentrar",
    "concentras": "concentrar",
    "concentrase": "concentrar",
    "concentraseis": "concentrar",
    "concentrasen": "concentrar",
    "concentrases": "concentrar",
    "concentraste": "concentrar",
    "concentrasteis": "concentrar",
    "concentre": "concentrar",
    "concentremos": "concentrar",
    "concentren": "concentrar",
    "concentres": "concentrar",
    "concentricidades": "concentricidad",
    "concentro": "concentrar",
    "concentrábamos": "concentrar",
    "concentráis": "concentrar",
    "concentráramos": "concentrar",
    "concentráremos": "concentrar",
    "concentrásemos": "concentrar",
    "concentré": "concentrar",
    "concentréis": "concentrar",
    "concentró": "concentrar",
    "concepcionalidades": "concepcionalidad",
    "concepciones": "concepción",
    "concepcionistas": "concepcionista",
    "conceptibles": "conceptible",
    "conceptismos": "conceptismo",
    "conceptistas": "conceptista",
    "conceptiva": "conceptivo",
    "conceptivas": "conceptivo",
    "conceptivos": "conceptivo",
    "conceptos": "concepto",
    "conceptuaba": "conceptuar",
    "conceptuabais": "conceptuar",
    "conceptuaban": "conceptuar",
    "conceptuabas": "conceptuar",
    "conceptuad": "conceptuar",
    "conceptuada": "conceptuar",
    "conceptuadas": "conceptuar",
    "conceptuado": "conceptuar",
    "conceptuados": "conceptuar",
    "conceptuales": "conceptual",
    "conceptualice": "conceptualizar",
    "conceptualicemos": "conceptualizar",
    "conceptualicen": "conceptualizar",
    "conceptualices": "conceptualizar",
    "conceptualicé": "conceptualizar",
    "conceptualicéis": "conceptualizar",
    "conceptualismos": "conceptualismo",
    "conceptualistas": "conceptualista",
    "conceptualiza": "conceptualizar",
    "conceptualizaba": "conceptualizar",
    "conceptualizabais": "conceptualizar",
    "conceptualizaban": "conceptualizar",
    "conceptualizabas": "conceptualizar",
    "conceptualizaciones": "conceptualización",
    "conceptualizad": "conceptualizar",
    "conceptualizada": "conceptualizar",
    "conceptualizadas": "conceptualizar",
    "conceptualizado": "conceptualizar",
    "conceptualizados": "conceptualizar",
    "conceptualizamos": "conceptualizar",
    "conceptualizan": "conceptualizar",
    "conceptualizando": "conceptualizar",
    "conceptualizara": "conceptualizar",
    "conceptualizarais": "conceptualizar",
    "conceptualizaran": "conceptualizar",
    "conceptualizaras": "conceptualizar",
    "conceptualizare": "conceptualizar",
    "conceptualizareis": "conceptualizar",
    "conceptualizaremos": "conceptualizar",
    "conceptualizaren": "conceptualizar",
    "conceptualizares": "conceptualizar",
    "conceptualizaron": "conceptualizar",
    "conceptualizará": "conceptualizar",
    "conceptualizarán": "conceptualizar",
    "conceptualizarás": "conceptualizar",
    "conceptualizaré": "conceptualizar",
    "conceptualizaréis": "conceptualizar",
    "conceptualizaría": "conceptualizar",
    "conceptualizaríais": "conceptualizar",
    "conceptualizaríamos": "conceptualizar",
    "conceptualizarían": "conceptualizar",
    "conceptualizarías": "conceptualizar",
    "conceptualizas": "conceptualizar",
    "conceptualizase": "conceptualizar",
    "conceptualizaseis": "conceptualizar",
    "conceptualizasen": "conceptualizar",
    "conceptualizases": "conceptualizar",
    "conceptualizaste": "conceptualizar",
    "conceptualizasteis": "conceptualizar",
    "conceptualizo": "conceptualizar",
    "conceptualizábamos": "conceptualizar",
    "conceptualizáis": "conceptualizar",
    "conceptualizáramos": "conceptualizar",
    "conceptualizáremos": "conceptualizar",
    "conceptualizásemos": "conceptualizar",
    "conceptualizó": "conceptualizar",
    "conceptuamos": "conceptuar",
    "conceptuando": "conceptuar",
    "conceptuara": "conceptuar",
    "conceptuarais": "conceptuar",
    "conceptuaran": "conceptuar",
    "conceptuaras": "conceptuar",
    "conceptuare": "conceptuar",
    "conceptuareis": "conceptuar",
    "conceptuaremos": "conceptuar",
    "conceptuaren": "conceptuar",
    "conceptuares": "conceptuar",
    "conceptuaron": "conceptuar",
    "conceptuará": "conceptuar",
    "conceptuarán": "conceptuar",
    "conceptuarás": "conceptuar",
    "conceptuaré": "conceptuar",
    "conceptuaréis": "conceptuar",
    "conceptuaría": "conceptuar",
    "conceptuaríais": "conceptuar",
    "conceptuaríamos": "conceptuar",
    "conceptuarían": "conceptuar",
    "conceptuarías": "conceptuar",
    "conceptuase": "conceptuar",
    "conceptuaseis": "conceptuar",
    "conceptuasen": "conceptuar",
    "conceptuases": "conceptuar",
    "conceptuaste": "conceptuar",
    "conceptuasteis": "conceptuar",
    "conceptuemos": "conceptuar",
    "conceptuosa": "conceptuoso",
    "conceptuosas": "conceptuoso",
    "conceptuosidades": "conceptuosidad",
    "conceptuosos": "conceptuoso",
    "conceptuábamos": "conceptuar",
    "conceptuáis": "conceptuar",
    "conceptuáramos": "conceptuar",
    "conceptuáremos": "conceptuar",
    "conceptuásemos": "conceptuar",
    "conceptué": "conceptuar",
    "conceptuéis": "conceptuar",
    "conceptuó": "conceptuar",
    "conceptúa": "conceptuar",
    "conceptúan": "conceptuar",
    "conceptúas": "conceptuar",
    "conceptúe": "conceptuar",
    "conceptúen": "conceptuar",
    "conceptúes": "conceptuar",
    "conceptúo": "conceptuar",
    "concercana": "concercano",
    "concercanas": "concercano",
    "concercanos": "concercano",
    "concernamos": "concernir",
    "concernencias": "concernencia",
    "concernid": "concernir",
    "concernida": "concernir",
    "concernidas": "concernir",
    "concernido": "concernir",
    "concernidos": "concernir",
    "concerniendo": "concernir",
    "concernientes": "concerniente",
    "concerniera": "concernir",
    "concernierais": "concernir",
    "concernieran": "concernir",
    "concernieras": "concernir",
    "concerniere": "concernir",
    "concerniereis": "concernir",
    "concernieren": "concernir",
    "concernieres": "concernir",
    "concernieron": "concernir",
    "concerniese": "concernir",
    "concernieseis": "concernir",
    "concerniesen": "concernir",
    "concernieses": "concernir",
    "concernimos": "concernir",
    "concerniremos": "concernir",
    "concernirá": "concernir",
    "concernirán": "concernir",
    "concernirás": "concernir",
    "concerniré": "concernir",
    "concerniréis": "concernir",
    "concerniría": "concernir",
    "concerniríais": "concernir",
    "concerniríamos": "concernir",
    "concernirían": "concernir",
    "concernirías": "concernir",
    "concerniste": "concernir",
    "concernisteis": "concernir",
    "concerniéramos": "concernir",
    "concerniéremos": "concernir",
    "concerniésemos": "concernir",
    "concernió": "concernir",
    "concernáis": "concernir",
    "concerní": "concernir",
    "concernía": "concernir",
    "concerníais": "concernir",
    "concerníamos": "concernir",
    "concernían": "concernir",
    "concernías": "concernir",
    "concernís": "concernir",
    "concertaba": "concertar",
    "concertabais": "concertar",
    "concertaban": "concertar",
    "concertabas": "concertar",
    "concertaciones": "concertación",
    "concertad": "concertar",
    "concertada": "concertar",
    "concertadas": "concertar",
    "concertado": "concertar",
    "concertados": "concertar",
    "concertamos": "concertar",
    "concertando": "concertar",
    "concertantes": "concertante",
    "concertara": "concertar",
    "concertarais": "concertar",
    "concertaran": "concertar",
    "concertaras": "concertar",
    "concertare": "concertar",
    "concertareis": "concertar",
    "concertaremos": "concertar",
    "concertaren": "concertar",
    "concertares": "concertar",
    "concertaron": "concertar",
    "concertará": "concertar",
    "concertarán": "concertar",
    "concertarás": "concertar",
    "concertaré": "concertar",
    "concertaréis": "concertar",
    "concertaría": "concertar",
    "concertaríais": "concertar",
    "concertaríamos": "concertar",
    "concertarían": "concertar",
    "concertarías": "concertar",
    "concertase": "concertar",
    "concertaseis": "concertar",
    "concertasen": "concertar",
    "concertases": "concertar",
    "concertaste": "concertar",
    "concertasteis": "concertar",
    "concertemos": "concertar",
    "concertinas": "concertina",
    "concertinos": "concertino",
    "concertistas": "concertista",
    "concertábamos": "concertar",
    "concertáis": "concertar",
    "concertáramos": "concertar",
    "concertáremos": "concertar",
    "concertásemos": "concertar",
    "concerté": "concertar",
    "concertéis": "concertar",
    "concertó": "concertar",
    "concesionales": "concesional",
    "concesionaria": "concesionario",
    "concesionarias": "concesionario",
    "concesionarios": "concesionario",
    "concesiones": "concesión",
    "concesiva": "concesivo",
    "concesivas": "concesivo",
    "concesivos": "concesivo",
    "conchaba": "conchabar",
    "conchababa": "conchabar",
    "conchababais": "conchabar",
    "conchababan": "conchabar",
    "conchababas": "conchabar",
    "conchabad": "conchabar",
    "conchabada": "conchabar",
    "conchabadas": "conchabar",
    "conchabado": "conchabar",
    "conchabados": "conchabar",
    "conchabamientos": "conchabamiento",
    "conchabamos": "conchabar",
    "conchaban": "conchabar",
    "conchabando": "conchabar",
    "conchabanzas": "conchabanza",
    "conchabara": "conchabar",
    "conchabarais": "conchabar",
    "conchabaran": "conchabar",
    "conchabaras": "conchabar",
    "conchabare": "conchabar",
    "conchabareis": "conchabar",
    "conchabaremos": "conchabar",
    "conchabaren": "conchabar",
    "conchabares": "conchabar",
    "conchabaron": "conchabar",
    "conchabará": "conchabar",
    "conchabarán": "conchabar",
    "conchabarás": "conchabar",
    "conchabaré": "conchabar",
    "conchabaréis": "conchabar",
    "conchabaría": "conchabar",
    "conchabaríais": "conchabar",
    "conchabaríamos": "conchabar",
    "conchabarían": "conchabar",
    "conchabarías": "conchabar",
    "conchabas": "conchabar",
    "conchabase": "conchabar",
    "conchabaseis": "conchabar",
    "conchabasen": "conchabar",
    "conchabases": "conchabar",
    "conchabaste": "conchabar",
    "conchabasteis": "conchabar",
    "conchabe": "conchabar",
    "conchabemos": "conchabar",
    "conchaben": "conchabar",
    "conchabes": "conchabar",
    "conchabo": "conchabar",
    "conchabos": "conchabo",
    "conchabábamos": "conchabar",
    "conchabáis": "conchabar",
    "conchabáramos": "conchabar",
    "conchabáremos": "conchabar",
    "conchabásemos": "conchabar",
    "conchabé": "conchabar",
    "conchabéis": "conchabar",
    "conchabó": "conchabar",
    "conchas": "concha",
    "concheros": "conchero",
    "conchoides": "conchoide",
    "conchuda": "conchudo",
    "conchudas": "conchudo",
    "conchudos": "conchudo",
    "conchuelas": "conchuela",
    "conchífera": "conchífero",
    "conchíferas": "conchífero",
    "conchíferos": "conchífero",
    "conciba": "concebir",
    "concibamos": "concebir",
    "conciban": "concebir",
    "concibas": "concebir",
    "concibe": "concebir",
    "conciben": "concebir",
    "concibes": "concebir",
    "concibiendo": "concebir",
    "concibiera": "concebir",
    "concibierais": "concebir",
    "concibieran": "concebir",
    "concibieras": "concebir",
    "concibiere": "concebir",
    "concibiereis": "concebir",
    "concibieren": "concebir",
    "concibieres": "concebir",
    "concibieron": "concebir",
    "concibiese": "concebir",
    "concibieseis": "concebir",
    "concibiesen": "concebir",
    "concibieses": "concebir",
    "concibiéramos": "concebir",
    "concibiéremos": "concebir",
    "concibiésemos": "concebir",
    "concibió": "concebir",
    "concibo": "concebir",
    "concibáis": "concebir",
    "conciencia": "concienciar",
    "concienciaba": "concienciar",
    "concienciabais": "concienciar",
    "concienciaban": "concienciar",
    "concienciabas": "concienciar",
    "concienciaciones": "concienciación",
    "concienciad": "concienciar",
    "concienciada": "concienciar",
    "concienciadas": "concienciar",
    "concienciado": "concienciar",
    "concienciadora": "concienciador",
    "concienciadoras": "concienciador",
    "concienciadores": "concienciador",
    "concienciados": "concienciar",
    "concienciamientos": "concienciamiento",
    "concienciamos": "concienciar",
    "conciencian": "concienciar",
    "concienciando": "concienciar",
    "concienciara": "concienciar",
    "concienciarais": "concienciar",
    "concienciaran": "concienciar",
    "concienciaras": "concienciar",
    "concienciare": "concienciar",
    "concienciareis": "concienciar",
    "concienciaremos": "concienciar",
    "concienciaren": "concienciar",
    "concienciares": "concienciar",
    "concienciaron": "concienciar",
    "concienciará": "concienciar",
    "concienciarán": "concienciar",
    "concienciarás": "concienciar",
    "concienciaré": "concienciar",
    "concienciaréis": "concienciar",
    "concienciaría": "concienciar",
    "concienciaríais": "concienciar",
    "concienciaríamos": "concienciar",
    "concienciarían": "concienciar",
    "concienciarías": "concienciar",
    "conciencias": "concienciar",
    "concienciase": "concienciar",
    "concienciaseis": "concienciar",
    "concienciasen": "concienciar",
    "concienciases": "concienciar",
    "concienciaste": "concienciar",
    "concienciasteis": "concienciar",
    "conciencie": "concienciar",
    "concienciemos": "concienciar",
    "conciencien": "concienciar",
    "conciencies": "concienciar",
    "conciencio": "concienciar",
    "concienciábamos": "concienciar",
    "concienciáis": "concienciar",
    "concienciáramos": "concienciar",
    "concienciáremos": "concienciar",
    "concienciásemos": "concienciar",
    "conciencié": "concienciar",
    "concienciéis": "concienciar",
    "concienció": "concienciar",
    "concienzuda": "concienzudo",
    "concienzudas": "concienzudo",
    "concienzudos": "concienzudo",
    "concierna": "concernir",
    "conciernan": "concernir",
    "conciernas": "concernir",
    "concierne": "concernir",
    "conciernen": "concernir",
    "conciernes": "concernir",
    "concierno": "concernir",
    "concierta": "concertar",
    "conciertan": "concertar",
    "conciertas": "concertar",
    "concierte": "concertar",
    "concierten": "concertar",
    "conciertes": "concertar",
    "concierto": "concertar",
    "conciertos": "concierto",
    "concilia": "conciliar",
    "conciliaba": "conciliar",
    "conciliabais": "conciliar",
    "conciliaban": "conciliar",
    "conciliabas": "conciliar",
    "conciliables": "conciliable",
    "conciliaciones": "conciliación",
    "conciliad": "conciliar",
    "conciliada": "conciliar",
    "conciliadas": "conciliar",
    "conciliado": "conciliar",
    "conciliadora": "conciliador",
    "conciliadoras": "conciliador",
    "conciliadores": "conciliador",
    "conciliados": "conciliar",
    "conciliamos": "conciliar",
    "concilian": "conciliar",
    "conciliando": "conciliar",
    "conciliara": "conciliar",
    "conciliarais": "conciliar",
    "conciliaran": "conciliar",
    "conciliaras": "conciliar",
    "conciliare": "conciliar",
    "conciliareis": "conciliar",
    "conciliaremos": "conciliar",
    "conciliaren": "conciliar",
    "conciliares": "conciliar",
    "conciliarismos": "conciliarismo",
    "conciliaristas": "conciliarista",
    "conciliaron": "conciliar",
    "conciliará": "conciliar",
    "conciliarán": "conciliar",
    "conciliarás": "conciliar",
    "conciliaré": "conciliar",
    "conciliaréis": "conciliar",
    "conciliaría": "conciliar",
    "conciliaríais": "conciliar",
    "conciliaríamos": "conciliar",
    "conciliarían": "conciliar",
    "conciliarías": "conciliar",
    "concilias": "conciliar",
    "conciliase": "conciliar",
    "conciliaseis": "conciliar",
    "conciliasen": "conciliar",
    "conciliases": "conciliar",
    "conciliaste": "conciliar",
    "conciliasteis": "conciliar",
    "conciliativa": "conciliativo",
    "conciliativas": "conciliativo",
    "conciliativos": "conciliativo",
    "conciliatoria": "conciliatorio",
    "conciliatorias": "conciliatorio",
    "conciliatorios": "conciliatorio",
    "concilie": "conciliar",
    "conciliemos": "conciliar",
    "concilien": "conciliar",
    "concilies": "conciliar",
    "concilio": "conciliar",
    "concilios": "concilio",
    "conciliábamos": "conciliar",
    "conciliábulos": "conciliábulo",
    "conciliáis": "conciliar",
    "conciliáramos": "conciliar",
    "conciliáremos": "conciliar",
    "conciliásemos": "conciliar",
    "concilié": "conciliar",
    "conciliéis": "conciliar",
    "concilió": "conciliar",
    "concina": "concino",
    "concinas": "concino",
    "concinidades": "concinidad",
    "concinos": "concino",
    "concisa": "conciso",
    "concisas": "conciso",
    "concisiones": "concisión",
    "concisos": "conciso",
    "concita": "concitar",
    "concitaba": "concitar",
    "concitabais": "concitar",
    "concitaban": "concitar",
    "concitabas": "concitar",
    "concitaciones": "concitación",
    "concitad": "concitar",
    "concitada": "concitar",
    "concitadas": "concitar",
    "concitado": "concitar",
    "concitados": "concitar",
    "concitamos": "concitar",
    "concitan": "concitar",
    "concitando": "concitar",
    "concitara": "concitar",
    "concitarais": "concitar",
    "concitaran": "concitar",
    "concitaras": "concitar",
    "concitare": "concitar",
    "concitareis": "concitar",
    "concitaremos": "concitar",
    "concitaren": "concitar",
    "concitares": "concitar",
    "concitaron": "concitar",
    "concitará": "concitar",
    "concitarán": "concitar",
    "concitarás": "concitar",
    "concitaré": "concitar",
    "concitaréis": "concitar",
    "concitaría": "concitar",
    "concitaríais": "concitar",
    "concitaríamos": "concitar",
    "concitarían": "concitar",
    "concitarías": "concitar",
    "concitas": "concitar",
    "concitase": "concitar",
    "concitaseis": "concitar",
    "concitasen": "concitar",
    "concitases": "concitar",
    "concitaste": "concitar",
    "concitasteis": "concitar",
    "concitativa": "concitativo",
    "concitativas": "concitativo",
    "concitativos": "concitativo",
    "concite": "concitar",
    "concitemos": "concitar",
    "conciten": "concitar",
    "concites": "concitar",
    "concito": "concitar",
    "concitábamos": "concitar",
    "concitáis": "concitar",
    "concitáramos": "concitar",
    "concitáremos": "concitar",
    "concitásemos": "concitar",
    "concité": "concitar",
    "concitéis": "concitar",
    "concitó": "concitar",
    "conciudadana": "conciudadano",
    "conciudadanas": "conciudadano",
    "conciudadanos": "conciudadano",
    "conciudadanías": "conciudadanía",
    "conclaves": "conclave",
    "conclavistas": "conclavista",
    "concluid": "concluir",
    "concluida": "concluir",
    "concluidas": "concluir",
    "concluido": "concluir",
    "concluidos": "concluir",
    "concluimos": "concluir",
    "concluiremos": "concluir",
    "concluirá": "concluir",
    "concluirán": "concluir",
    "concluirás": "concluir",
    "concluiré": "concluir",
    "concluiréis": "concluir",
    "concluiría": "concluir",
    "concluiríais": "concluir",
    "concluiríamos": "concluir",
    "concluirían": "concluir",
    "concluirías": "concluir",
    "concluiste": "concluir",
    "concluisteis": "concluir",
    "conclusa": "concluso",
    "conclusas": "concluso",
    "conclusiones": "conclusión",
    "conclusiva": "conclusivo",
    "conclusivas": "conclusivo",
    "conclusivos": "conclusivo",
    "conclusos": "concluso",
    "concluya": "concluir",
    "concluyamos": "concluir",
    "concluyan": "concluir",
    "concluyas": "concluir",
    "concluye": "concluir",
    "concluyen": "concluir",
    "concluyendo": "concluir",
    "concluyentes": "concluyente",
    "concluyera": "concluir",
    "concluyerais": "concluir",
    "concluyeran": "concluir",
    "concluyeras": "concluir",
    "concluyere": "concluir",
    "concluyereis": "concluir",
    "concluyeren": "concluir",
    "concluyeres": "concluir",
    "concluyeron": "concluir",
    "concluyes": "concluir",
    "concluyese": "concluir",
    "concluyeseis": "concluir",
    "concluyesen": "concluir",
    "concluyeses": "concluir",
    "concluyo": "concluir",
    "concluyáis": "concluir",
    "concluyéramos": "concluir",
    "concluyéremos": "concluir",
    "concluyésemos": "concluir",
    "concluyó": "concluir",
    "concluí": "concluir",
    "concluía": "concluir",
    "concluíais": "concluir",
    "concluíamos": "concluir",
    "concluían": "concluir",
    "concluías": "concluir",
    "concluís": "concluir",
    "concoidales": "concoidal",
    "concoidea": "concoideo",
    "concoideas": "concoideo",
    "concoideos": "concoideo",
    "concoides": "concoide",
    "concoma": "concomer",
    "concomamos": "concomer",
    "concoman": "concomer",
    "concomas": "concomer",
    "concome": "concomer",
    "concomed": "concomer",
    "concomemos": "concomer",
    "concomen": "concomer",
    "concomeremos": "concomer",
    "concomerá": "concomer",
    "concomerán": "concomer",
    "concomerás": "concomer",
    "concomeré": "concomer",
    "concomeréis": "concomer",
    "concomería": "concomer",
    "concomeríais": "concomer",
    "concomeríamos": "concomer",
    "concomerían": "concomer",
    "concomerías": "concomer",
    "concomes": "concomer",
    "concomida": "concomer",
    "concomidas": "concomer",
    "concomido": "concomer",
    "concomidos": "concomer",
    "concomiendo": "concomer",
    "concomiera": "concomer",
    "concomierais": "concomer",
    "concomieran": "concomer",
    "concomieras": "concomer",
    "concomiere": "concomer",
    "concomiereis": "concomer",
    "concomieren": "concomer",
    "concomieres": "concomer",
    "concomieron": "concomer",
    "concomiese": "concomer",
    "concomieseis": "concomer",
    "concomiesen": "concomer",
    "concomieses": "concomer",
    "concomimientos": "concomimiento",
    "concomimos": "concomer",
    "concomios": "concomio",
    "concomiste": "concomer",
    "concomisteis": "concomer",
    "concomita": "concomitar",
    "concomitaba": "concomitar",
    "concomitabais": "concomitar",
    "concomitaban": "concomitar",
    "concomitabas": "concomitar",
    "concomitad": "concomitar",
    "concomitada": "concomitar",
    "concomitadas": "concomitar",
    "concomitado": "concomitar",
    "concomitados": "concomitar",
    "concomitamos": "concomitar",
    "concomitan": "concomitar",
    "concomitancias": "concomitancia",
    "concomitando": "concomitar",
    "concomitantes": "concomitante",
    "concomitara": "concomitar",
    "concomitarais": "concomitar",
    "concomitaran": "concomitar",
    "concomitaras": "concomitar",
    "concomitare": "concomitar",
    "concomitareis": "concomitar",
    "concomitaremos": "concomitar",
    "concomitaren": "concomitar",
    "concomitares": "concomitar",
    "concomitaron": "concomitar",
    "concomitará": "concomitar",
    "concomitarán": "concomitar",
    "concomitarás": "concomitar",
    "concomitaré": "concomitar",
    "concomitaréis": "concomitar",
    "concomitaría": "concomitar",
    "concomitaríais": "concomitar",
    "concomitaríamos": "concomitar",
    "concomitarían": "concomitar",
    "concomitarías": "concomitar",
    "concomitas": "concomitar",
    "concomitase": "concomitar",
    "concomitaseis": "concomitar",
    "concomitasen": "concomitar",
    "concomitases": "concomitar",
    "concomitaste": "concomitar",
    "concomitasteis": "concomitar",
    "concomite": "concomitar",
    "concomitemos": "concomitar",
    "concomiten": "concomitar",
    "concomites": "concomitar",
    "concomito": "concomitar",
    "concomitábamos": "concomitar",
    "concomitáis": "concomitar",
    "concomitáramos": "concomitar",
    "concomitáremos": "concomitar",
    "concomitásemos": "concomitar",
    "concomité": "concomitar",
    "concomitéis": "concomitar",
    "concomitó": "concomitar",
    "concomiéramos": "concomer",
    "concomiéremos": "concomer",
    "concomiésemos": "concomer",
    "concomió": "concomer",
    "concomo": "concomer",
    "concomáis": "concomer",
    "concoméis": "concomer",
    "concomí": "concomer",
    "concomía": "concomer",
    "concomíais": "concomer",
    "concomíamos": "concomer",
    "concomían": "concomer",
    "concomías": "concomer",
    "concordaba": "concordar",
    "concordabais": "concordar",
    "concordaban": "concordar",
    "concordabas": "concordar",
    "concordaciones": "concordación",
    "concordad": "concordar",
    "concordada": "concordar",
    "concordadas": "concordar",
    "concordado": "concordar",
    "concordadora": "concordador",
    "concordadoras": "concordador",
    "concordadores": "concordador",
    "concordados": "concordar",
    "concordamos": "concordar",
    "concordancias": "concordancia",
    "concordando": "concordar",
    "concordantes": "concordante",
    "concordara": "concordar",
    "concordarais": "concordar",
    "concordaran": "concordar",
    "concordaras": "concordar",
    "concordare": "concordar",
    "concordareis": "concordar",
    "concordaremos": "concordar",
    "concordaren": "concordar",
    "concordares": "concordar",
    "concordaron": "concordar",
    "concordará": "concordar",
    "concordarán": "concordar",
    "concordarás": "concordar",
    "concordaré": "concordar",
    "concordaréis": "concordar",
    "concordaría": "concordar",
    "concordaríais": "concordar",
    "concordaríamos": "concordar",
    "concordarían": "concordar",
    "concordarías": "concordar",
    "concordase": "concordar",
    "concordaseis": "concordar",
    "concordasen": "concordar",
    "concordases": "concordar",
    "concordaste": "concordar",
    "concordasteis": "concordar",
    "concordataria": "concordatario",
    "concordatarias": "concordatario",
    "concordatarios": "concordatario",
    "concordativa": "concordativo",
    "concordativas": "concordativo",
    "concordativos": "concordativo",
    "concordatos": "concordato",
    "concordemos": "concordar",
    "concordes": "concorde",
    "concordias": "concordia",
    "concordábamos": "concordar",
    "concordáis": "concordar",
    "concordáramos": "concordar",
    "concordáremos": "concordar",
    "concordásemos": "concordar",
    "concordé": "concordar",
    "concordéis": "concordar",
    "concordó": "concordar",
    "concorpórea": "concorpóreo",
    "concorpóreas": "concorpóreo",
    "concorpóreos": "concorpóreo",
    "concreada": "concreado",
    "concreadas": "concreado",
    "concreados": "concreado",
    "concrecionamientos": "concrecionamiento",
    "concreciones": "concreción",
    "concrescencias": "concrescencia",
    "concrescentes": "concrescente",
    "concreta": "concreto",
    "concretaba": "concretar",
    "concretabais": "concretar",
    "concretaban": "concretar",
    "concretabas": "concretar",
    "concretad": "concretar",
    "concretada": "concretar",
    "concretadas": "concretar",
    "concretado": "concretar",
    "concretados": "concretar",
    "concretamos": "concretar",
    "concretan": "concretar",
    "concretando": "concretar",
    "concretara": "concretar",
    "concretarais": "concretar",
    "concretaran": "concretar",
    "concretaras": "concretar",
    "concretare": "concretar",
    "concretareis": "concretar",
    "concretaremos": "concretar",
    "concretaren": "concretar",
    "concretares": "concretar",
    "concretaron": "concretar",
    "concretará": "concretar",
    "concretarán": "concretar",
    "concretarás": "concretar",
    "concretaré": "concretar",
    "concretaréis": "concretar",
    "concretaría": "concretar",
    "concretaríais": "concretar",
    "concretaríamos": "concretar",
    "concretarían": "concretar",
    "concretarías": "concretar",
    "concretas": "concreto",
    "concretase": "concretar",
    "concretaseis": "concretar",
    "concretasen": "concretar",
    "concretases": "concretar",
    "concretaste": "concretar",
    "concretasteis": "concretar",
    "concrete": "concretar",
    "concretemos": "concretar",
    "concreten": "concretar",
    "concretes": "concretar",
    "concreto": "concretar",
    "concretos": "concreto",
    "concretábamos": "concretar",
    "concretáis": "concretar",
    "concretáramos": "concretar",
    "concretáremos": "concretar",
    "concretásemos": "concretar",
    "concreté": "concretar",
    "concretéis": "concretar",
    "concretó": "concretar",
    "concubinas": "concubina",
    "concubinatos": "concubinato",
    "concuerda": "concordar",
    "concuerdan": "concordar",
    "concuerdas": "concordar",
    "concuerde": "concordar",
    "concuerden": "concordar",
    "concuerdes": "concordar",
    "concuerdo": "concordar",
    "conculca": "conculcar",
    "conculcaba": "conculcar",
    "conculcabais": "conculcar",
    "conculcaban": "conculcar",
    "conculcabas": "conculcar",
    "conculcaciones": "conculcación",
    "conculcad": "conculcar",
    "conculcada": "conculcar",
    "conculcadas": "conculcar",
    "conculcado": "conculcar",
    "conculcadora": "conculcador",
    "conculcadoras": "conculcador",
    "conculcadores": "conculcador",
    "conculcados": "conculcar",
    "conculcamos": "conculcar",
    "conculcan": "conculcar",
    "conculcando": "conculcar",
    "conculcara": "conculcar",
    "conculcarais": "conculcar",
    "conculcaran": "conculcar",
    "conculcaras": "conculcar",
    "conculcare": "conculcar",
    "conculcareis": "conculcar",
    "conculcaremos": "conculcar",
    "conculcaren": "conculcar",
    "conculcares": "conculcar",
    "conculcaron": "conculcar",
    "conculcará": "conculcar",
    "conculcarán": "conculcar",
    "conculcarás": "conculcar",
    "conculcaré": "conculcar",
    "conculcaréis": "conculcar",
    "conculcaría": "conculcar",
    "conculcaríais": "conculcar",
    "conculcaríamos": "conculcar",
    "conculcarían": "conculcar",
    "conculcarías": "conculcar",
    "conculcas": "conculcar",
    "conculcase": "conculcar",
    "conculcaseis": "conculcar",
    "conculcasen": "conculcar",
    "conculcases": "conculcar",
    "conculcaste": "conculcar",
    "conculcasteis": "conculcar",
    "conculco": "conculcar",
    "conculcábamos": "conculcar",
    "conculcáis": "conculcar",
    "conculcáramos": "conculcar",
    "conculcáremos": "conculcar",
    "conculcásemos": "conculcar",
    "conculcó": "conculcar",
    "conculque": "conculcar",
    "conculquemos": "conculcar",
    "conculquen": "conculcar",
    "conculques": "conculcar",
    "conculqué": "conculcar",
    "conculquéis": "conculcar",
    "concupiscencias": "concupiscencia",
    "concupiscentes": "concupiscente",
    "concupiscibles": "concupiscible",
    "concurra": "concurrir",
    "concurramos": "concurrir",
    "concurran": "concurrir",
    "concurras": "concurrir",
    "concurre": "concurrir",
    "concurren": "concurrir",
    "concurrencias": "concurrencia",
    "concurrentes": "concurrente",
    "concurres": "concurrir",
    "concurrid": "concurrir",
    "concurrida": "concurrir",
    "concurridas": "concurrir",
    "concurrido": "concurrir",
    "concurridos": "concurrir",
    "concurriendo": "concurrir",
    "concurriera": "concurrir",
    "concurrierais": "concurrir",
    "concurrieran": "concurrir",
    "concurrieras": "concurrir",
    "concurriere": "concurrir",
    "concurriereis": "concurrir",
    "concurrieren": "concurrir",
    "concurrieres": "concurrir",
    "concurrieron": "concurrir",
    "concurriese": "concurrir",
    "concurrieseis": "concurrir",
    "concurriesen": "concurrir",
    "concurrieses": "concurrir",
    "concurrimos": "concurrir",
    "concurriremos": "concurrir",
    "concurrirá": "concurrir",
    "concurrirán": "concurrir",
    "concurrirás": "concurrir",
    "concurriré": "concurrir",
    "concurriréis": "concurrir",
    "concurriría": "concurrir",
    "concurriríais": "concurrir",
    "concurriríamos": "concurrir",
    "concurrirían": "concurrir",
    "concurrirías": "concurrir",
    "concurriste": "concurrir",
    "concurristeis": "concurrir",
    "concurriéramos": "concurrir",
    "concurriéremos": "concurrir",
    "concurriésemos": "concurrir",
    "concurrió": "concurrir",
    "concurro": "concurrir",
    "concurráis": "concurrir",
    "concurrí": "concurrir",
    "concurría": "concurrir",
    "concurríais": "concurrir",
    "concurríamos": "concurrir",
    "concurrían": "concurrir",
    "concurrías": "concurrir",
    "concurrís": "concurrir",
    "concursa": "concursar",
    "concursaba": "concursar",
    "concursabais": "concursar",
    "concursaban": "concursar",
    "concursabas": "concursar",
    "concursad": "concursar",
    "concursada": "concursar",
    "concursadas": "concursar",
    "concursado": "concursar",
    "concursados": "concursar",
    "concursales": "concursal",
    "concursamos": "concursar",
    "concursan": "concursar",
    "concursando": "concursar",
    "concursantes": "concursante",
    "concursara": "concursar",
    "concursarais": "concursar",
    "concursaran": "concursar",
    "concursaras": "concursar",
    "concursare": "concursar",
    "concursareis": "concursar",
    "concursaremos": "concursar",
    "concursaren": "concursar",
    "concursares": "concursar",
    "concursaron": "concursar",
    "concursará": "concursar",
    "concursarán": "concursar",
    "concursarás": "concursar",
    "concursaré": "concursar",
    "concursaréis": "concursar",
    "concursaría": "concursar",
    "concursaríais": "concursar",
    "concursaríamos": "concursar",
    "concursarían": "concursar",
    "concursarías": "concursar",
    "concursas": "concursar",
    "concursase": "concursar",
    "concursaseis": "concursar",
    "concursasen": "concursar",
    "concursases": "concursar",
    "concursaste": "concursar",
    "concursasteis": "concursar",
    "concurse": "concursar",
    "concursemos": "concursar",
    "concursen": "concursar",
    "concurses": "concursar",
    "concurso": "concursar",
    "concursos": "concurso",
    "concursábamos": "concursar",
    "concursáis": "concursar",
    "concursáramos": "concursar",
    "concursáremos": "concursar",
    "concursásemos": "concursar",
    "concursé": "concursar",
    "concurséis": "concursar",
    "concursó": "concursar",
    "concusionaria": "concusionario",
    "concusionarias": "concusionario",
    "concusionarios": "concusionario",
    "concusiones": "concusión",
    "concuña": "concuño",
    "concuñada": "concuñado",
    "concuñadas": "concuñado",
    "concuñados": "concuñado",
    "concuñas": "concuño",
    "concuños": "concuño",
    "concéntrica": "concéntrico",
    "concéntricas": "concéntrico",
    "concéntricos": "concéntrico",
    "concúbitos": "concúbito",
    "condados": "condado",
    "condales": "condal",
    "condecid": "condecir",
    "condecimos": "condecir",
    "condeciremos": "condecir",
    "condecirá": "condecir",
    "condecirán": "condecir",
    "condecirás": "condecir",
    "condeciré": "condecir",
    "condeciréis": "condecir",
    "condeciría": "condecir",
    "condeciríais": "condecir",
    "condeciríamos": "condecir",
    "condecirían": "condecir",
    "condecirías": "condecir",
    "condecora": "condecorar",
    "condecoraba": "condecorar",
    "condecorabais": "condecorar",
    "condecoraban": "condecorar",
    "condecorabas": "condecorar",
    "condecoraciones": "condecoración",
    "condecorad": "condecorar",
    "condecorada": "condecorar",
    "condecoradas": "condecorar",
    "condecorado": "condecorar",
    "condecorados": "condecorar",
    "condecoramos": "condecorar",
    "condecoran": "condecorar",
    "condecorando": "condecorar",
    "condecorara": "condecorar",
    "condecorarais": "condecorar",
    "condecoraran": "condecorar",
    "condecoraras": "condecorar",
    "condecorare": "condecorar",
    "condecorareis": "condecorar",
    "condecoraremos": "condecorar",
    "condecoraren": "condecorar",
    "condecorares": "condecorar",
    "condecoraron": "condecorar",
    "condecorará": "condecorar",
    "condecorarán": "condecorar",
    "condecorarás": "condecorar",
    "condecoraré": "condecorar",
    "condecoraréis": "condecorar",
    "condecoraría": "condecorar",
    "condecoraríais": "condecorar",
    "condecoraríamos": "condecorar",
    "condecorarían": "condecorar",
    "condecorarías": "condecorar",
    "condecoras": "condecorar",
    "condecorase": "condecorar",
    "condecoraseis": "condecorar",
    "condecorasen": "condecorar",
    "condecorases": "condecorar",
    "condecoraste": "condecorar",
    "condecorasteis": "condecorar",
    "condecore": "condecorar",
    "condecoremos": "condecorar",
    "condecoren": "condecorar",
    "condecores": "condecorar",
    "condecoro": "condecorar",
    "condecorábamos": "condecorar",
    "condecoráis": "condecorar",
    "condecoráramos": "condecorar",
    "condecoráremos": "condecorar",
    "condecorásemos": "condecorar",
    "condecoré": "condecorar",
    "condecoréis": "condecorar",
    "condecoró": "condecorar",
    "condecía": "condecir",
    "condecíais": "condecir",
    "condecíamos": "condecir",
    "condecían": "condecir",
    "condecías": "condecir",
    "condecís": "condecir",
    "condena": "condenar",
    "condenaba": "condenar",
    "condenabais": "condenar",
    "condenaban": "condenar",
    "condenabas": "condenar",
    "condenables": "condenable",
    "condenaciones": "condenación",
    "condenad": "condenar",
    "condenada": "condenar",
    "condenadas": "condenar",
    "condenado": "condenar",
    "condenadora": "condenador",
    "condenadoras": "condenador",
    "condenadores": "condenador",
    "condenados": "condenar",
    "condenamos": "condenar",
    "condenan": "condenar",
    "condenando": "condenar",
    "condenara": "condenar",
    "condenarais": "condenar",
    "condenaran": "condenar",
    "condenaras": "condenar",
    "condenare": "condenar",
    "condenareis": "condenar",
    "condenaremos": "condenar",
    "condenaren": "condenar",
    "condenares": "condenar",
    "condenaron": "condenar",
    "condenará": "condenar",
    "condenarán": "condenar",
    "condenarás": "condenar",
    "condenaré": "condenar",
    "condenaréis": "condenar",
    "condenaría": "condenar",
    "condenaríais": "condenar",
    "condenaríamos": "condenar",
    "condenarían": "condenar",
    "condenarías": "condenar",
    "condenas": "condenar",
    "condenase": "condenar",
    "condenaseis": "condenar",
    "condenasen": "condenar",
    "condenases": "condenar",
    "condenaste": "condenar",
    "condenasteis": "condenar",
    "condenatoria": "condenatorio",
    "condenatorias": "condenatorio",
    "condenatorios": "condenatorio",
    "condene": "condenar",
    "condenemos": "condenar",
    "condenen": "condenar",
    "condenes": "condenar",
    "condeno": "condenar",
    "condensa": "condensar",
    "condensaba": "condensar",
    "condensabais": "condensar",
    "condensaban": "condensar",
    "condensabas": "condensar",
    "condensabilidades": "condensabilidad",
    "condensables": "condensable",
    "condensaciones": "condensación",
    "condensad": "condensar",
    "condensada": "condensar",
    "condensadas": "condensar",
    "condensado": "condensar",
    "condensadora": "condensador",
    "condensadoras": "condensador",
    "condensadores": "condensador",
    "condensados": "condensar",
    "condensamos": "condensar",
    "condensan": "condensar",
    "condensando": "condensar",
    "condensara": "condensar",
    "condensarais": "condensar",
    "condensaran": "condensar",
    "condensaras": "condensar",
    "condensare": "condensar",
    "condensareis": "condensar",
    "condensaremos": "condensar",
    "condensaren": "condensar",
    "condensares": "condensar",
    "condensaron": "condensar",
    "condensará": "condensar",
    "condensarán": "condensar",
    "condensarás": "condensar",
    "condensaré": "condensar",
    "condensaréis": "condensar",
    "condensaría": "condensar",
    "condensaríais": "condensar",
    "condensaríamos": "condensar",
    "condensarían": "condensar",
    "condensarías": "condensar",
    "condensas": "condensar",
    "condensase": "condensar",
    "condensaseis": "condensar",
    "condensasen": "condensar",
    "condensases": "condensar",
    "condensaste": "condensar",
    "condensasteis": "condensar",
    "condensativa": "condensativo",
    "condensativas": "condensativo",
    "condensativos": "condensativo",
    "condense": "condensar",
    "condensemos": "condensar",
    "condensen": "condensar",
    "condenses": "condensar",
    "condenso": "condensar",
    "condensábamos": "condensar",
    "condensáis": "condensar",
    "condensáramos": "condensar",
    "condensáremos": "condensar",
    "condensásemos": "condensar",
    "condensé": "condensar",
    "condenséis": "condensar",
    "condensó": "condensar",
    "condenábamos": "condenar",
    "condenáis": "condenar",
    "condenáramos": "condenar",
    "condenáremos": "condenar",
    "condenásemos": "condenar",
    "condené": "condenar",
    "condenéis": "condenar",
    "condenó": "condenar",
    "condes": "conde",
    "condesas": "condesa",
    "condescendamos": "condescender",
    "condescended": "condescender",
    "condescendemos": "condescender",
    "condescendencias": "condescendencia",
    "condescenderemos": "condescender",
    "condescenderá": "condescender",
    "condescenderán": "condescender",
    "condescenderás": "condescender",
    "condescenderé": "condescender",
    "condescenderéis": "condescender",
    "condescendería": "condescender",
    "condescenderíais": "condescender",
    "condescenderíamos": "condescender",
    "condescenderían": "condescender",
    "condescenderías": "condescender",
    "condescendida": "condescender",
    "condescendidas": "condescender",
    "condescendido": "condescender",
    "condescendidos": "condescender",
    "condescendiendo": "condescender",
    "condescendientes": "condescendiente",
    "condescendiera": "condescender",
    "condescendierais": "condescender",
    "condescendieran": "condescender",
    "condescendieras": "condescender",
    "condescendiere": "condescender",
    "condescendiereis": "condescender",
    "condescendieren": "condescender",
    "condescendieres": "condescender",
    "condescendieron": "condescender",
    "condescendiese": "condescender",
    "condescendieseis": "condescender",
    "condescendiesen": "condescender",
    "condescendieses": "condescender",
    "condescendimos": "condescender",
    "condescendiste": "condescender",
    "condescendisteis": "condescender",
    "condescendiéramos": "condescender",
    "condescendiéremos": "condescender",
    "condescendiésemos": "condescender",
    "condescendió": "condescender",
    "condescendáis": "condescender",
    "condescendéis": "condescender",
    "condescendí": "condescender",
    "condescendía": "condescender",
    "condescendíais": "condescender",
    "condescendíamos": "condescender",
    "condescendían": "condescender",
    "condescendías": "condescender",
    "condescienda": "condescender",
    "condesciendan": "condescender",
    "condesciendas": "condescender",
    "condesciende": "condescender",
    "condescienden": "condescender",
    "condesciendes": "condescender",
    "condesciendo": "condescender",
    "condestables": "condestable",
    "condestablías": "condestablía",
    "condice": "condecir",
    "condicen": "condecir",
    "condices": "condecir",
    "condicha": "condecir",
    "condichas": "condecir",
    "condicho": "condecir",
    "condichos": "condecir",
    "condiciendo": "condecir",
    "condiciona": "condicionar",
    "condicionaba": "condicionar",
    "condicionabais": "condicionar",
    "condicionaban": "condicionar",
    "condicionabas": "condicionar",
    "condicionad": "condicionar",
    "condicionada": "condicionar",
    "condicionadas": "condicionar",
    "condicionado": "condicionar",
    "condicionados": "condicionar",
    "condicionales": "condicional",
    "condicionalidades": "condicionalidad",
    "condicionamientos": "condicionamiento",
    "condicionamos": "condicionar",
    "condicionan": "condicionar",
    "condicionando": "condicionar",
    "condicionantes": "condicionante",
    "condicionara": "condicionar",
    "condicionarais": "condicionar",
    "condicionaran": "condicionar",
    "condicionaras": "condicionar",
    "condicionare": "condicionar",
    "condicionareis": "condicionar",
    "condicionaremos": "condicionar",
    "condicionaren": "condicionar",
    "condicionares": "condicionar",
    "condicionaron": "condicionar",
    "condicionará": "condicionar",
    "condicionarán": "condicionar",
    "condicionarás": "condicionar",
    "condicionaré": "condicionar",
    "condicionaréis": "condicionar",
    "condicionaría": "condicionar",
    "condicionaríais": "condicionar",
    "condicionaríamos": "condicionar",
    "condicionarían": "condicionar",
    "condicionarías": "condicionar",
    "condicionas": "condicionar",
    "condicionase": "condicionar",
    "condicionaseis": "condicionar",
    "condicionasen": "condicionar",
    "condicionases": "condicionar",
    "condicionaste": "condicionar",
    "condicionasteis": "condicionar",
    "condicione": "condicionar",
    "condicionemos": "condicionar",
    "condicionen": "condicionar",
    "condiciones": "condicionar",
    "condiciono": "condicionar",
    "condicionábamos": "condicionar",
    "condicionáis": "condicionar",
    "condicionáramos": "condicionar",
    "condicionáremos": "condicionar",
    "condicionásemos": "condicionar",
    "condicioné": "condicionar",
    "condicionéis": "condicionar",
    "condicionó": "condicionar",
    "condiga": "condecir",
    "condigamos": "condecir",
    "condigan": "condecir",
    "condigas": "condecir",
    "condigna": "condigno",
    "condignas": "condigno",
    "condignos": "condigno",
    "condigo": "condecir",
    "condigáis": "condecir",
    "condije": "condecir",
    "condijera": "condecir",
    "condijerais": "condecir",
    "condijeran": "condecir",
    "condijeras": "condecir",
    "condijere": "condecir",
    "condijereis": "condecir",
    "condijeren": "condecir",
    "condijeres": "condecir",
    "condijeron": "condecir",
    "condijese": "condecir",
    "condijeseis": "condecir",
    "condijesen": "condecir",
    "condijeses": "condecir",
    "condijimos": "condecir",
    "condijiste": "condecir",
    "condijisteis": "condecir",
    "condijo": "condecir",
    "condijéramos": "condecir",
    "condijéremos": "condecir",
    "condijésemos": "condecir",
    "condilares": "condilar",
    "condilectomías": "condilectomía",
    "condiloidea": "condiloideo",
    "condiloideas": "condiloideo",
    "condiloideos": "condiloideo",
    "condimenta": "condimentar",
    "condimentaba": "condimentar",
    "condimentabais": "condimentar",
    "condimentaban": "condimentar",
    "condimentabas": "condimentar",
    "condimentaciones": "condimentación",
    "condimentad": "condimentar",
    "condimentada": "condimentar",
    "condimentadas": "condimentar",
    "condimentado": "condimentar",
    "condimentados": "condimentar",
    "condimentamos": "condimentar",
    "condimentan": "condimentar",
    "condimentando": "condimentar",
    "condimentara": "condimentar",
    "condimentarais": "condimentar",
    "condimentaran": "condimentar",
    "condimentaras": "condimentar",
    "condimentare": "condimentar",
    "condimentareis": "condimentar",
    "condimentaremos": "condimentar",
    "condimentaren": "condimentar",
    "condimentares": "condimentar",
    "condimentaron": "condimentar",
    "condimentará": "condimentar",
    "condimentarán": "condimentar",
    "condimentarás": "condimentar",
    "condimentaré": "condimentar",
    "condimentaréis": "condimentar",
    "condimentaría": "condimentar",
    "condimentaríais": "condimentar",
    "condimentaríamos": "condimentar",
    "condimentarían": "condimentar",
    "condimentarías": "condimentar",
    "condimentas": "condimentar",
    "condimentase": "condimentar",
    "condimentaseis": "condimentar",
    "condimentasen": "condimentar",
    "condimentases": "condimentar",
    "condimentaste": "condimentar",
    "condimentasteis": "condimentar",
    "condimente": "condimentar",
    "condimentemos": "condimentar",
    "condimenten": "condimentar",
    "condimentes": "condimentar",
    "condimento": "condimentar",
    "condimentos": "condimento",
    "condimentábamos": "condimentar",
    "condimentáis": "condimentar",
    "condimentáramos": "condimentar",
    "condimentáremos": "condimentar",
    "condimentásemos": "condimentar",
    "condimenté": "condimentar",
    "condimentéis": "condimentar",
    "condimentó": "condimentar",
    "condiscípula": "condiscípulo",
    "condiscípulas": "condiscípulo",
    "condiscípulos": "condiscípulo",
    "condolamos": "condoler",
    "condoled": "condoler",
    "condolemos": "condoler",
    "condolencias": "condolencia",
    "condoleremos": "condoler",
    "condolerá": "condoler",
    "condolerán": "condoler",
    "condolerás": "condoler",
    "condoleré": "condoler",
    "condoleréis": "condoler",
    "condolería": "condoler",
    "condoleríais": "condoler",
    "condoleríamos": "condoler",
    "condolerían": "condoler",
    "condolerías": "condoler",
    "condolida": "condoler",
    "condolidas": "condoler",
    "condolido": "condoler",
    "condolidos": "condoler",
    "condoliendo": "condoler",
    "condoliera": "condoler",
    "condolierais": "condoler",
    "condolieran": "condoler",
    "condolieras": "condoler",
    "condoliere": "condoler",
    "condoliereis": "condoler",
    "condolieren": "condoler",
    "condolieres": "condoler",
    "condolieron": "condoler",
    "condoliese": "condoler",
    "condolieseis": "condoler",
    "condoliesen": "condoler",
    "condolieses": "condoler",
    "condolimos": "condoler",
    "condoliste": "condoler",
    "condolisteis": "condoler",
    "condoliéramos": "condoler",
    "condoliéremos": "condoler",
    "condoliésemos": "condoler",
    "condolió": "condoler",
    "condoláis": "condoler",
    "condoléis": "condoler",
    "condolí": "condoler",
    "condolía": "condoler",
    "condolíais": "condoler",
    "condolíamos": "condoler",
    "condolían": "condoler",
    "condolías": "condoler",
    "condominios": "condominio",
    "condominos": "condomino",
    "condona": "condonar",
    "condonaba": "condonar",
    "condonabais": "condonar",
    "condonaban": "condonar",
    "condonabas": "condonar",
    "condonaciones": "condonación",
    "condonad": "condonar",
    "condonada": "condonar",
    "condonadas": "condonar",
    "condonado": "condonar",
    "condonados": "condonar",
    "condonamos": "condonar",
    "condonan": "condonar",
    "condonando": "condonar",
    "condonara": "condonar",
    "condonarais": "condonar",
    "condonaran": "condonar",
    "condonaras": "condonar",
    "condonare": "condonar",
    "condonareis": "condonar",
    "condonaremos": "condonar",
    "condonaren": "condonar",
    "condonares": "condonar",
    "condonaron": "condonar",
    "condonará": "condonar",
    "condonarán": "condonar",
    "condonarás": "condonar",
    "condonaré": "condonar",
    "condonaréis": "condonar",
    "condonaría": "condonar",
    "condonaríais": "condonar",
    "condonaríamos": "condonar",
    "condonarían": "condonar",
    "condonarías": "condonar",
    "condonas": "condonar",
    "condonase": "condonar",
    "condonaseis": "condonar",
    "condonasen": "condonar",
    "condonases": "condonar",
    "condonaste": "condonar",
    "condonasteis": "condonar",
    "condone": "condonar",
    "condonemos": "condonar",
    "condonen": "condonar",
    "condones": "condonar",
    "condono": "condonar",
    "condonábamos": "condonar",
    "condonáis": "condonar",
    "condonáramos": "condonar",
    "condonáremos": "condonar",
    "condonásemos": "condonar",
    "condoné": "condonar",
    "condonéis": "condonar",
    "condonó": "condonar",
    "condotieros": "condotiero",
    "condrilas": "condrila",
    "condriocontos": "condrioconto",
    "condriomas": "condrioma",
    "condriomitas": "condriomita",
    "condriomitos": "condriomito",
    "condriosomales": "condriosomal",
    "condriosomas": "condriosoma",
    "condritos": "condrito",
    "condroblastos": "condroblasto",
    "condrocitos": "condrocito",
    "condroclastos": "condroclasto",
    "condroditas": "condrodita",
    "condroectodérmica": "condroectodérmico",
    "condroectodérmicas": "condroectodérmico",
    "condroectodérmicos": "condroectodérmico",
    "condroesternales": "condroesternal",
    "condroglosa": "condrogloso",
    "condroglosas": "condrogloso",
    "condroglosos": "condrogloso",
    "condrografías": "condrografía",
    "condrográfica": "condrográfico",
    "condrográficas": "condrográfico",
    "condrográficos": "condrográfico",
    "condrologías": "condrología",
    "condromas": "condroma",
    "condroplastos": "condroplasto",
    "condroíctios": "condroíctio",
    "conducciones": "conducción",
    "conduce": "conducir",
    "conducen": "conducir",
    "conducentes": "conducente",
    "conduces": "conducir",
    "conduchos": "conducho",
    "conducid": "conducir",
    "conducida": "conducir",
    "conducidas": "conducir",
    "conducido": "conducir",
    "conducidos": "conducir",
    "conduciendo": "conducir",
    "conducimos": "conducir",
    "conduciremos": "conducir",
    "conducirá": "conducir",
    "conducirán": "conducir",
    "conducirás": "conducir",
    "conduciré": "conducir",
    "conduciréis": "conducir",
    "conduciría": "conducir",
    "conduciríais": "conducir",
    "conduciríamos": "conducir",
    "conducirían": "conducir",
    "conducirías": "conducir",
    "conductancias": "conductancia",
    "conductas": "conducta",
    "conductibilidades": "conductibilidad",
    "conductimetrías": "conductimetría",
    "conductismos": "conductismo",
    "conductistas": "conductista",
    "conductiva": "conductivo",
    "conductivas": "conductivo",
    "conductividades": "conductividad",
    "conductivos": "conductivo",
    "conductométrica": "conductométrico",
    "conductométricas": "conductométrico",
    "conductométricos": "conductométrico",
    "conductora": "conductor",
    "conductoras": "conductor",
    "conductores": "conductor",
    "conductos": "conducto",
    "conducía": "conducir",
    "conducíais": "conducir",
    "conducíamos": "conducir",
    "conducían": "conducir",
    "conducías": "conducir",
    "conducís": "conducir",
    "conduela": "condoler",
    "conduelan": "condoler",
    "conduelas": "condoler",
    "conduele": "condoler",
    "conduelen": "condoler",
    "condueles": "condoler",
    "conduelo": "condoler",
    "condueña": "condueño",
    "condueñas": "condueño",
    "condueños": "condueño",
    "conduje": "conducir",
    "condujera": "conducir",
    "condujerais": "conducir",
    "condujeran": "conducir",
    "condujeras": "conducir",
    "condujere": "conducir",
    "condujereis": "conducir",
    "condujeren": "conducir",
    "condujeres": "conducir",
    "condujeron": "conducir",
    "condujese": "conducir",
    "condujeseis": "conducir",
    "condujesen": "conducir",
    "condujeses": "conducir",
    "condujimos": "conducir",
    "condujiste": "conducir",
    "condujisteis": "conducir",
    "condujo": "conducir",
    "condujéramos": "conducir",
    "condujéremos": "conducir",
    "condujésemos": "conducir",
    "condumios": "condumio",
    "conduplicaciones": "conduplicación",
    "conduzca": "conducir",
    "conduzcamos": "conducir",
    "conduzcan": "conducir",
    "conduzcas": "conducir",
    "conduzco": "conducir",
    "conduzcáis": "conducir",
    "condílea": "condíleo",
    "condíleas": "condíleo",
    "condíleos": "condíleo",
    "conecta": "conectar",
    "conectaba": "conectar",
    "conectabais": "conectar",
    "conectaban": "conectar",
    "conectabas": "conectar",
    "conectabilidades": "conectabilidad",
    "conectables": "conectable",
    "conectad": "conectar",
    "conectada": "conectar",
    "conectadas": "conectar",
    "conectado": "conectar",
    "conectadora": "conectador",
    "conectadoras": "conectador",
    "conectadores": "conectador",
    "conectados": "conectar",
    "conectamos": "conectar",
    "conectan": "conectar",
    "conectando": "conectar",
    "conectantes": "conectante",
    "conectara": "conectar",
    "conectarais": "conectar",
    "conectaran": "conectar",
    "conectaras": "conectar",
    "conectare": "conectar",
    "conectareis": "conectar",
    "conectaremos": "conectar",
    "conectaren": "conectar",
    "conectares": "conectar",
    "conectaron": "conectar",
    "conectará": "conectar",
    "conectarán": "conectar",
    "conectarás": "conectar",
    "conectaré": "conectar",
    "conectaréis": "conectar",
    "conectaría": "conectar",
    "conectaríais": "conectar",
    "conectaríamos": "conectar",
    "conectarían": "conectar",
    "conectarías": "conectar",
    "conectas": "conectar",
    "conectase": "conectar",
    "conectaseis": "conectar",
    "conectasen": "conectar",
    "conectases": "conectar",
    "conectaste": "conectar",
    "conectasteis": "conectar",
    "conecte": "conectar",
    "conectemos": "conectar",
    "conecten": "conectar",
    "conectes": "conectar",
    "conectiva": "conectivo",
    "conectivas": "conectivo",
    "conectividades": "conectividad",
    "conectivos": "conectivo",
    "conecto": "conectar",
    "conectora": "conector",
    "conectoras": "conector",
    "conectores": "conector",
    "conectábamos": "conectar",
    "conectáis": "conectar",
    "conectáramos": "conectar",
    "conectáremos": "conectar",
    "conectásemos": "conectar",
    "conecté": "conectar",
    "conectéis": "conectar",
    "conectó": "conectar",
    "coneja": "conejo",
    "conejales": "conejal",
    "conejares": "conejar",
    "conejas": "conejo",
    "conejera": "conejero",
    "conejeras": "conejero",
    "conejeros": "conejero",
    "conejillos": "conejillo",
    "conejos": "conejo",
    "conejuna": "conejuno",
    "conejunas": "conejuno",
    "conejunos": "conejuno",
    "conexa": "conexo",
    "conexas": "conexo",
    "conexiona": "conexionar",
    "conexionaba": "conexionar",
    "conexionabais": "conexionar",
    "conexionaban": "conexionar",
    "conexionabas": "conexionar",
    "conexionad": "conexionar",
    "conexionada": "conexionar",
    "conexionadas": "conexionar",
    "conexionado": "conexionar",
    "conexionados": "conexionar",
    "conexionamos": "conexionar",
    "conexionan": "conexionar",
    "conexionando": "conexionar",
    "conexionara": "conexionar",
    "conexionarais": "conexionar",
    "conexionaran": "conexionar",
    "conexionaras": "conexionar",
    "conexionare": "conexionar",
    "conexionareis": "conexionar",
    "conexionaremos": "conexionar",
    "conexionaren": "conexionar",
    "conexionares": "conexionar",
    "conexionaron": "conexionar",
    "conexionará": "conexionar",
    "conexionarán": "conexionar",
    "conexionarás": "conexionar",
    "conexionaré": "conexionar",
    "conexionaréis": "conexionar",
    "conexionaría": "conexionar",
    "conexionaríais": "conexionar",
    "conexionaríamos": "conexionar",
    "conexionarían": "conexionar",
    "conexionarías": "conexionar",
    "conexionas": "conexionar",
    "conexionase": "conexionar",
    "conexionaseis": "conexionar",
    "conexionasen": "conexionar",
    "conexionases": "conexionar",
    "conexionaste": "conexionar",
    "conexionasteis": "conexionar",
    "conexione": "conexionar",
    "conexionemos": "conexionar",
    "conexionen": "conexionar",
    "conexiones": "conexionar",
    "conexiono": "conexionar",
    "conexionábamos": "conexionar",
    "conexionáis": "conexionar",
    "conexionáramos": "conexionar",
    "conexionáremos": "conexionar",
    "conexionásemos": "conexionar",
    "conexioné": "conexionar",
    "conexionéis": "conexionar",
    "conexionó": "conexionar",
    "conexiva": "conexivo",
    "conexivas": "conexivo",
    "conexivos": "conexivo",
    "conexos": "conexo",
    "confabula": "confabular",
    "confabulaba": "confabular",
    "confabulabais": "confabular",
    "confabulaban": "confabular",
    "confabulabas": "confabular",
    "confabulaciones": "confabulación",
    "confabulad": "confabular",
    "confabulada": "confabular",
    "confabuladas": "confabular",
    "confabulado": "confabular",
    "confabuladora": "confabulador",
    "confabuladoras": "confabulador",
    "confabuladores": "confabulador",
    "confabulados": "confabular",
    "confabulamos": "confabular",
    "confabulan": "confabular",
    "confabulando": "confabular",
    "confabulara": "confabular",
    "confabularais": "confabular",
    "confabularan": "confabular",
    "confabularas": "confabular",
    "confabulare": "confabular",
    "confabulareis": "confabular",
    "confabularemos": "confabular",
    "confabularen": "confabular",
    "confabulares": "confabular",
    "confabularon": "confabular",
    "confabulará": "confabular",
    "confabularán": "confabular",
    "confabularás": "confabular",
    "confabularé": "confabular",
    "confabularéis": "confabular",
    "confabularía": "confabular",
    "confabularíais": "confabular",
    "confabularíamos": "confabular",
    "confabularían": "confabular",
    "confabularías": "confabular",
    "confabulas": "confabular",
    "confabulase": "confabular",
    "confabulaseis": "confabular",
    "confabulasen": "confabular",
    "confabulases": "confabular",
    "confabulaste": "confabular",
    "confabulasteis": "confabular",
    "confabule": "confabular",
    "confabulemos": "confabular",
    "confabulen": "confabular",
    "confabules": "confabular",
    "confabulo": "confabular",
    "confabulábamos": "confabular",
    "confabuláis": "confabular",
    "confabuláramos": "confabular",
    "confabuláremos": "confabular",
    "confabulásemos": "confabular",
    "confabulé": "confabular",
    "confabuléis": "confabular",
    "confabuló": "confabular",
    "confalones": "confalón",
    "confaloniera": "confaloniero",
    "confalonieras": "confaloniero",
    "confalonieres": "confalonier",
    "confalonieros": "confaloniero",
    "confarreaciones": "confarreación",
    "confecciona": "confeccionar",
    "confeccionaba": "confeccionar",
    "confeccionabais": "confeccionar",
    "confeccionaban": "confeccionar",
    "confeccionabas": "confeccionar",
    "confeccionad": "confeccionar",
    "confeccionada": "confeccionar",
    "confeccionadas": "confeccionar",
    "confeccionado": "confeccionar",
    "confeccionadora": "confeccionador",
    "confeccionadoras": "confeccionador",
    "confeccionadores": "confeccionador",
    "confeccionados": "confeccionar",
    "confeccionamos": "confeccionar",
    "confeccionan": "confeccionar",
    "confeccionando": "confeccionar",
    "confeccionara": "confeccionar",
    "confeccionarais": "confeccionar",
    "confeccionaran": "confeccionar",
    "confeccionaras": "confeccionar",
    "confeccionare": "confeccionar",
    "confeccionareis": "confeccionar",
    "confeccionaremos": "confeccionar",
    "confeccionaren": "confeccionar",
    "confeccionares": "confeccionar",
    "confeccionaron": "confeccionar",
    "confeccionará": "confeccionar",
    "confeccionarán": "confeccionar",
    "confeccionarás": "confeccionar",
    "confeccionaré": "confeccionar",
    "confeccionaréis": "confeccionar",
    "confeccionaría": "confeccionar",
    "confeccionaríais": "confeccionar",
    "confeccionaríamos": "confeccionar",
    "confeccionarían": "confeccionar",
    "confeccionarías": "confeccionar",
    "confeccionas": "confeccionar",
    "confeccionase": "confeccionar",
    "confeccionaseis": "confeccionar",
    "confeccionasen": "confeccionar",
    "confeccionases": "confeccionar",
    "confeccionaste": "confeccionar",
    "confeccionasteis": "confeccionar",
    "confeccione": "confeccionar",
    "confeccionemos": "confeccionar",
    "confeccionen": "confeccionar",
    "confecciones": "confeccionar",
    "confeccionistas": "confeccionista",
    "confecciono": "confeccionar",
    "confeccionábamos": "confeccionar",
    "confeccionáis": "confeccionar",
    "confeccionáramos": "confeccionar",
    "confeccionáremos": "confeccionar",
    "confeccionásemos": "confeccionar",
    "confeccioné": "confeccionar",
    "confeccionéis": "confeccionar",
    "confeccionó": "confeccionar",
    "confectora": "confector",
    "confectoras": "confector",
    "confectores": "confector",
    "confedera": "confederar",
    "confederaba": "confederar",
    "confederabais": "confederar",
    "confederaban": "confederar",
    "confederabas": "confederar",
    "confederaciones": "confederación",
    "confederad": "confederar",
    "confederada": "confederar",
    "confederadas": "confederar",
    "confederado": "confederar",
    "confederados": "confederar",
    "confederales": "confederal",
    "confederamos": "confederar",
    "confederan": "confederar",
    "confederando": "confederar",
    "confederara": "confederar",
    "confederarais": "confederar",
    "confederaran": "confederar",
    "confederaras": "confederar",
    "confederare": "confederar",
    "confederareis": "confederar",
    "confederaremos": "confederar",
    "confederaren": "confederar",
    "confederares": "confederar",
    "confederaron": "confederar",
    "confederará": "confederar",
    "confederarán": "confederar",
    "confederarás": "confederar",
    "confederaré": "confederar",
    "confederaréis": "confederar",
    "confederaría": "confederar",
    "confederaríais": "confederar",
    "confederaríamos": "confederar",
    "confederarían": "confederar",
    "confederarías": "confederar",
    "confederas": "confederar",
    "confederase": "confederar",
    "confederaseis": "confederar",
    "confederasen": "confederar",
    "confederases": "confederar",
    "confederaste": "confederar",
    "confederasteis": "confederar",
    "confederativa": "confederativo",
    "confederativas": "confederativo",
    "confederativos": "confederativo",
    "confedere": "confederar",
    "confederemos": "confederar",
    "confederen": "confederar",
    "confederes": "confederar",
    "confedero": "confederar",
    "confederábamos": "confederar",
    "confederáis": "confederar",
    "confederáramos": "confederar",
    "confederáremos": "confederar",
    "confederásemos": "confederar",
    "confederé": "confederar",
    "confederéis": "confederar",
    "confederó": "confederar",
    "conferencia": "conferenciar",
    "conferenciaba": "conferenciar",
    "conferenciabais": "conferenciar",
    "conferenciaban": "conferenciar",
    "conferenciabas": "conferenciar",
    "conferenciad": "conferenciar",
    "conferenciada": "conferenciar",
    "conferenciadas": "conferenciar",
    "conferenciado": "conferenciar",
    "conferenciados": "conferenciar",
    "conferenciamos": "conferenciar",
    "conferencian": "conferenciar",
    "conferenciando": "conferenciar",
    "conferenciantes": "conferenciante",
    "conferenciara": "conferenciar",
    "conferenciarais": "conferenciar",
    "conferenciaran": "conferenciar",
    "conferenciaras": "conferenciar",
    "conferenciare": "conferenciar",
    "conferenciareis": "conferenciar",
    "conferenciaremos": "conferenciar",
    "conferenciaren": "conferenciar",
    "conferenciares": "conferenciar",
    "conferenciaron": "conferenciar",
    "conferenciará": "conferenciar",
    "conferenciarán": "conferenciar",
    "conferenciarás": "conferenciar",
    "conferenciaré": "conferenciar",
    "conferenciaréis": "conferenciar",
    "conferenciaría": "conferenciar",
    "conferenciaríais": "conferenciar",
    "conferenciaríamos": "conferenciar",
    "conferenciarían": "conferenciar",
    "conferenciarías": "conferenciar",
    "conferencias": "conferenciar",
    "conferenciase": "conferenciar",
    "conferenciaseis": "conferenciar",
    "conferenciasen": "conferenciar",
    "conferenciases": "conferenciar",
    "conferenciaste": "conferenciar",
    "conferenciasteis": "conferenciar",
    "conferencie": "conferenciar",
    "conferenciemos": "conferenciar",
    "conferencien": "conferenciar",
    "conferencies": "conferenciar",
    "conferencio": "conferenciar",
    "conferencistas": "conferencista",
    "conferenciábamos": "conferenciar",
    "conferenciáis": "conferenciar",
    "conferenciáramos": "conferenciar",
    "conferenciáremos": "conferenciar",
    "conferenciásemos": "conferenciar",
    "conferencié": "conferenciar",
    "conferenciéis": "conferenciar",
    "conferenció": "conferenciar",
    "conferid": "conferir",
    "conferida": "conferir",
    "conferidas": "conferir",
    "conferido": "conferir",
    "conferidos": "conferir",
    "conferimientos": "conferimiento",
    "conferimos": "conferir",
    "conferiremos": "conferir",
    "conferirá": "conferir",
    "conferirán": "conferir",
    "conferirás": "conferir",
    "conferiré": "conferir",
    "conferiréis": "conferir",
    "conferiría": "conferir",
    "conferiríais": "conferir",
    "conferiríamos": "conferir",
    "conferirían": "conferir",
    "conferirías": "conferir",
    "conferiste": "conferir",
    "conferisteis": "conferir",
    "conferí": "conferir",
    "confería": "conferir",
    "conferíais": "conferir",
    "conferíamos": "conferir",
    "conferían": "conferir",
    "conferías": "conferir",
    "conferís": "conferir",
    "confesa": "confeso",
    "confesaba": "confesar",
    "confesabais": "confesar",
    "confesaban": "confesar",
    "confesabas": "confesar",
    "confesad": "confesar",
    "confesada": "confesar",
    "confesadas": "confesar",
    "confesado": "confesar",
    "confesados": "confesar",
    "confesamos": "confesar",
    "confesando": "confesar",
    "confesara": "confesar",
    "confesarais": "confesar",
    "confesaran": "confesar",
    "confesaras": "confesar",
    "confesare": "confesar",
    "confesareis": "confesar",
    "confesaremos": "confesar",
    "confesaren": "confesar",
    "confesares": "confesar",
    "confesaron": "confesar",
    "confesará": "confesar",
    "confesarán": "confesar",
    "confesarás": "confesar",
    "confesaré": "confesar",
    "confesaréis": "confesar",
    "confesaría": "confesar",
    "confesaríais": "confesar",
    "confesaríamos": "confesar",
    "confesarían": "confesar",
    "confesarías": "confesar",
    "confesas": "confeso",
    "confesase": "confesar",
    "confesaseis": "confesar",
    "confesasen": "confesar",
    "confesases": "confesar",
    "confesaste": "confesar",
    "confesasteis": "confesar",
    "confesemos": "confesar",
    "confesionales": "confesional",
    "confesionalidades": "confesionalidad",
    "confesionarios": "confesionario",
    "confesiones": "confesión",
    "confesonarios": "confesonario",
    "confesora": "confesor",
    "confesoras": "confesor",
    "confesores": "confesor",
    "confesoria": "confesorio",
    "confesorias": "confesorio",
    "confesorios": "confesorio",
    "confesos": "confeso",
    "confesábamos": "confesar",
    "confesáis": "confesar",
    "confesáramos": "confesar",
    "confesáremos": "confesar",
    "confesásemos": "confesar",
    "confesé": "confesar",
    "confeséis": "confesar",
    "confesó": "confesar",
    "confetis": "confeti",
    "confiaba": "confiar",
    "confiabais": "confiar",
    "confiaban": "confiar",
    "confiabas": "confiar",
    "confiabilidades": "confiabilidad",
    "confiables": "confiable",
    "confiad": "confiar",
    "confiada": "confiar",
    "confiadas": "confiar",
    "confiado": "confiar",
    "confiados": "confiar",
    "confiamos": "confiar",
    "confiando": "confiar",
    "confianzas": "confianza",
    "confianzuda": "confianzudo",
    "confianzudas": "confianzudo",
    "confianzudos": "confianzudo",
    "confiara": "confiar",
    "confiarais": "confiar",
    "confiaran": "confiar",
    "confiaras": "confiar",
    "confiare": "confiar",
    "confiareis": "confiar",
    "confiaremos": "confiar",
    "confiaren": "confiar",
    "confiares": "confiar",
    "confiaron": "confiar",
    "confiará": "confiar",
    "confiarán": "confiar",
    "confiarás": "confiar",
    "confiaré": "confiar",
    "confiaréis": "confiar",
    "confiaría": "confiar",
    "confiaríais": "confiar",
    "confiaríamos": "confiar",
    "confiarían": "confiar",
    "confiarías": "confiar",
    "confiase": "confiar",
    "confiaseis": "confiar",
    "confiasen": "confiar",
    "confiases": "confiar",
    "confiaste": "confiar",
    "confiasteis": "confiar",
    "confidenciales": "confidencial",
    "confidencialidades": "confidencialidad",
    "confidencias": "confidencia",
    "confidentes": "confidente",
    "confiemos": "confiar",
    "confiera": "conferir",
    "confieran": "conferir",
    "confieras": "conferir",
    "confiere": "conferir",
    "confieren": "conferir",
    "confieres": "conferir",
    "confiero": "conferir",
    "confiesa": "confesar",
    "confiesan": "confesar",
    "confiesas": "confesar",
    "confiese": "confesar",
    "confiesen": "confesar",
    "confieses": "confesar",
    "confieso": "confesar",
    "configura": "configurar",
    "configuraba": "configurar",
    "configurabais": "configurar",
    "configuraban": "configurar",
    "configurabas": "configurar",
    "configurabilidades": "configurabilidad",
    "configurables": "configurable",
    "configuracionales": "configuracional",
    "configuraciones": "configuración",
    "configuracionismos": "configuracionismo",
    "configurad": "configurar",
    "configurada": "configurar",
    "configuradas": "configurar",
    "configurado": "configurar",
    "configuradores": "configurador",
    "configurados": "configurar",
    "configuramos": "configurar",
    "configuran": "configurar",
    "configurando": "configurar",
    "configurara": "configurar",
    "configurarais": "configurar",
    "configuraran": "configurar",
    "configuraras": "configurar",
    "configurare": "configurar",
    "configurareis": "configurar",
    "configuraremos": "configurar",
    "configuraren": "configurar",
    "configurares": "configurar",
    "configuraron": "configurar",
    "configurará": "configurar",
    "configurarán": "configurar",
    "configurarás": "configurar",
    "configuraré": "configurar",
    "configuraréis": "configurar",
    "configuraría": "configurar",
    "configuraríais": "configurar",
    "configuraríamos": "configurar",
    "configurarían": "configurar",
    "configurarías": "configurar",
    "configuras": "configurar",
    "configurase": "configurar",
    "configuraseis": "configurar",
    "configurasen": "configurar",
    "configurases": "configurar",
    "configuraste": "configurar",
    "configurasteis": "configurar",
    "configure": "configurar",
    "configuremos": "configurar",
    "configuren": "configurar",
    "configures": "configurar",
    "configuro": "configurar",
    "configurábamos": "configurar",
    "configuráis": "configurar",
    "configuráramos": "configurar",
    "configuráremos": "configurar",
    "configurásemos": "configurar",
    "configuré": "configurar",
    "configuréis": "configurar",
    "configuró": "configurar",
    "confina": "confinar",
    "confinaba": "confinar",
    "confinabais": "confinar",
    "confinaban": "confinar",
    "confinabas": "confinar",
    "confinaciones": "confinación",
    "confinad": "confinar",
    "confinada": "confinar",
    "confinadas": "confinar",
    "confinado": "confinar",
    "confinados": "confinar",
    "confinamientos": "confinamiento",
    "confinamos": "confinar",
    "confinan": "confinar",
    "confinando": "confinar",
    "confinara": "confinar",
    "confinarais": "confinar",
    "confinaran": "confinar",
    "confinaras": "confinar",
    "confinare": "confinar",
    "confinareis": "confinar",
    "confinaremos": "confinar",
    "confinaren": "confinar",
    "confinares": "confinar",
    "confinaron": "confinar",
    "confinará": "confinar",
    "confinarán": "confinar",
    "confinarás": "confinar",
    "confinaré": "confinar",
    "confinaréis": "confinar",
    "confinaría": "confinar",
    "confinaríais": "confinar",
    "confinaríamos": "confinar",
    "confinarían": "confinar",
    "confinarías": "confinar",
    "confinas": "confinar",
    "confinase": "confinar",
    "confinaseis": "confinar",
    "confinasen": "confinar",
    "confinases": "confinar",
    "confinaste": "confinar",
    "confinasteis": "confinar",
    "confine": "confinar",
    "confinemos": "confinar",
    "confinen": "confinar",
    "confines": "confinar",
    "confinidades": "confinidad",
    "confino": "confinar",
    "confinábamos": "confinar",
    "confináis": "confinar",
    "confináramos": "confinar",
    "confináremos": "confinar",
    "confinásemos": "confinar",
    "confiné": "confinar",
    "confinéis": "confinar",
    "confinó": "confinar",
    "confiramos": "conferir",
    "confiriendo": "conferir",
    "confiriera": "conferir",
    "confirierais": "conferir",
    "confirieran": "conferir",
    "confirieras": "conferir",
    "confiriere": "conferir",
    "confiriereis": "conferir",
    "confirieren": "conferir",
    "confirieres": "conferir",
    "confirieron": "conferir",
    "confiriese": "conferir",
    "confirieseis": "conferir",
    "confiriesen": "conferir",
    "confirieses": "conferir",
    "confiriéramos": "conferir",
    "confiriéremos": "conferir",
    "confiriésemos": "conferir",
    "confirió": "conferir",
    "confirma": "confirmar",
    "confirmaba": "confirmar",
    "confirmabais": "confirmar",
    "confirmaban": "confirmar",
    "confirmabas": "confirmar",
    "confirmables": "confirmable",
    "confirmaciones": "confirmación",
    "confirmad": "confirmar",
    "confirmada": "confirmar",
    "confirmadas": "confirmar",
    "confirmado": "confirmar",
    "confirmadora": "confirmador",
    "confirmadoras": "confirmador",
    "confirmadores": "confirmador",
    "confirmados": "confirmar",
    "confirmamos": "confirmar",
    "confirman": "confirmar",
    "confirmanda": "confirmando",
    "confirmandas": "confirmando",
    "confirmando": "confirmar",
    "confirmandos": "confirmando",
    "confirmara": "confirmar",
    "confirmarais": "confirmar",
    "confirmaran": "confirmar",
    "confirmaras": "confirmar",
    "confirmare": "confirmar",
    "confirmareis": "confirmar",
    "confirmaremos": "confirmar",
    "confirmaren": "confirmar",
    "confirmares": "confirmar",
    "confirmaron": "confirmar",
    "confirmará": "confirmar",
    "confirmarán": "confirmar",
    "confirmarás": "confirmar",
    "confirmaré": "confirmar",
    "confirmaréis": "confirmar",
    "confirmaría": "confirmar",
    "confirmaríais": "confirmar",
    "confirmaríamos": "confirmar",
    "confirmarían": "confirmar",
    "confirmarías": "confirmar",
    "confirmas": "confirmar",
    "confirmase": "confirmar",
    "confirmaseis": "confirmar",
    "confirmasen": "confirmar",
    "confirmases": "confirmar",
    "confirmaste": "confirmar",
    "confirmasteis": "confirmar",
    "confirmativa": "confirmativo",
    "confirmativas": "confirmativo",
    "confirmativos": "confirmativo",
    "confirmatoria": "confirmatorio",
    "confirmatorias": "confirmatorio",
    "confirmatorios": "confirmatorio",
    "confirme": "confirmar",
    "confirmemos": "confirmar",
    "confirmen": "confirmar",
    "confirmes": "confirmar",
    "confirmo": "confirmar",
    "confirmábamos": "confirmar",
    "confirmáis": "confirmar",
    "confirmáramos": "confirmar",
    "confirmáremos": "confirmar",
    "confirmásemos": "confirmar",
    "confirmé": "confirmar",
    "confirméis": "confirmar",
    "confirmó": "confirmar",
    "confiráis": "conferir",
    "confisca": "confiscar",
    "confiscaba": "confiscar",
    "confiscabais": "confiscar",
    "confiscaban": "confiscar",
    "confiscabas": "confiscar",
    "confiscaciones": "confiscación",
    "confiscad": "confiscar",
    "confiscada": "confiscar",
    "confiscadas": "confiscar",
    "confiscado": "confiscar",
    "confiscados": "confiscar",
    "confiscamos": "confiscar",
    "confiscan": "confiscar",
    "confiscando": "confiscar",
    "confiscara": "confiscar",
    "confiscarais": "confiscar",
    "confiscaran": "confiscar",
    "confiscaras": "confiscar",
    "confiscare": "confiscar",
    "confiscareis": "confiscar",
    "confiscaremos": "confiscar",
    "confiscaren": "confiscar",
    "confiscares": "confiscar",
    "confiscaron": "confiscar",
    "confiscará": "confiscar",
    "confiscarán": "confiscar",
    "confiscarás": "confiscar",
    "confiscaré": "confiscar",
    "confiscaréis": "confiscar",
    "confiscaría": "confiscar",
    "confiscaríais": "confiscar",
    "confiscaríamos": "confiscar",
    "confiscarían": "confiscar",
    "confiscarías": "confiscar",
    "confiscas": "confiscar",
    "confiscase": "confiscar",
    "confiscaseis": "confiscar",
    "confiscasen": "confiscar",
    "confiscases": "confiscar",
    "confiscaste": "confiscar",
    "confiscasteis": "confiscar",
    "confiscatoria": "confiscatorio",
    "confiscatorias": "confiscatorio",
    "confiscatoriedades": "confiscatoriedad",
    "confiscatorios": "confiscatorio",
    "confisco": "confiscar",
    "confiscábamos": "confiscar",
    "confiscáis": "confiscar",
    "confiscáramos": "confiscar",
    "confiscáremos": "confiscar",
    "confiscásemos": "confiscar",
    "confiscó": "confiscar",
    "confisque": "confiscar",
    "confisquemos": "confiscar",
    "confisquen": "confiscar",
    "confisques": "confiscar",
    "confisqué": "confiscar",
    "confisquéis": "confiscar",
    "confita": "confitar",
    "confitaba": "confitar",
    "confitabais": "confitar",
    "confitaban": "confitar",
    "confitabas": "confitar",
    "confitad": "confitar",
    "confitada": "confitar",
    "confitadas": "confitar",
    "confitado": "confitar",
    "confitados": "confitar",
    "confitamos": "confitar",
    "confitan": "confitar",
    "confitando": "confitar",
    "confitara": "confitar",
    "confitarais": "confitar",
    "confitaran": "confitar",
    "confitaras": "confitar",
    "confitare": "confitar",
    "confitareis": "confitar",
    "confitaremos": "confitar",
    "confitaren": "confitar",
    "confitares": "confitar",
    "confitaron": "confitar",
    "confitará": "confitar",
    "confitarán": "confitar",
    "confitarás": "confitar",
    "confitaré": "confitar",
    "confitaréis": "confitar",
    "confitaría": "confitar",
    "confitaríais": "confitar",
    "confitaríamos": "confitar",
    "confitarían": "confitar",
    "confitarías": "confitar",
    "confitas": "confitar",
    "confitase": "confitar",
    "confitaseis": "confitar",
    "confitasen": "confitar",
    "confitases": "confitar",
    "confitaste": "confitar",
    "confitasteis": "confitar",
    "confite": "confitar",
    "confitemos": "confitar",
    "confiten": "confitar",
    "confitera": "confitero",
    "confiteras": "confitero",
    "confiteros": "confitero",
    "confiterías": "confitería",
    "confites": "confite",
    "confito": "confitar",
    "confituras": "confitura",
    "confitábamos": "confitar",
    "confitáis": "confitar",
    "confitáramos": "confitar",
    "confitáremos": "confitar",
    "confitásemos": "confitar",
    "confité": "confitar",
    "confitéis": "confitar",
    "confitó": "confitar",
    "confiábamos": "confiar",
    "confiáis": "confiar",
    "confiáramos": "confiar",
    "confiáremos": "confiar",
    "confiásemos": "confiar",
    "confié": "confiar",
    "confiéis": "confiar",
    "confió": "confiar",
    "conflaciones": "conflación",
    "conflagra": "conflagrar",
    "conflagraba": "conflagrar",
    "conflagrabais": "conflagrar",
    "conflagraban": "conflagrar",
    "conflagrabas": "conflagrar",
    "conflagraciones": "conflagración",
    "conflagrad": "conflagrar",
    "conflagrada": "conflagrar",
    "conflagradas": "conflagrar",
    "conflagrado": "conflagrar",
    "conflagrados": "conflagrar",
    "conflagramos": "conflagrar",
    "conflagran": "conflagrar",
    "conflagrando": "conflagrar",
    "conflagrara": "conflagrar",
    "conflagrarais": "conflagrar",
    "conflagraran": "conflagrar",
    "conflagraras": "conflagrar",
    "conflagrare": "conflagrar",
    "conflagrareis": "conflagrar",
    "conflagraremos": "conflagrar",
    "conflagraren": "conflagrar",
    "conflagrares": "conflagrar",
    "conflagraron": "conflagrar",
    "conflagrará": "conflagrar",
    "conflagrarán": "conflagrar",
    "conflagrarás": "conflagrar",
    "conflagraré": "conflagrar",
    "conflagraréis": "conflagrar",
    "conflagraría": "conflagrar",
    "conflagraríais": "conflagrar",
    "conflagraríamos": "conflagrar",
    "conflagrarían": "conflagrar",
    "conflagrarías": "conflagrar",
    "conflagras": "conflagrar",
    "conflagrase": "conflagrar",
    "conflagraseis": "conflagrar",
    "conflagrasen": "conflagrar",
    "conflagrases": "conflagrar",
    "conflagraste": "conflagrar",
    "conflagrasteis": "conflagrar",
    "conflagre": "conflagrar",
    "conflagremos": "conflagrar",
    "conflagren": "conflagrar",
    "conflagres": "conflagrar",
    "conflagro": "conflagrar",
    "conflagrábamos": "conflagrar",
    "conflagráis": "conflagrar",
    "conflagráramos": "conflagrar",
    "conflagráremos": "conflagrar",
    "conflagrásemos": "conflagrar",
    "conflagré": "conflagrar",
    "conflagréis": "conflagrar",
    "conflagró": "conflagrar",
    "conflictiva": "conflictivo",
    "conflictivas": "conflictivo",
    "conflictividades": "conflictividad",
    "conflictivismos": "conflictivismo",
    "conflictivos": "conflictivo",
    "conflictos": "conflicto",
    "confluencias": "confluencia",
    "confluentes": "confluente",
    "confluid": "confluir",
    "confluida": "confluir",
    "confluidas": "confluir",
    "confluido": "confluir",
    "confluidos": "confluir",
    "confluimos": "confluir",
    "confluiremos": "confluir",
    "confluirá": "confluir",
    "confluirán": "confluir",
    "confluirás": "confluir",
    "confluiré": "confluir",
    "confluiréis": "confluir",
    "confluiría": "confluir",
    "confluiríais": "confluir",
    "confluiríamos": "confluir",
    "confluirían": "confluir",
    "confluirías": "confluir",
    "confluiste": "confluir",
    "confluisteis": "confluir",
    "confluya": "confluir",
    "confluyamos": "confluir",
    "confluyan": "confluir",
    "confluyas": "confluir",
    "confluye": "confluir",
    "confluyen": "confluir",
    "confluyendo": "confluir",
    "confluyera": "confluir",
    "confluyerais": "confluir",
    "confluyeran": "confluir",
    "confluyeras": "confluir",
    "confluyere": "confluir",
    "confluyereis": "confluir",
    "confluyeren": "confluir",
    "confluyeres": "confluir",
    "confluyeron": "confluir",
    "confluyes": "confluir",
    "confluyese": "confluir",
    "confluyeseis": "confluir",
    "confluyesen": "confluir",
    "confluyeses": "confluir",
    "confluyo": "confluir",
    "confluyáis": "confluir",
    "confluyéramos": "confluir",
    "confluyéremos": "confluir",
    "confluyésemos": "confluir",
    "confluyó": "confluir",
    "confluí": "confluir",
    "confluía": "confluir",
    "confluíais": "confluir",
    "confluíamos": "confluir",
    "confluían": "confluir",
    "confluías": "confluir",
    "confluís": "confluir",
    "conflátiles": "conflátil",
    "confocales": "confocal",
    "conforma": "conformar",
    "conformaba": "conformar",
    "conformabais": "conformar",
    "conformaban": "conformar",
    "conformabas": "conformar",
    "conformabilidades": "conformabilidad",
    "conformables": "conformable",
    "conformacionales": "conformacional",
    "conformaciones": "conformación",
    "conformad": "conformar",
    "conformada": "conformar",
    "conformadas": "conformar",
    "conformado": "conformar",
    "conformadores": "conformador",
    "conformados": "conformar",
    "conformamos": "conformar",
    "conforman": "conformar",
    "conformando": "conformar",
    "conformantes": "conformante",
    "conformara": "conformar",
    "conformarais": "conformar",
    "conformaran": "conformar",
    "conformaras": "conformar",
    "conformare": "conformar",
    "conformareis": "conformar",
    "conformaremos": "conformar",
    "conformaren": "conformar",
    "conformares": "conformar",
    "conformaron": "conformar",
    "conformará": "conformar",
    "conformarán": "conformar",
    "conformarás": "conformar",
    "conformaré": "conformar",
    "conformaréis": "conformar",
    "conformaría": "conformar",
    "conformaríais": "conformar",
    "conformaríamos": "conformar",
    "conformarían": "conformar",
    "conformarías": "conformar",
    "conformas": "conformar",
    "conformase": "conformar",
    "conformaseis": "conformar",
    "conformasen": "conformar",
    "conformases": "conformar",
    "conformaste": "conformar",
    "conformasteis": "conformar",
    "conforme": "conformar",
    "conformemos": "conformar",
    "conformen": "conformar",
    "conformes": "conforme",
    "conformidades": "conformidad",
    "conformismos": "conformismo",
    "conformistas": "conformista",
    "conformo": "conformar",
    "conformábamos": "conformar",
    "conformáis": "conformar",
    "conformáramos": "conformar",
    "conformáremos": "conformar",
    "conformásemos": "conformar",
    "conformé": "conformar",
    "conforméis": "conformar",
    "conformó": "conformar",
    "conforta": "confortar",
    "confortaba": "confortar",
    "confortabais": "confortar",
    "confortaban": "confortar",
    "confortabas": "confortar",
    "confortabilidades": "confortabilidad",
    "confortables": "confortable",
    "confortaciones": "confortación",
    "confortad": "confortar",
    "confortada": "confortar",
    "confortadas": "confortar",
    "confortado": "confortar",
    "confortadora": "confortador",
    "confortadoras": "confortador",
    "confortadores": "confortador",
    "confortados": "confortar",
    "confortamientos": "confortamiento",
    "confortamos": "confortar",
    "confortan": "confortar",
    "confortando": "confortar",
    "confortantes": "confortante",
    "confortara": "confortar",
    "confortarais": "confortar",
    "confortaran": "confortar",
    "confortaras": "confortar",
    "confortare": "confortar",
    "confortareis": "confortar",
    "confortaremos": "confortar",
    "confortaren": "confortar",
    "confortares": "confortar",
    "confortaron": "confortar",
    "confortará": "confortar",
    "confortarán": "confortar",
    "confortarás": "confortar",
    "confortaré": "confortar",
    "confortaréis": "confortar",
    "confortaría": "confortar",
    "confortaríais": "confortar",
    "confortaríamos": "confortar",
    "confortarían": "confortar",
    "confortarías": "confortar",
    "confortas": "confortar",
    "confortase": "confortar",
    "confortaseis": "confortar",
    "confortasen": "confortar",
    "confortases": "confortar",
    "confortaste": "confortar",
    "confortasteis": "confortar",
    "conforte": "confortar",
    "confortemos": "confortar",
    "conforten": "confortar",
    "confortes": "confortar",
    "conforto": "confortar",
    "conforts": "confort",
    "confortábamos": "confortar",
    "confortáis": "confortar",
    "confortáramos": "confortar",
    "confortáremos": "confortar",
    "confortásemos": "confortar",
    "conforté": "confortar",
    "confortéis": "confortar",
    "confortó": "confortar",
    "confracciones": "confracción",
    "confraterna": "confraternar",
    "confraternaba": "confraternar",
    "confraternabais": "confraternar",
    "confraternaban": "confraternar",
    "confraternabas": "confraternar",
    "confraternad": "confraternar",
    "confraternada": "confraternar",
    "confraternadas": "confraternar",
    "confraternado": "confraternar",
    "confraternados": "confraternar",
    "confraternamos": "confraternar",
    "confraternan": "confraternar",
    "confraternando": "confraternar",
    "confraternara": "confraternar",
    "confraternarais": "confraternar",
    "confraternaran": "confraternar",
    "confraternaras": "confraternar",
    "confraternare": "confraternar",
    "confraternareis": "confraternar",
    "confraternaremos": "confraternar",
    "confraternaren": "confraternar",
    "confraternares": "confraternar",
    "confraternaron": "confraternar",
    "confraternará": "confraternar",
    "confraternarán": "confraternar",
    "confraternarás": "confraternar",
    "confraternaré": "confraternar",
    "confraternaréis": "confraternar",
    "confraternaría": "confraternar",
    "confraternaríais": "confraternar",
    "confraternaríamos": "confraternar",
    "confraternarían": "confraternar",
    "confraternarías": "confraternar",
    "confraternas": "confraternar",
    "confraternase": "confraternar",
    "confraternaseis": "confraternar",
    "confraternasen": "confraternar",
    "confraternases": "confraternar",
    "confraternaste": "confraternar",
    "confraternasteis": "confraternar",
    "confraterne": "confraternar",
    "confraternemos": "confraternar",
    "confraternen": "confraternar",
    "confraternes": "confraternar",
    "confraternice": "confraternizar",
    "confraternicemos": "confraternizar",
    "confraternicen": "confraternizar",
    "confraternices": "confraternizar",
    "confraternicé": "confraternizar",
    "confraternicéis": "confraternizar",
    "confraternidades": "confraternidad",
    "confraterniza": "confraternizar",
    "confraternizaba": "confraternizar",
    "confraternizabais": "confraternizar",
    "confraternizaban": "confraternizar",
    "confraternizabas": "confraternizar",
    "confraternizaciones": "confraternización",
    "confraternizad": "confraternizar",
    "confraternizada": "confraternizar",
    "confraternizadas": "confraternizar",
    "confraternizado": "confraternizar",
    "confraternizados": "confraternizar",
    "confraternizamos": "confraternizar",
    "confraternizan": "confraternizar",
    "confraternizando": "confraternizar",
    "confraternizara": "confraternizar",
    "confraternizarais": "confraternizar",
    "confraternizaran": "confraternizar",
    "confraternizaras": "confraternizar",
    "confraternizare": "confraternizar",
    "confraternizareis": "confraternizar",
    "confraternizaremos": "confraternizar",
    "confraternizaren": "confraternizar",
    "confraternizares": "confraternizar",
    "confraternizaron": "confraternizar",
    "confraternizará": "confraternizar",
    "confraternizarán": "confraternizar",
    "confraternizarás": "confraternizar",
    "confraternizaré": "confraternizar",
    "confraternizaréis": "confraternizar",
    "confraternizaría": "confraternizar",
    "confraternizaríais": "confraternizar",
    "confraternizaríamos": "confraternizar",
    "confraternizarían": "confraternizar",
    "confraternizarías": "confraternizar",
    "confraternizas": "confraternizar",
    "confraternizase": "confraternizar",
    "confraternizaseis": "confraternizar",
    "confraternizasen": "confraternizar",
    "confraternizases": "confraternizar",
    "confraternizaste": "confraternizar",
    "confraternizasteis": "confraternizar",
    "confraternizo": "confraternizar",
    "confraternizábamos": "confraternizar",
    "confraternizáis": "confraternizar",
    "confraternizáramos": "confraternizar",
    "confraternizáremos": "confraternizar",
    "confraternizásemos": "confraternizar",
    "confraternizó": "confraternizar",
    "confraterno": "confraternar",
    "confraternábamos": "confraternar",
    "confraternáis": "confraternar",
    "confraternáramos": "confraternar",
    "confraternáremos": "confraternar",
    "confraternásemos": "confraternar",
    "confraterné": "confraternar",
    "confraternéis": "confraternar",
    "confraternó": "confraternar",
    "confricaciones": "confricación",
    "confronta": "confrontar",
    "confrontaba": "confrontar",
    "confrontabais": "confrontar",
    "confrontaban": "confrontar",
    "confrontabas": "confrontar",
    "confrontaciones": "confrontación",
    "confrontaciónes": "confrontación",
    "confrontad": "confrontar",
    "confrontada": "confrontar",
    "confrontadas": "confrontar",
    "confrontado": "confrontar",
    "confrontados": "confrontar",
    "confrontamos": "confrontar",
    "confrontan": "confrontar",
    "confrontando": "confrontar",
    "confrontara": "confrontar",
    "confrontarais": "confrontar",
    "confrontaran": "confrontar",
    "confrontaras": "confrontar",
    "confrontare": "confrontar",
    "confrontareis": "confrontar",
    "confrontaremos": "confrontar",
    "confrontaren": "confrontar",
    "confrontares": "confrontar",
    "confrontaron": "confrontar",
    "confrontará": "confrontar",
    "confrontarán": "confrontar",
    "confrontarás": "confrontar",
    "confrontaré": "confrontar",
    "confrontaréis": "confrontar",
    "confrontaría": "confrontar",
    "confrontaríais": "confrontar",
    "confrontaríamos": "confrontar",
    "confrontarían": "confrontar",
    "confrontarías": "confrontar",
    "confrontas": "confrontar",
    "confrontase": "confrontar",
    "confrontaseis": "confrontar",
    "confrontasen": "confrontar",
    "confrontases": "confrontar",
    "confrontaste": "confrontar",
    "confrontasteis": "confrontar",
    "confronte": "confrontar",
    "confrontemos": "confrontar",
    "confronten": "confrontar",
    "confrontes": "confrontar",
    "confronto": "confrontar",
    "confrontábamos": "confrontar",
    "confrontáis": "confrontar",
    "confrontáramos": "confrontar",
    "confrontáremos": "confrontar",
    "confrontásemos": "confrontar",
    "confronté": "confrontar",
    "confrontéis": "confrontar",
    "confrontó": "confrontar",
    "confuciana": "confuciano",
    "confucianas": "confuciano",
    "confucianismos": "confucianismo",
    "confucianistas": "confucianista",
    "confucianos": "confuciano",
    "confulgencias": "confulgencia",
    "confunda": "confundir",
    "confundamos": "confundir",
    "confundan": "confundir",
    "confundas": "confundir",
    "confunde": "confundir",
    "confunden": "confundir",
    "confundes": "confundir",
    "confundid": "confundir",
    "confundida": "confundir",
    "confundidas": "confundir",
    "confundido": "confundir",
    "confundidos": "confundir",
    "confundiendo": "confundir",
    "confundiera": "confundir",
    "confundierais": "confundir",
    "confundieran": "confundir",
    "confundieras": "confundir",
    "confundiere": "confundir",
    "confundiereis": "confundir",
    "confundieren": "confundir",
    "confundieres": "confundir",
    "confundieron": "confundir",
    "confundiese": "confundir",
    "confundieseis": "confundir",
    "confundiesen": "confundir",
    "confundieses": "confundir",
    "confundimos": "confundir",
    "confundiremos": "confundir",
    "confundirá": "confundir",
    "confundirán": "confundir",
    "confundirás": "confundir",
    "confundiré": "confundir",
    "confundiréis": "confundir",
    "confundiría": "confundir",
    "confundiríais": "confundir",
    "confundiríamos": "confundir",
    "confundirían": "confundir",
    "confundirías": "confundir",
    "confundiste": "confundir",
    "confundisteis": "confundir",
    "confundiéramos": "confundir",
    "confundiéremos": "confundir",
    "confundiésemos": "confundir",
    "confundió": "confundir",
    "confundo": "confundir",
    "confundáis": "confundir",
    "confundí": "confundir",
    "confundía": "confundir",
    "confundíais": "confundir",
    "confundíamos": "confundir",
    "confundían": "confundir",
    "confundías": "confundir",
    "confundís": "confundir",
    "confusa": "confuso",
    "confusas": "confuso",
    "confusiones": "confusión",
    "confusionismos": "confusionismo",
    "confusionistas": "confusionista",
    "confusos": "confuso",
    "confuta": "confutar",
    "confutaba": "confutar",
    "confutabais": "confutar",
    "confutaban": "confutar",
    "confutabas": "confutar",
    "confutaciones": "confutación",
    "confutad": "confutar",
    "confutada": "confutar",
    "confutadas": "confutar",
    "confutado": "confutar",
    "confutadora": "confutador",
    "confutadoras": "confutador",
    "confutadores": "confutador",
    "confutados": "confutar",
    "confutamos": "confutar",
    "confutan": "confutar",
    "confutando": "confutar",
    "confutara": "confutar",
    "confutarais": "confutar",
    "confutaran": "confutar",
    "confutaras": "confutar",
    "confutare": "confutar",
    "confutareis": "confutar",
    "confutaremos": "confutar",
    "confutaren": "confutar",
    "confutares": "confutar",
    "confutaron": "confutar",
    "confutará": "confutar",
    "confutarán": "confutar",
    "confutarás": "confutar",
    "confutaré": "confutar",
    "confutaréis": "confutar",
    "confutaría": "confutar",
    "confutaríais": "confutar",
    "confutaríamos": "confutar",
    "confutarían": "confutar",
    "confutarías": "confutar",
    "confutas": "confutar",
    "confutase": "confutar",
    "confutaseis": "confutar",
    "confutasen": "confutar",
    "confutases": "confutar",
    "confutaste": "confutar",
    "confutasteis": "confutar",
    "confutatoria": "confutatorio",
    "confutatorias": "confutatorio",
    "confutatorios": "confutatorio",
    "confute": "confutar",
    "confutemos": "confutar",
    "confuten": "confutar",
    "confutes": "confutar",
    "confuto": "confutar",
    "confutábamos": "confutar",
    "confutáis": "confutar",
    "confutáramos": "confutar",
    "confutáremos": "confutar",
    "confutásemos": "confutar",
    "confuté": "confutar",
    "confutéis": "confutar",
    "confutó": "confutar",
    "confía": "confiar",
    "confían": "confiar",
    "confías": "confiar",
    "confíe": "confiar",
    "confíen": "confiar",
    "confíes": "confiar",
    "confío": "confiar",
    "confíteores": "confíteor",
    "congas": "conga",
    "congela": "congelar",
    "congelaba": "congelar",
    "congelabais": "congelar",
    "congelaban": "congelar",
    "congelabas": "congelar",
    "congelables": "congelable",
    "congelaciones": "congelación",
    "congelad": "congelar",
    "congelada": "congelar",
    "congeladas": "congelar",
    "congelado": "congelar",
    "congeladora": "congelador",
    "congeladoras": "congelador",
    "congeladores": "congelador",
    "congelados": "congelar",
    "congelamientos": "congelamiento",
    "congelamos": "congelar",
    "congelan": "congelar",
    "congelando": "congelar",
    "congelara": "congelar",
    "congelarais": "congelar",
    "congelaran": "congelar",
    "congelaras": "congelar",
    "congelare": "congelar",
    "congelareis": "congelar",
    "congelaremos": "congelar",
    "congelaren": "congelar",
    "congelares": "congelar",
    "congelaron": "congelar",
    "congelará": "congelar",
    "congelarán": "congelar",
    "congelarás": "congelar",
    "congelaré": "congelar",
    "congelaréis": "congelar",
    "congelaría": "congelar",
    "congelaríais": "congelar",
    "congelaríamos": "congelar",
    "congelarían": "congelar",
    "congelarías": "congelar",
    "congelas": "congelar",
    "congelase": "congelar",
    "congelaseis": "congelar",
    "congelasen": "congelar",
    "congelases": "congelar",
    "congelaste": "congelar",
    "congelasteis": "congelar",
    "congele": "congelar",
    "congelemos": "congelar",
    "congelen": "congelar",
    "congeles": "congelar",
    "congelo": "congelar",
    "congelábamos": "congelar",
    "congeláis": "congelar",
    "congeláramos": "congelar",
    "congeláremos": "congelar",
    "congelásemos": "congelar",
    "congelé": "congelar",
    "congeléis": "congelar",
    "congeló": "congelar",
    "congenia": "congeniar",
    "congeniaba": "congeniar",
    "congeniabais": "congeniar",
    "congeniaban": "congeniar",
    "congeniabas": "congeniar",
    "congeniad": "congeniar",
    "congeniada": "congeniar",
    "congeniadas": "congeniar",
    "congeniado": "congeniar",
    "congeniados": "congeniar",
    "congeniamos": "congeniar",
    "congenian": "congeniar",
    "congeniando": "congeniar",
    "congeniara": "congeniar",
    "congeniarais": "congeniar",
    "congeniaran": "congeniar",
    "congeniaras": "congeniar",
    "congeniare": "congeniar",
    "congeniareis": "congeniar",
    "congeniaremos": "congeniar",
    "congeniaren": "congeniar",
    "congeniares": "congeniar",
    "congeniaron": "congeniar",
    "congeniará": "congeniar",
    "congeniarán": "congeniar",
    "congeniarás": "congeniar",
    "congeniaré": "congeniar",
    "congeniaréis": "congeniar",
    "congeniaría": "congeniar",
    "congeniaríais": "congeniar",
    "congeniaríamos": "congeniar",
    "congeniarían": "congeniar",
    "congeniarías": "congeniar",
    "congenias": "congeniar",
    "congeniase": "congeniar",
    "congeniaseis": "congeniar",
    "congeniasen": "congeniar",
    "congeniases": "congeniar",
    "congeniaste": "congeniar",
    "congeniasteis": "congeniar",
    "congenie": "congeniar",
    "congeniemos": "congeniar",
    "congenien": "congeniar",
    "congenies": "congeniar",
    "congenio": "congeniar",
    "congeniábamos": "congeniar",
    "congeniáis": "congeniar",
    "congeniáramos": "congeniar",
    "congeniáremos": "congeniar",
    "congeniásemos": "congeniar",
    "congenié": "congeniar",
    "congeniéis": "congeniar",
    "congenió": "congeniar",
    "congeries": "congerie",
    "congestiona": "congestionar",
    "congestionaba": "congestionar",
    "congestionabais": "congestionar",
    "congestionaban": "congestionar",
    "congestionabas": "congestionar",
    "congestionad": "congestionar",
    "congestionada": "congestionar",
    "congestionadas": "congestionar",
    "congestionado": "congestionar",
    "congestionados": "congestionar",
    "congestionamos": "congestionar",
    "congestionan": "congestionar",
    "congestionando": "congestionar",
    "congestionara": "congestionar",
    "congestionarais": "congestionar",
    "congestionaran": "congestionar",
    "congestionaras": "congestionar",
    "congestionare": "congestionar",
    "congestionareis": "congestionar",
    "congestionaremos": "congestionar",
    "congestionaren": "congestionar",
    "congestionares": "congestionar",
    "congestionaron": "congestionar",
    "congestionará": "congestionar",
    "congestionarán": "congestionar",
    "congestionarás": "congestionar",
    "congestionaré": "congestionar",
    "congestionaréis": "congestionar",
    "congestionaría": "congestionar",
    "congestionaríais": "congestionar",
    "congestionaríamos": "congestionar",
    "congestionarían": "congestionar",
    "congestionarías": "congestionar",
    "congestionas": "congestionar",
    "congestionase": "congestionar",
    "congestionaseis": "congestionar",
    "congestionasen": "congestionar",
    "congestionases": "congestionar",
    "congestionaste": "congestionar",
    "congestionasteis": "congestionar",
    "congestione": "congestionar",
    "congestionemos": "congestionar",
    "congestionen": "congestionar",
    "congestiones": "congestionar",
    "congestiono": "congestionar",
    "congestionábamos": "congestionar",
    "congestionáis": "congestionar",
    "congestionáramos": "congestionar",
    "congestionáremos": "congestionar",
    "congestionásemos": "congestionar",
    "congestioné": "congestionar",
    "congestionéis": "congestionar",
    "congestionó": "congestionar",
    "congestiva": "congestivo",
    "congestivas": "congestivo",
    "congestivos": "congestivo",
    "congiarios": "congiario",
    "congios": "congio",
    "congloba": "conglobar",
    "conglobaba": "conglobar",
    "conglobabais": "conglobar",
    "conglobaban": "conglobar",
    "conglobabas": "conglobar",
    "conglobaciones": "conglobación",
    "conglobad": "conglobar",
    "conglobada": "conglobar",
    "conglobadas": "conglobar",
    "conglobado": "conglobar",
    "conglobados": "conglobar",
    "conglobamos": "conglobar",
    "congloban": "conglobar",
    "conglobando": "conglobar",
    "conglobara": "conglobar",
    "conglobarais": "conglobar",
    "conglobaran": "conglobar",
    "conglobaras": "conglobar",
    "conglobare": "conglobar",
    "conglobareis": "conglobar",
    "conglobaremos": "conglobar",
    "conglobaren": "conglobar",
    "conglobares": "conglobar",
    "conglobaron": "conglobar",
    "conglobará": "conglobar",
    "conglobarán": "conglobar",
    "conglobarás": "conglobar",
    "conglobaré": "conglobar",
    "conglobaréis": "conglobar",
    "conglobaría": "conglobar",
    "conglobaríais": "conglobar",
    "conglobaríamos": "conglobar",
    "conglobarían": "conglobar",
    "conglobarías": "conglobar",
    "conglobas": "conglobar",
    "conglobase": "conglobar",
    "conglobaseis": "conglobar",
    "conglobasen": "conglobar",
    "conglobases": "conglobar",
    "conglobaste": "conglobar",
    "conglobasteis": "conglobar",
    "conglobe": "conglobar",
    "conglobemos": "conglobar",
    "congloben": "conglobar",
    "conglobes": "conglobar",
    "conglobo": "conglobar",
    "conglobábamos": "conglobar",
    "conglobáis": "conglobar",
    "conglobáramos": "conglobar",
    "conglobáremos": "conglobar",
    "conglobásemos": "conglobar",
    "conglobé": "conglobar",
    "conglobéis": "conglobar",
    "conglobó": "conglobar",
    "conglomera": "conglomerar",
    "conglomeraba": "conglomerar",
    "conglomerabais": "conglomerar",
    "conglomeraban": "conglomerar",
    "conglomerabas": "conglomerar",
    "conglomeraciones": "conglomeración",
    "conglomerad": "conglomerar",
    "conglomerada": "conglomerar",
    "conglomeradas": "conglomerar",
    "conglomerado": "conglomerar",
    "conglomerados": "conglomerar",
    "conglomeramos": "conglomerar",
    "conglomeran": "conglomerar",
    "conglomerando": "conglomerar",
    "conglomerantes": "conglomerante",
    "conglomerara": "conglomerar",
    "conglomerarais": "conglomerar",
    "conglomeraran": "conglomerar",
    "conglomeraras": "conglomerar",
    "conglomerare": "conglomerar",
    "conglomerareis": "conglomerar",
    "conglomeraremos": "conglomerar",
    "conglomeraren": "conglomerar",
    "conglomerares": "conglomerar",
    "conglomeraron": "conglomerar",
    "conglomerará": "conglomerar",
    "conglomerarán": "conglomerar",
    "conglomerarás": "conglomerar",
    "conglomeraré": "conglomerar",
    "conglomeraréis": "conglomerar",
    "conglomeraría": "conglomerar",
    "conglomeraríais": "conglomerar",
    "conglomeraríamos": "conglomerar",
    "conglomerarían": "conglomerar",
    "conglomerarías": "conglomerar",
    "conglomeras": "conglomerar",
    "conglomerase": "conglomerar",
    "conglomeraseis": "conglomerar",
    "conglomerasen": "conglomerar",
    "conglomerases": "conglomerar",
    "conglomeraste": "conglomerar",
    "conglomerasteis": "conglomerar",
    "conglomere": "conglomerar",
    "conglomeremos": "conglomerar",
    "conglomeren": "conglomerar",
    "conglomeres": "conglomerar",
    "conglomero": "conglomerar",
    "conglomerábamos": "conglomerar",
    "conglomeráis": "conglomerar",
    "conglomeráramos": "conglomerar",
    "conglomeráremos": "conglomerar",
    "conglomerásemos": "conglomerar",
    "conglomeré": "conglomerar",
    "conglomeréis": "conglomerar",
    "conglomeró": "conglomerar",
    "conglutina": "conglutinar",
    "conglutinaba": "conglutinar",
    "conglutinabais": "conglutinar",
    "conglutinaban": "conglutinar",
    "conglutinabas": "conglutinar",
    "conglutinaciones": "conglutinación",
    "conglutinad": "conglutinar",
    "conglutinada": "conglutinar",
    "conglutinadas": "conglutinar",
    "conglutinado": "conglutinar",
    "conglutinados": "conglutinar",
    "conglutinamos": "conglutinar",
    "conglutinan": "conglutinar",
    "conglutinando": "conglutinar",
    "conglutinantes": "conglutinante",
    "conglutinara": "conglutinar",
    "conglutinarais": "conglutinar",
    "conglutinaran": "conglutinar",
    "conglutinaras": "conglutinar",
    "conglutinare": "conglutinar",
    "conglutinareis": "conglutinar",
    "conglutinaremos": "conglutinar",
    "conglutinaren": "conglutinar",
    "conglutinares": "conglutinar",
    "conglutinaron": "conglutinar",
    "conglutinará": "conglutinar",
    "conglutinarán": "conglutinar",
    "conglutinarás": "conglutinar",
    "conglutinaré": "conglutinar",
    "conglutinaréis": "conglutinar",
    "conglutinaría": "conglutinar",
    "conglutinaríais": "conglutinar",
    "conglutinaríamos": "conglutinar",
    "conglutinarían": "conglutinar",
    "conglutinarías": "conglutinar",
    "conglutinas": "conglutinar",
    "conglutinase": "conglutinar",
    "conglutinaseis": "conglutinar",
    "conglutinasen": "conglutinar",
    "conglutinases": "conglutinar",
    "conglutinaste": "conglutinar",
    "conglutinasteis": "conglutinar",
    "conglutinativa": "conglutinativo",
    "conglutinativas": "conglutinativo",
    "conglutinativos": "conglutinativo",
    "conglutine": "conglutinar",
    "conglutinemos": "conglutinar",
    "conglutinen": "conglutinar",
    "conglutines": "conglutinar",
    "conglutininas": "conglutinina",
    "conglutino": "conglutinar",
    "conglutinábamos": "conglutinar",
    "conglutináis": "conglutinar",
    "conglutináramos": "conglutinar",
    "conglutináremos": "conglutinar",
    "conglutinásemos": "conglutinar",
    "conglutiné": "conglutinar",
    "conglutinéis": "conglutinar",
    "conglutinó": "conglutinar",
    "congoja": "congojar",
    "congojaba": "congojar",
    "congojabais": "congojar",
    "congojaban": "congojar",
    "congojabas": "congojar",
    "congojad": "congojar",
    "congojada": "congojar",
    "congojadas": "congojar",
    "congojado": "congojar",
    "congojados": "congojar",
    "congojamos": "congojar",
    "congojan": "congojar",
    "congojando": "congojar",
    "congojara": "congojar",
    "congojarais": "congojar",
    "congojaran": "congojar",
    "congojaras": "congojar",
    "congojare": "congojar",
    "congojareis": "congojar",
    "congojaremos": "congojar",
    "congojaren": "congojar",
    "congojares": "congojar",
    "congojaron": "congojar",
    "congojará": "congojar",
    "congojarán": "congojar",
    "congojarás": "congojar",
    "congojaré": "congojar",
    "congojaréis": "congojar",
    "congojaría": "congojar",
    "congojaríais": "congojar",
    "congojaríamos": "congojar",
    "congojarían": "congojar",
    "congojarías": "congojar",
    "congojas": "congojar",
    "congojase": "congojar",
    "congojaseis": "congojar",
    "congojasen": "congojar",
    "congojases": "congojar",
    "congojaste": "congojar",
    "congojasteis": "congojar",
    "congoje": "congojar",
    "congojemos": "congojar",
    "congojen": "congojar",
    "congojes": "congojar",
    "congojo": "congojar",
    "congojosa": "congojoso",
    "congojosas": "congojoso",
    "congojosos": "congojoso",
    "congojábamos": "congojar",
    "congojáis": "congojar",
    "congojáramos": "congojar",
    "congojáremos": "congojar",
    "congojásemos": "congojar",
    "congojé": "congojar",
    "congojéis": "congojar",
    "congojó": "congojar",
    "congolesa": "congolés",
    "congolesas": "congolés",
    "congoleses": "congolés",
    "congoleña": "congoleño",
    "congoleñas": "congoleño",
    "congoleños": "congoleño",
    "congostos": "congosto",
    "congracia": "congraciar",
    "congraciaba": "congraciar",
    "congraciabais": "congraciar",
    "congraciaban": "congraciar",
    "congraciabas": "congraciar",
    "congraciad": "congraciar",
    "congraciada": "congraciar",
    "congraciadas": "congraciar",
    "congraciado": "congraciar",
    "congraciados": "congraciar",
    "congraciamientos": "congraciamiento",
    "congraciamos": "congraciar",
    "congracian": "congraciar",
    "congraciando": "congraciar",
    "congraciara": "congraciar",
    "congraciarais": "congraciar",
    "congraciaran": "congraciar",
    "congraciaras": "congraciar",
    "congraciare": "congraciar",
    "congraciareis": "congraciar",
    "congraciaremos": "congraciar",
    "congraciaren": "congraciar",
    "congraciares": "congraciar",
    "congraciaron": "congraciar",
    "congraciará": "congraciar",
    "congraciarán": "congraciar",
    "congraciarás": "congraciar",
    "congraciaré": "congraciar",
    "congraciaréis": "congraciar",
    "congraciaría": "congraciar",
    "congraciaríais": "congraciar",
    "congraciaríamos": "congraciar",
    "congraciarían": "congraciar",
    "congraciarías": "congraciar",
    "congracias": "congraciar",
    "congraciase": "congraciar",
    "congraciaseis": "congraciar",
    "congraciasen": "congraciar",
    "congraciases": "congraciar",
    "congraciaste": "congraciar",
    "congraciasteis": "congraciar",
    "congracie": "congraciar",
    "congraciemos": "congraciar",
    "congracien": "congraciar",
    "congracies": "congraciar",
    "congracio": "congraciar",
    "congraciábamos": "congraciar",
    "congraciáis": "congraciar",
    "congraciáramos": "congraciar",
    "congraciáremos": "congraciar",
    "congraciásemos": "congraciar",
    "congracié": "congraciar",
    "congraciéis": "congraciar",
    "congració": "congraciar",
    "congratula": "congratular",
    "congratulaba": "congratular",
    "congratulabais": "congratular",
    "congratulaban": "congratular",
    "congratulabas": "congratular",
    "congratulaciones": "congratulación",
    "congratulad": "congratular",
    "congratulada": "congratular",
    "congratuladas": "congratular",
    "congratulado": "congratular",
    "congratulados": "congratular",
    "congratulamos": "congratular",
    "congratulan": "congratular",
    "congratulando": "congratular",
    "congratulara": "congratular",
    "congratularais": "congratular",
    "congratularan": "congratular",
    "congratularas": "congratular",
    "congratulare": "congratular",
    "congratulareis": "congratular",
    "congratularemos": "congratular",
    "congratularen": "congratular",
    "congratulares": "congratular",
    "congratularon": "congratular",
    "congratulará": "congratular",
    "congratularán": "congratular",
    "congratularás": "congratular",
    "congratularé": "congratular",
    "congratularéis": "congratular",
    "congratularía": "congratular",
    "congratularíais": "congratular",
    "congratularíamos": "congratular",
    "congratularían": "congratular",
    "congratularías": "congratular",
    "congratulas": "congratular",
    "congratulase": "congratular",
    "congratulaseis": "congratular",
    "congratulasen": "congratular",
    "congratulases": "congratular",
    "congratulaste": "congratular",
    "congratulasteis": "congratular",
    "congratulatoria": "congratulatorio",
    "congratulatorias": "congratulatorio",
    "congratulatorios": "congratulatorio",
    "congratule": "congratular",
    "congratulemos": "congratular",
    "congratulen": "congratular",
    "congratules": "congratular",
    "congratulo": "congratular",
    "congratulábamos": "congratular",
    "congratuláis": "congratular",
    "congratuláramos": "congratular",
    "congratuláremos": "congratular",
    "congratulásemos": "congratular",
    "congratulé": "congratular",
    "congratuléis": "congratular",
    "congratuló": "congratular",
    "congrega": "congregar",
    "congregaba": "congregar",
    "congregabais": "congregar",
    "congregaban": "congregar",
    "congregabas": "congregar",
    "congregacionalismos": "congregacionalismo",
    "congregacionalistas": "congregacionalista",
    "congregaciones": "congregación",
    "congregacionistas": "congregacionista",
    "congregad": "congregar",
    "congregada": "congregar",
    "congregadas": "congregar",
    "congregado": "congregar",
    "congregados": "congregar",
    "congregamos": "congregar",
    "congregan": "congregar",
    "congregando": "congregar",
    "congreganta": "congregante",
    "congregantas": "congregante",
    "congregantes": "congregante",
    "congregara": "congregar",
    "congregarais": "congregar",
    "congregaran": "congregar",
    "congregaras": "congregar",
    "congregare": "congregar",
    "congregareis": "congregar",
    "congregaremos": "congregar",
    "congregaren": "congregar",
    "congregares": "congregar",
    "congregaron": "congregar",
    "congregará": "congregar",
    "congregarán": "congregar",
    "congregarás": "congregar",
    "congregaré": "congregar",
    "congregaréis": "congregar",
    "congregaría": "congregar",
    "congregaríais": "congregar",
    "congregaríamos": "congregar",
    "congregarían": "congregar",
    "congregarías": "congregar",
    "congregas": "congregar",
    "congregase": "congregar",
    "congregaseis": "congregar",
    "congregasen": "congregar",
    "congregases": "congregar",
    "congregaste": "congregar",
    "congregasteis": "congregar",
    "congrego": "congregar",
    "congregue": "congregar",
    "congreguemos": "congregar",
    "congreguen": "congregar",
    "congregues": "congregar",
    "congregué": "congregar",
    "congreguéis": "congregar",
    "congregábamos": "congregar",
    "congregáis": "congregar",
    "congregáramos": "congregar",
    "congregáremos": "congregar",
    "congregásemos": "congregar",
    "congregó": "congregar",
    "congresionales": "congresional",
    "congresiones": "congresión",
    "congresistas": "congresista",
    "congresos": "congreso",
    "congresuales": "congresual",
    "congrios": "congrio",
    "congrua": "congruo",
    "congruas": "congruo",
    "congruencias": "congruencia",
    "congruentes": "congruente",
    "congruismos": "congruismo",
    "congruistas": "congruista",
    "congruos": "congruo",
    "congéneres": "congénere",
    "congénica": "congénico",
    "congénicas": "congénico",
    "congénicos": "congénico",
    "congénita": "congénito",
    "congénitas": "congénito",
    "congénitos": "congénito",
    "conicidades": "conicidad",
    "conicohelicoidales": "conicohelicoidal",
    "conidios": "conidio",
    "coniferales": "coniferal",
    "coniferinas": "coniferina",
    "coniformes": "coniforme",
    "coniinas": "coniina",
    "conimbricenses": "conimbricense",
    "coniotomías": "coniotomía",
    "conirrostra": "conirrostro",
    "conirrostras": "conirrostro",
    "conirrostros": "conirrostro",
    "conivalva": "conivalvo",
    "conivalvas": "conivalvo",
    "conivalvos": "conivalvo",
    "conjetura": "conjeturar",
    "conjeturaba": "conjeturar",
    "conjeturabais": "conjeturar",
    "conjeturaban": "conjeturar",
    "conjeturabas": "conjeturar",
    "conjeturables": "conjeturable",
    "conjeturad": "conjeturar",
    "conjeturada": "conjeturar",
    "conjeturadas": "conjeturar",
    "conjeturado": "conjeturar",
    "conjeturadora": "conjeturador",
    "conjeturadoras": "conjeturador",
    "conjeturadores": "conjeturador",
    "conjeturados": "conjeturar",
    "conjeturales": "conjetural",
    "conjeturamos": "conjeturar",
    "conjeturan": "conjeturar",
    "conjeturando": "conjeturar",
    "conjeturara": "conjeturar",
    "conjeturarais": "conjeturar",
    "conjeturaran": "conjeturar",
    "conjeturaras": "conjeturar",
    "conjeturare": "conjeturar",
    "conjeturareis": "conjeturar",
    "conjeturaremos": "conjeturar",
    "conjeturaren": "conjeturar",
    "conjeturares": "conjeturar",
    "conjeturaron": "conjeturar",
    "conjeturará": "conjeturar",
    "conjeturarán": "conjeturar",
    "conjeturarás": "conjeturar",
    "conjeturaré": "conjeturar",
    "conjeturaréis": "conjeturar",
    "conjeturaría": "conjeturar",
    "conjeturaríais": "conjeturar",
    "conjeturaríamos": "conjeturar",
    "conjeturarían": "conjeturar",
    "conjeturarías": "conjeturar",
    "conjeturas": "conjeturar",
    "conjeturase": "conjeturar",
    "conjeturaseis": "conjeturar",
    "conjeturasen": "conjeturar",
    "conjeturases": "conjeturar",
    "conjeturaste": "conjeturar",
    "conjeturasteis": "conjeturar",
    "conjeture": "conjeturar",
    "conjeturemos": "conjeturar",
    "conjeturen": "conjeturar",
    "conjetures": "conjeturar",
    "conjeturo": "conjeturar",
    "conjeturábamos": "conjeturar",
    "conjeturáis": "conjeturar",
    "conjeturáramos": "conjeturar",
    "conjeturáremos": "conjeturar",
    "conjeturásemos": "conjeturar",
    "conjeturé": "conjeturar",
    "conjeturéis": "conjeturar",
    "conjeturó": "conjeturar",
    "conjueces": "conjuez",
    "conjuga": "conjugar",
    "conjugaba": "conjugar",
    "conjugabais": "conjugar",
    "conjugaban": "conjugar",
    "conjugabas": "conjugar",
    "conjugaciones": "conjugación",
    "conjugad": "conjugar",
    "conjugada": "conjugar",
    "conjugadas": "conjugar",
    "conjugado": "conjugar",
    "conjugados": "conjugar",
    "conjugamos": "conjugar",
    "conjugan": "conjugar",
    "conjugando": "conjugar",
    "conjugara": "conjugar",
    "conjugarais": "conjugar",
    "conjugaran": "conjugar",
    "conjugaras": "conjugar",
    "conjugare": "conjugar",
    "conjugareis": "conjugar",
    "conjugaremos": "conjugar",
    "conjugaren": "conjugar",
    "conjugares": "conjugar",
    "conjugaron": "conjugar",
    "conjugará": "conjugar",
    "conjugarán": "conjugar",
    "conjugarás": "conjugar",
    "conjugaré": "conjugar",
    "conjugaréis": "conjugar",
    "conjugaría": "conjugar",
    "conjugaríais": "conjugar",
    "conjugaríamos": "conjugar",
    "conjugarían": "conjugar",
    "conjugarías": "conjugar",
    "conjugas": "conjugar",
    "conjugase": "conjugar",
    "conjugaseis": "conjugar",
    "conjugasen": "conjugar",
    "conjugases": "conjugar",
    "conjugaste": "conjugar",
    "conjugasteis": "conjugar",
    "conjugo": "conjugar",
    "conjugue": "conjugar",
    "conjuguemos": "conjugar",
    "conjuguen": "conjugar",
    "conjugues": "conjugar",
    "conjugué": "conjugar",
    "conjuguéis": "conjugar",
    "conjugábamos": "conjugar",
    "conjugáis": "conjugar",
    "conjugáramos": "conjugar",
    "conjugáremos": "conjugar",
    "conjugásemos": "conjugar",
    "conjugó": "conjugar",
    "conjunciones": "conjunción",
    "conjunta": "conjunto",
    "conjuntaba": "conjuntar",
    "conjuntabais": "conjuntar",
    "conjuntaban": "conjuntar",
    "conjuntabas": "conjuntar",
    "conjuntad": "conjuntar",
    "conjuntada": "conjuntar",
    "conjuntadas": "conjuntar",
    "conjuntado": "conjuntar",
    "conjuntados": "conjuntar",
    "conjuntamos": "conjuntar",
    "conjuntan": "conjuntar",
    "conjuntando": "conjuntar",
    "conjuntara": "conjuntar",
    "conjuntarais": "conjuntar",
    "conjuntaran": "conjuntar",
    "conjuntaras": "conjuntar",
    "conjuntare": "conjuntar",
    "conjuntareis": "conjuntar",
    "conjuntaremos": "conjuntar",
    "conjuntaren": "conjuntar",
    "conjuntares": "conjuntar",
    "conjuntaron": "conjuntar",
    "conjuntará": "conjuntar",
    "conjuntarán": "conjuntar",
    "conjuntarás": "conjuntar",
    "conjuntaré": "conjuntar",
    "conjuntaréis": "conjuntar",
    "conjuntaría": "conjuntar",
    "conjuntaríais": "conjuntar",
    "conjuntaríamos": "conjuntar",
    "conjuntarían": "conjuntar",
    "conjuntarías": "conjuntar",
    "conjuntas": "conjunto",
    "conjuntase": "conjuntar",
    "conjuntaseis": "conjuntar",
    "conjuntasen": "conjuntar",
    "conjuntases": "conjuntar",
    "conjuntaste": "conjuntar",
    "conjuntasteis": "conjuntar",
    "conjunte": "conjuntar",
    "conjuntemos": "conjuntar",
    "conjunten": "conjuntar",
    "conjuntes": "conjuntar",
    "conjuntiva": "conjuntivo",
    "conjuntivales": "conjuntival",
    "conjuntivas": "conjuntivo",
    "conjuntivodacriocistostomías": "conjuntivodacriocistostomía",
    "conjuntivomas": "conjuntivoma",
    "conjuntivoplastias": "conjuntivoplastia",
    "conjuntivorrinostomías": "conjuntivorrinostomía",
    "conjuntivos": "conjuntivo",
    "conjunto": "conjuntar",
    "conjuntores": "conjuntor",
    "conjuntos": "conjunto",
    "conjuntábamos": "conjuntar",
    "conjuntáis": "conjuntar",
    "conjuntáramos": "conjuntar",
    "conjuntáremos": "conjuntar",
    "conjuntásemos": "conjuntar",
    "conjunté": "conjuntar",
    "conjuntéis": "conjuntar",
    "conjuntó": "conjuntar",
    "conjura": "conjurar",
    "conjuraba": "conjurar",
    "conjurabais": "conjurar",
    "conjuraban": "conjurar",
    "conjurabas": "conjurar",
    "conjuraciones": "conjuración",
    "conjurad": "conjurar",
    "conjurada": "conjurar",
    "conjuradas": "conjurar",
    "conjurado": "conjurar",
    "conjurados": "conjurar",
    "conjuramenta": "conjuramentar",
    "conjuramentaba": "conjuramentar",
    "conjuramentabais": "conjuramentar",
    "conjuramentaban": "conjuramentar",
    "conjuramentabas": "conjuramentar",
    "conjuramentad": "conjuramentar",
    "conjuramentada": "conjuramentar",
    "conjuramentadas": "conjuramentar",
    "conjuramentado": "conjuramentar",
    "conjuramentados": "conjuramentar",
    "conjuramentamos": "conjuramentar",
    "conjuramentan": "conjuramentar",
    "conjuramentando": "conjuramentar",
    "conjuramentara": "conjuramentar",
    "conjuramentarais": "conjuramentar",
    "conjuramentaran": "conjuramentar",
    "conjuramentaras": "conjuramentar",
    "conjuramentare": "conjuramentar",
    "conjuramentareis": "conjuramentar",
    "conjuramentaremos": "conjuramentar",
    "conjuramentaren": "conjuramentar",
    "conjuramentares": "conjuramentar",
    "conjuramentaron": "conjuramentar",
    "conjuramentará": "conjuramentar",
    "conjuramentarán": "conjuramentar",
    "conjuramentarás": "conjuramentar",
    "conjuramentaré": "conjuramentar",
    "conjuramentaréis": "conjuramentar",
    "conjuramentaría": "conjuramentar",
    "conjuramentaríais": "conjuramentar",
    "conjuramentaríamos": "conjuramentar",
    "conjuramentarían": "conjuramentar",
    "conjuramentarías": "conjuramentar",
    "conjuramentas": "conjuramentar",
    "conjuramentase": "conjuramentar",
    "conjuramentaseis": "conjuramentar",
    "conjuramentasen": "conjuramentar",
    "conjuramentases": "conjuramentar",
    "conjuramentaste": "conjuramentar",
    "conjuramentasteis": "conjuramentar",
    "conjuramente": "conjuramentar",
    "conjuramentemos": "conjuramentar",
    "conjuramenten": "conjuramentar",
    "conjuramentes": "conjuramentar",
    "conjuramento": "conjuramentar",
    "conjuramentábamos": "conjuramentar",
    "conjuramentáis": "conjuramentar",
    "conjuramentáramos": "conjuramentar",
    "conjuramentáremos": "conjuramentar",
    "conjuramentásemos": "conjuramentar",
    "conjuramenté": "conjuramentar",
    "conjuramentéis": "conjuramentar",
    "conjuramentó": "conjuramentar",
    "conjuramos": "conjurar",
    "conjuran": "conjurar",
    "conjurando": "conjurar",
    "conjurara": "conjurar",
    "conjurarais": "conjurar",
    "conjuraran": "conjurar",
    "conjuraras": "conjurar",
    "conjurare": "conjurar",
    "conjurareis": "conjurar",
    "conjuraremos": "conjurar",
    "conjuraren": "conjurar",
    "conjurares": "conjurar",
    "conjuraron": "conjurar",
    "conjurará": "conjurar",
    "conjurarán": "conjurar",
    "conjurarás": "conjurar",
    "conjuraré": "conjurar",
    "conjuraréis": "conjurar",
    "conjuraría": "conjurar",
    "conjuraríais": "conjurar",
    "conjuraríamos": "conjurar",
    "conjurarían": "conjurar",
    "conjurarías": "conjurar",
    "conjuras": "conjurar",
    "conjurase": "conjurar",
    "conjuraseis": "conjurar",
    "conjurasen": "conjurar",
    "conjurases": "conjurar",
    "conjuraste": "conjurar",
    "conjurasteis": "conjurar",
    "conjure": "conjurar",
    "conjuremos": "conjurar",
    "conjuren": "conjurar",
    "conjures": "conjurar",
    "conjuro": "conjurar",
    "conjuros": "conjuro",
    "conjurábamos": "conjurar",
    "conjuráis": "conjurar",
    "conjuráramos": "conjurar",
    "conjuráremos": "conjurar",
    "conjurásemos": "conjurar",
    "conjuré": "conjurar",
    "conjuréis": "conjurar",
    "conjuró": "conjurar",
    "conlleva": "conllevar",
    "conllevaba": "conllevar",
    "conllevabais": "conllevar",
    "conllevaban": "conllevar",
    "conllevabas": "conllevar",
    "conllevad": "conllevar",
    "conllevada": "conllevar",
    "conllevadas": "conllevar",
    "conllevado": "conllevar",
    "conllevados": "conllevar",
    "conllevamos": "conllevar",
    "conllevan": "conllevar",
    "conllevando": "conllevar",
    "conllevara": "conllevar",
    "conllevarais": "conllevar",
    "conllevaran": "conllevar",
    "conllevaras": "conllevar",
    "conllevare": "conllevar",
    "conllevareis": "conllevar",
    "conllevaremos": "conllevar",
    "conllevaren": "conllevar",
    "conllevares": "conllevar",
    "conllevaron": "conllevar",
    "conllevará": "conllevar",
    "conllevarán": "conllevar",
    "conllevarás": "conllevar",
    "conllevaré": "conllevar",
    "conllevaréis": "conllevar",
    "conllevaría": "conllevar",
    "conllevaríais": "conllevar",
    "conllevaríamos": "conllevar",
    "conllevarían": "conllevar",
    "conllevarías": "conllevar",
    "conllevas": "conllevar",
    "conllevase": "conllevar",
    "conllevaseis": "conllevar",
    "conllevasen": "conllevar",
    "conllevases": "conllevar",
    "conllevaste": "conllevar",
    "conllevasteis": "conllevar",
    "conlleve": "conllevar",
    "conllevemos": "conllevar",
    "conlleven": "conllevar",
    "conlleves": "conllevar",
    "conllevo": "conllevar",
    "conllevábamos": "conllevar",
    "conlleváis": "conllevar",
    "conlleváramos": "conllevar",
    "conlleváremos": "conllevar",
    "conllevásemos": "conllevar",
    "conllevé": "conllevar",
    "conllevéis": "conllevar",
    "conllevó": "conllevar",
    "conmemora": "conmemorar",
    "conmemoraba": "conmemorar",
    "conmemorabais": "conmemorar",
    "conmemoraban": "conmemorar",
    "conmemorabas": "conmemorar",
    "conmemoraciones": "conmemoración",
    "conmemorad": "conmemorar",
    "conmemorada": "conmemorar",
    "conmemoradas": "conmemorar",
    "conmemorado": "conmemorar",
    "conmemorados": "conmemorar",
    "conmemoramos": "conmemorar",
    "conmemoran": "conmemorar",
    "conmemorando": "conmemorar",
    "conmemorara": "conmemorar",
    "conmemorarais": "conmemorar",
    "conmemoraran": "conmemorar",
    "conmemoraras": "conmemorar",
    "conmemorare": "conmemorar",
    "conmemorareis": "conmemorar",
    "conmemoraremos": "conmemorar",
    "conmemoraren": "conmemorar",
    "conmemorares": "conmemorar",
    "conmemoraron": "conmemorar",
    "conmemorará": "conmemorar",
    "conmemorarán": "conmemorar",
    "conmemorarás": "conmemorar",
    "conmemoraré": "conmemorar",
    "conmemoraréis": "conmemorar",
    "conmemoraría": "conmemorar",
    "conmemoraríais": "conmemorar",
    "conmemoraríamos": "conmemorar",
    "conmemorarían": "conmemorar",
    "conmemorarías": "conmemorar",
    "conmemoras": "conmemorar",
    "conmemorase": "conmemorar",
    "conmemoraseis": "conmemorar",
    "conmemorasen": "conmemorar",
    "conmemorases": "conmemorar",
    "conmemoraste": "conmemorar",
    "conmemorasteis": "conmemorar",
    "conmemorativa": "conmemorativo",
    "conmemorativas": "conmemorativo",
    "conmemorativos": "conmemorativo",
    "conmemore": "conmemorar",
    "conmemoremos": "conmemorar",
    "conmemoren": "conmemorar",
    "conmemores": "conmemorar",
    "conmemoro": "conmemorar",
    "conmemorábamos": "conmemorar",
    "conmemoráis": "conmemorar",
    "conmemoráramos": "conmemorar",
    "conmemoráremos": "conmemorar",
    "conmemorásemos": "conmemorar",
    "conmemoré": "conmemorar",
    "conmemoréis": "conmemorar",
    "conmemoró": "conmemorar",
    "conmensura": "conmensurar",
    "conmensuraba": "conmensurar",
    "conmensurabais": "conmensurar",
    "conmensuraban": "conmensurar",
    "conmensurabas": "conmensurar",
    "conmensurabilidades": "conmensurabilidad",
    "conmensurables": "conmensurable",
    "conmensuraciones": "conmensuración",
    "conmensurad": "conmensurar",
    "conmensurada": "conmensurar",
    "conmensuradas": "conmensurar",
    "conmensurado": "conmensurar",
    "conmensurados": "conmensurar",
    "conmensuramos": "conmensurar",
    "conmensuran": "conmensurar",
    "conmensurando": "conmensurar",
    "conmensurara": "conmensurar",
    "conmensurarais": "conmensurar",
    "conmensuraran": "conmensurar",
    "conmensuraras": "conmensurar",
    "conmensurare": "conmensurar",
    "conmensurareis": "conmensurar",
    "conmensuraremos": "conmensurar",
    "conmensuraren": "conmensurar",
    "conmensurares": "conmensurar",
    "conmensuraron": "conmensurar",
    "conmensurará": "conmensurar",
    "conmensurarán": "conmensurar",
    "conmensurarás": "conmensurar",
    "conmensuraré": "conmensurar",
    "conmensuraréis": "conmensurar",
    "conmensuraría": "conmensurar",
    "conmensuraríais": "conmensurar",
    "conmensuraríamos": "conmensurar",
    "conmensurarían": "conmensurar",
    "conmensurarías": "conmensurar",
    "conmensuras": "conmensurar",
    "conmensurase": "conmensurar",
    "conmensuraseis": "conmensurar",
    "conmensurasen": "conmensurar",
    "conmensurases": "conmensurar",
    "conmensuraste": "conmensurar",
    "conmensurasteis": "conmensurar",
    "conmensurativa": "conmensurativo",
    "conmensurativas": "conmensurativo",
    "conmensurativos": "conmensurativo",
    "conmensure": "conmensurar",
    "conmensuremos": "conmensurar",
    "conmensuren": "conmensurar",
    "conmensures": "conmensurar",
    "conmensuro": "conmensurar",
    "conmensurábamos": "conmensurar",
    "conmensuráis": "conmensurar",
    "conmensuráramos": "conmensurar",
    "conmensuráremos": "conmensurar",
    "conmensurásemos": "conmensurar",
    "conmensuré": "conmensurar",
    "conmensuréis": "conmensurar",
    "conmensuró": "conmensurar",
    "conmilitones": "conmilitón",
    "conmina": "conminar",
    "conminaba": "conminar",
    "conminabais": "conminar",
    "conminaban": "conminar",
    "conminabas": "conminar",
    "conminaciones": "conminación",
    "conminad": "conminar",
    "conminada": "conminar",
    "conminadas": "conminar",
    "conminado": "conminar",
    "conminados": "conminar",
    "conminamos": "conminar",
    "conminan": "conminar",
    "conminando": "conminar",
    "conminara": "conminar",
    "conminarais": "conminar",
    "conminaran": "conminar",
    "conminaras": "conminar",
    "conminare": "conminar",
    "conminareis": "conminar",
    "conminaremos": "conminar",
    "conminaren": "conminar",
    "conminares": "conminar",
    "conminaron": "conminar",
    "conminará": "conminar",
    "conminarán": "conminar",
    "conminarás": "conminar",
    "conminaré": "conminar",
    "conminaréis": "conminar",
    "conminaría": "conminar",
    "conminaríais": "conminar",
    "conminaríamos": "conminar",
    "conminarían": "conminar",
    "conminarías": "conminar",
    "conminas": "conminar",
    "conminase": "conminar",
    "conminaseis": "conminar",
    "conminasen": "conminar",
    "conminases": "conminar",
    "conminaste": "conminar",
    "conminasteis": "conminar",
    "conminativa": "conminativo",
    "conminativas": "conminativo",
    "conminativos": "conminativo",
    "conminatoria": "conminatorio",
    "conminatorias": "conminatorio",
    "conminatorios": "conminatorio",
    "conmine": "conminar",
    "conminemos": "conminar",
    "conminen": "conminar",
    "conmines": "conminar",
    "conmino": "conminar",
    "conminutas": "conminuta",
    "conminábamos": "conminar",
    "conmináis": "conminar",
    "conmináramos": "conminar",
    "conmináremos": "conminar",
    "conminásemos": "conminar",
    "conminé": "conminar",
    "conminéis": "conminar",
    "conminó": "conminar",
    "conmiseraciones": "conmiseración",
    "conmista": "conmisto",
    "conmistas": "conmisto",
    "conmistiones": "conmistión",
    "conmistos": "conmisto",
    "conmixta": "conmixto",
    "conmixtas": "conmixto",
    "conmixtiones": "conmixtión",
    "conmixtos": "conmixto",
    "conmociona": "conmocionar",
    "conmocionaba": "conmocionar",
    "conmocionabais": "conmocionar",
    "conmocionaban": "conmocionar",
    "conmocionabas": "conmocionar",
    "conmocionad": "conmocionar",
    "conmocionada": "conmocionar",
    "conmocionadas": "conmocionar",
    "conmocionado": "conmocionar",
    "conmocionados": "conmocionar",
    "conmocionamos": "conmocionar",
    "conmocionan": "conmocionar",
    "conmocionando": "conmocionar",
    "conmocionara": "conmocionar",
    "conmocionarais": "conmocionar",
    "conmocionaran": "conmocionar",
    "conmocionaras": "conmocionar",
    "conmocionare": "conmocionar",
    "conmocionareis": "conmocionar",
    "conmocionaremos": "conmocionar",
    "conmocionaren": "conmocionar",
    "conmocionares": "conmocionar",
    "conmocionaron": "conmocionar",
    "conmocionará": "conmocionar",
    "conmocionarán": "conmocionar",
    "conmocionarás": "conmocionar",
    "conmocionaré": "conmocionar",
    "conmocionaréis": "conmocionar",
    "conmocionaría": "conmocionar",
    "conmocionaríais": "conmocionar",
    "conmocionaríamos": "conmocionar",
    "conmocionarían": "conmocionar",
    "conmocionarías": "conmocionar",
    "conmocionas": "conmocionar",
    "conmocionase": "conmocionar",
    "conmocionaseis": "conmocionar",
    "conmocionasen": "conmocionar",
    "conmocionases": "conmocionar",
    "conmocionaste": "conmocionar",
    "conmocionasteis": "conmocionar",
    "conmocione": "conmocionar",
    "conmocionemos": "conmocionar",
    "conmocionen": "conmocionar",
    "conmociones": "conmocionar",
    "conmociono": "conmocionar",
    "conmocionábamos": "conmocionar",
    "conmocionáis": "conmocionar",
    "conmocionáramos": "conmocionar",
    "conmocionáremos": "conmocionar",
    "conmocionásemos": "conmocionar",
    "conmocioné": "conmocionar",
    "conmocionéis": "conmocionar",
    "conmocionó": "conmocionar",
    "conmonitorios": "conmonitorio",
    "conmoraciones": "conmoración",
    "conmovamos": "conmover",
    "conmoved": "conmover",
    "conmovedora": "conmovedor",
    "conmovedoras": "conmovedor",
    "conmovedores": "conmovedor",
    "conmovemos": "conmover",
    "conmoveremos": "conmover",
    "conmoverá": "conmover",
    "conmoverán": "conmover",
    "conmoverás": "conmover",
    "conmoveré": "conmover",
    "conmoveréis": "conmover",
    "conmovería": "conmover",
    "conmoveríais": "conmover",
    "conmoveríamos": "conmover",
    "conmoverían": "conmover",
    "conmoverías": "conmover",
    "conmovida": "conmover",
    "conmovidas": "conmover",
    "conmovido": "conmover",
    "conmovidos": "conmover",
    "conmoviendo": "conmover",
    "conmoviera": "conmover",
    "conmovierais": "conmover",
    "conmovieran": "conmover",
    "conmovieras": "conmover",
    "conmoviere": "conmover",
    "conmoviereis": "conmover",
    "conmovieren": "conmover",
    "conmovieres": "conmover",
    "conmovieron": "conmover",
    "conmoviese": "conmover",
    "conmovieseis": "conmover",
    "conmoviesen": "conmover",
    "conmovieses": "conmover",
    "conmovimos": "conmover",
    "conmoviste": "conmover",
    "conmovisteis": "conmover",
    "conmoviéramos": "conmover",
    "conmoviéremos": "conmover",
    "conmoviésemos": "conmover",
    "conmovió": "conmover",
    "conmováis": "conmover",
    "conmovéis": "conmover",
    "conmoví": "conmover",
    "conmovía": "conmover",
    "conmovíais": "conmover",
    "conmovíamos": "conmover",
    "conmovían": "conmover",
    "conmovías": "conmover",
    "conmueva": "conmover",
    "conmuevan": "conmover",
    "conmuevas": "conmover",
    "conmueve": "conmover",
    "conmueven": "conmover",
    "conmueves": "conmover",
    "conmuevo": "conmover",
    "conmuta": "conmutar",
    "conmutaba": "conmutar",
    "conmutabais": "conmutar",
    "conmutaban": "conmutar",
    "conmutabas": "conmutar",
    "conmutabilidades": "conmutabilidad",
    "conmutables": "conmutable",
    "conmutaciones": "conmutación",
    "conmutad": "conmutar",
    "conmutada": "conmutar",
    "conmutadas": "conmutar",
    "conmutado": "conmutar",
    "conmutadora": "conmutador",
    "conmutadoras": "conmutador",
    "conmutadores": "conmutador",
    "conmutados": "conmutar",
    "conmutamos": "conmutar",
    "conmutan": "conmutar",
    "conmutando": "conmutar",
    "conmutara": "conmutar",
    "conmutarais": "conmutar",
    "conmutaran": "conmutar",
    "conmutaras": "conmutar",
    "conmutare": "conmutar",
    "conmutareis": "conmutar",
    "conmutaremos": "conmutar",
    "conmutaren": "conmutar",
    "conmutares": "conmutar",
    "conmutaron": "conmutar",
    "conmutará": "conmutar",
    "conmutarán": "conmutar",
    "conmutarás": "conmutar",
    "conmutaré": "conmutar",
    "conmutaréis": "conmutar",
    "conmutaría": "conmutar",
    "conmutaríais": "conmutar",
    "conmutaríamos": "conmutar",
    "conmutarían": "conmutar",
    "conmutarías": "conmutar",
    "conmutas": "conmutar",
    "conmutase": "conmutar",
    "conmutaseis": "conmutar",
    "conmutasen": "conmutar",
    "conmutases": "conmutar",
    "conmutaste": "conmutar",
    "conmutasteis": "conmutar",
    "conmutativa": "conmutativo",
    "conmutativas": "conmutativo",
    "conmutatividades": "conmutatividad",
    "conmutativos": "conmutativo",
    "conmutatrices": "conmutatriz",
    "conmute": "conmutar",
    "conmutemos": "conmutar",
    "conmuten": "conmutar",
    "conmutes": "conmutar",
    "conmuto": "conmutar",
    "conmutábamos": "conmutar",
    "conmutáis": "conmutar",
    "conmutáramos": "conmutar",
    "conmutáremos": "conmutar",
    "conmutásemos": "conmutar",
    "conmuté": "conmutar",
    "conmutéis": "conmutar",
    "conmutó": "conmutar",
    "connaturales": "connatural",
    "connaturalice": "connaturalizar",
    "connaturalicemos": "connaturalizar",
    "connaturalicen": "connaturalizar",
    "connaturalices": "connaturalizar",
    "connaturalicé": "connaturalizar",
    "connaturalicéis": "connaturalizar",
    "connaturaliza": "connaturalizar",
    "connaturalizaba": "connaturalizar",
    "connaturalizabais": "connaturalizar",
    "connaturalizaban": "connaturalizar",
    "connaturalizabas": "connaturalizar",
    "connaturalizaciones": "connaturalización",
    "connaturalizad": "connaturalizar",
    "connaturalizada": "connaturalizar",
    "connaturalizadas": "connaturalizar",
    "connaturalizado": "connaturalizar",
    "connaturalizados": "connaturalizar",
    "connaturalizamos": "connaturalizar",
    "connaturalizan": "connaturalizar",
    "connaturalizando": "connaturalizar",
    "connaturalizara": "connaturalizar",
    "connaturalizarais": "connaturalizar",
    "connaturalizaran": "connaturalizar",
    "connaturalizaras": "connaturalizar",
    "connaturalizare": "connaturalizar",
    "connaturalizareis": "connaturalizar",
    "connaturalizaremos": "connaturalizar",
    "connaturalizaren": "connaturalizar",
    "connaturalizares": "connaturalizar",
    "connaturalizaron": "connaturalizar",
    "connaturalizará": "connaturalizar",
    "connaturalizarán": "connaturalizar",
    "connaturalizarás": "connaturalizar",
    "connaturalizaré": "connaturalizar",
    "connaturalizaréis": "connaturalizar",
    "connaturalizaría": "connaturalizar",
    "connaturalizaríais": "connaturalizar",
    "connaturalizaríamos": "connaturalizar",
    "connaturalizarían": "connaturalizar",
    "connaturalizarías": "connaturalizar",
    "connaturalizas": "connaturalizar",
    "connaturalizase": "connaturalizar",
    "connaturalizaseis": "connaturalizar",
    "connaturalizasen": "connaturalizar",
    "connaturalizases": "connaturalizar",
    "connaturalizaste": "connaturalizar",
    "connaturalizasteis": "connaturalizar",
    "connaturalizo": "connaturalizar",
    "connaturalizábamos": "connaturalizar",
    "connaturalizáis": "connaturalizar",
    "connaturalizáramos": "connaturalizar",
    "connaturalizáremos": "connaturalizar",
    "connaturalizásemos": "connaturalizar",
    "connaturalizó": "connaturalizar",
    "connivencias": "connivencia",
    "conniventes": "connivente",
    "connota": "connotar",
    "connotaba": "connotar",
    "connotabais": "connotar",
    "connotaban": "connotar",
    "connotabas": "connotar",
    "connotaciones": "connotación",
    "connotad": "connotar",
    "connotada": "connotar",
    "connotadas": "connotar",
    "connotado": "connotar",
    "connotados": "connotar",
    "connotamos": "connotar",
    "connotan": "connotar",
    "connotando": "connotar",
    "connotara": "connotar",
    "connotarais": "connotar",
    "connotaran": "connotar",
    "connotaras": "connotar",
    "connotare": "connotar",
    "connotareis": "connotar",
    "connotaremos": "connotar",
    "connotaren": "connotar",
    "connotares": "connotar",
    "connotaron": "connotar",
    "connotará": "connotar",
    "connotarán": "connotar",
    "connotarás": "connotar",
    "connotaré": "connotar",
    "connotaréis": "connotar",
    "connotaría": "connotar",
    "connotaríais": "connotar",
    "connotaríamos": "connotar",
    "connotarían": "connotar",
    "connotarías": "connotar",
    "connotas": "connotar",
    "connotase": "connotar",
    "connotaseis": "connotar",
    "connotasen": "connotar",
    "connotases": "connotar",
    "connotaste": "connotar",
    "connotasteis": "connotar",
    "connotativa": "connotativo",
    "connotativas": "connotativo",
    "connotativos": "connotativo",
    "connote": "connotar",
    "connotemos": "connotar",
    "connoten": "connotar",
    "connotes": "connotar",
    "connoto": "connotar",
    "connotábamos": "connotar",
    "connotáis": "connotar",
    "connotáramos": "connotar",
    "connotáremos": "connotar",
    "connotásemos": "connotar",
    "connoté": "connotar",
    "connotéis": "connotar",
    "connotó": "connotar",
    "connubios": "connubio",
    "conoce": "conocer",
    "conoced": "conocer",
    "conocedora": "conocedor",
    "conocedoras": "conocedor",
    "conocedores": "conocedor",
    "conocemos": "conocer",
    "conocen": "conocer",
    "conoceremos": "conocer",
    "conocerá": "conocer",
    "conocerán": "conocer",
    "conocerás": "conocer",
    "conoceré": "conocer",
    "conoceréis": "conocer",
    "conocería": "conocer",
    "conoceríais": "conocer",
    "conoceríamos": "conocer",
    "conocerían": "conocer",
    "conocerías": "conocer",
    "conoces": "conocer",
    "conocibles": "conocible",
    "conocida": "conocido",
    "conocidas": "conocido",
    "conocido": "conocer",
    "conocidos": "conocido",
    "conociendo": "conocer",
    "conociera": "conocer",
    "conocierais": "conocer",
    "conocieran": "conocer",
    "conocieras": "conocer",
    "conociere": "conocer",
    "conociereis": "conocer",
    "conocieren": "conocer",
    "conocieres": "conocer",
    "conocieron": "conocer",
    "conociese": "conocer",
    "conocieseis": "conocer",
    "conociesen": "conocer",
    "conocieses": "conocer",
    "conocimientos": "conocimiento",
    "conocimos": "conocer",
    "conociste": "conocer",
    "conocisteis": "conocer",
    "conociéramos": "conocer",
    "conociéremos": "conocer",
    "conociésemos": "conocer",
    "conoció": "conocer",
    "conocéis": "conocer",
    "conocí": "conocer",
    "conocía": "conocer",
    "conocíais": "conocer",
    "conocíamos": "conocer",
    "conocían": "conocer",
    "conocías": "conocer",
    "conoidales": "conoidal",
    "conoidea": "conoideo",
    "conoideas": "conoideo",
    "conoideos": "conoideo",
    "conoides": "conoide",
    "conopeos": "conopeo",
    "conopiales": "conopial",
    "conos": "cono",
    "conovedora": "conovedor",
    "conovedoras": "conovedor",
    "conovedores": "conovedor",
    "conozca": "conocer",
    "conozcamos": "conocer",
    "conozcan": "conocer",
    "conozcas": "conocer",
    "conozco": "conocer",
    "conozcáis": "conocer",
    "conquenses": "conquense",
    "conquiliformes": "conquiliforme",
    "conquiliologías": "conquiliología",
    "conquilióloga": "conquiliólogo",
    "conquiliólogas": "conquiliólogo",
    "conquiliólogos": "conquiliólogo",
    "conquiolinas": "conquiolina",
    "conquista": "conquistar",
    "conquistaba": "conquistar",
    "conquistabais": "conquistar",
    "conquistaban": "conquistar",
    "conquistabas": "conquistar",
    "conquistables": "conquistable",
    "conquistad": "conquistar",
    "conquistada": "conquistar",
    "conquistadas": "conquistar",
    "conquistado": "conquistar",
    "conquistadora": "conquistador",
    "conquistadoras": "conquistador",
    "conquistadores": "conquistador",
    "conquistados": "conquistar",
    "conquistamos": "conquistar",
    "conquistan": "conquistar",
    "conquistando": "conquistar",
    "conquistara": "conquistar",
    "conquistarais": "conquistar",
    "conquistaran": "conquistar",
    "conquistaras": "conquistar",
    "conquistare": "conquistar",
    "conquistareis": "conquistar",
    "conquistaremos": "conquistar",
    "conquistaren": "conquistar",
    "conquistares": "conquistar",
    "conquistaron": "conquistar",
    "conquistará": "conquistar",
    "conquistarán": "conquistar",
    "conquistarás": "conquistar",
    "conquistaré": "conquistar",
    "conquistaréis": "conquistar",
    "conquistaría": "conquistar",
    "conquistaríais": "conquistar",
    "conquistaríamos": "conquistar",
    "conquistarían": "conquistar",
    "conquistarías": "conquistar",
    "conquistas": "conquistar",
    "conquistase": "conquistar",
    "conquistaseis": "conquistar",
    "conquistasen": "conquistar",
    "conquistases": "conquistar",
    "conquistaste": "conquistar",
    "conquistasteis": "conquistar",
    "conquiste": "conquistar",
    "conquistemos": "conquistar",
    "conquisten": "conquistar",
    "conquistes": "conquistar",
    "conquisto": "conquistar",
    "conquistábamos": "conquistar",
    "conquistáis": "conquistar",
    "conquistáramos": "conquistar",
    "conquistáremos": "conquistar",
    "conquistásemos": "conquistar",
    "conquisté": "conquistar",
    "conquistéis": "conquistar",
    "conquistó": "conquistar",
    "conreos": "conreo",
    "consabida": "consabido",
    "consabidas": "consabido",
    "consabidos": "consabido",
    "consagra": "consagrar",
    "consagraba": "consagrar",
    "consagrabais": "consagrar",
    "consagraban": "consagrar",
    "consagrabas": "consagrar",
    "consagrables": "consagrable",
    "consagraciones": "consagración",
    "consagrad": "consagrar",
    "consagrada": "consagrar",
    "consagradas": "consagrar",
    "consagrado": "consagrar",
    "consagrados": "consagrar",
    "consagramos": "consagrar",
    "consagran": "consagrar",
    "consagrando": "consagrar",
    "consagrara": "consagrar",
    "consagrarais": "consagrar",
    "consagraran": "consagrar",
    "consagraras": "consagrar",
    "consagrare": "consagrar",
    "consagrareis": "consagrar",
    "consagraremos": "consagrar",
    "consagraren": "consagrar",
    "consagrares": "consagrar",
    "consagraron": "consagrar",
    "consagrará": "consagrar",
    "consagrarán": "consagrar",
    "consagrarás": "consagrar",
    "consagraré": "consagrar",
    "consagraréis": "consagrar",
    "consagraría": "consagrar",
    "consagraríais": "consagrar",
    "consagraríamos": "consagrar",
    "consagrarían": "consagrar",
    "consagrarías": "consagrar",
    "consagras": "consagrar",
    "consagrase": "consagrar",
    "consagraseis": "consagrar",
    "consagrasen": "consagrar",
    "consagrases": "consagrar",
    "consagraste": "consagrar",
    "consagrasteis": "consagrar",
    "consagratoria": "consagratorio",
    "consagratorias": "consagratorio",
    "consagratorios": "consagratorio",
    "consagre": "consagrar",
    "consagremos": "consagrar",
    "consagren": "consagrar",
    "consagres": "consagrar",
    "consagro": "consagrar",
    "consagrábamos": "consagrar",
    "consagráis": "consagrar",
    "consagráramos": "consagrar",
    "consagráremos": "consagrar",
    "consagrásemos": "consagrar",
    "consagré": "consagrar",
    "consagréis": "consagrar",
    "consagró": "consagrar",
    "consanguineidades": "consanguineidad",
    "consanguinidades": "consanguinidad",
    "consanguínea": "consanguíneo",
    "consanguíneas": "consanguíneo",
    "consanguíneos": "consanguíneo",
    "consciencias": "consciencia",
    "conscientes": "consciente",
    "conscripciones": "conscripción",
    "conscriptos": "conscripto",
    "consectaria": "consectario",
    "consectarias": "consectario",
    "consectarios": "consectario",
    "consecuciones": "consecución",
    "consecuencias": "consecuencia",
    "consecuentes": "consecuente",
    "consecutiva": "consecutivo",
    "consecutivas": "consecutivo",
    "consecutivos": "consecutivo",
    "conseguid": "conseguir",
    "conseguida": "conseguir",
    "conseguidas": "conseguir",
    "conseguido": "conseguir",
    "conseguidos": "conseguir",
    "conseguimientos": "conseguimiento",
    "conseguimos": "conseguir",
    "conseguiremos": "conseguir",
    "conseguirá": "conseguir",
    "conseguirán": "conseguir",
    "conseguirás": "conseguir",
    "conseguiré": "conseguir",
    "conseguiréis": "conseguir",
    "conseguiría": "conseguir",
    "conseguiríais": "conseguir",
    "conseguiríamos": "conseguir",
    "conseguirían": "conseguir",
    "conseguirías": "conseguir",
    "conseguiste": "conseguir",
    "conseguisteis": "conseguir",
    "conseguí": "conseguir",
    "conseguía": "conseguir",
    "conseguíais": "conseguir",
    "conseguíamos": "conseguir",
    "conseguían": "conseguir",
    "conseguías": "conseguir",
    "conseguís": "conseguir",
    "consejas": "conseja",
    "consejera": "consejero",
    "consejeras": "consejero",
    "consejeros": "consejero",
    "consejerías": "consejería",
    "consejos": "consejo",
    "conselleres": "conseller",
    "consellers": "conseller",
    "consensos": "consenso",
    "consensua": "consensuar",
    "consensuaba": "consensuar",
    "consensuabais": "consensuar",
    "consensuaban": "consensuar",
    "consensuabas": "consensuar",
    "consensuad": "consensuar",
    "consensuada": "consensuar",
    "consensuadas": "consensuar",
    "consensuado": "consensuar",
    "consensuados": "consensuar",
    "consensuales": "consensual",
    "consensuamos": "consensuar",
    "consensuan": "consensuar",
    "consensuando": "consensuar",
    "consensuara": "consensuar",
    "consensuarais": "consensuar",
    "consensuaran": "consensuar",
    "consensuaras": "consensuar",
    "consensuare": "consensuar",
    "consensuareis": "consensuar",
    "consensuaremos": "consensuar",
    "consensuaren": "consensuar",
    "consensuares": "consensuar",
    "consensuaron": "consensuar",
    "consensuará": "consensuar",
    "consensuarán": "consensuar",
    "consensuarás": "consensuar",
    "consensuaré": "consensuar",
    "consensuaréis": "consensuar",
    "consensuaría": "consensuar",
    "consensuaríais": "consensuar",
    "consensuaríamos": "consensuar",
    "consensuarían": "consensuar",
    "consensuarías": "consensuar",
    "consensuas": "consensuar",
    "consensuase": "consensuar",
    "consensuaseis": "consensuar",
    "consensuasen": "consensuar",
    "consensuases": "consensuar",
    "consensuaste": "consensuar",
    "consensuasteis": "consensuar",
    "consensue": "consensuar",
    "consensuemos": "consensuar",
    "consensuen": "consensuar",
    "consensues": "consensuar",
    "consensuo": "consensuar",
    "consensuábamos": "consensuar",
    "consensuáis": "consensuar",
    "consensuáramos": "consensuar",
    "consensuáremos": "consensuar",
    "consensuásemos": "consensuar",
    "consensué": "consensuar",
    "consensuéis": "consensuar",
    "consensuó": "consensuar",
    "consentid": "consentir",
    "consentida": "consentir",
    "consentidas": "consentir",
    "consentido": "consentir",
    "consentidora": "consentidor",
    "consentidoras": "consentidor",
    "consentidores": "consentidor",
    "consentidos": "consentir",
    "consentimientos": "consentimiento",
    "consentimos": "consentir",
    "consentiremos": "consentir",
    "consentirá": "consentir",
    "consentirán": "consentir",
    "consentirás": "consentir",
    "consentiré": "consentir",
    "consentiréis": "consentir",
    "consentiría": "consentir",
    "consentiríais": "consentir",
    "consentiríamos": "consentir",
    "consentirían": "consentir",
    "consentirías": "consentir",
    "consentiste": "consentir",
    "consentisteis": "consentir",
    "consentí": "consentir",
    "consentía": "consentir",
    "consentíais": "consentir",
    "consentíamos": "consentir",
    "consentían": "consentir",
    "consentías": "consentir",
    "consentís": "consentir",
    "conserjerías": "conserjería",
    "conserjes": "conserje",
    "conserva": "conservar",
    "conservaba": "conservar",
    "conservabais": "conservar",
    "conservaban": "conservar",
    "conservabas": "conservar",
    "conservabilidades": "conservabilidad",
    "conservaciones": "conservación",
    "conservacionismos": "conservacionismo",
    "conservacionistas": "conservacionista",
    "conservad": "conservar",
    "conservada": "conservar",
    "conservadas": "conservar",
    "conservado": "conservar",
    "conservadora": "conservador",
    "conservadoras": "conservador",
    "conservadores": "conservador",
    "conservados": "conservar",
    "conservadurismos": "conservadurismo",
    "conservamos": "conservar",
    "conservan": "conservar",
    "conservando": "conservar",
    "conservantes": "conservante",
    "conservara": "conservar",
    "conservarais": "conservar",
    "conservaran": "conservar",
    "conservaras": "conservar",
    "conservare": "conservar",
    "conservareis": "conservar",
    "conservaremos": "conservar",
    "conservaren": "conservar",
    "conservares": "conservar",
    "conservaron": "conservar",
    "conservará": "conservar",
    "conservarán": "conservar",
    "conservarás": "conservar",
    "conservaré": "conservar",
    "conservaréis": "conservar",
    "conservaría": "conservar",
    "conservaríais": "conservar",
    "conservaríamos": "conservar",
    "conservarían": "conservar",
    "conservarías": "conservar",
    "conservas": "conservar",
    "conservase": "conservar",
    "conservaseis": "conservar",
    "conservasen": "conservar",
    "conservases": "conservar",
    "conservaste": "conservar",
    "conservasteis": "conservar",
    "conservativa": "conservativo",
    "conservativas": "conservativo",
    "conservativos": "conservativo",
    "conservatorios": "conservatorio",
    "conserve": "conservar",
    "conservemos": "conservar",
    "conserven": "conservar",
    "conserverías": "conservería",
    "conserves": "conservar",
    "conservo": "conservar",
    "conservábamos": "conservar",
    "conserváis": "conservar",
    "conserváramos": "conservar",
    "conserváremos": "conservar",
    "conservásemos": "conservar",
    "conservé": "conservar",
    "conservéis": "conservar",
    "conservó": "conservar",
    "considera": "considerar",
    "consideraba": "considerar",
    "considerabais": "considerar",
    "consideraban": "considerar",
    "considerabas": "considerar",
    "considerables": "considerable",
    "consideraciones": "consideración",
    "considerad": "considerar",
    "considerada": "considerar",
    "consideradas": "considerar",
    "considerado": "considerar",
    "considerados": "considerar",
    "consideramos": "considerar",
    "consideran": "considerar",
    "considerando": "considerar",
    "considerara": "considerar",
    "considerarais": "considerar",
    "consideraran": "considerar",
    "consideraras": "considerar",
    "considerare": "considerar",
    "considerareis": "considerar",
    "consideraremos": "considerar",
    "consideraren": "considerar",
    "considerares": "considerar",
    "consideraron": "considerar",
    "considerará": "considerar",
    "considerarán": "considerar",
    "considerarás": "considerar",
    "consideraré": "considerar",
    "consideraréis": "considerar",
    "consideraría": "considerar",
    "consideraríais": "considerar",
    "consideraríamos": "considerar",
    "considerarían": "considerar",
    "considerarías": "considerar",
    "consideras": "considerar",
    "considerase": "considerar",
    "consideraseis": "considerar",
    "considerasen": "considerar",
    "considerases": "considerar",
    "consideraste": "considerar",
    "considerasteis": "considerar",
    "considere": "considerar",
    "consideremos": "considerar",
    "consideren": "considerar",
    "consideres": "considerar",
    "considero": "considerar",
    "considerábamos": "considerar",
    "consideráis": "considerar",
    "consideráramos": "considerar",
    "consideráremos": "considerar",
    "considerásemos": "considerar",
    "consideré": "considerar",
    "consideréis": "considerar",
    "consideró": "considerar",
    "consienta": "consentir",
    "consientan": "consentir",
    "consientas": "consentir",
    "consiente": "consentir",
    "consienten": "consentir",
    "consientes": "consentir",
    "consiento": "consentir",
    "consiga": "conseguir",
    "consigamos": "conseguir",
    "consigan": "conseguir",
    "consigas": "conseguir",
    "consigna": "consignar",
    "consignaba": "consignar",
    "consignabais": "consignar",
    "consignaban": "consignar",
    "consignabas": "consignar",
    "consignaciones": "consignación",
    "consignad": "consignar",
    "consignada": "consignar",
    "consignadas": "consignar",
    "consignado": "consignar",
    "consignadora": "consignador",
    "consignadoras": "consignador",
    "consignadores": "consignador",
    "consignados": "consignar",
    "consignamos": "consignar",
    "consignan": "consignar",
    "consignando": "consignar",
    "consignara": "consignar",
    "consignarais": "consignar",
    "consignaran": "consignar",
    "consignaras": "consignar",
    "consignare": "consignar",
    "consignareis": "consignar",
    "consignaremos": "consignar",
    "consignaren": "consignar",
    "consignares": "consignar",
    "consignaron": "consignar",
    "consignará": "consignar",
    "consignarán": "consignar",
    "consignarás": "consignar",
    "consignaré": "consignar",
    "consignaréis": "consignar",
    "consignaría": "consignar",
    "consignaríais": "consignar",
    "consignaríamos": "consignar",
    "consignarían": "consignar",
    "consignarías": "consignar",
    "consignas": "consignar",
    "consignase": "consignar",
    "consignaseis": "consignar",
    "consignasen": "consignar",
    "consignases": "consignar",
    "consignaste": "consignar",
    "consignasteis": "consignar",
    "consignataria": "consignatario",
    "consignatarias": "consignatario",
    "consignatarios": "consignatario",
    "consignativa": "consignativo",
    "consignativas": "consignativo",
    "consignativos": "consignativo",
    "consigne": "consignar",
    "consignemos": "consignar",
    "consignen": "consignar",
    "consignes": "consignar",
    "consigno": "consignar",
    "consignábamos": "consignar",
    "consignáis": "consignar",
    "consignáramos": "consignar",
    "consignáremos": "consignar",
    "consignásemos": "consignar",
    "consigné": "consignar",
    "consignéis": "consignar",
    "consignó": "consignar",
    "consigo": "conseguir",
    "consigue": "conseguir",
    "consiguen": "conseguir",
    "consigues": "conseguir",
    "consiguiendo": "conseguir",
    "consiguientes": "consiguiente",
    "consiguiera": "conseguir",
    "consiguierais": "conseguir",
    "consiguieran": "conseguir",
    "consiguieras": "conseguir",
    "consiguiere": "conseguir",
    "consiguiereis": "conseguir",
    "consiguieren": "conseguir",
    "consiguieres": "conseguir",
    "consiguieron": "conseguir",
    "consiguiese": "conseguir",
    "consiguieseis": "conseguir",
    "consiguiesen": "conseguir",
    "consiguieses": "conseguir",
    "consiguiéramos": "conseguir",
    "consiguiéremos": "conseguir",
    "consiguiésemos": "conseguir",
    "consiguió": "conseguir",
    "consigáis": "conseguir",
    "consiliaria": "consiliario",
    "consiliarias": "consiliario",
    "consiliarios": "consiliario",
    "consintamos": "consentir",
    "consintiendo": "consentir",
    "consintiera": "consentir",
    "consintierais": "consentir",
    "consintieran": "consentir",
    "consintieras": "consentir",
    "consintiere": "consentir",
    "consintiereis": "consentir",
    "consintieren": "consentir",
    "consintieres": "consentir",
    "consintieron": "consentir",
    "consintiese": "consentir",
    "consintieseis": "consentir",
    "consintiesen": "consentir",
    "consintieses": "consentir",
    "consintiéramos": "consentir",
    "consintiéremos": "consentir",
    "consintiésemos": "consentir",
    "consintió": "consentir",
    "consintáis": "consentir",
    "consista": "consistir",
    "consistamos": "consistir",
    "consistan": "consistir",
    "consistas": "consistir",
    "consiste": "consistir",
    "consisten": "consistir",
    "consistencias": "consistencia",
    "consistentes": "consistente",
    "consistes": "consistir",
    "consistid": "consistir",
    "consistida": "consistir",
    "consistidas": "consistir",
    "consistido": "consistir",
    "consistidos": "consistir",
    "consistiendo": "consistir",
    "consistiera": "consistir",
    "consistierais": "consistir",
    "consistieran": "consistir",
    "consistieras": "consistir",
    "consistiere": "consistir",
    "consistiereis": "consistir",
    "consistieren": "consistir",
    "consistieres": "consistir",
    "consistieron": "consistir",
    "consistiese": "consistir",
    "consistieseis": "consistir",
    "consistiesen": "consistir",
    "consistieses": "consistir",
    "consistimos": "consistir",
    "consistiremos": "consistir",
    "consistirá": "consistir",
    "consistirán": "consistir",
    "consistirás": "consistir",
    "consistiré": "consistir",
    "consistiréis": "consistir",
    "consistiría": "consistir",
    "consistiríais": "consistir",
    "consistiríamos": "consistir",
    "consistirían": "consistir",
    "consistirías": "consistir",
    "consististe": "consistir",
    "consististeis": "consistir",
    "consistiéramos": "consistir",
    "consistiéremos": "consistir",
    "consistiésemos": "consistir",
    "consistió": "consistir",
    "consisto": "consistir",
    "consistoriales": "consistorial",
    "consistorios": "consistorio",
    "consistáis": "consistir",
    "consistí": "consistir",
    "consistía": "consistir",
    "consistíais": "consistir",
    "consistíamos": "consistir",
    "consistían": "consistir",
    "consistías": "consistir",
    "consistís": "consistir",
    "consistómetros": "consistómetro",
    "consocia": "consocio",
    "consocias": "consocio",
    "consocios": "consocio",
    "consolaba": "consolar",
    "consolabais": "consolar",
    "consolaban": "consolar",
    "consolabas": "consolar",
    "consolaciones": "consolación",
    "consolad": "consolar",
    "consolada": "consolar",
    "consoladas": "consolar",
    "consolado": "consolar",
    "consoladora": "consolador",
    "consoladoras": "consolador",
    "consoladores": "consolador",
    "consolados": "consolar",
    "consolamos": "consolar",
    "consolando": "consolar",
    "consolara": "consolar",
    "consolarais": "consolar",
    "consolaran": "consolar",
    "consolaras": "consolar",
    "consolare": "consolar",
    "consolareis": "consolar",
    "consolaremos": "consolar",
    "consolaren": "consolar",
    "consolares": "consolar",
    "consolaron": "consolar",
    "consolará": "consolar",
    "consolarán": "consolar",
    "consolarás": "consolar",
    "consolaré": "consolar",
    "consolaréis": "consolar",
    "consolaría": "consolar",
    "consolaríais": "consolar",
    "consolaríamos": "consolar",
    "consolarían": "consolar",
    "consolarías": "consolar",
    "consolas": "consola",
    "consolase": "consolar",
    "consolaseis": "consolar",
    "consolasen": "consolar",
    "consolases": "consolar",
    "consolaste": "consolar",
    "consolasteis": "consolar",
    "consolemos": "consolar",
    "consolida": "consolidar",
    "consolidaba": "consolidar",
    "consolidabais": "consolidar",
    "consolidaban": "consolidar",
    "consolidabas": "consolidar",
    "consolidaciones": "consolidación",
    "consolidad": "consolidar",
    "consolidada": "consolidar",
    "consolidadas": "consolidar",
    "consolidado": "consolidar",
    "consolidados": "consolidar",
    "consolidamos": "consolidar",
    "consolidan": "consolidar",
    "consolidando": "consolidar",
    "consolidara": "consolidar",
    "consolidarais": "consolidar",
    "consolidaran": "consolidar",
    "consolidaras": "consolidar",
    "consolidare": "consolidar",
    "consolidareis": "consolidar",
    "consolidaremos": "consolidar",
    "consolidaren": "consolidar",
    "consolidares": "consolidar",
    "consolidaron": "consolidar",
    "consolidará": "consolidar",
    "consolidarán": "consolidar",
    "consolidarás": "consolidar",
    "consolidaré": "consolidar",
    "consolidaréis": "consolidar",
    "consolidaría": "consolidar",
    "consolidaríais": "consolidar",
    "consolidaríamos": "consolidar",
    "consolidarían": "consolidar",
    "consolidarías": "consolidar",
    "consolidas": "consolidar",
    "consolidase": "consolidar",
    "consolidaseis": "consolidar",
    "consolidasen": "consolidar",
    "consolidases": "consolidar",
    "consolidaste": "consolidar",
    "consolidasteis": "consolidar",
    "consolide": "consolidar",
    "consolidemos": "consolidar",
    "consoliden": "consolidar",
    "consolides": "consolidar",
    "consolido": "consolidar",
    "consolidábamos": "consolidar",
    "consolidáis": "consolidar",
    "consolidáramos": "consolidar",
    "consolidáremos": "consolidar",
    "consolidásemos": "consolidar",
    "consolidé": "consolidar",
    "consolidéis": "consolidar",
    "consolidó": "consolidar",
    "consolidómetros": "consolidómetro",
    "consolábamos": "consolar",
    "consoláis": "consolar",
    "consoláramos": "consolar",
    "consoláremos": "consolar",
    "consolásemos": "consolar",
    "consolé": "consolar",
    "consoléis": "consolar",
    "consoló": "consolar",
    "consomés": "consomé",
    "consonancias": "consonancia",
    "consonantes": "consonante",
    "consonantismos": "consonantismo",
    "consonantizaciones": "consonantización",
    "consonántica": "consonántico",
    "consonánticas": "consonántico",
    "consonánticos": "consonántico",
    "consorciales": "consorcial",
    "consorcios": "consorcio",
    "consortes": "consorte",
    "conspicua": "conspicuo",
    "conspicuas": "conspicuo",
    "conspicuos": "conspicuo",
    "conspira": "conspirar",
    "conspiraba": "conspirar",
    "conspirabais": "conspirar",
    "conspiraban": "conspirar",
    "conspirabas": "conspirar",
    "conspiraciones": "conspiración",
    "conspirad": "conspirar",
    "conspirada": "conspirar",
    "conspiradas": "conspirar",
    "conspirado": "conspirar",
    "conspiradora": "conspirador",
    "conspiradoras": "conspirador",
    "conspiradores": "conspirador",
    "conspirados": "conspirar",
    "conspiramos": "conspirar",
    "conspiran": "conspirar",
    "conspirando": "conspirar",
    "conspirara": "conspirar",
    "conspirarais": "conspirar",
    "conspiraran": "conspirar",
    "conspiraras": "conspirar",
    "conspirare": "conspirar",
    "conspirareis": "conspirar",
    "conspiraremos": "conspirar",
    "conspiraren": "conspirar",
    "conspirares": "conspirar",
    "conspiraron": "conspirar",
    "conspirará": "conspirar",
    "conspirarán": "conspirar",
    "conspirarás": "conspirar",
    "conspiraré": "conspirar",
    "conspiraréis": "conspirar",
    "conspiraría": "conspirar",
    "conspiraríais": "conspirar",
    "conspiraríamos": "conspirar",
    "conspirarían": "conspirar",
    "conspirarías": "conspirar",
    "conspiras": "conspirar",
    "conspirase": "conspirar",
    "conspiraseis": "conspirar",
    "conspirasen": "conspirar",
    "conspirases": "conspirar",
    "conspiraste": "conspirar",
    "conspirasteis": "conspirar",
    "conspiratoria": "conspiratorio",
    "conspiratorias": "conspiratorio",
    "conspiratorios": "conspiratorio",
    "conspire": "conspirar",
    "conspiremos": "conspirar",
    "conspiren": "conspirar",
    "conspires": "conspirar",
    "conspiro": "conspirar",
    "conspirábamos": "conspirar",
    "conspiráis": "conspirar",
    "conspiráramos": "conspirar",
    "conspiráremos": "conspirar",
    "conspirásemos": "conspirar",
    "conspiré": "conspirar",
    "conspiréis": "conspirar",
    "conspiró": "conspirar",
    "consta": "constar",
    "constaba": "constar",
    "constabais": "constar",
    "constaban": "constar",
    "constabas": "constar",
    "constad": "constar",
    "constada": "constar",
    "constadas": "constar",
    "constado": "constar",
    "constados": "constar",
    "constamos": "constar",
    "constan": "constar",
    "constancias": "constancia",
    "constando": "constar",
    "constantes": "constante",
    "constantinopolitana": "constantinopolitano",
    "constantinopolitanas": "constantinopolitano",
    "constantinopolitanos": "constantinopolitano",
    "constara": "constar",
    "constarais": "constar",
    "constaran": "constar",
    "constaras": "constar",
    "constare": "constar",
    "constareis": "constar",
    "constaremos": "constar",
    "constaren": "constar",
    "constares": "constar",
    "constaron": "constar",
    "constará": "constar",
    "constarán": "constar",
    "constarás": "constar",
    "constaré": "constar",
    "constaréis": "constar",
    "constaría": "constar",
    "constaríais": "constar",
    "constaríamos": "constar",
    "constarían": "constar",
    "constarías": "constar",
    "constas": "constar",
    "constase": "constar",
    "constaseis": "constar",
    "constasen": "constar",
    "constases": "constar",
    "constaste": "constar",
    "constasteis": "constar",
    "constata": "constatar",
    "constataba": "constatar",
    "constatabais": "constatar",
    "constataban": "constatar",
    "constatabas": "constatar",
    "constatables": "constatable",
    "constataciones": "constatación",
    "constatad": "constatar",
    "constatada": "constatar",
    "constatadas": "constatar",
    "constatado": "constatar",
    "constatados": "constatar",
    "constatamos": "constatar",
    "constatan": "constatar",
    "constatando": "constatar",
    "constatara": "constatar",
    "constatarais": "constatar",
    "constataran": "constatar",
    "constataras": "constatar",
    "constatare": "constatar",
    "constatareis": "constatar",
    "constataremos": "constatar",
    "constataren": "constatar",
    "constatares": "constatar",
    "constataron": "constatar",
    "constatará": "constatar",
    "constatarán": "constatar",
    "constatarás": "constatar",
    "constataré": "constatar",
    "constataréis": "constatar",
    "constataría": "constatar",
    "constataríais": "constatar",
    "constataríamos": "constatar",
    "constatarían": "constatar",
    "constatarías": "constatar",
    "constatas": "constatar",
    "constatase": "constatar",
    "constataseis": "constatar",
    "constatasen": "constatar",
    "constatases": "constatar",
    "constataste": "constatar",
    "constatasteis": "constatar",
    "constate": "constatar",
    "constatemos": "constatar",
    "constaten": "constatar",
    "constates": "constatar",
    "constato": "constatar",
    "constatábamos": "constatar",
    "constatáis": "constatar",
    "constatáramos": "constatar",
    "constatáremos": "constatar",
    "constatásemos": "constatar",
    "constaté": "constatar",
    "constatéis": "constatar",
    "constató": "constatar",
    "conste": "constar",
    "constela": "constelar",
    "constelaba": "constelar",
    "constelabais": "constelar",
    "constelaban": "constelar",
    "constelabas": "constelar",
    "constelaciones": "constelación",
    "constelad": "constelar",
    "constelada": "constelar",
    "consteladas": "constelar",
    "constelado": "constelar",
    "constelados": "constelar",
    "constelamos": "constelar",
    "constelan": "constelar",
    "constelando": "constelar",
    "constelara": "constelar",
    "constelarais": "constelar",
    "constelaran": "constelar",
    "constelaras": "constelar",
    "constelare": "constelar",
    "constelareis": "constelar",
    "constelaremos": "constelar",
    "constelaren": "constelar",
    "constelares": "constelar",
    "constelaron": "constelar",
    "constelará": "constelar",
    "constelarán": "constelar",
    "constelarás": "constelar",
    "constelaré": "constelar",
    "constelaréis": "constelar",
    "constelaría": "constelar",
    "constelaríais": "constelar",
    "constelaríamos": "constelar",
    "constelarían": "constelar",
    "constelarías": "constelar",
    "constelas": "constelar",
    "constelase": "constelar",
    "constelaseis": "constelar",
    "constelasen": "constelar",
    "constelases": "constelar",
    "constelaste": "constelar",
    "constelasteis": "constelar",
    "constele": "constelar",
    "constelemos": "constelar",
    "constelen": "constelar",
    "consteles": "constelar",
    "constelo": "constelar",
    "constelábamos": "constelar",
    "consteláis": "constelar",
    "consteláramos": "constelar",
    "consteláremos": "constelar",
    "constelásemos": "constelar",
    "constelé": "constelar",
    "consteléis": "constelar",
    "consteló": "constelar",
    "constemos": "constar",
    "consten": "constar",
    "consterna": "consternar",
    "consternaba": "consternar",
    "consternabais": "consternar",
    "consternaban": "consternar",
    "consternabas": "consternar",
    "consternaciones": "consternación",
    "consternad": "consternar",
    "consternada": "consternar",
    "consternadas": "consternar",
    "consternado": "consternar",
    "consternados": "consternar",
    "consternamos": "consternar",
    "consternan": "consternar",
    "consternando": "consternar",
    "consternara": "consternar",
    "consternarais": "consternar",
    "consternaran": "consternar",
    "consternaras": "consternar",
    "consternare": "consternar",
    "consternareis": "consternar",
    "consternaremos": "consternar",
    "consternaren": "consternar",
    "consternares": "consternar",
    "consternaron": "consternar",
    "consternará": "consternar",
    "consternarán": "consternar",
    "consternarás": "consternar",
    "consternaré": "consternar",
    "consternaréis": "consternar",
    "consternaría": "consternar",
    "consternaríais": "consternar",
    "consternaríamos": "consternar",
    "consternarían": "consternar",
    "consternarías": "consternar",
    "consternas": "consternar",
    "consternase": "consternar",
    "consternaseis": "consternar",
    "consternasen": "consternar",
    "consternases": "consternar",
    "consternaste": "consternar",
    "consternasteis": "consternar",
    "consterne": "consternar",
    "consternemos": "consternar",
    "consternen": "consternar",
    "consternes": "consternar",
    "consterno": "consternar",
    "consternábamos": "consternar",
    "consternáis": "consternar",
    "consternáramos": "consternar",
    "consternáremos": "consternar",
    "consternásemos": "consternar",
    "consterné": "consternar",
    "consternéis": "consternar",
    "consternó": "consternar",
    "constes": "constar",
    "constipa": "constipar",
    "constipaba": "constipar",
    "constipabais": "constipar",
    "constipaban": "constipar",
    "constipabas": "constipar",
    "constipaciones": "constipación",
    "constipad": "constipar",
    "constipada": "constipar",
    "constipadas": "constipar",
    "constipado": "constipar",
    "constipados": "constipar",
    "constipamos": "constipar",
    "constipan": "constipar",
    "constipando": "constipar",
    "constipara": "constipar",
    "constiparais": "constipar",
    "constiparan": "constipar",
    "constiparas": "constipar",
    "constipare": "constipar",
    "constipareis": "constipar",
    "constiparemos": "constipar",
    "constiparen": "constipar",
    "constipares": "constipar",
    "constiparon": "constipar",
    "constipará": "constipar",
    "constiparán": "constipar",
    "constiparás": "constipar",
    "constiparé": "constipar",
    "constiparéis": "constipar",
    "constiparía": "constipar",
    "constiparíais": "constipar",
    "constiparíamos": "constipar",
    "constiparían": "constipar",
    "constiparías": "constipar",
    "constipas": "constipar",
    "constipase": "constipar",
    "constipaseis": "constipar",
    "constipasen": "constipar",
    "constipases": "constipar",
    "constipaste": "constipar",
    "constipasteis": "constipar",
    "constipe": "constipar",
    "constipemos": "constipar",
    "constipen": "constipar",
    "constipes": "constipar",
    "constipo": "constipar",
    "constipábamos": "constipar",
    "constipáis": "constipar",
    "constipáramos": "constipar",
    "constipáremos": "constipar",
    "constipásemos": "constipar",
    "constipé": "constipar",
    "constipéis": "constipar",
    "constipó": "constipar",
    "constitucionales": "constitucional",
    "constitucionalice": "constitucionalizar",
    "constitucionalicemos": "constitucionalizar",
    "constitucionalicen": "constitucionalizar",
    "constitucionalices": "constitucionalizar",
    "constitucionalicé": "constitucionalizar",
    "constitucionalicéis": "constitucionalizar",
    "constitucionalidades": "constitucionalidad",
    "constitucionalismos": "constitucionalismo",
    "constitucionalistas": "constitucionalista",
    "constitucionaliza": "constitucionalizar",
    "constitucionalizaba": "constitucionalizar",
    "constitucionalizabais": "constitucionalizar",
    "constitucionalizaban": "constitucionalizar",
    "constitucionalizabas": "constitucionalizar",
    "constitucionalizad": "constitucionalizar",
    "constitucionalizada": "constitucionalizar",
    "constitucionalizadas": "constitucionalizar",
    "constitucionalizado": "constitucionalizar",
    "constitucionalizados": "constitucionalizar",
    "constitucionalizamos": "constitucionalizar",
    "constitucionalizan": "constitucionalizar",
    "constitucionalizando": "constitucionalizar",
    "constitucionalizara": "constitucionalizar",
    "constitucionalizarais": "constitucionalizar",
    "constitucionalizaran": "constitucionalizar",
    "constitucionalizaras": "constitucionalizar",
    "constitucionalizare": "constitucionalizar",
    "constitucionalizareis": "constitucionalizar",
    "constitucionalizaremos": "constitucionalizar",
    "constitucionalizaren": "constitucionalizar",
    "constitucionalizares": "constitucionalizar",
    "constitucionalizaron": "constitucionalizar",
    "constitucionalizará": "constitucionalizar",
    "constitucionalizarán": "constitucionalizar",
    "constitucionalizarás": "constitucionalizar",
    "constitucionalizaré": "constitucionalizar",
    "constitucionalizaréis": "constitucionalizar",
    "constitucionalizaría": "constitucionalizar",
    "constitucionalizaríais": "constitucionalizar",
    "constitucionalizaríamos": "constitucionalizar",
    "constitucionalizarían": "constitucionalizar",
    "constitucionalizarías": "constitucionalizar",
    "constitucionalizas": "constitucionalizar",
    "constitucionalizase": "constitucionalizar",
    "constitucionalizaseis": "constitucionalizar",
    "constitucionalizasen": "constitucionalizar",
    "constitucionalizases": "constitucionalizar",
    "constitucionalizaste": "constitucionalizar",
    "constitucionalizasteis": "constitucionalizar",
    "constitucionalizo": "constitucionalizar",
    "constitucionalizábamos": "constitucionalizar",
    "constitucionalizáis": "constitucionalizar",
    "constitucionalizáramos": "constitucionalizar",
    "constitucionalizáremos": "constitucionalizar",
    "constitucionalizásemos": "constitucionalizar",
    "constitucionalizó": "constitucionalizar",
    "constituciones": "constitución",
    "constituid": "constituir",
    "constituida": "constituir",
    "constituidas": "constituir",
    "constituido": "constituir",
    "constituidos": "constituir",
    "constituimos": "constituir",
    "constituiremos": "constituir",
    "constituirá": "constituir",
    "constituirán": "constituir",
    "constituirás": "constituir",
    "constituiré": "constituir",
    "constituiréis": "constituir",
    "constituiría": "constituir",
    "constituiríais": "constituir",
    "constituiríamos": "constituir",
    "constituirían": "constituir",
    "constituirías": "constituir",
    "constituiste": "constituir",
    "constituisteis": "constituir",
    "constitutiva": "constitutivo",
    "constitutivas": "constitutivo",
    "constitutivos": "constitutivo",
    "constituya": "constituir",
    "constituyamos": "constituir",
    "constituyan": "constituir",
    "constituyas": "constituir",
    "constituye": "constituir",
    "constituyen": "constituir",
    "constituyendo": "constituir",
    "constituyentes": "constituyente",
    "constituyera": "constituir",
    "constituyerais": "constituir",
    "constituyeran": "constituir",
    "constituyeras": "constituir",
    "constituyere": "constituir",
    "constituyereis": "constituir",
    "constituyeren": "constituir",
    "constituyeres": "constituir",
    "constituyeron": "constituir",
    "constituyes": "constituir",
    "constituyese": "constituir",
    "constituyeseis": "constituir",
    "constituyesen": "constituir",
    "constituyeses": "constituir",
    "constituyo": "constituir",
    "constituyáis": "constituir",
    "constituyéramos": "constituir",
    "constituyéremos": "constituir",
    "constituyésemos": "constituir",
    "constituyó": "constituir",
    "constituí": "constituir",
    "constituía": "constituir",
    "constituíais": "constituir",
    "constituíamos": "constituir",
    "constituían": "constituir",
    "constituías": "constituir",
    "constituís": "constituir",
    "consto": "constar",
    "constreñid": "constreñir",
    "constreñida": "constreñir",
    "constreñidas": "constreñir",
    "constreñido": "constreñir",
    "constreñidos": "constreñir",
    "constreñimos": "constreñir",
    "constreñiremos": "constreñir",
    "constreñirá": "constreñir",
    "constreñirán": "constreñir",
    "constreñirás": "constreñir",
    "constreñiré": "constreñir",
    "constreñiréis": "constreñir",
    "constreñiría": "constreñir",
    "constreñiríais": "constreñir",
    "constreñiríamos": "constreñir",
    "constreñirían": "constreñir",
    "constreñirías": "constreñir",
    "constreñiste": "constreñir",
    "constreñisteis": "constreñir",
    "constreñí": "constreñir",
    "constreñía": "constreñir",
    "constreñíais": "constreñir",
    "constreñíamos": "constreñir",
    "constreñían": "constreñir",
    "constreñías": "constreñir",
    "constreñís": "constreñir",
    "constricciones": "constricción",
    "constrictiva": "constrictivo",
    "constrictivas": "constrictivo",
    "constrictivos": "constrictivo",
    "constrictora": "constrictor",
    "constrictoras": "constrictor",
    "constrictores": "constrictor",
    "constringencias": "constringencia",
    "constriña": "constreñir",
    "constriñamos": "constreñir",
    "constriñan": "constreñir",
    "constriñas": "constreñir",
    "constriñe": "constreñir",
    "constriñen": "constreñir",
    "constriñendo": "constreñir",
    "constriñera": "constreñir",
    "constriñerais": "constreñir",
    "constriñeran": "constreñir",
    "constriñeras": "constreñir",
    "constriñere": "constreñir",
    "constriñereis": "constreñir",
    "constriñeren": "constreñir",
    "constriñeres": "constreñir",
    "constriñeron": "constreñir",
    "constriñes": "constreñir",
    "constriñese": "constreñir",
    "constriñeseis": "constreñir",
    "constriñesen": "constreñir",
    "constriñeses": "constreñir",
    "constriño": "constreñir",
    "constriñáis": "constreñir",
    "constriñéramos": "constreñir",
    "constriñéremos": "constreñir",
    "constriñésemos": "constreñir",
    "constriñó": "constreñir",
    "construcciones": "construcción",
    "constructiva": "constructivo",
    "constructivas": "constructivo",
    "constructivismos": "constructivismo",
    "constructivistas": "constructivista",
    "constructivos": "constructivo",
    "constructora": "constructor",
    "constructoras": "constructor",
    "constructores": "constructor",
    "construid": "construir",
    "construida": "construir",
    "construidas": "construir",
    "construido": "construir",
    "construidos": "construir",
    "construimos": "construir",
    "construiremos": "construir",
    "construirá": "construir",
    "construirán": "construir",
    "construirás": "construir",
    "construiré": "construir",
    "construiréis": "construir",
    "construiría": "construir",
    "construiríais": "construir",
    "construiríamos": "construir",
    "construirían": "construir",
    "construirías": "construir",
    "construiste": "construir",
    "construisteis": "construir",
    "construya": "construir",
    "construyamos": "construir",
    "construyan": "construir",
    "construyas": "construir",
    "construye": "construir",
    "construyen": "construir",
    "construyendo": "construir",
    "construyera": "construir",
    "construyerais": "construir",
    "construyeran": "construir",
    "construyeras": "construir",
    "construyere": "construir",
    "construyereis": "construir",
    "construyeren": "construir",
    "construyeres": "construir",
    "construyeron": "construir",
    "construyes": "construir",
    "construyese": "construir",
    "construyeseis": "construir",
    "construyesen": "construir",
    "construyeses": "construir",
    "construyo": "construir",
    "construyáis": "construir",
    "construyéramos": "construir",
    "construyéremos": "construir",
    "construyésemos": "construir",
    "construyó": "construir",
    "construí": "construir",
    "construía": "construir",
    "construíais": "construir",
    "construíamos": "construir",
    "construían": "construir",
    "construías": "construir",
    "construís": "construir",
    "constábamos": "constar",
    "constáis": "constar",
    "constáramos": "constar",
    "constáremos": "constar",
    "constásemos": "constar",
    "consté": "constar",
    "constéis": "constar",
    "constó": "constar",
    "consubstanciaciones": "consubstanciación",
    "consubstanciales": "consubstancial",
    "consuegra": "consuegro",
    "consuegraba": "consuegrar",
    "consuegrabais": "consuegrar",
    "consuegraban": "consuegrar",
    "consuegrabas": "consuegrar",
    "consuegrad": "consuegrar",
    "consuegrada": "consuegrar",
    "consuegradas": "consuegrar",
    "consuegrado": "consuegrar",
    "consuegrados": "consuegrar",
    "consuegramos": "consuegrar",
    "consuegran": "consuegrar",
    "consuegrando": "consuegrar",
    "consuegrara": "consuegrar",
    "consuegrarais": "consuegrar",
    "consuegraran": "consuegrar",
    "consuegraras": "consuegrar",
    "consuegrare": "consuegrar",
    "consuegrareis": "consuegrar",
    "consuegraremos": "consuegrar",
    "consuegraren": "consuegrar",
    "consuegrares": "consuegrar",
    "consuegraron": "consuegrar",
    "consuegrará": "consuegrar",
    "consuegrarán": "consuegrar",
    "consuegrarás": "consuegrar",
    "consuegraré": "consuegrar",
    "consuegraréis": "consuegrar",
    "consuegraría": "consuegrar",
    "consuegraríais": "consuegrar",
    "consuegraríamos": "consuegrar",
    "consuegrarían": "consuegrar",
    "consuegrarías": "consuegrar",
    "consuegras": "consuegro",
    "consuegrase": "consuegrar",
    "consuegraseis": "consuegrar",
    "consuegrasen": "consuegrar",
    "consuegrases": "consuegrar",
    "consuegraste": "consuegrar",
    "consuegrasteis": "consuegrar",
    "consuegre": "consuegrar",
    "consuegremos": "consuegrar",
    "consuegren": "consuegrar",
    "consuegres": "consuegrar",
    "consuegro": "consuegrar",
    "consuegros": "consuegro",
    "consuegrábamos": "consuegrar",
    "consuegráis": "consuegrar",
    "consuegráramos": "consuegrar",
    "consuegráremos": "consuegrar",
    "consuegrásemos": "consuegrar",
    "consuegré": "consuegrar",
    "consuegréis": "consuegrar",
    "consuegró": "consuegrar",
    "consuela": "consolar",
    "consuelan": "consolar",
    "consuelas": "consolar",
    "consueldas": "consuelda",
    "consuele": "consolar",
    "consuelen": "consolar",
    "consueles": "consolar",
    "consuelo": "consolar",
    "consuelos": "consuelo",
    "consuetas": "consueta",
    "consuetudes": "consuetud",
    "consuetudinaria": "consuetudinario",
    "consuetudinarias": "consuetudinario",
    "consuetudinarios": "consuetudinario",
    "consulados": "consulado",
    "consulares": "consular",
    "consulta": "consultar",
    "consultaba": "consultar",
    "consultabais": "consultar",
    "consultaban": "consultar",
    "consultabas": "consultar",
    "consultaciones": "consultación",
    "consultad": "consultar",
    "consultada": "consultar",
    "consultadas": "consultar",
    "consultado": "consultar",
    "consultados": "consultar",
    "consultamos": "consultar",
    "consultan": "consultar",
    "consultando": "consultar",
    "consultara": "consultar",
    "consultarais": "consultar",
    "consultaran": "consultar",
    "consultaras": "consultar",
    "consultare": "consultar",
    "consultareis": "consultar",
    "consultaremos": "consultar",
    "consultaren": "consultar",
    "consultares": "consultar",
    "consultaron": "consultar",
    "consultará": "consultar",
    "consultarán": "consultar",
    "consultarás": "consultar",
    "consultaré": "consultar",
    "consultaréis": "consultar",
    "consultaría": "consultar",
    "consultaríais": "consultar",
    "consultaríamos": "consultar",
    "consultarían": "consultar",
    "consultarías": "consultar",
    "consultas": "consultar",
    "consultase": "consultar",
    "consultaseis": "consultar",
    "consultasen": "consultar",
    "consultases": "consultar",
    "consultaste": "consultar",
    "consultasteis": "consultar",
    "consulte": "consultar",
    "consultemos": "consultar",
    "consulten": "consultar",
    "consultes": "consultar",
    "consultings": "consulting",
    "consultiva": "consultivo",
    "consultivas": "consultivo",
    "consultivos": "consultivo",
    "consulto": "consultar",
    "consultora": "consultor",
    "consultoras": "consultor",
    "consultores": "consultor",
    "consultorios": "consultorio",
    "consultorías": "consultoría",
    "consultábamos": "consultar",
    "consultáis": "consultar",
    "consultáramos": "consultar",
    "consultáremos": "consultar",
    "consultásemos": "consultar",
    "consulté": "consultar",
    "consultéis": "consultar",
    "consultó": "consultar",
    "consuma": "consumir",
    "consumaba": "consumar",
    "consumabais": "consumar",
    "consumaban": "consumar",
    "consumabas": "consumar",
    "consumaciones": "consumación",
    "consumad": "consumar",
    "consumada": "consumar",
    "consumadas": "consumar",
    "consumado": "consumar",
    "consumados": "consumar",
    "consumamos": "consumir",
    "consuman": "consumir",
    "consumando": "consumar",
    "consumara": "consumar",
    "consumarais": "consumar",
    "consumaran": "consumar",
    "consumaras": "consumar",
    "consumare": "consumar",
    "consumareis": "consumar",
    "consumaremos": "consumar",
    "consumaren": "consumar",
    "consumares": "consumar",
    "consumaron": "consumar",
    "consumará": "consumar",
    "consumarán": "consumar",
    "consumarás": "consumar",
    "consumaré": "consumar",
    "consumaréis": "consumar",
    "consumaría": "consumar",
    "consumaríais": "consumar",
    "consumaríamos": "consumar",
    "consumarían": "consumar",
    "consumarías": "consumar",
    "consumas": "consumir",
    "consumase": "consumar",
    "consumaseis": "consumar",
    "consumasen": "consumar",
    "consumases": "consumar",
    "consumaste": "consumar",
    "consumasteis": "consumar",
    "consumativa": "consumativo",
    "consumativas": "consumativo",
    "consumativos": "consumativo",
    "consume": "consumir",
    "consumemos": "consumar",
    "consumen": "consumir",
    "consumes": "consumir",
    "consumibles": "consumible",
    "consumiciones": "consumición",
    "consumid": "consumir",
    "consumida": "consumir",
    "consumidas": "consumir",
    "consumido": "consumir",
    "consumidora": "consumidor",
    "consumidoras": "consumidor",
    "consumidores": "consumidor",
    "consumidos": "consumir",
    "consumiendo": "consumir",
    "consumiera": "consumir",
    "consumierais": "consumir",
    "consumieran": "consumir",
    "consumieras": "consumir",
    "consumiere": "consumir",
    "consumiereis": "consumir",
    "consumieren": "consumir",
    "consumieres": "consumir",
    "consumieron": "consumir",
    "consumiese": "consumir",
    "consumieseis": "consumir",
    "consumiesen": "consumir",
    "consumieses": "consumir",
    "consumimientos": "consumimiento",
    "consumimos": "consumir",
    "consumiremos": "consumir",
    "consumirá": "consumir",
    "consumirán": "consumir",
    "consumirás": "consumir",
    "consumiré": "consumir",
    "consumiréis": "consumir",
    "consumiría": "consumir",
    "consumiríais": "consumir",
    "consumiríamos": "consumir",
    "consumirían": "consumir",
    "consumirías": "consumir",
    "consumismos": "consumismo",
    "consumistas": "consumista",
    "consumiste": "consumir",
    "consumisteis": "consumir",
    "consumiéramos": "consumir",
    "consumiéremos": "consumir",
    "consumiésemos": "consumir",
    "consumió": "consumir",
    "consumo": "consumir",
    "consumos": "consumo",
    "consumábamos": "consumar",
    "consumáis": "consumir",
    "consumáramos": "consumar",
    "consumáremos": "consumar",
    "consumásemos": "consumar",
    "consumé": "consumar",
    "consuméis": "consumar",
    "consumí": "consumir",
    "consumía": "consumir",
    "consumíais": "consumir",
    "consumíamos": "consumir",
    "consumían": "consumir",
    "consumías": "consumir",
    "consumís": "consumir",
    "consumó": "consumar",
    "consunciones": "consunción",
    "consuntiva": "consuntivo",
    "consuntivas": "consuntivo",
    "consuntivos": "consuntivo",
    "consustanciaciones": "consustanciación",
    "consustanciales": "consustancial",
    "contaba": "contar",
    "contabais": "contar",
    "contaban": "contar",
    "contabas": "contar",
    "contabilice": "contabilizar",
    "contabilicemos": "contabilizar",
    "contabilicen": "contabilizar",
    "contabilices": "contabilizar",
    "contabilicé": "contabilizar",
    "contabilicéis": "contabilizar",
    "contabilidades": "contabilidad",
    "contabiliza": "contabilizar",
    "contabilizaba": "contabilizar",
    "contabilizabais": "contabilizar",
    "contabilizaban": "contabilizar",
    "contabilizabas": "contabilizar",
    "contabilizables": "contabilizable",
    "contabilizaciones": "contabilización",
    "contabilizad": "contabilizar",
    "contabilizada": "contabilizar",
    "contabilizadas": "contabilizar",
    "contabilizado": "contabilizar",
    "contabilizados": "contabilizar",
    "contabilizamos": "contabilizar",
    "contabilizan": "contabilizar",
    "contabilizando": "contabilizar",
    "contabilizara": "contabilizar",
    "contabilizarais": "contabilizar",
    "contabilizaran": "contabilizar",
    "contabilizaras": "contabilizar",
    "contabilizare": "contabilizar",
    "contabilizareis": "contabilizar",
    "contabilizaremos": "contabilizar",
    "contabilizaren": "contabilizar",
    "contabilizares": "contabilizar",
    "contabilizaron": "contabilizar",
    "contabilizará": "contabilizar",
    "contabilizarán": "contabilizar",
    "contabilizarás": "contabilizar",
    "contabilizaré": "contabilizar",
    "contabilizaréis": "contabilizar",
    "contabilizaría": "contabilizar",
    "contabilizaríais": "contabilizar",
    "contabilizaríamos": "contabilizar",
    "contabilizarían": "contabilizar",
    "contabilizarías": "contabilizar",
    "contabilizas": "contabilizar",
    "contabilizase": "contabilizar",
    "contabilizaseis": "contabilizar",
    "contabilizasen": "contabilizar",
    "contabilizases": "contabilizar",
    "contabilizaste": "contabilizar",
    "contabilizasteis": "contabilizar",
    "contabilizo": "contabilizar",
    "contabilizábamos": "contabilizar",
    "contabilizáis": "contabilizar",
    "contabilizáramos": "contabilizar",
    "contabilizáremos": "contabilizar",
    "contabilizásemos": "contabilizar",
    "contabilizó": "contabilizar",
    "contables": "contable",
    "contacta": "contactar",
    "contactaba": "contactar",
    "contactabais": "contactar",
    "contactaban": "contactar",
    "contactabas": "contactar",
    "contactables": "contactable",
    "contactad": "contactar",
    "contactada": "contactar",
    "contactadas": "contactar",
    "contactado": "contactar",
    "contactados": "contactar",
    "contactamos": "contactar",
    "contactan": "contactar",
    "contactando": "contactar",
    "contactara": "contactar",
    "contactarais": "contactar",
    "contactaran": "contactar",
    "contactaras": "contactar",
    "contactare": "contactar",
    "contactareis": "contactar",
    "contactaremos": "contactar",
    "contactaren": "contactar",
    "contactares": "contactar",
    "contactaron": "contactar",
    "contactará": "contactar",
    "contactarán": "contactar",
    "contactarás": "contactar",
    "contactaré": "contactar",
    "contactaréis": "contactar",
    "contactaría": "contactar",
    "contactaríais": "contactar",
    "contactaríamos": "contactar",
    "contactarían": "contactar",
    "contactarías": "contactar",
    "contactas": "contactar",
    "contactase": "contactar",
    "contactaseis": "contactar",
    "contactasen": "contactar",
    "contactases": "contactar",
    "contactaste": "contactar",
    "contactasteis": "contactar",
    "contacte": "contactar",
    "contactemos": "contactar",
    "contacten": "contactar",
    "contactes": "contactar",
    "contacto": "contactar",
    "contactores": "contactor",
    "contactos": "contacto",
    "contactábamos": "contactar",
    "contactáis": "contactar",
    "contactáramos": "contactar",
    "contactáremos": "contactar",
    "contactásemos": "contactar",
    "contacté": "contactar",
    "contactéis": "contactar",
    "contactó": "contactar",
    "contad": "contar",
    "contada": "contar",
    "contadas": "contar",
    "contadera": "contadero",
    "contaderas": "contadero",
    "contaderos": "contadero",
    "contado": "contar",
    "contadora": "contador",
    "contadoras": "contador",
    "contadores": "contador",
    "contados": "contar",
    "contadurías": "contaduría",
    "contagia": "contagiar",
    "contagiaba": "contagiar",
    "contagiabais": "contagiar",
    "contagiaban": "contagiar",
    "contagiabas": "contagiar",
    "contagiad": "contagiar",
    "contagiada": "contagiar",
    "contagiadas": "contagiar",
    "contagiado": "contagiar",
    "contagiados": "contagiar",
    "contagiamos": "contagiar",
    "contagian": "contagiar",
    "contagiando": "contagiar",
    "contagiara": "contagiar",
    "contagiarais": "contagiar",
    "contagiaran": "contagiar",
    "contagiaras": "contagiar",
    "contagiare": "contagiar",
    "contagiareis": "contagiar",
    "contagiaremos": "contagiar",
    "contagiaren": "contagiar",
    "contagiares": "contagiar",
    "contagiaron": "contagiar",
    "contagiará": "contagiar",
    "contagiarán": "contagiar",
    "contagiarás": "contagiar",
    "contagiaré": "contagiar",
    "contagiaréis": "contagiar",
    "contagiaría": "contagiar",
    "contagiaríais": "contagiar",
    "contagiaríamos": "contagiar",
    "contagiarían": "contagiar",
    "contagiarías": "contagiar",
    "contagias": "contagiar",
    "contagiase": "contagiar",
    "contagiaseis": "contagiar",
    "contagiasen": "contagiar",
    "contagiases": "contagiar",
    "contagiaste": "contagiar",
    "contagiasteis": "contagiar",
    "contagie": "contagiar",
    "contagiemos": "contagiar",
    "contagien": "contagiar",
    "contagies": "contagiar",
    "contagio": "contagiar",
    "contagios": "contagio",
    "contagiosa": "contagioso",
    "contagiosas": "contagioso",
    "contagiosos": "contagioso",
    "contagiábamos": "contagiar",
    "contagiáis": "contagiar",
    "contagiáramos": "contagiar",
    "contagiáremos": "contagiar",
    "contagiásemos": "contagiar",
    "contagié": "contagiar",
    "contagiéis": "contagiar",
    "contagió": "contagiar",
    "containeres": "container",
    "containers": "container",
    "contajes": "contaje",
    "contales": "contal",
    "contamina": "contaminar",
    "contaminaba": "contaminar",
    "contaminabais": "contaminar",
    "contaminaban": "contaminar",
    "contaminabas": "contaminar",
    "contaminaciones": "contaminación",
    "contaminad": "contaminar",
    "contaminada": "contaminar",
    "contaminadas": "contaminar",
    "contaminado": "contaminar",
    "contaminadora": "contaminador",
    "contaminadoras": "contaminador",
    "contaminadores": "contaminador",
    "contaminados": "contaminar",
    "contaminamos": "contaminar",
    "contaminan": "contaminar",
    "contaminando": "contaminar",
    "contaminantes": "contaminante",
    "contaminara": "contaminar",
    "contaminarais": "contaminar",
    "contaminaran": "contaminar",
    "contaminaras": "contaminar",
    "contaminare": "contaminar",
    "contaminareis": "contaminar",
    "contaminaremos": "contaminar",
    "contaminaren": "contaminar",
    "contaminares": "contaminar",
    "contaminaron": "contaminar",
    "contaminará": "contaminar",
    "contaminarán": "contaminar",
    "contaminarás": "contaminar",
    "contaminaré": "contaminar",
    "contaminaréis": "contaminar",
    "contaminaría": "contaminar",
    "contaminaríais": "contaminar",
    "contaminaríamos": "contaminar",
    "contaminarían": "contaminar",
    "contaminarías": "contaminar",
    "contaminas": "contaminar",
    "contaminase": "contaminar",
    "contaminaseis": "contaminar",
    "contaminasen": "contaminar",
    "contaminases": "contaminar",
    "contaminaste": "contaminar",
    "contaminasteis": "contaminar",
    "contamine": "contaminar",
    "contaminemos": "contaminar",
    "contaminen": "contaminar",
    "contamines": "contaminar",
    "contamino": "contaminar",
    "contaminábamos": "contaminar",
    "contamináis": "contaminar",
    "contamináramos": "contaminar",
    "contamináremos": "contaminar",
    "contaminásemos": "contaminar",
    "contaminé": "contaminar",
    "contaminéis": "contaminar",
    "contaminó": "contaminar",
    "contamos": "contar",
    "contando": "contar",
    "contangos": "contango",
    "contantes": "contante",
    "contara": "contar",
    "contarais": "contar",
    "contaran": "contar",
    "contaras": "contar",
    "contare": "contar",
    "contareis": "contar",
    "contaremos": "contar",
    "contaren": "contar",
    "contares": "contar",
    "contarios": "contario",
    "contaron": "contar",
    "contará": "contar",
    "contarán": "contar",
    "contarás": "contar",
    "contaré": "contar",
    "contaréis": "contar",
    "contaría": "contar",
    "contaríais": "contar",
    "contaríamos": "contar",
    "contarían": "contar",
    "contarías": "contar",
    "contase": "contar",
    "contaseis": "contar",
    "contasen": "contar",
    "contases": "contar",
    "contaste": "contar",
    "contasteis": "contar",
    "contemos": "contar",
    "contempera": "contemperar",
    "contemperaba": "contemperar",
    "contemperabais": "contemperar",
    "contemperaban": "contemperar",
    "contemperabas": "contemperar",
    "contemperad": "contemperar",
    "contemperada": "contemperar",
    "contemperadas": "contemperar",
    "contemperado": "contemperar",
    "contemperados": "contemperar",
    "contemperamos": "contemperar",
    "contemperan": "contemperar",
    "contemperando": "contemperar",
    "contemperara": "contemperar",
    "contemperarais": "contemperar",
    "contemperaran": "contemperar",
    "contemperaras": "contemperar",
    "contemperare": "contemperar",
    "contemperareis": "contemperar",
    "contemperaremos": "contemperar",
    "contemperaren": "contemperar",
    "contemperares": "contemperar",
    "contemperaron": "contemperar",
    "contemperará": "contemperar",
    "contemperarán": "contemperar",
    "contemperarás": "contemperar",
    "contemperaré": "contemperar",
    "contemperaréis": "contemperar",
    "contemperaría": "contemperar",
    "contemperaríais": "contemperar",
    "contemperaríamos": "contemperar",
    "contemperarían": "contemperar",
    "contemperarías": "contemperar",
    "contemperas": "contemperar",
    "contemperase": "contemperar",
    "contemperaseis": "contemperar",
    "contemperasen": "contemperar",
    "contemperases": "contemperar",
    "contemperaste": "contemperar",
    "contemperasteis": "contemperar",
    "contempere": "contemperar",
    "contemperemos": "contemperar",
    "contemperen": "contemperar",
    "contemperes": "contemperar",
    "contempero": "contemperar",
    "contemperábamos": "contemperar",
    "contemperáis": "contemperar",
    "contemperáramos": "contemperar",
    "contemperáremos": "contemperar",
    "contemperásemos": "contemperar",
    "contemperé": "contemperar",
    "contemperéis": "contemperar",
    "contemperó": "contemperar",
    "contempla": "contemplar",
    "contemplaba": "contemplar",
    "contemplabais": "contemplar",
    "contemplaban": "contemplar",
    "contemplabas": "contemplar",
    "contemplaciones": "contemplación",
    "contemplad": "contemplar",
    "contemplada": "contemplar",
    "contempladas": "contemplar",
    "contemplado": "contemplar",
    "contempladora": "contemplador",
    "contempladoras": "contemplador",
    "contempladores": "contemplador",
    "contemplados": "contemplar",
    "contemplamos": "contemplar",
    "contemplan": "contemplar",
    "contemplando": "contemplar",
    "contemplara": "contemplar",
    "contemplarais": "contemplar",
    "contemplaran": "contemplar",
    "contemplaras": "contemplar",
    "contemplare": "contemplar",
    "contemplareis": "contemplar",
    "contemplaremos": "contemplar",
    "contemplaren": "contemplar",
    "contemplares": "contemplar",
    "contemplaron": "contemplar",
    "contemplará": "contemplar",
    "contemplarán": "contemplar",
    "contemplarás": "contemplar",
    "contemplaré": "contemplar",
    "contemplaréis": "contemplar",
    "contemplaría": "contemplar",
    "contemplaríais": "contemplar",
    "contemplaríamos": "contemplar",
    "contemplarían": "contemplar",
    "contemplarías": "contemplar",
    "contemplas": "contemplar",
    "contemplase": "contemplar",
    "contemplaseis": "contemplar",
    "contemplasen": "contemplar",
    "contemplases": "contemplar",
    "contemplaste": "contemplar",
    "contemplasteis": "contemplar",
    "contemplativa": "contemplativo",
    "contemplativas": "contemplativo",
    "contemplativos": "contemplativo",
    "contemple": "contemplar",
    "contemplemos": "contemplar",
    "contemplen": "contemplar",
    "contemples": "contemplar",
    "contemplo": "contemplar",
    "contemplábamos": "contemplar",
    "contempláis": "contemplar",
    "contempláramos": "contemplar",
    "contempláremos": "contemplar",
    "contemplásemos": "contemplar",
    "contemplé": "contemplar",
    "contempléis": "contemplar",
    "contempló": "contemplar",
    "contemporice": "contemporizar",
    "contemporicemos": "contemporizar",
    "contemporicen": "contemporizar",
    "contemporices": "contemporizar",
    "contemporicé": "contemporizar",
    "contemporicéis": "contemporizar",
    "contemporiza": "contemporizar",
    "contemporizaba": "contemporizar",
    "contemporizabais": "contemporizar",
    "contemporizaban": "contemporizar",
    "contemporizabas": "contemporizar",
    "contemporizaciones": "contemporización",
    "contemporizad": "contemporizar",
    "contemporizada": "contemporizar",
    "contemporizadas": "contemporizar",
    "contemporizado": "contemporizar",
    "contemporizados": "contemporizar",
    "contemporizamos": "contemporizar",
    "contemporizan": "contemporizar",
    "contemporizando": "contemporizar",
    "contemporizara": "contemporizar",
    "contemporizarais": "contemporizar",
    "contemporizaran": "contemporizar",
    "contemporizaras": "contemporizar",
    "contemporizare": "contemporizar",
    "contemporizareis": "contemporizar",
    "contemporizaremos": "contemporizar",
    "contemporizaren": "contemporizar",
    "contemporizares": "contemporizar",
    "contemporizaron": "contemporizar",
    "contemporizará": "contemporizar",
    "contemporizarán": "contemporizar",
    "contemporizarás": "contemporizar",
    "contemporizaré": "contemporizar",
    "contemporizaréis": "contemporizar",
    "contemporizaría": "contemporizar",
    "contemporizaríais": "contemporizar",
    "contemporizaríamos": "contemporizar",
    "contemporizarían": "contemporizar",
    "contemporizarías": "contemporizar",
    "contemporizas": "contemporizar",
    "contemporizase": "contemporizar",
    "contemporizaseis": "contemporizar",
    "contemporizasen": "contemporizar",
    "contemporizases": "contemporizar",
    "contemporizaste": "contemporizar",
    "contemporizasteis": "contemporizar",
    "contemporizo": "contemporizar",
    "contemporizábamos": "contemporizar",
    "contemporizáis": "contemporizar",
    "contemporizáramos": "contemporizar",
    "contemporizáremos": "contemporizar",
    "contemporizásemos": "contemporizar",
    "contemporizó": "contemporizar",
    "contemporánea": "contemporáneo",
    "contemporáneas": "contemporáneo",
    "contemporáneos": "contemporáneo",
    "contenciones": "contención",
    "contenciosa": "contencioso",
    "contenciosas": "contencioso",
    "contenciosos": "contencioso",
    "contendamos": "contender",
    "contended": "contender",
    "contendemos": "contender",
    "contenderemos": "contender",
    "contenderá": "contender",
    "contenderán": "contender",
    "contenderás": "contender",
    "contenderé": "contender",
    "contenderéis": "contender",
    "contendería": "contender",
    "contenderíais": "contender",
    "contenderíamos": "contender",
    "contenderían": "contender",
    "contenderías": "contender",
    "contendida": "contender",
    "contendidas": "contender",
    "contendido": "contender",
    "contendidos": "contender",
    "contendiendo": "contender",
    "contendientes": "contendiente",
    "contendiera": "contender",
    "contendierais": "contender",
    "contendieran": "contender",
    "contendieras": "contender",
    "contendiere": "contender",
    "contendiereis": "contender",
    "contendieren": "contender",
    "contendieres": "contender",
    "contendieron": "contender",
    "contendiese": "contender",
    "contendieseis": "contender",
    "contendiesen": "contender",
    "contendieses": "contender",
    "contendimos": "contender",
    "contendiste": "contender",
    "contendisteis": "contender",
    "contendiéramos": "contender",
    "contendiéremos": "contender",
    "contendiésemos": "contender",
    "contendió": "contender",
    "contendremos": "contener",
    "contendrá": "contener",
    "contendrán": "contener",
    "contendrás": "contener",
    "contendré": "contener",
    "contendréis": "contener",
    "contendría": "contener",
    "contendríais": "contener",
    "contendríamos": "contener",
    "contendrían": "contener",
    "contendrías": "contener",
    "contendáis": "contender",
    "contendéis": "contender",
    "contendí": "contender",
    "contendía": "contender",
    "contendíais": "contender",
    "contendíamos": "contender",
    "contendían": "contender",
    "contendías": "contender",
    "contened": "contener",
    "contenedora": "contenedor",
    "contenedoras": "contenedor",
    "contenedores": "contenedor",
    "contenedorizaciones": "contenedorización",
    "contenemos": "contener",
    "contenerizaciones": "contenerización",
    "contenga": "contener",
    "contengamos": "contener",
    "contengan": "contener",
    "contengas": "contener",
    "contengo": "contener",
    "contengáis": "contener",
    "contenida": "contener",
    "contenidas": "contener",
    "contenido": "contener",
    "contenidos": "contenido",
    "conteniendo": "contener",
    "contenta": "contento",
    "contentaba": "contentar",
    "contentabais": "contentar",
    "contentaban": "contentar",
    "contentabas": "contentar",
    "contentad": "contentar",
    "contentada": "contentar",
    "contentadas": "contentar",
    "contentadiza": "contentadizo",
    "contentadizas": "contentadizo",
    "contentadizos": "contentadizo",
    "contentado": "contentar",
    "contentados": "contentar",
    "contentamientos": "contentamiento",
    "contentamos": "contentar",
    "contentan": "contentar",
    "contentando": "contentar",
    "contentara": "contentar",
    "contentarais": "contentar",
    "contentaran": "contentar",
    "contentaras": "contentar",
    "contentare": "contentar",
    "contentareis": "contentar",
    "contentaremos": "contentar",
    "contentaren": "contentar",
    "contentares": "contentar",
    "contentaron": "contentar",
    "contentará": "contentar",
    "contentarán": "contentar",
    "contentarás": "contentar",
    "contentaré": "contentar",
    "contentaréis": "contentar",
    "contentaría": "contentar",
    "contentaríais": "contentar",
    "contentaríamos": "contentar",
    "contentarían": "contentar",
    "contentarías": "contentar",
    "contentas": "contento",
    "contentase": "contentar",
    "contentaseis": "contentar",
    "contentasen": "contentar",
    "contentases": "contentar",
    "contentaste": "contentar",
    "contentasteis": "contentar",
    "contente": "contentar",
    "contentemos": "contentar",
    "contenten": "contentar",
    "contentes": "contentar",
    "contentiva": "contentivo",
    "contentivas": "contentivo",
    "contentivos": "contentivo",
    "contento": "contentar",
    "contentos": "contento",
    "contentábamos": "contentar",
    "contentáis": "contentar",
    "contentáramos": "contentar",
    "contentáremos": "contentar",
    "contentásemos": "contentar",
    "contenté": "contentar",
    "contentéis": "contentar",
    "contentó": "contentar",
    "contenéis": "contener",
    "contenía": "contener",
    "conteníais": "contener",
    "conteníamos": "contener",
    "contenían": "contener",
    "contenías": "contener",
    "conteos": "conteo",
    "conteras": "contera",
    "conteros": "contero",
    "conterránea": "conterráneo",
    "conterráneas": "conterráneo",
    "conterráneos": "conterráneo",
    "contertulia": "contertulio",
    "contertuliana": "contertuliano",
    "contertulianas": "contertuliano",
    "contertulianos": "contertuliano",
    "contertulias": "contertulio",
    "contertulios": "contertulio",
    "contesta": "contestar",
    "contestaba": "contestar",
    "contestabais": "contestar",
    "contestaban": "contestar",
    "contestabas": "contestar",
    "contestabilidades": "contestabilidad",
    "contestaciones": "contestación",
    "contestad": "contestar",
    "contestada": "contestar",
    "contestadas": "contestar",
    "contestado": "contestar",
    "contestadores": "contestador",
    "contestados": "contestar",
    "contestamos": "contestar",
    "contestan": "contestar",
    "contestana": "contestano",
    "contestanas": "contestano",
    "contestando": "contestar",
    "contestanos": "contestano",
    "contestara": "contestar",
    "contestarais": "contestar",
    "contestaran": "contestar",
    "contestaras": "contestar",
    "contestare": "contestar",
    "contestareis": "contestar",
    "contestaremos": "contestar",
    "contestaren": "contestar",
    "contestares": "contestar",
    "contestaron": "contestar",
    "contestará": "contestar",
    "contestarán": "contestar",
    "contestarás": "contestar",
    "contestaré": "contestar",
    "contestaréis": "contestar",
    "contestaría": "contestar",
    "contestaríais": "contestar",
    "contestaríamos": "contestar",
    "contestarían": "contestar",
    "contestarías": "contestar",
    "contestas": "contestar",
    "contestase": "contestar",
    "contestaseis": "contestar",
    "contestasen": "contestar",
    "contestases": "contestar",
    "contestaste": "contestar",
    "contestasteis": "contestar",
    "contestataria": "contestatario",
    "contestatarias": "contestatario",
    "contestatarios": "contestatario",
    "conteste": "contestar",
    "contestemos": "contestar",
    "contesten": "contestar",
    "contestes": "conteste",
    "contesto": "contestar",
    "contestábamos": "contestar",
    "contestáis": "contestar",
    "contestáramos": "contestar",
    "contestáremos": "contestar",
    "contestásemos": "contestar",
    "contesté": "contestar",
    "contestéis": "contestar",
    "contestó": "contestar",
    "contextos": "contexto",
    "contextuaba": "contextuar",
    "contextuabais": "contextuar",
    "contextuaban": "contextuar",
    "contextuabas": "contextuar",
    "contextuad": "contextuar",
    "contextuada": "contextuar",
    "contextuadas": "contextuar",
    "contextuado": "contextuar",
    "contextuados": "contextuar",
    "contextuales": "contextual",
    "contextualice": "contextualizar",
    "contextualicemos": "contextualizar",
    "contextualicen": "contextualizar",
    "contextualices": "contextualizar",
    "contextualicé": "contextualizar",
    "contextualicéis": "contextualizar",
    "contextualiza": "contextualizar",
    "contextualizaba": "contextualizar",
    "contextualizabais": "contextualizar",
    "contextualizaban": "contextualizar",
    "contextualizabas": "contextualizar",
    "contextualizaciones": "contextualización",
    "contextualizad": "contextualizar",
    "contextualizada": "contextualizar",
    "contextualizadas": "contextualizar",
    "contextualizado": "contextualizar",
    "contextualizados": "contextualizar",
    "contextualizamos": "contextualizar",
    "contextualizan": "contextualizar",
    "contextualizando": "contextualizar",
    "contextualizara": "contextualizar",
    "contextualizarais": "contextualizar",
    "contextualizaran": "contextualizar",
    "contextualizaras": "contextualizar",
    "contextualizare": "contextualizar",
    "contextualizareis": "contextualizar",
    "contextualizaremos": "contextualizar",
    "contextualizaren": "contextualizar",
    "contextualizares": "contextualizar",
    "contextualizaron": "contextualizar",
    "contextualizará": "contextualizar",
    "contextualizarán": "contextualizar",
    "contextualizarás": "contextualizar",
    "contextualizaré": "contextualizar",
    "contextualizaréis": "contextualizar",
    "contextualizaría": "contextualizar",
    "contextualizaríais": "contextualizar",
    "contextualizaríamos": "contextualizar",
    "contextualizarían": "contextualizar",
    "contextualizarías": "contextualizar",
    "contextualizas": "contextualizar",
    "contextualizase": "contextualizar",
    "contextualizaseis": "contextualizar",
    "contextualizasen": "contextualizar",
    "contextualizases": "contextualizar",
    "contextualizaste": "contextualizar",
    "contextualizasteis": "contextualizar",
    "contextualizo": "contextualizar",
    "contextualizábamos": "contextualizar",
    "contextualizáis": "contextualizar",
    "contextualizáramos": "contextualizar",
    "contextualizáremos": "contextualizar",
    "contextualizásemos": "contextualizar",
    "contextualizó": "contextualizar",
    "contextuamos": "contextuar",
    "contextuando": "contextuar",
    "contextuara": "contextuar",
    "contextuarais": "contextuar",
    "contextuaran": "contextuar",
    "contextuaras": "contextuar",
    "contextuare": "contextuar",
    "contextuareis": "contextuar",
    "contextuaremos": "contextuar",
    "contextuaren": "contextuar",
    "contextuares": "contextuar",
    "contextuaron": "contextuar",
    "contextuará": "contextuar",
    "contextuarán": "contextuar",
    "contextuarás": "contextuar",
    "contextuaré": "contextuar",
    "contextuaréis": "contextuar",
    "contextuaría": "contextuar",
    "contextuaríais": "contextuar",
    "contextuaríamos": "contextuar",
    "contextuarían": "contextuar",
    "contextuarías": "contextuar",
    "contextuase": "contextuar",
    "contextuaseis": "contextuar",
    "contextuasen": "contextuar",
    "contextuases": "contextuar",
    "contextuaste": "contextuar",
    "contextuasteis": "contextuar",
    "contextuemos": "contextuar",
    "contexturas": "contextura",
    "contextuábamos": "contextuar",
    "contextuáis": "contextuar",
    "contextuáramos": "contextuar",
    "contextuáremos": "contextuar",
    "contextuásemos": "contextuar",
    "contextué": "contextuar",
    "contextuéis": "contextuar",
    "contextuó": "contextuar",
    "contextúa": "contextuar",
    "contextúan": "contextuar",
    "contextúas": "contextuar",
    "contextúe": "contextuar",
    "contextúen": "contextuar",
    "contextúes": "contextuar",
    "contextúo": "contextuar",
    "contienda": "contender",
    "contiendan": "contender",
    "contiendas": "contienda",
    "contiende": "contender",
    "contienden": "contender",
    "contiendes": "contender",
    "contiendo": "contender",
    "contiene": "contener",
    "contienen": "contener",
    "contienes": "contener",
    "contignaciones": "contignación",
    "contigua": "contiguo",
    "contiguas": "contiguo",
    "contiguos": "contiguo",
    "contigüidades": "contigüidad",
    "continencias": "continencia",
    "continentales": "continental",
    "continentalidades": "continentalidad",
    "continentes": "continente",
    "contingencias": "contingencia",
    "contingentes": "contingente",
    "contingibles": "contingible",
    "continua": "continuo",
    "continuaba": "continuar",
    "continuabais": "continuar",
    "continuaban": "continuar",
    "continuabas": "continuar",
    "continuaciones": "continuación",
    "continuad": "continuar",
    "continuada": "continuar",
    "continuadas": "continuar",
    "continuado": "continuar",
    "continuadora": "continuador",
    "continuadoras": "continuador",
    "continuadores": "continuador",
    "continuados": "continuar",
    "continuamos": "continuar",
    "continuando": "continuar",
    "continuara": "continuar",
    "continuarais": "continuar",
    "continuaran": "continuar",
    "continuaras": "continuar",
    "continuare": "continuar",
    "continuareis": "continuar",
    "continuaremos": "continuar",
    "continuaren": "continuar",
    "continuares": "continuar",
    "continuaron": "continuar",
    "continuará": "continuar",
    "continuarán": "continuar",
    "continuarás": "continuar",
    "continuaré": "continuar",
    "continuaréis": "continuar",
    "continuaría": "continuar",
    "continuaríais": "continuar",
    "continuaríamos": "continuar",
    "continuarían": "continuar",
    "continuarías": "continuar",
    "continuas": "continuo",
    "continuase": "continuar",
    "continuaseis": "continuar",
    "continuasen": "continuar",
    "continuases": "continuar",
    "continuaste": "continuar",
    "continuasteis": "continuar",
    "continuativa": "continuativo",
    "continuativas": "continuativo",
    "continuativos": "continuativo",
    "continuemos": "continuar",
    "continuidades": "continuidad",
    "continuismos": "continuismo",
    "continuistas": "continuista",
    "continuos": "continuo",
    "continuums": "continuum",
    "continuábamos": "continuar",
    "continuáis": "continuar",
    "continuáramos": "continuar",
    "continuáremos": "continuar",
    "continuásemos": "continuar",
    "continué": "continuar",
    "continuéis": "continuar",
    "continuó": "continuar",
    "continúa": "continuar",
    "continúan": "continuar",
    "continúas": "continuar",
    "continúe": "continuar",
    "continúen": "continuar",
    "continúes": "continuar",
    "continúo": "continuar",
    "contonea": "contonear",
    "contoneaba": "contonear",
    "contoneabais": "contonear",
    "contoneaban": "contonear",
    "contoneabas": "contonear",
    "contonead": "contonear",
    "contoneada": "contonear",
    "contoneadas": "contonear",
    "contoneado": "contonear",
    "contoneados": "contonear",
    "contoneamos": "contonear",
    "contonean": "contonear",
    "contoneando": "contonear",
    "contoneara": "contonear",
    "contonearais": "contonear",
    "contonearan": "contonear",
    "contonearas": "contonear",
    "contoneare": "contonear",
    "contoneareis": "contonear",
    "contonearemos": "contonear",
    "contonearen": "contonear",
    "contoneares": "contonear",
    "contonearon": "contonear",
    "contoneará": "contonear",
    "contonearán": "contonear",
    "contonearás": "contonear",
    "contonearé": "contonear",
    "contonearéis": "contonear",
    "contonearía": "contonear",
    "contonearíais": "contonear",
    "contonearíamos": "contonear",
    "contonearían": "contonear",
    "contonearías": "contonear",
    "contoneas": "contonear",
    "contonease": "contonear",
    "contoneaseis": "contonear",
    "contoneasen": "contonear",
    "contoneases": "contonear",
    "contoneaste": "contonear",
    "contoneasteis": "contonear",
    "contonee": "contonear",
    "contoneemos": "contonear",
    "contoneen": "contonear",
    "contonees": "contonear",
    "contoneo": "contonear",
    "contoneos": "contoneo",
    "contoneábamos": "contonear",
    "contoneáis": "contonear",
    "contoneáramos": "contonear",
    "contoneáremos": "contonear",
    "contoneásemos": "contonear",
    "contoneé": "contonear",
    "contoneéis": "contonear",
    "contoneó": "contonear",
    "contorna": "contornar",
    "contornaba": "contornar",
    "contornabais": "contornar",
    "contornaban": "contornar",
    "contornabas": "contornar",
    "contornad": "contornar",
    "contornada": "contornar",
    "contornadas": "contornar",
    "contornado": "contornar",
    "contornados": "contornar",
    "contornamos": "contornar",
    "contornan": "contornar",
    "contornando": "contornar",
    "contornara": "contornar",
    "contornarais": "contornar",
    "contornaran": "contornar",
    "contornaras": "contornar",
    "contornare": "contornar",
    "contornareis": "contornar",
    "contornaremos": "contornar",
    "contornaren": "contornar",
    "contornares": "contornar",
    "contornaron": "contornar",
    "contornará": "contornar",
    "contornarán": "contornar",
    "contornarás": "contornar",
    "contornaré": "contornar",
    "contornaréis": "contornar",
    "contornaría": "contornar",
    "contornaríais": "contornar",
    "contornaríamos": "contornar",
    "contornarían": "contornar",
    "contornarías": "contornar",
    "contornas": "contornar",
    "contornase": "contornar",
    "contornaseis": "contornar",
    "contornasen": "contornar",
    "contornases": "contornar",
    "contornaste": "contornar",
    "contornasteis": "contornar",
    "contorne": "contornar",
    "contornea": "contornear",
    "contorneaba": "contornear",
    "contorneabais": "contornear",
    "contorneaban": "contornear",
    "contorneabas": "contornear",
    "contornead": "contornear",
    "contorneada": "contornear",
    "contorneadas": "contornear",
    "contorneado": "contornear",
    "contorneados": "contornear",
    "contorneamientos": "contorneamiento",
    "contorneamos": "contornear",
    "contornean": "contornear",
    "contorneando": "contornear",
    "contorneara": "contornear",
    "contornearais": "contornear",
    "contornearan": "contornear",
    "contornearas": "contornear",
    "contorneare": "contornear",
    "contorneareis": "contornear",
    "contornearemos": "contornear",
    "contornearen": "contornear",
    "contorneares": "contornear",
    "contornearon": "contornear",
    "contorneará": "contornear",
    "contornearán": "contornear",
    "contornearás": "contornear",
    "contornearé": "contornear",
    "contornearéis": "contornear",
    "contornearía": "contornear",
    "contornearíais": "contornear",
    "contornearíamos": "contornear",
    "contornearían": "contornear",
    "contornearías": "contornear",
    "contorneas": "contornear",
    "contornease": "contornear",
    "contorneaseis": "contornear",
    "contorneasen": "contornear",
    "contorneases": "contornear",
    "contorneaste": "contornear",
    "contorneasteis": "contornear",
    "contornee": "contornear",
    "contorneemos": "contornear",
    "contorneen": "contornear",
    "contornees": "contornear",
    "contornemos": "contornar",
    "contornen": "contornar",
    "contorneo": "contornear",
    "contorneos": "contorneo",
    "contornes": "contornar",
    "contorneábamos": "contornear",
    "contorneáis": "contornear",
    "contorneáramos": "contornear",
    "contorneáremos": "contornear",
    "contorneásemos": "contornear",
    "contorneé": "contornear",
    "contorneéis": "contornear",
    "contorneó": "contornear",
    "contornitas": "contornita",
    "contorno": "contornar",
    "contornos": "contorno",
    "contornábamos": "contornar",
    "contornáis": "contornar",
    "contornáramos": "contornar",
    "contornáremos": "contornar",
    "contornásemos": "contornar",
    "contorné": "contornar",
    "contornéis": "contornar",
    "contornó": "contornar",
    "contorsiona": "contorsionar",
    "contorsionaba": "contorsionar",
    "contorsionabais": "contorsionar",
    "contorsionaban": "contorsionar",
    "contorsionabas": "contorsionar",
    "contorsionad": "contorsionar",
    "contorsionada": "contorsionar",
    "contorsionadas": "contorsionar",
    "contorsionado": "contorsionar",
    "contorsionados": "contorsionar",
    "contorsionamos": "contorsionar",
    "contorsionan": "contorsionar",
    "contorsionando": "contorsionar",
    "contorsionara": "contorsionar",
    "contorsionarais": "contorsionar",
    "contorsionaran": "contorsionar",
    "contorsionaras": "contorsionar",
    "contorsionare": "contorsionar",
    "contorsionareis": "contorsionar",
    "contorsionaremos": "contorsionar",
    "contorsionaren": "contorsionar",
    "contorsionares": "contorsionar",
    "contorsionaron": "contorsionar",
    "contorsionará": "contorsionar",
    "contorsionarán": "contorsionar",
    "contorsionarás": "contorsionar",
    "contorsionaré": "contorsionar",
    "contorsionaréis": "contorsionar",
    "contorsionaría": "contorsionar",
    "contorsionaríais": "contorsionar",
    "contorsionaríamos": "contorsionar",
    "contorsionarían": "contorsionar",
    "contorsionarías": "contorsionar",
    "contorsionas": "contorsionar",
    "contorsionase": "contorsionar",
    "contorsionaseis": "contorsionar",
    "contorsionasen": "contorsionar",
    "contorsionases": "contorsionar",
    "contorsionaste": "contorsionar",
    "contorsionasteis": "contorsionar",
    "contorsione": "contorsionar",
    "contorsionemos": "contorsionar",
    "contorsionen": "contorsionar",
    "contorsiones": "contorsionar",
    "contorsionismos": "contorsionismo",
    "contorsionistas": "contorsionista",
    "contorsiono": "contorsionar",
    "contorsionábamos": "contorsionar",
    "contorsionáis": "contorsionar",
    "contorsionáramos": "contorsionar",
    "contorsionáremos": "contorsionar",
    "contorsionásemos": "contorsionar",
    "contorsioné": "contorsionar",
    "contorsionéis": "contorsionar",
    "contorsionó": "contorsionar",
    "contraacciones": "contraacción",
    "contraalmirantes": "contraalmirante",
    "contraamuras": "contraamura",
    "contraantenas": "contraantena",
    "contraapelaciones": "contraapelación",
    "contraargumentos": "contraargumento",
    "contraarmaduras": "contraarmadura",
    "contraataca": "contraatacar",
    "contraatacaba": "contraatacar",
    "contraatacabais": "contraatacar",
    "contraatacaban": "contraatacar",
    "contraatacabas": "contraatacar",
    "contraatacad": "contraatacar",
    "contraatacada": "contraatacar",
    "contraatacadas": "contraatacar",
    "contraatacado": "contraatacar",
    "contraatacados": "contraatacar",
    "contraatacamos": "contraatacar",
    "contraatacan": "contraatacar",
    "contraatacando": "contraatacar",
    "contraatacara": "contraatacar",
    "contraatacarais": "contraatacar",
    "contraatacaran": "contraatacar",
    "contraatacaras": "contraatacar",
    "contraatacare": "contraatacar",
    "contraatacareis": "contraatacar",
    "contraatacaremos": "contraatacar",
    "contraatacaren": "contraatacar",
    "contraatacares": "contraatacar",
    "contraatacaron": "contraatacar",
    "contraatacará": "contraatacar",
    "contraatacarán": "contraatacar",
    "contraatacarás": "contraatacar",
    "contraatacaré": "contraatacar",
    "contraatacaréis": "contraatacar",
    "contraatacaría": "contraatacar",
    "contraatacaríais": "contraatacar",
    "contraatacaríamos": "contraatacar",
    "contraatacarían": "contraatacar",
    "contraatacarías": "contraatacar",
    "contraatacas": "contraatacar",
    "contraatacase": "contraatacar",
    "contraatacaseis": "contraatacar",
    "contraatacasen": "contraatacar",
    "contraatacases": "contraatacar",
    "contraatacaste": "contraatacar",
    "contraatacasteis": "contraatacar",
    "contraataco": "contraatacar",
    "contraatacábamos": "contraatacar",
    "contraatacáis": "contraatacar",
    "contraatacáramos": "contraatacar",
    "contraatacáremos": "contraatacar",
    "contraatacásemos": "contraatacar",
    "contraatacó": "contraatacar",
    "contraataguías": "contraataguía",
    "contraataque": "contraatacar",
    "contraataquemos": "contraatacar",
    "contraataquen": "contraatacar",
    "contraataques": "contraataque",
    "contraataqué": "contraatacar",
    "contraataquéis": "contraatacar",
    "contrabajetes": "contrabajete",
    "contrabajones": "contrabajón",
    "contrabajos": "contrabajo",
    "contrabalancea": "contrabalancear",
    "contrabalanceaba": "contrabalancear",
    "contrabalanceabais": "contrabalancear",
    "contrabalanceaban": "contrabalancear",
    "contrabalanceabas": "contrabalancear",
    "contrabalancead": "contrabalancear",
    "contrabalanceada": "contrabalancear",
    "contrabalanceadas": "contrabalancear",
    "contrabalanceado": "contrabalancear",
    "contrabalanceados": "contrabalancear",
    "contrabalanceamos": "contrabalancear",
    "contrabalancean": "contrabalancear",
    "contrabalanceando": "contrabalancear",
    "contrabalanceara": "contrabalancear",
    "contrabalancearais": "contrabalancear",
    "contrabalancearan": "contrabalancear",
    "contrabalancearas": "contrabalancear",
    "contrabalanceare": "contrabalancear",
    "contrabalanceareis": "contrabalancear",
    "contrabalancearemos": "contrabalancear",
    "contrabalancearen": "contrabalancear",
    "contrabalanceares": "contrabalancear",
    "contrabalancearon": "contrabalancear",
    "contrabalanceará": "contrabalancear",
    "contrabalancearán": "contrabalancear",
    "contrabalancearás": "contrabalancear",
    "contrabalancearé": "contrabalancear",
    "contrabalancearéis": "contrabalancear",
    "contrabalancearía": "contrabalancear",
    "contrabalancearíais": "contrabalancear",
    "contrabalancearíamos": "contrabalancear",
    "contrabalancearían": "contrabalancear",
    "contrabalancearías": "contrabalancear",
    "contrabalanceas": "contrabalancear",
    "contrabalancease": "contrabalancear",
    "contrabalanceaseis": "contrabalancear",
    "contrabalanceasen": "contrabalancear",
    "contrabalanceases": "contrabalancear",
    "contrabalanceaste": "contrabalancear",
    "contrabalanceasteis": "contrabalancear",
    "contrabalancee": "contrabalancear",
    "contrabalanceemos": "contrabalancear",
    "contrabalanceen": "contrabalancear",
    "contrabalancees": "contrabalancear",
    "contrabalanceo": "contrabalancear",
    "contrabalances": "contrabalance",
    "contrabalanceábamos": "contrabalancear",
    "contrabalanceáis": "contrabalancear",
    "contrabalanceáramos": "contrabalancear",
    "contrabalanceáremos": "contrabalancear",
    "contrabalanceásemos": "contrabalancear",
    "contrabalanceé": "contrabalancear",
    "contrabalanceéis": "contrabalancear",
    "contrabalanceó": "contrabalancear",
    "contrabalanzas": "contrabalanza",
    "contrabandas": "contrabanda",
    "contrabandea": "contrabandear",
    "contrabandeaba": "contrabandear",
    "contrabandeabais": "contrabandear",
    "contrabandeaban": "contrabandear",
    "contrabandeabas": "contrabandear",
    "contrabandead": "contrabandear",
    "contrabandeada": "contrabandear",
    "contrabandeadas": "contrabandear",
    "contrabandeado": "contrabandear",
    "contrabandeados": "contrabandear",
    "contrabandeamos": "contrabandear",
    "contrabandean": "contrabandear",
    "contrabandeando": "contrabandear",
    "contrabandeara": "contrabandear",
    "contrabandearais": "contrabandear",
    "contrabandearan": "contrabandear",
    "contrabandearas": "contrabandear",
    "contrabandeare": "contrabandear",
    "contrabandeareis": "contrabandear",
    "contrabandearemos": "contrabandear",
    "contrabandearen": "contrabandear",
    "contrabandeares": "contrabandear",
    "contrabandearon": "contrabandear",
    "contrabandeará": "contrabandear",
    "contrabandearán": "contrabandear",
    "contrabandearás": "contrabandear",
    "contrabandearé": "contrabandear",
    "contrabandearéis": "contrabandear",
    "contrabandearía": "contrabandear",
    "contrabandearíais": "contrabandear",
    "contrabandearíamos": "contrabandear",
    "contrabandearían": "contrabandear",
    "contrabandearías": "contrabandear",
    "contrabandeas": "contrabandear",
    "contrabandease": "contrabandear",
    "contrabandeaseis": "contrabandear",
    "contrabandeasen": "contrabandear",
    "contrabandeases": "contrabandear",
    "contrabandeaste": "contrabandear",
    "contrabandeasteis": "contrabandear",
    "contrabandee": "contrabandear",
    "contrabandeemos": "contrabandear",
    "contrabandeen": "contrabandear",
    "contrabandees": "contrabandear",
    "contrabandeo": "contrabandear",
    "contrabandeábamos": "contrabandear",
    "contrabandeáis": "contrabandear",
    "contrabandeáramos": "contrabandear",
    "contrabandeáremos": "contrabandear",
    "contrabandeásemos": "contrabandear",
    "contrabandeé": "contrabandear",
    "contrabandeéis": "contrabandear",
    "contrabandeó": "contrabandear",
    "contrabandistas": "contrabandista",
    "contrabandos": "contrabando",
    "contrabarreras": "contrabarrera",
    "contrabasas": "contrabasa",
    "contrabata": "contrabatir",
    "contrabatamos": "contrabatir",
    "contrabatan": "contrabatir",
    "contrabatas": "contrabatir",
    "contrabate": "contrabatir",
    "contrabaten": "contrabatir",
    "contrabaterías": "contrabatería",
    "contrabates": "contrabatir",
    "contrabatid": "contrabatir",
    "contrabatida": "contrabatir",
    "contrabatidas": "contrabatir",
    "contrabatido": "contrabatir",
    "contrabatidos": "contrabatir",
    "contrabatiendo": "contrabatir",
    "contrabatiera": "contrabatir",
    "contrabatierais": "contrabatir",
    "contrabatieran": "contrabatir",
    "contrabatieras": "contrabatir",
    "contrabatiere": "contrabatir",
    "contrabatiereis": "contrabatir",
    "contrabatieren": "contrabatir",
    "contrabatieres": "contrabatir",
    "contrabatieron": "contrabatir",
    "contrabatiese": "contrabatir",
    "contrabatieseis": "contrabatir",
    "contrabatiesen": "contrabatir",
    "contrabatieses": "contrabatir",
    "contrabatimos": "contrabatir",
    "contrabatiremos": "contrabatir",
    "contrabatirá": "contrabatir",
    "contrabatirán": "contrabatir",
    "contrabatirás": "contrabatir",
    "contrabatiré": "contrabatir",
    "contrabatiréis": "contrabatir",
    "contrabatiría": "contrabatir",
    "contrabatiríais": "contrabatir",
    "contrabatiríamos": "contrabatir",
    "contrabatirían": "contrabatir",
    "contrabatirías": "contrabatir",
    "contrabatiste": "contrabatir",
    "contrabatisteis": "contrabatir",
    "contrabatiéramos": "contrabatir",
    "contrabatiéremos": "contrabatir",
    "contrabatiésemos": "contrabatir",
    "contrabatió": "contrabatir",
    "contrabato": "contrabatir",
    "contrabatáis": "contrabatir",
    "contrabatí": "contrabatir",
    "contrabatía": "contrabatir",
    "contrabatíais": "contrabatir",
    "contrabatíamos": "contrabatir",
    "contrabatían": "contrabatir",
    "contrabatías": "contrabatir",
    "contrabatís": "contrabatir",
    "contrabloqueos": "contrabloqueo",
    "contrabolinas": "contrabolina",
    "contrabranques": "contrabranque",
    "contrabrazas": "contrabraza",
    "contracajas": "contracaja",
    "contracambios": "contracambio",
    "contracanales": "contracanal",
    "contracanchas": "contracancha",
    "contracarriles": "contracarril",
    "contracartas": "contracarta",
    "contracatalinas": "contracatalina",
    "contracciones": "contracción",
    "contracepciones": "contracepción",
    "contrachapa": "contrachapar",
    "contrachapaba": "contrachapar",
    "contrachapabais": "contrachapar",
    "contrachapaban": "contrachapar",
    "contrachapabas": "contrachapar",
    "contrachapad": "contrachapar",
    "contrachapada": "contrachapar",
    "contrachapadas": "contrachapar",
    "contrachapado": "contrachapar",
    "contrachapados": "contrachapar",
    "contrachapamos": "contrachapar",
    "contrachapan": "contrachapar",
    "contrachapando": "contrachapar",
    "contrachapara": "contrachapar",
    "contrachaparais": "contrachapar",
    "contrachaparan": "contrachapar",
    "contrachaparas": "contrachapar",
    "contrachapare": "contrachapar",
    "contrachapareis": "contrachapar",
    "contrachaparemos": "contrachapar",
    "contrachaparen": "contrachapar",
    "contrachapares": "contrachapar",
    "contrachaparon": "contrachapar",
    "contrachapará": "contrachapar",
    "contrachaparán": "contrachapar",
    "contrachaparás": "contrachapar",
    "contrachaparé": "contrachapar",
    "contrachaparéis": "contrachapar",
    "contrachaparía": "contrachapar",
    "contrachaparíais": "contrachapar",
    "contrachaparíamos": "contrachapar",
    "contrachaparían": "contrachapar",
    "contrachaparías": "contrachapar",
    "contrachapas": "contrachapar",
    "contrachapase": "contrachapar",
    "contrachapaseis": "contrachapar",
    "contrachapasen": "contrachapar",
    "contrachapases": "contrachapar",
    "contrachapaste": "contrachapar",
    "contrachapasteis": "contrachapar",
    "contrachape": "contrachapar",
    "contrachapea": "contrachapear",
    "contrachapeaba": "contrachapear",
    "contrachapeabais": "contrachapear",
    "contrachapeaban": "contrachapear",
    "contrachapeabas": "contrachapear",
    "contrachapead": "contrachapear",
    "contrachapeada": "contrachapear",
    "contrachapeadas": "contrachapear",
    "contrachapeado": "contrachapear",
    "contrachapeados": "contrachapear",
    "contrachapeamos": "contrachapear",
    "contrachapean": "contrachapear",
    "contrachapeando": "contrachapear",
    "contrachapeara": "contrachapear",
    "contrachapearais": "contrachapear",
    "contrachapearan": "contrachapear",
    "contrachapearas": "contrachapear",
    "contrachapeare": "contrachapear",
    "contrachapeareis": "contrachapear",
    "contrachapearemos": "contrachapear",
    "contrachapearen": "contrachapear",
    "contrachapeares": "contrachapear",
    "contrachapearon": "contrachapear",
    "contrachapeará": "contrachapear",
    "contrachapearán": "contrachapear",
    "contrachapearás": "contrachapear",
    "contrachapearé": "contrachapear",
    "contrachapearéis": "contrachapear",
    "contrachapearía": "contrachapear",
    "contrachapearíais": "contrachapear",
    "contrachapearíamos": "contrachapear",
    "contrachapearían": "contrachapear",
    "contrachapearías": "contrachapear",
    "contrachapeas": "contrachapear",
    "contrachapease": "contrachapear",
    "contrachapeaseis": "contrachapear",
    "contrachapeasen": "contrachapear",
    "contrachapeases": "contrachapear",
    "contrachapeaste": "contrachapear",
    "contrachapeasteis": "contrachapear",
    "contrachapee": "contrachapear",
    "contrachapeemos": "contrachapear",
    "contrachapeen": "contrachapear",
    "contrachapees": "contrachapear",
    "contrachapemos": "contrachapar",
    "contrachapen": "contrachapar",
    "contrachapeo": "contrachapear",
    "contrachapes": "contrachapar",
    "contrachapeábamos": "contrachapear",
    "contrachapeáis": "contrachapear",
    "contrachapeáramos": "contrachapear",
    "contrachapeáremos": "contrachapear",
    "contrachapeásemos": "contrachapear",
    "contrachapeé": "contrachapear",
    "contrachapeéis": "contrachapear",
    "contrachapeó": "contrachapear",
    "contrachapo": "contrachapar",
    "contrachapábamos": "contrachapar",
    "contrachapáis": "contrachapar",
    "contrachapáramos": "contrachapar",
    "contrachapáremos": "contrachapar",
    "contrachapásemos": "contrachapar",
    "contrachapé": "contrachapar",
    "contrachapéis": "contrachapar",
    "contrachapó": "contrachapar",
    "contracifras": "contracifra",
    "contraclaves": "contraclave",
    "contracodastes": "contracodaste",
    "contraconcepciones": "contraconcepción",
    "contracorrientes": "contracorriente",
    "contracostas": "contracosta",
    "contractiva": "contractivo",
    "contractivas": "contractivo",
    "contractivos": "contractivo",
    "contractuales": "contractual",
    "contracturas": "contractura",
    "contracubiertas": "contracubierta",
    "contracuentas": "contracuenta",
    "contraculturas": "contracultura",
    "contracunetas": "contracuneta",
    "contracurvas": "contracurva",
    "contracédulas": "contracédula",
    "contradanzas": "contradanza",
    "contradecid": "contradecir",
    "contradecimos": "contradecir",
    "contradeciremos": "contradecir",
    "contradecirá": "contradecir",
    "contradecirán": "contradecir",
    "contradecirás": "contradecir",
    "contradeciré": "contradecir",
    "contradeciréis": "contradecir",
    "contradeciría": "contradecir",
    "contradeciríais": "contradecir",
    "contradeciríamos": "contradecir",
    "contradecirían": "contradecir",
    "contradecirías": "contradecir",
    "contradeclaraciones": "contradeclaración",
    "contradecía": "contradecir",
    "contradecíais": "contradecir",
    "contradecíamos": "contradecir",
    "contradecían": "contradecir",
    "contradecías": "contradecir",
    "contradecís": "contradecir",
    "contrademandantes": "contrademandante",
    "contrademandas": "contrademanda",
    "contrademostraciones": "contrademostración",
    "contradenuncias": "contradenuncia",
    "contraderivas": "contraderiva",
    "contradicciones": "contradicción",
    "contradice": "contradecir",
    "contradicen": "contradecir",
    "contradices": "contradecir",
    "contradicha": "contradecir",
    "contradichas": "contradecir",
    "contradicho": "contradecir",
    "contradichos": "contradecir",
    "contradiciendo": "contradecir",
    "contradictoria": "contradictorio",
    "contradictorias": "contradictorio",
    "contradictorios": "contradictorio",
    "contradiga": "contradecir",
    "contradigamos": "contradecir",
    "contradigan": "contradecir",
    "contradigas": "contradecir",
    "contradigo": "contradecir",
    "contradigáis": "contradecir",
    "contradije": "contradecir",
    "contradijera": "contradecir",
    "contradijerais": "contradecir",
    "contradijeran": "contradecir",
    "contradijeras": "contradecir",
    "contradijere": "contradecir",
    "contradijereis": "contradecir",
    "contradijeren": "contradecir",
    "contradijeres": "contradecir",
    "contradijeron": "contradecir",
    "contradijese": "contradecir",
    "contradijeseis": "contradecir",
    "contradijesen": "contradecir",
    "contradijeses": "contradecir",
    "contradijimos": "contradecir",
    "contradijiste": "contradecir",
    "contradijisteis": "contradecir",
    "contradijo": "contradecir",
    "contradijéramos": "contradecir",
    "contradijéremos": "contradecir",
    "contradijésemos": "contradecir",
    "contradiques": "contradique",
    "contradrizas": "contradriza",
    "contrae": "contraer",
    "contraed": "contraer",
    "contraejemplos": "contraejemplo",
    "contraelectromotrices": "contraelectromotriz",
    "contraemboscadas": "contraemboscada",
    "contraemos": "contraer",
    "contraen": "contraer",
    "contraendosa": "contraendosar",
    "contraendosaba": "contraendosar",
    "contraendosabais": "contraendosar",
    "contraendosaban": "contraendosar",
    "contraendosabas": "contraendosar",
    "contraendosad": "contraendosar",
    "contraendosada": "contraendosar",
    "contraendosadas": "contraendosar",
    "contraendosado": "contraendosar",
    "contraendosados": "contraendosar",
    "contraendosamos": "contraendosar",
    "contraendosan": "contraendosar",
    "contraendosando": "contraendosar",
    "contraendosara": "contraendosar",
    "contraendosarais": "contraendosar",
    "contraendosaran": "contraendosar",
    "contraendosaras": "contraendosar",
    "contraendosare": "contraendosar",
    "contraendosareis": "contraendosar",
    "contraendosaremos": "contraendosar",
    "contraendosaren": "contraendosar",
    "contraendosares": "contraendosar",
    "contraendosaron": "contraendosar",
    "contraendosará": "contraendosar",
    "contraendosarán": "contraendosar",
    "contraendosarás": "contraendosar",
    "contraendosaré": "contraendosar",
    "contraendosaréis": "contraendosar",
    "contraendosaría": "contraendosar",
    "contraendosaríais": "contraendosar",
    "contraendosaríamos": "contraendosar",
    "contraendosarían": "contraendosar",
    "contraendosarías": "contraendosar",
    "contraendosas": "contraendosar",
    "contraendosase": "contraendosar",
    "contraendosaseis": "contraendosar",
    "contraendosasen": "contraendosar",
    "contraendosases": "contraendosar",
    "contraendosaste": "contraendosar",
    "contraendosasteis": "contraendosar",
    "contraendose": "contraendosar",
    "contraendosemos": "contraendosar",
    "contraendosen": "contraendosar",
    "contraendoses": "contraendosar",
    "contraendoso": "contraendosar",
    "contraendosábamos": "contraendosar",
    "contraendosáis": "contraendosar",
    "contraendosáramos": "contraendosar",
    "contraendosáremos": "contraendosar",
    "contraendosásemos": "contraendosar",
    "contraendosé": "contraendosar",
    "contraendoséis": "contraendosar",
    "contraendosó": "contraendosar",
    "contraeremos": "contraer",
    "contraerá": "contraer",
    "contraerán": "contraer",
    "contraerás": "contraer",
    "contraeré": "contraer",
    "contraeréis": "contraer",
    "contraería": "contraer",
    "contraeríais": "contraer",
    "contraeríamos": "contraer",
    "contraerían": "contraer",
    "contraerías": "contraer",
    "contraes": "contraer",
    "contraescarpas": "contraescarpa",
    "contraescotas": "contraescota",
    "contraescotines": "contraescotín",
    "contraescrituras": "contraescritura",
    "contraespeculaciones": "contraespeculación",
    "contraespionajes": "contraespionaje",
    "contraestayes": "contraestay",
    "contrafachadas": "contrafachada",
    "contrafajas": "contrafaja",
    "contrafalla": "contrafallar",
    "contrafallaba": "contrafallar",
    "contrafallabais": "contrafallar",
    "contrafallaban": "contrafallar",
    "contrafallabas": "contrafallar",
    "contrafallad": "contrafallar",
    "contrafallada": "contrafallar",
    "contrafalladas": "contrafallar",
    "contrafallado": "contrafallar",
    "contrafallados": "contrafallar",
    "contrafallamos": "contrafallar",
    "contrafallan": "contrafallar",
    "contrafallando": "contrafallar",
    "contrafallara": "contrafallar",
    "contrafallarais": "contrafallar",
    "contrafallaran": "contrafallar",
    "contrafallaras": "contrafallar",
    "contrafallare": "contrafallar",
    "contrafallareis": "contrafallar",
    "contrafallaremos": "contrafallar",
    "contrafallaren": "contrafallar",
    "contrafallares": "contrafallar",
    "contrafallaron": "contrafallar",
    "contrafallará": "contrafallar",
    "contrafallarán": "contrafallar",
    "contrafallarás": "contrafallar",
    "contrafallaré": "contrafallar",
    "contrafallaréis": "contrafallar",
    "contrafallaría": "contrafallar",
    "contrafallaríais": "contrafallar",
    "contrafallaríamos": "contrafallar",
    "contrafallarían": "contrafallar",
    "contrafallarías": "contrafallar",
    "contrafallas": "contrafallar",
    "contrafallase": "contrafallar",
    "contrafallaseis": "contrafallar",
    "contrafallasen": "contrafallar",
    "contrafallases": "contrafallar",
    "contrafallaste": "contrafallar",
    "contrafallasteis": "contrafallar",
    "contrafalle": "contrafallar",
    "contrafallemos": "contrafallar",
    "contrafallen": "contrafallar",
    "contrafalles": "contrafallar",
    "contrafallo": "contrafallar",
    "contrafallábamos": "contrafallar",
    "contrafalláis": "contrafallar",
    "contrafalláramos": "contrafallar",
    "contrafalláremos": "contrafallar",
    "contrafallásemos": "contrafallar",
    "contrafallé": "contrafallar",
    "contrafalléis": "contrafallar",
    "contrafalló": "contrafallar",
    "contrafases": "contrafase",
    "contrafianzas": "contrafianza",
    "contrafiguras": "contrafigura",
    "contrafilos": "contrafilo",
    "contrafinales": "contrafinal",
    "contrafirma": "contrafirmar",
    "contrafirmaba": "contrafirmar",
    "contrafirmabais": "contrafirmar",
    "contrafirmaban": "contrafirmar",
    "contrafirmabas": "contrafirmar",
    "contrafirmad": "contrafirmar",
    "contrafirmada": "contrafirmar",
    "contrafirmadas": "contrafirmar",
    "contrafirmado": "contrafirmar",
    "contrafirmados": "contrafirmar",
    "contrafirmamos": "contrafirmar",
    "contrafirman": "contrafirmar",
    "contrafirmando": "contrafirmar",
    "contrafirmara": "contrafirmar",
    "contrafirmarais": "contrafirmar",
    "contrafirmaran": "contrafirmar",
    "contrafirmaras": "contrafirmar",
    "contrafirmare": "contrafirmar",
    "contrafirmareis": "contrafirmar",
    "contrafirmaremos": "contrafirmar",
    "contrafirmaren": "contrafirmar",
    "contrafirmares": "contrafirmar",
    "contrafirmaron": "contrafirmar",
    "contrafirmará": "contrafirmar",
    "contrafirmarán": "contrafirmar",
    "contrafirmarás": "contrafirmar",
    "contrafirmaré": "contrafirmar",
    "contrafirmaréis": "contrafirmar",
    "contrafirmaría": "contrafirmar",
    "contrafirmaríais": "contrafirmar",
    "contrafirmaríamos": "contrafirmar",
    "contrafirmarían": "contrafirmar",
    "contrafirmarías": "contrafirmar",
    "contrafirmas": "contrafirmar",
    "contrafirmase": "contrafirmar",
    "contrafirmaseis": "contrafirmar",
    "contrafirmasen": "contrafirmar",
    "contrafirmases": "contrafirmar",
    "contrafirmaste": "contrafirmar",
    "contrafirmasteis": "contrafirmar",
    "contrafirme": "contrafirmar",
    "contrafirmemos": "contrafirmar",
    "contrafirmen": "contrafirmar",
    "contrafirmes": "contrafirmar",
    "contrafirmo": "contrafirmar",
    "contrafirmábamos": "contrafirmar",
    "contrafirmáis": "contrafirmar",
    "contrafirmáramos": "contrafirmar",
    "contrafirmáremos": "contrafirmar",
    "contrafirmásemos": "contrafirmar",
    "contrafirmé": "contrafirmar",
    "contrafirméis": "contrafirmar",
    "contrafirmó": "contrafirmar",
    "contrafoques": "contrafoque",
    "contrafosos": "contrafoso",
    "contrafuegos": "contrafuego",
    "contrafueros": "contrafuero",
    "contrafuertes": "contrafuerte",
    "contrafugas": "contrafuga",
    "contragolpes": "contragolpe",
    "contraguardias": "contraguardia",
    "contraguerrillas": "contraguerrilla",
    "contraguiña": "contraguiñar",
    "contraguiñaba": "contraguiñar",
    "contraguiñabais": "contraguiñar",
    "contraguiñaban": "contraguiñar",
    "contraguiñabas": "contraguiñar",
    "contraguiñad": "contraguiñar",
    "contraguiñada": "contraguiñar",
    "contraguiñadas": "contraguiñar",
    "contraguiñado": "contraguiñar",
    "contraguiñados": "contraguiñar",
    "contraguiñamos": "contraguiñar",
    "contraguiñan": "contraguiñar",
    "contraguiñando": "contraguiñar",
    "contraguiñara": "contraguiñar",
    "contraguiñarais": "contraguiñar",
    "contraguiñaran": "contraguiñar",
    "contraguiñaras": "contraguiñar",
    "contraguiñare": "contraguiñar",
    "contraguiñareis": "contraguiñar",
    "contraguiñaremos": "contraguiñar",
    "contraguiñaren": "contraguiñar",
    "contraguiñares": "contraguiñar",
    "contraguiñaron": "contraguiñar",
    "contraguiñará": "contraguiñar",
    "contraguiñarán": "contraguiñar",
    "contraguiñarás": "contraguiñar",
    "contraguiñaré": "contraguiñar",
    "contraguiñaréis": "contraguiñar",
    "contraguiñaría": "contraguiñar",
    "contraguiñaríais": "contraguiñar",
    "contraguiñaríamos": "contraguiñar",
    "contraguiñarían": "contraguiñar",
    "contraguiñarías": "contraguiñar",
    "contraguiñas": "contraguiñar",
    "contraguiñase": "contraguiñar",
    "contraguiñaseis": "contraguiñar",
    "contraguiñasen": "contraguiñar",
    "contraguiñases": "contraguiñar",
    "contraguiñaste": "contraguiñar",
    "contraguiñasteis": "contraguiñar",
    "contraguiñe": "contraguiñar",
    "contraguiñemos": "contraguiñar",
    "contraguiñen": "contraguiñar",
    "contraguiñes": "contraguiñar",
    "contraguiño": "contraguiñar",
    "contraguiñábamos": "contraguiñar",
    "contraguiñáis": "contraguiñar",
    "contraguiñáramos": "contraguiñar",
    "contraguiñáremos": "contraguiñar",
    "contraguiñásemos": "contraguiñar",
    "contraguiñé": "contraguiñar",
    "contraguiñéis": "contraguiñar",
    "contraguiñó": "contraguiñar",
    "contraguías": "contraguía",
    "contrahace": "contrahacer",
    "contrahaced": "contrahacer",
    "contrahacemos": "contrahacer",
    "contrahacen": "contrahacer",
    "contrahaces": "contrahacer",
    "contrahaciendo": "contrahacer",
    "contrahacéis": "contrahacer",
    "contrahacía": "contrahacer",
    "contrahacíais": "contrahacer",
    "contrahacíamos": "contrahacer",
    "contrahacían": "contrahacer",
    "contrahacías": "contrahacer",
    "contrahaga": "contrahacer",
    "contrahagamos": "contrahacer",
    "contrahagan": "contrahacer",
    "contrahagas": "contrahacer",
    "contrahago": "contrahacer",
    "contrahagáis": "contrahacer",
    "contraharemos": "contrahacer",
    "contrahará": "contrahacer",
    "contraharán": "contrahacer",
    "contraharás": "contrahacer",
    "contraharé": "contrahacer",
    "contraharéis": "contrahacer",
    "contraharía": "contrahacer",
    "contraharíais": "contrahacer",
    "contraharíamos": "contrahacer",
    "contraharían": "contrahacer",
    "contraharías": "contrahacer",
    "contrahaz": "contrahacer",
    "contrahecha": "contrahacer",
    "contrahechas": "contrahacer",
    "contrahecho": "contrahacer",
    "contrahechos": "contrahacer",
    "contrahechuras": "contrahechura",
    "contrahice": "contrahacer",
    "contrahiciera": "contrahacer",
    "contrahicierais": "contrahacer",
    "contrahicieran": "contrahacer",
    "contrahicieras": "contrahacer",
    "contrahiciere": "contrahacer",
    "contrahiciereis": "contrahacer",
    "contrahicieren": "contrahacer",
    "contrahicieres": "contrahacer",
    "contrahicieron": "contrahacer",
    "contrahiciese": "contrahacer",
    "contrahicieseis": "contrahacer",
    "contrahiciesen": "contrahacer",
    "contrahicieses": "contrahacer",
    "contrahicimos": "contrahacer",
    "contrahiciste": "contrahacer",
    "contrahicisteis": "contrahacer",
    "contrahiciéramos": "contrahacer",
    "contrahiciéremos": "contrahacer",
    "contrahiciésemos": "contrahacer",
    "contrahierbas": "contrahierba",
    "contrahizo": "contrahacer",
    "contrahojas": "contrahoja",
    "contrahuellas": "contrahuella",
    "contraiga": "contraer",
    "contraigamos": "contraer",
    "contraigan": "contraer",
    "contraigas": "contraer",
    "contraigo": "contraer",
    "contraigáis": "contraer",
    "contraindica": "contraindicar",
    "contraindicaba": "contraindicar",
    "contraindicabais": "contraindicar",
    "contraindicaban": "contraindicar",
    "contraindicabas": "contraindicar",
    "contraindicaciones": "contraindicación",
    "contraindicad": "contraindicar",
    "contraindicada": "contraindicar",
    "contraindicadas": "contraindicar",
    "contraindicado": "contraindicar",
    "contraindicados": "contraindicar",
    "contraindicamos": "contraindicar",
    "contraindican": "contraindicar",
    "contraindicando": "contraindicar",
    "contraindicara": "contraindicar",
    "contraindicarais": "contraindicar",
    "contraindicaran": "contraindicar",
    "contraindicaras": "contraindicar",
    "contraindicare": "contraindicar",
    "contraindicareis": "contraindicar",
    "contraindicaremos": "contraindicar",
    "contraindicaren": "contraindicar",
    "contraindicares": "contraindicar",
    "contraindicaron": "contraindicar",
    "contraindicará": "contraindicar",
    "contraindicarán": "contraindicar",
    "contraindicarás": "contraindicar",
    "contraindicaré": "contraindicar",
    "contraindicaréis": "contraindicar",
    "contraindicaría": "contraindicar",
    "contraindicaríais": "contraindicar",
    "contraindicaríamos": "contraindicar",
    "contraindicarían": "contraindicar",
    "contraindicarías": "contraindicar",
    "contraindicas": "contraindicar",
    "contraindicase": "contraindicar",
    "contraindicaseis": "contraindicar",
    "contraindicasen": "contraindicar",
    "contraindicases": "contraindicar",
    "contraindicaste": "contraindicar",
    "contraindicasteis": "contraindicar",
    "contraindico": "contraindicar",
    "contraindicábamos": "contraindicar",
    "contraindicáis": "contraindicar",
    "contraindicáramos": "contraindicar",
    "contraindicáremos": "contraindicar",
    "contraindicásemos": "contraindicar",
    "contraindicó": "contraindicar",
    "contraindique": "contraindicar",
    "contraindiquemos": "contraindicar",
    "contraindiquen": "contraindicar",
    "contraindiques": "contraindicar",
    "contraindiqué": "contraindicar",
    "contraindiquéis": "contraindicar",
    "contrainterrogatorios": "contrainterrogatorio",
    "contraje": "contraer",
    "contrajera": "contraer",
    "contrajerais": "contraer",
    "contrajeran": "contraer",
    "contrajeras": "contraer",
    "contrajere": "contraer",
    "contrajereis": "contraer",
    "contrajeren": "contraer",
    "contrajeres": "contraer",
    "contrajeron": "contraer",
    "contrajese": "contraer",
    "contrajeseis": "contraer",
    "contrajesen": "contraer",
    "contrajeses": "contraer",
    "contrajimos": "contraer",
    "contrajiste": "contraer",
    "contrajisteis": "contraer",
    "contrajo": "contraer",
    "contrajéramos": "contraer",
    "contrajéremos": "contraer",
    "contrajésemos": "contraer",
    "contralaterales": "contralateral",
    "contralizos": "contralizo",
    "contralmirantes": "contralmirante",
    "contralores": "contralor",
    "contraltos": "contralto",
    "contraluces": "contraluz",
    "contramaestres": "contramaestre",
    "contramalla": "contramallar",
    "contramallaba": "contramallar",
    "contramallabais": "contramallar",
    "contramallaban": "contramallar",
    "contramallabas": "contramallar",
    "contramallad": "contramallar",
    "contramallada": "contramallar",
    "contramalladas": "contramallar",
    "contramallado": "contramallar",
    "contramallados": "contramallar",
    "contramalladuras": "contramalladura",
    "contramallamos": "contramallar",
    "contramallan": "contramallar",
    "contramallando": "contramallar",
    "contramallara": "contramallar",
    "contramallarais": "contramallar",
    "contramallaran": "contramallar",
    "contramallaras": "contramallar",
    "contramallare": "contramallar",
    "contramallareis": "contramallar",
    "contramallaremos": "contramallar",
    "contramallaren": "contramallar",
    "contramallares": "contramallar",
    "contramallaron": "contramallar",
    "contramallará": "contramallar",
    "contramallarán": "contramallar",
    "contramallarás": "contramallar",
    "contramallaré": "contramallar",
    "contramallaréis": "contramallar",
    "contramallaría": "contramallar",
    "contramallaríais": "contramallar",
    "contramallaríamos": "contramallar",
    "contramallarían": "contramallar",
    "contramallarías": "contramallar",
    "contramallas": "contramallar",
    "contramallase": "contramallar",
    "contramallaseis": "contramallar",
    "contramallasen": "contramallar",
    "contramallases": "contramallar",
    "contramallaste": "contramallar",
    "contramallasteis": "contramallar",
    "contramalle": "contramallar",
    "contramallemos": "contramallar",
    "contramallen": "contramallar",
    "contramalles": "contramallar",
    "contramallo": "contramallar",
    "contramallábamos": "contramallar",
    "contramalláis": "contramallar",
    "contramalláramos": "contramallar",
    "contramalláremos": "contramallar",
    "contramallásemos": "contramallar",
    "contramallé": "contramallar",
    "contramalléis": "contramallar",
    "contramalló": "contramallar",
    "contramanda": "contramandar",
    "contramandaba": "contramandar",
    "contramandabais": "contramandar",
    "contramandaban": "contramandar",
    "contramandabas": "contramandar",
    "contramandad": "contramandar",
    "contramandada": "contramandar",
    "contramandadas": "contramandar",
    "contramandado": "contramandar",
    "contramandados": "contramandar",
    "contramandamos": "contramandar",
    "contramandan": "contramandar",
    "contramandando": "contramandar",
    "contramandara": "contramandar",
    "contramandarais": "contramandar",
    "contramandaran": "contramandar",
    "contramandaras": "contramandar",
    "contramandare": "contramandar",
    "contramandareis": "contramandar",
    "contramandaremos": "contramandar",
    "contramandaren": "contramandar",
    "contramandares": "contramandar",
    "contramandaron": "contramandar",
    "contramandará": "contramandar",
    "contramandarán": "contramandar",
    "contramandarás": "contramandar",
    "contramandaré": "contramandar",
    "contramandaréis": "contramandar",
    "contramandaría": "contramandar",
    "contramandaríais": "contramandar",
    "contramandaríamos": "contramandar",
    "contramandarían": "contramandar",
    "contramandarías": "contramandar",
    "contramandas": "contramandar",
    "contramandase": "contramandar",
    "contramandaseis": "contramandar",
    "contramandasen": "contramandar",
    "contramandases": "contramandar",
    "contramandaste": "contramandar",
    "contramandasteis": "contramandar",
    "contramande": "contramandar",
    "contramandemos": "contramandar",
    "contramanden": "contramandar",
    "contramandes": "contramandar",
    "contramando": "contramandar",
    "contramandábamos": "contramandar",
    "contramandáis": "contramandar",
    "contramandáramos": "contramandar",
    "contramandáremos": "contramandar",
    "contramandásemos": "contramandar",
    "contramandé": "contramandar",
    "contramandéis": "contramandar",
    "contramandó": "contramandar",
    "contramarca": "contramarcar",
    "contramarcaba": "contramarcar",
    "contramarcabais": "contramarcar",
    "contramarcaban": "contramarcar",
    "contramarcabas": "contramarcar",
    "contramarcad": "contramarcar",
    "contramarcada": "contramarcar",
    "contramarcadas": "contramarcar",
    "contramarcado": "contramarcar",
    "contramarcados": "contramarcar",
    "contramarcamos": "contramarcar",
    "contramarcan": "contramarcar",
    "contramarcando": "contramarcar",
    "contramarcara": "contramarcar",
    "contramarcarais": "contramarcar",
    "contramarcaran": "contramarcar",
    "contramarcaras": "contramarcar",
    "contramarcare": "contramarcar",
    "contramarcareis": "contramarcar",
    "contramarcaremos": "contramarcar",
    "contramarcaren": "contramarcar",
    "contramarcares": "contramarcar",
    "contramarcaron": "contramarcar",
    "contramarcará": "contramarcar",
    "contramarcarán": "contramarcar",
    "contramarcarás": "contramarcar",
    "contramarcaré": "contramarcar",
    "contramarcaréis": "contramarcar",
    "contramarcaría": "contramarcar",
    "contramarcaríais": "contramarcar",
    "contramarcaríamos": "contramarcar",
    "contramarcarían": "contramarcar",
    "contramarcarías": "contramarcar",
    "contramarcas": "contramarcar",
    "contramarcase": "contramarcar",
    "contramarcaseis": "contramarcar",
    "contramarcasen": "contramarcar",
    "contramarcases": "contramarcar",
    "contramarcaste": "contramarcar",
    "contramarcasteis": "contramarcar",
    "contramarcha": "contramarchar",
    "contramarchaba": "contramarchar",
    "contramarchabais": "contramarchar",
    "contramarchaban": "contramarchar",
    "contramarchabas": "contramarchar",
    "contramarchad": "contramarchar",
    "contramarchada": "contramarchar",
    "contramarchadas": "contramarchar",
    "contramarchado": "contramarchar",
    "contramarchados": "contramarchar",
    "contramarchamos": "contramarchar",
    "contramarchan": "contramarchar",
    "contramarchando": "contramarchar",
    "contramarchara": "contramarchar",
    "contramarcharais": "contramarchar",
    "contramarcharan": "contramarchar",
    "contramarcharas": "contramarchar",
    "contramarchare": "contramarchar",
    "contramarchareis": "contramarchar",
    "contramarcharemos": "contramarchar",
    "contramarcharen": "contramarchar",
    "contramarchares": "contramarchar",
    "contramarcharon": "contramarchar",
    "contramarchará": "contramarchar",
    "contramarcharán": "contramarchar",
    "contramarcharás": "contramarchar",
    "contramarcharé": "contramarchar",
    "contramarcharéis": "contramarchar",
    "contramarcharía": "contramarchar",
    "contramarcharíais": "contramarchar",
    "contramarcharíamos": "contramarchar",
    "contramarcharían": "contramarchar",
    "contramarcharías": "contramarchar",
    "contramarchas": "contramarchar",
    "contramarchase": "contramarchar",
    "contramarchaseis": "contramarchar",
    "contramarchasen": "contramarchar",
    "contramarchases": "contramarchar",
    "contramarchaste": "contramarchar",
    "contramarchasteis": "contramarchar",
    "contramarche": "contramarchar",
    "contramarchemos": "contramarchar",
    "contramarchen": "contramarchar",
    "contramarches": "contramarchar",
    "contramarcho": "contramarchar",
    "contramarchábamos": "contramarchar",
    "contramarcháis": "contramarchar",
    "contramarcháramos": "contramarchar",
    "contramarcháremos": "contramarchar",
    "contramarchásemos": "contramarchar",
    "contramarché": "contramarchar",
    "contramarchéis": "contramarchar",
    "contramarchó": "contramarchar",
    "contramarco": "contramarcar",
    "contramarcos": "contramarco",
    "contramarcábamos": "contramarcar",
    "contramarcáis": "contramarcar",
    "contramarcáramos": "contramarcar",
    "contramarcáremos": "contramarcar",
    "contramarcásemos": "contramarcar",
    "contramarcó": "contramarcar",
    "contramareas": "contramarea",
    "contramarque": "contramarcar",
    "contramarquemos": "contramarcar",
    "contramarquen": "contramarcar",
    "contramarques": "contramarcar",
    "contramarqué": "contramarcar",
    "contramarquéis": "contramarcar",
    "contramateriales": "contramaterial",
    "contramedidas": "contramedida",
    "contramesanas": "contramesana",
    "contramina": "contraminar",
    "contraminaba": "contraminar",
    "contraminabais": "contraminar",
    "contraminaban": "contraminar",
    "contraminabas": "contraminar",
    "contraminad": "contraminar",
    "contraminada": "contraminar",
    "contraminadas": "contraminar",
    "contraminado": "contraminar",
    "contraminados": "contraminar",
    "contraminamos": "contraminar",
    "contraminan": "contraminar",
    "contraminando": "contraminar",
    "contraminara": "contraminar",
    "contraminarais": "contraminar",
    "contraminaran": "contraminar",
    "contraminaras": "contraminar",
    "contraminare": "contraminar",
    "contraminareis": "contraminar",
    "contraminaremos": "contraminar",
    "contraminaren": "contraminar",
    "contraminares": "contraminar",
    "contraminaron": "contraminar",
    "contraminará": "contraminar",
    "contraminarán": "contraminar",
    "contraminarás": "contraminar",
    "contraminaré": "contraminar",
    "contraminaréis": "contraminar",
    "contraminaría": "contraminar",
    "contraminaríais": "contraminar",
    "contraminaríamos": "contraminar",
    "contraminarían": "contraminar",
    "contraminarías": "contraminar",
    "contraminas": "contraminar",
    "contraminase": "contraminar",
    "contraminaseis": "contraminar",
    "contraminasen": "contraminar",
    "contraminases": "contraminar",
    "contraminaste": "contraminar",
    "contraminasteis": "contraminar",
    "contramine": "contraminar",
    "contraminemos": "contraminar",
    "contraminen": "contraminar",
    "contramines": "contraminar",
    "contramino": "contraminar",
    "contraminábamos": "contraminar",
    "contramináis": "contraminar",
    "contramináramos": "contraminar",
    "contramináremos": "contraminar",
    "contraminásemos": "contraminar",
    "contraminé": "contraminar",
    "contraminéis": "contraminar",
    "contraminó": "contraminar",
    "contramuelles": "contramuelle",
    "contramurallas": "contramuralla",
    "contramuros": "contramuro",
    "contranaturales": "contranatural",
    "contraofensiva": "contraofensivo",
    "contraofensivas": "contraofensivo",
    "contraofensivos": "contraofensivo",
    "contraofertas": "contraoferta",
    "contraordena": "contraordenar",
    "contraordenaba": "contraordenar",
    "contraordenabais": "contraordenar",
    "contraordenaban": "contraordenar",
    "contraordenabas": "contraordenar",
    "contraordenad": "contraordenar",
    "contraordenada": "contraordenar",
    "contraordenadas": "contraordenar",
    "contraordenado": "contraordenar",
    "contraordenados": "contraordenar",
    "contraordenamos": "contraordenar",
    "contraordenan": "contraordenar",
    "contraordenando": "contraordenar",
    "contraordenara": "contraordenar",
    "contraordenarais": "contraordenar",
    "contraordenaran": "contraordenar",
    "contraordenaras": "contraordenar",
    "contraordenare": "contraordenar",
    "contraordenareis": "contraordenar",
    "contraordenaremos": "contraordenar",
    "contraordenaren": "contraordenar",
    "contraordenares": "contraordenar",
    "contraordenaron": "contraordenar",
    "contraordenará": "contraordenar",
    "contraordenarán": "contraordenar",
    "contraordenarás": "contraordenar",
    "contraordenaré": "contraordenar",
    "contraordenaréis": "contraordenar",
    "contraordenaría": "contraordenar",
    "contraordenaríais": "contraordenar",
    "contraordenaríamos": "contraordenar",
    "contraordenarían": "contraordenar",
    "contraordenarías": "contraordenar",
    "contraordenas": "contraordenar",
    "contraordenase": "contraordenar",
    "contraordenaseis": "contraordenar",
    "contraordenasen": "contraordenar",
    "contraordenases": "contraordenar",
    "contraordenaste": "contraordenar",
    "contraordenasteis": "contraordenar",
    "contraordene": "contraordenar",
    "contraordenemos": "contraordenar",
    "contraordenen": "contraordenar",
    "contraordenes": "contraordenar",
    "contraordeno": "contraordenar",
    "contraordenábamos": "contraordenar",
    "contraordenáis": "contraordenar",
    "contraordenáramos": "contraordenar",
    "contraordenáremos": "contraordenar",
    "contraordenásemos": "contraordenar",
    "contraordené": "contraordenar",
    "contraordenéis": "contraordenar",
    "contraordenó": "contraordenar",
    "contrapalada": "contrapalado",
    "contrapaladas": "contrapalado",
    "contrapalados": "contrapalado",
    "contrapalos": "contrapalo",
    "contrapares": "contrapar",
    "contrapartes": "contraparte",
    "contrapartidas": "contrapartida",
    "contrapasa": "contrapasar",
    "contrapasaba": "contrapasar",
    "contrapasabais": "contrapasar",
    "contrapasaban": "contrapasar",
    "contrapasabas": "contrapasar",
    "contrapasad": "contrapasar",
    "contrapasada": "contrapasar",
    "contrapasadas": "contrapasar",
    "contrapasado": "contrapasar",
    "contrapasados": "contrapasar",
    "contrapasamientos": "contrapasamiento",
    "contrapasamos": "contrapasar",
    "contrapasan": "contrapasar",
    "contrapasando": "contrapasar",
    "contrapasara": "contrapasar",
    "contrapasarais": "contrapasar",
    "contrapasaran": "contrapasar",
    "contrapasaras": "contrapasar",
    "contrapasare": "contrapasar",
    "contrapasareis": "contrapasar",
    "contrapasaremos": "contrapasar",
    "contrapasaren": "contrapasar",
    "contrapasares": "contrapasar",
    "contrapasaron": "contrapasar",
    "contrapasará": "contrapasar",
    "contrapasarán": "contrapasar",
    "contrapasarás": "contrapasar",
    "contrapasaré": "contrapasar",
    "contrapasaréis": "contrapasar",
    "contrapasaría": "contrapasar",
    "contrapasaríais": "contrapasar",
    "contrapasaríamos": "contrapasar",
    "contrapasarían": "contrapasar",
    "contrapasarías": "contrapasar",
    "contrapasas": "contrapasar",
    "contrapasase": "contrapasar",
    "contrapasaseis": "contrapasar",
    "contrapasasen": "contrapasar",
    "contrapasases": "contrapasar",
    "contrapasaste": "contrapasar",
    "contrapasasteis": "contrapasar",
    "contrapase": "contrapasar",
    "contrapasemos": "contrapasar",
    "contrapasen": "contrapasar",
    "contrapases": "contrapasar",
    "contrapaso": "contrapasar",
    "contrapasos": "contrapaso",
    "contrapasábamos": "contrapasar",
    "contrapasáis": "contrapasar",
    "contrapasáramos": "contrapasar",
    "contrapasáremos": "contrapasar",
    "contrapasásemos": "contrapasar",
    "contrapasé": "contrapasar",
    "contrapaséis": "contrapasar",
    "contrapasó": "contrapasar",
    "contrapea": "contrapear",
    "contrapeaba": "contrapear",
    "contrapeabais": "contrapear",
    "contrapeaban": "contrapear",
    "contrapeabas": "contrapear",
    "contrapead": "contrapear",
    "contrapeada": "contrapear",
    "contrapeadas": "contrapear",
    "contrapeado": "contrapear",
    "contrapeados": "contrapear",
    "contrapeamos": "contrapear",
    "contrapean": "contrapear",
    "contrapeando": "contrapear",
    "contrapeara": "contrapear",
    "contrapearais": "contrapear",
    "contrapearan": "contrapear",
    "contrapearas": "contrapear",
    "contrapeare": "contrapear",
    "contrapeareis": "contrapear",
    "contrapearemos": "contrapear",
    "contrapearen": "contrapear",
    "contrapeares": "contrapear",
    "contrapearon": "contrapear",
    "contrapeará": "contrapear",
    "contrapearán": "contrapear",
    "contrapearás": "contrapear",
    "contrapearé": "contrapear",
    "contrapearéis": "contrapear",
    "contrapearía": "contrapear",
    "contrapearíais": "contrapear",
    "contrapearíamos": "contrapear",
    "contrapearían": "contrapear",
    "contrapearías": "contrapear",
    "contrapeas": "contrapear",
    "contrapease": "contrapear",
    "contrapeaseis": "contrapear",
    "contrapeasen": "contrapear",
    "contrapeases": "contrapear",
    "contrapeaste": "contrapear",
    "contrapeasteis": "contrapear",
    "contrapedales": "contrapedal",
    "contrapee": "contrapear",
    "contrapeemos": "contrapear",
    "contrapeen": "contrapear",
    "contrapees": "contrapear",
    "contrapeo": "contrapear",
    "contrapesa": "contrapesar",
    "contrapesaba": "contrapesar",
    "contrapesabais": "contrapesar",
    "contrapesaban": "contrapesar",
    "contrapesabas": "contrapesar",
    "contrapesad": "contrapesar",
    "contrapesada": "contrapesar",
    "contrapesadas": "contrapesar",
    "contrapesado": "contrapesar",
    "contrapesados": "contrapesar",
    "contrapesamos": "contrapesar",
    "contrapesan": "contrapesar",
    "contrapesando": "contrapesar",
    "contrapesara": "contrapesar",
    "contrapesarais": "contrapesar",
    "contrapesaran": "contrapesar",
    "contrapesaras": "contrapesar",
    "contrapesare": "contrapesar",
    "contrapesareis": "contrapesar",
    "contrapesaremos": "contrapesar",
    "contrapesaren": "contrapesar",
    "contrapesares": "contrapesar",
    "contrapesaron": "contrapesar",
    "contrapesará": "contrapesar",
    "contrapesarán": "contrapesar",
    "contrapesarás": "contrapesar",
    "contrapesaré": "contrapesar",
    "contrapesaréis": "contrapesar",
    "contrapesaría": "contrapesar",
    "contrapesaríais": "contrapesar",
    "contrapesaríamos": "contrapesar",
    "contrapesarían": "contrapesar",
    "contrapesarías": "contrapesar",
    "contrapesas": "contrapesar",
    "contrapesase": "contrapesar",
    "contrapesaseis": "contrapesar",
    "contrapesasen": "contrapesar",
    "contrapesases": "contrapesar",
    "contrapesaste": "contrapesar",
    "contrapesasteis": "contrapesar",
    "contrapese": "contrapesar",
    "contrapesemos": "contrapesar",
    "contrapesen": "contrapesar",
    "contrapeses": "contrapesar",
    "contrapeso": "contrapesar",
    "contrapesos": "contrapeso",
    "contrapesábamos": "contrapesar",
    "contrapesáis": "contrapesar",
    "contrapesáramos": "contrapesar",
    "contrapesáremos": "contrapesar",
    "contrapesásemos": "contrapesar",
    "contrapesé": "contrapesar",
    "contrapeséis": "contrapesar",
    "contrapesó": "contrapesar",
    "contrapeábamos": "contrapear",
    "contrapeáis": "contrapear",
    "contrapeáramos": "contrapear",
    "contrapeáremos": "contrapear",
    "contrapeásemos": "contrapear",
    "contrapeé": "contrapear",
    "contrapeéis": "contrapear",
    "contrapeó": "contrapear",
    "contrapicada": "contrapicado",
    "contrapicadas": "contrapicado",
    "contrapicados": "contrapicado",
    "contrapilastras": "contrapilastra",
    "contraplacada": "contraplacado",
    "contraplacadas": "contraplacado",
    "contraplacados": "contraplacado",
    "contraplacas": "contraplaca",
    "contraplanos": "contraplano",
    "contrapondremos": "contraponer",
    "contrapondrá": "contraponer",
    "contrapondrán": "contraponer",
    "contrapondrás": "contraponer",
    "contrapondré": "contraponer",
    "contrapondréis": "contraponer",
    "contrapondría": "contraponer",
    "contrapondríais": "contraponer",
    "contrapondríamos": "contraponer",
    "contrapondrían": "contraponer",
    "contrapondrías": "contraponer",
    "contrapone": "contraponer",
    "contraponed": "contraponer",
    "contraponemos": "contraponer",
    "contraponen": "contraponer",
    "contrapones": "contraponer",
    "contraponga": "contraponer",
    "contrapongamos": "contraponer",
    "contrapongan": "contraponer",
    "contrapongas": "contraponer",
    "contrapongo": "contraponer",
    "contrapongáis": "contraponer",
    "contraponiendo": "contraponer",
    "contraponéis": "contraponer",
    "contraponía": "contraponer",
    "contraponíais": "contraponer",
    "contraponíamos": "contraponer",
    "contraponían": "contraponer",
    "contraponías": "contraponer",
    "contraportadas": "contraportada",
    "contraposiciones": "contraposición",
    "contrapozos": "contrapozo",
    "contrapresas": "contrapresa",
    "contrapresiones": "contrapresión",
    "contraprestaciones": "contraprestación",
    "contraprincipios": "contraprincipio",
    "contraproducentes": "contraproducente",
    "contrapropuestas": "contrapropuesta",
    "contraproyectos": "contraproyecto",
    "contrapruebas": "contraprueba",
    "contrapublicidades": "contrapublicidad",
    "contrapuertas": "contrapuerta",
    "contrapuesta": "contraponer",
    "contrapuestas": "contraponer",
    "contrapuesto": "contraponer",
    "contrapuestos": "contraponer",
    "contrapunce": "contrapunzar",
    "contrapuncemos": "contrapunzar",
    "contrapuncen": "contrapunzar",
    "contrapunces": "contrapunzar",
    "contrapuncé": "contrapunzar",
    "contrapuncéis": "contrapunzar",
    "contrapunta": "contrapuntar",
    "contrapuntaba": "contrapuntar",
    "contrapuntabais": "contrapuntar",
    "contrapuntaban": "contrapuntar",
    "contrapuntabas": "contrapuntar",
    "contrapuntad": "contrapuntar",
    "contrapuntada": "contrapuntar",
    "contrapuntadas": "contrapuntar",
    "contrapuntado": "contrapuntar",
    "contrapuntados": "contrapuntar",
    "contrapuntamos": "contrapuntar",
    "contrapuntan": "contrapuntar",
    "contrapuntando": "contrapuntar",
    "contrapuntara": "contrapuntar",
    "contrapuntarais": "contrapuntar",
    "contrapuntaran": "contrapuntar",
    "contrapuntaras": "contrapuntar",
    "contrapuntare": "contrapuntar",
    "contrapuntareis": "contrapuntar",
    "contrapuntaremos": "contrapuntar",
    "contrapuntaren": "contrapuntar",
    "contrapuntares": "contrapuntar",
    "contrapuntaron": "contrapuntar",
    "contrapuntará": "contrapuntar",
    "contrapuntarán": "contrapuntar",
    "contrapuntarás": "contrapuntar",
    "contrapuntaré": "contrapuntar",
    "contrapuntaréis": "contrapuntar",
    "contrapuntaría": "contrapuntar",
    "contrapuntaríais": "contrapuntar",
    "contrapuntaríamos": "contrapuntar",
    "contrapuntarían": "contrapuntar",
    "contrapuntarías": "contrapuntar",
    "contrapuntas": "contrapuntar",
    "contrapuntase": "contrapuntar",
    "contrapuntaseis": "contrapuntar",
    "contrapuntasen": "contrapuntar",
    "contrapuntases": "contrapuntar",
    "contrapuntaste": "contrapuntar",
    "contrapuntasteis": "contrapuntar",
    "contrapunte": "contrapuntar",
    "contrapuntea": "contrapuntear",
    "contrapunteaba": "contrapuntear",
    "contrapunteabais": "contrapuntear",
    "contrapunteaban": "contrapuntear",
    "contrapunteabas": "contrapuntear",
    "contrapuntead": "contrapuntear",
    "contrapunteada": "contrapuntear",
    "contrapunteadas": "contrapuntear",
    "contrapunteado": "contrapuntear",
    "contrapunteados": "contrapuntear",
    "contrapunteamos": "contrapuntear",
    "contrapuntean": "contrapuntear",
    "contrapunteando": "contrapuntear",
    "contrapunteara": "contrapuntear",
    "contrapuntearais": "contrapuntear",
    "contrapuntearan": "contrapuntear",
    "contrapuntearas": "contrapuntear",
    "contrapunteare": "contrapuntear",
    "contrapunteareis": "contrapuntear",
    "contrapuntearemos": "contrapuntear",
    "contrapuntearen": "contrapuntear",
    "contrapunteares": "contrapuntear",
    "contrapuntearon": "contrapuntear",
    "contrapunteará": "contrapuntear",
    "contrapuntearán": "contrapuntear",
    "contrapuntearás": "contrapuntear",
    "contrapuntearé": "contrapuntear",
    "contrapuntearéis": "contrapuntear",
    "contrapuntearía": "contrapuntear",
    "contrapuntearíais": "contrapuntear",
    "contrapuntearíamos": "contrapuntear",
    "contrapuntearían": "contrapuntear",
    "contrapuntearías": "contrapuntear",
    "contrapunteas": "contrapuntear",
    "contrapuntease": "contrapuntear",
    "contrapunteaseis": "contrapuntear",
    "contrapunteasen": "contrapuntear",
    "contrapunteases": "contrapuntear",
    "contrapunteaste": "contrapuntear",
    "contrapunteasteis": "contrapuntear",
    "contrapuntee": "contrapuntear",
    "contrapunteemos": "contrapuntear",
    "contrapunteen": "contrapuntear",
    "contrapuntees": "contrapuntear",
    "contrapuntemos": "contrapuntar",
    "contrapunten": "contrapuntar",
    "contrapunteo": "contrapuntear",
    "contrapunteos": "contrapunteo",
    "contrapuntes": "contrapuntar",
    "contrapunteábamos": "contrapuntear",
    "contrapunteáis": "contrapuntear",
    "contrapunteáramos": "contrapuntear",
    "contrapunteáremos": "contrapuntear",
    "contrapunteásemos": "contrapuntear",
    "contrapunteé": "contrapuntear",
    "contrapunteéis": "contrapuntear",
    "contrapunteó": "contrapuntear",
    "contrapuntistas": "contrapuntista",
    "contrapunto": "contrapuntar",
    "contrapuntos": "contrapunto",
    "contrapuntábamos": "contrapuntar",
    "contrapuntáis": "contrapuntar",
    "contrapuntáramos": "contrapuntar",
    "contrapuntáremos": "contrapuntar",
    "contrapuntásemos": "contrapuntar",
    "contrapunté": "contrapuntar",
    "contrapuntéis": "contrapuntar",
    "contrapuntó": "contrapuntar",
    "contrapunza": "contrapunzar",
    "contrapunzaba": "contrapunzar",
    "contrapunzabais": "contrapunzar",
    "contrapunzaban": "contrapunzar",
    "contrapunzabas": "contrapunzar",
    "contrapunzad": "contrapunzar",
    "contrapunzada": "contrapunzar",
    "contrapunzadas": "contrapunzar",
    "contrapunzado": "contrapunzar",
    "contrapunzados": "contrapunzar",
    "contrapunzamos": "contrapunzar",
    "contrapunzan": "contrapunzar",
    "contrapunzando": "contrapunzar",
    "contrapunzara": "contrapunzar",
    "contrapunzarais": "contrapunzar",
    "contrapunzaran": "contrapunzar",
    "contrapunzaras": "contrapunzar",
    "contrapunzare": "contrapunzar",
    "contrapunzareis": "contrapunzar",
    "contrapunzaremos": "contrapunzar",
    "contrapunzaren": "contrapunzar",
    "contrapunzares": "contrapunzar",
    "contrapunzaron": "contrapunzar",
    "contrapunzará": "contrapunzar",
    "contrapunzarán": "contrapunzar",
    "contrapunzarás": "contrapunzar",
    "contrapunzaré": "contrapunzar",
    "contrapunzaréis": "contrapunzar",
    "contrapunzaría": "contrapunzar",
    "contrapunzaríais": "contrapunzar",
    "contrapunzaríamos": "contrapunzar",
    "contrapunzarían": "contrapunzar",
    "contrapunzarías": "contrapunzar",
    "contrapunzas": "contrapunzar",
    "contrapunzase": "contrapunzar",
    "contrapunzaseis": "contrapunzar",
    "contrapunzasen": "contrapunzar",
    "contrapunzases": "contrapunzar",
    "contrapunzaste": "contrapunzar",
    "contrapunzasteis": "contrapunzar",
    "contrapunzo": "contrapunzar",
    "contrapunzones": "contrapunzón",
    "contrapunzábamos": "contrapunzar",
    "contrapunzáis": "contrapunzar",
    "contrapunzáramos": "contrapunzar",
    "contrapunzáremos": "contrapunzar",
    "contrapunzásemos": "contrapunzar",
    "contrapunzó": "contrapunzar",
    "contrapuse": "contraponer",
    "contrapusiera": "contraponer",
    "contrapusierais": "contraponer",
    "contrapusieran": "contraponer",
    "contrapusieras": "contraponer",
    "contrapusiere": "contraponer",
    "contrapusiereis": "contraponer",
    "contrapusieren": "contraponer",
    "contrapusieres": "contraponer",
    "contrapusieron": "contraponer",
    "contrapusiese": "contraponer",
    "contrapusieseis": "contraponer",
    "contrapusiesen": "contraponer",
    "contrapusieses": "contraponer",
    "contrapusimos": "contraponer",
    "contrapusiste": "contraponer",
    "contrapusisteis": "contraponer",
    "contrapusiéramos": "contraponer",
    "contrapusiéremos": "contraponer",
    "contrapusiésemos": "contraponer",
    "contrapuso": "contraponer",
    "contrapón": "contraponer",
    "contraquillas": "contraquilla",
    "contraria": "contrario",
    "contrariaba": "contrariar",
    "contrariabais": "contrariar",
    "contrariaban": "contrariar",
    "contrariabas": "contrariar",
    "contrariad": "contrariar",
    "contrariada": "contrariar",
    "contrariadas": "contrariar",
    "contrariado": "contrariar",
    "contrariados": "contrariar",
    "contrariamos": "contrariar",
    "contrariando": "contrariar",
    "contrariara": "contrariar",
    "contrariarais": "contrariar",
    "contrariaran": "contrariar",
    "contrariaras": "contrariar",
    "contrariare": "contrariar",
    "contrariareis": "contrariar",
    "contrariaremos": "contrariar",
    "contrariaren": "contrariar",
    "contrariares": "contrariar",
    "contrariaron": "contrariar",
    "contrariará": "contrariar",
    "contrariarán": "contrariar",
    "contrariarás": "contrariar",
    "contrariaré": "contrariar",
    "contrariaréis": "contrariar",
    "contrariaría": "contrariar",
    "contrariaríais": "contrariar",
    "contrariaríamos": "contrariar",
    "contrariarían": "contrariar",
    "contrariarías": "contrariar",
    "contrarias": "contrario",
    "contrariase": "contrariar",
    "contrariaseis": "contrariar",
    "contrariasen": "contrariar",
    "contrariases": "contrariar",
    "contrariaste": "contrariar",
    "contrariasteis": "contrariar",
    "contrariedades": "contrariedad",
    "contrariemos": "contrariar",
    "contrarios": "contrario",
    "contrariábamos": "contrariar",
    "contrariáis": "contrariar",
    "contrariáramos": "contrariar",
    "contrariáremos": "contrariar",
    "contrariásemos": "contrariar",
    "contrarié": "contrariar",
    "contrariéis": "contrariar",
    "contrarió": "contrariar",
    "contrarrayas": "contrarraya",
    "contrarreacciones": "contrarreacción",
    "contrarregueras": "contrarreguera",
    "contrarrelojes": "contrarreloj",
    "contrarrelojistas": "contrarrelojista",
    "contrarresta": "contrarrestar",
    "contrarrestaba": "contrarrestar",
    "contrarrestabais": "contrarrestar",
    "contrarrestaban": "contrarrestar",
    "contrarrestabas": "contrarrestar",
    "contrarrestad": "contrarrestar",
    "contrarrestada": "contrarrestar",
    "contrarrestadas": "contrarrestar",
    "contrarrestado": "contrarrestar",
    "contrarrestados": "contrarrestar",
    "contrarrestamos": "contrarrestar",
    "contrarrestan": "contrarrestar",
    "contrarrestando": "contrarrestar",
    "contrarrestara": "contrarrestar",
    "contrarrestarais": "contrarrestar",
    "contrarrestaran": "contrarrestar",
    "contrarrestaras": "contrarrestar",
    "contrarrestare": "contrarrestar",
    "contrarrestareis": "contrarrestar",
    "contrarrestaremos": "contrarrestar",
    "contrarrestaren": "contrarrestar",
    "contrarrestares": "contrarrestar",
    "contrarrestaron": "contrarrestar",
    "contrarrestará": "contrarrestar",
    "contrarrestarán": "contrarrestar",
    "contrarrestarás": "contrarrestar",
    "contrarrestaré": "contrarrestar",
    "contrarrestaréis": "contrarrestar",
    "contrarrestaría": "contrarrestar",
    "contrarrestaríais": "contrarrestar",
    "contrarrestaríamos": "contrarrestar",
    "contrarrestarían": "contrarrestar",
    "contrarrestarías": "contrarrestar",
    "contrarrestas": "contrarrestar",
    "contrarrestase": "contrarrestar",
    "contrarrestaseis": "contrarrestar",
    "contrarrestasen": "contrarrestar",
    "contrarrestases": "contrarrestar",
    "contrarrestaste": "contrarrestar",
    "contrarrestasteis": "contrarrestar",
    "contrarreste": "contrarrestar",
    "contrarrestemos": "contrarrestar",
    "contrarresten": "contrarrestar",
    "contrarrestes": "contrarrestar",
    "contrarresto": "contrarrestar",
    "contrarrestos": "contrarresto",
    "contrarrestábamos": "contrarrestar",
    "contrarrestáis": "contrarrestar",
    "contrarrestáramos": "contrarrestar",
    "contrarrestáremos": "contrarrestar",
    "contrarrestásemos": "contrarrestar",
    "contrarresté": "contrarrestar",
    "contrarrestéis": "contrarrestar",
    "contrarrestó": "contrarrestar",
    "contrarrevolucionaria": "contrarrevolucionario",
    "contrarrevolucionarias": "contrarrevolucionario",
    "contrarrevolucionarios": "contrarrevolucionario",
    "contrarrevoluciones": "contrarrevolución",
    "contrarrodas": "contrarroda",
    "contrarrondas": "contrarronda",
    "contrarrotaciones": "contrarrotación",
    "contrarrotativa": "contrarrotativo",
    "contrarrotativas": "contrarrotativo",
    "contrarrotativos": "contrarrotativo",
    "contrarréplicas": "contrarréplica",
    "contraría": "contrariar",
    "contrarían": "contrariar",
    "contrarías": "contrariar",
    "contraríe": "contrariar",
    "contraríen": "contrariar",
    "contraríes": "contrariar",
    "contrarío": "contrariar",
    "contras": "contra",
    "contrasalidas": "contrasalida",
    "contrasalvas": "contrasalva",
    "contraseguros": "contraseguro",
    "contrasella": "contrasellar",
    "contrasellaba": "contrasellar",
    "contrasellabais": "contrasellar",
    "contrasellaban": "contrasellar",
    "contrasellabas": "contrasellar",
    "contrasellad": "contrasellar",
    "contrasellada": "contrasellar",
    "contraselladas": "contrasellar",
    "contrasellado": "contrasellar",
    "contrasellados": "contrasellar",
    "contrasellamos": "contrasellar",
    "contrasellan": "contrasellar",
    "contrasellando": "contrasellar",
    "contrasellara": "contrasellar",
    "contrasellarais": "contrasellar",
    "contrasellaran": "contrasellar",
    "contrasellaras": "contrasellar",
    "contrasellare": "contrasellar",
    "contrasellareis": "contrasellar",
    "contrasellaremos": "contrasellar",
    "contrasellaren": "contrasellar",
    "contrasellares": "contrasellar",
    "contrasellaron": "contrasellar",
    "contrasellará": "contrasellar",
    "contrasellarán": "contrasellar",
    "contrasellarás": "contrasellar",
    "contrasellaré": "contrasellar",
    "contrasellaréis": "contrasellar",
    "contrasellaría": "contrasellar",
    "contrasellaríais": "contrasellar",
    "contrasellaríamos": "contrasellar",
    "contrasellarían": "contrasellar",
    "contrasellarías": "contrasellar",
    "contrasellas": "contrasellar",
    "contrasellase": "contrasellar",
    "contrasellaseis": "contrasellar",
    "contrasellasen": "contrasellar",
    "contrasellases": "contrasellar",
    "contrasellaste": "contrasellar",
    "contrasellasteis": "contrasellar",
    "contraselle": "contrasellar",
    "contrasellemos": "contrasellar",
    "contrasellen": "contrasellar",
    "contraselles": "contrasellar",
    "contrasello": "contrasellar",
    "contrasellos": "contrasello",
    "contrasellábamos": "contrasellar",
    "contraselláis": "contrasellar",
    "contraselláramos": "contrasellar",
    "contraselláremos": "contrasellar",
    "contrasellásemos": "contrasellar",
    "contrasellé": "contrasellar",
    "contraselléis": "contrasellar",
    "contraselló": "contrasellar",
    "contrasentidos": "contrasentido",
    "contraseñas": "contraseña",
    "contrasta": "contrastar",
    "contrastaba": "contrastar",
    "contrastabais": "contrastar",
    "contrastaban": "contrastar",
    "contrastabas": "contrastar",
    "contrastabilidades": "contrastabilidad",
    "contrastables": "contrastable",
    "contrastaciones": "contrastación",
    "contrastad": "contrastar",
    "contrastada": "contrastar",
    "contrastadas": "contrastar",
    "contrastado": "contrastar",
    "contrastados": "contrastar",
    "contrastamos": "contrastar",
    "contrastan": "contrastar",
    "contrastando": "contrastar",
    "contrastara": "contrastar",
    "contrastarais": "contrastar",
    "contrastaran": "contrastar",
    "contrastaras": "contrastar",
    "contrastare": "contrastar",
    "contrastareis": "contrastar",
    "contrastaremos": "contrastar",
    "contrastaren": "contrastar",
    "contrastares": "contrastar",
    "contrastaron": "contrastar",
    "contrastará": "contrastar",
    "contrastarán": "contrastar",
    "contrastarás": "contrastar",
    "contrastaré": "contrastar",
    "contrastaréis": "contrastar",
    "contrastaría": "contrastar",
    "contrastaríais": "contrastar",
    "contrastaríamos": "contrastar",
    "contrastarían": "contrastar",
    "contrastarías": "contrastar",
    "contrastas": "contrastar",
    "contrastase": "contrastar",
    "contrastaseis": "contrastar",
    "contrastasen": "contrastar",
    "contrastases": "contrastar",
    "contrastaste": "contrastar",
    "contrastasteis": "contrastar",
    "contraste": "contrastar",
    "contrastemos": "contrastar",
    "contrasten": "contrastar",
    "contrastes": "contraste",
    "contrasto": "contrastar",
    "contrastábamos": "contrastar",
    "contrastáis": "contrastar",
    "contrastáramos": "contrastar",
    "contrastáremos": "contrastar",
    "contrastásemos": "contrastar",
    "contrasté": "contrastar",
    "contrastéis": "contrastar",
    "contrastó": "contrastar",
    "contrata": "contratar",
    "contrataba": "contratar",
    "contratabais": "contratar",
    "contrataban": "contratar",
    "contratabas": "contratar",
    "contratables": "contratable",
    "contrataciones": "contratación",
    "contratad": "contratar",
    "contratada": "contratar",
    "contratadas": "contratar",
    "contratado": "contratar",
    "contratados": "contratar",
    "contrataladros": "contrataladro",
    "contrataludes": "contratalud",
    "contratamos": "contratar",
    "contratan": "contratar",
    "contratando": "contratar",
    "contratantes": "contratante",
    "contratapas": "contratapa",
    "contratara": "contratar",
    "contratarais": "contratar",
    "contrataran": "contratar",
    "contrataras": "contratar",
    "contratare": "contratar",
    "contratareis": "contratar",
    "contrataremos": "contratar",
    "contrataren": "contratar",
    "contratares": "contratar",
    "contrataron": "contratar",
    "contratará": "contratar",
    "contratarán": "contratar",
    "contratarás": "contratar",
    "contrataré": "contratar",
    "contrataréis": "contratar",
    "contrataría": "contratar",
    "contrataríais": "contratar",
    "contrataríamos": "contratar",
    "contratarían": "contratar",
    "contratarías": "contratar",
    "contratas": "contratar",
    "contratase": "contratar",
    "contrataseis": "contratar",
    "contratasen": "contratar",
    "contratases": "contratar",
    "contrataste": "contratar",
    "contratasteis": "contratar",
    "contrate": "contratar",
    "contratemos": "contratar",
    "contraten": "contratar",
    "contratensiones": "contratensión",
    "contrates": "contratar",
    "contratiempos": "contratiempo",
    "contratipos": "contratipo",
    "contratistas": "contratista",
    "contrato": "contratar",
    "contratorpederos": "contratorpedero",
    "contratos": "contrato",
    "contratransferencias": "contratransferencia",
    "contratretas": "contratreta",
    "contratrincheras": "contratrinchera",
    "contratuercas": "contratuerca",
    "contratábamos": "contratar",
    "contratáis": "contratar",
    "contratáramos": "contratar",
    "contratáremos": "contratar",
    "contratásemos": "contratar",
    "contraté": "contratar",
    "contratéis": "contratar",
    "contrató": "contratar",
    "contravala": "contravalar",
    "contravalaba": "contravalar",
    "contravalabais": "contravalar",
    "contravalaban": "contravalar",
    "contravalabas": "contravalar",
    "contravalaciones": "contravalación",
    "contravalad": "contravalar",
    "contravalada": "contravalar",
    "contravaladas": "contravalar",
    "contravalado": "contravalar",
    "contravalados": "contravalar",
    "contravalamos": "contravalar",
    "contravalan": "contravalar",
    "contravalando": "contravalar",
    "contravalara": "contravalar",
    "contravalarais": "contravalar",
    "contravalaran": "contravalar",
    "contravalaras": "contravalar",
    "contravalare": "contravalar",
    "contravalareis": "contravalar",
    "contravalaremos": "contravalar",
    "contravalaren": "contravalar",
    "contravalares": "contravalar",
    "contravalaron": "contravalar",
    "contravalará": "contravalar",
    "contravalarán": "contravalar",
    "contravalarás": "contravalar",
    "contravalaré": "contravalar",
    "contravalaréis": "contravalar",
    "contravalaría": "contravalar",
    "contravalaríais": "contravalar",
    "contravalaríamos": "contravalar",
    "contravalarían": "contravalar",
    "contravalarías": "contravalar",
    "contravalas": "contravalar",
    "contravalase": "contravalar",
    "contravalaseis": "contravalar",
    "contravalasen": "contravalar",
    "contravalases": "contravalar",
    "contravalaste": "contravalar",
    "contravalasteis": "contravalar",
    "contravale": "contravalar",
    "contravalemos": "contravalar",
    "contravalen": "contravalar",
    "contravales": "contravalar",
    "contravalo": "contravalar",
    "contravalores": "contravalor",
    "contravalábamos": "contravalar",
    "contravaláis": "contravalar",
    "contravaláramos": "contravalar",
    "contravaláremos": "contravalar",
    "contravalásemos": "contravalar",
    "contravalé": "contravalar",
    "contravaléis": "contravalar",
    "contravaló": "contravalar",
    "contravapores": "contravapor",
    "contravariantes": "contravariante",
    "contravarianzas": "contravarianza",
    "contraven": "contravenir",
    "contravenciones": "contravención",
    "contravendremos": "contravenir",
    "contravendrá": "contravenir",
    "contravendrán": "contravenir",
    "contravendrás": "contravenir",
    "contravendré": "contravenir",
    "contravendréis": "contravenir",
    "contravendría": "contravenir",
    "contravendríais": "contravenir",
    "contravendríamos": "contravenir",
    "contravendrían": "contravenir",
    "contravendrías": "contravenir",
    "contravenenos": "contraveneno",
    "contravenga": "contravenir",
    "contravengamos": "contravenir",
    "contravengan": "contravenir",
    "contravengas": "contravenir",
    "contravengo": "contravenir",
    "contravengáis": "contravenir",
    "contravenid": "contravenir",
    "contravenida": "contravenir",
    "contravenidas": "contravenir",
    "contravenido": "contravenir",
    "contravenidos": "contravenir",
    "contravenimos": "contravenir",
    "contraventanas": "contraventana",
    "contraventora": "contraventor",
    "contraventoras": "contraventor",
    "contraventores": "contraventor",
    "contraventuras": "contraventura",
    "contravenía": "contravenir",
    "contraveníais": "contravenir",
    "contraveníamos": "contravenir",
    "contravenían": "contravenir",
    "contravenías": "contravenir",
    "contravenís": "contravenir",
    "contravidrieras": "contravidriera",
    "contraviene": "contravenir",
    "contravienen": "contravenir",
    "contravienes": "contravenir",
    "contravientos": "contraviento",
    "contravine": "contravenir",
    "contraviniendo": "contravenir",
    "contraviniera": "contravenir",
    "contravinierais": "contravenir",
    "contravinieran": "contravenir",
    "contravinieras": "contravenir",
    "contraviniere": "contravenir",
    "contraviniereis": "contravenir",
    "contravinieren": "contravenir",
    "contravinieres": "contravenir",
    "contravinieron": "contravenir",
    "contraviniese": "contravenir",
    "contravinieseis": "contravenir",
    "contraviniesen": "contravenir",
    "contravinieses": "contravenir",
    "contravinimos": "contravenir",
    "contraviniste": "contravenir",
    "contravinisteis": "contravenir",
    "contraviniéramos": "contravenir",
    "contraviniéremos": "contravenir",
    "contraviniésemos": "contravenir",
    "contravino": "contravenir",
    "contravolutas": "contravoluta",
    "contrayendo": "contraer",
    "contrayentes": "contrayente",
    "contraángulos": "contraángulo",
    "contraéis": "contraer",
    "contraía": "contraer",
    "contraíais": "contraer",
    "contraíamos": "contraer",
    "contraían": "contraer",
    "contraías": "contraer",
    "contraída": "contraer",
    "contraídas": "contraer",
    "contraído": "contraer",
    "contraídos": "contraído",
    "contraórdenes": "contraorden",
    "contribuciones": "contribución",
    "contribuid": "contribuir",
    "contribuida": "contribuir",
    "contribuidas": "contribuir",
    "contribuido": "contribuir",
    "contribuidora": "contribuidor",
    "contribuidoras": "contribuidor",
    "contribuidores": "contribuidor",
    "contribuidos": "contribuir",
    "contribuimos": "contribuir",
    "contribuiremos": "contribuir",
    "contribuirá": "contribuir",
    "contribuirán": "contribuir",
    "contribuirás": "contribuir",
    "contribuiré": "contribuir",
    "contribuiréis": "contribuir",
    "contribuiría": "contribuir",
    "contribuiríais": "contribuir",
    "contribuiríamos": "contribuir",
    "contribuirían": "contribuir",
    "contribuirías": "contribuir",
    "contribuiste": "contribuir",
    "contribuisteis": "contribuir",
    "contributaria": "contributario",
    "contributarias": "contributario",
    "contributarios": "contributario",
    "contributiva": "contributivo",
    "contributivas": "contributivo",
    "contributivos": "contributivo",
    "contribuya": "contribuir",
    "contribuyamos": "contribuir",
    "contribuyan": "contribuir",
    "contribuyas": "contribuir",
    "contribuye": "contribuir",
    "contribuyen": "contribuir",
    "contribuyendo": "contribuir",
    "contribuyentes": "contribuyente",
    "contribuyera": "contribuir",
    "contribuyerais": "contribuir",
    "contribuyeran": "contribuir",
    "contribuyeras": "contribuir",
    "contribuyere": "contribuir",
    "contribuyereis": "contribuir",
    "contribuyeren": "contribuir",
    "contribuyeres": "contribuir",
    "contribuyeron": "contribuir",
    "contribuyes": "contribuir",
    "contribuyese": "contribuir",
    "contribuyeseis": "contribuir",
    "contribuyesen": "contribuir",
    "contribuyeses": "contribuir",
    "contribuyo": "contribuir",
    "contribuyáis": "contribuir",
    "contribuyéramos": "contribuir",
    "contribuyéremos": "contribuir",
    "contribuyésemos": "contribuir",
    "contribuyó": "contribuir",
    "contribuí": "contribuir",
    "contribuía": "contribuir",
    "contribuíais": "contribuir",
    "contribuíamos": "contribuir",
    "contribuían": "contribuir",
    "contribuías": "contribuir",
    "contribuís": "contribuir",
    "contricciones": "contricción",
    "contriciones": "contrición",
    "contrincantes": "contrincante",
    "contrista": "contristar",
    "contristaba": "contristar",
    "contristabais": "contristar",
    "contristaban": "contristar",
    "contristabas": "contristar",
    "contristad": "contristar",
    "contristada": "contristar",
    "contristadas": "contristar",
    "contristado": "contristar",
    "contristados": "contristar",
    "contristamos": "contristar",
    "contristan": "contristar",
    "contristando": "contristar",
    "contristara": "contristar",
    "contristarais": "contristar",
    "contristaran": "contristar",
    "contristaras": "contristar",
    "contristare": "contristar",
    "contristareis": "contristar",
    "contristaremos": "contristar",
    "contristaren": "contristar",
    "contristares": "contristar",
    "contristaron": "contristar",
    "contristará": "contristar",
    "contristarán": "contristar",
    "contristarás": "contristar",
    "contristaré": "contristar",
    "contristaréis": "contristar",
    "contristaría": "contristar",
    "contristaríais": "contristar",
    "contristaríamos": "contristar",
    "contristarían": "contristar",
    "contristarías": "contristar",
    "contristas": "contristar",
    "contristase": "contristar",
    "contristaseis": "contristar",
    "contristasen": "contristar",
    "contristases": "contristar",
    "contristaste": "contristar",
    "contristasteis": "contristar",
    "contriste": "contristar",
    "contristemos": "contristar",
    "contristen": "contristar",
    "contristes": "contristar",
    "contristo": "contristar",
    "contristábamos": "contristar",
    "contristáis": "contristar",
    "contristáramos": "contristar",
    "contristáremos": "contristar",
    "contristásemos": "contristar",
    "contristé": "contristar",
    "contristéis": "contristar",
    "contristó": "contristar",
    "contrita": "contrito",
    "contritas": "contrito",
    "contritos": "contrito",
    "controla": "controlar",
    "controlaba": "controlar",
    "controlabais": "controlar",
    "controlaban": "controlar",
    "controlabas": "controlar",
    "controlabilidades": "controlabilidad",
    "controlables": "controlable",
    "controlad": "controlar",
    "controlada": "controlar",
    "controladas": "controlar",
    "controlado": "controlar",
    "controladora": "controlador",
    "controladoras": "controlador",
    "controladores": "controlador",
    "controlados": "controlar",
    "controlamos": "controlar",
    "controlan": "controlar",
    "controlando": "controlar",
    "controlara": "controlar",
    "controlarais": "controlar",
    "controlaran": "controlar",
    "controlaras": "controlar",
    "controlare": "controlar",
    "controlareis": "controlar",
    "controlaremos": "controlar",
    "controlaren": "controlar",
    "controlares": "controlar",
    "controlaron": "controlar",
    "controlará": "controlar",
    "controlarán": "controlar",
    "controlarás": "controlar",
    "controlaré": "controlar",
    "controlaréis": "controlar",
    "controlaría": "controlar",
    "controlaríais": "controlar",
    "controlaríamos": "controlar",
    "controlarían": "controlar",
    "controlarías": "controlar",
    "controlas": "controlar",
    "controlase": "controlar",
    "controlaseis": "controlar",
    "controlasen": "controlar",
    "controlases": "controlar",
    "controlaste": "controlar",
    "controlasteis": "controlar",
    "controle": "controlar",
    "controlemos": "controlar",
    "controlen": "controlar",
    "controles": "controlar",
    "controlo": "controlar",
    "controlábamos": "controlar",
    "controláis": "controlar",
    "controláramos": "controlar",
    "controláremos": "controlar",
    "controlásemos": "controlar",
    "controlé": "controlar",
    "controléis": "controlar",
    "controló": "controlar",
    "controversias": "controversia",
    "controvertibles": "controvertible",
    "controvertid": "controvertir",
    "controvertida": "controvertir",
    "controvertidas": "controvertir",
    "controvertido": "controvertir",
    "controvertidos": "controvertir",
    "controvertimos": "controvertir",
    "controvertiremos": "controvertir",
    "controvertirá": "controvertir",
    "controvertirán": "controvertir",
    "controvertirás": "controvertir",
    "controvertiré": "controvertir",
    "controvertiréis": "controvertir",
    "controvertiría": "controvertir",
    "controvertiríais": "controvertir",
    "controvertiríamos": "controvertir",
    "controvertirían": "controvertir",
    "controvertirías": "controvertir",
    "controvertiste": "controvertir",
    "controvertisteis": "controvertir",
    "controvertí": "controvertir",
    "controvertía": "controvertir",
    "controvertíais": "controvertir",
    "controvertíamos": "controvertir",
    "controvertían": "controvertir",
    "controvertías": "controvertir",
    "controvertís": "controvertir",
    "controvierta": "controvertir",
    "controviertan": "controvertir",
    "controviertas": "controvertir",
    "controvierte": "controvertir",
    "controvierten": "controvertir",
    "controviertes": "controvertir",
    "controvierto": "controvertir",
    "controvirtamos": "controvertir",
    "controvirtiendo": "controvertir",
    "controvirtiera": "controvertir",
    "controvirtierais": "controvertir",
    "controvirtieran": "controvertir",
    "controvirtieras": "controvertir",
    "controvirtiere": "controvertir",
    "controvirtiereis": "controvertir",
    "controvirtieren": "controvertir",
    "controvirtieres": "controvertir",
    "controvirtieron": "controvertir",
    "controvirtiese": "controvertir",
    "controvirtieseis": "controvertir",
    "controvirtiesen": "controvertir",
    "controvirtieses": "controvertir",
    "controvirtiéramos": "controvertir",
    "controvirtiéremos": "controvertir",
    "controvirtiésemos": "controvertir",
    "controvirtió": "controvertir",
    "controvirtáis": "controvertir",
    "contráctiles": "contráctil",
    "contubernios": "contubernio",
    "contumaces": "contumaz",
    "contumacias": "contumacia",
    "contumazes": "contumaz",
    "contumelias": "contumelia",
    "contunda": "contundir",
    "contundamos": "contundir",
    "contundan": "contundir",
    "contundas": "contundir",
    "contunde": "contundir",
    "contunden": "contundir",
    "contundencias": "contundencia",
    "contundentes": "contundente",
    "contundes": "contundir",
    "contundid": "contundir",
    "contundida": "contundir",
    "contundidas": "contundir",
    "contundido": "contundir",
    "contundidos": "contundir",
    "contundiendo": "contundir",
    "contundiera": "contundir",
    "contundierais": "contundir",
    "contundieran": "contundir",
    "contundieras": "contundir",
    "contundiere": "contundir",
    "contundiereis": "contundir",
    "contundieren": "contundir",
    "contundieres": "contundir",
    "contundieron": "contundir",
    "contundiese": "contundir",
    "contundieseis": "contundir",
    "contundiesen": "contundir",
    "contundieses": "contundir",
    "contundimos": "contundir",
    "contundiremos": "contundir",
    "contundirá": "contundir",
    "contundirán": "contundir",
    "contundirás": "contundir",
    "contundiré": "contundir",
    "contundiréis": "contundir",
    "contundiría": "contundir",
    "contundiríais": "contundir",
    "contundiríamos": "contundir",
    "contundirían": "contundir",
    "contundirías": "contundir",
    "contundiste": "contundir",
    "contundisteis": "contundir",
    "contundiéramos": "contundir",
    "contundiéremos": "contundir",
    "contundiésemos": "contundir",
    "contundió": "contundir",
    "contundo": "contundir",
    "contundáis": "contundir",
    "contundí": "contundir",
    "contundía": "contundir",
    "contundíais": "contundir",
    "contundíamos": "contundir",
    "contundían": "contundir",
    "contundías": "contundir",
    "contundís": "contundir",
    "conturba": "conturbar",
    "conturbaba": "conturbar",
    "conturbabais": "conturbar",
    "conturbaban": "conturbar",
    "conturbabas": "conturbar",
    "conturbaciones": "conturbación",
    "conturbad": "conturbar",
    "conturbada": "conturbar",
    "conturbadas": "conturbar",
    "conturbado": "conturbar",
    "conturbadora": "conturbador",
    "conturbadoras": "conturbador",
    "conturbadores": "conturbador",
    "conturbados": "conturbar",
    "conturbamos": "conturbar",
    "conturban": "conturbar",
    "conturbando": "conturbar",
    "conturbara": "conturbar",
    "conturbarais": "conturbar",
    "conturbaran": "conturbar",
    "conturbaras": "conturbar",
    "conturbare": "conturbar",
    "conturbareis": "conturbar",
    "conturbaremos": "conturbar",
    "conturbaren": "conturbar",
    "conturbares": "conturbar",
    "conturbaron": "conturbar",
    "conturbará": "conturbar",
    "conturbarán": "conturbar",
    "conturbarás": "conturbar",
    "conturbaré": "conturbar",
    "conturbaréis": "conturbar",
    "conturbaría": "conturbar",
    "conturbaríais": "conturbar",
    "conturbaríamos": "conturbar",
    "conturbarían": "conturbar",
    "conturbarías": "conturbar",
    "conturbas": "conturbar",
    "conturbase": "conturbar",
    "conturbaseis": "conturbar",
    "conturbasen": "conturbar",
    "conturbases": "conturbar",
    "conturbaste": "conturbar",
    "conturbasteis": "conturbar",
    "conturbativa": "conturbativo",
    "conturbativas": "conturbativo",
    "conturbativos": "conturbativo",
    "conturbe": "conturbar",
    "conturbemos": "conturbar",
    "conturben": "conturbar",
    "conturbes": "conturbar",
    "conturbo": "conturbar",
    "conturbábamos": "conturbar",
    "conturbáis": "conturbar",
    "conturbáramos": "conturbar",
    "conturbáremos": "conturbar",
    "conturbásemos": "conturbar",
    "conturbé": "conturbar",
    "conturbéis": "conturbar",
    "conturbó": "conturbar",
    "contusa": "contuso",
    "contusas": "contuso",
    "contusiones": "contusión",
    "contusos": "contuso",
    "contuve": "contener",
    "contuviera": "contener",
    "contuvierais": "contener",
    "contuvieran": "contener",
    "contuvieras": "contener",
    "contuviere": "contener",
    "contuviereis": "contener",
    "contuvieren": "contener",
    "contuvieres": "contener",
    "contuvieron": "contener",
    "contuviese": "contener",
    "contuvieseis": "contener",
    "contuviesen": "contener",
    "contuvieses": "contener",
    "contuvimos": "contener",
    "contuviste": "contener",
    "contuvisteis": "contener",
    "contuviéramos": "contener",
    "contuviéremos": "contener",
    "contuviésemos": "contener",
    "contuvo": "contener",
    "contábamos": "contar",
    "contáis": "contar",
    "contáramos": "contar",
    "contáremos": "contar",
    "contásemos": "contar",
    "conté": "contar",
    "contéis": "contar",
    "contén": "contener",
    "contérmina": "contérmino",
    "contérminas": "contérmino",
    "contérminos": "contérmino",
    "contó": "contar",
    "conurbaciones": "conurbación",
    "convalece": "convalecer",
    "convaleced": "convalecer",
    "convalecemos": "convalecer",
    "convalecen": "convalecer",
    "convalecencias": "convalecencia",
    "convaleceremos": "convalecer",
    "convalecerá": "convalecer",
    "convalecerán": "convalecer",
    "convalecerás": "convalecer",
    "convaleceré": "convalecer",
    "convaleceréis": "convalecer",
    "convalecería": "convalecer",
    "convaleceríais": "convalecer",
    "convaleceríamos": "convalecer",
    "convalecerían": "convalecer",
    "convalecerías": "convalecer",
    "convaleces": "convalecer",
    "convalecida": "convalecer",
    "convalecidas": "convalecer",
    "convalecido": "convalecer",
    "convalecidos": "convalecer",
    "convaleciendo": "convalecer",
    "convalecientes": "convaleciente",
    "convaleciera": "convalecer",
    "convalecierais": "convalecer",
    "convalecieran": "convalecer",
    "convalecieras": "convalecer",
    "convaleciere": "convalecer",
    "convaleciereis": "convalecer",
    "convalecieren": "convalecer",
    "convalecieres": "convalecer",
    "convalecieron": "convalecer",
    "convaleciese": "convalecer",
    "convalecieseis": "convalecer",
    "convaleciesen": "convalecer",
    "convalecieses": "convalecer",
    "convalecimos": "convalecer",
    "convaleciste": "convalecer",
    "convalecisteis": "convalecer",
    "convaleciéramos": "convalecer",
    "convaleciéremos": "convalecer",
    "convaleciésemos": "convalecer",
    "convaleció": "convalecer",
    "convalecéis": "convalecer",
    "convalecí": "convalecer",
    "convalecía": "convalecer",
    "convalecíais": "convalecer",
    "convalecíamos": "convalecer",
    "convalecían": "convalecer",
    "convalecías": "convalecer",
    "convalezca": "convalecer",
    "convalezcamos": "convalecer",
    "convalezcan": "convalecer",
    "convalezcas": "convalecer",
    "convalezco": "convalecer",
    "convalezcáis": "convalecer",
    "convalida": "convalidar",
    "convalidaba": "convalidar",
    "convalidabais": "convalidar",
    "convalidaban": "convalidar",
    "convalidabas": "convalidar",
    "convalidaciones": "convalidación",
    "convalidad": "convalidar",
    "convalidada": "convalidar",
    "convalidadas": "convalidar",
    "convalidado": "convalidar",
    "convalidados": "convalidar",
    "convalidamos": "convalidar",
    "convalidan": "convalidar",
    "convalidando": "convalidar",
    "convalidara": "convalidar",
    "convalidarais": "convalidar",
    "convalidaran": "convalidar",
    "convalidaras": "convalidar",
    "convalidare": "convalidar",
    "convalidareis": "convalidar",
    "convalidaremos": "convalidar",
    "convalidaren": "convalidar",
    "convalidares": "convalidar",
    "convalidaron": "convalidar",
    "convalidará": "convalidar",
    "convalidarán": "convalidar",
    "convalidarás": "convalidar",
    "convalidaré": "convalidar",
    "convalidaréis": "convalidar",
    "convalidaría": "convalidar",
    "convalidaríais": "convalidar",
    "convalidaríamos": "convalidar",
    "convalidarían": "convalidar",
    "convalidarías": "convalidar",
    "convalidas": "convalidar",
    "convalidase": "convalidar",
    "convalidaseis": "convalidar",
    "convalidasen": "convalidar",
    "convalidases": "convalidar",
    "convalidaste": "convalidar",
    "convalidasteis": "convalidar",
    "convalide": "convalidar",
    "convalidemos": "convalidar",
    "convaliden": "convalidar",
    "convalides": "convalidar",
    "convalido": "convalidar",
    "convalidábamos": "convalidar",
    "convalidáis": "convalidar",
    "convalidáramos": "convalidar",
    "convalidáremos": "convalidar",
    "convalidásemos": "convalidar",
    "convalidé": "convalidar",
    "convalidéis": "convalidar",
    "convalidó": "convalidar",
    "convecciones": "convección",
    "convecina": "convecino",
    "convecinas": "convecino",
    "convecinos": "convecino",
    "convectiva": "convectivo",
    "convectivas": "convectivo",
    "convectivos": "convectivo",
    "convectores": "convector",
    "conven": "convenir",
    "convence": "convencer",
    "convenced": "convencer",
    "convencemos": "convencer",
    "convencen": "convencer",
    "convenceremos": "convencer",
    "convencerá": "convencer",
    "convencerán": "convencer",
    "convencerás": "convencer",
    "convenceré": "convencer",
    "convenceréis": "convencer",
    "convencería": "convencer",
    "convenceríais": "convencer",
    "convenceríamos": "convencer",
    "convencerían": "convencer",
    "convencerías": "convencer",
    "convences": "convencer",
    "convencida": "convencer",
    "convencidas": "convencer",
    "convencido": "convencer",
    "convencidos": "convencer",
    "convenciendo": "convencer",
    "convenciera": "convencer",
    "convencierais": "convencer",
    "convencieran": "convencer",
    "convencieras": "convencer",
    "convenciere": "convencer",
    "convenciereis": "convencer",
    "convencieren": "convencer",
    "convencieres": "convencer",
    "convencieron": "convencer",
    "convenciese": "convencer",
    "convencieseis": "convencer",
    "convenciesen": "convencer",
    "convencieses": "convencer",
    "convencimientos": "convencimiento",
    "convencimos": "convencer",
    "convencionales": "convencional",
    "convencionalismos": "convencionalismo",
    "convencionalistas": "convencionalista",
    "convenciones": "convención",
    "convenciste": "convencer",
    "convencisteis": "convencer",
    "convenciéramos": "convencer",
    "convenciéremos": "convencer",
    "convenciésemos": "convencer",
    "convenció": "convencer",
    "convencéis": "convencer",
    "convencí": "convencer",
    "convencía": "convencer",
    "convencíais": "convencer",
    "convencíamos": "convencer",
    "convencían": "convencer",
    "convencías": "convencer",
    "convendremos": "convenir",
    "convendrá": "convenir",
    "convendrán": "convenir",
    "convendrás": "convenir",
    "convendré": "convenir",
    "convendréis": "convenir",
    "convendría": "convenir",
    "convendríais": "convenir",
    "convendríamos": "convenir",
    "convendrían": "convenir",
    "convendrías": "convenir",
    "convenga": "convenir",
    "convengamos": "convenir",
    "convengan": "convenir",
    "convengas": "convenir",
    "convengo": "convenir",
    "convengáis": "convenir",
    "convenibles": "convenible",
    "convenid": "convenir",
    "convenida": "convenir",
    "convenidas": "convenir",
    "convenido": "convenir",
    "convenidos": "convenir",
    "conveniencias": "conveniencia",
    "convenientes": "conveniente",
    "convenimos": "convenir",
    "convenios": "convenio",
    "conventos": "convento",
    "conventuales": "conventual",
    "conventualidades": "conventualidad",
    "convenza": "convencer",
    "convenzamos": "convencer",
    "convenzan": "convencer",
    "convenzas": "convencer",
    "convenzo": "convencer",
    "convenzáis": "convencer",
    "convenía": "convenir",
    "conveníais": "convenir",
    "conveníamos": "convenir",
    "convenían": "convenir",
    "convenías": "convenir",
    "convenís": "convenir",
    "converge": "convergir",
    "converged": "converger",
    "convergemos": "converger",
    "convergen": "convergir",
    "convergencias": "convergencia",
    "convergentes": "convergente",
    "convergeremos": "converger",
    "convergerá": "converger",
    "convergerán": "converger",
    "convergerás": "converger",
    "convergeré": "converger",
    "convergeréis": "converger",
    "convergería": "converger",
    "convergeríais": "converger",
    "convergeríamos": "converger",
    "convergerían": "converger",
    "convergerías": "converger",
    "converges": "convergir",
    "convergid": "convergir",
    "convergida": "convergir",
    "convergidas": "convergir",
    "convergido": "convergir",
    "convergidos": "convergir",
    "convergiendo": "convergir",
    "convergiera": "convergir",
    "convergierais": "convergir",
    "convergieran": "convergir",
    "convergieras": "convergir",
    "convergiere": "convergir",
    "convergiereis": "convergir",
    "convergieren": "convergir",
    "convergieres": "convergir",
    "convergieron": "convergir",
    "convergiese": "convergir",
    "convergieseis": "convergir",
    "convergiesen": "convergir",
    "convergieses": "convergir",
    "convergimos": "convergir",
    "convergiremos": "convergir",
    "convergirá": "convergir",
    "convergirán": "convergir",
    "convergirás": "convergir",
    "convergiré": "convergir",
    "convergiréis": "convergir",
    "convergiría": "convergir",
    "convergiríais": "convergir",
    "convergiríamos": "convergir",
    "convergirían": "convergir",
    "convergirías": "convergir",
    "convergiste": "convergir",
    "convergisteis": "convergir",
    "convergiéramos": "convergir",
    "convergiéremos": "convergir",
    "convergiésemos": "convergir",
    "convergió": "convergir",
    "convergéis": "converger",
    "convergí": "convergir",
    "convergía": "convergir",
    "convergíais": "convergir",
    "convergíamos": "convergir",
    "convergían": "convergir",
    "convergías": "convergir",
    "convergís": "convergir",
    "converja": "convergir",
    "converjamos": "convergir",
    "converjan": "convergir",
    "converjas": "convergir",
    "converjo": "convergir",
    "converjáis": "convergir",
    "conversa": "converso",
    "conversaba": "conversar",
    "conversabais": "conversar",
    "conversaban": "conversar",
    "conversabas": "conversar",
    "conversacionales": "conversacional",
    "conversaciones": "conversación",
    "conversad": "conversar",
    "conversada": "conversar",
    "conversadas": "conversar",
    "conversado": "conversar",
    "conversadora": "conversador",
    "conversadoras": "conversador",
    "conversadores": "conversador",
    "conversados": "conversar",
    "conversamos": "conversar",
    "conversan": "conversar",
    "conversando": "conversar",
    "conversara": "conversar",
    "conversarais": "conversar",
    "conversaran": "conversar",
    "conversaras": "conversar",
    "conversare": "conversar",
    "conversareis": "conversar",
    "conversaremos": "conversar",
    "conversaren": "conversar",
    "conversares": "conversar",
    "conversaron": "conversar",
    "conversará": "conversar",
    "conversarán": "conversar",
    "conversarás": "conversar",
    "conversaré": "conversar",
    "conversaréis": "conversar",
    "conversaría": "conversar",
    "conversaríais": "conversar",
    "conversaríamos": "conversar",
    "conversarían": "conversar",
    "conversarías": "conversar",
    "conversas": "converso",
    "conversase": "conversar",
    "conversaseis": "conversar",
    "conversasen": "conversar",
    "conversases": "conversar",
    "conversaste": "conversar",
    "conversasteis": "conversar",
    "converse": "conversar",
    "conversemos": "conversar",
    "conversen": "conversar",
    "converses": "conversar",
    "conversiones": "conversión",
    "conversiva": "conversivo",
    "conversivas": "conversivo",
    "conversivos": "conversivo",
    "converso": "conversar",
    "conversores": "conversor",
    "conversos": "converso",
    "conversábamos": "conversar",
    "conversáis": "conversar",
    "conversáramos": "conversar",
    "conversáremos": "conversar",
    "conversásemos": "conversar",
    "conversé": "conversar",
    "converséis": "conversar",
    "conversó": "conversar",
    "convertibilidades": "convertibilidad",
    "convertibles": "convertible",
    "convertid": "convertir",
    "convertida": "convertir",
    "convertidas": "convertir",
    "convertido": "convertir",
    "convertidora": "convertidor",
    "convertidoras": "convertidor",
    "convertidores": "convertidor",
    "convertidos": "convertir",
    "convertimos": "convertir",
    "convertiremos": "convertir",
    "convertirá": "convertir",
    "convertirán": "convertir",
    "convertirás": "convertir",
    "convertiré": "convertir",
    "convertiréis": "convertir",
    "convertiría": "convertir",
    "convertiríais": "convertir",
    "convertiríamos": "convertir",
    "convertirían": "convertir",
    "convertirías": "convertir",
    "convertiste": "convertir",
    "convertisteis": "convertir",
    "convertí": "convertir",
    "convertía": "convertir",
    "convertíais": "convertir",
    "convertíamos": "convertir",
    "convertían": "convertir",
    "convertías": "convertir",
    "convertís": "convertir",
    "convexa": "convexo",
    "convexas": "convexo",
    "convexidades": "convexidad",
    "convexiones": "convexión",
    "convexocóncava": "convexocóncavo",
    "convexocóncavas": "convexocóncavo",
    "convexocóncavos": "convexocóncavo",
    "convexos": "convexo",
    "convicciones": "convicción",
    "convicta": "convicto",
    "convictas": "convicto",
    "convictorios": "convictorio",
    "convictos": "convicto",
    "convida": "convidar",
    "convidaba": "convidar",
    "convidabais": "convidar",
    "convidaban": "convidar",
    "convidabas": "convidar",
    "convidad": "convidar",
    "convidada": "convidar",
    "convidadas": "convidar",
    "convidado": "convidar",
    "convidados": "convidar",
    "convidamos": "convidar",
    "convidan": "convidar",
    "convidando": "convidar",
    "convidara": "convidar",
    "convidarais": "convidar",
    "convidaran": "convidar",
    "convidaras": "convidar",
    "convidare": "convidar",
    "convidareis": "convidar",
    "convidaremos": "convidar",
    "convidaren": "convidar",
    "convidares": "convidar",
    "convidaron": "convidar",
    "convidará": "convidar",
    "convidarán": "convidar",
    "convidarás": "convidar",
    "convidaré": "convidar",
    "convidaréis": "convidar",
    "convidaría": "convidar",
    "convidaríais": "convidar",
    "convidaríamos": "convidar",
    "convidarían": "convidar",
    "convidarías": "convidar",
    "convidas": "convidar",
    "convidase": "convidar",
    "convidaseis": "convidar",
    "convidasen": "convidar",
    "convidases": "convidar",
    "convidaste": "convidar",
    "convidasteis": "convidar",
    "convide": "convidar",
    "convidemos": "convidar",
    "conviden": "convidar",
    "convides": "convidar",
    "convido": "convidar",
    "convidábamos": "convidar",
    "convidáis": "convidar",
    "convidáramos": "convidar",
    "convidáremos": "convidar",
    "convidásemos": "convidar",
    "convidé": "convidar",
    "convidéis": "convidar",
    "convidó": "convidar",
    "conviene": "convenir",
    "convienen": "convenir",
    "convienes": "convenir",
    "convierta": "convertir",
    "conviertan": "convertir",
    "conviertas": "convertir",
    "convierte": "convertir",
    "convierten": "convertir",
    "conviertes": "convertir",
    "convierto": "convertir",
    "convincentes": "convincente",
    "convine": "convenir",
    "conviniendo": "convenir",
    "conviniera": "convenir",
    "convinierais": "convenir",
    "convinieran": "convenir",
    "convinieras": "convenir",
    "conviniere": "convenir",
    "conviniereis": "convenir",
    "convinieren": "convenir",
    "convinieres": "convenir",
    "convinieron": "convenir",
    "conviniese": "convenir",
    "convinieseis": "convenir",
    "conviniesen": "convenir",
    "convinieses": "convenir",
    "convinimos": "convenir",
    "conviniste": "convenir",
    "convinisteis": "convenir",
    "conviniéramos": "convenir",
    "conviniéremos": "convenir",
    "conviniésemos": "convenir",
    "convino": "convenir",
    "convirtamos": "convertir",
    "convirtiendo": "convertir",
    "convirtiera": "convertir",
    "convirtierais": "convertir",
    "convirtieran": "convertir",
    "convirtieras": "convertir",
    "convirtiere": "convertir",
    "convirtiereis": "convertir",
    "convirtieren": "convertir",
    "convirtieres": "convertir",
    "convirtieron": "convertir",
    "convirtiese": "convertir",
    "convirtieseis": "convertir",
    "convirtiesen": "convertir",
    "convirtieses": "convertir",
    "convirtiéramos": "convertir",
    "convirtiéremos": "convertir",
    "convirtiésemos": "convertir",
    "convirtió": "convertir",
    "convirtáis": "convertir",
    "convites": "convite",
    "conviva": "convivir",
    "convivamos": "convivir",
    "convivan": "convivir",
    "convivas": "convivir",
    "convive": "convivir",
    "conviven": "convivir",
    "convivenciales": "convivencial",
    "convivencias": "convivencia",
    "convives": "convivir",
    "convivid": "convivir",
    "convivida": "convivir",
    "convividas": "convivir",
    "convivido": "convivir",
    "convividos": "convivir",
    "conviviendo": "convivir",
    "convivientes": "conviviente",
    "conviviera": "convivir",
    "convivierais": "convivir",
    "convivieran": "convivir",
    "convivieras": "convivir",
    "conviviere": "convivir",
    "conviviereis": "convivir",
    "convivieren": "convivir",
    "convivieres": "convivir",
    "convivieron": "convivir",
    "conviviese": "convivir",
    "convivieseis": "convivir",
    "conviviesen": "convivir",
    "convivieses": "convivir",
    "convivimos": "convivir",
    "conviviremos": "convivir",
    "convivirá": "convivir",
    "convivirán": "convivir",
    "convivirás": "convivir",
    "conviviré": "convivir",
    "conviviréis": "convivir",
    "conviviría": "convivir",
    "conviviríais": "convivir",
    "conviviríamos": "convivir",
    "convivirían": "convivir",
    "convivirías": "convivir",
    "conviviste": "convivir",
    "convivisteis": "convivir",
    "conviviéramos": "convivir",
    "conviviéremos": "convivir",
    "conviviésemos": "convivir",
    "convivió": "convivir",
    "convivo": "convivir",
    "conviváis": "convivir",
    "conviví": "convivir",
    "convivía": "convivir",
    "convivíais": "convivir",
    "convivíamos": "convivir",
    "convivían": "convivir",
    "convivías": "convivir",
    "convivís": "convivir",
    "convoca": "convocar",
    "convocaba": "convocar",
    "convocabais": "convocar",
    "convocaban": "convocar",
    "convocabas": "convocar",
    "convocaciones": "convocación",
    "convocad": "convocar",
    "convocada": "convocar",
    "convocadas": "convocar",
    "convocado": "convocar",
    "convocadora": "convocador",
    "convocadoras": "convocador",
    "convocadores": "convocador",
    "convocados": "convocar",
    "convocamos": "convocar",
    "convocan": "convocar",
    "convocando": "convocar",
    "convocantes": "convocante",
    "convocara": "convocar",
    "convocarais": "convocar",
    "convocaran": "convocar",
    "convocaras": "convocar",
    "convocare": "convocar",
    "convocareis": "convocar",
    "convocaremos": "convocar",
    "convocaren": "convocar",
    "convocares": "convocar",
    "convocaron": "convocar",
    "convocará": "convocar",
    "convocarán": "convocar",
    "convocarás": "convocar",
    "convocaré": "convocar",
    "convocaréis": "convocar",
    "convocaría": "convocar",
    "convocaríais": "convocar",
    "convocaríamos": "convocar",
    "convocarían": "convocar",
    "convocarías": "convocar",
    "convocas": "convocar",
    "convocase": "convocar",
    "convocaseis": "convocar",
    "convocasen": "convocar",
    "convocases": "convocar",
    "convocaste": "convocar",
    "convocasteis": "convocar",
    "convocatoria": "convocatorio",
    "convocatorias": "convocatorio",
    "convocatorios": "convocatorio",
    "convoco": "convocar",
    "convocábamos": "convocar",
    "convocáis": "convocar",
    "convocáramos": "convocar",
    "convocáremos": "convocar",
    "convocásemos": "convocar",
    "convocó": "convocar",
    "convoluciones": "convolución",
    "convolvuláceas": "convolvulácea",
    "convoque": "convocar",
    "convoquemos": "convocar",
    "convoquen": "convocar",
    "convoques": "convocar",
    "convoqué": "convocar",
    "convoquéis": "convocar",
    "convoya": "convoyar",
    "convoyaba": "convoyar",
    "convoyabais": "convoyar",
    "convoyaban": "convoyar",
    "convoyabas": "convoyar",
    "convoyad": "convoyar",
    "convoyada": "convoyar",
    "convoyadas": "convoyar",
    "convoyado": "convoyar",
    "convoyados": "convoyar",
    "convoyamos": "convoyar",
    "convoyan": "convoyar",
    "convoyando": "convoyar",
    "convoyara": "convoyar",
    "convoyarais": "convoyar",
    "convoyaran": "convoyar",
    "convoyaras": "convoyar",
    "convoyare": "convoyar",
    "convoyareis": "convoyar",
    "convoyaremos": "convoyar",
    "convoyaren": "convoyar",
    "convoyares": "convoyar",
    "convoyaron": "convoyar",
    "convoyará": "convoyar",
    "convoyarán": "convoyar",
    "convoyarás": "convoyar",
    "convoyaré": "convoyar",
    "convoyaréis": "convoyar",
    "convoyaría": "convoyar",
    "convoyaríais": "convoyar",
    "convoyaríamos": "convoyar",
    "convoyarían": "convoyar",
    "convoyarías": "convoyar",
    "convoyas": "convoyar",
    "convoyase": "convoyar",
    "convoyaseis": "convoyar",
    "convoyasen": "convoyar",
    "convoyases": "convoyar",
    "convoyaste": "convoyar",
    "convoyasteis": "convoyar",
    "convoye": "convoyar",
    "convoyemos": "convoyar",
    "convoyen": "convoyar",
    "convoyes": "convoyar",
    "convoyo": "convoyar",
    "convoys": "convoy",
    "convoyábamos": "convoyar",
    "convoyáis": "convoyar",
    "convoyáramos": "convoyar",
    "convoyáremos": "convoyar",
    "convoyásemos": "convoyar",
    "convoyé": "convoyar",
    "convoyéis": "convoyar",
    "convoyó": "convoyar",
    "convulsa": "convulso",
    "convulsas": "convulso",
    "convulsiona": "convulsionar",
    "convulsionaba": "convulsionar",
    "convulsionabais": "convulsionar",
    "convulsionaban": "convulsionar",
    "convulsionabas": "convulsionar",
    "convulsionad": "convulsionar",
    "convulsionada": "convulsionar",
    "convulsionadas": "convulsionar",
    "convulsionado": "convulsionar",
    "convulsionados": "convulsionar",
    "convulsionamos": "convulsionar",
    "convulsionan": "convulsionar",
    "convulsionando": "convulsionar",
    "convulsionara": "convulsionar",
    "convulsionarais": "convulsionar",
    "convulsionaran": "convulsionar",
    "convulsionaras": "convulsionar",
    "convulsionare": "convulsionar",
    "convulsionareis": "convulsionar",
    "convulsionaremos": "convulsionar",
    "convulsionaren": "convulsionar",
    "convulsionares": "convulsionar",
    "convulsionaron": "convulsionar",
    "convulsionará": "convulsionar",
    "convulsionarán": "convulsionar",
    "convulsionarás": "convulsionar",
    "convulsionaré": "convulsionar",
    "convulsionaréis": "convulsionar",
    "convulsionaría": "convulsionar",
    "convulsionaríais": "convulsionar",
    "convulsionaríamos": "convulsionar",
    "convulsionarían": "convulsionar",
    "convulsionarías": "convulsionar",
    "convulsionas": "convulsionar",
    "convulsionase": "convulsionar",
    "convulsionaseis": "convulsionar",
    "convulsionasen": "convulsionar",
    "convulsionases": "convulsionar",
    "convulsionaste": "convulsionar",
    "convulsionasteis": "convulsionar",
    "convulsione": "convulsionar",
    "convulsionemos": "convulsionar",
    "convulsionen": "convulsionar",
    "convulsiones": "convulsionar",
    "convulsiono": "convulsionar",
    "convulsionábamos": "convulsionar",
    "convulsionáis": "convulsionar",
    "convulsionáramos": "convulsionar",
    "convulsionáremos": "convulsionar",
    "convulsionásemos": "convulsionar",
    "convulsioné": "convulsionar",
    "convulsionéis": "convulsionar",
    "convulsionó": "convulsionar",
    "convulsiva": "convulsivo",
    "convulsivas": "convulsivo",
    "convulsivos": "convulsivo",
    "convulsos": "convulso",
    "conyugales": "conyugal",
    "conyugicidios": "conyugicidio",
    "conífera": "conífero",
    "coníferas": "conífero",
    "coníferos": "conífero",
    "conímetros": "conímetro",
    "conúcleos": "conúcleo",
    "cooficiales": "cooficial",
    "cooficialidades": "cooficialidad",
    "coolies": "coolie",
    "coopera": "cooperar",
    "cooperaba": "cooperar",
    "cooperabais": "cooperar",
    "cooperaban": "cooperar",
    "cooperabas": "cooperar",
    "cooperaciones": "cooperación",
    "cooperad": "cooperar",
    "cooperada": "cooperar",
    "cooperadas": "cooperar",
    "cooperado": "cooperar",
    "cooperadora": "cooperador",
    "cooperadoras": "cooperador",
    "cooperadores": "cooperador",
    "cooperados": "cooperar",
    "cooperamos": "cooperar",
    "cooperan": "cooperar",
    "cooperando": "cooperar",
    "cooperantes": "cooperante",
    "cooperara": "cooperar",
    "cooperarais": "cooperar",
    "cooperaran": "cooperar",
    "cooperaras": "cooperar",
    "cooperare": "cooperar",
    "cooperareis": "cooperar",
    "cooperaremos": "cooperar",
    "cooperaren": "cooperar",
    "cooperares": "cooperar",
    "cooperaron": "cooperar",
    "cooperará": "cooperar",
    "cooperarán": "cooperar",
    "cooperarás": "cooperar",
    "cooperaré": "cooperar",
    "cooperaréis": "cooperar",
    "cooperaría": "cooperar",
    "cooperaríais": "cooperar",
    "cooperaríamos": "cooperar",
    "cooperarían": "cooperar",
    "cooperarías": "cooperar",
    "cooperas": "cooperar",
    "cooperase": "cooperar",
    "cooperaseis": "cooperar",
    "cooperasen": "cooperar",
    "cooperases": "cooperar",
    "cooperaste": "cooperar",
    "cooperasteis": "cooperar",
    "cooperativa": "cooperativo",
    "cooperativas": "cooperativo",
    "cooperativismos": "cooperativismo",
    "cooperativistas": "cooperativista",
    "cooperativos": "cooperativo",
    "coopere": "cooperar",
    "cooperemos": "cooperar",
    "cooperen": "cooperar",
    "cooperes": "cooperar",
    "coopero": "cooperar",
    "cooperábamos": "cooperar",
    "cooperáis": "cooperar",
    "cooperáramos": "cooperar",
    "cooperáremos": "cooperar",
    "cooperásemos": "cooperar",
    "cooperé": "cooperar",
    "cooperéis": "cooperar",
    "cooperó": "cooperar",
    "coopta": "cooptar",
    "cooptaba": "cooptar",
    "cooptabais": "cooptar",
    "cooptaban": "cooptar",
    "cooptabas": "cooptar",
    "cooptad": "cooptar",
    "cooptada": "cooptar",
    "cooptadas": "cooptar",
    "cooptado": "cooptar",
    "cooptados": "cooptar",
    "cooptamos": "cooptar",
    "cooptan": "cooptar",
    "cooptando": "cooptar",
    "cooptara": "cooptar",
    "cooptarais": "cooptar",
    "cooptaran": "cooptar",
    "cooptaras": "cooptar",
    "cooptare": "cooptar",
    "cooptareis": "cooptar",
    "cooptaremos": "cooptar",
    "cooptaren": "cooptar",
    "cooptares": "cooptar",
    "cooptaron": "cooptar",
    "cooptará": "cooptar",
    "cooptarán": "cooptar",
    "cooptarás": "cooptar",
    "cooptaré": "cooptar",
    "cooptaréis": "cooptar",
    "cooptaría": "cooptar",
    "cooptaríais": "cooptar",
    "cooptaríamos": "cooptar",
    "cooptarían": "cooptar",
    "cooptarías": "cooptar",
    "cooptas": "cooptar",
    "cooptase": "cooptar",
    "cooptaseis": "cooptar",
    "cooptasen": "cooptar",
    "cooptases": "cooptar",
    "cooptaste": "cooptar",
    "cooptasteis": "cooptar",
    "coopte": "cooptar",
    "cooptemos": "cooptar",
    "coopten": "cooptar",
    "cooptes": "cooptar",
    "coopto": "cooptar",
    "cooptábamos": "cooptar",
    "cooptáis": "cooptar",
    "cooptáramos": "cooptar",
    "cooptáremos": "cooptar",
    "cooptásemos": "cooptar",
    "coopté": "cooptar",
    "cooptéis": "cooptar",
    "cooptó": "cooptar",
    "coordenada": "coordenado",
    "coordenadas": "coordenado",
    "coordenados": "coordenado",
    "coordina": "coordinar",
    "coordinaba": "coordinar",
    "coordinabais": "coordinar",
    "coordinaban": "coordinar",
    "coordinabas": "coordinar",
    "coordinaciones": "coordinación",
    "coordinad": "coordinar",
    "coordinada": "coordinar",
    "coordinadas": "coordinar",
    "coordinado": "coordinar",
    "coordinadora": "coordinador",
    "coordinadoras": "coordinador",
    "coordinadores": "coordinador",
    "coordinados": "coordinar",
    "coordinamos": "coordinar",
    "coordinan": "coordinar",
    "coordinando": "coordinar",
    "coordinantes": "coordinante",
    "coordinara": "coordinar",
    "coordinarais": "coordinar",
    "coordinaran": "coordinar",
    "coordinaras": "coordinar",
    "coordinare": "coordinar",
    "coordinareis": "coordinar",
    "coordinaremos": "coordinar",
    "coordinaren": "coordinar",
    "coordinares": "coordinar",
    "coordinaron": "coordinar",
    "coordinará": "coordinar",
    "coordinarán": "coordinar",
    "coordinarás": "coordinar",
    "coordinaré": "coordinar",
    "coordinaréis": "coordinar",
    "coordinaría": "coordinar",
    "coordinaríais": "coordinar",
    "coordinaríamos": "coordinar",
    "coordinarían": "coordinar",
    "coordinarías": "coordinar",
    "coordinas": "coordinar",
    "coordinase": "coordinar",
    "coordinaseis": "coordinar",
    "coordinasen": "coordinar",
    "coordinases": "coordinar",
    "coordinaste": "coordinar",
    "coordinasteis": "coordinar",
    "coordinativa": "coordinativo",
    "coordinativas": "coordinativo",
    "coordinativos": "coordinativo",
    "coordinatógrafos": "coordinatógrafo",
    "coordine": "coordinar",
    "coordinemos": "coordinar",
    "coordinen": "coordinar",
    "coordines": "coordinar",
    "coordino": "coordinar",
    "coordinábamos": "coordinar",
    "coordináis": "coordinar",
    "coordináramos": "coordinar",
    "coordináremos": "coordinar",
    "coordinásemos": "coordinar",
    "coordiné": "coordinar",
    "coordinéis": "coordinar",
    "coordinó": "coordinar",
    "copa": "copar",
    "copaba": "copar",
    "copabais": "copar",
    "copaban": "copar",
    "copabas": "copar",
    "copad": "copar",
    "copada": "copar",
    "copadas": "copar",
    "copado": "copar",
    "copadores": "copador",
    "copados": "copar",
    "copaibas": "copaiba",
    "copales": "copal",
    "copamos": "copar",
    "copan": "copar",
    "copando": "copar",
    "copara": "copar",
    "coparais": "copar",
    "coparan": "copar",
    "coparas": "copar",
    "copare": "copar",
    "copareis": "copar",
    "coparemos": "copar",
    "coparen": "copar",
    "copares": "copar",
    "coparon": "copar",
    "coparticipaciones": "coparticipación",
    "copartícipes": "copartícipe",
    "copará": "copar",
    "coparán": "copar",
    "coparás": "copar",
    "coparé": "copar",
    "coparéis": "copar",
    "coparía": "copar",
    "coparíais": "copar",
    "coparíamos": "copar",
    "coparían": "copar",
    "coparías": "copar",
    "copas": "copar",
    "copase": "copar",
    "copaseis": "copar",
    "copasen": "copar",
    "copases": "copar",
    "copaste": "copar",
    "copasteis": "copar",
    "copayeros": "copayero",
    "copaínas": "copaína",
    "cope": "copar",
    "copea": "copear",
    "copeaba": "copear",
    "copeabais": "copear",
    "copeaban": "copear",
    "copeabas": "copear",
    "copead": "copear",
    "copeada": "copear",
    "copeadas": "copear",
    "copeado": "copear",
    "copeados": "copear",
    "copeamos": "copear",
    "copean": "copear",
    "copeando": "copear",
    "copeara": "copear",
    "copearais": "copear",
    "copearan": "copear",
    "copearas": "copear",
    "copeare": "copear",
    "copeareis": "copear",
    "copearemos": "copear",
    "copearen": "copear",
    "copeares": "copear",
    "copearon": "copear",
    "copeará": "copear",
    "copearán": "copear",
    "copearás": "copear",
    "copearé": "copear",
    "copearéis": "copear",
    "copearía": "copear",
    "copearíais": "copear",
    "copearíamos": "copear",
    "copearían": "copear",
    "copearías": "copear",
    "copeas": "copear",
    "copease": "copear",
    "copeaseis": "copear",
    "copeasen": "copear",
    "copeases": "copear",
    "copeaste": "copear",
    "copeasteis": "copear",
    "copecks": "copeck",
    "copee": "copear",
    "copeemos": "copear",
    "copeen": "copear",
    "copees": "copear",
    "copela": "copelar",
    "copelaba": "copelar",
    "copelabais": "copelar",
    "copelaban": "copelar",
    "copelabas": "copelar",
    "copelaciones": "copelación",
    "copelaciónes": "copelación",
    "copelad": "copelar",
    "copelada": "copelar",
    "copeladas": "copelar",
    "copelado": "copelar",
    "copelados": "copelar",
    "copelamos": "copelar",
    "copelan": "copelar",
    "copelando": "copelar",
    "copelara": "copelar",
    "copelarais": "copelar",
    "copelaran": "copelar",
    "copelaras": "copelar",
    "copelare": "copelar",
    "copelareis": "copelar",
    "copelaremos": "copelar",
    "copelaren": "copelar",
    "copelares": "copelar",
    "copelaron": "copelar",
    "copelará": "copelar",
    "copelarán": "copelar",
    "copelarás": "copelar",
    "copelaré": "copelar",
    "copelaréis": "copelar",
    "copelaría": "copelar",
    "copelaríais": "copelar",
    "copelaríamos": "copelar",
    "copelarían": "copelar",
    "copelarías": "copelar",
    "copelas": "copelar",
    "copelase": "copelar",
    "copelaseis": "copelar",
    "copelasen": "copelar",
    "copelases": "copelar",
    "copelaste": "copelar",
    "copelasteis": "copelar",
    "copele": "copelar",
    "copelemos": "copelar",
    "copelen": "copelar",
    "copeles": "copelar",
    "copelo": "copelar",
    "copelábamos": "copelar",
    "copeláis": "copelar",
    "copeláramos": "copelar",
    "copeláremos": "copelar",
    "copelásemos": "copelar",
    "copelé": "copelar",
    "copeléis": "copelar",
    "copeló": "copelar",
    "copemos": "copar",
    "copen": "copar",
    "copeo": "copear",
    "copeos": "copeo",
    "copera": "copero",
    "coperas": "copero",
    "copernicana": "copernicano",
    "copernicanas": "copernicano",
    "copernicanos": "copernicano",
    "coperos": "copero",
    "copes": "copar",
    "copetes": "copete",
    "copetines": "copetín",
    "copetuda": "copetudo",
    "copetudas": "copetudo",
    "copetudos": "copetudo",
    "copeábamos": "copear",
    "copeáis": "copear",
    "copeáramos": "copear",
    "copeáremos": "copear",
    "copeásemos": "copear",
    "copeé": "copear",
    "copeéis": "copear",
    "copeó": "copear",
    "copia": "copiar",
    "copiaba": "copiar",
    "copiabais": "copiar",
    "copiaban": "copiar",
    "copiabas": "copiar",
    "copiad": "copiar",
    "copiada": "copiar",
    "copiadas": "copiar",
    "copiado": "copiar",
    "copiadora": "copiador",
    "copiadoras": "copiadora",
    "copiadores": "copiador",
    "copiados": "copiar",
    "copiamos": "copiar",
    "copian": "copiar",
    "copiando": "copiar",
    "copiara": "copiar",
    "copiarais": "copiar",
    "copiaran": "copiar",
    "copiaras": "copiar",
    "copiare": "copiar",
    "copiareis": "copiar",
    "copiaremos": "copiar",
    "copiaren": "copiar",
    "copiares": "copiar",
    "copiaron": "copiar",
    "copiará": "copiar",
    "copiarán": "copiar",
    "copiarás": "copiar",
    "copiaré": "copiar",
    "copiaréis": "copiar",
    "copiaría": "copiar",
    "copiaríais": "copiar",
    "copiaríamos": "copiar",
    "copiarían": "copiar",
    "copiarías": "copiar",
    "copias": "copiar",
    "copiase": "copiar",
    "copiaseis": "copiar",
    "copiasen": "copiar",
    "copiases": "copiar",
    "copiaste": "copiar",
    "copiasteis": "copiar",
    "copie": "copiar",
    "copiemos": "copiar",
    "copien": "copiar",
    "copies": "copiar",
    "copilota": "copiloto",
    "copilotas": "copiloto",
    "copilotos": "copiloto",
    "copina": "copinar",
    "copinaba": "copinar",
    "copinabais": "copinar",
    "copinaban": "copinar",
    "copinabas": "copinar",
    "copinad": "copinar",
    "copinada": "copinar",
    "copinadas": "copinar",
    "copinado": "copinar",
    "copinados": "copinar",
    "copinamos": "copinar",
    "copinan": "copinar",
    "copinando": "copinar",
    "copinara": "copinar",
    "copinarais": "copinar",
    "copinaran": "copinar",
    "copinaras": "copinar",
    "copinare": "copinar",
    "copinareis": "copinar",
    "copinaremos": "copinar",
    "copinaren": "copinar",
    "copinares": "copinar",
    "copinaron": "copinar",
    "copinará": "copinar",
    "copinarán": "copinar",
    "copinarás": "copinar",
    "copinaré": "copinar",
    "copinaréis": "copinar",
    "copinaría": "copinar",
    "copinaríais": "copinar",
    "copinaríamos": "copinar",
    "copinarían": "copinar",
    "copinarías": "copinar",
    "copinas": "copinar",
    "copinase": "copinar",
    "copinaseis": "copinar",
    "copinasen": "copinar",
    "copinases": "copinar",
    "copinaste": "copinar",
    "copinasteis": "copinar",
    "copine": "copinar",
    "copinemos": "copinar",
    "copinen": "copinar",
    "copines": "copinar",
    "copino": "copinar",
    "copinábamos": "copinar",
    "copináis": "copinar",
    "copináramos": "copinar",
    "copináremos": "copinar",
    "copinásemos": "copinar",
    "copiné": "copinar",
    "copinéis": "copinar",
    "copinó": "copinar",
    "copio": "copiar",
    "copiona": "copión",
    "copionas": "copión",
    "copiones": "copión",
    "copiosa": "copioso",
    "copiosas": "copioso",
    "copiosos": "copioso",
    "copistas": "copista",
    "copitas": "copita",
    "copiábamos": "copiar",
    "copiáis": "copiar",
    "copiáramos": "copiar",
    "copiáremos": "copiar",
    "copiásemos": "copiar",
    "copié": "copiar",
    "copiéis": "copiar",
    "copió": "copiar",
    "coplana": "coplano",
    "coplanares": "coplanar",
    "coplanaria": "coplanario",
    "coplanarias": "coplanario",
    "coplanarios": "coplanario",
    "coplanas": "coplano",
    "coplanos": "coplano",
    "coplas": "copla",
    "coplea": "coplear",
    "copleaba": "coplear",
    "copleabais": "coplear",
    "copleaban": "coplear",
    "copleabas": "coplear",
    "coplead": "coplear",
    "copleada": "coplear",
    "copleadas": "coplear",
    "copleado": "coplear",
    "copleados": "coplear",
    "copleamos": "coplear",
    "coplean": "coplear",
    "copleando": "coplear",
    "copleara": "coplear",
    "coplearais": "coplear",
    "coplearan": "coplear",
    "coplearas": "coplear",
    "copleare": "coplear",
    "copleareis": "coplear",
    "coplearemos": "coplear",
    "coplearen": "coplear",
    "copleares": "coplear",
    "coplearon": "coplear",
    "copleará": "coplear",
    "coplearán": "coplear",
    "coplearás": "coplear",
    "coplearé": "coplear",
    "coplearéis": "coplear",
    "coplearía": "coplear",
    "coplearíais": "coplear",
    "coplearíamos": "coplear",
    "coplearían": "coplear",
    "coplearías": "coplear",
    "copleas": "coplear",
    "coplease": "coplear",
    "copleaseis": "coplear",
    "copleasen": "coplear",
    "copleases": "coplear",
    "copleaste": "coplear",
    "copleasteis": "coplear",
    "coplee": "coplear",
    "copleemos": "coplear",
    "copleen": "coplear",
    "coplees": "coplear",
    "copleo": "coplear",
    "coplera": "coplero",
    "copleras": "coplero",
    "copleros": "coplero",
    "copleábamos": "coplear",
    "copleáis": "coplear",
    "copleáramos": "coplear",
    "copleáremos": "coplear",
    "copleásemos": "coplear",
    "copleé": "coplear",
    "copleéis": "coplear",
    "copleó": "coplear",
    "coplistas": "coplista",
    "copo": "copar",
    "copolarizaciones": "copolarización",
    "copolicondensaciones": "copolicondensación",
    "copolimerizaciones": "copolimerización",
    "copolímeros": "copolímero",
    "copones": "copón",
    "copos": "copo",
    "copras": "copra",
    "coprecipitaciones": "coprecipitación",
    "copresidenta": "copresidente",
    "copresidentas": "copresidente",
    "copresidentes": "copresidente",
    "coproanticuerpos": "coproanticuerpo",
    "coprocesadores": "coprocesador",
    "coproducciones": "coproducción",
    "coproductora": "coproductor",
    "coproductoras": "coproductor",
    "coproductores": "coproductor",
    "coproductos": "coproducto",
    "coprofagias": "coprofagia",
    "coprolalias": "coprolalia",
    "coprolitos": "coprolito",
    "coprologías": "coprología",
    "copropiedades": "copropiedad",
    "copropietaria": "copropietario",
    "copropietarias": "copropietario",
    "copropietarios": "copropietario",
    "copropriedades": "copropriedad",
    "coprotagonistas": "coprotagonista",
    "coprófaga": "coprófago",
    "coprófagas": "coprófago",
    "coprófagos": "coprófago",
    "copta": "copto",
    "coptas": "copto",
    "coptos": "copto",
    "copuda": "copudo",
    "copudas": "copudo",
    "copudos": "copudo",
    "copula": "copular",
    "copulaba": "copular",
    "copulabais": "copular",
    "copulaban": "copular",
    "copulabas": "copular",
    "copulaciones": "copulación",
    "copulad": "copular",
    "copulada": "copular",
    "copuladas": "copular",
    "copulado": "copular",
    "copulados": "copular",
    "copulamos": "copular",
    "copulan": "copular",
    "copulando": "copular",
    "copulara": "copular",
    "copularais": "copular",
    "copularan": "copular",
    "copularas": "copular",
    "copulare": "copular",
    "copulareis": "copular",
    "copularemos": "copular",
    "copularen": "copular",
    "copulares": "copular",
    "copularon": "copular",
    "copulará": "copular",
    "copularán": "copular",
    "copularás": "copular",
    "copularé": "copular",
    "copularéis": "copular",
    "copularía": "copular",
    "copularíais": "copular",
    "copularíamos": "copular",
    "copularían": "copular",
    "copularías": "copular",
    "copulas": "copular",
    "copulase": "copular",
    "copulaseis": "copular",
    "copulasen": "copular",
    "copulases": "copular",
    "copulaste": "copular",
    "copulasteis": "copular",
    "copulativa": "copulativo",
    "copulativas": "copulativo",
    "copulativos": "copulativo",
    "copule": "copular",
    "copulemos": "copular",
    "copulen": "copular",
    "copules": "copular",
    "copulo": "copular",
    "copulábamos": "copular",
    "copuláis": "copular",
    "copuláramos": "copular",
    "copuláremos": "copular",
    "copulásemos": "copular",
    "copulé": "copular",
    "copuléis": "copular",
    "copuló": "copular",
    "copyrights": "copyright",
    "copábamos": "copar",
    "copáis": "copar",
    "copáramos": "copar",
    "copáremos": "copar",
    "copásemos": "copar",
    "copé": "copar",
    "copéis": "copar",
    "copó": "copar",
    "coque": "cocar",
    "coquemos": "cocar",
    "coquen": "cocar",
    "coqueras": "coquera",
    "coquerías": "coquería",
    "coques": "coque",
    "coqueta": "coqueto",
    "coquetas": "coqueto",
    "coquetea": "coquetear",
    "coqueteaba": "coquetear",
    "coqueteabais": "coquetear",
    "coqueteaban": "coquetear",
    "coqueteabas": "coquetear",
    "coquetead": "coquetear",
    "coqueteada": "coquetear",
    "coqueteadas": "coquetear",
    "coqueteado": "coquetear",
    "coqueteados": "coquetear",
    "coqueteamos": "coquetear",
    "coquetean": "coquetear",
    "coqueteando": "coquetear",
    "coqueteara": "coquetear",
    "coquetearais": "coquetear",
    "coquetearan": "coquetear",
    "coquetearas": "coquetear",
    "coqueteare": "coquetear",
    "coqueteareis": "coquetear",
    "coquetearemos": "coquetear",
    "coquetearen": "coquetear",
    "coqueteares": "coquetear",
    "coquetearon": "coquetear",
    "coqueteará": "coquetear",
    "coquetearán": "coquetear",
    "coquetearás": "coquetear",
    "coquetearé": "coquetear",
    "coquetearéis": "coquetear",
    "coquetearía": "coquetear",
    "coquetearíais": "coquetear",
    "coquetearíamos": "coquetear",
    "coquetearían": "coquetear",
    "coquetearías": "coquetear",
    "coqueteas": "coquetear",
    "coquetease": "coquetear",
    "coqueteaseis": "coquetear",
    "coqueteasen": "coquetear",
    "coqueteases": "coquetear",
    "coqueteaste": "coquetear",
    "coqueteasteis": "coquetear",
    "coquetee": "coquetear",
    "coqueteemos": "coquetear",
    "coqueteen": "coquetear",
    "coquetees": "coquetear",
    "coqueteo": "coquetear",
    "coqueteos": "coqueteo",
    "coqueterías": "coquetería",
    "coqueteábamos": "coquetear",
    "coqueteáis": "coquetear",
    "coqueteáramos": "coquetear",
    "coqueteáremos": "coquetear",
    "coqueteásemos": "coquetear",
    "coqueteé": "coquetear",
    "coqueteéis": "coquetear",
    "coqueteó": "coquetear",
    "coquetona": "coquetón",
    "coquetonas": "coquetón",
    "coquetones": "coquetón",
    "coquetos": "coqueto",
    "coquice": "coquizar",
    "coquicemos": "coquizar",
    "coquicen": "coquizar",
    "coquices": "coquizar",
    "coquicé": "coquizar",
    "coquicéis": "coquizar",
    "coquificaciones": "coquificación",
    "coquillas": "coquilla",
    "coquinas": "coquina",
    "coquiza": "coquizar",
    "coquizaba": "coquizar",
    "coquizabais": "coquizar",
    "coquizaban": "coquizar",
    "coquizabas": "coquizar",
    "coquizables": "coquizable",
    "coquizaciones": "coquización",
    "coquizad": "coquizar",
    "coquizada": "coquizar",
    "coquizadas": "coquizar",
    "coquizado": "coquizar",
    "coquizados": "coquizar",
    "coquizamos": "coquizar",
    "coquizan": "coquizar",
    "coquizando": "coquizar",
    "coquizara": "coquizar",
    "coquizarais": "coquizar",
    "coquizaran": "coquizar",
    "coquizaras": "coquizar",
    "coquizare": "coquizar",
    "coquizareis": "coquizar",
    "coquizaremos": "coquizar",
    "coquizaren": "coquizar",
    "coquizares": "coquizar",
    "coquizaron": "coquizar",
    "coquizará": "coquizar",
    "coquizarán": "coquizar",
    "coquizarás": "coquizar",
    "coquizaré": "coquizar",
    "coquizaréis": "coquizar",
    "coquizaría": "coquizar",
    "coquizaríais": "coquizar",
    "coquizaríamos": "coquizar",
    "coquizarían": "coquizar",
    "coquizarías": "coquizar",
    "coquizas": "coquizar",
    "coquizase": "coquizar",
    "coquizaseis": "coquizar",
    "coquizasen": "coquizar",
    "coquizases": "coquizar",
    "coquizaste": "coquizar",
    "coquizasteis": "coquizar",
    "coquizo": "coquizar",
    "coquizábamos": "coquizar",
    "coquizáis": "coquizar",
    "coquizáramos": "coquizar",
    "coquizáremos": "coquizar",
    "coquizásemos": "coquizar",
    "coquizó": "coquizar",
    "coqué": "cocar",
    "coquéis": "cocar",
    "coracanes": "coracán",
    "coraceros": "coracero",
    "corachas": "coracha",
    "coraciformes": "coraciforme",
    "coracinas": "coracina",
    "coracobraquiales": "coracobraquial",
    "coracoclaviculares": "coracoclavicular",
    "coracohumerales": "coracohumeral",
    "coracoidea": "coracoideo",
    "coracoideas": "coracoideo",
    "coracoideos": "coracoideo",
    "coradas": "corada",
    "corajes": "coraje",
    "corajinas": "corajina",
    "corajuda": "corajudo",
    "corajudas": "corajudo",
    "corajudos": "corajudo",
    "coralarios": "coralario",
    "corales": "coral",
    "coralina": "coralino",
    "coralinas": "coralino",
    "coralinos": "coralino",
    "coralináceas": "coralinácea",
    "coralífera": "coralífero",
    "coralíferas": "coralífero",
    "coralíferos": "coralífero",
    "coralígena": "coralígeno",
    "coralígenas": "coralígeno",
    "coralígenos": "coralígeno",
    "corambres": "corambre",
    "coraminas": "coramina",
    "coras": "cora",
    "corazas": "coraza",
    "coraznadas": "coraznada",
    "corazonadas": "corazonada",
    "corazoncillos": "corazoncillo",
    "corazones": "corazón",
    "corazonistas": "corazonista",
    "corbachos": "corbacho",
    "corballos": "corballo",
    "corbatas": "corbata",
    "corbatera": "corbatero",
    "corbateras": "corbatero",
    "corbateros": "corbatero",
    "corbaterías": "corbatería",
    "corbatines": "corbatín",
    "corbatos": "corbato",
    "corbetas": "corbeta",
    "corbonas": "corbona",
    "corceles": "corcel",
    "corcha": "corchar",
    "corchaba": "corchar",
    "corchabais": "corchar",
    "corchaban": "corchar",
    "corchabas": "corchar",
    "corchad": "corchar",
    "corchada": "corchar",
    "corchadas": "corchar",
    "corchado": "corchar",
    "corchados": "corchar",
    "corchamos": "corchar",
    "corchan": "corchar",
    "corchando": "corchar",
    "corchara": "corchar",
    "corcharais": "corchar",
    "corcharan": "corchar",
    "corcharas": "corchar",
    "corchare": "corchar",
    "corchareis": "corchar",
    "corcharemos": "corchar",
    "corcharen": "corchar",
    "corchares": "corchar",
    "corcharon": "corchar",
    "corchará": "corchar",
    "corcharán": "corchar",
    "corcharás": "corchar",
    "corcharé": "corchar",
    "corcharéis": "corchar",
    "corcharía": "corchar",
    "corcharíais": "corchar",
    "corcharíamos": "corchar",
    "corcharían": "corchar",
    "corcharías": "corchar",
    "corchas": "corchar",
    "corchase": "corchar",
    "corchaseis": "corchar",
    "corchasen": "corchar",
    "corchases": "corchar",
    "corchaste": "corchar",
    "corchasteis": "corchar",
    "corche": "corchar",
    "corcheas": "corchea",
    "corchemos": "corchar",
    "corchen": "corchar",
    "corchera": "corchero",
    "corcheras": "corchero",
    "corcheros": "corchero",
    "corches": "corche",
    "corchetas": "corcheta",
    "corchetes": "corchete",
    "corcho": "corchar",
    "corchos": "corcho",
    "corchosa": "corchoso",
    "corchosas": "corchoso",
    "corchosos": "corchoso",
    "corchotaponera": "corchotaponero",
    "corchotaponeras": "corchotaponero",
    "corchotaponeros": "corchotaponero",
    "corchábamos": "corchar",
    "corcháis": "corchar",
    "corcháramos": "corchar",
    "corcháremos": "corchar",
    "corchásemos": "corchar",
    "corché": "corchar",
    "corchéis": "corchar",
    "corchó": "corchar",
    "corcinos": "corcino",
    "corcores": "corcor",
    "corcova": "corcovar",
    "corcovaba": "corcovar",
    "corcovabais": "corcovar",
    "corcovaban": "corcovar",
    "corcovabas": "corcovar",
    "corcovad": "corcovar",
    "corcovada": "corcovar",
    "corcovadas": "corcovar",
    "corcovado": "corcovar",
    "corcovados": "corcovar",
    "corcovamos": "corcovar",
    "corcovan": "corcovar",
    "corcovando": "corcovar",
    "corcovara": "corcovar",
    "corcovarais": "corcovar",
    "corcovaran": "corcovar",
    "corcovaras": "corcovar",
    "corcovare": "corcovar",
    "corcovareis": "corcovar",
    "corcovaremos": "corcovar",
    "corcovaren": "corcovar",
    "corcovares": "corcovar",
    "corcovaron": "corcovar",
    "corcovará": "corcovar",
    "corcovarán": "corcovar",
    "corcovarás": "corcovar",
    "corcovaré": "corcovar",
    "corcovaréis": "corcovar",
    "corcovaría": "corcovar",
    "corcovaríais": "corcovar",
    "corcovaríamos": "corcovar",
    "corcovarían": "corcovar",
    "corcovarías": "corcovar",
    "corcovas": "corcovar",
    "corcovase": "corcovar",
    "corcovaseis": "corcovar",
    "corcovasen": "corcovar",
    "corcovases": "corcovar",
    "corcovaste": "corcovar",
    "corcovasteis": "corcovar",
    "corcove": "corcovar",
    "corcovea": "corcovear",
    "corcoveaba": "corcovear",
    "corcoveabais": "corcovear",
    "corcoveaban": "corcovear",
    "corcoveabas": "corcovear",
    "corcovead": "corcovear",
    "corcoveada": "corcovear",
    "corcoveadas": "corcovear",
    "corcoveado": "corcovear",
    "corcoveados": "corcovear",
    "corcoveamos": "corcovear",
    "corcovean": "corcovear",
    "corcoveando": "corcovear",
    "corcoveara": "corcovear",
    "corcovearais": "corcovear",
    "corcovearan": "corcovear",
    "corcovearas": "corcovear",
    "corcoveare": "corcovear",
    "corcoveareis": "corcovear",
    "corcovearemos": "corcovear",
    "corcovearen": "corcovear",
    "corcoveares": "corcovear",
    "corcovearon": "corcovear",
    "corcoveará": "corcovear",
    "corcovearán": "corcovear",
    "corcovearás": "corcovear",
    "corcovearé": "corcovear",
    "corcovearéis": "corcovear",
    "corcovearía": "corcovear",
    "corcovearíais": "corcovear",
    "corcovearíamos": "corcovear",
    "corcovearían": "corcovear",
    "corcovearías": "corcovear",
    "corcoveas": "corcovear",
    "corcovease": "corcovear",
    "corcoveaseis": "corcovear",
    "corcoveasen": "corcovear",
    "corcoveases": "corcovear",
    "corcoveaste": "corcovear",
    "corcoveasteis": "corcovear",
    "corcovee": "corcovear",
    "corcoveemos": "corcovear",
    "corcoveen": "corcovear",
    "corcovees": "corcovear",
    "corcovemos": "corcovar",
    "corcoven": "corcovar",
    "corcoveo": "corcovear",
    "corcoves": "corcovar",
    "corcovetas": "corcoveta",
    "corcoveábamos": "corcovear",
    "corcoveáis": "corcovear",
    "corcoveáramos": "corcovear",
    "corcoveáremos": "corcovear",
    "corcoveásemos": "corcovear",
    "corcoveé": "corcovear",
    "corcoveéis": "corcovear",
    "corcoveó": "corcovear",
    "corcovo": "corcovar",
    "corcovos": "corcovo",
    "corcovábamos": "corcovar",
    "corcováis": "corcovar",
    "corcováramos": "corcovar",
    "corcováremos": "corcovar",
    "corcovásemos": "corcovar",
    "corcové": "corcovar",
    "corcovéis": "corcovar",
    "corcovó": "corcovar",
    "corcusa": "corcusir",
    "corcusamos": "corcusir",
    "corcusan": "corcusir",
    "corcusas": "corcusir",
    "corcuse": "corcusir",
    "corcusen": "corcusir",
    "corcuses": "corcusir",
    "corcusid": "corcusir",
    "corcusida": "corcusir",
    "corcusidas": "corcusir",
    "corcusido": "corcusir",
    "corcusidos": "corcusir",
    "corcusiendo": "corcusir",
    "corcusiera": "corcusir",
    "corcusierais": "corcusir",
    "corcusieran": "corcusir",
    "corcusieras": "corcusir",
    "corcusiere": "corcusir",
    "corcusiereis": "corcusir",
    "corcusieren": "corcusir",
    "corcusieres": "corcusir",
    "corcusieron": "corcusir",
    "corcusiese": "corcusir",
    "corcusieseis": "corcusir",
    "corcusiesen": "corcusir",
    "corcusieses": "corcusir",
    "corcusimos": "corcusir",
    "corcusiremos": "corcusir",
    "corcusirá": "corcusir",
    "corcusirán": "corcusir",
    "corcusirás": "corcusir",
    "corcusiré": "corcusir",
    "corcusiréis": "corcusir",
    "corcusiría": "corcusir",
    "corcusiríais": "corcusir",
    "corcusiríamos": "corcusir",
    "corcusirían": "corcusir",
    "corcusirías": "corcusir",
    "corcusiste": "corcusir",
    "corcusisteis": "corcusir",
    "corcusiéramos": "corcusir",
    "corcusiéremos": "corcusir",
    "corcusiésemos": "corcusir",
    "corcusió": "corcusir",
    "corcuso": "corcusir",
    "corcusáis": "corcusir",
    "corcusí": "corcusir",
    "corcusía": "corcusir",
    "corcusíais": "corcusir",
    "corcusíamos": "corcusir",
    "corcusían": "corcusir",
    "corcusías": "corcusir",
    "corcusís": "corcusir",
    "cordada": "cordado",
    "cordadas": "cordado",
    "cordados": "cordado",
    "cordaitales": "cordaital",
    "cordajes": "cordaje",
    "cordales": "cordal",
    "cordata": "cordato",
    "cordatas": "cordato",
    "cordatos": "cordato",
    "cordectomías": "cordectomía",
    "cordelada": "cordelado",
    "cordeladas": "cordelado",
    "cordelados": "cordelado",
    "cordelazos": "cordelazo",
    "cordelejos": "cordelejo",
    "cordelera": "cordelero",
    "cordeleras": "cordelero",
    "cordeleros": "cordelero",
    "cordelerías": "cordelería",
    "cordeles": "cordel",
    "cordera": "cordero",
    "corderas": "cordero",
    "corderilla": "corderillo",
    "corderillas": "corderillo",
    "corderillos": "corderillo",
    "corderina": "corderino",
    "corderinas": "corderino",
    "corderinos": "corderino",
    "corderos": "cordero",
    "corderías": "cordería",
    "cordiales": "cordial",
    "cordialidades": "cordialidad",
    "cordieritas": "cordierita",
    "cordiformes": "cordiforme",
    "cordilas": "cordila",
    "cordillas": "cordilla",
    "cordillerana": "cordillerano",
    "cordilleranas": "cordillerano",
    "cordilleranos": "cordillerano",
    "cordilleras": "cordillera",
    "cordilos": "cordilo",
    "cordinos": "cordino",
    "corditas": "cordita",
    "cordobanera": "cordobanero",
    "cordobaneras": "cordobanero",
    "cordobaneros": "cordobanero",
    "cordobanes": "cordobán",
    "cordobesa": "cordobés",
    "cordobesas": "cordobés",
    "cordobeses": "cordobés",
    "cordonaduras": "cordonadura",
    "cordonales": "cordonal",
    "cordonazos": "cordonazo",
    "cordoncillos": "cordoncillo",
    "cordonera": "cordonero",
    "cordoneras": "cordonero",
    "cordoneros": "cordonero",
    "cordonerías": "cordonería",
    "cordones": "cordón",
    "cordopexias": "cordopexia",
    "cordotomías": "cordotomía",
    "cordubenses": "cordubense",
    "corduras": "cordura",
    "corea": "corear",
    "coreaba": "corear",
    "coreabais": "corear",
    "coreaban": "corear",
    "coreabas": "corear",
    "coreables": "coreable",
    "coread": "corear",
    "coreada": "corear",
    "coreadas": "corear",
    "coreado": "corear",
    "coreados": "corear",
    "coreamos": "corear",
    "corean": "corear",
    "coreana": "coreano",
    "coreanas": "coreano",
    "coreando": "corear",
    "coreanos": "coreano",
    "coreara": "corear",
    "corearais": "corear",
    "corearan": "corear",
    "corearas": "corear",
    "coreare": "corear",
    "coreareis": "corear",
    "corearemos": "corear",
    "corearen": "corear",
    "coreares": "corear",
    "corearon": "corear",
    "coreará": "corear",
    "corearán": "corear",
    "corearás": "corear",
    "corearé": "corear",
    "corearéis": "corear",
    "corearía": "corear",
    "corearíais": "corear",
    "corearíamos": "corear",
    "corearían": "corear",
    "corearías": "corear",
    "coreas": "corear",
    "corease": "corear",
    "coreaseis": "corear",
    "coreasen": "corear",
    "coreases": "corear",
    "coreaste": "corear",
    "coreasteis": "corear",
    "corectasias": "corectasia",
    "corectomías": "corectomía",
    "corectopias": "corectopia",
    "coree": "corear",
    "coreemos": "corear",
    "coreen": "corear",
    "corees": "corear",
    "coreligionaria": "coreligionario",
    "coreligionarias": "coreligionario",
    "coreligionarios": "coreligionario",
    "coreo": "corear",
    "coreografías": "coreografía",
    "coreográfica": "coreográfico",
    "coreográficas": "coreográfico",
    "coreográficos": "coreográfico",
    "coreoplastias": "coreoplastia",
    "coreos": "coreo",
    "corestenomas": "corestenoma",
    "corezuelos": "corezuelo",
    "coreábamos": "corear",
    "coreáis": "corear",
    "coreáramos": "corear",
    "coreáremos": "corear",
    "coreásemos": "corear",
    "coreé": "corear",
    "coreéis": "corear",
    "coreó": "corear",
    "coreógrafa": "coreógrafo",
    "coreógrafas": "coreógrafo",
    "coreógrafos": "coreógrafo",
    "coriambos": "coriambo",
    "corifeos": "corifeo",
    "corimbiformes": "corimbiforme",
    "corimbos": "corimbo",
    "corindones": "corindón",
    "corintia": "corintio",
    "corintias": "corintio",
    "corintios": "corintio",
    "coriocapilares": "coriocapilar",
    "coriorretinopatías": "coriorretinopatía",
    "coristas": "corista",
    "corita": "corito",
    "coritas": "corito",
    "coritos": "corito",
    "corizas": "coriza",
    "coriácea": "coriáceo",
    "coriáceas": "coriáceo",
    "coriáceos": "coriáceo",
    "coriámbica": "coriámbico",
    "coriámbicas": "coriámbico",
    "coriámbicos": "coriámbico",
    "corla": "corlar",
    "corlaba": "corlar",
    "corlabais": "corlar",
    "corlaban": "corlar",
    "corlabas": "corlar",
    "corlad": "corlar",
    "corlada": "corlar",
    "corladas": "corlar",
    "corlado": "corlar",
    "corlados": "corlar",
    "corladuras": "corladura",
    "corlamos": "corlar",
    "corlan": "corlar",
    "corlando": "corlar",
    "corlara": "corlar",
    "corlarais": "corlar",
    "corlaran": "corlar",
    "corlaras": "corlar",
    "corlare": "corlar",
    "corlareis": "corlar",
    "corlaremos": "corlar",
    "corlaren": "corlar",
    "corlares": "corlar",
    "corlaron": "corlar",
    "corlará": "corlar",
    "corlarán": "corlar",
    "corlarás": "corlar",
    "corlaré": "corlar",
    "corlaréis": "corlar",
    "corlaría": "corlar",
    "corlaríais": "corlar",
    "corlaríamos": "corlar",
    "corlarían": "corlar",
    "corlarías": "corlar",
    "corlas": "corlar",
    "corlase": "corlar",
    "corlaseis": "corlar",
    "corlasen": "corlar",
    "corlases": "corlar",
    "corlaste": "corlar",
    "corlasteis": "corlar",
    "corle": "corlar",
    "corlea": "corlear",
    "corleaba": "corlear",
    "corleabais": "corlear",
    "corleaban": "corlear",
    "corleabas": "corlear",
    "corlead": "corlear",
    "corleada": "corlear",
    "corleadas": "corlear",
    "corleado": "corlear",
    "corleados": "corlear",
    "corleamos": "corlear",
    "corlean": "corlear",
    "corleando": "corlear",
    "corleara": "corlear",
    "corlearais": "corlear",
    "corlearan": "corlear",
    "corlearas": "corlear",
    "corleare": "corlear",
    "corleareis": "corlear",
    "corlearemos": "corlear",
    "corlearen": "corlear",
    "corleares": "corlear",
    "corlearon": "corlear",
    "corleará": "corlear",
    "corlearán": "corlear",
    "corlearás": "corlear",
    "corlearé": "corlear",
    "corlearéis": "corlear",
    "corlearía": "corlear",
    "corlearíais": "corlear",
    "corlearíamos": "corlear",
    "corlearían": "corlear",
    "corlearías": "corlear",
    "corleas": "corlear",
    "corlease": "corlear",
    "corleaseis": "corlear",
    "corleasen": "corlear",
    "corleases": "corlear",
    "corleaste": "corlear",
    "corleasteis": "corlear",
    "corlee": "corlear",
    "corleemos": "corlear",
    "corleen": "corlear",
    "corlees": "corlear",
    "corlemos": "corlar",
    "corlen": "corlar",
    "corleo": "corlear",
    "corles": "corlar",
    "corleábamos": "corlear",
    "corleáis": "corlear",
    "corleáramos": "corlear",
    "corleáremos": "corlear",
    "corleásemos": "corlear",
    "corleé": "corlear",
    "corleéis": "corlear",
    "corleó": "corlear",
    "corlo": "corlar",
    "corlábamos": "corlar",
    "corláis": "corlar",
    "corláramos": "corlar",
    "corláremos": "corlar",
    "corlásemos": "corlar",
    "corlé": "corlar",
    "corléis": "corlar",
    "corló": "corlar",
    "cormas": "corma",
    "cormieras": "cormiera",
    "cormoranes": "cormorán",
    "cormos": "cormo",
    "cormófita": "cormófito",
    "cormófitas": "cormófito",
    "cormófitos": "cormófito",
    "cornacas": "cornaca",
    "cornacs": "cornac",
    "cornadas": "cornada",
    "cornaduras": "cornadura",
    "cornales": "cornal",
    "cornalinas": "cornalina",
    "cornalones": "cornalón",
    "cornamentas": "cornamenta",
    "cornamusas": "cornamusa",
    "cornatillos": "cornatillo",
    "cornea": "cornear",
    "corneaba": "cornear",
    "corneabais": "cornear",
    "corneaban": "cornear",
    "corneabas": "cornear",
    "cornead": "cornear",
    "corneada": "cornear",
    "corneadas": "cornear",
    "corneado": "cornear",
    "corneadora": "corneador",
    "corneadoras": "corneador",
    "corneadores": "corneador",
    "corneados": "cornear",
    "corneales": "corneal",
    "corneamos": "cornear",
    "cornean": "cornear",
    "corneanas": "corneana",
    "corneando": "cornear",
    "corneara": "cornear",
    "cornearais": "cornear",
    "cornearan": "cornear",
    "cornearas": "cornear",
    "corneare": "cornear",
    "corneareis": "cornear",
    "cornearemos": "cornear",
    "cornearen": "cornear",
    "corneares": "cornear",
    "cornearon": "cornear",
    "corneará": "cornear",
    "cornearán": "cornear",
    "cornearás": "cornear",
    "cornearé": "cornear",
    "cornearéis": "cornear",
    "cornearía": "cornear",
    "cornearíais": "cornear",
    "cornearíamos": "cornear",
    "cornearían": "cornear",
    "cornearías": "cornear",
    "corneas": "cornear",
    "cornease": "cornear",
    "corneaseis": "cornear",
    "corneasen": "cornear",
    "corneases": "cornear",
    "corneaste": "cornear",
    "corneasteis": "cornear",
    "cornee": "cornear",
    "corneemos": "cornear",
    "corneen": "cornear",
    "cornees": "cornear",
    "cornejales": "cornejal",
    "cornejas": "corneja",
    "cornejos": "cornejo",
    "corneo": "cornear",
    "corneoesclerótica": "corneoesclerótico",
    "corneoescleróticas": "corneoesclerótico",
    "corneoescleróticos": "corneoesclerótico",
    "cornetas": "corneta",
    "cornetes": "cornete",
    "cornetines": "cornetín",
    "cornetistas": "cornetista",
    "cornezuelos": "cornezuelo",
    "corneábamos": "cornear",
    "corneáis": "cornear",
    "corneáramos": "cornear",
    "corneáremos": "cornear",
    "corneásemos": "cornear",
    "corneé": "cornear",
    "corneéis": "cornear",
    "corneó": "cornear",
    "corniales": "cornial",
    "cornibrocha": "cornibrocho",
    "cornibrochas": "cornibrocho",
    "cornibrochos": "cornibrocho",
    "cornicabras": "cornicabra",
    "corniculada": "corniculado",
    "corniculadas": "corniculado",
    "corniculados": "corniculado",
    "corniformes": "corniforme",
    "cornigacha": "cornigacho",
    "cornigachas": "cornigacho",
    "cornigachos": "cornigacho",
    "cornijales": "cornijal",
    "cornijas": "cornija",
    "cornijones": "cornijón",
    "cornisamentos": "cornisamento",
    "cornisamientos": "cornisamiento",
    "cornisas": "cornisa",
    "corniveleta": "corniveleto",
    "corniveletas": "corniveleto",
    "corniveletos": "corniveleto",
    "cornizos": "cornizo",
    "cornos": "corno",
    "cornucopias": "cornucopia",
    "cornuda": "cornudo",
    "cornudas": "cornudo",
    "cornudos": "cornudo",
    "cornáceas": "cornácea",
    "cornúpetas": "cornúpeta",
    "cornúpetos": "cornúpeto",
    "corocromática": "corocromático",
    "corocromáticas": "corocromático",
    "corocromáticos": "corocromático",
    "corografías": "corografía",
    "corográfica": "corográfico",
    "corográficas": "corográfico",
    "corográficos": "corográfico",
    "coroidea": "coroideo",
    "coroideas": "coroideo",
    "coroideos": "coroideo",
    "coroideremias": "coroideremia",
    "coroidocapilares": "coroidocapilar",
    "coroidopatías": "coroidopatía",
    "corojos": "corojo",
    "corolarios": "corolario",
    "corolas": "corola",
    "corologías": "corología",
    "corona": "coronar",
    "coronaba": "coronar",
    "coronabais": "coronar",
    "coronaban": "coronar",
    "coronabas": "coronar",
    "coronaciones": "coronación",
    "coronad": "coronar",
    "coronada": "coronar",
    "coronadas": "coronar",
    "coronado": "coronar",
    "coronadora": "coronador",
    "coronadoras": "coronador",
    "coronadores": "coronador",
    "coronados": "coronar",
    "coronales": "coronal",
    "coronamientos": "coronamiento",
    "coronamos": "coronar",
    "coronan": "coronar",
    "coronando": "coronar",
    "coronara": "coronar",
    "coronarais": "coronar",
    "coronaran": "coronar",
    "coronaras": "coronar",
    "coronare": "coronar",
    "coronareis": "coronar",
    "coronaremos": "coronar",
    "coronaren": "coronar",
    "coronares": "coronar",
    "coronaria": "coronario",
    "coronarias": "coronario",
    "coronariopatías": "coronariopatía",
    "coronarios": "coronario",
    "coronaron": "coronar",
    "coronará": "coronar",
    "coronarán": "coronar",
    "coronarás": "coronar",
    "coronaré": "coronar",
    "coronaréis": "coronar",
    "coronaría": "coronar",
    "coronaríais": "coronar",
    "coronaríamos": "coronar",
    "coronarían": "coronar",
    "coronarías": "coronar",
    "coronas": "coronar",
    "coronase": "coronar",
    "coronaseis": "coronar",
    "coronasen": "coronar",
    "coronases": "coronar",
    "coronaste": "coronar",
    "coronasteis": "coronar",
    "corondeles": "corondel",
    "corone": "coronar",
    "coronelas": "coronela",
    "coroneles": "coronel",
    "coronelías": "coronelía",
    "coronemos": "coronar",
    "coronen": "coronar",
    "corones": "coronar",
    "coronillas": "coronilla",
    "corono": "coronar",
    "coronoidea": "coronoideo",
    "coronoideas": "coronoideo",
    "coronoidectomías": "coronoidectomía",
    "coronoideos": "coronoideo",
    "coronoplastias": "coronoplastia",
    "coronábamos": "coronar",
    "coronáis": "coronar",
    "coronáramos": "coronar",
    "coronáremos": "coronar",
    "coronásemos": "coronar",
    "coroné": "coronar",
    "coronéis": "coronar",
    "coronó": "coronar",
    "coronógrafos": "coronógrafo",
    "coronómetros": "coronómetro",
    "coroplastias": "coroplastia",
    "coropletas": "coropleta",
    "coroplética": "coroplético",
    "coropléticas": "coroplético",
    "coropléticos": "coroplético",
    "coros": "coro",
    "corozas": "coroza",
    "corpas": "corpa",
    "corpiños": "corpiño",
    "corporaciones": "corporación",
    "corporales": "corporal",
    "corporalidades": "corporalidad",
    "corporativa": "corporativo",
    "corporativas": "corporativo",
    "corporativismos": "corporativismo",
    "corporativistas": "corporativista",
    "corporativos": "corporativo",
    "corporifica": "corporificar",
    "corporificaba": "corporificar",
    "corporificabais": "corporificar",
    "corporificaban": "corporificar",
    "corporificabas": "corporificar",
    "corporificaciones": "corporificación",
    "corporificad": "corporificar",
    "corporificada": "corporificar",
    "corporificadas": "corporificar",
    "corporificado": "corporificar",
    "corporificados": "corporificar",
    "corporificamos": "corporificar",
    "corporifican": "corporificar",
    "corporificando": "corporificar",
    "corporificara": "corporificar",
    "corporificarais": "corporificar",
    "corporificaran": "corporificar",
    "corporificaras": "corporificar",
    "corporificare": "corporificar",
    "corporificareis": "corporificar",
    "corporificaremos": "corporificar",
    "corporificaren": "corporificar",
    "corporificares": "corporificar",
    "corporificaron": "corporificar",
    "corporificará": "corporificar",
    "corporificarán": "corporificar",
    "corporificarás": "corporificar",
    "corporificaré": "corporificar",
    "corporificaréis": "corporificar",
    "corporificaría": "corporificar",
    "corporificaríais": "corporificar",
    "corporificaríamos": "corporificar",
    "corporificarían": "corporificar",
    "corporificarías": "corporificar",
    "corporificas": "corporificar",
    "corporificase": "corporificar",
    "corporificaseis": "corporificar",
    "corporificasen": "corporificar",
    "corporificases": "corporificar",
    "corporificaste": "corporificar",
    "corporificasteis": "corporificar",
    "corporifico": "corporificar",
    "corporificábamos": "corporificar",
    "corporificáis": "corporificar",
    "corporificáramos": "corporificar",
    "corporificáremos": "corporificar",
    "corporificásemos": "corporificar",
    "corporificó": "corporificar",
    "corporifique": "corporificar",
    "corporifiquemos": "corporificar",
    "corporifiquen": "corporificar",
    "corporifiques": "corporificar",
    "corporifiqué": "corporificar",
    "corporifiquéis": "corporificar",
    "corpulencias": "corpulencia",
    "corpulenta": "corpulento",
    "corpulentas": "corpulento",
    "corpulentos": "corpulento",
    "corpusculares": "corpuscular",
    "corpórea": "corpóreo",
    "corpóreas": "corpóreo",
    "corpóreos": "corpóreo",
    "corpúsculos": "corpúsculo",
    "corra": "correr",
    "corralera": "corralero",
    "corraleras": "corralero",
    "corraleros": "corralero",
    "corrales": "corral",
    "corralizas": "corraliza",
    "corramos": "correr",
    "corran": "correr",
    "corras": "correr",
    "corrasiones": "corrasión",
    "corre": "correr",
    "correa": "correar",
    "correaba": "correar",
    "correabais": "correar",
    "correaban": "correar",
    "correabas": "correar",
    "corread": "correar",
    "correada": "correar",
    "correadas": "correar",
    "correado": "correar",
    "correados": "correar",
    "correajes": "correaje",
    "correales": "correal",
    "correamos": "correar",
    "correan": "correar",
    "correando": "correar",
    "correara": "correar",
    "correarais": "correar",
    "correaran": "correar",
    "correaras": "correar",
    "correare": "correar",
    "correareis": "correar",
    "correaremos": "correar",
    "correaren": "correar",
    "correares": "correar",
    "correaron": "correar",
    "correará": "correar",
    "correarán": "correar",
    "correarás": "correar",
    "correaré": "correar",
    "correaréis": "correar",
    "correaría": "correar",
    "correaríais": "correar",
    "correaríamos": "correar",
    "correarían": "correar",
    "correarías": "correar",
    "correas": "correar",
    "correase": "correar",
    "correaseis": "correar",
    "correasen": "correar",
    "correases": "correar",
    "correaste": "correar",
    "correasteis": "correar",
    "correazos": "correazo",
    "correccionales": "correccional",
    "correccionalismos": "correccionalismo",
    "correccionalistas": "correccionalista",
    "correcciones": "corrección",
    "correcta": "correcto",
    "correctas": "correcto",
    "correctiva": "correctivo",
    "correctivas": "correctivo",
    "correctivos": "correctivo",
    "correctora": "corrector",
    "correctoras": "corrector",
    "correctores": "corrector",
    "correctos": "correcto",
    "corred": "correr",
    "corredentora": "corredentor",
    "corredentoras": "corredentor",
    "corredentores": "corredentor",
    "corredera": "corredero",
    "correderas": "corredero",
    "correderos": "corredero",
    "corrediza": "corredizo",
    "corredizas": "corredizo",
    "corredizos": "corredizo",
    "corredora": "corredor",
    "corredoras": "corredor",
    "corredores": "corredor",
    "correduras": "corredura",
    "corredurías": "correduría",
    "corree": "correar",
    "correemos": "correar",
    "correen": "correar",
    "correera": "correero",
    "correeras": "correero",
    "correeros": "correero",
    "correerías": "correería",
    "correes": "correar",
    "correferencias": "correferencia",
    "correferentes": "correferente",
    "corregencias": "corregencia",
    "corregentes": "corregente",
    "corregibilidades": "corregibilidad",
    "corregibles": "corregible",
    "corregid": "corregir",
    "corregida": "corregir",
    "corregidas": "corregir",
    "corregido": "corregir",
    "corregidora": "corregidor",
    "corregidoras": "corregidor",
    "corregidores": "corregidor",
    "corregidos": "corregir",
    "corregimientos": "corregimiento",
    "corregimos": "corregir",
    "corregiremos": "corregir",
    "corregirá": "corregir",
    "corregirán": "corregir",
    "corregirás": "corregir",
    "corregiré": "corregir",
    "corregiréis": "corregir",
    "corregiría": "corregir",
    "corregiríais": "corregir",
    "corregiríamos": "corregir",
    "corregirían": "corregir",
    "corregirías": "corregir",
    "corregiste": "corregir",
    "corregisteis": "corregir",
    "corregí": "corregir",
    "corregía": "corregir",
    "corregíais": "corregir",
    "corregíamos": "corregir",
    "corregían": "corregir",
    "corregías": "corregir",
    "corregís": "corregir",
    "correhuelas": "correhuela",
    "correinados": "correinado",
    "correjeles": "correjel",
    "correlaciona": "correlacionar",
    "correlacionaba": "correlacionar",
    "correlacionabais": "correlacionar",
    "correlacionaban": "correlacionar",
    "correlacionabas": "correlacionar",
    "correlacionad": "correlacionar",
    "correlacionada": "correlacionar",
    "correlacionadas": "correlacionar",
    "correlacionado": "correlacionar",
    "correlacionados": "correlacionar",
    "correlacionamos": "correlacionar",
    "correlacionan": "correlacionar",
    "correlacionando": "correlacionar",
    "correlacionara": "correlacionar",
    "correlacionarais": "correlacionar",
    "correlacionaran": "correlacionar",
    "correlacionaras": "correlacionar",
    "correlacionare": "correlacionar",
    "correlacionareis": "correlacionar",
    "correlacionaremos": "correlacionar",
    "correlacionaren": "correlacionar",
    "correlacionares": "correlacionar",
    "correlacionaron": "correlacionar",
    "correlacionará": "correlacionar",
    "correlacionarán": "correlacionar",
    "correlacionarás": "correlacionar",
    "correlacionaré": "correlacionar",
    "correlacionaréis": "correlacionar",
    "correlacionaría": "correlacionar",
    "correlacionaríais": "correlacionar",
    "correlacionaríamos": "correlacionar",
    "correlacionarían": "correlacionar",
    "correlacionarías": "correlacionar",
    "correlacionas": "correlacionar",
    "correlacionase": "correlacionar",
    "correlacionaseis": "correlacionar",
    "correlacionasen": "correlacionar",
    "correlacionases": "correlacionar",
    "correlacionaste": "correlacionar",
    "correlacionasteis": "correlacionar",
    "correlacione": "correlacionar",
    "correlacionemos": "correlacionar",
    "correlacionen": "correlacionar",
    "correlaciones": "correlacionar",
    "correlaciono": "correlacionar",
    "correlacionábamos": "correlacionar",
    "correlacionáis": "correlacionar",
    "correlacionáramos": "correlacionar",
    "correlacionáremos": "correlacionar",
    "correlacionásemos": "correlacionar",
    "correlacioné": "correlacionar",
    "correlacionéis": "correlacionar",
    "correlacionó": "correlacionar",
    "correladores": "correlador",
    "correlativa": "correlativo",
    "correlativas": "correlativo",
    "correlativos": "correlativo",
    "correlatos": "correlato",
    "correligionaria": "correligionario",
    "correligionarias": "correligionario",
    "correligionarios": "correligionario",
    "correlimos": "correlimo",
    "correlogramas": "correlograma",
    "corremos": "correr",
    "corren": "correr",
    "correntadas": "correntada",
    "correntina": "correntino",
    "correntinas": "correntino",
    "correntinos": "correntino",
    "correntona": "correntón",
    "correntonas": "correntón",
    "correntones": "correntón",
    "correntosa": "correntoso",
    "correntosas": "correntoso",
    "correntosos": "correntoso",
    "correntía": "correntío",
    "correntías": "correntío",
    "correntíos": "correntío",
    "correntógrafos": "correntógrafo",
    "correntómetros": "correntómetro",
    "correo": "correar",
    "correos": "correo",
    "correosa": "correoso",
    "correosas": "correoso",
    "correosidades": "correosidad",
    "correosos": "correoso",
    "correremos": "correr",
    "correrá": "correr",
    "correrán": "correr",
    "correrás": "correr",
    "correré": "correr",
    "correréis": "correr",
    "correría": "correr",
    "correríais": "correr",
    "correríamos": "correr",
    "correrían": "correr",
    "correrías": "correría",
    "corres": "correr",
    "corresponda": "corresponder",
    "correspondamos": "corresponder",
    "correspondan": "corresponder",
    "correspondas": "corresponder",
    "corresponde": "corresponder",
    "corresponded": "corresponder",
    "correspondemos": "corresponder",
    "corresponden": "corresponder",
    "correspondencias": "correspondencia",
    "corresponderemos": "corresponder",
    "corresponderá": "corresponder",
    "corresponderán": "corresponder",
    "corresponderás": "corresponder",
    "corresponderé": "corresponder",
    "corresponderéis": "corresponder",
    "correspondería": "corresponder",
    "corresponderíais": "corresponder",
    "corresponderíamos": "corresponder",
    "corresponderían": "corresponder",
    "corresponderías": "corresponder",
    "correspondes": "corresponder",
    "correspondida": "corresponder",
    "correspondidas": "corresponder",
    "correspondido": "corresponder",
    "correspondidos": "corresponder",
    "correspondiendo": "corresponder",
    "correspondientes": "correspondiente",
    "correspondiera": "corresponder",
    "correspondierais": "corresponder",
    "correspondieran": "corresponder",
    "correspondieras": "corresponder",
    "correspondiere": "corresponder",
    "correspondiereis": "corresponder",
    "correspondieren": "corresponder",
    "correspondieres": "corresponder",
    "correspondieron": "corresponder",
    "correspondiese": "corresponder",
    "correspondieseis": "corresponder",
    "correspondiesen": "corresponder",
    "correspondieses": "corresponder",
    "correspondimos": "corresponder",
    "correspondiste": "corresponder",
    "correspondisteis": "corresponder",
    "correspondiéramos": "corresponder",
    "correspondiéremos": "corresponder",
    "correspondiésemos": "corresponder",
    "correspondió": "corresponder",
    "correspondo": "corresponder",
    "correspondáis": "corresponder",
    "correspondéis": "corresponder",
    "correspondí": "corresponder",
    "correspondía": "corresponder",
    "correspondíais": "corresponder",
    "correspondíamos": "corresponder",
    "correspondían": "corresponder",
    "correspondías": "corresponder",
    "corresponsabilidades": "corresponsabilidad",
    "corresponsables": "corresponsable",
    "corresponsales": "corresponsal",
    "corresponsalías": "corresponsalía",
    "corretajes": "corretaje",
    "corretea": "corretear",
    "correteaba": "corretear",
    "correteabais": "corretear",
    "correteaban": "corretear",
    "correteabas": "corretear",
    "corretead": "corretear",
    "correteada": "corretear",
    "correteadas": "corretear",
    "correteado": "corretear",
    "correteados": "corretear",
    "correteamos": "corretear",
    "corretean": "corretear",
    "correteando": "corretear",
    "correteara": "corretear",
    "corretearais": "corretear",
    "corretearan": "corretear",
    "corretearas": "corretear",
    "correteare": "corretear",
    "correteareis": "corretear",
    "corretearemos": "corretear",
    "corretearen": "corretear",
    "correteares": "corretear",
    "corretearon": "corretear",
    "correteará": "corretear",
    "corretearán": "corretear",
    "corretearás": "corretear",
    "corretearé": "corretear",
    "corretearéis": "corretear",
    "corretearía": "corretear",
    "corretearíais": "corretear",
    "corretearíamos": "corretear",
    "corretearían": "corretear",
    "corretearías": "corretear",
    "correteas": "corretear",
    "corretease": "corretear",
    "correteaseis": "corretear",
    "correteasen": "corretear",
    "correteases": "corretear",
    "correteaste": "corretear",
    "correteasteis": "corretear",
    "corretee": "corretear",
    "correteemos": "corretear",
    "correteen": "corretear",
    "corretees": "corretear",
    "correteo": "corretear",
    "correteos": "correteo",
    "corretera": "corretero",
    "correteras": "corretero",
    "correteros": "corretero",
    "correteábamos": "corretear",
    "correteáis": "corretear",
    "correteáramos": "corretear",
    "correteáremos": "corretear",
    "correteásemos": "corretear",
    "correteé": "corretear",
    "correteéis": "corretear",
    "correteó": "corretear",
    "correveidiles": "correveidile",
    "correábamos": "correar",
    "correáis": "correar",
    "correáramos": "correar",
    "correáremos": "correar",
    "correásemos": "correar",
    "correé": "correar",
    "correéis": "correar",
    "correístas": "correísta",
    "correó": "correar",
    "corrida": "correr",
    "corridas": "corrida",
    "corrido": "correr",
    "corridos": "corrido",
    "corriendo": "correr",
    "corrientes": "corriente",
    "corrientómetros": "corrientómetro",
    "corriera": "correr",
    "corrierais": "correr",
    "corrieran": "correr",
    "corrieras": "correr",
    "corriere": "correr",
    "corriereis": "correr",
    "corrieren": "correr",
    "corrieres": "correr",
    "corrieron": "correr",
    "corriese": "correr",
    "corrieseis": "correr",
    "corriesen": "correr",
    "corrieses": "correr",
    "corrige": "corregir",
    "corrigen": "corregir",
    "corrigenda": "corrigendo",
    "corrigendas": "corrigendo",
    "corrigendos": "corrigendo",
    "corriges": "corregir",
    "corrigiendo": "corregir",
    "corrigiera": "corregir",
    "corrigierais": "corregir",
    "corrigieran": "corregir",
    "corrigieras": "corregir",
    "corrigiere": "corregir",
    "corrigiereis": "corregir",
    "corrigieren": "corregir",
    "corrigieres": "corregir",
    "corrigieron": "corregir",
    "corrigiese": "corregir",
    "corrigieseis": "corregir",
    "corrigiesen": "corregir",
    "corrigieses": "corregir",
    "corrigiéramos": "corregir",
    "corrigiéremos": "corregir",
    "corrigiésemos": "corregir",
    "corrigió": "corregir",
    "corrija": "corregir",
    "corrijamos": "corregir",
    "corrijan": "corregir",
    "corrijas": "corregir",
    "corrijo": "corregir",
    "corrijáis": "corregir",
    "corrillos": "corrillo",
    "corrimientos": "corrimiento",
    "corrimos": "correr",
    "corriste": "correr",
    "corristeis": "correr",
    "corrivaciones": "corrivación",
    "corriéramos": "correr",
    "corriéremos": "correr",
    "corriésemos": "correr",
    "corrió": "correr",
    "corro": "correr",
    "corroa": "corroer",
    "corroamos": "corroer",
    "corroan": "corroer",
    "corroas": "corroer",
    "corrobora": "corroborar",
    "corroboraba": "corroborar",
    "corroborabais": "corroborar",
    "corroboraban": "corroborar",
    "corroborabas": "corroborar",
    "corroboraciones": "corroboración",
    "corroborad": "corroborar",
    "corroborada": "corroborar",
    "corroboradas": "corroborar",
    "corroborado": "corroborar",
    "corroborados": "corroborar",
    "corroboramos": "corroborar",
    "corroboran": "corroborar",
    "corroborando": "corroborar",
    "corroborantes": "corroborante",
    "corroborara": "corroborar",
    "corroborarais": "corroborar",
    "corroboraran": "corroborar",
    "corroboraras": "corroborar",
    "corroborare": "corroborar",
    "corroborareis": "corroborar",
    "corroboraremos": "corroborar",
    "corroboraren": "corroborar",
    "corroborares": "corroborar",
    "corroboraron": "corroborar",
    "corroborará": "corroborar",
    "corroborarán": "corroborar",
    "corroborarás": "corroborar",
    "corroboraré": "corroborar",
    "corroboraréis": "corroborar",
    "corroboraría": "corroborar",
    "corroboraríais": "corroborar",
    "corroboraríamos": "corroborar",
    "corroborarían": "corroborar",
    "corroborarías": "corroborar",
    "corroboras": "corroborar",
    "corroborase": "corroborar",
    "corroboraseis": "corroborar",
    "corroborasen": "corroborar",
    "corroborases": "corroborar",
    "corroboraste": "corroborar",
    "corroborasteis": "corroborar",
    "corroborativa": "corroborativo",
    "corroborativas": "corroborativo",
    "corroborativos": "corroborativo",
    "corrobore": "corroborar",
    "corroboremos": "corroborar",
    "corroboren": "corroborar",
    "corrobores": "corroborar",
    "corroboro": "corroborar",
    "corroborábamos": "corroborar",
    "corroboráis": "corroborar",
    "corroboráramos": "corroborar",
    "corroboráremos": "corroborar",
    "corroborásemos": "corroborar",
    "corroboré": "corroborar",
    "corroboréis": "corroborar",
    "corroboró": "corroborar",
    "corroe": "corroer",
    "corroed": "corroer",
    "corroemos": "corroer",
    "corroen": "corroer",
    "corroeremos": "corroer",
    "corroerá": "corroer",
    "corroerán": "corroer",
    "corroerás": "corroer",
    "corroeré": "corroer",
    "corroeréis": "corroer",
    "corroería": "corroer",
    "corroeríais": "corroer",
    "corroeríamos": "corroer",
    "corroerían": "corroer",
    "corroerías": "corroer",
    "corroes": "corroer",
    "corroiga": "corroer",
    "corroigamos": "corroer",
    "corroigan": "corroer",
    "corroigas": "corroer",
    "corroigo": "corroer",
    "corroigáis": "corroer",
    "corrompa": "corromper",
    "corrompamos": "corromper",
    "corrompan": "corromper",
    "corrompas": "corromper",
    "corrompe": "corromper",
    "corromped": "corromper",
    "corrompemos": "corromper",
    "corrompen": "corromper",
    "corromperemos": "corromper",
    "corromperá": "corromper",
    "corromperán": "corromper",
    "corromperás": "corromper",
    "corromperé": "corromper",
    "corromperéis": "corromper",
    "corrompería": "corromper",
    "corromperíais": "corromper",
    "corromperíamos": "corromper",
    "corromperían": "corromper",
    "corromperías": "corromper",
    "corrompes": "corromper",
    "corrompida": "corromper",
    "corrompidas": "corromper",
    "corrompido": "corromper",
    "corrompidos": "corromper",
    "corrompiendo": "corromper",
    "corrompiera": "corromper",
    "corrompierais": "corromper",
    "corrompieran": "corromper",
    "corrompieras": "corromper",
    "corrompiere": "corromper",
    "corrompiereis": "corromper",
    "corrompieren": "corromper",
    "corrompieres": "corromper",
    "corrompieron": "corromper",
    "corrompiese": "corromper",
    "corrompieseis": "corromper",
    "corrompiesen": "corromper",
    "corrompieses": "corromper",
    "corrompimos": "corromper",
    "corrompiste": "corromper",
    "corrompisteis": "corromper",
    "corrompiéramos": "corromper",
    "corrompiéremos": "corromper",
    "corrompiésemos": "corromper",
    "corrompió": "corromper",
    "corrompo": "corromper",
    "corrompáis": "corromper",
    "corrompéis": "corromper",
    "corrompí": "corromper",
    "corrompía": "corromper",
    "corrompíais": "corromper",
    "corrompíamos": "corromper",
    "corrompían": "corromper",
    "corrompías": "corromper",
    "corroo": "corroer",
    "corros": "corro",
    "corrosibles": "corrosible",
    "corrosiones": "corrosión",
    "corrosiva": "corrosivo",
    "corrosivas": "corrosivo",
    "corrosividades": "corrosividad",
    "corrosivos": "corrosivo",
    "corroya": "corroer",
    "corroyamos": "corroer",
    "corroyan": "corroer",
    "corroyas": "corroer",
    "corroyendo": "corroer",
    "corroyentes": "corroyente",
    "corroyera": "corroer",
    "corroyerais": "corroer",
    "corroyeran": "corroer",
    "corroyeras": "corroer",
    "corroyere": "corroer",
    "corroyereis": "corroer",
    "corroyeren": "corroer",
    "corroyeres": "corroer",
    "corroyeron": "corroer",
    "corroyese": "corroer",
    "corroyeseis": "corroer",
    "corroyesen": "corroer",
    "corroyeses": "corroer",
    "corroyo": "corroer",
    "corroyáis": "corroer",
    "corroyéramos": "corroer",
    "corroyéremos": "corroer",
    "corroyésemos": "corroer",
    "corroyó": "corroer",
    "corroáis": "corroer",
    "corroéis": "corroer",
    "corroí": "corroer",
    "corroía": "corroer",
    "corroíais": "corroer",
    "corroíamos": "corroer",
    "corroían": "corroer",
    "corroías": "corroer",
    "corroída": "corroer",
    "corroídas": "corroer",
    "corroído": "corroer",
    "corroídos": "corroer",
    "corroímos": "corroer",
    "corroíste": "corroer",
    "corroísteis": "corroer",
    "corrrespondientes": "corrrespondiente",
    "corruga": "corrugar",
    "corrugaba": "corrugar",
    "corrugabais": "corrugar",
    "corrugaban": "corrugar",
    "corrugabas": "corrugar",
    "corrugaciones": "corrugación",
    "corrugad": "corrugar",
    "corrugada": "corrugar",
    "corrugadas": "corrugar",
    "corrugado": "corrugar",
    "corrugados": "corrugar",
    "corrugamos": "corrugar",
    "corrugan": "corrugar",
    "corrugando": "corrugar",
    "corrugara": "corrugar",
    "corrugarais": "corrugar",
    "corrugaran": "corrugar",
    "corrugaras": "corrugar",
    "corrugare": "corrugar",
    "corrugareis": "corrugar",
    "corrugaremos": "corrugar",
    "corrugaren": "corrugar",
    "corrugares": "corrugar",
    "corrugaron": "corrugar",
    "corrugará": "corrugar",
    "corrugarán": "corrugar",
    "corrugarás": "corrugar",
    "corrugaré": "corrugar",
    "corrugaréis": "corrugar",
    "corrugaría": "corrugar",
    "corrugaríais": "corrugar",
    "corrugaríamos": "corrugar",
    "corrugarían": "corrugar",
    "corrugarías": "corrugar",
    "corrugas": "corrugar",
    "corrugase": "corrugar",
    "corrugaseis": "corrugar",
    "corrugasen": "corrugar",
    "corrugases": "corrugar",
    "corrugaste": "corrugar",
    "corrugasteis": "corrugar",
    "corrugo": "corrugar",
    "corrugue": "corrugar",
    "corruguemos": "corrugar",
    "corruguen": "corrugar",
    "corrugues": "corrugar",
    "corrugué": "corrugar",
    "corruguéis": "corrugar",
    "corrugábamos": "corrugar",
    "corrugáis": "corrugar",
    "corrugáramos": "corrugar",
    "corrugáremos": "corrugar",
    "corrugásemos": "corrugar",
    "corrugó": "corrugar",
    "corrupciones": "corrupción",
    "corrupta": "corrupto",
    "corruptas": "corrupto",
    "corruptelas": "corruptela",
    "corruptibilidades": "corruptibilidad",
    "corruptiva": "corruptivo",
    "corruptivas": "corruptivo",
    "corruptivos": "corruptivo",
    "corruptora": "corruptor",
    "corruptoras": "corruptor",
    "corruptores": "corruptor",
    "corruptos": "corrupto",
    "corruscos": "corrusco",
    "corráis": "correr",
    "corréis": "correr",
    "corrí": "correr",
    "corría": "correr",
    "corríais": "correr",
    "corríamos": "correr",
    "corrían": "correr",
    "corrías": "correr",
    "corsa": "corso",
    "corsaria": "corsario",
    "corsarias": "corsario",
    "corsarios": "corsario",
    "corsas": "corso",
    "corseletes": "corselete",
    "corsetera": "corsetero",
    "corseteras": "corsetero",
    "corseteros": "corsetero",
    "corseterías": "corsetería",
    "corsistas": "corsista",
    "corsos": "corso",
    "corsés": "corsé",
    "corta": "corto",
    "cortaba": "cortar",
    "cortabais": "cortar",
    "cortaban": "cortar",
    "cortabas": "cortar",
    "cortacéspedes": "cortacésped",
    "cortad": "cortar",
    "cortada": "cortar",
    "cortadas": "cortar",
    "cortaderas": "cortadera",
    "cortadilla": "cortadillo",
    "cortadillas": "cortadillo",
    "cortadillos": "cortadillo",
    "cortado": "cortar",
    "cortadora": "cortador",
    "cortadoras": "cortadora",
    "cortadores": "cortador",
    "cortados": "cortar",
    "cortaduras": "cortadura",
    "cortafríos": "cortafrío",
    "cortafuegos": "cortafuego",
    "cortamos": "cortar",
    "cortan": "cortar",
    "cortando": "cortar",
    "cortantes": "cortante",
    "cortapisas": "cortapisa",
    "cortara": "cortar",
    "cortarais": "cortar",
    "cortaran": "cortar",
    "cortaras": "cortar",
    "cortare": "cortar",
    "cortareis": "cortar",
    "cortaremos": "cortar",
    "cortaren": "cortar",
    "cortares": "cortar",
    "cortaron": "cortar",
    "cortará": "cortar",
    "cortarán": "cortar",
    "cortarás": "cortar",
    "cortaré": "cortar",
    "cortaréis": "cortar",
    "cortaría": "cortar",
    "cortaríais": "cortar",
    "cortaríamos": "cortar",
    "cortarían": "cortar",
    "cortarías": "cortar",
    "cortas": "corto",
    "cortase": "cortar",
    "cortaseis": "cortar",
    "cortasen": "cortar",
    "cortases": "cortar",
    "cortaste": "cortar",
    "cortasteis": "cortar",
    "cortatiros": "cortatiro",
    "cortavapores": "cortavapor",
    "corte": "cortar",
    "cortedades": "cortedad",
    "corteja": "cortejar",
    "cortejaba": "cortejar",
    "cortejabais": "cortejar",
    "cortejaban": "cortejar",
    "cortejabas": "cortejar",
    "cortejad": "cortejar",
    "cortejada": "cortejar",
    "cortejadas": "cortejar",
    "cortejado": "cortejar",
    "cortejados": "cortejar",
    "cortejamos": "cortejar",
    "cortejan": "cortejar",
    "cortejando": "cortejar",
    "cortejara": "cortejar",
    "cortejarais": "cortejar",
    "cortejaran": "cortejar",
    "cortejaras": "cortejar",
    "cortejare": "cortejar",
    "cortejareis": "cortejar",
    "cortejaremos": "cortejar",
    "cortejaren": "cortejar",
    "cortejares": "cortejar",
    "cortejaron": "cortejar",
    "cortejará": "cortejar",
    "cortejarán": "cortejar",
    "cortejarás": "cortejar",
    "cortejaré": "cortejar",
    "cortejaréis": "cortejar",
    "cortejaría": "cortejar",
    "cortejaríais": "cortejar",
    "cortejaríamos": "cortejar",
    "cortejarían": "cortejar",
    "cortejarías": "cortejar",
    "cortejas": "cortejar",
    "cortejase": "cortejar",
    "cortejaseis": "cortejar",
    "cortejasen": "cortejar",
    "cortejases": "cortejar",
    "cortejaste": "cortejar",
    "cortejasteis": "cortejar",
    "corteje": "cortejar",
    "cortejemos": "cortejar",
    "cortejen": "cortejar",
    "cortejes": "cortejar",
    "cortejo": "cortejar",
    "cortejos": "cortejo",
    "cortejábamos": "cortejar",
    "cortejáis": "cortejar",
    "cortejáramos": "cortejar",
    "cortejáremos": "cortejar",
    "cortejásemos": "cortejar",
    "cortejé": "cortejar",
    "cortejéis": "cortejar",
    "cortejó": "cortejar",
    "cortemos": "cortar",
    "corten": "cortar",
    "cortes": "corte",
    "cortesana": "cortesano",
    "cortesanas": "cortesano",
    "cortesanos": "cortesano",
    "cortesanías": "cortesanía",
    "corteses": "cortés",
    "cortesías": "cortesía",
    "cortezas": "corteza",
    "corticales": "cortical",
    "corticoespinales": "corticoespinal",
    "corticoides": "corticoide",
    "corticorreticulares": "corticorreticular",
    "corticosteroides": "corticosteroide",
    "corticosteronas": "corticosterona",
    "cortijadas": "cortijada",
    "cortijera": "cortijero",
    "cortijeras": "cortijero",
    "cortijeros": "cortijero",
    "cortijos": "cortijo",
    "cortiles": "cortil",
    "cortinajes": "cortinaje",
    "cortinales": "cortinal",
    "cortinas": "cortina",
    "cortinillas": "cortinilla",
    "cortisonas": "cortisona",
    "corto": "cortar",
    "cortocircuita": "cortocircuitar",
    "cortocircuitaba": "cortocircuitar",
    "cortocircuitabais": "cortocircuitar",
    "cortocircuitaban": "cortocircuitar",
    "cortocircuitabas": "cortocircuitar",
    "cortocircuitad": "cortocircuitar",
    "cortocircuitada": "cortocircuitar",
    "cortocircuitadas": "cortocircuitar",
    "cortocircuitado": "cortocircuitar",
    "cortocircuitados": "cortocircuitar",
    "cortocircuitamos": "cortocircuitar",
    "cortocircuitan": "cortocircuitar",
    "cortocircuitando": "cortocircuitar",
    "cortocircuitara": "cortocircuitar",
    "cortocircuitarais": "cortocircuitar",
    "cortocircuitaran": "cortocircuitar",
    "cortocircuitaras": "cortocircuitar",
    "cortocircuitare": "cortocircuitar",
    "cortocircuitareis": "cortocircuitar",
    "cortocircuitaremos": "cortocircuitar",
    "cortocircuitaren": "cortocircuitar",
    "cortocircuitares": "cortocircuitar",
    "cortocircuitaron": "cortocircuitar",
    "cortocircuitará": "cortocircuitar",
    "cortocircuitarán": "cortocircuitar",
    "cortocircuitarás": "cortocircuitar",
    "cortocircuitaré": "cortocircuitar",
    "cortocircuitaréis": "cortocircuitar",
    "cortocircuitaría": "cortocircuitar",
    "cortocircuitaríais": "cortocircuitar",
    "cortocircuitaríamos": "cortocircuitar",
    "cortocircuitarían": "cortocircuitar",
    "cortocircuitarías": "cortocircuitar",
    "cortocircuitas": "cortocircuitar",
    "cortocircuitase": "cortocircuitar",
    "cortocircuitaseis": "cortocircuitar",
    "cortocircuitasen": "cortocircuitar",
    "cortocircuitases": "cortocircuitar",
    "cortocircuitaste": "cortocircuitar",
    "cortocircuitasteis": "cortocircuitar",
    "cortocircuite": "cortocircuitar",
    "cortocircuitemos": "cortocircuitar",
    "cortocircuiten": "cortocircuitar",
    "cortocircuites": "cortocircuitar",
    "cortocircuito": "cortocircuitar",
    "cortocircuitos": "cortocircuito",
    "cortocircuitábamos": "cortocircuitar",
    "cortocircuitáis": "cortocircuitar",
    "cortocircuitáramos": "cortocircuitar",
    "cortocircuitáremos": "cortocircuitar",
    "cortocircuitásemos": "cortocircuitar",
    "cortocircuité": "cortocircuitar",
    "cortocircuitéis": "cortocircuitar",
    "cortocircuitó": "cortocircuitar",
    "cortometrajes": "cortometraje",
    "cortos": "corto",
    "cortábamos": "cortar",
    "cortáis": "cortar",
    "cortáramos": "cortar",
    "cortáremos": "cortar",
    "cortásemos": "cortar",
    "corté": "cortar",
    "cortéis": "cortar",
    "cortéses": "cortés",
    "cortó": "cortar",
    "corusca": "coruscar",
    "coruscaba": "coruscar",
    "coruscabais": "coruscar",
    "coruscaban": "coruscar",
    "coruscabas": "coruscar",
    "coruscad": "coruscar",
    "coruscada": "coruscar",
    "coruscadas": "coruscar",
    "coruscado": "coruscar",
    "coruscados": "coruscar",
    "coruscamos": "coruscar",
    "coruscan": "coruscar",
    "coruscando": "coruscar",
    "coruscara": "coruscar",
    "coruscarais": "coruscar",
    "coruscaran": "coruscar",
    "coruscaras": "coruscar",
    "coruscare": "coruscar",
    "coruscareis": "coruscar",
    "coruscaremos": "coruscar",
    "coruscaren": "coruscar",
    "coruscares": "coruscar",
    "coruscaron": "coruscar",
    "coruscará": "coruscar",
    "coruscarán": "coruscar",
    "coruscarás": "coruscar",
    "coruscaré": "coruscar",
    "coruscaréis": "coruscar",
    "coruscaría": "coruscar",
    "coruscaríais": "coruscar",
    "coruscaríamos": "coruscar",
    "coruscarían": "coruscar",
    "coruscarías": "coruscar",
    "coruscas": "coruscar",
    "coruscase": "coruscar",
    "coruscaseis": "coruscar",
    "coruscasen": "coruscar",
    "coruscases": "coruscar",
    "coruscaste": "coruscar",
    "coruscasteis": "coruscar",
    "corusco": "coruscar",
    "coruscábamos": "coruscar",
    "coruscáis": "coruscar",
    "coruscáramos": "coruscar",
    "coruscáremos": "coruscar",
    "coruscásemos": "coruscar",
    "coruscó": "coruscar",
    "corusque": "coruscar",
    "corusquemos": "coruscar",
    "corusquen": "coruscar",
    "corusques": "coruscar",
    "corusqué": "coruscar",
    "corusquéis": "coruscar",
    "coruñesa": "coruñés",
    "coruñesas": "coruñés",
    "coruñeses": "coruñés",
    "corva": "corvo",
    "corvaduras": "corvadura",
    "corvallos": "corvallo",
    "corvas": "corvo",
    "corvatos": "corvato",
    "corvazas": "corvaza",
    "corvejones": "corvejón",
    "corvetas": "corveta",
    "corvetea": "corvetear",
    "corveteaba": "corvetear",
    "corveteabais": "corvetear",
    "corveteaban": "corvetear",
    "corveteabas": "corvetear",
    "corvetead": "corvetear",
    "corveteada": "corvetear",
    "corveteadas": "corvetear",
    "corveteado": "corvetear",
    "corveteados": "corvetear",
    "corveteamos": "corvetear",
    "corvetean": "corvetear",
    "corveteando": "corvetear",
    "corveteara": "corvetear",
    "corvetearais": "corvetear",
    "corvetearan": "corvetear",
    "corvetearas": "corvetear",
    "corveteare": "corvetear",
    "corveteareis": "corvetear",
    "corvetearemos": "corvetear",
    "corvetearen": "corvetear",
    "corveteares": "corvetear",
    "corvetearon": "corvetear",
    "corveteará": "corvetear",
    "corvetearán": "corvetear",
    "corvetearás": "corvetear",
    "corvetearé": "corvetear",
    "corvetearéis": "corvetear",
    "corvetearía": "corvetear",
    "corvetearíais": "corvetear",
    "corvetearíamos": "corvetear",
    "corvetearían": "corvetear",
    "corvetearías": "corvetear",
    "corveteas": "corvetear",
    "corvetease": "corvetear",
    "corveteaseis": "corvetear",
    "corveteasen": "corvetear",
    "corveteases": "corvetear",
    "corveteaste": "corvetear",
    "corveteasteis": "corvetear",
    "corvetee": "corvetear",
    "corveteemos": "corvetear",
    "corveteen": "corvetear",
    "corvetees": "corvetear",
    "corveteo": "corvetear",
    "corveteábamos": "corvetear",
    "corveteáis": "corvetear",
    "corveteáramos": "corvetear",
    "corveteáremos": "corvetear",
    "corveteásemos": "corvetear",
    "corveteé": "corvetear",
    "corveteéis": "corvetear",
    "corveteó": "corvetear",
    "corvinas": "corvina",
    "corvos": "corvo",
    "corza": "corzo",
    "corzas": "corzo",
    "corzos": "corzo",
    "corzuelas": "corzuela",
    "corácidos": "corácido",
    "coránica": "coránico",
    "coránicas": "coránico",
    "coránicos": "coránico",
    "corégonos": "corégono",
    "coríntica": "coríntico",
    "corínticas": "coríntico",
    "corínticos": "coríntico",
    "corógrafa": "corógrafo",
    "corógrafas": "corógrafo",
    "corógrafos": "corógrafo",
    "cosa": "coser",
    "cosaca": "cosaco",
    "cosacas": "cosaco",
    "cosacos": "cosaco",
    "cosamos": "coser",
    "cosan": "coser",
    "cosaria": "cosario",
    "cosarias": "cosario",
    "cosarios": "cosario",
    "cosas": "coser",
    "cosca": "coscar",
    "coscaba": "coscar",
    "coscabais": "coscar",
    "coscaban": "coscar",
    "coscabas": "coscar",
    "coscad": "coscar",
    "coscada": "coscar",
    "coscadas": "coscar",
    "coscado": "coscar",
    "coscados": "coscar",
    "coscamos": "coscar",
    "coscan": "coscar",
    "coscando": "coscar",
    "coscara": "coscar",
    "coscarais": "coscar",
    "coscaran": "coscar",
    "coscaras": "coscar",
    "coscare": "coscar",
    "coscareis": "coscar",
    "coscaremos": "coscar",
    "coscaren": "coscar",
    "coscares": "coscar",
    "coscaron": "coscar",
    "coscará": "coscar",
    "coscarán": "coscar",
    "coscarás": "coscar",
    "coscaré": "coscar",
    "coscaréis": "coscar",
    "coscaría": "coscar",
    "coscaríais": "coscar",
    "coscaríamos": "coscar",
    "coscarían": "coscar",
    "coscarías": "coscar",
    "coscas": "coscar",
    "coscase": "coscar",
    "coscaseis": "coscar",
    "coscasen": "coscar",
    "coscases": "coscar",
    "coscaste": "coscar",
    "coscasteis": "coscar",
    "cosco": "coscar",
    "coscojas": "coscoja",
    "coscojos": "coscojo",
    "coscona": "coscón",
    "cosconas": "coscón",
    "coscones": "coscón",
    "coscorroneras": "coscorronera",
    "coscorrones": "coscorrón",
    "coscurros": "coscurro",
    "coscábamos": "coscar",
    "coscáis": "coscar",
    "coscáramos": "coscar",
    "coscáremos": "coscar",
    "coscásemos": "coscar",
    "coscó": "coscar",
    "cose": "coser",
    "cosecantes": "cosecante",
    "cosecha": "cosechar",
    "cosechaba": "cosechar",
    "cosechabais": "cosechar",
    "cosechaban": "cosechar",
    "cosechabas": "cosechar",
    "cosechad": "cosechar",
    "cosechada": "cosechar",
    "cosechadas": "cosechar",
    "cosechado": "cosechar",
    "cosechadora": "cosechador",
    "cosechadoras": "cosechadora",
    "cosechadores": "cosechador",
    "cosechados": "cosechar",
    "cosechamos": "cosechar",
    "cosechan": "cosechar",
    "cosechando": "cosechar",
    "cosechara": "cosechar",
    "cosecharais": "cosechar",
    "cosecharan": "cosechar",
    "cosecharas": "cosechar",
    "cosechare": "cosechar",
    "cosechareis": "cosechar",
    "cosecharemos": "cosechar",
    "cosecharen": "cosechar",
    "cosechares": "cosechar",
    "cosecharon": "cosechar",
    "cosechará": "cosechar",
    "cosecharán": "cosechar",
    "cosecharás": "cosechar",
    "cosecharé": "cosechar",
    "cosecharéis": "cosechar",
    "cosecharía": "cosechar",
    "cosecharíais": "cosechar",
    "cosecharíamos": "cosechar",
    "cosecharían": "cosechar",
    "cosecharías": "cosechar",
    "cosechas": "cosechar",
    "cosechase": "cosechar",
    "cosechaseis": "cosechar",
    "cosechasen": "cosechar",
    "cosechases": "cosechar",
    "cosechaste": "cosechar",
    "cosechasteis": "cosechar",
    "coseche": "cosechar",
    "cosechemos": "cosechar",
    "cosechen": "cosechar",
    "cosechera": "cosechero",
    "cosecheras": "cosechero",
    "cosecheros": "cosechero",
    "coseches": "cosechar",
    "cosecho": "cosechar",
    "cosechábamos": "cosechar",
    "cosecháis": "cosechar",
    "cosecháramos": "cosechar",
    "cosecháremos": "cosechar",
    "cosechásemos": "cosechar",
    "coseché": "cosechar",
    "cosechéis": "cosechar",
    "cosechó": "cosechar",
    "cosed": "coser",
    "cosedora": "cosedor",
    "cosedoras": "cosedora",
    "cosedores": "cosedor",
    "coseduras": "cosedura",
    "coseletes": "coselete",
    "cosemos": "coser",
    "cosen": "coser",
    "cosenos": "coseno",
    "coseremos": "coser",
    "coserá": "coser",
    "coserán": "coser",
    "coserás": "coser",
    "coseré": "coser",
    "coseréis": "coser",
    "cosería": "coser",
    "coseríais": "coser",
    "coseríamos": "coser",
    "coserían": "coser",
    "coserías": "coser",
    "coses": "coser",
    "cosetadas": "cosetada",
    "cosetana": "cosetano",
    "cosetanas": "cosetano",
    "cosetanos": "cosetano",
    "cosibles": "cosible",
    "cosida": "coser",
    "cosidas": "coser",
    "cosido": "coser",
    "cosidos": "cosido",
    "cosiendo": "coser",
    "cosiera": "coser",
    "cosierais": "coser",
    "cosieran": "coser",
    "cosieras": "coser",
    "cosiere": "coser",
    "cosiereis": "coser",
    "cosieren": "coser",
    "cosieres": "coser",
    "cosieron": "coser",
    "cosiese": "coser",
    "cosieseis": "coser",
    "cosiesen": "coser",
    "cosieses": "coser",
    "cosificaciones": "cosificación",
    "cosignataria": "cosignatario",
    "cosignatarias": "cosignatario",
    "cosignatarios": "cosignatario",
    "cosijos": "cosijo",
    "cosimos": "coser",
    "cosiste": "coser",
    "cosisteis": "coser",
    "cositas": "cosita",
    "cosiéramos": "coser",
    "cosiéremos": "coser",
    "cosiésemos": "coser",
    "cosió": "coser",
    "cosmetologías": "cosmetología",
    "cosmogonías": "cosmogonía",
    "cosmografías": "cosmografía",
    "cosmográfica": "cosmográfico",
    "cosmográficas": "cosmográfico",
    "cosmográficos": "cosmográfico",
    "cosmogónica": "cosmogónico",
    "cosmogónicas": "cosmogónico",
    "cosmogónicos": "cosmogónico",
    "cosmoidea": "cosmoideo",
    "cosmoideas": "cosmoideo",
    "cosmoideos": "cosmoideo",
    "cosmologías": "cosmología",
    "cosmológica": "cosmológico",
    "cosmológicas": "cosmológico",
    "cosmológicos": "cosmológico",
    "cosmonautas": "cosmonauta",
    "cosmonaves": "cosmonave",
    "cosmonáutica": "cosmonáutico",
    "cosmonáuticas": "cosmonáutico",
    "cosmonáuticos": "cosmonáutico",
    "cosmopolitas": "cosmopolita",
    "cosmopolitismos": "cosmopolitismo",
    "cosmoramas": "cosmorama",
    "cosmovisiones": "cosmovisión",
    "cosmética": "cosmético",
    "cosméticas": "cosmético",
    "cosméticos": "cosmético",
    "cosmódromos": "cosmódromo",
    "cosmógrafa": "cosmógrafo",
    "cosmógrafas": "cosmógrafo",
    "cosmógrafos": "cosmógrafo",
    "cosmóloga": "cosmólogo",
    "cosmólogas": "cosmólogo",
    "cosmólogos": "cosmólogo",
    "coso": "coser",
    "cosos": "coso",
    "cospeles": "cospel",
    "cospes": "cospe",
    "cosque": "coscar",
    "cosquemos": "coscar",
    "cosquen": "coscar",
    "cosques": "coscar",
    "cosquilla": "cosquillar",
    "cosquillaba": "cosquillar",
    "cosquillabais": "cosquillar",
    "cosquillaban": "cosquillar",
    "cosquillabas": "cosquillar",
    "cosquillad": "cosquillar",
    "cosquillada": "cosquillar",
    "cosquilladas": "cosquillar",
    "cosquillado": "cosquillar",
    "cosquillados": "cosquillar",
    "cosquillamos": "cosquillar",
    "cosquillan": "cosquillar",
    "cosquillando": "cosquillar",
    "cosquillara": "cosquillar",
    "cosquillarais": "cosquillar",
    "cosquillaran": "cosquillar",
    "cosquillaras": "cosquillar",
    "cosquillare": "cosquillar",
    "cosquillareis": "cosquillar",
    "cosquillaremos": "cosquillar",
    "cosquillaren": "cosquillar",
    "cosquillares": "cosquillar",
    "cosquillaron": "cosquillar",
    "cosquillará": "cosquillar",
    "cosquillarán": "cosquillar",
    "cosquillarás": "cosquillar",
    "cosquillaré": "cosquillar",
    "cosquillaréis": "cosquillar",
    "cosquillaría": "cosquillar",
    "cosquillaríais": "cosquillar",
    "cosquillaríamos": "cosquillar",
    "cosquillarían": "cosquillar",
    "cosquillarías": "cosquillar",
    "cosquillas": "cosquillar",
    "cosquillase": "cosquillar",
    "cosquillaseis": "cosquillar",
    "cosquillasen": "cosquillar",
    "cosquillases": "cosquillar",
    "cosquillaste": "cosquillar",
    "cosquillasteis": "cosquillar",
    "cosquille": "cosquillar",
    "cosquillea": "cosquillear",
    "cosquilleaba": "cosquillear",
    "cosquilleabais": "cosquillear",
    "cosquilleaban": "cosquillear",
    "cosquilleabas": "cosquillear",
    "cosquillead": "cosquillear",
    "cosquilleada": "cosquillear",
    "cosquilleadas": "cosquillear",
    "cosquilleado": "cosquillear",
    "cosquilleados": "cosquillear",
    "cosquilleamos": "cosquillear",
    "cosquillean": "cosquillear",
    "cosquilleando": "cosquillear",
    "cosquilleara": "cosquillear",
    "cosquillearais": "cosquillear",
    "cosquillearan": "cosquillear",
    "cosquillearas": "cosquillear",
    "cosquilleare": "cosquillear",
    "cosquilleareis": "cosquillear",
    "cosquillearemos": "cosquillear",
    "cosquillearen": "cosquillear",
    "cosquilleares": "cosquillear",
    "cosquillearon": "cosquillear",
    "cosquilleará": "cosquillear",
    "cosquillearán": "cosquillear",
    "cosquillearás": "cosquillear",
    "cosquillearé": "cosquillear",
    "cosquillearéis": "cosquillear",
    "cosquillearía": "cosquillear",
    "cosquillearíais": "cosquillear",
    "cosquillearíamos": "cosquillear",
    "cosquillearían": "cosquillear",
    "cosquillearías": "cosquillear",
    "cosquilleas": "cosquillear",
    "cosquillease": "cosquillear",
    "cosquilleaseis": "cosquillear",
    "cosquilleasen": "cosquillear",
    "cosquilleases": "cosquillear",
    "cosquilleaste": "cosquillear",
    "cosquilleasteis": "cosquillear",
    "cosquillee": "cosquillear",
    "cosquilleemos": "cosquillear",
    "cosquilleen": "cosquillear",
    "cosquillees": "cosquillear",
    "cosquillemos": "cosquillar",
    "cosquillen": "cosquillar",
    "cosquilleo": "cosquillear",
    "cosquilleos": "cosquilleo",
    "cosquilles": "cosquillar",
    "cosquilleábamos": "cosquillear",
    "cosquilleáis": "cosquillear",
    "cosquilleáramos": "cosquillear",
    "cosquilleáremos": "cosquillear",
    "cosquilleásemos": "cosquillear",
    "cosquilleé": "cosquillear",
    "cosquilleéis": "cosquillear",
    "cosquilleó": "cosquillear",
    "cosquillo": "cosquillar",
    "cosquillosa": "cosquilloso",
    "cosquillosas": "cosquilloso",
    "cosquillosos": "cosquilloso",
    "cosquillábamos": "cosquillar",
    "cosquilláis": "cosquillar",
    "cosquilláramos": "cosquillar",
    "cosquilláremos": "cosquillar",
    "cosquillásemos": "cosquillar",
    "cosquillé": "cosquillar",
    "cosquilléis": "cosquillar",
    "cosquilló": "cosquillar",
    "cosqué": "coscar",
    "cosquéis": "coscar",
    "costaba": "costar",
    "costabais": "costar",
    "costaban": "costar",
    "costabas": "costar",
    "costad": "costar",
    "costada": "costar",
    "costadas": "costar",
    "costado": "costar",
    "costados": "costar",
    "costaladas": "costalada",
    "costalazos": "costalazo",
    "costalera": "costalero",
    "costaleras": "costalero",
    "costaleros": "costalero",
    "costales": "costal",
    "costamos": "costar",
    "costanas": "costana",
    "costando": "costar",
    "costanera": "costanero",
    "costaneras": "costanero",
    "costaneros": "costanero",
    "costara": "costar",
    "costarais": "costar",
    "costaran": "costar",
    "costaras": "costar",
    "costare": "costar",
    "costareis": "costar",
    "costaremos": "costar",
    "costaren": "costar",
    "costares": "costar",
    "costaron": "costar",
    "costarricenses": "costarricense",
    "costarriqueña": "costarriqueño",
    "costarriqueñas": "costarriqueño",
    "costarriqueños": "costarriqueño",
    "costará": "costar",
    "costarán": "costar",
    "costarás": "costar",
    "costaré": "costar",
    "costaréis": "costar",
    "costaría": "costar",
    "costaríais": "costar",
    "costaríamos": "costar",
    "costarían": "costar",
    "costarías": "costar",
    "costas": "costa",
    "costase": "costar",
    "costaseis": "costar",
    "costasen": "costar",
    "costases": "costar",
    "costasoleña": "costasoleño",
    "costasoleñas": "costasoleño",
    "costasoleños": "costasoleño",
    "costaste": "costar",
    "costasteis": "costar",
    "costea": "costear",
    "costeaba": "costear",
    "costeabais": "costear",
    "costeaban": "costear",
    "costeabas": "costear",
    "costead": "costear",
    "costeada": "costear",
    "costeadas": "costear",
    "costeado": "costear",
    "costeados": "costear",
    "costeamos": "costear",
    "costean": "costear",
    "costeando": "costear",
    "costeara": "costear",
    "costearais": "costear",
    "costearan": "costear",
    "costearas": "costear",
    "costeare": "costear",
    "costeareis": "costear",
    "costearemos": "costear",
    "costearen": "costear",
    "costeares": "costear",
    "costearon": "costear",
    "costeará": "costear",
    "costearán": "costear",
    "costearás": "costear",
    "costearé": "costear",
    "costearéis": "costear",
    "costearía": "costear",
    "costearíais": "costear",
    "costearíamos": "costear",
    "costearían": "costear",
    "costearías": "costear",
    "costeas": "costear",
    "costease": "costear",
    "costeaseis": "costear",
    "costeasen": "costear",
    "costeases": "costear",
    "costeaste": "costear",
    "costeasteis": "costear",
    "costee": "costear",
    "costeemos": "costear",
    "costeen": "costear",
    "costees": "costear",
    "costemos": "costar",
    "costeo": "costear",
    "costeos": "costeo",
    "costera": "costero",
    "costeras": "costero",
    "costeros": "costero",
    "costes": "coste",
    "costeábamos": "costear",
    "costeáis": "costear",
    "costeáramos": "costear",
    "costeáremos": "costear",
    "costeásemos": "costear",
    "costeé": "costear",
    "costeéis": "costear",
    "costeña": "costeño",
    "costeñas": "costeño",
    "costeños": "costeño",
    "costeó": "costear",
    "costillajes": "costillaje",
    "costillares": "costillar",
    "costillas": "costilla",
    "costocervicales": "costocervical",
    "costoclaviculares": "costoclavicular",
    "costocondrales": "costocondral",
    "costodiafragmática": "costodiafragmático",
    "costodiafragmáticas": "costodiafragmático",
    "costodiafragmáticos": "costodiafragmático",
    "costomediastínica": "costomediastínico",
    "costomediastínicas": "costomediastínico",
    "costomediastínicos": "costomediastínico",
    "costos": "costo",
    "costosa": "costoso",
    "costosas": "costoso",
    "costosos": "costoso",
    "costotransversa": "costotransverso",
    "costotransversas": "costotransverso",
    "costotransversos": "costotransverso",
    "costovertebrales": "costovertebral",
    "costoxifoidea": "costoxifoideo",
    "costoxifoideas": "costoxifoideo",
    "costoxifoideos": "costoxifoideo",
    "costradas": "costrada",
    "costras": "costra",
    "costreñid": "costreñir",
    "costreñida": "costreñir",
    "costreñidas": "costreñir",
    "costreñido": "costreñir",
    "costreñidos": "costreñir",
    "costreñimos": "costreñir",
    "costreñiremos": "costreñir",
    "costreñirá": "costreñir",
    "costreñirán": "costreñir",
    "costreñirás": "costreñir",
    "costreñiré": "costreñir",
    "costreñiréis": "costreñir",
    "costreñiría": "costreñir",
    "costreñiríais": "costreñir",
    "costreñiríamos": "costreñir",
    "costreñirían": "costreñir",
    "costreñirías": "costreñir",
    "costreñiste": "costreñir",
    "costreñisteis": "costreñir",
    "costreñí": "costreñir",
    "costreñía": "costreñir",
    "costreñíais": "costreñir",
    "costreñíamos": "costreñir",
    "costreñían": "costreñir",
    "costreñías": "costreñir",
    "costreñís": "costreñir",
    "costriña": "costreñir",
    "costriñamos": "costreñir",
    "costriñan": "costreñir",
    "costriñas": "costreñir",
    "costriñe": "costreñir",
    "costriñen": "costreñir",
    "costriñendo": "costreñir",
    "costriñera": "costreñir",
    "costriñerais": "costreñir",
    "costriñeran": "costreñir",
    "costriñeras": "costreñir",
    "costriñere": "costreñir",
    "costriñereis": "costreñir",
    "costriñeren": "costreñir",
    "costriñeres": "costreñir",
    "costriñeron": "costreñir",
    "costriñes": "costreñir",
    "costriñese": "costreñir",
    "costriñeseis": "costreñir",
    "costriñesen": "costreñir",
    "costriñeses": "costreñir",
    "costriño": "costreñir",
    "costriñáis": "costreñir",
    "costriñéramos": "costreñir",
    "costriñéremos": "costreñir",
    "costriñésemos": "costreñir",
    "costriñó": "costreñir",
    "costrosa": "costroso",
    "costrosas": "costroso",
    "costrosos": "costroso",
    "costumbres": "costumbre",
    "costumbrismos": "costumbrismo",
    "costumbristas": "costumbrista",
    "costuras": "costura",
    "costureras": "costurera",
    "costureros": "costurero",
    "costurones": "costurón",
    "costábamos": "costar",
    "costáis": "costar",
    "costáramos": "costar",
    "costáremos": "costar",
    "costásemos": "costar",
    "costé": "costar",
    "costéis": "costar",
    "costó": "costar",
    "cosáis": "coser",
    "coséis": "coser",
    "cosí": "coser",
    "cosía": "coser",
    "cosíais": "coser",
    "cosíamos": "coser",
    "cosían": "coser",
    "cosías": "coser",
    "cota": "cotar",
    "cotaba": "cotar",
    "cotabais": "cotar",
    "cotaban": "cotar",
    "cotabas": "cotar",
    "cotad": "cotar",
    "cotada": "cotar",
    "cotadas": "cotar",
    "cotado": "cotar",
    "cotados": "cotar",
    "cotamos": "cotar",
    "cotan": "cotar",
    "cotanas": "cotana",
    "cotando": "cotar",
    "cotangentes": "cotangente",
    "cotara": "cotar",
    "cotarais": "cotar",
    "cotaran": "cotar",
    "cotaras": "cotar",
    "cotare": "cotar",
    "cotareis": "cotar",
    "cotaremos": "cotar",
    "cotaren": "cotar",
    "cotares": "cotar",
    "cotaron": "cotar",
    "cotarros": "cotarro",
    "cotará": "cotar",
    "cotarán": "cotar",
    "cotarás": "cotar",
    "cotaré": "cotar",
    "cotaréis": "cotar",
    "cotaría": "cotar",
    "cotaríais": "cotar",
    "cotaríamos": "cotar",
    "cotarían": "cotar",
    "cotarías": "cotar",
    "cotas": "cotar",
    "cotase": "cotar",
    "cotaseis": "cotar",
    "cotasen": "cotar",
    "cotases": "cotar",
    "cotaste": "cotar",
    "cotasteis": "cotar",
    "cote": "cotar",
    "coteja": "cotejar",
    "cotejaba": "cotejar",
    "cotejabais": "cotejar",
    "cotejaban": "cotejar",
    "cotejabas": "cotejar",
    "cotejables": "cotejable",
    "cotejad": "cotejar",
    "cotejada": "cotejar",
    "cotejadas": "cotejar",
    "cotejado": "cotejar",
    "cotejados": "cotejar",
    "cotejamos": "cotejar",
    "cotejan": "cotejar",
    "cotejando": "cotejar",
    "cotejara": "cotejar",
    "cotejarais": "cotejar",
    "cotejaran": "cotejar",
    "cotejaras": "cotejar",
    "cotejare": "cotejar",
    "cotejareis": "cotejar",
    "cotejaremos": "cotejar",
    "cotejaren": "cotejar",
    "cotejares": "cotejar",
    "cotejaron": "cotejar",
    "cotejará": "cotejar",
    "cotejarán": "cotejar",
    "cotejarás": "cotejar",
    "cotejaré": "cotejar",
    "cotejaréis": "cotejar",
    "cotejaría": "cotejar",
    "cotejaríais": "cotejar",
    "cotejaríamos": "cotejar",
    "cotejarían": "cotejar",
    "cotejarías": "cotejar",
    "cotejas": "cotejar",
    "cotejase": "cotejar",
    "cotejaseis": "cotejar",
    "cotejasen": "cotejar",
    "cotejases": "cotejar",
    "cotejaste": "cotejar",
    "cotejasteis": "cotejar",
    "coteje": "cotejar",
    "cotejemos": "cotejar",
    "cotejen": "cotejar",
    "cotejes": "cotejar",
    "cotejo": "cotejar",
    "cotejos": "cotejo",
    "cotejábamos": "cotejar",
    "cotejáis": "cotejar",
    "cotejáramos": "cotejar",
    "cotejáremos": "cotejar",
    "cotejásemos": "cotejar",
    "cotejé": "cotejar",
    "cotejéis": "cotejar",
    "cotejó": "cotejar",
    "cotemos": "cotar",
    "coten": "cotar",
    "coterminales": "coterminal",
    "coterránea": "coterráneo",
    "coterráneas": "coterráneo",
    "coterráneos": "coterráneo",
    "cotes": "cote",
    "cotice": "cotizar",
    "coticemos": "cotizar",
    "coticen": "cotizar",
    "cotices": "cotizar",
    "coticé": "cotizar",
    "coticéis": "cotizar",
    "cotidales": "cotidal",
    "cotidiana": "cotidiano",
    "cotidianas": "cotidiano",
    "cotidianidades": "cotidianidad",
    "cotidianos": "cotidiano",
    "cotilas": "cotila",
    "cotiledones": "cotiledón",
    "cotiledónea": "cotiledóneo",
    "cotiledóneas": "cotiledóneo",
    "cotiledóneos": "cotiledóneo",
    "cotillas": "cotilla",
    "cotillea": "cotillear",
    "cotilleaba": "cotillear",
    "cotilleabais": "cotillear",
    "cotilleaban": "cotillear",
    "cotilleabas": "cotillear",
    "cotillead": "cotillear",
    "cotilleada": "cotillear",
    "cotilleadas": "cotillear",
    "cotilleado": "cotillear",
    "cotilleados": "cotillear",
    "cotilleamos": "cotillear",
    "cotillean": "cotillear",
    "cotilleando": "cotillear",
    "cotilleara": "cotillear",
    "cotillearais": "cotillear",
    "cotillearan": "cotillear",
    "cotillearas": "cotillear",
    "cotilleare": "cotillear",
    "cotilleareis": "cotillear",
    "cotillearemos": "cotillear",
    "cotillearen": "cotillear",
    "cotilleares": "cotillear",
    "cotillearon": "cotillear",
    "cotilleará": "cotillear",
    "cotillearán": "cotillear",
    "cotillearás": "cotillear",
    "cotillearé": "cotillear",
    "cotillearéis": "cotillear",
    "cotillearía": "cotillear",
    "cotillearíais": "cotillear",
    "cotillearíamos": "cotillear",
    "cotillearían": "cotillear",
    "cotillearías": "cotillear",
    "cotilleas": "cotillear",
    "cotillease": "cotillear",
    "cotilleaseis": "cotillear",
    "cotilleasen": "cotillear",
    "cotilleases": "cotillear",
    "cotilleaste": "cotillear",
    "cotilleasteis": "cotillear",
    "cotillee": "cotillear",
    "cotilleemos": "cotillear",
    "cotilleen": "cotillear",
    "cotillees": "cotillear",
    "cotilleo": "cotillear",
    "cotilleos": "cotilleo",
    "cotillera": "cotillero",
    "cotilleras": "cotillero",
    "cotilleros": "cotillero",
    "cotilleábamos": "cotillear",
    "cotilleáis": "cotillear",
    "cotilleáramos": "cotillear",
    "cotilleáremos": "cotillear",
    "cotilleásemos": "cotillear",
    "cotilleé": "cotillear",
    "cotilleéis": "cotillear",
    "cotilleó": "cotillear",
    "cotillones": "cotillón",
    "cotillos": "cotillo",
    "cotiloidea": "cotiloideo",
    "cotiloideas": "cotiloideo",
    "cotiloideos": "cotiloideo",
    "cotiza": "cotizar",
    "cotizaba": "cotizar",
    "cotizabais": "cotizar",
    "cotizaban": "cotizar",
    "cotizabas": "cotizar",
    "cotizables": "cotizable",
    "cotizaciones": "cotización",
    "cotizad": "cotizar",
    "cotizada": "cotizar",
    "cotizadas": "cotizar",
    "cotizado": "cotizar",
    "cotizadores": "cotizador",
    "cotizados": "cotizar",
    "cotizamos": "cotizar",
    "cotizan": "cotizar",
    "cotizando": "cotizar",
    "cotizantes": "cotizante",
    "cotizara": "cotizar",
    "cotizarais": "cotizar",
    "cotizaran": "cotizar",
    "cotizaras": "cotizar",
    "cotizare": "cotizar",
    "cotizareis": "cotizar",
    "cotizaremos": "cotizar",
    "cotizaren": "cotizar",
    "cotizares": "cotizar",
    "cotizaron": "cotizar",
    "cotizará": "cotizar",
    "cotizarán": "cotizar",
    "cotizarás": "cotizar",
    "cotizaré": "cotizar",
    "cotizaréis": "cotizar",
    "cotizaría": "cotizar",
    "cotizaríais": "cotizar",
    "cotizaríamos": "cotizar",
    "cotizarían": "cotizar",
    "cotizarías": "cotizar",
    "cotizas": "cotizar",
    "cotizase": "cotizar",
    "cotizaseis": "cotizar",
    "cotizasen": "cotizar",
    "cotizases": "cotizar",
    "cotizaste": "cotizar",
    "cotizasteis": "cotizar",
    "cotizo": "cotizar",
    "cotizábamos": "cotizar",
    "cotizáis": "cotizar",
    "cotizáramos": "cotizar",
    "cotizáremos": "cotizar",
    "cotizásemos": "cotizar",
    "cotizó": "cotizar",
    "coto": "cotar",
    "cotonadas": "cotonada",
    "cotonas": "cotona",
    "cotoncillos": "cotoncillo",
    "cotones": "cotón",
    "cotonías": "cotonía",
    "cotorras": "cotorra",
    "cotorrea": "cotorrear",
    "cotorreaba": "cotorrear",
    "cotorreabais": "cotorrear",
    "cotorreaban": "cotorrear",
    "cotorreabas": "cotorrear",
    "cotorread": "cotorrear",
    "cotorreada": "cotorrear",
    "cotorreadas": "cotorrear",
    "cotorreado": "cotorrear",
    "cotorreados": "cotorrear",
    "cotorreamos": "cotorrear",
    "cotorrean": "cotorrear",
    "cotorreando": "cotorrear",
    "cotorreara": "cotorrear",
    "cotorrearais": "cotorrear",
    "cotorrearan": "cotorrear",
    "cotorrearas": "cotorrear",
    "cotorreare": "cotorrear",
    "cotorreareis": "cotorrear",
    "cotorrearemos": "cotorrear",
    "cotorrearen": "cotorrear",
    "cotorreares": "cotorrear",
    "cotorrearon": "cotorrear",
    "cotorreará": "cotorrear",
    "cotorrearán": "cotorrear",
    "cotorrearás": "cotorrear",
    "cotorrearé": "cotorrear",
    "cotorrearéis": "cotorrear",
    "cotorrearía": "cotorrear",
    "cotorrearíais": "cotorrear",
    "cotorrearíamos": "cotorrear",
    "cotorrearían": "cotorrear",
    "cotorrearías": "cotorrear",
    "cotorreas": "cotorrear",
    "cotorrease": "cotorrear",
    "cotorreaseis": "cotorrear",
    "cotorreasen": "cotorrear",
    "cotorreases": "cotorrear",
    "cotorreaste": "cotorrear",
    "cotorreasteis": "cotorrear",
    "cotorree": "cotorrear",
    "cotorreemos": "cotorrear",
    "cotorreen": "cotorrear",
    "cotorrees": "cotorrear",
    "cotorreo": "cotorrear",
    "cotorreos": "cotorreo",
    "cotorreábamos": "cotorrear",
    "cotorreáis": "cotorrear",
    "cotorreáramos": "cotorrear",
    "cotorreáremos": "cotorrear",
    "cotorreásemos": "cotorrear",
    "cotorreé": "cotorrear",
    "cotorreéis": "cotorrear",
    "cotorreó": "cotorrear",
    "cotorrona": "cotorrón",
    "cotorronas": "cotorrón",
    "cotorrones": "cotorrón",
    "cotos": "coto",
    "cotraduccionales": "cotraduccional",
    "cotufas": "cotufa",
    "coturnos": "coturno",
    "cotábamos": "cotar",
    "cotáis": "cotar",
    "cotáramos": "cotar",
    "cotáremos": "cotar",
    "cotásemos": "cotar",
    "coté": "cotar",
    "cotéis": "cotar",
    "cotó": "cotar",
    "coulombs": "coulomb",
    "countries": "country",
    "couriers": "courier",
    "covachas": "covacha",
    "covachuelas": "covachuela",
    "covachuelistas": "covachuelista",
    "covachuelos": "covachuelo",
    "covadas": "covada",
    "covalencias": "covalencia",
    "covalentes": "covalente",
    "covariancias": "covariancia",
    "covariantes": "covariante",
    "covarianzas": "covarianza",
    "covelitas": "covelita",
    "covellinas": "covellina",
    "covenants": "covenant",
    "cover-girls": "cover-girl",
    "coveres": "cover",
    "covergirls": "covergirl",
    "covers": "cover",
    "cowboys": "cowboy",
    "coxales": "coxal",
    "coxalgias": "coxalgia",
    "coxas": "coxa",
    "coxcojillas": "coxcojilla",
    "coxofemorales": "coxofemoral",
    "coxálgica": "coxálgico",
    "coxálgicas": "coxálgico",
    "coxálgicos": "coxálgico",
    "coyotera": "coyotero",
    "coyoteras": "coyotero",
    "coyoteros": "coyotero",
    "coyotes": "coyote",
    "coys": "coy",
    "coyundas": "coyunda",
    "coyunturales": "coyuntural",
    "coyunturas": "coyuntura",
    "cozamos": "cocer",
    "cozáis": "cocer",
    "coágulos": "coágulo",
    "coñacs": "coñac",
    "coñazos": "coñazo",
    "coñea": "coñear",
    "coñeaba": "coñear",
    "coñeabais": "coñear",
    "coñeaban": "coñear",
    "coñeabas": "coñear",
    "coñead": "coñear",
    "coñeada": "coñear",
    "coñeadas": "coñear",
    "coñeado": "coñear",
    "coñeados": "coñear",
    "coñeamos": "coñear",
    "coñean": "coñear",
    "coñeando": "coñear",
    "coñeara": "coñear",
    "coñearais": "coñear",
    "coñearan": "coñear",
    "coñearas": "coñear",
    "coñeare": "coñear",
    "coñeareis": "coñear",
    "coñearemos": "coñear",
    "coñearen": "coñear",
    "coñeares": "coñear",
    "coñearon": "coñear",
    "coñeará": "coñear",
    "coñearán": "coñear",
    "coñearás": "coñear",
    "coñearé": "coñear",
    "coñearéis": "coñear",
    "coñearía": "coñear",
    "coñearíais": "coñear",
    "coñearíamos": "coñear",
    "coñearían": "coñear",
    "coñearías": "coñear",
    "coñeas": "coñear",
    "coñease": "coñear",
    "coñeaseis": "coñear",
    "coñeasen": "coñear",
    "coñeases": "coñear",
    "coñeaste": "coñear",
    "coñeasteis": "coñear",
    "coñee": "coñear",
    "coñeemos": "coñear",
    "coñeen": "coñear",
    "coñees": "coñear",
    "coñeo": "coñear",
    "coñeos": "coñeo",
    "coñetes": "coñete",
    "coñeábamos": "coñear",
    "coñeáis": "coñear",
    "coñeáramos": "coñear",
    "coñeáremos": "coñear",
    "coñeásemos": "coñear",
    "coñeé": "coñear",
    "coñeéis": "coñear",
    "coñeó": "coñear",
    "coñona": "coñón",
    "coñonas": "coñón",
    "coñones": "coñón",
    "coños": "coño",
    "coñás": "coñá",
    "cpus": "cpu",
    "crackings": "cracking",
    "cracks": "crack",
    "cracoviana": "cracoviano",
    "cracovianas": "cracoviano",
    "cracovianos": "cracoviano",
    "cracs": "crac",
    "crampones": "crampón",
    "craneales": "craneal",
    "craneana": "craneano",
    "craneanas": "craneano",
    "craneanos": "craneano",
    "craneoencefálica": "craneoencefálico",
    "craneoencefálicas": "craneoencefálico",
    "craneoencefálicos": "craneoencefálico",
    "craneofaciales": "craneofacial",
    "craneologías": "craneología",
    "craneomaxilares": "craneomaxilar",
    "craneométrica": "craneométrico",
    "craneométricas": "craneométrico",
    "craneométricos": "craneométrico",
    "craneopatías": "craneopatía",
    "cranes": "cran",
    "craneógrafos": "craneógrafo",
    "craniana": "craniano",
    "cranianas": "craniano",
    "cranianos": "craniano",
    "craniometrías": "craniometría",
    "crapulosa": "crapuloso",
    "crapulosas": "crapuloso",
    "crapulosos": "crapuloso",
    "craqueos": "craqueo",
    "crasa": "craso",
    "crasas": "craso",
    "crascita": "crascitar",
    "crascitaba": "crascitar",
    "crascitabais": "crascitar",
    "crascitaban": "crascitar",
    "crascitabas": "crascitar",
    "crascitad": "crascitar",
    "crascitada": "crascitar",
    "crascitadas": "crascitar",
    "crascitado": "crascitar",
    "crascitados": "crascitar",
    "crascitamos": "crascitar",
    "crascitan": "crascitar",
    "crascitando": "crascitar",
    "crascitara": "crascitar",
    "crascitarais": "crascitar",
    "crascitaran": "crascitar",
    "crascitaras": "crascitar",
    "crascitare": "crascitar",
    "crascitareis": "crascitar",
    "crascitaremos": "crascitar",
    "crascitaren": "crascitar",
    "crascitares": "crascitar",
    "crascitaron": "crascitar",
    "crascitará": "crascitar",
    "crascitarán": "crascitar",
    "crascitarás": "crascitar",
    "crascitaré": "crascitar",
    "crascitaréis": "crascitar",
    "crascitaría": "crascitar",
    "crascitaríais": "crascitar",
    "crascitaríamos": "crascitar",
    "crascitarían": "crascitar",
    "crascitarías": "crascitar",
    "crascitas": "crascitar",
    "crascitase": "crascitar",
    "crascitaseis": "crascitar",
    "crascitasen": "crascitar",
    "crascitases": "crascitar",
    "crascitaste": "crascitar",
    "crascitasteis": "crascitar",
    "crascite": "crascitar",
    "crascitemos": "crascitar",
    "crasciten": "crascitar",
    "crascites": "crascitar",
    "crascito": "crascitar",
    "crascitábamos": "crascitar",
    "crascitáis": "crascitar",
    "crascitáramos": "crascitar",
    "crascitáremos": "crascitar",
    "crascitásemos": "crascitar",
    "crascité": "crascitar",
    "crascitéis": "crascitar",
    "crascitó": "crascitar",
    "crasitudes": "crasitud",
    "crasos": "craso",
    "crasuláceas": "crasulácea",
    "crateras": "cratera",
    "crateriformes": "crateriforme",
    "cratones": "cratón",
    "cratículas": "cratícula",
    "crawles": "crawl",
    "crawls": "crawl",
    "crazadas": "crazada",
    "crazas": "craza",
    "crea": "creer",
    "creaba": "crear",
    "creabais": "crear",
    "creaban": "crear",
    "creabas": "crear",
    "creaciones": "creación",
    "creacionismos": "creacionismo",
    "creacionistas": "creacionista",
    "cread": "crear",
    "creada": "crear",
    "creadas": "crear",
    "creado": "crear",
    "creadora": "creador",
    "creadoras": "creador",
    "creadores": "creador",
    "creados": "crear",
    "creamos": "creer",
    "crean": "creer",
    "creando": "crear",
    "creara": "crear",
    "crearais": "crear",
    "crearan": "crear",
    "crearas": "crear",
    "creare": "crear",
    "creareis": "crear",
    "crearemos": "crear",
    "crearen": "crear",
    "creares": "crear",
    "crearon": "crear",
    "creará": "crear",
    "crearán": "crear",
    "crearás": "crear",
    "crearé": "crear",
    "crearéis": "crear",
    "crearía": "crear",
    "crearíais": "crear",
    "crearíamos": "crear",
    "crearían": "crear",
    "crearías": "crear",
    "creas": "creer",
    "crease": "crear",
    "creaseis": "crear",
    "creasen": "crear",
    "creases": "crear",
    "creaste": "crear",
    "creasteis": "crear",
    "creativa": "creativo",
    "creativas": "creativo",
    "creatividades": "creatividad",
    "creativos": "creativo",
    "crece": "crecer",
    "creced": "crecer",
    "crecedera": "crecedero",
    "crecederas": "crecedero",
    "crecederos": "crecedero",
    "crecemos": "crecer",
    "crecen": "crecer",
    "creceremos": "crecer",
    "creceros": "crecero",
    "crecerá": "crecer",
    "crecerán": "crecer",
    "crecerás": "crecer",
    "creceré": "crecer",
    "creceréis": "crecer",
    "crecería": "crecer",
    "creceríais": "crecer",
    "creceríamos": "crecer",
    "crecerían": "crecer",
    "crecerías": "crecer",
    "creces": "crecer",
    "crecida": "crecer",
    "crecidas": "crecida",
    "crecido": "crecer",
    "crecidos": "crecer",
    "creciendo": "crecer",
    "crecientes": "creciente",
    "creciera": "crecer",
    "crecierais": "crecer",
    "crecieran": "crecer",
    "crecieras": "crecer",
    "creciere": "crecer",
    "creciereis": "crecer",
    "crecieren": "crecer",
    "crecieres": "crecer",
    "crecieron": "crecer",
    "creciese": "crecer",
    "crecieseis": "crecer",
    "creciesen": "crecer",
    "crecieses": "crecer",
    "crecimientos": "crecimiento",
    "crecimos": "crecer",
    "creciste": "crecer",
    "crecisteis": "crecer",
    "creciéramos": "crecer",
    "creciéremos": "crecer",
    "creciésemos": "crecer",
    "creció": "crecer",
    "crecéis": "crecer",
    "crecí": "crecer",
    "crecía": "crecer",
    "crecíais": "crecer",
    "crecíamos": "crecer",
    "crecían": "crecer",
    "crecías": "crecer",
    "credenciales": "credencial",
    "credencias": "credencia",
    "credibilidades": "credibilidad",
    "crediticia": "crediticio",
    "crediticias": "crediticio",
    "crediticios": "crediticio",
    "creditora": "creditor",
    "creditoras": "creditor",
    "creditores": "creditor",
    "credos": "credo",
    "credulidades": "credulidad",
    "cree": "creer",
    "creed": "creer",
    "creemos": "creer",
    "creen": "creer",
    "creencias": "creencia",
    "creeremos": "creer",
    "creerá": "creer",
    "creerán": "creer",
    "creerás": "creer",
    "creeré": "creer",
    "creeréis": "creer",
    "creería": "creer",
    "creeríais": "creer",
    "creeríamos": "creer",
    "creerían": "creer",
    "creerías": "creer",
    "crees": "creer",
    "cremaciones": "cremación",
    "cremalleras": "cremallera",
    "cremas": "crema",
    "cremastérica": "cremastérico",
    "cremastéricas": "cremastérico",
    "cremastéricos": "cremastérico",
    "crematoria": "crematorio",
    "crematorias": "crematorio",
    "crematorios": "crematorio",
    "crematística": "crematístico",
    "crematísticas": "crematístico",
    "crematísticos": "crematístico",
    "cremonas": "cremona",
    "cremonesa": "cremonés",
    "cremonesas": "cremonés",
    "cremoneses": "cremonés",
    "cremosa": "cremoso",
    "cremosas": "cremoso",
    "cremosidades": "cremosidad",
    "cremosos": "cremoso",
    "crenchas": "crencha",
    "crenocitos": "crenocito",
    "creo": "creer",
    "creosota": "creosotar",
    "creosotaba": "creosotar",
    "creosotabais": "creosotar",
    "creosotaban": "creosotar",
    "creosotabas": "creosotar",
    "creosotad": "creosotar",
    "creosotada": "creosotar",
    "creosotadas": "creosotar",
    "creosotado": "creosotar",
    "creosotados": "creosotar",
    "creosotamos": "creosotar",
    "creosotan": "creosotar",
    "creosotando": "creosotar",
    "creosotara": "creosotar",
    "creosotarais": "creosotar",
    "creosotaran": "creosotar",
    "creosotaras": "creosotar",
    "creosotare": "creosotar",
    "creosotareis": "creosotar",
    "creosotaremos": "creosotar",
    "creosotaren": "creosotar",
    "creosotares": "creosotar",
    "creosotaron": "creosotar",
    "creosotará": "creosotar",
    "creosotarán": "creosotar",
    "creosotarás": "creosotar",
    "creosotaré": "creosotar",
    "creosotaréis": "creosotar",
    "creosotaría": "creosotar",
    "creosotaríais": "creosotar",
    "creosotaríamos": "creosotar",
    "creosotarían": "creosotar",
    "creosotarías": "creosotar",
    "creosotas": "creosotar",
    "creosotase": "creosotar",
    "creosotaseis": "creosotar",
    "creosotasen": "creosotar",
    "creosotases": "creosotar",
    "creosotaste": "creosotar",
    "creosotasteis": "creosotar",
    "creosote": "creosotar",
    "creosotemos": "creosotar",
    "creosoten": "creosotar",
    "creosotes": "creosotar",
    "creosoto": "creosotar",
    "creosotábamos": "creosotar",
    "creosotáis": "creosotar",
    "creosotáramos": "creosotar",
    "creosotáremos": "creosotar",
    "creosotásemos": "creosotar",
    "creosoté": "creosotar",
    "creosotéis": "creosotar",
    "creosotó": "creosotar",
    "creperías": "crepería",
    "crepes": "crepe",
    "crepidomas": "crepidoma",
    "crepita": "crepitar",
    "crepitaba": "crepitar",
    "crepitabais": "crepitar",
    "crepitaban": "crepitar",
    "crepitabas": "crepitar",
    "crepitaciones": "crepitación",
    "crepitad": "crepitar",
    "crepitada": "crepitar",
    "crepitadas": "crepitar",
    "crepitado": "crepitar",
    "crepitados": "crepitar",
    "crepitamos": "crepitar",
    "crepitan": "crepitar",
    "crepitando": "crepitar",
    "crepitara": "crepitar",
    "crepitarais": "crepitar",
    "crepitaran": "crepitar",
    "crepitaras": "crepitar",
    "crepitare": "crepitar",
    "crepitareis": "crepitar",
    "crepitaremos": "crepitar",
    "crepitaren": "crepitar",
    "crepitares": "crepitar",
    "crepitaron": "crepitar",
    "crepitará": "crepitar",
    "crepitarán": "crepitar",
    "crepitarás": "crepitar",
    "crepitaré": "crepitar",
    "crepitaréis": "crepitar",
    "crepitaría": "crepitar",
    "crepitaríais": "crepitar",
    "crepitaríamos": "crepitar",
    "crepitarían": "crepitar",
    "crepitarías": "crepitar",
    "crepitas": "crepitar",
    "crepitase": "crepitar",
    "crepitaseis": "crepitar",
    "crepitasen": "crepitar",
    "crepitases": "crepitar",
    "crepitaste": "crepitar",
    "crepitasteis": "crepitar",
    "crepite": "crepitar",
    "crepitemos": "crepitar",
    "crepiten": "crepitar",
    "crepites": "crepitar",
    "crepito": "crepitar",
    "crepitábamos": "crepitar",
    "crepitáis": "crepitar",
    "crepitáramos": "crepitar",
    "crepitáremos": "crepitar",
    "crepitásemos": "crepitar",
    "crepité": "crepitar",
    "crepitéis": "crepitar",
    "crepitó": "crepitar",
    "crepusculares": "crepuscular",
    "crepés": "crepé",
    "crepúsculos": "crepúsculo",
    "cresas": "cresa",
    "crescendos": "crescendo",
    "cresoles": "cresol",
    "cresos": "creso",
    "cresotínica": "cresotínico",
    "cresotínicas": "cresotínico",
    "cresotínicos": "cresotínico",
    "crespa": "crespo",
    "crespas": "crespo",
    "crespinas": "crespina",
    "cresponada": "cresponado",
    "cresponadas": "cresponado",
    "cresponados": "cresponado",
    "crespones": "crespón",
    "crespos": "crespo",
    "crestada": "crestado",
    "crestadas": "crestado",
    "crestados": "crestado",
    "crestas": "cresta",
    "cresterías": "crestería",
    "crestomatías": "crestomatía",
    "crestones": "crestón",
    "crestuda": "crestudo",
    "crestudas": "crestudo",
    "crestudos": "crestudo",
    "cresílica": "cresílico",
    "cresílicas": "cresílico",
    "cresílicos": "cresílico",
    "cretas": "creta",
    "cretenses": "cretense",
    "cretina": "cretino",
    "cretinas": "cretino",
    "cretinismos": "cretinismo",
    "cretinos": "cretino",
    "cretonas": "cretona",
    "cretácea": "cretáceo",
    "cretáceas": "cretáceo",
    "cretáceos": "cretáceo",
    "cretácicos": "cretácico",
    "creyendo": "creer",
    "creyentes": "creyente",
    "creyera": "creer",
    "creyerais": "creer",
    "creyeran": "creer",
    "creyeras": "creer",
    "creyere": "creer",
    "creyereis": "creer",
    "creyeren": "creer",
    "creyeres": "creer",
    "creyeron": "creer",
    "creyese": "creer",
    "creyeseis": "creer",
    "creyesen": "creer",
    "creyeses": "creer",
    "creyéramos": "creer",
    "creyéremos": "creer",
    "creyésemos": "creer",
    "creyó": "creer",
    "crezca": "crecer",
    "crezcamos": "crecer",
    "crezcan": "crecer",
    "crezcas": "crecer",
    "crezco": "crecer",
    "crezcáis": "crecer",
    "creznejas": "crezneja",
    "creábamos": "crear",
    "creáis": "creer",
    "creáramos": "crear",
    "creáremos": "crear",
    "creásemos": "crear",
    "creé": "crear",
    "creéis": "creer",
    "creí": "creer",
    "creía": "creer",
    "creíais": "creer",
    "creíamos": "creer",
    "creían": "creer",
    "creías": "creer",
    "creíbles": "creíble",
    "creída": "creer",
    "creídas": "creer",
    "creído": "creer",
    "creídos": "creer",
    "creímos": "creer",
    "creíste": "creer",
    "creísteis": "creer",
    "creó": "crear",
    "criaba": "criar",
    "criabais": "criar",
    "criaban": "criar",
    "criabas": "criar",
    "criad": "criar",
    "criada": "criar",
    "criadas": "criar",
    "criadera": "criadero",
    "criaderas": "criadero",
    "criaderos": "criadero",
    "criadillas": "criadilla",
    "criado": "criar",
    "criadora": "criador",
    "criadoras": "criador",
    "criadores": "criador",
    "criados": "criar",
    "criamos": "criar",
    "criando": "criar",
    "crianzas": "crianza",
    "criara": "criar",
    "criarais": "criar",
    "criaran": "criar",
    "criaras": "criar",
    "criare": "criar",
    "criareis": "criar",
    "criaremos": "criar",
    "criaren": "criar",
    "criares": "criar",
    "criaron": "criar",
    "criará": "criar",
    "criarán": "criar",
    "criarás": "criar",
    "criaré": "criar",
    "criaréis": "criar",
    "criaría": "criar",
    "criaríais": "criar",
    "criaríamos": "criar",
    "criarían": "criar",
    "criarías": "criar",
    "criase": "criar",
    "criaseis": "criar",
    "criasen": "criar",
    "criases": "criar",
    "criaste": "criar",
    "criasteis": "criar",
    "criaturas": "criatura",
    "criba": "cribar",
    "cribaba": "cribar",
    "cribabais": "cribar",
    "cribaban": "cribar",
    "cribabas": "cribar",
    "cribad": "cribar",
    "cribada": "cribar",
    "cribadas": "cribar",
    "cribado": "cribar",
    "cribados": "cribar",
    "cribamos": "cribar",
    "criban": "cribar",
    "cribando": "cribar",
    "cribara": "cribar",
    "cribarais": "cribar",
    "cribaran": "cribar",
    "cribaras": "cribar",
    "cribare": "cribar",
    "cribareis": "cribar",
    "cribaremos": "cribar",
    "cribaren": "cribar",
    "cribares": "cribar",
    "cribaron": "cribar",
    "cribará": "cribar",
    "cribarán": "cribar",
    "cribarás": "cribar",
    "cribaré": "cribar",
    "cribaréis": "cribar",
    "cribaría": "cribar",
    "cribaríais": "cribar",
    "cribaríamos": "cribar",
    "cribarían": "cribar",
    "cribarías": "cribar",
    "cribas": "cribar",
    "cribase": "cribar",
    "cribaseis": "cribar",
    "cribasen": "cribar",
    "cribases": "cribar",
    "cribaste": "cribar",
    "cribasteis": "cribar",
    "cribe": "cribar",
    "cribelos": "cribelo",
    "cribemos": "cribar",
    "criben": "cribar",
    "cribes": "cribar",
    "cribiformes": "cribiforme",
    "cribo": "cribar",
    "cribosa": "criboso",
    "cribosas": "criboso",
    "cribosos": "criboso",
    "cribábamos": "cribar",
    "cribáis": "cribar",
    "cribáramos": "cribar",
    "cribáremos": "cribar",
    "cribásemos": "cribar",
    "cribé": "cribar",
    "cribéis": "cribar",
    "cribó": "cribar",
    "cricoaritenoidea": "cricoaritenoideo",
    "cricoaritenoideas": "cricoaritenoideo",
    "cricoaritenoideos": "cricoaritenoideo",
    "cricoesofágica": "cricoesofágico",
    "cricoesofágicas": "cricoesofágico",
    "cricoesofágicos": "cricoesofágico",
    "cricofaríngea": "cricofaríngeo",
    "cricofaríngeas": "cricofaríngeo",
    "cricofaríngeos": "cricofaríngeo",
    "cricotiroidea": "cricotiroideo",
    "cricotiroideas": "cricotiroideo",
    "cricotiroideos": "cricotiroideo",
    "cricotomías": "cricotomía",
    "cricotraqueales": "cricotraqueal",
    "cricovocales": "cricovocal",
    "crics": "cric",
    "criemos": "criar",
    "crimina": "criminar",
    "criminaba": "criminar",
    "criminabais": "criminar",
    "criminaban": "criminar",
    "criminabas": "criminar",
    "criminaciones": "criminación",
    "criminad": "criminar",
    "criminada": "criminar",
    "criminadas": "criminar",
    "criminado": "criminar",
    "criminados": "criminar",
    "criminales": "criminal",
    "criminalice": "criminalizar",
    "criminalicemos": "criminalizar",
    "criminalicen": "criminalizar",
    "criminalices": "criminalizar",
    "criminalicé": "criminalizar",
    "criminalicéis": "criminalizar",
    "criminalidades": "criminalidad",
    "criminalismos": "criminalismo",
    "criminalistas": "criminalista",
    "criminaliza": "criminalizar",
    "criminalizaba": "criminalizar",
    "criminalizabais": "criminalizar",
    "criminalizaban": "criminalizar",
    "criminalizabas": "criminalizar",
    "criminalizaciones": "criminalización",
    "criminalizad": "criminalizar",
    "criminalizada": "criminalizar",
    "criminalizadas": "criminalizar",
    "criminalizado": "criminalizar",
    "criminalizados": "criminalizar",
    "criminalizamos": "criminalizar",
    "criminalizan": "criminalizar",
    "criminalizando": "criminalizar",
    "criminalizara": "criminalizar",
    "criminalizarais": "criminalizar",
    "criminalizaran": "criminalizar",
    "criminalizaras": "criminalizar",
    "criminalizare": "criminalizar",
    "criminalizareis": "criminalizar",
    "criminalizaremos": "criminalizar",
    "criminalizaren": "criminalizar",
    "criminalizares": "criminalizar",
    "criminalizaron": "criminalizar",
    "criminalizará": "criminalizar",
    "criminalizarán": "criminalizar",
    "criminalizarás": "criminalizar",
    "criminalizaré": "criminalizar",
    "criminalizaréis": "criminalizar",
    "criminalizaría": "criminalizar",
    "criminalizaríais": "criminalizar",
    "criminalizaríamos": "criminalizar",
    "criminalizarían": "criminalizar",
    "criminalizarías": "criminalizar",
    "criminalizas": "criminalizar",
    "criminalizase": "criminalizar",
    "criminalizaseis": "criminalizar",
    "criminalizasen": "criminalizar",
    "criminalizases": "criminalizar",
    "criminalizaste": "criminalizar",
    "criminalizasteis": "criminalizar",
    "criminalizo": "criminalizar",
    "criminalizábamos": "criminalizar",
    "criminalizáis": "criminalizar",
    "criminalizáramos": "criminalizar",
    "criminalizáremos": "criminalizar",
    "criminalizásemos": "criminalizar",
    "criminalizó": "criminalizar",
    "criminalística": "criminalístico",
    "criminalísticas": "criminalístico",
    "criminalísticos": "criminalístico",
    "criminamos": "criminar",
    "criminan": "criminar",
    "criminando": "criminar",
    "criminara": "criminar",
    "criminarais": "criminar",
    "criminaran": "criminar",
    "criminaras": "criminar",
    "criminare": "criminar",
    "criminareis": "criminar",
    "criminaremos": "criminar",
    "criminaren": "criminar",
    "criminares": "criminar",
    "criminaron": "criminar",
    "criminará": "criminar",
    "criminarán": "criminar",
    "criminarás": "criminar",
    "criminaré": "criminar",
    "criminaréis": "criminar",
    "criminaría": "criminar",
    "criminaríais": "criminar",
    "criminaríamos": "criminar",
    "criminarían": "criminar",
    "criminarías": "criminar",
    "criminas": "criminar",
    "criminase": "criminar",
    "criminaseis": "criminar",
    "criminasen": "criminar",
    "criminases": "criminar",
    "criminaste": "criminar",
    "criminasteis": "criminar",
    "crimine": "criminar",
    "criminemos": "criminar",
    "criminen": "criminar",
    "crimines": "criminar",
    "crimino": "criminar",
    "criminologías": "criminología",
    "criminológica": "criminológico",
    "criminológicas": "criminológico",
    "criminológicos": "criminológico",
    "criminábamos": "criminar",
    "crimináis": "criminar",
    "crimináramos": "criminar",
    "crimináremos": "criminar",
    "criminásemos": "criminar",
    "criminé": "criminar",
    "criminéis": "criminar",
    "criminó": "criminar",
    "criminógena": "criminógeno",
    "criminógenas": "criminógeno",
    "criminógenos": "criminógeno",
    "criminóloga": "criminólogo",
    "criminólogas": "criminólogo",
    "criminólogos": "criminólogo",
    "crineras": "crinera",
    "crines": "crin",
    "crinofagias": "crinofagia",
    "crinoideos": "crinoideo",
    "crinolinas": "crinolina",
    "crinologías": "crinología",
    "crioaglutininas": "crioaglutinina",
    "crioaplicaciones": "crioaplicación",
    "criobiologías": "criobiología",
    "criobisturies": "criobisturí",
    "criobombas": "criobomba",
    "criocauterizaciones": "criocauterización",
    "criocauterizadores": "criocauterizador",
    "criocirugías": "criocirugía",
    "criocoagulaciones": "criocoagulación",
    "crioconservaciones": "crioconservación",
    "crioenfriamientos": "crioenfriamiento",
    "crioestabilizaciones": "crioestabilización",
    "crioextracciones": "crioextracción",
    "criofracturas": "criofractura",
    "criofílica": "criofílico",
    "criofílicas": "criofílico",
    "criofílicos": "criofílico",
    "criogena": "criogeno",
    "criogenas": "criogeno",
    "criogenias": "criogenia",
    "criogenos": "criogeno",
    "crioglobulinas": "crioglobulina",
    "crioglobulinemias": "crioglobulinemia",
    "criogénica": "criogénico",
    "criogénicas": "criogénico",
    "criogénicos": "criogénico",
    "criohidrática": "criohidrático",
    "criohidráticas": "criohidrático",
    "criohidráticos": "criohidrático",
    "crioinstrumentales": "crioinstrumental",
    "criointerceptores": "criointerceptor",
    "criolitas": "criolita",
    "criolla": "criollo",
    "criollas": "criollo",
    "criollismos": "criollismo",
    "criollizaciones": "criollización",
    "criollos": "criollo",
    "criologías": "criología",
    "crioluminiscencias": "crioluminiscencia",
    "criomagnetismos": "criomagnetismo",
    "criomedicinas": "criomedicina",
    "crionivales": "crionival",
    "crioprotectores": "crioprotector",
    "criorretinopexias": "criorretinopexia",
    "crioscopias": "crioscopia",
    "crioscópica": "crioscópico",
    "crioscópicas": "crioscópico",
    "crioscópicos": "crioscópico",
    "criosondas": "criosonda",
    "criostatos": "criostato",
    "criotemples": "criotemple",
    "crioterapias": "crioterapia",
    "criotratamientos": "criotratamiento",
    "criotrituraciones": "criotrituración",
    "criotrones": "criotrón",
    "crioturbaciones": "crioturbación",
    "crioválvulas": "crioválvula",
    "criptas": "cripta",
    "criptocristalina": "criptocristalino",
    "criptocristalinas": "criptocristalino",
    "criptocristalinos": "criptocristalino",
    "criptofitos": "criptofito",
    "criptofonías": "criptofonía",
    "criptoftalmías": "criptoftalmía",
    "criptogenética": "criptogenético",
    "criptogenéticas": "criptogenético",
    "criptogenéticos": "criptogenético",
    "criptografías": "criptografía",
    "criptogramas": "criptograma",
    "criptográfica": "criptográfico",
    "criptográficas": "criptográfico",
    "criptográficos": "criptográfico",
    "criptolalias": "criptolalia",
    "criptomerismos": "criptomerismo",
    "criptomeros": "criptomero",
    "criptones": "criptón",
    "criptopórticos": "criptopórtico",
    "criptotelefonías": "criptotelefonía",
    "criptógama": "criptógamo",
    "criptógamas": "criptógamo",
    "criptógamos": "criptógamo",
    "criptógrafa": "criptógrafo",
    "criptógrafas": "criptógrafo",
    "criptógrafos": "criptógrafo",
    "criptóloga": "criptólogo",
    "criptólogas": "criptólogo",
    "criptólogos": "criptólogo",
    "criptómetros": "criptómetro",
    "criquets": "criquet",
    "cris": "cri",
    "crisantemos": "crisantemo",
    "crisarrobinas": "crisarrobina",
    "criselefantina": "criselefantino",
    "criselefantinas": "criselefantino",
    "criselefantinos": "criselefantino",
    "crisenos": "criseno",
    "crismas": "crisma",
    "crismeras": "crismera",
    "crismones": "crismón",
    "crisnejas": "crisneja",
    "crisoberilos": "crisoberilo",
    "crisocolas": "crisocola",
    "crisofíceas": "crisofícea",
    "crisoladas": "crisolada",
    "crisoles": "crisol",
    "crisomonadales": "crisomonadal",
    "crisomélidos": "crisomélido",
    "crisopeyas": "crisopeya",
    "crisoprasas": "crisoprasa",
    "crisoterapias": "crisoterapia",
    "crisotilos": "crisotilo",
    "crispa": "crispar",
    "crispaba": "crispar",
    "crispabais": "crispar",
    "crispaban": "crispar",
    "crispabas": "crispar",
    "crispaciones": "crispación",
    "crispad": "crispar",
    "crispada": "crispar",
    "crispadas": "crispar",
    "crispado": "crispar",
    "crispadora": "crispador",
    "crispadoras": "crispador",
    "crispadores": "crispador",
    "crispados": "crispar",
    "crispaduras": "crispadura",
    "crispamientos": "crispamiento",
    "crispamos": "crispar",
    "crispan": "crispar",
    "crispando": "crispar",
    "crispara": "crispar",
    "crisparais": "crispar",
    "crisparan": "crispar",
    "crisparas": "crispar",
    "crispare": "crispar",
    "crispareis": "crispar",
    "crisparemos": "crispar",
    "crisparen": "crispar",
    "crispares": "crispar",
    "crisparon": "crispar",
    "crispará": "crispar",
    "crisparán": "crispar",
    "crisparás": "crispar",
    "crisparé": "crispar",
    "crisparéis": "crispar",
    "crisparía": "crispar",
    "crisparíais": "crispar",
    "crisparíamos": "crispar",
    "crisparían": "crispar",
    "crisparías": "crispar",
    "crispas": "crispar",
    "crispase": "crispar",
    "crispaseis": "crispar",
    "crispasen": "crispar",
    "crispases": "crispar",
    "crispaste": "crispar",
    "crispasteis": "crispar",
    "crispaturas": "crispatura",
    "crispe": "crispar",
    "crispemos": "crispar",
    "crispen": "crispar",
    "crispes": "crispar",
    "crispo": "crispar",
    "crispábamos": "crispar",
    "crispáis": "crispar",
    "crispáramos": "crispar",
    "crispáremos": "crispar",
    "crispásemos": "crispar",
    "crispé": "crispar",
    "crispéis": "crispar",
    "crispó": "crispar",
    "cristaleras": "cristalera",
    "cristaleros": "cristalero",
    "cristalerías": "cristalería",
    "cristales": "cristal",
    "cristalice": "cristalizar",
    "cristalicemos": "cristalizar",
    "cristalicen": "cristalizar",
    "cristalices": "cristalizar",
    "cristalicé": "cristalizar",
    "cristalicéis": "cristalizar",
    "cristalina": "cristalino",
    "cristalinas": "cristalino",
    "cristalinidades": "cristalinidad",
    "cristalinos": "cristalino",
    "cristalitas": "cristalita",
    "cristalitos": "cristalito",
    "cristaliza": "cristalizar",
    "cristalizaba": "cristalizar",
    "cristalizabais": "cristalizar",
    "cristalizaban": "cristalizar",
    "cristalizabas": "cristalizar",
    "cristalizaciones": "cristalización",
    "cristalizad": "cristalizar",
    "cristalizada": "cristalizar",
    "cristalizadas": "cristalizar",
    "cristalizado": "cristalizar",
    "cristalizadora": "cristalizador",
    "cristalizadoras": "cristalizador",
    "cristalizadores": "cristalizador",
    "cristalizados": "cristalizar",
    "cristalizamos": "cristalizar",
    "cristalizan": "cristalizar",
    "cristalizando": "cristalizar",
    "cristalizara": "cristalizar",
    "cristalizarais": "cristalizar",
    "cristalizaran": "cristalizar",
    "cristalizaras": "cristalizar",
    "cristalizare": "cristalizar",
    "cristalizareis": "cristalizar",
    "cristalizaremos": "cristalizar",
    "cristalizaren": "cristalizar",
    "cristalizares": "cristalizar",
    "cristalizaron": "cristalizar",
    "cristalizará": "cristalizar",
    "cristalizarán": "cristalizar",
    "cristalizarás": "cristalizar",
    "cristalizaré": "cristalizar",
    "cristalizaréis": "cristalizar",
    "cristalizaría": "cristalizar",
    "cristalizaríais": "cristalizar",
    "cristalizaríamos": "cristalizar",
    "cristalizarían": "cristalizar",
    "cristalizarías": "cristalizar",
    "cristalizas": "cristalizar",
    "cristalizase": "cristalizar",
    "cristalizaseis": "cristalizar",
    "cristalizasen": "cristalizar",
    "cristalizases": "cristalizar",
    "cristalizaste": "cristalizar",
    "cristalizasteis": "cristalizar",
    "cristalizo": "cristalizar",
    "cristalizábamos": "cristalizar",
    "cristalizáis": "cristalizar",
    "cristalizáramos": "cristalizar",
    "cristalizáremos": "cristalizar",
    "cristalizásemos": "cristalizar",
    "cristalizó": "cristalizar",
    "cristalofísicas": "cristalofísica",
    "cristalografías": "cristalografía",
    "cristalogramas": "cristalograma",
    "cristalográfica": "cristalográfico",
    "cristalográficas": "cristalográfico",
    "cristalográficos": "cristalográfico",
    "cristaloides": "cristaloide",
    "cristaloluminiscencias": "cristaloluminiscencia",
    "cristaloquímicas": "cristaloquímica",
    "cristalógrafa": "cristalógrafo",
    "cristalógrafas": "cristalógrafo",
    "cristalógrafos": "cristalógrafo",
    "cristiana": "cristiano",
    "cristianaba": "cristianar",
    "cristianabais": "cristianar",
    "cristianaban": "cristianar",
    "cristianabas": "cristianar",
    "cristianad": "cristianar",
    "cristianada": "cristianar",
    "cristianadas": "cristianar",
    "cristianado": "cristianar",
    "cristianados": "cristianar",
    "cristianamos": "cristianar",
    "cristianan": "cristianar",
    "cristianando": "cristianar",
    "cristianara": "cristianar",
    "cristianarais": "cristianar",
    "cristianaran": "cristianar",
    "cristianaras": "cristianar",
    "cristianare": "cristianar",
    "cristianareis": "cristianar",
    "cristianaremos": "cristianar",
    "cristianaren": "cristianar",
    "cristianares": "cristianar",
    "cristianaron": "cristianar",
    "cristianará": "cristianar",
    "cristianarán": "cristianar",
    "cristianarás": "cristianar",
    "cristianaré": "cristianar",
    "cristianaréis": "cristianar",
    "cristianaría": "cristianar",
    "cristianaríais": "cristianar",
    "cristianaríamos": "cristianar",
    "cristianarían": "cristianar",
    "cristianarías": "cristianar",
    "cristianas": "cristiano",
    "cristianase": "cristianar",
    "cristianaseis": "cristianar",
    "cristianasen": "cristianar",
    "cristianases": "cristianar",
    "cristianaste": "cristianar",
    "cristianasteis": "cristianar",
    "cristiandades": "cristiandad",
    "cristiane": "cristianar",
    "cristianemos": "cristianar",
    "cristianen": "cristianar",
    "cristianes": "cristianar",
    "cristianice": "cristianizar",
    "cristianicemos": "cristianizar",
    "cristianicen": "cristianizar",
    "cristianices": "cristianizar",
    "cristianicé": "cristianizar",
    "cristianicéis": "cristianizar",
    "cristianismos": "cristianismo",
    "cristianiza": "cristianizar",
    "cristianizaba": "cristianizar",
    "cristianizabais": "cristianizar",
    "cristianizaban": "cristianizar",
    "cristianizabas": "cristianizar",
    "cristianizaciones": "cristianización",
    "cristianizad": "cristianizar",
    "cristianizada": "cristianizar",
    "cristianizadas": "cristianizar",
    "cristianizado": "cristianizar",
    "cristianizados": "cristianizar",
    "cristianizamos": "cristianizar",
    "cristianizan": "cristianizar",
    "cristianizando": "cristianizar",
    "cristianizara": "cristianizar",
    "cristianizarais": "cristianizar",
    "cristianizaran": "cristianizar",
    "cristianizaras": "cristianizar",
    "cristianizare": "cristianizar",
    "cristianizareis": "cristianizar",
    "cristianizaremos": "cristianizar",
    "cristianizaren": "cristianizar",
    "cristianizares": "cristianizar",
    "cristianizaron": "cristianizar",
    "cristianizará": "cristianizar",
    "cristianizarán": "cristianizar",
    "cristianizarás": "cristianizar",
    "cristianizaré": "cristianizar",
    "cristianizaréis": "cristianizar",
    "cristianizaría": "cristianizar",
    "cristianizaríais": "cristianizar",
    "cristianizaríamos": "cristianizar",
    "cristianizarían": "cristianizar",
    "cristianizarías": "cristianizar",
    "cristianizas": "cristianizar",
    "cristianizase": "cristianizar",
    "cristianizaseis": "cristianizar",
    "cristianizasen": "cristianizar",
    "cristianizases": "cristianizar",
    "cristianizaste": "cristianizar",
    "cristianizasteis": "cristianizar",
    "cristianizo": "cristianizar",
    "cristianizábamos": "cristianizar",
    "cristianizáis": "cristianizar",
    "cristianizáramos": "cristianizar",
    "cristianizáremos": "cristianizar",
    "cristianizásemos": "cristianizar",
    "cristianizó": "cristianizar",
    "cristiano": "cristianar",
    "cristianodemócratas": "cristianodemócrata",
    "cristianos": "cristiano",
    "cristianábamos": "cristianar",
    "cristianáis": "cristianar",
    "cristianáramos": "cristianar",
    "cristianáremos": "cristianar",
    "cristianásemos": "cristianar",
    "cristiané": "cristianar",
    "cristianéis": "cristianar",
    "cristianó": "cristianar",
    "cristina": "cristino",
    "cristinas": "cristino",
    "cristinos": "cristino",
    "cristobalitas": "cristobalita",
    "cristologías": "cristología",
    "cristos": "cristo",
    "crisuelas": "crisuela",
    "crisálidas": "crisálida",
    "crisólitos": "crisólito",
    "criterios": "criterio",
    "critica": "criticar",
    "criticaba": "criticar",
    "criticabais": "criticar",
    "criticaban": "criticar",
    "criticabas": "criticar",
    "criticables": "criticable",
    "criticad": "criticar",
    "criticada": "criticar",
    "criticadas": "criticar",
    "criticado": "criticar",
    "criticadora": "criticador",
    "criticadoras": "criticador",
    "criticadores": "criticador",
    "criticados": "criticar",
    "criticamos": "criticar",
    "critican": "criticar",
    "criticando": "criticar",
    "criticara": "criticar",
    "criticarais": "criticar",
    "criticaran": "criticar",
    "criticaras": "criticar",
    "criticare": "criticar",
    "criticareis": "criticar",
    "criticaremos": "criticar",
    "criticaren": "criticar",
    "criticares": "criticar",
    "criticaron": "criticar",
    "criticará": "criticar",
    "criticarán": "criticar",
    "criticarás": "criticar",
    "criticaré": "criticar",
    "criticaréis": "criticar",
    "criticaría": "criticar",
    "criticaríais": "criticar",
    "criticaríamos": "criticar",
    "criticarían": "criticar",
    "criticarías": "criticar",
    "criticas": "criticar",
    "criticase": "criticar",
    "criticaseis": "criticar",
    "criticasen": "criticar",
    "criticases": "criticar",
    "criticaste": "criticar",
    "criticasteis": "criticar",
    "criticastros": "criticastro",
    "criticismos": "criticismo",
    "critico": "criticar",
    "criticona": "criticón",
    "criticonas": "criticón",
    "criticones": "criticón",
    "criticábamos": "criticar",
    "criticáis": "criticar",
    "criticáramos": "criticar",
    "criticáremos": "criticar",
    "criticásemos": "criticar",
    "criticó": "criticar",
    "criticónes": "criticón",
    "critique": "criticar",
    "critiquemos": "criticar",
    "critiquen": "criticar",
    "critiques": "criticar",
    "critiqué": "criticar",
    "critiquéis": "criticar",
    "criznejas": "crizneja",
    "criábamos": "criar",
    "criáis": "criar",
    "criáramos": "criar",
    "criáremos": "criar",
    "criásemos": "criar",
    "crié": "criar",
    "criéis": "criar",
    "crió": "criar",
    "criógenos": "criógeno",
    "criómetros": "criómetro",
    "croa": "croar",
    "croaba": "croar",
    "croabais": "croar",
    "croaban": "croar",
    "croabas": "croar",
    "croad": "croar",
    "croada": "croar",
    "croadas": "croar",
    "croado": "croar",
    "croados": "croar",
    "croamos": "croar",
    "croan": "croar",
    "croando": "croar",
    "croara": "croar",
    "croarais": "croar",
    "croaran": "croar",
    "croaras": "croar",
    "croare": "croar",
    "croareis": "croar",
    "croaremos": "croar",
    "croaren": "croar",
    "croares": "croar",
    "croaron": "croar",
    "croará": "croar",
    "croarán": "croar",
    "croarás": "croar",
    "croaré": "croar",
    "croaréis": "croar",
    "croaría": "croar",
    "croaríais": "croar",
    "croaríamos": "croar",
    "croarían": "croar",
    "croarías": "croar",
    "croas": "croar",
    "croase": "croar",
    "croaseis": "croar",
    "croasen": "croar",
    "croases": "croar",
    "croaste": "croar",
    "croasteis": "croar",
    "croatas": "croata",
    "crocantes": "crocante",
    "crocants": "crocant",
    "crochetages": "crochetage",
    "crochetes": "crochet",
    "crochets": "crochet",
    "crocidolitas": "crocidolita",
    "crocina": "crocino",
    "crocinas": "crocino",
    "crocinos": "crocino",
    "crocita": "crocitar",
    "crocitaba": "crocitar",
    "crocitabais": "crocitar",
    "crocitaban": "crocitar",
    "crocitabas": "crocitar",
    "crocitad": "crocitar",
    "crocitada": "crocitar",
    "crocitadas": "crocitar",
    "crocitado": "crocitar",
    "crocitados": "crocitar",
    "crocitamos": "crocitar",
    "crocitan": "crocitar",
    "crocitando": "crocitar",
    "crocitara": "crocitar",
    "crocitarais": "crocitar",
    "crocitaran": "crocitar",
    "crocitaras": "crocitar",
    "crocitare": "crocitar",
    "crocitareis": "crocitar",
    "crocitaremos": "crocitar",
    "crocitaren": "crocitar",
    "crocitares": "crocitar",
    "crocitaron": "crocitar",
    "crocitará": "crocitar",
    "crocitarán": "crocitar",
    "crocitarás": "crocitar",
    "crocitaré": "crocitar",
    "crocitaréis": "crocitar",
    "crocitaría": "crocitar",
    "crocitaríais": "crocitar",
    "crocitaríamos": "crocitar",
    "crocitarían": "crocitar",
    "crocitarías": "crocitar",
    "crocitas": "crocitar",
    "crocitase": "crocitar",
    "crocitaseis": "crocitar",
    "crocitasen": "crocitar",
    "crocitases": "crocitar",
    "crocitaste": "crocitar",
    "crocitasteis": "crocitar",
    "crocite": "crocitar",
    "crocitemos": "crocitar",
    "crociten": "crocitar",
    "crocites": "crocitar",
    "crocito": "crocitar",
    "crocitábamos": "crocitar",
    "crocitáis": "crocitar",
    "crocitáramos": "crocitar",
    "crocitáremos": "crocitar",
    "crocitásemos": "crocitar",
    "crocité": "crocitar",
    "crocitéis": "crocitar",
    "crocitó": "crocitar",
    "crocodílidos": "crocodílido",
    "crocoitas": "crocoita",
    "crocos": "croco",
    "croe": "croar",
    "croemos": "croar",
    "croen": "croar",
    "croes": "croar",
    "croissanterías": "croissantería",
    "croissantes": "croissant",
    "croissants": "croissant",
    "croles": "crol",
    "crolistas": "crolista",
    "croma": "cromar",
    "cromaba": "cromar",
    "cromabais": "cromar",
    "cromaban": "cromar",
    "cromabas": "cromar",
    "cromad": "cromar",
    "cromada": "cromar",
    "cromadas": "cromar",
    "cromado": "cromar",
    "cromados": "cromar",
    "cromamos": "cromar",
    "croman": "cromar",
    "cromando": "cromar",
    "cromara": "cromar",
    "cromarais": "cromar",
    "cromaran": "cromar",
    "cromaras": "cromar",
    "cromare": "cromar",
    "cromareis": "cromar",
    "cromaremos": "cromar",
    "cromaren": "cromar",
    "cromares": "cromar",
    "cromaron": "cromar",
    "cromará": "cromar",
    "cromarán": "cromar",
    "cromarás": "cromar",
    "cromaré": "cromar",
    "cromaréis": "cromar",
    "cromaría": "cromar",
    "cromaríais": "cromar",
    "cromaríamos": "cromar",
    "cromarían": "cromar",
    "cromarías": "cromar",
    "cromas": "cromar",
    "cromase": "cromar",
    "cromaseis": "cromar",
    "cromasen": "cromar",
    "cromases": "cromar",
    "cromaste": "cromar",
    "cromasteis": "cromar",
    "cromataciones": "cromatación",
    "cromaticidades": "cromaticidad",
    "cromatidios": "cromatidio",
    "cromatinas": "cromatina",
    "cromatismos": "cromatismo",
    "cromatoforos": "cromatoforo",
    "cromatografías": "cromatografía",
    "cromatogramas": "cromatograma",
    "cromatologías": "cromatología",
    "cromatometrías": "cromatometría",
    "cromatopsias": "cromatopsia",
    "cromatoptometrías": "cromatoptometría",
    "cromatoptómetros": "cromatoptómetro",
    "cromatos": "cromato",
    "cromatínica": "cromatínico",
    "cromatínicas": "cromatínico",
    "cromatínicos": "cromatínico",
    "cromatóforos": "cromatóforo",
    "cromatógena": "cromatógeno",
    "cromatógenas": "cromatógeno",
    "cromatógenos": "cromatógeno",
    "cromatógrafos": "cromatógrafo",
    "cromatómetros": "cromatómetro",
    "cromañones": "cromañón",
    "crome": "cromar",
    "cromemos": "cromar",
    "cromen": "cromar",
    "cromes": "cromar",
    "cromidios": "cromidio",
    "crominancias": "crominancia",
    "cromitas": "cromita",
    "cromizaciones": "cromización",
    "cromlechs": "cromlech",
    "cromo": "cromar",
    "cromocentros": "cromocentro",
    "cromocitos": "cromocito",
    "cromofobia": "cromofobio",
    "cromofobias": "cromofobio",
    "cromofobios": "cromofobio",
    "cromofílica": "cromofílico",
    "cromofílicas": "cromofílico",
    "cromofílicos": "cromofílico",
    "cromoglicatos": "cromoglicato",
    "cromogénica": "cromogénico",
    "cromogénicas": "cromogénico",
    "cromogénicos": "cromogénico",
    "cromoisomerismos": "cromoisomerismo",
    "cromolinas": "cromolina",
    "cromolitografías": "cromolitografía",
    "cromolitográfica": "cromolitográfico",
    "cromolitográficas": "cromolitográfico",
    "cromolitográficos": "cromolitográfico",
    "cromolitógrafa": "cromolitógrafo",
    "cromolitógrafas": "cromolitógrafo",
    "cromolitógrafos": "cromolitógrafo",
    "cromonemas": "cromonema",
    "cromoplastos": "cromoplasto",
    "cromoproteínas": "cromoproteína",
    "cromoptómetros": "cromoptómetro",
    "cromorretinografías": "cromorretinografía",
    "cromos": "cromo",
    "cromosa": "cromoso",
    "cromosas": "cromoso",
    "cromoscopias": "cromoscopia",
    "cromoscopios": "cromoscopio",
    "cromosferas": "cromosfera",
    "cromosomas": "cromosoma",
    "cromosominas": "cromosomina",
    "cromosomopatías": "cromosomopatía",
    "cromosos": "cromoso",
    "cromosómica": "cromosómico",
    "cromosómicas": "cromosómico",
    "cromosómicos": "cromosómico",
    "cromotipias": "cromotipia",
    "cromotipografías": "cromotipografía",
    "cromotipos": "cromotipo",
    "cromábamos": "cromar",
    "cromáis": "cromar",
    "cromáramos": "cromar",
    "cromáremos": "cromar",
    "cromásemos": "cromar",
    "cromática": "cromático",
    "cromáticas": "cromático",
    "cromáticos": "cromático",
    "cromátidas": "cromátida",
    "cromé": "cromar",
    "croméis": "cromar",
    "cromó": "cromar",
    "cromóforos": "cromóforo",
    "cromógena": "cromógeno",
    "cromógenas": "cromógeno",
    "cromógenos": "cromógeno",
    "cromómera": "cromómero",
    "cromómeras": "cromómero",
    "cromómeros": "cromómero",
    "cronicidades": "cronicidad",
    "cronistas": "cronista",
    "cronoescaladas": "cronoescalada",
    "cronoestratigráfica": "cronoestratigráfico",
    "cronoestratigráficas": "cronoestratigráfico",
    "cronoestratigráficos": "cronoestratigráfico",
    "cronofotografías": "cronofotografía",
    "cronografías": "cronografía",
    "cronogramas": "cronograma",
    "cronográfica": "cronográfico",
    "cronográficas": "cronográfico",
    "cronográficos": "cronográfico",
    "cronologistas": "cronologista",
    "cronologías": "cronología",
    "cronológica": "cronológico",
    "cronológicas": "cronológico",
    "cronológicos": "cronológico",
    "cronometra": "cronometrar",
    "cronometraba": "cronometrar",
    "cronometrabais": "cronometrar",
    "cronometraban": "cronometrar",
    "cronometrabas": "cronometrar",
    "cronometrad": "cronometrar",
    "cronometrada": "cronometrar",
    "cronometradas": "cronometrar",
    "cronometrado": "cronometrar",
    "cronometradora": "cronometrador",
    "cronometradora-jefe": "cronometrador-jefe",
    "cronometradoras": "cronometrador",
    "cronometradoras-jefe": "cronometrador-jefe",
    "cronometradores": "cronometrador",
    "cronometradores-jefe": "cronometrador-jefe",
    "cronometrados": "cronometrar",
    "cronometrajes": "cronometraje",
    "cronometramos": "cronometrar",
    "cronometran": "cronometrar",
    "cronometrando": "cronometrar",
    "cronometrara": "cronometrar",
    "cronometrarais": "cronometrar",
    "cronometraran": "cronometrar",
    "cronometraras": "cronometrar",
    "cronometrare": "cronometrar",
    "cronometrareis": "cronometrar",
    "cronometraremos": "cronometrar",
    "cronometraren": "cronometrar",
    "cronometrares": "cronometrar",
    "cronometraron": "cronometrar",
    "cronometrará": "cronometrar",
    "cronometrarán": "cronometrar",
    "cronometrarás": "cronometrar",
    "cronometraré": "cronometrar",
    "cronometraréis": "cronometrar",
    "cronometraría": "cronometrar",
    "cronometraríais": "cronometrar",
    "cronometraríamos": "cronometrar",
    "cronometrarían": "cronometrar",
    "cronometrarías": "cronometrar",
    "cronometras": "cronometrar",
    "cronometrase": "cronometrar",
    "cronometraseis": "cronometrar",
    "cronometrasen": "cronometrar",
    "cronometrases": "cronometrar",
    "cronometraste": "cronometrar",
    "cronometrasteis": "cronometrar",
    "cronometre": "cronometrar",
    "cronometremos": "cronometrar",
    "cronometren": "cronometrar",
    "cronometres": "cronometrar",
    "cronometro": "cronometrar",
    "cronometrábamos": "cronometrar",
    "cronometráis": "cronometrar",
    "cronometráramos": "cronometrar",
    "cronometráremos": "cronometrar",
    "cronometrásemos": "cronometrar",
    "cronometré": "cronometrar",
    "cronometréis": "cronometrar",
    "cronometrías": "cronometría",
    "cronometró": "cronometrar",
    "cronométrica": "cronométrico",
    "cronométricas": "cronométrico",
    "cronométricos": "cronométrico",
    "cronos": "crono",
    "cronosecuencias": "cronosecuencia",
    "cronotropismos": "cronotropismo",
    "cronística": "cronístico",
    "cronísticas": "cronístico",
    "cronísticos": "cronístico",
    "cronógrafa": "cronógrafo",
    "cronógrafas": "cronógrafo",
    "cronógrafos": "cronógrafo",
    "cronóloga": "cronólogo",
    "cronólogas": "cronólogo",
    "cronólogos": "cronólogo",
    "cronómetros": "cronómetro",
    "croo": "croar",
    "croques": "croque",
    "croquesitas": "croquesita",
    "croquetas": "croqueta",
    "croquetes": "croquet",
    "croquets": "croquet",
    "croscita": "croscitar",
    "croscitaba": "croscitar",
    "croscitabais": "croscitar",
    "croscitaban": "croscitar",
    "croscitabas": "croscitar",
    "croscitad": "croscitar",
    "croscitada": "croscitar",
    "croscitadas": "croscitar",
    "croscitado": "croscitar",
    "croscitados": "croscitar",
    "croscitamos": "croscitar",
    "croscitan": "croscitar",
    "croscitando": "croscitar",
    "croscitara": "croscitar",
    "croscitarais": "croscitar",
    "croscitaran": "croscitar",
    "croscitaras": "croscitar",
    "croscitare": "croscitar",
    "croscitareis": "croscitar",
    "croscitaremos": "croscitar",
    "croscitaren": "croscitar",
    "croscitares": "croscitar",
    "croscitaron": "croscitar",
    "croscitará": "croscitar",
    "croscitarán": "croscitar",
    "croscitarás": "croscitar",
    "croscitaré": "croscitar",
    "croscitaréis": "croscitar",
    "croscitaría": "croscitar",
    "croscitaríais": "croscitar",
    "croscitaríamos": "croscitar",
    "croscitarían": "croscitar",
    "croscitarías": "croscitar",
    "croscitas": "croscitar",
    "croscitase": "croscitar",
    "croscitaseis": "croscitar",
    "croscitasen": "croscitar",
    "croscitases": "croscitar",
    "croscitaste": "croscitar",
    "croscitasteis": "croscitar",
    "croscite": "croscitar",
    "croscitemos": "croscitar",
    "crosciten": "croscitar",
    "croscites": "croscitar",
    "croscito": "croscitar",
    "croscitábamos": "croscitar",
    "croscitáis": "croscitar",
    "croscitáramos": "croscitar",
    "croscitáremos": "croscitar",
    "croscitásemos": "croscitar",
    "croscité": "croscitar",
    "croscitéis": "croscitar",
    "croscitó": "croscitar",
    "croses": "cros",
    "cross-countries": "cross-country",
    "cross-countrys": "cross-country",
    "crosses": "cross",
    "crotamitones": "crotamitón",
    "crotonaldehídos": "crotonaldehído",
    "crotonatos": "crotonato",
    "crotones": "crotón",
    "crotora": "crotorar",
    "crotoraba": "crotorar",
    "crotorabais": "crotorar",
    "crotoraban": "crotorar",
    "crotorabas": "crotorar",
    "crotorad": "crotorar",
    "crotorada": "crotorar",
    "crotoradas": "crotorar",
    "crotorado": "crotorar",
    "crotorados": "crotorar",
    "crotoramos": "crotorar",
    "crotoran": "crotorar",
    "crotorando": "crotorar",
    "crotorara": "crotorar",
    "crotorarais": "crotorar",
    "crotoraran": "crotorar",
    "crotoraras": "crotorar",
    "crotorare": "crotorar",
    "crotorareis": "crotorar",
    "crotoraremos": "crotorar",
    "crotoraren": "crotorar",
    "crotorares": "crotorar",
    "crotoraron": "crotorar",
    "crotorará": "crotorar",
    "crotorarán": "crotorar",
    "crotorarás": "crotorar",
    "crotoraré": "crotorar",
    "crotoraréis": "crotorar",
    "crotoraría": "crotorar",
    "crotoraríais": "crotorar",
    "crotoraríamos": "crotorar",
    "crotorarían": "crotorar",
    "crotorarías": "crotorar",
    "crotoras": "crotorar",
    "crotorase": "crotorar",
    "crotoraseis": "crotorar",
    "crotorasen": "crotorar",
    "crotorases": "crotorar",
    "crotoraste": "crotorar",
    "crotorasteis": "crotorar",
    "crotore": "crotorar",
    "crotoremos": "crotorar",
    "crotoren": "crotorar",
    "crotores": "crotorar",
    "crotoro": "crotorar",
    "crotorábamos": "crotorar",
    "crotoráis": "crotorar",
    "crotoráramos": "crotorar",
    "crotoráremos": "crotorar",
    "crotorásemos": "crotorar",
    "crotoré": "crotorar",
    "crotoréis": "crotorar",
    "crotoró": "crotorar",
    "crotónica": "crotónico",
    "crotónicas": "crotónico",
    "crotónicos": "crotónico",
    "crowns": "crown",
    "crozas": "croza",
    "croábamos": "croar",
    "croáis": "croar",
    "croáramos": "croar",
    "croáremos": "croar",
    "croásemos": "croar",
    "croé": "croar",
    "croéis": "croar",
    "croó": "croar",
    "cruasanes": "cruasán",
    "cruce": "cruzar",
    "crucemos": "cruzar",
    "crucen": "cruzar",
    "cruceras": "crucera",
    "cruceros": "crucero",
    "crucerías": "crucería",
    "cruces": "cruzar",
    "crucetas": "cruceta",
    "cruciales": "crucial",
    "cruciferaria": "cruciferario",
    "cruciferarias": "cruciferario",
    "cruciferarios": "cruciferario",
    "crucifica": "crucificar",
    "crucificaba": "crucificar",
    "crucificabais": "crucificar",
    "crucificaban": "crucificar",
    "crucificabas": "crucificar",
    "crucificad": "crucificar",
    "crucificada": "crucificar",
    "crucificadas": "crucificar",
    "crucificado": "crucificar",
    "crucificados": "crucificar",
    "crucificamos": "crucificar",
    "crucifican": "crucificar",
    "crucificando": "crucificar",
    "crucificara": "crucificar",
    "crucificarais": "crucificar",
    "crucificaran": "crucificar",
    "crucificaras": "crucificar",
    "crucificare": "crucificar",
    "crucificareis": "crucificar",
    "crucificaremos": "crucificar",
    "crucificaren": "crucificar",
    "crucificares": "crucificar",
    "crucificaron": "crucificar",
    "crucificará": "crucificar",
    "crucificarán": "crucificar",
    "crucificarás": "crucificar",
    "crucificaré": "crucificar",
    "crucificaréis": "crucificar",
    "crucificaría": "crucificar",
    "crucificaríais": "crucificar",
    "crucificaríamos": "crucificar",
    "crucificarían": "crucificar",
    "crucificarías": "crucificar",
    "crucificas": "crucificar",
    "crucificase": "crucificar",
    "crucificaseis": "crucificar",
    "crucificasen": "crucificar",
    "crucificases": "crucificar",
    "crucificaste": "crucificar",
    "crucificasteis": "crucificar",
    "crucifico": "crucificar",
    "crucificábamos": "crucificar",
    "crucificáis": "crucificar",
    "crucificáramos": "crucificar",
    "crucificáremos": "crucificar",
    "crucificásemos": "crucificar",
    "crucificó": "crucificar",
    "crucifijos": "crucifijo",
    "crucifique": "crucificar",
    "crucifiquemos": "crucificar",
    "crucifiquen": "crucificar",
    "crucifiques": "crucificar",
    "crucifiqué": "crucificar",
    "crucifiquéis": "crucificar",
    "crucifixiones": "crucifixión",
    "crucifixora": "crucifixor",
    "crucifixoras": "crucifixor",
    "crucifixores": "crucifixor",
    "cruciformes": "cruciforme",
    "crucigramas": "crucigrama",
    "crucigramistas": "crucigramista",
    "crucé": "cruzar",
    "crucéis": "cruzar",
    "crucífera": "crucífero",
    "crucíferas": "crucífero",
    "crucíferos": "crucífero",
    "cruda": "crudo",
    "crudas": "crudo",
    "crudezas": "crudeza",
    "crudos": "crudo",
    "crueldades": "crueldad",
    "crueles": "cruel",
    "cruenta": "cruento",
    "cruentas": "cruento",
    "cruentos": "cruento",
    "cruja": "crujir",
    "crujamos": "crujir",
    "crujan": "crujir",
    "crujas": "crujir",
    "cruje": "crujir",
    "crujen": "crujir",
    "crujes": "crujir",
    "crujid": "crujir",
    "crujida": "crujir",
    "crujidas": "crujir",
    "crujido": "crujir",
    "crujidos": "crujir",
    "crujiendo": "crujir",
    "crujientes": "crujiente",
    "crujiera": "crujir",
    "crujierais": "crujir",
    "crujieran": "crujir",
    "crujieras": "crujir",
    "crujiere": "crujir",
    "crujiereis": "crujir",
    "crujieren": "crujir",
    "crujieres": "crujir",
    "crujieron": "crujir",
    "crujiese": "crujir",
    "crujieseis": "crujir",
    "crujiesen": "crujir",
    "crujieses": "crujir",
    "crujimos": "crujir",
    "crujiremos": "crujir",
    "crujirá": "crujir",
    "crujirán": "crujir",
    "crujirás": "crujir",
    "crujiré": "crujir",
    "crujiréis": "crujir",
    "crujiría": "crujir",
    "crujiríais": "crujir",
    "crujiríamos": "crujir",
    "crujirían": "crujir",
    "crujirías": "crujir",
    "crujiste": "crujir",
    "crujisteis": "crujir",
    "crujiéramos": "crujir",
    "crujiéremos": "crujir",
    "crujiésemos": "crujir",
    "crujió": "crujir",
    "crujo": "crujir",
    "crujáis": "crujir",
    "crují": "crujir",
    "crujía": "crujir",
    "crujíais": "crujir",
    "crujíamos": "crujir",
    "crujían": "crujir",
    "crujías": "crujir",
    "crujís": "crujir",
    "crunodos": "crunodo",
    "crupieres": "crupier",
    "crurales": "crural",
    "crustácea": "crustáceo",
    "crustáceas": "crustáceo",
    "crustáceos": "crustáceo",
    "cruza": "cruzar",
    "cruzaba": "cruzar",
    "cruzabais": "cruzar",
    "cruzaban": "cruzar",
    "cruzabas": "cruzar",
    "cruzad": "cruzar",
    "cruzada": "cruzar",
    "cruzadas": "cruzar",
    "cruzado": "cruzar",
    "cruzados": "cruzar",
    "cruzamientos": "cruzamiento",
    "cruzamos": "cruzar",
    "cruzan": "cruzar",
    "cruzando": "cruzar",
    "cruzara": "cruzar",
    "cruzarais": "cruzar",
    "cruzaran": "cruzar",
    "cruzaras": "cruzar",
    "cruzare": "cruzar",
    "cruzareis": "cruzar",
    "cruzaremos": "cruzar",
    "cruzaren": "cruzar",
    "cruzares": "cruzar",
    "cruzaron": "cruzar",
    "cruzará": "cruzar",
    "cruzarán": "cruzar",
    "cruzarás": "cruzar",
    "cruzaré": "cruzar",
    "cruzaréis": "cruzar",
    "cruzaría": "cruzar",
    "cruzaríais": "cruzar",
    "cruzaríamos": "cruzar",
    "cruzarían": "cruzar",
    "cruzarías": "cruzar",
    "cruzas": "cruzar",
    "cruzase": "cruzar",
    "cruzaseis": "cruzar",
    "cruzasen": "cruzar",
    "cruzases": "cruzar",
    "cruzaste": "cruzar",
    "cruzasteis": "cruzar",
    "cruzeiros": "cruzeiro",
    "cruzo": "cruzar",
    "cruzábamos": "cruzar",
    "cruzáis": "cruzar",
    "cruzáramos": "cruzar",
    "cruzáremos": "cruzar",
    "cruzásemos": "cruzar",
    "cruzó": "cruzar",
    "cráneo-encefálica": "cráneo-encefálico",
    "cráneo-encefálicas": "cráneo-encefálico",
    "cráneo-encefálicos": "cráneo-encefálico",
    "cráneos": "cráneo",
    "crápulas": "crápula",
    "cráteras": "crátera",
    "cráteres": "cráter",
    "créditos": "crédito",
    "crédula": "crédulo",
    "crédulas": "crédulo",
    "crédulos": "crédulo",
    "crética": "crético",
    "créticas": "crético",
    "créticos": "crético",
    "crêpes": "crêpe",
    "cría": "crío",
    "críalos": "críalo",
    "crían": "criar",
    "crías": "crío",
    "críe": "criar",
    "críen": "criar",
    "críes": "criar",
    "crímenes": "crimen",
    "crío": "criar",
    "críos": "crío",
    "críptica": "críptico",
    "crípticas": "críptico",
    "crípticos": "críptico",
    "crísidos": "crísido",
    "crítica": "crítico",
    "críticas": "crítico",
    "críticos": "crítico",
    "crómica": "crómico",
    "crómicas": "crómico",
    "crómicos": "crómico",
    "crónica": "crónico",
    "crónicas": "crónico",
    "crónicos": "crónico",
    "crónleches": "crónlech",
    "crótalos": "crótalo",
    "cta": "cuenta",
    "ctas": "cuenta",
    "cuacos": "cuaco",
    "cuadernas": "cuaderna",
    "cuadernillos": "cuadernillo",
    "cuadernos": "cuaderno",
    "cuadra": "cuadro",
    "cuadraba": "cuadrar",
    "cuadrabais": "cuadrar",
    "cuadraban": "cuadrar",
    "cuadrabas": "cuadrar",
    "cuadrables": "cuadrable",
    "cuadrad": "cuadrar",
    "cuadrada": "cuadrar",
    "cuadradas": "cuadrar",
    "cuadradillos": "cuadradillo",
    "cuadrado": "cuadrar",
    "cuadradores": "cuadrador",
    "cuadrados": "cuadrar",
    "cuadragenaria": "cuadragenario",
    "cuadragenarias": "cuadragenario",
    "cuadragenarios": "cuadragenario",
    "cuadragesimales": "cuadragesimal",
    "cuadragesimoctava": "48",
    "cuadragesimoctavas": "48",
    "cuadragesimoctavo": "48",
    "cuadragesimoctavos": "48",
    "cuadragesimonona": "49",
    "cuadragesimononas": "49",
    "cuadragesimonono": "49",
    "cuadragesimononos": "49",
    "cuadragesimonovena": "49",
    "cuadragesimonovenas": "49",
    "cuadragesimonoveno": "49",
    "cuadragesimonovenos": "49",
    "cuadragesimoprimera": "41",
    "cuadragesimoprimeras": "41",
    "cuadragesimoprimero": "41",
    "cuadragesimoprimeros": "41",
    "cuadragesimoquinta": "45",
    "cuadragesimoquintas": "45",
    "cuadragesimoquinto": "45",
    "cuadragesimoquintos": "45",
    "cuadragesimosegunda": "42",
    "cuadragesimosegundas": "42",
    "cuadragesimosegundo": "42",
    "cuadragesimosegundos": "42",
    "cuadragesimosexta": "46",
    "cuadragesimosextas": "46",
    "cuadragesimosexto": "46",
    "cuadragesimosextos": "46",
    "cuadragesimoséptima": "47",
    "cuadragesimoséptimas": "47",
    "cuadragesimoséptimo": "47",
    "cuadragesimoséptimos": "47",
    "cuadragesimotercera": "43",
    "cuadragesimoterceras": "43",
    "cuadragesimotercero": "43",
    "cuadragesimoterceros": "43",
    "cuadragésima": "40",
    "cuadragésimas": "cuadragésima",
    "cuadragésimo": "40",
    "cuadragésimos": "cuadragésimo",
    "cuadramos": "cuadrar",
    "cuadran": "cuadrar",
    "cuadrando": "cuadrar",
    "cuadrangulares": "cuadrangular",
    "cuadrantanopias": "cuadrantanopia",
    "cuadrantanopsias": "cuadrantanopsia",
    "cuadrantes": "cuadrante",
    "cuadrara": "cuadrar",
    "cuadrarais": "cuadrar",
    "cuadraran": "cuadrar",
    "cuadraras": "cuadrar",
    "cuadrare": "cuadrar",
    "cuadrareis": "cuadrar",
    "cuadraremos": "cuadrar",
    "cuadraren": "cuadrar",
    "cuadrares": "cuadrar",
    "cuadraron": "cuadrar",
    "cuadrará": "cuadrar",
    "cuadrarán": "cuadrar",
    "cuadrarás": "cuadrar",
    "cuadraré": "cuadrar",
    "cuadraréis": "cuadrar",
    "cuadraría": "cuadrar",
    "cuadraríais": "cuadrar",
    "cuadraríamos": "cuadrar",
    "cuadrarían": "cuadrar",
    "cuadrarías": "cuadrar",
    "cuadras": "cuadro",
    "cuadrase": "cuadrar",
    "cuadraseis": "cuadrar",
    "cuadrasen": "cuadrar",
    "cuadrases": "cuadrar",
    "cuadraste": "cuadrar",
    "cuadrasteis": "cuadrar",
    "cuadratines": "cuadratín",
    "cuadraturas": "cuadratura",
    "cuadre": "cuadrar",
    "cuadremos": "cuadrar",
    "cuadren": "cuadrar",
    "cuadres": "cuadrar",
    "cuadretes": "cuadrete",
    "cuadricenales": "cuadricenal",
    "cuadricula": "cuadricular",
    "cuadriculaba": "cuadricular",
    "cuadriculabais": "cuadricular",
    "cuadriculaban": "cuadricular",
    "cuadriculabas": "cuadricular",
    "cuadriculad": "cuadricular",
    "cuadriculada": "cuadricular",
    "cuadriculadas": "cuadricular",
    "cuadriculado": "cuadricular",
    "cuadriculados": "cuadricular",
    "cuadriculamos": "cuadricular",
    "cuadriculan": "cuadricular",
    "cuadriculando": "cuadricular",
    "cuadriculara": "cuadricular",
    "cuadricularais": "cuadricular",
    "cuadricularan": "cuadricular",
    "cuadricularas": "cuadricular",
    "cuadriculare": "cuadricular",
    "cuadriculareis": "cuadricular",
    "cuadricularemos": "cuadricular",
    "cuadricularen": "cuadricular",
    "cuadriculares": "cuadricular",
    "cuadricularon": "cuadricular",
    "cuadriculará": "cuadricular",
    "cuadricularán": "cuadricular",
    "cuadricularás": "cuadricular",
    "cuadricularé": "cuadricular",
    "cuadricularéis": "cuadricular",
    "cuadricularía": "cuadricular",
    "cuadricularíais": "cuadricular",
    "cuadricularíamos": "cuadricular",
    "cuadricularían": "cuadricular",
    "cuadricularías": "cuadricular",
    "cuadriculas": "cuadricular",
    "cuadriculase": "cuadricular",
    "cuadriculaseis": "cuadricular",
    "cuadriculasen": "cuadricular",
    "cuadriculases": "cuadricular",
    "cuadriculaste": "cuadricular",
    "cuadriculasteis": "cuadricular",
    "cuadricule": "cuadricular",
    "cuadriculemos": "cuadricular",
    "cuadriculen": "cuadricular",
    "cuadricules": "cuadricular",
    "cuadriculo": "cuadricular",
    "cuadriculábamos": "cuadricular",
    "cuadriculáis": "cuadricular",
    "cuadriculáramos": "cuadricular",
    "cuadriculáremos": "cuadricular",
    "cuadriculásemos": "cuadricular",
    "cuadriculé": "cuadricular",
    "cuadriculéis": "cuadricular",
    "cuadriculó": "cuadricular",
    "cuadrienales": "cuadrienal",
    "cuadrienios": "cuadrienio",
    "cuadrifilares": "cuadrifilar",
    "cuadriformes": "cuadriforme",
    "cuadrigas": "cuadriga",
    "cuadrigentésima": "cuadrigentésimo",
    "cuadrigentésimas": "cuadrigentésimo",
    "cuadrigentésimos": "cuadrigentésimo",
    "cuadriles": "cuadril",
    "cuadriliterales": "cuadriliteral",
    "cuadrillas": "cuadrilla",
    "cuadrillera": "cuadrillero",
    "cuadrilleras": "cuadrillero",
    "cuadrilleros": "cuadrillero",
    "cuadrilonga": "cuadrilongo",
    "cuadrilongas": "cuadrilongo",
    "cuadrilongos": "cuadrilongo",
    "cuadrilátera": "cuadrilátero",
    "cuadriláteras": "cuadrilátero",
    "cuadriláteros": "cuadrilátero",
    "cuadrilítera": "cuadrilítero",
    "cuadrilíteras": "cuadrilítero",
    "cuadrilíteros": "cuadrilítero",
    "cuadringentésima": "400",
    "cuadringentésimas": "400",
    "cuadringentésimo": "400",
    "cuadringentésimos": "400",
    "cuadrinomios": "cuadrinomio",
    "cuadripartida": "cuadripartido",
    "cuadripartidas": "cuadripartido",
    "cuadripartidos": "cuadripartido",
    "cuadriplica": "cuadriplicar",
    "cuadriplicaba": "cuadriplicar",
    "cuadriplicabais": "cuadriplicar",
    "cuadriplicaban": "cuadriplicar",
    "cuadriplicabas": "cuadriplicar",
    "cuadriplicaciones": "cuadriplicación",
    "cuadriplicad": "cuadriplicar",
    "cuadriplicada": "cuadriplicar",
    "cuadriplicadas": "cuadriplicar",
    "cuadriplicado": "cuadriplicar",
    "cuadriplicados": "cuadriplicar",
    "cuadriplicamos": "cuadriplicar",
    "cuadriplican": "cuadriplicar",
    "cuadriplicando": "cuadriplicar",
    "cuadriplicara": "cuadriplicar",
    "cuadriplicarais": "cuadriplicar",
    "cuadriplicaran": "cuadriplicar",
    "cuadriplicaras": "cuadriplicar",
    "cuadriplicare": "cuadriplicar",
    "cuadriplicareis": "cuadriplicar",
    "cuadriplicaremos": "cuadriplicar",
    "cuadriplicaren": "cuadriplicar",
    "cuadriplicares": "cuadriplicar",
    "cuadriplicaron": "cuadriplicar",
    "cuadriplicará": "cuadriplicar",
    "cuadriplicarán": "cuadriplicar",
    "cuadriplicarás": "cuadriplicar",
    "cuadriplicaré": "cuadriplicar",
    "cuadriplicaréis": "cuadriplicar",
    "cuadriplicaría": "cuadriplicar",
    "cuadriplicaríais": "cuadriplicar",
    "cuadriplicaríamos": "cuadriplicar",
    "cuadriplicarían": "cuadriplicar",
    "cuadriplicarías": "cuadriplicar",
    "cuadriplicas": "cuadriplicar",
    "cuadriplicase": "cuadriplicar",
    "cuadriplicaseis": "cuadriplicar",
    "cuadriplicasen": "cuadriplicar",
    "cuadriplicases": "cuadriplicar",
    "cuadriplicaste": "cuadriplicar",
    "cuadriplicasteis": "cuadriplicar",
    "cuadriplico": "cuadriplicar",
    "cuadriplicábamos": "cuadriplicar",
    "cuadriplicáis": "cuadriplicar",
    "cuadriplicáramos": "cuadriplicar",
    "cuadriplicáremos": "cuadriplicar",
    "cuadriplicásemos": "cuadriplicar",
    "cuadriplicó": "cuadriplicar",
    "cuadriplique": "cuadriplicar",
    "cuadripliquemos": "cuadriplicar",
    "cuadripliquen": "cuadriplicar",
    "cuadripliques": "cuadriplicar",
    "cuadripliqué": "cuadriplicar",
    "cuadripliquéis": "cuadriplicar",
    "cuadripolos": "cuadripolo",
    "cuadrisílaba": "cuadrisílabo",
    "cuadrisílabas": "cuadrisílabo",
    "cuadrisílabos": "cuadrisílabo",
    "cuadrivalentes": "cuadrivalente",
    "cuadrivios": "cuadrivio",
    "cuadro": "cuadrar",
    "cuadros": "cuadro",
    "cuadrumana": "cuadrúmano",
    "cuadrumanas": "cuadrúmano",
    "cuadrumanos": "cuadrúmano",
    "cuadrupedales": "cuadrupedal",
    "cuadrupletes": "cuadruplete",
    "cuadruplica": "cuadruplicar",
    "cuadruplicaba": "cuadruplicar",
    "cuadruplicabais": "cuadruplicar",
    "cuadruplicaban": "cuadruplicar",
    "cuadruplicabas": "cuadruplicar",
    "cuadruplicaciones": "cuadruplicación",
    "cuadruplicad": "cuadruplicar",
    "cuadruplicada": "cuadruplicar",
    "cuadruplicadas": "cuadruplicar",
    "cuadruplicado": "cuadruplicar",
    "cuadruplicados": "cuadruplicar",
    "cuadruplicamos": "cuadruplicar",
    "cuadruplican": "cuadruplicar",
    "cuadruplicando": "cuadruplicar",
    "cuadruplicara": "cuadruplicar",
    "cuadruplicarais": "cuadruplicar",
    "cuadruplicaran": "cuadruplicar",
    "cuadruplicaras": "cuadruplicar",
    "cuadruplicare": "cuadruplicar",
    "cuadruplicareis": "cuadruplicar",
    "cuadruplicaremos": "cuadruplicar",
    "cuadruplicaren": "cuadruplicar",
    "cuadruplicares": "cuadruplicar",
    "cuadruplicaron": "cuadruplicar",
    "cuadruplicará": "cuadruplicar",
    "cuadruplicarán": "cuadruplicar",
    "cuadruplicarás": "cuadruplicar",
    "cuadruplicaré": "cuadruplicar",
    "cuadruplicaréis": "cuadruplicar",
    "cuadruplicaría": "cuadruplicar",
    "cuadruplicaríais": "cuadruplicar",
    "cuadruplicaríamos": "cuadruplicar",
    "cuadruplicarían": "cuadruplicar",
    "cuadruplicarías": "cuadruplicar",
    "cuadruplicas": "cuadruplicar",
    "cuadruplicase": "cuadruplicar",
    "cuadruplicaseis": "cuadruplicar",
    "cuadruplicasen": "cuadruplicar",
    "cuadruplicases": "cuadruplicar",
    "cuadruplicaste": "cuadruplicar",
    "cuadruplicasteis": "cuadruplicar",
    "cuadruplico": "cuadruplicar",
    "cuadruplicábamos": "cuadruplicar",
    "cuadruplicáis": "cuadruplicar",
    "cuadruplicáramos": "cuadruplicar",
    "cuadruplicáremos": "cuadruplicar",
    "cuadruplicásemos": "cuadruplicar",
    "cuadruplicó": "cuadruplicar",
    "cuadruplique": "cuadruplicar",
    "cuadrupliquemos": "cuadruplicar",
    "cuadrupliquen": "cuadruplicar",
    "cuadrupliques": "cuadruplicar",
    "cuadrupliqué": "cuadruplicar",
    "cuadrupliquéis": "cuadruplicar",
    "cuadrábamos": "cuadrar",
    "cuadráis": "cuadrar",
    "cuadrángulos": "cuadrángulo",
    "cuadráramos": "cuadrar",
    "cuadráremos": "cuadrar",
    "cuadrásemos": "cuadrar",
    "cuadrática": "cuadrático",
    "cuadráticas": "cuadrático",
    "cuadráticos": "cuadrático",
    "cuadré": "cuadrar",
    "cuadréis": "cuadrar",
    "cuadrículas": "cuadrícula",
    "cuadró": "cuadrar",
    "cuadrúmana": "cuadrúmano",
    "cuadrúmanas": "cuadrúmano",
    "cuadrúmanos": "cuadrúmano",
    "cuadrúpedos": "cuadrúpedo",
    "cuaja": "cuajar",
    "cuajaba": "cuajar",
    "cuajabais": "cuajar",
    "cuajaban": "cuajar",
    "cuajabas": "cuajar",
    "cuajad": "cuajar",
    "cuajada": "cuajar",
    "cuajadas": "cuajar",
    "cuajadillos": "cuajadillo",
    "cuajado": "cuajar",
    "cuajados": "cuajar",
    "cuajaduras": "cuajadura",
    "cuajaleches": "cuajaleche",
    "cuajamos": "cuajar",
    "cuajan": "cuajar",
    "cuajando": "cuajar",
    "cuajara": "cuajar",
    "cuajarais": "cuajar",
    "cuajaran": "cuajar",
    "cuajaras": "cuajar",
    "cuajare": "cuajar",
    "cuajareis": "cuajar",
    "cuajaremos": "cuajar",
    "cuajaren": "cuajar",
    "cuajares": "cuajar",
    "cuajaron": "cuajar",
    "cuajarones": "cuajarón",
    "cuajará": "cuajar",
    "cuajarán": "cuajar",
    "cuajarás": "cuajar",
    "cuajaré": "cuajar",
    "cuajaréis": "cuajar",
    "cuajaría": "cuajar",
    "cuajaríais": "cuajar",
    "cuajaríamos": "cuajar",
    "cuajarían": "cuajar",
    "cuajarías": "cuajar",
    "cuajas": "cuajar",
    "cuajase": "cuajar",
    "cuajaseis": "cuajar",
    "cuajasen": "cuajar",
    "cuajases": "cuajar",
    "cuajaste": "cuajar",
    "cuajasteis": "cuajar",
    "cuaje": "cuajar",
    "cuajemos": "cuajar",
    "cuajen": "cuajar",
    "cuajes": "cuajar",
    "cuajilotes": "cuajilote",
    "cuajo": "cuajar",
    "cuajos": "cuajo",
    "cuajábamos": "cuajar",
    "cuajáis": "cuajar",
    "cuajáramos": "cuajar",
    "cuajáremos": "cuajar",
    "cuajásemos": "cuajar",
    "cuajé": "cuajar",
    "cuajéis": "cuajar",
    "cuajó": "cuajar",
    "cuakerismos": "cuakerismo",
    "cuales": "cual",
    "cualesquier": "cualquiera",
    "cualesquiera": "cualquiera",
    "cualidades": "cualidad",
    "cualifica": "cualificar",
    "cualificaba": "cualificar",
    "cualificabais": "cualificar",
    "cualificaban": "cualificar",
    "cualificabas": "cualificar",
    "cualificaciones": "cualificación",
    "cualificad": "cualificar",
    "cualificada": "cualificar",
    "cualificadas": "cualificar",
    "cualificado": "cualificar",
    "cualificadora": "cualificador",
    "cualificadoras": "cualificador",
    "cualificadores": "cualificador",
    "cualificados": "cualificar",
    "cualificamos": "cualificar",
    "cualifican": "cualificar",
    "cualificando": "cualificar",
    "cualificantes": "cualificante",
    "cualificara": "cualificar",
    "cualificarais": "cualificar",
    "cualificaran": "cualificar",
    "cualificaras": "cualificar",
    "cualificare": "cualificar",
    "cualificareis": "cualificar",
    "cualificaremos": "cualificar",
    "cualificaren": "cualificar",
    "cualificares": "cualificar",
    "cualificaron": "cualificar",
    "cualificará": "cualificar",
    "cualificarán": "cualificar",
    "cualificarás": "cualificar",
    "cualificaré": "cualificar",
    "cualificaréis": "cualificar",
    "cualificaría": "cualificar",
    "cualificaríais": "cualificar",
    "cualificaríamos": "cualificar",
    "cualificarían": "cualificar",
    "cualificarías": "cualificar",
    "cualificas": "cualificar",
    "cualificase": "cualificar",
    "cualificaseis": "cualificar",
    "cualificasen": "cualificar",
    "cualificases": "cualificar",
    "cualificaste": "cualificar",
    "cualificasteis": "cualificar",
    "cualifico": "cualificar",
    "cualificábamos": "cualificar",
    "cualificáis": "cualificar",
    "cualificáramos": "cualificar",
    "cualificáremos": "cualificar",
    "cualificásemos": "cualificar",
    "cualificó": "cualificar",
    "cualifique": "cualificar",
    "cualifiquemos": "cualificar",
    "cualifiquen": "cualificar",
    "cualifiques": "cualificar",
    "cualifiqué": "cualificar",
    "cualifiquéis": "cualificar",
    "cualitativa": "cualitativo",
    "cualitativas": "cualitativo",
    "cualitativos": "cualitativo",
    "cualquier": "cualquiera",
    "cuandrángula": "cuandrángulo",
    "cuandrángulas": "cuandrángulo",
    "cuandrángulos": "cuandrángulo",
    "cuanta": "cuanto",
    "cuantas": "cuanto",
    "cuantiaba": "cuantiar",
    "cuantiabais": "cuantiar",
    "cuantiaban": "cuantiar",
    "cuantiabas": "cuantiar",
    "cuantiad": "cuantiar",
    "cuantiada": "cuantiar",
    "cuantiadas": "cuantiar",
    "cuantiado": "cuantiar",
    "cuantiados": "cuantiar",
    "cuantiamos": "cuantiar",
    "cuantiando": "cuantiar",
    "cuantiara": "cuantiar",
    "cuantiarais": "cuantiar",
    "cuantiaran": "cuantiar",
    "cuantiaras": "cuantiar",
    "cuantiare": "cuantiar",
    "cuantiareis": "cuantiar",
    "cuantiaremos": "cuantiar",
    "cuantiaren": "cuantiar",
    "cuantiares": "cuantiar",
    "cuantiaron": "cuantiar",
    "cuantiará": "cuantiar",
    "cuantiarán": "cuantiar",
    "cuantiarás": "cuantiar",
    "cuantiaré": "cuantiar",
    "cuantiaréis": "cuantiar",
    "cuantiaría": "cuantiar",
    "cuantiaríais": "cuantiar",
    "cuantiaríamos": "cuantiar",
    "cuantiarían": "cuantiar",
    "cuantiarías": "cuantiar",
    "cuantiase": "cuantiar",
    "cuantiaseis": "cuantiar",
    "cuantiasen": "cuantiar",
    "cuantiases": "cuantiar",
    "cuantiaste": "cuantiar",
    "cuantiasteis": "cuantiar",
    "cuantidades": "cuantidad",
    "cuantiemos": "cuantiar",
    "cuantifica": "cuantificar",
    "cuantificaba": "cuantificar",
    "cuantificabais": "cuantificar",
    "cuantificaban": "cuantificar",
    "cuantificabas": "cuantificar",
    "cuantificables": "cuantificable",
    "cuantificaciones": "cuantificación",
    "cuantificad": "cuantificar",
    "cuantificada": "cuantificar",
    "cuantificadas": "cuantificar",
    "cuantificado": "cuantificar",
    "cuantificadores": "cuantificador",
    "cuantificados": "cuantificar",
    "cuantificamos": "cuantificar",
    "cuantifican": "cuantificar",
    "cuantificando": "cuantificar",
    "cuantificara": "cuantificar",
    "cuantificarais": "cuantificar",
    "cuantificaran": "cuantificar",
    "cuantificaras": "cuantificar",
    "cuantificare": "cuantificar",
    "cuantificareis": "cuantificar",
    "cuantificaremos": "cuantificar",
    "cuantificaren": "cuantificar",
    "cuantificares": "cuantificar",
    "cuantificaron": "cuantificar",
    "cuantificará": "cuantificar",
    "cuantificarán": "cuantificar",
    "cuantificarás": "cuantificar",
    "cuantificaré": "cuantificar",
    "cuantificaréis": "cuantificar",
    "cuantificaría": "cuantificar",
    "cuantificaríais": "cuantificar",
    "cuantificaríamos": "cuantificar",
    "cuantificarían": "cuantificar",
    "cuantificarías": "cuantificar",
    "cuantificas": "cuantificar",
    "cuantificase": "cuantificar",
    "cuantificaseis": "cuantificar",
    "cuantificasen": "cuantificar",
    "cuantificases": "cuantificar",
    "cuantificaste": "cuantificar",
    "cuantificasteis": "cuantificar",
    "cuantifico": "cuantificar",
    "cuantificábamos": "cuantificar",
    "cuantificáis": "cuantificar",
    "cuantificáramos": "cuantificar",
    "cuantificáremos": "cuantificar",
    "cuantificásemos": "cuantificar",
    "cuantificó": "cuantificar",
    "cuantifique": "cuantificar",
    "cuantifiquemos": "cuantificar",
    "cuantifiquen": "cuantificar",
    "cuantifiques": "cuantificar",
    "cuantifiqué": "cuantificar",
    "cuantifiquéis": "cuantificar",
    "cuantiosa": "cuantioso",
    "cuantiosas": "cuantioso",
    "cuantiosos": "cuantioso",
    "cuantitativa": "cuantitativo",
    "cuantitativas": "cuantitativo",
    "cuantitativos": "cuantitativo",
    "cuantizaciones": "cuantización",
    "cuantiábamos": "cuantiar",
    "cuantiáis": "cuantiar",
    "cuantiáramos": "cuantiar",
    "cuantiáremos": "cuantiar",
    "cuantiásemos": "cuantiar",
    "cuantié": "cuantiar",
    "cuantiéis": "cuantiar",
    "cuantió": "cuantiar",
    "cuantos": "cuanto",
    "cuantía": "cuantiar",
    "cuantían": "cuantiar",
    "cuantías": "cuantiar",
    "cuantíe": "cuantiar",
    "cuantíen": "cuantiar",
    "cuantíes": "cuantiar",
    "cuantío": "cuantiar",
    "cuaquerismos": "cuaquerismo",
    "cuarangos": "cuarango",
    "cuarcitas": "cuarcita",
    "cuarcífera": "cuarcífero",
    "cuarcíferas": "cuarcífero",
    "cuarcíferos": "cuarcífero",
    "cuarcítica": "cuarcítico",
    "cuarcíticas": "cuarcítico",
    "cuarcíticos": "cuarcítico",
    "cuarentaicincoava": "cuarentaicincoavo",
    "cuarentaicincoavas": "cuarentaicincoavo",
    "cuarentaicincoavo": "45",
    "cuarentaicincoavos": "cuarentaicincoavo",
    "cuarentaicuatroava": "cuarentaicuatroavo",
    "cuarentaicuatroavas": "cuarentaicuatroavo",
    "cuarentaicuatroavo": "44",
    "cuarentaicuatroavos": "cuarentaicuatroavo",
    "cuarentaidosava": "cuarentaidosavo",
    "cuarentaidosavas": "cuarentaidosavo",
    "cuarentaidosavo": "42",
    "cuarentaidosavos": "cuarentaidosavo",
    "cuarentainueveava": "cuarentainueveavo",
    "cuarentainueveavas": "cuarentainueveavo",
    "cuarentainueveavo": "49",
    "cuarentainueveavos": "cuarentainueveavo",
    "cuarentaiochava": "cuarentaiochoavo",
    "cuarentaiochavas": "cuarentaiochoavo",
    "cuarentaiochavo": "cuarentaiochoavo",
    "cuarentaiochavos": "cuarentaiochoavo",
    "cuarentaiochoava": "cuarentaiochoavo",
    "cuarentaiochoavas": "cuarentaiochoavo",
    "cuarentaiochoavo": "48",
    "cuarentaiochoavos": "cuarentaiochoavo",
    "cuarentaiseisava": "cuarentaiseisavo",
    "cuarentaiseisavas": "cuarentaiseisavo",
    "cuarentaiseisavo": "46",
    "cuarentaiseisavos": "cuarentaiseisavo",
    "cuarentaisieteava": "cuarentaisieteavo",
    "cuarentaisieteavas": "cuarentaisieteavo",
    "cuarentaisieteavo": "47",
    "cuarentaisieteavos": "cuarentaisieteavo",
    "cuarentaitresava": "cuarentaitresavo",
    "cuarentaitresavas": "cuarentaitresavo",
    "cuarentaitresavo": "43",
    "cuarentaitresavos": "cuarentaitresavo",
    "cuarentaiunava": "cuarentaiunavo",
    "cuarentaiunavas": "cuarentaiunavo",
    "cuarentaiunavo": "41",
    "cuarentaiunavos": "cuarentaiunavo",
    "cuarentava": "cuarentavo",
    "cuarentavas": "cuarentavo",
    "cuarentavo": "40",
    "cuarentavos": "cuarentavo",
    "cuarentenas": "cuarentena",
    "cuarentona": "cuarentón",
    "cuarentonas": "cuarentón",
    "cuarentones": "cuarentón",
    "cuaresmales": "cuaresmal",
    "cuaresmas": "cuaresma",
    "cuarta": "cuartar",
    "cuartaba": "cuartar",
    "cuartabais": "cuartar",
    "cuartaban": "cuartar",
    "cuartabas": "cuartar",
    "cuartad": "cuartar",
    "cuartada": "cuartar",
    "cuartadas": "cuartar",
    "cuartado": "cuartar",
    "cuartados": "cuartar",
    "cuartagos": "cuartago",
    "cuartamos": "cuartar",
    "cuartan": "cuartar",
    "cuartanas": "cuartana",
    "cuartando": "cuartar",
    "cuartara": "cuartar",
    "cuartarais": "cuartar",
    "cuartaran": "cuartar",
    "cuartaras": "cuartar",
    "cuartare": "cuartar",
    "cuartareis": "cuartar",
    "cuartaremos": "cuartar",
    "cuartaren": "cuartar",
    "cuartares": "cuartar",
    "cuartaron": "cuartar",
    "cuartará": "cuartar",
    "cuartarán": "cuartar",
    "cuartarás": "cuartar",
    "cuartaré": "cuartar",
    "cuartaréis": "cuartar",
    "cuartaría": "cuartar",
    "cuartaríais": "cuartar",
    "cuartaríamos": "cuartar",
    "cuartarían": "cuartar",
    "cuartarías": "cuartar",
    "cuartas": "cuartar",
    "cuartase": "cuartar",
    "cuartaseis": "cuartar",
    "cuartasen": "cuartar",
    "cuartases": "cuartar",
    "cuartaste": "cuartar",
    "cuartasteis": "cuartar",
    "cuarte": "cuartar",
    "cuartea": "cuartear",
    "cuarteaba": "cuartear",
    "cuarteabais": "cuartear",
    "cuarteaban": "cuartear",
    "cuarteabas": "cuartear",
    "cuartead": "cuartear",
    "cuarteada": "cuartear",
    "cuarteadas": "cuartear",
    "cuarteado": "cuartear",
    "cuarteados": "cuartear",
    "cuarteamos": "cuartear",
    "cuartean": "cuartear",
    "cuarteando": "cuartear",
    "cuarteara": "cuartear",
    "cuartearais": "cuartear",
    "cuartearan": "cuartear",
    "cuartearas": "cuartear",
    "cuarteare": "cuartear",
    "cuarteareis": "cuartear",
    "cuartearemos": "cuartear",
    "cuartearen": "cuartear",
    "cuarteares": "cuartear",
    "cuartearon": "cuartear",
    "cuarteará": "cuartear",
    "cuartearán": "cuartear",
    "cuartearás": "cuartear",
    "cuartearé": "cuartear",
    "cuartearéis": "cuartear",
    "cuartearía": "cuartear",
    "cuartearíais": "cuartear",
    "cuartearíamos": "cuartear",
    "cuartearían": "cuartear",
    "cuartearías": "cuartear",
    "cuarteas": "cuartear",
    "cuartease": "cuartear",
    "cuarteaseis": "cuartear",
    "cuarteasen": "cuartear",
    "cuarteases": "cuartear",
    "cuarteaste": "cuartear",
    "cuarteasteis": "cuartear",
    "cuartee": "cuartear",
    "cuarteemos": "cuartear",
    "cuarteen": "cuartear",
    "cuartees": "cuartear",
    "cuartela": "cuartelar",
    "cuartelaba": "cuartelar",
    "cuartelabais": "cuartelar",
    "cuartelaban": "cuartelar",
    "cuartelabas": "cuartelar",
    "cuartelad": "cuartelar",
    "cuartelada": "cuartelar",
    "cuarteladas": "cuartelar",
    "cuartelado": "cuartelar",
    "cuartelados": "cuartelar",
    "cuartelamos": "cuartelar",
    "cuartelan": "cuartelar",
    "cuartelando": "cuartelar",
    "cuartelara": "cuartelar",
    "cuartelarais": "cuartelar",
    "cuartelaran": "cuartelar",
    "cuartelaras": "cuartelar",
    "cuartelare": "cuartelar",
    "cuartelareis": "cuartelar",
    "cuartelaremos": "cuartelar",
    "cuartelaren": "cuartelar",
    "cuartelares": "cuartelar",
    "cuartelaron": "cuartelar",
    "cuartelará": "cuartelar",
    "cuartelarán": "cuartelar",
    "cuartelarás": "cuartelar",
    "cuartelaré": "cuartelar",
    "cuartelaréis": "cuartelar",
    "cuartelaría": "cuartelar",
    "cuartelaríais": "cuartelar",
    "cuartelaríamos": "cuartelar",
    "cuartelarían": "cuartelar",
    "cuartelarías": "cuartelar",
    "cuartelas": "cuartelar",
    "cuartelase": "cuartelar",
    "cuartelaseis": "cuartelar",
    "cuartelasen": "cuartelar",
    "cuartelases": "cuartelar",
    "cuartelaste": "cuartelar",
    "cuartelasteis": "cuartelar",
    "cuartele": "cuartelar",
    "cuartelemos": "cuartelar",
    "cuartelen": "cuartelar",
    "cuartelera": "cuartelero",
    "cuarteleras": "cuartelero",
    "cuarteleros": "cuartelero",
    "cuarteles": "cuartelar",
    "cuartelillos": "cuartelillo",
    "cuartelo": "cuartelar",
    "cuartelábamos": "cuartelar",
    "cuarteláis": "cuartelar",
    "cuarteláramos": "cuartelar",
    "cuarteláremos": "cuartelar",
    "cuartelásemos": "cuartelar",
    "cuartelé": "cuartelar",
    "cuarteléis": "cuartelar",
    "cuarteló": "cuartelar",
    "cuartemos": "cuartar",
    "cuarten": "cuartar",
    "cuarteo": "cuartear",
    "cuarteos": "cuarteo",
    "cuarterolas": "cuarterola",
    "cuarterona": "cuarterón",
    "cuarteronas": "cuarterón",
    "cuarterones": "cuarterón",
    "cuartes": "cuartar",
    "cuartetas": "cuarteta",
    "cuartetos": "cuarteto",
    "cuarteábamos": "cuartear",
    "cuarteáis": "cuartear",
    "cuarteáramos": "cuartear",
    "cuarteáremos": "cuartear",
    "cuarteásemos": "cuartear",
    "cuarteé": "cuartear",
    "cuarteéis": "cuartear",
    "cuarteó": "cuartear",
    "cuartiles": "cuartil",
    "cuartillas": "cuartilla",
    "cuartillera": "cuartillero",
    "cuartilleras": "cuartillero",
    "cuartilleros": "cuartillero",
    "cuartillos": "cuartillo",
    "cuarto": "cuartar",
    "cuartofinalistas": "cuartofinalista",
    "cuartones": "cuartón",
    "cuartos": "cuarto",
    "cuartábamos": "cuartar",
    "cuartáis": "cuartar",
    "cuartáramos": "cuartar",
    "cuartáremos": "cuartar",
    "cuartásemos": "cuartar",
    "cuarté": "cuartar",
    "cuartéis": "cuartar",
    "cuartó": "cuartar",
    "cuarzitas": "cuarzita",
    "cuarzos": "cuarzo",
    "cuarzosa": "cuarzoso",
    "cuarzosas": "cuarzoso",
    "cuarzosos": "cuarzoso",
    "cuasias": "cuasia",
    "cuasicontratos": "cuasicontrato",
    "cuasidelitos": "cuasidelito",
    "cuasimodos": "cuasimodo",
    "cuasiposesiones": "cuasiposesión",
    "cuaterna": "cuaterno",
    "cuaternaria": "cuaternario",
    "cuaternarias": "cuaternario",
    "cuaternarios": "cuaternario",
    "cuaternas": "cuaterno",
    "cuaternidades": "cuaternidad",
    "cuaternios": "cuaternio",
    "cuaternos": "cuaterno",
    "cuates": "cuate",
    "cuatorviros": "cuatorviro",
    "cuatralba": "cuatralbo",
    "cuatralbas": "cuatralbo",
    "cuatralbos": "cuatralbo",
    "cuatrera": "cuatrero",
    "cuatreras": "cuatrero",
    "cuatreros": "cuatrero",
    "cuatreña": "cuatreño",
    "cuatreñas": "cuatreño",
    "cuatreños": "cuatreño",
    "cuatriciclos": "cuatriciclo",
    "cuatricromías": "cuatricromía",
    "cuatrienios": "cuatrienio",
    "cuatrifilares": "cuatrifilar",
    "cuatrifolios": "cuatrifolio",
    "cuatrilingües": "cuatrilingüe",
    "cuatrilliza": "cuatrillizo",
    "cuatrillizas": "cuatrillizo",
    "cuatrillizos": "cuatrillizo",
    "cuatrillones": "cuatrillón",
    "cuatrillos": "cuatrillo",
    "cuatrimestrales": "cuatrimestral",
    "cuatrimestres": "cuatrimestre",
    "cuatrimotora": "cuatrimotor",
    "cuatrimotoras": "cuatrimotor",
    "cuatrimotores": "cuatrimotor",
    "cuatrincas": "cuatrinca",
    "cuatripartita": "cuatripartito",
    "cuatripartitas": "cuatripartito",
    "cuatripartitos": "cuatripartito",
    "cuatrisílaba": "cuatrisílabo",
    "cuatrisílabas": "cuatrisílabo",
    "cuatrisílabos": "cuatrisílabo",
    "cuatrocentistas": "cuatrocentista",
    "cuatrocientosava": "cuatrocientosavo",
    "cuatrocientosavas": "cuatrocientosavo",
    "cuatrocientosavo": "400",
    "cuatrocientosavos": "cuatrocientosavo",
    "cuatropeas": "cuatropea",
    "cuatrotantos": "cuatrotanto",
    "cubalibres": "cubalibre",
    "cubana": "cubano",
    "cubanas": "cubano",
    "cubanismos": "cubanismo",
    "cubanos": "cubano",
    "cubas": "cuba",
    "cubebas": "cubeba",
    "cubera": "cubero",
    "cuberas": "cubero",
    "cuberos": "cubero",
    "cubertadas": "cubertada",
    "cuberterías": "cubertería",
    "cuberías": "cubería",
    "cubetas": "cubeta",
    "cubetos": "cubeto",
    "cubica": "cubicar",
    "cubicaba": "cubicar",
    "cubicabais": "cubicar",
    "cubicaban": "cubicar",
    "cubicabas": "cubicar",
    "cubicaciones": "cubicación",
    "cubicad": "cubicar",
    "cubicada": "cubicar",
    "cubicadas": "cubicar",
    "cubicado": "cubicar",
    "cubicados": "cubicar",
    "cubicajes": "cubicaje",
    "cubicamos": "cubicar",
    "cubican": "cubicar",
    "cubicando": "cubicar",
    "cubicara": "cubicar",
    "cubicarais": "cubicar",
    "cubicaran": "cubicar",
    "cubicaras": "cubicar",
    "cubicare": "cubicar",
    "cubicareis": "cubicar",
    "cubicaremos": "cubicar",
    "cubicaren": "cubicar",
    "cubicares": "cubicar",
    "cubicaron": "cubicar",
    "cubicará": "cubicar",
    "cubicarán": "cubicar",
    "cubicarás": "cubicar",
    "cubicaré": "cubicar",
    "cubicaréis": "cubicar",
    "cubicaría": "cubicar",
    "cubicaríais": "cubicar",
    "cubicaríamos": "cubicar",
    "cubicarían": "cubicar",
    "cubicarías": "cubicar",
    "cubicas": "cubicar",
    "cubicase": "cubicar",
    "cubicaseis": "cubicar",
    "cubicasen": "cubicar",
    "cubicases": "cubicar",
    "cubicaste": "cubicar",
    "cubicasteis": "cubicar",
    "cubico": "cubicar",
    "cubicábamos": "cubicar",
    "cubicáis": "cubicar",
    "cubicáramos": "cubicar",
    "cubicáremos": "cubicar",
    "cubicásemos": "cubicar",
    "cubicó": "cubicar",
    "cubierta": "cubrir",
    "cubiertas": "cubrir",
    "cubierto": "cubrir",
    "cubiertos": "cubrir",
    "cubila": "cubilar",
    "cubilaba": "cubilar",
    "cubilabais": "cubilar",
    "cubilaban": "cubilar",
    "cubilabas": "cubilar",
    "cubilad": "cubilar",
    "cubilada": "cubilar",
    "cubiladas": "cubilar",
    "cubilado": "cubilar",
    "cubilados": "cubilar",
    "cubilamos": "cubilar",
    "cubilan": "cubilar",
    "cubilando": "cubilar",
    "cubilara": "cubilar",
    "cubilarais": "cubilar",
    "cubilaran": "cubilar",
    "cubilaras": "cubilar",
    "cubilare": "cubilar",
    "cubilareis": "cubilar",
    "cubilaremos": "cubilar",
    "cubilaren": "cubilar",
    "cubilares": "cubilar",
    "cubilaron": "cubilar",
    "cubilará": "cubilar",
    "cubilarán": "cubilar",
    "cubilarás": "cubilar",
    "cubilaré": "cubilar",
    "cubilaréis": "cubilar",
    "cubilaría": "cubilar",
    "cubilaríais": "cubilar",
    "cubilaríamos": "cubilar",
    "cubilarían": "cubilar",
    "cubilarías": "cubilar",
    "cubilas": "cubilar",
    "cubilase": "cubilar",
    "cubilaseis": "cubilar",
    "cubilasen": "cubilar",
    "cubilases": "cubilar",
    "cubilaste": "cubilar",
    "cubilasteis": "cubilar",
    "cubile": "cubilar",
    "cubilemos": "cubilar",
    "cubilen": "cubilar",
    "cubiles": "cubilar",
    "cubiletea": "cubiletear",
    "cubileteaba": "cubiletear",
    "cubileteabais": "cubiletear",
    "cubileteaban": "cubiletear",
    "cubileteabas": "cubiletear",
    "cubiletead": "cubiletear",
    "cubileteada": "cubiletear",
    "cubileteadas": "cubiletear",
    "cubileteado": "cubiletear",
    "cubileteados": "cubiletear",
    "cubileteamos": "cubiletear",
    "cubiletean": "cubiletear",
    "cubileteando": "cubiletear",
    "cubileteara": "cubiletear",
    "cubiletearais": "cubiletear",
    "cubiletearan": "cubiletear",
    "cubiletearas": "cubiletear",
    "cubileteare": "cubiletear",
    "cubileteareis": "cubiletear",
    "cubiletearemos": "cubiletear",
    "cubiletearen": "cubiletear",
    "cubileteares": "cubiletear",
    "cubiletearon": "cubiletear",
    "cubileteará": "cubiletear",
    "cubiletearán": "cubiletear",
    "cubiletearás": "cubiletear",
    "cubiletearé": "cubiletear",
    "cubiletearéis": "cubiletear",
    "cubiletearía": "cubiletear",
    "cubiletearíais": "cubiletear",
    "cubiletearíamos": "cubiletear",
    "cubiletearían": "cubiletear",
    "cubiletearías": "cubiletear",
    "cubileteas": "cubiletear",
    "cubiletease": "cubiletear",
    "cubileteaseis": "cubiletear",
    "cubileteasen": "cubiletear",
    "cubileteases": "cubiletear",
    "cubileteaste": "cubiletear",
    "cubileteasteis": "cubiletear",
    "cubiletee": "cubiletear",
    "cubileteemos": "cubiletear",
    "cubileteen": "cubiletear",
    "cubiletees": "cubiletear",
    "cubileteo": "cubiletear",
    "cubileteos": "cubileteo",
    "cubileteros": "cubiletero",
    "cubiletes": "cubilete",
    "cubileteábamos": "cubiletear",
    "cubileteáis": "cubiletear",
    "cubileteáramos": "cubiletear",
    "cubileteáremos": "cubiletear",
    "cubileteásemos": "cubiletear",
    "cubileteé": "cubiletear",
    "cubileteéis": "cubiletear",
    "cubileteó": "cubiletear",
    "cubillos": "cubillo",
    "cubilo": "cubilar",
    "cubilotes": "cubilote",
    "cubilábamos": "cubilar",
    "cubiláis": "cubilar",
    "cubiláramos": "cubilar",
    "cubiláremos": "cubilar",
    "cubilásemos": "cubilar",
    "cubilé": "cubilar",
    "cubiléis": "cubilar",
    "cubiló": "cubilar",
    "cubique": "cubicar",
    "cubiquemos": "cubicar",
    "cubiquen": "cubicar",
    "cubiques": "cubicar",
    "cubiqué": "cubicar",
    "cubiquéis": "cubicar",
    "cubismos": "cubismo",
    "cubistas": "cubista",
    "cubitales": "cubital",
    "cubiteras": "cubitera",
    "cubitos": "cubito",
    "cuboideonaviculares": "cuboideonavicular",
    "cubomedusas": "cubomedusa",
    "cubos": "cubo",
    "cubra": "cubrir",
    "cubramos": "cubrir",
    "cubran": "cubrir",
    "cubras": "cubrir",
    "cubre": "cubrir",
    "cubrecadenas": "cubrecadena",
    "cubrecamas": "cubrecama",
    "cubrecolchones": "cubrecolchón",
    "cubrejuntas": "cubrejunta",
    "cubremanteles": "cubremantel",
    "cubremuelles": "cubremuelle",
    "cubren": "cubrir",
    "cubrenucas": "cubrenuca",
    "cubres": "cubrir",
    "cubriciones": "cubrición",
    "cubrid": "cubrir",
    "cubriendo": "cubrir",
    "cubrientes": "cubriente",
    "cubriera": "cubrir",
    "cubrierais": "cubrir",
    "cubrieran": "cubrir",
    "cubrieras": "cubrir",
    "cubriere": "cubrir",
    "cubriereis": "cubrir",
    "cubrieren": "cubrir",
    "cubrieres": "cubrir",
    "cubrieron": "cubrir",
    "cubriese": "cubrir",
    "cubrieseis": "cubrir",
    "cubriesen": "cubrir",
    "cubrieses": "cubrir",
    "cubrimientos": "cubrimiento",
    "cubrimos": "cubrir",
    "cubriremos": "cubrir",
    "cubrirá": "cubrir",
    "cubrirán": "cubrir",
    "cubrirás": "cubrir",
    "cubriré": "cubrir",
    "cubriréis": "cubrir",
    "cubriría": "cubrir",
    "cubriríais": "cubrir",
    "cubriríamos": "cubrir",
    "cubrirían": "cubrir",
    "cubrirías": "cubrir",
    "cubriste": "cubrir",
    "cubristeis": "cubrir",
    "cubriéramos": "cubrir",
    "cubriéremos": "cubrir",
    "cubriésemos": "cubrir",
    "cubrió": "cubrir",
    "cubro": "cubrir",
    "cubráis": "cubrir",
    "cubrí": "cubrir",
    "cubría": "cubrir",
    "cubríais": "cubrir",
    "cubríamos": "cubrir",
    "cubrían": "cubrir",
    "cubrías": "cubrir",
    "cubrís": "cubrir",
    "cubículos": "cubículo",
    "cuca": "cuco",
    "cucaba": "cucar",
    "cucabais": "cucar",
    "cucaban": "cucar",
    "cucabas": "cucar",
    "cucaburras": "cucaburra",
    "cucad": "cucar",
    "cucada": "cucar",
    "cucadas": "cucar",
    "cucado": "cucar",
    "cucados": "cucar",
    "cucales": "cucal",
    "cucamas": "cucama",
    "cucamos": "cucar",
    "cucan": "cucar",
    "cucando": "cucar",
    "cucara": "cucar",
    "cucarachas": "cucaracha",
    "cucarais": "cucar",
    "cucaran": "cucar",
    "cucaras": "cucar",
    "cucardas": "cucarda",
    "cucare": "cucar",
    "cucareis": "cucar",
    "cucaremos": "cucar",
    "cucaren": "cucar",
    "cucares": "cucar",
    "cucaron": "cucar",
    "cucará": "cucar",
    "cucarán": "cucar",
    "cucarás": "cucar",
    "cucaré": "cucar",
    "cucaréis": "cucar",
    "cucaría": "cucar",
    "cucaríais": "cucar",
    "cucaríamos": "cucar",
    "cucarían": "cucar",
    "cucarías": "cucar",
    "cucas": "cuco",
    "cucase": "cucar",
    "cucaseis": "cucar",
    "cucasen": "cucar",
    "cucases": "cucar",
    "cucaste": "cucar",
    "cucasteis": "cucar",
    "cucañas": "cucaña",
    "cucañera": "cucañero",
    "cucañeras": "cucañero",
    "cucañeros": "cucañero",
    "cucharadas": "cucharada",
    "cucharaditas": "cucharadita",
    "cucharas": "cuchara",
    "cucharera": "cucharero",
    "cuchareras": "cucharero",
    "cuchareros": "cucharero",
    "cucharetas": "cuchareta",
    "cucharetea": "cucharetear",
    "cuchareteaba": "cucharetear",
    "cuchareteabais": "cucharetear",
    "cuchareteaban": "cucharetear",
    "cuchareteabas": "cucharetear",
    "cucharetead": "cucharetear",
    "cuchareteada": "cucharetear",
    "cuchareteadas": "cucharetear",
    "cuchareteado": "cucharetear",
    "cuchareteados": "cucharetear",
    "cuchareteamos": "cucharetear",
    "cucharetean": "cucharetear",
    "cuchareteando": "cucharetear",
    "cuchareteara": "cucharetear",
    "cucharetearais": "cucharetear",
    "cucharetearan": "cucharetear",
    "cucharetearas": "cucharetear",
    "cuchareteare": "cucharetear",
    "cuchareteareis": "cucharetear",
    "cucharetearemos": "cucharetear",
    "cucharetearen": "cucharetear",
    "cuchareteares": "cucharetear",
    "cucharetearon": "cucharetear",
    "cuchareteará": "cucharetear",
    "cucharetearán": "cucharetear",
    "cucharetearás": "cucharetear",
    "cucharetearé": "cucharetear",
    "cucharetearéis": "cucharetear",
    "cucharetearía": "cucharetear",
    "cucharetearíais": "cucharetear",
    "cucharetearíamos": "cucharetear",
    "cucharetearían": "cucharetear",
    "cucharetearías": "cucharetear",
    "cuchareteas": "cucharetear",
    "cucharetease": "cucharetear",
    "cuchareteaseis": "cucharetear",
    "cuchareteasen": "cucharetear",
    "cuchareteases": "cucharetear",
    "cuchareteaste": "cucharetear",
    "cuchareteasteis": "cucharetear",
    "cucharetee": "cucharetear",
    "cuchareteemos": "cucharetear",
    "cuchareteen": "cucharetear",
    "cucharetees": "cucharetear",
    "cuchareteo": "cucharetear",
    "cucharetera": "cucharetero",
    "cuchareteras": "cucharetero",
    "cuchareteros": "cucharetero",
    "cuchareteábamos": "cucharetear",
    "cuchareteáis": "cucharetear",
    "cuchareteáramos": "cucharetear",
    "cuchareteáremos": "cucharetear",
    "cuchareteásemos": "cucharetear",
    "cuchareteé": "cucharetear",
    "cuchareteéis": "cucharetear",
    "cuchareteó": "cucharetear",
    "cucharillas": "cucharilla",
    "cucharitas": "cucharita",
    "cucharones": "cucharón",
    "cuchichea": "cuchichear",
    "cuchicheaba": "cuchichear",
    "cuchicheabais": "cuchichear",
    "cuchicheaban": "cuchichear",
    "cuchicheabas": "cuchichear",
    "cuchichead": "cuchichear",
    "cuchicheada": "cuchichear",
    "cuchicheadas": "cuchichear",
    "cuchicheado": "cuchichear",
    "cuchicheados": "cuchichear",
    "cuchicheamos": "cuchichear",
    "cuchichean": "cuchichear",
    "cuchicheando": "cuchichear",
    "cuchicheara": "cuchichear",
    "cuchichearais": "cuchichear",
    "cuchichearan": "cuchichear",
    "cuchichearas": "cuchichear",
    "cuchicheare": "cuchichear",
    "cuchicheareis": "cuchichear",
    "cuchichearemos": "cuchichear",
    "cuchichearen": "cuchichear",
    "cuchicheares": "cuchichear",
    "cuchichearon": "cuchichear",
    "cuchicheará": "cuchichear",
    "cuchichearán": "cuchichear",
    "cuchichearás": "cuchichear",
    "cuchichearé": "cuchichear",
    "cuchichearéis": "cuchichear",
    "cuchichearía": "cuchichear",
    "cuchichearíais": "cuchichear",
    "cuchichearíamos": "cuchichear",
    "cuchichearían": "cuchichear",
    "cuchichearías": "cuchichear",
    "cuchicheas": "cuchichear",
    "cuchichease": "cuchichear",
    "cuchicheaseis": "cuchichear",
    "cuchicheasen": "cuchichear",
    "cuchicheases": "cuchichear",
    "cuchicheaste": "cuchichear",
    "cuchicheasteis": "cuchichear",
    "cuchichee": "cuchichear",
    "cuchicheemos": "cuchichear",
    "cuchicheen": "cuchichear",
    "cuchichees": "cuchichear",
    "cuchicheo": "cuchichear",
    "cuchicheos": "cuchicheo",
    "cuchicheábamos": "cuchichear",
    "cuchicheáis": "cuchichear",
    "cuchicheáramos": "cuchichear",
    "cuchicheáremos": "cuchichear",
    "cuchicheásemos": "cuchichear",
    "cuchicheé": "cuchichear",
    "cuchicheéis": "cuchichear",
    "cuchicheó": "cuchichear",
    "cuchichiaba": "cuchichiar",
    "cuchichiabais": "cuchichiar",
    "cuchichiaban": "cuchichiar",
    "cuchichiabas": "cuchichiar",
    "cuchichiad": "cuchichiar",
    "cuchichiada": "cuchichiar",
    "cuchichiadas": "cuchichiar",
    "cuchichiado": "cuchichiar",
    "cuchichiados": "cuchichiar",
    "cuchichiamos": "cuchichiar",
    "cuchichiando": "cuchichiar",
    "cuchichiara": "cuchichiar",
    "cuchichiarais": "cuchichiar",
    "cuchichiaran": "cuchichiar",
    "cuchichiaras": "cuchichiar",
    "cuchichiare": "cuchichiar",
    "cuchichiareis": "cuchichiar",
    "cuchichiaremos": "cuchichiar",
    "cuchichiaren": "cuchichiar",
    "cuchichiares": "cuchichiar",
    "cuchichiaron": "cuchichiar",
    "cuchichiará": "cuchichiar",
    "cuchichiarán": "cuchichiar",
    "cuchichiarás": "cuchichiar",
    "cuchichiaré": "cuchichiar",
    "cuchichiaréis": "cuchichiar",
    "cuchichiaría": "cuchichiar",
    "cuchichiaríais": "cuchichiar",
    "cuchichiaríamos": "cuchichiar",
    "cuchichiarían": "cuchichiar",
    "cuchichiarías": "cuchichiar",
    "cuchichiase": "cuchichiar",
    "cuchichiaseis": "cuchichiar",
    "cuchichiasen": "cuchichiar",
    "cuchichiases": "cuchichiar",
    "cuchichiaste": "cuchichiar",
    "cuchichiasteis": "cuchichiar",
    "cuchichiemos": "cuchichiar",
    "cuchichiábamos": "cuchichiar",
    "cuchichiáis": "cuchichiar",
    "cuchichiáramos": "cuchichiar",
    "cuchichiáremos": "cuchichiar",
    "cuchichiásemos": "cuchichiar",
    "cuchichié": "cuchichiar",
    "cuchichiéis": "cuchichiar",
    "cuchichió": "cuchichiar",
    "cuchichía": "cuchichiar",
    "cuchichían": "cuchichiar",
    "cuchichías": "cuchichiar",
    "cuchichíe": "cuchichiar",
    "cuchichíen": "cuchichiar",
    "cuchichíes": "cuchichiar",
    "cuchichío": "cuchichiar",
    "cuchilladas": "cuchillada",
    "cuchillares": "cuchillar",
    "cuchillas": "cuchilla",
    "cuchillazos": "cuchillazo",
    "cuchillejas": "cuchilleja",
    "cuchillera": "cuchillero",
    "cuchilleras": "cuchillero",
    "cuchilleros": "cuchillero",
    "cuchillerías": "cuchillería",
    "cuchillos": "cuchillo",
    "cuchipandas": "cuchipanda",
    "cuchitriles": "cuchitril",
    "cuchuchea": "cuchuchear",
    "cuchucheaba": "cuchuchear",
    "cuchucheabais": "cuchuchear",
    "cuchucheaban": "cuchuchear",
    "cuchucheabas": "cuchuchear",
    "cuchuchead": "cuchuchear",
    "cuchucheada": "cuchuchear",
    "cuchucheadas": "cuchuchear",
    "cuchucheado": "cuchuchear",
    "cuchucheados": "cuchuchear",
    "cuchucheamos": "cuchuchear",
    "cuchuchean": "cuchuchear",
    "cuchucheando": "cuchuchear",
    "cuchucheara": "cuchuchear",
    "cuchuchearais": "cuchuchear",
    "cuchuchearan": "cuchuchear",
    "cuchuchearas": "cuchuchear",
    "cuchucheare": "cuchuchear",
    "cuchucheareis": "cuchuchear",
    "cuchuchearemos": "cuchuchear",
    "cuchuchearen": "cuchuchear",
    "cuchucheares": "cuchuchear",
    "cuchuchearon": "cuchuchear",
    "cuchucheará": "cuchuchear",
    "cuchuchearán": "cuchuchear",
    "cuchuchearás": "cuchuchear",
    "cuchuchearé": "cuchuchear",
    "cuchuchearéis": "cuchuchear",
    "cuchuchearía": "cuchuchear",
    "cuchuchearíais": "cuchuchear",
    "cuchuchearíamos": "cuchuchear",
    "cuchuchearían": "cuchuchear",
    "cuchuchearías": "cuchuchear",
    "cuchucheas": "cuchuchear",
    "cuchuchease": "cuchuchear",
    "cuchucheaseis": "cuchuchear",
    "cuchucheasen": "cuchuchear",
    "cuchucheases": "cuchuchear",
    "cuchucheaste": "cuchuchear",
    "cuchucheasteis": "cuchuchear",
    "cuchuchee": "cuchuchear",
    "cuchucheemos": "cuchuchear",
    "cuchucheen": "cuchuchear",
    "cuchuchees": "cuchuchear",
    "cuchucheo": "cuchuchear",
    "cuchucheábamos": "cuchuchear",
    "cuchucheáis": "cuchuchear",
    "cuchucheáramos": "cuchuchear",
    "cuchucheáremos": "cuchuchear",
    "cuchucheásemos": "cuchuchear",
    "cuchucheé": "cuchuchear",
    "cuchucheéis": "cuchuchear",
    "cuchucheó": "cuchuchear",
    "cuchufletas": "cuchufleta",
    "cuchufletea": "cuchufletear",
    "cuchufleteaba": "cuchufletear",
    "cuchufleteabais": "cuchufletear",
    "cuchufleteaban": "cuchufletear",
    "cuchufleteabas": "cuchufletear",
    "cuchufletead": "cuchufletear",
    "cuchufleteada": "cuchufletear",
    "cuchufleteadas": "cuchufletear",
    "cuchufleteado": "cuchufletear",
    "cuchufleteados": "cuchufletear",
    "cuchufleteamos": "cuchufletear",
    "cuchufletean": "cuchufletear",
    "cuchufleteando": "cuchufletear",
    "cuchufleteara": "cuchufletear",
    "cuchufletearais": "cuchufletear",
    "cuchufletearan": "cuchufletear",
    "cuchufletearas": "cuchufletear",
    "cuchufleteare": "cuchufletear",
    "cuchufleteareis": "cuchufletear",
    "cuchufletearemos": "cuchufletear",
    "cuchufletearen": "cuchufletear",
    "cuchufleteares": "cuchufletear",
    "cuchufletearon": "cuchufletear",
    "cuchufleteará": "cuchufletear",
    "cuchufletearán": "cuchufletear",
    "cuchufletearás": "cuchufletear",
    "cuchufletearé": "cuchufletear",
    "cuchufletearéis": "cuchufletear",
    "cuchufletearía": "cuchufletear",
    "cuchufletearíais": "cuchufletear",
    "cuchufletearíamos": "cuchufletear",
    "cuchufletearían": "cuchufletear",
    "cuchufletearías": "cuchufletear",
    "cuchufleteas": "cuchufletear",
    "cuchufletease": "cuchufletear",
    "cuchufleteaseis": "cuchufletear",
    "cuchufleteasen": "cuchufletear",
    "cuchufleteases": "cuchufletear",
    "cuchufleteaste": "cuchufletear",
    "cuchufleteasteis": "cuchufletear",
    "cuchufletee": "cuchufletear",
    "cuchufleteemos": "cuchufletear",
    "cuchufleteen": "cuchufletear",
    "cuchufletees": "cuchufletear",
    "cuchufleteo": "cuchufletear",
    "cuchufletera": "cuchufletero",
    "cuchufleteras": "cuchufletero",
    "cuchufleteros": "cuchufletero",
    "cuchufleteábamos": "cuchufletear",
    "cuchufleteáis": "cuchufletear",
    "cuchufleteáramos": "cuchufletear",
    "cuchufleteáremos": "cuchufletear",
    "cuchufleteásemos": "cuchufletear",
    "cuchufleteé": "cuchufletear",
    "cuchufleteéis": "cuchufletear",
    "cuchufleteó": "cuchufletear",
    "cuchugos": "cuchugo",
    "cuchumbíes": "cuchumbí",
    "cuchumbís": "cuchumbí",
    "cuclillos": "cuclillo",
    "cuco": "cucar",
    "cucos": "cuco",
    "cucubás": "cucubá",
    "cucuizas": "cucuiza",
    "cucurbitáceas": "cucurbitácea",
    "cucuruchos": "cucurucho",
    "cucábamos": "cucar",
    "cucáis": "cucar",
    "cucáramos": "cucar",
    "cucáremos": "cucar",
    "cucásemos": "cucar",
    "cucó": "cucar",
    "cucúlidos": "cucúlido",
    "cucúrbitas": "cucúrbita",
    "cucús": "cucú",
    "cudría": "cudrío",
    "cudrías": "cudrío",
    "cudríos": "cudrío",
    "cudúes": "cudú",
    "cudús": "cudú",
    "cuecas": "cueca",
    "cuece": "cocer",
    "cuecen": "cocer",
    "cueces": "cocer",
    "cuela": "colar",
    "cuelan": "colar",
    "cuelas": "colar",
    "cuele": "colar",
    "cuelen": "colar",
    "cueles": "colar",
    "cuelga": "colgar",
    "cuelgan": "colgar",
    "cuelgas": "cuelga",
    "cuelgo": "colgar",
    "cuelgue": "colgar",
    "cuelguen": "colgar",
    "cuelgues": "colgar",
    "cuellos": "cuello",
    "cuelo": "colar",
    "cuencas": "cuenca",
    "cuencos": "cuenco",
    "cuendas": "cuenda",
    "cuenta": "contar",
    "cuentacorrentistas": "cuentacorrentista",
    "cuentadantes": "cuentadante",
    "cuentan": "contar",
    "cuentapartícipes": "cuentapartícipe",
    "cuentas": "cuenta",
    "cuente": "contar",
    "cuenten": "contar",
    "cuentera": "cuentero",
    "cuenteras": "cuentero",
    "cuenteros": "cuentero",
    "cuentes": "contar",
    "cuentistas": "cuentista",
    "cuento": "contar",
    "cuentos": "cuento",
    "cuerazos": "cuerazo",
    "cuerda": "cuerdo",
    "cuerdas": "cuerdo",
    "cuerdos": "cuerdo",
    "cuereadas": "cuereada",
    "cuerizas": "cueriza",
    "cuernas": "cuerna",
    "cuernecillos": "cuernecillo",
    "cuernos": "cuerno",
    "cueros": "cuero",
    "cuerpos": "cuerpo",
    "cuervos": "cuervo",
    "cuescos": "cuesco",
    "cuesta": "costar",
    "cuestaciones": "cuestación",
    "cuestan": "costar",
    "cuestas": "cuesta",
    "cueste": "costar",
    "cuesten": "costar",
    "cuestes": "costar",
    "cuestiona": "cuestionar",
    "cuestionaba": "cuestionar",
    "cuestionabais": "cuestionar",
    "cuestionaban": "cuestionar",
    "cuestionabas": "cuestionar",
    "cuestionables": "cuestionable",
    "cuestionad": "cuestionar",
    "cuestionada": "cuestionar",
    "cuestionadas": "cuestionar",
    "cuestionado": "cuestionar",
    "cuestionados": "cuestionar",
    "cuestionamientos": "cuestionamiento",
    "cuestionamos": "cuestionar",
    "cuestionan": "cuestionar",
    "cuestionando": "cuestionar",
    "cuestionara": "cuestionar",
    "cuestionarais": "cuestionar",
    "cuestionaran": "cuestionar",
    "cuestionaras": "cuestionar",
    "cuestionare": "cuestionar",
    "cuestionareis": "cuestionar",
    "cuestionaremos": "cuestionar",
    "cuestionaren": "cuestionar",
    "cuestionares": "cuestionar",
    "cuestionarios": "cuestionario",
    "cuestionaron": "cuestionar",
    "cuestionará": "cuestionar",
    "cuestionarán": "cuestionar",
    "cuestionarás": "cuestionar",
    "cuestionaré": "cuestionar",
    "cuestionaréis": "cuestionar",
    "cuestionaría": "cuestionar",
    "cuestionaríais": "cuestionar",
    "cuestionaríamos": "cuestionar",
    "cuestionarían": "cuestionar",
    "cuestionarías": "cuestionar",
    "cuestionas": "cuestionar",
    "cuestionase": "cuestionar",
    "cuestionaseis": "cuestionar",
    "cuestionasen": "cuestionar",
    "cuestionases": "cuestionar",
    "cuestionaste": "cuestionar",
    "cuestionasteis": "cuestionar",
    "cuestione": "cuestionar",
    "cuestionemos": "cuestionar",
    "cuestionen": "cuestionar",
    "cuestiones": "cuestionar",
    "cuestiono": "cuestionar",
    "cuestionábamos": "cuestionar",
    "cuestionáis": "cuestionar",
    "cuestionáramos": "cuestionar",
    "cuestionáremos": "cuestionar",
    "cuestionásemos": "cuestionar",
    "cuestioné": "cuestionar",
    "cuestionéis": "cuestionar",
    "cuestionó": "cuestionar",
    "cuesto": "costar",
    "cuestores": "cuestor",
    "cuestos": "cuesto",
    "cuesturas": "cuestura",
    "cuetos": "cueto",
    "cuevas": "cueva",
    "cueza": "cocer",
    "cuezan": "cocer",
    "cuezas": "cueza",
    "cuezo": "cocer",
    "cuezos": "cuezo",
    "cugullas": "cugulla",
    "cuicas": "cuica",
    "cuida": "cuidar",
    "cuidaba": "cuidar",
    "cuidabais": "cuidar",
    "cuidaban": "cuidar",
    "cuidabas": "cuidar",
    "cuidad": "cuidar",
    "cuidada": "cuidar",
    "cuidadas": "cuidar",
    "cuidado": "cuidar",
    "cuidadora": "cuidador",
    "cuidadoras": "cuidador",
    "cuidadores": "cuidador",
    "cuidados": "cuidar",
    "cuidadosa": "cuidadoso",
    "cuidadosas": "cuidadoso",
    "cuidadosos": "cuidadoso",
    "cuidamos": "cuidar",
    "cuidan": "cuidar",
    "cuidando": "cuidar",
    "cuidara": "cuidar",
    "cuidarais": "cuidar",
    "cuidaran": "cuidar",
    "cuidaras": "cuidar",
    "cuidare": "cuidar",
    "cuidareis": "cuidar",
    "cuidaremos": "cuidar",
    "cuidaren": "cuidar",
    "cuidares": "cuidar",
    "cuidaron": "cuidar",
    "cuidará": "cuidar",
    "cuidarán": "cuidar",
    "cuidarás": "cuidar",
    "cuidaré": "cuidar",
    "cuidaréis": "cuidar",
    "cuidaría": "cuidar",
    "cuidaríais": "cuidar",
    "cuidaríamos": "cuidar",
    "cuidarían": "cuidar",
    "cuidarías": "cuidar",
    "cuidas": "cuidar",
    "cuidase": "cuidar",
    "cuidaseis": "cuidar",
    "cuidasen": "cuidar",
    "cuidases": "cuidar",
    "cuidaste": "cuidar",
    "cuidasteis": "cuidar",
    "cuide": "cuidar",
    "cuidemos": "cuidar",
    "cuiden": "cuidar",
    "cuides": "cuidar",
    "cuido": "cuidar",
    "cuidábamos": "cuidar",
    "cuidáis": "cuidar",
    "cuidáramos": "cuidar",
    "cuidáremos": "cuidar",
    "cuidásemos": "cuidar",
    "cuidé": "cuidar",
    "cuidéis": "cuidar",
    "cuidó": "cuidar",
    "cuitada": "cuitado",
    "cuitadas": "cuitado",
    "cuitados": "cuitado",
    "cuitas": "cuita",
    "cujas": "cuja",
    "culadas": "culada",
    "culantrillos": "culantrillo",
    "culantros": "culantro",
    "culatas": "culata",
    "culatazos": "culatazo",
    "culea": "culear",
    "culeaba": "culear",
    "culeabais": "culear",
    "culeaban": "culear",
    "culeabas": "culear",
    "culead": "culear",
    "culeada": "culear",
    "culeadas": "culear",
    "culeado": "culear",
    "culeados": "culear",
    "culeamos": "culear",
    "culean": "culear",
    "culeando": "culear",
    "culeara": "culear",
    "culearais": "culear",
    "culearan": "culear",
    "culearas": "culear",
    "culeare": "culear",
    "culeareis": "culear",
    "culearemos": "culear",
    "culearen": "culear",
    "culeares": "culear",
    "culearon": "culear",
    "culeará": "culear",
    "culearán": "culear",
    "culearás": "culear",
    "culearé": "culear",
    "culearéis": "culear",
    "culearía": "culear",
    "culearíais": "culear",
    "culearíamos": "culear",
    "culearían": "culear",
    "culearías": "culear",
    "culeas": "culear",
    "culease": "culear",
    "culeaseis": "culear",
    "culeasen": "culear",
    "culeases": "culear",
    "culeaste": "culear",
    "culeasteis": "culear",
    "culebras": "culebra",
    "culebrea": "culebrear",
    "culebreaba": "culebrear",
    "culebreabais": "culebrear",
    "culebreaban": "culebrear",
    "culebreabas": "culebrear",
    "culebread": "culebrear",
    "culebreada": "culebrear",
    "culebreadas": "culebrear",
    "culebreado": "culebrear",
    "culebreados": "culebrear",
    "culebreamos": "culebrear",
    "culebrean": "culebrear",
    "culebreando": "culebrear",
    "culebreara": "culebrear",
    "culebrearais": "culebrear",
    "culebrearan": "culebrear",
    "culebrearas": "culebrear",
    "culebreare": "culebrear",
    "culebreareis": "culebrear",
    "culebrearemos": "culebrear",
    "culebrearen": "culebrear",
    "culebreares": "culebrear",
    "culebrearon": "culebrear",
    "culebreará": "culebrear",
    "culebrearán": "culebrear",
    "culebrearás": "culebrear",
    "culebrearé": "culebrear",
    "culebrearéis": "culebrear",
    "culebrearía": "culebrear",
    "culebrearíais": "culebrear",
    "culebrearíamos": "culebrear",
    "culebrearían": "culebrear",
    "culebrearías": "culebrear",
    "culebreas": "culebrear",
    "culebrease": "culebrear",
    "culebreaseis": "culebrear",
    "culebreasen": "culebrear",
    "culebreases": "culebrear",
    "culebreaste": "culebrear",
    "culebreasteis": "culebrear",
    "culebree": "culebrear",
    "culebreemos": "culebrear",
    "culebreen": "culebrear",
    "culebrees": "culebrear",
    "culebreo": "culebrear",
    "culebreos": "culebreo",
    "culebreábamos": "culebrear",
    "culebreáis": "culebrear",
    "culebreáramos": "culebrear",
    "culebreáremos": "culebrear",
    "culebreásemos": "culebrear",
    "culebreé": "culebrear",
    "culebreéis": "culebrear",
    "culebreó": "culebrear",
    "culebrillas": "culebrilla",
    "culebrinas": "culebrina",
    "culebrona": "culebrón",
    "culebronas": "culebrón",
    "culebrones": "culebrón",
    "culee": "culear",
    "culeemos": "culear",
    "culeen": "culear",
    "culees": "culear",
    "culeo": "culear",
    "culera": "culero",
    "culeras": "culero",
    "culeros": "culero",
    "cules": "culé",
    "culeábamos": "culear",
    "culeáis": "culear",
    "culeáramos": "culear",
    "culeáremos": "culear",
    "culeásemos": "culear",
    "culeé": "culear",
    "culeéis": "culear",
    "culeó": "culear",
    "culinaria": "culinario",
    "culinarias": "culinario",
    "culinarios": "culinario",
    "culis": "culi",
    "culitos": "culito",
    "culmina": "culminar",
    "culminaba": "culminar",
    "culminabais": "culminar",
    "culminaban": "culminar",
    "culminabas": "culminar",
    "culminaciones": "culminación",
    "culminad": "culminar",
    "culminada": "culminar",
    "culminadas": "culminar",
    "culminado": "culminar",
    "culminados": "culminar",
    "culminamos": "culminar",
    "culminan": "culminar",
    "culminando": "culminar",
    "culminantes": "culminante",
    "culminara": "culminar",
    "culminarais": "culminar",
    "culminaran": "culminar",
    "culminaras": "culminar",
    "culminare": "culminar",
    "culminareis": "culminar",
    "culminaremos": "culminar",
    "culminaren": "culminar",
    "culminares": "culminar",
    "culminaron": "culminar",
    "culminará": "culminar",
    "culminarán": "culminar",
    "culminarás": "culminar",
    "culminaré": "culminar",
    "culminaréis": "culminar",
    "culminaría": "culminar",
    "culminaríais": "culminar",
    "culminaríamos": "culminar",
    "culminarían": "culminar",
    "culminarías": "culminar",
    "culminas": "culminar",
    "culminase": "culminar",
    "culminaseis": "culminar",
    "culminasen": "culminar",
    "culminases": "culminar",
    "culminaste": "culminar",
    "culminasteis": "culminar",
    "culmine": "culminar",
    "culminemos": "culminar",
    "culminen": "culminar",
    "culmines": "culminar",
    "culmino": "culminar",
    "culminábamos": "culminar",
    "culmináis": "culminar",
    "culmináramos": "culminar",
    "culmináremos": "culminar",
    "culminásemos": "culminar",
    "culminé": "culminar",
    "culminéis": "culminar",
    "culminó": "culminar",
    "culombiana": "culombiano",
    "culombianas": "culombiano",
    "culombianos": "culombiano",
    "culombios": "culombio",
    "culona": "culón",
    "culonas": "culón",
    "culones": "culón",
    "culos": "culo",
    "culotes": "culote",
    "culpa": "culpar",
    "culpaba": "culpar",
    "culpabais": "culpar",
    "culpaban": "culpar",
    "culpabas": "culpar",
    "culpabilidades": "culpabilidad",
    "culpabilística": "culpabilístico",
    "culpabilísticas": "culpabilístico",
    "culpabilísticos": "culpabilístico",
    "culpables": "culpable",
    "culpaciones": "culpación",
    "culpad": "culpar",
    "culpada": "culpar",
    "culpadas": "culpar",
    "culpado": "culpar",
    "culpados": "culpar",
    "culpamos": "culpar",
    "culpan": "culpar",
    "culpando": "culpar",
    "culpara": "culpar",
    "culparais": "culpar",
    "culparan": "culpar",
    "culparas": "culpar",
    "culpare": "culpar",
    "culpareis": "culpar",
    "culparemos": "culpar",
    "culparen": "culpar",
    "culpares": "culpar",
    "culparon": "culpar",
    "culpará": "culpar",
    "culparán": "culpar",
    "culparás": "culpar",
    "culparé": "culpar",
    "culparéis": "culpar",
    "culparía": "culpar",
    "culparíais": "culpar",
    "culparíamos": "culpar",
    "culparían": "culpar",
    "culparías": "culpar",
    "culpas": "culpar",
    "culpase": "culpar",
    "culpaseis": "culpar",
    "culpasen": "culpar",
    "culpases": "culpar",
    "culpaste": "culpar",
    "culpasteis": "culpar",
    "culpe": "culpar",
    "culpemos": "culpar",
    "culpen": "culpar",
    "culpes": "culpar",
    "culpo": "culpar",
    "culposa": "culposo",
    "culposas": "culposo",
    "culposos": "culposo",
    "culpábamos": "culpar",
    "culpáis": "culpar",
    "culpáramos": "culpar",
    "culpáremos": "culpar",
    "culpásemos": "culpar",
    "culpé": "culpar",
    "culpéis": "culpar",
    "culpó": "culpar",
    "culta": "culto",
    "cultalatiniparlas": "cultalatiniparla",
    "cultas": "culto",
    "culterana": "culterano",
    "culteranas": "culterano",
    "culteranismos": "culteranismo",
    "culteranos": "culterano",
    "cultiparlistas": "cultiparlista",
    "cultismos": "cultismo",
    "cultiva": "cultivar",
    "cultivaba": "cultivar",
    "cultivabais": "cultivar",
    "cultivaban": "cultivar",
    "cultivabas": "cultivar",
    "cultivables": "cultivable",
    "cultivad": "cultivar",
    "cultivada": "cultivar",
    "cultivadas": "cultivar",
    "cultivado": "cultivar",
    "cultivadora": "cultivador",
    "cultivadoras": "cultivador",
    "cultivadores": "cultivador",
    "cultivados": "cultivar",
    "cultivamos": "cultivar",
    "cultivan": "cultivar",
    "cultivando": "cultivar",
    "cultivara": "cultivar",
    "cultivarais": "cultivar",
    "cultivaran": "cultivar",
    "cultivaras": "cultivar",
    "cultivare": "cultivar",
    "cultivareis": "cultivar",
    "cultivaremos": "cultivar",
    "cultivaren": "cultivar",
    "cultivares": "cultivar",
    "cultivaron": "cultivar",
    "cultivará": "cultivar",
    "cultivarán": "cultivar",
    "cultivarás": "cultivar",
    "cultivaré": "cultivar",
    "cultivaréis": "cultivar",
    "cultivaría": "cultivar",
    "cultivaríais": "cultivar",
    "cultivaríamos": "cultivar",
    "cultivarían": "cultivar",
    "cultivarías": "cultivar",
    "cultivas": "cultivar",
    "cultivase": "cultivar",
    "cultivaseis": "cultivar",
    "cultivasen": "cultivar",
    "cultivases": "cultivar",
    "cultivaste": "cultivar",
    "cultivasteis": "cultivar",
    "cultive": "cultivar",
    "cultivemos": "cultivar",
    "cultiven": "cultivar",
    "cultives": "cultivar",
    "cultivo": "cultivar",
    "cultivos": "cultivo",
    "cultivábamos": "cultivar",
    "cultiváis": "cultivar",
    "cultiváramos": "cultivar",
    "cultiváremos": "cultivar",
    "cultivásemos": "cultivar",
    "cultivé": "cultivar",
    "cultivéis": "cultivar",
    "cultivó": "cultivar",
    "cultos": "culto",
    "culturales": "cultural",
    "culturalismos": "culturalismo",
    "culturas": "cultura",
    "culturismos": "culturismo",
    "culturizaciones": "culturización",
    "culícidos": "culícido",
    "cumanagotos": "cumanagoto",
    "cumarinas": "cumarina",
    "cumaronas": "cumarona",
    "cumbias": "cumbia",
    "cumbreras": "cumbrera",
    "cumbres": "cumbre",
    "cumenos": "cumeno",
    "cumiches": "cumiche",
    "cumpla": "cumplir",
    "cumplamos": "cumplir",
    "cumplan": "cumplir",
    "cumplas": "cumplir",
    "cumple": "cumplir",
    "cumplen": "cumplir",
    "cumples": "cumplir",
    "cumplid": "cumplir",
    "cumplida": "cumplir",
    "cumplidas": "cumplir",
    "cumplidera": "cumplidero",
    "cumplideras": "cumplidero",
    "cumplideros": "cumplidero",
    "cumplido": "cumplir",
    "cumplidora": "cumplidor",
    "cumplidoras": "cumplidor",
    "cumplidores": "cumplidor",
    "cumplidos": "cumplir",
    "cumpliendo": "cumplir",
    "cumpliera": "cumplir",
    "cumplierais": "cumplir",
    "cumplieran": "cumplir",
    "cumplieras": "cumplir",
    "cumpliere": "cumplir",
    "cumpliereis": "cumplir",
    "cumplieren": "cumplir",
    "cumplieres": "cumplir",
    "cumplieron": "cumplir",
    "cumpliese": "cumplir",
    "cumplieseis": "cumplir",
    "cumpliesen": "cumplir",
    "cumplieses": "cumplir",
    "cumplimenta": "cumplimentar",
    "cumplimentaba": "cumplimentar",
    "cumplimentabais": "cumplimentar",
    "cumplimentaban": "cumplimentar",
    "cumplimentabas": "cumplimentar",
    "cumplimentaciones": "cumplimentación",
    "cumplimentad": "cumplimentar",
    "cumplimentada": "cumplimentar",
    "cumplimentadas": "cumplimentar",
    "cumplimentado": "cumplimentar",
    "cumplimentados": "cumplimentar",
    "cumplimentamos": "cumplimentar",
    "cumplimentan": "cumplimentar",
    "cumplimentando": "cumplimentar",
    "cumplimentara": "cumplimentar",
    "cumplimentarais": "cumplimentar",
    "cumplimentaran": "cumplimentar",
    "cumplimentaras": "cumplimentar",
    "cumplimentare": "cumplimentar",
    "cumplimentareis": "cumplimentar",
    "cumplimentaremos": "cumplimentar",
    "cumplimentaren": "cumplimentar",
    "cumplimentares": "cumplimentar",
    "cumplimentaron": "cumplimentar",
    "cumplimentará": "cumplimentar",
    "cumplimentarán": "cumplimentar",
    "cumplimentarás": "cumplimentar",
    "cumplimentaré": "cumplimentar",
    "cumplimentaréis": "cumplimentar",
    "cumplimentaría": "cumplimentar",
    "cumplimentaríais": "cumplimentar",
    "cumplimentaríamos": "cumplimentar",
    "cumplimentarían": "cumplimentar",
    "cumplimentarías": "cumplimentar",
    "cumplimentas": "cumplimentar",
    "cumplimentase": "cumplimentar",
    "cumplimentaseis": "cumplimentar",
    "cumplimentasen": "cumplimentar",
    "cumplimentases": "cumplimentar",
    "cumplimentaste": "cumplimentar",
    "cumplimentasteis": "cumplimentar",
    "cumplimente": "cumplimentar",
    "cumplimentemos": "cumplimentar",
    "cumplimenten": "cumplimentar",
    "cumplimentera": "cumplimentero",
    "cumplimenteras": "cumplimentero",
    "cumplimenteros": "cumplimentero",
    "cumplimentes": "cumplimentar",
    "cumplimento": "cumplimentar",
    "cumplimentábamos": "cumplimentar",
    "cumplimentáis": "cumplimentar",
    "cumplimentáramos": "cumplimentar",
    "cumplimentáremos": "cumplimentar",
    "cumplimentásemos": "cumplimentar",
    "cumplimenté": "cumplimentar",
    "cumplimentéis": "cumplimentar",
    "cumplimentó": "cumplimentar",
    "cumplimientos": "cumplimiento",
    "cumplimos": "cumplir",
    "cumpliremos": "cumplir",
    "cumplirá": "cumplir",
    "cumplirán": "cumplir",
    "cumplirás": "cumplir",
    "cumpliré": "cumplir",
    "cumpliréis": "cumplir",
    "cumpliría": "cumplir",
    "cumpliríais": "cumplir",
    "cumpliríamos": "cumplir",
    "cumplirían": "cumplir",
    "cumplirías": "cumplir",
    "cumpliste": "cumplir",
    "cumplisteis": "cumplir",
    "cumpliéramos": "cumplir",
    "cumpliéremos": "cumplir",
    "cumpliésemos": "cumplir",
    "cumplió": "cumplir",
    "cumplo": "cumplir",
    "cumpláis": "cumplir",
    "cumplí": "cumplir",
    "cumplía": "cumplir",
    "cumplíais": "cumplir",
    "cumplíamos": "cumplir",
    "cumplían": "cumplir",
    "cumplías": "cumplir",
    "cumplís": "cumplir",
    "cumulantes": "cumulante",
    "cumulativa": "cumulativo",
    "cumulativas": "cumulativo",
    "cumulativos": "cumulativo",
    "cumúlica": "cumúlico",
    "cumúlicas": "cumúlico",
    "cumúlicos": "cumúlico",
    "cuna": "cunar",
    "cunaba": "cunar",
    "cunabais": "cunar",
    "cunaban": "cunar",
    "cunabas": "cunar",
    "cunad": "cunar",
    "cunada": "cunar",
    "cunadas": "cunar",
    "cunado": "cunar",
    "cunados": "cunar",
    "cunamos": "cunar",
    "cunan": "cunar",
    "cunando": "cunar",
    "cunara": "cunar",
    "cunarais": "cunar",
    "cunaran": "cunar",
    "cunaras": "cunar",
    "cunare": "cunar",
    "cunareis": "cunar",
    "cunaremos": "cunar",
    "cunaren": "cunar",
    "cunares": "cunar",
    "cunaron": "cunar",
    "cunará": "cunar",
    "cunarán": "cunar",
    "cunarás": "cunar",
    "cunaré": "cunar",
    "cunaréis": "cunar",
    "cunaría": "cunar",
    "cunaríais": "cunar",
    "cunaríamos": "cunar",
    "cunarían": "cunar",
    "cunarías": "cunar",
    "cunas": "cunar",
    "cunase": "cunar",
    "cunaseis": "cunar",
    "cunasen": "cunar",
    "cunases": "cunar",
    "cunaste": "cunar",
    "cunasteis": "cunar",
    "cunda": "cundir",
    "cundamos": "cundir",
    "cundan": "cundir",
    "cundas": "cundir",
    "cunde": "cundir",
    "cunden": "cundir",
    "cundes": "cundir",
    "cundid": "cundir",
    "cundida": "cundir",
    "cundidas": "cundir",
    "cundido": "cundir",
    "cundidos": "cundir",
    "cundiendo": "cundir",
    "cundiera": "cundir",
    "cundierais": "cundir",
    "cundieran": "cundir",
    "cundieras": "cundir",
    "cundiere": "cundir",
    "cundiereis": "cundir",
    "cundieren": "cundir",
    "cundieres": "cundir",
    "cundieron": "cundir",
    "cundiese": "cundir",
    "cundieseis": "cundir",
    "cundiesen": "cundir",
    "cundieses": "cundir",
    "cundimos": "cundir",
    "cundiremos": "cundir",
    "cundirá": "cundir",
    "cundirán": "cundir",
    "cundirás": "cundir",
    "cundiré": "cundir",
    "cundiréis": "cundir",
    "cundiría": "cundir",
    "cundiríais": "cundir",
    "cundiríamos": "cundir",
    "cundirían": "cundir",
    "cundirías": "cundir",
    "cundiste": "cundir",
    "cundisteis": "cundir",
    "cundiéramos": "cundir",
    "cundiéremos": "cundir",
    "cundiésemos": "cundir",
    "cundió": "cundir",
    "cundo": "cundir",
    "cundáis": "cundir",
    "cundí": "cundir",
    "cundía": "cundir",
    "cundíais": "cundir",
    "cundíamos": "cundir",
    "cundían": "cundir",
    "cundías": "cundir",
    "cundís": "cundir",
    "cune": "cunar",
    "cunea": "cunear",
    "cuneaba": "cunear",
    "cuneabais": "cunear",
    "cuneaban": "cunear",
    "cuneabas": "cunear",
    "cunead": "cunear",
    "cuneada": "cunear",
    "cuneadas": "cunear",
    "cuneado": "cunear",
    "cuneados": "cunear",
    "cuneamos": "cunear",
    "cunean": "cunear",
    "cuneando": "cunear",
    "cuneara": "cunear",
    "cunearais": "cunear",
    "cunearan": "cunear",
    "cunearas": "cunear",
    "cuneare": "cunear",
    "cuneareis": "cunear",
    "cunearemos": "cunear",
    "cunearen": "cunear",
    "cuneares": "cunear",
    "cunearon": "cunear",
    "cuneará": "cunear",
    "cunearán": "cunear",
    "cunearás": "cunear",
    "cunearé": "cunear",
    "cunearéis": "cunear",
    "cunearía": "cunear",
    "cunearíais": "cunear",
    "cunearíamos": "cunear",
    "cunearían": "cunear",
    "cunearías": "cunear",
    "cuneas": "cunear",
    "cunease": "cunear",
    "cuneaseis": "cunear",
    "cuneasen": "cunear",
    "cuneases": "cunear",
    "cuneaste": "cunear",
    "cuneasteis": "cunear",
    "cunee": "cunear",
    "cuneemos": "cunear",
    "cuneen": "cunear",
    "cunees": "cunear",
    "cuneiformes": "cuneiforme",
    "cunemos": "cunar",
    "cunen": "cunar",
    "cuneo": "cunear",
    "cuneocuboidea": "cuneocuboideo",
    "cuneocuboideas": "cuneocuboideo",
    "cuneocuboideos": "cuneocuboideo",
    "cuneoescafoidea": "cuneoescafoideo",
    "cuneoescafoideas": "cuneoescafoideo",
    "cuneoescafoideos": "cuneoescafoideo",
    "cuneonaviculares": "cuneonavicular",
    "cuneos": "cuneo",
    "cunera": "cunero",
    "cuneras": "cunero",
    "cuneros": "cunero",
    "cunes": "cunar",
    "cunetas": "cuneta",
    "cuneábamos": "cunear",
    "cuneáis": "cunear",
    "cuneáramos": "cunear",
    "cuneáremos": "cunear",
    "cuneásemos": "cunear",
    "cuneé": "cunear",
    "cuneéis": "cunear",
    "cuneó": "cunear",
    "cunicultora": "cunicultor",
    "cunicultoras": "cunicultor",
    "cunicultores": "cunicultor",
    "cuniculturas": "cunicultura",
    "cunicuíes": "cunicuí",
    "cunicuís": "cunicuí",
    "cuno": "cunar",
    "cunábamos": "cunar",
    "cunáis": "cunar",
    "cunáramos": "cunar",
    "cunáremos": "cunar",
    "cunásemos": "cunar",
    "cuné": "cunar",
    "cunéis": "cunar",
    "cunó": "cunar",
    "cuodlibetos": "cuodlibeto",
    "cuotas": "cuota",
    "cuotidiana": "cuotidiano",
    "cuotidianas": "cuotidiano",
    "cuotidianos": "cuotidiano",
    "cupe": "caber",
    "cupidos": "cupido",
    "cupiera": "caber",
    "cupierais": "caber",
    "cupieran": "caber",
    "cupieras": "caber",
    "cupiere": "caber",
    "cupiereis": "caber",
    "cupieren": "caber",
    "cupieres": "caber",
    "cupieron": "caber",
    "cupiese": "caber",
    "cupieseis": "caber",
    "cupiesen": "caber",
    "cupieses": "caber",
    "cupimos": "caber",
    "cupiste": "caber",
    "cupisteis": "caber",
    "cupiéramos": "caber",
    "cupiéremos": "caber",
    "cupiésemos": "caber",
    "cupletistas": "cupletista",
    "cuplés": "cuplé",
    "cupo": "caber",
    "cupones": "cupón",
    "cupos": "cupo",
    "cupresáceas": "cupresácea",
    "cupritas": "cuprita",
    "cuproaluminios": "cuproaluminio",
    "cuproamoniacales": "cuproamoniacal",
    "cuproamoníacos": "cuproamoníaco",
    "cupromanganesos": "cupromanganeso",
    "cuproníqueles": "cuproníquel",
    "cuprosa": "cuproso",
    "cuprosas": "cuproso",
    "cuprosilicios": "cuprosilicio",
    "cuprosos": "cuproso",
    "cuprífera": "cuprífero",
    "cupríferas": "cuprífero",
    "cupríferos": "cuprífero",
    "cupulares": "cupular",
    "cupulinos": "cupulino",
    "cupulometrías": "cupulometría",
    "cupés": "cupé",
    "cuque": "cucar",
    "cuquea": "cuquear",
    "cuqueaba": "cuquear",
    "cuqueabais": "cuquear",
    "cuqueaban": "cuquear",
    "cuqueabas": "cuquear",
    "cuquead": "cuquear",
    "cuqueada": "cuquear",
    "cuqueadas": "cuquear",
    "cuqueado": "cuquear",
    "cuqueados": "cuquear",
    "cuqueamos": "cuquear",
    "cuquean": "cuquear",
    "cuqueando": "cuquear",
    "cuqueara": "cuquear",
    "cuquearais": "cuquear",
    "cuquearan": "cuquear",
    "cuquearas": "cuquear",
    "cuqueare": "cuquear",
    "cuqueareis": "cuquear",
    "cuquearemos": "cuquear",
    "cuquearen": "cuquear",
    "cuqueares": "cuquear",
    "cuquearon": "cuquear",
    "cuqueará": "cuquear",
    "cuquearán": "cuquear",
    "cuquearás": "cuquear",
    "cuquearé": "cuquear",
    "cuquearéis": "cuquear",
    "cuquearía": "cuquear",
    "cuquearíais": "cuquear",
    "cuquearíamos": "cuquear",
    "cuquearían": "cuquear",
    "cuquearías": "cuquear",
    "cuqueas": "cuquear",
    "cuquease": "cuquear",
    "cuqueaseis": "cuquear",
    "cuqueasen": "cuquear",
    "cuqueases": "cuquear",
    "cuqueaste": "cuquear",
    "cuqueasteis": "cuquear",
    "cuquee": "cuquear",
    "cuqueemos": "cuquear",
    "cuqueen": "cuquear",
    "cuquees": "cuquear",
    "cuquemos": "cucar",
    "cuquen": "cucar",
    "cuqueo": "cuquear",
    "cuquerías": "cuquería",
    "cuques": "cucar",
    "cuqueábamos": "cuquear",
    "cuqueáis": "cuquear",
    "cuqueáramos": "cuquear",
    "cuqueáremos": "cuquear",
    "cuqueásemos": "cuquear",
    "cuqueé": "cuquear",
    "cuqueéis": "cuquear",
    "cuqueó": "cuquear",
    "cuquillos": "cuquillo",
    "cuqué": "cucar",
    "cuquéis": "cucar",
    "cura": "curar",
    "curaba": "curar",
    "curabais": "curar",
    "curaban": "curar",
    "curabas": "curar",
    "curabilidades": "curabilidad",
    "curables": "curable",
    "curacas": "curaca",
    "curaciones": "curación",
    "curad": "curar",
    "curada": "curar",
    "curadas": "curar",
    "curadillos": "curadillo",
    "curado": "curar",
    "curadora": "curador",
    "curadoras": "curador",
    "curadores": "curador",
    "curados": "curar",
    "curadurías": "curaduría",
    "curalotodos": "curalotodo",
    "curamos": "curar",
    "curan": "curar",
    "curandera": "curandero",
    "curanderas": "curandero",
    "curanderiles": "curanderil",
    "curanderismos": "curanderismo",
    "curanderos": "curandero",
    "curanderías": "curandería",
    "curando": "curar",
    "curara": "curar",
    "curarais": "curar",
    "curaran": "curar",
    "curaras": "curar",
    "curare": "curar",
    "curareis": "curar",
    "curaremos": "curar",
    "curaren": "curar",
    "curares": "curare",
    "curarinas": "curarina",
    "curarizaciones": "curarización",
    "curaron": "curar",
    "curará": "curar",
    "curarán": "curar",
    "curarás": "curar",
    "curaré": "curar",
    "curaréis": "curar",
    "curaría": "curar",
    "curaríais": "curar",
    "curaríamos": "curar",
    "curarían": "curar",
    "curarías": "curar",
    "curas": "curar",
    "curasaos": "curasao",
    "curase": "curar",
    "curaseis": "curar",
    "curasen": "curar",
    "curases": "curar",
    "curaste": "curar",
    "curasteis": "curar",
    "curatelas": "curatela",
    "curativa": "curativo",
    "curativas": "curativo",
    "curativos": "curativo",
    "curatos": "curato",
    "curazaos": "curazao",
    "curaçaos": "curaçao",
    "curbariles": "curbaril",
    "curculiónidos": "curculiónido",
    "curcuminas": "curcumina",
    "curcusillas": "curcusilla",
    "curda": "curdo",
    "curdas": "curdo",
    "curdelas": "curdela",
    "curdos": "curdo",
    "cure": "curar",
    "curemos": "curar",
    "curen": "curar",
    "cures": "curar",
    "curescas": "curesca",
    "curetajes": "curetaje",
    "curetas": "cureta",
    "curetes": "curete",
    "cureñas": "cureña",
    "curiales": "curial",
    "curialesca": "curialesco",
    "curialescas": "curialesco",
    "curialescos": "curialesco",
    "curiaras": "curiara",
    "curias": "curia",
    "curies": "curie",
    "curios": "curio",
    "curiosa": "curioso",
    "curiosas": "curioso",
    "curiosea": "curiosear",
    "curioseaba": "curiosear",
    "curioseabais": "curiosear",
    "curioseaban": "curiosear",
    "curioseabas": "curiosear",
    "curiosead": "curiosear",
    "curioseada": "curiosear",
    "curioseadas": "curiosear",
    "curioseado": "curiosear",
    "curioseados": "curiosear",
    "curioseamos": "curiosear",
    "curiosean": "curiosear",
    "curioseando": "curiosear",
    "curioseara": "curiosear",
    "curiosearais": "curiosear",
    "curiosearan": "curiosear",
    "curiosearas": "curiosear",
    "curioseare": "curiosear",
    "curioseareis": "curiosear",
    "curiosearemos": "curiosear",
    "curiosearen": "curiosear",
    "curioseares": "curiosear",
    "curiosearon": "curiosear",
    "curioseará": "curiosear",
    "curiosearán": "curiosear",
    "curiosearás": "curiosear",
    "curiosearé": "curiosear",
    "curiosearéis": "curiosear",
    "curiosearía": "curiosear",
    "curiosearíais": "curiosear",
    "curiosearíamos": "curiosear",
    "curiosearían": "curiosear",
    "curiosearías": "curiosear",
    "curioseas": "curiosear",
    "curiosease": "curiosear",
    "curioseaseis": "curiosear",
    "curioseasen": "curiosear",
    "curioseases": "curiosear",
    "curioseaste": "curiosear",
    "curioseasteis": "curiosear",
    "curiosee": "curiosear",
    "curioseemos": "curiosear",
    "curioseen": "curiosear",
    "curiosees": "curiosear",
    "curioseo": "curiosear",
    "curioseos": "curioseo",
    "curioseábamos": "curiosear",
    "curioseáis": "curiosear",
    "curioseáramos": "curiosear",
    "curioseáremos": "curiosear",
    "curioseásemos": "curiosear",
    "curioseé": "curiosear",
    "curioseéis": "curiosear",
    "curioseó": "curiosear",
    "curiosidades": "curiosidad",
    "curiosos": "curioso",
    "curlings": "curling",
    "curls": "curl",
    "curo": "curar",
    "curra": "curro",
    "currantes": "currante",
    "curras": "curro",
    "currela": "currelar",
    "currelaba": "currelar",
    "currelabais": "currelar",
    "currelaban": "currelar",
    "currelabas": "currelar",
    "currelad": "currelar",
    "currelada": "currelar",
    "curreladas": "currelar",
    "currelado": "currelar",
    "currelados": "currelar",
    "currelamos": "currelar",
    "currelan": "currelar",
    "currelando": "currelar",
    "currelara": "currelar",
    "currelarais": "currelar",
    "currelaran": "currelar",
    "currelaras": "currelar",
    "currelare": "currelar",
    "currelareis": "currelar",
    "currelaremos": "currelar",
    "currelaren": "currelar",
    "currelares": "currelar",
    "currelaron": "currelar",
    "currelará": "currelar",
    "currelarán": "currelar",
    "currelarás": "currelar",
    "currelaré": "currelar",
    "currelaréis": "currelar",
    "currelaría": "currelar",
    "currelaríais": "currelar",
    "currelaríamos": "currelar",
    "currelarían": "currelar",
    "currelarías": "currelar",
    "currelas": "currelar",
    "currelase": "currelar",
    "currelaseis": "currelar",
    "currelasen": "currelar",
    "currelases": "currelar",
    "currelaste": "currelar",
    "currelasteis": "currelar",
    "currele": "currelar",
    "currelemos": "currelar",
    "currelen": "currelar",
    "curreles": "currelar",
    "currelo": "currelar",
    "currelábamos": "currelar",
    "curreláis": "currelar",
    "curreláramos": "currelar",
    "curreláremos": "currelar",
    "currelásemos": "currelar",
    "currelé": "currelar",
    "curreléis": "currelar",
    "curreló": "currelar",
    "currentilínea": "currentilíneo",
    "currentilíneas": "currentilíneo",
    "currentilíneos": "currentilíneo",
    "curricanes": "curricán",
    "curriculares": "curricular",
    "curriculums": "curriculum",
    "curries": "curry",
    "currinches": "currinche",
    "curros": "curro",
    "currucas": "curruca",
    "currutaca": "currutaco",
    "currutacas": "currutaco",
    "currutacos": "currutaco",
    "curryes": "curry",
    "currícula": "currículum",
    "currículos": "currículo",
    "currículums": "currículum",
    "currículumvitaes": "currículumvitae",
    "cursa": "cursar",
    "cursaba": "cursar",
    "cursabais": "cursar",
    "cursaban": "cursar",
    "cursabas": "cursar",
    "cursad": "cursar",
    "cursada": "cursar",
    "cursadas": "cursar",
    "cursado": "cursar",
    "cursados": "cursar",
    "cursamos": "cursar",
    "cursan": "cursar",
    "cursando": "cursar",
    "cursara": "cursar",
    "cursarais": "cursar",
    "cursaran": "cursar",
    "cursaras": "cursar",
    "cursare": "cursar",
    "cursareis": "cursar",
    "cursaremos": "cursar",
    "cursaren": "cursar",
    "cursares": "cursar",
    "cursaron": "cursar",
    "cursará": "cursar",
    "cursarán": "cursar",
    "cursarás": "cursar",
    "cursaré": "cursar",
    "cursaréis": "cursar",
    "cursaría": "cursar",
    "cursaríais": "cursar",
    "cursaríamos": "cursar",
    "cursarían": "cursar",
    "cursarías": "cursar",
    "cursas": "cursar",
    "cursase": "cursar",
    "cursaseis": "cursar",
    "cursasen": "cursar",
    "cursases": "cursar",
    "cursaste": "cursar",
    "cursasteis": "cursar",
    "curse": "cursar",
    "cursemos": "cursar",
    "cursen": "cursar",
    "curserías": "cursería",
    "curses": "cursar",
    "cursiladas": "cursilada",
    "cursilerías": "cursilería",
    "cursillistas": "cursillista",
    "cursillos": "cursillo",
    "cursilona": "cursilón",
    "cursilonas": "cursilón",
    "cursilones": "cursilón",
    "cursis": "cursi",
    "cursiva": "cursivo",
    "cursivas": "cursivo",
    "cursivos": "cursivo",
    "curso": "cursar",
    "cursores": "cursor",
    "cursos": "curso",
    "cursábamos": "cursar",
    "cursáis": "cursar",
    "cursáramos": "cursar",
    "cursáremos": "cursar",
    "cursásemos": "cursar",
    "cursé": "cursar",
    "curséis": "cursar",
    "cursó": "cursar",
    "cursómetros": "cursómetro",
    "curta": "curtir",
    "curtamos": "curtir",
    "curtan": "curtir",
    "curtas": "curtir",
    "curte": "curtir",
    "curten": "curtir",
    "curtes": "curtir",
    "curtid": "curtir",
    "curtida": "curtir",
    "curtidas": "curtir",
    "curtido": "curtir",
    "curtidora": "curtidor",
    "curtidoras": "curtidor",
    "curtidores": "curtidor",
    "curtidos": "curtir",
    "curtidurías": "curtiduría",
    "curtiembres": "curtiembre",
    "curtiendo": "curtir",
    "curtientes": "curtiente",
    "curtiera": "curtir",
    "curtierais": "curtir",
    "curtieran": "curtir",
    "curtieras": "curtir",
    "curtiere": "curtir",
    "curtiereis": "curtir",
    "curtieren": "curtir",
    "curtieres": "curtir",
    "curtieron": "curtir",
    "curtiese": "curtir",
    "curtieseis": "curtir",
    "curtiesen": "curtir",
    "curtieses": "curtir",
    "curtimientos": "curtimiento",
    "curtimos": "curtir",
    "curtiremos": "curtir",
    "curtirá": "curtir",
    "curtirán": "curtir",
    "curtirás": "curtir",
    "curtiré": "curtir",
    "curtiréis": "curtir",
    "curtiría": "curtir",
    "curtiríais": "curtir",
    "curtiríamos": "curtir",
    "curtirían": "curtir",
    "curtirías": "curtir",
    "curtiste": "curtir",
    "curtisteis": "curtir",
    "curtiéramos": "curtir",
    "curtiéremos": "curtir",
    "curtiésemos": "curtir",
    "curtió": "curtir",
    "curto": "curtir",
    "curtáis": "curtir",
    "curtí": "curtir",
    "curtía": "curtir",
    "curtíais": "curtir",
    "curtíamos": "curtir",
    "curtían": "curtir",
    "curtías": "curtir",
    "curtís": "curtir",
    "curules": "curul",
    "curva": "curvo",
    "curvaba": "curvar",
    "curvabais": "curvar",
    "curvaban": "curvar",
    "curvabas": "curvar",
    "curvad": "curvar",
    "curvada": "curvar",
    "curvadas": "curvar",
    "curvado": "curvar",
    "curvadoras": "curvadora",
    "curvados": "curvar",
    "curvamos": "curvar",
    "curvan": "curvar",
    "curvando": "curvar",
    "curvara": "curvar",
    "curvarais": "curvar",
    "curvaran": "curvar",
    "curvaras": "curvar",
    "curvare": "curvar",
    "curvareis": "curvar",
    "curvaremos": "curvar",
    "curvaren": "curvar",
    "curvares": "curvar",
    "curvaron": "curvar",
    "curvará": "curvar",
    "curvarán": "curvar",
    "curvarás": "curvar",
    "curvaré": "curvar",
    "curvaréis": "curvar",
    "curvaría": "curvar",
    "curvaríais": "curvar",
    "curvaríamos": "curvar",
    "curvarían": "curvar",
    "curvarías": "curvar",
    "curvas": "curvo",
    "curvase": "curvar",
    "curvaseis": "curvar",
    "curvasen": "curvar",
    "curvases": "curvar",
    "curvaste": "curvar",
    "curvasteis": "curvar",
    "curvaturas": "curvatura",
    "curve": "curvar",
    "curvemos": "curvar",
    "curven": "curvar",
    "curves": "curvar",
    "curvilínea": "curvilíneo",
    "curvilíneas": "curvilíneo",
    "curvilíneos": "curvilíneo",
    "curvo": "curvar",
    "curvos": "curvo",
    "curvábamos": "curvar",
    "curváis": "curvar",
    "curváramos": "curvar",
    "curváremos": "curvar",
    "curvásemos": "curvar",
    "curvé": "curvar",
    "curvéis": "curvar",
    "curvímetros": "curvímetro",
    "curvó": "curvar",
    "curábamos": "curar",
    "curáis": "curar",
    "curáramos": "curar",
    "curáremos": "curar",
    "curásemos": "curar",
    "curé": "curar",
    "curéis": "curar",
    "curíes": "curí",
    "curís": "curí",
    "curó": "curar",
    "cus": "cu",
    "cuscurros": "cuscurro",
    "cuscutas": "cuscuta",
    "cushitas": "cushita",
    "cuspidada": "cuspidado",
    "cuspidadas": "cuspidado",
    "cuspidados": "cuspidado",
    "cuspídea": "cuspídeo",
    "cuspídeas": "cuspídeo",
    "cuspídeos": "cuspídeo",
    "custodia": "custodio",
    "custodiaba": "custodiar",
    "custodiabais": "custodiar",
    "custodiaban": "custodiar",
    "custodiabas": "custodiar",
    "custodiad": "custodiar",
    "custodiada": "custodiar",
    "custodiadas": "custodiar",
    "custodiado": "custodiar",
    "custodiados": "custodiar",
    "custodiamos": "custodiar",
    "custodian": "custodiar",
    "custodiando": "custodiar",
    "custodiara": "custodiar",
    "custodiarais": "custodiar",
    "custodiaran": "custodiar",
    "custodiaras": "custodiar",
    "custodiare": "custodiar",
    "custodiareis": "custodiar",
    "custodiaremos": "custodiar",
    "custodiaren": "custodiar",
    "custodiares": "custodiar",
    "custodiaron": "custodiar",
    "custodiará": "custodiar",
    "custodiarán": "custodiar",
    "custodiarás": "custodiar",
    "custodiaré": "custodiar",
    "custodiaréis": "custodiar",
    "custodiaría": "custodiar",
    "custodiaríais": "custodiar",
    "custodiaríamos": "custodiar",
    "custodiarían": "custodiar",
    "custodiarías": "custodiar",
    "custodias": "custodio",
    "custodiase": "custodiar",
    "custodiaseis": "custodiar",
    "custodiasen": "custodiar",
    "custodiases": "custodiar",
    "custodiaste": "custodiar",
    "custodiasteis": "custodiar",
    "custodie": "custodiar",
    "custodiemos": "custodiar",
    "custodien": "custodiar",
    "custodies": "custodiar",
    "custodio": "custodiar",
    "custodios": "custodio",
    "custodiábamos": "custodiar",
    "custodiáis": "custodiar",
    "custodiáramos": "custodiar",
    "custodiáremos": "custodiar",
    "custodiásemos": "custodiar",
    "custodié": "custodiar",
    "custodiéis": "custodiar",
    "custodió": "custodiar",
    "cuta": "cuto",
    "cutarras": "cutarra",
    "cutas": "cuto",
    "cuticulares": "cuticular",
    "cutinas": "cutina",
    "cutinizaciones": "cutinización",
    "cutirreacciones": "cutirreacción",
    "cutises": "cutis",
    "cutizaciones": "cutización",
    "cutos": "cuto",
    "cutres": "cutre",
    "cutters": "cutter",
    "cutánea": "cutáneo",
    "cutáneas": "cutáneo",
    "cutáneos": "cutáneo",
    "cutículas": "cutícula",
    "cutíes": "cutí",
    "cutís": "cutí",
    "cuxiúes": "cuxiú",
    "cuxiús": "cuxiú",
    "cuya": "cuyo",
    "cuyas": "cuyo",
    "cuyos": "cuyo",
    "cuzcos": "cuzco",
    "cuzcuces": "cuzcuz",
    "cuádricas": "cuádrica",
    "cuádrupla": "cuádruplo",
    "cuádruplas": "cuádruplo",
    "cuádruples": "cuádruple",
    "cuádruplos": "cuádruplo",
    "cuáles": "cuál",
    "cuánta": "cuánto",
    "cuántas": "cuánto",
    "cuántica": "cuántico",
    "cuánticas": "cuántico",
    "cuánticos": "cuántico",
    "cuántos": "cuánto",
    "cuáquera": "cuáquero",
    "cuáqueras": "cuáquero",
    "cuáqueros": "cuáquero",
    "cuártica": "cuártico",
    "cuárticas": "cuártico",
    "cuárticos": "cuártico",
    "cuévanos": "cuévano",
    "cuñada": "cuñado",
    "cuñadas": "cuñado",
    "cuñados": "cuñado",
    "cuñas": "cuña",
    "cuñetes": "cuñete",
    "cuños": "cuño",
    "czardas": "czarda",
    "czares": "czar",
    "czarevitces": "czarevitz",
    "czarinas": "czarina",
    "cábalas": "cábala",
    "cábrios": "cábrio",
    "cáfilas": "cáfila",
    "cáfrida": "cáfrido",
    "cáfridas": "cáfrido",
    "cáfridos": "cáfrido",
    "cálamos": "cálamo",
    "cálaos": "cálao",
    "cálatos": "cálato",
    "cálceos": "cálceo",
    "cálcica": "cálcico",
    "cálcicas": "cálcico",
    "cálcicos": "cálcico",
    "cálculos": "cálculo",
    "cálices": "cáliz",
    "cálida": "cálido",
    "cálidas": "cálido",
    "cálidos": "cálido",
    "cámaras": "cámara",
    "cámbiumes": "cámbium",
    "cámbiums": "cámbium",
    "cámbrica": "cámbrico",
    "cámbricas": "cámbrico",
    "cámbricos": "cámbrico",
    "cáncamos": "cáncamo",
    "cáncanas": "cáncana",
    "cáncanos": "cáncano",
    "cánceres": "cáncer",
    "cándida": "cándido",
    "cándidas": "cándido",
    "cándidos": "cándido",
    "cánidos": "cánido",
    "cánones": "canon",
    "cántabra": "cántabro",
    "cántabras": "cántabro",
    "cántabros": "cántabro",
    "cántaras": "cántara",
    "cántaros": "cántaro",
    "cánticos": "cántico",
    "cánticumes": "cánticum",
    "cánticums": "cánticum",
    "cánulas": "cánula",
    "cápitas": "cápita",
    "cáprica": "cáprico",
    "cápricas": "cáprico",
    "cápricos": "cáprico",
    "cápsulas": "cápsula",
    "cárabes": "cárabe",
    "cárabos": "cárabo",
    "cárcamos": "cárcamo",
    "cárcavas": "cárcava",
    "cárcavos": "cárcavo",
    "cárceles": "cárcel",
    "cárcolas": "cárcola",
    "cárdena": "cárdeno",
    "cárdenas": "cárdeno",
    "cárdenos": "cárdeno",
    "cárdigans": "cárdigan",
    "cármenes": "carmen",
    "cárnica": "cárnico",
    "cárnicas": "cárnico",
    "cárnicos": "cárnico",
    "cárstica": "cárstico",
    "cársticas": "cárstico",
    "cársticos": "cárstico",
    "cártamos": "cártamo",
    "cárteles": "cártel",
    "cárteres": "cárter",
    "cáscaras": "cáscara",
    "cásicos": "cásico",
    "cástulas": "cástula",
    "cátara": "cátaro",
    "cátaras": "cátaro",
    "cátaros": "cátaro",
    "cátedras": "cátedra",
    "cátodos": "cátodo",
    "cáustica": "cáustico",
    "cáusticas": "cáustico",
    "cáusticos": "cáustico",
    "cáveas": "cávea",
    "cáñamos": "cáñamo",
    "cébidos": "cébido",
    "cécubos": "cécubo",
    "cédrides": "cédride",
    "cédulas": "cédula",
    "céfiros": "céfiro",
    "célebres": "célebre",
    "céleres": "célere",
    "célibes": "célibe",
    "célica": "célico",
    "célicas": "célico",
    "célicos": "célico",
    "céltica": "céltico",
    "célticas": "céltico",
    "célticos": "céltico",
    "células": "célula",
    "cént": "céntimo",
    "céntima": "céntimo",
    "céntimas": "céntimo",
    "céntimos": "céntimo",
    "céntrica": "céntrico",
    "céntricas": "céntrico",
    "céntricos": "céntrico",
    "cénts": "céntimo",
    "céntupla": "céntuplo",
    "céntuplas": "céntuplo",
    "céntuples": "céntuple",
    "céntuplos": "céntuplo",
    "cérea": "céreo",
    "céreas": "céreo",
    "céreos": "céreo",
    "cérica": "cérico",
    "céricas": "cérico",
    "céricos": "cérico",
    "cértidos": "cértido",
    "cérvidos": "cérvido",
    "césares": "césar",
    "céspedes": "césped",
    "cía": "compañía",
    "cían": "ciar",
    "cías": "ciar",
    "cíceras": "cícera",
    "cíceros": "cícero",
    "cíclica": "cíclico",
    "cíclicas": "cíclico",
    "cíclicos": "cíclico",
    "cíclopea": "ciclópeo",
    "cíclopeas": "ciclópeo",
    "cíclopeos": "ciclópeo",
    "cíclopes": "cíclope",
    "cíe": "ciar",
    "cíen": "ciar",
    "cíes": "ciar",
    "cílicas": "cílica",
    "címbalos": "címbalo",
    "cínclidos": "cínclido",
    "cíngara": "cíngaro",
    "cíngaras": "cíngaro",
    "cíngaros": "cíngaro",
    "cíngulos": "cíngulo",
    "cínica": "cínico",
    "cínicas": "cínico",
    "cínicos": "cínico",
    "cío": "ciar",
    "círculos": "círculo",
    "cística": "cístico",
    "císticas": "cístico",
    "císticos": "cístico",
    "cítaras": "cítara",
    "cítolas": "cítola",
    "cítrica": "cítrico",
    "cítricas": "cítrico",
    "cítricos": "cítrico",
    "cívica": "cívico",
    "cívicas": "cívico",
    "cívicos": "cívico",
    "cóccidos": "cóccido",
    "cóccixes": "cóccix",
    "cócleas": "cóclea",
    "cócoras": "cócora",
    "cócteles": "cóctel",
    "códices": "códice",
    "códigos": "código",
    "códones": "codon",
    "cóleras": "cólera",
    "cólica": "cólico",
    "cólicas": "cólico",
    "cólicos": "cólico",
    "cólones": "colon",
    "cólquicos": "cólquico",
    "cómica": "cómico",
    "cómicas": "cómico",
    "cómicos": "cómico",
    "cómics": "cómic",
    "cómitres": "cómitre",
    "cómoda": "cómodo",
    "cómodas": "cómodo",
    "cómodos": "cómodo",
    "cómplices": "cómplice",
    "cómputos": "cómputo",
    "cóncava": "cóncavo",
    "cóncavas": "cóncavo",
    "cóncavos": "cóncavo",
    "cónclaves": "cónclave",
    "cóndilos": "cóndilo",
    "cóndores": "cóndor",
    "cónica": "cónico",
    "cónicas": "cónico",
    "cónicos": "cónico",
    "cónsules": "cónsul",
    "cónyuges": "cónyuge",
    "cópecs": "cópec",
    "cópulas": "cópula",
    "córdobas": "córdoba",
    "córiones": "corion",
    "córnea": "córneo",
    "córneas": "córneo",
    "córneos": "córneo",
    "córneres": "córner",
    "córtexs": "córtex",
    "córvidos": "córvido",
    "cósmica": "cósmico",
    "cósmicas": "cósmico",
    "cósmicos": "cósmico",
    "cótilos": "cótilo",
    "cúbica": "cúbico",
    "cúbicas": "cúbico",
    "cúbicos": "cúbico",
    "cúbitos": "cúbito",
    "cúfica": "cúfico",
    "cúficas": "cúfico",
    "cúficos": "cúfico",
    "cúmeles": "cúmel",
    "cúmplases": "cúmplase",
    "cúmulos": "cúmulo",
    "cúneos": "cúneo",
    "cúprica": "cúprico",
    "cúpricas": "cúprico",
    "cúpricos": "cúprico",
    "cúpulas": "cúpula",
    "cúrbanas": "cúrbana",
    "cúrcumas": "cúrcuma",
    "cúspides": "cúspide",
    "cúteres": "cúter",
    "dB": "decibelio",
    "da": "dar",
    "daba": "dar",
    "dabais": "dar",
    "daban": "dar",
    "dabas": "dar",
    "dables": "dable",
    "dacia": "dacio",
    "dacias": "dacio",
    "daciones": "dación",
    "dacios": "dacio",
    "dacitas": "dacita",
    "dacotas": "dacota",
    "dacriagoga": "dacriagogo",
    "dacriagogas": "dacriagogo",
    "dacriagogatresias": "dacriagogatresia",
    "dacriagogos": "dacriagogo",
    "dacrioadenalgias": "dacrioadenalgia",
    "dacrioadenectomías": "dacrioadenectomía",
    "dacrioblenorreas": "dacrioblenorrea",
    "dacriocistalgias": "dacriocistalgia",
    "dacriocistectasias": "dacriocistectasia",
    "dacriocistectomías": "dacriocistectomía",
    "dacriocistorrinostomías": "dacriocistorrinostomía",
    "dacriocistosiringotomías": "dacriocistosiringotomía",
    "dacriocistotomías": "dacriocistotomía",
    "dacriohemorreas": "dacriohemorrea",
    "dacriolitos": "dacriolito",
    "dacriomas": "dacrioma",
    "dacriones": "dacrión",
    "dacriopiorreas": "dacriopiorrea",
    "dacriorreas": "dacriorrea",
    "dacriógena": "dacriógeno",
    "dacriógenas": "dacriógeno",
    "dacriógenos": "dacriógeno",
    "dacrones": "dacrón",
    "dactilares": "dactilar",
    "dactileras": "dactilera",
    "dactiliologías": "dactiliología",
    "dactilografiaba": "dactilografiar",
    "dactilografiabais": "dactilografiar",
    "dactilografiaban": "dactilografiar",
    "dactilografiabas": "dactilografiar",
    "dactilografiad": "dactilografiar",
    "dactilografiada": "dactilografiar",
    "dactilografiadas": "dactilografiar",
    "dactilografiado": "dactilografiar",
    "dactilografiados": "dactilografiar",
    "dactilografiamos": "dactilografiar",
    "dactilografiando": "dactilografiar",
    "dactilografiara": "dactilografiar",
    "dactilografiarais": "dactilografiar",
    "dactilografiaran": "dactilografiar",
    "dactilografiaras": "dactilografiar",
    "dactilografiare": "dactilografiar",
    "dactilografiareis": "dactilografiar",
    "dactilografiaremos": "dactilografiar",
    "dactilografiaren": "dactilografiar",
    "dactilografiares": "dactilografiar",
    "dactilografiaron": "dactilografiar",
    "dactilografiará": "dactilografiar",
    "dactilografiarán": "dactilografiar",
    "dactilografiarás": "dactilografiar",
    "dactilografiaré": "dactilografiar",
    "dactilografiaréis": "dactilografiar",
    "dactilografiaría": "dactilografiar",
    "dactilografiaríais": "dactilografiar",
    "dactilografiaríamos": "dactilografiar",
    "dactilografiarían": "dactilografiar",
    "dactilografiarías": "dactilografiar",
    "dactilografiase": "dactilografiar",
    "dactilografiaseis": "dactilografiar",
    "dactilografiasen": "dactilografiar",
    "dactilografiases": "dactilografiar",
    "dactilografiaste": "dactilografiar",
    "dactilografiasteis": "dactilografiar",
    "dactilografiemos": "dactilografiar",
    "dactilografiábamos": "dactilografiar",
    "dactilografiáis": "dactilografiar",
    "dactilografiáramos": "dactilografiar",
    "dactilografiáremos": "dactilografiar",
    "dactilografiásemos": "dactilografiar",
    "dactilografié": "dactilografiar",
    "dactilografiéis": "dactilografiar",
    "dactilografió": "dactilografiar",
    "dactilografía": "dactilografiar",
    "dactilografían": "dactilografiar",
    "dactilografías": "dactilografiar",
    "dactilografíe": "dactilografiar",
    "dactilografíen": "dactilografiar",
    "dactilografíes": "dactilografiar",
    "dactilografío": "dactilografiar",
    "dactilogramas": "dactilograma",
    "dactilográfica": "dactilográfico",
    "dactilográficas": "dactilográfico",
    "dactilográficos": "dactilográfico",
    "dactilologías": "dactilología",
    "dactilopodios": "dactilopodio",
    "dactiloscopias": "dactiloscopia",
    "dactiloscópica": "dactiloscópico",
    "dactiloscópicas": "dactiloscópico",
    "dactiloscópicos": "dactiloscópico",
    "dactilógrafa": "dactilógrafo",
    "dactilógrafas": "dactilógrafo",
    "dactilógrafos": "dactilógrafo",
    "dactílica": "dactílico",
    "dactílicas": "dactílico",
    "dactílicos": "dactílico",
    "dad": "dar",
    "dada": "dar",
    "dadas": "dar",
    "dadaísmos": "dadaísmo",
    "dadaístas": "dadaísta",
    "dadivosa": "dadivoso",
    "dadivosas": "dadivoso",
    "dadivosidades": "dadivosidad",
    "dadivosos": "dadivoso",
    "dado": "dar",
    "dadora": "dador",
    "dadoras": "dador",
    "dadores": "dador",
    "dados": "dar",
    "dagas": "daga",
    "daguerrotipias": "daguerrotipia",
    "daguerrotipos": "daguerrotipo",
    "dahires": "dahir",
    "dahireses": "dahír",
    "dahirs": "dahir",
    "dahíres": "dahír",
    "dahírs": "dahír",
    "daifas": "daifa",
    "daimios": "daimio",
    "dais": "dar",
    "daishis": "daishi",
    "dalailamas": "dalailama",
    "dalas": "dala",
    "dalasis": "dalasi",
    "dalecarlias": "dalecarlia",
    "dalias": "dalia",
    "dalla": "dallar",
    "dallaba": "dallar",
    "dallabais": "dallar",
    "dallaban": "dallar",
    "dallabas": "dallar",
    "dallad": "dallar",
    "dallada": "dallar",
    "dalladas": "dallar",
    "dallado": "dallar",
    "dallados": "dallar",
    "dallamos": "dallar",
    "dallan": "dallar",
    "dallando": "dallar",
    "dallara": "dallar",
    "dallarais": "dallar",
    "dallaran": "dallar",
    "dallaras": "dallar",
    "dallare": "dallar",
    "dallareis": "dallar",
    "dallaremos": "dallar",
    "dallaren": "dallar",
    "dallares": "dallar",
    "dallaron": "dallar",
    "dallará": "dallar",
    "dallarán": "dallar",
    "dallarás": "dallar",
    "dallaré": "dallar",
    "dallaréis": "dallar",
    "dallaría": "dallar",
    "dallaríais": "dallar",
    "dallaríamos": "dallar",
    "dallarían": "dallar",
    "dallarías": "dallar",
    "dallas": "dallar",
    "dallase": "dallar",
    "dallaseis": "dallar",
    "dallasen": "dallar",
    "dallases": "dallar",
    "dallaste": "dallar",
    "dallasteis": "dallar",
    "dalle": "dallar",
    "dallemos": "dallar",
    "dallen": "dallar",
    "dalles": "dalle",
    "dallo": "dallar",
    "dallábamos": "dallar",
    "dalláis": "dallar",
    "dalláramos": "dallar",
    "dalláremos": "dallar",
    "dallásemos": "dallar",
    "dallé": "dallar",
    "dalléis": "dallar",
    "dalló": "dallar",
    "dalmaciana": "dalmaciano",
    "dalmacianas": "dalmaciano",
    "dalmacianos": "dalmaciano",
    "dalmáticas": "dalmática",
    "dalmáticos": "dalmático",
    "daltoniana": "daltoniano",
    "daltonianas": "daltoniano",
    "daltonianos": "daltoniano",
    "daltonismos": "daltonismo",
    "daltónica": "daltónico",
    "daltónicas": "daltónico",
    "daltónicos": "daltónico",
    "damajaguas": "damajagua",
    "damajuanas": "damajuana",
    "damaliscos": "damalisco",
    "damanes": "damán",
    "damaras": "damara",
    "damas": "dama",
    "damascada": "damascado",
    "damascadas": "damascado",
    "damascados": "damascado",
    "damascena": "damasceno",
    "damascenas": "damasceno",
    "damascenos": "damasceno",
    "damascos": "damasco",
    "damasinas": "damasina",
    "damasquillos": "damasquillo",
    "damasquina": "damasquino",
    "damasquinaba": "damasquinar",
    "damasquinabais": "damasquinar",
    "damasquinaban": "damasquinar",
    "damasquinabas": "damasquinar",
    "damasquinad": "damasquinar",
    "damasquinada": "damasquinar",
    "damasquinadas": "damasquinar",
    "damasquinado": "damasquinar",
    "damasquinados": "damasquinar",
    "damasquinamos": "damasquinar",
    "damasquinan": "damasquinar",
    "damasquinando": "damasquinar",
    "damasquinara": "damasquinar",
    "damasquinarais": "damasquinar",
    "damasquinaran": "damasquinar",
    "damasquinaras": "damasquinar",
    "damasquinare": "damasquinar",
    "damasquinareis": "damasquinar",
    "damasquinaremos": "damasquinar",
    "damasquinaren": "damasquinar",
    "damasquinares": "damasquinar",
    "damasquinaron": "damasquinar",
    "damasquinará": "damasquinar",
    "damasquinarán": "damasquinar",
    "damasquinarás": "damasquinar",
    "damasquinaré": "damasquinar",
    "damasquinaréis": "damasquinar",
    "damasquinaría": "damasquinar",
    "damasquinaríais": "damasquinar",
    "damasquinaríamos": "damasquinar",
    "damasquinarían": "damasquinar",
    "damasquinarías": "damasquinar",
    "damasquinas": "damasquino",
    "damasquinase": "damasquinar",
    "damasquinaseis": "damasquinar",
    "damasquinasen": "damasquinar",
    "damasquinases": "damasquinar",
    "damasquinaste": "damasquinar",
    "damasquinasteis": "damasquinar",
    "damasquine": "damasquinar",
    "damasquinemos": "damasquinar",
    "damasquinen": "damasquinar",
    "damasquines": "damasquinar",
    "damasquino": "damasquinar",
    "damasquinos": "damasquino",
    "damasquinábamos": "damasquinar",
    "damasquináis": "damasquinar",
    "damasquináramos": "damasquinar",
    "damasquináremos": "damasquinar",
    "damasquinásemos": "damasquinar",
    "damasquiné": "damasquinar",
    "damasquinéis": "damasquinar",
    "damasquinó": "damasquinar",
    "damerogramas": "damerograma",
    "dameros": "damero",
    "damerías": "damería",
    "damiselas": "damisela",
    "damnifica": "damnificar",
    "damnificaba": "damnificar",
    "damnificabais": "damnificar",
    "damnificaban": "damnificar",
    "damnificabas": "damnificar",
    "damnificaciones": "damnificación",
    "damnificad": "damnificar",
    "damnificada": "damnificar",
    "damnificadas": "damnificar",
    "damnificado": "damnificar",
    "damnificados": "damnificar",
    "damnificamos": "damnificar",
    "damnifican": "damnificar",
    "damnificando": "damnificar",
    "damnificara": "damnificar",
    "damnificarais": "damnificar",
    "damnificaran": "damnificar",
    "damnificaras": "damnificar",
    "damnificare": "damnificar",
    "damnificareis": "damnificar",
    "damnificaremos": "damnificar",
    "damnificaren": "damnificar",
    "damnificares": "damnificar",
    "damnificaron": "damnificar",
    "damnificará": "damnificar",
    "damnificarán": "damnificar",
    "damnificarás": "damnificar",
    "damnificaré": "damnificar",
    "damnificaréis": "damnificar",
    "damnificaría": "damnificar",
    "damnificaríais": "damnificar",
    "damnificaríamos": "damnificar",
    "damnificarían": "damnificar",
    "damnificarías": "damnificar",
    "damnificas": "damnificar",
    "damnificase": "damnificar",
    "damnificaseis": "damnificar",
    "damnificasen": "damnificar",
    "damnificases": "damnificar",
    "damnificaste": "damnificar",
    "damnificasteis": "damnificar",
    "damnifico": "damnificar",
    "damnificábamos": "damnificar",
    "damnificáis": "damnificar",
    "damnificáramos": "damnificar",
    "damnificáremos": "damnificar",
    "damnificásemos": "damnificar",
    "damnificó": "damnificar",
    "damnifique": "damnificar",
    "damnifiquemos": "damnificar",
    "damnifiquen": "damnificar",
    "damnifiques": "damnificar",
    "damnifiqué": "damnificar",
    "damnifiquéis": "damnificar",
    "damos": "dar",
    "dan": "dar",
    "danaides": "danaide",
    "danburitas": "danburita",
    "dance": "danzar",
    "dancemos": "danzar",
    "dancen": "danzar",
    "dances": "danzar",
    "danchada": "danchado",
    "danchadas": "danchado",
    "danchados": "danchado",
    "dancinges": "dancing",
    "dancings": "dancing",
    "dancé": "danzar",
    "dancéis": "danzar",
    "dandies": "dandi",
    "dandis": "dandi",
    "dando": "dar",
    "dandys": "dandy",
    "danesa": "danés",
    "danesas": "danés",
    "daneses": "danés",
    "dantas": "danta",
    "dantellada": "dantellado",
    "dantelladas": "dantellado",
    "dantellados": "dantellado",
    "dantesca": "dantesco",
    "dantescas": "dantesco",
    "dantescos": "dantesco",
    "danubiana": "danubiano",
    "danubianas": "danubiano",
    "danubianos": "danubiano",
    "danza": "danzar",
    "danzaba": "danzar",
    "danzabais": "danzar",
    "danzaban": "danzar",
    "danzabas": "danzar",
    "danzad": "danzar",
    "danzada": "danzar",
    "danzadas": "danzar",
    "danzado": "danzar",
    "danzados": "danzar",
    "danzamos": "danzar",
    "danzan": "danzar",
    "danzando": "danzar",
    "danzara": "danzar",
    "danzarais": "danzar",
    "danzaran": "danzar",
    "danzaras": "danzar",
    "danzare": "danzar",
    "danzareis": "danzar",
    "danzaremos": "danzar",
    "danzaren": "danzar",
    "danzares": "danzar",
    "danzaron": "danzar",
    "danzará": "danzar",
    "danzarán": "danzar",
    "danzarás": "danzar",
    "danzaré": "danzar",
    "danzaréis": "danzar",
    "danzaría": "danzar",
    "danzaríais": "danzar",
    "danzaríamos": "danzar",
    "danzarían": "danzar",
    "danzarías": "danzar",
    "danzas": "danzar",
    "danzase": "danzar",
    "danzaseis": "danzar",
    "danzasen": "danzar",
    "danzases": "danzar",
    "danzaste": "danzar",
    "danzasteis": "danzar",
    "danzo": "danzar",
    "danzábamos": "danzar",
    "danzáis": "danzar",
    "danzáramos": "danzar",
    "danzáremos": "danzar",
    "danzásemos": "danzar",
    "danzó": "danzar",
    "dardabasíes": "dardabasí",
    "dardania": "dardanio",
    "dardanias": "dardanio",
    "dardanios": "dardanio",
    "dardanismos": "dardanismo",
    "dardos": "dardo",
    "daremos": "dar",
    "darviniana": "darviniano",
    "darvinianas": "darviniano",
    "darvinianos": "darviniano",
    "darvinismos": "darvinismo",
    "darvinistas": "darvinista",
    "darwinismos": "darwinismo",
    "darwinistas": "darwinista",
    "dará": "dar",
    "darán": "dar",
    "darás": "dar",
    "daré": "dar",
    "daréis": "dar",
    "daría": "dar",
    "daríais": "dar",
    "daríamos": "dar",
    "darían": "dar",
    "darías": "dar",
    "das": "dar",
    "dasimétrica": "dasimétrico",
    "dasimétricas": "dasimétrico",
    "dasimétricos": "dasimétrico",
    "dasiuros": "dasiuro",
    "dasocracias": "dasocracia",
    "dasocrática": "dasocrático",
    "dasocráticas": "dasocrático",
    "dasocráticos": "dasocrático",
    "dasonomías": "dasonomía",
    "dasonómica": "dasonómico",
    "dasonómicas": "dasonómico",
    "dasonómicos": "dasonómico",
    "dasímetros": "dasímetro",
    "data": "datar",
    "databa": "datar",
    "databais": "datar",
    "databan": "datar",
    "databas": "datar",
    "dataciones": "datación",
    "datad": "datar",
    "datada": "datar",
    "datadas": "datar",
    "datado": "datar",
    "datados": "datar",
    "datafaxes": "datafax",
    "datagramas": "datagrama",
    "datamos": "datar",
    "datan": "datar",
    "datando": "datar",
    "datara": "datar",
    "datarais": "datar",
    "dataran": "datar",
    "dataras": "datar",
    "datare": "datar",
    "datareis": "datar",
    "dataremos": "datar",
    "dataren": "datar",
    "datares": "datar",
    "dataron": "datar",
    "datará": "datar",
    "datarán": "datar",
    "datarás": "datar",
    "dataré": "datar",
    "dataréis": "datar",
    "dataría": "datar",
    "dataríais": "datar",
    "dataríamos": "datar",
    "datarían": "datar",
    "datarías": "dataría",
    "datas": "datar",
    "datase": "datar",
    "dataseis": "datar",
    "datasen": "datar",
    "datases": "datar",
    "dataste": "datar",
    "datasteis": "datar",
    "date": "datar",
    "datemos": "datar",
    "daten": "datar",
    "dates": "datar",
    "datilera": "datilero",
    "datileras": "datilero",
    "datileros": "datilero",
    "datismos": "datismo",
    "dativa": "dativo",
    "dativas": "dativo",
    "dativos": "dativo",
    "dato": "datar",
    "datolitas": "datolita",
    "datos": "dato",
    "datábamos": "datar",
    "datáis": "datar",
    "datáramos": "datar",
    "datáremos": "datar",
    "datásemos": "datar",
    "daté": "datar",
    "datéis": "datar",
    "dató": "datar",
    "daucos": "dauco",
    "davídica": "davídico",
    "davídicas": "davídico",
    "davídicos": "davídico",
    "daña": "dañar",
    "dañaba": "dañar",
    "dañabais": "dañar",
    "dañaban": "dañar",
    "dañabas": "dañar",
    "dañad": "dañar",
    "dañada": "dañar",
    "dañadas": "dañar",
    "dañado": "dañar",
    "dañados": "dañar",
    "dañamos": "dañar",
    "dañan": "dañar",
    "dañando": "dañar",
    "dañara": "dañar",
    "dañarais": "dañar",
    "dañaran": "dañar",
    "dañaras": "dañar",
    "dañare": "dañar",
    "dañareis": "dañar",
    "dañaremos": "dañar",
    "dañaren": "dañar",
    "dañares": "dañar",
    "dañaron": "dañar",
    "dañará": "dañar",
    "dañarán": "dañar",
    "dañarás": "dañar",
    "dañaré": "dañar",
    "dañaréis": "dañar",
    "dañaría": "dañar",
    "dañaríais": "dañar",
    "dañaríamos": "dañar",
    "dañarían": "dañar",
    "dañarías": "dañar",
    "dañas": "dañar",
    "dañase": "dañar",
    "dañaseis": "dañar",
    "dañasen": "dañar",
    "dañases": "dañar",
    "dañaste": "dañar",
    "dañasteis": "dañar",
    "dañe": "dañar",
    "dañemos": "dañar",
    "dañen": "dañar",
    "dañes": "dañar",
    "dañina": "dañino",
    "dañinas": "dañino",
    "dañinos": "dañino",
    "daño": "dañar",
    "daños": "daño",
    "dañosa": "dañoso",
    "dañosas": "dañoso",
    "dañosos": "dañoso",
    "dañábamos": "dañar",
    "dañáis": "dañar",
    "dañáramos": "dañar",
    "dañáremos": "dañar",
    "dañásemos": "dañar",
    "dañé": "dañar",
    "dañéis": "dañar",
    "dañó": "dañar",
    "deambula": "deambular",
    "deambulaba": "deambular",
    "deambulabais": "deambular",
    "deambulaban": "deambular",
    "deambulabas": "deambular",
    "deambulad": "deambular",
    "deambulada": "deambular",
    "deambuladas": "deambular",
    "deambulado": "deambular",
    "deambulados": "deambular",
    "deambulamos": "deambular",
    "deambulan": "deambular",
    "deambulando": "deambular",
    "deambulara": "deambular",
    "deambularais": "deambular",
    "deambularan": "deambular",
    "deambularas": "deambular",
    "deambulare": "deambular",
    "deambulareis": "deambular",
    "deambularemos": "deambular",
    "deambularen": "deambular",
    "deambulares": "deambular",
    "deambularon": "deambular",
    "deambulará": "deambular",
    "deambularán": "deambular",
    "deambularás": "deambular",
    "deambularé": "deambular",
    "deambularéis": "deambular",
    "deambularía": "deambular",
    "deambularíais": "deambular",
    "deambularíamos": "deambular",
    "deambularían": "deambular",
    "deambularías": "deambular",
    "deambulas": "deambular",
    "deambulase": "deambular",
    "deambulaseis": "deambular",
    "deambulasen": "deambular",
    "deambulases": "deambular",
    "deambulaste": "deambular",
    "deambulasteis": "deambular",
    "deambulatorios": "deambulatorio",
    "deambule": "deambular",
    "deambulemos": "deambular",
    "deambulen": "deambular",
    "deambules": "deambular",
    "deambulo": "deambular",
    "deambulábamos": "deambular",
    "deambuláis": "deambular",
    "deambuláramos": "deambular",
    "deambuláremos": "deambular",
    "deambulásemos": "deambular",
    "deambulé": "deambular",
    "deambuléis": "deambular",
    "deambuló": "deambular",
    "deanatos": "deanato",
    "deanazgos": "deanazgo",
    "deanes": "deán",
    "deas": "dea",
    "deba": "deber",
    "debacles": "debacle",
    "debamos": "deber",
    "deban": "deber",
    "debas": "deber",
    "debata": "debatir",
    "debatamos": "debatir",
    "debatan": "debatir",
    "debatas": "debatir",
    "debate": "debatir",
    "debaten": "debatir",
    "debates": "debatir",
    "debatid": "debatir",
    "debatida": "debatir",
    "debatidas": "debatir",
    "debatido": "debatir",
    "debatidos": "debatir",
    "debatiendo": "debatir",
    "debatiera": "debatir",
    "debatierais": "debatir",
    "debatieran": "debatir",
    "debatieras": "debatir",
    "debatiere": "debatir",
    "debatiereis": "debatir",
    "debatieren": "debatir",
    "debatieres": "debatir",
    "debatieron": "debatir",
    "debatiese": "debatir",
    "debatieseis": "debatir",
    "debatiesen": "debatir",
    "debatieses": "debatir",
    "debatimos": "debatir",
    "debatiremos": "debatir",
    "debatirá": "debatir",
    "debatirán": "debatir",
    "debatirás": "debatir",
    "debatiré": "debatir",
    "debatiréis": "debatir",
    "debatiría": "debatir",
    "debatiríais": "debatir",
    "debatiríamos": "debatir",
    "debatirían": "debatir",
    "debatirías": "debatir",
    "debatiste": "debatir",
    "debatisteis": "debatir",
    "debatiéramos": "debatir",
    "debatiéremos": "debatir",
    "debatiésemos": "debatir",
    "debatió": "debatir",
    "debato": "debatir",
    "debatáis": "debatir",
    "debatí": "debatir",
    "debatía": "debatir",
    "debatíais": "debatir",
    "debatíamos": "debatir",
    "debatían": "debatir",
    "debatías": "debatir",
    "debatís": "debatir",
    "debe": "deber",
    "debed": "deber",
    "debela": "debelar",
    "debelaba": "debelar",
    "debelabais": "debelar",
    "debelaban": "debelar",
    "debelabas": "debelar",
    "debelaciones": "debelación",
    "debelad": "debelar",
    "debelada": "debelar",
    "debeladas": "debelar",
    "debelado": "debelar",
    "debeladora": "debelador",
    "debeladoras": "debelador",
    "debeladores": "debelador",
    "debelados": "debelar",
    "debelamos": "debelar",
    "debelan": "debelar",
    "debelando": "debelar",
    "debelara": "debelar",
    "debelarais": "debelar",
    "debelaran": "debelar",
    "debelaras": "debelar",
    "debelare": "debelar",
    "debelareis": "debelar",
    "debelaremos": "debelar",
    "debelaren": "debelar",
    "debelares": "debelar",
    "debelaron": "debelar",
    "debelará": "debelar",
    "debelarán": "debelar",
    "debelarás": "debelar",
    "debelaré": "debelar",
    "debelaréis": "debelar",
    "debelaría": "debelar",
    "debelaríais": "debelar",
    "debelaríamos": "debelar",
    "debelarían": "debelar",
    "debelarías": "debelar",
    "debelas": "debelar",
    "debelase": "debelar",
    "debelaseis": "debelar",
    "debelasen": "debelar",
    "debelases": "debelar",
    "debelaste": "debelar",
    "debelasteis": "debelar",
    "debele": "debelar",
    "debelemos": "debelar",
    "debelen": "debelar",
    "debeles": "debelar",
    "debelo": "debelar",
    "debelábamos": "debelar",
    "debeláis": "debelar",
    "debeláramos": "debelar",
    "debeláremos": "debelar",
    "debelásemos": "debelar",
    "debelé": "debelar",
    "debeléis": "debelar",
    "debeló": "debelar",
    "debemos": "deber",
    "deben": "deber",
    "deberemos": "deber",
    "deberes": "deber",
    "deberá": "deber",
    "deberán": "deber",
    "deberás": "deber",
    "deberé": "deber",
    "deberéis": "deber",
    "debería": "deber",
    "deberíais": "deber",
    "deberíamos": "deber",
    "deberían": "deber",
    "deberías": "deber",
    "debes": "deber",
    "debida": "deber",
    "debidas": "deber",
    "debido": "deber",
    "debidos": "deber",
    "debiendo": "deber",
    "debiera": "deber",
    "debierais": "deber",
    "debieran": "deber",
    "debieras": "deber",
    "debiere": "deber",
    "debiereis": "deber",
    "debieren": "deber",
    "debieres": "deber",
    "debieron": "deber",
    "debiese": "deber",
    "debieseis": "deber",
    "debiesen": "deber",
    "debieses": "deber",
    "debilidades": "debilidad",
    "debilita": "debilitar",
    "debilitaba": "debilitar",
    "debilitabais": "debilitar",
    "debilitaban": "debilitar",
    "debilitabas": "debilitar",
    "debilitaciones": "debilitación",
    "debilitad": "debilitar",
    "debilitada": "debilitar",
    "debilitadas": "debilitar",
    "debilitado": "debilitar",
    "debilitados": "debilitar",
    "debilitamientos": "debilitamiento",
    "debilitamos": "debilitar",
    "debilitan": "debilitar",
    "debilitando": "debilitar",
    "debilitara": "debilitar",
    "debilitarais": "debilitar",
    "debilitaran": "debilitar",
    "debilitaras": "debilitar",
    "debilitare": "debilitar",
    "debilitareis": "debilitar",
    "debilitaremos": "debilitar",
    "debilitaren": "debilitar",
    "debilitares": "debilitar",
    "debilitaron": "debilitar",
    "debilitará": "debilitar",
    "debilitarán": "debilitar",
    "debilitarás": "debilitar",
    "debilitaré": "debilitar",
    "debilitaréis": "debilitar",
    "debilitaría": "debilitar",
    "debilitaríais": "debilitar",
    "debilitaríamos": "debilitar",
    "debilitarían": "debilitar",
    "debilitarías": "debilitar",
    "debilitas": "debilitar",
    "debilitase": "debilitar",
    "debilitaseis": "debilitar",
    "debilitasen": "debilitar",
    "debilitases": "debilitar",
    "debilitaste": "debilitar",
    "debilitasteis": "debilitar",
    "debilite": "debilitar",
    "debilitemos": "debilitar",
    "debiliten": "debilitar",
    "debilites": "debilitar",
    "debilito": "debilitar",
    "debilitábamos": "debilitar",
    "debilitáis": "debilitar",
    "debilitáramos": "debilitar",
    "debilitáremos": "debilitar",
    "debilitásemos": "debilitar",
    "debilité": "debilitar",
    "debilitéis": "debilitar",
    "debilitó": "debilitar",
    "debilucha": "debilucho",
    "debiluchas": "debilucho",
    "debiluchos": "debilucho",
    "debimos": "deber",
    "debiste": "deber",
    "debisteis": "deber",
    "debita": "debitar",
    "debitaba": "debitar",
    "debitabais": "debitar",
    "debitaban": "debitar",
    "debitabas": "debitar",
    "debitad": "debitar",
    "debitada": "debitar",
    "debitadas": "debitar",
    "debitado": "debitar",
    "debitados": "debitar",
    "debitamos": "debitar",
    "debitan": "debitar",
    "debitando": "debitar",
    "debitara": "debitar",
    "debitarais": "debitar",
    "debitaran": "debitar",
    "debitaras": "debitar",
    "debitare": "debitar",
    "debitareis": "debitar",
    "debitaremos": "debitar",
    "debitaren": "debitar",
    "debitares": "debitar",
    "debitaron": "debitar",
    "debitará": "debitar",
    "debitarán": "debitar",
    "debitarás": "debitar",
    "debitaré": "debitar",
    "debitaréis": "debitar",
    "debitaría": "debitar",
    "debitaríais": "debitar",
    "debitaríamos": "debitar",
    "debitarían": "debitar",
    "debitarías": "debitar",
    "debitas": "debitar",
    "debitase": "debitar",
    "debitaseis": "debitar",
    "debitasen": "debitar",
    "debitases": "debitar",
    "debitaste": "debitar",
    "debitasteis": "debitar",
    "debite": "debitar",
    "debitemos": "debitar",
    "debiten": "debitar",
    "debites": "debitar",
    "debito": "debitar",
    "debitábamos": "debitar",
    "debitáis": "debitar",
    "debitáramos": "debitar",
    "debitáremos": "debitar",
    "debitásemos": "debitar",
    "debité": "debitar",
    "debitéis": "debitar",
    "debitó": "debitar",
    "debiéramos": "deber",
    "debiéremos": "deber",
    "debiésemos": "deber",
    "debió": "deber",
    "deblas": "debla",
    "debo": "deber",
    "deboeses": "debó",
    "debuggers": "debugger",
    "debuta": "debutar",
    "debutaba": "debutar",
    "debutabais": "debutar",
    "debutaban": "debutar",
    "debutabas": "debutar",
    "debutad": "debutar",
    "debutada": "debutar",
    "debutadas": "debutar",
    "debutado": "debutar",
    "debutados": "debutar",
    "debutamos": "debutar",
    "debutan": "debutar",
    "debutando": "debutar",
    "debutantes": "debutante",
    "debutara": "debutar",
    "debutarais": "debutar",
    "debutaran": "debutar",
    "debutaras": "debutar",
    "debutare": "debutar",
    "debutareis": "debutar",
    "debutaremos": "debutar",
    "debutaren": "debutar",
    "debutares": "debutar",
    "debutaron": "debutar",
    "debutará": "debutar",
    "debutarán": "debutar",
    "debutarás": "debutar",
    "debutaré": "debutar",
    "debutaréis": "debutar",
    "debutaría": "debutar",
    "debutaríais": "debutar",
    "debutaríamos": "debutar",
    "debutarían": "debutar",
    "debutarías": "debutar",
    "debutas": "debutar",
    "debutase": "debutar",
    "debutaseis": "debutar",
    "debutasen": "debutar",
    "debutases": "debutar",
    "debutaste": "debutar",
    "debutasteis": "debutar",
    "debute": "debutar",
    "debutemos": "debutar",
    "debuten": "debutar",
    "debutes": "debutar",
    "debuto": "debutar",
    "debuts": "debut",
    "debutábamos": "debutar",
    "debutáis": "debutar",
    "debutáramos": "debutar",
    "debutáremos": "debutar",
    "debutásemos": "debutar",
    "debuté": "debutar",
    "debutéis": "debutar",
    "debutó": "debutar",
    "debáis": "deber",
    "debéis": "deber",
    "debí": "deber",
    "debía": "deber",
    "debíais": "deber",
    "debíamos": "deber",
    "debían": "deber",
    "debías": "deber",
    "debús": "debú",
    "decaboranos": "decaborano",
    "decabristas": "decabrista",
    "decadactilares": "decadactilar",
    "decadencias": "decadencia",
    "decadentes": "decadente",
    "decadentismos": "decadentismo",
    "decadentistas": "decadentista",
    "decae": "decaer",
    "decaed": "decaer",
    "decaedros": "decaedro",
    "decaemos": "decaer",
    "decaen": "decaer",
    "decaeremos": "decaer",
    "decaerá": "decaer",
    "decaerán": "decaer",
    "decaerás": "decaer",
    "decaeré": "decaer",
    "decaeréis": "decaer",
    "decaería": "decaer",
    "decaeríais": "decaer",
    "decaeríamos": "decaer",
    "decaerían": "decaer",
    "decaerías": "decaer",
    "decaes": "decaer",
    "decagonales": "decagonal",
    "decagramos": "decagramo",
    "decahidronaftalenos": "decahidronaftaleno",
    "decaiga": "decaer",
    "decaigamos": "decaer",
    "decaigan": "decaer",
    "decaigas": "decaer",
    "decaigo": "decaer",
    "decaigáis": "decaer",
    "decaimientos": "decaimiento",
    "decalajes": "decalaje",
    "decalcificaciones": "decalcificación",
    "decalescencias": "decalescencia",
    "decalescentes": "decalescente",
    "decalitros": "decalitro",
    "decalvaciones": "decalvación",
    "decampa": "decampar",
    "decampaba": "decampar",
    "decampabais": "decampar",
    "decampaban": "decampar",
    "decampabas": "decampar",
    "decampad": "decampar",
    "decampada": "decampar",
    "decampadas": "decampar",
    "decampado": "decampar",
    "decampados": "decampar",
    "decampamos": "decampar",
    "decampan": "decampar",
    "decampando": "decampar",
    "decampara": "decampar",
    "decamparais": "decampar",
    "decamparan": "decampar",
    "decamparas": "decampar",
    "decampare": "decampar",
    "decampareis": "decampar",
    "decamparemos": "decampar",
    "decamparen": "decampar",
    "decampares": "decampar",
    "decamparon": "decampar",
    "decampará": "decampar",
    "decamparán": "decampar",
    "decamparás": "decampar",
    "decamparé": "decampar",
    "decamparéis": "decampar",
    "decamparía": "decampar",
    "decamparíais": "decampar",
    "decamparíamos": "decampar",
    "decamparían": "decampar",
    "decamparías": "decampar",
    "decampas": "decampar",
    "decampase": "decampar",
    "decampaseis": "decampar",
    "decampasen": "decampar",
    "decampases": "decampar",
    "decampaste": "decampar",
    "decampasteis": "decampar",
    "decampe": "decampar",
    "decampemos": "decampar",
    "decampen": "decampar",
    "decampes": "decampar",
    "decampo": "decampar",
    "decampábamos": "decampar",
    "decampáis": "decampar",
    "decampáramos": "decampar",
    "decampáremos": "decampar",
    "decampásemos": "decampar",
    "decampé": "decampar",
    "decampéis": "decampar",
    "decampó": "decampar",
    "decamétrica": "decamétrico",
    "decamétricas": "decamétrico",
    "decamétricos": "decamétrico",
    "decana": "decano",
    "decanas": "decano",
    "decanatos": "decanato",
    "decanias": "decania",
    "decanos": "decano",
    "decanta": "decantar",
    "decantaba": "decantar",
    "decantabais": "decantar",
    "decantaban": "decantar",
    "decantabas": "decantar",
    "decantaciones": "decantación",
    "decantad": "decantar",
    "decantada": "decantar",
    "decantadas": "decantar",
    "decantado": "decantar",
    "decantadores": "decantador",
    "decantados": "decantar",
    "decantamientos": "decantamiento",
    "decantamos": "decantar",
    "decantan": "decantar",
    "decantando": "decantar",
    "decantara": "decantar",
    "decantarais": "decantar",
    "decantaran": "decantar",
    "decantaras": "decantar",
    "decantare": "decantar",
    "decantareis": "decantar",
    "decantaremos": "decantar",
    "decantaren": "decantar",
    "decantares": "decantar",
    "decantaron": "decantar",
    "decantará": "decantar",
    "decantarán": "decantar",
    "decantarás": "decantar",
    "decantaré": "decantar",
    "decantaréis": "decantar",
    "decantaría": "decantar",
    "decantaríais": "decantar",
    "decantaríamos": "decantar",
    "decantarían": "decantar",
    "decantarías": "decantar",
    "decantas": "decantar",
    "decantase": "decantar",
    "decantaseis": "decantar",
    "decantasen": "decantar",
    "decantases": "decantar",
    "decantaste": "decantar",
    "decantasteis": "decantar",
    "decante": "decantar",
    "decantemos": "decantar",
    "decanten": "decantar",
    "decantes": "decantar",
    "decanto": "decantar",
    "decantábamos": "decantar",
    "decantáis": "decantar",
    "decantáramos": "decantar",
    "decantáremos": "decantar",
    "decantásemos": "decantar",
    "decanté": "decantar",
    "decantéis": "decantar",
    "decantó": "decantar",
    "decapa": "decapar",
    "decapaba": "decapar",
    "decapabais": "decapar",
    "decapaban": "decapar",
    "decapabas": "decapar",
    "decapad": "decapar",
    "decapada": "decapar",
    "decapadas": "decapar",
    "decapado": "decapar",
    "decapados": "decapar",
    "decapamos": "decapar",
    "decapan": "decapar",
    "decapando": "decapar",
    "decapara": "decapar",
    "decaparais": "decapar",
    "decaparan": "decapar",
    "decaparas": "decapar",
    "decapare": "decapar",
    "decapareis": "decapar",
    "decaparemos": "decapar",
    "decaparen": "decapar",
    "decapares": "decapar",
    "decaparon": "decapar",
    "decapará": "decapar",
    "decaparán": "decapar",
    "decaparás": "decapar",
    "decaparé": "decapar",
    "decaparéis": "decapar",
    "decaparía": "decapar",
    "decaparíais": "decapar",
    "decaparíamos": "decapar",
    "decaparían": "decapar",
    "decaparías": "decapar",
    "decapas": "decapar",
    "decapase": "decapar",
    "decapaseis": "decapar",
    "decapasen": "decapar",
    "decapases": "decapar",
    "decapaste": "decapar",
    "decapasteis": "decapar",
    "decape": "decapar",
    "decapemos": "decapar",
    "decapen": "decapar",
    "decapes": "decapar",
    "decapita": "decapitar",
    "decapitaba": "decapitar",
    "decapitabais": "decapitar",
    "decapitaban": "decapitar",
    "decapitabas": "decapitar",
    "decapitaciones": "decapitación",
    "decapitad": "decapitar",
    "decapitada": "decapitar",
    "decapitadas": "decapitar",
    "decapitado": "decapitar",
    "decapitados": "decapitar",
    "decapitamos": "decapitar",
    "decapitan": "decapitar",
    "decapitando": "decapitar",
    "decapitara": "decapitar",
    "decapitarais": "decapitar",
    "decapitaran": "decapitar",
    "decapitaras": "decapitar",
    "decapitare": "decapitar",
    "decapitareis": "decapitar",
    "decapitaremos": "decapitar",
    "decapitaren": "decapitar",
    "decapitares": "decapitar",
    "decapitaron": "decapitar",
    "decapitará": "decapitar",
    "decapitarán": "decapitar",
    "decapitarás": "decapitar",
    "decapitaré": "decapitar",
    "decapitaréis": "decapitar",
    "decapitaría": "decapitar",
    "decapitaríais": "decapitar",
    "decapitaríamos": "decapitar",
    "decapitarían": "decapitar",
    "decapitarías": "decapitar",
    "decapitas": "decapitar",
    "decapitase": "decapitar",
    "decapitaseis": "decapitar",
    "decapitasen": "decapitar",
    "decapitases": "decapitar",
    "decapitaste": "decapitar",
    "decapitasteis": "decapitar",
    "decapite": "decapitar",
    "decapitemos": "decapitar",
    "decapiten": "decapitar",
    "decapites": "decapitar",
    "decapito": "decapitar",
    "decapitábamos": "decapitar",
    "decapitáis": "decapitar",
    "decapitáramos": "decapitar",
    "decapitáremos": "decapitar",
    "decapitásemos": "decapitar",
    "decapité": "decapitar",
    "decapitéis": "decapitar",
    "decapitó": "decapitar",
    "decapo": "decapar",
    "decapábamos": "decapar",
    "decapáis": "decapar",
    "decapáramos": "decapar",
    "decapáremos": "decapar",
    "decapásemos": "decapar",
    "decapé": "decapar",
    "decapéis": "decapar",
    "decapó": "decapar",
    "decarbonilaciones": "decarbonilación",
    "decarboxilaciones": "decarboxilación",
    "decarboxilasas": "decarboxilasa",
    "decarburaciones": "decarburación",
    "decasílaba": "decasílabo",
    "decasílabas": "decasílabo",
    "decasílabos": "decasílabo",
    "decathlones": "decathlon",
    "decathlons": "decathlon",
    "decatizadoras": "decatizadora",
    "decatizados": "decatizado",
    "decatlones": "decatlón",
    "decatloniana": "decatloniano",
    "decatlonianas": "decatloniano",
    "decatlonianos": "decatloniano",
    "decayendo": "decaer",
    "decayera": "decaer",
    "decayerais": "decaer",
    "decayeran": "decaer",
    "decayeras": "decaer",
    "decayere": "decaer",
    "decayereis": "decaer",
    "decayeren": "decaer",
    "decayeres": "decaer",
    "decayeron": "decaer",
    "decayese": "decaer",
    "decayeseis": "decaer",
    "decayesen": "decaer",
    "decayeses": "decaer",
    "decayéramos": "decaer",
    "decayéremos": "decaer",
    "decayésemos": "decaer",
    "decayó": "decaer",
    "decaéis": "decaer",
    "decaí": "decaer",
    "decaía": "decaer",
    "decaíais": "decaer",
    "decaíamos": "decaer",
    "decaían": "decaer",
    "decaías": "decaer",
    "decaída": "decaer",
    "decaídas": "decaer",
    "decaído": "decaer",
    "decaídos": "decaer",
    "decaímos": "decaer",
    "decaíste": "decaer",
    "decaísteis": "decaer",
    "deccas": "decca",
    "deceleraciones": "deceleración",
    "deceleradora": "decelerador",
    "deceleradoras": "decelerador",
    "deceleradores": "decelerador",
    "decembristas": "decembrista",
    "decenales": "decenal",
    "decenares": "decenar",
    "decenaria": "decenario",
    "decenarias": "decenario",
    "decenarios": "decenario",
    "decenas": "decena",
    "decencias": "decencia",
    "decenios": "decenio",
    "decentaba": "decentar",
    "decentabais": "decentar",
    "decentaban": "decentar",
    "decentabas": "decentar",
    "decentad": "decentar",
    "decentada": "decentar",
    "decentadas": "decentar",
    "decentado": "decentar",
    "decentados": "decentar",
    "decentamos": "decentar",
    "decentando": "decentar",
    "decentara": "decentar",
    "decentarais": "decentar",
    "decentaran": "decentar",
    "decentaras": "decentar",
    "decentare": "decentar",
    "decentareis": "decentar",
    "decentaremos": "decentar",
    "decentaren": "decentar",
    "decentares": "decentar",
    "decentaron": "decentar",
    "decentará": "decentar",
    "decentarán": "decentar",
    "decentarás": "decentar",
    "decentaré": "decentar",
    "decentaréis": "decentar",
    "decentaría": "decentar",
    "decentaríais": "decentar",
    "decentaríamos": "decentar",
    "decentarían": "decentar",
    "decentarías": "decentar",
    "decentase": "decentar",
    "decentaseis": "decentar",
    "decentasen": "decentar",
    "decentases": "decentar",
    "decentaste": "decentar",
    "decentasteis": "decentar",
    "decentemos": "decentar",
    "decentes": "decente",
    "decentábamos": "decentar",
    "decentáis": "decentar",
    "decentáramos": "decentar",
    "decentáremos": "decentar",
    "decentásemos": "decentar",
    "decenté": "decentar",
    "decentéis": "decentar",
    "decentó": "decentar",
    "decenviratos": "decenvirato",
    "decenviros": "decenviro",
    "decepciona": "decepcionar",
    "decepcionaba": "decepcionar",
    "decepcionabais": "decepcionar",
    "decepcionaban": "decepcionar",
    "decepcionabas": "decepcionar",
    "decepcionad": "decepcionar",
    "decepcionada": "decepcionar",
    "decepcionadas": "decepcionar",
    "decepcionado": "decepcionar",
    "decepcionados": "decepcionar",
    "decepcionamos": "decepcionar",
    "decepcionan": "decepcionar",
    "decepcionando": "decepcionar",
    "decepcionantes": "decepcionante",
    "decepcionara": "decepcionar",
    "decepcionarais": "decepcionar",
    "decepcionaran": "decepcionar",
    "decepcionaras": "decepcionar",
    "decepcionare": "decepcionar",
    "decepcionareis": "decepcionar",
    "decepcionaremos": "decepcionar",
    "decepcionaren": "decepcionar",
    "decepcionares": "decepcionar",
    "decepcionaron": "decepcionar",
    "decepcionará": "decepcionar",
    "decepcionarán": "decepcionar",
    "decepcionarás": "decepcionar",
    "decepcionaré": "decepcionar",
    "decepcionaréis": "decepcionar",
    "decepcionaría": "decepcionar",
    "decepcionaríais": "decepcionar",
    "decepcionaríamos": "decepcionar",
    "decepcionarían": "decepcionar",
    "decepcionarías": "decepcionar",
    "decepcionas": "decepcionar",
    "decepcionase": "decepcionar",
    "decepcionaseis": "decepcionar",
    "decepcionasen": "decepcionar",
    "decepcionases": "decepcionar",
    "decepcionaste": "decepcionar",
    "decepcionasteis": "decepcionar",
    "decepcione": "decepcionar",
    "decepcionemos": "decepcionar",
    "decepcionen": "decepcionar",
    "decepciones": "decepcionar",
    "decepciono": "decepcionar",
    "decepcionábamos": "decepcionar",
    "decepcionáis": "decepcionar",
    "decepcionáramos": "decepcionar",
    "decepcionáremos": "decepcionar",
    "decepcionásemos": "decepcionar",
    "decepcioné": "decepcionar",
    "decepcionéis": "decepcionar",
    "decepcionó": "decepcionar",
    "decesos": "deceso",
    "dechados": "dechado",
    "decibeles": "decibel",
    "decibelios": "decibelio",
    "decibeliómetros": "decibeliómetro",
    "decid": "decir",
    "decida": "decidir",
    "decidamos": "decidir",
    "decidan": "decidir",
    "decidas": "decidir",
    "decide": "decidir",
    "deciden": "decidir",
    "decides": "decidir",
    "decidibilidades": "decidibilidad",
    "decidid": "decidir",
    "decidida": "decidir",
    "decididas": "decidir",
    "decidido": "decidir",
    "decididos": "decidir",
    "decidiendo": "decidir",
    "decidiera": "decidir",
    "decidierais": "decidir",
    "decidieran": "decidir",
    "decidieras": "decidir",
    "decidiere": "decidir",
    "decidiereis": "decidir",
    "decidieren": "decidir",
    "decidieres": "decidir",
    "decidieron": "decidir",
    "decidiese": "decidir",
    "decidieseis": "decidir",
    "decidiesen": "decidir",
    "decidieses": "decidir",
    "decidimos": "decidir",
    "decidiremos": "decidir",
    "decidirá": "decidir",
    "decidirán": "decidir",
    "decidirás": "decidir",
    "decidiré": "decidir",
    "decidiréis": "decidir",
    "decidiría": "decidir",
    "decidiríais": "decidir",
    "decidiríamos": "decidir",
    "decidirían": "decidir",
    "decidirías": "decidir",
    "decidiste": "decidir",
    "decidisteis": "decidir",
    "decidiéramos": "decidir",
    "decidiéremos": "decidir",
    "decidiésemos": "decidir",
    "decidió": "decidir",
    "decido": "decidir",
    "decidora": "decidor",
    "decidoras": "decidor",
    "decidores": "decidor",
    "decidua": "deciduo",
    "deciduales": "decidual",
    "deciduas": "deciduo",
    "deciduos": "deciduo",
    "decidáis": "decidir",
    "decidí": "decidir",
    "decidía": "decidir",
    "decidíais": "decidir",
    "decidíamos": "decidir",
    "decidían": "decidir",
    "decidías": "decidir",
    "decidís": "decidir",
    "decienta": "decentar",
    "decientan": "decentar",
    "decientas": "decentar",
    "deciente": "decentar",
    "decienten": "decentar",
    "decientes": "decentar",
    "deciento": "decentar",
    "decigramos": "decigramo",
    "decilitros": "decilitro",
    "decimales": "decimal",
    "decimalizaciones": "decimalización",
    "decimanovenas": "decimanovena",
    "decimoctava": "18",
    "decimoctavas": "18",
    "decimoctavo": "18",
    "decimoctavos": "18",
    "decimocuarta": "14",
    "decimocuartas": "14",
    "decimocuarto": "14",
    "decimocuartos": "14",
    "decimonona": "19",
    "decimononas": "19",
    "decimonono": "19",
    "decimononos": "19",
    "decimonovena": "19",
    "decimonovenas": "19",
    "decimonoveno": "19",
    "decimonovenos": "19",
    "decimonónica": "decimonónico",
    "decimonónicas": "decimonónico",
    "decimonónicos": "decimonónico",
    "decimooctava": "decimooctavo",
    "decimooctavas": "decimooctavo",
    "decimooctavos": "decimooctavo",
    "decimoquinta": "15",
    "decimoquintas": "15",
    "decimoquinto": "15",
    "decimoquintos": "15",
    "decimos": "decir",
    "decimosexta": "16",
    "decimosextas": "16",
    "decimosexto": "16",
    "decimosextos": "16",
    "decimoséptima": "17",
    "decimoséptimas": "17",
    "decimoséptimo": "17",
    "decimoséptimos": "17",
    "decimotercera": "13",
    "decimoterceras": "13",
    "decimotercero": "13",
    "decimoterceros": "13",
    "decimotercia": "decimotercio",
    "decimotercias": "decimotercio",
    "decimotercios": "decimotercio",
    "decimétrica": "decimétrico",
    "decimétricas": "decimétrico",
    "decimétricos": "decimétrico",
    "decinormales": "decinormal",
    "decires": "decir",
    "decisiones": "decisión",
    "decisiva": "decisivo",
    "decisivas": "decisivo",
    "decisivos": "decisivo",
    "decisoria": "decisorio",
    "decisorias": "decisorio",
    "decisorios": "decisorio",
    "deciáreas": "deciárea",
    "declama": "declamar",
    "declamaba": "declamar",
    "declamabais": "declamar",
    "declamaban": "declamar",
    "declamabas": "declamar",
    "declamaciones": "declamación",
    "declamad": "declamar",
    "declamada": "declamar",
    "declamadas": "declamar",
    "declamado": "declamar",
    "declamadora": "declamador",
    "declamadoras": "declamador",
    "declamadores": "declamador",
    "declamados": "declamar",
    "declamamos": "declamar",
    "declaman": "declamar",
    "declamando": "declamar",
    "declamara": "declamar",
    "declamarais": "declamar",
    "declamaran": "declamar",
    "declamaras": "declamar",
    "declamare": "declamar",
    "declamareis": "declamar",
    "declamaremos": "declamar",
    "declamaren": "declamar",
    "declamares": "declamar",
    "declamaron": "declamar",
    "declamará": "declamar",
    "declamarán": "declamar",
    "declamarás": "declamar",
    "declamaré": "declamar",
    "declamaréis": "declamar",
    "declamaría": "declamar",
    "declamaríais": "declamar",
    "declamaríamos": "declamar",
    "declamarían": "declamar",
    "declamarías": "declamar",
    "declamas": "declamar",
    "declamase": "declamar",
    "declamaseis": "declamar",
    "declamasen": "declamar",
    "declamases": "declamar",
    "declamaste": "declamar",
    "declamasteis": "declamar",
    "declamatoria": "declamatorio",
    "declamatorias": "declamatorio",
    "declamatorios": "declamatorio",
    "declame": "declamar",
    "declamemos": "declamar",
    "declamen": "declamar",
    "declames": "declamar",
    "declamo": "declamar",
    "declamábamos": "declamar",
    "declamáis": "declamar",
    "declamáramos": "declamar",
    "declamáremos": "declamar",
    "declamásemos": "declamar",
    "declamé": "declamar",
    "declaméis": "declamar",
    "declamó": "declamar",
    "declara": "declarar",
    "declaraba": "declarar",
    "declarabais": "declarar",
    "declaraban": "declarar",
    "declarabas": "declarar",
    "declaraciones": "declaración",
    "declarad": "declarar",
    "declarada": "declarar",
    "declaradas": "declarar",
    "declarado": "declarar",
    "declaradora": "declarador",
    "declaradoras": "declarador",
    "declaradores": "declarador",
    "declarados": "declarar",
    "declaramos": "declarar",
    "declaran": "declarar",
    "declarando": "declarar",
    "declarantes": "declarante",
    "declarara": "declarar",
    "declararais": "declarar",
    "declararan": "declarar",
    "declararas": "declarar",
    "declarare": "declarar",
    "declarareis": "declarar",
    "declararemos": "declarar",
    "declararen": "declarar",
    "declarares": "declarar",
    "declararon": "declarar",
    "declarará": "declarar",
    "declararán": "declarar",
    "declararás": "declarar",
    "declararé": "declarar",
    "declararéis": "declarar",
    "declararía": "declarar",
    "declararíais": "declarar",
    "declararíamos": "declarar",
    "declararían": "declarar",
    "declararías": "declarar",
    "declaras": "declarar",
    "declarase": "declarar",
    "declaraseis": "declarar",
    "declarasen": "declarar",
    "declarases": "declarar",
    "declaraste": "declarar",
    "declarasteis": "declarar",
    "declarativa": "declarativo",
    "declarativas": "declarativo",
    "declarativos": "declarativo",
    "declaratoria": "declaratorio",
    "declaratorias": "declaratorio",
    "declaratorios": "declaratorio",
    "declare": "declarar",
    "declaremos": "declarar",
    "declaren": "declarar",
    "declares": "declarar",
    "declaro": "declarar",
    "declarábamos": "declarar",
    "declaráis": "declarar",
    "declaráramos": "declarar",
    "declaráremos": "declarar",
    "declarásemos": "declarar",
    "declaré": "declarar",
    "declaréis": "declarar",
    "declaró": "declarar",
    "declina": "declinar",
    "declinaba": "declinar",
    "declinabais": "declinar",
    "declinaban": "declinar",
    "declinabas": "declinar",
    "declinables": "declinable",
    "declinaciones": "declinación",
    "declinad": "declinar",
    "declinada": "declinar",
    "declinadas": "declinar",
    "declinado": "declinar",
    "declinados": "declinar",
    "declinamientos": "declinamiento",
    "declinamos": "declinar",
    "declinan": "declinar",
    "declinando": "declinar",
    "declinara": "declinar",
    "declinarais": "declinar",
    "declinaran": "declinar",
    "declinaras": "declinar",
    "declinare": "declinar",
    "declinareis": "declinar",
    "declinaremos": "declinar",
    "declinaren": "declinar",
    "declinares": "declinar",
    "declinaron": "declinar",
    "declinará": "declinar",
    "declinarán": "declinar",
    "declinarás": "declinar",
    "declinaré": "declinar",
    "declinaréis": "declinar",
    "declinaría": "declinar",
    "declinaríais": "declinar",
    "declinaríamos": "declinar",
    "declinarían": "declinar",
    "declinarías": "declinar",
    "declinas": "declinar",
    "declinase": "declinar",
    "declinaseis": "declinar",
    "declinasen": "declinar",
    "declinases": "declinar",
    "declinaste": "declinar",
    "declinasteis": "declinar",
    "declinatoria": "declinatorio",
    "declinatorias": "declinatorio",
    "declinatorios": "declinatorio",
    "decline": "declinar",
    "declinemos": "declinar",
    "declinen": "declinar",
    "declines": "declinar",
    "declino": "declinar",
    "declinábamos": "declinar",
    "declináis": "declinar",
    "declináramos": "declinar",
    "declináremos": "declinar",
    "declinásemos": "declinar",
    "decliné": "declinar",
    "declinéis": "declinar",
    "declinó": "declinar",
    "declinómetros": "declinómetro",
    "declives": "declive",
    "declividades": "declividad",
    "declivios": "declivio",
    "decocciones": "decocción",
    "decodifica": "decodificar",
    "decodificaba": "decodificar",
    "decodificabais": "decodificar",
    "decodificaban": "decodificar",
    "decodificabas": "decodificar",
    "decodificaciones": "decodificación",
    "decodificad": "decodificar",
    "decodificada": "decodificar",
    "decodificadas": "decodificar",
    "decodificado": "decodificar",
    "decodificadores": "decodificador",
    "decodificados": "decodificar",
    "decodificamos": "decodificar",
    "decodifican": "decodificar",
    "decodificando": "decodificar",
    "decodificara": "decodificar",
    "decodificarais": "decodificar",
    "decodificaran": "decodificar",
    "decodificaras": "decodificar",
    "decodificare": "decodificar",
    "decodificareis": "decodificar",
    "decodificaremos": "decodificar",
    "decodificaren": "decodificar",
    "decodificares": "decodificar",
    "decodificaron": "decodificar",
    "decodificará": "decodificar",
    "decodificarán": "decodificar",
    "decodificarás": "decodificar",
    "decodificaré": "decodificar",
    "decodificaréis": "decodificar",
    "decodificaría": "decodificar",
    "decodificaríais": "decodificar",
    "decodificaríamos": "decodificar",
    "decodificarían": "decodificar",
    "decodificarías": "decodificar",
    "decodificas": "decodificar",
    "decodificase": "decodificar",
    "decodificaseis": "decodificar",
    "decodificasen": "decodificar",
    "decodificases": "decodificar",
    "decodificaste": "decodificar",
    "decodificasteis": "decodificar",
    "decodifico": "decodificar",
    "decodificábamos": "decodificar",
    "decodificáis": "decodificar",
    "decodificáramos": "decodificar",
    "decodificáremos": "decodificar",
    "decodificásemos": "decodificar",
    "decodificó": "decodificar",
    "decodifique": "decodificar",
    "decodifiquemos": "decodificar",
    "decodifiquen": "decodificar",
    "decodifiques": "decodificar",
    "decodifiqué": "decodificar",
    "decodifiquéis": "decodificar",
    "decolora": "decolorar",
    "decoloraba": "decolorar",
    "decolorabais": "decolorar",
    "decoloraban": "decolorar",
    "decolorabas": "decolorar",
    "decoloraciones": "decoloración",
    "decolorad": "decolorar",
    "decolorada": "decolorar",
    "decoloradas": "decolorar",
    "decolorado": "decolorar",
    "decolorados": "decolorar",
    "decoloramos": "decolorar",
    "decoloran": "decolorar",
    "decolorando": "decolorar",
    "decolorara": "decolorar",
    "decolorarais": "decolorar",
    "decoloraran": "decolorar",
    "decoloraras": "decolorar",
    "decolorare": "decolorar",
    "decolorareis": "decolorar",
    "decoloraremos": "decolorar",
    "decoloraren": "decolorar",
    "decolorares": "decolorar",
    "decoloraron": "decolorar",
    "decolorará": "decolorar",
    "decolorarán": "decolorar",
    "decolorarás": "decolorar",
    "decoloraré": "decolorar",
    "decoloraréis": "decolorar",
    "decoloraría": "decolorar",
    "decoloraríais": "decolorar",
    "decoloraríamos": "decolorar",
    "decolorarían": "decolorar",
    "decolorarías": "decolorar",
    "decoloras": "decolorar",
    "decolorase": "decolorar",
    "decoloraseis": "decolorar",
    "decolorasen": "decolorar",
    "decolorases": "decolorar",
    "decoloraste": "decolorar",
    "decolorasteis": "decolorar",
    "decolore": "decolorar",
    "decoloremos": "decolorar",
    "decoloren": "decolorar",
    "decolores": "decolorar",
    "decoloro": "decolorar",
    "decolorábamos": "decolorar",
    "decoloráis": "decolorar",
    "decoloráramos": "decolorar",
    "decoloráremos": "decolorar",
    "decolorásemos": "decolorar",
    "decoloré": "decolorar",
    "decoloréis": "decolorar",
    "decoloró": "decolorar",
    "decomisa": "decomisar",
    "decomisaba": "decomisar",
    "decomisabais": "decomisar",
    "decomisaban": "decomisar",
    "decomisabas": "decomisar",
    "decomisad": "decomisar",
    "decomisada": "decomisar",
    "decomisadas": "decomisar",
    "decomisado": "decomisar",
    "decomisados": "decomisar",
    "decomisamos": "decomisar",
    "decomisan": "decomisar",
    "decomisando": "decomisar",
    "decomisara": "decomisar",
    "decomisarais": "decomisar",
    "decomisaran": "decomisar",
    "decomisaras": "decomisar",
    "decomisare": "decomisar",
    "decomisareis": "decomisar",
    "decomisaremos": "decomisar",
    "decomisaren": "decomisar",
    "decomisares": "decomisar",
    "decomisaron": "decomisar",
    "decomisará": "decomisar",
    "decomisarán": "decomisar",
    "decomisarás": "decomisar",
    "decomisaré": "decomisar",
    "decomisaréis": "decomisar",
    "decomisaría": "decomisar",
    "decomisaríais": "decomisar",
    "decomisaríamos": "decomisar",
    "decomisarían": "decomisar",
    "decomisarías": "decomisar",
    "decomisas": "decomisar",
    "decomisase": "decomisar",
    "decomisaseis": "decomisar",
    "decomisasen": "decomisar",
    "decomisases": "decomisar",
    "decomisaste": "decomisar",
    "decomisasteis": "decomisar",
    "decomise": "decomisar",
    "decomisemos": "decomisar",
    "decomisen": "decomisar",
    "decomises": "decomisar",
    "decomiso": "decomisar",
    "decomisos": "decomiso",
    "decomisábamos": "decomisar",
    "decomisáis": "decomisar",
    "decomisáramos": "decomisar",
    "decomisáremos": "decomisar",
    "decomisásemos": "decomisar",
    "decomisé": "decomisar",
    "decomiséis": "decomisar",
    "decomisó": "decomisar",
    "decora": "decorar",
    "decoraba": "decorar",
    "decorabais": "decorar",
    "decoraban": "decorar",
    "decorabas": "decorar",
    "decoraciones": "decoración",
    "decorad": "decorar",
    "decorada": "decorar",
    "decoradas": "decorar",
    "decorado": "decorar",
    "decoradora": "decorador",
    "decoradoras": "decorador",
    "decoradores": "decorador",
    "decorados": "decorar",
    "decoramos": "decorar",
    "decoran": "decorar",
    "decorando": "decorar",
    "decorara": "decorar",
    "decorarais": "decorar",
    "decoraran": "decorar",
    "decoraras": "decorar",
    "decorare": "decorar",
    "decorareis": "decorar",
    "decoraremos": "decorar",
    "decoraren": "decorar",
    "decorares": "decorar",
    "decoraron": "decorar",
    "decorará": "decorar",
    "decorarán": "decorar",
    "decorarás": "decorar",
    "decoraré": "decorar",
    "decoraréis": "decorar",
    "decoraría": "decorar",
    "decoraríais": "decorar",
    "decoraríamos": "decorar",
    "decorarían": "decorar",
    "decorarías": "decorar",
    "decoras": "decorar",
    "decorase": "decorar",
    "decoraseis": "decorar",
    "decorasen": "decorar",
    "decorases": "decorar",
    "decoraste": "decorar",
    "decorasteis": "decorar",
    "decorativa": "decorativo",
    "decorativas": "decorativo",
    "decorativos": "decorativo",
    "decore": "decorar",
    "decoremos": "decorar",
    "decoren": "decorar",
    "decores": "decorar",
    "decoro": "decorar",
    "decoros": "decoro",
    "decorosa": "decoroso",
    "decorosas": "decoroso",
    "decorosos": "decoroso",
    "decorticaciones": "decorticación",
    "decorábamos": "decorar",
    "decoráis": "decorar",
    "decoráramos": "decorar",
    "decoráremos": "decorar",
    "decorásemos": "decorar",
    "decoré": "decorar",
    "decoréis": "decorar",
    "decoró": "decorar",
    "decrece": "decrecer",
    "decreced": "decrecer",
    "decrecemos": "decrecer",
    "decrecen": "decrecer",
    "decreceremos": "decrecer",
    "decrecerá": "decrecer",
    "decrecerán": "decrecer",
    "decrecerás": "decrecer",
    "decreceré": "decrecer",
    "decreceréis": "decrecer",
    "decrecería": "decrecer",
    "decreceríais": "decrecer",
    "decreceríamos": "decrecer",
    "decrecerían": "decrecer",
    "decrecerías": "decrecer",
    "decreces": "decrecer",
    "decrecida": "decrecer",
    "decrecidas": "decrecer",
    "decrecido": "decrecer",
    "decrecidos": "decrecer",
    "decreciendo": "decrecer",
    "decrecientes": "decreciente",
    "decreciera": "decrecer",
    "decrecierais": "decrecer",
    "decrecieran": "decrecer",
    "decrecieras": "decrecer",
    "decreciere": "decrecer",
    "decreciereis": "decrecer",
    "decrecieren": "decrecer",
    "decrecieres": "decrecer",
    "decrecieron": "decrecer",
    "decreciese": "decrecer",
    "decrecieseis": "decrecer",
    "decreciesen": "decrecer",
    "decrecieses": "decrecer",
    "decrecimientos": "decrecimiento",
    "decrecimos": "decrecer",
    "decreciste": "decrecer",
    "decrecisteis": "decrecer",
    "decreciéramos": "decrecer",
    "decreciéremos": "decrecer",
    "decreciésemos": "decrecer",
    "decreció": "decrecer",
    "decrecéis": "decrecer",
    "decrecí": "decrecer",
    "decrecía": "decrecer",
    "decrecíais": "decrecer",
    "decrecíamos": "decrecer",
    "decrecían": "decrecer",
    "decrecías": "decrecer",
    "decrementa": "decrementar",
    "decrementaba": "decrementar",
    "decrementabais": "decrementar",
    "decrementaban": "decrementar",
    "decrementabas": "decrementar",
    "decrementad": "decrementar",
    "decrementada": "decrementar",
    "decrementadas": "decrementar",
    "decrementado": "decrementar",
    "decrementados": "decrementar",
    "decrementamos": "decrementar",
    "decrementan": "decrementar",
    "decrementando": "decrementar",
    "decrementara": "decrementar",
    "decrementarais": "decrementar",
    "decrementaran": "decrementar",
    "decrementaras": "decrementar",
    "decrementare": "decrementar",
    "decrementareis": "decrementar",
    "decrementaremos": "decrementar",
    "decrementaren": "decrementar",
    "decrementares": "decrementar",
    "decrementaron": "decrementar",
    "decrementará": "decrementar",
    "decrementarán": "decrementar",
    "decrementarás": "decrementar",
    "decrementaré": "decrementar",
    "decrementaréis": "decrementar",
    "decrementaría": "decrementar",
    "decrementaríais": "decrementar",
    "decrementaríamos": "decrementar",
    "decrementarían": "decrementar",
    "decrementarías": "decrementar",
    "decrementas": "decrementar",
    "decrementase": "decrementar",
    "decrementaseis": "decrementar",
    "decrementasen": "decrementar",
    "decrementases": "decrementar",
    "decrementaste": "decrementar",
    "decrementasteis": "decrementar",
    "decremente": "decrementar",
    "decrementemos": "decrementar",
    "decrementen": "decrementar",
    "decrementes": "decrementar",
    "decremento": "decrementar",
    "decrementos": "decremento",
    "decrementábamos": "decrementar",
    "decrementáis": "decrementar",
    "decrementáramos": "decrementar",
    "decrementáremos": "decrementar",
    "decrementásemos": "decrementar",
    "decrementé": "decrementar",
    "decrementéis": "decrementar",
    "decrementó": "decrementar",
    "decrepita": "decrepitar",
    "decrepitaba": "decrepitar",
    "decrepitabais": "decrepitar",
    "decrepitaban": "decrepitar",
    "decrepitabas": "decrepitar",
    "decrepitad": "decrepitar",
    "decrepitada": "decrepitar",
    "decrepitadas": "decrepitar",
    "decrepitado": "decrepitar",
    "decrepitados": "decrepitar",
    "decrepitamos": "decrepitar",
    "decrepitan": "decrepitar",
    "decrepitando": "decrepitar",
    "decrepitara": "decrepitar",
    "decrepitarais": "decrepitar",
    "decrepitaran": "decrepitar",
    "decrepitaras": "decrepitar",
    "decrepitare": "decrepitar",
    "decrepitareis": "decrepitar",
    "decrepitaremos": "decrepitar",
    "decrepitaren": "decrepitar",
    "decrepitares": "decrepitar",
    "decrepitaron": "decrepitar",
    "decrepitará": "decrepitar",
    "decrepitarán": "decrepitar",
    "decrepitarás": "decrepitar",
    "decrepitaré": "decrepitar",
    "decrepitaréis": "decrepitar",
    "decrepitaría": "decrepitar",
    "decrepitaríais": "decrepitar",
    "decrepitaríamos": "decrepitar",
    "decrepitarían": "decrepitar",
    "decrepitarías": "decrepitar",
    "decrepitas": "decrepitar",
    "decrepitase": "decrepitar",
    "decrepitaseis": "decrepitar",
    "decrepitasen": "decrepitar",
    "decrepitases": "decrepitar",
    "decrepitaste": "decrepitar",
    "decrepitasteis": "decrepitar",
    "decrepite": "decrepitar",
    "decrepitemos": "decrepitar",
    "decrepiten": "decrepitar",
    "decrepites": "decrepitar",
    "decrepito": "decrepitar",
    "decrepitudes": "decrepitud",
    "decrepitábamos": "decrepitar",
    "decrepitáis": "decrepitar",
    "decrepitáramos": "decrepitar",
    "decrepitáremos": "decrepitar",
    "decrepitásemos": "decrepitar",
    "decrepité": "decrepitar",
    "decrepitéis": "decrepitar",
    "decrepitó": "decrepitar",
    "decreta": "decretar",
    "decretaba": "decretar",
    "decretabais": "decretar",
    "decretaban": "decretar",
    "decretabas": "decretar",
    "decretad": "decretar",
    "decretada": "decretar",
    "decretadas": "decretar",
    "decretado": "decretar",
    "decretados": "decretar",
    "decretales": "decretal",
    "decretamos": "decretar",
    "decretan": "decretar",
    "decretando": "decretar",
    "decretara": "decretar",
    "decretarais": "decretar",
    "decretaran": "decretar",
    "decretaras": "decretar",
    "decretare": "decretar",
    "decretareis": "decretar",
    "decretaremos": "decretar",
    "decretaren": "decretar",
    "decretares": "decretar",
    "decretaron": "decretar",
    "decretará": "decretar",
    "decretarán": "decretar",
    "decretarás": "decretar",
    "decretaré": "decretar",
    "decretaréis": "decretar",
    "decretaría": "decretar",
    "decretaríais": "decretar",
    "decretaríamos": "decretar",
    "decretarían": "decretar",
    "decretarías": "decretar",
    "decretas": "decretar",
    "decretase": "decretar",
    "decretaseis": "decretar",
    "decretasen": "decretar",
    "decretases": "decretar",
    "decretaste": "decretar",
    "decretasteis": "decretar",
    "decrete": "decretar",
    "decretemos": "decretar",
    "decreten": "decretar",
    "decretes": "decretar",
    "decreto": "decretar",
    "decretos": "decreto",
    "decretos-ley": "decreto-ley",
    "decretábamos": "decretar",
    "decretáis": "decretar",
    "decretáramos": "decretar",
    "decretáremos": "decretar",
    "decretásemos": "decretar",
    "decreté": "decretar",
    "decretéis": "decretar",
    "decretó": "decretar",
    "decrezca": "decrecer",
    "decrezcamos": "decrecer",
    "decrezcan": "decrecer",
    "decrezcas": "decrecer",
    "decrezco": "decrecer",
    "decrezcáis": "decrecer",
    "decrépita": "decrépito",
    "decrépitas": "decrépito",
    "decrépitos": "decrépito",
    "decumbentes": "decumbente",
    "decuplica": "decuplicar",
    "decuplicaba": "decuplicar",
    "decuplicabais": "decuplicar",
    "decuplicaban": "decuplicar",
    "decuplicabas": "decuplicar",
    "decuplicad": "decuplicar",
    "decuplicada": "decuplicar",
    "decuplicadas": "decuplicar",
    "decuplicado": "decuplicar",
    "decuplicados": "decuplicar",
    "decuplicamos": "decuplicar",
    "decuplican": "decuplicar",
    "decuplicando": "decuplicar",
    "decuplicara": "decuplicar",
    "decuplicarais": "decuplicar",
    "decuplicaran": "decuplicar",
    "decuplicaras": "decuplicar",
    "decuplicare": "decuplicar",
    "decuplicareis": "decuplicar",
    "decuplicaremos": "decuplicar",
    "decuplicaren": "decuplicar",
    "decuplicares": "decuplicar",
    "decuplicaron": "decuplicar",
    "decuplicará": "decuplicar",
    "decuplicarán": "decuplicar",
    "decuplicarás": "decuplicar",
    "decuplicaré": "decuplicar",
    "decuplicaréis": "decuplicar",
    "decuplicaría": "decuplicar",
    "decuplicaríais": "decuplicar",
    "decuplicaríamos": "decuplicar",
    "decuplicarían": "decuplicar",
    "decuplicarías": "decuplicar",
    "decuplicas": "decuplicar",
    "decuplicase": "decuplicar",
    "decuplicaseis": "decuplicar",
    "decuplicasen": "decuplicar",
    "decuplicases": "decuplicar",
    "decuplicaste": "decuplicar",
    "decuplicasteis": "decuplicar",
    "decuplico": "decuplicar",
    "decuplicábamos": "decuplicar",
    "decuplicáis": "decuplicar",
    "decuplicáramos": "decuplicar",
    "decuplicáremos": "decuplicar",
    "decuplicásemos": "decuplicar",
    "decuplicó": "decuplicar",
    "decuplique": "decuplicar",
    "decupliquemos": "decuplicar",
    "decupliquen": "decuplicar",
    "decupliques": "decuplicar",
    "decupliqué": "decuplicar",
    "decupliquéis": "decuplicar",
    "decurias": "decuria",
    "decurionatos": "decurionato",
    "decuriones": "decurión",
    "decurrentes": "decurrente",
    "decursa": "decursas",
    "decursos": "decurso",
    "decurtaciones": "decurtación",
    "decusa": "decuso",
    "decusaciones": "decusación",
    "decusada": "decusado",
    "decusadas": "decusado",
    "decusados": "decusado",
    "decusas": "decuso",
    "decusos": "decuso",
    "decuspidada": "decuspidado",
    "decuspidadas": "decuspidado",
    "decuspidados": "decuspidado",
    "decágona": "decágono",
    "decágonas": "decágono",
    "decágonos": "decágono",
    "decálogos": "decálogo",
    "decámetros": "decámetro",
    "decápodos": "decápodo",
    "decáreas": "decárea",
    "decía": "decir",
    "decíais": "decir",
    "decíamos": "decir",
    "decían": "decir",
    "decías": "decir",
    "decímetros": "decímetro",
    "decís": "decir",
    "decúbitos": "decúbito",
    "dedadas": "dedada",
    "dedaleras": "dedalera",
    "dedales": "dedal",
    "dedeos": "dedeo",
    "dedica": "dedicar",
    "dedicaba": "dedicar",
    "dedicabais": "dedicar",
    "dedicaban": "dedicar",
    "dedicabas": "dedicar",
    "dedicaciones": "dedicación",
    "dedicad": "dedicar",
    "dedicada": "dedicar",
    "dedicadas": "dedicar",
    "dedicado": "dedicar",
    "dedicados": "dedicar",
    "dedicamos": "dedicar",
    "dedican": "dedicar",
    "dedicando": "dedicar",
    "dedicara": "dedicar",
    "dedicarais": "dedicar",
    "dedicaran": "dedicar",
    "dedicaras": "dedicar",
    "dedicare": "dedicar",
    "dedicareis": "dedicar",
    "dedicaremos": "dedicar",
    "dedicaren": "dedicar",
    "dedicares": "dedicar",
    "dedicaron": "dedicar",
    "dedicará": "dedicar",
    "dedicarán": "dedicar",
    "dedicarás": "dedicar",
    "dedicaré": "dedicar",
    "dedicaréis": "dedicar",
    "dedicaría": "dedicar",
    "dedicaríais": "dedicar",
    "dedicaríamos": "dedicar",
    "dedicarían": "dedicar",
    "dedicarías": "dedicar",
    "dedicas": "dedicar",
    "dedicase": "dedicar",
    "dedicaseis": "dedicar",
    "dedicasen": "dedicar",
    "dedicases": "dedicar",
    "dedicaste": "dedicar",
    "dedicasteis": "dedicar",
    "dedicatorias": "dedicatoria",
    "dediciones": "dedición",
    "dedico": "dedicar",
    "dedicábamos": "dedicar",
    "dedicáis": "dedicar",
    "dedicáramos": "dedicar",
    "dedicáremos": "dedicar",
    "dedicásemos": "dedicar",
    "dedicó": "dedicar",
    "dediles": "dedil",
    "dedique": "dedicar",
    "dediquemos": "dedicar",
    "dediquen": "dedicar",
    "dediques": "dedicar",
    "dediqué": "dedicar",
    "dediquéis": "dedicar",
    "dedos": "dedo",
    "deducciones": "deducción",
    "deduce": "deducir",
    "deducen": "deducir",
    "deduces": "deducir",
    "deducibles": "deducible",
    "deducid": "deducir",
    "deducida": "deducir",
    "deducidas": "deducir",
    "deducido": "deducir",
    "deducidos": "deducir",
    "deduciendo": "deducir",
    "deducimos": "deducir",
    "deduciremos": "deducir",
    "deducirá": "deducir",
    "deducirán": "deducir",
    "deducirás": "deducir",
    "deduciré": "deducir",
    "deduciréis": "deducir",
    "deduciría": "deducir",
    "deduciríais": "deducir",
    "deduciríamos": "deducir",
    "deducirían": "deducir",
    "deducirías": "deducir",
    "deductiva": "deductivo",
    "deductivas": "deductivo",
    "deductivos": "deductivo",
    "deducía": "deducir",
    "deducíais": "deducir",
    "deducíamos": "deducir",
    "deducían": "deducir",
    "deducías": "deducir",
    "deducís": "deducir",
    "deduje": "deducir",
    "dedujera": "deducir",
    "dedujerais": "deducir",
    "dedujeran": "deducir",
    "dedujeras": "deducir",
    "dedujere": "deducir",
    "dedujereis": "deducir",
    "dedujeren": "deducir",
    "dedujeres": "deducir",
    "dedujeron": "deducir",
    "dedujese": "deducir",
    "dedujeseis": "deducir",
    "dedujesen": "deducir",
    "dedujeses": "deducir",
    "dedujimos": "deducir",
    "dedujiste": "deducir",
    "dedujisteis": "deducir",
    "dedujo": "deducir",
    "dedujéramos": "deducir",
    "dedujéremos": "deducir",
    "dedujésemos": "deducir",
    "deduzca": "deducir",
    "deduzcamos": "deducir",
    "deduzcan": "deducir",
    "deduzcas": "deducir",
    "deduzco": "deducir",
    "deduzcáis": "deducir",
    "defalca": "defalcar",
    "defalcaba": "defalcar",
    "defalcabais": "defalcar",
    "defalcaban": "defalcar",
    "defalcabas": "defalcar",
    "defalcad": "defalcar",
    "defalcada": "defalcar",
    "defalcadas": "defalcar",
    "defalcado": "defalcar",
    "defalcados": "defalcar",
    "defalcamos": "defalcar",
    "defalcan": "defalcar",
    "defalcando": "defalcar",
    "defalcara": "defalcar",
    "defalcarais": "defalcar",
    "defalcaran": "defalcar",
    "defalcaras": "defalcar",
    "defalcare": "defalcar",
    "defalcareis": "defalcar",
    "defalcaremos": "defalcar",
    "defalcaren": "defalcar",
    "defalcares": "defalcar",
    "defalcaron": "defalcar",
    "defalcará": "defalcar",
    "defalcarán": "defalcar",
    "defalcarás": "defalcar",
    "defalcaré": "defalcar",
    "defalcaréis": "defalcar",
    "defalcaría": "defalcar",
    "defalcaríais": "defalcar",
    "defalcaríamos": "defalcar",
    "defalcarían": "defalcar",
    "defalcarías": "defalcar",
    "defalcas": "defalcar",
    "defalcase": "defalcar",
    "defalcaseis": "defalcar",
    "defalcasen": "defalcar",
    "defalcases": "defalcar",
    "defalcaste": "defalcar",
    "defalcasteis": "defalcar",
    "defalco": "defalcar",
    "defalcábamos": "defalcar",
    "defalcáis": "defalcar",
    "defalcáramos": "defalcar",
    "defalcáremos": "defalcar",
    "defalcásemos": "defalcar",
    "defalcó": "defalcar",
    "defalque": "defalcar",
    "defalquemos": "defalcar",
    "defalquen": "defalcar",
    "defalques": "defalcar",
    "defalqué": "defalcar",
    "defalquéis": "defalcar",
    "defasada": "defasado",
    "defasadas": "defasado",
    "defasadora": "defasador",
    "defasadoras": "defasador",
    "defasadores": "defasador",
    "defasados": "defasado",
    "defasajes": "defasaje",
    "defaunaciones": "defaunación",
    "defeca": "defecar",
    "defecaba": "defecar",
    "defecabais": "defecar",
    "defecaban": "defecar",
    "defecabas": "defecar",
    "defecaciones": "defecación",
    "defecad": "defecar",
    "defecada": "defecar",
    "defecadas": "defecar",
    "defecado": "defecar",
    "defecadora": "defecador",
    "defecadoras": "defecador",
    "defecadores": "defecador",
    "defecados": "defecar",
    "defecamos": "defecar",
    "defecan": "defecar",
    "defecando": "defecar",
    "defecara": "defecar",
    "defecarais": "defecar",
    "defecaran": "defecar",
    "defecaras": "defecar",
    "defecare": "defecar",
    "defecareis": "defecar",
    "defecaremos": "defecar",
    "defecaren": "defecar",
    "defecares": "defecar",
    "defecaron": "defecar",
    "defecará": "defecar",
    "defecarán": "defecar",
    "defecarás": "defecar",
    "defecaré": "defecar",
    "defecaréis": "defecar",
    "defecaría": "defecar",
    "defecaríais": "defecar",
    "defecaríamos": "defecar",
    "defecarían": "defecar",
    "defecarías": "defecar",
    "defecas": "defecar",
    "defecase": "defecar",
    "defecaseis": "defecar",
    "defecasen": "defecar",
    "defecases": "defecar",
    "defecaste": "defecar",
    "defecasteis": "defecar",
    "defecciones": "defección",
    "defeco": "defecar",
    "defectibilidades": "defectibilidad",
    "defectibles": "defectible",
    "defectiva": "defectivo",
    "defectivas": "defectivo",
    "defectividades": "defectividad",
    "defectivos": "defectivo",
    "defectos": "defecto",
    "defectuosa": "defectuoso",
    "defectuosas": "defectuoso",
    "defectuosos": "defectuoso",
    "defecábamos": "defecar",
    "defecáis": "defecar",
    "defecáramos": "defecar",
    "defecáremos": "defecar",
    "defecásemos": "defecar",
    "defecó": "defecar",
    "defendamos": "defender",
    "defended": "defender",
    "defendemos": "defender",
    "defenderemos": "defender",
    "defenderá": "defender",
    "defenderán": "defender",
    "defenderás": "defender",
    "defenderé": "defender",
    "defenderéis": "defender",
    "defendería": "defender",
    "defenderíais": "defender",
    "defenderíamos": "defender",
    "defenderían": "defender",
    "defenderías": "defender",
    "defendibles": "defendible",
    "defendida": "defendido",
    "defendidas": "defendido",
    "defendido": "defender",
    "defendidos": "defendido",
    "defendiendo": "defender",
    "defendiera": "defender",
    "defendierais": "defender",
    "defendieran": "defender",
    "defendieras": "defender",
    "defendiere": "defender",
    "defendiereis": "defender",
    "defendieren": "defender",
    "defendieres": "defender",
    "defendieron": "defender",
    "defendiese": "defender",
    "defendieseis": "defender",
    "defendiesen": "defender",
    "defendieses": "defender",
    "defendimos": "defender",
    "defendiste": "defender",
    "defendisteis": "defender",
    "defendiéramos": "defender",
    "defendiéremos": "defender",
    "defendiésemos": "defender",
    "defendió": "defender",
    "defendáis": "defender",
    "defendéis": "defender",
    "defendí": "defender",
    "defendía": "defender",
    "defendíais": "defender",
    "defendíamos": "defender",
    "defendían": "defender",
    "defendías": "defender",
    "defenestra": "defenestrar",
    "defenestraba": "defenestrar",
    "defenestrabais": "defenestrar",
    "defenestraban": "defenestrar",
    "defenestrabas": "defenestrar",
    "defenestraciones": "defenestración",
    "defenestrad": "defenestrar",
    "defenestrada": "defenestrar",
    "defenestradas": "defenestrar",
    "defenestrado": "defenestrar",
    "defenestrados": "defenestrar",
    "defenestramos": "defenestrar",
    "defenestran": "defenestrar",
    "defenestrando": "defenestrar",
    "defenestrara": "defenestrar",
    "defenestrarais": "defenestrar",
    "defenestraran": "defenestrar",
    "defenestraras": "defenestrar",
    "defenestrare": "defenestrar",
    "defenestrareis": "defenestrar",
    "defenestraremos": "defenestrar",
    "defenestraren": "defenestrar",
    "defenestrares": "defenestrar",
    "defenestraron": "defenestrar",
    "defenestrará": "defenestrar",
    "defenestrarán": "defenestrar",
    "defenestrarás": "defenestrar",
    "defenestraré": "defenestrar",
    "defenestraréis": "defenestrar",
    "defenestraría": "defenestrar",
    "defenestraríais": "defenestrar",
    "defenestraríamos": "defenestrar",
    "defenestrarían": "defenestrar",
    "defenestrarías": "defenestrar",
    "defenestras": "defenestrar",
    "defenestrase": "defenestrar",
    "defenestraseis": "defenestrar",
    "defenestrasen": "defenestrar",
    "defenestrases": "defenestrar",
    "defenestraste": "defenestrar",
    "defenestrasteis": "defenestrar",
    "defenestre": "defenestrar",
    "defenestremos": "defenestrar",
    "defenestren": "defenestrar",
    "defenestres": "defenestrar",
    "defenestro": "defenestrar",
    "defenestrábamos": "defenestrar",
    "defenestráis": "defenestrar",
    "defenestráramos": "defenestrar",
    "defenestráremos": "defenestrar",
    "defenestrásemos": "defenestrar",
    "defenestré": "defenestrar",
    "defenestréis": "defenestrar",
    "defenestró": "defenestrar",
    "defensas": "defensa",
    "defensiones": "defensión",
    "defensiva": "defensivo",
    "defensivas": "defensivo",
    "defensivos": "defensivo",
    "defensora": "defensor",
    "defensoras": "defensor",
    "defensores": "defensor",
    "defensorios": "defensorio",
    "defensorías": "defensoría",
    "defeque": "defecar",
    "defequemos": "defecar",
    "defequen": "defecar",
    "defeques": "defecar",
    "defequé": "defecar",
    "defequéis": "defecar",
    "deferenciales": "deferencial",
    "deferencias": "deferencia",
    "deferentes": "deferente",
    "deferid": "deferir",
    "deferida": "deferir",
    "deferidas": "deferir",
    "deferido": "deferir",
    "deferidos": "deferir",
    "deferimos": "deferir",
    "deferiremos": "deferir",
    "deferirá": "deferir",
    "deferirán": "deferir",
    "deferirás": "deferir",
    "deferiré": "deferir",
    "deferiréis": "deferir",
    "deferiría": "deferir",
    "deferiríais": "deferir",
    "deferiríamos": "deferir",
    "deferirían": "deferir",
    "deferirías": "deferir",
    "deferiste": "deferir",
    "deferisteis": "deferir",
    "deferí": "deferir",
    "defería": "deferir",
    "deferíais": "deferir",
    "deferíamos": "deferir",
    "deferían": "deferir",
    "deferías": "deferir",
    "deferís": "deferir",
    "defibrilaciones": "defibrilación",
    "deficiencias": "deficiencia",
    "deficientes": "deficiente",
    "deficitaria": "deficitario",
    "deficitarias": "deficitario",
    "deficitarios": "deficitario",
    "defienda": "defender",
    "defiendan": "defender",
    "defiendas": "defender",
    "defiende": "defender",
    "defienden": "defender",
    "defiendes": "defender",
    "defiendo": "defender",
    "defiera": "deferir",
    "defieran": "deferir",
    "defieras": "deferir",
    "defiere": "deferir",
    "defieren": "deferir",
    "defieres": "deferir",
    "defiero": "deferir",
    "defina": "definir",
    "definamos": "definir",
    "definan": "definir",
    "definas": "definir",
    "define": "definir",
    "definen": "definir",
    "defines": "definir",
    "definibles": "definible",
    "definiciones": "definición",
    "definid": "definir",
    "definida": "definir",
    "definidas": "definir",
    "definido": "definir",
    "definidora": "definidor",
    "definidoras": "definidor",
    "definidores": "definidor",
    "definidos": "definir",
    "definiendo": "definir",
    "definiera": "definir",
    "definierais": "definir",
    "definieran": "definir",
    "definieras": "definir",
    "definiere": "definir",
    "definiereis": "definir",
    "definieren": "definir",
    "definieres": "definir",
    "definieron": "definir",
    "definiese": "definir",
    "definieseis": "definir",
    "definiesen": "definir",
    "definieses": "definir",
    "definimos": "definir",
    "definiremos": "definir",
    "definirá": "definir",
    "definirán": "definir",
    "definirás": "definir",
    "definiré": "definir",
    "definiréis": "definir",
    "definiría": "definir",
    "definiríais": "definir",
    "definiríamos": "definir",
    "definirían": "definir",
    "definirías": "definir",
    "definiste": "definir",
    "definisteis": "definir",
    "definitiva": "definitivo",
    "definitivas": "definitivo",
    "definitivos": "definitivo",
    "definitoria": "definitorio",
    "definitorias": "definitorio",
    "definitorios": "definitorio",
    "definiéramos": "definir",
    "definiéremos": "definir",
    "definiésemos": "definir",
    "definió": "definir",
    "defino": "definir",
    "defináis": "definir",
    "definí": "definir",
    "definía": "definir",
    "definíais": "definir",
    "definíamos": "definir",
    "definían": "definir",
    "definías": "definir",
    "definís": "definir",
    "defiramos": "deferir",
    "defiriendo": "deferir",
    "defiriera": "deferir",
    "defirierais": "deferir",
    "defirieran": "deferir",
    "defirieras": "deferir",
    "defiriere": "deferir",
    "defiriereis": "deferir",
    "defirieren": "deferir",
    "defirieres": "deferir",
    "defirieron": "deferir",
    "defiriese": "deferir",
    "defirieseis": "deferir",
    "defiriesen": "deferir",
    "defirieses": "deferir",
    "defiriéramos": "deferir",
    "defiriéremos": "deferir",
    "defiriésemos": "deferir",
    "defirió": "deferir",
    "defiráis": "deferir",
    "deflacionaria": "deflacionario",
    "deflacionarias": "deflacionario",
    "deflacionarios": "deflacionario",
    "deflaciones": "deflación",
    "deflacionistas": "deflacionista",
    "deflactaciones": "deflactación",
    "deflactores": "deflactor",
    "deflagra": "deflagrar",
    "deflagraba": "deflagrar",
    "deflagrabais": "deflagrar",
    "deflagraban": "deflagrar",
    "deflagrabas": "deflagrar",
    "deflagraciones": "deflagración",
    "deflagrad": "deflagrar",
    "deflagrada": "deflagrar",
    "deflagradas": "deflagrar",
    "deflagrado": "deflagrar",
    "deflagradora": "deflagrador",
    "deflagradoras": "deflagrador",
    "deflagradores": "deflagrador",
    "deflagrados": "deflagrar",
    "deflagramos": "deflagrar",
    "deflagran": "deflagrar",
    "deflagrando": "deflagrar",
    "deflagrara": "deflagrar",
    "deflagrarais": "deflagrar",
    "deflagraran": "deflagrar",
    "deflagraras": "deflagrar",
    "deflagrare": "deflagrar",
    "deflagrareis": "deflagrar",
    "deflagraremos": "deflagrar",
    "deflagraren": "deflagrar",
    "deflagrares": "deflagrar",
    "deflagraron": "deflagrar",
    "deflagrará": "deflagrar",
    "deflagrarán": "deflagrar",
    "deflagrarás": "deflagrar",
    "deflagraré": "deflagrar",
    "deflagraréis": "deflagrar",
    "deflagraría": "deflagrar",
    "deflagraríais": "deflagrar",
    "deflagraríamos": "deflagrar",
    "deflagrarían": "deflagrar",
    "deflagrarías": "deflagrar",
    "deflagras": "deflagrar",
    "deflagrase": "deflagrar",
    "deflagraseis": "deflagrar",
    "deflagrasen": "deflagrar",
    "deflagrases": "deflagrar",
    "deflagraste": "deflagrar",
    "deflagrasteis": "deflagrar",
    "deflagre": "deflagrar",
    "deflagremos": "deflagrar",
    "deflagren": "deflagrar",
    "deflagres": "deflagrar",
    "deflagro": "deflagrar",
    "deflagrábamos": "deflagrar",
    "deflagráis": "deflagrar",
    "deflagráramos": "deflagrar",
    "deflagráremos": "deflagrar",
    "deflagrásemos": "deflagrar",
    "deflagré": "deflagrar",
    "deflagréis": "deflagrar",
    "deflagró": "deflagrar",
    "deflectora": "deflector",
    "deflectoras": "deflector",
    "deflectores": "deflector",
    "deflectómetros": "deflectómetro",
    "deflegma": "deflegmar",
    "deflegmaba": "deflegmar",
    "deflegmabais": "deflegmar",
    "deflegmaban": "deflegmar",
    "deflegmabas": "deflegmar",
    "deflegmad": "deflegmar",
    "deflegmada": "deflegmar",
    "deflegmadas": "deflegmar",
    "deflegmado": "deflegmar",
    "deflegmados": "deflegmar",
    "deflegmamos": "deflegmar",
    "deflegman": "deflegmar",
    "deflegmando": "deflegmar",
    "deflegmara": "deflegmar",
    "deflegmarais": "deflegmar",
    "deflegmaran": "deflegmar",
    "deflegmaras": "deflegmar",
    "deflegmare": "deflegmar",
    "deflegmareis": "deflegmar",
    "deflegmaremos": "deflegmar",
    "deflegmaren": "deflegmar",
    "deflegmares": "deflegmar",
    "deflegmaron": "deflegmar",
    "deflegmará": "deflegmar",
    "deflegmarán": "deflegmar",
    "deflegmarás": "deflegmar",
    "deflegmaré": "deflegmar",
    "deflegmaréis": "deflegmar",
    "deflegmaría": "deflegmar",
    "deflegmaríais": "deflegmar",
    "deflegmaríamos": "deflegmar",
    "deflegmarían": "deflegmar",
    "deflegmarías": "deflegmar",
    "deflegmas": "deflegmar",
    "deflegmase": "deflegmar",
    "deflegmaseis": "deflegmar",
    "deflegmasen": "deflegmar",
    "deflegmases": "deflegmar",
    "deflegmaste": "deflegmar",
    "deflegmasteis": "deflegmar",
    "deflegme": "deflegmar",
    "deflegmemos": "deflegmar",
    "deflegmen": "deflegmar",
    "deflegmes": "deflegmar",
    "deflegmo": "deflegmar",
    "deflegmábamos": "deflegmar",
    "deflegmáis": "deflegmar",
    "deflegmáramos": "deflegmar",
    "deflegmáremos": "deflegmar",
    "deflegmásemos": "deflegmar",
    "deflegmé": "deflegmar",
    "deflegméis": "deflegmar",
    "deflegmó": "deflegmar",
    "deflexa": "deflexo",
    "deflexas": "deflexo",
    "deflexiones": "deflexión",
    "deflexos": "deflexo",
    "defloculaciones": "defloculación",
    "defloculadora": "defloculador",
    "defloculadoras": "defloculador",
    "defloculadores": "defloculador",
    "deflujos": "deflujo",
    "defluoraciones": "defluoración",
    "defluviaciones": "defluviación",
    "defoliaciones": "defoliación",
    "defoliantes": "defoliante",
    "deforesta": "deforestar",
    "deforestaba": "deforestar",
    "deforestabais": "deforestar",
    "deforestaban": "deforestar",
    "deforestabas": "deforestar",
    "deforestaciones": "deforestación",
    "deforestad": "deforestar",
    "deforestada": "deforestar",
    "deforestadas": "deforestar",
    "deforestado": "deforestar",
    "deforestados": "deforestar",
    "deforestamos": "deforestar",
    "deforestan": "deforestar",
    "deforestando": "deforestar",
    "deforestara": "deforestar",
    "deforestarais": "deforestar",
    "deforestaran": "deforestar",
    "deforestaras": "deforestar",
    "deforestare": "deforestar",
    "deforestareis": "deforestar",
    "deforestaremos": "deforestar",
    "deforestaren": "deforestar",
    "deforestares": "deforestar",
    "deforestaron": "deforestar",
    "deforestará": "deforestar",
    "deforestarán": "deforestar",
    "deforestarás": "deforestar",
    "deforestaré": "deforestar",
    "deforestaréis": "deforestar",
    "deforestaría": "deforestar",
    "deforestaríais": "deforestar",
    "deforestaríamos": "deforestar",
    "deforestarían": "deforestar",
    "deforestarías": "deforestar",
    "deforestas": "deforestar",
    "deforestase": "deforestar",
    "deforestaseis": "deforestar",
    "deforestasen": "deforestar",
    "deforestases": "deforestar",
    "deforestaste": "deforestar",
    "deforestasteis": "deforestar",
    "deforeste": "deforestar",
    "deforestemos": "deforestar",
    "deforesten": "deforestar",
    "deforestes": "deforestar",
    "deforesto": "deforestar",
    "deforestábamos": "deforestar",
    "deforestáis": "deforestar",
    "deforestáramos": "deforestar",
    "deforestáremos": "deforestar",
    "deforestásemos": "deforestar",
    "deforesté": "deforestar",
    "deforestéis": "deforestar",
    "deforestó": "deforestar",
    "deforma": "deformar",
    "deformaba": "deformar",
    "deformabais": "deformar",
    "deformaban": "deformar",
    "deformabas": "deformar",
    "deformabilidades": "deformabilidad",
    "deformables": "deformable",
    "deformaciones": "deformación",
    "deformad": "deformar",
    "deformada": "deformar",
    "deformadas": "deformar",
    "deformado": "deformar",
    "deformadora": "deformador",
    "deformadoras": "deformador",
    "deformadores": "deformador",
    "deformados": "deformar",
    "deformamos": "deformar",
    "deforman": "deformar",
    "deformando": "deformar",
    "deformantes": "deformante",
    "deformara": "deformar",
    "deformarais": "deformar",
    "deformaran": "deformar",
    "deformaras": "deformar",
    "deformare": "deformar",
    "deformareis": "deformar",
    "deformaremos": "deformar",
    "deformaren": "deformar",
    "deformares": "deformar",
    "deformaron": "deformar",
    "deformará": "deformar",
    "deformarán": "deformar",
    "deformarás": "deformar",
    "deformaré": "deformar",
    "deformaréis": "deformar",
    "deformaría": "deformar",
    "deformaríais": "deformar",
    "deformaríamos": "deformar",
    "deformarían": "deformar",
    "deformarías": "deformar",
    "deformas": "deformar",
    "deformase": "deformar",
    "deformaseis": "deformar",
    "deformasen": "deformar",
    "deformases": "deformar",
    "deformaste": "deformar",
    "deformasteis": "deformar",
    "deformateos": "deformateo",
    "deformatoria": "deformatorio",
    "deformatorias": "deformatorio",
    "deformatorios": "deformatorio",
    "deforme": "deformar",
    "deformemos": "deformar",
    "deformen": "deformar",
    "deformes": "deforme",
    "deformidades": "deformidad",
    "deformo": "deformar",
    "deformábamos": "deformar",
    "deformáis": "deformar",
    "deformáramos": "deformar",
    "deformáremos": "deformar",
    "deformásemos": "deformar",
    "deformé": "deformar",
    "deforméis": "deformar",
    "deformó": "deformar",
    "defosforaciones": "defosforación",
    "defrauda": "defraudar",
    "defraudaba": "defraudar",
    "defraudabais": "defraudar",
    "defraudaban": "defraudar",
    "defraudabas": "defraudar",
    "defraudaciones": "defraudación",
    "defraudad": "defraudar",
    "defraudada": "defraudar",
    "defraudadas": "defraudar",
    "defraudado": "defraudar",
    "defraudadora": "defraudador",
    "defraudadoras": "defraudador",
    "defraudadores": "defraudador",
    "defraudados": "defraudar",
    "defraudamos": "defraudar",
    "defraudan": "defraudar",
    "defraudando": "defraudar",
    "defraudara": "defraudar",
    "defraudarais": "defraudar",
    "defraudaran": "defraudar",
    "defraudaras": "defraudar",
    "defraudare": "defraudar",
    "defraudareis": "defraudar",
    "defraudaremos": "defraudar",
    "defraudaren": "defraudar",
    "defraudares": "defraudar",
    "defraudaron": "defraudar",
    "defraudará": "defraudar",
    "defraudarán": "defraudar",
    "defraudarás": "defraudar",
    "defraudaré": "defraudar",
    "defraudaréis": "defraudar",
    "defraudaría": "defraudar",
    "defraudaríais": "defraudar",
    "defraudaríamos": "defraudar",
    "defraudarían": "defraudar",
    "defraudarías": "defraudar",
    "defraudas": "defraudar",
    "defraudase": "defraudar",
    "defraudaseis": "defraudar",
    "defraudasen": "defraudar",
    "defraudases": "defraudar",
    "defraudaste": "defraudar",
    "defraudasteis": "defraudar",
    "defraude": "defraudar",
    "defraudemos": "defraudar",
    "defrauden": "defraudar",
    "defraudes": "defraudar",
    "defraudo": "defraudar",
    "defraudábamos": "defraudar",
    "defraudáis": "defraudar",
    "defraudáramos": "defraudar",
    "defraudáremos": "defraudar",
    "defraudásemos": "defraudar",
    "defraudé": "defraudar",
    "defraudéis": "defraudar",
    "defraudó": "defraudar",
    "defunciones": "defunción",
    "degasificaciones": "degasificación",
    "degenera": "degenerar",
    "degeneraba": "degenerar",
    "degenerabais": "degenerar",
    "degeneraban": "degenerar",
    "degenerabas": "degenerar",
    "degeneraciones": "degeneración",
    "degenerad": "degenerar",
    "degenerada": "degenerar",
    "degeneradas": "degenerar",
    "degenerado": "degenerar",
    "degenerados": "degenerar",
    "degeneramos": "degenerar",
    "degeneran": "degenerar",
    "degenerando": "degenerar",
    "degenerantes": "degenerante",
    "degenerara": "degenerar",
    "degenerarais": "degenerar",
    "degeneraran": "degenerar",
    "degeneraras": "degenerar",
    "degenerare": "degenerar",
    "degenerareis": "degenerar",
    "degeneraremos": "degenerar",
    "degeneraren": "degenerar",
    "degenerares": "degenerar",
    "degeneraron": "degenerar",
    "degenerará": "degenerar",
    "degenerarán": "degenerar",
    "degenerarás": "degenerar",
    "degeneraré": "degenerar",
    "degeneraréis": "degenerar",
    "degeneraría": "degenerar",
    "degeneraríais": "degenerar",
    "degeneraríamos": "degenerar",
    "degenerarían": "degenerar",
    "degenerarías": "degenerar",
    "degeneras": "degenerar",
    "degenerase": "degenerar",
    "degeneraseis": "degenerar",
    "degenerasen": "degenerar",
    "degenerases": "degenerar",
    "degeneraste": "degenerar",
    "degenerasteis": "degenerar",
    "degenerativa": "degenerativo",
    "degenerativas": "degenerativo",
    "degenerativos": "degenerativo",
    "degenere": "degenerar",
    "degeneremos": "degenerar",
    "degeneren": "degenerar",
    "degeneres": "degenerar",
    "degenero": "degenerar",
    "degenerábamos": "degenerar",
    "degeneráis": "degenerar",
    "degeneráramos": "degenerar",
    "degeneráremos": "degenerar",
    "degenerásemos": "degenerar",
    "degeneré": "degenerar",
    "degeneréis": "degenerar",
    "degeneró": "degenerar",
    "degluciones": "deglución",
    "degluta": "deglutir",
    "deglutamos": "deglutir",
    "deglutan": "deglutir",
    "deglutas": "deglutir",
    "deglute": "deglutir",
    "degluten": "deglutir",
    "deglutes": "deglutir",
    "deglutid": "deglutir",
    "deglutida": "deglutir",
    "deglutidas": "deglutir",
    "deglutido": "deglutir",
    "deglutidos": "deglutir",
    "deglutiendo": "deglutir",
    "deglutiera": "deglutir",
    "deglutierais": "deglutir",
    "deglutieran": "deglutir",
    "deglutieras": "deglutir",
    "deglutiere": "deglutir",
    "deglutiereis": "deglutir",
    "deglutieren": "deglutir",
    "deglutieres": "deglutir",
    "deglutieron": "deglutir",
    "deglutiese": "deglutir",
    "deglutieseis": "deglutir",
    "deglutiesen": "deglutir",
    "deglutieses": "deglutir",
    "deglutimos": "deglutir",
    "deglutiremos": "deglutir",
    "deglutirá": "deglutir",
    "deglutirán": "deglutir",
    "deglutirás": "deglutir",
    "deglutiré": "deglutir",
    "deglutiréis": "deglutir",
    "deglutiría": "deglutir",
    "deglutiríais": "deglutir",
    "deglutiríamos": "deglutir",
    "deglutirían": "deglutir",
    "deglutirías": "deglutir",
    "deglutiste": "deglutir",
    "deglutisteis": "deglutir",
    "deglutiéramos": "deglutir",
    "deglutiéremos": "deglutir",
    "deglutiésemos": "deglutir",
    "deglutió": "deglutir",
    "degluto": "deglutir",
    "deglutáis": "deglutir",
    "deglutí": "deglutir",
    "deglutía": "deglutir",
    "deglutíais": "deglutir",
    "deglutíamos": "deglutir",
    "deglutían": "deglutir",
    "deglutías": "deglutir",
    "deglutís": "deglutir",
    "degollaba": "degollar",
    "degollabais": "degollar",
    "degollaban": "degollar",
    "degollabas": "degollar",
    "degollaciones": "degollación",
    "degollad": "degollar",
    "degollada": "degollar",
    "degolladas": "degollar",
    "degolladeros": "degolladero",
    "degollado": "degollar",
    "degolladora": "degollador",
    "degolladoras": "degollador",
    "degolladores": "degollador",
    "degollados": "degollar",
    "degolladuras": "degolladura",
    "degollamientos": "degollamiento",
    "degollamos": "degollar",
    "degollando": "degollar",
    "degollara": "degollar",
    "degollarais": "degollar",
    "degollaran": "degollar",
    "degollaras": "degollar",
    "degollare": "degollar",
    "degollareis": "degollar",
    "degollaremos": "degollar",
    "degollaren": "degollar",
    "degollares": "degollar",
    "degollaron": "degollar",
    "degollará": "degollar",
    "degollarán": "degollar",
    "degollarás": "degollar",
    "degollaré": "degollar",
    "degollaréis": "degollar",
    "degollaría": "degollar",
    "degollaríais": "degollar",
    "degollaríamos": "degollar",
    "degollarían": "degollar",
    "degollarías": "degollar",
    "degollase": "degollar",
    "degollaseis": "degollar",
    "degollasen": "degollar",
    "degollases": "degollar",
    "degollaste": "degollar",
    "degollasteis": "degollar",
    "degollemos": "degollar",
    "degollinas": "degollina",
    "degollábamos": "degollar",
    "degolláis": "degollar",
    "degolláramos": "degollar",
    "degolláremos": "degollar",
    "degollásemos": "degollar",
    "degollé": "degollar",
    "degolléis": "degollar",
    "degolló": "degollar",
    "degrada": "degradar",
    "degradaba": "degradar",
    "degradabais": "degradar",
    "degradaban": "degradar",
    "degradabas": "degradar",
    "degradabilidades": "degradabilidad",
    "degradables": "degradable",
    "degradaciones": "degradación",
    "degradad": "degradar",
    "degradada": "degradar",
    "degradadas": "degradar",
    "degradado": "degradar",
    "degradados": "degradar",
    "degradamos": "degradar",
    "degradan": "degradar",
    "degradando": "degradar",
    "degradantes": "degradante",
    "degradara": "degradar",
    "degradarais": "degradar",
    "degradaran": "degradar",
    "degradaras": "degradar",
    "degradare": "degradar",
    "degradareis": "degradar",
    "degradaremos": "degradar",
    "degradaren": "degradar",
    "degradares": "degradar",
    "degradaron": "degradar",
    "degradará": "degradar",
    "degradarán": "degradar",
    "degradarás": "degradar",
    "degradaré": "degradar",
    "degradaréis": "degradar",
    "degradaría": "degradar",
    "degradaríais": "degradar",
    "degradaríamos": "degradar",
    "degradarían": "degradar",
    "degradarías": "degradar",
    "degradas": "degradar",
    "degradase": "degradar",
    "degradaseis": "degradar",
    "degradasen": "degradar",
    "degradases": "degradar",
    "degradaste": "degradar",
    "degradasteis": "degradar",
    "degrade": "degradar",
    "degrademos": "degradar",
    "degraden": "degradar",
    "degrades": "degradar",
    "degrado": "degradar",
    "degradábamos": "degradar",
    "degradáis": "degradar",
    "degradáramos": "degradar",
    "degradáremos": "degradar",
    "degradásemos": "degradar",
    "degradé": "degradar",
    "degradéis": "degradar",
    "degradó": "degradar",
    "degresiva": "degresivo",
    "degresivas": "degresivo",
    "degresivos": "degresivo",
    "degusta": "degustar",
    "degustaba": "degustar",
    "degustabais": "degustar",
    "degustaban": "degustar",
    "degustabas": "degustar",
    "degustaciones": "degustación",
    "degustad": "degustar",
    "degustada": "degustar",
    "degustadas": "degustar",
    "degustado": "degustar",
    "degustadora": "degustador",
    "degustadoras": "degustador",
    "degustadores": "degustador",
    "degustados": "degustar",
    "degustamos": "degustar",
    "degustan": "degustar",
    "degustando": "degustar",
    "degustara": "degustar",
    "degustarais": "degustar",
    "degustaran": "degustar",
    "degustaras": "degustar",
    "degustare": "degustar",
    "degustareis": "degustar",
    "degustaremos": "degustar",
    "degustaren": "degustar",
    "degustares": "degustar",
    "degustaron": "degustar",
    "degustará": "degustar",
    "degustarán": "degustar",
    "degustarás": "degustar",
    "degustaré": "degustar",
    "degustaréis": "degustar",
    "degustaría": "degustar",
    "degustaríais": "degustar",
    "degustaríamos": "degustar",
    "degustarían": "degustar",
    "degustarías": "degustar",
    "degustas": "degustar",
    "degustase": "degustar",
    "degustaseis": "degustar",
    "degustasen": "degustar",
    "degustases": "degustar",
    "degustaste": "degustar",
    "degustasteis": "degustar",
    "deguste": "degustar",
    "degustemos": "degustar",
    "degusten": "degustar",
    "degustes": "degustar",
    "degusto": "degustar",
    "degustábamos": "degustar",
    "degustáis": "degustar",
    "degustáramos": "degustar",
    "degustáremos": "degustar",
    "degustásemos": "degustar",
    "degusté": "degustar",
    "degustéis": "degustar",
    "degustó": "degustar",
    "degúes": "degú",
    "degús": "degú",
    "degüella": "degollar",
    "degüellan": "degollar",
    "degüellas": "degollar",
    "degüelle": "degollar",
    "degüellen": "degollar",
    "degüelles": "degollar",
    "degüello": "degollar",
    "degüellos": "degüello",
    "dehesas": "dehesa",
    "dehidroacetatos": "dehidroacetato",
    "dehidroacética": "dehidroacético",
    "dehidroacéticas": "dehidroacético",
    "dehidroacéticos": "dehidroacético",
    "dehidrogenasas": "dehidrogenasa",
    "dehidrotiotoluidinas": "dehidrotiotoluidina",
    "dehiscencias": "dehiscencia",
    "dehiscentes": "dehiscente",
    "deicidas": "deicida",
    "deicidios": "deicidio",
    "deidades": "deidad",
    "deifica": "deificar",
    "deificaba": "deificar",
    "deificabais": "deificar",
    "deificaban": "deificar",
    "deificabas": "deificar",
    "deificaciones": "deificación",
    "deificad": "deificar",
    "deificada": "deificar",
    "deificadas": "deificar",
    "deificado": "deificar",
    "deificados": "deificar",
    "deificamos": "deificar",
    "deifican": "deificar",
    "deificando": "deificar",
    "deificara": "deificar",
    "deificarais": "deificar",
    "deificaran": "deificar",
    "deificaras": "deificar",
    "deificare": "deificar",
    "deificareis": "deificar",
    "deificaremos": "deificar",
    "deificaren": "deificar",
    "deificares": "deificar",
    "deificaron": "deificar",
    "deificará": "deificar",
    "deificarán": "deificar",
    "deificarás": "deificar",
    "deificaré": "deificar",
    "deificaréis": "deificar",
    "deificaría": "deificar",
    "deificaríais": "deificar",
    "deificaríamos": "deificar",
    "deificarían": "deificar",
    "deificarías": "deificar",
    "deificas": "deificar",
    "deificase": "deificar",
    "deificaseis": "deificar",
    "deificasen": "deificar",
    "deificases": "deificar",
    "deificaste": "deificar",
    "deificasteis": "deificar",
    "deifico": "deificar",
    "deificábamos": "deificar",
    "deificáis": "deificar",
    "deificáramos": "deificar",
    "deificáremos": "deificar",
    "deificásemos": "deificar",
    "deificó": "deificar",
    "deifique": "deificar",
    "deifiquemos": "deificar",
    "deifiquen": "deificar",
    "deifiques": "deificar",
    "deifiqué": "deificar",
    "deifiquéis": "deificar",
    "deis": "dar",
    "deja": "dejar",
    "dejaba": "dejar",
    "dejabais": "dejar",
    "dejaban": "dejar",
    "dejabas": "dejar",
    "dejaciones": "dejación",
    "dejaciónes": "dejación",
    "dejad": "dejar",
    "dejada": "dejar",
    "dejadas": "dejar",
    "dejadeces": "dejadez",
    "dejado": "dejar",
    "dejados": "dejar",
    "dejamientos": "dejamiento",
    "dejamos": "dejar",
    "dejan": "dejar",
    "dejando": "dejar",
    "dejara": "dejar",
    "dejarais": "dejar",
    "dejaran": "dejar",
    "dejaras": "dejar",
    "dejare": "dejar",
    "dejareis": "dejar",
    "dejaremos": "dejar",
    "dejaren": "dejar",
    "dejares": "dejar",
    "dejaron": "dejar",
    "dejará": "dejar",
    "dejarán": "dejar",
    "dejarás": "dejar",
    "dejaré": "dejar",
    "dejaréis": "dejar",
    "dejaría": "dejar",
    "dejaríais": "dejar",
    "dejaríamos": "dejar",
    "dejarían": "dejar",
    "dejarías": "dejar",
    "dejas": "dejar",
    "dejase": "dejar",
    "dejaseis": "dejar",
    "dejasen": "dejar",
    "dejases": "dejar",
    "dejaste": "dejar",
    "dejasteis": "dejar",
    "dejativa": "dejativo",
    "dejativas": "dejativo",
    "dejativos": "dejativo",
    "deje": "dejar",
    "dejemos": "dejar",
    "dejen": "dejar",
    "dejes": "deje",
    "dejo": "dejar",
    "dejos": "dejo",
    "dejábamos": "dejar",
    "dejáis": "dejar",
    "dejáramos": "dejar",
    "dejáremos": "dejar",
    "dejásemos": "dejar",
    "dejé": "dejar",
    "dejéis": "dejar",
    "dejó": "dejar",
    "delaciones": "delación",
    "delantales": "delantal",
    "delantera": "delantero",
    "delanteras": "delantero",
    "delanteros": "delantero",
    "delata": "delatar",
    "delataba": "delatar",
    "delatabais": "delatar",
    "delataban": "delatar",
    "delatabas": "delatar",
    "delatables": "delatable",
    "delatad": "delatar",
    "delatada": "delatar",
    "delatadas": "delatar",
    "delatado": "delatar",
    "delatadora": "delatador",
    "delatadoras": "delatador",
    "delatadores": "delatador",
    "delatados": "delatar",
    "delatamos": "delatar",
    "delatan": "delatar",
    "delatando": "delatar",
    "delatara": "delatar",
    "delatarais": "delatar",
    "delataran": "delatar",
    "delataras": "delatar",
    "delatare": "delatar",
    "delatareis": "delatar",
    "delataremos": "delatar",
    "delataren": "delatar",
    "delatares": "delatar",
    "delataron": "delatar",
    "delatará": "delatar",
    "delatarán": "delatar",
    "delatarás": "delatar",
    "delataré": "delatar",
    "delataréis": "delatar",
    "delataría": "delatar",
    "delataríais": "delatar",
    "delataríamos": "delatar",
    "delatarían": "delatar",
    "delatarías": "delatar",
    "delatas": "delatar",
    "delatase": "delatar",
    "delataseis": "delatar",
    "delatasen": "delatar",
    "delatases": "delatar",
    "delataste": "delatar",
    "delatasteis": "delatar",
    "delate": "delatar",
    "delatemos": "delatar",
    "delaten": "delatar",
    "delates": "delatar",
    "delato": "delatar",
    "delatora": "delator",
    "delatoras": "delator",
    "delatores": "delator",
    "delatábamos": "delatar",
    "delatáis": "delatar",
    "delatáramos": "delatar",
    "delatáremos": "delatar",
    "delatásemos": "delatar",
    "delaté": "delatar",
    "delatéis": "delatar",
    "delató": "delatar",
    "delcos": "delco",
    "delebles": "deleble",
    "delecciones": "delección",
    "deleciones": "deleción",
    "delectaciones": "delectación",
    "delega": "delegar",
    "delegaba": "delegar",
    "delegabais": "delegar",
    "delegaban": "delegar",
    "delegabas": "delegar",
    "delegables": "delegable",
    "delegaciones": "delegación",
    "delegad": "delegar",
    "delegada": "delegar",
    "delegadas": "delegar",
    "delegado": "delegar",
    "delegados": "delegar",
    "delegamos": "delegar",
    "delegan": "delegar",
    "delegando": "delegar",
    "delegara": "delegar",
    "delegarais": "delegar",
    "delegaran": "delegar",
    "delegaras": "delegar",
    "delegare": "delegar",
    "delegareis": "delegar",
    "delegaremos": "delegar",
    "delegaren": "delegar",
    "delegares": "delegar",
    "delegaron": "delegar",
    "delegará": "delegar",
    "delegarán": "delegar",
    "delegarás": "delegar",
    "delegaré": "delegar",
    "delegaréis": "delegar",
    "delegaría": "delegar",
    "delegaríais": "delegar",
    "delegaríamos": "delegar",
    "delegarían": "delegar",
    "delegarías": "delegar",
    "delegas": "delegar",
    "delegase": "delegar",
    "delegaseis": "delegar",
    "delegasen": "delegar",
    "delegases": "delegar",
    "delegaste": "delegar",
    "delegasteis": "delegar",
    "delegativa": "delegativo",
    "delegativas": "delegativo",
    "delegativos": "delegativo",
    "delegatoria": "delegatorio",
    "delegatorias": "delegatorio",
    "delegatorios": "delegatorio",
    "delego": "delegar",
    "delegue": "delegar",
    "deleguemos": "delegar",
    "deleguen": "delegar",
    "delegues": "delegar",
    "delegué": "delegar",
    "deleguéis": "delegar",
    "delegábamos": "delegar",
    "delegáis": "delegar",
    "delegáramos": "delegar",
    "delegáremos": "delegar",
    "delegásemos": "delegar",
    "delegó": "delegar",
    "deleita": "deleitar",
    "deleitaba": "deleitar",
    "deleitabais": "deleitar",
    "deleitaban": "deleitar",
    "deleitabas": "deleitar",
    "deleitables": "deleitable",
    "deleitaciones": "deleitación",
    "deleitad": "deleitar",
    "deleitada": "deleitar",
    "deleitadas": "deleitar",
    "deleitado": "deleitar",
    "deleitados": "deleitar",
    "deleitamientos": "deleitamiento",
    "deleitamos": "deleitar",
    "deleitan": "deleitar",
    "deleitando": "deleitar",
    "deleitara": "deleitar",
    "deleitarais": "deleitar",
    "deleitaran": "deleitar",
    "deleitaras": "deleitar",
    "deleitare": "deleitar",
    "deleitareis": "deleitar",
    "deleitaremos": "deleitar",
    "deleitaren": "deleitar",
    "deleitares": "deleitar",
    "deleitaron": "deleitar",
    "deleitará": "deleitar",
    "deleitarán": "deleitar",
    "deleitarás": "deleitar",
    "deleitaré": "deleitar",
    "deleitaréis": "deleitar",
    "deleitaría": "deleitar",
    "deleitaríais": "deleitar",
    "deleitaríamos": "deleitar",
    "deleitarían": "deleitar",
    "deleitarías": "deleitar",
    "deleitas": "deleitar",
    "deleitase": "deleitar",
    "deleitaseis": "deleitar",
    "deleitasen": "deleitar",
    "deleitases": "deleitar",
    "deleitaste": "deleitar",
    "deleitasteis": "deleitar",
    "deleite": "deleitar",
    "deleitemos": "deleitar",
    "deleiten": "deleitar",
    "deleites": "deleite",
    "deleito": "deleitar",
    "deleitosa": "deleitoso",
    "deleitosas": "deleitoso",
    "deleitosos": "deleitoso",
    "deleitábamos": "deleitar",
    "deleitáis": "deleitar",
    "deleitáramos": "deleitar",
    "deleitáremos": "deleitar",
    "deleitásemos": "deleitar",
    "deleité": "deleitar",
    "deleitéis": "deleitar",
    "deleitó": "deleitar",
    "deles": "dele",
    "deletrea": "deletrear",
    "deletreaba": "deletrear",
    "deletreabais": "deletrear",
    "deletreaban": "deletrear",
    "deletreabas": "deletrear",
    "deletread": "deletrear",
    "deletreada": "deletrear",
    "deletreadas": "deletrear",
    "deletreado": "deletrear",
    "deletreados": "deletrear",
    "deletreamos": "deletrear",
    "deletrean": "deletrear",
    "deletreando": "deletrear",
    "deletreara": "deletrear",
    "deletrearais": "deletrear",
    "deletrearan": "deletrear",
    "deletrearas": "deletrear",
    "deletreare": "deletrear",
    "deletreareis": "deletrear",
    "deletrearemos": "deletrear",
    "deletrearen": "deletrear",
    "deletreares": "deletrear",
    "deletrearon": "deletrear",
    "deletreará": "deletrear",
    "deletrearán": "deletrear",
    "deletrearás": "deletrear",
    "deletrearé": "deletrear",
    "deletrearéis": "deletrear",
    "deletrearía": "deletrear",
    "deletrearíais": "deletrear",
    "deletrearíamos": "deletrear",
    "deletrearían": "deletrear",
    "deletrearías": "deletrear",
    "deletreas": "deletrear",
    "deletrease": "deletrear",
    "deletreaseis": "deletrear",
    "deletreasen": "deletrear",
    "deletreases": "deletrear",
    "deletreaste": "deletrear",
    "deletreasteis": "deletrear",
    "deletree": "deletrear",
    "deletreemos": "deletrear",
    "deletreen": "deletrear",
    "deletrees": "deletrear",
    "deletreo": "deletrear",
    "deletreos": "deletreo",
    "deletreábamos": "deletrear",
    "deletreáis": "deletrear",
    "deletreáramos": "deletrear",
    "deletreáremos": "deletrear",
    "deletreásemos": "deletrear",
    "deletreé": "deletrear",
    "deletreéis": "deletrear",
    "deletreó": "deletrear",
    "deletérea": "deletéreo",
    "deletéreas": "deletéreo",
    "deletéreos": "deletéreo",
    "deleznables": "deleznable",
    "deleátures": "deleátur",
    "delfines": "delfín",
    "delfininas": "delfinina",
    "delfínidos": "delfínido",
    "delgada": "delgado",
    "delgadas": "delgado",
    "delgadeces": "delgadez",
    "delgados": "delgado",
    "delgaducha": "delgaducho",
    "delgaduchas": "delgaducho",
    "delgaduchos": "delgaducho",
    "delgaduita": "delgaduito",
    "delgaduitas": "delgaduito",
    "delgaduitos": "delgaduito",
    "delgas": "delga",
    "delibera": "deliberar",
    "deliberaba": "deliberar",
    "deliberabais": "deliberar",
    "deliberaban": "deliberar",
    "deliberabas": "deliberar",
    "deliberaciones": "deliberación",
    "deliberad": "deliberar",
    "deliberada": "deliberar",
    "deliberadas": "deliberar",
    "deliberado": "deliberar",
    "deliberadora": "deliberador",
    "deliberadoras": "deliberador",
    "deliberadores": "deliberador",
    "deliberados": "deliberar",
    "deliberamos": "deliberar",
    "deliberan": "deliberar",
    "deliberando": "deliberar",
    "deliberantes": "deliberante",
    "deliberara": "deliberar",
    "deliberarais": "deliberar",
    "deliberaran": "deliberar",
    "deliberaras": "deliberar",
    "deliberare": "deliberar",
    "deliberareis": "deliberar",
    "deliberaremos": "deliberar",
    "deliberaren": "deliberar",
    "deliberares": "deliberar",
    "deliberaron": "deliberar",
    "deliberará": "deliberar",
    "deliberarán": "deliberar",
    "deliberarás": "deliberar",
    "deliberaré": "deliberar",
    "deliberaréis": "deliberar",
    "deliberaría": "deliberar",
    "deliberaríais": "deliberar",
    "deliberaríamos": "deliberar",
    "deliberarían": "deliberar",
    "deliberarías": "deliberar",
    "deliberas": "deliberar",
    "deliberase": "deliberar",
    "deliberaseis": "deliberar",
    "deliberasen": "deliberar",
    "deliberases": "deliberar",
    "deliberaste": "deliberar",
    "deliberasteis": "deliberar",
    "deliberativa": "deliberativo",
    "deliberativas": "deliberativo",
    "deliberativos": "deliberativo",
    "delibere": "deliberar",
    "deliberemos": "deliberar",
    "deliberen": "deliberar",
    "deliberes": "deliberar",
    "delibero": "deliberar",
    "deliberábamos": "deliberar",
    "deliberáis": "deliberar",
    "deliberáramos": "deliberar",
    "deliberáremos": "deliberar",
    "deliberásemos": "deliberar",
    "deliberé": "deliberar",
    "deliberéis": "deliberar",
    "deliberó": "deliberar",
    "delicada": "delicado",
    "delicadas": "delicado",
    "delicadeces": "delicadez",
    "delicadezas": "delicadeza",
    "delicados": "delicado",
    "delicaducha": "delicaducho",
    "delicaduchas": "delicaducho",
    "delicaduchos": "delicaducho",
    "delicias": "delicia",
    "deliciosa": "delicioso",
    "deliciosas": "delicioso",
    "deliciosos": "delicioso",
    "delictiva": "delictivo",
    "delictivas": "delictivo",
    "delictivos": "delictivo",
    "delictuales": "delictual",
    "delictuosa": "delictuoso",
    "delictuosas": "delictuoso",
    "delictuosos": "delictuoso",
    "delicuenciales": "delicuencial",
    "delicuescencias": "delicuescencia",
    "delicuescentes": "delicuescente",
    "delimita": "delimitar",
    "delimitaba": "delimitar",
    "delimitabais": "delimitar",
    "delimitaban": "delimitar",
    "delimitabas": "delimitar",
    "delimitables": "delimitable",
    "delimitaciones": "delimitación",
    "delimitad": "delimitar",
    "delimitada": "delimitar",
    "delimitadas": "delimitar",
    "delimitado": "delimitar",
    "delimitadora": "delimitador",
    "delimitadoras": "delimitador",
    "delimitadores": "delimitador",
    "delimitados": "delimitar",
    "delimitamos": "delimitar",
    "delimitan": "delimitar",
    "delimitando": "delimitar",
    "delimitantes": "delimitante",
    "delimitara": "delimitar",
    "delimitarais": "delimitar",
    "delimitaran": "delimitar",
    "delimitaras": "delimitar",
    "delimitare": "delimitar",
    "delimitareis": "delimitar",
    "delimitaremos": "delimitar",
    "delimitaren": "delimitar",
    "delimitares": "delimitar",
    "delimitaron": "delimitar",
    "delimitará": "delimitar",
    "delimitarán": "delimitar",
    "delimitarás": "delimitar",
    "delimitaré": "delimitar",
    "delimitaréis": "delimitar",
    "delimitaría": "delimitar",
    "delimitaríais": "delimitar",
    "delimitaríamos": "delimitar",
    "delimitarían": "delimitar",
    "delimitarías": "delimitar",
    "delimitas": "delimitar",
    "delimitase": "delimitar",
    "delimitaseis": "delimitar",
    "delimitasen": "delimitar",
    "delimitases": "delimitar",
    "delimitaste": "delimitar",
    "delimitasteis": "delimitar",
    "delimite": "delimitar",
    "delimitemos": "delimitar",
    "delimiten": "delimitar",
    "delimites": "delimitar",
    "delimito": "delimitar",
    "delimitábamos": "delimitar",
    "delimitáis": "delimitar",
    "delimitáramos": "delimitar",
    "delimitáremos": "delimitar",
    "delimitásemos": "delimitar",
    "delimité": "delimitar",
    "delimitéis": "delimitar",
    "delimitó": "delimitar",
    "delinca": "delinquir",
    "delincamos": "delinquir",
    "delincan": "delinquir",
    "delincas": "delinquir",
    "delinco": "delinquir",
    "delincuenciales": "delincuencial",
    "delincuencias": "delincuencia",
    "delincuentes": "delincuente",
    "delincáis": "delinquir",
    "delinea": "delinear",
    "delineaba": "delinear",
    "delineabais": "delinear",
    "delineaban": "delinear",
    "delineabas": "delinear",
    "delineaciones": "delineación",
    "delinead": "delinear",
    "delineada": "delinear",
    "delineadas": "delinear",
    "delineado": "delinear",
    "delineadora": "delineador",
    "delineadoras": "delineador",
    "delineadores": "delineador",
    "delineados": "delinear",
    "delineamientos": "delineamiento",
    "delineamos": "delinear",
    "delinean": "delinear",
    "delineando": "delinear",
    "delineantes": "delineante",
    "delineara": "delinear",
    "delinearais": "delinear",
    "delinearan": "delinear",
    "delinearas": "delinear",
    "delineare": "delinear",
    "delineareis": "delinear",
    "delinearemos": "delinear",
    "delinearen": "delinear",
    "delineares": "delinear",
    "delinearon": "delinear",
    "delineará": "delinear",
    "delinearán": "delinear",
    "delinearás": "delinear",
    "delinearé": "delinear",
    "delinearéis": "delinear",
    "delinearía": "delinear",
    "delinearíais": "delinear",
    "delinearíamos": "delinear",
    "delinearían": "delinear",
    "delinearías": "delinear",
    "delineas": "delinear",
    "delinease": "delinear",
    "delineaseis": "delinear",
    "delineasen": "delinear",
    "delineases": "delinear",
    "delineaste": "delinear",
    "delineasteis": "delinear",
    "delinee": "delinear",
    "delineemos": "delinear",
    "delineen": "delinear",
    "delinees": "delinear",
    "delineo": "delinear",
    "delineábamos": "delinear",
    "delineáis": "delinear",
    "delineáramos": "delinear",
    "delineáremos": "delinear",
    "delineásemos": "delinear",
    "delineé": "delinear",
    "delineéis": "delinear",
    "delineó": "delinear",
    "delinque": "delinquir",
    "delinquen": "delinquir",
    "delinques": "delinquir",
    "delinquid": "delinquir",
    "delinquida": "delinquir",
    "delinquidas": "delinquir",
    "delinquido": "delinquir",
    "delinquidos": "delinquir",
    "delinquiendo": "delinquir",
    "delinquiera": "delinquir",
    "delinquierais": "delinquir",
    "delinquieran": "delinquir",
    "delinquieras": "delinquir",
    "delinquiere": "delinquir",
    "delinquiereis": "delinquir",
    "delinquieren": "delinquir",
    "delinquieres": "delinquir",
    "delinquieron": "delinquir",
    "delinquiese": "delinquir",
    "delinquieseis": "delinquir",
    "delinquiesen": "delinquir",
    "delinquieses": "delinquir",
    "delinquimientos": "delinquimiento",
    "delinquimos": "delinquir",
    "delinquiremos": "delinquir",
    "delinquirá": "delinquir",
    "delinquirán": "delinquir",
    "delinquirás": "delinquir",
    "delinquiré": "delinquir",
    "delinquiréis": "delinquir",
    "delinquiría": "delinquir",
    "delinquiríais": "delinquir",
    "delinquiríamos": "delinquir",
    "delinquirían": "delinquir",
    "delinquirías": "delinquir",
    "delinquiste": "delinquir",
    "delinquisteis": "delinquir",
    "delinquiéramos": "delinquir",
    "delinquiéremos": "delinquir",
    "delinquiésemos": "delinquir",
    "delinquió": "delinquir",
    "delinquí": "delinquir",
    "delinquía": "delinquir",
    "delinquíais": "delinquir",
    "delinquíamos": "delinquir",
    "delinquían": "delinquir",
    "delinquías": "delinquir",
    "delinquís": "delinquir",
    "delira": "delirar",
    "deliraba": "delirar",
    "delirabais": "delirar",
    "deliraban": "delirar",
    "delirabas": "delirar",
    "delirad": "delirar",
    "delirada": "delirar",
    "deliradas": "delirar",
    "delirado": "delirar",
    "delirados": "delirar",
    "deliramos": "delirar",
    "deliran": "delirar",
    "delirando": "delirar",
    "delirantes": "delirante",
    "delirara": "delirar",
    "delirarais": "delirar",
    "deliraran": "delirar",
    "deliraras": "delirar",
    "delirare": "delirar",
    "delirareis": "delirar",
    "deliraremos": "delirar",
    "deliraren": "delirar",
    "delirares": "delirar",
    "deliraron": "delirar",
    "delirará": "delirar",
    "delirarán": "delirar",
    "delirarás": "delirar",
    "deliraré": "delirar",
    "deliraréis": "delirar",
    "deliraría": "delirar",
    "deliraríais": "delirar",
    "deliraríamos": "delirar",
    "delirarían": "delirar",
    "delirarías": "delirar",
    "deliras": "delirar",
    "delirase": "delirar",
    "deliraseis": "delirar",
    "delirasen": "delirar",
    "delirases": "delirar",
    "deliraste": "delirar",
    "delirasteis": "delirar",
    "delire": "delirar",
    "deliremos": "delirar",
    "deliren": "delirar",
    "delires": "delirar",
    "delirios": "delirio",
    "deliro": "delirar",
    "delirábamos": "delirar",
    "deliráis": "delirar",
    "deliráramos": "delirar",
    "deliráremos": "delirar",
    "delirásemos": "delirar",
    "deliré": "delirar",
    "deliréis": "delirar",
    "deliró": "delirar",
    "delitescencias": "delitescencia",
    "delitos": "delito",
    "delocutiva": "delocutivo",
    "delocutivas": "delocutivo",
    "delocutivos": "delocutivo",
    "deltaica": "deltaico",
    "deltaicas": "deltaico",
    "deltaicos": "deltaico",
    "deltas": "delta",
    "deltoidea": "deltoideo",
    "deltoideas": "deltoideo",
    "deltoideos": "deltoideo",
    "delusiones": "delusión",
    "delusiva": "delusivo",
    "delusivas": "delusivo",
    "delusivos": "delusivo",
    "delusoria": "delusorio",
    "delusorias": "delusorio",
    "delusorios": "delusorio",
    "delustrantes": "delustrante",
    "demacra": "demacrar",
    "demacraba": "demacrar",
    "demacrabais": "demacrar",
    "demacraban": "demacrar",
    "demacrabas": "demacrar",
    "demacraciones": "demacración",
    "demacrad": "demacrar",
    "demacrada": "demacrar",
    "demacradas": "demacrar",
    "demacrado": "demacrar",
    "demacrados": "demacrar",
    "demacramos": "demacrar",
    "demacran": "demacrar",
    "demacrando": "demacrar",
    "demacrara": "demacrar",
    "demacrarais": "demacrar",
    "demacraran": "demacrar",
    "demacraras": "demacrar",
    "demacrare": "demacrar",
    "demacrareis": "demacrar",
    "demacraremos": "demacrar",
    "demacraren": "demacrar",
    "demacrares": "demacrar",
    "demacraron": "demacrar",
    "demacrará": "demacrar",
    "demacrarán": "demacrar",
    "demacrarás": "demacrar",
    "demacraré": "demacrar",
    "demacraréis": "demacrar",
    "demacraría": "demacrar",
    "demacraríais": "demacrar",
    "demacraríamos": "demacrar",
    "demacrarían": "demacrar",
    "demacrarías": "demacrar",
    "demacras": "demacrar",
    "demacrase": "demacrar",
    "demacraseis": "demacrar",
    "demacrasen": "demacrar",
    "demacrases": "demacrar",
    "demacraste": "demacrar",
    "demacrasteis": "demacrar",
    "demacre": "demacrar",
    "demacremos": "demacrar",
    "demacren": "demacrar",
    "demacres": "demacrar",
    "demacro": "demacrar",
    "demacrábamos": "demacrar",
    "demacráis": "demacrar",
    "demacráramos": "demacrar",
    "demacráremos": "demacrar",
    "demacrásemos": "demacrar",
    "demacré": "demacrar",
    "demacréis": "demacrar",
    "demacró": "demacrar",
    "demagoga": "demagogo",
    "demagogas": "demagogo",
    "demagogias": "demagogia",
    "demagogismos": "demagogismo",
    "demagogos": "demagogo",
    "demagógica": "demagógico",
    "demagógicas": "demagógico",
    "demagógicos": "demagógico",
    "demanda": "demandar",
    "demandaba": "demandar",
    "demandabais": "demandar",
    "demandaban": "demandar",
    "demandabas": "demandar",
    "demandad": "demandar",
    "demandada": "demandar",
    "demandadas": "demandar",
    "demandadera": "demandadero",
    "demandaderas": "demandadero",
    "demandaderos": "demandadero",
    "demandado": "demandar",
    "demandadora": "demandador",
    "demandadoras": "demandador",
    "demandadores": "demandador",
    "demandados": "demandar",
    "demandamos": "demandar",
    "demandan": "demandar",
    "demandando": "demandar",
    "demandantes": "demandante",
    "demandara": "demandar",
    "demandarais": "demandar",
    "demandaran": "demandar",
    "demandaras": "demandar",
    "demandare": "demandar",
    "demandareis": "demandar",
    "demandaremos": "demandar",
    "demandaren": "demandar",
    "demandares": "demandar",
    "demandaron": "demandar",
    "demandará": "demandar",
    "demandarán": "demandar",
    "demandarás": "demandar",
    "demandaré": "demandar",
    "demandaréis": "demandar",
    "demandaría": "demandar",
    "demandaríais": "demandar",
    "demandaríamos": "demandar",
    "demandarían": "demandar",
    "demandarías": "demandar",
    "demandas": "demandar",
    "demandase": "demandar",
    "demandaseis": "demandar",
    "demandasen": "demandar",
    "demandases": "demandar",
    "demandaste": "demandar",
    "demandasteis": "demandar",
    "demande": "demandar",
    "demandemos": "demandar",
    "demanden": "demandar",
    "demandes": "demandar",
    "demando": "demandar",
    "demandábamos": "demandar",
    "demandáis": "demandar",
    "demandáramos": "demandar",
    "demandáremos": "demandar",
    "demandásemos": "demandar",
    "demandé": "demandar",
    "demandéis": "demandar",
    "demandó": "demandar",
    "demaniales": "demanial",
    "demanios": "demanio",
    "demantoides": "demantoide",
    "demarca": "demarcar",
    "demarcaba": "demarcar",
    "demarcabais": "demarcar",
    "demarcaban": "demarcar",
    "demarcabas": "demarcar",
    "demarcaciones": "demarcación",
    "demarcad": "demarcar",
    "demarcada": "demarcar",
    "demarcadas": "demarcar",
    "demarcado": "demarcar",
    "demarcadora": "demarcador",
    "demarcadoras": "demarcador",
    "demarcadores": "demarcador",
    "demarcados": "demarcar",
    "demarcamos": "demarcar",
    "demarcan": "demarcar",
    "demarcando": "demarcar",
    "demarcara": "demarcar",
    "demarcarais": "demarcar",
    "demarcaran": "demarcar",
    "demarcaras": "demarcar",
    "demarcare": "demarcar",
    "demarcareis": "demarcar",
    "demarcaremos": "demarcar",
    "demarcaren": "demarcar",
    "demarcares": "demarcar",
    "demarcaron": "demarcar",
    "demarcará": "demarcar",
    "demarcarán": "demarcar",
    "demarcarás": "demarcar",
    "demarcaré": "demarcar",
    "demarcaréis": "demarcar",
    "demarcaría": "demarcar",
    "demarcaríais": "demarcar",
    "demarcaríamos": "demarcar",
    "demarcarían": "demarcar",
    "demarcarías": "demarcar",
    "demarcas": "demarcar",
    "demarcase": "demarcar",
    "demarcaseis": "demarcar",
    "demarcasen": "demarcar",
    "demarcases": "demarcar",
    "demarcaste": "demarcar",
    "demarcasteis": "demarcar",
    "demarco": "demarcar",
    "demarcábamos": "demarcar",
    "demarcáis": "demarcar",
    "demarcáramos": "demarcar",
    "demarcáremos": "demarcar",
    "demarcásemos": "demarcar",
    "demarcó": "demarcar",
    "demarque": "demarcar",
    "demarquemos": "demarcar",
    "demarquen": "demarcar",
    "demarques": "demarcar",
    "demarqué": "demarcar",
    "demarquéis": "demarcar",
    "demarrajes": "demarraje",
    "demasia": "demasiar",
    "demasiaba": "demasiar",
    "demasiabais": "demasiar",
    "demasiaban": "demasiar",
    "demasiabas": "demasiar",
    "demasiad": "demasiar",
    "demasiada": "demasiar",
    "demasiadas": "demasiar",
    "demasiado": "demasiar",
    "demasiados": "demasiar",
    "demasiamos": "demasiar",
    "demasian": "demasiar",
    "demasiando": "demasiar",
    "demasiara": "demasiar",
    "demasiarais": "demasiar",
    "demasiaran": "demasiar",
    "demasiaras": "demasiar",
    "demasiare": "demasiar",
    "demasiareis": "demasiar",
    "demasiaremos": "demasiar",
    "demasiaren": "demasiar",
    "demasiares": "demasiar",
    "demasiaron": "demasiar",
    "demasiará": "demasiar",
    "demasiarán": "demasiar",
    "demasiarás": "demasiar",
    "demasiaré": "demasiar",
    "demasiaréis": "demasiar",
    "demasiaría": "demasiar",
    "demasiaríais": "demasiar",
    "demasiaríamos": "demasiar",
    "demasiarían": "demasiar",
    "demasiarías": "demasiar",
    "demasias": "demasiar",
    "demasiase": "demasiar",
    "demasiaseis": "demasiar",
    "demasiasen": "demasiar",
    "demasiases": "demasiar",
    "demasiaste": "demasiar",
    "demasiasteis": "demasiar",
    "demasie": "demasiar",
    "demasiemos": "demasiar",
    "demasien": "demasiar",
    "demasies": "demasiar",
    "demasio": "demasiar",
    "demasiábamos": "demasiar",
    "demasiáis": "demasiar",
    "demasiáramos": "demasiar",
    "demasiáremos": "demasiar",
    "demasiásemos": "demasiar",
    "demasié": "demasiar",
    "demasiéis": "demasiar",
    "demasió": "demasiar",
    "demasías": "demasía",
    "demenciales": "demencial",
    "demencias": "demencia",
    "dementes": "demente",
    "demeritoria": "demeritorio",
    "demeritorias": "demeritorio",
    "demeritorios": "demeritorio",
    "demiurgos": "demiurgo",
    "democracias": "democracia",
    "democratacristiana": "democratacristiano",
    "democratacristianas": "democratacristiano",
    "democratacristianos": "democratacristiano",
    "democratice": "democratizar",
    "democraticemos": "democratizar",
    "democraticen": "democratizar",
    "democratices": "democratizar",
    "democraticé": "democratizar",
    "democraticéis": "democratizar",
    "democratiza": "democratizar",
    "democratizaba": "democratizar",
    "democratizabais": "democratizar",
    "democratizaban": "democratizar",
    "democratizabas": "democratizar",
    "democratizaciones": "democratización",
    "democratizad": "democratizar",
    "democratizada": "democratizar",
    "democratizadas": "democratizar",
    "democratizado": "democratizar",
    "democratizadora": "democratizador",
    "democratizadoras": "democratizador",
    "democratizadores": "democratizador",
    "democratizados": "democratizar",
    "democratizamos": "democratizar",
    "democratizan": "democratizar",
    "democratizando": "democratizar",
    "democratizara": "democratizar",
    "democratizarais": "democratizar",
    "democratizaran": "democratizar",
    "democratizaras": "democratizar",
    "democratizare": "democratizar",
    "democratizareis": "democratizar",
    "democratizaremos": "democratizar",
    "democratizaren": "democratizar",
    "democratizares": "democratizar",
    "democratizaron": "democratizar",
    "democratizará": "democratizar",
    "democratizarán": "democratizar",
    "democratizarás": "democratizar",
    "democratizaré": "democratizar",
    "democratizaréis": "democratizar",
    "democratizaría": "democratizar",
    "democratizaríais": "democratizar",
    "democratizaríamos": "democratizar",
    "democratizarían": "democratizar",
    "democratizarías": "democratizar",
    "democratizas": "democratizar",
    "democratizase": "democratizar",
    "democratizaseis": "democratizar",
    "democratizasen": "democratizar",
    "democratizases": "democratizar",
    "democratizaste": "democratizar",
    "democratizasteis": "democratizar",
    "democratizo": "democratizar",
    "democratizábamos": "democratizar",
    "democratizáis": "democratizar",
    "democratizáramos": "democratizar",
    "democratizáremos": "democratizar",
    "democratizásemos": "democratizar",
    "democratizó": "democratizar",
    "democristiana": "democristiano",
    "democristianas": "democristiano",
    "democristianos": "democristiano",
    "democrática": "democrático",
    "democráticas": "democrático",
    "democráticos": "democrático",
    "demodula": "demodular",
    "demodulaba": "demodular",
    "demodulabais": "demodular",
    "demodulaban": "demodular",
    "demodulabas": "demodular",
    "demodulaciones": "demodulación",
    "demodulad": "demodular",
    "demodulada": "demodular",
    "demoduladas": "demodular",
    "demodulado": "demodular",
    "demoduladores": "demodulador",
    "demodulados": "demodular",
    "demodulamos": "demodular",
    "demodulan": "demodular",
    "demodulando": "demodular",
    "demodulara": "demodular",
    "demodularais": "demodular",
    "demodularan": "demodular",
    "demodularas": "demodular",
    "demodulare": "demodular",
    "demodulareis": "demodular",
    "demodularemos": "demodular",
    "demodularen": "demodular",
    "demodulares": "demodular",
    "demodularon": "demodular",
    "demodulará": "demodular",
    "demodularán": "demodular",
    "demodularás": "demodular",
    "demodularé": "demodular",
    "demodularéis": "demodular",
    "demodularía": "demodular",
    "demodularíais": "demodular",
    "demodularíamos": "demodular",
    "demodularían": "demodular",
    "demodularías": "demodular",
    "demodulas": "demodular",
    "demodulase": "demodular",
    "demodulaseis": "demodular",
    "demodulasen": "demodular",
    "demodulases": "demodular",
    "demodulaste": "demodular",
    "demodulasteis": "demodular",
    "demodule": "demodular",
    "demodulemos": "demodular",
    "demodulen": "demodular",
    "demodules": "demodular",
    "demodulo": "demodular",
    "demodulábamos": "demodular",
    "demoduláis": "demodular",
    "demoduláramos": "demodular",
    "demoduláremos": "demodular",
    "demodulásemos": "demodular",
    "demodulé": "demodular",
    "demoduléis": "demodular",
    "demoduló": "demodular",
    "demografías": "demografía",
    "demográfica": "demográfico",
    "demográficas": "demográfico",
    "demográficos": "demográfico",
    "demolamos": "demoler",
    "demoled": "demoler",
    "demoledora": "demoledor",
    "demoledoras": "demoledor",
    "demoledores": "demoledor",
    "demolemos": "demoler",
    "demoleremos": "demoler",
    "demolerá": "demoler",
    "demolerán": "demoler",
    "demolerás": "demoler",
    "demoleré": "demoler",
    "demoleréis": "demoler",
    "demolería": "demoler",
    "demoleríais": "demoler",
    "demoleríamos": "demoler",
    "demolerían": "demoler",
    "demolerías": "demoler",
    "demoliciones": "demolición",
    "demolida": "demoler",
    "demolidas": "demoler",
    "demolido": "demoler",
    "demolidos": "demoler",
    "demoliendo": "demoler",
    "demoliera": "demoler",
    "demolierais": "demoler",
    "demolieran": "demoler",
    "demolieras": "demoler",
    "demoliere": "demoler",
    "demoliereis": "demoler",
    "demolieren": "demoler",
    "demolieres": "demoler",
    "demolieron": "demoler",
    "demoliese": "demoler",
    "demolieseis": "demoler",
    "demoliesen": "demoler",
    "demolieses": "demoler",
    "demolimos": "demoler",
    "demoliste": "demoler",
    "demolisteis": "demoler",
    "demoliéramos": "demoler",
    "demoliéremos": "demoler",
    "demoliésemos": "demoler",
    "demolió": "demoler",
    "demoláis": "demoler",
    "demoléis": "demoler",
    "demolí": "demoler",
    "demolía": "demoler",
    "demolíais": "demoler",
    "demolíamos": "demoler",
    "demolían": "demoler",
    "demolías": "demoler",
    "demonches": "demonche",
    "demoniaca": "demoniaco",
    "demoniacas": "demoniaco",
    "demoniacos": "demoniaco",
    "demoniomanías": "demoniomanía",
    "demonios": "demonio",
    "demonolatrías": "demonolatría",
    "demonologías": "demonología",
    "demonológica": "demonológico",
    "demonológicas": "demonológico",
    "demonológicos": "demonológico",
    "demonomancias": "demonomancia",
    "demonomancías": "demonomancía",
    "demonomanías": "demonomanía",
    "demonopatías": "demonopatía",
    "demontres": "demontre",
    "demoníaca": "demoníaco",
    "demoníacas": "demoníaco",
    "demoníacos": "demoníaco",
    "demora": "demorar",
    "demoraba": "demorar",
    "demorabais": "demorar",
    "demoraban": "demorar",
    "demorabas": "demorar",
    "demorad": "demorar",
    "demorada": "demorar",
    "demoradas": "demorar",
    "demorado": "demorar",
    "demorados": "demorar",
    "demoramos": "demorar",
    "demoran": "demorar",
    "demorando": "demorar",
    "demorara": "demorar",
    "demorarais": "demorar",
    "demoraran": "demorar",
    "demoraras": "demorar",
    "demorare": "demorar",
    "demorareis": "demorar",
    "demoraremos": "demorar",
    "demoraren": "demorar",
    "demorares": "demorar",
    "demoraron": "demorar",
    "demorará": "demorar",
    "demorarán": "demorar",
    "demorarás": "demorar",
    "demoraré": "demorar",
    "demoraréis": "demorar",
    "demoraría": "demorar",
    "demoraríais": "demorar",
    "demoraríamos": "demorar",
    "demorarían": "demorar",
    "demorarías": "demorar",
    "demoras": "demorar",
    "demorase": "demorar",
    "demoraseis": "demorar",
    "demorasen": "demorar",
    "demorases": "demorar",
    "demoraste": "demorar",
    "demorasteis": "demorar",
    "demore": "demorar",
    "demoremos": "demorar",
    "demoren": "demorar",
    "demores": "demorar",
    "demoro": "demorar",
    "demorábamos": "demorar",
    "demoráis": "demorar",
    "demoráramos": "demorar",
    "demoráremos": "demorar",
    "demorásemos": "demorar",
    "demoré": "demorar",
    "demoréis": "demorar",
    "demoró": "demorar",
    "demos": "demo",
    "demosofías": "demosofía",
    "demostraba": "demostrar",
    "demostrabais": "demostrar",
    "demostraban": "demostrar",
    "demostrabas": "demostrar",
    "demostrabilidades": "demostrabilidad",
    "demostrables": "demostrable",
    "demostraciones": "demostración",
    "demostrad": "demostrar",
    "demostrada": "demostrar",
    "demostradas": "demostrar",
    "demostrado": "demostrar",
    "demostradora": "demostrador",
    "demostradoras": "demostrador",
    "demostradores": "demostrador",
    "demostrados": "demostrar",
    "demostramos": "demostrar",
    "demostrando": "demostrar",
    "demostrara": "demostrar",
    "demostrarais": "demostrar",
    "demostraran": "demostrar",
    "demostraras": "demostrar",
    "demostrare": "demostrar",
    "demostrareis": "demostrar",
    "demostraremos": "demostrar",
    "demostraren": "demostrar",
    "demostrares": "demostrar",
    "demostraron": "demostrar",
    "demostrará": "demostrar",
    "demostrarán": "demostrar",
    "demostrarás": "demostrar",
    "demostraré": "demostrar",
    "demostraréis": "demostrar",
    "demostraría": "demostrar",
    "demostraríais": "demostrar",
    "demostraríamos": "demostrar",
    "demostrarían": "demostrar",
    "demostrarías": "demostrar",
    "demostrase": "demostrar",
    "demostraseis": "demostrar",
    "demostrasen": "demostrar",
    "demostrases": "demostrar",
    "demostraste": "demostrar",
    "demostrasteis": "demostrar",
    "demostrativa": "demostrativo",
    "demostrativas": "demostrativo",
    "demostrativos": "demostrativo",
    "demostremos": "demostrar",
    "demostrábamos": "demostrar",
    "demostráis": "demostrar",
    "demostráramos": "demostrar",
    "demostráremos": "demostrar",
    "demostrásemos": "demostrar",
    "demostré": "demostrar",
    "demostréis": "demostrar",
    "demostró": "demostrar",
    "demuda": "demudar",
    "demudaba": "demudar",
    "demudabais": "demudar",
    "demudaban": "demudar",
    "demudabas": "demudar",
    "demudaciones": "demudación",
    "demudad": "demudar",
    "demudada": "demudar",
    "demudadas": "demudar",
    "demudado": "demudar",
    "demudados": "demudar",
    "demudamientos": "demudamiento",
    "demudamos": "demudar",
    "demudan": "demudar",
    "demudando": "demudar",
    "demudara": "demudar",
    "demudarais": "demudar",
    "demudaran": "demudar",
    "demudaras": "demudar",
    "demudare": "demudar",
    "demudareis": "demudar",
    "demudaremos": "demudar",
    "demudaren": "demudar",
    "demudares": "demudar",
    "demudaron": "demudar",
    "demudará": "demudar",
    "demudarán": "demudar",
    "demudarás": "demudar",
    "demudaré": "demudar",
    "demudaréis": "demudar",
    "demudaría": "demudar",
    "demudaríais": "demudar",
    "demudaríamos": "demudar",
    "demudarían": "demudar",
    "demudarías": "demudar",
    "demudas": "demudar",
    "demudase": "demudar",
    "demudaseis": "demudar",
    "demudasen": "demudar",
    "demudases": "demudar",
    "demudaste": "demudar",
    "demudasteis": "demudar",
    "demude": "demudar",
    "demudemos": "demudar",
    "demuden": "demudar",
    "demudes": "demudar",
    "demudo": "demudar",
    "demudábamos": "demudar",
    "demudáis": "demudar",
    "demudáramos": "demudar",
    "demudáremos": "demudar",
    "demudásemos": "demudar",
    "demudé": "demudar",
    "demudéis": "demudar",
    "demudó": "demudar",
    "demuela": "demoler",
    "demuelan": "demoler",
    "demuelas": "demoler",
    "demuele": "demoler",
    "demuelen": "demoler",
    "demueles": "demoler",
    "demuelo": "demoler",
    "demuestra": "demostrar",
    "demuestran": "demostrar",
    "demuestras": "demostrar",
    "demuestre": "demostrar",
    "demuestren": "demostrar",
    "demuestres": "demostrar",
    "demuestro": "demostrar",
    "demulcentes": "demulcente",
    "demulsibilidades": "demulsibilidad",
    "demultiplexores": "demultiplexor",
    "demultiplicaciones": "demultiplicación",
    "deméritos": "demérito",
    "demócratas": "demócrata",
    "demógrafos": "demógrafo",
    "demótica": "demótico",
    "demóticas": "demótico",
    "demóticos": "demótico",
    "den": "dar",
    "denaria": "denario",
    "denarias": "denario",
    "denarios": "denario",
    "dendriformes": "dendriforme",
    "dendritas": "dendrita",
    "dendrocronologías": "dendrocronología",
    "dendrografías": "dendrografía",
    "dendrogramas": "dendrograma",
    "dendrográfica": "dendrográfico",
    "dendrográficas": "dendrográfico",
    "dendrográficos": "dendrográfico",
    "dendroidea": "dendroideo",
    "dendroideas": "dendroideo",
    "dendroideos": "dendroideo",
    "dendroides": "dendroide",
    "dendrologías": "dendrología",
    "dendrológica": "dendrológico",
    "dendrológicas": "dendrológico",
    "dendrológicos": "dendrológico",
    "dendrometrías": "dendrometría",
    "dendrométrica": "dendrométrico",
    "dendrométricas": "dendrométrico",
    "dendrométricos": "dendrométrico",
    "dendrítica": "dendrítico",
    "dendríticas": "dendrítico",
    "dendríticos": "dendrítico",
    "dendrómetros": "dendrómetro",
    "denegaba": "denegar",
    "denegabais": "denegar",
    "denegaban": "denegar",
    "denegabas": "denegar",
    "denegaciones": "denegación",
    "denegaciónes": "denegación",
    "denegad": "denegar",
    "denegada": "denegar",
    "denegadas": "denegar",
    "denegado": "denegar",
    "denegados": "denegar",
    "denegamos": "denegar",
    "denegando": "denegar",
    "denegara": "denegar",
    "denegarais": "denegar",
    "denegaran": "denegar",
    "denegaras": "denegar",
    "denegare": "denegar",
    "denegareis": "denegar",
    "denegaremos": "denegar",
    "denegaren": "denegar",
    "denegares": "denegar",
    "denegaron": "denegar",
    "denegará": "denegar",
    "denegarán": "denegar",
    "denegarás": "denegar",
    "denegaré": "denegar",
    "denegaréis": "denegar",
    "denegaría": "denegar",
    "denegaríais": "denegar",
    "denegaríamos": "denegar",
    "denegarían": "denegar",
    "denegarías": "denegar",
    "denegase": "denegar",
    "denegaseis": "denegar",
    "denegasen": "denegar",
    "denegases": "denegar",
    "denegaste": "denegar",
    "denegasteis": "denegar",
    "denegatoria": "denegatorio",
    "denegatorias": "denegatorio",
    "denegatorios": "denegatorio",
    "denegrece": "denegrecer",
    "denegreced": "denegrecer",
    "denegrecemos": "denegrecer",
    "denegrecen": "denegrecer",
    "denegreceremos": "denegrecer",
    "denegrecerá": "denegrecer",
    "denegrecerán": "denegrecer",
    "denegrecerás": "denegrecer",
    "denegreceré": "denegrecer",
    "denegreceréis": "denegrecer",
    "denegrecería": "denegrecer",
    "denegreceríais": "denegrecer",
    "denegreceríamos": "denegrecer",
    "denegrecerían": "denegrecer",
    "denegrecerías": "denegrecer",
    "denegreces": "denegrecer",
    "denegrecida": "denegrecer",
    "denegrecidas": "denegrecer",
    "denegrecido": "denegrecer",
    "denegrecidos": "denegrecer",
    "denegreciendo": "denegrecer",
    "denegreciera": "denegrecer",
    "denegrecierais": "denegrecer",
    "denegrecieran": "denegrecer",
    "denegrecieras": "denegrecer",
    "denegreciere": "denegrecer",
    "denegreciereis": "denegrecer",
    "denegrecieren": "denegrecer",
    "denegrecieres": "denegrecer",
    "denegrecieron": "denegrecer",
    "denegreciese": "denegrecer",
    "denegrecieseis": "denegrecer",
    "denegreciesen": "denegrecer",
    "denegrecieses": "denegrecer",
    "denegrecimientos": "denegrecimiento",
    "denegrecimos": "denegrecer",
    "denegreciste": "denegrecer",
    "denegrecisteis": "denegrecer",
    "denegreciéramos": "denegrecer",
    "denegreciéremos": "denegrecer",
    "denegreciésemos": "denegrecer",
    "denegreció": "denegrecer",
    "denegrecéis": "denegrecer",
    "denegrecí": "denegrecer",
    "denegrecía": "denegrecer",
    "denegrecíais": "denegrecer",
    "denegrecíamos": "denegrecer",
    "denegrecían": "denegrecer",
    "denegrecías": "denegrecer",
    "denegrezca": "denegrecer",
    "denegrezcamos": "denegrecer",
    "denegrezcan": "denegrecer",
    "denegrezcas": "denegrecer",
    "denegrezco": "denegrecer",
    "denegrezcáis": "denegrecer",
    "deneguemos": "denegar",
    "denegué": "denegar",
    "deneguéis": "denegar",
    "denegábamos": "denegar",
    "denegáis": "denegar",
    "denegáramos": "denegar",
    "denegáremos": "denegar",
    "denegásemos": "denegar",
    "denegó": "denegar",
    "dengosa": "dengoso",
    "dengosas": "dengoso",
    "dengosos": "dengoso",
    "denguea": "denguear",
    "dengueaba": "denguear",
    "dengueabais": "denguear",
    "dengueaban": "denguear",
    "dengueabas": "denguear",
    "denguead": "denguear",
    "dengueada": "denguear",
    "dengueadas": "denguear",
    "dengueado": "denguear",
    "dengueados": "denguear",
    "dengueamos": "denguear",
    "denguean": "denguear",
    "dengueando": "denguear",
    "dengueara": "denguear",
    "denguearais": "denguear",
    "denguearan": "denguear",
    "denguearas": "denguear",
    "dengueare": "denguear",
    "dengueareis": "denguear",
    "denguearemos": "denguear",
    "denguearen": "denguear",
    "dengueares": "denguear",
    "denguearon": "denguear",
    "dengueará": "denguear",
    "denguearán": "denguear",
    "denguearás": "denguear",
    "denguearé": "denguear",
    "denguearéis": "denguear",
    "denguearía": "denguear",
    "denguearíais": "denguear",
    "denguearíamos": "denguear",
    "denguearían": "denguear",
    "denguearías": "denguear",
    "dengueas": "denguear",
    "denguease": "denguear",
    "dengueaseis": "denguear",
    "dengueasen": "denguear",
    "dengueases": "denguear",
    "dengueaste": "denguear",
    "dengueasteis": "denguear",
    "denguee": "denguear",
    "dengueemos": "denguear",
    "dengueen": "denguear",
    "denguees": "denguear",
    "dengueo": "denguear",
    "denguera": "denguero",
    "dengueras": "denguero",
    "dengueros": "denguero",
    "dengues": "dengue",
    "dengueábamos": "denguear",
    "dengueáis": "denguear",
    "dengueáramos": "denguear",
    "dengueáremos": "denguear",
    "dengueásemos": "denguear",
    "dengueé": "denguear",
    "dengueéis": "denguear",
    "dengueó": "denguear",
    "deniega": "denegar",
    "deniegan": "denegar",
    "deniegas": "denegar",
    "deniego": "denegar",
    "deniegue": "denegar",
    "denieguen": "denegar",
    "deniegues": "denegar",
    "denigra": "denigrar",
    "denigraba": "denigrar",
    "denigrabais": "denigrar",
    "denigraban": "denigrar",
    "denigrabas": "denigrar",
    "denigraciones": "denigración",
    "denigrad": "denigrar",
    "denigrada": "denigrar",
    "denigradas": "denigrar",
    "denigrado": "denigrar",
    "denigradora": "denigrador",
    "denigradoras": "denigrador",
    "denigradores": "denigrador",
    "denigrados": "denigrar",
    "denigramos": "denigrar",
    "denigran": "denigrar",
    "denigrando": "denigrar",
    "denigrantes": "denigrante",
    "denigrara": "denigrar",
    "denigrarais": "denigrar",
    "denigraran": "denigrar",
    "denigraras": "denigrar",
    "denigrare": "denigrar",
    "denigrareis": "denigrar",
    "denigraremos": "denigrar",
    "denigraren": "denigrar",
    "denigrares": "denigrar",
    "denigraron": "denigrar",
    "denigrará": "denigrar",
    "denigrarán": "denigrar",
    "denigrarás": "denigrar",
    "denigraré": "denigrar",
    "denigraréis": "denigrar",
    "denigraría": "denigrar",
    "denigraríais": "denigrar",
    "denigraríamos": "denigrar",
    "denigrarían": "denigrar",
    "denigrarías": "denigrar",
    "denigras": "denigrar",
    "denigrase": "denigrar",
    "denigraseis": "denigrar",
    "denigrasen": "denigrar",
    "denigrases": "denigrar",
    "denigraste": "denigrar",
    "denigrasteis": "denigrar",
    "denigrativa": "denigrativo",
    "denigrativas": "denigrativo",
    "denigrativos": "denigrativo",
    "denigratoria": "denigratorio",
    "denigratorias": "denigratorio",
    "denigratorios": "denigratorio",
    "denigre": "denigrar",
    "denigremos": "denigrar",
    "denigren": "denigrar",
    "denigres": "denigrar",
    "denigro": "denigrar",
    "denigrábamos": "denigrar",
    "denigráis": "denigrar",
    "denigráramos": "denigrar",
    "denigráremos": "denigrar",
    "denigrásemos": "denigrar",
    "denigré": "denigrar",
    "denigréis": "denigrar",
    "denigró": "denigrar",
    "denodada": "denodado",
    "denodadas": "denodado",
    "denodados": "denodado",
    "denomina": "denominar",
    "denominaba": "denominar",
    "denominabais": "denominar",
    "denominaban": "denominar",
    "denominabas": "denominar",
    "denominaciones": "denominación",
    "denominad": "denominar",
    "denominada": "denominar",
    "denominadas": "denominar",
    "denominado": "denominar",
    "denominadora": "denominador",
    "denominadoras": "denominador",
    "denominadores": "denominador",
    "denominados": "denominar",
    "denominamos": "denominar",
    "denominan": "denominar",
    "denominando": "denominar",
    "denominara": "denominar",
    "denominarais": "denominar",
    "denominaran": "denominar",
    "denominaras": "denominar",
    "denominare": "denominar",
    "denominareis": "denominar",
    "denominaremos": "denominar",
    "denominaren": "denominar",
    "denominares": "denominar",
    "denominaron": "denominar",
    "denominará": "denominar",
    "denominarán": "denominar",
    "denominarás": "denominar",
    "denominaré": "denominar",
    "denominaréis": "denominar",
    "denominaría": "denominar",
    "denominaríais": "denominar",
    "denominaríamos": "denominar",
    "denominarían": "denominar",
    "denominarías": "denominar",
    "denominas": "denominar",
    "denominase": "denominar",
    "denominaseis": "denominar",
    "denominasen": "denominar",
    "denominases": "denominar",
    "denominaste": "denominar",
    "denominasteis": "denominar",
    "denominativa": "denominativo",
    "denominativas": "denominativo",
    "denominativos": "denominativo",
    "denomine": "denominar",
    "denominemos": "denominar",
    "denominen": "denominar",
    "denomines": "denominar",
    "denomino": "denominar",
    "denominábamos": "denominar",
    "denomináis": "denominar",
    "denomináramos": "denominar",
    "denomináremos": "denominar",
    "denominásemos": "denominar",
    "denominé": "denominar",
    "denominéis": "denominar",
    "denominó": "denominar",
    "denostaba": "denostar",
    "denostabais": "denostar",
    "denostaban": "denostar",
    "denostabas": "denostar",
    "denostad": "denostar",
    "denostada": "denostar",
    "denostadas": "denostar",
    "denostado": "denostar",
    "denostados": "denostar",
    "denostamos": "denostar",
    "denostando": "denostar",
    "denostara": "denostar",
    "denostarais": "denostar",
    "denostaran": "denostar",
    "denostaras": "denostar",
    "denostare": "denostar",
    "denostareis": "denostar",
    "denostaremos": "denostar",
    "denostaren": "denostar",
    "denostares": "denostar",
    "denostaron": "denostar",
    "denostará": "denostar",
    "denostarán": "denostar",
    "denostarás": "denostar",
    "denostaré": "denostar",
    "denostaréis": "denostar",
    "denostaría": "denostar",
    "denostaríais": "denostar",
    "denostaríamos": "denostar",
    "denostarían": "denostar",
    "denostarías": "denostar",
    "denostase": "denostar",
    "denostaseis": "denostar",
    "denostasen": "denostar",
    "denostases": "denostar",
    "denostaste": "denostar",
    "denostasteis": "denostar",
    "denostemos": "denostar",
    "denostábamos": "denostar",
    "denostáis": "denostar",
    "denostáramos": "denostar",
    "denostáremos": "denostar",
    "denostásemos": "denostar",
    "denosté": "denostar",
    "denostéis": "denostar",
    "denostó": "denostar",
    "denota": "denotar",
    "denotaba": "denotar",
    "denotabais": "denotar",
    "denotaban": "denotar",
    "denotabas": "denotar",
    "denotaciones": "denotación",
    "denotad": "denotar",
    "denotada": "denotar",
    "denotadas": "denotar",
    "denotado": "denotar",
    "denotados": "denotar",
    "denotamos": "denotar",
    "denotan": "denotar",
    "denotando": "denotar",
    "denotara": "denotar",
    "denotarais": "denotar",
    "denotaran": "denotar",
    "denotaras": "denotar",
    "denotare": "denotar",
    "denotareis": "denotar",
    "denotaremos": "denotar",
    "denotaren": "denotar",
    "denotares": "denotar",
    "denotaron": "denotar",
    "denotará": "denotar",
    "denotarán": "denotar",
    "denotarás": "denotar",
    "denotaré": "denotar",
    "denotaréis": "denotar",
    "denotaría": "denotar",
    "denotaríais": "denotar",
    "denotaríamos": "denotar",
    "denotarían": "denotar",
    "denotarías": "denotar",
    "denotas": "denotar",
    "denotase": "denotar",
    "denotaseis": "denotar",
    "denotasen": "denotar",
    "denotases": "denotar",
    "denotaste": "denotar",
    "denotasteis": "denotar",
    "denotativa": "denotativo",
    "denotativas": "denotativo",
    "denotativos": "denotativo",
    "denote": "denotar",
    "denotemos": "denotar",
    "denoten": "denotar",
    "denotes": "denotar",
    "denoto": "denotar",
    "denotábamos": "denotar",
    "denotáis": "denotar",
    "denotáramos": "denotar",
    "denotáremos": "denotar",
    "denotásemos": "denotar",
    "denoté": "denotar",
    "denotéis": "denotar",
    "denotó": "denotar",
    "densa": "denso",
    "densas": "denso",
    "densidades": "densidad",
    "densifica": "densificar",
    "densificaba": "densificar",
    "densificabais": "densificar",
    "densificaban": "densificar",
    "densificabas": "densificar",
    "densificaciones": "densificación",
    "densificad": "densificar",
    "densificada": "densificar",
    "densificadas": "densificar",
    "densificado": "densificar",
    "densificados": "densificar",
    "densificamos": "densificar",
    "densifican": "densificar",
    "densificando": "densificar",
    "densificara": "densificar",
    "densificarais": "densificar",
    "densificaran": "densificar",
    "densificaras": "densificar",
    "densificare": "densificar",
    "densificareis": "densificar",
    "densificaremos": "densificar",
    "densificaren": "densificar",
    "densificares": "densificar",
    "densificaron": "densificar",
    "densificará": "densificar",
    "densificarán": "densificar",
    "densificarás": "densificar",
    "densificaré": "densificar",
    "densificaréis": "densificar",
    "densificaría": "densificar",
    "densificaríais": "densificar",
    "densificaríamos": "densificar",
    "densificarían": "densificar",
    "densificarías": "densificar",
    "densificas": "densificar",
    "densificase": "densificar",
    "densificaseis": "densificar",
    "densificasen": "densificar",
    "densificases": "densificar",
    "densificaste": "densificar",
    "densificasteis": "densificar",
    "densifico": "densificar",
    "densificábamos": "densificar",
    "densificáis": "densificar",
    "densificáramos": "densificar",
    "densificáremos": "densificar",
    "densificásemos": "densificar",
    "densificó": "densificar",
    "densifique": "densificar",
    "densifiquemos": "densificar",
    "densifiquen": "densificar",
    "densifiques": "densificar",
    "densifiqué": "densificar",
    "densifiquéis": "densificar",
    "densimetrías": "densimetría",
    "densimétrica": "densimétrico",
    "densimétricas": "densimétrico",
    "densimétricos": "densimétrico",
    "densitometrías": "densitometría",
    "densitométrica": "densitométrico",
    "densitométricas": "densitométrico",
    "densitométricos": "densitométrico",
    "densitómetros": "densitómetro",
    "densos": "denso",
    "densímetros": "densímetro",
    "dentaba": "dentar",
    "dentabais": "dentar",
    "dentaban": "dentar",
    "dentabas": "dentar",
    "dentad": "dentar",
    "dentada": "dentar",
    "dentadas": "dentar",
    "dentado": "dentar",
    "dentados": "dentar",
    "dentaduras": "dentadura",
    "dentales": "dental",
    "dentalizaciones": "dentalización",
    "dentamos": "dentar",
    "dentando": "dentar",
    "dentara": "dentar",
    "dentarais": "dentar",
    "dentaran": "dentar",
    "dentaras": "dentar",
    "dentare": "dentar",
    "dentareis": "dentar",
    "dentaremos": "dentar",
    "dentaren": "dentar",
    "dentares": "dentar",
    "dentaria": "dentario",
    "dentarias": "dentario",
    "dentarios": "dentario",
    "dentaron": "dentar",
    "dentará": "dentar",
    "dentarán": "dentar",
    "dentarás": "dentar",
    "dentaré": "dentar",
    "dentaréis": "dentar",
    "dentaría": "dentar",
    "dentaríais": "dentar",
    "dentaríamos": "dentar",
    "dentarían": "dentar",
    "dentarías": "dentar",
    "dentase": "dentar",
    "dentaseis": "dentar",
    "dentasen": "dentar",
    "dentases": "dentar",
    "dentaste": "dentar",
    "dentasteis": "dentar",
    "dentejones": "dentejón",
    "dentella": "dentellar",
    "dentellaba": "dentellar",
    "dentellabais": "dentellar",
    "dentellaban": "dentellar",
    "dentellabas": "dentellar",
    "dentellad": "dentellar",
    "dentellada": "dentellar",
    "dentelladas": "dentellar",
    "dentellado": "dentellar",
    "dentellados": "dentellar",
    "dentellamos": "dentellar",
    "dentellan": "dentellar",
    "dentellando": "dentellar",
    "dentellara": "dentellar",
    "dentellarais": "dentellar",
    "dentellaran": "dentellar",
    "dentellaras": "dentellar",
    "dentellare": "dentellar",
    "dentellareis": "dentellar",
    "dentellaremos": "dentellar",
    "dentellaren": "dentellar",
    "dentellares": "dentellar",
    "dentellaron": "dentellar",
    "dentellará": "dentellar",
    "dentellarán": "dentellar",
    "dentellarás": "dentellar",
    "dentellaré": "dentellar",
    "dentellaréis": "dentellar",
    "dentellaría": "dentellar",
    "dentellaríais": "dentellar",
    "dentellaríamos": "dentellar",
    "dentellarían": "dentellar",
    "dentellarías": "dentellar",
    "dentellas": "dentellar",
    "dentellase": "dentellar",
    "dentellaseis": "dentellar",
    "dentellasen": "dentellar",
    "dentellases": "dentellar",
    "dentellaste": "dentellar",
    "dentellasteis": "dentellar",
    "dentelle": "dentellar",
    "dentellea": "dentellear",
    "dentelleaba": "dentellear",
    "dentelleabais": "dentellear",
    "dentelleaban": "dentellear",
    "dentelleabas": "dentellear",
    "dentellead": "dentellear",
    "dentelleada": "dentellear",
    "dentelleadas": "dentellear",
    "dentelleado": "dentellear",
    "dentelleados": "dentellear",
    "dentelleamos": "dentellear",
    "dentellean": "dentellear",
    "dentelleando": "dentellear",
    "dentelleara": "dentellear",
    "dentellearais": "dentellear",
    "dentellearan": "dentellear",
    "dentellearas": "dentellear",
    "dentelleare": "dentellear",
    "dentelleareis": "dentellear",
    "dentellearemos": "dentellear",
    "dentellearen": "dentellear",
    "dentelleares": "dentellear",
    "dentellearon": "dentellear",
    "dentelleará": "dentellear",
    "dentellearán": "dentellear",
    "dentellearás": "dentellear",
    "dentellearé": "dentellear",
    "dentellearéis": "dentellear",
    "dentellearía": "dentellear",
    "dentellearíais": "dentellear",
    "dentellearíamos": "dentellear",
    "dentellearían": "dentellear",
    "dentellearías": "dentellear",
    "dentelleas": "dentellear",
    "dentellease": "dentellear",
    "dentelleaseis": "dentellear",
    "dentelleasen": "dentellear",
    "dentelleases": "dentellear",
    "dentelleaste": "dentellear",
    "dentelleasteis": "dentellear",
    "dentellee": "dentellear",
    "dentelleemos": "dentellear",
    "dentelleen": "dentellear",
    "dentellees": "dentellear",
    "dentellemos": "dentellar",
    "dentellen": "dentellar",
    "dentelleo": "dentellear",
    "dentelles": "dentellar",
    "dentelleábamos": "dentellear",
    "dentelleáis": "dentellear",
    "dentelleáramos": "dentellear",
    "dentelleáremos": "dentellear",
    "dentelleásemos": "dentellear",
    "dentelleé": "dentellear",
    "dentelleéis": "dentellear",
    "dentelleó": "dentellear",
    "dentello": "dentellar",
    "dentellones": "dentellón",
    "dentellábamos": "dentellar",
    "dentelláis": "dentellar",
    "dentelláramos": "dentellar",
    "dentelláremos": "dentellar",
    "dentellásemos": "dentellar",
    "dentellé": "dentellar",
    "dentelléis": "dentellar",
    "dentelló": "dentellar",
    "dentemos": "dentar",
    "denteras": "dentera",
    "denticinas": "denticina",
    "denticiones": "dentición",
    "denticulada": "denticulado",
    "denticuladas": "denticulado",
    "denticulados": "denticulado",
    "denticulares": "denticular",
    "dentiformes": "dentiforme",
    "dentinales": "dentinal",
    "dentinas": "dentina",
    "dentinoclasias": "dentinoclasia",
    "dentinoclastos": "dentinoclasto",
    "dentinogingivales": "dentinogingival",
    "dentinoides": "dentinoide",
    "dentirrostra": "dentirrostro",
    "dentirrostras": "dentirrostro",
    "dentirrostros": "dentirrostro",
    "dentistas": "dentista",
    "dentivana": "dentivano",
    "dentivanas": "dentivano",
    "dentivanos": "dentivano",
    "dentoalveolares": "dentoalveolar",
    "dentofaciales": "dentofacial",
    "dentofobias": "dentofobia",
    "dentogenias": "dentogenia",
    "dentogingivales": "dentogingival",
    "dentoides": "dentoide",
    "dentomaxilofaciales": "dentomaxilofacial",
    "dentona": "dentón",
    "dentonas": "dentón",
    "dentones": "dentón",
    "dentoperióstica": "dentoperióstico",
    "dentoperiósticas": "dentoperióstico",
    "dentoperiósticos": "dentoperióstico",
    "dentuda": "dentudo",
    "dentudas": "dentudo",
    "dentudos": "dentudo",
    "dentábamos": "dentar",
    "dentáis": "dentar",
    "dentáramos": "dentar",
    "dentáremos": "dentar",
    "dentásemos": "dentar",
    "denté": "dentar",
    "dentéis": "dentar",
    "dentículos": "dentículo",
    "dentífrica": "dentífrico",
    "dentífricas": "dentífrico",
    "dentífricos": "dentífrico",
    "dentígera": "dentígero",
    "dentígeras": "dentígero",
    "dentígeros": "dentígero",
    "dentímetros": "dentímetro",
    "dentó": "dentar",
    "denuda": "denudar",
    "denudaba": "denudar",
    "denudabais": "denudar",
    "denudaban": "denudar",
    "denudabas": "denudar",
    "denudaciones": "denudación",
    "denudad": "denudar",
    "denudada": "denudar",
    "denudadas": "denudar",
    "denudado": "denudar",
    "denudados": "denudar",
    "denudamos": "denudar",
    "denudan": "denudar",
    "denudando": "denudar",
    "denudara": "denudar",
    "denudarais": "denudar",
    "denudaran": "denudar",
    "denudaras": "denudar",
    "denudare": "denudar",
    "denudareis": "denudar",
    "denudaremos": "denudar",
    "denudaren": "denudar",
    "denudares": "denudar",
    "denudaron": "denudar",
    "denudará": "denudar",
    "denudarán": "denudar",
    "denudarás": "denudar",
    "denudaré": "denudar",
    "denudaréis": "denudar",
    "denudaría": "denudar",
    "denudaríais": "denudar",
    "denudaríamos": "denudar",
    "denudarían": "denudar",
    "denudarías": "denudar",
    "denudas": "denudar",
    "denudase": "denudar",
    "denudaseis": "denudar",
    "denudasen": "denudar",
    "denudases": "denudar",
    "denudaste": "denudar",
    "denudasteis": "denudar",
    "denude": "denudar",
    "denudemos": "denudar",
    "denuden": "denudar",
    "denudes": "denudar",
    "denudo": "denudar",
    "denudábamos": "denudar",
    "denudáis": "denudar",
    "denudáramos": "denudar",
    "denudáremos": "denudar",
    "denudásemos": "denudar",
    "denudé": "denudar",
    "denudéis": "denudar",
    "denudó": "denudar",
    "denuedos": "denuedo",
    "denuesta": "denostar",
    "denuestan": "denostar",
    "denuestas": "denostar",
    "denueste": "denostar",
    "denuesten": "denostar",
    "denuestes": "denostar",
    "denuesto": "denostar",
    "denuestos": "denuesto",
    "denuncia": "denunciar",
    "denunciaba": "denunciar",
    "denunciabais": "denunciar",
    "denunciaban": "denunciar",
    "denunciabas": "denunciar",
    "denunciables": "denunciable",
    "denunciad": "denunciar",
    "denunciada": "denunciar",
    "denunciadas": "denunciar",
    "denunciado": "denunciar",
    "denunciadora": "denunciador",
    "denunciadoras": "denunciador",
    "denunciadores": "denunciador",
    "denunciados": "denunciar",
    "denunciamos": "denunciar",
    "denuncian": "denunciar",
    "denunciando": "denunciar",
    "denunciantes": "denunciante",
    "denunciara": "denunciar",
    "denunciarais": "denunciar",
    "denunciaran": "denunciar",
    "denunciaras": "denunciar",
    "denunciare": "denunciar",
    "denunciareis": "denunciar",
    "denunciaremos": "denunciar",
    "denunciaren": "denunciar",
    "denunciares": "denunciar",
    "denunciaron": "denunciar",
    "denunciará": "denunciar",
    "denunciarán": "denunciar",
    "denunciarás": "denunciar",
    "denunciaré": "denunciar",
    "denunciaréis": "denunciar",
    "denunciaría": "denunciar",
    "denunciaríais": "denunciar",
    "denunciaríamos": "denunciar",
    "denunciarían": "denunciar",
    "denunciarías": "denunciar",
    "denuncias": "denunciar",
    "denunciase": "denunciar",
    "denunciaseis": "denunciar",
    "denunciasen": "denunciar",
    "denunciases": "denunciar",
    "denunciaste": "denunciar",
    "denunciasteis": "denunciar",
    "denunciatoria": "denunciatorio",
    "denunciatorias": "denunciatorio",
    "denunciatorios": "denunciatorio",
    "denuncie": "denunciar",
    "denunciemos": "denunciar",
    "denuncien": "denunciar",
    "denuncies": "denunciar",
    "denuncio": "denunciar",
    "denuncios": "denuncio",
    "denunciábamos": "denunciar",
    "denunciáis": "denunciar",
    "denunciáramos": "denunciar",
    "denunciáremos": "denunciar",
    "denunciásemos": "denunciar",
    "denuncié": "denunciar",
    "denunciéis": "denunciar",
    "denunció": "denunciar",
    "deontologías": "deontología",
    "deontológica": "deontológico",
    "deontológicas": "deontológico",
    "deontológicos": "deontológico",
    "depara": "deparar",
    "deparaba": "deparar",
    "deparabais": "deparar",
    "deparaban": "deparar",
    "deparabas": "deparar",
    "deparad": "deparar",
    "deparada": "deparar",
    "deparadas": "deparar",
    "deparado": "deparar",
    "deparadora": "deparador",
    "deparadoras": "deparador",
    "deparadores": "deparador",
    "deparados": "deparar",
    "deparamos": "deparar",
    "deparan": "deparar",
    "deparando": "deparar",
    "deparara": "deparar",
    "depararais": "deparar",
    "depararan": "deparar",
    "depararas": "deparar",
    "deparare": "deparar",
    "deparareis": "deparar",
    "depararemos": "deparar",
    "depararen": "deparar",
    "deparares": "deparar",
    "depararon": "deparar",
    "deparará": "deparar",
    "depararán": "deparar",
    "depararás": "deparar",
    "depararé": "deparar",
    "depararéis": "deparar",
    "depararía": "deparar",
    "depararíais": "deparar",
    "depararíamos": "deparar",
    "depararían": "deparar",
    "depararías": "deparar",
    "deparas": "deparar",
    "deparase": "deparar",
    "deparaseis": "deparar",
    "deparasen": "deparar",
    "deparases": "deparar",
    "deparaste": "deparar",
    "deparasteis": "deparar",
    "depare": "deparar",
    "deparemos": "deparar",
    "deparen": "deparar",
    "depares": "deparar",
    "deparo": "deparar",
    "departa": "departir",
    "departamentales": "departamental",
    "departamentalizaciones": "departamentalización",
    "departamentos": "departamento",
    "departamos": "departir",
    "departan": "departir",
    "departas": "departir",
    "departe": "departir",
    "departen": "departir",
    "departes": "departir",
    "departid": "departir",
    "departida": "departir",
    "departidas": "departir",
    "departido": "departir",
    "departidora": "departidor",
    "departidoras": "departidor",
    "departidores": "departidor",
    "departidos": "departir",
    "departiendo": "departir",
    "departiera": "departir",
    "departierais": "departir",
    "departieran": "departir",
    "departieras": "departir",
    "departiere": "departir",
    "departiereis": "departir",
    "departieren": "departir",
    "departieres": "departir",
    "departieron": "departir",
    "departiese": "departir",
    "departieseis": "departir",
    "departiesen": "departir",
    "departieses": "departir",
    "departimos": "departir",
    "departiremos": "departir",
    "departirá": "departir",
    "departirán": "departir",
    "departirás": "departir",
    "departiré": "departir",
    "departiréis": "departir",
    "departiría": "departir",
    "departiríais": "departir",
    "departiríamos": "departir",
    "departirían": "departir",
    "departirías": "departir",
    "departiste": "departir",
    "departisteis": "departir",
    "departiéramos": "departir",
    "departiéremos": "departir",
    "departiésemos": "departir",
    "departió": "departir",
    "departo": "departir",
    "departáis": "departir",
    "departí": "departir",
    "departía": "departir",
    "departíais": "departir",
    "departíamos": "departir",
    "departían": "departir",
    "departías": "departir",
    "departís": "departir",
    "deparábamos": "deparar",
    "deparáis": "deparar",
    "deparáramos": "deparar",
    "deparáremos": "deparar",
    "deparásemos": "deparar",
    "deparé": "deparar",
    "deparéis": "deparar",
    "deparó": "deparar",
    "depaupera": "depauperar",
    "depauperaba": "depauperar",
    "depauperabais": "depauperar",
    "depauperaban": "depauperar",
    "depauperabas": "depauperar",
    "depauperaciones": "depauperación",
    "depauperad": "depauperar",
    "depauperada": "depauperar",
    "depauperadas": "depauperar",
    "depauperado": "depauperar",
    "depauperados": "depauperar",
    "depauperamos": "depauperar",
    "depauperan": "depauperar",
    "depauperando": "depauperar",
    "depauperara": "depauperar",
    "depauperarais": "depauperar",
    "depauperaran": "depauperar",
    "depauperaras": "depauperar",
    "depauperare": "depauperar",
    "depauperareis": "depauperar",
    "depauperaremos": "depauperar",
    "depauperaren": "depauperar",
    "depauperares": "depauperar",
    "depauperaron": "depauperar",
    "depauperará": "depauperar",
    "depauperarán": "depauperar",
    "depauperarás": "depauperar",
    "depauperaré": "depauperar",
    "depauperaréis": "depauperar",
    "depauperaría": "depauperar",
    "depauperaríais": "depauperar",
    "depauperaríamos": "depauperar",
    "depauperarían": "depauperar",
    "depauperarías": "depauperar",
    "depauperas": "depauperar",
    "depauperase": "depauperar",
    "depauperaseis": "depauperar",
    "depauperasen": "depauperar",
    "depauperases": "depauperar",
    "depauperaste": "depauperar",
    "depauperasteis": "depauperar",
    "depaupere": "depauperar",
    "depauperemos": "depauperar",
    "depauperen": "depauperar",
    "depauperes": "depauperar",
    "depaupero": "depauperar",
    "depauperábamos": "depauperar",
    "depauperáis": "depauperar",
    "depauperáramos": "depauperar",
    "depauperáremos": "depauperar",
    "depauperásemos": "depauperar",
    "depauperé": "depauperar",
    "depauperéis": "depauperar",
    "depauperó": "depauperar",
    "dependa": "depender",
    "dependamos": "depender",
    "dependan": "depender",
    "dependas": "depender",
    "depende": "depender",
    "depended": "depender",
    "dependemos": "depender",
    "dependen": "depender",
    "dependencias": "dependencia",
    "dependeremos": "depender",
    "dependerá": "depender",
    "dependerán": "depender",
    "dependerás": "depender",
    "dependeré": "depender",
    "dependeréis": "depender",
    "dependería": "depender",
    "dependeríais": "depender",
    "dependeríamos": "depender",
    "dependerían": "depender",
    "dependerías": "depender",
    "dependes": "depender",
    "dependida": "depender",
    "dependidas": "depender",
    "dependido": "depender",
    "dependidos": "depender",
    "dependiendo": "depender",
    "dependienta": "dependiente",
    "dependientas": "dependiente",
    "dependientes": "dependiente",
    "dependiera": "depender",
    "dependierais": "depender",
    "dependieran": "depender",
    "dependieras": "depender",
    "dependiere": "depender",
    "dependiereis": "depender",
    "dependieren": "depender",
    "dependieres": "depender",
    "dependieron": "depender",
    "dependiese": "depender",
    "dependieseis": "depender",
    "dependiesen": "depender",
    "dependieses": "depender",
    "dependimos": "depender",
    "dependiste": "depender",
    "dependisteis": "depender",
    "dependiéramos": "depender",
    "dependiéremos": "depender",
    "dependiésemos": "depender",
    "dependió": "depender",
    "dependo": "depender",
    "dependáis": "depender",
    "dependéis": "depender",
    "dependí": "depender",
    "dependía": "depender",
    "dependíais": "depender",
    "dependíamos": "depender",
    "dependían": "depender",
    "dependías": "depender",
    "depila": "depilar",
    "depilaba": "depilar",
    "depilabais": "depilar",
    "depilaban": "depilar",
    "depilabas": "depilar",
    "depilaciones": "depilación",
    "depilad": "depilar",
    "depilada": "depilar",
    "depiladas": "depilar",
    "depilado": "depilar",
    "depiladora": "depilador",
    "depiladoras": "depilador",
    "depiladores": "depilador",
    "depilados": "depilar",
    "depilamos": "depilar",
    "depilan": "depilar",
    "depilando": "depilar",
    "depilara": "depilar",
    "depilarais": "depilar",
    "depilaran": "depilar",
    "depilaras": "depilar",
    "depilare": "depilar",
    "depilareis": "depilar",
    "depilaremos": "depilar",
    "depilaren": "depilar",
    "depilares": "depilar",
    "depilaron": "depilar",
    "depilará": "depilar",
    "depilarán": "depilar",
    "depilarás": "depilar",
    "depilaré": "depilar",
    "depilaréis": "depilar",
    "depilaría": "depilar",
    "depilaríais": "depilar",
    "depilaríamos": "depilar",
    "depilarían": "depilar",
    "depilarías": "depilar",
    "depilas": "depilar",
    "depilase": "depilar",
    "depilaseis": "depilar",
    "depilasen": "depilar",
    "depilases": "depilar",
    "depilaste": "depilar",
    "depilasteis": "depilar",
    "depilatoria": "depilatorio",
    "depilatorias": "depilatorio",
    "depilatorios": "depilatorio",
    "depile": "depilar",
    "depilemos": "depilar",
    "depilen": "depilar",
    "depiles": "depilar",
    "depilo": "depilar",
    "depilábamos": "depilar",
    "depiláis": "depilar",
    "depiláramos": "depilar",
    "depiláremos": "depilar",
    "depilásemos": "depilar",
    "depilé": "depilar",
    "depiléis": "depilar",
    "depiló": "depilar",
    "depleciones": "depleción",
    "deplora": "deplorar",
    "deploraba": "deplorar",
    "deplorabais": "deplorar",
    "deploraban": "deplorar",
    "deplorabas": "deplorar",
    "deplorables": "deplorable",
    "deplorad": "deplorar",
    "deplorada": "deplorar",
    "deploradas": "deplorar",
    "deplorado": "deplorar",
    "deplorados": "deplorar",
    "deploramos": "deplorar",
    "deploran": "deplorar",
    "deplorando": "deplorar",
    "deplorara": "deplorar",
    "deplorarais": "deplorar",
    "deploraran": "deplorar",
    "deploraras": "deplorar",
    "deplorare": "deplorar",
    "deplorareis": "deplorar",
    "deploraremos": "deplorar",
    "deploraren": "deplorar",
    "deplorares": "deplorar",
    "deploraron": "deplorar",
    "deplorará": "deplorar",
    "deplorarán": "deplorar",
    "deplorarás": "deplorar",
    "deploraré": "deplorar",
    "deploraréis": "deplorar",
    "deploraría": "deplorar",
    "deploraríais": "deplorar",
    "deploraríamos": "deplorar",
    "deplorarían": "deplorar",
    "deplorarías": "deplorar",
    "deploras": "deplorar",
    "deplorase": "deplorar",
    "deploraseis": "deplorar",
    "deplorasen": "deplorar",
    "deplorases": "deplorar",
    "deploraste": "deplorar",
    "deplorasteis": "deplorar",
    "deplore": "deplorar",
    "deploremos": "deplorar",
    "deploren": "deplorar",
    "deplores": "deplorar",
    "deploro": "deplorar",
    "deplorábamos": "deplorar",
    "deploráis": "deplorar",
    "deploráramos": "deplorar",
    "deploráremos": "deplorar",
    "deplorásemos": "deplorar",
    "deploré": "deplorar",
    "deploréis": "deplorar",
    "deploró": "deplorar",
    "depondremos": "deponer",
    "depondrá": "deponer",
    "depondrán": "deponer",
    "depondrás": "deponer",
    "depondré": "deponer",
    "depondréis": "deponer",
    "depondría": "deponer",
    "depondríais": "deponer",
    "depondríamos": "deponer",
    "depondrían": "deponer",
    "depondrías": "deponer",
    "depone": "deponer",
    "deponed": "deponer",
    "deponemos": "deponer",
    "deponen": "deponer",
    "deponentes": "deponente",
    "depones": "deponer",
    "deponga": "deponer",
    "depongamos": "deponer",
    "depongan": "deponer",
    "depongas": "deponer",
    "depongo": "deponer",
    "depongáis": "deponer",
    "deponiendo": "deponer",
    "deponéis": "deponer",
    "deponía": "deponer",
    "deponíais": "deponer",
    "deponíamos": "deponer",
    "deponían": "deponer",
    "deponías": "deponer",
    "deporta": "deportar",
    "deportaba": "deportar",
    "deportabais": "deportar",
    "deportaban": "deportar",
    "deportabas": "deportar",
    "deportaciones": "deportación",
    "deportad": "deportar",
    "deportada": "deportar",
    "deportadas": "deportar",
    "deportado": "deportar",
    "deportados": "deportar",
    "deportamos": "deportar",
    "deportan": "deportar",
    "deportando": "deportar",
    "deportara": "deportar",
    "deportarais": "deportar",
    "deportaran": "deportar",
    "deportaras": "deportar",
    "deportare": "deportar",
    "deportareis": "deportar",
    "deportaremos": "deportar",
    "deportaren": "deportar",
    "deportares": "deportar",
    "deportaron": "deportar",
    "deportará": "deportar",
    "deportarán": "deportar",
    "deportarás": "deportar",
    "deportaré": "deportar",
    "deportaréis": "deportar",
    "deportaría": "deportar",
    "deportaríais": "deportar",
    "deportaríamos": "deportar",
    "deportarían": "deportar",
    "deportarías": "deportar",
    "deportas": "deportar",
    "deportase": "deportar",
    "deportaseis": "deportar",
    "deportasen": "deportar",
    "deportases": "deportar",
    "deportaste": "deportar",
    "deportasteis": "deportar",
    "deporte": "deportar",
    "deportemos": "deportar",
    "deporten": "deportar",
    "deportes": "deporte",
    "deportismos": "deportismo",
    "deportistas": "deportista",
    "deportiva": "deportivo",
    "deportivas": "deportivo",
    "deportividades": "deportividad",
    "deportivismos": "deportivismo",
    "deportivos": "deportivo",
    "deporto": "deportar",
    "deportábamos": "deportar",
    "deportáis": "deportar",
    "deportáramos": "deportar",
    "deportáremos": "deportar",
    "deportásemos": "deportar",
    "deporté": "deportar",
    "deportéis": "deportar",
    "deportó": "deportar",
    "deposiciones": "deposición",
    "deposita": "depositar",
    "depositaba": "depositar",
    "depositabais": "depositar",
    "depositaban": "depositar",
    "depositabas": "depositar",
    "depositad": "depositar",
    "depositada": "depositar",
    "depositadas": "depositar",
    "depositado": "depositar",
    "depositadora": "depositador",
    "depositadoras": "depositador",
    "depositadores": "depositador",
    "depositados": "depositar",
    "depositamos": "depositar",
    "depositan": "depositar",
    "depositando": "depositar",
    "depositantes": "depositante",
    "depositara": "depositar",
    "depositarais": "depositar",
    "depositaran": "depositar",
    "depositaras": "depositar",
    "depositare": "depositar",
    "depositareis": "depositar",
    "depositaremos": "depositar",
    "depositaren": "depositar",
    "depositares": "depositar",
    "depositaria": "depositario",
    "depositarias": "depositario",
    "depositarios": "depositario",
    "depositaron": "depositar",
    "depositará": "depositar",
    "depositarán": "depositar",
    "depositarás": "depositar",
    "depositaré": "depositar",
    "depositaréis": "depositar",
    "depositaría": "depositar",
    "depositaríais": "depositar",
    "depositaríamos": "depositar",
    "depositarían": "depositar",
    "depositarías": "depositaría",
    "depositas": "depositar",
    "depositase": "depositar",
    "depositaseis": "depositar",
    "depositasen": "depositar",
    "depositases": "depositar",
    "depositaste": "depositar",
    "depositasteis": "depositar",
    "deposite": "depositar",
    "depositemos": "depositar",
    "depositen": "depositar",
    "deposites": "depositar",
    "deposito": "depositar",
    "depositábamos": "depositar",
    "depositáis": "depositar",
    "depositáramos": "depositar",
    "depositáremos": "depositar",
    "depositásemos": "depositar",
    "deposité": "depositar",
    "depositéis": "depositar",
    "depositó": "depositar",
    "deprava": "depravar",
    "depravaba": "depravar",
    "depravabais": "depravar",
    "depravaban": "depravar",
    "depravabas": "depravar",
    "depravaciones": "depravación",
    "depravad": "depravar",
    "depravada": "depravar",
    "depravadas": "depravar",
    "depravado": "depravar",
    "depravadora": "depravador",
    "depravadoras": "depravador",
    "depravadores": "depravador",
    "depravados": "depravar",
    "depravamos": "depravar",
    "depravan": "depravar",
    "depravando": "depravar",
    "depravara": "depravar",
    "depravarais": "depravar",
    "depravaran": "depravar",
    "depravaras": "depravar",
    "depravare": "depravar",
    "depravareis": "depravar",
    "depravaremos": "depravar",
    "depravaren": "depravar",
    "depravares": "depravar",
    "depravaron": "depravar",
    "depravará": "depravar",
    "depravarán": "depravar",
    "depravarás": "depravar",
    "depravaré": "depravar",
    "depravaréis": "depravar",
    "depravaría": "depravar",
    "depravaríais": "depravar",
    "depravaríamos": "depravar",
    "depravarían": "depravar",
    "depravarías": "depravar",
    "depravas": "depravar",
    "depravase": "depravar",
    "depravaseis": "depravar",
    "depravasen": "depravar",
    "depravases": "depravar",
    "depravaste": "depravar",
    "depravasteis": "depravar",
    "deprave": "depravar",
    "depravemos": "depravar",
    "depraven": "depravar",
    "depraves": "depravar",
    "depravo": "depravar",
    "depravábamos": "depravar",
    "depraváis": "depravar",
    "depraváramos": "depravar",
    "depraváremos": "depravar",
    "depravásemos": "depravar",
    "depravé": "depravar",
    "depravéis": "depravar",
    "depravó": "depravar",
    "depreca": "deprecar",
    "deprecaba": "deprecar",
    "deprecabais": "deprecar",
    "deprecaban": "deprecar",
    "deprecabas": "deprecar",
    "deprecaciones": "deprecación",
    "deprecad": "deprecar",
    "deprecada": "deprecar",
    "deprecadas": "deprecar",
    "deprecado": "deprecar",
    "deprecados": "deprecar",
    "deprecamos": "deprecar",
    "deprecan": "deprecar",
    "deprecando": "deprecar",
    "deprecara": "deprecar",
    "deprecarais": "deprecar",
    "deprecaran": "deprecar",
    "deprecaras": "deprecar",
    "deprecare": "deprecar",
    "deprecareis": "deprecar",
    "deprecaremos": "deprecar",
    "deprecaren": "deprecar",
    "deprecares": "deprecar",
    "deprecaron": "deprecar",
    "deprecará": "deprecar",
    "deprecarán": "deprecar",
    "deprecarás": "deprecar",
    "deprecaré": "deprecar",
    "deprecaréis": "deprecar",
    "deprecaría": "deprecar",
    "deprecaríais": "deprecar",
    "deprecaríamos": "deprecar",
    "deprecarían": "deprecar",
    "deprecarías": "deprecar",
    "deprecas": "deprecar",
    "deprecase": "deprecar",
    "deprecaseis": "deprecar",
    "deprecasen": "deprecar",
    "deprecases": "deprecar",
    "deprecaste": "deprecar",
    "deprecasteis": "deprecar",
    "deprecativa": "deprecativo",
    "deprecativas": "deprecativo",
    "deprecativos": "deprecativo",
    "deprecatoria": "deprecatorio",
    "deprecatorias": "deprecatorio",
    "deprecatorios": "deprecatorio",
    "deprecia": "depreciar",
    "depreciaba": "depreciar",
    "depreciabais": "depreciar",
    "depreciaban": "depreciar",
    "depreciabas": "depreciar",
    "depreciaciones": "depreciación",
    "depreciad": "depreciar",
    "depreciada": "depreciar",
    "depreciadas": "depreciar",
    "depreciado": "depreciar",
    "depreciados": "depreciar",
    "depreciamos": "depreciar",
    "deprecian": "depreciar",
    "depreciando": "depreciar",
    "depreciara": "depreciar",
    "depreciarais": "depreciar",
    "depreciaran": "depreciar",
    "depreciaras": "depreciar",
    "depreciare": "depreciar",
    "depreciareis": "depreciar",
    "depreciaremos": "depreciar",
    "depreciaren": "depreciar",
    "depreciares": "depreciar",
    "depreciaron": "depreciar",
    "depreciará": "depreciar",
    "depreciarán": "depreciar",
    "depreciarás": "depreciar",
    "depreciaré": "depreciar",
    "depreciaréis": "depreciar",
    "depreciaría": "depreciar",
    "depreciaríais": "depreciar",
    "depreciaríamos": "depreciar",
    "depreciarían": "depreciar",
    "depreciarías": "depreciar",
    "deprecias": "depreciar",
    "depreciase": "depreciar",
    "depreciaseis": "depreciar",
    "depreciasen": "depreciar",
    "depreciases": "depreciar",
    "depreciaste": "depreciar",
    "depreciasteis": "depreciar",
    "deprecie": "depreciar",
    "depreciemos": "depreciar",
    "deprecien": "depreciar",
    "deprecies": "depreciar",
    "deprecio": "depreciar",
    "depreciábamos": "depreciar",
    "depreciáis": "depreciar",
    "depreciáramos": "depreciar",
    "depreciáremos": "depreciar",
    "depreciásemos": "depreciar",
    "deprecié": "depreciar",
    "depreciéis": "depreciar",
    "depreció": "depreciar",
    "depreco": "deprecar",
    "deprecábamos": "deprecar",
    "deprecáis": "deprecar",
    "deprecáramos": "deprecar",
    "deprecáremos": "deprecar",
    "deprecásemos": "deprecar",
    "deprecó": "deprecar",
    "depreda": "depredar",
    "depredaba": "depredar",
    "depredabais": "depredar",
    "depredaban": "depredar",
    "depredabas": "depredar",
    "depredaciones": "depredación",
    "depredad": "depredar",
    "depredada": "depredar",
    "depredadas": "depredar",
    "depredado": "depredar",
    "depredadora": "depredador",
    "depredadoras": "depredador",
    "depredadores": "depredador",
    "depredados": "depredar",
    "depredamos": "depredar",
    "depredan": "depredar",
    "depredando": "depredar",
    "depredara": "depredar",
    "depredarais": "depredar",
    "depredaran": "depredar",
    "depredaras": "depredar",
    "depredare": "depredar",
    "depredareis": "depredar",
    "depredaremos": "depredar",
    "depredaren": "depredar",
    "depredares": "depredar",
    "depredaron": "depredar",
    "depredará": "depredar",
    "depredarán": "depredar",
    "depredarás": "depredar",
    "depredaré": "depredar",
    "depredaréis": "depredar",
    "depredaría": "depredar",
    "depredaríais": "depredar",
    "depredaríamos": "depredar",
    "depredarían": "depredar",
    "depredarías": "depredar",
    "depredas": "depredar",
    "depredase": "depredar",
    "depredaseis": "depredar",
    "depredasen": "depredar",
    "depredases": "depredar",
    "depredaste": "depredar",
    "depredasteis": "depredar",
    "deprede": "depredar",
    "depredemos": "depredar",
    "depreden": "depredar",
    "depredes": "depredar",
    "depredo": "depredar",
    "depredábamos": "depredar",
    "depredáis": "depredar",
    "depredáramos": "depredar",
    "depredáremos": "depredar",
    "depredásemos": "depredar",
    "depredé": "depredar",
    "depredéis": "depredar",
    "depredó": "depredar",
    "depreque": "deprecar",
    "deprequemos": "deprecar",
    "deprequen": "deprecar",
    "depreques": "deprecar",
    "deprequé": "deprecar",
    "deprequéis": "deprecar",
    "depresiones": "depresión",
    "depresiva": "depresivo",
    "depresivas": "depresivo",
    "depresivos": "depresivo",
    "depresiómetros": "depresiómetro",
    "depresora": "depresor",
    "depresoras": "depresor",
    "depresores": "depresor",
    "deprima": "deprimir",
    "deprimamos": "deprimir",
    "depriman": "deprimir",
    "deprimas": "deprimir",
    "deprime": "deprimir",
    "deprimen": "deprimir",
    "deprimentes": "deprimente",
    "deprimes": "deprimir",
    "deprimid": "deprimir",
    "deprimida": "deprimir",
    "deprimidas": "deprimir",
    "deprimido": "deprimir",
    "deprimidos": "deprimir",
    "deprimiendo": "deprimir",
    "deprimiera": "deprimir",
    "deprimierais": "deprimir",
    "deprimieran": "deprimir",
    "deprimieras": "deprimir",
    "deprimiere": "deprimir",
    "deprimiereis": "deprimir",
    "deprimieren": "deprimir",
    "deprimieres": "deprimir",
    "deprimieron": "deprimir",
    "deprimiese": "deprimir",
    "deprimieseis": "deprimir",
    "deprimiesen": "deprimir",
    "deprimieses": "deprimir",
    "deprimimos": "deprimir",
    "deprimiremos": "deprimir",
    "deprimirá": "deprimir",
    "deprimirán": "deprimir",
    "deprimirás": "deprimir",
    "deprimiré": "deprimir",
    "deprimiréis": "deprimir",
    "deprimiría": "deprimir",
    "deprimiríais": "deprimir",
    "deprimiríamos": "deprimir",
    "deprimirían": "deprimir",
    "deprimirías": "deprimir",
    "deprimiste": "deprimir",
    "deprimisteis": "deprimir",
    "deprimiéramos": "deprimir",
    "deprimiéremos": "deprimir",
    "deprimiésemos": "deprimir",
    "deprimió": "deprimir",
    "deprimo": "deprimir",
    "deprimáis": "deprimir",
    "deprimí": "deprimir",
    "deprimía": "deprimir",
    "deprimíais": "deprimir",
    "deprimíamos": "deprimir",
    "deprimían": "deprimir",
    "deprimías": "deprimir",
    "deprimís": "deprimir",
    "deprimómetros": "deprimómetro",
    "depuesta": "deponer",
    "depuestas": "deponer",
    "depuesto": "deponer",
    "depuestos": "deponer",
    "depulpada": "depulpado",
    "depulpadas": "depulpado",
    "depulpados": "depulpado",
    "depura": "depurar",
    "depuraba": "depurar",
    "depurabais": "depurar",
    "depuraban": "depurar",
    "depurabas": "depurar",
    "depuraciones": "depuración",
    "depurad": "depurar",
    "depurada": "depurar",
    "depuradas": "depurar",
    "depurado": "depurar",
    "depuradora": "depurador",
    "depuradoras": "depuradora",
    "depuradores": "depurador",
    "depurados": "depurar",
    "depuramos": "depurar",
    "depuran": "depurar",
    "depurando": "depurar",
    "depurara": "depurar",
    "depurarais": "depurar",
    "depuraran": "depurar",
    "depuraras": "depurar",
    "depurare": "depurar",
    "depurareis": "depurar",
    "depuraremos": "depurar",
    "depuraren": "depurar",
    "depurares": "depurar",
    "depuraron": "depurar",
    "depurará": "depurar",
    "depurarán": "depurar",
    "depurarás": "depurar",
    "depuraré": "depurar",
    "depuraréis": "depurar",
    "depuraría": "depurar",
    "depuraríais": "depurar",
    "depuraríamos": "depurar",
    "depurarían": "depurar",
    "depurarías": "depurar",
    "depuras": "depurar",
    "depurase": "depurar",
    "depuraseis": "depurar",
    "depurasen": "depurar",
    "depurases": "depurar",
    "depuraste": "depurar",
    "depurasteis": "depurar",
    "depurativa": "depurativo",
    "depurativas": "depurativo",
    "depurativos": "depurativo",
    "depuratoria": "depuratorio",
    "depuratorias": "depuratorio",
    "depuratorios": "depuratorio",
    "depure": "depurar",
    "depuremos": "depurar",
    "depuren": "depurar",
    "depures": "depurar",
    "depuro": "depurar",
    "depurábamos": "depurar",
    "depuráis": "depurar",
    "depuráramos": "depurar",
    "depuráremos": "depurar",
    "depurásemos": "depurar",
    "depuré": "depurar",
    "depuréis": "depurar",
    "depuró": "depurar",
    "depuse": "deponer",
    "depusiera": "deponer",
    "depusierais": "deponer",
    "depusieran": "deponer",
    "depusieras": "deponer",
    "depusiere": "deponer",
    "depusiereis": "deponer",
    "depusieren": "deponer",
    "depusieres": "deponer",
    "depusieron": "deponer",
    "depusiese": "deponer",
    "depusieseis": "deponer",
    "depusiesen": "deponer",
    "depusieses": "deponer",
    "depusimos": "deponer",
    "depusiste": "deponer",
    "depusisteis": "deponer",
    "depusiéramos": "deponer",
    "depusiéremos": "deponer",
    "depusiésemos": "deponer",
    "depuso": "deponer",
    "depón": "deponer",
    "depósitos": "depósito",
    "derbies": "derby",
    "derbis": "derbi",
    "derbys": "derby",
    "derecha": "derecho",
    "derechas": "derecho",
    "derechazos": "derechazo",
    "derecheras": "derechera",
    "derechistas": "derechista",
    "derechohabientes": "derechohabiente",
    "derechos": "derecho",
    "derechuras": "derechura",
    "deriva": "derivar",
    "derivaba": "derivar",
    "derivabais": "derivar",
    "derivaban": "derivar",
    "derivabas": "derivar",
    "derivables": "derivable",
    "derivaciones": "derivación",
    "derivad": "derivar",
    "derivada": "derivar",
    "derivadas": "derivar",
    "derivado": "derivar",
    "derivadora": "derivador",
    "derivadoras": "derivador",
    "derivadores": "derivador",
    "derivados": "derivar",
    "derivamos": "derivar",
    "derivan": "derivar",
    "derivando": "derivar",
    "derivara": "derivar",
    "derivarais": "derivar",
    "derivaran": "derivar",
    "derivaras": "derivar",
    "derivare": "derivar",
    "derivareis": "derivar",
    "derivaremos": "derivar",
    "derivaren": "derivar",
    "derivares": "derivar",
    "derivaron": "derivar",
    "derivará": "derivar",
    "derivarán": "derivar",
    "derivarás": "derivar",
    "derivaré": "derivar",
    "derivaréis": "derivar",
    "derivaría": "derivar",
    "derivaríais": "derivar",
    "derivaríamos": "derivar",
    "derivarían": "derivar",
    "derivarías": "derivar",
    "derivas": "derivar",
    "derivase": "derivar",
    "derivaseis": "derivar",
    "derivasen": "derivar",
    "derivases": "derivar",
    "derivaste": "derivar",
    "derivasteis": "derivar",
    "derivativa": "derivativo",
    "derivativas": "derivativo",
    "derivativos": "derivativo",
    "derive": "derivar",
    "derivemos": "derivar",
    "deriven": "derivar",
    "derives": "derivar",
    "derivo": "derivar",
    "derivos": "derivo",
    "derivábamos": "derivar",
    "deriváis": "derivar",
    "deriváramos": "derivar",
    "deriváremos": "derivar",
    "derivásemos": "derivar",
    "derivé": "derivar",
    "derivéis": "derivar",
    "derivímetros": "derivímetro",
    "derivó": "derivar",
    "dermalgias": "dermalgia",
    "dermatologías": "dermatología",
    "dermatológica": "dermatológico",
    "dermatológicas": "dermatológico",
    "dermatológicos": "dermatológico",
    "dermatosomas": "dermatosoma",
    "dermatóloga": "dermatólogo",
    "dermatólogas": "dermatólogo",
    "dermatólogos": "dermatólogo",
    "dermises": "dermis",
    "dermoblastos": "dermoblasto",
    "dermápteros": "dermáptero",
    "dernys": "derny",
    "deroga": "derogar",
    "derogaba": "derogar",
    "derogabais": "derogar",
    "derogaban": "derogar",
    "derogabas": "derogar",
    "derogaciones": "derogación",
    "derogad": "derogar",
    "derogada": "derogar",
    "derogadas": "derogar",
    "derogado": "derogar",
    "derogadora": "derogador",
    "derogadoras": "derogador",
    "derogadores": "derogador",
    "derogados": "derogar",
    "derogamos": "derogar",
    "derogan": "derogar",
    "derogando": "derogar",
    "derogara": "derogar",
    "derogarais": "derogar",
    "derogaran": "derogar",
    "derogaras": "derogar",
    "derogare": "derogar",
    "derogareis": "derogar",
    "derogaremos": "derogar",
    "derogaren": "derogar",
    "derogares": "derogar",
    "derogaron": "derogar",
    "derogará": "derogar",
    "derogarán": "derogar",
    "derogarás": "derogar",
    "derogaré": "derogar",
    "derogaréis": "derogar",
    "derogaría": "derogar",
    "derogaríais": "derogar",
    "derogaríamos": "derogar",
    "derogarían": "derogar",
    "derogarías": "derogar",
    "derogas": "derogar",
    "derogase": "derogar",
    "derogaseis": "derogar",
    "derogasen": "derogar",
    "derogases": "derogar",
    "derogaste": "derogar",
    "derogasteis": "derogar",
    "derogatoria": "derogatorio",
    "derogatorias": "derogatorio",
    "derogatorios": "derogatorio",
    "derogo": "derogar",
    "derogue": "derogar",
    "deroguemos": "derogar",
    "deroguen": "derogar",
    "derogues": "derogar",
    "derogué": "derogar",
    "deroguéis": "derogar",
    "derogábamos": "derogar",
    "derogáis": "derogar",
    "derogáramos": "derogar",
    "derogáremos": "derogar",
    "derogásemos": "derogar",
    "derogó": "derogar",
    "derrabaduras": "derrabadura",
    "derrama": "derramar",
    "derramaba": "derramar",
    "derramabais": "derramar",
    "derramaban": "derramar",
    "derramabas": "derramar",
    "derramad": "derramar",
    "derramada": "derramar",
    "derramadas": "derramar",
    "derramaderos": "derramadero",
    "derramado": "derramar",
    "derramadora": "derramador",
    "derramadoras": "derramador",
    "derramadores": "derramador",
    "derramados": "derramar",
    "derramamientos": "derramamiento",
    "derramamos": "derramar",
    "derraman": "derramar",
    "derramando": "derramar",
    "derramara": "derramar",
    "derramarais": "derramar",
    "derramaran": "derramar",
    "derramaras": "derramar",
    "derramare": "derramar",
    "derramareis": "derramar",
    "derramaremos": "derramar",
    "derramaren": "derramar",
    "derramares": "derramar",
    "derramaron": "derramar",
    "derramará": "derramar",
    "derramarán": "derramar",
    "derramarás": "derramar",
    "derramaré": "derramar",
    "derramaréis": "derramar",
    "derramaría": "derramar",
    "derramaríais": "derramar",
    "derramaríamos": "derramar",
    "derramarían": "derramar",
    "derramarías": "derramar",
    "derramas": "derramar",
    "derramase": "derramar",
    "derramaseis": "derramar",
    "derramasen": "derramar",
    "derramases": "derramar",
    "derramaste": "derramar",
    "derramasteis": "derramar",
    "derrame": "derramar",
    "derramemos": "derramar",
    "derramen": "derramar",
    "derrames": "derrame",
    "derramo": "derramar",
    "derramábamos": "derramar",
    "derramáis": "derramar",
    "derramáramos": "derramar",
    "derramáremos": "derramar",
    "derramásemos": "derramar",
    "derramé": "derramar",
    "derraméis": "derramar",
    "derramó": "derramar",
    "derrapa": "derrapar",
    "derrapaba": "derrapar",
    "derrapabais": "derrapar",
    "derrapaban": "derrapar",
    "derrapabas": "derrapar",
    "derrapad": "derrapar",
    "derrapada": "derrapar",
    "derrapadas": "derrapar",
    "derrapado": "derrapar",
    "derrapados": "derrapar",
    "derrapajes": "derrapaje",
    "derrapamos": "derrapar",
    "derrapan": "derrapar",
    "derrapando": "derrapar",
    "derrapara": "derrapar",
    "derraparais": "derrapar",
    "derraparan": "derrapar",
    "derraparas": "derrapar",
    "derrapare": "derrapar",
    "derrapareis": "derrapar",
    "derraparemos": "derrapar",
    "derraparen": "derrapar",
    "derrapares": "derrapar",
    "derraparon": "derrapar",
    "derrapará": "derrapar",
    "derraparán": "derrapar",
    "derraparás": "derrapar",
    "derraparé": "derrapar",
    "derraparéis": "derrapar",
    "derraparía": "derrapar",
    "derraparíais": "derrapar",
    "derraparíamos": "derrapar",
    "derraparían": "derrapar",
    "derraparías": "derrapar",
    "derrapas": "derrapar",
    "derrapase": "derrapar",
    "derrapaseis": "derrapar",
    "derrapasen": "derrapar",
    "derrapases": "derrapar",
    "derrapaste": "derrapar",
    "derrapasteis": "derrapar",
    "derrape": "derrapar",
    "derrapemos": "derrapar",
    "derrapen": "derrapar",
    "derrapes": "derrape",
    "derrapo": "derrapar",
    "derrapábamos": "derrapar",
    "derrapáis": "derrapar",
    "derrapáramos": "derrapar",
    "derrapáremos": "derrapar",
    "derrapásemos": "derrapar",
    "derrapé": "derrapar",
    "derrapéis": "derrapar",
    "derrapó": "derrapar",
    "derredores": "derredor",
    "derreferenciaciones": "derreferenciación",
    "derrelicta": "derrelicto",
    "derrelictas": "derrelicto",
    "derrelictos": "derrelicto",
    "derrelinca": "derrelinquir",
    "derrelincamos": "derrelinquir",
    "derrelincan": "derrelinquir",
    "derrelincas": "derrelinquir",
    "derrelinco": "derrelinquir",
    "derrelincáis": "derrelinquir",
    "derrelinque": "derrelinquir",
    "derrelinquen": "derrelinquir",
    "derrelinques": "derrelinquir",
    "derrelinquid": "derrelinquir",
    "derrelinquida": "derrelinquir",
    "derrelinquidas": "derrelinquir",
    "derrelinquido": "derrelinquir",
    "derrelinquidos": "derrelinquir",
    "derrelinquiendo": "derrelinquir",
    "derrelinquiera": "derrelinquir",
    "derrelinquierais": "derrelinquir",
    "derrelinquieran": "derrelinquir",
    "derrelinquieras": "derrelinquir",
    "derrelinquiere": "derrelinquir",
    "derrelinquiereis": "derrelinquir",
    "derrelinquieren": "derrelinquir",
    "derrelinquieres": "derrelinquir",
    "derrelinquieron": "derrelinquir",
    "derrelinquiese": "derrelinquir",
    "derrelinquieseis": "derrelinquir",
    "derrelinquiesen": "derrelinquir",
    "derrelinquieses": "derrelinquir",
    "derrelinquimos": "derrelinquir",
    "derrelinquiremos": "derrelinquir",
    "derrelinquirá": "derrelinquir",
    "derrelinquirán": "derrelinquir",
    "derrelinquirás": "derrelinquir",
    "derrelinquiré": "derrelinquir",
    "derrelinquiréis": "derrelinquir",
    "derrelinquiría": "derrelinquir",
    "derrelinquiríais": "derrelinquir",
    "derrelinquiríamos": "derrelinquir",
    "derrelinquirían": "derrelinquir",
    "derrelinquirías": "derrelinquir",
    "derrelinquiste": "derrelinquir",
    "derrelinquisteis": "derrelinquir",
    "derrelinquiéramos": "derrelinquir",
    "derrelinquiéremos": "derrelinquir",
    "derrelinquiésemos": "derrelinquir",
    "derrelinquió": "derrelinquir",
    "derrelinquí": "derrelinquir",
    "derrelinquía": "derrelinquir",
    "derrelinquíais": "derrelinquir",
    "derrelinquíamos": "derrelinquir",
    "derrelinquían": "derrelinquir",
    "derrelinquías": "derrelinquir",
    "derrelinquís": "derrelinquir",
    "derrengaba": "derrengar",
    "derrengabais": "derrengar",
    "derrengaban": "derrengar",
    "derrengabas": "derrengar",
    "derrengad": "derrengar",
    "derrengada": "derrengar",
    "derrengadas": "derrengar",
    "derrengado": "derrengar",
    "derrengados": "derrengar",
    "derrengaduras": "derrengadura",
    "derrengamos": "derrengar",
    "derrengando": "derrengar",
    "derrengara": "derrengar",
    "derrengarais": "derrengar",
    "derrengaran": "derrengar",
    "derrengaras": "derrengar",
    "derrengare": "derrengar",
    "derrengareis": "derrengar",
    "derrengaremos": "derrengar",
    "derrengaren": "derrengar",
    "derrengares": "derrengar",
    "derrengaron": "derrengar",
    "derrengará": "derrengar",
    "derrengarán": "derrengar",
    "derrengarás": "derrengar",
    "derrengaré": "derrengar",
    "derrengaréis": "derrengar",
    "derrengaría": "derrengar",
    "derrengaríais": "derrengar",
    "derrengaríamos": "derrengar",
    "derrengarían": "derrengar",
    "derrengarías": "derrengar",
    "derrengase": "derrengar",
    "derrengaseis": "derrengar",
    "derrengasen": "derrengar",
    "derrengases": "derrengar",
    "derrengaste": "derrengar",
    "derrengasteis": "derrengar",
    "derrenguemos": "derrengar",
    "derrengué": "derrengar",
    "derrenguéis": "derrengar",
    "derrengábamos": "derrengar",
    "derrengáis": "derrengar",
    "derrengáramos": "derrengar",
    "derrengáremos": "derrengar",
    "derrengásemos": "derrengar",
    "derrengó": "derrengar",
    "derreniegos": "derreniego",
    "derretid": "derretir",
    "derretida": "derretir",
    "derretidas": "derretir",
    "derretido": "derretir",
    "derretidos": "derretir",
    "derretimientos": "derretimiento",
    "derretimos": "derretir",
    "derretiremos": "derretir",
    "derretirá": "derretir",
    "derretirán": "derretir",
    "derretirás": "derretir",
    "derretiré": "derretir",
    "derretiréis": "derretir",
    "derretiría": "derretir",
    "derretiríais": "derretir",
    "derretiríamos": "derretir",
    "derretirían": "derretir",
    "derretirías": "derretir",
    "derretiste": "derretir",
    "derretisteis": "derretir",
    "derretí": "derretir",
    "derretía": "derretir",
    "derretíais": "derretir",
    "derretíamos": "derretir",
    "derretían": "derretir",
    "derretías": "derretir",
    "derretís": "derretir",
    "derriba": "derribar",
    "derribaba": "derribar",
    "derribabais": "derribar",
    "derribaban": "derribar",
    "derribabas": "derribar",
    "derribad": "derribar",
    "derribada": "derribar",
    "derribadas": "derribar",
    "derribado": "derribar",
    "derribados": "derribar",
    "derribamos": "derribar",
    "derriban": "derribar",
    "derribando": "derribar",
    "derribara": "derribar",
    "derribarais": "derribar",
    "derribaran": "derribar",
    "derribaras": "derribar",
    "derribare": "derribar",
    "derribareis": "derribar",
    "derribaremos": "derribar",
    "derribaren": "derribar",
    "derribares": "derribar",
    "derribaron": "derribar",
    "derribará": "derribar",
    "derribarán": "derribar",
    "derribarás": "derribar",
    "derribaré": "derribar",
    "derribaréis": "derribar",
    "derribaría": "derribar",
    "derribaríais": "derribar",
    "derribaríamos": "derribar",
    "derribarían": "derribar",
    "derribarías": "derribar",
    "derribas": "derribar",
    "derribase": "derribar",
    "derribaseis": "derribar",
    "derribasen": "derribar",
    "derribases": "derribar",
    "derribaste": "derribar",
    "derribasteis": "derribar",
    "derribe": "derribar",
    "derribemos": "derribar",
    "derriben": "derribar",
    "derribes": "derribar",
    "derribo": "derribar",
    "derribos": "derribo",
    "derribábamos": "derribar",
    "derribáis": "derribar",
    "derribáramos": "derribar",
    "derribáremos": "derribar",
    "derribásemos": "derribar",
    "derribé": "derribar",
    "derribéis": "derribar",
    "derribó": "derribar",
    "derrickes": "derrick",
    "derricks": "derrick",
    "derrienga": "derrengar",
    "derriengan": "derrengar",
    "derriengas": "derrengar",
    "derriengo": "derrengar",
    "derriengue": "derrengar",
    "derrienguen": "derrengar",
    "derriengues": "derrengar",
    "derrita": "derretir",
    "derritamos": "derretir",
    "derritan": "derretir",
    "derritas": "derretir",
    "derrite": "derretir",
    "derriten": "derretir",
    "derrites": "derretir",
    "derritiendo": "derretir",
    "derritiera": "derretir",
    "derritierais": "derretir",
    "derritieran": "derretir",
    "derritieras": "derretir",
    "derritiere": "derretir",
    "derritiereis": "derretir",
    "derritieren": "derretir",
    "derritieres": "derretir",
    "derritieron": "derretir",
    "derritiese": "derretir",
    "derritieseis": "derretir",
    "derritiesen": "derretir",
    "derritieses": "derretir",
    "derritiéramos": "derretir",
    "derritiéremos": "derretir",
    "derritiésemos": "derretir",
    "derritió": "derretir",
    "derrito": "derretir",
    "derritáis": "derretir",
    "derroca": "derrocar",
    "derrocaba": "derrocar",
    "derrocabais": "derrocar",
    "derrocaban": "derrocar",
    "derrocabas": "derrocar",
    "derrocad": "derrocar",
    "derrocada": "derrocar",
    "derrocadas": "derrocar",
    "derrocaderos": "derrocadero",
    "derrocado": "derrocar",
    "derrocados": "derrocar",
    "derrocamientos": "derrocamiento",
    "derrocamos": "derrocar",
    "derrocan": "derrocar",
    "derrocando": "derrocar",
    "derrocara": "derrocar",
    "derrocarais": "derrocar",
    "derrocaran": "derrocar",
    "derrocaras": "derrocar",
    "derrocare": "derrocar",
    "derrocareis": "derrocar",
    "derrocaremos": "derrocar",
    "derrocaren": "derrocar",
    "derrocares": "derrocar",
    "derrocaron": "derrocar",
    "derrocará": "derrocar",
    "derrocarán": "derrocar",
    "derrocarás": "derrocar",
    "derrocaré": "derrocar",
    "derrocaréis": "derrocar",
    "derrocaría": "derrocar",
    "derrocaríais": "derrocar",
    "derrocaríamos": "derrocar",
    "derrocarían": "derrocar",
    "derrocarías": "derrocar",
    "derrocas": "derrocar",
    "derrocase": "derrocar",
    "derrocaseis": "derrocar",
    "derrocasen": "derrocar",
    "derrocases": "derrocar",
    "derrocaste": "derrocar",
    "derrocasteis": "derrocar",
    "derrocha": "derrochar",
    "derrochaba": "derrochar",
    "derrochabais": "derrochar",
    "derrochaban": "derrochar",
    "derrochabas": "derrochar",
    "derrochad": "derrochar",
    "derrochada": "derrochar",
    "derrochadas": "derrochar",
    "derrochado": "derrochar",
    "derrochadora": "derrochador",
    "derrochadoras": "derrochador",
    "derrochadores": "derrochador",
    "derrochados": "derrochar",
    "derrochamos": "derrochar",
    "derrochan": "derrochar",
    "derrochando": "derrochar",
    "derrochara": "derrochar",
    "derrocharais": "derrochar",
    "derrocharan": "derrochar",
    "derrocharas": "derrochar",
    "derrochare": "derrochar",
    "derrochareis": "derrochar",
    "derrocharemos": "derrochar",
    "derrocharen": "derrochar",
    "derrochares": "derrochar",
    "derrocharon": "derrochar",
    "derrochará": "derrochar",
    "derrocharán": "derrochar",
    "derrocharás": "derrochar",
    "derrocharé": "derrochar",
    "derrocharéis": "derrochar",
    "derrocharía": "derrochar",
    "derrocharíais": "derrochar",
    "derrocharíamos": "derrochar",
    "derrocharían": "derrochar",
    "derrocharías": "derrochar",
    "derrochas": "derrochar",
    "derrochase": "derrochar",
    "derrochaseis": "derrochar",
    "derrochasen": "derrochar",
    "derrochases": "derrochar",
    "derrochaste": "derrochar",
    "derrochasteis": "derrochar",
    "derroche": "derrochar",
    "derrochemos": "derrochar",
    "derrochen": "derrochar",
    "derroches": "derroche",
    "derrocho": "derrochar",
    "derrochábamos": "derrochar",
    "derrocháis": "derrochar",
    "derrocháramos": "derrochar",
    "derrocháremos": "derrochar",
    "derrochásemos": "derrochar",
    "derroché": "derrochar",
    "derrochéis": "derrochar",
    "derrochó": "derrochar",
    "derroco": "derrocar",
    "derrocábamos": "derrocar",
    "derrocáis": "derrocar",
    "derrocáramos": "derrocar",
    "derrocáremos": "derrocar",
    "derrocásemos": "derrocar",
    "derrocó": "derrocar",
    "derroque": "derrocar",
    "derroquemos": "derrocar",
    "derroquen": "derrocar",
    "derroques": "derrocar",
    "derroqué": "derrocar",
    "derroquéis": "derrocar",
    "derrota": "derrotar",
    "derrotaba": "derrotar",
    "derrotabais": "derrotar",
    "derrotaban": "derrotar",
    "derrotabas": "derrotar",
    "derrotad": "derrotar",
    "derrotada": "derrotar",
    "derrotadas": "derrotar",
    "derrotado": "derrotar",
    "derrotados": "derrotar",
    "derrotamos": "derrotar",
    "derrotan": "derrotar",
    "derrotando": "derrotar",
    "derrotara": "derrotar",
    "derrotarais": "derrotar",
    "derrotaran": "derrotar",
    "derrotaras": "derrotar",
    "derrotare": "derrotar",
    "derrotareis": "derrotar",
    "derrotaremos": "derrotar",
    "derrotaren": "derrotar",
    "derrotares": "derrotar",
    "derrotaron": "derrotar",
    "derrotará": "derrotar",
    "derrotarán": "derrotar",
    "derrotarás": "derrotar",
    "derrotaré": "derrotar",
    "derrotaréis": "derrotar",
    "derrotaría": "derrotar",
    "derrotaríais": "derrotar",
    "derrotaríamos": "derrotar",
    "derrotarían": "derrotar",
    "derrotarías": "derrotar",
    "derrotas": "derrotar",
    "derrotase": "derrotar",
    "derrotaseis": "derrotar",
    "derrotasen": "derrotar",
    "derrotases": "derrotar",
    "derrotaste": "derrotar",
    "derrotasteis": "derrotar",
    "derrote": "derrotar",
    "derrotemos": "derrotar",
    "derroten": "derrotar",
    "derroteros": "derrotero",
    "derrotes": "derrote",
    "derrotismos": "derrotismo",
    "derrotistas": "derrotista",
    "derroto": "derrotar",
    "derrotábamos": "derrotar",
    "derrotáis": "derrotar",
    "derrotáramos": "derrotar",
    "derrotáremos": "derrotar",
    "derrotásemos": "derrotar",
    "derroté": "derrotar",
    "derrotéis": "derrotar",
    "derrotó": "derrotar",
    "derrubia": "derrubiar",
    "derrubiaba": "derrubiar",
    "derrubiabais": "derrubiar",
    "derrubiaban": "derrubiar",
    "derrubiabas": "derrubiar",
    "derrubiad": "derrubiar",
    "derrubiada": "derrubiar",
    "derrubiadas": "derrubiar",
    "derrubiado": "derrubiar",
    "derrubiados": "derrubiar",
    "derrubiamos": "derrubiar",
    "derrubian": "derrubiar",
    "derrubiando": "derrubiar",
    "derrubiara": "derrubiar",
    "derrubiarais": "derrubiar",
    "derrubiaran": "derrubiar",
    "derrubiaras": "derrubiar",
    "derrubiare": "derrubiar",
    "derrubiareis": "derrubiar",
    "derrubiaremos": "derrubiar",
    "derrubiaren": "derrubiar",
    "derrubiares": "derrubiar",
    "derrubiaron": "derrubiar",
    "derrubiará": "derrubiar",
    "derrubiarán": "derrubiar",
    "derrubiarás": "derrubiar",
    "derrubiaré": "derrubiar",
    "derrubiaréis": "derrubiar",
    "derrubiaría": "derrubiar",
    "derrubiaríais": "derrubiar",
    "derrubiaríamos": "derrubiar",
    "derrubiarían": "derrubiar",
    "derrubiarías": "derrubiar",
    "derrubias": "derrubiar",
    "derrubiase": "derrubiar",
    "derrubiaseis": "derrubiar",
    "derrubiasen": "derrubiar",
    "derrubiases": "derrubiar",
    "derrubiaste": "derrubiar",
    "derrubiasteis": "derrubiar",
    "derrubie": "derrubiar",
    "derrubiemos": "derrubiar",
    "derrubien": "derrubiar",
    "derrubies": "derrubiar",
    "derrubio": "derrubiar",
    "derrubios": "derrubio",
    "derrubiábamos": "derrubiar",
    "derrubiáis": "derrubiar",
    "derrubiáramos": "derrubiar",
    "derrubiáremos": "derrubiar",
    "derrubiásemos": "derrubiar",
    "derrubié": "derrubiar",
    "derrubiéis": "derrubiar",
    "derrubió": "derrubiar",
    "derruid": "derruir",
    "derruida": "derruir",
    "derruidas": "derruir",
    "derruido": "derruir",
    "derruidos": "derruir",
    "derruimos": "derruir",
    "derruiremos": "derruir",
    "derruirá": "derruir",
    "derruirán": "derruir",
    "derruirás": "derruir",
    "derruiré": "derruir",
    "derruiréis": "derruir",
    "derruiría": "derruir",
    "derruiríais": "derruir",
    "derruiríamos": "derruir",
    "derruirían": "derruir",
    "derruirías": "derruir",
    "derruiste": "derruir",
    "derruisteis": "derruir",
    "derrumba": "derrumbar",
    "derrumbaba": "derrumbar",
    "derrumbabais": "derrumbar",
    "derrumbaban": "derrumbar",
    "derrumbabas": "derrumbar",
    "derrumbad": "derrumbar",
    "derrumbada": "derrumbar",
    "derrumbadas": "derrumbar",
    "derrumbaderos": "derrumbadero",
    "derrumbado": "derrumbar",
    "derrumbados": "derrumbar",
    "derrumbamientos": "derrumbamiento",
    "derrumbamos": "derrumbar",
    "derrumban": "derrumbar",
    "derrumbando": "derrumbar",
    "derrumbara": "derrumbar",
    "derrumbarais": "derrumbar",
    "derrumbaran": "derrumbar",
    "derrumbaras": "derrumbar",
    "derrumbare": "derrumbar",
    "derrumbareis": "derrumbar",
    "derrumbaremos": "derrumbar",
    "derrumbaren": "derrumbar",
    "derrumbares": "derrumbar",
    "derrumbaron": "derrumbar",
    "derrumbará": "derrumbar",
    "derrumbarán": "derrumbar",
    "derrumbarás": "derrumbar",
    "derrumbaré": "derrumbar",
    "derrumbaréis": "derrumbar",
    "derrumbaría": "derrumbar",
    "derrumbaríais": "derrumbar",
    "derrumbaríamos": "derrumbar",
    "derrumbarían": "derrumbar",
    "derrumbarías": "derrumbar",
    "derrumbas": "derrumbar",
    "derrumbase": "derrumbar",
    "derrumbaseis": "derrumbar",
    "derrumbasen": "derrumbar",
    "derrumbases": "derrumbar",
    "derrumbaste": "derrumbar",
    "derrumbasteis": "derrumbar",
    "derrumbe": "derrumbar",
    "derrumbemos": "derrumbar",
    "derrumben": "derrumbar",
    "derrumbes": "derrumbe",
    "derrumbo": "derrumbar",
    "derrumbábamos": "derrumbar",
    "derrumbáis": "derrumbar",
    "derrumbáramos": "derrumbar",
    "derrumbáremos": "derrumbar",
    "derrumbásemos": "derrumbar",
    "derrumbé": "derrumbar",
    "derrumbéis": "derrumbar",
    "derrumbó": "derrumbar",
    "derruya": "derruir",
    "derruyamos": "derruir",
    "derruyan": "derruir",
    "derruyas": "derruir",
    "derruye": "derruir",
    "derruyen": "derruir",
    "derruyendo": "derruir",
    "derruyera": "derruir",
    "derruyerais": "derruir",
    "derruyeran": "derruir",
    "derruyeras": "derruir",
    "derruyere": "derruir",
    "derruyereis": "derruir",
    "derruyeren": "derruir",
    "derruyeres": "derruir",
    "derruyeron": "derruir",
    "derruyes": "derruir",
    "derruyese": "derruir",
    "derruyeseis": "derruir",
    "derruyesen": "derruir",
    "derruyeses": "derruir",
    "derruyo": "derruir",
    "derruyáis": "derruir",
    "derruyéramos": "derruir",
    "derruyéremos": "derruir",
    "derruyésemos": "derruir",
    "derruyó": "derruir",
    "derruí": "derruir",
    "derruía": "derruir",
    "derruíais": "derruir",
    "derruíamos": "derruir",
    "derruían": "derruir",
    "derruías": "derruir",
    "derruís": "derruir",
    "derviches": "derviche",
    "des": "dar",
    "desabarranca": "desabarrancar",
    "desabarrancaba": "desabarrancar",
    "desabarrancabais": "desabarrancar",
    "desabarrancaban": "desabarrancar",
    "desabarrancabas": "desabarrancar",
    "desabarrancad": "desabarrancar",
    "desabarrancada": "desabarrancar",
    "desabarrancadas": "desabarrancar",
    "desabarrancado": "desabarrancar",
    "desabarrancados": "desabarrancar",
    "desabarrancamos": "desabarrancar",
    "desabarrancan": "desabarrancar",
    "desabarrancando": "desabarrancar",
    "desabarrancara": "desabarrancar",
    "desabarrancarais": "desabarrancar",
    "desabarrancaran": "desabarrancar",
    "desabarrancaras": "desabarrancar",
    "desabarrancare": "desabarrancar",
    "desabarrancareis": "desabarrancar",
    "desabarrancaremos": "desabarrancar",
    "desabarrancaren": "desabarrancar",
    "desabarrancares": "desabarrancar",
    "desabarrancaron": "desabarrancar",
    "desabarrancará": "desabarrancar",
    "desabarrancarán": "desabarrancar",
    "desabarrancarás": "desabarrancar",
    "desabarrancaré": "desabarrancar",
    "desabarrancaréis": "desabarrancar",
    "desabarrancaría": "desabarrancar",
    "desabarrancaríais": "desabarrancar",
    "desabarrancaríamos": "desabarrancar",
    "desabarrancarían": "desabarrancar",
    "desabarrancarías": "desabarrancar",
    "desabarrancas": "desabarrancar",
    "desabarrancase": "desabarrancar",
    "desabarrancaseis": "desabarrancar",
    "desabarrancasen": "desabarrancar",
    "desabarrancases": "desabarrancar",
    "desabarrancaste": "desabarrancar",
    "desabarrancasteis": "desabarrancar",
    "desabarranco": "desabarrancar",
    "desabarrancábamos": "desabarrancar",
    "desabarrancáis": "desabarrancar",
    "desabarrancáramos": "desabarrancar",
    "desabarrancáremos": "desabarrancar",
    "desabarrancásemos": "desabarrancar",
    "desabarrancó": "desabarrancar",
    "desabarranque": "desabarrancar",
    "desabarranquemos": "desabarrancar",
    "desabarranquen": "desabarrancar",
    "desabarranques": "desabarrancar",
    "desabarranqué": "desabarrancar",
    "desabarranquéis": "desabarrancar",
    "desabastece": "desabastecer",
    "desabasteced": "desabastecer",
    "desabastecemos": "desabastecer",
    "desabastecen": "desabastecer",
    "desabasteceremos": "desabastecer",
    "desabastecerá": "desabastecer",
    "desabastecerán": "desabastecer",
    "desabastecerás": "desabastecer",
    "desabasteceré": "desabastecer",
    "desabasteceréis": "desabastecer",
    "desabastecería": "desabastecer",
    "desabasteceríais": "desabastecer",
    "desabasteceríamos": "desabastecer",
    "desabastecerían": "desabastecer",
    "desabastecerías": "desabastecer",
    "desabasteces": "desabastecer",
    "desabastecida": "desabastecer",
    "desabastecidas": "desabastecer",
    "desabastecido": "desabastecer",
    "desabastecidos": "desabastecer",
    "desabasteciendo": "desabastecer",
    "desabasteciera": "desabastecer",
    "desabastecierais": "desabastecer",
    "desabastecieran": "desabastecer",
    "desabastecieras": "desabastecer",
    "desabasteciere": "desabastecer",
    "desabasteciereis": "desabastecer",
    "desabastecieren": "desabastecer",
    "desabastecieres": "desabastecer",
    "desabastecieron": "desabastecer",
    "desabasteciese": "desabastecer",
    "desabastecieseis": "desabastecer",
    "desabasteciesen": "desabastecer",
    "desabastecieses": "desabastecer",
    "desabastecimientos": "desabastecimiento",
    "desabastecimos": "desabastecer",
    "desabasteciste": "desabastecer",
    "desabastecisteis": "desabastecer",
    "desabasteciéramos": "desabastecer",
    "desabasteciéremos": "desabastecer",
    "desabasteciésemos": "desabastecer",
    "desabasteció": "desabastecer",
    "desabastecéis": "desabastecer",
    "desabastecí": "desabastecer",
    "desabastecía": "desabastecer",
    "desabastecíais": "desabastecer",
    "desabastecíamos": "desabastecer",
    "desabastecían": "desabastecer",
    "desabastecías": "desabastecer",
    "desabastezca": "desabastecer",
    "desabastezcamos": "desabastecer",
    "desabastezcan": "desabastecer",
    "desabastezcas": "desabastecer",
    "desabastezco": "desabastecer",
    "desabastezcáis": "desabastecer",
    "desabierta": "desabrir",
    "desabiertas": "desabrir",
    "desabierto": "desabrir",
    "desabiertos": "desabrir",
    "desabolla": "desabollar",
    "desabollaba": "desabollar",
    "desabollabais": "desabollar",
    "desabollaban": "desabollar",
    "desabollabas": "desabollar",
    "desabollad": "desabollar",
    "desabollada": "desabollar",
    "desabolladas": "desabollar",
    "desabollado": "desabollar",
    "desabollados": "desabollar",
    "desabollamos": "desabollar",
    "desabollan": "desabollar",
    "desabollando": "desabollar",
    "desabollara": "desabollar",
    "desabollarais": "desabollar",
    "desabollaran": "desabollar",
    "desabollaras": "desabollar",
    "desabollare": "desabollar",
    "desabollareis": "desabollar",
    "desabollaremos": "desabollar",
    "desabollaren": "desabollar",
    "desabollares": "desabollar",
    "desabollaron": "desabollar",
    "desabollará": "desabollar",
    "desabollarán": "desabollar",
    "desabollarás": "desabollar",
    "desabollaré": "desabollar",
    "desabollaréis": "desabollar",
    "desabollaría": "desabollar",
    "desabollaríais": "desabollar",
    "desabollaríamos": "desabollar",
    "desabollarían": "desabollar",
    "desabollarías": "desabollar",
    "desabollas": "desabollar",
    "desabollase": "desabollar",
    "desabollaseis": "desabollar",
    "desabollasen": "desabollar",
    "desabollases": "desabollar",
    "desabollaste": "desabollar",
    "desabollasteis": "desabollar",
    "desabolle": "desabollar",
    "desabollemos": "desabollar",
    "desabollen": "desabollar",
    "desabolles": "desabollar",
    "desabollo": "desabollar",
    "desabollábamos": "desabollar",
    "desabolláis": "desabollar",
    "desabolláramos": "desabollar",
    "desabolláremos": "desabollar",
    "desabollásemos": "desabollar",
    "desabollé": "desabollar",
    "desabolléis": "desabollar",
    "desabolló": "desabollar",
    "desabona": "desabonar",
    "desabonaba": "desabonar",
    "desabonabais": "desabonar",
    "desabonaban": "desabonar",
    "desabonabas": "desabonar",
    "desabonad": "desabonar",
    "desabonada": "desabonar",
    "desabonadas": "desabonar",
    "desabonado": "desabonar",
    "desabonados": "desabonar",
    "desabonamos": "desabonar",
    "desabonan": "desabonar",
    "desabonando": "desabonar",
    "desabonara": "desabonar",
    "desabonarais": "desabonar",
    "desabonaran": "desabonar",
    "desabonaras": "desabonar",
    "desabonare": "desabonar",
    "desabonareis": "desabonar",
    "desabonaremos": "desabonar",
    "desabonaren": "desabonar",
    "desabonares": "desabonar",
    "desabonaron": "desabonar",
    "desabonará": "desabonar",
    "desabonarán": "desabonar",
    "desabonarás": "desabonar",
    "desabonaré": "desabonar",
    "desabonaréis": "desabonar",
    "desabonaría": "desabonar",
    "desabonaríais": "desabonar",
    "desabonaríamos": "desabonar",
    "desabonarían": "desabonar",
    "desabonarías": "desabonar",
    "desabonas": "desabonar",
    "desabonase": "desabonar",
    "desabonaseis": "desabonar",
    "desabonasen": "desabonar",
    "desabonases": "desabonar",
    "desabonaste": "desabonar",
    "desabonasteis": "desabonar",
    "desabone": "desabonar",
    "desabonemos": "desabonar",
    "desabonen": "desabonar",
    "desabones": "desabonar",
    "desabono": "desabonar",
    "desabonos": "desabono",
    "desabonábamos": "desabonar",
    "desabonáis": "desabonar",
    "desabonáramos": "desabonar",
    "desabonáremos": "desabonar",
    "desabonásemos": "desabonar",
    "desaboné": "desabonar",
    "desabonéis": "desabonar",
    "desabonó": "desabonar",
    "desabores": "desabor",
    "desaborida": "desaborido",
    "desaboridas": "desaborido",
    "desaboridos": "desaborido",
    "desabotona": "desabotonar",
    "desabotonaba": "desabotonar",
    "desabotonabais": "desabotonar",
    "desabotonaban": "desabotonar",
    "desabotonabas": "desabotonar",
    "desabotonad": "desabotonar",
    "desabotonada": "desabotonar",
    "desabotonadas": "desabotonar",
    "desabotonado": "desabotonar",
    "desabotonados": "desabotonar",
    "desabotonamos": "desabotonar",
    "desabotonan": "desabotonar",
    "desabotonando": "desabotonar",
    "desabotonara": "desabotonar",
    "desabotonarais": "desabotonar",
    "desabotonaran": "desabotonar",
    "desabotonaras": "desabotonar",
    "desabotonare": "desabotonar",
    "desabotonareis": "desabotonar",
    "desabotonaremos": "desabotonar",
    "desabotonaren": "desabotonar",
    "desabotonares": "desabotonar",
    "desabotonaron": "desabotonar",
    "desabotonará": "desabotonar",
    "desabotonarán": "desabotonar",
    "desabotonarás": "desabotonar",
    "desabotonaré": "desabotonar",
    "desabotonaréis": "desabotonar",
    "desabotonaría": "desabotonar",
    "desabotonaríais": "desabotonar",
    "desabotonaríamos": "desabotonar",
    "desabotonarían": "desabotonar",
    "desabotonarías": "desabotonar",
    "desabotonas": "desabotonar",
    "desabotonase": "desabotonar",
    "desabotonaseis": "desabotonar",
    "desabotonasen": "desabotonar",
    "desabotonases": "desabotonar",
    "desabotonaste": "desabotonar",
    "desabotonasteis": "desabotonar",
    "desabotone": "desabotonar",
    "desabotonemos": "desabotonar",
    "desabotonen": "desabotonar",
    "desabotones": "desabotonar",
    "desabotono": "desabotonar",
    "desabotonábamos": "desabotonar",
    "desabotonáis": "desabotonar",
    "desabotonáramos": "desabotonar",
    "desabotonáremos": "desabotonar",
    "desabotonásemos": "desabotonar",
    "desabotoné": "desabotonar",
    "desabotonéis": "desabotonar",
    "desabotonó": "desabotonar",
    "desabra": "desabrir",
    "desabramos": "desabrir",
    "desabran": "desabrir",
    "desabras": "desabrir",
    "desabre": "desabrir",
    "desabren": "desabrir",
    "desabres": "desabrir",
    "desabrid": "desabrir",
    "desabriendo": "desabrir",
    "desabriera": "desabrir",
    "desabrierais": "desabrir",
    "desabrieran": "desabrir",
    "desabrieras": "desabrir",
    "desabriere": "desabrir",
    "desabriereis": "desabrir",
    "desabrieren": "desabrir",
    "desabrieres": "desabrir",
    "desabrieron": "desabrir",
    "desabriese": "desabrir",
    "desabrieseis": "desabrir",
    "desabriesen": "desabrir",
    "desabrieses": "desabrir",
    "desabriga": "desabrigar",
    "desabrigaba": "desabrigar",
    "desabrigabais": "desabrigar",
    "desabrigaban": "desabrigar",
    "desabrigabas": "desabrigar",
    "desabrigad": "desabrigar",
    "desabrigada": "desabrigar",
    "desabrigadas": "desabrigar",
    "desabrigado": "desabrigar",
    "desabrigados": "desabrigar",
    "desabrigamos": "desabrigar",
    "desabrigan": "desabrigar",
    "desabrigando": "desabrigar",
    "desabrigara": "desabrigar",
    "desabrigarais": "desabrigar",
    "desabrigaran": "desabrigar",
    "desabrigaras": "desabrigar",
    "desabrigare": "desabrigar",
    "desabrigareis": "desabrigar",
    "desabrigaremos": "desabrigar",
    "desabrigaren": "desabrigar",
    "desabrigares": "desabrigar",
    "desabrigaron": "desabrigar",
    "desabrigará": "desabrigar",
    "desabrigarán": "desabrigar",
    "desabrigarás": "desabrigar",
    "desabrigaré": "desabrigar",
    "desabrigaréis": "desabrigar",
    "desabrigaría": "desabrigar",
    "desabrigaríais": "desabrigar",
    "desabrigaríamos": "desabrigar",
    "desabrigarían": "desabrigar",
    "desabrigarías": "desabrigar",
    "desabrigas": "desabrigar",
    "desabrigase": "desabrigar",
    "desabrigaseis": "desabrigar",
    "desabrigasen": "desabrigar",
    "desabrigases": "desabrigar",
    "desabrigaste": "desabrigar",
    "desabrigasteis": "desabrigar",
    "desabrigo": "desabrigar",
    "desabrigos": "desabrigo",
    "desabrigue": "desabrigar",
    "desabriguemos": "desabrigar",
    "desabriguen": "desabrigar",
    "desabrigues": "desabrigar",
    "desabrigué": "desabrigar",
    "desabriguéis": "desabrigar",
    "desabrigábamos": "desabrigar",
    "desabrigáis": "desabrigar",
    "desabrigáramos": "desabrigar",
    "desabrigáremos": "desabrigar",
    "desabrigásemos": "desabrigar",
    "desabrigó": "desabrigar",
    "desabrimientos": "desabrimiento",
    "desabrimos": "desabrir",
    "desabriremos": "desabrir",
    "desabrirá": "desabrir",
    "desabrirán": "desabrir",
    "desabrirás": "desabrir",
    "desabriré": "desabrir",
    "desabriréis": "desabrir",
    "desabriría": "desabrir",
    "desabriríais": "desabrir",
    "desabriríamos": "desabrir",
    "desabrirían": "desabrir",
    "desabrirías": "desabrir",
    "desabriste": "desabrir",
    "desabristeis": "desabrir",
    "desabriéramos": "desabrir",
    "desabriéremos": "desabrir",
    "desabriésemos": "desabrir",
    "desabrió": "desabrir",
    "desabro": "desabrir",
    "desabrocha": "desabrochar",
    "desabrochaba": "desabrochar",
    "desabrochabais": "desabrochar",
    "desabrochaban": "desabrochar",
    "desabrochabas": "desabrochar",
    "desabrochad": "desabrochar",
    "desabrochada": "desabrochar",
    "desabrochadas": "desabrochar",
    "desabrochado": "desabrochar",
    "desabrochados": "desabrochar",
    "desabrochamos": "desabrochar",
    "desabrochan": "desabrochar",
    "desabrochando": "desabrochar",
    "desabrochara": "desabrochar",
    "desabrocharais": "desabrochar",
    "desabrocharan": "desabrochar",
    "desabrocharas": "desabrochar",
    "desabrochare": "desabrochar",
    "desabrochareis": "desabrochar",
    "desabrocharemos": "desabrochar",
    "desabrocharen": "desabrochar",
    "desabrochares": "desabrochar",
    "desabrocharon": "desabrochar",
    "desabrochará": "desabrochar",
    "desabrocharán": "desabrochar",
    "desabrocharás": "desabrochar",
    "desabrocharé": "desabrochar",
    "desabrocharéis": "desabrochar",
    "desabrocharía": "desabrochar",
    "desabrocharíais": "desabrochar",
    "desabrocharíamos": "desabrochar",
    "desabrocharían": "desabrochar",
    "desabrocharías": "desabrochar",
    "desabrochas": "desabrochar",
    "desabrochase": "desabrochar",
    "desabrochaseis": "desabrochar",
    "desabrochasen": "desabrochar",
    "desabrochases": "desabrochar",
    "desabrochaste": "desabrochar",
    "desabrochasteis": "desabrochar",
    "desabroche": "desabrochar",
    "desabrochemos": "desabrochar",
    "desabrochen": "desabrochar",
    "desabroches": "desabrochar",
    "desabrocho": "desabrochar",
    "desabrochábamos": "desabrochar",
    "desabrocháis": "desabrochar",
    "desabrocháramos": "desabrochar",
    "desabrocháremos": "desabrochar",
    "desabrochásemos": "desabrochar",
    "desabroché": "desabrochar",
    "desabrochéis": "desabrochar",
    "desabrochó": "desabrochar",
    "desabráis": "desabrir",
    "desabrí": "desabrir",
    "desabría": "desabrir",
    "desabríais": "desabrir",
    "desabríamos": "desabrir",
    "desabrían": "desabrir",
    "desabrías": "desabrir",
    "desabrís": "desabrir",
    "desacalora": "desacalorar",
    "desacaloraba": "desacalorar",
    "desacalorabais": "desacalorar",
    "desacaloraban": "desacalorar",
    "desacalorabas": "desacalorar",
    "desacalorad": "desacalorar",
    "desacalorada": "desacalorar",
    "desacaloradas": "desacalorar",
    "desacalorado": "desacalorar",
    "desacalorados": "desacalorar",
    "desacaloramos": "desacalorar",
    "desacaloran": "desacalorar",
    "desacalorando": "desacalorar",
    "desacalorara": "desacalorar",
    "desacalorarais": "desacalorar",
    "desacaloraran": "desacalorar",
    "desacaloraras": "desacalorar",
    "desacalorare": "desacalorar",
    "desacalorareis": "desacalorar",
    "desacaloraremos": "desacalorar",
    "desacaloraren": "desacalorar",
    "desacalorares": "desacalorar",
    "desacaloraron": "desacalorar",
    "desacalorará": "desacalorar",
    "desacalorarán": "desacalorar",
    "desacalorarás": "desacalorar",
    "desacaloraré": "desacalorar",
    "desacaloraréis": "desacalorar",
    "desacaloraría": "desacalorar",
    "desacaloraríais": "desacalorar",
    "desacaloraríamos": "desacalorar",
    "desacalorarían": "desacalorar",
    "desacalorarías": "desacalorar",
    "desacaloras": "desacalorar",
    "desacalorase": "desacalorar",
    "desacaloraseis": "desacalorar",
    "desacalorasen": "desacalorar",
    "desacalorases": "desacalorar",
    "desacaloraste": "desacalorar",
    "desacalorasteis": "desacalorar",
    "desacalore": "desacalorar",
    "desacaloremos": "desacalorar",
    "desacaloren": "desacalorar",
    "desacalores": "desacalorar",
    "desacaloro": "desacalorar",
    "desacalorábamos": "desacalorar",
    "desacaloráis": "desacalorar",
    "desacaloráramos": "desacalorar",
    "desacaloráremos": "desacalorar",
    "desacalorásemos": "desacalorar",
    "desacaloré": "desacalorar",
    "desacaloréis": "desacalorar",
    "desacaloró": "desacalorar",
    "desacata": "desacatar",
    "desacataba": "desacatar",
    "desacatabais": "desacatar",
    "desacataban": "desacatar",
    "desacatabas": "desacatar",
    "desacatad": "desacatar",
    "desacatada": "desacatar",
    "desacatadas": "desacatar",
    "desacatado": "desacatar",
    "desacatados": "desacatar",
    "desacatamientos": "desacatamiento",
    "desacatamos": "desacatar",
    "desacatan": "desacatar",
    "desacatando": "desacatar",
    "desacatara": "desacatar",
    "desacatarais": "desacatar",
    "desacataran": "desacatar",
    "desacataras": "desacatar",
    "desacatare": "desacatar",
    "desacatareis": "desacatar",
    "desacataremos": "desacatar",
    "desacataren": "desacatar",
    "desacatares": "desacatar",
    "desacataron": "desacatar",
    "desacatará": "desacatar",
    "desacatarán": "desacatar",
    "desacatarás": "desacatar",
    "desacataré": "desacatar",
    "desacataréis": "desacatar",
    "desacataría": "desacatar",
    "desacataríais": "desacatar",
    "desacataríamos": "desacatar",
    "desacatarían": "desacatar",
    "desacatarías": "desacatar",
    "desacatas": "desacatar",
    "desacatase": "desacatar",
    "desacataseis": "desacatar",
    "desacatasen": "desacatar",
    "desacatases": "desacatar",
    "desacataste": "desacatar",
    "desacatasteis": "desacatar",
    "desacate": "desacatar",
    "desacatemos": "desacatar",
    "desacaten": "desacatar",
    "desacates": "desacatar",
    "desacato": "desacatar",
    "desacatos": "desacato",
    "desacatábamos": "desacatar",
    "desacatáis": "desacatar",
    "desacatáramos": "desacatar",
    "desacatáremos": "desacatar",
    "desacatásemos": "desacatar",
    "desacaté": "desacatar",
    "desacatéis": "desacatar",
    "desacató": "desacatar",
    "desaceita": "desaceitar",
    "desaceitaba": "desaceitar",
    "desaceitabais": "desaceitar",
    "desaceitaban": "desaceitar",
    "desaceitabas": "desaceitar",
    "desaceitad": "desaceitar",
    "desaceitada": "desaceitar",
    "desaceitadas": "desaceitar",
    "desaceitado": "desaceitar",
    "desaceitados": "desaceitar",
    "desaceitamos": "desaceitar",
    "desaceitan": "desaceitar",
    "desaceitando": "desaceitar",
    "desaceitara": "desaceitar",
    "desaceitarais": "desaceitar",
    "desaceitaran": "desaceitar",
    "desaceitaras": "desaceitar",
    "desaceitare": "desaceitar",
    "desaceitareis": "desaceitar",
    "desaceitaremos": "desaceitar",
    "desaceitaren": "desaceitar",
    "desaceitares": "desaceitar",
    "desaceitaron": "desaceitar",
    "desaceitará": "desaceitar",
    "desaceitarán": "desaceitar",
    "desaceitarás": "desaceitar",
    "desaceitaré": "desaceitar",
    "desaceitaréis": "desaceitar",
    "desaceitaría": "desaceitar",
    "desaceitaríais": "desaceitar",
    "desaceitaríamos": "desaceitar",
    "desaceitarían": "desaceitar",
    "desaceitarías": "desaceitar",
    "desaceitas": "desaceitar",
    "desaceitase": "desaceitar",
    "desaceitaseis": "desaceitar",
    "desaceitasen": "desaceitar",
    "desaceitases": "desaceitar",
    "desaceitaste": "desaceitar",
    "desaceitasteis": "desaceitar",
    "desaceite": "desaceitar",
    "desaceitemos": "desaceitar",
    "desaceiten": "desaceitar",
    "desaceites": "desaceitar",
    "desaceito": "desaceitar",
    "desaceitábamos": "desaceitar",
    "desaceitáis": "desaceitar",
    "desaceitáramos": "desaceitar",
    "desaceitáremos": "desaceitar",
    "desaceitásemos": "desaceitar",
    "desaceité": "desaceitar",
    "desaceitéis": "desaceitar",
    "desaceitó": "desaceitar",
    "desacelera": "desacelerar",
    "desaceleraba": "desacelerar",
    "desacelerabais": "desacelerar",
    "desaceleraban": "desacelerar",
    "desacelerabas": "desacelerar",
    "desaceleraciones": "desaceleración",
    "desacelerad": "desacelerar",
    "desacelerada": "desacelerar",
    "desaceleradas": "desacelerar",
    "desacelerado": "desacelerar",
    "desacelerados": "desacelerar",
    "desaceleramos": "desacelerar",
    "desaceleran": "desacelerar",
    "desacelerando": "desacelerar",
    "desacelerara": "desacelerar",
    "desacelerarais": "desacelerar",
    "desaceleraran": "desacelerar",
    "desaceleraras": "desacelerar",
    "desacelerare": "desacelerar",
    "desacelerareis": "desacelerar",
    "desaceleraremos": "desacelerar",
    "desaceleraren": "desacelerar",
    "desacelerares": "desacelerar",
    "desaceleraron": "desacelerar",
    "desacelerará": "desacelerar",
    "desacelerarán": "desacelerar",
    "desacelerarás": "desacelerar",
    "desaceleraré": "desacelerar",
    "desaceleraréis": "desacelerar",
    "desaceleraría": "desacelerar",
    "desaceleraríais": "desacelerar",
    "desaceleraríamos": "desacelerar",
    "desacelerarían": "desacelerar",
    "desacelerarías": "desacelerar",
    "desaceleras": "desacelerar",
    "desacelerase": "desacelerar",
    "desaceleraseis": "desacelerar",
    "desacelerasen": "desacelerar",
    "desacelerases": "desacelerar",
    "desaceleraste": "desacelerar",
    "desacelerasteis": "desacelerar",
    "desacelere": "desacelerar",
    "desaceleremos": "desacelerar",
    "desaceleren": "desacelerar",
    "desaceleres": "desacelerar",
    "desacelero": "desacelerar",
    "desacelerábamos": "desacelerar",
    "desaceleráis": "desacelerar",
    "desaceleráramos": "desacelerar",
    "desaceleráremos": "desacelerar",
    "desacelerásemos": "desacelerar",
    "desaceleré": "desacelerar",
    "desaceleréis": "desacelerar",
    "desaceleró": "desacelerar",
    "desacentuaciones": "desacentuación",
    "desacera": "desacerar",
    "desaceraba": "desacerar",
    "desacerabais": "desacerar",
    "desaceraban": "desacerar",
    "desacerabas": "desacerar",
    "desaceraciones": "desaceración",
    "desacerad": "desacerar",
    "desacerada": "desacerar",
    "desaceradas": "desacerar",
    "desacerado": "desacerar",
    "desacerados": "desacerar",
    "desaceramos": "desacerar",
    "desaceran": "desacerar",
    "desacerando": "desacerar",
    "desacerara": "desacerar",
    "desacerarais": "desacerar",
    "desaceraran": "desacerar",
    "desaceraras": "desacerar",
    "desacerare": "desacerar",
    "desacerareis": "desacerar",
    "desaceraremos": "desacerar",
    "desaceraren": "desacerar",
    "desacerares": "desacerar",
    "desaceraron": "desacerar",
    "desacerará": "desacerar",
    "desacerarán": "desacerar",
    "desacerarás": "desacerar",
    "desaceraré": "desacerar",
    "desaceraréis": "desacerar",
    "desaceraría": "desacerar",
    "desaceraríais": "desacerar",
    "desaceraríamos": "desacerar",
    "desacerarían": "desacerar",
    "desacerarías": "desacerar",
    "desaceras": "desacerar",
    "desacerase": "desacerar",
    "desaceraseis": "desacerar",
    "desacerasen": "desacerar",
    "desacerases": "desacerar",
    "desaceraste": "desacerar",
    "desacerasteis": "desacerar",
    "desacere": "desacerar",
    "desaceremos": "desacerar",
    "desaceren": "desacerar",
    "desaceres": "desacerar",
    "desacero": "desacerar",
    "desacertaba": "desacertar",
    "desacertabais": "desacertar",
    "desacertaban": "desacertar",
    "desacertabas": "desacertar",
    "desacertad": "desacertar",
    "desacertada": "desacertar",
    "desacertadas": "desacertar",
    "desacertado": "desacertar",
    "desacertados": "desacertar",
    "desacertamos": "desacertar",
    "desacertando": "desacertar",
    "desacertara": "desacertar",
    "desacertarais": "desacertar",
    "desacertaran": "desacertar",
    "desacertaras": "desacertar",
    "desacertare": "desacertar",
    "desacertareis": "desacertar",
    "desacertaremos": "desacertar",
    "desacertaren": "desacertar",
    "desacertares": "desacertar",
    "desacertaron": "desacertar",
    "desacertará": "desacertar",
    "desacertarán": "desacertar",
    "desacertarás": "desacertar",
    "desacertaré": "desacertar",
    "desacertaréis": "desacertar",
    "desacertaría": "desacertar",
    "desacertaríais": "desacertar",
    "desacertaríamos": "desacertar",
    "desacertarían": "desacertar",
    "desacertarías": "desacertar",
    "desacertase": "desacertar",
    "desacertaseis": "desacertar",
    "desacertasen": "desacertar",
    "desacertases": "desacertar",
    "desacertaste": "desacertar",
    "desacertasteis": "desacertar",
    "desacertemos": "desacertar",
    "desacertábamos": "desacertar",
    "desacertáis": "desacertar",
    "desacertáramos": "desacertar",
    "desacertáremos": "desacertar",
    "desacertásemos": "desacertar",
    "desacerté": "desacertar",
    "desacertéis": "desacertar",
    "desacertó": "desacertar",
    "desacerábamos": "desacerar",
    "desaceráis": "desacerar",
    "desaceráramos": "desacerar",
    "desaceráremos": "desacerar",
    "desacerásemos": "desacerar",
    "desaceré": "desacerar",
    "desaceréis": "desacerar",
    "desaceró": "desacerar",
    "desacidificaciones": "desacidificación",
    "desacierta": "desacertar",
    "desaciertan": "desacertar",
    "desaciertas": "desacertar",
    "desacierte": "desacertar",
    "desacierten": "desacertar",
    "desaciertes": "desacertar",
    "desacierto": "desacertar",
    "desaciertos": "desacierto",
    "desacobarda": "desacobardar",
    "desacobardaba": "desacobardar",
    "desacobardabais": "desacobardar",
    "desacobardaban": "desacobardar",
    "desacobardabas": "desacobardar",
    "desacobardad": "desacobardar",
    "desacobardada": "desacobardar",
    "desacobardadas": "desacobardar",
    "desacobardado": "desacobardar",
    "desacobardados": "desacobardar",
    "desacobardamientos": "desacobardamiento",
    "desacobardamos": "desacobardar",
    "desacobardan": "desacobardar",
    "desacobardando": "desacobardar",
    "desacobardara": "desacobardar",
    "desacobardarais": "desacobardar",
    "desacobardaran": "desacobardar",
    "desacobardaras": "desacobardar",
    "desacobardare": "desacobardar",
    "desacobardareis": "desacobardar",
    "desacobardaremos": "desacobardar",
    "desacobardaren": "desacobardar",
    "desacobardares": "desacobardar",
    "desacobardaron": "desacobardar",
    "desacobardará": "desacobardar",
    "desacobardarán": "desacobardar",
    "desacobardarás": "desacobardar",
    "desacobardaré": "desacobardar",
    "desacobardaréis": "desacobardar",
    "desacobardaría": "desacobardar",
    "desacobardaríais": "desacobardar",
    "desacobardaríamos": "desacobardar",
    "desacobardarían": "desacobardar",
    "desacobardarías": "desacobardar",
    "desacobardas": "desacobardar",
    "desacobardase": "desacobardar",
    "desacobardaseis": "desacobardar",
    "desacobardasen": "desacobardar",
    "desacobardases": "desacobardar",
    "desacobardaste": "desacobardar",
    "desacobardasteis": "desacobardar",
    "desacobarde": "desacobardar",
    "desacobardemos": "desacobardar",
    "desacobarden": "desacobardar",
    "desacobardes": "desacobardar",
    "desacobardo": "desacobardar",
    "desacobardábamos": "desacobardar",
    "desacobardáis": "desacobardar",
    "desacobardáramos": "desacobardar",
    "desacobardáremos": "desacobardar",
    "desacobardásemos": "desacobardar",
    "desacobardé": "desacobardar",
    "desacobardéis": "desacobardar",
    "desacobardó": "desacobardar",
    "desacomoda": "desacomodar",
    "desacomodaba": "desacomodar",
    "desacomodabais": "desacomodar",
    "desacomodaban": "desacomodar",
    "desacomodabas": "desacomodar",
    "desacomodad": "desacomodar",
    "desacomodada": "desacomodar",
    "desacomodadas": "desacomodar",
    "desacomodado": "desacomodar",
    "desacomodados": "desacomodar",
    "desacomodamos": "desacomodar",
    "desacomodan": "desacomodar",
    "desacomodando": "desacomodar",
    "desacomodara": "desacomodar",
    "desacomodarais": "desacomodar",
    "desacomodaran": "desacomodar",
    "desacomodaras": "desacomodar",
    "desacomodare": "desacomodar",
    "desacomodareis": "desacomodar",
    "desacomodaremos": "desacomodar",
    "desacomodaren": "desacomodar",
    "desacomodares": "desacomodar",
    "desacomodaron": "desacomodar",
    "desacomodará": "desacomodar",
    "desacomodarán": "desacomodar",
    "desacomodarás": "desacomodar",
    "desacomodaré": "desacomodar",
    "desacomodaréis": "desacomodar",
    "desacomodaría": "desacomodar",
    "desacomodaríais": "desacomodar",
    "desacomodaríamos": "desacomodar",
    "desacomodarían": "desacomodar",
    "desacomodarías": "desacomodar",
    "desacomodas": "desacomodar",
    "desacomodase": "desacomodar",
    "desacomodaseis": "desacomodar",
    "desacomodasen": "desacomodar",
    "desacomodases": "desacomodar",
    "desacomodaste": "desacomodar",
    "desacomodasteis": "desacomodar",
    "desacomode": "desacomodar",
    "desacomodemos": "desacomodar",
    "desacomoden": "desacomodar",
    "desacomodes": "desacomodar",
    "desacomodo": "desacomodar",
    "desacomodos": "desacomodo",
    "desacomodábamos": "desacomodar",
    "desacomodáis": "desacomodar",
    "desacomodáramos": "desacomodar",
    "desacomodáremos": "desacomodar",
    "desacomodásemos": "desacomodar",
    "desacomodé": "desacomodar",
    "desacomodéis": "desacomodar",
    "desacomodó": "desacomodar",
    "desacompaña": "desacompañar",
    "desacompañaba": "desacompañar",
    "desacompañabais": "desacompañar",
    "desacompañaban": "desacompañar",
    "desacompañabas": "desacompañar",
    "desacompañad": "desacompañar",
    "desacompañada": "desacompañar",
    "desacompañadas": "desacompañar",
    "desacompañado": "desacompañar",
    "desacompañados": "desacompañar",
    "desacompañamientos": "desacompañamiento",
    "desacompañamos": "desacompañar",
    "desacompañan": "desacompañar",
    "desacompañando": "desacompañar",
    "desacompañara": "desacompañar",
    "desacompañarais": "desacompañar",
    "desacompañaran": "desacompañar",
    "desacompañaras": "desacompañar",
    "desacompañare": "desacompañar",
    "desacompañareis": "desacompañar",
    "desacompañaremos": "desacompañar",
    "desacompañaren": "desacompañar",
    "desacompañares": "desacompañar",
    "desacompañaron": "desacompañar",
    "desacompañará": "desacompañar",
    "desacompañarán": "desacompañar",
    "desacompañarás": "desacompañar",
    "desacompañaré": "desacompañar",
    "desacompañaréis": "desacompañar",
    "desacompañaría": "desacompañar",
    "desacompañaríais": "desacompañar",
    "desacompañaríamos": "desacompañar",
    "desacompañarían": "desacompañar",
    "desacompañarías": "desacompañar",
    "desacompañas": "desacompañar",
    "desacompañase": "desacompañar",
    "desacompañaseis": "desacompañar",
    "desacompañasen": "desacompañar",
    "desacompañases": "desacompañar",
    "desacompañaste": "desacompañar",
    "desacompañasteis": "desacompañar",
    "desacompañe": "desacompañar",
    "desacompañemos": "desacompañar",
    "desacompañen": "desacompañar",
    "desacompañes": "desacompañar",
    "desacompaño": "desacompañar",
    "desacompañábamos": "desacompañar",
    "desacompañáis": "desacompañar",
    "desacompañáramos": "desacompañar",
    "desacompañáremos": "desacompañar",
    "desacompañásemos": "desacompañar",
    "desacompañé": "desacompañar",
    "desacompañéis": "desacompañar",
    "desacompañó": "desacompañar",
    "desacondiciona": "desacondicionar",
    "desacondicionaba": "desacondicionar",
    "desacondicionabais": "desacondicionar",
    "desacondicionaban": "desacondicionar",
    "desacondicionabas": "desacondicionar",
    "desacondicionad": "desacondicionar",
    "desacondicionada": "desacondicionar",
    "desacondicionadas": "desacondicionar",
    "desacondicionado": "desacondicionar",
    "desacondicionados": "desacondicionar",
    "desacondicionamos": "desacondicionar",
    "desacondicionan": "desacondicionar",
    "desacondicionando": "desacondicionar",
    "desacondicionara": "desacondicionar",
    "desacondicionarais": "desacondicionar",
    "desacondicionaran": "desacondicionar",
    "desacondicionaras": "desacondicionar",
    "desacondicionare": "desacondicionar",
    "desacondicionareis": "desacondicionar",
    "desacondicionaremos": "desacondicionar",
    "desacondicionaren": "desacondicionar",
    "desacondicionares": "desacondicionar",
    "desacondicionaron": "desacondicionar",
    "desacondicionará": "desacondicionar",
    "desacondicionarán": "desacondicionar",
    "desacondicionarás": "desacondicionar",
    "desacondicionaré": "desacondicionar",
    "desacondicionaréis": "desacondicionar",
    "desacondicionaría": "desacondicionar",
    "desacondicionaríais": "desacondicionar",
    "desacondicionaríamos": "desacondicionar",
    "desacondicionarían": "desacondicionar",
    "desacondicionarías": "desacondicionar",
    "desacondicionas": "desacondicionar",
    "desacondicionase": "desacondicionar",
    "desacondicionaseis": "desacondicionar",
    "desacondicionasen": "desacondicionar",
    "desacondicionases": "desacondicionar",
    "desacondicionaste": "desacondicionar",
    "desacondicionasteis": "desacondicionar",
    "desacondicione": "desacondicionar",
    "desacondicionemos": "desacondicionar",
    "desacondicionen": "desacondicionar",
    "desacondiciones": "desacondicionar",
    "desacondiciono": "desacondicionar",
    "desacondicionábamos": "desacondicionar",
    "desacondicionáis": "desacondicionar",
    "desacondicionáramos": "desacondicionar",
    "desacondicionáremos": "desacondicionar",
    "desacondicionásemos": "desacondicionar",
    "desacondicioné": "desacondicionar",
    "desacondicionéis": "desacondicionar",
    "desacondicionó": "desacondicionar",
    "desaconseja": "desaconsejar",
    "desaconsejaba": "desaconsejar",
    "desaconsejabais": "desaconsejar",
    "desaconsejaban": "desaconsejar",
    "desaconsejabas": "desaconsejar",
    "desaconsejables": "desaconsejable",
    "desaconsejad": "desaconsejar",
    "desaconsejada": "desaconsejar",
    "desaconsejadas": "desaconsejar",
    "desaconsejado": "desaconsejar",
    "desaconsejados": "desaconsejar",
    "desaconsejamos": "desaconsejar",
    "desaconsejan": "desaconsejar",
    "desaconsejando": "desaconsejar",
    "desaconsejara": "desaconsejar",
    "desaconsejarais": "desaconsejar",
    "desaconsejaran": "desaconsejar",
    "desaconsejaras": "desaconsejar",
    "desaconsejare": "desaconsejar",
    "desaconsejareis": "desaconsejar",
    "desaconsejaremos": "desaconsejar",
    "desaconsejaren": "desaconsejar",
    "desaconsejares": "desaconsejar",
    "desaconsejaron": "desaconsejar",
    "desaconsejará": "desaconsejar",
    "desaconsejarán": "desaconsejar",
    "desaconsejarás": "desaconsejar",
    "desaconsejaré": "desaconsejar",
    "desaconsejaréis": "desaconsejar",
    "desaconsejaría": "desaconsejar",
    "desaconsejaríais": "desaconsejar",
    "desaconsejaríamos": "desaconsejar",
    "desaconsejarían": "desaconsejar",
    "desaconsejarías": "desaconsejar",
    "desaconsejas": "desaconsejar",
    "desaconsejase": "desaconsejar",
    "desaconsejaseis": "desaconsejar",
    "desaconsejasen": "desaconsejar",
    "desaconsejases": "desaconsejar",
    "desaconsejaste": "desaconsejar",
    "desaconsejasteis": "desaconsejar",
    "desaconseje": "desaconsejar",
    "desaconsejemos": "desaconsejar",
    "desaconsejen": "desaconsejar",
    "desaconsejes": "desaconsejar",
    "desaconsejo": "desaconsejar",
    "desaconsejábamos": "desaconsejar",
    "desaconsejáis": "desaconsejar",
    "desaconsejáramos": "desaconsejar",
    "desaconsejáremos": "desaconsejar",
    "desaconsejásemos": "desaconsejar",
    "desaconsejé": "desaconsejar",
    "desaconsejéis": "desaconsejar",
    "desaconsejó": "desaconsejar",
    "desacopla": "desacoplar",
    "desacoplaba": "desacoplar",
    "desacoplabais": "desacoplar",
    "desacoplaban": "desacoplar",
    "desacoplabas": "desacoplar",
    "desacoplad": "desacoplar",
    "desacoplada": "desacoplar",
    "desacopladas": "desacoplar",
    "desacoplado": "desacoplar",
    "desacoplados": "desacoplar",
    "desacoplamientos": "desacoplamiento",
    "desacoplamos": "desacoplar",
    "desacoplan": "desacoplar",
    "desacoplando": "desacoplar",
    "desacoplara": "desacoplar",
    "desacoplarais": "desacoplar",
    "desacoplaran": "desacoplar",
    "desacoplaras": "desacoplar",
    "desacoplare": "desacoplar",
    "desacoplareis": "desacoplar",
    "desacoplaremos": "desacoplar",
    "desacoplaren": "desacoplar",
    "desacoplares": "desacoplar",
    "desacoplaron": "desacoplar",
    "desacoplará": "desacoplar",
    "desacoplarán": "desacoplar",
    "desacoplarás": "desacoplar",
    "desacoplaré": "desacoplar",
    "desacoplaréis": "desacoplar",
    "desacoplaría": "desacoplar",
    "desacoplaríais": "desacoplar",
    "desacoplaríamos": "desacoplar",
    "desacoplarían": "desacoplar",
    "desacoplarías": "desacoplar",
    "desacoplas": "desacoplar",
    "desacoplase": "desacoplar",
    "desacoplaseis": "desacoplar",
    "desacoplasen": "desacoplar",
    "desacoplases": "desacoplar",
    "desacoplaste": "desacoplar",
    "desacoplasteis": "desacoplar",
    "desacople": "desacoplar",
    "desacoplemos": "desacoplar",
    "desacoplen": "desacoplar",
    "desacoples": "desacoplar",
    "desacoplo": "desacoplar",
    "desacoplábamos": "desacoplar",
    "desacopláis": "desacoplar",
    "desacopláramos": "desacoplar",
    "desacopláremos": "desacoplar",
    "desacoplásemos": "desacoplar",
    "desacoplé": "desacoplar",
    "desacopléis": "desacoplar",
    "desacopló": "desacoplar",
    "desacordaba": "desacordar",
    "desacordabais": "desacordar",
    "desacordaban": "desacordar",
    "desacordabas": "desacordar",
    "desacordad": "desacordar",
    "desacordada": "desacordar",
    "desacordadas": "desacordar",
    "desacordado": "desacordar",
    "desacordados": "desacordar",
    "desacordamos": "desacordar",
    "desacordando": "desacordar",
    "desacordara": "desacordar",
    "desacordarais": "desacordar",
    "desacordaran": "desacordar",
    "desacordaras": "desacordar",
    "desacordare": "desacordar",
    "desacordareis": "desacordar",
    "desacordaremos": "desacordar",
    "desacordaren": "desacordar",
    "desacordares": "desacordar",
    "desacordaron": "desacordar",
    "desacordará": "desacordar",
    "desacordarán": "desacordar",
    "desacordarás": "desacordar",
    "desacordaré": "desacordar",
    "desacordaréis": "desacordar",
    "desacordaría": "desacordar",
    "desacordaríais": "desacordar",
    "desacordaríamos": "desacordar",
    "desacordarían": "desacordar",
    "desacordarías": "desacordar",
    "desacordase": "desacordar",
    "desacordaseis": "desacordar",
    "desacordasen": "desacordar",
    "desacordases": "desacordar",
    "desacordaste": "desacordar",
    "desacordasteis": "desacordar",
    "desacordemos": "desacordar",
    "desacordes": "desacorde",
    "desacordábamos": "desacordar",
    "desacordáis": "desacordar",
    "desacordáramos": "desacordar",
    "desacordáremos": "desacordar",
    "desacordásemos": "desacordar",
    "desacordé": "desacordar",
    "desacordéis": "desacordar",
    "desacordó": "desacordar",
    "desacorralamientos": "desacorralamiento",
    "desacostumbra": "desacostumbrar",
    "desacostumbraba": "desacostumbrar",
    "desacostumbrabais": "desacostumbrar",
    "desacostumbraban": "desacostumbrar",
    "desacostumbrabas": "desacostumbrar",
    "desacostumbrad": "desacostumbrar",
    "desacostumbrada": "desacostumbrar",
    "desacostumbradas": "desacostumbrar",
    "desacostumbrado": "desacostumbrar",
    "desacostumbrados": "desacostumbrar",
    "desacostumbramientos": "desacostumbramiento",
    "desacostumbramos": "desacostumbrar",
    "desacostumbran": "desacostumbrar",
    "desacostumbrando": "desacostumbrar",
    "desacostumbrara": "desacostumbrar",
    "desacostumbrarais": "desacostumbrar",
    "desacostumbraran": "desacostumbrar",
    "desacostumbraras": "desacostumbrar",
    "desacostumbrare": "desacostumbrar",
    "desacostumbrareis": "desacostumbrar",
    "desacostumbraremos": "desacostumbrar",
    "desacostumbraren": "desacostumbrar",
    "desacostumbrares": "desacostumbrar",
    "desacostumbraron": "desacostumbrar",
    "desacostumbrará": "desacostumbrar",
    "desacostumbrarán": "desacostumbrar",
    "desacostumbrarás": "desacostumbrar",
    "desacostumbraré": "desacostumbrar",
    "desacostumbraréis": "desacostumbrar",
    "desacostumbraría": "desacostumbrar",
    "desacostumbraríais": "desacostumbrar",
    "desacostumbraríamos": "desacostumbrar",
    "desacostumbrarían": "desacostumbrar",
    "desacostumbrarías": "desacostumbrar",
    "desacostumbras": "desacostumbrar",
    "desacostumbrase": "desacostumbrar",
    "desacostumbraseis": "desacostumbrar",
    "desacostumbrasen": "desacostumbrar",
    "desacostumbrases": "desacostumbrar",
    "desacostumbraste": "desacostumbrar",
    "desacostumbrasteis": "desacostumbrar",
    "desacostumbre": "desacostumbrar",
    "desacostumbremos": "desacostumbrar",
    "desacostumbren": "desacostumbrar",
    "desacostumbres": "desacostumbrar",
    "desacostumbro": "desacostumbrar",
    "desacostumbrábamos": "desacostumbrar",
    "desacostumbráis": "desacostumbrar",
    "desacostumbráramos": "desacostumbrar",
    "desacostumbráremos": "desacostumbrar",
    "desacostumbrásemos": "desacostumbrar",
    "desacostumbré": "desacostumbrar",
    "desacostumbréis": "desacostumbrar",
    "desacostumbró": "desacostumbrar",
    "desacota": "desacotar",
    "desacotaba": "desacotar",
    "desacotabais": "desacotar",
    "desacotaban": "desacotar",
    "desacotabas": "desacotar",
    "desacotad": "desacotar",
    "desacotada": "desacotar",
    "desacotadas": "desacotar",
    "desacotado": "desacotar",
    "desacotados": "desacotar",
    "desacotamos": "desacotar",
    "desacotan": "desacotar",
    "desacotando": "desacotar",
    "desacotara": "desacotar",
    "desacotarais": "desacotar",
    "desacotaran": "desacotar",
    "desacotaras": "desacotar",
    "desacotare": "desacotar",
    "desacotareis": "desacotar",
    "desacotaremos": "desacotar",
    "desacotaren": "desacotar",
    "desacotares": "desacotar",
    "desacotaron": "desacotar",
    "desacotará": "desacotar",
    "desacotarán": "desacotar",
    "desacotarás": "desacotar",
    "desacotaré": "desacotar",
    "desacotaréis": "desacotar",
    "desacotaría": "desacotar",
    "desacotaríais": "desacotar",
    "desacotaríamos": "desacotar",
    "desacotarían": "desacotar",
    "desacotarías": "desacotar",
    "desacotas": "desacotar",
    "desacotase": "desacotar",
    "desacotaseis": "desacotar",
    "desacotasen": "desacotar",
    "desacotases": "desacotar",
    "desacotaste": "desacotar",
    "desacotasteis": "desacotar",
    "desacote": "desacotar",
    "desacotemos": "desacotar",
    "desacoten": "desacotar",
    "desacotes": "desacotar",
    "desacoto": "desacotar",
    "desacotos": "desacoto",
    "desacotábamos": "desacotar",
    "desacotáis": "desacotar",
    "desacotáramos": "desacotar",
    "desacotáremos": "desacotar",
    "desacotásemos": "desacotar",
    "desacoté": "desacotar",
    "desacotéis": "desacotar",
    "desacotó": "desacotar",
    "desacredita": "desacreditar",
    "desacreditaba": "desacreditar",
    "desacreditabais": "desacreditar",
    "desacreditaban": "desacreditar",
    "desacreditabas": "desacreditar",
    "desacreditaciones": "desacreditación",
    "desacreditad": "desacreditar",
    "desacreditada": "desacreditar",
    "desacreditadas": "desacreditar",
    "desacreditado": "desacreditar",
    "desacreditados": "desacreditar",
    "desacreditamos": "desacreditar",
    "desacreditan": "desacreditar",
    "desacreditando": "desacreditar",
    "desacreditara": "desacreditar",
    "desacreditarais": "desacreditar",
    "desacreditaran": "desacreditar",
    "desacreditaras": "desacreditar",
    "desacreditare": "desacreditar",
    "desacreditareis": "desacreditar",
    "desacreditaremos": "desacreditar",
    "desacreditaren": "desacreditar",
    "desacreditares": "desacreditar",
    "desacreditaron": "desacreditar",
    "desacreditará": "desacreditar",
    "desacreditarán": "desacreditar",
    "desacreditarás": "desacreditar",
    "desacreditaré": "desacreditar",
    "desacreditaréis": "desacreditar",
    "desacreditaría": "desacreditar",
    "desacreditaríais": "desacreditar",
    "desacreditaríamos": "desacreditar",
    "desacreditarían": "desacreditar",
    "desacreditarías": "desacreditar",
    "desacreditas": "desacreditar",
    "desacreditase": "desacreditar",
    "desacreditaseis": "desacreditar",
    "desacreditasen": "desacreditar",
    "desacreditases": "desacreditar",
    "desacreditaste": "desacreditar",
    "desacreditasteis": "desacreditar",
    "desacredite": "desacreditar",
    "desacreditemos": "desacreditar",
    "desacrediten": "desacreditar",
    "desacredites": "desacreditar",
    "desacredito": "desacreditar",
    "desacreditábamos": "desacreditar",
    "desacreditáis": "desacreditar",
    "desacreditáramos": "desacreditar",
    "desacreditáremos": "desacreditar",
    "desacreditásemos": "desacreditar",
    "desacredité": "desacreditar",
    "desacreditéis": "desacreditar",
    "desacreditó": "desacreditar",
    "desactiva": "desactivar",
    "desactivaba": "desactivar",
    "desactivabais": "desactivar",
    "desactivaban": "desactivar",
    "desactivabas": "desactivar",
    "desactivaciones": "desactivación",
    "desactivad": "desactivar",
    "desactivada": "desactivar",
    "desactivadas": "desactivar",
    "desactivado": "desactivar",
    "desactivados": "desactivar",
    "desactivamos": "desactivar",
    "desactivan": "desactivar",
    "desactivando": "desactivar",
    "desactivara": "desactivar",
    "desactivarais": "desactivar",
    "desactivaran": "desactivar",
    "desactivaras": "desactivar",
    "desactivare": "desactivar",
    "desactivareis": "desactivar",
    "desactivaremos": "desactivar",
    "desactivaren": "desactivar",
    "desactivares": "desactivar",
    "desactivaron": "desactivar",
    "desactivará": "desactivar",
    "desactivarán": "desactivar",
    "desactivarás": "desactivar",
    "desactivaré": "desactivar",
    "desactivaréis": "desactivar",
    "desactivaría": "desactivar",
    "desactivaríais": "desactivar",
    "desactivaríamos": "desactivar",
    "desactivarían": "desactivar",
    "desactivarías": "desactivar",
    "desactivas": "desactivar",
    "desactivase": "desactivar",
    "desactivaseis": "desactivar",
    "desactivasen": "desactivar",
    "desactivases": "desactivar",
    "desactivaste": "desactivar",
    "desactivasteis": "desactivar",
    "desactive": "desactivar",
    "desactivemos": "desactivar",
    "desactiven": "desactivar",
    "desactives": "desactivar",
    "desactivo": "desactivar",
    "desactivábamos": "desactivar",
    "desactiváis": "desactivar",
    "desactiváramos": "desactivar",
    "desactiváremos": "desactivar",
    "desactivásemos": "desactivar",
    "desactivé": "desactivar",
    "desactivéis": "desactivar",
    "desactivó": "desactivar",
    "desacuartelamientos": "desacuartelamiento",
    "desacuerda": "desacordar",
    "desacuerdan": "desacordar",
    "desacuerdas": "desacordar",
    "desacuerde": "desacordar",
    "desacuerden": "desacordar",
    "desacuerdes": "desacordar",
    "desacuerdo": "desacordar",
    "desacuerdos": "desacuerdo",
    "desadeuda": "desadeudar",
    "desadeudaba": "desadeudar",
    "desadeudabais": "desadeudar",
    "desadeudaban": "desadeudar",
    "desadeudabas": "desadeudar",
    "desadeudad": "desadeudar",
    "desadeudada": "desadeudar",
    "desadeudadas": "desadeudar",
    "desadeudado": "desadeudar",
    "desadeudados": "desadeudar",
    "desadeudamos": "desadeudar",
    "desadeudan": "desadeudar",
    "desadeudando": "desadeudar",
    "desadeudara": "desadeudar",
    "desadeudarais": "desadeudar",
    "desadeudaran": "desadeudar",
    "desadeudaras": "desadeudar",
    "desadeudare": "desadeudar",
    "desadeudareis": "desadeudar",
    "desadeudaremos": "desadeudar",
    "desadeudaren": "desadeudar",
    "desadeudares": "desadeudar",
    "desadeudaron": "desadeudar",
    "desadeudará": "desadeudar",
    "desadeudarán": "desadeudar",
    "desadeudarás": "desadeudar",
    "desadeudaré": "desadeudar",
    "desadeudaréis": "desadeudar",
    "desadeudaría": "desadeudar",
    "desadeudaríais": "desadeudar",
    "desadeudaríamos": "desadeudar",
    "desadeudarían": "desadeudar",
    "desadeudarías": "desadeudar",
    "desadeudas": "desadeudar",
    "desadeudase": "desadeudar",
    "desadeudaseis": "desadeudar",
    "desadeudasen": "desadeudar",
    "desadeudases": "desadeudar",
    "desadeudaste": "desadeudar",
    "desadeudasteis": "desadeudar",
    "desadeude": "desadeudar",
    "desadeudemos": "desadeudar",
    "desadeuden": "desadeudar",
    "desadeudes": "desadeudar",
    "desadeudo": "desadeudar",
    "desadeudábamos": "desadeudar",
    "desadeudáis": "desadeudar",
    "desadeudáramos": "desadeudar",
    "desadeudáremos": "desadeudar",
    "desadeudásemos": "desadeudar",
    "desadeudé": "desadeudar",
    "desadeudéis": "desadeudar",
    "desadeudó": "desadeudar",
    "desadormece": "desadormecer",
    "desadormeced": "desadormecer",
    "desadormecemos": "desadormecer",
    "desadormecen": "desadormecer",
    "desadormeceremos": "desadormecer",
    "desadormecerá": "desadormecer",
    "desadormecerán": "desadormecer",
    "desadormecerás": "desadormecer",
    "desadormeceré": "desadormecer",
    "desadormeceréis": "desadormecer",
    "desadormecería": "desadormecer",
    "desadormeceríais": "desadormecer",
    "desadormeceríamos": "desadormecer",
    "desadormecerían": "desadormecer",
    "desadormecerías": "desadormecer",
    "desadormeces": "desadormecer",
    "desadormecida": "desadormecer",
    "desadormecidas": "desadormecer",
    "desadormecido": "desadormecer",
    "desadormecidos": "desadormecer",
    "desadormeciendo": "desadormecer",
    "desadormeciera": "desadormecer",
    "desadormecierais": "desadormecer",
    "desadormecieran": "desadormecer",
    "desadormecieras": "desadormecer",
    "desadormeciere": "desadormecer",
    "desadormeciereis": "desadormecer",
    "desadormecieren": "desadormecer",
    "desadormecieres": "desadormecer",
    "desadormecieron": "desadormecer",
    "desadormeciese": "desadormecer",
    "desadormecieseis": "desadormecer",
    "desadormeciesen": "desadormecer",
    "desadormecieses": "desadormecer",
    "desadormecimos": "desadormecer",
    "desadormeciste": "desadormecer",
    "desadormecisteis": "desadormecer",
    "desadormeciéramos": "desadormecer",
    "desadormeciéremos": "desadormecer",
    "desadormeciésemos": "desadormecer",
    "desadormeció": "desadormecer",
    "desadormecéis": "desadormecer",
    "desadormecí": "desadormecer",
    "desadormecía": "desadormecer",
    "desadormecíais": "desadormecer",
    "desadormecíamos": "desadormecer",
    "desadormecían": "desadormecer",
    "desadormecías": "desadormecer",
    "desadormezca": "desadormecer",
    "desadormezcamos": "desadormecer",
    "desadormezcan": "desadormecer",
    "desadormezcas": "desadormecer",
    "desadormezco": "desadormecer",
    "desadormezcáis": "desadormecer",
    "desadorna": "desadornar",
    "desadornaba": "desadornar",
    "desadornabais": "desadornar",
    "desadornaban": "desadornar",
    "desadornabas": "desadornar",
    "desadornad": "desadornar",
    "desadornada": "desadornar",
    "desadornadas": "desadornar",
    "desadornado": "desadornar",
    "desadornados": "desadornar",
    "desadornamos": "desadornar",
    "desadornan": "desadornar",
    "desadornando": "desadornar",
    "desadornara": "desadornar",
    "desadornarais": "desadornar",
    "desadornaran": "desadornar",
    "desadornaras": "desadornar",
    "desadornare": "desadornar",
    "desadornareis": "desadornar",
    "desadornaremos": "desadornar",
    "desadornaren": "desadornar",
    "desadornares": "desadornar",
    "desadornaron": "desadornar",
    "desadornará": "desadornar",
    "desadornarán": "desadornar",
    "desadornarás": "desadornar",
    "desadornaré": "desadornar",
    "desadornaréis": "desadornar",
    "desadornaría": "desadornar",
    "desadornaríais": "desadornar",
    "desadornaríamos": "desadornar",
    "desadornarían": "desadornar",
    "desadornarías": "desadornar",
    "desadornas": "desadornar",
    "desadornase": "desadornar",
    "desadornaseis": "desadornar",
    "desadornasen": "desadornar",
    "desadornases": "desadornar",
    "desadornaste": "desadornar",
    "desadornasteis": "desadornar",
    "desadorne": "desadornar",
    "desadornemos": "desadornar",
    "desadornen": "desadornar",
    "desadornes": "desadornar",
    "desadorno": "desadornar",
    "desadornos": "desadorno",
    "desadornábamos": "desadornar",
    "desadornáis": "desadornar",
    "desadornáramos": "desadornar",
    "desadornáremos": "desadornar",
    "desadornásemos": "desadornar",
    "desadorné": "desadornar",
    "desadornéis": "desadornar",
    "desadornó": "desadornar",
    "desadvertid": "desadvertir",
    "desadvertida": "desadvertir",
    "desadvertidas": "desadvertir",
    "desadvertido": "desadvertir",
    "desadvertidos": "desadvertir",
    "desadvertimos": "desadvertir",
    "desadvertiremos": "desadvertir",
    "desadvertirá": "desadvertir",
    "desadvertirán": "desadvertir",
    "desadvertirás": "desadvertir",
    "desadvertiré": "desadvertir",
    "desadvertiréis": "desadvertir",
    "desadvertiría": "desadvertir",
    "desadvertiríais": "desadvertir",
    "desadvertiríamos": "desadvertir",
    "desadvertirían": "desadvertir",
    "desadvertirías": "desadvertir",
    "desadvertiste": "desadvertir",
    "desadvertisteis": "desadvertir",
    "desadvertí": "desadvertir",
    "desadvertía": "desadvertir",
    "desadvertíais": "desadvertir",
    "desadvertíamos": "desadvertir",
    "desadvertían": "desadvertir",
    "desadvertías": "desadvertir",
    "desadvertís": "desadvertir",
    "desadvierta": "desadvertir",
    "desadviertan": "desadvertir",
    "desadviertas": "desadvertir",
    "desadvierte": "desadvertir",
    "desadvierten": "desadvertir",
    "desadviertes": "desadvertir",
    "desadvierto": "desadvertir",
    "desadvirtamos": "desadvertir",
    "desadvirtiendo": "desadvertir",
    "desadvirtiera": "desadvertir",
    "desadvirtierais": "desadvertir",
    "desadvirtieran": "desadvertir",
    "desadvirtieras": "desadvertir",
    "desadvirtiere": "desadvertir",
    "desadvirtiereis": "desadvertir",
    "desadvirtieren": "desadvertir",
    "desadvirtieres": "desadvertir",
    "desadvirtieron": "desadvertir",
    "desadvirtiese": "desadvertir",
    "desadvirtieseis": "desadvertir",
    "desadvirtiesen": "desadvertir",
    "desadvirtieses": "desadvertir",
    "desadvirtiéramos": "desadvertir",
    "desadvirtiéremos": "desadvertir",
    "desadvirtiésemos": "desadvertir",
    "desadvirtió": "desadvertir",
    "desadvirtáis": "desadvertir",
    "desaeraciones": "desaeración",
    "desafea": "desafear",
    "desafeaba": "desafear",
    "desafeabais": "desafear",
    "desafeaban": "desafear",
    "desafeabas": "desafear",
    "desafead": "desafear",
    "desafeada": "desafear",
    "desafeadas": "desafear",
    "desafeado": "desafear",
    "desafeados": "desafear",
    "desafeamos": "desafear",
    "desafean": "desafear",
    "desafeando": "desafear",
    "desafeara": "desafear",
    "desafearais": "desafear",
    "desafearan": "desafear",
    "desafearas": "desafear",
    "desafeare": "desafear",
    "desafeareis": "desafear",
    "desafearemos": "desafear",
    "desafearen": "desafear",
    "desafeares": "desafear",
    "desafearon": "desafear",
    "desafeará": "desafear",
    "desafearán": "desafear",
    "desafearás": "desafear",
    "desafearé": "desafear",
    "desafearéis": "desafear",
    "desafearía": "desafear",
    "desafearíais": "desafear",
    "desafearíamos": "desafear",
    "desafearían": "desafear",
    "desafearías": "desafear",
    "desafeas": "desafear",
    "desafease": "desafear",
    "desafeaseis": "desafear",
    "desafeasen": "desafear",
    "desafeases": "desafear",
    "desafeaste": "desafear",
    "desafeasteis": "desafear",
    "desafecciones": "desafección",
    "desafecta": "desafecto",
    "desafectas": "desafecto",
    "desafectos": "desafecto",
    "desafee": "desafear",
    "desafeemos": "desafear",
    "desafeen": "desafear",
    "desafees": "desafear",
    "desafeo": "desafear",
    "desaferra": "desaferrar",
    "desaferraba": "desaferrar",
    "desaferrabais": "desaferrar",
    "desaferraban": "desaferrar",
    "desaferrabas": "desaferrar",
    "desaferrad": "desaferrar",
    "desaferrada": "desaferrar",
    "desaferradas": "desaferrar",
    "desaferrado": "desaferrar",
    "desaferrados": "desaferrar",
    "desaferramos": "desaferrar",
    "desaferran": "desaferrar",
    "desaferrando": "desaferrar",
    "desaferrara": "desaferrar",
    "desaferrarais": "desaferrar",
    "desaferraran": "desaferrar",
    "desaferraras": "desaferrar",
    "desaferrare": "desaferrar",
    "desaferrareis": "desaferrar",
    "desaferraremos": "desaferrar",
    "desaferraren": "desaferrar",
    "desaferrares": "desaferrar",
    "desaferraron": "desaferrar",
    "desaferrará": "desaferrar",
    "desaferrarán": "desaferrar",
    "desaferrarás": "desaferrar",
    "desaferraré": "desaferrar",
    "desaferraréis": "desaferrar",
    "desaferraría": "desaferrar",
    "desaferraríais": "desaferrar",
    "desaferraríamos": "desaferrar",
    "desaferrarían": "desaferrar",
    "desaferrarías": "desaferrar",
    "desaferras": "desaferrar",
    "desaferrase": "desaferrar",
    "desaferraseis": "desaferrar",
    "desaferrasen": "desaferrar",
    "desaferrases": "desaferrar",
    "desaferraste": "desaferrar",
    "desaferrasteis": "desaferrar",
    "desaferre": "desaferrar",
    "desaferremos": "desaferrar",
    "desaferren": "desaferrar",
    "desaferres": "desaferrar",
    "desaferro": "desaferrar",
    "desaferrábamos": "desaferrar",
    "desaferráis": "desaferrar",
    "desaferráramos": "desaferrar",
    "desaferráremos": "desaferrar",
    "desaferrásemos": "desaferrar",
    "desaferré": "desaferrar",
    "desaferréis": "desaferrar",
    "desaferró": "desaferrar",
    "desafeábamos": "desafear",
    "desafeáis": "desafear",
    "desafeáramos": "desafear",
    "desafeáremos": "desafear",
    "desafeásemos": "desafear",
    "desafeé": "desafear",
    "desafeéis": "desafear",
    "desafeó": "desafear",
    "desafiaba": "desafiar",
    "desafiabais": "desafiar",
    "desafiaban": "desafiar",
    "desafiabas": "desafiar",
    "desafiad": "desafiar",
    "desafiada": "desafiar",
    "desafiadas": "desafiar",
    "desafiado": "desafiar",
    "desafiadora": "desafiador",
    "desafiadoras": "desafiador",
    "desafiadores": "desafiador",
    "desafiados": "desafiar",
    "desafiamos": "desafiar",
    "desafiando": "desafiar",
    "desafiantes": "desafiante",
    "desafiara": "desafiar",
    "desafiarais": "desafiar",
    "desafiaran": "desafiar",
    "desafiaras": "desafiar",
    "desafiare": "desafiar",
    "desafiareis": "desafiar",
    "desafiaremos": "desafiar",
    "desafiaren": "desafiar",
    "desafiares": "desafiar",
    "desafiaron": "desafiar",
    "desafiará": "desafiar",
    "desafiarán": "desafiar",
    "desafiarás": "desafiar",
    "desafiaré": "desafiar",
    "desafiaréis": "desafiar",
    "desafiaría": "desafiar",
    "desafiaríais": "desafiar",
    "desafiaríamos": "desafiar",
    "desafiarían": "desafiar",
    "desafiarías": "desafiar",
    "desafiase": "desafiar",
    "desafiaseis": "desafiar",
    "desafiasen": "desafiar",
    "desafiases": "desafiar",
    "desafiaste": "desafiar",
    "desafiasteis": "desafiar",
    "desaficiona": "desaficionar",
    "desaficionaba": "desaficionar",
    "desaficionabais": "desaficionar",
    "desaficionaban": "desaficionar",
    "desaficionabas": "desaficionar",
    "desaficionad": "desaficionar",
    "desaficionada": "desaficionar",
    "desaficionadas": "desaficionar",
    "desaficionado": "desaficionar",
    "desaficionados": "desaficionar",
    "desaficionamos": "desaficionar",
    "desaficionan": "desaficionar",
    "desaficionando": "desaficionar",
    "desaficionara": "desaficionar",
    "desaficionarais": "desaficionar",
    "desaficionaran": "desaficionar",
    "desaficionaras": "desaficionar",
    "desaficionare": "desaficionar",
    "desaficionareis": "desaficionar",
    "desaficionaremos": "desaficionar",
    "desaficionaren": "desaficionar",
    "desaficionares": "desaficionar",
    "desaficionaron": "desaficionar",
    "desaficionará": "desaficionar",
    "desaficionarán": "desaficionar",
    "desaficionarás": "desaficionar",
    "desaficionaré": "desaficionar",
    "desaficionaréis": "desaficionar",
    "desaficionaría": "desaficionar",
    "desaficionaríais": "desaficionar",
    "desaficionaríamos": "desaficionar",
    "desaficionarían": "desaficionar",
    "desaficionarías": "desaficionar",
    "desaficionas": "desaficionar",
    "desaficionase": "desaficionar",
    "desaficionaseis": "desaficionar",
    "desaficionasen": "desaficionar",
    "desaficionases": "desaficionar",
    "desaficionaste": "desaficionar",
    "desaficionasteis": "desaficionar",
    "desaficione": "desaficionar",
    "desaficionemos": "desaficionar",
    "desaficionen": "desaficionar",
    "desaficiones": "desaficionar",
    "desaficiono": "desaficionar",
    "desaficionábamos": "desaficionar",
    "desaficionáis": "desaficionar",
    "desaficionáramos": "desaficionar",
    "desaficionáremos": "desaficionar",
    "desaficionásemos": "desaficionar",
    "desaficioné": "desaficionar",
    "desaficionéis": "desaficionar",
    "desaficionó": "desaficionar",
    "desafiemos": "desafiar",
    "desafila": "desafilar",
    "desafilaba": "desafilar",
    "desafilabais": "desafilar",
    "desafilaban": "desafilar",
    "desafilabas": "desafilar",
    "desafilad": "desafilar",
    "desafilada": "desafilar",
    "desafiladas": "desafilar",
    "desafilado": "desafilar",
    "desafilados": "desafilar",
    "desafilamientos": "desafilamiento",
    "desafilamos": "desafilar",
    "desafilan": "desafilar",
    "desafilando": "desafilar",
    "desafilara": "desafilar",
    "desafilarais": "desafilar",
    "desafilaran": "desafilar",
    "desafilaras": "desafilar",
    "desafilare": "desafilar",
    "desafilareis": "desafilar",
    "desafilaremos": "desafilar",
    "desafilaren": "desafilar",
    "desafilares": "desafilar",
    "desafilaron": "desafilar",
    "desafilará": "desafilar",
    "desafilarán": "desafilar",
    "desafilarás": "desafilar",
    "desafilaré": "desafilar",
    "desafilaréis": "desafilar",
    "desafilaría": "desafilar",
    "desafilaríais": "desafilar",
    "desafilaríamos": "desafilar",
    "desafilarían": "desafilar",
    "desafilarías": "desafilar",
    "desafilas": "desafilar",
    "desafilase": "desafilar",
    "desafilaseis": "desafilar",
    "desafilasen": "desafilar",
    "desafilases": "desafilar",
    "desafilaste": "desafilar",
    "desafilasteis": "desafilar",
    "desafile": "desafilar",
    "desafilemos": "desafilar",
    "desafilen": "desafilar",
    "desafiles": "desafilar",
    "desafilo": "desafilar",
    "desafilábamos": "desafilar",
    "desafiláis": "desafilar",
    "desafiláramos": "desafilar",
    "desafiláremos": "desafilar",
    "desafilásemos": "desafilar",
    "desafilé": "desafilar",
    "desafiléis": "desafilar",
    "desafiló": "desafilar",
    "desafina": "desafinar",
    "desafinaba": "desafinar",
    "desafinabais": "desafinar",
    "desafinaban": "desafinar",
    "desafinabas": "desafinar",
    "desafinaciones": "desafinación",
    "desafinad": "desafinar",
    "desafinada": "desafinar",
    "desafinadas": "desafinar",
    "desafinado": "desafinar",
    "desafinados": "desafinar",
    "desafinamos": "desafinar",
    "desafinan": "desafinar",
    "desafinando": "desafinar",
    "desafinara": "desafinar",
    "desafinarais": "desafinar",
    "desafinaran": "desafinar",
    "desafinaras": "desafinar",
    "desafinare": "desafinar",
    "desafinareis": "desafinar",
    "desafinaremos": "desafinar",
    "desafinaren": "desafinar",
    "desafinares": "desafinar",
    "desafinaron": "desafinar",
    "desafinará": "desafinar",
    "desafinarán": "desafinar",
    "desafinarás": "desafinar",
    "desafinaré": "desafinar",
    "desafinaréis": "desafinar",
    "desafinaría": "desafinar",
    "desafinaríais": "desafinar",
    "desafinaríamos": "desafinar",
    "desafinarían": "desafinar",
    "desafinarías": "desafinar",
    "desafinas": "desafinar",
    "desafinase": "desafinar",
    "desafinaseis": "desafinar",
    "desafinasen": "desafinar",
    "desafinases": "desafinar",
    "desafinaste": "desafinar",
    "desafinasteis": "desafinar",
    "desafine": "desafinar",
    "desafinemos": "desafinar",
    "desafinen": "desafinar",
    "desafines": "desafinar",
    "desafino": "desafinar",
    "desafinábamos": "desafinar",
    "desafináis": "desafinar",
    "desafináramos": "desafinar",
    "desafináremos": "desafinar",
    "desafinásemos": "desafinar",
    "desafiné": "desafinar",
    "desafinéis": "desafinar",
    "desafinó": "desafinar",
    "desafiábamos": "desafiar",
    "desafiáis": "desafiar",
    "desafiáramos": "desafiar",
    "desafiáremos": "desafiar",
    "desafiásemos": "desafiar",
    "desafié": "desafiar",
    "desafiéis": "desafiar",
    "desafió": "desafiar",
    "desaforaba": "desaforar",
    "desaforabais": "desaforar",
    "desaforaban": "desaforar",
    "desaforabas": "desaforar",
    "desaforad": "desaforar",
    "desaforada": "desaforar",
    "desaforadas": "desaforar",
    "desaforado": "desaforar",
    "desaforados": "desaforar",
    "desaforamos": "desaforar",
    "desaforando": "desaforar",
    "desaforara": "desaforar",
    "desaforarais": "desaforar",
    "desaforaran": "desaforar",
    "desaforaras": "desaforar",
    "desaforare": "desaforar",
    "desaforareis": "desaforar",
    "desaforaremos": "desaforar",
    "desaforaren": "desaforar",
    "desaforares": "desaforar",
    "desaforaron": "desaforar",
    "desaforará": "desaforar",
    "desaforarán": "desaforar",
    "desaforarás": "desaforar",
    "desaforaré": "desaforar",
    "desaforaréis": "desaforar",
    "desaforaría": "desaforar",
    "desaforaríais": "desaforar",
    "desaforaríamos": "desaforar",
    "desaforarían": "desaforar",
    "desaforarías": "desaforar",
    "desaforase": "desaforar",
    "desaforaseis": "desaforar",
    "desaforasen": "desaforar",
    "desaforases": "desaforar",
    "desaforaste": "desaforar",
    "desaforasteis": "desaforar",
    "desaforemos": "desaforar",
    "desaforra": "desaforrar",
    "desaforraba": "desaforrar",
    "desaforrabais": "desaforrar",
    "desaforraban": "desaforrar",
    "desaforrabas": "desaforrar",
    "desaforrad": "desaforrar",
    "desaforrada": "desaforrar",
    "desaforradas": "desaforrar",
    "desaforrado": "desaforrar",
    "desaforrados": "desaforrar",
    "desaforramos": "desaforrar",
    "desaforran": "desaforrar",
    "desaforrando": "desaforrar",
    "desaforrara": "desaforrar",
    "desaforrarais": "desaforrar",
    "desaforraran": "desaforrar",
    "desaforraras": "desaforrar",
    "desaforrare": "desaforrar",
    "desaforrareis": "desaforrar",
    "desaforraremos": "desaforrar",
    "desaforraren": "desaforrar",
    "desaforrares": "desaforrar",
    "desaforraron": "desaforrar",
    "desaforrará": "desaforrar",
    "desaforrarán": "desaforrar",
    "desaforrarás": "desaforrar",
    "desaforraré": "desaforrar",
    "desaforraréis": "desaforrar",
    "desaforraría": "desaforrar",
    "desaforraríais": "desaforrar",
    "desaforraríamos": "desaforrar",
    "desaforrarían": "desaforrar",
    "desaforrarías": "desaforrar",
    "desaforras": "desaforrar",
    "desaforrase": "desaforrar",
    "desaforraseis": "desaforrar",
    "desaforrasen": "desaforrar",
    "desaforrases": "desaforrar",
    "desaforraste": "desaforrar",
    "desaforrasteis": "desaforrar",
    "desaforre": "desaforrar",
    "desaforremos": "desaforrar",
    "desaforren": "desaforrar",
    "desaforres": "desaforrar",
    "desaforro": "desaforrar",
    "desaforrábamos": "desaforrar",
    "desaforráis": "desaforrar",
    "desaforráramos": "desaforrar",
    "desaforráremos": "desaforrar",
    "desaforrásemos": "desaforrar",
    "desaforré": "desaforrar",
    "desaforréis": "desaforrar",
    "desaforró": "desaforrar",
    "desafortunada": "desafortunado",
    "desafortunadas": "desafortunado",
    "desafortunados": "desafortunado",
    "desaforábamos": "desaforar",
    "desaforáis": "desaforar",
    "desaforáramos": "desaforar",
    "desaforáremos": "desaforar",
    "desaforásemos": "desaforar",
    "desaforé": "desaforar",
    "desaforéis": "desaforar",
    "desaforó": "desaforar",
    "desafuera": "desaforar",
    "desafueran": "desaforar",
    "desafueras": "desaforar",
    "desafuere": "desaforar",
    "desafueren": "desaforar",
    "desafueres": "desaforar",
    "desafuero": "desaforar",
    "desafueros": "desafuero",
    "desafía": "desafiar",
    "desafían": "desafiar",
    "desafías": "desafiar",
    "desafíe": "desafiar",
    "desafíen": "desafiar",
    "desafíes": "desafiar",
    "desafío": "desafiar",
    "desafíos": "desafío",
    "desagarra": "desagarrar",
    "desagarraba": "desagarrar",
    "desagarrabais": "desagarrar",
    "desagarraban": "desagarrar",
    "desagarrabas": "desagarrar",
    "desagarrad": "desagarrar",
    "desagarrada": "desagarrar",
    "desagarradas": "desagarrar",
    "desagarrado": "desagarrar",
    "desagarrados": "desagarrar",
    "desagarramos": "desagarrar",
    "desagarran": "desagarrar",
    "desagarrando": "desagarrar",
    "desagarrara": "desagarrar",
    "desagarrarais": "desagarrar",
    "desagarraran": "desagarrar",
    "desagarraras": "desagarrar",
    "desagarrare": "desagarrar",
    "desagarrareis": "desagarrar",
    "desagarraremos": "desagarrar",
    "desagarraren": "desagarrar",
    "desagarrares": "desagarrar",
    "desagarraron": "desagarrar",
    "desagarrará": "desagarrar",
    "desagarrarán": "desagarrar",
    "desagarrarás": "desagarrar",
    "desagarraré": "desagarrar",
    "desagarraréis": "desagarrar",
    "desagarraría": "desagarrar",
    "desagarraríais": "desagarrar",
    "desagarraríamos": "desagarrar",
    "desagarrarían": "desagarrar",
    "desagarrarías": "desagarrar",
    "desagarras": "desagarrar",
    "desagarrase": "desagarrar",
    "desagarraseis": "desagarrar",
    "desagarrasen": "desagarrar",
    "desagarrases": "desagarrar",
    "desagarraste": "desagarrar",
    "desagarrasteis": "desagarrar",
    "desagarre": "desagarrar",
    "desagarremos": "desagarrar",
    "desagarren": "desagarrar",
    "desagarres": "desagarrar",
    "desagarro": "desagarrar",
    "desagarrábamos": "desagarrar",
    "desagarráis": "desagarrar",
    "desagarráramos": "desagarrar",
    "desagarráremos": "desagarrar",
    "desagarrásemos": "desagarrar",
    "desagarré": "desagarrar",
    "desagarréis": "desagarrar",
    "desagarró": "desagarrar",
    "desagracia": "desagraciar",
    "desagraciaba": "desagraciar",
    "desagraciabais": "desagraciar",
    "desagraciaban": "desagraciar",
    "desagraciabas": "desagraciar",
    "desagraciad": "desagraciar",
    "desagraciada": "desagraciar",
    "desagraciadas": "desagraciar",
    "desagraciado": "desagraciar",
    "desagraciados": "desagraciar",
    "desagraciamos": "desagraciar",
    "desagracian": "desagraciar",
    "desagraciando": "desagraciar",
    "desagraciara": "desagraciar",
    "desagraciarais": "desagraciar",
    "desagraciaran": "desagraciar",
    "desagraciaras": "desagraciar",
    "desagraciare": "desagraciar",
    "desagraciareis": "desagraciar",
    "desagraciaremos": "desagraciar",
    "desagraciaren": "desagraciar",
    "desagraciares": "desagraciar",
    "desagraciaron": "desagraciar",
    "desagraciará": "desagraciar",
    "desagraciarán": "desagraciar",
    "desagraciarás": "desagraciar",
    "desagraciaré": "desagraciar",
    "desagraciaréis": "desagraciar",
    "desagraciaría": "desagraciar",
    "desagraciaríais": "desagraciar",
    "desagraciaríamos": "desagraciar",
    "desagraciarían": "desagraciar",
    "desagraciarías": "desagraciar",
    "desagracias": "desagraciar",
    "desagraciase": "desagraciar",
    "desagraciaseis": "desagraciar",
    "desagraciasen": "desagraciar",
    "desagraciases": "desagraciar",
    "desagraciaste": "desagraciar",
    "desagraciasteis": "desagraciar",
    "desagracie": "desagraciar",
    "desagraciemos": "desagraciar",
    "desagracien": "desagraciar",
    "desagracies": "desagraciar",
    "desagracio": "desagraciar",
    "desagraciábamos": "desagraciar",
    "desagraciáis": "desagraciar",
    "desagraciáramos": "desagraciar",
    "desagraciáremos": "desagraciar",
    "desagraciásemos": "desagraciar",
    "desagracié": "desagraciar",
    "desagraciéis": "desagraciar",
    "desagració": "desagraciar",
    "desagrada": "desagradar",
    "desagradaba": "desagradar",
    "desagradabais": "desagradar",
    "desagradaban": "desagradar",
    "desagradabas": "desagradar",
    "desagradables": "desagradable",
    "desagradad": "desagradar",
    "desagradada": "desagradar",
    "desagradadas": "desagradar",
    "desagradado": "desagradar",
    "desagradados": "desagradar",
    "desagradamos": "desagradar",
    "desagradan": "desagradar",
    "desagradando": "desagradar",
    "desagradara": "desagradar",
    "desagradarais": "desagradar",
    "desagradaran": "desagradar",
    "desagradaras": "desagradar",
    "desagradare": "desagradar",
    "desagradareis": "desagradar",
    "desagradaremos": "desagradar",
    "desagradaren": "desagradar",
    "desagradares": "desagradar",
    "desagradaron": "desagradar",
    "desagradará": "desagradar",
    "desagradarán": "desagradar",
    "desagradarás": "desagradar",
    "desagradaré": "desagradar",
    "desagradaréis": "desagradar",
    "desagradaría": "desagradar",
    "desagradaríais": "desagradar",
    "desagradaríamos": "desagradar",
    "desagradarían": "desagradar",
    "desagradarías": "desagradar",
    "desagradas": "desagradar",
    "desagradase": "desagradar",
    "desagradaseis": "desagradar",
    "desagradasen": "desagradar",
    "desagradases": "desagradar",
    "desagradaste": "desagradar",
    "desagradasteis": "desagradar",
    "desagrade": "desagradar",
    "desagradece": "desagradecer",
    "desagradeced": "desagradecer",
    "desagradecemos": "desagradecer",
    "desagradecen": "desagradecer",
    "desagradeceremos": "desagradecer",
    "desagradecerá": "desagradecer",
    "desagradecerán": "desagradecer",
    "desagradecerás": "desagradecer",
    "desagradeceré": "desagradecer",
    "desagradeceréis": "desagradecer",
    "desagradecería": "desagradecer",
    "desagradeceríais": "desagradecer",
    "desagradeceríamos": "desagradecer",
    "desagradecerían": "desagradecer",
    "desagradecerías": "desagradecer",
    "desagradeces": "desagradecer",
    "desagradecida": "desagradecido",
    "desagradecidas": "desagradecido",
    "desagradecido": "desagradecer",
    "desagradecidos": "desagradecido",
    "desagradeciendo": "desagradecer",
    "desagradeciera": "desagradecer",
    "desagradecierais": "desagradecer",
    "desagradecieran": "desagradecer",
    "desagradecieras": "desagradecer",
    "desagradeciere": "desagradecer",
    "desagradeciereis": "desagradecer",
    "desagradecieren": "desagradecer",
    "desagradecieres": "desagradecer",
    "desagradecieron": "desagradecer",
    "desagradeciese": "desagradecer",
    "desagradecieseis": "desagradecer",
    "desagradeciesen": "desagradecer",
    "desagradecieses": "desagradecer",
    "desagradecimientos": "desagradecimiento",
    "desagradecimos": "desagradecer",
    "desagradeciste": "desagradecer",
    "desagradecisteis": "desagradecer",
    "desagradeciéramos": "desagradecer",
    "desagradeciéremos": "desagradecer",
    "desagradeciésemos": "desagradecer",
    "desagradeció": "desagradecer",
    "desagradecéis": "desagradecer",
    "desagradecí": "desagradecer",
    "desagradecía": "desagradecer",
    "desagradecíais": "desagradecer",
    "desagradecíamos": "desagradecer",
    "desagradecían": "desagradecer",
    "desagradecías": "desagradecer",
    "desagrademos": "desagradar",
    "desagraden": "desagradar",
    "desagrades": "desagradar",
    "desagradezca": "desagradecer",
    "desagradezcamos": "desagradecer",
    "desagradezcan": "desagradecer",
    "desagradezcas": "desagradecer",
    "desagradezco": "desagradecer",
    "desagradezcáis": "desagradecer",
    "desagrado": "desagradar",
    "desagrados": "desagrado",
    "desagradábamos": "desagradar",
    "desagradáis": "desagradar",
    "desagradáramos": "desagradar",
    "desagradáremos": "desagradar",
    "desagradásemos": "desagradar",
    "desagradé": "desagradar",
    "desagradéis": "desagradar",
    "desagradó": "desagradar",
    "desagravia": "desagraviar",
    "desagraviaba": "desagraviar",
    "desagraviabais": "desagraviar",
    "desagraviaban": "desagraviar",
    "desagraviabas": "desagraviar",
    "desagraviad": "desagraviar",
    "desagraviada": "desagraviar",
    "desagraviadas": "desagraviar",
    "desagraviado": "desagraviar",
    "desagraviados": "desagraviar",
    "desagraviamos": "desagraviar",
    "desagravian": "desagraviar",
    "desagraviando": "desagraviar",
    "desagraviara": "desagraviar",
    "desagraviarais": "desagraviar",
    "desagraviaran": "desagraviar",
    "desagraviaras": "desagraviar",
    "desagraviare": "desagraviar",
    "desagraviareis": "desagraviar",
    "desagraviaremos": "desagraviar",
    "desagraviaren": "desagraviar",
    "desagraviares": "desagraviar",
    "desagraviaron": "desagraviar",
    "desagraviará": "desagraviar",
    "desagraviarán": "desagraviar",
    "desagraviarás": "desagraviar",
    "desagraviaré": "desagraviar",
    "desagraviaréis": "desagraviar",
    "desagraviaría": "desagraviar",
    "desagraviaríais": "desagraviar",
    "desagraviaríamos": "desagraviar",
    "desagraviarían": "desagraviar",
    "desagraviarías": "desagraviar",
    "desagravias": "desagraviar",
    "desagraviase": "desagraviar",
    "desagraviaseis": "desagraviar",
    "desagraviasen": "desagraviar",
    "desagraviases": "desagraviar",
    "desagraviaste": "desagraviar",
    "desagraviasteis": "desagraviar",
    "desagravie": "desagraviar",
    "desagraviemos": "desagraviar",
    "desagravien": "desagraviar",
    "desagravies": "desagraviar",
    "desagravio": "desagraviar",
    "desagravios": "desagravio",
    "desagraviábamos": "desagraviar",
    "desagraviáis": "desagraviar",
    "desagraviáramos": "desagraviar",
    "desagraviáremos": "desagraviar",
    "desagraviásemos": "desagraviar",
    "desagravié": "desagraviar",
    "desagraviéis": "desagraviar",
    "desagravió": "desagraviar",
    "desagrega": "desagregar",
    "desagregaba": "desagregar",
    "desagregabais": "desagregar",
    "desagregaban": "desagregar",
    "desagregabas": "desagregar",
    "desagregaciones": "desagregación",
    "desagregad": "desagregar",
    "desagregada": "desagregar",
    "desagregadas": "desagregar",
    "desagregado": "desagregar",
    "desagregadores": "desagregador",
    "desagregados": "desagregar",
    "desagregamos": "desagregar",
    "desagregan": "desagregar",
    "desagregando": "desagregar",
    "desagregara": "desagregar",
    "desagregarais": "desagregar",
    "desagregaran": "desagregar",
    "desagregaras": "desagregar",
    "desagregare": "desagregar",
    "desagregareis": "desagregar",
    "desagregaremos": "desagregar",
    "desagregaren": "desagregar",
    "desagregares": "desagregar",
    "desagregaron": "desagregar",
    "desagregará": "desagregar",
    "desagregarán": "desagregar",
    "desagregarás": "desagregar",
    "desagregaré": "desagregar",
    "desagregaréis": "desagregar",
    "desagregaría": "desagregar",
    "desagregaríais": "desagregar",
    "desagregaríamos": "desagregar",
    "desagregarían": "desagregar",
    "desagregarías": "desagregar",
    "desagregas": "desagregar",
    "desagregase": "desagregar",
    "desagregaseis": "desagregar",
    "desagregasen": "desagregar",
    "desagregases": "desagregar",
    "desagregaste": "desagregar",
    "desagregasteis": "desagregar",
    "desagrego": "desagregar",
    "desagregue": "desagregar",
    "desagreguemos": "desagregar",
    "desagreguen": "desagregar",
    "desagregues": "desagregar",
    "desagregué": "desagregar",
    "desagreguéis": "desagregar",
    "desagregábamos": "desagregar",
    "desagregáis": "desagregar",
    "desagregáramos": "desagregar",
    "desagregáremos": "desagregar",
    "desagregásemos": "desagregar",
    "desagregó": "desagregar",
    "desagua": "desaguar",
    "desaguaba": "desaguar",
    "desaguabais": "desaguar",
    "desaguaban": "desaguar",
    "desaguabas": "desaguar",
    "desaguace": "desaguazar",
    "desaguacemos": "desaguazar",
    "desaguacen": "desaguazar",
    "desaguaces": "desaguazar",
    "desaguacé": "desaguazar",
    "desaguacéis": "desaguazar",
    "desaguad": "desaguar",
    "desaguada": "desaguar",
    "desaguadas": "desaguar",
    "desaguaderos": "desaguadero",
    "desaguado": "desaguar",
    "desaguadores": "desaguador",
    "desaguados": "desaguar",
    "desaguamos": "desaguar",
    "desaguan": "desaguar",
    "desaguando": "desaguar",
    "desaguara": "desaguar",
    "desaguarais": "desaguar",
    "desaguaran": "desaguar",
    "desaguaras": "desaguar",
    "desaguare": "desaguar",
    "desaguareis": "desaguar",
    "desaguaremos": "desaguar",
    "desaguaren": "desaguar",
    "desaguares": "desaguar",
    "desaguaron": "desaguar",
    "desaguará": "desaguar",
    "desaguarán": "desaguar",
    "desaguarás": "desaguar",
    "desaguaré": "desaguar",
    "desaguaréis": "desaguar",
    "desaguaría": "desaguar",
    "desaguaríais": "desaguar",
    "desaguaríamos": "desaguar",
    "desaguarían": "desaguar",
    "desaguarías": "desaguar",
    "desaguas": "desaguar",
    "desaguase": "desaguar",
    "desaguaseis": "desaguar",
    "desaguasen": "desaguar",
    "desaguases": "desaguar",
    "desaguaste": "desaguar",
    "desaguasteis": "desaguar",
    "desaguaza": "desaguazar",
    "desaguazaba": "desaguazar",
    "desaguazabais": "desaguazar",
    "desaguazaban": "desaguazar",
    "desaguazabas": "desaguazar",
    "desaguazad": "desaguazar",
    "desaguazada": "desaguazar",
    "desaguazadas": "desaguazar",
    "desaguazado": "desaguazar",
    "desaguazados": "desaguazar",
    "desaguazamos": "desaguazar",
    "desaguazan": "desaguazar",
    "desaguazando": "desaguazar",
    "desaguazara": "desaguazar",
    "desaguazarais": "desaguazar",
    "desaguazaran": "desaguazar",
    "desaguazaras": "desaguazar",
    "desaguazare": "desaguazar",
    "desaguazareis": "desaguazar",
    "desaguazaremos": "desaguazar",
    "desaguazaren": "desaguazar",
    "desaguazares": "desaguazar",
    "desaguazaron": "desaguazar",
    "desaguazará": "desaguazar",
    "desaguazarán": "desaguazar",
    "desaguazarás": "desaguazar",
    "desaguazaré": "desaguazar",
    "desaguazaréis": "desaguazar",
    "desaguazaría": "desaguazar",
    "desaguazaríais": "desaguazar",
    "desaguazaríamos": "desaguazar",
    "desaguazarían": "desaguazar",
    "desaguazarías": "desaguazar",
    "desaguazas": "desaguazar",
    "desaguazase": "desaguazar",
    "desaguazaseis": "desaguazar",
    "desaguazasen": "desaguazar",
    "desaguazases": "desaguazar",
    "desaguazaste": "desaguazar",
    "desaguazasteis": "desaguazar",
    "desaguazo": "desaguazar",
    "desaguazábamos": "desaguazar",
    "desaguazáis": "desaguazar",
    "desaguazáramos": "desaguazar",
    "desaguazáremos": "desaguazar",
    "desaguazásemos": "desaguazar",
    "desaguazó": "desaguazar",
    "desaguisada": "desaguisado",
    "desaguisadas": "desaguisado",
    "desaguisados": "desaguisado",
    "desaguo": "desaguar",
    "desaguábamos": "desaguar",
    "desaguáis": "desaguar",
    "desaguáramos": "desaguar",
    "desaguáremos": "desaguar",
    "desaguásemos": "desaguar",
    "desaguó": "desaguar",
    "desagüe": "desaguar",
    "desagüemos": "desaguar",
    "desagüen": "desaguar",
    "desagües": "desagüe",
    "desagüé": "desaguar",
    "desagüéis": "desaguar",
    "desaherroja": "desaherrojar",
    "desaherrojaba": "desaherrojar",
    "desaherrojabais": "desaherrojar",
    "desaherrojaban": "desaherrojar",
    "desaherrojabas": "desaherrojar",
    "desaherrojad": "desaherrojar",
    "desaherrojada": "desaherrojar",
    "desaherrojadas": "desaherrojar",
    "desaherrojado": "desaherrojar",
    "desaherrojados": "desaherrojar",
    "desaherrojamos": "desaherrojar",
    "desaherrojan": "desaherrojar",
    "desaherrojando": "desaherrojar",
    "desaherrojara": "desaherrojar",
    "desaherrojarais": "desaherrojar",
    "desaherrojaran": "desaherrojar",
    "desaherrojaras": "desaherrojar",
    "desaherrojare": "desaherrojar",
    "desaherrojareis": "desaherrojar",
    "desaherrojaremos": "desaherrojar",
    "desaherrojaren": "desaherrojar",
    "desaherrojares": "desaherrojar",
    "desaherrojaron": "desaherrojar",
    "desaherrojará": "desaherrojar",
    "desaherrojarán": "desaherrojar",
    "desaherrojarás": "desaherrojar",
    "desaherrojaré": "desaherrojar",
    "desaherrojaréis": "desaherrojar",
    "desaherrojaría": "desaherrojar",
    "desaherrojaríais": "desaherrojar",
    "desaherrojaríamos": "desaherrojar",
    "desaherrojarían": "desaherrojar",
    "desaherrojarías": "desaherrojar",
    "desaherrojas": "desaherrojar",
    "desaherrojase": "desaherrojar",
    "desaherrojaseis": "desaherrojar",
    "desaherrojasen": "desaherrojar",
    "desaherrojases": "desaherrojar",
    "desaherrojaste": "desaherrojar",
    "desaherrojasteis": "desaherrojar",
    "desaherroje": "desaherrojar",
    "desaherrojemos": "desaherrojar",
    "desaherrojen": "desaherrojar",
    "desaherrojes": "desaherrojar",
    "desaherrojo": "desaherrojar",
    "desaherrojábamos": "desaherrojar",
    "desaherrojáis": "desaherrojar",
    "desaherrojáramos": "desaherrojar",
    "desaherrojáremos": "desaherrojar",
    "desaherrojásemos": "desaherrojar",
    "desaherrojé": "desaherrojar",
    "desaherrojéis": "desaherrojar",
    "desaherrojó": "desaherrojar",
    "desahijaba": "desahijar",
    "desahijabais": "desahijar",
    "desahijaban": "desahijar",
    "desahijabas": "desahijar",
    "desahijad": "desahijar",
    "desahijada": "desahijar",
    "desahijadas": "desahijar",
    "desahijado": "desahijar",
    "desahijados": "desahijar",
    "desahijamos": "desahijar",
    "desahijando": "desahijar",
    "desahijara": "desahijar",
    "desahijarais": "desahijar",
    "desahijaran": "desahijar",
    "desahijaras": "desahijar",
    "desahijare": "desahijar",
    "desahijareis": "desahijar",
    "desahijaremos": "desahijar",
    "desahijaren": "desahijar",
    "desahijares": "desahijar",
    "desahijaron": "desahijar",
    "desahijará": "desahijar",
    "desahijarán": "desahijar",
    "desahijarás": "desahijar",
    "desahijaré": "desahijar",
    "desahijaréis": "desahijar",
    "desahijaría": "desahijar",
    "desahijaríais": "desahijar",
    "desahijaríamos": "desahijar",
    "desahijarían": "desahijar",
    "desahijarías": "desahijar",
    "desahijase": "desahijar",
    "desahijaseis": "desahijar",
    "desahijasen": "desahijar",
    "desahijases": "desahijar",
    "desahijaste": "desahijar",
    "desahijasteis": "desahijar",
    "desahijemos": "desahijar",
    "desahijábamos": "desahijar",
    "desahijáis": "desahijar",
    "desahijáramos": "desahijar",
    "desahijáremos": "desahijar",
    "desahijásemos": "desahijar",
    "desahijé": "desahijar",
    "desahijéis": "desahijar",
    "desahijó": "desahijar",
    "desahoga": "desahogar",
    "desahogaba": "desahogar",
    "desahogabais": "desahogar",
    "desahogaban": "desahogar",
    "desahogabas": "desahogar",
    "desahogad": "desahogar",
    "desahogada": "desahogar",
    "desahogadas": "desahogar",
    "desahogado": "desahogar",
    "desahogados": "desahogar",
    "desahogamos": "desahogar",
    "desahogan": "desahogar",
    "desahogando": "desahogar",
    "desahogara": "desahogar",
    "desahogarais": "desahogar",
    "desahogaran": "desahogar",
    "desahogaras": "desahogar",
    "desahogare": "desahogar",
    "desahogareis": "desahogar",
    "desahogaremos": "desahogar",
    "desahogaren": "desahogar",
    "desahogares": "desahogar",
    "desahogaron": "desahogar",
    "desahogará": "desahogar",
    "desahogarán": "desahogar",
    "desahogarás": "desahogar",
    "desahogaré": "desahogar",
    "desahogaréis": "desahogar",
    "desahogaría": "desahogar",
    "desahogaríais": "desahogar",
    "desahogaríamos": "desahogar",
    "desahogarían": "desahogar",
    "desahogarías": "desahogar",
    "desahogas": "desahogar",
    "desahogase": "desahogar",
    "desahogaseis": "desahogar",
    "desahogasen": "desahogar",
    "desahogases": "desahogar",
    "desahogaste": "desahogar",
    "desahogasteis": "desahogar",
    "desahogo": "desahogar",
    "desahogos": "desahogo",
    "desahogue": "desahogar",
    "desahoguemos": "desahogar",
    "desahoguen": "desahogar",
    "desahogues": "desahogar",
    "desahogué": "desahogar",
    "desahoguéis": "desahogar",
    "desahogábamos": "desahogar",
    "desahogáis": "desahogar",
    "desahogáramos": "desahogar",
    "desahogáremos": "desahogar",
    "desahogásemos": "desahogar",
    "desahogó": "desahogar",
    "desahorros": "desahorro",
    "desahucia": "desahuciar",
    "desahuciaba": "desahuciar",
    "desahuciabais": "desahuciar",
    "desahuciaban": "desahuciar",
    "desahuciabas": "desahuciar",
    "desahuciad": "desahuciar",
    "desahuciada": "desahuciar",
    "desahuciadas": "desahuciar",
    "desahuciado": "desahuciar",
    "desahuciados": "desahuciar",
    "desahuciamos": "desahuciar",
    "desahucian": "desahuciar",
    "desahuciando": "desahuciar",
    "desahuciara": "desahuciar",
    "desahuciarais": "desahuciar",
    "desahuciaran": "desahuciar",
    "desahuciaras": "desahuciar",
    "desahuciare": "desahuciar",
    "desahuciareis": "desahuciar",
    "desahuciaremos": "desahuciar",
    "desahuciaren": "desahuciar",
    "desahuciares": "desahuciar",
    "desahuciaron": "desahuciar",
    "desahuciará": "desahuciar",
    "desahuciarán": "desahuciar",
    "desahuciarás": "desahuciar",
    "desahuciaré": "desahuciar",
    "desahuciaréis": "desahuciar",
    "desahuciaría": "desahuciar",
    "desahuciaríais": "desahuciar",
    "desahuciaríamos": "desahuciar",
    "desahuciarían": "desahuciar",
    "desahuciarías": "desahuciar",
    "desahucias": "desahuciar",
    "desahuciase": "desahuciar",
    "desahuciaseis": "desahuciar",
    "desahuciasen": "desahuciar",
    "desahuciases": "desahuciar",
    "desahuciaste": "desahuciar",
    "desahuciasteis": "desahuciar",
    "desahucie": "desahuciar",
    "desahuciemos": "desahuciar",
    "desahucien": "desahuciar",
    "desahucies": "desahuciar",
    "desahucio": "desahuciar",
    "desahucios": "desahucio",
    "desahuciábamos": "desahuciar",
    "desahuciáis": "desahuciar",
    "desahuciáramos": "desahuciar",
    "desahuciáremos": "desahuciar",
    "desahuciásemos": "desahuciar",
    "desahucié": "desahuciar",
    "desahuciéis": "desahuciar",
    "desahució": "desahuciar",
    "desahumada": "desahumado",
    "desahumadas": "desahumado",
    "desahumados": "desahumado",
    "desahíja": "desahijar",
    "desahíjan": "desahijar",
    "desahíjas": "desahijar",
    "desahíje": "desahijar",
    "desahíjen": "desahijar",
    "desahíjes": "desahijar",
    "desahíjo": "desahijar",
    "desainaba": "desainar",
    "desainabais": "desainar",
    "desainaban": "desainar",
    "desainabas": "desainar",
    "desainad": "desainar",
    "desainada": "desainar",
    "desainadas": "desainar",
    "desainado": "desainar",
    "desainados": "desainar",
    "desainaduras": "desainadura",
    "desainamos": "desainar",
    "desainando": "desainar",
    "desainara": "desainar",
    "desainarais": "desainar",
    "desainaran": "desainar",
    "desainaras": "desainar",
    "desainare": "desainar",
    "desainareis": "desainar",
    "desainaremos": "desainar",
    "desainaren": "desainar",
    "desainares": "desainar",
    "desainaron": "desainar",
    "desainará": "desainar",
    "desainarán": "desainar",
    "desainarás": "desainar",
    "desainaré": "desainar",
    "desainaréis": "desainar",
    "desainaría": "desainar",
    "desainaríais": "desainar",
    "desainaríamos": "desainar",
    "desainarían": "desainar",
    "desainarías": "desainar",
    "desainase": "desainar",
    "desainaseis": "desainar",
    "desainasen": "desainar",
    "desainases": "desainar",
    "desainaste": "desainar",
    "desainasteis": "desainar",
    "desainemos": "desainar",
    "desainábamos": "desainar",
    "desaináis": "desainar",
    "desaináramos": "desainar",
    "desaináremos": "desainar",
    "desainásemos": "desainar",
    "desainé": "desainar",
    "desainéis": "desainar",
    "desainó": "desainar",
    "desaira": "desairar",
    "desairaba": "desairar",
    "desairabais": "desairar",
    "desairaban": "desairar",
    "desairabas": "desairar",
    "desairad": "desairar",
    "desairada": "desairar",
    "desairadas": "desairar",
    "desairado": "desairar",
    "desairados": "desairar",
    "desairamos": "desairar",
    "desairan": "desairar",
    "desairando": "desairar",
    "desairara": "desairar",
    "desairarais": "desairar",
    "desairaran": "desairar",
    "desairaras": "desairar",
    "desairare": "desairar",
    "desairareis": "desairar",
    "desairaremos": "desairar",
    "desairaren": "desairar",
    "desairares": "desairar",
    "desairaron": "desairar",
    "desairará": "desairar",
    "desairarán": "desairar",
    "desairarás": "desairar",
    "desairaré": "desairar",
    "desairaréis": "desairar",
    "desairaría": "desairar",
    "desairaríais": "desairar",
    "desairaríamos": "desairar",
    "desairarían": "desairar",
    "desairarías": "desairar",
    "desairas": "desairar",
    "desairase": "desairar",
    "desairaseis": "desairar",
    "desairasen": "desairar",
    "desairases": "desairar",
    "desairaste": "desairar",
    "desairasteis": "desairar",
    "desaire": "desairar",
    "desairemos": "desairar",
    "desairen": "desairar",
    "desaires": "desaire",
    "desairo": "desairar",
    "desairábamos": "desairar",
    "desairáis": "desairar",
    "desairáramos": "desairar",
    "desairáremos": "desairar",
    "desairásemos": "desairar",
    "desairé": "desairar",
    "desairéis": "desairar",
    "desairó": "desairar",
    "desajusta": "desajustar",
    "desajustaba": "desajustar",
    "desajustabais": "desajustar",
    "desajustaban": "desajustar",
    "desajustabas": "desajustar",
    "desajustad": "desajustar",
    "desajustada": "desajustar",
    "desajustadas": "desajustar",
    "desajustado": "desajustar",
    "desajustados": "desajustar",
    "desajustamos": "desajustar",
    "desajustan": "desajustar",
    "desajustando": "desajustar",
    "desajustara": "desajustar",
    "desajustarais": "desajustar",
    "desajustaran": "desajustar",
    "desajustaras": "desajustar",
    "desajustare": "desajustar",
    "desajustareis": "desajustar",
    "desajustaremos": "desajustar",
    "desajustaren": "desajustar",
    "desajustares": "desajustar",
    "desajustaron": "desajustar",
    "desajustará": "desajustar",
    "desajustarán": "desajustar",
    "desajustarás": "desajustar",
    "desajustaré": "desajustar",
    "desajustaréis": "desajustar",
    "desajustaría": "desajustar",
    "desajustaríais": "desajustar",
    "desajustaríamos": "desajustar",
    "desajustarían": "desajustar",
    "desajustarías": "desajustar",
    "desajustas": "desajustar",
    "desajustase": "desajustar",
    "desajustaseis": "desajustar",
    "desajustasen": "desajustar",
    "desajustases": "desajustar",
    "desajustaste": "desajustar",
    "desajustasteis": "desajustar",
    "desajuste": "desajustar",
    "desajustemos": "desajustar",
    "desajusten": "desajustar",
    "desajustes": "desajuste",
    "desajusto": "desajustar",
    "desajustábamos": "desajustar",
    "desajustáis": "desajustar",
    "desajustáramos": "desajustar",
    "desajustáremos": "desajustar",
    "desajustásemos": "desajustar",
    "desajusté": "desajustar",
    "desajustéis": "desajustar",
    "desajustó": "desajustar",
    "desala": "desalar",
    "desalaba": "desalar",
    "desalababa": "desalabar",
    "desalababais": "desalabar",
    "desalababan": "desalabar",
    "desalababas": "desalabar",
    "desalabad": "desalabar",
    "desalabada": "desalabar",
    "desalabadas": "desalabar",
    "desalabado": "desalabar",
    "desalabados": "desalabar",
    "desalabais": "desalar",
    "desalabamos": "desalabar",
    "desalaban": "desalar",
    "desalabando": "desalabar",
    "desalabanzas": "desalabanza",
    "desalabara": "desalabar",
    "desalabarais": "desalabar",
    "desalabaran": "desalabar",
    "desalabaras": "desalabar",
    "desalabare": "desalabar",
    "desalabareis": "desalabar",
    "desalabaremos": "desalabar",
    "desalabaren": "desalabar",
    "desalabares": "desalabar",
    "desalabaron": "desalabar",
    "desalabará": "desalabar",
    "desalabarán": "desalabar",
    "desalabarás": "desalabar",
    "desalabaré": "desalabar",
    "desalabaréis": "desalabar",
    "desalabaría": "desalabar",
    "desalabaríais": "desalabar",
    "desalabaríamos": "desalabar",
    "desalabarían": "desalabar",
    "desalabarías": "desalabar",
    "desalabas": "desalar",
    "desalabase": "desalabar",
    "desalabaseis": "desalabar",
    "desalabasen": "desalabar",
    "desalabases": "desalabar",
    "desalabaste": "desalabar",
    "desalabasteis": "desalabar",
    "desalabe": "desalabar",
    "desalabea": "desalabear",
    "desalabeaba": "desalabear",
    "desalabeabais": "desalabear",
    "desalabeaban": "desalabear",
    "desalabeabas": "desalabear",
    "desalabead": "desalabear",
    "desalabeada": "desalabear",
    "desalabeadas": "desalabear",
    "desalabeado": "desalabear",
    "desalabeados": "desalabear",
    "desalabeamos": "desalabear",
    "desalabean": "desalabear",
    "desalabeando": "desalabear",
    "desalabeara": "desalabear",
    "desalabearais": "desalabear",
    "desalabearan": "desalabear",
    "desalabearas": "desalabear",
    "desalabeare": "desalabear",
    "desalabeareis": "desalabear",
    "desalabearemos": "desalabear",
    "desalabearen": "desalabear",
    "desalabeares": "desalabear",
    "desalabearon": "desalabear",
    "desalabeará": "desalabear",
    "desalabearán": "desalabear",
    "desalabearás": "desalabear",
    "desalabearé": "desalabear",
    "desalabearéis": "desalabear",
    "desalabearía": "desalabear",
    "desalabearíais": "desalabear",
    "desalabearíamos": "desalabear",
    "desalabearían": "desalabear",
    "desalabearías": "desalabear",
    "desalabeas": "desalabear",
    "desalabease": "desalabear",
    "desalabeaseis": "desalabear",
    "desalabeasen": "desalabear",
    "desalabeases": "desalabear",
    "desalabeaste": "desalabear",
    "desalabeasteis": "desalabear",
    "desalabee": "desalabear",
    "desalabeemos": "desalabear",
    "desalabeen": "desalabear",
    "desalabees": "desalabear",
    "desalabemos": "desalabar",
    "desalaben": "desalabar",
    "desalabeo": "desalabear",
    "desalabeos": "desalabeo",
    "desalabes": "desalabar",
    "desalabeábamos": "desalabear",
    "desalabeáis": "desalabear",
    "desalabeáramos": "desalabear",
    "desalabeáremos": "desalabear",
    "desalabeásemos": "desalabear",
    "desalabeé": "desalabear",
    "desalabeéis": "desalabear",
    "desalabeó": "desalabear",
    "desalabo": "desalabar",
    "desalabábamos": "desalabar",
    "desalabáis": "desalabar",
    "desalabáramos": "desalabar",
    "desalabáremos": "desalabar",
    "desalabásemos": "desalabar",
    "desalabé": "desalabar",
    "desalabéis": "desalabar",
    "desalabó": "desalabar",
    "desalaciones": "desalación",
    "desalad": "desalar",
    "desalada": "desalar",
    "desaladas": "desalar",
    "desalado": "desalar",
    "desalados": "desalar",
    "desalamos": "desalar",
    "desalan": "desalar",
    "desalando": "desalar",
    "desalara": "desalar",
    "desalarais": "desalar",
    "desalaran": "desalar",
    "desalaras": "desalar",
    "desalare": "desalar",
    "desalareis": "desalar",
    "desalaremos": "desalar",
    "desalaren": "desalar",
    "desalares": "desalar",
    "desalaron": "desalar",
    "desalará": "desalar",
    "desalarán": "desalar",
    "desalarás": "desalar",
    "desalaré": "desalar",
    "desalaréis": "desalar",
    "desalaría": "desalar",
    "desalaríais": "desalar",
    "desalaríamos": "desalar",
    "desalarían": "desalar",
    "desalarías": "desalar",
    "desalas": "desalar",
    "desalase": "desalar",
    "desalaseis": "desalar",
    "desalasen": "desalar",
    "desalases": "desalar",
    "desalaste": "desalar",
    "desalasteis": "desalar",
    "desalbarda": "desalbardar",
    "desalbardaba": "desalbardar",
    "desalbardabais": "desalbardar",
    "desalbardaban": "desalbardar",
    "desalbardabas": "desalbardar",
    "desalbardad": "desalbardar",
    "desalbardada": "desalbardar",
    "desalbardadas": "desalbardar",
    "desalbardado": "desalbardar",
    "desalbardados": "desalbardar",
    "desalbardamos": "desalbardar",
    "desalbardan": "desalbardar",
    "desalbardando": "desalbardar",
    "desalbardara": "desalbardar",
    "desalbardarais": "desalbardar",
    "desalbardaran": "desalbardar",
    "desalbardaras": "desalbardar",
    "desalbardare": "desalbardar",
    "desalbardareis": "desalbardar",
    "desalbardaremos": "desalbardar",
    "desalbardaren": "desalbardar",
    "desalbardares": "desalbardar",
    "desalbardaron": "desalbardar",
    "desalbardará": "desalbardar",
    "desalbardarán": "desalbardar",
    "desalbardarás": "desalbardar",
    "desalbardaré": "desalbardar",
    "desalbardaréis": "desalbardar",
    "desalbardaría": "desalbardar",
    "desalbardaríais": "desalbardar",
    "desalbardaríamos": "desalbardar",
    "desalbardarían": "desalbardar",
    "desalbardarías": "desalbardar",
    "desalbardas": "desalbardar",
    "desalbardase": "desalbardar",
    "desalbardaseis": "desalbardar",
    "desalbardasen": "desalbardar",
    "desalbardases": "desalbardar",
    "desalbardaste": "desalbardar",
    "desalbardasteis": "desalbardar",
    "desalbarde": "desalbardar",
    "desalbardemos": "desalbardar",
    "desalbarden": "desalbardar",
    "desalbardes": "desalbardar",
    "desalbardo": "desalbardar",
    "desalbardábamos": "desalbardar",
    "desalbardáis": "desalbardar",
    "desalbardáramos": "desalbardar",
    "desalbardáremos": "desalbardar",
    "desalbardásemos": "desalbardar",
    "desalbardé": "desalbardar",
    "desalbardéis": "desalbardar",
    "desalbardó": "desalbardar",
    "desalcatorizadores": "desalcatorizador",
    "desalcoholizaciones": "desalcoholización",
    "desale": "desalar",
    "desaleatorizadores": "desaleatorizador",
    "desalemos": "desalar",
    "desalen": "desalar",
    "desalentaba": "desalentar",
    "desalentabais": "desalentar",
    "desalentaban": "desalentar",
    "desalentabas": "desalentar",
    "desalentad": "desalentar",
    "desalentada": "desalentar",
    "desalentadas": "desalentar",
    "desalentado": "desalentar",
    "desalentadora": "desalentador",
    "desalentadoras": "desalentador",
    "desalentadores": "desalentador",
    "desalentados": "desalentar",
    "desalentamos": "desalentar",
    "desalentando": "desalentar",
    "desalentara": "desalentar",
    "desalentarais": "desalentar",
    "desalentaran": "desalentar",
    "desalentaras": "desalentar",
    "desalentare": "desalentar",
    "desalentareis": "desalentar",
    "desalentaremos": "desalentar",
    "desalentaren": "desalentar",
    "desalentares": "desalentar",
    "desalentaron": "desalentar",
    "desalentará": "desalentar",
    "desalentarán": "desalentar",
    "desalentarás": "desalentar",
    "desalentaré": "desalentar",
    "desalentaréis": "desalentar",
    "desalentaría": "desalentar",
    "desalentaríais": "desalentar",
    "desalentaríamos": "desalentar",
    "desalentarían": "desalentar",
    "desalentarías": "desalentar",
    "desalentase": "desalentar",
    "desalentaseis": "desalentar",
    "desalentasen": "desalentar",
    "desalentases": "desalentar",
    "desalentaste": "desalentar",
    "desalentasteis": "desalentar",
    "desalentemos": "desalentar",
    "desalentábamos": "desalentar",
    "desalentáis": "desalentar",
    "desalentáramos": "desalentar",
    "desalentáremos": "desalentar",
    "desalentásemos": "desalentar",
    "desalenté": "desalentar",
    "desalentéis": "desalentar",
    "desalentó": "desalentar",
    "desales": "desalar",
    "desalfombra": "desalfombrar",
    "desalfombraba": "desalfombrar",
    "desalfombrabais": "desalfombrar",
    "desalfombraban": "desalfombrar",
    "desalfombrabas": "desalfombrar",
    "desalfombrad": "desalfombrar",
    "desalfombrada": "desalfombrar",
    "desalfombradas": "desalfombrar",
    "desalfombrado": "desalfombrar",
    "desalfombrados": "desalfombrar",
    "desalfombramos": "desalfombrar",
    "desalfombran": "desalfombrar",
    "desalfombrando": "desalfombrar",
    "desalfombrara": "desalfombrar",
    "desalfombrarais": "desalfombrar",
    "desalfombraran": "desalfombrar",
    "desalfombraras": "desalfombrar",
    "desalfombrare": "desalfombrar",
    "desalfombrareis": "desalfombrar",
    "desalfombraremos": "desalfombrar",
    "desalfombraren": "desalfombrar",
    "desalfombrares": "desalfombrar",
    "desalfombraron": "desalfombrar",
    "desalfombrará": "desalfombrar",
    "desalfombrarán": "desalfombrar",
    "desalfombrarás": "desalfombrar",
    "desalfombraré": "desalfombrar",
    "desalfombraréis": "desalfombrar",
    "desalfombraría": "desalfombrar",
    "desalfombraríais": "desalfombrar",
    "desalfombraríamos": "desalfombrar",
    "desalfombrarían": "desalfombrar",
    "desalfombrarías": "desalfombrar",
    "desalfombras": "desalfombrar",
    "desalfombrase": "desalfombrar",
    "desalfombraseis": "desalfombrar",
    "desalfombrasen": "desalfombrar",
    "desalfombrases": "desalfombrar",
    "desalfombraste": "desalfombrar",
    "desalfombrasteis": "desalfombrar",
    "desalfombre": "desalfombrar",
    "desalfombremos": "desalfombrar",
    "desalfombren": "desalfombrar",
    "desalfombres": "desalfombrar",
    "desalfombro": "desalfombrar",
    "desalfombrábamos": "desalfombrar",
    "desalfombráis": "desalfombrar",
    "desalfombráramos": "desalfombrar",
    "desalfombráremos": "desalfombrar",
    "desalfombrásemos": "desalfombrar",
    "desalfombré": "desalfombrar",
    "desalfombréis": "desalfombrar",
    "desalfombró": "desalfombrar",
    "desalforja": "desalforjar",
    "desalforjaba": "desalforjar",
    "desalforjabais": "desalforjar",
    "desalforjaban": "desalforjar",
    "desalforjabas": "desalforjar",
    "desalforjad": "desalforjar",
    "desalforjada": "desalforjar",
    "desalforjadas": "desalforjar",
    "desalforjado": "desalforjar",
    "desalforjados": "desalforjar",
    "desalforjamos": "desalforjar",
    "desalforjan": "desalforjar",
    "desalforjando": "desalforjar",
    "desalforjara": "desalforjar",
    "desalforjarais": "desalforjar",
    "desalforjaran": "desalforjar",
    "desalforjaras": "desalforjar",
    "desalforjare": "desalforjar",
    "desalforjareis": "desalforjar",
    "desalforjaremos": "desalforjar",
    "desalforjaren": "desalforjar",
    "desalforjares": "desalforjar",
    "desalforjaron": "desalforjar",
    "desalforjará": "desalforjar",
    "desalforjarán": "desalforjar",
    "desalforjarás": "desalforjar",
    "desalforjaré": "desalforjar",
    "desalforjaréis": "desalforjar",
    "desalforjaría": "desalforjar",
    "desalforjaríais": "desalforjar",
    "desalforjaríamos": "desalforjar",
    "desalforjarían": "desalforjar",
    "desalforjarías": "desalforjar",
    "desalforjas": "desalforjar",
    "desalforjase": "desalforjar",
    "desalforjaseis": "desalforjar",
    "desalforjasen": "desalforjar",
    "desalforjases": "desalforjar",
    "desalforjaste": "desalforjar",
    "desalforjasteis": "desalforjar",
    "desalforje": "desalforjar",
    "desalforjemos": "desalforjar",
    "desalforjen": "desalforjar",
    "desalforjes": "desalforjar",
    "desalforjo": "desalforjar",
    "desalforjábamos": "desalforjar",
    "desalforjáis": "desalforjar",
    "desalforjáramos": "desalforjar",
    "desalforjáremos": "desalforjar",
    "desalforjásemos": "desalforjar",
    "desalforjé": "desalforjar",
    "desalforjéis": "desalforjar",
    "desalforjó": "desalforjar",
    "desalhaja": "desalhajar",
    "desalhajaba": "desalhajar",
    "desalhajabais": "desalhajar",
    "desalhajaban": "desalhajar",
    "desalhajabas": "desalhajar",
    "desalhajad": "desalhajar",
    "desalhajada": "desalhajar",
    "desalhajadas": "desalhajar",
    "desalhajado": "desalhajar",
    "desalhajados": "desalhajar",
    "desalhajamos": "desalhajar",
    "desalhajan": "desalhajar",
    "desalhajando": "desalhajar",
    "desalhajara": "desalhajar",
    "desalhajarais": "desalhajar",
    "desalhajaran": "desalhajar",
    "desalhajaras": "desalhajar",
    "desalhajare": "desalhajar",
    "desalhajareis": "desalhajar",
    "desalhajaremos": "desalhajar",
    "desalhajaren": "desalhajar",
    "desalhajares": "desalhajar",
    "desalhajaron": "desalhajar",
    "desalhajará": "desalhajar",
    "desalhajarán": "desalhajar",
    "desalhajarás": "desalhajar",
    "desalhajaré": "desalhajar",
    "desalhajaréis": "desalhajar",
    "desalhajaría": "desalhajar",
    "desalhajaríais": "desalhajar",
    "desalhajaríamos": "desalhajar",
    "desalhajarían": "desalhajar",
    "desalhajarías": "desalhajar",
    "desalhajas": "desalhajar",
    "desalhajase": "desalhajar",
    "desalhajaseis": "desalhajar",
    "desalhajasen": "desalhajar",
    "desalhajases": "desalhajar",
    "desalhajaste": "desalhajar",
    "desalhajasteis": "desalhajar",
    "desalhaje": "desalhajar",
    "desalhajemos": "desalhajar",
    "desalhajen": "desalhajar",
    "desalhajes": "desalhajar",
    "desalhajo": "desalhajar",
    "desalhajábamos": "desalhajar",
    "desalhajáis": "desalhajar",
    "desalhajáramos": "desalhajar",
    "desalhajáremos": "desalhajar",
    "desalhajásemos": "desalhajar",
    "desalhajé": "desalhajar",
    "desalhajéis": "desalhajar",
    "desalhajó": "desalhajar",
    "desalienta": "desalentar",
    "desalientan": "desalentar",
    "desalientas": "desalentar",
    "desaliente": "desalentar",
    "desalienten": "desalentar",
    "desalientes": "desalentar",
    "desaliento": "desalentar",
    "desalientos": "desaliento",
    "desalificaciones": "desalificación",
    "desalinea": "desalinear",
    "desalineaba": "desalinear",
    "desalineabais": "desalinear",
    "desalineaban": "desalinear",
    "desalineabas": "desalinear",
    "desalineaciones": "desalineación",
    "desalinead": "desalinear",
    "desalineada": "desalinear",
    "desalineadas": "desalinear",
    "desalineado": "desalinear",
    "desalineados": "desalinear",
    "desalineamientos": "desalineamiento",
    "desalineamos": "desalinear",
    "desalinean": "desalinear",
    "desalineando": "desalinear",
    "desalineara": "desalinear",
    "desalinearais": "desalinear",
    "desalinearan": "desalinear",
    "desalinearas": "desalinear",
    "desalineare": "desalinear",
    "desalineareis": "desalinear",
    "desalinearemos": "desalinear",
    "desalinearen": "desalinear",
    "desalineares": "desalinear",
    "desalinearon": "desalinear",
    "desalineará": "desalinear",
    "desalinearán": "desalinear",
    "desalinearás": "desalinear",
    "desalinearé": "desalinear",
    "desalinearéis": "desalinear",
    "desalinearía": "desalinear",
    "desalinearíais": "desalinear",
    "desalinearíamos": "desalinear",
    "desalinearían": "desalinear",
    "desalinearías": "desalinear",
    "desalineas": "desalinear",
    "desalinease": "desalinear",
    "desalineaseis": "desalinear",
    "desalineasen": "desalinear",
    "desalineases": "desalinear",
    "desalineaste": "desalinear",
    "desalineasteis": "desalinear",
    "desalinee": "desalinear",
    "desalineemos": "desalinear",
    "desalineen": "desalinear",
    "desalinees": "desalinear",
    "desalineo": "desalinear",
    "desalineábamos": "desalinear",
    "desalineáis": "desalinear",
    "desalineáramos": "desalinear",
    "desalineáremos": "desalinear",
    "desalineásemos": "desalinear",
    "desalineé": "desalinear",
    "desalineéis": "desalinear",
    "desalineó": "desalinear",
    "desalinice": "desalinizar",
    "desalinicemos": "desalinizar",
    "desalinicen": "desalinizar",
    "desalinices": "desalinizar",
    "desalinicé": "desalinizar",
    "desalinicéis": "desalinizar",
    "desaliniza": "desalinizar",
    "desalinizaba": "desalinizar",
    "desalinizabais": "desalinizar",
    "desalinizaban": "desalinizar",
    "desalinizabas": "desalinizar",
    "desalinizaciones": "desalinización",
    "desalinizad": "desalinizar",
    "desalinizada": "desalinizar",
    "desalinizadas": "desalinizar",
    "desalinizado": "desalinizar",
    "desalinizados": "desalinizar",
    "desalinizamos": "desalinizar",
    "desalinizan": "desalinizar",
    "desalinizando": "desalinizar",
    "desalinizara": "desalinizar",
    "desalinizarais": "desalinizar",
    "desalinizaran": "desalinizar",
    "desalinizaras": "desalinizar",
    "desalinizare": "desalinizar",
    "desalinizareis": "desalinizar",
    "desalinizaremos": "desalinizar",
    "desalinizaren": "desalinizar",
    "desalinizares": "desalinizar",
    "desalinizaron": "desalinizar",
    "desalinizará": "desalinizar",
    "desalinizarán": "desalinizar",
    "desalinizarás": "desalinizar",
    "desalinizaré": "desalinizar",
    "desalinizaréis": "desalinizar",
    "desalinizaría": "desalinizar",
    "desalinizaríais": "desalinizar",
    "desalinizaríamos": "desalinizar",
    "desalinizarían": "desalinizar",
    "desalinizarías": "desalinizar",
    "desalinizas": "desalinizar",
    "desalinizase": "desalinizar",
    "desalinizaseis": "desalinizar",
    "desalinizasen": "desalinizar",
    "desalinizases": "desalinizar",
    "desalinizaste": "desalinizar",
    "desalinizasteis": "desalinizar",
    "desalinizo": "desalinizar",
    "desalinizábamos": "desalinizar",
    "desalinizáis": "desalinizar",
    "desalinizáramos": "desalinizar",
    "desalinizáremos": "desalinizar",
    "desalinizásemos": "desalinizar",
    "desalinizó": "desalinizar",
    "desaliña": "desaliñar",
    "desaliñaba": "desaliñar",
    "desaliñabais": "desaliñar",
    "desaliñaban": "desaliñar",
    "desaliñabas": "desaliñar",
    "desaliñad": "desaliñar",
    "desaliñada": "desaliñar",
    "desaliñadas": "desaliñar",
    "desaliñado": "desaliñar",
    "desaliñados": "desaliñar",
    "desaliñamos": "desaliñar",
    "desaliñan": "desaliñar",
    "desaliñando": "desaliñar",
    "desaliñara": "desaliñar",
    "desaliñarais": "desaliñar",
    "desaliñaran": "desaliñar",
    "desaliñaras": "desaliñar",
    "desaliñare": "desaliñar",
    "desaliñareis": "desaliñar",
    "desaliñaremos": "desaliñar",
    "desaliñaren": "desaliñar",
    "desaliñares": "desaliñar",
    "desaliñaron": "desaliñar",
    "desaliñará": "desaliñar",
    "desaliñarán": "desaliñar",
    "desaliñarás": "desaliñar",
    "desaliñaré": "desaliñar",
    "desaliñaréis": "desaliñar",
    "desaliñaría": "desaliñar",
    "desaliñaríais": "desaliñar",
    "desaliñaríamos": "desaliñar",
    "desaliñarían": "desaliñar",
    "desaliñarías": "desaliñar",
    "desaliñas": "desaliñar",
    "desaliñase": "desaliñar",
    "desaliñaseis": "desaliñar",
    "desaliñasen": "desaliñar",
    "desaliñases": "desaliñar",
    "desaliñaste": "desaliñar",
    "desaliñasteis": "desaliñar",
    "desaliñe": "desaliñar",
    "desaliñemos": "desaliñar",
    "desaliñen": "desaliñar",
    "desaliñes": "desaliñar",
    "desaliño": "desaliñar",
    "desaliños": "desaliño",
    "desaliñábamos": "desaliñar",
    "desaliñáis": "desaliñar",
    "desaliñáramos": "desaliñar",
    "desaliñáremos": "desaliñar",
    "desaliñásemos": "desaliñar",
    "desaliñé": "desaliñar",
    "desaliñéis": "desaliñar",
    "desaliñó": "desaliñar",
    "desalma": "desalmar",
    "desalmaba": "desalmar",
    "desalmabais": "desalmar",
    "desalmaban": "desalmar",
    "desalmabas": "desalmar",
    "desalmad": "desalmar",
    "desalmada": "desalmar",
    "desalmadas": "desalmar",
    "desalmado": "desalmar",
    "desalmados": "desalmar",
    "desalmamos": "desalmar",
    "desalman": "desalmar",
    "desalmando": "desalmar",
    "desalmara": "desalmar",
    "desalmarais": "desalmar",
    "desalmaran": "desalmar",
    "desalmaras": "desalmar",
    "desalmare": "desalmar",
    "desalmareis": "desalmar",
    "desalmaremos": "desalmar",
    "desalmaren": "desalmar",
    "desalmares": "desalmar",
    "desalmaron": "desalmar",
    "desalmará": "desalmar",
    "desalmarán": "desalmar",
    "desalmarás": "desalmar",
    "desalmaré": "desalmar",
    "desalmaréis": "desalmar",
    "desalmaría": "desalmar",
    "desalmaríais": "desalmar",
    "desalmaríamos": "desalmar",
    "desalmarían": "desalmar",
    "desalmarías": "desalmar",
    "desalmas": "desalmar",
    "desalmase": "desalmar",
    "desalmaseis": "desalmar",
    "desalmasen": "desalmar",
    "desalmases": "desalmar",
    "desalmaste": "desalmar",
    "desalmasteis": "desalmar",
    "desalme": "desalmar",
    "desalmemos": "desalmar",
    "desalmen": "desalmar",
    "desalmes": "desalmar",
    "desalmidona": "desalmidonar",
    "desalmidonaba": "desalmidonar",
    "desalmidonabais": "desalmidonar",
    "desalmidonaban": "desalmidonar",
    "desalmidonabas": "desalmidonar",
    "desalmidonad": "desalmidonar",
    "desalmidonada": "desalmidonar",
    "desalmidonadas": "desalmidonar",
    "desalmidonado": "desalmidonar",
    "desalmidonados": "desalmidonar",
    "desalmidonamos": "desalmidonar",
    "desalmidonan": "desalmidonar",
    "desalmidonando": "desalmidonar",
    "desalmidonara": "desalmidonar",
    "desalmidonarais": "desalmidonar",
    "desalmidonaran": "desalmidonar",
    "desalmidonaras": "desalmidonar",
    "desalmidonare": "desalmidonar",
    "desalmidonareis": "desalmidonar",
    "desalmidonaremos": "desalmidonar",
    "desalmidonaren": "desalmidonar",
    "desalmidonares": "desalmidonar",
    "desalmidonaron": "desalmidonar",
    "desalmidonará": "desalmidonar",
    "desalmidonarán": "desalmidonar",
    "desalmidonarás": "desalmidonar",
    "desalmidonaré": "desalmidonar",
    "desalmidonaréis": "desalmidonar",
    "desalmidonaría": "desalmidonar",
    "desalmidonaríais": "desalmidonar",
    "desalmidonaríamos": "desalmidonar",
    "desalmidonarían": "desalmidonar",
    "desalmidonarías": "desalmidonar",
    "desalmidonas": "desalmidonar",
    "desalmidonase": "desalmidonar",
    "desalmidonaseis": "desalmidonar",
    "desalmidonasen": "desalmidonar",
    "desalmidonases": "desalmidonar",
    "desalmidonaste": "desalmidonar",
    "desalmidonasteis": "desalmidonar",
    "desalmidone": "desalmidonar",
    "desalmidonemos": "desalmidonar",
    "desalmidonen": "desalmidonar",
    "desalmidones": "desalmidonar",
    "desalmidono": "desalmidonar",
    "desalmidonábamos": "desalmidonar",
    "desalmidonáis": "desalmidonar",
    "desalmidonáramos": "desalmidonar",
    "desalmidonáremos": "desalmidonar",
    "desalmidonásemos": "desalmidonar",
    "desalmidoné": "desalmidonar",
    "desalmidonéis": "desalmidonar",
    "desalmidonó": "desalmidonar",
    "desalmo": "desalmar",
    "desalmábamos": "desalmar",
    "desalmáis": "desalmar",
    "desalmáramos": "desalmar",
    "desalmáremos": "desalmar",
    "desalmásemos": "desalmar",
    "desalmé": "desalmar",
    "desalméis": "desalmar",
    "desalmó": "desalmar",
    "desalo": "desalar",
    "desaloja": "desalojar",
    "desalojaba": "desalojar",
    "desalojabais": "desalojar",
    "desalojaban": "desalojar",
    "desalojabas": "desalojar",
    "desalojad": "desalojar",
    "desalojada": "desalojar",
    "desalojadas": "desalojar",
    "desalojado": "desalojar",
    "desalojados": "desalojar",
    "desalojamientos": "desalojamiento",
    "desalojamos": "desalojar",
    "desalojan": "desalojar",
    "desalojando": "desalojar",
    "desalojara": "desalojar",
    "desalojarais": "desalojar",
    "desalojaran": "desalojar",
    "desalojaras": "desalojar",
    "desalojare": "desalojar",
    "desalojareis": "desalojar",
    "desalojaremos": "desalojar",
    "desalojaren": "desalojar",
    "desalojares": "desalojar",
    "desalojaron": "desalojar",
    "desalojará": "desalojar",
    "desalojarán": "desalojar",
    "desalojarás": "desalojar",
    "desalojaré": "desalojar",
    "desalojaréis": "desalojar",
    "desalojaría": "desalojar",
    "desalojaríais": "desalojar",
    "desalojaríamos": "desalojar",
    "desalojarían": "desalojar",
    "desalojarías": "desalojar",
    "desalojas": "desalojar",
    "desalojase": "desalojar",
    "desalojaseis": "desalojar",
    "desalojasen": "desalojar",
    "desalojases": "desalojar",
    "desalojaste": "desalojar",
    "desalojasteis": "desalojar",
    "desaloje": "desalojar",
    "desalojemos": "desalojar",
    "desalojen": "desalojar",
    "desalojes": "desalojar",
    "desalojo": "desalojar",
    "desalojos": "desalojo",
    "desalojábamos": "desalojar",
    "desalojáis": "desalojar",
    "desalojáramos": "desalojar",
    "desalojáremos": "desalojar",
    "desalojásemos": "desalojar",
    "desalojé": "desalojar",
    "desalojéis": "desalojar",
    "desalojó": "desalojar",
    "desalquila": "desalquilar",
    "desalquilaba": "desalquilar",
    "desalquilabais": "desalquilar",
    "desalquilaban": "desalquilar",
    "desalquilabas": "desalquilar",
    "desalquilad": "desalquilar",
    "desalquilada": "desalquilar",
    "desalquiladas": "desalquilar",
    "desalquilado": "desalquilar",
    "desalquilados": "desalquilar",
    "desalquilamos": "desalquilar",
    "desalquilan": "desalquilar",
    "desalquilando": "desalquilar",
    "desalquilara": "desalquilar",
    "desalquilarais": "desalquilar",
    "desalquilaran": "desalquilar",
    "desalquilaras": "desalquilar",
    "desalquilare": "desalquilar",
    "desalquilareis": "desalquilar",
    "desalquilaremos": "desalquilar",
    "desalquilaren": "desalquilar",
    "desalquilares": "desalquilar",
    "desalquilaron": "desalquilar",
    "desalquilará": "desalquilar",
    "desalquilarán": "desalquilar",
    "desalquilarás": "desalquilar",
    "desalquilaré": "desalquilar",
    "desalquilaréis": "desalquilar",
    "desalquilaría": "desalquilar",
    "desalquilaríais": "desalquilar",
    "desalquilaríamos": "desalquilar",
    "desalquilarían": "desalquilar",
    "desalquilarías": "desalquilar",
    "desalquilas": "desalquilar",
    "desalquilase": "desalquilar",
    "desalquilaseis": "desalquilar",
    "desalquilasen": "desalquilar",
    "desalquilases": "desalquilar",
    "desalquilaste": "desalquilar",
    "desalquilasteis": "desalquilar",
    "desalquile": "desalquilar",
    "desalquilemos": "desalquilar",
    "desalquilen": "desalquilar",
    "desalquiles": "desalquilar",
    "desalquilo": "desalquilar",
    "desalquilábamos": "desalquilar",
    "desalquiláis": "desalquilar",
    "desalquiláramos": "desalquilar",
    "desalquiláremos": "desalquilar",
    "desalquilásemos": "desalquilar",
    "desalquilé": "desalquilar",
    "desalquiléis": "desalquilar",
    "desalquiló": "desalquilar",
    "desalquitrana": "desalquitranar",
    "desalquitranaba": "desalquitranar",
    "desalquitranabais": "desalquitranar",
    "desalquitranaban": "desalquitranar",
    "desalquitranabas": "desalquitranar",
    "desalquitranad": "desalquitranar",
    "desalquitranada": "desalquitranar",
    "desalquitranadas": "desalquitranar",
    "desalquitranado": "desalquitranar",
    "desalquitranados": "desalquitranar",
    "desalquitranamos": "desalquitranar",
    "desalquitranan": "desalquitranar",
    "desalquitranando": "desalquitranar",
    "desalquitranara": "desalquitranar",
    "desalquitranarais": "desalquitranar",
    "desalquitranaran": "desalquitranar",
    "desalquitranaras": "desalquitranar",
    "desalquitranare": "desalquitranar",
    "desalquitranareis": "desalquitranar",
    "desalquitranaremos": "desalquitranar",
    "desalquitranaren": "desalquitranar",
    "desalquitranares": "desalquitranar",
    "desalquitranaron": "desalquitranar",
    "desalquitranará": "desalquitranar",
    "desalquitranarán": "desalquitranar",
    "desalquitranarás": "desalquitranar",
    "desalquitranaré": "desalquitranar",
    "desalquitranaréis": "desalquitranar",
    "desalquitranaría": "desalquitranar",
    "desalquitranaríais": "desalquitranar",
    "desalquitranaríamos": "desalquitranar",
    "desalquitranarían": "desalquitranar",
    "desalquitranarías": "desalquitranar",
    "desalquitranas": "desalquitranar",
    "desalquitranase": "desalquitranar",
    "desalquitranaseis": "desalquitranar",
    "desalquitranasen": "desalquitranar",
    "desalquitranases": "desalquitranar",
    "desalquitranaste": "desalquitranar",
    "desalquitranasteis": "desalquitranar",
    "desalquitrane": "desalquitranar",
    "desalquitranemos": "desalquitranar",
    "desalquitranen": "desalquitranar",
    "desalquitranes": "desalquitranar",
    "desalquitrano": "desalquitranar",
    "desalquitranábamos": "desalquitranar",
    "desalquitranáis": "desalquitranar",
    "desalquitranáramos": "desalquitranar",
    "desalquitranáremos": "desalquitranar",
    "desalquitranásemos": "desalquitranar",
    "desalquitrané": "desalquitranar",
    "desalquitranéis": "desalquitranar",
    "desalquitranó": "desalquitranar",
    "desaltera": "desalterar",
    "desalteraba": "desalterar",
    "desalterabais": "desalterar",
    "desalteraban": "desalterar",
    "desalterabas": "desalterar",
    "desalteraciones": "desalteración",
    "desalterad": "desalterar",
    "desalterada": "desalterar",
    "desalteradas": "desalterar",
    "desalterado": "desalterar",
    "desalterados": "desalterar",
    "desalteramos": "desalterar",
    "desalteran": "desalterar",
    "desalterando": "desalterar",
    "desalterara": "desalterar",
    "desalterarais": "desalterar",
    "desalteraran": "desalterar",
    "desalteraras": "desalterar",
    "desalterare": "desalterar",
    "desalterareis": "desalterar",
    "desalteraremos": "desalterar",
    "desalteraren": "desalterar",
    "desalterares": "desalterar",
    "desalteraron": "desalterar",
    "desalterará": "desalterar",
    "desalterarán": "desalterar",
    "desalterarás": "desalterar",
    "desalteraré": "desalterar",
    "desalteraréis": "desalterar",
    "desalteraría": "desalterar",
    "desalteraríais": "desalterar",
    "desalteraríamos": "desalterar",
    "desalterarían": "desalterar",
    "desalterarías": "desalterar",
    "desalteras": "desalterar",
    "desalterase": "desalterar",
    "desalteraseis": "desalterar",
    "desalterasen": "desalterar",
    "desalterases": "desalterar",
    "desalteraste": "desalterar",
    "desalterasteis": "desalterar",
    "desaltere": "desalterar",
    "desalteremos": "desalterar",
    "desalteren": "desalterar",
    "desalteres": "desalterar",
    "desaltero": "desalterar",
    "desalterábamos": "desalterar",
    "desalteráis": "desalterar",
    "desalteráramos": "desalterar",
    "desalteráremos": "desalterar",
    "desalterásemos": "desalterar",
    "desalteré": "desalterar",
    "desalteréis": "desalterar",
    "desalteró": "desalterar",
    "desalumbrada": "desalumbrado",
    "desalumbradas": "desalumbrado",
    "desalumbrados": "desalumbrado",
    "desalumbramientos": "desalumbramiento",
    "desaluminizaciones": "desaluminización",
    "desalábamos": "desalar",
    "desaláis": "desalar",
    "desaláramos": "desalar",
    "desaláremos": "desalar",
    "desalásemos": "desalar",
    "desalé": "desalar",
    "desaléis": "desalar",
    "desaló": "desalar",
    "desama": "desamar",
    "desamaba": "desamar",
    "desamabais": "desamar",
    "desamaban": "desamar",
    "desamabas": "desamar",
    "desamad": "desamar",
    "desamada": "desamar",
    "desamadas": "desamar",
    "desamado": "desamar",
    "desamados": "desamar",
    "desamamos": "desamar",
    "desaman": "desamar",
    "desamando": "desamar",
    "desamara": "desamar",
    "desamarais": "desamar",
    "desamaran": "desamar",
    "desamaras": "desamar",
    "desamare": "desamar",
    "desamareis": "desamar",
    "desamaremos": "desamar",
    "desamaren": "desamar",
    "desamares": "desamar",
    "desamaron": "desamar",
    "desamarra": "desamarrar",
    "desamarraba": "desamarrar",
    "desamarrabais": "desamarrar",
    "desamarraban": "desamarrar",
    "desamarrabas": "desamarrar",
    "desamarrad": "desamarrar",
    "desamarrada": "desamarrar",
    "desamarradas": "desamarrar",
    "desamarrado": "desamarrar",
    "desamarrados": "desamarrar",
    "desamarramos": "desamarrar",
    "desamarran": "desamarrar",
    "desamarrando": "desamarrar",
    "desamarrara": "desamarrar",
    "desamarrarais": "desamarrar",
    "desamarraran": "desamarrar",
    "desamarraras": "desamarrar",
    "desamarrare": "desamarrar",
    "desamarrareis": "desamarrar",
    "desamarraremos": "desamarrar",
    "desamarraren": "desamarrar",
    "desamarrares": "desamarrar",
    "desamarraron": "desamarrar",
    "desamarrará": "desamarrar",
    "desamarrarán": "desamarrar",
    "desamarrarás": "desamarrar",
    "desamarraré": "desamarrar",
    "desamarraréis": "desamarrar",
    "desamarraría": "desamarrar",
    "desamarraríais": "desamarrar",
    "desamarraríamos": "desamarrar",
    "desamarrarían": "desamarrar",
    "desamarrarías": "desamarrar",
    "desamarras": "desamarrar",
    "desamarrase": "desamarrar",
    "desamarraseis": "desamarrar",
    "desamarrasen": "desamarrar",
    "desamarrases": "desamarrar",
    "desamarraste": "desamarrar",
    "desamarrasteis": "desamarrar",
    "desamarre": "desamarrar",
    "desamarremos": "desamarrar",
    "desamarren": "desamarrar",
    "desamarres": "desamarrar",
    "desamarro": "desamarrar",
    "desamarrábamos": "desamarrar",
    "desamarráis": "desamarrar",
    "desamarráramos": "desamarrar",
    "desamarráremos": "desamarrar",
    "desamarrásemos": "desamarrar",
    "desamarré": "desamarrar",
    "desamarréis": "desamarrar",
    "desamarró": "desamarrar",
    "desamartelamientos": "desamartelamiento",
    "desamará": "desamar",
    "desamarán": "desamar",
    "desamarás": "desamar",
    "desamaré": "desamar",
    "desamaréis": "desamar",
    "desamaría": "desamar",
    "desamaríais": "desamar",
    "desamaríamos": "desamar",
    "desamarían": "desamar",
    "desamarías": "desamar",
    "desamas": "desamar",
    "desamasada": "desamasado",
    "desamasadas": "desamasado",
    "desamasados": "desamasado",
    "desamase": "desamar",
    "desamaseis": "desamar",
    "desamasen": "desamar",
    "desamases": "desamar",
    "desamaste": "desamar",
    "desamasteis": "desamar",
    "desambientada": "desambientado",
    "desambientadas": "desambientado",
    "desambientados": "desambientado",
    "desambiguaciones": "desambiguación",
    "desame": "desamar",
    "desamelga": "desamelgar",
    "desamelgaba": "desamelgar",
    "desamelgabais": "desamelgar",
    "desamelgaban": "desamelgar",
    "desamelgabas": "desamelgar",
    "desamelgad": "desamelgar",
    "desamelgada": "desamelgar",
    "desamelgadas": "desamelgar",
    "desamelgado": "desamelgar",
    "desamelgados": "desamelgar",
    "desamelgamos": "desamelgar",
    "desamelgan": "desamelgar",
    "desamelgando": "desamelgar",
    "desamelgara": "desamelgar",
    "desamelgarais": "desamelgar",
    "desamelgaran": "desamelgar",
    "desamelgaras": "desamelgar",
    "desamelgare": "desamelgar",
    "desamelgareis": "desamelgar",
    "desamelgaremos": "desamelgar",
    "desamelgaren": "desamelgar",
    "desamelgares": "desamelgar",
    "desamelgaron": "desamelgar",
    "desamelgará": "desamelgar",
    "desamelgarán": "desamelgar",
    "desamelgarás": "desamelgar",
    "desamelgaré": "desamelgar",
    "desamelgaréis": "desamelgar",
    "desamelgaría": "desamelgar",
    "desamelgaríais": "desamelgar",
    "desamelgaríamos": "desamelgar",
    "desamelgarían": "desamelgar",
    "desamelgarías": "desamelgar",
    "desamelgas": "desamelgar",
    "desamelgase": "desamelgar",
    "desamelgaseis": "desamelgar",
    "desamelgasen": "desamelgar",
    "desamelgases": "desamelgar",
    "desamelgaste": "desamelgar",
    "desamelgasteis": "desamelgar",
    "desamelgo": "desamelgar",
    "desamelgue": "desamelgar",
    "desamelguemos": "desamelgar",
    "desamelguen": "desamelgar",
    "desamelgues": "desamelgar",
    "desamelgué": "desamelgar",
    "desamelguéis": "desamelgar",
    "desamelgábamos": "desamelgar",
    "desamelgáis": "desamelgar",
    "desamelgáramos": "desamelgar",
    "desamelgáremos": "desamelgar",
    "desamelgásemos": "desamelgar",
    "desamelgó": "desamelgar",
    "desamemos": "desamar",
    "desamen": "desamar",
    "desames": "desamar",
    "desaminasas": "desaminasa",
    "desamista": "desamistar",
    "desamistaba": "desamistar",
    "desamistabais": "desamistar",
    "desamistaban": "desamistar",
    "desamistabas": "desamistar",
    "desamistad": "desamistar",
    "desamistada": "desamistar",
    "desamistadas": "desamistar",
    "desamistado": "desamistar",
    "desamistados": "desamistar",
    "desamistamos": "desamistar",
    "desamistan": "desamistar",
    "desamistando": "desamistar",
    "desamistara": "desamistar",
    "desamistarais": "desamistar",
    "desamistaran": "desamistar",
    "desamistaras": "desamistar",
    "desamistare": "desamistar",
    "desamistareis": "desamistar",
    "desamistaremos": "desamistar",
    "desamistaren": "desamistar",
    "desamistares": "desamistar",
    "desamistaron": "desamistar",
    "desamistará": "desamistar",
    "desamistarán": "desamistar",
    "desamistarás": "desamistar",
    "desamistaré": "desamistar",
    "desamistaréis": "desamistar",
    "desamistaría": "desamistar",
    "desamistaríais": "desamistar",
    "desamistaríamos": "desamistar",
    "desamistarían": "desamistar",
    "desamistarías": "desamistar",
    "desamistas": "desamistar",
    "desamistase": "desamistar",
    "desamistaseis": "desamistar",
    "desamistasen": "desamistar",
    "desamistases": "desamistar",
    "desamistaste": "desamistar",
    "desamistasteis": "desamistar",
    "desamiste": "desamistar",
    "desamistemos": "desamistar",
    "desamisten": "desamistar",
    "desamistes": "desamistar",
    "desamisto": "desamistar",
    "desamistábamos": "desamistar",
    "desamistáis": "desamistar",
    "desamistáramos": "desamistar",
    "desamistáremos": "desamistar",
    "desamistásemos": "desamistar",
    "desamisté": "desamistar",
    "desamistéis": "desamistar",
    "desamistó": "desamistar",
    "desamo": "desamar",
    "desamoblaba": "desamoblar",
    "desamoblabais": "desamoblar",
    "desamoblaban": "desamoblar",
    "desamoblabas": "desamoblar",
    "desamoblad": "desamoblar",
    "desamoblada": "desamoblar",
    "desamobladas": "desamoblar",
    "desamoblado": "desamoblar",
    "desamoblados": "desamoblar",
    "desamoblamos": "desamoblar",
    "desamoblando": "desamoblar",
    "desamoblara": "desamoblar",
    "desamoblarais": "desamoblar",
    "desamoblaran": "desamoblar",
    "desamoblaras": "desamoblar",
    "desamoblare": "desamoblar",
    "desamoblareis": "desamoblar",
    "desamoblaremos": "desamoblar",
    "desamoblaren": "desamoblar",
    "desamoblares": "desamoblar",
    "desamoblaron": "desamoblar",
    "desamoblará": "desamoblar",
    "desamoblarán": "desamoblar",
    "desamoblarás": "desamoblar",
    "desamoblaré": "desamoblar",
    "desamoblaréis": "desamoblar",
    "desamoblaría": "desamoblar",
    "desamoblaríais": "desamoblar",
    "desamoblaríamos": "desamoblar",
    "desamoblarían": "desamoblar",
    "desamoblarías": "desamoblar",
    "desamoblase": "desamoblar",
    "desamoblaseis": "desamoblar",
    "desamoblasen": "desamoblar",
    "desamoblases": "desamoblar",
    "desamoblaste": "desamoblar",
    "desamoblasteis": "desamoblar",
    "desamoblemos": "desamoblar",
    "desamoblábamos": "desamoblar",
    "desamobláis": "desamoblar",
    "desamobláramos": "desamoblar",
    "desamobláremos": "desamoblar",
    "desamoblásemos": "desamoblar",
    "desamoblé": "desamoblar",
    "desamobléis": "desamoblar",
    "desamobló": "desamoblar",
    "desamolda": "desamoldar",
    "desamoldaba": "desamoldar",
    "desamoldabais": "desamoldar",
    "desamoldaban": "desamoldar",
    "desamoldabas": "desamoldar",
    "desamoldad": "desamoldar",
    "desamoldada": "desamoldar",
    "desamoldadas": "desamoldar",
    "desamoldado": "desamoldar",
    "desamoldados": "desamoldar",
    "desamoldamos": "desamoldar",
    "desamoldan": "desamoldar",
    "desamoldando": "desamoldar",
    "desamoldara": "desamoldar",
    "desamoldarais": "desamoldar",
    "desamoldaran": "desamoldar",
    "desamoldaras": "desamoldar",
    "desamoldare": "desamoldar",
    "desamoldareis": "desamoldar",
    "desamoldaremos": "desamoldar",
    "desamoldaren": "desamoldar",
    "desamoldares": "desamoldar",
    "desamoldaron": "desamoldar",
    "desamoldará": "desamoldar",
    "desamoldarán": "desamoldar",
    "desamoldarás": "desamoldar",
    "desamoldaré": "desamoldar",
    "desamoldaréis": "desamoldar",
    "desamoldaría": "desamoldar",
    "desamoldaríais": "desamoldar",
    "desamoldaríamos": "desamoldar",
    "desamoldarían": "desamoldar",
    "desamoldarías": "desamoldar",
    "desamoldas": "desamoldar",
    "desamoldase": "desamoldar",
    "desamoldaseis": "desamoldar",
    "desamoldasen": "desamoldar",
    "desamoldases": "desamoldar",
    "desamoldaste": "desamoldar",
    "desamoldasteis": "desamoldar",
    "desamolde": "desamoldar",
    "desamoldemos": "desamoldar",
    "desamolden": "desamoldar",
    "desamoldes": "desamoldar",
    "desamoldo": "desamoldar",
    "desamoldábamos": "desamoldar",
    "desamoldáis": "desamoldar",
    "desamoldáramos": "desamoldar",
    "desamoldáremos": "desamoldar",
    "desamoldásemos": "desamoldar",
    "desamoldé": "desamoldar",
    "desamoldéis": "desamoldar",
    "desamoldó": "desamoldar",
    "desamontona": "desamontonar",
    "desamontonaba": "desamontonar",
    "desamontonabais": "desamontonar",
    "desamontonaban": "desamontonar",
    "desamontonabas": "desamontonar",
    "desamontonad": "desamontonar",
    "desamontonada": "desamontonar",
    "desamontonadas": "desamontonar",
    "desamontonado": "desamontonar",
    "desamontonados": "desamontonar",
    "desamontonamos": "desamontonar",
    "desamontonan": "desamontonar",
    "desamontonando": "desamontonar",
    "desamontonara": "desamontonar",
    "desamontonarais": "desamontonar",
    "desamontonaran": "desamontonar",
    "desamontonaras": "desamontonar",
    "desamontonare": "desamontonar",
    "desamontonareis": "desamontonar",
    "desamontonaremos": "desamontonar",
    "desamontonaren": "desamontonar",
    "desamontonares": "desamontonar",
    "desamontonaron": "desamontonar",
    "desamontonará": "desamontonar",
    "desamontonarán": "desamontonar",
    "desamontonarás": "desamontonar",
    "desamontonaré": "desamontonar",
    "desamontonaréis": "desamontonar",
    "desamontonaría": "desamontonar",
    "desamontonaríais": "desamontonar",
    "desamontonaríamos": "desamontonar",
    "desamontonarían": "desamontonar",
    "desamontonarías": "desamontonar",
    "desamontonas": "desamontonar",
    "desamontonase": "desamontonar",
    "desamontonaseis": "desamontonar",
    "desamontonasen": "desamontonar",
    "desamontonases": "desamontonar",
    "desamontonaste": "desamontonar",
    "desamontonasteis": "desamontonar",
    "desamontone": "desamontonar",
    "desamontonemos": "desamontonar",
    "desamontonen": "desamontonar",
    "desamontones": "desamontonar",
    "desamontono": "desamontonar",
    "desamontonábamos": "desamontonar",
    "desamontonáis": "desamontonar",
    "desamontonáramos": "desamontonar",
    "desamontonáremos": "desamontonar",
    "desamontonásemos": "desamontonar",
    "desamontoné": "desamontonar",
    "desamontonéis": "desamontonar",
    "desamontonó": "desamontonar",
    "desamora": "desamorar",
    "desamoraba": "desamorar",
    "desamorabais": "desamorar",
    "desamoraban": "desamorar",
    "desamorabas": "desamorar",
    "desamorad": "desamorar",
    "desamorada": "desamorar",
    "desamoradas": "desamorar",
    "desamorado": "desamorar",
    "desamorados": "desamorar",
    "desamoramos": "desamorar",
    "desamoran": "desamorar",
    "desamorando": "desamorar",
    "desamorara": "desamorar",
    "desamorarais": "desamorar",
    "desamoraran": "desamorar",
    "desamoraras": "desamorar",
    "desamorare": "desamorar",
    "desamorareis": "desamorar",
    "desamoraremos": "desamorar",
    "desamoraren": "desamorar",
    "desamorares": "desamorar",
    "desamoraron": "desamorar",
    "desamorará": "desamorar",
    "desamorarán": "desamorar",
    "desamorarás": "desamorar",
    "desamoraré": "desamorar",
    "desamoraréis": "desamorar",
    "desamoraría": "desamorar",
    "desamoraríais": "desamorar",
    "desamoraríamos": "desamorar",
    "desamorarían": "desamorar",
    "desamorarías": "desamorar",
    "desamoras": "desamorar",
    "desamorase": "desamorar",
    "desamoraseis": "desamorar",
    "desamorasen": "desamorar",
    "desamorases": "desamorar",
    "desamoraste": "desamorar",
    "desamorasteis": "desamorar",
    "desamore": "desamorar",
    "desamoremos": "desamorar",
    "desamoren": "desamorar",
    "desamores": "desamorar",
    "desamoro": "desamorar",
    "desamortice": "desamortizar",
    "desamorticemos": "desamortizar",
    "desamorticen": "desamortizar",
    "desamortices": "desamortizar",
    "desamorticé": "desamortizar",
    "desamorticéis": "desamortizar",
    "desamortiza": "desamortizar",
    "desamortizaba": "desamortizar",
    "desamortizabais": "desamortizar",
    "desamortizaban": "desamortizar",
    "desamortizabas": "desamortizar",
    "desamortizables": "desamortizable",
    "desamortizaciones": "desamortización",
    "desamortizad": "desamortizar",
    "desamortizada": "desamortizar",
    "desamortizadas": "desamortizar",
    "desamortizado": "desamortizar",
    "desamortizadora": "desamortizador",
    "desamortizadoras": "desamortizador",
    "desamortizadores": "desamortizador",
    "desamortizados": "desamortizar",
    "desamortizamos": "desamortizar",
    "desamortizan": "desamortizar",
    "desamortizando": "desamortizar",
    "desamortizara": "desamortizar",
    "desamortizarais": "desamortizar",
    "desamortizaran": "desamortizar",
    "desamortizaras": "desamortizar",
    "desamortizare": "desamortizar",
    "desamortizareis": "desamortizar",
    "desamortizaremos": "desamortizar",
    "desamortizaren": "desamortizar",
    "desamortizares": "desamortizar",
    "desamortizaron": "desamortizar",
    "desamortizará": "desamortizar",
    "desamortizarán": "desamortizar",
    "desamortizarás": "desamortizar",
    "desamortizaré": "desamortizar",
    "desamortizaréis": "desamortizar",
    "desamortizaría": "desamortizar",
    "desamortizaríais": "desamortizar",
    "desamortizaríamos": "desamortizar",
    "desamortizarían": "desamortizar",
    "desamortizarías": "desamortizar",
    "desamortizas": "desamortizar",
    "desamortizase": "desamortizar",
    "desamortizaseis": "desamortizar",
    "desamortizasen": "desamortizar",
    "desamortizases": "desamortizar",
    "desamortizaste": "desamortizar",
    "desamortizasteis": "desamortizar",
    "desamortizo": "desamortizar",
    "desamortizábamos": "desamortizar",
    "desamortizáis": "desamortizar",
    "desamortizáramos": "desamortizar",
    "desamortizáremos": "desamortizar",
    "desamortizásemos": "desamortizar",
    "desamortizó": "desamortizar",
    "desamorábamos": "desamorar",
    "desamoráis": "desamorar",
    "desamoráramos": "desamorar",
    "desamoráremos": "desamorar",
    "desamorásemos": "desamorar",
    "desamoré": "desamorar",
    "desamoréis": "desamorar",
    "desamoró": "desamorar",
    "desamotinamientos": "desamotinamiento",
    "desampara": "desamparar",
    "desamparaba": "desamparar",
    "desamparabais": "desamparar",
    "desamparaban": "desamparar",
    "desamparabas": "desamparar",
    "desamparad": "desamparar",
    "desamparada": "desamparar",
    "desamparadas": "desamparar",
    "desamparado": "desamparar",
    "desamparados": "desamparar",
    "desamparamos": "desamparar",
    "desamparan": "desamparar",
    "desamparando": "desamparar",
    "desamparara": "desamparar",
    "desampararais": "desamparar",
    "desampararan": "desamparar",
    "desampararas": "desamparar",
    "desamparare": "desamparar",
    "desamparareis": "desamparar",
    "desampararemos": "desamparar",
    "desampararen": "desamparar",
    "desamparares": "desamparar",
    "desampararon": "desamparar",
    "desamparará": "desamparar",
    "desampararán": "desamparar",
    "desampararás": "desamparar",
    "desampararé": "desamparar",
    "desampararéis": "desamparar",
    "desampararía": "desamparar",
    "desampararíais": "desamparar",
    "desampararíamos": "desamparar",
    "desampararían": "desamparar",
    "desampararías": "desamparar",
    "desamparas": "desamparar",
    "desamparase": "desamparar",
    "desamparaseis": "desamparar",
    "desamparasen": "desamparar",
    "desamparases": "desamparar",
    "desamparaste": "desamparar",
    "desamparasteis": "desamparar",
    "desampare": "desamparar",
    "desamparemos": "desamparar",
    "desamparen": "desamparar",
    "desampares": "desamparar",
    "desamparo": "desamparar",
    "desamparos": "desamparo",
    "desamparábamos": "desamparar",
    "desamparáis": "desamparar",
    "desamparáramos": "desamparar",
    "desamparáremos": "desamparar",
    "desamparásemos": "desamparar",
    "desamparé": "desamparar",
    "desamparéis": "desamparar",
    "desamparó": "desamparar",
    "desamuebla": "desamueblar",
    "desamueblaba": "desamueblar",
    "desamueblabais": "desamueblar",
    "desamueblaban": "desamueblar",
    "desamueblabas": "desamueblar",
    "desamueblad": "desamueblar",
    "desamueblada": "desamueblar",
    "desamuebladas": "desamueblar",
    "desamueblado": "desamueblar",
    "desamueblados": "desamueblar",
    "desamueblamos": "desamueblar",
    "desamueblan": "desamueblar",
    "desamueblando": "desamueblar",
    "desamueblara": "desamueblar",
    "desamueblarais": "desamueblar",
    "desamueblaran": "desamueblar",
    "desamueblaras": "desamueblar",
    "desamueblare": "desamueblar",
    "desamueblareis": "desamueblar",
    "desamueblaremos": "desamueblar",
    "desamueblaren": "desamueblar",
    "desamueblares": "desamueblar",
    "desamueblaron": "desamueblar",
    "desamueblará": "desamueblar",
    "desamueblarán": "desamueblar",
    "desamueblarás": "desamueblar",
    "desamueblaré": "desamueblar",
    "desamueblaréis": "desamueblar",
    "desamueblaría": "desamueblar",
    "desamueblaríais": "desamueblar",
    "desamueblaríamos": "desamueblar",
    "desamueblarían": "desamueblar",
    "desamueblarías": "desamueblar",
    "desamueblas": "desamueblar",
    "desamueblase": "desamueblar",
    "desamueblaseis": "desamueblar",
    "desamueblasen": "desamueblar",
    "desamueblases": "desamueblar",
    "desamueblaste": "desamueblar",
    "desamueblasteis": "desamueblar",
    "desamueble": "desamueblar",
    "desamueblemos": "desamueblar",
    "desamueblen": "desamueblar",
    "desamuebles": "desamueblar",
    "desamueblo": "desamueblar",
    "desamueblábamos": "desamueblar",
    "desamuebláis": "desamueblar",
    "desamuebláramos": "desamueblar",
    "desamuebláremos": "desamueblar",
    "desamueblásemos": "desamueblar",
    "desamueblé": "desamueblar",
    "desamuebléis": "desamueblar",
    "desamuebló": "desamueblar",
    "desamábamos": "desamar",
    "desamáis": "desamar",
    "desamáramos": "desamar",
    "desamáremos": "desamar",
    "desamásemos": "desamar",
    "desamé": "desamar",
    "desaméis": "desamar",
    "desamó": "desamar",
    "desancla": "desanclar",
    "desanclaba": "desanclar",
    "desanclabais": "desanclar",
    "desanclaban": "desanclar",
    "desanclabas": "desanclar",
    "desanclad": "desanclar",
    "desanclada": "desanclar",
    "desancladas": "desanclar",
    "desanclado": "desanclar",
    "desanclados": "desanclar",
    "desanclamos": "desanclar",
    "desanclan": "desanclar",
    "desanclando": "desanclar",
    "desanclara": "desanclar",
    "desanclarais": "desanclar",
    "desanclaran": "desanclar",
    "desanclaras": "desanclar",
    "desanclare": "desanclar",
    "desanclareis": "desanclar",
    "desanclaremos": "desanclar",
    "desanclaren": "desanclar",
    "desanclares": "desanclar",
    "desanclaron": "desanclar",
    "desanclará": "desanclar",
    "desanclarán": "desanclar",
    "desanclarás": "desanclar",
    "desanclaré": "desanclar",
    "desanclaréis": "desanclar",
    "desanclaría": "desanclar",
    "desanclaríais": "desanclar",
    "desanclaríamos": "desanclar",
    "desanclarían": "desanclar",
    "desanclarías": "desanclar",
    "desanclas": "desanclar",
    "desanclase": "desanclar",
    "desanclaseis": "desanclar",
    "desanclasen": "desanclar",
    "desanclases": "desanclar",
    "desanclaste": "desanclar",
    "desanclasteis": "desanclar",
    "desancle": "desanclar",
    "desanclemos": "desanclar",
    "desanclen": "desanclar",
    "desancles": "desanclar",
    "desanclo": "desanclar",
    "desanclábamos": "desanclar",
    "desancláis": "desanclar",
    "desancláramos": "desanclar",
    "desancláremos": "desanclar",
    "desanclásemos": "desanclar",
    "desanclé": "desanclar",
    "desancléis": "desanclar",
    "desancló": "desanclar",
    "desancora": "desancorar",
    "desancoraba": "desancorar",
    "desancorabais": "desancorar",
    "desancoraban": "desancorar",
    "desancorabas": "desancorar",
    "desancorad": "desancorar",
    "desancorada": "desancorar",
    "desancoradas": "desancorar",
    "desancorado": "desancorar",
    "desancorados": "desancorar",
    "desancoramos": "desancorar",
    "desancoran": "desancorar",
    "desancorando": "desancorar",
    "desancorara": "desancorar",
    "desancorarais": "desancorar",
    "desancoraran": "desancorar",
    "desancoraras": "desancorar",
    "desancorare": "desancorar",
    "desancorareis": "desancorar",
    "desancoraremos": "desancorar",
    "desancoraren": "desancorar",
    "desancorares": "desancorar",
    "desancoraron": "desancorar",
    "desancorará": "desancorar",
    "desancorarán": "desancorar",
    "desancorarás": "desancorar",
    "desancoraré": "desancorar",
    "desancoraréis": "desancorar",
    "desancoraría": "desancorar",
    "desancoraríais": "desancorar",
    "desancoraríamos": "desancorar",
    "desancorarían": "desancorar",
    "desancorarías": "desancorar",
    "desancoras": "desancorar",
    "desancorase": "desancorar",
    "desancoraseis": "desancorar",
    "desancorasen": "desancorar",
    "desancorases": "desancorar",
    "desancoraste": "desancorar",
    "desancorasteis": "desancorar",
    "desancore": "desancorar",
    "desancoremos": "desancorar",
    "desancoren": "desancorar",
    "desancores": "desancorar",
    "desancoro": "desancorar",
    "desancorábamos": "desancorar",
    "desancoráis": "desancorar",
    "desancoráramos": "desancorar",
    "desancoráremos": "desancorar",
    "desancorásemos": "desancorar",
    "desancoré": "desancorar",
    "desancoréis": "desancorar",
    "desancoró": "desancorar",
    "desanda": "desandar",
    "desandaba": "desandar",
    "desandabais": "desandar",
    "desandaban": "desandar",
    "desandabas": "desandar",
    "desandad": "desandar",
    "desandada": "desandar",
    "desandadas": "desandar",
    "desandado": "desandar",
    "desandados": "desandar",
    "desandamos": "desandar",
    "desandan": "desandar",
    "desandando": "desandar",
    "desandaremos": "desandar",
    "desandará": "desandar",
    "desandarán": "desandar",
    "desandarás": "desandar",
    "desandaré": "desandar",
    "desandaréis": "desandar",
    "desandaría": "desandar",
    "desandaríais": "desandar",
    "desandaríamos": "desandar",
    "desandarían": "desandar",
    "desandarías": "desandar",
    "desandas": "desandar",
    "desande": "desandar",
    "desandemos": "desandar",
    "desanden": "desandar",
    "desandes": "desandar",
    "desando": "desandar",
    "desandrajada": "desandrajado",
    "desandrajadas": "desandrajado",
    "desandrajados": "desandrajado",
    "desanduve": "desandar",
    "desanduviera": "desandar",
    "desanduvierais": "desandar",
    "desanduvieran": "desandar",
    "desanduvieras": "desandar",
    "desanduviere": "desandar",
    "desanduviereis": "desandar",
    "desanduvieren": "desandar",
    "desanduvieres": "desandar",
    "desanduvieron": "desandar",
    "desanduviese": "desandar",
    "desanduvieseis": "desandar",
    "desanduviesen": "desandar",
    "desanduvieses": "desandar",
    "desanduvimos": "desandar",
    "desanduviste": "desandar",
    "desanduvisteis": "desandar",
    "desanduviéramos": "desandar",
    "desanduviéremos": "desandar",
    "desanduviésemos": "desandar",
    "desanduvo": "desandar",
    "desandábamos": "desandar",
    "desandáis": "desandar",
    "desandéis": "desandar",
    "desangelada": "desangelado",
    "desangeladas": "desangelado",
    "desangelados": "desangelado",
    "desangra": "desangrar",
    "desangraba": "desangrar",
    "desangrabais": "desangrar",
    "desangraban": "desangrar",
    "desangrabas": "desangrar",
    "desangrad": "desangrar",
    "desangrada": "desangrar",
    "desangradas": "desangrar",
    "desangrado": "desangrar",
    "desangrados": "desangrar",
    "desangramientos": "desangramiento",
    "desangramos": "desangrar",
    "desangran": "desangrar",
    "desangrando": "desangrar",
    "desangrara": "desangrar",
    "desangrarais": "desangrar",
    "desangraran": "desangrar",
    "desangraras": "desangrar",
    "desangrare": "desangrar",
    "desangrareis": "desangrar",
    "desangraremos": "desangrar",
    "desangraren": "desangrar",
    "desangrares": "desangrar",
    "desangraron": "desangrar",
    "desangrará": "desangrar",
    "desangrarán": "desangrar",
    "desangrarás": "desangrar",
    "desangraré": "desangrar",
    "desangraréis": "desangrar",
    "desangraría": "desangrar",
    "desangraríais": "desangrar",
    "desangraríamos": "desangrar",
    "desangrarían": "desangrar",
    "desangrarías": "desangrar",
    "desangras": "desangrar",
    "desangrase": "desangrar",
    "desangraseis": "desangrar",
    "desangrasen": "desangrar",
    "desangrases": "desangrar",
    "desangraste": "desangrar",
    "desangrasteis": "desangrar",
    "desangre": "desangrar",
    "desangremos": "desangrar",
    "desangren": "desangrar",
    "desangres": "desangrar",
    "desangro": "desangrar",
    "desangrábamos": "desangrar",
    "desangráis": "desangrar",
    "desangráramos": "desangrar",
    "desangráremos": "desangrar",
    "desangrásemos": "desangrar",
    "desangré": "desangrar",
    "desangréis": "desangrar",
    "desangró": "desangrar",
    "desanida": "desanidar",
    "desanidaba": "desanidar",
    "desanidabais": "desanidar",
    "desanidaban": "desanidar",
    "desanidabas": "desanidar",
    "desanidad": "desanidar",
    "desanidada": "desanidar",
    "desanidadas": "desanidar",
    "desanidado": "desanidar",
    "desanidados": "desanidar",
    "desanidamos": "desanidar",
    "desanidan": "desanidar",
    "desanidando": "desanidar",
    "desanidara": "desanidar",
    "desanidarais": "desanidar",
    "desanidaran": "desanidar",
    "desanidaras": "desanidar",
    "desanidare": "desanidar",
    "desanidareis": "desanidar",
    "desanidaremos": "desanidar",
    "desanidaren": "desanidar",
    "desanidares": "desanidar",
    "desanidaron": "desanidar",
    "desanidará": "desanidar",
    "desanidarán": "desanidar",
    "desanidarás": "desanidar",
    "desanidaré": "desanidar",
    "desanidaréis": "desanidar",
    "desanidaría": "desanidar",
    "desanidaríais": "desanidar",
    "desanidaríamos": "desanidar",
    "desanidarían": "desanidar",
    "desanidarías": "desanidar",
    "desanidas": "desanidar",
    "desanidase": "desanidar",
    "desanidaseis": "desanidar",
    "desanidasen": "desanidar",
    "desanidases": "desanidar",
    "desanidaste": "desanidar",
    "desanidasteis": "desanidar",
    "desanide": "desanidar",
    "desanidemos": "desanidar",
    "desaniden": "desanidar",
    "desanides": "desanidar",
    "desanido": "desanidar",
    "desanidábamos": "desanidar",
    "desanidáis": "desanidar",
    "desanidáramos": "desanidar",
    "desanidáremos": "desanidar",
    "desanidásemos": "desanidar",
    "desanidé": "desanidar",
    "desanidéis": "desanidar",
    "desanidó": "desanidar",
    "desanima": "desanimar",
    "desanimaba": "desanimar",
    "desanimabais": "desanimar",
    "desanimaban": "desanimar",
    "desanimabas": "desanimar",
    "desanimad": "desanimar",
    "desanimada": "desanimar",
    "desanimadas": "desanimar",
    "desanimado": "desanimar",
    "desanimados": "desanimar",
    "desanimamos": "desanimar",
    "desaniman": "desanimar",
    "desanimando": "desanimar",
    "desanimara": "desanimar",
    "desanimarais": "desanimar",
    "desanimaran": "desanimar",
    "desanimaras": "desanimar",
    "desanimare": "desanimar",
    "desanimareis": "desanimar",
    "desanimaremos": "desanimar",
    "desanimaren": "desanimar",
    "desanimares": "desanimar",
    "desanimaron": "desanimar",
    "desanimará": "desanimar",
    "desanimarán": "desanimar",
    "desanimarás": "desanimar",
    "desanimaré": "desanimar",
    "desanimaréis": "desanimar",
    "desanimaría": "desanimar",
    "desanimaríais": "desanimar",
    "desanimaríamos": "desanimar",
    "desanimarían": "desanimar",
    "desanimarías": "desanimar",
    "desanimas": "desanimar",
    "desanimase": "desanimar",
    "desanimaseis": "desanimar",
    "desanimasen": "desanimar",
    "desanimases": "desanimar",
    "desanimaste": "desanimar",
    "desanimasteis": "desanimar",
    "desanime": "desanimar",
    "desanimemos": "desanimar",
    "desanimen": "desanimar",
    "desanimes": "desanimar",
    "desanimo": "desanimar",
    "desanimábamos": "desanimar",
    "desanimáis": "desanimar",
    "desanimáramos": "desanimar",
    "desanimáremos": "desanimar",
    "desanimásemos": "desanimar",
    "desanimé": "desanimar",
    "desaniméis": "desanimar",
    "desanimó": "desanimar",
    "desanubla": "desanublar",
    "desanublaba": "desanublar",
    "desanublabais": "desanublar",
    "desanublaban": "desanublar",
    "desanublabas": "desanublar",
    "desanublad": "desanublar",
    "desanublada": "desanublar",
    "desanubladas": "desanublar",
    "desanublado": "desanublar",
    "desanublados": "desanublar",
    "desanublamos": "desanublar",
    "desanublan": "desanublar",
    "desanublando": "desanublar",
    "desanublara": "desanublar",
    "desanublarais": "desanublar",
    "desanublaran": "desanublar",
    "desanublaras": "desanublar",
    "desanublare": "desanublar",
    "desanublareis": "desanublar",
    "desanublaremos": "desanublar",
    "desanublaren": "desanublar",
    "desanublares": "desanublar",
    "desanublaron": "desanublar",
    "desanublará": "desanublar",
    "desanublarán": "desanublar",
    "desanublarás": "desanublar",
    "desanublaré": "desanublar",
    "desanublaréis": "desanublar",
    "desanublaría": "desanublar",
    "desanublaríais": "desanublar",
    "desanublaríamos": "desanublar",
    "desanublarían": "desanublar",
    "desanublarías": "desanublar",
    "desanublas": "desanublar",
    "desanublase": "desanublar",
    "desanublaseis": "desanublar",
    "desanublasen": "desanublar",
    "desanublases": "desanublar",
    "desanublaste": "desanublar",
    "desanublasteis": "desanublar",
    "desanuble": "desanublar",
    "desanublemos": "desanublar",
    "desanublen": "desanublar",
    "desanubles": "desanublar",
    "desanublo": "desanublar",
    "desanublábamos": "desanublar",
    "desanubláis": "desanublar",
    "desanubláramos": "desanublar",
    "desanubláremos": "desanublar",
    "desanublásemos": "desanublar",
    "desanublé": "desanublar",
    "desanubléis": "desanublar",
    "desanubló": "desanublar",
    "desanuda": "desanudar",
    "desanudaba": "desanudar",
    "desanudabais": "desanudar",
    "desanudaban": "desanudar",
    "desanudabas": "desanudar",
    "desanudad": "desanudar",
    "desanudada": "desanudar",
    "desanudadas": "desanudar",
    "desanudado": "desanudar",
    "desanudados": "desanudar",
    "desanudamos": "desanudar",
    "desanudan": "desanudar",
    "desanudando": "desanudar",
    "desanudara": "desanudar",
    "desanudarais": "desanudar",
    "desanudaran": "desanudar",
    "desanudaras": "desanudar",
    "desanudare": "desanudar",
    "desanudareis": "desanudar",
    "desanudaremos": "desanudar",
    "desanudaren": "desanudar",
    "desanudares": "desanudar",
    "desanudaron": "desanudar",
    "desanudará": "desanudar",
    "desanudarán": "desanudar",
    "desanudarás": "desanudar",
    "desanudaré": "desanudar",
    "desanudaréis": "desanudar",
    "desanudaría": "desanudar",
    "desanudaríais": "desanudar",
    "desanudaríamos": "desanudar",
    "desanudarían": "desanudar",
    "desanudarías": "desanudar",
    "desanudas": "desanudar",
    "desanudase": "desanudar",
    "desanudaseis": "desanudar",
    "desanudasen": "desanudar",
    "desanudases": "desanudar",
    "desanudaste": "desanudar",
    "desanudasteis": "desanudar",
    "desanude": "desanudar",
    "desanudemos": "desanudar",
    "desanuden": "desanudar",
    "desanudes": "desanudar",
    "desanudo": "desanudar",
    "desanudábamos": "desanudar",
    "desanudáis": "desanudar",
    "desanudáramos": "desanudar",
    "desanudáremos": "desanudar",
    "desanudásemos": "desanudar",
    "desanudé": "desanudar",
    "desanudéis": "desanudar",
    "desanudó": "desanudar",
    "desapacibilidades": "desapacibilidad",
    "desapacibles": "desapacible",
    "desapadrina": "desapadrinar",
    "desapadrinaba": "desapadrinar",
    "desapadrinabais": "desapadrinar",
    "desapadrinaban": "desapadrinar",
    "desapadrinabas": "desapadrinar",
    "desapadrinad": "desapadrinar",
    "desapadrinada": "desapadrinar",
    "desapadrinadas": "desapadrinar",
    "desapadrinado": "desapadrinar",
    "desapadrinados": "desapadrinar",
    "desapadrinamos": "desapadrinar",
    "desapadrinan": "desapadrinar",
    "desapadrinando": "desapadrinar",
    "desapadrinara": "desapadrinar",
    "desapadrinarais": "desapadrinar",
    "desapadrinaran": "desapadrinar",
    "desapadrinaras": "desapadrinar",
    "desapadrinare": "desapadrinar",
    "desapadrinareis": "desapadrinar",
    "desapadrinaremos": "desapadrinar",
    "desapadrinaren": "desapadrinar",
    "desapadrinares": "desapadrinar",
    "desapadrinaron": "desapadrinar",
    "desapadrinará": "desapadrinar",
    "desapadrinarán": "desapadrinar",
    "desapadrinarás": "desapadrinar",
    "desapadrinaré": "desapadrinar",
    "desapadrinaréis": "desapadrinar",
    "desapadrinaría": "desapadrinar",
    "desapadrinaríais": "desapadrinar",
    "desapadrinaríamos": "desapadrinar",
    "desapadrinarían": "desapadrinar",
    "desapadrinarías": "desapadrinar",
    "desapadrinas": "desapadrinar",
    "desapadrinase": "desapadrinar",
    "desapadrinaseis": "desapadrinar",
    "desapadrinasen": "desapadrinar",
    "desapadrinases": "desapadrinar",
    "desapadrinaste": "desapadrinar",
    "desapadrinasteis": "desapadrinar",
    "desapadrine": "desapadrinar",
    "desapadrinemos": "desapadrinar",
    "desapadrinen": "desapadrinar",
    "desapadrines": "desapadrinar",
    "desapadrino": "desapadrinar",
    "desapadrinábamos": "desapadrinar",
    "desapadrináis": "desapadrinar",
    "desapadrináramos": "desapadrinar",
    "desapadrináremos": "desapadrinar",
    "desapadrinásemos": "desapadrinar",
    "desapadriné": "desapadrinar",
    "desapadrinéis": "desapadrinar",
    "desapadrinó": "desapadrinar",
    "desaparea": "desaparear",
    "desapareaba": "desaparear",
    "desapareabais": "desaparear",
    "desapareaban": "desaparear",
    "desapareabas": "desaparear",
    "desaparead": "desaparear",
    "desapareada": "desaparear",
    "desapareadas": "desaparear",
    "desapareado": "desaparear",
    "desapareados": "desaparear",
    "desapareamientos": "desapareamiento",
    "desapareamos": "desaparear",
    "desaparean": "desaparear",
    "desapareando": "desaparear",
    "desapareara": "desaparear",
    "desaparearais": "desaparear",
    "desaparearan": "desaparear",
    "desaparearas": "desaparear",
    "desapareare": "desaparear",
    "desapareareis": "desaparear",
    "desaparearemos": "desaparear",
    "desaparearen": "desaparear",
    "desapareares": "desaparear",
    "desaparearon": "desaparear",
    "desapareará": "desaparear",
    "desaparearán": "desaparear",
    "desaparearás": "desaparear",
    "desaparearé": "desaparear",
    "desaparearéis": "desaparear",
    "desaparearía": "desaparear",
    "desaparearíais": "desaparear",
    "desaparearíamos": "desaparear",
    "desaparearían": "desaparear",
    "desaparearías": "desaparear",
    "desapareas": "desaparear",
    "desaparease": "desaparear",
    "desapareaseis": "desaparear",
    "desapareasen": "desaparear",
    "desapareases": "desaparear",
    "desapareaste": "desaparear",
    "desapareasteis": "desaparear",
    "desaparece": "desaparecer",
    "desapareced": "desaparecer",
    "desaparecemos": "desaparecer",
    "desaparecen": "desaparecer",
    "desapareceremos": "desaparecer",
    "desaparecerá": "desaparecer",
    "desaparecerán": "desaparecer",
    "desaparecerás": "desaparecer",
    "desapareceré": "desaparecer",
    "desapareceréis": "desaparecer",
    "desaparecería": "desaparecer",
    "desapareceríais": "desaparecer",
    "desapareceríamos": "desaparecer",
    "desaparecerían": "desaparecer",
    "desaparecerías": "desaparecer",
    "desapareces": "desaparecer",
    "desaparecida": "desaparecido",
    "desaparecidas": "desaparecido",
    "desaparecido": "desaparecer",
    "desaparecidos": "desaparecido",
    "desapareciendo": "desaparecer",
    "desapareciera": "desaparecer",
    "desaparecierais": "desaparecer",
    "desaparecieran": "desaparecer",
    "desaparecieras": "desaparecer",
    "desapareciere": "desaparecer",
    "desapareciereis": "desaparecer",
    "desaparecieren": "desaparecer",
    "desaparecieres": "desaparecer",
    "desaparecieron": "desaparecer",
    "desapareciese": "desaparecer",
    "desaparecieseis": "desaparecer",
    "desapareciesen": "desaparecer",
    "desaparecieses": "desaparecer",
    "desaparecimos": "desaparecer",
    "desapareciste": "desaparecer",
    "desaparecisteis": "desaparecer",
    "desapareciéramos": "desaparecer",
    "desapareciéremos": "desaparecer",
    "desapareciésemos": "desaparecer",
    "desapareció": "desaparecer",
    "desaparecéis": "desaparecer",
    "desaparecí": "desaparecer",
    "desaparecía": "desaparecer",
    "desaparecíais": "desaparecer",
    "desaparecíamos": "desaparecer",
    "desaparecían": "desaparecer",
    "desaparecías": "desaparecer",
    "desaparee": "desaparear",
    "desapareemos": "desaparear",
    "desapareen": "desaparear",
    "desaparees": "desaparear",
    "desapareja": "desaparejar",
    "desaparejaba": "desaparejar",
    "desaparejabais": "desaparejar",
    "desaparejaban": "desaparejar",
    "desaparejabas": "desaparejar",
    "desaparejad": "desaparejar",
    "desaparejada": "desaparejar",
    "desaparejadas": "desaparejar",
    "desaparejado": "desaparejar",
    "desaparejados": "desaparejar",
    "desaparejamientos": "desaparejamiento",
    "desaparejamos": "desaparejar",
    "desaparejan": "desaparejar",
    "desaparejando": "desaparejar",
    "desaparejara": "desaparejar",
    "desaparejarais": "desaparejar",
    "desaparejaran": "desaparejar",
    "desaparejaras": "desaparejar",
    "desaparejare": "desaparejar",
    "desaparejareis": "desaparejar",
    "desaparejaremos": "desaparejar",
    "desaparejaren": "desaparejar",
    "desaparejares": "desaparejar",
    "desaparejaron": "desaparejar",
    "desaparejará": "desaparejar",
    "desaparejarán": "desaparejar",
    "desaparejarás": "desaparejar",
    "desaparejaré": "desaparejar",
    "desaparejaréis": "desaparejar",
    "desaparejaría": "desaparejar",
    "desaparejaríais": "desaparejar",
    "desaparejaríamos": "desaparejar",
    "desaparejarían": "desaparejar",
    "desaparejarías": "desaparejar",
    "desaparejas": "desaparejar",
    "desaparejase": "desaparejar",
    "desaparejaseis": "desaparejar",
    "desaparejasen": "desaparejar",
    "desaparejases": "desaparejar",
    "desaparejaste": "desaparejar",
    "desaparejasteis": "desaparejar",
    "desapareje": "desaparejar",
    "desaparejemos": "desaparejar",
    "desaparejen": "desaparejar",
    "desaparejes": "desaparejar",
    "desaparejo": "desaparejar",
    "desaparejábamos": "desaparejar",
    "desaparejáis": "desaparejar",
    "desaparejáramos": "desaparejar",
    "desaparejáremos": "desaparejar",
    "desaparejásemos": "desaparejar",
    "desaparejé": "desaparejar",
    "desaparejéis": "desaparejar",
    "desaparejó": "desaparejar",
    "desapareo": "desaparear",
    "desaparezca": "desaparecer",
    "desaparezcamos": "desaparecer",
    "desaparezcan": "desaparecer",
    "desaparezcas": "desaparecer",
    "desaparezco": "desaparecer",
    "desaparezcáis": "desaparecer",
    "desapareábamos": "desaparear",
    "desapareáis": "desaparear",
    "desapareáramos": "desaparear",
    "desapareáremos": "desaparear",
    "desapareásemos": "desaparear",
    "desapareé": "desaparear",
    "desapareéis": "desaparear",
    "desapareó": "desaparear",
    "desapariciones": "desaparición",
    "desapasiona": "desapasionar",
    "desapasionaba": "desapasionar",
    "desapasionabais": "desapasionar",
    "desapasionaban": "desapasionar",
    "desapasionabas": "desapasionar",
    "desapasionad": "desapasionar",
    "desapasionada": "desapasionar",
    "desapasionadas": "desapasionar",
    "desapasionado": "desapasionar",
    "desapasionados": "desapasionar",
    "desapasionamientos": "desapasionamiento",
    "desapasionamos": "desapasionar",
    "desapasionan": "desapasionar",
    "desapasionando": "desapasionar",
    "desapasionara": "desapasionar",
    "desapasionarais": "desapasionar",
    "desapasionaran": "desapasionar",
    "desapasionaras": "desapasionar",
    "desapasionare": "desapasionar",
    "desapasionareis": "desapasionar",
    "desapasionaremos": "desapasionar",
    "desapasionaren": "desapasionar",
    "desapasionares": "desapasionar",
    "desapasionaron": "desapasionar",
    "desapasionará": "desapasionar",
    "desapasionarán": "desapasionar",
    "desapasionarás": "desapasionar",
    "desapasionaré": "desapasionar",
    "desapasionaréis": "desapasionar",
    "desapasionaría": "desapasionar",
    "desapasionaríais": "desapasionar",
    "desapasionaríamos": "desapasionar",
    "desapasionarían": "desapasionar",
    "desapasionarías": "desapasionar",
    "desapasionas": "desapasionar",
    "desapasionase": "desapasionar",
    "desapasionaseis": "desapasionar",
    "desapasionasen": "desapasionar",
    "desapasionases": "desapasionar",
    "desapasionaste": "desapasionar",
    "desapasionasteis": "desapasionar",
    "desapasione": "desapasionar",
    "desapasionemos": "desapasionar",
    "desapasionen": "desapasionar",
    "desapasiones": "desapasionar",
    "desapasiono": "desapasionar",
    "desapasionábamos": "desapasionar",
    "desapasionáis": "desapasionar",
    "desapasionáramos": "desapasionar",
    "desapasionáremos": "desapasionar",
    "desapasionásemos": "desapasionar",
    "desapasioné": "desapasionar",
    "desapasionéis": "desapasionar",
    "desapasionó": "desapasionar",
    "desapega": "desapegar",
    "desapegaba": "desapegar",
    "desapegabais": "desapegar",
    "desapegaban": "desapegar",
    "desapegabas": "desapegar",
    "desapegad": "desapegar",
    "desapegada": "desapegar",
    "desapegadas": "desapegar",
    "desapegado": "desapegar",
    "desapegados": "desapegar",
    "desapegamos": "desapegar",
    "desapegan": "desapegar",
    "desapegando": "desapegar",
    "desapegara": "desapegar",
    "desapegarais": "desapegar",
    "desapegaran": "desapegar",
    "desapegaras": "desapegar",
    "desapegare": "desapegar",
    "desapegareis": "desapegar",
    "desapegaremos": "desapegar",
    "desapegaren": "desapegar",
    "desapegares": "desapegar",
    "desapegaron": "desapegar",
    "desapegará": "desapegar",
    "desapegarán": "desapegar",
    "desapegarás": "desapegar",
    "desapegaré": "desapegar",
    "desapegaréis": "desapegar",
    "desapegaría": "desapegar",
    "desapegaríais": "desapegar",
    "desapegaríamos": "desapegar",
    "desapegarían": "desapegar",
    "desapegarías": "desapegar",
    "desapegas": "desapegar",
    "desapegase": "desapegar",
    "desapegaseis": "desapegar",
    "desapegasen": "desapegar",
    "desapegases": "desapegar",
    "desapegaste": "desapegar",
    "desapegasteis": "desapegar",
    "desapego": "desapegar",
    "desapegos": "desapego",
    "desapegue": "desapegar",
    "desapeguemos": "desapegar",
    "desapeguen": "desapegar",
    "desapegues": "desapegar",
    "desapegué": "desapegar",
    "desapeguéis": "desapegar",
    "desapegábamos": "desapegar",
    "desapegáis": "desapegar",
    "desapegáramos": "desapegar",
    "desapegáremos": "desapegar",
    "desapegásemos": "desapegar",
    "desapegó": "desapegar",
    "desapercibida": "desapercibido",
    "desapercibidas": "desapercibido",
    "desapercibidos": "desapercibido",
    "desapercibimientos": "desapercibimiento",
    "desapesta": "desapestar",
    "desapestaba": "desapestar",
    "desapestabais": "desapestar",
    "desapestaban": "desapestar",
    "desapestabas": "desapestar",
    "desapestad": "desapestar",
    "desapestada": "desapestar",
    "desapestadas": "desapestar",
    "desapestado": "desapestar",
    "desapestados": "desapestar",
    "desapestamos": "desapestar",
    "desapestan": "desapestar",
    "desapestando": "desapestar",
    "desapestara": "desapestar",
    "desapestarais": "desapestar",
    "desapestaran": "desapestar",
    "desapestaras": "desapestar",
    "desapestare": "desapestar",
    "desapestareis": "desapestar",
    "desapestaremos": "desapestar",
    "desapestaren": "desapestar",
    "desapestares": "desapestar",
    "desapestaron": "desapestar",
    "desapestará": "desapestar",
    "desapestarán": "desapestar",
    "desapestarás": "desapestar",
    "desapestaré": "desapestar",
    "desapestaréis": "desapestar",
    "desapestaría": "desapestar",
    "desapestaríais": "desapestar",
    "desapestaríamos": "desapestar",
    "desapestarían": "desapestar",
    "desapestarías": "desapestar",
    "desapestas": "desapestar",
    "desapestase": "desapestar",
    "desapestaseis": "desapestar",
    "desapestasen": "desapestar",
    "desapestases": "desapestar",
    "desapestaste": "desapestar",
    "desapestasteis": "desapestar",
    "desapeste": "desapestar",
    "desapestemos": "desapestar",
    "desapesten": "desapestar",
    "desapestes": "desapestar",
    "desapesto": "desapestar",
    "desapestábamos": "desapestar",
    "desapestáis": "desapestar",
    "desapestáramos": "desapestar",
    "desapestáremos": "desapestar",
    "desapestásemos": "desapestar",
    "desapesté": "desapestar",
    "desapestéis": "desapestar",
    "desapestó": "desapestar",
    "desapiadada": "desapiadado",
    "desapiadadas": "desapiadado",
    "desapiadados": "desapiadado",
    "desaplicaciones": "desaplicación",
    "desaplicada": "desaplicado",
    "desaplicadas": "desaplicado",
    "desaplicados": "desaplicado",
    "desaploma": "desaplomar",
    "desaplomaba": "desaplomar",
    "desaplomabais": "desaplomar",
    "desaplomaban": "desaplomar",
    "desaplomabas": "desaplomar",
    "desaplomad": "desaplomar",
    "desaplomada": "desaplomar",
    "desaplomadas": "desaplomar",
    "desaplomado": "desaplomar",
    "desaplomados": "desaplomar",
    "desaplomamos": "desaplomar",
    "desaploman": "desaplomar",
    "desaplomando": "desaplomar",
    "desaplomara": "desaplomar",
    "desaplomarais": "desaplomar",
    "desaplomaran": "desaplomar",
    "desaplomaras": "desaplomar",
    "desaplomare": "desaplomar",
    "desaplomareis": "desaplomar",
    "desaplomaremos": "desaplomar",
    "desaplomaren": "desaplomar",
    "desaplomares": "desaplomar",
    "desaplomaron": "desaplomar",
    "desaplomará": "desaplomar",
    "desaplomarán": "desaplomar",
    "desaplomarás": "desaplomar",
    "desaplomaré": "desaplomar",
    "desaplomaréis": "desaplomar",
    "desaplomaría": "desaplomar",
    "desaplomaríais": "desaplomar",
    "desaplomaríamos": "desaplomar",
    "desaplomarían": "desaplomar",
    "desaplomarías": "desaplomar",
    "desaplomas": "desaplomar",
    "desaplomase": "desaplomar",
    "desaplomaseis": "desaplomar",
    "desaplomasen": "desaplomar",
    "desaplomases": "desaplomar",
    "desaplomaste": "desaplomar",
    "desaplomasteis": "desaplomar",
    "desaplome": "desaplomar",
    "desaplomemos": "desaplomar",
    "desaplomen": "desaplomar",
    "desaplomes": "desaplomar",
    "desaplomo": "desaplomar",
    "desaplomábamos": "desaplomar",
    "desaplomáis": "desaplomar",
    "desaplomáramos": "desaplomar",
    "desaplomáremos": "desaplomar",
    "desaplomásemos": "desaplomar",
    "desaplomé": "desaplomar",
    "desaploméis": "desaplomar",
    "desaplomó": "desaplomar",
    "desapodera": "desapoderar",
    "desapoderaba": "desapoderar",
    "desapoderabais": "desapoderar",
    "desapoderaban": "desapoderar",
    "desapoderabas": "desapoderar",
    "desapoderad": "desapoderar",
    "desapoderada": "desapoderar",
    "desapoderadas": "desapoderar",
    "desapoderado": "desapoderar",
    "desapoderados": "desapoderar",
    "desapoderamos": "desapoderar",
    "desapoderan": "desapoderar",
    "desapoderando": "desapoderar",
    "desapoderara": "desapoderar",
    "desapoderarais": "desapoderar",
    "desapoderaran": "desapoderar",
    "desapoderaras": "desapoderar",
    "desapoderare": "desapoderar",
    "desapoderareis": "desapoderar",
    "desapoderaremos": "desapoderar",
    "desapoderaren": "desapoderar",
    "desapoderares": "desapoderar",
    "desapoderaron": "desapoderar",
    "desapoderará": "desapoderar",
    "desapoderarán": "desapoderar",
    "desapoderarás": "desapoderar",
    "desapoderaré": "desapoderar",
    "desapoderaréis": "desapoderar",
    "desapoderaría": "desapoderar",
    "desapoderaríais": "desapoderar",
    "desapoderaríamos": "desapoderar",
    "desapoderarían": "desapoderar",
    "desapoderarías": "desapoderar",
    "desapoderas": "desapoderar",
    "desapoderase": "desapoderar",
    "desapoderaseis": "desapoderar",
    "desapoderasen": "desapoderar",
    "desapoderases": "desapoderar",
    "desapoderaste": "desapoderar",
    "desapoderasteis": "desapoderar",
    "desapodere": "desapoderar",
    "desapoderemos": "desapoderar",
    "desapoderen": "desapoderar",
    "desapoderes": "desapoderar",
    "desapodero": "desapoderar",
    "desapoderábamos": "desapoderar",
    "desapoderáis": "desapoderar",
    "desapoderáramos": "desapoderar",
    "desapoderáremos": "desapoderar",
    "desapoderásemos": "desapoderar",
    "desapoderé": "desapoderar",
    "desapoderéis": "desapoderar",
    "desapoderó": "desapoderar",
    "desapolilla": "desapolillar",
    "desapolillaba": "desapolillar",
    "desapolillabais": "desapolillar",
    "desapolillaban": "desapolillar",
    "desapolillabas": "desapolillar",
    "desapolillad": "desapolillar",
    "desapolillada": "desapolillar",
    "desapolilladas": "desapolillar",
    "desapolillado": "desapolillar",
    "desapolillados": "desapolillar",
    "desapolillamos": "desapolillar",
    "desapolillan": "desapolillar",
    "desapolillando": "desapolillar",
    "desapolillara": "desapolillar",
    "desapolillarais": "desapolillar",
    "desapolillaran": "desapolillar",
    "desapolillaras": "desapolillar",
    "desapolillare": "desapolillar",
    "desapolillareis": "desapolillar",
    "desapolillaremos": "desapolillar",
    "desapolillaren": "desapolillar",
    "desapolillares": "desapolillar",
    "desapolillaron": "desapolillar",
    "desapolillará": "desapolillar",
    "desapolillarán": "desapolillar",
    "desapolillarás": "desapolillar",
    "desapolillaré": "desapolillar",
    "desapolillaréis": "desapolillar",
    "desapolillaría": "desapolillar",
    "desapolillaríais": "desapolillar",
    "desapolillaríamos": "desapolillar",
    "desapolillarían": "desapolillar",
    "desapolillarías": "desapolillar",
    "desapolillas": "desapolillar",
    "desapolillase": "desapolillar",
    "desapolillaseis": "desapolillar",
    "desapolillasen": "desapolillar",
    "desapolillases": "desapolillar",
    "desapolillaste": "desapolillar",
    "desapolillasteis": "desapolillar",
    "desapolille": "desapolillar",
    "desapolillemos": "desapolillar",
    "desapolillen": "desapolillar",
    "desapolilles": "desapolillar",
    "desapolillo": "desapolillar",
    "desapolillábamos": "desapolillar",
    "desapolilláis": "desapolillar",
    "desapolilláramos": "desapolillar",
    "desapolilláremos": "desapolillar",
    "desapolillásemos": "desapolillar",
    "desapolillé": "desapolillar",
    "desapolilléis": "desapolillar",
    "desapolilló": "desapolillar",
    "desaporcaba": "desaporcar",
    "desaporcabais": "desaporcar",
    "desaporcaban": "desaporcar",
    "desaporcabas": "desaporcar",
    "desaporcad": "desaporcar",
    "desaporcada": "desaporcar",
    "desaporcadas": "desaporcar",
    "desaporcado": "desaporcar",
    "desaporcados": "desaporcar",
    "desaporcamos": "desaporcar",
    "desaporcando": "desaporcar",
    "desaporcara": "desaporcar",
    "desaporcarais": "desaporcar",
    "desaporcaran": "desaporcar",
    "desaporcaras": "desaporcar",
    "desaporcare": "desaporcar",
    "desaporcareis": "desaporcar",
    "desaporcaremos": "desaporcar",
    "desaporcaren": "desaporcar",
    "desaporcares": "desaporcar",
    "desaporcaron": "desaporcar",
    "desaporcará": "desaporcar",
    "desaporcarán": "desaporcar",
    "desaporcarás": "desaporcar",
    "desaporcaré": "desaporcar",
    "desaporcaréis": "desaporcar",
    "desaporcaría": "desaporcar",
    "desaporcaríais": "desaporcar",
    "desaporcaríamos": "desaporcar",
    "desaporcarían": "desaporcar",
    "desaporcarías": "desaporcar",
    "desaporcase": "desaporcar",
    "desaporcaseis": "desaporcar",
    "desaporcasen": "desaporcar",
    "desaporcases": "desaporcar",
    "desaporcaste": "desaporcar",
    "desaporcasteis": "desaporcar",
    "desaporcábamos": "desaporcar",
    "desaporcáis": "desaporcar",
    "desaporcáramos": "desaporcar",
    "desaporcáremos": "desaporcar",
    "desaporcásemos": "desaporcar",
    "desaporcó": "desaporcar",
    "desaporquemos": "desaporcar",
    "desaporqué": "desaporcar",
    "desaporquéis": "desaporcar",
    "desaposenta": "desaposentar",
    "desaposentaba": "desaposentar",
    "desaposentabais": "desaposentar",
    "desaposentaban": "desaposentar",
    "desaposentabas": "desaposentar",
    "desaposentad": "desaposentar",
    "desaposentada": "desaposentar",
    "desaposentadas": "desaposentar",
    "desaposentado": "desaposentar",
    "desaposentados": "desaposentar",
    "desaposentamos": "desaposentar",
    "desaposentan": "desaposentar",
    "desaposentando": "desaposentar",
    "desaposentara": "desaposentar",
    "desaposentarais": "desaposentar",
    "desaposentaran": "desaposentar",
    "desaposentaras": "desaposentar",
    "desaposentare": "desaposentar",
    "desaposentareis": "desaposentar",
    "desaposentaremos": "desaposentar",
    "desaposentaren": "desaposentar",
    "desaposentares": "desaposentar",
    "desaposentaron": "desaposentar",
    "desaposentará": "desaposentar",
    "desaposentarán": "desaposentar",
    "desaposentarás": "desaposentar",
    "desaposentaré": "desaposentar",
    "desaposentaréis": "desaposentar",
    "desaposentaría": "desaposentar",
    "desaposentaríais": "desaposentar",
    "desaposentaríamos": "desaposentar",
    "desaposentarían": "desaposentar",
    "desaposentarías": "desaposentar",
    "desaposentas": "desaposentar",
    "desaposentase": "desaposentar",
    "desaposentaseis": "desaposentar",
    "desaposentasen": "desaposentar",
    "desaposentases": "desaposentar",
    "desaposentaste": "desaposentar",
    "desaposentasteis": "desaposentar",
    "desaposente": "desaposentar",
    "desaposentemos": "desaposentar",
    "desaposenten": "desaposentar",
    "desaposentes": "desaposentar",
    "desaposento": "desaposentar",
    "desaposentábamos": "desaposentar",
    "desaposentáis": "desaposentar",
    "desaposentáramos": "desaposentar",
    "desaposentáremos": "desaposentar",
    "desaposentásemos": "desaposentar",
    "desaposenté": "desaposentar",
    "desaposentéis": "desaposentar",
    "desaposentó": "desaposentar",
    "desapoya": "desapoyar",
    "desapoyaba": "desapoyar",
    "desapoyabais": "desapoyar",
    "desapoyaban": "desapoyar",
    "desapoyabas": "desapoyar",
    "desapoyad": "desapoyar",
    "desapoyada": "desapoyar",
    "desapoyadas": "desapoyar",
    "desapoyado": "desapoyar",
    "desapoyados": "desapoyar",
    "desapoyamos": "desapoyar",
    "desapoyan": "desapoyar",
    "desapoyando": "desapoyar",
    "desapoyara": "desapoyar",
    "desapoyarais": "desapoyar",
    "desapoyaran": "desapoyar",
    "desapoyaras": "desapoyar",
    "desapoyare": "desapoyar",
    "desapoyareis": "desapoyar",
    "desapoyaremos": "desapoyar",
    "desapoyaren": "desapoyar",
    "desapoyares": "desapoyar",
    "desapoyaron": "desapoyar",
    "desapoyará": "desapoyar",
    "desapoyarán": "desapoyar",
    "desapoyarás": "desapoyar",
    "desapoyaré": "desapoyar",
    "desapoyaréis": "desapoyar",
    "desapoyaría": "desapoyar",
    "desapoyaríais": "desapoyar",
    "desapoyaríamos": "desapoyar",
    "desapoyarían": "desapoyar",
    "desapoyarías": "desapoyar",
    "desapoyas": "desapoyar",
    "desapoyase": "desapoyar",
    "desapoyaseis": "desapoyar",
    "desapoyasen": "desapoyar",
    "desapoyases": "desapoyar",
    "desapoyaste": "desapoyar",
    "desapoyasteis": "desapoyar",
    "desapoye": "desapoyar",
    "desapoyemos": "desapoyar",
    "desapoyen": "desapoyar",
    "desapoyes": "desapoyar",
    "desapoyo": "desapoyar",
    "desapoyábamos": "desapoyar",
    "desapoyáis": "desapoyar",
    "desapoyáramos": "desapoyar",
    "desapoyáremos": "desapoyar",
    "desapoyásemos": "desapoyar",
    "desapoyé": "desapoyar",
    "desapoyéis": "desapoyar",
    "desapoyó": "desapoyar",
    "desaprecia": "desapreciar",
    "desapreciaba": "desapreciar",
    "desapreciabais": "desapreciar",
    "desapreciaban": "desapreciar",
    "desapreciabas": "desapreciar",
    "desapreciad": "desapreciar",
    "desapreciada": "desapreciar",
    "desapreciadas": "desapreciar",
    "desapreciado": "desapreciar",
    "desapreciados": "desapreciar",
    "desapreciamos": "desapreciar",
    "desaprecian": "desapreciar",
    "desapreciando": "desapreciar",
    "desapreciara": "desapreciar",
    "desapreciarais": "desapreciar",
    "desapreciaran": "desapreciar",
    "desapreciaras": "desapreciar",
    "desapreciare": "desapreciar",
    "desapreciareis": "desapreciar",
    "desapreciaremos": "desapreciar",
    "desapreciaren": "desapreciar",
    "desapreciares": "desapreciar",
    "desapreciaron": "desapreciar",
    "desapreciará": "desapreciar",
    "desapreciarán": "desapreciar",
    "desapreciarás": "desapreciar",
    "desapreciaré": "desapreciar",
    "desapreciaréis": "desapreciar",
    "desapreciaría": "desapreciar",
    "desapreciaríais": "desapreciar",
    "desapreciaríamos": "desapreciar",
    "desapreciarían": "desapreciar",
    "desapreciarías": "desapreciar",
    "desaprecias": "desapreciar",
    "desapreciase": "desapreciar",
    "desapreciaseis": "desapreciar",
    "desapreciasen": "desapreciar",
    "desapreciases": "desapreciar",
    "desapreciaste": "desapreciar",
    "desapreciasteis": "desapreciar",
    "desaprecie": "desapreciar",
    "desapreciemos": "desapreciar",
    "desaprecien": "desapreciar",
    "desaprecies": "desapreciar",
    "desaprecio": "desapreciar",
    "desapreciábamos": "desapreciar",
    "desapreciáis": "desapreciar",
    "desapreciáramos": "desapreciar",
    "desapreciáremos": "desapreciar",
    "desapreciásemos": "desapreciar",
    "desaprecié": "desapreciar",
    "desapreciéis": "desapreciar",
    "desapreció": "desapreciar",
    "desaprenda": "desaprender",
    "desaprendamos": "desaprender",
    "desaprendan": "desaprender",
    "desaprendas": "desaprender",
    "desaprende": "desaprender",
    "desaprended": "desaprender",
    "desaprendemos": "desaprender",
    "desaprenden": "desaprender",
    "desaprenderemos": "desaprender",
    "desaprenderá": "desaprender",
    "desaprenderán": "desaprender",
    "desaprenderás": "desaprender",
    "desaprenderé": "desaprender",
    "desaprenderéis": "desaprender",
    "desaprendería": "desaprender",
    "desaprenderíais": "desaprender",
    "desaprenderíamos": "desaprender",
    "desaprenderían": "desaprender",
    "desaprenderías": "desaprender",
    "desaprendes": "desaprender",
    "desaprendida": "desaprender",
    "desaprendidas": "desaprender",
    "desaprendido": "desaprender",
    "desaprendidos": "desaprender",
    "desaprendiendo": "desaprender",
    "desaprendiera": "desaprender",
    "desaprendierais": "desaprender",
    "desaprendieran": "desaprender",
    "desaprendieras": "desaprender",
    "desaprendiere": "desaprender",
    "desaprendiereis": "desaprender",
    "desaprendieren": "desaprender",
    "desaprendieres": "desaprender",
    "desaprendieron": "desaprender",
    "desaprendiese": "desaprender",
    "desaprendieseis": "desaprender",
    "desaprendiesen": "desaprender",
    "desaprendieses": "desaprender",
    "desaprendimos": "desaprender",
    "desaprendiste": "desaprender",
    "desaprendisteis": "desaprender",
    "desaprendiéramos": "desaprender",
    "desaprendiéremos": "desaprender",
    "desaprendiésemos": "desaprender",
    "desaprendió": "desaprender",
    "desaprendo": "desaprender",
    "desaprendáis": "desaprender",
    "desaprendéis": "desaprender",
    "desaprendí": "desaprender",
    "desaprendía": "desaprender",
    "desaprendíais": "desaprender",
    "desaprendíamos": "desaprender",
    "desaprendían": "desaprender",
    "desaprendías": "desaprender",
    "desaprensa": "desaprensar",
    "desaprensaba": "desaprensar",
    "desaprensabais": "desaprensar",
    "desaprensaban": "desaprensar",
    "desaprensabas": "desaprensar",
    "desaprensad": "desaprensar",
    "desaprensada": "desaprensar",
    "desaprensadas": "desaprensar",
    "desaprensado": "desaprensar",
    "desaprensados": "desaprensar",
    "desaprensamos": "desaprensar",
    "desaprensan": "desaprensar",
    "desaprensando": "desaprensar",
    "desaprensara": "desaprensar",
    "desaprensarais": "desaprensar",
    "desaprensaran": "desaprensar",
    "desaprensaras": "desaprensar",
    "desaprensare": "desaprensar",
    "desaprensareis": "desaprensar",
    "desaprensaremos": "desaprensar",
    "desaprensaren": "desaprensar",
    "desaprensares": "desaprensar",
    "desaprensaron": "desaprensar",
    "desaprensará": "desaprensar",
    "desaprensarán": "desaprensar",
    "desaprensarás": "desaprensar",
    "desaprensaré": "desaprensar",
    "desaprensaréis": "desaprensar",
    "desaprensaría": "desaprensar",
    "desaprensaríais": "desaprensar",
    "desaprensaríamos": "desaprensar",
    "desaprensarían": "desaprensar",
    "desaprensarías": "desaprensar",
    "desaprensas": "desaprensar",
    "desaprensase": "desaprensar",
    "desaprensaseis": "desaprensar",
    "desaprensasen": "desaprensar",
    "desaprensases": "desaprensar",
    "desaprensaste": "desaprensar",
    "desaprensasteis": "desaprensar",
    "desaprense": "desaprensar",
    "desaprensemos": "desaprensar",
    "desaprensen": "desaprensar",
    "desaprenses": "desaprensar",
    "desaprensiones": "desaprensión",
    "desaprensiva": "desaprensivo",
    "desaprensivas": "desaprensivo",
    "desaprensivos": "desaprensivo",
    "desaprenso": "desaprensar",
    "desaprensábamos": "desaprensar",
    "desaprensáis": "desaprensar",
    "desaprensáramos": "desaprensar",
    "desaprensáremos": "desaprensar",
    "desaprensásemos": "desaprensar",
    "desaprensé": "desaprensar",
    "desaprenséis": "desaprensar",
    "desaprensó": "desaprensar",
    "desapretaba": "desapretar",
    "desapretabais": "desapretar",
    "desapretaban": "desapretar",
    "desapretabas": "desapretar",
    "desapretad": "desapretar",
    "desapretada": "desapretar",
    "desapretadas": "desapretar",
    "desapretado": "desapretar",
    "desapretados": "desapretar",
    "desapretamos": "desapretar",
    "desapretando": "desapretar",
    "desapretara": "desapretar",
    "desapretarais": "desapretar",
    "desapretaran": "desapretar",
    "desapretaras": "desapretar",
    "desapretare": "desapretar",
    "desapretareis": "desapretar",
    "desapretaremos": "desapretar",
    "desapretaren": "desapretar",
    "desapretares": "desapretar",
    "desapretaron": "desapretar",
    "desapretará": "desapretar",
    "desapretarán": "desapretar",
    "desapretarás": "desapretar",
    "desapretaré": "desapretar",
    "desapretaréis": "desapretar",
    "desapretaría": "desapretar",
    "desapretaríais": "desapretar",
    "desapretaríamos": "desapretar",
    "desapretarían": "desapretar",
    "desapretarías": "desapretar",
    "desapretase": "desapretar",
    "desapretaseis": "desapretar",
    "desapretasen": "desapretar",
    "desapretases": "desapretar",
    "desapretaste": "desapretar",
    "desapretasteis": "desapretar",
    "desapretemos": "desapretar",
    "desapretábamos": "desapretar",
    "desapretáis": "desapretar",
    "desapretáramos": "desapretar",
    "desapretáremos": "desapretar",
    "desapretásemos": "desapretar",
    "desapreté": "desapretar",
    "desapretéis": "desapretar",
    "desapretó": "desapretar",
    "desaprieta": "desapretar",
    "desaprietan": "desapretar",
    "desaprietas": "desapretar",
    "desapriete": "desapretar",
    "desaprieten": "desapretar",
    "desaprietes": "desapretar",
    "desaprieto": "desapretar",
    "desaprobaba": "desaprobar",
    "desaprobabais": "desaprobar",
    "desaprobaban": "desaprobar",
    "desaprobabas": "desaprobar",
    "desaprobaciones": "desaprobación",
    "desaprobad": "desaprobar",
    "desaprobada": "desaprobar",
    "desaprobadas": "desaprobar",
    "desaprobado": "desaprobar",
    "desaprobadora": "desaprobador",
    "desaprobadoras": "desaprobador",
    "desaprobadores": "desaprobador",
    "desaprobados": "desaprobar",
    "desaprobamos": "desaprobar",
    "desaprobando": "desaprobar",
    "desaprobara": "desaprobar",
    "desaprobarais": "desaprobar",
    "desaprobaran": "desaprobar",
    "desaprobaras": "desaprobar",
    "desaprobare": "desaprobar",
    "desaprobareis": "desaprobar",
    "desaprobaremos": "desaprobar",
    "desaprobaren": "desaprobar",
    "desaprobares": "desaprobar",
    "desaprobaron": "desaprobar",
    "desaprobará": "desaprobar",
    "desaprobarán": "desaprobar",
    "desaprobarás": "desaprobar",
    "desaprobaré": "desaprobar",
    "desaprobaréis": "desaprobar",
    "desaprobaría": "desaprobar",
    "desaprobaríais": "desaprobar",
    "desaprobaríamos": "desaprobar",
    "desaprobarían": "desaprobar",
    "desaprobarías": "desaprobar",
    "desaprobase": "desaprobar",
    "desaprobaseis": "desaprobar",
    "desaprobasen": "desaprobar",
    "desaprobases": "desaprobar",
    "desaprobaste": "desaprobar",
    "desaprobasteis": "desaprobar",
    "desaprobemos": "desaprobar",
    "desaprobábamos": "desaprobar",
    "desaprobáis": "desaprobar",
    "desaprobáramos": "desaprobar",
    "desaprobáremos": "desaprobar",
    "desaprobásemos": "desaprobar",
    "desaprobé": "desaprobar",
    "desaprobéis": "desaprobar",
    "desaprobó": "desaprobar",
    "desapropia": "desapropiar",
    "desapropiaba": "desapropiar",
    "desapropiabais": "desapropiar",
    "desapropiaban": "desapropiar",
    "desapropiabas": "desapropiar",
    "desapropiaciones": "desapropiación",
    "desapropiad": "desapropiar",
    "desapropiada": "desapropiar",
    "desapropiadas": "desapropiar",
    "desapropiado": "desapropiar",
    "desapropiados": "desapropiar",
    "desapropiamientos": "desapropiamiento",
    "desapropiamos": "desapropiar",
    "desapropian": "desapropiar",
    "desapropiando": "desapropiar",
    "desapropiara": "desapropiar",
    "desapropiarais": "desapropiar",
    "desapropiaran": "desapropiar",
    "desapropiaras": "desapropiar",
    "desapropiare": "desapropiar",
    "desapropiareis": "desapropiar",
    "desapropiaremos": "desapropiar",
    "desapropiaren": "desapropiar",
    "desapropiares": "desapropiar",
    "desapropiaron": "desapropiar",
    "desapropiará": "desapropiar",
    "desapropiarán": "desapropiar",
    "desapropiarás": "desapropiar",
    "desapropiaré": "desapropiar",
    "desapropiaréis": "desapropiar",
    "desapropiaría": "desapropiar",
    "desapropiaríais": "desapropiar",
    "desapropiaríamos": "desapropiar",
    "desapropiarían": "desapropiar",
    "desapropiarías": "desapropiar",
    "desapropias": "desapropiar",
    "desapropiase": "desapropiar",
    "desapropiaseis": "desapropiar",
    "desapropiasen": "desapropiar",
    "desapropiases": "desapropiar",
    "desapropiaste": "desapropiar",
    "desapropiasteis": "desapropiar",
    "desapropie": "desapropiar",
    "desapropiemos": "desapropiar",
    "desapropien": "desapropiar",
    "desapropies": "desapropiar",
    "desapropio": "desapropiar",
    "desapropios": "desapropio",
    "desapropiábamos": "desapropiar",
    "desapropiáis": "desapropiar",
    "desapropiáramos": "desapropiar",
    "desapropiáremos": "desapropiar",
    "desapropiásemos": "desapropiar",
    "desapropié": "desapropiar",
    "desapropiéis": "desapropiar",
    "desapropió": "desapropiar",
    "desaprovecha": "desaprovechar",
    "desaprovechaba": "desaprovechar",
    "desaprovechabais": "desaprovechar",
    "desaprovechaban": "desaprovechar",
    "desaprovechabas": "desaprovechar",
    "desaprovechad": "desaprovechar",
    "desaprovechada": "desaprovechar",
    "desaprovechadas": "desaprovechar",
    "desaprovechado": "desaprovechar",
    "desaprovechados": "desaprovechar",
    "desaprovechamientos": "desaprovechamiento",
    "desaprovechamos": "desaprovechar",
    "desaprovechan": "desaprovechar",
    "desaprovechando": "desaprovechar",
    "desaprovechara": "desaprovechar",
    "desaprovecharais": "desaprovechar",
    "desaprovecharan": "desaprovechar",
    "desaprovecharas": "desaprovechar",
    "desaprovechare": "desaprovechar",
    "desaprovechareis": "desaprovechar",
    "desaprovecharemos": "desaprovechar",
    "desaprovecharen": "desaprovechar",
    "desaprovechares": "desaprovechar",
    "desaprovecharon": "desaprovechar",
    "desaprovechará": "desaprovechar",
    "desaprovecharán": "desaprovechar",
    "desaprovecharás": "desaprovechar",
    "desaprovecharé": "desaprovechar",
    "desaprovecharéis": "desaprovechar",
    "desaprovecharía": "desaprovechar",
    "desaprovecharíais": "desaprovechar",
    "desaprovecharíamos": "desaprovechar",
    "desaprovecharían": "desaprovechar",
    "desaprovecharías": "desaprovechar",
    "desaprovechas": "desaprovechar",
    "desaprovechase": "desaprovechar",
    "desaprovechaseis": "desaprovechar",
    "desaprovechasen": "desaprovechar",
    "desaprovechases": "desaprovechar",
    "desaprovechaste": "desaprovechar",
    "desaprovechasteis": "desaprovechar",
    "desaproveche": "desaprovechar",
    "desaprovechemos": "desaprovechar",
    "desaprovechen": "desaprovechar",
    "desaproveches": "desaprovechar",
    "desaprovecho": "desaprovechar",
    "desaprovechábamos": "desaprovechar",
    "desaprovecháis": "desaprovechar",
    "desaprovecháramos": "desaprovechar",
    "desaprovecháremos": "desaprovechar",
    "desaprovechásemos": "desaprovechar",
    "desaproveché": "desaprovechar",
    "desaprovechéis": "desaprovechar",
    "desaprovechó": "desaprovechar",
    "desaprueba": "desaprobar",
    "desaprueban": "desaprobar",
    "desapruebas": "desaprobar",
    "desapruebe": "desaprobar",
    "desaprueben": "desaprobar",
    "desapruebes": "desaprobar",
    "desapruebo": "desaprobar",
    "desapuerca": "desaporcar",
    "desapuercan": "desaporcar",
    "desapuercas": "desaporcar",
    "desapuerco": "desaporcar",
    "desapuerque": "desaporcar",
    "desapuerquen": "desaporcar",
    "desapuerques": "desaporcar",
    "desapunta": "desapuntar",
    "desapuntaba": "desapuntar",
    "desapuntabais": "desapuntar",
    "desapuntaban": "desapuntar",
    "desapuntabas": "desapuntar",
    "desapuntad": "desapuntar",
    "desapuntada": "desapuntar",
    "desapuntadas": "desapuntar",
    "desapuntado": "desapuntar",
    "desapuntados": "desapuntar",
    "desapuntala": "desapuntalar",
    "desapuntalaba": "desapuntalar",
    "desapuntalabais": "desapuntalar",
    "desapuntalaban": "desapuntalar",
    "desapuntalabas": "desapuntalar",
    "desapuntalad": "desapuntalar",
    "desapuntalada": "desapuntalar",
    "desapuntaladas": "desapuntalar",
    "desapuntalado": "desapuntalar",
    "desapuntalados": "desapuntalar",
    "desapuntalamientos": "desapuntalamiento",
    "desapuntalamos": "desapuntalar",
    "desapuntalan": "desapuntalar",
    "desapuntalando": "desapuntalar",
    "desapuntalara": "desapuntalar",
    "desapuntalarais": "desapuntalar",
    "desapuntalaran": "desapuntalar",
    "desapuntalaras": "desapuntalar",
    "desapuntalare": "desapuntalar",
    "desapuntalareis": "desapuntalar",
    "desapuntalaremos": "desapuntalar",
    "desapuntalaren": "desapuntalar",
    "desapuntalares": "desapuntalar",
    "desapuntalaron": "desapuntalar",
    "desapuntalará": "desapuntalar",
    "desapuntalarán": "desapuntalar",
    "desapuntalarás": "desapuntalar",
    "desapuntalaré": "desapuntalar",
    "desapuntalaréis": "desapuntalar",
    "desapuntalaría": "desapuntalar",
    "desapuntalaríais": "desapuntalar",
    "desapuntalaríamos": "desapuntalar",
    "desapuntalarían": "desapuntalar",
    "desapuntalarías": "desapuntalar",
    "desapuntalas": "desapuntalar",
    "desapuntalase": "desapuntalar",
    "desapuntalaseis": "desapuntalar",
    "desapuntalasen": "desapuntalar",
    "desapuntalases": "desapuntalar",
    "desapuntalaste": "desapuntalar",
    "desapuntalasteis": "desapuntalar",
    "desapuntale": "desapuntalar",
    "desapuntalemos": "desapuntalar",
    "desapuntalen": "desapuntalar",
    "desapuntales": "desapuntalar",
    "desapuntalo": "desapuntalar",
    "desapuntalábamos": "desapuntalar",
    "desapuntaláis": "desapuntalar",
    "desapuntaláramos": "desapuntalar",
    "desapuntaláremos": "desapuntalar",
    "desapuntalásemos": "desapuntalar",
    "desapuntalé": "desapuntalar",
    "desapuntaléis": "desapuntalar",
    "desapuntaló": "desapuntalar",
    "desapuntamos": "desapuntar",
    "desapuntan": "desapuntar",
    "desapuntando": "desapuntar",
    "desapuntara": "desapuntar",
    "desapuntarais": "desapuntar",
    "desapuntaran": "desapuntar",
    "desapuntaras": "desapuntar",
    "desapuntare": "desapuntar",
    "desapuntareis": "desapuntar",
    "desapuntaremos": "desapuntar",
    "desapuntaren": "desapuntar",
    "desapuntares": "desapuntar",
    "desapuntaron": "desapuntar",
    "desapuntará": "desapuntar",
    "desapuntarán": "desapuntar",
    "desapuntarás": "desapuntar",
    "desapuntaré": "desapuntar",
    "desapuntaréis": "desapuntar",
    "desapuntaría": "desapuntar",
    "desapuntaríais": "desapuntar",
    "desapuntaríamos": "desapuntar",
    "desapuntarían": "desapuntar",
    "desapuntarías": "desapuntar",
    "desapuntas": "desapuntar",
    "desapuntase": "desapuntar",
    "desapuntaseis": "desapuntar",
    "desapuntasen": "desapuntar",
    "desapuntases": "desapuntar",
    "desapuntaste": "desapuntar",
    "desapuntasteis": "desapuntar",
    "desapunte": "desapuntar",
    "desapuntemos": "desapuntar",
    "desapunten": "desapuntar",
    "desapuntes": "desapuntar",
    "desapunto": "desapuntar",
    "desapuntábamos": "desapuntar",
    "desapuntáis": "desapuntar",
    "desapuntáramos": "desapuntar",
    "desapuntáremos": "desapuntar",
    "desapuntásemos": "desapuntar",
    "desapunté": "desapuntar",
    "desapuntéis": "desapuntar",
    "desapuntó": "desapuntar",
    "desarbola": "desarbolar",
    "desarbolaba": "desarbolar",
    "desarbolabais": "desarbolar",
    "desarbolaban": "desarbolar",
    "desarbolabas": "desarbolar",
    "desarbolad": "desarbolar",
    "desarbolada": "desarbolar",
    "desarboladas": "desarbolar",
    "desarbolado": "desarbolar",
    "desarbolados": "desarbolar",
    "desarbolamientos": "desarbolamiento",
    "desarbolamos": "desarbolar",
    "desarbolan": "desarbolar",
    "desarbolando": "desarbolar",
    "desarbolara": "desarbolar",
    "desarbolarais": "desarbolar",
    "desarbolaran": "desarbolar",
    "desarbolaras": "desarbolar",
    "desarbolare": "desarbolar",
    "desarbolareis": "desarbolar",
    "desarbolaremos": "desarbolar",
    "desarbolaren": "desarbolar",
    "desarbolares": "desarbolar",
    "desarbolaron": "desarbolar",
    "desarbolará": "desarbolar",
    "desarbolarán": "desarbolar",
    "desarbolarás": "desarbolar",
    "desarbolaré": "desarbolar",
    "desarbolaréis": "desarbolar",
    "desarbolaría": "desarbolar",
    "desarbolaríais": "desarbolar",
    "desarbolaríamos": "desarbolar",
    "desarbolarían": "desarbolar",
    "desarbolarías": "desarbolar",
    "desarbolas": "desarbolar",
    "desarbolase": "desarbolar",
    "desarbolaseis": "desarbolar",
    "desarbolasen": "desarbolar",
    "desarbolases": "desarbolar",
    "desarbolaste": "desarbolar",
    "desarbolasteis": "desarbolar",
    "desarbole": "desarbolar",
    "desarbolemos": "desarbolar",
    "desarbolen": "desarbolar",
    "desarboles": "desarbolar",
    "desarbolo": "desarbolar",
    "desarbolos": "desarbolo",
    "desarbolábamos": "desarbolar",
    "desarboláis": "desarbolar",
    "desarboláramos": "desarbolar",
    "desarboláremos": "desarbolar",
    "desarbolásemos": "desarbolar",
    "desarbolé": "desarbolar",
    "desarboléis": "desarbolar",
    "desarboló": "desarbolar",
    "desarena": "desarenar",
    "desarenaba": "desarenar",
    "desarenabais": "desarenar",
    "desarenaban": "desarenar",
    "desarenabas": "desarenar",
    "desarenaciones": "desarenación",
    "desarenad": "desarenar",
    "desarenada": "desarenar",
    "desarenadas": "desarenar",
    "desarenado": "desarenar",
    "desarenadores": "desarenador",
    "desarenados": "desarenar",
    "desarenamos": "desarenar",
    "desarenan": "desarenar",
    "desarenando": "desarenar",
    "desarenara": "desarenar",
    "desarenarais": "desarenar",
    "desarenaran": "desarenar",
    "desarenaras": "desarenar",
    "desarenare": "desarenar",
    "desarenareis": "desarenar",
    "desarenaremos": "desarenar",
    "desarenaren": "desarenar",
    "desarenares": "desarenar",
    "desarenaron": "desarenar",
    "desarenará": "desarenar",
    "desarenarán": "desarenar",
    "desarenarás": "desarenar",
    "desarenaré": "desarenar",
    "desarenaréis": "desarenar",
    "desarenaría": "desarenar",
    "desarenaríais": "desarenar",
    "desarenaríamos": "desarenar",
    "desarenarían": "desarenar",
    "desarenarías": "desarenar",
    "desarenas": "desarenar",
    "desarenase": "desarenar",
    "desarenaseis": "desarenar",
    "desarenasen": "desarenar",
    "desarenases": "desarenar",
    "desarenaste": "desarenar",
    "desarenasteis": "desarenar",
    "desarene": "desarenar",
    "desarenemos": "desarenar",
    "desarenen": "desarenar",
    "desarenes": "desarenar",
    "desareno": "desarenar",
    "desarenos": "desareno",
    "desarenábamos": "desarenar",
    "desarenáis": "desarenar",
    "desarenáramos": "desarenar",
    "desarenáremos": "desarenar",
    "desarenásemos": "desarenar",
    "desarené": "desarenar",
    "desarenéis": "desarenar",
    "desarenó": "desarenar",
    "desarma": "desarmar",
    "desarmaba": "desarmar",
    "desarmabais": "desarmar",
    "desarmaban": "desarmar",
    "desarmabas": "desarmar",
    "desarmad": "desarmar",
    "desarmada": "desarmar",
    "desarmadas": "desarmar",
    "desarmado": "desarmar",
    "desarmadores": "desarmador",
    "desarmados": "desarmar",
    "desarmamos": "desarmar",
    "desarman": "desarmar",
    "desarmando": "desarmar",
    "desarmara": "desarmar",
    "desarmarais": "desarmar",
    "desarmaran": "desarmar",
    "desarmaras": "desarmar",
    "desarmare": "desarmar",
    "desarmareis": "desarmar",
    "desarmaremos": "desarmar",
    "desarmaren": "desarmar",
    "desarmares": "desarmar",
    "desarmaron": "desarmar",
    "desarmará": "desarmar",
    "desarmarán": "desarmar",
    "desarmarás": "desarmar",
    "desarmaré": "desarmar",
    "desarmaréis": "desarmar",
    "desarmaría": "desarmar",
    "desarmaríais": "desarmar",
    "desarmaríamos": "desarmar",
    "desarmarían": "desarmar",
    "desarmarías": "desarmar",
    "desarmas": "desarmar",
    "desarmase": "desarmar",
    "desarmaseis": "desarmar",
    "desarmasen": "desarmar",
    "desarmases": "desarmar",
    "desarmaste": "desarmar",
    "desarmasteis": "desarmar",
    "desarme": "desarmar",
    "desarmemos": "desarmar",
    "desarmen": "desarmar",
    "desarmes": "desarme",
    "desarmo": "desarmar",
    "desarmonice": "desarmonizar",
    "desarmonicemos": "desarmonizar",
    "desarmonicen": "desarmonizar",
    "desarmonices": "desarmonizar",
    "desarmonicé": "desarmonizar",
    "desarmonicéis": "desarmonizar",
    "desarmoniza": "desarmonizar",
    "desarmonizaba": "desarmonizar",
    "desarmonizabais": "desarmonizar",
    "desarmonizaban": "desarmonizar",
    "desarmonizabas": "desarmonizar",
    "desarmonizad": "desarmonizar",
    "desarmonizada": "desarmonizar",
    "desarmonizadas": "desarmonizar",
    "desarmonizado": "desarmonizar",
    "desarmonizados": "desarmonizar",
    "desarmonizamos": "desarmonizar",
    "desarmonizan": "desarmonizar",
    "desarmonizando": "desarmonizar",
    "desarmonizara": "desarmonizar",
    "desarmonizarais": "desarmonizar",
    "desarmonizaran": "desarmonizar",
    "desarmonizaras": "desarmonizar",
    "desarmonizare": "desarmonizar",
    "desarmonizareis": "desarmonizar",
    "desarmonizaremos": "desarmonizar",
    "desarmonizaren": "desarmonizar",
    "desarmonizares": "desarmonizar",
    "desarmonizaron": "desarmonizar",
    "desarmonizará": "desarmonizar",
    "desarmonizarán": "desarmonizar",
    "desarmonizarás": "desarmonizar",
    "desarmonizaré": "desarmonizar",
    "desarmonizaréis": "desarmonizar",
    "desarmonizaría": "desarmonizar",
    "desarmonizaríais": "desarmonizar",
    "desarmonizaríamos": "desarmonizar",
    "desarmonizarían": "desarmonizar",
    "desarmonizarías": "desarmonizar",
    "desarmonizas": "desarmonizar",
    "desarmonizase": "desarmonizar",
    "desarmonizaseis": "desarmonizar",
    "desarmonizasen": "desarmonizar",
    "desarmonizases": "desarmonizar",
    "desarmonizaste": "desarmonizar",
    "desarmonizasteis": "desarmonizar",
    "desarmonizo": "desarmonizar",
    "desarmonizábamos": "desarmonizar",
    "desarmonizáis": "desarmonizar",
    "desarmonizáramos": "desarmonizar",
    "desarmonizáremos": "desarmonizar",
    "desarmonizásemos": "desarmonizar",
    "desarmonizó": "desarmonizar",
    "desarmábamos": "desarmar",
    "desarmáis": "desarmar",
    "desarmáramos": "desarmar",
    "desarmáremos": "desarmar",
    "desarmásemos": "desarmar",
    "desarmé": "desarmar",
    "desarméis": "desarmar",
    "desarmó": "desarmar",
    "desaromatizaciones": "desaromatización",
    "desarraiga": "desarraigar",
    "desarraigaba": "desarraigar",
    "desarraigabais": "desarraigar",
    "desarraigaban": "desarraigar",
    "desarraigabas": "desarraigar",
    "desarraigad": "desarraigar",
    "desarraigada": "desarraigar",
    "desarraigadas": "desarraigar",
    "desarraigado": "desarraigar",
    "desarraigados": "desarraigar",
    "desarraigamos": "desarraigar",
    "desarraigan": "desarraigar",
    "desarraigando": "desarraigar",
    "desarraigara": "desarraigar",
    "desarraigarais": "desarraigar",
    "desarraigaran": "desarraigar",
    "desarraigaras": "desarraigar",
    "desarraigare": "desarraigar",
    "desarraigareis": "desarraigar",
    "desarraigaremos": "desarraigar",
    "desarraigaren": "desarraigar",
    "desarraigares": "desarraigar",
    "desarraigaron": "desarraigar",
    "desarraigará": "desarraigar",
    "desarraigarán": "desarraigar",
    "desarraigarás": "desarraigar",
    "desarraigaré": "desarraigar",
    "desarraigaréis": "desarraigar",
    "desarraigaría": "desarraigar",
    "desarraigaríais": "desarraigar",
    "desarraigaríamos": "desarraigar",
    "desarraigarían": "desarraigar",
    "desarraigarías": "desarraigar",
    "desarraigas": "desarraigar",
    "desarraigase": "desarraigar",
    "desarraigaseis": "desarraigar",
    "desarraigasen": "desarraigar",
    "desarraigases": "desarraigar",
    "desarraigaste": "desarraigar",
    "desarraigasteis": "desarraigar",
    "desarraigo": "desarraigar",
    "desarraigos": "desarraigo",
    "desarraigue": "desarraigar",
    "desarraiguemos": "desarraigar",
    "desarraiguen": "desarraigar",
    "desarraigues": "desarraigar",
    "desarraigué": "desarraigar",
    "desarraiguéis": "desarraigar",
    "desarraigábamos": "desarraigar",
    "desarraigáis": "desarraigar",
    "desarraigáramos": "desarraigar",
    "desarraigáremos": "desarraigar",
    "desarraigásemos": "desarraigar",
    "desarraigó": "desarraigar",
    "desarrapada": "desarrapado",
    "desarrapadas": "desarrapado",
    "desarrapados": "desarrapado",
    "desarreboce": "desarrebozar",
    "desarrebocemos": "desarrebozar",
    "desarrebocen": "desarrebozar",
    "desarreboces": "desarrebozar",
    "desarrebocé": "desarrebozar",
    "desarrebocéis": "desarrebozar",
    "desarreboza": "desarrebozar",
    "desarrebozaba": "desarrebozar",
    "desarrebozabais": "desarrebozar",
    "desarrebozaban": "desarrebozar",
    "desarrebozabas": "desarrebozar",
    "desarrebozad": "desarrebozar",
    "desarrebozada": "desarrebozar",
    "desarrebozadas": "desarrebozar",
    "desarrebozado": "desarrebozar",
    "desarrebozados": "desarrebozar",
    "desarrebozamientos": "desarrebozamiento",
    "desarrebozamos": "desarrebozar",
    "desarrebozan": "desarrebozar",
    "desarrebozando": "desarrebozar",
    "desarrebozara": "desarrebozar",
    "desarrebozarais": "desarrebozar",
    "desarrebozaran": "desarrebozar",
    "desarrebozaras": "desarrebozar",
    "desarrebozare": "desarrebozar",
    "desarrebozareis": "desarrebozar",
    "desarrebozaremos": "desarrebozar",
    "desarrebozaren": "desarrebozar",
    "desarrebozares": "desarrebozar",
    "desarrebozaron": "desarrebozar",
    "desarrebozará": "desarrebozar",
    "desarrebozarán": "desarrebozar",
    "desarrebozarás": "desarrebozar",
    "desarrebozaré": "desarrebozar",
    "desarrebozaréis": "desarrebozar",
    "desarrebozaría": "desarrebozar",
    "desarrebozaríais": "desarrebozar",
    "desarrebozaríamos": "desarrebozar",
    "desarrebozarían": "desarrebozar",
    "desarrebozarías": "desarrebozar",
    "desarrebozas": "desarrebozar",
    "desarrebozase": "desarrebozar",
    "desarrebozaseis": "desarrebozar",
    "desarrebozasen": "desarrebozar",
    "desarrebozases": "desarrebozar",
    "desarrebozaste": "desarrebozar",
    "desarrebozasteis": "desarrebozar",
    "desarrebozo": "desarrebozar",
    "desarrebozábamos": "desarrebozar",
    "desarrebozáis": "desarrebozar",
    "desarrebozáramos": "desarrebozar",
    "desarrebozáremos": "desarrebozar",
    "desarrebozásemos": "desarrebozar",
    "desarrebozó": "desarrebozar",
    "desarregla": "desarreglar",
    "desarreglaba": "desarreglar",
    "desarreglabais": "desarreglar",
    "desarreglaban": "desarreglar",
    "desarreglabas": "desarreglar",
    "desarreglad": "desarreglar",
    "desarreglada": "desarreglar",
    "desarregladas": "desarreglar",
    "desarreglado": "desarreglar",
    "desarreglados": "desarreglar",
    "desarreglamos": "desarreglar",
    "desarreglan": "desarreglar",
    "desarreglando": "desarreglar",
    "desarreglara": "desarreglar",
    "desarreglarais": "desarreglar",
    "desarreglaran": "desarreglar",
    "desarreglaras": "desarreglar",
    "desarreglare": "desarreglar",
    "desarreglareis": "desarreglar",
    "desarreglaremos": "desarreglar",
    "desarreglaren": "desarreglar",
    "desarreglares": "desarreglar",
    "desarreglaron": "desarreglar",
    "desarreglará": "desarreglar",
    "desarreglarán": "desarreglar",
    "desarreglarás": "desarreglar",
    "desarreglaré": "desarreglar",
    "desarreglaréis": "desarreglar",
    "desarreglaría": "desarreglar",
    "desarreglaríais": "desarreglar",
    "desarreglaríamos": "desarreglar",
    "desarreglarían": "desarreglar",
    "desarreglarías": "desarreglar",
    "desarreglas": "desarreglar",
    "desarreglase": "desarreglar",
    "desarreglaseis": "desarreglar",
    "desarreglasen": "desarreglar",
    "desarreglases": "desarreglar",
    "desarreglaste": "desarreglar",
    "desarreglasteis": "desarreglar",
    "desarregle": "desarreglar",
    "desarreglemos": "desarreglar",
    "desarreglen": "desarreglar",
    "desarregles": "desarreglar",
    "desarreglo": "desarreglar",
    "desarreglos": "desarreglo",
    "desarreglábamos": "desarreglar",
    "desarregláis": "desarreglar",
    "desarregláramos": "desarreglar",
    "desarregláremos": "desarreglar",
    "desarreglásemos": "desarreglar",
    "desarreglé": "desarreglar",
    "desarregléis": "desarreglar",
    "desarregló": "desarreglar",
    "desarremanga": "desarremangar",
    "desarremangaba": "desarremangar",
    "desarremangabais": "desarremangar",
    "desarremangaban": "desarremangar",
    "desarremangabas": "desarremangar",
    "desarremangad": "desarremangar",
    "desarremangada": "desarremangar",
    "desarremangadas": "desarremangar",
    "desarremangado": "desarremangar",
    "desarremangados": "desarremangar",
    "desarremangamos": "desarremangar",
    "desarremangan": "desarremangar",
    "desarremangando": "desarremangar",
    "desarremangara": "desarremangar",
    "desarremangarais": "desarremangar",
    "desarremangaran": "desarremangar",
    "desarremangaras": "desarremangar",
    "desarremangare": "desarremangar",
    "desarremangareis": "desarremangar",
    "desarremangaremos": "desarremangar",
    "desarremangaren": "desarremangar",
    "desarremangares": "desarremangar",
    "desarremangaron": "desarremangar",
    "desarremangará": "desarremangar",
    "desarremangarán": "desarremangar",
    "desarremangarás": "desarremangar",
    "desarremangaré": "desarremangar",
    "desarremangaréis": "desarremangar",
    "desarremangaría": "desarremangar",
    "desarremangaríais": "desarremangar",
    "desarremangaríamos": "desarremangar",
    "desarremangarían": "desarremangar",
    "desarremangarías": "desarremangar",
    "desarremangas": "desarremangar",
    "desarremangase": "desarremangar",
    "desarremangaseis": "desarremangar",
    "desarremangasen": "desarremangar",
    "desarremangases": "desarremangar",
    "desarremangaste": "desarremangar",
    "desarremangasteis": "desarremangar",
    "desarremango": "desarremangar",
    "desarremangue": "desarremangar",
    "desarremanguemos": "desarremangar",
    "desarremanguen": "desarremangar",
    "desarremangues": "desarremangar",
    "desarremangué": "desarremangar",
    "desarremanguéis": "desarremangar",
    "desarremangábamos": "desarremangar",
    "desarremangáis": "desarremangar",
    "desarremangáramos": "desarremangar",
    "desarremangáremos": "desarremangar",
    "desarremangásemos": "desarremangar",
    "desarremangó": "desarremangar",
    "desarrendaba": "desarrendar",
    "desarrendabais": "desarrendar",
    "desarrendaban": "desarrendar",
    "desarrendabas": "desarrendar",
    "desarrendad": "desarrendar",
    "desarrendada": "desarrendar",
    "desarrendadas": "desarrendar",
    "desarrendado": "desarrendar",
    "desarrendados": "desarrendar",
    "desarrendamos": "desarrendar",
    "desarrendando": "desarrendar",
    "desarrendara": "desarrendar",
    "desarrendarais": "desarrendar",
    "desarrendaran": "desarrendar",
    "desarrendaras": "desarrendar",
    "desarrendare": "desarrendar",
    "desarrendareis": "desarrendar",
    "desarrendaremos": "desarrendar",
    "desarrendaren": "desarrendar",
    "desarrendares": "desarrendar",
    "desarrendaron": "desarrendar",
    "desarrendará": "desarrendar",
    "desarrendarán": "desarrendar",
    "desarrendarás": "desarrendar",
    "desarrendaré": "desarrendar",
    "desarrendaréis": "desarrendar",
    "desarrendaría": "desarrendar",
    "desarrendaríais": "desarrendar",
    "desarrendaríamos": "desarrendar",
    "desarrendarían": "desarrendar",
    "desarrendarías": "desarrendar",
    "desarrendase": "desarrendar",
    "desarrendaseis": "desarrendar",
    "desarrendasen": "desarrendar",
    "desarrendases": "desarrendar",
    "desarrendaste": "desarrendar",
    "desarrendasteis": "desarrendar",
    "desarrendemos": "desarrendar",
    "desarrendábamos": "desarrendar",
    "desarrendáis": "desarrendar",
    "desarrendáramos": "desarrendar",
    "desarrendáremos": "desarrendar",
    "desarrendásemos": "desarrendar",
    "desarrendé": "desarrendar",
    "desarrendéis": "desarrendar",
    "desarrendó": "desarrendar",
    "desarrienda": "desarrendar",
    "desarriendan": "desarrendar",
    "desarriendas": "desarrendar",
    "desarriende": "desarrendar",
    "desarrienden": "desarrendar",
    "desarriendes": "desarrendar",
    "desarriendo": "desarrendar",
    "desarrima": "desarrimar",
    "desarrimaba": "desarrimar",
    "desarrimabais": "desarrimar",
    "desarrimaban": "desarrimar",
    "desarrimabas": "desarrimar",
    "desarrimad": "desarrimar",
    "desarrimada": "desarrimar",
    "desarrimadas": "desarrimar",
    "desarrimado": "desarrimar",
    "desarrimados": "desarrimar",
    "desarrimamos": "desarrimar",
    "desarriman": "desarrimar",
    "desarrimando": "desarrimar",
    "desarrimara": "desarrimar",
    "desarrimarais": "desarrimar",
    "desarrimaran": "desarrimar",
    "desarrimaras": "desarrimar",
    "desarrimare": "desarrimar",
    "desarrimareis": "desarrimar",
    "desarrimaremos": "desarrimar",
    "desarrimaren": "desarrimar",
    "desarrimares": "desarrimar",
    "desarrimaron": "desarrimar",
    "desarrimará": "desarrimar",
    "desarrimarán": "desarrimar",
    "desarrimarás": "desarrimar",
    "desarrimaré": "desarrimar",
    "desarrimaréis": "desarrimar",
    "desarrimaría": "desarrimar",
    "desarrimaríais": "desarrimar",
    "desarrimaríamos": "desarrimar",
    "desarrimarían": "desarrimar",
    "desarrimarías": "desarrimar",
    "desarrimas": "desarrimar",
    "desarrimase": "desarrimar",
    "desarrimaseis": "desarrimar",
    "desarrimasen": "desarrimar",
    "desarrimases": "desarrimar",
    "desarrimaste": "desarrimar",
    "desarrimasteis": "desarrimar",
    "desarrime": "desarrimar",
    "desarrimemos": "desarrimar",
    "desarrimen": "desarrimar",
    "desarrimes": "desarrimar",
    "desarrimo": "desarrimar",
    "desarrimos": "desarrimo",
    "desarrimábamos": "desarrimar",
    "desarrimáis": "desarrimar",
    "desarrimáramos": "desarrimar",
    "desarrimáremos": "desarrimar",
    "desarrimásemos": "desarrimar",
    "desarrimé": "desarrimar",
    "desarriméis": "desarrimar",
    "desarrimó": "desarrimar",
    "desarrolla": "desarrollar",
    "desarrollaba": "desarrollar",
    "desarrollabais": "desarrollar",
    "desarrollaban": "desarrollar",
    "desarrollabas": "desarrollar",
    "desarrollables": "desarrollable",
    "desarrollad": "desarrollar",
    "desarrollada": "desarrollar",
    "desarrolladas": "desarrollar",
    "desarrollado": "desarrollar",
    "desarrolladora": "desarrollador",
    "desarrolladoras": "desarrollador",
    "desarrolladores": "desarrollador",
    "desarrollados": "desarrollar",
    "desarrollamos": "desarrollar",
    "desarrollan": "desarrollar",
    "desarrollando": "desarrollar",
    "desarrollara": "desarrollar",
    "desarrollarais": "desarrollar",
    "desarrollaran": "desarrollar",
    "desarrollaras": "desarrollar",
    "desarrollare": "desarrollar",
    "desarrollareis": "desarrollar",
    "desarrollaremos": "desarrollar",
    "desarrollaren": "desarrollar",
    "desarrollares": "desarrollar",
    "desarrollaron": "desarrollar",
    "desarrollará": "desarrollar",
    "desarrollarán": "desarrollar",
    "desarrollarás": "desarrollar",
    "desarrollaré": "desarrollar",
    "desarrollaréis": "desarrollar",
    "desarrollaría": "desarrollar",
    "desarrollaríais": "desarrollar",
    "desarrollaríamos": "desarrollar",
    "desarrollarían": "desarrollar",
    "desarrollarías": "desarrollar",
    "desarrollas": "desarrollar",
    "desarrollase": "desarrollar",
    "desarrollaseis": "desarrollar",
    "desarrollasen": "desarrollar",
    "desarrollases": "desarrollar",
    "desarrollaste": "desarrollar",
    "desarrollasteis": "desarrollar",
    "desarrolle": "desarrollar",
    "desarrollemos": "desarrollar",
    "desarrollen": "desarrollar",
    "desarrolles": "desarrollar",
    "desarrollismos": "desarrollismo",
    "desarrollo": "desarrollar",
    "desarrollos": "desarrollo",
    "desarrollábamos": "desarrollar",
    "desarrolláis": "desarrollar",
    "desarrolláramos": "desarrollar",
    "desarrolláremos": "desarrollar",
    "desarrollásemos": "desarrollar",
    "desarrollé": "desarrollar",
    "desarrolléis": "desarrollar",
    "desarrolló": "desarrollar",
    "desarropa": "desarropar",
    "desarropaba": "desarropar",
    "desarropabais": "desarropar",
    "desarropaban": "desarropar",
    "desarropabas": "desarropar",
    "desarropad": "desarropar",
    "desarropada": "desarropar",
    "desarropadas": "desarropar",
    "desarropado": "desarropar",
    "desarropados": "desarropar",
    "desarropamos": "desarropar",
    "desarropan": "desarropar",
    "desarropando": "desarropar",
    "desarropara": "desarropar",
    "desarroparais": "desarropar",
    "desarroparan": "desarropar",
    "desarroparas": "desarropar",
    "desarropare": "desarropar",
    "desarropareis": "desarropar",
    "desarroparemos": "desarropar",
    "desarroparen": "desarropar",
    "desarropares": "desarropar",
    "desarroparon": "desarropar",
    "desarropará": "desarropar",
    "desarroparán": "desarropar",
    "desarroparás": "desarropar",
    "desarroparé": "desarropar",
    "desarroparéis": "desarropar",
    "desarroparía": "desarropar",
    "desarroparíais": "desarropar",
    "desarroparíamos": "desarropar",
    "desarroparían": "desarropar",
    "desarroparías": "desarropar",
    "desarropas": "desarropar",
    "desarropase": "desarropar",
    "desarropaseis": "desarropar",
    "desarropasen": "desarropar",
    "desarropases": "desarropar",
    "desarropaste": "desarropar",
    "desarropasteis": "desarropar",
    "desarrope": "desarropar",
    "desarropemos": "desarropar",
    "desarropen": "desarropar",
    "desarropes": "desarropar",
    "desarropo": "desarropar",
    "desarropábamos": "desarropar",
    "desarropáis": "desarropar",
    "desarropáramos": "desarropar",
    "desarropáremos": "desarropar",
    "desarropásemos": "desarropar",
    "desarropé": "desarropar",
    "desarropéis": "desarropar",
    "desarropó": "desarropar",
    "desarruga": "desarrugar",
    "desarrugaba": "desarrugar",
    "desarrugabais": "desarrugar",
    "desarrugaban": "desarrugar",
    "desarrugabas": "desarrugar",
    "desarrugad": "desarrugar",
    "desarrugada": "desarrugar",
    "desarrugadas": "desarrugar",
    "desarrugado": "desarrugar",
    "desarrugados": "desarrugar",
    "desarrugaduras": "desarrugadura",
    "desarrugamos": "desarrugar",
    "desarrugan": "desarrugar",
    "desarrugando": "desarrugar",
    "desarrugara": "desarrugar",
    "desarrugarais": "desarrugar",
    "desarrugaran": "desarrugar",
    "desarrugaras": "desarrugar",
    "desarrugare": "desarrugar",
    "desarrugareis": "desarrugar",
    "desarrugaremos": "desarrugar",
    "desarrugaren": "desarrugar",
    "desarrugares": "desarrugar",
    "desarrugaron": "desarrugar",
    "desarrugará": "desarrugar",
    "desarrugarán": "desarrugar",
    "desarrugarás": "desarrugar",
    "desarrugaré": "desarrugar",
    "desarrugaréis": "desarrugar",
    "desarrugaría": "desarrugar",
    "desarrugaríais": "desarrugar",
    "desarrugaríamos": "desarrugar",
    "desarrugarían": "desarrugar",
    "desarrugarías": "desarrugar",
    "desarrugas": "desarrugar",
    "desarrugase": "desarrugar",
    "desarrugaseis": "desarrugar",
    "desarrugasen": "desarrugar",
    "desarrugases": "desarrugar",
    "desarrugaste": "desarrugar",
    "desarrugasteis": "desarrugar",
    "desarrugo": "desarrugar",
    "desarrugue": "desarrugar",
    "desarruguemos": "desarrugar",
    "desarruguen": "desarrugar",
    "desarrugues": "desarrugar",
    "desarrugué": "desarrugar",
    "desarruguéis": "desarrugar",
    "desarrugábamos": "desarrugar",
    "desarrugáis": "desarrugar",
    "desarrugáramos": "desarrugar",
    "desarrugáremos": "desarrugar",
    "desarrugásemos": "desarrugar",
    "desarrugó": "desarrugar",
    "desarruma": "desarrumar",
    "desarrumaba": "desarrumar",
    "desarrumabais": "desarrumar",
    "desarrumaban": "desarrumar",
    "desarrumabas": "desarrumar",
    "desarrumad": "desarrumar",
    "desarrumada": "desarrumar",
    "desarrumadas": "desarrumar",
    "desarrumado": "desarrumar",
    "desarrumados": "desarrumar",
    "desarrumamos": "desarrumar",
    "desarruman": "desarrumar",
    "desarrumando": "desarrumar",
    "desarrumara": "desarrumar",
    "desarrumarais": "desarrumar",
    "desarrumaran": "desarrumar",
    "desarrumaras": "desarrumar",
    "desarrumare": "desarrumar",
    "desarrumareis": "desarrumar",
    "desarrumaremos": "desarrumar",
    "desarrumaren": "desarrumar",
    "desarrumares": "desarrumar",
    "desarrumaron": "desarrumar",
    "desarrumará": "desarrumar",
    "desarrumarán": "desarrumar",
    "desarrumarás": "desarrumar",
    "desarrumaré": "desarrumar",
    "desarrumaréis": "desarrumar",
    "desarrumaría": "desarrumar",
    "desarrumaríais": "desarrumar",
    "desarrumaríamos": "desarrumar",
    "desarrumarían": "desarrumar",
    "desarrumarías": "desarrumar",
    "desarrumas": "desarrumar",
    "desarrumase": "desarrumar",
    "desarrumaseis": "desarrumar",
    "desarrumasen": "desarrumar",
    "desarrumases": "desarrumar",
    "desarrumaste": "desarrumar",
    "desarrumasteis": "desarrumar",
    "desarrume": "desarrumar",
    "desarrumemos": "desarrumar",
    "desarrumen": "desarrumar",
    "desarrumes": "desarrumar",
    "desarrumo": "desarrumar",
    "desarrumábamos": "desarrumar",
    "desarrumáis": "desarrumar",
    "desarrumáramos": "desarrumar",
    "desarrumáremos": "desarrumar",
    "desarrumásemos": "desarrumar",
    "desarrumé": "desarrumar",
    "desarruméis": "desarrumar",
    "desarrumó": "desarrumar",
    "desarticula": "desarticular",
    "desarticulaba": "desarticular",
    "desarticulabais": "desarticular",
    "desarticulaban": "desarticular",
    "desarticulabas": "desarticular",
    "desarticulaciones": "desarticulación",
    "desarticulad": "desarticular",
    "desarticulada": "desarticular",
    "desarticuladas": "desarticular",
    "desarticulado": "desarticular",
    "desarticulados": "desarticular",
    "desarticulamos": "desarticular",
    "desarticulan": "desarticular",
    "desarticulando": "desarticular",
    "desarticulara": "desarticular",
    "desarticularais": "desarticular",
    "desarticularan": "desarticular",
    "desarticularas": "desarticular",
    "desarticulare": "desarticular",
    "desarticulareis": "desarticular",
    "desarticularemos": "desarticular",
    "desarticularen": "desarticular",
    "desarticulares": "desarticular",
    "desarticularon": "desarticular",
    "desarticulará": "desarticular",
    "desarticularán": "desarticular",
    "desarticularás": "desarticular",
    "desarticularé": "desarticular",
    "desarticularéis": "desarticular",
    "desarticularía": "desarticular",
    "desarticularíais": "desarticular",
    "desarticularíamos": "desarticular",
    "desarticularían": "desarticular",
    "desarticularías": "desarticular",
    "desarticulas": "desarticular",
    "desarticulase": "desarticular",
    "desarticulaseis": "desarticular",
    "desarticulasen": "desarticular",
    "desarticulases": "desarticular",
    "desarticulaste": "desarticular",
    "desarticulasteis": "desarticular",
    "desarticule": "desarticular",
    "desarticulemos": "desarticular",
    "desarticulen": "desarticular",
    "desarticules": "desarticular",
    "desarticulo": "desarticular",
    "desarticulábamos": "desarticular",
    "desarticuláis": "desarticular",
    "desarticuláramos": "desarticular",
    "desarticuláremos": "desarticular",
    "desarticulásemos": "desarticular",
    "desarticulé": "desarticular",
    "desarticuléis": "desarticular",
    "desarticuló": "desarticular",
    "desartilla": "desartillar",
    "desartillaba": "desartillar",
    "desartillabais": "desartillar",
    "desartillaban": "desartillar",
    "desartillabas": "desartillar",
    "desartillad": "desartillar",
    "desartillada": "desartillar",
    "desartilladas": "desartillar",
    "desartillado": "desartillar",
    "desartillados": "desartillar",
    "desartillamos": "desartillar",
    "desartillan": "desartillar",
    "desartillando": "desartillar",
    "desartillara": "desartillar",
    "desartillarais": "desartillar",
    "desartillaran": "desartillar",
    "desartillaras": "desartillar",
    "desartillare": "desartillar",
    "desartillareis": "desartillar",
    "desartillaremos": "desartillar",
    "desartillaren": "desartillar",
    "desartillares": "desartillar",
    "desartillaron": "desartillar",
    "desartillará": "desartillar",
    "desartillarán": "desartillar",
    "desartillarás": "desartillar",
    "desartillaré": "desartillar",
    "desartillaréis": "desartillar",
    "desartillaría": "desartillar",
    "desartillaríais": "desartillar",
    "desartillaríamos": "desartillar",
    "desartillarían": "desartillar",
    "desartillarías": "desartillar",
    "desartillas": "desartillar",
    "desartillase": "desartillar",
    "desartillaseis": "desartillar",
    "desartillasen": "desartillar",
    "desartillases": "desartillar",
    "desartillaste": "desartillar",
    "desartillasteis": "desartillar",
    "desartille": "desartillar",
    "desartillemos": "desartillar",
    "desartillen": "desartillar",
    "desartilles": "desartillar",
    "desartillo": "desartillar",
    "desartillábamos": "desartillar",
    "desartilláis": "desartillar",
    "desartilláramos": "desartillar",
    "desartilláremos": "desartillar",
    "desartillásemos": "desartillar",
    "desartillé": "desartillar",
    "desartilléis": "desartillar",
    "desartilló": "desartillar",
    "desarzona": "desarzonar",
    "desarzonaba": "desarzonar",
    "desarzonabais": "desarzonar",
    "desarzonaban": "desarzonar",
    "desarzonabas": "desarzonar",
    "desarzonad": "desarzonar",
    "desarzonada": "desarzonar",
    "desarzonadas": "desarzonar",
    "desarzonado": "desarzonar",
    "desarzonados": "desarzonar",
    "desarzonamos": "desarzonar",
    "desarzonan": "desarzonar",
    "desarzonando": "desarzonar",
    "desarzonara": "desarzonar",
    "desarzonarais": "desarzonar",
    "desarzonaran": "desarzonar",
    "desarzonaras": "desarzonar",
    "desarzonare": "desarzonar",
    "desarzonareis": "desarzonar",
    "desarzonaremos": "desarzonar",
    "desarzonaren": "desarzonar",
    "desarzonares": "desarzonar",
    "desarzonaron": "desarzonar",
    "desarzonará": "desarzonar",
    "desarzonarán": "desarzonar",
    "desarzonarás": "desarzonar",
    "desarzonaré": "desarzonar",
    "desarzonaréis": "desarzonar",
    "desarzonaría": "desarzonar",
    "desarzonaríais": "desarzonar",
    "desarzonaríamos": "desarzonar",
    "desarzonarían": "desarzonar",
    "desarzonarías": "desarzonar",
    "desarzonas": "desarzonar",
    "desarzonase": "desarzonar",
    "desarzonaseis": "desarzonar",
    "desarzonasen": "desarzonar",
    "desarzonases": "desarzonar",
    "desarzonaste": "desarzonar",
    "desarzonasteis": "desarzonar",
    "desarzone": "desarzonar",
    "desarzonemos": "desarzonar",
    "desarzonen": "desarzonar",
    "desarzones": "desarzonar",
    "desarzono": "desarzonar",
    "desarzonábamos": "desarzonar",
    "desarzonáis": "desarzonar",
    "desarzonáramos": "desarzonar",
    "desarzonáremos": "desarzonar",
    "desarzonásemos": "desarzonar",
    "desarzoné": "desarzonar",
    "desarzonéis": "desarzonar",
    "desarzonó": "desarzonar",
    "desasada": "desasado",
    "desasadas": "desasado",
    "desasados": "desasado",
    "desase": "desasir",
    "desasea": "desasear",
    "desaseaba": "desasear",
    "desaseabais": "desasear",
    "desaseaban": "desasear",
    "desaseabas": "desasear",
    "desasead": "desasear",
    "desaseada": "desasear",
    "desaseadas": "desasear",
    "desaseado": "desasear",
    "desaseados": "desasear",
    "desaseamos": "desasear",
    "desasean": "desasear",
    "desaseando": "desasear",
    "desaseara": "desasear",
    "desasearais": "desasear",
    "desasearan": "desasear",
    "desasearas": "desasear",
    "desaseare": "desasear",
    "desaseareis": "desasear",
    "desasearemos": "desasear",
    "desasearen": "desasear",
    "desaseares": "desasear",
    "desasearon": "desasear",
    "desaseará": "desasear",
    "desasearán": "desasear",
    "desasearás": "desasear",
    "desasearé": "desasear",
    "desasearéis": "desasear",
    "desasearía": "desasear",
    "desasearíais": "desasear",
    "desasearíamos": "desasear",
    "desasearían": "desasear",
    "desasearías": "desasear",
    "desaseas": "desasear",
    "desasease": "desasear",
    "desaseaseis": "desasear",
    "desaseasen": "desasear",
    "desaseases": "desasear",
    "desaseaste": "desasear",
    "desaseasteis": "desasear",
    "desasee": "desasear",
    "desaseemos": "desasear",
    "desaseen": "desasear",
    "desasees": "desasear",
    "desasen": "desasir",
    "desasentaba": "desasentar",
    "desasentabais": "desasentar",
    "desasentaban": "desasentar",
    "desasentabas": "desasentar",
    "desasentad": "desasentar",
    "desasentada": "desasentar",
    "desasentadas": "desasentar",
    "desasentado": "desasentar",
    "desasentados": "desasentar",
    "desasentamos": "desasentar",
    "desasentando": "desasentar",
    "desasentara": "desasentar",
    "desasentarais": "desasentar",
    "desasentaran": "desasentar",
    "desasentaras": "desasentar",
    "desasentare": "desasentar",
    "desasentareis": "desasentar",
    "desasentaremos": "desasentar",
    "desasentaren": "desasentar",
    "desasentares": "desasentar",
    "desasentaron": "desasentar",
    "desasentará": "desasentar",
    "desasentarán": "desasentar",
    "desasentarás": "desasentar",
    "desasentaré": "desasentar",
    "desasentaréis": "desasentar",
    "desasentaría": "desasentar",
    "desasentaríais": "desasentar",
    "desasentaríamos": "desasentar",
    "desasentarían": "desasentar",
    "desasentarías": "desasentar",
    "desasentase": "desasentar",
    "desasentaseis": "desasentar",
    "desasentasen": "desasentar",
    "desasentases": "desasentar",
    "desasentaste": "desasentar",
    "desasentasteis": "desasentar",
    "desasentemos": "desasentar",
    "desasentábamos": "desasentar",
    "desasentáis": "desasentar",
    "desasentáramos": "desasentar",
    "desasentáremos": "desasentar",
    "desasentásemos": "desasentar",
    "desasenté": "desasentar",
    "desasentéis": "desasentar",
    "desasentó": "desasentar",
    "desaseo": "desasear",
    "desaseos": "desaseo",
    "desases": "desasir",
    "desaseábamos": "desasear",
    "desaseáis": "desasear",
    "desaseáramos": "desasear",
    "desaseáremos": "desasear",
    "desaseásemos": "desasear",
    "desaseé": "desasear",
    "desaseéis": "desasear",
    "desaseó": "desasear",
    "desasfaltizaciones": "desasfaltización",
    "desasga": "desasir",
    "desasgamos": "desasir",
    "desasgan": "desasir",
    "desasgas": "desasir",
    "desasgo": "desasir",
    "desasgáis": "desasir",
    "desasid": "desasir",
    "desasida": "desasir",
    "desasidas": "desasir",
    "desasido": "desasir",
    "desasidos": "desasir",
    "desasiendo": "desasir",
    "desasienta": "desasentar",
    "desasientan": "desasentar",
    "desasientas": "desasentar",
    "desasiente": "desasentar",
    "desasienten": "desasentar",
    "desasientes": "desasentar",
    "desasiento": "desasentar",
    "desasiera": "desasir",
    "desasierais": "desasir",
    "desasieran": "desasir",
    "desasieras": "desasir",
    "desasiere": "desasir",
    "desasiereis": "desasir",
    "desasieren": "desasir",
    "desasieres": "desasir",
    "desasieron": "desasir",
    "desasiese": "desasir",
    "desasieseis": "desasir",
    "desasiesen": "desasir",
    "desasieses": "desasir",
    "desasimientos": "desasimiento",
    "desasimila": "desasimilar",
    "desasimilaba": "desasimilar",
    "desasimilabais": "desasimilar",
    "desasimilaban": "desasimilar",
    "desasimilabas": "desasimilar",
    "desasimilaciones": "desasimilación",
    "desasimilad": "desasimilar",
    "desasimilada": "desasimilar",
    "desasimiladas": "desasimilar",
    "desasimilado": "desasimilar",
    "desasimilados": "desasimilar",
    "desasimilamos": "desasimilar",
    "desasimilan": "desasimilar",
    "desasimilando": "desasimilar",
    "desasimilara": "desasimilar",
    "desasimilarais": "desasimilar",
    "desasimilaran": "desasimilar",
    "desasimilaras": "desasimilar",
    "desasimilare": "desasimilar",
    "desasimilareis": "desasimilar",
    "desasimilaremos": "desasimilar",
    "desasimilaren": "desasimilar",
    "desasimilares": "desasimilar",
    "desasimilaron": "desasimilar",
    "desasimilará": "desasimilar",
    "desasimilarán": "desasimilar",
    "desasimilarás": "desasimilar",
    "desasimilaré": "desasimilar",
    "desasimilaréis": "desasimilar",
    "desasimilaría": "desasimilar",
    "desasimilaríais": "desasimilar",
    "desasimilaríamos": "desasimilar",
    "desasimilarían": "desasimilar",
    "desasimilarías": "desasimilar",
    "desasimilas": "desasimilar",
    "desasimilase": "desasimilar",
    "desasimilaseis": "desasimilar",
    "desasimilasen": "desasimilar",
    "desasimilases": "desasimilar",
    "desasimilaste": "desasimilar",
    "desasimilasteis": "desasimilar",
    "desasimile": "desasimilar",
    "desasimilemos": "desasimilar",
    "desasimilen": "desasimilar",
    "desasimiles": "desasimilar",
    "desasimilo": "desasimilar",
    "desasimilábamos": "desasimilar",
    "desasimiláis": "desasimilar",
    "desasimiláramos": "desasimilar",
    "desasimiláremos": "desasimilar",
    "desasimilásemos": "desasimilar",
    "desasimilé": "desasimilar",
    "desasimiléis": "desasimilar",
    "desasimiló": "desasimilar",
    "desasimos": "desasir",
    "desasiremos": "desasir",
    "desasirá": "desasir",
    "desasirán": "desasir",
    "desasirás": "desasir",
    "desasiré": "desasir",
    "desasiréis": "desasir",
    "desasiría": "desasir",
    "desasiríais": "desasir",
    "desasiríamos": "desasir",
    "desasirían": "desasir",
    "desasirías": "desasir",
    "desasista": "desasistir",
    "desasistamos": "desasistir",
    "desasistan": "desasistir",
    "desasistas": "desasistir",
    "desasiste": "desasistir",
    "desasisteis": "desasir",
    "desasisten": "desasistir",
    "desasistencias": "desasistencia",
    "desasistes": "desasistir",
    "desasistid": "desasistir",
    "desasistida": "desasistir",
    "desasistidas": "desasistir",
    "desasistido": "desasistir",
    "desasistidos": "desasistir",
    "desasistiendo": "desasistir",
    "desasistiera": "desasistir",
    "desasistierais": "desasistir",
    "desasistieran": "desasistir",
    "desasistieras": "desasistir",
    "desasistiere": "desasistir",
    "desasistiereis": "desasistir",
    "desasistieren": "desasistir",
    "desasistieres": "desasistir",
    "desasistieron": "desasistir",
    "desasistiese": "desasistir",
    "desasistieseis": "desasistir",
    "desasistiesen": "desasistir",
    "desasistieses": "desasistir",
    "desasistimos": "desasistir",
    "desasistiremos": "desasistir",
    "desasistirá": "desasistir",
    "desasistirán": "desasistir",
    "desasistirás": "desasistir",
    "desasistiré": "desasistir",
    "desasistiréis": "desasistir",
    "desasistiría": "desasistir",
    "desasistiríais": "desasistir",
    "desasistiríamos": "desasistir",
    "desasistirían": "desasistir",
    "desasistirías": "desasistir",
    "desasististe": "desasistir",
    "desasististeis": "desasistir",
    "desasistiéramos": "desasistir",
    "desasistiéremos": "desasistir",
    "desasistiésemos": "desasistir",
    "desasistió": "desasistir",
    "desasisto": "desasistir",
    "desasistáis": "desasistir",
    "desasistí": "desasistir",
    "desasistía": "desasistir",
    "desasistíais": "desasistir",
    "desasistíamos": "desasistir",
    "desasistían": "desasistir",
    "desasistías": "desasistir",
    "desasistís": "desasistir",
    "desasiéramos": "desasir",
    "desasiéremos": "desasir",
    "desasiésemos": "desasir",
    "desasió": "desasir",
    "desasna": "desasnar",
    "desasnaba": "desasnar",
    "desasnabais": "desasnar",
    "desasnaban": "desasnar",
    "desasnabas": "desasnar",
    "desasnad": "desasnar",
    "desasnada": "desasnar",
    "desasnadas": "desasnar",
    "desasnado": "desasnar",
    "desasnados": "desasnar",
    "desasnamos": "desasnar",
    "desasnan": "desasnar",
    "desasnando": "desasnar",
    "desasnara": "desasnar",
    "desasnarais": "desasnar",
    "desasnaran": "desasnar",
    "desasnaras": "desasnar",
    "desasnare": "desasnar",
    "desasnareis": "desasnar",
    "desasnaremos": "desasnar",
    "desasnaren": "desasnar",
    "desasnares": "desasnar",
    "desasnaron": "desasnar",
    "desasnará": "desasnar",
    "desasnarán": "desasnar",
    "desasnarás": "desasnar",
    "desasnaré": "desasnar",
    "desasnaréis": "desasnar",
    "desasnaría": "desasnar",
    "desasnaríais": "desasnar",
    "desasnaríamos": "desasnar",
    "desasnarían": "desasnar",
    "desasnarías": "desasnar",
    "desasnas": "desasnar",
    "desasnase": "desasnar",
    "desasnaseis": "desasnar",
    "desasnasen": "desasnar",
    "desasnases": "desasnar",
    "desasnaste": "desasnar",
    "desasnasteis": "desasnar",
    "desasne": "desasnar",
    "desasnemos": "desasnar",
    "desasnen": "desasnar",
    "desasnes": "desasnar",
    "desasno": "desasnar",
    "desasnábamos": "desasnar",
    "desasnáis": "desasnar",
    "desasnáramos": "desasnar",
    "desasnáremos": "desasnar",
    "desasnásemos": "desasnar",
    "desasné": "desasnar",
    "desasnéis": "desasnar",
    "desasnó": "desasnar",
    "desasocia": "desasociar",
    "desasociaba": "desasociar",
    "desasociabais": "desasociar",
    "desasociaban": "desasociar",
    "desasociabas": "desasociar",
    "desasociad": "desasociar",
    "desasociada": "desasociar",
    "desasociadas": "desasociar",
    "desasociado": "desasociar",
    "desasociados": "desasociar",
    "desasociamos": "desasociar",
    "desasocian": "desasociar",
    "desasociando": "desasociar",
    "desasociara": "desasociar",
    "desasociarais": "desasociar",
    "desasociaran": "desasociar",
    "desasociaras": "desasociar",
    "desasociare": "desasociar",
    "desasociareis": "desasociar",
    "desasociaremos": "desasociar",
    "desasociaren": "desasociar",
    "desasociares": "desasociar",
    "desasociaron": "desasociar",
    "desasociará": "desasociar",
    "desasociarán": "desasociar",
    "desasociarás": "desasociar",
    "desasociaré": "desasociar",
    "desasociaréis": "desasociar",
    "desasociaría": "desasociar",
    "desasociaríais": "desasociar",
    "desasociaríamos": "desasociar",
    "desasociarían": "desasociar",
    "desasociarías": "desasociar",
    "desasocias": "desasociar",
    "desasociase": "desasociar",
    "desasociaseis": "desasociar",
    "desasociasen": "desasociar",
    "desasociases": "desasociar",
    "desasociaste": "desasociar",
    "desasociasteis": "desasociar",
    "desasocie": "desasociar",
    "desasociemos": "desasociar",
    "desasocien": "desasociar",
    "desasocies": "desasociar",
    "desasocio": "desasociar",
    "desasociábamos": "desasociar",
    "desasociáis": "desasociar",
    "desasociáramos": "desasociar",
    "desasociáremos": "desasociar",
    "desasociásemos": "desasociar",
    "desasocié": "desasociar",
    "desasociéis": "desasociar",
    "desasoció": "desasociar",
    "desasosegaba": "desasosegar",
    "desasosegabais": "desasosegar",
    "desasosegaban": "desasosegar",
    "desasosegabas": "desasosegar",
    "desasosegad": "desasosegar",
    "desasosegada": "desasosegar",
    "desasosegadas": "desasosegar",
    "desasosegado": "desasosegar",
    "desasosegados": "desasosegar",
    "desasosegamos": "desasosegar",
    "desasosegando": "desasosegar",
    "desasosegara": "desasosegar",
    "desasosegarais": "desasosegar",
    "desasosegaran": "desasosegar",
    "desasosegaras": "desasosegar",
    "desasosegare": "desasosegar",
    "desasosegareis": "desasosegar",
    "desasosegaremos": "desasosegar",
    "desasosegaren": "desasosegar",
    "desasosegares": "desasosegar",
    "desasosegaron": "desasosegar",
    "desasosegará": "desasosegar",
    "desasosegarán": "desasosegar",
    "desasosegarás": "desasosegar",
    "desasosegaré": "desasosegar",
    "desasosegaréis": "desasosegar",
    "desasosegaría": "desasosegar",
    "desasosegaríais": "desasosegar",
    "desasosegaríamos": "desasosegar",
    "desasosegarían": "desasosegar",
    "desasosegarías": "desasosegar",
    "desasosegase": "desasosegar",
    "desasosegaseis": "desasosegar",
    "desasosegasen": "desasosegar",
    "desasosegases": "desasosegar",
    "desasosegaste": "desasosegar",
    "desasosegasteis": "desasosegar",
    "desasoseguemos": "desasosegar",
    "desasosegué": "desasosegar",
    "desasoseguéis": "desasosegar",
    "desasosegábamos": "desasosegar",
    "desasosegáis": "desasosegar",
    "desasosegáramos": "desasosegar",
    "desasosegáremos": "desasosegar",
    "desasosegásemos": "desasosegar",
    "desasosegó": "desasosegar",
    "desasosiega": "desasosegar",
    "desasosiegan": "desasosegar",
    "desasosiegas": "desasosegar",
    "desasosiego": "desasosegar",
    "desasosiegos": "desasosiego",
    "desasosiegue": "desasosegar",
    "desasosieguen": "desasosegar",
    "desasosiegues": "desasosegar",
    "desastrada": "desastrado",
    "desastradas": "desastrado",
    "desastrados": "desastrado",
    "desastres": "desastre",
    "desastrosa": "desastroso",
    "desastrosas": "desastroso",
    "desastrosos": "desastroso",
    "desasí": "desasir",
    "desasía": "desasir",
    "desasíais": "desasir",
    "desasíamos": "desasir",
    "desasían": "desasir",
    "desasías": "desasir",
    "desasís": "desasir",
    "desata": "desatar",
    "desataba": "desatar",
    "desatabais": "desatar",
    "desataban": "desatar",
    "desatabas": "desatar",
    "desataca": "desatacar",
    "desatacaba": "desatacar",
    "desatacabais": "desatacar",
    "desatacaban": "desatacar",
    "desatacabas": "desatacar",
    "desatacad": "desatacar",
    "desatacada": "desatacar",
    "desatacadas": "desatacar",
    "desatacado": "desatacar",
    "desatacados": "desatacar",
    "desatacamos": "desatacar",
    "desatacan": "desatacar",
    "desatacando": "desatacar",
    "desatacara": "desatacar",
    "desatacarais": "desatacar",
    "desatacaran": "desatacar",
    "desatacaras": "desatacar",
    "desatacare": "desatacar",
    "desatacareis": "desatacar",
    "desatacaremos": "desatacar",
    "desatacaren": "desatacar",
    "desatacares": "desatacar",
    "desatacaron": "desatacar",
    "desatacará": "desatacar",
    "desatacarán": "desatacar",
    "desatacarás": "desatacar",
    "desatacaré": "desatacar",
    "desatacaréis": "desatacar",
    "desatacaría": "desatacar",
    "desatacaríais": "desatacar",
    "desatacaríamos": "desatacar",
    "desatacarían": "desatacar",
    "desatacarías": "desatacar",
    "desatacas": "desatacar",
    "desatacase": "desatacar",
    "desatacaseis": "desatacar",
    "desatacasen": "desatacar",
    "desatacases": "desatacar",
    "desatacaste": "desatacar",
    "desatacasteis": "desatacar",
    "desataco": "desatacar",
    "desatacábamos": "desatacar",
    "desatacáis": "desatacar",
    "desatacáramos": "desatacar",
    "desatacáremos": "desatacar",
    "desatacásemos": "desatacar",
    "desatacó": "desatacar",
    "desatad": "desatar",
    "desatada": "desatar",
    "desatadas": "desatar",
    "desatado": "desatar",
    "desatadora": "desatador",
    "desatadoras": "desatador",
    "desatadores": "desatador",
    "desatados": "desatar",
    "desataduras": "desatadura",
    "desatamos": "desatar",
    "desatan": "desatar",
    "desatando": "desatar",
    "desataque": "desatacar",
    "desataquemos": "desatacar",
    "desataquen": "desatacar",
    "desataques": "desatacar",
    "desataqué": "desatacar",
    "desataquéis": "desatacar",
    "desatara": "desatar",
    "desatarais": "desatar",
    "desataran": "desatar",
    "desataras": "desatar",
    "desatare": "desatar",
    "desatareis": "desatar",
    "desataremos": "desatar",
    "desataren": "desatar",
    "desatares": "desatar",
    "desataron": "desatar",
    "desatará": "desatar",
    "desatarán": "desatar",
    "desatarás": "desatar",
    "desataré": "desatar",
    "desataréis": "desatar",
    "desataría": "desatar",
    "desataríais": "desatar",
    "desataríamos": "desatar",
    "desatarían": "desatar",
    "desatarías": "desatar",
    "desatas": "desatar",
    "desatasca": "desatascar",
    "desatascaba": "desatascar",
    "desatascabais": "desatascar",
    "desatascaban": "desatascar",
    "desatascabas": "desatascar",
    "desatascad": "desatascar",
    "desatascada": "desatascar",
    "desatascadas": "desatascar",
    "desatascado": "desatascar",
    "desatascadores": "desatascador",
    "desatascados": "desatascar",
    "desatascamos": "desatascar",
    "desatascan": "desatascar",
    "desatascando": "desatascar",
    "desatascara": "desatascar",
    "desatascarais": "desatascar",
    "desatascaran": "desatascar",
    "desatascaras": "desatascar",
    "desatascare": "desatascar",
    "desatascareis": "desatascar",
    "desatascaremos": "desatascar",
    "desatascaren": "desatascar",
    "desatascares": "desatascar",
    "desatascaron": "desatascar",
    "desatascará": "desatascar",
    "desatascarán": "desatascar",
    "desatascarás": "desatascar",
    "desatascaré": "desatascar",
    "desatascaréis": "desatascar",
    "desatascaría": "desatascar",
    "desatascaríais": "desatascar",
    "desatascaríamos": "desatascar",
    "desatascarían": "desatascar",
    "desatascarías": "desatascar",
    "desatascas": "desatascar",
    "desatascase": "desatascar",
    "desatascaseis": "desatascar",
    "desatascasen": "desatascar",
    "desatascases": "desatascar",
    "desatascaste": "desatascar",
    "desatascasteis": "desatascar",
    "desatasco": "desatascar",
    "desatascábamos": "desatascar",
    "desatascáis": "desatascar",
    "desatascáramos": "desatascar",
    "desatascáremos": "desatascar",
    "desatascásemos": "desatascar",
    "desatascó": "desatascar",
    "desatase": "desatar",
    "desataseis": "desatar",
    "desatasen": "desatar",
    "desatases": "desatar",
    "desatasque": "desatascar",
    "desatasquemos": "desatascar",
    "desatasquen": "desatascar",
    "desatasques": "desatascar",
    "desatasqué": "desatascar",
    "desatasquéis": "desatascar",
    "desataste": "desatar",
    "desatasteis": "desatar",
    "desataviaba": "desataviar",
    "desataviabais": "desataviar",
    "desataviaban": "desataviar",
    "desataviabas": "desataviar",
    "desataviad": "desataviar",
    "desataviada": "desataviar",
    "desataviadas": "desataviar",
    "desataviado": "desataviar",
    "desataviados": "desataviar",
    "desataviamos": "desataviar",
    "desataviando": "desataviar",
    "desataviara": "desataviar",
    "desataviarais": "desataviar",
    "desataviaran": "desataviar",
    "desataviaras": "desataviar",
    "desataviare": "desataviar",
    "desataviareis": "desataviar",
    "desataviaremos": "desataviar",
    "desataviaren": "desataviar",
    "desataviares": "desataviar",
    "desataviaron": "desataviar",
    "desataviará": "desataviar",
    "desataviarán": "desataviar",
    "desataviarás": "desataviar",
    "desataviaré": "desataviar",
    "desataviaréis": "desataviar",
    "desataviaría": "desataviar",
    "desataviaríais": "desataviar",
    "desataviaríamos": "desataviar",
    "desataviarían": "desataviar",
    "desataviarías": "desataviar",
    "desataviase": "desataviar",
    "desataviaseis": "desataviar",
    "desataviasen": "desataviar",
    "desataviases": "desataviar",
    "desataviaste": "desataviar",
    "desataviasteis": "desataviar",
    "desataviemos": "desataviar",
    "desataviábamos": "desataviar",
    "desataviáis": "desataviar",
    "desataviáramos": "desataviar",
    "desataviáremos": "desataviar",
    "desataviásemos": "desataviar",
    "desatavié": "desataviar",
    "desataviéis": "desataviar",
    "desatavió": "desataviar",
    "desatavía": "desataviar",
    "desatavían": "desataviar",
    "desatavías": "desataviar",
    "desatavíe": "desataviar",
    "desatavíen": "desataviar",
    "desatavíes": "desataviar",
    "desatavío": "desataviar",
    "desate": "desatar",
    "desatemos": "desatar",
    "desaten": "desatar",
    "desatenciones": "desatención",
    "desatendamos": "desatender",
    "desatended": "desatender",
    "desatendemos": "desatender",
    "desatenderemos": "desatender",
    "desatenderá": "desatender",
    "desatenderán": "desatender",
    "desatenderás": "desatender",
    "desatenderé": "desatender",
    "desatenderéis": "desatender",
    "desatendería": "desatender",
    "desatenderíais": "desatender",
    "desatenderíamos": "desatender",
    "desatenderían": "desatender",
    "desatenderías": "desatender",
    "desatendida": "desatender",
    "desatendidas": "desatender",
    "desatendido": "desatender",
    "desatendidos": "desatender",
    "desatendiendo": "desatender",
    "desatendiera": "desatender",
    "desatendierais": "desatender",
    "desatendieran": "desatender",
    "desatendieras": "desatender",
    "desatendiere": "desatender",
    "desatendiereis": "desatender",
    "desatendieren": "desatender",
    "desatendieres": "desatender",
    "desatendieron": "desatender",
    "desatendiese": "desatender",
    "desatendieseis": "desatender",
    "desatendiesen": "desatender",
    "desatendieses": "desatender",
    "desatendimos": "desatender",
    "desatendiste": "desatender",
    "desatendisteis": "desatender",
    "desatendiéramos": "desatender",
    "desatendiéremos": "desatender",
    "desatendiésemos": "desatender",
    "desatendió": "desatender",
    "desatendáis": "desatender",
    "desatendéis": "desatender",
    "desatendí": "desatender",
    "desatendía": "desatender",
    "desatendíais": "desatender",
    "desatendíamos": "desatender",
    "desatendían": "desatender",
    "desatendías": "desatender",
    "desatenta": "desatento",
    "desatentaba": "desatentar",
    "desatentabais": "desatentar",
    "desatentaban": "desatentar",
    "desatentabas": "desatentar",
    "desatentad": "desatentar",
    "desatentada": "desatentar",
    "desatentadas": "desatentar",
    "desatentado": "desatentar",
    "desatentados": "desatentar",
    "desatentamos": "desatentar",
    "desatentando": "desatentar",
    "desatentara": "desatentar",
    "desatentarais": "desatentar",
    "desatentaran": "desatentar",
    "desatentaras": "desatentar",
    "desatentare": "desatentar",
    "desatentareis": "desatentar",
    "desatentaremos": "desatentar",
    "desatentaren": "desatentar",
    "desatentares": "desatentar",
    "desatentaron": "desatentar",
    "desatentará": "desatentar",
    "desatentarán": "desatentar",
    "desatentarás": "desatentar",
    "desatentaré": "desatentar",
    "desatentaréis": "desatentar",
    "desatentaría": "desatentar",
    "desatentaríais": "desatentar",
    "desatentaríamos": "desatentar",
    "desatentarían": "desatentar",
    "desatentarías": "desatentar",
    "desatentas": "desatento",
    "desatentase": "desatentar",
    "desatentaseis": "desatentar",
    "desatentasen": "desatentar",
    "desatentases": "desatentar",
    "desatentaste": "desatentar",
    "desatentasteis": "desatentar",
    "desatentemos": "desatentar",
    "desatentos": "desatento",
    "desatentábamos": "desatentar",
    "desatentáis": "desatentar",
    "desatentáramos": "desatentar",
    "desatentáremos": "desatentar",
    "desatentásemos": "desatentar",
    "desatenté": "desatentar",
    "desatentéis": "desatentar",
    "desatentó": "desatentar",
    "desaterraba": "desaterrar",
    "desaterrabais": "desaterrar",
    "desaterraban": "desaterrar",
    "desaterrabas": "desaterrar",
    "desaterrad": "desaterrar",
    "desaterrada": "desaterrar",
    "desaterradas": "desaterrar",
    "desaterrado": "desaterrar",
    "desaterrados": "desaterrar",
    "desaterramos": "desaterrar",
    "desaterrando": "desaterrar",
    "desaterrara": "desaterrar",
    "desaterrarais": "desaterrar",
    "desaterraran": "desaterrar",
    "desaterraras": "desaterrar",
    "desaterrare": "desaterrar",
    "desaterrareis": "desaterrar",
    "desaterraremos": "desaterrar",
    "desaterraren": "desaterrar",
    "desaterrares": "desaterrar",
    "desaterraron": "desaterrar",
    "desaterrará": "desaterrar",
    "desaterrarán": "desaterrar",
    "desaterrarás": "desaterrar",
    "desaterraré": "desaterrar",
    "desaterraréis": "desaterrar",
    "desaterraría": "desaterrar",
    "desaterraríais": "desaterrar",
    "desaterraríamos": "desaterrar",
    "desaterrarían": "desaterrar",
    "desaterrarías": "desaterrar",
    "desaterrase": "desaterrar",
    "desaterraseis": "desaterrar",
    "desaterrasen": "desaterrar",
    "desaterrases": "desaterrar",
    "desaterraste": "desaterrar",
    "desaterrasteis": "desaterrar",
    "desaterremos": "desaterrar",
    "desaterrábamos": "desaterrar",
    "desaterráis": "desaterrar",
    "desaterráramos": "desaterrar",
    "desaterráremos": "desaterrar",
    "desaterrásemos": "desaterrar",
    "desaterré": "desaterrar",
    "desaterréis": "desaterrar",
    "desaterró": "desaterrar",
    "desates": "desatar",
    "desatienda": "desatender",
    "desatiendan": "desatender",
    "desatiendas": "desatender",
    "desatiende": "desatender",
    "desatienden": "desatender",
    "desatiendes": "desatender",
    "desatiendo": "desatender",
    "desatienta": "desatentar",
    "desatientan": "desatentar",
    "desatientas": "desatentar",
    "desatiente": "desatentar",
    "desatienten": "desatentar",
    "desatientes": "desatentar",
    "desatiento": "desatentar",
    "desatientos": "desatiento",
    "desatierra": "desaterrar",
    "desatierran": "desaterrar",
    "desatierras": "desaterrar",
    "desatierre": "desaterrar",
    "desatierren": "desaterrar",
    "desatierres": "desaterrar",
    "desatierro": "desaterrar",
    "desatina": "desatinar",
    "desatinaba": "desatinar",
    "desatinabais": "desatinar",
    "desatinaban": "desatinar",
    "desatinabas": "desatinar",
    "desatinad": "desatinar",
    "desatinada": "desatinar",
    "desatinadas": "desatinar",
    "desatinado": "desatinar",
    "desatinados": "desatinar",
    "desatinamos": "desatinar",
    "desatinan": "desatinar",
    "desatinando": "desatinar",
    "desatinara": "desatinar",
    "desatinarais": "desatinar",
    "desatinaran": "desatinar",
    "desatinaras": "desatinar",
    "desatinare": "desatinar",
    "desatinareis": "desatinar",
    "desatinaremos": "desatinar",
    "desatinaren": "desatinar",
    "desatinares": "desatinar",
    "desatinaron": "desatinar",
    "desatinará": "desatinar",
    "desatinarán": "desatinar",
    "desatinarás": "desatinar",
    "desatinaré": "desatinar",
    "desatinaréis": "desatinar",
    "desatinaría": "desatinar",
    "desatinaríais": "desatinar",
    "desatinaríamos": "desatinar",
    "desatinarían": "desatinar",
    "desatinarías": "desatinar",
    "desatinas": "desatinar",
    "desatinase": "desatinar",
    "desatinaseis": "desatinar",
    "desatinasen": "desatinar",
    "desatinases": "desatinar",
    "desatinaste": "desatinar",
    "desatinasteis": "desatinar",
    "desatine": "desatinar",
    "desatinemos": "desatinar",
    "desatinen": "desatinar",
    "desatines": "desatinar",
    "desatino": "desatinar",
    "desatinos": "desatino",
    "desatinábamos": "desatinar",
    "desatináis": "desatinar",
    "desatináramos": "desatinar",
    "desatináremos": "desatinar",
    "desatinásemos": "desatinar",
    "desatiné": "desatinar",
    "desatinéis": "desatinar",
    "desatinó": "desatinar",
    "desato": "desatar",
    "desatolla": "desatollar",
    "desatollaba": "desatollar",
    "desatollabais": "desatollar",
    "desatollaban": "desatollar",
    "desatollabas": "desatollar",
    "desatollad": "desatollar",
    "desatollada": "desatollar",
    "desatolladas": "desatollar",
    "desatollado": "desatollar",
    "desatollados": "desatollar",
    "desatollamos": "desatollar",
    "desatollan": "desatollar",
    "desatollando": "desatollar",
    "desatollara": "desatollar",
    "desatollarais": "desatollar",
    "desatollaran": "desatollar",
    "desatollaras": "desatollar",
    "desatollare": "desatollar",
    "desatollareis": "desatollar",
    "desatollaremos": "desatollar",
    "desatollaren": "desatollar",
    "desatollares": "desatollar",
    "desatollaron": "desatollar",
    "desatollará": "desatollar",
    "desatollarán": "desatollar",
    "desatollarás": "desatollar",
    "desatollaré": "desatollar",
    "desatollaréis": "desatollar",
    "desatollaría": "desatollar",
    "desatollaríais": "desatollar",
    "desatollaríamos": "desatollar",
    "desatollarían": "desatollar",
    "desatollarías": "desatollar",
    "desatollas": "desatollar",
    "desatollase": "desatollar",
    "desatollaseis": "desatollar",
    "desatollasen": "desatollar",
    "desatollases": "desatollar",
    "desatollaste": "desatollar",
    "desatollasteis": "desatollar",
    "desatolle": "desatollar",
    "desatollemos": "desatollar",
    "desatollen": "desatollar",
    "desatolles": "desatollar",
    "desatollo": "desatollar",
    "desatollábamos": "desatollar",
    "desatolláis": "desatollar",
    "desatolláramos": "desatollar",
    "desatolláremos": "desatollar",
    "desatollásemos": "desatollar",
    "desatollé": "desatollar",
    "desatolléis": "desatollar",
    "desatolló": "desatollar",
    "desatolondra": "desatolondrar",
    "desatolondraba": "desatolondrar",
    "desatolondrabais": "desatolondrar",
    "desatolondraban": "desatolondrar",
    "desatolondrabas": "desatolondrar",
    "desatolondrad": "desatolondrar",
    "desatolondrada": "desatolondrar",
    "desatolondradas": "desatolondrar",
    "desatolondrado": "desatolondrar",
    "desatolondrados": "desatolondrar",
    "desatolondramos": "desatolondrar",
    "desatolondran": "desatolondrar",
    "desatolondrando": "desatolondrar",
    "desatolondrara": "desatolondrar",
    "desatolondrarais": "desatolondrar",
    "desatolondraran": "desatolondrar",
    "desatolondraras": "desatolondrar",
    "desatolondrare": "desatolondrar",
    "desatolondrareis": "desatolondrar",
    "desatolondraremos": "desatolondrar",
    "desatolondraren": "desatolondrar",
    "desatolondrares": "desatolondrar",
    "desatolondraron": "desatolondrar",
    "desatolondrará": "desatolondrar",
    "desatolondrarán": "desatolondrar",
    "desatolondrarás": "desatolondrar",
    "desatolondraré": "desatolondrar",
    "desatolondraréis": "desatolondrar",
    "desatolondraría": "desatolondrar",
    "desatolondraríais": "desatolondrar",
    "desatolondraríamos": "desatolondrar",
    "desatolondrarían": "desatolondrar",
    "desatolondrarías": "desatolondrar",
    "desatolondras": "desatolondrar",
    "desatolondrase": "desatolondrar",
    "desatolondraseis": "desatolondrar",
    "desatolondrasen": "desatolondrar",
    "desatolondrases": "desatolondrar",
    "desatolondraste": "desatolondrar",
    "desatolondrasteis": "desatolondrar",
    "desatolondre": "desatolondrar",
    "desatolondremos": "desatolondrar",
    "desatolondren": "desatolondrar",
    "desatolondres": "desatolondrar",
    "desatolondro": "desatolondrar",
    "desatolondrábamos": "desatolondrar",
    "desatolondráis": "desatolondrar",
    "desatolondráramos": "desatolondrar",
    "desatolondráremos": "desatolondrar",
    "desatolondrásemos": "desatolondrar",
    "desatolondré": "desatolondrar",
    "desatolondréis": "desatolondrar",
    "desatolondró": "desatolondrar",
    "desatonta": "desatontar",
    "desatontaba": "desatontar",
    "desatontabais": "desatontar",
    "desatontaban": "desatontar",
    "desatontabas": "desatontar",
    "desatontad": "desatontar",
    "desatontada": "desatontar",
    "desatontadas": "desatontar",
    "desatontado": "desatontar",
    "desatontados": "desatontar",
    "desatontamos": "desatontar",
    "desatontan": "desatontar",
    "desatontando": "desatontar",
    "desatontara": "desatontar",
    "desatontarais": "desatontar",
    "desatontaran": "desatontar",
    "desatontaras": "desatontar",
    "desatontare": "desatontar",
    "desatontareis": "desatontar",
    "desatontaremos": "desatontar",
    "desatontaren": "desatontar",
    "desatontares": "desatontar",
    "desatontaron": "desatontar",
    "desatontará": "desatontar",
    "desatontarán": "desatontar",
    "desatontarás": "desatontar",
    "desatontaré": "desatontar",
    "desatontaréis": "desatontar",
    "desatontaría": "desatontar",
    "desatontaríais": "desatontar",
    "desatontaríamos": "desatontar",
    "desatontarían": "desatontar",
    "desatontarías": "desatontar",
    "desatontas": "desatontar",
    "desatontase": "desatontar",
    "desatontaseis": "desatontar",
    "desatontasen": "desatontar",
    "desatontases": "desatontar",
    "desatontaste": "desatontar",
    "desatontasteis": "desatontar",
    "desatonte": "desatontar",
    "desatontemos": "desatontar",
    "desatonten": "desatontar",
    "desatontes": "desatontar",
    "desatonto": "desatontar",
    "desatontábamos": "desatontar",
    "desatontáis": "desatontar",
    "desatontáramos": "desatontar",
    "desatontáremos": "desatontar",
    "desatontásemos": "desatontar",
    "desatonté": "desatontar",
    "desatontéis": "desatontar",
    "desatontó": "desatontar",
    "desatora": "desatorar",
    "desatoraba": "desatorar",
    "desatorabais": "desatorar",
    "desatoraban": "desatorar",
    "desatorabas": "desatorar",
    "desatorad": "desatorar",
    "desatorada": "desatorar",
    "desatoradas": "desatorar",
    "desatorado": "desatorar",
    "desatorados": "desatorar",
    "desatoramos": "desatorar",
    "desatoran": "desatorar",
    "desatorando": "desatorar",
    "desatorara": "desatorar",
    "desatorarais": "desatorar",
    "desatoraran": "desatorar",
    "desatoraras": "desatorar",
    "desatorare": "desatorar",
    "desatorareis": "desatorar",
    "desatoraremos": "desatorar",
    "desatoraren": "desatorar",
    "desatorares": "desatorar",
    "desatoraron": "desatorar",
    "desatorará": "desatorar",
    "desatorarán": "desatorar",
    "desatorarás": "desatorar",
    "desatoraré": "desatorar",
    "desatoraréis": "desatorar",
    "desatoraría": "desatorar",
    "desatoraríais": "desatorar",
    "desatoraríamos": "desatorar",
    "desatorarían": "desatorar",
    "desatorarías": "desatorar",
    "desatoras": "desatorar",
    "desatorase": "desatorar",
    "desatoraseis": "desatorar",
    "desatorasen": "desatorar",
    "desatorases": "desatorar",
    "desatoraste": "desatorar",
    "desatorasteis": "desatorar",
    "desatore": "desatorar",
    "desatoremos": "desatorar",
    "desatoren": "desatorar",
    "desatores": "desatorar",
    "desatornilla": "desatornillar",
    "desatornillaba": "desatornillar",
    "desatornillabais": "desatornillar",
    "desatornillaban": "desatornillar",
    "desatornillabas": "desatornillar",
    "desatornillad": "desatornillar",
    "desatornillada": "desatornillar",
    "desatornilladas": "desatornillar",
    "desatornillado": "desatornillar",
    "desatornilladores": "desatornillador",
    "desatornillados": "desatornillar",
    "desatornillamos": "desatornillar",
    "desatornillan": "desatornillar",
    "desatornillando": "desatornillar",
    "desatornillara": "desatornillar",
    "desatornillarais": "desatornillar",
    "desatornillaran": "desatornillar",
    "desatornillaras": "desatornillar",
    "desatornillare": "desatornillar",
    "desatornillareis": "desatornillar",
    "desatornillaremos": "desatornillar",
    "desatornillaren": "desatornillar",
    "desatornillares": "desatornillar",
    "desatornillaron": "desatornillar",
    "desatornillará": "desatornillar",
    "desatornillarán": "desatornillar",
    "desatornillarás": "desatornillar",
    "desatornillaré": "desatornillar",
    "desatornillaréis": "desatornillar",
    "desatornillaría": "desatornillar",
    "desatornillaríais": "desatornillar",
    "desatornillaríamos": "desatornillar",
    "desatornillarían": "desatornillar",
    "desatornillarías": "desatornillar",
    "desatornillas": "desatornillar",
    "desatornillase": "desatornillar",
    "desatornillaseis": "desatornillar",
    "desatornillasen": "desatornillar",
    "desatornillases": "desatornillar",
    "desatornillaste": "desatornillar",
    "desatornillasteis": "desatornillar",
    "desatornille": "desatornillar",
    "desatornillemos": "desatornillar",
    "desatornillen": "desatornillar",
    "desatornilles": "desatornillar",
    "desatornillo": "desatornillar",
    "desatornillábamos": "desatornillar",
    "desatornilláis": "desatornillar",
    "desatornilláramos": "desatornillar",
    "desatornilláremos": "desatornillar",
    "desatornillásemos": "desatornillar",
    "desatornillé": "desatornillar",
    "desatornilléis": "desatornillar",
    "desatornilló": "desatornillar",
    "desatoro": "desatorar",
    "desatorábamos": "desatorar",
    "desatoráis": "desatorar",
    "desatoráramos": "desatorar",
    "desatoráremos": "desatorar",
    "desatorásemos": "desatorar",
    "desatoré": "desatorar",
    "desatoréis": "desatorar",
    "desatoró": "desatorar",
    "desatraca": "desatracar",
    "desatracaba": "desatracar",
    "desatracabais": "desatracar",
    "desatracaban": "desatracar",
    "desatracabas": "desatracar",
    "desatracad": "desatracar",
    "desatracada": "desatracar",
    "desatracadas": "desatracar",
    "desatracado": "desatracar",
    "desatracados": "desatracar",
    "desatracamos": "desatracar",
    "desatracan": "desatracar",
    "desatracando": "desatracar",
    "desatracara": "desatracar",
    "desatracarais": "desatracar",
    "desatracaran": "desatracar",
    "desatracaras": "desatracar",
    "desatracare": "desatracar",
    "desatracareis": "desatracar",
    "desatracaremos": "desatracar",
    "desatracaren": "desatracar",
    "desatracares": "desatracar",
    "desatracaron": "desatracar",
    "desatracará": "desatracar",
    "desatracarán": "desatracar",
    "desatracarás": "desatracar",
    "desatracaré": "desatracar",
    "desatracaréis": "desatracar",
    "desatracaría": "desatracar",
    "desatracaríais": "desatracar",
    "desatracaríamos": "desatracar",
    "desatracarían": "desatracar",
    "desatracarías": "desatracar",
    "desatracas": "desatracar",
    "desatracase": "desatracar",
    "desatracaseis": "desatracar",
    "desatracasen": "desatracar",
    "desatracases": "desatracar",
    "desatracaste": "desatracar",
    "desatracasteis": "desatracar",
    "desatraco": "desatracar",
    "desatracábamos": "desatracar",
    "desatracáis": "desatracar",
    "desatracáramos": "desatracar",
    "desatracáremos": "desatracar",
    "desatracásemos": "desatracar",
    "desatracó": "desatracar",
    "desatraganta": "desatragantar",
    "desatragantaba": "desatragantar",
    "desatragantabais": "desatragantar",
    "desatragantaban": "desatragantar",
    "desatragantabas": "desatragantar",
    "desatragantad": "desatragantar",
    "desatragantada": "desatragantar",
    "desatragantadas": "desatragantar",
    "desatragantado": "desatragantar",
    "desatragantados": "desatragantar",
    "desatragantamos": "desatragantar",
    "desatragantan": "desatragantar",
    "desatragantando": "desatragantar",
    "desatragantara": "desatragantar",
    "desatragantarais": "desatragantar",
    "desatragantaran": "desatragantar",
    "desatragantaras": "desatragantar",
    "desatragantare": "desatragantar",
    "desatragantareis": "desatragantar",
    "desatragantaremos": "desatragantar",
    "desatragantaren": "desatragantar",
    "desatragantares": "desatragantar",
    "desatragantaron": "desatragantar",
    "desatragantará": "desatragantar",
    "desatragantarán": "desatragantar",
    "desatragantarás": "desatragantar",
    "desatragantaré": "desatragantar",
    "desatragantaréis": "desatragantar",
    "desatragantaría": "desatragantar",
    "desatragantaríais": "desatragantar",
    "desatragantaríamos": "desatragantar",
    "desatragantarían": "desatragantar",
    "desatragantarías": "desatragantar",
    "desatragantas": "desatragantar",
    "desatragantase": "desatragantar",
    "desatragantaseis": "desatragantar",
    "desatragantasen": "desatragantar",
    "desatragantases": "desatragantar",
    "desatragantaste": "desatragantar",
    "desatragantasteis": "desatragantar",
    "desatragante": "desatragantar",
    "desatragantemos": "desatragantar",
    "desatraganten": "desatragantar",
    "desatragantes": "desatragantar",
    "desatraganto": "desatragantar",
    "desatragantábamos": "desatragantar",
    "desatragantáis": "desatragantar",
    "desatragantáramos": "desatragantar",
    "desatragantáremos": "desatragantar",
    "desatragantásemos": "desatragantar",
    "desatraganté": "desatragantar",
    "desatragantéis": "desatragantar",
    "desatragantó": "desatragantar",
    "desatrailla": "desatraillar",
    "desatraillaba": "desatraillar",
    "desatraillabais": "desatraillar",
    "desatraillaban": "desatraillar",
    "desatraillabas": "desatraillar",
    "desatraillad": "desatraillar",
    "desatraillada": "desatraillar",
    "desatrailladas": "desatraillar",
    "desatraillado": "desatraillar",
    "desatraillados": "desatraillar",
    "desatraillamos": "desatraillar",
    "desatraillan": "desatraillar",
    "desatraillando": "desatraillar",
    "desatraillara": "desatraillar",
    "desatraillarais": "desatraillar",
    "desatraillaran": "desatraillar",
    "desatraillaras": "desatraillar",
    "desatraillare": "desatraillar",
    "desatraillareis": "desatraillar",
    "desatraillaremos": "desatraillar",
    "desatraillaren": "desatraillar",
    "desatraillares": "desatraillar",
    "desatraillaron": "desatraillar",
    "desatraillará": "desatraillar",
    "desatraillarán": "desatraillar",
    "desatraillarás": "desatraillar",
    "desatraillaré": "desatraillar",
    "desatraillaréis": "desatraillar",
    "desatraillaría": "desatraillar",
    "desatraillaríais": "desatraillar",
    "desatraillaríamos": "desatraillar",
    "desatraillarían": "desatraillar",
    "desatraillarías": "desatraillar",
    "desatraillas": "desatraillar",
    "desatraillase": "desatraillar",
    "desatraillaseis": "desatraillar",
    "desatraillasen": "desatraillar",
    "desatraillases": "desatraillar",
    "desatraillaste": "desatraillar",
    "desatraillasteis": "desatraillar",
    "desatraille": "desatraillar",
    "desatraillemos": "desatraillar",
    "desatraillen": "desatraillar",
    "desatrailles": "desatraillar",
    "desatraillo": "desatraillar",
    "desatraillábamos": "desatraillar",
    "desatrailláis": "desatraillar",
    "desatrailláramos": "desatraillar",
    "desatrailláremos": "desatraillar",
    "desatraillásemos": "desatraillar",
    "desatraillé": "desatraillar",
    "desatrailléis": "desatraillar",
    "desatrailló": "desatraillar",
    "desatranca": "desatrancar",
    "desatrancaba": "desatrancar",
    "desatrancabais": "desatrancar",
    "desatrancaban": "desatrancar",
    "desatrancabas": "desatrancar",
    "desatrancad": "desatrancar",
    "desatrancada": "desatrancar",
    "desatrancadas": "desatrancar",
    "desatrancado": "desatrancar",
    "desatrancados": "desatrancar",
    "desatrancamos": "desatrancar",
    "desatrancan": "desatrancar",
    "desatrancando": "desatrancar",
    "desatrancara": "desatrancar",
    "desatrancarais": "desatrancar",
    "desatrancaran": "desatrancar",
    "desatrancaras": "desatrancar",
    "desatrancare": "desatrancar",
    "desatrancareis": "desatrancar",
    "desatrancaremos": "desatrancar",
    "desatrancaren": "desatrancar",
    "desatrancares": "desatrancar",
    "desatrancaron": "desatrancar",
    "desatrancará": "desatrancar",
    "desatrancarán": "desatrancar",
    "desatrancarás": "desatrancar",
    "desatrancaré": "desatrancar",
    "desatrancaréis": "desatrancar",
    "desatrancaría": "desatrancar",
    "desatrancaríais": "desatrancar",
    "desatrancaríamos": "desatrancar",
    "desatrancarían": "desatrancar",
    "desatrancarías": "desatrancar",
    "desatrancas": "desatrancar",
    "desatrancase": "desatrancar",
    "desatrancaseis": "desatrancar",
    "desatrancasen": "desatrancar",
    "desatrancases": "desatrancar",
    "desatrancaste": "desatrancar",
    "desatrancasteis": "desatrancar",
    "desatranco": "desatrancar",
    "desatrancábamos": "desatrancar",
    "desatrancáis": "desatrancar",
    "desatrancáramos": "desatrancar",
    "desatrancáremos": "desatrancar",
    "desatrancásemos": "desatrancar",
    "desatrancó": "desatrancar",
    "desatranque": "desatrancar",
    "desatranquemos": "desatrancar",
    "desatranquen": "desatrancar",
    "desatranques": "desatrancar",
    "desatranqué": "desatrancar",
    "desatranquéis": "desatrancar",
    "desatraque": "desatracar",
    "desatraquemos": "desatracar",
    "desatraquen": "desatracar",
    "desatraques": "desatracar",
    "desatraqué": "desatracar",
    "desatraquéis": "desatracar",
    "desatufa": "desatufar",
    "desatufaba": "desatufar",
    "desatufabais": "desatufar",
    "desatufaban": "desatufar",
    "desatufabas": "desatufar",
    "desatufad": "desatufar",
    "desatufada": "desatufar",
    "desatufadas": "desatufar",
    "desatufado": "desatufar",
    "desatufados": "desatufar",
    "desatufamos": "desatufar",
    "desatufan": "desatufar",
    "desatufando": "desatufar",
    "desatufara": "desatufar",
    "desatufarais": "desatufar",
    "desatufaran": "desatufar",
    "desatufaras": "desatufar",
    "desatufare": "desatufar",
    "desatufareis": "desatufar",
    "desatufaremos": "desatufar",
    "desatufaren": "desatufar",
    "desatufares": "desatufar",
    "desatufaron": "desatufar",
    "desatufará": "desatufar",
    "desatufarán": "desatufar",
    "desatufarás": "desatufar",
    "desatufaré": "desatufar",
    "desatufaréis": "desatufar",
    "desatufaría": "desatufar",
    "desatufaríais": "desatufar",
    "desatufaríamos": "desatufar",
    "desatufarían": "desatufar",
    "desatufarías": "desatufar",
    "desatufas": "desatufar",
    "desatufase": "desatufar",
    "desatufaseis": "desatufar",
    "desatufasen": "desatufar",
    "desatufases": "desatufar",
    "desatufaste": "desatufar",
    "desatufasteis": "desatufar",
    "desatufe": "desatufar",
    "desatufemos": "desatufar",
    "desatufen": "desatufar",
    "desatufes": "desatufar",
    "desatufo": "desatufar",
    "desatufábamos": "desatufar",
    "desatufáis": "desatufar",
    "desatufáramos": "desatufar",
    "desatufáremos": "desatufar",
    "desatufásemos": "desatufar",
    "desatufé": "desatufar",
    "desatuféis": "desatufar",
    "desatufó": "desatufar",
    "desaturaciones": "desaturación",
    "desaturda": "desaturdir",
    "desaturdamos": "desaturdir",
    "desaturdan": "desaturdir",
    "desaturdas": "desaturdir",
    "desaturde": "desaturdir",
    "desaturden": "desaturdir",
    "desaturdes": "desaturdir",
    "desaturdid": "desaturdir",
    "desaturdida": "desaturdir",
    "desaturdidas": "desaturdir",
    "desaturdido": "desaturdir",
    "desaturdidos": "desaturdir",
    "desaturdiendo": "desaturdir",
    "desaturdiera": "desaturdir",
    "desaturdierais": "desaturdir",
    "desaturdieran": "desaturdir",
    "desaturdieras": "desaturdir",
    "desaturdiere": "desaturdir",
    "desaturdiereis": "desaturdir",
    "desaturdieren": "desaturdir",
    "desaturdieres": "desaturdir",
    "desaturdieron": "desaturdir",
    "desaturdiese": "desaturdir",
    "desaturdieseis": "desaturdir",
    "desaturdiesen": "desaturdir",
    "desaturdieses": "desaturdir",
    "desaturdimientos": "desaturdimiento",
    "desaturdimos": "desaturdir",
    "desaturdiremos": "desaturdir",
    "desaturdirá": "desaturdir",
    "desaturdirán": "desaturdir",
    "desaturdirás": "desaturdir",
    "desaturdiré": "desaturdir",
    "desaturdiréis": "desaturdir",
    "desaturdiría": "desaturdir",
    "desaturdiríais": "desaturdir",
    "desaturdiríamos": "desaturdir",
    "desaturdirían": "desaturdir",
    "desaturdirías": "desaturdir",
    "desaturdiste": "desaturdir",
    "desaturdisteis": "desaturdir",
    "desaturdiéramos": "desaturdir",
    "desaturdiéremos": "desaturdir",
    "desaturdiésemos": "desaturdir",
    "desaturdió": "desaturdir",
    "desaturdo": "desaturdir",
    "desaturdáis": "desaturdir",
    "desaturdí": "desaturdir",
    "desaturdía": "desaturdir",
    "desaturdíais": "desaturdir",
    "desaturdíamos": "desaturdir",
    "desaturdían": "desaturdir",
    "desaturdías": "desaturdir",
    "desaturdís": "desaturdir",
    "desatábamos": "desatar",
    "desatáis": "desatar",
    "desatáramos": "desatar",
    "desatáremos": "desatar",
    "desatásemos": "desatar",
    "desaté": "desatar",
    "desatéis": "desatar",
    "desató": "desatar",
    "desautorice": "desautorizar",
    "desautoricemos": "desautorizar",
    "desautoricen": "desautorizar",
    "desautorices": "desautorizar",
    "desautoricé": "desautorizar",
    "desautoricéis": "desautorizar",
    "desautoriza": "desautorizar",
    "desautorizaba": "desautorizar",
    "desautorizabais": "desautorizar",
    "desautorizaban": "desautorizar",
    "desautorizabas": "desautorizar",
    "desautorizaciones": "desautorización",
    "desautorizad": "desautorizar",
    "desautorizada": "desautorizar",
    "desautorizadas": "desautorizar",
    "desautorizado": "desautorizar",
    "desautorizados": "desautorizar",
    "desautorizamos": "desautorizar",
    "desautorizan": "desautorizar",
    "desautorizando": "desautorizar",
    "desautorizara": "desautorizar",
    "desautorizarais": "desautorizar",
    "desautorizaran": "desautorizar",
    "desautorizaras": "desautorizar",
    "desautorizare": "desautorizar",
    "desautorizareis": "desautorizar",
    "desautorizaremos": "desautorizar",
    "desautorizaren": "desautorizar",
    "desautorizares": "desautorizar",
    "desautorizaron": "desautorizar",
    "desautorizará": "desautorizar",
    "desautorizarán": "desautorizar",
    "desautorizarás": "desautorizar",
    "desautorizaré": "desautorizar",
    "desautorizaréis": "desautorizar",
    "desautorizaría": "desautorizar",
    "desautorizaríais": "desautorizar",
    "desautorizaríamos": "desautorizar",
    "desautorizarían": "desautorizar",
    "desautorizarías": "desautorizar",
    "desautorizas": "desautorizar",
    "desautorizase": "desautorizar",
    "desautorizaseis": "desautorizar",
    "desautorizasen": "desautorizar",
    "desautorizases": "desautorizar",
    "desautorizaste": "desautorizar",
    "desautorizasteis": "desautorizar",
    "desautorizo": "desautorizar",
    "desautorizábamos": "desautorizar",
    "desautorizáis": "desautorizar",
    "desautorizáramos": "desautorizar",
    "desautorizáremos": "desautorizar",
    "desautorizásemos": "desautorizar",
    "desautorizó": "desautorizar",
    "desaven": "desavenir",
    "desavendremos": "desavenir",
    "desavendrá": "desavenir",
    "desavendrán": "desavenir",
    "desavendrás": "desavenir",
    "desavendré": "desavenir",
    "desavendréis": "desavenir",
    "desavendría": "desavenir",
    "desavendríais": "desavenir",
    "desavendríamos": "desavenir",
    "desavendrían": "desavenir",
    "desavendrías": "desavenir",
    "desavenencias": "desavenencia",
    "desavenga": "desavenir",
    "desavengamos": "desavenir",
    "desavengan": "desavenir",
    "desavengas": "desavenir",
    "desavengo": "desavenir",
    "desavengáis": "desavenir",
    "desavenid": "desavenir",
    "desavenida": "desavenir",
    "desavenidas": "desavenir",
    "desavenido": "desavenir",
    "desavenidos": "desavenir",
    "desavenimientos": "desavenimiento",
    "desavenimos": "desavenir",
    "desaventajada": "desaventajado",
    "desaventajadas": "desaventajado",
    "desaventajados": "desaventajado",
    "desavenía": "desavenir",
    "desaveníais": "desavenir",
    "desaveníamos": "desavenir",
    "desavenían": "desavenir",
    "desavenías": "desavenir",
    "desavenís": "desavenir",
    "desaviaba": "desaviar",
    "desaviabais": "desaviar",
    "desaviaban": "desaviar",
    "desaviabas": "desaviar",
    "desaviad": "desaviar",
    "desaviada": "desaviar",
    "desaviadas": "desaviar",
    "desaviado": "desaviar",
    "desaviados": "desaviar",
    "desaviamos": "desaviar",
    "desaviando": "desaviar",
    "desaviara": "desaviar",
    "desaviarais": "desaviar",
    "desaviaran": "desaviar",
    "desaviaras": "desaviar",
    "desaviare": "desaviar",
    "desaviareis": "desaviar",
    "desaviaremos": "desaviar",
    "desaviaren": "desaviar",
    "desaviares": "desaviar",
    "desaviaron": "desaviar",
    "desaviará": "desaviar",
    "desaviarán": "desaviar",
    "desaviarás": "desaviar",
    "desaviaré": "desaviar",
    "desaviaréis": "desaviar",
    "desaviaría": "desaviar",
    "desaviaríais": "desaviar",
    "desaviaríamos": "desaviar",
    "desaviarían": "desaviar",
    "desaviarías": "desaviar",
    "desaviase": "desaviar",
    "desaviaseis": "desaviar",
    "desaviasen": "desaviar",
    "desaviases": "desaviar",
    "desaviaste": "desaviar",
    "desaviasteis": "desaviar",
    "desaviemos": "desaviar",
    "desaviene": "desavenir",
    "desavienen": "desavenir",
    "desavienes": "desavenir",
    "desavine": "desavenir",
    "desaviniendo": "desavenir",
    "desaviniera": "desavenir",
    "desavinierais": "desavenir",
    "desavinieran": "desavenir",
    "desavinieras": "desavenir",
    "desaviniere": "desavenir",
    "desaviniereis": "desavenir",
    "desavinieren": "desavenir",
    "desavinieres": "desavenir",
    "desavinieron": "desavenir",
    "desaviniese": "desavenir",
    "desavinieseis": "desavenir",
    "desaviniesen": "desavenir",
    "desavinieses": "desavenir",
    "desavinimos": "desavenir",
    "desaviniste": "desavenir",
    "desavinisteis": "desavenir",
    "desaviniéramos": "desavenir",
    "desaviniéremos": "desavenir",
    "desaviniésemos": "desavenir",
    "desavino": "desavenir",
    "desavisada": "desavisado",
    "desavisadas": "desavisado",
    "desavisados": "desavisado",
    "desaviábamos": "desaviar",
    "desaviáis": "desaviar",
    "desaviáramos": "desaviar",
    "desaviáremos": "desaviar",
    "desaviásemos": "desaviar",
    "desavié": "desaviar",
    "desaviéis": "desaviar",
    "desavió": "desaviar",
    "desavía": "desaviar",
    "desavían": "desaviar",
    "desavías": "desaviar",
    "desavíe": "desaviar",
    "desavíen": "desaviar",
    "desavíes": "desaviar",
    "desavío": "desaviar",
    "desavíos": "desavío",
    "desayuna": "desayunar",
    "desayunaba": "desayunar",
    "desayunabais": "desayunar",
    "desayunaban": "desayunar",
    "desayunabas": "desayunar",
    "desayunad": "desayunar",
    "desayunada": "desayunar",
    "desayunadas": "desayunar",
    "desayunado": "desayunar",
    "desayunados": "desayunar",
    "desayunamos": "desayunar",
    "desayunan": "desayunar",
    "desayunando": "desayunar",
    "desayunara": "desayunar",
    "desayunarais": "desayunar",
    "desayunaran": "desayunar",
    "desayunaras": "desayunar",
    "desayunare": "desayunar",
    "desayunareis": "desayunar",
    "desayunaremos": "desayunar",
    "desayunaren": "desayunar",
    "desayunares": "desayunar",
    "desayunaron": "desayunar",
    "desayunará": "desayunar",
    "desayunarán": "desayunar",
    "desayunarás": "desayunar",
    "desayunaré": "desayunar",
    "desayunaréis": "desayunar",
    "desayunaría": "desayunar",
    "desayunaríais": "desayunar",
    "desayunaríamos": "desayunar",
    "desayunarían": "desayunar",
    "desayunarías": "desayunar",
    "desayunas": "desayunar",
    "desayunase": "desayunar",
    "desayunaseis": "desayunar",
    "desayunasen": "desayunar",
    "desayunases": "desayunar",
    "desayunaste": "desayunar",
    "desayunasteis": "desayunar",
    "desayune": "desayunar",
    "desayunemos": "desayunar",
    "desayunen": "desayunar",
    "desayunes": "desayunar",
    "desayuno": "desayunar",
    "desayunos": "desayuno",
    "desayunábamos": "desayunar",
    "desayunáis": "desayunar",
    "desayunáramos": "desayunar",
    "desayunáremos": "desayunar",
    "desayunásemos": "desayunar",
    "desayuné": "desayunar",
    "desayunéis": "desayunar",
    "desayunó": "desayunar",
    "desazoga": "desazogar",
    "desazogaba": "desazogar",
    "desazogabais": "desazogar",
    "desazogaban": "desazogar",
    "desazogabas": "desazogar",
    "desazogad": "desazogar",
    "desazogada": "desazogar",
    "desazogadas": "desazogar",
    "desazogado": "desazogar",
    "desazogados": "desazogar",
    "desazogamos": "desazogar",
    "desazogan": "desazogar",
    "desazogando": "desazogar",
    "desazogara": "desazogar",
    "desazogarais": "desazogar",
    "desazogaran": "desazogar",
    "desazogaras": "desazogar",
    "desazogare": "desazogar",
    "desazogareis": "desazogar",
    "desazogaremos": "desazogar",
    "desazogaren": "desazogar",
    "desazogares": "desazogar",
    "desazogaron": "desazogar",
    "desazogará": "desazogar",
    "desazogarán": "desazogar",
    "desazogarás": "desazogar",
    "desazogaré": "desazogar",
    "desazogaréis": "desazogar",
    "desazogaría": "desazogar",
    "desazogaríais": "desazogar",
    "desazogaríamos": "desazogar",
    "desazogarían": "desazogar",
    "desazogarías": "desazogar",
    "desazogas": "desazogar",
    "desazogase": "desazogar",
    "desazogaseis": "desazogar",
    "desazogasen": "desazogar",
    "desazogases": "desazogar",
    "desazogaste": "desazogar",
    "desazogasteis": "desazogar",
    "desazogo": "desazogar",
    "desazogue": "desazogar",
    "desazoguemos": "desazogar",
    "desazoguen": "desazogar",
    "desazogues": "desazogar",
    "desazogué": "desazogar",
    "desazoguéis": "desazogar",
    "desazogábamos": "desazogar",
    "desazogáis": "desazogar",
    "desazogáramos": "desazogar",
    "desazogáremos": "desazogar",
    "desazogásemos": "desazogar",
    "desazogó": "desazogar",
    "desazona": "desazonar",
    "desazonaba": "desazonar",
    "desazonabais": "desazonar",
    "desazonaban": "desazonar",
    "desazonabas": "desazonar",
    "desazonad": "desazonar",
    "desazonada": "desazonar",
    "desazonadas": "desazonar",
    "desazonado": "desazonar",
    "desazonados": "desazonar",
    "desazonamos": "desazonar",
    "desazonan": "desazonar",
    "desazonando": "desazonar",
    "desazonara": "desazonar",
    "desazonarais": "desazonar",
    "desazonaran": "desazonar",
    "desazonaras": "desazonar",
    "desazonare": "desazonar",
    "desazonareis": "desazonar",
    "desazonaremos": "desazonar",
    "desazonaren": "desazonar",
    "desazonares": "desazonar",
    "desazonaron": "desazonar",
    "desazonará": "desazonar",
    "desazonarán": "desazonar",
    "desazonarás": "desazonar",
    "desazonaré": "desazonar",
    "desazonaréis": "desazonar",
    "desazonaría": "desazonar",
    "desazonaríais": "desazonar",
    "desazonaríamos": "desazonar",
    "desazonarían": "desazonar",
    "desazonarías": "desazonar",
    "desazonas": "desazonar",
    "desazonase": "desazonar",
    "desazonaseis": "desazonar",
    "desazonasen": "desazonar",
    "desazonases": "desazonar",
    "desazonaste": "desazonar",
    "desazonasteis": "desazonar",
    "desazone": "desazonar",
    "desazonemos": "desazonar",
    "desazonen": "desazonar",
    "desazones": "desazonar",
    "desazono": "desazonar",
    "desazonos": "desazono",
    "desazonábamos": "desazonar",
    "desazonáis": "desazonar",
    "desazonáramos": "desazonar",
    "desazonáremos": "desazonar",
    "desazonásemos": "desazonar",
    "desazoné": "desazonar",
    "desazonéis": "desazonar",
    "desazonó": "desazonar",
    "desaína": "desainar",
    "desaínan": "desainar",
    "desaínas": "desainar",
    "desaíne": "desainar",
    "desaínen": "desainar",
    "desaínes": "desainar",
    "desaíno": "desainar",
    "desbaba": "desbabar",
    "desbababa": "desbabar",
    "desbababais": "desbabar",
    "desbababan": "desbabar",
    "desbababas": "desbabar",
    "desbabad": "desbabar",
    "desbabada": "desbabar",
    "desbabadas": "desbabar",
    "desbabado": "desbabar",
    "desbabados": "desbabar",
    "desbabamos": "desbabar",
    "desbaban": "desbabar",
    "desbabando": "desbabar",
    "desbabara": "desbabar",
    "desbabarais": "desbabar",
    "desbabaran": "desbabar",
    "desbabaras": "desbabar",
    "desbabare": "desbabar",
    "desbabareis": "desbabar",
    "desbabaremos": "desbabar",
    "desbabaren": "desbabar",
    "desbabares": "desbabar",
    "desbabaron": "desbabar",
    "desbabará": "desbabar",
    "desbabarán": "desbabar",
    "desbabarás": "desbabar",
    "desbabaré": "desbabar",
    "desbabaréis": "desbabar",
    "desbabaría": "desbabar",
    "desbabaríais": "desbabar",
    "desbabaríamos": "desbabar",
    "desbabarían": "desbabar",
    "desbabarías": "desbabar",
    "desbabas": "desbabar",
    "desbabase": "desbabar",
    "desbabaseis": "desbabar",
    "desbabasen": "desbabar",
    "desbabases": "desbabar",
    "desbabaste": "desbabar",
    "desbabasteis": "desbabar",
    "desbabe": "desbabar",
    "desbabemos": "desbabar",
    "desbaben": "desbabar",
    "desbabes": "desbabar",
    "desbabo": "desbabar",
    "desbabábamos": "desbabar",
    "desbabáis": "desbabar",
    "desbabáramos": "desbabar",
    "desbabáremos": "desbabar",
    "desbabásemos": "desbabar",
    "desbabé": "desbabar",
    "desbabéis": "desbabar",
    "desbabó": "desbabar",
    "desbaga": "desbagar",
    "desbagaba": "desbagar",
    "desbagabais": "desbagar",
    "desbagaban": "desbagar",
    "desbagabas": "desbagar",
    "desbagad": "desbagar",
    "desbagada": "desbagar",
    "desbagadas": "desbagar",
    "desbagado": "desbagar",
    "desbagados": "desbagar",
    "desbagamos": "desbagar",
    "desbagan": "desbagar",
    "desbagando": "desbagar",
    "desbagara": "desbagar",
    "desbagarais": "desbagar",
    "desbagaran": "desbagar",
    "desbagaras": "desbagar",
    "desbagare": "desbagar",
    "desbagareis": "desbagar",
    "desbagaremos": "desbagar",
    "desbagaren": "desbagar",
    "desbagares": "desbagar",
    "desbagaron": "desbagar",
    "desbagará": "desbagar",
    "desbagarán": "desbagar",
    "desbagarás": "desbagar",
    "desbagaré": "desbagar",
    "desbagaréis": "desbagar",
    "desbagaría": "desbagar",
    "desbagaríais": "desbagar",
    "desbagaríamos": "desbagar",
    "desbagarían": "desbagar",
    "desbagarías": "desbagar",
    "desbagas": "desbagar",
    "desbagase": "desbagar",
    "desbagaseis": "desbagar",
    "desbagasen": "desbagar",
    "desbagases": "desbagar",
    "desbagaste": "desbagar",
    "desbagasteis": "desbagar",
    "desbago": "desbagar",
    "desbague": "desbagar",
    "desbaguemos": "desbagar",
    "desbaguen": "desbagar",
    "desbagues": "desbagar",
    "desbagué": "desbagar",
    "desbaguéis": "desbagar",
    "desbagábamos": "desbagar",
    "desbagáis": "desbagar",
    "desbagáramos": "desbagar",
    "desbagáremos": "desbagar",
    "desbagásemos": "desbagar",
    "desbagó": "desbagar",
    "desbalanceada": "desbalanceado",
    "desbalanceadas": "desbalanceado",
    "desbalanceados": "desbalanceado",
    "desbanca": "desbancar",
    "desbancaba": "desbancar",
    "desbancabais": "desbancar",
    "desbancaban": "desbancar",
    "desbancabas": "desbancar",
    "desbancad": "desbancar",
    "desbancada": "desbancar",
    "desbancadas": "desbancar",
    "desbancado": "desbancar",
    "desbancados": "desbancar",
    "desbancamos": "desbancar",
    "desbancan": "desbancar",
    "desbancando": "desbancar",
    "desbancara": "desbancar",
    "desbancarais": "desbancar",
    "desbancaran": "desbancar",
    "desbancaras": "desbancar",
    "desbancare": "desbancar",
    "desbancareis": "desbancar",
    "desbancaremos": "desbancar",
    "desbancaren": "desbancar",
    "desbancares": "desbancar",
    "desbancaron": "desbancar",
    "desbancará": "desbancar",
    "desbancarán": "desbancar",
    "desbancarás": "desbancar",
    "desbancaré": "desbancar",
    "desbancaréis": "desbancar",
    "desbancaría": "desbancar",
    "desbancaríais": "desbancar",
    "desbancaríamos": "desbancar",
    "desbancarían": "desbancar",
    "desbancarías": "desbancar",
    "desbancas": "desbancar",
    "desbancase": "desbancar",
    "desbancaseis": "desbancar",
    "desbancasen": "desbancar",
    "desbancases": "desbancar",
    "desbancaste": "desbancar",
    "desbancasteis": "desbancar",
    "desbanco": "desbancar",
    "desbancábamos": "desbancar",
    "desbancáis": "desbancar",
    "desbancáramos": "desbancar",
    "desbancáremos": "desbancar",
    "desbancásemos": "desbancar",
    "desbancó": "desbancar",
    "desbanda": "desbandar",
    "desbandaba": "desbandar",
    "desbandabais": "desbandar",
    "desbandaban": "desbandar",
    "desbandabas": "desbandar",
    "desbandad": "desbandar",
    "desbandada": "desbandar",
    "desbandadas": "desbandar",
    "desbandado": "desbandar",
    "desbandados": "desbandar",
    "desbandamos": "desbandar",
    "desbandan": "desbandar",
    "desbandando": "desbandar",
    "desbandara": "desbandar",
    "desbandarais": "desbandar",
    "desbandaran": "desbandar",
    "desbandaras": "desbandar",
    "desbandare": "desbandar",
    "desbandareis": "desbandar",
    "desbandaremos": "desbandar",
    "desbandaren": "desbandar",
    "desbandares": "desbandar",
    "desbandaron": "desbandar",
    "desbandará": "desbandar",
    "desbandarán": "desbandar",
    "desbandarás": "desbandar",
    "desbandaré": "desbandar",
    "desbandaréis": "desbandar",
    "desbandaría": "desbandar",
    "desbandaríais": "desbandar",
    "desbandaríamos": "desbandar",
    "desbandarían": "desbandar",
    "desbandarías": "desbandar",
    "desbandas": "desbandar",
    "desbandase": "desbandar",
    "desbandaseis": "desbandar",
    "desbandasen": "desbandar",
    "desbandases": "desbandar",
    "desbandaste": "desbandar",
    "desbandasteis": "desbandar",
    "desbande": "desbandar",
    "desbandemos": "desbandar",
    "desbanden": "desbandar",
    "desbandes": "desbandar",
    "desbando": "desbandar",
    "desbandábamos": "desbandar",
    "desbandáis": "desbandar",
    "desbandáramos": "desbandar",
    "desbandáremos": "desbandar",
    "desbandásemos": "desbandar",
    "desbandé": "desbandar",
    "desbandéis": "desbandar",
    "desbandó": "desbandar",
    "desbanque": "desbancar",
    "desbanquemos": "desbancar",
    "desbanquen": "desbancar",
    "desbanques": "desbancar",
    "desbanqué": "desbancar",
    "desbanquéis": "desbancar",
    "desbarajusta": "desbarajustar",
    "desbarajustaba": "desbarajustar",
    "desbarajustabais": "desbarajustar",
    "desbarajustaban": "desbarajustar",
    "desbarajustabas": "desbarajustar",
    "desbarajustad": "desbarajustar",
    "desbarajustada": "desbarajustar",
    "desbarajustadas": "desbarajustar",
    "desbarajustado": "desbarajustar",
    "desbarajustados": "desbarajustar",
    "desbarajustamos": "desbarajustar",
    "desbarajustan": "desbarajustar",
    "desbarajustando": "desbarajustar",
    "desbarajustara": "desbarajustar",
    "desbarajustarais": "desbarajustar",
    "desbarajustaran": "desbarajustar",
    "desbarajustaras": "desbarajustar",
    "desbarajustare": "desbarajustar",
    "desbarajustareis": "desbarajustar",
    "desbarajustaremos": "desbarajustar",
    "desbarajustaren": "desbarajustar",
    "desbarajustares": "desbarajustar",
    "desbarajustaron": "desbarajustar",
    "desbarajustará": "desbarajustar",
    "desbarajustarán": "desbarajustar",
    "desbarajustarás": "desbarajustar",
    "desbarajustaré": "desbarajustar",
    "desbarajustaréis": "desbarajustar",
    "desbarajustaría": "desbarajustar",
    "desbarajustaríais": "desbarajustar",
    "desbarajustaríamos": "desbarajustar",
    "desbarajustarían": "desbarajustar",
    "desbarajustarías": "desbarajustar",
    "desbarajustas": "desbarajustar",
    "desbarajustase": "desbarajustar",
    "desbarajustaseis": "desbarajustar",
    "desbarajustasen": "desbarajustar",
    "desbarajustases": "desbarajustar",
    "desbarajustaste": "desbarajustar",
    "desbarajustasteis": "desbarajustar",
    "desbarajuste": "desbarajustar",
    "desbarajustemos": "desbarajustar",
    "desbarajusten": "desbarajustar",
    "desbarajustes": "desbarajuste",
    "desbarajusto": "desbarajustar",
    "desbarajustábamos": "desbarajustar",
    "desbarajustáis": "desbarajustar",
    "desbarajustáramos": "desbarajustar",
    "desbarajustáremos": "desbarajustar",
    "desbarajustásemos": "desbarajustar",
    "desbarajusté": "desbarajustar",
    "desbarajustéis": "desbarajustar",
    "desbarajustó": "desbarajustar",
    "desbarata": "desbaratar",
    "desbarataba": "desbaratar",
    "desbaratabais": "desbaratar",
    "desbarataban": "desbaratar",
    "desbaratabas": "desbaratar",
    "desbaratad": "desbaratar",
    "desbaratada": "desbaratar",
    "desbaratadas": "desbaratar",
    "desbaratado": "desbaratar",
    "desbaratadora": "desbaratador",
    "desbaratadoras": "desbaratador",
    "desbaratadores": "desbaratador",
    "desbaratados": "desbaratar",
    "desbaratamientos": "desbaratamiento",
    "desbaratamos": "desbaratar",
    "desbaratan": "desbaratar",
    "desbaratando": "desbaratar",
    "desbaratara": "desbaratar",
    "desbaratarais": "desbaratar",
    "desbarataran": "desbaratar",
    "desbarataras": "desbaratar",
    "desbaratare": "desbaratar",
    "desbaratareis": "desbaratar",
    "desbarataremos": "desbaratar",
    "desbarataren": "desbaratar",
    "desbaratares": "desbaratar",
    "desbarataron": "desbaratar",
    "desbaratará": "desbaratar",
    "desbaratarán": "desbaratar",
    "desbaratarás": "desbaratar",
    "desbarataré": "desbaratar",
    "desbarataréis": "desbaratar",
    "desbarataría": "desbaratar",
    "desbarataríais": "desbaratar",
    "desbarataríamos": "desbaratar",
    "desbaratarían": "desbaratar",
    "desbaratarías": "desbaratar",
    "desbaratas": "desbaratar",
    "desbaratase": "desbaratar",
    "desbarataseis": "desbaratar",
    "desbaratasen": "desbaratar",
    "desbaratases": "desbaratar",
    "desbarataste": "desbaratar",
    "desbaratasteis": "desbaratar",
    "desbarate": "desbaratar",
    "desbaratemos": "desbaratar",
    "desbaraten": "desbaratar",
    "desbarates": "desbarate",
    "desbarato": "desbaratar",
    "desbaratábamos": "desbaratar",
    "desbaratáis": "desbaratar",
    "desbaratáramos": "desbaratar",
    "desbaratáremos": "desbaratar",
    "desbaratásemos": "desbaratar",
    "desbaraté": "desbaratar",
    "desbaratéis": "desbaratar",
    "desbarató": "desbaratar",
    "desbarba": "desbarbar",
    "desbarbaba": "desbarbar",
    "desbarbabais": "desbarbar",
    "desbarbaban": "desbarbar",
    "desbarbabas": "desbarbar",
    "desbarbad": "desbarbar",
    "desbarbada": "desbarbar",
    "desbarbadas": "desbarbar",
    "desbarbado": "desbarbar",
    "desbarbadora": "desbarbador",
    "desbarbadoras": "desbarbadora",
    "desbarbadores": "desbarbador",
    "desbarbados": "desbarbar",
    "desbarbajes": "desbarbaje",
    "desbarbamos": "desbarbar",
    "desbarban": "desbarbar",
    "desbarbando": "desbarbar",
    "desbarbara": "desbarbar",
    "desbarbarais": "desbarbar",
    "desbarbaran": "desbarbar",
    "desbarbaras": "desbarbar",
    "desbarbare": "desbarbar",
    "desbarbareis": "desbarbar",
    "desbarbaremos": "desbarbar",
    "desbarbaren": "desbarbar",
    "desbarbares": "desbarbar",
    "desbarbaron": "desbarbar",
    "desbarbará": "desbarbar",
    "desbarbarán": "desbarbar",
    "desbarbarás": "desbarbar",
    "desbarbaré": "desbarbar",
    "desbarbaréis": "desbarbar",
    "desbarbaría": "desbarbar",
    "desbarbaríais": "desbarbar",
    "desbarbaríamos": "desbarbar",
    "desbarbarían": "desbarbar",
    "desbarbarías": "desbarbar",
    "desbarbas": "desbarbar",
    "desbarbase": "desbarbar",
    "desbarbaseis": "desbarbar",
    "desbarbasen": "desbarbar",
    "desbarbases": "desbarbar",
    "desbarbaste": "desbarbar",
    "desbarbasteis": "desbarbar",
    "desbarbe": "desbarbar",
    "desbarbemos": "desbarbar",
    "desbarben": "desbarbar",
    "desbarbes": "desbarbar",
    "desbarbilla": "desbarbillar",
    "desbarbillaba": "desbarbillar",
    "desbarbillabais": "desbarbillar",
    "desbarbillaban": "desbarbillar",
    "desbarbillabas": "desbarbillar",
    "desbarbillad": "desbarbillar",
    "desbarbillada": "desbarbillar",
    "desbarbilladas": "desbarbillar",
    "desbarbillado": "desbarbillar",
    "desbarbillados": "desbarbillar",
    "desbarbillamos": "desbarbillar",
    "desbarbillan": "desbarbillar",
    "desbarbillando": "desbarbillar",
    "desbarbillara": "desbarbillar",
    "desbarbillarais": "desbarbillar",
    "desbarbillaran": "desbarbillar",
    "desbarbillaras": "desbarbillar",
    "desbarbillare": "desbarbillar",
    "desbarbillareis": "desbarbillar",
    "desbarbillaremos": "desbarbillar",
    "desbarbillaren": "desbarbillar",
    "desbarbillares": "desbarbillar",
    "desbarbillaron": "desbarbillar",
    "desbarbillará": "desbarbillar",
    "desbarbillarán": "desbarbillar",
    "desbarbillarás": "desbarbillar",
    "desbarbillaré": "desbarbillar",
    "desbarbillaréis": "desbarbillar",
    "desbarbillaría": "desbarbillar",
    "desbarbillaríais": "desbarbillar",
    "desbarbillaríamos": "desbarbillar",
    "desbarbillarían": "desbarbillar",
    "desbarbillarías": "desbarbillar",
    "desbarbillas": "desbarbillar",
    "desbarbillase": "desbarbillar",
    "desbarbillaseis": "desbarbillar",
    "desbarbillasen": "desbarbillar",
    "desbarbillases": "desbarbillar",
    "desbarbillaste": "desbarbillar",
    "desbarbillasteis": "desbarbillar",
    "desbarbille": "desbarbillar",
    "desbarbillemos": "desbarbillar",
    "desbarbillen": "desbarbillar",
    "desbarbilles": "desbarbillar",
    "desbarbillo": "desbarbillar",
    "desbarbillábamos": "desbarbillar",
    "desbarbilláis": "desbarbillar",
    "desbarbilláramos": "desbarbillar",
    "desbarbilláremos": "desbarbillar",
    "desbarbillásemos": "desbarbillar",
    "desbarbillé": "desbarbillar",
    "desbarbilléis": "desbarbillar",
    "desbarbilló": "desbarbillar",
    "desbarbo": "desbarbar",
    "desbarbábamos": "desbarbar",
    "desbarbáis": "desbarbar",
    "desbarbáramos": "desbarbar",
    "desbarbáremos": "desbarbar",
    "desbarbásemos": "desbarbar",
    "desbarbé": "desbarbar",
    "desbarbéis": "desbarbar",
    "desbarbó": "desbarbar",
    "desbarda": "desbardar",
    "desbardaba": "desbardar",
    "desbardabais": "desbardar",
    "desbardaban": "desbardar",
    "desbardabas": "desbardar",
    "desbardad": "desbardar",
    "desbardada": "desbardar",
    "desbardadas": "desbardar",
    "desbardado": "desbardar",
    "desbardados": "desbardar",
    "desbardamos": "desbardar",
    "desbardan": "desbardar",
    "desbardando": "desbardar",
    "desbardara": "desbardar",
    "desbardarais": "desbardar",
    "desbardaran": "desbardar",
    "desbardaras": "desbardar",
    "desbardare": "desbardar",
    "desbardareis": "desbardar",
    "desbardaremos": "desbardar",
    "desbardaren": "desbardar",
    "desbardares": "desbardar",
    "desbardaron": "desbardar",
    "desbardará": "desbardar",
    "desbardarán": "desbardar",
    "desbardarás": "desbardar",
    "desbardaré": "desbardar",
    "desbardaréis": "desbardar",
    "desbardaría": "desbardar",
    "desbardaríais": "desbardar",
    "desbardaríamos": "desbardar",
    "desbardarían": "desbardar",
    "desbardarías": "desbardar",
    "desbardas": "desbardar",
    "desbardase": "desbardar",
    "desbardaseis": "desbardar",
    "desbardasen": "desbardar",
    "desbardases": "desbardar",
    "desbardaste": "desbardar",
    "desbardasteis": "desbardar",
    "desbarde": "desbardar",
    "desbardemos": "desbardar",
    "desbarden": "desbardar",
    "desbardes": "desbardar",
    "desbardo": "desbardar",
    "desbardábamos": "desbardar",
    "desbardáis": "desbardar",
    "desbardáramos": "desbardar",
    "desbardáremos": "desbardar",
    "desbardásemos": "desbardar",
    "desbardé": "desbardar",
    "desbardéis": "desbardar",
    "desbardó": "desbardar",
    "desbarnice": "desbarnizar",
    "desbarnicemos": "desbarnizar",
    "desbarnicen": "desbarnizar",
    "desbarnices": "desbarnizar",
    "desbarnicé": "desbarnizar",
    "desbarnicéis": "desbarnizar",
    "desbarniza": "desbarnizar",
    "desbarnizaba": "desbarnizar",
    "desbarnizabais": "desbarnizar",
    "desbarnizaban": "desbarnizar",
    "desbarnizabas": "desbarnizar",
    "desbarnizad": "desbarnizar",
    "desbarnizada": "desbarnizar",
    "desbarnizadas": "desbarnizar",
    "desbarnizado": "desbarnizar",
    "desbarnizados": "desbarnizar",
    "desbarnizamos": "desbarnizar",
    "desbarnizan": "desbarnizar",
    "desbarnizando": "desbarnizar",
    "desbarnizara": "desbarnizar",
    "desbarnizarais": "desbarnizar",
    "desbarnizaran": "desbarnizar",
    "desbarnizaras": "desbarnizar",
    "desbarnizare": "desbarnizar",
    "desbarnizareis": "desbarnizar",
    "desbarnizaremos": "desbarnizar",
    "desbarnizaren": "desbarnizar",
    "desbarnizares": "desbarnizar",
    "desbarnizaron": "desbarnizar",
    "desbarnizará": "desbarnizar",
    "desbarnizarán": "desbarnizar",
    "desbarnizarás": "desbarnizar",
    "desbarnizaré": "desbarnizar",
    "desbarnizaréis": "desbarnizar",
    "desbarnizaría": "desbarnizar",
    "desbarnizaríais": "desbarnizar",
    "desbarnizaríamos": "desbarnizar",
    "desbarnizarían": "desbarnizar",
    "desbarnizarías": "desbarnizar",
    "desbarnizas": "desbarnizar",
    "desbarnizase": "desbarnizar",
    "desbarnizaseis": "desbarnizar",
    "desbarnizasen": "desbarnizar",
    "desbarnizases": "desbarnizar",
    "desbarnizaste": "desbarnizar",
    "desbarnizasteis": "desbarnizar",
    "desbarnizo": "desbarnizar",
    "desbarnizábamos": "desbarnizar",
    "desbarnizáis": "desbarnizar",
    "desbarnizáramos": "desbarnizar",
    "desbarnizáremos": "desbarnizar",
    "desbarnizásemos": "desbarnizar",
    "desbarnizó": "desbarnizar",
    "desbarra": "desbarrar",
    "desbarraba": "desbarrar",
    "desbarrabais": "desbarrar",
    "desbarraban": "desbarrar",
    "desbarrabas": "desbarrar",
    "desbarrad": "desbarrar",
    "desbarrada": "desbarrar",
    "desbarradas": "desbarrar",
    "desbarrado": "desbarrar",
    "desbarrados": "desbarrar",
    "desbarramos": "desbarrar",
    "desbarran": "desbarrar",
    "desbarrando": "desbarrar",
    "desbarrara": "desbarrar",
    "desbarrarais": "desbarrar",
    "desbarraran": "desbarrar",
    "desbarraras": "desbarrar",
    "desbarrare": "desbarrar",
    "desbarrareis": "desbarrar",
    "desbarraremos": "desbarrar",
    "desbarraren": "desbarrar",
    "desbarrares": "desbarrar",
    "desbarraron": "desbarrar",
    "desbarrará": "desbarrar",
    "desbarrarán": "desbarrar",
    "desbarrarás": "desbarrar",
    "desbarraré": "desbarrar",
    "desbarraréis": "desbarrar",
    "desbarraría": "desbarrar",
    "desbarraríais": "desbarrar",
    "desbarraríamos": "desbarrar",
    "desbarrarían": "desbarrar",
    "desbarrarías": "desbarrar",
    "desbarras": "desbarrar",
    "desbarrase": "desbarrar",
    "desbarraseis": "desbarrar",
    "desbarrasen": "desbarrar",
    "desbarrases": "desbarrar",
    "desbarraste": "desbarrar",
    "desbarrasteis": "desbarrar",
    "desbarre": "desbarrar",
    "desbarremos": "desbarrar",
    "desbarren": "desbarrar",
    "desbarres": "desbarrar",
    "desbarrigada": "desbarrigado",
    "desbarrigadas": "desbarrigado",
    "desbarrigados": "desbarrigado",
    "desbarro": "desbarrar",
    "desbarros": "desbarro",
    "desbarrábamos": "desbarrar",
    "desbarráis": "desbarrar",
    "desbarráramos": "desbarrar",
    "desbarráremos": "desbarrar",
    "desbarrásemos": "desbarrar",
    "desbarré": "desbarrar",
    "desbarréis": "desbarrar",
    "desbarró": "desbarrar",
    "desbasta": "desbastar",
    "desbastaba": "desbastar",
    "desbastabais": "desbastar",
    "desbastaban": "desbastar",
    "desbastabas": "desbastar",
    "desbastad": "desbastar",
    "desbastada": "desbastar",
    "desbastadas": "desbastar",
    "desbastado": "desbastar",
    "desbastadora": "desbastador",
    "desbastadoras": "desbastador",
    "desbastadores": "desbastador",
    "desbastados": "desbastar",
    "desbastaduras": "desbastadura",
    "desbastamos": "desbastar",
    "desbastan": "desbastar",
    "desbastando": "desbastar",
    "desbastara": "desbastar",
    "desbastarais": "desbastar",
    "desbastaran": "desbastar",
    "desbastaras": "desbastar",
    "desbastare": "desbastar",
    "desbastareis": "desbastar",
    "desbastaremos": "desbastar",
    "desbastaren": "desbastar",
    "desbastares": "desbastar",
    "desbastaron": "desbastar",
    "desbastará": "desbastar",
    "desbastarán": "desbastar",
    "desbastarás": "desbastar",
    "desbastaré": "desbastar",
    "desbastaréis": "desbastar",
    "desbastaría": "desbastar",
    "desbastaríais": "desbastar",
    "desbastaríamos": "desbastar",
    "desbastarían": "desbastar",
    "desbastarías": "desbastar",
    "desbastas": "desbastar",
    "desbastase": "desbastar",
    "desbastaseis": "desbastar",
    "desbastasen": "desbastar",
    "desbastases": "desbastar",
    "desbastaste": "desbastar",
    "desbastasteis": "desbastar",
    "desbaste": "desbastar",
    "desbastecida": "desbastecido",
    "desbastecidas": "desbastecido",
    "desbastecidos": "desbastecido",
    "desbastemos": "desbastar",
    "desbasten": "desbastar",
    "desbastes": "desbaste",
    "desbasto": "desbastar",
    "desbastábamos": "desbastar",
    "desbastáis": "desbastar",
    "desbastáramos": "desbastar",
    "desbastáremos": "desbastar",
    "desbastásemos": "desbastar",
    "desbasté": "desbastar",
    "desbastéis": "desbastar",
    "desbastó": "desbastar",
    "desbazaderos": "desbazadero",
    "desbecerra": "desbecerrar",
    "desbecerraba": "desbecerrar",
    "desbecerrabais": "desbecerrar",
    "desbecerraban": "desbecerrar",
    "desbecerrabas": "desbecerrar",
    "desbecerrad": "desbecerrar",
    "desbecerrada": "desbecerrar",
    "desbecerradas": "desbecerrar",
    "desbecerrado": "desbecerrar",
    "desbecerrados": "desbecerrar",
    "desbecerramos": "desbecerrar",
    "desbecerran": "desbecerrar",
    "desbecerrando": "desbecerrar",
    "desbecerrara": "desbecerrar",
    "desbecerrarais": "desbecerrar",
    "desbecerraran": "desbecerrar",
    "desbecerraras": "desbecerrar",
    "desbecerrare": "desbecerrar",
    "desbecerrareis": "desbecerrar",
    "desbecerraremos": "desbecerrar",
    "desbecerraren": "desbecerrar",
    "desbecerrares": "desbecerrar",
    "desbecerraron": "desbecerrar",
    "desbecerrará": "desbecerrar",
    "desbecerrarán": "desbecerrar",
    "desbecerrarás": "desbecerrar",
    "desbecerraré": "desbecerrar",
    "desbecerraréis": "desbecerrar",
    "desbecerraría": "desbecerrar",
    "desbecerraríais": "desbecerrar",
    "desbecerraríamos": "desbecerrar",
    "desbecerrarían": "desbecerrar",
    "desbecerrarías": "desbecerrar",
    "desbecerras": "desbecerrar",
    "desbecerrase": "desbecerrar",
    "desbecerraseis": "desbecerrar",
    "desbecerrasen": "desbecerrar",
    "desbecerrases": "desbecerrar",
    "desbecerraste": "desbecerrar",
    "desbecerrasteis": "desbecerrar",
    "desbecerre": "desbecerrar",
    "desbecerremos": "desbecerrar",
    "desbecerren": "desbecerrar",
    "desbecerres": "desbecerrar",
    "desbecerro": "desbecerrar",
    "desbecerrábamos": "desbecerrar",
    "desbecerráis": "desbecerrar",
    "desbecerráramos": "desbecerrar",
    "desbecerráremos": "desbecerrar",
    "desbecerrásemos": "desbecerrar",
    "desbecerré": "desbecerrar",
    "desbecerréis": "desbecerrar",
    "desbecerró": "desbecerrar",
    "desblanquecida": "desblanquecido",
    "desblanquecidas": "desblanquecido",
    "desblanquecidos": "desblanquecido",
    "desbloquea": "desbloquear",
    "desbloqueaba": "desbloquear",
    "desbloqueabais": "desbloquear",
    "desbloqueaban": "desbloquear",
    "desbloqueabas": "desbloquear",
    "desbloquead": "desbloquear",
    "desbloqueada": "desbloquear",
    "desbloqueadas": "desbloquear",
    "desbloqueado": "desbloquear",
    "desbloqueados": "desbloquear",
    "desbloqueamientos": "desbloqueamiento",
    "desbloqueamos": "desbloquear",
    "desbloquean": "desbloquear",
    "desbloqueando": "desbloquear",
    "desbloqueara": "desbloquear",
    "desbloquearais": "desbloquear",
    "desbloquearan": "desbloquear",
    "desbloquearas": "desbloquear",
    "desbloqueare": "desbloquear",
    "desbloqueareis": "desbloquear",
    "desbloquearemos": "desbloquear",
    "desbloquearen": "desbloquear",
    "desbloqueares": "desbloquear",
    "desbloquearon": "desbloquear",
    "desbloqueará": "desbloquear",
    "desbloquearán": "desbloquear",
    "desbloquearás": "desbloquear",
    "desbloquearé": "desbloquear",
    "desbloquearéis": "desbloquear",
    "desbloquearía": "desbloquear",
    "desbloquearíais": "desbloquear",
    "desbloquearíamos": "desbloquear",
    "desbloquearían": "desbloquear",
    "desbloquearías": "desbloquear",
    "desbloqueas": "desbloquear",
    "desbloquease": "desbloquear",
    "desbloqueaseis": "desbloquear",
    "desbloqueasen": "desbloquear",
    "desbloqueases": "desbloquear",
    "desbloqueaste": "desbloquear",
    "desbloqueasteis": "desbloquear",
    "desbloquee": "desbloquear",
    "desbloqueemos": "desbloquear",
    "desbloqueen": "desbloquear",
    "desbloquees": "desbloquear",
    "desbloqueo": "desbloquear",
    "desbloqueos": "desbloqueo",
    "desbloqueábamos": "desbloquear",
    "desbloqueáis": "desbloquear",
    "desbloqueáramos": "desbloquear",
    "desbloqueáremos": "desbloquear",
    "desbloqueásemos": "desbloquear",
    "desbloqueé": "desbloquear",
    "desbloqueéis": "desbloquear",
    "desbloqueó": "desbloquear",
    "desbobinadores": "desbobinador",
    "desboca": "desbocar",
    "desbocaba": "desbocar",
    "desbocabais": "desbocar",
    "desbocaban": "desbocar",
    "desbocabas": "desbocar",
    "desbocad": "desbocar",
    "desbocada": "desbocar",
    "desbocadas": "desbocar",
    "desbocado": "desbocar",
    "desbocados": "desbocar",
    "desbocamientos": "desbocamiento",
    "desbocamos": "desbocar",
    "desbocan": "desbocar",
    "desbocando": "desbocar",
    "desbocara": "desbocar",
    "desbocarais": "desbocar",
    "desbocaran": "desbocar",
    "desbocaras": "desbocar",
    "desbocare": "desbocar",
    "desbocareis": "desbocar",
    "desbocaremos": "desbocar",
    "desbocaren": "desbocar",
    "desbocares": "desbocar",
    "desbocaron": "desbocar",
    "desbocará": "desbocar",
    "desbocarán": "desbocar",
    "desbocarás": "desbocar",
    "desbocaré": "desbocar",
    "desbocaréis": "desbocar",
    "desbocaría": "desbocar",
    "desbocaríais": "desbocar",
    "desbocaríamos": "desbocar",
    "desbocarían": "desbocar",
    "desbocarías": "desbocar",
    "desbocas": "desbocar",
    "desbocase": "desbocar",
    "desbocaseis": "desbocar",
    "desbocasen": "desbocar",
    "desbocases": "desbocar",
    "desbocaste": "desbocar",
    "desbocasteis": "desbocar",
    "desboco": "desbocar",
    "desbocábamos": "desbocar",
    "desbocáis": "desbocar",
    "desbocáramos": "desbocar",
    "desbocáremos": "desbocar",
    "desbocásemos": "desbocar",
    "desbocó": "desbocar",
    "desboque": "desbocar",
    "desboquemos": "desbocar",
    "desboquen": "desbocar",
    "desboques": "desbocar",
    "desboquilla": "desboquillar",
    "desboquillaba": "desboquillar",
    "desboquillabais": "desboquillar",
    "desboquillaban": "desboquillar",
    "desboquillabas": "desboquillar",
    "desboquillad": "desboquillar",
    "desboquillada": "desboquillar",
    "desboquilladas": "desboquillar",
    "desboquillado": "desboquillar",
    "desboquillados": "desboquillar",
    "desboquillamos": "desboquillar",
    "desboquillan": "desboquillar",
    "desboquillando": "desboquillar",
    "desboquillara": "desboquillar",
    "desboquillarais": "desboquillar",
    "desboquillaran": "desboquillar",
    "desboquillaras": "desboquillar",
    "desboquillare": "desboquillar",
    "desboquillareis": "desboquillar",
    "desboquillaremos": "desboquillar",
    "desboquillaren": "desboquillar",
    "desboquillares": "desboquillar",
    "desboquillaron": "desboquillar",
    "desboquillará": "desboquillar",
    "desboquillarán": "desboquillar",
    "desboquillarás": "desboquillar",
    "desboquillaré": "desboquillar",
    "desboquillaréis": "desboquillar",
    "desboquillaría": "desboquillar",
    "desboquillaríais": "desboquillar",
    "desboquillaríamos": "desboquillar",
    "desboquillarían": "desboquillar",
    "desboquillarías": "desboquillar",
    "desboquillas": "desboquillar",
    "desboquillase": "desboquillar",
    "desboquillaseis": "desboquillar",
    "desboquillasen": "desboquillar",
    "desboquillases": "desboquillar",
    "desboquillaste": "desboquillar",
    "desboquillasteis": "desboquillar",
    "desboquille": "desboquillar",
    "desboquillemos": "desboquillar",
    "desboquillen": "desboquillar",
    "desboquilles": "desboquillar",
    "desboquillo": "desboquillar",
    "desboquillábamos": "desboquillar",
    "desboquilláis": "desboquillar",
    "desboquilláramos": "desboquillar",
    "desboquilláremos": "desboquillar",
    "desboquillásemos": "desboquillar",
    "desboquillé": "desboquillar",
    "desboquilléis": "desboquillar",
    "desboquilló": "desboquillar",
    "desboqué": "desbocar",
    "desboquéis": "desbocar",
    "desborda": "desbordar",
    "desbordaba": "desbordar",
    "desbordabais": "desbordar",
    "desbordaban": "desbordar",
    "desbordabas": "desbordar",
    "desbordad": "desbordar",
    "desbordada": "desbordar",
    "desbordadas": "desbordar",
    "desbordado": "desbordar",
    "desbordados": "desbordar",
    "desbordamientos": "desbordamiento",
    "desbordamos": "desbordar",
    "desbordan": "desbordar",
    "desbordando": "desbordar",
    "desbordantes": "desbordante",
    "desbordara": "desbordar",
    "desbordarais": "desbordar",
    "desbordaran": "desbordar",
    "desbordaras": "desbordar",
    "desbordare": "desbordar",
    "desbordareis": "desbordar",
    "desbordaremos": "desbordar",
    "desbordaren": "desbordar",
    "desbordares": "desbordar",
    "desbordaron": "desbordar",
    "desbordará": "desbordar",
    "desbordarán": "desbordar",
    "desbordarás": "desbordar",
    "desbordaré": "desbordar",
    "desbordaréis": "desbordar",
    "desbordaría": "desbordar",
    "desbordaríais": "desbordar",
    "desbordaríamos": "desbordar",
    "desbordarían": "desbordar",
    "desbordarías": "desbordar",
    "desbordas": "desbordar",
    "desbordase": "desbordar",
    "desbordaseis": "desbordar",
    "desbordasen": "desbordar",
    "desbordases": "desbordar",
    "desbordaste": "desbordar",
    "desbordasteis": "desbordar",
    "desborde": "desbordar",
    "desbordemos": "desbordar",
    "desborden": "desbordar",
    "desbordes": "desbordar",
    "desbordo": "desbordar",
    "desbordábamos": "desbordar",
    "desbordáis": "desbordar",
    "desbordáramos": "desbordar",
    "desbordáremos": "desbordar",
    "desbordásemos": "desbordar",
    "desbordé": "desbordar",
    "desbordéis": "desbordar",
    "desbordó": "desbordar",
    "desborra": "desborrar",
    "desborraba": "desborrar",
    "desborrabais": "desborrar",
    "desborraban": "desborrar",
    "desborrabas": "desborrar",
    "desborrad": "desborrar",
    "desborrada": "desborrar",
    "desborradas": "desborrar",
    "desborrado": "desborrar",
    "desborrados": "desborrar",
    "desborramos": "desborrar",
    "desborran": "desborrar",
    "desborrando": "desborrar",
    "desborrara": "desborrar",
    "desborrarais": "desborrar",
    "desborraran": "desborrar",
    "desborraras": "desborrar",
    "desborrare": "desborrar",
    "desborrareis": "desborrar",
    "desborraremos": "desborrar",
    "desborraren": "desborrar",
    "desborrares": "desborrar",
    "desborraron": "desborrar",
    "desborrará": "desborrar",
    "desborrarán": "desborrar",
    "desborrarás": "desborrar",
    "desborraré": "desborrar",
    "desborraréis": "desborrar",
    "desborraría": "desborrar",
    "desborraríais": "desborrar",
    "desborraríamos": "desborrar",
    "desborrarían": "desborrar",
    "desborrarías": "desborrar",
    "desborras": "desborrar",
    "desborrase": "desborrar",
    "desborraseis": "desborrar",
    "desborrasen": "desborrar",
    "desborrases": "desborrar",
    "desborraste": "desborrar",
    "desborrasteis": "desborrar",
    "desborre": "desborrar",
    "desborremos": "desborrar",
    "desborren": "desborrar",
    "desborres": "desborrar",
    "desborro": "desborrar",
    "desborrábamos": "desborrar",
    "desborráis": "desborrar",
    "desborráramos": "desborrar",
    "desborráremos": "desborrar",
    "desborrásemos": "desborrar",
    "desborré": "desborrar",
    "desborréis": "desborrar",
    "desborró": "desborrar",
    "desbotona": "desbotonar",
    "desbotonaba": "desbotonar",
    "desbotonabais": "desbotonar",
    "desbotonaban": "desbotonar",
    "desbotonabas": "desbotonar",
    "desbotonad": "desbotonar",
    "desbotonada": "desbotonar",
    "desbotonadas": "desbotonar",
    "desbotonado": "desbotonar",
    "desbotonados": "desbotonar",
    "desbotonamos": "desbotonar",
    "desbotonan": "desbotonar",
    "desbotonando": "desbotonar",
    "desbotonara": "desbotonar",
    "desbotonarais": "desbotonar",
    "desbotonaran": "desbotonar",
    "desbotonaras": "desbotonar",
    "desbotonare": "desbotonar",
    "desbotonareis": "desbotonar",
    "desbotonaremos": "desbotonar",
    "desbotonaren": "desbotonar",
    "desbotonares": "desbotonar",
    "desbotonaron": "desbotonar",
    "desbotonará": "desbotonar",
    "desbotonarán": "desbotonar",
    "desbotonarás": "desbotonar",
    "desbotonaré": "desbotonar",
    "desbotonaréis": "desbotonar",
    "desbotonaría": "desbotonar",
    "desbotonaríais": "desbotonar",
    "desbotonaríamos": "desbotonar",
    "desbotonarían": "desbotonar",
    "desbotonarías": "desbotonar",
    "desbotonas": "desbotonar",
    "desbotonase": "desbotonar",
    "desbotonaseis": "desbotonar",
    "desbotonasen": "desbotonar",
    "desbotonases": "desbotonar",
    "desbotonaste": "desbotonar",
    "desbotonasteis": "desbotonar",
    "desbotone": "desbotonar",
    "desbotonemos": "desbotonar",
    "desbotonen": "desbotonar",
    "desbotones": "desbotonar",
    "desbotono": "desbotonar",
    "desbotonábamos": "desbotonar",
    "desbotonáis": "desbotonar",
    "desbotonáramos": "desbotonar",
    "desbotonáremos": "desbotonar",
    "desbotonásemos": "desbotonar",
    "desbotoné": "desbotonar",
    "desbotonéis": "desbotonar",
    "desbotonó": "desbotonar",
    "desbragada": "desbragado",
    "desbragadas": "desbragado",
    "desbragados": "desbragado",
    "desbraguetada": "desbraguetado",
    "desbraguetadas": "desbraguetado",
    "desbraguetados": "desbraguetado",
    "desbrava": "desbravar",
    "desbravaba": "desbravar",
    "desbravabais": "desbravar",
    "desbravaban": "desbravar",
    "desbravabas": "desbravar",
    "desbravad": "desbravar",
    "desbravada": "desbravar",
    "desbravadas": "desbravar",
    "desbravado": "desbravar",
    "desbravados": "desbravar",
    "desbravamientos": "desbravamiento",
    "desbravamos": "desbravar",
    "desbravan": "desbravar",
    "desbravando": "desbravar",
    "desbravara": "desbravar",
    "desbravarais": "desbravar",
    "desbravaran": "desbravar",
    "desbravaras": "desbravar",
    "desbravare": "desbravar",
    "desbravareis": "desbravar",
    "desbravaremos": "desbravar",
    "desbravaren": "desbravar",
    "desbravares": "desbravar",
    "desbravaron": "desbravar",
    "desbravará": "desbravar",
    "desbravarán": "desbravar",
    "desbravarás": "desbravar",
    "desbravaré": "desbravar",
    "desbravaréis": "desbravar",
    "desbravaría": "desbravar",
    "desbravaríais": "desbravar",
    "desbravaríamos": "desbravar",
    "desbravarían": "desbravar",
    "desbravarías": "desbravar",
    "desbravas": "desbravar",
    "desbravase": "desbravar",
    "desbravaseis": "desbravar",
    "desbravasen": "desbravar",
    "desbravases": "desbravar",
    "desbravaste": "desbravar",
    "desbravasteis": "desbravar",
    "desbrave": "desbravar",
    "desbravecimientos": "desbravecimiento",
    "desbravemos": "desbravar",
    "desbraven": "desbravar",
    "desbraves": "desbravar",
    "desbravo": "desbravar",
    "desbravábamos": "desbravar",
    "desbraváis": "desbravar",
    "desbraváramos": "desbravar",
    "desbraváremos": "desbravar",
    "desbravásemos": "desbravar",
    "desbravé": "desbravar",
    "desbravéis": "desbravar",
    "desbravó": "desbravar",
    "desbrida": "desbridar",
    "desbridaba": "desbridar",
    "desbridabais": "desbridar",
    "desbridaban": "desbridar",
    "desbridabas": "desbridar",
    "desbridad": "desbridar",
    "desbridada": "desbridar",
    "desbridadas": "desbridar",
    "desbridado": "desbridar",
    "desbridados": "desbridar",
    "desbridamientos": "desbridamiento",
    "desbridamos": "desbridar",
    "desbridan": "desbridar",
    "desbridando": "desbridar",
    "desbridara": "desbridar",
    "desbridarais": "desbridar",
    "desbridaran": "desbridar",
    "desbridaras": "desbridar",
    "desbridare": "desbridar",
    "desbridareis": "desbridar",
    "desbridaremos": "desbridar",
    "desbridaren": "desbridar",
    "desbridares": "desbridar",
    "desbridaron": "desbridar",
    "desbridará": "desbridar",
    "desbridarán": "desbridar",
    "desbridarás": "desbridar",
    "desbridaré": "desbridar",
    "desbridaréis": "desbridar",
    "desbridaría": "desbridar",
    "desbridaríais": "desbridar",
    "desbridaríamos": "desbridar",
    "desbridarían": "desbridar",
    "desbridarías": "desbridar",
    "desbridas": "desbridar",
    "desbridase": "desbridar",
    "desbridaseis": "desbridar",
    "desbridasen": "desbridar",
    "desbridases": "desbridar",
    "desbridaste": "desbridar",
    "desbridasteis": "desbridar",
    "desbride": "desbridar",
    "desbridemos": "desbridar",
    "desbriden": "desbridar",
    "desbrides": "desbridar",
    "desbrido": "desbridar",
    "desbridábamos": "desbridar",
    "desbridáis": "desbridar",
    "desbridáramos": "desbridar",
    "desbridáremos": "desbridar",
    "desbridásemos": "desbridar",
    "desbridé": "desbridar",
    "desbridéis": "desbridar",
    "desbridó": "desbridar",
    "desbrizna": "desbriznar",
    "desbriznaba": "desbriznar",
    "desbriznabais": "desbriznar",
    "desbriznaban": "desbriznar",
    "desbriznabas": "desbriznar",
    "desbriznad": "desbriznar",
    "desbriznada": "desbriznar",
    "desbriznadas": "desbriznar",
    "desbriznado": "desbriznar",
    "desbriznados": "desbriznar",
    "desbriznamos": "desbriznar",
    "desbriznan": "desbriznar",
    "desbriznando": "desbriznar",
    "desbriznara": "desbriznar",
    "desbriznarais": "desbriznar",
    "desbriznaran": "desbriznar",
    "desbriznaras": "desbriznar",
    "desbriznare": "desbriznar",
    "desbriznareis": "desbriznar",
    "desbriznaremos": "desbriznar",
    "desbriznaren": "desbriznar",
    "desbriznares": "desbriznar",
    "desbriznaron": "desbriznar",
    "desbriznará": "desbriznar",
    "desbriznarán": "desbriznar",
    "desbriznarás": "desbriznar",
    "desbriznaré": "desbriznar",
    "desbriznaréis": "desbriznar",
    "desbriznaría": "desbriznar",
    "desbriznaríais": "desbriznar",
    "desbriznaríamos": "desbriznar",
    "desbriznarían": "desbriznar",
    "desbriznarías": "desbriznar",
    "desbriznas": "desbriznar",
    "desbriznase": "desbriznar",
    "desbriznaseis": "desbriznar",
    "desbriznasen": "desbriznar",
    "desbriznases": "desbriznar",
    "desbriznaste": "desbriznar",
    "desbriznasteis": "desbriznar",
    "desbrizne": "desbriznar",
    "desbriznemos": "desbriznar",
    "desbriznen": "desbriznar",
    "desbriznes": "desbriznar",
    "desbrizno": "desbriznar",
    "desbriznábamos": "desbriznar",
    "desbriznáis": "desbriznar",
    "desbriznáramos": "desbriznar",
    "desbriznáremos": "desbriznar",
    "desbriznásemos": "desbriznar",
    "desbrizné": "desbriznar",
    "desbriznéis": "desbriznar",
    "desbriznó": "desbriznar",
    "desbroce": "desbrozar",
    "desbrocemos": "desbrozar",
    "desbrocen": "desbrozar",
    "desbroces": "desbrozar",
    "desbrocé": "desbrozar",
    "desbrocéis": "desbrozar",
    "desbroza": "desbrozar",
    "desbrozaba": "desbrozar",
    "desbrozabais": "desbrozar",
    "desbrozaban": "desbrozar",
    "desbrozabas": "desbrozar",
    "desbrozad": "desbrozar",
    "desbrozada": "desbrozar",
    "desbrozadas": "desbrozar",
    "desbrozado": "desbrozar",
    "desbrozadoras": "desbrozadora",
    "desbrozadores": "desbrozador",
    "desbrozados": "desbrozar",
    "desbrozamos": "desbrozar",
    "desbrozan": "desbrozar",
    "desbrozando": "desbrozar",
    "desbrozara": "desbrozar",
    "desbrozarais": "desbrozar",
    "desbrozaran": "desbrozar",
    "desbrozaras": "desbrozar",
    "desbrozare": "desbrozar",
    "desbrozareis": "desbrozar",
    "desbrozaremos": "desbrozar",
    "desbrozaren": "desbrozar",
    "desbrozares": "desbrozar",
    "desbrozaron": "desbrozar",
    "desbrozará": "desbrozar",
    "desbrozarán": "desbrozar",
    "desbrozarás": "desbrozar",
    "desbrozaré": "desbrozar",
    "desbrozaréis": "desbrozar",
    "desbrozaría": "desbrozar",
    "desbrozaríais": "desbrozar",
    "desbrozaríamos": "desbrozar",
    "desbrozarían": "desbrozar",
    "desbrozarías": "desbrozar",
    "desbrozas": "desbrozar",
    "desbrozase": "desbrozar",
    "desbrozaseis": "desbrozar",
    "desbrozasen": "desbrozar",
    "desbrozases": "desbrozar",
    "desbrozaste": "desbrozar",
    "desbrozasteis": "desbrozar",
    "desbrozo": "desbrozar",
    "desbrozos": "desbrozo",
    "desbrozábamos": "desbrozar",
    "desbrozáis": "desbrozar",
    "desbrozáramos": "desbrozar",
    "desbrozáremos": "desbrozar",
    "desbrozásemos": "desbrozar",
    "desbrozó": "desbrozar",
    "desbulla": "desbullar",
    "desbullaba": "desbullar",
    "desbullabais": "desbullar",
    "desbullaban": "desbullar",
    "desbullabas": "desbullar",
    "desbullad": "desbullar",
    "desbullada": "desbullar",
    "desbulladas": "desbullar",
    "desbullado": "desbullar",
    "desbulladores": "desbullador",
    "desbullados": "desbullar",
    "desbullamos": "desbullar",
    "desbullan": "desbullar",
    "desbullando": "desbullar",
    "desbullara": "desbullar",
    "desbullarais": "desbullar",
    "desbullaran": "desbullar",
    "desbullaras": "desbullar",
    "desbullare": "desbullar",
    "desbullareis": "desbullar",
    "desbullaremos": "desbullar",
    "desbullaren": "desbullar",
    "desbullares": "desbullar",
    "desbullaron": "desbullar",
    "desbullará": "desbullar",
    "desbullarán": "desbullar",
    "desbullarás": "desbullar",
    "desbullaré": "desbullar",
    "desbullaréis": "desbullar",
    "desbullaría": "desbullar",
    "desbullaríais": "desbullar",
    "desbullaríamos": "desbullar",
    "desbullarían": "desbullar",
    "desbullarías": "desbullar",
    "desbullas": "desbullar",
    "desbullase": "desbullar",
    "desbullaseis": "desbullar",
    "desbullasen": "desbullar",
    "desbullases": "desbullar",
    "desbullaste": "desbullar",
    "desbullasteis": "desbullar",
    "desbulle": "desbullar",
    "desbullemos": "desbullar",
    "desbullen": "desbullar",
    "desbulles": "desbullar",
    "desbullo": "desbullar",
    "desbullábamos": "desbullar",
    "desbulláis": "desbullar",
    "desbulláramos": "desbullar",
    "desbulláremos": "desbullar",
    "desbullásemos": "desbullar",
    "desbullé": "desbullar",
    "desbulléis": "desbullar",
    "desbulló": "desbullar",
    "descabala": "descabalar",
    "descabalaba": "descabalar",
    "descabalabais": "descabalar",
    "descabalaban": "descabalar",
    "descabalabas": "descabalar",
    "descabalad": "descabalar",
    "descabalada": "descabalar",
    "descabaladas": "descabalar",
    "descabalado": "descabalar",
    "descabalados": "descabalar",
    "descabalamientos": "descabalamiento",
    "descabalamos": "descabalar",
    "descabalan": "descabalar",
    "descabalando": "descabalar",
    "descabalara": "descabalar",
    "descabalarais": "descabalar",
    "descabalaran": "descabalar",
    "descabalaras": "descabalar",
    "descabalare": "descabalar",
    "descabalareis": "descabalar",
    "descabalaremos": "descabalar",
    "descabalaren": "descabalar",
    "descabalares": "descabalar",
    "descabalaron": "descabalar",
    "descabalará": "descabalar",
    "descabalarán": "descabalar",
    "descabalarás": "descabalar",
    "descabalaré": "descabalar",
    "descabalaréis": "descabalar",
    "descabalaría": "descabalar",
    "descabalaríais": "descabalar",
    "descabalaríamos": "descabalar",
    "descabalarían": "descabalar",
    "descabalarías": "descabalar",
    "descabalas": "descabalar",
    "descabalase": "descabalar",
    "descabalaseis": "descabalar",
    "descabalasen": "descabalar",
    "descabalases": "descabalar",
    "descabalaste": "descabalar",
    "descabalasteis": "descabalar",
    "descabale": "descabalar",
    "descabalemos": "descabalar",
    "descabalen": "descabalar",
    "descabales": "descabalar",
    "descabalga": "descabalgar",
    "descabalgaba": "descabalgar",
    "descabalgabais": "descabalgar",
    "descabalgaban": "descabalgar",
    "descabalgabas": "descabalgar",
    "descabalgad": "descabalgar",
    "descabalgada": "descabalgar",
    "descabalgadas": "descabalgar",
    "descabalgado": "descabalgar",
    "descabalgados": "descabalgar",
    "descabalgaduras": "descabalgadura",
    "descabalgamientos": "descabalgamiento",
    "descabalgamos": "descabalgar",
    "descabalgan": "descabalgar",
    "descabalgando": "descabalgar",
    "descabalgara": "descabalgar",
    "descabalgarais": "descabalgar",
    "descabalgaran": "descabalgar",
    "descabalgaras": "descabalgar",
    "descabalgare": "descabalgar",
    "descabalgareis": "descabalgar",
    "descabalgaremos": "descabalgar",
    "descabalgaren": "descabalgar",
    "descabalgares": "descabalgar",
    "descabalgaron": "descabalgar",
    "descabalgará": "descabalgar",
    "descabalgarán": "descabalgar",
    "descabalgarás": "descabalgar",
    "descabalgaré": "descabalgar",
    "descabalgaréis": "descabalgar",
    "descabalgaría": "descabalgar",
    "descabalgaríais": "descabalgar",
    "descabalgaríamos": "descabalgar",
    "descabalgarían": "descabalgar",
    "descabalgarías": "descabalgar",
    "descabalgas": "descabalgar",
    "descabalgase": "descabalgar",
    "descabalgaseis": "descabalgar",
    "descabalgasen": "descabalgar",
    "descabalgases": "descabalgar",
    "descabalgaste": "descabalgar",
    "descabalgasteis": "descabalgar",
    "descabalgo": "descabalgar",
    "descabalgue": "descabalgar",
    "descabalguemos": "descabalgar",
    "descabalguen": "descabalgar",
    "descabalgues": "descabalgar",
    "descabalgué": "descabalgar",
    "descabalguéis": "descabalgar",
    "descabalgábamos": "descabalgar",
    "descabalgáis": "descabalgar",
    "descabalgáramos": "descabalgar",
    "descabalgáremos": "descabalgar",
    "descabalgásemos": "descabalgar",
    "descabalgó": "descabalgar",
    "descabalo": "descabalar",
    "descabalábamos": "descabalar",
    "descabaláis": "descabalar",
    "descabaláramos": "descabalar",
    "descabaláremos": "descabalar",
    "descabalásemos": "descabalar",
    "descabalé": "descabalar",
    "descabaléis": "descabalar",
    "descabaló": "descabalar",
    "descabece": "descabezar",
    "descabecemos": "descabezar",
    "descabecen": "descabezar",
    "descabeces": "descabezar",
    "descabecé": "descabezar",
    "descabecéis": "descabezar",
    "descabella": "descabellar",
    "descabellaba": "descabellar",
    "descabellabais": "descabellar",
    "descabellaban": "descabellar",
    "descabellabas": "descabellar",
    "descabellad": "descabellar",
    "descabellada": "descabellar",
    "descabelladas": "descabellar",
    "descabellado": "descabellar",
    "descabellados": "descabellar",
    "descabellamientos": "descabellamiento",
    "descabellamos": "descabellar",
    "descabellan": "descabellar",
    "descabellando": "descabellar",
    "descabellara": "descabellar",
    "descabellarais": "descabellar",
    "descabellaran": "descabellar",
    "descabellaras": "descabellar",
    "descabellare": "descabellar",
    "descabellareis": "descabellar",
    "descabellaremos": "descabellar",
    "descabellaren": "descabellar",
    "descabellares": "descabellar",
    "descabellaron": "descabellar",
    "descabellará": "descabellar",
    "descabellarán": "descabellar",
    "descabellarás": "descabellar",
    "descabellaré": "descabellar",
    "descabellaréis": "descabellar",
    "descabellaría": "descabellar",
    "descabellaríais": "descabellar",
    "descabellaríamos": "descabellar",
    "descabellarían": "descabellar",
    "descabellarías": "descabellar",
    "descabellas": "descabellar",
    "descabellase": "descabellar",
    "descabellaseis": "descabellar",
    "descabellasen": "descabellar",
    "descabellases": "descabellar",
    "descabellaste": "descabellar",
    "descabellasteis": "descabellar",
    "descabelle": "descabellar",
    "descabellemos": "descabellar",
    "descabellen": "descabellar",
    "descabelles": "descabellar",
    "descabello": "descabellar",
    "descabellos": "descabello",
    "descabellábamos": "descabellar",
    "descabelláis": "descabellar",
    "descabelláramos": "descabellar",
    "descabelláremos": "descabellar",
    "descabellásemos": "descabellar",
    "descabellé": "descabellar",
    "descabelléis": "descabellar",
    "descabelló": "descabellar",
    "descabeza": "descabezar",
    "descabezaba": "descabezar",
    "descabezabais": "descabezar",
    "descabezaban": "descabezar",
    "descabezabas": "descabezar",
    "descabezad": "descabezar",
    "descabezada": "descabezar",
    "descabezadas": "descabezar",
    "descabezado": "descabezar",
    "descabezados": "descabezar",
    "descabezamientos": "descabezamiento",
    "descabezamos": "descabezar",
    "descabezan": "descabezar",
    "descabezando": "descabezar",
    "descabezara": "descabezar",
    "descabezarais": "descabezar",
    "descabezaran": "descabezar",
    "descabezaras": "descabezar",
    "descabezare": "descabezar",
    "descabezareis": "descabezar",
    "descabezaremos": "descabezar",
    "descabezaren": "descabezar",
    "descabezares": "descabezar",
    "descabezaron": "descabezar",
    "descabezará": "descabezar",
    "descabezarán": "descabezar",
    "descabezarás": "descabezar",
    "descabezaré": "descabezar",
    "descabezaréis": "descabezar",
    "descabezaría": "descabezar",
    "descabezaríais": "descabezar",
    "descabezaríamos": "descabezar",
    "descabezarían": "descabezar",
    "descabezarías": "descabezar",
    "descabezas": "descabezar",
    "descabezase": "descabezar",
    "descabezaseis": "descabezar",
    "descabezasen": "descabezar",
    "descabezases": "descabezar",
    "descabezaste": "descabezar",
    "descabezasteis": "descabezar",
    "descabezo": "descabezar",
    "descabezábamos": "descabezar",
    "descabezáis": "descabezar",
    "descabezáramos": "descabezar",
    "descabezáremos": "descabezar",
    "descabezásemos": "descabezar",
    "descabezó": "descabezar",
    "descabrita": "descabritar",
    "descabritaba": "descabritar",
    "descabritabais": "descabritar",
    "descabritaban": "descabritar",
    "descabritabas": "descabritar",
    "descabritad": "descabritar",
    "descabritada": "descabritar",
    "descabritadas": "descabritar",
    "descabritado": "descabritar",
    "descabritados": "descabritar",
    "descabritamos": "descabritar",
    "descabritan": "descabritar",
    "descabritando": "descabritar",
    "descabritara": "descabritar",
    "descabritarais": "descabritar",
    "descabritaran": "descabritar",
    "descabritaras": "descabritar",
    "descabritare": "descabritar",
    "descabritareis": "descabritar",
    "descabritaremos": "descabritar",
    "descabritaren": "descabritar",
    "descabritares": "descabritar",
    "descabritaron": "descabritar",
    "descabritará": "descabritar",
    "descabritarán": "descabritar",
    "descabritarás": "descabritar",
    "descabritaré": "descabritar",
    "descabritaréis": "descabritar",
    "descabritaría": "descabritar",
    "descabritaríais": "descabritar",
    "descabritaríamos": "descabritar",
    "descabritarían": "descabritar",
    "descabritarías": "descabritar",
    "descabritas": "descabritar",
    "descabritase": "descabritar",
    "descabritaseis": "descabritar",
    "descabritasen": "descabritar",
    "descabritases": "descabritar",
    "descabritaste": "descabritar",
    "descabritasteis": "descabritar",
    "descabrite": "descabritar",
    "descabritemos": "descabritar",
    "descabriten": "descabritar",
    "descabrites": "descabritar",
    "descabrito": "descabritar",
    "descabritábamos": "descabritar",
    "descabritáis": "descabritar",
    "descabritáramos": "descabritar",
    "descabritáremos": "descabritar",
    "descabritásemos": "descabritar",
    "descabrité": "descabritar",
    "descabritéis": "descabritar",
    "descabritó": "descabritar",
    "descadera": "descaderar",
    "descaderaba": "descaderar",
    "descaderabais": "descaderar",
    "descaderaban": "descaderar",
    "descaderabas": "descaderar",
    "descaderad": "descaderar",
    "descaderada": "descaderar",
    "descaderadas": "descaderar",
    "descaderado": "descaderar",
    "descaderados": "descaderar",
    "descaderamos": "descaderar",
    "descaderan": "descaderar",
    "descaderando": "descaderar",
    "descaderara": "descaderar",
    "descaderarais": "descaderar",
    "descaderaran": "descaderar",
    "descaderaras": "descaderar",
    "descaderare": "descaderar",
    "descaderareis": "descaderar",
    "descaderaremos": "descaderar",
    "descaderaren": "descaderar",
    "descaderares": "descaderar",
    "descaderaron": "descaderar",
    "descaderará": "descaderar",
    "descaderarán": "descaderar",
    "descaderarás": "descaderar",
    "descaderaré": "descaderar",
    "descaderaréis": "descaderar",
    "descaderaría": "descaderar",
    "descaderaríais": "descaderar",
    "descaderaríamos": "descaderar",
    "descaderarían": "descaderar",
    "descaderarías": "descaderar",
    "descaderas": "descaderar",
    "descaderase": "descaderar",
    "descaderaseis": "descaderar",
    "descaderasen": "descaderar",
    "descaderases": "descaderar",
    "descaderaste": "descaderar",
    "descaderasteis": "descaderar",
    "descadere": "descaderar",
    "descaderemos": "descaderar",
    "descaderen": "descaderar",
    "descaderes": "descaderar",
    "descadero": "descaderar",
    "descaderábamos": "descaderar",
    "descaderáis": "descaderar",
    "descaderáramos": "descaderar",
    "descaderáremos": "descaderar",
    "descaderásemos": "descaderar",
    "descaderé": "descaderar",
    "descaderéis": "descaderar",
    "descaderó": "descaderar",
    "descadilla": "descadillar",
    "descadillaba": "descadillar",
    "descadillabais": "descadillar",
    "descadillaban": "descadillar",
    "descadillabas": "descadillar",
    "descadillad": "descadillar",
    "descadillada": "descadillar",
    "descadilladas": "descadillar",
    "descadillado": "descadillar",
    "descadillados": "descadillar",
    "descadillamos": "descadillar",
    "descadillan": "descadillar",
    "descadillando": "descadillar",
    "descadillara": "descadillar",
    "descadillarais": "descadillar",
    "descadillaran": "descadillar",
    "descadillaras": "descadillar",
    "descadillare": "descadillar",
    "descadillareis": "descadillar",
    "descadillaremos": "descadillar",
    "descadillaren": "descadillar",
    "descadillares": "descadillar",
    "descadillaron": "descadillar",
    "descadillará": "descadillar",
    "descadillarán": "descadillar",
    "descadillarás": "descadillar",
    "descadillaré": "descadillar",
    "descadillaréis": "descadillar",
    "descadillaría": "descadillar",
    "descadillaríais": "descadillar",
    "descadillaríamos": "descadillar",
    "descadillarían": "descadillar",
    "descadillarías": "descadillar",
    "descadillas": "descadillar",
    "descadillase": "descadillar",
    "descadillaseis": "descadillar",
    "descadillasen": "descadillar",
    "descadillases": "descadillar",
    "descadillaste": "descadillar",
    "descadillasteis": "descadillar",
    "descadille": "descadillar",
    "descadillemos": "descadillar",
    "descadillen": "descadillar",
    "descadilles": "descadillar",
    "descadillo": "descadillar",
    "descadillábamos": "descadillar",
    "descadilláis": "descadillar",
    "descadilláramos": "descadillar",
    "descadilláremos": "descadillar",
    "descadillásemos": "descadillar",
    "descadillé": "descadillar",
    "descadilléis": "descadillar",
    "descadilló": "descadillar",
    "descaece": "descaecer",
    "descaeced": "descaecer",
    "descaecemos": "descaecer",
    "descaecen": "descaecer",
    "descaeceremos": "descaecer",
    "descaecerá": "descaecer",
    "descaecerán": "descaecer",
    "descaecerás": "descaecer",
    "descaeceré": "descaecer",
    "descaeceréis": "descaecer",
    "descaecería": "descaecer",
    "descaeceríais": "descaecer",
    "descaeceríamos": "descaecer",
    "descaecerían": "descaecer",
    "descaecerías": "descaecer",
    "descaeces": "descaecer",
    "descaecida": "descaecer",
    "descaecidas": "descaecer",
    "descaecido": "descaecer",
    "descaecidos": "descaecer",
    "descaeciendo": "descaecer",
    "descaeciera": "descaecer",
    "descaecierais": "descaecer",
    "descaecieran": "descaecer",
    "descaecieras": "descaecer",
    "descaeciere": "descaecer",
    "descaeciereis": "descaecer",
    "descaecieren": "descaecer",
    "descaecieres": "descaecer",
    "descaecieron": "descaecer",
    "descaeciese": "descaecer",
    "descaecieseis": "descaecer",
    "descaeciesen": "descaecer",
    "descaecieses": "descaecer",
    "descaecimos": "descaecer",
    "descaeciste": "descaecer",
    "descaecisteis": "descaecer",
    "descaeciéramos": "descaecer",
    "descaeciéremos": "descaecer",
    "descaeciésemos": "descaecer",
    "descaeció": "descaecer",
    "descaecéis": "descaecer",
    "descaecí": "descaecer",
    "descaecía": "descaecer",
    "descaecíais": "descaecer",
    "descaecíamos": "descaecer",
    "descaecían": "descaecer",
    "descaecías": "descaecer",
    "descaezca": "descaecer",
    "descaezcamos": "descaecer",
    "descaezcan": "descaecer",
    "descaezcas": "descaecer",
    "descaezco": "descaecer",
    "descaezcáis": "descaecer",
    "descafeinaba": "descafeinar",
    "descafeinabais": "descafeinar",
    "descafeinaban": "descafeinar",
    "descafeinabas": "descafeinar",
    "descafeinaciones": "descafeinación",
    "descafeinad": "descafeinar",
    "descafeinada": "descafeinar",
    "descafeinadas": "descafeinar",
    "descafeinado": "descafeinar",
    "descafeinados": "descafeinar",
    "descafeinamos": "descafeinar",
    "descafeinando": "descafeinar",
    "descafeinara": "descafeinar",
    "descafeinarais": "descafeinar",
    "descafeinaran": "descafeinar",
    "descafeinaras": "descafeinar",
    "descafeinare": "descafeinar",
    "descafeinareis": "descafeinar",
    "descafeinaremos": "descafeinar",
    "descafeinaren": "descafeinar",
    "descafeinares": "descafeinar",
    "descafeinaron": "descafeinar",
    "descafeinará": "descafeinar",
    "descafeinarán": "descafeinar",
    "descafeinarás": "descafeinar",
    "descafeinaré": "descafeinar",
    "descafeinaréis": "descafeinar",
    "descafeinaría": "descafeinar",
    "descafeinaríais": "descafeinar",
    "descafeinaríamos": "descafeinar",
    "descafeinarían": "descafeinar",
    "descafeinarías": "descafeinar",
    "descafeinase": "descafeinar",
    "descafeinaseis": "descafeinar",
    "descafeinasen": "descafeinar",
    "descafeinases": "descafeinar",
    "descafeinaste": "descafeinar",
    "descafeinasteis": "descafeinar",
    "descafeinemos": "descafeinar",
    "descafeinábamos": "descafeinar",
    "descafeináis": "descafeinar",
    "descafeináramos": "descafeinar",
    "descafeináremos": "descafeinar",
    "descafeinásemos": "descafeinar",
    "descafeiné": "descafeinar",
    "descafeinéis": "descafeinar",
    "descafeinó": "descafeinar",
    "descafeína": "descafeinar",
    "descafeínan": "descafeinar",
    "descafeínas": "descafeinar",
    "descafeíne": "descafeinar",
    "descafeínen": "descafeinar",
    "descafeínes": "descafeinar",
    "descafeíno": "descafeinar",
    "descalabace": "descalabazar",
    "descalabacemos": "descalabazar",
    "descalabacen": "descalabazar",
    "descalabaces": "descalabazar",
    "descalabacé": "descalabazar",
    "descalabacéis": "descalabazar",
    "descalabaza": "descalabazar",
    "descalabazaba": "descalabazar",
    "descalabazabais": "descalabazar",
    "descalabazaban": "descalabazar",
    "descalabazabas": "descalabazar",
    "descalabazad": "descalabazar",
    "descalabazada": "descalabazar",
    "descalabazadas": "descalabazar",
    "descalabazado": "descalabazar",
    "descalabazados": "descalabazar",
    "descalabazamos": "descalabazar",
    "descalabazan": "descalabazar",
    "descalabazando": "descalabazar",
    "descalabazara": "descalabazar",
    "descalabazarais": "descalabazar",
    "descalabazaran": "descalabazar",
    "descalabazaras": "descalabazar",
    "descalabazare": "descalabazar",
    "descalabazareis": "descalabazar",
    "descalabazaremos": "descalabazar",
    "descalabazaren": "descalabazar",
    "descalabazares": "descalabazar",
    "descalabazaron": "descalabazar",
    "descalabazará": "descalabazar",
    "descalabazarán": "descalabazar",
    "descalabazarás": "descalabazar",
    "descalabazaré": "descalabazar",
    "descalabazaréis": "descalabazar",
    "descalabazaría": "descalabazar",
    "descalabazaríais": "descalabazar",
    "descalabazaríamos": "descalabazar",
    "descalabazarían": "descalabazar",
    "descalabazarías": "descalabazar",
    "descalabazas": "descalabazar",
    "descalabazase": "descalabazar",
    "descalabazaseis": "descalabazar",
    "descalabazasen": "descalabazar",
    "descalabazases": "descalabazar",
    "descalabazaste": "descalabazar",
    "descalabazasteis": "descalabazar",
    "descalabazo": "descalabazar",
    "descalabazábamos": "descalabazar",
    "descalabazáis": "descalabazar",
    "descalabazáramos": "descalabazar",
    "descalabazáremos": "descalabazar",
    "descalabazásemos": "descalabazar",
    "descalabazó": "descalabazar",
    "descalabra": "descalabrar",
    "descalabraba": "descalabrar",
    "descalabrabais": "descalabrar",
    "descalabraban": "descalabrar",
    "descalabrabas": "descalabrar",
    "descalabrad": "descalabrar",
    "descalabrada": "descalabrar",
    "descalabradas": "descalabrar",
    "descalabrado": "descalabrar",
    "descalabrados": "descalabrar",
    "descalabraduras": "descalabradura",
    "descalabramos": "descalabrar",
    "descalabran": "descalabrar",
    "descalabrando": "descalabrar",
    "descalabrara": "descalabrar",
    "descalabrarais": "descalabrar",
    "descalabraran": "descalabrar",
    "descalabraras": "descalabrar",
    "descalabrare": "descalabrar",
    "descalabrareis": "descalabrar",
    "descalabraremos": "descalabrar",
    "descalabraren": "descalabrar",
    "descalabrares": "descalabrar",
    "descalabraron": "descalabrar",
    "descalabrará": "descalabrar",
    "descalabrarán": "descalabrar",
    "descalabrarás": "descalabrar",
    "descalabraré": "descalabrar",
    "descalabraréis": "descalabrar",
    "descalabraría": "descalabrar",
    "descalabraríais": "descalabrar",
    "descalabraríamos": "descalabrar",
    "descalabrarían": "descalabrar",
    "descalabrarías": "descalabrar",
    "descalabras": "descalabrar",
    "descalabrase": "descalabrar",
    "descalabraseis": "descalabrar",
    "descalabrasen": "descalabrar",
    "descalabrases": "descalabrar",
    "descalabraste": "descalabrar",
    "descalabrasteis": "descalabrar",
    "descalabre": "descalabrar",
    "descalabremos": "descalabrar",
    "descalabren": "descalabrar",
    "descalabres": "descalabrar",
    "descalabro": "descalabrar",
    "descalabros": "descalabro",
    "descalabrábamos": "descalabrar",
    "descalabráis": "descalabrar",
    "descalabráramos": "descalabrar",
    "descalabráremos": "descalabrar",
    "descalabrásemos": "descalabrar",
    "descalabré": "descalabrar",
    "descalabréis": "descalabrar",
    "descalabró": "descalabrar",
    "descalcadores": "descalcador",
    "descalce": "descalzar",
    "descalceces": "descalcez",
    "descalcemos": "descalzar",
    "descalcen": "descalzar",
    "descalces": "descalzar",
    "descalcifica": "descalcificar",
    "descalcificaba": "descalcificar",
    "descalcificabais": "descalcificar",
    "descalcificaban": "descalcificar",
    "descalcificabas": "descalcificar",
    "descalcificaciones": "descalcificación",
    "descalcificad": "descalcificar",
    "descalcificada": "descalcificar",
    "descalcificadas": "descalcificar",
    "descalcificado": "descalcificar",
    "descalcificados": "descalcificar",
    "descalcificamos": "descalcificar",
    "descalcifican": "descalcificar",
    "descalcificando": "descalcificar",
    "descalcificara": "descalcificar",
    "descalcificarais": "descalcificar",
    "descalcificaran": "descalcificar",
    "descalcificaras": "descalcificar",
    "descalcificare": "descalcificar",
    "descalcificareis": "descalcificar",
    "descalcificaremos": "descalcificar",
    "descalcificaren": "descalcificar",
    "descalcificares": "descalcificar",
    "descalcificaron": "descalcificar",
    "descalcificará": "descalcificar",
    "descalcificarán": "descalcificar",
    "descalcificarás": "descalcificar",
    "descalcificaré": "descalcificar",
    "descalcificaréis": "descalcificar",
    "descalcificaría": "descalcificar",
    "descalcificaríais": "descalcificar",
    "descalcificaríamos": "descalcificar",
    "descalcificarían": "descalcificar",
    "descalcificarías": "descalcificar",
    "descalcificas": "descalcificar",
    "descalcificase": "descalcificar",
    "descalcificaseis": "descalcificar",
    "descalcificasen": "descalcificar",
    "descalcificases": "descalcificar",
    "descalcificaste": "descalcificar",
    "descalcificasteis": "descalcificar",
    "descalcifico": "descalcificar",
    "descalcificábamos": "descalcificar",
    "descalcificáis": "descalcificar",
    "descalcificáramos": "descalcificar",
    "descalcificáremos": "descalcificar",
    "descalcificásemos": "descalcificar",
    "descalcificó": "descalcificar",
    "descalcifique": "descalcificar",
    "descalcifiquemos": "descalcificar",
    "descalcifiquen": "descalcificar",
    "descalcifiques": "descalcificar",
    "descalcifiqué": "descalcificar",
    "descalcifiquéis": "descalcificar",
    "descalcé": "descalzar",
    "descalcéis": "descalzar",
    "descalifica": "descalificar",
    "descalificaba": "descalificar",
    "descalificabais": "descalificar",
    "descalificaban": "descalificar",
    "descalificabas": "descalificar",
    "descalificaciones": "descalificación",
    "descalificad": "descalificar",
    "descalificada": "descalificar",
    "descalificadas": "descalificar",
    "descalificado": "descalificar",
    "descalificados": "descalificar",
    "descalificamos": "descalificar",
    "descalifican": "descalificar",
    "descalificando": "descalificar",
    "descalificantes": "descalificante",
    "descalificara": "descalificar",
    "descalificarais": "descalificar",
    "descalificaran": "descalificar",
    "descalificaras": "descalificar",
    "descalificare": "descalificar",
    "descalificareis": "descalificar",
    "descalificaremos": "descalificar",
    "descalificaren": "descalificar",
    "descalificares": "descalificar",
    "descalificaron": "descalificar",
    "descalificará": "descalificar",
    "descalificarán": "descalificar",
    "descalificarás": "descalificar",
    "descalificaré": "descalificar",
    "descalificaréis": "descalificar",
    "descalificaría": "descalificar",
    "descalificaríais": "descalificar",
    "descalificaríamos": "descalificar",
    "descalificarían": "descalificar",
    "descalificarías": "descalificar",
    "descalificas": "descalificar",
    "descalificase": "descalificar",
    "descalificaseis": "descalificar",
    "descalificasen": "descalificar",
    "descalificases": "descalificar",
    "descalificaste": "descalificar",
    "descalificasteis": "descalificar",
    "descalifico": "descalificar",
    "descalificábamos": "descalificar",
    "descalificáis": "descalificar",
    "descalificáramos": "descalificar",
    "descalificáremos": "descalificar",
    "descalificásemos": "descalificar",
    "descalificó": "descalificar",
    "descalifique": "descalificar",
    "descalifiquemos": "descalificar",
    "descalifiquen": "descalificar",
    "descalifiques": "descalificar",
    "descalifiqué": "descalificar",
    "descalifiquéis": "descalificar",
    "descalostrada": "descalostrado",
    "descalostradas": "descalostrado",
    "descalostrados": "descalostrado",
    "descalza": "descalzo",
    "descalzaba": "descalzar",
    "descalzabais": "descalzar",
    "descalzaban": "descalzar",
    "descalzabas": "descalzar",
    "descalzad": "descalzar",
    "descalzada": "descalzar",
    "descalzadas": "descalzar",
    "descalzado": "descalzar",
    "descalzados": "descalzar",
    "descalzamos": "descalzar",
    "descalzan": "descalzar",
    "descalzando": "descalzar",
    "descalzara": "descalzar",
    "descalzarais": "descalzar",
    "descalzaran": "descalzar",
    "descalzaras": "descalzar",
    "descalzare": "descalzar",
    "descalzareis": "descalzar",
    "descalzaremos": "descalzar",
    "descalzaren": "descalzar",
    "descalzares": "descalzar",
    "descalzaron": "descalzar",
    "descalzará": "descalzar",
    "descalzarán": "descalzar",
    "descalzarás": "descalzar",
    "descalzaré": "descalzar",
    "descalzaréis": "descalzar",
    "descalzaría": "descalzar",
    "descalzaríais": "descalzar",
    "descalzaríamos": "descalzar",
    "descalzarían": "descalzar",
    "descalzarías": "descalzar",
    "descalzas": "descalzo",
    "descalzase": "descalzar",
    "descalzaseis": "descalzar",
    "descalzasen": "descalzar",
    "descalzases": "descalzar",
    "descalzaste": "descalzar",
    "descalzasteis": "descalzar",
    "descalzo": "descalzar",
    "descalzos": "descalzo",
    "descalzábamos": "descalzar",
    "descalzáis": "descalzar",
    "descalzáramos": "descalzar",
    "descalzáremos": "descalzar",
    "descalzásemos": "descalzar",
    "descalzó": "descalzar",
    "descama": "descamar",
    "descamaba": "descamar",
    "descamabais": "descamar",
    "descamaban": "descamar",
    "descamabas": "descamar",
    "descamaciones": "descamación",
    "descamad": "descamar",
    "descamada": "descamar",
    "descamadas": "descamar",
    "descamado": "descamar",
    "descamados": "descamar",
    "descamamos": "descamar",
    "descaman": "descamar",
    "descamando": "descamar",
    "descamara": "descamar",
    "descamarais": "descamar",
    "descamaran": "descamar",
    "descamaras": "descamar",
    "descamare": "descamar",
    "descamareis": "descamar",
    "descamaremos": "descamar",
    "descamaren": "descamar",
    "descamares": "descamar",
    "descamaron": "descamar",
    "descamará": "descamar",
    "descamarán": "descamar",
    "descamarás": "descamar",
    "descamaré": "descamar",
    "descamaréis": "descamar",
    "descamaría": "descamar",
    "descamaríais": "descamar",
    "descamaríamos": "descamar",
    "descamarían": "descamar",
    "descamarías": "descamar",
    "descamas": "descamar",
    "descamase": "descamar",
    "descamaseis": "descamar",
    "descamasen": "descamar",
    "descamases": "descamar",
    "descamaste": "descamar",
    "descamasteis": "descamar",
    "descamativa": "descamativo",
    "descamativas": "descamativo",
    "descamativos": "descamativo",
    "descambia": "descambiar",
    "descambiaba": "descambiar",
    "descambiabais": "descambiar",
    "descambiaban": "descambiar",
    "descambiabas": "descambiar",
    "descambiad": "descambiar",
    "descambiada": "descambiar",
    "descambiadas": "descambiar",
    "descambiado": "descambiar",
    "descambiados": "descambiar",
    "descambiamos": "descambiar",
    "descambian": "descambiar",
    "descambiando": "descambiar",
    "descambiara": "descambiar",
    "descambiarais": "descambiar",
    "descambiaran": "descambiar",
    "descambiaras": "descambiar",
    "descambiare": "descambiar",
    "descambiareis": "descambiar",
    "descambiaremos": "descambiar",
    "descambiaren": "descambiar",
    "descambiares": "descambiar",
    "descambiaron": "descambiar",
    "descambiará": "descambiar",
    "descambiarán": "descambiar",
    "descambiarás": "descambiar",
    "descambiaré": "descambiar",
    "descambiaréis": "descambiar",
    "descambiaría": "descambiar",
    "descambiaríais": "descambiar",
    "descambiaríamos": "descambiar",
    "descambiarían": "descambiar",
    "descambiarías": "descambiar",
    "descambias": "descambiar",
    "descambiase": "descambiar",
    "descambiaseis": "descambiar",
    "descambiasen": "descambiar",
    "descambiases": "descambiar",
    "descambiaste": "descambiar",
    "descambiasteis": "descambiar",
    "descambie": "descambiar",
    "descambiemos": "descambiar",
    "descambien": "descambiar",
    "descambies": "descambiar",
    "descambio": "descambiar",
    "descambiábamos": "descambiar",
    "descambiáis": "descambiar",
    "descambiáramos": "descambiar",
    "descambiáremos": "descambiar",
    "descambiásemos": "descambiar",
    "descambié": "descambiar",
    "descambiéis": "descambiar",
    "descambió": "descambiar",
    "descame": "descamar",
    "descamemos": "descamar",
    "descamen": "descamar",
    "descames": "descamar",
    "descamina": "descaminar",
    "descaminaba": "descaminar",
    "descaminabais": "descaminar",
    "descaminaban": "descaminar",
    "descaminabas": "descaminar",
    "descaminad": "descaminar",
    "descaminada": "descaminar",
    "descaminadas": "descaminar",
    "descaminado": "descaminar",
    "descaminados": "descaminar",
    "descaminamos": "descaminar",
    "descaminan": "descaminar",
    "descaminando": "descaminar",
    "descaminara": "descaminar",
    "descaminarais": "descaminar",
    "descaminaran": "descaminar",
    "descaminaras": "descaminar",
    "descaminare": "descaminar",
    "descaminareis": "descaminar",
    "descaminaremos": "descaminar",
    "descaminaren": "descaminar",
    "descaminares": "descaminar",
    "descaminaron": "descaminar",
    "descaminará": "descaminar",
    "descaminarán": "descaminar",
    "descaminarás": "descaminar",
    "descaminaré": "descaminar",
    "descaminaréis": "descaminar",
    "descaminaría": "descaminar",
    "descaminaríais": "descaminar",
    "descaminaríamos": "descaminar",
    "descaminarían": "descaminar",
    "descaminarías": "descaminar",
    "descaminas": "descaminar",
    "descaminase": "descaminar",
    "descaminaseis": "descaminar",
    "descaminasen": "descaminar",
    "descaminases": "descaminar",
    "descaminaste": "descaminar",
    "descaminasteis": "descaminar",
    "descamine": "descaminar",
    "descaminemos": "descaminar",
    "descaminen": "descaminar",
    "descamines": "descaminar",
    "descamino": "descaminar",
    "descaminos": "descamino",
    "descaminábamos": "descaminar",
    "descamináis": "descaminar",
    "descamináramos": "descaminar",
    "descamináremos": "descaminar",
    "descaminásemos": "descaminar",
    "descaminé": "descaminar",
    "descaminéis": "descaminar",
    "descaminó": "descaminar",
    "descamisa": "descamisar",
    "descamisaba": "descamisar",
    "descamisabais": "descamisar",
    "descamisaban": "descamisar",
    "descamisabas": "descamisar",
    "descamisad": "descamisar",
    "descamisada": "descamisar",
    "descamisadas": "descamisar",
    "descamisado": "descamisar",
    "descamisados": "descamisar",
    "descamisamientos": "descamisamiento",
    "descamisamos": "descamisar",
    "descamisan": "descamisar",
    "descamisando": "descamisar",
    "descamisara": "descamisar",
    "descamisarais": "descamisar",
    "descamisaran": "descamisar",
    "descamisaras": "descamisar",
    "descamisare": "descamisar",
    "descamisareis": "descamisar",
    "descamisaremos": "descamisar",
    "descamisaren": "descamisar",
    "descamisares": "descamisar",
    "descamisaron": "descamisar",
    "descamisará": "descamisar",
    "descamisarán": "descamisar",
    "descamisarás": "descamisar",
    "descamisaré": "descamisar",
    "descamisaréis": "descamisar",
    "descamisaría": "descamisar",
    "descamisaríais": "descamisar",
    "descamisaríamos": "descamisar",
    "descamisarían": "descamisar",
    "descamisarías": "descamisar",
    "descamisas": "descamisar",
    "descamisase": "descamisar",
    "descamisaseis": "descamisar",
    "descamisasen": "descamisar",
    "descamisases": "descamisar",
    "descamisaste": "descamisar",
    "descamisasteis": "descamisar",
    "descamise": "descamisar",
    "descamisemos": "descamisar",
    "descamisen": "descamisar",
    "descamises": "descamisar",
    "descamiso": "descamisar",
    "descamisábamos": "descamisar",
    "descamisáis": "descamisar",
    "descamisáramos": "descamisar",
    "descamisáremos": "descamisar",
    "descamisásemos": "descamisar",
    "descamisé": "descamisar",
    "descamiséis": "descamisar",
    "descamisó": "descamisar",
    "descamo": "descamar",
    "descampa": "descampar",
    "descampaba": "descampar",
    "descampabais": "descampar",
    "descampaban": "descampar",
    "descampabas": "descampar",
    "descampad": "descampar",
    "descampada": "descampar",
    "descampadas": "descampar",
    "descampado": "descampar",
    "descampados": "descampar",
    "descampamos": "descampar",
    "descampan": "descampar",
    "descampando": "descampar",
    "descampara": "descampar",
    "descamparais": "descampar",
    "descamparan": "descampar",
    "descamparas": "descampar",
    "descampare": "descampar",
    "descampareis": "descampar",
    "descamparemos": "descampar",
    "descamparen": "descampar",
    "descampares": "descampar",
    "descamparon": "descampar",
    "descampará": "descampar",
    "descamparán": "descampar",
    "descamparás": "descampar",
    "descamparé": "descampar",
    "descamparéis": "descampar",
    "descamparía": "descampar",
    "descamparíais": "descampar",
    "descamparíamos": "descampar",
    "descamparían": "descampar",
    "descamparías": "descampar",
    "descampas": "descampar",
    "descampase": "descampar",
    "descampaseis": "descampar",
    "descampasen": "descampar",
    "descampases": "descampar",
    "descampaste": "descampar",
    "descampasteis": "descampar",
    "descampe": "descampar",
    "descampemos": "descampar",
    "descampen": "descampar",
    "descampes": "descampar",
    "descampo": "descampar",
    "descampábamos": "descampar",
    "descampáis": "descampar",
    "descampáramos": "descampar",
    "descampáremos": "descampar",
    "descampásemos": "descampar",
    "descampé": "descampar",
    "descampéis": "descampar",
    "descampó": "descampar",
    "descamábamos": "descamar",
    "descamáis": "descamar",
    "descamáramos": "descamar",
    "descamáremos": "descamar",
    "descamásemos": "descamar",
    "descamé": "descamar",
    "descaméis": "descamar",
    "descamó": "descamar",
    "descansa": "descansar",
    "descansaba": "descansar",
    "descansabais": "descansar",
    "descansaban": "descansar",
    "descansabas": "descansar",
    "descansad": "descansar",
    "descansada": "descansar",
    "descansadas": "descansar",
    "descansaderos": "descansadero",
    "descansado": "descansar",
    "descansados": "descansar",
    "descansamos": "descansar",
    "descansan": "descansar",
    "descansando": "descansar",
    "descansara": "descansar",
    "descansarais": "descansar",
    "descansaran": "descansar",
    "descansaras": "descansar",
    "descansare": "descansar",
    "descansareis": "descansar",
    "descansaremos": "descansar",
    "descansaren": "descansar",
    "descansares": "descansar",
    "descansaron": "descansar",
    "descansará": "descansar",
    "descansarán": "descansar",
    "descansarás": "descansar",
    "descansaré": "descansar",
    "descansaréis": "descansar",
    "descansaría": "descansar",
    "descansaríais": "descansar",
    "descansaríamos": "descansar",
    "descansarían": "descansar",
    "descansarías": "descansar",
    "descansas": "descansar",
    "descansase": "descansar",
    "descansaseis": "descansar",
    "descansasen": "descansar",
    "descansases": "descansar",
    "descansaste": "descansar",
    "descansasteis": "descansar",
    "descanse": "descansar",
    "descansemos": "descansar",
    "descansen": "descansar",
    "descanses": "descansar",
    "descansillos": "descansillo",
    "descanso": "descansar",
    "descansos": "descanso",
    "descansábamos": "descansar",
    "descansáis": "descansar",
    "descansáramos": "descansar",
    "descansáremos": "descansar",
    "descansásemos": "descansar",
    "descansé": "descansar",
    "descanséis": "descansar",
    "descansó": "descansar",
    "descantilla": "descantillar",
    "descantillaba": "descantillar",
    "descantillabais": "descantillar",
    "descantillaban": "descantillar",
    "descantillabas": "descantillar",
    "descantillad": "descantillar",
    "descantillada": "descantillar",
    "descantilladas": "descantillar",
    "descantillado": "descantillar",
    "descantillados": "descantillar",
    "descantillamos": "descantillar",
    "descantillan": "descantillar",
    "descantillando": "descantillar",
    "descantillara": "descantillar",
    "descantillarais": "descantillar",
    "descantillaran": "descantillar",
    "descantillaras": "descantillar",
    "descantillare": "descantillar",
    "descantillareis": "descantillar",
    "descantillaremos": "descantillar",
    "descantillaren": "descantillar",
    "descantillares": "descantillar",
    "descantillaron": "descantillar",
    "descantillará": "descantillar",
    "descantillarán": "descantillar",
    "descantillarás": "descantillar",
    "descantillaré": "descantillar",
    "descantillaréis": "descantillar",
    "descantillaría": "descantillar",
    "descantillaríais": "descantillar",
    "descantillaríamos": "descantillar",
    "descantillarían": "descantillar",
    "descantillarías": "descantillar",
    "descantillas": "descantillar",
    "descantillase": "descantillar",
    "descantillaseis": "descantillar",
    "descantillasen": "descantillar",
    "descantillases": "descantillar",
    "descantillaste": "descantillar",
    "descantillasteis": "descantillar",
    "descantille": "descantillar",
    "descantillemos": "descantillar",
    "descantillen": "descantillar",
    "descantilles": "descantillar",
    "descantillo": "descantillar",
    "descantillones": "descantillón",
    "descantillábamos": "descantillar",
    "descantilláis": "descantillar",
    "descantilláramos": "descantillar",
    "descantilláremos": "descantillar",
    "descantillásemos": "descantillar",
    "descantillé": "descantillar",
    "descantilléis": "descantillar",
    "descantilló": "descantillar",
    "descantona": "descantonar",
    "descantonaba": "descantonar",
    "descantonabais": "descantonar",
    "descantonaban": "descantonar",
    "descantonabas": "descantonar",
    "descantonad": "descantonar",
    "descantonada": "descantonar",
    "descantonadas": "descantonar",
    "descantonado": "descantonar",
    "descantonados": "descantonar",
    "descantonamos": "descantonar",
    "descantonan": "descantonar",
    "descantonando": "descantonar",
    "descantonara": "descantonar",
    "descantonarais": "descantonar",
    "descantonaran": "descantonar",
    "descantonaras": "descantonar",
    "descantonare": "descantonar",
    "descantonareis": "descantonar",
    "descantonaremos": "descantonar",
    "descantonaren": "descantonar",
    "descantonares": "descantonar",
    "descantonaron": "descantonar",
    "descantonará": "descantonar",
    "descantonarán": "descantonar",
    "descantonarás": "descantonar",
    "descantonaré": "descantonar",
    "descantonaréis": "descantonar",
    "descantonaría": "descantonar",
    "descantonaríais": "descantonar",
    "descantonaríamos": "descantonar",
    "descantonarían": "descantonar",
    "descantonarías": "descantonar",
    "descantonas": "descantonar",
    "descantonase": "descantonar",
    "descantonaseis": "descantonar",
    "descantonasen": "descantonar",
    "descantonases": "descantonar",
    "descantonaste": "descantonar",
    "descantonasteis": "descantonar",
    "descantone": "descantonar",
    "descantonemos": "descantonar",
    "descantonen": "descantonar",
    "descantones": "descantonar",
    "descantono": "descantonar",
    "descantonábamos": "descantonar",
    "descantonáis": "descantonar",
    "descantonáramos": "descantonar",
    "descantonáremos": "descantonar",
    "descantonásemos": "descantonar",
    "descantoné": "descantonar",
    "descantonéis": "descantonar",
    "descantonó": "descantonar",
    "descaperuce": "descaperuzar",
    "descaperucemos": "descaperuzar",
    "descaperucen": "descaperuzar",
    "descaperuces": "descaperuzar",
    "descaperucé": "descaperuzar",
    "descaperucéis": "descaperuzar",
    "descaperuza": "descaperuzar",
    "descaperuzaba": "descaperuzar",
    "descaperuzabais": "descaperuzar",
    "descaperuzaban": "descaperuzar",
    "descaperuzabas": "descaperuzar",
    "descaperuzad": "descaperuzar",
    "descaperuzada": "descaperuzar",
    "descaperuzadas": "descaperuzar",
    "descaperuzado": "descaperuzar",
    "descaperuzados": "descaperuzar",
    "descaperuzamientos": "descaperuzamiento",
    "descaperuzamos": "descaperuzar",
    "descaperuzan": "descaperuzar",
    "descaperuzando": "descaperuzar",
    "descaperuzara": "descaperuzar",
    "descaperuzarais": "descaperuzar",
    "descaperuzaran": "descaperuzar",
    "descaperuzaras": "descaperuzar",
    "descaperuzare": "descaperuzar",
    "descaperuzareis": "descaperuzar",
    "descaperuzaremos": "descaperuzar",
    "descaperuzaren": "descaperuzar",
    "descaperuzares": "descaperuzar",
    "descaperuzaron": "descaperuzar",
    "descaperuzará": "descaperuzar",
    "descaperuzarán": "descaperuzar",
    "descaperuzarás": "descaperuzar",
    "descaperuzaré": "descaperuzar",
    "descaperuzaréis": "descaperuzar",
    "descaperuzaría": "descaperuzar",
    "descaperuzaríais": "descaperuzar",
    "descaperuzaríamos": "descaperuzar",
    "descaperuzarían": "descaperuzar",
    "descaperuzarías": "descaperuzar",
    "descaperuzas": "descaperuzar",
    "descaperuzase": "descaperuzar",
    "descaperuzaseis": "descaperuzar",
    "descaperuzasen": "descaperuzar",
    "descaperuzases": "descaperuzar",
    "descaperuzaste": "descaperuzar",
    "descaperuzasteis": "descaperuzar",
    "descaperuzo": "descaperuzar",
    "descaperuzos": "descaperuzo",
    "descaperuzábamos": "descaperuzar",
    "descaperuzáis": "descaperuzar",
    "descaperuzáramos": "descaperuzar",
    "descaperuzáremos": "descaperuzar",
    "descaperuzásemos": "descaperuzar",
    "descaperuzó": "descaperuzar",
    "descapirota": "descapirotar",
    "descapirotaba": "descapirotar",
    "descapirotabais": "descapirotar",
    "descapirotaban": "descapirotar",
    "descapirotabas": "descapirotar",
    "descapirotad": "descapirotar",
    "descapirotada": "descapirotar",
    "descapirotadas": "descapirotar",
    "descapirotado": "descapirotar",
    "descapirotados": "descapirotar",
    "descapirotamos": "descapirotar",
    "descapirotan": "descapirotar",
    "descapirotando": "descapirotar",
    "descapirotara": "descapirotar",
    "descapirotarais": "descapirotar",
    "descapirotaran": "descapirotar",
    "descapirotaras": "descapirotar",
    "descapirotare": "descapirotar",
    "descapirotareis": "descapirotar",
    "descapirotaremos": "descapirotar",
    "descapirotaren": "descapirotar",
    "descapirotares": "descapirotar",
    "descapirotaron": "descapirotar",
    "descapirotará": "descapirotar",
    "descapirotarán": "descapirotar",
    "descapirotarás": "descapirotar",
    "descapirotaré": "descapirotar",
    "descapirotaréis": "descapirotar",
    "descapirotaría": "descapirotar",
    "descapirotaríais": "descapirotar",
    "descapirotaríamos": "descapirotar",
    "descapirotarían": "descapirotar",
    "descapirotarías": "descapirotar",
    "descapirotas": "descapirotar",
    "descapirotase": "descapirotar",
    "descapirotaseis": "descapirotar",
    "descapirotasen": "descapirotar",
    "descapirotases": "descapirotar",
    "descapirotaste": "descapirotar",
    "descapirotasteis": "descapirotar",
    "descapirote": "descapirotar",
    "descapirotemos": "descapirotar",
    "descapiroten": "descapirotar",
    "descapirotes": "descapirotar",
    "descapiroto": "descapirotar",
    "descapirotábamos": "descapirotar",
    "descapirotáis": "descapirotar",
    "descapirotáramos": "descapirotar",
    "descapirotáremos": "descapirotar",
    "descapirotásemos": "descapirotar",
    "descapiroté": "descapirotar",
    "descapirotéis": "descapirotar",
    "descapirotó": "descapirotar",
    "descapitalizaciones": "descapitalización",
    "descapota": "descapotar",
    "descapotaba": "descapotar",
    "descapotabais": "descapotar",
    "descapotaban": "descapotar",
    "descapotabas": "descapotar",
    "descapotables": "descapotable",
    "descapotad": "descapotar",
    "descapotada": "descapotar",
    "descapotadas": "descapotar",
    "descapotado": "descapotar",
    "descapotados": "descapotar",
    "descapotamientos": "descapotamiento",
    "descapotamos": "descapotar",
    "descapotan": "descapotar",
    "descapotando": "descapotar",
    "descapotara": "descapotar",
    "descapotarais": "descapotar",
    "descapotaran": "descapotar",
    "descapotaras": "descapotar",
    "descapotare": "descapotar",
    "descapotareis": "descapotar",
    "descapotaremos": "descapotar",
    "descapotaren": "descapotar",
    "descapotares": "descapotar",
    "descapotaron": "descapotar",
    "descapotará": "descapotar",
    "descapotarán": "descapotar",
    "descapotarás": "descapotar",
    "descapotaré": "descapotar",
    "descapotaréis": "descapotar",
    "descapotaría": "descapotar",
    "descapotaríais": "descapotar",
    "descapotaríamos": "descapotar",
    "descapotarían": "descapotar",
    "descapotarías": "descapotar",
    "descapotas": "descapotar",
    "descapotase": "descapotar",
    "descapotaseis": "descapotar",
    "descapotasen": "descapotar",
    "descapotases": "descapotar",
    "descapotaste": "descapotar",
    "descapotasteis": "descapotar",
    "descapote": "descapotar",
    "descapotemos": "descapotar",
    "descapoten": "descapotar",
    "descapotes": "descapotar",
    "descapoto": "descapotar",
    "descapotábamos": "descapotar",
    "descapotáis": "descapotar",
    "descapotáramos": "descapotar",
    "descapotáremos": "descapotar",
    "descapotásemos": "descapotar",
    "descapoté": "descapotar",
    "descapotéis": "descapotar",
    "descapotó": "descapotar",
    "descara": "descarar",
    "descaraba": "descarar",
    "descarabais": "descarar",
    "descaraban": "descarar",
    "descarabas": "descarar",
    "descarad": "descarar",
    "descarada": "descarar",
    "descaradas": "descarar",
    "descarado": "descarar",
    "descarados": "descarar",
    "descaramos": "descarar",
    "descaran": "descarar",
    "descarando": "descarar",
    "descarara": "descarar",
    "descararais": "descarar",
    "descararan": "descarar",
    "descararas": "descarar",
    "descarare": "descarar",
    "descarareis": "descarar",
    "descararemos": "descarar",
    "descararen": "descarar",
    "descarares": "descarar",
    "descararon": "descarar",
    "descarará": "descarar",
    "descararán": "descarar",
    "descararás": "descarar",
    "descararé": "descarar",
    "descararéis": "descarar",
    "descararía": "descarar",
    "descararíais": "descarar",
    "descararíamos": "descarar",
    "descararían": "descarar",
    "descararías": "descarar",
    "descaras": "descarar",
    "descarase": "descarar",
    "descaraseis": "descarar",
    "descarasen": "descarar",
    "descarases": "descarar",
    "descaraste": "descarar",
    "descarasteis": "descarar",
    "descarbonata": "descarbonatar",
    "descarbonataba": "descarbonatar",
    "descarbonatabais": "descarbonatar",
    "descarbonataban": "descarbonatar",
    "descarbonatabas": "descarbonatar",
    "descarbonataciones": "descarbonatación",
    "descarbonatad": "descarbonatar",
    "descarbonatada": "descarbonatar",
    "descarbonatadas": "descarbonatar",
    "descarbonatado": "descarbonatar",
    "descarbonatados": "descarbonatar",
    "descarbonatamos": "descarbonatar",
    "descarbonatan": "descarbonatar",
    "descarbonatando": "descarbonatar",
    "descarbonatara": "descarbonatar",
    "descarbonatarais": "descarbonatar",
    "descarbonataran": "descarbonatar",
    "descarbonataras": "descarbonatar",
    "descarbonatare": "descarbonatar",
    "descarbonatareis": "descarbonatar",
    "descarbonataremos": "descarbonatar",
    "descarbonataren": "descarbonatar",
    "descarbonatares": "descarbonatar",
    "descarbonataron": "descarbonatar",
    "descarbonatará": "descarbonatar",
    "descarbonatarán": "descarbonatar",
    "descarbonatarás": "descarbonatar",
    "descarbonataré": "descarbonatar",
    "descarbonataréis": "descarbonatar",
    "descarbonataría": "descarbonatar",
    "descarbonataríais": "descarbonatar",
    "descarbonataríamos": "descarbonatar",
    "descarbonatarían": "descarbonatar",
    "descarbonatarías": "descarbonatar",
    "descarbonatas": "descarbonatar",
    "descarbonatase": "descarbonatar",
    "descarbonataseis": "descarbonatar",
    "descarbonatasen": "descarbonatar",
    "descarbonatases": "descarbonatar",
    "descarbonataste": "descarbonatar",
    "descarbonatasteis": "descarbonatar",
    "descarbonate": "descarbonatar",
    "descarbonatemos": "descarbonatar",
    "descarbonaten": "descarbonatar",
    "descarbonates": "descarbonatar",
    "descarbonato": "descarbonatar",
    "descarbonatábamos": "descarbonatar",
    "descarbonatáis": "descarbonatar",
    "descarbonatáramos": "descarbonatar",
    "descarbonatáremos": "descarbonatar",
    "descarbonatásemos": "descarbonatar",
    "descarbonaté": "descarbonatar",
    "descarbonatéis": "descarbonatar",
    "descarbonató": "descarbonatar",
    "descarbonotamientos": "descarbonotamiento",
    "descarboxilasas": "descarboxilasa",
    "descarbura": "descarburar",
    "descarburaba": "descarburar",
    "descarburabais": "descarburar",
    "descarburaban": "descarburar",
    "descarburabas": "descarburar",
    "descarburaciones": "descarburación",
    "descarburad": "descarburar",
    "descarburada": "descarburar",
    "descarburadas": "descarburar",
    "descarburado": "descarburar",
    "descarburados": "descarburar",
    "descarburamos": "descarburar",
    "descarburan": "descarburar",
    "descarburando": "descarburar",
    "descarburara": "descarburar",
    "descarburarais": "descarburar",
    "descarburaran": "descarburar",
    "descarburaras": "descarburar",
    "descarburare": "descarburar",
    "descarburareis": "descarburar",
    "descarburaremos": "descarburar",
    "descarburaren": "descarburar",
    "descarburares": "descarburar",
    "descarburaron": "descarburar",
    "descarburará": "descarburar",
    "descarburarán": "descarburar",
    "descarburarás": "descarburar",
    "descarburaré": "descarburar",
    "descarburaréis": "descarburar",
    "descarburaría": "descarburar",
    "descarburaríais": "descarburar",
    "descarburaríamos": "descarburar",
    "descarburarían": "descarburar",
    "descarburarías": "descarburar",
    "descarburas": "descarburar",
    "descarburase": "descarburar",
    "descarburaseis": "descarburar",
    "descarburasen": "descarburar",
    "descarburases": "descarburar",
    "descarburaste": "descarburar",
    "descarburasteis": "descarburar",
    "descarbure": "descarburar",
    "descarburemos": "descarburar",
    "descarburen": "descarburar",
    "descarbures": "descarburar",
    "descarburo": "descarburar",
    "descarburábamos": "descarburar",
    "descarburáis": "descarburar",
    "descarburáramos": "descarburar",
    "descarburáremos": "descarburar",
    "descarburásemos": "descarburar",
    "descarburé": "descarburar",
    "descarburéis": "descarburar",
    "descarburó": "descarburar",
    "descare": "descarar",
    "descaremos": "descarar",
    "descaren": "descarar",
    "descares": "descarar",
    "descarga": "descargar",
    "descargaba": "descargar",
    "descargabais": "descargar",
    "descargaban": "descargar",
    "descargabas": "descargar",
    "descargad": "descargar",
    "descargada": "descargar",
    "descargadas": "descargar",
    "descargaderos": "descargadero",
    "descargado": "descargar",
    "descargadores": "descargador",
    "descargados": "descargar",
    "descargamos": "descargar",
    "descargan": "descargar",
    "descargando": "descargar",
    "descargara": "descargar",
    "descargarais": "descargar",
    "descargaran": "descargar",
    "descargaras": "descargar",
    "descargare": "descargar",
    "descargareis": "descargar",
    "descargaremos": "descargar",
    "descargaren": "descargar",
    "descargares": "descargar",
    "descargaron": "descargar",
    "descargará": "descargar",
    "descargarán": "descargar",
    "descargarás": "descargar",
    "descargaré": "descargar",
    "descargaréis": "descargar",
    "descargaría": "descargar",
    "descargaríais": "descargar",
    "descargaríamos": "descargar",
    "descargarían": "descargar",
    "descargarías": "descargar",
    "descargas": "descargar",
    "descargase": "descargar",
    "descargaseis": "descargar",
    "descargasen": "descargar",
    "descargases": "descargar",
    "descargaste": "descargar",
    "descargasteis": "descargar",
    "descargo": "descargar",
    "descargos": "descargo",
    "descargue": "descargar",
    "descarguemos": "descargar",
    "descarguen": "descargar",
    "descargues": "descargue",
    "descargué": "descargar",
    "descarguéis": "descargar",
    "descargábamos": "descargar",
    "descargáis": "descargar",
    "descargáramos": "descargar",
    "descargáremos": "descargar",
    "descargásemos": "descargar",
    "descargó": "descargar",
    "descariños": "descariño",
    "descarna": "descarnar",
    "descarnaba": "descarnar",
    "descarnabais": "descarnar",
    "descarnaban": "descarnar",
    "descarnabas": "descarnar",
    "descarnad": "descarnar",
    "descarnada": "descarnar",
    "descarnadas": "descarnar",
    "descarnado": "descarnar",
    "descarnadores": "descarnador",
    "descarnados": "descarnar",
    "descarnaduras": "descarnadura",
    "descarnamos": "descarnar",
    "descarnan": "descarnar",
    "descarnando": "descarnar",
    "descarnara": "descarnar",
    "descarnarais": "descarnar",
    "descarnaran": "descarnar",
    "descarnaras": "descarnar",
    "descarnare": "descarnar",
    "descarnareis": "descarnar",
    "descarnaremos": "descarnar",
    "descarnaren": "descarnar",
    "descarnares": "descarnar",
    "descarnaron": "descarnar",
    "descarnará": "descarnar",
    "descarnarán": "descarnar",
    "descarnarás": "descarnar",
    "descarnaré": "descarnar",
    "descarnaréis": "descarnar",
    "descarnaría": "descarnar",
    "descarnaríais": "descarnar",
    "descarnaríamos": "descarnar",
    "descarnarían": "descarnar",
    "descarnarías": "descarnar",
    "descarnas": "descarnar",
    "descarnase": "descarnar",
    "descarnaseis": "descarnar",
    "descarnasen": "descarnar",
    "descarnases": "descarnar",
    "descarnaste": "descarnar",
    "descarnasteis": "descarnar",
    "descarne": "descarnar",
    "descarnemos": "descarnar",
    "descarnen": "descarnar",
    "descarnes": "descarnar",
    "descarno": "descarnar",
    "descarnábamos": "descarnar",
    "descarnáis": "descarnar",
    "descarnáramos": "descarnar",
    "descarnáremos": "descarnar",
    "descarnásemos": "descarnar",
    "descarné": "descarnar",
    "descarnéis": "descarnar",
    "descarnó": "descarnar",
    "descaro": "descarar",
    "descaroce": "descarozar",
    "descarocemos": "descarozar",
    "descarocen": "descarozar",
    "descaroces": "descarozar",
    "descarocé": "descarozar",
    "descarocéis": "descarozar",
    "descaros": "descaro",
    "descaroza": "descarozar",
    "descarozaba": "descarozar",
    "descarozabais": "descarozar",
    "descarozaban": "descarozar",
    "descarozabas": "descarozar",
    "descarozad": "descarozar",
    "descarozada": "descarozar",
    "descarozadas": "descarozar",
    "descarozado": "descarozar",
    "descarozados": "descarozar",
    "descarozamos": "descarozar",
    "descarozan": "descarozar",
    "descarozando": "descarozar",
    "descarozara": "descarozar",
    "descarozarais": "descarozar",
    "descarozaran": "descarozar",
    "descarozaras": "descarozar",
    "descarozare": "descarozar",
    "descarozareis": "descarozar",
    "descarozaremos": "descarozar",
    "descarozaren": "descarozar",
    "descarozares": "descarozar",
    "descarozaron": "descarozar",
    "descarozará": "descarozar",
    "descarozarán": "descarozar",
    "descarozarás": "descarozar",
    "descarozaré": "descarozar",
    "descarozaréis": "descarozar",
    "descarozaría": "descarozar",
    "descarozaríais": "descarozar",
    "descarozaríamos": "descarozar",
    "descarozarían": "descarozar",
    "descarozarías": "descarozar",
    "descarozas": "descarozar",
    "descarozase": "descarozar",
    "descarozaseis": "descarozar",
    "descarozasen": "descarozar",
    "descarozases": "descarozar",
    "descarozaste": "descarozar",
    "descarozasteis": "descarozar",
    "descarozo": "descarozar",
    "descarozábamos": "descarozar",
    "descarozáis": "descarozar",
    "descarozáramos": "descarozar",
    "descarozáremos": "descarozar",
    "descarozásemos": "descarozar",
    "descarozó": "descarozar",
    "descarriaba": "descarriar",
    "descarriabais": "descarriar",
    "descarriaban": "descarriar",
    "descarriabas": "descarriar",
    "descarriad": "descarriar",
    "descarriada": "descarriar",
    "descarriadas": "descarriar",
    "descarriado": "descarriar",
    "descarriados": "descarriar",
    "descarriamientos": "descarriamiento",
    "descarriamos": "descarriar",
    "descarriando": "descarriar",
    "descarriara": "descarriar",
    "descarriarais": "descarriar",
    "descarriaran": "descarriar",
    "descarriaras": "descarriar",
    "descarriare": "descarriar",
    "descarriareis": "descarriar",
    "descarriaremos": "descarriar",
    "descarriaren": "descarriar",
    "descarriares": "descarriar",
    "descarriaron": "descarriar",
    "descarriará": "descarriar",
    "descarriarán": "descarriar",
    "descarriarás": "descarriar",
    "descarriaré": "descarriar",
    "descarriaréis": "descarriar",
    "descarriaría": "descarriar",
    "descarriaríais": "descarriar",
    "descarriaríamos": "descarriar",
    "descarriarían": "descarriar",
    "descarriarías": "descarriar",
    "descarriase": "descarriar",
    "descarriaseis": "descarriar",
    "descarriasen": "descarriar",
    "descarriases": "descarriar",
    "descarriaste": "descarriar",
    "descarriasteis": "descarriar",
    "descarriemos": "descarriar",
    "descarrila": "descarrilar",
    "descarrilaba": "descarrilar",
    "descarrilabais": "descarrilar",
    "descarrilaban": "descarrilar",
    "descarrilabas": "descarrilar",
    "descarrilad": "descarrilar",
    "descarrilada": "descarrilar",
    "descarriladas": "descarrilar",
    "descarrilado": "descarrilar",
    "descarrilados": "descarrilar",
    "descarriladuras": "descarriladura",
    "descarrilamientos": "descarrilamiento",
    "descarrilamos": "descarrilar",
    "descarrilan": "descarrilar",
    "descarrilando": "descarrilar",
    "descarrilara": "descarrilar",
    "descarrilarais": "descarrilar",
    "descarrilaran": "descarrilar",
    "descarrilaras": "descarrilar",
    "descarrilare": "descarrilar",
    "descarrilareis": "descarrilar",
    "descarrilaremos": "descarrilar",
    "descarrilaren": "descarrilar",
    "descarrilares": "descarrilar",
    "descarrilaron": "descarrilar",
    "descarrilará": "descarrilar",
    "descarrilarán": "descarrilar",
    "descarrilarás": "descarrilar",
    "descarrilaré": "descarrilar",
    "descarrilaréis": "descarrilar",
    "descarrilaría": "descarrilar",
    "descarrilaríais": "descarrilar",
    "descarrilaríamos": "descarrilar",
    "descarrilarían": "descarrilar",
    "descarrilarías": "descarrilar",
    "descarrilas": "descarrilar",
    "descarrilase": "descarrilar",
    "descarrilaseis": "descarrilar",
    "descarrilasen": "descarrilar",
    "descarrilases": "descarrilar",
    "descarrilaste": "descarrilar",
    "descarrilasteis": "descarrilar",
    "descarrile": "descarrilar",
    "descarrilemos": "descarrilar",
    "descarrilen": "descarrilar",
    "descarriles": "descarrilar",
    "descarrilladuras": "descarrilladura",
    "descarrilo": "descarrilar",
    "descarrilábamos": "descarrilar",
    "descarriláis": "descarrilar",
    "descarriláramos": "descarrilar",
    "descarriláremos": "descarrilar",
    "descarrilásemos": "descarrilar",
    "descarrilé": "descarrilar",
    "descarriléis": "descarrilar",
    "descarriló": "descarrilar",
    "descarriábamos": "descarriar",
    "descarriáis": "descarriar",
    "descarriáramos": "descarriar",
    "descarriáremos": "descarriar",
    "descarriásemos": "descarriar",
    "descarrié": "descarriar",
    "descarriéis": "descarriar",
    "descarrió": "descarriar",
    "descarría": "descarriar",
    "descarrían": "descarriar",
    "descarrías": "descarriar",
    "descarríe": "descarriar",
    "descarríen": "descarriar",
    "descarríes": "descarriar",
    "descarrío": "descarriar",
    "descarríos": "descarrío",
    "descarta": "descartar",
    "descartaba": "descartar",
    "descartabais": "descartar",
    "descartaban": "descartar",
    "descartabas": "descartar",
    "descartad": "descartar",
    "descartada": "descartar",
    "descartadas": "descartar",
    "descartado": "descartar",
    "descartados": "descartar",
    "descartamos": "descartar",
    "descartan": "descartar",
    "descartando": "descartar",
    "descartara": "descartar",
    "descartarais": "descartar",
    "descartaran": "descartar",
    "descartaras": "descartar",
    "descartare": "descartar",
    "descartareis": "descartar",
    "descartaremos": "descartar",
    "descartaren": "descartar",
    "descartares": "descartar",
    "descartaron": "descartar",
    "descartará": "descartar",
    "descartarán": "descartar",
    "descartarás": "descartar",
    "descartaré": "descartar",
    "descartaréis": "descartar",
    "descartaría": "descartar",
    "descartaríais": "descartar",
    "descartaríamos": "descartar",
    "descartarían": "descartar",
    "descartarías": "descartar",
    "descartas": "descartar",
    "descartase": "descartar",
    "descartaseis": "descartar",
    "descartasen": "descartar",
    "descartases": "descartar",
    "descartaste": "descartar",
    "descartasteis": "descartar",
    "descarte": "descartar",
    "descartemos": "descartar",
    "descarten": "descartar",
    "descartes": "descarte",
    "descarto": "descartar",
    "descartábamos": "descartar",
    "descartáis": "descartar",
    "descartáramos": "descartar",
    "descartáremos": "descartar",
    "descartásemos": "descartar",
    "descarté": "descartar",
    "descartéis": "descartar",
    "descartó": "descartar",
    "descarábamos": "descarar",
    "descaráis": "descarar",
    "descaráramos": "descarar",
    "descaráremos": "descarar",
    "descarásemos": "descarar",
    "descaré": "descarar",
    "descaréis": "descarar",
    "descaró": "descarar",
    "descasa": "descasar",
    "descasaba": "descasar",
    "descasabais": "descasar",
    "descasaban": "descasar",
    "descasabas": "descasar",
    "descasad": "descasar",
    "descasada": "descasar",
    "descasadas": "descasar",
    "descasado": "descasar",
    "descasados": "descasar",
    "descasamientos": "descasamiento",
    "descasamos": "descasar",
    "descasan": "descasar",
    "descasando": "descasar",
    "descasara": "descasar",
    "descasarais": "descasar",
    "descasaran": "descasar",
    "descasaras": "descasar",
    "descasare": "descasar",
    "descasareis": "descasar",
    "descasaremos": "descasar",
    "descasaren": "descasar",
    "descasares": "descasar",
    "descasaron": "descasar",
    "descasará": "descasar",
    "descasarán": "descasar",
    "descasarás": "descasar",
    "descasaré": "descasar",
    "descasaréis": "descasar",
    "descasaría": "descasar",
    "descasaríais": "descasar",
    "descasaríamos": "descasar",
    "descasarían": "descasar",
    "descasarías": "descasar",
    "descasas": "descasar",
    "descasase": "descasar",
    "descasaseis": "descasar",
    "descasasen": "descasar",
    "descasases": "descasar",
    "descasaste": "descasar",
    "descasasteis": "descasar",
    "descasca": "descascar",
    "descascaba": "descascar",
    "descascabais": "descascar",
    "descascaban": "descascar",
    "descascabas": "descascar",
    "descascad": "descascar",
    "descascada": "descascar",
    "descascadas": "descascar",
    "descascado": "descascar",
    "descascados": "descascar",
    "descascamos": "descascar",
    "descascan": "descascar",
    "descascando": "descascar",
    "descascara": "descascarar",
    "descascaraba": "descascarar",
    "descascarabais": "descascarar",
    "descascaraban": "descascarar",
    "descascarabas": "descascarar",
    "descascarad": "descascarar",
    "descascarada": "descascarar",
    "descascaradas": "descascarar",
    "descascarado": "descascarar",
    "descascaradoras": "descascaradora",
    "descascarados": "descascarar",
    "descascarais": "descascar",
    "descascaramos": "descascarar",
    "descascaran": "descascarar",
    "descascarando": "descascarar",
    "descascarara": "descascarar",
    "descascararais": "descascarar",
    "descascararan": "descascarar",
    "descascararas": "descascarar",
    "descascarare": "descascarar",
    "descascarareis": "descascarar",
    "descascararemos": "descascarar",
    "descascararen": "descascarar",
    "descascarares": "descascarar",
    "descascararon": "descascarar",
    "descascarará": "descascarar",
    "descascararán": "descascarar",
    "descascararás": "descascarar",
    "descascararé": "descascarar",
    "descascararéis": "descascarar",
    "descascararía": "descascarar",
    "descascararíais": "descascarar",
    "descascararíamos": "descascarar",
    "descascararían": "descascarar",
    "descascararías": "descascarar",
    "descascaras": "descascarar",
    "descascarase": "descascarar",
    "descascaraseis": "descascarar",
    "descascarasen": "descascarar",
    "descascarases": "descascarar",
    "descascaraste": "descascarar",
    "descascarasteis": "descascarar",
    "descascare": "descascarar",
    "descascareis": "descascar",
    "descascaremos": "descascarar",
    "descascaren": "descascarar",
    "descascares": "descascarar",
    "descascarilla": "descascarillar",
    "descascarillaba": "descascarillar",
    "descascarillabais": "descascarillar",
    "descascarillaban": "descascarillar",
    "descascarillabas": "descascarillar",
    "descascarillad": "descascarillar",
    "descascarillada": "descascarillar",
    "descascarilladas": "descascarillar",
    "descascarillado": "descascarillar",
    "descascarilladoras": "descascarilladora",
    "descascarillados": "descascarillar",
    "descascarillamientos": "descascarillamiento",
    "descascarillamos": "descascarillar",
    "descascarillan": "descascarillar",
    "descascarillando": "descascarillar",
    "descascarillara": "descascarillar",
    "descascarillarais": "descascarillar",
    "descascarillaran": "descascarillar",
    "descascarillaras": "descascarillar",
    "descascarillare": "descascarillar",
    "descascarillareis": "descascarillar",
    "descascarillaremos": "descascarillar",
    "descascarillaren": "descascarillar",
    "descascarillares": "descascarillar",
    "descascarillaron": "descascarillar",
    "descascarillará": "descascarillar",
    "descascarillarán": "descascarillar",
    "descascarillarás": "descascarillar",
    "descascarillaré": "descascarillar",
    "descascarillaréis": "descascarillar",
    "descascarillaría": "descascarillar",
    "descascarillaríais": "descascarillar",
    "descascarillaríamos": "descascarillar",
    "descascarillarían": "descascarillar",
    "descascarillarías": "descascarillar",
    "descascarillas": "descascarillar",
    "descascarillase": "descascarillar",
    "descascarillaseis": "descascarillar",
    "descascarillasen": "descascarillar",
    "descascarillases": "descascarillar",
    "descascarillaste": "descascarillar",
    "descascarillasteis": "descascarillar",
    "descascarille": "descascarillar",
    "descascarillemos": "descascarillar",
    "descascarillen": "descascarillar",
    "descascarilles": "descascarillar",
    "descascarillo": "descascarillar",
    "descascarillábamos": "descascarillar",
    "descascarilláis": "descascarillar",
    "descascarilláramos": "descascarillar",
    "descascarilláremos": "descascarillar",
    "descascarillásemos": "descascarillar",
    "descascarillé": "descascarillar",
    "descascarilléis": "descascarillar",
    "descascarilló": "descascarillar",
    "descascaro": "descascarar",
    "descascaron": "descascar",
    "descascará": "descascar",
    "descascarábamos": "descascarar",
    "descascaráis": "descascarar",
    "descascarán": "descascar",
    "descascaráramos": "descascarar",
    "descascaráremos": "descascarar",
    "descascarás": "descascar",
    "descascarásemos": "descascarar",
    "descascaré": "descascarar",
    "descascaréis": "descascarar",
    "descascaría": "descascar",
    "descascaríais": "descascar",
    "descascaríamos": "descascar",
    "descascarían": "descascar",
    "descascarías": "descascar",
    "descascaró": "descascarar",
    "descascas": "descascar",
    "descascase": "descascar",
    "descascaseis": "descascar",
    "descascasen": "descascar",
    "descascases": "descascar",
    "descascaste": "descascar",
    "descascasteis": "descascar",
    "descasco": "descascar",
    "descascábamos": "descascar",
    "descascáis": "descascar",
    "descascáramos": "descascar",
    "descascáremos": "descascar",
    "descascásemos": "descascar",
    "descascó": "descascar",
    "descase": "descasar",
    "descasemos": "descasar",
    "descasen": "descasar",
    "descases": "descasar",
    "descaso": "descasar",
    "descaspa": "descaspar",
    "descaspaba": "descaspar",
    "descaspabais": "descaspar",
    "descaspaban": "descaspar",
    "descaspabas": "descaspar",
    "descaspad": "descaspar",
    "descaspada": "descaspar",
    "descaspadas": "descaspar",
    "descaspado": "descaspar",
    "descaspados": "descaspar",
    "descaspamos": "descaspar",
    "descaspan": "descaspar",
    "descaspando": "descaspar",
    "descaspara": "descaspar",
    "descasparais": "descaspar",
    "descasparan": "descaspar",
    "descasparas": "descaspar",
    "descaspare": "descaspar",
    "descaspareis": "descaspar",
    "descasparemos": "descaspar",
    "descasparen": "descaspar",
    "descaspares": "descaspar",
    "descasparon": "descaspar",
    "descaspará": "descaspar",
    "descasparán": "descaspar",
    "descasparás": "descaspar",
    "descasparé": "descaspar",
    "descasparéis": "descaspar",
    "descasparía": "descaspar",
    "descasparíais": "descaspar",
    "descasparíamos": "descaspar",
    "descasparían": "descaspar",
    "descasparías": "descaspar",
    "descaspas": "descaspar",
    "descaspase": "descaspar",
    "descaspaseis": "descaspar",
    "descaspasen": "descaspar",
    "descaspases": "descaspar",
    "descaspaste": "descaspar",
    "descaspasteis": "descaspar",
    "descaspe": "descaspar",
    "descaspemos": "descaspar",
    "descaspen": "descaspar",
    "descaspes": "descaspar",
    "descaspo": "descaspar",
    "descaspábamos": "descaspar",
    "descaspáis": "descaspar",
    "descaspáramos": "descaspar",
    "descaspáremos": "descaspar",
    "descaspásemos": "descaspar",
    "descaspé": "descaspar",
    "descaspéis": "descaspar",
    "descaspó": "descaspar",
    "descasque": "descascar",
    "descasquemos": "descascar",
    "descasquen": "descascar",
    "descasques": "descasque",
    "descasqué": "descascar",
    "descasquéis": "descascar",
    "descasta": "descastar",
    "descastaba": "descastar",
    "descastabais": "descastar",
    "descastaban": "descastar",
    "descastabas": "descastar",
    "descastad": "descastar",
    "descastada": "descastar",
    "descastadas": "descastar",
    "descastado": "descastar",
    "descastados": "descastar",
    "descastamos": "descastar",
    "descastan": "descastar",
    "descastando": "descastar",
    "descastara": "descastar",
    "descastarais": "descastar",
    "descastaran": "descastar",
    "descastaras": "descastar",
    "descastare": "descastar",
    "descastareis": "descastar",
    "descastaremos": "descastar",
    "descastaren": "descastar",
    "descastares": "descastar",
    "descastaron": "descastar",
    "descastará": "descastar",
    "descastarán": "descastar",
    "descastarás": "descastar",
    "descastaré": "descastar",
    "descastaréis": "descastar",
    "descastaría": "descastar",
    "descastaríais": "descastar",
    "descastaríamos": "descastar",
    "descastarían": "descastar",
    "descastarías": "descastar",
    "descastas": "descastar",
    "descastase": "descastar",
    "descastaseis": "descastar",
    "descastasen": "descastar",
    "descastases": "descastar",
    "descastaste": "descastar",
    "descastasteis": "descastar",
    "descaste": "descastar",
    "descastemos": "descastar",
    "descasten": "descastar",
    "descastes": "descastar",
    "descasto": "descastar",
    "descastábamos": "descastar",
    "descastáis": "descastar",
    "descastáramos": "descastar",
    "descastáremos": "descastar",
    "descastásemos": "descastar",
    "descasté": "descastar",
    "descastéis": "descastar",
    "descastó": "descastar",
    "descasábamos": "descasar",
    "descasáis": "descasar",
    "descasáramos": "descasar",
    "descasáremos": "descasar",
    "descasásemos": "descasar",
    "descasé": "descasar",
    "descaséis": "descasar",
    "descasó": "descasar",
    "descatolice": "descatolizar",
    "descatolicemos": "descatolizar",
    "descatolicen": "descatolizar",
    "descatolices": "descatolizar",
    "descatolicé": "descatolizar",
    "descatolicéis": "descatolizar",
    "descatoliza": "descatolizar",
    "descatolizaba": "descatolizar",
    "descatolizabais": "descatolizar",
    "descatolizaban": "descatolizar",
    "descatolizabas": "descatolizar",
    "descatolizaciones": "descatolización",
    "descatolizad": "descatolizar",
    "descatolizada": "descatolizar",
    "descatolizadas": "descatolizar",
    "descatolizado": "descatolizar",
    "descatolizados": "descatolizar",
    "descatolizamos": "descatolizar",
    "descatolizan": "descatolizar",
    "descatolizando": "descatolizar",
    "descatolizara": "descatolizar",
    "descatolizarais": "descatolizar",
    "descatolizaran": "descatolizar",
    "descatolizaras": "descatolizar",
    "descatolizare": "descatolizar",
    "descatolizareis": "descatolizar",
    "descatolizaremos": "descatolizar",
    "descatolizaren": "descatolizar",
    "descatolizares": "descatolizar",
    "descatolizaron": "descatolizar",
    "descatolizará": "descatolizar",
    "descatolizarán": "descatolizar",
    "descatolizarás": "descatolizar",
    "descatolizaré": "descatolizar",
    "descatolizaréis": "descatolizar",
    "descatolizaría": "descatolizar",
    "descatolizaríais": "descatolizar",
    "descatolizaríamos": "descatolizar",
    "descatolizarían": "descatolizar",
    "descatolizarías": "descatolizar",
    "descatolizas": "descatolizar",
    "descatolizase": "descatolizar",
    "descatolizaseis": "descatolizar",
    "descatolizasen": "descatolizar",
    "descatolizases": "descatolizar",
    "descatolizaste": "descatolizar",
    "descatolizasteis": "descatolizar",
    "descatolizo": "descatolizar",
    "descatolizábamos": "descatolizar",
    "descatolizáis": "descatolizar",
    "descatolizáramos": "descatolizar",
    "descatolizáremos": "descatolizar",
    "descatolizásemos": "descatolizar",
    "descatolizó": "descatolizar",
    "descaudalada": "descaudalado",
    "descaudaladas": "descaudalado",
    "descaudalados": "descaudalado",
    "descañona": "descañonar",
    "descañonaba": "descañonar",
    "descañonabais": "descañonar",
    "descañonaban": "descañonar",
    "descañonabas": "descañonar",
    "descañonad": "descañonar",
    "descañonada": "descañonar",
    "descañonadas": "descañonar",
    "descañonado": "descañonar",
    "descañonados": "descañonar",
    "descañonamos": "descañonar",
    "descañonan": "descañonar",
    "descañonando": "descañonar",
    "descañonara": "descañonar",
    "descañonarais": "descañonar",
    "descañonaran": "descañonar",
    "descañonaras": "descañonar",
    "descañonare": "descañonar",
    "descañonareis": "descañonar",
    "descañonaremos": "descañonar",
    "descañonaren": "descañonar",
    "descañonares": "descañonar",
    "descañonaron": "descañonar",
    "descañonará": "descañonar",
    "descañonarán": "descañonar",
    "descañonarás": "descañonar",
    "descañonaré": "descañonar",
    "descañonaréis": "descañonar",
    "descañonaría": "descañonar",
    "descañonaríais": "descañonar",
    "descañonaríamos": "descañonar",
    "descañonarían": "descañonar",
    "descañonarías": "descañonar",
    "descañonas": "descañonar",
    "descañonase": "descañonar",
    "descañonaseis": "descañonar",
    "descañonasen": "descañonar",
    "descañonases": "descañonar",
    "descañonaste": "descañonar",
    "descañonasteis": "descañonar",
    "descañone": "descañonar",
    "descañonemos": "descañonar",
    "descañonen": "descañonar",
    "descañones": "descañonar",
    "descañono": "descañonar",
    "descañonábamos": "descañonar",
    "descañonáis": "descañonar",
    "descañonáramos": "descañonar",
    "descañonáremos": "descañonar",
    "descañonásemos": "descañonar",
    "descañoné": "descañonar",
    "descañonéis": "descañonar",
    "descañonó": "descañonar",
    "desceba": "descebar",
    "descebaba": "descebar",
    "descebabais": "descebar",
    "descebaban": "descebar",
    "descebabas": "descebar",
    "descebad": "descebar",
    "descebada": "descebar",
    "descebadas": "descebar",
    "descebado": "descebar",
    "descebados": "descebar",
    "descebamos": "descebar",
    "desceban": "descebar",
    "descebando": "descebar",
    "descebara": "descebar",
    "descebarais": "descebar",
    "descebaran": "descebar",
    "descebaras": "descebar",
    "descebare": "descebar",
    "descebareis": "descebar",
    "descebaremos": "descebar",
    "descebaren": "descebar",
    "descebares": "descebar",
    "descebaron": "descebar",
    "descebará": "descebar",
    "descebarán": "descebar",
    "descebarás": "descebar",
    "descebaré": "descebar",
    "descebaréis": "descebar",
    "descebaría": "descebar",
    "descebaríais": "descebar",
    "descebaríamos": "descebar",
    "descebarían": "descebar",
    "descebarías": "descebar",
    "descebas": "descebar",
    "descebase": "descebar",
    "descebaseis": "descebar",
    "descebasen": "descebar",
    "descebases": "descebar",
    "descebaste": "descebar",
    "descebasteis": "descebar",
    "descebe": "descebar",
    "descebemos": "descebar",
    "desceben": "descebar",
    "descebes": "descebe",
    "descebo": "descebar",
    "descebábamos": "descebar",
    "descebáis": "descebar",
    "descebáramos": "descebar",
    "descebáremos": "descebar",
    "descebásemos": "descebar",
    "descebé": "descebar",
    "descebéis": "descebar",
    "descebó": "descebar",
    "descendamos": "descender",
    "descended": "descender",
    "descendemos": "descender",
    "descendencias": "descendencia",
    "descendentes": "descendente",
    "descenderemos": "descender",
    "descenderá": "descender",
    "descenderán": "descender",
    "descenderás": "descender",
    "descenderé": "descender",
    "descenderéis": "descender",
    "descendería": "descender",
    "descenderíais": "descender",
    "descenderíamos": "descender",
    "descenderían": "descender",
    "descenderías": "descender",
    "descendida": "descender",
    "descendidas": "descender",
    "descendido": "descender",
    "descendidos": "descender",
    "descendiendo": "descender",
    "descendientes": "descendiente",
    "descendiera": "descender",
    "descendierais": "descender",
    "descendieran": "descender",
    "descendieras": "descender",
    "descendiere": "descender",
    "descendiereis": "descender",
    "descendieren": "descender",
    "descendieres": "descender",
    "descendieron": "descender",
    "descendiese": "descender",
    "descendieseis": "descender",
    "descendiesen": "descender",
    "descendieses": "descender",
    "descendimientos": "descendimiento",
    "descendimos": "descender",
    "descendiste": "descender",
    "descendisteis": "descender",
    "descendiéramos": "descender",
    "descendiéremos": "descender",
    "descendiésemos": "descender",
    "descendió": "descender",
    "descendáis": "descender",
    "descendéis": "descender",
    "descendí": "descender",
    "descendía": "descender",
    "descendíais": "descender",
    "descendíamos": "descender",
    "descendían": "descender",
    "descendías": "descender",
    "descensos": "descenso",
    "descentra": "descentrar",
    "descentraba": "descentrar",
    "descentrabais": "descentrar",
    "descentraban": "descentrar",
    "descentrabas": "descentrar",
    "descentrad": "descentrar",
    "descentrada": "descentrar",
    "descentradas": "descentrar",
    "descentrado": "descentrar",
    "descentrados": "descentrar",
    "descentralice": "descentralizar",
    "descentralicemos": "descentralizar",
    "descentralicen": "descentralizar",
    "descentralices": "descentralizar",
    "descentralicé": "descentralizar",
    "descentralicéis": "descentralizar",
    "descentraliza": "descentralizar",
    "descentralizaba": "descentralizar",
    "descentralizabais": "descentralizar",
    "descentralizaban": "descentralizar",
    "descentralizabas": "descentralizar",
    "descentralizaciones": "descentralización",
    "descentralizad": "descentralizar",
    "descentralizada": "descentralizar",
    "descentralizadas": "descentralizar",
    "descentralizado": "descentralizar",
    "descentralizadora": "descentralizador",
    "descentralizadoras": "descentralizador",
    "descentralizadores": "descentralizador",
    "descentralizados": "descentralizar",
    "descentralizamos": "descentralizar",
    "descentralizan": "descentralizar",
    "descentralizando": "descentralizar",
    "descentralizara": "descentralizar",
    "descentralizarais": "descentralizar",
    "descentralizaran": "descentralizar",
    "descentralizaras": "descentralizar",
    "descentralizare": "descentralizar",
    "descentralizareis": "descentralizar",
    "descentralizaremos": "descentralizar",
    "descentralizaren": "descentralizar",
    "descentralizares": "descentralizar",
    "descentralizaron": "descentralizar",
    "descentralizará": "descentralizar",
    "descentralizarán": "descentralizar",
    "descentralizarás": "descentralizar",
    "descentralizaré": "descentralizar",
    "descentralizaréis": "descentralizar",
    "descentralizaría": "descentralizar",
    "descentralizaríais": "descentralizar",
    "descentralizaríamos": "descentralizar",
    "descentralizarían": "descentralizar",
    "descentralizarías": "descentralizar",
    "descentralizas": "descentralizar",
    "descentralizase": "descentralizar",
    "descentralizaseis": "descentralizar",
    "descentralizasen": "descentralizar",
    "descentralizases": "descentralizar",
    "descentralizaste": "descentralizar",
    "descentralizasteis": "descentralizar",
    "descentralizo": "descentralizar",
    "descentralizábamos": "descentralizar",
    "descentralizáis": "descentralizar",
    "descentralizáramos": "descentralizar",
    "descentralizáremos": "descentralizar",
    "descentralizásemos": "descentralizar",
    "descentralizó": "descentralizar",
    "descentramientos": "descentramiento",
    "descentramos": "descentrar",
    "descentran": "descentrar",
    "descentrando": "descentrar",
    "descentrara": "descentrar",
    "descentrarais": "descentrar",
    "descentraran": "descentrar",
    "descentraras": "descentrar",
    "descentrare": "descentrar",
    "descentrareis": "descentrar",
    "descentraremos": "descentrar",
    "descentraren": "descentrar",
    "descentrares": "descentrar",
    "descentraron": "descentrar",
    "descentrará": "descentrar",
    "descentrarán": "descentrar",
    "descentrarás": "descentrar",
    "descentraré": "descentrar",
    "descentraréis": "descentrar",
    "descentraría": "descentrar",
    "descentraríais": "descentrar",
    "descentraríamos": "descentrar",
    "descentrarían": "descentrar",
    "descentrarías": "descentrar",
    "descentras": "descentrar",
    "descentrase": "descentrar",
    "descentraseis": "descentrar",
    "descentrasen": "descentrar",
    "descentrases": "descentrar",
    "descentraste": "descentrar",
    "descentrasteis": "descentrar",
    "descentre": "descentrar",
    "descentremos": "descentrar",
    "descentren": "descentrar",
    "descentres": "descentrar",
    "descentro": "descentrar",
    "descentrábamos": "descentrar",
    "descentráis": "descentrar",
    "descentráramos": "descentrar",
    "descentráremos": "descentrar",
    "descentrásemos": "descentrar",
    "descentré": "descentrar",
    "descentréis": "descentrar",
    "descentró": "descentrar",
    "descepa": "descepar",
    "descepaba": "descepar",
    "descepabais": "descepar",
    "descepaban": "descepar",
    "descepabas": "descepar",
    "descepad": "descepar",
    "descepada": "descepar",
    "descepadas": "descepar",
    "descepado": "descepar",
    "descepados": "descepar",
    "descepamos": "descepar",
    "descepan": "descepar",
    "descepando": "descepar",
    "descepara": "descepar",
    "desceparais": "descepar",
    "desceparan": "descepar",
    "desceparas": "descepar",
    "descepare": "descepar",
    "descepareis": "descepar",
    "desceparemos": "descepar",
    "desceparen": "descepar",
    "descepares": "descepar",
    "desceparon": "descepar",
    "descepará": "descepar",
    "desceparán": "descepar",
    "desceparás": "descepar",
    "desceparé": "descepar",
    "desceparéis": "descepar",
    "desceparía": "descepar",
    "desceparíais": "descepar",
    "desceparíamos": "descepar",
    "desceparían": "descepar",
    "desceparías": "descepar",
    "descepas": "descepar",
    "descepase": "descepar",
    "descepaseis": "descepar",
    "descepasen": "descepar",
    "descepases": "descepar",
    "descepaste": "descepar",
    "descepasteis": "descepar",
    "descepe": "descepar",
    "descepemos": "descepar",
    "descepen": "descepar",
    "descepes": "descepar",
    "descepo": "descepar",
    "descepábamos": "descepar",
    "descepáis": "descepar",
    "descepáramos": "descepar",
    "descepáremos": "descepar",
    "descepásemos": "descepar",
    "descepé": "descepar",
    "descepéis": "descepar",
    "descepó": "descepar",
    "descerca": "descercar",
    "descercaba": "descercar",
    "descercabais": "descercar",
    "descercaban": "descercar",
    "descercabas": "descercar",
    "descercad": "descercar",
    "descercada": "descercar",
    "descercadas": "descercar",
    "descercado": "descercar",
    "descercadores": "descercador",
    "descercados": "descercar",
    "descercamos": "descercar",
    "descercan": "descercar",
    "descercando": "descercar",
    "descercara": "descercar",
    "descercarais": "descercar",
    "descercaran": "descercar",
    "descercaras": "descercar",
    "descercare": "descercar",
    "descercareis": "descercar",
    "descercaremos": "descercar",
    "descercaren": "descercar",
    "descercares": "descercar",
    "descercaron": "descercar",
    "descercará": "descercar",
    "descercarán": "descercar",
    "descercarás": "descercar",
    "descercaré": "descercar",
    "descercaréis": "descercar",
    "descercaría": "descercar",
    "descercaríais": "descercar",
    "descercaríamos": "descercar",
    "descercarían": "descercar",
    "descercarías": "descercar",
    "descercas": "descercar",
    "descercase": "descercar",
    "descercaseis": "descercar",
    "descercasen": "descercar",
    "descercases": "descercar",
    "descercaste": "descercar",
    "descercasteis": "descercar",
    "descerco": "descercar",
    "descercos": "descerco",
    "descercábamos": "descercar",
    "descercáis": "descercar",
    "descercáramos": "descercar",
    "descercáremos": "descercar",
    "descercásemos": "descercar",
    "descercó": "descercar",
    "descerebrada": "descerebrado",
    "descerebradas": "descerebrado",
    "descerebrados": "descerebrado",
    "descerece": "descerezar",
    "descerecemos": "descerezar",
    "descerecen": "descerezar",
    "descereces": "descerezar",
    "descerecé": "descerezar",
    "descerecéis": "descerezar",
    "descereza": "descerezar",
    "descerezaba": "descerezar",
    "descerezabais": "descerezar",
    "descerezaban": "descerezar",
    "descerezabas": "descerezar",
    "descerezad": "descerezar",
    "descerezada": "descerezar",
    "descerezadas": "descerezar",
    "descerezado": "descerezar",
    "descerezados": "descerezar",
    "descerezamos": "descerezar",
    "descerezan": "descerezar",
    "descerezando": "descerezar",
    "descerezara": "descerezar",
    "descerezarais": "descerezar",
    "descerezaran": "descerezar",
    "descerezaras": "descerezar",
    "descerezare": "descerezar",
    "descerezareis": "descerezar",
    "descerezaremos": "descerezar",
    "descerezaren": "descerezar",
    "descerezares": "descerezar",
    "descerezaron": "descerezar",
    "descerezará": "descerezar",
    "descerezarán": "descerezar",
    "descerezarás": "descerezar",
    "descerezaré": "descerezar",
    "descerezaréis": "descerezar",
    "descerezaría": "descerezar",
    "descerezaríais": "descerezar",
    "descerezaríamos": "descerezar",
    "descerezarían": "descerezar",
    "descerezarías": "descerezar",
    "descerezas": "descerezar",
    "descerezase": "descerezar",
    "descerezaseis": "descerezar",
    "descerezasen": "descerezar",
    "descerezases": "descerezar",
    "descerezaste": "descerezar",
    "descerezasteis": "descerezar",
    "descerezo": "descerezar",
    "descerezábamos": "descerezar",
    "descerezáis": "descerezar",
    "descerezáramos": "descerezar",
    "descerezáremos": "descerezar",
    "descerezásemos": "descerezar",
    "descerezó": "descerezar",
    "descerque": "descercar",
    "descerquemos": "descercar",
    "descerquen": "descercar",
    "descerques": "descercar",
    "descerqué": "descercar",
    "descerquéis": "descercar",
    "descerraja": "descerrajar",
    "descerrajaba": "descerrajar",
    "descerrajabais": "descerrajar",
    "descerrajaban": "descerrajar",
    "descerrajabas": "descerrajar",
    "descerrajad": "descerrajar",
    "descerrajada": "descerrajar",
    "descerrajadas": "descerrajar",
    "descerrajado": "descerrajar",
    "descerrajados": "descerrajar",
    "descerrajaduras": "descerrajadura",
    "descerrajamientos": "descerrajamiento",
    "descerrajamos": "descerrajar",
    "descerrajan": "descerrajar",
    "descerrajando": "descerrajar",
    "descerrajara": "descerrajar",
    "descerrajarais": "descerrajar",
    "descerrajaran": "descerrajar",
    "descerrajaras": "descerrajar",
    "descerrajare": "descerrajar",
    "descerrajareis": "descerrajar",
    "descerrajaremos": "descerrajar",
    "descerrajaren": "descerrajar",
    "descerrajares": "descerrajar",
    "descerrajaron": "descerrajar",
    "descerrajará": "descerrajar",
    "descerrajarán": "descerrajar",
    "descerrajarás": "descerrajar",
    "descerrajaré": "descerrajar",
    "descerrajaréis": "descerrajar",
    "descerrajaría": "descerrajar",
    "descerrajaríais": "descerrajar",
    "descerrajaríamos": "descerrajar",
    "descerrajarían": "descerrajar",
    "descerrajarías": "descerrajar",
    "descerrajas": "descerrajar",
    "descerrajase": "descerrajar",
    "descerrajaseis": "descerrajar",
    "descerrajasen": "descerrajar",
    "descerrajases": "descerrajar",
    "descerrajaste": "descerrajar",
    "descerrajasteis": "descerrajar",
    "descerraje": "descerrajar",
    "descerrajemos": "descerrajar",
    "descerrajen": "descerrajar",
    "descerrajes": "descerrajar",
    "descerrajo": "descerrajar",
    "descerrajábamos": "descerrajar",
    "descerrajáis": "descerrajar",
    "descerrajáramos": "descerrajar",
    "descerrajáremos": "descerrajar",
    "descerrajásemos": "descerrajar",
    "descerrajé": "descerrajar",
    "descerrajéis": "descerrajar",
    "descerrajó": "descerrajar",
    "descervigamientos": "descervigamiento",
    "desceñid": "desceñir",
    "desceñida": "desceñir",
    "desceñidas": "desceñir",
    "desceñido": "desceñir",
    "desceñidos": "desceñir",
    "desceñiduras": "desceñidura",
    "desceñimos": "desceñir",
    "desceñiremos": "desceñir",
    "desceñirá": "desceñir",
    "desceñirán": "desceñir",
    "desceñirás": "desceñir",
    "desceñiré": "desceñir",
    "desceñiréis": "desceñir",
    "desceñiría": "desceñir",
    "desceñiríais": "desceñir",
    "desceñiríamos": "desceñir",
    "desceñirían": "desceñir",
    "desceñirías": "desceñir",
    "desceñiste": "desceñir",
    "desceñisteis": "desceñir",
    "desceñí": "desceñir",
    "desceñía": "desceñir",
    "desceñíais": "desceñir",
    "desceñíamos": "desceñir",
    "desceñían": "desceñir",
    "desceñías": "desceñir",
    "desceñís": "desceñir",
    "descienda": "descender",
    "desciendan": "descender",
    "desciendas": "descender",
    "desciende": "descender",
    "descienden": "descender",
    "desciendes": "descender",
    "desciendo": "descender",
    "descifra": "descifrar",
    "descifraba": "descifrar",
    "descifrabais": "descifrar",
    "descifraban": "descifrar",
    "descifrabas": "descifrar",
    "descifrables": "descifrable",
    "descifrad": "descifrar",
    "descifrada": "descifrar",
    "descifradas": "descifrar",
    "descifrado": "descifrar",
    "descifradora": "descifrador",
    "descifradoras": "descifrador",
    "descifradores": "descifrador",
    "descifrados": "descifrar",
    "desciframientos": "desciframiento",
    "desciframos": "descifrar",
    "descifran": "descifrar",
    "descifrando": "descifrar",
    "descifrara": "descifrar",
    "descifrarais": "descifrar",
    "descifraran": "descifrar",
    "descifraras": "descifrar",
    "descifrare": "descifrar",
    "descifrareis": "descifrar",
    "descifraremos": "descifrar",
    "descifraren": "descifrar",
    "descifrares": "descifrar",
    "descifraron": "descifrar",
    "descifrará": "descifrar",
    "descifrarán": "descifrar",
    "descifrarás": "descifrar",
    "descifraré": "descifrar",
    "descifraréis": "descifrar",
    "descifraría": "descifrar",
    "descifraríais": "descifrar",
    "descifraríamos": "descifrar",
    "descifrarían": "descifrar",
    "descifrarías": "descifrar",
    "descifras": "descifrar",
    "descifrase": "descifrar",
    "descifraseis": "descifrar",
    "descifrasen": "descifrar",
    "descifrases": "descifrar",
    "descifraste": "descifrar",
    "descifrasteis": "descifrar",
    "descifre": "descifrar",
    "descifremos": "descifrar",
    "descifren": "descifrar",
    "descifres": "descifrar",
    "descifro": "descifrar",
    "descifrábamos": "descifrar",
    "descifráis": "descifrar",
    "descifráramos": "descifrar",
    "descifráremos": "descifrar",
    "descifrásemos": "descifrar",
    "descifré": "descifrar",
    "descifréis": "descifrar",
    "descifró": "descifrar",
    "descimbra": "descimbrar",
    "descimbraba": "descimbrar",
    "descimbrabais": "descimbrar",
    "descimbraban": "descimbrar",
    "descimbrabas": "descimbrar",
    "descimbrad": "descimbrar",
    "descimbrada": "descimbrar",
    "descimbradas": "descimbrar",
    "descimbrado": "descimbrar",
    "descimbrados": "descimbrar",
    "descimbramientos": "descimbramiento",
    "descimbramos": "descimbrar",
    "descimbran": "descimbrar",
    "descimbrando": "descimbrar",
    "descimbrara": "descimbrar",
    "descimbrarais": "descimbrar",
    "descimbraran": "descimbrar",
    "descimbraras": "descimbrar",
    "descimbrare": "descimbrar",
    "descimbrareis": "descimbrar",
    "descimbraremos": "descimbrar",
    "descimbraren": "descimbrar",
    "descimbrares": "descimbrar",
    "descimbraron": "descimbrar",
    "descimbrará": "descimbrar",
    "descimbrarán": "descimbrar",
    "descimbrarás": "descimbrar",
    "descimbraré": "descimbrar",
    "descimbraréis": "descimbrar",
    "descimbraría": "descimbrar",
    "descimbraríais": "descimbrar",
    "descimbraríamos": "descimbrar",
    "descimbrarían": "descimbrar",
    "descimbrarías": "descimbrar",
    "descimbras": "descimbrar",
    "descimbrase": "descimbrar",
    "descimbraseis": "descimbrar",
    "descimbrasen": "descimbrar",
    "descimbrases": "descimbrar",
    "descimbraste": "descimbrar",
    "descimbrasteis": "descimbrar",
    "descimbre": "descimbrar",
    "descimbremos": "descimbrar",
    "descimbren": "descimbrar",
    "descimbres": "descimbrar",
    "descimbro": "descimbrar",
    "descimbrábamos": "descimbrar",
    "descimbráis": "descimbrar",
    "descimbráramos": "descimbrar",
    "descimbráremos": "descimbrar",
    "descimbrásemos": "descimbrar",
    "descimbré": "descimbrar",
    "descimbréis": "descimbrar",
    "descimbró": "descimbrar",
    "descincha": "descinchar",
    "descinchaba": "descinchar",
    "descinchabais": "descinchar",
    "descinchaban": "descinchar",
    "descinchabas": "descinchar",
    "descinchad": "descinchar",
    "descinchada": "descinchar",
    "descinchadas": "descinchar",
    "descinchado": "descinchar",
    "descinchados": "descinchar",
    "descinchamos": "descinchar",
    "descinchan": "descinchar",
    "descinchando": "descinchar",
    "descinchara": "descinchar",
    "descincharais": "descinchar",
    "descincharan": "descinchar",
    "descincharas": "descinchar",
    "descinchare": "descinchar",
    "descinchareis": "descinchar",
    "descincharemos": "descinchar",
    "descincharen": "descinchar",
    "descinchares": "descinchar",
    "descincharon": "descinchar",
    "descinchará": "descinchar",
    "descincharán": "descinchar",
    "descincharás": "descinchar",
    "descincharé": "descinchar",
    "descincharéis": "descinchar",
    "descincharía": "descinchar",
    "descincharíais": "descinchar",
    "descincharíamos": "descinchar",
    "descincharían": "descinchar",
    "descincharías": "descinchar",
    "descinchas": "descinchar",
    "descinchase": "descinchar",
    "descinchaseis": "descinchar",
    "descinchasen": "descinchar",
    "descinchases": "descinchar",
    "descinchaste": "descinchar",
    "descinchasteis": "descinchar",
    "descinche": "descinchar",
    "descinchemos": "descinchar",
    "descinchen": "descinchar",
    "descinches": "descinchar",
    "descincho": "descinchar",
    "descinchábamos": "descinchar",
    "descincháis": "descinchar",
    "descincháramos": "descinchar",
    "descincháremos": "descinchar",
    "descinchásemos": "descinchar",
    "descinché": "descinchar",
    "descinchéis": "descinchar",
    "descinchó": "descinchar",
    "desciña": "desceñir",
    "desciñamos": "desceñir",
    "desciñan": "desceñir",
    "desciñas": "desceñir",
    "desciñe": "desceñir",
    "desciñen": "desceñir",
    "desciñendo": "desceñir",
    "desciñera": "desceñir",
    "desciñerais": "desceñir",
    "desciñeran": "desceñir",
    "desciñeras": "desceñir",
    "desciñere": "desceñir",
    "desciñereis": "desceñir",
    "desciñeren": "desceñir",
    "desciñeres": "desceñir",
    "desciñeron": "desceñir",
    "desciñes": "desceñir",
    "desciñese": "desceñir",
    "desciñeseis": "desceñir",
    "desciñesen": "desceñir",
    "desciñeses": "desceñir",
    "desciño": "desceñir",
    "desciñáis": "desceñir",
    "desciñéramos": "desceñir",
    "desciñéremos": "desceñir",
    "desciñésemos": "desceñir",
    "desciñó": "desceñir",
    "desclasificaciones": "desclasificación",
    "desclava": "desclavar",
    "desclavaba": "desclavar",
    "desclavabais": "desclavar",
    "desclavaban": "desclavar",
    "desclavabas": "desclavar",
    "desclavad": "desclavar",
    "desclavada": "desclavar",
    "desclavadas": "desclavar",
    "desclavado": "desclavar",
    "desclavadores": "desclavador",
    "desclavados": "desclavar",
    "desclavamos": "desclavar",
    "desclavan": "desclavar",
    "desclavando": "desclavar",
    "desclavara": "desclavar",
    "desclavarais": "desclavar",
    "desclavaran": "desclavar",
    "desclavaras": "desclavar",
    "desclavare": "desclavar",
    "desclavareis": "desclavar",
    "desclavaremos": "desclavar",
    "desclavaren": "desclavar",
    "desclavares": "desclavar",
    "desclavaron": "desclavar",
    "desclavará": "desclavar",
    "desclavarán": "desclavar",
    "desclavarás": "desclavar",
    "desclavaré": "desclavar",
    "desclavaréis": "desclavar",
    "desclavaría": "desclavar",
    "desclavaríais": "desclavar",
    "desclavaríamos": "desclavar",
    "desclavarían": "desclavar",
    "desclavarías": "desclavar",
    "desclavas": "desclavar",
    "desclavase": "desclavar",
    "desclavaseis": "desclavar",
    "desclavasen": "desclavar",
    "desclavases": "desclavar",
    "desclavaste": "desclavar",
    "desclavasteis": "desclavar",
    "desclave": "desclavar",
    "desclavemos": "desclavar",
    "desclaven": "desclavar",
    "desclaves": "desclavar",
    "desclavo": "desclavar",
    "desclavábamos": "desclavar",
    "desclaváis": "desclavar",
    "desclaváramos": "desclavar",
    "desclaváremos": "desclavar",
    "desclavásemos": "desclavar",
    "desclavé": "desclavar",
    "desclavéis": "desclavar",
    "desclavó": "desclavar",
    "descoagula": "descoagular",
    "descoagulaba": "descoagular",
    "descoagulabais": "descoagular",
    "descoagulaban": "descoagular",
    "descoagulabas": "descoagular",
    "descoagulad": "descoagular",
    "descoagulada": "descoagular",
    "descoaguladas": "descoagular",
    "descoagulado": "descoagular",
    "descoagulados": "descoagular",
    "descoagulamos": "descoagular",
    "descoagulan": "descoagular",
    "descoagulando": "descoagular",
    "descoagulantes": "descoagulante",
    "descoagulara": "descoagular",
    "descoagularais": "descoagular",
    "descoagularan": "descoagular",
    "descoagularas": "descoagular",
    "descoagulare": "descoagular",
    "descoagulareis": "descoagular",
    "descoagularemos": "descoagular",
    "descoagularen": "descoagular",
    "descoagulares": "descoagular",
    "descoagularon": "descoagular",
    "descoagulará": "descoagular",
    "descoagularán": "descoagular",
    "descoagularás": "descoagular",
    "descoagularé": "descoagular",
    "descoagularéis": "descoagular",
    "descoagularía": "descoagular",
    "descoagularíais": "descoagular",
    "descoagularíamos": "descoagular",
    "descoagularían": "descoagular",
    "descoagularías": "descoagular",
    "descoagulas": "descoagular",
    "descoagulase": "descoagular",
    "descoagulaseis": "descoagular",
    "descoagulasen": "descoagular",
    "descoagulases": "descoagular",
    "descoagulaste": "descoagular",
    "descoagulasteis": "descoagular",
    "descoagule": "descoagular",
    "descoagulemos": "descoagular",
    "descoagulen": "descoagular",
    "descoagules": "descoagular",
    "descoagulo": "descoagular",
    "descoagulábamos": "descoagular",
    "descoaguláis": "descoagular",
    "descoaguláramos": "descoagular",
    "descoaguláremos": "descoagular",
    "descoagulásemos": "descoagular",
    "descoagulé": "descoagular",
    "descoaguléis": "descoagular",
    "descoaguló": "descoagular",
    "descobaja": "descobajar",
    "descobajaba": "descobajar",
    "descobajabais": "descobajar",
    "descobajaban": "descobajar",
    "descobajabas": "descobajar",
    "descobajad": "descobajar",
    "descobajada": "descobajar",
    "descobajadas": "descobajar",
    "descobajado": "descobajar",
    "descobajados": "descobajar",
    "descobajamos": "descobajar",
    "descobajan": "descobajar",
    "descobajando": "descobajar",
    "descobajara": "descobajar",
    "descobajarais": "descobajar",
    "descobajaran": "descobajar",
    "descobajaras": "descobajar",
    "descobajare": "descobajar",
    "descobajareis": "descobajar",
    "descobajaremos": "descobajar",
    "descobajaren": "descobajar",
    "descobajares": "descobajar",
    "descobajaron": "descobajar",
    "descobajará": "descobajar",
    "descobajarán": "descobajar",
    "descobajarás": "descobajar",
    "descobajaré": "descobajar",
    "descobajaréis": "descobajar",
    "descobajaría": "descobajar",
    "descobajaríais": "descobajar",
    "descobajaríamos": "descobajar",
    "descobajarían": "descobajar",
    "descobajarías": "descobajar",
    "descobajas": "descobajar",
    "descobajase": "descobajar",
    "descobajaseis": "descobajar",
    "descobajasen": "descobajar",
    "descobajases": "descobajar",
    "descobajaste": "descobajar",
    "descobajasteis": "descobajar",
    "descobaje": "descobajar",
    "descobajemos": "descobajar",
    "descobajen": "descobajar",
    "descobajes": "descobajar",
    "descobajo": "descobajar",
    "descobajábamos": "descobajar",
    "descobajáis": "descobajar",
    "descobajáramos": "descobajar",
    "descobajáremos": "descobajar",
    "descobajásemos": "descobajar",
    "descobajé": "descobajar",
    "descobajéis": "descobajar",
    "descobajó": "descobajar",
    "descobija": "descobijar",
    "descobijaba": "descobijar",
    "descobijabais": "descobijar",
    "descobijaban": "descobijar",
    "descobijabas": "descobijar",
    "descobijad": "descobijar",
    "descobijada": "descobijar",
    "descobijadas": "descobijar",
    "descobijado": "descobijar",
    "descobijados": "descobijar",
    "descobijamos": "descobijar",
    "descobijan": "descobijar",
    "descobijando": "descobijar",
    "descobijara": "descobijar",
    "descobijarais": "descobijar",
    "descobijaran": "descobijar",
    "descobijaras": "descobijar",
    "descobijare": "descobijar",
    "descobijareis": "descobijar",
    "descobijaremos": "descobijar",
    "descobijaren": "descobijar",
    "descobijares": "descobijar",
    "descobijaron": "descobijar",
    "descobijará": "descobijar",
    "descobijarán": "descobijar",
    "descobijarás": "descobijar",
    "descobijaré": "descobijar",
    "descobijaréis": "descobijar",
    "descobijaría": "descobijar",
    "descobijaríais": "descobijar",
    "descobijaríamos": "descobijar",
    "descobijarían": "descobijar",
    "descobijarías": "descobijar",
    "descobijas": "descobijar",
    "descobijase": "descobijar",
    "descobijaseis": "descobijar",
    "descobijasen": "descobijar",
    "descobijases": "descobijar",
    "descobijaste": "descobijar",
    "descobijasteis": "descobijar",
    "descobije": "descobijar",
    "descobijemos": "descobijar",
    "descobijen": "descobijar",
    "descobijes": "descobijar",
    "descobijo": "descobijar",
    "descobijábamos": "descobijar",
    "descobijáis": "descobijar",
    "descobijáramos": "descobijar",
    "descobijáremos": "descobijar",
    "descobijásemos": "descobijar",
    "descobijé": "descobijar",
    "descobijéis": "descobijar",
    "descobijó": "descobijar",
    "descoca": "descocar",
    "descocaba": "descocar",
    "descocabais": "descocar",
    "descocaban": "descocar",
    "descocabas": "descocar",
    "descocad": "descocar",
    "descocada": "descocar",
    "descocadas": "descocar",
    "descocado": "descocar",
    "descocadores": "descocador",
    "descocados": "descocar",
    "descocamientos": "descocamiento",
    "descocamos": "descocar",
    "descocan": "descocar",
    "descocando": "descocar",
    "descocara": "descocar",
    "descocarais": "descocar",
    "descocaran": "descocar",
    "descocaras": "descocar",
    "descocare": "descocar",
    "descocareis": "descocar",
    "descocaremos": "descocar",
    "descocaren": "descocar",
    "descocares": "descocar",
    "descocaron": "descocar",
    "descocará": "descocar",
    "descocarán": "descocar",
    "descocarás": "descocar",
    "descocaré": "descocar",
    "descocaréis": "descocar",
    "descocaría": "descocar",
    "descocaríais": "descocar",
    "descocaríamos": "descocar",
    "descocarían": "descocar",
    "descocarías": "descocar",
    "descocas": "descocar",
    "descocase": "descocar",
    "descocaseis": "descocar",
    "descocasen": "descocar",
    "descocases": "descocar",
    "descocaste": "descocar",
    "descocasteis": "descocar",
    "descoco": "descocar",
    "descocábamos": "descocar",
    "descocáis": "descocar",
    "descocáramos": "descocar",
    "descocáremos": "descocar",
    "descocásemos": "descocar",
    "descocó": "descocar",
    "descodifica": "descodificar",
    "descodificaba": "descodificar",
    "descodificabais": "descodificar",
    "descodificaban": "descodificar",
    "descodificabas": "descodificar",
    "descodificaciones": "descodificación",
    "descodificad": "descodificar",
    "descodificada": "descodificar",
    "descodificadas": "descodificar",
    "descodificado": "descodificar",
    "descodificadores": "descodificador",
    "descodificados": "descodificar",
    "descodificamos": "descodificar",
    "descodifican": "descodificar",
    "descodificando": "descodificar",
    "descodificara": "descodificar",
    "descodificarais": "descodificar",
    "descodificaran": "descodificar",
    "descodificaras": "descodificar",
    "descodificare": "descodificar",
    "descodificareis": "descodificar",
    "descodificaremos": "descodificar",
    "descodificaren": "descodificar",
    "descodificares": "descodificar",
    "descodificaron": "descodificar",
    "descodificará": "descodificar",
    "descodificarán": "descodificar",
    "descodificarás": "descodificar",
    "descodificaré": "descodificar",
    "descodificaréis": "descodificar",
    "descodificaría": "descodificar",
    "descodificaríais": "descodificar",
    "descodificaríamos": "descodificar",
    "descodificarían": "descodificar",
    "descodificarías": "descodificar",
    "descodificas": "descodificar",
    "descodificase": "descodificar",
    "descodificaseis": "descodificar",
    "descodificasen": "descodificar",
    "descodificases": "descodificar",
    "descodificaste": "descodificar",
    "descodificasteis": "descodificar",
    "descodifico": "descodificar",
    "descodificábamos": "descodificar",
    "descodificáis": "descodificar",
    "descodificáramos": "descodificar",
    "descodificáremos": "descodificar",
    "descodificásemos": "descodificar",
    "descodificó": "descodificar",
    "descodifique": "descodificar",
    "descodifiquemos": "descodificar",
    "descodifiquen": "descodificar",
    "descodifiques": "descodificar",
    "descodifiqué": "descodificar",
    "descodifiquéis": "descodificar",
    "descoge": "descoger",
    "descoged": "descoger",
    "descogemos": "descoger",
    "descogen": "descoger",
    "descogeremos": "descoger",
    "descogerá": "descoger",
    "descogerán": "descoger",
    "descogerás": "descoger",
    "descogeré": "descoger",
    "descogeréis": "descoger",
    "descogería": "descoger",
    "descogeríais": "descoger",
    "descogeríamos": "descoger",
    "descogerían": "descoger",
    "descogerías": "descoger",
    "descoges": "descoger",
    "descogida": "descoger",
    "descogidas": "descoger",
    "descogido": "descoger",
    "descogidos": "descoger",
    "descogiendo": "descoger",
    "descogiera": "descoger",
    "descogierais": "descoger",
    "descogieran": "descoger",
    "descogieras": "descoger",
    "descogiere": "descoger",
    "descogiereis": "descoger",
    "descogieren": "descoger",
    "descogieres": "descoger",
    "descogieron": "descoger",
    "descogiese": "descoger",
    "descogieseis": "descoger",
    "descogiesen": "descoger",
    "descogieses": "descoger",
    "descogimos": "descoger",
    "descogiste": "descoger",
    "descogisteis": "descoger",
    "descogiéramos": "descoger",
    "descogiéremos": "descoger",
    "descogiésemos": "descoger",
    "descogió": "descoger",
    "descogolla": "descogollar",
    "descogollaba": "descogollar",
    "descogollabais": "descogollar",
    "descogollaban": "descogollar",
    "descogollabas": "descogollar",
    "descogollad": "descogollar",
    "descogollada": "descogollar",
    "descogolladas": "descogollar",
    "descogollado": "descogollar",
    "descogollados": "descogollar",
    "descogollamos": "descogollar",
    "descogollan": "descogollar",
    "descogollando": "descogollar",
    "descogollara": "descogollar",
    "descogollarais": "descogollar",
    "descogollaran": "descogollar",
    "descogollaras": "descogollar",
    "descogollare": "descogollar",
    "descogollareis": "descogollar",
    "descogollaremos": "descogollar",
    "descogollaren": "descogollar",
    "descogollares": "descogollar",
    "descogollaron": "descogollar",
    "descogollará": "descogollar",
    "descogollarán": "descogollar",
    "descogollarás": "descogollar",
    "descogollaré": "descogollar",
    "descogollaréis": "descogollar",
    "descogollaría": "descogollar",
    "descogollaríais": "descogollar",
    "descogollaríamos": "descogollar",
    "descogollarían": "descogollar",
    "descogollarías": "descogollar",
    "descogollas": "descogollar",
    "descogollase": "descogollar",
    "descogollaseis": "descogollar",
    "descogollasen": "descogollar",
    "descogollases": "descogollar",
    "descogollaste": "descogollar",
    "descogollasteis": "descogollar",
    "descogolle": "descogollar",
    "descogollemos": "descogollar",
    "descogollen": "descogollar",
    "descogolles": "descogollar",
    "descogollo": "descogollar",
    "descogollábamos": "descogollar",
    "descogolláis": "descogollar",
    "descogolláramos": "descogollar",
    "descogolláremos": "descogollar",
    "descogollásemos": "descogollar",
    "descogollé": "descogollar",
    "descogolléis": "descogollar",
    "descogolló": "descogollar",
    "descogota": "descogotar",
    "descogotaba": "descogotar",
    "descogotabais": "descogotar",
    "descogotaban": "descogotar",
    "descogotabas": "descogotar",
    "descogotad": "descogotar",
    "descogotada": "descogotar",
    "descogotadas": "descogotar",
    "descogotado": "descogotar",
    "descogotados": "descogotar",
    "descogotamos": "descogotar",
    "descogotan": "descogotar",
    "descogotando": "descogotar",
    "descogotara": "descogotar",
    "descogotarais": "descogotar",
    "descogotaran": "descogotar",
    "descogotaras": "descogotar",
    "descogotare": "descogotar",
    "descogotareis": "descogotar",
    "descogotaremos": "descogotar",
    "descogotaren": "descogotar",
    "descogotares": "descogotar",
    "descogotaron": "descogotar",
    "descogotará": "descogotar",
    "descogotarán": "descogotar",
    "descogotarás": "descogotar",
    "descogotaré": "descogotar",
    "descogotaréis": "descogotar",
    "descogotaría": "descogotar",
    "descogotaríais": "descogotar",
    "descogotaríamos": "descogotar",
    "descogotarían": "descogotar",
    "descogotarías": "descogotar",
    "descogotas": "descogotar",
    "descogotase": "descogotar",
    "descogotaseis": "descogotar",
    "descogotasen": "descogotar",
    "descogotases": "descogotar",
    "descogotaste": "descogotar",
    "descogotasteis": "descogotar",
    "descogote": "descogotar",
    "descogotemos": "descogotar",
    "descogoten": "descogotar",
    "descogotes": "descogotar",
    "descogoto": "descogotar",
    "descogotábamos": "descogotar",
    "descogotáis": "descogotar",
    "descogotáramos": "descogotar",
    "descogotáremos": "descogotar",
    "descogotásemos": "descogotar",
    "descogoté": "descogotar",
    "descogotéis": "descogotar",
    "descogotó": "descogotar",
    "descogéis": "descoger",
    "descogí": "descoger",
    "descogía": "descoger",
    "descogíais": "descoger",
    "descogíamos": "descoger",
    "descogían": "descoger",
    "descogías": "descoger",
    "descohesionada": "descohesionado",
    "descohesionadas": "descohesionado",
    "descohesionados": "descohesionado",
    "descoja": "descoger",
    "descojamos": "descoger",
    "descojan": "descoger",
    "descojas": "descoger",
    "descojo": "descoger",
    "descojáis": "descoger",
    "descola": "descolar",
    "descolaba": "descolar",
    "descolabais": "descolar",
    "descolaban": "descolar",
    "descolabas": "descolar",
    "descolad": "descolar",
    "descolada": "descolar",
    "descoladas": "descolar",
    "descolado": "descolar",
    "descolados": "descolar",
    "descolamos": "descolar",
    "descolan": "descolar",
    "descolando": "descolar",
    "descolara": "descolar",
    "descolarais": "descolar",
    "descolaran": "descolar",
    "descolaras": "descolar",
    "descolare": "descolar",
    "descolareis": "descolar",
    "descolaremos": "descolar",
    "descolaren": "descolar",
    "descolares": "descolar",
    "descolaron": "descolar",
    "descolará": "descolar",
    "descolarán": "descolar",
    "descolarás": "descolar",
    "descolaré": "descolar",
    "descolaréis": "descolar",
    "descolaría": "descolar",
    "descolaríais": "descolar",
    "descolaríamos": "descolar",
    "descolarían": "descolar",
    "descolarías": "descolar",
    "descolas": "descolar",
    "descolase": "descolar",
    "descolaseis": "descolar",
    "descolasen": "descolar",
    "descolases": "descolar",
    "descolaste": "descolar",
    "descolasteis": "descolar",
    "descolcha": "descolchar",
    "descolchaba": "descolchar",
    "descolchabais": "descolchar",
    "descolchaban": "descolchar",
    "descolchabas": "descolchar",
    "descolchad": "descolchar",
    "descolchada": "descolchar",
    "descolchadas": "descolchar",
    "descolchado": "descolchar",
    "descolchados": "descolchar",
    "descolchamos": "descolchar",
    "descolchan": "descolchar",
    "descolchando": "descolchar",
    "descolchara": "descolchar",
    "descolcharais": "descolchar",
    "descolcharan": "descolchar",
    "descolcharas": "descolchar",
    "descolchare": "descolchar",
    "descolchareis": "descolchar",
    "descolcharemos": "descolchar",
    "descolcharen": "descolchar",
    "descolchares": "descolchar",
    "descolcharon": "descolchar",
    "descolchará": "descolchar",
    "descolcharán": "descolchar",
    "descolcharás": "descolchar",
    "descolcharé": "descolchar",
    "descolcharéis": "descolchar",
    "descolcharía": "descolchar",
    "descolcharíais": "descolchar",
    "descolcharíamos": "descolchar",
    "descolcharían": "descolchar",
    "descolcharías": "descolchar",
    "descolchas": "descolchar",
    "descolchase": "descolchar",
    "descolchaseis": "descolchar",
    "descolchasen": "descolchar",
    "descolchases": "descolchar",
    "descolchaste": "descolchar",
    "descolchasteis": "descolchar",
    "descolche": "descolchar",
    "descolchemos": "descolchar",
    "descolchen": "descolchar",
    "descolches": "descolchar",
    "descolcho": "descolchar",
    "descolchábamos": "descolchar",
    "descolcháis": "descolchar",
    "descolcháramos": "descolchar",
    "descolcháremos": "descolchar",
    "descolchásemos": "descolchar",
    "descolché": "descolchar",
    "descolchéis": "descolchar",
    "descolchó": "descolchar",
    "descole": "descolar",
    "descolemos": "descolar",
    "descolen": "descolar",
    "descoles": "descolar",
    "descolgaba": "descolgar",
    "descolgabais": "descolgar",
    "descolgaban": "descolgar",
    "descolgabas": "descolgar",
    "descolgad": "descolgar",
    "descolgada": "descolgar",
    "descolgadas": "descolgar",
    "descolgado": "descolgar",
    "descolgados": "descolgar",
    "descolgamientos": "descolgamiento",
    "descolgamos": "descolgar",
    "descolgando": "descolgar",
    "descolgara": "descolgar",
    "descolgarais": "descolgar",
    "descolgaran": "descolgar",
    "descolgaras": "descolgar",
    "descolgare": "descolgar",
    "descolgareis": "descolgar",
    "descolgaremos": "descolgar",
    "descolgaren": "descolgar",
    "descolgares": "descolgar",
    "descolgaron": "descolgar",
    "descolgará": "descolgar",
    "descolgarán": "descolgar",
    "descolgarás": "descolgar",
    "descolgaré": "descolgar",
    "descolgaréis": "descolgar",
    "descolgaría": "descolgar",
    "descolgaríais": "descolgar",
    "descolgaríamos": "descolgar",
    "descolgarían": "descolgar",
    "descolgarías": "descolgar",
    "descolgase": "descolgar",
    "descolgaseis": "descolgar",
    "descolgasen": "descolgar",
    "descolgases": "descolgar",
    "descolgaste": "descolgar",
    "descolgasteis": "descolgar",
    "descolguemos": "descolgar",
    "descolgué": "descolgar",
    "descolguéis": "descolgar",
    "descolgábamos": "descolgar",
    "descolgáis": "descolgar",
    "descolgáramos": "descolgar",
    "descolgáremos": "descolgar",
    "descolgásemos": "descolgar",
    "descolgó": "descolgar",
    "descoligada": "descoligado",
    "descoligadas": "descoligado",
    "descoligados": "descoligado",
    "descollaba": "descollar",
    "descollabais": "descollar",
    "descollaban": "descollar",
    "descollabas": "descollar",
    "descollad": "descollar",
    "descollada": "descollar",
    "descolladas": "descollar",
    "descollado": "descollar",
    "descollados": "descollar",
    "descollamientos": "descollamiento",
    "descollamos": "descollar",
    "descollando": "descollar",
    "descollara": "descollar",
    "descollarais": "descollar",
    "descollaran": "descollar",
    "descollaras": "descollar",
    "descollare": "descollar",
    "descollareis": "descollar",
    "descollaremos": "descollar",
    "descollaren": "descollar",
    "descollares": "descollar",
    "descollaron": "descollar",
    "descollará": "descollar",
    "descollarán": "descollar",
    "descollarás": "descollar",
    "descollaré": "descollar",
    "descollaréis": "descollar",
    "descollaría": "descollar",
    "descollaríais": "descollar",
    "descollaríamos": "descollar",
    "descollarían": "descollar",
    "descollarías": "descollar",
    "descollase": "descollar",
    "descollaseis": "descollar",
    "descollasen": "descollar",
    "descollases": "descollar",
    "descollaste": "descollar",
    "descollasteis": "descollar",
    "descollemos": "descollar",
    "descollábamos": "descollar",
    "descolláis": "descollar",
    "descolláramos": "descollar",
    "descolláremos": "descollar",
    "descollásemos": "descollar",
    "descollé": "descollar",
    "descolléis": "descollar",
    "descolló": "descollar",
    "descolmatajes": "descolmataje",
    "descolmilla": "descolmillar",
    "descolmillaba": "descolmillar",
    "descolmillabais": "descolmillar",
    "descolmillaban": "descolmillar",
    "descolmillabas": "descolmillar",
    "descolmillad": "descolmillar",
    "descolmillada": "descolmillar",
    "descolmilladas": "descolmillar",
    "descolmillado": "descolmillar",
    "descolmillados": "descolmillar",
    "descolmillamos": "descolmillar",
    "descolmillan": "descolmillar",
    "descolmillando": "descolmillar",
    "descolmillara": "descolmillar",
    "descolmillarais": "descolmillar",
    "descolmillaran": "descolmillar",
    "descolmillaras": "descolmillar",
    "descolmillare": "descolmillar",
    "descolmillareis": "descolmillar",
    "descolmillaremos": "descolmillar",
    "descolmillaren": "descolmillar",
    "descolmillares": "descolmillar",
    "descolmillaron": "descolmillar",
    "descolmillará": "descolmillar",
    "descolmillarán": "descolmillar",
    "descolmillarás": "descolmillar",
    "descolmillaré": "descolmillar",
    "descolmillaréis": "descolmillar",
    "descolmillaría": "descolmillar",
    "descolmillaríais": "descolmillar",
    "descolmillaríamos": "descolmillar",
    "descolmillarían": "descolmillar",
    "descolmillarías": "descolmillar",
    "descolmillas": "descolmillar",
    "descolmillase": "descolmillar",
    "descolmillaseis": "descolmillar",
    "descolmillasen": "descolmillar",
    "descolmillases": "descolmillar",
    "descolmillaste": "descolmillar",
    "descolmillasteis": "descolmillar",
    "descolmille": "descolmillar",
    "descolmillemos": "descolmillar",
    "descolmillen": "descolmillar",
    "descolmilles": "descolmillar",
    "descolmillo": "descolmillar",
    "descolmillábamos": "descolmillar",
    "descolmilláis": "descolmillar",
    "descolmilláramos": "descolmillar",
    "descolmilláremos": "descolmillar",
    "descolmillásemos": "descolmillar",
    "descolmillé": "descolmillar",
    "descolmilléis": "descolmillar",
    "descolmilló": "descolmillar",
    "descolo": "descolar",
    "descoloca": "descolocar",
    "descolocaba": "descolocar",
    "descolocabais": "descolocar",
    "descolocaban": "descolocar",
    "descolocabas": "descolocar",
    "descolocad": "descolocar",
    "descolocada": "descolocar",
    "descolocadas": "descolocar",
    "descolocado": "descolocar",
    "descolocados": "descolocar",
    "descolocamos": "descolocar",
    "descolocan": "descolocar",
    "descolocando": "descolocar",
    "descolocara": "descolocar",
    "descolocarais": "descolocar",
    "descolocaran": "descolocar",
    "descolocaras": "descolocar",
    "descolocare": "descolocar",
    "descolocareis": "descolocar",
    "descolocaremos": "descolocar",
    "descolocaren": "descolocar",
    "descolocares": "descolocar",
    "descolocaron": "descolocar",
    "descolocará": "descolocar",
    "descolocarán": "descolocar",
    "descolocarás": "descolocar",
    "descolocaré": "descolocar",
    "descolocaréis": "descolocar",
    "descolocaría": "descolocar",
    "descolocaríais": "descolocar",
    "descolocaríamos": "descolocar",
    "descolocarían": "descolocar",
    "descolocarías": "descolocar",
    "descolocas": "descolocar",
    "descolocase": "descolocar",
    "descolocaseis": "descolocar",
    "descolocasen": "descolocar",
    "descolocases": "descolocar",
    "descolocaste": "descolocar",
    "descolocasteis": "descolocar",
    "descoloco": "descolocar",
    "descolocábamos": "descolocar",
    "descolocáis": "descolocar",
    "descolocáramos": "descolocar",
    "descolocáremos": "descolocar",
    "descolocásemos": "descolocar",
    "descolocó": "descolocar",
    "descolonice": "descolonizar",
    "descolonicemos": "descolonizar",
    "descolonicen": "descolonizar",
    "descolonices": "descolonizar",
    "descolonicé": "descolonizar",
    "descolonicéis": "descolonizar",
    "descoloniza": "descolonizar",
    "descolonizaba": "descolonizar",
    "descolonizabais": "descolonizar",
    "descolonizaban": "descolonizar",
    "descolonizabas": "descolonizar",
    "descolonizaciones": "descolonización",
    "descolonizad": "descolonizar",
    "descolonizada": "descolonizar",
    "descolonizadas": "descolonizar",
    "descolonizado": "descolonizar",
    "descolonizadora": "descolonizador",
    "descolonizadoras": "descolonizador",
    "descolonizadores": "descolonizador",
    "descolonizados": "descolonizar",
    "descolonizamos": "descolonizar",
    "descolonizan": "descolonizar",
    "descolonizando": "descolonizar",
    "descolonizara": "descolonizar",
    "descolonizarais": "descolonizar",
    "descolonizaran": "descolonizar",
    "descolonizaras": "descolonizar",
    "descolonizare": "descolonizar",
    "descolonizareis": "descolonizar",
    "descolonizaremos": "descolonizar",
    "descolonizaren": "descolonizar",
    "descolonizares": "descolonizar",
    "descolonizaron": "descolonizar",
    "descolonizará": "descolonizar",
    "descolonizarán": "descolonizar",
    "descolonizarás": "descolonizar",
    "descolonizaré": "descolonizar",
    "descolonizaréis": "descolonizar",
    "descolonizaría": "descolonizar",
    "descolonizaríais": "descolonizar",
    "descolonizaríamos": "descolonizar",
    "descolonizarían": "descolonizar",
    "descolonizarías": "descolonizar",
    "descolonizas": "descolonizar",
    "descolonizase": "descolonizar",
    "descolonizaseis": "descolonizar",
    "descolonizasen": "descolonizar",
    "descolonizases": "descolonizar",
    "descolonizaste": "descolonizar",
    "descolonizasteis": "descolonizar",
    "descolonizo": "descolonizar",
    "descolonizábamos": "descolonizar",
    "descolonizáis": "descolonizar",
    "descolonizáramos": "descolonizar",
    "descolonizáremos": "descolonizar",
    "descolonizásemos": "descolonizar",
    "descolonizó": "descolonizar",
    "descoloque": "descolocar",
    "descoloquemos": "descolocar",
    "descoloquen": "descolocar",
    "descoloques": "descolocar",
    "descoloqué": "descolocar",
    "descoloquéis": "descolocar",
    "descolora": "descolorir",
    "descoloraba": "descolorar",
    "descolorabais": "descolorar",
    "descoloraban": "descolorar",
    "descolorabas": "descolorar",
    "descolorad": "descolorar",
    "descolorada": "descolorar",
    "descoloradas": "descolorar",
    "descolorado": "descolorar",
    "descolorados": "descolorar",
    "descoloramientos": "descoloramiento",
    "descoloramos": "descolorir",
    "descoloran": "descolorir",
    "descolorando": "descolorar",
    "descolorara": "descolorar",
    "descolorarais": "descolorar",
    "descoloraran": "descolorar",
    "descoloraras": "descolorar",
    "descolorare": "descolorar",
    "descolorareis": "descolorar",
    "descoloraremos": "descolorar",
    "descoloraren": "descolorar",
    "descolorares": "descolorar",
    "descoloraron": "descolorar",
    "descolorará": "descolorar",
    "descolorarán": "descolorar",
    "descolorarás": "descolorar",
    "descoloraré": "descolorar",
    "descoloraréis": "descolorar",
    "descoloraría": "descolorar",
    "descoloraríais": "descolorar",
    "descoloraríamos": "descolorar",
    "descolorarían": "descolorar",
    "descolorarías": "descolorar",
    "descoloras": "descolorir",
    "descolorase": "descolorar",
    "descoloraseis": "descolorar",
    "descolorasen": "descolorar",
    "descolorases": "descolorar",
    "descoloraste": "descolorar",
    "descolorasteis": "descolorar",
    "descolore": "descolorir",
    "descoloremos": "descolorar",
    "descoloren": "descolorir",
    "descolores": "descolorir",
    "descolorid": "descolorir",
    "descolorida": "descolorir",
    "descoloridas": "descolorir",
    "descolorido": "descolorir",
    "descoloridos": "descolorir",
    "descoloriendo": "descolorir",
    "descoloriera": "descolorir",
    "descolorierais": "descolorir",
    "descolorieran": "descolorir",
    "descolorieras": "descolorir",
    "descoloriere": "descolorir",
    "descoloriereis": "descolorir",
    "descolorieren": "descolorir",
    "descolorieres": "descolorir",
    "descolorieron": "descolorir",
    "descoloriese": "descolorir",
    "descolorieseis": "descolorir",
    "descoloriesen": "descolorir",
    "descolorieses": "descolorir",
    "descolorimos": "descolorir",
    "descoloriremos": "descolorir",
    "descolorirá": "descolorir",
    "descolorirán": "descolorir",
    "descolorirás": "descolorir",
    "descoloriré": "descolorir",
    "descoloriréis": "descolorir",
    "descoloriría": "descolorir",
    "descoloriríais": "descolorir",
    "descoloriríamos": "descolorir",
    "descolorirían": "descolorir",
    "descolorirías": "descolorir",
    "descoloriste": "descolorir",
    "descoloristeis": "descolorir",
    "descoloriéramos": "descolorir",
    "descoloriéremos": "descolorir",
    "descoloriésemos": "descolorir",
    "descolorió": "descolorir",
    "descoloro": "descolorir",
    "descolorábamos": "descolorar",
    "descoloráis": "descolorir",
    "descoloráramos": "descolorar",
    "descoloráremos": "descolorar",
    "descolorásemos": "descolorar",
    "descoloré": "descolorar",
    "descoloréis": "descolorar",
    "descolorí": "descolorir",
    "descoloría": "descolorir",
    "descoloríais": "descolorir",
    "descoloríamos": "descolorir",
    "descolorían": "descolorir",
    "descolorías": "descolorir",
    "descolorís": "descolorir",
    "descoloró": "descolorar",
    "descolábamos": "descolar",
    "descoláis": "descolar",
    "descoláramos": "descolar",
    "descoláremos": "descolar",
    "descolásemos": "descolar",
    "descolé": "descolar",
    "descoléis": "descolar",
    "descoló": "descolar",
    "descombra": "descombrar",
    "descombraba": "descombrar",
    "descombrabais": "descombrar",
    "descombraban": "descombrar",
    "descombrabas": "descombrar",
    "descombrad": "descombrar",
    "descombrada": "descombrar",
    "descombradas": "descombrar",
    "descombrado": "descombrar",
    "descombrados": "descombrar",
    "descombramos": "descombrar",
    "descombran": "descombrar",
    "descombrando": "descombrar",
    "descombrara": "descombrar",
    "descombrarais": "descombrar",
    "descombraran": "descombrar",
    "descombraras": "descombrar",
    "descombrare": "descombrar",
    "descombrareis": "descombrar",
    "descombraremos": "descombrar",
    "descombraren": "descombrar",
    "descombrares": "descombrar",
    "descombraron": "descombrar",
    "descombrará": "descombrar",
    "descombrarán": "descombrar",
    "descombrarás": "descombrar",
    "descombraré": "descombrar",
    "descombraréis": "descombrar",
    "descombraría": "descombrar",
    "descombraríais": "descombrar",
    "descombraríamos": "descombrar",
    "descombrarían": "descombrar",
    "descombrarías": "descombrar",
    "descombras": "descombrar",
    "descombrase": "descombrar",
    "descombraseis": "descombrar",
    "descombrasen": "descombrar",
    "descombrases": "descombrar",
    "descombraste": "descombrar",
    "descombrasteis": "descombrar",
    "descombre": "descombrar",
    "descombremos": "descombrar",
    "descombren": "descombrar",
    "descombres": "descombrar",
    "descombro": "descombrar",
    "descombros": "descombro",
    "descombrábamos": "descombrar",
    "descombráis": "descombrar",
    "descombráramos": "descombrar",
    "descombráremos": "descombrar",
    "descombrásemos": "descombrar",
    "descombré": "descombrar",
    "descombréis": "descombrar",
    "descombró": "descombrar",
    "descomedid": "descomedir",
    "descomedida": "descomedir",
    "descomedidas": "descomedir",
    "descomedido": "descomedir",
    "descomedidos": "descomedir",
    "descomedimientos": "descomedimiento",
    "descomedimos": "descomedir",
    "descomediremos": "descomedir",
    "descomedirá": "descomedir",
    "descomedirán": "descomedir",
    "descomedirás": "descomedir",
    "descomediré": "descomedir",
    "descomediréis": "descomedir",
    "descomediría": "descomedir",
    "descomediríais": "descomedir",
    "descomediríamos": "descomedir",
    "descomedirían": "descomedir",
    "descomedirías": "descomedir",
    "descomediste": "descomedir",
    "descomedisteis": "descomedir",
    "descomedí": "descomedir",
    "descomedía": "descomedir",
    "descomedíais": "descomedir",
    "descomedíamos": "descomedir",
    "descomedían": "descomedir",
    "descomedías": "descomedir",
    "descomedís": "descomedir",
    "descomida": "descomedir",
    "descomidamos": "descomedir",
    "descomidan": "descomedir",
    "descomidas": "descomedir",
    "descomide": "descomedir",
    "descomiden": "descomedir",
    "descomides": "descomedir",
    "descomidiendo": "descomedir",
    "descomidiera": "descomedir",
    "descomidierais": "descomedir",
    "descomidieran": "descomedir",
    "descomidieras": "descomedir",
    "descomidiere": "descomedir",
    "descomidiereis": "descomedir",
    "descomidieren": "descomedir",
    "descomidieres": "descomedir",
    "descomidieron": "descomedir",
    "descomidiese": "descomedir",
    "descomidieseis": "descomedir",
    "descomidiesen": "descomedir",
    "descomidieses": "descomedir",
    "descomidiéramos": "descomedir",
    "descomidiéremos": "descomedir",
    "descomidiésemos": "descomedir",
    "descomidió": "descomedir",
    "descomido": "descomedir",
    "descomidáis": "descomedir",
    "descompadra": "descompadrar",
    "descompadraba": "descompadrar",
    "descompadrabais": "descompadrar",
    "descompadraban": "descompadrar",
    "descompadrabas": "descompadrar",
    "descompadrad": "descompadrar",
    "descompadrada": "descompadrar",
    "descompadradas": "descompadrar",
    "descompadrado": "descompadrar",
    "descompadrados": "descompadrar",
    "descompadramos": "descompadrar",
    "descompadran": "descompadrar",
    "descompadrando": "descompadrar",
    "descompadrara": "descompadrar",
    "descompadrarais": "descompadrar",
    "descompadraran": "descompadrar",
    "descompadraras": "descompadrar",
    "descompadrare": "descompadrar",
    "descompadrareis": "descompadrar",
    "descompadraremos": "descompadrar",
    "descompadraren": "descompadrar",
    "descompadrares": "descompadrar",
    "descompadraron": "descompadrar",
    "descompadrará": "descompadrar",
    "descompadrarán": "descompadrar",
    "descompadrarás": "descompadrar",
    "descompadraré": "descompadrar",
    "descompadraréis": "descompadrar",
    "descompadraría": "descompadrar",
    "descompadraríais": "descompadrar",
    "descompadraríamos": "descompadrar",
    "descompadrarían": "descompadrar",
    "descompadrarías": "descompadrar",
    "descompadras": "descompadrar",
    "descompadrase": "descompadrar",
    "descompadraseis": "descompadrar",
    "descompadrasen": "descompadrar",
    "descompadrases": "descompadrar",
    "descompadraste": "descompadrar",
    "descompadrasteis": "descompadrar",
    "descompadre": "descompadrar",
    "descompadremos": "descompadrar",
    "descompadren": "descompadrar",
    "descompadres": "descompadrar",
    "descompadro": "descompadrar",
    "descompadrábamos": "descompadrar",
    "descompadráis": "descompadrar",
    "descompadráramos": "descompadrar",
    "descompadráremos": "descompadrar",
    "descompadrásemos": "descompadrar",
    "descompadré": "descompadrar",
    "descompadréis": "descompadrar",
    "descompadró": "descompadrar",
    "descompagina": "descompaginar",
    "descompaginaba": "descompaginar",
    "descompaginabais": "descompaginar",
    "descompaginaban": "descompaginar",
    "descompaginabas": "descompaginar",
    "descompaginaciones": "descompaginación",
    "descompaginad": "descompaginar",
    "descompaginada": "descompaginar",
    "descompaginadas": "descompaginar",
    "descompaginado": "descompaginar",
    "descompaginados": "descompaginar",
    "descompaginamos": "descompaginar",
    "descompaginan": "descompaginar",
    "descompaginando": "descompaginar",
    "descompaginara": "descompaginar",
    "descompaginarais": "descompaginar",
    "descompaginaran": "descompaginar",
    "descompaginaras": "descompaginar",
    "descompaginare": "descompaginar",
    "descompaginareis": "descompaginar",
    "descompaginaremos": "descompaginar",
    "descompaginaren": "descompaginar",
    "descompaginares": "descompaginar",
    "descompaginaron": "descompaginar",
    "descompaginará": "descompaginar",
    "descompaginarán": "descompaginar",
    "descompaginarás": "descompaginar",
    "descompaginaré": "descompaginar",
    "descompaginaréis": "descompaginar",
    "descompaginaría": "descompaginar",
    "descompaginaríais": "descompaginar",
    "descompaginaríamos": "descompaginar",
    "descompaginarían": "descompaginar",
    "descompaginarías": "descompaginar",
    "descompaginas": "descompaginar",
    "descompaginase": "descompaginar",
    "descompaginaseis": "descompaginar",
    "descompaginasen": "descompaginar",
    "descompaginases": "descompaginar",
    "descompaginaste": "descompaginar",
    "descompaginasteis": "descompaginar",
    "descompagine": "descompaginar",
    "descompaginemos": "descompaginar",
    "descompaginen": "descompaginar",
    "descompagines": "descompaginar",
    "descompagino": "descompaginar",
    "descompaginábamos": "descompaginar",
    "descompagináis": "descompaginar",
    "descompagináramos": "descompaginar",
    "descompagináremos": "descompaginar",
    "descompaginásemos": "descompaginar",
    "descompaginé": "descompaginar",
    "descompaginéis": "descompaginar",
    "descompaginó": "descompaginar",
    "descompasa": "descompasar",
    "descompasaba": "descompasar",
    "descompasabais": "descompasar",
    "descompasaban": "descompasar",
    "descompasabas": "descompasar",
    "descompasad": "descompasar",
    "descompasada": "descompasar",
    "descompasadas": "descompasar",
    "descompasado": "descompasar",
    "descompasados": "descompasar",
    "descompasamos": "descompasar",
    "descompasan": "descompasar",
    "descompasando": "descompasar",
    "descompasara": "descompasar",
    "descompasarais": "descompasar",
    "descompasaran": "descompasar",
    "descompasaras": "descompasar",
    "descompasare": "descompasar",
    "descompasareis": "descompasar",
    "descompasaremos": "descompasar",
    "descompasaren": "descompasar",
    "descompasares": "descompasar",
    "descompasaron": "descompasar",
    "descompasará": "descompasar",
    "descompasarán": "descompasar",
    "descompasarás": "descompasar",
    "descompasaré": "descompasar",
    "descompasaréis": "descompasar",
    "descompasaría": "descompasar",
    "descompasaríais": "descompasar",
    "descompasaríamos": "descompasar",
    "descompasarían": "descompasar",
    "descompasarías": "descompasar",
    "descompasas": "descompasar",
    "descompasase": "descompasar",
    "descompasaseis": "descompasar",
    "descompasasen": "descompasar",
    "descompasases": "descompasar",
    "descompasaste": "descompasar",
    "descompasasteis": "descompasar",
    "descompase": "descompasar",
    "descompasemos": "descompasar",
    "descompasen": "descompasar",
    "descompases": "descompasar",
    "descompaso": "descompasar",
    "descompasábamos": "descompasar",
    "descompasáis": "descompasar",
    "descompasáramos": "descompasar",
    "descompasáremos": "descompasar",
    "descompasásemos": "descompasar",
    "descompasé": "descompasar",
    "descompaséis": "descompasar",
    "descompasó": "descompasar",
    "descompensa": "descompensar",
    "descompensaba": "descompensar",
    "descompensabais": "descompensar",
    "descompensaban": "descompensar",
    "descompensabas": "descompensar",
    "descompensaciones": "descompensación",
    "descompensad": "descompensar",
    "descompensada": "descompensar",
    "descompensadas": "descompensar",
    "descompensado": "descompensar",
    "descompensados": "descompensar",
    "descompensamos": "descompensar",
    "descompensan": "descompensar",
    "descompensando": "descompensar",
    "descompensara": "descompensar",
    "descompensarais": "descompensar",
    "descompensaran": "descompensar",
    "descompensaras": "descompensar",
    "descompensare": "descompensar",
    "descompensareis": "descompensar",
    "descompensaremos": "descompensar",
    "descompensaren": "descompensar",
    "descompensares": "descompensar",
    "descompensaron": "descompensar",
    "descompensará": "descompensar",
    "descompensarán": "descompensar",
    "descompensarás": "descompensar",
    "descompensaré": "descompensar",
    "descompensaréis": "descompensar",
    "descompensaría": "descompensar",
    "descompensaríais": "descompensar",
    "descompensaríamos": "descompensar",
    "descompensarían": "descompensar",
    "descompensarías": "descompensar",
    "descompensas": "descompensar",
    "descompensase": "descompensar",
    "descompensaseis": "descompensar",
    "descompensasen": "descompensar",
    "descompensases": "descompensar",
    "descompensaste": "descompensar",
    "descompensasteis": "descompensar",
    "descompense": "descompensar",
    "descompensemos": "descompensar",
    "descompensen": "descompensar",
    "descompenses": "descompensar",
    "descompenso": "descompensar",
    "descompensábamos": "descompensar",
    "descompensáis": "descompensar",
    "descompensáramos": "descompensar",
    "descompensáremos": "descompensar",
    "descompensásemos": "descompensar",
    "descompensé": "descompensar",
    "descompenséis": "descompensar",
    "descompensó": "descompensar",
    "descompondremos": "descomponer",
    "descompondrá": "descomponer",
    "descompondrán": "descomponer",
    "descompondrás": "descomponer",
    "descompondré": "descomponer",
    "descompondréis": "descomponer",
    "descompondría": "descomponer",
    "descompondríais": "descomponer",
    "descompondríamos": "descomponer",
    "descompondrían": "descomponer",
    "descompondrías": "descomponer",
    "descompone": "descomponer",
    "descomponed": "descomponer",
    "descomponedores": "descomponedor",
    "descomponemos": "descomponer",
    "descomponen": "descomponer",
    "descompones": "descomponer",
    "descomponga": "descomponer",
    "descompongamos": "descomponer",
    "descompongan": "descomponer",
    "descompongas": "descomponer",
    "descompongo": "descomponer",
    "descompongáis": "descomponer",
    "descomponibles": "descomponible",
    "descomponiendo": "descomponer",
    "descomponéis": "descomponer",
    "descomponía": "descomponer",
    "descomponíais": "descomponer",
    "descomponíamos": "descomponer",
    "descomponían": "descomponer",
    "descomponías": "descomponer",
    "descomposiciones": "descomposición",
    "descompositores": "descompositor",
    "descomposturas": "descompostura",
    "descompresiones": "descompresión",
    "descompresores": "descompresor",
    "descomprima": "descomprimir",
    "descomprimamos": "descomprimir",
    "descompriman": "descomprimir",
    "descomprimas": "descomprimir",
    "descomprime": "descomprimir",
    "descomprimen": "descomprimir",
    "descomprimes": "descomprimir",
    "descomprimid": "descomprimir",
    "descomprimida": "descomprimir",
    "descomprimidas": "descomprimir",
    "descomprimido": "descomprimir",
    "descomprimidos": "descomprimir",
    "descomprimiendo": "descomprimir",
    "descomprimiera": "descomprimir",
    "descomprimierais": "descomprimir",
    "descomprimieran": "descomprimir",
    "descomprimieras": "descomprimir",
    "descomprimiere": "descomprimir",
    "descomprimiereis": "descomprimir",
    "descomprimieren": "descomprimir",
    "descomprimieres": "descomprimir",
    "descomprimieron": "descomprimir",
    "descomprimiese": "descomprimir",
    "descomprimieseis": "descomprimir",
    "descomprimiesen": "descomprimir",
    "descomprimieses": "descomprimir",
    "descomprimimos": "descomprimir",
    "descomprimiremos": "descomprimir",
    "descomprimirá": "descomprimir",
    "descomprimirán": "descomprimir",
    "descomprimirás": "descomprimir",
    "descomprimiré": "descomprimir",
    "descomprimiréis": "descomprimir",
    "descomprimiría": "descomprimir",
    "descomprimiríais": "descomprimir",
    "descomprimiríamos": "descomprimir",
    "descomprimirían": "descomprimir",
    "descomprimirías": "descomprimir",
    "descomprimiste": "descomprimir",
    "descomprimisteis": "descomprimir",
    "descomprimiéramos": "descomprimir",
    "descomprimiéremos": "descomprimir",
    "descomprimiésemos": "descomprimir",
    "descomprimió": "descomprimir",
    "descomprimo": "descomprimir",
    "descomprimáis": "descomprimir",
    "descomprimí": "descomprimir",
    "descomprimía": "descomprimir",
    "descomprimíais": "descomprimir",
    "descomprimíamos": "descomprimir",
    "descomprimían": "descomprimir",
    "descomprimías": "descomprimir",
    "descomprimís": "descomprimir",
    "descompuesta": "descomponer",
    "descompuestas": "descomponer",
    "descompuesto": "descomponer",
    "descompuestos": "descomponer",
    "descompuse": "descomponer",
    "descompusiera": "descomponer",
    "descompusierais": "descomponer",
    "descompusieran": "descomponer",
    "descompusieras": "descomponer",
    "descompusiere": "descomponer",
    "descompusiereis": "descomponer",
    "descompusieren": "descomponer",
    "descompusieres": "descomponer",
    "descompusieron": "descomponer",
    "descompusiese": "descomponer",
    "descompusieseis": "descomponer",
    "descompusiesen": "descomponer",
    "descompusieses": "descomponer",
    "descompusimos": "descomponer",
    "descompusiste": "descomponer",
    "descompusisteis": "descomponer",
    "descompusiéramos": "descomponer",
    "descompusiéremos": "descomponer",
    "descompusiésemos": "descomponer",
    "descompuso": "descomponer",
    "descompón": "descomponer",
    "descomunales": "descomunal",
    "descomuniones": "descomunión",
    "desconcentraciones": "desconcentración",
    "desconcentrada": "desconcentrado",
    "desconcentradas": "desconcentrado",
    "desconcentradora": "desconcentrador",
    "desconcentradoras": "desconcentrador",
    "desconcentradores": "desconcentrador",
    "desconcentrados": "desconcentrado",
    "desconceptuaba": "desconceptuar",
    "desconceptuabais": "desconceptuar",
    "desconceptuaban": "desconceptuar",
    "desconceptuabas": "desconceptuar",
    "desconceptuad": "desconceptuar",
    "desconceptuada": "desconceptuar",
    "desconceptuadas": "desconceptuar",
    "desconceptuado": "desconceptuar",
    "desconceptuados": "desconceptuar",
    "desconceptualizaciones": "desconceptualización",
    "desconceptuamos": "desconceptuar",
    "desconceptuando": "desconceptuar",
    "desconceptuara": "desconceptuar",
    "desconceptuarais": "desconceptuar",
    "desconceptuaran": "desconceptuar",
    "desconceptuaras": "desconceptuar",
    "desconceptuare": "desconceptuar",
    "desconceptuareis": "desconceptuar",
    "desconceptuaremos": "desconceptuar",
    "desconceptuaren": "desconceptuar",
    "desconceptuares": "desconceptuar",
    "desconceptuaron": "desconceptuar",
    "desconceptuará": "desconceptuar",
    "desconceptuarán": "desconceptuar",
    "desconceptuarás": "desconceptuar",
    "desconceptuaré": "desconceptuar",
    "desconceptuaréis": "desconceptuar",
    "desconceptuaría": "desconceptuar",
    "desconceptuaríais": "desconceptuar",
    "desconceptuaríamos": "desconceptuar",
    "desconceptuarían": "desconceptuar",
    "desconceptuarías": "desconceptuar",
    "desconceptuase": "desconceptuar",
    "desconceptuaseis": "desconceptuar",
    "desconceptuasen": "desconceptuar",
    "desconceptuases": "desconceptuar",
    "desconceptuaste": "desconceptuar",
    "desconceptuasteis": "desconceptuar",
    "desconceptuemos": "desconceptuar",
    "desconceptuábamos": "desconceptuar",
    "desconceptuáis": "desconceptuar",
    "desconceptuáramos": "desconceptuar",
    "desconceptuáremos": "desconceptuar",
    "desconceptuásemos": "desconceptuar",
    "desconceptué": "desconceptuar",
    "desconceptuéis": "desconceptuar",
    "desconceptuó": "desconceptuar",
    "desconceptúa": "desconceptuar",
    "desconceptúan": "desconceptuar",
    "desconceptúas": "desconceptuar",
    "desconceptúe": "desconceptuar",
    "desconceptúen": "desconceptuar",
    "desconceptúes": "desconceptuar",
    "desconceptúo": "desconceptuar",
    "desconcertaba": "desconcertar",
    "desconcertabais": "desconcertar",
    "desconcertaban": "desconcertar",
    "desconcertabas": "desconcertar",
    "desconcertad": "desconcertar",
    "desconcertada": "desconcertar",
    "desconcertadas": "desconcertar",
    "desconcertado": "desconcertar",
    "desconcertadora": "desconcertador",
    "desconcertadoras": "desconcertador",
    "desconcertadores": "desconcertador",
    "desconcertados": "desconcertar",
    "desconcertamos": "desconcertar",
    "desconcertando": "desconcertar",
    "desconcertantes": "desconcertante",
    "desconcertara": "desconcertar",
    "desconcertarais": "desconcertar",
    "desconcertaran": "desconcertar",
    "desconcertaras": "desconcertar",
    "desconcertare": "desconcertar",
    "desconcertareis": "desconcertar",
    "desconcertaremos": "desconcertar",
    "desconcertaren": "desconcertar",
    "desconcertares": "desconcertar",
    "desconcertaron": "desconcertar",
    "desconcertará": "desconcertar",
    "desconcertarán": "desconcertar",
    "desconcertarás": "desconcertar",
    "desconcertaré": "desconcertar",
    "desconcertaréis": "desconcertar",
    "desconcertaría": "desconcertar",
    "desconcertaríais": "desconcertar",
    "desconcertaríamos": "desconcertar",
    "desconcertarían": "desconcertar",
    "desconcertarías": "desconcertar",
    "desconcertase": "desconcertar",
    "desconcertaseis": "desconcertar",
    "desconcertasen": "desconcertar",
    "desconcertases": "desconcertar",
    "desconcertaste": "desconcertar",
    "desconcertasteis": "desconcertar",
    "desconcertemos": "desconcertar",
    "desconcertábamos": "desconcertar",
    "desconcertáis": "desconcertar",
    "desconcertáramos": "desconcertar",
    "desconcertáremos": "desconcertar",
    "desconcertásemos": "desconcertar",
    "desconcerté": "desconcertar",
    "desconcertéis": "desconcertar",
    "desconcertó": "desconcertar",
    "desconcha": "desconchar",
    "desconchaba": "desconchar",
    "desconchabais": "desconchar",
    "desconchaban": "desconchar",
    "desconchabas": "desconchar",
    "desconchad": "desconchar",
    "desconchada": "desconchar",
    "desconchadas": "desconchar",
    "desconchado": "desconchar",
    "desconchados": "desconchar",
    "desconchaduras": "desconchadura",
    "desconchamos": "desconchar",
    "desconchan": "desconchar",
    "desconchando": "desconchar",
    "desconchara": "desconchar",
    "desconcharais": "desconchar",
    "desconcharan": "desconchar",
    "desconcharas": "desconchar",
    "desconchare": "desconchar",
    "desconchareis": "desconchar",
    "desconcharemos": "desconchar",
    "desconcharen": "desconchar",
    "desconchares": "desconchar",
    "desconcharon": "desconchar",
    "desconchará": "desconchar",
    "desconcharán": "desconchar",
    "desconcharás": "desconchar",
    "desconcharé": "desconchar",
    "desconcharéis": "desconchar",
    "desconcharía": "desconchar",
    "desconcharíais": "desconchar",
    "desconcharíamos": "desconchar",
    "desconcharían": "desconchar",
    "desconcharías": "desconchar",
    "desconchas": "desconchar",
    "desconchase": "desconchar",
    "desconchaseis": "desconchar",
    "desconchasen": "desconchar",
    "desconchases": "desconchar",
    "desconchaste": "desconchar",
    "desconchasteis": "desconchar",
    "desconche": "desconchar",
    "desconchemos": "desconchar",
    "desconchen": "desconchar",
    "desconches": "desconchar",
    "desconcho": "desconchar",
    "desconchones": "desconchón",
    "desconchábamos": "desconchar",
    "desconcháis": "desconchar",
    "desconcháramos": "desconchar",
    "desconcháremos": "desconchar",
    "desconchásemos": "desconchar",
    "desconché": "desconchar",
    "desconchéis": "desconchar",
    "desconchó": "desconchar",
    "desconcierta": "desconcertar",
    "desconciertan": "desconcertar",
    "desconciertas": "desconcertar",
    "desconcierte": "desconcertar",
    "desconcierten": "desconcertar",
    "desconciertes": "desconcertar",
    "desconcierto": "desconcertar",
    "desconciertos": "desconcierto",
    "descondicionamientos": "descondicionamiento",
    "desconecta": "desconectar",
    "desconectaba": "desconectar",
    "desconectabais": "desconectar",
    "desconectaban": "desconectar",
    "desconectabas": "desconectar",
    "desconectables": "desconectable",
    "desconectad": "desconectar",
    "desconectada": "desconectar",
    "desconectadas": "desconectar",
    "desconectado": "desconectar",
    "desconectadores": "desconectador",
    "desconectados": "desconectar",
    "desconectamos": "desconectar",
    "desconectan": "desconectar",
    "desconectando": "desconectar",
    "desconectara": "desconectar",
    "desconectarais": "desconectar",
    "desconectaran": "desconectar",
    "desconectaras": "desconectar",
    "desconectare": "desconectar",
    "desconectareis": "desconectar",
    "desconectaremos": "desconectar",
    "desconectaren": "desconectar",
    "desconectares": "desconectar",
    "desconectaron": "desconectar",
    "desconectará": "desconectar",
    "desconectarán": "desconectar",
    "desconectarás": "desconectar",
    "desconectaré": "desconectar",
    "desconectaréis": "desconectar",
    "desconectaría": "desconectar",
    "desconectaríais": "desconectar",
    "desconectaríamos": "desconectar",
    "desconectarían": "desconectar",
    "desconectarías": "desconectar",
    "desconectas": "desconectar",
    "desconectase": "desconectar",
    "desconectaseis": "desconectar",
    "desconectasen": "desconectar",
    "desconectases": "desconectar",
    "desconectaste": "desconectar",
    "desconectasteis": "desconectar",
    "desconecte": "desconectar",
    "desconectemos": "desconectar",
    "desconecten": "desconectar",
    "desconectes": "desconectar",
    "desconecto": "desconectar",
    "desconectábamos": "desconectar",
    "desconectáis": "desconectar",
    "desconectáramos": "desconectar",
    "desconectáremos": "desconectar",
    "desconectásemos": "desconectar",
    "desconecté": "desconectar",
    "desconectéis": "desconectar",
    "desconectó": "desconectar",
    "desconexiones": "desconexión",
    "desconfiaba": "desconfiar",
    "desconfiabais": "desconfiar",
    "desconfiaban": "desconfiar",
    "desconfiabas": "desconfiar",
    "desconfiad": "desconfiar",
    "desconfiada": "desconfiar",
    "desconfiadas": "desconfiar",
    "desconfiado": "desconfiar",
    "desconfiados": "desconfiar",
    "desconfiamos": "desconfiar",
    "desconfiando": "desconfiar",
    "desconfianzas": "desconfianza",
    "desconfiara": "desconfiar",
    "desconfiarais": "desconfiar",
    "desconfiaran": "desconfiar",
    "desconfiaras": "desconfiar",
    "desconfiare": "desconfiar",
    "desconfiareis": "desconfiar",
    "desconfiaremos": "desconfiar",
    "desconfiaren": "desconfiar",
    "desconfiares": "desconfiar",
    "desconfiaron": "desconfiar",
    "desconfiará": "desconfiar",
    "desconfiarán": "desconfiar",
    "desconfiarás": "desconfiar",
    "desconfiaré": "desconfiar",
    "desconfiaréis": "desconfiar",
    "desconfiaría": "desconfiar",
    "desconfiaríais": "desconfiar",
    "desconfiaríamos": "desconfiar",
    "desconfiarían": "desconfiar",
    "desconfiarías": "desconfiar",
    "desconfiase": "desconfiar",
    "desconfiaseis": "desconfiar",
    "desconfiasen": "desconfiar",
    "desconfiases": "desconfiar",
    "desconfiaste": "desconfiar",
    "desconfiasteis": "desconfiar",
    "desconfiemos": "desconfiar",
    "desconfiábamos": "desconfiar",
    "desconfiáis": "desconfiar",
    "desconfiáramos": "desconfiar",
    "desconfiáremos": "desconfiar",
    "desconfiásemos": "desconfiar",
    "desconfié": "desconfiar",
    "desconfiéis": "desconfiar",
    "desconfió": "desconfiar",
    "desconforma": "desconformar",
    "desconformaba": "desconformar",
    "desconformabais": "desconformar",
    "desconformaban": "desconformar",
    "desconformabas": "desconformar",
    "desconformad": "desconformar",
    "desconformada": "desconformar",
    "desconformadas": "desconformar",
    "desconformado": "desconformar",
    "desconformados": "desconformar",
    "desconformamos": "desconformar",
    "desconforman": "desconformar",
    "desconformando": "desconformar",
    "desconformara": "desconformar",
    "desconformarais": "desconformar",
    "desconformaran": "desconformar",
    "desconformaras": "desconformar",
    "desconformare": "desconformar",
    "desconformareis": "desconformar",
    "desconformaremos": "desconformar",
    "desconformaren": "desconformar",
    "desconformares": "desconformar",
    "desconformaron": "desconformar",
    "desconformará": "desconformar",
    "desconformarán": "desconformar",
    "desconformarás": "desconformar",
    "desconformaré": "desconformar",
    "desconformaréis": "desconformar",
    "desconformaría": "desconformar",
    "desconformaríais": "desconformar",
    "desconformaríamos": "desconformar",
    "desconformarían": "desconformar",
    "desconformarías": "desconformar",
    "desconformas": "desconformar",
    "desconformase": "desconformar",
    "desconformaseis": "desconformar",
    "desconformasen": "desconformar",
    "desconformases": "desconformar",
    "desconformaste": "desconformar",
    "desconformasteis": "desconformar",
    "desconforme": "desconformar",
    "desconformemos": "desconformar",
    "desconformen": "desconformar",
    "desconformes": "desconforme",
    "desconformidades": "desconformidad",
    "desconformo": "desconformar",
    "desconformábamos": "desconformar",
    "desconformáis": "desconformar",
    "desconformáramos": "desconformar",
    "desconformáremos": "desconformar",
    "desconformásemos": "desconformar",
    "desconformé": "desconformar",
    "desconforméis": "desconformar",
    "desconformó": "desconformar",
    "desconfía": "desconfiar",
    "desconfían": "desconfiar",
    "desconfías": "desconfiar",
    "desconfíe": "desconfiar",
    "desconfíen": "desconfiar",
    "desconfíes": "desconfiar",
    "desconfío": "desconfiar",
    "descongela": "descongelar",
    "descongelaba": "descongelar",
    "descongelabais": "descongelar",
    "descongelaban": "descongelar",
    "descongelabas": "descongelar",
    "descongelaciones": "descongelación",
    "descongelad": "descongelar",
    "descongelada": "descongelar",
    "descongeladas": "descongelar",
    "descongelado": "descongelar",
    "descongeladores": "descongelador",
    "descongelados": "descongelar",
    "descongelamos": "descongelar",
    "descongelan": "descongelar",
    "descongelando": "descongelar",
    "descongelara": "descongelar",
    "descongelarais": "descongelar",
    "descongelaran": "descongelar",
    "descongelaras": "descongelar",
    "descongelare": "descongelar",
    "descongelareis": "descongelar",
    "descongelaremos": "descongelar",
    "descongelaren": "descongelar",
    "descongelares": "descongelar",
    "descongelaron": "descongelar",
    "descongelará": "descongelar",
    "descongelarán": "descongelar",
    "descongelarás": "descongelar",
    "descongelaré": "descongelar",
    "descongelaréis": "descongelar",
    "descongelaría": "descongelar",
    "descongelaríais": "descongelar",
    "descongelaríamos": "descongelar",
    "descongelarían": "descongelar",
    "descongelarías": "descongelar",
    "descongelas": "descongelar",
    "descongelase": "descongelar",
    "descongelaseis": "descongelar",
    "descongelasen": "descongelar",
    "descongelases": "descongelar",
    "descongelaste": "descongelar",
    "descongelasteis": "descongelar",
    "descongele": "descongelar",
    "descongelemos": "descongelar",
    "descongelen": "descongelar",
    "descongeles": "descongelar",
    "descongelo": "descongelar",
    "descongelábamos": "descongelar",
    "descongeláis": "descongelar",
    "descongeláramos": "descongelar",
    "descongeláremos": "descongelar",
    "descongelásemos": "descongelar",
    "descongelé": "descongelar",
    "descongeléis": "descongelar",
    "descongeló": "descongelar",
    "descongestiona": "descongestionar",
    "descongestionaba": "descongestionar",
    "descongestionabais": "descongestionar",
    "descongestionaban": "descongestionar",
    "descongestionabas": "descongestionar",
    "descongestionad": "descongestionar",
    "descongestionada": "descongestionar",
    "descongestionadas": "descongestionar",
    "descongestionado": "descongestionar",
    "descongestionados": "descongestionar",
    "descongestionamos": "descongestionar",
    "descongestionan": "descongestionar",
    "descongestionando": "descongestionar",
    "descongestionantes": "descongestionante",
    "descongestionara": "descongestionar",
    "descongestionarais": "descongestionar",
    "descongestionaran": "descongestionar",
    "descongestionaras": "descongestionar",
    "descongestionare": "descongestionar",
    "descongestionareis": "descongestionar",
    "descongestionaremos": "descongestionar",
    "descongestionaren": "descongestionar",
    "descongestionares": "descongestionar",
    "descongestionaron": "descongestionar",
    "descongestionará": "descongestionar",
    "descongestionarán": "descongestionar",
    "descongestionarás": "descongestionar",
    "descongestionaré": "descongestionar",
    "descongestionaréis": "descongestionar",
    "descongestionaría": "descongestionar",
    "descongestionaríais": "descongestionar",
    "descongestionaríamos": "descongestionar",
    "descongestionarían": "descongestionar",
    "descongestionarías": "descongestionar",
    "descongestionas": "descongestionar",
    "descongestionase": "descongestionar",
    "descongestionaseis": "descongestionar",
    "descongestionasen": "descongestionar",
    "descongestionases": "descongestionar",
    "descongestionaste": "descongestionar",
    "descongestionasteis": "descongestionar",
    "descongestione": "descongestionar",
    "descongestionemos": "descongestionar",
    "descongestionen": "descongestionar",
    "descongestiones": "descongestionar",
    "descongestiono": "descongestionar",
    "descongestionábamos": "descongestionar",
    "descongestionáis": "descongestionar",
    "descongestionáramos": "descongestionar",
    "descongestionáremos": "descongestionar",
    "descongestionásemos": "descongestionar",
    "descongestioné": "descongestionar",
    "descongestionéis": "descongestionar",
    "descongestionó": "descongestionar",
    "desconoce": "desconocer",
    "desconoced": "desconocer",
    "desconocedora": "desconocedor",
    "desconocedoras": "desconocedor",
    "desconocedores": "desconocedor",
    "desconocemos": "desconocer",
    "desconocen": "desconocer",
    "desconoceremos": "desconocer",
    "desconocerá": "desconocer",
    "desconocerán": "desconocer",
    "desconocerás": "desconocer",
    "desconoceré": "desconocer",
    "desconoceréis": "desconocer",
    "desconocería": "desconocer",
    "desconoceríais": "desconocer",
    "desconoceríamos": "desconocer",
    "desconocerían": "desconocer",
    "desconocerías": "desconocer",
    "desconoces": "desconocer",
    "desconocida": "desconocido",
    "desconocidas": "desconocido",
    "desconocido": "desconocer",
    "desconocidos": "desconocido",
    "desconociendo": "desconocer",
    "desconociera": "desconocer",
    "desconocierais": "desconocer",
    "desconocieran": "desconocer",
    "desconocieras": "desconocer",
    "desconociere": "desconocer",
    "desconociereis": "desconocer",
    "desconocieren": "desconocer",
    "desconocieres": "desconocer",
    "desconocieron": "desconocer",
    "desconociese": "desconocer",
    "desconocieseis": "desconocer",
    "desconociesen": "desconocer",
    "desconocieses": "desconocer",
    "desconocimientos": "desconocimiento",
    "desconocimos": "desconocer",
    "desconociste": "desconocer",
    "desconocisteis": "desconocer",
    "desconociéramos": "desconocer",
    "desconociéremos": "desconocer",
    "desconociésemos": "desconocer",
    "desconoció": "desconocer",
    "desconocéis": "desconocer",
    "desconocí": "desconocer",
    "desconocía": "desconocer",
    "desconocíais": "desconocer",
    "desconocíamos": "desconocer",
    "desconocían": "desconocer",
    "desconocías": "desconocer",
    "desconozca": "desconocer",
    "desconozcamos": "desconocer",
    "desconozcan": "desconocer",
    "desconozcas": "desconocer",
    "desconozco": "desconocer",
    "desconozcáis": "desconocer",
    "desconsentimientos": "desconsentimiento",
    "desconsidera": "desconsiderar",
    "desconsideraba": "desconsiderar",
    "desconsiderabais": "desconsiderar",
    "desconsideraban": "desconsiderar",
    "desconsiderabas": "desconsiderar",
    "desconsideraciones": "desconsideración",
    "desconsiderad": "desconsiderar",
    "desconsiderada": "desconsiderar",
    "desconsideradas": "desconsiderar",
    "desconsiderado": "desconsiderar",
    "desconsiderados": "desconsiderar",
    "desconsideramos": "desconsiderar",
    "desconsideran": "desconsiderar",
    "desconsiderando": "desconsiderar",
    "desconsiderara": "desconsiderar",
    "desconsiderarais": "desconsiderar",
    "desconsideraran": "desconsiderar",
    "desconsideraras": "desconsiderar",
    "desconsiderare": "desconsiderar",
    "desconsiderareis": "desconsiderar",
    "desconsideraremos": "desconsiderar",
    "desconsideraren": "desconsiderar",
    "desconsiderares": "desconsiderar",
    "desconsideraron": "desconsiderar",
    "desconsiderará": "desconsiderar",
    "desconsiderarán": "desconsiderar",
    "desconsiderarás": "desconsiderar",
    "desconsideraré": "desconsiderar",
    "desconsideraréis": "desconsiderar",
    "desconsideraría": "desconsiderar",
    "desconsideraríais": "desconsiderar",
    "desconsideraríamos": "desconsiderar",
    "desconsiderarían": "desconsiderar",
    "desconsiderarías": "desconsiderar",
    "desconsideras": "desconsiderar",
    "desconsiderase": "desconsiderar",
    "desconsideraseis": "desconsiderar",
    "desconsiderasen": "desconsiderar",
    "desconsiderases": "desconsiderar",
    "desconsideraste": "desconsiderar",
    "desconsiderasteis": "desconsiderar",
    "desconsidere": "desconsiderar",
    "desconsideremos": "desconsiderar",
    "desconsideren": "desconsiderar",
    "desconsideres": "desconsiderar",
    "desconsidero": "desconsiderar",
    "desconsiderábamos": "desconsiderar",
    "desconsideráis": "desconsiderar",
    "desconsideráramos": "desconsiderar",
    "desconsideráremos": "desconsiderar",
    "desconsiderásemos": "desconsiderar",
    "desconsideré": "desconsiderar",
    "desconsideréis": "desconsiderar",
    "desconsideró": "desconsiderar",
    "desconsolaba": "desconsolar",
    "desconsolabais": "desconsolar",
    "desconsolaban": "desconsolar",
    "desconsolabas": "desconsolar",
    "desconsolaciones": "desconsolación",
    "desconsolad": "desconsolar",
    "desconsolada": "desconsolar",
    "desconsoladas": "desconsolar",
    "desconsolado": "desconsolar",
    "desconsoladora": "desconsolador",
    "desconsoladoras": "desconsolador",
    "desconsoladores": "desconsolador",
    "desconsolados": "desconsolar",
    "desconsolamos": "desconsolar",
    "desconsolando": "desconsolar",
    "desconsolara": "desconsolar",
    "desconsolarais": "desconsolar",
    "desconsolaran": "desconsolar",
    "desconsolaras": "desconsolar",
    "desconsolare": "desconsolar",
    "desconsolareis": "desconsolar",
    "desconsolaremos": "desconsolar",
    "desconsolaren": "desconsolar",
    "desconsolares": "desconsolar",
    "desconsolaron": "desconsolar",
    "desconsolará": "desconsolar",
    "desconsolarán": "desconsolar",
    "desconsolarás": "desconsolar",
    "desconsolaré": "desconsolar",
    "desconsolaréis": "desconsolar",
    "desconsolaría": "desconsolar",
    "desconsolaríais": "desconsolar",
    "desconsolaríamos": "desconsolar",
    "desconsolarían": "desconsolar",
    "desconsolarías": "desconsolar",
    "desconsolase": "desconsolar",
    "desconsolaseis": "desconsolar",
    "desconsolasen": "desconsolar",
    "desconsolases": "desconsolar",
    "desconsolaste": "desconsolar",
    "desconsolasteis": "desconsolar",
    "desconsolemos": "desconsolar",
    "desconsolábamos": "desconsolar",
    "desconsoláis": "desconsolar",
    "desconsoláramos": "desconsolar",
    "desconsoláremos": "desconsolar",
    "desconsolásemos": "desconsolar",
    "desconsolé": "desconsolar",
    "desconsoléis": "desconsolar",
    "desconsoló": "desconsolar",
    "desconsuela": "desconsolar",
    "desconsuelan": "desconsolar",
    "desconsuelas": "desconsolar",
    "desconsuele": "desconsolar",
    "desconsuelen": "desconsolar",
    "desconsueles": "desconsolar",
    "desconsuelo": "desconsolar",
    "desconsuelos": "desconsuelo",
    "descontaba": "descontar",
    "descontabais": "descontar",
    "descontaban": "descontar",
    "descontabas": "descontar",
    "descontables": "descontable",
    "descontad": "descontar",
    "descontada": "descontar",
    "descontadas": "descontar",
    "descontado": "descontar",
    "descontados": "descontar",
    "descontamina": "descontaminar",
    "descontaminaba": "descontaminar",
    "descontaminabais": "descontaminar",
    "descontaminaban": "descontaminar",
    "descontaminabas": "descontaminar",
    "descontaminaciones": "descontaminación",
    "descontaminad": "descontaminar",
    "descontaminada": "descontaminar",
    "descontaminadas": "descontaminar",
    "descontaminado": "descontaminar",
    "descontaminadora": "descontaminador",
    "descontaminadoras": "descontaminador",
    "descontaminadores": "descontaminador",
    "descontaminados": "descontaminar",
    "descontaminamos": "descontaminar",
    "descontaminan": "descontaminar",
    "descontaminando": "descontaminar",
    "descontaminara": "descontaminar",
    "descontaminarais": "descontaminar",
    "descontaminaran": "descontaminar",
    "descontaminaras": "descontaminar",
    "descontaminare": "descontaminar",
    "descontaminareis": "descontaminar",
    "descontaminaremos": "descontaminar",
    "descontaminaren": "descontaminar",
    "descontaminares": "descontaminar",
    "descontaminaron": "descontaminar",
    "descontaminará": "descontaminar",
    "descontaminarán": "descontaminar",
    "descontaminarás": "descontaminar",
    "descontaminaré": "descontaminar",
    "descontaminaréis": "descontaminar",
    "descontaminaría": "descontaminar",
    "descontaminaríais": "descontaminar",
    "descontaminaríamos": "descontaminar",
    "descontaminarían": "descontaminar",
    "descontaminarías": "descontaminar",
    "descontaminas": "descontaminar",
    "descontaminase": "descontaminar",
    "descontaminaseis": "descontaminar",
    "descontaminasen": "descontaminar",
    "descontaminases": "descontaminar",
    "descontaminaste": "descontaminar",
    "descontaminasteis": "descontaminar",
    "descontamine": "descontaminar",
    "descontaminemos": "descontaminar",
    "descontaminen": "descontaminar",
    "descontamines": "descontaminar",
    "descontamino": "descontaminar",
    "descontaminábamos": "descontaminar",
    "descontamináis": "descontaminar",
    "descontamináramos": "descontaminar",
    "descontamináremos": "descontaminar",
    "descontaminásemos": "descontaminar",
    "descontaminé": "descontaminar",
    "descontaminéis": "descontaminar",
    "descontaminó": "descontaminar",
    "descontamos": "descontar",
    "descontando": "descontar",
    "descontara": "descontar",
    "descontarais": "descontar",
    "descontaran": "descontar",
    "descontaras": "descontar",
    "descontare": "descontar",
    "descontareis": "descontar",
    "descontaremos": "descontar",
    "descontaren": "descontar",
    "descontares": "descontar",
    "descontaron": "descontar",
    "descontará": "descontar",
    "descontarán": "descontar",
    "descontarás": "descontar",
    "descontaré": "descontar",
    "descontaréis": "descontar",
    "descontaría": "descontar",
    "descontaríais": "descontar",
    "descontaríamos": "descontar",
    "descontarían": "descontar",
    "descontarías": "descontar",
    "descontase": "descontar",
    "descontaseis": "descontar",
    "descontasen": "descontar",
    "descontases": "descontar",
    "descontaste": "descontar",
    "descontasteis": "descontar",
    "descontemos": "descontar",
    "descontenta": "descontento",
    "descontentaba": "descontentar",
    "descontentabais": "descontentar",
    "descontentaban": "descontentar",
    "descontentabas": "descontentar",
    "descontentad": "descontentar",
    "descontentada": "descontentar",
    "descontentadas": "descontentar",
    "descontentadiza": "descontentadizo",
    "descontentadizas": "descontentadizo",
    "descontentadizos": "descontentadizo",
    "descontentado": "descontentar",
    "descontentados": "descontentar",
    "descontentamos": "descontentar",
    "descontentan": "descontentar",
    "descontentando": "descontentar",
    "descontentara": "descontentar",
    "descontentarais": "descontentar",
    "descontentaran": "descontentar",
    "descontentaras": "descontentar",
    "descontentare": "descontentar",
    "descontentareis": "descontentar",
    "descontentaremos": "descontentar",
    "descontentaren": "descontentar",
    "descontentares": "descontentar",
    "descontentaron": "descontentar",
    "descontentará": "descontentar",
    "descontentarán": "descontentar",
    "descontentarás": "descontentar",
    "descontentaré": "descontentar",
    "descontentaréis": "descontentar",
    "descontentaría": "descontentar",
    "descontentaríais": "descontentar",
    "descontentaríamos": "descontentar",
    "descontentarían": "descontentar",
    "descontentarías": "descontentar",
    "descontentas": "descontento",
    "descontentase": "descontentar",
    "descontentaseis": "descontentar",
    "descontentasen": "descontentar",
    "descontentases": "descontentar",
    "descontentaste": "descontentar",
    "descontentasteis": "descontentar",
    "descontente": "descontentar",
    "descontentemos": "descontentar",
    "descontenten": "descontentar",
    "descontentes": "descontentar",
    "descontento": "descontentar",
    "descontentos": "descontento",
    "descontentábamos": "descontentar",
    "descontentáis": "descontentar",
    "descontentáramos": "descontentar",
    "descontentáremos": "descontentar",
    "descontentásemos": "descontentar",
    "descontenté": "descontentar",
    "descontentéis": "descontentar",
    "descontentó": "descontentar",
    "descontextualice": "descontextualizar",
    "descontextualicemos": "descontextualizar",
    "descontextualicen": "descontextualizar",
    "descontextualices": "descontextualizar",
    "descontextualicé": "descontextualizar",
    "descontextualicéis": "descontextualizar",
    "descontextualiza": "descontextualizar",
    "descontextualizaba": "descontextualizar",
    "descontextualizabais": "descontextualizar",
    "descontextualizaban": "descontextualizar",
    "descontextualizabas": "descontextualizar",
    "descontextualizad": "descontextualizar",
    "descontextualizada": "descontextualizar",
    "descontextualizadas": "descontextualizar",
    "descontextualizado": "descontextualizar",
    "descontextualizados": "descontextualizar",
    "descontextualizamos": "descontextualizar",
    "descontextualizan": "descontextualizar",
    "descontextualizando": "descontextualizar",
    "descontextualizara": "descontextualizar",
    "descontextualizarais": "descontextualizar",
    "descontextualizaran": "descontextualizar",
    "descontextualizaras": "descontextualizar",
    "descontextualizare": "descontextualizar",
    "descontextualizareis": "descontextualizar",
    "descontextualizaremos": "descontextualizar",
    "descontextualizaren": "descontextualizar",
    "descontextualizares": "descontextualizar",
    "descontextualizaron": "descontextualizar",
    "descontextualizará": "descontextualizar",
    "descontextualizarán": "descontextualizar",
    "descontextualizarás": "descontextualizar",
    "descontextualizaré": "descontextualizar",
    "descontextualizaréis": "descontextualizar",
    "descontextualizaría": "descontextualizar",
    "descontextualizaríais": "descontextualizar",
    "descontextualizaríamos": "descontextualizar",
    "descontextualizarían": "descontextualizar",
    "descontextualizarías": "descontextualizar",
    "descontextualizas": "descontextualizar",
    "descontextualizase": "descontextualizar",
    "descontextualizaseis": "descontextualizar",
    "descontextualizasen": "descontextualizar",
    "descontextualizases": "descontextualizar",
    "descontextualizaste": "descontextualizar",
    "descontextualizasteis": "descontextualizar",
    "descontextualizo": "descontextualizar",
    "descontextualizábamos": "descontextualizar",
    "descontextualizáis": "descontextualizar",
    "descontextualizáramos": "descontextualizar",
    "descontextualizáremos": "descontextualizar",
    "descontextualizásemos": "descontextualizar",
    "descontextualizó": "descontextualizar",
    "descontinua": "descontinuo",
    "descontinuaciones": "descontinuación",
    "descontinuas": "descontinuo",
    "descontinuos": "descontinuo",
    "descontrola": "descontrolar",
    "descontrolaba": "descontrolar",
    "descontrolabais": "descontrolar",
    "descontrolaban": "descontrolar",
    "descontrolabas": "descontrolar",
    "descontrolad": "descontrolar",
    "descontrolada": "descontrolar",
    "descontroladas": "descontrolar",
    "descontrolado": "descontrolar",
    "descontrolados": "descontrolar",
    "descontrolamos": "descontrolar",
    "descontrolan": "descontrolar",
    "descontrolando": "descontrolar",
    "descontrolara": "descontrolar",
    "descontrolarais": "descontrolar",
    "descontrolaran": "descontrolar",
    "descontrolaras": "descontrolar",
    "descontrolare": "descontrolar",
    "descontrolareis": "descontrolar",
    "descontrolaremos": "descontrolar",
    "descontrolaren": "descontrolar",
    "descontrolares": "descontrolar",
    "descontrolaron": "descontrolar",
    "descontrolará": "descontrolar",
    "descontrolarán": "descontrolar",
    "descontrolarás": "descontrolar",
    "descontrolaré": "descontrolar",
    "descontrolaréis": "descontrolar",
    "descontrolaría": "descontrolar",
    "descontrolaríais": "descontrolar",
    "descontrolaríamos": "descontrolar",
    "descontrolarían": "descontrolar",
    "descontrolarías": "descontrolar",
    "descontrolas": "descontrolar",
    "descontrolase": "descontrolar",
    "descontrolaseis": "descontrolar",
    "descontrolasen": "descontrolar",
    "descontrolases": "descontrolar",
    "descontrolaste": "descontrolar",
    "descontrolasteis": "descontrolar",
    "descontrole": "descontrolar",
    "descontrolemos": "descontrolar",
    "descontrolen": "descontrolar",
    "descontroles": "descontrolar",
    "descontrolo": "descontrolar",
    "descontrolábamos": "descontrolar",
    "descontroláis": "descontrolar",
    "descontroláramos": "descontrolar",
    "descontroláremos": "descontrolar",
    "descontrolásemos": "descontrolar",
    "descontrolé": "descontrolar",
    "descontroléis": "descontrolar",
    "descontroló": "descontrolar",
    "descontábamos": "descontar",
    "descontáis": "descontar",
    "descontáramos": "descontar",
    "descontáremos": "descontar",
    "descontásemos": "descontar",
    "desconté": "descontar",
    "descontéis": "descontar",
    "descontó": "descontar",
    "desconven": "desconvenir",
    "desconvendremos": "desconvenir",
    "desconvendrá": "desconvenir",
    "desconvendrán": "desconvenir",
    "desconvendrás": "desconvenir",
    "desconvendré": "desconvenir",
    "desconvendréis": "desconvenir",
    "desconvendría": "desconvenir",
    "desconvendríais": "desconvenir",
    "desconvendríamos": "desconvenir",
    "desconvendrían": "desconvenir",
    "desconvendrías": "desconvenir",
    "desconvenga": "desconvenir",
    "desconvengamos": "desconvenir",
    "desconvengan": "desconvenir",
    "desconvengas": "desconvenir",
    "desconvengo": "desconvenir",
    "desconvengáis": "desconvenir",
    "desconvenid": "desconvenir",
    "desconvenida": "desconvenir",
    "desconvenidas": "desconvenir",
    "desconvenido": "desconvenir",
    "desconvenidos": "desconvenir",
    "desconveniencias": "desconveniencia",
    "desconvenimos": "desconvenir",
    "desconvenía": "desconvenir",
    "desconveníais": "desconvenir",
    "desconveníamos": "desconvenir",
    "desconvenían": "desconvenir",
    "desconvenías": "desconvenir",
    "desconvenís": "desconvenir",
    "desconversables": "desconversable",
    "desconviene": "desconvenir",
    "desconvienen": "desconvenir",
    "desconvienes": "desconvenir",
    "desconvine": "desconvenir",
    "desconviniendo": "desconvenir",
    "desconviniera": "desconvenir",
    "desconvinierais": "desconvenir",
    "desconvinieran": "desconvenir",
    "desconvinieras": "desconvenir",
    "desconviniere": "desconvenir",
    "desconviniereis": "desconvenir",
    "desconvinieren": "desconvenir",
    "desconvinieres": "desconvenir",
    "desconvinieron": "desconvenir",
    "desconviniese": "desconvenir",
    "desconvinieseis": "desconvenir",
    "desconviniesen": "desconvenir",
    "desconvinieses": "desconvenir",
    "desconvinimos": "desconvenir",
    "desconviniste": "desconvenir",
    "desconvinisteis": "desconvenir",
    "desconviniéramos": "desconvenir",
    "desconviniéremos": "desconvenir",
    "desconviniésemos": "desconvenir",
    "desconvino": "desconvenir",
    "desconvoca": "desconvocar",
    "desconvocaba": "desconvocar",
    "desconvocabais": "desconvocar",
    "desconvocaban": "desconvocar",
    "desconvocabas": "desconvocar",
    "desconvocad": "desconvocar",
    "desconvocada": "desconvocar",
    "desconvocadas": "desconvocar",
    "desconvocado": "desconvocar",
    "desconvocados": "desconvocar",
    "desconvocamos": "desconvocar",
    "desconvocan": "desconvocar",
    "desconvocando": "desconvocar",
    "desconvocara": "desconvocar",
    "desconvocarais": "desconvocar",
    "desconvocaran": "desconvocar",
    "desconvocaras": "desconvocar",
    "desconvocare": "desconvocar",
    "desconvocareis": "desconvocar",
    "desconvocaremos": "desconvocar",
    "desconvocaren": "desconvocar",
    "desconvocares": "desconvocar",
    "desconvocaron": "desconvocar",
    "desconvocará": "desconvocar",
    "desconvocarán": "desconvocar",
    "desconvocarás": "desconvocar",
    "desconvocaré": "desconvocar",
    "desconvocaréis": "desconvocar",
    "desconvocaría": "desconvocar",
    "desconvocaríais": "desconvocar",
    "desconvocaríamos": "desconvocar",
    "desconvocarían": "desconvocar",
    "desconvocarías": "desconvocar",
    "desconvocas": "desconvocar",
    "desconvocase": "desconvocar",
    "desconvocaseis": "desconvocar",
    "desconvocasen": "desconvocar",
    "desconvocases": "desconvocar",
    "desconvocaste": "desconvocar",
    "desconvocasteis": "desconvocar",
    "desconvocatorias": "desconvocatoria",
    "desconvoco": "desconvocar",
    "desconvocábamos": "desconvocar",
    "desconvocáis": "desconvocar",
    "desconvocáramos": "desconvocar",
    "desconvocáremos": "desconvocar",
    "desconvocásemos": "desconvocar",
    "desconvocó": "desconvocar",
    "desconvoque": "desconvocar",
    "desconvoquemos": "desconvocar",
    "desconvoquen": "desconvocar",
    "desconvoques": "desconvocar",
    "desconvoqué": "desconvocar",
    "desconvoquéis": "desconvocar",
    "descoordinaciones": "descoordinación",
    "descoordinada": "descoordinado",
    "descoordinadas": "descoordinado",
    "descoordinados": "descoordinado",
    "descoque": "descocar",
    "descoquemos": "descocar",
    "descoquen": "descocar",
    "descoques": "descocar",
    "descoqué": "descocar",
    "descoquéis": "descocar",
    "descorazona": "descorazonar",
    "descorazonaba": "descorazonar",
    "descorazonabais": "descorazonar",
    "descorazonaban": "descorazonar",
    "descorazonabas": "descorazonar",
    "descorazonad": "descorazonar",
    "descorazonada": "descorazonar",
    "descorazonadas": "descorazonar",
    "descorazonado": "descorazonar",
    "descorazonadora": "descorazonador",
    "descorazonadoras": "descorazonador",
    "descorazonadores": "descorazonador",
    "descorazonados": "descorazonar",
    "descorazonamientos": "descorazonamiento",
    "descorazonamos": "descorazonar",
    "descorazonan": "descorazonar",
    "descorazonando": "descorazonar",
    "descorazonara": "descorazonar",
    "descorazonarais": "descorazonar",
    "descorazonaran": "descorazonar",
    "descorazonaras": "descorazonar",
    "descorazonare": "descorazonar",
    "descorazonareis": "descorazonar",
    "descorazonaremos": "descorazonar",
    "descorazonaren": "descorazonar",
    "descorazonares": "descorazonar",
    "descorazonaron": "descorazonar",
    "descorazonará": "descorazonar",
    "descorazonarán": "descorazonar",
    "descorazonarás": "descorazonar",
    "descorazonaré": "descorazonar",
    "descorazonaréis": "descorazonar",
    "descorazonaría": "descorazonar",
    "descorazonaríais": "descorazonar",
    "descorazonaríamos": "descorazonar",
    "descorazonarían": "descorazonar",
    "descorazonarías": "descorazonar",
    "descorazonas": "descorazonar",
    "descorazonase": "descorazonar",
    "descorazonaseis": "descorazonar",
    "descorazonasen": "descorazonar",
    "descorazonases": "descorazonar",
    "descorazonaste": "descorazonar",
    "descorazonasteis": "descorazonar",
    "descorazone": "descorazonar",
    "descorazonemos": "descorazonar",
    "descorazonen": "descorazonar",
    "descorazones": "descorazonar",
    "descorazono": "descorazonar",
    "descorazonábamos": "descorazonar",
    "descorazonáis": "descorazonar",
    "descorazonáramos": "descorazonar",
    "descorazonáremos": "descorazonar",
    "descorazonásemos": "descorazonar",
    "descorazoné": "descorazonar",
    "descorazonéis": "descorazonar",
    "descorazonó": "descorazonar",
    "descorcha": "descorchar",
    "descorchaba": "descorchar",
    "descorchabais": "descorchar",
    "descorchaban": "descorchar",
    "descorchabas": "descorchar",
    "descorchad": "descorchar",
    "descorchada": "descorchar",
    "descorchadas": "descorchar",
    "descorchado": "descorchar",
    "descorchadores": "descorchador",
    "descorchados": "descorchar",
    "descorchamos": "descorchar",
    "descorchan": "descorchar",
    "descorchando": "descorchar",
    "descorchara": "descorchar",
    "descorcharais": "descorchar",
    "descorcharan": "descorchar",
    "descorcharas": "descorchar",
    "descorchare": "descorchar",
    "descorchareis": "descorchar",
    "descorcharemos": "descorchar",
    "descorcharen": "descorchar",
    "descorchares": "descorchar",
    "descorcharon": "descorchar",
    "descorchará": "descorchar",
    "descorcharán": "descorchar",
    "descorcharás": "descorchar",
    "descorcharé": "descorchar",
    "descorcharéis": "descorchar",
    "descorcharía": "descorchar",
    "descorcharíais": "descorchar",
    "descorcharíamos": "descorchar",
    "descorcharían": "descorchar",
    "descorcharías": "descorchar",
    "descorchas": "descorchar",
    "descorchase": "descorchar",
    "descorchaseis": "descorchar",
    "descorchasen": "descorchar",
    "descorchases": "descorchar",
    "descorchaste": "descorchar",
    "descorchasteis": "descorchar",
    "descorche": "descorchar",
    "descorchemos": "descorchar",
    "descorchen": "descorchar",
    "descorches": "descorche",
    "descorcho": "descorchar",
    "descorchábamos": "descorchar",
    "descorcháis": "descorchar",
    "descorcháramos": "descorchar",
    "descorcháremos": "descorchar",
    "descorchásemos": "descorchar",
    "descorché": "descorchar",
    "descorchéis": "descorchar",
    "descorchó": "descorchar",
    "descordaba": "descordar",
    "descordabais": "descordar",
    "descordaban": "descordar",
    "descordabas": "descordar",
    "descordad": "descordar",
    "descordada": "descordar",
    "descordadas": "descordar",
    "descordado": "descordar",
    "descordados": "descordar",
    "descordamos": "descordar",
    "descordando": "descordar",
    "descordara": "descordar",
    "descordarais": "descordar",
    "descordaran": "descordar",
    "descordaras": "descordar",
    "descordare": "descordar",
    "descordareis": "descordar",
    "descordaremos": "descordar",
    "descordaren": "descordar",
    "descordares": "descordar",
    "descordaron": "descordar",
    "descordará": "descordar",
    "descordarán": "descordar",
    "descordarás": "descordar",
    "descordaré": "descordar",
    "descordaréis": "descordar",
    "descordaría": "descordar",
    "descordaríais": "descordar",
    "descordaríamos": "descordar",
    "descordarían": "descordar",
    "descordarías": "descordar",
    "descordase": "descordar",
    "descordaseis": "descordar",
    "descordasen": "descordar",
    "descordases": "descordar",
    "descordaste": "descordar",
    "descordasteis": "descordar",
    "descordemos": "descordar",
    "descordábamos": "descordar",
    "descordáis": "descordar",
    "descordáramos": "descordar",
    "descordáremos": "descordar",
    "descordásemos": "descordar",
    "descordé": "descordar",
    "descordéis": "descordar",
    "descordó": "descordar",
    "descorita": "descoritar",
    "descoritaba": "descoritar",
    "descoritabais": "descoritar",
    "descoritaban": "descoritar",
    "descoritabas": "descoritar",
    "descoritad": "descoritar",
    "descoritada": "descoritar",
    "descoritadas": "descoritar",
    "descoritado": "descoritar",
    "descoritados": "descoritar",
    "descoritamos": "descoritar",
    "descoritan": "descoritar",
    "descoritando": "descoritar",
    "descoritara": "descoritar",
    "descoritarais": "descoritar",
    "descoritaran": "descoritar",
    "descoritaras": "descoritar",
    "descoritare": "descoritar",
    "descoritareis": "descoritar",
    "descoritaremos": "descoritar",
    "descoritaren": "descoritar",
    "descoritares": "descoritar",
    "descoritaron": "descoritar",
    "descoritará": "descoritar",
    "descoritarán": "descoritar",
    "descoritarás": "descoritar",
    "descoritaré": "descoritar",
    "descoritaréis": "descoritar",
    "descoritaría": "descoritar",
    "descoritaríais": "descoritar",
    "descoritaríamos": "descoritar",
    "descoritarían": "descoritar",
    "descoritarías": "descoritar",
    "descoritas": "descoritar",
    "descoritase": "descoritar",
    "descoritaseis": "descoritar",
    "descoritasen": "descoritar",
    "descoritases": "descoritar",
    "descoritaste": "descoritar",
    "descoritasteis": "descoritar",
    "descorite": "descoritar",
    "descoritemos": "descoritar",
    "descoriten": "descoritar",
    "descorites": "descoritar",
    "descorito": "descoritar",
    "descoritábamos": "descoritar",
    "descoritáis": "descoritar",
    "descoritáramos": "descoritar",
    "descoritáremos": "descoritar",
    "descoritásemos": "descoritar",
    "descorité": "descoritar",
    "descoritéis": "descoritar",
    "descoritó": "descoritar",
    "descornaba": "descornar",
    "descornabais": "descornar",
    "descornaban": "descornar",
    "descornabas": "descornar",
    "descornad": "descornar",
    "descornada": "descornar",
    "descornadas": "descornar",
    "descornado": "descornar",
    "descornadora": "descornador",
    "descornadoras": "descornadora",
    "descornadores": "descornador",
    "descornados": "descornar",
    "descornamos": "descornar",
    "descornando": "descornar",
    "descornara": "descornar",
    "descornarais": "descornar",
    "descornaran": "descornar",
    "descornaras": "descornar",
    "descornare": "descornar",
    "descornareis": "descornar",
    "descornaremos": "descornar",
    "descornaren": "descornar",
    "descornares": "descornar",
    "descornaron": "descornar",
    "descornará": "descornar",
    "descornarán": "descornar",
    "descornarás": "descornar",
    "descornaré": "descornar",
    "descornaréis": "descornar",
    "descornaría": "descornar",
    "descornaríais": "descornar",
    "descornaríamos": "descornar",
    "descornarían": "descornar",
    "descornarías": "descornar",
    "descornase": "descornar",
    "descornaseis": "descornar",
    "descornasen": "descornar",
    "descornases": "descornar",
    "descornaste": "descornar",
    "descornasteis": "descornar",
    "descornemos": "descornar",
    "descornes": "descorne",
    "descornábamos": "descornar",
    "descornáis": "descornar",
    "descornáramos": "descornar",
    "descornáremos": "descornar",
    "descornásemos": "descornar",
    "descorné": "descornar",
    "descornéis": "descornar",
    "descornó": "descornar",
    "descorona": "descoronar",
    "descoronaba": "descoronar",
    "descoronabais": "descoronar",
    "descoronaban": "descoronar",
    "descoronabas": "descoronar",
    "descoronad": "descoronar",
    "descoronada": "descoronar",
    "descoronadas": "descoronar",
    "descoronado": "descoronar",
    "descoronadoras": "descoronadora",
    "descoronados": "descoronar",
    "descoronamos": "descoronar",
    "descoronan": "descoronar",
    "descoronando": "descoronar",
    "descoronara": "descoronar",
    "descoronarais": "descoronar",
    "descoronaran": "descoronar",
    "descoronaras": "descoronar",
    "descoronare": "descoronar",
    "descoronareis": "descoronar",
    "descoronaremos": "descoronar",
    "descoronaren": "descoronar",
    "descoronares": "descoronar",
    "descoronaron": "descoronar",
    "descoronará": "descoronar",
    "descoronarán": "descoronar",
    "descoronarás": "descoronar",
    "descoronaré": "descoronar",
    "descoronaréis": "descoronar",
    "descoronaría": "descoronar",
    "descoronaríais": "descoronar",
    "descoronaríamos": "descoronar",
    "descoronarían": "descoronar",
    "descoronarías": "descoronar",
    "descoronas": "descoronar",
    "descoronase": "descoronar",
    "descoronaseis": "descoronar",
    "descoronasen": "descoronar",
    "descoronases": "descoronar",
    "descoronaste": "descoronar",
    "descoronasteis": "descoronar",
    "descorone": "descoronar",
    "descoronemos": "descoronar",
    "descoronen": "descoronar",
    "descorones": "descoronar",
    "descorono": "descoronar",
    "descoronábamos": "descoronar",
    "descoronáis": "descoronar",
    "descoronáramos": "descoronar",
    "descoronáremos": "descoronar",
    "descoronásemos": "descoronar",
    "descoroné": "descoronar",
    "descoronéis": "descoronar",
    "descoronó": "descoronar",
    "descorra": "descorrer",
    "descorramos": "descorrer",
    "descorran": "descorrer",
    "descorras": "descorrer",
    "descorre": "descorrer",
    "descorred": "descorrer",
    "descorrelaciones": "descorrelación",
    "descorremos": "descorrer",
    "descorren": "descorrer",
    "descorreremos": "descorrer",
    "descorrerá": "descorrer",
    "descorrerán": "descorrer",
    "descorrerás": "descorrer",
    "descorreré": "descorrer",
    "descorreréis": "descorrer",
    "descorrería": "descorrer",
    "descorreríais": "descorrer",
    "descorreríamos": "descorrer",
    "descorrerían": "descorrer",
    "descorrerías": "descorrer",
    "descorres": "descorrer",
    "descorrida": "descorrer",
    "descorridas": "descorrer",
    "descorrido": "descorrer",
    "descorridos": "descorrer",
    "descorriendo": "descorrer",
    "descorriera": "descorrer",
    "descorrierais": "descorrer",
    "descorrieran": "descorrer",
    "descorrieras": "descorrer",
    "descorriere": "descorrer",
    "descorriereis": "descorrer",
    "descorrieren": "descorrer",
    "descorrieres": "descorrer",
    "descorrieron": "descorrer",
    "descorriese": "descorrer",
    "descorrieseis": "descorrer",
    "descorriesen": "descorrer",
    "descorrieses": "descorrer",
    "descorrimientos": "descorrimiento",
    "descorrimos": "descorrer",
    "descorriste": "descorrer",
    "descorristeis": "descorrer",
    "descorriéramos": "descorrer",
    "descorriéremos": "descorrer",
    "descorriésemos": "descorrer",
    "descorrió": "descorrer",
    "descorro": "descorrer",
    "descorráis": "descorrer",
    "descorréis": "descorrer",
    "descorrí": "descorrer",
    "descorría": "descorrer",
    "descorríais": "descorrer",
    "descorríamos": "descorrer",
    "descorrían": "descorrer",
    "descorrías": "descorrer",
    "descortece": "descortezar",
    "descortecemos": "descortezar",
    "descortecen": "descortezar",
    "descorteces": "descortezar",
    "descortecé": "descortezar",
    "descortecéis": "descortezar",
    "descorteses": "descortés",
    "descortesías": "descortesía",
    "descorteza": "descortezar",
    "descortezaba": "descortezar",
    "descortezabais": "descortezar",
    "descortezaban": "descortezar",
    "descortezabas": "descortezar",
    "descortezad": "descortezar",
    "descortezada": "descortezar",
    "descortezadas": "descortezar",
    "descortezado": "descortezar",
    "descortezadoras": "descortezadora",
    "descortezados": "descortezar",
    "descortezaduras": "descortezadura",
    "descortezamientos": "descortezamiento",
    "descortezamos": "descortezar",
    "descortezan": "descortezar",
    "descortezando": "descortezar",
    "descortezara": "descortezar",
    "descortezarais": "descortezar",
    "descortezaran": "descortezar",
    "descortezaras": "descortezar",
    "descortezare": "descortezar",
    "descortezareis": "descortezar",
    "descortezaremos": "descortezar",
    "descortezaren": "descortezar",
    "descortezares": "descortezar",
    "descortezaron": "descortezar",
    "descortezará": "descortezar",
    "descortezarán": "descortezar",
    "descortezarás": "descortezar",
    "descortezaré": "descortezar",
    "descortezaréis": "descortezar",
    "descortezaría": "descortezar",
    "descortezaríais": "descortezar",
    "descortezaríamos": "descortezar",
    "descortezarían": "descortezar",
    "descortezarías": "descortezar",
    "descortezas": "descortezar",
    "descortezase": "descortezar",
    "descortezaseis": "descortezar",
    "descortezasen": "descortezar",
    "descortezases": "descortezar",
    "descortezaste": "descortezar",
    "descortezasteis": "descortezar",
    "descortezo": "descortezar",
    "descortezos": "descortezo",
    "descortezábamos": "descortezar",
    "descortezáis": "descortezar",
    "descortezáramos": "descortezar",
    "descortezáremos": "descortezar",
    "descortezásemos": "descortezar",
    "descortezó": "descortezar",
    "descosa": "descoser",
    "descosamos": "descoser",
    "descosan": "descoser",
    "descosas": "descoser",
    "descose": "descoser",
    "descosed": "descoser",
    "descosemos": "descoser",
    "descosen": "descoser",
    "descoseremos": "descoser",
    "descoserá": "descoser",
    "descoserán": "descoser",
    "descoserás": "descoser",
    "descoseré": "descoser",
    "descoseréis": "descoser",
    "descosería": "descoser",
    "descoseríais": "descoser",
    "descoseríamos": "descoser",
    "descoserían": "descoser",
    "descoserías": "descoser",
    "descoses": "descoser",
    "descosida": "descoser",
    "descosidas": "descoser",
    "descosido": "descoser",
    "descosidos": "descosido",
    "descosiendo": "descoser",
    "descosiera": "descoser",
    "descosierais": "descoser",
    "descosieran": "descoser",
    "descosieras": "descoser",
    "descosiere": "descoser",
    "descosiereis": "descoser",
    "descosieren": "descoser",
    "descosieres": "descoser",
    "descosieron": "descoser",
    "descosiese": "descoser",
    "descosieseis": "descoser",
    "descosiesen": "descoser",
    "descosieses": "descoser",
    "descosimos": "descoser",
    "descosiste": "descoser",
    "descosisteis": "descoser",
    "descosiéramos": "descoser",
    "descosiéremos": "descoser",
    "descosiésemos": "descoser",
    "descosió": "descoser",
    "descoso": "descoser",
    "descostaba": "descostar",
    "descostabais": "descostar",
    "descostaban": "descostar",
    "descostabas": "descostar",
    "descostad": "descostar",
    "descostada": "descostar",
    "descostadas": "descostar",
    "descostado": "descostar",
    "descostados": "descostar",
    "descostamos": "descostar",
    "descostando": "descostar",
    "descostara": "descostar",
    "descostarais": "descostar",
    "descostaran": "descostar",
    "descostaras": "descostar",
    "descostare": "descostar",
    "descostareis": "descostar",
    "descostaremos": "descostar",
    "descostaren": "descostar",
    "descostares": "descostar",
    "descostaron": "descostar",
    "descostará": "descostar",
    "descostarán": "descostar",
    "descostarás": "descostar",
    "descostaré": "descostar",
    "descostaréis": "descostar",
    "descostaría": "descostar",
    "descostaríais": "descostar",
    "descostaríamos": "descostar",
    "descostarían": "descostar",
    "descostarías": "descostar",
    "descostase": "descostar",
    "descostaseis": "descostar",
    "descostasen": "descostar",
    "descostases": "descostar",
    "descostaste": "descostar",
    "descostasteis": "descostar",
    "descostemos": "descostar",
    "descostilla": "descostillar",
    "descostillaba": "descostillar",
    "descostillabais": "descostillar",
    "descostillaban": "descostillar",
    "descostillabas": "descostillar",
    "descostillad": "descostillar",
    "descostillada": "descostillar",
    "descostilladas": "descostillar",
    "descostillado": "descostillar",
    "descostillados": "descostillar",
    "descostillamos": "descostillar",
    "descostillan": "descostillar",
    "descostillando": "descostillar",
    "descostillara": "descostillar",
    "descostillarais": "descostillar",
    "descostillaran": "descostillar",
    "descostillaras": "descostillar",
    "descostillare": "descostillar",
    "descostillareis": "descostillar",
    "descostillaremos": "descostillar",
    "descostillaren": "descostillar",
    "descostillares": "descostillar",
    "descostillaron": "descostillar",
    "descostillará": "descostillar",
    "descostillarán": "descostillar",
    "descostillarás": "descostillar",
    "descostillaré": "descostillar",
    "descostillaréis": "descostillar",
    "descostillaría": "descostillar",
    "descostillaríais": "descostillar",
    "descostillaríamos": "descostillar",
    "descostillarían": "descostillar",
    "descostillarías": "descostillar",
    "descostillas": "descostillar",
    "descostillase": "descostillar",
    "descostillaseis": "descostillar",
    "descostillasen": "descostillar",
    "descostillases": "descostillar",
    "descostillaste": "descostillar",
    "descostillasteis": "descostillar",
    "descostille": "descostillar",
    "descostillemos": "descostillar",
    "descostillen": "descostillar",
    "descostilles": "descostillar",
    "descostillo": "descostillar",
    "descostillábamos": "descostillar",
    "descostilláis": "descostillar",
    "descostilláramos": "descostillar",
    "descostilláremos": "descostillar",
    "descostillásemos": "descostillar",
    "descostillé": "descostillar",
    "descostilléis": "descostillar",
    "descostilló": "descostillar",
    "descostra": "descostrar",
    "descostraba": "descostrar",
    "descostrabais": "descostrar",
    "descostraban": "descostrar",
    "descostrabas": "descostrar",
    "descostrad": "descostrar",
    "descostrada": "descostrar",
    "descostradas": "descostrar",
    "descostrado": "descostrar",
    "descostrados": "descostrar",
    "descostramos": "descostrar",
    "descostran": "descostrar",
    "descostrando": "descostrar",
    "descostrara": "descostrar",
    "descostrarais": "descostrar",
    "descostraran": "descostrar",
    "descostraras": "descostrar",
    "descostrare": "descostrar",
    "descostrareis": "descostrar",
    "descostraremos": "descostrar",
    "descostraren": "descostrar",
    "descostrares": "descostrar",
    "descostraron": "descostrar",
    "descostrará": "descostrar",
    "descostrarán": "descostrar",
    "descostrarás": "descostrar",
    "descostraré": "descostrar",
    "descostraréis": "descostrar",
    "descostraría": "descostrar",
    "descostraríais": "descostrar",
    "descostraríamos": "descostrar",
    "descostrarían": "descostrar",
    "descostrarías": "descostrar",
    "descostras": "descostrar",
    "descostrase": "descostrar",
    "descostraseis": "descostrar",
    "descostrasen": "descostrar",
    "descostrases": "descostrar",
    "descostraste": "descostrar",
    "descostrasteis": "descostrar",
    "descostre": "descostrar",
    "descostremos": "descostrar",
    "descostren": "descostrar",
    "descostres": "descostrar",
    "descostro": "descostrar",
    "descostrábamos": "descostrar",
    "descostráis": "descostrar",
    "descostráramos": "descostrar",
    "descostráremos": "descostrar",
    "descostrásemos": "descostrar",
    "descostré": "descostrar",
    "descostréis": "descostrar",
    "descostró": "descostrar",
    "descostábamos": "descostar",
    "descostáis": "descostar",
    "descostáramos": "descostar",
    "descostáremos": "descostar",
    "descostásemos": "descostar",
    "descosté": "descostar",
    "descostéis": "descostar",
    "descostó": "descostar",
    "descosáis": "descoser",
    "descoséis": "descoser",
    "descosí": "descoser",
    "descosía": "descoser",
    "descosíais": "descoser",
    "descosíamos": "descoser",
    "descosían": "descoser",
    "descosías": "descoser",
    "descota": "descotar",
    "descotaba": "descotar",
    "descotabais": "descotar",
    "descotaban": "descotar",
    "descotabas": "descotar",
    "descotad": "descotar",
    "descotada": "descotar",
    "descotadas": "descotar",
    "descotado": "descotar",
    "descotados": "descotar",
    "descotamos": "descotar",
    "descotan": "descotar",
    "descotando": "descotar",
    "descotara": "descotar",
    "descotarais": "descotar",
    "descotaran": "descotar",
    "descotaras": "descotar",
    "descotare": "descotar",
    "descotareis": "descotar",
    "descotaremos": "descotar",
    "descotaren": "descotar",
    "descotares": "descotar",
    "descotaron": "descotar",
    "descotará": "descotar",
    "descotarán": "descotar",
    "descotarás": "descotar",
    "descotaré": "descotar",
    "descotaréis": "descotar",
    "descotaría": "descotar",
    "descotaríais": "descotar",
    "descotaríamos": "descotar",
    "descotarían": "descotar",
    "descotarías": "descotar",
    "descotas": "descotar",
    "descotase": "descotar",
    "descotaseis": "descotar",
    "descotasen": "descotar",
    "descotases": "descotar",
    "descotaste": "descotar",
    "descotasteis": "descotar",
    "descote": "descotar",
    "descotemos": "descotar",
    "descoten": "descotar",
    "descotes": "descote",
    "descoto": "descotar",
    "descotábamos": "descotar",
    "descotáis": "descotar",
    "descotáramos": "descotar",
    "descotáremos": "descotar",
    "descotásemos": "descotar",
    "descoté": "descotar",
    "descotéis": "descotar",
    "descotó": "descotar",
    "descoyunta": "descoyuntar",
    "descoyuntaba": "descoyuntar",
    "descoyuntabais": "descoyuntar",
    "descoyuntaban": "descoyuntar",
    "descoyuntabas": "descoyuntar",
    "descoyuntad": "descoyuntar",
    "descoyuntada": "descoyuntar",
    "descoyuntadas": "descoyuntar",
    "descoyuntado": "descoyuntar",
    "descoyuntados": "descoyuntar",
    "descoyuntamientos": "descoyuntamiento",
    "descoyuntamos": "descoyuntar",
    "descoyuntan": "descoyuntar",
    "descoyuntando": "descoyuntar",
    "descoyuntara": "descoyuntar",
    "descoyuntarais": "descoyuntar",
    "descoyuntaran": "descoyuntar",
    "descoyuntaras": "descoyuntar",
    "descoyuntare": "descoyuntar",
    "descoyuntareis": "descoyuntar",
    "descoyuntaremos": "descoyuntar",
    "descoyuntaren": "descoyuntar",
    "descoyuntares": "descoyuntar",
    "descoyuntaron": "descoyuntar",
    "descoyuntará": "descoyuntar",
    "descoyuntarán": "descoyuntar",
    "descoyuntarás": "descoyuntar",
    "descoyuntaré": "descoyuntar",
    "descoyuntaréis": "descoyuntar",
    "descoyuntaría": "descoyuntar",
    "descoyuntaríais": "descoyuntar",
    "descoyuntaríamos": "descoyuntar",
    "descoyuntarían": "descoyuntar",
    "descoyuntarías": "descoyuntar",
    "descoyuntas": "descoyuntar",
    "descoyuntase": "descoyuntar",
    "descoyuntaseis": "descoyuntar",
    "descoyuntasen": "descoyuntar",
    "descoyuntases": "descoyuntar",
    "descoyuntaste": "descoyuntar",
    "descoyuntasteis": "descoyuntar",
    "descoyunte": "descoyuntar",
    "descoyuntemos": "descoyuntar",
    "descoyunten": "descoyuntar",
    "descoyuntes": "descoyuntar",
    "descoyunto": "descoyuntar",
    "descoyuntos": "descoyunto",
    "descoyuntábamos": "descoyuntar",
    "descoyuntáis": "descoyuntar",
    "descoyuntáramos": "descoyuntar",
    "descoyuntáremos": "descoyuntar",
    "descoyuntásemos": "descoyuntar",
    "descoyunté": "descoyuntar",
    "descoyuntéis": "descoyuntar",
    "descoyuntó": "descoyuntar",
    "descreimientos": "descreimiento",
    "descrema": "descremar",
    "descremaba": "descremar",
    "descremabais": "descremar",
    "descremaban": "descremar",
    "descremabas": "descremar",
    "descremaciones": "descremación",
    "descremad": "descremar",
    "descremada": "descremar",
    "descremadas": "descremar",
    "descremado": "descremar",
    "descremadoras": "descremadora",
    "descremados": "descremar",
    "descremamos": "descremar",
    "descreman": "descremar",
    "descremando": "descremar",
    "descremara": "descremar",
    "descremarais": "descremar",
    "descremaran": "descremar",
    "descremaras": "descremar",
    "descremare": "descremar",
    "descremareis": "descremar",
    "descremaremos": "descremar",
    "descremaren": "descremar",
    "descremares": "descremar",
    "descremaron": "descremar",
    "descremará": "descremar",
    "descremarán": "descremar",
    "descremarás": "descremar",
    "descremaré": "descremar",
    "descremaréis": "descremar",
    "descremaría": "descremar",
    "descremaríais": "descremar",
    "descremaríamos": "descremar",
    "descremarían": "descremar",
    "descremarías": "descremar",
    "descremas": "descremar",
    "descremase": "descremar",
    "descremaseis": "descremar",
    "descremasen": "descremar",
    "descremases": "descremar",
    "descremaste": "descremar",
    "descremasteis": "descremar",
    "descreme": "descremar",
    "descrememos": "descremar",
    "descremen": "descremar",
    "descremes": "descremar",
    "descremo": "descremar",
    "descremábamos": "descremar",
    "descremáis": "descremar",
    "descremáramos": "descremar",
    "descremáremos": "descremar",
    "descremásemos": "descremar",
    "descremé": "descremar",
    "descreméis": "descremar",
    "descremó": "descremar",
    "descresta": "descrestar",
    "descrestaba": "descrestar",
    "descrestabais": "descrestar",
    "descrestaban": "descrestar",
    "descrestabas": "descrestar",
    "descrestad": "descrestar",
    "descrestada": "descrestar",
    "descrestadas": "descrestar",
    "descrestado": "descrestar",
    "descrestados": "descrestar",
    "descrestamos": "descrestar",
    "descrestan": "descrestar",
    "descrestando": "descrestar",
    "descrestara": "descrestar",
    "descrestarais": "descrestar",
    "descrestaran": "descrestar",
    "descrestaras": "descrestar",
    "descrestare": "descrestar",
    "descrestareis": "descrestar",
    "descrestaremos": "descrestar",
    "descrestaren": "descrestar",
    "descrestares": "descrestar",
    "descrestaron": "descrestar",
    "descrestará": "descrestar",
    "descrestarán": "descrestar",
    "descrestarás": "descrestar",
    "descrestaré": "descrestar",
    "descrestaréis": "descrestar",
    "descrestaría": "descrestar",
    "descrestaríais": "descrestar",
    "descrestaríamos": "descrestar",
    "descrestarían": "descrestar",
    "descrestarías": "descrestar",
    "descrestas": "descrestar",
    "descrestase": "descrestar",
    "descrestaseis": "descrestar",
    "descrestasen": "descrestar",
    "descrestases": "descrestar",
    "descrestaste": "descrestar",
    "descrestasteis": "descrestar",
    "descreste": "descrestar",
    "descrestemos": "descrestar",
    "descresten": "descrestar",
    "descrestes": "descrestar",
    "descresto": "descrestar",
    "descrestábamos": "descrestar",
    "descrestáis": "descrestar",
    "descrestáramos": "descrestar",
    "descrestáremos": "descrestar",
    "descrestásemos": "descrestar",
    "descresté": "descrestar",
    "descrestéis": "descrestar",
    "descrestó": "descrestar",
    "descreída": "descreído",
    "descreídas": "descreído",
    "descreídos": "descreído",
    "descriaba": "descriar",
    "descriabais": "descriar",
    "descriaban": "descriar",
    "descriabas": "descriar",
    "descriad": "descriar",
    "descriada": "descriar",
    "descriadas": "descriar",
    "descriado": "descriar",
    "descriados": "descriar",
    "descriamos": "descriar",
    "descriando": "descriar",
    "descriara": "descriar",
    "descriarais": "descriar",
    "descriaran": "descriar",
    "descriaras": "descriar",
    "descriare": "descriar",
    "descriareis": "descriar",
    "descriaremos": "descriar",
    "descriaren": "descriar",
    "descriares": "descriar",
    "descriaron": "descriar",
    "descriará": "descriar",
    "descriarán": "descriar",
    "descriarás": "descriar",
    "descriaré": "descriar",
    "descriaréis": "descriar",
    "descriaría": "descriar",
    "descriaríais": "descriar",
    "descriaríamos": "descriar",
    "descriarían": "descriar",
    "descriarías": "descriar",
    "descriase": "descriar",
    "descriaseis": "descriar",
    "descriasen": "descriar",
    "descriases": "descriar",
    "descriaste": "descriar",
    "descriasteis": "descriar",
    "describa": "describir",
    "describamos": "describir",
    "describan": "describir",
    "describas": "describir",
    "describe": "describir",
    "describen": "describir",
    "describes": "describir",
    "describid": "describir",
    "describiendo": "describir",
    "describiera": "describir",
    "describierais": "describir",
    "describieran": "describir",
    "describieras": "describir",
    "describiere": "describir",
    "describiereis": "describir",
    "describieren": "describir",
    "describieres": "describir",
    "describieron": "describir",
    "describiese": "describir",
    "describieseis": "describir",
    "describiesen": "describir",
    "describieses": "describir",
    "describimos": "describir",
    "describiremos": "describir",
    "describirá": "describir",
    "describirán": "describir",
    "describirás": "describir",
    "describiré": "describir",
    "describiréis": "describir",
    "describiría": "describir",
    "describiríais": "describir",
    "describiríamos": "describir",
    "describirían": "describir",
    "describirías": "describir",
    "describiste": "describir",
    "describisteis": "describir",
    "describiéramos": "describir",
    "describiéremos": "describir",
    "describiésemos": "describir",
    "describió": "describir",
    "describo": "describir",
    "describáis": "describir",
    "describí": "describir",
    "describía": "describir",
    "describíais": "describir",
    "describíamos": "describir",
    "describían": "describir",
    "describías": "describir",
    "describís": "describir",
    "descriemos": "descriar",
    "descriminalizaciones": "descriminalización",
    "descripciones": "descripción",
    "descriptibles": "descriptible",
    "descriptiva": "descriptivo",
    "descriptivas": "descriptivo",
    "descriptivismos": "descriptivismo",
    "descriptivistas": "descriptivista",
    "descriptivos": "descriptivo",
    "descriptora": "descriptor",
    "descriptoras": "descriptor",
    "descriptores": "descriptor",
    "descrisma": "descrismar",
    "descrismaba": "descrismar",
    "descrismabais": "descrismar",
    "descrismaban": "descrismar",
    "descrismabas": "descrismar",
    "descrismad": "descrismar",
    "descrismada": "descrismar",
    "descrismadas": "descrismar",
    "descrismado": "descrismar",
    "descrismados": "descrismar",
    "descrismamos": "descrismar",
    "descrisman": "descrismar",
    "descrismando": "descrismar",
    "descrismara": "descrismar",
    "descrismarais": "descrismar",
    "descrismaran": "descrismar",
    "descrismaras": "descrismar",
    "descrismare": "descrismar",
    "descrismareis": "descrismar",
    "descrismaremos": "descrismar",
    "descrismaren": "descrismar",
    "descrismares": "descrismar",
    "descrismaron": "descrismar",
    "descrismará": "descrismar",
    "descrismarán": "descrismar",
    "descrismarás": "descrismar",
    "descrismaré": "descrismar",
    "descrismaréis": "descrismar",
    "descrismaría": "descrismar",
    "descrismaríais": "descrismar",
    "descrismaríamos": "descrismar",
    "descrismarían": "descrismar",
    "descrismarías": "descrismar",
    "descrismas": "descrismar",
    "descrismase": "descrismar",
    "descrismaseis": "descrismar",
    "descrismasen": "descrismar",
    "descrismases": "descrismar",
    "descrismaste": "descrismar",
    "descrismasteis": "descrismar",
    "descrisme": "descrismar",
    "descrismemos": "descrismar",
    "descrismen": "descrismar",
    "descrismes": "descrismar",
    "descrismo": "descrismar",
    "descrismábamos": "descrismar",
    "descrismáis": "descrismar",
    "descrismáramos": "descrismar",
    "descrismáremos": "descrismar",
    "descrismásemos": "descrismar",
    "descrismé": "descrismar",
    "descrisméis": "descrismar",
    "descrismó": "descrismar",
    "descristiana": "descristianar",
    "descristianaba": "descristianar",
    "descristianabais": "descristianar",
    "descristianaban": "descristianar",
    "descristianabas": "descristianar",
    "descristianad": "descristianar",
    "descristianada": "descristianar",
    "descristianadas": "descristianar",
    "descristianado": "descristianar",
    "descristianados": "descristianar",
    "descristianamos": "descristianar",
    "descristianan": "descristianar",
    "descristianando": "descristianar",
    "descristianara": "descristianar",
    "descristianarais": "descristianar",
    "descristianaran": "descristianar",
    "descristianaras": "descristianar",
    "descristianare": "descristianar",
    "descristianareis": "descristianar",
    "descristianaremos": "descristianar",
    "descristianaren": "descristianar",
    "descristianares": "descristianar",
    "descristianaron": "descristianar",
    "descristianará": "descristianar",
    "descristianarán": "descristianar",
    "descristianarás": "descristianar",
    "descristianaré": "descristianar",
    "descristianaréis": "descristianar",
    "descristianaría": "descristianar",
    "descristianaríais": "descristianar",
    "descristianaríamos": "descristianar",
    "descristianarían": "descristianar",
    "descristianarías": "descristianar",
    "descristianas": "descristianar",
    "descristianase": "descristianar",
    "descristianaseis": "descristianar",
    "descristianasen": "descristianar",
    "descristianases": "descristianar",
    "descristianaste": "descristianar",
    "descristianasteis": "descristianar",
    "descristiane": "descristianar",
    "descristianemos": "descristianar",
    "descristianen": "descristianar",
    "descristianes": "descristianar",
    "descristianice": "descristianizar",
    "descristianicemos": "descristianizar",
    "descristianicen": "descristianizar",
    "descristianices": "descristianizar",
    "descristianicé": "descristianizar",
    "descristianicéis": "descristianizar",
    "descristianiza": "descristianizar",
    "descristianizaba": "descristianizar",
    "descristianizabais": "descristianizar",
    "descristianizaban": "descristianizar",
    "descristianizabas": "descristianizar",
    "descristianizaciones": "descristianización",
    "descristianizad": "descristianizar",
    "descristianizada": "descristianizar",
    "descristianizadas": "descristianizar",
    "descristianizado": "descristianizar",
    "descristianizados": "descristianizar",
    "descristianizamos": "descristianizar",
    "descristianizan": "descristianizar",
    "descristianizando": "descristianizar",
    "descristianizara": "descristianizar",
    "descristianizarais": "descristianizar",
    "descristianizaran": "descristianizar",
    "descristianizaras": "descristianizar",
    "descristianizare": "descristianizar",
    "descristianizareis": "descristianizar",
    "descristianizaremos": "descristianizar",
    "descristianizaren": "descristianizar",
    "descristianizares": "descristianizar",
    "descristianizaron": "descristianizar",
    "descristianizará": "descristianizar",
    "descristianizarán": "descristianizar",
    "descristianizarás": "descristianizar",
    "descristianizaré": "descristianizar",
    "descristianizaréis": "descristianizar",
    "descristianizaría": "descristianizar",
    "descristianizaríais": "descristianizar",
    "descristianizaríamos": "descristianizar",
    "descristianizarían": "descristianizar",
    "descristianizarías": "descristianizar",
    "descristianizas": "descristianizar",
    "descristianizase": "descristianizar",
    "descristianizaseis": "descristianizar",
    "descristianizasen": "descristianizar",
    "descristianizases": "descristianizar",
    "descristianizaste": "descristianizar",
    "descristianizasteis": "descristianizar",
    "descristianizo": "descristianizar",
    "descristianizábamos": "descristianizar",
    "descristianizáis": "descristianizar",
    "descristianizáramos": "descristianizar",
    "descristianizáremos": "descristianizar",
    "descristianizásemos": "descristianizar",
    "descristianizó": "descristianizar",
    "descristiano": "descristianar",
    "descristianábamos": "descristianar",
    "descristianáis": "descristianar",
    "descristianáramos": "descristianar",
    "descristianáremos": "descristianar",
    "descristianásemos": "descristianar",
    "descristiané": "descristianar",
    "descristianéis": "descristianar",
    "descristianó": "descristianar",
    "descrita": "describir",
    "descritas": "describir",
    "descrito": "describir",
    "descritos": "describir",
    "descriábamos": "descriar",
    "descriáis": "descriar",
    "descriáramos": "descriar",
    "descriáremos": "descriar",
    "descriásemos": "descriar",
    "descrié": "descriar",
    "descriéis": "descriar",
    "descrió": "descriar",
    "descromados": "descromado",
    "descruce": "descruzar",
    "descrucemos": "descruzar",
    "descrucen": "descruzar",
    "descruces": "descruzar",
    "descrucé": "descruzar",
    "descrucéis": "descruzar",
    "descrudados": "descrudado",
    "descruza": "descruzar",
    "descruzaba": "descruzar",
    "descruzabais": "descruzar",
    "descruzaban": "descruzar",
    "descruzabas": "descruzar",
    "descruzad": "descruzar",
    "descruzada": "descruzar",
    "descruzadas": "descruzar",
    "descruzado": "descruzar",
    "descruzados": "descruzar",
    "descruzamos": "descruzar",
    "descruzan": "descruzar",
    "descruzando": "descruzar",
    "descruzara": "descruzar",
    "descruzarais": "descruzar",
    "descruzaran": "descruzar",
    "descruzaras": "descruzar",
    "descruzare": "descruzar",
    "descruzareis": "descruzar",
    "descruzaremos": "descruzar",
    "descruzaren": "descruzar",
    "descruzares": "descruzar",
    "descruzaron": "descruzar",
    "descruzará": "descruzar",
    "descruzarán": "descruzar",
    "descruzarás": "descruzar",
    "descruzaré": "descruzar",
    "descruzaréis": "descruzar",
    "descruzaría": "descruzar",
    "descruzaríais": "descruzar",
    "descruzaríamos": "descruzar",
    "descruzarían": "descruzar",
    "descruzarías": "descruzar",
    "descruzas": "descruzar",
    "descruzase": "descruzar",
    "descruzaseis": "descruzar",
    "descruzasen": "descruzar",
    "descruzases": "descruzar",
    "descruzaste": "descruzar",
    "descruzasteis": "descruzar",
    "descruzo": "descruzar",
    "descruzábamos": "descruzar",
    "descruzáis": "descruzar",
    "descruzáramos": "descruzar",
    "descruzáremos": "descruzar",
    "descruzásemos": "descruzar",
    "descruzó": "descruzar",
    "descréditos": "descrédito",
    "descría": "descriar",
    "descrían": "descriar",
    "descrías": "descriar",
    "descríe": "descriar",
    "descríen": "descriar",
    "descríes": "descriar",
    "descrío": "descriar",
    "descuaderna": "descuadernar",
    "descuadernaba": "descuadernar",
    "descuadernabais": "descuadernar",
    "descuadernaban": "descuadernar",
    "descuadernabas": "descuadernar",
    "descuadernad": "descuadernar",
    "descuadernada": "descuadernar",
    "descuadernadas": "descuadernar",
    "descuadernado": "descuadernar",
    "descuadernados": "descuadernar",
    "descuadernamientos": "descuadernamiento",
    "descuadernamos": "descuadernar",
    "descuadernan": "descuadernar",
    "descuadernando": "descuadernar",
    "descuadernara": "descuadernar",
    "descuadernarais": "descuadernar",
    "descuadernaran": "descuadernar",
    "descuadernaras": "descuadernar",
    "descuadernare": "descuadernar",
    "descuadernareis": "descuadernar",
    "descuadernaremos": "descuadernar",
    "descuadernaren": "descuadernar",
    "descuadernares": "descuadernar",
    "descuadernaron": "descuadernar",
    "descuadernará": "descuadernar",
    "descuadernarán": "descuadernar",
    "descuadernarás": "descuadernar",
    "descuadernaré": "descuadernar",
    "descuadernaréis": "descuadernar",
    "descuadernaría": "descuadernar",
    "descuadernaríais": "descuadernar",
    "descuadernaríamos": "descuadernar",
    "descuadernarían": "descuadernar",
    "descuadernarías": "descuadernar",
    "descuadernas": "descuadernar",
    "descuadernase": "descuadernar",
    "descuadernaseis": "descuadernar",
    "descuadernasen": "descuadernar",
    "descuadernases": "descuadernar",
    "descuadernaste": "descuadernar",
    "descuadernasteis": "descuadernar",
    "descuaderne": "descuadernar",
    "descuadernemos": "descuadernar",
    "descuadernen": "descuadernar",
    "descuadernes": "descuadernar",
    "descuaderno": "descuadernar",
    "descuadernábamos": "descuadernar",
    "descuadernáis": "descuadernar",
    "descuadernáramos": "descuadernar",
    "descuadernáremos": "descuadernar",
    "descuadernásemos": "descuadernar",
    "descuaderné": "descuadernar",
    "descuadernéis": "descuadernar",
    "descuadernó": "descuadernar",
    "descuadrilla": "descuadrillar",
    "descuadrillaba": "descuadrillar",
    "descuadrillabais": "descuadrillar",
    "descuadrillaban": "descuadrillar",
    "descuadrillabas": "descuadrillar",
    "descuadrillad": "descuadrillar",
    "descuadrillada": "descuadrillar",
    "descuadrilladas": "descuadrillar",
    "descuadrillado": "descuadrillar",
    "descuadrillados": "descuadrillar",
    "descuadrillamos": "descuadrillar",
    "descuadrillan": "descuadrillar",
    "descuadrillando": "descuadrillar",
    "descuadrillara": "descuadrillar",
    "descuadrillarais": "descuadrillar",
    "descuadrillaran": "descuadrillar",
    "descuadrillaras": "descuadrillar",
    "descuadrillare": "descuadrillar",
    "descuadrillareis": "descuadrillar",
    "descuadrillaremos": "descuadrillar",
    "descuadrillaren": "descuadrillar",
    "descuadrillares": "descuadrillar",
    "descuadrillaron": "descuadrillar",
    "descuadrillará": "descuadrillar",
    "descuadrillarán": "descuadrillar",
    "descuadrillarás": "descuadrillar",
    "descuadrillaré": "descuadrillar",
    "descuadrillaréis": "descuadrillar",
    "descuadrillaría": "descuadrillar",
    "descuadrillaríais": "descuadrillar",
    "descuadrillaríamos": "descuadrillar",
    "descuadrillarían": "descuadrillar",
    "descuadrillarías": "descuadrillar",
    "descuadrillas": "descuadrillar",
    "descuadrillase": "descuadrillar",
    "descuadrillaseis": "descuadrillar",
    "descuadrillasen": "descuadrillar",
    "descuadrillases": "descuadrillar",
    "descuadrillaste": "descuadrillar",
    "descuadrillasteis": "descuadrillar",
    "descuadrille": "descuadrillar",
    "descuadrillemos": "descuadrillar",
    "descuadrillen": "descuadrillar",
    "descuadrilles": "descuadrillar",
    "descuadrillo": "descuadrillar",
    "descuadrillábamos": "descuadrillar",
    "descuadrilláis": "descuadrillar",
    "descuadrilláramos": "descuadrillar",
    "descuadrilláremos": "descuadrillar",
    "descuadrillásemos": "descuadrillar",
    "descuadrillé": "descuadrillar",
    "descuadrilléis": "descuadrillar",
    "descuadrilló": "descuadrillar",
    "descuaja": "descuajar",
    "descuajaba": "descuajar",
    "descuajabais": "descuajar",
    "descuajaban": "descuajar",
    "descuajabas": "descuajar",
    "descuajad": "descuajar",
    "descuajada": "descuajar",
    "descuajadas": "descuajar",
    "descuajado": "descuajar",
    "descuajados": "descuajar",
    "descuajamos": "descuajar",
    "descuajan": "descuajar",
    "descuajando": "descuajar",
    "descuajara": "descuajar",
    "descuajarais": "descuajar",
    "descuajaran": "descuajar",
    "descuajaras": "descuajar",
    "descuajare": "descuajar",
    "descuajareis": "descuajar",
    "descuajaremos": "descuajar",
    "descuajaren": "descuajar",
    "descuajares": "descuajar",
    "descuajaringa": "descuajaringar",
    "descuajaringaba": "descuajaringar",
    "descuajaringabais": "descuajaringar",
    "descuajaringaban": "descuajaringar",
    "descuajaringabas": "descuajaringar",
    "descuajaringad": "descuajaringar",
    "descuajaringada": "descuajaringar",
    "descuajaringadas": "descuajaringar",
    "descuajaringado": "descuajaringar",
    "descuajaringados": "descuajaringar",
    "descuajaringamos": "descuajaringar",
    "descuajaringan": "descuajaringar",
    "descuajaringando": "descuajaringar",
    "descuajaringara": "descuajaringar",
    "descuajaringarais": "descuajaringar",
    "descuajaringaran": "descuajaringar",
    "descuajaringaras": "descuajaringar",
    "descuajaringare": "descuajaringar",
    "descuajaringareis": "descuajaringar",
    "descuajaringaremos": "descuajaringar",
    "descuajaringaren": "descuajaringar",
    "descuajaringares": "descuajaringar",
    "descuajaringaron": "descuajaringar",
    "descuajaringará": "descuajaringar",
    "descuajaringarán": "descuajaringar",
    "descuajaringarás": "descuajaringar",
    "descuajaringaré": "descuajaringar",
    "descuajaringaréis": "descuajaringar",
    "descuajaringaría": "descuajaringar",
    "descuajaringaríais": "descuajaringar",
    "descuajaringaríamos": "descuajaringar",
    "descuajaringarían": "descuajaringar",
    "descuajaringarías": "descuajaringar",
    "descuajaringas": "descuajaringar",
    "descuajaringase": "descuajaringar",
    "descuajaringaseis": "descuajaringar",
    "descuajaringasen": "descuajaringar",
    "descuajaringases": "descuajaringar",
    "descuajaringaste": "descuajaringar",
    "descuajaringasteis": "descuajaringar",
    "descuajaringo": "descuajaringar",
    "descuajaringue": "descuajaringar",
    "descuajaringuemos": "descuajaringar",
    "descuajaringuen": "descuajaringar",
    "descuajaringues": "descuajaringar",
    "descuajaringué": "descuajaringar",
    "descuajaringuéis": "descuajaringar",
    "descuajaringábamos": "descuajaringar",
    "descuajaringáis": "descuajaringar",
    "descuajaringáramos": "descuajaringar",
    "descuajaringáremos": "descuajaringar",
    "descuajaringásemos": "descuajaringar",
    "descuajaringó": "descuajaringar",
    "descuajaron": "descuajar",
    "descuajará": "descuajar",
    "descuajarán": "descuajar",
    "descuajarás": "descuajar",
    "descuajaré": "descuajar",
    "descuajaréis": "descuajar",
    "descuajaría": "descuajar",
    "descuajaríais": "descuajar",
    "descuajaríamos": "descuajar",
    "descuajarían": "descuajar",
    "descuajarías": "descuajar",
    "descuajas": "descuajar",
    "descuajase": "descuajar",
    "descuajaseis": "descuajar",
    "descuajasen": "descuajar",
    "descuajases": "descuajar",
    "descuajaste": "descuajar",
    "descuajasteis": "descuajar",
    "descuaje": "descuajar",
    "descuajemos": "descuajar",
    "descuajen": "descuajar",
    "descuajes": "descuaje",
    "descuajo": "descuajar",
    "descuajos": "descuajo",
    "descuajábamos": "descuajar",
    "descuajáis": "descuajar",
    "descuajáramos": "descuajar",
    "descuajáremos": "descuajar",
    "descuajásemos": "descuajar",
    "descuajé": "descuajar",
    "descuajéis": "descuajar",
    "descuajó": "descuajar",
    "descuartice": "descuartizar",
    "descuarticemos": "descuartizar",
    "descuarticen": "descuartizar",
    "descuartices": "descuartizar",
    "descuarticé": "descuartizar",
    "descuarticéis": "descuartizar",
    "descuartiza": "descuartizar",
    "descuartizaba": "descuartizar",
    "descuartizabais": "descuartizar",
    "descuartizaban": "descuartizar",
    "descuartizabas": "descuartizar",
    "descuartizad": "descuartizar",
    "descuartizada": "descuartizar",
    "descuartizadas": "descuartizar",
    "descuartizado": "descuartizar",
    "descuartizadora": "descuartizador",
    "descuartizadoras": "descuartizador",
    "descuartizadores": "descuartizador",
    "descuartizados": "descuartizar",
    "descuartizamientos": "descuartizamiento",
    "descuartizamos": "descuartizar",
    "descuartizan": "descuartizar",
    "descuartizando": "descuartizar",
    "descuartizara": "descuartizar",
    "descuartizarais": "descuartizar",
    "descuartizaran": "descuartizar",
    "descuartizaras": "descuartizar",
    "descuartizare": "descuartizar",
    "descuartizareis": "descuartizar",
    "descuartizaremos": "descuartizar",
    "descuartizaren": "descuartizar",
    "descuartizares": "descuartizar",
    "descuartizaron": "descuartizar",
    "descuartizará": "descuartizar",
    "descuartizarán": "descuartizar",
    "descuartizarás": "descuartizar",
    "descuartizaré": "descuartizar",
    "descuartizaréis": "descuartizar",
    "descuartizaría": "descuartizar",
    "descuartizaríais": "descuartizar",
    "descuartizaríamos": "descuartizar",
    "descuartizarían": "descuartizar",
    "descuartizarías": "descuartizar",
    "descuartizas": "descuartizar",
    "descuartizase": "descuartizar",
    "descuartizaseis": "descuartizar",
    "descuartizasen": "descuartizar",
    "descuartizases": "descuartizar",
    "descuartizaste": "descuartizar",
    "descuartizasteis": "descuartizar",
    "descuartizo": "descuartizar",
    "descuartizábamos": "descuartizar",
    "descuartizáis": "descuartizar",
    "descuartizáramos": "descuartizar",
    "descuartizáremos": "descuartizar",
    "descuartizásemos": "descuartizar",
    "descuartizó": "descuartizar",
    "descubierta": "descubrir",
    "descubiertas": "descubrir",
    "descubierto": "descubrir",
    "descubiertos": "descubrir",
    "descubra": "descubrir",
    "descubramos": "descubrir",
    "descubran": "descubrir",
    "descubras": "descubrir",
    "descubre": "descubrir",
    "descubren": "descubrir",
    "descubres": "descubrir",
    "descubrid": "descubrir",
    "descubrideros": "descubridero",
    "descubridora": "descubridor",
    "descubridoras": "descubridor",
    "descubridores": "descubridor",
    "descubriendo": "descubrir",
    "descubriera": "descubrir",
    "descubrierais": "descubrir",
    "descubrieran": "descubrir",
    "descubrieras": "descubrir",
    "descubriere": "descubrir",
    "descubriereis": "descubrir",
    "descubrieren": "descubrir",
    "descubrieres": "descubrir",
    "descubrieron": "descubrir",
    "descubriese": "descubrir",
    "descubrieseis": "descubrir",
    "descubriesen": "descubrir",
    "descubrieses": "descubrir",
    "descubrimientos": "descubrimiento",
    "descubrimos": "descubrir",
    "descubriremos": "descubrir",
    "descubrirá": "descubrir",
    "descubrirán": "descubrir",
    "descubrirás": "descubrir",
    "descubriré": "descubrir",
    "descubriréis": "descubrir",
    "descubriría": "descubrir",
    "descubriríais": "descubrir",
    "descubriríamos": "descubrir",
    "descubrirían": "descubrir",
    "descubrirías": "descubrir",
    "descubriste": "descubrir",
    "descubristeis": "descubrir",
    "descubriéramos": "descubrir",
    "descubriéremos": "descubrir",
    "descubriésemos": "descubrir",
    "descubrió": "descubrir",
    "descubro": "descubrir",
    "descubráis": "descubrir",
    "descubrí": "descubrir",
    "descubría": "descubrir",
    "descubríais": "descubrir",
    "descubríamos": "descubrir",
    "descubrían": "descubrir",
    "descubrías": "descubrir",
    "descubrís": "descubrir",
    "descuelga": "descolgar",
    "descuelgan": "descolgar",
    "descuelgas": "descolgar",
    "descuelgo": "descolgar",
    "descuelgue": "descolgar",
    "descuelguen": "descolgar",
    "descuelgues": "descolgar",
    "descuella": "descollar",
    "descuellan": "descollar",
    "descuellas": "descollar",
    "descuelle": "descollar",
    "descuellen": "descollar",
    "descuelles": "descollar",
    "descuello": "descollar",
    "descuellos": "descuello",
    "descuenta": "descontar",
    "descuentan": "descontar",
    "descuentas": "descontar",
    "descuente": "descontar",
    "descuenten": "descontar",
    "descuentes": "descontar",
    "descuento": "descontar",
    "descuentos": "descuento",
    "descuera": "descuerar",
    "descueraba": "descuerar",
    "descuerabais": "descuerar",
    "descueraban": "descuerar",
    "descuerabas": "descuerar",
    "descuerad": "descuerar",
    "descuerada": "descuerar",
    "descueradas": "descuerar",
    "descuerado": "descuerar",
    "descuerados": "descuerar",
    "descueramos": "descuerar",
    "descueran": "descuerar",
    "descuerando": "descuerar",
    "descuerara": "descuerar",
    "descuerarais": "descuerar",
    "descueraran": "descuerar",
    "descueraras": "descuerar",
    "descuerare": "descuerar",
    "descuerareis": "descuerar",
    "descueraremos": "descuerar",
    "descueraren": "descuerar",
    "descuerares": "descuerar",
    "descueraron": "descuerar",
    "descuerará": "descuerar",
    "descuerarán": "descuerar",
    "descuerarás": "descuerar",
    "descueraré": "descuerar",
    "descueraréis": "descuerar",
    "descueraría": "descuerar",
    "descueraríais": "descuerar",
    "descueraríamos": "descuerar",
    "descuerarían": "descuerar",
    "descuerarías": "descuerar",
    "descueras": "descuerar",
    "descuerase": "descuerar",
    "descueraseis": "descuerar",
    "descuerasen": "descuerar",
    "descuerases": "descuerar",
    "descueraste": "descuerar",
    "descuerasteis": "descuerar",
    "descuerda": "descordar",
    "descuerdan": "descordar",
    "descuerdas": "descordar",
    "descuerde": "descordar",
    "descuerden": "descordar",
    "descuerdes": "descordar",
    "descuerdo": "descordar",
    "descuere": "descuerar",
    "descueremos": "descuerar",
    "descueren": "descuerar",
    "descueres": "descuerar",
    "descuerna": "descornar",
    "descuernan": "descornar",
    "descuernas": "descornar",
    "descuerne": "descornar",
    "descuernen": "descornar",
    "descuernes": "descornar",
    "descuerno": "descornar",
    "descuernos": "descuerno",
    "descuero": "descuerar",
    "descuerábamos": "descuerar",
    "descueráis": "descuerar",
    "descueráramos": "descuerar",
    "descueráremos": "descuerar",
    "descuerásemos": "descuerar",
    "descueré": "descuerar",
    "descueréis": "descuerar",
    "descueró": "descuerar",
    "descuesta": "descostar",
    "descuestan": "descostar",
    "descuestas": "descostar",
    "descueste": "descostar",
    "descuesten": "descostar",
    "descuestes": "descostar",
    "descuesto": "descostar",
    "descuida": "descuidar",
    "descuidaba": "descuidar",
    "descuidabais": "descuidar",
    "descuidaban": "descuidar",
    "descuidabas": "descuidar",
    "descuidad": "descuidar",
    "descuidada": "descuidar",
    "descuidadas": "descuidar",
    "descuidado": "descuidar",
    "descuidados": "descuidar",
    "descuidamos": "descuidar",
    "descuidan": "descuidar",
    "descuidando": "descuidar",
    "descuidara": "descuidar",
    "descuidarais": "descuidar",
    "descuidaran": "descuidar",
    "descuidaras": "descuidar",
    "descuidare": "descuidar",
    "descuidareis": "descuidar",
    "descuidaremos": "descuidar",
    "descuidaren": "descuidar",
    "descuidares": "descuidar",
    "descuidaron": "descuidar",
    "descuidará": "descuidar",
    "descuidarán": "descuidar",
    "descuidarás": "descuidar",
    "descuidaré": "descuidar",
    "descuidaréis": "descuidar",
    "descuidaría": "descuidar",
    "descuidaríais": "descuidar",
    "descuidaríamos": "descuidar",
    "descuidarían": "descuidar",
    "descuidarías": "descuidar",
    "descuidas": "descuidar",
    "descuidase": "descuidar",
    "descuidaseis": "descuidar",
    "descuidasen": "descuidar",
    "descuidases": "descuidar",
    "descuidaste": "descuidar",
    "descuidasteis": "descuidar",
    "descuide": "descuidar",
    "descuidemos": "descuidar",
    "descuiden": "descuidar",
    "descuidera": "descuidero",
    "descuideras": "descuidero",
    "descuideros": "descuidero",
    "descuides": "descuidar",
    "descuido": "descuidar",
    "descuidos": "descuido",
    "descuidábamos": "descuidar",
    "descuidáis": "descuidar",
    "descuidáramos": "descuidar",
    "descuidáremos": "descuidar",
    "descuidásemos": "descuidar",
    "descuidé": "descuidar",
    "descuidéis": "descuidar",
    "descuidó": "descuidar",
    "descuitada": "descuitado",
    "descuitadas": "descuitado",
    "descuitados": "descuitado",
    "descula": "descular",
    "desculaba": "descular",
    "desculabais": "descular",
    "desculaban": "descular",
    "desculabas": "descular",
    "desculad": "descular",
    "desculada": "descular",
    "desculadas": "descular",
    "desculado": "descular",
    "desculados": "descular",
    "desculamos": "descular",
    "desculan": "descular",
    "desculando": "descular",
    "desculara": "descular",
    "descularais": "descular",
    "descularan": "descular",
    "descularas": "descular",
    "desculare": "descular",
    "desculareis": "descular",
    "descularemos": "descular",
    "descularen": "descular",
    "desculares": "descular",
    "descularon": "descular",
    "desculará": "descular",
    "descularán": "descular",
    "descularás": "descular",
    "descularé": "descular",
    "descularéis": "descular",
    "descularía": "descular",
    "descularíais": "descular",
    "descularíamos": "descular",
    "descularían": "descular",
    "descularías": "descular",
    "desculas": "descular",
    "desculase": "descular",
    "desculaseis": "descular",
    "desculasen": "descular",
    "desculases": "descular",
    "desculaste": "descular",
    "desculasteis": "descular",
    "descule": "descular",
    "desculemos": "descular",
    "desculen": "descular",
    "descules": "descular",
    "desculo": "descular",
    "desculábamos": "descular",
    "desculáis": "descular",
    "desculáramos": "descular",
    "desculáremos": "descular",
    "desculásemos": "descular",
    "desculé": "descular",
    "desculéis": "descular",
    "desculó": "descular",
    "descumbrada": "descumbrado",
    "descumbradas": "descumbrado",
    "descumbrados": "descumbrado",
    "descuscutadoras": "descuscutadora",
    "desdecid": "desdecir",
    "desdecimientos": "desdecimiento",
    "desdecimos": "desdecir",
    "desdeciremos": "desdecir",
    "desdecirá": "desdecir",
    "desdecirán": "desdecir",
    "desdecirás": "desdecir",
    "desdeciré": "desdecir",
    "desdeciréis": "desdecir",
    "desdeciría": "desdecir",
    "desdeciríais": "desdecir",
    "desdeciríamos": "desdecir",
    "desdecirían": "desdecir",
    "desdecirías": "desdecir",
    "desdecía": "desdecir",
    "desdecíais": "desdecir",
    "desdecíamos": "desdecir",
    "desdecían": "desdecir",
    "desdecías": "desdecir",
    "desdecís": "desdecir",
    "desdenes": "desdén",
    "desdentaba": "desdentar",
    "desdentabais": "desdentar",
    "desdentaban": "desdentar",
    "desdentabas": "desdentar",
    "desdentad": "desdentar",
    "desdentada": "desdentar",
    "desdentadas": "desdentar",
    "desdentado": "desdentar",
    "desdentados": "desdentar",
    "desdentamos": "desdentar",
    "desdentando": "desdentar",
    "desdentara": "desdentar",
    "desdentarais": "desdentar",
    "desdentaran": "desdentar",
    "desdentaras": "desdentar",
    "desdentare": "desdentar",
    "desdentareis": "desdentar",
    "desdentaremos": "desdentar",
    "desdentaren": "desdentar",
    "desdentares": "desdentar",
    "desdentaron": "desdentar",
    "desdentará": "desdentar",
    "desdentarán": "desdentar",
    "desdentarás": "desdentar",
    "desdentaré": "desdentar",
    "desdentaréis": "desdentar",
    "desdentaría": "desdentar",
    "desdentaríais": "desdentar",
    "desdentaríamos": "desdentar",
    "desdentarían": "desdentar",
    "desdentarías": "desdentar",
    "desdentase": "desdentar",
    "desdentaseis": "desdentar",
    "desdentasen": "desdentar",
    "desdentases": "desdentar",
    "desdentaste": "desdentar",
    "desdentasteis": "desdentar",
    "desdentemos": "desdentar",
    "desdentábamos": "desdentar",
    "desdentáis": "desdentar",
    "desdentáramos": "desdentar",
    "desdentáremos": "desdentar",
    "desdentásemos": "desdentar",
    "desdenté": "desdentar",
    "desdentéis": "desdentar",
    "desdentó": "desdentar",
    "desdeña": "desdeñar",
    "desdeñaba": "desdeñar",
    "desdeñabais": "desdeñar",
    "desdeñaban": "desdeñar",
    "desdeñabas": "desdeñar",
    "desdeñablee": "desdeñable",
    "desdeñablees": "desdeñable",
    "desdeñables": "desdeñable",
    "desdeñad": "desdeñar",
    "desdeñada": "desdeñar",
    "desdeñadas": "desdeñar",
    "desdeñado": "desdeñar",
    "desdeñadora": "desdeñador",
    "desdeñadoras": "desdeñador",
    "desdeñadores": "desdeñador",
    "desdeñados": "desdeñar",
    "desdeñamos": "desdeñar",
    "desdeñan": "desdeñar",
    "desdeñando": "desdeñar",
    "desdeñara": "desdeñar",
    "desdeñarais": "desdeñar",
    "desdeñaran": "desdeñar",
    "desdeñaras": "desdeñar",
    "desdeñare": "desdeñar",
    "desdeñareis": "desdeñar",
    "desdeñaremos": "desdeñar",
    "desdeñaren": "desdeñar",
    "desdeñares": "desdeñar",
    "desdeñaron": "desdeñar",
    "desdeñará": "desdeñar",
    "desdeñarán": "desdeñar",
    "desdeñarás": "desdeñar",
    "desdeñaré": "desdeñar",
    "desdeñaréis": "desdeñar",
    "desdeñaría": "desdeñar",
    "desdeñaríais": "desdeñar",
    "desdeñaríamos": "desdeñar",
    "desdeñarían": "desdeñar",
    "desdeñarías": "desdeñar",
    "desdeñas": "desdeñar",
    "desdeñase": "desdeñar",
    "desdeñaseis": "desdeñar",
    "desdeñasen": "desdeñar",
    "desdeñases": "desdeñar",
    "desdeñaste": "desdeñar",
    "desdeñasteis": "desdeñar",
    "desdeñe": "desdeñar",
    "desdeñemos": "desdeñar",
    "desdeñen": "desdeñar",
    "desdeñes": "desdeñar",
    "desdeño": "desdeñar",
    "desdeñosa": "desdeñoso",
    "desdeñosas": "desdeñoso",
    "desdeñosos": "desdeñoso",
    "desdeñábamos": "desdeñar",
    "desdeñáis": "desdeñar",
    "desdeñáramos": "desdeñar",
    "desdeñáremos": "desdeñar",
    "desdeñásemos": "desdeñar",
    "desdeñé": "desdeñar",
    "desdeñéis": "desdeñar",
    "desdeñó": "desdeñar",
    "desdibuja": "desdibujar",
    "desdibujaba": "desdibujar",
    "desdibujabais": "desdibujar",
    "desdibujaban": "desdibujar",
    "desdibujabas": "desdibujar",
    "desdibujad": "desdibujar",
    "desdibujada": "desdibujar",
    "desdibujadas": "desdibujar",
    "desdibujado": "desdibujar",
    "desdibujados": "desdibujar",
    "desdibujamientos": "desdibujamiento",
    "desdibujamos": "desdibujar",
    "desdibujan": "desdibujar",
    "desdibujando": "desdibujar",
    "desdibujara": "desdibujar",
    "desdibujarais": "desdibujar",
    "desdibujaran": "desdibujar",
    "desdibujaras": "desdibujar",
    "desdibujare": "desdibujar",
    "desdibujareis": "desdibujar",
    "desdibujaremos": "desdibujar",
    "desdibujaren": "desdibujar",
    "desdibujares": "desdibujar",
    "desdibujaron": "desdibujar",
    "desdibujará": "desdibujar",
    "desdibujarán": "desdibujar",
    "desdibujarás": "desdibujar",
    "desdibujaré": "desdibujar",
    "desdibujaréis": "desdibujar",
    "desdibujaría": "desdibujar",
    "desdibujaríais": "desdibujar",
    "desdibujaríamos": "desdibujar",
    "desdibujarían": "desdibujar",
    "desdibujarías": "desdibujar",
    "desdibujas": "desdibujar",
    "desdibujase": "desdibujar",
    "desdibujaseis": "desdibujar",
    "desdibujasen": "desdibujar",
    "desdibujases": "desdibujar",
    "desdibujaste": "desdibujar",
    "desdibujasteis": "desdibujar",
    "desdibuje": "desdibujar",
    "desdibujemos": "desdibujar",
    "desdibujen": "desdibujar",
    "desdibujes": "desdibujar",
    "desdibujo": "desdibujar",
    "desdibujábamos": "desdibujar",
    "desdibujáis": "desdibujar",
    "desdibujáramos": "desdibujar",
    "desdibujáremos": "desdibujar",
    "desdibujásemos": "desdibujar",
    "desdibujé": "desdibujar",
    "desdibujéis": "desdibujar",
    "desdibujó": "desdibujar",
    "desdice": "desdecir",
    "desdicen": "desdecir",
    "desdices": "desdecir",
    "desdicha": "desdecir",
    "desdichada": "desdichado",
    "desdichadas": "desdichado",
    "desdichados": "desdichado",
    "desdichas": "desdicha",
    "desdicho": "desdecir",
    "desdichos": "desdecir",
    "desdiciendo": "desdecir",
    "desdienta": "desdentar",
    "desdientan": "desdentar",
    "desdientas": "desdentar",
    "desdiente": "desdentar",
    "desdienten": "desdentar",
    "desdientes": "desdentar",
    "desdiento": "desdentar",
    "desdiga": "desdecir",
    "desdigamos": "desdecir",
    "desdigan": "desdecir",
    "desdigas": "desdecir",
    "desdigo": "desdecir",
    "desdigáis": "desdecir",
    "desdije": "desdecir",
    "desdijera": "desdecir",
    "desdijerais": "desdecir",
    "desdijeran": "desdecir",
    "desdijeras": "desdecir",
    "desdijere": "desdecir",
    "desdijereis": "desdecir",
    "desdijeren": "desdecir",
    "desdijeres": "desdecir",
    "desdijeron": "desdecir",
    "desdijese": "desdecir",
    "desdijeseis": "desdecir",
    "desdijesen": "desdecir",
    "desdijeses": "desdecir",
    "desdijimos": "desdecir",
    "desdijiste": "desdecir",
    "desdijisteis": "desdecir",
    "desdijo": "desdecir",
    "desdijéramos": "desdecir",
    "desdijéremos": "desdecir",
    "desdijésemos": "desdecir",
    "desdobla": "desdoblar",
    "desdoblaba": "desdoblar",
    "desdoblabais": "desdoblar",
    "desdoblaban": "desdoblar",
    "desdoblabas": "desdoblar",
    "desdoblad": "desdoblar",
    "desdoblada": "desdoblar",
    "desdobladas": "desdoblar",
    "desdoblado": "desdoblar",
    "desdoblados": "desdoblar",
    "desdoblamientos": "desdoblamiento",
    "desdoblamos": "desdoblar",
    "desdoblan": "desdoblar",
    "desdoblando": "desdoblar",
    "desdoblara": "desdoblar",
    "desdoblarais": "desdoblar",
    "desdoblaran": "desdoblar",
    "desdoblaras": "desdoblar",
    "desdoblare": "desdoblar",
    "desdoblareis": "desdoblar",
    "desdoblaremos": "desdoblar",
    "desdoblaren": "desdoblar",
    "desdoblares": "desdoblar",
    "desdoblaron": "desdoblar",
    "desdoblará": "desdoblar",
    "desdoblarán": "desdoblar",
    "desdoblarás": "desdoblar",
    "desdoblaré": "desdoblar",
    "desdoblaréis": "desdoblar",
    "desdoblaría": "desdoblar",
    "desdoblaríais": "desdoblar",
    "desdoblaríamos": "desdoblar",
    "desdoblarían": "desdoblar",
    "desdoblarías": "desdoblar",
    "desdoblas": "desdoblar",
    "desdoblase": "desdoblar",
    "desdoblaseis": "desdoblar",
    "desdoblasen": "desdoblar",
    "desdoblases": "desdoblar",
    "desdoblaste": "desdoblar",
    "desdoblasteis": "desdoblar",
    "desdoble": "desdoblar",
    "desdoblemos": "desdoblar",
    "desdoblen": "desdoblar",
    "desdobles": "desdoblar",
    "desdoblo": "desdoblar",
    "desdoblábamos": "desdoblar",
    "desdobláis": "desdoblar",
    "desdobláramos": "desdoblar",
    "desdobláremos": "desdoblar",
    "desdoblásemos": "desdoblar",
    "desdoblé": "desdoblar",
    "desdobléis": "desdoblar",
    "desdobló": "desdoblar",
    "desdora": "desdorar",
    "desdoraba": "desdorar",
    "desdorabais": "desdorar",
    "desdoraban": "desdorar",
    "desdorabas": "desdorar",
    "desdorad": "desdorar",
    "desdorada": "desdorar",
    "desdoradas": "desdorar",
    "desdorado": "desdorar",
    "desdorados": "desdorar",
    "desdoramos": "desdorar",
    "desdoran": "desdorar",
    "desdorando": "desdorar",
    "desdorara": "desdorar",
    "desdorarais": "desdorar",
    "desdoraran": "desdorar",
    "desdoraras": "desdorar",
    "desdorare": "desdorar",
    "desdorareis": "desdorar",
    "desdoraremos": "desdorar",
    "desdoraren": "desdorar",
    "desdorares": "desdorar",
    "desdoraron": "desdorar",
    "desdorará": "desdorar",
    "desdorarán": "desdorar",
    "desdorarás": "desdorar",
    "desdoraré": "desdorar",
    "desdoraréis": "desdorar",
    "desdoraría": "desdorar",
    "desdoraríais": "desdorar",
    "desdoraríamos": "desdorar",
    "desdorarían": "desdorar",
    "desdorarías": "desdorar",
    "desdoras": "desdorar",
    "desdorase": "desdorar",
    "desdoraseis": "desdorar",
    "desdorasen": "desdorar",
    "desdorases": "desdorar",
    "desdoraste": "desdorar",
    "desdorasteis": "desdorar",
    "desdore": "desdorar",
    "desdoremos": "desdorar",
    "desdoren": "desdorar",
    "desdores": "desdorar",
    "desdoro": "desdorar",
    "desdoros": "desdoro",
    "desdorosa": "desdoroso",
    "desdorosas": "desdoroso",
    "desdorosos": "desdoroso",
    "desdorábamos": "desdorar",
    "desdoráis": "desdorar",
    "desdoráramos": "desdorar",
    "desdoráremos": "desdorar",
    "desdorásemos": "desdorar",
    "desdoré": "desdorar",
    "desdoréis": "desdorar",
    "desdoró": "desdorar",
    "desdramatice": "desdramatizar",
    "desdramaticemos": "desdramatizar",
    "desdramaticen": "desdramatizar",
    "desdramatices": "desdramatizar",
    "desdramaticé": "desdramatizar",
    "desdramaticéis": "desdramatizar",
    "desdramatiza": "desdramatizar",
    "desdramatizaba": "desdramatizar",
    "desdramatizabais": "desdramatizar",
    "desdramatizaban": "desdramatizar",
    "desdramatizabas": "desdramatizar",
    "desdramatizad": "desdramatizar",
    "desdramatizada": "desdramatizar",
    "desdramatizadas": "desdramatizar",
    "desdramatizado": "desdramatizar",
    "desdramatizados": "desdramatizar",
    "desdramatizamos": "desdramatizar",
    "desdramatizan": "desdramatizar",
    "desdramatizando": "desdramatizar",
    "desdramatizara": "desdramatizar",
    "desdramatizarais": "desdramatizar",
    "desdramatizaran": "desdramatizar",
    "desdramatizaras": "desdramatizar",
    "desdramatizare": "desdramatizar",
    "desdramatizareis": "desdramatizar",
    "desdramatizaremos": "desdramatizar",
    "desdramatizaren": "desdramatizar",
    "desdramatizares": "desdramatizar",
    "desdramatizaron": "desdramatizar",
    "desdramatizará": "desdramatizar",
    "desdramatizarán": "desdramatizar",
    "desdramatizarás": "desdramatizar",
    "desdramatizaré": "desdramatizar",
    "desdramatizaréis": "desdramatizar",
    "desdramatizaría": "desdramatizar",
    "desdramatizaríais": "desdramatizar",
    "desdramatizaríamos": "desdramatizar",
    "desdramatizarían": "desdramatizar",
    "desdramatizarías": "desdramatizar",
    "desdramatizas": "desdramatizar",
    "desdramatizase": "desdramatizar",
    "desdramatizaseis": "desdramatizar",
    "desdramatizasen": "desdramatizar",
    "desdramatizases": "desdramatizar",
    "desdramatizaste": "desdramatizar",
    "desdramatizasteis": "desdramatizar",
    "desdramatizo": "desdramatizar",
    "desdramatizábamos": "desdramatizar",
    "desdramatizáis": "desdramatizar",
    "desdramatizáramos": "desdramatizar",
    "desdramatizáremos": "desdramatizar",
    "desdramatizásemos": "desdramatizar",
    "desdramatizó": "desdramatizar",
    "desea": "desear",
    "deseaba": "desear",
    "deseabais": "desear",
    "deseaban": "desear",
    "deseabas": "desear",
    "deseables": "deseable",
    "desead": "desear",
    "deseada": "desear",
    "deseadas": "desear",
    "deseado": "desear",
    "deseados": "desear",
    "deseamos": "desear",
    "desean": "desear",
    "deseando": "desear",
    "deseara": "desear",
    "desearais": "desear",
    "desearan": "desear",
    "desearas": "desear",
    "deseare": "desear",
    "deseareis": "desear",
    "desearemos": "desear",
    "desearen": "desear",
    "deseares": "desear",
    "desearon": "desear",
    "deseará": "desear",
    "desearán": "desear",
    "desearás": "desear",
    "desearé": "desear",
    "desearéis": "desear",
    "desearía": "desear",
    "desearíais": "desear",
    "desearíamos": "desear",
    "desearían": "desear",
    "desearías": "desear",
    "deseas": "desear",
    "desease": "desear",
    "deseaseis": "desear",
    "deseasen": "desear",
    "deseases": "desear",
    "deseaste": "desear",
    "deseasteis": "desear",
    "deseca": "desecar",
    "desecaba": "desecar",
    "desecabais": "desecar",
    "desecaban": "desecar",
    "desecabas": "desecar",
    "desecaciones": "desecación",
    "desecad": "desecar",
    "desecada": "desecar",
    "desecadas": "desecar",
    "desecado": "desecar",
    "desecadora": "desecador",
    "desecadoras": "desecador",
    "desecadores": "desecador",
    "desecados": "desecar",
    "desecamientos": "desecamiento",
    "desecamos": "desecar",
    "desecan": "desecar",
    "desecando": "desecar",
    "desecantes": "desecante",
    "desecara": "desecar",
    "desecarais": "desecar",
    "desecaran": "desecar",
    "desecaras": "desecar",
    "desecare": "desecar",
    "desecareis": "desecar",
    "desecaremos": "desecar",
    "desecaren": "desecar",
    "desecares": "desecar",
    "desecaron": "desecar",
    "desecará": "desecar",
    "desecarán": "desecar",
    "desecarás": "desecar",
    "desecaré": "desecar",
    "desecaréis": "desecar",
    "desecaría": "desecar",
    "desecaríais": "desecar",
    "desecaríamos": "desecar",
    "desecarían": "desecar",
    "desecarías": "desecar",
    "desecas": "desecar",
    "desecase": "desecar",
    "desecaseis": "desecar",
    "desecasen": "desecar",
    "desecases": "desecar",
    "desecaste": "desecar",
    "desecasteis": "desecar",
    "desecativa": "desecativo",
    "desecativas": "desecativo",
    "desecativos": "desecativo",
    "desecha": "desechar",
    "desechaba": "desechar",
    "desechabais": "desechar",
    "desechaban": "desechar",
    "desechabas": "desechar",
    "desechables": "desechable",
    "desechad": "desechar",
    "desechada": "desechar",
    "desechadas": "desechar",
    "desechado": "desechar",
    "desechados": "desechar",
    "desechamos": "desechar",
    "desechan": "desechar",
    "desechando": "desechar",
    "desechara": "desechar",
    "desecharais": "desechar",
    "desecharan": "desechar",
    "desecharas": "desechar",
    "desechare": "desechar",
    "desechareis": "desechar",
    "desecharemos": "desechar",
    "desecharen": "desechar",
    "desechares": "desechar",
    "desecharon": "desechar",
    "desechará": "desechar",
    "desecharán": "desechar",
    "desecharás": "desechar",
    "desecharé": "desechar",
    "desecharéis": "desechar",
    "desecharía": "desechar",
    "desecharíais": "desechar",
    "desecharíamos": "desechar",
    "desecharían": "desechar",
    "desecharías": "desechar",
    "desechas": "desechar",
    "desechase": "desechar",
    "desechaseis": "desechar",
    "desechasen": "desechar",
    "desechases": "desechar",
    "desechaste": "desechar",
    "desechasteis": "desechar",
    "deseche": "desechar",
    "desechemos": "desechar",
    "desechen": "desechar",
    "deseches": "desechar",
    "desecho": "desechar",
    "desechos": "desecho",
    "desechábamos": "desechar",
    "desecháis": "desechar",
    "desecháramos": "desechar",
    "desecháremos": "desechar",
    "desechásemos": "desechar",
    "deseché": "desechar",
    "desechéis": "desechar",
    "desechó": "desechar",
    "deseco": "desecar",
    "deseconomías": "deseconomía",
    "desecábamos": "desecar",
    "desecáis": "desecar",
    "desecáramos": "desecar",
    "desecáremos": "desecar",
    "desecásemos": "desecar",
    "desecó": "desecar",
    "desedificaciones": "desedificación",
    "desee": "desear",
    "deseemos": "desear",
    "deseen": "desear",
    "desees": "desear",
    "deselectrice": "deselectrizar",
    "deselectricemos": "deselectrizar",
    "deselectricen": "deselectrizar",
    "deselectrices": "deselectrizar",
    "deselectricé": "deselectrizar",
    "deselectricéis": "deselectrizar",
    "deselectriza": "deselectrizar",
    "deselectrizaba": "deselectrizar",
    "deselectrizabais": "deselectrizar",
    "deselectrizaban": "deselectrizar",
    "deselectrizabas": "deselectrizar",
    "deselectrizaciones": "deselectrización",
    "deselectrizad": "deselectrizar",
    "deselectrizada": "deselectrizar",
    "deselectrizadas": "deselectrizar",
    "deselectrizado": "deselectrizar",
    "deselectrizados": "deselectrizar",
    "deselectrizamientos": "deselectrizamiento",
    "deselectrizamos": "deselectrizar",
    "deselectrizan": "deselectrizar",
    "deselectrizando": "deselectrizar",
    "deselectrizara": "deselectrizar",
    "deselectrizarais": "deselectrizar",
    "deselectrizaran": "deselectrizar",
    "deselectrizaras": "deselectrizar",
    "deselectrizare": "deselectrizar",
    "deselectrizareis": "deselectrizar",
    "deselectrizaremos": "deselectrizar",
    "deselectrizaren": "deselectrizar",
    "deselectrizares": "deselectrizar",
    "deselectrizaron": "deselectrizar",
    "deselectrizará": "deselectrizar",
    "deselectrizarán": "deselectrizar",
    "deselectrizarás": "deselectrizar",
    "deselectrizaré": "deselectrizar",
    "deselectrizaréis": "deselectrizar",
    "deselectrizaría": "deselectrizar",
    "deselectrizaríais": "deselectrizar",
    "deselectrizaríamos": "deselectrizar",
    "deselectrizarían": "deselectrizar",
    "deselectrizarías": "deselectrizar",
    "deselectrizas": "deselectrizar",
    "deselectrizase": "deselectrizar",
    "deselectrizaseis": "deselectrizar",
    "deselectrizasen": "deselectrizar",
    "deselectrizases": "deselectrizar",
    "deselectrizaste": "deselectrizar",
    "deselectrizasteis": "deselectrizar",
    "deselectrizo": "deselectrizar",
    "deselectrizábamos": "deselectrizar",
    "deselectrizáis": "deselectrizar",
    "deselectrizáramos": "deselectrizar",
    "deselectrizáremos": "deselectrizar",
    "deselectrizásemos": "deselectrizar",
    "deselectrizó": "deselectrizar",
    "desella": "desellar",
    "desellaba": "desellar",
    "desellabais": "desellar",
    "desellaban": "desellar",
    "desellabas": "desellar",
    "desellad": "desellar",
    "desellada": "desellar",
    "deselladas": "desellar",
    "desellado": "desellar",
    "desellados": "desellar",
    "deselladuras": "deselladura",
    "desellamos": "desellar",
    "desellan": "desellar",
    "desellando": "desellar",
    "desellara": "desellar",
    "desellarais": "desellar",
    "desellaran": "desellar",
    "desellaras": "desellar",
    "desellare": "desellar",
    "desellareis": "desellar",
    "desellaremos": "desellar",
    "desellaren": "desellar",
    "desellares": "desellar",
    "desellaron": "desellar",
    "desellará": "desellar",
    "desellarán": "desellar",
    "desellarás": "desellar",
    "desellaré": "desellar",
    "desellaréis": "desellar",
    "desellaría": "desellar",
    "desellaríais": "desellar",
    "desellaríamos": "desellar",
    "desellarían": "desellar",
    "desellarías": "desellar",
    "desellas": "desellar",
    "desellase": "desellar",
    "desellaseis": "desellar",
    "desellasen": "desellar",
    "desellases": "desellar",
    "desellaste": "desellar",
    "desellasteis": "desellar",
    "deselle": "desellar",
    "desellemos": "desellar",
    "desellen": "desellar",
    "deselles": "desellar",
    "desello": "desellar",
    "desellábamos": "desellar",
    "deselláis": "desellar",
    "deselláramos": "desellar",
    "deselláremos": "desellar",
    "desellásemos": "desellar",
    "desellé": "desellar",
    "deselléis": "desellar",
    "deselló": "desellar",
    "desembala": "desembalar",
    "desembalaba": "desembalar",
    "desembalabais": "desembalar",
    "desembalaban": "desembalar",
    "desembalabas": "desembalar",
    "desembalad": "desembalar",
    "desembalada": "desembalar",
    "desembaladas": "desembalar",
    "desembalado": "desembalar",
    "desembalados": "desembalar",
    "desembalajes": "desembalaje",
    "desembalamos": "desembalar",
    "desembalan": "desembalar",
    "desembalando": "desembalar",
    "desembalara": "desembalar",
    "desembalarais": "desembalar",
    "desembalaran": "desembalar",
    "desembalaras": "desembalar",
    "desembalare": "desembalar",
    "desembalareis": "desembalar",
    "desembalaremos": "desembalar",
    "desembalaren": "desembalar",
    "desembalares": "desembalar",
    "desembalaron": "desembalar",
    "desembalará": "desembalar",
    "desembalarán": "desembalar",
    "desembalarás": "desembalar",
    "desembalaré": "desembalar",
    "desembalaréis": "desembalar",
    "desembalaría": "desembalar",
    "desembalaríais": "desembalar",
    "desembalaríamos": "desembalar",
    "desembalarían": "desembalar",
    "desembalarías": "desembalar",
    "desembalas": "desembalar",
    "desembalase": "desembalar",
    "desembalaseis": "desembalar",
    "desembalasen": "desembalar",
    "desembalases": "desembalar",
    "desembalaste": "desembalar",
    "desembalasteis": "desembalar",
    "desembaldosa": "desembaldosar",
    "desembaldosaba": "desembaldosar",
    "desembaldosabais": "desembaldosar",
    "desembaldosaban": "desembaldosar",
    "desembaldosabas": "desembaldosar",
    "desembaldosad": "desembaldosar",
    "desembaldosada": "desembaldosar",
    "desembaldosadas": "desembaldosar",
    "desembaldosado": "desembaldosar",
    "desembaldosados": "desembaldosar",
    "desembaldosamientos": "desembaldosamiento",
    "desembaldosamos": "desembaldosar",
    "desembaldosan": "desembaldosar",
    "desembaldosando": "desembaldosar",
    "desembaldosara": "desembaldosar",
    "desembaldosarais": "desembaldosar",
    "desembaldosaran": "desembaldosar",
    "desembaldosaras": "desembaldosar",
    "desembaldosare": "desembaldosar",
    "desembaldosareis": "desembaldosar",
    "desembaldosaremos": "desembaldosar",
    "desembaldosaren": "desembaldosar",
    "desembaldosares": "desembaldosar",
    "desembaldosaron": "desembaldosar",
    "desembaldosará": "desembaldosar",
    "desembaldosarán": "desembaldosar",
    "desembaldosarás": "desembaldosar",
    "desembaldosaré": "desembaldosar",
    "desembaldosaréis": "desembaldosar",
    "desembaldosaría": "desembaldosar",
    "desembaldosaríais": "desembaldosar",
    "desembaldosaríamos": "desembaldosar",
    "desembaldosarían": "desembaldosar",
    "desembaldosarías": "desembaldosar",
    "desembaldosas": "desembaldosar",
    "desembaldosase": "desembaldosar",
    "desembaldosaseis": "desembaldosar",
    "desembaldosasen": "desembaldosar",
    "desembaldosases": "desembaldosar",
    "desembaldosaste": "desembaldosar",
    "desembaldosasteis": "desembaldosar",
    "desembaldose": "desembaldosar",
    "desembaldosemos": "desembaldosar",
    "desembaldosen": "desembaldosar",
    "desembaldoses": "desembaldosar",
    "desembaldoso": "desembaldosar",
    "desembaldosábamos": "desembaldosar",
    "desembaldosáis": "desembaldosar",
    "desembaldosáramos": "desembaldosar",
    "desembaldosáremos": "desembaldosar",
    "desembaldosásemos": "desembaldosar",
    "desembaldosé": "desembaldosar",
    "desembaldoséis": "desembaldosar",
    "desembaldosó": "desembaldosar",
    "desembale": "desembalar",
    "desembalemos": "desembalar",
    "desembalen": "desembalar",
    "desembales": "desembalar",
    "desemballestamientos": "desemballestamiento",
    "desembalo": "desembalar",
    "desembalábamos": "desembalar",
    "desembaláis": "desembalar",
    "desembaláramos": "desembalar",
    "desembaláremos": "desembalar",
    "desembalásemos": "desembalar",
    "desembalé": "desembalar",
    "desembaléis": "desembalar",
    "desembaló": "desembalar",
    "desembanasta": "desembanastar",
    "desembanastaba": "desembanastar",
    "desembanastabais": "desembanastar",
    "desembanastaban": "desembanastar",
    "desembanastabas": "desembanastar",
    "desembanastad": "desembanastar",
    "desembanastada": "desembanastar",
    "desembanastadas": "desembanastar",
    "desembanastado": "desembanastar",
    "desembanastados": "desembanastar",
    "desembanastamos": "desembanastar",
    "desembanastan": "desembanastar",
    "desembanastando": "desembanastar",
    "desembanastara": "desembanastar",
    "desembanastarais": "desembanastar",
    "desembanastaran": "desembanastar",
    "desembanastaras": "desembanastar",
    "desembanastare": "desembanastar",
    "desembanastareis": "desembanastar",
    "desembanastaremos": "desembanastar",
    "desembanastaren": "desembanastar",
    "desembanastares": "desembanastar",
    "desembanastaron": "desembanastar",
    "desembanastará": "desembanastar",
    "desembanastarán": "desembanastar",
    "desembanastarás": "desembanastar",
    "desembanastaré": "desembanastar",
    "desembanastaréis": "desembanastar",
    "desembanastaría": "desembanastar",
    "desembanastaríais": "desembanastar",
    "desembanastaríamos": "desembanastar",
    "desembanastarían": "desembanastar",
    "desembanastarías": "desembanastar",
    "desembanastas": "desembanastar",
    "desembanastase": "desembanastar",
    "desembanastaseis": "desembanastar",
    "desembanastasen": "desembanastar",
    "desembanastases": "desembanastar",
    "desembanastaste": "desembanastar",
    "desembanastasteis": "desembanastar",
    "desembanaste": "desembanastar",
    "desembanastemos": "desembanastar",
    "desembanasten": "desembanastar",
    "desembanastes": "desembanastar",
    "desembanasto": "desembanastar",
    "desembanastábamos": "desembanastar",
    "desembanastáis": "desembanastar",
    "desembanastáramos": "desembanastar",
    "desembanastáremos": "desembanastar",
    "desembanastásemos": "desembanastar",
    "desembanasté": "desembanastar",
    "desembanastéis": "desembanastar",
    "desembanastó": "desembanastar",
    "desembarace": "desembarazar",
    "desembaracemos": "desembarazar",
    "desembaracen": "desembarazar",
    "desembaraces": "desembarazar",
    "desembaracé": "desembarazar",
    "desembaracéis": "desembarazar",
    "desembaraza": "desembarazar",
    "desembarazaba": "desembarazar",
    "desembarazabais": "desembarazar",
    "desembarazaban": "desembarazar",
    "desembarazabas": "desembarazar",
    "desembarazad": "desembarazar",
    "desembarazada": "desembarazar",
    "desembarazadas": "desembarazar",
    "desembarazado": "desembarazar",
    "desembarazados": "desembarazar",
    "desembarazamientos": "desembarazamiento",
    "desembarazamos": "desembarazar",
    "desembarazan": "desembarazar",
    "desembarazando": "desembarazar",
    "desembarazara": "desembarazar",
    "desembarazarais": "desembarazar",
    "desembarazaran": "desembarazar",
    "desembarazaras": "desembarazar",
    "desembarazare": "desembarazar",
    "desembarazareis": "desembarazar",
    "desembarazaremos": "desembarazar",
    "desembarazaren": "desembarazar",
    "desembarazares": "desembarazar",
    "desembarazaron": "desembarazar",
    "desembarazará": "desembarazar",
    "desembarazarán": "desembarazar",
    "desembarazarás": "desembarazar",
    "desembarazaré": "desembarazar",
    "desembarazaréis": "desembarazar",
    "desembarazaría": "desembarazar",
    "desembarazaríais": "desembarazar",
    "desembarazaríamos": "desembarazar",
    "desembarazarían": "desembarazar",
    "desembarazarías": "desembarazar",
    "desembarazas": "desembarazar",
    "desembarazase": "desembarazar",
    "desembarazaseis": "desembarazar",
    "desembarazasen": "desembarazar",
    "desembarazases": "desembarazar",
    "desembarazaste": "desembarazar",
    "desembarazasteis": "desembarazar",
    "desembarazo": "desembarazar",
    "desembarazos": "desembarazo",
    "desembarazábamos": "desembarazar",
    "desembarazáis": "desembarazar",
    "desembarazáramos": "desembarazar",
    "desembarazáremos": "desembarazar",
    "desembarazásemos": "desembarazar",
    "desembarazó": "desembarazar",
    "desembarca": "desembarcar",
    "desembarcaba": "desembarcar",
    "desembarcabais": "desembarcar",
    "desembarcaban": "desembarcar",
    "desembarcabas": "desembarcar",
    "desembarcad": "desembarcar",
    "desembarcada": "desembarcar",
    "desembarcadas": "desembarcar",
    "desembarcaderos": "desembarcadero",
    "desembarcado": "desembarcar",
    "desembarcados": "desembarcar",
    "desembarcamos": "desembarcar",
    "desembarcan": "desembarcar",
    "desembarcando": "desembarcar",
    "desembarcara": "desembarcar",
    "desembarcarais": "desembarcar",
    "desembarcaran": "desembarcar",
    "desembarcaras": "desembarcar",
    "desembarcare": "desembarcar",
    "desembarcareis": "desembarcar",
    "desembarcaremos": "desembarcar",
    "desembarcaren": "desembarcar",
    "desembarcares": "desembarcar",
    "desembarcaron": "desembarcar",
    "desembarcará": "desembarcar",
    "desembarcarán": "desembarcar",
    "desembarcarás": "desembarcar",
    "desembarcaré": "desembarcar",
    "desembarcaréis": "desembarcar",
    "desembarcaría": "desembarcar",
    "desembarcaríais": "desembarcar",
    "desembarcaríamos": "desembarcar",
    "desembarcarían": "desembarcar",
    "desembarcarías": "desembarcar",
    "desembarcas": "desembarcar",
    "desembarcase": "desembarcar",
    "desembarcaseis": "desembarcar",
    "desembarcasen": "desembarcar",
    "desembarcases": "desembarcar",
    "desembarcaste": "desembarcar",
    "desembarcasteis": "desembarcar",
    "desembarco": "desembarcar",
    "desembarcos": "desembarco",
    "desembarcábamos": "desembarcar",
    "desembarcáis": "desembarcar",
    "desembarcáramos": "desembarcar",
    "desembarcáremos": "desembarcar",
    "desembarcásemos": "desembarcar",
    "desembarcó": "desembarcar",
    "desembarga": "desembargar",
    "desembargaba": "desembargar",
    "desembargabais": "desembargar",
    "desembargaban": "desembargar",
    "desembargabas": "desembargar",
    "desembargad": "desembargar",
    "desembargada": "desembargar",
    "desembargadas": "desembargar",
    "desembargado": "desembargar",
    "desembargados": "desembargar",
    "desembargamos": "desembargar",
    "desembargan": "desembargar",
    "desembargando": "desembargar",
    "desembargara": "desembargar",
    "desembargarais": "desembargar",
    "desembargaran": "desembargar",
    "desembargaras": "desembargar",
    "desembargare": "desembargar",
    "desembargareis": "desembargar",
    "desembargaremos": "desembargar",
    "desembargaren": "desembargar",
    "desembargares": "desembargar",
    "desembargaron": "desembargar",
    "desembargará": "desembargar",
    "desembargarán": "desembargar",
    "desembargarás": "desembargar",
    "desembargaré": "desembargar",
    "desembargaréis": "desembargar",
    "desembargaría": "desembargar",
    "desembargaríais": "desembargar",
    "desembargaríamos": "desembargar",
    "desembargarían": "desembargar",
    "desembargarías": "desembargar",
    "desembargas": "desembargar",
    "desembargase": "desembargar",
    "desembargaseis": "desembargar",
    "desembargasen": "desembargar",
    "desembargases": "desembargar",
    "desembargaste": "desembargar",
    "desembargasteis": "desembargar",
    "desembargo": "desembargar",
    "desembargos": "desembargo",
    "desembargue": "desembargar",
    "desembarguemos": "desembargar",
    "desembarguen": "desembargar",
    "desembargues": "desembargar",
    "desembargué": "desembargar",
    "desembarguéis": "desembargar",
    "desembargábamos": "desembargar",
    "desembargáis": "desembargar",
    "desembargáramos": "desembargar",
    "desembargáremos": "desembargar",
    "desembargásemos": "desembargar",
    "desembargó": "desembargar",
    "desembarque": "desembarcar",
    "desembarquemos": "desembarcar",
    "desembarquen": "desembarcar",
    "desembarques": "desembarque",
    "desembarqué": "desembarcar",
    "desembarquéis": "desembarcar",
    "desembarra": "desembarrar",
    "desembarraba": "desembarrar",
    "desembarrabais": "desembarrar",
    "desembarraban": "desembarrar",
    "desembarrabas": "desembarrar",
    "desembarrad": "desembarrar",
    "desembarrada": "desembarrar",
    "desembarradas": "desembarrar",
    "desembarrado": "desembarrar",
    "desembarrados": "desembarrar",
    "desembarramos": "desembarrar",
    "desembarran": "desembarrar",
    "desembarranca": "desembarrancar",
    "desembarrancaba": "desembarrancar",
    "desembarrancabais": "desembarrancar",
    "desembarrancaban": "desembarrancar",
    "desembarrancabas": "desembarrancar",
    "desembarrancad": "desembarrancar",
    "desembarrancada": "desembarrancar",
    "desembarrancadas": "desembarrancar",
    "desembarrancado": "desembarrancar",
    "desembarrancados": "desembarrancar",
    "desembarrancamientos": "desembarrancamiento",
    "desembarrancamos": "desembarrancar",
    "desembarrancan": "desembarrancar",
    "desembarrancando": "desembarrancar",
    "desembarrancara": "desembarrancar",
    "desembarrancarais": "desembarrancar",
    "desembarrancaran": "desembarrancar",
    "desembarrancaras": "desembarrancar",
    "desembarrancare": "desembarrancar",
    "desembarrancareis": "desembarrancar",
    "desembarrancaremos": "desembarrancar",
    "desembarrancaren": "desembarrancar",
    "desembarrancares": "desembarrancar",
    "desembarrancaron": "desembarrancar",
    "desembarrancará": "desembarrancar",
    "desembarrancarán": "desembarrancar",
    "desembarrancarás": "desembarrancar",
    "desembarrancaré": "desembarrancar",
    "desembarrancaréis": "desembarrancar",
    "desembarrancaría": "desembarrancar",
    "desembarrancaríais": "desembarrancar",
    "desembarrancaríamos": "desembarrancar",
    "desembarrancarían": "desembarrancar",
    "desembarrancarías": "desembarrancar",
    "desembarrancas": "desembarrancar",
    "desembarrancase": "desembarrancar",
    "desembarrancaseis": "desembarrancar",
    "desembarrancasen": "desembarrancar",
    "desembarrancases": "desembarrancar",
    "desembarrancaste": "desembarrancar",
    "desembarrancasteis": "desembarrancar",
    "desembarranco": "desembarrancar",
    "desembarrancábamos": "desembarrancar",
    "desembarrancáis": "desembarrancar",
    "desembarrancáramos": "desembarrancar",
    "desembarrancáremos": "desembarrancar",
    "desembarrancásemos": "desembarrancar",
    "desembarrancó": "desembarrancar",
    "desembarrando": "desembarrar",
    "desembarranque": "desembarrancar",
    "desembarranquemos": "desembarrancar",
    "desembarranquen": "desembarrancar",
    "desembarranques": "desembarrancar",
    "desembarranqué": "desembarrancar",
    "desembarranquéis": "desembarrancar",
    "desembarrara": "desembarrar",
    "desembarrarais": "desembarrar",
    "desembarraran": "desembarrar",
    "desembarraras": "desembarrar",
    "desembarrare": "desembarrar",
    "desembarrareis": "desembarrar",
    "desembarraremos": "desembarrar",
    "desembarraren": "desembarrar",
    "desembarrares": "desembarrar",
    "desembarraron": "desembarrar",
    "desembarrará": "desembarrar",
    "desembarrarán": "desembarrar",
    "desembarrarás": "desembarrar",
    "desembarraré": "desembarrar",
    "desembarraréis": "desembarrar",
    "desembarraría": "desembarrar",
    "desembarraríais": "desembarrar",
    "desembarraríamos": "desembarrar",
    "desembarrarían": "desembarrar",
    "desembarrarías": "desembarrar",
    "desembarras": "desembarrar",
    "desembarrase": "desembarrar",
    "desembarraseis": "desembarrar",
    "desembarrasen": "desembarrar",
    "desembarrases": "desembarrar",
    "desembarraste": "desembarrar",
    "desembarrasteis": "desembarrar",
    "desembarre": "desembarrar",
    "desembarremos": "desembarrar",
    "desembarren": "desembarrar",
    "desembarres": "desembarrar",
    "desembarro": "desembarrar",
    "desembarrábamos": "desembarrar",
    "desembarráis": "desembarrar",
    "desembarráramos": "desembarrar",
    "desembarráremos": "desembarrar",
    "desembarrásemos": "desembarrar",
    "desembarré": "desembarrar",
    "desembarréis": "desembarrar",
    "desembarró": "desembarrar",
    "desembaulaba": "desembaular",
    "desembaulabais": "desembaular",
    "desembaulaban": "desembaular",
    "desembaulabas": "desembaular",
    "desembaulad": "desembaular",
    "desembaulada": "desembaular",
    "desembauladas": "desembaular",
    "desembaulado": "desembaular",
    "desembaulados": "desembaular",
    "desembaulamos": "desembaular",
    "desembaulando": "desembaular",
    "desembaulara": "desembaular",
    "desembaularais": "desembaular",
    "desembaularan": "desembaular",
    "desembaularas": "desembaular",
    "desembaulare": "desembaular",
    "desembaulareis": "desembaular",
    "desembaularemos": "desembaular",
    "desembaularen": "desembaular",
    "desembaulares": "desembaular",
    "desembaularon": "desembaular",
    "desembaulará": "desembaular",
    "desembaularán": "desembaular",
    "desembaularás": "desembaular",
    "desembaularé": "desembaular",
    "desembaularéis": "desembaular",
    "desembaularía": "desembaular",
    "desembaularíais": "desembaular",
    "desembaularíamos": "desembaular",
    "desembaularían": "desembaular",
    "desembaularías": "desembaular",
    "desembaulase": "desembaular",
    "desembaulaseis": "desembaular",
    "desembaulasen": "desembaular",
    "desembaulases": "desembaular",
    "desembaulaste": "desembaular",
    "desembaulasteis": "desembaular",
    "desembaulemos": "desembaular",
    "desembaulábamos": "desembaular",
    "desembauláis": "desembaular",
    "desembauláramos": "desembaular",
    "desembauláremos": "desembaular",
    "desembaulásemos": "desembaular",
    "desembaulé": "desembaular",
    "desembauléis": "desembaular",
    "desembauló": "desembaular",
    "desembaúla": "desembaular",
    "desembaúlan": "desembaular",
    "desembaúlas": "desembaular",
    "desembaúle": "desembaular",
    "desembaúlen": "desembaular",
    "desembaúles": "desembaular",
    "desembaúlo": "desembaular",
    "desembelesa": "desembelesar",
    "desembelesaba": "desembelesar",
    "desembelesabais": "desembelesar",
    "desembelesaban": "desembelesar",
    "desembelesabas": "desembelesar",
    "desembelesad": "desembelesar",
    "desembelesada": "desembelesar",
    "desembelesadas": "desembelesar",
    "desembelesado": "desembelesar",
    "desembelesados": "desembelesar",
    "desembelesamientos": "desembelesamiento",
    "desembelesamos": "desembelesar",
    "desembelesan": "desembelesar",
    "desembelesando": "desembelesar",
    "desembelesara": "desembelesar",
    "desembelesarais": "desembelesar",
    "desembelesaran": "desembelesar",
    "desembelesaras": "desembelesar",
    "desembelesare": "desembelesar",
    "desembelesareis": "desembelesar",
    "desembelesaremos": "desembelesar",
    "desembelesaren": "desembelesar",
    "desembelesares": "desembelesar",
    "desembelesaron": "desembelesar",
    "desembelesará": "desembelesar",
    "desembelesarán": "desembelesar",
    "desembelesarás": "desembelesar",
    "desembelesaré": "desembelesar",
    "desembelesaréis": "desembelesar",
    "desembelesaría": "desembelesar",
    "desembelesaríais": "desembelesar",
    "desembelesaríamos": "desembelesar",
    "desembelesarían": "desembelesar",
    "desembelesarías": "desembelesar",
    "desembelesas": "desembelesar",
    "desembelesase": "desembelesar",
    "desembelesaseis": "desembelesar",
    "desembelesasen": "desembelesar",
    "desembelesases": "desembelesar",
    "desembelesaste": "desembelesar",
    "desembelesasteis": "desembelesar",
    "desembelese": "desembelesar",
    "desembelesemos": "desembelesar",
    "desembelesen": "desembelesar",
    "desembeleses": "desembelesar",
    "desembeleso": "desembelesar",
    "desembelesábamos": "desembelesar",
    "desembelesáis": "desembelesar",
    "desembelesáramos": "desembelesar",
    "desembelesáremos": "desembelesar",
    "desembelesásemos": "desembelesar",
    "desembelesé": "desembelesar",
    "desembeleséis": "desembelesar",
    "desembelesó": "desembelesar",
    "desemboca": "desembocar",
    "desembocaba": "desembocar",
    "desembocabais": "desembocar",
    "desembocaban": "desembocar",
    "desembocabas": "desembocar",
    "desembocad": "desembocar",
    "desembocada": "desembocar",
    "desembocadas": "desembocar",
    "desembocaderos": "desembocadero",
    "desembocado": "desembocar",
    "desembocados": "desembocar",
    "desembocaduras": "desembocadura",
    "desembocamos": "desembocar",
    "desembocan": "desembocar",
    "desembocando": "desembocar",
    "desembocara": "desembocar",
    "desembocarais": "desembocar",
    "desembocaran": "desembocar",
    "desembocaras": "desembocar",
    "desembocare": "desembocar",
    "desembocareis": "desembocar",
    "desembocaremos": "desembocar",
    "desembocaren": "desembocar",
    "desembocares": "desembocar",
    "desembocaron": "desembocar",
    "desembocará": "desembocar",
    "desembocarán": "desembocar",
    "desembocarás": "desembocar",
    "desembocaré": "desembocar",
    "desembocaréis": "desembocar",
    "desembocaría": "desembocar",
    "desembocaríais": "desembocar",
    "desembocaríamos": "desembocar",
    "desembocarían": "desembocar",
    "desembocarías": "desembocar",
    "desembocas": "desembocar",
    "desembocase": "desembocar",
    "desembocaseis": "desembocar",
    "desembocasen": "desembocar",
    "desembocases": "desembocar",
    "desembocaste": "desembocar",
    "desembocasteis": "desembocar",
    "desemboce": "desembozar",
    "desembocemos": "desembozar",
    "desembocen": "desembozar",
    "desemboces": "desembozar",
    "desemboco": "desembocar",
    "desembocábamos": "desembocar",
    "desembocáis": "desembocar",
    "desembocáramos": "desembocar",
    "desembocáremos": "desembocar",
    "desembocásemos": "desembocar",
    "desembocé": "desembozar",
    "desembocéis": "desembozar",
    "desembocó": "desembocar",
    "desemboja": "desembojar",
    "desembojaba": "desembojar",
    "desembojabais": "desembojar",
    "desembojaban": "desembojar",
    "desembojabas": "desembojar",
    "desembojad": "desembojar",
    "desembojada": "desembojar",
    "desembojadas": "desembojar",
    "desembojado": "desembojar",
    "desembojados": "desembojar",
    "desembojamos": "desembojar",
    "desembojan": "desembojar",
    "desembojando": "desembojar",
    "desembojara": "desembojar",
    "desembojarais": "desembojar",
    "desembojaran": "desembojar",
    "desembojaras": "desembojar",
    "desembojare": "desembojar",
    "desembojareis": "desembojar",
    "desembojaremos": "desembojar",
    "desembojaren": "desembojar",
    "desembojares": "desembojar",
    "desembojaron": "desembojar",
    "desembojará": "desembojar",
    "desembojarán": "desembojar",
    "desembojarás": "desembojar",
    "desembojaré": "desembojar",
    "desembojaréis": "desembojar",
    "desembojaría": "desembojar",
    "desembojaríais": "desembojar",
    "desembojaríamos": "desembojar",
    "desembojarían": "desembojar",
    "desembojarías": "desembojar",
    "desembojas": "desembojar",
    "desembojase": "desembojar",
    "desembojaseis": "desembojar",
    "desembojasen": "desembojar",
    "desembojases": "desembojar",
    "desembojaste": "desembojar",
    "desembojasteis": "desembojar",
    "desemboje": "desembojar",
    "desembojemos": "desembojar",
    "desembojen": "desembojar",
    "desembojes": "desembojar",
    "desembojo": "desembojar",
    "desembojábamos": "desembojar",
    "desembojáis": "desembojar",
    "desembojáramos": "desembojar",
    "desembojáremos": "desembojar",
    "desembojásemos": "desembojar",
    "desembojé": "desembojar",
    "desembojéis": "desembojar",
    "desembojó": "desembojar",
    "desembolsa": "desembolsar",
    "desembolsaba": "desembolsar",
    "desembolsabais": "desembolsar",
    "desembolsaban": "desembolsar",
    "desembolsabas": "desembolsar",
    "desembolsad": "desembolsar",
    "desembolsada": "desembolsar",
    "desembolsadas": "desembolsar",
    "desembolsado": "desembolsar",
    "desembolsados": "desembolsar",
    "desembolsamos": "desembolsar",
    "desembolsan": "desembolsar",
    "desembolsando": "desembolsar",
    "desembolsara": "desembolsar",
    "desembolsarais": "desembolsar",
    "desembolsaran": "desembolsar",
    "desembolsaras": "desembolsar",
    "desembolsare": "desembolsar",
    "desembolsareis": "desembolsar",
    "desembolsaremos": "desembolsar",
    "desembolsaren": "desembolsar",
    "desembolsares": "desembolsar",
    "desembolsaron": "desembolsar",
    "desembolsará": "desembolsar",
    "desembolsarán": "desembolsar",
    "desembolsarás": "desembolsar",
    "desembolsaré": "desembolsar",
    "desembolsaréis": "desembolsar",
    "desembolsaría": "desembolsar",
    "desembolsaríais": "desembolsar",
    "desembolsaríamos": "desembolsar",
    "desembolsarían": "desembolsar",
    "desembolsarías": "desembolsar",
    "desembolsas": "desembolsar",
    "desembolsase": "desembolsar",
    "desembolsaseis": "desembolsar",
    "desembolsasen": "desembolsar",
    "desembolsases": "desembolsar",
    "desembolsaste": "desembolsar",
    "desembolsasteis": "desembolsar",
    "desembolse": "desembolsar",
    "desembolsemos": "desembolsar",
    "desembolsen": "desembolsar",
    "desembolses": "desembolsar",
    "desembolso": "desembolsar",
    "desembolsos": "desembolso",
    "desembolsábamos": "desembolsar",
    "desembolsáis": "desembolsar",
    "desembolsáramos": "desembolsar",
    "desembolsáremos": "desembolsar",
    "desembolsásemos": "desembolsar",
    "desembolsé": "desembolsar",
    "desembolséis": "desembolsar",
    "desembolsó": "desembolsar",
    "desemboque": "desembocar",
    "desemboquemos": "desembocar",
    "desemboquen": "desembocar",
    "desemboques": "desembocar",
    "desemboqué": "desembocar",
    "desemboquéis": "desembocar",
    "desemborracha": "desemborrachar",
    "desemborrachaba": "desemborrachar",
    "desemborrachabais": "desemborrachar",
    "desemborrachaban": "desemborrachar",
    "desemborrachabas": "desemborrachar",
    "desemborrachad": "desemborrachar",
    "desemborrachada": "desemborrachar",
    "desemborrachadas": "desemborrachar",
    "desemborrachado": "desemborrachar",
    "desemborrachados": "desemborrachar",
    "desemborrachamos": "desemborrachar",
    "desemborrachan": "desemborrachar",
    "desemborrachando": "desemborrachar",
    "desemborrachara": "desemborrachar",
    "desemborracharais": "desemborrachar",
    "desemborracharan": "desemborrachar",
    "desemborracharas": "desemborrachar",
    "desemborrachare": "desemborrachar",
    "desemborrachareis": "desemborrachar",
    "desemborracharemos": "desemborrachar",
    "desemborracharen": "desemborrachar",
    "desemborrachares": "desemborrachar",
    "desemborracharon": "desemborrachar",
    "desemborrachará": "desemborrachar",
    "desemborracharán": "desemborrachar",
    "desemborracharás": "desemborrachar",
    "desemborracharé": "desemborrachar",
    "desemborracharéis": "desemborrachar",
    "desemborracharía": "desemborrachar",
    "desemborracharíais": "desemborrachar",
    "desemborracharíamos": "desemborrachar",
    "desemborracharían": "desemborrachar",
    "desemborracharías": "desemborrachar",
    "desemborrachas": "desemborrachar",
    "desemborrachase": "desemborrachar",
    "desemborrachaseis": "desemborrachar",
    "desemborrachasen": "desemborrachar",
    "desemborrachases": "desemborrachar",
    "desemborrachaste": "desemborrachar",
    "desemborrachasteis": "desemborrachar",
    "desemborrache": "desemborrachar",
    "desemborrachemos": "desemborrachar",
    "desemborrachen": "desemborrachar",
    "desemborraches": "desemborrachar",
    "desemborracho": "desemborrachar",
    "desemborrachábamos": "desemborrachar",
    "desemborracháis": "desemborrachar",
    "desemborracháramos": "desemborrachar",
    "desemborracháremos": "desemborrachar",
    "desemborrachásemos": "desemborrachar",
    "desemborraché": "desemborrachar",
    "desemborrachéis": "desemborrachar",
    "desemborrachó": "desemborrachar",
    "desembota": "desembotar",
    "desembotaba": "desembotar",
    "desembotabais": "desembotar",
    "desembotaban": "desembotar",
    "desembotabas": "desembotar",
    "desembotad": "desembotar",
    "desembotada": "desembotar",
    "desembotadas": "desembotar",
    "desembotado": "desembotar",
    "desembotados": "desembotar",
    "desembotamientos": "desembotamiento",
    "desembotamos": "desembotar",
    "desembotan": "desembotar",
    "desembotando": "desembotar",
    "desembotara": "desembotar",
    "desembotarais": "desembotar",
    "desembotaran": "desembotar",
    "desembotaras": "desembotar",
    "desembotare": "desembotar",
    "desembotareis": "desembotar",
    "desembotaremos": "desembotar",
    "desembotaren": "desembotar",
    "desembotares": "desembotar",
    "desembotaron": "desembotar",
    "desembotará": "desembotar",
    "desembotarán": "desembotar",
    "desembotarás": "desembotar",
    "desembotaré": "desembotar",
    "desembotaréis": "desembotar",
    "desembotaría": "desembotar",
    "desembotaríais": "desembotar",
    "desembotaríamos": "desembotar",
    "desembotarían": "desembotar",
    "desembotarías": "desembotar",
    "desembotas": "desembotar",
    "desembotase": "desembotar",
    "desembotaseis": "desembotar",
    "desembotasen": "desembotar",
    "desembotases": "desembotar",
    "desembotaste": "desembotar",
    "desembotasteis": "desembotar",
    "desembote": "desembotar",
    "desembotemos": "desembotar",
    "desemboten": "desembotar",
    "desembotes": "desembotar",
    "desemboto": "desembotar",
    "desembotábamos": "desembotar",
    "desembotáis": "desembotar",
    "desembotáramos": "desembotar",
    "desembotáremos": "desembotar",
    "desembotásemos": "desembotar",
    "desemboté": "desembotar",
    "desembotéis": "desembotar",
    "desembotó": "desembotar",
    "desemboza": "desembozar",
    "desembozaba": "desembozar",
    "desembozabais": "desembozar",
    "desembozaban": "desembozar",
    "desembozabas": "desembozar",
    "desembozad": "desembozar",
    "desembozada": "desembozar",
    "desembozadas": "desembozar",
    "desembozado": "desembozar",
    "desembozados": "desembozar",
    "desembozamos": "desembozar",
    "desembozan": "desembozar",
    "desembozando": "desembozar",
    "desembozara": "desembozar",
    "desembozarais": "desembozar",
    "desembozaran": "desembozar",
    "desembozaras": "desembozar",
    "desembozare": "desembozar",
    "desembozareis": "desembozar",
    "desembozaremos": "desembozar",
    "desembozaren": "desembozar",
    "desembozares": "desembozar",
    "desembozaron": "desembozar",
    "desembozará": "desembozar",
    "desembozarán": "desembozar",
    "desembozarás": "desembozar",
    "desembozaré": "desembozar",
    "desembozaréis": "desembozar",
    "desembozaría": "desembozar",
    "desembozaríais": "desembozar",
    "desembozaríamos": "desembozar",
    "desembozarían": "desembozar",
    "desembozarías": "desembozar",
    "desembozas": "desembozar",
    "desembozase": "desembozar",
    "desembozaseis": "desembozar",
    "desembozasen": "desembozar",
    "desembozases": "desembozar",
    "desembozaste": "desembozar",
    "desembozasteis": "desembozar",
    "desembozo": "desembozar",
    "desembozos": "desembozo",
    "desembozábamos": "desembozar",
    "desembozáis": "desembozar",
    "desembozáramos": "desembozar",
    "desembozáremos": "desembozar",
    "desembozásemos": "desembozar",
    "desembozó": "desembozar",
    "desembraga": "desembragar",
    "desembragaba": "desembragar",
    "desembragabais": "desembragar",
    "desembragaban": "desembragar",
    "desembragabas": "desembragar",
    "desembragables": "desembragable",
    "desembragad": "desembragar",
    "desembragada": "desembragar",
    "desembragadas": "desembragar",
    "desembragado": "desembragar",
    "desembragados": "desembragar",
    "desembragamos": "desembragar",
    "desembragan": "desembragar",
    "desembragando": "desembragar",
    "desembragara": "desembragar",
    "desembragarais": "desembragar",
    "desembragaran": "desembragar",
    "desembragaras": "desembragar",
    "desembragare": "desembragar",
    "desembragareis": "desembragar",
    "desembragaremos": "desembragar",
    "desembragaren": "desembragar",
    "desembragares": "desembragar",
    "desembragaron": "desembragar",
    "desembragará": "desembragar",
    "desembragarán": "desembragar",
    "desembragarás": "desembragar",
    "desembragaré": "desembragar",
    "desembragaréis": "desembragar",
    "desembragaría": "desembragar",
    "desembragaríais": "desembragar",
    "desembragaríamos": "desembragar",
    "desembragarían": "desembragar",
    "desembragarías": "desembragar",
    "desembragas": "desembragar",
    "desembragase": "desembragar",
    "desembragaseis": "desembragar",
    "desembragasen": "desembragar",
    "desembragases": "desembragar",
    "desembragaste": "desembragar",
    "desembragasteis": "desembragar",
    "desembrago": "desembragar",
    "desembrague": "desembragar",
    "desembraguemos": "desembragar",
    "desembraguen": "desembragar",
    "desembragues": "desembrague",
    "desembragué": "desembragar",
    "desembraguéis": "desembragar",
    "desembragábamos": "desembragar",
    "desembragáis": "desembragar",
    "desembragáramos": "desembragar",
    "desembragáremos": "desembragar",
    "desembragásemos": "desembragar",
    "desembragó": "desembragar",
    "desembravece": "desembravecer",
    "desembraveced": "desembravecer",
    "desembravecemos": "desembravecer",
    "desembravecen": "desembravecer",
    "desembraveceremos": "desembravecer",
    "desembravecerá": "desembravecer",
    "desembravecerán": "desembravecer",
    "desembravecerás": "desembravecer",
    "desembraveceré": "desembravecer",
    "desembraveceréis": "desembravecer",
    "desembravecería": "desembravecer",
    "desembraveceríais": "desembravecer",
    "desembraveceríamos": "desembravecer",
    "desembravecerían": "desembravecer",
    "desembravecerías": "desembravecer",
    "desembraveces": "desembravecer",
    "desembravecida": "desembravecer",
    "desembravecidas": "desembravecer",
    "desembravecido": "desembravecer",
    "desembravecidos": "desembravecer",
    "desembraveciendo": "desembravecer",
    "desembraveciera": "desembravecer",
    "desembravecierais": "desembravecer",
    "desembravecieran": "desembravecer",
    "desembravecieras": "desembravecer",
    "desembraveciere": "desembravecer",
    "desembraveciereis": "desembravecer",
    "desembravecieren": "desembravecer",
    "desembravecieres": "desembravecer",
    "desembravecieron": "desembravecer",
    "desembraveciese": "desembravecer",
    "desembravecieseis": "desembravecer",
    "desembraveciesen": "desembravecer",
    "desembravecieses": "desembravecer",
    "desembravecimientos": "desembravecimiento",
    "desembravecimos": "desembravecer",
    "desembraveciste": "desembravecer",
    "desembravecisteis": "desembravecer",
    "desembraveciéramos": "desembravecer",
    "desembraveciéremos": "desembravecer",
    "desembraveciésemos": "desembravecer",
    "desembraveció": "desembravecer",
    "desembravecéis": "desembravecer",
    "desembravecí": "desembravecer",
    "desembravecía": "desembravecer",
    "desembravecíais": "desembravecer",
    "desembravecíamos": "desembravecer",
    "desembravecían": "desembravecer",
    "desembravecías": "desembravecer",
    "desembravezca": "desembravecer",
    "desembravezcamos": "desembravecer",
    "desembravezcan": "desembravecer",
    "desembravezcas": "desembravecer",
    "desembravezco": "desembravecer",
    "desembravezcáis": "desembravecer",
    "desembriaga": "desembriagar",
    "desembriagaba": "desembriagar",
    "desembriagabais": "desembriagar",
    "desembriagaban": "desembriagar",
    "desembriagabas": "desembriagar",
    "desembriagad": "desembriagar",
    "desembriagada": "desembriagar",
    "desembriagadas": "desembriagar",
    "desembriagado": "desembriagar",
    "desembriagados": "desembriagar",
    "desembriagamos": "desembriagar",
    "desembriagan": "desembriagar",
    "desembriagando": "desembriagar",
    "desembriagara": "desembriagar",
    "desembriagarais": "desembriagar",
    "desembriagaran": "desembriagar",
    "desembriagaras": "desembriagar",
    "desembriagare": "desembriagar",
    "desembriagareis": "desembriagar",
    "desembriagaremos": "desembriagar",
    "desembriagaren": "desembriagar",
    "desembriagares": "desembriagar",
    "desembriagaron": "desembriagar",
    "desembriagará": "desembriagar",
    "desembriagarán": "desembriagar",
    "desembriagarás": "desembriagar",
    "desembriagaré": "desembriagar",
    "desembriagaréis": "desembriagar",
    "desembriagaría": "desembriagar",
    "desembriagaríais": "desembriagar",
    "desembriagaríamos": "desembriagar",
    "desembriagarían": "desembriagar",
    "desembriagarías": "desembriagar",
    "desembriagas": "desembriagar",
    "desembriagase": "desembriagar",
    "desembriagaseis": "desembriagar",
    "desembriagasen": "desembriagar",
    "desembriagases": "desembriagar",
    "desembriagaste": "desembriagar",
    "desembriagasteis": "desembriagar",
    "desembriago": "desembriagar",
    "desembriague": "desembriagar",
    "desembriaguemos": "desembriagar",
    "desembriaguen": "desembriagar",
    "desembriagues": "desembriagar",
    "desembriagué": "desembriagar",
    "desembriaguéis": "desembriagar",
    "desembriagábamos": "desembriagar",
    "desembriagáis": "desembriagar",
    "desembriagáramos": "desembriagar",
    "desembriagáremos": "desembriagar",
    "desembriagásemos": "desembriagar",
    "desembriagó": "desembriagar",
    "desembrida": "desembridar",
    "desembridaba": "desembridar",
    "desembridabais": "desembridar",
    "desembridaban": "desembridar",
    "desembridabas": "desembridar",
    "desembridad": "desembridar",
    "desembridada": "desembridar",
    "desembridadas": "desembridar",
    "desembridado": "desembridar",
    "desembridados": "desembridar",
    "desembridamos": "desembridar",
    "desembridan": "desembridar",
    "desembridando": "desembridar",
    "desembridara": "desembridar",
    "desembridarais": "desembridar",
    "desembridaran": "desembridar",
    "desembridaras": "desembridar",
    "desembridare": "desembridar",
    "desembridareis": "desembridar",
    "desembridaremos": "desembridar",
    "desembridaren": "desembridar",
    "desembridares": "desembridar",
    "desembridaron": "desembridar",
    "desembridará": "desembridar",
    "desembridarán": "desembridar",
    "desembridarás": "desembridar",
    "desembridaré": "desembridar",
    "desembridaréis": "desembridar",
    "desembridaría": "desembridar",
    "desembridaríais": "desembridar",
    "desembridaríamos": "desembridar",
    "desembridarían": "desembridar",
    "desembridarías": "desembridar",
    "desembridas": "desembridar",
    "desembridase": "desembridar",
    "desembridaseis": "desembridar",
    "desembridasen": "desembridar",
    "desembridases": "desembridar",
    "desembridaste": "desembridar",
    "desembridasteis": "desembridar",
    "desembride": "desembridar",
    "desembridemos": "desembridar",
    "desembriden": "desembridar",
    "desembrides": "desembridar",
    "desembrido": "desembridar",
    "desembridábamos": "desembridar",
    "desembridáis": "desembridar",
    "desembridáramos": "desembridar",
    "desembridáremos": "desembridar",
    "desembridásemos": "desembridar",
    "desembridé": "desembridar",
    "desembridéis": "desembridar",
    "desembridó": "desembridar",
    "desembroce": "desembrozar",
    "desembrocemos": "desembrozar",
    "desembrocen": "desembrozar",
    "desembroces": "desembrozar",
    "desembrocé": "desembrozar",
    "desembrocéis": "desembrozar",
    "desembrolla": "desembrollar",
    "desembrollaba": "desembrollar",
    "desembrollabais": "desembrollar",
    "desembrollaban": "desembrollar",
    "desembrollabas": "desembrollar",
    "desembrollad": "desembrollar",
    "desembrollada": "desembrollar",
    "desembrolladas": "desembrollar",
    "desembrollado": "desembrollar",
    "desembrollados": "desembrollar",
    "desembrollamos": "desembrollar",
    "desembrollan": "desembrollar",
    "desembrollando": "desembrollar",
    "desembrollara": "desembrollar",
    "desembrollarais": "desembrollar",
    "desembrollaran": "desembrollar",
    "desembrollaras": "desembrollar",
    "desembrollare": "desembrollar",
    "desembrollareis": "desembrollar",
    "desembrollaremos": "desembrollar",
    "desembrollaren": "desembrollar",
    "desembrollares": "desembrollar",
    "desembrollaron": "desembrollar",
    "desembrollará": "desembrollar",
    "desembrollarán": "desembrollar",
    "desembrollarás": "desembrollar",
    "desembrollaré": "desembrollar",
    "desembrollaréis": "desembrollar",
    "desembrollaría": "desembrollar",
    "desembrollaríais": "desembrollar",
    "desembrollaríamos": "desembrollar",
    "desembrollarían": "desembrollar",
    "desembrollarías": "desembrollar",
    "desembrollas": "desembrollar",
    "desembrollase": "desembrollar",
    "desembrollaseis": "desembrollar",
    "desembrollasen": "desembrollar",
    "desembrollases": "desembrollar",
    "desembrollaste": "desembrollar",
    "desembrollasteis": "desembrollar",
    "desembrolle": "desembrollar",
    "desembrollemos": "desembrollar",
    "desembrollen": "desembrollar",
    "desembrolles": "desembrollar",
    "desembrollo": "desembrollar",
    "desembrollábamos": "desembrollar",
    "desembrolláis": "desembrollar",
    "desembrolláramos": "desembrollar",
    "desembrolláremos": "desembrollar",
    "desembrollásemos": "desembrollar",
    "desembrollé": "desembrollar",
    "desembrolléis": "desembrollar",
    "desembrolló": "desembrollar",
    "desembroza": "desembrozar",
    "desembrozaba": "desembrozar",
    "desembrozabais": "desembrozar",
    "desembrozaban": "desembrozar",
    "desembrozabas": "desembrozar",
    "desembrozad": "desembrozar",
    "desembrozada": "desembrozar",
    "desembrozadas": "desembrozar",
    "desembrozado": "desembrozar",
    "desembrozados": "desembrozar",
    "desembrozamos": "desembrozar",
    "desembrozan": "desembrozar",
    "desembrozando": "desembrozar",
    "desembrozara": "desembrozar",
    "desembrozarais": "desembrozar",
    "desembrozaran": "desembrozar",
    "desembrozaras": "desembrozar",
    "desembrozare": "desembrozar",
    "desembrozareis": "desembrozar",
    "desembrozaremos": "desembrozar",
    "desembrozaren": "desembrozar",
    "desembrozares": "desembrozar",
    "desembrozaron": "desembrozar",
    "desembrozará": "desembrozar",
    "desembrozarán": "desembrozar",
    "desembrozarás": "desembrozar",
    "desembrozaré": "desembrozar",
    "desembrozaréis": "desembrozar",
    "desembrozaría": "desembrozar",
    "desembrozaríais": "desembrozar",
    "desembrozaríamos": "desembrozar",
    "desembrozarían": "desembrozar",
    "desembrozarías": "desembrozar",
    "desembrozas": "desembrozar",
    "desembrozase": "desembrozar",
    "desembrozaseis": "desembrozar",
    "desembrozasen": "desembrozar",
    "desembrozases": "desembrozar",
    "desembrozaste": "desembrozar",
    "desembrozasteis": "desembrozar",
    "desembrozo": "desembrozar",
    "desembrozábamos": "desembrozar",
    "desembrozáis": "desembrozar",
    "desembrozáramos": "desembrozar",
    "desembrozáremos": "desembrozar",
    "desembrozásemos": "desembrozar",
    "desembrozó": "desembrozar",
    "desembruja": "desembrujar",
    "desembrujaba": "desembrujar",
    "desembrujabais": "desembrujar",
    "desembrujaban": "desembrujar",
    "desembrujabas": "desembrujar",
    "desembrujad": "desembrujar",
    "desembrujada": "desembrujar",
    "desembrujadas": "desembrujar",
    "desembrujado": "desembrujar",
    "desembrujados": "desembrujar",
    "desembrujamos": "desembrujar",
    "desembrujan": "desembrujar",
    "desembrujando": "desembrujar",
    "desembrujara": "desembrujar",
    "desembrujarais": "desembrujar",
    "desembrujaran": "desembrujar",
    "desembrujaras": "desembrujar",
    "desembrujare": "desembrujar",
    "desembrujareis": "desembrujar",
    "desembrujaremos": "desembrujar",
    "desembrujaren": "desembrujar",
    "desembrujares": "desembrujar",
    "desembrujaron": "desembrujar",
    "desembrujará": "desembrujar",
    "desembrujarán": "desembrujar",
    "desembrujarás": "desembrujar",
    "desembrujaré": "desembrujar",
    "desembrujaréis": "desembrujar",
    "desembrujaría": "desembrujar",
    "desembrujaríais": "desembrujar",
    "desembrujaríamos": "desembrujar",
    "desembrujarían": "desembrujar",
    "desembrujarías": "desembrujar",
    "desembrujas": "desembrujar",
    "desembrujase": "desembrujar",
    "desembrujaseis": "desembrujar",
    "desembrujasen": "desembrujar",
    "desembrujases": "desembrujar",
    "desembrujaste": "desembrujar",
    "desembrujasteis": "desembrujar",
    "desembruje": "desembrujar",
    "desembrujemos": "desembrujar",
    "desembrujen": "desembrujar",
    "desembrujes": "desembrujar",
    "desembrujo": "desembrujar",
    "desembrujábamos": "desembrujar",
    "desembrujáis": "desembrujar",
    "desembrujáramos": "desembrujar",
    "desembrujáremos": "desembrujar",
    "desembrujásemos": "desembrujar",
    "desembrujé": "desembrujar",
    "desembrujéis": "desembrujar",
    "desembrujó": "desembrujar",
    "desembucha": "desembuchar",
    "desembuchaba": "desembuchar",
    "desembuchabais": "desembuchar",
    "desembuchaban": "desembuchar",
    "desembuchabas": "desembuchar",
    "desembuchad": "desembuchar",
    "desembuchada": "desembuchar",
    "desembuchadas": "desembuchar",
    "desembuchado": "desembuchar",
    "desembuchados": "desembuchar",
    "desembuchamos": "desembuchar",
    "desembuchan": "desembuchar",
    "desembuchando": "desembuchar",
    "desembuchara": "desembuchar",
    "desembucharais": "desembuchar",
    "desembucharan": "desembuchar",
    "desembucharas": "desembuchar",
    "desembuchare": "desembuchar",
    "desembuchareis": "desembuchar",
    "desembucharemos": "desembuchar",
    "desembucharen": "desembuchar",
    "desembuchares": "desembuchar",
    "desembucharon": "desembuchar",
    "desembuchará": "desembuchar",
    "desembucharán": "desembuchar",
    "desembucharás": "desembuchar",
    "desembucharé": "desembuchar",
    "desembucharéis": "desembuchar",
    "desembucharía": "desembuchar",
    "desembucharíais": "desembuchar",
    "desembucharíamos": "desembuchar",
    "desembucharían": "desembuchar",
    "desembucharías": "desembuchar",
    "desembuchas": "desembuchar",
    "desembuchase": "desembuchar",
    "desembuchaseis": "desembuchar",
    "desembuchasen": "desembuchar",
    "desembuchases": "desembuchar",
    "desembuchaste": "desembuchar",
    "desembuchasteis": "desembuchar",
    "desembuche": "desembuchar",
    "desembuchemos": "desembuchar",
    "desembuchen": "desembuchar",
    "desembuches": "desembuchar",
    "desembucho": "desembuchar",
    "desembuchábamos": "desembuchar",
    "desembucháis": "desembuchar",
    "desembucháramos": "desembuchar",
    "desembucháremos": "desembuchar",
    "desembuchásemos": "desembuchar",
    "desembuché": "desembuchar",
    "desembuchéis": "desembuchar",
    "desembuchó": "desembuchar",
    "desemeja": "desemejar",
    "desemejaba": "desemejar",
    "desemejabais": "desemejar",
    "desemejaban": "desemejar",
    "desemejabas": "desemejar",
    "desemejad": "desemejar",
    "desemejada": "desemejar",
    "desemejadas": "desemejar",
    "desemejado": "desemejar",
    "desemejados": "desemejar",
    "desemejamos": "desemejar",
    "desemejan": "desemejar",
    "desemejando": "desemejar",
    "desemejantes": "desemejante",
    "desemejanzas": "desemejanza",
    "desemejara": "desemejar",
    "desemejarais": "desemejar",
    "desemejaran": "desemejar",
    "desemejaras": "desemejar",
    "desemejare": "desemejar",
    "desemejareis": "desemejar",
    "desemejaremos": "desemejar",
    "desemejaren": "desemejar",
    "desemejares": "desemejar",
    "desemejaron": "desemejar",
    "desemejará": "desemejar",
    "desemejarán": "desemejar",
    "desemejarás": "desemejar",
    "desemejaré": "desemejar",
    "desemejaréis": "desemejar",
    "desemejaría": "desemejar",
    "desemejaríais": "desemejar",
    "desemejaríamos": "desemejar",
    "desemejarían": "desemejar",
    "desemejarías": "desemejar",
    "desemejas": "desemejar",
    "desemejase": "desemejar",
    "desemejaseis": "desemejar",
    "desemejasen": "desemejar",
    "desemejases": "desemejar",
    "desemejaste": "desemejar",
    "desemejasteis": "desemejar",
    "desemeje": "desemejar",
    "desemejemos": "desemejar",
    "desemejen": "desemejar",
    "desemejes": "desemejar",
    "desemejo": "desemejar",
    "desemejábamos": "desemejar",
    "desemejáis": "desemejar",
    "desemejáramos": "desemejar",
    "desemejáremos": "desemejar",
    "desemejásemos": "desemejar",
    "desemejé": "desemejar",
    "desemejéis": "desemejar",
    "desemejó": "desemejar",
    "desempaca": "desempacar",
    "desempacaba": "desempacar",
    "desempacabais": "desempacar",
    "desempacaban": "desempacar",
    "desempacabas": "desempacar",
    "desempacad": "desempacar",
    "desempacada": "desempacar",
    "desempacadas": "desempacar",
    "desempacado": "desempacar",
    "desempacados": "desempacar",
    "desempacamos": "desempacar",
    "desempacan": "desempacar",
    "desempacando": "desempacar",
    "desempacara": "desempacar",
    "desempacarais": "desempacar",
    "desempacaran": "desempacar",
    "desempacaras": "desempacar",
    "desempacare": "desempacar",
    "desempacareis": "desempacar",
    "desempacaremos": "desempacar",
    "desempacaren": "desempacar",
    "desempacares": "desempacar",
    "desempacaron": "desempacar",
    "desempacará": "desempacar",
    "desempacarán": "desempacar",
    "desempacarás": "desempacar",
    "desempacaré": "desempacar",
    "desempacaréis": "desempacar",
    "desempacaría": "desempacar",
    "desempacaríais": "desempacar",
    "desempacaríamos": "desempacar",
    "desempacarían": "desempacar",
    "desempacarías": "desempacar",
    "desempacas": "desempacar",
    "desempacase": "desempacar",
    "desempacaseis": "desempacar",
    "desempacasen": "desempacar",
    "desempacases": "desempacar",
    "desempacaste": "desempacar",
    "desempacasteis": "desempacar",
    "desempacha": "desempachar",
    "desempachaba": "desempachar",
    "desempachabais": "desempachar",
    "desempachaban": "desempachar",
    "desempachabas": "desempachar",
    "desempachad": "desempachar",
    "desempachada": "desempachar",
    "desempachadas": "desempachar",
    "desempachado": "desempachar",
    "desempachados": "desempachar",
    "desempachamos": "desempachar",
    "desempachan": "desempachar",
    "desempachando": "desempachar",
    "desempachara": "desempachar",
    "desempacharais": "desempachar",
    "desempacharan": "desempachar",
    "desempacharas": "desempachar",
    "desempachare": "desempachar",
    "desempachareis": "desempachar",
    "desempacharemos": "desempachar",
    "desempacharen": "desempachar",
    "desempachares": "desempachar",
    "desempacharon": "desempachar",
    "desempachará": "desempachar",
    "desempacharán": "desempachar",
    "desempacharás": "desempachar",
    "desempacharé": "desempachar",
    "desempacharéis": "desempachar",
    "desempacharía": "desempachar",
    "desempacharíais": "desempachar",
    "desempacharíamos": "desempachar",
    "desempacharían": "desempachar",
    "desempacharías": "desempachar",
    "desempachas": "desempachar",
    "desempachase": "desempachar",
    "desempachaseis": "desempachar",
    "desempachasen": "desempachar",
    "desempachases": "desempachar",
    "desempachaste": "desempachar",
    "desempachasteis": "desempachar",
    "desempache": "desempachar",
    "desempachemos": "desempachar",
    "desempachen": "desempachar",
    "desempaches": "desempachar",
    "desempacho": "desempachar",
    "desempachos": "desempacho",
    "desempachábamos": "desempachar",
    "desempacháis": "desempachar",
    "desempacháramos": "desempachar",
    "desempacháremos": "desempachar",
    "desempachásemos": "desempachar",
    "desempaché": "desempachar",
    "desempachéis": "desempachar",
    "desempachó": "desempachar",
    "desempaco": "desempacar",
    "desempacábamos": "desempacar",
    "desempacáis": "desempacar",
    "desempacáramos": "desempacar",
    "desempacáremos": "desempacar",
    "desempacásemos": "desempacar",
    "desempacó": "desempacar",
    "desempalaga": "desempalagar",
    "desempalagaba": "desempalagar",
    "desempalagabais": "desempalagar",
    "desempalagaban": "desempalagar",
    "desempalagabas": "desempalagar",
    "desempalagad": "desempalagar",
    "desempalagada": "desempalagar",
    "desempalagadas": "desempalagar",
    "desempalagado": "desempalagar",
    "desempalagados": "desempalagar",
    "desempalagamos": "desempalagar",
    "desempalagan": "desempalagar",
    "desempalagando": "desempalagar",
    "desempalagara": "desempalagar",
    "desempalagarais": "desempalagar",
    "desempalagaran": "desempalagar",
    "desempalagaras": "desempalagar",
    "desempalagare": "desempalagar",
    "desempalagareis": "desempalagar",
    "desempalagaremos": "desempalagar",
    "desempalagaren": "desempalagar",
    "desempalagares": "desempalagar",
    "desempalagaron": "desempalagar",
    "desempalagará": "desempalagar",
    "desempalagarán": "desempalagar",
    "desempalagarás": "desempalagar",
    "desempalagaré": "desempalagar",
    "desempalagaréis": "desempalagar",
    "desempalagaría": "desempalagar",
    "desempalagaríais": "desempalagar",
    "desempalagaríamos": "desempalagar",
    "desempalagarían": "desempalagar",
    "desempalagarías": "desempalagar",
    "desempalagas": "desempalagar",
    "desempalagase": "desempalagar",
    "desempalagaseis": "desempalagar",
    "desempalagasen": "desempalagar",
    "desempalagases": "desempalagar",
    "desempalagaste": "desempalagar",
    "desempalagasteis": "desempalagar",
    "desempalago": "desempalagar",
    "desempalague": "desempalagar",
    "desempalaguemos": "desempalagar",
    "desempalaguen": "desempalagar",
    "desempalagues": "desempalagar",
    "desempalagué": "desempalagar",
    "desempalaguéis": "desempalagar",
    "desempalagábamos": "desempalagar",
    "desempalagáis": "desempalagar",
    "desempalagáramos": "desempalagar",
    "desempalagáremos": "desempalagar",
    "desempalagásemos": "desempalagar",
    "desempalagó": "desempalagar",
    "desempalma": "desempalmar",
    "desempalmaba": "desempalmar",
    "desempalmabais": "desempalmar",
    "desempalmaban": "desempalmar",
    "desempalmabas": "desempalmar",
    "desempalmad": "desempalmar",
    "desempalmada": "desempalmar",
    "desempalmadas": "desempalmar",
    "desempalmado": "desempalmar",
    "desempalmados": "desempalmar",
    "desempalmamos": "desempalmar",
    "desempalman": "desempalmar",
    "desempalmando": "desempalmar",
    "desempalmara": "desempalmar",
    "desempalmarais": "desempalmar",
    "desempalmaran": "desempalmar",
    "desempalmaras": "desempalmar",
    "desempalmare": "desempalmar",
    "desempalmareis": "desempalmar",
    "desempalmaremos": "desempalmar",
    "desempalmaren": "desempalmar",
    "desempalmares": "desempalmar",
    "desempalmaron": "desempalmar",
    "desempalmará": "desempalmar",
    "desempalmarán": "desempalmar",
    "desempalmarás": "desempalmar",
    "desempalmaré": "desempalmar",
    "desempalmaréis": "desempalmar",
    "desempalmaría": "desempalmar",
    "desempalmaríais": "desempalmar",
    "desempalmaríamos": "desempalmar",
    "desempalmarían": "desempalmar",
    "desempalmarías": "desempalmar",
    "desempalmas": "desempalmar",
    "desempalmase": "desempalmar",
    "desempalmaseis": "desempalmar",
    "desempalmasen": "desempalmar",
    "desempalmases": "desempalmar",
    "desempalmaste": "desempalmar",
    "desempalmasteis": "desempalmar",
    "desempalme": "desempalmar",
    "desempalmemos": "desempalmar",
    "desempalmen": "desempalmar",
    "desempalmes": "desempalmar",
    "desempalmo": "desempalmar",
    "desempalmábamos": "desempalmar",
    "desempalmáis": "desempalmar",
    "desempalmáramos": "desempalmar",
    "desempalmáremos": "desempalmar",
    "desempalmásemos": "desempalmar",
    "desempalmé": "desempalmar",
    "desempalméis": "desempalmar",
    "desempalmó": "desempalmar",
    "desempapela": "desempapelar",
    "desempapelaba": "desempapelar",
    "desempapelabais": "desempapelar",
    "desempapelaban": "desempapelar",
    "desempapelabas": "desempapelar",
    "desempapelad": "desempapelar",
    "desempapelada": "desempapelar",
    "desempapeladas": "desempapelar",
    "desempapelado": "desempapelar",
    "desempapelados": "desempapelar",
    "desempapelamos": "desempapelar",
    "desempapelan": "desempapelar",
    "desempapelando": "desempapelar",
    "desempapelara": "desempapelar",
    "desempapelarais": "desempapelar",
    "desempapelaran": "desempapelar",
    "desempapelaras": "desempapelar",
    "desempapelare": "desempapelar",
    "desempapelareis": "desempapelar",
    "desempapelaremos": "desempapelar",
    "desempapelaren": "desempapelar",
    "desempapelares": "desempapelar",
    "desempapelaron": "desempapelar",
    "desempapelará": "desempapelar",
    "desempapelarán": "desempapelar",
    "desempapelarás": "desempapelar",
    "desempapelaré": "desempapelar",
    "desempapelaréis": "desempapelar",
    "desempapelaría": "desempapelar",
    "desempapelaríais": "desempapelar",
    "desempapelaríamos": "desempapelar",
    "desempapelarían": "desempapelar",
    "desempapelarías": "desempapelar",
    "desempapelas": "desempapelar",
    "desempapelase": "desempapelar",
    "desempapelaseis": "desempapelar",
    "desempapelasen": "desempapelar",
    "desempapelases": "desempapelar",
    "desempapelaste": "desempapelar",
    "desempapelasteis": "desempapelar",
    "desempapele": "desempapelar",
    "desempapelemos": "desempapelar",
    "desempapelen": "desempapelar",
    "desempapeles": "desempapelar",
    "desempapelo": "desempapelar",
    "desempapelábamos": "desempapelar",
    "desempapeláis": "desempapelar",
    "desempapeláramos": "desempapelar",
    "desempapeláremos": "desempapelar",
    "desempapelásemos": "desempapelar",
    "desempapelé": "desempapelar",
    "desempapeléis": "desempapelar",
    "desempapeló": "desempapelar",
    "desempaque": "desempacar",
    "desempaquemos": "desempacar",
    "desempaquen": "desempacar",
    "desempaques": "desempaque",
    "desempaqueta": "desempaquetar",
    "desempaquetaba": "desempaquetar",
    "desempaquetabais": "desempaquetar",
    "desempaquetaban": "desempaquetar",
    "desempaquetabas": "desempaquetar",
    "desempaquetad": "desempaquetar",
    "desempaquetada": "desempaquetar",
    "desempaquetadas": "desempaquetar",
    "desempaquetado": "desempaquetar",
    "desempaquetados": "desempaquetar",
    "desempaquetamos": "desempaquetar",
    "desempaquetan": "desempaquetar",
    "desempaquetando": "desempaquetar",
    "desempaquetara": "desempaquetar",
    "desempaquetarais": "desempaquetar",
    "desempaquetaran": "desempaquetar",
    "desempaquetaras": "desempaquetar",
    "desempaquetare": "desempaquetar",
    "desempaquetareis": "desempaquetar",
    "desempaquetaremos": "desempaquetar",
    "desempaquetaren": "desempaquetar",
    "desempaquetares": "desempaquetar",
    "desempaquetaron": "desempaquetar",
    "desempaquetará": "desempaquetar",
    "desempaquetarán": "desempaquetar",
    "desempaquetarás": "desempaquetar",
    "desempaquetaré": "desempaquetar",
    "desempaquetaréis": "desempaquetar",
    "desempaquetaría": "desempaquetar",
    "desempaquetaríais": "desempaquetar",
    "desempaquetaríamos": "desempaquetar",
    "desempaquetarían": "desempaquetar",
    "desempaquetarías": "desempaquetar",
    "desempaquetas": "desempaquetar",
    "desempaquetase": "desempaquetar",
    "desempaquetaseis": "desempaquetar",
    "desempaquetasen": "desempaquetar",
    "desempaquetases": "desempaquetar",
    "desempaquetaste": "desempaquetar",
    "desempaquetasteis": "desempaquetar",
    "desempaquete": "desempaquetar",
    "desempaquetemos": "desempaquetar",
    "desempaqueten": "desempaquetar",
    "desempaquetes": "desempaquetar",
    "desempaqueto": "desempaquetar",
    "desempaquetábamos": "desempaquetar",
    "desempaquetáis": "desempaquetar",
    "desempaquetáramos": "desempaquetar",
    "desempaquetáremos": "desempaquetar",
    "desempaquetásemos": "desempaquetar",
    "desempaqueté": "desempaquetar",
    "desempaquetéis": "desempaquetar",
    "desempaquetó": "desempaquetar",
    "desempaqué": "desempacar",
    "desempaquéis": "desempacar",
    "desempareja": "desemparejar",
    "desemparejaba": "desemparejar",
    "desemparejabais": "desemparejar",
    "desemparejaban": "desemparejar",
    "desemparejabas": "desemparejar",
    "desemparejad": "desemparejar",
    "desemparejada": "desemparejar",
    "desemparejadas": "desemparejar",
    "desemparejado": "desemparejar",
    "desemparejados": "desemparejar",
    "desemparejamos": "desemparejar",
    "desemparejan": "desemparejar",
    "desemparejando": "desemparejar",
    "desemparejara": "desemparejar",
    "desemparejarais": "desemparejar",
    "desemparejaran": "desemparejar",
    "desemparejaras": "desemparejar",
    "desemparejare": "desemparejar",
    "desemparejareis": "desemparejar",
    "desemparejaremos": "desemparejar",
    "desemparejaren": "desemparejar",
    "desemparejares": "desemparejar",
    "desemparejaron": "desemparejar",
    "desemparejará": "desemparejar",
    "desemparejarán": "desemparejar",
    "desemparejarás": "desemparejar",
    "desemparejaré": "desemparejar",
    "desemparejaréis": "desemparejar",
    "desemparejaría": "desemparejar",
    "desemparejaríais": "desemparejar",
    "desemparejaríamos": "desemparejar",
    "desemparejarían": "desemparejar",
    "desemparejarías": "desemparejar",
    "desemparejas": "desemparejar",
    "desemparejase": "desemparejar",
    "desemparejaseis": "desemparejar",
    "desemparejasen": "desemparejar",
    "desemparejases": "desemparejar",
    "desemparejaste": "desemparejar",
    "desemparejasteis": "desemparejar",
    "desempareje": "desemparejar",
    "desemparejemos": "desemparejar",
    "desemparejen": "desemparejar",
    "desemparejes": "desemparejar",
    "desemparejo": "desemparejar",
    "desemparejábamos": "desemparejar",
    "desemparejáis": "desemparejar",
    "desemparejáramos": "desemparejar",
    "desemparejáremos": "desemparejar",
    "desemparejásemos": "desemparejar",
    "desemparejé": "desemparejar",
    "desemparejéis": "desemparejar",
    "desemparejó": "desemparejar",
    "desemparentada": "desemparentado",
    "desemparentadas": "desemparentado",
    "desemparentados": "desemparentado",
    "desemparva": "desemparvar",
    "desemparvaba": "desemparvar",
    "desemparvabais": "desemparvar",
    "desemparvaban": "desemparvar",
    "desemparvabas": "desemparvar",
    "desemparvad": "desemparvar",
    "desemparvada": "desemparvar",
    "desemparvadas": "desemparvar",
    "desemparvado": "desemparvar",
    "desemparvados": "desemparvar",
    "desemparvamos": "desemparvar",
    "desemparvan": "desemparvar",
    "desemparvando": "desemparvar",
    "desemparvara": "desemparvar",
    "desemparvarais": "desemparvar",
    "desemparvaran": "desemparvar",
    "desemparvaras": "desemparvar",
    "desemparvare": "desemparvar",
    "desemparvareis": "desemparvar",
    "desemparvaremos": "desemparvar",
    "desemparvaren": "desemparvar",
    "desemparvares": "desemparvar",
    "desemparvaron": "desemparvar",
    "desemparvará": "desemparvar",
    "desemparvarán": "desemparvar",
    "desemparvarás": "desemparvar",
    "desemparvaré": "desemparvar",
    "desemparvaréis": "desemparvar",
    "desemparvaría": "desemparvar",
    "desemparvaríais": "desemparvar",
    "desemparvaríamos": "desemparvar",
    "desemparvarían": "desemparvar",
    "desemparvarías": "desemparvar",
    "desemparvas": "desemparvar",
    "desemparvase": "desemparvar",
    "desemparvaseis": "desemparvar",
    "desemparvasen": "desemparvar",
    "desemparvases": "desemparvar",
    "desemparvaste": "desemparvar",
    "desemparvasteis": "desemparvar",
    "desemparve": "desemparvar",
    "desemparvemos": "desemparvar",
    "desemparven": "desemparvar",
    "desemparves": "desemparvar",
    "desemparvo": "desemparvar",
    "desemparvábamos": "desemparvar",
    "desemparváis": "desemparvar",
    "desemparváramos": "desemparvar",
    "desemparváremos": "desemparvar",
    "desemparvásemos": "desemparvar",
    "desemparvé": "desemparvar",
    "desemparvéis": "desemparvar",
    "desemparvó": "desemparvar",
    "desempasta": "desempastar",
    "desempastaba": "desempastar",
    "desempastabais": "desempastar",
    "desempastaban": "desempastar",
    "desempastabas": "desempastar",
    "desempastad": "desempastar",
    "desempastada": "desempastar",
    "desempastadas": "desempastar",
    "desempastado": "desempastar",
    "desempastados": "desempastar",
    "desempastamos": "desempastar",
    "desempastan": "desempastar",
    "desempastando": "desempastar",
    "desempastara": "desempastar",
    "desempastarais": "desempastar",
    "desempastaran": "desempastar",
    "desempastaras": "desempastar",
    "desempastare": "desempastar",
    "desempastareis": "desempastar",
    "desempastaremos": "desempastar",
    "desempastaren": "desempastar",
    "desempastares": "desempastar",
    "desempastaron": "desempastar",
    "desempastará": "desempastar",
    "desempastarán": "desempastar",
    "desempastarás": "desempastar",
    "desempastaré": "desempastar",
    "desempastaréis": "desempastar",
    "desempastaría": "desempastar",
    "desempastaríais": "desempastar",
    "desempastaríamos": "desempastar",
    "desempastarían": "desempastar",
    "desempastarías": "desempastar",
    "desempastas": "desempastar",
    "desempastase": "desempastar",
    "desempastaseis": "desempastar",
    "desempastasen": "desempastar",
    "desempastases": "desempastar",
    "desempastaste": "desempastar",
    "desempastasteis": "desempastar",
    "desempaste": "desempastar",
    "desempastemos": "desempastar",
    "desempasten": "desempastar",
    "desempastes": "desempastar",
    "desempasto": "desempastar",
    "desempastábamos": "desempastar",
    "desempastáis": "desempastar",
    "desempastáramos": "desempastar",
    "desempastáremos": "desempastar",
    "desempastásemos": "desempastar",
    "desempasté": "desempastar",
    "desempastéis": "desempastar",
    "desempastó": "desempastar",
    "desempata": "desempatar",
    "desempataba": "desempatar",
    "desempatabais": "desempatar",
    "desempataban": "desempatar",
    "desempatabas": "desempatar",
    "desempatad": "desempatar",
    "desempatada": "desempatar",
    "desempatadas": "desempatar",
    "desempatado": "desempatar",
    "desempatados": "desempatar",
    "desempatamos": "desempatar",
    "desempatan": "desempatar",
    "desempatando": "desempatar",
    "desempatara": "desempatar",
    "desempatarais": "desempatar",
    "desempataran": "desempatar",
    "desempataras": "desempatar",
    "desempatare": "desempatar",
    "desempatareis": "desempatar",
    "desempataremos": "desempatar",
    "desempataren": "desempatar",
    "desempatares": "desempatar",
    "desempataron": "desempatar",
    "desempatará": "desempatar",
    "desempatarán": "desempatar",
    "desempatarás": "desempatar",
    "desempataré": "desempatar",
    "desempataréis": "desempatar",
    "desempataría": "desempatar",
    "desempataríais": "desempatar",
    "desempataríamos": "desempatar",
    "desempatarían": "desempatar",
    "desempatarías": "desempatar",
    "desempatas": "desempatar",
    "desempatase": "desempatar",
    "desempataseis": "desempatar",
    "desempatasen": "desempatar",
    "desempatases": "desempatar",
    "desempataste": "desempatar",
    "desempatasteis": "desempatar",
    "desempate": "desempatar",
    "desempatemos": "desempatar",
    "desempaten": "desempatar",
    "desempates": "desempate",
    "desempato": "desempatar",
    "desempatábamos": "desempatar",
    "desempatáis": "desempatar",
    "desempatáramos": "desempatar",
    "desempatáremos": "desempatar",
    "desempatásemos": "desempatar",
    "desempaté": "desempatar",
    "desempatéis": "desempatar",
    "desempató": "desempatar",
    "desempaña": "desempañar",
    "desempañaba": "desempañar",
    "desempañabais": "desempañar",
    "desempañaban": "desempañar",
    "desempañabas": "desempañar",
    "desempañad": "desempañar",
    "desempañada": "desempañar",
    "desempañadas": "desempañar",
    "desempañado": "desempañar",
    "desempañadores": "desempañador",
    "desempañados": "desempañar",
    "desempañamos": "desempañar",
    "desempañan": "desempañar",
    "desempañando": "desempañar",
    "desempañara": "desempañar",
    "desempañarais": "desempañar",
    "desempañaran": "desempañar",
    "desempañaras": "desempañar",
    "desempañare": "desempañar",
    "desempañareis": "desempañar",
    "desempañaremos": "desempañar",
    "desempañaren": "desempañar",
    "desempañares": "desempañar",
    "desempañaron": "desempañar",
    "desempañará": "desempañar",
    "desempañarán": "desempañar",
    "desempañarás": "desempañar",
    "desempañaré": "desempañar",
    "desempañaréis": "desempañar",
    "desempañaría": "desempañar",
    "desempañaríais": "desempañar",
    "desempañaríamos": "desempañar",
    "desempañarían": "desempañar",
    "desempañarías": "desempañar",
    "desempañas": "desempañar",
    "desempañase": "desempañar",
    "desempañaseis": "desempañar",
    "desempañasen": "desempañar",
    "desempañases": "desempañar",
    "desempañaste": "desempañar",
    "desempañasteis": "desempañar",
    "desempañe": "desempañar",
    "desempañemos": "desempañar",
    "desempañen": "desempañar",
    "desempañes": "desempañar",
    "desempaño": "desempañar",
    "desempañábamos": "desempañar",
    "desempañáis": "desempañar",
    "desempañáramos": "desempañar",
    "desempañáremos": "desempañar",
    "desempañásemos": "desempañar",
    "desempañé": "desempañar",
    "desempañéis": "desempañar",
    "desempañó": "desempañar",
    "desempedraba": "desempedrar",
    "desempedrabais": "desempedrar",
    "desempedraban": "desempedrar",
    "desempedrabas": "desempedrar",
    "desempedrad": "desempedrar",
    "desempedrada": "desempedrar",
    "desempedradas": "desempedrar",
    "desempedrado": "desempedrar",
    "desempedrados": "desempedrar",
    "desempedramientos": "desempedramiento",
    "desempedramos": "desempedrar",
    "desempedrando": "desempedrar",
    "desempedrara": "desempedrar",
    "desempedrarais": "desempedrar",
    "desempedraran": "desempedrar",
    "desempedraras": "desempedrar",
    "desempedrare": "desempedrar",
    "desempedrareis": "desempedrar",
    "desempedraremos": "desempedrar",
    "desempedraren": "desempedrar",
    "desempedrares": "desempedrar",
    "desempedraron": "desempedrar",
    "desempedrará": "desempedrar",
    "desempedrarán": "desempedrar",
    "desempedrarás": "desempedrar",
    "desempedraré": "desempedrar",
    "desempedraréis": "desempedrar",
    "desempedraría": "desempedrar",
    "desempedraríais": "desempedrar",
    "desempedraríamos": "desempedrar",
    "desempedrarían": "desempedrar",
    "desempedrarías": "desempedrar",
    "desempedrase": "desempedrar",
    "desempedraseis": "desempedrar",
    "desempedrasen": "desempedrar",
    "desempedrases": "desempedrar",
    "desempedraste": "desempedrar",
    "desempedrasteis": "desempedrar",
    "desempedremos": "desempedrar",
    "desempedrábamos": "desempedrar",
    "desempedráis": "desempedrar",
    "desempedráramos": "desempedrar",
    "desempedráremos": "desempedrar",
    "desempedrásemos": "desempedrar",
    "desempedré": "desempedrar",
    "desempedréis": "desempedrar",
    "desempedró": "desempedrar",
    "desempega": "desempegar",
    "desempegaba": "desempegar",
    "desempegabais": "desempegar",
    "desempegaban": "desempegar",
    "desempegabas": "desempegar",
    "desempegad": "desempegar",
    "desempegada": "desempegar",
    "desempegadas": "desempegar",
    "desempegado": "desempegar",
    "desempegados": "desempegar",
    "desempegamos": "desempegar",
    "desempegan": "desempegar",
    "desempegando": "desempegar",
    "desempegara": "desempegar",
    "desempegarais": "desempegar",
    "desempegaran": "desempegar",
    "desempegaras": "desempegar",
    "desempegare": "desempegar",
    "desempegareis": "desempegar",
    "desempegaremos": "desempegar",
    "desempegaren": "desempegar",
    "desempegares": "desempegar",
    "desempegaron": "desempegar",
    "desempegará": "desempegar",
    "desempegarán": "desempegar",
    "desempegarás": "desempegar",
    "desempegaré": "desempegar",
    "desempegaréis": "desempegar",
    "desempegaría": "desempegar",
    "desempegaríais": "desempegar",
    "desempegaríamos": "desempegar",
    "desempegarían": "desempegar",
    "desempegarías": "desempegar",
    "desempegas": "desempegar",
    "desempegase": "desempegar",
    "desempegaseis": "desempegar",
    "desempegasen": "desempegar",
    "desempegases": "desempegar",
    "desempegaste": "desempegar",
    "desempegasteis": "desempegar",
    "desempego": "desempegar",
    "desempegue": "desempegar",
    "desempeguemos": "desempegar",
    "desempeguen": "desempegar",
    "desempegues": "desempegar",
    "desempegué": "desempegar",
    "desempeguéis": "desempegar",
    "desempegábamos": "desempegar",
    "desempegáis": "desempegar",
    "desempegáramos": "desempegar",
    "desempegáremos": "desempegar",
    "desempegásemos": "desempegar",
    "desempegó": "desempegar",
    "desemperece": "desemperezar",
    "desemperecemos": "desemperezar",
    "desemperecen": "desemperezar",
    "desempereces": "desemperezar",
    "desemperecé": "desemperezar",
    "desemperecéis": "desemperezar",
    "desempereza": "desemperezar",
    "desemperezaba": "desemperezar",
    "desemperezabais": "desemperezar",
    "desemperezaban": "desemperezar",
    "desemperezabas": "desemperezar",
    "desemperezad": "desemperezar",
    "desemperezada": "desemperezar",
    "desemperezadas": "desemperezar",
    "desemperezado": "desemperezar",
    "desemperezados": "desemperezar",
    "desemperezamos": "desemperezar",
    "desemperezan": "desemperezar",
    "desemperezando": "desemperezar",
    "desemperezara": "desemperezar",
    "desemperezarais": "desemperezar",
    "desemperezaran": "desemperezar",
    "desemperezaras": "desemperezar",
    "desemperezare": "desemperezar",
    "desemperezareis": "desemperezar",
    "desemperezaremos": "desemperezar",
    "desemperezaren": "desemperezar",
    "desemperezares": "desemperezar",
    "desemperezaron": "desemperezar",
    "desemperezará": "desemperezar",
    "desemperezarán": "desemperezar",
    "desemperezarás": "desemperezar",
    "desemperezaré": "desemperezar",
    "desemperezaréis": "desemperezar",
    "desemperezaría": "desemperezar",
    "desemperezaríais": "desemperezar",
    "desemperezaríamos": "desemperezar",
    "desemperezarían": "desemperezar",
    "desemperezarías": "desemperezar",
    "desemperezas": "desemperezar",
    "desemperezase": "desemperezar",
    "desemperezaseis": "desemperezar",
    "desemperezasen": "desemperezar",
    "desemperezases": "desemperezar",
    "desemperezaste": "desemperezar",
    "desemperezasteis": "desemperezar",
    "desemperezo": "desemperezar",
    "desemperezábamos": "desemperezar",
    "desemperezáis": "desemperezar",
    "desemperezáramos": "desemperezar",
    "desemperezáremos": "desemperezar",
    "desemperezásemos": "desemperezar",
    "desemperezó": "desemperezar",
    "desemperna": "desempernar",
    "desempernaba": "desempernar",
    "desempernabais": "desempernar",
    "desempernaban": "desempernar",
    "desempernabas": "desempernar",
    "desempernad": "desempernar",
    "desempernada": "desempernar",
    "desempernadas": "desempernar",
    "desempernado": "desempernar",
    "desempernados": "desempernar",
    "desempernamos": "desempernar",
    "desempernan": "desempernar",
    "desempernando": "desempernar",
    "desempernara": "desempernar",
    "desempernarais": "desempernar",
    "desempernaran": "desempernar",
    "desempernaras": "desempernar",
    "desempernare": "desempernar",
    "desempernareis": "desempernar",
    "desempernaremos": "desempernar",
    "desempernaren": "desempernar",
    "desempernares": "desempernar",
    "desempernaron": "desempernar",
    "desempernará": "desempernar",
    "desempernarán": "desempernar",
    "desempernarás": "desempernar",
    "desempernaré": "desempernar",
    "desempernaréis": "desempernar",
    "desempernaría": "desempernar",
    "desempernaríais": "desempernar",
    "desempernaríamos": "desempernar",
    "desempernarían": "desempernar",
    "desempernarías": "desempernar",
    "desempernas": "desempernar",
    "desempernase": "desempernar",
    "desempernaseis": "desempernar",
    "desempernasen": "desempernar",
    "desempernases": "desempernar",
    "desempernaste": "desempernar",
    "desempernasteis": "desempernar",
    "desemperne": "desempernar",
    "desempernemos": "desempernar",
    "desempernen": "desempernar",
    "desempernes": "desempernar",
    "desemperno": "desempernar",
    "desempernábamos": "desempernar",
    "desempernáis": "desempernar",
    "desempernáramos": "desempernar",
    "desempernáremos": "desempernar",
    "desempernásemos": "desempernar",
    "desemperné": "desempernar",
    "desempernéis": "desempernar",
    "desempernó": "desempernar",
    "desempeña": "desempeñar",
    "desempeñaba": "desempeñar",
    "desempeñabais": "desempeñar",
    "desempeñaban": "desempeñar",
    "desempeñabas": "desempeñar",
    "desempeñad": "desempeñar",
    "desempeñada": "desempeñar",
    "desempeñadas": "desempeñar",
    "desempeñado": "desempeñar",
    "desempeñados": "desempeñar",
    "desempeñamos": "desempeñar",
    "desempeñan": "desempeñar",
    "desempeñando": "desempeñar",
    "desempeñara": "desempeñar",
    "desempeñarais": "desempeñar",
    "desempeñaran": "desempeñar",
    "desempeñaras": "desempeñar",
    "desempeñare": "desempeñar",
    "desempeñareis": "desempeñar",
    "desempeñaremos": "desempeñar",
    "desempeñaren": "desempeñar",
    "desempeñares": "desempeñar",
    "desempeñaron": "desempeñar",
    "desempeñará": "desempeñar",
    "desempeñarán": "desempeñar",
    "desempeñarás": "desempeñar",
    "desempeñaré": "desempeñar",
    "desempeñaréis": "desempeñar",
    "desempeñaría": "desempeñar",
    "desempeñaríais": "desempeñar",
    "desempeñaríamos": "desempeñar",
    "desempeñarían": "desempeñar",
    "desempeñarías": "desempeñar",
    "desempeñas": "desempeñar",
    "desempeñase": "desempeñar",
    "desempeñaseis": "desempeñar",
    "desempeñasen": "desempeñar",
    "desempeñases": "desempeñar",
    "desempeñaste": "desempeñar",
    "desempeñasteis": "desempeñar",
    "desempeñe": "desempeñar",
    "desempeñemos": "desempeñar",
    "desempeñen": "desempeñar",
    "desempeñes": "desempeñar",
    "desempeño": "desempeñar",
    "desempeños": "desempeño",
    "desempeñábamos": "desempeñar",
    "desempeñáis": "desempeñar",
    "desempeñáramos": "desempeñar",
    "desempeñáremos": "desempeñar",
    "desempeñásemos": "desempeñar",
    "desempeñé": "desempeñar",
    "desempeñéis": "desempeñar",
    "desempeñó": "desempeñar",
    "desempiedra": "desempedrar",
    "desempiedran": "desempedrar",
    "desempiedras": "desempedrar",
    "desempiedre": "desempedrar",
    "desempiedren": "desempedrar",
    "desempiedres": "desempedrar",
    "desempiedro": "desempedrar",
    "desempleada": "desempleado",
    "desempleadas": "desempleado",
    "desempleados": "desempleado",
    "desempleos": "desempleo",
    "desempoce": "desempozar",
    "desempocemos": "desempozar",
    "desempocen": "desempozar",
    "desempoces": "desempozar",
    "desempocé": "desempozar",
    "desempocéis": "desempozar",
    "desempolva": "desempolvar",
    "desempolvaba": "desempolvar",
    "desempolvabais": "desempolvar",
    "desempolvaban": "desempolvar",
    "desempolvabas": "desempolvar",
    "desempolvad": "desempolvar",
    "desempolvada": "desempolvar",
    "desempolvadas": "desempolvar",
    "desempolvado": "desempolvar",
    "desempolvadores": "desempolvador",
    "desempolvados": "desempolvar",
    "desempolvaduras": "desempolvadura",
    "desempolvamos": "desempolvar",
    "desempolvan": "desempolvar",
    "desempolvando": "desempolvar",
    "desempolvara": "desempolvar",
    "desempolvarais": "desempolvar",
    "desempolvaran": "desempolvar",
    "desempolvaras": "desempolvar",
    "desempolvare": "desempolvar",
    "desempolvareis": "desempolvar",
    "desempolvaremos": "desempolvar",
    "desempolvaren": "desempolvar",
    "desempolvares": "desempolvar",
    "desempolvaron": "desempolvar",
    "desempolvará": "desempolvar",
    "desempolvarán": "desempolvar",
    "desempolvarás": "desempolvar",
    "desempolvaré": "desempolvar",
    "desempolvaréis": "desempolvar",
    "desempolvaría": "desempolvar",
    "desempolvaríais": "desempolvar",
    "desempolvaríamos": "desempolvar",
    "desempolvarían": "desempolvar",
    "desempolvarías": "desempolvar",
    "desempolvas": "desempolvar",
    "desempolvase": "desempolvar",
    "desempolvaseis": "desempolvar",
    "desempolvasen": "desempolvar",
    "desempolvases": "desempolvar",
    "desempolvaste": "desempolvar",
    "desempolvasteis": "desempolvar",
    "desempolve": "desempolvar",
    "desempolvemos": "desempolvar",
    "desempolven": "desempolvar",
    "desempolves": "desempolvar",
    "desempolvo": "desempolvar",
    "desempolvábamos": "desempolvar",
    "desempolváis": "desempolvar",
    "desempolváramos": "desempolvar",
    "desempolváremos": "desempolvar",
    "desempolvásemos": "desempolvar",
    "desempolvé": "desempolvar",
    "desempolvéis": "desempolvar",
    "desempolvó": "desempolvar",
    "desemponzoña": "desemponzoñar",
    "desemponzoñaba": "desemponzoñar",
    "desemponzoñabais": "desemponzoñar",
    "desemponzoñaban": "desemponzoñar",
    "desemponzoñabas": "desemponzoñar",
    "desemponzoñad": "desemponzoñar",
    "desemponzoñada": "desemponzoñar",
    "desemponzoñadas": "desemponzoñar",
    "desemponzoñado": "desemponzoñar",
    "desemponzoñados": "desemponzoñar",
    "desemponzoñamientos": "desemponzoñamiento",
    "desemponzoñamos": "desemponzoñar",
    "desemponzoñan": "desemponzoñar",
    "desemponzoñando": "desemponzoñar",
    "desemponzoñara": "desemponzoñar",
    "desemponzoñarais": "desemponzoñar",
    "desemponzoñaran": "desemponzoñar",
    "desemponzoñaras": "desemponzoñar",
    "desemponzoñare": "desemponzoñar",
    "desemponzoñareis": "desemponzoñar",
    "desemponzoñaremos": "desemponzoñar",
    "desemponzoñaren": "desemponzoñar",
    "desemponzoñares": "desemponzoñar",
    "desemponzoñaron": "desemponzoñar",
    "desemponzoñará": "desemponzoñar",
    "desemponzoñarán": "desemponzoñar",
    "desemponzoñarás": "desemponzoñar",
    "desemponzoñaré": "desemponzoñar",
    "desemponzoñaréis": "desemponzoñar",
    "desemponzoñaría": "desemponzoñar",
    "desemponzoñaríais": "desemponzoñar",
    "desemponzoñaríamos": "desemponzoñar",
    "desemponzoñarían": "desemponzoñar",
    "desemponzoñarías": "desemponzoñar",
    "desemponzoñas": "desemponzoñar",
    "desemponzoñase": "desemponzoñar",
    "desemponzoñaseis": "desemponzoñar",
    "desemponzoñasen": "desemponzoñar",
    "desemponzoñases": "desemponzoñar",
    "desemponzoñaste": "desemponzoñar",
    "desemponzoñasteis": "desemponzoñar",
    "desemponzoñe": "desemponzoñar",
    "desemponzoñemos": "desemponzoñar",
    "desemponzoñen": "desemponzoñar",
    "desemponzoñes": "desemponzoñar",
    "desemponzoño": "desemponzoñar",
    "desemponzoñábamos": "desemponzoñar",
    "desemponzoñáis": "desemponzoñar",
    "desemponzoñáramos": "desemponzoñar",
    "desemponzoñáremos": "desemponzoñar",
    "desemponzoñásemos": "desemponzoñar",
    "desemponzoñé": "desemponzoñar",
    "desemponzoñéis": "desemponzoñar",
    "desemponzoñó": "desemponzoñar",
    "desempotra": "desempotrar",
    "desempotraba": "desempotrar",
    "desempotrabais": "desempotrar",
    "desempotraban": "desempotrar",
    "desempotrabas": "desempotrar",
    "desempotrad": "desempotrar",
    "desempotrada": "desempotrar",
    "desempotradas": "desempotrar",
    "desempotrado": "desempotrar",
    "desempotrados": "desempotrar",
    "desempotramientos": "desempotramiento",
    "desempotramos": "desempotrar",
    "desempotran": "desempotrar",
    "desempotrando": "desempotrar",
    "desempotrara": "desempotrar",
    "desempotrarais": "desempotrar",
    "desempotraran": "desempotrar",
    "desempotraras": "desempotrar",
    "desempotrare": "desempotrar",
    "desempotrareis": "desempotrar",
    "desempotraremos": "desempotrar",
    "desempotraren": "desempotrar",
    "desempotrares": "desempotrar",
    "desempotraron": "desempotrar",
    "desempotrará": "desempotrar",
    "desempotrarán": "desempotrar",
    "desempotrarás": "desempotrar",
    "desempotraré": "desempotrar",
    "desempotraréis": "desempotrar",
    "desempotraría": "desempotrar",
    "desempotraríais": "desempotrar",
    "desempotraríamos": "desempotrar",
    "desempotrarían": "desempotrar",
    "desempotrarías": "desempotrar",
    "desempotras": "desempotrar",
    "desempotrase": "desempotrar",
    "desempotraseis": "desempotrar",
    "desempotrasen": "desempotrar",
    "desempotrases": "desempotrar",
    "desempotraste": "desempotrar",
    "desempotrasteis": "desempotrar",
    "desempotre": "desempotrar",
    "desempotremos": "desempotrar",
    "desempotren": "desempotrar",
    "desempotres": "desempotrar",
    "desempotro": "desempotrar",
    "desempotrábamos": "desempotrar",
    "desempotráis": "desempotrar",
    "desempotráramos": "desempotrar",
    "desempotráremos": "desempotrar",
    "desempotrásemos": "desempotrar",
    "desempotré": "desempotrar",
    "desempotréis": "desempotrar",
    "desempotró": "desempotrar",
    "desempoza": "desempozar",
    "desempozaba": "desempozar",
    "desempozabais": "desempozar",
    "desempozaban": "desempozar",
    "desempozabas": "desempozar",
    "desempozad": "desempozar",
    "desempozada": "desempozar",
    "desempozadas": "desempozar",
    "desempozado": "desempozar",
    "desempozados": "desempozar",
    "desempozamos": "desempozar",
    "desempozan": "desempozar",
    "desempozando": "desempozar",
    "desempozara": "desempozar",
    "desempozarais": "desempozar",
    "desempozaran": "desempozar",
    "desempozaras": "desempozar",
    "desempozare": "desempozar",
    "desempozareis": "desempozar",
    "desempozaremos": "desempozar",
    "desempozaren": "desempozar",
    "desempozares": "desempozar",
    "desempozaron": "desempozar",
    "desempozará": "desempozar",
    "desempozarán": "desempozar",
    "desempozarás": "desempozar",
    "desempozaré": "desempozar",
    "desempozaréis": "desempozar",
    "desempozaría": "desempozar",
    "desempozaríais": "desempozar",
    "desempozaríamos": "desempozar",
    "desempozarían": "desempozar",
    "desempozarías": "desempozar",
    "desempozas": "desempozar",
    "desempozase": "desempozar",
    "desempozaseis": "desempozar",
    "desempozasen": "desempozar",
    "desempozases": "desempozar",
    "desempozaste": "desempozar",
    "desempozasteis": "desempozar",
    "desempozo": "desempozar",
    "desempozábamos": "desempozar",
    "desempozáis": "desempozar",
    "desempozáramos": "desempozar",
    "desempozáremos": "desempozar",
    "desempozásemos": "desempozar",
    "desempozó": "desempozar",
    "desempuña": "desempuñar",
    "desempuñaba": "desempuñar",
    "desempuñabais": "desempuñar",
    "desempuñaban": "desempuñar",
    "desempuñabas": "desempuñar",
    "desempuñad": "desempuñar",
    "desempuñada": "desempuñar",
    "desempuñadas": "desempuñar",
    "desempuñado": "desempuñar",
    "desempuñados": "desempuñar",
    "desempuñamos": "desempuñar",
    "desempuñan": "desempuñar",
    "desempuñando": "desempuñar",
    "desempuñara": "desempuñar",
    "desempuñarais": "desempuñar",
    "desempuñaran": "desempuñar",
    "desempuñaras": "desempuñar",
    "desempuñare": "desempuñar",
    "desempuñareis": "desempuñar",
    "desempuñaremos": "desempuñar",
    "desempuñaren": "desempuñar",
    "desempuñares": "desempuñar",
    "desempuñaron": "desempuñar",
    "desempuñará": "desempuñar",
    "desempuñarán": "desempuñar",
    "desempuñarás": "desempuñar",
    "desempuñaré": "desempuñar",
    "desempuñaréis": "desempuñar",
    "desempuñaría": "desempuñar",
    "desempuñaríais": "desempuñar",
    "desempuñaríamos": "desempuñar",
    "desempuñarían": "desempuñar",
    "desempuñarías": "desempuñar",
    "desempuñas": "desempuñar",
    "desempuñase": "desempuñar",
    "desempuñaseis": "desempuñar",
    "desempuñasen": "desempuñar",
    "desempuñases": "desempuñar",
    "desempuñaste": "desempuñar",
    "desempuñasteis": "desempuñar",
    "desempuñe": "desempuñar",
    "desempuñemos": "desempuñar",
    "desempuñen": "desempuñar",
    "desempuñes": "desempuñar",
    "desempuño": "desempuñar",
    "desempuñábamos": "desempuñar",
    "desempuñáis": "desempuñar",
    "desempuñáramos": "desempuñar",
    "desempuñáremos": "desempuñar",
    "desempuñásemos": "desempuñar",
    "desempuñé": "desempuñar",
    "desempuñéis": "desempuñar",
    "desempuñó": "desempuñar",
    "desenamora": "desenamorar",
    "desenamoraba": "desenamorar",
    "desenamorabais": "desenamorar",
    "desenamoraban": "desenamorar",
    "desenamorabas": "desenamorar",
    "desenamorad": "desenamorar",
    "desenamorada": "desenamorar",
    "desenamoradas": "desenamorar",
    "desenamorado": "desenamorar",
    "desenamorados": "desenamorar",
    "desenamoramientos": "desenamoramiento",
    "desenamoramos": "desenamorar",
    "desenamoran": "desenamorar",
    "desenamorando": "desenamorar",
    "desenamorara": "desenamorar",
    "desenamorarais": "desenamorar",
    "desenamoraran": "desenamorar",
    "desenamoraras": "desenamorar",
    "desenamorare": "desenamorar",
    "desenamorareis": "desenamorar",
    "desenamoraremos": "desenamorar",
    "desenamoraren": "desenamorar",
    "desenamorares": "desenamorar",
    "desenamoraron": "desenamorar",
    "desenamorará": "desenamorar",
    "desenamorarán": "desenamorar",
    "desenamorarás": "desenamorar",
    "desenamoraré": "desenamorar",
    "desenamoraréis": "desenamorar",
    "desenamoraría": "desenamorar",
    "desenamoraríais": "desenamorar",
    "desenamoraríamos": "desenamorar",
    "desenamorarían": "desenamorar",
    "desenamorarías": "desenamorar",
    "desenamoras": "desenamorar",
    "desenamorase": "desenamorar",
    "desenamoraseis": "desenamorar",
    "desenamorasen": "desenamorar",
    "desenamorases": "desenamorar",
    "desenamoraste": "desenamorar",
    "desenamorasteis": "desenamorar",
    "desenamore": "desenamorar",
    "desenamoremos": "desenamorar",
    "desenamoren": "desenamorar",
    "desenamores": "desenamorar",
    "desenamoro": "desenamorar",
    "desenamorábamos": "desenamorar",
    "desenamoráis": "desenamorar",
    "desenamoráramos": "desenamorar",
    "desenamoráremos": "desenamorar",
    "desenamorásemos": "desenamorar",
    "desenamoré": "desenamorar",
    "desenamoréis": "desenamorar",
    "desenamoró": "desenamorar",
    "desencabalgamientos": "desencabalgamiento",
    "desencabestra": "desencabestrar",
    "desencabestraba": "desencabestrar",
    "desencabestrabais": "desencabestrar",
    "desencabestraban": "desencabestrar",
    "desencabestrabas": "desencabestrar",
    "desencabestrad": "desencabestrar",
    "desencabestrada": "desencabestrar",
    "desencabestradas": "desencabestrar",
    "desencabestrado": "desencabestrar",
    "desencabestrados": "desencabestrar",
    "desencabestramos": "desencabestrar",
    "desencabestran": "desencabestrar",
    "desencabestrando": "desencabestrar",
    "desencabestrara": "desencabestrar",
    "desencabestrarais": "desencabestrar",
    "desencabestraran": "desencabestrar",
    "desencabestraras": "desencabestrar",
    "desencabestrare": "desencabestrar",
    "desencabestrareis": "desencabestrar",
    "desencabestraremos": "desencabestrar",
    "desencabestraren": "desencabestrar",
    "desencabestrares": "desencabestrar",
    "desencabestraron": "desencabestrar",
    "desencabestrará": "desencabestrar",
    "desencabestrarán": "desencabestrar",
    "desencabestrarás": "desencabestrar",
    "desencabestraré": "desencabestrar",
    "desencabestraréis": "desencabestrar",
    "desencabestraría": "desencabestrar",
    "desencabestraríais": "desencabestrar",
    "desencabestraríamos": "desencabestrar",
    "desencabestrarían": "desencabestrar",
    "desencabestrarías": "desencabestrar",
    "desencabestras": "desencabestrar",
    "desencabestrase": "desencabestrar",
    "desencabestraseis": "desencabestrar",
    "desencabestrasen": "desencabestrar",
    "desencabestrases": "desencabestrar",
    "desencabestraste": "desencabestrar",
    "desencabestrasteis": "desencabestrar",
    "desencabestre": "desencabestrar",
    "desencabestremos": "desencabestrar",
    "desencabestren": "desencabestrar",
    "desencabestres": "desencabestrar",
    "desencabestro": "desencabestrar",
    "desencabestrábamos": "desencabestrar",
    "desencabestráis": "desencabestrar",
    "desencabestráramos": "desencabestrar",
    "desencabestráremos": "desencabestrar",
    "desencabestrásemos": "desencabestrar",
    "desencabestré": "desencabestrar",
    "desencabestréis": "desencabestrar",
    "desencabestró": "desencabestrar",
    "desencadena": "desencadenar",
    "desencadenaba": "desencadenar",
    "desencadenabais": "desencadenar",
    "desencadenaban": "desencadenar",
    "desencadenabas": "desencadenar",
    "desencadenad": "desencadenar",
    "desencadenada": "desencadenar",
    "desencadenadas": "desencadenar",
    "desencadenado": "desencadenar",
    "desencadenados": "desencadenar",
    "desencadenamientos": "desencadenamiento",
    "desencadenamos": "desencadenar",
    "desencadenan": "desencadenar",
    "desencadenando": "desencadenar",
    "desencadenantes": "desencadenante",
    "desencadenara": "desencadenar",
    "desencadenarais": "desencadenar",
    "desencadenaran": "desencadenar",
    "desencadenaras": "desencadenar",
    "desencadenare": "desencadenar",
    "desencadenareis": "desencadenar",
    "desencadenaremos": "desencadenar",
    "desencadenaren": "desencadenar",
    "desencadenares": "desencadenar",
    "desencadenaron": "desencadenar",
    "desencadenará": "desencadenar",
    "desencadenarán": "desencadenar",
    "desencadenarás": "desencadenar",
    "desencadenaré": "desencadenar",
    "desencadenaréis": "desencadenar",
    "desencadenaría": "desencadenar",
    "desencadenaríais": "desencadenar",
    "desencadenaríamos": "desencadenar",
    "desencadenarían": "desencadenar",
    "desencadenarías": "desencadenar",
    "desencadenas": "desencadenar",
    "desencadenase": "desencadenar",
    "desencadenaseis": "desencadenar",
    "desencadenasen": "desencadenar",
    "desencadenases": "desencadenar",
    "desencadenaste": "desencadenar",
    "desencadenasteis": "desencadenar",
    "desencadene": "desencadenar",
    "desencadenemos": "desencadenar",
    "desencadenen": "desencadenar",
    "desencadenes": "desencadenar",
    "desencadeno": "desencadenar",
    "desencadenábamos": "desencadenar",
    "desencadenáis": "desencadenar",
    "desencadenáramos": "desencadenar",
    "desencadenáremos": "desencadenar",
    "desencadenásemos": "desencadenar",
    "desencadené": "desencadenar",
    "desencadenéis": "desencadenar",
    "desencadenó": "desencadenar",
    "desencaja": "desencajar",
    "desencajaba": "desencajar",
    "desencajabais": "desencajar",
    "desencajaban": "desencajar",
    "desencajabas": "desencajar",
    "desencajad": "desencajar",
    "desencajada": "desencajar",
    "desencajadas": "desencajar",
    "desencajado": "desencajar",
    "desencajados": "desencajar",
    "desencajaduras": "desencajadura",
    "desencajamientos": "desencajamiento",
    "desencajamos": "desencajar",
    "desencajan": "desencajar",
    "desencajando": "desencajar",
    "desencajara": "desencajar",
    "desencajarais": "desencajar",
    "desencajaran": "desencajar",
    "desencajaras": "desencajar",
    "desencajare": "desencajar",
    "desencajareis": "desencajar",
    "desencajaremos": "desencajar",
    "desencajaren": "desencajar",
    "desencajares": "desencajar",
    "desencajaron": "desencajar",
    "desencajará": "desencajar",
    "desencajarán": "desencajar",
    "desencajarás": "desencajar",
    "desencajaré": "desencajar",
    "desencajaréis": "desencajar",
    "desencajaría": "desencajar",
    "desencajaríais": "desencajar",
    "desencajaríamos": "desencajar",
    "desencajarían": "desencajar",
    "desencajarías": "desencajar",
    "desencajas": "desencajar",
    "desencajase": "desencajar",
    "desencajaseis": "desencajar",
    "desencajasen": "desencajar",
    "desencajases": "desencajar",
    "desencajaste": "desencajar",
    "desencajasteis": "desencajar",
    "desencaje": "desencajar",
    "desencajemos": "desencajar",
    "desencajen": "desencajar",
    "desencajes": "desencaje",
    "desencajo": "desencajar",
    "desencajona": "desencajonar",
    "desencajonaba": "desencajonar",
    "desencajonabais": "desencajonar",
    "desencajonaban": "desencajonar",
    "desencajonabas": "desencajonar",
    "desencajonad": "desencajonar",
    "desencajonada": "desencajonar",
    "desencajonadas": "desencajonar",
    "desencajonado": "desencajonar",
    "desencajonados": "desencajonar",
    "desencajonamos": "desencajonar",
    "desencajonan": "desencajonar",
    "desencajonando": "desencajonar",
    "desencajonara": "desencajonar",
    "desencajonarais": "desencajonar",
    "desencajonaran": "desencajonar",
    "desencajonaras": "desencajonar",
    "desencajonare": "desencajonar",
    "desencajonareis": "desencajonar",
    "desencajonaremos": "desencajonar",
    "desencajonaren": "desencajonar",
    "desencajonares": "desencajonar",
    "desencajonaron": "desencajonar",
    "desencajonará": "desencajonar",
    "desencajonarán": "desencajonar",
    "desencajonarás": "desencajonar",
    "desencajonaré": "desencajonar",
    "desencajonaréis": "desencajonar",
    "desencajonaría": "desencajonar",
    "desencajonaríais": "desencajonar",
    "desencajonaríamos": "desencajonar",
    "desencajonarían": "desencajonar",
    "desencajonarías": "desencajonar",
    "desencajonas": "desencajonar",
    "desencajonase": "desencajonar",
    "desencajonaseis": "desencajonar",
    "desencajonasen": "desencajonar",
    "desencajonases": "desencajonar",
    "desencajonaste": "desencajonar",
    "desencajonasteis": "desencajonar",
    "desencajone": "desencajonar",
    "desencajonemos": "desencajonar",
    "desencajonen": "desencajonar",
    "desencajones": "desencajonar",
    "desencajono": "desencajonar",
    "desencajonábamos": "desencajonar",
    "desencajonáis": "desencajonar",
    "desencajonáramos": "desencajonar",
    "desencajonáremos": "desencajonar",
    "desencajonásemos": "desencajonar",
    "desencajoné": "desencajonar",
    "desencajonéis": "desencajonar",
    "desencajonó": "desencajonar",
    "desencajábamos": "desencajar",
    "desencajáis": "desencajar",
    "desencajáramos": "desencajar",
    "desencajáremos": "desencajar",
    "desencajásemos": "desencajar",
    "desencajé": "desencajar",
    "desencajéis": "desencajar",
    "desencajó": "desencajar",
    "desencalla": "desencallar",
    "desencallaba": "desencallar",
    "desencallabais": "desencallar",
    "desencallaban": "desencallar",
    "desencallabas": "desencallar",
    "desencallad": "desencallar",
    "desencallada": "desencallar",
    "desencalladas": "desencallar",
    "desencallado": "desencallar",
    "desencallados": "desencallar",
    "desencallamientos": "desencallamiento",
    "desencallamos": "desencallar",
    "desencallan": "desencallar",
    "desencallando": "desencallar",
    "desencallara": "desencallar",
    "desencallarais": "desencallar",
    "desencallaran": "desencallar",
    "desencallaras": "desencallar",
    "desencallare": "desencallar",
    "desencallareis": "desencallar",
    "desencallaremos": "desencallar",
    "desencallaren": "desencallar",
    "desencallares": "desencallar",
    "desencallaron": "desencallar",
    "desencallará": "desencallar",
    "desencallarán": "desencallar",
    "desencallarás": "desencallar",
    "desencallaré": "desencallar",
    "desencallaréis": "desencallar",
    "desencallaría": "desencallar",
    "desencallaríais": "desencallar",
    "desencallaríamos": "desencallar",
    "desencallarían": "desencallar",
    "desencallarías": "desencallar",
    "desencallas": "desencallar",
    "desencallase": "desencallar",
    "desencallaseis": "desencallar",
    "desencallasen": "desencallar",
    "desencallases": "desencallar",
    "desencallaste": "desencallar",
    "desencallasteis": "desencallar",
    "desencalle": "desencallar",
    "desencallemos": "desencallar",
    "desencallen": "desencallar",
    "desencalles": "desencallar",
    "desencallo": "desencallar",
    "desencallábamos": "desencallar",
    "desencalláis": "desencallar",
    "desencalláramos": "desencallar",
    "desencalláremos": "desencallar",
    "desencallásemos": "desencallar",
    "desencallé": "desencallar",
    "desencalléis": "desencallar",
    "desencalló": "desencallar",
    "desencamina": "desencaminar",
    "desencaminaba": "desencaminar",
    "desencaminabais": "desencaminar",
    "desencaminaban": "desencaminar",
    "desencaminabas": "desencaminar",
    "desencaminad": "desencaminar",
    "desencaminada": "desencaminar",
    "desencaminadas": "desencaminar",
    "desencaminado": "desencaminar",
    "desencaminados": "desencaminar",
    "desencaminamientos": "desencaminamiento",
    "desencaminamos": "desencaminar",
    "desencaminan": "desencaminar",
    "desencaminando": "desencaminar",
    "desencaminara": "desencaminar",
    "desencaminarais": "desencaminar",
    "desencaminaran": "desencaminar",
    "desencaminaras": "desencaminar",
    "desencaminare": "desencaminar",
    "desencaminareis": "desencaminar",
    "desencaminaremos": "desencaminar",
    "desencaminaren": "desencaminar",
    "desencaminares": "desencaminar",
    "desencaminaron": "desencaminar",
    "desencaminará": "desencaminar",
    "desencaminarán": "desencaminar",
    "desencaminarás": "desencaminar",
    "desencaminaré": "desencaminar",
    "desencaminaréis": "desencaminar",
    "desencaminaría": "desencaminar",
    "desencaminaríais": "desencaminar",
    "desencaminaríamos": "desencaminar",
    "desencaminarían": "desencaminar",
    "desencaminarías": "desencaminar",
    "desencaminas": "desencaminar",
    "desencaminase": "desencaminar",
    "desencaminaseis": "desencaminar",
    "desencaminasen": "desencaminar",
    "desencaminases": "desencaminar",
    "desencaminaste": "desencaminar",
    "desencaminasteis": "desencaminar",
    "desencamine": "desencaminar",
    "desencaminemos": "desencaminar",
    "desencaminen": "desencaminar",
    "desencamines": "desencaminar",
    "desencamino": "desencaminar",
    "desencaminábamos": "desencaminar",
    "desencamináis": "desencaminar",
    "desencamináramos": "desencaminar",
    "desencamináremos": "desencaminar",
    "desencaminásemos": "desencaminar",
    "desencaminé": "desencaminar",
    "desencaminéis": "desencaminar",
    "desencaminó": "desencaminar",
    "desencanta": "desencantar",
    "desencantaba": "desencantar",
    "desencantabais": "desencantar",
    "desencantaban": "desencantar",
    "desencantabas": "desencantar",
    "desencantad": "desencantar",
    "desencantada": "desencantar",
    "desencantadas": "desencantar",
    "desencantado": "desencantar",
    "desencantados": "desencantar",
    "desencantamientos": "desencantamiento",
    "desencantamos": "desencantar",
    "desencantan": "desencantar",
    "desencantando": "desencantar",
    "desencantara": "desencantar",
    "desencantaraciones": "desencantaración",
    "desencantarais": "desencantar",
    "desencantaran": "desencantar",
    "desencantaras": "desencantar",
    "desencantare": "desencantar",
    "desencantareis": "desencantar",
    "desencantaremos": "desencantar",
    "desencantaren": "desencantar",
    "desencantares": "desencantar",
    "desencantaron": "desencantar",
    "desencantará": "desencantar",
    "desencantarán": "desencantar",
    "desencantarás": "desencantar",
    "desencantaré": "desencantar",
    "desencantaréis": "desencantar",
    "desencantaría": "desencantar",
    "desencantaríais": "desencantar",
    "desencantaríamos": "desencantar",
    "desencantarían": "desencantar",
    "desencantarías": "desencantar",
    "desencantas": "desencantar",
    "desencantase": "desencantar",
    "desencantaseis": "desencantar",
    "desencantasen": "desencantar",
    "desencantases": "desencantar",
    "desencantaste": "desencantar",
    "desencantasteis": "desencantar",
    "desencante": "desencantar",
    "desencantemos": "desencantar",
    "desencanten": "desencantar",
    "desencantes": "desencantar",
    "desencanto": "desencantar",
    "desencantos": "desencanto",
    "desencantábamos": "desencantar",
    "desencantáis": "desencantar",
    "desencantáramos": "desencantar",
    "desencantáremos": "desencantar",
    "desencantásemos": "desencantar",
    "desencanté": "desencantar",
    "desencantéis": "desencantar",
    "desencantó": "desencantar",
    "desencapilla": "desencapillar",
    "desencapillaba": "desencapillar",
    "desencapillabais": "desencapillar",
    "desencapillaban": "desencapillar",
    "desencapillabas": "desencapillar",
    "desencapillad": "desencapillar",
    "desencapillada": "desencapillar",
    "desencapilladas": "desencapillar",
    "desencapillado": "desencapillar",
    "desencapillados": "desencapillar",
    "desencapillamos": "desencapillar",
    "desencapillan": "desencapillar",
    "desencapillando": "desencapillar",
    "desencapillara": "desencapillar",
    "desencapillarais": "desencapillar",
    "desencapillaran": "desencapillar",
    "desencapillaras": "desencapillar",
    "desencapillare": "desencapillar",
    "desencapillareis": "desencapillar",
    "desencapillaremos": "desencapillar",
    "desencapillaren": "desencapillar",
    "desencapillares": "desencapillar",
    "desencapillaron": "desencapillar",
    "desencapillará": "desencapillar",
    "desencapillarán": "desencapillar",
    "desencapillarás": "desencapillar",
    "desencapillaré": "desencapillar",
    "desencapillaréis": "desencapillar",
    "desencapillaría": "desencapillar",
    "desencapillaríais": "desencapillar",
    "desencapillaríamos": "desencapillar",
    "desencapillarían": "desencapillar",
    "desencapillarías": "desencapillar",
    "desencapillas": "desencapillar",
    "desencapillase": "desencapillar",
    "desencapillaseis": "desencapillar",
    "desencapillasen": "desencapillar",
    "desencapillases": "desencapillar",
    "desencapillaste": "desencapillar",
    "desencapillasteis": "desencapillar",
    "desencapille": "desencapillar",
    "desencapillemos": "desencapillar",
    "desencapillen": "desencapillar",
    "desencapilles": "desencapillar",
    "desencapillo": "desencapillar",
    "desencapillábamos": "desencapillar",
    "desencapilláis": "desencapillar",
    "desencapilláramos": "desencapillar",
    "desencapilláremos": "desencapillar",
    "desencapillásemos": "desencapillar",
    "desencapillé": "desencapillar",
    "desencapilléis": "desencapillar",
    "desencapilló": "desencapillar",
    "desencapota": "desencapotar",
    "desencapotaba": "desencapotar",
    "desencapotabais": "desencapotar",
    "desencapotaban": "desencapotar",
    "desencapotabas": "desencapotar",
    "desencapotad": "desencapotar",
    "desencapotada": "desencapotar",
    "desencapotadas": "desencapotar",
    "desencapotado": "desencapotar",
    "desencapotados": "desencapotar",
    "desencapotamientos": "desencapotamiento",
    "desencapotamos": "desencapotar",
    "desencapotan": "desencapotar",
    "desencapotando": "desencapotar",
    "desencapotara": "desencapotar",
    "desencapotarais": "desencapotar",
    "desencapotaran": "desencapotar",
    "desencapotaras": "desencapotar",
    "desencapotare": "desencapotar",
    "desencapotareis": "desencapotar",
    "desencapotaremos": "desencapotar",
    "desencapotaren": "desencapotar",
    "desencapotares": "desencapotar",
    "desencapotaron": "desencapotar",
    "desencapotará": "desencapotar",
    "desencapotarán": "desencapotar",
    "desencapotarás": "desencapotar",
    "desencapotaré": "desencapotar",
    "desencapotaréis": "desencapotar",
    "desencapotaría": "desencapotar",
    "desencapotaríais": "desencapotar",
    "desencapotaríamos": "desencapotar",
    "desencapotarían": "desencapotar",
    "desencapotarías": "desencapotar",
    "desencapotas": "desencapotar",
    "desencapotase": "desencapotar",
    "desencapotaseis": "desencapotar",
    "desencapotasen": "desencapotar",
    "desencapotases": "desencapotar",
    "desencapotaste": "desencapotar",
    "desencapotasteis": "desencapotar",
    "desencapote": "desencapotar",
    "desencapotemos": "desencapotar",
    "desencapoten": "desencapotar",
    "desencapotes": "desencapotar",
    "desencapoto": "desencapotar",
    "desencapotábamos": "desencapotar",
    "desencapotáis": "desencapotar",
    "desencapotáramos": "desencapotar",
    "desencapotáremos": "desencapotar",
    "desencapotásemos": "desencapotar",
    "desencapoté": "desencapotar",
    "desencapotéis": "desencapotar",
    "desencapotó": "desencapotar",
    "desencapricha": "desencaprichar",
    "desencaprichaba": "desencaprichar",
    "desencaprichabais": "desencaprichar",
    "desencaprichaban": "desencaprichar",
    "desencaprichabas": "desencaprichar",
    "desencaprichad": "desencaprichar",
    "desencaprichada": "desencaprichar",
    "desencaprichadas": "desencaprichar",
    "desencaprichado": "desencaprichar",
    "desencaprichados": "desencaprichar",
    "desencaprichamientos": "desencaprichamiento",
    "desencaprichamos": "desencaprichar",
    "desencaprichan": "desencaprichar",
    "desencaprichando": "desencaprichar",
    "desencaprichara": "desencaprichar",
    "desencapricharais": "desencaprichar",
    "desencapricharan": "desencaprichar",
    "desencapricharas": "desencaprichar",
    "desencaprichare": "desencaprichar",
    "desencaprichareis": "desencaprichar",
    "desencapricharemos": "desencaprichar",
    "desencapricharen": "desencaprichar",
    "desencaprichares": "desencaprichar",
    "desencapricharon": "desencaprichar",
    "desencaprichará": "desencaprichar",
    "desencapricharán": "desencaprichar",
    "desencapricharás": "desencaprichar",
    "desencapricharé": "desencaprichar",
    "desencapricharéis": "desencaprichar",
    "desencapricharía": "desencaprichar",
    "desencapricharíais": "desencaprichar",
    "desencapricharíamos": "desencaprichar",
    "desencapricharían": "desencaprichar",
    "desencapricharías": "desencaprichar",
    "desencaprichas": "desencaprichar",
    "desencaprichase": "desencaprichar",
    "desencaprichaseis": "desencaprichar",
    "desencaprichasen": "desencaprichar",
    "desencaprichases": "desencaprichar",
    "desencaprichaste": "desencaprichar",
    "desencaprichasteis": "desencaprichar",
    "desencapriche": "desencaprichar",
    "desencaprichemos": "desencaprichar",
    "desencaprichen": "desencaprichar",
    "desencapriches": "desencaprichar",
    "desencapricho": "desencaprichar",
    "desencaprichábamos": "desencaprichar",
    "desencapricháis": "desencaprichar",
    "desencapricháramos": "desencaprichar",
    "desencapricháremos": "desencaprichar",
    "desencaprichásemos": "desencaprichar",
    "desencapriché": "desencaprichar",
    "desencaprichéis": "desencaprichar",
    "desencaprichó": "desencaprichar",
    "desencarcela": "desencarcelar",
    "desencarcelaba": "desencarcelar",
    "desencarcelabais": "desencarcelar",
    "desencarcelaban": "desencarcelar",
    "desencarcelabas": "desencarcelar",
    "desencarcelad": "desencarcelar",
    "desencarcelada": "desencarcelar",
    "desencarceladas": "desencarcelar",
    "desencarcelado": "desencarcelar",
    "desencarcelados": "desencarcelar",
    "desencarcelamientos": "desencarcelamiento",
    "desencarcelamos": "desencarcelar",
    "desencarcelan": "desencarcelar",
    "desencarcelando": "desencarcelar",
    "desencarcelara": "desencarcelar",
    "desencarcelarais": "desencarcelar",
    "desencarcelaran": "desencarcelar",
    "desencarcelaras": "desencarcelar",
    "desencarcelare": "desencarcelar",
    "desencarcelareis": "desencarcelar",
    "desencarcelaremos": "desencarcelar",
    "desencarcelaren": "desencarcelar",
    "desencarcelares": "desencarcelar",
    "desencarcelaron": "desencarcelar",
    "desencarcelará": "desencarcelar",
    "desencarcelarán": "desencarcelar",
    "desencarcelarás": "desencarcelar",
    "desencarcelaré": "desencarcelar",
    "desencarcelaréis": "desencarcelar",
    "desencarcelaría": "desencarcelar",
    "desencarcelaríais": "desencarcelar",
    "desencarcelaríamos": "desencarcelar",
    "desencarcelarían": "desencarcelar",
    "desencarcelarías": "desencarcelar",
    "desencarcelas": "desencarcelar",
    "desencarcelase": "desencarcelar",
    "desencarcelaseis": "desencarcelar",
    "desencarcelasen": "desencarcelar",
    "desencarcelases": "desencarcelar",
    "desencarcelaste": "desencarcelar",
    "desencarcelasteis": "desencarcelar",
    "desencarcele": "desencarcelar",
    "desencarcelemos": "desencarcelar",
    "desencarcelen": "desencarcelar",
    "desencarceles": "desencarcelar",
    "desencarcelo": "desencarcelar",
    "desencarcelábamos": "desencarcelar",
    "desencarceláis": "desencarcelar",
    "desencarceláramos": "desencarcelar",
    "desencarceláremos": "desencarcelar",
    "desencarcelásemos": "desencarcelar",
    "desencarcelé": "desencarcelar",
    "desencarceléis": "desencarcelar",
    "desencarceló": "desencarcelar",
    "desencarecimientos": "desencarecimiento",
    "desencarga": "desencargar",
    "desencargaba": "desencargar",
    "desencargabais": "desencargar",
    "desencargaban": "desencargar",
    "desencargabas": "desencargar",
    "desencargad": "desencargar",
    "desencargada": "desencargar",
    "desencargadas": "desencargar",
    "desencargado": "desencargar",
    "desencargados": "desencargar",
    "desencargamos": "desencargar",
    "desencargan": "desencargar",
    "desencargando": "desencargar",
    "desencargara": "desencargar",
    "desencargarais": "desencargar",
    "desencargaran": "desencargar",
    "desencargaras": "desencargar",
    "desencargare": "desencargar",
    "desencargareis": "desencargar",
    "desencargaremos": "desencargar",
    "desencargaren": "desencargar",
    "desencargares": "desencargar",
    "desencargaron": "desencargar",
    "desencargará": "desencargar",
    "desencargarán": "desencargar",
    "desencargarás": "desencargar",
    "desencargaré": "desencargar",
    "desencargaréis": "desencargar",
    "desencargaría": "desencargar",
    "desencargaríais": "desencargar",
    "desencargaríamos": "desencargar",
    "desencargarían": "desencargar",
    "desencargarías": "desencargar",
    "desencargas": "desencargar",
    "desencargase": "desencargar",
    "desencargaseis": "desencargar",
    "desencargasen": "desencargar",
    "desencargases": "desencargar",
    "desencargaste": "desencargar",
    "desencargasteis": "desencargar",
    "desencargo": "desencargar",
    "desencargue": "desencargar",
    "desencarguemos": "desencargar",
    "desencarguen": "desencargar",
    "desencargues": "desencargar",
    "desencargué": "desencargar",
    "desencarguéis": "desencargar",
    "desencargábamos": "desencargar",
    "desencargáis": "desencargar",
    "desencargáramos": "desencargar",
    "desencargáremos": "desencargar",
    "desencargásemos": "desencargar",
    "desencargó": "desencargar",
    "desencarna": "desencarnar",
    "desencarnaba": "desencarnar",
    "desencarnabais": "desencarnar",
    "desencarnaban": "desencarnar",
    "desencarnabas": "desencarnar",
    "desencarnad": "desencarnar",
    "desencarnada": "desencarnar",
    "desencarnadas": "desencarnar",
    "desencarnado": "desencarnar",
    "desencarnados": "desencarnar",
    "desencarnamos": "desencarnar",
    "desencarnan": "desencarnar",
    "desencarnando": "desencarnar",
    "desencarnara": "desencarnar",
    "desencarnarais": "desencarnar",
    "desencarnaran": "desencarnar",
    "desencarnaras": "desencarnar",
    "desencarnare": "desencarnar",
    "desencarnareis": "desencarnar",
    "desencarnaremos": "desencarnar",
    "desencarnaren": "desencarnar",
    "desencarnares": "desencarnar",
    "desencarnaron": "desencarnar",
    "desencarnará": "desencarnar",
    "desencarnarán": "desencarnar",
    "desencarnarás": "desencarnar",
    "desencarnaré": "desencarnar",
    "desencarnaréis": "desencarnar",
    "desencarnaría": "desencarnar",
    "desencarnaríais": "desencarnar",
    "desencarnaríamos": "desencarnar",
    "desencarnarían": "desencarnar",
    "desencarnarías": "desencarnar",
    "desencarnas": "desencarnar",
    "desencarnase": "desencarnar",
    "desencarnaseis": "desencarnar",
    "desencarnasen": "desencarnar",
    "desencarnases": "desencarnar",
    "desencarnaste": "desencarnar",
    "desencarnasteis": "desencarnar",
    "desencarne": "desencarnar",
    "desencarnemos": "desencarnar",
    "desencarnen": "desencarnar",
    "desencarnes": "desencarnar",
    "desencarno": "desencarnar",
    "desencarnábamos": "desencarnar",
    "desencarnáis": "desencarnar",
    "desencarnáramos": "desencarnar",
    "desencarnáremos": "desencarnar",
    "desencarnásemos": "desencarnar",
    "desencarné": "desencarnar",
    "desencarnéis": "desencarnar",
    "desencarnó": "desencarnar",
    "desencasquilla": "desencasquillar",
    "desencasquillaba": "desencasquillar",
    "desencasquillabais": "desencasquillar",
    "desencasquillaban": "desencasquillar",
    "desencasquillabas": "desencasquillar",
    "desencasquillad": "desencasquillar",
    "desencasquillada": "desencasquillar",
    "desencasquilladas": "desencasquillar",
    "desencasquillado": "desencasquillar",
    "desencasquillados": "desencasquillar",
    "desencasquillamos": "desencasquillar",
    "desencasquillan": "desencasquillar",
    "desencasquillando": "desencasquillar",
    "desencasquillara": "desencasquillar",
    "desencasquillarais": "desencasquillar",
    "desencasquillaran": "desencasquillar",
    "desencasquillaras": "desencasquillar",
    "desencasquillare": "desencasquillar",
    "desencasquillareis": "desencasquillar",
    "desencasquillaremos": "desencasquillar",
    "desencasquillaren": "desencasquillar",
    "desencasquillares": "desencasquillar",
    "desencasquillaron": "desencasquillar",
    "desencasquillará": "desencasquillar",
    "desencasquillarán": "desencasquillar",
    "desencasquillarás": "desencasquillar",
    "desencasquillaré": "desencasquillar",
    "desencasquillaréis": "desencasquillar",
    "desencasquillaría": "desencasquillar",
    "desencasquillaríais": "desencasquillar",
    "desencasquillaríamos": "desencasquillar",
    "desencasquillarían": "desencasquillar",
    "desencasquillarías": "desencasquillar",
    "desencasquillas": "desencasquillar",
    "desencasquillase": "desencasquillar",
    "desencasquillaseis": "desencasquillar",
    "desencasquillasen": "desencasquillar",
    "desencasquillases": "desencasquillar",
    "desencasquillaste": "desencasquillar",
    "desencasquillasteis": "desencasquillar",
    "desencasquille": "desencasquillar",
    "desencasquillemos": "desencasquillar",
    "desencasquillen": "desencasquillar",
    "desencasquilles": "desencasquillar",
    "desencasquillo": "desencasquillar",
    "desencasquillábamos": "desencasquillar",
    "desencasquilláis": "desencasquillar",
    "desencasquilláramos": "desencasquillar",
    "desencasquilláremos": "desencasquillar",
    "desencasquillásemos": "desencasquillar",
    "desencasquillé": "desencasquillar",
    "desencasquilléis": "desencasquillar",
    "desencasquilló": "desencasquillar",
    "desencerraba": "desencerrar",
    "desencerrabais": "desencerrar",
    "desencerraban": "desencerrar",
    "desencerrabas": "desencerrar",
    "desencerrad": "desencerrar",
    "desencerrada": "desencerrar",
    "desencerradas": "desencerrar",
    "desencerrado": "desencerrar",
    "desencerrados": "desencerrar",
    "desencerramos": "desencerrar",
    "desencerrando": "desencerrar",
    "desencerrara": "desencerrar",
    "desencerrarais": "desencerrar",
    "desencerraran": "desencerrar",
    "desencerraras": "desencerrar",
    "desencerrare": "desencerrar",
    "desencerrareis": "desencerrar",
    "desencerraremos": "desencerrar",
    "desencerraren": "desencerrar",
    "desencerrares": "desencerrar",
    "desencerraron": "desencerrar",
    "desencerrará": "desencerrar",
    "desencerrarán": "desencerrar",
    "desencerrarás": "desencerrar",
    "desencerraré": "desencerrar",
    "desencerraréis": "desencerrar",
    "desencerraría": "desencerrar",
    "desencerraríais": "desencerrar",
    "desencerraríamos": "desencerrar",
    "desencerrarían": "desencerrar",
    "desencerrarías": "desencerrar",
    "desencerrase": "desencerrar",
    "desencerraseis": "desencerrar",
    "desencerrasen": "desencerrar",
    "desencerrases": "desencerrar",
    "desencerraste": "desencerrar",
    "desencerrasteis": "desencerrar",
    "desencerremos": "desencerrar",
    "desencerrábamos": "desencerrar",
    "desencerráis": "desencerrar",
    "desencerráramos": "desencerrar",
    "desencerráremos": "desencerrar",
    "desencerrásemos": "desencerrar",
    "desencerré": "desencerrar",
    "desencerréis": "desencerrar",
    "desencerró": "desencerrar",
    "desenchufa": "desenchufar",
    "desenchufaba": "desenchufar",
    "desenchufabais": "desenchufar",
    "desenchufaban": "desenchufar",
    "desenchufabas": "desenchufar",
    "desenchufables": "desenchufable",
    "desenchufad": "desenchufar",
    "desenchufada": "desenchufar",
    "desenchufadas": "desenchufar",
    "desenchufado": "desenchufar",
    "desenchufados": "desenchufar",
    "desenchufamos": "desenchufar",
    "desenchufan": "desenchufar",
    "desenchufando": "desenchufar",
    "desenchufara": "desenchufar",
    "desenchufarais": "desenchufar",
    "desenchufaran": "desenchufar",
    "desenchufaras": "desenchufar",
    "desenchufare": "desenchufar",
    "desenchufareis": "desenchufar",
    "desenchufaremos": "desenchufar",
    "desenchufaren": "desenchufar",
    "desenchufares": "desenchufar",
    "desenchufaron": "desenchufar",
    "desenchufará": "desenchufar",
    "desenchufarán": "desenchufar",
    "desenchufarás": "desenchufar",
    "desenchufaré": "desenchufar",
    "desenchufaréis": "desenchufar",
    "desenchufaría": "desenchufar",
    "desenchufaríais": "desenchufar",
    "desenchufaríamos": "desenchufar",
    "desenchufarían": "desenchufar",
    "desenchufarías": "desenchufar",
    "desenchufas": "desenchufar",
    "desenchufase": "desenchufar",
    "desenchufaseis": "desenchufar",
    "desenchufasen": "desenchufar",
    "desenchufases": "desenchufar",
    "desenchufaste": "desenchufar",
    "desenchufasteis": "desenchufar",
    "desenchufe": "desenchufar",
    "desenchufemos": "desenchufar",
    "desenchufen": "desenchufar",
    "desenchufes": "desenchufar",
    "desenchufo": "desenchufar",
    "desenchufábamos": "desenchufar",
    "desenchufáis": "desenchufar",
    "desenchufáramos": "desenchufar",
    "desenchufáremos": "desenchufar",
    "desenchufásemos": "desenchufar",
    "desenchufé": "desenchufar",
    "desenchuféis": "desenchufar",
    "desenchufó": "desenchufar",
    "desencierra": "desencerrar",
    "desencierran": "desencerrar",
    "desencierras": "desencerrar",
    "desencierre": "desencerrar",
    "desencierren": "desencerrar",
    "desencierres": "desencerrar",
    "desencierro": "desencerrar",
    "desenclava": "desenclavar",
    "desenclavaba": "desenclavar",
    "desenclavabais": "desenclavar",
    "desenclavaban": "desenclavar",
    "desenclavabas": "desenclavar",
    "desenclavad": "desenclavar",
    "desenclavada": "desenclavar",
    "desenclavadas": "desenclavar",
    "desenclavado": "desenclavar",
    "desenclavados": "desenclavar",
    "desenclavamos": "desenclavar",
    "desenclavan": "desenclavar",
    "desenclavando": "desenclavar",
    "desenclavara": "desenclavar",
    "desenclavarais": "desenclavar",
    "desenclavaran": "desenclavar",
    "desenclavaras": "desenclavar",
    "desenclavare": "desenclavar",
    "desenclavareis": "desenclavar",
    "desenclavaremos": "desenclavar",
    "desenclavaren": "desenclavar",
    "desenclavares": "desenclavar",
    "desenclavaron": "desenclavar",
    "desenclavará": "desenclavar",
    "desenclavarán": "desenclavar",
    "desenclavarás": "desenclavar",
    "desenclavaré": "desenclavar",
    "desenclavaréis": "desenclavar",
    "desenclavaría": "desenclavar",
    "desenclavaríais": "desenclavar",
    "desenclavaríamos": "desenclavar",
    "desenclavarían": "desenclavar",
    "desenclavarías": "desenclavar",
    "desenclavas": "desenclavar",
    "desenclavase": "desenclavar",
    "desenclavaseis": "desenclavar",
    "desenclavasen": "desenclavar",
    "desenclavases": "desenclavar",
    "desenclavaste": "desenclavar",
    "desenclavasteis": "desenclavar",
    "desenclave": "desenclavar",
    "desenclavemos": "desenclavar",
    "desenclaven": "desenclavar",
    "desenclaves": "desenclavar",
    "desenclavija": "desenclavijar",
    "desenclavijaba": "desenclavijar",
    "desenclavijabais": "desenclavijar",
    "desenclavijaban": "desenclavijar",
    "desenclavijabas": "desenclavijar",
    "desenclavijad": "desenclavijar",
    "desenclavijada": "desenclavijar",
    "desenclavijadas": "desenclavijar",
    "desenclavijado": "desenclavijar",
    "desenclavijados": "desenclavijar",
    "desenclavijamos": "desenclavijar",
    "desenclavijan": "desenclavijar",
    "desenclavijando": "desenclavijar",
    "desenclavijara": "desenclavijar",
    "desenclavijarais": "desenclavijar",
    "desenclavijaran": "desenclavijar",
    "desenclavijaras": "desenclavijar",
    "desenclavijare": "desenclavijar",
    "desenclavijareis": "desenclavijar",
    "desenclavijaremos": "desenclavijar",
    "desenclavijaren": "desenclavijar",
    "desenclavijares": "desenclavijar",
    "desenclavijaron": "desenclavijar",
    "desenclavijará": "desenclavijar",
    "desenclavijarán": "desenclavijar",
    "desenclavijarás": "desenclavijar",
    "desenclavijaré": "desenclavijar",
    "desenclavijaréis": "desenclavijar",
    "desenclavijaría": "desenclavijar",
    "desenclavijaríais": "desenclavijar",
    "desenclavijaríamos": "desenclavijar",
    "desenclavijarían": "desenclavijar",
    "desenclavijarías": "desenclavijar",
    "desenclavijas": "desenclavijar",
    "desenclavijase": "desenclavijar",
    "desenclavijaseis": "desenclavijar",
    "desenclavijasen": "desenclavijar",
    "desenclavijases": "desenclavijar",
    "desenclavijaste": "desenclavijar",
    "desenclavijasteis": "desenclavijar",
    "desenclavije": "desenclavijar",
    "desenclavijemos": "desenclavijar",
    "desenclavijen": "desenclavijar",
    "desenclavijes": "desenclavijar",
    "desenclavijo": "desenclavijar",
    "desenclavijábamos": "desenclavijar",
    "desenclavijáis": "desenclavijar",
    "desenclavijáramos": "desenclavijar",
    "desenclavijáremos": "desenclavijar",
    "desenclavijásemos": "desenclavijar",
    "desenclavijé": "desenclavijar",
    "desenclavijéis": "desenclavijar",
    "desenclavijó": "desenclavijar",
    "desenclavo": "desenclavar",
    "desenclavábamos": "desenclavar",
    "desenclaváis": "desenclavar",
    "desenclaváramos": "desenclavar",
    "desenclaváremos": "desenclavar",
    "desenclavásemos": "desenclavar",
    "desenclavé": "desenclavar",
    "desenclavéis": "desenclavar",
    "desenclavó": "desenclavar",
    "desencofra": "desencofrar",
    "desencofraba": "desencofrar",
    "desencofrabais": "desencofrar",
    "desencofraban": "desencofrar",
    "desencofrabas": "desencofrar",
    "desencofrad": "desencofrar",
    "desencofrada": "desencofrar",
    "desencofradas": "desencofrar",
    "desencofrado": "desencofrar",
    "desencofrados": "desencofrar",
    "desencoframos": "desencofrar",
    "desencofran": "desencofrar",
    "desencofrando": "desencofrar",
    "desencofrara": "desencofrar",
    "desencofrarais": "desencofrar",
    "desencofraran": "desencofrar",
    "desencofraras": "desencofrar",
    "desencofrare": "desencofrar",
    "desencofrareis": "desencofrar",
    "desencofraremos": "desencofrar",
    "desencofraren": "desencofrar",
    "desencofrares": "desencofrar",
    "desencofraron": "desencofrar",
    "desencofrará": "desencofrar",
    "desencofrarán": "desencofrar",
    "desencofrarás": "desencofrar",
    "desencofraré": "desencofrar",
    "desencofraréis": "desencofrar",
    "desencofraría": "desencofrar",
    "desencofraríais": "desencofrar",
    "desencofraríamos": "desencofrar",
    "desencofrarían": "desencofrar",
    "desencofrarías": "desencofrar",
    "desencofras": "desencofrar",
    "desencofrase": "desencofrar",
    "desencofraseis": "desencofrar",
    "desencofrasen": "desencofrar",
    "desencofrases": "desencofrar",
    "desencofraste": "desencofrar",
    "desencofrasteis": "desencofrar",
    "desencofre": "desencofrar",
    "desencofremos": "desencofrar",
    "desencofren": "desencofrar",
    "desencofres": "desencofrar",
    "desencofro": "desencofrar",
    "desencofrábamos": "desencofrar",
    "desencofráis": "desencofrar",
    "desencofráramos": "desencofrar",
    "desencofráremos": "desencofrar",
    "desencofrásemos": "desencofrar",
    "desencofré": "desencofrar",
    "desencofréis": "desencofrar",
    "desencofró": "desencofrar",
    "desencoge": "desencoger",
    "desencoged": "desencoger",
    "desencogemos": "desencoger",
    "desencogen": "desencoger",
    "desencogeremos": "desencoger",
    "desencogerá": "desencoger",
    "desencogerán": "desencoger",
    "desencogerás": "desencoger",
    "desencogeré": "desencoger",
    "desencogeréis": "desencoger",
    "desencogería": "desencoger",
    "desencogeríais": "desencoger",
    "desencogeríamos": "desencoger",
    "desencogerían": "desencoger",
    "desencogerías": "desencoger",
    "desencoges": "desencoger",
    "desencogida": "desencoger",
    "desencogidas": "desencoger",
    "desencogido": "desencoger",
    "desencogidos": "desencoger",
    "desencogiendo": "desencoger",
    "desencogiera": "desencoger",
    "desencogierais": "desencoger",
    "desencogieran": "desencoger",
    "desencogieras": "desencoger",
    "desencogiere": "desencoger",
    "desencogiereis": "desencoger",
    "desencogieren": "desencoger",
    "desencogieres": "desencoger",
    "desencogieron": "desencoger",
    "desencogiese": "desencoger",
    "desencogieseis": "desencoger",
    "desencogiesen": "desencoger",
    "desencogieses": "desencoger",
    "desencogimientos": "desencogimiento",
    "desencogimos": "desencoger",
    "desencogiste": "desencoger",
    "desencogisteis": "desencoger",
    "desencogiéramos": "desencoger",
    "desencogiéremos": "desencoger",
    "desencogiésemos": "desencoger",
    "desencogió": "desencoger",
    "desencogéis": "desencoger",
    "desencogí": "desencoger",
    "desencogía": "desencoger",
    "desencogíais": "desencoger",
    "desencogíamos": "desencoger",
    "desencogían": "desencoger",
    "desencogías": "desencoger",
    "desencoja": "desencoger",
    "desencojamos": "desencoger",
    "desencojan": "desencoger",
    "desencojas": "desencoger",
    "desencojo": "desencoger",
    "desencojáis": "desencoger",
    "desencola": "desencolar",
    "desencolaba": "desencolar",
    "desencolabais": "desencolar",
    "desencolaban": "desencolar",
    "desencolabas": "desencolar",
    "desencolad": "desencolar",
    "desencolada": "desencolar",
    "desencoladas": "desencolar",
    "desencolado": "desencolar",
    "desencolados": "desencolar",
    "desencoladuras": "desencoladura",
    "desencolamos": "desencolar",
    "desencolan": "desencolar",
    "desencolando": "desencolar",
    "desencolara": "desencolar",
    "desencolarais": "desencolar",
    "desencolaran": "desencolar",
    "desencolaras": "desencolar",
    "desencolare": "desencolar",
    "desencolareis": "desencolar",
    "desencolaremos": "desencolar",
    "desencolaren": "desencolar",
    "desencolares": "desencolar",
    "desencolaron": "desencolar",
    "desencolará": "desencolar",
    "desencolarán": "desencolar",
    "desencolarás": "desencolar",
    "desencolaré": "desencolar",
    "desencolaréis": "desencolar",
    "desencolaría": "desencolar",
    "desencolaríais": "desencolar",
    "desencolaríamos": "desencolar",
    "desencolarían": "desencolar",
    "desencolarías": "desencolar",
    "desencolas": "desencolar",
    "desencolase": "desencolar",
    "desencolaseis": "desencolar",
    "desencolasen": "desencolar",
    "desencolases": "desencolar",
    "desencolaste": "desencolar",
    "desencolasteis": "desencolar",
    "desencole": "desencolar",
    "desencolemos": "desencolar",
    "desencolen": "desencolar",
    "desencolerice": "desencolerizar",
    "desencolericemos": "desencolerizar",
    "desencolericen": "desencolerizar",
    "desencolerices": "desencolerizar",
    "desencolericé": "desencolerizar",
    "desencolericéis": "desencolerizar",
    "desencoleriza": "desencolerizar",
    "desencolerizaba": "desencolerizar",
    "desencolerizabais": "desencolerizar",
    "desencolerizaban": "desencolerizar",
    "desencolerizabas": "desencolerizar",
    "desencolerizad": "desencolerizar",
    "desencolerizada": "desencolerizar",
    "desencolerizadas": "desencolerizar",
    "desencolerizado": "desencolerizar",
    "desencolerizados": "desencolerizar",
    "desencolerizamientos": "desencolerizamiento",
    "desencolerizamos": "desencolerizar",
    "desencolerizan": "desencolerizar",
    "desencolerizando": "desencolerizar",
    "desencolerizara": "desencolerizar",
    "desencolerizarais": "desencolerizar",
    "desencolerizaran": "desencolerizar",
    "desencolerizaras": "desencolerizar",
    "desencolerizare": "desencolerizar",
    "desencolerizareis": "desencolerizar",
    "desencolerizaremos": "desencolerizar",
    "desencolerizaren": "desencolerizar",
    "desencolerizares": "desencolerizar",
    "desencolerizaron": "desencolerizar",
    "desencolerizará": "desencolerizar",
    "desencolerizarán": "desencolerizar",
    "desencolerizarás": "desencolerizar",
    "desencolerizaré": "desencolerizar",
    "desencolerizaréis": "desencolerizar",
    "desencolerizaría": "desencolerizar",
    "desencolerizaríais": "desencolerizar",
    "desencolerizaríamos": "desencolerizar",
    "desencolerizarían": "desencolerizar",
    "desencolerizarías": "desencolerizar",
    "desencolerizas": "desencolerizar",
    "desencolerizase": "desencolerizar",
    "desencolerizaseis": "desencolerizar",
    "desencolerizasen": "desencolerizar",
    "desencolerizases": "desencolerizar",
    "desencolerizaste": "desencolerizar",
    "desencolerizasteis": "desencolerizar",
    "desencolerizo": "desencolerizar",
    "desencolerizábamos": "desencolerizar",
    "desencolerizáis": "desencolerizar",
    "desencolerizáramos": "desencolerizar",
    "desencolerizáremos": "desencolerizar",
    "desencolerizásemos": "desencolerizar",
    "desencolerizó": "desencolerizar",
    "desencoles": "desencolar",
    "desencolo": "desencolar",
    "desencolábamos": "desencolar",
    "desencoláis": "desencolar",
    "desencoláramos": "desencolar",
    "desencoláremos": "desencolar",
    "desencolásemos": "desencolar",
    "desencolé": "desencolar",
    "desencoléis": "desencolar",
    "desencoló": "desencolar",
    "desencona": "desenconar",
    "desenconaba": "desenconar",
    "desenconabais": "desenconar",
    "desenconaban": "desenconar",
    "desenconabas": "desenconar",
    "desenconad": "desenconar",
    "desenconada": "desenconar",
    "desenconadas": "desenconar",
    "desenconado": "desenconar",
    "desenconados": "desenconar",
    "desenconamientos": "desenconamiento",
    "desenconamos": "desenconar",
    "desenconan": "desenconar",
    "desenconando": "desenconar",
    "desenconara": "desenconar",
    "desenconarais": "desenconar",
    "desenconaran": "desenconar",
    "desenconaras": "desenconar",
    "desenconare": "desenconar",
    "desenconareis": "desenconar",
    "desenconaremos": "desenconar",
    "desenconaren": "desenconar",
    "desenconares": "desenconar",
    "desenconaron": "desenconar",
    "desenconará": "desenconar",
    "desenconarán": "desenconar",
    "desenconarás": "desenconar",
    "desenconaré": "desenconar",
    "desenconaréis": "desenconar",
    "desenconaría": "desenconar",
    "desenconaríais": "desenconar",
    "desenconaríamos": "desenconar",
    "desenconarían": "desenconar",
    "desenconarías": "desenconar",
    "desenconas": "desenconar",
    "desenconase": "desenconar",
    "desenconaseis": "desenconar",
    "desenconasen": "desenconar",
    "desenconases": "desenconar",
    "desenconaste": "desenconar",
    "desenconasteis": "desenconar",
    "desencone": "desenconar",
    "desenconemos": "desenconar",
    "desenconen": "desenconar",
    "desencones": "desenconar",
    "desencono": "desenconar",
    "desenconos": "desencono",
    "desenconábamos": "desenconar",
    "desenconáis": "desenconar",
    "desenconáramos": "desenconar",
    "desenconáremos": "desenconar",
    "desenconásemos": "desenconar",
    "desenconé": "desenconar",
    "desenconéis": "desenconar",
    "desenconó": "desenconar",
    "desencordela": "desencordelar",
    "desencordelaba": "desencordelar",
    "desencordelabais": "desencordelar",
    "desencordelaban": "desencordelar",
    "desencordelabas": "desencordelar",
    "desencordelad": "desencordelar",
    "desencordelada": "desencordelar",
    "desencordeladas": "desencordelar",
    "desencordelado": "desencordelar",
    "desencordelados": "desencordelar",
    "desencordelamos": "desencordelar",
    "desencordelan": "desencordelar",
    "desencordelando": "desencordelar",
    "desencordelara": "desencordelar",
    "desencordelarais": "desencordelar",
    "desencordelaran": "desencordelar",
    "desencordelaras": "desencordelar",
    "desencordelare": "desencordelar",
    "desencordelareis": "desencordelar",
    "desencordelaremos": "desencordelar",
    "desencordelaren": "desencordelar",
    "desencordelares": "desencordelar",
    "desencordelaron": "desencordelar",
    "desencordelará": "desencordelar",
    "desencordelarán": "desencordelar",
    "desencordelarás": "desencordelar",
    "desencordelaré": "desencordelar",
    "desencordelaréis": "desencordelar",
    "desencordelaría": "desencordelar",
    "desencordelaríais": "desencordelar",
    "desencordelaríamos": "desencordelar",
    "desencordelarían": "desencordelar",
    "desencordelarías": "desencordelar",
    "desencordelas": "desencordelar",
    "desencordelase": "desencordelar",
    "desencordelaseis": "desencordelar",
    "desencordelasen": "desencordelar",
    "desencordelases": "desencordelar",
    "desencordelaste": "desencordelar",
    "desencordelasteis": "desencordelar",
    "desencordele": "desencordelar",
    "desencordelemos": "desencordelar",
    "desencordelen": "desencordelar",
    "desencordeles": "desencordelar",
    "desencordelo": "desencordelar",
    "desencordelábamos": "desencordelar",
    "desencordeláis": "desencordelar",
    "desencordeláramos": "desencordelar",
    "desencordeláremos": "desencordelar",
    "desencordelásemos": "desencordelar",
    "desencordelé": "desencordelar",
    "desencordeléis": "desencordelar",
    "desencordeló": "desencordelar",
    "desencorva": "desencorvar",
    "desencorvaba": "desencorvar",
    "desencorvabais": "desencorvar",
    "desencorvaban": "desencorvar",
    "desencorvabas": "desencorvar",
    "desencorvad": "desencorvar",
    "desencorvada": "desencorvar",
    "desencorvadas": "desencorvar",
    "desencorvado": "desencorvar",
    "desencorvados": "desencorvar",
    "desencorvamos": "desencorvar",
    "desencorvan": "desencorvar",
    "desencorvando": "desencorvar",
    "desencorvara": "desencorvar",
    "desencorvarais": "desencorvar",
    "desencorvaran": "desencorvar",
    "desencorvaras": "desencorvar",
    "desencorvare": "desencorvar",
    "desencorvareis": "desencorvar",
    "desencorvaremos": "desencorvar",
    "desencorvaren": "desencorvar",
    "desencorvares": "desencorvar",
    "desencorvaron": "desencorvar",
    "desencorvará": "desencorvar",
    "desencorvarán": "desencorvar",
    "desencorvarás": "desencorvar",
    "desencorvaré": "desencorvar",
    "desencorvaréis": "desencorvar",
    "desencorvaría": "desencorvar",
    "desencorvaríais": "desencorvar",
    "desencorvaríamos": "desencorvar",
    "desencorvarían": "desencorvar",
    "desencorvarías": "desencorvar",
    "desencorvas": "desencorvar",
    "desencorvase": "desencorvar",
    "desencorvaseis": "desencorvar",
    "desencorvasen": "desencorvar",
    "desencorvases": "desencorvar",
    "desencorvaste": "desencorvar",
    "desencorvasteis": "desencorvar",
    "desencorve": "desencorvar",
    "desencorvemos": "desencorvar",
    "desencorven": "desencorvar",
    "desencorves": "desencorvar",
    "desencorvo": "desencorvar",
    "desencorvábamos": "desencorvar",
    "desencorváis": "desencorvar",
    "desencorváramos": "desencorvar",
    "desencorváremos": "desencorvar",
    "desencorvásemos": "desencorvar",
    "desencorvé": "desencorvar",
    "desencorvéis": "desencorvar",
    "desencorvó": "desencorvar",
    "desencrespa": "desencrespar",
    "desencrespaba": "desencrespar",
    "desencrespabais": "desencrespar",
    "desencrespaban": "desencrespar",
    "desencrespabas": "desencrespar",
    "desencrespad": "desencrespar",
    "desencrespada": "desencrespar",
    "desencrespadas": "desencrespar",
    "desencrespado": "desencrespar",
    "desencrespados": "desencrespar",
    "desencrespamos": "desencrespar",
    "desencrespan": "desencrespar",
    "desencrespando": "desencrespar",
    "desencrespara": "desencrespar",
    "desencresparais": "desencrespar",
    "desencresparan": "desencrespar",
    "desencresparas": "desencrespar",
    "desencrespare": "desencrespar",
    "desencrespareis": "desencrespar",
    "desencresparemos": "desencrespar",
    "desencresparen": "desencrespar",
    "desencrespares": "desencrespar",
    "desencresparon": "desencrespar",
    "desencrespará": "desencrespar",
    "desencresparán": "desencrespar",
    "desencresparás": "desencrespar",
    "desencresparé": "desencrespar",
    "desencresparéis": "desencrespar",
    "desencresparía": "desencrespar",
    "desencresparíais": "desencrespar",
    "desencresparíamos": "desencrespar",
    "desencresparían": "desencrespar",
    "desencresparías": "desencrespar",
    "desencrespas": "desencrespar",
    "desencrespase": "desencrespar",
    "desencrespaseis": "desencrespar",
    "desencrespasen": "desencrespar",
    "desencrespases": "desencrespar",
    "desencrespaste": "desencrespar",
    "desencrespasteis": "desencrespar",
    "desencrespe": "desencrespar",
    "desencrespemos": "desencrespar",
    "desencrespen": "desencrespar",
    "desencrespes": "desencrespar",
    "desencrespo": "desencrespar",
    "desencrespábamos": "desencrespar",
    "desencrespáis": "desencrespar",
    "desencrespáramos": "desencrespar",
    "desencrespáremos": "desencrespar",
    "desencrespásemos": "desencrespar",
    "desencrespé": "desencrespar",
    "desencrespéis": "desencrespar",
    "desencrespó": "desencrespar",
    "desencuaderna": "desencuadernar",
    "desencuadernaba": "desencuadernar",
    "desencuadernabais": "desencuadernar",
    "desencuadernaban": "desencuadernar",
    "desencuadernabas": "desencuadernar",
    "desencuadernaciones": "desencuadernación",
    "desencuadernad": "desencuadernar",
    "desencuadernada": "desencuadernar",
    "desencuadernadas": "desencuadernar",
    "desencuadernado": "desencuadernar",
    "desencuadernados": "desencuadernar",
    "desencuadernamos": "desencuadernar",
    "desencuadernan": "desencuadernar",
    "desencuadernando": "desencuadernar",
    "desencuadernara": "desencuadernar",
    "desencuadernarais": "desencuadernar",
    "desencuadernaran": "desencuadernar",
    "desencuadernaras": "desencuadernar",
    "desencuadernare": "desencuadernar",
    "desencuadernareis": "desencuadernar",
    "desencuadernaremos": "desencuadernar",
    "desencuadernaren": "desencuadernar",
    "desencuadernares": "desencuadernar",
    "desencuadernaron": "desencuadernar",
    "desencuadernará": "desencuadernar",
    "desencuadernarán": "desencuadernar",
    "desencuadernarás": "desencuadernar",
    "desencuadernaré": "desencuadernar",
    "desencuadernaréis": "desencuadernar",
    "desencuadernaría": "desencuadernar",
    "desencuadernaríais": "desencuadernar",
    "desencuadernaríamos": "desencuadernar",
    "desencuadernarían": "desencuadernar",
    "desencuadernarías": "desencuadernar",
    "desencuadernas": "desencuadernar",
    "desencuadernase": "desencuadernar",
    "desencuadernaseis": "desencuadernar",
    "desencuadernasen": "desencuadernar",
    "desencuadernases": "desencuadernar",
    "desencuadernaste": "desencuadernar",
    "desencuadernasteis": "desencuadernar",
    "desencuaderne": "desencuadernar",
    "desencuadernemos": "desencuadernar",
    "desencuadernen": "desencuadernar",
    "desencuadernes": "desencuadernar",
    "desencuaderno": "desencuadernar",
    "desencuadernábamos": "desencuadernar",
    "desencuadernáis": "desencuadernar",
    "desencuadernáramos": "desencuadernar",
    "desencuadernáremos": "desencuadernar",
    "desencuadernásemos": "desencuadernar",
    "desencuaderné": "desencuadernar",
    "desencuadernéis": "desencuadernar",
    "desencuadernó": "desencuadernar",
    "desendemonia": "desendemoniar",
    "desendemoniaba": "desendemoniar",
    "desendemoniabais": "desendemoniar",
    "desendemoniaban": "desendemoniar",
    "desendemoniabas": "desendemoniar",
    "desendemoniad": "desendemoniar",
    "desendemoniada": "desendemoniar",
    "desendemoniadas": "desendemoniar",
    "desendemoniado": "desendemoniar",
    "desendemoniados": "desendemoniar",
    "desendemoniamos": "desendemoniar",
    "desendemonian": "desendemoniar",
    "desendemoniando": "desendemoniar",
    "desendemoniara": "desendemoniar",
    "desendemoniarais": "desendemoniar",
    "desendemoniaran": "desendemoniar",
    "desendemoniaras": "desendemoniar",
    "desendemoniare": "desendemoniar",
    "desendemoniareis": "desendemoniar",
    "desendemoniaremos": "desendemoniar",
    "desendemoniaren": "desendemoniar",
    "desendemoniares": "desendemoniar",
    "desendemoniaron": "desendemoniar",
    "desendemoniará": "desendemoniar",
    "desendemoniarán": "desendemoniar",
    "desendemoniarás": "desendemoniar",
    "desendemoniaré": "desendemoniar",
    "desendemoniaréis": "desendemoniar",
    "desendemoniaría": "desendemoniar",
    "desendemoniaríais": "desendemoniar",
    "desendemoniaríamos": "desendemoniar",
    "desendemoniarían": "desendemoniar",
    "desendemoniarías": "desendemoniar",
    "desendemonias": "desendemoniar",
    "desendemoniase": "desendemoniar",
    "desendemoniaseis": "desendemoniar",
    "desendemoniasen": "desendemoniar",
    "desendemoniases": "desendemoniar",
    "desendemoniaste": "desendemoniar",
    "desendemoniasteis": "desendemoniar",
    "desendemonie": "desendemoniar",
    "desendemoniemos": "desendemoniar",
    "desendemonien": "desendemoniar",
    "desendemonies": "desendemoniar",
    "desendemonio": "desendemoniar",
    "desendemoniábamos": "desendemoniar",
    "desendemoniáis": "desendemoniar",
    "desendemoniáramos": "desendemoniar",
    "desendemoniáremos": "desendemoniar",
    "desendemoniásemos": "desendemoniar",
    "desendemonié": "desendemoniar",
    "desendemoniéis": "desendemoniar",
    "desendemonió": "desendemoniar",
    "desendeuda": "desendeudar",
    "desendeudaba": "desendeudar",
    "desendeudabais": "desendeudar",
    "desendeudaban": "desendeudar",
    "desendeudabas": "desendeudar",
    "desendeudad": "desendeudar",
    "desendeudada": "desendeudar",
    "desendeudadas": "desendeudar",
    "desendeudado": "desendeudar",
    "desendeudados": "desendeudar",
    "desendeudamos": "desendeudar",
    "desendeudan": "desendeudar",
    "desendeudando": "desendeudar",
    "desendeudara": "desendeudar",
    "desendeudarais": "desendeudar",
    "desendeudaran": "desendeudar",
    "desendeudaras": "desendeudar",
    "desendeudare": "desendeudar",
    "desendeudareis": "desendeudar",
    "desendeudaremos": "desendeudar",
    "desendeudaren": "desendeudar",
    "desendeudares": "desendeudar",
    "desendeudaron": "desendeudar",
    "desendeudará": "desendeudar",
    "desendeudarán": "desendeudar",
    "desendeudarás": "desendeudar",
    "desendeudaré": "desendeudar",
    "desendeudaréis": "desendeudar",
    "desendeudaría": "desendeudar",
    "desendeudaríais": "desendeudar",
    "desendeudaríamos": "desendeudar",
    "desendeudarían": "desendeudar",
    "desendeudarías": "desendeudar",
    "desendeudas": "desendeudar",
    "desendeudase": "desendeudar",
    "desendeudaseis": "desendeudar",
    "desendeudasen": "desendeudar",
    "desendeudases": "desendeudar",
    "desendeudaste": "desendeudar",
    "desendeudasteis": "desendeudar",
    "desendeude": "desendeudar",
    "desendeudemos": "desendeudar",
    "desendeuden": "desendeudar",
    "desendeudes": "desendeudar",
    "desendeudo": "desendeudar",
    "desendeudábamos": "desendeudar",
    "desendeudáis": "desendeudar",
    "desendeudáramos": "desendeudar",
    "desendeudáremos": "desendeudar",
    "desendeudásemos": "desendeudar",
    "desendeudé": "desendeudar",
    "desendeudéis": "desendeudar",
    "desendeudó": "desendeudar",
    "desendiabla": "desendiablar",
    "desendiablaba": "desendiablar",
    "desendiablabais": "desendiablar",
    "desendiablaban": "desendiablar",
    "desendiablabas": "desendiablar",
    "desendiablad": "desendiablar",
    "desendiablada": "desendiablar",
    "desendiabladas": "desendiablar",
    "desendiablado": "desendiablar",
    "desendiablados": "desendiablar",
    "desendiablamos": "desendiablar",
    "desendiablan": "desendiablar",
    "desendiablando": "desendiablar",
    "desendiablara": "desendiablar",
    "desendiablarais": "desendiablar",
    "desendiablaran": "desendiablar",
    "desendiablaras": "desendiablar",
    "desendiablare": "desendiablar",
    "desendiablareis": "desendiablar",
    "desendiablaremos": "desendiablar",
    "desendiablaren": "desendiablar",
    "desendiablares": "desendiablar",
    "desendiablaron": "desendiablar",
    "desendiablará": "desendiablar",
    "desendiablarán": "desendiablar",
    "desendiablarás": "desendiablar",
    "desendiablaré": "desendiablar",
    "desendiablaréis": "desendiablar",
    "desendiablaría": "desendiablar",
    "desendiablaríais": "desendiablar",
    "desendiablaríamos": "desendiablar",
    "desendiablarían": "desendiablar",
    "desendiablarías": "desendiablar",
    "desendiablas": "desendiablar",
    "desendiablase": "desendiablar",
    "desendiablaseis": "desendiablar",
    "desendiablasen": "desendiablar",
    "desendiablases": "desendiablar",
    "desendiablaste": "desendiablar",
    "desendiablasteis": "desendiablar",
    "desendiable": "desendiablar",
    "desendiablemos": "desendiablar",
    "desendiablen": "desendiablar",
    "desendiables": "desendiablar",
    "desendiablo": "desendiablar",
    "desendiablábamos": "desendiablar",
    "desendiabláis": "desendiablar",
    "desendiabláramos": "desendiablar",
    "desendiabláremos": "desendiablar",
    "desendiablásemos": "desendiablar",
    "desendiablé": "desendiablar",
    "desendiabléis": "desendiablar",
    "desendiabló": "desendiablar",
    "desendiosa": "desendiosar",
    "desendiosaba": "desendiosar",
    "desendiosabais": "desendiosar",
    "desendiosaban": "desendiosar",
    "desendiosabas": "desendiosar",
    "desendiosad": "desendiosar",
    "desendiosada": "desendiosar",
    "desendiosadas": "desendiosar",
    "desendiosado": "desendiosar",
    "desendiosados": "desendiosar",
    "desendiosamos": "desendiosar",
    "desendiosan": "desendiosar",
    "desendiosando": "desendiosar",
    "desendiosara": "desendiosar",
    "desendiosarais": "desendiosar",
    "desendiosaran": "desendiosar",
    "desendiosaras": "desendiosar",
    "desendiosare": "desendiosar",
    "desendiosareis": "desendiosar",
    "desendiosaremos": "desendiosar",
    "desendiosaren": "desendiosar",
    "desendiosares": "desendiosar",
    "desendiosaron": "desendiosar",
    "desendiosará": "desendiosar",
    "desendiosarán": "desendiosar",
    "desendiosarás": "desendiosar",
    "desendiosaré": "desendiosar",
    "desendiosaréis": "desendiosar",
    "desendiosaría": "desendiosar",
    "desendiosaríais": "desendiosar",
    "desendiosaríamos": "desendiosar",
    "desendiosarían": "desendiosar",
    "desendiosarías": "desendiosar",
    "desendiosas": "desendiosar",
    "desendiosase": "desendiosar",
    "desendiosaseis": "desendiosar",
    "desendiosasen": "desendiosar",
    "desendiosases": "desendiosar",
    "desendiosaste": "desendiosar",
    "desendiosasteis": "desendiosar",
    "desendiose": "desendiosar",
    "desendiosemos": "desendiosar",
    "desendiosen": "desendiosar",
    "desendioses": "desendiosar",
    "desendioso": "desendiosar",
    "desendiosábamos": "desendiosar",
    "desendiosáis": "desendiosar",
    "desendiosáramos": "desendiosar",
    "desendiosáremos": "desendiosar",
    "desendiosásemos": "desendiosar",
    "desendiosé": "desendiosar",
    "desendioséis": "desendiosar",
    "desendiosó": "desendiosar",
    "desenfada": "desenfadar",
    "desenfadaba": "desenfadar",
    "desenfadabais": "desenfadar",
    "desenfadaban": "desenfadar",
    "desenfadabas": "desenfadar",
    "desenfadad": "desenfadar",
    "desenfadada": "desenfadar",
    "desenfadadas": "desenfadar",
    "desenfadado": "desenfadar",
    "desenfadados": "desenfadar",
    "desenfadamos": "desenfadar",
    "desenfadan": "desenfadar",
    "desenfadando": "desenfadar",
    "desenfadara": "desenfadar",
    "desenfadarais": "desenfadar",
    "desenfadaran": "desenfadar",
    "desenfadaras": "desenfadar",
    "desenfadare": "desenfadar",
    "desenfadareis": "desenfadar",
    "desenfadaremos": "desenfadar",
    "desenfadaren": "desenfadar",
    "desenfadares": "desenfadar",
    "desenfadaron": "desenfadar",
    "desenfadará": "desenfadar",
    "desenfadarán": "desenfadar",
    "desenfadarás": "desenfadar",
    "desenfadaré": "desenfadar",
    "desenfadaréis": "desenfadar",
    "desenfadaría": "desenfadar",
    "desenfadaríais": "desenfadar",
    "desenfadaríamos": "desenfadar",
    "desenfadarían": "desenfadar",
    "desenfadarías": "desenfadar",
    "desenfadas": "desenfadar",
    "desenfadase": "desenfadar",
    "desenfadaseis": "desenfadar",
    "desenfadasen": "desenfadar",
    "desenfadases": "desenfadar",
    "desenfadaste": "desenfadar",
    "desenfadasteis": "desenfadar",
    "desenfade": "desenfadar",
    "desenfademos": "desenfadar",
    "desenfaden": "desenfadar",
    "desenfades": "desenfadar",
    "desenfado": "desenfadar",
    "desenfados": "desenfado",
    "desenfadábamos": "desenfadar",
    "desenfadáis": "desenfadar",
    "desenfadáramos": "desenfadar",
    "desenfadáremos": "desenfadar",
    "desenfadásemos": "desenfadar",
    "desenfadé": "desenfadar",
    "desenfadéis": "desenfadar",
    "desenfadó": "desenfadar",
    "desenfarda": "desenfardar",
    "desenfardaba": "desenfardar",
    "desenfardabais": "desenfardar",
    "desenfardaban": "desenfardar",
    "desenfardabas": "desenfardar",
    "desenfardad": "desenfardar",
    "desenfardada": "desenfardar",
    "desenfardadas": "desenfardar",
    "desenfardado": "desenfardar",
    "desenfardados": "desenfardar",
    "desenfardamos": "desenfardar",
    "desenfardan": "desenfardar",
    "desenfardando": "desenfardar",
    "desenfardara": "desenfardar",
    "desenfardarais": "desenfardar",
    "desenfardaran": "desenfardar",
    "desenfardaras": "desenfardar",
    "desenfardare": "desenfardar",
    "desenfardareis": "desenfardar",
    "desenfardaremos": "desenfardar",
    "desenfardaren": "desenfardar",
    "desenfardares": "desenfardar",
    "desenfardaron": "desenfardar",
    "desenfardará": "desenfardar",
    "desenfardarán": "desenfardar",
    "desenfardarás": "desenfardar",
    "desenfardaré": "desenfardar",
    "desenfardaréis": "desenfardar",
    "desenfardaría": "desenfardar",
    "desenfardaríais": "desenfardar",
    "desenfardaríamos": "desenfardar",
    "desenfardarían": "desenfardar",
    "desenfardarías": "desenfardar",
    "desenfardas": "desenfardar",
    "desenfardase": "desenfardar",
    "desenfardaseis": "desenfardar",
    "desenfardasen": "desenfardar",
    "desenfardases": "desenfardar",
    "desenfardaste": "desenfardar",
    "desenfardasteis": "desenfardar",
    "desenfarde": "desenfardar",
    "desenfardela": "desenfardelar",
    "desenfardelaba": "desenfardelar",
    "desenfardelabais": "desenfardelar",
    "desenfardelaban": "desenfardelar",
    "desenfardelabas": "desenfardelar",
    "desenfardelad": "desenfardelar",
    "desenfardelada": "desenfardelar",
    "desenfardeladas": "desenfardelar",
    "desenfardelado": "desenfardelar",
    "desenfardelados": "desenfardelar",
    "desenfardelamos": "desenfardelar",
    "desenfardelan": "desenfardelar",
    "desenfardelando": "desenfardelar",
    "desenfardelara": "desenfardelar",
    "desenfardelarais": "desenfardelar",
    "desenfardelaran": "desenfardelar",
    "desenfardelaras": "desenfardelar",
    "desenfardelare": "desenfardelar",
    "desenfardelareis": "desenfardelar",
    "desenfardelaremos": "desenfardelar",
    "desenfardelaren": "desenfardelar",
    "desenfardelares": "desenfardelar",
    "desenfardelaron": "desenfardelar",
    "desenfardelará": "desenfardelar",
    "desenfardelarán": "desenfardelar",
    "desenfardelarás": "desenfardelar",
    "desenfardelaré": "desenfardelar",
    "desenfardelaréis": "desenfardelar",
    "desenfardelaría": "desenfardelar",
    "desenfardelaríais": "desenfardelar",
    "desenfardelaríamos": "desenfardelar",
    "desenfardelarían": "desenfardelar",
    "desenfardelarías": "desenfardelar",
    "desenfardelas": "desenfardelar",
    "desenfardelase": "desenfardelar",
    "desenfardelaseis": "desenfardelar",
    "desenfardelasen": "desenfardelar",
    "desenfardelases": "desenfardelar",
    "desenfardelaste": "desenfardelar",
    "desenfardelasteis": "desenfardelar",
    "desenfardele": "desenfardelar",
    "desenfardelemos": "desenfardelar",
    "desenfardelen": "desenfardelar",
    "desenfardeles": "desenfardelar",
    "desenfardelo": "desenfardelar",
    "desenfardelábamos": "desenfardelar",
    "desenfardeláis": "desenfardelar",
    "desenfardeláramos": "desenfardelar",
    "desenfardeláremos": "desenfardelar",
    "desenfardelásemos": "desenfardelar",
    "desenfardelé": "desenfardelar",
    "desenfardeléis": "desenfardelar",
    "desenfardeló": "desenfardelar",
    "desenfardemos": "desenfardar",
    "desenfarden": "desenfardar",
    "desenfardes": "desenfardar",
    "desenfardo": "desenfardar",
    "desenfardábamos": "desenfardar",
    "desenfardáis": "desenfardar",
    "desenfardáramos": "desenfardar",
    "desenfardáremos": "desenfardar",
    "desenfardásemos": "desenfardar",
    "desenfardé": "desenfardar",
    "desenfardéis": "desenfardar",
    "desenfardó": "desenfardar",
    "desenfila": "desenfilar",
    "desenfilaba": "desenfilar",
    "desenfilabais": "desenfilar",
    "desenfilaban": "desenfilar",
    "desenfilabas": "desenfilar",
    "desenfilad": "desenfilar",
    "desenfilada": "desenfilar",
    "desenfiladas": "desenfilar",
    "desenfilado": "desenfilar",
    "desenfilados": "desenfilar",
    "desenfilamientos": "desenfilamiento",
    "desenfilamos": "desenfilar",
    "desenfilan": "desenfilar",
    "desenfilando": "desenfilar",
    "desenfilara": "desenfilar",
    "desenfilarais": "desenfilar",
    "desenfilaran": "desenfilar",
    "desenfilaras": "desenfilar",
    "desenfilare": "desenfilar",
    "desenfilareis": "desenfilar",
    "desenfilaremos": "desenfilar",
    "desenfilaren": "desenfilar",
    "desenfilares": "desenfilar",
    "desenfilaron": "desenfilar",
    "desenfilará": "desenfilar",
    "desenfilarán": "desenfilar",
    "desenfilarás": "desenfilar",
    "desenfilaré": "desenfilar",
    "desenfilaréis": "desenfilar",
    "desenfilaría": "desenfilar",
    "desenfilaríais": "desenfilar",
    "desenfilaríamos": "desenfilar",
    "desenfilarían": "desenfilar",
    "desenfilarías": "desenfilar",
    "desenfilas": "desenfilar",
    "desenfilase": "desenfilar",
    "desenfilaseis": "desenfilar",
    "desenfilasen": "desenfilar",
    "desenfilases": "desenfilar",
    "desenfilaste": "desenfilar",
    "desenfilasteis": "desenfilar",
    "desenfile": "desenfilar",
    "desenfilemos": "desenfilar",
    "desenfilen": "desenfilar",
    "desenfiles": "desenfilar",
    "desenfilo": "desenfilar",
    "desenfilábamos": "desenfilar",
    "desenfiláis": "desenfilar",
    "desenfiláramos": "desenfilar",
    "desenfiláremos": "desenfilar",
    "desenfilásemos": "desenfilar",
    "desenfilé": "desenfilar",
    "desenfiléis": "desenfilar",
    "desenfiló": "desenfilar",
    "desenfoca": "desenfocar",
    "desenfocaba": "desenfocar",
    "desenfocabais": "desenfocar",
    "desenfocaban": "desenfocar",
    "desenfocabas": "desenfocar",
    "desenfocad": "desenfocar",
    "desenfocada": "desenfocar",
    "desenfocadas": "desenfocar",
    "desenfocado": "desenfocar",
    "desenfocados": "desenfocar",
    "desenfocamos": "desenfocar",
    "desenfocan": "desenfocar",
    "desenfocando": "desenfocar",
    "desenfocara": "desenfocar",
    "desenfocarais": "desenfocar",
    "desenfocaran": "desenfocar",
    "desenfocaras": "desenfocar",
    "desenfocare": "desenfocar",
    "desenfocareis": "desenfocar",
    "desenfocaremos": "desenfocar",
    "desenfocaren": "desenfocar",
    "desenfocares": "desenfocar",
    "desenfocaron": "desenfocar",
    "desenfocará": "desenfocar",
    "desenfocarán": "desenfocar",
    "desenfocarás": "desenfocar",
    "desenfocaré": "desenfocar",
    "desenfocaréis": "desenfocar",
    "desenfocaría": "desenfocar",
    "desenfocaríais": "desenfocar",
    "desenfocaríamos": "desenfocar",
    "desenfocarían": "desenfocar",
    "desenfocarías": "desenfocar",
    "desenfocas": "desenfocar",
    "desenfocase": "desenfocar",
    "desenfocaseis": "desenfocar",
    "desenfocasen": "desenfocar",
    "desenfocases": "desenfocar",
    "desenfocaste": "desenfocar",
    "desenfocasteis": "desenfocar",
    "desenfoco": "desenfocar",
    "desenfocábamos": "desenfocar",
    "desenfocáis": "desenfocar",
    "desenfocáramos": "desenfocar",
    "desenfocáremos": "desenfocar",
    "desenfocásemos": "desenfocar",
    "desenfocó": "desenfocar",
    "desenfoque": "desenfocar",
    "desenfoquemos": "desenfocar",
    "desenfoquen": "desenfocar",
    "desenfoques": "desenfoque",
    "desenfoqué": "desenfocar",
    "desenfoquéis": "desenfocar",
    "desenfrena": "desenfrenar",
    "desenfrenaba": "desenfrenar",
    "desenfrenabais": "desenfrenar",
    "desenfrenaban": "desenfrenar",
    "desenfrenabas": "desenfrenar",
    "desenfrenad": "desenfrenar",
    "desenfrenada": "desenfrenar",
    "desenfrenadas": "desenfrenar",
    "desenfrenado": "desenfrenar",
    "desenfrenados": "desenfrenar",
    "desenfrenamos": "desenfrenar",
    "desenfrenan": "desenfrenar",
    "desenfrenando": "desenfrenar",
    "desenfrenara": "desenfrenar",
    "desenfrenarais": "desenfrenar",
    "desenfrenaran": "desenfrenar",
    "desenfrenaras": "desenfrenar",
    "desenfrenare": "desenfrenar",
    "desenfrenareis": "desenfrenar",
    "desenfrenaremos": "desenfrenar",
    "desenfrenaren": "desenfrenar",
    "desenfrenares": "desenfrenar",
    "desenfrenaron": "desenfrenar",
    "desenfrenará": "desenfrenar",
    "desenfrenarán": "desenfrenar",
    "desenfrenarás": "desenfrenar",
    "desenfrenaré": "desenfrenar",
    "desenfrenaréis": "desenfrenar",
    "desenfrenaría": "desenfrenar",
    "desenfrenaríais": "desenfrenar",
    "desenfrenaríamos": "desenfrenar",
    "desenfrenarían": "desenfrenar",
    "desenfrenarías": "desenfrenar",
    "desenfrenas": "desenfrenar",
    "desenfrenase": "desenfrenar",
    "desenfrenaseis": "desenfrenar",
    "desenfrenasen": "desenfrenar",
    "desenfrenases": "desenfrenar",
    "desenfrenaste": "desenfrenar",
    "desenfrenasteis": "desenfrenar",
    "desenfrene": "desenfrenar",
    "desenfrenemos": "desenfrenar",
    "desenfrenen": "desenfrenar",
    "desenfrenes": "desenfrenar",
    "desenfreno": "desenfrenar",
    "desenfrenos": "desenfreno",
    "desenfrenábamos": "desenfrenar",
    "desenfrenáis": "desenfrenar",
    "desenfrenáramos": "desenfrenar",
    "desenfrenáremos": "desenfrenar",
    "desenfrenásemos": "desenfrenar",
    "desenfrené": "desenfrenar",
    "desenfrenéis": "desenfrenar",
    "desenfrenó": "desenfrenar",
    "desenfunda": "desenfundar",
    "desenfundaba": "desenfundar",
    "desenfundabais": "desenfundar",
    "desenfundaban": "desenfundar",
    "desenfundabas": "desenfundar",
    "desenfundad": "desenfundar",
    "desenfundada": "desenfundar",
    "desenfundadas": "desenfundar",
    "desenfundado": "desenfundar",
    "desenfundados": "desenfundar",
    "desenfundamos": "desenfundar",
    "desenfundan": "desenfundar",
    "desenfundando": "desenfundar",
    "desenfundara": "desenfundar",
    "desenfundarais": "desenfundar",
    "desenfundaran": "desenfundar",
    "desenfundaras": "desenfundar",
    "desenfundare": "desenfundar",
    "desenfundareis": "desenfundar",
    "desenfundaremos": "desenfundar",
    "desenfundaren": "desenfundar",
    "desenfundares": "desenfundar",
    "desenfundaron": "desenfundar",
    "desenfundará": "desenfundar",
    "desenfundarán": "desenfundar",
    "desenfundarás": "desenfundar",
    "desenfundaré": "desenfundar",
    "desenfundaréis": "desenfundar",
    "desenfundaría": "desenfundar",
    "desenfundaríais": "desenfundar",
    "desenfundaríamos": "desenfundar",
    "desenfundarían": "desenfundar",
    "desenfundarías": "desenfundar",
    "desenfundas": "desenfundar",
    "desenfundase": "desenfundar",
    "desenfundaseis": "desenfundar",
    "desenfundasen": "desenfundar",
    "desenfundases": "desenfundar",
    "desenfundaste": "desenfundar",
    "desenfundasteis": "desenfundar",
    "desenfunde": "desenfundar",
    "desenfundemos": "desenfundar",
    "desenfunden": "desenfundar",
    "desenfundes": "desenfundar",
    "desenfundo": "desenfundar",
    "desenfundábamos": "desenfundar",
    "desenfundáis": "desenfundar",
    "desenfundáramos": "desenfundar",
    "desenfundáremos": "desenfundar",
    "desenfundásemos": "desenfundar",
    "desenfundé": "desenfundar",
    "desenfundéis": "desenfundar",
    "desenfundó": "desenfundar",
    "desenfurece": "desenfurecer",
    "desenfureced": "desenfurecer",
    "desenfurecemos": "desenfurecer",
    "desenfurecen": "desenfurecer",
    "desenfureceremos": "desenfurecer",
    "desenfurecerá": "desenfurecer",
    "desenfurecerán": "desenfurecer",
    "desenfurecerás": "desenfurecer",
    "desenfureceré": "desenfurecer",
    "desenfureceréis": "desenfurecer",
    "desenfurecería": "desenfurecer",
    "desenfureceríais": "desenfurecer",
    "desenfureceríamos": "desenfurecer",
    "desenfurecerían": "desenfurecer",
    "desenfurecerías": "desenfurecer",
    "desenfureces": "desenfurecer",
    "desenfurecida": "desenfurecer",
    "desenfurecidas": "desenfurecer",
    "desenfurecido": "desenfurecer",
    "desenfurecidos": "desenfurecer",
    "desenfureciendo": "desenfurecer",
    "desenfureciera": "desenfurecer",
    "desenfurecierais": "desenfurecer",
    "desenfurecieran": "desenfurecer",
    "desenfurecieras": "desenfurecer",
    "desenfureciere": "desenfurecer",
    "desenfureciereis": "desenfurecer",
    "desenfurecieren": "desenfurecer",
    "desenfurecieres": "desenfurecer",
    "desenfurecieron": "desenfurecer",
    "desenfureciese": "desenfurecer",
    "desenfurecieseis": "desenfurecer",
    "desenfureciesen": "desenfurecer",
    "desenfurecieses": "desenfurecer",
    "desenfurecimientos": "desenfurecimiento",
    "desenfurecimos": "desenfurecer",
    "desenfureciste": "desenfurecer",
    "desenfurecisteis": "desenfurecer",
    "desenfureciéramos": "desenfurecer",
    "desenfureciéremos": "desenfurecer",
    "desenfureciésemos": "desenfurecer",
    "desenfureció": "desenfurecer",
    "desenfurecéis": "desenfurecer",
    "desenfurecí": "desenfurecer",
    "desenfurecía": "desenfurecer",
    "desenfurecíais": "desenfurecer",
    "desenfurecíamos": "desenfurecer",
    "desenfurecían": "desenfurecer",
    "desenfurecías": "desenfurecer",
    "desenfurezca": "desenfurecer",
    "desenfurezcamos": "desenfurecer",
    "desenfurezcan": "desenfurecer",
    "desenfurezcas": "desenfurecer",
    "desenfurezco": "desenfurecer",
    "desenfurezcáis": "desenfurecer",
    "desenfurruñamientos": "desenfurruñamiento",
    "desengancha": "desenganchar",
    "desenganchaba": "desenganchar",
    "desenganchabais": "desenganchar",
    "desenganchaban": "desenganchar",
    "desenganchabas": "desenganchar",
    "desenganchad": "desenganchar",
    "desenganchada": "desenganchar",
    "desenganchadas": "desenganchar",
    "desenganchado": "desenganchar",
    "desenganchados": "desenganchar",
    "desenganchamos": "desenganchar",
    "desenganchan": "desenganchar",
    "desenganchando": "desenganchar",
    "desenganchara": "desenganchar",
    "desengancharais": "desenganchar",
    "desengancharan": "desenganchar",
    "desengancharas": "desenganchar",
    "desenganchare": "desenganchar",
    "desenganchareis": "desenganchar",
    "desengancharemos": "desenganchar",
    "desengancharen": "desenganchar",
    "desenganchares": "desenganchar",
    "desengancharon": "desenganchar",
    "desenganchará": "desenganchar",
    "desengancharán": "desenganchar",
    "desengancharás": "desenganchar",
    "desengancharé": "desenganchar",
    "desengancharéis": "desenganchar",
    "desengancharía": "desenganchar",
    "desengancharíais": "desenganchar",
    "desengancharíamos": "desenganchar",
    "desengancharían": "desenganchar",
    "desengancharías": "desenganchar",
    "desenganchas": "desenganchar",
    "desenganchase": "desenganchar",
    "desenganchaseis": "desenganchar",
    "desenganchasen": "desenganchar",
    "desenganchases": "desenganchar",
    "desenganchaste": "desenganchar",
    "desenganchasteis": "desenganchar",
    "desenganche": "desenganchar",
    "desenganchemos": "desenganchar",
    "desenganchen": "desenganchar",
    "desenganches": "desenganche",
    "desengancho": "desenganchar",
    "desenganchábamos": "desenganchar",
    "desengancháis": "desenganchar",
    "desengancháramos": "desenganchar",
    "desengancháremos": "desenganchar",
    "desenganchásemos": "desenganchar",
    "desenganché": "desenganchar",
    "desenganchéis": "desenganchar",
    "desenganchó": "desenganchar",
    "desengarce": "desengarzar",
    "desengarcemos": "desengarzar",
    "desengarcen": "desengarzar",
    "desengarces": "desengarzar",
    "desengarcé": "desengarzar",
    "desengarcéis": "desengarzar",
    "desengarza": "desengarzar",
    "desengarzaba": "desengarzar",
    "desengarzabais": "desengarzar",
    "desengarzaban": "desengarzar",
    "desengarzabas": "desengarzar",
    "desengarzad": "desengarzar",
    "desengarzada": "desengarzar",
    "desengarzadas": "desengarzar",
    "desengarzado": "desengarzar",
    "desengarzados": "desengarzar",
    "desengarzamientos": "desengarzamiento",
    "desengarzamos": "desengarzar",
    "desengarzan": "desengarzar",
    "desengarzando": "desengarzar",
    "desengarzara": "desengarzar",
    "desengarzarais": "desengarzar",
    "desengarzaran": "desengarzar",
    "desengarzaras": "desengarzar",
    "desengarzare": "desengarzar",
    "desengarzareis": "desengarzar",
    "desengarzaremos": "desengarzar",
    "desengarzaren": "desengarzar",
    "desengarzares": "desengarzar",
    "desengarzaron": "desengarzar",
    "desengarzará": "desengarzar",
    "desengarzarán": "desengarzar",
    "desengarzarás": "desengarzar",
    "desengarzaré": "desengarzar",
    "desengarzaréis": "desengarzar",
    "desengarzaría": "desengarzar",
    "desengarzaríais": "desengarzar",
    "desengarzaríamos": "desengarzar",
    "desengarzarían": "desengarzar",
    "desengarzarías": "desengarzar",
    "desengarzas": "desengarzar",
    "desengarzase": "desengarzar",
    "desengarzaseis": "desengarzar",
    "desengarzasen": "desengarzar",
    "desengarzases": "desengarzar",
    "desengarzaste": "desengarzar",
    "desengarzasteis": "desengarzar",
    "desengarzo": "desengarzar",
    "desengarzábamos": "desengarzar",
    "desengarzáis": "desengarzar",
    "desengarzáramos": "desengarzar",
    "desengarzáremos": "desengarzar",
    "desengarzásemos": "desengarzar",
    "desengarzó": "desengarzar",
    "desengasta": "desengastar",
    "desengastaba": "desengastar",
    "desengastabais": "desengastar",
    "desengastaban": "desengastar",
    "desengastabas": "desengastar",
    "desengastad": "desengastar",
    "desengastada": "desengastar",
    "desengastadas": "desengastar",
    "desengastado": "desengastar",
    "desengastados": "desengastar",
    "desengastamos": "desengastar",
    "desengastan": "desengastar",
    "desengastando": "desengastar",
    "desengastara": "desengastar",
    "desengastarais": "desengastar",
    "desengastaran": "desengastar",
    "desengastaras": "desengastar",
    "desengastare": "desengastar",
    "desengastareis": "desengastar",
    "desengastaremos": "desengastar",
    "desengastaren": "desengastar",
    "desengastares": "desengastar",
    "desengastaron": "desengastar",
    "desengastará": "desengastar",
    "desengastarán": "desengastar",
    "desengastarás": "desengastar",
    "desengastaré": "desengastar",
    "desengastaréis": "desengastar",
    "desengastaría": "desengastar",
    "desengastaríais": "desengastar",
    "desengastaríamos": "desengastar",
    "desengastarían": "desengastar",
    "desengastarías": "desengastar",
    "desengastas": "desengastar",
    "desengastase": "desengastar",
    "desengastaseis": "desengastar",
    "desengastasen": "desengastar",
    "desengastases": "desengastar",
    "desengastaste": "desengastar",
    "desengastasteis": "desengastar",
    "desengaste": "desengastar",
    "desengastemos": "desengastar",
    "desengasten": "desengastar",
    "desengastes": "desengastar",
    "desengasto": "desengastar",
    "desengastábamos": "desengastar",
    "desengastáis": "desengastar",
    "desengastáramos": "desengastar",
    "desengastáremos": "desengastar",
    "desengastásemos": "desengastar",
    "desengasté": "desengastar",
    "desengastéis": "desengastar",
    "desengastó": "desengastar",
    "desengaña": "desengañar",
    "desengañaba": "desengañar",
    "desengañabais": "desengañar",
    "desengañaban": "desengañar",
    "desengañabas": "desengañar",
    "desengañad": "desengañar",
    "desengañada": "desengañar",
    "desengañadas": "desengañar",
    "desengañado": "desengañar",
    "desengañados": "desengañar",
    "desengañamos": "desengañar",
    "desengañan": "desengañar",
    "desengañando": "desengañar",
    "desengañara": "desengañar",
    "desengañarais": "desengañar",
    "desengañaran": "desengañar",
    "desengañaras": "desengañar",
    "desengañare": "desengañar",
    "desengañareis": "desengañar",
    "desengañaremos": "desengañar",
    "desengañaren": "desengañar",
    "desengañares": "desengañar",
    "desengañaron": "desengañar",
    "desengañará": "desengañar",
    "desengañarán": "desengañar",
    "desengañarás": "desengañar",
    "desengañaré": "desengañar",
    "desengañaréis": "desengañar",
    "desengañaría": "desengañar",
    "desengañaríais": "desengañar",
    "desengañaríamos": "desengañar",
    "desengañarían": "desengañar",
    "desengañarías": "desengañar",
    "desengañas": "desengañar",
    "desengañase": "desengañar",
    "desengañaseis": "desengañar",
    "desengañasen": "desengañar",
    "desengañases": "desengañar",
    "desengañaste": "desengañar",
    "desengañasteis": "desengañar",
    "desengañe": "desengañar",
    "desengañemos": "desengañar",
    "desengañen": "desengañar",
    "desengañes": "desengañar",
    "desengaño": "desengañar",
    "desengaños": "desengaño",
    "desengañábamos": "desengañar",
    "desengañáis": "desengañar",
    "desengañáramos": "desengañar",
    "desengañáremos": "desengañar",
    "desengañásemos": "desengañar",
    "desengañé": "desengañar",
    "desengañéis": "desengañar",
    "desengañó": "desengañar",
    "desengoma": "desengomar",
    "desengomaba": "desengomar",
    "desengomabais": "desengomar",
    "desengomaban": "desengomar",
    "desengomabas": "desengomar",
    "desengomad": "desengomar",
    "desengomada": "desengomar",
    "desengomadas": "desengomar",
    "desengomado": "desengomar",
    "desengomados": "desengomar",
    "desengomamos": "desengomar",
    "desengoman": "desengomar",
    "desengomando": "desengomar",
    "desengomara": "desengomar",
    "desengomarais": "desengomar",
    "desengomaran": "desengomar",
    "desengomaras": "desengomar",
    "desengomare": "desengomar",
    "desengomareis": "desengomar",
    "desengomaremos": "desengomar",
    "desengomaren": "desengomar",
    "desengomares": "desengomar",
    "desengomaron": "desengomar",
    "desengomará": "desengomar",
    "desengomarán": "desengomar",
    "desengomarás": "desengomar",
    "desengomaré": "desengomar",
    "desengomaréis": "desengomar",
    "desengomaría": "desengomar",
    "desengomaríais": "desengomar",
    "desengomaríamos": "desengomar",
    "desengomarían": "desengomar",
    "desengomarías": "desengomar",
    "desengomas": "desengomar",
    "desengomase": "desengomar",
    "desengomaseis": "desengomar",
    "desengomasen": "desengomar",
    "desengomases": "desengomar",
    "desengomaste": "desengomar",
    "desengomasteis": "desengomar",
    "desengome": "desengomar",
    "desengomemos": "desengomar",
    "desengomen": "desengomar",
    "desengomes": "desengomar",
    "desengomo": "desengomar",
    "desengomábamos": "desengomar",
    "desengomáis": "desengomar",
    "desengomáramos": "desengomar",
    "desengomáremos": "desengomar",
    "desengomásemos": "desengomar",
    "desengomé": "desengomar",
    "desengoméis": "desengomar",
    "desengomó": "desengomar",
    "desengozna": "desengoznar",
    "desengoznaba": "desengoznar",
    "desengoznabais": "desengoznar",
    "desengoznaban": "desengoznar",
    "desengoznabas": "desengoznar",
    "desengoznad": "desengoznar",
    "desengoznada": "desengoznar",
    "desengoznadas": "desengoznar",
    "desengoznado": "desengoznar",
    "desengoznados": "desengoznar",
    "desengoznamos": "desengoznar",
    "desengoznan": "desengoznar",
    "desengoznando": "desengoznar",
    "desengoznara": "desengoznar",
    "desengoznarais": "desengoznar",
    "desengoznaran": "desengoznar",
    "desengoznaras": "desengoznar",
    "desengoznare": "desengoznar",
    "desengoznareis": "desengoznar",
    "desengoznaremos": "desengoznar",
    "desengoznaren": "desengoznar",
    "desengoznares": "desengoznar",
    "desengoznaron": "desengoznar",
    "desengoznará": "desengoznar",
    "desengoznarán": "desengoznar",
    "desengoznarás": "desengoznar",
    "desengoznaré": "desengoznar",
    "desengoznaréis": "desengoznar",
    "desengoznaría": "desengoznar",
    "desengoznaríais": "desengoznar",
    "desengoznaríamos": "desengoznar",
    "desengoznarían": "desengoznar",
    "desengoznarías": "desengoznar",
    "desengoznas": "desengoznar",
    "desengoznase": "desengoznar",
    "desengoznaseis": "desengoznar",
    "desengoznasen": "desengoznar",
    "desengoznases": "desengoznar",
    "desengoznaste": "desengoznar",
    "desengoznasteis": "desengoznar",
    "desengozne": "desengoznar",
    "desengoznemos": "desengoznar",
    "desengoznen": "desengoznar",
    "desengoznes": "desengoznar",
    "desengozno": "desengoznar",
    "desengoznábamos": "desengoznar",
    "desengoznáis": "desengoznar",
    "desengoznáramos": "desengoznar",
    "desengoznáremos": "desengoznar",
    "desengoznásemos": "desengoznar",
    "desengozné": "desengoznar",
    "desengoznéis": "desengoznar",
    "desengoznó": "desengoznar",
    "desengrana": "desengranar",
    "desengranaba": "desengranar",
    "desengranabais": "desengranar",
    "desengranaban": "desengranar",
    "desengranabas": "desengranar",
    "desengranad": "desengranar",
    "desengranada": "desengranar",
    "desengranadas": "desengranar",
    "desengranado": "desengranar",
    "desengranados": "desengranar",
    "desengranamos": "desengranar",
    "desengranan": "desengranar",
    "desengranando": "desengranar",
    "desengranara": "desengranar",
    "desengranarais": "desengranar",
    "desengranaran": "desengranar",
    "desengranaras": "desengranar",
    "desengranare": "desengranar",
    "desengranareis": "desengranar",
    "desengranaremos": "desengranar",
    "desengranaren": "desengranar",
    "desengranares": "desengranar",
    "desengranaron": "desengranar",
    "desengranará": "desengranar",
    "desengranarán": "desengranar",
    "desengranarás": "desengranar",
    "desengranaré": "desengranar",
    "desengranaréis": "desengranar",
    "desengranaría": "desengranar",
    "desengranaríais": "desengranar",
    "desengranaríamos": "desengranar",
    "desengranarían": "desengranar",
    "desengranarías": "desengranar",
    "desengranas": "desengranar",
    "desengranase": "desengranar",
    "desengranaseis": "desengranar",
    "desengranasen": "desengranar",
    "desengranases": "desengranar",
    "desengranaste": "desengranar",
    "desengranasteis": "desengranar",
    "desengrane": "desengranar",
    "desengranemos": "desengranar",
    "desengranen": "desengranar",
    "desengranes": "desengranar",
    "desengrano": "desengranar",
    "desengranábamos": "desengranar",
    "desengranáis": "desengranar",
    "desengranáramos": "desengranar",
    "desengranáremos": "desengranar",
    "desengranásemos": "desengranar",
    "desengrané": "desengranar",
    "desengranéis": "desengranar",
    "desengranó": "desengranar",
    "desengrasa": "desengrasar",
    "desengrasaba": "desengrasar",
    "desengrasabais": "desengrasar",
    "desengrasaban": "desengrasar",
    "desengrasabas": "desengrasar",
    "desengrasad": "desengrasar",
    "desengrasada": "desengrasar",
    "desengrasadas": "desengrasar",
    "desengrasado": "desengrasar",
    "desengrasados": "desengrasar",
    "desengrasamos": "desengrasar",
    "desengrasan": "desengrasar",
    "desengrasando": "desengrasar",
    "desengrasantes": "desengrasante",
    "desengrasara": "desengrasar",
    "desengrasarais": "desengrasar",
    "desengrasaran": "desengrasar",
    "desengrasaras": "desengrasar",
    "desengrasare": "desengrasar",
    "desengrasareis": "desengrasar",
    "desengrasaremos": "desengrasar",
    "desengrasaren": "desengrasar",
    "desengrasares": "desengrasar",
    "desengrasaron": "desengrasar",
    "desengrasará": "desengrasar",
    "desengrasarán": "desengrasar",
    "desengrasarás": "desengrasar",
    "desengrasaré": "desengrasar",
    "desengrasaréis": "desengrasar",
    "desengrasaría": "desengrasar",
    "desengrasaríais": "desengrasar",
    "desengrasaríamos": "desengrasar",
    "desengrasarían": "desengrasar",
    "desengrasarías": "desengrasar",
    "desengrasas": "desengrasar",
    "desengrasase": "desengrasar",
    "desengrasaseis": "desengrasar",
    "desengrasasen": "desengrasar",
    "desengrasases": "desengrasar",
    "desengrasaste": "desengrasar",
    "desengrasasteis": "desengrasar",
    "desengrase": "desengrasar",
    "desengrasemos": "desengrasar",
    "desengrasen": "desengrasar",
    "desengrases": "desengrase",
    "desengraso": "desengrasar",
    "desengrasábamos": "desengrasar",
    "desengrasáis": "desengrasar",
    "desengrasáramos": "desengrasar",
    "desengrasáremos": "desengrasar",
    "desengrasásemos": "desengrasar",
    "desengrasé": "desengrasar",
    "desengraséis": "desengrasar",
    "desengrasó": "desengrasar",
    "desengrosaba": "desengrosar",
    "desengrosabais": "desengrosar",
    "desengrosaban": "desengrosar",
    "desengrosabas": "desengrosar",
    "desengrosad": "desengrosar",
    "desengrosada": "desengrosar",
    "desengrosadas": "desengrosar",
    "desengrosado": "desengrosar",
    "desengrosados": "desengrosar",
    "desengrosamos": "desengrosar",
    "desengrosando": "desengrosar",
    "desengrosara": "desengrosar",
    "desengrosarais": "desengrosar",
    "desengrosaran": "desengrosar",
    "desengrosaras": "desengrosar",
    "desengrosare": "desengrosar",
    "desengrosareis": "desengrosar",
    "desengrosaremos": "desengrosar",
    "desengrosaren": "desengrosar",
    "desengrosares": "desengrosar",
    "desengrosaron": "desengrosar",
    "desengrosará": "desengrosar",
    "desengrosarán": "desengrosar",
    "desengrosarás": "desengrosar",
    "desengrosaré": "desengrosar",
    "desengrosaréis": "desengrosar",
    "desengrosaría": "desengrosar",
    "desengrosaríais": "desengrosar",
    "desengrosaríamos": "desengrosar",
    "desengrosarían": "desengrosar",
    "desengrosarías": "desengrosar",
    "desengrosase": "desengrosar",
    "desengrosaseis": "desengrosar",
    "desengrosasen": "desengrosar",
    "desengrosases": "desengrosar",
    "desengrosaste": "desengrosar",
    "desengrosasteis": "desengrosar",
    "desengrosemos": "desengrosar",
    "desengrosábamos": "desengrosar",
    "desengrosáis": "desengrosar",
    "desengrosáramos": "desengrosar",
    "desengrosáremos": "desengrosar",
    "desengrosásemos": "desengrosar",
    "desengrosé": "desengrosar",
    "desengroséis": "desengrosar",
    "desengrosó": "desengrosar",
    "desengruesa": "desengrosar",
    "desengruesan": "desengrosar",
    "desengruesas": "desengrosar",
    "desengruese": "desengrosar",
    "desengruesen": "desengrosar",
    "desengrueses": "desengrosar",
    "desengrueso": "desengrosar",
    "desenguanta": "desenguantar",
    "desenguantaba": "desenguantar",
    "desenguantabais": "desenguantar",
    "desenguantaban": "desenguantar",
    "desenguantabas": "desenguantar",
    "desenguantad": "desenguantar",
    "desenguantada": "desenguantar",
    "desenguantadas": "desenguantar",
    "desenguantado": "desenguantar",
    "desenguantados": "desenguantar",
    "desenguantamos": "desenguantar",
    "desenguantan": "desenguantar",
    "desenguantando": "desenguantar",
    "desenguantara": "desenguantar",
    "desenguantarais": "desenguantar",
    "desenguantaran": "desenguantar",
    "desenguantaras": "desenguantar",
    "desenguantare": "desenguantar",
    "desenguantareis": "desenguantar",
    "desenguantaremos": "desenguantar",
    "desenguantaren": "desenguantar",
    "desenguantares": "desenguantar",
    "desenguantaron": "desenguantar",
    "desenguantará": "desenguantar",
    "desenguantarán": "desenguantar",
    "desenguantarás": "desenguantar",
    "desenguantaré": "desenguantar",
    "desenguantaréis": "desenguantar",
    "desenguantaría": "desenguantar",
    "desenguantaríais": "desenguantar",
    "desenguantaríamos": "desenguantar",
    "desenguantarían": "desenguantar",
    "desenguantarías": "desenguantar",
    "desenguantas": "desenguantar",
    "desenguantase": "desenguantar",
    "desenguantaseis": "desenguantar",
    "desenguantasen": "desenguantar",
    "desenguantases": "desenguantar",
    "desenguantaste": "desenguantar",
    "desenguantasteis": "desenguantar",
    "desenguante": "desenguantar",
    "desenguantemos": "desenguantar",
    "desenguanten": "desenguantar",
    "desenguantes": "desenguantar",
    "desenguanto": "desenguantar",
    "desenguantábamos": "desenguantar",
    "desenguantáis": "desenguantar",
    "desenguantáramos": "desenguantar",
    "desenguantáremos": "desenguantar",
    "desenguantásemos": "desenguantar",
    "desenguanté": "desenguantar",
    "desenguantéis": "desenguantar",
    "desenguantó": "desenguantar",
    "desenhebra": "desenhebrar",
    "desenhebraba": "desenhebrar",
    "desenhebrabais": "desenhebrar",
    "desenhebraban": "desenhebrar",
    "desenhebrabas": "desenhebrar",
    "desenhebrad": "desenhebrar",
    "desenhebrada": "desenhebrar",
    "desenhebradas": "desenhebrar",
    "desenhebrado": "desenhebrar",
    "desenhebrados": "desenhebrar",
    "desenhebramos": "desenhebrar",
    "desenhebran": "desenhebrar",
    "desenhebrando": "desenhebrar",
    "desenhebrara": "desenhebrar",
    "desenhebrarais": "desenhebrar",
    "desenhebraran": "desenhebrar",
    "desenhebraras": "desenhebrar",
    "desenhebrare": "desenhebrar",
    "desenhebrareis": "desenhebrar",
    "desenhebraremos": "desenhebrar",
    "desenhebraren": "desenhebrar",
    "desenhebrares": "desenhebrar",
    "desenhebraron": "desenhebrar",
    "desenhebrará": "desenhebrar",
    "desenhebrarán": "desenhebrar",
    "desenhebrarás": "desenhebrar",
    "desenhebraré": "desenhebrar",
    "desenhebraréis": "desenhebrar",
    "desenhebraría": "desenhebrar",
    "desenhebraríais": "desenhebrar",
    "desenhebraríamos": "desenhebrar",
    "desenhebrarían": "desenhebrar",
    "desenhebrarías": "desenhebrar",
    "desenhebras": "desenhebrar",
    "desenhebrase": "desenhebrar",
    "desenhebraseis": "desenhebrar",
    "desenhebrasen": "desenhebrar",
    "desenhebrases": "desenhebrar",
    "desenhebraste": "desenhebrar",
    "desenhebrasteis": "desenhebrar",
    "desenhebre": "desenhebrar",
    "desenhebremos": "desenhebrar",
    "desenhebren": "desenhebrar",
    "desenhebres": "desenhebrar",
    "desenhebro": "desenhebrar",
    "desenhebrábamos": "desenhebrar",
    "desenhebráis": "desenhebrar",
    "desenhebráramos": "desenhebrar",
    "desenhebráremos": "desenhebrar",
    "desenhebrásemos": "desenhebrar",
    "desenhebré": "desenhebrar",
    "desenhebréis": "desenhebrar",
    "desenhebró": "desenhebrar",
    "desenhorna": "desenhornar",
    "desenhornaba": "desenhornar",
    "desenhornabais": "desenhornar",
    "desenhornaban": "desenhornar",
    "desenhornabas": "desenhornar",
    "desenhornad": "desenhornar",
    "desenhornada": "desenhornar",
    "desenhornadas": "desenhornar",
    "desenhornado": "desenhornar",
    "desenhornados": "desenhornar",
    "desenhornamos": "desenhornar",
    "desenhornan": "desenhornar",
    "desenhornando": "desenhornar",
    "desenhornara": "desenhornar",
    "desenhornarais": "desenhornar",
    "desenhornaran": "desenhornar",
    "desenhornaras": "desenhornar",
    "desenhornare": "desenhornar",
    "desenhornareis": "desenhornar",
    "desenhornaremos": "desenhornar",
    "desenhornaren": "desenhornar",
    "desenhornares": "desenhornar",
    "desenhornaron": "desenhornar",
    "desenhornará": "desenhornar",
    "desenhornarán": "desenhornar",
    "desenhornarás": "desenhornar",
    "desenhornaré": "desenhornar",
    "desenhornaréis": "desenhornar",
    "desenhornaría": "desenhornar",
    "desenhornaríais": "desenhornar",
    "desenhornaríamos": "desenhornar",
    "desenhornarían": "desenhornar",
    "desenhornarías": "desenhornar",
    "desenhornas": "desenhornar",
    "desenhornase": "desenhornar",
    "desenhornaseis": "desenhornar",
    "desenhornasen": "desenhornar",
    "desenhornases": "desenhornar",
    "desenhornaste": "desenhornar",
    "desenhornasteis": "desenhornar",
    "desenhorne": "desenhornar",
    "desenhornemos": "desenhornar",
    "desenhornen": "desenhornar",
    "desenhornes": "desenhornar",
    "desenhorno": "desenhornar",
    "desenhornábamos": "desenhornar",
    "desenhornáis": "desenhornar",
    "desenhornáramos": "desenhornar",
    "desenhornáremos": "desenhornar",
    "desenhornásemos": "desenhornar",
    "desenhorné": "desenhornar",
    "desenhornéis": "desenhornar",
    "desenhornó": "desenhornar",
    "desenjaece": "desenjaezar",
    "desenjaecemos": "desenjaezar",
    "desenjaecen": "desenjaezar",
    "desenjaeces": "desenjaezar",
    "desenjaecé": "desenjaezar",
    "desenjaecéis": "desenjaezar",
    "desenjaeza": "desenjaezar",
    "desenjaezaba": "desenjaezar",
    "desenjaezabais": "desenjaezar",
    "desenjaezaban": "desenjaezar",
    "desenjaezabas": "desenjaezar",
    "desenjaezad": "desenjaezar",
    "desenjaezada": "desenjaezar",
    "desenjaezadas": "desenjaezar",
    "desenjaezado": "desenjaezar",
    "desenjaezados": "desenjaezar",
    "desenjaezamos": "desenjaezar",
    "desenjaezan": "desenjaezar",
    "desenjaezando": "desenjaezar",
    "desenjaezara": "desenjaezar",
    "desenjaezarais": "desenjaezar",
    "desenjaezaran": "desenjaezar",
    "desenjaezaras": "desenjaezar",
    "desenjaezare": "desenjaezar",
    "desenjaezareis": "desenjaezar",
    "desenjaezaremos": "desenjaezar",
    "desenjaezaren": "desenjaezar",
    "desenjaezares": "desenjaezar",
    "desenjaezaron": "desenjaezar",
    "desenjaezará": "desenjaezar",
    "desenjaezarán": "desenjaezar",
    "desenjaezarás": "desenjaezar",
    "desenjaezaré": "desenjaezar",
    "desenjaezaréis": "desenjaezar",
    "desenjaezaría": "desenjaezar",
    "desenjaezaríais": "desenjaezar",
    "desenjaezaríamos": "desenjaezar",
    "desenjaezarían": "desenjaezar",
    "desenjaezarías": "desenjaezar",
    "desenjaezas": "desenjaezar",
    "desenjaezase": "desenjaezar",
    "desenjaezaseis": "desenjaezar",
    "desenjaezasen": "desenjaezar",
    "desenjaezases": "desenjaezar",
    "desenjaezaste": "desenjaezar",
    "desenjaezasteis": "desenjaezar",
    "desenjaezo": "desenjaezar",
    "desenjaezábamos": "desenjaezar",
    "desenjaezáis": "desenjaezar",
    "desenjaezáramos": "desenjaezar",
    "desenjaezáremos": "desenjaezar",
    "desenjaezásemos": "desenjaezar",
    "desenjaezó": "desenjaezar",
    "desenjalma": "desenjalmar",
    "desenjalmaba": "desenjalmar",
    "desenjalmabais": "desenjalmar",
    "desenjalmaban": "desenjalmar",
    "desenjalmabas": "desenjalmar",
    "desenjalmad": "desenjalmar",
    "desenjalmada": "desenjalmar",
    "desenjalmadas": "desenjalmar",
    "desenjalmado": "desenjalmar",
    "desenjalmados": "desenjalmar",
    "desenjalmamos": "desenjalmar",
    "desenjalman": "desenjalmar",
    "desenjalmando": "desenjalmar",
    "desenjalmara": "desenjalmar",
    "desenjalmarais": "desenjalmar",
    "desenjalmaran": "desenjalmar",
    "desenjalmaras": "desenjalmar",
    "desenjalmare": "desenjalmar",
    "desenjalmareis": "desenjalmar",
    "desenjalmaremos": "desenjalmar",
    "desenjalmaren": "desenjalmar",
    "desenjalmares": "desenjalmar",
    "desenjalmaron": "desenjalmar",
    "desenjalmará": "desenjalmar",
    "desenjalmarán": "desenjalmar",
    "desenjalmarás": "desenjalmar",
    "desenjalmaré": "desenjalmar",
    "desenjalmaréis": "desenjalmar",
    "desenjalmaría": "desenjalmar",
    "desenjalmaríais": "desenjalmar",
    "desenjalmaríamos": "desenjalmar",
    "desenjalmarían": "desenjalmar",
    "desenjalmarías": "desenjalmar",
    "desenjalmas": "desenjalmar",
    "desenjalmase": "desenjalmar",
    "desenjalmaseis": "desenjalmar",
    "desenjalmasen": "desenjalmar",
    "desenjalmases": "desenjalmar",
    "desenjalmaste": "desenjalmar",
    "desenjalmasteis": "desenjalmar",
    "desenjalme": "desenjalmar",
    "desenjalmemos": "desenjalmar",
    "desenjalmen": "desenjalmar",
    "desenjalmes": "desenjalmar",
    "desenjalmo": "desenjalmar",
    "desenjalmábamos": "desenjalmar",
    "desenjalmáis": "desenjalmar",
    "desenjalmáramos": "desenjalmar",
    "desenjalmáremos": "desenjalmar",
    "desenjalmásemos": "desenjalmar",
    "desenjalmé": "desenjalmar",
    "desenjalméis": "desenjalmar",
    "desenjalmó": "desenjalmar",
    "desenjaula": "desenjaular",
    "desenjaulaba": "desenjaular",
    "desenjaulabais": "desenjaular",
    "desenjaulaban": "desenjaular",
    "desenjaulabas": "desenjaular",
    "desenjaulad": "desenjaular",
    "desenjaulada": "desenjaular",
    "desenjauladas": "desenjaular",
    "desenjaulado": "desenjaular",
    "desenjaulados": "desenjaular",
    "desenjaulamos": "desenjaular",
    "desenjaulan": "desenjaular",
    "desenjaulando": "desenjaular",
    "desenjaulara": "desenjaular",
    "desenjaularais": "desenjaular",
    "desenjaularan": "desenjaular",
    "desenjaularas": "desenjaular",
    "desenjaulare": "desenjaular",
    "desenjaulareis": "desenjaular",
    "desenjaularemos": "desenjaular",
    "desenjaularen": "desenjaular",
    "desenjaulares": "desenjaular",
    "desenjaularon": "desenjaular",
    "desenjaulará": "desenjaular",
    "desenjaularán": "desenjaular",
    "desenjaularás": "desenjaular",
    "desenjaularé": "desenjaular",
    "desenjaularéis": "desenjaular",
    "desenjaularía": "desenjaular",
    "desenjaularíais": "desenjaular",
    "desenjaularíamos": "desenjaular",
    "desenjaularían": "desenjaular",
    "desenjaularías": "desenjaular",
    "desenjaulas": "desenjaular",
    "desenjaulase": "desenjaular",
    "desenjaulaseis": "desenjaular",
    "desenjaulasen": "desenjaular",
    "desenjaulases": "desenjaular",
    "desenjaulaste": "desenjaular",
    "desenjaulasteis": "desenjaular",
    "desenjaule": "desenjaular",
    "desenjaulemos": "desenjaular",
    "desenjaulen": "desenjaular",
    "desenjaules": "desenjaular",
    "desenjaulo": "desenjaular",
    "desenjaulábamos": "desenjaular",
    "desenjauláis": "desenjaular",
    "desenjauláramos": "desenjaular",
    "desenjauláremos": "desenjaular",
    "desenjaulásemos": "desenjaular",
    "desenjaulé": "desenjaular",
    "desenjauléis": "desenjaular",
    "desenjauló": "desenjaular",
    "desenlace": "desenlazar",
    "desenlacemos": "desenlazar",
    "desenlacen": "desenlazar",
    "desenlaces": "desenlazar",
    "desenlacé": "desenlazar",
    "desenlacéis": "desenlazar",
    "desenladrilla": "desenladrillar",
    "desenladrillaba": "desenladrillar",
    "desenladrillabais": "desenladrillar",
    "desenladrillaban": "desenladrillar",
    "desenladrillabas": "desenladrillar",
    "desenladrillad": "desenladrillar",
    "desenladrillada": "desenladrillar",
    "desenladrilladas": "desenladrillar",
    "desenladrillado": "desenladrillar",
    "desenladrillados": "desenladrillar",
    "desenladrillamientos": "desenladrillamiento",
    "desenladrillamos": "desenladrillar",
    "desenladrillan": "desenladrillar",
    "desenladrillando": "desenladrillar",
    "desenladrillara": "desenladrillar",
    "desenladrillarais": "desenladrillar",
    "desenladrillaran": "desenladrillar",
    "desenladrillaras": "desenladrillar",
    "desenladrillare": "desenladrillar",
    "desenladrillareis": "desenladrillar",
    "desenladrillaremos": "desenladrillar",
    "desenladrillaren": "desenladrillar",
    "desenladrillares": "desenladrillar",
    "desenladrillaron": "desenladrillar",
    "desenladrillará": "desenladrillar",
    "desenladrillarán": "desenladrillar",
    "desenladrillarás": "desenladrillar",
    "desenladrillaré": "desenladrillar",
    "desenladrillaréis": "desenladrillar",
    "desenladrillaría": "desenladrillar",
    "desenladrillaríais": "desenladrillar",
    "desenladrillaríamos": "desenladrillar",
    "desenladrillarían": "desenladrillar",
    "desenladrillarías": "desenladrillar",
    "desenladrillas": "desenladrillar",
    "desenladrillase": "desenladrillar",
    "desenladrillaseis": "desenladrillar",
    "desenladrillasen": "desenladrillar",
    "desenladrillases": "desenladrillar",
    "desenladrillaste": "desenladrillar",
    "desenladrillasteis": "desenladrillar",
    "desenladrille": "desenladrillar",
    "desenladrillemos": "desenladrillar",
    "desenladrillen": "desenladrillar",
    "desenladrilles": "desenladrillar",
    "desenladrillo": "desenladrillar",
    "desenladrillábamos": "desenladrillar",
    "desenladrilláis": "desenladrillar",
    "desenladrilláramos": "desenladrillar",
    "desenladrilláremos": "desenladrillar",
    "desenladrillásemos": "desenladrillar",
    "desenladrillé": "desenladrillar",
    "desenladrilléis": "desenladrillar",
    "desenladrilló": "desenladrillar",
    "desenlaza": "desenlazar",
    "desenlazaba": "desenlazar",
    "desenlazabais": "desenlazar",
    "desenlazaban": "desenlazar",
    "desenlazabas": "desenlazar",
    "desenlazad": "desenlazar",
    "desenlazada": "desenlazar",
    "desenlazadas": "desenlazar",
    "desenlazado": "desenlazar",
    "desenlazados": "desenlazar",
    "desenlazamos": "desenlazar",
    "desenlazan": "desenlazar",
    "desenlazando": "desenlazar",
    "desenlazara": "desenlazar",
    "desenlazarais": "desenlazar",
    "desenlazaran": "desenlazar",
    "desenlazaras": "desenlazar",
    "desenlazare": "desenlazar",
    "desenlazareis": "desenlazar",
    "desenlazaremos": "desenlazar",
    "desenlazaren": "desenlazar",
    "desenlazares": "desenlazar",
    "desenlazaron": "desenlazar",
    "desenlazará": "desenlazar",
    "desenlazarán": "desenlazar",
    "desenlazarás": "desenlazar",
    "desenlazaré": "desenlazar",
    "desenlazaréis": "desenlazar",
    "desenlazaría": "desenlazar",
    "desenlazaríais": "desenlazar",
    "desenlazaríamos": "desenlazar",
    "desenlazarían": "desenlazar",
    "desenlazarías": "desenlazar",
    "desenlazas": "desenlazar",
    "desenlazase": "desenlazar",
    "desenlazaseis": "desenlazar",
    "desenlazasen": "desenlazar",
    "desenlazases": "desenlazar",
    "desenlazaste": "desenlazar",
    "desenlazasteis": "desenlazar",
    "desenlazo": "desenlazar",
    "desenlazábamos": "desenlazar",
    "desenlazáis": "desenlazar",
    "desenlazáramos": "desenlazar",
    "desenlazáremos": "desenlazar",
    "desenlazásemos": "desenlazar",
    "desenlazó": "desenlazar",
    "desenloda": "desenlodar",
    "desenlodaba": "desenlodar",
    "desenlodabais": "desenlodar",
    "desenlodaban": "desenlodar",
    "desenlodabas": "desenlodar",
    "desenlodad": "desenlodar",
    "desenlodada": "desenlodar",
    "desenlodadas": "desenlodar",
    "desenlodado": "desenlodar",
    "desenlodados": "desenlodar",
    "desenlodamos": "desenlodar",
    "desenlodan": "desenlodar",
    "desenlodando": "desenlodar",
    "desenlodara": "desenlodar",
    "desenlodarais": "desenlodar",
    "desenlodaran": "desenlodar",
    "desenlodaras": "desenlodar",
    "desenlodare": "desenlodar",
    "desenlodareis": "desenlodar",
    "desenlodaremos": "desenlodar",
    "desenlodaren": "desenlodar",
    "desenlodares": "desenlodar",
    "desenlodaron": "desenlodar",
    "desenlodará": "desenlodar",
    "desenlodarán": "desenlodar",
    "desenlodarás": "desenlodar",
    "desenlodaré": "desenlodar",
    "desenlodaréis": "desenlodar",
    "desenlodaría": "desenlodar",
    "desenlodaríais": "desenlodar",
    "desenlodaríamos": "desenlodar",
    "desenlodarían": "desenlodar",
    "desenlodarías": "desenlodar",
    "desenlodas": "desenlodar",
    "desenlodase": "desenlodar",
    "desenlodaseis": "desenlodar",
    "desenlodasen": "desenlodar",
    "desenlodases": "desenlodar",
    "desenlodaste": "desenlodar",
    "desenlodasteis": "desenlodar",
    "desenlode": "desenlodar",
    "desenlodemos": "desenlodar",
    "desenloden": "desenlodar",
    "desenlodes": "desenlodar",
    "desenlodo": "desenlodar",
    "desenlodábamos": "desenlodar",
    "desenlodáis": "desenlodar",
    "desenlodáramos": "desenlodar",
    "desenlodáremos": "desenlodar",
    "desenlodásemos": "desenlodar",
    "desenlodé": "desenlodar",
    "desenlodéis": "desenlodar",
    "desenlodó": "desenlodar",
    "desenlosa": "desenlosar",
    "desenlosaba": "desenlosar",
    "desenlosabais": "desenlosar",
    "desenlosaban": "desenlosar",
    "desenlosabas": "desenlosar",
    "desenlosad": "desenlosar",
    "desenlosada": "desenlosar",
    "desenlosadas": "desenlosar",
    "desenlosado": "desenlosar",
    "desenlosados": "desenlosar",
    "desenlosamos": "desenlosar",
    "desenlosan": "desenlosar",
    "desenlosando": "desenlosar",
    "desenlosara": "desenlosar",
    "desenlosarais": "desenlosar",
    "desenlosaran": "desenlosar",
    "desenlosaras": "desenlosar",
    "desenlosare": "desenlosar",
    "desenlosareis": "desenlosar",
    "desenlosaremos": "desenlosar",
    "desenlosaren": "desenlosar",
    "desenlosares": "desenlosar",
    "desenlosaron": "desenlosar",
    "desenlosará": "desenlosar",
    "desenlosarán": "desenlosar",
    "desenlosarás": "desenlosar",
    "desenlosaré": "desenlosar",
    "desenlosaréis": "desenlosar",
    "desenlosaría": "desenlosar",
    "desenlosaríais": "desenlosar",
    "desenlosaríamos": "desenlosar",
    "desenlosarían": "desenlosar",
    "desenlosarías": "desenlosar",
    "desenlosas": "desenlosar",
    "desenlosase": "desenlosar",
    "desenlosaseis": "desenlosar",
    "desenlosasen": "desenlosar",
    "desenlosases": "desenlosar",
    "desenlosaste": "desenlosar",
    "desenlosasteis": "desenlosar",
    "desenlose": "desenlosar",
    "desenlosemos": "desenlosar",
    "desenlosen": "desenlosar",
    "desenloses": "desenlosar",
    "desenloso": "desenlosar",
    "desenlosábamos": "desenlosar",
    "desenlosáis": "desenlosar",
    "desenlosáramos": "desenlosar",
    "desenlosáremos": "desenlosar",
    "desenlosásemos": "desenlosar",
    "desenlosé": "desenlosar",
    "desenloséis": "desenlosar",
    "desenlosó": "desenlosar",
    "desenluta": "desenlutar",
    "desenlutaba": "desenlutar",
    "desenlutabais": "desenlutar",
    "desenlutaban": "desenlutar",
    "desenlutabas": "desenlutar",
    "desenlutad": "desenlutar",
    "desenlutada": "desenlutar",
    "desenlutadas": "desenlutar",
    "desenlutado": "desenlutar",
    "desenlutados": "desenlutar",
    "desenlutamos": "desenlutar",
    "desenlutan": "desenlutar",
    "desenlutando": "desenlutar",
    "desenlutara": "desenlutar",
    "desenlutarais": "desenlutar",
    "desenlutaran": "desenlutar",
    "desenlutaras": "desenlutar",
    "desenlutare": "desenlutar",
    "desenlutareis": "desenlutar",
    "desenlutaremos": "desenlutar",
    "desenlutaren": "desenlutar",
    "desenlutares": "desenlutar",
    "desenlutaron": "desenlutar",
    "desenlutará": "desenlutar",
    "desenlutarán": "desenlutar",
    "desenlutarás": "desenlutar",
    "desenlutaré": "desenlutar",
    "desenlutaréis": "desenlutar",
    "desenlutaría": "desenlutar",
    "desenlutaríais": "desenlutar",
    "desenlutaríamos": "desenlutar",
    "desenlutarían": "desenlutar",
    "desenlutarías": "desenlutar",
    "desenlutas": "desenlutar",
    "desenlutase": "desenlutar",
    "desenlutaseis": "desenlutar",
    "desenlutasen": "desenlutar",
    "desenlutases": "desenlutar",
    "desenlutaste": "desenlutar",
    "desenlutasteis": "desenlutar",
    "desenlute": "desenlutar",
    "desenlutemos": "desenlutar",
    "desenluten": "desenlutar",
    "desenlutes": "desenlutar",
    "desenluto": "desenlutar",
    "desenlutábamos": "desenlutar",
    "desenlutáis": "desenlutar",
    "desenlutáramos": "desenlutar",
    "desenlutáremos": "desenlutar",
    "desenlutásemos": "desenlutar",
    "desenluté": "desenlutar",
    "desenlutéis": "desenlutar",
    "desenlutó": "desenlutar",
    "desenmaraña": "desenmarañar",
    "desenmarañaba": "desenmarañar",
    "desenmarañabais": "desenmarañar",
    "desenmarañaban": "desenmarañar",
    "desenmarañabas": "desenmarañar",
    "desenmarañad": "desenmarañar",
    "desenmarañada": "desenmarañar",
    "desenmarañadas": "desenmarañar",
    "desenmarañado": "desenmarañar",
    "desenmarañados": "desenmarañar",
    "desenmarañamientos": "desenmarañamiento",
    "desenmarañamos": "desenmarañar",
    "desenmarañan": "desenmarañar",
    "desenmarañando": "desenmarañar",
    "desenmarañara": "desenmarañar",
    "desenmarañarais": "desenmarañar",
    "desenmarañaran": "desenmarañar",
    "desenmarañaras": "desenmarañar",
    "desenmarañare": "desenmarañar",
    "desenmarañareis": "desenmarañar",
    "desenmarañaremos": "desenmarañar",
    "desenmarañaren": "desenmarañar",
    "desenmarañares": "desenmarañar",
    "desenmarañaron": "desenmarañar",
    "desenmarañará": "desenmarañar",
    "desenmarañarán": "desenmarañar",
    "desenmarañarás": "desenmarañar",
    "desenmarañaré": "desenmarañar",
    "desenmarañaréis": "desenmarañar",
    "desenmarañaría": "desenmarañar",
    "desenmarañaríais": "desenmarañar",
    "desenmarañaríamos": "desenmarañar",
    "desenmarañarían": "desenmarañar",
    "desenmarañarías": "desenmarañar",
    "desenmarañas": "desenmarañar",
    "desenmarañase": "desenmarañar",
    "desenmarañaseis": "desenmarañar",
    "desenmarañasen": "desenmarañar",
    "desenmarañases": "desenmarañar",
    "desenmarañaste": "desenmarañar",
    "desenmarañasteis": "desenmarañar",
    "desenmarañe": "desenmarañar",
    "desenmarañemos": "desenmarañar",
    "desenmarañen": "desenmarañar",
    "desenmarañes": "desenmarañar",
    "desenmaraño": "desenmarañar",
    "desenmarañábamos": "desenmarañar",
    "desenmarañáis": "desenmarañar",
    "desenmarañáramos": "desenmarañar",
    "desenmarañáremos": "desenmarañar",
    "desenmarañásemos": "desenmarañar",
    "desenmarañé": "desenmarañar",
    "desenmarañéis": "desenmarañar",
    "desenmarañó": "desenmarañar",
    "desenmascara": "desenmascarar",
    "desenmascaraba": "desenmascarar",
    "desenmascarabais": "desenmascarar",
    "desenmascaraban": "desenmascarar",
    "desenmascarabas": "desenmascarar",
    "desenmascarad": "desenmascarar",
    "desenmascarada": "desenmascarar",
    "desenmascaradas": "desenmascarar",
    "desenmascarado": "desenmascarar",
    "desenmascarados": "desenmascarar",
    "desenmascaramientos": "desenmascaramiento",
    "desenmascaramos": "desenmascarar",
    "desenmascaran": "desenmascarar",
    "desenmascarando": "desenmascarar",
    "desenmascarara": "desenmascarar",
    "desenmascararais": "desenmascarar",
    "desenmascararan": "desenmascarar",
    "desenmascararas": "desenmascarar",
    "desenmascarare": "desenmascarar",
    "desenmascarareis": "desenmascarar",
    "desenmascararemos": "desenmascarar",
    "desenmascararen": "desenmascarar",
    "desenmascarares": "desenmascarar",
    "desenmascararon": "desenmascarar",
    "desenmascarará": "desenmascarar",
    "desenmascararán": "desenmascarar",
    "desenmascararás": "desenmascarar",
    "desenmascararé": "desenmascarar",
    "desenmascararéis": "desenmascarar",
    "desenmascararía": "desenmascarar",
    "desenmascararíais": "desenmascarar",
    "desenmascararíamos": "desenmascarar",
    "desenmascararían": "desenmascarar",
    "desenmascararías": "desenmascarar",
    "desenmascaras": "desenmascarar",
    "desenmascarase": "desenmascarar",
    "desenmascaraseis": "desenmascarar",
    "desenmascarasen": "desenmascarar",
    "desenmascarases": "desenmascarar",
    "desenmascaraste": "desenmascarar",
    "desenmascarasteis": "desenmascarar",
    "desenmascare": "desenmascarar",
    "desenmascaremos": "desenmascarar",
    "desenmascaren": "desenmascarar",
    "desenmascares": "desenmascarar",
    "desenmascaro": "desenmascarar",
    "desenmascarábamos": "desenmascarar",
    "desenmascaráis": "desenmascarar",
    "desenmascaráramos": "desenmascarar",
    "desenmascaráremos": "desenmascarar",
    "desenmascarásemos": "desenmascarar",
    "desenmascaré": "desenmascarar",
    "desenmascaréis": "desenmascarar",
    "desenmascaró": "desenmascarar",
    "desenmohece": "desenmohecer",
    "desenmoheced": "desenmohecer",
    "desenmohecemos": "desenmohecer",
    "desenmohecen": "desenmohecer",
    "desenmoheceremos": "desenmohecer",
    "desenmohecerá": "desenmohecer",
    "desenmohecerán": "desenmohecer",
    "desenmohecerás": "desenmohecer",
    "desenmoheceré": "desenmohecer",
    "desenmoheceréis": "desenmohecer",
    "desenmohecería": "desenmohecer",
    "desenmoheceríais": "desenmohecer",
    "desenmoheceríamos": "desenmohecer",
    "desenmohecerían": "desenmohecer",
    "desenmohecerías": "desenmohecer",
    "desenmoheces": "desenmohecer",
    "desenmohecida": "desenmohecer",
    "desenmohecidas": "desenmohecer",
    "desenmohecido": "desenmohecer",
    "desenmohecidos": "desenmohecer",
    "desenmoheciendo": "desenmohecer",
    "desenmoheciera": "desenmohecer",
    "desenmohecierais": "desenmohecer",
    "desenmohecieran": "desenmohecer",
    "desenmohecieras": "desenmohecer",
    "desenmoheciere": "desenmohecer",
    "desenmoheciereis": "desenmohecer",
    "desenmohecieren": "desenmohecer",
    "desenmohecieres": "desenmohecer",
    "desenmohecieron": "desenmohecer",
    "desenmoheciese": "desenmohecer",
    "desenmohecieseis": "desenmohecer",
    "desenmoheciesen": "desenmohecer",
    "desenmohecieses": "desenmohecer",
    "desenmohecimientos": "desenmohecimiento",
    "desenmohecimos": "desenmohecer",
    "desenmoheciste": "desenmohecer",
    "desenmohecisteis": "desenmohecer",
    "desenmoheciéramos": "desenmohecer",
    "desenmoheciéremos": "desenmohecer",
    "desenmoheciésemos": "desenmohecer",
    "desenmoheció": "desenmohecer",
    "desenmohecéis": "desenmohecer",
    "desenmohecí": "desenmohecer",
    "desenmohecía": "desenmohecer",
    "desenmohecíais": "desenmohecer",
    "desenmohecíamos": "desenmohecer",
    "desenmohecían": "desenmohecer",
    "desenmohecías": "desenmohecer",
    "desenmohezca": "desenmohecer",
    "desenmohezcamos": "desenmohecer",
    "desenmohezcan": "desenmohecer",
    "desenmohezcas": "desenmohecer",
    "desenmohezco": "desenmohecer",
    "desenmohezcáis": "desenmohecer",
    "desenmudece": "desenmudecer",
    "desenmudeced": "desenmudecer",
    "desenmudecemos": "desenmudecer",
    "desenmudecen": "desenmudecer",
    "desenmudeceremos": "desenmudecer",
    "desenmudecerá": "desenmudecer",
    "desenmudecerán": "desenmudecer",
    "desenmudecerás": "desenmudecer",
    "desenmudeceré": "desenmudecer",
    "desenmudeceréis": "desenmudecer",
    "desenmudecería": "desenmudecer",
    "desenmudeceríais": "desenmudecer",
    "desenmudeceríamos": "desenmudecer",
    "desenmudecerían": "desenmudecer",
    "desenmudecerías": "desenmudecer",
    "desenmudeces": "desenmudecer",
    "desenmudecida": "desenmudecer",
    "desenmudecidas": "desenmudecer",
    "desenmudecido": "desenmudecer",
    "desenmudecidos": "desenmudecer",
    "desenmudeciendo": "desenmudecer",
    "desenmudeciera": "desenmudecer",
    "desenmudecierais": "desenmudecer",
    "desenmudecieran": "desenmudecer",
    "desenmudecieras": "desenmudecer",
    "desenmudeciere": "desenmudecer",
    "desenmudeciereis": "desenmudecer",
    "desenmudecieren": "desenmudecer",
    "desenmudecieres": "desenmudecer",
    "desenmudecieron": "desenmudecer",
    "desenmudeciese": "desenmudecer",
    "desenmudecieseis": "desenmudecer",
    "desenmudeciesen": "desenmudecer",
    "desenmudecieses": "desenmudecer",
    "desenmudecimientos": "desenmudecimiento",
    "desenmudecimos": "desenmudecer",
    "desenmudeciste": "desenmudecer",
    "desenmudecisteis": "desenmudecer",
    "desenmudeciéramos": "desenmudecer",
    "desenmudeciéremos": "desenmudecer",
    "desenmudeciésemos": "desenmudecer",
    "desenmudeció": "desenmudecer",
    "desenmudecéis": "desenmudecer",
    "desenmudecí": "desenmudecer",
    "desenmudecía": "desenmudecer",
    "desenmudecíais": "desenmudecer",
    "desenmudecíamos": "desenmudecer",
    "desenmudecían": "desenmudecer",
    "desenmudecías": "desenmudecer",
    "desenmudezca": "desenmudecer",
    "desenmudezcamos": "desenmudecer",
    "desenmudezcan": "desenmudecer",
    "desenmudezcas": "desenmudecer",
    "desenmudezco": "desenmudecer",
    "desenmudezcáis": "desenmudecer",
    "desenoja": "desenojar",
    "desenojaba": "desenojar",
    "desenojabais": "desenojar",
    "desenojaban": "desenojar",
    "desenojabas": "desenojar",
    "desenojad": "desenojar",
    "desenojada": "desenojar",
    "desenojadas": "desenojar",
    "desenojado": "desenojar",
    "desenojados": "desenojar",
    "desenojamos": "desenojar",
    "desenojan": "desenojar",
    "desenojando": "desenojar",
    "desenojara": "desenojar",
    "desenojarais": "desenojar",
    "desenojaran": "desenojar",
    "desenojaras": "desenojar",
    "desenojare": "desenojar",
    "desenojareis": "desenojar",
    "desenojaremos": "desenojar",
    "desenojaren": "desenojar",
    "desenojares": "desenojar",
    "desenojaron": "desenojar",
    "desenojará": "desenojar",
    "desenojarán": "desenojar",
    "desenojarás": "desenojar",
    "desenojaré": "desenojar",
    "desenojaréis": "desenojar",
    "desenojaría": "desenojar",
    "desenojaríais": "desenojar",
    "desenojaríamos": "desenojar",
    "desenojarían": "desenojar",
    "desenojarías": "desenojar",
    "desenojas": "desenojar",
    "desenojase": "desenojar",
    "desenojaseis": "desenojar",
    "desenojasen": "desenojar",
    "desenojases": "desenojar",
    "desenojaste": "desenojar",
    "desenojasteis": "desenojar",
    "desenoje": "desenojar",
    "desenojemos": "desenojar",
    "desenojen": "desenojar",
    "desenojes": "desenojar",
    "desenojo": "desenojar",
    "desenojos": "desenojo",
    "desenojábamos": "desenojar",
    "desenojáis": "desenojar",
    "desenojáramos": "desenojar",
    "desenojáremos": "desenojar",
    "desenojásemos": "desenojar",
    "desenojé": "desenojar",
    "desenojéis": "desenojar",
    "desenojó": "desenojar",
    "desenreda": "desenredar",
    "desenredaba": "desenredar",
    "desenredabais": "desenredar",
    "desenredaban": "desenredar",
    "desenredabas": "desenredar",
    "desenredad": "desenredar",
    "desenredada": "desenredar",
    "desenredadas": "desenredar",
    "desenredado": "desenredar",
    "desenredados": "desenredar",
    "desenredamos": "desenredar",
    "desenredan": "desenredar",
    "desenredando": "desenredar",
    "desenredara": "desenredar",
    "desenredarais": "desenredar",
    "desenredaran": "desenredar",
    "desenredaras": "desenredar",
    "desenredare": "desenredar",
    "desenredareis": "desenredar",
    "desenredaremos": "desenredar",
    "desenredaren": "desenredar",
    "desenredares": "desenredar",
    "desenredaron": "desenredar",
    "desenredará": "desenredar",
    "desenredarán": "desenredar",
    "desenredarás": "desenredar",
    "desenredaré": "desenredar",
    "desenredaréis": "desenredar",
    "desenredaría": "desenredar",
    "desenredaríais": "desenredar",
    "desenredaríamos": "desenredar",
    "desenredarían": "desenredar",
    "desenredarías": "desenredar",
    "desenredas": "desenredar",
    "desenredase": "desenredar",
    "desenredaseis": "desenredar",
    "desenredasen": "desenredar",
    "desenredases": "desenredar",
    "desenredaste": "desenredar",
    "desenredasteis": "desenredar",
    "desenrede": "desenredar",
    "desenredemos": "desenredar",
    "desenreden": "desenredar",
    "desenredes": "desenredar",
    "desenredo": "desenredar",
    "desenredos": "desenredo",
    "desenredábamos": "desenredar",
    "desenredáis": "desenredar",
    "desenredáramos": "desenredar",
    "desenredáremos": "desenredar",
    "desenredásemos": "desenredar",
    "desenredé": "desenredar",
    "desenredéis": "desenredar",
    "desenredó": "desenredar",
    "desenrolla": "desenrollar",
    "desenrollaba": "desenrollar",
    "desenrollabais": "desenrollar",
    "desenrollaban": "desenrollar",
    "desenrollabas": "desenrollar",
    "desenrollad": "desenrollar",
    "desenrollada": "desenrollar",
    "desenrolladas": "desenrollar",
    "desenrollado": "desenrollar",
    "desenrolladores": "desenrollador",
    "desenrollados": "desenrollar",
    "desenrollamientos": "desenrollamiento",
    "desenrollamos": "desenrollar",
    "desenrollan": "desenrollar",
    "desenrollando": "desenrollar",
    "desenrollara": "desenrollar",
    "desenrollarais": "desenrollar",
    "desenrollaran": "desenrollar",
    "desenrollaras": "desenrollar",
    "desenrollare": "desenrollar",
    "desenrollareis": "desenrollar",
    "desenrollaremos": "desenrollar",
    "desenrollaren": "desenrollar",
    "desenrollares": "desenrollar",
    "desenrollaron": "desenrollar",
    "desenrollará": "desenrollar",
    "desenrollarán": "desenrollar",
    "desenrollarás": "desenrollar",
    "desenrollaré": "desenrollar",
    "desenrollaréis": "desenrollar",
    "desenrollaría": "desenrollar",
    "desenrollaríais": "desenrollar",
    "desenrollaríamos": "desenrollar",
    "desenrollarían": "desenrollar",
    "desenrollarías": "desenrollar",
    "desenrollas": "desenrollar",
    "desenrollase": "desenrollar",
    "desenrollaseis": "desenrollar",
    "desenrollasen": "desenrollar",
    "desenrollases": "desenrollar",
    "desenrollaste": "desenrollar",
    "desenrollasteis": "desenrollar",
    "desenrolle": "desenrollar",
    "desenrollemos": "desenrollar",
    "desenrollen": "desenrollar",
    "desenrolles": "desenrollar",
    "desenrollo": "desenrollar",
    "desenrollábamos": "desenrollar",
    "desenrolláis": "desenrollar",
    "desenrolláramos": "desenrollar",
    "desenrolláremos": "desenrollar",
    "desenrollásemos": "desenrollar",
    "desenrollé": "desenrollar",
    "desenrolléis": "desenrollar",
    "desenrolló": "desenrollar",
    "desenrosca": "desenroscar",
    "desenroscaba": "desenroscar",
    "desenroscabais": "desenroscar",
    "desenroscaban": "desenroscar",
    "desenroscabas": "desenroscar",
    "desenroscad": "desenroscar",
    "desenroscada": "desenroscar",
    "desenroscadas": "desenroscar",
    "desenroscado": "desenroscar",
    "desenroscados": "desenroscar",
    "desenroscamos": "desenroscar",
    "desenroscan": "desenroscar",
    "desenroscando": "desenroscar",
    "desenroscara": "desenroscar",
    "desenroscarais": "desenroscar",
    "desenroscaran": "desenroscar",
    "desenroscaras": "desenroscar",
    "desenroscare": "desenroscar",
    "desenroscareis": "desenroscar",
    "desenroscaremos": "desenroscar",
    "desenroscaren": "desenroscar",
    "desenroscares": "desenroscar",
    "desenroscaron": "desenroscar",
    "desenroscará": "desenroscar",
    "desenroscarán": "desenroscar",
    "desenroscarás": "desenroscar",
    "desenroscaré": "desenroscar",
    "desenroscaréis": "desenroscar",
    "desenroscaría": "desenroscar",
    "desenroscaríais": "desenroscar",
    "desenroscaríamos": "desenroscar",
    "desenroscarían": "desenroscar",
    "desenroscarías": "desenroscar",
    "desenroscas": "desenroscar",
    "desenroscase": "desenroscar",
    "desenroscaseis": "desenroscar",
    "desenroscasen": "desenroscar",
    "desenroscases": "desenroscar",
    "desenroscaste": "desenroscar",
    "desenroscasteis": "desenroscar",
    "desenrosco": "desenroscar",
    "desenroscábamos": "desenroscar",
    "desenroscáis": "desenroscar",
    "desenroscáramos": "desenroscar",
    "desenroscáremos": "desenroscar",
    "desenroscásemos": "desenroscar",
    "desenroscó": "desenroscar",
    "desenrosque": "desenroscar",
    "desenrosquemos": "desenroscar",
    "desenrosquen": "desenroscar",
    "desenrosques": "desenroscar",
    "desenrosqué": "desenroscar",
    "desenrosquéis": "desenroscar",
    "desenrudecimientos": "desenrudecimiento",
    "desensambla": "desensamblar",
    "desensamblaba": "desensamblar",
    "desensamblabais": "desensamblar",
    "desensamblaban": "desensamblar",
    "desensamblabas": "desensamblar",
    "desensamblad": "desensamblar",
    "desensamblada": "desensamblar",
    "desensambladas": "desensamblar",
    "desensamblado": "desensamblar",
    "desensambladores": "desensamblador",
    "desensamblados": "desensamblar",
    "desensamblamos": "desensamblar",
    "desensamblan": "desensamblar",
    "desensamblando": "desensamblar",
    "desensamblara": "desensamblar",
    "desensamblarais": "desensamblar",
    "desensamblaran": "desensamblar",
    "desensamblaras": "desensamblar",
    "desensamblare": "desensamblar",
    "desensamblareis": "desensamblar",
    "desensamblaremos": "desensamblar",
    "desensamblaren": "desensamblar",
    "desensamblares": "desensamblar",
    "desensamblaron": "desensamblar",
    "desensamblará": "desensamblar",
    "desensamblarán": "desensamblar",
    "desensamblarás": "desensamblar",
    "desensamblaré": "desensamblar",
    "desensamblaréis": "desensamblar",
    "desensamblaría": "desensamblar",
    "desensamblaríais": "desensamblar",
    "desensamblaríamos": "desensamblar",
    "desensamblarían": "desensamblar",
    "desensamblarías": "desensamblar",
    "desensamblas": "desensamblar",
    "desensamblase": "desensamblar",
    "desensamblaseis": "desensamblar",
    "desensamblasen": "desensamblar",
    "desensamblases": "desensamblar",
    "desensamblaste": "desensamblar",
    "desensamblasteis": "desensamblar",
    "desensamble": "desensamblar",
    "desensamblemos": "desensamblar",
    "desensamblen": "desensamblar",
    "desensambles": "desensamblar",
    "desensamblo": "desensamblar",
    "desensamblábamos": "desensamblar",
    "desensambláis": "desensamblar",
    "desensambláramos": "desensamblar",
    "desensambláremos": "desensamblar",
    "desensamblásemos": "desensamblar",
    "desensamblé": "desensamblar",
    "desensambléis": "desensamblar",
    "desensambló": "desensamblar",
    "desensarta": "desensartar",
    "desensartaba": "desensartar",
    "desensartabais": "desensartar",
    "desensartaban": "desensartar",
    "desensartabas": "desensartar",
    "desensartad": "desensartar",
    "desensartada": "desensartar",
    "desensartadas": "desensartar",
    "desensartado": "desensartar",
    "desensartados": "desensartar",
    "desensartamos": "desensartar",
    "desensartan": "desensartar",
    "desensartando": "desensartar",
    "desensartara": "desensartar",
    "desensartarais": "desensartar",
    "desensartaran": "desensartar",
    "desensartaras": "desensartar",
    "desensartare": "desensartar",
    "desensartareis": "desensartar",
    "desensartaremos": "desensartar",
    "desensartaren": "desensartar",
    "desensartares": "desensartar",
    "desensartaron": "desensartar",
    "desensartará": "desensartar",
    "desensartarán": "desensartar",
    "desensartarás": "desensartar",
    "desensartaré": "desensartar",
    "desensartaréis": "desensartar",
    "desensartaría": "desensartar",
    "desensartaríais": "desensartar",
    "desensartaríamos": "desensartar",
    "desensartarían": "desensartar",
    "desensartarías": "desensartar",
    "desensartas": "desensartar",
    "desensartase": "desensartar",
    "desensartaseis": "desensartar",
    "desensartasen": "desensartar",
    "desensartases": "desensartar",
    "desensartaste": "desensartar",
    "desensartasteis": "desensartar",
    "desensarte": "desensartar",
    "desensartemos": "desensartar",
    "desensarten": "desensartar",
    "desensartes": "desensartar",
    "desensarto": "desensartar",
    "desensartábamos": "desensartar",
    "desensartáis": "desensartar",
    "desensartáramos": "desensartar",
    "desensartáremos": "desensartar",
    "desensartásemos": "desensartar",
    "desensarté": "desensartar",
    "desensartéis": "desensartar",
    "desensartó": "desensartar",
    "desensaña": "desensañar",
    "desensañaba": "desensañar",
    "desensañabais": "desensañar",
    "desensañaban": "desensañar",
    "desensañabas": "desensañar",
    "desensañad": "desensañar",
    "desensañada": "desensañar",
    "desensañadas": "desensañar",
    "desensañado": "desensañar",
    "desensañados": "desensañar",
    "desensañamos": "desensañar",
    "desensañan": "desensañar",
    "desensañando": "desensañar",
    "desensañara": "desensañar",
    "desensañarais": "desensañar",
    "desensañaran": "desensañar",
    "desensañaras": "desensañar",
    "desensañare": "desensañar",
    "desensañareis": "desensañar",
    "desensañaremos": "desensañar",
    "desensañaren": "desensañar",
    "desensañares": "desensañar",
    "desensañaron": "desensañar",
    "desensañará": "desensañar",
    "desensañarán": "desensañar",
    "desensañarás": "desensañar",
    "desensañaré": "desensañar",
    "desensañaréis": "desensañar",
    "desensañaría": "desensañar",
    "desensañaríais": "desensañar",
    "desensañaríamos": "desensañar",
    "desensañarían": "desensañar",
    "desensañarías": "desensañar",
    "desensañas": "desensañar",
    "desensañase": "desensañar",
    "desensañaseis": "desensañar",
    "desensañasen": "desensañar",
    "desensañases": "desensañar",
    "desensañaste": "desensañar",
    "desensañasteis": "desensañar",
    "desensañe": "desensañar",
    "desensañemos": "desensañar",
    "desensañen": "desensañar",
    "desensañes": "desensañar",
    "desensaño": "desensañar",
    "desensañábamos": "desensañar",
    "desensañáis": "desensañar",
    "desensañáramos": "desensañar",
    "desensañáremos": "desensañar",
    "desensañásemos": "desensañar",
    "desensañé": "desensañar",
    "desensañéis": "desensañar",
    "desensañó": "desensañar",
    "desensibilice": "desensibilizar",
    "desensibilicemos": "desensibilizar",
    "desensibilicen": "desensibilizar",
    "desensibilices": "desensibilizar",
    "desensibilicé": "desensibilizar",
    "desensibilicéis": "desensibilizar",
    "desensibiliza": "desensibilizar",
    "desensibilizaba": "desensibilizar",
    "desensibilizabais": "desensibilizar",
    "desensibilizaban": "desensibilizar",
    "desensibilizabas": "desensibilizar",
    "desensibilizaciones": "desensibilización",
    "desensibilizad": "desensibilizar",
    "desensibilizada": "desensibilizar",
    "desensibilizadas": "desensibilizar",
    "desensibilizado": "desensibilizar",
    "desensibilizados": "desensibilizar",
    "desensibilizamos": "desensibilizar",
    "desensibilizan": "desensibilizar",
    "desensibilizando": "desensibilizar",
    "desensibilizara": "desensibilizar",
    "desensibilizarais": "desensibilizar",
    "desensibilizaran": "desensibilizar",
    "desensibilizaras": "desensibilizar",
    "desensibilizare": "desensibilizar",
    "desensibilizareis": "desensibilizar",
    "desensibilizaremos": "desensibilizar",
    "desensibilizaren": "desensibilizar",
    "desensibilizares": "desensibilizar",
    "desensibilizaron": "desensibilizar",
    "desensibilizará": "desensibilizar",
    "desensibilizarán": "desensibilizar",
    "desensibilizarás": "desensibilizar",
    "desensibilizaré": "desensibilizar",
    "desensibilizaréis": "desensibilizar",
    "desensibilizaría": "desensibilizar",
    "desensibilizaríais": "desensibilizar",
    "desensibilizaríamos": "desensibilizar",
    "desensibilizarían": "desensibilizar",
    "desensibilizarías": "desensibilizar",
    "desensibilizas": "desensibilizar",
    "desensibilizase": "desensibilizar",
    "desensibilizaseis": "desensibilizar",
    "desensibilizasen": "desensibilizar",
    "desensibilizases": "desensibilizar",
    "desensibilizaste": "desensibilizar",
    "desensibilizasteis": "desensibilizar",
    "desensibilizo": "desensibilizar",
    "desensibilizábamos": "desensibilizar",
    "desensibilizáis": "desensibilizar",
    "desensibilizáramos": "desensibilizar",
    "desensibilizáremos": "desensibilizar",
    "desensibilizásemos": "desensibilizar",
    "desensibilizó": "desensibilizar",
    "desensiladoras": "desensiladora",
    "desensilla": "desensillar",
    "desensillaba": "desensillar",
    "desensillabais": "desensillar",
    "desensillaban": "desensillar",
    "desensillabas": "desensillar",
    "desensillad": "desensillar",
    "desensillada": "desensillar",
    "desensilladas": "desensillar",
    "desensillado": "desensillar",
    "desensillados": "desensillar",
    "desensillamos": "desensillar",
    "desensillan": "desensillar",
    "desensillando": "desensillar",
    "desensillara": "desensillar",
    "desensillarais": "desensillar",
    "desensillaran": "desensillar",
    "desensillaras": "desensillar",
    "desensillare": "desensillar",
    "desensillareis": "desensillar",
    "desensillaremos": "desensillar",
    "desensillaren": "desensillar",
    "desensillares": "desensillar",
    "desensillaron": "desensillar",
    "desensillará": "desensillar",
    "desensillarán": "desensillar",
    "desensillarás": "desensillar",
    "desensillaré": "desensillar",
    "desensillaréis": "desensillar",
    "desensillaría": "desensillar",
    "desensillaríais": "desensillar",
    "desensillaríamos": "desensillar",
    "desensillarían": "desensillar",
    "desensillarías": "desensillar",
    "desensillas": "desensillar",
    "desensillase": "desensillar",
    "desensillaseis": "desensillar",
    "desensillasen": "desensillar",
    "desensillases": "desensillar",
    "desensillaste": "desensillar",
    "desensillasteis": "desensillar",
    "desensille": "desensillar",
    "desensillemos": "desensillar",
    "desensillen": "desensillar",
    "desensilles": "desensillar",
    "desensillo": "desensillar",
    "desensillábamos": "desensillar",
    "desensilláis": "desensillar",
    "desensilláramos": "desensillar",
    "desensilláremos": "desensillar",
    "desensillásemos": "desensillar",
    "desensillé": "desensillar",
    "desensilléis": "desensillar",
    "desensilló": "desensillar",
    "desensoberbece": "desensoberbecer",
    "desensoberbeced": "desensoberbecer",
    "desensoberbecemos": "desensoberbecer",
    "desensoberbecen": "desensoberbecer",
    "desensoberbeceremos": "desensoberbecer",
    "desensoberbecerá": "desensoberbecer",
    "desensoberbecerán": "desensoberbecer",
    "desensoberbecerás": "desensoberbecer",
    "desensoberbeceré": "desensoberbecer",
    "desensoberbeceréis": "desensoberbecer",
    "desensoberbecería": "desensoberbecer",
    "desensoberbeceríais": "desensoberbecer",
    "desensoberbeceríamos": "desensoberbecer",
    "desensoberbecerían": "desensoberbecer",
    "desensoberbecerías": "desensoberbecer",
    "desensoberbeces": "desensoberbecer",
    "desensoberbecida": "desensoberbecer",
    "desensoberbecidas": "desensoberbecer",
    "desensoberbecido": "desensoberbecer",
    "desensoberbecidos": "desensoberbecer",
    "desensoberbeciendo": "desensoberbecer",
    "desensoberbeciera": "desensoberbecer",
    "desensoberbecierais": "desensoberbecer",
    "desensoberbecieran": "desensoberbecer",
    "desensoberbecieras": "desensoberbecer",
    "desensoberbeciere": "desensoberbecer",
    "desensoberbeciereis": "desensoberbecer",
    "desensoberbecieren": "desensoberbecer",
    "desensoberbecieres": "desensoberbecer",
    "desensoberbecieron": "desensoberbecer",
    "desensoberbeciese": "desensoberbecer",
    "desensoberbecieseis": "desensoberbecer",
    "desensoberbeciesen": "desensoberbecer",
    "desensoberbecieses": "desensoberbecer",
    "desensoberbecimos": "desensoberbecer",
    "desensoberbeciste": "desensoberbecer",
    "desensoberbecisteis": "desensoberbecer",
    "desensoberbeciéramos": "desensoberbecer",
    "desensoberbeciéremos": "desensoberbecer",
    "desensoberbeciésemos": "desensoberbecer",
    "desensoberbeció": "desensoberbecer",
    "desensoberbecéis": "desensoberbecer",
    "desensoberbecí": "desensoberbecer",
    "desensoberbecía": "desensoberbecer",
    "desensoberbecíais": "desensoberbecer",
    "desensoberbecíamos": "desensoberbecer",
    "desensoberbecían": "desensoberbecer",
    "desensoberbecías": "desensoberbecer",
    "desensoberbezca": "desensoberbecer",
    "desensoberbezcamos": "desensoberbecer",
    "desensoberbezcan": "desensoberbecer",
    "desensoberbezcas": "desensoberbecer",
    "desensoberbezco": "desensoberbecer",
    "desensoberbezcáis": "desensoberbecer",
    "desensortijada": "desensortijado",
    "desensortijadas": "desensortijado",
    "desensortijados": "desensortijado",
    "desentabla": "desentablar",
    "desentablaba": "desentablar",
    "desentablabais": "desentablar",
    "desentablaban": "desentablar",
    "desentablabas": "desentablar",
    "desentablad": "desentablar",
    "desentablada": "desentablar",
    "desentabladas": "desentablar",
    "desentablado": "desentablar",
    "desentablados": "desentablar",
    "desentablamos": "desentablar",
    "desentablan": "desentablar",
    "desentablando": "desentablar",
    "desentablara": "desentablar",
    "desentablarais": "desentablar",
    "desentablaran": "desentablar",
    "desentablaras": "desentablar",
    "desentablare": "desentablar",
    "desentablareis": "desentablar",
    "desentablaremos": "desentablar",
    "desentablaren": "desentablar",
    "desentablares": "desentablar",
    "desentablaron": "desentablar",
    "desentablará": "desentablar",
    "desentablarán": "desentablar",
    "desentablarás": "desentablar",
    "desentablaré": "desentablar",
    "desentablaréis": "desentablar",
    "desentablaría": "desentablar",
    "desentablaríais": "desentablar",
    "desentablaríamos": "desentablar",
    "desentablarían": "desentablar",
    "desentablarías": "desentablar",
    "desentablas": "desentablar",
    "desentablase": "desentablar",
    "desentablaseis": "desentablar",
    "desentablasen": "desentablar",
    "desentablases": "desentablar",
    "desentablaste": "desentablar",
    "desentablasteis": "desentablar",
    "desentable": "desentablar",
    "desentablemos": "desentablar",
    "desentablen": "desentablar",
    "desentables": "desentablar",
    "desentablo": "desentablar",
    "desentablábamos": "desentablar",
    "desentabláis": "desentablar",
    "desentabláramos": "desentablar",
    "desentabláremos": "desentablar",
    "desentablásemos": "desentablar",
    "desentablé": "desentablar",
    "desentabléis": "desentablar",
    "desentabló": "desentablar",
    "desentalinga": "desentalingar",
    "desentalingaba": "desentalingar",
    "desentalingabais": "desentalingar",
    "desentalingaban": "desentalingar",
    "desentalingabas": "desentalingar",
    "desentalingad": "desentalingar",
    "desentalingada": "desentalingar",
    "desentalingadas": "desentalingar",
    "desentalingado": "desentalingar",
    "desentalingados": "desentalingar",
    "desentalingamos": "desentalingar",
    "desentalingan": "desentalingar",
    "desentalingando": "desentalingar",
    "desentalingara": "desentalingar",
    "desentalingarais": "desentalingar",
    "desentalingaran": "desentalingar",
    "desentalingaras": "desentalingar",
    "desentalingare": "desentalingar",
    "desentalingareis": "desentalingar",
    "desentalingaremos": "desentalingar",
    "desentalingaren": "desentalingar",
    "desentalingares": "desentalingar",
    "desentalingaron": "desentalingar",
    "desentalingará": "desentalingar",
    "desentalingarán": "desentalingar",
    "desentalingarás": "desentalingar",
    "desentalingaré": "desentalingar",
    "desentalingaréis": "desentalingar",
    "desentalingaría": "desentalingar",
    "desentalingaríais": "desentalingar",
    "desentalingaríamos": "desentalingar",
    "desentalingarían": "desentalingar",
    "desentalingarías": "desentalingar",
    "desentalingas": "desentalingar",
    "desentalingase": "desentalingar",
    "desentalingaseis": "desentalingar",
    "desentalingasen": "desentalingar",
    "desentalingases": "desentalingar",
    "desentalingaste": "desentalingar",
    "desentalingasteis": "desentalingar",
    "desentalingo": "desentalingar",
    "desentalingue": "desentalingar",
    "desentalinguemos": "desentalingar",
    "desentalinguen": "desentalingar",
    "desentalingues": "desentalingar",
    "desentalingué": "desentalingar",
    "desentalinguéis": "desentalingar",
    "desentalingábamos": "desentalingar",
    "desentalingáis": "desentalingar",
    "desentalingáramos": "desentalingar",
    "desentalingáremos": "desentalingar",
    "desentalingásemos": "desentalingar",
    "desentalingó": "desentalingar",
    "desentarima": "desentarimar",
    "desentarimaba": "desentarimar",
    "desentarimabais": "desentarimar",
    "desentarimaban": "desentarimar",
    "desentarimabas": "desentarimar",
    "desentarimad": "desentarimar",
    "desentarimada": "desentarimar",
    "desentarimadas": "desentarimar",
    "desentarimado": "desentarimar",
    "desentarimados": "desentarimar",
    "desentarimamos": "desentarimar",
    "desentariman": "desentarimar",
    "desentarimando": "desentarimar",
    "desentarimara": "desentarimar",
    "desentarimarais": "desentarimar",
    "desentarimaran": "desentarimar",
    "desentarimaras": "desentarimar",
    "desentarimare": "desentarimar",
    "desentarimareis": "desentarimar",
    "desentarimaremos": "desentarimar",
    "desentarimaren": "desentarimar",
    "desentarimares": "desentarimar",
    "desentarimaron": "desentarimar",
    "desentarimará": "desentarimar",
    "desentarimarán": "desentarimar",
    "desentarimarás": "desentarimar",
    "desentarimaré": "desentarimar",
    "desentarimaréis": "desentarimar",
    "desentarimaría": "desentarimar",
    "desentarimaríais": "desentarimar",
    "desentarimaríamos": "desentarimar",
    "desentarimarían": "desentarimar",
    "desentarimarías": "desentarimar",
    "desentarimas": "desentarimar",
    "desentarimase": "desentarimar",
    "desentarimaseis": "desentarimar",
    "desentarimasen": "desentarimar",
    "desentarimases": "desentarimar",
    "desentarimaste": "desentarimar",
    "desentarimasteis": "desentarimar",
    "desentarime": "desentarimar",
    "desentarimemos": "desentarimar",
    "desentarimen": "desentarimar",
    "desentarimes": "desentarimar",
    "desentarimo": "desentarimar",
    "desentarimábamos": "desentarimar",
    "desentarimáis": "desentarimar",
    "desentarimáramos": "desentarimar",
    "desentarimáremos": "desentarimar",
    "desentarimásemos": "desentarimar",
    "desentarimé": "desentarimar",
    "desentariméis": "desentarimar",
    "desentarimó": "desentarimar",
    "desentendamos": "desentender",
    "desentended": "desentender",
    "desentendemos": "desentender",
    "desentenderemos": "desentender",
    "desentenderá": "desentender",
    "desentenderán": "desentender",
    "desentenderás": "desentender",
    "desentenderé": "desentender",
    "desentenderéis": "desentender",
    "desentendería": "desentender",
    "desentenderíais": "desentender",
    "desentenderíamos": "desentender",
    "desentenderían": "desentender",
    "desentenderías": "desentender",
    "desentendida": "desentender",
    "desentendidas": "desentender",
    "desentendido": "desentender",
    "desentendidos": "desentender",
    "desentendiendo": "desentender",
    "desentendiera": "desentender",
    "desentendierais": "desentender",
    "desentendieran": "desentender",
    "desentendieras": "desentender",
    "desentendiere": "desentender",
    "desentendiereis": "desentender",
    "desentendieren": "desentender",
    "desentendieres": "desentender",
    "desentendieron": "desentender",
    "desentendiese": "desentender",
    "desentendieseis": "desentender",
    "desentendiesen": "desentender",
    "desentendieses": "desentender",
    "desentendimientos": "desentendimiento",
    "desentendimos": "desentender",
    "desentendiste": "desentender",
    "desentendisteis": "desentender",
    "desentendiéramos": "desentender",
    "desentendiéremos": "desentender",
    "desentendiésemos": "desentender",
    "desentendió": "desentender",
    "desentendáis": "desentender",
    "desentendéis": "desentender",
    "desentendí": "desentender",
    "desentendía": "desentender",
    "desentendíais": "desentender",
    "desentendíamos": "desentender",
    "desentendían": "desentender",
    "desentendías": "desentender",
    "desenterraba": "desenterrar",
    "desenterrabais": "desenterrar",
    "desenterraban": "desenterrar",
    "desenterrabas": "desenterrar",
    "desenterrad": "desenterrar",
    "desenterrada": "desenterrar",
    "desenterradas": "desenterrar",
    "desenterrado": "desenterrar",
    "desenterradores": "desenterrador",
    "desenterrados": "desenterrar",
    "desenterramientos": "desenterramiento",
    "desenterramos": "desenterrar",
    "desenterrando": "desenterrar",
    "desenterrara": "desenterrar",
    "desenterrarais": "desenterrar",
    "desenterraran": "desenterrar",
    "desenterraras": "desenterrar",
    "desenterrare": "desenterrar",
    "desenterrareis": "desenterrar",
    "desenterraremos": "desenterrar",
    "desenterraren": "desenterrar",
    "desenterrares": "desenterrar",
    "desenterraron": "desenterrar",
    "desenterrará": "desenterrar",
    "desenterrarán": "desenterrar",
    "desenterrarás": "desenterrar",
    "desenterraré": "desenterrar",
    "desenterraréis": "desenterrar",
    "desenterraría": "desenterrar",
    "desenterraríais": "desenterrar",
    "desenterraríamos": "desenterrar",
    "desenterrarían": "desenterrar",
    "desenterrarías": "desenterrar",
    "desenterrase": "desenterrar",
    "desenterraseis": "desenterrar",
    "desenterrasen": "desenterrar",
    "desenterrases": "desenterrar",
    "desenterraste": "desenterrar",
    "desenterrasteis": "desenterrar",
    "desenterremos": "desenterrar",
    "desenterrábamos": "desenterrar",
    "desenterráis": "desenterrar",
    "desenterráramos": "desenterrar",
    "desenterráremos": "desenterrar",
    "desenterrásemos": "desenterrar",
    "desenterré": "desenterrar",
    "desenterréis": "desenterrar",
    "desenterró": "desenterrar",
    "desentienda": "desentender",
    "desentiendan": "desentender",
    "desentiendas": "desentender",
    "desentiende": "desentender",
    "desentienden": "desentender",
    "desentiendes": "desentender",
    "desentiendo": "desentender",
    "desentierra": "desenterrar",
    "desentierran": "desenterrar",
    "desentierras": "desenterrar",
    "desentierre": "desenterrar",
    "desentierren": "desenterrar",
    "desentierres": "desenterrar",
    "desentierro": "desenterrar",
    "desentolda": "desentoldar",
    "desentoldaba": "desentoldar",
    "desentoldabais": "desentoldar",
    "desentoldaban": "desentoldar",
    "desentoldabas": "desentoldar",
    "desentoldad": "desentoldar",
    "desentoldada": "desentoldar",
    "desentoldadas": "desentoldar",
    "desentoldado": "desentoldar",
    "desentoldados": "desentoldar",
    "desentoldamos": "desentoldar",
    "desentoldan": "desentoldar",
    "desentoldando": "desentoldar",
    "desentoldara": "desentoldar",
    "desentoldarais": "desentoldar",
    "desentoldaran": "desentoldar",
    "desentoldaras": "desentoldar",
    "desentoldare": "desentoldar",
    "desentoldareis": "desentoldar",
    "desentoldaremos": "desentoldar",
    "desentoldaren": "desentoldar",
    "desentoldares": "desentoldar",
    "desentoldaron": "desentoldar",
    "desentoldará": "desentoldar",
    "desentoldarán": "desentoldar",
    "desentoldarás": "desentoldar",
    "desentoldaré": "desentoldar",
    "desentoldaréis": "desentoldar",
    "desentoldaría": "desentoldar",
    "desentoldaríais": "desentoldar",
    "desentoldaríamos": "desentoldar",
    "desentoldarían": "desentoldar",
    "desentoldarías": "desentoldar",
    "desentoldas": "desentoldar",
    "desentoldase": "desentoldar",
    "desentoldaseis": "desentoldar",
    "desentoldasen": "desentoldar",
    "desentoldases": "desentoldar",
    "desentoldaste": "desentoldar",
    "desentoldasteis": "desentoldar",
    "desentolde": "desentoldar",
    "desentoldemos": "desentoldar",
    "desentolden": "desentoldar",
    "desentoldes": "desentoldar",
    "desentoldo": "desentoldar",
    "desentoldábamos": "desentoldar",
    "desentoldáis": "desentoldar",
    "desentoldáramos": "desentoldar",
    "desentoldáremos": "desentoldar",
    "desentoldásemos": "desentoldar",
    "desentoldé": "desentoldar",
    "desentoldéis": "desentoldar",
    "desentoldó": "desentoldar",
    "desentona": "desentonar",
    "desentonaba": "desentonar",
    "desentonabais": "desentonar",
    "desentonaban": "desentonar",
    "desentonabas": "desentonar",
    "desentonaciones": "desentonación",
    "desentonad": "desentonar",
    "desentonada": "desentonar",
    "desentonadas": "desentonar",
    "desentonado": "desentonar",
    "desentonados": "desentonar",
    "desentonamientos": "desentonamiento",
    "desentonamos": "desentonar",
    "desentonan": "desentonar",
    "desentonando": "desentonar",
    "desentonara": "desentonar",
    "desentonarais": "desentonar",
    "desentonaran": "desentonar",
    "desentonaras": "desentonar",
    "desentonare": "desentonar",
    "desentonareis": "desentonar",
    "desentonaremos": "desentonar",
    "desentonaren": "desentonar",
    "desentonares": "desentonar",
    "desentonaron": "desentonar",
    "desentonará": "desentonar",
    "desentonarán": "desentonar",
    "desentonarás": "desentonar",
    "desentonaré": "desentonar",
    "desentonaréis": "desentonar",
    "desentonaría": "desentonar",
    "desentonaríais": "desentonar",
    "desentonaríamos": "desentonar",
    "desentonarían": "desentonar",
    "desentonarías": "desentonar",
    "desentonas": "desentonar",
    "desentonase": "desentonar",
    "desentonaseis": "desentonar",
    "desentonasen": "desentonar",
    "desentonases": "desentonar",
    "desentonaste": "desentonar",
    "desentonasteis": "desentonar",
    "desentone": "desentonar",
    "desentonemos": "desentonar",
    "desentonen": "desentonar",
    "desentones": "desentonar",
    "desentono": "desentonar",
    "desentonos": "desentono",
    "desentonábamos": "desentonar",
    "desentonáis": "desentonar",
    "desentonáramos": "desentonar",
    "desentonáremos": "desentonar",
    "desentonásemos": "desentonar",
    "desentoné": "desentonar",
    "desentonéis": "desentonar",
    "desentonó": "desentonar",
    "desentorpece": "desentorpecer",
    "desentorpeced": "desentorpecer",
    "desentorpecemos": "desentorpecer",
    "desentorpecen": "desentorpecer",
    "desentorpeceremos": "desentorpecer",
    "desentorpecerá": "desentorpecer",
    "desentorpecerán": "desentorpecer",
    "desentorpecerás": "desentorpecer",
    "desentorpeceré": "desentorpecer",
    "desentorpeceréis": "desentorpecer",
    "desentorpecería": "desentorpecer",
    "desentorpeceríais": "desentorpecer",
    "desentorpeceríamos": "desentorpecer",
    "desentorpecerían": "desentorpecer",
    "desentorpecerías": "desentorpecer",
    "desentorpeces": "desentorpecer",
    "desentorpecida": "desentorpecer",
    "desentorpecidas": "desentorpecer",
    "desentorpecido": "desentorpecer",
    "desentorpecidos": "desentorpecer",
    "desentorpeciendo": "desentorpecer",
    "desentorpeciera": "desentorpecer",
    "desentorpecierais": "desentorpecer",
    "desentorpecieran": "desentorpecer",
    "desentorpecieras": "desentorpecer",
    "desentorpeciere": "desentorpecer",
    "desentorpeciereis": "desentorpecer",
    "desentorpecieren": "desentorpecer",
    "desentorpecieres": "desentorpecer",
    "desentorpecieron": "desentorpecer",
    "desentorpeciese": "desentorpecer",
    "desentorpecieseis": "desentorpecer",
    "desentorpeciesen": "desentorpecer",
    "desentorpecieses": "desentorpecer",
    "desentorpecimientos": "desentorpecimiento",
    "desentorpecimos": "desentorpecer",
    "desentorpeciste": "desentorpecer",
    "desentorpecisteis": "desentorpecer",
    "desentorpeciéramos": "desentorpecer",
    "desentorpeciéremos": "desentorpecer",
    "desentorpeciésemos": "desentorpecer",
    "desentorpeció": "desentorpecer",
    "desentorpecéis": "desentorpecer",
    "desentorpecí": "desentorpecer",
    "desentorpecía": "desentorpecer",
    "desentorpecíais": "desentorpecer",
    "desentorpecíamos": "desentorpecer",
    "desentorpecían": "desentorpecer",
    "desentorpecías": "desentorpecer",
    "desentorpezca": "desentorpecer",
    "desentorpezcamos": "desentorpecer",
    "desentorpezcan": "desentorpecer",
    "desentorpezcas": "desentorpecer",
    "desentorpezco": "desentorpecer",
    "desentorpezcáis": "desentorpecer",
    "desentrampa": "desentrampar",
    "desentrampaba": "desentrampar",
    "desentrampabais": "desentrampar",
    "desentrampaban": "desentrampar",
    "desentrampabas": "desentrampar",
    "desentrampad": "desentrampar",
    "desentrampada": "desentrampar",
    "desentrampadas": "desentrampar",
    "desentrampado": "desentrampar",
    "desentrampados": "desentrampar",
    "desentrampamos": "desentrampar",
    "desentrampan": "desentrampar",
    "desentrampando": "desentrampar",
    "desentrampara": "desentrampar",
    "desentramparais": "desentrampar",
    "desentramparan": "desentrampar",
    "desentramparas": "desentrampar",
    "desentrampare": "desentrampar",
    "desentrampareis": "desentrampar",
    "desentramparemos": "desentrampar",
    "desentramparen": "desentrampar",
    "desentrampares": "desentrampar",
    "desentramparon": "desentrampar",
    "desentrampará": "desentrampar",
    "desentramparán": "desentrampar",
    "desentramparás": "desentrampar",
    "desentramparé": "desentrampar",
    "desentramparéis": "desentrampar",
    "desentramparía": "desentrampar",
    "desentramparíais": "desentrampar",
    "desentramparíamos": "desentrampar",
    "desentramparían": "desentrampar",
    "desentramparías": "desentrampar",
    "desentrampas": "desentrampar",
    "desentrampase": "desentrampar",
    "desentrampaseis": "desentrampar",
    "desentrampasen": "desentrampar",
    "desentrampases": "desentrampar",
    "desentrampaste": "desentrampar",
    "desentrampasteis": "desentrampar",
    "desentrampe": "desentrampar",
    "desentrampemos": "desentrampar",
    "desentrampen": "desentrampar",
    "desentrampes": "desentrampar",
    "desentrampo": "desentrampar",
    "desentrampábamos": "desentrampar",
    "desentrampáis": "desentrampar",
    "desentrampáramos": "desentrampar",
    "desentrampáremos": "desentrampar",
    "desentrampásemos": "desentrampar",
    "desentrampé": "desentrampar",
    "desentrampéis": "desentrampar",
    "desentrampó": "desentrampar",
    "desentraña": "desentrañar",
    "desentrañaba": "desentrañar",
    "desentrañabais": "desentrañar",
    "desentrañaban": "desentrañar",
    "desentrañabas": "desentrañar",
    "desentrañad": "desentrañar",
    "desentrañada": "desentrañar",
    "desentrañadas": "desentrañar",
    "desentrañado": "desentrañar",
    "desentrañados": "desentrañar",
    "desentrañamientos": "desentrañamiento",
    "desentrañamos": "desentrañar",
    "desentrañan": "desentrañar",
    "desentrañando": "desentrañar",
    "desentrañara": "desentrañar",
    "desentrañarais": "desentrañar",
    "desentrañaran": "desentrañar",
    "desentrañaras": "desentrañar",
    "desentrañare": "desentrañar",
    "desentrañareis": "desentrañar",
    "desentrañaremos": "desentrañar",
    "desentrañaren": "desentrañar",
    "desentrañares": "desentrañar",
    "desentrañaron": "desentrañar",
    "desentrañará": "desentrañar",
    "desentrañarán": "desentrañar",
    "desentrañarás": "desentrañar",
    "desentrañaré": "desentrañar",
    "desentrañaréis": "desentrañar",
    "desentrañaría": "desentrañar",
    "desentrañaríais": "desentrañar",
    "desentrañaríamos": "desentrañar",
    "desentrañarían": "desentrañar",
    "desentrañarías": "desentrañar",
    "desentrañas": "desentrañar",
    "desentrañase": "desentrañar",
    "desentrañaseis": "desentrañar",
    "desentrañasen": "desentrañar",
    "desentrañases": "desentrañar",
    "desentrañaste": "desentrañar",
    "desentrañasteis": "desentrañar",
    "desentrañe": "desentrañar",
    "desentrañemos": "desentrañar",
    "desentrañen": "desentrañar",
    "desentrañes": "desentrañar",
    "desentraño": "desentrañar",
    "desentrañábamos": "desentrañar",
    "desentrañáis": "desentrañar",
    "desentrañáramos": "desentrañar",
    "desentrañáremos": "desentrañar",
    "desentrañásemos": "desentrañar",
    "desentrañé": "desentrañar",
    "desentrañéis": "desentrañar",
    "desentrañó": "desentrañar",
    "desentrena": "desentrenar",
    "desentrenaba": "desentrenar",
    "desentrenabais": "desentrenar",
    "desentrenaban": "desentrenar",
    "desentrenabas": "desentrenar",
    "desentrenad": "desentrenar",
    "desentrenada": "desentrenar",
    "desentrenadas": "desentrenar",
    "desentrenado": "desentrenar",
    "desentrenados": "desentrenar",
    "desentrenamientos": "desentrenamiento",
    "desentrenamos": "desentrenar",
    "desentrenan": "desentrenar",
    "desentrenando": "desentrenar",
    "desentrenara": "desentrenar",
    "desentrenarais": "desentrenar",
    "desentrenaran": "desentrenar",
    "desentrenaras": "desentrenar",
    "desentrenare": "desentrenar",
    "desentrenareis": "desentrenar",
    "desentrenaremos": "desentrenar",
    "desentrenaren": "desentrenar",
    "desentrenares": "desentrenar",
    "desentrenaron": "desentrenar",
    "desentrenará": "desentrenar",
    "desentrenarán": "desentrenar",
    "desentrenarás": "desentrenar",
    "desentrenaré": "desentrenar",
    "desentrenaréis": "desentrenar",
    "desentrenaría": "desentrenar",
    "desentrenaríais": "desentrenar",
    "desentrenaríamos": "desentrenar",
    "desentrenarían": "desentrenar",
    "desentrenarías": "desentrenar",
    "desentrenas": "desentrenar",
    "desentrenase": "desentrenar",
    "desentrenaseis": "desentrenar",
    "desentrenasen": "desentrenar",
    "desentrenases": "desentrenar",
    "desentrenaste": "desentrenar",
    "desentrenasteis": "desentrenar",
    "desentrene": "desentrenar",
    "desentrenemos": "desentrenar",
    "desentrenen": "desentrenar",
    "desentrenes": "desentrenar",
    "desentreno": "desentrenar",
    "desentrenábamos": "desentrenar",
    "desentrenáis": "desentrenar",
    "desentrenáramos": "desentrenar",
    "desentrenáremos": "desentrenar",
    "desentrenásemos": "desentrenar",
    "desentrené": "desentrenar",
    "desentrenéis": "desentrenar",
    "desentrenó": "desentrenar",
    "desentronizaciones": "desentronización",
    "desentuma": "desentumir",
    "desentumamos": "desentumir",
    "desentuman": "desentumir",
    "desentumas": "desentumir",
    "desentume": "desentumir",
    "desentumece": "desentumecer",
    "desentumeced": "desentumecer",
    "desentumecemos": "desentumecer",
    "desentumecen": "desentumecer",
    "desentumeceremos": "desentumecer",
    "desentumecerá": "desentumecer",
    "desentumecerán": "desentumecer",
    "desentumecerás": "desentumecer",
    "desentumeceré": "desentumecer",
    "desentumeceréis": "desentumecer",
    "desentumecería": "desentumecer",
    "desentumeceríais": "desentumecer",
    "desentumeceríamos": "desentumecer",
    "desentumecerían": "desentumecer",
    "desentumecerías": "desentumecer",
    "desentumeces": "desentumecer",
    "desentumecida": "desentumecer",
    "desentumecidas": "desentumecer",
    "desentumecido": "desentumecer",
    "desentumecidos": "desentumecer",
    "desentumeciendo": "desentumecer",
    "desentumeciera": "desentumecer",
    "desentumecierais": "desentumecer",
    "desentumecieran": "desentumecer",
    "desentumecieras": "desentumecer",
    "desentumeciere": "desentumecer",
    "desentumeciereis": "desentumecer",
    "desentumecieren": "desentumecer",
    "desentumecieres": "desentumecer",
    "desentumecieron": "desentumecer",
    "desentumeciese": "desentumecer",
    "desentumecieseis": "desentumecer",
    "desentumeciesen": "desentumecer",
    "desentumecieses": "desentumecer",
    "desentumecimientos": "desentumecimiento",
    "desentumecimos": "desentumecer",
    "desentumeciste": "desentumecer",
    "desentumecisteis": "desentumecer",
    "desentumeciéramos": "desentumecer",
    "desentumeciéremos": "desentumecer",
    "desentumeciésemos": "desentumecer",
    "desentumeció": "desentumecer",
    "desentumecéis": "desentumecer",
    "desentumecí": "desentumecer",
    "desentumecía": "desentumecer",
    "desentumecíais": "desentumecer",
    "desentumecíamos": "desentumecer",
    "desentumecían": "desentumecer",
    "desentumecías": "desentumecer",
    "desentumen": "desentumir",
    "desentumes": "desentumir",
    "desentumezca": "desentumecer",
    "desentumezcamos": "desentumecer",
    "desentumezcan": "desentumecer",
    "desentumezcas": "desentumecer",
    "desentumezco": "desentumecer",
    "desentumezcáis": "desentumecer",
    "desentumid": "desentumir",
    "desentumida": "desentumir",
    "desentumidas": "desentumir",
    "desentumido": "desentumir",
    "desentumidos": "desentumir",
    "desentumiendo": "desentumir",
    "desentumiera": "desentumir",
    "desentumierais": "desentumir",
    "desentumieran": "desentumir",
    "desentumieras": "desentumir",
    "desentumiere": "desentumir",
    "desentumiereis": "desentumir",
    "desentumieren": "desentumir",
    "desentumieres": "desentumir",
    "desentumieron": "desentumir",
    "desentumiese": "desentumir",
    "desentumieseis": "desentumir",
    "desentumiesen": "desentumir",
    "desentumieses": "desentumir",
    "desentumimos": "desentumir",
    "desentumiremos": "desentumir",
    "desentumirá": "desentumir",
    "desentumirán": "desentumir",
    "desentumirás": "desentumir",
    "desentumiré": "desentumir",
    "desentumiréis": "desentumir",
    "desentumiría": "desentumir",
    "desentumiríais": "desentumir",
    "desentumiríamos": "desentumir",
    "desentumirían": "desentumir",
    "desentumirías": "desentumir",
    "desentumiste": "desentumir",
    "desentumisteis": "desentumir",
    "desentumiéramos": "desentumir",
    "desentumiéremos": "desentumir",
    "desentumiésemos": "desentumir",
    "desentumió": "desentumir",
    "desentumo": "desentumir",
    "desentumáis": "desentumir",
    "desentumí": "desentumir",
    "desentumía": "desentumir",
    "desentumíais": "desentumir",
    "desentumíamos": "desentumir",
    "desentumían": "desentumir",
    "desentumías": "desentumir",
    "desentumís": "desentumir",
    "desenvaina": "desenvainar",
    "desenvainaba": "desenvainar",
    "desenvainabais": "desenvainar",
    "desenvainaban": "desenvainar",
    "desenvainabas": "desenvainar",
    "desenvainad": "desenvainar",
    "desenvainada": "desenvainar",
    "desenvainadas": "desenvainar",
    "desenvainado": "desenvainar",
    "desenvainados": "desenvainar",
    "desenvainamos": "desenvainar",
    "desenvainan": "desenvainar",
    "desenvainando": "desenvainar",
    "desenvainara": "desenvainar",
    "desenvainarais": "desenvainar",
    "desenvainaran": "desenvainar",
    "desenvainaras": "desenvainar",
    "desenvainare": "desenvainar",
    "desenvainareis": "desenvainar",
    "desenvainaremos": "desenvainar",
    "desenvainaren": "desenvainar",
    "desenvainares": "desenvainar",
    "desenvainaron": "desenvainar",
    "desenvainará": "desenvainar",
    "desenvainarán": "desenvainar",
    "desenvainarás": "desenvainar",
    "desenvainaré": "desenvainar",
    "desenvainaréis": "desenvainar",
    "desenvainaría": "desenvainar",
    "desenvainaríais": "desenvainar",
    "desenvainaríamos": "desenvainar",
    "desenvainarían": "desenvainar",
    "desenvainarías": "desenvainar",
    "desenvainas": "desenvainar",
    "desenvainase": "desenvainar",
    "desenvainaseis": "desenvainar",
    "desenvainasen": "desenvainar",
    "desenvainases": "desenvainar",
    "desenvainaste": "desenvainar",
    "desenvainasteis": "desenvainar",
    "desenvaine": "desenvainar",
    "desenvainemos": "desenvainar",
    "desenvainen": "desenvainar",
    "desenvaines": "desenvainar",
    "desenvaino": "desenvainar",
    "desenvainábamos": "desenvainar",
    "desenvaináis": "desenvainar",
    "desenvaináramos": "desenvainar",
    "desenvaináremos": "desenvainar",
    "desenvainásemos": "desenvainar",
    "desenvainé": "desenvainar",
    "desenvainéis": "desenvainar",
    "desenvainó": "desenvainar",
    "desenverga": "desenvergar",
    "desenvergaba": "desenvergar",
    "desenvergabais": "desenvergar",
    "desenvergaban": "desenvergar",
    "desenvergabas": "desenvergar",
    "desenvergad": "desenvergar",
    "desenvergada": "desenvergar",
    "desenvergadas": "desenvergar",
    "desenvergado": "desenvergar",
    "desenvergados": "desenvergar",
    "desenvergamos": "desenvergar",
    "desenvergan": "desenvergar",
    "desenvergando": "desenvergar",
    "desenvergara": "desenvergar",
    "desenvergarais": "desenvergar",
    "desenvergaran": "desenvergar",
    "desenvergaras": "desenvergar",
    "desenvergare": "desenvergar",
    "desenvergareis": "desenvergar",
    "desenvergaremos": "desenvergar",
    "desenvergaren": "desenvergar",
    "desenvergares": "desenvergar",
    "desenvergaron": "desenvergar",
    "desenvergará": "desenvergar",
    "desenvergarán": "desenvergar",
    "desenvergarás": "desenvergar",
    "desenvergaré": "desenvergar",
    "desenvergaréis": "desenvergar",
    "desenvergaría": "desenvergar",
    "desenvergaríais": "desenvergar",
    "desenvergaríamos": "desenvergar",
    "desenvergarían": "desenvergar",
    "desenvergarías": "desenvergar",
    "desenvergas": "desenvergar",
    "desenvergase": "desenvergar",
    "desenvergaseis": "desenvergar",
    "desenvergasen": "desenvergar",
    "desenvergases": "desenvergar",
    "desenvergaste": "desenvergar",
    "desenvergasteis": "desenvergar",
    "desenvergo": "desenvergar",
    "desenvergue": "desenvergar",
    "desenverguemos": "desenvergar",
    "desenverguen": "desenvergar",
    "desenvergues": "desenvergar",
    "desenvergué": "desenvergar",
    "desenverguéis": "desenvergar",
    "desenvergábamos": "desenvergar",
    "desenvergáis": "desenvergar",
    "desenvergáramos": "desenvergar",
    "desenvergáremos": "desenvergar",
    "desenvergásemos": "desenvergar",
    "desenvergó": "desenvergar",
    "desenvolturas": "desenvoltura",
    "desenvolvamos": "desenvolver",
    "desenvolved": "desenvolver",
    "desenvolvemos": "desenvolver",
    "desenvolveremos": "desenvolver",
    "desenvolverá": "desenvolver",
    "desenvolverán": "desenvolver",
    "desenvolverás": "desenvolver",
    "desenvolveré": "desenvolver",
    "desenvolveréis": "desenvolver",
    "desenvolvería": "desenvolver",
    "desenvolveríais": "desenvolver",
    "desenvolveríamos": "desenvolver",
    "desenvolverían": "desenvolver",
    "desenvolverías": "desenvolver",
    "desenvolviendo": "desenvolver",
    "desenvolviera": "desenvolver",
    "desenvolvierais": "desenvolver",
    "desenvolvieran": "desenvolver",
    "desenvolvieras": "desenvolver",
    "desenvolviere": "desenvolver",
    "desenvolviereis": "desenvolver",
    "desenvolvieren": "desenvolver",
    "desenvolvieres": "desenvolver",
    "desenvolvieron": "desenvolver",
    "desenvolviese": "desenvolver",
    "desenvolvieseis": "desenvolver",
    "desenvolviesen": "desenvolver",
    "desenvolvieses": "desenvolver",
    "desenvolvimientos": "desenvolvimiento",
    "desenvolvimos": "desenvolver",
    "desenvolviste": "desenvolver",
    "desenvolvisteis": "desenvolver",
    "desenvolviéramos": "desenvolver",
    "desenvolviéremos": "desenvolver",
    "desenvolviésemos": "desenvolver",
    "desenvolvió": "desenvolver",
    "desenvolváis": "desenvolver",
    "desenvolvéis": "desenvolver",
    "desenvolví": "desenvolver",
    "desenvolvía": "desenvolver",
    "desenvolvíais": "desenvolver",
    "desenvolvíamos": "desenvolver",
    "desenvolvían": "desenvolver",
    "desenvolvías": "desenvolver",
    "desenvuelta": "desenvolver",
    "desenvueltas": "desenvolver",
    "desenvuelto": "desenvolver",
    "desenvueltos": "desenvolver",
    "desenvuelva": "desenvolver",
    "desenvuelvan": "desenvolver",
    "desenvuelvas": "desenvolver",
    "desenvuelve": "desenvolver",
    "desenvuelven": "desenvolver",
    "desenvuelves": "desenvolver",
    "desenvuelvo": "desenvolver",
    "desenzarce": "desenzarzar",
    "desenzarcemos": "desenzarzar",
    "desenzarcen": "desenzarzar",
    "desenzarces": "desenzarzar",
    "desenzarcé": "desenzarzar",
    "desenzarcéis": "desenzarzar",
    "desenzarza": "desenzarzar",
    "desenzarzaba": "desenzarzar",
    "desenzarzabais": "desenzarzar",
    "desenzarzaban": "desenzarzar",
    "desenzarzabas": "desenzarzar",
    "desenzarzad": "desenzarzar",
    "desenzarzada": "desenzarzar",
    "desenzarzadas": "desenzarzar",
    "desenzarzado": "desenzarzar",
    "desenzarzados": "desenzarzar",
    "desenzarzamientos": "desenzarzamiento",
    "desenzarzamos": "desenzarzar",
    "desenzarzan": "desenzarzar",
    "desenzarzando": "desenzarzar",
    "desenzarzara": "desenzarzar",
    "desenzarzarais": "desenzarzar",
    "desenzarzaran": "desenzarzar",
    "desenzarzaras": "desenzarzar",
    "desenzarzare": "desenzarzar",
    "desenzarzareis": "desenzarzar",
    "desenzarzaremos": "desenzarzar",
    "desenzarzaren": "desenzarzar",
    "desenzarzares": "desenzarzar",
    "desenzarzaron": "desenzarzar",
    "desenzarzará": "desenzarzar",
    "desenzarzarán": "desenzarzar",
    "desenzarzarás": "desenzarzar",
    "desenzarzaré": "desenzarzar",
    "desenzarzaréis": "desenzarzar",
    "desenzarzaría": "desenzarzar",
    "desenzarzaríais": "desenzarzar",
    "desenzarzaríamos": "desenzarzar",
    "desenzarzarían": "desenzarzar",
    "desenzarzarías": "desenzarzar",
    "desenzarzas": "desenzarzar",
    "desenzarzase": "desenzarzar",
    "desenzarzaseis": "desenzarzar",
    "desenzarzasen": "desenzarzar",
    "desenzarzases": "desenzarzar",
    "desenzarzaste": "desenzarzar",
    "desenzarzasteis": "desenzarzar",
    "desenzarzo": "desenzarzar",
    "desenzarzábamos": "desenzarzar",
    "desenzarzáis": "desenzarzar",
    "desenzarzáramos": "desenzarzar",
    "desenzarzáremos": "desenzarzar",
    "desenzarzásemos": "desenzarzar",
    "desenzarzó": "desenzarzar",
    "deseo": "desear",
    "deseos": "deseo",
    "deseosa": "deseoso",
    "deseosas": "deseoso",
    "deseosos": "deseoso",
    "deseque": "desecar",
    "desequemos": "desecar",
    "desequen": "desecar",
    "deseques": "desecar",
    "desequida": "desequido",
    "desequidas": "desequido",
    "desequidos": "desequido",
    "desequilibra": "desequilibrar",
    "desequilibraba": "desequilibrar",
    "desequilibrabais": "desequilibrar",
    "desequilibraban": "desequilibrar",
    "desequilibrabas": "desequilibrar",
    "desequilibrad": "desequilibrar",
    "desequilibrada": "desequilibrar",
    "desequilibradas": "desequilibrar",
    "desequilibrado": "desequilibrar",
    "desequilibrados": "desequilibrar",
    "desequilibramos": "desequilibrar",
    "desequilibran": "desequilibrar",
    "desequilibrando": "desequilibrar",
    "desequilibrantes": "desequilibrante",
    "desequilibrara": "desequilibrar",
    "desequilibrarais": "desequilibrar",
    "desequilibraran": "desequilibrar",
    "desequilibraras": "desequilibrar",
    "desequilibrare": "desequilibrar",
    "desequilibrareis": "desequilibrar",
    "desequilibraremos": "desequilibrar",
    "desequilibraren": "desequilibrar",
    "desequilibrares": "desequilibrar",
    "desequilibraron": "desequilibrar",
    "desequilibrará": "desequilibrar",
    "desequilibrarán": "desequilibrar",
    "desequilibrarás": "desequilibrar",
    "desequilibraré": "desequilibrar",
    "desequilibraréis": "desequilibrar",
    "desequilibraría": "desequilibrar",
    "desequilibraríais": "desequilibrar",
    "desequilibraríamos": "desequilibrar",
    "desequilibrarían": "desequilibrar",
    "desequilibrarías": "desequilibrar",
    "desequilibras": "desequilibrar",
    "desequilibrase": "desequilibrar",
    "desequilibraseis": "desequilibrar",
    "desequilibrasen": "desequilibrar",
    "desequilibrases": "desequilibrar",
    "desequilibraste": "desequilibrar",
    "desequilibrasteis": "desequilibrar",
    "desequilibre": "desequilibrar",
    "desequilibremos": "desequilibrar",
    "desequilibren": "desequilibrar",
    "desequilibres": "desequilibrar",
    "desequilibrios": "desequilibrio",
    "desequilibro": "desequilibrar",
    "desequilibrábamos": "desequilibrar",
    "desequilibráis": "desequilibrar",
    "desequilibráramos": "desequilibrar",
    "desequilibráremos": "desequilibrar",
    "desequilibrásemos": "desequilibrar",
    "desequilibré": "desequilibrar",
    "desequilibréis": "desequilibrar",
    "desequilibró": "desequilibrar",
    "desequé": "desecar",
    "desequéis": "desecar",
    "deserciones": "deserción",
    "deserpidinas": "deserpidina",
    "deserrada": "deserrado",
    "deserradas": "deserrado",
    "deserrados": "deserrado",
    "deserta": "desertar",
    "desertaba": "desertar",
    "desertabais": "desertar",
    "desertaban": "desertar",
    "desertabas": "desertar",
    "desertad": "desertar",
    "desertada": "desertar",
    "desertadas": "desertar",
    "desertado": "desertar",
    "desertados": "desertar",
    "desertamos": "desertar",
    "desertan": "desertar",
    "desertando": "desertar",
    "desertara": "desertar",
    "desertarais": "desertar",
    "desertaran": "desertar",
    "desertaras": "desertar",
    "desertare": "desertar",
    "desertareis": "desertar",
    "desertaremos": "desertar",
    "desertaren": "desertar",
    "desertares": "desertar",
    "desertaron": "desertar",
    "desertará": "desertar",
    "desertarán": "desertar",
    "desertarás": "desertar",
    "desertaré": "desertar",
    "desertaréis": "desertar",
    "desertaría": "desertar",
    "desertaríais": "desertar",
    "desertaríamos": "desertar",
    "desertarían": "desertar",
    "desertarías": "desertar",
    "desertas": "desertar",
    "desertase": "desertar",
    "desertaseis": "desertar",
    "desertasen": "desertar",
    "desertases": "desertar",
    "desertaste": "desertar",
    "desertasteis": "desertar",
    "deserte": "desertar",
    "desertemos": "desertar",
    "deserten": "desertar",
    "desertes": "desertar",
    "desertificaciones": "desertificación",
    "desertizaciones": "desertización",
    "deserto": "desertar",
    "desertora": "desertor",
    "desertoras": "desertor",
    "desertores": "desertor",
    "desertábamos": "desertar",
    "desertáis": "desertar",
    "desertáramos": "desertar",
    "desertáremos": "desertar",
    "desertásemos": "desertar",
    "deserté": "desertar",
    "desertéis": "desertar",
    "desertícolas": "desertícola",
    "desertó": "desertar",
    "deservicios": "deservicio",
    "desescombra": "desescombrar",
    "desescombraba": "desescombrar",
    "desescombrabais": "desescombrar",
    "desescombraban": "desescombrar",
    "desescombrabas": "desescombrar",
    "desescombrad": "desescombrar",
    "desescombrada": "desescombrar",
    "desescombradas": "desescombrar",
    "desescombrado": "desescombrar",
    "desescombrados": "desescombrar",
    "desescombramos": "desescombrar",
    "desescombran": "desescombrar",
    "desescombrando": "desescombrar",
    "desescombrara": "desescombrar",
    "desescombrarais": "desescombrar",
    "desescombraran": "desescombrar",
    "desescombraras": "desescombrar",
    "desescombrare": "desescombrar",
    "desescombrareis": "desescombrar",
    "desescombraremos": "desescombrar",
    "desescombraren": "desescombrar",
    "desescombrares": "desescombrar",
    "desescombraron": "desescombrar",
    "desescombrará": "desescombrar",
    "desescombrarán": "desescombrar",
    "desescombrarás": "desescombrar",
    "desescombraré": "desescombrar",
    "desescombraréis": "desescombrar",
    "desescombraría": "desescombrar",
    "desescombraríais": "desescombrar",
    "desescombraríamos": "desescombrar",
    "desescombrarían": "desescombrar",
    "desescombrarías": "desescombrar",
    "desescombras": "desescombrar",
    "desescombrase": "desescombrar",
    "desescombraseis": "desescombrar",
    "desescombrasen": "desescombrar",
    "desescombrases": "desescombrar",
    "desescombraste": "desescombrar",
    "desescombrasteis": "desescombrar",
    "desescombre": "desescombrar",
    "desescombremos": "desescombrar",
    "desescombren": "desescombrar",
    "desescombres": "desescombrar",
    "desescombro": "desescombrar",
    "desescombros": "desescombro",
    "desescombrábamos": "desescombrar",
    "desescombráis": "desescombrar",
    "desescombráramos": "desescombrar",
    "desescombráremos": "desescombrar",
    "desescombrásemos": "desescombrar",
    "desescombré": "desescombrar",
    "desescombréis": "desescombrar",
    "desescombró": "desescombrar",
    "deseslabona": "deseslabonar",
    "deseslabonaba": "deseslabonar",
    "deseslabonabais": "deseslabonar",
    "deseslabonaban": "deseslabonar",
    "deseslabonabas": "deseslabonar",
    "deseslabonad": "deseslabonar",
    "deseslabonada": "deseslabonar",
    "deseslabonadas": "deseslabonar",
    "deseslabonado": "deseslabonar",
    "deseslabonados": "deseslabonar",
    "deseslabonamos": "deseslabonar",
    "deseslabonan": "deseslabonar",
    "deseslabonando": "deseslabonar",
    "deseslabonara": "deseslabonar",
    "deseslabonarais": "deseslabonar",
    "deseslabonaran": "deseslabonar",
    "deseslabonaras": "deseslabonar",
    "deseslabonare": "deseslabonar",
    "deseslabonareis": "deseslabonar",
    "deseslabonaremos": "deseslabonar",
    "deseslabonaren": "deseslabonar",
    "deseslabonares": "deseslabonar",
    "deseslabonaron": "deseslabonar",
    "deseslabonará": "deseslabonar",
    "deseslabonarán": "deseslabonar",
    "deseslabonarás": "deseslabonar",
    "deseslabonaré": "deseslabonar",
    "deseslabonaréis": "deseslabonar",
    "deseslabonaría": "deseslabonar",
    "deseslabonaríais": "deseslabonar",
    "deseslabonaríamos": "deseslabonar",
    "deseslabonarían": "deseslabonar",
    "deseslabonarías": "deseslabonar",
    "deseslabonas": "deseslabonar",
    "deseslabonase": "deseslabonar",
    "deseslabonaseis": "deseslabonar",
    "deseslabonasen": "deseslabonar",
    "deseslabonases": "deseslabonar",
    "deseslabonaste": "deseslabonar",
    "deseslabonasteis": "deseslabonar",
    "deseslabone": "deseslabonar",
    "deseslabonemos": "deseslabonar",
    "deseslabonen": "deseslabonar",
    "deseslabones": "deseslabonar",
    "deseslabono": "deseslabonar",
    "deseslabonábamos": "deseslabonar",
    "deseslabonáis": "deseslabonar",
    "deseslabonáramos": "deseslabonar",
    "deseslabonáremos": "deseslabonar",
    "deseslabonásemos": "deseslabonar",
    "deseslaboné": "deseslabonar",
    "deseslabonéis": "deseslabonar",
    "deseslabonó": "deseslabonar",
    "desespalda": "desespaldar",
    "desespaldaba": "desespaldar",
    "desespaldabais": "desespaldar",
    "desespaldaban": "desespaldar",
    "desespaldabas": "desespaldar",
    "desespaldad": "desespaldar",
    "desespaldada": "desespaldar",
    "desespaldadas": "desespaldar",
    "desespaldado": "desespaldar",
    "desespaldados": "desespaldar",
    "desespaldamos": "desespaldar",
    "desespaldan": "desespaldar",
    "desespaldando": "desespaldar",
    "desespaldara": "desespaldar",
    "desespaldarais": "desespaldar",
    "desespaldaran": "desespaldar",
    "desespaldaras": "desespaldar",
    "desespaldare": "desespaldar",
    "desespaldareis": "desespaldar",
    "desespaldaremos": "desespaldar",
    "desespaldaren": "desespaldar",
    "desespaldares": "desespaldar",
    "desespaldaron": "desespaldar",
    "desespaldará": "desespaldar",
    "desespaldarán": "desespaldar",
    "desespaldarás": "desespaldar",
    "desespaldaré": "desespaldar",
    "desespaldaréis": "desespaldar",
    "desespaldaría": "desespaldar",
    "desespaldaríais": "desespaldar",
    "desespaldaríamos": "desespaldar",
    "desespaldarían": "desespaldar",
    "desespaldarías": "desespaldar",
    "desespaldas": "desespaldar",
    "desespaldase": "desespaldar",
    "desespaldaseis": "desespaldar",
    "desespaldasen": "desespaldar",
    "desespaldases": "desespaldar",
    "desespaldaste": "desespaldar",
    "desespaldasteis": "desespaldar",
    "desespalde": "desespaldar",
    "desespaldemos": "desespaldar",
    "desespalden": "desespaldar",
    "desespaldes": "desespaldar",
    "desespaldo": "desespaldar",
    "desespaldábamos": "desespaldar",
    "desespaldáis": "desespaldar",
    "desespaldáramos": "desespaldar",
    "desespaldáremos": "desespaldar",
    "desespaldásemos": "desespaldar",
    "desespaldé": "desespaldar",
    "desespaldéis": "desespaldar",
    "desespaldó": "desespaldar",
    "desespera": "desesperar",
    "desesperaba": "desesperar",
    "desesperabais": "desesperar",
    "desesperaban": "desesperar",
    "desesperabas": "desesperar",
    "desesperaciones": "desesperación",
    "desesperad": "desesperar",
    "desesperada": "desesperar",
    "desesperadas": "desesperar",
    "desesperado": "desesperar",
    "desesperados": "desesperar",
    "desesperamos": "desesperar",
    "desesperan": "desesperar",
    "desesperance": "desesperanzar",
    "desesperancemos": "desesperanzar",
    "desesperancen": "desesperanzar",
    "desesperances": "desesperanzar",
    "desesperancé": "desesperanzar",
    "desesperancéis": "desesperanzar",
    "desesperando": "desesperar",
    "desesperantes": "desesperante",
    "desesperanza": "desesperanzar",
    "desesperanzaba": "desesperanzar",
    "desesperanzabais": "desesperanzar",
    "desesperanzaban": "desesperanzar",
    "desesperanzabas": "desesperanzar",
    "desesperanzad": "desesperanzar",
    "desesperanzada": "desesperanzar",
    "desesperanzadas": "desesperanzar",
    "desesperanzado": "desesperanzar",
    "desesperanzadora": "desesperanzador",
    "desesperanzadoras": "desesperanzador",
    "desesperanzadores": "desesperanzador",
    "desesperanzados": "desesperanzar",
    "desesperanzamos": "desesperanzar",
    "desesperanzan": "desesperanzar",
    "desesperanzando": "desesperanzar",
    "desesperanzara": "desesperanzar",
    "desesperanzarais": "desesperanzar",
    "desesperanzaran": "desesperanzar",
    "desesperanzaras": "desesperanzar",
    "desesperanzare": "desesperanzar",
    "desesperanzareis": "desesperanzar",
    "desesperanzaremos": "desesperanzar",
    "desesperanzaren": "desesperanzar",
    "desesperanzares": "desesperanzar",
    "desesperanzaron": "desesperanzar",
    "desesperanzará": "desesperanzar",
    "desesperanzarán": "desesperanzar",
    "desesperanzarás": "desesperanzar",
    "desesperanzaré": "desesperanzar",
    "desesperanzaréis": "desesperanzar",
    "desesperanzaría": "desesperanzar",
    "desesperanzaríais": "desesperanzar",
    "desesperanzaríamos": "desesperanzar",
    "desesperanzarían": "desesperanzar",
    "desesperanzarías": "desesperanzar",
    "desesperanzas": "desesperanzar",
    "desesperanzase": "desesperanzar",
    "desesperanzaseis": "desesperanzar",
    "desesperanzasen": "desesperanzar",
    "desesperanzases": "desesperanzar",
    "desesperanzaste": "desesperanzar",
    "desesperanzasteis": "desesperanzar",
    "desesperanzo": "desesperanzar",
    "desesperanzábamos": "desesperanzar",
    "desesperanzáis": "desesperanzar",
    "desesperanzáramos": "desesperanzar",
    "desesperanzáremos": "desesperanzar",
    "desesperanzásemos": "desesperanzar",
    "desesperanzó": "desesperanzar",
    "desesperara": "desesperar",
    "desesperarais": "desesperar",
    "desesperaran": "desesperar",
    "desesperaras": "desesperar",
    "desesperare": "desesperar",
    "desesperareis": "desesperar",
    "desesperaremos": "desesperar",
    "desesperaren": "desesperar",
    "desesperares": "desesperar",
    "desesperaron": "desesperar",
    "desesperará": "desesperar",
    "desesperarán": "desesperar",
    "desesperarás": "desesperar",
    "desesperaré": "desesperar",
    "desesperaréis": "desesperar",
    "desesperaría": "desesperar",
    "desesperaríais": "desesperar",
    "desesperaríamos": "desesperar",
    "desesperarían": "desesperar",
    "desesperarías": "desesperar",
    "desesperas": "desesperar",
    "desesperase": "desesperar",
    "desesperaseis": "desesperar",
    "desesperasen": "desesperar",
    "desesperases": "desesperar",
    "desesperaste": "desesperar",
    "desesperasteis": "desesperar",
    "desespere": "desesperar",
    "desesperemos": "desesperar",
    "desesperen": "desesperar",
    "desesperes": "desesperar",
    "desespero": "desesperar",
    "desesperos": "desespero",
    "desesperábamos": "desesperar",
    "desesperáis": "desesperar",
    "desesperáramos": "desesperar",
    "desesperáremos": "desesperar",
    "desesperásemos": "desesperar",
    "desesperé": "desesperar",
    "desesperéis": "desesperar",
    "desesperó": "desesperar",
    "desestabilice": "desestabilizar",
    "desestabilicemos": "desestabilizar",
    "desestabilicen": "desestabilizar",
    "desestabilices": "desestabilizar",
    "desestabilicé": "desestabilizar",
    "desestabilicéis": "desestabilizar",
    "desestabiliza": "desestabilizar",
    "desestabilizaba": "desestabilizar",
    "desestabilizabais": "desestabilizar",
    "desestabilizaban": "desestabilizar",
    "desestabilizabas": "desestabilizar",
    "desestabilizaciones": "desestabilización",
    "desestabilizad": "desestabilizar",
    "desestabilizada": "desestabilizar",
    "desestabilizadas": "desestabilizar",
    "desestabilizado": "desestabilizar",
    "desestabilizados": "desestabilizar",
    "desestabilizamos": "desestabilizar",
    "desestabilizan": "desestabilizar",
    "desestabilizando": "desestabilizar",
    "desestabilizara": "desestabilizar",
    "desestabilizarais": "desestabilizar",
    "desestabilizaran": "desestabilizar",
    "desestabilizaras": "desestabilizar",
    "desestabilizare": "desestabilizar",
    "desestabilizareis": "desestabilizar",
    "desestabilizaremos": "desestabilizar",
    "desestabilizaren": "desestabilizar",
    "desestabilizares": "desestabilizar",
    "desestabilizaron": "desestabilizar",
    "desestabilizará": "desestabilizar",
    "desestabilizarán": "desestabilizar",
    "desestabilizarás": "desestabilizar",
    "desestabilizaré": "desestabilizar",
    "desestabilizaréis": "desestabilizar",
    "desestabilizaría": "desestabilizar",
    "desestabilizaríais": "desestabilizar",
    "desestabilizaríamos": "desestabilizar",
    "desestabilizarían": "desestabilizar",
    "desestabilizarías": "desestabilizar",
    "desestabilizas": "desestabilizar",
    "desestabilizase": "desestabilizar",
    "desestabilizaseis": "desestabilizar",
    "desestabilizasen": "desestabilizar",
    "desestabilizases": "desestabilizar",
    "desestabilizaste": "desestabilizar",
    "desestabilizasteis": "desestabilizar",
    "desestabilizo": "desestabilizar",
    "desestabilizábamos": "desestabilizar",
    "desestabilizáis": "desestabilizar",
    "desestabilizáramos": "desestabilizar",
    "desestabilizáremos": "desestabilizar",
    "desestabilizásemos": "desestabilizar",
    "desestabilizó": "desestabilizar",
    "desestacionalizaciones": "desestacionalización",
    "desestalinizaciones": "desestalinización",
    "desestancamientos": "desestancamiento",
    "desestancos": "desestanco",
    "desestaña": "desestañar",
    "desestañaba": "desestañar",
    "desestañabais": "desestañar",
    "desestañaban": "desestañar",
    "desestañabas": "desestañar",
    "desestañad": "desestañar",
    "desestañada": "desestañar",
    "desestañadas": "desestañar",
    "desestañado": "desestañar",
    "desestañados": "desestañar",
    "desestañamos": "desestañar",
    "desestañan": "desestañar",
    "desestañando": "desestañar",
    "desestañara": "desestañar",
    "desestañarais": "desestañar",
    "desestañaran": "desestañar",
    "desestañaras": "desestañar",
    "desestañare": "desestañar",
    "desestañareis": "desestañar",
    "desestañaremos": "desestañar",
    "desestañaren": "desestañar",
    "desestañares": "desestañar",
    "desestañaron": "desestañar",
    "desestañará": "desestañar",
    "desestañarán": "desestañar",
    "desestañarás": "desestañar",
    "desestañaré": "desestañar",
    "desestañaréis": "desestañar",
    "desestañaría": "desestañar",
    "desestañaríais": "desestañar",
    "desestañaríamos": "desestañar",
    "desestañarían": "desestañar",
    "desestañarías": "desestañar",
    "desestañas": "desestañar",
    "desestañase": "desestañar",
    "desestañaseis": "desestañar",
    "desestañasen": "desestañar",
    "desestañases": "desestañar",
    "desestañaste": "desestañar",
    "desestañasteis": "desestañar",
    "desestañe": "desestañar",
    "desestañemos": "desestañar",
    "desestañen": "desestañar",
    "desestañes": "desestañar",
    "desestaño": "desestañar",
    "desestañábamos": "desestañar",
    "desestañáis": "desestañar",
    "desestañáramos": "desestañar",
    "desestañáremos": "desestañar",
    "desestañásemos": "desestañar",
    "desestañé": "desestañar",
    "desestañéis": "desestañar",
    "desestañó": "desestañar",
    "desestera": "desesterar",
    "desesteraba": "desesterar",
    "desesterabais": "desesterar",
    "desesteraban": "desesterar",
    "desesterabas": "desesterar",
    "desesterad": "desesterar",
    "desesterada": "desesterar",
    "desesteradas": "desesterar",
    "desesterado": "desesterar",
    "desesterados": "desesterar",
    "desesteramos": "desesterar",
    "desesteran": "desesterar",
    "desesterando": "desesterar",
    "desesterara": "desesterar",
    "desesterarais": "desesterar",
    "desesteraran": "desesterar",
    "desesteraras": "desesterar",
    "desesterare": "desesterar",
    "desesterareis": "desesterar",
    "desesteraremos": "desesterar",
    "desesteraren": "desesterar",
    "desesterares": "desesterar",
    "desesteraron": "desesterar",
    "desesterará": "desesterar",
    "desesterarán": "desesterar",
    "desesterarás": "desesterar",
    "desesteraré": "desesterar",
    "desesteraréis": "desesterar",
    "desesteraría": "desesterar",
    "desesteraríais": "desesterar",
    "desesteraríamos": "desesterar",
    "desesterarían": "desesterar",
    "desesterarías": "desesterar",
    "desesteras": "desesterar",
    "desesterase": "desesterar",
    "desesteraseis": "desesterar",
    "desesterasen": "desesterar",
    "desesterases": "desesterar",
    "desesteraste": "desesterar",
    "desesterasteis": "desesterar",
    "desestere": "desesterar",
    "desesteremos": "desesterar",
    "desesteren": "desesterar",
    "desesteres": "desesterar",
    "desestero": "desesterar",
    "desesteros": "desestero",
    "desesterábamos": "desesterar",
    "desesteráis": "desesterar",
    "desesteráramos": "desesterar",
    "desesteráremos": "desesterar",
    "desesterásemos": "desesterar",
    "desesteré": "desesterar",
    "desesteréis": "desesterar",
    "desesteró": "desesterar",
    "desestima": "desestimar",
    "desestimaba": "desestimar",
    "desestimabais": "desestimar",
    "desestimaban": "desestimar",
    "desestimabas": "desestimar",
    "desestimaciones": "desestimación",
    "desestimad": "desestimar",
    "desestimada": "desestimar",
    "desestimadas": "desestimar",
    "desestimado": "desestimar",
    "desestimadora": "desestimador",
    "desestimadoras": "desestimador",
    "desestimadores": "desestimador",
    "desestimados": "desestimar",
    "desestimamos": "desestimar",
    "desestiman": "desestimar",
    "desestimando": "desestimar",
    "desestimara": "desestimar",
    "desestimarais": "desestimar",
    "desestimaran": "desestimar",
    "desestimaras": "desestimar",
    "desestimare": "desestimar",
    "desestimareis": "desestimar",
    "desestimaremos": "desestimar",
    "desestimaren": "desestimar",
    "desestimares": "desestimar",
    "desestimaron": "desestimar",
    "desestimará": "desestimar",
    "desestimarán": "desestimar",
    "desestimarás": "desestimar",
    "desestimaré": "desestimar",
    "desestimaréis": "desestimar",
    "desestimaría": "desestimar",
    "desestimaríais": "desestimar",
    "desestimaríamos": "desestimar",
    "desestimarían": "desestimar",
    "desestimarías": "desestimar",
    "desestimas": "desestimar",
    "desestimase": "desestimar",
    "desestimaseis": "desestimar",
    "desestimasen": "desestimar",
    "desestimases": "desestimar",
    "desestimaste": "desestimar",
    "desestimasteis": "desestimar",
    "desestimatoria": "desestimatorio",
    "desestimatorias": "desestimatorio",
    "desestimatorios": "desestimatorio",
    "desestime": "desestimar",
    "desestimemos": "desestimar",
    "desestimen": "desestimar",
    "desestimes": "desestimar",
    "desestimo": "desestimar",
    "desestimábamos": "desestimar",
    "desestimáis": "desestimar",
    "desestimáramos": "desestimar",
    "desestimáremos": "desestimar",
    "desestimásemos": "desestimar",
    "desestimé": "desestimar",
    "desestiméis": "desestimar",
    "desestimó": "desestimar",
    "desestructuraciones": "desestructuración",
    "desestímulos": "desestímulo",
    "desexualizaciones": "desexualización",
    "deseábamos": "desear",
    "deseáis": "desear",
    "deseáramos": "desear",
    "deseáremos": "desear",
    "deseásemos": "desear",
    "deseé": "desear",
    "deseéis": "desear",
    "deseó": "desear",
    "desface": "desfacer",
    "desfaced": "desfacer",
    "desfacemos": "desfacer",
    "desfacen": "desfacer",
    "desfaces": "desfacer",
    "desfachatada": "desfachatado",
    "desfachatadas": "desfachatado",
    "desfachatados": "desfachatado",
    "desfachateces": "desfachatez",
    "desfaciendo": "desfacer",
    "desfacéis": "desfacer",
    "desfacía": "desfacer",
    "desfacíais": "desfacer",
    "desfacíamos": "desfacer",
    "desfacían": "desfacer",
    "desfacías": "desfacer",
    "desfaga": "desfacer",
    "desfagamos": "desfacer",
    "desfagan": "desfacer",
    "desfagas": "desfacer",
    "desfago": "desfacer",
    "desfagáis": "desfacer",
    "desfaja": "desfajar",
    "desfajaba": "desfajar",
    "desfajabais": "desfajar",
    "desfajaban": "desfajar",
    "desfajabas": "desfajar",
    "desfajad": "desfajar",
    "desfajada": "desfajar",
    "desfajadas": "desfajar",
    "desfajado": "desfajar",
    "desfajados": "desfajar",
    "desfajamos": "desfajar",
    "desfajan": "desfajar",
    "desfajando": "desfajar",
    "desfajara": "desfajar",
    "desfajarais": "desfajar",
    "desfajaran": "desfajar",
    "desfajaras": "desfajar",
    "desfajare": "desfajar",
    "desfajareis": "desfajar",
    "desfajaremos": "desfajar",
    "desfajaren": "desfajar",
    "desfajares": "desfajar",
    "desfajaron": "desfajar",
    "desfajará": "desfajar",
    "desfajarán": "desfajar",
    "desfajarás": "desfajar",
    "desfajaré": "desfajar",
    "desfajaréis": "desfajar",
    "desfajaría": "desfajar",
    "desfajaríais": "desfajar",
    "desfajaríamos": "desfajar",
    "desfajarían": "desfajar",
    "desfajarías": "desfajar",
    "desfajas": "desfajar",
    "desfajase": "desfajar",
    "desfajaseis": "desfajar",
    "desfajasen": "desfajar",
    "desfajases": "desfajar",
    "desfajaste": "desfajar",
    "desfajasteis": "desfajar",
    "desfaje": "desfajar",
    "desfajemos": "desfajar",
    "desfajen": "desfajar",
    "desfajes": "desfajar",
    "desfajo": "desfajar",
    "desfajábamos": "desfajar",
    "desfajáis": "desfajar",
    "desfajáramos": "desfajar",
    "desfajáremos": "desfajar",
    "desfajásemos": "desfajar",
    "desfajé": "desfajar",
    "desfajéis": "desfajar",
    "desfajó": "desfajar",
    "desfalca": "desfalcar",
    "desfalcaba": "desfalcar",
    "desfalcabais": "desfalcar",
    "desfalcaban": "desfalcar",
    "desfalcabas": "desfalcar",
    "desfalcad": "desfalcar",
    "desfalcada": "desfalcar",
    "desfalcadas": "desfalcar",
    "desfalcado": "desfalcar",
    "desfalcadora": "desfalcador",
    "desfalcadoras": "desfalcador",
    "desfalcadores": "desfalcador",
    "desfalcados": "desfalcar",
    "desfalcamos": "desfalcar",
    "desfalcan": "desfalcar",
    "desfalcando": "desfalcar",
    "desfalcara": "desfalcar",
    "desfalcarais": "desfalcar",
    "desfalcaran": "desfalcar",
    "desfalcaras": "desfalcar",
    "desfalcare": "desfalcar",
    "desfalcareis": "desfalcar",
    "desfalcaremos": "desfalcar",
    "desfalcaren": "desfalcar",
    "desfalcares": "desfalcar",
    "desfalcaron": "desfalcar",
    "desfalcará": "desfalcar",
    "desfalcarán": "desfalcar",
    "desfalcarás": "desfalcar",
    "desfalcaré": "desfalcar",
    "desfalcaréis": "desfalcar",
    "desfalcaría": "desfalcar",
    "desfalcaríais": "desfalcar",
    "desfalcaríamos": "desfalcar",
    "desfalcarían": "desfalcar",
    "desfalcarías": "desfalcar",
    "desfalcas": "desfalcar",
    "desfalcase": "desfalcar",
    "desfalcaseis": "desfalcar",
    "desfalcasen": "desfalcar",
    "desfalcases": "desfalcar",
    "desfalcaste": "desfalcar",
    "desfalcasteis": "desfalcar",
    "desfalco": "desfalcar",
    "desfalcos": "desfalco",
    "desfalcábamos": "desfalcar",
    "desfalcáis": "desfalcar",
    "desfalcáramos": "desfalcar",
    "desfalcáremos": "desfalcar",
    "desfalcásemos": "desfalcar",
    "desfalcó": "desfalcar",
    "desfallece": "desfallecer",
    "desfalleced": "desfallecer",
    "desfallecemos": "desfallecer",
    "desfallecen": "desfallecer",
    "desfalleceremos": "desfallecer",
    "desfallecerá": "desfallecer",
    "desfallecerán": "desfallecer",
    "desfallecerás": "desfallecer",
    "desfalleceré": "desfallecer",
    "desfalleceréis": "desfallecer",
    "desfallecería": "desfallecer",
    "desfalleceríais": "desfallecer",
    "desfalleceríamos": "desfallecer",
    "desfallecerían": "desfallecer",
    "desfallecerías": "desfallecer",
    "desfalleces": "desfallecer",
    "desfallecida": "desfallecer",
    "desfallecidas": "desfallecer",
    "desfallecido": "desfallecer",
    "desfallecidos": "desfallecer",
    "desfalleciendo": "desfallecer",
    "desfalleciera": "desfallecer",
    "desfallecierais": "desfallecer",
    "desfallecieran": "desfallecer",
    "desfallecieras": "desfallecer",
    "desfalleciere": "desfallecer",
    "desfalleciereis": "desfallecer",
    "desfallecieren": "desfallecer",
    "desfallecieres": "desfallecer",
    "desfallecieron": "desfallecer",
    "desfalleciese": "desfallecer",
    "desfallecieseis": "desfallecer",
    "desfalleciesen": "desfallecer",
    "desfallecieses": "desfallecer",
    "desfallecimientos": "desfallecimiento",
    "desfallecimos": "desfallecer",
    "desfalleciste": "desfallecer",
    "desfallecisteis": "desfallecer",
    "desfalleciéramos": "desfallecer",
    "desfalleciéremos": "desfallecer",
    "desfalleciésemos": "desfallecer",
    "desfalleció": "desfallecer",
    "desfallecéis": "desfallecer",
    "desfallecí": "desfallecer",
    "desfallecía": "desfallecer",
    "desfallecíais": "desfallecer",
    "desfallecíamos": "desfallecer",
    "desfallecían": "desfallecer",
    "desfallecías": "desfallecer",
    "desfallezca": "desfallecer",
    "desfallezcamos": "desfallecer",
    "desfallezcan": "desfallecer",
    "desfallezcas": "desfallecer",
    "desfallezco": "desfallecer",
    "desfallezcáis": "desfallecer",
    "desfalque": "desfalcar",
    "desfalquemos": "desfalcar",
    "desfalquen": "desfalcar",
    "desfalques": "desfalcar",
    "desfalqué": "desfalcar",
    "desfalquéis": "desfalcar",
    "desfaremos": "desfacer",
    "desfará": "desfacer",
    "desfarán": "desfacer",
    "desfarás": "desfacer",
    "desfaré": "desfacer",
    "desfaréis": "desfacer",
    "desfaría": "desfacer",
    "desfaríais": "desfacer",
    "desfaríamos": "desfacer",
    "desfarían": "desfacer",
    "desfarías": "desfacer",
    "desfasa": "desfasar",
    "desfasaba": "desfasar",
    "desfasabais": "desfasar",
    "desfasaban": "desfasar",
    "desfasabas": "desfasar",
    "desfasad": "desfasar",
    "desfasada": "desfasar",
    "desfasadas": "desfasar",
    "desfasado": "desfasar",
    "desfasadora": "desfasador",
    "desfasadoras": "desfasador",
    "desfasadores": "desfasador",
    "desfasados": "desfasar",
    "desfasajes": "desfasaje",
    "desfasamos": "desfasar",
    "desfasan": "desfasar",
    "desfasando": "desfasar",
    "desfasara": "desfasar",
    "desfasarais": "desfasar",
    "desfasaran": "desfasar",
    "desfasaras": "desfasar",
    "desfasare": "desfasar",
    "desfasareis": "desfasar",
    "desfasaremos": "desfasar",
    "desfasaren": "desfasar",
    "desfasares": "desfasar",
    "desfasaron": "desfasar",
    "desfasará": "desfasar",
    "desfasarán": "desfasar",
    "desfasarás": "desfasar",
    "desfasaré": "desfasar",
    "desfasaréis": "desfasar",
    "desfasaría": "desfasar",
    "desfasaríais": "desfasar",
    "desfasaríamos": "desfasar",
    "desfasarían": "desfasar",
    "desfasarías": "desfasar",
    "desfasas": "desfasar",
    "desfasase": "desfasar",
    "desfasaseis": "desfasar",
    "desfasasen": "desfasar",
    "desfasases": "desfasar",
    "desfasaste": "desfasar",
    "desfasasteis": "desfasar",
    "desfase": "desfasar",
    "desfasemos": "desfasar",
    "desfasen": "desfasar",
    "desfases": "desfase",
    "desfaso": "desfasar",
    "desfasábamos": "desfasar",
    "desfasáis": "desfasar",
    "desfasáramos": "desfasar",
    "desfasáremos": "desfasar",
    "desfasásemos": "desfasar",
    "desfasé": "desfasar",
    "desfaséis": "desfasar",
    "desfasó": "desfasar",
    "desfavorables": "desfavorable",
    "desfavorece": "desfavorecer",
    "desfavoreced": "desfavorecer",
    "desfavorecedora": "desfavorecedor",
    "desfavorecedoras": "desfavorecedor",
    "desfavorecedores": "desfavorecedor",
    "desfavorecemos": "desfavorecer",
    "desfavorecen": "desfavorecer",
    "desfavoreceremos": "desfavorecer",
    "desfavorecerá": "desfavorecer",
    "desfavorecerán": "desfavorecer",
    "desfavorecerás": "desfavorecer",
    "desfavoreceré": "desfavorecer",
    "desfavoreceréis": "desfavorecer",
    "desfavorecería": "desfavorecer",
    "desfavoreceríais": "desfavorecer",
    "desfavoreceríamos": "desfavorecer",
    "desfavorecerían": "desfavorecer",
    "desfavorecerías": "desfavorecer",
    "desfavoreces": "desfavorecer",
    "desfavorecida": "desfavorecer",
    "desfavorecidas": "desfavorecer",
    "desfavorecido": "desfavorecer",
    "desfavorecidos": "desfavorecer",
    "desfavoreciendo": "desfavorecer",
    "desfavoreciera": "desfavorecer",
    "desfavorecierais": "desfavorecer",
    "desfavorecieran": "desfavorecer",
    "desfavorecieras": "desfavorecer",
    "desfavoreciere": "desfavorecer",
    "desfavoreciereis": "desfavorecer",
    "desfavorecieren": "desfavorecer",
    "desfavorecieres": "desfavorecer",
    "desfavorecieron": "desfavorecer",
    "desfavoreciese": "desfavorecer",
    "desfavorecieseis": "desfavorecer",
    "desfavoreciesen": "desfavorecer",
    "desfavorecieses": "desfavorecer",
    "desfavorecimientos": "desfavorecimiento",
    "desfavorecimos": "desfavorecer",
    "desfavoreciste": "desfavorecer",
    "desfavorecisteis": "desfavorecer",
    "desfavoreciéramos": "desfavorecer",
    "desfavoreciéremos": "desfavorecer",
    "desfavoreciésemos": "desfavorecer",
    "desfavoreció": "desfavorecer",
    "desfavorecéis": "desfavorecer",
    "desfavorecí": "desfavorecer",
    "desfavorecía": "desfavorecer",
    "desfavorecíais": "desfavorecer",
    "desfavorecíamos": "desfavorecer",
    "desfavorecían": "desfavorecer",
    "desfavorecías": "desfavorecer",
    "desfavores": "desfavor",
    "desfavorezca": "desfavorecer",
    "desfavorezcamos": "desfavorecer",
    "desfavorezcan": "desfavorecer",
    "desfavorezcas": "desfavorecer",
    "desfavorezco": "desfavorecer",
    "desfavorezcáis": "desfavorecer",
    "desfaz": "desfacer",
    "desfecha": "desfacer",
    "desfechas": "desfacer",
    "desfecho": "desfacer",
    "desfechos": "desfacer",
    "desferroxiaminas": "desferroxiamina",
    "desfibra": "desfibrar",
    "desfibraba": "desfibrar",
    "desfibrabais": "desfibrar",
    "desfibraban": "desfibrar",
    "desfibrabas": "desfibrar",
    "desfibrad": "desfibrar",
    "desfibrada": "desfibrar",
    "desfibradas": "desfibrar",
    "desfibrado": "desfibrar",
    "desfibradora": "desfibrador",
    "desfibradoras": "desfibrador",
    "desfibradores": "desfibrador",
    "desfibrados": "desfibrar",
    "desfibramientos": "desfibramiento",
    "desfibramos": "desfibrar",
    "desfibran": "desfibrar",
    "desfibrando": "desfibrar",
    "desfibrara": "desfibrar",
    "desfibrarais": "desfibrar",
    "desfibraran": "desfibrar",
    "desfibraras": "desfibrar",
    "desfibrare": "desfibrar",
    "desfibrareis": "desfibrar",
    "desfibraremos": "desfibrar",
    "desfibraren": "desfibrar",
    "desfibrares": "desfibrar",
    "desfibraron": "desfibrar",
    "desfibrará": "desfibrar",
    "desfibrarán": "desfibrar",
    "desfibrarás": "desfibrar",
    "desfibraré": "desfibrar",
    "desfibraréis": "desfibrar",
    "desfibraría": "desfibrar",
    "desfibraríais": "desfibrar",
    "desfibraríamos": "desfibrar",
    "desfibrarían": "desfibrar",
    "desfibrarías": "desfibrar",
    "desfibras": "desfibrar",
    "desfibrase": "desfibrar",
    "desfibraseis": "desfibrar",
    "desfibrasen": "desfibrar",
    "desfibrases": "desfibrar",
    "desfibraste": "desfibrar",
    "desfibrasteis": "desfibrar",
    "desfibre": "desfibrar",
    "desfibremos": "desfibrar",
    "desfibren": "desfibrar",
    "desfibres": "desfibrar",
    "desfibrinaciones": "desfibrinación",
    "desfibro": "desfibrar",
    "desfibrábamos": "desfibrar",
    "desfibráis": "desfibrar",
    "desfibráramos": "desfibrar",
    "desfibráremos": "desfibrar",
    "desfibrásemos": "desfibrar",
    "desfibré": "desfibrar",
    "desfibréis": "desfibrar",
    "desfibró": "desfibrar",
    "desfice": "desfacer",
    "desficiera": "desfacer",
    "desficierais": "desfacer",
    "desficieran": "desfacer",
    "desficieras": "desfacer",
    "desficiere": "desfacer",
    "desficiereis": "desfacer",
    "desficieren": "desfacer",
    "desficieres": "desfacer",
    "desficieron": "desfacer",
    "desficiese": "desfacer",
    "desficieseis": "desfacer",
    "desficiesen": "desfacer",
    "desficieses": "desfacer",
    "desficimos": "desfacer",
    "desficiste": "desfacer",
    "desficisteis": "desfacer",
    "desficiéramos": "desfacer",
    "desficiéremos": "desfacer",
    "desficiésemos": "desfacer",
    "desfigura": "desfigurar",
    "desfiguraba": "desfigurar",
    "desfigurabais": "desfigurar",
    "desfiguraban": "desfigurar",
    "desfigurabas": "desfigurar",
    "desfiguraciones": "desfiguración",
    "desfigurad": "desfigurar",
    "desfigurada": "desfigurar",
    "desfiguradas": "desfigurar",
    "desfigurado": "desfigurar",
    "desfigurados": "desfigurar",
    "desfiguramientos": "desfiguramiento",
    "desfiguramos": "desfigurar",
    "desfiguran": "desfigurar",
    "desfigurando": "desfigurar",
    "desfigurara": "desfigurar",
    "desfigurarais": "desfigurar",
    "desfiguraran": "desfigurar",
    "desfiguraras": "desfigurar",
    "desfigurare": "desfigurar",
    "desfigurareis": "desfigurar",
    "desfiguraremos": "desfigurar",
    "desfiguraren": "desfigurar",
    "desfigurares": "desfigurar",
    "desfiguraron": "desfigurar",
    "desfigurará": "desfigurar",
    "desfigurarán": "desfigurar",
    "desfigurarás": "desfigurar",
    "desfiguraré": "desfigurar",
    "desfiguraréis": "desfigurar",
    "desfiguraría": "desfigurar",
    "desfiguraríais": "desfigurar",
    "desfiguraríamos": "desfigurar",
    "desfigurarían": "desfigurar",
    "desfigurarías": "desfigurar",
    "desfiguras": "desfigurar",
    "desfigurase": "desfigurar",
    "desfiguraseis": "desfigurar",
    "desfigurasen": "desfigurar",
    "desfigurases": "desfigurar",
    "desfiguraste": "desfigurar",
    "desfigurasteis": "desfigurar",
    "desfigure": "desfigurar",
    "desfiguremos": "desfigurar",
    "desfiguren": "desfigurar",
    "desfigures": "desfigurar",
    "desfiguro": "desfigurar",
    "desfigurábamos": "desfigurar",
    "desfiguráis": "desfigurar",
    "desfiguráramos": "desfigurar",
    "desfiguráremos": "desfigurar",
    "desfigurásemos": "desfigurar",
    "desfiguré": "desfigurar",
    "desfiguréis": "desfigurar",
    "desfiguró": "desfigurar",
    "desfila": "desfilar",
    "desfilaba": "desfilar",
    "desfilabais": "desfilar",
    "desfilaban": "desfilar",
    "desfilabas": "desfilar",
    "desfilacha": "desfilachar",
    "desfilachaba": "desfilachar",
    "desfilachabais": "desfilachar",
    "desfilachaban": "desfilachar",
    "desfilachabas": "desfilachar",
    "desfilachad": "desfilachar",
    "desfilachada": "desfilachar",
    "desfilachadas": "desfilachar",
    "desfilachado": "desfilachar",
    "desfilachados": "desfilachar",
    "desfilachamos": "desfilachar",
    "desfilachan": "desfilachar",
    "desfilachando": "desfilachar",
    "desfilachara": "desfilachar",
    "desfilacharais": "desfilachar",
    "desfilacharan": "desfilachar",
    "desfilacharas": "desfilachar",
    "desfilachare": "desfilachar",
    "desfilachareis": "desfilachar",
    "desfilacharemos": "desfilachar",
    "desfilacharen": "desfilachar",
    "desfilachares": "desfilachar",
    "desfilacharon": "desfilachar",
    "desfilachará": "desfilachar",
    "desfilacharán": "desfilachar",
    "desfilacharás": "desfilachar",
    "desfilacharé": "desfilachar",
    "desfilacharéis": "desfilachar",
    "desfilacharía": "desfilachar",
    "desfilacharíais": "desfilachar",
    "desfilacharíamos": "desfilachar",
    "desfilacharían": "desfilachar",
    "desfilacharías": "desfilachar",
    "desfilachas": "desfilachar",
    "desfilachase": "desfilachar",
    "desfilachaseis": "desfilachar",
    "desfilachasen": "desfilachar",
    "desfilachases": "desfilachar",
    "desfilachaste": "desfilachar",
    "desfilachasteis": "desfilachar",
    "desfilache": "desfilachar",
    "desfilachemos": "desfilachar",
    "desfilachen": "desfilachar",
    "desfilaches": "desfilachar",
    "desfilacho": "desfilachar",
    "desfilachábamos": "desfilachar",
    "desfilacháis": "desfilachar",
    "desfilacháramos": "desfilachar",
    "desfilacháremos": "desfilachar",
    "desfilachásemos": "desfilachar",
    "desfilaché": "desfilachar",
    "desfilachéis": "desfilachar",
    "desfilachó": "desfilachar",
    "desfilad": "desfilar",
    "desfilada": "desfilar",
    "desfiladas": "desfilar",
    "desfiladeros": "desfiladero",
    "desfilado": "desfilar",
    "desfilados": "desfilar",
    "desfilamos": "desfilar",
    "desfilan": "desfilar",
    "desfilando": "desfilar",
    "desfilara": "desfilar",
    "desfilarais": "desfilar",
    "desfilaran": "desfilar",
    "desfilaras": "desfilar",
    "desfilare": "desfilar",
    "desfilareis": "desfilar",
    "desfilaremos": "desfilar",
    "desfilaren": "desfilar",
    "desfilares": "desfilar",
    "desfilaron": "desfilar",
    "desfilará": "desfilar",
    "desfilarán": "desfilar",
    "desfilarás": "desfilar",
    "desfilaré": "desfilar",
    "desfilaréis": "desfilar",
    "desfilaría": "desfilar",
    "desfilaríais": "desfilar",
    "desfilaríamos": "desfilar",
    "desfilarían": "desfilar",
    "desfilarías": "desfilar",
    "desfilas": "desfilar",
    "desfilase": "desfilar",
    "desfilaseis": "desfilar",
    "desfilasen": "desfilar",
    "desfilases": "desfilar",
    "desfilaste": "desfilar",
    "desfilasteis": "desfilar",
    "desfile": "desfilar",
    "desfilemos": "desfilar",
    "desfilen": "desfilar",
    "desfiles": "desfile",
    "desfilo": "desfilar",
    "desfilábamos": "desfilar",
    "desfiláis": "desfilar",
    "desfiláramos": "desfilar",
    "desfiláremos": "desfilar",
    "desfilásemos": "desfilar",
    "desfilé": "desfilar",
    "desfiléis": "desfilar",
    "desfiló": "desfilar",
    "desfizo": "desfacer",
    "desfleca": "desflecar",
    "desflecaba": "desflecar",
    "desflecabais": "desflecar",
    "desflecaban": "desflecar",
    "desflecabas": "desflecar",
    "desflecad": "desflecar",
    "desflecada": "desflecar",
    "desflecadas": "desflecar",
    "desflecado": "desflecar",
    "desflecados": "desflecar",
    "desflecamos": "desflecar",
    "desflecan": "desflecar",
    "desflecando": "desflecar",
    "desflecara": "desflecar",
    "desflecarais": "desflecar",
    "desflecaran": "desflecar",
    "desflecaras": "desflecar",
    "desflecare": "desflecar",
    "desflecareis": "desflecar",
    "desflecaremos": "desflecar",
    "desflecaren": "desflecar",
    "desflecares": "desflecar",
    "desflecaron": "desflecar",
    "desflecará": "desflecar",
    "desflecarán": "desflecar",
    "desflecarás": "desflecar",
    "desflecaré": "desflecar",
    "desflecaréis": "desflecar",
    "desflecaría": "desflecar",
    "desflecaríais": "desflecar",
    "desflecaríamos": "desflecar",
    "desflecarían": "desflecar",
    "desflecarías": "desflecar",
    "desflecas": "desflecar",
    "desflecase": "desflecar",
    "desflecaseis": "desflecar",
    "desflecasen": "desflecar",
    "desflecases": "desflecar",
    "desflecaste": "desflecar",
    "desflecasteis": "desflecar",
    "desfleco": "desflecar",
    "desflecábamos": "desflecar",
    "desflecáis": "desflecar",
    "desflecáramos": "desflecar",
    "desflecáremos": "desflecar",
    "desflecásemos": "desflecar",
    "desflecó": "desflecar",
    "desflema": "desflemar",
    "desflemaba": "desflemar",
    "desflemabais": "desflemar",
    "desflemaban": "desflemar",
    "desflemabas": "desflemar",
    "desflemad": "desflemar",
    "desflemada": "desflemar",
    "desflemadas": "desflemar",
    "desflemado": "desflemar",
    "desflemadores": "desflemador",
    "desflemados": "desflemar",
    "desflemamos": "desflemar",
    "desfleman": "desflemar",
    "desflemando": "desflemar",
    "desflemara": "desflemar",
    "desflemarais": "desflemar",
    "desflemaran": "desflemar",
    "desflemaras": "desflemar",
    "desflemare": "desflemar",
    "desflemareis": "desflemar",
    "desflemaremos": "desflemar",
    "desflemaren": "desflemar",
    "desflemares": "desflemar",
    "desflemaron": "desflemar",
    "desflemará": "desflemar",
    "desflemarán": "desflemar",
    "desflemarás": "desflemar",
    "desflemaré": "desflemar",
    "desflemaréis": "desflemar",
    "desflemaría": "desflemar",
    "desflemaríais": "desflemar",
    "desflemaríamos": "desflemar",
    "desflemarían": "desflemar",
    "desflemarías": "desflemar",
    "desflemas": "desflemar",
    "desflemase": "desflemar",
    "desflemaseis": "desflemar",
    "desflemasen": "desflemar",
    "desflemases": "desflemar",
    "desflemaste": "desflemar",
    "desflemasteis": "desflemar",
    "desfleme": "desflemar",
    "desflememos": "desflemar",
    "desflemen": "desflemar",
    "desflemes": "desflemar",
    "desflemo": "desflemar",
    "desflemábamos": "desflemar",
    "desflemáis": "desflemar",
    "desflemáramos": "desflemar",
    "desflemáremos": "desflemar",
    "desflemásemos": "desflemar",
    "desflemé": "desflemar",
    "desfleméis": "desflemar",
    "desflemó": "desflemar",
    "desfleque": "desflecar",
    "desflequemos": "desflecar",
    "desflequen": "desflecar",
    "desfleques": "desflecar",
    "desflequé": "desflecar",
    "desflequéis": "desflecar",
    "desflora": "desflorar",
    "desfloraba": "desflorar",
    "desflorabais": "desflorar",
    "desfloraban": "desflorar",
    "desflorabas": "desflorar",
    "desfloraciones": "desfloración",
    "desflorad": "desflorar",
    "desflorada": "desflorar",
    "desfloradas": "desflorar",
    "desflorado": "desflorar",
    "desflorados": "desflorar",
    "desfloramientos": "desfloramiento",
    "desfloramos": "desflorar",
    "desfloran": "desflorar",
    "desflorando": "desflorar",
    "desflorara": "desflorar",
    "desflorarais": "desflorar",
    "desfloraran": "desflorar",
    "desfloraras": "desflorar",
    "desflorare": "desflorar",
    "desflorareis": "desflorar",
    "desfloraremos": "desflorar",
    "desfloraren": "desflorar",
    "desflorares": "desflorar",
    "desfloraron": "desflorar",
    "desflorará": "desflorar",
    "desflorarán": "desflorar",
    "desflorarás": "desflorar",
    "desfloraré": "desflorar",
    "desfloraréis": "desflorar",
    "desfloraría": "desflorar",
    "desfloraríais": "desflorar",
    "desfloraríamos": "desflorar",
    "desflorarían": "desflorar",
    "desflorarías": "desflorar",
    "desfloras": "desflorar",
    "desflorase": "desflorar",
    "desfloraseis": "desflorar",
    "desflorasen": "desflorar",
    "desflorases": "desflorar",
    "desfloraste": "desflorar",
    "desflorasteis": "desflorar",
    "desflore": "desflorar",
    "desflorece": "desflorecer",
    "desfloreced": "desflorecer",
    "desflorecemos": "desflorecer",
    "desflorecen": "desflorecer",
    "desfloreceremos": "desflorecer",
    "desflorecerá": "desflorecer",
    "desflorecerán": "desflorecer",
    "desflorecerás": "desflorecer",
    "desfloreceré": "desflorecer",
    "desfloreceréis": "desflorecer",
    "desflorecería": "desflorecer",
    "desfloreceríais": "desflorecer",
    "desfloreceríamos": "desflorecer",
    "desflorecerían": "desflorecer",
    "desflorecerías": "desflorecer",
    "desfloreces": "desflorecer",
    "desflorecida": "desflorecer",
    "desflorecidas": "desflorecer",
    "desflorecido": "desflorecer",
    "desflorecidos": "desflorecer",
    "desfloreciendo": "desflorecer",
    "desfloreciera": "desflorecer",
    "desflorecierais": "desflorecer",
    "desflorecieran": "desflorecer",
    "desflorecieras": "desflorecer",
    "desfloreciere": "desflorecer",
    "desfloreciereis": "desflorecer",
    "desflorecieren": "desflorecer",
    "desflorecieres": "desflorecer",
    "desflorecieron": "desflorecer",
    "desfloreciese": "desflorecer",
    "desflorecieseis": "desflorecer",
    "desfloreciesen": "desflorecer",
    "desflorecieses": "desflorecer",
    "desflorecimientos": "desflorecimiento",
    "desflorecimos": "desflorecer",
    "desfloreciste": "desflorecer",
    "desflorecisteis": "desflorecer",
    "desfloreciéramos": "desflorecer",
    "desfloreciéremos": "desflorecer",
    "desfloreciésemos": "desflorecer",
    "desfloreció": "desflorecer",
    "desflorecéis": "desflorecer",
    "desflorecí": "desflorecer",
    "desflorecía": "desflorecer",
    "desflorecíais": "desflorecer",
    "desflorecíamos": "desflorecer",
    "desflorecían": "desflorecer",
    "desflorecías": "desflorecer",
    "desfloremos": "desflorar",
    "desfloren": "desflorar",
    "desflores": "desflorar",
    "desflorezca": "desflorecer",
    "desflorezcamos": "desflorecer",
    "desflorezcan": "desflorecer",
    "desflorezcas": "desflorecer",
    "desflorezco": "desflorecer",
    "desflorezcáis": "desflorecer",
    "desfloro": "desflorar",
    "desflorábamos": "desflorar",
    "desfloráis": "desflorar",
    "desfloráramos": "desflorar",
    "desfloráremos": "desflorar",
    "desflorásemos": "desflorar",
    "desfloré": "desflorar",
    "desfloréis": "desflorar",
    "desfloró": "desflorar",
    "desfoga": "desfogar",
    "desfogaba": "desfogar",
    "desfogabais": "desfogar",
    "desfogaban": "desfogar",
    "desfogabas": "desfogar",
    "desfogad": "desfogar",
    "desfogada": "desfogar",
    "desfogadas": "desfogar",
    "desfogado": "desfogar",
    "desfogados": "desfogar",
    "desfogamos": "desfogar",
    "desfogan": "desfogar",
    "desfogando": "desfogar",
    "desfogara": "desfogar",
    "desfogarais": "desfogar",
    "desfogaran": "desfogar",
    "desfogaras": "desfogar",
    "desfogare": "desfogar",
    "desfogareis": "desfogar",
    "desfogaremos": "desfogar",
    "desfogaren": "desfogar",
    "desfogares": "desfogar",
    "desfogaron": "desfogar",
    "desfogará": "desfogar",
    "desfogarán": "desfogar",
    "desfogarás": "desfogar",
    "desfogaré": "desfogar",
    "desfogaréis": "desfogar",
    "desfogaría": "desfogar",
    "desfogaríais": "desfogar",
    "desfogaríamos": "desfogar",
    "desfogarían": "desfogar",
    "desfogarías": "desfogar",
    "desfogas": "desfogar",
    "desfogase": "desfogar",
    "desfogaseis": "desfogar",
    "desfogasen": "desfogar",
    "desfogases": "desfogar",
    "desfogaste": "desfogar",
    "desfogasteis": "desfogar",
    "desfogo": "desfogar",
    "desfogue": "desfogar",
    "desfoguemos": "desfogar",
    "desfoguen": "desfogar",
    "desfogues": "desfogue",
    "desfogué": "desfogar",
    "desfoguéis": "desfogar",
    "desfogábamos": "desfogar",
    "desfogáis": "desfogar",
    "desfogáramos": "desfogar",
    "desfogáremos": "desfogar",
    "desfogásemos": "desfogar",
    "desfogó": "desfogar",
    "desfonda": "desfondar",
    "desfondaba": "desfondar",
    "desfondabais": "desfondar",
    "desfondaban": "desfondar",
    "desfondabas": "desfondar",
    "desfondables": "desfondable",
    "desfondad": "desfondar",
    "desfondada": "desfondar",
    "desfondadas": "desfondar",
    "desfondado": "desfondar",
    "desfondados": "desfondar",
    "desfondamientos": "desfondamiento",
    "desfondamos": "desfondar",
    "desfondan": "desfondar",
    "desfondando": "desfondar",
    "desfondara": "desfondar",
    "desfondarais": "desfondar",
    "desfondaran": "desfondar",
    "desfondaras": "desfondar",
    "desfondare": "desfondar",
    "desfondareis": "desfondar",
    "desfondaremos": "desfondar",
    "desfondaren": "desfondar",
    "desfondares": "desfondar",
    "desfondaron": "desfondar",
    "desfondará": "desfondar",
    "desfondarán": "desfondar",
    "desfondarás": "desfondar",
    "desfondaré": "desfondar",
    "desfondaréis": "desfondar",
    "desfondaría": "desfondar",
    "desfondaríais": "desfondar",
    "desfondaríamos": "desfondar",
    "desfondarían": "desfondar",
    "desfondarías": "desfondar",
    "desfondas": "desfondar",
    "desfondase": "desfondar",
    "desfondaseis": "desfondar",
    "desfondasen": "desfondar",
    "desfondases": "desfondar",
    "desfondaste": "desfondar",
    "desfondasteis": "desfondar",
    "desfonde": "desfondar",
    "desfondemos": "desfondar",
    "desfonden": "desfondar",
    "desfondes": "desfondar",
    "desfondo": "desfondar",
    "desfondábamos": "desfondar",
    "desfondáis": "desfondar",
    "desfondáramos": "desfondar",
    "desfondáremos": "desfondar",
    "desfondásemos": "desfondar",
    "desfondé": "desfondar",
    "desfondéis": "desfondar",
    "desfondó": "desfondar",
    "desfonologizaciones": "desfonologización",
    "desforestaciones": "desforestación",
    "desforma": "desformar",
    "desformaba": "desformar",
    "desformabais": "desformar",
    "desformaban": "desformar",
    "desformabas": "desformar",
    "desformad": "desformar",
    "desformada": "desformar",
    "desformadas": "desformar",
    "desformado": "desformar",
    "desformados": "desformar",
    "desformamos": "desformar",
    "desforman": "desformar",
    "desformando": "desformar",
    "desformara": "desformar",
    "desformarais": "desformar",
    "desformaran": "desformar",
    "desformaras": "desformar",
    "desformare": "desformar",
    "desformareis": "desformar",
    "desformaremos": "desformar",
    "desformaren": "desformar",
    "desformares": "desformar",
    "desformaron": "desformar",
    "desformará": "desformar",
    "desformarán": "desformar",
    "desformarás": "desformar",
    "desformaré": "desformar",
    "desformaréis": "desformar",
    "desformaría": "desformar",
    "desformaríais": "desformar",
    "desformaríamos": "desformar",
    "desformarían": "desformar",
    "desformarías": "desformar",
    "desformas": "desformar",
    "desformase": "desformar",
    "desformaseis": "desformar",
    "desformasen": "desformar",
    "desformases": "desformar",
    "desformaste": "desformar",
    "desformasteis": "desformar",
    "desforme": "desformar",
    "desformemos": "desformar",
    "desformen": "desformar",
    "desformes": "desformar",
    "desformo": "desformar",
    "desformábamos": "desformar",
    "desformáis": "desformar",
    "desformáramos": "desformar",
    "desformáremos": "desformar",
    "desformásemos": "desformar",
    "desformé": "desformar",
    "desforméis": "desformar",
    "desformó": "desformar",
    "desfortalecimientos": "desfortalecimiento",
    "desfosforaciones": "desfosforación",
    "desfosforizaciones": "desfosforización",
    "desfragmentaciones": "desfragmentación",
    "desfrena": "desfrenar",
    "desfrenaba": "desfrenar",
    "desfrenabais": "desfrenar",
    "desfrenaban": "desfrenar",
    "desfrenabas": "desfrenar",
    "desfrenad": "desfrenar",
    "desfrenada": "desfrenar",
    "desfrenadas": "desfrenar",
    "desfrenado": "desfrenar",
    "desfrenados": "desfrenar",
    "desfrenamientos": "desfrenamiento",
    "desfrenamos": "desfrenar",
    "desfrenan": "desfrenar",
    "desfrenando": "desfrenar",
    "desfrenara": "desfrenar",
    "desfrenarais": "desfrenar",
    "desfrenaran": "desfrenar",
    "desfrenaras": "desfrenar",
    "desfrenare": "desfrenar",
    "desfrenareis": "desfrenar",
    "desfrenaremos": "desfrenar",
    "desfrenaren": "desfrenar",
    "desfrenares": "desfrenar",
    "desfrenaron": "desfrenar",
    "desfrenará": "desfrenar",
    "desfrenarán": "desfrenar",
    "desfrenarás": "desfrenar",
    "desfrenaré": "desfrenar",
    "desfrenaréis": "desfrenar",
    "desfrenaría": "desfrenar",
    "desfrenaríais": "desfrenar",
    "desfrenaríamos": "desfrenar",
    "desfrenarían": "desfrenar",
    "desfrenarías": "desfrenar",
    "desfrenas": "desfrenar",
    "desfrenase": "desfrenar",
    "desfrenaseis": "desfrenar",
    "desfrenasen": "desfrenar",
    "desfrenases": "desfrenar",
    "desfrenaste": "desfrenar",
    "desfrenasteis": "desfrenar",
    "desfrene": "desfrenar",
    "desfrenemos": "desfrenar",
    "desfrenen": "desfrenar",
    "desfrenes": "desfrenar",
    "desfreno": "desfrenar",
    "desfrenábamos": "desfrenar",
    "desfrenáis": "desfrenar",
    "desfrenáramos": "desfrenar",
    "desfrenáremos": "desfrenar",
    "desfrenásemos": "desfrenar",
    "desfrené": "desfrenar",
    "desfrenéis": "desfrenar",
    "desfrenó": "desfrenar",
    "desfrunce": "desfruncir",
    "desfruncen": "desfruncir",
    "desfrunces": "desfruncir",
    "desfruncid": "desfruncir",
    "desfruncida": "desfruncir",
    "desfruncidas": "desfruncir",
    "desfruncido": "desfruncir",
    "desfruncidos": "desfruncir",
    "desfrunciendo": "desfruncir",
    "desfrunciera": "desfruncir",
    "desfruncierais": "desfruncir",
    "desfruncieran": "desfruncir",
    "desfruncieras": "desfruncir",
    "desfrunciere": "desfruncir",
    "desfrunciereis": "desfruncir",
    "desfruncieren": "desfruncir",
    "desfruncieres": "desfruncir",
    "desfruncieron": "desfruncir",
    "desfrunciese": "desfruncir",
    "desfruncieseis": "desfruncir",
    "desfrunciesen": "desfruncir",
    "desfruncieses": "desfruncir",
    "desfruncimos": "desfruncir",
    "desfrunciremos": "desfruncir",
    "desfruncirá": "desfruncir",
    "desfruncirán": "desfruncir",
    "desfruncirás": "desfruncir",
    "desfrunciré": "desfruncir",
    "desfrunciréis": "desfruncir",
    "desfrunciría": "desfruncir",
    "desfrunciríais": "desfruncir",
    "desfrunciríamos": "desfruncir",
    "desfruncirían": "desfruncir",
    "desfruncirías": "desfruncir",
    "desfrunciste": "desfruncir",
    "desfruncisteis": "desfruncir",
    "desfrunciéramos": "desfruncir",
    "desfrunciéremos": "desfruncir",
    "desfrunciésemos": "desfruncir",
    "desfrunció": "desfruncir",
    "desfruncí": "desfruncir",
    "desfruncía": "desfruncir",
    "desfruncíais": "desfruncir",
    "desfruncíamos": "desfruncir",
    "desfruncían": "desfruncir",
    "desfruncías": "desfruncir",
    "desfruncís": "desfruncir",
    "desfrunza": "desfruncir",
    "desfrunzamos": "desfruncir",
    "desfrunzan": "desfruncir",
    "desfrunzas": "desfruncir",
    "desfrunzo": "desfruncir",
    "desfrunzáis": "desfruncir",
    "desgaires": "desgaire",
    "desgaja": "desgajar",
    "desgajaba": "desgajar",
    "desgajabais": "desgajar",
    "desgajaban": "desgajar",
    "desgajabas": "desgajar",
    "desgajad": "desgajar",
    "desgajada": "desgajar",
    "desgajadas": "desgajar",
    "desgajado": "desgajar",
    "desgajados": "desgajar",
    "desgajaduras": "desgajadura",
    "desgajamientos": "desgajamiento",
    "desgajamos": "desgajar",
    "desgajan": "desgajar",
    "desgajando": "desgajar",
    "desgajara": "desgajar",
    "desgajarais": "desgajar",
    "desgajaran": "desgajar",
    "desgajaras": "desgajar",
    "desgajare": "desgajar",
    "desgajareis": "desgajar",
    "desgajaremos": "desgajar",
    "desgajaren": "desgajar",
    "desgajares": "desgajar",
    "desgajaron": "desgajar",
    "desgajará": "desgajar",
    "desgajarán": "desgajar",
    "desgajarás": "desgajar",
    "desgajaré": "desgajar",
    "desgajaréis": "desgajar",
    "desgajaría": "desgajar",
    "desgajaríais": "desgajar",
    "desgajaríamos": "desgajar",
    "desgajarían": "desgajar",
    "desgajarías": "desgajar",
    "desgajas": "desgajar",
    "desgajase": "desgajar",
    "desgajaseis": "desgajar",
    "desgajasen": "desgajar",
    "desgajases": "desgajar",
    "desgajaste": "desgajar",
    "desgajasteis": "desgajar",
    "desgaje": "desgajar",
    "desgajemos": "desgajar",
    "desgajen": "desgajar",
    "desgajes": "desgaje",
    "desgajo": "desgajar",
    "desgajábamos": "desgajar",
    "desgajáis": "desgajar",
    "desgajáramos": "desgajar",
    "desgajáremos": "desgajar",
    "desgajásemos": "desgajar",
    "desgajé": "desgajar",
    "desgajéis": "desgajar",
    "desgajó": "desgajar",
    "desgalgaderos": "desgalgadero",
    "desgalibada": "desgalibado",
    "desgalibadas": "desgalibado",
    "desgalibados": "desgalibado",
    "desgalichada": "desgalichado",
    "desgalichadas": "desgalichado",
    "desgalichados": "desgalichado",
    "desgalichaduras": "desgalichadura",
    "desgana": "desganar",
    "desganaba": "desganar",
    "desganabais": "desganar",
    "desganaban": "desganar",
    "desganabas": "desganar",
    "desganad": "desganar",
    "desganada": "desganar",
    "desganadas": "desganar",
    "desganado": "desganar",
    "desganados": "desganar",
    "desganamos": "desganar",
    "desganan": "desganar",
    "desganando": "desganar",
    "desganara": "desganar",
    "desganarais": "desganar",
    "desganaran": "desganar",
    "desganaras": "desganar",
    "desganare": "desganar",
    "desganareis": "desganar",
    "desganaremos": "desganar",
    "desganaren": "desganar",
    "desganares": "desganar",
    "desganaron": "desganar",
    "desganará": "desganar",
    "desganarán": "desganar",
    "desganarás": "desganar",
    "desganaré": "desganar",
    "desganaréis": "desganar",
    "desganaría": "desganar",
    "desganaríais": "desganar",
    "desganaríamos": "desganar",
    "desganarían": "desganar",
    "desganarías": "desganar",
    "desganas": "desganar",
    "desganase": "desganar",
    "desganaseis": "desganar",
    "desganasen": "desganar",
    "desganases": "desganar",
    "desganaste": "desganar",
    "desganasteis": "desganar",
    "desgane": "desganar",
    "desganemos": "desganar",
    "desganen": "desganar",
    "desganes": "desganar",
    "desgano": "desganar",
    "desganos": "desgano",
    "desganábamos": "desganar",
    "desganáis": "desganar",
    "desganáramos": "desganar",
    "desganáremos": "desganar",
    "desganásemos": "desganar",
    "desgané": "desganar",
    "desganéis": "desganar",
    "desganó": "desganar",
    "desgarbada": "desgarbado",
    "desgarbadas": "desgarbado",
    "desgarbados": "desgarbado",
    "desgarbos": "desgarbo",
    "desgarganta": "desgargantar",
    "desgargantaba": "desgargantar",
    "desgargantabais": "desgargantar",
    "desgargantaban": "desgargantar",
    "desgargantabas": "desgargantar",
    "desgargantad": "desgargantar",
    "desgargantada": "desgargantar",
    "desgargantadas": "desgargantar",
    "desgargantado": "desgargantar",
    "desgargantados": "desgargantar",
    "desgargantamos": "desgargantar",
    "desgargantan": "desgargantar",
    "desgargantando": "desgargantar",
    "desgargantara": "desgargantar",
    "desgargantarais": "desgargantar",
    "desgargantaran": "desgargantar",
    "desgargantaras": "desgargantar",
    "desgargantare": "desgargantar",
    "desgargantareis": "desgargantar",
    "desgargantaremos": "desgargantar",
    "desgargantaren": "desgargantar",
    "desgargantares": "desgargantar",
    "desgargantaron": "desgargantar",
    "desgargantará": "desgargantar",
    "desgargantarán": "desgargantar",
    "desgargantarás": "desgargantar",
    "desgargantaré": "desgargantar",
    "desgargantaréis": "desgargantar",
    "desgargantaría": "desgargantar",
    "desgargantaríais": "desgargantar",
    "desgargantaríamos": "desgargantar",
    "desgargantarían": "desgargantar",
    "desgargantarías": "desgargantar",
    "desgargantas": "desgargantar",
    "desgargantase": "desgargantar",
    "desgargantaseis": "desgargantar",
    "desgargantasen": "desgargantar",
    "desgargantases": "desgargantar",
    "desgargantaste": "desgargantar",
    "desgargantasteis": "desgargantar",
    "desgargante": "desgargantar",
    "desgargantemos": "desgargantar",
    "desgarganten": "desgargantar",
    "desgargantes": "desgargantar",
    "desgarganto": "desgargantar",
    "desgargantábamos": "desgargantar",
    "desgargantáis": "desgargantar",
    "desgargantáramos": "desgargantar",
    "desgargantáremos": "desgargantar",
    "desgargantásemos": "desgargantar",
    "desgarganté": "desgargantar",
    "desgargantéis": "desgargantar",
    "desgargantó": "desgargantar",
    "desgarita": "desgaritar",
    "desgaritaba": "desgaritar",
    "desgaritabais": "desgaritar",
    "desgaritaban": "desgaritar",
    "desgaritabas": "desgaritar",
    "desgaritad": "desgaritar",
    "desgaritada": "desgaritar",
    "desgaritadas": "desgaritar",
    "desgaritado": "desgaritar",
    "desgaritados": "desgaritar",
    "desgaritamos": "desgaritar",
    "desgaritan": "desgaritar",
    "desgaritando": "desgaritar",
    "desgaritara": "desgaritar",
    "desgaritarais": "desgaritar",
    "desgaritaran": "desgaritar",
    "desgaritaras": "desgaritar",
    "desgaritare": "desgaritar",
    "desgaritareis": "desgaritar",
    "desgaritaremos": "desgaritar",
    "desgaritaren": "desgaritar",
    "desgaritares": "desgaritar",
    "desgaritaron": "desgaritar",
    "desgaritará": "desgaritar",
    "desgaritarán": "desgaritar",
    "desgaritarás": "desgaritar",
    "desgaritaré": "desgaritar",
    "desgaritaréis": "desgaritar",
    "desgaritaría": "desgaritar",
    "desgaritaríais": "desgaritar",
    "desgaritaríamos": "desgaritar",
    "desgaritarían": "desgaritar",
    "desgaritarías": "desgaritar",
    "desgaritas": "desgaritar",
    "desgaritase": "desgaritar",
    "desgaritaseis": "desgaritar",
    "desgaritasen": "desgaritar",
    "desgaritases": "desgaritar",
    "desgaritaste": "desgaritar",
    "desgaritasteis": "desgaritar",
    "desgarite": "desgaritar",
    "desgaritemos": "desgaritar",
    "desgariten": "desgaritar",
    "desgarites": "desgaritar",
    "desgarito": "desgaritar",
    "desgaritábamos": "desgaritar",
    "desgaritáis": "desgaritar",
    "desgaritáramos": "desgaritar",
    "desgaritáremos": "desgaritar",
    "desgaritásemos": "desgaritar",
    "desgarité": "desgaritar",
    "desgaritéis": "desgaritar",
    "desgaritó": "desgaritar",
    "desgarra": "desgarrar",
    "desgarraba": "desgarrar",
    "desgarrabais": "desgarrar",
    "desgarraban": "desgarrar",
    "desgarrabas": "desgarrar",
    "desgarrad": "desgarrar",
    "desgarrada": "desgarrar",
    "desgarradas": "desgarrar",
    "desgarrado": "desgarrar",
    "desgarradora": "desgarrador",
    "desgarradoras": "desgarrador",
    "desgarradores": "desgarrador",
    "desgarrados": "desgarrar",
    "desgarraduras": "desgarradura",
    "desgarramientos": "desgarramiento",
    "desgarramos": "desgarrar",
    "desgarran": "desgarrar",
    "desgarrando": "desgarrar",
    "desgarrara": "desgarrar",
    "desgarrarais": "desgarrar",
    "desgarraran": "desgarrar",
    "desgarraras": "desgarrar",
    "desgarrare": "desgarrar",
    "desgarrareis": "desgarrar",
    "desgarraremos": "desgarrar",
    "desgarraren": "desgarrar",
    "desgarrares": "desgarrar",
    "desgarraron": "desgarrar",
    "desgarrará": "desgarrar",
    "desgarrarán": "desgarrar",
    "desgarrarás": "desgarrar",
    "desgarraré": "desgarrar",
    "desgarraréis": "desgarrar",
    "desgarraría": "desgarrar",
    "desgarraríais": "desgarrar",
    "desgarraríamos": "desgarrar",
    "desgarrarían": "desgarrar",
    "desgarrarías": "desgarrar",
    "desgarras": "desgarrar",
    "desgarrase": "desgarrar",
    "desgarraseis": "desgarrar",
    "desgarrasen": "desgarrar",
    "desgarrases": "desgarrar",
    "desgarraste": "desgarrar",
    "desgarrasteis": "desgarrar",
    "desgarre": "desgarrar",
    "desgarremos": "desgarrar",
    "desgarren": "desgarrar",
    "desgarres": "desgarrar",
    "desgarro": "desgarrar",
    "desgarrones": "desgarrón",
    "desgarros": "desgarro",
    "desgarrábamos": "desgarrar",
    "desgarráis": "desgarrar",
    "desgarráramos": "desgarrar",
    "desgarráremos": "desgarrar",
    "desgarrásemos": "desgarrar",
    "desgarré": "desgarrar",
    "desgarréis": "desgarrar",
    "desgarró": "desgarrar",
    "desgasada": "desgasado",
    "desgasadas": "desgasado",
    "desgasados": "desgasado",
    "desgasifica": "desgasificar",
    "desgasificaba": "desgasificar",
    "desgasificabais": "desgasificar",
    "desgasificaban": "desgasificar",
    "desgasificabas": "desgasificar",
    "desgasificaciones": "desgasificación",
    "desgasificad": "desgasificar",
    "desgasificada": "desgasificar",
    "desgasificadas": "desgasificar",
    "desgasificado": "desgasificar",
    "desgasificadores": "desgasificador",
    "desgasificados": "desgasificar",
    "desgasificamos": "desgasificar",
    "desgasifican": "desgasificar",
    "desgasificando": "desgasificar",
    "desgasificara": "desgasificar",
    "desgasificarais": "desgasificar",
    "desgasificaran": "desgasificar",
    "desgasificaras": "desgasificar",
    "desgasificare": "desgasificar",
    "desgasificareis": "desgasificar",
    "desgasificaremos": "desgasificar",
    "desgasificaren": "desgasificar",
    "desgasificares": "desgasificar",
    "desgasificaron": "desgasificar",
    "desgasificará": "desgasificar",
    "desgasificarán": "desgasificar",
    "desgasificarás": "desgasificar",
    "desgasificaré": "desgasificar",
    "desgasificaréis": "desgasificar",
    "desgasificaría": "desgasificar",
    "desgasificaríais": "desgasificar",
    "desgasificaríamos": "desgasificar",
    "desgasificarían": "desgasificar",
    "desgasificarías": "desgasificar",
    "desgasificas": "desgasificar",
    "desgasificase": "desgasificar",
    "desgasificaseis": "desgasificar",
    "desgasificasen": "desgasificar",
    "desgasificases": "desgasificar",
    "desgasificaste": "desgasificar",
    "desgasificasteis": "desgasificar",
    "desgasifico": "desgasificar",
    "desgasificábamos": "desgasificar",
    "desgasificáis": "desgasificar",
    "desgasificáramos": "desgasificar",
    "desgasificáremos": "desgasificar",
    "desgasificásemos": "desgasificar",
    "desgasificó": "desgasificar",
    "desgasifique": "desgasificar",
    "desgasifiquemos": "desgasificar",
    "desgasifiquen": "desgasificar",
    "desgasifiques": "desgasificar",
    "desgasifiqué": "desgasificar",
    "desgasifiquéis": "desgasificar",
    "desgasta": "desgastar",
    "desgastaba": "desgastar",
    "desgastabais": "desgastar",
    "desgastaban": "desgastar",
    "desgastabas": "desgastar",
    "desgastad": "desgastar",
    "desgastada": "desgastar",
    "desgastadas": "desgastar",
    "desgastado": "desgastar",
    "desgastadora": "desgastador",
    "desgastadoras": "desgastador",
    "desgastadores": "desgastador",
    "desgastados": "desgastar",
    "desgastamos": "desgastar",
    "desgastan": "desgastar",
    "desgastando": "desgastar",
    "desgastara": "desgastar",
    "desgastarais": "desgastar",
    "desgastaran": "desgastar",
    "desgastaras": "desgastar",
    "desgastare": "desgastar",
    "desgastareis": "desgastar",
    "desgastaremos": "desgastar",
    "desgastaren": "desgastar",
    "desgastares": "desgastar",
    "desgastaron": "desgastar",
    "desgastará": "desgastar",
    "desgastarán": "desgastar",
    "desgastarás": "desgastar",
    "desgastaré": "desgastar",
    "desgastaréis": "desgastar",
    "desgastaría": "desgastar",
    "desgastaríais": "desgastar",
    "desgastaríamos": "desgastar",
    "desgastarían": "desgastar",
    "desgastarías": "desgastar",
    "desgastas": "desgastar",
    "desgastase": "desgastar",
    "desgastaseis": "desgastar",
    "desgastasen": "desgastar",
    "desgastases": "desgastar",
    "desgastaste": "desgastar",
    "desgastasteis": "desgastar",
    "desgaste": "desgastar",
    "desgastemos": "desgastar",
    "desgasten": "desgastar",
    "desgastes": "desgaste",
    "desgasto": "desgastar",
    "desgastábamos": "desgastar",
    "desgastáis": "desgastar",
    "desgastáramos": "desgastar",
    "desgastáremos": "desgastar",
    "desgastásemos": "desgastar",
    "desgasté": "desgastar",
    "desgastéis": "desgastar",
    "desgastó": "desgastar",
    "desgaznata": "desgaznatar",
    "desgaznataba": "desgaznatar",
    "desgaznatabais": "desgaznatar",
    "desgaznataban": "desgaznatar",
    "desgaznatabas": "desgaznatar",
    "desgaznatad": "desgaznatar",
    "desgaznatada": "desgaznatar",
    "desgaznatadas": "desgaznatar",
    "desgaznatado": "desgaznatar",
    "desgaznatados": "desgaznatar",
    "desgaznatamos": "desgaznatar",
    "desgaznatan": "desgaznatar",
    "desgaznatando": "desgaznatar",
    "desgaznatara": "desgaznatar",
    "desgaznatarais": "desgaznatar",
    "desgaznataran": "desgaznatar",
    "desgaznataras": "desgaznatar",
    "desgaznatare": "desgaznatar",
    "desgaznatareis": "desgaznatar",
    "desgaznataremos": "desgaznatar",
    "desgaznataren": "desgaznatar",
    "desgaznatares": "desgaznatar",
    "desgaznataron": "desgaznatar",
    "desgaznatará": "desgaznatar",
    "desgaznatarán": "desgaznatar",
    "desgaznatarás": "desgaznatar",
    "desgaznataré": "desgaznatar",
    "desgaznataréis": "desgaznatar",
    "desgaznataría": "desgaznatar",
    "desgaznataríais": "desgaznatar",
    "desgaznataríamos": "desgaznatar",
    "desgaznatarían": "desgaznatar",
    "desgaznatarías": "desgaznatar",
    "desgaznatas": "desgaznatar",
    "desgaznatase": "desgaznatar",
    "desgaznataseis": "desgaznatar",
    "desgaznatasen": "desgaznatar",
    "desgaznatases": "desgaznatar",
    "desgaznataste": "desgaznatar",
    "desgaznatasteis": "desgaznatar",
    "desgaznate": "desgaznatar",
    "desgaznatemos": "desgaznatar",
    "desgaznaten": "desgaznatar",
    "desgaznates": "desgaznatar",
    "desgaznato": "desgaznatar",
    "desgaznatábamos": "desgaznatar",
    "desgaznatáis": "desgaznatar",
    "desgaznatáramos": "desgaznatar",
    "desgaznatáremos": "desgaznatar",
    "desgaznatásemos": "desgaznatar",
    "desgaznaté": "desgaznatar",
    "desgaznatéis": "desgaznatar",
    "desgaznató": "desgaznatar",
    "desgañita": "desgañitar",
    "desgañitaba": "desgañitar",
    "desgañitabais": "desgañitar",
    "desgañitaban": "desgañitar",
    "desgañitabas": "desgañitar",
    "desgañitad": "desgañitar",
    "desgañitada": "desgañitar",
    "desgañitadas": "desgañitar",
    "desgañitado": "desgañitar",
    "desgañitados": "desgañitar",
    "desgañitamos": "desgañitar",
    "desgañitan": "desgañitar",
    "desgañitando": "desgañitar",
    "desgañitara": "desgañitar",
    "desgañitarais": "desgañitar",
    "desgañitaran": "desgañitar",
    "desgañitaras": "desgañitar",
    "desgañitare": "desgañitar",
    "desgañitareis": "desgañitar",
    "desgañitaremos": "desgañitar",
    "desgañitaren": "desgañitar",
    "desgañitares": "desgañitar",
    "desgañitaron": "desgañitar",
    "desgañitará": "desgañitar",
    "desgañitarán": "desgañitar",
    "desgañitarás": "desgañitar",
    "desgañitaré": "desgañitar",
    "desgañitaréis": "desgañitar",
    "desgañitaría": "desgañitar",
    "desgañitaríais": "desgañitar",
    "desgañitaríamos": "desgañitar",
    "desgañitarían": "desgañitar",
    "desgañitarías": "desgañitar",
    "desgañitas": "desgañitar",
    "desgañitase": "desgañitar",
    "desgañitaseis": "desgañitar",
    "desgañitasen": "desgañitar",
    "desgañitases": "desgañitar",
    "desgañitaste": "desgañitar",
    "desgañitasteis": "desgañitar",
    "desgañite": "desgañitar",
    "desgañitemos": "desgañitar",
    "desgañiten": "desgañitar",
    "desgañites": "desgañitar",
    "desgañito": "desgañitar",
    "desgañitábamos": "desgañitar",
    "desgañitáis": "desgañitar",
    "desgañitáramos": "desgañitar",
    "desgañitáremos": "desgañitar",
    "desgañitásemos": "desgañitar",
    "desgañité": "desgañitar",
    "desgañitéis": "desgañitar",
    "desgañitó": "desgañitar",
    "desglosa": "desglosar",
    "desglosaba": "desglosar",
    "desglosabais": "desglosar",
    "desglosaban": "desglosar",
    "desglosabas": "desglosar",
    "desglosad": "desglosar",
    "desglosada": "desglosar",
    "desglosadas": "desglosar",
    "desglosado": "desglosar",
    "desglosados": "desglosar",
    "desglosamos": "desglosar",
    "desglosan": "desglosar",
    "desglosando": "desglosar",
    "desglosara": "desglosar",
    "desglosarais": "desglosar",
    "desglosaran": "desglosar",
    "desglosaras": "desglosar",
    "desglosare": "desglosar",
    "desglosareis": "desglosar",
    "desglosaremos": "desglosar",
    "desglosaren": "desglosar",
    "desglosares": "desglosar",
    "desglosaron": "desglosar",
    "desglosará": "desglosar",
    "desglosarán": "desglosar",
    "desglosarás": "desglosar",
    "desglosaré": "desglosar",
    "desglosaréis": "desglosar",
    "desglosaría": "desglosar",
    "desglosaríais": "desglosar",
    "desglosaríamos": "desglosar",
    "desglosarían": "desglosar",
    "desglosarías": "desglosar",
    "desglosas": "desglosar",
    "desglosase": "desglosar",
    "desglosaseis": "desglosar",
    "desglosasen": "desglosar",
    "desglosases": "desglosar",
    "desglosaste": "desglosar",
    "desglosasteis": "desglosar",
    "desglose": "desglosar",
    "desglosemos": "desglosar",
    "desglosen": "desglosar",
    "desgloses": "desglose",
    "desgloso": "desglosar",
    "desglosábamos": "desglosar",
    "desglosáis": "desglosar",
    "desglosáramos": "desglosar",
    "desglosáremos": "desglosar",
    "desglosásemos": "desglosar",
    "desglosé": "desglosar",
    "desgloséis": "desglosar",
    "desglosó": "desglosar",
    "desgobernaba": "desgobernar",
    "desgobernabais": "desgobernar",
    "desgobernaban": "desgobernar",
    "desgobernabas": "desgobernar",
    "desgobernad": "desgobernar",
    "desgobernada": "desgobernar",
    "desgobernadas": "desgobernar",
    "desgobernado": "desgobernar",
    "desgobernados": "desgobernar",
    "desgobernamos": "desgobernar",
    "desgobernando": "desgobernar",
    "desgobernara": "desgobernar",
    "desgobernarais": "desgobernar",
    "desgobernaran": "desgobernar",
    "desgobernaras": "desgobernar",
    "desgobernare": "desgobernar",
    "desgobernareis": "desgobernar",
    "desgobernaremos": "desgobernar",
    "desgobernaren": "desgobernar",
    "desgobernares": "desgobernar",
    "desgobernaron": "desgobernar",
    "desgobernará": "desgobernar",
    "desgobernarán": "desgobernar",
    "desgobernarás": "desgobernar",
    "desgobernaré": "desgobernar",
    "desgobernaréis": "desgobernar",
    "desgobernaría": "desgobernar",
    "desgobernaríais": "desgobernar",
    "desgobernaríamos": "desgobernar",
    "desgobernarían": "desgobernar",
    "desgobernarías": "desgobernar",
    "desgobernase": "desgobernar",
    "desgobernaseis": "desgobernar",
    "desgobernasen": "desgobernar",
    "desgobernases": "desgobernar",
    "desgobernaste": "desgobernar",
    "desgobernasteis": "desgobernar",
    "desgobernemos": "desgobernar",
    "desgobernábamos": "desgobernar",
    "desgobernáis": "desgobernar",
    "desgobernáramos": "desgobernar",
    "desgobernáremos": "desgobernar",
    "desgobernásemos": "desgobernar",
    "desgoberné": "desgobernar",
    "desgobernéis": "desgobernar",
    "desgobernó": "desgobernar",
    "desgobierna": "desgobernar",
    "desgobiernan": "desgobernar",
    "desgobiernas": "desgobernar",
    "desgobierne": "desgobernar",
    "desgobiernen": "desgobernar",
    "desgobiernes": "desgobernar",
    "desgobierno": "desgobernar",
    "desgobiernos": "desgobierno",
    "desgolleta": "desgolletar",
    "desgolletaba": "desgolletar",
    "desgolletabais": "desgolletar",
    "desgolletaban": "desgolletar",
    "desgolletabas": "desgolletar",
    "desgolletad": "desgolletar",
    "desgolletada": "desgolletar",
    "desgolletadas": "desgolletar",
    "desgolletado": "desgolletar",
    "desgolletados": "desgolletar",
    "desgolletamos": "desgolletar",
    "desgolletan": "desgolletar",
    "desgolletando": "desgolletar",
    "desgolletara": "desgolletar",
    "desgolletarais": "desgolletar",
    "desgolletaran": "desgolletar",
    "desgolletaras": "desgolletar",
    "desgolletare": "desgolletar",
    "desgolletareis": "desgolletar",
    "desgolletaremos": "desgolletar",
    "desgolletaren": "desgolletar",
    "desgolletares": "desgolletar",
    "desgolletaron": "desgolletar",
    "desgolletará": "desgolletar",
    "desgolletarán": "desgolletar",
    "desgolletarás": "desgolletar",
    "desgolletaré": "desgolletar",
    "desgolletaréis": "desgolletar",
    "desgolletaría": "desgolletar",
    "desgolletaríais": "desgolletar",
    "desgolletaríamos": "desgolletar",
    "desgolletarían": "desgolletar",
    "desgolletarías": "desgolletar",
    "desgolletas": "desgolletar",
    "desgolletase": "desgolletar",
    "desgolletaseis": "desgolletar",
    "desgolletasen": "desgolletar",
    "desgolletases": "desgolletar",
    "desgolletaste": "desgolletar",
    "desgolletasteis": "desgolletar",
    "desgollete": "desgolletar",
    "desgolletemos": "desgolletar",
    "desgolleten": "desgolletar",
    "desgolletes": "desgolletar",
    "desgolleto": "desgolletar",
    "desgolletábamos": "desgolletar",
    "desgolletáis": "desgolletar",
    "desgolletáramos": "desgolletar",
    "desgolletáremos": "desgolletar",
    "desgolletásemos": "desgolletar",
    "desgolleté": "desgolletar",
    "desgolletéis": "desgolletar",
    "desgolletó": "desgolletar",
    "desgoma": "desgomar",
    "desgomaba": "desgomar",
    "desgomabais": "desgomar",
    "desgomaban": "desgomar",
    "desgomabas": "desgomar",
    "desgomad": "desgomar",
    "desgomada": "desgomar",
    "desgomadas": "desgomar",
    "desgomado": "desgomar",
    "desgomados": "desgomar",
    "desgomamos": "desgomar",
    "desgoman": "desgomar",
    "desgomando": "desgomar",
    "desgomara": "desgomar",
    "desgomarais": "desgomar",
    "desgomaran": "desgomar",
    "desgomaras": "desgomar",
    "desgomare": "desgomar",
    "desgomareis": "desgomar",
    "desgomaremos": "desgomar",
    "desgomaren": "desgomar",
    "desgomares": "desgomar",
    "desgomaron": "desgomar",
    "desgomará": "desgomar",
    "desgomarán": "desgomar",
    "desgomarás": "desgomar",
    "desgomaré": "desgomar",
    "desgomaréis": "desgomar",
    "desgomaría": "desgomar",
    "desgomaríais": "desgomar",
    "desgomaríamos": "desgomar",
    "desgomarían": "desgomar",
    "desgomarías": "desgomar",
    "desgomas": "desgomar",
    "desgomase": "desgomar",
    "desgomaseis": "desgomar",
    "desgomasen": "desgomar",
    "desgomases": "desgomar",
    "desgomaste": "desgomar",
    "desgomasteis": "desgomar",
    "desgome": "desgomar",
    "desgomemos": "desgomar",
    "desgomen": "desgomar",
    "desgomes": "desgomar",
    "desgomo": "desgomar",
    "desgomábamos": "desgomar",
    "desgomáis": "desgomar",
    "desgomáramos": "desgomar",
    "desgomáremos": "desgomar",
    "desgomásemos": "desgomar",
    "desgomé": "desgomar",
    "desgoméis": "desgomar",
    "desgomó": "desgomar",
    "desgonce": "desgonzar",
    "desgoncemos": "desgonzar",
    "desgoncen": "desgonzar",
    "desgonces": "desgonzar",
    "desgoncé": "desgonzar",
    "desgoncéis": "desgonzar",
    "desgonza": "desgonzar",
    "desgonzaba": "desgonzar",
    "desgonzabais": "desgonzar",
    "desgonzaban": "desgonzar",
    "desgonzabas": "desgonzar",
    "desgonzad": "desgonzar",
    "desgonzada": "desgonzar",
    "desgonzadas": "desgonzar",
    "desgonzado": "desgonzar",
    "desgonzados": "desgonzar",
    "desgonzamos": "desgonzar",
    "desgonzan": "desgonzar",
    "desgonzando": "desgonzar",
    "desgonzara": "desgonzar",
    "desgonzarais": "desgonzar",
    "desgonzaran": "desgonzar",
    "desgonzaras": "desgonzar",
    "desgonzare": "desgonzar",
    "desgonzareis": "desgonzar",
    "desgonzaremos": "desgonzar",
    "desgonzaren": "desgonzar",
    "desgonzares": "desgonzar",
    "desgonzaron": "desgonzar",
    "desgonzará": "desgonzar",
    "desgonzarán": "desgonzar",
    "desgonzarás": "desgonzar",
    "desgonzaré": "desgonzar",
    "desgonzaréis": "desgonzar",
    "desgonzaría": "desgonzar",
    "desgonzaríais": "desgonzar",
    "desgonzaríamos": "desgonzar",
    "desgonzarían": "desgonzar",
    "desgonzarías": "desgonzar",
    "desgonzas": "desgonzar",
    "desgonzase": "desgonzar",
    "desgonzaseis": "desgonzar",
    "desgonzasen": "desgonzar",
    "desgonzases": "desgonzar",
    "desgonzaste": "desgonzar",
    "desgonzasteis": "desgonzar",
    "desgonzo": "desgonzar",
    "desgonzábamos": "desgonzar",
    "desgonzáis": "desgonzar",
    "desgonzáramos": "desgonzar",
    "desgonzáremos": "desgonzar",
    "desgonzásemos": "desgonzar",
    "desgonzó": "desgonzar",
    "desgozna": "desgoznar",
    "desgoznaba": "desgoznar",
    "desgoznabais": "desgoznar",
    "desgoznaban": "desgoznar",
    "desgoznabas": "desgoznar",
    "desgoznad": "desgoznar",
    "desgoznada": "desgoznar",
    "desgoznadas": "desgoznar",
    "desgoznado": "desgoznar",
    "desgoznados": "desgoznar",
    "desgoznamos": "desgoznar",
    "desgoznan": "desgoznar",
    "desgoznando": "desgoznar",
    "desgoznara": "desgoznar",
    "desgoznarais": "desgoznar",
    "desgoznaran": "desgoznar",
    "desgoznaras": "desgoznar",
    "desgoznare": "desgoznar",
    "desgoznareis": "desgoznar",
    "desgoznaremos": "desgoznar",
    "desgoznaren": "desgoznar",
    "desgoznares": "desgoznar",
    "desgoznaron": "desgoznar",
    "desgoznará": "desgoznar",
    "desgoznarán": "desgoznar",
    "desgoznarás": "desgoznar",
    "desgoznaré": "desgoznar",
    "desgoznaréis": "desgoznar",
    "desgoznaría": "desgoznar",
    "desgoznaríais": "desgoznar",
    "desgoznaríamos": "desgoznar",
    "desgoznarían": "desgoznar",
    "desgoznarías": "desgoznar",
    "desgoznas": "desgoznar",
    "desgoznase": "desgoznar",
    "desgoznaseis": "desgoznar",
    "desgoznasen": "desgoznar",
    "desgoznases": "desgoznar",
    "desgoznaste": "desgoznar",
    "desgoznasteis": "desgoznar",
    "desgozne": "desgoznar",
    "desgoznemos": "desgoznar",
    "desgoznen": "desgoznar",
    "desgoznes": "desgoznar",
    "desgozno": "desgoznar",
    "desgoznábamos": "desgoznar",
    "desgoznáis": "desgoznar",
    "desgoznáramos": "desgoznar",
    "desgoznáremos": "desgoznar",
    "desgoznásemos": "desgoznar",
    "desgozné": "desgoznar",
    "desgoznéis": "desgoznar",
    "desgoznó": "desgoznar",
    "desgracia": "desgraciar",
    "desgraciaba": "desgraciar",
    "desgraciabais": "desgraciar",
    "desgraciaban": "desgraciar",
    "desgraciabas": "desgraciar",
    "desgraciad": "desgraciar",
    "desgraciada": "desgraciar",
    "desgraciadas": "desgraciar",
    "desgraciado": "desgraciar",
    "desgraciados": "desgraciar",
    "desgraciamos": "desgraciar",
    "desgracian": "desgraciar",
    "desgraciando": "desgraciar",
    "desgraciara": "desgraciar",
    "desgraciarais": "desgraciar",
    "desgraciaran": "desgraciar",
    "desgraciaras": "desgraciar",
    "desgraciare": "desgraciar",
    "desgraciareis": "desgraciar",
    "desgraciaremos": "desgraciar",
    "desgraciaren": "desgraciar",
    "desgraciares": "desgraciar",
    "desgraciaron": "desgraciar",
    "desgraciará": "desgraciar",
    "desgraciarán": "desgraciar",
    "desgraciarás": "desgraciar",
    "desgraciaré": "desgraciar",
    "desgraciaréis": "desgraciar",
    "desgraciaría": "desgraciar",
    "desgraciaríais": "desgraciar",
    "desgraciaríamos": "desgraciar",
    "desgraciarían": "desgraciar",
    "desgraciarías": "desgraciar",
    "desgracias": "desgraciar",
    "desgraciase": "desgraciar",
    "desgraciaseis": "desgraciar",
    "desgraciasen": "desgraciar",
    "desgraciases": "desgraciar",
    "desgraciaste": "desgraciar",
    "desgraciasteis": "desgraciar",
    "desgracie": "desgraciar",
    "desgraciemos": "desgraciar",
    "desgracien": "desgraciar",
    "desgracies": "desgraciar",
    "desgracio": "desgraciar",
    "desgraciábamos": "desgraciar",
    "desgraciáis": "desgraciar",
    "desgraciáramos": "desgraciar",
    "desgraciáremos": "desgraciar",
    "desgraciásemos": "desgraciar",
    "desgracié": "desgraciar",
    "desgraciéis": "desgraciar",
    "desgració": "desgraciar",
    "desgrama": "desgramar",
    "desgramaba": "desgramar",
    "desgramabais": "desgramar",
    "desgramaban": "desgramar",
    "desgramabas": "desgramar",
    "desgramad": "desgramar",
    "desgramada": "desgramar",
    "desgramadas": "desgramar",
    "desgramado": "desgramar",
    "desgramados": "desgramar",
    "desgramamos": "desgramar",
    "desgraman": "desgramar",
    "desgramando": "desgramar",
    "desgramara": "desgramar",
    "desgramarais": "desgramar",
    "desgramaran": "desgramar",
    "desgramaras": "desgramar",
    "desgramare": "desgramar",
    "desgramareis": "desgramar",
    "desgramaremos": "desgramar",
    "desgramaren": "desgramar",
    "desgramares": "desgramar",
    "desgramaron": "desgramar",
    "desgramará": "desgramar",
    "desgramarán": "desgramar",
    "desgramarás": "desgramar",
    "desgramaré": "desgramar",
    "desgramaréis": "desgramar",
    "desgramaría": "desgramar",
    "desgramaríais": "desgramar",
    "desgramaríamos": "desgramar",
    "desgramarían": "desgramar",
    "desgramarías": "desgramar",
    "desgramas": "desgramar",
    "desgramase": "desgramar",
    "desgramaseis": "desgramar",
    "desgramasen": "desgramar",
    "desgramases": "desgramar",
    "desgramaste": "desgramar",
    "desgramasteis": "desgramar",
    "desgrame": "desgramar",
    "desgramemos": "desgramar",
    "desgramen": "desgramar",
    "desgrames": "desgramar",
    "desgramo": "desgramar",
    "desgramábamos": "desgramar",
    "desgramáis": "desgramar",
    "desgramáramos": "desgramar",
    "desgramáremos": "desgramar",
    "desgramásemos": "desgramar",
    "desgramé": "desgramar",
    "desgraméis": "desgramar",
    "desgramó": "desgramar",
    "desgrana": "desgranar",
    "desgranaba": "desgranar",
    "desgranabais": "desgranar",
    "desgranaban": "desgranar",
    "desgranabas": "desgranar",
    "desgranad": "desgranar",
    "desgranada": "desgranar",
    "desgranadas": "desgranar",
    "desgranado": "desgranar",
    "desgranadora": "desgranador",
    "desgranadoras": "desgranadora",
    "desgranadores": "desgranador",
    "desgranados": "desgranar",
    "desgranamientos": "desgranamiento",
    "desgranamos": "desgranar",
    "desgranan": "desgranar",
    "desgranando": "desgranar",
    "desgranara": "desgranar",
    "desgranarais": "desgranar",
    "desgranaran": "desgranar",
    "desgranaras": "desgranar",
    "desgranare": "desgranar",
    "desgranareis": "desgranar",
    "desgranaremos": "desgranar",
    "desgranaren": "desgranar",
    "desgranares": "desgranar",
    "desgranaron": "desgranar",
    "desgranará": "desgranar",
    "desgranarán": "desgranar",
    "desgranarás": "desgranar",
    "desgranaré": "desgranar",
    "desgranaréis": "desgranar",
    "desgranaría": "desgranar",
    "desgranaríais": "desgranar",
    "desgranaríamos": "desgranar",
    "desgranarían": "desgranar",
    "desgranarías": "desgranar",
    "desgranas": "desgranar",
    "desgranase": "desgranar",
    "desgranaseis": "desgranar",
    "desgranasen": "desgranar",
    "desgranases": "desgranar",
    "desgranaste": "desgranar",
    "desgranasteis": "desgranar",
    "desgrance": "desgranzar",
    "desgrancemos": "desgranzar",
    "desgrancen": "desgranzar",
    "desgrances": "desgranzar",
    "desgrancé": "desgranzar",
    "desgrancéis": "desgranzar",
    "desgrane": "desgranar",
    "desgranemos": "desgranar",
    "desgranen": "desgranar",
    "desgranes": "desgrane",
    "desgrano": "desgranar",
    "desgranulaciones": "desgranulación",
    "desgranza": "desgranzar",
    "desgranzaba": "desgranzar",
    "desgranzabais": "desgranzar",
    "desgranzaban": "desgranzar",
    "desgranzabas": "desgranzar",
    "desgranzad": "desgranzar",
    "desgranzada": "desgranzar",
    "desgranzadas": "desgranzar",
    "desgranzado": "desgranzar",
    "desgranzados": "desgranzar",
    "desgranzamos": "desgranzar",
    "desgranzan": "desgranzar",
    "desgranzando": "desgranzar",
    "desgranzara": "desgranzar",
    "desgranzarais": "desgranzar",
    "desgranzaran": "desgranzar",
    "desgranzaras": "desgranzar",
    "desgranzare": "desgranzar",
    "desgranzareis": "desgranzar",
    "desgranzaremos": "desgranzar",
    "desgranzaren": "desgranzar",
    "desgranzares": "desgranzar",
    "desgranzaron": "desgranzar",
    "desgranzará": "desgranzar",
    "desgranzarán": "desgranzar",
    "desgranzarás": "desgranzar",
    "desgranzaré": "desgranzar",
    "desgranzaréis": "desgranzar",
    "desgranzaría": "desgranzar",
    "desgranzaríais": "desgranzar",
    "desgranzaríamos": "desgranzar",
    "desgranzarían": "desgranzar",
    "desgranzarías": "desgranzar",
    "desgranzas": "desgranzar",
    "desgranzase": "desgranzar",
    "desgranzaseis": "desgranzar",
    "desgranzasen": "desgranzar",
    "desgranzases": "desgranzar",
    "desgranzaste": "desgranzar",
    "desgranzasteis": "desgranzar",
    "desgranzo": "desgranzar",
    "desgranzábamos": "desgranzar",
    "desgranzáis": "desgranzar",
    "desgranzáramos": "desgranzar",
    "desgranzáremos": "desgranzar",
    "desgranzásemos": "desgranzar",
    "desgranzó": "desgranzar",
    "desgranábamos": "desgranar",
    "desgranáis": "desgranar",
    "desgranáramos": "desgranar",
    "desgranáremos": "desgranar",
    "desgranásemos": "desgranar",
    "desgrané": "desgranar",
    "desgranéis": "desgranar",
    "desgranó": "desgranar",
    "desgrapadoras": "desgrapadora",
    "desgrasa": "desgrasar",
    "desgrasaba": "desgrasar",
    "desgrasabais": "desgrasar",
    "desgrasaban": "desgrasar",
    "desgrasabas": "desgrasar",
    "desgrasad": "desgrasar",
    "desgrasada": "desgrasar",
    "desgrasadas": "desgrasar",
    "desgrasado": "desgrasar",
    "desgrasados": "desgrasar",
    "desgrasamos": "desgrasar",
    "desgrasan": "desgrasar",
    "desgrasando": "desgrasar",
    "desgrasara": "desgrasar",
    "desgrasarais": "desgrasar",
    "desgrasaran": "desgrasar",
    "desgrasaras": "desgrasar",
    "desgrasare": "desgrasar",
    "desgrasareis": "desgrasar",
    "desgrasaremos": "desgrasar",
    "desgrasaren": "desgrasar",
    "desgrasares": "desgrasar",
    "desgrasaron": "desgrasar",
    "desgrasará": "desgrasar",
    "desgrasarán": "desgrasar",
    "desgrasarás": "desgrasar",
    "desgrasaré": "desgrasar",
    "desgrasaréis": "desgrasar",
    "desgrasaría": "desgrasar",
    "desgrasaríais": "desgrasar",
    "desgrasaríamos": "desgrasar",
    "desgrasarían": "desgrasar",
    "desgrasarías": "desgrasar",
    "desgrasas": "desgrasar",
    "desgrasase": "desgrasar",
    "desgrasaseis": "desgrasar",
    "desgrasasen": "desgrasar",
    "desgrasases": "desgrasar",
    "desgrasaste": "desgrasar",
    "desgrasasteis": "desgrasar",
    "desgrase": "desgrasar",
    "desgrasemos": "desgrasar",
    "desgrasen": "desgrasar",
    "desgrases": "desgrase",
    "desgraso": "desgrasar",
    "desgrasábamos": "desgrasar",
    "desgrasáis": "desgrasar",
    "desgrasáramos": "desgrasar",
    "desgrasáremos": "desgrasar",
    "desgrasásemos": "desgrasar",
    "desgrasé": "desgrasar",
    "desgraséis": "desgrasar",
    "desgrasó": "desgrasar",
    "desgrava": "desgravar",
    "desgravaba": "desgravar",
    "desgravabais": "desgravar",
    "desgravaban": "desgravar",
    "desgravabas": "desgravar",
    "desgravables": "desgravable",
    "desgravaciones": "desgravación",
    "desgravad": "desgravar",
    "desgravada": "desgravar",
    "desgravadas": "desgravar",
    "desgravado": "desgravar",
    "desgravados": "desgravar",
    "desgravamos": "desgravar",
    "desgravan": "desgravar",
    "desgravando": "desgravar",
    "desgravara": "desgravar",
    "desgravarais": "desgravar",
    "desgravaran": "desgravar",
    "desgravaras": "desgravar",
    "desgravare": "desgravar",
    "desgravareis": "desgravar",
    "desgravaremos": "desgravar",
    "desgravaren": "desgravar",
    "desgravares": "desgravar",
    "desgravaron": "desgravar",
    "desgravará": "desgravar",
    "desgravarán": "desgravar",
    "desgravarás": "desgravar",
    "desgravaré": "desgravar",
    "desgravaréis": "desgravar",
    "desgravaría": "desgravar",
    "desgravaríais": "desgravar",
    "desgravaríamos": "desgravar",
    "desgravarían": "desgravar",
    "desgravarías": "desgravar",
    "desgravas": "desgravar",
    "desgravase": "desgravar",
    "desgravaseis": "desgravar",
    "desgravasen": "desgravar",
    "desgravases": "desgravar",
    "desgravaste": "desgravar",
    "desgravasteis": "desgravar",
    "desgrave": "desgravar",
    "desgravemos": "desgravar",
    "desgraven": "desgravar",
    "desgraves": "desgravar",
    "desgravo": "desgravar",
    "desgravábamos": "desgravar",
    "desgraváis": "desgravar",
    "desgravámenes": "desgravamen",
    "desgraváramos": "desgravar",
    "desgraváremos": "desgravar",
    "desgravásemos": "desgravar",
    "desgravé": "desgravar",
    "desgravéis": "desgravar",
    "desgravó": "desgravar",
    "desgreña": "desgreñar",
    "desgreñaba": "desgreñar",
    "desgreñabais": "desgreñar",
    "desgreñaban": "desgreñar",
    "desgreñabas": "desgreñar",
    "desgreñad": "desgreñar",
    "desgreñada": "desgreñar",
    "desgreñadas": "desgreñar",
    "desgreñado": "desgreñar",
    "desgreñados": "desgreñar",
    "desgreñamos": "desgreñar",
    "desgreñan": "desgreñar",
    "desgreñando": "desgreñar",
    "desgreñara": "desgreñar",
    "desgreñarais": "desgreñar",
    "desgreñaran": "desgreñar",
    "desgreñaras": "desgreñar",
    "desgreñare": "desgreñar",
    "desgreñareis": "desgreñar",
    "desgreñaremos": "desgreñar",
    "desgreñaren": "desgreñar",
    "desgreñares": "desgreñar",
    "desgreñaron": "desgreñar",
    "desgreñará": "desgreñar",
    "desgreñarán": "desgreñar",
    "desgreñarás": "desgreñar",
    "desgreñaré": "desgreñar",
    "desgreñaréis": "desgreñar",
    "desgreñaría": "desgreñar",
    "desgreñaríais": "desgreñar",
    "desgreñaríamos": "desgreñar",
    "desgreñarían": "desgreñar",
    "desgreñarías": "desgreñar",
    "desgreñas": "desgreñar",
    "desgreñase": "desgreñar",
    "desgreñaseis": "desgreñar",
    "desgreñasen": "desgreñar",
    "desgreñases": "desgreñar",
    "desgreñaste": "desgreñar",
    "desgreñasteis": "desgreñar",
    "desgreñe": "desgreñar",
    "desgreñemos": "desgreñar",
    "desgreñen": "desgreñar",
    "desgreñes": "desgreñar",
    "desgreño": "desgreñar",
    "desgreñábamos": "desgreñar",
    "desgreñáis": "desgreñar",
    "desgreñáramos": "desgreñar",
    "desgreñáremos": "desgreñar",
    "desgreñásemos": "desgreñar",
    "desgreñé": "desgreñar",
    "desgreñéis": "desgreñar",
    "desgreñó": "desgreñar",
    "desguace": "desguazar",
    "desguacemos": "desguazar",
    "desguacen": "desguazar",
    "desguaces": "desguazar",
    "desguacé": "desguazar",
    "desguacéis": "desguazar",
    "desguarnece": "desguarnecer",
    "desguarneced": "desguarnecer",
    "desguarnecemos": "desguarnecer",
    "desguarnecen": "desguarnecer",
    "desguarneceremos": "desguarnecer",
    "desguarnecerá": "desguarnecer",
    "desguarnecerán": "desguarnecer",
    "desguarnecerás": "desguarnecer",
    "desguarneceré": "desguarnecer",
    "desguarneceréis": "desguarnecer",
    "desguarnecería": "desguarnecer",
    "desguarneceríais": "desguarnecer",
    "desguarneceríamos": "desguarnecer",
    "desguarnecerían": "desguarnecer",
    "desguarnecerías": "desguarnecer",
    "desguarneces": "desguarnecer",
    "desguarnecida": "desguarnecer",
    "desguarnecidas": "desguarnecer",
    "desguarnecido": "desguarnecer",
    "desguarnecidos": "desguarnecer",
    "desguarneciendo": "desguarnecer",
    "desguarneciera": "desguarnecer",
    "desguarnecierais": "desguarnecer",
    "desguarnecieran": "desguarnecer",
    "desguarnecieras": "desguarnecer",
    "desguarneciere": "desguarnecer",
    "desguarneciereis": "desguarnecer",
    "desguarnecieren": "desguarnecer",
    "desguarnecieres": "desguarnecer",
    "desguarnecieron": "desguarnecer",
    "desguarneciese": "desguarnecer",
    "desguarnecieseis": "desguarnecer",
    "desguarneciesen": "desguarnecer",
    "desguarnecieses": "desguarnecer",
    "desguarnecimientos": "desguarnecimiento",
    "desguarnecimos": "desguarnecer",
    "desguarneciste": "desguarnecer",
    "desguarnecisteis": "desguarnecer",
    "desguarneciéramos": "desguarnecer",
    "desguarneciéremos": "desguarnecer",
    "desguarneciésemos": "desguarnecer",
    "desguarneció": "desguarnecer",
    "desguarnecéis": "desguarnecer",
    "desguarnecí": "desguarnecer",
    "desguarnecía": "desguarnecer",
    "desguarnecíais": "desguarnecer",
    "desguarnecíamos": "desguarnecer",
    "desguarnecían": "desguarnecer",
    "desguarnecías": "desguarnecer",
    "desguarnezca": "desguarnecer",
    "desguarnezcamos": "desguarnecer",
    "desguarnezcan": "desguarnecer",
    "desguarnezcas": "desguarnecer",
    "desguarnezco": "desguarnecer",
    "desguarnezcáis": "desguarnecer",
    "desguarniciones": "desguarnición",
    "desguaza": "desguazar",
    "desguazaba": "desguazar",
    "desguazabais": "desguazar",
    "desguazaban": "desguazar",
    "desguazabas": "desguazar",
    "desguazad": "desguazar",
    "desguazada": "desguazar",
    "desguazadas": "desguazar",
    "desguazado": "desguazar",
    "desguazados": "desguazar",
    "desguazamos": "desguazar",
    "desguazan": "desguazar",
    "desguazando": "desguazar",
    "desguazara": "desguazar",
    "desguazarais": "desguazar",
    "desguazaran": "desguazar",
    "desguazaras": "desguazar",
    "desguazare": "desguazar",
    "desguazareis": "desguazar",
    "desguazaremos": "desguazar",
    "desguazaren": "desguazar",
    "desguazares": "desguazar",
    "desguazaron": "desguazar",
    "desguazará": "desguazar",
    "desguazarán": "desguazar",
    "desguazarás": "desguazar",
    "desguazaré": "desguazar",
    "desguazaréis": "desguazar",
    "desguazaría": "desguazar",
    "desguazaríais": "desguazar",
    "desguazaríamos": "desguazar",
    "desguazarían": "desguazar",
    "desguazarías": "desguazar",
    "desguazas": "desguazar",
    "desguazase": "desguazar",
    "desguazaseis": "desguazar",
    "desguazasen": "desguazar",
    "desguazases": "desguazar",
    "desguazaste": "desguazar",
    "desguazasteis": "desguazar",
    "desguazo": "desguazar",
    "desguazábamos": "desguazar",
    "desguazáis": "desguazar",
    "desguazáramos": "desguazar",
    "desguazáremos": "desguazar",
    "desguazásemos": "desguazar",
    "desguazó": "desguazar",
    "desguañangada": "desguañangado",
    "desguañangadas": "desguañangado",
    "desguañangados": "desguañangado",
    "desguince": "desguinzar",
    "desguincemos": "desguinzar",
    "desguincen": "desguinzar",
    "desguinces": "desguinzar",
    "desguincé": "desguinzar",
    "desguincéis": "desguinzar",
    "desguinda": "desguindar",
    "desguindaba": "desguindar",
    "desguindabais": "desguindar",
    "desguindaban": "desguindar",
    "desguindabas": "desguindar",
    "desguindad": "desguindar",
    "desguindada": "desguindar",
    "desguindadas": "desguindar",
    "desguindado": "desguindar",
    "desguindados": "desguindar",
    "desguindamos": "desguindar",
    "desguindan": "desguindar",
    "desguindando": "desguindar",
    "desguindara": "desguindar",
    "desguindarais": "desguindar",
    "desguindaran": "desguindar",
    "desguindaras": "desguindar",
    "desguindare": "desguindar",
    "desguindareis": "desguindar",
    "desguindaremos": "desguindar",
    "desguindaren": "desguindar",
    "desguindares": "desguindar",
    "desguindaron": "desguindar",
    "desguindará": "desguindar",
    "desguindarán": "desguindar",
    "desguindarás": "desguindar",
    "desguindaré": "desguindar",
    "desguindaréis": "desguindar",
    "desguindaría": "desguindar",
    "desguindaríais": "desguindar",
    "desguindaríamos": "desguindar",
    "desguindarían": "desguindar",
    "desguindarías": "desguindar",
    "desguindas": "desguindar",
    "desguindase": "desguindar",
    "desguindaseis": "desguindar",
    "desguindasen": "desguindar",
    "desguindases": "desguindar",
    "desguindaste": "desguindar",
    "desguindasteis": "desguindar",
    "desguinde": "desguindar",
    "desguindemos": "desguindar",
    "desguinden": "desguindar",
    "desguindes": "desguindar",
    "desguindo": "desguindar",
    "desguindábamos": "desguindar",
    "desguindáis": "desguindar",
    "desguindáramos": "desguindar",
    "desguindáremos": "desguindar",
    "desguindásemos": "desguindar",
    "desguindé": "desguindar",
    "desguindéis": "desguindar",
    "desguindó": "desguindar",
    "desguinza": "desguinzar",
    "desguinzaba": "desguinzar",
    "desguinzabais": "desguinzar",
    "desguinzaban": "desguinzar",
    "desguinzabas": "desguinzar",
    "desguinzad": "desguinzar",
    "desguinzada": "desguinzar",
    "desguinzadas": "desguinzar",
    "desguinzado": "desguinzar",
    "desguinzados": "desguinzar",
    "desguinzamos": "desguinzar",
    "desguinzan": "desguinzar",
    "desguinzando": "desguinzar",
    "desguinzara": "desguinzar",
    "desguinzarais": "desguinzar",
    "desguinzaran": "desguinzar",
    "desguinzaras": "desguinzar",
    "desguinzare": "desguinzar",
    "desguinzareis": "desguinzar",
    "desguinzaremos": "desguinzar",
    "desguinzaren": "desguinzar",
    "desguinzares": "desguinzar",
    "desguinzaron": "desguinzar",
    "desguinzará": "desguinzar",
    "desguinzarán": "desguinzar",
    "desguinzarás": "desguinzar",
    "desguinzaré": "desguinzar",
    "desguinzaréis": "desguinzar",
    "desguinzaría": "desguinzar",
    "desguinzaríais": "desguinzar",
    "desguinzaríamos": "desguinzar",
    "desguinzarían": "desguinzar",
    "desguinzarías": "desguinzar",
    "desguinzas": "desguinzar",
    "desguinzase": "desguinzar",
    "desguinzaseis": "desguinzar",
    "desguinzasen": "desguinzar",
    "desguinzases": "desguinzar",
    "desguinzaste": "desguinzar",
    "desguinzasteis": "desguinzar",
    "desguinzo": "desguinzar",
    "desguinzábamos": "desguinzar",
    "desguinzáis": "desguinzar",
    "desguinzáramos": "desguinzar",
    "desguinzáremos": "desguinzar",
    "desguinzásemos": "desguinzar",
    "desguinzó": "desguinzar",
    "desgüaces": "desgüace",
    "deshabillés": "deshabillé",
    "deshabita": "deshabitar",
    "deshabitaba": "deshabitar",
    "deshabitabais": "deshabitar",
    "deshabitaban": "deshabitar",
    "deshabitabas": "deshabitar",
    "deshabitad": "deshabitar",
    "deshabitada": "deshabitar",
    "deshabitadas": "deshabitar",
    "deshabitado": "deshabitar",
    "deshabitados": "deshabitar",
    "deshabitamos": "deshabitar",
    "deshabitan": "deshabitar",
    "deshabitando": "deshabitar",
    "deshabitara": "deshabitar",
    "deshabitarais": "deshabitar",
    "deshabitaran": "deshabitar",
    "deshabitaras": "deshabitar",
    "deshabitare": "deshabitar",
    "deshabitareis": "deshabitar",
    "deshabitaremos": "deshabitar",
    "deshabitaren": "deshabitar",
    "deshabitares": "deshabitar",
    "deshabitaron": "deshabitar",
    "deshabitará": "deshabitar",
    "deshabitarán": "deshabitar",
    "deshabitarás": "deshabitar",
    "deshabitaré": "deshabitar",
    "deshabitaréis": "deshabitar",
    "deshabitaría": "deshabitar",
    "deshabitaríais": "deshabitar",
    "deshabitaríamos": "deshabitar",
    "deshabitarían": "deshabitar",
    "deshabitarías": "deshabitar",
    "deshabitas": "deshabitar",
    "deshabitase": "deshabitar",
    "deshabitaseis": "deshabitar",
    "deshabitasen": "deshabitar",
    "deshabitases": "deshabitar",
    "deshabitaste": "deshabitar",
    "deshabitasteis": "deshabitar",
    "deshabite": "deshabitar",
    "deshabitemos": "deshabitar",
    "deshabiten": "deshabitar",
    "deshabites": "deshabitar",
    "deshabito": "deshabitar",
    "deshabituaba": "deshabituar",
    "deshabituabais": "deshabituar",
    "deshabituaban": "deshabituar",
    "deshabituabas": "deshabituar",
    "deshabituaciones": "deshabituación",
    "deshabituad": "deshabituar",
    "deshabituada": "deshabituar",
    "deshabituadas": "deshabituar",
    "deshabituado": "deshabituar",
    "deshabituados": "deshabituar",
    "deshabituamos": "deshabituar",
    "deshabituando": "deshabituar",
    "deshabituara": "deshabituar",
    "deshabituarais": "deshabituar",
    "deshabituaran": "deshabituar",
    "deshabituaras": "deshabituar",
    "deshabituare": "deshabituar",
    "deshabituareis": "deshabituar",
    "deshabituaremos": "deshabituar",
    "deshabituaren": "deshabituar",
    "deshabituares": "deshabituar",
    "deshabituaron": "deshabituar",
    "deshabituará": "deshabituar",
    "deshabituarán": "deshabituar",
    "deshabituarás": "deshabituar",
    "deshabituaré": "deshabituar",
    "deshabituaréis": "deshabituar",
    "deshabituaría": "deshabituar",
    "deshabituaríais": "deshabituar",
    "deshabituaríamos": "deshabituar",
    "deshabituarían": "deshabituar",
    "deshabituarías": "deshabituar",
    "deshabituase": "deshabituar",
    "deshabituaseis": "deshabituar",
    "deshabituasen": "deshabituar",
    "deshabituases": "deshabituar",
    "deshabituaste": "deshabituar",
    "deshabituasteis": "deshabituar",
    "deshabituemos": "deshabituar",
    "deshabituábamos": "deshabituar",
    "deshabituáis": "deshabituar",
    "deshabituáramos": "deshabituar",
    "deshabituáremos": "deshabituar",
    "deshabituásemos": "deshabituar",
    "deshabitué": "deshabituar",
    "deshabituéis": "deshabituar",
    "deshabituó": "deshabituar",
    "deshabitábamos": "deshabitar",
    "deshabitáis": "deshabitar",
    "deshabitáramos": "deshabitar",
    "deshabitáremos": "deshabitar",
    "deshabitásemos": "deshabitar",
    "deshabité": "deshabitar",
    "deshabitéis": "deshabitar",
    "deshabitó": "deshabitar",
    "deshabitúa": "deshabituar",
    "deshabitúan": "deshabituar",
    "deshabitúas": "deshabituar",
    "deshabitúe": "deshabituar",
    "deshabitúen": "deshabituar",
    "deshabitúes": "deshabituar",
    "deshabitúo": "deshabituar",
    "deshace": "deshacer",
    "deshaced": "deshacer",
    "deshacemos": "deshacer",
    "deshacen": "deshacer",
    "deshaces": "deshacer",
    "deshaciendo": "deshacer",
    "deshacéis": "deshacer",
    "deshacía": "deshacer",
    "deshacíais": "deshacer",
    "deshacíamos": "deshacer",
    "deshacían": "deshacer",
    "deshacías": "deshacer",
    "deshaga": "deshacer",
    "deshagamos": "deshacer",
    "deshagan": "deshacer",
    "deshagas": "deshacer",
    "deshago": "deshacer",
    "deshagáis": "deshacer",
    "deshaldos": "deshaldo",
    "deshambrida": "deshambrido",
    "deshambridas": "deshambrido",
    "deshambridos": "deshambrido",
    "desharemos": "deshacer",
    "desharrapada": "desharrapado",
    "desharrapadas": "desharrapado",
    "desharrapados": "desharrapado",
    "desharrapamientos": "desharrapamiento",
    "deshará": "deshacer",
    "desharán": "deshacer",
    "desharás": "deshacer",
    "desharé": "deshacer",
    "desharéis": "deshacer",
    "desharía": "deshacer",
    "desharíais": "deshacer",
    "desharíamos": "deshacer",
    "desharían": "deshacer",
    "desharías": "deshacer",
    "deshaz": "deshacer",
    "deshebilla": "deshebillar",
    "deshebillaba": "deshebillar",
    "deshebillabais": "deshebillar",
    "deshebillaban": "deshebillar",
    "deshebillabas": "deshebillar",
    "deshebillad": "deshebillar",
    "deshebillada": "deshebillar",
    "deshebilladas": "deshebillar",
    "deshebillado": "deshebillar",
    "deshebillados": "deshebillar",
    "deshebillamos": "deshebillar",
    "deshebillan": "deshebillar",
    "deshebillando": "deshebillar",
    "deshebillara": "deshebillar",
    "deshebillarais": "deshebillar",
    "deshebillaran": "deshebillar",
    "deshebillaras": "deshebillar",
    "deshebillare": "deshebillar",
    "deshebillareis": "deshebillar",
    "deshebillaremos": "deshebillar",
    "deshebillaren": "deshebillar",
    "deshebillares": "deshebillar",
    "deshebillaron": "deshebillar",
    "deshebillará": "deshebillar",
    "deshebillarán": "deshebillar",
    "deshebillarás": "deshebillar",
    "deshebillaré": "deshebillar",
    "deshebillaréis": "deshebillar",
    "deshebillaría": "deshebillar",
    "deshebillaríais": "deshebillar",
    "deshebillaríamos": "deshebillar",
    "deshebillarían": "deshebillar",
    "deshebillarías": "deshebillar",
    "deshebillas": "deshebillar",
    "deshebillase": "deshebillar",
    "deshebillaseis": "deshebillar",
    "deshebillasen": "deshebillar",
    "deshebillases": "deshebillar",
    "deshebillaste": "deshebillar",
    "deshebillasteis": "deshebillar",
    "deshebille": "deshebillar",
    "deshebillemos": "deshebillar",
    "deshebillen": "deshebillar",
    "deshebilles": "deshebillar",
    "deshebillo": "deshebillar",
    "deshebillábamos": "deshebillar",
    "deshebilláis": "deshebillar",
    "deshebilláramos": "deshebillar",
    "deshebilláremos": "deshebillar",
    "deshebillásemos": "deshebillar",
    "deshebillé": "deshebillar",
    "deshebilléis": "deshebillar",
    "deshebilló": "deshebillar",
    "deshebra": "deshebrar",
    "deshebraba": "deshebrar",
    "deshebrabais": "deshebrar",
    "deshebraban": "deshebrar",
    "deshebrabas": "deshebrar",
    "deshebrad": "deshebrar",
    "deshebrada": "deshebrar",
    "deshebradas": "deshebrar",
    "deshebrado": "deshebrar",
    "deshebrados": "deshebrar",
    "deshebramos": "deshebrar",
    "deshebran": "deshebrar",
    "deshebrando": "deshebrar",
    "deshebrara": "deshebrar",
    "deshebrarais": "deshebrar",
    "deshebraran": "deshebrar",
    "deshebraras": "deshebrar",
    "deshebrare": "deshebrar",
    "deshebrareis": "deshebrar",
    "deshebraremos": "deshebrar",
    "deshebraren": "deshebrar",
    "deshebrares": "deshebrar",
    "deshebraron": "deshebrar",
    "deshebrará": "deshebrar",
    "deshebrarán": "deshebrar",
    "deshebrarás": "deshebrar",
    "deshebraré": "deshebrar",
    "deshebraréis": "deshebrar",
    "deshebraría": "deshebrar",
    "deshebraríais": "deshebrar",
    "deshebraríamos": "deshebrar",
    "deshebrarían": "deshebrar",
    "deshebrarías": "deshebrar",
    "deshebras": "deshebrar",
    "deshebrase": "deshebrar",
    "deshebraseis": "deshebrar",
    "deshebrasen": "deshebrar",
    "deshebrases": "deshebrar",
    "deshebraste": "deshebrar",
    "deshebrasteis": "deshebrar",
    "deshebre": "deshebrar",
    "deshebremos": "deshebrar",
    "deshebren": "deshebrar",
    "deshebres": "deshebrar",
    "deshebro": "deshebrar",
    "deshebrábamos": "deshebrar",
    "deshebráis": "deshebrar",
    "deshebráramos": "deshebrar",
    "deshebráremos": "deshebrar",
    "deshebrásemos": "deshebrar",
    "deshebré": "deshebrar",
    "deshebréis": "deshebrar",
    "deshebró": "deshebrar",
    "deshecha": "deshacer",
    "deshechas": "deshacer",
    "deshechice": "deshechizar",
    "deshechicemos": "deshechizar",
    "deshechicen": "deshechizar",
    "deshechices": "deshechizar",
    "deshechicé": "deshechizar",
    "deshechicéis": "deshechizar",
    "deshechiza": "deshechizar",
    "deshechizaba": "deshechizar",
    "deshechizabais": "deshechizar",
    "deshechizaban": "deshechizar",
    "deshechizabas": "deshechizar",
    "deshechizad": "deshechizar",
    "deshechizada": "deshechizar",
    "deshechizadas": "deshechizar",
    "deshechizado": "deshechizar",
    "deshechizados": "deshechizar",
    "deshechizamientos": "deshechizamiento",
    "deshechizamos": "deshechizar",
    "deshechizan": "deshechizar",
    "deshechizando": "deshechizar",
    "deshechizara": "deshechizar",
    "deshechizarais": "deshechizar",
    "deshechizaran": "deshechizar",
    "deshechizaras": "deshechizar",
    "deshechizare": "deshechizar",
    "deshechizareis": "deshechizar",
    "deshechizaremos": "deshechizar",
    "deshechizaren": "deshechizar",
    "deshechizares": "deshechizar",
    "deshechizaron": "deshechizar",
    "deshechizará": "deshechizar",
    "deshechizarán": "deshechizar",
    "deshechizarás": "deshechizar",
    "deshechizaré": "deshechizar",
    "deshechizaréis": "deshechizar",
    "deshechizaría": "deshechizar",
    "deshechizaríais": "deshechizar",
    "deshechizaríamos": "deshechizar",
    "deshechizarían": "deshechizar",
    "deshechizarías": "deshechizar",
    "deshechizas": "deshechizar",
    "deshechizase": "deshechizar",
    "deshechizaseis": "deshechizar",
    "deshechizasen": "deshechizar",
    "deshechizases": "deshechizar",
    "deshechizaste": "deshechizar",
    "deshechizasteis": "deshechizar",
    "deshechizo": "deshechizar",
    "deshechizábamos": "deshechizar",
    "deshechizáis": "deshechizar",
    "deshechizáramos": "deshechizar",
    "deshechizáremos": "deshechizar",
    "deshechizásemos": "deshechizar",
    "deshechizó": "deshechizar",
    "deshecho": "deshacer",
    "deshechos": "deshecho",
    "deshederaciones": "deshederación",
    "deshelaba": "deshelar",
    "deshelabais": "deshelar",
    "deshelaban": "deshelar",
    "deshelabas": "deshelar",
    "deshelad": "deshelar",
    "deshelada": "deshelar",
    "desheladas": "deshelar",
    "deshelado": "deshelar",
    "deshelados": "deshelar",
    "deshelamos": "deshelar",
    "deshelando": "deshelar",
    "deshelara": "deshelar",
    "deshelarais": "deshelar",
    "deshelaran": "deshelar",
    "deshelaras": "deshelar",
    "deshelare": "deshelar",
    "deshelareis": "deshelar",
    "deshelaremos": "deshelar",
    "deshelaren": "deshelar",
    "deshelares": "deshelar",
    "deshelaron": "deshelar",
    "deshelará": "deshelar",
    "deshelarán": "deshelar",
    "deshelarás": "deshelar",
    "deshelaré": "deshelar",
    "deshelaréis": "deshelar",
    "deshelaría": "deshelar",
    "deshelaríais": "deshelar",
    "deshelaríamos": "deshelar",
    "deshelarían": "deshelar",
    "deshelarías": "deshelar",
    "deshelase": "deshelar",
    "deshelaseis": "deshelar",
    "deshelasen": "deshelar",
    "deshelases": "deshelar",
    "deshelaste": "deshelar",
    "deshelasteis": "deshelar",
    "deshelemos": "deshelar",
    "deshelábamos": "deshelar",
    "desheláis": "deshelar",
    "desheláramos": "deshelar",
    "desheláremos": "deshelar",
    "deshelásemos": "deshelar",
    "deshelé": "deshelar",
    "desheléis": "deshelar",
    "desheló": "deshelar",
    "desherbaba": "desherbar",
    "desherbabais": "desherbar",
    "desherbaban": "desherbar",
    "desherbabas": "desherbar",
    "desherbad": "desherbar",
    "desherbada": "desherbar",
    "desherbadas": "desherbar",
    "desherbado": "desherbar",
    "desherbadoras": "desherbadora",
    "desherbados": "desherbar",
    "desherbamos": "desherbar",
    "desherbando": "desherbar",
    "desherbara": "desherbar",
    "desherbarais": "desherbar",
    "desherbaran": "desherbar",
    "desherbaras": "desherbar",
    "desherbare": "desherbar",
    "desherbareis": "desherbar",
    "desherbaremos": "desherbar",
    "desherbaren": "desherbar",
    "desherbares": "desherbar",
    "desherbaron": "desherbar",
    "desherbará": "desherbar",
    "desherbarán": "desherbar",
    "desherbarás": "desherbar",
    "desherbaré": "desherbar",
    "desherbaréis": "desherbar",
    "desherbaría": "desherbar",
    "desherbaríais": "desherbar",
    "desherbaríamos": "desherbar",
    "desherbarían": "desherbar",
    "desherbarías": "desherbar",
    "desherbase": "desherbar",
    "desherbaseis": "desherbar",
    "desherbasen": "desherbar",
    "desherbases": "desherbar",
    "desherbaste": "desherbar",
    "desherbasteis": "desherbar",
    "desherbemos": "desherbar",
    "desherbábamos": "desherbar",
    "desherbáis": "desherbar",
    "desherbáramos": "desherbar",
    "desherbáremos": "desherbar",
    "desherbásemos": "desherbar",
    "desherbé": "desherbar",
    "desherbéis": "desherbar",
    "desherbó": "desherbar",
    "deshereda": "desheredar",
    "desheredaba": "desheredar",
    "desheredabais": "desheredar",
    "desheredaban": "desheredar",
    "desheredabas": "desheredar",
    "desheredaciones": "desheredación",
    "desheredad": "desheredar",
    "desheredada": "desheredar",
    "desheredadas": "desheredar",
    "desheredado": "desheredar",
    "desheredados": "desheredar",
    "desheredamientos": "desheredamiento",
    "desheredamos": "desheredar",
    "desheredan": "desheredar",
    "desheredando": "desheredar",
    "desheredara": "desheredar",
    "desheredarais": "desheredar",
    "desheredaran": "desheredar",
    "desheredaras": "desheredar",
    "desheredare": "desheredar",
    "desheredareis": "desheredar",
    "desheredaremos": "desheredar",
    "desheredaren": "desheredar",
    "desheredares": "desheredar",
    "desheredaron": "desheredar",
    "desheredará": "desheredar",
    "desheredarán": "desheredar",
    "desheredarás": "desheredar",
    "desheredaré": "desheredar",
    "desheredaréis": "desheredar",
    "desheredaría": "desheredar",
    "desheredaríais": "desheredar",
    "desheredaríamos": "desheredar",
    "desheredarían": "desheredar",
    "desheredarías": "desheredar",
    "desheredas": "desheredar",
    "desheredase": "desheredar",
    "desheredaseis": "desheredar",
    "desheredasen": "desheredar",
    "desheredases": "desheredar",
    "desheredaste": "desheredar",
    "desheredasteis": "desheredar",
    "desherede": "desheredar",
    "desheredemos": "desheredar",
    "deshereden": "desheredar",
    "desheredes": "desheredar",
    "desheredo": "desheredar",
    "desheredábamos": "desheredar",
    "desheredáis": "desheredar",
    "desheredáramos": "desheredar",
    "desheredáremos": "desheredar",
    "desheredásemos": "desheredar",
    "desheredé": "desheredar",
    "desheredéis": "desheredar",
    "desheredó": "desheredar",
    "deshermana": "deshermanar",
    "deshermanaba": "deshermanar",
    "deshermanabais": "deshermanar",
    "deshermanaban": "deshermanar",
    "deshermanabas": "deshermanar",
    "deshermanad": "deshermanar",
    "deshermanada": "deshermanar",
    "deshermanadas": "deshermanar",
    "deshermanado": "deshermanar",
    "deshermanados": "deshermanar",
    "deshermanamos": "deshermanar",
    "deshermanan": "deshermanar",
    "deshermanando": "deshermanar",
    "deshermanara": "deshermanar",
    "deshermanarais": "deshermanar",
    "deshermanaran": "deshermanar",
    "deshermanaras": "deshermanar",
    "deshermanare": "deshermanar",
    "deshermanareis": "deshermanar",
    "deshermanaremos": "deshermanar",
    "deshermanaren": "deshermanar",
    "deshermanares": "deshermanar",
    "deshermanaron": "deshermanar",
    "deshermanará": "deshermanar",
    "deshermanarán": "deshermanar",
    "deshermanarás": "deshermanar",
    "deshermanaré": "deshermanar",
    "deshermanaréis": "deshermanar",
    "deshermanaría": "deshermanar",
    "deshermanaríais": "deshermanar",
    "deshermanaríamos": "deshermanar",
    "deshermanarían": "deshermanar",
    "deshermanarías": "deshermanar",
    "deshermanas": "deshermanar",
    "deshermanase": "deshermanar",
    "deshermanaseis": "deshermanar",
    "deshermanasen": "deshermanar",
    "deshermanases": "deshermanar",
    "deshermanaste": "deshermanar",
    "deshermanasteis": "deshermanar",
    "deshermane": "deshermanar",
    "deshermanemos": "deshermanar",
    "deshermanen": "deshermanar",
    "deshermanes": "deshermanar",
    "deshermano": "deshermanar",
    "deshermanábamos": "deshermanar",
    "deshermanáis": "deshermanar",
    "deshermanáramos": "deshermanar",
    "deshermanáremos": "deshermanar",
    "deshermanásemos": "deshermanar",
    "deshermané": "deshermanar",
    "deshermanéis": "deshermanar",
    "deshermanó": "deshermanar",
    "desherraba": "desherrar",
    "desherrabais": "desherrar",
    "desherraban": "desherrar",
    "desherrabas": "desherrar",
    "desherrad": "desherrar",
    "desherrada": "desherrar",
    "desherradas": "desherrar",
    "desherrado": "desherrar",
    "desherrados": "desherrar",
    "desherraduras": "desherradura",
    "desherramos": "desherrar",
    "desherrando": "desherrar",
    "desherrara": "desherrar",
    "desherrarais": "desherrar",
    "desherraran": "desherrar",
    "desherraras": "desherrar",
    "desherrare": "desherrar",
    "desherrareis": "desherrar",
    "desherraremos": "desherrar",
    "desherraren": "desherrar",
    "desherrares": "desherrar",
    "desherraron": "desherrar",
    "desherrará": "desherrar",
    "desherrarán": "desherrar",
    "desherrarás": "desherrar",
    "desherraré": "desherrar",
    "desherraréis": "desherrar",
    "desherraría": "desherrar",
    "desherraríais": "desherrar",
    "desherraríamos": "desherrar",
    "desherrarían": "desherrar",
    "desherrarías": "desherrar",
    "desherrase": "desherrar",
    "desherraseis": "desherrar",
    "desherrasen": "desherrar",
    "desherrases": "desherrar",
    "desherraste": "desherrar",
    "desherrasteis": "desherrar",
    "desherremos": "desherrar",
    "desherrumbra": "desherrumbrar",
    "desherrumbraba": "desherrumbrar",
    "desherrumbrabais": "desherrumbrar",
    "desherrumbraban": "desherrumbrar",
    "desherrumbrabas": "desherrumbrar",
    "desherrumbrad": "desherrumbrar",
    "desherrumbrada": "desherrumbrar",
    "desherrumbradas": "desherrumbrar",
    "desherrumbrado": "desherrumbrar",
    "desherrumbrados": "desherrumbrar",
    "desherrumbramientos": "desherrumbramiento",
    "desherrumbramos": "desherrumbrar",
    "desherrumbran": "desherrumbrar",
    "desherrumbrando": "desherrumbrar",
    "desherrumbrara": "desherrumbrar",
    "desherrumbrarais": "desherrumbrar",
    "desherrumbraran": "desherrumbrar",
    "desherrumbraras": "desherrumbrar",
    "desherrumbrare": "desherrumbrar",
    "desherrumbrareis": "desherrumbrar",
    "desherrumbraremos": "desherrumbrar",
    "desherrumbraren": "desherrumbrar",
    "desherrumbrares": "desherrumbrar",
    "desherrumbraron": "desherrumbrar",
    "desherrumbrará": "desherrumbrar",
    "desherrumbrarán": "desherrumbrar",
    "desherrumbrarás": "desherrumbrar",
    "desherrumbraré": "desherrumbrar",
    "desherrumbraréis": "desherrumbrar",
    "desherrumbraría": "desherrumbrar",
    "desherrumbraríais": "desherrumbrar",
    "desherrumbraríamos": "desherrumbrar",
    "desherrumbrarían": "desherrumbrar",
    "desherrumbrarías": "desherrumbrar",
    "desherrumbras": "desherrumbrar",
    "desherrumbrase": "desherrumbrar",
    "desherrumbraseis": "desherrumbrar",
    "desherrumbrasen": "desherrumbrar",
    "desherrumbrases": "desherrumbrar",
    "desherrumbraste": "desherrumbrar",
    "desherrumbrasteis": "desherrumbrar",
    "desherrumbre": "desherrumbrar",
    "desherrumbremos": "desherrumbrar",
    "desherrumbren": "desherrumbrar",
    "desherrumbres": "desherrumbrar",
    "desherrumbro": "desherrumbrar",
    "desherrumbrábamos": "desherrumbrar",
    "desherrumbráis": "desherrumbrar",
    "desherrumbráramos": "desherrumbrar",
    "desherrumbráremos": "desherrumbrar",
    "desherrumbrásemos": "desherrumbrar",
    "desherrumbré": "desherrumbrar",
    "desherrumbréis": "desherrumbrar",
    "desherrumbró": "desherrumbrar",
    "desherrábamos": "desherrar",
    "desherráis": "desherrar",
    "desherráramos": "desherrar",
    "desherráremos": "desherrar",
    "desherrásemos": "desherrar",
    "desherré": "desherrar",
    "desherréis": "desherrar",
    "desherró": "desherrar",
    "deshice": "deshacer",
    "deshiciera": "deshacer",
    "deshicierais": "deshacer",
    "deshicieran": "deshacer",
    "deshicieras": "deshacer",
    "deshiciere": "deshacer",
    "deshiciereis": "deshacer",
    "deshicieren": "deshacer",
    "deshicieres": "deshacer",
    "deshicieron": "deshacer",
    "deshiciese": "deshacer",
    "deshicieseis": "deshacer",
    "deshiciesen": "deshacer",
    "deshicieses": "deshacer",
    "deshicimos": "deshacer",
    "deshiciste": "deshacer",
    "deshicisteis": "deshacer",
    "deshiciéramos": "deshacer",
    "deshiciéremos": "deshacer",
    "deshiciésemos": "deshacer",
    "deshidrasas": "deshidrasa",
    "deshidrata": "deshidratar",
    "deshidrataba": "deshidratar",
    "deshidratabais": "deshidratar",
    "deshidrataban": "deshidratar",
    "deshidratabas": "deshidratar",
    "deshidrataciones": "deshidratación",
    "deshidratad": "deshidratar",
    "deshidratada": "deshidratar",
    "deshidratadas": "deshidratar",
    "deshidratado": "deshidratar",
    "deshidratadora": "deshidratador",
    "deshidratadoras": "deshidratadora",
    "deshidratadores": "deshidratador",
    "deshidratados": "deshidratar",
    "deshidratamos": "deshidratar",
    "deshidratan": "deshidratar",
    "deshidratando": "deshidratar",
    "deshidratantes": "deshidratante",
    "deshidratara": "deshidratar",
    "deshidratarais": "deshidratar",
    "deshidrataran": "deshidratar",
    "deshidrataras": "deshidratar",
    "deshidratare": "deshidratar",
    "deshidratareis": "deshidratar",
    "deshidrataremos": "deshidratar",
    "deshidrataren": "deshidratar",
    "deshidratares": "deshidratar",
    "deshidrataron": "deshidratar",
    "deshidratará": "deshidratar",
    "deshidratarán": "deshidratar",
    "deshidratarás": "deshidratar",
    "deshidrataré": "deshidratar",
    "deshidrataréis": "deshidratar",
    "deshidrataría": "deshidratar",
    "deshidrataríais": "deshidratar",
    "deshidrataríamos": "deshidratar",
    "deshidratarían": "deshidratar",
    "deshidratarías": "deshidratar",
    "deshidratas": "deshidratar",
    "deshidratase": "deshidratar",
    "deshidrataseis": "deshidratar",
    "deshidratasen": "deshidratar",
    "deshidratases": "deshidratar",
    "deshidrataste": "deshidratar",
    "deshidratasteis": "deshidratar",
    "deshidrate": "deshidratar",
    "deshidratemos": "deshidratar",
    "deshidraten": "deshidratar",
    "deshidrates": "deshidratar",
    "deshidrato": "deshidratar",
    "deshidratábamos": "deshidratar",
    "deshidratáis": "deshidratar",
    "deshidratáramos": "deshidratar",
    "deshidratáremos": "deshidratar",
    "deshidratásemos": "deshidratar",
    "deshidraté": "deshidratar",
    "deshidratéis": "deshidratar",
    "deshidrató": "deshidratar",
    "deshidrociclizaciones": "deshidrociclización",
    "deshidrogena": "deshidrogenar",
    "deshidrogenaba": "deshidrogenar",
    "deshidrogenabais": "deshidrogenar",
    "deshidrogenaban": "deshidrogenar",
    "deshidrogenabas": "deshidrogenar",
    "deshidrogenaciones": "deshidrogenación",
    "deshidrogenad": "deshidrogenar",
    "deshidrogenada": "deshidrogenar",
    "deshidrogenadas": "deshidrogenar",
    "deshidrogenado": "deshidrogenar",
    "deshidrogenados": "deshidrogenar",
    "deshidrogenamos": "deshidrogenar",
    "deshidrogenan": "deshidrogenar",
    "deshidrogenando": "deshidrogenar",
    "deshidrogenara": "deshidrogenar",
    "deshidrogenarais": "deshidrogenar",
    "deshidrogenaran": "deshidrogenar",
    "deshidrogenaras": "deshidrogenar",
    "deshidrogenare": "deshidrogenar",
    "deshidrogenareis": "deshidrogenar",
    "deshidrogenaremos": "deshidrogenar",
    "deshidrogenaren": "deshidrogenar",
    "deshidrogenares": "deshidrogenar",
    "deshidrogenaron": "deshidrogenar",
    "deshidrogenará": "deshidrogenar",
    "deshidrogenarán": "deshidrogenar",
    "deshidrogenarás": "deshidrogenar",
    "deshidrogenaré": "deshidrogenar",
    "deshidrogenaréis": "deshidrogenar",
    "deshidrogenaría": "deshidrogenar",
    "deshidrogenaríais": "deshidrogenar",
    "deshidrogenaríamos": "deshidrogenar",
    "deshidrogenarían": "deshidrogenar",
    "deshidrogenarías": "deshidrogenar",
    "deshidrogenas": "deshidrogenar",
    "deshidrogenasas": "deshidrogenasa",
    "deshidrogenase": "deshidrogenar",
    "deshidrogenaseis": "deshidrogenar",
    "deshidrogenasen": "deshidrogenar",
    "deshidrogenases": "deshidrogenar",
    "deshidrogenaste": "deshidrogenar",
    "deshidrogenasteis": "deshidrogenar",
    "deshidrogene": "deshidrogenar",
    "deshidrogenemos": "deshidrogenar",
    "deshidrogenen": "deshidrogenar",
    "deshidrogenes": "deshidrogenar",
    "deshidrogenizaciones": "deshidrogenización",
    "deshidrogeno": "deshidrogenar",
    "deshidrogenábamos": "deshidrogenar",
    "deshidrogenáis": "deshidrogenar",
    "deshidrogenáramos": "deshidrogenar",
    "deshidrogenáremos": "deshidrogenar",
    "deshidrogenásemos": "deshidrogenar",
    "deshidrogené": "deshidrogenar",
    "deshidrogenéis": "deshidrogenar",
    "deshidrogenó": "deshidrogenar",
    "deshiela": "deshelar",
    "deshielan": "deshelar",
    "deshielas": "deshelar",
    "deshiele": "deshelar",
    "deshielen": "deshelar",
    "deshieles": "deshelar",
    "deshielo": "deshelar",
    "deshielos": "deshielo",
    "deshierba": "desherbar",
    "deshierban": "desherbar",
    "deshierbas": "deshierba",
    "deshierbe": "desherbar",
    "deshierben": "desherbar",
    "deshierbes": "desherbar",
    "deshierbo": "desherbar",
    "deshierra": "desherrar",
    "deshierran": "desherrar",
    "deshierras": "desherrar",
    "deshierre": "desherrar",
    "deshierren": "desherrar",
    "deshierres": "desherrar",
    "deshierro": "desherrar",
    "deshija": "deshijar",
    "deshijaba": "deshijar",
    "deshijabais": "deshijar",
    "deshijaban": "deshijar",
    "deshijabas": "deshijar",
    "deshijad": "deshijar",
    "deshijada": "deshijar",
    "deshijadas": "deshijar",
    "deshijado": "deshijar",
    "deshijados": "deshijar",
    "deshijamos": "deshijar",
    "deshijan": "deshijar",
    "deshijando": "deshijar",
    "deshijara": "deshijar",
    "deshijarais": "deshijar",
    "deshijaran": "deshijar",
    "deshijaras": "deshijar",
    "deshijare": "deshijar",
    "deshijareis": "deshijar",
    "deshijaremos": "deshijar",
    "deshijaren": "deshijar",
    "deshijares": "deshijar",
    "deshijaron": "deshijar",
    "deshijará": "deshijar",
    "deshijarán": "deshijar",
    "deshijarás": "deshijar",
    "deshijaré": "deshijar",
    "deshijaréis": "deshijar",
    "deshijaría": "deshijar",
    "deshijaríais": "deshijar",
    "deshijaríamos": "deshijar",
    "deshijarían": "deshijar",
    "deshijarías": "deshijar",
    "deshijas": "deshijar",
    "deshijase": "deshijar",
    "deshijaseis": "deshijar",
    "deshijasen": "deshijar",
    "deshijases": "deshijar",
    "deshijaste": "deshijar",
    "deshijasteis": "deshijar",
    "deshije": "deshijar",
    "deshijemos": "deshijar",
    "deshijen": "deshijar",
    "deshijes": "deshijar",
    "deshijo": "deshijar",
    "deshijábamos": "deshijar",
    "deshijáis": "deshijar",
    "deshijáramos": "deshijar",
    "deshijáremos": "deshijar",
    "deshijásemos": "deshijar",
    "deshijé": "deshijar",
    "deshijéis": "deshijar",
    "deshijó": "deshijar",
    "deshila": "deshilar",
    "deshilaba": "deshilar",
    "deshilabais": "deshilar",
    "deshilaban": "deshilar",
    "deshilabas": "deshilar",
    "deshilacha": "deshilachar",
    "deshilachaba": "deshilachar",
    "deshilachabais": "deshilachar",
    "deshilachaban": "deshilachar",
    "deshilachabas": "deshilachar",
    "deshilachad": "deshilachar",
    "deshilachada": "deshilachar",
    "deshilachadas": "deshilachar",
    "deshilachado": "deshilachar",
    "deshilachados": "deshilachar",
    "deshilachamos": "deshilachar",
    "deshilachan": "deshilachar",
    "deshilachando": "deshilachar",
    "deshilachara": "deshilachar",
    "deshilacharais": "deshilachar",
    "deshilacharan": "deshilachar",
    "deshilacharas": "deshilachar",
    "deshilachare": "deshilachar",
    "deshilachareis": "deshilachar",
    "deshilacharemos": "deshilachar",
    "deshilacharen": "deshilachar",
    "deshilachares": "deshilachar",
    "deshilacharon": "deshilachar",
    "deshilachará": "deshilachar",
    "deshilacharán": "deshilachar",
    "deshilacharás": "deshilachar",
    "deshilacharé": "deshilachar",
    "deshilacharéis": "deshilachar",
    "deshilacharía": "deshilachar",
    "deshilacharíais": "deshilachar",
    "deshilacharíamos": "deshilachar",
    "deshilacharían": "deshilachar",
    "deshilacharías": "deshilachar",
    "deshilachas": "deshilachar",
    "deshilachase": "deshilachar",
    "deshilachaseis": "deshilachar",
    "deshilachasen": "deshilachar",
    "deshilachases": "deshilachar",
    "deshilachaste": "deshilachar",
    "deshilachasteis": "deshilachar",
    "deshilache": "deshilachar",
    "deshilachemos": "deshilachar",
    "deshilachen": "deshilachar",
    "deshilaches": "deshilachar",
    "deshilacho": "deshilachar",
    "deshilachábamos": "deshilachar",
    "deshilacháis": "deshilachar",
    "deshilacháramos": "deshilachar",
    "deshilacháremos": "deshilachar",
    "deshilachásemos": "deshilachar",
    "deshilaché": "deshilachar",
    "deshilachéis": "deshilachar",
    "deshilachó": "deshilachar",
    "deshilad": "deshilar",
    "deshilada": "deshilar",
    "deshiladas": "deshilar",
    "deshilado": "deshilar",
    "deshilados": "deshilar",
    "deshiladuras": "deshiladura",
    "deshilamos": "deshilar",
    "deshilan": "deshilar",
    "deshilando": "deshilar",
    "deshilara": "deshilar",
    "deshilarais": "deshilar",
    "deshilaran": "deshilar",
    "deshilaras": "deshilar",
    "deshilare": "deshilar",
    "deshilareis": "deshilar",
    "deshilaremos": "deshilar",
    "deshilaren": "deshilar",
    "deshilares": "deshilar",
    "deshilaron": "deshilar",
    "deshilará": "deshilar",
    "deshilarán": "deshilar",
    "deshilarás": "deshilar",
    "deshilaré": "deshilar",
    "deshilaréis": "deshilar",
    "deshilaría": "deshilar",
    "deshilaríais": "deshilar",
    "deshilaríamos": "deshilar",
    "deshilarían": "deshilar",
    "deshilarías": "deshilar",
    "deshilas": "deshilar",
    "deshilase": "deshilar",
    "deshilaseis": "deshilar",
    "deshilasen": "deshilar",
    "deshilases": "deshilar",
    "deshilaste": "deshilar",
    "deshilasteis": "deshilar",
    "deshile": "deshilar",
    "deshilemos": "deshilar",
    "deshilen": "deshilar",
    "deshiles": "deshilar",
    "deshilo": "deshilar",
    "deshilos": "deshilo",
    "deshilvana": "deshilvanar",
    "deshilvanaba": "deshilvanar",
    "deshilvanabais": "deshilvanar",
    "deshilvanaban": "deshilvanar",
    "deshilvanabas": "deshilvanar",
    "deshilvanad": "deshilvanar",
    "deshilvanada": "deshilvanar",
    "deshilvanadas": "deshilvanar",
    "deshilvanado": "deshilvanar",
    "deshilvanados": "deshilvanar",
    "deshilvanamos": "deshilvanar",
    "deshilvanan": "deshilvanar",
    "deshilvanando": "deshilvanar",
    "deshilvanara": "deshilvanar",
    "deshilvanarais": "deshilvanar",
    "deshilvanaran": "deshilvanar",
    "deshilvanaras": "deshilvanar",
    "deshilvanare": "deshilvanar",
    "deshilvanareis": "deshilvanar",
    "deshilvanaremos": "deshilvanar",
    "deshilvanaren": "deshilvanar",
    "deshilvanares": "deshilvanar",
    "deshilvanaron": "deshilvanar",
    "deshilvanará": "deshilvanar",
    "deshilvanarán": "deshilvanar",
    "deshilvanarás": "deshilvanar",
    "deshilvanaré": "deshilvanar",
    "deshilvanaréis": "deshilvanar",
    "deshilvanaría": "deshilvanar",
    "deshilvanaríais": "deshilvanar",
    "deshilvanaríamos": "deshilvanar",
    "deshilvanarían": "deshilvanar",
    "deshilvanarías": "deshilvanar",
    "deshilvanas": "deshilvanar",
    "deshilvanase": "deshilvanar",
    "deshilvanaseis": "deshilvanar",
    "deshilvanasen": "deshilvanar",
    "deshilvanases": "deshilvanar",
    "deshilvanaste": "deshilvanar",
    "deshilvanasteis": "deshilvanar",
    "deshilvane": "deshilvanar",
    "deshilvanemos": "deshilvanar",
    "deshilvanen": "deshilvanar",
    "deshilvanes": "deshilvanar",
    "deshilvano": "deshilvanar",
    "deshilvanábamos": "deshilvanar",
    "deshilvanáis": "deshilvanar",
    "deshilvanáramos": "deshilvanar",
    "deshilvanáremos": "deshilvanar",
    "deshilvanásemos": "deshilvanar",
    "deshilvané": "deshilvanar",
    "deshilvanéis": "deshilvanar",
    "deshilvanó": "deshilvanar",
    "deshilábamos": "deshilar",
    "deshiláis": "deshilar",
    "deshiláramos": "deshilar",
    "deshiláremos": "deshilar",
    "deshilásemos": "deshilar",
    "deshilé": "deshilar",
    "deshiléis": "deshilar",
    "deshiló": "deshilar",
    "deshinca": "deshincar",
    "deshincaba": "deshincar",
    "deshincabais": "deshincar",
    "deshincaban": "deshincar",
    "deshincabas": "deshincar",
    "deshincad": "deshincar",
    "deshincada": "deshincar",
    "deshincadas": "deshincar",
    "deshincado": "deshincar",
    "deshincados": "deshincar",
    "deshincaduras": "deshincadura",
    "deshincamos": "deshincar",
    "deshincan": "deshincar",
    "deshincando": "deshincar",
    "deshincara": "deshincar",
    "deshincarais": "deshincar",
    "deshincaran": "deshincar",
    "deshincaras": "deshincar",
    "deshincare": "deshincar",
    "deshincareis": "deshincar",
    "deshincaremos": "deshincar",
    "deshincaren": "deshincar",
    "deshincares": "deshincar",
    "deshincaron": "deshincar",
    "deshincará": "deshincar",
    "deshincarán": "deshincar",
    "deshincarás": "deshincar",
    "deshincaré": "deshincar",
    "deshincaréis": "deshincar",
    "deshincaría": "deshincar",
    "deshincaríais": "deshincar",
    "deshincaríamos": "deshincar",
    "deshincarían": "deshincar",
    "deshincarías": "deshincar",
    "deshincas": "deshincar",
    "deshincase": "deshincar",
    "deshincaseis": "deshincar",
    "deshincasen": "deshincar",
    "deshincases": "deshincar",
    "deshincaste": "deshincar",
    "deshincasteis": "deshincar",
    "deshincha": "deshinchar",
    "deshinchaba": "deshinchar",
    "deshinchabais": "deshinchar",
    "deshinchaban": "deshinchar",
    "deshinchabas": "deshinchar",
    "deshinchad": "deshinchar",
    "deshinchada": "deshinchar",
    "deshinchadas": "deshinchar",
    "deshinchado": "deshinchar",
    "deshinchados": "deshinchar",
    "deshinchaduras": "deshinchadura",
    "deshinchamos": "deshinchar",
    "deshinchan": "deshinchar",
    "deshinchando": "deshinchar",
    "deshinchara": "deshinchar",
    "deshincharais": "deshinchar",
    "deshincharan": "deshinchar",
    "deshincharas": "deshinchar",
    "deshinchare": "deshinchar",
    "deshinchareis": "deshinchar",
    "deshincharemos": "deshinchar",
    "deshincharen": "deshinchar",
    "deshinchares": "deshinchar",
    "deshincharon": "deshinchar",
    "deshinchará": "deshinchar",
    "deshincharán": "deshinchar",
    "deshincharás": "deshinchar",
    "deshincharé": "deshinchar",
    "deshincharéis": "deshinchar",
    "deshincharía": "deshinchar",
    "deshincharíais": "deshinchar",
    "deshincharíamos": "deshinchar",
    "deshincharían": "deshinchar",
    "deshincharías": "deshinchar",
    "deshinchas": "deshinchar",
    "deshinchase": "deshinchar",
    "deshinchaseis": "deshinchar",
    "deshinchasen": "deshinchar",
    "deshinchases": "deshinchar",
    "deshinchaste": "deshinchar",
    "deshinchasteis": "deshinchar",
    "deshinche": "deshinchar",
    "deshinchemos": "deshinchar",
    "deshinchen": "deshinchar",
    "deshinches": "deshinchar",
    "deshincho": "deshinchar",
    "deshinchábamos": "deshinchar",
    "deshincháis": "deshinchar",
    "deshincháramos": "deshinchar",
    "deshincháremos": "deshinchar",
    "deshinchásemos": "deshinchar",
    "deshinché": "deshinchar",
    "deshinchéis": "deshinchar",
    "deshinchó": "deshinchar",
    "deshinco": "deshincar",
    "deshincábamos": "deshincar",
    "deshincáis": "deshincar",
    "deshincáramos": "deshincar",
    "deshincáremos": "deshincar",
    "deshincásemos": "deshincar",
    "deshincó": "deshincar",
    "deshinque": "deshincar",
    "deshinquemos": "deshincar",
    "deshinquen": "deshincar",
    "deshinques": "deshincar",
    "deshinqué": "deshincar",
    "deshinquéis": "deshincar",
    "deshipoteca": "deshipotecar",
    "deshipotecaba": "deshipotecar",
    "deshipotecabais": "deshipotecar",
    "deshipotecaban": "deshipotecar",
    "deshipotecabas": "deshipotecar",
    "deshipotecad": "deshipotecar",
    "deshipotecada": "deshipotecar",
    "deshipotecadas": "deshipotecar",
    "deshipotecado": "deshipotecar",
    "deshipotecados": "deshipotecar",
    "deshipotecamientos": "deshipotecamiento",
    "deshipotecamos": "deshipotecar",
    "deshipotecan": "deshipotecar",
    "deshipotecando": "deshipotecar",
    "deshipotecara": "deshipotecar",
    "deshipotecarais": "deshipotecar",
    "deshipotecaran": "deshipotecar",
    "deshipotecaras": "deshipotecar",
    "deshipotecare": "deshipotecar",
    "deshipotecareis": "deshipotecar",
    "deshipotecaremos": "deshipotecar",
    "deshipotecaren": "deshipotecar",
    "deshipotecares": "deshipotecar",
    "deshipotecaron": "deshipotecar",
    "deshipotecará": "deshipotecar",
    "deshipotecarán": "deshipotecar",
    "deshipotecarás": "deshipotecar",
    "deshipotecaré": "deshipotecar",
    "deshipotecaréis": "deshipotecar",
    "deshipotecaría": "deshipotecar",
    "deshipotecaríais": "deshipotecar",
    "deshipotecaríamos": "deshipotecar",
    "deshipotecarían": "deshipotecar",
    "deshipotecarías": "deshipotecar",
    "deshipotecas": "deshipotecar",
    "deshipotecase": "deshipotecar",
    "deshipotecaseis": "deshipotecar",
    "deshipotecasen": "deshipotecar",
    "deshipotecases": "deshipotecar",
    "deshipotecaste": "deshipotecar",
    "deshipotecasteis": "deshipotecar",
    "deshipoteco": "deshipotecar",
    "deshipotecábamos": "deshipotecar",
    "deshipotecáis": "deshipotecar",
    "deshipotecáramos": "deshipotecar",
    "deshipotecáremos": "deshipotecar",
    "deshipotecásemos": "deshipotecar",
    "deshipotecó": "deshipotecar",
    "deshipoteque": "deshipotecar",
    "deshipotequemos": "deshipotecar",
    "deshipotequen": "deshipotecar",
    "deshipoteques": "deshipotecar",
    "deshipotequé": "deshipotecar",
    "deshipotequéis": "deshipotecar",
    "deshizo": "deshacer",
    "deshoja": "deshojar",
    "deshojaba": "deshojar",
    "deshojabais": "deshojar",
    "deshojaban": "deshojar",
    "deshojabas": "deshojar",
    "deshojad": "deshojar",
    "deshojada": "deshojar",
    "deshojadas": "deshojar",
    "deshojado": "deshojar",
    "deshojadora": "deshojador",
    "deshojadoras": "deshojador",
    "deshojadores": "deshojador",
    "deshojados": "deshojar",
    "deshojaduras": "deshojadura",
    "deshojamientos": "deshojamiento",
    "deshojamos": "deshojar",
    "deshojan": "deshojar",
    "deshojando": "deshojar",
    "deshojara": "deshojar",
    "deshojarais": "deshojar",
    "deshojaran": "deshojar",
    "deshojaras": "deshojar",
    "deshojare": "deshojar",
    "deshojareis": "deshojar",
    "deshojaremos": "deshojar",
    "deshojaren": "deshojar",
    "deshojares": "deshojar",
    "deshojaron": "deshojar",
    "deshojará": "deshojar",
    "deshojarán": "deshojar",
    "deshojarás": "deshojar",
    "deshojaré": "deshojar",
    "deshojaréis": "deshojar",
    "deshojaría": "deshojar",
    "deshojaríais": "deshojar",
    "deshojaríamos": "deshojar",
    "deshojarían": "deshojar",
    "deshojarías": "deshojar",
    "deshojas": "deshojar",
    "deshojase": "deshojar",
    "deshojaseis": "deshojar",
    "deshojasen": "deshojar",
    "deshojases": "deshojar",
    "deshojaste": "deshojar",
    "deshojasteis": "deshojar",
    "deshoje": "deshojar",
    "deshojemos": "deshojar",
    "deshojen": "deshojar",
    "deshojes": "deshoje",
    "deshojo": "deshojar",
    "deshojábamos": "deshojar",
    "deshojáis": "deshojar",
    "deshojáramos": "deshojar",
    "deshojáremos": "deshojar",
    "deshojásemos": "deshojar",
    "deshojé": "deshojar",
    "deshojéis": "deshojar",
    "deshojó": "deshojar",
    "desholleja": "deshollejar",
    "deshollejaba": "deshollejar",
    "deshollejabais": "deshollejar",
    "deshollejaban": "deshollejar",
    "deshollejabas": "deshollejar",
    "deshollejad": "deshollejar",
    "deshollejada": "deshollejar",
    "deshollejadas": "deshollejar",
    "deshollejado": "deshollejar",
    "deshollejados": "deshollejar",
    "deshollejamos": "deshollejar",
    "deshollejan": "deshollejar",
    "deshollejando": "deshollejar",
    "deshollejara": "deshollejar",
    "deshollejarais": "deshollejar",
    "deshollejaran": "deshollejar",
    "deshollejaras": "deshollejar",
    "deshollejare": "deshollejar",
    "deshollejareis": "deshollejar",
    "deshollejaremos": "deshollejar",
    "deshollejaren": "deshollejar",
    "deshollejares": "deshollejar",
    "deshollejaron": "deshollejar",
    "deshollejará": "deshollejar",
    "deshollejarán": "deshollejar",
    "deshollejarás": "deshollejar",
    "deshollejaré": "deshollejar",
    "deshollejaréis": "deshollejar",
    "deshollejaría": "deshollejar",
    "deshollejaríais": "deshollejar",
    "deshollejaríamos": "deshollejar",
    "deshollejarían": "deshollejar",
    "deshollejarías": "deshollejar",
    "deshollejas": "deshollejar",
    "deshollejase": "deshollejar",
    "deshollejaseis": "deshollejar",
    "deshollejasen": "deshollejar",
    "deshollejases": "deshollejar",
    "deshollejaste": "deshollejar",
    "deshollejasteis": "deshollejar",
    "desholleje": "deshollejar",
    "deshollejemos": "deshollejar",
    "deshollejen": "deshollejar",
    "deshollejes": "deshollejar",
    "deshollejo": "deshollejar",
    "deshollejábamos": "deshollejar",
    "deshollejáis": "deshollejar",
    "deshollejáramos": "deshollejar",
    "deshollejáremos": "deshollejar",
    "deshollejásemos": "deshollejar",
    "deshollejé": "deshollejar",
    "deshollejéis": "deshollejar",
    "deshollejó": "deshollejar",
    "deshollina": "deshollinar",
    "deshollinaba": "deshollinar",
    "deshollinabais": "deshollinar",
    "deshollinaban": "deshollinar",
    "deshollinabas": "deshollinar",
    "deshollinad": "deshollinar",
    "deshollinada": "deshollinar",
    "deshollinadas": "deshollinar",
    "deshollinaderas": "deshollinadera",
    "deshollinado": "deshollinar",
    "deshollinadora": "deshollinador",
    "deshollinadoras": "deshollinador",
    "deshollinadores": "deshollinador",
    "deshollinados": "deshollinar",
    "deshollinamos": "deshollinar",
    "deshollinan": "deshollinar",
    "deshollinando": "deshollinar",
    "deshollinara": "deshollinar",
    "deshollinarais": "deshollinar",
    "deshollinaran": "deshollinar",
    "deshollinaras": "deshollinar",
    "deshollinare": "deshollinar",
    "deshollinareis": "deshollinar",
    "deshollinaremos": "deshollinar",
    "deshollinaren": "deshollinar",
    "deshollinares": "deshollinar",
    "deshollinaron": "deshollinar",
    "deshollinará": "deshollinar",
    "deshollinarán": "deshollinar",
    "deshollinarás": "deshollinar",
    "deshollinaré": "deshollinar",
    "deshollinaréis": "deshollinar",
    "deshollinaría": "deshollinar",
    "deshollinaríais": "deshollinar",
    "deshollinaríamos": "deshollinar",
    "deshollinarían": "deshollinar",
    "deshollinarías": "deshollinar",
    "deshollinas": "deshollinar",
    "deshollinase": "deshollinar",
    "deshollinaseis": "deshollinar",
    "deshollinasen": "deshollinar",
    "deshollinases": "deshollinar",
    "deshollinaste": "deshollinar",
    "deshollinasteis": "deshollinar",
    "desholline": "deshollinar",
    "deshollinemos": "deshollinar",
    "deshollinen": "deshollinar",
    "deshollines": "deshollinar",
    "deshollino": "deshollinar",
    "deshollinábamos": "deshollinar",
    "deshollináis": "deshollinar",
    "deshollináramos": "deshollinar",
    "deshollináremos": "deshollinar",
    "deshollinásemos": "deshollinar",
    "desholliné": "deshollinar",
    "deshollinéis": "deshollinar",
    "deshollinó": "deshollinar",
    "deshomologa": "deshomologar",
    "deshomologaba": "deshomologar",
    "deshomologabais": "deshomologar",
    "deshomologaban": "deshomologar",
    "deshomologabas": "deshomologar",
    "deshomologad": "deshomologar",
    "deshomologada": "deshomologar",
    "deshomologadas": "deshomologar",
    "deshomologado": "deshomologar",
    "deshomologados": "deshomologar",
    "deshomologamos": "deshomologar",
    "deshomologan": "deshomologar",
    "deshomologando": "deshomologar",
    "deshomologara": "deshomologar",
    "deshomologarais": "deshomologar",
    "deshomologaran": "deshomologar",
    "deshomologaras": "deshomologar",
    "deshomologare": "deshomologar",
    "deshomologareis": "deshomologar",
    "deshomologaremos": "deshomologar",
    "deshomologaren": "deshomologar",
    "deshomologares": "deshomologar",
    "deshomologaron": "deshomologar",
    "deshomologará": "deshomologar",
    "deshomologarán": "deshomologar",
    "deshomologarás": "deshomologar",
    "deshomologaré": "deshomologar",
    "deshomologaréis": "deshomologar",
    "deshomologaría": "deshomologar",
    "deshomologaríais": "deshomologar",
    "deshomologaríamos": "deshomologar",
    "deshomologarían": "deshomologar",
    "deshomologarías": "deshomologar",
    "deshomologas": "deshomologar",
    "deshomologase": "deshomologar",
    "deshomologaseis": "deshomologar",
    "deshomologasen": "deshomologar",
    "deshomologases": "deshomologar",
    "deshomologaste": "deshomologar",
    "deshomologasteis": "deshomologar",
    "deshomologo": "deshomologar",
    "deshomologue": "deshomologar",
    "deshomologuemos": "deshomologar",
    "deshomologuen": "deshomologar",
    "deshomologues": "deshomologar",
    "deshomologué": "deshomologar",
    "deshomologuéis": "deshomologar",
    "deshomologábamos": "deshomologar",
    "deshomologáis": "deshomologar",
    "deshomologáramos": "deshomologar",
    "deshomologáremos": "deshomologar",
    "deshomologásemos": "deshomologar",
    "deshomologó": "deshomologar",
    "deshonesta": "deshonesto",
    "deshonestas": "deshonesto",
    "deshonestidades": "deshonestidad",
    "deshonestos": "deshonesto",
    "deshonora": "deshonorar",
    "deshonoraba": "deshonorar",
    "deshonorabais": "deshonorar",
    "deshonoraban": "deshonorar",
    "deshonorabas": "deshonorar",
    "deshonorad": "deshonorar",
    "deshonorada": "deshonorar",
    "deshonoradas": "deshonorar",
    "deshonorado": "deshonorar",
    "deshonorados": "deshonorar",
    "deshonoramos": "deshonorar",
    "deshonoran": "deshonorar",
    "deshonorando": "deshonorar",
    "deshonorara": "deshonorar",
    "deshonorarais": "deshonorar",
    "deshonoraran": "deshonorar",
    "deshonoraras": "deshonorar",
    "deshonorare": "deshonorar",
    "deshonorareis": "deshonorar",
    "deshonoraremos": "deshonorar",
    "deshonoraren": "deshonorar",
    "deshonorares": "deshonorar",
    "deshonoraron": "deshonorar",
    "deshonorará": "deshonorar",
    "deshonorarán": "deshonorar",
    "deshonorarás": "deshonorar",
    "deshonoraré": "deshonorar",
    "deshonoraréis": "deshonorar",
    "deshonoraría": "deshonorar",
    "deshonoraríais": "deshonorar",
    "deshonoraríamos": "deshonorar",
    "deshonorarían": "deshonorar",
    "deshonorarías": "deshonorar",
    "deshonoras": "deshonorar",
    "deshonorase": "deshonorar",
    "deshonoraseis": "deshonorar",
    "deshonorasen": "deshonorar",
    "deshonorases": "deshonorar",
    "deshonoraste": "deshonorar",
    "deshonorasteis": "deshonorar",
    "deshonore": "deshonorar",
    "deshonoremos": "deshonorar",
    "deshonoren": "deshonorar",
    "deshonores": "deshonorar",
    "deshonoro": "deshonorar",
    "deshonorábamos": "deshonorar",
    "deshonoráis": "deshonorar",
    "deshonoráramos": "deshonorar",
    "deshonoráremos": "deshonorar",
    "deshonorásemos": "deshonorar",
    "deshonoré": "deshonorar",
    "deshonoréis": "deshonorar",
    "deshonoró": "deshonorar",
    "deshonra": "deshonrar",
    "deshonraba": "deshonrar",
    "deshonrabais": "deshonrar",
    "deshonraban": "deshonrar",
    "deshonrabas": "deshonrar",
    "deshonrad": "deshonrar",
    "deshonrada": "deshonrar",
    "deshonradas": "deshonrar",
    "deshonrado": "deshonrar",
    "deshonradora": "deshonrador",
    "deshonradoras": "deshonrador",
    "deshonradores": "deshonrador",
    "deshonrados": "deshonrar",
    "deshonramos": "deshonrar",
    "deshonran": "deshonrar",
    "deshonrando": "deshonrar",
    "deshonrara": "deshonrar",
    "deshonrarais": "deshonrar",
    "deshonraran": "deshonrar",
    "deshonraras": "deshonrar",
    "deshonrare": "deshonrar",
    "deshonrareis": "deshonrar",
    "deshonraremos": "deshonrar",
    "deshonraren": "deshonrar",
    "deshonrares": "deshonrar",
    "deshonraron": "deshonrar",
    "deshonrará": "deshonrar",
    "deshonrarán": "deshonrar",
    "deshonrarás": "deshonrar",
    "deshonraré": "deshonrar",
    "deshonraréis": "deshonrar",
    "deshonraría": "deshonrar",
    "deshonraríais": "deshonrar",
    "deshonraríamos": "deshonrar",
    "deshonrarían": "deshonrar",
    "deshonrarías": "deshonrar",
    "deshonras": "deshonrar",
    "deshonrase": "deshonrar",
    "deshonraseis": "deshonrar",
    "deshonrasen": "deshonrar",
    "deshonrases": "deshonrar",
    "deshonraste": "deshonrar",
    "deshonrasteis": "deshonrar",
    "deshonre": "deshonrar",
    "deshonremos": "deshonrar",
    "deshonren": "deshonrar",
    "deshonres": "deshonrar",
    "deshonribles": "deshonrible",
    "deshonro": "deshonrar",
    "deshonrosa": "deshonroso",
    "deshonrosas": "deshonroso",
    "deshonrosos": "deshonroso",
    "deshonrábamos": "deshonrar",
    "deshonráis": "deshonrar",
    "deshonráramos": "deshonrar",
    "deshonráremos": "deshonrar",
    "deshonrásemos": "deshonrar",
    "deshonré": "deshonrar",
    "deshonréis": "deshonrar",
    "deshonró": "deshonrar",
    "deshoras": "deshora",
    "deshorna": "deshornar",
    "deshornaba": "deshornar",
    "deshornabais": "deshornar",
    "deshornaban": "deshornar",
    "deshornabas": "deshornar",
    "deshornad": "deshornar",
    "deshornada": "deshornar",
    "deshornadas": "deshornar",
    "deshornado": "deshornar",
    "deshornados": "deshornar",
    "deshornamos": "deshornar",
    "deshornan": "deshornar",
    "deshornando": "deshornar",
    "deshornara": "deshornar",
    "deshornarais": "deshornar",
    "deshornaran": "deshornar",
    "deshornaras": "deshornar",
    "deshornare": "deshornar",
    "deshornareis": "deshornar",
    "deshornaremos": "deshornar",
    "deshornaren": "deshornar",
    "deshornares": "deshornar",
    "deshornaron": "deshornar",
    "deshornará": "deshornar",
    "deshornarán": "deshornar",
    "deshornarás": "deshornar",
    "deshornaré": "deshornar",
    "deshornaréis": "deshornar",
    "deshornaría": "deshornar",
    "deshornaríais": "deshornar",
    "deshornaríamos": "deshornar",
    "deshornarían": "deshornar",
    "deshornarías": "deshornar",
    "deshornas": "deshornar",
    "deshornase": "deshornar",
    "deshornaseis": "deshornar",
    "deshornasen": "deshornar",
    "deshornases": "deshornar",
    "deshornaste": "deshornar",
    "deshornasteis": "deshornar",
    "deshorne": "deshornar",
    "deshornemos": "deshornar",
    "deshornen": "deshornar",
    "deshornes": "deshornar",
    "deshorno": "deshornar",
    "deshornábamos": "deshornar",
    "deshornáis": "deshornar",
    "deshornáramos": "deshornar",
    "deshornáremos": "deshornar",
    "deshornásemos": "deshornar",
    "deshorné": "deshornar",
    "deshornéis": "deshornar",
    "deshornó": "deshornar",
    "deshuesa": "desosar",
    "deshuesaba": "deshuesar",
    "deshuesabais": "deshuesar",
    "deshuesaban": "deshuesar",
    "deshuesabas": "deshuesar",
    "deshuesad": "deshuesar",
    "deshuesada": "deshuesar",
    "deshuesadas": "deshuesar",
    "deshuesado": "deshuesar",
    "deshuesadoras": "deshuesadora",
    "deshuesados": "deshuesar",
    "deshuesamos": "deshuesar",
    "deshuesan": "desosar",
    "deshuesando": "deshuesar",
    "deshuesara": "deshuesar",
    "deshuesarais": "deshuesar",
    "deshuesaran": "deshuesar",
    "deshuesaras": "deshuesar",
    "deshuesare": "deshuesar",
    "deshuesareis": "deshuesar",
    "deshuesaremos": "deshuesar",
    "deshuesaren": "deshuesar",
    "deshuesares": "deshuesar",
    "deshuesaron": "deshuesar",
    "deshuesará": "deshuesar",
    "deshuesarán": "deshuesar",
    "deshuesarás": "deshuesar",
    "deshuesaré": "deshuesar",
    "deshuesaréis": "deshuesar",
    "deshuesaría": "deshuesar",
    "deshuesaríais": "deshuesar",
    "deshuesaríamos": "deshuesar",
    "deshuesarían": "deshuesar",
    "deshuesarías": "deshuesar",
    "deshuesas": "desosar",
    "deshuesase": "deshuesar",
    "deshuesaseis": "deshuesar",
    "deshuesasen": "deshuesar",
    "deshuesases": "deshuesar",
    "deshuesaste": "deshuesar",
    "deshuesasteis": "deshuesar",
    "deshuese": "desosar",
    "deshuesemos": "deshuesar",
    "deshuesen": "desosar",
    "deshueses": "desosar",
    "deshueso": "desosar",
    "deshuesábamos": "deshuesar",
    "deshuesáis": "deshuesar",
    "deshuesáramos": "deshuesar",
    "deshuesáremos": "deshuesar",
    "deshuesásemos": "deshuesar",
    "deshuesé": "deshuesar",
    "deshueséis": "deshuesar",
    "deshuesó": "deshuesar",
    "deshumana": "deshumano",
    "deshumanas": "deshumano",
    "deshumanice": "deshumanizar",
    "deshumanicemos": "deshumanizar",
    "deshumanicen": "deshumanizar",
    "deshumanices": "deshumanizar",
    "deshumanicé": "deshumanizar",
    "deshumanicéis": "deshumanizar",
    "deshumaniza": "deshumanizar",
    "deshumanizaba": "deshumanizar",
    "deshumanizabais": "deshumanizar",
    "deshumanizaban": "deshumanizar",
    "deshumanizabas": "deshumanizar",
    "deshumanizaciones": "deshumanización",
    "deshumanizad": "deshumanizar",
    "deshumanizada": "deshumanizar",
    "deshumanizadas": "deshumanizar",
    "deshumanizado": "deshumanizar",
    "deshumanizados": "deshumanizar",
    "deshumanizamos": "deshumanizar",
    "deshumanizan": "deshumanizar",
    "deshumanizando": "deshumanizar",
    "deshumanizantes": "deshumanizante",
    "deshumanizara": "deshumanizar",
    "deshumanizarais": "deshumanizar",
    "deshumanizaran": "deshumanizar",
    "deshumanizaras": "deshumanizar",
    "deshumanizare": "deshumanizar",
    "deshumanizareis": "deshumanizar",
    "deshumanizaremos": "deshumanizar",
    "deshumanizaren": "deshumanizar",
    "deshumanizares": "deshumanizar",
    "deshumanizaron": "deshumanizar",
    "deshumanizará": "deshumanizar",
    "deshumanizarán": "deshumanizar",
    "deshumanizarás": "deshumanizar",
    "deshumanizaré": "deshumanizar",
    "deshumanizaréis": "deshumanizar",
    "deshumanizaría": "deshumanizar",
    "deshumanizaríais": "deshumanizar",
    "deshumanizaríamos": "deshumanizar",
    "deshumanizarían": "deshumanizar",
    "deshumanizarías": "deshumanizar",
    "deshumanizas": "deshumanizar",
    "deshumanizase": "deshumanizar",
    "deshumanizaseis": "deshumanizar",
    "deshumanizasen": "deshumanizar",
    "deshumanizases": "deshumanizar",
    "deshumanizaste": "deshumanizar",
    "deshumanizasteis": "deshumanizar",
    "deshumanizo": "deshumanizar",
    "deshumanizábamos": "deshumanizar",
    "deshumanizáis": "deshumanizar",
    "deshumanizáramos": "deshumanizar",
    "deshumanizáremos": "deshumanizar",
    "deshumanizásemos": "deshumanizar",
    "deshumanizó": "deshumanizar",
    "deshumanos": "deshumano",
    "deshumedece": "deshumedecer",
    "deshumedeced": "deshumedecer",
    "deshumedecemos": "deshumedecer",
    "deshumedecen": "deshumedecer",
    "deshumedeceremos": "deshumedecer",
    "deshumedecerá": "deshumedecer",
    "deshumedecerán": "deshumedecer",
    "deshumedecerás": "deshumedecer",
    "deshumedeceré": "deshumedecer",
    "deshumedeceréis": "deshumedecer",
    "deshumedecería": "deshumedecer",
    "deshumedeceríais": "deshumedecer",
    "deshumedeceríamos": "deshumedecer",
    "deshumedecerían": "deshumedecer",
    "deshumedecerías": "deshumedecer",
    "deshumedeces": "deshumedecer",
    "deshumedecida": "deshumedecer",
    "deshumedecidas": "deshumedecer",
    "deshumedecido": "deshumedecer",
    "deshumedecidos": "deshumedecer",
    "deshumedeciendo": "deshumedecer",
    "deshumedeciera": "deshumedecer",
    "deshumedecierais": "deshumedecer",
    "deshumedecieran": "deshumedecer",
    "deshumedecieras": "deshumedecer",
    "deshumedeciere": "deshumedecer",
    "deshumedeciereis": "deshumedecer",
    "deshumedecieren": "deshumedecer",
    "deshumedecieres": "deshumedecer",
    "deshumedecieron": "deshumedecer",
    "deshumedeciese": "deshumedecer",
    "deshumedecieseis": "deshumedecer",
    "deshumedeciesen": "deshumedecer",
    "deshumedecieses": "deshumedecer",
    "deshumedecimientos": "deshumedecimiento",
    "deshumedecimos": "deshumedecer",
    "deshumedeciste": "deshumedecer",
    "deshumedecisteis": "deshumedecer",
    "deshumedeciéramos": "deshumedecer",
    "deshumedeciéremos": "deshumedecer",
    "deshumedeciésemos": "deshumedecer",
    "deshumedeció": "deshumedecer",
    "deshumedecéis": "deshumedecer",
    "deshumedecí": "deshumedecer",
    "deshumedecía": "deshumedecer",
    "deshumedecíais": "deshumedecer",
    "deshumedecíamos": "deshumedecer",
    "deshumedecían": "deshumedecer",
    "deshumedecías": "deshumedecer",
    "deshumedezca": "deshumedecer",
    "deshumedezcamos": "deshumedecer",
    "deshumedezcan": "deshumedecer",
    "deshumedezcas": "deshumedecer",
    "deshumedezco": "deshumedecer",
    "deshumedezcáis": "deshumedecer",
    "deshumidificadores": "deshumidificador",
    "desiderables": "desiderable",
    "desiderata": "desiderátum",
    "desiderativa": "desiderativo",
    "desiderativas": "desiderativo",
    "desiderativos": "desiderativo",
    "desidias": "desidia",
    "desidiosa": "desidioso",
    "desidiosas": "desidioso",
    "desidiosos": "desidioso",
    "desierta": "desierto",
    "desiertas": "desierto",
    "desiertos": "desierto",
    "designa": "designar",
    "designaba": "designar",
    "designabais": "designar",
    "designaban": "designar",
    "designabas": "designar",
    "designaciones": "designación",
    "designad": "designar",
    "designada": "designar",
    "designadas": "designar",
    "designado": "designar",
    "designadores": "designador",
    "designados": "designar",
    "designamos": "designar",
    "designan": "designar",
    "designando": "designar",
    "designara": "designar",
    "designarais": "designar",
    "designaran": "designar",
    "designaras": "designar",
    "designare": "designar",
    "designareis": "designar",
    "designaremos": "designar",
    "designaren": "designar",
    "designares": "designar",
    "designaron": "designar",
    "designará": "designar",
    "designarán": "designar",
    "designarás": "designar",
    "designaré": "designar",
    "designaréis": "designar",
    "designaría": "designar",
    "designaríais": "designar",
    "designaríamos": "designar",
    "designarían": "designar",
    "designarías": "designar",
    "designas": "designar",
    "designase": "designar",
    "designaseis": "designar",
    "designasen": "designar",
    "designases": "designar",
    "designaste": "designar",
    "designasteis": "designar",
    "designativa": "designativo",
    "designativas": "designativo",
    "designativos": "designativo",
    "designe": "designar",
    "designemos": "designar",
    "designen": "designar",
    "designes": "designar",
    "designios": "designio",
    "designo": "designar",
    "designábamos": "designar",
    "designáis": "designar",
    "designáramos": "designar",
    "designáremos": "designar",
    "designásemos": "designar",
    "designé": "designar",
    "designéis": "designar",
    "designó": "designar",
    "desiguala": "desigualar",
    "desigualaba": "desigualar",
    "desigualabais": "desigualar",
    "desigualaban": "desigualar",
    "desigualabas": "desigualar",
    "desigualad": "desigualar",
    "desigualada": "desigualar",
    "desigualadas": "desigualar",
    "desigualado": "desigualar",
    "desigualados": "desigualar",
    "desigualamos": "desigualar",
    "desigualan": "desigualar",
    "desigualando": "desigualar",
    "desigualara": "desigualar",
    "desigualarais": "desigualar",
    "desigualaran": "desigualar",
    "desigualaras": "desigualar",
    "desigualare": "desigualar",
    "desigualareis": "desigualar",
    "desigualaremos": "desigualar",
    "desigualaren": "desigualar",
    "desigualares": "desigualar",
    "desigualaron": "desigualar",
    "desigualará": "desigualar",
    "desigualarán": "desigualar",
    "desigualarás": "desigualar",
    "desigualaré": "desigualar",
    "desigualaréis": "desigualar",
    "desigualaría": "desigualar",
    "desigualaríais": "desigualar",
    "desigualaríamos": "desigualar",
    "desigualarían": "desigualar",
    "desigualarías": "desigualar",
    "desigualas": "desigualar",
    "desigualase": "desigualar",
    "desigualaseis": "desigualar",
    "desigualasen": "desigualar",
    "desigualases": "desigualar",
    "desigualaste": "desigualar",
    "desigualasteis": "desigualar",
    "desigualdades": "desigualdad",
    "desiguale": "desigualar",
    "desigualemos": "desigualar",
    "desigualen": "desigualar",
    "desiguales": "desigualar",
    "desigualo": "desigualar",
    "desigualábamos": "desigualar",
    "desigualáis": "desigualar",
    "desigualáramos": "desigualar",
    "desigualáremos": "desigualar",
    "desigualásemos": "desigualar",
    "desigualé": "desigualar",
    "desigualéis": "desigualar",
    "desigualó": "desigualar",
    "desilusiona": "desilusionar",
    "desilusionaba": "desilusionar",
    "desilusionabais": "desilusionar",
    "desilusionaban": "desilusionar",
    "desilusionabas": "desilusionar",
    "desilusionad": "desilusionar",
    "desilusionada": "desilusionar",
    "desilusionadas": "desilusionar",
    "desilusionado": "desilusionar",
    "desilusionados": "desilusionar",
    "desilusionamos": "desilusionar",
    "desilusionan": "desilusionar",
    "desilusionando": "desilusionar",
    "desilusionara": "desilusionar",
    "desilusionarais": "desilusionar",
    "desilusionaran": "desilusionar",
    "desilusionaras": "desilusionar",
    "desilusionare": "desilusionar",
    "desilusionareis": "desilusionar",
    "desilusionaremos": "desilusionar",
    "desilusionaren": "desilusionar",
    "desilusionares": "desilusionar",
    "desilusionaron": "desilusionar",
    "desilusionará": "desilusionar",
    "desilusionarán": "desilusionar",
    "desilusionarás": "desilusionar",
    "desilusionaré": "desilusionar",
    "desilusionaréis": "desilusionar",
    "desilusionaría": "desilusionar",
    "desilusionaríais": "desilusionar",
    "desilusionaríamos": "desilusionar",
    "desilusionarían": "desilusionar",
    "desilusionarías": "desilusionar",
    "desilusionas": "desilusionar",
    "desilusionase": "desilusionar",
    "desilusionaseis": "desilusionar",
    "desilusionasen": "desilusionar",
    "desilusionases": "desilusionar",
    "desilusionaste": "desilusionar",
    "desilusionasteis": "desilusionar",
    "desilusione": "desilusionar",
    "desilusionemos": "desilusionar",
    "desilusionen": "desilusionar",
    "desilusiones": "desilusionar",
    "desilusiono": "desilusionar",
    "desilusionábamos": "desilusionar",
    "desilusionáis": "desilusionar",
    "desilusionáramos": "desilusionar",
    "desilusionáremos": "desilusionar",
    "desilusionásemos": "desilusionar",
    "desilusioné": "desilusionar",
    "desilusionéis": "desilusionar",
    "desilusionó": "desilusionar",
    "desimana": "desimanar",
    "desimanaba": "desimanar",
    "desimanabais": "desimanar",
    "desimanaban": "desimanar",
    "desimanabas": "desimanar",
    "desimanaciones": "desimanación",
    "desimanad": "desimanar",
    "desimanada": "desimanar",
    "desimanadas": "desimanar",
    "desimanado": "desimanar",
    "desimanadora": "desimanador",
    "desimanadoras": "desimanador",
    "desimanadores": "desimanador",
    "desimanados": "desimanar",
    "desimanamos": "desimanar",
    "desimanan": "desimanar",
    "desimanando": "desimanar",
    "desimanara": "desimanar",
    "desimanarais": "desimanar",
    "desimanaran": "desimanar",
    "desimanaras": "desimanar",
    "desimanare": "desimanar",
    "desimanareis": "desimanar",
    "desimanaremos": "desimanar",
    "desimanaren": "desimanar",
    "desimanares": "desimanar",
    "desimanaron": "desimanar",
    "desimanará": "desimanar",
    "desimanarán": "desimanar",
    "desimanarás": "desimanar",
    "desimanaré": "desimanar",
    "desimanaréis": "desimanar",
    "desimanaría": "desimanar",
    "desimanaríais": "desimanar",
    "desimanaríamos": "desimanar",
    "desimanarían": "desimanar",
    "desimanarías": "desimanar",
    "desimanas": "desimanar",
    "desimanase": "desimanar",
    "desimanaseis": "desimanar",
    "desimanasen": "desimanar",
    "desimanases": "desimanar",
    "desimanaste": "desimanar",
    "desimanasteis": "desimanar",
    "desimane": "desimanar",
    "desimanemos": "desimanar",
    "desimanen": "desimanar",
    "desimanes": "desimanar",
    "desimano": "desimanar",
    "desimanta": "desimantar",
    "desimantaba": "desimantar",
    "desimantabais": "desimantar",
    "desimantaban": "desimantar",
    "desimantabas": "desimantar",
    "desimantaciones": "desimantación",
    "desimantad": "desimantar",
    "desimantada": "desimantar",
    "desimantadas": "desimantar",
    "desimantado": "desimantar",
    "desimantados": "desimantar",
    "desimantamos": "desimantar",
    "desimantan": "desimantar",
    "desimantando": "desimantar",
    "desimantantes": "desimantante",
    "desimantara": "desimantar",
    "desimantarais": "desimantar",
    "desimantaran": "desimantar",
    "desimantaras": "desimantar",
    "desimantare": "desimantar",
    "desimantareis": "desimantar",
    "desimantaremos": "desimantar",
    "desimantaren": "desimantar",
    "desimantares": "desimantar",
    "desimantaron": "desimantar",
    "desimantará": "desimantar",
    "desimantarán": "desimantar",
    "desimantarás": "desimantar",
    "desimantaré": "desimantar",
    "desimantaréis": "desimantar",
    "desimantaría": "desimantar",
    "desimantaríais": "desimantar",
    "desimantaríamos": "desimantar",
    "desimantarían": "desimantar",
    "desimantarías": "desimantar",
    "desimantas": "desimantar",
    "desimantase": "desimantar",
    "desimantaseis": "desimantar",
    "desimantasen": "desimantar",
    "desimantases": "desimantar",
    "desimantaste": "desimantar",
    "desimantasteis": "desimantar",
    "desimante": "desimantar",
    "desimantemos": "desimantar",
    "desimanten": "desimantar",
    "desimantes": "desimantar",
    "desimanto": "desimantar",
    "desimantábamos": "desimantar",
    "desimantáis": "desimantar",
    "desimantáramos": "desimantar",
    "desimantáremos": "desimantar",
    "desimantásemos": "desimantar",
    "desimanté": "desimantar",
    "desimantéis": "desimantar",
    "desimantó": "desimantar",
    "desimanábamos": "desimanar",
    "desimanáis": "desimanar",
    "desimanáramos": "desimanar",
    "desimanáremos": "desimanar",
    "desimanásemos": "desimanar",
    "desimané": "desimanar",
    "desimanéis": "desimanar",
    "desimanó": "desimanar",
    "desimpregnaciones": "desimpregnación",
    "desimpresiona": "desimpresionar",
    "desimpresionaba": "desimpresionar",
    "desimpresionabais": "desimpresionar",
    "desimpresionaban": "desimpresionar",
    "desimpresionabas": "desimpresionar",
    "desimpresionad": "desimpresionar",
    "desimpresionada": "desimpresionar",
    "desimpresionadas": "desimpresionar",
    "desimpresionado": "desimpresionar",
    "desimpresionados": "desimpresionar",
    "desimpresionamos": "desimpresionar",
    "desimpresionan": "desimpresionar",
    "desimpresionando": "desimpresionar",
    "desimpresionara": "desimpresionar",
    "desimpresionarais": "desimpresionar",
    "desimpresionaran": "desimpresionar",
    "desimpresionaras": "desimpresionar",
    "desimpresionare": "desimpresionar",
    "desimpresionareis": "desimpresionar",
    "desimpresionaremos": "desimpresionar",
    "desimpresionaren": "desimpresionar",
    "desimpresionares": "desimpresionar",
    "desimpresionaron": "desimpresionar",
    "desimpresionará": "desimpresionar",
    "desimpresionarán": "desimpresionar",
    "desimpresionarás": "desimpresionar",
    "desimpresionaré": "desimpresionar",
    "desimpresionaréis": "desimpresionar",
    "desimpresionaría": "desimpresionar",
    "desimpresionaríais": "desimpresionar",
    "desimpresionaríamos": "desimpresionar",
    "desimpresionarían": "desimpresionar",
    "desimpresionarías": "desimpresionar",
    "desimpresionas": "desimpresionar",
    "desimpresionase": "desimpresionar",
    "desimpresionaseis": "desimpresionar",
    "desimpresionasen": "desimpresionar",
    "desimpresionases": "desimpresionar",
    "desimpresionaste": "desimpresionar",
    "desimpresionasteis": "desimpresionar",
    "desimpresione": "desimpresionar",
    "desimpresionemos": "desimpresionar",
    "desimpresionen": "desimpresionar",
    "desimpresiones": "desimpresionar",
    "desimpresiono": "desimpresionar",
    "desimpresionábamos": "desimpresionar",
    "desimpresionáis": "desimpresionar",
    "desimpresionáramos": "desimpresionar",
    "desimpresionáremos": "desimpresionar",
    "desimpresionásemos": "desimpresionar",
    "desimpresioné": "desimpresionar",
    "desimpresionéis": "desimpresionar",
    "desimpresionó": "desimpresionar",
    "desincorporaciones": "desincorporación",
    "desincronizada": "desincronizado",
    "desincronizadas": "desincronizado",
    "desincronizados": "desincronizado",
    "desincrusta": "desincrustar",
    "desincrustaba": "desincrustar",
    "desincrustabais": "desincrustar",
    "desincrustaban": "desincrustar",
    "desincrustabas": "desincrustar",
    "desincrustabilidades": "desincrustabilidad",
    "desincrustaciones": "desincrustación",
    "desincrustad": "desincrustar",
    "desincrustada": "desincrustar",
    "desincrustadas": "desincrustar",
    "desincrustado": "desincrustar",
    "desincrustados": "desincrustar",
    "desincrustamos": "desincrustar",
    "desincrustan": "desincrustar",
    "desincrustando": "desincrustar",
    "desincrustantes": "desincrustante",
    "desincrustara": "desincrustar",
    "desincrustarais": "desincrustar",
    "desincrustaran": "desincrustar",
    "desincrustaras": "desincrustar",
    "desincrustare": "desincrustar",
    "desincrustareis": "desincrustar",
    "desincrustaremos": "desincrustar",
    "desincrustaren": "desincrustar",
    "desincrustares": "desincrustar",
    "desincrustaron": "desincrustar",
    "desincrustará": "desincrustar",
    "desincrustarán": "desincrustar",
    "desincrustarás": "desincrustar",
    "desincrustaré": "desincrustar",
    "desincrustaréis": "desincrustar",
    "desincrustaría": "desincrustar",
    "desincrustaríais": "desincrustar",
    "desincrustaríamos": "desincrustar",
    "desincrustarían": "desincrustar",
    "desincrustarías": "desincrustar",
    "desincrustas": "desincrustar",
    "desincrustase": "desincrustar",
    "desincrustaseis": "desincrustar",
    "desincrustasen": "desincrustar",
    "desincrustases": "desincrustar",
    "desincrustaste": "desincrustar",
    "desincrustasteis": "desincrustar",
    "desincruste": "desincrustar",
    "desincrustemos": "desincrustar",
    "desincrusten": "desincrustar",
    "desincrustes": "desincrustar",
    "desincrusto": "desincrustar",
    "desincrustábamos": "desincrustar",
    "desincrustáis": "desincrustar",
    "desincrustáramos": "desincrustar",
    "desincrustáremos": "desincrustar",
    "desincrustásemos": "desincrustar",
    "desincrusté": "desincrustar",
    "desincrustéis": "desincrustar",
    "desincrustó": "desincrustar",
    "desindustrializaciones": "desindustrialización",
    "desinenciales": "desinencial",
    "desinencias": "desinencia",
    "desinfartaciones": "desinfartación",
    "desinfecciones": "desinfección",
    "desinfecta": "desinfectar",
    "desinfectaba": "desinfectar",
    "desinfectabais": "desinfectar",
    "desinfectaban": "desinfectar",
    "desinfectabas": "desinfectar",
    "desinfectad": "desinfectar",
    "desinfectada": "desinfectar",
    "desinfectadas": "desinfectar",
    "desinfectado": "desinfectar",
    "desinfectadoras": "desinfectadora",
    "desinfectados": "desinfectar",
    "desinfectamos": "desinfectar",
    "desinfectan": "desinfectar",
    "desinfectando": "desinfectar",
    "desinfectantes": "desinfectante",
    "desinfectara": "desinfectar",
    "desinfectarais": "desinfectar",
    "desinfectaran": "desinfectar",
    "desinfectaras": "desinfectar",
    "desinfectare": "desinfectar",
    "desinfectareis": "desinfectar",
    "desinfectaremos": "desinfectar",
    "desinfectaren": "desinfectar",
    "desinfectares": "desinfectar",
    "desinfectaron": "desinfectar",
    "desinfectará": "desinfectar",
    "desinfectarán": "desinfectar",
    "desinfectarás": "desinfectar",
    "desinfectaré": "desinfectar",
    "desinfectaréis": "desinfectar",
    "desinfectaría": "desinfectar",
    "desinfectaríais": "desinfectar",
    "desinfectaríamos": "desinfectar",
    "desinfectarían": "desinfectar",
    "desinfectarías": "desinfectar",
    "desinfectas": "desinfectar",
    "desinfectase": "desinfectar",
    "desinfectaseis": "desinfectar",
    "desinfectasen": "desinfectar",
    "desinfectases": "desinfectar",
    "desinfectaste": "desinfectar",
    "desinfectasteis": "desinfectar",
    "desinfecte": "desinfectar",
    "desinfectemos": "desinfectar",
    "desinfecten": "desinfectar",
    "desinfectes": "desinfectar",
    "desinfecto": "desinfectar",
    "desinfectábamos": "desinfectar",
    "desinfectáis": "desinfectar",
    "desinfectáramos": "desinfectar",
    "desinfectáremos": "desinfectar",
    "desinfectásemos": "desinfectar",
    "desinfecté": "desinfectar",
    "desinfectéis": "desinfectar",
    "desinfectó": "desinfectar",
    "desinfestaciones": "desinfestación",
    "desinfla": "desinflar",
    "desinflaba": "desinflar",
    "desinflabais": "desinflar",
    "desinflaban": "desinflar",
    "desinflabas": "desinflar",
    "desinflacionaria": "desinflacionario",
    "desinflacionarias": "desinflacionario",
    "desinflacionarios": "desinflacionario",
    "desinflaciones": "desinflación",
    "desinflacionistas": "desinflacionista",
    "desinflad": "desinflar",
    "desinflada": "desinflar",
    "desinfladas": "desinflar",
    "desinflado": "desinflar",
    "desinflados": "desinflar",
    "desinflama": "desinflamar",
    "desinflamaba": "desinflamar",
    "desinflamabais": "desinflamar",
    "desinflamaban": "desinflamar",
    "desinflamabas": "desinflamar",
    "desinflamaciones": "desinflamación",
    "desinflamad": "desinflamar",
    "desinflamada": "desinflamar",
    "desinflamadas": "desinflamar",
    "desinflamado": "desinflamar",
    "desinflamados": "desinflamar",
    "desinflamamos": "desinflamar",
    "desinflaman": "desinflamar",
    "desinflamando": "desinflamar",
    "desinflamara": "desinflamar",
    "desinflamarais": "desinflamar",
    "desinflamaran": "desinflamar",
    "desinflamaras": "desinflamar",
    "desinflamare": "desinflamar",
    "desinflamareis": "desinflamar",
    "desinflamaremos": "desinflamar",
    "desinflamaren": "desinflamar",
    "desinflamares": "desinflamar",
    "desinflamaron": "desinflamar",
    "desinflamará": "desinflamar",
    "desinflamarán": "desinflamar",
    "desinflamarás": "desinflamar",
    "desinflamaré": "desinflamar",
    "desinflamaréis": "desinflamar",
    "desinflamaría": "desinflamar",
    "desinflamaríais": "desinflamar",
    "desinflamaríamos": "desinflamar",
    "desinflamarían": "desinflamar",
    "desinflamarías": "desinflamar",
    "desinflamas": "desinflamar",
    "desinflamase": "desinflamar",
    "desinflamaseis": "desinflamar",
    "desinflamasen": "desinflamar",
    "desinflamases": "desinflamar",
    "desinflamaste": "desinflamar",
    "desinflamasteis": "desinflamar",
    "desinflame": "desinflamar",
    "desinflamemos": "desinflamar",
    "desinflamen": "desinflamar",
    "desinflames": "desinflamar",
    "desinflamo": "desinflamar",
    "desinflamos": "desinflar",
    "desinflamábamos": "desinflamar",
    "desinflamáis": "desinflamar",
    "desinflamáramos": "desinflamar",
    "desinflamáremos": "desinflamar",
    "desinflamásemos": "desinflamar",
    "desinflamé": "desinflamar",
    "desinflaméis": "desinflamar",
    "desinflamó": "desinflamar",
    "desinflan": "desinflar",
    "desinflando": "desinflar",
    "desinflara": "desinflar",
    "desinflarais": "desinflar",
    "desinflaran": "desinflar",
    "desinflaras": "desinflar",
    "desinflare": "desinflar",
    "desinflareis": "desinflar",
    "desinflaremos": "desinflar",
    "desinflaren": "desinflar",
    "desinflares": "desinflar",
    "desinflaron": "desinflar",
    "desinflará": "desinflar",
    "desinflarán": "desinflar",
    "desinflarás": "desinflar",
    "desinflaré": "desinflar",
    "desinflaréis": "desinflar",
    "desinflaría": "desinflar",
    "desinflaríais": "desinflar",
    "desinflaríamos": "desinflar",
    "desinflarían": "desinflar",
    "desinflarías": "desinflar",
    "desinflas": "desinflar",
    "desinflase": "desinflar",
    "desinflaseis": "desinflar",
    "desinflasen": "desinflar",
    "desinflases": "desinflar",
    "desinflaste": "desinflar",
    "desinflasteis": "desinflar",
    "desinfle": "desinflar",
    "desinflemos": "desinflar",
    "desinflen": "desinflar",
    "desinfles": "desinflar",
    "desinflo": "desinflar",
    "desinflábamos": "desinflar",
    "desinfláis": "desinflar",
    "desinfláramos": "desinflar",
    "desinfláremos": "desinflar",
    "desinflásemos": "desinflar",
    "desinflé": "desinflar",
    "desinfléis": "desinflar",
    "desinfló": "desinflar",
    "desinformaciones": "desinformación",
    "desinhibiciones": "desinhibición",
    "desinhibida": "desinhibido",
    "desinhibidas": "desinhibido",
    "desinhibidos": "desinhibido",
    "desinsacula": "desinsacular",
    "desinsaculaba": "desinsacular",
    "desinsaculabais": "desinsacular",
    "desinsaculaban": "desinsacular",
    "desinsaculabas": "desinsacular",
    "desinsaculaciones": "desinsaculación",
    "desinsaculad": "desinsacular",
    "desinsaculada": "desinsacular",
    "desinsaculadas": "desinsacular",
    "desinsaculado": "desinsacular",
    "desinsaculados": "desinsacular",
    "desinsaculamos": "desinsacular",
    "desinsaculan": "desinsacular",
    "desinsaculando": "desinsacular",
    "desinsaculara": "desinsacular",
    "desinsacularais": "desinsacular",
    "desinsacularan": "desinsacular",
    "desinsacularas": "desinsacular",
    "desinsaculare": "desinsacular",
    "desinsaculareis": "desinsacular",
    "desinsacularemos": "desinsacular",
    "desinsacularen": "desinsacular",
    "desinsaculares": "desinsacular",
    "desinsacularon": "desinsacular",
    "desinsaculará": "desinsacular",
    "desinsacularán": "desinsacular",
    "desinsacularás": "desinsacular",
    "desinsacularé": "desinsacular",
    "desinsacularéis": "desinsacular",
    "desinsacularía": "desinsacular",
    "desinsacularíais": "desinsacular",
    "desinsacularíamos": "desinsacular",
    "desinsacularían": "desinsacular",
    "desinsacularías": "desinsacular",
    "desinsaculas": "desinsacular",
    "desinsaculase": "desinsacular",
    "desinsaculaseis": "desinsacular",
    "desinsaculasen": "desinsacular",
    "desinsaculases": "desinsacular",
    "desinsaculaste": "desinsacular",
    "desinsaculasteis": "desinsacular",
    "desinsacule": "desinsacular",
    "desinsaculemos": "desinsacular",
    "desinsaculen": "desinsacular",
    "desinsacules": "desinsacular",
    "desinsaculo": "desinsacular",
    "desinsaculábamos": "desinsacular",
    "desinsaculáis": "desinsacular",
    "desinsaculáramos": "desinsacular",
    "desinsaculáremos": "desinsacular",
    "desinsaculásemos": "desinsacular",
    "desinsaculé": "desinsacular",
    "desinsaculéis": "desinsacular",
    "desinsaculó": "desinsacular",
    "desinsecta": "desinsectar",
    "desinsectaba": "desinsectar",
    "desinsectabais": "desinsectar",
    "desinsectaban": "desinsectar",
    "desinsectabas": "desinsectar",
    "desinsectaciones": "desinsectación",
    "desinsectad": "desinsectar",
    "desinsectada": "desinsectar",
    "desinsectadas": "desinsectar",
    "desinsectado": "desinsectar",
    "desinsectados": "desinsectar",
    "desinsectamos": "desinsectar",
    "desinsectan": "desinsectar",
    "desinsectando": "desinsectar",
    "desinsectara": "desinsectar",
    "desinsectarais": "desinsectar",
    "desinsectaran": "desinsectar",
    "desinsectaras": "desinsectar",
    "desinsectare": "desinsectar",
    "desinsectareis": "desinsectar",
    "desinsectaremos": "desinsectar",
    "desinsectaren": "desinsectar",
    "desinsectares": "desinsectar",
    "desinsectaron": "desinsectar",
    "desinsectará": "desinsectar",
    "desinsectarán": "desinsectar",
    "desinsectarás": "desinsectar",
    "desinsectaré": "desinsectar",
    "desinsectaréis": "desinsectar",
    "desinsectaría": "desinsectar",
    "desinsectaríais": "desinsectar",
    "desinsectaríamos": "desinsectar",
    "desinsectarían": "desinsectar",
    "desinsectarías": "desinsectar",
    "desinsectas": "desinsectar",
    "desinsectase": "desinsectar",
    "desinsectaseis": "desinsectar",
    "desinsectasen": "desinsectar",
    "desinsectases": "desinsectar",
    "desinsectaste": "desinsectar",
    "desinsectasteis": "desinsectar",
    "desinsecte": "desinsectar",
    "desinsectemos": "desinsectar",
    "desinsecten": "desinsectar",
    "desinsectes": "desinsectar",
    "desinsectizaciones": "desinsectización",
    "desinsecto": "desinsectar",
    "desinsectábamos": "desinsectar",
    "desinsectáis": "desinsectar",
    "desinsectáramos": "desinsectar",
    "desinsectáremos": "desinsectar",
    "desinsectásemos": "desinsectar",
    "desinsecté": "desinsectar",
    "desinsectéis": "desinsectar",
    "desinsectó": "desinsectar",
    "desinstitucionalizaciones": "desinstitucionalización",
    "desintegra": "desintegrar",
    "desintegraba": "desintegrar",
    "desintegrabais": "desintegrar",
    "desintegraban": "desintegrar",
    "desintegrabas": "desintegrar",
    "desintegraciones": "desintegración",
    "desintegrad": "desintegrar",
    "desintegrada": "desintegrar",
    "desintegradas": "desintegrar",
    "desintegrado": "desintegrar",
    "desintegradora": "desintegrador",
    "desintegradoras": "desintegrador",
    "desintegradores": "desintegrador",
    "desintegrados": "desintegrar",
    "desintegramos": "desintegrar",
    "desintegran": "desintegrar",
    "desintegrando": "desintegrar",
    "desintegrara": "desintegrar",
    "desintegrarais": "desintegrar",
    "desintegraran": "desintegrar",
    "desintegraras": "desintegrar",
    "desintegrare": "desintegrar",
    "desintegrareis": "desintegrar",
    "desintegraremos": "desintegrar",
    "desintegraren": "desintegrar",
    "desintegrares": "desintegrar",
    "desintegraron": "desintegrar",
    "desintegrará": "desintegrar",
    "desintegrarán": "desintegrar",
    "desintegrarás": "desintegrar",
    "desintegraré": "desintegrar",
    "desintegraréis": "desintegrar",
    "desintegraría": "desintegrar",
    "desintegraríais": "desintegrar",
    "desintegraríamos": "desintegrar",
    "desintegrarían": "desintegrar",
    "desintegrarías": "desintegrar",
    "desintegras": "desintegrar",
    "desintegrase": "desintegrar",
    "desintegraseis": "desintegrar",
    "desintegrasen": "desintegrar",
    "desintegrases": "desintegrar",
    "desintegraste": "desintegrar",
    "desintegrasteis": "desintegrar",
    "desintegre": "desintegrar",
    "desintegremos": "desintegrar",
    "desintegren": "desintegrar",
    "desintegres": "desintegrar",
    "desintegro": "desintegrar",
    "desintegrábamos": "desintegrar",
    "desintegráis": "desintegrar",
    "desintegráramos": "desintegrar",
    "desintegráremos": "desintegrar",
    "desintegrásemos": "desintegrar",
    "desintegré": "desintegrar",
    "desintegréis": "desintegrar",
    "desintegró": "desintegrar",
    "desinteresa": "desinteresar",
    "desinteresaba": "desinteresar",
    "desinteresabais": "desinteresar",
    "desinteresaban": "desinteresar",
    "desinteresabas": "desinteresar",
    "desinteresad": "desinteresar",
    "desinteresada": "desinteresar",
    "desinteresadas": "desinteresar",
    "desinteresado": "desinteresar",
    "desinteresados": "desinteresar",
    "desinteresamos": "desinteresar",
    "desinteresan": "desinteresar",
    "desinteresando": "desinteresar",
    "desinteresara": "desinteresar",
    "desinteresarais": "desinteresar",
    "desinteresaran": "desinteresar",
    "desinteresaras": "desinteresar",
    "desinteresare": "desinteresar",
    "desinteresareis": "desinteresar",
    "desinteresaremos": "desinteresar",
    "desinteresaren": "desinteresar",
    "desinteresares": "desinteresar",
    "desinteresaron": "desinteresar",
    "desinteresará": "desinteresar",
    "desinteresarán": "desinteresar",
    "desinteresarás": "desinteresar",
    "desinteresaré": "desinteresar",
    "desinteresaréis": "desinteresar",
    "desinteresaría": "desinteresar",
    "desinteresaríais": "desinteresar",
    "desinteresaríamos": "desinteresar",
    "desinteresarían": "desinteresar",
    "desinteresarías": "desinteresar",
    "desinteresas": "desinteresar",
    "desinteresase": "desinteresar",
    "desinteresaseis": "desinteresar",
    "desinteresasen": "desinteresar",
    "desinteresases": "desinteresar",
    "desinteresaste": "desinteresar",
    "desinteresasteis": "desinteresar",
    "desinterese": "desinteresar",
    "desinteresemos": "desinteresar",
    "desinteresen": "desinteresar",
    "desintereses": "desinteresar",
    "desintereso": "desinteresar",
    "desinteresábamos": "desinteresar",
    "desinteresáis": "desinteresar",
    "desinteresáramos": "desinteresar",
    "desinteresáremos": "desinteresar",
    "desinteresásemos": "desinteresar",
    "desinteresé": "desinteresar",
    "desintereséis": "desinteresar",
    "desinteresó": "desinteresar",
    "desintermediaciones": "desintermediación",
    "desintoxica": "desintoxicar",
    "desintoxicaba": "desintoxicar",
    "desintoxicabais": "desintoxicar",
    "desintoxicaban": "desintoxicar",
    "desintoxicabas": "desintoxicar",
    "desintoxicaciones": "desintoxicación",
    "desintoxicad": "desintoxicar",
    "desintoxicada": "desintoxicar",
    "desintoxicadas": "desintoxicar",
    "desintoxicado": "desintoxicar",
    "desintoxicados": "desintoxicar",
    "desintoxicamos": "desintoxicar",
    "desintoxican": "desintoxicar",
    "desintoxicando": "desintoxicar",
    "desintoxicara": "desintoxicar",
    "desintoxicarais": "desintoxicar",
    "desintoxicaran": "desintoxicar",
    "desintoxicaras": "desintoxicar",
    "desintoxicare": "desintoxicar",
    "desintoxicareis": "desintoxicar",
    "desintoxicaremos": "desintoxicar",
    "desintoxicaren": "desintoxicar",
    "desintoxicares": "desintoxicar",
    "desintoxicaron": "desintoxicar",
    "desintoxicará": "desintoxicar",
    "desintoxicarán": "desintoxicar",
    "desintoxicarás": "desintoxicar",
    "desintoxicaré": "desintoxicar",
    "desintoxicaréis": "desintoxicar",
    "desintoxicaría": "desintoxicar",
    "desintoxicaríais": "desintoxicar",
    "desintoxicaríamos": "desintoxicar",
    "desintoxicarían": "desintoxicar",
    "desintoxicarías": "desintoxicar",
    "desintoxicas": "desintoxicar",
    "desintoxicase": "desintoxicar",
    "desintoxicaseis": "desintoxicar",
    "desintoxicasen": "desintoxicar",
    "desintoxicases": "desintoxicar",
    "desintoxicaste": "desintoxicar",
    "desintoxicasteis": "desintoxicar",
    "desintoxico": "desintoxicar",
    "desintoxicábamos": "desintoxicar",
    "desintoxicáis": "desintoxicar",
    "desintoxicáramos": "desintoxicar",
    "desintoxicáremos": "desintoxicar",
    "desintoxicásemos": "desintoxicar",
    "desintoxicó": "desintoxicar",
    "desintoxique": "desintoxicar",
    "desintoxiquemos": "desintoxicar",
    "desintoxiquen": "desintoxicar",
    "desintoxiques": "desintoxicar",
    "desintoxiqué": "desintoxicar",
    "desintoxiquéis": "desintoxicar",
    "desinversiones": "desinversión",
    "desionizaciones": "desionización",
    "desista": "desistir",
    "desistamos": "desistir",
    "desistan": "desistir",
    "desistas": "desistir",
    "desiste": "desistir",
    "desisten": "desistir",
    "desistes": "desistir",
    "desistid": "desistir",
    "desistida": "desistir",
    "desistidas": "desistir",
    "desistido": "desistir",
    "desistidos": "desistir",
    "desistiendo": "desistir",
    "desistiera": "desistir",
    "desistierais": "desistir",
    "desistieran": "desistir",
    "desistieras": "desistir",
    "desistiere": "desistir",
    "desistiereis": "desistir",
    "desistieren": "desistir",
    "desistieres": "desistir",
    "desistieron": "desistir",
    "desistiese": "desistir",
    "desistieseis": "desistir",
    "desistiesen": "desistir",
    "desistieses": "desistir",
    "desistimientos": "desistimiento",
    "desistimos": "desistir",
    "desistiremos": "desistir",
    "desistirá": "desistir",
    "desistirán": "desistir",
    "desistirás": "desistir",
    "desistiré": "desistir",
    "desistiréis": "desistir",
    "desistiría": "desistir",
    "desistiríais": "desistir",
    "desistiríamos": "desistir",
    "desistirían": "desistir",
    "desistirías": "desistir",
    "desististe": "desistir",
    "desististeis": "desistir",
    "desistiéramos": "desistir",
    "desistiéremos": "desistir",
    "desistiésemos": "desistir",
    "desistió": "desistir",
    "desisto": "desistir",
    "desistáis": "desistir",
    "desistí": "desistir",
    "desistía": "desistir",
    "desistíais": "desistir",
    "desistíamos": "desistir",
    "desistían": "desistir",
    "desistías": "desistir",
    "desistís": "desistir",
    "desjarreta": "desjarretar",
    "desjarretaba": "desjarretar",
    "desjarretabais": "desjarretar",
    "desjarretaban": "desjarretar",
    "desjarretabas": "desjarretar",
    "desjarretad": "desjarretar",
    "desjarretada": "desjarretar",
    "desjarretadas": "desjarretar",
    "desjarretaderas": "desjarretadera",
    "desjarretado": "desjarretar",
    "desjarretados": "desjarretar",
    "desjarretamos": "desjarretar",
    "desjarretan": "desjarretar",
    "desjarretando": "desjarretar",
    "desjarretara": "desjarretar",
    "desjarretarais": "desjarretar",
    "desjarretaran": "desjarretar",
    "desjarretaras": "desjarretar",
    "desjarretare": "desjarretar",
    "desjarretareis": "desjarretar",
    "desjarretaremos": "desjarretar",
    "desjarretaren": "desjarretar",
    "desjarretares": "desjarretar",
    "desjarretaron": "desjarretar",
    "desjarretará": "desjarretar",
    "desjarretarán": "desjarretar",
    "desjarretarás": "desjarretar",
    "desjarretaré": "desjarretar",
    "desjarretaréis": "desjarretar",
    "desjarretaría": "desjarretar",
    "desjarretaríais": "desjarretar",
    "desjarretaríamos": "desjarretar",
    "desjarretarían": "desjarretar",
    "desjarretarías": "desjarretar",
    "desjarretas": "desjarretar",
    "desjarretase": "desjarretar",
    "desjarretaseis": "desjarretar",
    "desjarretasen": "desjarretar",
    "desjarretases": "desjarretar",
    "desjarretaste": "desjarretar",
    "desjarretasteis": "desjarretar",
    "desjarrete": "desjarretar",
    "desjarretemos": "desjarretar",
    "desjarreten": "desjarretar",
    "desjarretes": "desjarrete",
    "desjarreto": "desjarretar",
    "desjarretábamos": "desjarretar",
    "desjarretáis": "desjarretar",
    "desjarretáramos": "desjarretar",
    "desjarretáremos": "desjarretar",
    "desjarretásemos": "desjarretar",
    "desjarreté": "desjarretar",
    "desjarretéis": "desjarretar",
    "desjarretó": "desjarretar",
    "desjudicializaciones": "desjudicialización",
    "desjuga": "desjugar",
    "desjugaba": "desjugar",
    "desjugabais": "desjugar",
    "desjugaban": "desjugar",
    "desjugabas": "desjugar",
    "desjugad": "desjugar",
    "desjugada": "desjugar",
    "desjugadas": "desjugar",
    "desjugado": "desjugar",
    "desjugados": "desjugar",
    "desjugamos": "desjugar",
    "desjugan": "desjugar",
    "desjugando": "desjugar",
    "desjugara": "desjugar",
    "desjugarais": "desjugar",
    "desjugaran": "desjugar",
    "desjugaras": "desjugar",
    "desjugare": "desjugar",
    "desjugareis": "desjugar",
    "desjugaremos": "desjugar",
    "desjugaren": "desjugar",
    "desjugares": "desjugar",
    "desjugaron": "desjugar",
    "desjugará": "desjugar",
    "desjugarán": "desjugar",
    "desjugarás": "desjugar",
    "desjugaré": "desjugar",
    "desjugaréis": "desjugar",
    "desjugaría": "desjugar",
    "desjugaríais": "desjugar",
    "desjugaríamos": "desjugar",
    "desjugarían": "desjugar",
    "desjugarías": "desjugar",
    "desjugas": "desjugar",
    "desjugase": "desjugar",
    "desjugaseis": "desjugar",
    "desjugasen": "desjugar",
    "desjugases": "desjugar",
    "desjugaste": "desjugar",
    "desjugasteis": "desjugar",
    "desjugo": "desjugar",
    "desjugue": "desjugar",
    "desjuguemos": "desjugar",
    "desjuguen": "desjugar",
    "desjugues": "desjugar",
    "desjugué": "desjugar",
    "desjuguéis": "desjugar",
    "desjugábamos": "desjugar",
    "desjugáis": "desjugar",
    "desjugáramos": "desjugar",
    "desjugáremos": "desjugar",
    "desjugásemos": "desjugar",
    "desjugó": "desjugar",
    "desjuiciada": "desjuiciado",
    "desjuiciadas": "desjuiciado",
    "desjuiciados": "desjuiciado",
    "desjunta": "desjuntar",
    "desjuntaba": "desjuntar",
    "desjuntabais": "desjuntar",
    "desjuntaban": "desjuntar",
    "desjuntabas": "desjuntar",
    "desjuntad": "desjuntar",
    "desjuntada": "desjuntar",
    "desjuntadas": "desjuntar",
    "desjuntado": "desjuntar",
    "desjuntados": "desjuntar",
    "desjuntamientos": "desjuntamiento",
    "desjuntamos": "desjuntar",
    "desjuntan": "desjuntar",
    "desjuntando": "desjuntar",
    "desjuntara": "desjuntar",
    "desjuntarais": "desjuntar",
    "desjuntaran": "desjuntar",
    "desjuntaras": "desjuntar",
    "desjuntare": "desjuntar",
    "desjuntareis": "desjuntar",
    "desjuntaremos": "desjuntar",
    "desjuntaren": "desjuntar",
    "desjuntares": "desjuntar",
    "desjuntaron": "desjuntar",
    "desjuntará": "desjuntar",
    "desjuntarán": "desjuntar",
    "desjuntarás": "desjuntar",
    "desjuntaré": "desjuntar",
    "desjuntaréis": "desjuntar",
    "desjuntaría": "desjuntar",
    "desjuntaríais": "desjuntar",
    "desjuntaríamos": "desjuntar",
    "desjuntarían": "desjuntar",
    "desjuntarías": "desjuntar",
    "desjuntas": "desjuntar",
    "desjuntase": "desjuntar",
    "desjuntaseis": "desjuntar",
    "desjuntasen": "desjuntar",
    "desjuntases": "desjuntar",
    "desjuntaste": "desjuntar",
    "desjuntasteis": "desjuntar",
    "desjunte": "desjuntar",
    "desjuntemos": "desjuntar",
    "desjunten": "desjuntar",
    "desjuntes": "desjuntar",
    "desjunto": "desjuntar",
    "desjuntábamos": "desjuntar",
    "desjuntáis": "desjuntar",
    "desjuntáramos": "desjuntar",
    "desjuntáremos": "desjuntar",
    "desjuntásemos": "desjuntar",
    "desjunté": "desjuntar",
    "desjuntéis": "desjuntar",
    "desjuntó": "desjuntar",
    "deslabazada": "deslabazado",
    "deslabazadas": "deslabazado",
    "deslabazados": "deslabazado",
    "deslabializaciones": "deslabialización",
    "deslabona": "deslabonar",
    "deslabonaba": "deslabonar",
    "deslabonabais": "deslabonar",
    "deslabonaban": "deslabonar",
    "deslabonabas": "deslabonar",
    "deslabonad": "deslabonar",
    "deslabonada": "deslabonar",
    "deslabonadas": "deslabonar",
    "deslabonado": "deslabonar",
    "deslabonados": "deslabonar",
    "deslabonamos": "deslabonar",
    "deslabonan": "deslabonar",
    "deslabonando": "deslabonar",
    "deslabonara": "deslabonar",
    "deslabonarais": "deslabonar",
    "deslabonaran": "deslabonar",
    "deslabonaras": "deslabonar",
    "deslabonare": "deslabonar",
    "deslabonareis": "deslabonar",
    "deslabonaremos": "deslabonar",
    "deslabonaren": "deslabonar",
    "deslabonares": "deslabonar",
    "deslabonaron": "deslabonar",
    "deslabonará": "deslabonar",
    "deslabonarán": "deslabonar",
    "deslabonarás": "deslabonar",
    "deslabonaré": "deslabonar",
    "deslabonaréis": "deslabonar",
    "deslabonaría": "deslabonar",
    "deslabonaríais": "deslabonar",
    "deslabonaríamos": "deslabonar",
    "deslabonarían": "deslabonar",
    "deslabonarías": "deslabonar",
    "deslabonas": "deslabonar",
    "deslabonase": "deslabonar",
    "deslabonaseis": "deslabonar",
    "deslabonasen": "deslabonar",
    "deslabonases": "deslabonar",
    "deslabonaste": "deslabonar",
    "deslabonasteis": "deslabonar",
    "deslabone": "deslabonar",
    "deslabonemos": "deslabonar",
    "deslabonen": "deslabonar",
    "deslabones": "deslabonar",
    "deslabono": "deslabonar",
    "deslabonábamos": "deslabonar",
    "deslabonáis": "deslabonar",
    "deslabonáramos": "deslabonar",
    "deslabonáremos": "deslabonar",
    "deslabonásemos": "deslabonar",
    "deslaboné": "deslabonar",
    "deslabonéis": "deslabonar",
    "deslabonó": "deslabonar",
    "deslace": "deslazar",
    "deslacemos": "deslazar",
    "deslacen": "deslazar",
    "deslaces": "deslazar",
    "deslacé": "deslazar",
    "deslacéis": "deslazar",
    "desladrilla": "desladrillar",
    "desladrillaba": "desladrillar",
    "desladrillabais": "desladrillar",
    "desladrillaban": "desladrillar",
    "desladrillabas": "desladrillar",
    "desladrillad": "desladrillar",
    "desladrillada": "desladrillar",
    "desladrilladas": "desladrillar",
    "desladrillado": "desladrillar",
    "desladrillados": "desladrillar",
    "desladrillamos": "desladrillar",
    "desladrillan": "desladrillar",
    "desladrillando": "desladrillar",
    "desladrillara": "desladrillar",
    "desladrillarais": "desladrillar",
    "desladrillaran": "desladrillar",
    "desladrillaras": "desladrillar",
    "desladrillare": "desladrillar",
    "desladrillareis": "desladrillar",
    "desladrillaremos": "desladrillar",
    "desladrillaren": "desladrillar",
    "desladrillares": "desladrillar",
    "desladrillaron": "desladrillar",
    "desladrillará": "desladrillar",
    "desladrillarán": "desladrillar",
    "desladrillarás": "desladrillar",
    "desladrillaré": "desladrillar",
    "desladrillaréis": "desladrillar",
    "desladrillaría": "desladrillar",
    "desladrillaríais": "desladrillar",
    "desladrillaríamos": "desladrillar",
    "desladrillarían": "desladrillar",
    "desladrillarías": "desladrillar",
    "desladrillas": "desladrillar",
    "desladrillase": "desladrillar",
    "desladrillaseis": "desladrillar",
    "desladrillasen": "desladrillar",
    "desladrillases": "desladrillar",
    "desladrillaste": "desladrillar",
    "desladrillasteis": "desladrillar",
    "desladrille": "desladrillar",
    "desladrillemos": "desladrillar",
    "desladrillen": "desladrillar",
    "desladrilles": "desladrillar",
    "desladrillo": "desladrillar",
    "desladrillábamos": "desladrillar",
    "desladrilláis": "desladrillar",
    "desladrilláramos": "desladrillar",
    "desladrilláremos": "desladrillar",
    "desladrillásemos": "desladrillar",
    "desladrillé": "desladrillar",
    "desladrilléis": "desladrillar",
    "desladrilló": "desladrillar",
    "deslamina": "deslaminar",
    "deslaminaba": "deslaminar",
    "deslaminabais": "deslaminar",
    "deslaminaban": "deslaminar",
    "deslaminabas": "deslaminar",
    "deslaminad": "deslaminar",
    "deslaminada": "deslaminar",
    "deslaminadas": "deslaminar",
    "deslaminado": "deslaminar",
    "deslaminados": "deslaminar",
    "deslaminamos": "deslaminar",
    "deslaminan": "deslaminar",
    "deslaminando": "deslaminar",
    "deslaminara": "deslaminar",
    "deslaminarais": "deslaminar",
    "deslaminaran": "deslaminar",
    "deslaminaras": "deslaminar",
    "deslaminare": "deslaminar",
    "deslaminareis": "deslaminar",
    "deslaminaremos": "deslaminar",
    "deslaminaren": "deslaminar",
    "deslaminares": "deslaminar",
    "deslaminaron": "deslaminar",
    "deslaminará": "deslaminar",
    "deslaminarán": "deslaminar",
    "deslaminarás": "deslaminar",
    "deslaminaré": "deslaminar",
    "deslaminaréis": "deslaminar",
    "deslaminaría": "deslaminar",
    "deslaminaríais": "deslaminar",
    "deslaminaríamos": "deslaminar",
    "deslaminarían": "deslaminar",
    "deslaminarías": "deslaminar",
    "deslaminas": "deslaminar",
    "deslaminase": "deslaminar",
    "deslaminaseis": "deslaminar",
    "deslaminasen": "deslaminar",
    "deslaminases": "deslaminar",
    "deslaminaste": "deslaminar",
    "deslaminasteis": "deslaminar",
    "deslamine": "deslaminar",
    "deslaminemos": "deslaminar",
    "deslaminen": "deslaminar",
    "deslamines": "deslaminar",
    "deslamino": "deslaminar",
    "deslaminábamos": "deslaminar",
    "deslamináis": "deslaminar",
    "deslamináramos": "deslaminar",
    "deslamináremos": "deslaminar",
    "deslaminásemos": "deslaminar",
    "deslaminé": "deslaminar",
    "deslaminéis": "deslaminar",
    "deslaminó": "deslaminar",
    "deslastra": "deslastrar",
    "deslastraba": "deslastrar",
    "deslastrabais": "deslastrar",
    "deslastraban": "deslastrar",
    "deslastrabas": "deslastrar",
    "deslastrad": "deslastrar",
    "deslastrada": "deslastrar",
    "deslastradas": "deslastrar",
    "deslastrado": "deslastrar",
    "deslastrados": "deslastrar",
    "deslastramos": "deslastrar",
    "deslastran": "deslastrar",
    "deslastrando": "deslastrar",
    "deslastrara": "deslastrar",
    "deslastrarais": "deslastrar",
    "deslastraran": "deslastrar",
    "deslastraras": "deslastrar",
    "deslastrare": "deslastrar",
    "deslastrareis": "deslastrar",
    "deslastraremos": "deslastrar",
    "deslastraren": "deslastrar",
    "deslastrares": "deslastrar",
    "deslastraron": "deslastrar",
    "deslastrará": "deslastrar",
    "deslastrarán": "deslastrar",
    "deslastrarás": "deslastrar",
    "deslastraré": "deslastrar",
    "deslastraréis": "deslastrar",
    "deslastraría": "deslastrar",
    "deslastraríais": "deslastrar",
    "deslastraríamos": "deslastrar",
    "deslastrarían": "deslastrar",
    "deslastrarías": "deslastrar",
    "deslastras": "deslastrar",
    "deslastrase": "deslastrar",
    "deslastraseis": "deslastrar",
    "deslastrasen": "deslastrar",
    "deslastrases": "deslastrar",
    "deslastraste": "deslastrar",
    "deslastrasteis": "deslastrar",
    "deslastre": "deslastrar",
    "deslastremos": "deslastrar",
    "deslastren": "deslastrar",
    "deslastres": "deslastrar",
    "deslastro": "deslastrar",
    "deslastrábamos": "deslastrar",
    "deslastráis": "deslastrar",
    "deslastráramos": "deslastrar",
    "deslastráremos": "deslastrar",
    "deslastrásemos": "deslastrar",
    "deslastré": "deslastrar",
    "deslastréis": "deslastrar",
    "deslastró": "deslastrar",
    "deslava": "deslavar",
    "deslavaba": "deslavar",
    "deslavabais": "deslavar",
    "deslavaban": "deslavar",
    "deslavabas": "deslavar",
    "deslavace": "deslavazar",
    "deslavacemos": "deslavazar",
    "deslavacen": "deslavazar",
    "deslavaces": "deslavazar",
    "deslavacé": "deslavazar",
    "deslavacéis": "deslavazar",
    "deslavad": "deslavar",
    "deslavada": "deslavar",
    "deslavadas": "deslavar",
    "deslavado": "deslavar",
    "deslavados": "deslavar",
    "deslavaduras": "deslavadura",
    "deslavamos": "deslavar",
    "deslavan": "deslavar",
    "deslavando": "deslavar",
    "deslavara": "deslavar",
    "deslavarais": "deslavar",
    "deslavaran": "deslavar",
    "deslavaras": "deslavar",
    "deslavare": "deslavar",
    "deslavareis": "deslavar",
    "deslavaremos": "deslavar",
    "deslavaren": "deslavar",
    "deslavares": "deslavar",
    "deslavaron": "deslavar",
    "deslavará": "deslavar",
    "deslavarán": "deslavar",
    "deslavarás": "deslavar",
    "deslavaré": "deslavar",
    "deslavaréis": "deslavar",
    "deslavaría": "deslavar",
    "deslavaríais": "deslavar",
    "deslavaríamos": "deslavar",
    "deslavarían": "deslavar",
    "deslavarías": "deslavar",
    "deslavas": "deslavar",
    "deslavase": "deslavar",
    "deslavaseis": "deslavar",
    "deslavasen": "deslavar",
    "deslavases": "deslavar",
    "deslavaste": "deslavar",
    "deslavasteis": "deslavar",
    "deslavaza": "deslavazar",
    "deslavazaba": "deslavazar",
    "deslavazabais": "deslavazar",
    "deslavazaban": "deslavazar",
    "deslavazabas": "deslavazar",
    "deslavazad": "deslavazar",
    "deslavazada": "deslavazar",
    "deslavazadas": "deslavazar",
    "deslavazado": "deslavazar",
    "deslavazados": "deslavazar",
    "deslavazamientos": "deslavazamiento",
    "deslavazamos": "deslavazar",
    "deslavazan": "deslavazar",
    "deslavazando": "deslavazar",
    "deslavazara": "deslavazar",
    "deslavazarais": "deslavazar",
    "deslavazaran": "deslavazar",
    "deslavazaras": "deslavazar",
    "deslavazare": "deslavazar",
    "deslavazareis": "deslavazar",
    "deslavazaremos": "deslavazar",
    "deslavazaren": "deslavazar",
    "deslavazares": "deslavazar",
    "deslavazaron": "deslavazar",
    "deslavazará": "deslavazar",
    "deslavazarán": "deslavazar",
    "deslavazarás": "deslavazar",
    "deslavazaré": "deslavazar",
    "deslavazaréis": "deslavazar",
    "deslavazaría": "deslavazar",
    "deslavazaríais": "deslavazar",
    "deslavazaríamos": "deslavazar",
    "deslavazarían": "deslavazar",
    "deslavazarías": "deslavazar",
    "deslavazas": "deslavazar",
    "deslavazase": "deslavazar",
    "deslavazaseis": "deslavazar",
    "deslavazasen": "deslavazar",
    "deslavazases": "deslavazar",
    "deslavazaste": "deslavazar",
    "deslavazasteis": "deslavazar",
    "deslavazo": "deslavazar",
    "deslavazábamos": "deslavazar",
    "deslavazáis": "deslavazar",
    "deslavazáramos": "deslavazar",
    "deslavazáremos": "deslavazar",
    "deslavazásemos": "deslavazar",
    "deslavazó": "deslavazar",
    "deslave": "deslavar",
    "deslavemos": "deslavar",
    "deslaven": "deslavar",
    "deslaves": "deslave",
    "deslavo": "deslavar",
    "deslavábamos": "deslavar",
    "deslaváis": "deslavar",
    "deslaváramos": "deslavar",
    "deslaváremos": "deslavar",
    "deslavásemos": "deslavar",
    "deslavé": "deslavar",
    "deslavéis": "deslavar",
    "deslavó": "deslavar",
    "deslaza": "deslazar",
    "deslazaba": "deslazar",
    "deslazabais": "deslazar",
    "deslazaban": "deslazar",
    "deslazabas": "deslazar",
    "deslazad": "deslazar",
    "deslazada": "deslazar",
    "deslazadas": "deslazar",
    "deslazado": "deslazar",
    "deslazados": "deslazar",
    "deslazamientos": "deslazamiento",
    "deslazamos": "deslazar",
    "deslazan": "deslazar",
    "deslazando": "deslazar",
    "deslazara": "deslazar",
    "deslazarais": "deslazar",
    "deslazaran": "deslazar",
    "deslazaras": "deslazar",
    "deslazare": "deslazar",
    "deslazareis": "deslazar",
    "deslazaremos": "deslazar",
    "deslazaren": "deslazar",
    "deslazares": "deslazar",
    "deslazaron": "deslazar",
    "deslazará": "deslazar",
    "deslazarán": "deslazar",
    "deslazarás": "deslazar",
    "deslazaré": "deslazar",
    "deslazaréis": "deslazar",
    "deslazaría": "deslazar",
    "deslazaríais": "deslazar",
    "deslazaríamos": "deslazar",
    "deslazarían": "deslazar",
    "deslazarías": "deslazar",
    "deslazas": "deslazar",
    "deslazase": "deslazar",
    "deslazaseis": "deslazar",
    "deslazasen": "deslazar",
    "deslazases": "deslazar",
    "deslazaste": "deslazar",
    "deslazasteis": "deslazar",
    "deslazo": "deslazar",
    "deslazábamos": "deslazar",
    "deslazáis": "deslazar",
    "deslazáramos": "deslazar",
    "deslazáremos": "deslazar",
    "deslazásemos": "deslazar",
    "deslazó": "deslazar",
    "desleales": "desleal",
    "deslealtades": "deslealtad",
    "deslechuga": "deslechugar",
    "deslechugaba": "deslechugar",
    "deslechugabais": "deslechugar",
    "deslechugaban": "deslechugar",
    "deslechugabas": "deslechugar",
    "deslechugad": "deslechugar",
    "deslechugada": "deslechugar",
    "deslechugadas": "deslechugar",
    "deslechugado": "deslechugar",
    "deslechugadora": "deslechugador",
    "deslechugadoras": "deslechugador",
    "deslechugadores": "deslechugador",
    "deslechugados": "deslechugar",
    "deslechugamos": "deslechugar",
    "deslechugan": "deslechugar",
    "deslechugando": "deslechugar",
    "deslechugara": "deslechugar",
    "deslechugarais": "deslechugar",
    "deslechugaran": "deslechugar",
    "deslechugaras": "deslechugar",
    "deslechugare": "deslechugar",
    "deslechugareis": "deslechugar",
    "deslechugaremos": "deslechugar",
    "deslechugaren": "deslechugar",
    "deslechugares": "deslechugar",
    "deslechugaron": "deslechugar",
    "deslechugará": "deslechugar",
    "deslechugarán": "deslechugar",
    "deslechugarás": "deslechugar",
    "deslechugaré": "deslechugar",
    "deslechugaréis": "deslechugar",
    "deslechugaría": "deslechugar",
    "deslechugaríais": "deslechugar",
    "deslechugaríamos": "deslechugar",
    "deslechugarían": "deslechugar",
    "deslechugarías": "deslechugar",
    "deslechugas": "deslechugar",
    "deslechugase": "deslechugar",
    "deslechugaseis": "deslechugar",
    "deslechugasen": "deslechugar",
    "deslechugases": "deslechugar",
    "deslechugaste": "deslechugar",
    "deslechugasteis": "deslechugar",
    "deslechugo": "deslechugar",
    "deslechugue": "deslechugar",
    "deslechuguemos": "deslechugar",
    "deslechuguen": "deslechugar",
    "deslechugues": "deslechugar",
    "deslechuguilla": "deslechuguillar",
    "deslechuguillaba": "deslechuguillar",
    "deslechuguillabais": "deslechuguillar",
    "deslechuguillaban": "deslechuguillar",
    "deslechuguillabas": "deslechuguillar",
    "deslechuguillad": "deslechuguillar",
    "deslechuguillada": "deslechuguillar",
    "deslechuguilladas": "deslechuguillar",
    "deslechuguillado": "deslechuguillar",
    "deslechuguillados": "deslechuguillar",
    "deslechuguillamos": "deslechuguillar",
    "deslechuguillan": "deslechuguillar",
    "deslechuguillando": "deslechuguillar",
    "deslechuguillara": "deslechuguillar",
    "deslechuguillarais": "deslechuguillar",
    "deslechuguillaran": "deslechuguillar",
    "deslechuguillaras": "deslechuguillar",
    "deslechuguillare": "deslechuguillar",
    "deslechuguillareis": "deslechuguillar",
    "deslechuguillaremos": "deslechuguillar",
    "deslechuguillaren": "deslechuguillar",
    "deslechuguillares": "deslechuguillar",
    "deslechuguillaron": "deslechuguillar",
    "deslechuguillará": "deslechuguillar",
    "deslechuguillarán": "deslechuguillar",
    "deslechuguillarás": "deslechuguillar",
    "deslechuguillaré": "deslechuguillar",
    "deslechuguillaréis": "deslechuguillar",
    "deslechuguillaría": "deslechuguillar",
    "deslechuguillaríais": "deslechuguillar",
    "deslechuguillaríamos": "deslechuguillar",
    "deslechuguillarían": "deslechuguillar",
    "deslechuguillarías": "deslechuguillar",
    "deslechuguillas": "deslechuguillar",
    "deslechuguillase": "deslechuguillar",
    "deslechuguillaseis": "deslechuguillar",
    "deslechuguillasen": "deslechuguillar",
    "deslechuguillases": "deslechuguillar",
    "deslechuguillaste": "deslechuguillar",
    "deslechuguillasteis": "deslechuguillar",
    "deslechuguille": "deslechuguillar",
    "deslechuguillemos": "deslechuguillar",
    "deslechuguillen": "deslechuguillar",
    "deslechuguilles": "deslechuguillar",
    "deslechuguillo": "deslechuguillar",
    "deslechuguillábamos": "deslechuguillar",
    "deslechuguilláis": "deslechuguillar",
    "deslechuguilláramos": "deslechuguillar",
    "deslechuguilláremos": "deslechuguillar",
    "deslechuguillásemos": "deslechuguillar",
    "deslechuguillé": "deslechuguillar",
    "deslechuguilléis": "deslechuguillar",
    "deslechuguilló": "deslechuguillar",
    "deslechugué": "deslechugar",
    "deslechuguéis": "deslechugar",
    "deslechugábamos": "deslechugar",
    "deslechugáis": "deslechugar",
    "deslechugáramos": "deslechugar",
    "deslechugáremos": "deslechugar",
    "deslechugásemos": "deslechugar",
    "deslechugó": "deslechugar",
    "deslegitima": "deslegitimar",
    "deslegitimaba": "deslegitimar",
    "deslegitimabais": "deslegitimar",
    "deslegitimaban": "deslegitimar",
    "deslegitimabas": "deslegitimar",
    "deslegitimad": "deslegitimar",
    "deslegitimada": "deslegitimar",
    "deslegitimadas": "deslegitimar",
    "deslegitimado": "deslegitimar",
    "deslegitimados": "deslegitimar",
    "deslegitimamos": "deslegitimar",
    "deslegitiman": "deslegitimar",
    "deslegitimando": "deslegitimar",
    "deslegitimara": "deslegitimar",
    "deslegitimarais": "deslegitimar",
    "deslegitimaran": "deslegitimar",
    "deslegitimaras": "deslegitimar",
    "deslegitimare": "deslegitimar",
    "deslegitimareis": "deslegitimar",
    "deslegitimaremos": "deslegitimar",
    "deslegitimaren": "deslegitimar",
    "deslegitimares": "deslegitimar",
    "deslegitimaron": "deslegitimar",
    "deslegitimará": "deslegitimar",
    "deslegitimarán": "deslegitimar",
    "deslegitimarás": "deslegitimar",
    "deslegitimaré": "deslegitimar",
    "deslegitimaréis": "deslegitimar",
    "deslegitimaría": "deslegitimar",
    "deslegitimaríais": "deslegitimar",
    "deslegitimaríamos": "deslegitimar",
    "deslegitimarían": "deslegitimar",
    "deslegitimarías": "deslegitimar",
    "deslegitimas": "deslegitimar",
    "deslegitimase": "deslegitimar",
    "deslegitimaseis": "deslegitimar",
    "deslegitimasen": "deslegitimar",
    "deslegitimases": "deslegitimar",
    "deslegitimaste": "deslegitimar",
    "deslegitimasteis": "deslegitimar",
    "deslegitime": "deslegitimar",
    "deslegitimemos": "deslegitimar",
    "deslegitimen": "deslegitimar",
    "deslegitimes": "deslegitimar",
    "deslegitimo": "deslegitimar",
    "deslegitimábamos": "deslegitimar",
    "deslegitimáis": "deslegitimar",
    "deslegitimáramos": "deslegitimar",
    "deslegitimáremos": "deslegitimar",
    "deslegitimásemos": "deslegitimar",
    "deslegitimé": "deslegitimar",
    "deslegitiméis": "deslegitimar",
    "deslegitimó": "deslegitimar",
    "desleiduras": "desleidura",
    "desleimientos": "desleimiento",
    "desleiremos": "desleír",
    "desleirá": "desleír",
    "desleirán": "desleír",
    "desleirás": "desleír",
    "desleiré": "desleír",
    "desleiréis": "desleír",
    "desleiría": "desleír",
    "desleiríais": "desleír",
    "desleiríamos": "desleír",
    "desleirían": "desleír",
    "desleirías": "desleír",
    "deslendraba": "deslendrar",
    "deslendrabais": "deslendrar",
    "deslendraban": "deslendrar",
    "deslendrabas": "deslendrar",
    "deslendrad": "deslendrar",
    "deslendrada": "deslendrar",
    "deslendradas": "deslendrar",
    "deslendrado": "deslendrar",
    "deslendrados": "deslendrar",
    "deslendramos": "deslendrar",
    "deslendrando": "deslendrar",
    "deslendrara": "deslendrar",
    "deslendrarais": "deslendrar",
    "deslendraran": "deslendrar",
    "deslendraras": "deslendrar",
    "deslendrare": "deslendrar",
    "deslendrareis": "deslendrar",
    "deslendraremos": "deslendrar",
    "deslendraren": "deslendrar",
    "deslendrares": "deslendrar",
    "deslendraron": "deslendrar",
    "deslendrará": "deslendrar",
    "deslendrarán": "deslendrar",
    "deslendrarás": "deslendrar",
    "deslendraré": "deslendrar",
    "deslendraréis": "deslendrar",
    "deslendraría": "deslendrar",
    "deslendraríais": "deslendrar",
    "deslendraríamos": "deslendrar",
    "deslendrarían": "deslendrar",
    "deslendrarías": "deslendrar",
    "deslendrase": "deslendrar",
    "deslendraseis": "deslendrar",
    "deslendrasen": "deslendrar",
    "deslendrases": "deslendrar",
    "deslendraste": "deslendrar",
    "deslendrasteis": "deslendrar",
    "deslendremos": "deslendrar",
    "deslendrábamos": "deslendrar",
    "deslendráis": "deslendrar",
    "deslendráramos": "deslendrar",
    "deslendráremos": "deslendrar",
    "deslendrásemos": "deslendrar",
    "deslendré": "deslendrar",
    "deslendréis": "deslendrar",
    "deslendró": "deslendrar",
    "deslengua": "deslenguar",
    "deslenguaba": "deslenguar",
    "deslenguabais": "deslenguar",
    "deslenguaban": "deslenguar",
    "deslenguabas": "deslenguar",
    "deslenguad": "deslenguar",
    "deslenguada": "deslenguar",
    "deslenguadas": "deslenguar",
    "deslenguado": "deslenguar",
    "deslenguados": "deslenguar",
    "deslenguamientos": "deslenguamiento",
    "deslenguamos": "deslenguar",
    "deslenguan": "deslenguar",
    "deslenguando": "deslenguar",
    "deslenguara": "deslenguar",
    "deslenguarais": "deslenguar",
    "deslenguaran": "deslenguar",
    "deslenguaras": "deslenguar",
    "deslenguare": "deslenguar",
    "deslenguareis": "deslenguar",
    "deslenguaremos": "deslenguar",
    "deslenguaren": "deslenguar",
    "deslenguares": "deslenguar",
    "deslenguaron": "deslenguar",
    "deslenguará": "deslenguar",
    "deslenguarán": "deslenguar",
    "deslenguarás": "deslenguar",
    "deslenguaré": "deslenguar",
    "deslenguaréis": "deslenguar",
    "deslenguaría": "deslenguar",
    "deslenguaríais": "deslenguar",
    "deslenguaríamos": "deslenguar",
    "deslenguarían": "deslenguar",
    "deslenguarías": "deslenguar",
    "deslenguas": "deslenguar",
    "deslenguase": "deslenguar",
    "deslenguaseis": "deslenguar",
    "deslenguasen": "deslenguar",
    "deslenguases": "deslenguar",
    "deslenguaste": "deslenguar",
    "deslenguasteis": "deslenguar",
    "deslenguo": "deslenguar",
    "deslenguábamos": "deslenguar",
    "deslenguáis": "deslenguar",
    "deslenguáramos": "deslenguar",
    "deslenguáremos": "deslenguar",
    "deslenguásemos": "deslenguar",
    "deslenguó": "deslenguar",
    "deslengüe": "deslenguar",
    "deslengüemos": "deslenguar",
    "deslengüen": "deslenguar",
    "deslengües": "deslenguar",
    "deslengüé": "deslenguar",
    "deslengüéis": "deslenguar",
    "desleí": "desleír",
    "desleía": "desleír",
    "desleíais": "desleír",
    "desleíamos": "desleír",
    "desleían": "desleír",
    "desleías": "desleír",
    "desleíd": "desleír",
    "desleída": "desleír",
    "desleídas": "desleír",
    "desleído": "desleír",
    "desleídos": "desleír",
    "desleímos": "desleír",
    "desleís": "desleír",
    "desleíste": "desleír",
    "desleísteis": "desleír",
    "desliaba": "desliar",
    "desliabais": "desliar",
    "desliaban": "desliar",
    "desliabas": "desliar",
    "desliad": "desliar",
    "desliada": "desliar",
    "desliadas": "desliar",
    "desliado": "desliar",
    "desliados": "desliar",
    "desliamos": "desliar",
    "desliando": "desliar",
    "desliara": "desliar",
    "desliarais": "desliar",
    "desliaran": "desliar",
    "desliaras": "desliar",
    "desliare": "desliar",
    "desliareis": "desliar",
    "desliaremos": "desliar",
    "desliaren": "desliar",
    "desliares": "desliar",
    "desliaron": "desliar",
    "desliará": "desliar",
    "desliarán": "desliar",
    "desliarás": "desliar",
    "desliaré": "desliar",
    "desliaréis": "desliar",
    "desliaría": "desliar",
    "desliaríais": "desliar",
    "desliaríamos": "desliar",
    "desliarían": "desliar",
    "desliarías": "desliar",
    "desliase": "desliar",
    "desliaseis": "desliar",
    "desliasen": "desliar",
    "desliases": "desliar",
    "desliaste": "desliar",
    "desliasteis": "desliar",
    "deslice": "deslizar",
    "deslicemos": "deslizar",
    "deslicen": "deslizar",
    "deslices": "deslizar",
    "deslicé": "deslizar",
    "deslicéis": "deslizar",
    "desliemos": "desliar",
    "desliendo": "desleír",
    "desliendra": "deslendrar",
    "desliendran": "deslendrar",
    "desliendras": "deslendrar",
    "desliendre": "deslendrar",
    "desliendren": "deslendrar",
    "desliendres": "deslendrar",
    "desliendro": "deslendrar",
    "desliera": "desleír",
    "deslierais": "desleír",
    "deslieran": "desleír",
    "deslieras": "desleír",
    "desliere": "desleír",
    "desliereis": "desleír",
    "deslieren": "desleír",
    "deslieres": "desleír",
    "deslieron": "desleír",
    "desliese": "desleír",
    "deslieseis": "desleír",
    "desliesen": "desleír",
    "deslieses": "desleír",
    "desliga": "desligar",
    "desligaba": "desligar",
    "desligabais": "desligar",
    "desligaban": "desligar",
    "desligabas": "desligar",
    "desligad": "desligar",
    "desligada": "desligar",
    "desligadas": "desligar",
    "desligado": "desligar",
    "desligados": "desligar",
    "desligaduras": "desligadura",
    "desligamientos": "desligamiento",
    "desligamos": "desligar",
    "desligan": "desligar",
    "desligando": "desligar",
    "desligara": "desligar",
    "desligarais": "desligar",
    "desligaran": "desligar",
    "desligaras": "desligar",
    "desligare": "desligar",
    "desligareis": "desligar",
    "desligaremos": "desligar",
    "desligaren": "desligar",
    "desligares": "desligar",
    "desligaron": "desligar",
    "desligará": "desligar",
    "desligarán": "desligar",
    "desligarás": "desligar",
    "desligaré": "desligar",
    "desligaréis": "desligar",
    "desligaría": "desligar",
    "desligaríais": "desligar",
    "desligaríamos": "desligar",
    "desligarían": "desligar",
    "desligarías": "desligar",
    "desligas": "desligar",
    "desligase": "desligar",
    "desligaseis": "desligar",
    "desligasen": "desligar",
    "desligases": "desligar",
    "desligaste": "desligar",
    "desligasteis": "desligar",
    "desligo": "desligar",
    "desligue": "desligar",
    "desliguemos": "desligar",
    "desliguen": "desligar",
    "desligues": "desligar",
    "desligué": "desligar",
    "desliguéis": "desligar",
    "desligábamos": "desligar",
    "desligáis": "desligar",
    "desligáramos": "desligar",
    "desligáremos": "desligar",
    "desligásemos": "desligar",
    "desligó": "desligar",
    "deslinda": "deslindar",
    "deslindaba": "deslindar",
    "deslindabais": "deslindar",
    "deslindaban": "deslindar",
    "deslindabas": "deslindar",
    "deslindad": "deslindar",
    "deslindada": "deslindar",
    "deslindadas": "deslindar",
    "deslindado": "deslindar",
    "deslindados": "deslindar",
    "deslindamientos": "deslindamiento",
    "deslindamos": "deslindar",
    "deslindan": "deslindar",
    "deslindando": "deslindar",
    "deslindara": "deslindar",
    "deslindarais": "deslindar",
    "deslindaran": "deslindar",
    "deslindaras": "deslindar",
    "deslindare": "deslindar",
    "deslindareis": "deslindar",
    "deslindaremos": "deslindar",
    "deslindaren": "deslindar",
    "deslindares": "deslindar",
    "deslindaron": "deslindar",
    "deslindará": "deslindar",
    "deslindarán": "deslindar",
    "deslindarás": "deslindar",
    "deslindaré": "deslindar",
    "deslindaréis": "deslindar",
    "deslindaría": "deslindar",
    "deslindaríais": "deslindar",
    "deslindaríamos": "deslindar",
    "deslindarían": "deslindar",
    "deslindarías": "deslindar",
    "deslindas": "deslindar",
    "deslindase": "deslindar",
    "deslindaseis": "deslindar",
    "deslindasen": "deslindar",
    "deslindases": "deslindar",
    "deslindaste": "deslindar",
    "deslindasteis": "deslindar",
    "deslinde": "deslindar",
    "deslindemos": "deslindar",
    "deslinden": "deslindar",
    "deslindes": "deslinde",
    "deslindo": "deslindar",
    "deslindábamos": "deslindar",
    "deslindáis": "deslindar",
    "deslindáramos": "deslindar",
    "deslindáremos": "deslindar",
    "deslindásemos": "deslindar",
    "deslindé": "deslindar",
    "deslindéis": "deslindar",
    "deslindó": "deslindar",
    "desliza": "deslizar",
    "deslizaba": "deslizar",
    "deslizabais": "deslizar",
    "deslizaban": "deslizar",
    "deslizabas": "deslizar",
    "deslizables": "deslizable",
    "deslizad": "deslizar",
    "deslizada": "deslizar",
    "deslizadas": "deslizar",
    "deslizadera": "deslizadero",
    "deslizaderas": "deslizadero",
    "deslizaderos": "deslizadero",
    "deslizadiza": "deslizadizo",
    "deslizadizas": "deslizadizo",
    "deslizadizos": "deslizadizo",
    "deslizado": "deslizar",
    "deslizadora": "deslizador",
    "deslizadoras": "deslizadora",
    "deslizadores": "deslizador",
    "deslizados": "deslizar",
    "deslizamientos": "deslizamiento",
    "deslizamos": "deslizar",
    "deslizan": "deslizar",
    "deslizando": "deslizar",
    "deslizantes": "deslizante",
    "deslizara": "deslizar",
    "deslizarais": "deslizar",
    "deslizaran": "deslizar",
    "deslizaras": "deslizar",
    "deslizare": "deslizar",
    "deslizareis": "deslizar",
    "deslizaremos": "deslizar",
    "deslizaren": "deslizar",
    "deslizares": "deslizar",
    "deslizaron": "deslizar",
    "deslizará": "deslizar",
    "deslizarán": "deslizar",
    "deslizarás": "deslizar",
    "deslizaré": "deslizar",
    "deslizaréis": "deslizar",
    "deslizaría": "deslizar",
    "deslizaríais": "deslizar",
    "deslizaríamos": "deslizar",
    "deslizarían": "deslizar",
    "deslizarías": "deslizar",
    "deslizas": "deslizar",
    "deslizase": "deslizar",
    "deslizaseis": "deslizar",
    "deslizasen": "deslizar",
    "deslizases": "deslizar",
    "deslizaste": "deslizar",
    "deslizasteis": "deslizar",
    "deslizo": "deslizar",
    "deslizábamos": "deslizar",
    "deslizáis": "deslizar",
    "deslizáramos": "deslizar",
    "deslizáremos": "deslizar",
    "deslizásemos": "deslizar",
    "deslizó": "deslizar",
    "desliábamos": "desliar",
    "desliáis": "desliar",
    "desliáramos": "desliar",
    "desliáremos": "desliar",
    "desliásemos": "desliar",
    "deslié": "desliar",
    "desliéis": "desliar",
    "desliéramos": "desleír",
    "desliéremos": "desleír",
    "desliésemos": "desleír",
    "desliña": "desliñar",
    "desliñaba": "desliñar",
    "desliñabais": "desliñar",
    "desliñaban": "desliñar",
    "desliñabas": "desliñar",
    "desliñad": "desliñar",
    "desliñada": "desliñar",
    "desliñadas": "desliñar",
    "desliñado": "desliñar",
    "desliñados": "desliñar",
    "desliñamos": "desliñar",
    "desliñan": "desliñar",
    "desliñando": "desliñar",
    "desliñara": "desliñar",
    "desliñarais": "desliñar",
    "desliñaran": "desliñar",
    "desliñaras": "desliñar",
    "desliñare": "desliñar",
    "desliñareis": "desliñar",
    "desliñaremos": "desliñar",
    "desliñaren": "desliñar",
    "desliñares": "desliñar",
    "desliñaron": "desliñar",
    "desliñará": "desliñar",
    "desliñarán": "desliñar",
    "desliñarás": "desliñar",
    "desliñaré": "desliñar",
    "desliñaréis": "desliñar",
    "desliñaría": "desliñar",
    "desliñaríais": "desliñar",
    "desliñaríamos": "desliñar",
    "desliñarían": "desliñar",
    "desliñarías": "desliñar",
    "desliñas": "desliñar",
    "desliñase": "desliñar",
    "desliñaseis": "desliñar",
    "desliñasen": "desliñar",
    "desliñases": "desliñar",
    "desliñaste": "desliñar",
    "desliñasteis": "desliñar",
    "desliñe": "desliñar",
    "desliñemos": "desliñar",
    "desliñen": "desliñar",
    "desliñes": "desliñar",
    "desliño": "desliñar",
    "desliñábamos": "desliñar",
    "desliñáis": "desliñar",
    "desliñáramos": "desliñar",
    "desliñáremos": "desliñar",
    "desliñásemos": "desliñar",
    "desliñé": "desliñar",
    "desliñéis": "desliñar",
    "desliñó": "desliñar",
    "deslió": "desliar",
    "deslocalizaciones": "deslocalización",
    "desloma": "deslomar",
    "deslomaba": "deslomar",
    "deslomabais": "deslomar",
    "deslomaban": "deslomar",
    "deslomabas": "deslomar",
    "deslomad": "deslomar",
    "deslomada": "deslomar",
    "deslomadas": "deslomar",
    "deslomado": "deslomar",
    "deslomados": "deslomar",
    "deslomaduras": "deslomadura",
    "deslomamos": "deslomar",
    "desloman": "deslomar",
    "deslomando": "deslomar",
    "deslomara": "deslomar",
    "deslomarais": "deslomar",
    "deslomaran": "deslomar",
    "deslomaras": "deslomar",
    "deslomare": "deslomar",
    "deslomareis": "deslomar",
    "deslomaremos": "deslomar",
    "deslomaren": "deslomar",
    "deslomares": "deslomar",
    "deslomaron": "deslomar",
    "deslomará": "deslomar",
    "deslomarán": "deslomar",
    "deslomarás": "deslomar",
    "deslomaré": "deslomar",
    "deslomaréis": "deslomar",
    "deslomaría": "deslomar",
    "deslomaríais": "deslomar",
    "deslomaríamos": "deslomar",
    "deslomarían": "deslomar",
    "deslomarías": "deslomar",
    "deslomas": "deslomar",
    "deslomase": "deslomar",
    "deslomaseis": "deslomar",
    "deslomasen": "deslomar",
    "deslomases": "deslomar",
    "deslomaste": "deslomar",
    "deslomasteis": "deslomar",
    "deslome": "deslomar",
    "deslomemos": "deslomar",
    "deslomen": "deslomar",
    "deslomes": "deslomar",
    "deslomo": "deslomar",
    "deslomábamos": "deslomar",
    "deslomáis": "deslomar",
    "deslomáramos": "deslomar",
    "deslomáremos": "deslomar",
    "deslomásemos": "deslomar",
    "deslomé": "deslomar",
    "desloméis": "deslomar",
    "deslomó": "deslomar",
    "desluce": "deslucir",
    "deslucen": "deslucir",
    "desluces": "deslucir",
    "deslucid": "deslucir",
    "deslucida": "deslucir",
    "deslucidas": "deslucir",
    "deslucido": "deslucir",
    "deslucidos": "deslucir",
    "desluciendo": "deslucir",
    "desluciera": "deslucir",
    "deslucierais": "deslucir",
    "deslucieran": "deslucir",
    "deslucieras": "deslucir",
    "desluciere": "deslucir",
    "desluciereis": "deslucir",
    "deslucieren": "deslucir",
    "deslucieres": "deslucir",
    "deslucieron": "deslucir",
    "desluciese": "deslucir",
    "deslucieseis": "deslucir",
    "desluciesen": "deslucir",
    "deslucieses": "deslucir",
    "deslucimientos": "deslucimiento",
    "deslucimos": "deslucir",
    "desluciremos": "deslucir",
    "deslucirá": "deslucir",
    "deslucirán": "deslucir",
    "deslucirás": "deslucir",
    "desluciré": "deslucir",
    "desluciréis": "deslucir",
    "desluciría": "deslucir",
    "desluciríais": "deslucir",
    "desluciríamos": "deslucir",
    "deslucirían": "deslucir",
    "deslucirías": "deslucir",
    "desluciste": "deslucir",
    "deslucisteis": "deslucir",
    "desluciéramos": "deslucir",
    "desluciéremos": "deslucir",
    "desluciésemos": "deslucir",
    "deslució": "deslucir",
    "deslucí": "deslucir",
    "deslucía": "deslucir",
    "deslucíais": "deslucir",
    "deslucíamos": "deslucir",
    "deslucían": "deslucir",
    "deslucías": "deslucir",
    "deslucís": "deslucir",
    "deslumbra": "deslumbrar",
    "deslumbraba": "deslumbrar",
    "deslumbrabais": "deslumbrar",
    "deslumbraban": "deslumbrar",
    "deslumbrabas": "deslumbrar",
    "deslumbrad": "deslumbrar",
    "deslumbrada": "deslumbrar",
    "deslumbradas": "deslumbrar",
    "deslumbrado": "deslumbrar",
    "deslumbradora": "deslumbrador",
    "deslumbradoras": "deslumbrador",
    "deslumbradores": "deslumbrador",
    "deslumbrados": "deslumbrar",
    "deslumbramientos": "deslumbramiento",
    "deslumbramos": "deslumbrar",
    "deslumbran": "deslumbrar",
    "deslumbrando": "deslumbrar",
    "deslumbrantes": "deslumbrante",
    "deslumbrara": "deslumbrar",
    "deslumbrarais": "deslumbrar",
    "deslumbraran": "deslumbrar",
    "deslumbraras": "deslumbrar",
    "deslumbrare": "deslumbrar",
    "deslumbrareis": "deslumbrar",
    "deslumbraremos": "deslumbrar",
    "deslumbraren": "deslumbrar",
    "deslumbrares": "deslumbrar",
    "deslumbraron": "deslumbrar",
    "deslumbrará": "deslumbrar",
    "deslumbrarán": "deslumbrar",
    "deslumbrarás": "deslumbrar",
    "deslumbraré": "deslumbrar",
    "deslumbraréis": "deslumbrar",
    "deslumbraría": "deslumbrar",
    "deslumbraríais": "deslumbrar",
    "deslumbraríamos": "deslumbrar",
    "deslumbrarían": "deslumbrar",
    "deslumbrarías": "deslumbrar",
    "deslumbras": "deslumbrar",
    "deslumbrase": "deslumbrar",
    "deslumbraseis": "deslumbrar",
    "deslumbrasen": "deslumbrar",
    "deslumbrases": "deslumbrar",
    "deslumbraste": "deslumbrar",
    "deslumbrasteis": "deslumbrar",
    "deslumbre": "deslumbrar",
    "deslumbremos": "deslumbrar",
    "deslumbren": "deslumbrar",
    "deslumbres": "deslumbrar",
    "deslumbro": "deslumbrar",
    "deslumbrábamos": "deslumbrar",
    "deslumbráis": "deslumbrar",
    "deslumbráramos": "deslumbrar",
    "deslumbráremos": "deslumbrar",
    "deslumbrásemos": "deslumbrar",
    "deslumbré": "deslumbrar",
    "deslumbréis": "deslumbrar",
    "deslumbró": "deslumbrar",
    "deslustra": "deslustrar",
    "deslustraba": "deslustrar",
    "deslustrabais": "deslustrar",
    "deslustraban": "deslustrar",
    "deslustrabas": "deslustrar",
    "deslustraciones": "deslustración",
    "deslustrad": "deslustrar",
    "deslustrada": "deslustrar",
    "deslustradas": "deslustrar",
    "deslustrado": "deslustrar",
    "deslustradora": "deslustrador",
    "deslustradoras": "deslustrador",
    "deslustradores": "deslustrador",
    "deslustrados": "deslustrar",
    "deslustramos": "deslustrar",
    "deslustran": "deslustrar",
    "deslustrando": "deslustrar",
    "deslustrara": "deslustrar",
    "deslustrarais": "deslustrar",
    "deslustraran": "deslustrar",
    "deslustraras": "deslustrar",
    "deslustrare": "deslustrar",
    "deslustrareis": "deslustrar",
    "deslustraremos": "deslustrar",
    "deslustraren": "deslustrar",
    "deslustrares": "deslustrar",
    "deslustraron": "deslustrar",
    "deslustrará": "deslustrar",
    "deslustrarán": "deslustrar",
    "deslustrarás": "deslustrar",
    "deslustraré": "deslustrar",
    "deslustraréis": "deslustrar",
    "deslustraría": "deslustrar",
    "deslustraríais": "deslustrar",
    "deslustraríamos": "deslustrar",
    "deslustrarían": "deslustrar",
    "deslustrarías": "deslustrar",
    "deslustras": "deslustrar",
    "deslustrase": "deslustrar",
    "deslustraseis": "deslustrar",
    "deslustrasen": "deslustrar",
    "deslustrases": "deslustrar",
    "deslustraste": "deslustrar",
    "deslustrasteis": "deslustrar",
    "deslustre": "deslustrar",
    "deslustremos": "deslustrar",
    "deslustren": "deslustrar",
    "deslustres": "deslustre",
    "deslustro": "deslustrar",
    "deslustrosa": "deslustroso",
    "deslustrosas": "deslustroso",
    "deslustrosos": "deslustroso",
    "deslustrábamos": "deslustrar",
    "deslustráis": "deslustrar",
    "deslustráramos": "deslustrar",
    "deslustráremos": "deslustrar",
    "deslustrásemos": "deslustrar",
    "deslustré": "deslustrar",
    "deslustréis": "deslustrar",
    "deslustró": "deslustrar",
    "desluzca": "deslucir",
    "desluzcamos": "deslucir",
    "desluzcan": "deslucir",
    "desluzcas": "deslucir",
    "desluzco": "deslucir",
    "desluzcáis": "deslucir",
    "deslía": "desliar",
    "deslían": "desliar",
    "deslías": "desliar",
    "deslíe": "desliar",
    "deslíen": "desliar",
    "deslíes": "desliar",
    "deslío": "desliar",
    "deslíos": "deslío",
    "desmadeja": "desmadejar",
    "desmadejaba": "desmadejar",
    "desmadejabais": "desmadejar",
    "desmadejaban": "desmadejar",
    "desmadejabas": "desmadejar",
    "desmadejad": "desmadejar",
    "desmadejada": "desmadejar",
    "desmadejadas": "desmadejar",
    "desmadejado": "desmadejar",
    "desmadejados": "desmadejar",
    "desmadejamientos": "desmadejamiento",
    "desmadejamos": "desmadejar",
    "desmadejan": "desmadejar",
    "desmadejando": "desmadejar",
    "desmadejara": "desmadejar",
    "desmadejarais": "desmadejar",
    "desmadejaran": "desmadejar",
    "desmadejaras": "desmadejar",
    "desmadejare": "desmadejar",
    "desmadejareis": "desmadejar",
    "desmadejaremos": "desmadejar",
    "desmadejaren": "desmadejar",
    "desmadejares": "desmadejar",
    "desmadejaron": "desmadejar",
    "desmadejará": "desmadejar",
    "desmadejarán": "desmadejar",
    "desmadejarás": "desmadejar",
    "desmadejaré": "desmadejar",
    "desmadejaréis": "desmadejar",
    "desmadejaría": "desmadejar",
    "desmadejaríais": "desmadejar",
    "desmadejaríamos": "desmadejar",
    "desmadejarían": "desmadejar",
    "desmadejarías": "desmadejar",
    "desmadejas": "desmadejar",
    "desmadejase": "desmadejar",
    "desmadejaseis": "desmadejar",
    "desmadejasen": "desmadejar",
    "desmadejases": "desmadejar",
    "desmadejaste": "desmadejar",
    "desmadejasteis": "desmadejar",
    "desmadeje": "desmadejar",
    "desmadejemos": "desmadejar",
    "desmadejen": "desmadejar",
    "desmadejes": "desmadejar",
    "desmadejo": "desmadejar",
    "desmadejábamos": "desmadejar",
    "desmadejáis": "desmadejar",
    "desmadejáramos": "desmadejar",
    "desmadejáremos": "desmadejar",
    "desmadejásemos": "desmadejar",
    "desmadejé": "desmadejar",
    "desmadejéis": "desmadejar",
    "desmadejó": "desmadejar",
    "desmadra": "desmadrar",
    "desmadraba": "desmadrar",
    "desmadrabais": "desmadrar",
    "desmadraban": "desmadrar",
    "desmadrabas": "desmadrar",
    "desmadrad": "desmadrar",
    "desmadrada": "desmadrar",
    "desmadradas": "desmadrar",
    "desmadrado": "desmadrar",
    "desmadrados": "desmadrar",
    "desmadramos": "desmadrar",
    "desmadran": "desmadrar",
    "desmadrando": "desmadrar",
    "desmadrara": "desmadrar",
    "desmadrarais": "desmadrar",
    "desmadraran": "desmadrar",
    "desmadraras": "desmadrar",
    "desmadrare": "desmadrar",
    "desmadrareis": "desmadrar",
    "desmadraremos": "desmadrar",
    "desmadraren": "desmadrar",
    "desmadrares": "desmadrar",
    "desmadraron": "desmadrar",
    "desmadrará": "desmadrar",
    "desmadrarán": "desmadrar",
    "desmadrarás": "desmadrar",
    "desmadraré": "desmadrar",
    "desmadraréis": "desmadrar",
    "desmadraría": "desmadrar",
    "desmadraríais": "desmadrar",
    "desmadraríamos": "desmadrar",
    "desmadrarían": "desmadrar",
    "desmadrarías": "desmadrar",
    "desmadras": "desmadrar",
    "desmadrase": "desmadrar",
    "desmadraseis": "desmadrar",
    "desmadrasen": "desmadrar",
    "desmadrases": "desmadrar",
    "desmadraste": "desmadrar",
    "desmadrasteis": "desmadrar",
    "desmadre": "desmadrar",
    "desmadremos": "desmadrar",
    "desmadren": "desmadrar",
    "desmadres": "desmadrar",
    "desmadro": "desmadrar",
    "desmadrábamos": "desmadrar",
    "desmadráis": "desmadrar",
    "desmadráramos": "desmadrar",
    "desmadráremos": "desmadrar",
    "desmadrásemos": "desmadrar",
    "desmadré": "desmadrar",
    "desmadréis": "desmadrar",
    "desmadró": "desmadrar",
    "desmagnetice": "desmagnetizar",
    "desmagneticemos": "desmagnetizar",
    "desmagneticen": "desmagnetizar",
    "desmagnetices": "desmagnetizar",
    "desmagneticé": "desmagnetizar",
    "desmagneticéis": "desmagnetizar",
    "desmagnetiza": "desmagnetizar",
    "desmagnetizaba": "desmagnetizar",
    "desmagnetizabais": "desmagnetizar",
    "desmagnetizaban": "desmagnetizar",
    "desmagnetizabas": "desmagnetizar",
    "desmagnetizaciones": "desmagnetización",
    "desmagnetizad": "desmagnetizar",
    "desmagnetizada": "desmagnetizar",
    "desmagnetizadas": "desmagnetizar",
    "desmagnetizado": "desmagnetizar",
    "desmagnetizadores": "desmagnetizador",
    "desmagnetizados": "desmagnetizar",
    "desmagnetizamos": "desmagnetizar",
    "desmagnetizan": "desmagnetizar",
    "desmagnetizando": "desmagnetizar",
    "desmagnetizantes": "desmagnetizante",
    "desmagnetizara": "desmagnetizar",
    "desmagnetizarais": "desmagnetizar",
    "desmagnetizaran": "desmagnetizar",
    "desmagnetizaras": "desmagnetizar",
    "desmagnetizare": "desmagnetizar",
    "desmagnetizareis": "desmagnetizar",
    "desmagnetizaremos": "desmagnetizar",
    "desmagnetizaren": "desmagnetizar",
    "desmagnetizares": "desmagnetizar",
    "desmagnetizaron": "desmagnetizar",
    "desmagnetizará": "desmagnetizar",
    "desmagnetizarán": "desmagnetizar",
    "desmagnetizarás": "desmagnetizar",
    "desmagnetizaré": "desmagnetizar",
    "desmagnetizaréis": "desmagnetizar",
    "desmagnetizaría": "desmagnetizar",
    "desmagnetizaríais": "desmagnetizar",
    "desmagnetizaríamos": "desmagnetizar",
    "desmagnetizarían": "desmagnetizar",
    "desmagnetizarías": "desmagnetizar",
    "desmagnetizas": "desmagnetizar",
    "desmagnetizase": "desmagnetizar",
    "desmagnetizaseis": "desmagnetizar",
    "desmagnetizasen": "desmagnetizar",
    "desmagnetizases": "desmagnetizar",
    "desmagnetizaste": "desmagnetizar",
    "desmagnetizasteis": "desmagnetizar",
    "desmagnetizo": "desmagnetizar",
    "desmagnetizábamos": "desmagnetizar",
    "desmagnetizáis": "desmagnetizar",
    "desmagnetizáramos": "desmagnetizar",
    "desmagnetizáremos": "desmagnetizar",
    "desmagnetizásemos": "desmagnetizar",
    "desmagnetizó": "desmagnetizar",
    "desmajolaba": "desmajolar",
    "desmajolabais": "desmajolar",
    "desmajolaban": "desmajolar",
    "desmajolabas": "desmajolar",
    "desmajolad": "desmajolar",
    "desmajolada": "desmajolar",
    "desmajoladas": "desmajolar",
    "desmajolado": "desmajolar",
    "desmajolados": "desmajolar",
    "desmajolamos": "desmajolar",
    "desmajolando": "desmajolar",
    "desmajolara": "desmajolar",
    "desmajolarais": "desmajolar",
    "desmajolaran": "desmajolar",
    "desmajolaras": "desmajolar",
    "desmajolare": "desmajolar",
    "desmajolareis": "desmajolar",
    "desmajolaremos": "desmajolar",
    "desmajolaren": "desmajolar",
    "desmajolares": "desmajolar",
    "desmajolaron": "desmajolar",
    "desmajolará": "desmajolar",
    "desmajolarán": "desmajolar",
    "desmajolarás": "desmajolar",
    "desmajolaré": "desmajolar",
    "desmajolaréis": "desmajolar",
    "desmajolaría": "desmajolar",
    "desmajolaríais": "desmajolar",
    "desmajolaríamos": "desmajolar",
    "desmajolarían": "desmajolar",
    "desmajolarías": "desmajolar",
    "desmajolase": "desmajolar",
    "desmajolaseis": "desmajolar",
    "desmajolasen": "desmajolar",
    "desmajolases": "desmajolar",
    "desmajolaste": "desmajolar",
    "desmajolasteis": "desmajolar",
    "desmajolemos": "desmajolar",
    "desmajolábamos": "desmajolar",
    "desmajoláis": "desmajolar",
    "desmajoláramos": "desmajolar",
    "desmajoláremos": "desmajolar",
    "desmajolásemos": "desmajolar",
    "desmajolé": "desmajolar",
    "desmajoléis": "desmajolar",
    "desmajoló": "desmajolar",
    "desmajuela": "desmajolar",
    "desmajuelan": "desmajolar",
    "desmajuelas": "desmajolar",
    "desmajuele": "desmajolar",
    "desmajuelen": "desmajolar",
    "desmajueles": "desmajolar",
    "desmajuelo": "desmajolar",
    "desmalazada": "desmalazado",
    "desmalazadas": "desmalazado",
    "desmalazados": "desmalazado",
    "desmalece": "desmalezar",
    "desmalecemos": "desmalezar",
    "desmalecen": "desmalezar",
    "desmaleces": "desmalezar",
    "desmalecé": "desmalezar",
    "desmalecéis": "desmalezar",
    "desmaleza": "desmalezar",
    "desmalezaba": "desmalezar",
    "desmalezabais": "desmalezar",
    "desmalezaban": "desmalezar",
    "desmalezabas": "desmalezar",
    "desmalezad": "desmalezar",
    "desmalezada": "desmalezar",
    "desmalezadas": "desmalezar",
    "desmalezado": "desmalezar",
    "desmalezados": "desmalezar",
    "desmalezamos": "desmalezar",
    "desmalezan": "desmalezar",
    "desmalezando": "desmalezar",
    "desmalezara": "desmalezar",
    "desmalezarais": "desmalezar",
    "desmalezaran": "desmalezar",
    "desmalezaras": "desmalezar",
    "desmalezare": "desmalezar",
    "desmalezareis": "desmalezar",
    "desmalezaremos": "desmalezar",
    "desmalezaren": "desmalezar",
    "desmalezares": "desmalezar",
    "desmalezaron": "desmalezar",
    "desmalezará": "desmalezar",
    "desmalezarán": "desmalezar",
    "desmalezarás": "desmalezar",
    "desmalezaré": "desmalezar",
    "desmalezaréis": "desmalezar",
    "desmalezaría": "desmalezar",
    "desmalezaríais": "desmalezar",
    "desmalezaríamos": "desmalezar",
    "desmalezarían": "desmalezar",
    "desmalezarías": "desmalezar",
    "desmalezas": "desmalezar",
    "desmalezase": "desmalezar",
    "desmalezaseis": "desmalezar",
    "desmalezasen": "desmalezar",
    "desmalezases": "desmalezar",
    "desmalezaste": "desmalezar",
    "desmalezasteis": "desmalezar",
    "desmalezo": "desmalezar",
    "desmalezábamos": "desmalezar",
    "desmalezáis": "desmalezar",
    "desmalezáramos": "desmalezar",
    "desmalezáremos": "desmalezar",
    "desmalezásemos": "desmalezar",
    "desmalezó": "desmalezar",
    "desmalla": "desmallar",
    "desmallaba": "desmallar",
    "desmallabais": "desmallar",
    "desmallaban": "desmallar",
    "desmallabas": "desmallar",
    "desmallad": "desmallar",
    "desmallada": "desmallar",
    "desmalladas": "desmallar",
    "desmallado": "desmallar",
    "desmalladora": "desmallador",
    "desmalladoras": "desmallador",
    "desmalladores": "desmallador",
    "desmallados": "desmallar",
    "desmalladuras": "desmalladura",
    "desmallamos": "desmallar",
    "desmallan": "desmallar",
    "desmallando": "desmallar",
    "desmallara": "desmallar",
    "desmallarais": "desmallar",
    "desmallaran": "desmallar",
    "desmallaras": "desmallar",
    "desmallare": "desmallar",
    "desmallareis": "desmallar",
    "desmallaremos": "desmallar",
    "desmallaren": "desmallar",
    "desmallares": "desmallar",
    "desmallaron": "desmallar",
    "desmallará": "desmallar",
    "desmallarán": "desmallar",
    "desmallarás": "desmallar",
    "desmallaré": "desmallar",
    "desmallaréis": "desmallar",
    "desmallaría": "desmallar",
    "desmallaríais": "desmallar",
    "desmallaríamos": "desmallar",
    "desmallarían": "desmallar",
    "desmallarías": "desmallar",
    "desmallas": "desmallar",
    "desmallase": "desmallar",
    "desmallaseis": "desmallar",
    "desmallasen": "desmallar",
    "desmallases": "desmallar",
    "desmallaste": "desmallar",
    "desmallasteis": "desmallar",
    "desmalle": "desmallar",
    "desmallemos": "desmallar",
    "desmallen": "desmallar",
    "desmalles": "desmallar",
    "desmallo": "desmallar",
    "desmallábamos": "desmallar",
    "desmalláis": "desmallar",
    "desmalláramos": "desmallar",
    "desmalláremos": "desmallar",
    "desmallásemos": "desmallar",
    "desmallé": "desmallar",
    "desmalléis": "desmallar",
    "desmalló": "desmallar",
    "desmama": "desmamar",
    "desmamaba": "desmamar",
    "desmamabais": "desmamar",
    "desmamaban": "desmamar",
    "desmamabas": "desmamar",
    "desmamad": "desmamar",
    "desmamada": "desmamar",
    "desmamadas": "desmamar",
    "desmamado": "desmamar",
    "desmamados": "desmamar",
    "desmamamos": "desmamar",
    "desmaman": "desmamar",
    "desmamando": "desmamar",
    "desmamara": "desmamar",
    "desmamarais": "desmamar",
    "desmamaran": "desmamar",
    "desmamaras": "desmamar",
    "desmamare": "desmamar",
    "desmamareis": "desmamar",
    "desmamaremos": "desmamar",
    "desmamaren": "desmamar",
    "desmamares": "desmamar",
    "desmamaron": "desmamar",
    "desmamará": "desmamar",
    "desmamarán": "desmamar",
    "desmamarás": "desmamar",
    "desmamaré": "desmamar",
    "desmamaréis": "desmamar",
    "desmamaría": "desmamar",
    "desmamaríais": "desmamar",
    "desmamaríamos": "desmamar",
    "desmamarían": "desmamar",
    "desmamarías": "desmamar",
    "desmamas": "desmamar",
    "desmamase": "desmamar",
    "desmamaseis": "desmamar",
    "desmamasen": "desmamar",
    "desmamases": "desmamar",
    "desmamaste": "desmamar",
    "desmamasteis": "desmamar",
    "desmame": "desmamar",
    "desmamemos": "desmamar",
    "desmamen": "desmamar",
    "desmames": "desmamar",
    "desmamo": "desmamar",
    "desmamábamos": "desmamar",
    "desmamáis": "desmamar",
    "desmamáramos": "desmamar",
    "desmamáremos": "desmamar",
    "desmamásemos": "desmamar",
    "desmamé": "desmamar",
    "desmaméis": "desmamar",
    "desmamó": "desmamar",
    "desmancha": "desmanchar",
    "desmanchaba": "desmanchar",
    "desmanchabais": "desmanchar",
    "desmanchaban": "desmanchar",
    "desmanchabas": "desmanchar",
    "desmanchad": "desmanchar",
    "desmanchada": "desmanchar",
    "desmanchadas": "desmanchar",
    "desmanchado": "desmanchar",
    "desmanchados": "desmanchar",
    "desmanchamos": "desmanchar",
    "desmanchan": "desmanchar",
    "desmanchando": "desmanchar",
    "desmanchara": "desmanchar",
    "desmancharais": "desmanchar",
    "desmancharan": "desmanchar",
    "desmancharas": "desmanchar",
    "desmanchare": "desmanchar",
    "desmanchareis": "desmanchar",
    "desmancharemos": "desmanchar",
    "desmancharen": "desmanchar",
    "desmanchares": "desmanchar",
    "desmancharon": "desmanchar",
    "desmanchará": "desmanchar",
    "desmancharán": "desmanchar",
    "desmancharás": "desmanchar",
    "desmancharé": "desmanchar",
    "desmancharéis": "desmanchar",
    "desmancharía": "desmanchar",
    "desmancharíais": "desmanchar",
    "desmancharíamos": "desmanchar",
    "desmancharían": "desmanchar",
    "desmancharías": "desmanchar",
    "desmanchas": "desmanchar",
    "desmanchase": "desmanchar",
    "desmanchaseis": "desmanchar",
    "desmanchasen": "desmanchar",
    "desmanchases": "desmanchar",
    "desmanchaste": "desmanchar",
    "desmanchasteis": "desmanchar",
    "desmanche": "desmanchar",
    "desmanchemos": "desmanchar",
    "desmanchen": "desmanchar",
    "desmanches": "desmanchar",
    "desmancho": "desmanchar",
    "desmanchábamos": "desmanchar",
    "desmancháis": "desmanchar",
    "desmancháramos": "desmanchar",
    "desmancháremos": "desmanchar",
    "desmanchásemos": "desmanchar",
    "desmanché": "desmanchar",
    "desmanchéis": "desmanchar",
    "desmanchó": "desmanchar",
    "desmanda": "desmandar",
    "desmandaba": "desmandar",
    "desmandabais": "desmandar",
    "desmandaban": "desmandar",
    "desmandabas": "desmandar",
    "desmandad": "desmandar",
    "desmandada": "desmandar",
    "desmandadas": "desmandar",
    "desmandado": "desmandar",
    "desmandados": "desmandar",
    "desmandamientos": "desmandamiento",
    "desmandamos": "desmandar",
    "desmandan": "desmandar",
    "desmandando": "desmandar",
    "desmandara": "desmandar",
    "desmandarais": "desmandar",
    "desmandaran": "desmandar",
    "desmandaras": "desmandar",
    "desmandare": "desmandar",
    "desmandareis": "desmandar",
    "desmandaremos": "desmandar",
    "desmandaren": "desmandar",
    "desmandares": "desmandar",
    "desmandaron": "desmandar",
    "desmandará": "desmandar",
    "desmandarán": "desmandar",
    "desmandarás": "desmandar",
    "desmandaré": "desmandar",
    "desmandaréis": "desmandar",
    "desmandaría": "desmandar",
    "desmandaríais": "desmandar",
    "desmandaríamos": "desmandar",
    "desmandarían": "desmandar",
    "desmandarías": "desmandar",
    "desmandas": "desmandar",
    "desmandase": "desmandar",
    "desmandaseis": "desmandar",
    "desmandasen": "desmandar",
    "desmandases": "desmandar",
    "desmandaste": "desmandar",
    "desmandasteis": "desmandar",
    "desmande": "desmandar",
    "desmandemos": "desmandar",
    "desmanden": "desmandar",
    "desmandes": "desmandar",
    "desmando": "desmandar",
    "desmandábamos": "desmandar",
    "desmandáis": "desmandar",
    "desmandáramos": "desmandar",
    "desmandáremos": "desmandar",
    "desmandásemos": "desmandar",
    "desmandé": "desmandar",
    "desmandéis": "desmandar",
    "desmandó": "desmandar",
    "desmanea": "desmanear",
    "desmaneaba": "desmanear",
    "desmaneabais": "desmanear",
    "desmaneaban": "desmanear",
    "desmaneabas": "desmanear",
    "desmanead": "desmanear",
    "desmaneada": "desmanear",
    "desmaneadas": "desmanear",
    "desmaneado": "desmanear",
    "desmaneados": "desmanear",
    "desmaneamos": "desmanear",
    "desmanean": "desmanear",
    "desmaneando": "desmanear",
    "desmaneara": "desmanear",
    "desmanearais": "desmanear",
    "desmanearan": "desmanear",
    "desmanearas": "desmanear",
    "desmaneare": "desmanear",
    "desmaneareis": "desmanear",
    "desmanearemos": "desmanear",
    "desmanearen": "desmanear",
    "desmaneares": "desmanear",
    "desmanearon": "desmanear",
    "desmaneará": "desmanear",
    "desmanearán": "desmanear",
    "desmanearás": "desmanear",
    "desmanearé": "desmanear",
    "desmanearéis": "desmanear",
    "desmanearía": "desmanear",
    "desmanearíais": "desmanear",
    "desmanearíamos": "desmanear",
    "desmanearían": "desmanear",
    "desmanearías": "desmanear",
    "desmaneas": "desmanear",
    "desmanease": "desmanear",
    "desmaneaseis": "desmanear",
    "desmaneasen": "desmanear",
    "desmaneases": "desmanear",
    "desmaneaste": "desmanear",
    "desmaneasteis": "desmanear",
    "desmanee": "desmanear",
    "desmaneemos": "desmanear",
    "desmaneen": "desmanear",
    "desmanees": "desmanear",
    "desmaneo": "desmanear",
    "desmanes": "desmán",
    "desmaneábamos": "desmanear",
    "desmaneáis": "desmanear",
    "desmaneáramos": "desmanear",
    "desmaneáremos": "desmanear",
    "desmaneásemos": "desmanear",
    "desmaneé": "desmanear",
    "desmaneéis": "desmanear",
    "desmaneó": "desmanear",
    "desmanga": "desmangar",
    "desmangaba": "desmangar",
    "desmangabais": "desmangar",
    "desmangaban": "desmangar",
    "desmangabas": "desmangar",
    "desmangad": "desmangar",
    "desmangada": "desmangar",
    "desmangadas": "desmangar",
    "desmangado": "desmangar",
    "desmangados": "desmangar",
    "desmangamos": "desmangar",
    "desmangan": "desmangar",
    "desmangando": "desmangar",
    "desmangara": "desmangar",
    "desmangarais": "desmangar",
    "desmangaran": "desmangar",
    "desmangaras": "desmangar",
    "desmangare": "desmangar",
    "desmangareis": "desmangar",
    "desmangaremos": "desmangar",
    "desmangaren": "desmangar",
    "desmangares": "desmangar",
    "desmangaron": "desmangar",
    "desmangará": "desmangar",
    "desmangarán": "desmangar",
    "desmangarás": "desmangar",
    "desmangaré": "desmangar",
    "desmangaréis": "desmangar",
    "desmangaría": "desmangar",
    "desmangaríais": "desmangar",
    "desmangaríamos": "desmangar",
    "desmangarían": "desmangar",
    "desmangarías": "desmangar",
    "desmangas": "desmangar",
    "desmangase": "desmangar",
    "desmangaseis": "desmangar",
    "desmangasen": "desmangar",
    "desmangases": "desmangar",
    "desmangaste": "desmangar",
    "desmangasteis": "desmangar",
    "desmango": "desmangar",
    "desmangue": "desmangar",
    "desmanguemos": "desmangar",
    "desmanguen": "desmangar",
    "desmangues": "desmangar",
    "desmangué": "desmangar",
    "desmanguéis": "desmangar",
    "desmangábamos": "desmangar",
    "desmangáis": "desmangar",
    "desmangáramos": "desmangar",
    "desmangáremos": "desmangar",
    "desmangásemos": "desmangar",
    "desmangó": "desmangar",
    "desmanotada": "desmanotado",
    "desmanotadas": "desmanotado",
    "desmanotados": "desmanotado",
    "desmanteca": "desmantecar",
    "desmantecaba": "desmantecar",
    "desmantecabais": "desmantecar",
    "desmantecaban": "desmantecar",
    "desmantecabas": "desmantecar",
    "desmantecad": "desmantecar",
    "desmantecada": "desmantecar",
    "desmantecadas": "desmantecar",
    "desmantecado": "desmantecar",
    "desmantecados": "desmantecar",
    "desmantecamos": "desmantecar",
    "desmantecan": "desmantecar",
    "desmantecando": "desmantecar",
    "desmantecara": "desmantecar",
    "desmantecarais": "desmantecar",
    "desmantecaran": "desmantecar",
    "desmantecaras": "desmantecar",
    "desmantecare": "desmantecar",
    "desmantecareis": "desmantecar",
    "desmantecaremos": "desmantecar",
    "desmantecaren": "desmantecar",
    "desmantecares": "desmantecar",
    "desmantecaron": "desmantecar",
    "desmantecará": "desmantecar",
    "desmantecarán": "desmantecar",
    "desmantecarás": "desmantecar",
    "desmantecaré": "desmantecar",
    "desmantecaréis": "desmantecar",
    "desmantecaría": "desmantecar",
    "desmantecaríais": "desmantecar",
    "desmantecaríamos": "desmantecar",
    "desmantecarían": "desmantecar",
    "desmantecarías": "desmantecar",
    "desmantecas": "desmantecar",
    "desmantecase": "desmantecar",
    "desmantecaseis": "desmantecar",
    "desmantecasen": "desmantecar",
    "desmantecases": "desmantecar",
    "desmantecaste": "desmantecar",
    "desmantecasteis": "desmantecar",
    "desmanteco": "desmantecar",
    "desmantecábamos": "desmantecar",
    "desmantecáis": "desmantecar",
    "desmantecáramos": "desmantecar",
    "desmantecáremos": "desmantecar",
    "desmantecásemos": "desmantecar",
    "desmantecó": "desmantecar",
    "desmantela": "desmantelar",
    "desmantelaba": "desmantelar",
    "desmantelabais": "desmantelar",
    "desmantelaban": "desmantelar",
    "desmantelabas": "desmantelar",
    "desmantelad": "desmantelar",
    "desmantelada": "desmantelar",
    "desmanteladas": "desmantelar",
    "desmantelado": "desmantelar",
    "desmantelados": "desmantelar",
    "desmantelamientos": "desmantelamiento",
    "desmantelamos": "desmantelar",
    "desmantelan": "desmantelar",
    "desmantelando": "desmantelar",
    "desmantelara": "desmantelar",
    "desmantelarais": "desmantelar",
    "desmantelaran": "desmantelar",
    "desmantelaras": "desmantelar",
    "desmantelare": "desmantelar",
    "desmantelareis": "desmantelar",
    "desmantelaremos": "desmantelar",
    "desmantelaren": "desmantelar",
    "desmantelares": "desmantelar",
    "desmantelaron": "desmantelar",
    "desmantelará": "desmantelar",
    "desmantelarán": "desmantelar",
    "desmantelarás": "desmantelar",
    "desmantelaré": "desmantelar",
    "desmantelaréis": "desmantelar",
    "desmantelaría": "desmantelar",
    "desmantelaríais": "desmantelar",
    "desmantelaríamos": "desmantelar",
    "desmantelarían": "desmantelar",
    "desmantelarías": "desmantelar",
    "desmantelas": "desmantelar",
    "desmantelase": "desmantelar",
    "desmantelaseis": "desmantelar",
    "desmantelasen": "desmantelar",
    "desmantelases": "desmantelar",
    "desmantelaste": "desmantelar",
    "desmantelasteis": "desmantelar",
    "desmantele": "desmantelar",
    "desmantelemos": "desmantelar",
    "desmantelen": "desmantelar",
    "desmanteles": "desmantelar",
    "desmantelo": "desmantelar",
    "desmantelábamos": "desmantelar",
    "desmanteláis": "desmantelar",
    "desmanteláramos": "desmantelar",
    "desmanteláremos": "desmantelar",
    "desmantelásemos": "desmantelar",
    "desmantelé": "desmantelar",
    "desmanteléis": "desmantelar",
    "desmanteló": "desmantelar",
    "desmanteque": "desmantecar",
    "desmantequemos": "desmantecar",
    "desmantequen": "desmantecar",
    "desmanteques": "desmantecar",
    "desmantequé": "desmantecar",
    "desmantequéis": "desmantecar",
    "desmaquilla": "desmaquillar",
    "desmaquillaba": "desmaquillar",
    "desmaquillabais": "desmaquillar",
    "desmaquillaban": "desmaquillar",
    "desmaquillabas": "desmaquillar",
    "desmaquillad": "desmaquillar",
    "desmaquillada": "desmaquillar",
    "desmaquilladas": "desmaquillar",
    "desmaquillado": "desmaquillar",
    "desmaquilladora": "desmaquillador",
    "desmaquilladoras": "desmaquillador",
    "desmaquilladores": "desmaquillador",
    "desmaquillados": "desmaquillar",
    "desmaquillamos": "desmaquillar",
    "desmaquillan": "desmaquillar",
    "desmaquillando": "desmaquillar",
    "desmaquillara": "desmaquillar",
    "desmaquillarais": "desmaquillar",
    "desmaquillaran": "desmaquillar",
    "desmaquillaras": "desmaquillar",
    "desmaquillare": "desmaquillar",
    "desmaquillareis": "desmaquillar",
    "desmaquillaremos": "desmaquillar",
    "desmaquillaren": "desmaquillar",
    "desmaquillares": "desmaquillar",
    "desmaquillaron": "desmaquillar",
    "desmaquillará": "desmaquillar",
    "desmaquillarán": "desmaquillar",
    "desmaquillarás": "desmaquillar",
    "desmaquillaré": "desmaquillar",
    "desmaquillaréis": "desmaquillar",
    "desmaquillaría": "desmaquillar",
    "desmaquillaríais": "desmaquillar",
    "desmaquillaríamos": "desmaquillar",
    "desmaquillarían": "desmaquillar",
    "desmaquillarías": "desmaquillar",
    "desmaquillas": "desmaquillar",
    "desmaquillase": "desmaquillar",
    "desmaquillaseis": "desmaquillar",
    "desmaquillasen": "desmaquillar",
    "desmaquillases": "desmaquillar",
    "desmaquillaste": "desmaquillar",
    "desmaquillasteis": "desmaquillar",
    "desmaquille": "desmaquillar",
    "desmaquillemos": "desmaquillar",
    "desmaquillen": "desmaquillar",
    "desmaquilles": "desmaquillar",
    "desmaquillo": "desmaquillar",
    "desmaquillábamos": "desmaquillar",
    "desmaquilláis": "desmaquillar",
    "desmaquilláramos": "desmaquillar",
    "desmaquilláremos": "desmaquillar",
    "desmaquillásemos": "desmaquillar",
    "desmaquillé": "desmaquillar",
    "desmaquilléis": "desmaquillar",
    "desmaquilló": "desmaquillar",
    "desmaraña": "desmarañar",
    "desmarañaba": "desmarañar",
    "desmarañabais": "desmarañar",
    "desmarañaban": "desmarañar",
    "desmarañabas": "desmarañar",
    "desmarañad": "desmarañar",
    "desmarañada": "desmarañar",
    "desmarañadas": "desmarañar",
    "desmarañado": "desmarañar",
    "desmarañados": "desmarañar",
    "desmarañamientos": "desmarañamiento",
    "desmarañamos": "desmarañar",
    "desmarañan": "desmarañar",
    "desmarañando": "desmarañar",
    "desmarañara": "desmarañar",
    "desmarañarais": "desmarañar",
    "desmarañaran": "desmarañar",
    "desmarañaras": "desmarañar",
    "desmarañare": "desmarañar",
    "desmarañareis": "desmarañar",
    "desmarañaremos": "desmarañar",
    "desmarañaren": "desmarañar",
    "desmarañares": "desmarañar",
    "desmarañaron": "desmarañar",
    "desmarañará": "desmarañar",
    "desmarañarán": "desmarañar",
    "desmarañarás": "desmarañar",
    "desmarañaré": "desmarañar",
    "desmarañaréis": "desmarañar",
    "desmarañaría": "desmarañar",
    "desmarañaríais": "desmarañar",
    "desmarañaríamos": "desmarañar",
    "desmarañarían": "desmarañar",
    "desmarañarías": "desmarañar",
    "desmarañas": "desmarañar",
    "desmarañase": "desmarañar",
    "desmarañaseis": "desmarañar",
    "desmarañasen": "desmarañar",
    "desmarañases": "desmarañar",
    "desmarañaste": "desmarañar",
    "desmarañasteis": "desmarañar",
    "desmarañe": "desmarañar",
    "desmarañemos": "desmarañar",
    "desmarañen": "desmarañar",
    "desmarañes": "desmarañar",
    "desmaraño": "desmarañar",
    "desmarañábamos": "desmarañar",
    "desmarañáis": "desmarañar",
    "desmarañáramos": "desmarañar",
    "desmarañáremos": "desmarañar",
    "desmarañásemos": "desmarañar",
    "desmarañé": "desmarañar",
    "desmarañéis": "desmarañar",
    "desmarañó": "desmarañar",
    "desmarca": "desmarcar",
    "desmarcaba": "desmarcar",
    "desmarcabais": "desmarcar",
    "desmarcaban": "desmarcar",
    "desmarcabas": "desmarcar",
    "desmarcad": "desmarcar",
    "desmarcada": "desmarcar",
    "desmarcadas": "desmarcar",
    "desmarcado": "desmarcar",
    "desmarcados": "desmarcar",
    "desmarcamos": "desmarcar",
    "desmarcan": "desmarcar",
    "desmarcando": "desmarcar",
    "desmarcara": "desmarcar",
    "desmarcarais": "desmarcar",
    "desmarcaran": "desmarcar",
    "desmarcaras": "desmarcar",
    "desmarcare": "desmarcar",
    "desmarcareis": "desmarcar",
    "desmarcaremos": "desmarcar",
    "desmarcaren": "desmarcar",
    "desmarcares": "desmarcar",
    "desmarcaron": "desmarcar",
    "desmarcará": "desmarcar",
    "desmarcarán": "desmarcar",
    "desmarcarás": "desmarcar",
    "desmarcaré": "desmarcar",
    "desmarcaréis": "desmarcar",
    "desmarcaría": "desmarcar",
    "desmarcaríais": "desmarcar",
    "desmarcaríamos": "desmarcar",
    "desmarcarían": "desmarcar",
    "desmarcarías": "desmarcar",
    "desmarcas": "desmarcar",
    "desmarcase": "desmarcar",
    "desmarcaseis": "desmarcar",
    "desmarcasen": "desmarcar",
    "desmarcases": "desmarcar",
    "desmarcaste": "desmarcar",
    "desmarcasteis": "desmarcar",
    "desmarco": "desmarcar",
    "desmarcábamos": "desmarcar",
    "desmarcáis": "desmarcar",
    "desmarcáramos": "desmarcar",
    "desmarcáremos": "desmarcar",
    "desmarcásemos": "desmarcar",
    "desmarcó": "desmarcar",
    "desmarque": "desmarcar",
    "desmarquemos": "desmarcar",
    "desmarquen": "desmarcar",
    "desmarques": "desmarque",
    "desmarqué": "desmarcar",
    "desmarquéis": "desmarcar",
    "desmata": "desmatar",
    "desmataba": "desmatar",
    "desmatabais": "desmatar",
    "desmataban": "desmatar",
    "desmatabas": "desmatar",
    "desmatad": "desmatar",
    "desmatada": "desmatar",
    "desmatadas": "desmatar",
    "desmatado": "desmatar",
    "desmatados": "desmatar",
    "desmatamos": "desmatar",
    "desmatan": "desmatar",
    "desmatando": "desmatar",
    "desmatara": "desmatar",
    "desmatarais": "desmatar",
    "desmataran": "desmatar",
    "desmataras": "desmatar",
    "desmatare": "desmatar",
    "desmatareis": "desmatar",
    "desmataremos": "desmatar",
    "desmataren": "desmatar",
    "desmatares": "desmatar",
    "desmataron": "desmatar",
    "desmatará": "desmatar",
    "desmatarán": "desmatar",
    "desmatarás": "desmatar",
    "desmataré": "desmatar",
    "desmataréis": "desmatar",
    "desmataría": "desmatar",
    "desmataríais": "desmatar",
    "desmataríamos": "desmatar",
    "desmatarían": "desmatar",
    "desmatarías": "desmatar",
    "desmatas": "desmatar",
    "desmatase": "desmatar",
    "desmataseis": "desmatar",
    "desmatasen": "desmatar",
    "desmatases": "desmatar",
    "desmataste": "desmatar",
    "desmatasteis": "desmatar",
    "desmate": "desmatar",
    "desmatemos": "desmatar",
    "desmaten": "desmatar",
    "desmaterializaciones": "desmaterialización",
    "desmates": "desmatar",
    "desmato": "desmatar",
    "desmatábamos": "desmatar",
    "desmatáis": "desmatar",
    "desmatáramos": "desmatar",
    "desmatáremos": "desmatar",
    "desmatásemos": "desmatar",
    "desmaté": "desmatar",
    "desmatéis": "desmatar",
    "desmató": "desmatar",
    "desmaya": "desmayar",
    "desmayaba": "desmayar",
    "desmayabais": "desmayar",
    "desmayaban": "desmayar",
    "desmayabas": "desmayar",
    "desmayad": "desmayar",
    "desmayada": "desmayar",
    "desmayadas": "desmayar",
    "desmayado": "desmayar",
    "desmayados": "desmayar",
    "desmayamos": "desmayar",
    "desmayan": "desmayar",
    "desmayando": "desmayar",
    "desmayara": "desmayar",
    "desmayarais": "desmayar",
    "desmayaran": "desmayar",
    "desmayaras": "desmayar",
    "desmayare": "desmayar",
    "desmayareis": "desmayar",
    "desmayaremos": "desmayar",
    "desmayaren": "desmayar",
    "desmayares": "desmayar",
    "desmayaron": "desmayar",
    "desmayará": "desmayar",
    "desmayarán": "desmayar",
    "desmayarás": "desmayar",
    "desmayaré": "desmayar",
    "desmayaréis": "desmayar",
    "desmayaría": "desmayar",
    "desmayaríais": "desmayar",
    "desmayaríamos": "desmayar",
    "desmayarían": "desmayar",
    "desmayarías": "desmayar",
    "desmayas": "desmayar",
    "desmayase": "desmayar",
    "desmayaseis": "desmayar",
    "desmayasen": "desmayar",
    "desmayases": "desmayar",
    "desmayaste": "desmayar",
    "desmayasteis": "desmayar",
    "desmaye": "desmayar",
    "desmayemos": "desmayar",
    "desmayen": "desmayar",
    "desmayes": "desmayar",
    "desmayo": "desmayar",
    "desmayos": "desmayo",
    "desmayábamos": "desmayar",
    "desmayáis": "desmayar",
    "desmayáramos": "desmayar",
    "desmayáremos": "desmayar",
    "desmayásemos": "desmayar",
    "desmayé": "desmayar",
    "desmayéis": "desmayar",
    "desmayó": "desmayar",
    "desmazalada": "desmazalado",
    "desmazaladas": "desmazalado",
    "desmazalados": "desmazalado",
    "desmañada": "desmañado",
    "desmañadas": "desmañado",
    "desmañados": "desmañado",
    "desmañas": "desmaña",
    "desmaños": "desmaño",
    "desmedid": "desmedir",
    "desmedida": "desmedir",
    "desmedidas": "desmedir",
    "desmedido": "desmedir",
    "desmedidos": "desmedir",
    "desmedimos": "desmedir",
    "desmediremos": "desmedir",
    "desmedirá": "desmedir",
    "desmedirán": "desmedir",
    "desmedirás": "desmedir",
    "desmediré": "desmedir",
    "desmediréis": "desmedir",
    "desmediría": "desmedir",
    "desmediríais": "desmedir",
    "desmediríamos": "desmedir",
    "desmedirían": "desmedir",
    "desmedirías": "desmedir",
    "desmediste": "desmedir",
    "desmedisteis": "desmedir",
    "desmedra": "desmedrar",
    "desmedraba": "desmedrar",
    "desmedrabais": "desmedrar",
    "desmedraban": "desmedrar",
    "desmedrabas": "desmedrar",
    "desmedrad": "desmedrar",
    "desmedrada": "desmedrar",
    "desmedradas": "desmedrar",
    "desmedrado": "desmedrar",
    "desmedrados": "desmedrar",
    "desmedramos": "desmedrar",
    "desmedran": "desmedrar",
    "desmedrando": "desmedrar",
    "desmedrara": "desmedrar",
    "desmedrarais": "desmedrar",
    "desmedraran": "desmedrar",
    "desmedraras": "desmedrar",
    "desmedrare": "desmedrar",
    "desmedrareis": "desmedrar",
    "desmedraremos": "desmedrar",
    "desmedraren": "desmedrar",
    "desmedrares": "desmedrar",
    "desmedraron": "desmedrar",
    "desmedrará": "desmedrar",
    "desmedrarán": "desmedrar",
    "desmedrarás": "desmedrar",
    "desmedraré": "desmedrar",
    "desmedraréis": "desmedrar",
    "desmedraría": "desmedrar",
    "desmedraríais": "desmedrar",
    "desmedraríamos": "desmedrar",
    "desmedrarían": "desmedrar",
    "desmedrarías": "desmedrar",
    "desmedras": "desmedrar",
    "desmedrase": "desmedrar",
    "desmedraseis": "desmedrar",
    "desmedrasen": "desmedrar",
    "desmedrases": "desmedrar",
    "desmedraste": "desmedrar",
    "desmedrasteis": "desmedrar",
    "desmedre": "desmedrar",
    "desmedremos": "desmedrar",
    "desmedren": "desmedrar",
    "desmedres": "desmedrar",
    "desmedro": "desmedrar",
    "desmedros": "desmedro",
    "desmedrábamos": "desmedrar",
    "desmedráis": "desmedrar",
    "desmedráramos": "desmedrar",
    "desmedráremos": "desmedrar",
    "desmedrásemos": "desmedrar",
    "desmedré": "desmedrar",
    "desmedréis": "desmedrar",
    "desmedró": "desmedrar",
    "desmedí": "desmedir",
    "desmedía": "desmedir",
    "desmedíais": "desmedir",
    "desmedíamos": "desmedir",
    "desmedían": "desmedir",
    "desmedías": "desmedir",
    "desmedís": "desmedir",
    "desmejora": "desmejorar",
    "desmejoraba": "desmejorar",
    "desmejorabais": "desmejorar",
    "desmejoraban": "desmejorar",
    "desmejorabas": "desmejorar",
    "desmejorad": "desmejorar",
    "desmejorada": "desmejorar",
    "desmejoradas": "desmejorar",
    "desmejorado": "desmejorar",
    "desmejorados": "desmejorar",
    "desmejoramientos": "desmejoramiento",
    "desmejoramos": "desmejorar",
    "desmejoran": "desmejorar",
    "desmejorando": "desmejorar",
    "desmejorara": "desmejorar",
    "desmejorarais": "desmejorar",
    "desmejoraran": "desmejorar",
    "desmejoraras": "desmejorar",
    "desmejorare": "desmejorar",
    "desmejorareis": "desmejorar",
    "desmejoraremos": "desmejorar",
    "desmejoraren": "desmejorar",
    "desmejorares": "desmejorar",
    "desmejoraron": "desmejorar",
    "desmejorará": "desmejorar",
    "desmejorarán": "desmejorar",
    "desmejorarás": "desmejorar",
    "desmejoraré": "desmejorar",
    "desmejoraréis": "desmejorar",
    "desmejoraría": "desmejorar",
    "desmejoraríais": "desmejorar",
    "desmejoraríamos": "desmejorar",
    "desmejorarían": "desmejorar",
    "desmejorarías": "desmejorar",
    "desmejoras": "desmejorar",
    "desmejorase": "desmejorar",
    "desmejoraseis": "desmejorar",
    "desmejorasen": "desmejorar",
    "desmejorases": "desmejorar",
    "desmejoraste": "desmejorar",
    "desmejorasteis": "desmejorar",
    "desmejore": "desmejorar",
    "desmejoremos": "desmejorar",
    "desmejoren": "desmejorar",
    "desmejores": "desmejorar",
    "desmejoro": "desmejorar",
    "desmejorábamos": "desmejorar",
    "desmejoráis": "desmejorar",
    "desmejoráramos": "desmejorar",
    "desmejoráremos": "desmejorar",
    "desmejorásemos": "desmejorar",
    "desmejoré": "desmejorar",
    "desmejoréis": "desmejorar",
    "desmejoró": "desmejorar",
    "desmelancolizaciones": "desmelancolización",
    "desmelena": "desmelenar",
    "desmelenaba": "desmelenar",
    "desmelenabais": "desmelenar",
    "desmelenaban": "desmelenar",
    "desmelenabas": "desmelenar",
    "desmelenad": "desmelenar",
    "desmelenada": "desmelenar",
    "desmelenadas": "desmelenar",
    "desmelenado": "desmelenar",
    "desmelenados": "desmelenar",
    "desmelenamos": "desmelenar",
    "desmelenan": "desmelenar",
    "desmelenando": "desmelenar",
    "desmelenara": "desmelenar",
    "desmelenarais": "desmelenar",
    "desmelenaran": "desmelenar",
    "desmelenaras": "desmelenar",
    "desmelenare": "desmelenar",
    "desmelenareis": "desmelenar",
    "desmelenaremos": "desmelenar",
    "desmelenaren": "desmelenar",
    "desmelenares": "desmelenar",
    "desmelenaron": "desmelenar",
    "desmelenará": "desmelenar",
    "desmelenarán": "desmelenar",
    "desmelenarás": "desmelenar",
    "desmelenaré": "desmelenar",
    "desmelenaréis": "desmelenar",
    "desmelenaría": "desmelenar",
    "desmelenaríais": "desmelenar",
    "desmelenaríamos": "desmelenar",
    "desmelenarían": "desmelenar",
    "desmelenarías": "desmelenar",
    "desmelenas": "desmelenar",
    "desmelenase": "desmelenar",
    "desmelenaseis": "desmelenar",
    "desmelenasen": "desmelenar",
    "desmelenases": "desmelenar",
    "desmelenaste": "desmelenar",
    "desmelenasteis": "desmelenar",
    "desmelene": "desmelenar",
    "desmelenemos": "desmelenar",
    "desmelenen": "desmelenar",
    "desmelenes": "desmelenar",
    "desmeleno": "desmelenar",
    "desmelenábamos": "desmelenar",
    "desmelenáis": "desmelenar",
    "desmelenáramos": "desmelenar",
    "desmelenáremos": "desmelenar",
    "desmelenásemos": "desmelenar",
    "desmelené": "desmelenar",
    "desmelenéis": "desmelenar",
    "desmelenó": "desmelenar",
    "desmembraba": "desmembrar",
    "desmembrabais": "desmembrar",
    "desmembraban": "desmembrar",
    "desmembrabas": "desmembrar",
    "desmembraciones": "desmembración",
    "desmembrad": "desmembrar",
    "desmembrada": "desmembrar",
    "desmembradas": "desmembrar",
    "desmembrado": "desmembrar",
    "desmembradora": "desmembrador",
    "desmembradoras": "desmembrador",
    "desmembradores": "desmembrador",
    "desmembrados": "desmembrar",
    "desmembramientos": "desmembramiento",
    "desmembramos": "desmembrar",
    "desmembrando": "desmembrar",
    "desmembrara": "desmembrar",
    "desmembrarais": "desmembrar",
    "desmembraran": "desmembrar",
    "desmembraras": "desmembrar",
    "desmembrare": "desmembrar",
    "desmembrareis": "desmembrar",
    "desmembraremos": "desmembrar",
    "desmembraren": "desmembrar",
    "desmembrares": "desmembrar",
    "desmembraron": "desmembrar",
    "desmembrará": "desmembrar",
    "desmembrarán": "desmembrar",
    "desmembrarás": "desmembrar",
    "desmembraré": "desmembrar",
    "desmembraréis": "desmembrar",
    "desmembraría": "desmembrar",
    "desmembraríais": "desmembrar",
    "desmembraríamos": "desmembrar",
    "desmembrarían": "desmembrar",
    "desmembrarías": "desmembrar",
    "desmembrase": "desmembrar",
    "desmembraseis": "desmembrar",
    "desmembrasen": "desmembrar",
    "desmembrases": "desmembrar",
    "desmembraste": "desmembrar",
    "desmembrasteis": "desmembrar",
    "desmembremos": "desmembrar",
    "desmembrábamos": "desmembrar",
    "desmembráis": "desmembrar",
    "desmembráramos": "desmembrar",
    "desmembráremos": "desmembrar",
    "desmembrásemos": "desmembrar",
    "desmembré": "desmembrar",
    "desmembréis": "desmembrar",
    "desmembró": "desmembrar",
    "desmemoria": "desmemoriar",
    "desmemoriaba": "desmemoriar",
    "desmemoriabais": "desmemoriar",
    "desmemoriaban": "desmemoriar",
    "desmemoriabas": "desmemoriar",
    "desmemoriad": "desmemoriar",
    "desmemoriada": "desmemoriar",
    "desmemoriadas": "desmemoriar",
    "desmemoriado": "desmemoriar",
    "desmemoriados": "desmemoriar",
    "desmemoriamos": "desmemoriar",
    "desmemorian": "desmemoriar",
    "desmemoriando": "desmemoriar",
    "desmemoriara": "desmemoriar",
    "desmemoriarais": "desmemoriar",
    "desmemoriaran": "desmemoriar",
    "desmemoriaras": "desmemoriar",
    "desmemoriare": "desmemoriar",
    "desmemoriareis": "desmemoriar",
    "desmemoriaremos": "desmemoriar",
    "desmemoriaren": "desmemoriar",
    "desmemoriares": "desmemoriar",
    "desmemoriaron": "desmemoriar",
    "desmemoriará": "desmemoriar",
    "desmemoriarán": "desmemoriar",
    "desmemoriarás": "desmemoriar",
    "desmemoriaré": "desmemoriar",
    "desmemoriaréis": "desmemoriar",
    "desmemoriaría": "desmemoriar",
    "desmemoriaríais": "desmemoriar",
    "desmemoriaríamos": "desmemoriar",
    "desmemoriarían": "desmemoriar",
    "desmemoriarías": "desmemoriar",
    "desmemorias": "desmemoriar",
    "desmemoriase": "desmemoriar",
    "desmemoriaseis": "desmemoriar",
    "desmemoriasen": "desmemoriar",
    "desmemoriases": "desmemoriar",
    "desmemoriaste": "desmemoriar",
    "desmemoriasteis": "desmemoriar",
    "desmemorie": "desmemoriar",
    "desmemoriemos": "desmemoriar",
    "desmemorien": "desmemoriar",
    "desmemories": "desmemoriar",
    "desmemorio": "desmemoriar",
    "desmemoriábamos": "desmemoriar",
    "desmemoriáis": "desmemoriar",
    "desmemoriáramos": "desmemoriar",
    "desmemoriáremos": "desmemoriar",
    "desmemoriásemos": "desmemoriar",
    "desmemorié": "desmemoriar",
    "desmemoriéis": "desmemoriar",
    "desmemorió": "desmemoriar",
    "desmengua": "desmenguar",
    "desmenguaba": "desmenguar",
    "desmenguabais": "desmenguar",
    "desmenguaban": "desmenguar",
    "desmenguabas": "desmenguar",
    "desmenguad": "desmenguar",
    "desmenguada": "desmenguar",
    "desmenguadas": "desmenguar",
    "desmenguado": "desmenguar",
    "desmenguados": "desmenguar",
    "desmenguamos": "desmenguar",
    "desmenguan": "desmenguar",
    "desmenguando": "desmenguar",
    "desmenguara": "desmenguar",
    "desmenguarais": "desmenguar",
    "desmenguaran": "desmenguar",
    "desmenguaras": "desmenguar",
    "desmenguare": "desmenguar",
    "desmenguareis": "desmenguar",
    "desmenguaremos": "desmenguar",
    "desmenguaren": "desmenguar",
    "desmenguares": "desmenguar",
    "desmenguaron": "desmenguar",
    "desmenguará": "desmenguar",
    "desmenguarán": "desmenguar",
    "desmenguarás": "desmenguar",
    "desmenguaré": "desmenguar",
    "desmenguaréis": "desmenguar",
    "desmenguaría": "desmenguar",
    "desmenguaríais": "desmenguar",
    "desmenguaríamos": "desmenguar",
    "desmenguarían": "desmenguar",
    "desmenguarías": "desmenguar",
    "desmenguas": "desmenguar",
    "desmenguase": "desmenguar",
    "desmenguaseis": "desmenguar",
    "desmenguasen": "desmenguar",
    "desmenguases": "desmenguar",
    "desmenguaste": "desmenguar",
    "desmenguasteis": "desmenguar",
    "desmenguo": "desmenguar",
    "desmenguábamos": "desmenguar",
    "desmenguáis": "desmenguar",
    "desmenguáramos": "desmenguar",
    "desmenguáremos": "desmenguar",
    "desmenguásemos": "desmenguar",
    "desmenguó": "desmenguar",
    "desmengüe": "desmenguar",
    "desmengüemos": "desmenguar",
    "desmengüen": "desmenguar",
    "desmengües": "desmenguar",
    "desmengüé": "desmenguar",
    "desmengüéis": "desmenguar",
    "desmentid": "desmentir",
    "desmentida": "desmentir",
    "desmentidas": "desmentir",
    "desmentido": "desmentir",
    "desmentidora": "desmentidor",
    "desmentidoras": "desmentidor",
    "desmentidores": "desmentidor",
    "desmentidos": "desmentir",
    "desmentimos": "desmentir",
    "desmentiremos": "desmentir",
    "desmentirá": "desmentir",
    "desmentirán": "desmentir",
    "desmentirás": "desmentir",
    "desmentiré": "desmentir",
    "desmentiréis": "desmentir",
    "desmentiría": "desmentir",
    "desmentiríais": "desmentir",
    "desmentiríamos": "desmentir",
    "desmentirían": "desmentir",
    "desmentirías": "desmentir",
    "desmentiste": "desmentir",
    "desmentisteis": "desmentir",
    "desmentí": "desmentir",
    "desmentía": "desmentir",
    "desmentíais": "desmentir",
    "desmentíamos": "desmentir",
    "desmentían": "desmentir",
    "desmentías": "desmentir",
    "desmentís": "desmentir",
    "desmenuce": "desmenuzar",
    "desmenucemos": "desmenuzar",
    "desmenucen": "desmenuzar",
    "desmenuces": "desmenuzar",
    "desmenucé": "desmenuzar",
    "desmenucéis": "desmenuzar",
    "desmenuza": "desmenuzar",
    "desmenuzaba": "desmenuzar",
    "desmenuzabais": "desmenuzar",
    "desmenuzaban": "desmenuzar",
    "desmenuzabas": "desmenuzar",
    "desmenuzables": "desmenuzable",
    "desmenuzad": "desmenuzar",
    "desmenuzada": "desmenuzar",
    "desmenuzadas": "desmenuzar",
    "desmenuzado": "desmenuzar",
    "desmenuzadora": "desmenuzador",
    "desmenuzadoras": "desmenuzador",
    "desmenuzadores": "desmenuzador",
    "desmenuzados": "desmenuzar",
    "desmenuzamientos": "desmenuzamiento",
    "desmenuzamos": "desmenuzar",
    "desmenuzan": "desmenuzar",
    "desmenuzando": "desmenuzar",
    "desmenuzara": "desmenuzar",
    "desmenuzarais": "desmenuzar",
    "desmenuzaran": "desmenuzar",
    "desmenuzaras": "desmenuzar",
    "desmenuzare": "desmenuzar",
    "desmenuzareis": "desmenuzar",
    "desmenuzaremos": "desmenuzar",
    "desmenuzaren": "desmenuzar",
    "desmenuzares": "desmenuzar",
    "desmenuzaron": "desmenuzar",
    "desmenuzará": "desmenuzar",
    "desmenuzarán": "desmenuzar",
    "desmenuzarás": "desmenuzar",
    "desmenuzaré": "desmenuzar",
    "desmenuzaréis": "desmenuzar",
    "desmenuzaría": "desmenuzar",
    "desmenuzaríais": "desmenuzar",
    "desmenuzaríamos": "desmenuzar",
    "desmenuzarían": "desmenuzar",
    "desmenuzarías": "desmenuzar",
    "desmenuzas": "desmenuzar",
    "desmenuzase": "desmenuzar",
    "desmenuzaseis": "desmenuzar",
    "desmenuzasen": "desmenuzar",
    "desmenuzases": "desmenuzar",
    "desmenuzaste": "desmenuzar",
    "desmenuzasteis": "desmenuzar",
    "desmenuzo": "desmenuzar",
    "desmenuzábamos": "desmenuzar",
    "desmenuzáis": "desmenuzar",
    "desmenuzáramos": "desmenuzar",
    "desmenuzáremos": "desmenuzar",
    "desmenuzásemos": "desmenuzar",
    "desmenuzó": "desmenuzar",
    "desmeolla": "desmeollar",
    "desmeollaba": "desmeollar",
    "desmeollabais": "desmeollar",
    "desmeollaban": "desmeollar",
    "desmeollabas": "desmeollar",
    "desmeollad": "desmeollar",
    "desmeollada": "desmeollar",
    "desmeolladas": "desmeollar",
    "desmeollado": "desmeollar",
    "desmeollados": "desmeollar",
    "desmeollamientos": "desmeollamiento",
    "desmeollamos": "desmeollar",
    "desmeollan": "desmeollar",
    "desmeollando": "desmeollar",
    "desmeollara": "desmeollar",
    "desmeollarais": "desmeollar",
    "desmeollaran": "desmeollar",
    "desmeollaras": "desmeollar",
    "desmeollare": "desmeollar",
    "desmeollareis": "desmeollar",
    "desmeollaremos": "desmeollar",
    "desmeollaren": "desmeollar",
    "desmeollares": "desmeollar",
    "desmeollaron": "desmeollar",
    "desmeollará": "desmeollar",
    "desmeollarán": "desmeollar",
    "desmeollarás": "desmeollar",
    "desmeollaré": "desmeollar",
    "desmeollaréis": "desmeollar",
    "desmeollaría": "desmeollar",
    "desmeollaríais": "desmeollar",
    "desmeollaríamos": "desmeollar",
    "desmeollarían": "desmeollar",
    "desmeollarías": "desmeollar",
    "desmeollas": "desmeollar",
    "desmeollase": "desmeollar",
    "desmeollaseis": "desmeollar",
    "desmeollasen": "desmeollar",
    "desmeollases": "desmeollar",
    "desmeollaste": "desmeollar",
    "desmeollasteis": "desmeollar",
    "desmeolle": "desmeollar",
    "desmeollemos": "desmeollar",
    "desmeollen": "desmeollar",
    "desmeolles": "desmeollar",
    "desmeollo": "desmeollar",
    "desmeollábamos": "desmeollar",
    "desmeolláis": "desmeollar",
    "desmeolláramos": "desmeollar",
    "desmeolláremos": "desmeollar",
    "desmeollásemos": "desmeollar",
    "desmeollé": "desmeollar",
    "desmeolléis": "desmeollar",
    "desmeolló": "desmeollar",
    "desmerece": "desmerecer",
    "desmereced": "desmerecer",
    "desmerecedora": "desmerecedor",
    "desmerecedoras": "desmerecedor",
    "desmerecedores": "desmerecedor",
    "desmerecemos": "desmerecer",
    "desmerecen": "desmerecer",
    "desmereceremos": "desmerecer",
    "desmerecerá": "desmerecer",
    "desmerecerán": "desmerecer",
    "desmerecerás": "desmerecer",
    "desmereceré": "desmerecer",
    "desmereceréis": "desmerecer",
    "desmerecería": "desmerecer",
    "desmereceríais": "desmerecer",
    "desmereceríamos": "desmerecer",
    "desmerecerían": "desmerecer",
    "desmerecerías": "desmerecer",
    "desmereces": "desmerecer",
    "desmerecida": "desmerecer",
    "desmerecidas": "desmerecer",
    "desmerecido": "desmerecer",
    "desmerecidos": "desmerecer",
    "desmereciendo": "desmerecer",
    "desmereciera": "desmerecer",
    "desmerecierais": "desmerecer",
    "desmerecieran": "desmerecer",
    "desmerecieras": "desmerecer",
    "desmereciere": "desmerecer",
    "desmereciereis": "desmerecer",
    "desmerecieren": "desmerecer",
    "desmerecieres": "desmerecer",
    "desmerecieron": "desmerecer",
    "desmereciese": "desmerecer",
    "desmerecieseis": "desmerecer",
    "desmereciesen": "desmerecer",
    "desmerecieses": "desmerecer",
    "desmerecimientos": "desmerecimiento",
    "desmerecimos": "desmerecer",
    "desmereciste": "desmerecer",
    "desmerecisteis": "desmerecer",
    "desmereciéramos": "desmerecer",
    "desmereciéremos": "desmerecer",
    "desmereciésemos": "desmerecer",
    "desmereció": "desmerecer",
    "desmerecéis": "desmerecer",
    "desmerecí": "desmerecer",
    "desmerecía": "desmerecer",
    "desmerecíais": "desmerecer",
    "desmerecíamos": "desmerecer",
    "desmerecían": "desmerecer",
    "desmerecías": "desmerecer",
    "desmerezca": "desmerecer",
    "desmerezcamos": "desmerecer",
    "desmerezcan": "desmerecer",
    "desmerezcas": "desmerecer",
    "desmerezco": "desmerecer",
    "desmerezcáis": "desmerecer",
    "desmesura": "desmesurar",
    "desmesuraba": "desmesurar",
    "desmesurabais": "desmesurar",
    "desmesuraban": "desmesurar",
    "desmesurabas": "desmesurar",
    "desmesurad": "desmesurar",
    "desmesurada": "desmesurar",
    "desmesuradas": "desmesurar",
    "desmesurado": "desmesurar",
    "desmesurados": "desmesurar",
    "desmesuramos": "desmesurar",
    "desmesuran": "desmesurar",
    "desmesurando": "desmesurar",
    "desmesurara": "desmesurar",
    "desmesurarais": "desmesurar",
    "desmesuraran": "desmesurar",
    "desmesuraras": "desmesurar",
    "desmesurare": "desmesurar",
    "desmesurareis": "desmesurar",
    "desmesuraremos": "desmesurar",
    "desmesuraren": "desmesurar",
    "desmesurares": "desmesurar",
    "desmesuraron": "desmesurar",
    "desmesurará": "desmesurar",
    "desmesurarán": "desmesurar",
    "desmesurarás": "desmesurar",
    "desmesuraré": "desmesurar",
    "desmesuraréis": "desmesurar",
    "desmesuraría": "desmesurar",
    "desmesuraríais": "desmesurar",
    "desmesuraríamos": "desmesurar",
    "desmesurarían": "desmesurar",
    "desmesurarías": "desmesurar",
    "desmesuras": "desmesurar",
    "desmesurase": "desmesurar",
    "desmesuraseis": "desmesurar",
    "desmesurasen": "desmesurar",
    "desmesurases": "desmesurar",
    "desmesuraste": "desmesurar",
    "desmesurasteis": "desmesurar",
    "desmesure": "desmesurar",
    "desmesuremos": "desmesurar",
    "desmesuren": "desmesurar",
    "desmesures": "desmesurar",
    "desmesuro": "desmesurar",
    "desmesurábamos": "desmesurar",
    "desmesuráis": "desmesurar",
    "desmesuráramos": "desmesurar",
    "desmesuráremos": "desmesurar",
    "desmesurásemos": "desmesurar",
    "desmesuré": "desmesurar",
    "desmesuréis": "desmesurar",
    "desmesuró": "desmesurar",
    "desmida": "desmedir",
    "desmidamos": "desmedir",
    "desmidan": "desmedir",
    "desmidas": "desmedir",
    "desmide": "desmedir",
    "desmiden": "desmedir",
    "desmides": "desmedir",
    "desmidiendo": "desmedir",
    "desmidiera": "desmedir",
    "desmidierais": "desmedir",
    "desmidieran": "desmedir",
    "desmidieras": "desmedir",
    "desmidiere": "desmedir",
    "desmidiereis": "desmedir",
    "desmidieren": "desmedir",
    "desmidieres": "desmedir",
    "desmidieron": "desmedir",
    "desmidiese": "desmedir",
    "desmidieseis": "desmedir",
    "desmidiesen": "desmedir",
    "desmidieses": "desmedir",
    "desmidiéramos": "desmedir",
    "desmidiéremos": "desmedir",
    "desmidiésemos": "desmedir",
    "desmidió": "desmedir",
    "desmido": "desmedir",
    "desmidáis": "desmedir",
    "desmiembra": "desmembrar",
    "desmiembran": "desmembrar",
    "desmiembras": "desmembrar",
    "desmiembre": "desmembrar",
    "desmiembren": "desmembrar",
    "desmiembres": "desmembrar",
    "desmiembro": "desmembrar",
    "desmienta": "desmentir",
    "desmientan": "desmentir",
    "desmientas": "desmentir",
    "desmiente": "desmentir",
    "desmienten": "desmentir",
    "desmientes": "desmentir",
    "desmiento": "desmentir",
    "desmiga": "desmigar",
    "desmigaba": "desmigar",
    "desmigabais": "desmigar",
    "desmigaban": "desmigar",
    "desmigabas": "desmigar",
    "desmigad": "desmigar",
    "desmigada": "desmigar",
    "desmigadas": "desmigar",
    "desmigado": "desmigar",
    "desmigados": "desmigar",
    "desmigaja": "desmigajar",
    "desmigajaba": "desmigajar",
    "desmigajabais": "desmigajar",
    "desmigajaban": "desmigajar",
    "desmigajabas": "desmigajar",
    "desmigajad": "desmigajar",
    "desmigajada": "desmigajar",
    "desmigajadas": "desmigajar",
    "desmigajado": "desmigajar",
    "desmigajados": "desmigajar",
    "desmigajamos": "desmigajar",
    "desmigajan": "desmigajar",
    "desmigajando": "desmigajar",
    "desmigajara": "desmigajar",
    "desmigajarais": "desmigajar",
    "desmigajaran": "desmigajar",
    "desmigajaras": "desmigajar",
    "desmigajare": "desmigajar",
    "desmigajareis": "desmigajar",
    "desmigajaremos": "desmigajar",
    "desmigajaren": "desmigajar",
    "desmigajares": "desmigajar",
    "desmigajaron": "desmigajar",
    "desmigajará": "desmigajar",
    "desmigajarán": "desmigajar",
    "desmigajarás": "desmigajar",
    "desmigajaré": "desmigajar",
    "desmigajaréis": "desmigajar",
    "desmigajaría": "desmigajar",
    "desmigajaríais": "desmigajar",
    "desmigajaríamos": "desmigajar",
    "desmigajarían": "desmigajar",
    "desmigajarías": "desmigajar",
    "desmigajas": "desmigajar",
    "desmigajase": "desmigajar",
    "desmigajaseis": "desmigajar",
    "desmigajasen": "desmigajar",
    "desmigajases": "desmigajar",
    "desmigajaste": "desmigajar",
    "desmigajasteis": "desmigajar",
    "desmigaje": "desmigajar",
    "desmigajemos": "desmigajar",
    "desmigajen": "desmigajar",
    "desmigajes": "desmigajar",
    "desmigajo": "desmigajar",
    "desmigajábamos": "desmigajar",
    "desmigajáis": "desmigajar",
    "desmigajáramos": "desmigajar",
    "desmigajáremos": "desmigajar",
    "desmigajásemos": "desmigajar",
    "desmigajé": "desmigajar",
    "desmigajéis": "desmigajar",
    "desmigajó": "desmigajar",
    "desmigamos": "desmigar",
    "desmigan": "desmigar",
    "desmigando": "desmigar",
    "desmigara": "desmigar",
    "desmigarais": "desmigar",
    "desmigaran": "desmigar",
    "desmigaras": "desmigar",
    "desmigare": "desmigar",
    "desmigareis": "desmigar",
    "desmigaremos": "desmigar",
    "desmigaren": "desmigar",
    "desmigares": "desmigar",
    "desmigaron": "desmigar",
    "desmigará": "desmigar",
    "desmigarán": "desmigar",
    "desmigarás": "desmigar",
    "desmigaré": "desmigar",
    "desmigaréis": "desmigar",
    "desmigaría": "desmigar",
    "desmigaríais": "desmigar",
    "desmigaríamos": "desmigar",
    "desmigarían": "desmigar",
    "desmigarías": "desmigar",
    "desmigas": "desmigar",
    "desmigase": "desmigar",
    "desmigaseis": "desmigar",
    "desmigasen": "desmigar",
    "desmigases": "desmigar",
    "desmigaste": "desmigar",
    "desmigasteis": "desmigar",
    "desmigo": "desmigar",
    "desmigue": "desmigar",
    "desmiguemos": "desmigar",
    "desmiguen": "desmigar",
    "desmigues": "desmigar",
    "desmigué": "desmigar",
    "desmiguéis": "desmigar",
    "desmigábamos": "desmigar",
    "desmigáis": "desmigar",
    "desmigáramos": "desmigar",
    "desmigáremos": "desmigar",
    "desmigásemos": "desmigar",
    "desmigó": "desmigar",
    "desmilitarice": "desmilitarizar",
    "desmilitaricemos": "desmilitarizar",
    "desmilitaricen": "desmilitarizar",
    "desmilitarices": "desmilitarizar",
    "desmilitaricé": "desmilitarizar",
    "desmilitaricéis": "desmilitarizar",
    "desmilitariza": "desmilitarizar",
    "desmilitarizaba": "desmilitarizar",
    "desmilitarizabais": "desmilitarizar",
    "desmilitarizaban": "desmilitarizar",
    "desmilitarizabas": "desmilitarizar",
    "desmilitarizaciones": "desmilitarización",
    "desmilitarizad": "desmilitarizar",
    "desmilitarizada": "desmilitarizar",
    "desmilitarizadas": "desmilitarizar",
    "desmilitarizado": "desmilitarizar",
    "desmilitarizados": "desmilitarizar",
    "desmilitarizamos": "desmilitarizar",
    "desmilitarizan": "desmilitarizar",
    "desmilitarizando": "desmilitarizar",
    "desmilitarizara": "desmilitarizar",
    "desmilitarizarais": "desmilitarizar",
    "desmilitarizaran": "desmilitarizar",
    "desmilitarizaras": "desmilitarizar",
    "desmilitarizare": "desmilitarizar",
    "desmilitarizareis": "desmilitarizar",
    "desmilitarizaremos": "desmilitarizar",
    "desmilitarizaren": "desmilitarizar",
    "desmilitarizares": "desmilitarizar",
    "desmilitarizaron": "desmilitarizar",
    "desmilitarizará": "desmilitarizar",
    "desmilitarizarán": "desmilitarizar",
    "desmilitarizarás": "desmilitarizar",
    "desmilitarizaré": "desmilitarizar",
    "desmilitarizaréis": "desmilitarizar",
    "desmilitarizaría": "desmilitarizar",
    "desmilitarizaríais": "desmilitarizar",
    "desmilitarizaríamos": "desmilitarizar",
    "desmilitarizarían": "desmilitarizar",
    "desmilitarizarías": "desmilitarizar",
    "desmilitarizas": "desmilitarizar",
    "desmilitarizase": "desmilitarizar",
    "desmilitarizaseis": "desmilitarizar",
    "desmilitarizasen": "desmilitarizar",
    "desmilitarizases": "desmilitarizar",
    "desmilitarizaste": "desmilitarizar",
    "desmilitarizasteis": "desmilitarizar",
    "desmilitarizo": "desmilitarizar",
    "desmilitarizábamos": "desmilitarizar",
    "desmilitarizáis": "desmilitarizar",
    "desmilitarizáramos": "desmilitarizar",
    "desmilitarizáremos": "desmilitarizar",
    "desmilitarizásemos": "desmilitarizar",
    "desmilitarizó": "desmilitarizar",
    "desmineralice": "desmineralizar",
    "desmineralicemos": "desmineralizar",
    "desmineralicen": "desmineralizar",
    "desmineralices": "desmineralizar",
    "desmineralicé": "desmineralizar",
    "desmineralicéis": "desmineralizar",
    "desmineraliza": "desmineralizar",
    "desmineralizaba": "desmineralizar",
    "desmineralizabais": "desmineralizar",
    "desmineralizaban": "desmineralizar",
    "desmineralizabas": "desmineralizar",
    "desmineralizaciones": "desmineralización",
    "desmineralizad": "desmineralizar",
    "desmineralizada": "desmineralizar",
    "desmineralizadas": "desmineralizar",
    "desmineralizado": "desmineralizar",
    "desmineralizados": "desmineralizar",
    "desmineralizamos": "desmineralizar",
    "desmineralizan": "desmineralizar",
    "desmineralizando": "desmineralizar",
    "desmineralizara": "desmineralizar",
    "desmineralizarais": "desmineralizar",
    "desmineralizaran": "desmineralizar",
    "desmineralizaras": "desmineralizar",
    "desmineralizare": "desmineralizar",
    "desmineralizareis": "desmineralizar",
    "desmineralizaremos": "desmineralizar",
    "desmineralizaren": "desmineralizar",
    "desmineralizares": "desmineralizar",
    "desmineralizaron": "desmineralizar",
    "desmineralizará": "desmineralizar",
    "desmineralizarán": "desmineralizar",
    "desmineralizarás": "desmineralizar",
    "desmineralizaré": "desmineralizar",
    "desmineralizaréis": "desmineralizar",
    "desmineralizaría": "desmineralizar",
    "desmineralizaríais": "desmineralizar",
    "desmineralizaríamos": "desmineralizar",
    "desmineralizarían": "desmineralizar",
    "desmineralizarías": "desmineralizar",
    "desmineralizas": "desmineralizar",
    "desmineralizase": "desmineralizar",
    "desmineralizaseis": "desmineralizar",
    "desmineralizasen": "desmineralizar",
    "desmineralizases": "desmineralizar",
    "desmineralizaste": "desmineralizar",
    "desmineralizasteis": "desmineralizar",
    "desmineralizo": "desmineralizar",
    "desmineralizábamos": "desmineralizar",
    "desmineralizáis": "desmineralizar",
    "desmineralizáramos": "desmineralizar",
    "desmineralizáremos": "desmineralizar",
    "desmineralizásemos": "desmineralizar",
    "desmineralizó": "desmineralizar",
    "desmintamos": "desmentir",
    "desmintiendo": "desmentir",
    "desmintiera": "desmentir",
    "desmintierais": "desmentir",
    "desmintieran": "desmentir",
    "desmintieras": "desmentir",
    "desmintiere": "desmentir",
    "desmintiereis": "desmentir",
    "desmintieren": "desmentir",
    "desmintieres": "desmentir",
    "desmintieron": "desmentir",
    "desmintiese": "desmentir",
    "desmintieseis": "desmentir",
    "desmintiesen": "desmentir",
    "desmintieses": "desmentir",
    "desmintiéramos": "desmentir",
    "desmintiéremos": "desmentir",
    "desmintiésemos": "desmentir",
    "desmintió": "desmentir",
    "desmintáis": "desmentir",
    "desmirriada": "desmirriado",
    "desmirriadas": "desmirriado",
    "desmirriados": "desmirriado",
    "desmitifica": "desmitificar",
    "desmitificaba": "desmitificar",
    "desmitificabais": "desmitificar",
    "desmitificaban": "desmitificar",
    "desmitificabas": "desmitificar",
    "desmitificaciones": "desmitificación",
    "desmitificad": "desmitificar",
    "desmitificada": "desmitificar",
    "desmitificadas": "desmitificar",
    "desmitificado": "desmitificar",
    "desmitificadora": "desmitificador",
    "desmitificadoras": "desmitificador",
    "desmitificadores": "desmitificador",
    "desmitificados": "desmitificar",
    "desmitificamos": "desmitificar",
    "desmitifican": "desmitificar",
    "desmitificando": "desmitificar",
    "desmitificara": "desmitificar",
    "desmitificarais": "desmitificar",
    "desmitificaran": "desmitificar",
    "desmitificaras": "desmitificar",
    "desmitificare": "desmitificar",
    "desmitificareis": "desmitificar",
    "desmitificaremos": "desmitificar",
    "desmitificaren": "desmitificar",
    "desmitificares": "desmitificar",
    "desmitificaron": "desmitificar",
    "desmitificará": "desmitificar",
    "desmitificarán": "desmitificar",
    "desmitificarás": "desmitificar",
    "desmitificaré": "desmitificar",
    "desmitificaréis": "desmitificar",
    "desmitificaría": "desmitificar",
    "desmitificaríais": "desmitificar",
    "desmitificaríamos": "desmitificar",
    "desmitificarían": "desmitificar",
    "desmitificarías": "desmitificar",
    "desmitificas": "desmitificar",
    "desmitificase": "desmitificar",
    "desmitificaseis": "desmitificar",
    "desmitificasen": "desmitificar",
    "desmitificases": "desmitificar",
    "desmitificaste": "desmitificar",
    "desmitificasteis": "desmitificar",
    "desmitifico": "desmitificar",
    "desmitificábamos": "desmitificar",
    "desmitificáis": "desmitificar",
    "desmitificáramos": "desmitificar",
    "desmitificáremos": "desmitificar",
    "desmitificásemos": "desmitificar",
    "desmitificó": "desmitificar",
    "desmitifique": "desmitificar",
    "desmitifiquemos": "desmitificar",
    "desmitifiquen": "desmitificar",
    "desmitifiques": "desmitificar",
    "desmitifiqué": "desmitificar",
    "desmitifiquéis": "desmitificar",
    "desmitologizaciones": "desmitologización",
    "desmocha": "desmochar",
    "desmochaba": "desmochar",
    "desmochabais": "desmochar",
    "desmochaban": "desmochar",
    "desmochabas": "desmochar",
    "desmochad": "desmochar",
    "desmochada": "desmochar",
    "desmochadas": "desmochar",
    "desmochado": "desmochar",
    "desmochados": "desmochar",
    "desmochaduras": "desmochadura",
    "desmochamos": "desmochar",
    "desmochan": "desmochar",
    "desmochando": "desmochar",
    "desmochara": "desmochar",
    "desmocharais": "desmochar",
    "desmocharan": "desmochar",
    "desmocharas": "desmochar",
    "desmochare": "desmochar",
    "desmochareis": "desmochar",
    "desmocharemos": "desmochar",
    "desmocharen": "desmochar",
    "desmochares": "desmochar",
    "desmocharon": "desmochar",
    "desmochará": "desmochar",
    "desmocharán": "desmochar",
    "desmocharás": "desmochar",
    "desmocharé": "desmochar",
    "desmocharéis": "desmochar",
    "desmocharía": "desmochar",
    "desmocharíais": "desmochar",
    "desmocharíamos": "desmochar",
    "desmocharían": "desmochar",
    "desmocharías": "desmochar",
    "desmochas": "desmochar",
    "desmochase": "desmochar",
    "desmochaseis": "desmochar",
    "desmochasen": "desmochar",
    "desmochases": "desmochar",
    "desmochaste": "desmochar",
    "desmochasteis": "desmochar",
    "desmoche": "desmochar",
    "desmochemos": "desmochar",
    "desmochen": "desmochar",
    "desmoches": "desmoche",
    "desmocho": "desmochar",
    "desmochos": "desmocho",
    "desmochábamos": "desmochar",
    "desmocháis": "desmochar",
    "desmocháramos": "desmochar",
    "desmocháremos": "desmochar",
    "desmochásemos": "desmochar",
    "desmoché": "desmochar",
    "desmochéis": "desmochar",
    "desmochó": "desmochar",
    "desmocráneos": "desmocráneo",
    "desmodontales": "desmodontal",
    "desmodrómica": "desmodrómico",
    "desmodrómicas": "desmodrómico",
    "desmodrómicos": "desmodrómico",
    "desmodulaciones": "desmodulación",
    "desmoduladores": "desmodulador",
    "desmoga": "desmogar",
    "desmogaba": "desmogar",
    "desmogabais": "desmogar",
    "desmogaban": "desmogar",
    "desmogabas": "desmogar",
    "desmogad": "desmogar",
    "desmogada": "desmogar",
    "desmogadas": "desmogar",
    "desmogado": "desmogar",
    "desmogados": "desmogar",
    "desmogamos": "desmogar",
    "desmogan": "desmogar",
    "desmogando": "desmogar",
    "desmogara": "desmogar",
    "desmogarais": "desmogar",
    "desmogaran": "desmogar",
    "desmogaras": "desmogar",
    "desmogare": "desmogar",
    "desmogareis": "desmogar",
    "desmogaremos": "desmogar",
    "desmogaren": "desmogar",
    "desmogares": "desmogar",
    "desmogaron": "desmogar",
    "desmogará": "desmogar",
    "desmogarán": "desmogar",
    "desmogarás": "desmogar",
    "desmogaré": "desmogar",
    "desmogaréis": "desmogar",
    "desmogaría": "desmogar",
    "desmogaríais": "desmogar",
    "desmogaríamos": "desmogar",
    "desmogarían": "desmogar",
    "desmogarías": "desmogar",
    "desmogas": "desmogar",
    "desmogase": "desmogar",
    "desmogaseis": "desmogar",
    "desmogasen": "desmogar",
    "desmogases": "desmogar",
    "desmogaste": "desmogar",
    "desmogasteis": "desmogar",
    "desmogleínas": "desmogleína",
    "desmogo": "desmogar",
    "desmogue": "desmogar",
    "desmoguemos": "desmogar",
    "desmoguen": "desmogar",
    "desmogues": "desmogue",
    "desmogué": "desmogar",
    "desmoguéis": "desmogar",
    "desmogábamos": "desmogar",
    "desmogáis": "desmogar",
    "desmogáramos": "desmogar",
    "desmogáremos": "desmogar",
    "desmogásemos": "desmogar",
    "desmogó": "desmogar",
    "desmolada": "desmolado",
    "desmoladas": "desmolado",
    "desmolados": "desmolado",
    "desmolamos": "desmoler",
    "desmolda": "desmoldar",
    "desmoldaba": "desmoldar",
    "desmoldabais": "desmoldar",
    "desmoldaban": "desmoldar",
    "desmoldabas": "desmoldar",
    "desmoldad": "desmoldar",
    "desmoldada": "desmoldar",
    "desmoldadas": "desmoldar",
    "desmoldado": "desmoldar",
    "desmoldados": "desmoldar",
    "desmoldamos": "desmoldar",
    "desmoldan": "desmoldar",
    "desmoldando": "desmoldar",
    "desmoldara": "desmoldar",
    "desmoldarais": "desmoldar",
    "desmoldaran": "desmoldar",
    "desmoldaras": "desmoldar",
    "desmoldare": "desmoldar",
    "desmoldareis": "desmoldar",
    "desmoldaremos": "desmoldar",
    "desmoldaren": "desmoldar",
    "desmoldares": "desmoldar",
    "desmoldaron": "desmoldar",
    "desmoldará": "desmoldar",
    "desmoldarán": "desmoldar",
    "desmoldarás": "desmoldar",
    "desmoldaré": "desmoldar",
    "desmoldaréis": "desmoldar",
    "desmoldaría": "desmoldar",
    "desmoldaríais": "desmoldar",
    "desmoldaríamos": "desmoldar",
    "desmoldarían": "desmoldar",
    "desmoldarías": "desmoldar",
    "desmoldas": "desmoldar",
    "desmoldase": "desmoldar",
    "desmoldaseis": "desmoldar",
    "desmoldasen": "desmoldar",
    "desmoldases": "desmoldar",
    "desmoldaste": "desmoldar",
    "desmoldasteis": "desmoldar",
    "desmolde": "desmoldar",
    "desmoldeadora": "desmoldeador",
    "desmoldeadoras": "desmoldeador",
    "desmoldeadores": "desmoldeador",
    "desmoldeados": "desmoldeado",
    "desmoldemos": "desmoldar",
    "desmolden": "desmoldar",
    "desmoldeos": "desmoldeo",
    "desmoldes": "desmoldar",
    "desmoldo": "desmoldar",
    "desmoldábamos": "desmoldar",
    "desmoldáis": "desmoldar",
    "desmoldáramos": "desmoldar",
    "desmoldáremos": "desmoldar",
    "desmoldásemos": "desmoldar",
    "desmoldé": "desmoldar",
    "desmoldéis": "desmoldar",
    "desmoldó": "desmoldar",
    "desmoled": "desmoler",
    "desmolemos": "desmoler",
    "desmoleremos": "desmoler",
    "desmolerá": "desmoler",
    "desmolerán": "desmoler",
    "desmolerás": "desmoler",
    "desmoleré": "desmoler",
    "desmoleréis": "desmoler",
    "desmolería": "desmoler",
    "desmoleríais": "desmoler",
    "desmoleríamos": "desmoler",
    "desmolerían": "desmoler",
    "desmolerías": "desmoler",
    "desmolida": "desmoler",
    "desmolidas": "desmoler",
    "desmolido": "desmoler",
    "desmolidos": "desmoler",
    "desmoliendo": "desmoler",
    "desmoliera": "desmoler",
    "desmolierais": "desmoler",
    "desmolieran": "desmoler",
    "desmolieras": "desmoler",
    "desmoliere": "desmoler",
    "desmoliereis": "desmoler",
    "desmolieren": "desmoler",
    "desmolieres": "desmoler",
    "desmolieron": "desmoler",
    "desmoliese": "desmoler",
    "desmolieseis": "desmoler",
    "desmoliesen": "desmoler",
    "desmolieses": "desmoler",
    "desmolimos": "desmoler",
    "desmoliste": "desmoler",
    "desmolisteis": "desmoler",
    "desmoliéramos": "desmoler",
    "desmoliéremos": "desmoler",
    "desmoliésemos": "desmoler",
    "desmolió": "desmoler",
    "desmologías": "desmología",
    "desmoláis": "desmoler",
    "desmoléis": "desmoler",
    "desmolí": "desmoler",
    "desmolía": "desmoler",
    "desmolíais": "desmoler",
    "desmolíamos": "desmoler",
    "desmolían": "desmoler",
    "desmolías": "desmoler",
    "desmonetice": "desmonetizar",
    "desmoneticemos": "desmonetizar",
    "desmoneticen": "desmonetizar",
    "desmonetices": "desmonetizar",
    "desmoneticé": "desmonetizar",
    "desmoneticéis": "desmonetizar",
    "desmonetiza": "desmonetizar",
    "desmonetizaba": "desmonetizar",
    "desmonetizabais": "desmonetizar",
    "desmonetizaban": "desmonetizar",
    "desmonetizabas": "desmonetizar",
    "desmonetizaciones": "desmonetización",
    "desmonetizad": "desmonetizar",
    "desmonetizada": "desmonetizar",
    "desmonetizadas": "desmonetizar",
    "desmonetizado": "desmonetizar",
    "desmonetizados": "desmonetizar",
    "desmonetizamos": "desmonetizar",
    "desmonetizan": "desmonetizar",
    "desmonetizando": "desmonetizar",
    "desmonetizara": "desmonetizar",
    "desmonetizarais": "desmonetizar",
    "desmonetizaran": "desmonetizar",
    "desmonetizaras": "desmonetizar",
    "desmonetizare": "desmonetizar",
    "desmonetizareis": "desmonetizar",
    "desmonetizaremos": "desmonetizar",
    "desmonetizaren": "desmonetizar",
    "desmonetizares": "desmonetizar",
    "desmonetizaron": "desmonetizar",
    "desmonetizará": "desmonetizar",
    "desmonetizarán": "desmonetizar",
    "desmonetizarás": "desmonetizar",
    "desmonetizaré": "desmonetizar",
    "desmonetizaréis": "desmonetizar",
    "desmonetizaría": "desmonetizar",
    "desmonetizaríais": "desmonetizar",
    "desmonetizaríamos": "desmonetizar",
    "desmonetizarían": "desmonetizar",
    "desmonetizarías": "desmonetizar",
    "desmonetizas": "desmonetizar",
    "desmonetizase": "desmonetizar",
    "desmonetizaseis": "desmonetizar",
    "desmonetizasen": "desmonetizar",
    "desmonetizases": "desmonetizar",
    "desmonetizaste": "desmonetizar",
    "desmonetizasteis": "desmonetizar",
    "desmonetizo": "desmonetizar",
    "desmonetizábamos": "desmonetizar",
    "desmonetizáis": "desmonetizar",
    "desmonetizáramos": "desmonetizar",
    "desmonetizáremos": "desmonetizar",
    "desmonetizásemos": "desmonetizar",
    "desmonetizó": "desmonetizar",
    "desmonta": "desmontar",
    "desmontaba": "desmontar",
    "desmontabais": "desmontar",
    "desmontaban": "desmontar",
    "desmontabas": "desmontar",
    "desmontables": "desmontable",
    "desmontad": "desmontar",
    "desmontada": "desmontar",
    "desmontadas": "desmontar",
    "desmontado": "desmontar",
    "desmontadores": "desmontador",
    "desmontados": "desmontar",
    "desmontaduras": "desmontadura",
    "desmontajes": "desmontaje",
    "desmontamos": "desmontar",
    "desmontan": "desmontar",
    "desmontando": "desmontar",
    "desmontara": "desmontar",
    "desmontarais": "desmontar",
    "desmontaran": "desmontar",
    "desmontaras": "desmontar",
    "desmontare": "desmontar",
    "desmontareis": "desmontar",
    "desmontaremos": "desmontar",
    "desmontaren": "desmontar",
    "desmontares": "desmontar",
    "desmontaron": "desmontar",
    "desmontará": "desmontar",
    "desmontarán": "desmontar",
    "desmontarás": "desmontar",
    "desmontaré": "desmontar",
    "desmontaréis": "desmontar",
    "desmontaría": "desmontar",
    "desmontaríais": "desmontar",
    "desmontaríamos": "desmontar",
    "desmontarían": "desmontar",
    "desmontarías": "desmontar",
    "desmontas": "desmontar",
    "desmontase": "desmontar",
    "desmontaseis": "desmontar",
    "desmontasen": "desmontar",
    "desmontases": "desmontar",
    "desmontaste": "desmontar",
    "desmontasteis": "desmontar",
    "desmonte": "desmontar",
    "desmontemos": "desmontar",
    "desmonten": "desmontar",
    "desmontes": "desmonte",
    "desmonto": "desmontar",
    "desmontábamos": "desmontar",
    "desmontáis": "desmontar",
    "desmontáramos": "desmontar",
    "desmontáremos": "desmontar",
    "desmontásemos": "desmontar",
    "desmonté": "desmontar",
    "desmontéis": "desmontar",
    "desmontó": "desmontar",
    "desmoplaquinas": "desmoplaquina",
    "desmoralice": "desmoralizar",
    "desmoralicemos": "desmoralizar",
    "desmoralicen": "desmoralizar",
    "desmoralices": "desmoralizar",
    "desmoralicé": "desmoralizar",
    "desmoralicéis": "desmoralizar",
    "desmoraliza": "desmoralizar",
    "desmoralizaba": "desmoralizar",
    "desmoralizabais": "desmoralizar",
    "desmoralizaban": "desmoralizar",
    "desmoralizabas": "desmoralizar",
    "desmoralizaciones": "desmoralización",
    "desmoralizad": "desmoralizar",
    "desmoralizada": "desmoralizar",
    "desmoralizadas": "desmoralizar",
    "desmoralizado": "desmoralizar",
    "desmoralizadora": "desmoralizador",
    "desmoralizadoras": "desmoralizador",
    "desmoralizadores": "desmoralizador",
    "desmoralizados": "desmoralizar",
    "desmoralizamos": "desmoralizar",
    "desmoralizan": "desmoralizar",
    "desmoralizando": "desmoralizar",
    "desmoralizara": "desmoralizar",
    "desmoralizarais": "desmoralizar",
    "desmoralizaran": "desmoralizar",
    "desmoralizaras": "desmoralizar",
    "desmoralizare": "desmoralizar",
    "desmoralizareis": "desmoralizar",
    "desmoralizaremos": "desmoralizar",
    "desmoralizaren": "desmoralizar",
    "desmoralizares": "desmoralizar",
    "desmoralizaron": "desmoralizar",
    "desmoralizará": "desmoralizar",
    "desmoralizarán": "desmoralizar",
    "desmoralizarás": "desmoralizar",
    "desmoralizaré": "desmoralizar",
    "desmoralizaréis": "desmoralizar",
    "desmoralizaría": "desmoralizar",
    "desmoralizaríais": "desmoralizar",
    "desmoralizaríamos": "desmoralizar",
    "desmoralizarían": "desmoralizar",
    "desmoralizarías": "desmoralizar",
    "desmoralizas": "desmoralizar",
    "desmoralizase": "desmoralizar",
    "desmoralizaseis": "desmoralizar",
    "desmoralizasen": "desmoralizar",
    "desmoralizases": "desmoralizar",
    "desmoralizaste": "desmoralizar",
    "desmoralizasteis": "desmoralizar",
    "desmoralizo": "desmoralizar",
    "desmoralizábamos": "desmoralizar",
    "desmoralizáis": "desmoralizar",
    "desmoralizáramos": "desmoralizar",
    "desmoralizáremos": "desmoralizar",
    "desmoralizásemos": "desmoralizar",
    "desmoralizó": "desmoralizar",
    "desmorece": "desmorecer",
    "desmoreced": "desmorecer",
    "desmorecemos": "desmorecer",
    "desmorecen": "desmorecer",
    "desmoreceremos": "desmorecer",
    "desmorecerá": "desmorecer",
    "desmorecerán": "desmorecer",
    "desmorecerás": "desmorecer",
    "desmoreceré": "desmorecer",
    "desmoreceréis": "desmorecer",
    "desmorecería": "desmorecer",
    "desmoreceríais": "desmorecer",
    "desmoreceríamos": "desmorecer",
    "desmorecerían": "desmorecer",
    "desmorecerías": "desmorecer",
    "desmoreces": "desmorecer",
    "desmorecida": "desmorecer",
    "desmorecidas": "desmorecer",
    "desmorecido": "desmorecer",
    "desmorecidos": "desmorecer",
    "desmoreciendo": "desmorecer",
    "desmoreciera": "desmorecer",
    "desmorecierais": "desmorecer",
    "desmorecieran": "desmorecer",
    "desmorecieras": "desmorecer",
    "desmoreciere": "desmorecer",
    "desmoreciereis": "desmorecer",
    "desmorecieren": "desmorecer",
    "desmorecieres": "desmorecer",
    "desmorecieron": "desmorecer",
    "desmoreciese": "desmorecer",
    "desmorecieseis": "desmorecer",
    "desmoreciesen": "desmorecer",
    "desmorecieses": "desmorecer",
    "desmorecimos": "desmorecer",
    "desmoreciste": "desmorecer",
    "desmorecisteis": "desmorecer",
    "desmoreciéramos": "desmorecer",
    "desmoreciéremos": "desmorecer",
    "desmoreciésemos": "desmorecer",
    "desmoreció": "desmorecer",
    "desmorecéis": "desmorecer",
    "desmorecí": "desmorecer",
    "desmorecía": "desmorecer",
    "desmorecíais": "desmorecer",
    "desmorecíamos": "desmorecer",
    "desmorecían": "desmorecer",
    "desmorecías": "desmorecer",
    "desmorezca": "desmorecer",
    "desmorezcamos": "desmorecer",
    "desmorezcan": "desmorecer",
    "desmorezcas": "desmorecer",
    "desmorezco": "desmorecer",
    "desmorezcáis": "desmorecer",
    "desmorona": "desmoronar",
    "desmoronaba": "desmoronar",
    "desmoronabais": "desmoronar",
    "desmoronaban": "desmoronar",
    "desmoronabas": "desmoronar",
    "desmoronad": "desmoronar",
    "desmoronada": "desmoronar",
    "desmoronadas": "desmoronar",
    "desmoronadiza": "desmoronadizo",
    "desmoronadizas": "desmoronadizo",
    "desmoronadizos": "desmoronadizo",
    "desmoronado": "desmoronar",
    "desmoronados": "desmoronar",
    "desmoronamientos": "desmoronamiento",
    "desmoronamos": "desmoronar",
    "desmoronan": "desmoronar",
    "desmoronando": "desmoronar",
    "desmoronara": "desmoronar",
    "desmoronarais": "desmoronar",
    "desmoronaran": "desmoronar",
    "desmoronaras": "desmoronar",
    "desmoronare": "desmoronar",
    "desmoronareis": "desmoronar",
    "desmoronaremos": "desmoronar",
    "desmoronaren": "desmoronar",
    "desmoronares": "desmoronar",
    "desmoronaron": "desmoronar",
    "desmoronará": "desmoronar",
    "desmoronarán": "desmoronar",
    "desmoronarás": "desmoronar",
    "desmoronaré": "desmoronar",
    "desmoronaréis": "desmoronar",
    "desmoronaría": "desmoronar",
    "desmoronaríais": "desmoronar",
    "desmoronaríamos": "desmoronar",
    "desmoronarían": "desmoronar",
    "desmoronarías": "desmoronar",
    "desmoronas": "desmoronar",
    "desmoronase": "desmoronar",
    "desmoronaseis": "desmoronar",
    "desmoronasen": "desmoronar",
    "desmoronases": "desmoronar",
    "desmoronaste": "desmoronar",
    "desmoronasteis": "desmoronar",
    "desmorone": "desmoronar",
    "desmoronemos": "desmoronar",
    "desmoronen": "desmoronar",
    "desmorones": "desmoronar",
    "desmorono": "desmoronar",
    "desmoronábamos": "desmoronar",
    "desmoronáis": "desmoronar",
    "desmoronáramos": "desmoronar",
    "desmoronáremos": "desmoronar",
    "desmoronásemos": "desmoronar",
    "desmoroné": "desmoronar",
    "desmoronéis": "desmoronar",
    "desmoronó": "desmoronar",
    "desmosomas": "desmosoma",
    "desmota": "desmotar",
    "desmotaba": "desmotar",
    "desmotabais": "desmotar",
    "desmotaban": "desmotar",
    "desmotabas": "desmotar",
    "desmotad": "desmotar",
    "desmotada": "desmotar",
    "desmotadas": "desmotar",
    "desmotado": "desmotar",
    "desmotadoras": "desmotadora",
    "desmotados": "desmotar",
    "desmotamos": "desmotar",
    "desmotan": "desmotar",
    "desmotando": "desmotar",
    "desmotara": "desmotar",
    "desmotarais": "desmotar",
    "desmotaran": "desmotar",
    "desmotaras": "desmotar",
    "desmotare": "desmotar",
    "desmotareis": "desmotar",
    "desmotaremos": "desmotar",
    "desmotaren": "desmotar",
    "desmotares": "desmotar",
    "desmotaron": "desmotar",
    "desmotará": "desmotar",
    "desmotarán": "desmotar",
    "desmotarás": "desmotar",
    "desmotaré": "desmotar",
    "desmotaréis": "desmotar",
    "desmotaría": "desmotar",
    "desmotaríais": "desmotar",
    "desmotaríamos": "desmotar",
    "desmotarían": "desmotar",
    "desmotarías": "desmotar",
    "desmotas": "desmotar",
    "desmotase": "desmotar",
    "desmotaseis": "desmotar",
    "desmotasen": "desmotar",
    "desmotases": "desmotar",
    "desmotaste": "desmotar",
    "desmotasteis": "desmotar",
    "desmote": "desmotar",
    "desmotemos": "desmotar",
    "desmoten": "desmotar",
    "desmotes": "desmote",
    "desmoto": "desmotar",
    "desmotábamos": "desmotar",
    "desmotáis": "desmotar",
    "desmotáramos": "desmotar",
    "desmotáremos": "desmotar",
    "desmotásemos": "desmotar",
    "desmoté": "desmotar",
    "desmotéis": "desmotar",
    "desmotó": "desmotar",
    "desmotúbulos": "desmotúbulo",
    "desmovilice": "desmovilizar",
    "desmovilicemos": "desmovilizar",
    "desmovilicen": "desmovilizar",
    "desmovilices": "desmovilizar",
    "desmovilicé": "desmovilizar",
    "desmovilicéis": "desmovilizar",
    "desmoviliza": "desmovilizar",
    "desmovilizaba": "desmovilizar",
    "desmovilizabais": "desmovilizar",
    "desmovilizaban": "desmovilizar",
    "desmovilizabas": "desmovilizar",
    "desmovilizaciones": "desmovilización",
    "desmovilizad": "desmovilizar",
    "desmovilizada": "desmovilizar",
    "desmovilizadas": "desmovilizar",
    "desmovilizado": "desmovilizar",
    "desmovilizadora": "desmovilizador",
    "desmovilizadoras": "desmovilizador",
    "desmovilizadores": "desmovilizador",
    "desmovilizados": "desmovilizar",
    "desmovilizamos": "desmovilizar",
    "desmovilizan": "desmovilizar",
    "desmovilizando": "desmovilizar",
    "desmovilizara": "desmovilizar",
    "desmovilizarais": "desmovilizar",
    "desmovilizaran": "desmovilizar",
    "desmovilizaras": "desmovilizar",
    "desmovilizare": "desmovilizar",
    "desmovilizareis": "desmovilizar",
    "desmovilizaremos": "desmovilizar",
    "desmovilizaren": "desmovilizar",
    "desmovilizares": "desmovilizar",
    "desmovilizaron": "desmovilizar",
    "desmovilizará": "desmovilizar",
    "desmovilizarán": "desmovilizar",
    "desmovilizarás": "desmovilizar",
    "desmovilizaré": "desmovilizar",
    "desmovilizaréis": "desmovilizar",
    "desmovilizaría": "desmovilizar",
    "desmovilizaríais": "desmovilizar",
    "desmovilizaríamos": "desmovilizar",
    "desmovilizarían": "desmovilizar",
    "desmovilizarías": "desmovilizar",
    "desmovilizas": "desmovilizar",
    "desmovilizase": "desmovilizar",
    "desmovilizaseis": "desmovilizar",
    "desmovilizasen": "desmovilizar",
    "desmovilizases": "desmovilizar",
    "desmovilizaste": "desmovilizar",
    "desmovilizasteis": "desmovilizar",
    "desmovilizo": "desmovilizar",
    "desmovilizábamos": "desmovilizar",
    "desmovilizáis": "desmovilizar",
    "desmovilizáramos": "desmovilizar",
    "desmovilizáremos": "desmovilizar",
    "desmovilizásemos": "desmovilizar",
    "desmovilizó": "desmovilizar",
    "desmoña": "desmoñar",
    "desmoñaba": "desmoñar",
    "desmoñabais": "desmoñar",
    "desmoñaban": "desmoñar",
    "desmoñabas": "desmoñar",
    "desmoñad": "desmoñar",
    "desmoñada": "desmoñar",
    "desmoñadas": "desmoñar",
    "desmoñado": "desmoñar",
    "desmoñados": "desmoñar",
    "desmoñamos": "desmoñar",
    "desmoñan": "desmoñar",
    "desmoñando": "desmoñar",
    "desmoñara": "desmoñar",
    "desmoñarais": "desmoñar",
    "desmoñaran": "desmoñar",
    "desmoñaras": "desmoñar",
    "desmoñare": "desmoñar",
    "desmoñareis": "desmoñar",
    "desmoñaremos": "desmoñar",
    "desmoñaren": "desmoñar",
    "desmoñares": "desmoñar",
    "desmoñaron": "desmoñar",
    "desmoñará": "desmoñar",
    "desmoñarán": "desmoñar",
    "desmoñarás": "desmoñar",
    "desmoñaré": "desmoñar",
    "desmoñaréis": "desmoñar",
    "desmoñaría": "desmoñar",
    "desmoñaríais": "desmoñar",
    "desmoñaríamos": "desmoñar",
    "desmoñarían": "desmoñar",
    "desmoñarías": "desmoñar",
    "desmoñas": "desmoñar",
    "desmoñase": "desmoñar",
    "desmoñaseis": "desmoñar",
    "desmoñasen": "desmoñar",
    "desmoñases": "desmoñar",
    "desmoñaste": "desmoñar",
    "desmoñasteis": "desmoñar",
    "desmoñe": "desmoñar",
    "desmoñemos": "desmoñar",
    "desmoñen": "desmoñar",
    "desmoñes": "desmoñar",
    "desmoño": "desmoñar",
    "desmoñábamos": "desmoñar",
    "desmoñáis": "desmoñar",
    "desmoñáramos": "desmoñar",
    "desmoñáremos": "desmoñar",
    "desmoñásemos": "desmoñar",
    "desmoñé": "desmoñar",
    "desmoñéis": "desmoñar",
    "desmoñó": "desmoñar",
    "desmuela": "desmoler",
    "desmuelan": "desmoler",
    "desmuelas": "desmoler",
    "desmuele": "desmoler",
    "desmuelen": "desmoler",
    "desmueles": "desmoler",
    "desmuelo": "desmoler",
    "desmultiplexaciones": "desmultiplexación",
    "desmultiplexores": "desmultiplexor",
    "desmultiplica": "desmultiplicar",
    "desmultiplicaba": "desmultiplicar",
    "desmultiplicabais": "desmultiplicar",
    "desmultiplicaban": "desmultiplicar",
    "desmultiplicabas": "desmultiplicar",
    "desmultiplicaciones": "desmultiplicación",
    "desmultiplicad": "desmultiplicar",
    "desmultiplicada": "desmultiplicar",
    "desmultiplicadas": "desmultiplicar",
    "desmultiplicado": "desmultiplicar",
    "desmultiplicados": "desmultiplicar",
    "desmultiplicamos": "desmultiplicar",
    "desmultiplican": "desmultiplicar",
    "desmultiplicando": "desmultiplicar",
    "desmultiplicara": "desmultiplicar",
    "desmultiplicarais": "desmultiplicar",
    "desmultiplicaran": "desmultiplicar",
    "desmultiplicaras": "desmultiplicar",
    "desmultiplicare": "desmultiplicar",
    "desmultiplicareis": "desmultiplicar",
    "desmultiplicaremos": "desmultiplicar",
    "desmultiplicaren": "desmultiplicar",
    "desmultiplicares": "desmultiplicar",
    "desmultiplicaron": "desmultiplicar",
    "desmultiplicará": "desmultiplicar",
    "desmultiplicarán": "desmultiplicar",
    "desmultiplicarás": "desmultiplicar",
    "desmultiplicaré": "desmultiplicar",
    "desmultiplicaréis": "desmultiplicar",
    "desmultiplicaría": "desmultiplicar",
    "desmultiplicaríais": "desmultiplicar",
    "desmultiplicaríamos": "desmultiplicar",
    "desmultiplicarían": "desmultiplicar",
    "desmultiplicarías": "desmultiplicar",
    "desmultiplicas": "desmultiplicar",
    "desmultiplicase": "desmultiplicar",
    "desmultiplicaseis": "desmultiplicar",
    "desmultiplicasen": "desmultiplicar",
    "desmultiplicases": "desmultiplicar",
    "desmultiplicaste": "desmultiplicar",
    "desmultiplicasteis": "desmultiplicar",
    "desmultiplico": "desmultiplicar",
    "desmultiplicábamos": "desmultiplicar",
    "desmultiplicáis": "desmultiplicar",
    "desmultiplicáramos": "desmultiplicar",
    "desmultiplicáremos": "desmultiplicar",
    "desmultiplicásemos": "desmultiplicar",
    "desmultiplicó": "desmultiplicar",
    "desmultiplique": "desmultiplicar",
    "desmultipliquemos": "desmultiplicar",
    "desmultipliquen": "desmultiplicar",
    "desmultipliques": "desmultiplicar",
    "desmultipliqué": "desmultiplicar",
    "desmultipliquéis": "desmultiplicar",
    "desmánes": "desmán",
    "desnacionalice": "desnacionalizar",
    "desnacionalicemos": "desnacionalizar",
    "desnacionalicen": "desnacionalizar",
    "desnacionalices": "desnacionalizar",
    "desnacionalicé": "desnacionalizar",
    "desnacionalicéis": "desnacionalizar",
    "desnacionaliza": "desnacionalizar",
    "desnacionalizaba": "desnacionalizar",
    "desnacionalizabais": "desnacionalizar",
    "desnacionalizaban": "desnacionalizar",
    "desnacionalizabas": "desnacionalizar",
    "desnacionalizaciones": "desnacionalización",
    "desnacionalizad": "desnacionalizar",
    "desnacionalizada": "desnacionalizar",
    "desnacionalizadas": "desnacionalizar",
    "desnacionalizado": "desnacionalizar",
    "desnacionalizados": "desnacionalizar",
    "desnacionalizamos": "desnacionalizar",
    "desnacionalizan": "desnacionalizar",
    "desnacionalizando": "desnacionalizar",
    "desnacionalizara": "desnacionalizar",
    "desnacionalizarais": "desnacionalizar",
    "desnacionalizaran": "desnacionalizar",
    "desnacionalizaras": "desnacionalizar",
    "desnacionalizare": "desnacionalizar",
    "desnacionalizareis": "desnacionalizar",
    "desnacionalizaremos": "desnacionalizar",
    "desnacionalizaren": "desnacionalizar",
    "desnacionalizares": "desnacionalizar",
    "desnacionalizaron": "desnacionalizar",
    "desnacionalizará": "desnacionalizar",
    "desnacionalizarán": "desnacionalizar",
    "desnacionalizarás": "desnacionalizar",
    "desnacionalizaré": "desnacionalizar",
    "desnacionalizaréis": "desnacionalizar",
    "desnacionalizaría": "desnacionalizar",
    "desnacionalizaríais": "desnacionalizar",
    "desnacionalizaríamos": "desnacionalizar",
    "desnacionalizarían": "desnacionalizar",
    "desnacionalizarías": "desnacionalizar",
    "desnacionalizas": "desnacionalizar",
    "desnacionalizase": "desnacionalizar",
    "desnacionalizaseis": "desnacionalizar",
    "desnacionalizasen": "desnacionalizar",
    "desnacionalizases": "desnacionalizar",
    "desnacionalizaste": "desnacionalizar",
    "desnacionalizasteis": "desnacionalizar",
    "desnacionalizo": "desnacionalizar",
    "desnacionalizábamos": "desnacionalizar",
    "desnacionalizáis": "desnacionalizar",
    "desnacionalizáramos": "desnacionalizar",
    "desnacionalizáremos": "desnacionalizar",
    "desnacionalizásemos": "desnacionalizar",
    "desnacionalizó": "desnacionalizar",
    "desnariga": "desnarigar",
    "desnarigaba": "desnarigar",
    "desnarigabais": "desnarigar",
    "desnarigaban": "desnarigar",
    "desnarigabas": "desnarigar",
    "desnarigad": "desnarigar",
    "desnarigada": "desnarigar",
    "desnarigadas": "desnarigar",
    "desnarigado": "desnarigar",
    "desnarigados": "desnarigar",
    "desnarigamos": "desnarigar",
    "desnarigan": "desnarigar",
    "desnarigando": "desnarigar",
    "desnarigara": "desnarigar",
    "desnarigarais": "desnarigar",
    "desnarigaran": "desnarigar",
    "desnarigaras": "desnarigar",
    "desnarigare": "desnarigar",
    "desnarigareis": "desnarigar",
    "desnarigaremos": "desnarigar",
    "desnarigaren": "desnarigar",
    "desnarigares": "desnarigar",
    "desnarigaron": "desnarigar",
    "desnarigará": "desnarigar",
    "desnarigarán": "desnarigar",
    "desnarigarás": "desnarigar",
    "desnarigaré": "desnarigar",
    "desnarigaréis": "desnarigar",
    "desnarigaría": "desnarigar",
    "desnarigaríais": "desnarigar",
    "desnarigaríamos": "desnarigar",
    "desnarigarían": "desnarigar",
    "desnarigarías": "desnarigar",
    "desnarigas": "desnarigar",
    "desnarigase": "desnarigar",
    "desnarigaseis": "desnarigar",
    "desnarigasen": "desnarigar",
    "desnarigases": "desnarigar",
    "desnarigaste": "desnarigar",
    "desnarigasteis": "desnarigar",
    "desnarigo": "desnarigar",
    "desnarigue": "desnarigar",
    "desnariguemos": "desnarigar",
    "desnariguen": "desnarigar",
    "desnarigues": "desnarigar",
    "desnarigué": "desnarigar",
    "desnariguéis": "desnarigar",
    "desnarigábamos": "desnarigar",
    "desnarigáis": "desnarigar",
    "desnarigáramos": "desnarigar",
    "desnarigáremos": "desnarigar",
    "desnarigásemos": "desnarigar",
    "desnarigó": "desnarigar",
    "desnata": "desnatar",
    "desnataba": "desnatar",
    "desnatabais": "desnatar",
    "desnataban": "desnatar",
    "desnatabas": "desnatar",
    "desnatad": "desnatar",
    "desnatada": "desnatar",
    "desnatadas": "desnatar",
    "desnatado": "desnatar",
    "desnatadoras": "desnatadora",
    "desnatados": "desnatar",
    "desnatamos": "desnatar",
    "desnatan": "desnatar",
    "desnatando": "desnatar",
    "desnatara": "desnatar",
    "desnatarais": "desnatar",
    "desnataran": "desnatar",
    "desnataras": "desnatar",
    "desnatare": "desnatar",
    "desnatareis": "desnatar",
    "desnataremos": "desnatar",
    "desnataren": "desnatar",
    "desnatares": "desnatar",
    "desnataron": "desnatar",
    "desnatará": "desnatar",
    "desnatarán": "desnatar",
    "desnatarás": "desnatar",
    "desnataré": "desnatar",
    "desnataréis": "desnatar",
    "desnataría": "desnatar",
    "desnataríais": "desnatar",
    "desnataríamos": "desnatar",
    "desnatarían": "desnatar",
    "desnatarías": "desnatar",
    "desnatas": "desnatar",
    "desnatase": "desnatar",
    "desnataseis": "desnatar",
    "desnatasen": "desnatar",
    "desnatases": "desnatar",
    "desnataste": "desnatar",
    "desnatasteis": "desnatar",
    "desnate": "desnatar",
    "desnatemos": "desnatar",
    "desnaten": "desnatar",
    "desnates": "desnate",
    "desnato": "desnatar",
    "desnaturalice": "desnaturalizar",
    "desnaturalicemos": "desnaturalizar",
    "desnaturalicen": "desnaturalizar",
    "desnaturalices": "desnaturalizar",
    "desnaturalicé": "desnaturalizar",
    "desnaturalicéis": "desnaturalizar",
    "desnaturaliza": "desnaturalizar",
    "desnaturalizaba": "desnaturalizar",
    "desnaturalizabais": "desnaturalizar",
    "desnaturalizaban": "desnaturalizar",
    "desnaturalizabas": "desnaturalizar",
    "desnaturalizaciones": "desnaturalización",
    "desnaturalizad": "desnaturalizar",
    "desnaturalizada": "desnaturalizar",
    "desnaturalizadas": "desnaturalizar",
    "desnaturalizado": "desnaturalizar",
    "desnaturalizados": "desnaturalizar",
    "desnaturalizamos": "desnaturalizar",
    "desnaturalizan": "desnaturalizar",
    "desnaturalizando": "desnaturalizar",
    "desnaturalizara": "desnaturalizar",
    "desnaturalizarais": "desnaturalizar",
    "desnaturalizaran": "desnaturalizar",
    "desnaturalizaras": "desnaturalizar",
    "desnaturalizare": "desnaturalizar",
    "desnaturalizareis": "desnaturalizar",
    "desnaturalizaremos": "desnaturalizar",
    "desnaturalizaren": "desnaturalizar",
    "desnaturalizares": "desnaturalizar",
    "desnaturalizaron": "desnaturalizar",
    "desnaturalizará": "desnaturalizar",
    "desnaturalizarán": "desnaturalizar",
    "desnaturalizarás": "desnaturalizar",
    "desnaturalizaré": "desnaturalizar",
    "desnaturalizaréis": "desnaturalizar",
    "desnaturalizaría": "desnaturalizar",
    "desnaturalizaríais": "desnaturalizar",
    "desnaturalizaríamos": "desnaturalizar",
    "desnaturalizarían": "desnaturalizar",
    "desnaturalizarías": "desnaturalizar",
    "desnaturalizas": "desnaturalizar",
    "desnaturalizase": "desnaturalizar",
    "desnaturalizaseis": "desnaturalizar",
    "desnaturalizasen": "desnaturalizar",
    "desnaturalizases": "desnaturalizar",
    "desnaturalizaste": "desnaturalizar",
    "desnaturalizasteis": "desnaturalizar",
    "desnaturalizo": "desnaturalizar",
    "desnaturalizábamos": "desnaturalizar",
    "desnaturalizáis": "desnaturalizar",
    "desnaturalizáramos": "desnaturalizar",
    "desnaturalizáremos": "desnaturalizar",
    "desnaturalizásemos": "desnaturalizar",
    "desnaturalizó": "desnaturalizar",
    "desnatábamos": "desnatar",
    "desnatáis": "desnatar",
    "desnatáramos": "desnatar",
    "desnatáremos": "desnatar",
    "desnatásemos": "desnatar",
    "desnaté": "desnatar",
    "desnatéis": "desnatar",
    "desnató": "desnatar",
    "desnegamientos": "desnegamiento",
    "desnerva": "desnervar",
    "desnervaba": "desnervar",
    "desnervabais": "desnervar",
    "desnervaban": "desnervar",
    "desnervabas": "desnervar",
    "desnervaciones": "desnervación",
    "desnervad": "desnervar",
    "desnervada": "desnervar",
    "desnervadas": "desnervar",
    "desnervado": "desnervar",
    "desnervados": "desnervar",
    "desnervamos": "desnervar",
    "desnervan": "desnervar",
    "desnervando": "desnervar",
    "desnervara": "desnervar",
    "desnervarais": "desnervar",
    "desnervaran": "desnervar",
    "desnervaras": "desnervar",
    "desnervare": "desnervar",
    "desnervareis": "desnervar",
    "desnervaremos": "desnervar",
    "desnervaren": "desnervar",
    "desnervares": "desnervar",
    "desnervaron": "desnervar",
    "desnervará": "desnervar",
    "desnervarán": "desnervar",
    "desnervarás": "desnervar",
    "desnervaré": "desnervar",
    "desnervaréis": "desnervar",
    "desnervaría": "desnervar",
    "desnervaríais": "desnervar",
    "desnervaríamos": "desnervar",
    "desnervarían": "desnervar",
    "desnervarías": "desnervar",
    "desnervas": "desnervar",
    "desnervase": "desnervar",
    "desnervaseis": "desnervar",
    "desnervasen": "desnervar",
    "desnervases": "desnervar",
    "desnervaste": "desnervar",
    "desnervasteis": "desnervar",
    "desnerve": "desnervar",
    "desnervemos": "desnervar",
    "desnerven": "desnervar",
    "desnerves": "desnervar",
    "desnervo": "desnervar",
    "desnervábamos": "desnervar",
    "desnerváis": "desnervar",
    "desnerváramos": "desnervar",
    "desnerváremos": "desnervar",
    "desnervásemos": "desnervar",
    "desnervé": "desnervar",
    "desnervéis": "desnervar",
    "desnervó": "desnervar",
    "desnevaba": "desnevar",
    "desnevabais": "desnevar",
    "desnevaban": "desnevar",
    "desnevabas": "desnevar",
    "desnevad": "desnevar",
    "desnevada": "desnevar",
    "desnevadas": "desnevar",
    "desnevado": "desnevar",
    "desnevados": "desnevar",
    "desnevamos": "desnevar",
    "desnevando": "desnevar",
    "desnevara": "desnevar",
    "desnevarais": "desnevar",
    "desnevaran": "desnevar",
    "desnevaras": "desnevar",
    "desnevare": "desnevar",
    "desnevareis": "desnevar",
    "desnevaremos": "desnevar",
    "desnevaren": "desnevar",
    "desnevares": "desnevar",
    "desnevaron": "desnevar",
    "desnevará": "desnevar",
    "desnevarán": "desnevar",
    "desnevarás": "desnevar",
    "desnevaré": "desnevar",
    "desnevaréis": "desnevar",
    "desnevaría": "desnevar",
    "desnevaríais": "desnevar",
    "desnevaríamos": "desnevar",
    "desnevarían": "desnevar",
    "desnevarías": "desnevar",
    "desnevase": "desnevar",
    "desnevaseis": "desnevar",
    "desnevasen": "desnevar",
    "desnevases": "desnevar",
    "desnevaste": "desnevar",
    "desnevasteis": "desnevar",
    "desnevemos": "desnevar",
    "desnevábamos": "desnevar",
    "desneváis": "desnevar",
    "desneváramos": "desnevar",
    "desneváremos": "desnevar",
    "desnevásemos": "desnevar",
    "desnevé": "desnevar",
    "desnevéis": "desnevar",
    "desnevó": "desnevar",
    "desnieva": "desnevar",
    "desnievan": "desnevar",
    "desnievas": "desnevar",
    "desnieve": "desnevar",
    "desnieven": "desnevar",
    "desnieves": "desnieve",
    "desnievo": "desnevar",
    "desnitrificaciones": "desnitrificación",
    "desnivela": "desnivelar",
    "desnivelaba": "desnivelar",
    "desnivelabais": "desnivelar",
    "desnivelaban": "desnivelar",
    "desnivelabas": "desnivelar",
    "desnivelaciones": "desnivelación",
    "desnivelad": "desnivelar",
    "desnivelada": "desnivelar",
    "desniveladas": "desnivelar",
    "desnivelado": "desnivelar",
    "desnivelados": "desnivelar",
    "desnivelamos": "desnivelar",
    "desnivelan": "desnivelar",
    "desnivelando": "desnivelar",
    "desnivelara": "desnivelar",
    "desnivelarais": "desnivelar",
    "desnivelaran": "desnivelar",
    "desnivelaras": "desnivelar",
    "desnivelare": "desnivelar",
    "desnivelareis": "desnivelar",
    "desnivelaremos": "desnivelar",
    "desnivelaren": "desnivelar",
    "desnivelares": "desnivelar",
    "desnivelaron": "desnivelar",
    "desnivelará": "desnivelar",
    "desnivelarán": "desnivelar",
    "desnivelarás": "desnivelar",
    "desnivelaré": "desnivelar",
    "desnivelaréis": "desnivelar",
    "desnivelaría": "desnivelar",
    "desnivelaríais": "desnivelar",
    "desnivelaríamos": "desnivelar",
    "desnivelarían": "desnivelar",
    "desnivelarías": "desnivelar",
    "desnivelas": "desnivelar",
    "desnivelase": "desnivelar",
    "desnivelaseis": "desnivelar",
    "desnivelasen": "desnivelar",
    "desnivelases": "desnivelar",
    "desnivelaste": "desnivelar",
    "desnivelasteis": "desnivelar",
    "desnivele": "desnivelar",
    "desnivelemos": "desnivelar",
    "desnivelen": "desnivelar",
    "desniveles": "desnivelar",
    "desnivelo": "desnivelar",
    "desnivelábamos": "desnivelar",
    "desniveláis": "desnivelar",
    "desniveláramos": "desnivelar",
    "desniveláremos": "desnivelar",
    "desnivelásemos": "desnivelar",
    "desnivelé": "desnivelar",
    "desniveléis": "desnivelar",
    "desniveló": "desnivelar",
    "desnuca": "desnucar",
    "desnucaba": "desnucar",
    "desnucabais": "desnucar",
    "desnucaban": "desnucar",
    "desnucabas": "desnucar",
    "desnucad": "desnucar",
    "desnucada": "desnucar",
    "desnucadas": "desnucar",
    "desnucado": "desnucar",
    "desnucados": "desnucar",
    "desnucamientos": "desnucamiento",
    "desnucamos": "desnucar",
    "desnucan": "desnucar",
    "desnucando": "desnucar",
    "desnucara": "desnucar",
    "desnucarais": "desnucar",
    "desnucaran": "desnucar",
    "desnucaras": "desnucar",
    "desnucare": "desnucar",
    "desnucareis": "desnucar",
    "desnucaremos": "desnucar",
    "desnucaren": "desnucar",
    "desnucares": "desnucar",
    "desnucaron": "desnucar",
    "desnucará": "desnucar",
    "desnucarán": "desnucar",
    "desnucarás": "desnucar",
    "desnucaré": "desnucar",
    "desnucaréis": "desnucar",
    "desnucaría": "desnucar",
    "desnucaríais": "desnucar",
    "desnucaríamos": "desnucar",
    "desnucarían": "desnucar",
    "desnucarías": "desnucar",
    "desnucas": "desnucar",
    "desnucase": "desnucar",
    "desnucaseis": "desnucar",
    "desnucasen": "desnucar",
    "desnucases": "desnucar",
    "desnucaste": "desnucar",
    "desnucasteis": "desnucar",
    "desnuclearizada": "desnuclearizado",
    "desnuclearizadas": "desnuclearizado",
    "desnuclearizados": "desnuclearizado",
    "desnuco": "desnucar",
    "desnucábamos": "desnucar",
    "desnucáis": "desnucar",
    "desnucáramos": "desnucar",
    "desnucáremos": "desnucar",
    "desnucásemos": "desnucar",
    "desnucó": "desnucar",
    "desnuda": "desnudo",
    "desnudaba": "desnudar",
    "desnudabais": "desnudar",
    "desnudaban": "desnudar",
    "desnudabas": "desnudar",
    "desnudad": "desnudar",
    "desnudada": "desnudar",
    "desnudadas": "desnudar",
    "desnudado": "desnudar",
    "desnudados": "desnudar",
    "desnudamientos": "desnudamiento",
    "desnudamos": "desnudar",
    "desnudan": "desnudar",
    "desnudando": "desnudar",
    "desnudara": "desnudar",
    "desnudarais": "desnudar",
    "desnudaran": "desnudar",
    "desnudaras": "desnudar",
    "desnudare": "desnudar",
    "desnudareis": "desnudar",
    "desnudaremos": "desnudar",
    "desnudaren": "desnudar",
    "desnudares": "desnudar",
    "desnudaron": "desnudar",
    "desnudará": "desnudar",
    "desnudarán": "desnudar",
    "desnudarás": "desnudar",
    "desnudaré": "desnudar",
    "desnudaréis": "desnudar",
    "desnudaría": "desnudar",
    "desnudaríais": "desnudar",
    "desnudaríamos": "desnudar",
    "desnudarían": "desnudar",
    "desnudarías": "desnudar",
    "desnudas": "desnudo",
    "desnudase": "desnudar",
    "desnudaseis": "desnudar",
    "desnudasen": "desnudar",
    "desnudases": "desnudar",
    "desnudaste": "desnudar",
    "desnudasteis": "desnudar",
    "desnude": "desnudar",
    "desnudeces": "desnudez",
    "desnudemos": "desnudar",
    "desnuden": "desnudar",
    "desnudes": "desnudar",
    "desnudismos": "desnudismo",
    "desnudo": "desnudar",
    "desnudos": "desnudo",
    "desnudábamos": "desnudar",
    "desnudáis": "desnudar",
    "desnudáramos": "desnudar",
    "desnudáremos": "desnudar",
    "desnudásemos": "desnudar",
    "desnudé": "desnudar",
    "desnudéis": "desnudar",
    "desnudó": "desnudar",
    "desnuque": "desnucar",
    "desnuquemos": "desnucar",
    "desnuquen": "desnucar",
    "desnuques": "desnucar",
    "desnuqué": "desnucar",
    "desnuquéis": "desnucar",
    "desnutra": "desnutrir",
    "desnutramos": "desnutrir",
    "desnutran": "desnutrir",
    "desnutras": "desnutrir",
    "desnutre": "desnutrir",
    "desnutren": "desnutrir",
    "desnutres": "desnutrir",
    "desnutriciones": "desnutrición",
    "desnutrid": "desnutrir",
    "desnutrida": "desnutrir",
    "desnutridas": "desnutrir",
    "desnutrido": "desnutrir",
    "desnutridos": "desnutrir",
    "desnutriendo": "desnutrir",
    "desnutriera": "desnutrir",
    "desnutrierais": "desnutrir",
    "desnutrieran": "desnutrir",
    "desnutrieras": "desnutrir",
    "desnutriere": "desnutrir",
    "desnutriereis": "desnutrir",
    "desnutrieren": "desnutrir",
    "desnutrieres": "desnutrir",
    "desnutrieron": "desnutrir",
    "desnutriese": "desnutrir",
    "desnutrieseis": "desnutrir",
    "desnutriesen": "desnutrir",
    "desnutrieses": "desnutrir",
    "desnutrimos": "desnutrir",
    "desnutriremos": "desnutrir",
    "desnutrirá": "desnutrir",
    "desnutrirán": "desnutrir",
    "desnutrirás": "desnutrir",
    "desnutriré": "desnutrir",
    "desnutriréis": "desnutrir",
    "desnutriría": "desnutrir",
    "desnutriríais": "desnutrir",
    "desnutriríamos": "desnutrir",
    "desnutrirían": "desnutrir",
    "desnutrirías": "desnutrir",
    "desnutriste": "desnutrir",
    "desnutristeis": "desnutrir",
    "desnutriéramos": "desnutrir",
    "desnutriéremos": "desnutrir",
    "desnutriésemos": "desnutrir",
    "desnutrió": "desnutrir",
    "desnutro": "desnutrir",
    "desnutráis": "desnutrir",
    "desnutrí": "desnutrir",
    "desnutría": "desnutrir",
    "desnutríais": "desnutrir",
    "desnutríamos": "desnutrir",
    "desnutrían": "desnutrir",
    "desnutrías": "desnutrir",
    "desnutrís": "desnutrir",
    "desobedece": "desobedecer",
    "desobedeced": "desobedecer",
    "desobedecemos": "desobedecer",
    "desobedecen": "desobedecer",
    "desobedeceremos": "desobedecer",
    "desobedecerá": "desobedecer",
    "desobedecerán": "desobedecer",
    "desobedecerás": "desobedecer",
    "desobedeceré": "desobedecer",
    "desobedeceréis": "desobedecer",
    "desobedecería": "desobedecer",
    "desobedeceríais": "desobedecer",
    "desobedeceríamos": "desobedecer",
    "desobedecerían": "desobedecer",
    "desobedecerías": "desobedecer",
    "desobedeces": "desobedecer",
    "desobedecida": "desobedecer",
    "desobedecidas": "desobedecer",
    "desobedecido": "desobedecer",
    "desobedecidos": "desobedecer",
    "desobedeciendo": "desobedecer",
    "desobedeciera": "desobedecer",
    "desobedecierais": "desobedecer",
    "desobedecieran": "desobedecer",
    "desobedecieras": "desobedecer",
    "desobedeciere": "desobedecer",
    "desobedeciereis": "desobedecer",
    "desobedecieren": "desobedecer",
    "desobedecieres": "desobedecer",
    "desobedecieron": "desobedecer",
    "desobedeciese": "desobedecer",
    "desobedecieseis": "desobedecer",
    "desobedeciesen": "desobedecer",
    "desobedecieses": "desobedecer",
    "desobedecimos": "desobedecer",
    "desobedeciste": "desobedecer",
    "desobedecisteis": "desobedecer",
    "desobedeciéramos": "desobedecer",
    "desobedeciéremos": "desobedecer",
    "desobedeciésemos": "desobedecer",
    "desobedeció": "desobedecer",
    "desobedecéis": "desobedecer",
    "desobedecí": "desobedecer",
    "desobedecía": "desobedecer",
    "desobedecíais": "desobedecer",
    "desobedecíamos": "desobedecer",
    "desobedecían": "desobedecer",
    "desobedecías": "desobedecer",
    "desobedezca": "desobedecer",
    "desobedezcamos": "desobedecer",
    "desobedezcan": "desobedecer",
    "desobedezcas": "desobedecer",
    "desobedezco": "desobedecer",
    "desobedezcáis": "desobedecer",
    "desobediencias": "desobediencia",
    "desobedientes": "desobediente",
    "desobliga": "desobligar",
    "desobligaba": "desobligar",
    "desobligabais": "desobligar",
    "desobligaban": "desobligar",
    "desobligabas": "desobligar",
    "desobligad": "desobligar",
    "desobligada": "desobligar",
    "desobligadas": "desobligar",
    "desobligado": "desobligar",
    "desobligados": "desobligar",
    "desobligamos": "desobligar",
    "desobligan": "desobligar",
    "desobligando": "desobligar",
    "desobligara": "desobligar",
    "desobligarais": "desobligar",
    "desobligaran": "desobligar",
    "desobligaras": "desobligar",
    "desobligare": "desobligar",
    "desobligareis": "desobligar",
    "desobligaremos": "desobligar",
    "desobligaren": "desobligar",
    "desobligares": "desobligar",
    "desobligaron": "desobligar",
    "desobligará": "desobligar",
    "desobligarán": "desobligar",
    "desobligarás": "desobligar",
    "desobligaré": "desobligar",
    "desobligaréis": "desobligar",
    "desobligaría": "desobligar",
    "desobligaríais": "desobligar",
    "desobligaríamos": "desobligar",
    "desobligarían": "desobligar",
    "desobligarías": "desobligar",
    "desobligas": "desobligar",
    "desobligase": "desobligar",
    "desobligaseis": "desobligar",
    "desobligasen": "desobligar",
    "desobligases": "desobligar",
    "desobligaste": "desobligar",
    "desobligasteis": "desobligar",
    "desobligo": "desobligar",
    "desobligue": "desobligar",
    "desobliguemos": "desobligar",
    "desobliguen": "desobligar",
    "desobligues": "desobligar",
    "desobligué": "desobligar",
    "desobliguéis": "desobligar",
    "desobligábamos": "desobligar",
    "desobligáis": "desobligar",
    "desobligáramos": "desobligar",
    "desobligáremos": "desobligar",
    "desobligásemos": "desobligar",
    "desobligó": "desobligar",
    "desobstrucciones": "desobstrucción",
    "desobstruid": "desobstruir",
    "desobstruida": "desobstruir",
    "desobstruidas": "desobstruir",
    "desobstruido": "desobstruir",
    "desobstruidos": "desobstruir",
    "desobstruimos": "desobstruir",
    "desobstruiremos": "desobstruir",
    "desobstruirá": "desobstruir",
    "desobstruirán": "desobstruir",
    "desobstruirás": "desobstruir",
    "desobstruiré": "desobstruir",
    "desobstruiréis": "desobstruir",
    "desobstruiría": "desobstruir",
    "desobstruiríais": "desobstruir",
    "desobstruiríamos": "desobstruir",
    "desobstruirían": "desobstruir",
    "desobstruirías": "desobstruir",
    "desobstruiste": "desobstruir",
    "desobstruisteis": "desobstruir",
    "desobstruya": "desobstruir",
    "desobstruyamos": "desobstruir",
    "desobstruyan": "desobstruir",
    "desobstruyas": "desobstruir",
    "desobstruye": "desobstruir",
    "desobstruyen": "desobstruir",
    "desobstruyendo": "desobstruir",
    "desobstruyera": "desobstruir",
    "desobstruyerais": "desobstruir",
    "desobstruyeran": "desobstruir",
    "desobstruyeras": "desobstruir",
    "desobstruyere": "desobstruir",
    "desobstruyereis": "desobstruir",
    "desobstruyeren": "desobstruir",
    "desobstruyeres": "desobstruir",
    "desobstruyeron": "desobstruir",
    "desobstruyes": "desobstruir",
    "desobstruyese": "desobstruir",
    "desobstruyeseis": "desobstruir",
    "desobstruyesen": "desobstruir",
    "desobstruyeses": "desobstruir",
    "desobstruyo": "desobstruir",
    "desobstruyáis": "desobstruir",
    "desobstruyéramos": "desobstruir",
    "desobstruyéremos": "desobstruir",
    "desobstruyésemos": "desobstruir",
    "desobstruyó": "desobstruir",
    "desobstruí": "desobstruir",
    "desobstruía": "desobstruir",
    "desobstruíais": "desobstruir",
    "desobstruíamos": "desobstruir",
    "desobstruían": "desobstruir",
    "desobstruías": "desobstruir",
    "desobstruís": "desobstruir",
    "desocasionada": "desocasionado",
    "desocasionadas": "desocasionado",
    "desocasionados": "desocasionado",
    "desocializaciones": "desocialización",
    "desocializantes": "desocializante",
    "desocupa": "desocupar",
    "desocupaba": "desocupar",
    "desocupabais": "desocupar",
    "desocupaban": "desocupar",
    "desocupabas": "desocupar",
    "desocupaciones": "desocupación",
    "desocupad": "desocupar",
    "desocupada": "desocupar",
    "desocupadas": "desocupar",
    "desocupado": "desocupar",
    "desocupados": "desocupar",
    "desocupamos": "desocupar",
    "desocupan": "desocupar",
    "desocupando": "desocupar",
    "desocupara": "desocupar",
    "desocuparais": "desocupar",
    "desocuparan": "desocupar",
    "desocuparas": "desocupar",
    "desocupare": "desocupar",
    "desocupareis": "desocupar",
    "desocuparemos": "desocupar",
    "desocuparen": "desocupar",
    "desocupares": "desocupar",
    "desocuparon": "desocupar",
    "desocupará": "desocupar",
    "desocuparán": "desocupar",
    "desocuparás": "desocupar",
    "desocuparé": "desocupar",
    "desocuparéis": "desocupar",
    "desocuparía": "desocupar",
    "desocuparíais": "desocupar",
    "desocuparíamos": "desocupar",
    "desocuparían": "desocupar",
    "desocuparías": "desocupar",
    "desocupas": "desocupar",
    "desocupase": "desocupar",
    "desocupaseis": "desocupar",
    "desocupasen": "desocupar",
    "desocupases": "desocupar",
    "desocupaste": "desocupar",
    "desocupasteis": "desocupar",
    "desocupe": "desocupar",
    "desocupemos": "desocupar",
    "desocupen": "desocupar",
    "desocupes": "desocupar",
    "desocupo": "desocupar",
    "desocupábamos": "desocupar",
    "desocupáis": "desocupar",
    "desocupáramos": "desocupar",
    "desocupáremos": "desocupar",
    "desocupásemos": "desocupar",
    "desocupé": "desocupar",
    "desocupéis": "desocupar",
    "desocupó": "desocupar",
    "desodoraciones": "desodoración",
    "desodorantes": "desodorante",
    "desodorice": "desodorizar",
    "desodoricemos": "desodorizar",
    "desodoricen": "desodorizar",
    "desodorices": "desodorizar",
    "desodoricé": "desodorizar",
    "desodoricéis": "desodorizar",
    "desodoriza": "desodorizar",
    "desodorizaba": "desodorizar",
    "desodorizabais": "desodorizar",
    "desodorizaban": "desodorizar",
    "desodorizabas": "desodorizar",
    "desodorizaciones": "desodorización",
    "desodorizad": "desodorizar",
    "desodorizada": "desodorizar",
    "desodorizadas": "desodorizar",
    "desodorizado": "desodorizar",
    "desodorizados": "desodorizar",
    "desodorizamos": "desodorizar",
    "desodorizan": "desodorizar",
    "desodorizando": "desodorizar",
    "desodorizara": "desodorizar",
    "desodorizarais": "desodorizar",
    "desodorizaran": "desodorizar",
    "desodorizaras": "desodorizar",
    "desodorizare": "desodorizar",
    "desodorizareis": "desodorizar",
    "desodorizaremos": "desodorizar",
    "desodorizaren": "desodorizar",
    "desodorizares": "desodorizar",
    "desodorizaron": "desodorizar",
    "desodorizará": "desodorizar",
    "desodorizarán": "desodorizar",
    "desodorizarás": "desodorizar",
    "desodorizaré": "desodorizar",
    "desodorizaréis": "desodorizar",
    "desodorizaría": "desodorizar",
    "desodorizaríais": "desodorizar",
    "desodorizaríamos": "desodorizar",
    "desodorizarían": "desodorizar",
    "desodorizarías": "desodorizar",
    "desodorizas": "desodorizar",
    "desodorizase": "desodorizar",
    "desodorizaseis": "desodorizar",
    "desodorizasen": "desodorizar",
    "desodorizases": "desodorizar",
    "desodorizaste": "desodorizar",
    "desodorizasteis": "desodorizar",
    "desodorizo": "desodorizar",
    "desodorizábamos": "desodorizar",
    "desodorizáis": "desodorizar",
    "desodorizáramos": "desodorizar",
    "desodorizáremos": "desodorizar",
    "desodorizásemos": "desodorizar",
    "desodorizó": "desodorizar",
    "desoiga": "desoír",
    "desoigamos": "desoír",
    "desoigan": "desoír",
    "desoigas": "desoír",
    "desoigo": "desoír",
    "desoigáis": "desoír",
    "desoiremos": "desoír",
    "desoirá": "desoír",
    "desoirán": "desoír",
    "desoirás": "desoír",
    "desoiré": "desoír",
    "desoiréis": "desoír",
    "desoiría": "desoír",
    "desoiríais": "desoír",
    "desoiríamos": "desoír",
    "desoirían": "desoír",
    "desoirías": "desoír",
    "desoja": "desojar",
    "desojaba": "desojar",
    "desojabais": "desojar",
    "desojaban": "desojar",
    "desojabas": "desojar",
    "desojad": "desojar",
    "desojada": "desojar",
    "desojadas": "desojar",
    "desojado": "desojar",
    "desojados": "desojar",
    "desojamos": "desojar",
    "desojan": "desojar",
    "desojando": "desojar",
    "desojara": "desojar",
    "desojarais": "desojar",
    "desojaran": "desojar",
    "desojaras": "desojar",
    "desojare": "desojar",
    "desojareis": "desojar",
    "desojaremos": "desojar",
    "desojaren": "desojar",
    "desojares": "desojar",
    "desojaron": "desojar",
    "desojará": "desojar",
    "desojarán": "desojar",
    "desojarás": "desojar",
    "desojaré": "desojar",
    "desojaréis": "desojar",
    "desojaría": "desojar",
    "desojaríais": "desojar",
    "desojaríamos": "desojar",
    "desojarían": "desojar",
    "desojarías": "desojar",
    "desojas": "desojar",
    "desojase": "desojar",
    "desojaseis": "desojar",
    "desojasen": "desojar",
    "desojases": "desojar",
    "desojaste": "desojar",
    "desojasteis": "desojar",
    "desoje": "desojar",
    "desojemos": "desojar",
    "desojen": "desojar",
    "desojes": "desojar",
    "desojo": "desojar",
    "desojábamos": "desojar",
    "desojáis": "desojar",
    "desojáramos": "desojar",
    "desojáremos": "desojar",
    "desojásemos": "desojar",
    "desojé": "desojar",
    "desojéis": "desojar",
    "desojó": "desojar",
    "desola": "desolar",
    "desolaba": "desolar",
    "desolabais": "desolar",
    "desolaban": "desolar",
    "desolabas": "desolar",
    "desolaciones": "desolación",
    "desolad": "desolar",
    "desolada": "desolar",
    "desoladas": "desolar",
    "desolado": "desolar",
    "desoladora": "desolador",
    "desoladoras": "desolador",
    "desoladores": "desolador",
    "desolados": "desolar",
    "desolamos": "desolar",
    "desolan": "desolar",
    "desolando": "desolar",
    "desolara": "desolar",
    "desolarais": "desolar",
    "desolaran": "desolar",
    "desolaras": "desolar",
    "desolare": "desolar",
    "desolareis": "desolar",
    "desolaremos": "desolar",
    "desolaren": "desolar",
    "desolares": "desolar",
    "desolaron": "desolar",
    "desolará": "desolar",
    "desolarán": "desolar",
    "desolarás": "desolar",
    "desolaré": "desolar",
    "desolaréis": "desolar",
    "desolaría": "desolar",
    "desolaríais": "desolar",
    "desolaríamos": "desolar",
    "desolarían": "desolar",
    "desolarías": "desolar",
    "desolas": "desolar",
    "desolase": "desolar",
    "desolaseis": "desolar",
    "desolasen": "desolar",
    "desolases": "desolar",
    "desolaste": "desolar",
    "desolasteis": "desolar",
    "desoldaba": "desoldar",
    "desoldabais": "desoldar",
    "desoldaban": "desoldar",
    "desoldabas": "desoldar",
    "desoldad": "desoldar",
    "desoldada": "desoldar",
    "desoldadas": "desoldar",
    "desoldado": "desoldar",
    "desoldados": "desoldar",
    "desoldaduras": "desoldadura",
    "desoldamos": "desoldar",
    "desoldando": "desoldar",
    "desoldara": "desoldar",
    "desoldarais": "desoldar",
    "desoldaran": "desoldar",
    "desoldaras": "desoldar",
    "desoldare": "desoldar",
    "desoldareis": "desoldar",
    "desoldaremos": "desoldar",
    "desoldaren": "desoldar",
    "desoldares": "desoldar",
    "desoldaron": "desoldar",
    "desoldará": "desoldar",
    "desoldarán": "desoldar",
    "desoldarás": "desoldar",
    "desoldaré": "desoldar",
    "desoldaréis": "desoldar",
    "desoldaría": "desoldar",
    "desoldaríais": "desoldar",
    "desoldaríamos": "desoldar",
    "desoldarían": "desoldar",
    "desoldarías": "desoldar",
    "desoldase": "desoldar",
    "desoldaseis": "desoldar",
    "desoldasen": "desoldar",
    "desoldases": "desoldar",
    "desoldaste": "desoldar",
    "desoldasteis": "desoldar",
    "desoldemos": "desoldar",
    "desoldábamos": "desoldar",
    "desoldáis": "desoldar",
    "desoldáramos": "desoldar",
    "desoldáremos": "desoldar",
    "desoldásemos": "desoldar",
    "desoldé": "desoldar",
    "desoldéis": "desoldar",
    "desoldó": "desoldar",
    "desole": "desolar",
    "desolemos": "desolar",
    "desolen": "desolar",
    "desoles": "desolar",
    "desolidarice": "desolidarizar",
    "desolidaricemos": "desolidarizar",
    "desolidaricen": "desolidarizar",
    "desolidarices": "desolidarizar",
    "desolidaricé": "desolidarizar",
    "desolidaricéis": "desolidarizar",
    "desolidariza": "desolidarizar",
    "desolidarizaba": "desolidarizar",
    "desolidarizabais": "desolidarizar",
    "desolidarizaban": "desolidarizar",
    "desolidarizabas": "desolidarizar",
    "desolidarizad": "desolidarizar",
    "desolidarizada": "desolidarizar",
    "desolidarizadas": "desolidarizar",
    "desolidarizado": "desolidarizar",
    "desolidarizados": "desolidarizar",
    "desolidarizamos": "desolidarizar",
    "desolidarizan": "desolidarizar",
    "desolidarizando": "desolidarizar",
    "desolidarizara": "desolidarizar",
    "desolidarizarais": "desolidarizar",
    "desolidarizaran": "desolidarizar",
    "desolidarizaras": "desolidarizar",
    "desolidarizare": "desolidarizar",
    "desolidarizareis": "desolidarizar",
    "desolidarizaremos": "desolidarizar",
    "desolidarizaren": "desolidarizar",
    "desolidarizares": "desolidarizar",
    "desolidarizaron": "desolidarizar",
    "desolidarizará": "desolidarizar",
    "desolidarizarán": "desolidarizar",
    "desolidarizarás": "desolidarizar",
    "desolidarizaré": "desolidarizar",
    "desolidarizaréis": "desolidarizar",
    "desolidarizaría": "desolidarizar",
    "desolidarizaríais": "desolidarizar",
    "desolidarizaríamos": "desolidarizar",
    "desolidarizarían": "desolidarizar",
    "desolidarizarías": "desolidarizar",
    "desolidarizas": "desolidarizar",
    "desolidarizase": "desolidarizar",
    "desolidarizaseis": "desolidarizar",
    "desolidarizasen": "desolidarizar",
    "desolidarizases": "desolidarizar",
    "desolidarizaste": "desolidarizar",
    "desolidarizasteis": "desolidarizar",
    "desolidarizo": "desolidarizar",
    "desolidarizábamos": "desolidarizar",
    "desolidarizáis": "desolidarizar",
    "desolidarizáramos": "desolidarizar",
    "desolidarizáremos": "desolidarizar",
    "desolidarizásemos": "desolidarizar",
    "desolidarizó": "desolidarizar",
    "desollaba": "desollar",
    "desollabais": "desollar",
    "desollaban": "desollar",
    "desollabas": "desollar",
    "desollad": "desollar",
    "desollada": "desollar",
    "desolladas": "desollar",
    "desolladeros": "desolladero",
    "desollado": "desollar",
    "desolladora": "desollador",
    "desolladoras": "desollador",
    "desolladores": "desollador",
    "desollados": "desollar",
    "desolladuras": "desolladura",
    "desollamos": "desollar",
    "desollando": "desollar",
    "desollara": "desollar",
    "desollarais": "desollar",
    "desollaran": "desollar",
    "desollaras": "desollar",
    "desollare": "desollar",
    "desollareis": "desollar",
    "desollaremos": "desollar",
    "desollaren": "desollar",
    "desollares": "desollar",
    "desollaron": "desollar",
    "desollará": "desollar",
    "desollarán": "desollar",
    "desollarás": "desollar",
    "desollaré": "desollar",
    "desollaréis": "desollar",
    "desollaría": "desollar",
    "desollaríais": "desollar",
    "desollaríamos": "desollar",
    "desollarían": "desollar",
    "desollarías": "desollar",
    "desollase": "desollar",
    "desollaseis": "desollar",
    "desollasen": "desollar",
    "desollases": "desollar",
    "desollaste": "desollar",
    "desollasteis": "desollar",
    "desollemos": "desollar",
    "desollones": "desollón",
    "desollábamos": "desollar",
    "desolláis": "desollar",
    "desolláramos": "desollar",
    "desolláremos": "desollar",
    "desollásemos": "desollar",
    "desollé": "desollar",
    "desolléis": "desollar",
    "desolló": "desollar",
    "desolo": "desolar",
    "desolábamos": "desolar",
    "desoláis": "desolar",
    "desoláramos": "desolar",
    "desoláremos": "desolar",
    "desolásemos": "desolar",
    "desolé": "desolar",
    "desoléis": "desolar",
    "desoló": "desolar",
    "desonces": "desonce",
    "desonorizaciones": "desonorización",
    "desopila": "desopilar",
    "desopilaba": "desopilar",
    "desopilabais": "desopilar",
    "desopilaban": "desopilar",
    "desopilabas": "desopilar",
    "desopilaciones": "desopilación",
    "desopilad": "desopilar",
    "desopilada": "desopilar",
    "desopiladas": "desopilar",
    "desopilado": "desopilar",
    "desopilados": "desopilar",
    "desopilamos": "desopilar",
    "desopilan": "desopilar",
    "desopilando": "desopilar",
    "desopilara": "desopilar",
    "desopilarais": "desopilar",
    "desopilaran": "desopilar",
    "desopilaras": "desopilar",
    "desopilare": "desopilar",
    "desopilareis": "desopilar",
    "desopilaremos": "desopilar",
    "desopilaren": "desopilar",
    "desopilares": "desopilar",
    "desopilaron": "desopilar",
    "desopilará": "desopilar",
    "desopilarán": "desopilar",
    "desopilarás": "desopilar",
    "desopilaré": "desopilar",
    "desopilaréis": "desopilar",
    "desopilaría": "desopilar",
    "desopilaríais": "desopilar",
    "desopilaríamos": "desopilar",
    "desopilarían": "desopilar",
    "desopilarías": "desopilar",
    "desopilas": "desopilar",
    "desopilase": "desopilar",
    "desopilaseis": "desopilar",
    "desopilasen": "desopilar",
    "desopilases": "desopilar",
    "desopilaste": "desopilar",
    "desopilasteis": "desopilar",
    "desopile": "desopilar",
    "desopilemos": "desopilar",
    "desopilen": "desopilar",
    "desopiles": "desopilar",
    "desopilo": "desopilar",
    "desopilábamos": "desopilar",
    "desopiláis": "desopilar",
    "desopiláramos": "desopilar",
    "desopiláremos": "desopilar",
    "desopilásemos": "desopilar",
    "desopilé": "desopilar",
    "desopiléis": "desopilar",
    "desopiló": "desopilar",
    "desopresiones": "desopresión",
    "desoprima": "desoprimir",
    "desoprimamos": "desoprimir",
    "desopriman": "desoprimir",
    "desoprimas": "desoprimir",
    "desoprime": "desoprimir",
    "desoprimen": "desoprimir",
    "desoprimes": "desoprimir",
    "desoprimid": "desoprimir",
    "desoprimida": "desoprimir",
    "desoprimidas": "desoprimir",
    "desoprimido": "desoprimir",
    "desoprimidos": "desoprimir",
    "desoprimiendo": "desoprimir",
    "desoprimiera": "desoprimir",
    "desoprimierais": "desoprimir",
    "desoprimieran": "desoprimir",
    "desoprimieras": "desoprimir",
    "desoprimiere": "desoprimir",
    "desoprimiereis": "desoprimir",
    "desoprimieren": "desoprimir",
    "desoprimieres": "desoprimir",
    "desoprimieron": "desoprimir",
    "desoprimiese": "desoprimir",
    "desoprimieseis": "desoprimir",
    "desoprimiesen": "desoprimir",
    "desoprimieses": "desoprimir",
    "desoprimimos": "desoprimir",
    "desoprimiremos": "desoprimir",
    "desoprimirá": "desoprimir",
    "desoprimirán": "desoprimir",
    "desoprimirás": "desoprimir",
    "desoprimiré": "desoprimir",
    "desoprimiréis": "desoprimir",
    "desoprimiría": "desoprimir",
    "desoprimiríais": "desoprimir",
    "desoprimiríamos": "desoprimir",
    "desoprimirían": "desoprimir",
    "desoprimirías": "desoprimir",
    "desoprimiste": "desoprimir",
    "desoprimisteis": "desoprimir",
    "desoprimiéramos": "desoprimir",
    "desoprimiéremos": "desoprimir",
    "desoprimiésemos": "desoprimir",
    "desoprimió": "desoprimir",
    "desoprimo": "desoprimir",
    "desoprimáis": "desoprimir",
    "desoprimí": "desoprimir",
    "desoprimía": "desoprimir",
    "desoprimíais": "desoprimir",
    "desoprimíamos": "desoprimir",
    "desoprimían": "desoprimir",
    "desoprimías": "desoprimir",
    "desoprimís": "desoprimir",
    "desorbita": "desorbitar",
    "desorbitaba": "desorbitar",
    "desorbitabais": "desorbitar",
    "desorbitaban": "desorbitar",
    "desorbitabas": "desorbitar",
    "desorbitaciones": "desorbitación",
    "desorbitad": "desorbitar",
    "desorbitada": "desorbitar",
    "desorbitadas": "desorbitar",
    "desorbitado": "desorbitar",
    "desorbitados": "desorbitar",
    "desorbitamos": "desorbitar",
    "desorbitan": "desorbitar",
    "desorbitando": "desorbitar",
    "desorbitara": "desorbitar",
    "desorbitarais": "desorbitar",
    "desorbitaran": "desorbitar",
    "desorbitaras": "desorbitar",
    "desorbitare": "desorbitar",
    "desorbitareis": "desorbitar",
    "desorbitaremos": "desorbitar",
    "desorbitaren": "desorbitar",
    "desorbitares": "desorbitar",
    "desorbitaron": "desorbitar",
    "desorbitará": "desorbitar",
    "desorbitarán": "desorbitar",
    "desorbitarás": "desorbitar",
    "desorbitaré": "desorbitar",
    "desorbitaréis": "desorbitar",
    "desorbitaría": "desorbitar",
    "desorbitaríais": "desorbitar",
    "desorbitaríamos": "desorbitar",
    "desorbitarían": "desorbitar",
    "desorbitarías": "desorbitar",
    "desorbitas": "desorbitar",
    "desorbitase": "desorbitar",
    "desorbitaseis": "desorbitar",
    "desorbitasen": "desorbitar",
    "desorbitases": "desorbitar",
    "desorbitaste": "desorbitar",
    "desorbitasteis": "desorbitar",
    "desorbite": "desorbitar",
    "desorbitemos": "desorbitar",
    "desorbiten": "desorbitar",
    "desorbites": "desorbitar",
    "desorbito": "desorbitar",
    "desorbitábamos": "desorbitar",
    "desorbitáis": "desorbitar",
    "desorbitáramos": "desorbitar",
    "desorbitáremos": "desorbitar",
    "desorbitásemos": "desorbitar",
    "desorbité": "desorbitar",
    "desorbitéis": "desorbitar",
    "desorbitó": "desorbitar",
    "desorciones": "desorción",
    "desordena": "desordenar",
    "desordenaba": "desordenar",
    "desordenabais": "desordenar",
    "desordenaban": "desordenar",
    "desordenabas": "desordenar",
    "desordenad": "desordenar",
    "desordenada": "desordenar",
    "desordenadas": "desordenar",
    "desordenado": "desordenar",
    "desordenados": "desordenar",
    "desordenamos": "desordenar",
    "desordenan": "desordenar",
    "desordenando": "desordenar",
    "desordenara": "desordenar",
    "desordenarais": "desordenar",
    "desordenaran": "desordenar",
    "desordenaras": "desordenar",
    "desordenare": "desordenar",
    "desordenareis": "desordenar",
    "desordenaremos": "desordenar",
    "desordenaren": "desordenar",
    "desordenares": "desordenar",
    "desordenaron": "desordenar",
    "desordenará": "desordenar",
    "desordenarán": "desordenar",
    "desordenarás": "desordenar",
    "desordenaré": "desordenar",
    "desordenaréis": "desordenar",
    "desordenaría": "desordenar",
    "desordenaríais": "desordenar",
    "desordenaríamos": "desordenar",
    "desordenarían": "desordenar",
    "desordenarías": "desordenar",
    "desordenas": "desordenar",
    "desordenase": "desordenar",
    "desordenaseis": "desordenar",
    "desordenasen": "desordenar",
    "desordenases": "desordenar",
    "desordenaste": "desordenar",
    "desordenasteis": "desordenar",
    "desordene": "desordenar",
    "desordenemos": "desordenar",
    "desordenen": "desordenar",
    "desordenes": "desordenar",
    "desordeno": "desordenar",
    "desordenábamos": "desordenar",
    "desordenáis": "desordenar",
    "desordenáramos": "desordenar",
    "desordenáremos": "desordenar",
    "desordenásemos": "desordenar",
    "desordené": "desordenar",
    "desordenéis": "desordenar",
    "desordenó": "desordenar",
    "desoreja": "desorejar",
    "desorejaba": "desorejar",
    "desorejabais": "desorejar",
    "desorejaban": "desorejar",
    "desorejabas": "desorejar",
    "desorejad": "desorejar",
    "desorejada": "desorejar",
    "desorejadas": "desorejar",
    "desorejado": "desorejar",
    "desorejados": "desorejar",
    "desorejamientos": "desorejamiento",
    "desorejamos": "desorejar",
    "desorejan": "desorejar",
    "desorejando": "desorejar",
    "desorejara": "desorejar",
    "desorejarais": "desorejar",
    "desorejaran": "desorejar",
    "desorejaras": "desorejar",
    "desorejare": "desorejar",
    "desorejareis": "desorejar",
    "desorejaremos": "desorejar",
    "desorejaren": "desorejar",
    "desorejares": "desorejar",
    "desorejaron": "desorejar",
    "desorejará": "desorejar",
    "desorejarán": "desorejar",
    "desorejarás": "desorejar",
    "desorejaré": "desorejar",
    "desorejaréis": "desorejar",
    "desorejaría": "desorejar",
    "desorejaríais": "desorejar",
    "desorejaríamos": "desorejar",
    "desorejarían": "desorejar",
    "desorejarías": "desorejar",
    "desorejas": "desorejar",
    "desorejase": "desorejar",
    "desorejaseis": "desorejar",
    "desorejasen": "desorejar",
    "desorejases": "desorejar",
    "desorejaste": "desorejar",
    "desorejasteis": "desorejar",
    "desoreje": "desorejar",
    "desorejemos": "desorejar",
    "desorejen": "desorejar",
    "desorejes": "desorejar",
    "desorejo": "desorejar",
    "desorejábamos": "desorejar",
    "desorejáis": "desorejar",
    "desorejáramos": "desorejar",
    "desorejáremos": "desorejar",
    "desorejásemos": "desorejar",
    "desorejé": "desorejar",
    "desorejéis": "desorejar",
    "desorejó": "desorejar",
    "desorganice": "desorganizar",
    "desorganicemos": "desorganizar",
    "desorganicen": "desorganizar",
    "desorganices": "desorganizar",
    "desorganicé": "desorganizar",
    "desorganicéis": "desorganizar",
    "desorganiza": "desorganizar",
    "desorganizaba": "desorganizar",
    "desorganizabais": "desorganizar",
    "desorganizaban": "desorganizar",
    "desorganizabas": "desorganizar",
    "desorganizaciones": "desorganización",
    "desorganizad": "desorganizar",
    "desorganizada": "desorganizar",
    "desorganizadas": "desorganizar",
    "desorganizado": "desorganizar",
    "desorganizadora": "desorganizador",
    "desorganizadoras": "desorganizador",
    "desorganizadores": "desorganizador",
    "desorganizados": "desorganizar",
    "desorganizamos": "desorganizar",
    "desorganizan": "desorganizar",
    "desorganizando": "desorganizar",
    "desorganizara": "desorganizar",
    "desorganizarais": "desorganizar",
    "desorganizaran": "desorganizar",
    "desorganizaras": "desorganizar",
    "desorganizare": "desorganizar",
    "desorganizareis": "desorganizar",
    "desorganizaremos": "desorganizar",
    "desorganizaren": "desorganizar",
    "desorganizares": "desorganizar",
    "desorganizaron": "desorganizar",
    "desorganizará": "desorganizar",
    "desorganizarán": "desorganizar",
    "desorganizarás": "desorganizar",
    "desorganizaré": "desorganizar",
    "desorganizaréis": "desorganizar",
    "desorganizaría": "desorganizar",
    "desorganizaríais": "desorganizar",
    "desorganizaríamos": "desorganizar",
    "desorganizarían": "desorganizar",
    "desorganizarías": "desorganizar",
    "desorganizas": "desorganizar",
    "desorganizase": "desorganizar",
    "desorganizaseis": "desorganizar",
    "desorganizasen": "desorganizar",
    "desorganizases": "desorganizar",
    "desorganizaste": "desorganizar",
    "desorganizasteis": "desorganizar",
    "desorganizo": "desorganizar",
    "desorganizábamos": "desorganizar",
    "desorganizáis": "desorganizar",
    "desorganizáramos": "desorganizar",
    "desorganizáremos": "desorganizar",
    "desorganizásemos": "desorganizar",
    "desorganizó": "desorganizar",
    "desorienta": "desorientar",
    "desorientaba": "desorientar",
    "desorientabais": "desorientar",
    "desorientaban": "desorientar",
    "desorientabas": "desorientar",
    "desorientaciones": "desorientación",
    "desorientad": "desorientar",
    "desorientada": "desorientar",
    "desorientadas": "desorientar",
    "desorientado": "desorientar",
    "desorientadora": "desorientador",
    "desorientadoras": "desorientador",
    "desorientadores": "desorientador",
    "desorientados": "desorientar",
    "desorientamos": "desorientar",
    "desorientan": "desorientar",
    "desorientando": "desorientar",
    "desorientara": "desorientar",
    "desorientarais": "desorientar",
    "desorientaran": "desorientar",
    "desorientaras": "desorientar",
    "desorientare": "desorientar",
    "desorientareis": "desorientar",
    "desorientaremos": "desorientar",
    "desorientaren": "desorientar",
    "desorientares": "desorientar",
    "desorientaron": "desorientar",
    "desorientará": "desorientar",
    "desorientarán": "desorientar",
    "desorientarás": "desorientar",
    "desorientaré": "desorientar",
    "desorientaréis": "desorientar",
    "desorientaría": "desorientar",
    "desorientaríais": "desorientar",
    "desorientaríamos": "desorientar",
    "desorientarían": "desorientar",
    "desorientarías": "desorientar",
    "desorientas": "desorientar",
    "desorientase": "desorientar",
    "desorientaseis": "desorientar",
    "desorientasen": "desorientar",
    "desorientases": "desorientar",
    "desorientaste": "desorientar",
    "desorientasteis": "desorientar",
    "desoriente": "desorientar",
    "desorientemos": "desorientar",
    "desorienten": "desorientar",
    "desorientes": "desorientar",
    "desoriento": "desorientar",
    "desorientábamos": "desorientar",
    "desorientáis": "desorientar",
    "desorientáramos": "desorientar",
    "desorientáremos": "desorientar",
    "desorientásemos": "desorientar",
    "desorienté": "desorientar",
    "desorientéis": "desorientar",
    "desorientó": "desorientar",
    "desorilla": "desorillar",
    "desorillaba": "desorillar",
    "desorillabais": "desorillar",
    "desorillaban": "desorillar",
    "desorillabas": "desorillar",
    "desorillad": "desorillar",
    "desorillada": "desorillar",
    "desorilladas": "desorillar",
    "desorillado": "desorillar",
    "desorillados": "desorillar",
    "desorillamos": "desorillar",
    "desorillan": "desorillar",
    "desorillando": "desorillar",
    "desorillara": "desorillar",
    "desorillarais": "desorillar",
    "desorillaran": "desorillar",
    "desorillaras": "desorillar",
    "desorillare": "desorillar",
    "desorillareis": "desorillar",
    "desorillaremos": "desorillar",
    "desorillaren": "desorillar",
    "desorillares": "desorillar",
    "desorillaron": "desorillar",
    "desorillará": "desorillar",
    "desorillarán": "desorillar",
    "desorillarás": "desorillar",
    "desorillaré": "desorillar",
    "desorillaréis": "desorillar",
    "desorillaría": "desorillar",
    "desorillaríais": "desorillar",
    "desorillaríamos": "desorillar",
    "desorillarían": "desorillar",
    "desorillarías": "desorillar",
    "desorillas": "desorillar",
    "desorillase": "desorillar",
    "desorillaseis": "desorillar",
    "desorillasen": "desorillar",
    "desorillases": "desorillar",
    "desorillaste": "desorillar",
    "desorillasteis": "desorillar",
    "desorille": "desorillar",
    "desorillemos": "desorillar",
    "desorillen": "desorillar",
    "desorilles": "desorillar",
    "desorillo": "desorillar",
    "desorillábamos": "desorillar",
    "desorilláis": "desorillar",
    "desorilláramos": "desorillar",
    "desorilláremos": "desorillar",
    "desorillásemos": "desorillar",
    "desorillé": "desorillar",
    "desorilléis": "desorillar",
    "desorilló": "desorillar",
    "desornamentada": "desornamentado",
    "desornamentadas": "desornamentado",
    "desornamentados": "desornamentado",
    "desortija": "desortijar",
    "desortijaba": "desortijar",
    "desortijabais": "desortijar",
    "desortijaban": "desortijar",
    "desortijabas": "desortijar",
    "desortijad": "desortijar",
    "desortijada": "desortijar",
    "desortijadas": "desortijar",
    "desortijado": "desortijar",
    "desortijados": "desortijar",
    "desortijamos": "desortijar",
    "desortijan": "desortijar",
    "desortijando": "desortijar",
    "desortijara": "desortijar",
    "desortijarais": "desortijar",
    "desortijaran": "desortijar",
    "desortijaras": "desortijar",
    "desortijare": "desortijar",
    "desortijareis": "desortijar",
    "desortijaremos": "desortijar",
    "desortijaren": "desortijar",
    "desortijares": "desortijar",
    "desortijaron": "desortijar",
    "desortijará": "desortijar",
    "desortijarán": "desortijar",
    "desortijarás": "desortijar",
    "desortijaré": "desortijar",
    "desortijaréis": "desortijar",
    "desortijaría": "desortijar",
    "desortijaríais": "desortijar",
    "desortijaríamos": "desortijar",
    "desortijarían": "desortijar",
    "desortijarías": "desortijar",
    "desortijas": "desortijar",
    "desortijase": "desortijar",
    "desortijaseis": "desortijar",
    "desortijasen": "desortijar",
    "desortijases": "desortijar",
    "desortijaste": "desortijar",
    "desortijasteis": "desortijar",
    "desortije": "desortijar",
    "desortijemos": "desortijar",
    "desortijen": "desortijar",
    "desortijes": "desortijar",
    "desortijo": "desortijar",
    "desortijábamos": "desortijar",
    "desortijáis": "desortijar",
    "desortijáramos": "desortijar",
    "desortijáremos": "desortijar",
    "desortijásemos": "desortijar",
    "desortijé": "desortijar",
    "desortijéis": "desortijar",
    "desortijó": "desortijar",
    "desosaba": "desosar",
    "desosabais": "desosar",
    "desosaban": "desosar",
    "desosabas": "desosar",
    "desosad": "desosar",
    "desosada": "desosar",
    "desosadas": "desosar",
    "desosado": "desosar",
    "desosados": "desosar",
    "desosamos": "desosar",
    "desosando": "desosar",
    "desosara": "desosar",
    "desosarais": "desosar",
    "desosaran": "desosar",
    "desosaras": "desosar",
    "desosare": "desosar",
    "desosareis": "desosar",
    "desosaremos": "desosar",
    "desosaren": "desosar",
    "desosares": "desosar",
    "desosaron": "desosar",
    "desosará": "desosar",
    "desosarán": "desosar",
    "desosarás": "desosar",
    "desosaré": "desosar",
    "desosaréis": "desosar",
    "desosaría": "desosar",
    "desosaríais": "desosar",
    "desosaríamos": "desosar",
    "desosarían": "desosar",
    "desosarías": "desosar",
    "desosase": "desosar",
    "desosaseis": "desosar",
    "desosasen": "desosar",
    "desosases": "desosar",
    "desosaste": "desosar",
    "desosasteis": "desosar",
    "desosemos": "desosar",
    "desosiegos": "desosiego",
    "desosábamos": "desosar",
    "desosáis": "desosar",
    "desosáramos": "desosar",
    "desosáremos": "desosar",
    "desosásemos": "desosar",
    "desosé": "desosar",
    "desoséis": "desosar",
    "desosó": "desosar",
    "desova": "desovar",
    "desovaba": "desovar",
    "desovabais": "desovar",
    "desovaban": "desovar",
    "desovabas": "desovar",
    "desovad": "desovar",
    "desovada": "desovar",
    "desovadas": "desovar",
    "desovado": "desovar",
    "desovados": "desovar",
    "desovamos": "desovar",
    "desovan": "desovar",
    "desovando": "desovar",
    "desovara": "desovar",
    "desovarais": "desovar",
    "desovaran": "desovar",
    "desovaras": "desovar",
    "desovare": "desovar",
    "desovareis": "desovar",
    "desovaremos": "desovar",
    "desovaren": "desovar",
    "desovares": "desovar",
    "desovaron": "desovar",
    "desovará": "desovar",
    "desovarán": "desovar",
    "desovarás": "desovar",
    "desovaré": "desovar",
    "desovaréis": "desovar",
    "desovaría": "desovar",
    "desovaríais": "desovar",
    "desovaríamos": "desovar",
    "desovarían": "desovar",
    "desovarías": "desovar",
    "desovas": "desovar",
    "desovase": "desovar",
    "desovaseis": "desovar",
    "desovasen": "desovar",
    "desovases": "desovar",
    "desovaste": "desovar",
    "desovasteis": "desovar",
    "desove": "desovar",
    "desovemos": "desovar",
    "desoven": "desovar",
    "desoves": "desove",
    "desovilla": "desovillar",
    "desovillaba": "desovillar",
    "desovillabais": "desovillar",
    "desovillaban": "desovillar",
    "desovillabas": "desovillar",
    "desovillad": "desovillar",
    "desovillada": "desovillar",
    "desovilladas": "desovillar",
    "desovillado": "desovillar",
    "desovillados": "desovillar",
    "desovillamos": "desovillar",
    "desovillan": "desovillar",
    "desovillando": "desovillar",
    "desovillara": "desovillar",
    "desovillarais": "desovillar",
    "desovillaran": "desovillar",
    "desovillaras": "desovillar",
    "desovillare": "desovillar",
    "desovillareis": "desovillar",
    "desovillaremos": "desovillar",
    "desovillaren": "desovillar",
    "desovillares": "desovillar",
    "desovillaron": "desovillar",
    "desovillará": "desovillar",
    "desovillarán": "desovillar",
    "desovillarás": "desovillar",
    "desovillaré": "desovillar",
    "desovillaréis": "desovillar",
    "desovillaría": "desovillar",
    "desovillaríais": "desovillar",
    "desovillaríamos": "desovillar",
    "desovillarían": "desovillar",
    "desovillarías": "desovillar",
    "desovillas": "desovillar",
    "desovillase": "desovillar",
    "desovillaseis": "desovillar",
    "desovillasen": "desovillar",
    "desovillases": "desovillar",
    "desovillaste": "desovillar",
    "desovillasteis": "desovillar",
    "desoville": "desovillar",
    "desovillemos": "desovillar",
    "desovillen": "desovillar",
    "desovilles": "desovillar",
    "desovillo": "desovillar",
    "desovillábamos": "desovillar",
    "desovilláis": "desovillar",
    "desovilláramos": "desovillar",
    "desovilláremos": "desovillar",
    "desovillásemos": "desovillar",
    "desovillé": "desovillar",
    "desovilléis": "desovillar",
    "desovilló": "desovillar",
    "desovo": "desovar",
    "desovábamos": "desovar",
    "desováis": "desovar",
    "desováramos": "desovar",
    "desováremos": "desovar",
    "desovásemos": "desovar",
    "desové": "desovar",
    "desovéis": "desovar",
    "desovó": "desovar",
    "desoxicorticosteronas": "desoxicorticosterona",
    "desoxida": "desoxidar",
    "desoxidaba": "desoxidar",
    "desoxidabais": "desoxidar",
    "desoxidaban": "desoxidar",
    "desoxidabas": "desoxidar",
    "desoxidables": "desoxidable",
    "desoxidaciones": "desoxidación",
    "desoxidad": "desoxidar",
    "desoxidada": "desoxidar",
    "desoxidadas": "desoxidar",
    "desoxidado": "desoxidar",
    "desoxidados": "desoxidar",
    "desoxidamos": "desoxidar",
    "desoxidan": "desoxidar",
    "desoxidando": "desoxidar",
    "desoxidantes": "desoxidante",
    "desoxidara": "desoxidar",
    "desoxidarais": "desoxidar",
    "desoxidaran": "desoxidar",
    "desoxidaras": "desoxidar",
    "desoxidare": "desoxidar",
    "desoxidareis": "desoxidar",
    "desoxidaremos": "desoxidar",
    "desoxidaren": "desoxidar",
    "desoxidares": "desoxidar",
    "desoxidaron": "desoxidar",
    "desoxidará": "desoxidar",
    "desoxidarán": "desoxidar",
    "desoxidarás": "desoxidar",
    "desoxidaré": "desoxidar",
    "desoxidaréis": "desoxidar",
    "desoxidaría": "desoxidar",
    "desoxidaríais": "desoxidar",
    "desoxidaríamos": "desoxidar",
    "desoxidarían": "desoxidar",
    "desoxidarías": "desoxidar",
    "desoxidas": "desoxidar",
    "desoxidase": "desoxidar",
    "desoxidaseis": "desoxidar",
    "desoxidasen": "desoxidar",
    "desoxidases": "desoxidar",
    "desoxidaste": "desoxidar",
    "desoxidasteis": "desoxidar",
    "desoxide": "desoxidar",
    "desoxidemos": "desoxidar",
    "desoxiden": "desoxidar",
    "desoxides": "desoxidar",
    "desoxido": "desoxidar",
    "desoxidorribonucleicos": "desoxidorribonucleico",
    "desoxidábamos": "desoxidar",
    "desoxidáis": "desoxidar",
    "desoxidáramos": "desoxidar",
    "desoxidáremos": "desoxidar",
    "desoxidásemos": "desoxidar",
    "desoxidé": "desoxidar",
    "desoxidéis": "desoxidar",
    "desoxidó": "desoxidar",
    "desoxigena": "desoxigenar",
    "desoxigenaba": "desoxigenar",
    "desoxigenabais": "desoxigenar",
    "desoxigenaban": "desoxigenar",
    "desoxigenabas": "desoxigenar",
    "desoxigenaciones": "desoxigenación",
    "desoxigenad": "desoxigenar",
    "desoxigenada": "desoxigenar",
    "desoxigenadas": "desoxigenar",
    "desoxigenado": "desoxigenar",
    "desoxigenados": "desoxigenar",
    "desoxigenamos": "desoxigenar",
    "desoxigenan": "desoxigenar",
    "desoxigenando": "desoxigenar",
    "desoxigenantes": "desoxigenante",
    "desoxigenara": "desoxigenar",
    "desoxigenarais": "desoxigenar",
    "desoxigenaran": "desoxigenar",
    "desoxigenaras": "desoxigenar",
    "desoxigenare": "desoxigenar",
    "desoxigenareis": "desoxigenar",
    "desoxigenaremos": "desoxigenar",
    "desoxigenaren": "desoxigenar",
    "desoxigenares": "desoxigenar",
    "desoxigenaron": "desoxigenar",
    "desoxigenará": "desoxigenar",
    "desoxigenarán": "desoxigenar",
    "desoxigenarás": "desoxigenar",
    "desoxigenaré": "desoxigenar",
    "desoxigenaréis": "desoxigenar",
    "desoxigenaría": "desoxigenar",
    "desoxigenaríais": "desoxigenar",
    "desoxigenaríamos": "desoxigenar",
    "desoxigenarían": "desoxigenar",
    "desoxigenarías": "desoxigenar",
    "desoxigenas": "desoxigenar",
    "desoxigenase": "desoxigenar",
    "desoxigenaseis": "desoxigenar",
    "desoxigenasen": "desoxigenar",
    "desoxigenases": "desoxigenar",
    "desoxigenaste": "desoxigenar",
    "desoxigenasteis": "desoxigenar",
    "desoxigene": "desoxigenar",
    "desoxigenemos": "desoxigenar",
    "desoxigenen": "desoxigenar",
    "desoxigenes": "desoxigenar",
    "desoxigeno": "desoxigenar",
    "desoxigenábamos": "desoxigenar",
    "desoxigenáis": "desoxigenar",
    "desoxigenáramos": "desoxigenar",
    "desoxigenáremos": "desoxigenar",
    "desoxigenásemos": "desoxigenar",
    "desoxigené": "desoxigenar",
    "desoxigenéis": "desoxigenar",
    "desoxigenó": "desoxigenar",
    "desoxirribonucleica": "desoxirribonucleico",
    "desoxirribonucleicas": "desoxirribonucleico",
    "desoxirribonucleicos": "desoxirribonucleico",
    "desoxirribosas": "desoxirribosa",
    "desoye": "desoír",
    "desoyen": "desoír",
    "desoyendo": "desoír",
    "desoyera": "desoír",
    "desoyerais": "desoír",
    "desoyeran": "desoír",
    "desoyeras": "desoír",
    "desoyere": "desoír",
    "desoyereis": "desoír",
    "desoyeren": "desoír",
    "desoyeres": "desoír",
    "desoyeron": "desoír",
    "desoyes": "desoír",
    "desoyese": "desoír",
    "desoyeseis": "desoír",
    "desoyesen": "desoír",
    "desoyeses": "desoír",
    "desoyéramos": "desoír",
    "desoyéremos": "desoír",
    "desoyésemos": "desoír",
    "desoyó": "desoír",
    "desoí": "desoír",
    "desoía": "desoír",
    "desoíais": "desoír",
    "desoíamos": "desoír",
    "desoían": "desoír",
    "desoías": "desoír",
    "desoíd": "desoír",
    "desoída": "desoír",
    "desoídas": "desoír",
    "desoído": "desoír",
    "desoídos": "desoír",
    "desoímos": "desoír",
    "desoís": "desoír",
    "desoíste": "desoír",
    "desoísteis": "desoír",
    "despabila": "despabilar",
    "despabilaba": "despabilar",
    "despabilabais": "despabilar",
    "despabilaban": "despabilar",
    "despabilabas": "despabilar",
    "despabilad": "despabilar",
    "despabilada": "despabilar",
    "despabiladas": "despabilar",
    "despabilado": "despabilar",
    "despabiladora": "despabilador",
    "despabiladoras": "despabilador",
    "despabiladores": "despabilador",
    "despabilados": "despabilar",
    "despabiladuras": "despabiladura",
    "despabilamientos": "despabilamiento",
    "despabilamos": "despabilar",
    "despabilan": "despabilar",
    "despabilando": "despabilar",
    "despabilara": "despabilar",
    "despabilarais": "despabilar",
    "despabilaran": "despabilar",
    "despabilaras": "despabilar",
    "despabilare": "despabilar",
    "despabilareis": "despabilar",
    "despabilaremos": "despabilar",
    "despabilaren": "despabilar",
    "despabilares": "despabilar",
    "despabilaron": "despabilar",
    "despabilará": "despabilar",
    "despabilarán": "despabilar",
    "despabilarás": "despabilar",
    "despabilaré": "despabilar",
    "despabilaréis": "despabilar",
    "despabilaría": "despabilar",
    "despabilaríais": "despabilar",
    "despabilaríamos": "despabilar",
    "despabilarían": "despabilar",
    "despabilarías": "despabilar",
    "despabilas": "despabilar",
    "despabilase": "despabilar",
    "despabilaseis": "despabilar",
    "despabilasen": "despabilar",
    "despabilases": "despabilar",
    "despabilaste": "despabilar",
    "despabilasteis": "despabilar",
    "despabile": "despabilar",
    "despabilemos": "despabilar",
    "despabilen": "despabilar",
    "despabiles": "despabilar",
    "despabilo": "despabilar",
    "despabilábamos": "despabilar",
    "despabiláis": "despabilar",
    "despabiláramos": "despabilar",
    "despabiláremos": "despabilar",
    "despabilásemos": "despabilar",
    "despabilé": "despabilar",
    "despabiléis": "despabilar",
    "despabiló": "despabilar",
    "despacha": "despachar",
    "despachaba": "despachar",
    "despachabais": "despachar",
    "despachaban": "despachar",
    "despachabas": "despachar",
    "despachad": "despachar",
    "despachada": "despachar",
    "despachadas": "despachar",
    "despachado": "despachar",
    "despachadora": "despachador",
    "despachadoras": "despachador",
    "despachadores": "despachador",
    "despachados": "despachar",
    "despachamos": "despachar",
    "despachan": "despachar",
    "despachando": "despachar",
    "despachara": "despachar",
    "despacharais": "despachar",
    "despacharan": "despachar",
    "despacharas": "despachar",
    "despachare": "despachar",
    "despachareis": "despachar",
    "despacharemos": "despachar",
    "despacharen": "despachar",
    "despachares": "despachar",
    "despacharon": "despachar",
    "despachará": "despachar",
    "despacharán": "despachar",
    "despacharás": "despachar",
    "despacharé": "despachar",
    "despacharéis": "despachar",
    "despacharía": "despachar",
    "despacharíais": "despachar",
    "despacharíamos": "despachar",
    "despacharían": "despachar",
    "despacharías": "despachar",
    "despachas": "despachar",
    "despachase": "despachar",
    "despachaseis": "despachar",
    "despachasen": "despachar",
    "despachases": "despachar",
    "despachaste": "despachar",
    "despachasteis": "despachar",
    "despache": "despachar",
    "despachemos": "despachar",
    "despachen": "despachar",
    "despaches": "despachar",
    "despacho": "despachar",
    "despachos": "despacho",
    "despachurra": "despachurrar",
    "despachurraba": "despachurrar",
    "despachurrabais": "despachurrar",
    "despachurraban": "despachurrar",
    "despachurrabas": "despachurrar",
    "despachurrad": "despachurrar",
    "despachurrada": "despachurrar",
    "despachurradas": "despachurrar",
    "despachurrado": "despachurrar",
    "despachurrados": "despachurrar",
    "despachurramientos": "despachurramiento",
    "despachurramos": "despachurrar",
    "despachurran": "despachurrar",
    "despachurrando": "despachurrar",
    "despachurrara": "despachurrar",
    "despachurrarais": "despachurrar",
    "despachurraran": "despachurrar",
    "despachurraras": "despachurrar",
    "despachurrare": "despachurrar",
    "despachurrareis": "despachurrar",
    "despachurraremos": "despachurrar",
    "despachurraren": "despachurrar",
    "despachurrares": "despachurrar",
    "despachurraron": "despachurrar",
    "despachurrará": "despachurrar",
    "despachurrarán": "despachurrar",
    "despachurrarás": "despachurrar",
    "despachurraré": "despachurrar",
    "despachurraréis": "despachurrar",
    "despachurraría": "despachurrar",
    "despachurraríais": "despachurrar",
    "despachurraríamos": "despachurrar",
    "despachurrarían": "despachurrar",
    "despachurrarías": "despachurrar",
    "despachurras": "despachurrar",
    "despachurrase": "despachurrar",
    "despachurraseis": "despachurrar",
    "despachurrasen": "despachurrar",
    "despachurrases": "despachurrar",
    "despachurraste": "despachurrar",
    "despachurrasteis": "despachurrar",
    "despachurre": "despachurrar",
    "despachurremos": "despachurrar",
    "despachurren": "despachurrar",
    "despachurres": "despachurrar",
    "despachurro": "despachurrar",
    "despachurrábamos": "despachurrar",
    "despachurráis": "despachurrar",
    "despachurráramos": "despachurrar",
    "despachurráremos": "despachurrar",
    "despachurrásemos": "despachurrar",
    "despachurré": "despachurrar",
    "despachurréis": "despachurrar",
    "despachurró": "despachurrar",
    "despachábamos": "despachar",
    "despacháis": "despachar",
    "despacháramos": "despachar",
    "despacháremos": "despachar",
    "despachásemos": "despachar",
    "despaché": "despachar",
    "despachéis": "despachar",
    "despachó": "despachar",
    "despacios": "despacio",
    "despaciosa": "despacioso",
    "despaciosas": "despacioso",
    "despaciosos": "despacioso",
    "despaja": "despajar",
    "despajaba": "despajar",
    "despajabais": "despajar",
    "despajaban": "despajar",
    "despajabas": "despajar",
    "despajad": "despajar",
    "despajada": "despajar",
    "despajadas": "despajar",
    "despajado": "despajar",
    "despajadora": "despajador",
    "despajadoras": "despajador",
    "despajadores": "despajador",
    "despajados": "despajar",
    "despajaduras": "despajadura",
    "despajamos": "despajar",
    "despajan": "despajar",
    "despajando": "despajar",
    "despajara": "despajar",
    "despajarais": "despajar",
    "despajaran": "despajar",
    "despajaras": "despajar",
    "despajare": "despajar",
    "despajareis": "despajar",
    "despajaremos": "despajar",
    "despajaren": "despajar",
    "despajares": "despajar",
    "despajaron": "despajar",
    "despajará": "despajar",
    "despajarán": "despajar",
    "despajarás": "despajar",
    "despajaré": "despajar",
    "despajaréis": "despajar",
    "despajaría": "despajar",
    "despajaríais": "despajar",
    "despajaríamos": "despajar",
    "despajarían": "despajar",
    "despajarías": "despajar",
    "despajas": "despajar",
    "despajase": "despajar",
    "despajaseis": "despajar",
    "despajasen": "despajar",
    "despajases": "despajar",
    "despajaste": "despajar",
    "despajasteis": "despajar",
    "despaje": "despajar",
    "despajemos": "despajar",
    "despajen": "despajar",
    "despajes": "despajar",
    "despajo": "despajar",
    "despajos": "despajo",
    "despajábamos": "despajar",
    "despajáis": "despajar",
    "despajáramos": "despajar",
    "despajáremos": "despajar",
    "despajásemos": "despajar",
    "despajé": "despajar",
    "despajéis": "despajar",
    "despajó": "despajar",
    "despalatalizaciones": "despalatalización",
    "despalda": "despaldar",
    "despaldaba": "despaldar",
    "despaldabais": "despaldar",
    "despaldaban": "despaldar",
    "despaldabas": "despaldar",
    "despaldad": "despaldar",
    "despaldada": "despaldar",
    "despaldadas": "despaldar",
    "despaldado": "despaldar",
    "despaldados": "despaldar",
    "despaldamos": "despaldar",
    "despaldan": "despaldar",
    "despaldando": "despaldar",
    "despaldara": "despaldar",
    "despaldarais": "despaldar",
    "despaldaran": "despaldar",
    "despaldaras": "despaldar",
    "despaldare": "despaldar",
    "despaldareis": "despaldar",
    "despaldaremos": "despaldar",
    "despaldaren": "despaldar",
    "despaldares": "despaldar",
    "despaldaron": "despaldar",
    "despaldará": "despaldar",
    "despaldarán": "despaldar",
    "despaldarás": "despaldar",
    "despaldaré": "despaldar",
    "despaldaréis": "despaldar",
    "despaldaría": "despaldar",
    "despaldaríais": "despaldar",
    "despaldaríamos": "despaldar",
    "despaldarían": "despaldar",
    "despaldarías": "despaldar",
    "despaldas": "despaldar",
    "despaldase": "despaldar",
    "despaldaseis": "despaldar",
    "despaldasen": "despaldar",
    "despaldases": "despaldar",
    "despaldaste": "despaldar",
    "despaldasteis": "despaldar",
    "despalde": "despaldar",
    "despaldemos": "despaldar",
    "despalden": "despaldar",
    "despaldes": "despaldar",
    "despaldilla": "despaldillar",
    "despaldillaba": "despaldillar",
    "despaldillabais": "despaldillar",
    "despaldillaban": "despaldillar",
    "despaldillabas": "despaldillar",
    "despaldillad": "despaldillar",
    "despaldillada": "despaldillar",
    "despaldilladas": "despaldillar",
    "despaldillado": "despaldillar",
    "despaldillados": "despaldillar",
    "despaldilladuras": "despaldilladura",
    "despaldillamos": "despaldillar",
    "despaldillan": "despaldillar",
    "despaldillando": "despaldillar",
    "despaldillara": "despaldillar",
    "despaldillarais": "despaldillar",
    "despaldillaran": "despaldillar",
    "despaldillaras": "despaldillar",
    "despaldillare": "despaldillar",
    "despaldillareis": "despaldillar",
    "despaldillaremos": "despaldillar",
    "despaldillaren": "despaldillar",
    "despaldillares": "despaldillar",
    "despaldillaron": "despaldillar",
    "despaldillará": "despaldillar",
    "despaldillarán": "despaldillar",
    "despaldillarás": "despaldillar",
    "despaldillaré": "despaldillar",
    "despaldillaréis": "despaldillar",
    "despaldillaría": "despaldillar",
    "despaldillaríais": "despaldillar",
    "despaldillaríamos": "despaldillar",
    "despaldillarían": "despaldillar",
    "despaldillarías": "despaldillar",
    "despaldillas": "despaldillar",
    "despaldillase": "despaldillar",
    "despaldillaseis": "despaldillar",
    "despaldillasen": "despaldillar",
    "despaldillases": "despaldillar",
    "despaldillaste": "despaldillar",
    "despaldillasteis": "despaldillar",
    "despaldille": "despaldillar",
    "despaldillemos": "despaldillar",
    "despaldillen": "despaldillar",
    "despaldilles": "despaldillar",
    "despaldillo": "despaldillar",
    "despaldillábamos": "despaldillar",
    "despaldilláis": "despaldillar",
    "despaldilláramos": "despaldillar",
    "despaldilláremos": "despaldillar",
    "despaldillásemos": "despaldillar",
    "despaldillé": "despaldillar",
    "despaldilléis": "despaldillar",
    "despaldilló": "despaldillar",
    "despaldo": "despaldar",
    "despaldábamos": "despaldar",
    "despaldáis": "despaldar",
    "despaldáramos": "despaldar",
    "despaldáremos": "despaldar",
    "despaldásemos": "despaldar",
    "despaldé": "despaldar",
    "despaldéis": "despaldar",
    "despaldó": "despaldar",
    "despaletilla": "despaletillar",
    "despaletillaba": "despaletillar",
    "despaletillabais": "despaletillar",
    "despaletillaban": "despaletillar",
    "despaletillabas": "despaletillar",
    "despaletillad": "despaletillar",
    "despaletillada": "despaletillar",
    "despaletilladas": "despaletillar",
    "despaletillado": "despaletillar",
    "despaletillados": "despaletillar",
    "despaletillamos": "despaletillar",
    "despaletillan": "despaletillar",
    "despaletillando": "despaletillar",
    "despaletillara": "despaletillar",
    "despaletillarais": "despaletillar",
    "despaletillaran": "despaletillar",
    "despaletillaras": "despaletillar",
    "despaletillare": "despaletillar",
    "despaletillareis": "despaletillar",
    "despaletillaremos": "despaletillar",
    "despaletillaren": "despaletillar",
    "despaletillares": "despaletillar",
    "despaletillaron": "despaletillar",
    "despaletillará": "despaletillar",
    "despaletillarán": "despaletillar",
    "despaletillarás": "despaletillar",
    "despaletillaré": "despaletillar",
    "despaletillaréis": "despaletillar",
    "despaletillaría": "despaletillar",
    "despaletillaríais": "despaletillar",
    "despaletillaríamos": "despaletillar",
    "despaletillarían": "despaletillar",
    "despaletillarías": "despaletillar",
    "despaletillas": "despaletillar",
    "despaletillase": "despaletillar",
    "despaletillaseis": "despaletillar",
    "despaletillasen": "despaletillar",
    "despaletillases": "despaletillar",
    "despaletillaste": "despaletillar",
    "despaletillasteis": "despaletillar",
    "despaletille": "despaletillar",
    "despaletillemos": "despaletillar",
    "despaletillen": "despaletillar",
    "despaletilles": "despaletillar",
    "despaletillo": "despaletillar",
    "despaletillábamos": "despaletillar",
    "despaletilláis": "despaletillar",
    "despaletilláramos": "despaletillar",
    "despaletilláremos": "despaletillar",
    "despaletillásemos": "despaletillar",
    "despaletillé": "despaletillar",
    "despaletilléis": "despaletillar",
    "despaletilló": "despaletillar",
    "despalillados": "despalillado",
    "despalma": "despalmar",
    "despalmaba": "despalmar",
    "despalmabais": "despalmar",
    "despalmaban": "despalmar",
    "despalmabas": "despalmar",
    "despalmad": "despalmar",
    "despalmada": "despalmar",
    "despalmadas": "despalmar",
    "despalmado": "despalmar",
    "despalmadores": "despalmador",
    "despalmados": "despalmar",
    "despalmaduras": "despalmadura",
    "despalmamos": "despalmar",
    "despalman": "despalmar",
    "despalmando": "despalmar",
    "despalmara": "despalmar",
    "despalmarais": "despalmar",
    "despalmaran": "despalmar",
    "despalmaras": "despalmar",
    "despalmare": "despalmar",
    "despalmareis": "despalmar",
    "despalmaremos": "despalmar",
    "despalmaren": "despalmar",
    "despalmares": "despalmar",
    "despalmaron": "despalmar",
    "despalmará": "despalmar",
    "despalmarán": "despalmar",
    "despalmarás": "despalmar",
    "despalmaré": "despalmar",
    "despalmaréis": "despalmar",
    "despalmaría": "despalmar",
    "despalmaríais": "despalmar",
    "despalmaríamos": "despalmar",
    "despalmarían": "despalmar",
    "despalmarías": "despalmar",
    "despalmas": "despalmar",
    "despalmase": "despalmar",
    "despalmaseis": "despalmar",
    "despalmasen": "despalmar",
    "despalmases": "despalmar",
    "despalmaste": "despalmar",
    "despalmasteis": "despalmar",
    "despalme": "despalmar",
    "despalmemos": "despalmar",
    "despalmen": "despalmar",
    "despalmes": "despalme",
    "despalmo": "despalmar",
    "despalmábamos": "despalmar",
    "despalmáis": "despalmar",
    "despalmáramos": "despalmar",
    "despalmáremos": "despalmar",
    "despalmásemos": "despalmar",
    "despalmé": "despalmar",
    "despalméis": "despalmar",
    "despalmó": "despalmar",
    "despampana": "despampanar",
    "despampanaba": "despampanar",
    "despampanabais": "despampanar",
    "despampanaban": "despampanar",
    "despampanabas": "despampanar",
    "despampanad": "despampanar",
    "despampanada": "despampanar",
    "despampanadas": "despampanar",
    "despampanado": "despampanar",
    "despampanados": "despampanar",
    "despampanaduras": "despampanadura",
    "despampanamos": "despampanar",
    "despampanan": "despampanar",
    "despampanando": "despampanar",
    "despampanantes": "despampanante",
    "despampanara": "despampanar",
    "despampanarais": "despampanar",
    "despampanaran": "despampanar",
    "despampanaras": "despampanar",
    "despampanare": "despampanar",
    "despampanareis": "despampanar",
    "despampanaremos": "despampanar",
    "despampanaren": "despampanar",
    "despampanares": "despampanar",
    "despampanaron": "despampanar",
    "despampanará": "despampanar",
    "despampanarán": "despampanar",
    "despampanarás": "despampanar",
    "despampanaré": "despampanar",
    "despampanaréis": "despampanar",
    "despampanaría": "despampanar",
    "despampanaríais": "despampanar",
    "despampanaríamos": "despampanar",
    "despampanarían": "despampanar",
    "despampanarías": "despampanar",
    "despampanas": "despampanar",
    "despampanase": "despampanar",
    "despampanaseis": "despampanar",
    "despampanasen": "despampanar",
    "despampanases": "despampanar",
    "despampanaste": "despampanar",
    "despampanasteis": "despampanar",
    "despampane": "despampanar",
    "despampanemos": "despampanar",
    "despampanen": "despampanar",
    "despampanes": "despampanar",
    "despampanilla": "despampanillar",
    "despampanillaba": "despampanillar",
    "despampanillabais": "despampanillar",
    "despampanillaban": "despampanillar",
    "despampanillabas": "despampanillar",
    "despampanillad": "despampanillar",
    "despampanillada": "despampanillar",
    "despampanilladas": "despampanillar",
    "despampanillado": "despampanillar",
    "despampanillados": "despampanillar",
    "despampanillamos": "despampanillar",
    "despampanillan": "despampanillar",
    "despampanillando": "despampanillar",
    "despampanillara": "despampanillar",
    "despampanillarais": "despampanillar",
    "despampanillaran": "despampanillar",
    "despampanillaras": "despampanillar",
    "despampanillare": "despampanillar",
    "despampanillareis": "despampanillar",
    "despampanillaremos": "despampanillar",
    "despampanillaren": "despampanillar",
    "despampanillares": "despampanillar",
    "despampanillaron": "despampanillar",
    "despampanillará": "despampanillar",
    "despampanillarán": "despampanillar",
    "despampanillarás": "despampanillar",
    "despampanillaré": "despampanillar",
    "despampanillaréis": "despampanillar",
    "despampanillaría": "despampanillar",
    "despampanillaríais": "despampanillar",
    "despampanillaríamos": "despampanillar",
    "despampanillarían": "despampanillar",
    "despampanillarías": "despampanillar",
    "despampanillas": "despampanillar",
    "despampanillase": "despampanillar",
    "despampanillaseis": "despampanillar",
    "despampanillasen": "despampanillar",
    "despampanillases": "despampanillar",
    "despampanillaste": "despampanillar",
    "despampanillasteis": "despampanillar",
    "despampanille": "despampanillar",
    "despampanillemos": "despampanillar",
    "despampanillen": "despampanillar",
    "despampanilles": "despampanillar",
    "despampanillo": "despampanillar",
    "despampanillábamos": "despampanillar",
    "despampanilláis": "despampanillar",
    "despampanilláramos": "despampanillar",
    "despampanilláremos": "despampanillar",
    "despampanillásemos": "despampanillar",
    "despampanillé": "despampanillar",
    "despampanilléis": "despampanillar",
    "despampanilló": "despampanillar",
    "despampano": "despampanar",
    "despampanos": "despampano",
    "despampanábamos": "despampanar",
    "despampanáis": "despampanar",
    "despampanáramos": "despampanar",
    "despampanáremos": "despampanar",
    "despampanásemos": "despampanar",
    "despampané": "despampanar",
    "despampanéis": "despampanar",
    "despampanó": "despampanar",
    "despanchurra": "despanchurrar",
    "despanchurraba": "despanchurrar",
    "despanchurrabais": "despanchurrar",
    "despanchurraban": "despanchurrar",
    "despanchurrabas": "despanchurrar",
    "despanchurrad": "despanchurrar",
    "despanchurrada": "despanchurrar",
    "despanchurradas": "despanchurrar",
    "despanchurrado": "despanchurrar",
    "despanchurrados": "despanchurrar",
    "despanchurramos": "despanchurrar",
    "despanchurran": "despanchurrar",
    "despanchurrando": "despanchurrar",
    "despanchurrara": "despanchurrar",
    "despanchurrarais": "despanchurrar",
    "despanchurraran": "despanchurrar",
    "despanchurraras": "despanchurrar",
    "despanchurrare": "despanchurrar",
    "despanchurrareis": "despanchurrar",
    "despanchurraremos": "despanchurrar",
    "despanchurraren": "despanchurrar",
    "despanchurrares": "despanchurrar",
    "despanchurraron": "despanchurrar",
    "despanchurrará": "despanchurrar",
    "despanchurrarán": "despanchurrar",
    "despanchurrarás": "despanchurrar",
    "despanchurraré": "despanchurrar",
    "despanchurraréis": "despanchurrar",
    "despanchurraría": "despanchurrar",
    "despanchurraríais": "despanchurrar",
    "despanchurraríamos": "despanchurrar",
    "despanchurrarían": "despanchurrar",
    "despanchurrarías": "despanchurrar",
    "despanchurras": "despanchurrar",
    "despanchurrase": "despanchurrar",
    "despanchurraseis": "despanchurrar",
    "despanchurrasen": "despanchurrar",
    "despanchurrases": "despanchurrar",
    "despanchurraste": "despanchurrar",
    "despanchurrasteis": "despanchurrar",
    "despanchurre": "despanchurrar",
    "despanchurremos": "despanchurrar",
    "despanchurren": "despanchurrar",
    "despanchurres": "despanchurrar",
    "despanchurro": "despanchurrar",
    "despanchurrábamos": "despanchurrar",
    "despanchurráis": "despanchurrar",
    "despanchurráramos": "despanchurrar",
    "despanchurráremos": "despanchurrar",
    "despanchurrásemos": "despanchurrar",
    "despanchurré": "despanchurrar",
    "despanchurréis": "despanchurrar",
    "despanchurró": "despanchurrar",
    "despanzurra": "despanzurrar",
    "despanzurraba": "despanzurrar",
    "despanzurrabais": "despanzurrar",
    "despanzurraban": "despanzurrar",
    "despanzurrabas": "despanzurrar",
    "despanzurrad": "despanzurrar",
    "despanzurrada": "despanzurrar",
    "despanzurradas": "despanzurrar",
    "despanzurrado": "despanzurrar",
    "despanzurrados": "despanzurrar",
    "despanzurramientos": "despanzurramiento",
    "despanzurramos": "despanzurrar",
    "despanzurran": "despanzurrar",
    "despanzurrando": "despanzurrar",
    "despanzurrara": "despanzurrar",
    "despanzurrarais": "despanzurrar",
    "despanzurraran": "despanzurrar",
    "despanzurraras": "despanzurrar",
    "despanzurrare": "despanzurrar",
    "despanzurrareis": "despanzurrar",
    "despanzurraremos": "despanzurrar",
    "despanzurraren": "despanzurrar",
    "despanzurrares": "despanzurrar",
    "despanzurraron": "despanzurrar",
    "despanzurrará": "despanzurrar",
    "despanzurrarán": "despanzurrar",
    "despanzurrarás": "despanzurrar",
    "despanzurraré": "despanzurrar",
    "despanzurraréis": "despanzurrar",
    "despanzurraría": "despanzurrar",
    "despanzurraríais": "despanzurrar",
    "despanzurraríamos": "despanzurrar",
    "despanzurrarían": "despanzurrar",
    "despanzurrarías": "despanzurrar",
    "despanzurras": "despanzurrar",
    "despanzurrase": "despanzurrar",
    "despanzurraseis": "despanzurrar",
    "despanzurrasen": "despanzurrar",
    "despanzurrases": "despanzurrar",
    "despanzurraste": "despanzurrar",
    "despanzurrasteis": "despanzurrar",
    "despanzurre": "despanzurrar",
    "despanzurremos": "despanzurrar",
    "despanzurren": "despanzurrar",
    "despanzurres": "despanzurrar",
    "despanzurro": "despanzurrar",
    "despanzurrábamos": "despanzurrar",
    "despanzurráis": "despanzurrar",
    "despanzurráramos": "despanzurrar",
    "despanzurráremos": "despanzurrar",
    "despanzurrásemos": "despanzurrar",
    "despanzurré": "despanzurrar",
    "despanzurréis": "despanzurrar",
    "despanzurró": "despanzurrar",
    "despapa": "despapar",
    "despapaba": "despapar",
    "despapabais": "despapar",
    "despapaban": "despapar",
    "despapabas": "despapar",
    "despapad": "despapar",
    "despapada": "despapar",
    "despapadas": "despapar",
    "despapado": "despapar",
    "despapados": "despapar",
    "despapamos": "despapar",
    "despapan": "despapar",
    "despapando": "despapar",
    "despapara": "despapar",
    "despaparais": "despapar",
    "despaparan": "despapar",
    "despaparas": "despapar",
    "despapare": "despapar",
    "despapareis": "despapar",
    "despaparemos": "despapar",
    "despaparen": "despapar",
    "despapares": "despapar",
    "despaparon": "despapar",
    "despapará": "despapar",
    "despaparán": "despapar",
    "despaparás": "despapar",
    "despaparé": "despapar",
    "despaparéis": "despapar",
    "despaparía": "despapar",
    "despaparíais": "despapar",
    "despaparíamos": "despapar",
    "despaparían": "despapar",
    "despaparías": "despapar",
    "despapas": "despapar",
    "despapase": "despapar",
    "despapaseis": "despapar",
    "despapasen": "despapar",
    "despapases": "despapar",
    "despapaste": "despapar",
    "despapasteis": "despapar",
    "despape": "despapar",
    "despapemos": "despapar",
    "despapen": "despapar",
    "despapes": "despapar",
    "despapo": "despapar",
    "despapuchos": "despapucho",
    "despapábamos": "despapar",
    "despapáis": "despapar",
    "despapáramos": "despapar",
    "despapáremos": "despapar",
    "despapásemos": "despapar",
    "despapé": "despapar",
    "despapéis": "despapar",
    "despapó": "despapar",
    "desparasitajes": "desparasitaje",
    "desparea": "desparear",
    "despareaba": "desparear",
    "despareabais": "desparear",
    "despareaban": "desparear",
    "despareabas": "desparear",
    "desparead": "desparear",
    "despareada": "desparear",
    "despareadas": "desparear",
    "despareado": "desparear",
    "despareados": "desparear",
    "despareamos": "desparear",
    "desparean": "desparear",
    "despareando": "desparear",
    "despareara": "desparear",
    "desparearais": "desparear",
    "desparearan": "desparear",
    "desparearas": "desparear",
    "despareare": "desparear",
    "despareareis": "desparear",
    "desparearemos": "desparear",
    "desparearen": "desparear",
    "despareares": "desparear",
    "desparearon": "desparear",
    "despareará": "desparear",
    "desparearán": "desparear",
    "desparearás": "desparear",
    "desparearé": "desparear",
    "desparearéis": "desparear",
    "desparearía": "desparear",
    "desparearíais": "desparear",
    "desparearíamos": "desparear",
    "desparearían": "desparear",
    "desparearías": "desparear",
    "despareas": "desparear",
    "desparease": "desparear",
    "despareaseis": "desparear",
    "despareasen": "desparear",
    "despareases": "desparear",
    "despareaste": "desparear",
    "despareasteis": "desparear",
    "despareda": "desparedar",
    "desparedaba": "desparedar",
    "desparedabais": "desparedar",
    "desparedaban": "desparedar",
    "desparedabas": "desparedar",
    "desparedad": "desparedar",
    "desparedada": "desparedar",
    "desparedadas": "desparedar",
    "desparedado": "desparedar",
    "desparedados": "desparedar",
    "desparedamos": "desparedar",
    "desparedan": "desparedar",
    "desparedando": "desparedar",
    "desparedara": "desparedar",
    "desparedarais": "desparedar",
    "desparedaran": "desparedar",
    "desparedaras": "desparedar",
    "desparedare": "desparedar",
    "desparedareis": "desparedar",
    "desparedaremos": "desparedar",
    "desparedaren": "desparedar",
    "desparedares": "desparedar",
    "desparedaron": "desparedar",
    "desparedará": "desparedar",
    "desparedarán": "desparedar",
    "desparedarás": "desparedar",
    "desparedaré": "desparedar",
    "desparedaréis": "desparedar",
    "desparedaría": "desparedar",
    "desparedaríais": "desparedar",
    "desparedaríamos": "desparedar",
    "desparedarían": "desparedar",
    "desparedarías": "desparedar",
    "desparedas": "desparedar",
    "desparedase": "desparedar",
    "desparedaseis": "desparedar",
    "desparedasen": "desparedar",
    "desparedases": "desparedar",
    "desparedaste": "desparedar",
    "desparedasteis": "desparedar",
    "desparede": "desparedar",
    "desparedemos": "desparedar",
    "despareden": "desparedar",
    "desparedes": "desparedar",
    "desparedo": "desparedar",
    "desparedábamos": "desparedar",
    "desparedáis": "desparedar",
    "desparedáramos": "desparedar",
    "desparedáremos": "desparedar",
    "desparedásemos": "desparedar",
    "desparedé": "desparedar",
    "desparedéis": "desparedar",
    "desparedó": "desparedar",
    "desparee": "desparear",
    "despareemos": "desparear",
    "despareen": "desparear",
    "desparees": "desparear",
    "despareja": "desparejo",
    "desparejaba": "desparejar",
    "desparejabais": "desparejar",
    "desparejaban": "desparejar",
    "desparejabas": "desparejar",
    "desparejad": "desparejar",
    "desparejada": "desparejar",
    "desparejadas": "desparejar",
    "desparejado": "desparejar",
    "desparejados": "desparejar",
    "desparejamos": "desparejar",
    "desparejan": "desparejar",
    "desparejando": "desparejar",
    "desparejara": "desparejar",
    "desparejarais": "desparejar",
    "desparejaran": "desparejar",
    "desparejaras": "desparejar",
    "desparejare": "desparejar",
    "desparejareis": "desparejar",
    "desparejaremos": "desparejar",
    "desparejaren": "desparejar",
    "desparejares": "desparejar",
    "desparejaron": "desparejar",
    "desparejará": "desparejar",
    "desparejarán": "desparejar",
    "desparejarás": "desparejar",
    "desparejaré": "desparejar",
    "desparejaréis": "desparejar",
    "desparejaría": "desparejar",
    "desparejaríais": "desparejar",
    "desparejaríamos": "desparejar",
    "desparejarían": "desparejar",
    "desparejarías": "desparejar",
    "desparejas": "desparejo",
    "desparejase": "desparejar",
    "desparejaseis": "desparejar",
    "desparejasen": "desparejar",
    "desparejases": "desparejar",
    "desparejaste": "desparejar",
    "desparejasteis": "desparejar",
    "despareje": "desparejar",
    "desparejemos": "desparejar",
    "desparejen": "desparejar",
    "desparejes": "desparejar",
    "desparejo": "desparejar",
    "desparejos": "desparejo",
    "desparejábamos": "desparejar",
    "desparejáis": "desparejar",
    "desparejáramos": "desparejar",
    "desparejáremos": "desparejar",
    "desparejásemos": "desparejar",
    "desparejé": "desparejar",
    "desparejéis": "desparejar",
    "desparejó": "desparejar",
    "despareo": "desparear",
    "despareábamos": "desparear",
    "despareáis": "desparear",
    "despareáramos": "desparear",
    "despareáremos": "desparear",
    "despareásemos": "desparear",
    "despareé": "desparear",
    "despareéis": "desparear",
    "despareó": "desparear",
    "desparpaja": "desparpajar",
    "desparpajaba": "desparpajar",
    "desparpajabais": "desparpajar",
    "desparpajaban": "desparpajar",
    "desparpajabas": "desparpajar",
    "desparpajad": "desparpajar",
    "desparpajada": "desparpajar",
    "desparpajadas": "desparpajar",
    "desparpajado": "desparpajar",
    "desparpajados": "desparpajar",
    "desparpajamos": "desparpajar",
    "desparpajan": "desparpajar",
    "desparpajando": "desparpajar",
    "desparpajara": "desparpajar",
    "desparpajarais": "desparpajar",
    "desparpajaran": "desparpajar",
    "desparpajaras": "desparpajar",
    "desparpajare": "desparpajar",
    "desparpajareis": "desparpajar",
    "desparpajaremos": "desparpajar",
    "desparpajaren": "desparpajar",
    "desparpajares": "desparpajar",
    "desparpajaron": "desparpajar",
    "desparpajará": "desparpajar",
    "desparpajarán": "desparpajar",
    "desparpajarás": "desparpajar",
    "desparpajaré": "desparpajar",
    "desparpajaréis": "desparpajar",
    "desparpajaría": "desparpajar",
    "desparpajaríais": "desparpajar",
    "desparpajaríamos": "desparpajar",
    "desparpajarían": "desparpajar",
    "desparpajarías": "desparpajar",
    "desparpajas": "desparpajar",
    "desparpajase": "desparpajar",
    "desparpajaseis": "desparpajar",
    "desparpajasen": "desparpajar",
    "desparpajases": "desparpajar",
    "desparpajaste": "desparpajar",
    "desparpajasteis": "desparpajar",
    "desparpaje": "desparpajar",
    "desparpajemos": "desparpajar",
    "desparpajen": "desparpajar",
    "desparpajes": "desparpajar",
    "desparpajo": "desparpajar",
    "desparpajos": "desparpajo",
    "desparpajábamos": "desparpajar",
    "desparpajáis": "desparpajar",
    "desparpajáramos": "desparpajar",
    "desparpajáremos": "desparpajar",
    "desparpajásemos": "desparpajar",
    "desparpajé": "desparpajar",
    "desparpajéis": "desparpajar",
    "desparpajó": "desparpajar",
    "desparrama": "desparramar",
    "desparramaba": "desparramar",
    "desparramabais": "desparramar",
    "desparramaban": "desparramar",
    "desparramabas": "desparramar",
    "desparramad": "desparramar",
    "desparramada": "desparramar",
    "desparramadas": "desparramar",
    "desparramado": "desparramar",
    "desparramadora": "desparramador",
    "desparramadoras": "desparramadora",
    "desparramadores": "desparramador",
    "desparramados": "desparramar",
    "desparramamientos": "desparramamiento",
    "desparramamos": "desparramar",
    "desparraman": "desparramar",
    "desparramando": "desparramar",
    "desparramara": "desparramar",
    "desparramarais": "desparramar",
    "desparramaran": "desparramar",
    "desparramaras": "desparramar",
    "desparramare": "desparramar",
    "desparramareis": "desparramar",
    "desparramaremos": "desparramar",
    "desparramaren": "desparramar",
    "desparramares": "desparramar",
    "desparramaron": "desparramar",
    "desparramará": "desparramar",
    "desparramarán": "desparramar",
    "desparramarás": "desparramar",
    "desparramaré": "desparramar",
    "desparramaréis": "desparramar",
    "desparramaría": "desparramar",
    "desparramaríais": "desparramar",
    "desparramaríamos": "desparramar",
    "desparramarían": "desparramar",
    "desparramarías": "desparramar",
    "desparramas": "desparramar",
    "desparramase": "desparramar",
    "desparramaseis": "desparramar",
    "desparramasen": "desparramar",
    "desparramases": "desparramar",
    "desparramaste": "desparramar",
    "desparramasteis": "desparramar",
    "desparrame": "desparramar",
    "desparramemos": "desparramar",
    "desparramen": "desparramar",
    "desparrames": "desparramar",
    "desparramo": "desparramar",
    "desparramábamos": "desparramar",
    "desparramáis": "desparramar",
    "desparramáramos": "desparramar",
    "desparramáremos": "desparramar",
    "desparramásemos": "desparramar",
    "desparramé": "desparramar",
    "desparraméis": "desparramar",
    "desparramó": "desparramar",
    "despartimientos": "despartimiento",
    "desparva": "desparvar",
    "desparvaba": "desparvar",
    "desparvabais": "desparvar",
    "desparvaban": "desparvar",
    "desparvabas": "desparvar",
    "desparvad": "desparvar",
    "desparvada": "desparvar",
    "desparvadas": "desparvar",
    "desparvado": "desparvar",
    "desparvados": "desparvar",
    "desparvamos": "desparvar",
    "desparvan": "desparvar",
    "desparvando": "desparvar",
    "desparvara": "desparvar",
    "desparvarais": "desparvar",
    "desparvaran": "desparvar",
    "desparvaras": "desparvar",
    "desparvare": "desparvar",
    "desparvareis": "desparvar",
    "desparvaremos": "desparvar",
    "desparvaren": "desparvar",
    "desparvares": "desparvar",
    "desparvaron": "desparvar",
    "desparvará": "desparvar",
    "desparvarán": "desparvar",
    "desparvarás": "desparvar",
    "desparvaré": "desparvar",
    "desparvaréis": "desparvar",
    "desparvaría": "desparvar",
    "desparvaríais": "desparvar",
    "desparvaríamos": "desparvar",
    "desparvarían": "desparvar",
    "desparvarías": "desparvar",
    "desparvas": "desparvar",
    "desparvase": "desparvar",
    "desparvaseis": "desparvar",
    "desparvasen": "desparvar",
    "desparvases": "desparvar",
    "desparvaste": "desparvar",
    "desparvasteis": "desparvar",
    "desparve": "desparvar",
    "desparvemos": "desparvar",
    "desparven": "desparvar",
    "desparves": "desparvar",
    "desparvo": "desparvar",
    "desparvábamos": "desparvar",
    "desparváis": "desparvar",
    "desparváramos": "desparvar",
    "desparváremos": "desparvar",
    "desparvásemos": "desparvar",
    "desparvé": "desparvar",
    "desparvéis": "desparvar",
    "desparvó": "desparvar",
    "despasivaciones": "despasivación",
    "despatarra": "despatarrar",
    "despatarraba": "despatarrar",
    "despatarrabais": "despatarrar",
    "despatarraban": "despatarrar",
    "despatarrabas": "despatarrar",
    "despatarrad": "despatarrar",
    "despatarrada": "despatarrar",
    "despatarradas": "despatarrar",
    "despatarrado": "despatarrar",
    "despatarrados": "despatarrar",
    "despatarramos": "despatarrar",
    "despatarran": "despatarrar",
    "despatarrando": "despatarrar",
    "despatarrara": "despatarrar",
    "despatarrarais": "despatarrar",
    "despatarraran": "despatarrar",
    "despatarraras": "despatarrar",
    "despatarrare": "despatarrar",
    "despatarrareis": "despatarrar",
    "despatarraremos": "despatarrar",
    "despatarraren": "despatarrar",
    "despatarrares": "despatarrar",
    "despatarraron": "despatarrar",
    "despatarrará": "despatarrar",
    "despatarrarán": "despatarrar",
    "despatarrarás": "despatarrar",
    "despatarraré": "despatarrar",
    "despatarraréis": "despatarrar",
    "despatarraría": "despatarrar",
    "despatarraríais": "despatarrar",
    "despatarraríamos": "despatarrar",
    "despatarrarían": "despatarrar",
    "despatarrarías": "despatarrar",
    "despatarras": "despatarrar",
    "despatarrase": "despatarrar",
    "despatarraseis": "despatarrar",
    "despatarrasen": "despatarrar",
    "despatarrases": "despatarrar",
    "despatarraste": "despatarrar",
    "despatarrasteis": "despatarrar",
    "despatarre": "despatarrar",
    "despatarremos": "despatarrar",
    "despatarren": "despatarrar",
    "despatarres": "despatarrar",
    "despatarro": "despatarrar",
    "despatarrábamos": "despatarrar",
    "despatarráis": "despatarrar",
    "despatarráramos": "despatarrar",
    "despatarráremos": "despatarrar",
    "despatarrásemos": "despatarrar",
    "despatarré": "despatarrar",
    "despatarréis": "despatarrar",
    "despatarró": "despatarrar",
    "despatilla": "despatillar",
    "despatillaba": "despatillar",
    "despatillabais": "despatillar",
    "despatillaban": "despatillar",
    "despatillabas": "despatillar",
    "despatillad": "despatillar",
    "despatillada": "despatillar",
    "despatilladas": "despatillar",
    "despatillado": "despatillar",
    "despatillados": "despatillar",
    "despatillamos": "despatillar",
    "despatillan": "despatillar",
    "despatillando": "despatillar",
    "despatillara": "despatillar",
    "despatillarais": "despatillar",
    "despatillaran": "despatillar",
    "despatillaras": "despatillar",
    "despatillare": "despatillar",
    "despatillareis": "despatillar",
    "despatillaremos": "despatillar",
    "despatillaren": "despatillar",
    "despatillares": "despatillar",
    "despatillaron": "despatillar",
    "despatillará": "despatillar",
    "despatillarán": "despatillar",
    "despatillarás": "despatillar",
    "despatillaré": "despatillar",
    "despatillaréis": "despatillar",
    "despatillaría": "despatillar",
    "despatillaríais": "despatillar",
    "despatillaríamos": "despatillar",
    "despatillarían": "despatillar",
    "despatillarías": "despatillar",
    "despatillas": "despatillar",
    "despatillase": "despatillar",
    "despatillaseis": "despatillar",
    "despatillasen": "despatillar",
    "despatillases": "despatillar",
    "despatillaste": "despatillar",
    "despatillasteis": "despatillar",
    "despatille": "despatillar",
    "despatillemos": "despatillar",
    "despatillen": "despatillar",
    "despatilles": "despatillar",
    "despatillo": "despatillar",
    "despatillábamos": "despatillar",
    "despatilláis": "despatillar",
    "despatilláramos": "despatillar",
    "despatilláremos": "despatillar",
    "despatillásemos": "despatillar",
    "despatillé": "despatillar",
    "despatilléis": "despatillar",
    "despatilló": "despatillar",
    "despavesa": "despavesar",
    "despavesaba": "despavesar",
    "despavesabais": "despavesar",
    "despavesaban": "despavesar",
    "despavesabas": "despavesar",
    "despavesad": "despavesar",
    "despavesada": "despavesar",
    "despavesadas": "despavesar",
    "despavesado": "despavesar",
    "despavesados": "despavesar",
    "despavesaduras": "despavesadura",
    "despavesamos": "despavesar",
    "despavesan": "despavesar",
    "despavesando": "despavesar",
    "despavesara": "despavesar",
    "despavesarais": "despavesar",
    "despavesaran": "despavesar",
    "despavesaras": "despavesar",
    "despavesare": "despavesar",
    "despavesareis": "despavesar",
    "despavesaremos": "despavesar",
    "despavesaren": "despavesar",
    "despavesares": "despavesar",
    "despavesaron": "despavesar",
    "despavesará": "despavesar",
    "despavesarán": "despavesar",
    "despavesarás": "despavesar",
    "despavesaré": "despavesar",
    "despavesaréis": "despavesar",
    "despavesaría": "despavesar",
    "despavesaríais": "despavesar",
    "despavesaríamos": "despavesar",
    "despavesarían": "despavesar",
    "despavesarías": "despavesar",
    "despavesas": "despavesar",
    "despavesase": "despavesar",
    "despavesaseis": "despavesar",
    "despavesasen": "despavesar",
    "despavesases": "despavesar",
    "despavesaste": "despavesar",
    "despavesasteis": "despavesar",
    "despavese": "despavesar",
    "despavesemos": "despavesar",
    "despavesen": "despavesar",
    "despaveses": "despavesar",
    "despaveso": "despavesar",
    "despavesábamos": "despavesar",
    "despavesáis": "despavesar",
    "despavesáramos": "despavesar",
    "despavesáremos": "despavesar",
    "despavesásemos": "despavesar",
    "despavesé": "despavesar",
    "despaveséis": "despavesar",
    "despavesó": "despavesar",
    "despavona": "despavonar",
    "despavonaba": "despavonar",
    "despavonabais": "despavonar",
    "despavonaban": "despavonar",
    "despavonabas": "despavonar",
    "despavonad": "despavonar",
    "despavonada": "despavonar",
    "despavonadas": "despavonar",
    "despavonado": "despavonar",
    "despavonados": "despavonar",
    "despavonamos": "despavonar",
    "despavonan": "despavonar",
    "despavonando": "despavonar",
    "despavonara": "despavonar",
    "despavonarais": "despavonar",
    "despavonaran": "despavonar",
    "despavonaras": "despavonar",
    "despavonare": "despavonar",
    "despavonareis": "despavonar",
    "despavonaremos": "despavonar",
    "despavonaren": "despavonar",
    "despavonares": "despavonar",
    "despavonaron": "despavonar",
    "despavonará": "despavonar",
    "despavonarán": "despavonar",
    "despavonarás": "despavonar",
    "despavonaré": "despavonar",
    "despavonaréis": "despavonar",
    "despavonaría": "despavonar",
    "despavonaríais": "despavonar",
    "despavonaríamos": "despavonar",
    "despavonarían": "despavonar",
    "despavonarías": "despavonar",
    "despavonas": "despavonar",
    "despavonase": "despavonar",
    "despavonaseis": "despavonar",
    "despavonasen": "despavonar",
    "despavonases": "despavonar",
    "despavonaste": "despavonar",
    "despavonasteis": "despavonar",
    "despavone": "despavonar",
    "despavonemos": "despavonar",
    "despavonen": "despavonar",
    "despavones": "despavonar",
    "despavono": "despavonar",
    "despavonábamos": "despavonar",
    "despavonáis": "despavonar",
    "despavonáramos": "despavonar",
    "despavonáremos": "despavonar",
    "despavonásemos": "despavonar",
    "despavoné": "despavonar",
    "despavonéis": "despavonar",
    "despavonó": "despavonar",
    "despavora": "despavorir",
    "despavoramos": "despavorir",
    "despavoran": "despavorir",
    "despavoras": "despavorir",
    "despavore": "despavorir",
    "despavoren": "despavorir",
    "despavores": "despavorir",
    "despavorid": "despavorir",
    "despavorida": "despavorir",
    "despavoridas": "despavorir",
    "despavorido": "despavorir",
    "despavoridos": "despavorir",
    "despavoriendo": "despavorir",
    "despavoriera": "despavorir",
    "despavorierais": "despavorir",
    "despavorieran": "despavorir",
    "despavorieras": "despavorir",
    "despavoriere": "despavorir",
    "despavoriereis": "despavorir",
    "despavorieren": "despavorir",
    "despavorieres": "despavorir",
    "despavorieron": "despavorir",
    "despavoriese": "despavorir",
    "despavorieseis": "despavorir",
    "despavoriesen": "despavorir",
    "despavorieses": "despavorir",
    "despavorimos": "despavorir",
    "despavoriremos": "despavorir",
    "despavorirá": "despavorir",
    "despavorirán": "despavorir",
    "despavorirás": "despavorir",
    "despavoriré": "despavorir",
    "despavoriréis": "despavorir",
    "despavoriría": "despavorir",
    "despavoriríais": "despavorir",
    "despavoriríamos": "despavorir",
    "despavorirían": "despavorir",
    "despavorirías": "despavorir",
    "despavoriste": "despavorir",
    "despavoristeis": "despavorir",
    "despavoriéramos": "despavorir",
    "despavoriéremos": "despavorir",
    "despavoriésemos": "despavorir",
    "despavorió": "despavorir",
    "despavoro": "despavorir",
    "despavoráis": "despavorir",
    "despavorí": "despavorir",
    "despavoría": "despavorir",
    "despavoríais": "despavorir",
    "despavoríamos": "despavorir",
    "despavorían": "despavorir",
    "despavorías": "despavorir",
    "despavorís": "despavorir",
    "despañamientos": "despañamiento",
    "despea": "despear",
    "despeaba": "despear",
    "despeabais": "despear",
    "despeaban": "despear",
    "despeabas": "despear",
    "despead": "despear",
    "despeada": "despear",
    "despeadas": "despear",
    "despeado": "despear",
    "despeados": "despear",
    "despeaduras": "despeadura",
    "despeamientos": "despeamiento",
    "despeamos": "despear",
    "despean": "despear",
    "despeando": "despear",
    "despeara": "despear",
    "despearais": "despear",
    "despearan": "despear",
    "despearas": "despear",
    "despeare": "despear",
    "despeareis": "despear",
    "despearemos": "despear",
    "despearen": "despear",
    "despeares": "despear",
    "despearon": "despear",
    "despeará": "despear",
    "despearán": "despear",
    "despearás": "despear",
    "despearé": "despear",
    "despearéis": "despear",
    "despearía": "despear",
    "despearíais": "despear",
    "despearíamos": "despear",
    "despearían": "despear",
    "despearías": "despear",
    "despeas": "despear",
    "despease": "despear",
    "despeaseis": "despear",
    "despeasen": "despear",
    "despeases": "despear",
    "despeaste": "despear",
    "despeasteis": "despear",
    "despecha": "despechar",
    "despechaba": "despechar",
    "despechabais": "despechar",
    "despechaban": "despechar",
    "despechabas": "despechar",
    "despechad": "despechar",
    "despechada": "despechar",
    "despechadas": "despechar",
    "despechado": "despechar",
    "despechados": "despechar",
    "despechamos": "despechar",
    "despechan": "despechar",
    "despechando": "despechar",
    "despechara": "despechar",
    "despecharais": "despechar",
    "despecharan": "despechar",
    "despecharas": "despechar",
    "despechare": "despechar",
    "despechareis": "despechar",
    "despecharemos": "despechar",
    "despecharen": "despechar",
    "despechares": "despechar",
    "despecharon": "despechar",
    "despechará": "despechar",
    "despecharán": "despechar",
    "despecharás": "despechar",
    "despecharé": "despechar",
    "despecharéis": "despechar",
    "despecharía": "despechar",
    "despecharíais": "despechar",
    "despecharíamos": "despechar",
    "despecharían": "despechar",
    "despecharías": "despechar",
    "despechas": "despechar",
    "despechase": "despechar",
    "despechaseis": "despechar",
    "despechasen": "despechar",
    "despechases": "despechar",
    "despechaste": "despechar",
    "despechasteis": "despechar",
    "despeche": "despechar",
    "despechemos": "despechar",
    "despechen": "despechar",
    "despecheretada": "despecheretado",
    "despecheretadas": "despecheretado",
    "despecheretados": "despecheretado",
    "despeches": "despechar",
    "despecho": "despechar",
    "despechos": "despecho",
    "despechuga": "despechugar",
    "despechugaba": "despechugar",
    "despechugabais": "despechugar",
    "despechugaban": "despechugar",
    "despechugabas": "despechugar",
    "despechugad": "despechugar",
    "despechugada": "despechugar",
    "despechugadas": "despechugar",
    "despechugado": "despechugar",
    "despechugados": "despechugar",
    "despechugaduras": "despechugadura",
    "despechugamos": "despechugar",
    "despechugan": "despechugar",
    "despechugando": "despechugar",
    "despechugara": "despechugar",
    "despechugarais": "despechugar",
    "despechugaran": "despechugar",
    "despechugaras": "despechugar",
    "despechugare": "despechugar",
    "despechugareis": "despechugar",
    "despechugaremos": "despechugar",
    "despechugaren": "despechugar",
    "despechugares": "despechugar",
    "despechugaron": "despechugar",
    "despechugará": "despechugar",
    "despechugarán": "despechugar",
    "despechugarás": "despechugar",
    "despechugaré": "despechugar",
    "despechugaréis": "despechugar",
    "despechugaría": "despechugar",
    "despechugaríais": "despechugar",
    "despechugaríamos": "despechugar",
    "despechugarían": "despechugar",
    "despechugarías": "despechugar",
    "despechugas": "despechugar",
    "despechugase": "despechugar",
    "despechugaseis": "despechugar",
    "despechugasen": "despechugar",
    "despechugases": "despechugar",
    "despechugaste": "despechugar",
    "despechugasteis": "despechugar",
    "despechugo": "despechugar",
    "despechugue": "despechugar",
    "despechuguemos": "despechugar",
    "despechuguen": "despechugar",
    "despechugues": "despechugar",
    "despechugué": "despechugar",
    "despechuguéis": "despechugar",
    "despechugábamos": "despechugar",
    "despechugáis": "despechugar",
    "despechugáramos": "despechugar",
    "despechugáremos": "despechugar",
    "despechugásemos": "despechugar",
    "despechugó": "despechugar",
    "despechábamos": "despechar",
    "despecháis": "despechar",
    "despecháramos": "despechar",
    "despecháremos": "despechar",
    "despechásemos": "despechar",
    "despeché": "despechar",
    "despechéis": "despechar",
    "despechó": "despechar",
    "despectiva": "despectivo",
    "despectivas": "despectivo",
    "despectivos": "despectivo",
    "despedace": "despedazar",
    "despedacemos": "despedazar",
    "despedacen": "despedazar",
    "despedaces": "despedazar",
    "despedacé": "despedazar",
    "despedacéis": "despedazar",
    "despedaza": "despedazar",
    "despedazaba": "despedazar",
    "despedazabais": "despedazar",
    "despedazaban": "despedazar",
    "despedazabas": "despedazar",
    "despedazad": "despedazar",
    "despedazada": "despedazar",
    "despedazadas": "despedazar",
    "despedazado": "despedazar",
    "despedazadora": "despedazador",
    "despedazadoras": "despedazador",
    "despedazadores": "despedazador",
    "despedazados": "despedazar",
    "despedazamientos": "despedazamiento",
    "despedazamos": "despedazar",
    "despedazan": "despedazar",
    "despedazando": "despedazar",
    "despedazara": "despedazar",
    "despedazarais": "despedazar",
    "despedazaran": "despedazar",
    "despedazaras": "despedazar",
    "despedazare": "despedazar",
    "despedazareis": "despedazar",
    "despedazaremos": "despedazar",
    "despedazaren": "despedazar",
    "despedazares": "despedazar",
    "despedazaron": "despedazar",
    "despedazará": "despedazar",
    "despedazarán": "despedazar",
    "despedazarás": "despedazar",
    "despedazaré": "despedazar",
    "despedazaréis": "despedazar",
    "despedazaría": "despedazar",
    "despedazaríais": "despedazar",
    "despedazaríamos": "despedazar",
    "despedazarían": "despedazar",
    "despedazarías": "despedazar",
    "despedazas": "despedazar",
    "despedazase": "despedazar",
    "despedazaseis": "despedazar",
    "despedazasen": "despedazar",
    "despedazases": "despedazar",
    "despedazaste": "despedazar",
    "despedazasteis": "despedazar",
    "despedazo": "despedazar",
    "despedazábamos": "despedazar",
    "despedazáis": "despedazar",
    "despedazáramos": "despedazar",
    "despedazáremos": "despedazar",
    "despedazásemos": "despedazar",
    "despedazó": "despedazar",
    "despedid": "despedir",
    "despedida": "despedir",
    "despedidas": "despedir",
    "despedido": "despedir",
    "despedidos": "despedir",
    "despedimos": "despedir",
    "despediremos": "despedir",
    "despedirá": "despedir",
    "despedirán": "despedir",
    "despedirás": "despedir",
    "despediré": "despedir",
    "despediréis": "despedir",
    "despediría": "despedir",
    "despediríais": "despedir",
    "despediríamos": "despedir",
    "despedirían": "despedir",
    "despedirías": "despedir",
    "despediste": "despedir",
    "despedisteis": "despedir",
    "despedraba": "despedrar",
    "despedrabais": "despedrar",
    "despedraban": "despedrar",
    "despedrabas": "despedrar",
    "despedrad": "despedrar",
    "despedrada": "despedrar",
    "despedradas": "despedrar",
    "despedrado": "despedrar",
    "despedrados": "despedrar",
    "despedramos": "despedrar",
    "despedrando": "despedrar",
    "despedrara": "despedrar",
    "despedrarais": "despedrar",
    "despedraran": "despedrar",
    "despedraras": "despedrar",
    "despedrare": "despedrar",
    "despedrareis": "despedrar",
    "despedraremos": "despedrar",
    "despedraren": "despedrar",
    "despedrares": "despedrar",
    "despedraron": "despedrar",
    "despedrará": "despedrar",
    "despedrarán": "despedrar",
    "despedrarás": "despedrar",
    "despedraré": "despedrar",
    "despedraréis": "despedrar",
    "despedraría": "despedrar",
    "despedraríais": "despedrar",
    "despedraríamos": "despedrar",
    "despedrarían": "despedrar",
    "despedrarías": "despedrar",
    "despedrase": "despedrar",
    "despedraseis": "despedrar",
    "despedrasen": "despedrar",
    "despedrases": "despedrar",
    "despedraste": "despedrar",
    "despedrasteis": "despedrar",
    "despedrega": "despedregar",
    "despedregaba": "despedregar",
    "despedregabais": "despedregar",
    "despedregaban": "despedregar",
    "despedregabas": "despedregar",
    "despedregad": "despedregar",
    "despedregada": "despedregar",
    "despedregadas": "despedregar",
    "despedregado": "despedregar",
    "despedregadoras": "despedregadora",
    "despedregadores": "despedregador",
    "despedregados": "despedregar",
    "despedregamos": "despedregar",
    "despedregan": "despedregar",
    "despedregando": "despedregar",
    "despedregara": "despedregar",
    "despedregarais": "despedregar",
    "despedregaran": "despedregar",
    "despedregaras": "despedregar",
    "despedregare": "despedregar",
    "despedregareis": "despedregar",
    "despedregaremos": "despedregar",
    "despedregaren": "despedregar",
    "despedregares": "despedregar",
    "despedregaron": "despedregar",
    "despedregará": "despedregar",
    "despedregarán": "despedregar",
    "despedregarás": "despedregar",
    "despedregaré": "despedregar",
    "despedregaréis": "despedregar",
    "despedregaría": "despedregar",
    "despedregaríais": "despedregar",
    "despedregaríamos": "despedregar",
    "despedregarían": "despedregar",
    "despedregarías": "despedregar",
    "despedregas": "despedregar",
    "despedregase": "despedregar",
    "despedregaseis": "despedregar",
    "despedregasen": "despedregar",
    "despedregases": "despedregar",
    "despedregaste": "despedregar",
    "despedregasteis": "despedregar",
    "despedrego": "despedregar",
    "despedregue": "despedregar",
    "despedreguemos": "despedregar",
    "despedreguen": "despedregar",
    "despedregues": "despedregar",
    "despedregué": "despedregar",
    "despedreguéis": "despedregar",
    "despedregábamos": "despedregar",
    "despedregáis": "despedregar",
    "despedregáramos": "despedregar",
    "despedregáremos": "despedregar",
    "despedregásemos": "despedregar",
    "despedregó": "despedregar",
    "despedremos": "despedrar",
    "despedrábamos": "despedrar",
    "despedráis": "despedrar",
    "despedráramos": "despedrar",
    "despedráremos": "despedrar",
    "despedrásemos": "despedrar",
    "despedré": "despedrar",
    "despedréis": "despedrar",
    "despedró": "despedrar",
    "despedí": "despedir",
    "despedía": "despedir",
    "despedíais": "despedir",
    "despedíamos": "despedir",
    "despedían": "despedir",
    "despedías": "despedir",
    "despedís": "despedir",
    "despee": "despear",
    "despeemos": "despear",
    "despeen": "despear",
    "despees": "despear",
    "despega": "despegar",
    "despegaba": "despegar",
    "despegabais": "despegar",
    "despegaban": "despegar",
    "despegabas": "despegar",
    "despegables": "despegable",
    "despegad": "despegar",
    "despegada": "despegar",
    "despegadas": "despegar",
    "despegado": "despegar",
    "despegadora": "despegador",
    "despegadoras": "despegador",
    "despegadores": "despegador",
    "despegados": "despegar",
    "despegaduras": "despegadura",
    "despegamientos": "despegamiento",
    "despegamos": "despegar",
    "despegan": "despegar",
    "despegando": "despegar",
    "despegara": "despegar",
    "despegarais": "despegar",
    "despegaran": "despegar",
    "despegaras": "despegar",
    "despegare": "despegar",
    "despegareis": "despegar",
    "despegaremos": "despegar",
    "despegaren": "despegar",
    "despegares": "despegar",
    "despegaron": "despegar",
    "despegará": "despegar",
    "despegarán": "despegar",
    "despegarás": "despegar",
    "despegaré": "despegar",
    "despegaréis": "despegar",
    "despegaría": "despegar",
    "despegaríais": "despegar",
    "despegaríamos": "despegar",
    "despegarían": "despegar",
    "despegarías": "despegar",
    "despegas": "despegar",
    "despegase": "despegar",
    "despegaseis": "despegar",
    "despegasen": "despegar",
    "despegases": "despegar",
    "despegaste": "despegar",
    "despegasteis": "despegar",
    "despego": "despegar",
    "despegos": "despego",
    "despegue": "despegar",
    "despeguemos": "despegar",
    "despeguen": "despegar",
    "despegues": "despegue",
    "despegué": "despegar",
    "despeguéis": "despegar",
    "despegábamos": "despegar",
    "despegáis": "despegar",
    "despegáramos": "despegar",
    "despegáremos": "despegar",
    "despegásemos": "despegar",
    "despegó": "despegar",
    "despeina": "despeinar",
    "despeinaba": "despeinar",
    "despeinabais": "despeinar",
    "despeinaban": "despeinar",
    "despeinabas": "despeinar",
    "despeinad": "despeinar",
    "despeinada": "despeinar",
    "despeinadas": "despeinar",
    "despeinado": "despeinar",
    "despeinados": "despeinar",
    "despeinamos": "despeinar",
    "despeinan": "despeinar",
    "despeinando": "despeinar",
    "despeinara": "despeinar",
    "despeinarais": "despeinar",
    "despeinaran": "despeinar",
    "despeinaras": "despeinar",
    "despeinare": "despeinar",
    "despeinareis": "despeinar",
    "despeinaremos": "despeinar",
    "despeinaren": "despeinar",
    "despeinares": "despeinar",
    "despeinaron": "despeinar",
    "despeinará": "despeinar",
    "despeinarán": "despeinar",
    "despeinarás": "despeinar",
    "despeinaré": "despeinar",
    "despeinaréis": "despeinar",
    "despeinaría": "despeinar",
    "despeinaríais": "despeinar",
    "despeinaríamos": "despeinar",
    "despeinarían": "despeinar",
    "despeinarías": "despeinar",
    "despeinas": "despeinar",
    "despeinase": "despeinar",
    "despeinaseis": "despeinar",
    "despeinasen": "despeinar",
    "despeinases": "despeinar",
    "despeinaste": "despeinar",
    "despeinasteis": "despeinar",
    "despeine": "despeinar",
    "despeinemos": "despeinar",
    "despeinen": "despeinar",
    "despeines": "despeinar",
    "despeino": "despeinar",
    "despeinábamos": "despeinar",
    "despeináis": "despeinar",
    "despeináramos": "despeinar",
    "despeináremos": "despeinar",
    "despeinásemos": "despeinar",
    "despeiné": "despeinar",
    "despeinéis": "despeinar",
    "despeinó": "despeinar",
    "despeja": "despejar",
    "despejaba": "despejar",
    "despejabais": "despejar",
    "despejaban": "despejar",
    "despejabas": "despejar",
    "despejad": "despejar",
    "despejada": "despejar",
    "despejadas": "despejar",
    "despejado": "despejar",
    "despejados": "despejar",
    "despejamos": "despejar",
    "despejan": "despejar",
    "despejando": "despejar",
    "despejara": "despejar",
    "despejarais": "despejar",
    "despejaran": "despejar",
    "despejaras": "despejar",
    "despejare": "despejar",
    "despejareis": "despejar",
    "despejaremos": "despejar",
    "despejaren": "despejar",
    "despejares": "despejar",
    "despejaron": "despejar",
    "despejará": "despejar",
    "despejarán": "despejar",
    "despejarás": "despejar",
    "despejaré": "despejar",
    "despejaréis": "despejar",
    "despejaría": "despejar",
    "despejaríais": "despejar",
    "despejaríamos": "despejar",
    "despejarían": "despejar",
    "despejarías": "despejar",
    "despejas": "despejar",
    "despejase": "despejar",
    "despejaseis": "despejar",
    "despejasen": "despejar",
    "despejases": "despejar",
    "despejaste": "despejar",
    "despejasteis": "despejar",
    "despeje": "despejar",
    "despejemos": "despejar",
    "despejen": "despejar",
    "despejes": "despeje",
    "despejo": "despejar",
    "despejos": "despejo",
    "despejábamos": "despejar",
    "despejáis": "despejar",
    "despejáramos": "despejar",
    "despejáremos": "despejar",
    "despejásemos": "despejar",
    "despejé": "despejar",
    "despejéis": "despejar",
    "despejó": "despejar",
    "despelleja": "despellejar",
    "despellejaba": "despellejar",
    "despellejabais": "despellejar",
    "despellejaban": "despellejar",
    "despellejabas": "despellejar",
    "despellejad": "despellejar",
    "despellejada": "despellejar",
    "despellejadas": "despellejar",
    "despellejado": "despellejar",
    "despellejados": "despellejar",
    "despellejaduras": "despellejadura",
    "despellejamos": "despellejar",
    "despellejan": "despellejar",
    "despellejando": "despellejar",
    "despellejara": "despellejar",
    "despellejarais": "despellejar",
    "despellejaran": "despellejar",
    "despellejaras": "despellejar",
    "despellejare": "despellejar",
    "despellejareis": "despellejar",
    "despellejaremos": "despellejar",
    "despellejaren": "despellejar",
    "despellejares": "despellejar",
    "despellejaron": "despellejar",
    "despellejará": "despellejar",
    "despellejarán": "despellejar",
    "despellejarás": "despellejar",
    "despellejaré": "despellejar",
    "despellejaréis": "despellejar",
    "despellejaría": "despellejar",
    "despellejaríais": "despellejar",
    "despellejaríamos": "despellejar",
    "despellejarían": "despellejar",
    "despellejarías": "despellejar",
    "despellejas": "despellejar",
    "despellejase": "despellejar",
    "despellejaseis": "despellejar",
    "despellejasen": "despellejar",
    "despellejases": "despellejar",
    "despellejaste": "despellejar",
    "despellejasteis": "despellejar",
    "despelleje": "despellejar",
    "despellejemos": "despellejar",
    "despellejen": "despellejar",
    "despellejes": "despellejar",
    "despellejo": "despellejar",
    "despellejábamos": "despellejar",
    "despellejáis": "despellejar",
    "despellejáramos": "despellejar",
    "despellejáremos": "despellejar",
    "despellejásemos": "despellejar",
    "despellejé": "despellejar",
    "despellejéis": "despellejar",
    "despellejó": "despellejar",
    "despelota": "despelotar",
    "despelotaba": "despelotar",
    "despelotabais": "despelotar",
    "despelotaban": "despelotar",
    "despelotabas": "despelotar",
    "despelotad": "despelotar",
    "despelotada": "despelotar",
    "despelotadas": "despelotar",
    "despelotado": "despelotar",
    "despelotados": "despelotar",
    "despelotamos": "despelotar",
    "despelotan": "despelotar",
    "despelotando": "despelotar",
    "despelotara": "despelotar",
    "despelotarais": "despelotar",
    "despelotaran": "despelotar",
    "despelotaras": "despelotar",
    "despelotare": "despelotar",
    "despelotareis": "despelotar",
    "despelotaremos": "despelotar",
    "despelotaren": "despelotar",
    "despelotares": "despelotar",
    "despelotaron": "despelotar",
    "despelotará": "despelotar",
    "despelotarán": "despelotar",
    "despelotarás": "despelotar",
    "despelotaré": "despelotar",
    "despelotaréis": "despelotar",
    "despelotaría": "despelotar",
    "despelotaríais": "despelotar",
    "despelotaríamos": "despelotar",
    "despelotarían": "despelotar",
    "despelotarías": "despelotar",
    "despelotas": "despelotar",
    "despelotase": "despelotar",
    "despelotaseis": "despelotar",
    "despelotasen": "despelotar",
    "despelotases": "despelotar",
    "despelotaste": "despelotar",
    "despelotasteis": "despelotar",
    "despelote": "despelotar",
    "despelotemos": "despelotar",
    "despeloten": "despelotar",
    "despelotes": "despelotar",
    "despeloto": "despelotar",
    "despelotábamos": "despelotar",
    "despelotáis": "despelotar",
    "despelotáramos": "despelotar",
    "despelotáremos": "despelotar",
    "despelotásemos": "despelotar",
    "despeloté": "despelotar",
    "despelotéis": "despelotar",
    "despelotó": "despelotar",
    "despeluce": "despeluzar",
    "despelucemos": "despeluzar",
    "despelucen": "despeluzar",
    "despeluces": "despeluzar",
    "despelucé": "despeluzar",
    "despelucéis": "despeluzar",
    "despeluza": "despeluzar",
    "despeluzaba": "despeluzar",
    "despeluzabais": "despeluzar",
    "despeluzaban": "despeluzar",
    "despeluzabas": "despeluzar",
    "despeluzad": "despeluzar",
    "despeluzada": "despeluzar",
    "despeluzadas": "despeluzar",
    "despeluzado": "despeluzar",
    "despeluzados": "despeluzar",
    "despeluzamos": "despeluzar",
    "despeluzan": "despeluzar",
    "despeluzando": "despeluzar",
    "despeluzara": "despeluzar",
    "despeluzarais": "despeluzar",
    "despeluzaran": "despeluzar",
    "despeluzaras": "despeluzar",
    "despeluzare": "despeluzar",
    "despeluzareis": "despeluzar",
    "despeluzaremos": "despeluzar",
    "despeluzaren": "despeluzar",
    "despeluzares": "despeluzar",
    "despeluzaron": "despeluzar",
    "despeluzará": "despeluzar",
    "despeluzarán": "despeluzar",
    "despeluzarás": "despeluzar",
    "despeluzaré": "despeluzar",
    "despeluzaréis": "despeluzar",
    "despeluzaría": "despeluzar",
    "despeluzaríais": "despeluzar",
    "despeluzaríamos": "despeluzar",
    "despeluzarían": "despeluzar",
    "despeluzarías": "despeluzar",
    "despeluzas": "despeluzar",
    "despeluzase": "despeluzar",
    "despeluzaseis": "despeluzar",
    "despeluzasen": "despeluzar",
    "despeluzases": "despeluzar",
    "despeluzaste": "despeluzar",
    "despeluzasteis": "despeluzar",
    "despeluzna": "despeluznar",
    "despeluznaba": "despeluznar",
    "despeluznabais": "despeluznar",
    "despeluznaban": "despeluznar",
    "despeluznabas": "despeluznar",
    "despeluznad": "despeluznar",
    "despeluznada": "despeluznar",
    "despeluznadas": "despeluznar",
    "despeluznado": "despeluznar",
    "despeluznados": "despeluznar",
    "despeluznamos": "despeluznar",
    "despeluznan": "despeluznar",
    "despeluznando": "despeluznar",
    "despeluznantes": "despeluznante",
    "despeluznara": "despeluznar",
    "despeluznarais": "despeluznar",
    "despeluznaran": "despeluznar",
    "despeluznaras": "despeluznar",
    "despeluznare": "despeluznar",
    "despeluznareis": "despeluznar",
    "despeluznaremos": "despeluznar",
    "despeluznaren": "despeluznar",
    "despeluznares": "despeluznar",
    "despeluznaron": "despeluznar",
    "despeluznará": "despeluznar",
    "despeluznarán": "despeluznar",
    "despeluznarás": "despeluznar",
    "despeluznaré": "despeluznar",
    "despeluznaréis": "despeluznar",
    "despeluznaría": "despeluznar",
    "despeluznaríais": "despeluznar",
    "despeluznaríamos": "despeluznar",
    "despeluznarían": "despeluznar",
    "despeluznarías": "despeluznar",
    "despeluznas": "despeluznar",
    "despeluznase": "despeluznar",
    "despeluznaseis": "despeluznar",
    "despeluznasen": "despeluznar",
    "despeluznases": "despeluznar",
    "despeluznaste": "despeluznar",
    "despeluznasteis": "despeluznar",
    "despeluzne": "despeluznar",
    "despeluznemos": "despeluznar",
    "despeluznen": "despeluznar",
    "despeluznes": "despeluznar",
    "despeluzno": "despeluznar",
    "despeluznábamos": "despeluznar",
    "despeluznáis": "despeluznar",
    "despeluznáramos": "despeluznar",
    "despeluznáremos": "despeluznar",
    "despeluznásemos": "despeluznar",
    "despeluzné": "despeluznar",
    "despeluznéis": "despeluznar",
    "despeluznó": "despeluznar",
    "despeluzo": "despeluzar",
    "despeluzábamos": "despeluzar",
    "despeluzáis": "despeluzar",
    "despeluzáramos": "despeluzar",
    "despeluzáremos": "despeluzar",
    "despeluzásemos": "despeluzar",
    "despeluzó": "despeluzar",
    "despena": "despenar",
    "despenaba": "despenar",
    "despenabais": "despenar",
    "despenaban": "despenar",
    "despenabas": "despenar",
    "despenad": "despenar",
    "despenada": "despenar",
    "despenadas": "despenar",
    "despenado": "despenar",
    "despenadora": "despenador",
    "despenadoras": "despenador",
    "despenadores": "despenador",
    "despenados": "despenar",
    "despenalizaciones": "despenalización",
    "despenalizadora": "despenalizador",
    "despenalizadoras": "despenalizador",
    "despenalizadores": "despenalizador",
    "despenamos": "despenar",
    "despenan": "despenar",
    "despenando": "despenar",
    "despenara": "despenar",
    "despenarais": "despenar",
    "despenaran": "despenar",
    "despenaras": "despenar",
    "despenare": "despenar",
    "despenareis": "despenar",
    "despenaremos": "despenar",
    "despenaren": "despenar",
    "despenares": "despenar",
    "despenaron": "despenar",
    "despenará": "despenar",
    "despenarán": "despenar",
    "despenarás": "despenar",
    "despenaré": "despenar",
    "despenaréis": "despenar",
    "despenaría": "despenar",
    "despenaríais": "despenar",
    "despenaríamos": "despenar",
    "despenarían": "despenar",
    "despenarías": "despenar",
    "despenas": "despenar",
    "despenase": "despenar",
    "despenaseis": "despenar",
    "despenasen": "despenar",
    "despenases": "despenar",
    "despenaste": "despenar",
    "despenasteis": "despenar",
    "despendedora": "despendedor",
    "despendedoras": "despendedor",
    "despendedores": "despendedor",
    "despene": "despenar",
    "despenemos": "despenar",
    "despenen": "despenar",
    "despenes": "despenar",
    "despeno": "despenar",
    "despensas": "despensa",
    "despenserías": "despensería",
    "despenábamos": "despenar",
    "despenáis": "despenar",
    "despenáramos": "despenar",
    "despenáremos": "despenar",
    "despenásemos": "despenar",
    "despené": "despenar",
    "despenéis": "despenar",
    "despenó": "despenar",
    "despeo": "despear",
    "despepita": "despepitar",
    "despepitaba": "despepitar",
    "despepitabais": "despepitar",
    "despepitaban": "despepitar",
    "despepitabas": "despepitar",
    "despepitad": "despepitar",
    "despepitada": "despepitar",
    "despepitadas": "despepitar",
    "despepitado": "despepitar",
    "despepitados": "despepitar",
    "despepitamos": "despepitar",
    "despepitan": "despepitar",
    "despepitando": "despepitar",
    "despepitara": "despepitar",
    "despepitarais": "despepitar",
    "despepitaran": "despepitar",
    "despepitaras": "despepitar",
    "despepitare": "despepitar",
    "despepitareis": "despepitar",
    "despepitaremos": "despepitar",
    "despepitaren": "despepitar",
    "despepitares": "despepitar",
    "despepitaron": "despepitar",
    "despepitará": "despepitar",
    "despepitarán": "despepitar",
    "despepitarás": "despepitar",
    "despepitaré": "despepitar",
    "despepitaréis": "despepitar",
    "despepitaría": "despepitar",
    "despepitaríais": "despepitar",
    "despepitaríamos": "despepitar",
    "despepitarían": "despepitar",
    "despepitarías": "despepitar",
    "despepitas": "despepitar",
    "despepitase": "despepitar",
    "despepitaseis": "despepitar",
    "despepitasen": "despepitar",
    "despepitases": "despepitar",
    "despepitaste": "despepitar",
    "despepitasteis": "despepitar",
    "despepite": "despepitar",
    "despepitemos": "despepitar",
    "despepiten": "despepitar",
    "despepites": "despepitar",
    "despepito": "despepitar",
    "despepitábamos": "despepitar",
    "despepitáis": "despepitar",
    "despepitáramos": "despepitar",
    "despepitáremos": "despepitar",
    "despepitásemos": "despepitar",
    "despepité": "despepitar",
    "despepitéis": "despepitar",
    "despepitó": "despepitar",
    "desperdicia": "desperdiciar",
    "desperdiciaba": "desperdiciar",
    "desperdiciabais": "desperdiciar",
    "desperdiciaban": "desperdiciar",
    "desperdiciabas": "desperdiciar",
    "desperdiciad": "desperdiciar",
    "desperdiciada": "desperdiciar",
    "desperdiciadas": "desperdiciar",
    "desperdiciado": "desperdiciar",
    "desperdiciadora": "desperdiciador",
    "desperdiciadoras": "desperdiciador",
    "desperdiciadores": "desperdiciador",
    "desperdiciados": "desperdiciar",
    "desperdiciamos": "desperdiciar",
    "desperdician": "desperdiciar",
    "desperdiciando": "desperdiciar",
    "desperdiciara": "desperdiciar",
    "desperdiciarais": "desperdiciar",
    "desperdiciaran": "desperdiciar",
    "desperdiciaras": "desperdiciar",
    "desperdiciare": "desperdiciar",
    "desperdiciareis": "desperdiciar",
    "desperdiciaremos": "desperdiciar",
    "desperdiciaren": "desperdiciar",
    "desperdiciares": "desperdiciar",
    "desperdiciaron": "desperdiciar",
    "desperdiciará": "desperdiciar",
    "desperdiciarán": "desperdiciar",
    "desperdiciarás": "desperdiciar",
    "desperdiciaré": "desperdiciar",
    "desperdiciaréis": "desperdiciar",
    "desperdiciaría": "desperdiciar",
    "desperdiciaríais": "desperdiciar",
    "desperdiciaríamos": "desperdiciar",
    "desperdiciarían": "desperdiciar",
    "desperdiciarías": "desperdiciar",
    "desperdicias": "desperdiciar",
    "desperdiciase": "desperdiciar",
    "desperdiciaseis": "desperdiciar",
    "desperdiciasen": "desperdiciar",
    "desperdiciases": "desperdiciar",
    "desperdiciaste": "desperdiciar",
    "desperdiciasteis": "desperdiciar",
    "desperdicie": "desperdiciar",
    "desperdiciemos": "desperdiciar",
    "desperdicien": "desperdiciar",
    "desperdicies": "desperdiciar",
    "desperdicio": "desperdiciar",
    "desperdicios": "desperdicio",
    "desperdiciábamos": "desperdiciar",
    "desperdiciáis": "desperdiciar",
    "desperdiciáramos": "desperdiciar",
    "desperdiciáremos": "desperdiciar",
    "desperdiciásemos": "desperdiciar",
    "desperdicié": "desperdiciar",
    "desperdiciéis": "desperdiciar",
    "desperdició": "desperdiciar",
    "desperdiga": "desperdigar",
    "desperdigaba": "desperdigar",
    "desperdigabais": "desperdigar",
    "desperdigaban": "desperdigar",
    "desperdigabas": "desperdigar",
    "desperdigad": "desperdigar",
    "desperdigada": "desperdigar",
    "desperdigadas": "desperdigar",
    "desperdigado": "desperdigar",
    "desperdigados": "desperdigar",
    "desperdigamientos": "desperdigamiento",
    "desperdigamos": "desperdigar",
    "desperdigan": "desperdigar",
    "desperdigando": "desperdigar",
    "desperdigara": "desperdigar",
    "desperdigarais": "desperdigar",
    "desperdigaran": "desperdigar",
    "desperdigaras": "desperdigar",
    "desperdigare": "desperdigar",
    "desperdigareis": "desperdigar",
    "desperdigaremos": "desperdigar",
    "desperdigaren": "desperdigar",
    "desperdigares": "desperdigar",
    "desperdigaron": "desperdigar",
    "desperdigará": "desperdigar",
    "desperdigarán": "desperdigar",
    "desperdigarás": "desperdigar",
    "desperdigaré": "desperdigar",
    "desperdigaréis": "desperdigar",
    "desperdigaría": "desperdigar",
    "desperdigaríais": "desperdigar",
    "desperdigaríamos": "desperdigar",
    "desperdigarían": "desperdigar",
    "desperdigarías": "desperdigar",
    "desperdigas": "desperdigar",
    "desperdigase": "desperdigar",
    "desperdigaseis": "desperdigar",
    "desperdigasen": "desperdigar",
    "desperdigases": "desperdigar",
    "desperdigaste": "desperdigar",
    "desperdigasteis": "desperdigar",
    "desperdigo": "desperdigar",
    "desperdigue": "desperdigar",
    "desperdiguemos": "desperdigar",
    "desperdiguen": "desperdigar",
    "desperdigues": "desperdigar",
    "desperdigué": "desperdigar",
    "desperdiguéis": "desperdigar",
    "desperdigábamos": "desperdigar",
    "desperdigáis": "desperdigar",
    "desperdigáramos": "desperdigar",
    "desperdigáremos": "desperdigar",
    "desperdigásemos": "desperdigar",
    "desperdigó": "desperdigar",
    "desperece": "desperezar",
    "despereced": "desperecer",
    "desperecemos": "desperezar",
    "desperecen": "desperezar",
    "despereceremos": "desperecer",
    "desperecerá": "desperecer",
    "desperecerán": "desperecer",
    "desperecerás": "desperecer",
    "despereceré": "desperecer",
    "despereceréis": "desperecer",
    "desperecería": "desperecer",
    "despereceríais": "desperecer",
    "despereceríamos": "desperecer",
    "desperecerían": "desperecer",
    "desperecerías": "desperecer",
    "despereces": "desperezar",
    "desperecida": "desperecer",
    "desperecidas": "desperecer",
    "desperecido": "desperecer",
    "desperecidos": "desperecer",
    "despereciendo": "desperecer",
    "despereciera": "desperecer",
    "desperecierais": "desperecer",
    "desperecieran": "desperecer",
    "desperecieras": "desperecer",
    "despereciere": "desperecer",
    "despereciereis": "desperecer",
    "desperecieren": "desperecer",
    "desperecieres": "desperecer",
    "desperecieron": "desperecer",
    "despereciese": "desperecer",
    "desperecieseis": "desperecer",
    "despereciesen": "desperecer",
    "desperecieses": "desperecer",
    "desperecimos": "desperecer",
    "despereciste": "desperecer",
    "desperecisteis": "desperecer",
    "despereciéramos": "desperecer",
    "despereciéremos": "desperecer",
    "despereciésemos": "desperecer",
    "despereció": "desperecer",
    "desperecé": "desperezar",
    "desperecéis": "desperezar",
    "desperecí": "desperecer",
    "desperecía": "desperecer",
    "desperecíais": "desperecer",
    "desperecíamos": "desperecer",
    "desperecían": "desperecer",
    "desperecías": "desperecer",
    "despereza": "desperezar",
    "desperezaba": "desperezar",
    "desperezabais": "desperezar",
    "desperezaban": "desperezar",
    "desperezabas": "desperezar",
    "desperezad": "desperezar",
    "desperezada": "desperezar",
    "desperezadas": "desperezar",
    "desperezado": "desperezar",
    "desperezados": "desperezar",
    "desperezamientos": "desperezamiento",
    "desperezamos": "desperezar",
    "desperezan": "desperezar",
    "desperezando": "desperezar",
    "desperezara": "desperezar",
    "desperezarais": "desperezar",
    "desperezaran": "desperezar",
    "desperezaras": "desperezar",
    "desperezare": "desperezar",
    "desperezareis": "desperezar",
    "desperezaremos": "desperezar",
    "desperezaren": "desperezar",
    "desperezares": "desperezar",
    "desperezaron": "desperezar",
    "desperezará": "desperezar",
    "desperezarán": "desperezar",
    "desperezarás": "desperezar",
    "desperezaré": "desperezar",
    "desperezaréis": "desperezar",
    "desperezaría": "desperezar",
    "desperezaríais": "desperezar",
    "desperezaríamos": "desperezar",
    "desperezarían": "desperezar",
    "desperezarías": "desperezar",
    "desperezas": "desperezar",
    "desperezase": "desperezar",
    "desperezaseis": "desperezar",
    "desperezasen": "desperezar",
    "desperezases": "desperezar",
    "desperezaste": "desperezar",
    "desperezasteis": "desperezar",
    "desperezca": "desperecer",
    "desperezcamos": "desperecer",
    "desperezcan": "desperecer",
    "desperezcas": "desperecer",
    "desperezco": "desperecer",
    "desperezcáis": "desperecer",
    "desperezo": "desperezar",
    "desperezos": "desperezo",
    "desperezábamos": "desperezar",
    "desperezáis": "desperezar",
    "desperezáramos": "desperezar",
    "desperezáremos": "desperezar",
    "desperezásemos": "desperezar",
    "desperezó": "desperezar",
    "desperfectos": "desperfecto",
    "desperfila": "desperfilar",
    "desperfilaba": "desperfilar",
    "desperfilabais": "desperfilar",
    "desperfilaban": "desperfilar",
    "desperfilabas": "desperfilar",
    "desperfilad": "desperfilar",
    "desperfilada": "desperfilar",
    "desperfiladas": "desperfilar",
    "desperfilado": "desperfilar",
    "desperfilados": "desperfilar",
    "desperfilamos": "desperfilar",
    "desperfilan": "desperfilar",
    "desperfilando": "desperfilar",
    "desperfilara": "desperfilar",
    "desperfilarais": "desperfilar",
    "desperfilaran": "desperfilar",
    "desperfilaras": "desperfilar",
    "desperfilare": "desperfilar",
    "desperfilareis": "desperfilar",
    "desperfilaremos": "desperfilar",
    "desperfilaren": "desperfilar",
    "desperfilares": "desperfilar",
    "desperfilaron": "desperfilar",
    "desperfilará": "desperfilar",
    "desperfilarán": "desperfilar",
    "desperfilarás": "desperfilar",
    "desperfilaré": "desperfilar",
    "desperfilaréis": "desperfilar",
    "desperfilaría": "desperfilar",
    "desperfilaríais": "desperfilar",
    "desperfilaríamos": "desperfilar",
    "desperfilarían": "desperfilar",
    "desperfilarías": "desperfilar",
    "desperfilas": "desperfilar",
    "desperfilase": "desperfilar",
    "desperfilaseis": "desperfilar",
    "desperfilasen": "desperfilar",
    "desperfilases": "desperfilar",
    "desperfilaste": "desperfilar",
    "desperfilasteis": "desperfilar",
    "desperfile": "desperfilar",
    "desperfilemos": "desperfilar",
    "desperfilen": "desperfilar",
    "desperfiles": "desperfilar",
    "desperfilo": "desperfilar",
    "desperfilábamos": "desperfilar",
    "desperfiláis": "desperfilar",
    "desperfiláramos": "desperfilar",
    "desperfiláremos": "desperfilar",
    "desperfilásemos": "desperfilar",
    "desperfilé": "desperfilar",
    "desperfiléis": "desperfilar",
    "desperfiló": "desperfilar",
    "despernaba": "despernar",
    "despernabais": "despernar",
    "despernaban": "despernar",
    "despernabas": "despernar",
    "despernad": "despernar",
    "despernada": "despernar",
    "despernadas": "despernar",
    "despernado": "despernar",
    "despernados": "despernar",
    "despernamos": "despernar",
    "despernanca": "despernancar",
    "despernancaba": "despernancar",
    "despernancabais": "despernancar",
    "despernancaban": "despernancar",
    "despernancabas": "despernancar",
    "despernancad": "despernancar",
    "despernancada": "despernancar",
    "despernancadas": "despernancar",
    "despernancado": "despernancar",
    "despernancados": "despernancar",
    "despernancamos": "despernancar",
    "despernancan": "despernancar",
    "despernancando": "despernancar",
    "despernancara": "despernancar",
    "despernancarais": "despernancar",
    "despernancaran": "despernancar",
    "despernancaras": "despernancar",
    "despernancare": "despernancar",
    "despernancareis": "despernancar",
    "despernancaremos": "despernancar",
    "despernancaren": "despernancar",
    "despernancares": "despernancar",
    "despernancaron": "despernancar",
    "despernancará": "despernancar",
    "despernancarán": "despernancar",
    "despernancarás": "despernancar",
    "despernancaré": "despernancar",
    "despernancaréis": "despernancar",
    "despernancaría": "despernancar",
    "despernancaríais": "despernancar",
    "despernancaríamos": "despernancar",
    "despernancarían": "despernancar",
    "despernancarías": "despernancar",
    "despernancas": "despernancar",
    "despernancase": "despernancar",
    "despernancaseis": "despernancar",
    "despernancasen": "despernancar",
    "despernancases": "despernancar",
    "despernancaste": "despernancar",
    "despernancasteis": "despernancar",
    "despernanco": "despernancar",
    "despernancábamos": "despernancar",
    "despernancáis": "despernancar",
    "despernancáramos": "despernancar",
    "despernancáremos": "despernancar",
    "despernancásemos": "despernancar",
    "despernancó": "despernancar",
    "despernando": "despernar",
    "despernanque": "despernancar",
    "despernanquemos": "despernancar",
    "despernanquen": "despernancar",
    "despernanques": "despernancar",
    "despernanqué": "despernancar",
    "despernanquéis": "despernancar",
    "despernara": "despernar",
    "despernarais": "despernar",
    "despernaran": "despernar",
    "despernaras": "despernar",
    "despernare": "despernar",
    "despernareis": "despernar",
    "despernaremos": "despernar",
    "despernaren": "despernar",
    "despernares": "despernar",
    "despernaron": "despernar",
    "despernará": "despernar",
    "despernarán": "despernar",
    "despernarás": "despernar",
    "despernaré": "despernar",
    "despernaréis": "despernar",
    "despernaría": "despernar",
    "despernaríais": "despernar",
    "despernaríamos": "despernar",
    "despernarían": "despernar",
    "despernarías": "despernar",
    "despernase": "despernar",
    "despernaseis": "despernar",
    "despernasen": "despernar",
    "despernases": "despernar",
    "despernaste": "despernar",
    "despernasteis": "despernar",
    "despernemos": "despernar",
    "despernábamos": "despernar",
    "despernáis": "despernar",
    "despernáramos": "despernar",
    "despernáremos": "despernar",
    "despernásemos": "despernar",
    "desperné": "despernar",
    "despernéis": "despernar",
    "despernó": "despernar",
    "despersonalice": "despersonalizar",
    "despersonalicemos": "despersonalizar",
    "despersonalicen": "despersonalizar",
    "despersonalices": "despersonalizar",
    "despersonalicé": "despersonalizar",
    "despersonalicéis": "despersonalizar",
    "despersonaliza": "despersonalizar",
    "despersonalizaba": "despersonalizar",
    "despersonalizabais": "despersonalizar",
    "despersonalizaban": "despersonalizar",
    "despersonalizabas": "despersonalizar",
    "despersonalizaciones": "despersonalización",
    "despersonalizad": "despersonalizar",
    "despersonalizada": "despersonalizar",
    "despersonalizadas": "despersonalizar",
    "despersonalizado": "despersonalizar",
    "despersonalizados": "despersonalizar",
    "despersonalizamos": "despersonalizar",
    "despersonalizan": "despersonalizar",
    "despersonalizando": "despersonalizar",
    "despersonalizara": "despersonalizar",
    "despersonalizarais": "despersonalizar",
    "despersonalizaran": "despersonalizar",
    "despersonalizaras": "despersonalizar",
    "despersonalizare": "despersonalizar",
    "despersonalizareis": "despersonalizar",
    "despersonalizaremos": "despersonalizar",
    "despersonalizaren": "despersonalizar",
    "despersonalizares": "despersonalizar",
    "despersonalizaron": "despersonalizar",
    "despersonalizará": "despersonalizar",
    "despersonalizarán": "despersonalizar",
    "despersonalizarás": "despersonalizar",
    "despersonalizaré": "despersonalizar",
    "despersonalizaréis": "despersonalizar",
    "despersonalizaría": "despersonalizar",
    "despersonalizaríais": "despersonalizar",
    "despersonalizaríamos": "despersonalizar",
    "despersonalizarían": "despersonalizar",
    "despersonalizarías": "despersonalizar",
    "despersonalizas": "despersonalizar",
    "despersonalizase": "despersonalizar",
    "despersonalizaseis": "despersonalizar",
    "despersonalizasen": "despersonalizar",
    "despersonalizases": "despersonalizar",
    "despersonalizaste": "despersonalizar",
    "despersonalizasteis": "despersonalizar",
    "despersonalizo": "despersonalizar",
    "despersonalizábamos": "despersonalizar",
    "despersonalizáis": "despersonalizar",
    "despersonalizáramos": "despersonalizar",
    "despersonalizáremos": "despersonalizar",
    "despersonalizásemos": "despersonalizar",
    "despersonalizó": "despersonalizar",
    "despertaba": "despertar",
    "despertabais": "despertar",
    "despertaban": "despertar",
    "despertabas": "despertar",
    "despertad": "despertar",
    "despertada": "despertar",
    "despertadas": "despertar",
    "despertado": "despertar",
    "despertadora": "despertador",
    "despertadoras": "despertador",
    "despertadores": "despertador",
    "despertados": "despertar",
    "despertamos": "despertar",
    "despertando": "despertar",
    "despertara": "despertar",
    "despertarais": "despertar",
    "despertaran": "despertar",
    "despertaras": "despertar",
    "despertare": "despertar",
    "despertareis": "despertar",
    "despertaremos": "despertar",
    "despertaren": "despertar",
    "despertares": "despertar",
    "despertaron": "despertar",
    "despertará": "despertar",
    "despertarán": "despertar",
    "despertarás": "despertar",
    "despertaré": "despertar",
    "despertaréis": "despertar",
    "despertaría": "despertar",
    "despertaríais": "despertar",
    "despertaríamos": "despertar",
    "despertarían": "despertar",
    "despertarías": "despertar",
    "despertase": "despertar",
    "despertaseis": "despertar",
    "despertasen": "despertar",
    "despertases": "despertar",
    "despertaste": "despertar",
    "despertasteis": "despertar",
    "despertemos": "despertar",
    "despertábamos": "despertar",
    "despertáis": "despertar",
    "despertáramos": "despertar",
    "despertáremos": "despertar",
    "despertásemos": "despertar",
    "desperté": "despertar",
    "despertéis": "despertar",
    "despertó": "despertar",
    "despesares": "despesar",
    "despestaña": "despestañar",
    "despestañaba": "despestañar",
    "despestañabais": "despestañar",
    "despestañaban": "despestañar",
    "despestañabas": "despestañar",
    "despestañad": "despestañar",
    "despestañada": "despestañar",
    "despestañadas": "despestañar",
    "despestañado": "despestañar",
    "despestañados": "despestañar",
    "despestañamos": "despestañar",
    "despestañan": "despestañar",
    "despestañando": "despestañar",
    "despestañara": "despestañar",
    "despestañarais": "despestañar",
    "despestañaran": "despestañar",
    "despestañaras": "despestañar",
    "despestañare": "despestañar",
    "despestañareis": "despestañar",
    "despestañaremos": "despestañar",
    "despestañaren": "despestañar",
    "despestañares": "despestañar",
    "despestañaron": "despestañar",
    "despestañará": "despestañar",
    "despestañarán": "despestañar",
    "despestañarás": "despestañar",
    "despestañaré": "despestañar",
    "despestañaréis": "despestañar",
    "despestañaría": "despestañar",
    "despestañaríais": "despestañar",
    "despestañaríamos": "despestañar",
    "despestañarían": "despestañar",
    "despestañarías": "despestañar",
    "despestañas": "despestañar",
    "despestañase": "despestañar",
    "despestañaseis": "despestañar",
    "despestañasen": "despestañar",
    "despestañases": "despestañar",
    "despestañaste": "despestañar",
    "despestañasteis": "despestañar",
    "despestañe": "despestañar",
    "despestañemos": "despestañar",
    "despestañen": "despestañar",
    "despestañes": "despestañar",
    "despestaño": "despestañar",
    "despestañábamos": "despestañar",
    "despestañáis": "despestañar",
    "despestañáramos": "despestañar",
    "despestañáremos": "despestañar",
    "despestañásemos": "despestañar",
    "despestañé": "despestañar",
    "despestañéis": "despestañar",
    "despestañó": "despestañar",
    "despezona": "despezonar",
    "despezonaba": "despezonar",
    "despezonabais": "despezonar",
    "despezonaban": "despezonar",
    "despezonabas": "despezonar",
    "despezonad": "despezonar",
    "despezonada": "despezonar",
    "despezonadas": "despezonar",
    "despezonado": "despezonar",
    "despezonados": "despezonar",
    "despezonamos": "despezonar",
    "despezonan": "despezonar",
    "despezonando": "despezonar",
    "despezonara": "despezonar",
    "despezonarais": "despezonar",
    "despezonaran": "despezonar",
    "despezonaras": "despezonar",
    "despezonare": "despezonar",
    "despezonareis": "despezonar",
    "despezonaremos": "despezonar",
    "despezonaren": "despezonar",
    "despezonares": "despezonar",
    "despezonaron": "despezonar",
    "despezonará": "despezonar",
    "despezonarán": "despezonar",
    "despezonarás": "despezonar",
    "despezonaré": "despezonar",
    "despezonaréis": "despezonar",
    "despezonaría": "despezonar",
    "despezonaríais": "despezonar",
    "despezonaríamos": "despezonar",
    "despezonarían": "despezonar",
    "despezonarías": "despezonar",
    "despezonas": "despezonar",
    "despezonase": "despezonar",
    "despezonaseis": "despezonar",
    "despezonasen": "despezonar",
    "despezonases": "despezonar",
    "despezonaste": "despezonar",
    "despezonasteis": "despezonar",
    "despezone": "despezonar",
    "despezonemos": "despezonar",
    "despezonen": "despezonar",
    "despezones": "despezonar",
    "despezono": "despezonar",
    "despezonábamos": "despezonar",
    "despezonáis": "despezonar",
    "despezonáramos": "despezonar",
    "despezonáremos": "despezonar",
    "despezonásemos": "despezonar",
    "despezoné": "despezonar",
    "despezonéis": "despezonar",
    "despezonó": "despezonar",
    "despezos": "despezo",
    "despezuña": "despezuñar",
    "despezuñaba": "despezuñar",
    "despezuñabais": "despezuñar",
    "despezuñaban": "despezuñar",
    "despezuñabas": "despezuñar",
    "despezuñad": "despezuñar",
    "despezuñada": "despezuñar",
    "despezuñadas": "despezuñar",
    "despezuñado": "despezuñar",
    "despezuñados": "despezuñar",
    "despezuñamos": "despezuñar",
    "despezuñan": "despezuñar",
    "despezuñando": "despezuñar",
    "despezuñara": "despezuñar",
    "despezuñarais": "despezuñar",
    "despezuñaran": "despezuñar",
    "despezuñaras": "despezuñar",
    "despezuñare": "despezuñar",
    "despezuñareis": "despezuñar",
    "despezuñaremos": "despezuñar",
    "despezuñaren": "despezuñar",
    "despezuñares": "despezuñar",
    "despezuñaron": "despezuñar",
    "despezuñará": "despezuñar",
    "despezuñarán": "despezuñar",
    "despezuñarás": "despezuñar",
    "despezuñaré": "despezuñar",
    "despezuñaréis": "despezuñar",
    "despezuñaría": "despezuñar",
    "despezuñaríais": "despezuñar",
    "despezuñaríamos": "despezuñar",
    "despezuñarían": "despezuñar",
    "despezuñarías": "despezuñar",
    "despezuñas": "despezuñar",
    "despezuñase": "despezuñar",
    "despezuñaseis": "despezuñar",
    "despezuñasen": "despezuñar",
    "despezuñases": "despezuñar",
    "despezuñaste": "despezuñar",
    "despezuñasteis": "despezuñar",
    "despezuñe": "despezuñar",
    "despezuñemos": "despezuñar",
    "despezuñen": "despezuñar",
    "despezuñes": "despezuñar",
    "despezuño": "despezuñar",
    "despezuñábamos": "despezuñar",
    "despezuñáis": "despezuñar",
    "despezuñáramos": "despezuñar",
    "despezuñáremos": "despezuñar",
    "despezuñásemos": "despezuñar",
    "despezuñé": "despezuñar",
    "despezuñéis": "despezuñar",
    "despezuñó": "despezuñar",
    "despeábamos": "despear",
    "despeáis": "despear",
    "despeáramos": "despear",
    "despeáremos": "despear",
    "despeásemos": "despear",
    "despeé": "despear",
    "despeéis": "despear",
    "despeña": "despeñar",
    "despeñaba": "despeñar",
    "despeñabais": "despeñar",
    "despeñaban": "despeñar",
    "despeñabas": "despeñar",
    "despeñad": "despeñar",
    "despeñada": "despeñar",
    "despeñadas": "despeñar",
    "despeñadera": "despeñadero",
    "despeñaderas": "despeñadero",
    "despeñaderos": "despeñadero",
    "despeñadiza": "despeñadizo",
    "despeñadizas": "despeñadizo",
    "despeñadizos": "despeñadizo",
    "despeñado": "despeñar",
    "despeñados": "despeñar",
    "despeñamos": "despeñar",
    "despeñan": "despeñar",
    "despeñando": "despeñar",
    "despeñara": "despeñar",
    "despeñarais": "despeñar",
    "despeñaran": "despeñar",
    "despeñaras": "despeñar",
    "despeñare": "despeñar",
    "despeñareis": "despeñar",
    "despeñaremos": "despeñar",
    "despeñaren": "despeñar",
    "despeñares": "despeñar",
    "despeñaron": "despeñar",
    "despeñará": "despeñar",
    "despeñarán": "despeñar",
    "despeñarás": "despeñar",
    "despeñaré": "despeñar",
    "despeñaréis": "despeñar",
    "despeñaría": "despeñar",
    "despeñaríais": "despeñar",
    "despeñaríamos": "despeñar",
    "despeñarían": "despeñar",
    "despeñarías": "despeñar",
    "despeñas": "despeñar",
    "despeñase": "despeñar",
    "despeñaseis": "despeñar",
    "despeñasen": "despeñar",
    "despeñases": "despeñar",
    "despeñaste": "despeñar",
    "despeñasteis": "despeñar",
    "despeñe": "despeñar",
    "despeñemos": "despeñar",
    "despeñen": "despeñar",
    "despeñes": "despeñar",
    "despeño": "despeñar",
    "despeños": "despeño",
    "despeñábamos": "despeñar",
    "despeñáis": "despeñar",
    "despeñáramos": "despeñar",
    "despeñáremos": "despeñar",
    "despeñásemos": "despeñar",
    "despeñé": "despeñar",
    "despeñéis": "despeñar",
    "despeñó": "despeñar",
    "despeó": "despear",
    "despiadada": "despiadado",
    "despiadadas": "despiadado",
    "despiadados": "despiadado",
    "despica": "despicar",
    "despicaba": "despicar",
    "despicabais": "despicar",
    "despicaban": "despicar",
    "despicabas": "despicar",
    "despicad": "despicar",
    "despicada": "despicar",
    "despicadas": "despicar",
    "despicado": "despicar",
    "despicados": "despicar",
    "despicamos": "despicar",
    "despican": "despicar",
    "despicando": "despicar",
    "despicara": "despicar",
    "despicarais": "despicar",
    "despicaran": "despicar",
    "despicaras": "despicar",
    "despicare": "despicar",
    "despicareis": "despicar",
    "despicaremos": "despicar",
    "despicaren": "despicar",
    "despicares": "despicar",
    "despicaron": "despicar",
    "despicará": "despicar",
    "despicarán": "despicar",
    "despicarás": "despicar",
    "despicaré": "despicar",
    "despicaréis": "despicar",
    "despicaría": "despicar",
    "despicaríais": "despicar",
    "despicaríamos": "despicar",
    "despicarían": "despicar",
    "despicarías": "despicar",
    "despicas": "despicar",
    "despicase": "despicar",
    "despicaseis": "despicar",
    "despicasen": "despicar",
    "despicases": "despicar",
    "despicaste": "despicar",
    "despicasteis": "despicar",
    "despicha": "despichar",
    "despichaba": "despichar",
    "despichabais": "despichar",
    "despichaban": "despichar",
    "despichabas": "despichar",
    "despichad": "despichar",
    "despichada": "despichar",
    "despichadas": "despichar",
    "despichado": "despichar",
    "despichados": "despichar",
    "despichamos": "despichar",
    "despichan": "despichar",
    "despichando": "despichar",
    "despichara": "despichar",
    "despicharais": "despichar",
    "despicharan": "despichar",
    "despicharas": "despichar",
    "despichare": "despichar",
    "despichareis": "despichar",
    "despicharemos": "despichar",
    "despicharen": "despichar",
    "despichares": "despichar",
    "despicharon": "despichar",
    "despichará": "despichar",
    "despicharán": "despichar",
    "despicharás": "despichar",
    "despicharé": "despichar",
    "despicharéis": "despichar",
    "despicharía": "despichar",
    "despicharíais": "despichar",
    "despicharíamos": "despichar",
    "despicharían": "despichar",
    "despicharías": "despichar",
    "despichas": "despichar",
    "despichase": "despichar",
    "despichaseis": "despichar",
    "despichasen": "despichar",
    "despichases": "despichar",
    "despichaste": "despichar",
    "despichasteis": "despichar",
    "despiche": "despichar",
    "despichemos": "despichar",
    "despichen": "despichar",
    "despiches": "despichar",
    "despicho": "despichar",
    "despichábamos": "despichar",
    "despicháis": "despichar",
    "despicháramos": "despichar",
    "despicháremos": "despichar",
    "despichásemos": "despichar",
    "despiché": "despichar",
    "despichéis": "despichar",
    "despichó": "despichar",
    "despico": "despicar",
    "despicábamos": "despicar",
    "despicáis": "despicar",
    "despicáramos": "despicar",
    "despicáremos": "despicar",
    "despicásemos": "despicar",
    "despicó": "despicar",
    "despida": "despedir",
    "despidamos": "despedir",
    "despidan": "despedir",
    "despidas": "despedir",
    "despide": "despedir",
    "despiden": "despedir",
    "despides": "despedir",
    "despidiendo": "despedir",
    "despidientes": "despidiente",
    "despidiera": "despedir",
    "despidierais": "despedir",
    "despidieran": "despedir",
    "despidieras": "despedir",
    "despidiere": "despedir",
    "despidiereis": "despedir",
    "despidieren": "despedir",
    "despidieres": "despedir",
    "despidieron": "despedir",
    "despidiese": "despedir",
    "despidieseis": "despedir",
    "despidiesen": "despedir",
    "despidieses": "despedir",
    "despidiéramos": "despedir",
    "despidiéremos": "despedir",
    "despidiésemos": "despedir",
    "despidió": "despedir",
    "despido": "despedir",
    "despidos": "despido",
    "despidáis": "despedir",
    "despieces": "despiece",
    "despiedra": "despedrar",
    "despiedran": "despedrar",
    "despiedras": "despedrar",
    "despiedre": "despedrar",
    "despiedren": "despedrar",
    "despiedres": "despedrar",
    "despiedro": "despedrar",
    "despierna": "despernar",
    "despiernan": "despernar",
    "despiernas": "despernar",
    "despierne": "despernar",
    "despiernen": "despernar",
    "despiernes": "despernar",
    "despierno": "despernar",
    "despierta": "despierto",
    "despiertan": "despertar",
    "despiertas": "despierto",
    "despierte": "despertar",
    "despierten": "despertar",
    "despiertes": "despertar",
    "despierto": "despertar",
    "despiertos": "despierto",
    "despiezos": "despiezo",
    "despigmentaciones": "despigmentación",
    "despilara": "despilarar",
    "despilaraba": "despilarar",
    "despilarabais": "despilarar",
    "despilaraban": "despilarar",
    "despilarabas": "despilarar",
    "despilarad": "despilarar",
    "despilarada": "despilarar",
    "despilaradas": "despilarar",
    "despilarado": "despilarar",
    "despilarados": "despilarar",
    "despilaramientos": "despilaramiento",
    "despilaramos": "despilarar",
    "despilaran": "despilarar",
    "despilarando": "despilarar",
    "despilarara": "despilarar",
    "despilararais": "despilarar",
    "despilararan": "despilarar",
    "despilararas": "despilarar",
    "despilarare": "despilarar",
    "despilarareis": "despilarar",
    "despilararemos": "despilarar",
    "despilararen": "despilarar",
    "despilarares": "despilarar",
    "despilararon": "despilarar",
    "despilarará": "despilarar",
    "despilararán": "despilarar",
    "despilararás": "despilarar",
    "despilararé": "despilarar",
    "despilararéis": "despilarar",
    "despilararía": "despilarar",
    "despilararíais": "despilarar",
    "despilararíamos": "despilarar",
    "despilararían": "despilarar",
    "despilararías": "despilarar",
    "despilaras": "despilarar",
    "despilarase": "despilarar",
    "despilaraseis": "despilarar",
    "despilarasen": "despilarar",
    "despilarases": "despilarar",
    "despilaraste": "despilarar",
    "despilarasteis": "despilarar",
    "despilare": "despilarar",
    "despilaremos": "despilarar",
    "despilaren": "despilarar",
    "despilares": "despilarar",
    "despilaro": "despilarar",
    "despilarábamos": "despilarar",
    "despilaráis": "despilarar",
    "despilaráramos": "despilarar",
    "despilaráremos": "despilarar",
    "despilarásemos": "despilarar",
    "despilaré": "despilarar",
    "despilaréis": "despilarar",
    "despilaró": "despilarar",
    "despilfarra": "despilfarrar",
    "despilfarraba": "despilfarrar",
    "despilfarrabais": "despilfarrar",
    "despilfarraban": "despilfarrar",
    "despilfarrabas": "despilfarrar",
    "despilfarrad": "despilfarrar",
    "despilfarrada": "despilfarrar",
    "despilfarradas": "despilfarrar",
    "despilfarrado": "despilfarrar",
    "despilfarradora": "despilfarrador",
    "despilfarradoras": "despilfarrador",
    "despilfarradores": "despilfarrador",
    "despilfarrados": "despilfarrar",
    "despilfarramos": "despilfarrar",
    "despilfarran": "despilfarrar",
    "despilfarrando": "despilfarrar",
    "despilfarrara": "despilfarrar",
    "despilfarrarais": "despilfarrar",
    "despilfarraran": "despilfarrar",
    "despilfarraras": "despilfarrar",
    "despilfarrare": "despilfarrar",
    "despilfarrareis": "despilfarrar",
    "despilfarraremos": "despilfarrar",
    "despilfarraren": "despilfarrar",
    "despilfarrares": "despilfarrar",
    "despilfarraron": "despilfarrar",
    "despilfarrará": "despilfarrar",
    "despilfarrarán": "despilfarrar",
    "despilfarrarás": "despilfarrar",
    "despilfarraré": "despilfarrar",
    "despilfarraréis": "despilfarrar",
    "despilfarraría": "despilfarrar",
    "despilfarraríais": "despilfarrar",
    "despilfarraríamos": "despilfarrar",
    "despilfarrarían": "despilfarrar",
    "despilfarrarías": "despilfarrar",
    "despilfarras": "despilfarrar",
    "despilfarrase": "despilfarrar",
    "despilfarraseis": "despilfarrar",
    "despilfarrasen": "despilfarrar",
    "despilfarrases": "despilfarrar",
    "despilfarraste": "despilfarrar",
    "despilfarrasteis": "despilfarrar",
    "despilfarre": "despilfarrar",
    "despilfarremos": "despilfarrar",
    "despilfarren": "despilfarrar",
    "despilfarres": "despilfarrar",
    "despilfarro": "despilfarrar",
    "despilfarros": "despilfarro",
    "despilfarrábamos": "despilfarrar",
    "despilfarráis": "despilfarrar",
    "despilfarráramos": "despilfarrar",
    "despilfarráremos": "despilfarrar",
    "despilfarrásemos": "despilfarrar",
    "despilfarré": "despilfarrar",
    "despilfarréis": "despilfarrar",
    "despilfarró": "despilfarrar",
    "despimpolla": "despimpollar",
    "despimpollaba": "despimpollar",
    "despimpollabais": "despimpollar",
    "despimpollaban": "despimpollar",
    "despimpollabas": "despimpollar",
    "despimpollad": "despimpollar",
    "despimpollada": "despimpollar",
    "despimpolladas": "despimpollar",
    "despimpollado": "despimpollar",
    "despimpollados": "despimpollar",
    "despimpollamos": "despimpollar",
    "despimpollan": "despimpollar",
    "despimpollando": "despimpollar",
    "despimpollara": "despimpollar",
    "despimpollarais": "despimpollar",
    "despimpollaran": "despimpollar",
    "despimpollaras": "despimpollar",
    "despimpollare": "despimpollar",
    "despimpollareis": "despimpollar",
    "despimpollaremos": "despimpollar",
    "despimpollaren": "despimpollar",
    "despimpollares": "despimpollar",
    "despimpollaron": "despimpollar",
    "despimpollará": "despimpollar",
    "despimpollarán": "despimpollar",
    "despimpollarás": "despimpollar",
    "despimpollaré": "despimpollar",
    "despimpollaréis": "despimpollar",
    "despimpollaría": "despimpollar",
    "despimpollaríais": "despimpollar",
    "despimpollaríamos": "despimpollar",
    "despimpollarían": "despimpollar",
    "despimpollarías": "despimpollar",
    "despimpollas": "despimpollar",
    "despimpollase": "despimpollar",
    "despimpollaseis": "despimpollar",
    "despimpollasen": "despimpollar",
    "despimpollases": "despimpollar",
    "despimpollaste": "despimpollar",
    "despimpollasteis": "despimpollar",
    "despimpolle": "despimpollar",
    "despimpollemos": "despimpollar",
    "despimpollen": "despimpollar",
    "despimpolles": "despimpollar",
    "despimpollo": "despimpollar",
    "despimpollábamos": "despimpollar",
    "despimpolláis": "despimpollar",
    "despimpolláramos": "despimpollar",
    "despimpolláremos": "despimpollar",
    "despimpollásemos": "despimpollar",
    "despimpollé": "despimpollar",
    "despimpolléis": "despimpollar",
    "despimpolló": "despimpollar",
    "despince": "despinzar",
    "despincemos": "despinzar",
    "despincen": "despinzar",
    "despinces": "despinzar",
    "despincé": "despinzar",
    "despincéis": "despinzar",
    "despinocha": "despinochar",
    "despinochaba": "despinochar",
    "despinochabais": "despinochar",
    "despinochaban": "despinochar",
    "despinochabas": "despinochar",
    "despinochad": "despinochar",
    "despinochada": "despinochar",
    "despinochadas": "despinochar",
    "despinochado": "despinochar",
    "despinochados": "despinochar",
    "despinochamos": "despinochar",
    "despinochan": "despinochar",
    "despinochando": "despinochar",
    "despinochara": "despinochar",
    "despinocharais": "despinochar",
    "despinocharan": "despinochar",
    "despinocharas": "despinochar",
    "despinochare": "despinochar",
    "despinochareis": "despinochar",
    "despinocharemos": "despinochar",
    "despinocharen": "despinochar",
    "despinochares": "despinochar",
    "despinocharon": "despinochar",
    "despinochará": "despinochar",
    "despinocharán": "despinochar",
    "despinocharás": "despinochar",
    "despinocharé": "despinochar",
    "despinocharéis": "despinochar",
    "despinocharía": "despinochar",
    "despinocharíais": "despinochar",
    "despinocharíamos": "despinochar",
    "despinocharían": "despinochar",
    "despinocharías": "despinochar",
    "despinochas": "despinochar",
    "despinochase": "despinochar",
    "despinochaseis": "despinochar",
    "despinochasen": "despinochar",
    "despinochases": "despinochar",
    "despinochaste": "despinochar",
    "despinochasteis": "despinochar",
    "despinoche": "despinochar",
    "despinochemos": "despinochar",
    "despinochen": "despinochar",
    "despinoches": "despinochar",
    "despinocho": "despinochar",
    "despinochábamos": "despinochar",
    "despinocháis": "despinochar",
    "despinocháramos": "despinochar",
    "despinocháremos": "despinochar",
    "despinochásemos": "despinochar",
    "despinoché": "despinochar",
    "despinochéis": "despinochar",
    "despinochó": "despinochar",
    "despinta": "despintar",
    "despintaba": "despintar",
    "despintabais": "despintar",
    "despintaban": "despintar",
    "despintabas": "despintar",
    "despintad": "despintar",
    "despintada": "despintar",
    "despintadas": "despintar",
    "despintado": "despintar",
    "despintados": "despintar",
    "despintamos": "despintar",
    "despintan": "despintar",
    "despintando": "despintar",
    "despintara": "despintar",
    "despintarais": "despintar",
    "despintaran": "despintar",
    "despintaras": "despintar",
    "despintare": "despintar",
    "despintareis": "despintar",
    "despintaremos": "despintar",
    "despintaren": "despintar",
    "despintares": "despintar",
    "despintaron": "despintar",
    "despintará": "despintar",
    "despintarán": "despintar",
    "despintarás": "despintar",
    "despintaré": "despintar",
    "despintaréis": "despintar",
    "despintaría": "despintar",
    "despintaríais": "despintar",
    "despintaríamos": "despintar",
    "despintarían": "despintar",
    "despintarías": "despintar",
    "despintas": "despintar",
    "despintase": "despintar",
    "despintaseis": "despintar",
    "despintasen": "despintar",
    "despintases": "despintar",
    "despintaste": "despintar",
    "despintasteis": "despintar",
    "despinte": "despintar",
    "despintemos": "despintar",
    "despinten": "despintar",
    "despintes": "despinte",
    "despinto": "despintar",
    "despintábamos": "despintar",
    "despintáis": "despintar",
    "despintáramos": "despintar",
    "despintáremos": "despintar",
    "despintásemos": "despintar",
    "despinté": "despintar",
    "despintéis": "despintar",
    "despintó": "despintar",
    "despinza": "despinzar",
    "despinzaba": "despinzar",
    "despinzabais": "despinzar",
    "despinzaban": "despinzar",
    "despinzabas": "despinzar",
    "despinzad": "despinzar",
    "despinzada": "despinzar",
    "despinzadas": "despinzar",
    "despinzaderas": "despinzadera",
    "despinzado": "despinzar",
    "despinzadora": "despinzador",
    "despinzadoras": "despinzador",
    "despinzadores": "despinzador",
    "despinzados": "despinzar",
    "despinzamientos": "despinzamiento",
    "despinzamos": "despinzar",
    "despinzan": "despinzar",
    "despinzando": "despinzar",
    "despinzara": "despinzar",
    "despinzarais": "despinzar",
    "despinzaran": "despinzar",
    "despinzaras": "despinzar",
    "despinzare": "despinzar",
    "despinzareis": "despinzar",
    "despinzaremos": "despinzar",
    "despinzaren": "despinzar",
    "despinzares": "despinzar",
    "despinzaron": "despinzar",
    "despinzará": "despinzar",
    "despinzarán": "despinzar",
    "despinzarás": "despinzar",
    "despinzaré": "despinzar",
    "despinzaréis": "despinzar",
    "despinzaría": "despinzar",
    "despinzaríais": "despinzar",
    "despinzaríamos": "despinzar",
    "despinzarían": "despinzar",
    "despinzarías": "despinzar",
    "despinzas": "despinzar",
    "despinzase": "despinzar",
    "despinzaseis": "despinzar",
    "despinzasen": "despinzar",
    "despinzases": "despinzar",
    "despinzaste": "despinzar",
    "despinzasteis": "despinzar",
    "despinzo": "despinzar",
    "despinzábamos": "despinzar",
    "despinzáis": "despinzar",
    "despinzáramos": "despinzar",
    "despinzáremos": "despinzar",
    "despinzásemos": "despinzar",
    "despinzó": "despinzar",
    "despioja": "despiojar",
    "despiojaba": "despiojar",
    "despiojabais": "despiojar",
    "despiojaban": "despiojar",
    "despiojabas": "despiojar",
    "despiojad": "despiojar",
    "despiojada": "despiojar",
    "despiojadas": "despiojar",
    "despiojado": "despiojar",
    "despiojadores": "despiojador",
    "despiojados": "despiojar",
    "despiojamos": "despiojar",
    "despiojan": "despiojar",
    "despiojando": "despiojar",
    "despiojara": "despiojar",
    "despiojarais": "despiojar",
    "despiojaran": "despiojar",
    "despiojaras": "despiojar",
    "despiojare": "despiojar",
    "despiojareis": "despiojar",
    "despiojaremos": "despiojar",
    "despiojaren": "despiojar",
    "despiojares": "despiojar",
    "despiojaron": "despiojar",
    "despiojará": "despiojar",
    "despiojarán": "despiojar",
    "despiojarás": "despiojar",
    "despiojaré": "despiojar",
    "despiojaréis": "despiojar",
    "despiojaría": "despiojar",
    "despiojaríais": "despiojar",
    "despiojaríamos": "despiojar",
    "despiojarían": "despiojar",
    "despiojarías": "despiojar",
    "despiojas": "despiojar",
    "despiojase": "despiojar",
    "despiojaseis": "despiojar",
    "despiojasen": "despiojar",
    "despiojases": "despiojar",
    "despiojaste": "despiojar",
    "despiojasteis": "despiojar",
    "despioje": "despiojar",
    "despiojemos": "despiojar",
    "despiojen": "despiojar",
    "despiojes": "despioje",
    "despiojo": "despiojar",
    "despiojábamos": "despiojar",
    "despiojáis": "despiojar",
    "despiojáramos": "despiojar",
    "despiojáremos": "despiojar",
    "despiojásemos": "despiojar",
    "despiojé": "despiojar",
    "despiojéis": "despiojar",
    "despiojó": "despiojar",
    "despique": "despicar",
    "despiquemos": "despicar",
    "despiquen": "despicar",
    "despiques": "despique",
    "despiqué": "despicar",
    "despiquéis": "despicar",
    "despista": "despistar",
    "despistaba": "despistar",
    "despistabais": "despistar",
    "despistaban": "despistar",
    "despistabas": "despistar",
    "despistad": "despistar",
    "despistada": "despistar",
    "despistadas": "despistar",
    "despistado": "despistar",
    "despistados": "despistar",
    "despistamos": "despistar",
    "despistan": "despistar",
    "despistando": "despistar",
    "despistara": "despistar",
    "despistarais": "despistar",
    "despistaran": "despistar",
    "despistaras": "despistar",
    "despistare": "despistar",
    "despistareis": "despistar",
    "despistaremos": "despistar",
    "despistaren": "despistar",
    "despistares": "despistar",
    "despistaron": "despistar",
    "despistará": "despistar",
    "despistarán": "despistar",
    "despistarás": "despistar",
    "despistaré": "despistar",
    "despistaréis": "despistar",
    "despistaría": "despistar",
    "despistaríais": "despistar",
    "despistaríamos": "despistar",
    "despistarían": "despistar",
    "despistarías": "despistar",
    "despistas": "despistar",
    "despistase": "despistar",
    "despistaseis": "despistar",
    "despistasen": "despistar",
    "despistases": "despistar",
    "despistaste": "despistar",
    "despistasteis": "despistar",
    "despiste": "despistar",
    "despistemos": "despistar",
    "despisten": "despistar",
    "despistes": "despiste",
    "despisto": "despistar",
    "despistábamos": "despistar",
    "despistáis": "despistar",
    "despistáramos": "despistar",
    "despistáremos": "despistar",
    "despistásemos": "despistar",
    "despisté": "despistar",
    "despistéis": "despistar",
    "despistó": "despistar",
    "desplace": "desplazar",
    "desplaced": "desplacer",
    "desplacemos": "desplazar",
    "desplacen": "desplazar",
    "desplaceremos": "desplacer",
    "desplaceres": "desplacer",
    "desplacerá": "desplacer",
    "desplacerán": "desplacer",
    "desplacerás": "desplacer",
    "desplaceré": "desplacer",
    "desplaceréis": "desplacer",
    "desplacería": "desplacer",
    "desplaceríais": "desplacer",
    "desplaceríamos": "desplacer",
    "desplacerían": "desplacer",
    "desplacerías": "desplacer",
    "desplaces": "desplazar",
    "desplacida": "desplacer",
    "desplacidas": "desplacer",
    "desplacido": "desplacer",
    "desplacidos": "desplacer",
    "desplaciendo": "desplacer",
    "desplaciera": "desplacer",
    "desplacierais": "desplacer",
    "desplacieran": "desplacer",
    "desplacieras": "desplacer",
    "desplaciere": "desplacer",
    "desplaciereis": "desplacer",
    "desplacieren": "desplacer",
    "desplacieres": "desplacer",
    "desplacieron": "desplacer",
    "desplaciese": "desplacer",
    "desplacieseis": "desplacer",
    "desplaciesen": "desplacer",
    "desplacieses": "desplacer",
    "desplacimos": "desplacer",
    "desplaciste": "desplacer",
    "desplacisteis": "desplacer",
    "desplaciéramos": "desplacer",
    "desplaciéremos": "desplacer",
    "desplaciésemos": "desplacer",
    "desplació": "desplacer",
    "desplacé": "desplazar",
    "desplacéis": "desplazar",
    "desplací": "desplacer",
    "desplacía": "desplacer",
    "desplacíais": "desplacer",
    "desplacíamos": "desplacer",
    "desplacían": "desplacer",
    "desplacías": "desplacer",
    "desplanta": "desplantar",
    "desplantaba": "desplantar",
    "desplantabais": "desplantar",
    "desplantaban": "desplantar",
    "desplantabas": "desplantar",
    "desplantaciones": "desplantación",
    "desplantad": "desplantar",
    "desplantada": "desplantar",
    "desplantadas": "desplantar",
    "desplantado": "desplantar",
    "desplantadora": "desplantador",
    "desplantadoras": "desplantador",
    "desplantadores": "desplantador",
    "desplantados": "desplantar",
    "desplantamos": "desplantar",
    "desplantan": "desplantar",
    "desplantando": "desplantar",
    "desplantara": "desplantar",
    "desplantarais": "desplantar",
    "desplantaran": "desplantar",
    "desplantaras": "desplantar",
    "desplantare": "desplantar",
    "desplantareis": "desplantar",
    "desplantaremos": "desplantar",
    "desplantaren": "desplantar",
    "desplantares": "desplantar",
    "desplantaron": "desplantar",
    "desplantará": "desplantar",
    "desplantarán": "desplantar",
    "desplantarás": "desplantar",
    "desplantaré": "desplantar",
    "desplantaréis": "desplantar",
    "desplantaría": "desplantar",
    "desplantaríais": "desplantar",
    "desplantaríamos": "desplantar",
    "desplantarían": "desplantar",
    "desplantarías": "desplantar",
    "desplantas": "desplantar",
    "desplantase": "desplantar",
    "desplantaseis": "desplantar",
    "desplantasen": "desplantar",
    "desplantases": "desplantar",
    "desplantaste": "desplantar",
    "desplantasteis": "desplantar",
    "desplante": "desplantar",
    "desplantemos": "desplantar",
    "desplanten": "desplantar",
    "desplantes": "desplante",
    "desplanto": "desplantar",
    "desplantábamos": "desplantar",
    "desplantáis": "desplantar",
    "desplantáramos": "desplantar",
    "desplantáremos": "desplantar",
    "desplantásemos": "desplantar",
    "desplanté": "desplantar",
    "desplantéis": "desplantar",
    "desplantó": "desplantar",
    "desplata": "desplatar",
    "desplataba": "desplatar",
    "desplatabais": "desplatar",
    "desplataban": "desplatar",
    "desplatabas": "desplatar",
    "desplataciones": "desplatación",
    "desplatad": "desplatar",
    "desplatada": "desplatar",
    "desplatadas": "desplatar",
    "desplatado": "desplatar",
    "desplatados": "desplatar",
    "desplatamos": "desplatar",
    "desplatan": "desplatar",
    "desplatando": "desplatar",
    "desplatara": "desplatar",
    "desplatarais": "desplatar",
    "desplataran": "desplatar",
    "desplataras": "desplatar",
    "desplatare": "desplatar",
    "desplatareis": "desplatar",
    "desplataremos": "desplatar",
    "desplataren": "desplatar",
    "desplatares": "desplatar",
    "desplataron": "desplatar",
    "desplatará": "desplatar",
    "desplatarán": "desplatar",
    "desplatarás": "desplatar",
    "desplataré": "desplatar",
    "desplataréis": "desplatar",
    "desplataría": "desplatar",
    "desplataríais": "desplatar",
    "desplataríamos": "desplatar",
    "desplatarían": "desplatar",
    "desplatarías": "desplatar",
    "desplatas": "desplatar",
    "desplatase": "desplatar",
    "desplataseis": "desplatar",
    "desplatasen": "desplatar",
    "desplatases": "desplatar",
    "desplataste": "desplatar",
    "desplatasteis": "desplatar",
    "desplate": "desplatar",
    "desplatemos": "desplatar",
    "desplaten": "desplatar",
    "desplates": "desplate",
    "desplato": "desplatar",
    "desplatábamos": "desplatar",
    "desplatáis": "desplatar",
    "desplatáramos": "desplatar",
    "desplatáremos": "desplatar",
    "desplatásemos": "desplatar",
    "desplaté": "desplatar",
    "desplatéis": "desplatar",
    "desplató": "desplatar",
    "desplaza": "desplazar",
    "desplazaba": "desplazar",
    "desplazabais": "desplazar",
    "desplazaban": "desplazar",
    "desplazabas": "desplazar",
    "desplazables": "desplazable",
    "desplazad": "desplazar",
    "desplazada": "desplazar",
    "desplazadas": "desplazar",
    "desplazado": "desplazar",
    "desplazadores": "desplazador",
    "desplazados": "desplazar",
    "desplazamientos": "desplazamiento",
    "desplazamos": "desplazar",
    "desplazan": "desplazar",
    "desplazando": "desplazar",
    "desplazara": "desplazar",
    "desplazarais": "desplazar",
    "desplazaran": "desplazar",
    "desplazaras": "desplazar",
    "desplazare": "desplazar",
    "desplazareis": "desplazar",
    "desplazaremos": "desplazar",
    "desplazaren": "desplazar",
    "desplazares": "desplazar",
    "desplazaron": "desplazar",
    "desplazará": "desplazar",
    "desplazarán": "desplazar",
    "desplazarás": "desplazar",
    "desplazaré": "desplazar",
    "desplazaréis": "desplazar",
    "desplazaría": "desplazar",
    "desplazaríais": "desplazar",
    "desplazaríamos": "desplazar",
    "desplazarían": "desplazar",
    "desplazarías": "desplazar",
    "desplazas": "desplazar",
    "desplazase": "desplazar",
    "desplazaseis": "desplazar",
    "desplazasen": "desplazar",
    "desplazases": "desplazar",
    "desplazaste": "desplazar",
    "desplazasteis": "desplazar",
    "desplazca": "desplacer",
    "desplazcamos": "desplacer",
    "desplazcan": "desplacer",
    "desplazcas": "desplacer",
    "desplazco": "desplacer",
    "desplazcáis": "desplacer",
    "desplazo": "desplazar",
    "desplazábamos": "desplazar",
    "desplazáis": "desplazar",
    "desplazáramos": "desplazar",
    "desplazáremos": "desplazar",
    "desplazásemos": "desplazar",
    "desplazó": "desplazar",
    "desplegaba": "desplegar",
    "desplegabais": "desplegar",
    "desplegaban": "desplegar",
    "desplegabas": "desplegar",
    "desplegables": "desplegable",
    "desplegad": "desplegar",
    "desplegada": "desplegar",
    "desplegadas": "desplegar",
    "desplegado": "desplegar",
    "desplegados": "desplegar",
    "desplegaduras": "desplegadura",
    "desplegamos": "desplegar",
    "desplegando": "desplegar",
    "desplegara": "desplegar",
    "desplegarais": "desplegar",
    "desplegaran": "desplegar",
    "desplegaras": "desplegar",
    "desplegare": "desplegar",
    "desplegareis": "desplegar",
    "desplegaremos": "desplegar",
    "desplegaren": "desplegar",
    "desplegares": "desplegar",
    "desplegaron": "desplegar",
    "desplegará": "desplegar",
    "desplegarán": "desplegar",
    "desplegarás": "desplegar",
    "desplegaré": "desplegar",
    "desplegaréis": "desplegar",
    "desplegaría": "desplegar",
    "desplegaríais": "desplegar",
    "desplegaríamos": "desplegar",
    "desplegarían": "desplegar",
    "desplegarías": "desplegar",
    "desplegase": "desplegar",
    "desplegaseis": "desplegar",
    "desplegasen": "desplegar",
    "desplegases": "desplegar",
    "desplegaste": "desplegar",
    "desplegasteis": "desplegar",
    "desplegos": "desplego",
    "despleguemos": "desplegar",
    "despleguetea": "despleguetear",
    "desplegueteaba": "despleguetear",
    "desplegueteabais": "despleguetear",
    "desplegueteaban": "despleguetear",
    "desplegueteabas": "despleguetear",
    "despleguetead": "despleguetear",
    "desplegueteada": "despleguetear",
    "desplegueteadas": "despleguetear",
    "desplegueteado": "despleguetear",
    "desplegueteados": "despleguetear",
    "desplegueteamos": "despleguetear",
    "despleguetean": "despleguetear",
    "desplegueteando": "despleguetear",
    "desplegueteara": "despleguetear",
    "despleguetearais": "despleguetear",
    "despleguetearan": "despleguetear",
    "despleguetearas": "despleguetear",
    "desplegueteare": "despleguetear",
    "desplegueteareis": "despleguetear",
    "despleguetearemos": "despleguetear",
    "despleguetearen": "despleguetear",
    "desplegueteares": "despleguetear",
    "despleguetearon": "despleguetear",
    "desplegueteará": "despleguetear",
    "despleguetearán": "despleguetear",
    "despleguetearás": "despleguetear",
    "despleguetearé": "despleguetear",
    "despleguetearéis": "despleguetear",
    "despleguetearía": "despleguetear",
    "despleguetearíais": "despleguetear",
    "despleguetearíamos": "despleguetear",
    "despleguetearían": "despleguetear",
    "despleguetearías": "despleguetear",
    "desplegueteas": "despleguetear",
    "despleguetease": "despleguetear",
    "desplegueteaseis": "despleguetear",
    "desplegueteasen": "despleguetear",
    "desplegueteases": "despleguetear",
    "desplegueteaste": "despleguetear",
    "desplegueteasteis": "despleguetear",
    "despleguetee": "despleguetear",
    "desplegueteemos": "despleguetear",
    "desplegueteen": "despleguetear",
    "despleguetees": "despleguetear",
    "desplegueteo": "despleguetear",
    "desplegueteábamos": "despleguetear",
    "desplegueteáis": "despleguetear",
    "desplegueteáramos": "despleguetear",
    "desplegueteáremos": "despleguetear",
    "desplegueteásemos": "despleguetear",
    "desplegueteé": "despleguetear",
    "desplegueteéis": "despleguetear",
    "desplegueteó": "despleguetear",
    "desplegué": "desplegar",
    "despleguéis": "desplegar",
    "desplegábamos": "desplegar",
    "desplegáis": "desplegar",
    "desplegáramos": "desplegar",
    "desplegáremos": "desplegar",
    "desplegásemos": "desplegar",
    "desplegó": "desplegar",
    "despliega": "desplegar",
    "despliegan": "desplegar",
    "despliegas": "desplegar",
    "despliego": "desplegar",
    "despliegue": "desplegar",
    "desplieguen": "desplegar",
    "despliegues": "despliegue",
    "desploma": "desplomar",
    "desplomaba": "desplomar",
    "desplomabais": "desplomar",
    "desplomaban": "desplomar",
    "desplomabas": "desplomar",
    "desplomad": "desplomar",
    "desplomada": "desplomar",
    "desplomadas": "desplomar",
    "desplomado": "desplomar",
    "desplomados": "desplomar",
    "desplomamos": "desplomar",
    "desploman": "desplomar",
    "desplomando": "desplomar",
    "desplomara": "desplomar",
    "desplomarais": "desplomar",
    "desplomaran": "desplomar",
    "desplomaras": "desplomar",
    "desplomare": "desplomar",
    "desplomareis": "desplomar",
    "desplomaremos": "desplomar",
    "desplomaren": "desplomar",
    "desplomares": "desplomar",
    "desplomaron": "desplomar",
    "desplomará": "desplomar",
    "desplomarán": "desplomar",
    "desplomarás": "desplomar",
    "desplomaré": "desplomar",
    "desplomaréis": "desplomar",
    "desplomaría": "desplomar",
    "desplomaríais": "desplomar",
    "desplomaríamos": "desplomar",
    "desplomarían": "desplomar",
    "desplomarías": "desplomar",
    "desplomas": "desplomar",
    "desplomase": "desplomar",
    "desplomaseis": "desplomar",
    "desplomasen": "desplomar",
    "desplomases": "desplomar",
    "desplomaste": "desplomar",
    "desplomasteis": "desplomar",
    "desplome": "desplomar",
    "desplomemos": "desplomar",
    "desplomen": "desplomar",
    "desplomes": "desplome",
    "desplomo": "desplomar",
    "desplomos": "desplomo",
    "desplomábamos": "desplomar",
    "desplomáis": "desplomar",
    "desplomáramos": "desplomar",
    "desplomáremos": "desplomar",
    "desplomásemos": "desplomar",
    "desplomé": "desplomar",
    "desploméis": "desplomar",
    "desplomó": "desplomar",
    "despluma": "desplumar",
    "desplumaba": "desplumar",
    "desplumabais": "desplumar",
    "desplumaban": "desplumar",
    "desplumabas": "desplumar",
    "desplumad": "desplumar",
    "desplumada": "desplumar",
    "desplumadas": "desplumar",
    "desplumado": "desplumar",
    "desplumados": "desplumar",
    "desplumaduras": "desplumadura",
    "desplumamos": "desplumar",
    "despluman": "desplumar",
    "desplumando": "desplumar",
    "desplumara": "desplumar",
    "desplumarais": "desplumar",
    "desplumaran": "desplumar",
    "desplumaras": "desplumar",
    "desplumare": "desplumar",
    "desplumareis": "desplumar",
    "desplumaremos": "desplumar",
    "desplumaren": "desplumar",
    "desplumares": "desplumar",
    "desplumaron": "desplumar",
    "desplumará": "desplumar",
    "desplumarán": "desplumar",
    "desplumarás": "desplumar",
    "desplumaré": "desplumar",
    "desplumaréis": "desplumar",
    "desplumaría": "desplumar",
    "desplumaríais": "desplumar",
    "desplumaríamos": "desplumar",
    "desplumarían": "desplumar",
    "desplumarías": "desplumar",
    "desplumas": "desplumar",
    "desplumase": "desplumar",
    "desplumaseis": "desplumar",
    "desplumasen": "desplumar",
    "desplumases": "desplumar",
    "desplumaste": "desplumar",
    "desplumasteis": "desplumar",
    "desplume": "desplumar",
    "desplumemos": "desplumar",
    "desplumen": "desplumar",
    "desplumes": "desplume",
    "desplumo": "desplumar",
    "desplumábamos": "desplumar",
    "desplumáis": "desplumar",
    "desplumáramos": "desplumar",
    "desplumáremos": "desplumar",
    "desplumásemos": "desplumar",
    "desplumé": "desplumar",
    "despluméis": "desplumar",
    "desplumó": "desplumar",
    "despoblaba": "despoblar",
    "despoblabais": "despoblar",
    "despoblaban": "despoblar",
    "despoblabas": "despoblar",
    "despoblaciones": "despoblación",
    "despoblad": "despoblar",
    "despoblada": "despoblar",
    "despobladas": "despoblar",
    "despoblado": "despoblar",
    "despobladora": "despoblador",
    "despobladoras": "despoblador",
    "despobladores": "despoblador",
    "despoblados": "despoblar",
    "despoblamientos": "despoblamiento",
    "despoblamos": "despoblar",
    "despoblando": "despoblar",
    "despoblara": "despoblar",
    "despoblarais": "despoblar",
    "despoblaran": "despoblar",
    "despoblaras": "despoblar",
    "despoblare": "despoblar",
    "despoblareis": "despoblar",
    "despoblaremos": "despoblar",
    "despoblaren": "despoblar",
    "despoblares": "despoblar",
    "despoblaron": "despoblar",
    "despoblará": "despoblar",
    "despoblarán": "despoblar",
    "despoblarás": "despoblar",
    "despoblaré": "despoblar",
    "despoblaréis": "despoblar",
    "despoblaría": "despoblar",
    "despoblaríais": "despoblar",
    "despoblaríamos": "despoblar",
    "despoblarían": "despoblar",
    "despoblarías": "despoblar",
    "despoblase": "despoblar",
    "despoblaseis": "despoblar",
    "despoblasen": "despoblar",
    "despoblases": "despoblar",
    "despoblaste": "despoblar",
    "despoblasteis": "despoblar",
    "despoblemos": "despoblar",
    "despoblábamos": "despoblar",
    "despobláis": "despoblar",
    "despobláramos": "despoblar",
    "despobláremos": "despoblar",
    "despoblásemos": "despoblar",
    "despoblé": "despoblar",
    "despobléis": "despoblar",
    "despobló": "despoblar",
    "despoetizamientos": "despoetizamiento",
    "despoja": "despojar",
    "despojaba": "despojar",
    "despojabais": "despojar",
    "despojaban": "despojar",
    "despojabas": "despojar",
    "despojad": "despojar",
    "despojada": "despojar",
    "despojadas": "despojar",
    "despojado": "despojar",
    "despojadora": "despojador",
    "despojadoras": "despojador",
    "despojadores": "despojador",
    "despojados": "despojar",
    "despojamos": "despojar",
    "despojan": "despojar",
    "despojando": "despojar",
    "despojara": "despojar",
    "despojarais": "despojar",
    "despojaran": "despojar",
    "despojaras": "despojar",
    "despojare": "despojar",
    "despojareis": "despojar",
    "despojaremos": "despojar",
    "despojaren": "despojar",
    "despojares": "despojar",
    "despojaron": "despojar",
    "despojará": "despojar",
    "despojarán": "despojar",
    "despojarás": "despojar",
    "despojaré": "despojar",
    "despojaréis": "despojar",
    "despojaría": "despojar",
    "despojaríais": "despojar",
    "despojaríamos": "despojar",
    "despojarían": "despojar",
    "despojarías": "despojar",
    "despojas": "despojar",
    "despojase": "despojar",
    "despojaseis": "despojar",
    "despojasen": "despojar",
    "despojases": "despojar",
    "despojaste": "despojar",
    "despojasteis": "despojar",
    "despoje": "despojar",
    "despojemos": "despojar",
    "despojen": "despojar",
    "despojes": "despojar",
    "despojo": "despojar",
    "despojos": "despojo",
    "despojábamos": "despojar",
    "despojáis": "despojar",
    "despojáramos": "despojar",
    "despojáremos": "despojar",
    "despojásemos": "despojar",
    "despojé": "despojar",
    "despojéis": "despojar",
    "despojó": "despojar",
    "despolarice": "despolarizar",
    "despolaricemos": "despolarizar",
    "despolaricen": "despolarizar",
    "despolarices": "despolarizar",
    "despolaricé": "despolarizar",
    "despolaricéis": "despolarizar",
    "despolariza": "despolarizar",
    "despolarizaba": "despolarizar",
    "despolarizabais": "despolarizar",
    "despolarizaban": "despolarizar",
    "despolarizabas": "despolarizar",
    "despolarizaciones": "despolarización",
    "despolarizad": "despolarizar",
    "despolarizada": "despolarizar",
    "despolarizadas": "despolarizar",
    "despolarizado": "despolarizar",
    "despolarizadores": "despolarizador",
    "despolarizados": "despolarizar",
    "despolarizamos": "despolarizar",
    "despolarizan": "despolarizar",
    "despolarizando": "despolarizar",
    "despolarizantes": "despolarizante",
    "despolarizara": "despolarizar",
    "despolarizarais": "despolarizar",
    "despolarizaran": "despolarizar",
    "despolarizaras": "despolarizar",
    "despolarizare": "despolarizar",
    "despolarizareis": "despolarizar",
    "despolarizaremos": "despolarizar",
    "despolarizaren": "despolarizar",
    "despolarizares": "despolarizar",
    "despolarizaron": "despolarizar",
    "despolarizará": "despolarizar",
    "despolarizarán": "despolarizar",
    "despolarizarás": "despolarizar",
    "despolarizaré": "despolarizar",
    "despolarizaréis": "despolarizar",
    "despolarizaría": "despolarizar",
    "despolarizaríais": "despolarizar",
    "despolarizaríamos": "despolarizar",
    "despolarizarían": "despolarizar",
    "despolarizarías": "despolarizar",
    "despolarizas": "despolarizar",
    "despolarizase": "despolarizar",
    "despolarizaseis": "despolarizar",
    "despolarizasen": "despolarizar",
    "despolarizases": "despolarizar",
    "despolarizaste": "despolarizar",
    "despolarizasteis": "despolarizar",
    "despolarizo": "despolarizar",
    "despolarizábamos": "despolarizar",
    "despolarizáis": "despolarizar",
    "despolarizáramos": "despolarizar",
    "despolarizáremos": "despolarizar",
    "despolarizásemos": "despolarizar",
    "despolarizó": "despolarizar",
    "despolimerizaciones": "despolimerización",
    "despolitizaciones": "despolitización",
    "despolva": "despolvar",
    "despolvaba": "despolvar",
    "despolvabais": "despolvar",
    "despolvaban": "despolvar",
    "despolvabas": "despolvar",
    "despolvad": "despolvar",
    "despolvada": "despolvar",
    "despolvadas": "despolvar",
    "despolvado": "despolvar",
    "despolvados": "despolvar",
    "despolvamos": "despolvar",
    "despolvan": "despolvar",
    "despolvando": "despolvar",
    "despolvara": "despolvar",
    "despolvarais": "despolvar",
    "despolvaran": "despolvar",
    "despolvaras": "despolvar",
    "despolvare": "despolvar",
    "despolvareis": "despolvar",
    "despolvaremos": "despolvar",
    "despolvaren": "despolvar",
    "despolvares": "despolvar",
    "despolvaron": "despolvar",
    "despolvará": "despolvar",
    "despolvarán": "despolvar",
    "despolvarás": "despolvar",
    "despolvaré": "despolvar",
    "despolvaréis": "despolvar",
    "despolvaría": "despolvar",
    "despolvaríais": "despolvar",
    "despolvaríamos": "despolvar",
    "despolvarían": "despolvar",
    "despolvarías": "despolvar",
    "despolvas": "despolvar",
    "despolvase": "despolvar",
    "despolvaseis": "despolvar",
    "despolvasen": "despolvar",
    "despolvases": "despolvar",
    "despolvaste": "despolvar",
    "despolvasteis": "despolvar",
    "despolve": "despolvar",
    "despolvemos": "despolvar",
    "despolven": "despolvar",
    "despolves": "despolvar",
    "despolvo": "despolvar",
    "despolvorea": "despolvorear",
    "despolvoreaba": "despolvorear",
    "despolvoreabais": "despolvorear",
    "despolvoreaban": "despolvorear",
    "despolvoreabas": "despolvorear",
    "despolvoread": "despolvorear",
    "despolvoreada": "despolvorear",
    "despolvoreadas": "despolvorear",
    "despolvoreado": "despolvorear",
    "despolvoreadores": "despolvoreador",
    "despolvoreados": "despolvorear",
    "despolvoreamos": "despolvorear",
    "despolvorean": "despolvorear",
    "despolvoreando": "despolvorear",
    "despolvoreara": "despolvorear",
    "despolvorearais": "despolvorear",
    "despolvorearan": "despolvorear",
    "despolvorearas": "despolvorear",
    "despolvoreare": "despolvorear",
    "despolvoreareis": "despolvorear",
    "despolvorearemos": "despolvorear",
    "despolvorearen": "despolvorear",
    "despolvoreares": "despolvorear",
    "despolvorearon": "despolvorear",
    "despolvoreará": "despolvorear",
    "despolvorearán": "despolvorear",
    "despolvorearás": "despolvorear",
    "despolvorearé": "despolvorear",
    "despolvorearéis": "despolvorear",
    "despolvorearía": "despolvorear",
    "despolvorearíais": "despolvorear",
    "despolvorearíamos": "despolvorear",
    "despolvorearían": "despolvorear",
    "despolvorearías": "despolvorear",
    "despolvoreas": "despolvorear",
    "despolvorease": "despolvorear",
    "despolvoreaseis": "despolvorear",
    "despolvoreasen": "despolvorear",
    "despolvoreases": "despolvorear",
    "despolvoreaste": "despolvorear",
    "despolvoreasteis": "despolvorear",
    "despolvoree": "despolvorear",
    "despolvoreemos": "despolvorear",
    "despolvoreen": "despolvorear",
    "despolvorees": "despolvorear",
    "despolvoreo": "despolvorear",
    "despolvoreos": "despolvoreo",
    "despolvoreábamos": "despolvorear",
    "despolvoreáis": "despolvorear",
    "despolvoreáramos": "despolvorear",
    "despolvoreáremos": "despolvorear",
    "despolvoreásemos": "despolvorear",
    "despolvoreé": "despolvorear",
    "despolvoreéis": "despolvorear",
    "despolvoreó": "despolvorear",
    "despolvábamos": "despolvar",
    "despolváis": "despolvar",
    "despolváramos": "despolvar",
    "despolváremos": "despolvar",
    "despolvásemos": "despolvar",
    "despolvé": "despolvar",
    "despolvéis": "despolvar",
    "despolvó": "despolvar",
    "despopularizaciones": "despopularización",
    "desportilla": "desportillar",
    "desportillaba": "desportillar",
    "desportillabais": "desportillar",
    "desportillaban": "desportillar",
    "desportillabas": "desportillar",
    "desportillad": "desportillar",
    "desportillada": "desportillar",
    "desportilladas": "desportillar",
    "desportillado": "desportillar",
    "desportillados": "desportillar",
    "desportilladuras": "desportilladura",
    "desportillamos": "desportillar",
    "desportillan": "desportillar",
    "desportillando": "desportillar",
    "desportillara": "desportillar",
    "desportillarais": "desportillar",
    "desportillaran": "desportillar",
    "desportillaras": "desportillar",
    "desportillare": "desportillar",
    "desportillareis": "desportillar",
    "desportillaremos": "desportillar",
    "desportillaren": "desportillar",
    "desportillares": "desportillar",
    "desportillaron": "desportillar",
    "desportillará": "desportillar",
    "desportillarán": "desportillar",
    "desportillarás": "desportillar",
    "desportillaré": "desportillar",
    "desportillaréis": "desportillar",
    "desportillaría": "desportillar",
    "desportillaríais": "desportillar",
    "desportillaríamos": "desportillar",
    "desportillarían": "desportillar",
    "desportillarías": "desportillar",
    "desportillas": "desportillar",
    "desportillase": "desportillar",
    "desportillaseis": "desportillar",
    "desportillasen": "desportillar",
    "desportillases": "desportillar",
    "desportillaste": "desportillar",
    "desportillasteis": "desportillar",
    "desportille": "desportillar",
    "desportillemos": "desportillar",
    "desportillen": "desportillar",
    "desportilles": "desportillar",
    "desportillo": "desportillar",
    "desportillábamos": "desportillar",
    "desportilláis": "desportillar",
    "desportilláramos": "desportillar",
    "desportilláremos": "desportillar",
    "desportillásemos": "desportillar",
    "desportillé": "desportillar",
    "desportilléis": "desportillar",
    "desportilló": "desportillar",
    "desposa": "desposar",
    "desposaba": "desposar",
    "desposabais": "desposar",
    "desposaban": "desposar",
    "desposabas": "desposar",
    "desposad": "desposar",
    "desposada": "desposar",
    "desposadas": "desposar",
    "desposado": "desposar",
    "desposados": "desposar",
    "desposamos": "desposar",
    "desposan": "desposar",
    "desposando": "desposar",
    "desposara": "desposar",
    "desposarais": "desposar",
    "desposaran": "desposar",
    "desposaras": "desposar",
    "desposare": "desposar",
    "desposareis": "desposar",
    "desposaremos": "desposar",
    "desposaren": "desposar",
    "desposares": "desposar",
    "desposaron": "desposar",
    "desposará": "desposar",
    "desposarán": "desposar",
    "desposarás": "desposar",
    "desposaré": "desposar",
    "desposaréis": "desposar",
    "desposaría": "desposar",
    "desposaríais": "desposar",
    "desposaríamos": "desposar",
    "desposarían": "desposar",
    "desposarías": "desposar",
    "desposas": "desposar",
    "desposase": "desposar",
    "desposaseis": "desposar",
    "desposasen": "desposar",
    "desposases": "desposar",
    "desposaste": "desposar",
    "desposasteis": "desposar",
    "despose": "desposar",
    "desposea": "desposeer",
    "desposeamos": "desposeer",
    "desposean": "desposeer",
    "desposeas": "desposeer",
    "desposee": "desposeer",
    "desposeed": "desposeer",
    "desposeemos": "desposeer",
    "desposeen": "desposeer",
    "desposeeremos": "desposeer",
    "desposeerá": "desposeer",
    "desposeerán": "desposeer",
    "desposeerás": "desposeer",
    "desposeeré": "desposeer",
    "desposeeréis": "desposeer",
    "desposeería": "desposeer",
    "desposeeríais": "desposeer",
    "desposeeríamos": "desposeer",
    "desposeerían": "desposeer",
    "desposeerías": "desposeer",
    "desposees": "desposeer",
    "desposeimientos": "desposeimiento",
    "desposemos": "desposar",
    "desposen": "desposar",
    "desposeo": "desposeer",
    "desposes": "desposar",
    "desposesiones": "desposesión",
    "desposeyendo": "desposeer",
    "desposeyera": "desposeer",
    "desposeyerais": "desposeer",
    "desposeyeran": "desposeer",
    "desposeyeras": "desposeer",
    "desposeyere": "desposeer",
    "desposeyereis": "desposeer",
    "desposeyeren": "desposeer",
    "desposeyeres": "desposeer",
    "desposeyeron": "desposeer",
    "desposeyese": "desposeer",
    "desposeyeseis": "desposeer",
    "desposeyesen": "desposeer",
    "desposeyeses": "desposeer",
    "desposeyéramos": "desposeer",
    "desposeyéremos": "desposeer",
    "desposeyésemos": "desposeer",
    "desposeyó": "desposeer",
    "desposeáis": "desposeer",
    "desposeéis": "desposeer",
    "desposeí": "desposeer",
    "desposeía": "desposeer",
    "desposeíais": "desposeer",
    "desposeíamos": "desposeer",
    "desposeían": "desposeer",
    "desposeías": "desposeer",
    "desposeída": "desposeído",
    "desposeídas": "desposeído",
    "desposeído": "desposeer",
    "desposeídos": "desposeído",
    "desposeímos": "desposeer",
    "desposeíste": "desposeer",
    "desposeísteis": "desposeer",
    "desposo": "desposar",
    "desposorios": "desposorio",
    "desposta": "despostar",
    "despostaba": "despostar",
    "despostabais": "despostar",
    "despostaban": "despostar",
    "despostabas": "despostar",
    "despostad": "despostar",
    "despostada": "despostar",
    "despostadas": "despostar",
    "despostado": "despostar",
    "despostados": "despostar",
    "despostamos": "despostar",
    "despostan": "despostar",
    "despostando": "despostar",
    "despostara": "despostar",
    "despostarais": "despostar",
    "despostaran": "despostar",
    "despostaras": "despostar",
    "despostare": "despostar",
    "despostareis": "despostar",
    "despostaremos": "despostar",
    "despostaren": "despostar",
    "despostares": "despostar",
    "despostaron": "despostar",
    "despostará": "despostar",
    "despostarán": "despostar",
    "despostarás": "despostar",
    "despostaré": "despostar",
    "despostaréis": "despostar",
    "despostaría": "despostar",
    "despostaríais": "despostar",
    "despostaríamos": "despostar",
    "despostarían": "despostar",
    "despostarías": "despostar",
    "despostas": "despostar",
    "despostase": "despostar",
    "despostaseis": "despostar",
    "despostasen": "despostar",
    "despostases": "despostar",
    "despostaste": "despostar",
    "despostasteis": "despostar",
    "desposte": "despostar",
    "despostemos": "despostar",
    "desposten": "despostar",
    "despostes": "desposte",
    "despostilla": "despostillar",
    "despostillaba": "despostillar",
    "despostillabais": "despostillar",
    "despostillaban": "despostillar",
    "despostillabas": "despostillar",
    "despostillad": "despostillar",
    "despostillada": "despostillar",
    "despostilladas": "despostillar",
    "despostillado": "despostillar",
    "despostillados": "despostillar",
    "despostillamos": "despostillar",
    "despostillan": "despostillar",
    "despostillando": "despostillar",
    "despostillara": "despostillar",
    "despostillarais": "despostillar",
    "despostillaran": "despostillar",
    "despostillaras": "despostillar",
    "despostillare": "despostillar",
    "despostillareis": "despostillar",
    "despostillaremos": "despostillar",
    "despostillaren": "despostillar",
    "despostillares": "despostillar",
    "despostillaron": "despostillar",
    "despostillará": "despostillar",
    "despostillarán": "despostillar",
    "despostillarás": "despostillar",
    "despostillaré": "despostillar",
    "despostillaréis": "despostillar",
    "despostillaría": "despostillar",
    "despostillaríais": "despostillar",
    "despostillaríamos": "despostillar",
    "despostillarían": "despostillar",
    "despostillarías": "despostillar",
    "despostillas": "despostillar",
    "despostillase": "despostillar",
    "despostillaseis": "despostillar",
    "despostillasen": "despostillar",
    "despostillases": "despostillar",
    "despostillaste": "despostillar",
    "despostillasteis": "despostillar",
    "despostille": "despostillar",
    "despostillemos": "despostillar",
    "despostillen": "despostillar",
    "despostilles": "despostillar",
    "despostillo": "despostillar",
    "despostillábamos": "despostillar",
    "despostilláis": "despostillar",
    "despostilláramos": "despostillar",
    "despostilláremos": "despostillar",
    "despostillásemos": "despostillar",
    "despostillé": "despostillar",
    "despostilléis": "despostillar",
    "despostilló": "despostillar",
    "desposto": "despostar",
    "despostábamos": "despostar",
    "despostáis": "despostar",
    "despostáramos": "despostar",
    "despostáremos": "despostar",
    "despostásemos": "despostar",
    "desposté": "despostar",
    "despostéis": "despostar",
    "despostó": "despostar",
    "desposábamos": "desposar",
    "desposáis": "desposar",
    "desposáramos": "desposar",
    "desposáremos": "desposar",
    "desposásemos": "desposar",
    "desposé": "desposar",
    "desposéis": "desposar",
    "desposó": "desposar",
    "despotice": "despotizar",
    "despoticemos": "despotizar",
    "despoticen": "despotizar",
    "despotices": "despotizar",
    "despoticé": "despotizar",
    "despoticéis": "despotizar",
    "despotismos": "despotismo",
    "despotiza": "despotizar",
    "despotizaba": "despotizar",
    "despotizabais": "despotizar",
    "despotizaban": "despotizar",
    "despotizabas": "despotizar",
    "despotizad": "despotizar",
    "despotizada": "despotizar",
    "despotizadas": "despotizar",
    "despotizado": "despotizar",
    "despotizados": "despotizar",
    "despotizamos": "despotizar",
    "despotizan": "despotizar",
    "despotizando": "despotizar",
    "despotizara": "despotizar",
    "despotizarais": "despotizar",
    "despotizaran": "despotizar",
    "despotizaras": "despotizar",
    "despotizare": "despotizar",
    "despotizareis": "despotizar",
    "despotizaremos": "despotizar",
    "despotizaren": "despotizar",
    "despotizares": "despotizar",
    "despotizaron": "despotizar",
    "despotizará": "despotizar",
    "despotizarán": "despotizar",
    "despotizarás": "despotizar",
    "despotizaré": "despotizar",
    "despotizaréis": "despotizar",
    "despotizaría": "despotizar",
    "despotizaríais": "despotizar",
    "despotizaríamos": "despotizar",
    "despotizarían": "despotizar",
    "despotizarías": "despotizar",
    "despotizas": "despotizar",
    "despotizase": "despotizar",
    "despotizaseis": "despotizar",
    "despotizasen": "despotizar",
    "despotizases": "despotizar",
    "despotizaste": "despotizar",
    "despotizasteis": "despotizar",
    "despotizo": "despotizar",
    "despotizábamos": "despotizar",
    "despotizáis": "despotizar",
    "despotizáramos": "despotizar",
    "despotizáremos": "despotizar",
    "despotizásemos": "despotizar",
    "despotizó": "despotizar",
    "despotrica": "despotricar",
    "despotricaba": "despotricar",
    "despotricabais": "despotricar",
    "despotricaban": "despotricar",
    "despotricabas": "despotricar",
    "despotricad": "despotricar",
    "despotricada": "despotricar",
    "despotricadas": "despotricar",
    "despotricado": "despotricar",
    "despotricados": "despotricar",
    "despotricamos": "despotricar",
    "despotrican": "despotricar",
    "despotricando": "despotricar",
    "despotricara": "despotricar",
    "despotricarais": "despotricar",
    "despotricaran": "despotricar",
    "despotricaras": "despotricar",
    "despotricare": "despotricar",
    "despotricareis": "despotricar",
    "despotricaremos": "despotricar",
    "despotricaren": "despotricar",
    "despotricares": "despotricar",
    "despotricaron": "despotricar",
    "despotricará": "despotricar",
    "despotricarán": "despotricar",
    "despotricarás": "despotricar",
    "despotricaré": "despotricar",
    "despotricaréis": "despotricar",
    "despotricaría": "despotricar",
    "despotricaríais": "despotricar",
    "despotricaríamos": "despotricar",
    "despotricarían": "despotricar",
    "despotricarías": "despotricar",
    "despotricas": "despotricar",
    "despotricase": "despotricar",
    "despotricaseis": "despotricar",
    "despotricasen": "despotricar",
    "despotricases": "despotricar",
    "despotricaste": "despotricar",
    "despotricasteis": "despotricar",
    "despotrico": "despotricar",
    "despotricábamos": "despotricar",
    "despotricáis": "despotricar",
    "despotricáramos": "despotricar",
    "despotricáremos": "despotricar",
    "despotricásemos": "despotricar",
    "despotricó": "despotricar",
    "despotrique": "despotricar",
    "despotriquemos": "despotricar",
    "despotriquen": "despotricar",
    "despotriques": "despotrique",
    "despotriqué": "despotricar",
    "despotriquéis": "despotricar",
    "desprecia": "despreciar",
    "despreciaba": "despreciar",
    "despreciabais": "despreciar",
    "despreciaban": "despreciar",
    "despreciabas": "despreciar",
    "despreciables": "despreciable",
    "despreciad": "despreciar",
    "despreciada": "despreciar",
    "despreciadas": "despreciar",
    "despreciado": "despreciar",
    "despreciadora": "despreciador",
    "despreciadoras": "despreciador",
    "despreciadores": "despreciador",
    "despreciados": "despreciar",
    "despreciamos": "despreciar",
    "desprecian": "despreciar",
    "despreciando": "despreciar",
    "despreciara": "despreciar",
    "despreciarais": "despreciar",
    "despreciaran": "despreciar",
    "despreciaras": "despreciar",
    "despreciare": "despreciar",
    "despreciareis": "despreciar",
    "despreciaremos": "despreciar",
    "despreciaren": "despreciar",
    "despreciares": "despreciar",
    "despreciaron": "despreciar",
    "despreciará": "despreciar",
    "despreciarán": "despreciar",
    "despreciarás": "despreciar",
    "despreciaré": "despreciar",
    "despreciaréis": "despreciar",
    "despreciaría": "despreciar",
    "despreciaríais": "despreciar",
    "despreciaríamos": "despreciar",
    "despreciarían": "despreciar",
    "despreciarías": "despreciar",
    "desprecias": "despreciar",
    "despreciase": "despreciar",
    "despreciaseis": "despreciar",
    "despreciasen": "despreciar",
    "despreciases": "despreciar",
    "despreciaste": "despreciar",
    "despreciasteis": "despreciar",
    "despreciativa": "despreciativo",
    "despreciativas": "despreciativo",
    "despreciativos": "despreciativo",
    "desprecie": "despreciar",
    "despreciemos": "despreciar",
    "desprecien": "despreciar",
    "desprecies": "despreciar",
    "desprecinta": "desprecintar",
    "desprecintaba": "desprecintar",
    "desprecintabais": "desprecintar",
    "desprecintaban": "desprecintar",
    "desprecintabas": "desprecintar",
    "desprecintad": "desprecintar",
    "desprecintada": "desprecintar",
    "desprecintadas": "desprecintar",
    "desprecintado": "desprecintar",
    "desprecintados": "desprecintar",
    "desprecintamos": "desprecintar",
    "desprecintan": "desprecintar",
    "desprecintando": "desprecintar",
    "desprecintara": "desprecintar",
    "desprecintarais": "desprecintar",
    "desprecintaran": "desprecintar",
    "desprecintaras": "desprecintar",
    "desprecintare": "desprecintar",
    "desprecintareis": "desprecintar",
    "desprecintaremos": "desprecintar",
    "desprecintaren": "desprecintar",
    "desprecintares": "desprecintar",
    "desprecintaron": "desprecintar",
    "desprecintará": "desprecintar",
    "desprecintarán": "desprecintar",
    "desprecintarás": "desprecintar",
    "desprecintaré": "desprecintar",
    "desprecintaréis": "desprecintar",
    "desprecintaría": "desprecintar",
    "desprecintaríais": "desprecintar",
    "desprecintaríamos": "desprecintar",
    "desprecintarían": "desprecintar",
    "desprecintarías": "desprecintar",
    "desprecintas": "desprecintar",
    "desprecintase": "desprecintar",
    "desprecintaseis": "desprecintar",
    "desprecintasen": "desprecintar",
    "desprecintases": "desprecintar",
    "desprecintaste": "desprecintar",
    "desprecintasteis": "desprecintar",
    "desprecinte": "desprecintar",
    "desprecintemos": "desprecintar",
    "desprecinten": "desprecintar",
    "desprecintes": "desprecintar",
    "desprecinto": "desprecintar",
    "desprecintábamos": "desprecintar",
    "desprecintáis": "desprecintar",
    "desprecintáramos": "desprecintar",
    "desprecintáremos": "desprecintar",
    "desprecintásemos": "desprecintar",
    "desprecinté": "desprecintar",
    "desprecintéis": "desprecintar",
    "desprecintó": "desprecintar",
    "desprecio": "despreciar",
    "desprecios": "desprecio",
    "despreciábamos": "despreciar",
    "despreciáis": "despreciar",
    "despreciáramos": "despreciar",
    "despreciáremos": "despreciar",
    "despreciásemos": "despreciar",
    "desprecié": "despreciar",
    "despreciéis": "despreciar",
    "despreció": "despreciar",
    "desprenda": "desprender",
    "desprendamos": "desprender",
    "desprendan": "desprender",
    "desprendas": "desprender",
    "desprende": "desprender",
    "desprended": "desprender",
    "desprendemos": "desprender",
    "desprenden": "desprender",
    "desprenderemos": "desprender",
    "desprenderá": "desprender",
    "desprenderán": "desprender",
    "desprenderás": "desprender",
    "desprenderé": "desprender",
    "desprenderéis": "desprender",
    "desprendería": "desprender",
    "desprenderíais": "desprender",
    "desprenderíamos": "desprender",
    "desprenderían": "desprender",
    "desprenderías": "desprender",
    "desprendes": "desprender",
    "desprendida": "desprender",
    "desprendidas": "desprender",
    "desprendido": "desprender",
    "desprendidos": "desprender",
    "desprendiendo": "desprender",
    "desprendiera": "desprender",
    "desprendierais": "desprender",
    "desprendieran": "desprender",
    "desprendieras": "desprender",
    "desprendiere": "desprender",
    "desprendiereis": "desprender",
    "desprendieren": "desprender",
    "desprendieres": "desprender",
    "desprendieron": "desprender",
    "desprendiese": "desprender",
    "desprendieseis": "desprender",
    "desprendiesen": "desprender",
    "desprendieses": "desprender",
    "desprendimientos": "desprendimiento",
    "desprendimos": "desprender",
    "desprendiste": "desprender",
    "desprendisteis": "desprender",
    "desprendiéramos": "desprender",
    "desprendiéremos": "desprender",
    "desprendiésemos": "desprender",
    "desprendió": "desprender",
    "desprendo": "desprender",
    "desprendáis": "desprender",
    "desprendéis": "desprender",
    "desprendí": "desprender",
    "desprendía": "desprender",
    "desprendíais": "desprender",
    "desprendíamos": "desprender",
    "desprendían": "desprender",
    "desprendías": "desprender",
    "despreocupa": "despreocupar",
    "despreocupaba": "despreocupar",
    "despreocupabais": "despreocupar",
    "despreocupaban": "despreocupar",
    "despreocupabas": "despreocupar",
    "despreocupaciones": "despreocupación",
    "despreocupad": "despreocupar",
    "despreocupada": "despreocupar",
    "despreocupadas": "despreocupar",
    "despreocupado": "despreocupar",
    "despreocupados": "despreocupar",
    "despreocupamos": "despreocupar",
    "despreocupan": "despreocupar",
    "despreocupando": "despreocupar",
    "despreocupara": "despreocupar",
    "despreocuparais": "despreocupar",
    "despreocuparan": "despreocupar",
    "despreocuparas": "despreocupar",
    "despreocupare": "despreocupar",
    "despreocupareis": "despreocupar",
    "despreocuparemos": "despreocupar",
    "despreocuparen": "despreocupar",
    "despreocupares": "despreocupar",
    "despreocuparon": "despreocupar",
    "despreocupará": "despreocupar",
    "despreocuparán": "despreocupar",
    "despreocuparás": "despreocupar",
    "despreocuparé": "despreocupar",
    "despreocuparéis": "despreocupar",
    "despreocuparía": "despreocupar",
    "despreocuparíais": "despreocupar",
    "despreocuparíamos": "despreocupar",
    "despreocuparían": "despreocupar",
    "despreocuparías": "despreocupar",
    "despreocupas": "despreocupar",
    "despreocupase": "despreocupar",
    "despreocupaseis": "despreocupar",
    "despreocupasen": "despreocupar",
    "despreocupases": "despreocupar",
    "despreocupaste": "despreocupar",
    "despreocupasteis": "despreocupar",
    "despreocupe": "despreocupar",
    "despreocupemos": "despreocupar",
    "despreocupen": "despreocupar",
    "despreocupes": "despreocupar",
    "despreocupo": "despreocupar",
    "despreocupábamos": "despreocupar",
    "despreocupáis": "despreocupar",
    "despreocupáramos": "despreocupar",
    "despreocupáremos": "despreocupar",
    "despreocupásemos": "despreocupar",
    "despreocupé": "despreocupar",
    "despreocupéis": "despreocupar",
    "despreocupó": "despreocupar",
    "desprestigia": "desprestigiar",
    "desprestigiaba": "desprestigiar",
    "desprestigiabais": "desprestigiar",
    "desprestigiaban": "desprestigiar",
    "desprestigiabas": "desprestigiar",
    "desprestigiad": "desprestigiar",
    "desprestigiada": "desprestigiar",
    "desprestigiadas": "desprestigiar",
    "desprestigiado": "desprestigiar",
    "desprestigiados": "desprestigiar",
    "desprestigiamos": "desprestigiar",
    "desprestigian": "desprestigiar",
    "desprestigiando": "desprestigiar",
    "desprestigiara": "desprestigiar",
    "desprestigiarais": "desprestigiar",
    "desprestigiaran": "desprestigiar",
    "desprestigiaras": "desprestigiar",
    "desprestigiare": "desprestigiar",
    "desprestigiareis": "desprestigiar",
    "desprestigiaremos": "desprestigiar",
    "desprestigiaren": "desprestigiar",
    "desprestigiares": "desprestigiar",
    "desprestigiaron": "desprestigiar",
    "desprestigiará": "desprestigiar",
    "desprestigiarán": "desprestigiar",
    "desprestigiarás": "desprestigiar",
    "desprestigiaré": "desprestigiar",
    "desprestigiaréis": "desprestigiar",
    "desprestigiaría": "desprestigiar",
    "desprestigiaríais": "desprestigiar",
    "desprestigiaríamos": "desprestigiar",
    "desprestigiarían": "desprestigiar",
    "desprestigiarías": "desprestigiar",
    "desprestigias": "desprestigiar",
    "desprestigiase": "desprestigiar",
    "desprestigiaseis": "desprestigiar",
    "desprestigiasen": "desprestigiar",
    "desprestigiases": "desprestigiar",
    "desprestigiaste": "desprestigiar",
    "desprestigiasteis": "desprestigiar",
    "desprestigie": "desprestigiar",
    "desprestigiemos": "desprestigiar",
    "desprestigien": "desprestigiar",
    "desprestigies": "desprestigiar",
    "desprestigio": "desprestigiar",
    "desprestigios": "desprestigio",
    "desprestigiábamos": "desprestigiar",
    "desprestigiáis": "desprestigiar",
    "desprestigiáramos": "desprestigiar",
    "desprestigiáremos": "desprestigiar",
    "desprestigiásemos": "desprestigiar",
    "desprestigié": "desprestigiar",
    "desprestigiéis": "desprestigiar",
    "desprestigió": "desprestigiar",
    "despresurice": "despresurizar",
    "despresuricemos": "despresurizar",
    "despresuricen": "despresurizar",
    "despresurices": "despresurizar",
    "despresuricé": "despresurizar",
    "despresuricéis": "despresurizar",
    "despresuriza": "despresurizar",
    "despresurizaba": "despresurizar",
    "despresurizabais": "despresurizar",
    "despresurizaban": "despresurizar",
    "despresurizabas": "despresurizar",
    "despresurizad": "despresurizar",
    "despresurizada": "despresurizar",
    "despresurizadas": "despresurizar",
    "despresurizado": "despresurizar",
    "despresurizados": "despresurizar",
    "despresurizamos": "despresurizar",
    "despresurizan": "despresurizar",
    "despresurizando": "despresurizar",
    "despresurizara": "despresurizar",
    "despresurizarais": "despresurizar",
    "despresurizaran": "despresurizar",
    "despresurizaras": "despresurizar",
    "despresurizare": "despresurizar",
    "despresurizareis": "despresurizar",
    "despresurizaremos": "despresurizar",
    "despresurizaren": "despresurizar",
    "despresurizares": "despresurizar",
    "despresurizaron": "despresurizar",
    "despresurizará": "despresurizar",
    "despresurizarán": "despresurizar",
    "despresurizarás": "despresurizar",
    "despresurizaré": "despresurizar",
    "despresurizaréis": "despresurizar",
    "despresurizaría": "despresurizar",
    "despresurizaríais": "despresurizar",
    "despresurizaríamos": "despresurizar",
    "despresurizarían": "despresurizar",
    "despresurizarías": "despresurizar",
    "despresurizas": "despresurizar",
    "despresurizase": "despresurizar",
    "despresurizaseis": "despresurizar",
    "despresurizasen": "despresurizar",
    "despresurizases": "despresurizar",
    "despresurizaste": "despresurizar",
    "despresurizasteis": "despresurizar",
    "despresurizo": "despresurizar",
    "despresurizábamos": "despresurizar",
    "despresurizáis": "despresurizar",
    "despresurizáramos": "despresurizar",
    "despresurizáremos": "despresurizar",
    "despresurizásemos": "despresurizar",
    "despresurizó": "despresurizar",
    "desprevenciones": "desprevención",
    "desprevenida": "desprevenido",
    "desprevenidas": "desprevenido",
    "desprevenidos": "desprevenido",
    "desprivatizaciones": "desprivatización",
    "desprograma": "desprogramar",
    "desprogramaba": "desprogramar",
    "desprogramabais": "desprogramar",
    "desprogramaban": "desprogramar",
    "desprogramabas": "desprogramar",
    "desprogramad": "desprogramar",
    "desprogramada": "desprogramar",
    "desprogramadas": "desprogramar",
    "desprogramado": "desprogramar",
    "desprogramados": "desprogramar",
    "desprogramamos": "desprogramar",
    "desprograman": "desprogramar",
    "desprogramando": "desprogramar",
    "desprogramara": "desprogramar",
    "desprogramarais": "desprogramar",
    "desprogramaran": "desprogramar",
    "desprogramaras": "desprogramar",
    "desprogramare": "desprogramar",
    "desprogramareis": "desprogramar",
    "desprogramaremos": "desprogramar",
    "desprogramaren": "desprogramar",
    "desprogramares": "desprogramar",
    "desprogramaron": "desprogramar",
    "desprogramará": "desprogramar",
    "desprogramarán": "desprogramar",
    "desprogramarás": "desprogramar",
    "desprogramaré": "desprogramar",
    "desprogramaréis": "desprogramar",
    "desprogramaría": "desprogramar",
    "desprogramaríais": "desprogramar",
    "desprogramaríamos": "desprogramar",
    "desprogramarían": "desprogramar",
    "desprogramarías": "desprogramar",
    "desprogramas": "desprogramar",
    "desprogramase": "desprogramar",
    "desprogramaseis": "desprogramar",
    "desprogramasen": "desprogramar",
    "desprogramases": "desprogramar",
    "desprogramaste": "desprogramar",
    "desprogramasteis": "desprogramar",
    "desprograme": "desprogramar",
    "desprogramemos": "desprogramar",
    "desprogramen": "desprogramar",
    "desprogrames": "desprogramar",
    "desprogramo": "desprogramar",
    "desprogramábamos": "desprogramar",
    "desprogramáis": "desprogramar",
    "desprogramáramos": "desprogramar",
    "desprogramáremos": "desprogramar",
    "desprogramásemos": "desprogramar",
    "desprogramé": "desprogramar",
    "desprograméis": "desprogramar",
    "desprogramó": "desprogramar",
    "despropanizadores": "despropanizador",
    "desproporciona": "desproporcionar",
    "desproporcionaba": "desproporcionar",
    "desproporcionabais": "desproporcionar",
    "desproporcionaban": "desproporcionar",
    "desproporcionabas": "desproporcionar",
    "desproporcionad": "desproporcionar",
    "desproporcionada": "desproporcionar",
    "desproporcionadas": "desproporcionar",
    "desproporcionado": "desproporcionar",
    "desproporcionados": "desproporcionar",
    "desproporcionamos": "desproporcionar",
    "desproporcionan": "desproporcionar",
    "desproporcionando": "desproporcionar",
    "desproporcionara": "desproporcionar",
    "desproporcionarais": "desproporcionar",
    "desproporcionaran": "desproporcionar",
    "desproporcionaras": "desproporcionar",
    "desproporcionare": "desproporcionar",
    "desproporcionareis": "desproporcionar",
    "desproporcionaremos": "desproporcionar",
    "desproporcionaren": "desproporcionar",
    "desproporcionares": "desproporcionar",
    "desproporcionaron": "desproporcionar",
    "desproporcionará": "desproporcionar",
    "desproporcionarán": "desproporcionar",
    "desproporcionarás": "desproporcionar",
    "desproporcionaré": "desproporcionar",
    "desproporcionaréis": "desproporcionar",
    "desproporcionaría": "desproporcionar",
    "desproporcionaríais": "desproporcionar",
    "desproporcionaríamos": "desproporcionar",
    "desproporcionarían": "desproporcionar",
    "desproporcionarías": "desproporcionar",
    "desproporcionas": "desproporcionar",
    "desproporcionase": "desproporcionar",
    "desproporcionaseis": "desproporcionar",
    "desproporcionasen": "desproporcionar",
    "desproporcionases": "desproporcionar",
    "desproporcionaste": "desproporcionar",
    "desproporcionasteis": "desproporcionar",
    "desproporcione": "desproporcionar",
    "desproporcionemos": "desproporcionar",
    "desproporcionen": "desproporcionar",
    "desproporciones": "desproporcionar",
    "desproporciono": "desproporcionar",
    "desproporcionábamos": "desproporcionar",
    "desproporcionáis": "desproporcionar",
    "desproporcionáramos": "desproporcionar",
    "desproporcionáremos": "desproporcionar",
    "desproporcionásemos": "desproporcionar",
    "desproporcioné": "desproporcionar",
    "desproporcionéis": "desproporcionar",
    "desproporcionó": "desproporcionar",
    "despropositada": "despropositado",
    "despropositadas": "despropositado",
    "despropositados": "despropositado",
    "despropósitos": "despropósito",
    "desprotege": "desproteger",
    "desproteged": "desproteger",
    "desprotegemos": "desproteger",
    "desprotegen": "desproteger",
    "desprotegeremos": "desproteger",
    "desprotegerá": "desproteger",
    "desprotegerán": "desproteger",
    "desprotegerás": "desproteger",
    "desprotegeré": "desproteger",
    "desprotegeréis": "desproteger",
    "desprotegería": "desproteger",
    "desprotegeríais": "desproteger",
    "desprotegeríamos": "desproteger",
    "desprotegerían": "desproteger",
    "desprotegerías": "desproteger",
    "desproteges": "desproteger",
    "desprotegida": "desproteger",
    "desprotegidas": "desproteger",
    "desprotegido": "desproteger",
    "desprotegidos": "desproteger",
    "desprotegiendo": "desproteger",
    "desprotegiera": "desproteger",
    "desprotegierais": "desproteger",
    "desprotegieran": "desproteger",
    "desprotegieras": "desproteger",
    "desprotegiere": "desproteger",
    "desprotegiereis": "desproteger",
    "desprotegieren": "desproteger",
    "desprotegieres": "desproteger",
    "desprotegieron": "desproteger",
    "desprotegiese": "desproteger",
    "desprotegieseis": "desproteger",
    "desprotegiesen": "desproteger",
    "desprotegieses": "desproteger",
    "desprotegimos": "desproteger",
    "desprotegiste": "desproteger",
    "desprotegisteis": "desproteger",
    "desprotegiéramos": "desproteger",
    "desprotegiéremos": "desproteger",
    "desprotegiésemos": "desproteger",
    "desprotegió": "desproteger",
    "desprotegéis": "desproteger",
    "desprotegí": "desproteger",
    "desprotegía": "desproteger",
    "desprotegíais": "desproteger",
    "desprotegíamos": "desproteger",
    "desprotegían": "desproteger",
    "desprotegías": "desproteger",
    "desproteinizaciones": "desproteinización",
    "desproteja": "desproteger",
    "desprotejamos": "desproteger",
    "desprotejan": "desproteger",
    "desprotejas": "desproteger",
    "desprotejo": "desproteger",
    "desprotejáis": "desproteger",
    "desprovea": "desproveer",
    "desproveamos": "desproveer",
    "desprovean": "desproveer",
    "desproveas": "desproveer",
    "desprovee": "desproveer",
    "desproveed": "desproveer",
    "desproveemos": "desproveer",
    "desproveen": "desproveer",
    "desproveeremos": "desproveer",
    "desproveerá": "desproveer",
    "desproveerán": "desproveer",
    "desproveerás": "desproveer",
    "desproveeré": "desproveer",
    "desproveeréis": "desproveer",
    "desproveería": "desproveer",
    "desproveeríais": "desproveer",
    "desproveeríamos": "desproveer",
    "desproveerían": "desproveer",
    "desproveerías": "desproveer",
    "desprovees": "desproveer",
    "desproveimientos": "desproveimiento",
    "desproveo": "desproveer",
    "desproveyendo": "desproveer",
    "desproveyera": "desproveer",
    "desproveyerais": "desproveer",
    "desproveyeran": "desproveer",
    "desproveyeras": "desproveer",
    "desproveyere": "desproveer",
    "desproveyereis": "desproveer",
    "desproveyeren": "desproveer",
    "desproveyeres": "desproveer",
    "desproveyeron": "desproveer",
    "desproveyese": "desproveer",
    "desproveyeseis": "desproveer",
    "desproveyesen": "desproveer",
    "desproveyeses": "desproveer",
    "desproveyéramos": "desproveer",
    "desproveyéremos": "desproveer",
    "desproveyésemos": "desproveer",
    "desproveyó": "desproveer",
    "desproveáis": "desproveer",
    "desproveéis": "desproveer",
    "desproveí": "desproveer",
    "desproveía": "desproveer",
    "desproveíais": "desproveer",
    "desproveíamos": "desproveer",
    "desproveían": "desproveer",
    "desproveías": "desproveer",
    "desproveída": "desproveer",
    "desproveídas": "desproveer",
    "desproveído": "desproveer",
    "desproveídos": "desproveer",
    "desproveímos": "desproveer",
    "desproveíste": "desproveer",
    "desproveísteis": "desproveer",
    "desprovista": "desproveer",
    "desprovistas": "desproveer",
    "desprovisto": "desproveer",
    "desprovistos": "desproveer",
    "despuebla": "despoblar",
    "despueblan": "despoblar",
    "despueblas": "despoblar",
    "despueble": "despoblar",
    "despueblen": "despoblar",
    "despuebles": "despueble",
    "despueblo": "despoblar",
    "despueblos": "despueblo",
    "despullas": "despulla",
    "despulpa": "despulpar",
    "despulpaba": "despulpar",
    "despulpabais": "despulpar",
    "despulpaban": "despulpar",
    "despulpabas": "despulpar",
    "despulpad": "despulpar",
    "despulpada": "despulpar",
    "despulpadas": "despulpar",
    "despulpado": "despulpar",
    "despulpadores": "despulpador",
    "despulpados": "despulpar",
    "despulpamos": "despulpar",
    "despulpan": "despulpar",
    "despulpando": "despulpar",
    "despulpara": "despulpar",
    "despulparais": "despulpar",
    "despulparan": "despulpar",
    "despulparas": "despulpar",
    "despulpare": "despulpar",
    "despulpareis": "despulpar",
    "despulparemos": "despulpar",
    "despulparen": "despulpar",
    "despulpares": "despulpar",
    "despulparon": "despulpar",
    "despulpará": "despulpar",
    "despulparán": "despulpar",
    "despulparás": "despulpar",
    "despulparé": "despulpar",
    "despulparéis": "despulpar",
    "despulparía": "despulpar",
    "despulparíais": "despulpar",
    "despulparíamos": "despulpar",
    "despulparían": "despulpar",
    "despulparías": "despulpar",
    "despulpas": "despulpar",
    "despulpase": "despulpar",
    "despulpaseis": "despulpar",
    "despulpasen": "despulpar",
    "despulpases": "despulpar",
    "despulpaste": "despulpar",
    "despulpasteis": "despulpar",
    "despulpe": "despulpar",
    "despulpemos": "despulpar",
    "despulpen": "despulpar",
    "despulpes": "despulpar",
    "despulpo": "despulpar",
    "despulpábamos": "despulpar",
    "despulpáis": "despulpar",
    "despulpáramos": "despulpar",
    "despulpáremos": "despulpar",
    "despulpásemos": "despulpar",
    "despulpé": "despulpar",
    "despulpéis": "despulpar",
    "despulpó": "despulpar",
    "despulsamientos": "despulsamiento",
    "despuma": "despumar",
    "despumaba": "despumar",
    "despumabais": "despumar",
    "despumaban": "despumar",
    "despumabas": "despumar",
    "despumaciones": "despumación",
    "despumad": "despumar",
    "despumada": "despumar",
    "despumadas": "despumar",
    "despumado": "despumar",
    "despumadores": "despumador",
    "despumados": "despumar",
    "despumamos": "despumar",
    "despuman": "despumar",
    "despumando": "despumar",
    "despumara": "despumar",
    "despumarais": "despumar",
    "despumaran": "despumar",
    "despumaras": "despumar",
    "despumare": "despumar",
    "despumareis": "despumar",
    "despumaremos": "despumar",
    "despumaren": "despumar",
    "despumares": "despumar",
    "despumaron": "despumar",
    "despumará": "despumar",
    "despumarán": "despumar",
    "despumarás": "despumar",
    "despumaré": "despumar",
    "despumaréis": "despumar",
    "despumaría": "despumar",
    "despumaríais": "despumar",
    "despumaríamos": "despumar",
    "despumarían": "despumar",
    "despumarías": "despumar",
    "despumas": "despumar",
    "despumase": "despumar",
    "despumaseis": "despumar",
    "despumasen": "despumar",
    "despumases": "despumar",
    "despumaste": "despumar",
    "despumasteis": "despumar",
    "despume": "despumar",
    "despumemos": "despumar",
    "despumen": "despumar",
    "despumes": "despumar",
    "despumo": "despumar",
    "despumábamos": "despumar",
    "despumáis": "despumar",
    "despumáramos": "despumar",
    "despumáremos": "despumar",
    "despumásemos": "despumar",
    "despumé": "despumar",
    "despuméis": "despumar",
    "despumó": "despumar",
    "despunta": "despuntar",
    "despuntaba": "despuntar",
    "despuntabais": "despuntar",
    "despuntaban": "despuntar",
    "despuntabas": "despuntar",
    "despuntad": "despuntar",
    "despuntada": "despuntar",
    "despuntadas": "despuntar",
    "despuntado": "despuntar",
    "despuntados": "despuntar",
    "despuntaduras": "despuntadura",
    "despuntamos": "despuntar",
    "despuntan": "despuntar",
    "despuntando": "despuntar",
    "despuntara": "despuntar",
    "despuntarais": "despuntar",
    "despuntaran": "despuntar",
    "despuntaras": "despuntar",
    "despuntare": "despuntar",
    "despuntareis": "despuntar",
    "despuntaremos": "despuntar",
    "despuntaren": "despuntar",
    "despuntares": "despuntar",
    "despuntaron": "despuntar",
    "despuntará": "despuntar",
    "despuntarán": "despuntar",
    "despuntarás": "despuntar",
    "despuntaré": "despuntar",
    "despuntaréis": "despuntar",
    "despuntaría": "despuntar",
    "despuntaríais": "despuntar",
    "despuntaríamos": "despuntar",
    "despuntarían": "despuntar",
    "despuntarías": "despuntar",
    "despuntas": "despuntar",
    "despuntase": "despuntar",
    "despuntaseis": "despuntar",
    "despuntasen": "despuntar",
    "despuntases": "despuntar",
    "despuntaste": "despuntar",
    "despuntasteis": "despuntar",
    "despunte": "despuntar",
    "despuntemos": "despuntar",
    "despunten": "despuntar",
    "despuntes": "despunte",
    "despunto": "despuntar",
    "despuntábamos": "despuntar",
    "despuntáis": "despuntar",
    "despuntáramos": "despuntar",
    "despuntáremos": "despuntar",
    "despuntásemos": "despuntar",
    "despunté": "despuntar",
    "despuntéis": "despuntar",
    "despuntó": "despuntar",
    "despótica": "despótico",
    "despóticas": "despótico",
    "despóticos": "despótico",
    "desquebraja": "desquebrajar",
    "desquebrajaba": "desquebrajar",
    "desquebrajabais": "desquebrajar",
    "desquebrajaban": "desquebrajar",
    "desquebrajabas": "desquebrajar",
    "desquebrajad": "desquebrajar",
    "desquebrajada": "desquebrajar",
    "desquebrajadas": "desquebrajar",
    "desquebrajado": "desquebrajar",
    "desquebrajados": "desquebrajar",
    "desquebrajamos": "desquebrajar",
    "desquebrajan": "desquebrajar",
    "desquebrajando": "desquebrajar",
    "desquebrajara": "desquebrajar",
    "desquebrajarais": "desquebrajar",
    "desquebrajaran": "desquebrajar",
    "desquebrajaras": "desquebrajar",
    "desquebrajare": "desquebrajar",
    "desquebrajareis": "desquebrajar",
    "desquebrajaremos": "desquebrajar",
    "desquebrajaren": "desquebrajar",
    "desquebrajares": "desquebrajar",
    "desquebrajaron": "desquebrajar",
    "desquebrajará": "desquebrajar",
    "desquebrajarán": "desquebrajar",
    "desquebrajarás": "desquebrajar",
    "desquebrajaré": "desquebrajar",
    "desquebrajaréis": "desquebrajar",
    "desquebrajaría": "desquebrajar",
    "desquebrajaríais": "desquebrajar",
    "desquebrajaríamos": "desquebrajar",
    "desquebrajarían": "desquebrajar",
    "desquebrajarías": "desquebrajar",
    "desquebrajas": "desquebrajar",
    "desquebrajase": "desquebrajar",
    "desquebrajaseis": "desquebrajar",
    "desquebrajasen": "desquebrajar",
    "desquebrajases": "desquebrajar",
    "desquebrajaste": "desquebrajar",
    "desquebrajasteis": "desquebrajar",
    "desquebraje": "desquebrajar",
    "desquebrajemos": "desquebrajar",
    "desquebrajen": "desquebrajar",
    "desquebrajes": "desquebrajar",
    "desquebrajo": "desquebrajar",
    "desquebrajábamos": "desquebrajar",
    "desquebrajáis": "desquebrajar",
    "desquebrajáramos": "desquebrajar",
    "desquebrajáremos": "desquebrajar",
    "desquebrajásemos": "desquebrajar",
    "desquebrajé": "desquebrajar",
    "desquebrajéis": "desquebrajar",
    "desquebrajó": "desquebrajar",
    "desqueja": "desquejar",
    "desquejaba": "desquejar",
    "desquejabais": "desquejar",
    "desquejaban": "desquejar",
    "desquejabas": "desquejar",
    "desquejad": "desquejar",
    "desquejada": "desquejar",
    "desquejadas": "desquejar",
    "desquejado": "desquejar",
    "desquejados": "desquejar",
    "desquejamos": "desquejar",
    "desquejan": "desquejar",
    "desquejando": "desquejar",
    "desquejara": "desquejar",
    "desquejarais": "desquejar",
    "desquejaran": "desquejar",
    "desquejaras": "desquejar",
    "desquejare": "desquejar",
    "desquejareis": "desquejar",
    "desquejaremos": "desquejar",
    "desquejaren": "desquejar",
    "desquejares": "desquejar",
    "desquejaron": "desquejar",
    "desquejará": "desquejar",
    "desquejarán": "desquejar",
    "desquejarás": "desquejar",
    "desquejaré": "desquejar",
    "desquejaréis": "desquejar",
    "desquejaría": "desquejar",
    "desquejaríais": "desquejar",
    "desquejaríamos": "desquejar",
    "desquejarían": "desquejar",
    "desquejarías": "desquejar",
    "desquejas": "desquejar",
    "desquejase": "desquejar",
    "desquejaseis": "desquejar",
    "desquejasen": "desquejar",
    "desquejases": "desquejar",
    "desquejaste": "desquejar",
    "desquejasteis": "desquejar",
    "desqueje": "desquejar",
    "desquejemos": "desquejar",
    "desquejen": "desquejar",
    "desquejes": "desqueje",
    "desquejo": "desquejar",
    "desquejábamos": "desquejar",
    "desquejáis": "desquejar",
    "desquejáramos": "desquejar",
    "desquejáremos": "desquejar",
    "desquejásemos": "desquejar",
    "desquejé": "desquejar",
    "desquejéis": "desquejar",
    "desquejó": "desquejar",
    "desqueramos": "desquerer",
    "desquered": "desquerer",
    "desqueremos": "desquerer",
    "desquerida": "desquerer",
    "desqueridas": "desquerer",
    "desquerido": "desquerer",
    "desqueridos": "desquerer",
    "desqueriendo": "desquerer",
    "desquerremos": "desquerer",
    "desquerrá": "desquerer",
    "desquerrán": "desquerer",
    "desquerrás": "desquerer",
    "desquerré": "desquerer",
    "desquerréis": "desquerer",
    "desquerría": "desquerer",
    "desquerríais": "desquerer",
    "desquerríamos": "desquerer",
    "desquerrían": "desquerer",
    "desquerrías": "desquerer",
    "desqueráis": "desquerer",
    "desqueréis": "desquerer",
    "desquería": "desquerer",
    "desqueríais": "desquerer",
    "desqueríamos": "desquerer",
    "desquerían": "desquerer",
    "desquerías": "desquerer",
    "desquicia": "desquiciar",
    "desquiciaba": "desquiciar",
    "desquiciabais": "desquiciar",
    "desquiciaban": "desquiciar",
    "desquiciabas": "desquiciar",
    "desquiciad": "desquiciar",
    "desquiciada": "desquiciar",
    "desquiciadas": "desquiciar",
    "desquiciado": "desquiciar",
    "desquiciadora": "desquiciador",
    "desquiciadoras": "desquiciador",
    "desquiciadores": "desquiciador",
    "desquiciados": "desquiciar",
    "desquiciamientos": "desquiciamiento",
    "desquiciamos": "desquiciar",
    "desquician": "desquiciar",
    "desquiciando": "desquiciar",
    "desquiciara": "desquiciar",
    "desquiciarais": "desquiciar",
    "desquiciaran": "desquiciar",
    "desquiciaras": "desquiciar",
    "desquiciare": "desquiciar",
    "desquiciareis": "desquiciar",
    "desquiciaremos": "desquiciar",
    "desquiciaren": "desquiciar",
    "desquiciares": "desquiciar",
    "desquiciaron": "desquiciar",
    "desquiciará": "desquiciar",
    "desquiciarán": "desquiciar",
    "desquiciarás": "desquiciar",
    "desquiciaré": "desquiciar",
    "desquiciaréis": "desquiciar",
    "desquiciaría": "desquiciar",
    "desquiciaríais": "desquiciar",
    "desquiciaríamos": "desquiciar",
    "desquiciarían": "desquiciar",
    "desquiciarías": "desquiciar",
    "desquicias": "desquiciar",
    "desquiciase": "desquiciar",
    "desquiciaseis": "desquiciar",
    "desquiciasen": "desquiciar",
    "desquiciases": "desquiciar",
    "desquiciaste": "desquiciar",
    "desquiciasteis": "desquiciar",
    "desquicie": "desquiciar",
    "desquiciemos": "desquiciar",
    "desquicien": "desquiciar",
    "desquicies": "desquiciar",
    "desquicio": "desquiciar",
    "desquiciábamos": "desquiciar",
    "desquiciáis": "desquiciar",
    "desquiciáramos": "desquiciar",
    "desquiciáremos": "desquiciar",
    "desquiciásemos": "desquiciar",
    "desquicié": "desquiciar",
    "desquiciéis": "desquiciar",
    "desquició": "desquiciar",
    "desquiera": "desquerer",
    "desquieran": "desquerer",
    "desquieras": "desquerer",
    "desquiere": "desquerer",
    "desquieren": "desquerer",
    "desquieres": "desquerer",
    "desquiero": "desquerer",
    "desquijara": "desquijarar",
    "desquijaraba": "desquijarar",
    "desquijarabais": "desquijarar",
    "desquijaraban": "desquijarar",
    "desquijarabas": "desquijarar",
    "desquijarad": "desquijarar",
    "desquijarada": "desquijarar",
    "desquijaradas": "desquijarar",
    "desquijarado": "desquijarar",
    "desquijarados": "desquijarar",
    "desquijaramientos": "desquijaramiento",
    "desquijaramos": "desquijarar",
    "desquijaran": "desquijarar",
    "desquijarando": "desquijarar",
    "desquijarara": "desquijarar",
    "desquijararais": "desquijarar",
    "desquijararan": "desquijarar",
    "desquijararas": "desquijarar",
    "desquijarare": "desquijarar",
    "desquijarareis": "desquijarar",
    "desquijararemos": "desquijarar",
    "desquijararen": "desquijarar",
    "desquijarares": "desquijarar",
    "desquijararon": "desquijarar",
    "desquijarará": "desquijarar",
    "desquijararán": "desquijarar",
    "desquijararás": "desquijarar",
    "desquijararé": "desquijarar",
    "desquijararéis": "desquijarar",
    "desquijararía": "desquijarar",
    "desquijararíais": "desquijarar",
    "desquijararíamos": "desquijarar",
    "desquijararían": "desquijarar",
    "desquijararías": "desquijarar",
    "desquijaras": "desquijarar",
    "desquijarase": "desquijarar",
    "desquijaraseis": "desquijarar",
    "desquijarasen": "desquijarar",
    "desquijarases": "desquijarar",
    "desquijaraste": "desquijarar",
    "desquijarasteis": "desquijarar",
    "desquijare": "desquijarar",
    "desquijaremos": "desquijarar",
    "desquijaren": "desquijarar",
    "desquijares": "desquijarar",
    "desquijaro": "desquijarar",
    "desquijarábamos": "desquijarar",
    "desquijaráis": "desquijarar",
    "desquijaráramos": "desquijarar",
    "desquijaráremos": "desquijarar",
    "desquijarásemos": "desquijarar",
    "desquijaré": "desquijarar",
    "desquijaréis": "desquijarar",
    "desquijaró": "desquijarar",
    "desquijera": "desquijerar",
    "desquijeraba": "desquijerar",
    "desquijerabais": "desquijerar",
    "desquijeraban": "desquijerar",
    "desquijerabas": "desquijerar",
    "desquijerad": "desquijerar",
    "desquijerada": "desquijerar",
    "desquijeradas": "desquijerar",
    "desquijerado": "desquijerar",
    "desquijerados": "desquijerar",
    "desquijeramos": "desquijerar",
    "desquijeran": "desquijerar",
    "desquijerando": "desquijerar",
    "desquijerara": "desquijerar",
    "desquijerarais": "desquijerar",
    "desquijeraran": "desquijerar",
    "desquijeraras": "desquijerar",
    "desquijerare": "desquijerar",
    "desquijerareis": "desquijerar",
    "desquijeraremos": "desquijerar",
    "desquijeraren": "desquijerar",
    "desquijerares": "desquijerar",
    "desquijeraron": "desquijerar",
    "desquijerará": "desquijerar",
    "desquijerarán": "desquijerar",
    "desquijerarás": "desquijerar",
    "desquijeraré": "desquijerar",
    "desquijeraréis": "desquijerar",
    "desquijeraría": "desquijerar",
    "desquijeraríais": "desquijerar",
    "desquijeraríamos": "desquijerar",
    "desquijerarían": "desquijerar",
    "desquijerarías": "desquijerar",
    "desquijeras": "desquijerar",
    "desquijerase": "desquijerar",
    "desquijeraseis": "desquijerar",
    "desquijerasen": "desquijerar",
    "desquijerases": "desquijerar",
    "desquijeraste": "desquijerar",
    "desquijerasteis": "desquijerar",
    "desquijere": "desquijerar",
    "desquijeremos": "desquijerar",
    "desquijeren": "desquijerar",
    "desquijeres": "desquijerar",
    "desquijero": "desquijerar",
    "desquijerábamos": "desquijerar",
    "desquijeráis": "desquijerar",
    "desquijeráramos": "desquijerar",
    "desquijeráremos": "desquijerar",
    "desquijerásemos": "desquijerar",
    "desquijeré": "desquijerar",
    "desquijeréis": "desquijerar",
    "desquijeró": "desquijerar",
    "desquilata": "desquilatar",
    "desquilataba": "desquilatar",
    "desquilatabais": "desquilatar",
    "desquilataban": "desquilatar",
    "desquilatabas": "desquilatar",
    "desquilatad": "desquilatar",
    "desquilatada": "desquilatar",
    "desquilatadas": "desquilatar",
    "desquilatado": "desquilatar",
    "desquilatados": "desquilatar",
    "desquilatamos": "desquilatar",
    "desquilatan": "desquilatar",
    "desquilatando": "desquilatar",
    "desquilatara": "desquilatar",
    "desquilatarais": "desquilatar",
    "desquilataran": "desquilatar",
    "desquilataras": "desquilatar",
    "desquilatare": "desquilatar",
    "desquilatareis": "desquilatar",
    "desquilataremos": "desquilatar",
    "desquilataren": "desquilatar",
    "desquilatares": "desquilatar",
    "desquilataron": "desquilatar",
    "desquilatará": "desquilatar",
    "desquilatarán": "desquilatar",
    "desquilatarás": "desquilatar",
    "desquilataré": "desquilatar",
    "desquilataréis": "desquilatar",
    "desquilataría": "desquilatar",
    "desquilataríais": "desquilatar",
    "desquilataríamos": "desquilatar",
    "desquilatarían": "desquilatar",
    "desquilatarías": "desquilatar",
    "desquilatas": "desquilatar",
    "desquilatase": "desquilatar",
    "desquilataseis": "desquilatar",
    "desquilatasen": "desquilatar",
    "desquilatases": "desquilatar",
    "desquilataste": "desquilatar",
    "desquilatasteis": "desquilatar",
    "desquilate": "desquilatar",
    "desquilatemos": "desquilatar",
    "desquilaten": "desquilatar",
    "desquilates": "desquilatar",
    "desquilato": "desquilatar",
    "desquilatábamos": "desquilatar",
    "desquilatáis": "desquilatar",
    "desquilatáramos": "desquilatar",
    "desquilatáremos": "desquilatar",
    "desquilatásemos": "desquilatar",
    "desquilaté": "desquilatar",
    "desquilatéis": "desquilatar",
    "desquilató": "desquilatar",
    "desquise": "desquerer",
    "desquisiera": "desquerer",
    "desquisierais": "desquerer",
    "desquisieran": "desquerer",
    "desquisieras": "desquerer",
    "desquisiere": "desquerer",
    "desquisiereis": "desquerer",
    "desquisieren": "desquerer",
    "desquisieres": "desquerer",
    "desquisieron": "desquerer",
    "desquisiese": "desquerer",
    "desquisieseis": "desquerer",
    "desquisiesen": "desquerer",
    "desquisieses": "desquerer",
    "desquisimos": "desquerer",
    "desquisiste": "desquerer",
    "desquisisteis": "desquerer",
    "desquisiéramos": "desquerer",
    "desquisiéremos": "desquerer",
    "desquisiésemos": "desquerer",
    "desquiso": "desquerer",
    "desquita": "desquitar",
    "desquitaba": "desquitar",
    "desquitabais": "desquitar",
    "desquitaban": "desquitar",
    "desquitabas": "desquitar",
    "desquitad": "desquitar",
    "desquitada": "desquitar",
    "desquitadas": "desquitar",
    "desquitado": "desquitar",
    "desquitados": "desquitar",
    "desquitamos": "desquitar",
    "desquitan": "desquitar",
    "desquitando": "desquitar",
    "desquitara": "desquitar",
    "desquitarais": "desquitar",
    "desquitaran": "desquitar",
    "desquitaras": "desquitar",
    "desquitare": "desquitar",
    "desquitareis": "desquitar",
    "desquitaremos": "desquitar",
    "desquitaren": "desquitar",
    "desquitares": "desquitar",
    "desquitaron": "desquitar",
    "desquitará": "desquitar",
    "desquitarán": "desquitar",
    "desquitarás": "desquitar",
    "desquitaré": "desquitar",
    "desquitaréis": "desquitar",
    "desquitaría": "desquitar",
    "desquitaríais": "desquitar",
    "desquitaríamos": "desquitar",
    "desquitarían": "desquitar",
    "desquitarías": "desquitar",
    "desquitas": "desquitar",
    "desquitase": "desquitar",
    "desquitaseis": "desquitar",
    "desquitasen": "desquitar",
    "desquitases": "desquitar",
    "desquitaste": "desquitar",
    "desquitasteis": "desquitar",
    "desquite": "desquitar",
    "desquitemos": "desquitar",
    "desquiten": "desquitar",
    "desquites": "desquite",
    "desquito": "desquitar",
    "desquitábamos": "desquitar",
    "desquitáis": "desquitar",
    "desquitáramos": "desquitar",
    "desquitáremos": "desquitar",
    "desquitásemos": "desquitar",
    "desquité": "desquitar",
    "desquitéis": "desquitar",
    "desquitó": "desquitar",
    "desrama": "desramar",
    "desramaba": "desramar",
    "desramabais": "desramar",
    "desramaban": "desramar",
    "desramabas": "desramar",
    "desramad": "desramar",
    "desramada": "desramar",
    "desramadas": "desramar",
    "desramado": "desramar",
    "desramados": "desramar",
    "desramamos": "desramar",
    "desraman": "desramar",
    "desramando": "desramar",
    "desramara": "desramar",
    "desramarais": "desramar",
    "desramaran": "desramar",
    "desramaras": "desramar",
    "desramare": "desramar",
    "desramareis": "desramar",
    "desramaremos": "desramar",
    "desramaren": "desramar",
    "desramares": "desramar",
    "desramaron": "desramar",
    "desramará": "desramar",
    "desramarán": "desramar",
    "desramarás": "desramar",
    "desramaré": "desramar",
    "desramaréis": "desramar",
    "desramaría": "desramar",
    "desramaríais": "desramar",
    "desramaríamos": "desramar",
    "desramarían": "desramar",
    "desramarías": "desramar",
    "desramas": "desramar",
    "desramase": "desramar",
    "desramaseis": "desramar",
    "desramasen": "desramar",
    "desramases": "desramar",
    "desramaste": "desramar",
    "desramasteis": "desramar",
    "desrame": "desramar",
    "desramemos": "desramar",
    "desramen": "desramar",
    "desrames": "desramar",
    "desramo": "desramar",
    "desramábamos": "desramar",
    "desramáis": "desramar",
    "desramáramos": "desramar",
    "desramáremos": "desramar",
    "desramásemos": "desramar",
    "desramé": "desramar",
    "desraméis": "desramar",
    "desramó": "desramar",
    "desratice": "desratizar",
    "desraticemos": "desratizar",
    "desraticen": "desratizar",
    "desratices": "desratizar",
    "desraticé": "desratizar",
    "desraticéis": "desratizar",
    "desratiza": "desratizar",
    "desratizaba": "desratizar",
    "desratizabais": "desratizar",
    "desratizaban": "desratizar",
    "desratizabas": "desratizar",
    "desratizaciones": "desratización",
    "desratizad": "desratizar",
    "desratizada": "desratizar",
    "desratizadas": "desratizar",
    "desratizado": "desratizar",
    "desratizados": "desratizar",
    "desratizamientos": "desratizamiento",
    "desratizamos": "desratizar",
    "desratizan": "desratizar",
    "desratizando": "desratizar",
    "desratizara": "desratizar",
    "desratizarais": "desratizar",
    "desratizaran": "desratizar",
    "desratizaras": "desratizar",
    "desratizare": "desratizar",
    "desratizareis": "desratizar",
    "desratizaremos": "desratizar",
    "desratizaren": "desratizar",
    "desratizares": "desratizar",
    "desratizaron": "desratizar",
    "desratizará": "desratizar",
    "desratizarán": "desratizar",
    "desratizarás": "desratizar",
    "desratizaré": "desratizar",
    "desratizaréis": "desratizar",
    "desratizaría": "desratizar",
    "desratizaríais": "desratizar",
    "desratizaríamos": "desratizar",
    "desratizarían": "desratizar",
    "desratizarías": "desratizar",
    "desratizas": "desratizar",
    "desratizase": "desratizar",
    "desratizaseis": "desratizar",
    "desratizasen": "desratizar",
    "desratizases": "desratizar",
    "desratizaste": "desratizar",
    "desratizasteis": "desratizar",
    "desratizo": "desratizar",
    "desratizábamos": "desratizar",
    "desratizáis": "desratizar",
    "desratizáramos": "desratizar",
    "desratizáremos": "desratizar",
    "desratizásemos": "desratizar",
    "desratizó": "desratizar",
    "desrazonables": "desrazonable",
    "desregula": "desregular",
    "desregulaba": "desregular",
    "desregulabais": "desregular",
    "desregulaban": "desregular",
    "desregulabas": "desregular",
    "desregulaciones": "desregulación",
    "desregulad": "desregular",
    "desregulada": "desregular",
    "desreguladas": "desregular",
    "desregulado": "desregular",
    "desregulados": "desregular",
    "desregulamos": "desregular",
    "desregulan": "desregular",
    "desregulando": "desregular",
    "desregulara": "desregular",
    "desregularais": "desregular",
    "desregularan": "desregular",
    "desregularas": "desregular",
    "desregulare": "desregular",
    "desregulareis": "desregular",
    "desregularemos": "desregular",
    "desregularen": "desregular",
    "desregulares": "desregular",
    "desregularon": "desregular",
    "desregulará": "desregular",
    "desregularán": "desregular",
    "desregularás": "desregular",
    "desregularé": "desregular",
    "desregularéis": "desregular",
    "desregularía": "desregular",
    "desregularíais": "desregular",
    "desregularíamos": "desregular",
    "desregularían": "desregular",
    "desregularías": "desregular",
    "desregulas": "desregular",
    "desregulase": "desregular",
    "desregulaseis": "desregular",
    "desregulasen": "desregular",
    "desregulases": "desregular",
    "desregulaste": "desregular",
    "desregulasteis": "desregular",
    "desregule": "desregular",
    "desregulemos": "desregular",
    "desregulen": "desregular",
    "desregules": "desregular",
    "desregulo": "desregular",
    "desregulábamos": "desregular",
    "desreguláis": "desregular",
    "desreguláramos": "desregular",
    "desreguláremos": "desregular",
    "desregulásemos": "desregular",
    "desregulé": "desregular",
    "desreguléis": "desregular",
    "desreguló": "desregular",
    "desrice": "desrizar",
    "desricemos": "desrizar",
    "desricen": "desrizar",
    "desrices": "desrizar",
    "desricé": "desrizar",
    "desricéis": "desrizar",
    "desriela": "desrielar",
    "desrielaba": "desrielar",
    "desrielabais": "desrielar",
    "desrielaban": "desrielar",
    "desrielabas": "desrielar",
    "desrielad": "desrielar",
    "desrielada": "desrielar",
    "desrieladas": "desrielar",
    "desrielado": "desrielar",
    "desrielados": "desrielar",
    "desrielamos": "desrielar",
    "desrielan": "desrielar",
    "desrielando": "desrielar",
    "desrielara": "desrielar",
    "desrielarais": "desrielar",
    "desrielaran": "desrielar",
    "desrielaras": "desrielar",
    "desrielare": "desrielar",
    "desrielareis": "desrielar",
    "desrielaremos": "desrielar",
    "desrielaren": "desrielar",
    "desrielares": "desrielar",
    "desrielaron": "desrielar",
    "desrielará": "desrielar",
    "desrielarán": "desrielar",
    "desrielarás": "desrielar",
    "desrielaré": "desrielar",
    "desrielaréis": "desrielar",
    "desrielaría": "desrielar",
    "desrielaríais": "desrielar",
    "desrielaríamos": "desrielar",
    "desrielarían": "desrielar",
    "desrielarías": "desrielar",
    "desrielas": "desrielar",
    "desrielase": "desrielar",
    "desrielaseis": "desrielar",
    "desrielasen": "desrielar",
    "desrielases": "desrielar",
    "desrielaste": "desrielar",
    "desrielasteis": "desrielar",
    "desriele": "desrielar",
    "desrielemos": "desrielar",
    "desrielen": "desrielar",
    "desrieles": "desrielar",
    "desrielo": "desrielar",
    "desrielábamos": "desrielar",
    "desrieláis": "desrielar",
    "desrieláramos": "desrielar",
    "desrieláremos": "desrielar",
    "desrielásemos": "desrielar",
    "desrielé": "desrielar",
    "desrieléis": "desrielar",
    "desrieló": "desrielar",
    "desrisca": "desriscar",
    "desriscaba": "desriscar",
    "desriscabais": "desriscar",
    "desriscaban": "desriscar",
    "desriscabas": "desriscar",
    "desriscad": "desriscar",
    "desriscada": "desriscar",
    "desriscadas": "desriscar",
    "desriscado": "desriscar",
    "desriscados": "desriscar",
    "desriscamos": "desriscar",
    "desriscan": "desriscar",
    "desriscando": "desriscar",
    "desriscara": "desriscar",
    "desriscarais": "desriscar",
    "desriscaran": "desriscar",
    "desriscaras": "desriscar",
    "desriscare": "desriscar",
    "desriscareis": "desriscar",
    "desriscaremos": "desriscar",
    "desriscaren": "desriscar",
    "desriscares": "desriscar",
    "desriscaron": "desriscar",
    "desriscará": "desriscar",
    "desriscarán": "desriscar",
    "desriscarás": "desriscar",
    "desriscaré": "desriscar",
    "desriscaréis": "desriscar",
    "desriscaría": "desriscar",
    "desriscaríais": "desriscar",
    "desriscaríamos": "desriscar",
    "desriscarían": "desriscar",
    "desriscarías": "desriscar",
    "desriscas": "desriscar",
    "desriscase": "desriscar",
    "desriscaseis": "desriscar",
    "desriscasen": "desriscar",
    "desriscases": "desriscar",
    "desriscaste": "desriscar",
    "desriscasteis": "desriscar",
    "desrisco": "desriscar",
    "desriscábamos": "desriscar",
    "desriscáis": "desriscar",
    "desriscáramos": "desriscar",
    "desriscáremos": "desriscar",
    "desriscásemos": "desriscar",
    "desriscó": "desriscar",
    "desrisque": "desriscar",
    "desrisquemos": "desriscar",
    "desrisquen": "desriscar",
    "desrisques": "desriscar",
    "desrisqué": "desriscar",
    "desrisquéis": "desriscar",
    "desriza": "desrizar",
    "desrizaba": "desrizar",
    "desrizabais": "desrizar",
    "desrizaban": "desrizar",
    "desrizabas": "desrizar",
    "desrizad": "desrizar",
    "desrizada": "desrizar",
    "desrizadas": "desrizar",
    "desrizado": "desrizar",
    "desrizados": "desrizar",
    "desrizamos": "desrizar",
    "desrizan": "desrizar",
    "desrizando": "desrizar",
    "desrizara": "desrizar",
    "desrizarais": "desrizar",
    "desrizaran": "desrizar",
    "desrizaras": "desrizar",
    "desrizare": "desrizar",
    "desrizareis": "desrizar",
    "desrizaremos": "desrizar",
    "desrizaren": "desrizar",
    "desrizares": "desrizar",
    "desrizaron": "desrizar",
    "desrizará": "desrizar",
    "desrizarán": "desrizar",
    "desrizarás": "desrizar",
    "desrizaré": "desrizar",
    "desrizaréis": "desrizar",
    "desrizaría": "desrizar",
    "desrizaríais": "desrizar",
    "desrizaríamos": "desrizar",
    "desrizarían": "desrizar",
    "desrizarías": "desrizar",
    "desrizas": "desrizar",
    "desrizase": "desrizar",
    "desrizaseis": "desrizar",
    "desrizasen": "desrizar",
    "desrizases": "desrizar",
    "desrizaste": "desrizar",
    "desrizasteis": "desrizar",
    "desrizo": "desrizar",
    "desrizábamos": "desrizar",
    "desrizáis": "desrizar",
    "desrizáramos": "desrizar",
    "desrizáremos": "desrizar",
    "desrizásemos": "desrizar",
    "desrizó": "desrizar",
    "desriñona": "desriñonar",
    "desriñonaba": "desriñonar",
    "desriñonabais": "desriñonar",
    "desriñonaban": "desriñonar",
    "desriñonabas": "desriñonar",
    "desriñonad": "desriñonar",
    "desriñonada": "desriñonar",
    "desriñonadas": "desriñonar",
    "desriñonado": "desriñonar",
    "desriñonados": "desriñonar",
    "desriñonamos": "desriñonar",
    "desriñonan": "desriñonar",
    "desriñonando": "desriñonar",
    "desriñonara": "desriñonar",
    "desriñonarais": "desriñonar",
    "desriñonaran": "desriñonar",
    "desriñonaras": "desriñonar",
    "desriñonare": "desriñonar",
    "desriñonareis": "desriñonar",
    "desriñonaremos": "desriñonar",
    "desriñonaren": "desriñonar",
    "desriñonares": "desriñonar",
    "desriñonaron": "desriñonar",
    "desriñonará": "desriñonar",
    "desriñonarán": "desriñonar",
    "desriñonarás": "desriñonar",
    "desriñonaré": "desriñonar",
    "desriñonaréis": "desriñonar",
    "desriñonaría": "desriñonar",
    "desriñonaríais": "desriñonar",
    "desriñonaríamos": "desriñonar",
    "desriñonarían": "desriñonar",
    "desriñonarías": "desriñonar",
    "desriñonas": "desriñonar",
    "desriñonase": "desriñonar",
    "desriñonaseis": "desriñonar",
    "desriñonasen": "desriñonar",
    "desriñonases": "desriñonar",
    "desriñonaste": "desriñonar",
    "desriñonasteis": "desriñonar",
    "desriñone": "desriñonar",
    "desriñonemos": "desriñonar",
    "desriñonen": "desriñonar",
    "desriñones": "desriñonar",
    "desriñono": "desriñonar",
    "desriñonábamos": "desriñonar",
    "desriñonáis": "desriñonar",
    "desriñonáramos": "desriñonar",
    "desriñonáremos": "desriñonar",
    "desriñonásemos": "desriñonar",
    "desriñoné": "desriñonar",
    "desriñonéis": "desriñonar",
    "desriñonó": "desriñonar",
    "desrobla": "desroblar",
    "desroblaba": "desroblar",
    "desroblabais": "desroblar",
    "desroblaban": "desroblar",
    "desroblabas": "desroblar",
    "desroblad": "desroblar",
    "desroblada": "desroblar",
    "desrobladas": "desroblar",
    "desroblado": "desroblar",
    "desroblados": "desroblar",
    "desroblamos": "desroblar",
    "desroblan": "desroblar",
    "desroblando": "desroblar",
    "desroblara": "desroblar",
    "desroblarais": "desroblar",
    "desroblaran": "desroblar",
    "desroblaras": "desroblar",
    "desroblare": "desroblar",
    "desroblareis": "desroblar",
    "desroblaremos": "desroblar",
    "desroblaren": "desroblar",
    "desroblares": "desroblar",
    "desroblaron": "desroblar",
    "desroblará": "desroblar",
    "desroblarán": "desroblar",
    "desroblarás": "desroblar",
    "desroblaré": "desroblar",
    "desroblaréis": "desroblar",
    "desroblaría": "desroblar",
    "desroblaríais": "desroblar",
    "desroblaríamos": "desroblar",
    "desroblarían": "desroblar",
    "desroblarías": "desroblar",
    "desroblas": "desroblar",
    "desroblase": "desroblar",
    "desroblaseis": "desroblar",
    "desroblasen": "desroblar",
    "desroblases": "desroblar",
    "desroblaste": "desroblar",
    "desroblasteis": "desroblar",
    "desroble": "desroblar",
    "desroblemos": "desroblar",
    "desroblen": "desroblar",
    "desrobles": "desroblar",
    "desroblo": "desroblar",
    "desroblábamos": "desroblar",
    "desrobláis": "desroblar",
    "desrobláramos": "desroblar",
    "desrobláremos": "desroblar",
    "desroblásemos": "desroblar",
    "desroblé": "desroblar",
    "desrobléis": "desroblar",
    "desrobló": "desroblar",
    "destaca": "destacar",
    "destacaba": "destacar",
    "destacabais": "destacar",
    "destacaban": "destacar",
    "destacabas": "destacar",
    "destacables": "destacable",
    "destacad": "destacar",
    "destacada": "destacar",
    "destacadas": "destacar",
    "destacado": "destacar",
    "destacados": "destacar",
    "destacamentos": "destacamento",
    "destacamos": "destacar",
    "destacan": "destacar",
    "destacando": "destacar",
    "destacara": "destacar",
    "destacarais": "destacar",
    "destacaran": "destacar",
    "destacaras": "destacar",
    "destacare": "destacar",
    "destacareis": "destacar",
    "destacaremos": "destacar",
    "destacaren": "destacar",
    "destacares": "destacar",
    "destacaron": "destacar",
    "destacará": "destacar",
    "destacarán": "destacar",
    "destacarás": "destacar",
    "destacaré": "destacar",
    "destacaréis": "destacar",
    "destacaría": "destacar",
    "destacaríais": "destacar",
    "destacaríamos": "destacar",
    "destacarían": "destacar",
    "destacarías": "destacar",
    "destacas": "destacar",
    "destacase": "destacar",
    "destacaseis": "destacar",
    "destacasen": "destacar",
    "destacases": "destacar",
    "destacaste": "destacar",
    "destacasteis": "destacar",
    "destace": "destazar",
    "destacemos": "destazar",
    "destacen": "destazar",
    "destaces": "destazar",
    "destaco": "destacar",
    "destacábamos": "destacar",
    "destacáis": "destacar",
    "destacáramos": "destacar",
    "destacáremos": "destacar",
    "destacásemos": "destacar",
    "destacé": "destazar",
    "destacéis": "destazar",
    "destacó": "destacar",
    "destaja": "destajar",
    "destajaba": "destajar",
    "destajabais": "destajar",
    "destajaban": "destajar",
    "destajabas": "destajar",
    "destajad": "destajar",
    "destajada": "destajar",
    "destajadas": "destajar",
    "destajado": "destajar",
    "destajadores": "destajador",
    "destajados": "destajar",
    "destajamos": "destajar",
    "destajan": "destajar",
    "destajando": "destajar",
    "destajara": "destajar",
    "destajarais": "destajar",
    "destajaran": "destajar",
    "destajaras": "destajar",
    "destajare": "destajar",
    "destajareis": "destajar",
    "destajaremos": "destajar",
    "destajaren": "destajar",
    "destajares": "destajar",
    "destajaron": "destajar",
    "destajará": "destajar",
    "destajarán": "destajar",
    "destajarás": "destajar",
    "destajaré": "destajar",
    "destajaréis": "destajar",
    "destajaría": "destajar",
    "destajaríais": "destajar",
    "destajaríamos": "destajar",
    "destajarían": "destajar",
    "destajarías": "destajar",
    "destajas": "destajar",
    "destajase": "destajar",
    "destajaseis": "destajar",
    "destajasen": "destajar",
    "destajases": "destajar",
    "destajaste": "destajar",
    "destajasteis": "destajar",
    "destaje": "destajar",
    "destajemos": "destajar",
    "destajen": "destajar",
    "destajera": "destajero",
    "destajeras": "destajero",
    "destajeros": "destajero",
    "destajes": "destajar",
    "destajistas": "destajista",
    "destajo": "destajar",
    "destajos": "destajo",
    "destajábamos": "destajar",
    "destajáis": "destajar",
    "destajáramos": "destajar",
    "destajáremos": "destajar",
    "destajásemos": "destajar",
    "destajé": "destajar",
    "destajéis": "destajar",
    "destajó": "destajar",
    "destalla": "destallar",
    "destallaba": "destallar",
    "destallabais": "destallar",
    "destallaban": "destallar",
    "destallabas": "destallar",
    "destallad": "destallar",
    "destallada": "destallar",
    "destalladas": "destallar",
    "destallado": "destallar",
    "destallados": "destallar",
    "destallamos": "destallar",
    "destallan": "destallar",
    "destallando": "destallar",
    "destallara": "destallar",
    "destallarais": "destallar",
    "destallaran": "destallar",
    "destallaras": "destallar",
    "destallare": "destallar",
    "destallareis": "destallar",
    "destallaremos": "destallar",
    "destallaren": "destallar",
    "destallares": "destallar",
    "destallaron": "destallar",
    "destallará": "destallar",
    "destallarán": "destallar",
    "destallarás": "destallar",
    "destallaré": "destallar",
    "destallaréis": "destallar",
    "destallaría": "destallar",
    "destallaríais": "destallar",
    "destallaríamos": "destallar",
    "destallarían": "destallar",
    "destallarías": "destallar",
    "destallas": "destallar",
    "destallase": "destallar",
    "destallaseis": "destallar",
    "destallasen": "destallar",
    "destallases": "destallar",
    "destallaste": "destallar",
    "destallasteis": "destallar",
    "destalle": "destallar",
    "destallemos": "destallar",
    "destallen": "destallar",
    "destalles": "destallar",
    "destallo": "destallar",
    "destallábamos": "destallar",
    "destalláis": "destallar",
    "destalláramos": "destallar",
    "destalláremos": "destallar",
    "destallásemos": "destallar",
    "destallé": "destallar",
    "destalléis": "destallar",
    "destalló": "destallar",
    "destalona": "destalonar",
    "destalonaba": "destalonar",
    "destalonabais": "destalonar",
    "destalonaban": "destalonar",
    "destalonabas": "destalonar",
    "destalonad": "destalonar",
    "destalonada": "destalonar",
    "destalonadas": "destalonar",
    "destalonado": "destalonar",
    "destalonados": "destalonar",
    "destalonamos": "destalonar",
    "destalonan": "destalonar",
    "destalonando": "destalonar",
    "destalonara": "destalonar",
    "destalonarais": "destalonar",
    "destalonaran": "destalonar",
    "destalonaras": "destalonar",
    "destalonare": "destalonar",
    "destalonareis": "destalonar",
    "destalonaremos": "destalonar",
    "destalonaren": "destalonar",
    "destalonares": "destalonar",
    "destalonaron": "destalonar",
    "destalonará": "destalonar",
    "destalonarán": "destalonar",
    "destalonarás": "destalonar",
    "destalonaré": "destalonar",
    "destalonaréis": "destalonar",
    "destalonaría": "destalonar",
    "destalonaríais": "destalonar",
    "destalonaríamos": "destalonar",
    "destalonarían": "destalonar",
    "destalonarías": "destalonar",
    "destalonas": "destalonar",
    "destalonase": "destalonar",
    "destalonaseis": "destalonar",
    "destalonasen": "destalonar",
    "destalonases": "destalonar",
    "destalonaste": "destalonar",
    "destalonasteis": "destalonar",
    "destalone": "destalonar",
    "destalonemos": "destalonar",
    "destalonen": "destalonar",
    "destalones": "destalonar",
    "destalono": "destalonar",
    "destalonábamos": "destalonar",
    "destalonáis": "destalonar",
    "destalonáramos": "destalonar",
    "destalonáremos": "destalonar",
    "destalonásemos": "destalonar",
    "destaloné": "destalonar",
    "destalonéis": "destalonar",
    "destalonó": "destalonar",
    "destapa": "destapar",
    "destapaba": "destapar",
    "destapabais": "destapar",
    "destapaban": "destapar",
    "destapabas": "destapar",
    "destapad": "destapar",
    "destapada": "destapar",
    "destapadas": "destapar",
    "destapado": "destapar",
    "destapados": "destapar",
    "destapaduras": "destapadura",
    "destapamos": "destapar",
    "destapan": "destapar",
    "destapando": "destapar",
    "destapara": "destapar",
    "destaparais": "destapar",
    "destaparan": "destapar",
    "destaparas": "destapar",
    "destapare": "destapar",
    "destapareis": "destapar",
    "destaparemos": "destapar",
    "destaparen": "destapar",
    "destapares": "destapar",
    "destaparon": "destapar",
    "destapará": "destapar",
    "destaparán": "destapar",
    "destaparás": "destapar",
    "destaparé": "destapar",
    "destaparéis": "destapar",
    "destaparía": "destapar",
    "destaparíais": "destapar",
    "destaparíamos": "destapar",
    "destaparían": "destapar",
    "destaparías": "destapar",
    "destapas": "destapar",
    "destapase": "destapar",
    "destapaseis": "destapar",
    "destapasen": "destapar",
    "destapases": "destapar",
    "destapaste": "destapar",
    "destapasteis": "destapar",
    "destape": "destapar",
    "destapemos": "destapar",
    "destapen": "destapar",
    "destapes": "destape",
    "destapia": "destapiar",
    "destapiaba": "destapiar",
    "destapiabais": "destapiar",
    "destapiaban": "destapiar",
    "destapiabas": "destapiar",
    "destapiad": "destapiar",
    "destapiada": "destapiar",
    "destapiadas": "destapiar",
    "destapiado": "destapiar",
    "destapiados": "destapiar",
    "destapiamos": "destapiar",
    "destapian": "destapiar",
    "destapiando": "destapiar",
    "destapiara": "destapiar",
    "destapiarais": "destapiar",
    "destapiaran": "destapiar",
    "destapiaras": "destapiar",
    "destapiare": "destapiar",
    "destapiareis": "destapiar",
    "destapiaremos": "destapiar",
    "destapiaren": "destapiar",
    "destapiares": "destapiar",
    "destapiaron": "destapiar",
    "destapiará": "destapiar",
    "destapiarán": "destapiar",
    "destapiarás": "destapiar",
    "destapiaré": "destapiar",
    "destapiaréis": "destapiar",
    "destapiaría": "destapiar",
    "destapiaríais": "destapiar",
    "destapiaríamos": "destapiar",
    "destapiarían": "destapiar",
    "destapiarías": "destapiar",
    "destapias": "destapiar",
    "destapiase": "destapiar",
    "destapiaseis": "destapiar",
    "destapiasen": "destapiar",
    "destapiases": "destapiar",
    "destapiaste": "destapiar",
    "destapiasteis": "destapiar",
    "destapie": "destapiar",
    "destapiemos": "destapiar",
    "destapien": "destapiar",
    "destapies": "destapiar",
    "destapio": "destapiar",
    "destapiábamos": "destapiar",
    "destapiáis": "destapiar",
    "destapiáramos": "destapiar",
    "destapiáremos": "destapiar",
    "destapiásemos": "destapiar",
    "destapié": "destapiar",
    "destapiéis": "destapiar",
    "destapió": "destapiar",
    "destapo": "destapar",
    "destapona": "destaponar",
    "destaponaba": "destaponar",
    "destaponabais": "destaponar",
    "destaponaban": "destaponar",
    "destaponabas": "destaponar",
    "destaponad": "destaponar",
    "destaponada": "destaponar",
    "destaponadas": "destaponar",
    "destaponado": "destaponar",
    "destaponados": "destaponar",
    "destaponamientos": "destaponamiento",
    "destaponamos": "destaponar",
    "destaponan": "destaponar",
    "destaponando": "destaponar",
    "destaponara": "destaponar",
    "destaponarais": "destaponar",
    "destaponaran": "destaponar",
    "destaponaras": "destaponar",
    "destaponare": "destaponar",
    "destaponareis": "destaponar",
    "destaponaremos": "destaponar",
    "destaponaren": "destaponar",
    "destaponares": "destaponar",
    "destaponaron": "destaponar",
    "destaponará": "destaponar",
    "destaponarán": "destaponar",
    "destaponarás": "destaponar",
    "destaponaré": "destaponar",
    "destaponaréis": "destaponar",
    "destaponaría": "destaponar",
    "destaponaríais": "destaponar",
    "destaponaríamos": "destaponar",
    "destaponarían": "destaponar",
    "destaponarías": "destaponar",
    "destaponas": "destaponar",
    "destaponase": "destaponar",
    "destaponaseis": "destaponar",
    "destaponasen": "destaponar",
    "destaponases": "destaponar",
    "destaponaste": "destaponar",
    "destaponasteis": "destaponar",
    "destapone": "destaponar",
    "destaponemos": "destaponar",
    "destaponen": "destaponar",
    "destapones": "destaponar",
    "destapono": "destaponar",
    "destaponábamos": "destaponar",
    "destaponáis": "destaponar",
    "destaponáramos": "destaponar",
    "destaponáremos": "destaponar",
    "destaponásemos": "destaponar",
    "destaponé": "destaponar",
    "destaponéis": "destaponar",
    "destaponó": "destaponar",
    "destapábamos": "destapar",
    "destapáis": "destapar",
    "destapáramos": "destapar",
    "destapáremos": "destapar",
    "destapásemos": "destapar",
    "destapé": "destapar",
    "destapéis": "destapar",
    "destapó": "destapar",
    "destaque": "destacar",
    "destaquemos": "destacar",
    "destaquen": "destacar",
    "destaques": "destacar",
    "destaqué": "destacar",
    "destaquéis": "destacar",
    "destara": "destarar",
    "destaraba": "destarar",
    "destarabais": "destarar",
    "destaraban": "destarar",
    "destarabas": "destarar",
    "destarad": "destarar",
    "destarada": "destarar",
    "destaradas": "destarar",
    "destarado": "destarar",
    "destarados": "destarar",
    "destaramos": "destarar",
    "destaran": "destarar",
    "destarando": "destarar",
    "destarara": "destarar",
    "destararais": "destarar",
    "destararan": "destarar",
    "destararas": "destarar",
    "destarare": "destarar",
    "destarareis": "destarar",
    "destararemos": "destarar",
    "destararen": "destarar",
    "destarares": "destarar",
    "destararon": "destarar",
    "destarará": "destarar",
    "destararán": "destarar",
    "destararás": "destarar",
    "destararé": "destarar",
    "destararéis": "destarar",
    "destararía": "destarar",
    "destararíais": "destarar",
    "destararíamos": "destarar",
    "destararían": "destarar",
    "destararías": "destarar",
    "destaras": "destarar",
    "destarase": "destarar",
    "destaraseis": "destarar",
    "destarasen": "destarar",
    "destarases": "destarar",
    "destaraste": "destarar",
    "destarasteis": "destarar",
    "destare": "destarar",
    "destaremos": "destarar",
    "destaren": "destarar",
    "destares": "destarar",
    "destaro": "destarar",
    "destartalada": "destartalado",
    "destartaladas": "destartalado",
    "destartalados": "destartalado",
    "destartalos": "destartalo",
    "destarábamos": "destarar",
    "destaráis": "destarar",
    "destaráramos": "destarar",
    "destaráremos": "destarar",
    "destarásemos": "destarar",
    "destaré": "destarar",
    "destaréis": "destarar",
    "destaró": "destarar",
    "destaza": "destazar",
    "destazaba": "destazar",
    "destazabais": "destazar",
    "destazaban": "destazar",
    "destazabas": "destazar",
    "destazad": "destazar",
    "destazada": "destazar",
    "destazadas": "destazar",
    "destazado": "destazar",
    "destazadores": "destazador",
    "destazados": "destazar",
    "destazamos": "destazar",
    "destazan": "destazar",
    "destazando": "destazar",
    "destazara": "destazar",
    "destazarais": "destazar",
    "destazaran": "destazar",
    "destazaras": "destazar",
    "destazare": "destazar",
    "destazareis": "destazar",
    "destazaremos": "destazar",
    "destazaren": "destazar",
    "destazares": "destazar",
    "destazaron": "destazar",
    "destazará": "destazar",
    "destazarán": "destazar",
    "destazarás": "destazar",
    "destazaré": "destazar",
    "destazaréis": "destazar",
    "destazaría": "destazar",
    "destazaríais": "destazar",
    "destazaríamos": "destazar",
    "destazarían": "destazar",
    "destazarías": "destazar",
    "destazas": "destazar",
    "destazase": "destazar",
    "destazaseis": "destazar",
    "destazasen": "destazar",
    "destazases": "destazar",
    "destazaste": "destazar",
    "destazasteis": "destazar",
    "destazo": "destazar",
    "destazábamos": "destazar",
    "destazáis": "destazar",
    "destazáramos": "destazar",
    "destazáremos": "destazar",
    "destazásemos": "destazar",
    "destazó": "destazar",
    "destecha": "destechar",
    "destechaba": "destechar",
    "destechabais": "destechar",
    "destechaban": "destechar",
    "destechabas": "destechar",
    "destechad": "destechar",
    "destechada": "destechar",
    "destechadas": "destechar",
    "destechado": "destechar",
    "destechados": "destechar",
    "destechaduras": "destechadura",
    "destechamos": "destechar",
    "destechan": "destechar",
    "destechando": "destechar",
    "destechara": "destechar",
    "destecharais": "destechar",
    "destecharan": "destechar",
    "destecharas": "destechar",
    "destechare": "destechar",
    "destechareis": "destechar",
    "destecharemos": "destechar",
    "destecharen": "destechar",
    "destechares": "destechar",
    "destecharon": "destechar",
    "destechará": "destechar",
    "destecharán": "destechar",
    "destecharás": "destechar",
    "destecharé": "destechar",
    "destecharéis": "destechar",
    "destecharía": "destechar",
    "destecharíais": "destechar",
    "destecharíamos": "destechar",
    "destecharían": "destechar",
    "destecharías": "destechar",
    "destechas": "destechar",
    "destechase": "destechar",
    "destechaseis": "destechar",
    "destechasen": "destechar",
    "destechases": "destechar",
    "destechaste": "destechar",
    "destechasteis": "destechar",
    "desteche": "destechar",
    "destechemos": "destechar",
    "destechen": "destechar",
    "desteches": "destechar",
    "destecho": "destechar",
    "destechábamos": "destechar",
    "destecháis": "destechar",
    "destecháramos": "destechar",
    "destecháremos": "destechar",
    "destechásemos": "destechar",
    "desteché": "destechar",
    "destechéis": "destechar",
    "destechó": "destechar",
    "desteja": "destejer",
    "destejaba": "destejar",
    "destejabais": "destejar",
    "destejaban": "destejar",
    "destejabas": "destejar",
    "destejad": "destejar",
    "destejada": "destejar",
    "destejadas": "destejar",
    "destejado": "destejar",
    "destejados": "destejar",
    "destejamos": "destejer",
    "destejan": "destejer",
    "destejando": "destejar",
    "destejara": "destejar",
    "destejarais": "destejar",
    "destejaran": "destejar",
    "destejaras": "destejar",
    "destejare": "destejar",
    "destejareis": "destejar",
    "destejaremos": "destejar",
    "destejaren": "destejar",
    "destejares": "destejar",
    "destejaron": "destejar",
    "destejará": "destejar",
    "destejarán": "destejar",
    "destejarás": "destejar",
    "destejaré": "destejar",
    "destejaréis": "destejar",
    "destejaría": "destejar",
    "destejaríais": "destejar",
    "destejaríamos": "destejar",
    "destejarían": "destejar",
    "destejarías": "destejar",
    "destejas": "destejer",
    "destejase": "destejar",
    "destejaseis": "destejar",
    "destejasen": "destejar",
    "destejases": "destejar",
    "destejaste": "destejar",
    "destejasteis": "destejar",
    "desteje": "destejer",
    "destejed": "destejer",
    "destejemos": "destejer",
    "destejen": "destejer",
    "destejeremos": "destejer",
    "destejerá": "destejer",
    "destejerán": "destejer",
    "destejerás": "destejer",
    "destejeré": "destejer",
    "destejeréis": "destejer",
    "destejería": "destejer",
    "destejeríais": "destejer",
    "destejeríamos": "destejer",
    "destejerían": "destejer",
    "destejerías": "destejer",
    "destejes": "destejer",
    "destejida": "destejer",
    "destejidas": "destejer",
    "destejido": "destejer",
    "destejidos": "destejer",
    "destejiendo": "destejer",
    "destejiera": "destejer",
    "destejierais": "destejer",
    "destejieran": "destejer",
    "destejieras": "destejer",
    "destejiere": "destejer",
    "destejiereis": "destejer",
    "destejieren": "destejer",
    "destejieres": "destejer",
    "destejieron": "destejer",
    "destejiese": "destejer",
    "destejieseis": "destejer",
    "destejiesen": "destejer",
    "destejieses": "destejer",
    "destejimos": "destejer",
    "destejiste": "destejer",
    "destejisteis": "destejer",
    "destejiéramos": "destejer",
    "destejiéremos": "destejer",
    "destejiésemos": "destejer",
    "destejió": "destejer",
    "destejo": "destejer",
    "destejábamos": "destejar",
    "destejáis": "destejer",
    "destejáramos": "destejar",
    "destejáremos": "destejar",
    "destejásemos": "destejar",
    "destejé": "destejar",
    "destejéis": "destejer",
    "destejí": "destejer",
    "destejía": "destejer",
    "destejíais": "destejer",
    "destejíamos": "destejer",
    "destejían": "destejer",
    "destejías": "destejer",
    "destejó": "destejar",
    "destella": "destellar",
    "destellaba": "destellar",
    "destellabais": "destellar",
    "destellaban": "destellar",
    "destellabas": "destellar",
    "destellad": "destellar",
    "destellada": "destellar",
    "destelladas": "destellar",
    "destellado": "destellar",
    "destellados": "destellar",
    "destellamos": "destellar",
    "destellan": "destellar",
    "destellando": "destellar",
    "destellantes": "destellante",
    "destellara": "destellar",
    "destellarais": "destellar",
    "destellaran": "destellar",
    "destellaras": "destellar",
    "destellare": "destellar",
    "destellareis": "destellar",
    "destellaremos": "destellar",
    "destellaren": "destellar",
    "destellares": "destellar",
    "destellaron": "destellar",
    "destellará": "destellar",
    "destellarán": "destellar",
    "destellarás": "destellar",
    "destellaré": "destellar",
    "destellaréis": "destellar",
    "destellaría": "destellar",
    "destellaríais": "destellar",
    "destellaríamos": "destellar",
    "destellarían": "destellar",
    "destellarías": "destellar",
    "destellas": "destellar",
    "destellase": "destellar",
    "destellaseis": "destellar",
    "destellasen": "destellar",
    "destellases": "destellar",
    "destellaste": "destellar",
    "destellasteis": "destellar",
    "destelle": "destellar",
    "destelleantes": "destelleante",
    "destellemos": "destellar",
    "destellen": "destellar",
    "destelles": "destellar",
    "destello": "destellar",
    "destellos": "destello",
    "destellábamos": "destellar",
    "destelláis": "destellar",
    "destelláramos": "destellar",
    "destelláremos": "destellar",
    "destellásemos": "destellar",
    "destellé": "destellar",
    "destelléis": "destellar",
    "destelló": "destellar",
    "destempla": "destemplar",
    "destemplaba": "destemplar",
    "destemplabais": "destemplar",
    "destemplaban": "destemplar",
    "destemplabas": "destemplar",
    "destemplad": "destemplar",
    "destemplada": "destemplar",
    "destempladas": "destemplar",
    "destemplado": "destemplar",
    "destempladora": "destemplador",
    "destempladoras": "destemplador",
    "destempladores": "destemplador",
    "destemplados": "destemplar",
    "destemplamos": "destemplar",
    "destemplan": "destemplar",
    "destemplando": "destemplar",
    "destemplanzas": "destemplanza",
    "destemplara": "destemplar",
    "destemplarais": "destemplar",
    "destemplaran": "destemplar",
    "destemplaras": "destemplar",
    "destemplare": "destemplar",
    "destemplareis": "destemplar",
    "destemplaremos": "destemplar",
    "destemplaren": "destemplar",
    "destemplares": "destemplar",
    "destemplaron": "destemplar",
    "destemplará": "destemplar",
    "destemplarán": "destemplar",
    "destemplarás": "destemplar",
    "destemplaré": "destemplar",
    "destemplaréis": "destemplar",
    "destemplaría": "destemplar",
    "destemplaríais": "destemplar",
    "destemplaríamos": "destemplar",
    "destemplarían": "destemplar",
    "destemplarías": "destemplar",
    "destemplas": "destemplar",
    "destemplase": "destemplar",
    "destemplaseis": "destemplar",
    "destemplasen": "destemplar",
    "destemplases": "destemplar",
    "destemplaste": "destemplar",
    "destemplasteis": "destemplar",
    "destemple": "destemplar",
    "destemplemos": "destemplar",
    "destemplen": "destemplar",
    "destemples": "destemple",
    "destemplo": "destemplar",
    "destemplábamos": "destemplar",
    "destempláis": "destemplar",
    "destempláramos": "destemplar",
    "destempláremos": "destemplar",
    "destemplásemos": "destemplar",
    "destemplé": "destemplar",
    "destempléis": "destemplar",
    "destempló": "destemplar",
    "destensa": "destensar",
    "destensaba": "destensar",
    "destensabais": "destensar",
    "destensaban": "destensar",
    "destensabas": "destensar",
    "destensad": "destensar",
    "destensada": "destensar",
    "destensadas": "destensar",
    "destensado": "destensar",
    "destensados": "destensar",
    "destensamos": "destensar",
    "destensan": "destensar",
    "destensando": "destensar",
    "destensara": "destensar",
    "destensarais": "destensar",
    "destensaran": "destensar",
    "destensaras": "destensar",
    "destensare": "destensar",
    "destensareis": "destensar",
    "destensaremos": "destensar",
    "destensaren": "destensar",
    "destensares": "destensar",
    "destensaron": "destensar",
    "destensará": "destensar",
    "destensarán": "destensar",
    "destensarás": "destensar",
    "destensaré": "destensar",
    "destensaréis": "destensar",
    "destensaría": "destensar",
    "destensaríais": "destensar",
    "destensaríamos": "destensar",
    "destensarían": "destensar",
    "destensarías": "destensar",
    "destensas": "destensar",
    "destensase": "destensar",
    "destensaseis": "destensar",
    "destensasen": "destensar",
    "destensases": "destensar",
    "destensaste": "destensar",
    "destensasteis": "destensar",
    "destense": "destensar",
    "destensemos": "destensar",
    "destensen": "destensar",
    "destenses": "destensar",
    "destenso": "destensar",
    "destensábamos": "destensar",
    "destensáis": "destensar",
    "destensáramos": "destensar",
    "destensáremos": "destensar",
    "destensásemos": "destensar",
    "destensé": "destensar",
    "destenséis": "destensar",
    "destensó": "destensar",
    "destentaba": "destentar",
    "destentabais": "destentar",
    "destentaban": "destentar",
    "destentabas": "destentar",
    "destentad": "destentar",
    "destentada": "destentar",
    "destentadas": "destentar",
    "destentado": "destentar",
    "destentados": "destentar",
    "destentamos": "destentar",
    "destentando": "destentar",
    "destentara": "destentar",
    "destentarais": "destentar",
    "destentaran": "destentar",
    "destentaras": "destentar",
    "destentare": "destentar",
    "destentareis": "destentar",
    "destentaremos": "destentar",
    "destentaren": "destentar",
    "destentares": "destentar",
    "destentaron": "destentar",
    "destentará": "destentar",
    "destentarán": "destentar",
    "destentarás": "destentar",
    "destentaré": "destentar",
    "destentaréis": "destentar",
    "destentaría": "destentar",
    "destentaríais": "destentar",
    "destentaríamos": "destentar",
    "destentarían": "destentar",
    "destentarías": "destentar",
    "destentase": "destentar",
    "destentaseis": "destentar",
    "destentasen": "destentar",
    "destentases": "destentar",
    "destentaste": "destentar",
    "destentasteis": "destentar",
    "destentemos": "destentar",
    "destentábamos": "destentar",
    "destentáis": "destentar",
    "destentáramos": "destentar",
    "destentáremos": "destentar",
    "destentásemos": "destentar",
    "destenté": "destentar",
    "destentéis": "destentar",
    "destentó": "destentar",
    "desternera": "desternerar",
    "desterneraba": "desternerar",
    "desternerabais": "desternerar",
    "desterneraban": "desternerar",
    "desternerabas": "desternerar",
    "desternerad": "desternerar",
    "desternerada": "desternerar",
    "desterneradas": "desternerar",
    "desternerado": "desternerar",
    "desternerados": "desternerar",
    "desterneramos": "desternerar",
    "desterneran": "desternerar",
    "desternerando": "desternerar",
    "desternerara": "desternerar",
    "desternerarais": "desternerar",
    "desterneraran": "desternerar",
    "desterneraras": "desternerar",
    "desternerare": "desternerar",
    "desternerareis": "desternerar",
    "desterneraremos": "desternerar",
    "desterneraren": "desternerar",
    "desternerares": "desternerar",
    "desterneraron": "desternerar",
    "desternerará": "desternerar",
    "desternerarán": "desternerar",
    "desternerarás": "desternerar",
    "desterneraré": "desternerar",
    "desterneraréis": "desternerar",
    "desterneraría": "desternerar",
    "desterneraríais": "desternerar",
    "desterneraríamos": "desternerar",
    "desternerarían": "desternerar",
    "desternerarías": "desternerar",
    "desterneras": "desternerar",
    "desternerase": "desternerar",
    "desterneraseis": "desternerar",
    "desternerasen": "desternerar",
    "desternerases": "desternerar",
    "desterneraste": "desternerar",
    "desternerasteis": "desternerar",
    "desternere": "desternerar",
    "desterneremos": "desternerar",
    "desterneren": "desternerar",
    "desterneres": "desternerar",
    "desternero": "desternerar",
    "desternerábamos": "desternerar",
    "desterneráis": "desternerar",
    "desterneráramos": "desternerar",
    "desterneráremos": "desternerar",
    "desternerásemos": "desternerar",
    "desterneré": "desternerar",
    "desterneréis": "desternerar",
    "desterneró": "desternerar",
    "desternilla": "desternillar",
    "desternillaba": "desternillar",
    "desternillabais": "desternillar",
    "desternillaban": "desternillar",
    "desternillabas": "desternillar",
    "desternillad": "desternillar",
    "desternillada": "desternillar",
    "desternilladas": "desternillar",
    "desternillado": "desternillar",
    "desternillados": "desternillar",
    "desternillamos": "desternillar",
    "desternillan": "desternillar",
    "desternillando": "desternillar",
    "desternillara": "desternillar",
    "desternillarais": "desternillar",
    "desternillaran": "desternillar",
    "desternillaras": "desternillar",
    "desternillare": "desternillar",
    "desternillareis": "desternillar",
    "desternillaremos": "desternillar",
    "desternillaren": "desternillar",
    "desternillares": "desternillar",
    "desternillaron": "desternillar",
    "desternillará": "desternillar",
    "desternillarán": "desternillar",
    "desternillarás": "desternillar",
    "desternillaré": "desternillar",
    "desternillaréis": "desternillar",
    "desternillaría": "desternillar",
    "desternillaríais": "desternillar",
    "desternillaríamos": "desternillar",
    "desternillarían": "desternillar",
    "desternillarías": "desternillar",
    "desternillas": "desternillar",
    "desternillase": "desternillar",
    "desternillaseis": "desternillar",
    "desternillasen": "desternillar",
    "desternillases": "desternillar",
    "desternillaste": "desternillar",
    "desternillasteis": "desternillar",
    "desternille": "desternillar",
    "desternillemos": "desternillar",
    "desternillen": "desternillar",
    "desternilles": "desternillar",
    "desternillo": "desternillar",
    "desternillábamos": "desternillar",
    "desternilláis": "desternillar",
    "desternilláramos": "desternillar",
    "desternilláremos": "desternillar",
    "desternillásemos": "desternillar",
    "desternillé": "desternillar",
    "desternilléis": "desternillar",
    "desternilló": "desternillar",
    "desterraba": "desterrar",
    "desterrabais": "desterrar",
    "desterraban": "desterrar",
    "desterrabas": "desterrar",
    "desterrad": "desterrar",
    "desterrada": "desterrar",
    "desterradas": "desterrar",
    "desterrado": "desterrar",
    "desterrados": "desterrar",
    "desterramos": "desterrar",
    "desterrando": "desterrar",
    "desterrara": "desterrar",
    "desterrarais": "desterrar",
    "desterraran": "desterrar",
    "desterraras": "desterrar",
    "desterrare": "desterrar",
    "desterrareis": "desterrar",
    "desterraremos": "desterrar",
    "desterraren": "desterrar",
    "desterrares": "desterrar",
    "desterraron": "desterrar",
    "desterrará": "desterrar",
    "desterrarán": "desterrar",
    "desterrarás": "desterrar",
    "desterraré": "desterrar",
    "desterraréis": "desterrar",
    "desterraría": "desterrar",
    "desterraríais": "desterrar",
    "desterraríamos": "desterrar",
    "desterrarían": "desterrar",
    "desterrarías": "desterrar",
    "desterrase": "desterrar",
    "desterraseis": "desterrar",
    "desterrasen": "desterrar",
    "desterrases": "desterrar",
    "desterraste": "desterrar",
    "desterrasteis": "desterrar",
    "desterremos": "desterrar",
    "desterrona": "desterronar",
    "desterronaba": "desterronar",
    "desterronabais": "desterronar",
    "desterronaban": "desterronar",
    "desterronabas": "desterronar",
    "desterronad": "desterronar",
    "desterronada": "desterronar",
    "desterronadas": "desterronar",
    "desterronado": "desterronar",
    "desterronadora": "desterronador",
    "desterronadoras": "desterronador",
    "desterronadores": "desterronador",
    "desterronados": "desterronar",
    "desterronamientos": "desterronamiento",
    "desterronamos": "desterronar",
    "desterronan": "desterronar",
    "desterronando": "desterronar",
    "desterronara": "desterronar",
    "desterronarais": "desterronar",
    "desterronaran": "desterronar",
    "desterronaras": "desterronar",
    "desterronare": "desterronar",
    "desterronareis": "desterronar",
    "desterronaremos": "desterronar",
    "desterronaren": "desterronar",
    "desterronares": "desterronar",
    "desterronaron": "desterronar",
    "desterronará": "desterronar",
    "desterronarán": "desterronar",
    "desterronarás": "desterronar",
    "desterronaré": "desterronar",
    "desterronaréis": "desterronar",
    "desterronaría": "desterronar",
    "desterronaríais": "desterronar",
    "desterronaríamos": "desterronar",
    "desterronarían": "desterronar",
    "desterronarías": "desterronar",
    "desterronas": "desterronar",
    "desterronase": "desterronar",
    "desterronaseis": "desterronar",
    "desterronasen": "desterronar",
    "desterronases": "desterronar",
    "desterronaste": "desterronar",
    "desterronasteis": "desterronar",
    "desterrone": "desterronar",
    "desterronemos": "desterronar",
    "desterronen": "desterronar",
    "desterrones": "desterronar",
    "desterrono": "desterronar",
    "desterronábamos": "desterronar",
    "desterronáis": "desterronar",
    "desterronáramos": "desterronar",
    "desterronáremos": "desterronar",
    "desterronásemos": "desterronar",
    "desterroné": "desterronar",
    "desterronéis": "desterronar",
    "desterronó": "desterronar",
    "desterrábamos": "desterrar",
    "desterráis": "desterrar",
    "desterráramos": "desterrar",
    "desterráremos": "desterrar",
    "desterrásemos": "desterrar",
    "desterré": "desterrar",
    "desterréis": "desterrar",
    "desterró": "desterrar",
    "desteta": "destetar",
    "destetaba": "destetar",
    "destetabais": "destetar",
    "destetaban": "destetar",
    "destetabas": "destetar",
    "destetad": "destetar",
    "destetada": "destetar",
    "destetadas": "destetar",
    "destetaderas": "destetadera",
    "destetado": "destetar",
    "destetados": "destetar",
    "destetamientos": "destetamiento",
    "destetamos": "destetar",
    "destetan": "destetar",
    "destetando": "destetar",
    "destetara": "destetar",
    "destetarais": "destetar",
    "destetaran": "destetar",
    "destetaras": "destetar",
    "destetare": "destetar",
    "destetareis": "destetar",
    "destetaremos": "destetar",
    "destetaren": "destetar",
    "destetares": "destetar",
    "destetaron": "destetar",
    "destetará": "destetar",
    "destetarán": "destetar",
    "destetarás": "destetar",
    "destetaré": "destetar",
    "destetaréis": "destetar",
    "destetaría": "destetar",
    "destetaríais": "destetar",
    "destetaríamos": "destetar",
    "destetarían": "destetar",
    "destetarías": "destetar",
    "destetas": "destetar",
    "destetase": "destetar",
    "destetaseis": "destetar",
    "destetasen": "destetar",
    "destetases": "destetar",
    "destetaste": "destetar",
    "destetasteis": "destetar",
    "destete": "destetar",
    "destetemos": "destetar",
    "desteten": "destetar",
    "destetes": "destete",
    "destetilla": "destetillar",
    "destetillaba": "destetillar",
    "destetillabais": "destetillar",
    "destetillaban": "destetillar",
    "destetillabas": "destetillar",
    "destetillad": "destetillar",
    "destetillada": "destetillar",
    "destetilladas": "destetillar",
    "destetillado": "destetillar",
    "destetillados": "destetillar",
    "destetillamos": "destetillar",
    "destetillan": "destetillar",
    "destetillando": "destetillar",
    "destetillara": "destetillar",
    "destetillarais": "destetillar",
    "destetillaran": "destetillar",
    "destetillaras": "destetillar",
    "destetillare": "destetillar",
    "destetillareis": "destetillar",
    "destetillaremos": "destetillar",
    "destetillaren": "destetillar",
    "destetillares": "destetillar",
    "destetillaron": "destetillar",
    "destetillará": "destetillar",
    "destetillarán": "destetillar",
    "destetillarás": "destetillar",
    "destetillaré": "destetillar",
    "destetillaréis": "destetillar",
    "destetillaría": "destetillar",
    "destetillaríais": "destetillar",
    "destetillaríamos": "destetillar",
    "destetillarían": "destetillar",
    "destetillarías": "destetillar",
    "destetillas": "destetillar",
    "destetillase": "destetillar",
    "destetillaseis": "destetillar",
    "destetillasen": "destetillar",
    "destetillases": "destetillar",
    "destetillaste": "destetillar",
    "destetillasteis": "destetillar",
    "destetille": "destetillar",
    "destetillemos": "destetillar",
    "destetillen": "destetillar",
    "destetilles": "destetillar",
    "destetillo": "destetillar",
    "destetillábamos": "destetillar",
    "destetilláis": "destetillar",
    "destetilláramos": "destetillar",
    "destetilláremos": "destetillar",
    "destetillásemos": "destetillar",
    "destetillé": "destetillar",
    "destetilléis": "destetillar",
    "destetilló": "destetillar",
    "desteto": "destetar",
    "destetos": "desteto",
    "destetábamos": "destetar",
    "destetáis": "destetar",
    "destetáramos": "destetar",
    "destetáremos": "destetar",
    "destetásemos": "destetar",
    "desteté": "destetar",
    "destetéis": "destetar",
    "destetó": "destetar",
    "desteñid": "desteñir",
    "desteñida": "desteñir",
    "desteñidas": "desteñir",
    "desteñido": "desteñir",
    "desteñidos": "desteñir",
    "desteñimos": "desteñir",
    "desteñiremos": "desteñir",
    "desteñirá": "desteñir",
    "desteñirán": "desteñir",
    "desteñirás": "desteñir",
    "desteñiré": "desteñir",
    "desteñiréis": "desteñir",
    "desteñiría": "desteñir",
    "desteñiríais": "desteñir",
    "desteñiríamos": "desteñir",
    "desteñirían": "desteñir",
    "desteñirías": "desteñir",
    "desteñiste": "desteñir",
    "desteñisteis": "desteñir",
    "desteñí": "desteñir",
    "desteñía": "desteñir",
    "desteñíais": "desteñir",
    "desteñíamos": "desteñir",
    "desteñían": "desteñir",
    "desteñías": "desteñir",
    "desteñís": "desteñir",
    "destienta": "destentar",
    "destientan": "destentar",
    "destientas": "destentar",
    "destiente": "destentar",
    "destienten": "destentar",
    "destientes": "destentar",
    "destiento": "destentar",
    "destientos": "destiento",
    "destierra": "desterrar",
    "destierran": "desterrar",
    "destierras": "desterrar",
    "destierre": "desterrar",
    "destierren": "desterrar",
    "destierres": "destierre",
    "destierro": "desterrar",
    "destierros": "destierro",
    "destila": "destilar",
    "destilaba": "destilar",
    "destilabais": "destilar",
    "destilaban": "destilar",
    "destilabas": "destilar",
    "destilables": "destilable",
    "destilaciones": "destilación",
    "destilad": "destilar",
    "destilada": "destilar",
    "destiladas": "destilar",
    "destiladeras": "destiladera",
    "destilado": "destilar",
    "destiladora": "destilador",
    "destiladoras": "destilador",
    "destiladores": "destilador",
    "destilados": "destilar",
    "destilamos": "destilar",
    "destilan": "destilar",
    "destilando": "destilar",
    "destilara": "destilar",
    "destilarais": "destilar",
    "destilaran": "destilar",
    "destilaras": "destilar",
    "destilare": "destilar",
    "destilareis": "destilar",
    "destilaremos": "destilar",
    "destilaren": "destilar",
    "destilares": "destilar",
    "destilaron": "destilar",
    "destilará": "destilar",
    "destilarán": "destilar",
    "destilarás": "destilar",
    "destilaré": "destilar",
    "destilaréis": "destilar",
    "destilaría": "destilar",
    "destilaríais": "destilar",
    "destilaríamos": "destilar",
    "destilarían": "destilar",
    "destilarías": "destilar",
    "destilas": "destilar",
    "destilase": "destilar",
    "destilaseis": "destilar",
    "destilasen": "destilar",
    "destilases": "destilar",
    "destilaste": "destilar",
    "destilasteis": "destilar",
    "destilatoria": "destilatorio",
    "destilatorias": "destilatorio",
    "destilatorios": "destilatorio",
    "destile": "destilar",
    "destilemos": "destilar",
    "destilen": "destilar",
    "destilerías": "destilería",
    "destiles": "destilar",
    "destilo": "destilar",
    "destilábamos": "destilar",
    "destiláis": "destilar",
    "destiláramos": "destilar",
    "destiláremos": "destilar",
    "destilásemos": "destilar",
    "destilé": "destilar",
    "destiléis": "destilar",
    "destiló": "destilar",
    "destina": "destinar",
    "destinaba": "destinar",
    "destinabais": "destinar",
    "destinaban": "destinar",
    "destinabas": "destinar",
    "destinaciones": "destinación",
    "destinad": "destinar",
    "destinada": "destinar",
    "destinadas": "destinar",
    "destinado": "destinar",
    "destinados": "destinar",
    "destinamos": "destinar",
    "destinan": "destinar",
    "destinando": "destinar",
    "destinara": "destinar",
    "destinarais": "destinar",
    "destinaran": "destinar",
    "destinaras": "destinar",
    "destinare": "destinar",
    "destinareis": "destinar",
    "destinaremos": "destinar",
    "destinaren": "destinar",
    "destinares": "destinar",
    "destinaron": "destinar",
    "destinará": "destinar",
    "destinarán": "destinar",
    "destinarás": "destinar",
    "destinaré": "destinar",
    "destinaréis": "destinar",
    "destinaría": "destinar",
    "destinaríais": "destinar",
    "destinaríamos": "destinar",
    "destinarían": "destinar",
    "destinarías": "destinar",
    "destinas": "destinar",
    "destinase": "destinar",
    "destinaseis": "destinar",
    "destinasen": "destinar",
    "destinases": "destinar",
    "destinaste": "destinar",
    "destinasteis": "destinar",
    "destinataria": "destinatario",
    "destinatarias": "destinatario",
    "destinatarios": "destinatario",
    "destine": "destinar",
    "destinemos": "destinar",
    "destinen": "destinar",
    "destines": "destinar",
    "destino": "destinar",
    "destinos": "destino",
    "destinábamos": "destinar",
    "destináis": "destinar",
    "destináramos": "destinar",
    "destináremos": "destinar",
    "destinásemos": "destinar",
    "destiné": "destinar",
    "destinéis": "destinar",
    "destinó": "destinar",
    "destiranizada": "destiranizado",
    "destiranizadas": "destiranizado",
    "destiranizados": "destiranizado",
    "destituciones": "destitución",
    "destituibles": "destituible",
    "destituid": "destituir",
    "destituida": "destituir",
    "destituidas": "destituir",
    "destituido": "destituir",
    "destituidora": "destituidor",
    "destituidoras": "destituidor",
    "destituidores": "destituidor",
    "destituidos": "destituir",
    "destituimos": "destituir",
    "destituiremos": "destituir",
    "destituirá": "destituir",
    "destituirán": "destituir",
    "destituirás": "destituir",
    "destituiré": "destituir",
    "destituiréis": "destituir",
    "destituiría": "destituir",
    "destituiríais": "destituir",
    "destituiríamos": "destituir",
    "destituirían": "destituir",
    "destituirías": "destituir",
    "destituiste": "destituir",
    "destituisteis": "destituir",
    "destitulada": "destitulado",
    "destituladas": "destitulado",
    "destitulados": "destitulado",
    "destituya": "destituir",
    "destituyamos": "destituir",
    "destituyan": "destituir",
    "destituyas": "destituir",
    "destituye": "destituir",
    "destituyen": "destituir",
    "destituyendo": "destituir",
    "destituyera": "destituir",
    "destituyerais": "destituir",
    "destituyeran": "destituir",
    "destituyeras": "destituir",
    "destituyere": "destituir",
    "destituyereis": "destituir",
    "destituyeren": "destituir",
    "destituyeres": "destituir",
    "destituyeron": "destituir",
    "destituyes": "destituir",
    "destituyese": "destituir",
    "destituyeseis": "destituir",
    "destituyesen": "destituir",
    "destituyeses": "destituir",
    "destituyo": "destituir",
    "destituyáis": "destituir",
    "destituyéramos": "destituir",
    "destituyéremos": "destituir",
    "destituyésemos": "destituir",
    "destituyó": "destituir",
    "destituí": "destituir",
    "destituía": "destituir",
    "destituíais": "destituir",
    "destituíamos": "destituir",
    "destituían": "destituir",
    "destituías": "destituir",
    "destituís": "destituir",
    "destiña": "desteñir",
    "destiñamos": "desteñir",
    "destiñan": "desteñir",
    "destiñas": "desteñir",
    "destiñe": "desteñir",
    "destiñen": "desteñir",
    "destiñendo": "desteñir",
    "destiñera": "desteñir",
    "destiñerais": "desteñir",
    "destiñeran": "desteñir",
    "destiñeras": "desteñir",
    "destiñere": "desteñir",
    "destiñereis": "desteñir",
    "destiñeren": "desteñir",
    "destiñeres": "desteñir",
    "destiñeron": "desteñir",
    "destiñes": "desteñir",
    "destiñese": "desteñir",
    "destiñeseis": "desteñir",
    "destiñesen": "desteñir",
    "destiñeses": "desteñir",
    "destiño": "desteñir",
    "destiñáis": "desteñir",
    "destiñéramos": "desteñir",
    "destiñéremos": "desteñir",
    "destiñésemos": "desteñir",
    "destiñó": "desteñir",
    "destoca": "destocar",
    "destocaba": "destocar",
    "destocabais": "destocar",
    "destocaban": "destocar",
    "destocabas": "destocar",
    "destocad": "destocar",
    "destocada": "destocar",
    "destocadas": "destocar",
    "destocado": "destocar",
    "destocados": "destocar",
    "destocamos": "destocar",
    "destocan": "destocar",
    "destocando": "destocar",
    "destocara": "destocar",
    "destocarais": "destocar",
    "destocaran": "destocar",
    "destocaras": "destocar",
    "destocare": "destocar",
    "destocareis": "destocar",
    "destocaremos": "destocar",
    "destocaren": "destocar",
    "destocares": "destocar",
    "destocaron": "destocar",
    "destocará": "destocar",
    "destocarán": "destocar",
    "destocarás": "destocar",
    "destocaré": "destocar",
    "destocaréis": "destocar",
    "destocaría": "destocar",
    "destocaríais": "destocar",
    "destocaríamos": "destocar",
    "destocarían": "destocar",
    "destocarías": "destocar",
    "destocas": "destocar",
    "destocase": "destocar",
    "destocaseis": "destocar",
    "destocasen": "destocar",
    "destocases": "destocar",
    "destocaste": "destocar",
    "destocasteis": "destocar",
    "destoco": "destocar",
    "destocábamos": "destocar",
    "destocáis": "destocar",
    "destocáramos": "destocar",
    "destocáremos": "destocar",
    "destocásemos": "destocar",
    "destocó": "destocar",
    "destopes": "destope",
    "destoque": "destocar",
    "destoquemos": "destocar",
    "destoquen": "destocar",
    "destoques": "destocar",
    "destoqué": "destocar",
    "destoquéis": "destocar",
    "destorced": "destorcer",
    "destorceduras": "destorcedura",
    "destorcemos": "destorcer",
    "destorceremos": "destorcer",
    "destorcerá": "destorcer",
    "destorcerán": "destorcer",
    "destorcerás": "destorcer",
    "destorceré": "destorcer",
    "destorceréis": "destorcer",
    "destorcería": "destorcer",
    "destorceríais": "destorcer",
    "destorceríamos": "destorcer",
    "destorcerían": "destorcer",
    "destorcerías": "destorcer",
    "destorcida": "destorcer",
    "destorcidas": "destorcer",
    "destorcido": "destorcer",
    "destorcidos": "destorcer",
    "destorciendo": "destorcer",
    "destorciera": "destorcer",
    "destorcierais": "destorcer",
    "destorcieran": "destorcer",
    "destorcieras": "destorcer",
    "destorciere": "destorcer",
    "destorciereis": "destorcer",
    "destorcieren": "destorcer",
    "destorcieres": "destorcer",
    "destorcieron": "destorcer",
    "destorciese": "destorcer",
    "destorcieseis": "destorcer",
    "destorciesen": "destorcer",
    "destorcieses": "destorcer",
    "destorcimos": "destorcer",
    "destorciste": "destorcer",
    "destorcisteis": "destorcer",
    "destorciéramos": "destorcer",
    "destorciéremos": "destorcer",
    "destorciésemos": "destorcer",
    "destorció": "destorcer",
    "destorcéis": "destorcer",
    "destorcí": "destorcer",
    "destorcía": "destorcer",
    "destorcíais": "destorcer",
    "destorcíamos": "destorcer",
    "destorcían": "destorcer",
    "destorcías": "destorcer",
    "destornilla": "destornillar",
    "destornillaba": "destornillar",
    "destornillabais": "destornillar",
    "destornillaban": "destornillar",
    "destornillabas": "destornillar",
    "destornillad": "destornillar",
    "destornillada": "destornillar",
    "destornilladas": "destornillar",
    "destornillado": "destornillar",
    "destornilladora": "destornillador",
    "destornilladoras": "destornillador",
    "destornilladores": "destornillador",
    "destornillados": "destornillar",
    "destornillamientos": "destornillamiento",
    "destornillamos": "destornillar",
    "destornillan": "destornillar",
    "destornillando": "destornillar",
    "destornillara": "destornillar",
    "destornillarais": "destornillar",
    "destornillaran": "destornillar",
    "destornillaras": "destornillar",
    "destornillare": "destornillar",
    "destornillareis": "destornillar",
    "destornillaremos": "destornillar",
    "destornillaren": "destornillar",
    "destornillares": "destornillar",
    "destornillaron": "destornillar",
    "destornillará": "destornillar",
    "destornillarán": "destornillar",
    "destornillarás": "destornillar",
    "destornillaré": "destornillar",
    "destornillaréis": "destornillar",
    "destornillaría": "destornillar",
    "destornillaríais": "destornillar",
    "destornillaríamos": "destornillar",
    "destornillarían": "destornillar",
    "destornillarías": "destornillar",
    "destornillas": "destornillar",
    "destornillase": "destornillar",
    "destornillaseis": "destornillar",
    "destornillasen": "destornillar",
    "destornillases": "destornillar",
    "destornillaste": "destornillar",
    "destornillasteis": "destornillar",
    "destornille": "destornillar",
    "destornillemos": "destornillar",
    "destornillen": "destornillar",
    "destornilles": "destornillar",
    "destornillo": "destornillar",
    "destornillábamos": "destornillar",
    "destornilláis": "destornillar",
    "destornilláramos": "destornillar",
    "destornilláremos": "destornillar",
    "destornillásemos": "destornillar",
    "destornillé": "destornillar",
    "destornilléis": "destornillar",
    "destornilló": "destornillar",
    "destorzamos": "destorcer",
    "destorzáis": "destorcer",
    "destoxicaciones": "destoxicación",
    "destoxificaciones": "destoxificación",
    "destraba": "destrabar",
    "destrababa": "destrabar",
    "destrababais": "destrabar",
    "destrababan": "destrabar",
    "destrababas": "destrabar",
    "destrabad": "destrabar",
    "destrabada": "destrabar",
    "destrabadas": "destrabar",
    "destrabado": "destrabar",
    "destrabados": "destrabar",
    "destrabamos": "destrabar",
    "destraban": "destrabar",
    "destrabando": "destrabar",
    "destrabara": "destrabar",
    "destrabarais": "destrabar",
    "destrabaran": "destrabar",
    "destrabaras": "destrabar",
    "destrabare": "destrabar",
    "destrabareis": "destrabar",
    "destrabaremos": "destrabar",
    "destrabaren": "destrabar",
    "destrabares": "destrabar",
    "destrabaron": "destrabar",
    "destrabará": "destrabar",
    "destrabarán": "destrabar",
    "destrabarás": "destrabar",
    "destrabaré": "destrabar",
    "destrabaréis": "destrabar",
    "destrabaría": "destrabar",
    "destrabaríais": "destrabar",
    "destrabaríamos": "destrabar",
    "destrabarían": "destrabar",
    "destrabarías": "destrabar",
    "destrabas": "destrabar",
    "destrabase": "destrabar",
    "destrabaseis": "destrabar",
    "destrabasen": "destrabar",
    "destrabases": "destrabar",
    "destrabaste": "destrabar",
    "destrabasteis": "destrabar",
    "destrabazones": "destrabazón",
    "destrabe": "destrabar",
    "destrabemos": "destrabar",
    "destraben": "destrabar",
    "destrabes": "destrabar",
    "destrabo": "destrabar",
    "destrabábamos": "destrabar",
    "destrabáis": "destrabar",
    "destrabáramos": "destrabar",
    "destrabáremos": "destrabar",
    "destrabásemos": "destrabar",
    "destrabé": "destrabar",
    "destrabéis": "destrabar",
    "destrabó": "destrabar",
    "destrales": "destral",
    "destrama": "destramar",
    "destramaba": "destramar",
    "destramabais": "destramar",
    "destramaban": "destramar",
    "destramabas": "destramar",
    "destramad": "destramar",
    "destramada": "destramar",
    "destramadas": "destramar",
    "destramado": "destramar",
    "destramados": "destramar",
    "destramamos": "destramar",
    "destraman": "destramar",
    "destramando": "destramar",
    "destramara": "destramar",
    "destramarais": "destramar",
    "destramaran": "destramar",
    "destramaras": "destramar",
    "destramare": "destramar",
    "destramareis": "destramar",
    "destramaremos": "destramar",
    "destramaren": "destramar",
    "destramares": "destramar",
    "destramaron": "destramar",
    "destramará": "destramar",
    "destramarán": "destramar",
    "destramarás": "destramar",
    "destramaré": "destramar",
    "destramaréis": "destramar",
    "destramaría": "destramar",
    "destramaríais": "destramar",
    "destramaríamos": "destramar",
    "destramarían": "destramar",
    "destramarías": "destramar",
    "destramas": "destramar",
    "destramase": "destramar",
    "destramaseis": "destramar",
    "destramasen": "destramar",
    "destramases": "destramar",
    "destramaste": "destramar",
    "destramasteis": "destramar",
    "destrame": "destramar",
    "destramemos": "destramar",
    "destramen": "destramar",
    "destrames": "destramar",
    "destramo": "destramar",
    "destramábamos": "destramar",
    "destramáis": "destramar",
    "destramáramos": "destramar",
    "destramáremos": "destramar",
    "destramásemos": "destramar",
    "destramé": "destramar",
    "destraméis": "destramar",
    "destramó": "destramar",
    "destrence": "destrenzar",
    "destrencemos": "destrenzar",
    "destrencen": "destrenzar",
    "destrences": "destrenzar",
    "destrencé": "destrenzar",
    "destrencéis": "destrenzar",
    "destrenza": "destrenzar",
    "destrenzaba": "destrenzar",
    "destrenzabais": "destrenzar",
    "destrenzaban": "destrenzar",
    "destrenzabas": "destrenzar",
    "destrenzad": "destrenzar",
    "destrenzada": "destrenzar",
    "destrenzadas": "destrenzar",
    "destrenzado": "destrenzar",
    "destrenzados": "destrenzar",
    "destrenzamos": "destrenzar",
    "destrenzan": "destrenzar",
    "destrenzando": "destrenzar",
    "destrenzara": "destrenzar",
    "destrenzarais": "destrenzar",
    "destrenzaran": "destrenzar",
    "destrenzaras": "destrenzar",
    "destrenzare": "destrenzar",
    "destrenzareis": "destrenzar",
    "destrenzaremos": "destrenzar",
    "destrenzaren": "destrenzar",
    "destrenzares": "destrenzar",
    "destrenzaron": "destrenzar",
    "destrenzará": "destrenzar",
    "destrenzarán": "destrenzar",
    "destrenzarás": "destrenzar",
    "destrenzaré": "destrenzar",
    "destrenzaréis": "destrenzar",
    "destrenzaría": "destrenzar",
    "destrenzaríais": "destrenzar",
    "destrenzaríamos": "destrenzar",
    "destrenzarían": "destrenzar",
    "destrenzarías": "destrenzar",
    "destrenzas": "destrenzar",
    "destrenzase": "destrenzar",
    "destrenzaseis": "destrenzar",
    "destrenzasen": "destrenzar",
    "destrenzases": "destrenzar",
    "destrenzaste": "destrenzar",
    "destrenzasteis": "destrenzar",
    "destrenzo": "destrenzar",
    "destrenzábamos": "destrenzar",
    "destrenzáis": "destrenzar",
    "destrenzáramos": "destrenzar",
    "destrenzáremos": "destrenzar",
    "destrenzásemos": "destrenzar",
    "destrenzó": "destrenzar",
    "destrezas": "destreza",
    "destribalizaciones": "destribalización",
    "destrice": "destrizar",
    "destricemos": "destrizar",
    "destricen": "destrizar",
    "destrices": "destrizar",
    "destricé": "destrizar",
    "destricéis": "destrizar",
    "destripa": "destripar",
    "destripaba": "destripar",
    "destripabais": "destripar",
    "destripaban": "destripar",
    "destripabas": "destripar",
    "destripad": "destripar",
    "destripada": "destripar",
    "destripadas": "destripar",
    "destripado": "destripar",
    "destripadora": "destripador",
    "destripadoras": "destripador",
    "destripadores": "destripador",
    "destripados": "destripar",
    "destripamientos": "destripamiento",
    "destripamos": "destripar",
    "destripan": "destripar",
    "destripando": "destripar",
    "destripara": "destripar",
    "destriparais": "destripar",
    "destriparan": "destripar",
    "destriparas": "destripar",
    "destripare": "destripar",
    "destripareis": "destripar",
    "destriparemos": "destripar",
    "destriparen": "destripar",
    "destripares": "destripar",
    "destriparon": "destripar",
    "destripará": "destripar",
    "destriparán": "destripar",
    "destriparás": "destripar",
    "destriparé": "destripar",
    "destriparéis": "destripar",
    "destriparía": "destripar",
    "destriparíais": "destripar",
    "destriparíamos": "destripar",
    "destriparían": "destripar",
    "destriparías": "destripar",
    "destripas": "destripar",
    "destripase": "destripar",
    "destripaseis": "destripar",
    "destripasen": "destripar",
    "destripases": "destripar",
    "destripaste": "destripar",
    "destripasteis": "destripar",
    "destripe": "destripar",
    "destripemos": "destripar",
    "destripen": "destripar",
    "destripes": "destripar",
    "destripo": "destripar",
    "destripábamos": "destripar",
    "destripáis": "destripar",
    "destripáramos": "destripar",
    "destripáremos": "destripar",
    "destripásemos": "destripar",
    "destripé": "destripar",
    "destripéis": "destripar",
    "destripó": "destripar",
    "destriunfa": "destriunfar",
    "destriunfaba": "destriunfar",
    "destriunfabais": "destriunfar",
    "destriunfaban": "destriunfar",
    "destriunfabas": "destriunfar",
    "destriunfad": "destriunfar",
    "destriunfada": "destriunfar",
    "destriunfadas": "destriunfar",
    "destriunfado": "destriunfar",
    "destriunfados": "destriunfar",
    "destriunfamos": "destriunfar",
    "destriunfan": "destriunfar",
    "destriunfando": "destriunfar",
    "destriunfara": "destriunfar",
    "destriunfarais": "destriunfar",
    "destriunfaran": "destriunfar",
    "destriunfaras": "destriunfar",
    "destriunfare": "destriunfar",
    "destriunfareis": "destriunfar",
    "destriunfaremos": "destriunfar",
    "destriunfaren": "destriunfar",
    "destriunfares": "destriunfar",
    "destriunfaron": "destriunfar",
    "destriunfará": "destriunfar",
    "destriunfarán": "destriunfar",
    "destriunfarás": "destriunfar",
    "destriunfaré": "destriunfar",
    "destriunfaréis": "destriunfar",
    "destriunfaría": "destriunfar",
    "destriunfaríais": "destriunfar",
    "destriunfaríamos": "destriunfar",
    "destriunfarían": "destriunfar",
    "destriunfarías": "destriunfar",
    "destriunfas": "destriunfar",
    "destriunfase": "destriunfar",
    "destriunfaseis": "destriunfar",
    "destriunfasen": "destriunfar",
    "destriunfases": "destriunfar",
    "destriunfaste": "destriunfar",
    "destriunfasteis": "destriunfar",
    "destriunfe": "destriunfar",
    "destriunfemos": "destriunfar",
    "destriunfen": "destriunfar",
    "destriunfes": "destriunfar",
    "destriunfo": "destriunfar",
    "destriunfábamos": "destriunfar",
    "destriunfáis": "destriunfar",
    "destriunfáramos": "destriunfar",
    "destriunfáremos": "destriunfar",
    "destriunfásemos": "destriunfar",
    "destriunfé": "destriunfar",
    "destriunféis": "destriunfar",
    "destriunfó": "destriunfar",
    "destriza": "destrizar",
    "destrizaba": "destrizar",
    "destrizabais": "destrizar",
    "destrizaban": "destrizar",
    "destrizabas": "destrizar",
    "destrizad": "destrizar",
    "destrizada": "destrizar",
    "destrizadas": "destrizar",
    "destrizado": "destrizar",
    "destrizados": "destrizar",
    "destrizamos": "destrizar",
    "destrizan": "destrizar",
    "destrizando": "destrizar",
    "destrizara": "destrizar",
    "destrizarais": "destrizar",
    "destrizaran": "destrizar",
    "destrizaras": "destrizar",
    "destrizare": "destrizar",
    "destrizareis": "destrizar",
    "destrizaremos": "destrizar",
    "destrizaren": "destrizar",
    "destrizares": "destrizar",
    "destrizaron": "destrizar",
    "destrizará": "destrizar",
    "destrizarán": "destrizar",
    "destrizarás": "destrizar",
    "destrizaré": "destrizar",
    "destrizaréis": "destrizar",
    "destrizaría": "destrizar",
    "destrizaríais": "destrizar",
    "destrizaríamos": "destrizar",
    "destrizarían": "destrizar",
    "destrizarías": "destrizar",
    "destrizas": "destrizar",
    "destrizase": "destrizar",
    "destrizaseis": "destrizar",
    "destrizasen": "destrizar",
    "destrizases": "destrizar",
    "destrizaste": "destrizar",
    "destrizasteis": "destrizar",
    "destrizo": "destrizar",
    "destrizábamos": "destrizar",
    "destrizáis": "destrizar",
    "destrizáramos": "destrizar",
    "destrizáremos": "destrizar",
    "destrizásemos": "destrizar",
    "destrizó": "destrizar",
    "destrocaba": "destrocar",
    "destrocabais": "destrocar",
    "destrocaban": "destrocar",
    "destrocabas": "destrocar",
    "destrocad": "destrocar",
    "destrocada": "destrocar",
    "destrocadas": "destrocar",
    "destrocado": "destrocar",
    "destrocados": "destrocar",
    "destrocamos": "destrocar",
    "destrocando": "destrocar",
    "destrocara": "destrocar",
    "destrocarais": "destrocar",
    "destrocaran": "destrocar",
    "destrocaras": "destrocar",
    "destrocare": "destrocar",
    "destrocareis": "destrocar",
    "destrocaremos": "destrocar",
    "destrocaren": "destrocar",
    "destrocares": "destrocar",
    "destrocaron": "destrocar",
    "destrocará": "destrocar",
    "destrocarán": "destrocar",
    "destrocarás": "destrocar",
    "destrocaré": "destrocar",
    "destrocaréis": "destrocar",
    "destrocaría": "destrocar",
    "destrocaríais": "destrocar",
    "destrocaríamos": "destrocar",
    "destrocarían": "destrocar",
    "destrocarías": "destrocar",
    "destrocase": "destrocar",
    "destrocaseis": "destrocar",
    "destrocasen": "destrocar",
    "destrocases": "destrocar",
    "destrocaste": "destrocar",
    "destrocasteis": "destrocar",
    "destroce": "destrozar",
    "destrocemos": "destrozar",
    "destrocen": "destrozar",
    "destroces": "destrozar",
    "destrocábamos": "destrocar",
    "destrocáis": "destrocar",
    "destrocáramos": "destrocar",
    "destrocáremos": "destrocar",
    "destrocásemos": "destrocar",
    "destrocé": "destrozar",
    "destrocéis": "destrozar",
    "destrocó": "destrocar",
    "destrona": "destronar",
    "destronaba": "destronar",
    "destronabais": "destronar",
    "destronaban": "destronar",
    "destronabas": "destronar",
    "destronad": "destronar",
    "destronada": "destronar",
    "destronadas": "destronar",
    "destronado": "destronar",
    "destronados": "destronar",
    "destronamientos": "destronamiento",
    "destronamos": "destronar",
    "destronan": "destronar",
    "destronando": "destronar",
    "destronara": "destronar",
    "destronarais": "destronar",
    "destronaran": "destronar",
    "destronaras": "destronar",
    "destronare": "destronar",
    "destronareis": "destronar",
    "destronaremos": "destronar",
    "destronaren": "destronar",
    "destronares": "destronar",
    "destronaron": "destronar",
    "destronará": "destronar",
    "destronarán": "destronar",
    "destronarás": "destronar",
    "destronaré": "destronar",
    "destronaréis": "destronar",
    "destronaría": "destronar",
    "destronaríais": "destronar",
    "destronaríamos": "destronar",
    "destronarían": "destronar",
    "destronarías": "destronar",
    "destronas": "destronar",
    "destronase": "destronar",
    "destronaseis": "destronar",
    "destronasen": "destronar",
    "destronases": "destronar",
    "destronaste": "destronar",
    "destronasteis": "destronar",
    "destronca": "destroncar",
    "destroncaba": "destroncar",
    "destroncabais": "destroncar",
    "destroncaban": "destroncar",
    "destroncabas": "destroncar",
    "destroncad": "destroncar",
    "destroncada": "destroncar",
    "destroncadas": "destroncar",
    "destroncado": "destroncar",
    "destroncados": "destroncar",
    "destroncamos": "destroncar",
    "destroncan": "destroncar",
    "destroncando": "destroncar",
    "destroncara": "destroncar",
    "destroncarais": "destroncar",
    "destroncaran": "destroncar",
    "destroncaras": "destroncar",
    "destroncare": "destroncar",
    "destroncareis": "destroncar",
    "destroncaremos": "destroncar",
    "destroncaren": "destroncar",
    "destroncares": "destroncar",
    "destroncaron": "destroncar",
    "destroncará": "destroncar",
    "destroncarán": "destroncar",
    "destroncarás": "destroncar",
    "destroncaré": "destroncar",
    "destroncaréis": "destroncar",
    "destroncaría": "destroncar",
    "destroncaríais": "destroncar",
    "destroncaríamos": "destroncar",
    "destroncarían": "destroncar",
    "destroncarías": "destroncar",
    "destroncas": "destroncar",
    "destroncase": "destroncar",
    "destroncaseis": "destroncar",
    "destroncasen": "destroncar",
    "destroncases": "destroncar",
    "destroncaste": "destroncar",
    "destroncasteis": "destroncar",
    "destronco": "destroncar",
    "destroncábamos": "destroncar",
    "destroncáis": "destroncar",
    "destroncáramos": "destroncar",
    "destroncáremos": "destroncar",
    "destroncásemos": "destroncar",
    "destroncó": "destroncar",
    "destrone": "destronar",
    "destronemos": "destronar",
    "destronen": "destronar",
    "destrones": "destronar",
    "destrono": "destronar",
    "destronque": "destroncar",
    "destronquemos": "destroncar",
    "destronquen": "destroncar",
    "destronques": "destroncar",
    "destronqué": "destroncar",
    "destronquéis": "destroncar",
    "destronábamos": "destronar",
    "destronáis": "destronar",
    "destronáramos": "destronar",
    "destronáremos": "destronar",
    "destronásemos": "destronar",
    "destroné": "destronar",
    "destronéis": "destronar",
    "destronó": "destronar",
    "destroza": "destrozar",
    "destrozaba": "destrozar",
    "destrozabais": "destrozar",
    "destrozaban": "destrozar",
    "destrozabas": "destrozar",
    "destrozad": "destrozar",
    "destrozada": "destrozar",
    "destrozadas": "destrozar",
    "destrozado": "destrozar",
    "destrozados": "destrozar",
    "destrozamos": "destrozar",
    "destrozan": "destrozar",
    "destrozando": "destrozar",
    "destrozara": "destrozar",
    "destrozarais": "destrozar",
    "destrozaran": "destrozar",
    "destrozaras": "destrozar",
    "destrozare": "destrozar",
    "destrozareis": "destrozar",
    "destrozaremos": "destrozar",
    "destrozaren": "destrozar",
    "destrozares": "destrozar",
    "destrozaron": "destrozar",
    "destrozará": "destrozar",
    "destrozarán": "destrozar",
    "destrozarás": "destrozar",
    "destrozaré": "destrozar",
    "destrozaréis": "destrozar",
    "destrozaría": "destrozar",
    "destrozaríais": "destrozar",
    "destrozaríamos": "destrozar",
    "destrozarían": "destrozar",
    "destrozarías": "destrozar",
    "destrozas": "destrozar",
    "destrozase": "destrozar",
    "destrozaseis": "destrozar",
    "destrozasen": "destrozar",
    "destrozases": "destrozar",
    "destrozaste": "destrozar",
    "destrozasteis": "destrozar",
    "destrozo": "destrozar",
    "destrozona": "destrozón",
    "destrozonas": "destrozón",
    "destrozones": "destrozón",
    "destrozos": "destrozo",
    "destrozábamos": "destrozar",
    "destrozáis": "destrozar",
    "destrozáramos": "destrozar",
    "destrozáremos": "destrozar",
    "destrozásemos": "destrozar",
    "destrozó": "destrozar",
    "destrucciones": "destrucción",
    "destructibilidades": "destructibilidad",
    "destructibles": "destructible",
    "destructiva": "destructivo",
    "destructivas": "destructivo",
    "destructivos": "destructivo",
    "destructora": "destructor",
    "destructoras": "destructor",
    "destructores": "destructor",
    "destructoria": "destructorio",
    "destructorias": "destructorio",
    "destructorios": "destructorio",
    "destrueca": "destrocar",
    "destruecan": "destrocar",
    "destruecas": "destrocar",
    "destrueco": "destrocar",
    "destruecos": "destrueco",
    "destrueque": "destrocar",
    "destruequen": "destrocar",
    "destrueques": "destrueque",
    "destruibles": "destruible",
    "destruid": "destruir",
    "destruida": "destruir",
    "destruidas": "destruir",
    "destruido": "destruir",
    "destruidora": "destruidor",
    "destruidoras": "destruidor",
    "destruidores": "destruidor",
    "destruidos": "destruir",
    "destruimos": "destruir",
    "destruiremos": "destruir",
    "destruirá": "destruir",
    "destruirán": "destruir",
    "destruirás": "destruir",
    "destruiré": "destruir",
    "destruiréis": "destruir",
    "destruiría": "destruir",
    "destruiríais": "destruir",
    "destruiríamos": "destruir",
    "destruirían": "destruir",
    "destruirías": "destruir",
    "destruiste": "destruir",
    "destruisteis": "destruir",
    "destruya": "destruir",
    "destruyamos": "destruir",
    "destruyan": "destruir",
    "destruyas": "destruir",
    "destruye": "destruir",
    "destruyen": "destruir",
    "destruyendo": "destruir",
    "destruyera": "destruir",
    "destruyerais": "destruir",
    "destruyeran": "destruir",
    "destruyeras": "destruir",
    "destruyere": "destruir",
    "destruyereis": "destruir",
    "destruyeren": "destruir",
    "destruyeres": "destruir",
    "destruyeron": "destruir",
    "destruyes": "destruir",
    "destruyese": "destruir",
    "destruyeseis": "destruir",
    "destruyesen": "destruir",
    "destruyeses": "destruir",
    "destruyo": "destruir",
    "destruyáis": "destruir",
    "destruyéramos": "destruir",
    "destruyéremos": "destruir",
    "destruyésemos": "destruir",
    "destruyó": "destruir",
    "destruí": "destruir",
    "destruía": "destruir",
    "destruíais": "destruir",
    "destruíamos": "destruir",
    "destruían": "destruir",
    "destruías": "destruir",
    "destruís": "destruir",
    "destróyers": "destróyer",
    "destuerce": "destorcer",
    "destuercen": "destorcer",
    "destuerces": "destorcer",
    "destuerza": "destorcer",
    "destuerzan": "destorcer",
    "destuerzas": "destorcer",
    "destuerzo": "destorcer",
    "destusa": "destusar",
    "destusaba": "destusar",
    "destusabais": "destusar",
    "destusaban": "destusar",
    "destusabas": "destusar",
    "destusad": "destusar",
    "destusada": "destusar",
    "destusadas": "destusar",
    "destusado": "destusar",
    "destusados": "destusar",
    "destusamos": "destusar",
    "destusan": "destusar",
    "destusando": "destusar",
    "destusara": "destusar",
    "destusarais": "destusar",
    "destusaran": "destusar",
    "destusaras": "destusar",
    "destusare": "destusar",
    "destusareis": "destusar",
    "destusaremos": "destusar",
    "destusaren": "destusar",
    "destusares": "destusar",
    "destusaron": "destusar",
    "destusará": "destusar",
    "destusarán": "destusar",
    "destusarás": "destusar",
    "destusaré": "destusar",
    "destusaréis": "destusar",
    "destusaría": "destusar",
    "destusaríais": "destusar",
    "destusaríamos": "destusar",
    "destusarían": "destusar",
    "destusarías": "destusar",
    "destusas": "destusar",
    "destusase": "destusar",
    "destusaseis": "destusar",
    "destusasen": "destusar",
    "destusases": "destusar",
    "destusaste": "destusar",
    "destusasteis": "destusar",
    "destuse": "destusar",
    "destusemos": "destusar",
    "destusen": "destusar",
    "destuses": "destusar",
    "destuso": "destusar",
    "destusábamos": "destusar",
    "destusáis": "destusar",
    "destusáramos": "destusar",
    "destusáremos": "destusar",
    "destusásemos": "destusar",
    "destusé": "destusar",
    "destuséis": "destusar",
    "destusó": "destusar",
    "desubstancia": "desubstanciar",
    "desubstanciaba": "desubstanciar",
    "desubstanciabais": "desubstanciar",
    "desubstanciaban": "desubstanciar",
    "desubstanciabas": "desubstanciar",
    "desubstanciad": "desubstanciar",
    "desubstanciada": "desubstanciar",
    "desubstanciadas": "desubstanciar",
    "desubstanciado": "desubstanciar",
    "desubstanciados": "desubstanciar",
    "desubstanciamientos": "desubstanciamiento",
    "desubstanciamos": "desubstanciar",
    "desubstancian": "desubstanciar",
    "desubstanciando": "desubstanciar",
    "desubstanciara": "desubstanciar",
    "desubstanciarais": "desubstanciar",
    "desubstanciaran": "desubstanciar",
    "desubstanciaras": "desubstanciar",
    "desubstanciare": "desubstanciar",
    "desubstanciareis": "desubstanciar",
    "desubstanciaremos": "desubstanciar",
    "desubstanciaren": "desubstanciar",
    "desubstanciares": "desubstanciar",
    "desubstanciaron": "desubstanciar",
    "desubstanciará": "desubstanciar",
    "desubstanciarán": "desubstanciar",
    "desubstanciarás": "desubstanciar",
    "desubstanciaré": "desubstanciar",
    "desubstanciaréis": "desubstanciar",
    "desubstanciaría": "desubstanciar",
    "desubstanciaríais": "desubstanciar",
    "desubstanciaríamos": "desubstanciar",
    "desubstanciarían": "desubstanciar",
    "desubstanciarías": "desubstanciar",
    "desubstancias": "desubstanciar",
    "desubstanciase": "desubstanciar",
    "desubstanciaseis": "desubstanciar",
    "desubstanciasen": "desubstanciar",
    "desubstanciases": "desubstanciar",
    "desubstanciaste": "desubstanciar",
    "desubstanciasteis": "desubstanciar",
    "desubstancie": "desubstanciar",
    "desubstanciemos": "desubstanciar",
    "desubstancien": "desubstanciar",
    "desubstancies": "desubstanciar",
    "desubstancio": "desubstanciar",
    "desubstanciábamos": "desubstanciar",
    "desubstanciáis": "desubstanciar",
    "desubstanciáramos": "desubstanciar",
    "desubstanciáremos": "desubstanciar",
    "desubstanciásemos": "desubstanciar",
    "desubstancié": "desubstanciar",
    "desubstanciéis": "desubstanciar",
    "desubstanció": "desubstanciar",
    "desucaciones": "desucación",
    "desuda": "desudar",
    "desudaba": "desudar",
    "desudabais": "desudar",
    "desudaban": "desudar",
    "desudabas": "desudar",
    "desudaciones": "desudación",
    "desudad": "desudar",
    "desudada": "desudar",
    "desudadas": "desudar",
    "desudado": "desudar",
    "desudados": "desudar",
    "desudamos": "desudar",
    "desudan": "desudar",
    "desudando": "desudar",
    "desudara": "desudar",
    "desudarais": "desudar",
    "desudaran": "desudar",
    "desudaras": "desudar",
    "desudare": "desudar",
    "desudareis": "desudar",
    "desudaremos": "desudar",
    "desudaren": "desudar",
    "desudares": "desudar",
    "desudaron": "desudar",
    "desudará": "desudar",
    "desudarán": "desudar",
    "desudarás": "desudar",
    "desudaré": "desudar",
    "desudaréis": "desudar",
    "desudaría": "desudar",
    "desudaríais": "desudar",
    "desudaríamos": "desudar",
    "desudarían": "desudar",
    "desudarías": "desudar",
    "desudas": "desudar",
    "desudase": "desudar",
    "desudaseis": "desudar",
    "desudasen": "desudar",
    "desudases": "desudar",
    "desudaste": "desudar",
    "desudasteis": "desudar",
    "desude": "desudar",
    "desudemos": "desudar",
    "desuden": "desudar",
    "desudes": "desudar",
    "desudo": "desudar",
    "desudábamos": "desudar",
    "desudáis": "desudar",
    "desudáramos": "desudar",
    "desudáremos": "desudar",
    "desudásemos": "desudar",
    "desudé": "desudar",
    "desudéis": "desudar",
    "desudó": "desudar",
    "desuelda": "desoldar",
    "desueldan": "desoldar",
    "desueldas": "desoldar",
    "desuelde": "desoldar",
    "desuelden": "desoldar",
    "desueldes": "desoldar",
    "desueldo": "desoldar",
    "desuella": "desollar",
    "desuellan": "desollar",
    "desuellas": "desollar",
    "desuelle": "desollar",
    "desuellen": "desollar",
    "desuelles": "desollar",
    "desuello": "desollar",
    "desuellos": "desuello",
    "desulfitaciones": "desulfitación",
    "desulfonaciones": "desulfonación",
    "desulfura": "desulfurar",
    "desulfuraba": "desulfurar",
    "desulfurabais": "desulfurar",
    "desulfuraban": "desulfurar",
    "desulfurabas": "desulfurar",
    "desulfuraciones": "desulfuración",
    "desulfurad": "desulfurar",
    "desulfurada": "desulfurar",
    "desulfuradas": "desulfurar",
    "desulfurado": "desulfurar",
    "desulfurados": "desulfurar",
    "desulfuramos": "desulfurar",
    "desulfuran": "desulfurar",
    "desulfurando": "desulfurar",
    "desulfurara": "desulfurar",
    "desulfurarais": "desulfurar",
    "desulfuraran": "desulfurar",
    "desulfuraras": "desulfurar",
    "desulfurare": "desulfurar",
    "desulfurareis": "desulfurar",
    "desulfuraremos": "desulfurar",
    "desulfuraren": "desulfurar",
    "desulfurares": "desulfurar",
    "desulfuraron": "desulfurar",
    "desulfurará": "desulfurar",
    "desulfurarán": "desulfurar",
    "desulfurarás": "desulfurar",
    "desulfuraré": "desulfurar",
    "desulfuraréis": "desulfurar",
    "desulfuraría": "desulfurar",
    "desulfuraríais": "desulfurar",
    "desulfuraríamos": "desulfurar",
    "desulfurarían": "desulfurar",
    "desulfurarías": "desulfurar",
    "desulfuras": "desulfurar",
    "desulfurase": "desulfurar",
    "desulfuraseis": "desulfurar",
    "desulfurasen": "desulfurar",
    "desulfurases": "desulfurar",
    "desulfuraste": "desulfurar",
    "desulfurasteis": "desulfurar",
    "desulfure": "desulfurar",
    "desulfuremos": "desulfurar",
    "desulfuren": "desulfurar",
    "desulfures": "desulfurar",
    "desulfuro": "desulfurar",
    "desulfurábamos": "desulfurar",
    "desulfuráis": "desulfurar",
    "desulfuráramos": "desulfurar",
    "desulfuráremos": "desulfurar",
    "desulfurásemos": "desulfurar",
    "desulfuré": "desulfurar",
    "desulfuréis": "desulfurar",
    "desulfuró": "desulfurar",
    "desuna": "desunir",
    "desunamos": "desunir",
    "desunan": "desunir",
    "desunas": "desunir",
    "desunce": "desuncir",
    "desuncen": "desuncir",
    "desunces": "desuncir",
    "desuncid": "desuncir",
    "desuncida": "desuncir",
    "desuncidas": "desuncir",
    "desuncido": "desuncir",
    "desuncidos": "desuncir",
    "desunciendo": "desuncir",
    "desunciera": "desuncir",
    "desuncierais": "desuncir",
    "desuncieran": "desuncir",
    "desuncieras": "desuncir",
    "desunciere": "desuncir",
    "desunciereis": "desuncir",
    "desuncieren": "desuncir",
    "desuncieres": "desuncir",
    "desuncieron": "desuncir",
    "desunciese": "desuncir",
    "desuncieseis": "desuncir",
    "desunciesen": "desuncir",
    "desuncieses": "desuncir",
    "desuncimos": "desuncir",
    "desunciones": "desunción",
    "desunciremos": "desuncir",
    "desuncirá": "desuncir",
    "desuncirán": "desuncir",
    "desuncirás": "desuncir",
    "desunciré": "desuncir",
    "desunciréis": "desuncir",
    "desunciría": "desuncir",
    "desunciríais": "desuncir",
    "desunciríamos": "desuncir",
    "desuncirían": "desuncir",
    "desuncirías": "desuncir",
    "desunciste": "desuncir",
    "desuncisteis": "desuncir",
    "desunciéramos": "desuncir",
    "desunciéremos": "desuncir",
    "desunciésemos": "desuncir",
    "desunció": "desuncir",
    "desuncí": "desuncir",
    "desuncía": "desuncir",
    "desuncíais": "desuncir",
    "desuncíamos": "desuncir",
    "desuncían": "desuncir",
    "desuncías": "desuncir",
    "desuncís": "desuncir",
    "desune": "desunir",
    "desunen": "desunir",
    "desunes": "desunir",
    "desunid": "desunir",
    "desunida": "desunir",
    "desunidas": "desunir",
    "desunido": "desunir",
    "desunidos": "desunir",
    "desuniendo": "desunir",
    "desuniera": "desunir",
    "desunierais": "desunir",
    "desunieran": "desunir",
    "desunieras": "desunir",
    "desuniere": "desunir",
    "desuniereis": "desunir",
    "desunieren": "desunir",
    "desunieres": "desunir",
    "desunieron": "desunir",
    "desuniese": "desunir",
    "desunieseis": "desunir",
    "desuniesen": "desunir",
    "desunieses": "desunir",
    "desunimos": "desunir",
    "desuniones": "desunión",
    "desuniremos": "desunir",
    "desunirá": "desunir",
    "desunirán": "desunir",
    "desunirás": "desunir",
    "desuniré": "desunir",
    "desuniréis": "desunir",
    "desuniría": "desunir",
    "desuniríais": "desunir",
    "desuniríamos": "desunir",
    "desunirían": "desunir",
    "desunirías": "desunir",
    "desuniste": "desunir",
    "desunisteis": "desunir",
    "desuniéramos": "desunir",
    "desuniéremos": "desunir",
    "desuniésemos": "desunir",
    "desunió": "desunir",
    "desuno": "desunir",
    "desunza": "desuncir",
    "desunzamos": "desuncir",
    "desunzan": "desuncir",
    "desunzas": "desuncir",
    "desunzo": "desuncir",
    "desunzáis": "desuncir",
    "desunáis": "desunir",
    "desuní": "desunir",
    "desunía": "desunir",
    "desuníais": "desunir",
    "desuníamos": "desunir",
    "desunían": "desunir",
    "desunías": "desunir",
    "desunís": "desunir",
    "desurda": "desurdir",
    "desurdamos": "desurdir",
    "desurdan": "desurdir",
    "desurdas": "desurdir",
    "desurde": "desurdir",
    "desurden": "desurdir",
    "desurdes": "desurdir",
    "desurdid": "desurdir",
    "desurdida": "desurdir",
    "desurdidas": "desurdir",
    "desurdido": "desurdir",
    "desurdidos": "desurdir",
    "desurdiendo": "desurdir",
    "desurdiera": "desurdir",
    "desurdierais": "desurdir",
    "desurdieran": "desurdir",
    "desurdieras": "desurdir",
    "desurdiere": "desurdir",
    "desurdiereis": "desurdir",
    "desurdieren": "desurdir",
    "desurdieres": "desurdir",
    "desurdieron": "desurdir",
    "desurdiese": "desurdir",
    "desurdieseis": "desurdir",
    "desurdiesen": "desurdir",
    "desurdieses": "desurdir",
    "desurdimos": "desurdir",
    "desurdiremos": "desurdir",
    "desurdirá": "desurdir",
    "desurdirán": "desurdir",
    "desurdirás": "desurdir",
    "desurdiré": "desurdir",
    "desurdiréis": "desurdir",
    "desurdiría": "desurdir",
    "desurdiríais": "desurdir",
    "desurdiríamos": "desurdir",
    "desurdirían": "desurdir",
    "desurdirías": "desurdir",
    "desurdiste": "desurdir",
    "desurdisteis": "desurdir",
    "desurdiéramos": "desurdir",
    "desurdiéremos": "desurdir",
    "desurdiésemos": "desurdir",
    "desurdió": "desurdir",
    "desurdo": "desurdir",
    "desurdáis": "desurdir",
    "desurdí": "desurdir",
    "desurdía": "desurdir",
    "desurdíais": "desurdir",
    "desurdíamos": "desurdir",
    "desurdían": "desurdir",
    "desurdías": "desurdir",
    "desurdís": "desurdir",
    "desusa": "desusar",
    "desusaba": "desusar",
    "desusabais": "desusar",
    "desusaban": "desusar",
    "desusabas": "desusar",
    "desusad": "desusar",
    "desusada": "desusar",
    "desusadas": "desusar",
    "desusado": "desusar",
    "desusados": "desusar",
    "desusamos": "desusar",
    "desusan": "desusar",
    "desusando": "desusar",
    "desusara": "desusar",
    "desusarais": "desusar",
    "desusaran": "desusar",
    "desusaras": "desusar",
    "desusare": "desusar",
    "desusareis": "desusar",
    "desusaremos": "desusar",
    "desusaren": "desusar",
    "desusares": "desusar",
    "desusaron": "desusar",
    "desusará": "desusar",
    "desusarán": "desusar",
    "desusarás": "desusar",
    "desusaré": "desusar",
    "desusaréis": "desusar",
    "desusaría": "desusar",
    "desusaríais": "desusar",
    "desusaríamos": "desusar",
    "desusarían": "desusar",
    "desusarías": "desusar",
    "desusas": "desusar",
    "desusase": "desusar",
    "desusaseis": "desusar",
    "desusasen": "desusar",
    "desusases": "desusar",
    "desusaste": "desusar",
    "desusasteis": "desusar",
    "desuse": "desusar",
    "desusemos": "desusar",
    "desusen": "desusar",
    "desuses": "desusar",
    "desuso": "desusar",
    "desusos": "desuso",
    "desustancia": "desustanciar",
    "desustanciaba": "desustanciar",
    "desustanciabais": "desustanciar",
    "desustanciaban": "desustanciar",
    "desustanciabas": "desustanciar",
    "desustanciad": "desustanciar",
    "desustanciada": "desustanciar",
    "desustanciadas": "desustanciar",
    "desustanciado": "desustanciar",
    "desustanciados": "desustanciar",
    "desustanciamientos": "desustanciamiento",
    "desustanciamos": "desustanciar",
    "desustancian": "desustanciar",
    "desustanciando": "desustanciar",
    "desustanciara": "desustanciar",
    "desustanciarais": "desustanciar",
    "desustanciaran": "desustanciar",
    "desustanciaras": "desustanciar",
    "desustanciare": "desustanciar",
    "desustanciareis": "desustanciar",
    "desustanciaremos": "desustanciar",
    "desustanciaren": "desustanciar",
    "desustanciares": "desustanciar",
    "desustanciaron": "desustanciar",
    "desustanciará": "desustanciar",
    "desustanciarán": "desustanciar",
    "desustanciarás": "desustanciar",
    "desustanciaré": "desustanciar",
    "desustanciaréis": "desustanciar",
    "desustanciaría": "desustanciar",
    "desustanciaríais": "desustanciar",
    "desustanciaríamos": "desustanciar",
    "desustanciarían": "desustanciar",
    "desustanciarías": "desustanciar",
    "desustancias": "desustanciar",
    "desustanciase": "desustanciar",
    "desustanciaseis": "desustanciar",
    "desustanciasen": "desustanciar",
    "desustanciases": "desustanciar",
    "desustanciaste": "desustanciar",
    "desustanciasteis": "desustanciar",
    "desustancie": "desustanciar",
    "desustanciemos": "desustanciar",
    "desustancien": "desustanciar",
    "desustancies": "desustanciar",
    "desustancio": "desustanciar",
    "desustanciábamos": "desustanciar",
    "desustanciáis": "desustanciar",
    "desustanciáramos": "desustanciar",
    "desustanciáremos": "desustanciar",
    "desustanciásemos": "desustanciar",
    "desustancié": "desustanciar",
    "desustanciéis": "desustanciar",
    "desustanció": "desustanciar",
    "desusábamos": "desusar",
    "desusáis": "desusar",
    "desusáramos": "desusar",
    "desusáremos": "desusar",
    "desusásemos": "desusar",
    "desusé": "desusar",
    "desuséis": "desusar",
    "desusó": "desusar",
    "desutilidades": "desutilidad",
    "desuña": "desuñar",
    "desuñaba": "desuñar",
    "desuñabais": "desuñar",
    "desuñaban": "desuñar",
    "desuñabas": "desuñar",
    "desuñad": "desuñar",
    "desuñada": "desuñar",
    "desuñadas": "desuñar",
    "desuñado": "desuñar",
    "desuñados": "desuñar",
    "desuñamos": "desuñar",
    "desuñan": "desuñar",
    "desuñando": "desuñar",
    "desuñara": "desuñar",
    "desuñarais": "desuñar",
    "desuñaran": "desuñar",
    "desuñaras": "desuñar",
    "desuñare": "desuñar",
    "desuñareis": "desuñar",
    "desuñaremos": "desuñar",
    "desuñaren": "desuñar",
    "desuñares": "desuñar",
    "desuñaron": "desuñar",
    "desuñará": "desuñar",
    "desuñarán": "desuñar",
    "desuñarás": "desuñar",
    "desuñaré": "desuñar",
    "desuñaréis": "desuñar",
    "desuñaría": "desuñar",
    "desuñaríais": "desuñar",
    "desuñaríamos": "desuñar",
    "desuñarían": "desuñar",
    "desuñarías": "desuñar",
    "desuñas": "desuñar",
    "desuñase": "desuñar",
    "desuñaseis": "desuñar",
    "desuñasen": "desuñar",
    "desuñases": "desuñar",
    "desuñaste": "desuñar",
    "desuñasteis": "desuñar",
    "desuñe": "desuñar",
    "desuñemos": "desuñar",
    "desuñen": "desuñar",
    "desuñes": "desuñar",
    "desuño": "desuñar",
    "desuñábamos": "desuñar",
    "desuñáis": "desuñar",
    "desuñáramos": "desuñar",
    "desuñáremos": "desuñar",
    "desuñásemos": "desuñar",
    "desuñé": "desuñar",
    "desuñéis": "desuñar",
    "desuñó": "desuñar",
    "desvaha": "desvahar",
    "desvahaba": "desvahar",
    "desvahabais": "desvahar",
    "desvahaban": "desvahar",
    "desvahabas": "desvahar",
    "desvahad": "desvahar",
    "desvahada": "desvahar",
    "desvahadas": "desvahar",
    "desvahado": "desvahar",
    "desvahados": "desvahar",
    "desvahamos": "desvahar",
    "desvahan": "desvahar",
    "desvahando": "desvahar",
    "desvahara": "desvahar",
    "desvaharais": "desvahar",
    "desvaharan": "desvahar",
    "desvaharas": "desvahar",
    "desvahare": "desvahar",
    "desvahareis": "desvahar",
    "desvaharemos": "desvahar",
    "desvaharen": "desvahar",
    "desvahares": "desvahar",
    "desvaharon": "desvahar",
    "desvahará": "desvahar",
    "desvaharán": "desvahar",
    "desvaharás": "desvahar",
    "desvaharé": "desvahar",
    "desvaharéis": "desvahar",
    "desvaharía": "desvahar",
    "desvaharíais": "desvahar",
    "desvaharíamos": "desvahar",
    "desvaharían": "desvahar",
    "desvaharías": "desvahar",
    "desvahas": "desvahar",
    "desvahase": "desvahar",
    "desvahaseis": "desvahar",
    "desvahasen": "desvahar",
    "desvahases": "desvahar",
    "desvahaste": "desvahar",
    "desvahasteis": "desvahar",
    "desvahe": "desvahar",
    "desvahemos": "desvahar",
    "desvahen": "desvahar",
    "desvahes": "desvahar",
    "desvaho": "desvahar",
    "desvahábamos": "desvahar",
    "desvaháis": "desvahar",
    "desvaháramos": "desvahar",
    "desvaháremos": "desvahar",
    "desvahásemos": "desvahar",
    "desvahé": "desvahar",
    "desvahéis": "desvahar",
    "desvahó": "desvahar",
    "desvaina": "desvainar",
    "desvainaba": "desvainar",
    "desvainabais": "desvainar",
    "desvainaban": "desvainar",
    "desvainabas": "desvainar",
    "desvainad": "desvainar",
    "desvainada": "desvainar",
    "desvainadas": "desvainar",
    "desvainado": "desvainar",
    "desvainados": "desvainar",
    "desvainaduras": "desvainadura",
    "desvainamos": "desvainar",
    "desvainan": "desvainar",
    "desvainando": "desvainar",
    "desvainara": "desvainar",
    "desvainarais": "desvainar",
    "desvainaran": "desvainar",
    "desvainaras": "desvainar",
    "desvainare": "desvainar",
    "desvainareis": "desvainar",
    "desvainaremos": "desvainar",
    "desvainaren": "desvainar",
    "desvainares": "desvainar",
    "desvainaron": "desvainar",
    "desvainará": "desvainar",
    "desvainarán": "desvainar",
    "desvainarás": "desvainar",
    "desvainaré": "desvainar",
    "desvainaréis": "desvainar",
    "desvainaría": "desvainar",
    "desvainaríais": "desvainar",
    "desvainaríamos": "desvainar",
    "desvainarían": "desvainar",
    "desvainarías": "desvainar",
    "desvainas": "desvainar",
    "desvainase": "desvainar",
    "desvainaseis": "desvainar",
    "desvainasen": "desvainar",
    "desvainases": "desvainar",
    "desvainaste": "desvainar",
    "desvainasteis": "desvainar",
    "desvaine": "desvainar",
    "desvainemos": "desvainar",
    "desvainen": "desvainar",
    "desvaines": "desvainar",
    "desvaino": "desvainar",
    "desvainábamos": "desvainar",
    "desvaináis": "desvainar",
    "desvaináramos": "desvainar",
    "desvaináremos": "desvainar",
    "desvainásemos": "desvainar",
    "desvainé": "desvainar",
    "desvainéis": "desvainar",
    "desvainó": "desvainar",
    "desvalida": "desvalido",
    "desvalidas": "desvalido",
    "desvalidos": "desvalido",
    "desvalija": "desvalijar",
    "desvalijaba": "desvalijar",
    "desvalijabais": "desvalijar",
    "desvalijaban": "desvalijar",
    "desvalijabas": "desvalijar",
    "desvalijad": "desvalijar",
    "desvalijada": "desvalijar",
    "desvalijadas": "desvalijar",
    "desvalijado": "desvalijar",
    "desvalijadora": "desvalijador",
    "desvalijadoras": "desvalijador",
    "desvalijadores": "desvalijador",
    "desvalijados": "desvalijar",
    "desvalijamientos": "desvalijamiento",
    "desvalijamos": "desvalijar",
    "desvalijan": "desvalijar",
    "desvalijando": "desvalijar",
    "desvalijara": "desvalijar",
    "desvalijarais": "desvalijar",
    "desvalijaran": "desvalijar",
    "desvalijaras": "desvalijar",
    "desvalijare": "desvalijar",
    "desvalijareis": "desvalijar",
    "desvalijaremos": "desvalijar",
    "desvalijaren": "desvalijar",
    "desvalijares": "desvalijar",
    "desvalijaron": "desvalijar",
    "desvalijará": "desvalijar",
    "desvalijarán": "desvalijar",
    "desvalijarás": "desvalijar",
    "desvalijaré": "desvalijar",
    "desvalijaréis": "desvalijar",
    "desvalijaría": "desvalijar",
    "desvalijaríais": "desvalijar",
    "desvalijaríamos": "desvalijar",
    "desvalijarían": "desvalijar",
    "desvalijarías": "desvalijar",
    "desvalijas": "desvalijar",
    "desvalijase": "desvalijar",
    "desvalijaseis": "desvalijar",
    "desvalijasen": "desvalijar",
    "desvalijases": "desvalijar",
    "desvalijaste": "desvalijar",
    "desvalijasteis": "desvalijar",
    "desvalije": "desvalijar",
    "desvalijemos": "desvalijar",
    "desvalijen": "desvalijar",
    "desvalijes": "desvalijar",
    "desvalijo": "desvalijar",
    "desvalijos": "desvalijo",
    "desvalijábamos": "desvalijar",
    "desvalijáis": "desvalijar",
    "desvalijáramos": "desvalijar",
    "desvalijáremos": "desvalijar",
    "desvalijásemos": "desvalijar",
    "desvalijé": "desvalijar",
    "desvalijéis": "desvalijar",
    "desvalijó": "desvalijar",
    "desvalimientos": "desvalimiento",
    "desvalora": "desvalorar",
    "desvaloraba": "desvalorar",
    "desvalorabais": "desvalorar",
    "desvaloraban": "desvalorar",
    "desvalorabas": "desvalorar",
    "desvalorad": "desvalorar",
    "desvalorada": "desvalorar",
    "desvaloradas": "desvalorar",
    "desvalorado": "desvalorar",
    "desvalorados": "desvalorar",
    "desvaloramos": "desvalorar",
    "desvaloran": "desvalorar",
    "desvalorando": "desvalorar",
    "desvalorara": "desvalorar",
    "desvalorarais": "desvalorar",
    "desvaloraran": "desvalorar",
    "desvaloraras": "desvalorar",
    "desvalorare": "desvalorar",
    "desvalorareis": "desvalorar",
    "desvaloraremos": "desvalorar",
    "desvaloraren": "desvalorar",
    "desvalorares": "desvalorar",
    "desvaloraron": "desvalorar",
    "desvalorará": "desvalorar",
    "desvalorarán": "desvalorar",
    "desvalorarás": "desvalorar",
    "desvaloraré": "desvalorar",
    "desvaloraréis": "desvalorar",
    "desvaloraría": "desvalorar",
    "desvaloraríais": "desvalorar",
    "desvaloraríamos": "desvalorar",
    "desvalorarían": "desvalorar",
    "desvalorarías": "desvalorar",
    "desvaloras": "desvalorar",
    "desvalorase": "desvalorar",
    "desvaloraseis": "desvalorar",
    "desvalorasen": "desvalorar",
    "desvalorases": "desvalorar",
    "desvaloraste": "desvalorar",
    "desvalorasteis": "desvalorar",
    "desvalore": "desvalorar",
    "desvaloremos": "desvalorar",
    "desvaloren": "desvalorar",
    "desvalores": "desvalorar",
    "desvalorice": "desvalorizar",
    "desvaloricemos": "desvalorizar",
    "desvaloricen": "desvalorizar",
    "desvalorices": "desvalorizar",
    "desvaloricé": "desvalorizar",
    "desvaloricéis": "desvalorizar",
    "desvaloriza": "desvalorizar",
    "desvalorizaba": "desvalorizar",
    "desvalorizabais": "desvalorizar",
    "desvalorizaban": "desvalorizar",
    "desvalorizabas": "desvalorizar",
    "desvalorizaciones": "desvalorización",
    "desvalorizad": "desvalorizar",
    "desvalorizada": "desvalorizar",
    "desvalorizadas": "desvalorizar",
    "desvalorizado": "desvalorizar",
    "desvalorizados": "desvalorizar",
    "desvalorizamos": "desvalorizar",
    "desvalorizan": "desvalorizar",
    "desvalorizando": "desvalorizar",
    "desvalorizara": "desvalorizar",
    "desvalorizarais": "desvalorizar",
    "desvalorizaran": "desvalorizar",
    "desvalorizaras": "desvalorizar",
    "desvalorizare": "desvalorizar",
    "desvalorizareis": "desvalorizar",
    "desvalorizaremos": "desvalorizar",
    "desvalorizaren": "desvalorizar",
    "desvalorizares": "desvalorizar",
    "desvalorizaron": "desvalorizar",
    "desvalorizará": "desvalorizar",
    "desvalorizarán": "desvalorizar",
    "desvalorizarás": "desvalorizar",
    "desvalorizaré": "desvalorizar",
    "desvalorizaréis": "desvalorizar",
    "desvalorizaría": "desvalorizar",
    "desvalorizaríais": "desvalorizar",
    "desvalorizaríamos": "desvalorizar",
    "desvalorizarían": "desvalorizar",
    "desvalorizarías": "desvalorizar",
    "desvalorizas": "desvalorizar",
    "desvalorizase": "desvalorizar",
    "desvalorizaseis": "desvalorizar",
    "desvalorizasen": "desvalorizar",
    "desvalorizases": "desvalorizar",
    "desvalorizaste": "desvalorizar",
    "desvalorizasteis": "desvalorizar",
    "desvalorizo": "desvalorizar",
    "desvalorizábamos": "desvalorizar",
    "desvalorizáis": "desvalorizar",
    "desvalorizáramos": "desvalorizar",
    "desvalorizáremos": "desvalorizar",
    "desvalorizásemos": "desvalorizar",
    "desvalorizó": "desvalorizar",
    "desvaloro": "desvalorar",
    "desvalorábamos": "desvalorar",
    "desvaloráis": "desvalorar",
    "desvaloráramos": "desvalorar",
    "desvaloráremos": "desvalorar",
    "desvalorásemos": "desvalorar",
    "desvaloré": "desvalorar",
    "desvaloréis": "desvalorar",
    "desvaloró": "desvalorar",
    "desvanece": "desvanecer",
    "desvaneced": "desvanecer",
    "desvanecedora": "desvanecedor",
    "desvanecedoras": "desvanecedor",
    "desvanecedores": "desvanecedor",
    "desvanecemos": "desvanecer",
    "desvanecen": "desvanecer",
    "desvaneceremos": "desvanecer",
    "desvanecerá": "desvanecer",
    "desvanecerán": "desvanecer",
    "desvanecerás": "desvanecer",
    "desvaneceré": "desvanecer",
    "desvaneceréis": "desvanecer",
    "desvanecería": "desvanecer",
    "desvaneceríais": "desvanecer",
    "desvaneceríamos": "desvanecer",
    "desvanecerían": "desvanecer",
    "desvanecerías": "desvanecer",
    "desvaneces": "desvanecer",
    "desvanecida": "desvanecer",
    "desvanecidas": "desvanecer",
    "desvanecido": "desvanecer",
    "desvanecidos": "desvanecer",
    "desvaneciendo": "desvanecer",
    "desvaneciera": "desvanecer",
    "desvanecierais": "desvanecer",
    "desvanecieran": "desvanecer",
    "desvanecieras": "desvanecer",
    "desvaneciere": "desvanecer",
    "desvaneciereis": "desvanecer",
    "desvanecieren": "desvanecer",
    "desvanecieres": "desvanecer",
    "desvanecieron": "desvanecer",
    "desvaneciese": "desvanecer",
    "desvanecieseis": "desvanecer",
    "desvaneciesen": "desvanecer",
    "desvanecieses": "desvanecer",
    "desvanecimientos": "desvanecimiento",
    "desvanecimos": "desvanecer",
    "desvaneciste": "desvanecer",
    "desvanecisteis": "desvanecer",
    "desvaneciéramos": "desvanecer",
    "desvaneciéremos": "desvanecer",
    "desvaneciésemos": "desvanecer",
    "desvaneció": "desvanecer",
    "desvanecéis": "desvanecer",
    "desvanecí": "desvanecer",
    "desvanecía": "desvanecer",
    "desvanecíais": "desvanecer",
    "desvanecíamos": "desvanecer",
    "desvanecían": "desvanecer",
    "desvanecías": "desvanecer",
    "desvanes": "desván",
    "desvanezca": "desvanecer",
    "desvanezcamos": "desvanecer",
    "desvanezcan": "desvanecer",
    "desvanezcas": "desvanecer",
    "desvanezco": "desvanecer",
    "desvanezcáis": "desvanecer",
    "desvaporizaciones": "desvaporización",
    "desvaporizaderos": "desvaporizadero",
    "desvara": "desvarar",
    "desvaraba": "desvarar",
    "desvarabais": "desvarar",
    "desvaraban": "desvarar",
    "desvarabas": "desvarar",
    "desvarad": "desvarar",
    "desvarada": "desvarar",
    "desvaradas": "desvarar",
    "desvarado": "desvarar",
    "desvarados": "desvarar",
    "desvaramos": "desvarar",
    "desvaran": "desvarar",
    "desvarando": "desvarar",
    "desvarara": "desvarar",
    "desvararais": "desvarar",
    "desvararan": "desvarar",
    "desvararas": "desvarar",
    "desvarare": "desvarar",
    "desvarareis": "desvarar",
    "desvararemos": "desvarar",
    "desvararen": "desvarar",
    "desvarares": "desvarar",
    "desvararon": "desvarar",
    "desvarará": "desvarar",
    "desvararán": "desvarar",
    "desvararás": "desvarar",
    "desvararé": "desvarar",
    "desvararéis": "desvarar",
    "desvararía": "desvarar",
    "desvararíais": "desvarar",
    "desvararíamos": "desvarar",
    "desvararían": "desvarar",
    "desvararías": "desvarar",
    "desvaras": "desvarar",
    "desvarase": "desvarar",
    "desvaraseis": "desvarar",
    "desvarasen": "desvarar",
    "desvarases": "desvarar",
    "desvaraste": "desvarar",
    "desvarasteis": "desvarar",
    "desvare": "desvarar",
    "desvaremos": "desvarar",
    "desvaren": "desvarar",
    "desvares": "desvarar",
    "desvariaba": "desvariar",
    "desvariabais": "desvariar",
    "desvariaban": "desvariar",
    "desvariabas": "desvariar",
    "desvariad": "desvariar",
    "desvariada": "desvariar",
    "desvariadas": "desvariar",
    "desvariado": "desvariar",
    "desvariados": "desvariar",
    "desvariamos": "desvariar",
    "desvariando": "desvariar",
    "desvariara": "desvariar",
    "desvariarais": "desvariar",
    "desvariaran": "desvariar",
    "desvariaras": "desvariar",
    "desvariare": "desvariar",
    "desvariareis": "desvariar",
    "desvariaremos": "desvariar",
    "desvariaren": "desvariar",
    "desvariares": "desvariar",
    "desvariaron": "desvariar",
    "desvariará": "desvariar",
    "desvariarán": "desvariar",
    "desvariarás": "desvariar",
    "desvariaré": "desvariar",
    "desvariaréis": "desvariar",
    "desvariaría": "desvariar",
    "desvariaríais": "desvariar",
    "desvariaríamos": "desvariar",
    "desvariarían": "desvariar",
    "desvariarías": "desvariar",
    "desvariase": "desvariar",
    "desvariaseis": "desvariar",
    "desvariasen": "desvariar",
    "desvariases": "desvariar",
    "desvariaste": "desvariar",
    "desvariasteis": "desvariar",
    "desvariemos": "desvariar",
    "desvariábamos": "desvariar",
    "desvariáis": "desvariar",
    "desvariáramos": "desvariar",
    "desvariáremos": "desvariar",
    "desvariásemos": "desvariar",
    "desvarié": "desvariar",
    "desvariéis": "desvariar",
    "desvarió": "desvariar",
    "desvaro": "desvarar",
    "desvarábamos": "desvarar",
    "desvaráis": "desvarar",
    "desvaráramos": "desvarar",
    "desvaráremos": "desvarar",
    "desvarásemos": "desvarar",
    "desvaré": "desvarar",
    "desvaréis": "desvarar",
    "desvaría": "desvariar",
    "desvarían": "desvariar",
    "desvarías": "desvariar",
    "desvaríe": "desvariar",
    "desvaríen": "desvariar",
    "desvaríes": "desvariar",
    "desvarío": "desvariar",
    "desvaríos": "desvarío",
    "desvaró": "desvarar",
    "desvastiga": "desvastigar",
    "desvastigaba": "desvastigar",
    "desvastigabais": "desvastigar",
    "desvastigaban": "desvastigar",
    "desvastigabas": "desvastigar",
    "desvastigad": "desvastigar",
    "desvastigada": "desvastigar",
    "desvastigadas": "desvastigar",
    "desvastigado": "desvastigar",
    "desvastigados": "desvastigar",
    "desvastigamos": "desvastigar",
    "desvastigan": "desvastigar",
    "desvastigando": "desvastigar",
    "desvastigara": "desvastigar",
    "desvastigarais": "desvastigar",
    "desvastigaran": "desvastigar",
    "desvastigaras": "desvastigar",
    "desvastigare": "desvastigar",
    "desvastigareis": "desvastigar",
    "desvastigaremos": "desvastigar",
    "desvastigaren": "desvastigar",
    "desvastigares": "desvastigar",
    "desvastigaron": "desvastigar",
    "desvastigará": "desvastigar",
    "desvastigarán": "desvastigar",
    "desvastigarás": "desvastigar",
    "desvastigaré": "desvastigar",
    "desvastigaréis": "desvastigar",
    "desvastigaría": "desvastigar",
    "desvastigaríais": "desvastigar",
    "desvastigaríamos": "desvastigar",
    "desvastigarían": "desvastigar",
    "desvastigarías": "desvastigar",
    "desvastigas": "desvastigar",
    "desvastigase": "desvastigar",
    "desvastigaseis": "desvastigar",
    "desvastigasen": "desvastigar",
    "desvastigases": "desvastigar",
    "desvastigaste": "desvastigar",
    "desvastigasteis": "desvastigar",
    "desvastigo": "desvastigar",
    "desvastigue": "desvastigar",
    "desvastiguemos": "desvastigar",
    "desvastiguen": "desvastigar",
    "desvastigues": "desvastigar",
    "desvastigué": "desvastigar",
    "desvastiguéis": "desvastigar",
    "desvastigábamos": "desvastigar",
    "desvastigáis": "desvastigar",
    "desvastigáramos": "desvastigar",
    "desvastigáremos": "desvastigar",
    "desvastigásemos": "desvastigar",
    "desvastigó": "desvastigar",
    "desvatada": "desvatado",
    "desvatadas": "desvatado",
    "desvatados": "desvatado",
    "desvaída": "desvaído",
    "desvaídas": "desvaído",
    "desvaídos": "desvaído",
    "desveda": "desvedar",
    "desvedaba": "desvedar",
    "desvedabais": "desvedar",
    "desvedaban": "desvedar",
    "desvedabas": "desvedar",
    "desvedad": "desvedar",
    "desvedada": "desvedar",
    "desvedadas": "desvedar",
    "desvedado": "desvedar",
    "desvedados": "desvedar",
    "desvedamos": "desvedar",
    "desvedan": "desvedar",
    "desvedando": "desvedar",
    "desvedara": "desvedar",
    "desvedarais": "desvedar",
    "desvedaran": "desvedar",
    "desvedaras": "desvedar",
    "desvedare": "desvedar",
    "desvedareis": "desvedar",
    "desvedaremos": "desvedar",
    "desvedaren": "desvedar",
    "desvedares": "desvedar",
    "desvedaron": "desvedar",
    "desvedará": "desvedar",
    "desvedarán": "desvedar",
    "desvedarás": "desvedar",
    "desvedaré": "desvedar",
    "desvedaréis": "desvedar",
    "desvedaría": "desvedar",
    "desvedaríais": "desvedar",
    "desvedaríamos": "desvedar",
    "desvedarían": "desvedar",
    "desvedarías": "desvedar",
    "desvedas": "desvedar",
    "desvedase": "desvedar",
    "desvedaseis": "desvedar",
    "desvedasen": "desvedar",
    "desvedases": "desvedar",
    "desvedaste": "desvedar",
    "desvedasteis": "desvedar",
    "desvede": "desvedar",
    "desvedemos": "desvedar",
    "desveden": "desvedar",
    "desvedes": "desvedar",
    "desvedo": "desvedar",
    "desvedábamos": "desvedar",
    "desvedáis": "desvedar",
    "desvedáramos": "desvedar",
    "desvedáremos": "desvedar",
    "desvedásemos": "desvedar",
    "desvedé": "desvedar",
    "desvedéis": "desvedar",
    "desvedó": "desvedar",
    "desvela": "desvelar",
    "desvelaba": "desvelar",
    "desvelabais": "desvelar",
    "desvelaban": "desvelar",
    "desvelabas": "desvelar",
    "desvelad": "desvelar",
    "desvelada": "desvelar",
    "desveladas": "desvelar",
    "desvelado": "desvelar",
    "desvelados": "desvelar",
    "desvelamos": "desvelar",
    "desvelan": "desvelar",
    "desvelando": "desvelar",
    "desvelara": "desvelar",
    "desvelarais": "desvelar",
    "desvelaran": "desvelar",
    "desvelaras": "desvelar",
    "desvelare": "desvelar",
    "desvelareis": "desvelar",
    "desvelaremos": "desvelar",
    "desvelaren": "desvelar",
    "desvelares": "desvelar",
    "desvelaron": "desvelar",
    "desvelará": "desvelar",
    "desvelarán": "desvelar",
    "desvelarás": "desvelar",
    "desvelaré": "desvelar",
    "desvelaréis": "desvelar",
    "desvelaría": "desvelar",
    "desvelaríais": "desvelar",
    "desvelaríamos": "desvelar",
    "desvelarían": "desvelar",
    "desvelarías": "desvelar",
    "desvelas": "desvelar",
    "desvelase": "desvelar",
    "desvelaseis": "desvelar",
    "desvelasen": "desvelar",
    "desvelases": "desvelar",
    "desvelaste": "desvelar",
    "desvelasteis": "desvelar",
    "desvele": "desvelar",
    "desvelemos": "desvelar",
    "desvelen": "desvelar",
    "desveles": "desvelar",
    "desvelo": "desvelar",
    "desvelos": "desvelo",
    "desvelábamos": "desvelar",
    "desveláis": "desvelar",
    "desveláramos": "desvelar",
    "desveláremos": "desvelar",
    "desvelásemos": "desvelar",
    "desvelé": "desvelar",
    "desveléis": "desvelar",
    "desveló": "desvelar",
    "desvena": "desvenar",
    "desvenaba": "desvenar",
    "desvenabais": "desvenar",
    "desvenaban": "desvenar",
    "desvenabas": "desvenar",
    "desvenad": "desvenar",
    "desvenada": "desvenar",
    "desvenadas": "desvenar",
    "desvenado": "desvenar",
    "desvenados": "desvenar",
    "desvenamos": "desvenar",
    "desvenan": "desvenar",
    "desvenando": "desvenar",
    "desvenara": "desvenar",
    "desvenarais": "desvenar",
    "desvenaran": "desvenar",
    "desvenaras": "desvenar",
    "desvenare": "desvenar",
    "desvenareis": "desvenar",
    "desvenaremos": "desvenar",
    "desvenaren": "desvenar",
    "desvenares": "desvenar",
    "desvenaron": "desvenar",
    "desvenará": "desvenar",
    "desvenarán": "desvenar",
    "desvenarás": "desvenar",
    "desvenaré": "desvenar",
    "desvenaréis": "desvenar",
    "desvenaría": "desvenar",
    "desvenaríais": "desvenar",
    "desvenaríamos": "desvenar",
    "desvenarían": "desvenar",
    "desvenarías": "desvenar",
    "desvenas": "desvenar",
    "desvenase": "desvenar",
    "desvenaseis": "desvenar",
    "desvenasen": "desvenar",
    "desvenases": "desvenar",
    "desvenaste": "desvenar",
    "desvenasteis": "desvenar",
    "desvencija": "desvencijar",
    "desvencijaba": "desvencijar",
    "desvencijabais": "desvencijar",
    "desvencijaban": "desvencijar",
    "desvencijabas": "desvencijar",
    "desvencijad": "desvencijar",
    "desvencijada": "desvencijar",
    "desvencijadas": "desvencijar",
    "desvencijado": "desvencijar",
    "desvencijados": "desvencijar",
    "desvencijamientos": "desvencijamiento",
    "desvencijamos": "desvencijar",
    "desvencijan": "desvencijar",
    "desvencijando": "desvencijar",
    "desvencijara": "desvencijar",
    "desvencijarais": "desvencijar",
    "desvencijaran": "desvencijar",
    "desvencijaras": "desvencijar",
    "desvencijare": "desvencijar",
    "desvencijareis": "desvencijar",
    "desvencijaremos": "desvencijar",
    "desvencijaren": "desvencijar",
    "desvencijares": "desvencijar",
    "desvencijaron": "desvencijar",
    "desvencijará": "desvencijar",
    "desvencijarán": "desvencijar",
    "desvencijarás": "desvencijar",
    "desvencijaré": "desvencijar",
    "desvencijaréis": "desvencijar",
    "desvencijaría": "desvencijar",
    "desvencijaríais": "desvencijar",
    "desvencijaríamos": "desvencijar",
    "desvencijarían": "desvencijar",
    "desvencijarías": "desvencijar",
    "desvencijas": "desvencijar",
    "desvencijase": "desvencijar",
    "desvencijaseis": "desvencijar",
    "desvencijasen": "desvencijar",
    "desvencijases": "desvencijar",
    "desvencijaste": "desvencijar",
    "desvencijasteis": "desvencijar",
    "desvencije": "desvencijar",
    "desvencijemos": "desvencijar",
    "desvencijen": "desvencijar",
    "desvencijes": "desvencijar",
    "desvencijo": "desvencijar",
    "desvencijábamos": "desvencijar",
    "desvencijáis": "desvencijar",
    "desvencijáramos": "desvencijar",
    "desvencijáremos": "desvencijar",
    "desvencijásemos": "desvencijar",
    "desvencijé": "desvencijar",
    "desvencijéis": "desvencijar",
    "desvencijó": "desvencijar",
    "desvenda": "desvendar",
    "desvendaba": "desvendar",
    "desvendabais": "desvendar",
    "desvendaban": "desvendar",
    "desvendabas": "desvendar",
    "desvendad": "desvendar",
    "desvendada": "desvendar",
    "desvendadas": "desvendar",
    "desvendado": "desvendar",
    "desvendados": "desvendar",
    "desvendamos": "desvendar",
    "desvendan": "desvendar",
    "desvendando": "desvendar",
    "desvendara": "desvendar",
    "desvendarais": "desvendar",
    "desvendaran": "desvendar",
    "desvendaras": "desvendar",
    "desvendare": "desvendar",
    "desvendareis": "desvendar",
    "desvendaremos": "desvendar",
    "desvendaren": "desvendar",
    "desvendares": "desvendar",
    "desvendaron": "desvendar",
    "desvendará": "desvendar",
    "desvendarán": "desvendar",
    "desvendarás": "desvendar",
    "desvendaré": "desvendar",
    "desvendaréis": "desvendar",
    "desvendaría": "desvendar",
    "desvendaríais": "desvendar",
    "desvendaríamos": "desvendar",
    "desvendarían": "desvendar",
    "desvendarías": "desvendar",
    "desvendas": "desvendar",
    "desvendase": "desvendar",
    "desvendaseis": "desvendar",
    "desvendasen": "desvendar",
    "desvendases": "desvendar",
    "desvendaste": "desvendar",
    "desvendasteis": "desvendar",
    "desvende": "desvendar",
    "desvendemos": "desvendar",
    "desvenden": "desvendar",
    "desvendes": "desvendar",
    "desvendo": "desvendar",
    "desvendábamos": "desvendar",
    "desvendáis": "desvendar",
    "desvendáramos": "desvendar",
    "desvendáremos": "desvendar",
    "desvendásemos": "desvendar",
    "desvendé": "desvendar",
    "desvendéis": "desvendar",
    "desvendó": "desvendar",
    "desvene": "desvenar",
    "desvenemos": "desvenar",
    "desvenen": "desvenar",
    "desvenes": "desvenar",
    "desveno": "desvenar",
    "desvenos": "desveno",
    "desventajas": "desventaja",
    "desventajosa": "desventajoso",
    "desventajosas": "desventajoso",
    "desventajosos": "desventajoso",
    "desventamientos": "desventamiento",
    "desventurada": "desventurado",
    "desventuradas": "desventurado",
    "desventurados": "desventurado",
    "desventuras": "desventura",
    "desvenábamos": "desvenar",
    "desvenáis": "desvenar",
    "desvenáramos": "desvenar",
    "desvenáremos": "desvenar",
    "desvenásemos": "desvenar",
    "desvené": "desvenar",
    "desvenéis": "desvenar",
    "desvenó": "desvenar",
    "desvergoncemos": "desvergonzar",
    "desvergoncé": "desvergonzar",
    "desvergoncéis": "desvergonzar",
    "desvergonzaba": "desvergonzar",
    "desvergonzabais": "desvergonzar",
    "desvergonzaban": "desvergonzar",
    "desvergonzabas": "desvergonzar",
    "desvergonzad": "desvergonzar",
    "desvergonzada": "desvergonzar",
    "desvergonzadas": "desvergonzar",
    "desvergonzado": "desvergonzar",
    "desvergonzados": "desvergonzar",
    "desvergonzamos": "desvergonzar",
    "desvergonzando": "desvergonzar",
    "desvergonzara": "desvergonzar",
    "desvergonzarais": "desvergonzar",
    "desvergonzaran": "desvergonzar",
    "desvergonzaras": "desvergonzar",
    "desvergonzare": "desvergonzar",
    "desvergonzareis": "desvergonzar",
    "desvergonzaremos": "desvergonzar",
    "desvergonzaren": "desvergonzar",
    "desvergonzares": "desvergonzar",
    "desvergonzaron": "desvergonzar",
    "desvergonzará": "desvergonzar",
    "desvergonzarán": "desvergonzar",
    "desvergonzarás": "desvergonzar",
    "desvergonzaré": "desvergonzar",
    "desvergonzaréis": "desvergonzar",
    "desvergonzaría": "desvergonzar",
    "desvergonzaríais": "desvergonzar",
    "desvergonzaríamos": "desvergonzar",
    "desvergonzarían": "desvergonzar",
    "desvergonzarías": "desvergonzar",
    "desvergonzase": "desvergonzar",
    "desvergonzaseis": "desvergonzar",
    "desvergonzasen": "desvergonzar",
    "desvergonzases": "desvergonzar",
    "desvergonzaste": "desvergonzar",
    "desvergonzasteis": "desvergonzar",
    "desvergonzábamos": "desvergonzar",
    "desvergonzáis": "desvergonzar",
    "desvergonzáramos": "desvergonzar",
    "desvergonzáremos": "desvergonzar",
    "desvergonzásemos": "desvergonzar",
    "desvergonzó": "desvergonzar",
    "desvergüence": "desvergonzar",
    "desvergüencen": "desvergonzar",
    "desvergüences": "desvergonzar",
    "desvergüenza": "desvergonzar",
    "desvergüenzan": "desvergonzar",
    "desvergüenzas": "desvergüenza",
    "desvergüenzo": "desvergonzar",
    "desvestid": "desvestir",
    "desvestida": "desvestir",
    "desvestidas": "desvestir",
    "desvestido": "desvestir",
    "desvestidos": "desvestir",
    "desvestimos": "desvestir",
    "desvestiremos": "desvestir",
    "desvestirá": "desvestir",
    "desvestirán": "desvestir",
    "desvestirás": "desvestir",
    "desvestiré": "desvestir",
    "desvestiréis": "desvestir",
    "desvestiría": "desvestir",
    "desvestiríais": "desvestir",
    "desvestiríamos": "desvestir",
    "desvestirían": "desvestir",
    "desvestirías": "desvestir",
    "desvestiste": "desvestir",
    "desvestisteis": "desvestir",
    "desvestí": "desvestir",
    "desvestía": "desvestir",
    "desvestíais": "desvestir",
    "desvestíamos": "desvestir",
    "desvestían": "desvestir",
    "desvestías": "desvestir",
    "desvestís": "desvestir",
    "desviaba": "desviar",
    "desviabais": "desviar",
    "desviaban": "desviar",
    "desviabas": "desviar",
    "desviaciones": "desviación",
    "desviacionismos": "desviacionismo",
    "desviacionistas": "desviacionista",
    "desviad": "desviar",
    "desviada": "desviar",
    "desviadas": "desviar",
    "desviado": "desviar",
    "desviadora": "desviador",
    "desviadoras": "desviador",
    "desviadores": "desviador",
    "desviados": "desviar",
    "desviamos": "desviar",
    "desviando": "desviar",
    "desviara": "desviar",
    "desviarais": "desviar",
    "desviaran": "desviar",
    "desviaras": "desviar",
    "desviare": "desviar",
    "desviareis": "desviar",
    "desviaremos": "desviar",
    "desviaren": "desviar",
    "desviares": "desviar",
    "desviaron": "desviar",
    "desviará": "desviar",
    "desviarán": "desviar",
    "desviarás": "desviar",
    "desviaré": "desviar",
    "desviaréis": "desviar",
    "desviaría": "desviar",
    "desviaríais": "desviar",
    "desviaríamos": "desviar",
    "desviarían": "desviar",
    "desviarías": "desviar",
    "desviase": "desviar",
    "desviaseis": "desviar",
    "desviasen": "desviar",
    "desviases": "desviar",
    "desviaste": "desviar",
    "desviasteis": "desviar",
    "desviatada": "desviatado",
    "desviatadas": "desviatado",
    "desviatados": "desviatado",
    "desviejes": "desvieje",
    "desviemos": "desviar",
    "desvincula": "desvincular",
    "desvinculaba": "desvincular",
    "desvinculabais": "desvincular",
    "desvinculaban": "desvincular",
    "desvinculabas": "desvincular",
    "desvinculaciones": "desvinculación",
    "desvinculad": "desvincular",
    "desvinculada": "desvincular",
    "desvinculadas": "desvincular",
    "desvinculado": "desvincular",
    "desvinculados": "desvincular",
    "desvinculamos": "desvincular",
    "desvinculan": "desvincular",
    "desvinculando": "desvincular",
    "desvinculara": "desvincular",
    "desvincularais": "desvincular",
    "desvincularan": "desvincular",
    "desvincularas": "desvincular",
    "desvinculare": "desvincular",
    "desvinculareis": "desvincular",
    "desvincularemos": "desvincular",
    "desvincularen": "desvincular",
    "desvinculares": "desvincular",
    "desvincularon": "desvincular",
    "desvinculará": "desvincular",
    "desvincularán": "desvincular",
    "desvincularás": "desvincular",
    "desvincularé": "desvincular",
    "desvincularéis": "desvincular",
    "desvincularía": "desvincular",
    "desvincularíais": "desvincular",
    "desvincularíamos": "desvincular",
    "desvincularían": "desvincular",
    "desvincularías": "desvincular",
    "desvinculas": "desvincular",
    "desvinculase": "desvincular",
    "desvinculaseis": "desvincular",
    "desvinculasen": "desvincular",
    "desvinculases": "desvincular",
    "desvinculaste": "desvincular",
    "desvinculasteis": "desvincular",
    "desvincule": "desvincular",
    "desvinculemos": "desvincular",
    "desvinculen": "desvincular",
    "desvincules": "desvincular",
    "desvinculo": "desvincular",
    "desvinculábamos": "desvincular",
    "desvinculáis": "desvincular",
    "desvinculáramos": "desvincular",
    "desvinculáremos": "desvincular",
    "desvinculásemos": "desvincular",
    "desvinculé": "desvincular",
    "desvinculéis": "desvincular",
    "desvinculó": "desvincular",
    "desvira": "desvirar",
    "desviraba": "desvirar",
    "desvirabais": "desvirar",
    "desviraban": "desvirar",
    "desvirabas": "desvirar",
    "desvirad": "desvirar",
    "desvirada": "desvirar",
    "desviradas": "desvirar",
    "desvirado": "desvirar",
    "desvirados": "desvirar",
    "desviramos": "desvirar",
    "desviran": "desvirar",
    "desvirando": "desvirar",
    "desvirara": "desvirar",
    "desvirarais": "desvirar",
    "desviraran": "desvirar",
    "desviraras": "desvirar",
    "desvirare": "desvirar",
    "desvirareis": "desvirar",
    "desviraremos": "desvirar",
    "desviraren": "desvirar",
    "desvirares": "desvirar",
    "desviraron": "desvirar",
    "desvirará": "desvirar",
    "desvirarán": "desvirar",
    "desvirarás": "desvirar",
    "desviraré": "desvirar",
    "desviraréis": "desvirar",
    "desviraría": "desvirar",
    "desviraríais": "desvirar",
    "desviraríamos": "desvirar",
    "desvirarían": "desvirar",
    "desvirarías": "desvirar",
    "desviras": "desvirar",
    "desvirase": "desvirar",
    "desviraseis": "desvirar",
    "desvirasen": "desvirar",
    "desvirases": "desvirar",
    "desviraste": "desvirar",
    "desvirasteis": "desvirar",
    "desvire": "desvirar",
    "desviremos": "desvirar",
    "desviren": "desvirar",
    "desvires": "desvirar",
    "desvirga": "desvirgar",
    "desvirgaba": "desvirgar",
    "desvirgabais": "desvirgar",
    "desvirgaban": "desvirgar",
    "desvirgabas": "desvirgar",
    "desvirgad": "desvirgar",
    "desvirgada": "desvirgar",
    "desvirgadas": "desvirgar",
    "desvirgado": "desvirgar",
    "desvirgados": "desvirgar",
    "desvirgamientos": "desvirgamiento",
    "desvirgamos": "desvirgar",
    "desvirgan": "desvirgar",
    "desvirgando": "desvirgar",
    "desvirgara": "desvirgar",
    "desvirgarais": "desvirgar",
    "desvirgaran": "desvirgar",
    "desvirgaras": "desvirgar",
    "desvirgare": "desvirgar",
    "desvirgareis": "desvirgar",
    "desvirgaremos": "desvirgar",
    "desvirgaren": "desvirgar",
    "desvirgares": "desvirgar",
    "desvirgaron": "desvirgar",
    "desvirgará": "desvirgar",
    "desvirgarán": "desvirgar",
    "desvirgarás": "desvirgar",
    "desvirgaré": "desvirgar",
    "desvirgaréis": "desvirgar",
    "desvirgaría": "desvirgar",
    "desvirgaríais": "desvirgar",
    "desvirgaríamos": "desvirgar",
    "desvirgarían": "desvirgar",
    "desvirgarías": "desvirgar",
    "desvirgas": "desvirgar",
    "desvirgase": "desvirgar",
    "desvirgaseis": "desvirgar",
    "desvirgasen": "desvirgar",
    "desvirgases": "desvirgar",
    "desvirgaste": "desvirgar",
    "desvirgasteis": "desvirgar",
    "desvirgo": "desvirgar",
    "desvirgue": "desvirgar",
    "desvirguemos": "desvirgar",
    "desvirguen": "desvirgar",
    "desvirgues": "desvirgar",
    "desvirgué": "desvirgar",
    "desvirguéis": "desvirgar",
    "desvirgábamos": "desvirgar",
    "desvirgáis": "desvirgar",
    "desvirgáramos": "desvirgar",
    "desvirgáremos": "desvirgar",
    "desvirgásemos": "desvirgar",
    "desvirgó": "desvirgar",
    "desviro": "desvirar",
    "desvirtuaba": "desvirtuar",
    "desvirtuabais": "desvirtuar",
    "desvirtuaban": "desvirtuar",
    "desvirtuabas": "desvirtuar",
    "desvirtuaciones": "desvirtuación",
    "desvirtuad": "desvirtuar",
    "desvirtuada": "desvirtuar",
    "desvirtuadas": "desvirtuar",
    "desvirtuado": "desvirtuar",
    "desvirtuados": "desvirtuar",
    "desvirtuamos": "desvirtuar",
    "desvirtuando": "desvirtuar",
    "desvirtuara": "desvirtuar",
    "desvirtuarais": "desvirtuar",
    "desvirtuaran": "desvirtuar",
    "desvirtuaras": "desvirtuar",
    "desvirtuare": "desvirtuar",
    "desvirtuareis": "desvirtuar",
    "desvirtuaremos": "desvirtuar",
    "desvirtuaren": "desvirtuar",
    "desvirtuares": "desvirtuar",
    "desvirtuaron": "desvirtuar",
    "desvirtuará": "desvirtuar",
    "desvirtuarán": "desvirtuar",
    "desvirtuarás": "desvirtuar",
    "desvirtuaré": "desvirtuar",
    "desvirtuaréis": "desvirtuar",
    "desvirtuaría": "desvirtuar",
    "desvirtuaríais": "desvirtuar",
    "desvirtuaríamos": "desvirtuar",
    "desvirtuarían": "desvirtuar",
    "desvirtuarías": "desvirtuar",
    "desvirtuase": "desvirtuar",
    "desvirtuaseis": "desvirtuar",
    "desvirtuasen": "desvirtuar",
    "desvirtuases": "desvirtuar",
    "desvirtuaste": "desvirtuar",
    "desvirtuasteis": "desvirtuar",
    "desvirtuemos": "desvirtuar",
    "desvirtuábamos": "desvirtuar",
    "desvirtuáis": "desvirtuar",
    "desvirtuáramos": "desvirtuar",
    "desvirtuáremos": "desvirtuar",
    "desvirtuásemos": "desvirtuar",
    "desvirtué": "desvirtuar",
    "desvirtuéis": "desvirtuar",
    "desvirtuó": "desvirtuar",
    "desvirtúa": "desvirtuar",
    "desvirtúan": "desvirtuar",
    "desvirtúas": "desvirtuar",
    "desvirtúe": "desvirtuar",
    "desvirtúen": "desvirtuar",
    "desvirtúes": "desvirtuar",
    "desvirtúo": "desvirtuar",
    "desvirábamos": "desvirar",
    "desviráis": "desvirar",
    "desviráramos": "desvirar",
    "desviráremos": "desvirar",
    "desvirásemos": "desvirar",
    "desviré": "desvirar",
    "desviréis": "desvirar",
    "desviró": "desvirar",
    "desvista": "desvestir",
    "desvistamos": "desvestir",
    "desvistan": "desvestir",
    "desvistas": "desvestir",
    "desviste": "desvestir",
    "desvisten": "desvestir",
    "desvistes": "desvestir",
    "desvistiendo": "desvestir",
    "desvistiera": "desvestir",
    "desvistierais": "desvestir",
    "desvistieran": "desvestir",
    "desvistieras": "desvestir",
    "desvistiere": "desvestir",
    "desvistiereis": "desvestir",
    "desvistieren": "desvestir",
    "desvistieres": "desvestir",
    "desvistieron": "desvestir",
    "desvistiese": "desvestir",
    "desvistieseis": "desvestir",
    "desvistiesen": "desvestir",
    "desvistieses": "desvestir",
    "desvistiéramos": "desvestir",
    "desvistiéremos": "desvestir",
    "desvistiésemos": "desvestir",
    "desvistió": "desvestir",
    "desvisto": "desvestir",
    "desvistáis": "desvestir",
    "desvitalice": "desvitalizar",
    "desvitalicemos": "desvitalizar",
    "desvitalicen": "desvitalizar",
    "desvitalices": "desvitalizar",
    "desvitalicé": "desvitalizar",
    "desvitalicéis": "desvitalizar",
    "desvitaliza": "desvitalizar",
    "desvitalizaba": "desvitalizar",
    "desvitalizabais": "desvitalizar",
    "desvitalizaban": "desvitalizar",
    "desvitalizabas": "desvitalizar",
    "desvitalizaciones": "desvitalización",
    "desvitalizad": "desvitalizar",
    "desvitalizada": "desvitalizar",
    "desvitalizadas": "desvitalizar",
    "desvitalizado": "desvitalizar",
    "desvitalizados": "desvitalizar",
    "desvitalizamos": "desvitalizar",
    "desvitalizan": "desvitalizar",
    "desvitalizando": "desvitalizar",
    "desvitalizara": "desvitalizar",
    "desvitalizarais": "desvitalizar",
    "desvitalizaran": "desvitalizar",
    "desvitalizaras": "desvitalizar",
    "desvitalizare": "desvitalizar",
    "desvitalizareis": "desvitalizar",
    "desvitalizaremos": "desvitalizar",
    "desvitalizaren": "desvitalizar",
    "desvitalizares": "desvitalizar",
    "desvitalizaron": "desvitalizar",
    "desvitalizará": "desvitalizar",
    "desvitalizarán": "desvitalizar",
    "desvitalizarás": "desvitalizar",
    "desvitalizaré": "desvitalizar",
    "desvitalizaréis": "desvitalizar",
    "desvitalizaría": "desvitalizar",
    "desvitalizaríais": "desvitalizar",
    "desvitalizaríamos": "desvitalizar",
    "desvitalizarían": "desvitalizar",
    "desvitalizarías": "desvitalizar",
    "desvitalizas": "desvitalizar",
    "desvitalizase": "desvitalizar",
    "desvitalizaseis": "desvitalizar",
    "desvitalizasen": "desvitalizar",
    "desvitalizases": "desvitalizar",
    "desvitalizaste": "desvitalizar",
    "desvitalizasteis": "desvitalizar",
    "desvitalizo": "desvitalizar",
    "desvitalizábamos": "desvitalizar",
    "desvitalizáis": "desvitalizar",
    "desvitalizáramos": "desvitalizar",
    "desvitalizáremos": "desvitalizar",
    "desvitalizásemos": "desvitalizar",
    "desvitalizó": "desvitalizar",
    "desvitrifica": "desvitrificar",
    "desvitrificaba": "desvitrificar",
    "desvitrificabais": "desvitrificar",
    "desvitrificaban": "desvitrificar",
    "desvitrificabas": "desvitrificar",
    "desvitrificaciones": "desvitrificación",
    "desvitrificad": "desvitrificar",
    "desvitrificada": "desvitrificar",
    "desvitrificadas": "desvitrificar",
    "desvitrificado": "desvitrificar",
    "desvitrificados": "desvitrificar",
    "desvitrificamos": "desvitrificar",
    "desvitrifican": "desvitrificar",
    "desvitrificando": "desvitrificar",
    "desvitrificara": "desvitrificar",
    "desvitrificarais": "desvitrificar",
    "desvitrificaran": "desvitrificar",
    "desvitrificaras": "desvitrificar",
    "desvitrificare": "desvitrificar",
    "desvitrificareis": "desvitrificar",
    "desvitrificaremos": "desvitrificar",
    "desvitrificaren": "desvitrificar",
    "desvitrificares": "desvitrificar",
    "desvitrificaron": "desvitrificar",
    "desvitrificará": "desvitrificar",
    "desvitrificarán": "desvitrificar",
    "desvitrificarás": "desvitrificar",
    "desvitrificaré": "desvitrificar",
    "desvitrificaréis": "desvitrificar",
    "desvitrificaría": "desvitrificar",
    "desvitrificaríais": "desvitrificar",
    "desvitrificaríamos": "desvitrificar",
    "desvitrificarían": "desvitrificar",
    "desvitrificarías": "desvitrificar",
    "desvitrificas": "desvitrificar",
    "desvitrificase": "desvitrificar",
    "desvitrificaseis": "desvitrificar",
    "desvitrificasen": "desvitrificar",
    "desvitrificases": "desvitrificar",
    "desvitrificaste": "desvitrificar",
    "desvitrificasteis": "desvitrificar",
    "desvitrifico": "desvitrificar",
    "desvitrificábamos": "desvitrificar",
    "desvitrificáis": "desvitrificar",
    "desvitrificáramos": "desvitrificar",
    "desvitrificáremos": "desvitrificar",
    "desvitrificásemos": "desvitrificar",
    "desvitrificó": "desvitrificar",
    "desvitrifique": "desvitrificar",
    "desvitrifiquemos": "desvitrificar",
    "desvitrifiquen": "desvitrificar",
    "desvitrifiques": "desvitrificar",
    "desvitrifiqué": "desvitrificar",
    "desvitrifiquéis": "desvitrificar",
    "desviva": "desvivir",
    "desvivamos": "desvivir",
    "desvivan": "desvivir",
    "desvivas": "desvivir",
    "desvive": "desvivir",
    "desviven": "desvivir",
    "desvives": "desvivir",
    "desvivid": "desvivir",
    "desvivida": "desvivir",
    "desvividas": "desvivir",
    "desvivido": "desvivir",
    "desvividos": "desvivir",
    "desviviendo": "desvivir",
    "desviviera": "desvivir",
    "desvivierais": "desvivir",
    "desvivieran": "desvivir",
    "desvivieras": "desvivir",
    "desviviere": "desvivir",
    "desviviereis": "desvivir",
    "desvivieren": "desvivir",
    "desvivieres": "desvivir",
    "desvivieron": "desvivir",
    "desviviese": "desvivir",
    "desvivieseis": "desvivir",
    "desviviesen": "desvivir",
    "desvivieses": "desvivir",
    "desvivimos": "desvivir",
    "desviviremos": "desvivir",
    "desvivirá": "desvivir",
    "desvivirán": "desvivir",
    "desvivirás": "desvivir",
    "desviviré": "desvivir",
    "desviviréis": "desvivir",
    "desviviría": "desvivir",
    "desviviríais": "desvivir",
    "desviviríamos": "desvivir",
    "desvivirían": "desvivir",
    "desvivirías": "desvivir",
    "desviviste": "desvivir",
    "desvivisteis": "desvivir",
    "desviviéramos": "desvivir",
    "desviviéremos": "desvivir",
    "desviviésemos": "desvivir",
    "desvivió": "desvivir",
    "desvivo": "desvivir",
    "desviváis": "desvivir",
    "desviví": "desvivir",
    "desvivía": "desvivir",
    "desvivíais": "desvivir",
    "desvivíamos": "desvivir",
    "desvivían": "desvivir",
    "desvivías": "desvivir",
    "desvivís": "desvivir",
    "desviábamos": "desviar",
    "desviáis": "desviar",
    "desviáramos": "desviar",
    "desviáremos": "desviar",
    "desviásemos": "desviar",
    "desvié": "desviar",
    "desviéis": "desviar",
    "desvió": "desviar",
    "desvolcana": "desvolcanar",
    "desvolcanaba": "desvolcanar",
    "desvolcanabais": "desvolcanar",
    "desvolcanaban": "desvolcanar",
    "desvolcanabas": "desvolcanar",
    "desvolcanad": "desvolcanar",
    "desvolcanada": "desvolcanar",
    "desvolcanadas": "desvolcanar",
    "desvolcanado": "desvolcanar",
    "desvolcanados": "desvolcanar",
    "desvolcanamos": "desvolcanar",
    "desvolcanan": "desvolcanar",
    "desvolcanando": "desvolcanar",
    "desvolcanara": "desvolcanar",
    "desvolcanarais": "desvolcanar",
    "desvolcanaran": "desvolcanar",
    "desvolcanaras": "desvolcanar",
    "desvolcanare": "desvolcanar",
    "desvolcanareis": "desvolcanar",
    "desvolcanaremos": "desvolcanar",
    "desvolcanaren": "desvolcanar",
    "desvolcanares": "desvolcanar",
    "desvolcanaron": "desvolcanar",
    "desvolcanará": "desvolcanar",
    "desvolcanarán": "desvolcanar",
    "desvolcanarás": "desvolcanar",
    "desvolcanaré": "desvolcanar",
    "desvolcanaréis": "desvolcanar",
    "desvolcanaría": "desvolcanar",
    "desvolcanaríais": "desvolcanar",
    "desvolcanaríamos": "desvolcanar",
    "desvolcanarían": "desvolcanar",
    "desvolcanarías": "desvolcanar",
    "desvolcanas": "desvolcanar",
    "desvolcanase": "desvolcanar",
    "desvolcanaseis": "desvolcanar",
    "desvolcanasen": "desvolcanar",
    "desvolcanases": "desvolcanar",
    "desvolcanaste": "desvolcanar",
    "desvolcanasteis": "desvolcanar",
    "desvolcane": "desvolcanar",
    "desvolcanemos": "desvolcanar",
    "desvolcanen": "desvolcanar",
    "desvolcanes": "desvolcanar",
    "desvolcano": "desvolcanar",
    "desvolcanábamos": "desvolcanar",
    "desvolcanáis": "desvolcanar",
    "desvolcanáramos": "desvolcanar",
    "desvolcanáremos": "desvolcanar",
    "desvolcanásemos": "desvolcanar",
    "desvolcané": "desvolcanar",
    "desvolcanéis": "desvolcanar",
    "desvolcanó": "desvolcanar",
    "desvulcanizaciones": "desvulcanización",
    "desvía": "desviar",
    "desvían": "desviar",
    "desvías": "desviar",
    "desvíe": "desviar",
    "desvíen": "desviar",
    "desvíes": "desviar",
    "desvío": "desviar",
    "desvíos": "desvío",
    "desyema": "desyemar",
    "desyemaba": "desyemar",
    "desyemabais": "desyemar",
    "desyemaban": "desyemar",
    "desyemabas": "desyemar",
    "desyemad": "desyemar",
    "desyemada": "desyemar",
    "desyemadas": "desyemar",
    "desyemado": "desyemar",
    "desyemados": "desyemar",
    "desyemamos": "desyemar",
    "desyeman": "desyemar",
    "desyemando": "desyemar",
    "desyemara": "desyemar",
    "desyemarais": "desyemar",
    "desyemaran": "desyemar",
    "desyemaras": "desyemar",
    "desyemare": "desyemar",
    "desyemareis": "desyemar",
    "desyemaremos": "desyemar",
    "desyemaren": "desyemar",
    "desyemares": "desyemar",
    "desyemaron": "desyemar",
    "desyemará": "desyemar",
    "desyemarán": "desyemar",
    "desyemarás": "desyemar",
    "desyemaré": "desyemar",
    "desyemaréis": "desyemar",
    "desyemaría": "desyemar",
    "desyemaríais": "desyemar",
    "desyemaríamos": "desyemar",
    "desyemarían": "desyemar",
    "desyemarías": "desyemar",
    "desyemas": "desyemar",
    "desyemase": "desyemar",
    "desyemaseis": "desyemar",
    "desyemasen": "desyemar",
    "desyemases": "desyemar",
    "desyemaste": "desyemar",
    "desyemasteis": "desyemar",
    "desyeme": "desyemar",
    "desyememos": "desyemar",
    "desyemen": "desyemar",
    "desyemes": "desyemar",
    "desyemo": "desyemar",
    "desyemábamos": "desyemar",
    "desyemáis": "desyemar",
    "desyemáramos": "desyemar",
    "desyemáremos": "desyemar",
    "desyemásemos": "desyemar",
    "desyemé": "desyemar",
    "desyeméis": "desyemar",
    "desyemó": "desyemar",
    "desyerba": "desyerbar",
    "desyerbaba": "desyerbar",
    "desyerbabais": "desyerbar",
    "desyerbaban": "desyerbar",
    "desyerbabas": "desyerbar",
    "desyerbad": "desyerbar",
    "desyerbada": "desyerbar",
    "desyerbadas": "desyerbar",
    "desyerbado": "desyerbar",
    "desyerbadora": "desyerbador",
    "desyerbadoras": "desyerbador",
    "desyerbadores": "desyerbador",
    "desyerbados": "desyerbar",
    "desyerbamos": "desyerbar",
    "desyerban": "desyerbar",
    "desyerbando": "desyerbar",
    "desyerbara": "desyerbar",
    "desyerbarais": "desyerbar",
    "desyerbaran": "desyerbar",
    "desyerbaras": "desyerbar",
    "desyerbare": "desyerbar",
    "desyerbareis": "desyerbar",
    "desyerbaremos": "desyerbar",
    "desyerbaren": "desyerbar",
    "desyerbares": "desyerbar",
    "desyerbaron": "desyerbar",
    "desyerbará": "desyerbar",
    "desyerbarán": "desyerbar",
    "desyerbarás": "desyerbar",
    "desyerbaré": "desyerbar",
    "desyerbaréis": "desyerbar",
    "desyerbaría": "desyerbar",
    "desyerbaríais": "desyerbar",
    "desyerbaríamos": "desyerbar",
    "desyerbarían": "desyerbar",
    "desyerbarías": "desyerbar",
    "desyerbas": "desyerbar",
    "desyerbase": "desyerbar",
    "desyerbaseis": "desyerbar",
    "desyerbasen": "desyerbar",
    "desyerbases": "desyerbar",
    "desyerbaste": "desyerbar",
    "desyerbasteis": "desyerbar",
    "desyerbe": "desyerbar",
    "desyerbemos": "desyerbar",
    "desyerben": "desyerbar",
    "desyerbes": "desyerbar",
    "desyerbo": "desyerbar",
    "desyerbábamos": "desyerbar",
    "desyerbáis": "desyerbar",
    "desyerbáramos": "desyerbar",
    "desyerbáremos": "desyerbar",
    "desyerbásemos": "desyerbar",
    "desyerbé": "desyerbar",
    "desyerbéis": "desyerbar",
    "desyerbó": "desyerbar",
    "desyuga": "desyugar",
    "desyugaba": "desyugar",
    "desyugabais": "desyugar",
    "desyugaban": "desyugar",
    "desyugabas": "desyugar",
    "desyugad": "desyugar",
    "desyugada": "desyugar",
    "desyugadas": "desyugar",
    "desyugado": "desyugar",
    "desyugados": "desyugar",
    "desyugamos": "desyugar",
    "desyugan": "desyugar",
    "desyugando": "desyugar",
    "desyugara": "desyugar",
    "desyugarais": "desyugar",
    "desyugaran": "desyugar",
    "desyugaras": "desyugar",
    "desyugare": "desyugar",
    "desyugareis": "desyugar",
    "desyugaremos": "desyugar",
    "desyugaren": "desyugar",
    "desyugares": "desyugar",
    "desyugaron": "desyugar",
    "desyugará": "desyugar",
    "desyugarán": "desyugar",
    "desyugarás": "desyugar",
    "desyugaré": "desyugar",
    "desyugaréis": "desyugar",
    "desyugaría": "desyugar",
    "desyugaríais": "desyugar",
    "desyugaríamos": "desyugar",
    "desyugarían": "desyugar",
    "desyugarías": "desyugar",
    "desyugas": "desyugar",
    "desyugase": "desyugar",
    "desyugaseis": "desyugar",
    "desyugasen": "desyugar",
    "desyugases": "desyugar",
    "desyugaste": "desyugar",
    "desyugasteis": "desyugar",
    "desyugo": "desyugar",
    "desyugue": "desyugar",
    "desyuguemos": "desyugar",
    "desyuguen": "desyugar",
    "desyugues": "desyugar",
    "desyugué": "desyugar",
    "desyuguéis": "desyugar",
    "desyugábamos": "desyugar",
    "desyugáis": "desyugar",
    "desyugáramos": "desyugar",
    "desyugáremos": "desyugar",
    "desyugásemos": "desyugar",
    "desyugó": "desyugar",
    "deszulaca": "deszulacar",
    "deszulacaba": "deszulacar",
    "deszulacabais": "deszulacar",
    "deszulacaban": "deszulacar",
    "deszulacabas": "deszulacar",
    "deszulacad": "deszulacar",
    "deszulacada": "deszulacar",
    "deszulacadas": "deszulacar",
    "deszulacado": "deszulacar",
    "deszulacados": "deszulacar",
    "deszulacamos": "deszulacar",
    "deszulacan": "deszulacar",
    "deszulacando": "deszulacar",
    "deszulacara": "deszulacar",
    "deszulacarais": "deszulacar",
    "deszulacaran": "deszulacar",
    "deszulacaras": "deszulacar",
    "deszulacare": "deszulacar",
    "deszulacareis": "deszulacar",
    "deszulacaremos": "deszulacar",
    "deszulacaren": "deszulacar",
    "deszulacares": "deszulacar",
    "deszulacaron": "deszulacar",
    "deszulacará": "deszulacar",
    "deszulacarán": "deszulacar",
    "deszulacarás": "deszulacar",
    "deszulacaré": "deszulacar",
    "deszulacaréis": "deszulacar",
    "deszulacaría": "deszulacar",
    "deszulacaríais": "deszulacar",
    "deszulacaríamos": "deszulacar",
    "deszulacarían": "deszulacar",
    "deszulacarías": "deszulacar",
    "deszulacas": "deszulacar",
    "deszulacase": "deszulacar",
    "deszulacaseis": "deszulacar",
    "deszulacasen": "deszulacar",
    "deszulacases": "deszulacar",
    "deszulacaste": "deszulacar",
    "deszulacasteis": "deszulacar",
    "deszulaco": "deszulacar",
    "deszulacábamos": "deszulacar",
    "deszulacáis": "deszulacar",
    "deszulacáramos": "deszulacar",
    "deszulacáremos": "deszulacar",
    "deszulacásemos": "deszulacar",
    "deszulacó": "deszulacar",
    "deszulaque": "deszulacar",
    "deszulaquemos": "deszulacar",
    "deszulaquen": "deszulacar",
    "deszulaques": "deszulacar",
    "deszulaqué": "deszulacar",
    "deszulaquéis": "deszulacar",
    "deszuma": "deszumar",
    "deszumaba": "deszumar",
    "deszumabais": "deszumar",
    "deszumaban": "deszumar",
    "deszumabas": "deszumar",
    "deszumad": "deszumar",
    "deszumada": "deszumar",
    "deszumadas": "deszumar",
    "deszumado": "deszumar",
    "deszumados": "deszumar",
    "deszumamos": "deszumar",
    "deszuman": "deszumar",
    "deszumando": "deszumar",
    "deszumara": "deszumar",
    "deszumarais": "deszumar",
    "deszumaran": "deszumar",
    "deszumaras": "deszumar",
    "deszumare": "deszumar",
    "deszumareis": "deszumar",
    "deszumaremos": "deszumar",
    "deszumaren": "deszumar",
    "deszumares": "deszumar",
    "deszumaron": "deszumar",
    "deszumará": "deszumar",
    "deszumarán": "deszumar",
    "deszumarás": "deszumar",
    "deszumaré": "deszumar",
    "deszumaréis": "deszumar",
    "deszumaría": "deszumar",
    "deszumaríais": "deszumar",
    "deszumaríamos": "deszumar",
    "deszumarían": "deszumar",
    "deszumarías": "deszumar",
    "deszumas": "deszumar",
    "deszumase": "deszumar",
    "deszumaseis": "deszumar",
    "deszumasen": "deszumar",
    "deszumases": "deszumar",
    "deszumaste": "deszumar",
    "deszumasteis": "deszumar",
    "deszume": "deszumar",
    "deszumemos": "deszumar",
    "deszumen": "deszumar",
    "deszumes": "deszumar",
    "deszumo": "deszumar",
    "deszumábamos": "deszumar",
    "deszumáis": "deszumar",
    "deszumáramos": "deszumar",
    "deszumáremos": "deszumar",
    "deszumásemos": "deszumar",
    "deszumé": "deszumar",
    "deszuméis": "deszumar",
    "deszumó": "deszumar",
    "desánimos": "desánimo",
    "desértica": "desértico",
    "desérticas": "desértico",
    "desérticos": "desértico",
    "desórdenes": "desorden",
    "detalla": "detallar",
    "detallaba": "detallar",
    "detallabais": "detallar",
    "detallaban": "detallar",
    "detallabas": "detallar",
    "detallad": "detallar",
    "detallada": "detallar",
    "detalladas": "detallar",
    "detallado": "detallar",
    "detallados": "detallar",
    "detallamos": "detallar",
    "detallan": "detallar",
    "detallando": "detallar",
    "detallara": "detallar",
    "detallarais": "detallar",
    "detallaran": "detallar",
    "detallaras": "detallar",
    "detallare": "detallar",
    "detallareis": "detallar",
    "detallaremos": "detallar",
    "detallaren": "detallar",
    "detallares": "detallar",
    "detallaron": "detallar",
    "detallará": "detallar",
    "detallarán": "detallar",
    "detallarás": "detallar",
    "detallaré": "detallar",
    "detallaréis": "detallar",
    "detallaría": "detallar",
    "detallaríais": "detallar",
    "detallaríamos": "detallar",
    "detallarían": "detallar",
    "detallarías": "detallar",
    "detallas": "detallar",
    "detallase": "detallar",
    "detallaseis": "detallar",
    "detallasen": "detallar",
    "detallases": "detallar",
    "detallaste": "detallar",
    "detallasteis": "detallar",
    "detalle": "detallar",
    "detallemos": "detallar",
    "detallen": "detallar",
    "detalles": "detalle",
    "detallistas": "detallista",
    "detallo": "detallar",
    "detallábamos": "detallar",
    "detalláis": "detallar",
    "detalláramos": "detallar",
    "detalláremos": "detallar",
    "detallásemos": "detallar",
    "detallé": "detallar",
    "detalléis": "detallar",
    "detalló": "detallar",
    "detasas": "detasa",
    "detecciones": "detección",
    "detecta": "detectar",
    "detectaba": "detectar",
    "detectabais": "detectar",
    "detectaban": "detectar",
    "detectabas": "detectar",
    "detectables": "detectable",
    "detectad": "detectar",
    "detectada": "detectar",
    "detectadas": "detectar",
    "detectado": "detectar",
    "detectados": "detectar",
    "detectamos": "detectar",
    "detectan": "detectar",
    "detectando": "detectar",
    "detectara": "detectar",
    "detectarais": "detectar",
    "detectaran": "detectar",
    "detectaras": "detectar",
    "detectare": "detectar",
    "detectareis": "detectar",
    "detectaremos": "detectar",
    "detectaren": "detectar",
    "detectares": "detectar",
    "detectaron": "detectar",
    "detectará": "detectar",
    "detectarán": "detectar",
    "detectarás": "detectar",
    "detectaré": "detectar",
    "detectaréis": "detectar",
    "detectaría": "detectar",
    "detectaríais": "detectar",
    "detectaríamos": "detectar",
    "detectarían": "detectar",
    "detectarías": "detectar",
    "detectas": "detectar",
    "detectase": "detectar",
    "detectaseis": "detectar",
    "detectasen": "detectar",
    "detectases": "detectar",
    "detectaste": "detectar",
    "detectasteis": "detectar",
    "detecte": "detectar",
    "detectemos": "detectar",
    "detecten": "detectar",
    "detectes": "detectar",
    "detectives": "detective",
    "detectivesca": "detectivesco",
    "detectivescas": "detectivesco",
    "detectivescos": "detectivesco",
    "detecto": "detectar",
    "detectora": "detector",
    "detectoras": "detector",
    "detectores": "detector",
    "detectábamos": "detectar",
    "detectáis": "detectar",
    "detectáramos": "detectar",
    "detectáremos": "detectar",
    "detectásemos": "detectar",
    "detecté": "detectar",
    "detectéis": "detectar",
    "detectó": "detectar",
    "detenciones": "detención",
    "detendremos": "detener",
    "detendrá": "detener",
    "detendrán": "detener",
    "detendrás": "detener",
    "detendré": "detener",
    "detendréis": "detener",
    "detendría": "detener",
    "detendríais": "detener",
    "detendríamos": "detener",
    "detendrían": "detener",
    "detendrías": "detener",
    "detened": "detener",
    "detenemos": "detener",
    "detenga": "detener",
    "detengamos": "detener",
    "detengan": "detener",
    "detengas": "detener",
    "detengo": "detener",
    "detengáis": "detener",
    "detenida": "detenido",
    "detenidas": "detenido",
    "detenido": "detener",
    "detenidos": "detenido",
    "deteniendo": "detener",
    "detenimientos": "detenimiento",
    "detenta": "detentar",
    "detentaba": "detentar",
    "detentabais": "detentar",
    "detentaban": "detentar",
    "detentabas": "detentar",
    "detentaciones": "detentación",
    "detentad": "detentar",
    "detentada": "detentar",
    "detentadas": "detentar",
    "detentado": "detentar",
    "detentadora": "detentador",
    "detentadoras": "detentador",
    "detentadores": "detentador",
    "detentados": "detentar",
    "detentamos": "detentar",
    "detentan": "detentar",
    "detentando": "detentar",
    "detentara": "detentar",
    "detentarais": "detentar",
    "detentaran": "detentar",
    "detentaras": "detentar",
    "detentare": "detentar",
    "detentareis": "detentar",
    "detentaremos": "detentar",
    "detentaren": "detentar",
    "detentares": "detentar",
    "detentaron": "detentar",
    "detentará": "detentar",
    "detentarán": "detentar",
    "detentarás": "detentar",
    "detentaré": "detentar",
    "detentaréis": "detentar",
    "detentaría": "detentar",
    "detentaríais": "detentar",
    "detentaríamos": "detentar",
    "detentarían": "detentar",
    "detentarías": "detentar",
    "detentas": "detentar",
    "detentase": "detentar",
    "detentaseis": "detentar",
    "detentasen": "detentar",
    "detentases": "detentar",
    "detentaste": "detentar",
    "detentasteis": "detentar",
    "detente": "detentar",
    "detentemos": "detentar",
    "detenten": "detentar",
    "detentes": "detente",
    "detento": "detentar",
    "detentora": "detentor",
    "detentoras": "detentor",
    "detentores": "detentor",
    "detentábamos": "detentar",
    "detentáis": "detentar",
    "detentáramos": "detentar",
    "detentáremos": "detentar",
    "detentásemos": "detentar",
    "detenté": "detentar",
    "detentéis": "detentar",
    "detentó": "detentar",
    "detenéis": "detener",
    "detenía": "detener",
    "deteníais": "detener",
    "deteníamos": "detener",
    "detenían": "detener",
    "detenías": "detener",
    "deterge": "deterger",
    "deterged": "deterger",
    "detergemos": "deterger",
    "detergen": "deterger",
    "detergentes": "detergente",
    "detergeremos": "deterger",
    "detergerá": "deterger",
    "detergerán": "deterger",
    "detergerás": "deterger",
    "detergeré": "deterger",
    "detergeréis": "deterger",
    "detergería": "deterger",
    "detergeríais": "deterger",
    "detergeríamos": "deterger",
    "detergerían": "deterger",
    "detergerías": "deterger",
    "deterges": "deterger",
    "detergida": "deterger",
    "detergidas": "deterger",
    "detergido": "deterger",
    "detergidos": "deterger",
    "detergiendo": "deterger",
    "detergiera": "deterger",
    "detergierais": "deterger",
    "detergieran": "deterger",
    "detergieras": "deterger",
    "detergiere": "deterger",
    "detergiereis": "deterger",
    "detergieren": "deterger",
    "detergieres": "deterger",
    "detergieron": "deterger",
    "detergiese": "deterger",
    "detergieseis": "deterger",
    "detergiesen": "deterger",
    "detergieses": "deterger",
    "detergimos": "deterger",
    "detergiste": "deterger",
    "detergisteis": "deterger",
    "detergiéramos": "deterger",
    "detergiéremos": "deterger",
    "detergiésemos": "deterger",
    "detergió": "deterger",
    "detergéis": "deterger",
    "detergí": "deterger",
    "detergía": "deterger",
    "detergíais": "deterger",
    "detergíamos": "deterger",
    "detergían": "deterger",
    "detergías": "deterger",
    "deteriora": "deteriorar",
    "deterioraba": "deteriorar",
    "deteriorabais": "deteriorar",
    "deterioraban": "deteriorar",
    "deteriorabas": "deteriorar",
    "deterioraciones": "deterioración",
    "deteriorad": "deteriorar",
    "deteriorada": "deteriorar",
    "deterioradas": "deteriorar",
    "deteriorado": "deteriorar",
    "deteriorados": "deteriorar",
    "deterioramientos": "deterioramiento",
    "deterioramos": "deteriorar",
    "deterioran": "deteriorar",
    "deteriorando": "deteriorar",
    "deteriorara": "deteriorar",
    "deteriorarais": "deteriorar",
    "deterioraran": "deteriorar",
    "deterioraras": "deteriorar",
    "deteriorare": "deteriorar",
    "deteriorareis": "deteriorar",
    "deterioraremos": "deteriorar",
    "deterioraren": "deteriorar",
    "deteriorares": "deteriorar",
    "deterioraron": "deteriorar",
    "deteriorará": "deteriorar",
    "deteriorarán": "deteriorar",
    "deteriorarás": "deteriorar",
    "deterioraré": "deteriorar",
    "deterioraréis": "deteriorar",
    "deterioraría": "deteriorar",
    "deterioraríais": "deteriorar",
    "deterioraríamos": "deteriorar",
    "deteriorarían": "deteriorar",
    "deteriorarías": "deteriorar",
    "deterioras": "deteriorar",
    "deteriorase": "deteriorar",
    "deterioraseis": "deteriorar",
    "deteriorasen": "deteriorar",
    "deteriorases": "deteriorar",
    "deterioraste": "deteriorar",
    "deteriorasteis": "deteriorar",
    "deteriore": "deteriorar",
    "deterioremos": "deteriorar",
    "deterioren": "deteriorar",
    "deteriores": "deteriorar",
    "deterioro": "deteriorar",
    "deterioros": "deterioro",
    "deteriorábamos": "deteriorar",
    "deterioráis": "deteriorar",
    "deterioráramos": "deteriorar",
    "deterioráremos": "deteriorar",
    "deteriorásemos": "deteriorar",
    "deterioré": "deteriorar",
    "deterioréis": "deteriorar",
    "deterioró": "deteriorar",
    "deterja": "deterger",
    "deterjamos": "deterger",
    "deterjan": "deterger",
    "deterjas": "deterger",
    "deterjo": "deterger",
    "deterjáis": "deterger",
    "determina": "determinar",
    "determinaba": "determinar",
    "determinabais": "determinar",
    "determinaban": "determinar",
    "determinabas": "determinar",
    "determinables": "determinable",
    "determinaciones": "determinación",
    "determinad": "determinar",
    "determinada": "determinar",
    "determinadas": "determinar",
    "determinado": "determinar",
    "determinados": "determinar",
    "determinamos": "determinar",
    "determinan": "determinar",
    "determinando": "determinar",
    "determinantes": "determinante",
    "determinara": "determinar",
    "determinarais": "determinar",
    "determinaran": "determinar",
    "determinaras": "determinar",
    "determinare": "determinar",
    "determinareis": "determinar",
    "determinaremos": "determinar",
    "determinaren": "determinar",
    "determinares": "determinar",
    "determinaron": "determinar",
    "determinará": "determinar",
    "determinarán": "determinar",
    "determinarás": "determinar",
    "determinaré": "determinar",
    "determinaréis": "determinar",
    "determinaría": "determinar",
    "determinaríais": "determinar",
    "determinaríamos": "determinar",
    "determinarían": "determinar",
    "determinarías": "determinar",
    "determinas": "determinar",
    "determinase": "determinar",
    "determinaseis": "determinar",
    "determinasen": "determinar",
    "determinases": "determinar",
    "determinaste": "determinar",
    "determinasteis": "determinar",
    "determinativa": "determinativo",
    "determinativas": "determinativo",
    "determinativos": "determinativo",
    "determine": "determinar",
    "determinemos": "determinar",
    "determinen": "determinar",
    "determines": "determinar",
    "determinismos": "determinismo",
    "deterministas": "determinista",
    "determino": "determinar",
    "determinábamos": "determinar",
    "determináis": "determinar",
    "determináramos": "determinar",
    "determináremos": "determinar",
    "determinásemos": "determinar",
    "determiné": "determinar",
    "determinéis": "determinar",
    "determinística": "determinístico",
    "determinísticas": "determinístico",
    "determinísticos": "determinístico",
    "determinó": "determinar",
    "detersiones": "detersión",
    "detersiva": "detersivo",
    "detersivas": "detersivo",
    "detersivos": "detersivo",
    "detersoria": "detersorio",
    "detersorias": "detersorio",
    "detersorios": "detersorio",
    "detesta": "detestar",
    "detestaba": "detestar",
    "detestabais": "detestar",
    "detestaban": "detestar",
    "detestabas": "detestar",
    "detestables": "detestable",
    "detestaciones": "detestación",
    "detestad": "detestar",
    "detestada": "detestar",
    "detestadas": "detestar",
    "detestado": "detestar",
    "detestados": "detestar",
    "detestamos": "detestar",
    "detestan": "detestar",
    "detestando": "detestar",
    "detestara": "detestar",
    "detestarais": "detestar",
    "detestaran": "detestar",
    "detestaras": "detestar",
    "detestare": "detestar",
    "detestareis": "detestar",
    "detestaremos": "detestar",
    "detestaren": "detestar",
    "detestares": "detestar",
    "detestaron": "detestar",
    "detestará": "detestar",
    "detestarán": "detestar",
    "detestarás": "detestar",
    "detestaré": "detestar",
    "detestaréis": "detestar",
    "detestaría": "detestar",
    "detestaríais": "detestar",
    "detestaríamos": "detestar",
    "detestarían": "detestar",
    "detestarías": "detestar",
    "detestas": "detestar",
    "detestase": "detestar",
    "detestaseis": "detestar",
    "detestasen": "detestar",
    "detestases": "detestar",
    "detestaste": "detestar",
    "detestasteis": "detestar",
    "deteste": "detestar",
    "detestemos": "detestar",
    "detesten": "detestar",
    "detestes": "detestar",
    "detesto": "detestar",
    "detestábamos": "detestar",
    "detestáis": "detestar",
    "detestáramos": "detestar",
    "detestáremos": "detestar",
    "detestásemos": "detestar",
    "detesté": "detestar",
    "detestéis": "detestar",
    "detestó": "detestar",
    "detiene": "detener",
    "detienebueyes": "detienebuey",
    "detienen": "detener",
    "detienes": "detener",
    "detona": "detonar",
    "detonaba": "detonar",
    "detonabais": "detonar",
    "detonaban": "detonar",
    "detonabas": "detonar",
    "detonaciones": "detonación",
    "detonad": "detonar",
    "detonada": "detonar",
    "detonadas": "detonar",
    "detonado": "detonar",
    "detonadora": "detonador",
    "detonadoras": "detonador",
    "detonadores": "detonador",
    "detonados": "detonar",
    "detonamos": "detonar",
    "detonan": "detonar",
    "detonando": "detonar",
    "detonantes": "detonante",
    "detonara": "detonar",
    "detonarais": "detonar",
    "detonaran": "detonar",
    "detonaras": "detonar",
    "detonare": "detonar",
    "detonareis": "detonar",
    "detonaremos": "detonar",
    "detonaren": "detonar",
    "detonares": "detonar",
    "detonaron": "detonar",
    "detonará": "detonar",
    "detonarán": "detonar",
    "detonarás": "detonar",
    "detonaré": "detonar",
    "detonaréis": "detonar",
    "detonaría": "detonar",
    "detonaríais": "detonar",
    "detonaríamos": "detonar",
    "detonarían": "detonar",
    "detonarías": "detonar",
    "detonas": "detonar",
    "detonase": "detonar",
    "detonaseis": "detonar",
    "detonasen": "detonar",
    "detonases": "detonar",
    "detonaste": "detonar",
    "detonasteis": "detonar",
    "detone": "detonar",
    "detonemos": "detonar",
    "detonen": "detonar",
    "detones": "detonar",
    "detono": "detonar",
    "detonábamos": "detonar",
    "detonáis": "detonar",
    "detonáramos": "detonar",
    "detonáremos": "detonar",
    "detonásemos": "detonar",
    "detoné": "detonar",
    "detonéis": "detonar",
    "detonó": "detonar",
    "detorsiones": "detorsión",
    "detoxificaciones": "detoxificación",
    "detracciones": "detracción",
    "detracta": "detractar",
    "detractaba": "detractar",
    "detractabais": "detractar",
    "detractaban": "detractar",
    "detractabas": "detractar",
    "detractaciones": "detractación",
    "detractad": "detractar",
    "detractada": "detractar",
    "detractadas": "detractar",
    "detractado": "detractar",
    "detractados": "detractar",
    "detractamos": "detractar",
    "detractan": "detractar",
    "detractando": "detractar",
    "detractara": "detractar",
    "detractarais": "detractar",
    "detractaran": "detractar",
    "detractaras": "detractar",
    "detractare": "detractar",
    "detractareis": "detractar",
    "detractaremos": "detractar",
    "detractaren": "detractar",
    "detractares": "detractar",
    "detractaron": "detractar",
    "detractará": "detractar",
    "detractarán": "detractar",
    "detractarás": "detractar",
    "detractaré": "detractar",
    "detractaréis": "detractar",
    "detractaría": "detractar",
    "detractaríais": "detractar",
    "detractaríamos": "detractar",
    "detractarían": "detractar",
    "detractarías": "detractar",
    "detractas": "detractar",
    "detractase": "detractar",
    "detractaseis": "detractar",
    "detractasen": "detractar",
    "detractases": "detractar",
    "detractaste": "detractar",
    "detractasteis": "detractar",
    "detracte": "detractar",
    "detractemos": "detractar",
    "detracten": "detractar",
    "detractes": "detractar",
    "detracto": "detractar",
    "detractora": "detractor",
    "detractoras": "detractor",
    "detractores": "detractor",
    "detractábamos": "detractar",
    "detractáis": "detractar",
    "detractáramos": "detractar",
    "detractáremos": "detractar",
    "detractásemos": "detractar",
    "detracté": "detractar",
    "detractéis": "detractar",
    "detractó": "detractar",
    "detrae": "detraer",
    "detraed": "detraer",
    "detraemos": "detraer",
    "detraen": "detraer",
    "detraeremos": "detraer",
    "detraerá": "detraer",
    "detraerán": "detraer",
    "detraerás": "detraer",
    "detraeré": "detraer",
    "detraeréis": "detraer",
    "detraería": "detraer",
    "detraeríais": "detraer",
    "detraeríamos": "detraer",
    "detraerían": "detraer",
    "detraerías": "detraer",
    "detraes": "detraer",
    "detraiga": "detraer",
    "detraigamos": "detraer",
    "detraigan": "detraer",
    "detraigas": "detraer",
    "detraigo": "detraer",
    "detraigáis": "detraer",
    "detraimientos": "detraimiento",
    "detraje": "detraer",
    "detrajera": "detraer",
    "detrajerais": "detraer",
    "detrajeran": "detraer",
    "detrajeras": "detraer",
    "detrajere": "detraer",
    "detrajereis": "detraer",
    "detrajeren": "detraer",
    "detrajeres": "detraer",
    "detrajeron": "detraer",
    "detrajese": "detraer",
    "detrajeseis": "detraer",
    "detrajesen": "detraer",
    "detrajeses": "detraer",
    "detrajimos": "detraer",
    "detrajiste": "detraer",
    "detrajisteis": "detraer",
    "detrajo": "detraer",
    "detrajéramos": "detraer",
    "detrajéremos": "detraer",
    "detrajésemos": "detraer",
    "detrayendo": "detraer",
    "detraéis": "detraer",
    "detraía": "detraer",
    "detraíais": "detraer",
    "detraíamos": "detraer",
    "detraían": "detraer",
    "detraías": "detraer",
    "detraída": "detraer",
    "detraídas": "detraer",
    "detraído": "detraer",
    "detraídos": "detraer",
    "detrimentos": "detrimento",
    "detritos": "detrito",
    "detroquemos": "destrocar",
    "detroqué": "destrocar",
    "detroquéis": "destrocar",
    "detrítica": "detrítico",
    "detríticas": "detrítico",
    "detríticos": "detrítico",
    "detuve": "detener",
    "detuviera": "detener",
    "detuvierais": "detener",
    "detuvieran": "detener",
    "detuvieras": "detener",
    "detuviere": "detener",
    "detuviereis": "detener",
    "detuvieren": "detener",
    "detuvieres": "detener",
    "detuvieron": "detener",
    "detuviese": "detener",
    "detuvieseis": "detener",
    "detuviesen": "detener",
    "detuvieses": "detener",
    "detuvimos": "detener",
    "detuviste": "detener",
    "detuvisteis": "detener",
    "detuviéramos": "detener",
    "detuviéremos": "detener",
    "detuviésemos": "detener",
    "detuvo": "detener",
    "detén": "detener",
    "deudas": "deuda",
    "deudora": "deudor",
    "deudoras": "deudor",
    "deudores": "deudor",
    "deudos": "deudo",
    "deuteranomalopías": "deuteranomalopía",
    "deuteranomalías": "deuteranomalía",
    "deuteranopías": "deuteranopía",
    "deuterios": "deuterio",
    "deuterologías": "deuterología",
    "deuterones": "deuterón",
    "deuteropatías": "deuteropatía",
    "deutóxidos": "deutóxido",
    "devala": "devalar",
    "devalaba": "devalar",
    "devalabais": "devalar",
    "devalaban": "devalar",
    "devalabas": "devalar",
    "devalad": "devalar",
    "devalada": "devalar",
    "devaladas": "devalar",
    "devalado": "devalar",
    "devalados": "devalar",
    "devalamos": "devalar",
    "devalan": "devalar",
    "devalando": "devalar",
    "devalara": "devalar",
    "devalarais": "devalar",
    "devalaran": "devalar",
    "devalaras": "devalar",
    "devalare": "devalar",
    "devalareis": "devalar",
    "devalaremos": "devalar",
    "devalaren": "devalar",
    "devalares": "devalar",
    "devalaron": "devalar",
    "devalará": "devalar",
    "devalarán": "devalar",
    "devalarás": "devalar",
    "devalaré": "devalar",
    "devalaréis": "devalar",
    "devalaría": "devalar",
    "devalaríais": "devalar",
    "devalaríamos": "devalar",
    "devalarían": "devalar",
    "devalarías": "devalar",
    "devalas": "devalar",
    "devalase": "devalar",
    "devalaseis": "devalar",
    "devalasen": "devalar",
    "devalases": "devalar",
    "devalaste": "devalar",
    "devalasteis": "devalar",
    "devale": "devalar",
    "devalemos": "devalar",
    "devalen": "devalar",
    "devales": "devalar",
    "devalo": "devalar",
    "devaluaba": "devaluar",
    "devaluabais": "devaluar",
    "devaluaban": "devaluar",
    "devaluabas": "devaluar",
    "devaluaciones": "devaluación",
    "devaluad": "devaluar",
    "devaluada": "devaluar",
    "devaluadas": "devaluar",
    "devaluado": "devaluar",
    "devaluados": "devaluar",
    "devaluamos": "devaluar",
    "devaluando": "devaluar",
    "devaluara": "devaluar",
    "devaluarais": "devaluar",
    "devaluaran": "devaluar",
    "devaluaras": "devaluar",
    "devaluare": "devaluar",
    "devaluareis": "devaluar",
    "devaluaremos": "devaluar",
    "devaluaren": "devaluar",
    "devaluares": "devaluar",
    "devaluaron": "devaluar",
    "devaluará": "devaluar",
    "devaluarán": "devaluar",
    "devaluarás": "devaluar",
    "devaluaré": "devaluar",
    "devaluaréis": "devaluar",
    "devaluaría": "devaluar",
    "devaluaríais": "devaluar",
    "devaluaríamos": "devaluar",
    "devaluarían": "devaluar",
    "devaluarías": "devaluar",
    "devaluase": "devaluar",
    "devaluaseis": "devaluar",
    "devaluasen": "devaluar",
    "devaluases": "devaluar",
    "devaluaste": "devaluar",
    "devaluasteis": "devaluar",
    "devaluemos": "devaluar",
    "devaluábamos": "devaluar",
    "devaluáis": "devaluar",
    "devaluáramos": "devaluar",
    "devaluáremos": "devaluar",
    "devaluásemos": "devaluar",
    "devalué": "devaluar",
    "devaluéis": "devaluar",
    "devaluó": "devaluar",
    "devalábamos": "devalar",
    "devaláis": "devalar",
    "devaláramos": "devalar",
    "devaláremos": "devalar",
    "devalásemos": "devalar",
    "devalé": "devalar",
    "devaléis": "devalar",
    "devaló": "devalar",
    "devalúa": "devaluar",
    "devalúan": "devaluar",
    "devalúas": "devaluar",
    "devalúe": "devaluar",
    "devalúen": "devaluar",
    "devalúes": "devaluar",
    "devalúo": "devaluar",
    "devana": "devanar",
    "devanaba": "devanar",
    "devanabais": "devanar",
    "devanaban": "devanar",
    "devanabas": "devanar",
    "devanad": "devanar",
    "devanada": "devanar",
    "devanadas": "devanar",
    "devanaderas": "devanadera",
    "devanado": "devanar",
    "devanadora": "devanador",
    "devanadoras": "devanadora",
    "devanadores": "devanador",
    "devanados": "devanar",
    "devanagaris": "devanagari",
    "devanamientos": "devanamiento",
    "devanamos": "devanar",
    "devanan": "devanar",
    "devanando": "devanar",
    "devanara": "devanar",
    "devanarais": "devanar",
    "devanaran": "devanar",
    "devanaras": "devanar",
    "devanare": "devanar",
    "devanareis": "devanar",
    "devanaremos": "devanar",
    "devanaren": "devanar",
    "devanares": "devanar",
    "devanaron": "devanar",
    "devanará": "devanar",
    "devanarán": "devanar",
    "devanarás": "devanar",
    "devanaré": "devanar",
    "devanaréis": "devanar",
    "devanaría": "devanar",
    "devanaríais": "devanar",
    "devanaríamos": "devanar",
    "devanarían": "devanar",
    "devanarías": "devanar",
    "devanas": "devanar",
    "devanase": "devanar",
    "devanaseis": "devanar",
    "devanasen": "devanar",
    "devanases": "devanar",
    "devanaste": "devanar",
    "devanasteis": "devanar",
    "devane": "devanar",
    "devanea": "devanear",
    "devaneaba": "devanear",
    "devaneabais": "devanear",
    "devaneaban": "devanear",
    "devaneabas": "devanear",
    "devanead": "devanear",
    "devaneada": "devanear",
    "devaneadas": "devanear",
    "devaneado": "devanear",
    "devaneados": "devanear",
    "devaneamos": "devanear",
    "devanean": "devanear",
    "devaneando": "devanear",
    "devaneara": "devanear",
    "devanearais": "devanear",
    "devanearan": "devanear",
    "devanearas": "devanear",
    "devaneare": "devanear",
    "devaneareis": "devanear",
    "devanearemos": "devanear",
    "devanearen": "devanear",
    "devaneares": "devanear",
    "devanearon": "devanear",
    "devaneará": "devanear",
    "devanearán": "devanear",
    "devanearás": "devanear",
    "devanearé": "devanear",
    "devanearéis": "devanear",
    "devanearía": "devanear",
    "devanearíais": "devanear",
    "devanearíamos": "devanear",
    "devanearían": "devanear",
    "devanearías": "devanear",
    "devaneas": "devanear",
    "devanease": "devanear",
    "devaneaseis": "devanear",
    "devaneasen": "devanear",
    "devaneases": "devanear",
    "devaneaste": "devanear",
    "devaneasteis": "devanear",
    "devanee": "devanear",
    "devaneemos": "devanear",
    "devaneen": "devanear",
    "devanees": "devanear",
    "devanemos": "devanar",
    "devanen": "devanar",
    "devaneo": "devanear",
    "devaneos": "devaneo",
    "devanes": "devanar",
    "devaneábamos": "devanear",
    "devaneáis": "devanear",
    "devaneáramos": "devanear",
    "devaneáremos": "devanear",
    "devaneásemos": "devanear",
    "devaneé": "devanear",
    "devaneéis": "devanear",
    "devaneó": "devanear",
    "devano": "devanar",
    "devanábamos": "devanar",
    "devanáis": "devanar",
    "devanáramos": "devanar",
    "devanáremos": "devanar",
    "devanásemos": "devanar",
    "devané": "devanar",
    "devanéis": "devanar",
    "devanó": "devanar",
    "devas": "deva",
    "devasta": "devastar",
    "devastaba": "devastar",
    "devastabais": "devastar",
    "devastaban": "devastar",
    "devastabas": "devastar",
    "devastaciones": "devastación",
    "devastad": "devastar",
    "devastada": "devastar",
    "devastadas": "devastar",
    "devastado": "devastar",
    "devastadora": "devastador",
    "devastadoras": "devastador",
    "devastadores": "devastador",
    "devastados": "devastar",
    "devastamos": "devastar",
    "devastan": "devastar",
    "devastando": "devastar",
    "devastara": "devastar",
    "devastarais": "devastar",
    "devastaran": "devastar",
    "devastaras": "devastar",
    "devastare": "devastar",
    "devastareis": "devastar",
    "devastaremos": "devastar",
    "devastaren": "devastar",
    "devastares": "devastar",
    "devastaron": "devastar",
    "devastará": "devastar",
    "devastarán": "devastar",
    "devastarás": "devastar",
    "devastaré": "devastar",
    "devastaréis": "devastar",
    "devastaría": "devastar",
    "devastaríais": "devastar",
    "devastaríamos": "devastar",
    "devastarían": "devastar",
    "devastarías": "devastar",
    "devastas": "devastar",
    "devastase": "devastar",
    "devastaseis": "devastar",
    "devastasen": "devastar",
    "devastases": "devastar",
    "devastaste": "devastar",
    "devastasteis": "devastar",
    "devaste": "devastar",
    "devastemos": "devastar",
    "devasten": "devastar",
    "devastes": "devastar",
    "devasto": "devastar",
    "devastábamos": "devastar",
    "devastáis": "devastar",
    "devastáramos": "devastar",
    "devastáremos": "devastar",
    "devastásemos": "devastar",
    "devasté": "devastar",
    "devastéis": "devastar",
    "devastó": "devastar",
    "devatada": "devatado",
    "devatadas": "devatado",
    "devatados": "devatado",
    "devatiada": "devatiado",
    "devatiadas": "devatiado",
    "devatiados": "devatiado",
    "deven": "devenir",
    "devendremos": "devenir",
    "devendrá": "devenir",
    "devendrán": "devenir",
    "devendrás": "devenir",
    "devendré": "devenir",
    "devendréis": "devenir",
    "devendría": "devenir",
    "devendríais": "devenir",
    "devendríamos": "devenir",
    "devendrían": "devenir",
    "devendrías": "devenir",
    "devenga": "devenir",
    "devengaba": "devengar",
    "devengabais": "devengar",
    "devengaban": "devengar",
    "devengabas": "devengar",
    "devengables": "devengable",
    "devengad": "devengar",
    "devengada": "devengar",
    "devengadas": "devengar",
    "devengado": "devengar",
    "devengados": "devengar",
    "devengamos": "devenir",
    "devengan": "devenir",
    "devengando": "devengar",
    "devengara": "devengar",
    "devengarais": "devengar",
    "devengaran": "devengar",
    "devengaras": "devengar",
    "devengare": "devengar",
    "devengareis": "devengar",
    "devengaremos": "devengar",
    "devengaren": "devengar",
    "devengares": "devengar",
    "devengaron": "devengar",
    "devengará": "devengar",
    "devengarán": "devengar",
    "devengarás": "devengar",
    "devengaré": "devengar",
    "devengaréis": "devengar",
    "devengaría": "devengar",
    "devengaríais": "devengar",
    "devengaríamos": "devengar",
    "devengarían": "devengar",
    "devengarías": "devengar",
    "devengas": "devenir",
    "devengase": "devengar",
    "devengaseis": "devengar",
    "devengasen": "devengar",
    "devengases": "devengar",
    "devengaste": "devengar",
    "devengasteis": "devengar",
    "devengo": "devenir",
    "devengos": "devengo",
    "devengue": "devengar",
    "devenguemos": "devengar",
    "devenguen": "devengar",
    "devengues": "devengar",
    "devengué": "devengar",
    "devenguéis": "devengar",
    "devengábamos": "devengar",
    "devengáis": "devenir",
    "devengáramos": "devengar",
    "devengáremos": "devengar",
    "devengásemos": "devengar",
    "devengó": "devengar",
    "devenid": "devenir",
    "devenida": "devenir",
    "devenidas": "devenir",
    "devenido": "devenir",
    "devenidos": "devenir",
    "devenimos": "devenir",
    "devenires": "devenir",
    "devenía": "devenir",
    "deveníais": "devenir",
    "deveníamos": "devenir",
    "devenían": "devenir",
    "devenías": "devenir",
    "devenís": "devenir",
    "deverbales": "deverbal",
    "deverbativa": "deverbativo",
    "deverbativas": "deverbativo",
    "deverbativos": "deverbativo",
    "deviene": "devenir",
    "devienen": "devenir",
    "devienes": "devenir",
    "devine": "devenir",
    "deviniendo": "devenir",
    "deviniera": "devenir",
    "devinierais": "devenir",
    "devinieran": "devenir",
    "devinieras": "devenir",
    "deviniere": "devenir",
    "deviniereis": "devenir",
    "devinieren": "devenir",
    "devinieres": "devenir",
    "devinieron": "devenir",
    "deviniese": "devenir",
    "devinieseis": "devenir",
    "deviniesen": "devenir",
    "devinieses": "devenir",
    "devinimos": "devenir",
    "deviniste": "devenir",
    "devinisteis": "devenir",
    "deviniéramos": "devenir",
    "deviniéremos": "devenir",
    "deviniésemos": "devenir",
    "devino": "devenir",
    "devisas": "devisa",
    "devocionarios": "devocionario",
    "devociones": "devoción",
    "devoluciones": "devolución",
    "devolucionistas": "devolucionista",
    "devoluta": "devoluto",
    "devolutas": "devoluto",
    "devolutiva": "devolutivo",
    "devolutivas": "devolutivo",
    "devolutivos": "devolutivo",
    "devolutos": "devoluto",
    "devolvamos": "devolver",
    "devolved": "devolver",
    "devolvemos": "devolver",
    "devolveremos": "devolver",
    "devolverá": "devolver",
    "devolverán": "devolver",
    "devolverás": "devolver",
    "devolveré": "devolver",
    "devolveréis": "devolver",
    "devolvería": "devolver",
    "devolveríais": "devolver",
    "devolveríamos": "devolver",
    "devolverían": "devolver",
    "devolverías": "devolver",
    "devolviendo": "devolver",
    "devolviera": "devolver",
    "devolvierais": "devolver",
    "devolvieran": "devolver",
    "devolvieras": "devolver",
    "devolviere": "devolver",
    "devolviereis": "devolver",
    "devolvieren": "devolver",
    "devolvieres": "devolver",
    "devolvieron": "devolver",
    "devolviese": "devolver",
    "devolvieseis": "devolver",
    "devolviesen": "devolver",
    "devolvieses": "devolver",
    "devolvimos": "devolver",
    "devolviste": "devolver",
    "devolvisteis": "devolver",
    "devolviéramos": "devolver",
    "devolviéremos": "devolver",
    "devolviésemos": "devolver",
    "devolvió": "devolver",
    "devolváis": "devolver",
    "devolvéis": "devolver",
    "devolví": "devolver",
    "devolvía": "devolver",
    "devolvíais": "devolver",
    "devolvíamos": "devolver",
    "devolvían": "devolver",
    "devolvías": "devolver",
    "devoniana": "devoniano",
    "devonianas": "devoniano",
    "devonianos": "devoniano",
    "devora": "devorar",
    "devoraba": "devorar",
    "devorabais": "devorar",
    "devoraban": "devorar",
    "devorabas": "devorar",
    "devorad": "devorar",
    "devorada": "devorar",
    "devoradas": "devorar",
    "devorado": "devorar",
    "devoradora": "devorador",
    "devoradoras": "devorador",
    "devoradores": "devorador",
    "devorados": "devorar",
    "devoramos": "devorar",
    "devoran": "devorar",
    "devorando": "devorar",
    "devorara": "devorar",
    "devorarais": "devorar",
    "devoraran": "devorar",
    "devoraras": "devorar",
    "devorare": "devorar",
    "devorareis": "devorar",
    "devoraremos": "devorar",
    "devoraren": "devorar",
    "devorares": "devorar",
    "devoraron": "devorar",
    "devorará": "devorar",
    "devorarán": "devorar",
    "devorarás": "devorar",
    "devoraré": "devorar",
    "devoraréis": "devorar",
    "devoraría": "devorar",
    "devoraríais": "devorar",
    "devoraríamos": "devorar",
    "devorarían": "devorar",
    "devorarías": "devorar",
    "devoras": "devorar",
    "devorase": "devorar",
    "devoraseis": "devorar",
    "devorasen": "devorar",
    "devorases": "devorar",
    "devoraste": "devorar",
    "devorasteis": "devorar",
    "devore": "devorar",
    "devoremos": "devorar",
    "devoren": "devorar",
    "devores": "devorar",
    "devoro": "devorar",
    "devorábamos": "devorar",
    "devoráis": "devorar",
    "devoráramos": "devorar",
    "devoráremos": "devorar",
    "devorásemos": "devorar",
    "devoré": "devorar",
    "devoréis": "devorar",
    "devoró": "devorar",
    "devota": "devoto",
    "devotas": "devoto",
    "devoterías": "devotería",
    "devotos": "devoto",
    "devuelta": "devolver",
    "devueltas": "devolver",
    "devuelto": "devolver",
    "devueltos": "devolver",
    "devuelva": "devolver",
    "devuelvan": "devolver",
    "devuelvas": "devolver",
    "devuelve": "devolver",
    "devuelven": "devolver",
    "devuelves": "devolver",
    "devuelvo": "devolver",
    "dexiocardias": "dexiocardia",
    "dextranes": "dextrán",
    "dextranos": "dextrano",
    "dextrinas": "dextrina",
    "dextrismos": "dextrismo",
    "dextrocardias": "dextrocardia",
    "dextrodelta": "dextrodelto",
    "dextrodeltas": "dextrodelto",
    "dextrodeltos": "dextrodelto",
    "dextrogiratoria": "dextrogiratorio",
    "dextrogiratorias": "dextrogiratorio",
    "dextrogiratorios": "dextrogiratorio",
    "dextrorrotatoria": "dextrorrotatorio",
    "dextrorrotatorias": "dextrorrotatorio",
    "dextrorrotatorios": "dextrorrotatorio",
    "dextrorsa": "dextrorso",
    "dextrorsas": "dextrorso",
    "dextrorsos": "dextrorso",
    "dextrosas": "dextrosa",
    "dextrógira": "dextrógiro",
    "dextrógiras": "dextrógiro",
    "dextrógiros": "dextrógiro",
    "deyecciones": "deyección",
    "deyectora": "deyector",
    "deyectoras": "deyector",
    "deyectores": "deyector",
    "deyes": "dey",
    "dezmaba": "dezmar",
    "dezmabais": "dezmar",
    "dezmaban": "dezmar",
    "dezmabas": "dezmar",
    "dezmad": "dezmar",
    "dezmada": "dezmar",
    "dezmadas": "dezmar",
    "dezmado": "dezmar",
    "dezmados": "dezmar",
    "dezmamos": "dezmar",
    "dezmando": "dezmar",
    "dezmara": "dezmar",
    "dezmarais": "dezmar",
    "dezmaran": "dezmar",
    "dezmaras": "dezmar",
    "dezmare": "dezmar",
    "dezmareis": "dezmar",
    "dezmaremos": "dezmar",
    "dezmaren": "dezmar",
    "dezmares": "dezmar",
    "dezmaron": "dezmar",
    "dezmará": "dezmar",
    "dezmarán": "dezmar",
    "dezmarás": "dezmar",
    "dezmaré": "dezmar",
    "dezmaréis": "dezmar",
    "dezmaría": "dezmar",
    "dezmaríais": "dezmar",
    "dezmaríamos": "dezmar",
    "dezmarían": "dezmar",
    "dezmarías": "dezmar",
    "dezmase": "dezmar",
    "dezmaseis": "dezmar",
    "dezmasen": "dezmar",
    "dezmases": "dezmar",
    "dezmaste": "dezmar",
    "dezmasteis": "dezmar",
    "dezmemos": "dezmar",
    "dezmera": "dezmero",
    "dezmeras": "dezmero",
    "dezmeros": "dezmero",
    "dezmerías": "dezmería",
    "dezmeña": "dezmeño",
    "dezmeñas": "dezmeño",
    "dezmeños": "dezmeño",
    "dezmábamos": "dezmar",
    "dezmáis": "dezmar",
    "dezmáramos": "dezmar",
    "dezmáremos": "dezmar",
    "dezmásemos": "dezmar",
    "dezmé": "dezmar",
    "dezméis": "dezmar",
    "dezmó": "dezmar",
    "deíctica": "deíctico",
    "deícticas": "deíctico",
    "deícticos": "deíctico",
    "deífica": "deífico",
    "deíficas": "deífico",
    "deíficos": "deífico",
    "deíparas": "deípara",
    "deísmos": "deísmo",
    "deístas": "deísta",
    "dg": "decigramo",
    "dharmas": "dharma",
    "di": "decir",
    "diabasas": "diabasa",
    "diabetologías": "diabetología",
    "diabetos": "diabeto",
    "diablas": "diabla",
    "diablea": "diablear",
    "diableaba": "diablear",
    "diableabais": "diablear",
    "diableaban": "diablear",
    "diableabas": "diablear",
    "diablead": "diablear",
    "diableada": "diablear",
    "diableadas": "diablear",
    "diableado": "diablear",
    "diableados": "diablear",
    "diableamos": "diablear",
    "diablean": "diablear",
    "diableando": "diablear",
    "diableara": "diablear",
    "diablearais": "diablear",
    "diablearan": "diablear",
    "diablearas": "diablear",
    "diableare": "diablear",
    "diableareis": "diablear",
    "diablearemos": "diablear",
    "diablearen": "diablear",
    "diableares": "diablear",
    "diablearon": "diablear",
    "diableará": "diablear",
    "diablearán": "diablear",
    "diablearás": "diablear",
    "diablearé": "diablear",
    "diablearéis": "diablear",
    "diablearía": "diablear",
    "diablearíais": "diablear",
    "diablearíamos": "diablear",
    "diablearían": "diablear",
    "diablearías": "diablear",
    "diableas": "diablear",
    "diablease": "diablear",
    "diableaseis": "diablear",
    "diableasen": "diablear",
    "diableases": "diablear",
    "diableaste": "diablear",
    "diableasteis": "diablear",
    "diablee": "diablear",
    "diableemos": "diablear",
    "diableen": "diablear",
    "diablees": "diablear",
    "diableo": "diablear",
    "diableros": "diablero",
    "diablesas": "diablesa",
    "diablesca": "diablesco",
    "diablescas": "diablesco",
    "diablescos": "diablesco",
    "diableábamos": "diablear",
    "diableáis": "diablear",
    "diableáramos": "diablear",
    "diableáremos": "diablear",
    "diableásemos": "diablear",
    "diableé": "diablear",
    "diableéis": "diablear",
    "diableó": "diablear",
    "diablillos": "diablillo",
    "diablos": "diablo",
    "diabluras": "diablura",
    "diabolines": "diabolín",
    "diabética": "diabético",
    "diabéticas": "diabético",
    "diabéticos": "diabético",
    "diabólica": "diabólico",
    "diabólicas": "diabólico",
    "diabólicos": "diabólico",
    "diacetilmorfinas": "diacetilmorfina",
    "diacetilos": "diacetilo",
    "diacetinas": "diacetina",
    "diacilgliceroles": "diacilglicerol",
    "diacinética": "diacinético",
    "diacinéticas": "diacinético",
    "diacinéticos": "diacinético",
    "diaclasas": "diaclasa",
    "diacodiones": "diacodión",
    "diacona": "diaconar",
    "diaconaba": "diaconar",
    "diaconabais": "diaconar",
    "diaconaban": "diaconar",
    "diaconabas": "diaconar",
    "diaconad": "diaconar",
    "diaconada": "diaconar",
    "diaconadas": "diaconar",
    "diaconado": "diaconar",
    "diaconados": "diaconar",
    "diaconales": "diaconal",
    "diaconamos": "diaconar",
    "diaconan": "diaconar",
    "diaconando": "diaconar",
    "diaconara": "diaconar",
    "diaconarais": "diaconar",
    "diaconaran": "diaconar",
    "diaconaras": "diaconar",
    "diaconare": "diaconar",
    "diaconareis": "diaconar",
    "diaconaremos": "diaconar",
    "diaconaren": "diaconar",
    "diaconares": "diaconar",
    "diaconaron": "diaconar",
    "diaconará": "diaconar",
    "diaconarán": "diaconar",
    "diaconarás": "diaconar",
    "diaconaré": "diaconar",
    "diaconaréis": "diaconar",
    "diaconaría": "diaconar",
    "diaconaríais": "diaconar",
    "diaconaríamos": "diaconar",
    "diaconarían": "diaconar",
    "diaconarías": "diaconar",
    "diaconas": "diaconar",
    "diaconase": "diaconar",
    "diaconaseis": "diaconar",
    "diaconasen": "diaconar",
    "diaconases": "diaconar",
    "diaconaste": "diaconar",
    "diaconasteis": "diaconar",
    "diaconatos": "diaconato",
    "diacone": "diaconar",
    "diaconemos": "diaconar",
    "diaconen": "diaconar",
    "diacones": "diaconar",
    "diaconisas": "diaconisa",
    "diacono": "diaconar",
    "diaconábamos": "diaconar",
    "diaconáis": "diaconar",
    "diaconáramos": "diaconar",
    "diaconáremos": "diaconar",
    "diaconásemos": "diaconar",
    "diaconé": "diaconar",
    "diaconéis": "diaconar",
    "diaconías": "diaconía",
    "diaconó": "diaconar",
    "diacronías": "diacronía",
    "diacrítica": "diacrítico",
    "diacríticas": "diacrítico",
    "diacríticos": "diacrítico",
    "diacrónica": "diacrónico",
    "diacrónicas": "diacrónico",
    "diacrónicos": "diacrónico",
    "diacústicas": "diacústica",
    "diadelfa": "diadelfo",
    "diadelfas": "diadelfo",
    "diadelfias": "diadelfia",
    "diadelfos": "diadelfo",
    "diademada": "diademado",
    "diademadas": "diademado",
    "diademados": "diademado",
    "diademas": "diadema",
    "diadocos": "diadoco",
    "diafanidades": "diafanidad",
    "diafanizaciones": "diafanización",
    "diafanoscopias": "diafanoscopia",
    "diafanómetros": "diafanómetro",
    "diafonias": "diafonia",
    "diafonías": "diafonía",
    "diafragma": "diafragmar",
    "diafragmaba": "diafragmar",
    "diafragmabais": "diafragmar",
    "diafragmaban": "diafragmar",
    "diafragmabas": "diafragmar",
    "diafragmad": "diafragmar",
    "diafragmada": "diafragmar",
    "diafragmadas": "diafragmar",
    "diafragmado": "diafragmar",
    "diafragmados": "diafragmar",
    "diafragmamos": "diafragmar",
    "diafragman": "diafragmar",
    "diafragmando": "diafragmar",
    "diafragmara": "diafragmar",
    "diafragmarais": "diafragmar",
    "diafragmaran": "diafragmar",
    "diafragmaras": "diafragmar",
    "diafragmare": "diafragmar",
    "diafragmareis": "diafragmar",
    "diafragmaremos": "diafragmar",
    "diafragmaren": "diafragmar",
    "diafragmares": "diafragmar",
    "diafragmaron": "diafragmar",
    "diafragmará": "diafragmar",
    "diafragmarán": "diafragmar",
    "diafragmarás": "diafragmar",
    "diafragmaré": "diafragmar",
    "diafragmaréis": "diafragmar",
    "diafragmaría": "diafragmar",
    "diafragmaríais": "diafragmar",
    "diafragmaríamos": "diafragmar",
    "diafragmarían": "diafragmar",
    "diafragmarías": "diafragmar",
    "diafragmas": "diafragmar",
    "diafragmase": "diafragmar",
    "diafragmaseis": "diafragmar",
    "diafragmasen": "diafragmar",
    "diafragmases": "diafragmar",
    "diafragmaste": "diafragmar",
    "diafragmasteis": "diafragmar",
    "diafragme": "diafragmar",
    "diafragmemos": "diafragmar",
    "diafragmen": "diafragmar",
    "diafragmes": "diafragmar",
    "diafragmo": "diafragmar",
    "diafragmábamos": "diafragmar",
    "diafragmáis": "diafragmar",
    "diafragmáramos": "diafragmar",
    "diafragmáremos": "diafragmar",
    "diafragmásemos": "diafragmar",
    "diafragmática": "diafragmático",
    "diafragmáticas": "diafragmático",
    "diafragmáticos": "diafragmático",
    "diafragmé": "diafragmar",
    "diafragméis": "diafragmar",
    "diafragmó": "diafragmar",
    "diagnostica": "diagnosticar",
    "diagnosticaba": "diagnosticar",
    "diagnosticabais": "diagnosticar",
    "diagnosticaban": "diagnosticar",
    "diagnosticabas": "diagnosticar",
    "diagnosticad": "diagnosticar",
    "diagnosticada": "diagnosticar",
    "diagnosticadas": "diagnosticar",
    "diagnosticado": "diagnosticar",
    "diagnosticados": "diagnosticar",
    "diagnosticamos": "diagnosticar",
    "diagnostican": "diagnosticar",
    "diagnosticando": "diagnosticar",
    "diagnosticara": "diagnosticar",
    "diagnosticarais": "diagnosticar",
    "diagnosticaran": "diagnosticar",
    "diagnosticaras": "diagnosticar",
    "diagnosticare": "diagnosticar",
    "diagnosticareis": "diagnosticar",
    "diagnosticaremos": "diagnosticar",
    "diagnosticaren": "diagnosticar",
    "diagnosticares": "diagnosticar",
    "diagnosticaron": "diagnosticar",
    "diagnosticará": "diagnosticar",
    "diagnosticarán": "diagnosticar",
    "diagnosticarás": "diagnosticar",
    "diagnosticaré": "diagnosticar",
    "diagnosticaréis": "diagnosticar",
    "diagnosticaría": "diagnosticar",
    "diagnosticaríais": "diagnosticar",
    "diagnosticaríamos": "diagnosticar",
    "diagnosticarían": "diagnosticar",
    "diagnosticarías": "diagnosticar",
    "diagnosticas": "diagnosticar",
    "diagnosticase": "diagnosticar",
    "diagnosticaseis": "diagnosticar",
    "diagnosticasen": "diagnosticar",
    "diagnosticases": "diagnosticar",
    "diagnosticaste": "diagnosticar",
    "diagnosticasteis": "diagnosticar",
    "diagnostico": "diagnosticar",
    "diagnosticábamos": "diagnosticar",
    "diagnosticáis": "diagnosticar",
    "diagnosticáramos": "diagnosticar",
    "diagnosticáremos": "diagnosticar",
    "diagnosticásemos": "diagnosticar",
    "diagnosticó": "diagnosticar",
    "diagnostique": "diagnosticar",
    "diagnostiquemos": "diagnosticar",
    "diagnostiquen": "diagnosticar",
    "diagnostiques": "diagnosticar",
    "diagnostiqué": "diagnosticar",
    "diagnostiquéis": "diagnosticar",
    "diagnóstica": "diagnóstico",
    "diagnósticas": "diagnóstico",
    "diagnósticos": "diagnóstico",
    "diagometrías": "diagometría",
    "diagonales": "diagonal",
    "diagonalizaciones": "diagonalización",
    "diagramas": "diagrama",
    "diagramática": "diagramático",
    "diagramáticas": "diagramático",
    "diagramáticos": "diagramático",
    "diagómetros": "diagómetro",
    "dialcoholes": "dialcohol",
    "dialdehídos": "dialdehído",
    "dialectales": "dialectal",
    "dialectalismos": "dialectalismo",
    "dialectologías": "dialectología",
    "dialectos": "dialecto",
    "dialectóloga": "dialectólogo",
    "dialectólogas": "dialectólogo",
    "dialectólogos": "dialectólogo",
    "diales": "dial",
    "dialice": "dializar",
    "dialicemos": "dializar",
    "dialicen": "dializar",
    "dialices": "dializar",
    "dialicé": "dializar",
    "dialicéis": "dializar",
    "dialilbarbitúrica": "dialilbarbitúrico",
    "dialilbarbitúricas": "dialilbarbitúrico",
    "dialilbarbitúricos": "dialilbarbitúrico",
    "dialipétalas": "dialipétala",
    "dialisépala": "dialisépalo",
    "dialisépalas": "dialisépalo",
    "dialisépalos": "dialisépalo",
    "dializa": "dializar",
    "dializaba": "dializar",
    "dializabais": "dializar",
    "dializaban": "dializar",
    "dializabas": "dializar",
    "dializables": "dializable",
    "dializaciones": "dialización",
    "dializad": "dializar",
    "dializada": "dializar",
    "dializadas": "dializar",
    "dializado": "dializar",
    "dializadores": "dializador",
    "dializados": "dializar",
    "dializamos": "dializar",
    "dializan": "dializar",
    "dializando": "dializar",
    "dializara": "dializar",
    "dializarais": "dializar",
    "dializaran": "dializar",
    "dializaras": "dializar",
    "dializare": "dializar",
    "dializareis": "dializar",
    "dializaremos": "dializar",
    "dializaren": "dializar",
    "dializares": "dializar",
    "dializaron": "dializar",
    "dializará": "dializar",
    "dializarán": "dializar",
    "dializarás": "dializar",
    "dializaré": "dializar",
    "dializaréis": "dializar",
    "dializaría": "dializar",
    "dializaríais": "dializar",
    "dializaríamos": "dializar",
    "dializarían": "dializar",
    "dializarías": "dializar",
    "dializas": "dializar",
    "dializase": "dializar",
    "dializaseis": "dializar",
    "dializasen": "dializar",
    "dializases": "dializar",
    "dializaste": "dializar",
    "dializasteis": "dializar",
    "dializo": "dializar",
    "dializábamos": "dializar",
    "dializáis": "dializar",
    "dializáramos": "dializar",
    "dializáremos": "dializar",
    "dializásemos": "dializar",
    "dializó": "dializar",
    "dialoga": "dialogar",
    "dialogaba": "dialogar",
    "dialogabais": "dialogar",
    "dialogaban": "dialogar",
    "dialogabas": "dialogar",
    "dialogad": "dialogar",
    "dialogada": "dialogar",
    "dialogadas": "dialogar",
    "dialogado": "dialogar",
    "dialogados": "dialogar",
    "dialogales": "dialogal",
    "dialogamos": "dialogar",
    "dialogan": "dialogar",
    "dialogando": "dialogar",
    "dialogantes": "dialogante",
    "dialogara": "dialogar",
    "dialogarais": "dialogar",
    "dialogaran": "dialogar",
    "dialogaras": "dialogar",
    "dialogare": "dialogar",
    "dialogareis": "dialogar",
    "dialogaremos": "dialogar",
    "dialogaren": "dialogar",
    "dialogares": "dialogar",
    "dialogaron": "dialogar",
    "dialogará": "dialogar",
    "dialogarán": "dialogar",
    "dialogarás": "dialogar",
    "dialogaré": "dialogar",
    "dialogaréis": "dialogar",
    "dialogaría": "dialogar",
    "dialogaríais": "dialogar",
    "dialogaríamos": "dialogar",
    "dialogarían": "dialogar",
    "dialogarías": "dialogar",
    "dialogas": "dialogar",
    "dialogase": "dialogar",
    "dialogaseis": "dialogar",
    "dialogasen": "dialogar",
    "dialogases": "dialogar",
    "dialogaste": "dialogar",
    "dialogasteis": "dialogar",
    "dialogismos": "dialogismo",
    "dialogitas": "dialogita",
    "dialogo": "dialogar",
    "dialogue": "dialogar",
    "dialoguemos": "dialogar",
    "dialoguen": "dialogar",
    "dialogues": "dialogar",
    "dialogué": "dialogar",
    "dialoguéis": "dialogar",
    "dialogábamos": "dialogar",
    "dialogáis": "dialogar",
    "dialogáramos": "dialogar",
    "dialogáremos": "dialogar",
    "dialogásemos": "dialogar",
    "dialogística": "dialogístico",
    "dialogísticas": "dialogístico",
    "dialogísticos": "dialogístico",
    "dialogó": "dialogar",
    "dialéctica": "dialéctico",
    "dialécticas": "dialéctico",
    "dialécticos": "dialéctico",
    "dialítica": "dialítico",
    "dialíticas": "dialítico",
    "dialíticos": "dialítico",
    "diamagnetismos": "diamagnetismo",
    "diamagnética": "diamagnético",
    "diamagnéticas": "diamagnético",
    "diamagnéticos": "diamagnético",
    "diamanta": "diamantar",
    "diamantaba": "diamantar",
    "diamantabais": "diamantar",
    "diamantaban": "diamantar",
    "diamantabas": "diamantar",
    "diamantad": "diamantar",
    "diamantada": "diamantar",
    "diamantadas": "diamantar",
    "diamantado": "diamantar",
    "diamantados": "diamantar",
    "diamantamos": "diamantar",
    "diamantan": "diamantar",
    "diamantando": "diamantar",
    "diamantara": "diamantar",
    "diamantarais": "diamantar",
    "diamantaran": "diamantar",
    "diamantaras": "diamantar",
    "diamantare": "diamantar",
    "diamantareis": "diamantar",
    "diamantaremos": "diamantar",
    "diamantaren": "diamantar",
    "diamantares": "diamantar",
    "diamantaron": "diamantar",
    "diamantará": "diamantar",
    "diamantarán": "diamantar",
    "diamantarás": "diamantar",
    "diamantaré": "diamantar",
    "diamantaréis": "diamantar",
    "diamantaría": "diamantar",
    "diamantaríais": "diamantar",
    "diamantaríamos": "diamantar",
    "diamantarían": "diamantar",
    "diamantarías": "diamantar",
    "diamantas": "diamantar",
    "diamantase": "diamantar",
    "diamantaseis": "diamantar",
    "diamantasen": "diamantar",
    "diamantases": "diamantar",
    "diamantaste": "diamantar",
    "diamantasteis": "diamantar",
    "diamante": "diamantar",
    "diamantemos": "diamantar",
    "diamanten": "diamantar",
    "diamantes": "diamante",
    "diamantina": "diamantino",
    "diamantinas": "diamantino",
    "diamantinos": "diamantino",
    "diamantistas": "diamantista",
    "diamanto": "diamantar",
    "diamantábamos": "diamantar",
    "diamantáis": "diamantar",
    "diamantáramos": "diamantar",
    "diamantáremos": "diamantar",
    "diamantásemos": "diamantar",
    "diamanté": "diamantar",
    "diamantéis": "diamantar",
    "diamantífera": "diamantífero",
    "diamantíferas": "diamantífero",
    "diamantíferos": "diamantífero",
    "diamantó": "diamantar",
    "diametrales": "diametral",
    "diamidas": "diamida",
    "diamilfenoles": "diamilfenol",
    "diaminas": "diamina",
    "diaminooxidasas": "diaminooxidasa",
    "dianas": "diana",
    "diandra": "diandro",
    "diandras": "diandro",
    "diandros": "diandro",
    "dianegativa": "dianegativo",
    "dianegativas": "dianegativo",
    "dianegativos": "dianegativo",
    "dianisidinas": "dianisidina",
    "dianterales": "dianteral",
    "diantres": "diantre",
    "diapalmas": "diapalma",
    "diapasones": "diapasón",
    "diapentes": "diapente",
    "diapirismos": "diapirismo",
    "diapiros": "diapiro",
    "diapneusias": "diapneusia",
    "diaporamas": "diaporama",
    "diapositiva": "diapositivo",
    "diapositivas": "diapositivo",
    "diapositivos": "diapositivo",
    "diapreas": "diaprea",
    "diaquilones": "diaquilón",
    "diaria": "diario",
    "diarias": "diario",
    "diarios": "diario",
    "diarismos": "diarismo",
    "diarquías": "diarquía",
    "diarreas": "diarrea",
    "diarreica": "diarreico",
    "diarreicas": "diarreico",
    "diarreicos": "diarreico",
    "diartrodiales": "diartrodial",
    "diascopios": "diascopio",
    "diasistemas": "diasistema",
    "diasporámetros": "diasporámetro",
    "diaspros": "diaspro",
    "diastasas": "diastasa",
    "diastemas": "diastema",
    "diastemática": "diastemático",
    "diastemáticas": "diastemático",
    "diastemáticos": "diastemático",
    "diastimómetros": "diastimómetro",
    "diastrofias": "diastrofia",
    "diastrofismos": "diastrofismo",
    "diastática": "diastático",
    "diastáticas": "diastático",
    "diastáticos": "diastático",
    "diastólica": "diastólico",
    "diastólicas": "diastólico",
    "diastólicos": "diastólico",
    "diatermias": "diatermia",
    "diatesarones": "diatesarón",
    "diatomeas": "diatomea",
    "diatomitas": "diatomita",
    "diatribas": "diatriba",
    "diatrizoatos": "diatrizoato",
    "diatérmana": "diatérmano",
    "diatérmanas": "diatérmano",
    "diatérmanos": "diatérmano",
    "diatérmica": "diatérmico",
    "diatérmicas": "diatérmico",
    "diatérmicos": "diatérmico",
    "diatésica": "diatésico",
    "diatésicas": "diatésico",
    "diatésicos": "diatésico",
    "diatómica": "diatómico",
    "diatómicas": "diatómico",
    "diatómicos": "diatómico",
    "diatónica": "diatónico",
    "diatónicas": "diatónico",
    "diatónicos": "diatónico",
    "diazoaciones": "diazoación",
    "diazocompuestos": "diazocompuesto",
    "diazometanos": "diazometano",
    "diazonios": "diazonio",
    "diazonizaciones": "diazonización",
    "diazotizaciones": "diazotización",
    "dibenzantronas": "dibenzantrona",
    "dibenzofuranos": "dibenzofurano",
    "dibromodifluometanos": "dibromodifluometano",
    "dibromomalónica": "dibromomalónico",
    "dibromomalónicas": "dibromomalónico",
    "dibromomalónicos": "dibromomalónico",
    "dibromopropanoles": "dibromopropanol",
    "dibromuros": "dibromuro",
    "dibucaínas": "dibucaína",
    "dibuja": "dibujar",
    "dibujaba": "dibujar",
    "dibujabais": "dibujar",
    "dibujaban": "dibujar",
    "dibujabas": "dibujar",
    "dibujad": "dibujar",
    "dibujada": "dibujar",
    "dibujadas": "dibujar",
    "dibujado": "dibujar",
    "dibujados": "dibujar",
    "dibujamos": "dibujar",
    "dibujan": "dibujar",
    "dibujando": "dibujar",
    "dibujantes": "dibujante",
    "dibujara": "dibujar",
    "dibujarais": "dibujar",
    "dibujaran": "dibujar",
    "dibujaras": "dibujar",
    "dibujare": "dibujar",
    "dibujareis": "dibujar",
    "dibujaremos": "dibujar",
    "dibujaren": "dibujar",
    "dibujares": "dibujar",
    "dibujaron": "dibujar",
    "dibujará": "dibujar",
    "dibujarán": "dibujar",
    "dibujarás": "dibujar",
    "dibujaré": "dibujar",
    "dibujaréis": "dibujar",
    "dibujaría": "dibujar",
    "dibujaríais": "dibujar",
    "dibujaríamos": "dibujar",
    "dibujarían": "dibujar",
    "dibujarías": "dibujar",
    "dibujas": "dibujar",
    "dibujase": "dibujar",
    "dibujaseis": "dibujar",
    "dibujasen": "dibujar",
    "dibujases": "dibujar",
    "dibujaste": "dibujar",
    "dibujasteis": "dibujar",
    "dibuje": "dibujar",
    "dibujemos": "dibujar",
    "dibujen": "dibujar",
    "dibujes": "dibujar",
    "dibujo": "dibujar",
    "dibujos": "dibujo",
    "dibujábamos": "dibujar",
    "dibujáis": "dibujar",
    "dibujáramos": "dibujar",
    "dibujáremos": "dibujar",
    "dibujásemos": "dibujar",
    "dibujé": "dibujar",
    "dibujéis": "dibujar",
    "dibujó": "dibujar",
    "dibutilditiocarbamatos": "dibutilditiocarbamato",
    "dibutiltioureas": "dibutiltiourea",
    "dibutoxitetraglicoles": "dibutoxitetraglicol",
    "dibásica": "dibásico",
    "dibásicas": "dibásico",
    "dibásicos": "dibásico",
    "dic.": "diciembre",
    "dicarboximidas": "dicarboximida",
    "diccionarios": "diccionario",
    "diccionaristas": "diccionarista",
    "dicciones": "dicción",
    "dice": "decir",
    "dicen": "decir",
    "dices": "decir",
    "dicetenas": "dicetena",
    "dicetonas": "dicetona",
    "dicha": "decir",
    "dicharachera": "dicharachero",
    "dicharacheras": "dicharachero",
    "dicharacheros": "dicharachero",
    "dicharachos": "dicharacho",
    "dichas": "dicha",
    "dicho": "decir",
    "dichos": "dicho",
    "dichosa": "dichoso",
    "dichosas": "dichoso",
    "dichosos": "dichoso",
    "diciandiamidas": "diciandiamida",
    "diciclohexilaminas": "diciclohexilamina",
    "diciclopentadienos": "diciclopentadieno",
    "diciembres": "diciembre",
    "diciendo": "decir",
    "dickitas": "dickita",
    "diclamídea": "diclamídeo",
    "diclamídeas": "diclamídeo",
    "diclamídeos": "diclamídeo",
    "diclina": "diclino",
    "diclinas": "diclino",
    "diclinos": "diclino",
    "dicloroacetatos": "dicloroacetato",
    "diclorobencenos": "diclorobenceno",
    "diclorodifluometanos": "diclorodifluometano",
    "dicloroestearatos": "dicloroestearato",
    "diclorofeniltriclorosilanos": "diclorofeniltriclorosilano",
    "diclorofenos": "diclorofeno",
    "diclorofenoxiacetatos": "diclorofenoxiacetato",
    "diclorofluometanos": "diclorofluometano",
    "dicloroisocianuratos": "dicloroisocianurato",
    "diclorometanos": "diclorometano",
    "dicloropentanos": "dicloropentano",
    "dicloruros": "dicloruro",
    "dicogamias": "dicogamia",
    "dicoreos": "dicoreo",
    "dicotiledónea": "dicotiledóneo",
    "dicotiledóneas": "dicotiledóneo",
    "dicotiledóneos": "dicotiledóneo",
    "dicotomías": "dicotomía",
    "dicotómica": "dicotómico",
    "dicotómicas": "dicotómico",
    "dicotómicos": "dicotómico",
    "dicroica": "dicroico",
    "dicroicas": "dicroico",
    "dicroicos": "dicroico",
    "dicromatismos": "dicromatismo",
    "dicromatopsias": "dicromatopsia",
    "dicromatos": "dicromato",
    "dicromática": "dicromático",
    "dicromáticas": "dicromático",
    "dicromáticos": "dicromático",
    "dicroscopios": "dicroscopio",
    "dicroísmos": "dicroísmo",
    "dicroítas": "dicroíta",
    "dicta": "dictar",
    "dictaba": "dictar",
    "dictabais": "dictar",
    "dictaban": "dictar",
    "dictabas": "dictar",
    "dictad": "dictar",
    "dictada": "dictar",
    "dictadas": "dictar",
    "dictado": "dictar",
    "dictadora": "dictador",
    "dictadoras": "dictador",
    "dictadores": "dictador",
    "dictados": "dictar",
    "dictaduras": "dictadura",
    "dictamina": "dictaminar",
    "dictaminaba": "dictaminar",
    "dictaminabais": "dictaminar",
    "dictaminaban": "dictaminar",
    "dictaminabas": "dictaminar",
    "dictaminad": "dictaminar",
    "dictaminada": "dictaminar",
    "dictaminadas": "dictaminar",
    "dictaminado": "dictaminar",
    "dictaminadora": "dictaminador",
    "dictaminadoras": "dictaminador",
    "dictaminadores": "dictaminador",
    "dictaminados": "dictaminar",
    "dictaminamos": "dictaminar",
    "dictaminan": "dictaminar",
    "dictaminando": "dictaminar",
    "dictaminara": "dictaminar",
    "dictaminarais": "dictaminar",
    "dictaminaran": "dictaminar",
    "dictaminaras": "dictaminar",
    "dictaminare": "dictaminar",
    "dictaminareis": "dictaminar",
    "dictaminaremos": "dictaminar",
    "dictaminaren": "dictaminar",
    "dictaminares": "dictaminar",
    "dictaminaron": "dictaminar",
    "dictaminará": "dictaminar",
    "dictaminarán": "dictaminar",
    "dictaminarás": "dictaminar",
    "dictaminaré": "dictaminar",
    "dictaminaréis": "dictaminar",
    "dictaminaría": "dictaminar",
    "dictaminaríais": "dictaminar",
    "dictaminaríamos": "dictaminar",
    "dictaminarían": "dictaminar",
    "dictaminarías": "dictaminar",
    "dictaminas": "dictaminar",
    "dictaminase": "dictaminar",
    "dictaminaseis": "dictaminar",
    "dictaminasen": "dictaminar",
    "dictaminases": "dictaminar",
    "dictaminaste": "dictaminar",
    "dictaminasteis": "dictaminar",
    "dictamine": "dictaminar",
    "dictaminemos": "dictaminar",
    "dictaminen": "dictaminar",
    "dictamines": "dictaminar",
    "dictamino": "dictaminar",
    "dictaminábamos": "dictaminar",
    "dictamináis": "dictaminar",
    "dictamináramos": "dictaminar",
    "dictamináremos": "dictaminar",
    "dictaminásemos": "dictaminar",
    "dictaminé": "dictaminar",
    "dictaminéis": "dictaminar",
    "dictaminó": "dictaminar",
    "dictamos": "dictar",
    "dictan": "dictar",
    "dictando": "dictar",
    "dictara": "dictar",
    "dictarais": "dictar",
    "dictaran": "dictar",
    "dictaras": "dictar",
    "dictare": "dictar",
    "dictareis": "dictar",
    "dictaremos": "dictar",
    "dictaren": "dictar",
    "dictares": "dictar",
    "dictaron": "dictar",
    "dictará": "dictar",
    "dictarán": "dictar",
    "dictarás": "dictar",
    "dictaré": "dictar",
    "dictaréis": "dictar",
    "dictaría": "dictar",
    "dictaríais": "dictar",
    "dictaríamos": "dictar",
    "dictarían": "dictar",
    "dictarías": "dictar",
    "dictas": "dictar",
    "dictase": "dictar",
    "dictaseis": "dictar",
    "dictasen": "dictar",
    "dictases": "dictar",
    "dictaste": "dictar",
    "dictasteis": "dictar",
    "dictatoria": "dictatorio",
    "dictatoriales": "dictatorial",
    "dictatorias": "dictatorio",
    "dictatorios": "dictatorio",
    "dicte": "dictar",
    "dictemos": "dictar",
    "dicten": "dictar",
    "dicterios": "dicterio",
    "dictes": "dictar",
    "dictiosomas": "dictiosoma",
    "dicto": "dictar",
    "dictábamos": "dictar",
    "dictáfonos": "dictáfono",
    "dictáis": "dictar",
    "dictámenes": "dictamen",
    "dictáramos": "dictar",
    "dictáremos": "dictar",
    "dictásemos": "dictar",
    "dicté": "dictar",
    "dictéis": "dictar",
    "dictó": "dictar",
    "dictógrafos": "dictógrafo",
    "dicógama": "dicógamo",
    "dicógamas": "dicógamo",
    "dicógamos": "dicógamo",
    "dicótoma": "dicótomo",
    "dicótomas": "dicótomo",
    "dicótomos": "dicótomo",
    "didacticismos": "didacticismo",
    "didactismos": "didactismo",
    "didascalias": "didascalia",
    "didascálica": "didascálico",
    "didascálicas": "didascálico",
    "didascálicos": "didascálico",
    "didelfa": "didelfo",
    "didelfas": "didelfo",
    "didelfos": "didelfo",
    "didimios": "didimio",
    "didracmas": "didracma",
    "didáctica": "didáctico",
    "didácticas": "didáctico",
    "didácticos": "didáctico",
    "didáctila": "didáctilo",
    "didáctilas": "didáctilo",
    "didáctilos": "didáctilo",
    "didélfidos": "didélfido",
    "didímea": "didímeo",
    "didímeas": "didímeo",
    "didímeos": "didímeo",
    "diecinueveava": "diecinueveavo",
    "diecinueveavas": "diecinueveavo",
    "diecinueveavo": "19",
    "diecinueveavos": "diecinueveavo",
    "dieciochava": "dieciochoavo",
    "dieciochavas": "dieciochoavo",
    "dieciochavo": "dieciochoavo",
    "dieciochavos": "dieciochoavo",
    "dieciochena": "dieciocheno",
    "dieciochenas": "dieciocheno",
    "dieciochenos": "dieciocheno",
    "dieciochesca": "dieciochesco",
    "dieciochescas": "dieciochesco",
    "dieciochescos": "dieciochesco",
    "dieciochismos": "dieciochismo",
    "dieciochistas": "dieciochista",
    "dieciochoava": "dieciochoavo",
    "dieciochoavas": "dieciochoavo",
    "dieciochoavo": "18",
    "dieciochoavos": "dieciochoavo",
    "dieciseisava": "dieciseisavo",
    "dieciseisavas": "dieciseisavo",
    "dieciseisavo": "16",
    "dieciseisavos": "dieciseisavo",
    "diecisieteava": "diecisieteavo",
    "diecisieteavas": "diecisieteavo",
    "diecisieteavo": "17",
    "diecisieteavos": "diecisieteavo",
    "diedros": "diedro",
    "dieldrinas": "dieldrina",
    "dieléctrica": "dieléctrico",
    "dieléctricas": "dieléctrico",
    "dieléctricos": "dieléctrico",
    "diencéfalos": "diencéfalo",
    "dienestroles": "dienestrol",
    "dienos": "dieno",
    "dienta": "dentar",
    "dientan": "dentar",
    "dientas": "dentar",
    "diente": "dentar",
    "dienten": "dentar",
    "dientes": "diente",
    "dientimellada": "dientimellado",
    "dientimelladas": "dientimellado",
    "dientimellados": "dientimellado",
    "diento": "dentar",
    "diera": "dar",
    "dierais": "dar",
    "dieran": "dar",
    "dieras": "dar",
    "diere": "dar",
    "diereis": "dar",
    "dieren": "dar",
    "dieres": "dar",
    "dieron": "dar",
    "diese": "dar",
    "dieseis": "dar",
    "dieseleléctrica": "dieseleléctrico",
    "dieseleléctricas": "dieseleléctrico",
    "dieseleléctricos": "dieseleléctrico",
    "diesen": "dar",
    "dieses": "dar",
    "diesis": "diesi",
    "diestra": "diestro",
    "diestras": "diestro",
    "diestros": "diestro",
    "dietanolaminas": "dietanolamina",
    "dietarios": "dietario",
    "dietas": "dieta",
    "dietilaminas": "dietilamina",
    "dietilaminoetanoles": "dietilaminoetanol",
    "dietilbencenos": "dietilbenceno",
    "dietildifenilureas": "dietildifenilurea",
    "dietilditiocarbamatos": "dietilditiocarbamato",
    "dietilditiocarbámica": "dietilditiocarbámico",
    "dietilditiocarbámicas": "dietilditiocarbámico",
    "dietilditiocarbámicos": "dietilditiocarbámico",
    "dietilentriaminas": "dietilentriamina",
    "dietilestilbestroles": "dietilestilbestrol",
    "dietilnitrosaminas": "dietilnitrosamina",
    "dietistas": "dietista",
    "dietologías": "dietología",
    "dietética": "dietético",
    "dietéticas": "dietético",
    "dietéticos": "dietético",
    "diezma": "diezmar",
    "diezmaba": "diezmar",
    "diezmabais": "diezmar",
    "diezmaban": "diezmar",
    "diezmabas": "diezmar",
    "diezmad": "diezmar",
    "diezmada": "diezmar",
    "diezmadas": "diezmar",
    "diezmado": "diezmar",
    "diezmados": "diezmar",
    "diezmamos": "diezmar",
    "diezman": "diezmar",
    "diezmando": "diezmar",
    "diezmara": "diezmar",
    "diezmarais": "diezmar",
    "diezmaran": "diezmar",
    "diezmaras": "diezmar",
    "diezmare": "diezmar",
    "diezmareis": "diezmar",
    "diezmaremos": "diezmar",
    "diezmaren": "diezmar",
    "diezmares": "diezmar",
    "diezmaron": "diezmar",
    "diezmará": "diezmar",
    "diezmarán": "diezmar",
    "diezmarás": "diezmar",
    "diezmaré": "diezmar",
    "diezmaréis": "diezmar",
    "diezmaría": "diezmar",
    "diezmaríais": "diezmar",
    "diezmaríamos": "diezmar",
    "diezmarían": "diezmar",
    "diezmarías": "diezmar",
    "diezmas": "diezmar",
    "diezmase": "diezmar",
    "diezmaseis": "diezmar",
    "diezmasen": "diezmar",
    "diezmases": "diezmar",
    "diezmaste": "diezmar",
    "diezmasteis": "diezmar",
    "diezme": "diezmar",
    "diezmemos": "diezmar",
    "diezmen": "diezmar",
    "diezmes": "diezmar",
    "diezmesina": "diezmesino",
    "diezmesinas": "diezmesino",
    "diezmesinos": "diezmesino",
    "diezmillonésima": "diezmillonésimo",
    "diezmillonésimas": "diezmillonésimo",
    "diezmillonésimo": "10000000",
    "diezmillonésimos": "diezmillonésimo",
    "diezmilmillonésima": "diezmilmillonésimo",
    "diezmilmillonésimas": "diezmilmillonésimo",
    "diezmilmillonésimo": "10000000000",
    "diezmilmillonésimos": "diezmilmillonésimo",
    "diezmilésima": "diezmilésimo",
    "diezmilésimas": "diezmilésimo",
    "diezmilésimo": "10000",
    "diezmilésimos": "diezmilésimo",
    "diezmilímetros": "diezmilímetro",
    "diezmo": "diezmar",
    "diezmos": "diezmo",
    "diezmábamos": "diezmar",
    "diezmáis": "diezmar",
    "diezmáramos": "diezmar",
    "diezmáremos": "diezmar",
    "diezmásemos": "diezmar",
    "diezmé": "diezmar",
    "diezméis": "diezmar",
    "diezmó": "diezmar",
    "difama": "difamar",
    "difamaba": "difamar",
    "difamabais": "difamar",
    "difamaban": "difamar",
    "difamabas": "difamar",
    "difamaciones": "difamación",
    "difamad": "difamar",
    "difamada": "difamar",
    "difamadas": "difamar",
    "difamado": "difamar",
    "difamadora": "difamador",
    "difamadoras": "difamador",
    "difamadores": "difamador",
    "difamados": "difamar",
    "difamamos": "difamar",
    "difaman": "difamar",
    "difamando": "difamar",
    "difamara": "difamar",
    "difamarais": "difamar",
    "difamaran": "difamar",
    "difamaras": "difamar",
    "difamare": "difamar",
    "difamareis": "difamar",
    "difamaremos": "difamar",
    "difamaren": "difamar",
    "difamares": "difamar",
    "difamaron": "difamar",
    "difamará": "difamar",
    "difamarán": "difamar",
    "difamarás": "difamar",
    "difamaré": "difamar",
    "difamaréis": "difamar",
    "difamaría": "difamar",
    "difamaríais": "difamar",
    "difamaríamos": "difamar",
    "difamarían": "difamar",
    "difamarías": "difamar",
    "difamas": "difamar",
    "difamase": "difamar",
    "difamaseis": "difamar",
    "difamasen": "difamar",
    "difamases": "difamar",
    "difamaste": "difamar",
    "difamasteis": "difamar",
    "difamatoria": "difamatorio",
    "difamatorias": "difamatorio",
    "difamatorios": "difamatorio",
    "difame": "difamar",
    "difamemos": "difamar",
    "difamen": "difamar",
    "difames": "difamar",
    "difamo": "difamar",
    "difamábamos": "difamar",
    "difamáis": "difamar",
    "difamáramos": "difamar",
    "difamáremos": "difamar",
    "difamásemos": "difamar",
    "difamé": "difamar",
    "difaméis": "difamar",
    "difamó": "difamar",
    "difarreaciones": "difarreación",
    "difenilacetonitrilos": "difenilacetonitrilo",
    "difenilaminas": "difenilamina",
    "difenilbencidinas": "difenilbencidina",
    "difenilcarbacidas": "difenilcarbacida",
    "difenildiclorosilanos": "difenildiclorosilano",
    "difenilguanidinas": "difenilguanidina",
    "difenilhidantoínas": "difenilhidantoína",
    "difenilmetanos": "difenilmetano",
    "difenilnaftilendiaminas": "difenilnaftilendiamina",
    "difenilos": "difenilo",
    "difenilsilanodioles": "difenilsilanodiol",
    "diferencia": "diferenciar",
    "diferenciaba": "diferenciar",
    "diferenciabais": "diferenciar",
    "diferenciaban": "diferenciar",
    "diferenciabas": "diferenciar",
    "diferenciables": "diferenciable",
    "diferenciaciones": "diferenciación",
    "diferenciad": "diferenciar",
    "diferenciada": "diferenciar",
    "diferenciadas": "diferenciar",
    "diferenciado": "diferenciar",
    "diferenciadora": "diferenciador",
    "diferenciadoras": "diferenciador",
    "diferenciadores": "diferenciador",
    "diferenciados": "diferenciar",
    "diferenciales": "diferencial",
    "diferenciamos": "diferenciar",
    "diferencian": "diferenciar",
    "diferenciando": "diferenciar",
    "diferenciara": "diferenciar",
    "diferenciarais": "diferenciar",
    "diferenciaran": "diferenciar",
    "diferenciaras": "diferenciar",
    "diferenciare": "diferenciar",
    "diferenciareis": "diferenciar",
    "diferenciaremos": "diferenciar",
    "diferenciaren": "diferenciar",
    "diferenciares": "diferenciar",
    "diferenciaron": "diferenciar",
    "diferenciará": "diferenciar",
    "diferenciarán": "diferenciar",
    "diferenciarás": "diferenciar",
    "diferenciaré": "diferenciar",
    "diferenciaréis": "diferenciar",
    "diferenciaría": "diferenciar",
    "diferenciaríais": "diferenciar",
    "diferenciaríamos": "diferenciar",
    "diferenciarían": "diferenciar",
    "diferenciarías": "diferenciar",
    "diferencias": "diferenciar",
    "diferenciase": "diferenciar",
    "diferenciaseis": "diferenciar",
    "diferenciasen": "diferenciar",
    "diferenciases": "diferenciar",
    "diferenciaste": "diferenciar",
    "diferenciasteis": "diferenciar",
    "diferencie": "diferenciar",
    "diferenciemos": "diferenciar",
    "diferencien": "diferenciar",
    "diferencies": "diferenciar",
    "diferencio": "diferenciar",
    "diferenciábamos": "diferenciar",
    "diferenciáis": "diferenciar",
    "diferenciáramos": "diferenciar",
    "diferenciáremos": "diferenciar",
    "diferenciásemos": "diferenciar",
    "diferencié": "diferenciar",
    "diferenciéis": "diferenciar",
    "diferenció": "diferenciar",
    "diferencógrafos": "diferencógrafo",
    "diferendos": "diferendo",
    "diferentes": "diferente",
    "diferid": "diferir",
    "diferida": "diferir",
    "diferidas": "diferir",
    "diferido": "diferir",
    "diferidos": "diferir",
    "diferimos": "diferir",
    "diferiremos": "diferir",
    "diferirá": "diferir",
    "diferirán": "diferir",
    "diferirás": "diferir",
    "diferiré": "diferir",
    "diferiréis": "diferir",
    "diferiría": "diferir",
    "diferiríais": "diferir",
    "diferiríamos": "diferir",
    "diferirían": "diferir",
    "diferirías": "diferir",
    "diferiste": "diferir",
    "diferisteis": "diferir",
    "diferí": "diferir",
    "difería": "diferir",
    "diferíais": "diferir",
    "diferíamos": "diferir",
    "diferían": "diferir",
    "diferías": "diferir",
    "diferís": "diferir",
    "dificulta": "dificultar",
    "dificultaba": "dificultar",
    "dificultabais": "dificultar",
    "dificultaban": "dificultar",
    "dificultabas": "dificultar",
    "dificultad": "dificultar",
    "dificultada": "dificultar",
    "dificultadas": "dificultar",
    "dificultades": "dificultad",
    "dificultado": "dificultar",
    "dificultadora": "dificultador",
    "dificultadoras": "dificultador",
    "dificultadores": "dificultador",
    "dificultados": "dificultar",
    "dificultamos": "dificultar",
    "dificultan": "dificultar",
    "dificultando": "dificultar",
    "dificultara": "dificultar",
    "dificultarais": "dificultar",
    "dificultaran": "dificultar",
    "dificultaras": "dificultar",
    "dificultare": "dificultar",
    "dificultareis": "dificultar",
    "dificultaremos": "dificultar",
    "dificultaren": "dificultar",
    "dificultares": "dificultar",
    "dificultaron": "dificultar",
    "dificultará": "dificultar",
    "dificultarán": "dificultar",
    "dificultarás": "dificultar",
    "dificultaré": "dificultar",
    "dificultaréis": "dificultar",
    "dificultaría": "dificultar",
    "dificultaríais": "dificultar",
    "dificultaríamos": "dificultar",
    "dificultarían": "dificultar",
    "dificultarías": "dificultar",
    "dificultas": "dificultar",
    "dificultase": "dificultar",
    "dificultaseis": "dificultar",
    "dificultasen": "dificultar",
    "dificultases": "dificultar",
    "dificultaste": "dificultar",
    "dificultasteis": "dificultar",
    "dificulte": "dificultar",
    "dificultemos": "dificultar",
    "dificulten": "dificultar",
    "dificultes": "dificultar",
    "dificulto": "dificultar",
    "dificultosa": "dificultoso",
    "dificultosas": "dificultoso",
    "dificultosos": "dificultoso",
    "dificultábamos": "dificultar",
    "dificultáis": "dificultar",
    "dificultáramos": "dificultar",
    "dificultáremos": "dificultar",
    "dificultásemos": "dificultar",
    "dificulté": "dificultar",
    "dificultéis": "dificultar",
    "dificultó": "dificultar",
    "difidaciones": "difidación",
    "difidencias": "difidencia",
    "difidentes": "difidente",
    "difiera": "diferir",
    "difieran": "diferir",
    "difieras": "diferir",
    "difiere": "diferir",
    "difieren": "diferir",
    "difieres": "diferir",
    "difiero": "diferir",
    "difilinas": "difilina",
    "difiodonta": "difiodonto",
    "difiodontas": "difiodonto",
    "difiodontos": "difiodonto",
    "difiramos": "diferir",
    "difiriendo": "diferir",
    "difiriera": "diferir",
    "difirierais": "diferir",
    "difirieran": "diferir",
    "difirieras": "diferir",
    "difiriere": "diferir",
    "difiriereis": "diferir",
    "difirieren": "diferir",
    "difirieres": "diferir",
    "difirieron": "diferir",
    "difiriese": "diferir",
    "difirieseis": "diferir",
    "difiriesen": "diferir",
    "difirieses": "diferir",
    "difiriéramos": "diferir",
    "difiriéremos": "diferir",
    "difiriésemos": "diferir",
    "difirió": "diferir",
    "difiráis": "diferir",
    "difluencias": "difluencia",
    "difluentes": "difluente",
    "difluid": "difluir",
    "difluida": "difluir",
    "difluidas": "difluir",
    "difluido": "difluir",
    "difluidos": "difluir",
    "difluimos": "difluir",
    "difluiremos": "difluir",
    "difluirá": "difluir",
    "difluirán": "difluir",
    "difluirás": "difluir",
    "difluiré": "difluir",
    "difluiréis": "difluir",
    "difluiría": "difluir",
    "difluiríais": "difluir",
    "difluiríamos": "difluir",
    "difluirían": "difluir",
    "difluirías": "difluir",
    "difluiste": "difluir",
    "difluisteis": "difluir",
    "difluya": "difluir",
    "difluyamos": "difluir",
    "difluyan": "difluir",
    "difluyas": "difluir",
    "difluye": "difluir",
    "difluyen": "difluir",
    "difluyendo": "difluir",
    "difluyera": "difluir",
    "difluyerais": "difluir",
    "difluyeran": "difluir",
    "difluyeras": "difluir",
    "difluyere": "difluir",
    "difluyereis": "difluir",
    "difluyeren": "difluir",
    "difluyeres": "difluir",
    "difluyeron": "difluir",
    "difluyes": "difluir",
    "difluyese": "difluir",
    "difluyeseis": "difluir",
    "difluyesen": "difluir",
    "difluyeses": "difluir",
    "difluyo": "difluir",
    "difluyáis": "difluir",
    "difluyéramos": "difluir",
    "difluyéremos": "difluir",
    "difluyésemos": "difluir",
    "difluyó": "difluir",
    "difluí": "difluir",
    "difluía": "difluir",
    "difluíais": "difluir",
    "difluíamos": "difluir",
    "difluían": "difluir",
    "difluías": "difluir",
    "difluís": "difluir",
    "difonemas": "difonema",
    "difosfatos": "difosfato",
    "difracciones": "difracción",
    "difracta": "difractar",
    "difractaba": "difractar",
    "difractabais": "difractar",
    "difractaban": "difractar",
    "difractabas": "difractar",
    "difractaciones": "difractación",
    "difractad": "difractar",
    "difractada": "difractar",
    "difractadas": "difractar",
    "difractado": "difractar",
    "difractados": "difractar",
    "difractamos": "difractar",
    "difractan": "difractar",
    "difractando": "difractar",
    "difractara": "difractar",
    "difractarais": "difractar",
    "difractaran": "difractar",
    "difractaras": "difractar",
    "difractare": "difractar",
    "difractareis": "difractar",
    "difractaremos": "difractar",
    "difractaren": "difractar",
    "difractares": "difractar",
    "difractaron": "difractar",
    "difractará": "difractar",
    "difractarán": "difractar",
    "difractarás": "difractar",
    "difractaré": "difractar",
    "difractaréis": "difractar",
    "difractaría": "difractar",
    "difractaríais": "difractar",
    "difractaríamos": "difractar",
    "difractarían": "difractar",
    "difractarías": "difractar",
    "difractas": "difractar",
    "difractase": "difractar",
    "difractaseis": "difractar",
    "difractasen": "difractar",
    "difractases": "difractar",
    "difractaste": "difractar",
    "difractasteis": "difractar",
    "difracte": "difractar",
    "difractemos": "difractar",
    "difracten": "difractar",
    "difractes": "difractar",
    "difracto": "difractar",
    "difractábamos": "difractar",
    "difractáis": "difractar",
    "difractáramos": "difractar",
    "difractáremos": "difractar",
    "difractásemos": "difractar",
    "difracté": "difractar",
    "difractéis": "difractar",
    "difractó": "difractar",
    "difractómetros": "difractómetro",
    "difrangentes": "difrangente",
    "difterias": "difteria",
    "diftérica": "diftérico",
    "diftéricas": "diftérico",
    "diftéricos": "diftérico",
    "difuma": "difumar",
    "difumaba": "difumar",
    "difumabais": "difumar",
    "difumaban": "difumar",
    "difumabas": "difumar",
    "difumad": "difumar",
    "difumada": "difumar",
    "difumadas": "difumar",
    "difumado": "difumar",
    "difumados": "difumar",
    "difumamos": "difumar",
    "difuman": "difumar",
    "difumando": "difumar",
    "difumara": "difumar",
    "difumarais": "difumar",
    "difumaran": "difumar",
    "difumaras": "difumar",
    "difumare": "difumar",
    "difumareis": "difumar",
    "difumaremos": "difumar",
    "difumaren": "difumar",
    "difumares": "difumar",
    "difumaron": "difumar",
    "difumará": "difumar",
    "difumarán": "difumar",
    "difumarás": "difumar",
    "difumaré": "difumar",
    "difumaréis": "difumar",
    "difumaría": "difumar",
    "difumaríais": "difumar",
    "difumaríamos": "difumar",
    "difumarían": "difumar",
    "difumarías": "difumar",
    "difumas": "difumar",
    "difumase": "difumar",
    "difumaseis": "difumar",
    "difumasen": "difumar",
    "difumases": "difumar",
    "difumaste": "difumar",
    "difumasteis": "difumar",
    "difume": "difumar",
    "difumemos": "difumar",
    "difumen": "difumar",
    "difumes": "difumar",
    "difumina": "difuminar",
    "difuminaba": "difuminar",
    "difuminabais": "difuminar",
    "difuminaban": "difuminar",
    "difuminabas": "difuminar",
    "difuminaciones": "difuminación",
    "difuminad": "difuminar",
    "difuminada": "difuminar",
    "difuminadas": "difuminar",
    "difuminado": "difuminar",
    "difuminadora": "difuminador",
    "difuminadoras": "difuminador",
    "difuminadores": "difuminador",
    "difuminados": "difuminar",
    "difuminamos": "difuminar",
    "difuminan": "difuminar",
    "difuminando": "difuminar",
    "difuminara": "difuminar",
    "difuminarais": "difuminar",
    "difuminaran": "difuminar",
    "difuminaras": "difuminar",
    "difuminare": "difuminar",
    "difuminareis": "difuminar",
    "difuminaremos": "difuminar",
    "difuminaren": "difuminar",
    "difuminares": "difuminar",
    "difuminaron": "difuminar",
    "difuminará": "difuminar",
    "difuminarán": "difuminar",
    "difuminarás": "difuminar",
    "difuminaré": "difuminar",
    "difuminaréis": "difuminar",
    "difuminaría": "difuminar",
    "difuminaríais": "difuminar",
    "difuminaríamos": "difuminar",
    "difuminarían": "difuminar",
    "difuminarías": "difuminar",
    "difuminas": "difuminar",
    "difuminase": "difuminar",
    "difuminaseis": "difuminar",
    "difuminasen": "difuminar",
    "difuminases": "difuminar",
    "difuminaste": "difuminar",
    "difuminasteis": "difuminar",
    "difumine": "difuminar",
    "difuminemos": "difuminar",
    "difuminen": "difuminar",
    "difumines": "difuminar",
    "difumino": "difuminar",
    "difuminos": "difumino",
    "difuminábamos": "difuminar",
    "difumináis": "difuminar",
    "difumináramos": "difuminar",
    "difumináremos": "difuminar",
    "difuminásemos": "difuminar",
    "difuminé": "difuminar",
    "difuminéis": "difuminar",
    "difuminó": "difuminar",
    "difumo": "difumar",
    "difumábamos": "difumar",
    "difumáis": "difumar",
    "difumáramos": "difumar",
    "difumáremos": "difumar",
    "difumásemos": "difumar",
    "difumé": "difumar",
    "difuméis": "difumar",
    "difumó": "difumar",
    "difunda": "difundir",
    "difundamos": "difundir",
    "difundan": "difundir",
    "difundas": "difundir",
    "difunde": "difundir",
    "difunden": "difundir",
    "difundes": "difundir",
    "difundid": "difundir",
    "difundida": "difundir",
    "difundidas": "difundir",
    "difundido": "difundir",
    "difundidos": "difundir",
    "difundiendo": "difundir",
    "difundiera": "difundir",
    "difundierais": "difundir",
    "difundieran": "difundir",
    "difundieras": "difundir",
    "difundiere": "difundir",
    "difundiereis": "difundir",
    "difundieren": "difundir",
    "difundieres": "difundir",
    "difundieron": "difundir",
    "difundiese": "difundir",
    "difundieseis": "difundir",
    "difundiesen": "difundir",
    "difundieses": "difundir",
    "difundimos": "difundir",
    "difundiremos": "difundir",
    "difundirá": "difundir",
    "difundirán": "difundir",
    "difundirás": "difundir",
    "difundiré": "difundir",
    "difundiréis": "difundir",
    "difundiría": "difundir",
    "difundiríais": "difundir",
    "difundiríamos": "difundir",
    "difundirían": "difundir",
    "difundirías": "difundir",
    "difundiste": "difundir",
    "difundisteis": "difundir",
    "difundiéramos": "difundir",
    "difundiéremos": "difundir",
    "difundiésemos": "difundir",
    "difundió": "difundir",
    "difundo": "difundir",
    "difundáis": "difundir",
    "difundí": "difundir",
    "difundía": "difundir",
    "difundíais": "difundir",
    "difundíamos": "difundir",
    "difundían": "difundir",
    "difundías": "difundir",
    "difundís": "difundir",
    "difunta": "difunto",
    "difuntas": "difunto",
    "difuntos": "difunto",
    "difusa": "difuso",
    "difusas": "difuso",
    "difusibilidades": "difusibilidad",
    "difusionales": "difusional",
    "difusiones": "difusión",
    "difusionismos": "difusionismo",
    "difusiva": "difusivo",
    "difusivas": "difusivo",
    "difusividades": "difusividad",
    "difusivos": "difusivo",
    "difusora": "difusor",
    "difusoras": "difusor",
    "difusores": "difusor",
    "difusos": "difuso",
    "difásica": "difásico",
    "difásicas": "difásico",
    "difásicos": "difásico",
    "difénica": "difénico",
    "difénicas": "difénico",
    "difénicos": "difénico",
    "difíciles": "difícil",
    "diga": "decir",
    "digametias": "digametia",
    "digamos": "decir",
    "digamética": "digamético",
    "digaméticas": "digamético",
    "digaméticos": "digamético",
    "digan": "decir",
    "digas": "decir",
    "digeribles": "digerible",
    "digerid": "digerir",
    "digerida": "digerir",
    "digeridas": "digerir",
    "digerido": "digerir",
    "digeridos": "digerir",
    "digerimos": "digerir",
    "digeriremos": "digerir",
    "digerirá": "digerir",
    "digerirán": "digerir",
    "digerirás": "digerir",
    "digeriré": "digerir",
    "digeriréis": "digerir",
    "digeriría": "digerir",
    "digeriríais": "digerir",
    "digeriríamos": "digerir",
    "digerirían": "digerir",
    "digerirías": "digerir",
    "digeriste": "digerir",
    "digeristeis": "digerir",
    "digerí": "digerir",
    "digería": "digerir",
    "digeríais": "digerir",
    "digeríamos": "digerir",
    "digerían": "digerir",
    "digerías": "digerir",
    "digerís": "digerir",
    "digestibilidades": "digestibilidad",
    "digestibles": "digestible",
    "digestiones": "digestión",
    "digestiva": "digestivo",
    "digestivas": "digestivo",
    "digestivos": "digestivo",
    "digestores": "digestor",
    "digestos": "digesto",
    "digestóloga": "digestólogo",
    "digestólogas": "digestólogo",
    "digestólogos": "digestólogo",
    "digiera": "digerir",
    "digieran": "digerir",
    "digieras": "digerir",
    "digiere": "digerir",
    "digieren": "digerir",
    "digieres": "digerir",
    "digiero": "digerir",
    "digiramos": "digerir",
    "digiriendo": "digerir",
    "digiriera": "digerir",
    "digirierais": "digerir",
    "digirieran": "digerir",
    "digirieras": "digerir",
    "digiriere": "digerir",
    "digiriereis": "digerir",
    "digirieren": "digerir",
    "digirieres": "digerir",
    "digirieron": "digerir",
    "digiriese": "digerir",
    "digirieseis": "digerir",
    "digiriesen": "digerir",
    "digirieses": "digerir",
    "digiriéramos": "digerir",
    "digiriéremos": "digerir",
    "digiriésemos": "digerir",
    "digirió": "digerir",
    "digiráis": "digerir",
    "digitaciones": "digitación",
    "digitada": "digitado",
    "digitadas": "digitado",
    "digitados": "digitado",
    "digitales": "digital",
    "digitalice": "digitalizar",
    "digitalicemos": "digitalizar",
    "digitalicen": "digitalizar",
    "digitalices": "digitalizar",
    "digitalicé": "digitalizar",
    "digitalicéis": "digitalizar",
    "digitalinas": "digitalina",
    "digitalismos": "digitalismo",
    "digitaliza": "digitalizar",
    "digitalizaba": "digitalizar",
    "digitalizabais": "digitalizar",
    "digitalizaban": "digitalizar",
    "digitalizabas": "digitalizar",
    "digitalizaciones": "digitalización",
    "digitalizad": "digitalizar",
    "digitalizada": "digitalizar",
    "digitalizadas": "digitalizar",
    "digitalizado": "digitalizar",
    "digitalizadora": "digitalizador",
    "digitalizadoras": "digitalizador",
    "digitalizadores": "digitalizador",
    "digitalizados": "digitalizar",
    "digitalizamos": "digitalizar",
    "digitalizan": "digitalizar",
    "digitalizando": "digitalizar",
    "digitalizara": "digitalizar",
    "digitalizarais": "digitalizar",
    "digitalizaran": "digitalizar",
    "digitalizaras": "digitalizar",
    "digitalizare": "digitalizar",
    "digitalizareis": "digitalizar",
    "digitalizaremos": "digitalizar",
    "digitalizaren": "digitalizar",
    "digitalizares": "digitalizar",
    "digitalizaron": "digitalizar",
    "digitalizará": "digitalizar",
    "digitalizarán": "digitalizar",
    "digitalizarás": "digitalizar",
    "digitalizaré": "digitalizar",
    "digitalizaréis": "digitalizar",
    "digitalizaría": "digitalizar",
    "digitalizaríais": "digitalizar",
    "digitalizaríamos": "digitalizar",
    "digitalizarían": "digitalizar",
    "digitalizarías": "digitalizar",
    "digitalizas": "digitalizar",
    "digitalizase": "digitalizar",
    "digitalizaseis": "digitalizar",
    "digitalizasen": "digitalizar",
    "digitalizases": "digitalizar",
    "digitalizaste": "digitalizar",
    "digitalizasteis": "digitalizar",
    "digitalizo": "digitalizar",
    "digitalizábamos": "digitalizar",
    "digitalizáis": "digitalizar",
    "digitalizáramos": "digitalizar",
    "digitalizáremos": "digitalizar",
    "digitalizásemos": "digitalizar",
    "digitalizó": "digitalizar",
    "digitiformes": "digitiforme",
    "digitoxinas": "digitoxina",
    "digitálica": "digitálico",
    "digitálicas": "digitálico",
    "digitálicos": "digitálico",
    "digitígrada": "digitígrado",
    "digitígradas": "digitígrado",
    "digitígrados": "digitígrado",
    "diglicólica": "diglicólico",
    "diglicólicas": "diglicólico",
    "diglicólicos": "diglicólico",
    "diglosias": "diglosia",
    "digna": "digno",
    "dignaba": "dignar",
    "dignabais": "dignar",
    "dignaban": "dignar",
    "dignabas": "dignar",
    "dignaciones": "dignación",
    "dignad": "dignar",
    "dignada": "dignar",
    "dignadas": "dignar",
    "dignado": "dignar",
    "dignados": "dignar",
    "dignamos": "dignar",
    "dignan": "dignar",
    "dignando": "dignar",
    "dignara": "dignar",
    "dignarais": "dignar",
    "dignaran": "dignar",
    "dignaras": "dignar",
    "dignare": "dignar",
    "dignareis": "dignar",
    "dignaremos": "dignar",
    "dignaren": "dignar",
    "dignares": "dignar",
    "dignaron": "dignar",
    "dignará": "dignar",
    "dignarán": "dignar",
    "dignarás": "dignar",
    "dignaré": "dignar",
    "dignaréis": "dignar",
    "dignaría": "dignar",
    "dignaríais": "dignar",
    "dignaríamos": "dignar",
    "dignarían": "dignar",
    "dignarías": "dignar",
    "dignas": "digno",
    "dignase": "dignar",
    "dignaseis": "dignar",
    "dignasen": "dignar",
    "dignases": "dignar",
    "dignaste": "dignar",
    "dignasteis": "dignar",
    "dignataria": "dignatario",
    "dignatarias": "dignatario",
    "dignatarios": "dignatario",
    "digne": "dignar",
    "dignemos": "dignar",
    "dignen": "dignar",
    "dignes": "dignar",
    "dignidades": "dignidad",
    "dignifica": "dignificar",
    "dignificaba": "dignificar",
    "dignificabais": "dignificar",
    "dignificaban": "dignificar",
    "dignificabas": "dignificar",
    "dignificables": "dignificable",
    "dignificaciones": "dignificación",
    "dignificad": "dignificar",
    "dignificada": "dignificar",
    "dignificadas": "dignificar",
    "dignificado": "dignificar",
    "dignificados": "dignificar",
    "dignificamos": "dignificar",
    "dignifican": "dignificar",
    "dignificando": "dignificar",
    "dignificara": "dignificar",
    "dignificarais": "dignificar",
    "dignificaran": "dignificar",
    "dignificaras": "dignificar",
    "dignificare": "dignificar",
    "dignificareis": "dignificar",
    "dignificaremos": "dignificar",
    "dignificaren": "dignificar",
    "dignificares": "dignificar",
    "dignificaron": "dignificar",
    "dignificará": "dignificar",
    "dignificarán": "dignificar",
    "dignificarás": "dignificar",
    "dignificaré": "dignificar",
    "dignificaréis": "dignificar",
    "dignificaría": "dignificar",
    "dignificaríais": "dignificar",
    "dignificaríamos": "dignificar",
    "dignificarían": "dignificar",
    "dignificarías": "dignificar",
    "dignificas": "dignificar",
    "dignificase": "dignificar",
    "dignificaseis": "dignificar",
    "dignificasen": "dignificar",
    "dignificases": "dignificar",
    "dignificaste": "dignificar",
    "dignificasteis": "dignificar",
    "dignifico": "dignificar",
    "dignificábamos": "dignificar",
    "dignificáis": "dignificar",
    "dignificáramos": "dignificar",
    "dignificáremos": "dignificar",
    "dignificásemos": "dignificar",
    "dignificó": "dignificar",
    "dignifique": "dignificar",
    "dignifiquemos": "dignificar",
    "dignifiquen": "dignificar",
    "dignifiques": "dignificar",
    "dignifiqué": "dignificar",
    "dignifiquéis": "dignificar",
    "digno": "dignar",
    "dignos": "digno",
    "dignábamos": "dignar",
    "dignáis": "dignar",
    "dignáramos": "dignar",
    "dignáremos": "dignar",
    "dignásemos": "dignar",
    "digné": "dignar",
    "dignéis": "dignar",
    "dignó": "dignar",
    "digo": "decir",
    "digresiones": "digresión",
    "digresiva": "digresivo",
    "digresivas": "digresivo",
    "digresivos": "digresivo",
    "digáis": "decir",
    "digástrica": "digástrico",
    "digástricas": "digástrico",
    "digástricos": "digástrico",
    "dihidrocolesteroles": "dihidrocolesterol",
    "dihidroestreptomicinas": "dihidroestreptomicina",
    "dihidroxiacetonas": "dihidroxiacetona",
    "diisobutilaminas": "diisobutilamina",
    "diisobutilenos": "diisobutileno",
    "diisocianatos": "diisocianato",
    "diisopropanolaminas": "diisopropanolamina",
    "diisopropiltioureas": "diisopropiltiourea",
    "dije": "decir",
    "dijera": "decir",
    "dijerais": "decir",
    "dijeran": "decir",
    "dijeras": "decir",
    "dijere": "decir",
    "dijereis": "decir",
    "dijeren": "decir",
    "dijeres": "decir",
    "dijeron": "decir",
    "dijes": "dije",
    "dijese": "decir",
    "dijeseis": "decir",
    "dijesen": "decir",
    "dijeses": "decir",
    "dijimos": "decir",
    "dijiste": "decir",
    "dijisteis": "decir",
    "dijo": "decir",
    "dijéramos": "decir",
    "dijéremos": "decir",
    "dijésemos": "decir",
    "dilacera": "dilacerar",
    "dilaceraba": "dilacerar",
    "dilacerabais": "dilacerar",
    "dilaceraban": "dilacerar",
    "dilacerabas": "dilacerar",
    "dilaceraciones": "dilaceración",
    "dilacerad": "dilacerar",
    "dilacerada": "dilacerar",
    "dilaceradas": "dilacerar",
    "dilacerado": "dilacerar",
    "dilaceradores": "dilacerador",
    "dilacerados": "dilacerar",
    "dilaceramos": "dilacerar",
    "dilaceran": "dilacerar",
    "dilacerando": "dilacerar",
    "dilacerara": "dilacerar",
    "dilacerarais": "dilacerar",
    "dilaceraran": "dilacerar",
    "dilaceraras": "dilacerar",
    "dilacerare": "dilacerar",
    "dilacerareis": "dilacerar",
    "dilaceraremos": "dilacerar",
    "dilaceraren": "dilacerar",
    "dilacerares": "dilacerar",
    "dilaceraron": "dilacerar",
    "dilacerará": "dilacerar",
    "dilacerarán": "dilacerar",
    "dilacerarás": "dilacerar",
    "dilaceraré": "dilacerar",
    "dilaceraréis": "dilacerar",
    "dilaceraría": "dilacerar",
    "dilaceraríais": "dilacerar",
    "dilaceraríamos": "dilacerar",
    "dilacerarían": "dilacerar",
    "dilacerarías": "dilacerar",
    "dilaceras": "dilacerar",
    "dilacerase": "dilacerar",
    "dilaceraseis": "dilacerar",
    "dilacerasen": "dilacerar",
    "dilacerases": "dilacerar",
    "dilaceraste": "dilacerar",
    "dilacerasteis": "dilacerar",
    "dilacere": "dilacerar",
    "dilaceremos": "dilacerar",
    "dilaceren": "dilacerar",
    "dilaceres": "dilacerar",
    "dilacero": "dilacerar",
    "dilacerábamos": "dilacerar",
    "dilaceráis": "dilacerar",
    "dilaceráramos": "dilacerar",
    "dilaceráremos": "dilacerar",
    "dilacerásemos": "dilacerar",
    "dilaceré": "dilacerar",
    "dilaceréis": "dilacerar",
    "dilaceró": "dilacerar",
    "dilaciones": "dilación",
    "dilapida": "dilapidar",
    "dilapidaba": "dilapidar",
    "dilapidabais": "dilapidar",
    "dilapidaban": "dilapidar",
    "dilapidabas": "dilapidar",
    "dilapidaciones": "dilapidación",
    "dilapidad": "dilapidar",
    "dilapidada": "dilapidar",
    "dilapidadas": "dilapidar",
    "dilapidado": "dilapidar",
    "dilapidadora": "dilapidador",
    "dilapidadoras": "dilapidador",
    "dilapidadores": "dilapidador",
    "dilapidados": "dilapidar",
    "dilapidamos": "dilapidar",
    "dilapidan": "dilapidar",
    "dilapidando": "dilapidar",
    "dilapidara": "dilapidar",
    "dilapidarais": "dilapidar",
    "dilapidaran": "dilapidar",
    "dilapidaras": "dilapidar",
    "dilapidare": "dilapidar",
    "dilapidareis": "dilapidar",
    "dilapidaremos": "dilapidar",
    "dilapidaren": "dilapidar",
    "dilapidares": "dilapidar",
    "dilapidaron": "dilapidar",
    "dilapidará": "dilapidar",
    "dilapidarán": "dilapidar",
    "dilapidarás": "dilapidar",
    "dilapidaré": "dilapidar",
    "dilapidaréis": "dilapidar",
    "dilapidaría": "dilapidar",
    "dilapidaríais": "dilapidar",
    "dilapidaríamos": "dilapidar",
    "dilapidarían": "dilapidar",
    "dilapidarías": "dilapidar",
    "dilapidas": "dilapidar",
    "dilapidase": "dilapidar",
    "dilapidaseis": "dilapidar",
    "dilapidasen": "dilapidar",
    "dilapidases": "dilapidar",
    "dilapidaste": "dilapidar",
    "dilapidasteis": "dilapidar",
    "dilapide": "dilapidar",
    "dilapidemos": "dilapidar",
    "dilapiden": "dilapidar",
    "dilapides": "dilapidar",
    "dilapido": "dilapidar",
    "dilapidábamos": "dilapidar",
    "dilapidáis": "dilapidar",
    "dilapidáramos": "dilapidar",
    "dilapidáremos": "dilapidar",
    "dilapidásemos": "dilapidar",
    "dilapidé": "dilapidar",
    "dilapidéis": "dilapidar",
    "dilapidó": "dilapidar",
    "dilata": "dilatar",
    "dilataba": "dilatar",
    "dilatabais": "dilatar",
    "dilataban": "dilatar",
    "dilatabas": "dilatar",
    "dilatabilidades": "dilatabilidad",
    "dilatables": "dilatable",
    "dilataciones": "dilatación",
    "dilatad": "dilatar",
    "dilatada": "dilatar",
    "dilatadas": "dilatar",
    "dilatado": "dilatar",
    "dilatadora": "dilatador",
    "dilatadoras": "dilatador",
    "dilatadores": "dilatador",
    "dilatados": "dilatar",
    "dilatamos": "dilatar",
    "dilatan": "dilatar",
    "dilatancias": "dilatancia",
    "dilatando": "dilatar",
    "dilatara": "dilatar",
    "dilatarais": "dilatar",
    "dilataran": "dilatar",
    "dilataras": "dilatar",
    "dilatare": "dilatar",
    "dilatareis": "dilatar",
    "dilataremos": "dilatar",
    "dilataren": "dilatar",
    "dilatares": "dilatar",
    "dilataron": "dilatar",
    "dilatará": "dilatar",
    "dilatarán": "dilatar",
    "dilatarás": "dilatar",
    "dilataré": "dilatar",
    "dilataréis": "dilatar",
    "dilataría": "dilatar",
    "dilataríais": "dilatar",
    "dilataríamos": "dilatar",
    "dilatarían": "dilatar",
    "dilatarías": "dilatar",
    "dilatas": "dilatar",
    "dilatase": "dilatar",
    "dilataseis": "dilatar",
    "dilatasen": "dilatar",
    "dilatases": "dilatar",
    "dilataste": "dilatar",
    "dilatasteis": "dilatar",
    "dilatativa": "dilatativo",
    "dilatativas": "dilatativo",
    "dilatativos": "dilatativo",
    "dilate": "dilatar",
    "dilatemos": "dilatar",
    "dilaten": "dilatar",
    "dilates": "dilatar",
    "dilativa": "dilativo",
    "dilativas": "dilativo",
    "dilativos": "dilativo",
    "dilato": "dilatar",
    "dilatométrica": "dilatométrico",
    "dilatométricas": "dilatométrico",
    "dilatométricos": "dilatométrico",
    "dilatoria": "dilatorio",
    "dilatorias": "dilatorio",
    "dilatorios": "dilatorio",
    "dilatábamos": "dilatar",
    "dilatáis": "dilatar",
    "dilatáramos": "dilatar",
    "dilatáremos": "dilatar",
    "dilatásemos": "dilatar",
    "dilaté": "dilatar",
    "dilatéis": "dilatar",
    "dilató": "dilatar",
    "dilatómetros": "dilatómetro",
    "dilauratos": "dilaurato",
    "dilecciones": "dilección",
    "dilecta": "dilecto",
    "dilectas": "dilecto",
    "dilectos": "dilecto",
    "dilemas": "dilema",
    "dilemática": "dilemático",
    "dilemáticas": "dilemático",
    "dilemáticos": "dilemático",
    "dileniácea": "dileniáceo",
    "dileniáceas": "dileniáceo",
    "dileniáceos": "dileniáceo",
    "diletantes": "diletante",
    "diletantismos": "diletantismo",
    "diligencia": "diligenciar",
    "diligenciaba": "diligenciar",
    "diligenciabais": "diligenciar",
    "diligenciaban": "diligenciar",
    "diligenciabas": "diligenciar",
    "diligenciad": "diligenciar",
    "diligenciada": "diligenciar",
    "diligenciadas": "diligenciar",
    "diligenciado": "diligenciar",
    "diligenciados": "diligenciar",
    "diligenciamos": "diligenciar",
    "diligencian": "diligenciar",
    "diligenciando": "diligenciar",
    "diligenciara": "diligenciar",
    "diligenciarais": "diligenciar",
    "diligenciaran": "diligenciar",
    "diligenciaras": "diligenciar",
    "diligenciare": "diligenciar",
    "diligenciareis": "diligenciar",
    "diligenciaremos": "diligenciar",
    "diligenciaren": "diligenciar",
    "diligenciares": "diligenciar",
    "diligenciaron": "diligenciar",
    "diligenciará": "diligenciar",
    "diligenciarán": "diligenciar",
    "diligenciarás": "diligenciar",
    "diligenciaré": "diligenciar",
    "diligenciaréis": "diligenciar",
    "diligenciaría": "diligenciar",
    "diligenciaríais": "diligenciar",
    "diligenciaríamos": "diligenciar",
    "diligenciarían": "diligenciar",
    "diligenciarías": "diligenciar",
    "diligencias": "diligenciar",
    "diligenciase": "diligenciar",
    "diligenciaseis": "diligenciar",
    "diligenciasen": "diligenciar",
    "diligenciases": "diligenciar",
    "diligenciaste": "diligenciar",
    "diligenciasteis": "diligenciar",
    "diligencie": "diligenciar",
    "diligenciemos": "diligenciar",
    "diligencien": "diligenciar",
    "diligenciera": "diligenciero",
    "diligencieras": "diligenciero",
    "diligencieros": "diligenciero",
    "diligencies": "diligenciar",
    "diligencio": "diligenciar",
    "diligenciábamos": "diligenciar",
    "diligenciáis": "diligenciar",
    "diligenciáramos": "diligenciar",
    "diligenciáremos": "diligenciar",
    "diligenciásemos": "diligenciar",
    "diligencié": "diligenciar",
    "diligenciéis": "diligenciar",
    "diligenció": "diligenciar",
    "diligentes": "diligente",
    "dilinoleica": "dilinoleico",
    "dilinoleicas": "dilinoleico",
    "dilinoleicos": "dilinoleico",
    "dilogías": "dilogía",
    "dilucida": "dilucidar",
    "dilucidaba": "dilucidar",
    "dilucidabais": "dilucidar",
    "dilucidaban": "dilucidar",
    "dilucidabas": "dilucidar",
    "dilucidaciones": "dilucidación",
    "dilucidad": "dilucidar",
    "dilucidada": "dilucidar",
    "dilucidadas": "dilucidar",
    "dilucidado": "dilucidar",
    "dilucidadora": "dilucidador",
    "dilucidadoras": "dilucidador",
    "dilucidadores": "dilucidador",
    "dilucidados": "dilucidar",
    "dilucidamos": "dilucidar",
    "dilucidan": "dilucidar",
    "dilucidando": "dilucidar",
    "dilucidara": "dilucidar",
    "dilucidarais": "dilucidar",
    "dilucidaran": "dilucidar",
    "dilucidaras": "dilucidar",
    "dilucidare": "dilucidar",
    "dilucidareis": "dilucidar",
    "dilucidaremos": "dilucidar",
    "dilucidaren": "dilucidar",
    "dilucidares": "dilucidar",
    "dilucidaron": "dilucidar",
    "dilucidará": "dilucidar",
    "dilucidarán": "dilucidar",
    "dilucidarás": "dilucidar",
    "dilucidaré": "dilucidar",
    "dilucidaréis": "dilucidar",
    "dilucidaría": "dilucidar",
    "dilucidaríais": "dilucidar",
    "dilucidaríamos": "dilucidar",
    "dilucidarían": "dilucidar",
    "dilucidarías": "dilucidar",
    "dilucidas": "dilucidar",
    "dilucidase": "dilucidar",
    "dilucidaseis": "dilucidar",
    "dilucidasen": "dilucidar",
    "dilucidases": "dilucidar",
    "dilucidaste": "dilucidar",
    "dilucidasteis": "dilucidar",
    "dilucide": "dilucidar",
    "dilucidemos": "dilucidar",
    "diluciden": "dilucidar",
    "dilucides": "dilucidar",
    "dilucido": "dilucidar",
    "dilucidábamos": "dilucidar",
    "dilucidáis": "dilucidar",
    "dilucidáramos": "dilucidar",
    "dilucidáremos": "dilucidar",
    "dilucidásemos": "dilucidar",
    "dilucidé": "dilucidar",
    "dilucidéis": "dilucidar",
    "dilucidó": "dilucidar",
    "diluciones": "dilución",
    "diluentes": "diluente",
    "diluid": "diluir",
    "diluida": "diluir",
    "diluidas": "diluir",
    "diluido": "diluir",
    "diluidos": "diluir",
    "diluimos": "diluir",
    "diluiremos": "diluir",
    "diluirá": "diluir",
    "diluirán": "diluir",
    "diluirás": "diluir",
    "diluiré": "diluir",
    "diluiréis": "diluir",
    "diluiría": "diluir",
    "diluiríais": "diluir",
    "diluiríamos": "diluir",
    "diluirían": "diluir",
    "diluirías": "diluir",
    "diluiste": "diluir",
    "diluisteis": "diluir",
    "diluvia": "diluviar",
    "diluviaba": "diluviar",
    "diluviabais": "diluviar",
    "diluviaban": "diluviar",
    "diluviabas": "diluviar",
    "diluviad": "diluviar",
    "diluviada": "diluviar",
    "diluviadas": "diluviar",
    "diluviado": "diluviar",
    "diluviados": "diluviar",
    "diluviales": "diluvial",
    "diluviamos": "diluviar",
    "diluvian": "diluviar",
    "diluviando": "diluviar",
    "diluviara": "diluviar",
    "diluviarais": "diluviar",
    "diluviaran": "diluviar",
    "diluviaras": "diluviar",
    "diluviare": "diluviar",
    "diluviareis": "diluviar",
    "diluviaremos": "diluviar",
    "diluviaren": "diluviar",
    "diluviares": "diluviar",
    "diluviaron": "diluviar",
    "diluviará": "diluviar",
    "diluviarán": "diluviar",
    "diluviarás": "diluviar",
    "diluviaré": "diluviar",
    "diluviaréis": "diluviar",
    "diluviaría": "diluviar",
    "diluviaríais": "diluviar",
    "diluviaríamos": "diluviar",
    "diluviarían": "diluviar",
    "diluviarías": "diluviar",
    "diluvias": "diluviar",
    "diluviase": "diluviar",
    "diluviaseis": "diluviar",
    "diluviasen": "diluviar",
    "diluviases": "diluviar",
    "diluviaste": "diluviar",
    "diluviasteis": "diluviar",
    "diluvie": "diluviar",
    "diluviemos": "diluviar",
    "diluvien": "diluviar",
    "diluvies": "diluviar",
    "diluvio": "diluviar",
    "diluvios": "diluvio",
    "diluviábamos": "diluviar",
    "diluviáis": "diluviar",
    "diluviáramos": "diluviar",
    "diluviáremos": "diluviar",
    "diluviásemos": "diluviar",
    "diluvié": "diluviar",
    "diluviéis": "diluviar",
    "diluvió": "diluviar",
    "diluya": "diluir",
    "diluyamos": "diluir",
    "diluyan": "diluir",
    "diluyas": "diluir",
    "diluye": "diluir",
    "diluyen": "diluir",
    "diluyendo": "diluir",
    "diluyentes": "diluyente",
    "diluyera": "diluir",
    "diluyerais": "diluir",
    "diluyeran": "diluir",
    "diluyeras": "diluir",
    "diluyere": "diluir",
    "diluyereis": "diluir",
    "diluyeren": "diluir",
    "diluyeres": "diluir",
    "diluyeron": "diluir",
    "diluyes": "diluir",
    "diluyese": "diluir",
    "diluyeseis": "diluir",
    "diluyesen": "diluir",
    "diluyeses": "diluir",
    "diluyo": "diluir",
    "diluyáis": "diluir",
    "diluyéramos": "diluir",
    "diluyéremos": "diluir",
    "diluyésemos": "diluir",
    "diluyó": "diluir",
    "diluí": "diluir",
    "diluía": "diluir",
    "diluíais": "diluir",
    "diluíamos": "diluir",
    "diluían": "diluir",
    "diluías": "diluir",
    "diluís": "diluir",
    "dilúculos": "dilúculo",
    "dimaleatos": "dimaleato",
    "dimana": "dimanar",
    "dimanaba": "dimanar",
    "dimanabais": "dimanar",
    "dimanaban": "dimanar",
    "dimanabas": "dimanar",
    "dimanaciones": "dimanación",
    "dimanad": "dimanar",
    "dimanada": "dimanar",
    "dimanadas": "dimanar",
    "dimanado": "dimanar",
    "dimanados": "dimanar",
    "dimanamos": "dimanar",
    "dimanan": "dimanar",
    "dimanando": "dimanar",
    "dimanantes": "dimanante",
    "dimanara": "dimanar",
    "dimanarais": "dimanar",
    "dimanaran": "dimanar",
    "dimanaras": "dimanar",
    "dimanare": "dimanar",
    "dimanareis": "dimanar",
    "dimanaremos": "dimanar",
    "dimanaren": "dimanar",
    "dimanares": "dimanar",
    "dimanaron": "dimanar",
    "dimanará": "dimanar",
    "dimanarán": "dimanar",
    "dimanarás": "dimanar",
    "dimanaré": "dimanar",
    "dimanaréis": "dimanar",
    "dimanaría": "dimanar",
    "dimanaríais": "dimanar",
    "dimanaríamos": "dimanar",
    "dimanarían": "dimanar",
    "dimanarías": "dimanar",
    "dimanas": "dimanar",
    "dimanase": "dimanar",
    "dimanaseis": "dimanar",
    "dimanasen": "dimanar",
    "dimanases": "dimanar",
    "dimanaste": "dimanar",
    "dimanasteis": "dimanar",
    "dimane": "dimanar",
    "dimanemos": "dimanar",
    "dimanen": "dimanar",
    "dimanes": "dimanar",
    "dimano": "dimanar",
    "dimanábamos": "dimanar",
    "dimanáis": "dimanar",
    "dimanáramos": "dimanar",
    "dimanáremos": "dimanar",
    "dimanásemos": "dimanar",
    "dimané": "dimanar",
    "dimanéis": "dimanar",
    "dimanó": "dimanar",
    "dimenhidrinatos": "dimenhidrinato",
    "dimensiona": "dimensionar",
    "dimensionaba": "dimensionar",
    "dimensionabais": "dimensionar",
    "dimensionaban": "dimensionar",
    "dimensionabas": "dimensionar",
    "dimensionad": "dimensionar",
    "dimensionada": "dimensionar",
    "dimensionadas": "dimensionar",
    "dimensionado": "dimensionar",
    "dimensionados": "dimensionar",
    "dimensionales": "dimensional",
    "dimensionalidades": "dimensionalidad",
    "dimensionamientos": "dimensionamiento",
    "dimensionamos": "dimensionar",
    "dimensionan": "dimensionar",
    "dimensionando": "dimensionar",
    "dimensionara": "dimensionar",
    "dimensionarais": "dimensionar",
    "dimensionaran": "dimensionar",
    "dimensionaras": "dimensionar",
    "dimensionare": "dimensionar",
    "dimensionareis": "dimensionar",
    "dimensionaremos": "dimensionar",
    "dimensionaren": "dimensionar",
    "dimensionares": "dimensionar",
    "dimensionaron": "dimensionar",
    "dimensionará": "dimensionar",
    "dimensionarán": "dimensionar",
    "dimensionarás": "dimensionar",
    "dimensionaré": "dimensionar",
    "dimensionaréis": "dimensionar",
    "dimensionaría": "dimensionar",
    "dimensionaríais": "dimensionar",
    "dimensionaríamos": "dimensionar",
    "dimensionarían": "dimensionar",
    "dimensionarías": "dimensionar",
    "dimensionas": "dimensionar",
    "dimensionase": "dimensionar",
    "dimensionaseis": "dimensionar",
    "dimensionasen": "dimensionar",
    "dimensionases": "dimensionar",
    "dimensionaste": "dimensionar",
    "dimensionasteis": "dimensionar",
    "dimensione": "dimensionar",
    "dimensionemos": "dimensionar",
    "dimensionen": "dimensionar",
    "dimensiones": "dimensionar",
    "dimensiono": "dimensionar",
    "dimensionábamos": "dimensionar",
    "dimensionáis": "dimensionar",
    "dimensionáramos": "dimensionar",
    "dimensionáremos": "dimensionar",
    "dimensionásemos": "dimensionar",
    "dimensioné": "dimensionar",
    "dimensionéis": "dimensionar",
    "dimensionó": "dimensionar",
    "dimercaptoacetatos": "dimercaptoacetato",
    "dimerias": "dimeria",
    "dimerizaciones": "dimerización",
    "dimeticonas": "dimeticona",
    "dimetilacetales": "dimetilacetal",
    "dimetilacetamidas": "dimetilacetamida",
    "dimetilaminas": "dimetilamina",
    "dimetilaminoazobencenos": "dimetilaminoazobenceno",
    "dimetilaminopropilaminas": "dimetilaminopropilamina",
    "dimetilanilinas": "dimetilanilina",
    "dimetilbencilcarbinoles": "dimetilbencilcarbinol",
    "dimetilciclohexanos": "dimetilciclohexano",
    "dimetildiclorosilanos": "dimetildiclorosilano",
    "dimetildifenilureas": "dimetildifenilurea",
    "dimetilditiocarbamatos": "dimetilditiocarbamato",
    "dimetilglioximas": "dimetilglioxima",
    "dimetilhexanodioles": "dimetilhexanodiol",
    "dimetilhidantoínas": "dimetilhidantoína",
    "dimetilisopropanolaminas": "dimetilisopropanolamina",
    "dimetiloctanodioles": "dimetiloctanodiol",
    "dimetiloctinodioles": "dimetiloctinodiol",
    "dimetilpiperacinas": "dimetilpiperacina",
    "dimetoxitetraglicoles": "dimetoxitetraglicol",
    "diminuendos": "diminuendo",
    "diminuid": "diminuir",
    "diminuida": "diminuir",
    "diminuidas": "diminuir",
    "diminuido": "diminuir",
    "diminuidos": "diminuir",
    "diminuimos": "diminuir",
    "diminuiremos": "diminuir",
    "diminuirá": "diminuir",
    "diminuirán": "diminuir",
    "diminuirás": "diminuir",
    "diminuiré": "diminuir",
    "diminuiréis": "diminuir",
    "diminuiría": "diminuir",
    "diminuiríais": "diminuir",
    "diminuiríamos": "diminuir",
    "diminuirían": "diminuir",
    "diminuirías": "diminuir",
    "diminuiste": "diminuir",
    "diminuisteis": "diminuir",
    "diminuta": "diminuto",
    "diminutas": "diminuto",
    "diminutiva": "diminutivo",
    "diminutivas": "diminutivo",
    "diminutivos": "diminutivo",
    "diminutos": "diminuto",
    "diminuya": "diminuir",
    "diminuyamos": "diminuir",
    "diminuyan": "diminuir",
    "diminuyas": "diminuir",
    "diminuye": "diminuir",
    "diminuyen": "diminuir",
    "diminuyendo": "diminuir",
    "diminuyera": "diminuir",
    "diminuyerais": "diminuir",
    "diminuyeran": "diminuir",
    "diminuyeras": "diminuir",
    "diminuyere": "diminuir",
    "diminuyereis": "diminuir",
    "diminuyeren": "diminuir",
    "diminuyeres": "diminuir",
    "diminuyeron": "diminuir",
    "diminuyes": "diminuir",
    "diminuyese": "diminuir",
    "diminuyeseis": "diminuir",
    "diminuyesen": "diminuir",
    "diminuyeses": "diminuir",
    "diminuyo": "diminuir",
    "diminuyáis": "diminuir",
    "diminuyéramos": "diminuir",
    "diminuyéremos": "diminuir",
    "diminuyésemos": "diminuir",
    "diminuyó": "diminuir",
    "diminuí": "diminuir",
    "diminuía": "diminuir",
    "diminuíais": "diminuir",
    "diminuíamos": "diminuir",
    "diminuían": "diminuir",
    "diminuías": "diminuir",
    "diminuís": "diminuir",
    "dimisionaria": "dimisionario",
    "dimisionarias": "dimisionario",
    "dimisionarios": "dimisionario",
    "dimisiones": "dimisión",
    "dimita": "dimitir",
    "dimitamos": "dimitir",
    "dimitan": "dimitir",
    "dimitas": "dimitir",
    "dimite": "dimitir",
    "dimiten": "dimitir",
    "dimitentes": "dimitente",
    "dimites": "dimitir",
    "dimitid": "dimitir",
    "dimitida": "dimitir",
    "dimitidas": "dimitir",
    "dimitido": "dimitir",
    "dimitidos": "dimitir",
    "dimitiendo": "dimitir",
    "dimitiera": "dimitir",
    "dimitierais": "dimitir",
    "dimitieran": "dimitir",
    "dimitieras": "dimitir",
    "dimitiere": "dimitir",
    "dimitiereis": "dimitir",
    "dimitieren": "dimitir",
    "dimitieres": "dimitir",
    "dimitieron": "dimitir",
    "dimitiese": "dimitir",
    "dimitieseis": "dimitir",
    "dimitiesen": "dimitir",
    "dimitieses": "dimitir",
    "dimitimos": "dimitir",
    "dimitiremos": "dimitir",
    "dimitirá": "dimitir",
    "dimitirán": "dimitir",
    "dimitirás": "dimitir",
    "dimitiré": "dimitir",
    "dimitiréis": "dimitir",
    "dimitiría": "dimitir",
    "dimitiríais": "dimitir",
    "dimitiríamos": "dimitir",
    "dimitirían": "dimitir",
    "dimitirías": "dimitir",
    "dimitiste": "dimitir",
    "dimitisteis": "dimitir",
    "dimitiéramos": "dimitir",
    "dimitiéremos": "dimitir",
    "dimitiésemos": "dimitir",
    "dimitió": "dimitir",
    "dimito": "dimitir",
    "dimitáis": "dimitir",
    "dimití": "dimitir",
    "dimitía": "dimitir",
    "dimitíais": "dimitir",
    "dimitíamos": "dimitir",
    "dimitían": "dimitir",
    "dimitías": "dimitir",
    "dimitís": "dimitir",
    "dimorfa": "dimorfo",
    "dimorfas": "dimorfo",
    "dimorfismos": "dimorfismo",
    "dimorfos": "dimorfo",
    "dimos": "dar",
    "dimérica": "dimérico",
    "diméricas": "dimérico",
    "diméricos": "dimérico",
    "dimétrica": "dimétrico",
    "dimétricas": "dimétrico",
    "dimétricos": "dimétrico",
    "dimórfica": "dimórfico",
    "dimórficas": "dimórfico",
    "dimórficos": "dimórfico",
    "dinamarquesa": "dinamarqués",
    "dinamarquesas": "dinamarqués",
    "dinamarqueses": "dinamarqués",
    "dinamice": "dinamizar",
    "dinamicemos": "dinamizar",
    "dinamicen": "dinamizar",
    "dinamices": "dinamizar",
    "dinamicé": "dinamizar",
    "dinamicéis": "dinamizar",
    "dinamismos": "dinamismo",
    "dinamistas": "dinamista",
    "dinamita": "dinamitar",
    "dinamitaba": "dinamitar",
    "dinamitabais": "dinamitar",
    "dinamitaban": "dinamitar",
    "dinamitabas": "dinamitar",
    "dinamitad": "dinamitar",
    "dinamitada": "dinamitar",
    "dinamitadas": "dinamitar",
    "dinamitado": "dinamitar",
    "dinamitados": "dinamitar",
    "dinamitamos": "dinamitar",
    "dinamitan": "dinamitar",
    "dinamitando": "dinamitar",
    "dinamitara": "dinamitar",
    "dinamitarais": "dinamitar",
    "dinamitaran": "dinamitar",
    "dinamitaras": "dinamitar",
    "dinamitare": "dinamitar",
    "dinamitareis": "dinamitar",
    "dinamitaremos": "dinamitar",
    "dinamitaren": "dinamitar",
    "dinamitares": "dinamitar",
    "dinamitaron": "dinamitar",
    "dinamitará": "dinamitar",
    "dinamitarán": "dinamitar",
    "dinamitarás": "dinamitar",
    "dinamitaré": "dinamitar",
    "dinamitaréis": "dinamitar",
    "dinamitaría": "dinamitar",
    "dinamitaríais": "dinamitar",
    "dinamitaríamos": "dinamitar",
    "dinamitarían": "dinamitar",
    "dinamitarías": "dinamitar",
    "dinamitas": "dinamitar",
    "dinamitase": "dinamitar",
    "dinamitaseis": "dinamitar",
    "dinamitasen": "dinamitar",
    "dinamitases": "dinamitar",
    "dinamitaste": "dinamitar",
    "dinamitasteis": "dinamitar",
    "dinamite": "dinamitar",
    "dinamitemos": "dinamitar",
    "dinamiten": "dinamitar",
    "dinamitera": "dinamitero",
    "dinamiteras": "dinamitero",
    "dinamiteros": "dinamitero",
    "dinamites": "dinamitar",
    "dinamito": "dinamitar",
    "dinamitábamos": "dinamitar",
    "dinamitáis": "dinamitar",
    "dinamitáramos": "dinamitar",
    "dinamitáremos": "dinamitar",
    "dinamitásemos": "dinamitar",
    "dinamité": "dinamitar",
    "dinamitéis": "dinamitar",
    "dinamitó": "dinamitar",
    "dinamiza": "dinamizar",
    "dinamizaba": "dinamizar",
    "dinamizabais": "dinamizar",
    "dinamizaban": "dinamizar",
    "dinamizabas": "dinamizar",
    "dinamizaciones": "dinamización",
    "dinamizad": "dinamizar",
    "dinamizada": "dinamizar",
    "dinamizadas": "dinamizar",
    "dinamizado": "dinamizar",
    "dinamizadores": "dinamizador",
    "dinamizados": "dinamizar",
    "dinamizamos": "dinamizar",
    "dinamizan": "dinamizar",
    "dinamizando": "dinamizar",
    "dinamizara": "dinamizar",
    "dinamizarais": "dinamizar",
    "dinamizaran": "dinamizar",
    "dinamizaras": "dinamizar",
    "dinamizare": "dinamizar",
    "dinamizareis": "dinamizar",
    "dinamizaremos": "dinamizar",
    "dinamizaren": "dinamizar",
    "dinamizares": "dinamizar",
    "dinamizaron": "dinamizar",
    "dinamizará": "dinamizar",
    "dinamizarán": "dinamizar",
    "dinamizarás": "dinamizar",
    "dinamizaré": "dinamizar",
    "dinamizaréis": "dinamizar",
    "dinamizaría": "dinamizar",
    "dinamizaríais": "dinamizar",
    "dinamizaríamos": "dinamizar",
    "dinamizarían": "dinamizar",
    "dinamizarías": "dinamizar",
    "dinamizas": "dinamizar",
    "dinamizase": "dinamizar",
    "dinamizaseis": "dinamizar",
    "dinamizasen": "dinamizar",
    "dinamizases": "dinamizar",
    "dinamizaste": "dinamizar",
    "dinamizasteis": "dinamizar",
    "dinamizo": "dinamizar",
    "dinamizábamos": "dinamizar",
    "dinamizáis": "dinamizar",
    "dinamizáramos": "dinamizar",
    "dinamizáremos": "dinamizar",
    "dinamizásemos": "dinamizar",
    "dinamizó": "dinamizar",
    "dinamoeléctrica": "dinamoeléctrico",
    "dinamoeléctricas": "dinamoeléctrico",
    "dinamoeléctricos": "dinamoeléctrico",
    "dinamometamorfismos": "dinamometamorfismo",
    "dinamométrica": "dinamométrico",
    "dinamométricas": "dinamométrico",
    "dinamométricos": "dinamométrico",
    "dinamos": "dinamo",
    "dinamómetros": "dinamómetro",
    "dinares": "dinar",
    "dinas": "dina",
    "dinastas": "dinasta",
    "dinastías": "dinastía",
    "dineradas": "dinerada",
    "dineraladas": "dineralada",
    "dinerales": "dineral",
    "dineraria": "dinerario",
    "dinerarias": "dinerario",
    "dinerarios": "dinerario",
    "dinerillos": "dinerillo",
    "dineros": "dinero",
    "dinerosa": "dineroso",
    "dinerosas": "dineroso",
    "dinerosos": "dineroso",
    "dingolondangos": "dingolondango",
    "dingos": "dingo",
    "dinitrilos": "dinitrilo",
    "dinitrobencenos": "dinitrobenceno",
    "dinitrofeniles": "dinitrofenil",
    "dinitrofenoles": "dinitrofenol",
    "dinitronaftalenos": "dinitronaftaleno",
    "dinitrotoluenos": "dinitrotolueno",
    "dinoflageladas": "dinoflagelada",
    "dinofíceas": "dinofícea",
    "dinosaurios": "dinosaurio",
    "dinoterios": "dinoterio",
    "dinteles": "dintel",
    "dintornos": "dintorno",
    "dinámetros": "dinámetro",
    "dinámica": "dinámico",
    "dinámicas": "dinámico",
    "dinámicos": "dinámico",
    "dinárica": "dinárico",
    "dináricas": "dinárico",
    "dináricos": "dinárico",
    "dinástica": "dinástico",
    "dinásticas": "dinástico",
    "dinásticos": "dinástico",
    "dinérica": "dinérico",
    "dinéricas": "dinérico",
    "dinéricos": "dinérico",
    "dio": "dar",
    "diocesana": "diocesano",
    "diocesanas": "diocesano",
    "diocesanos": "diocesano",
    "dioctaédrica": "dioctaédrico",
    "dioctaédricas": "dioctaédrico",
    "dioctaédricos": "dioctaédrico",
    "diodos": "diodo",
    "diofántica": "diofántico",
    "diofánticas": "diofántico",
    "diofánticos": "diofántico",
    "dioica": "dioico",
    "dioicas": "dioico",
    "dioicos": "dioico",
    "dionisiaca": "dionisíaco",
    "dionisiacas": "dionisíaco",
    "dionisiacos": "dionisíaco",
    "dionisias": "dionisia",
    "dionisíaca": "dionisíaco",
    "dionisíacas": "dionisíaco",
    "dionisíacos": "dionisíaco",
    "dioptasas": "dioptasa",
    "dioptras": "dioptra",
    "dioptrios": "dioptrio",
    "dioptrías": "dioptría",
    "dioptrómetros": "dioptrómetro",
    "dioramas": "diorama",
    "dioritas": "diorita",
    "diorítica": "diorítico",
    "dioríticas": "diorítico",
    "dioríticos": "diorítico",
    "diosa": "dios",
    "diosas": "dios",
    "dioses": "dios",
    "dioxanos": "dioxano",
    "dioxinas": "dioxina",
    "dioxolanos": "dioxolano",
    "dipentaeritritoles": "dipentaeritritol",
    "dipentenos": "dipenteno",
    "diplexores": "diplexor",
    "diplobacilos": "diplobacilo",
    "diplocromosomas": "diplocromosoma",
    "diploedros": "diploedro",
    "diplofases": "diplofase",
    "diploica": "diploico",
    "diploicas": "diploico",
    "diploicos": "diploico",
    "diploides": "diploide",
    "diploidias": "diploidia",
    "diploidizaciones": "diploidización",
    "diplomacias": "diplomacia",
    "diplomada": "diplomado",
    "diplomadas": "diplomado",
    "diplomados": "diplomado",
    "diplomas": "diploma",
    "diplomaturas": "diplomatura",
    "diplomática": "diplomático",
    "diplomáticas": "diplomático",
    "diplomáticos": "diplomático",
    "diplontes": "diplonte",
    "diplopiómetros": "diplopiómetro",
    "diplopías": "diplopía",
    "diploscopios": "diploscopio",
    "diplosomas": "diplosoma",
    "diplotenas": "diplotena",
    "diplópodos": "diplópodo",
    "dipnea": "dipneo",
    "dipneas": "dipneo",
    "dipneos": "dipneo",
    "dipnonas": "dipnona",
    "dipodias": "dipodia",
    "dipolares": "dipolar",
    "dipolos": "dipolo",
    "dipropionatos": "dipropionato",
    "dipsacáceas": "dipsacácea",
    "dipsomaniaca": "dipsomaníaco",
    "dipsomaniacas": "dipsomaníaco",
    "dipsomaniacos": "dipsomaníaco",
    "dipsomaníaca": "dipsomaníaco",
    "dipsomaníacas": "dipsomaníaco",
    "dipsomaníacos": "dipsomaníaco",
    "dipsomanías": "dipsomanía",
    "dipsáceas": "dipsácea",
    "dipsómana": "dipsómano",
    "dipsómanas": "dipsómano",
    "dipsómanos": "dipsómano",
    "dipterocarpáceas": "dipterocarpácea",
    "diptonga": "diptongar",
    "diptongaba": "diptongar",
    "diptongabais": "diptongar",
    "diptongaban": "diptongar",
    "diptongabas": "diptongar",
    "diptongaciones": "diptongación",
    "diptongad": "diptongar",
    "diptongada": "diptongar",
    "diptongadas": "diptongar",
    "diptongado": "diptongar",
    "diptongados": "diptongar",
    "diptongamos": "diptongar",
    "diptongan": "diptongar",
    "diptongando": "diptongar",
    "diptongara": "diptongar",
    "diptongarais": "diptongar",
    "diptongaran": "diptongar",
    "diptongaras": "diptongar",
    "diptongare": "diptongar",
    "diptongareis": "diptongar",
    "diptongaremos": "diptongar",
    "diptongaren": "diptongar",
    "diptongares": "diptongar",
    "diptongaron": "diptongar",
    "diptongará": "diptongar",
    "diptongarán": "diptongar",
    "diptongarás": "diptongar",
    "diptongaré": "diptongar",
    "diptongaréis": "diptongar",
    "diptongaría": "diptongar",
    "diptongaríais": "diptongar",
    "diptongaríamos": "diptongar",
    "diptongarían": "diptongar",
    "diptongarías": "diptongar",
    "diptongas": "diptongar",
    "diptongase": "diptongar",
    "diptongaseis": "diptongar",
    "diptongasen": "diptongar",
    "diptongases": "diptongar",
    "diptongaste": "diptongar",
    "diptongasteis": "diptongar",
    "diptongo": "diptongar",
    "diptongos": "diptongo",
    "diptongue": "diptongar",
    "diptonguemos": "diptongar",
    "diptonguen": "diptongar",
    "diptongues": "diptongar",
    "diptongué": "diptongar",
    "diptonguéis": "diptongar",
    "diptongábamos": "diptongar",
    "diptongáis": "diptongar",
    "diptongáramos": "diptongar",
    "diptongáremos": "diptongar",
    "diptongásemos": "diptongar",
    "diptongó": "diptongar",
    "diputa": "diputar",
    "diputaba": "diputar",
    "diputabais": "diputar",
    "diputaban": "diputar",
    "diputabas": "diputar",
    "diputaciones": "diputación",
    "diputad": "diputar",
    "diputada": "diputar",
    "diputadas": "diputar",
    "diputado": "diputar",
    "diputados": "diputar",
    "diputamos": "diputar",
    "diputan": "diputar",
    "diputando": "diputar",
    "diputara": "diputar",
    "diputarais": "diputar",
    "diputaran": "diputar",
    "diputaras": "diputar",
    "diputare": "diputar",
    "diputareis": "diputar",
    "diputaremos": "diputar",
    "diputaren": "diputar",
    "diputares": "diputar",
    "diputaron": "diputar",
    "diputará": "diputar",
    "diputarán": "diputar",
    "diputarás": "diputar",
    "diputaré": "diputar",
    "diputaréis": "diputar",
    "diputaría": "diputar",
    "diputaríais": "diputar",
    "diputaríamos": "diputar",
    "diputarían": "diputar",
    "diputarías": "diputar",
    "diputas": "diputar",
    "diputase": "diputar",
    "diputaseis": "diputar",
    "diputasen": "diputar",
    "diputases": "diputar",
    "diputaste": "diputar",
    "diputasteis": "diputar",
    "dipute": "diputar",
    "diputemos": "diputar",
    "diputen": "diputar",
    "diputes": "diputar",
    "diputo": "diputar",
    "diputábamos": "diputar",
    "diputáis": "diputar",
    "diputáramos": "diputar",
    "diputáremos": "diputar",
    "diputásemos": "diputar",
    "diputé": "diputar",
    "diputéis": "diputar",
    "diputó": "diputar",
    "dipétalas": "dipétala",
    "diquela": "diquelar",
    "diquelaba": "diquelar",
    "diquelabais": "diquelar",
    "diquelaban": "diquelar",
    "diquelabas": "diquelar",
    "diquelad": "diquelar",
    "diquelada": "diquelar",
    "diqueladas": "diquelar",
    "diquelado": "diquelar",
    "diquelados": "diquelar",
    "diquelamos": "diquelar",
    "diquelan": "diquelar",
    "diquelando": "diquelar",
    "diquelara": "diquelar",
    "diquelarais": "diquelar",
    "diquelaran": "diquelar",
    "diquelaras": "diquelar",
    "diquelare": "diquelar",
    "diquelareis": "diquelar",
    "diquelaremos": "diquelar",
    "diquelaren": "diquelar",
    "diquelares": "diquelar",
    "diquelaron": "diquelar",
    "diquelará": "diquelar",
    "diquelarán": "diquelar",
    "diquelarás": "diquelar",
    "diquelaré": "diquelar",
    "diquelaréis": "diquelar",
    "diquelaría": "diquelar",
    "diquelaríais": "diquelar",
    "diquelaríamos": "diquelar",
    "diquelarían": "diquelar",
    "diquelarías": "diquelar",
    "diquelas": "diquelar",
    "diquelase": "diquelar",
    "diquelaseis": "diquelar",
    "diquelasen": "diquelar",
    "diquelases": "diquelar",
    "diquelaste": "diquelar",
    "diquelasteis": "diquelar",
    "diquele": "diquelar",
    "diquelemos": "diquelar",
    "diquelen": "diquelar",
    "diqueles": "diquelar",
    "diquelo": "diquelar",
    "diquelábamos": "diquelar",
    "diqueláis": "diquelar",
    "diqueláramos": "diquelar",
    "diqueláremos": "diquelar",
    "diquelásemos": "diquelar",
    "diquelé": "diquelar",
    "diqueléis": "diquelar",
    "diqueló": "diquelar",
    "diques": "dique",
    "dircea": "dirceo",
    "dirceas": "dirceo",
    "dirceos": "dirceo",
    "direcciona": "direccionar",
    "direccionaba": "direccionar",
    "direccionabais": "direccionar",
    "direccionaban": "direccionar",
    "direccionabas": "direccionar",
    "direccionables": "direccionable",
    "direccionad": "direccionar",
    "direccionada": "direccionar",
    "direccionadas": "direccionar",
    "direccionado": "direccionar",
    "direccionados": "direccionar",
    "direccionales": "direccional",
    "direccionalidades": "direccionalidad",
    "direccionamientos": "direccionamiento",
    "direccionamos": "direccionar",
    "direccionan": "direccionar",
    "direccionando": "direccionar",
    "direccionara": "direccionar",
    "direccionarais": "direccionar",
    "direccionaran": "direccionar",
    "direccionaras": "direccionar",
    "direccionare": "direccionar",
    "direccionareis": "direccionar",
    "direccionaremos": "direccionar",
    "direccionaren": "direccionar",
    "direccionares": "direccionar",
    "direccionaron": "direccionar",
    "direccionará": "direccionar",
    "direccionarán": "direccionar",
    "direccionarás": "direccionar",
    "direccionaré": "direccionar",
    "direccionaréis": "direccionar",
    "direccionaría": "direccionar",
    "direccionaríais": "direccionar",
    "direccionaríamos": "direccionar",
    "direccionarían": "direccionar",
    "direccionarías": "direccionar",
    "direccionas": "direccionar",
    "direccionase": "direccionar",
    "direccionaseis": "direccionar",
    "direccionasen": "direccionar",
    "direccionases": "direccionar",
    "direccionaste": "direccionar",
    "direccionasteis": "direccionar",
    "direccione": "direccionar",
    "direccionemos": "direccionar",
    "direccionen": "direccionar",
    "direcciones": "direccionar",
    "direcciono": "direccionar",
    "direccionábamos": "direccionar",
    "direccionáis": "direccionar",
    "direccionáramos": "direccionar",
    "direccionáremos": "direccionar",
    "direccionásemos": "direccionar",
    "direccioné": "direccionar",
    "direccionéis": "direccionar",
    "direccionó": "direccionar",
    "directa": "directo",
    "directas": "directo",
    "directiva": "directivo",
    "directivas": "directivo",
    "directividades": "directividad",
    "directivos": "directivo",
    "directora": "director",
    "directorales": "directoral",
    "directoras": "director",
    "directores": "director",
    "directoria": "directorio",
    "directoriales": "directorial",
    "directorias": "directorio",
    "directorios": "directorio",
    "directos": "directo",
    "directrices": "directriz",
    "diremos": "decir",
    "diresorcinas": "diresorcina",
    "dirhames": "dirham",
    "dirhams": "dirham",
    "dirhemes": "dirhem",
    "dirhems": "dirhem",
    "dirige": "dirigir",
    "dirigen": "dirigir",
    "dirigencias": "dirigencia",
    "dirigentes": "dirigente",
    "diriges": "dirigir",
    "dirigibilidades": "dirigibilidad",
    "dirigibles": "dirigible",
    "dirigid": "dirigir",
    "dirigida": "dirigir",
    "dirigidas": "dirigir",
    "dirigido": "dirigir",
    "dirigidos": "dirigir",
    "dirigiendo": "dirigir",
    "dirigiera": "dirigir",
    "dirigierais": "dirigir",
    "dirigieran": "dirigir",
    "dirigieras": "dirigir",
    "dirigiere": "dirigir",
    "dirigiereis": "dirigir",
    "dirigieren": "dirigir",
    "dirigieres": "dirigir",
    "dirigieron": "dirigir",
    "dirigiese": "dirigir",
    "dirigieseis": "dirigir",
    "dirigiesen": "dirigir",
    "dirigieses": "dirigir",
    "dirigimos": "dirigir",
    "dirigiremos": "dirigir",
    "dirigirá": "dirigir",
    "dirigirán": "dirigir",
    "dirigirás": "dirigir",
    "dirigiré": "dirigir",
    "dirigiréis": "dirigir",
    "dirigiría": "dirigir",
    "dirigiríais": "dirigir",
    "dirigiríamos": "dirigir",
    "dirigirían": "dirigir",
    "dirigirías": "dirigir",
    "dirigismos": "dirigismo",
    "dirigistas": "dirigista",
    "dirigiste": "dirigir",
    "dirigisteis": "dirigir",
    "dirigiéramos": "dirigir",
    "dirigiéremos": "dirigir",
    "dirigiésemos": "dirigir",
    "dirigió": "dirigir",
    "dirigí": "dirigir",
    "dirigía": "dirigir",
    "dirigíais": "dirigir",
    "dirigíamos": "dirigir",
    "dirigían": "dirigir",
    "dirigías": "dirigir",
    "dirigís": "dirigir",
    "dirija": "dirigir",
    "dirijamos": "dirigir",
    "dirijan": "dirigir",
    "dirijas": "dirigir",
    "dirijo": "dirigir",
    "dirijáis": "dirigir",
    "dirima": "dirimir",
    "dirimamos": "dirimir",
    "diriman": "dirimir",
    "dirimas": "dirimir",
    "dirime": "dirimir",
    "dirimen": "dirimir",
    "dirimentes": "dirimente",
    "dirimes": "dirimir",
    "dirimibles": "dirimible",
    "dirimid": "dirimir",
    "dirimida": "dirimir",
    "dirimidas": "dirimir",
    "dirimido": "dirimir",
    "dirimidos": "dirimir",
    "dirimiendo": "dirimir",
    "dirimiera": "dirimir",
    "dirimierais": "dirimir",
    "dirimieran": "dirimir",
    "dirimieras": "dirimir",
    "dirimiere": "dirimir",
    "dirimiereis": "dirimir",
    "dirimieren": "dirimir",
    "dirimieres": "dirimir",
    "dirimieron": "dirimir",
    "dirimiese": "dirimir",
    "dirimieseis": "dirimir",
    "dirimiesen": "dirimir",
    "dirimieses": "dirimir",
    "dirimimos": "dirimir",
    "dirimiremos": "dirimir",
    "dirimirá": "dirimir",
    "dirimirán": "dirimir",
    "dirimirás": "dirimir",
    "dirimiré": "dirimir",
    "dirimiréis": "dirimir",
    "dirimiría": "dirimir",
    "dirimiríais": "dirimir",
    "dirimiríamos": "dirimir",
    "dirimirían": "dirimir",
    "dirimirías": "dirimir",
    "dirimiste": "dirimir",
    "dirimisteis": "dirimir",
    "dirimiéramos": "dirimir",
    "dirimiéremos": "dirimir",
    "dirimiésemos": "dirimir",
    "dirimió": "dirimir",
    "dirimo": "dirimir",
    "dirimáis": "dirimir",
    "dirimí": "dirimir",
    "dirimía": "dirimir",
    "dirimíais": "dirimir",
    "dirimíamos": "dirimir",
    "dirimían": "dirimir",
    "dirimías": "dirimir",
    "dirimís": "dirimir",
    "dirá": "decir",
    "dirán": "decir",
    "dirás": "decir",
    "diré": "decir",
    "diréis": "decir",
    "diría": "decir",
    "diríais": "decir",
    "diríamos": "decir",
    "dirían": "decir",
    "dirías": "decir",
    "disacáridos": "disacárido",
    "disaraxias": "disaraxia",
    "disartrias": "disartria",
    "discales": "discal",
    "discapacidades": "discapacidad",
    "discapacitada": "discapacitado",
    "discapacitadas": "discapacitado",
    "discapacitados": "discapacitado",
    "disceptaciones": "disceptación",
    "discernamos": "discernir",
    "discernibles": "discernible",
    "discernid": "discernir",
    "discernida": "discernir",
    "discernidas": "discernir",
    "discernido": "discernir",
    "discernidora": "discernidor",
    "discernidoras": "discernidor",
    "discernidores": "discernidor",
    "discernidos": "discernir",
    "discerniendo": "discernir",
    "discerniera": "discernir",
    "discernierais": "discernir",
    "discernieran": "discernir",
    "discernieras": "discernir",
    "discerniere": "discernir",
    "discerniereis": "discernir",
    "discernieren": "discernir",
    "discernieres": "discernir",
    "discernieron": "discernir",
    "discerniese": "discernir",
    "discernieseis": "discernir",
    "discerniesen": "discernir",
    "discernieses": "discernir",
    "discernimientos": "discernimiento",
    "discernimos": "discernir",
    "discerniremos": "discernir",
    "discernirá": "discernir",
    "discernirán": "discernir",
    "discernirás": "discernir",
    "discerniré": "discernir",
    "discerniréis": "discernir",
    "discerniría": "discernir",
    "discerniríais": "discernir",
    "discerniríamos": "discernir",
    "discernirían": "discernir",
    "discernirías": "discernir",
    "discerniste": "discernir",
    "discernisteis": "discernir",
    "discerniéramos": "discernir",
    "discerniéremos": "discernir",
    "discerniésemos": "discernir",
    "discernió": "discernir",
    "discernáis": "discernir",
    "discerní": "discernir",
    "discernía": "discernir",
    "discerníais": "discernir",
    "discerníamos": "discernir",
    "discernían": "discernir",
    "discernías": "discernir",
    "discernís": "discernir",
    "discierna": "discernir",
    "disciernan": "discernir",
    "disciernas": "discernir",
    "discierne": "discernir",
    "disciernen": "discernir",
    "disciernes": "discernir",
    "discierno": "discernir",
    "disciformes": "disciforme",
    "discinesias": "discinesia",
    "disciplina": "disciplinar",
    "disciplinaba": "disciplinar",
    "disciplinabais": "disciplinar",
    "disciplinaban": "disciplinar",
    "disciplinabas": "disciplinar",
    "disciplinad": "disciplinar",
    "disciplinada": "disciplinar",
    "disciplinadas": "disciplinar",
    "disciplinado": "disciplinar",
    "disciplinados": "disciplinar",
    "disciplinales": "disciplinal",
    "disciplinamos": "disciplinar",
    "disciplinan": "disciplinar",
    "disciplinando": "disciplinar",
    "disciplinantes": "disciplinante",
    "disciplinara": "disciplinar",
    "disciplinarais": "disciplinar",
    "disciplinaran": "disciplinar",
    "disciplinaras": "disciplinar",
    "disciplinare": "disciplinar",
    "disciplinareis": "disciplinar",
    "disciplinaremos": "disciplinar",
    "disciplinaren": "disciplinar",
    "disciplinares": "disciplinar",
    "disciplinaria": "disciplinario",
    "disciplinarias": "disciplinario",
    "disciplinarios": "disciplinario",
    "disciplinaron": "disciplinar",
    "disciplinará": "disciplinar",
    "disciplinarán": "disciplinar",
    "disciplinarás": "disciplinar",
    "disciplinaré": "disciplinar",
    "disciplinaréis": "disciplinar",
    "disciplinaría": "disciplinar",
    "disciplinaríais": "disciplinar",
    "disciplinaríamos": "disciplinar",
    "disciplinarían": "disciplinar",
    "disciplinarías": "disciplinar",
    "disciplinas": "disciplinar",
    "disciplinase": "disciplinar",
    "disciplinaseis": "disciplinar",
    "disciplinasen": "disciplinar",
    "disciplinases": "disciplinar",
    "disciplinaste": "disciplinar",
    "disciplinasteis": "disciplinar",
    "discipline": "disciplinar",
    "disciplinemos": "disciplinar",
    "disciplinen": "disciplinar",
    "disciplines": "disciplinar",
    "disciplino": "disciplinar",
    "disciplinábamos": "disciplinar",
    "disciplináis": "disciplinar",
    "disciplináramos": "disciplinar",
    "disciplináremos": "disciplinar",
    "disciplinásemos": "disciplinar",
    "discipliné": "disciplinar",
    "disciplinéis": "disciplinar",
    "disciplinó": "disciplinar",
    "discipulados": "discipulado",
    "discografías": "discografía",
    "discográfica": "discográfico",
    "discográficas": "discográfico",
    "discográficos": "discográfico",
    "discoidales": "discoidal",
    "discoides": "discoide",
    "discolora": "discoloro",
    "discoloras": "discoloro",
    "discoloros": "discoloro",
    "disconformes": "disconforme",
    "disconformidades": "disconformidad",
    "discontinua": "discontinuo",
    "discontinuaba": "discontinuar",
    "discontinuabais": "discontinuar",
    "discontinuaban": "discontinuar",
    "discontinuabas": "discontinuar",
    "discontinuaciones": "discontinuación",
    "discontinuad": "discontinuar",
    "discontinuada": "discontinuar",
    "discontinuadas": "discontinuar",
    "discontinuado": "discontinuar",
    "discontinuados": "discontinuar",
    "discontinuamos": "discontinuar",
    "discontinuando": "discontinuar",
    "discontinuara": "discontinuar",
    "discontinuarais": "discontinuar",
    "discontinuaran": "discontinuar",
    "discontinuaras": "discontinuar",
    "discontinuare": "discontinuar",
    "discontinuareis": "discontinuar",
    "discontinuaremos": "discontinuar",
    "discontinuaren": "discontinuar",
    "discontinuares": "discontinuar",
    "discontinuaron": "discontinuar",
    "discontinuará": "discontinuar",
    "discontinuarán": "discontinuar",
    "discontinuarás": "discontinuar",
    "discontinuaré": "discontinuar",
    "discontinuaréis": "discontinuar",
    "discontinuaría": "discontinuar",
    "discontinuaríais": "discontinuar",
    "discontinuaríamos": "discontinuar",
    "discontinuarían": "discontinuar",
    "discontinuarías": "discontinuar",
    "discontinuas": "discontinuo",
    "discontinuase": "discontinuar",
    "discontinuaseis": "discontinuar",
    "discontinuasen": "discontinuar",
    "discontinuases": "discontinuar",
    "discontinuaste": "discontinuar",
    "discontinuasteis": "discontinuar",
    "discontinuemos": "discontinuar",
    "discontinuidades": "discontinuidad",
    "discontinuos": "discontinuo",
    "discontinuábamos": "discontinuar",
    "discontinuáis": "discontinuar",
    "discontinuáramos": "discontinuar",
    "discontinuáremos": "discontinuar",
    "discontinuásemos": "discontinuar",
    "discontinué": "discontinuar",
    "discontinuéis": "discontinuar",
    "discontinuó": "discontinuar",
    "discontinúa": "discontinuar",
    "discontinúan": "discontinuar",
    "discontinúas": "discontinuar",
    "discontinúe": "discontinuar",
    "discontinúen": "discontinuar",
    "discontinúes": "discontinuar",
    "discontinúo": "discontinuar",
    "disconven": "disconvenir",
    "disconvendremos": "disconvenir",
    "disconvendrá": "disconvenir",
    "disconvendrán": "disconvenir",
    "disconvendrás": "disconvenir",
    "disconvendré": "disconvenir",
    "disconvendréis": "disconvenir",
    "disconvendría": "disconvenir",
    "disconvendríais": "disconvenir",
    "disconvendríamos": "disconvenir",
    "disconvendrían": "disconvenir",
    "disconvendrías": "disconvenir",
    "disconvenga": "disconvenir",
    "disconvengamos": "disconvenir",
    "disconvengan": "disconvenir",
    "disconvengas": "disconvenir",
    "disconvengo": "disconvenir",
    "disconvengáis": "disconvenir",
    "disconvenid": "disconvenir",
    "disconvenida": "disconvenir",
    "disconvenidas": "disconvenir",
    "disconvenido": "disconvenir",
    "disconvenidos": "disconvenir",
    "disconveniencias": "disconveniencia",
    "disconvenimos": "disconvenir",
    "disconvenía": "disconvenir",
    "disconveníais": "disconvenir",
    "disconveníamos": "disconvenir",
    "disconvenían": "disconvenir",
    "disconvenías": "disconvenir",
    "disconvenís": "disconvenir",
    "disconviene": "disconvenir",
    "disconvienen": "disconvenir",
    "disconvienes": "disconvenir",
    "disconvine": "disconvenir",
    "disconviniendo": "disconvenir",
    "disconviniera": "disconvenir",
    "disconvinierais": "disconvenir",
    "disconvinieran": "disconvenir",
    "disconvinieras": "disconvenir",
    "disconviniere": "disconvenir",
    "disconviniereis": "disconvenir",
    "disconvinieren": "disconvenir",
    "disconvinieres": "disconvenir",
    "disconvinieron": "disconvenir",
    "disconviniese": "disconvenir",
    "disconvinieseis": "disconvenir",
    "disconviniesen": "disconvenir",
    "disconvinieses": "disconvenir",
    "disconvinimos": "disconvenir",
    "disconviniste": "disconvenir",
    "disconvinisteis": "disconvenir",
    "disconviniéramos": "disconvenir",
    "disconviniéremos": "disconvenir",
    "disconviniésemos": "disconvenir",
    "disconvino": "disconvenir",
    "discordaba": "discordar",
    "discordabais": "discordar",
    "discordaban": "discordar",
    "discordabas": "discordar",
    "discordad": "discordar",
    "discordada": "discordar",
    "discordadas": "discordar",
    "discordado": "discordar",
    "discordados": "discordar",
    "discordamos": "discordar",
    "discordancias": "discordancia",
    "discordando": "discordar",
    "discordantes": "discordante",
    "discordara": "discordar",
    "discordarais": "discordar",
    "discordaran": "discordar",
    "discordaras": "discordar",
    "discordare": "discordar",
    "discordareis": "discordar",
    "discordaremos": "discordar",
    "discordaren": "discordar",
    "discordares": "discordar",
    "discordaron": "discordar",
    "discordará": "discordar",
    "discordarán": "discordar",
    "discordarás": "discordar",
    "discordaré": "discordar",
    "discordaréis": "discordar",
    "discordaría": "discordar",
    "discordaríais": "discordar",
    "discordaríamos": "discordar",
    "discordarían": "discordar",
    "discordarías": "discordar",
    "discordase": "discordar",
    "discordaseis": "discordar",
    "discordasen": "discordar",
    "discordases": "discordar",
    "discordaste": "discordar",
    "discordasteis": "discordar",
    "discordemos": "discordar",
    "discordes": "discorde",
    "discordias": "discordia",
    "discordábamos": "discordar",
    "discordáis": "discordar",
    "discordáramos": "discordar",
    "discordáremos": "discordar",
    "discordásemos": "discordar",
    "discordé": "discordar",
    "discordéis": "discordar",
    "discordó": "discordar",
    "discos": "disco",
    "discotecas": "discoteca",
    "discotequera": "discotequero",
    "discotequeras": "discotequero",
    "discotequeros": "discotequero",
    "discrasias": "discrasia",
    "discrasitas": "discrasita",
    "discrecionales": "discrecional",
    "discrecionalidades": "discrecionalidad",
    "discreciones": "discreción",
    "discrepa": "discrepar",
    "discrepaba": "discrepar",
    "discrepabais": "discrepar",
    "discrepaban": "discrepar",
    "discrepabas": "discrepar",
    "discrepad": "discrepar",
    "discrepada": "discrepar",
    "discrepadas": "discrepar",
    "discrepado": "discrepar",
    "discrepados": "discrepar",
    "discrepamos": "discrepar",
    "discrepan": "discrepar",
    "discrepancias": "discrepancia",
    "discrepando": "discrepar",
    "discrepantes": "discrepante",
    "discrepara": "discrepar",
    "discreparais": "discrepar",
    "discreparan": "discrepar",
    "discreparas": "discrepar",
    "discrepare": "discrepar",
    "discrepareis": "discrepar",
    "discreparemos": "discrepar",
    "discreparen": "discrepar",
    "discrepares": "discrepar",
    "discreparon": "discrepar",
    "discrepará": "discrepar",
    "discreparán": "discrepar",
    "discreparás": "discrepar",
    "discreparé": "discrepar",
    "discreparéis": "discrepar",
    "discreparía": "discrepar",
    "discreparíais": "discrepar",
    "discreparíamos": "discrepar",
    "discreparían": "discrepar",
    "discreparías": "discrepar",
    "discrepas": "discrepar",
    "discrepase": "discrepar",
    "discrepaseis": "discrepar",
    "discrepasen": "discrepar",
    "discrepases": "discrepar",
    "discrepaste": "discrepar",
    "discrepasteis": "discrepar",
    "discrepe": "discrepar",
    "discrepemos": "discrepar",
    "discrepen": "discrepar",
    "discrepes": "discrepar",
    "discrepo": "discrepar",
    "discrepábamos": "discrepar",
    "discrepáis": "discrepar",
    "discrepáramos": "discrepar",
    "discrepáremos": "discrepar",
    "discrepásemos": "discrepar",
    "discrepé": "discrepar",
    "discrepéis": "discrepar",
    "discrepó": "discrepar",
    "discreta": "discreto",
    "discretas": "discreto",
    "discretea": "discretear",
    "discreteaba": "discretear",
    "discreteabais": "discretear",
    "discreteaban": "discretear",
    "discreteabas": "discretear",
    "discretead": "discretear",
    "discreteada": "discretear",
    "discreteadas": "discretear",
    "discreteado": "discretear",
    "discreteados": "discretear",
    "discreteamos": "discretear",
    "discretean": "discretear",
    "discreteando": "discretear",
    "discreteara": "discretear",
    "discretearais": "discretear",
    "discretearan": "discretear",
    "discretearas": "discretear",
    "discreteare": "discretear",
    "discreteareis": "discretear",
    "discretearemos": "discretear",
    "discretearen": "discretear",
    "discreteares": "discretear",
    "discretearon": "discretear",
    "discreteará": "discretear",
    "discretearán": "discretear",
    "discretearás": "discretear",
    "discretearé": "discretear",
    "discretearéis": "discretear",
    "discretearía": "discretear",
    "discretearíais": "discretear",
    "discretearíamos": "discretear",
    "discretearían": "discretear",
    "discretearías": "discretear",
    "discreteas": "discretear",
    "discretease": "discretear",
    "discreteaseis": "discretear",
    "discreteasen": "discretear",
    "discreteases": "discretear",
    "discreteaste": "discretear",
    "discreteasteis": "discretear",
    "discretee": "discretear",
    "discreteemos": "discretear",
    "discreteen": "discretear",
    "discretees": "discretear",
    "discreteo": "discretear",
    "discreteos": "discreteo",
    "discreteábamos": "discretear",
    "discreteáis": "discretear",
    "discreteáramos": "discretear",
    "discreteáremos": "discretear",
    "discreteásemos": "discretear",
    "discreteé": "discretear",
    "discreteéis": "discretear",
    "discreteó": "discretear",
    "discretorios": "discretorio",
    "discretos": "discreto",
    "discrimina": "discriminar",
    "discriminaba": "discriminar",
    "discriminabais": "discriminar",
    "discriminaban": "discriminar",
    "discriminabas": "discriminar",
    "discriminaciones": "discriminación",
    "discriminad": "discriminar",
    "discriminada": "discriminar",
    "discriminadas": "discriminar",
    "discriminado": "discriminar",
    "discriminadora": "discriminador",
    "discriminadoras": "discriminador",
    "discriminadores": "discriminador",
    "discriminados": "discriminar",
    "discriminamos": "discriminar",
    "discriminan": "discriminar",
    "discriminando": "discriminar",
    "discriminantes": "discriminante",
    "discriminara": "discriminar",
    "discriminarais": "discriminar",
    "discriminaran": "discriminar",
    "discriminaras": "discriminar",
    "discriminare": "discriminar",
    "discriminareis": "discriminar",
    "discriminaremos": "discriminar",
    "discriminaren": "discriminar",
    "discriminares": "discriminar",
    "discriminaron": "discriminar",
    "discriminará": "discriminar",
    "discriminarán": "discriminar",
    "discriminarás": "discriminar",
    "discriminaré": "discriminar",
    "discriminaréis": "discriminar",
    "discriminaría": "discriminar",
    "discriminaríais": "discriminar",
    "discriminaríamos": "discriminar",
    "discriminarían": "discriminar",
    "discriminarías": "discriminar",
    "discriminas": "discriminar",
    "discriminase": "discriminar",
    "discriminaseis": "discriminar",
    "discriminasen": "discriminar",
    "discriminases": "discriminar",
    "discriminaste": "discriminar",
    "discriminasteis": "discriminar",
    "discriminatoria": "discriminatorio",
    "discriminatorias": "discriminatorio",
    "discriminatorios": "discriminatorio",
    "discrimine": "discriminar",
    "discriminemos": "discriminar",
    "discriminen": "discriminar",
    "discrimines": "discriminar",
    "discrimino": "discriminar",
    "discriminábamos": "discriminar",
    "discrimináis": "discriminar",
    "discrimináramos": "discriminar",
    "discrimináremos": "discriminar",
    "discriminásemos": "discriminar",
    "discriminé": "discriminar",
    "discriminéis": "discriminar",
    "discriminó": "discriminar",
    "discromatopsias": "discromatopsia",
    "discrímeneses": "discrimen",
    "discuerda": "discordar",
    "discuerdan": "discordar",
    "discuerdas": "discordar",
    "discuerde": "discordar",
    "discuerden": "discordar",
    "discuerdes": "discordar",
    "discuerdo": "discordar",
    "disculpa": "disculpar",
    "disculpaba": "disculpar",
    "disculpabais": "disculpar",
    "disculpaban": "disculpar",
    "disculpabas": "disculpar",
    "disculpables": "disculpable",
    "disculpad": "disculpar",
    "disculpada": "disculpar",
    "disculpadas": "disculpar",
    "disculpado": "disculpar",
    "disculpados": "disculpar",
    "disculpamos": "disculpar",
    "disculpan": "disculpar",
    "disculpando": "disculpar",
    "disculpantes": "disculpante",
    "disculpara": "disculpar",
    "disculparais": "disculpar",
    "disculparan": "disculpar",
    "disculparas": "disculpar",
    "disculpare": "disculpar",
    "disculpareis": "disculpar",
    "disculparemos": "disculpar",
    "disculparen": "disculpar",
    "disculpares": "disculpar",
    "disculparon": "disculpar",
    "disculpará": "disculpar",
    "disculparán": "disculpar",
    "disculparás": "disculpar",
    "disculparé": "disculpar",
    "disculparéis": "disculpar",
    "disculparía": "disculpar",
    "disculparíais": "disculpar",
    "disculparíamos": "disculpar",
    "disculparían": "disculpar",
    "disculparías": "disculpar",
    "disculpas": "disculpar",
    "disculpase": "disculpar",
    "disculpaseis": "disculpar",
    "disculpasen": "disculpar",
    "disculpases": "disculpar",
    "disculpaste": "disculpar",
    "disculpasteis": "disculpar",
    "disculpe": "disculpar",
    "disculpemos": "disculpar",
    "disculpen": "disculpar",
    "disculpes": "disculpar",
    "disculpo": "disculpar",
    "disculpábamos": "disculpar",
    "disculpáis": "disculpar",
    "disculpáramos": "disculpar",
    "disculpáremos": "disculpar",
    "disculpásemos": "disculpar",
    "disculpé": "disculpar",
    "disculpéis": "disculpar",
    "disculpó": "disculpar",
    "discurra": "discurrir",
    "discurramos": "discurrir",
    "discurran": "discurrir",
    "discurras": "discurrir",
    "discurre": "discurrir",
    "discurren": "discurrir",
    "discurres": "discurrir",
    "discurrid": "discurrir",
    "discurrida": "discurrir",
    "discurridas": "discurrir",
    "discurrido": "discurrir",
    "discurridos": "discurrir",
    "discurriendo": "discurrir",
    "discurriera": "discurrir",
    "discurrierais": "discurrir",
    "discurrieran": "discurrir",
    "discurrieras": "discurrir",
    "discurriere": "discurrir",
    "discurriereis": "discurrir",
    "discurrieren": "discurrir",
    "discurrieres": "discurrir",
    "discurrieron": "discurrir",
    "discurriese": "discurrir",
    "discurrieseis": "discurrir",
    "discurriesen": "discurrir",
    "discurrieses": "discurrir",
    "discurrimientos": "discurrimiento",
    "discurrimos": "discurrir",
    "discurriremos": "discurrir",
    "discurrirá": "discurrir",
    "discurrirán": "discurrir",
    "discurrirás": "discurrir",
    "discurriré": "discurrir",
    "discurriréis": "discurrir",
    "discurriría": "discurrir",
    "discurriríais": "discurrir",
    "discurriríamos": "discurrir",
    "discurrirían": "discurrir",
    "discurrirías": "discurrir",
    "discurriste": "discurrir",
    "discurristeis": "discurrir",
    "discurriéramos": "discurrir",
    "discurriéremos": "discurrir",
    "discurriésemos": "discurrir",
    "discurrió": "discurrir",
    "discurro": "discurrir",
    "discurráis": "discurrir",
    "discurrí": "discurrir",
    "discurría": "discurrir",
    "discurríais": "discurrir",
    "discurríamos": "discurrir",
    "discurrían": "discurrir",
    "discurrías": "discurrir",
    "discurrís": "discurrir",
    "discursa": "discursar",
    "discursaba": "discursar",
    "discursabais": "discursar",
    "discursaban": "discursar",
    "discursabas": "discursar",
    "discursad": "discursar",
    "discursada": "discursar",
    "discursadas": "discursar",
    "discursado": "discursar",
    "discursados": "discursar",
    "discursamos": "discursar",
    "discursan": "discursar",
    "discursando": "discursar",
    "discursara": "discursar",
    "discursarais": "discursar",
    "discursaran": "discursar",
    "discursaras": "discursar",
    "discursare": "discursar",
    "discursareis": "discursar",
    "discursaremos": "discursar",
    "discursaren": "discursar",
    "discursares": "discursar",
    "discursaron": "discursar",
    "discursará": "discursar",
    "discursarán": "discursar",
    "discursarás": "discursar",
    "discursaré": "discursar",
    "discursaréis": "discursar",
    "discursaría": "discursar",
    "discursaríais": "discursar",
    "discursaríamos": "discursar",
    "discursarían": "discursar",
    "discursarías": "discursar",
    "discursas": "discursar",
    "discursase": "discursar",
    "discursaseis": "discursar",
    "discursasen": "discursar",
    "discursases": "discursar",
    "discursaste": "discursar",
    "discursasteis": "discursar",
    "discurse": "discursar",
    "discursea": "discursear",
    "discurseaba": "discursear",
    "discurseabais": "discursear",
    "discurseaban": "discursear",
    "discurseabas": "discursear",
    "discursead": "discursear",
    "discurseada": "discursear",
    "discurseadas": "discursear",
    "discurseado": "discursear",
    "discurseados": "discursear",
    "discurseamos": "discursear",
    "discursean": "discursear",
    "discurseando": "discursear",
    "discurseara": "discursear",
    "discursearais": "discursear",
    "discursearan": "discursear",
    "discursearas": "discursear",
    "discurseare": "discursear",
    "discurseareis": "discursear",
    "discursearemos": "discursear",
    "discursearen": "discursear",
    "discurseares": "discursear",
    "discursearon": "discursear",
    "discurseará": "discursear",
    "discursearán": "discursear",
    "discursearás": "discursear",
    "discursearé": "discursear",
    "discursearéis": "discursear",
    "discursearía": "discursear",
    "discursearíais": "discursear",
    "discursearíamos": "discursear",
    "discursearían": "discursear",
    "discursearías": "discursear",
    "discurseas": "discursear",
    "discursease": "discursear",
    "discurseaseis": "discursear",
    "discurseasen": "discursear",
    "discurseases": "discursear",
    "discurseaste": "discursear",
    "discurseasteis": "discursear",
    "discursee": "discursear",
    "discurseemos": "discursear",
    "discurseen": "discursear",
    "discursees": "discursear",
    "discursemos": "discursar",
    "discursen": "discursar",
    "discurseo": "discursear",
    "discurses": "discursar",
    "discurseábamos": "discursear",
    "discurseáis": "discursear",
    "discurseáramos": "discursear",
    "discurseáremos": "discursear",
    "discurseásemos": "discursear",
    "discurseé": "discursear",
    "discurseéis": "discursear",
    "discurseó": "discursear",
    "discursistas": "discursista",
    "discursiva": "discursivo",
    "discursivas": "discursivo",
    "discursivos": "discursivo",
    "discurso": "discursar",
    "discursos": "discurso",
    "discursábamos": "discursar",
    "discursáis": "discursar",
    "discursáramos": "discursar",
    "discursáremos": "discursar",
    "discursásemos": "discursar",
    "discursé": "discursar",
    "discurséis": "discursar",
    "discursó": "discursar",
    "discusiones": "discusión",
    "discuta": "discutir",
    "discutamos": "discutir",
    "discutan": "discutir",
    "discutas": "discutir",
    "discute": "discutir",
    "discuten": "discutir",
    "discutes": "discutir",
    "discutibles": "discutible",
    "discutid": "discutir",
    "discutida": "discutir",
    "discutidas": "discutir",
    "discutido": "discutir",
    "discutidora": "discutidor",
    "discutidoras": "discutidor",
    "discutidores": "discutidor",
    "discutidos": "discutir",
    "discutiendo": "discutir",
    "discutiera": "discutir",
    "discutierais": "discutir",
    "discutieran": "discutir",
    "discutieras": "discutir",
    "discutiere": "discutir",
    "discutiereis": "discutir",
    "discutieren": "discutir",
    "discutieres": "discutir",
    "discutieron": "discutir",
    "discutiese": "discutir",
    "discutieseis": "discutir",
    "discutiesen": "discutir",
    "discutieses": "discutir",
    "discutimos": "discutir",
    "discutiremos": "discutir",
    "discutirá": "discutir",
    "discutirán": "discutir",
    "discutirás": "discutir",
    "discutiré": "discutir",
    "discutiréis": "discutir",
    "discutiría": "discutir",
    "discutiríais": "discutir",
    "discutiríamos": "discutir",
    "discutirían": "discutir",
    "discutirías": "discutir",
    "discutiste": "discutir",
    "discutisteis": "discutir",
    "discutiéramos": "discutir",
    "discutiéremos": "discutir",
    "discutiésemos": "discutir",
    "discutió": "discutir",
    "discuto": "discutir",
    "discutáis": "discutir",
    "discutí": "discutir",
    "discutía": "discutir",
    "discutíais": "discutir",
    "discutíamos": "discutir",
    "discutían": "discutir",
    "discutías": "discutir",
    "discutís": "discutir",
    "discípula": "discípulo",
    "discípulas": "discípulo",
    "discípulos": "discípulo",
    "discóbolos": "discóbolo",
    "diseca": "disecar",
    "disecaba": "disecar",
    "disecabais": "disecar",
    "disecaban": "disecar",
    "disecabas": "disecar",
    "disecaciones": "disecación",
    "disecad": "disecar",
    "disecada": "disecar",
    "disecadas": "disecar",
    "disecado": "disecar",
    "disecados": "disecar",
    "disecamos": "disecar",
    "disecan": "disecar",
    "disecando": "disecar",
    "disecara": "disecar",
    "disecarais": "disecar",
    "disecaran": "disecar",
    "disecaras": "disecar",
    "disecare": "disecar",
    "disecareis": "disecar",
    "disecaremos": "disecar",
    "disecaren": "disecar",
    "disecares": "disecar",
    "disecaron": "disecar",
    "disecará": "disecar",
    "disecarán": "disecar",
    "disecarás": "disecar",
    "disecaré": "disecar",
    "disecaréis": "disecar",
    "disecaría": "disecar",
    "disecaríais": "disecar",
    "disecaríamos": "disecar",
    "disecarían": "disecar",
    "disecarías": "disecar",
    "disecas": "disecar",
    "disecase": "disecar",
    "disecaseis": "disecar",
    "disecasen": "disecar",
    "disecases": "disecar",
    "disecaste": "disecar",
    "disecasteis": "disecar",
    "disecciona": "diseccionar",
    "diseccionaba": "diseccionar",
    "diseccionabais": "diseccionar",
    "diseccionaban": "diseccionar",
    "diseccionabas": "diseccionar",
    "diseccionad": "diseccionar",
    "diseccionada": "diseccionar",
    "diseccionadas": "diseccionar",
    "diseccionado": "diseccionar",
    "diseccionados": "diseccionar",
    "diseccionamos": "diseccionar",
    "diseccionan": "diseccionar",
    "diseccionando": "diseccionar",
    "diseccionara": "diseccionar",
    "diseccionarais": "diseccionar",
    "diseccionaran": "diseccionar",
    "diseccionaras": "diseccionar",
    "diseccionare": "diseccionar",
    "diseccionareis": "diseccionar",
    "diseccionaremos": "diseccionar",
    "diseccionaren": "diseccionar",
    "diseccionares": "diseccionar",
    "diseccionaron": "diseccionar",
    "diseccionará": "diseccionar",
    "diseccionarán": "diseccionar",
    "diseccionarás": "diseccionar",
    "diseccionaré": "diseccionar",
    "diseccionaréis": "diseccionar",
    "diseccionaría": "diseccionar",
    "diseccionaríais": "diseccionar",
    "diseccionaríamos": "diseccionar",
    "diseccionarían": "diseccionar",
    "diseccionarías": "diseccionar",
    "diseccionas": "diseccionar",
    "diseccionase": "diseccionar",
    "diseccionaseis": "diseccionar",
    "diseccionasen": "diseccionar",
    "diseccionases": "diseccionar",
    "diseccionaste": "diseccionar",
    "diseccionasteis": "diseccionar",
    "diseccione": "diseccionar",
    "diseccionemos": "diseccionar",
    "diseccionen": "diseccionar",
    "disecciones": "diseccionar",
    "disecciono": "diseccionar",
    "diseccionábamos": "diseccionar",
    "diseccionáis": "diseccionar",
    "diseccionáramos": "diseccionar",
    "diseccionáremos": "diseccionar",
    "diseccionásemos": "diseccionar",
    "diseccioné": "diseccionar",
    "diseccionéis": "diseccionar",
    "diseccionó": "diseccionar",
    "diseceas": "disecea",
    "diseco": "disecar",
    "disectora": "disector",
    "disectoras": "disector",
    "disectores": "disector",
    "disecábamos": "disecar",
    "disecáis": "disecar",
    "disecáramos": "disecar",
    "disecáremos": "disecar",
    "disecásemos": "disecar",
    "disecó": "disecar",
    "disemias": "disemia",
    "disemina": "diseminar",
    "diseminaba": "diseminar",
    "diseminabais": "diseminar",
    "diseminaban": "diseminar",
    "diseminabas": "diseminar",
    "diseminaciones": "diseminación",
    "diseminad": "diseminar",
    "diseminada": "diseminar",
    "diseminadas": "diseminar",
    "diseminado": "diseminar",
    "diseminadora": "diseminador",
    "diseminadoras": "diseminador",
    "diseminadores": "diseminador",
    "diseminados": "diseminar",
    "diseminamos": "diseminar",
    "diseminan": "diseminar",
    "diseminando": "diseminar",
    "diseminara": "diseminar",
    "diseminarais": "diseminar",
    "diseminaran": "diseminar",
    "diseminaras": "diseminar",
    "diseminare": "diseminar",
    "diseminareis": "diseminar",
    "diseminaremos": "diseminar",
    "diseminaren": "diseminar",
    "diseminares": "diseminar",
    "diseminaron": "diseminar",
    "diseminará": "diseminar",
    "diseminarán": "diseminar",
    "diseminarás": "diseminar",
    "diseminaré": "diseminar",
    "diseminaréis": "diseminar",
    "diseminaría": "diseminar",
    "diseminaríais": "diseminar",
    "diseminaríamos": "diseminar",
    "diseminarían": "diseminar",
    "diseminarías": "diseminar",
    "diseminas": "diseminar",
    "diseminase": "diseminar",
    "diseminaseis": "diseminar",
    "diseminasen": "diseminar",
    "diseminases": "diseminar",
    "diseminaste": "diseminar",
    "diseminasteis": "diseminar",
    "disemine": "diseminar",
    "diseminemos": "diseminar",
    "diseminen": "diseminar",
    "disemines": "diseminar",
    "disemino": "diseminar",
    "diseminábamos": "diseminar",
    "disemináis": "diseminar",
    "disemináramos": "diseminar",
    "disemináremos": "diseminar",
    "diseminásemos": "diseminar",
    "diseminé": "diseminar",
    "diseminéis": "diseminar",
    "diseminó": "diseminar",
    "disemínulos": "disemínulo",
    "disensiones": "disensión",
    "disensos": "disenso",
    "disenterías": "disentería",
    "disentid": "disentir",
    "disentida": "disentir",
    "disentidas": "disentir",
    "disentido": "disentir",
    "disentidos": "disentir",
    "disentimientos": "disentimiento",
    "disentimos": "disentir",
    "disentiremos": "disentir",
    "disentirá": "disentir",
    "disentirán": "disentir",
    "disentirás": "disentir",
    "disentiré": "disentir",
    "disentiréis": "disentir",
    "disentiría": "disentir",
    "disentiríais": "disentir",
    "disentiríamos": "disentir",
    "disentirían": "disentir",
    "disentirías": "disentir",
    "disentiste": "disentir",
    "disentisteis": "disentir",
    "disentérica": "disentérico",
    "disentéricas": "disentérico",
    "disentéricos": "disentérico",
    "disentí": "disentir",
    "disentía": "disentir",
    "disentíais": "disentir",
    "disentíamos": "disentir",
    "disentían": "disentir",
    "disentías": "disentir",
    "disentís": "disentir",
    "diseque": "disecar",
    "disequemos": "disecar",
    "disequen": "disecar",
    "diseques": "disecar",
    "disequé": "disecar",
    "disequéis": "disecar",
    "diserta": "diserto",
    "disertaba": "disertar",
    "disertabais": "disertar",
    "disertaban": "disertar",
    "disertabas": "disertar",
    "disertaciones": "disertación",
    "disertad": "disertar",
    "disertada": "disertar",
    "disertadas": "disertar",
    "disertado": "disertar",
    "disertadora": "disertador",
    "disertadoras": "disertador",
    "disertadores": "disertador",
    "disertados": "disertar",
    "disertamos": "disertar",
    "disertan": "disertar",
    "disertando": "disertar",
    "disertantes": "disertante",
    "disertara": "disertar",
    "disertarais": "disertar",
    "disertaran": "disertar",
    "disertaras": "disertar",
    "disertare": "disertar",
    "disertareis": "disertar",
    "disertaremos": "disertar",
    "disertaren": "disertar",
    "disertares": "disertar",
    "disertaron": "disertar",
    "disertará": "disertar",
    "disertarán": "disertar",
    "disertarás": "disertar",
    "disertaré": "disertar",
    "disertaréis": "disertar",
    "disertaría": "disertar",
    "disertaríais": "disertar",
    "disertaríamos": "disertar",
    "disertarían": "disertar",
    "disertarías": "disertar",
    "disertas": "diserto",
    "disertase": "disertar",
    "disertaseis": "disertar",
    "disertasen": "disertar",
    "disertases": "disertar",
    "disertaste": "disertar",
    "disertasteis": "disertar",
    "diserte": "disertar",
    "disertemos": "disertar",
    "diserten": "disertar",
    "disertes": "disertar",
    "diserto": "disertar",
    "disertos": "diserto",
    "disertábamos": "disertar",
    "disertáis": "disertar",
    "disertáramos": "disertar",
    "disertáremos": "disertar",
    "disertásemos": "disertar",
    "diserté": "disertar",
    "disertéis": "disertar",
    "disertó": "disertar",
    "disestesias": "disestesia",
    "diseña": "diseñar",
    "diseñaba": "diseñar",
    "diseñabais": "diseñar",
    "diseñaban": "diseñar",
    "diseñabas": "diseñar",
    "diseñad": "diseñar",
    "diseñada": "diseñar",
    "diseñadas": "diseñar",
    "diseñado": "diseñar",
    "diseñadora": "diseñador",
    "diseñadoras": "diseñador",
    "diseñadores": "diseñador",
    "diseñados": "diseñar",
    "diseñamos": "diseñar",
    "diseñan": "diseñar",
    "diseñando": "diseñar",
    "diseñara": "diseñar",
    "diseñarais": "diseñar",
    "diseñaran": "diseñar",
    "diseñaras": "diseñar",
    "diseñare": "diseñar",
    "diseñareis": "diseñar",
    "diseñaremos": "diseñar",
    "diseñaren": "diseñar",
    "diseñares": "diseñar",
    "diseñaron": "diseñar",
    "diseñará": "diseñar",
    "diseñarán": "diseñar",
    "diseñarás": "diseñar",
    "diseñaré": "diseñar",
    "diseñaréis": "diseñar",
    "diseñaría": "diseñar",
    "diseñaríais": "diseñar",
    "diseñaríamos": "diseñar",
    "diseñarían": "diseñar",
    "diseñarías": "diseñar",
    "diseñas": "diseñar",
    "diseñase": "diseñar",
    "diseñaseis": "diseñar",
    "diseñasen": "diseñar",
    "diseñases": "diseñar",
    "diseñaste": "diseñar",
    "diseñasteis": "diseñar",
    "diseñe": "diseñar",
    "diseñemos": "diseñar",
    "diseñen": "diseñar",
    "diseñes": "diseñar",
    "diseño": "diseñar",
    "diseños": "diseño",
    "diseñábamos": "diseñar",
    "diseñáis": "diseñar",
    "diseñáramos": "diseñar",
    "diseñáremos": "diseñar",
    "diseñásemos": "diseñar",
    "diseñé": "diseñar",
    "diseñéis": "diseñar",
    "diseñó": "diseñar",
    "disfagias": "disfagia",
    "disfasias": "disfasia",
    "disfavores": "disfavor",
    "disfemias": "disfemia",
    "disfemismos": "disfemismo",
    "disfonías": "disfonía",
    "disforma": "disformar",
    "disformaba": "disformar",
    "disformabais": "disformar",
    "disformaban": "disformar",
    "disformabas": "disformar",
    "disformad": "disformar",
    "disformada": "disformar",
    "disformadas": "disformar",
    "disformado": "disformar",
    "disformados": "disformar",
    "disformamos": "disformar",
    "disforman": "disformar",
    "disformando": "disformar",
    "disformara": "disformar",
    "disformarais": "disformar",
    "disformaran": "disformar",
    "disformaras": "disformar",
    "disformare": "disformar",
    "disformareis": "disformar",
    "disformaremos": "disformar",
    "disformaren": "disformar",
    "disformares": "disformar",
    "disformaron": "disformar",
    "disformará": "disformar",
    "disformarán": "disformar",
    "disformarás": "disformar",
    "disformaré": "disformar",
    "disformaréis": "disformar",
    "disformaría": "disformar",
    "disformaríais": "disformar",
    "disformaríamos": "disformar",
    "disformarían": "disformar",
    "disformarías": "disformar",
    "disformas": "disformar",
    "disformase": "disformar",
    "disformaseis": "disformar",
    "disformasen": "disformar",
    "disformases": "disformar",
    "disformaste": "disformar",
    "disformasteis": "disformar",
    "disforme": "disformar",
    "disformemos": "disformar",
    "disformen": "disformar",
    "disformes": "disforme",
    "disformidades": "disformidad",
    "disformo": "disformar",
    "disformábamos": "disformar",
    "disformáis": "disformar",
    "disformáramos": "disformar",
    "disformáremos": "disformar",
    "disformásemos": "disformar",
    "disformé": "disformar",
    "disforméis": "disformar",
    "disformó": "disformar",
    "disfrace": "disfrazar",
    "disfracemos": "disfrazar",
    "disfracen": "disfrazar",
    "disfraces": "disfrazar",
    "disfracé": "disfrazar",
    "disfracéis": "disfrazar",
    "disfraza": "disfrazar",
    "disfrazaba": "disfrazar",
    "disfrazabais": "disfrazar",
    "disfrazaban": "disfrazar",
    "disfrazabas": "disfrazar",
    "disfrazad": "disfrazar",
    "disfrazada": "disfrazar",
    "disfrazadas": "disfrazar",
    "disfrazado": "disfrazar",
    "disfrazados": "disfrazar",
    "disfrazamos": "disfrazar",
    "disfrazan": "disfrazar",
    "disfrazando": "disfrazar",
    "disfrazara": "disfrazar",
    "disfrazarais": "disfrazar",
    "disfrazaran": "disfrazar",
    "disfrazaras": "disfrazar",
    "disfrazare": "disfrazar",
    "disfrazareis": "disfrazar",
    "disfrazaremos": "disfrazar",
    "disfrazaren": "disfrazar",
    "disfrazares": "disfrazar",
    "disfrazaron": "disfrazar",
    "disfrazará": "disfrazar",
    "disfrazarán": "disfrazar",
    "disfrazarás": "disfrazar",
    "disfrazaré": "disfrazar",
    "disfrazaréis": "disfrazar",
    "disfrazaría": "disfrazar",
    "disfrazaríais": "disfrazar",
    "disfrazaríamos": "disfrazar",
    "disfrazarían": "disfrazar",
    "disfrazarías": "disfrazar",
    "disfrazas": "disfrazar",
    "disfrazase": "disfrazar",
    "disfrazaseis": "disfrazar",
    "disfrazasen": "disfrazar",
    "disfrazases": "disfrazar",
    "disfrazaste": "disfrazar",
    "disfrazasteis": "disfrazar",
    "disfrazo": "disfrazar",
    "disfrazábamos": "disfrazar",
    "disfrazáis": "disfrazar",
    "disfrazáramos": "disfrazar",
    "disfrazáremos": "disfrazar",
    "disfrazásemos": "disfrazar",
    "disfrazó": "disfrazar",
    "disfruta": "disfrutar",
    "disfrutaba": "disfrutar",
    "disfrutabais": "disfrutar",
    "disfrutaban": "disfrutar",
    "disfrutabas": "disfrutar",
    "disfrutad": "disfrutar",
    "disfrutada": "disfrutar",
    "disfrutadas": "disfrutar",
    "disfrutado": "disfrutar",
    "disfrutados": "disfrutar",
    "disfrutamos": "disfrutar",
    "disfrutan": "disfrutar",
    "disfrutando": "disfrutar",
    "disfrutara": "disfrutar",
    "disfrutarais": "disfrutar",
    "disfrutaran": "disfrutar",
    "disfrutaras": "disfrutar",
    "disfrutare": "disfrutar",
    "disfrutareis": "disfrutar",
    "disfrutaremos": "disfrutar",
    "disfrutaren": "disfrutar",
    "disfrutares": "disfrutar",
    "disfrutaron": "disfrutar",
    "disfrutará": "disfrutar",
    "disfrutarán": "disfrutar",
    "disfrutarás": "disfrutar",
    "disfrutaré": "disfrutar",
    "disfrutaréis": "disfrutar",
    "disfrutaría": "disfrutar",
    "disfrutaríais": "disfrutar",
    "disfrutaríamos": "disfrutar",
    "disfrutarían": "disfrutar",
    "disfrutarías": "disfrutar",
    "disfrutas": "disfrutar",
    "disfrutase": "disfrutar",
    "disfrutaseis": "disfrutar",
    "disfrutasen": "disfrutar",
    "disfrutases": "disfrutar",
    "disfrutaste": "disfrutar",
    "disfrutasteis": "disfrutar",
    "disfrute": "disfrutar",
    "disfrutemos": "disfrutar",
    "disfruten": "disfrutar",
    "disfrutes": "disfrute",
    "disfruto": "disfrutar",
    "disfrutábamos": "disfrutar",
    "disfrutáis": "disfrutar",
    "disfrutáramos": "disfrutar",
    "disfrutáremos": "disfrutar",
    "disfrutásemos": "disfrutar",
    "disfruté": "disfrutar",
    "disfrutéis": "disfrutar",
    "disfrutó": "disfrutar",
    "disfuma": "disfumar",
    "disfumaba": "disfumar",
    "disfumabais": "disfumar",
    "disfumaban": "disfumar",
    "disfumabas": "disfumar",
    "disfumad": "disfumar",
    "disfumada": "disfumar",
    "disfumadas": "disfumar",
    "disfumado": "disfumar",
    "disfumados": "disfumar",
    "disfumamos": "disfumar",
    "disfuman": "disfumar",
    "disfumando": "disfumar",
    "disfumara": "disfumar",
    "disfumarais": "disfumar",
    "disfumaran": "disfumar",
    "disfumaras": "disfumar",
    "disfumare": "disfumar",
    "disfumareis": "disfumar",
    "disfumaremos": "disfumar",
    "disfumaren": "disfumar",
    "disfumares": "disfumar",
    "disfumaron": "disfumar",
    "disfumará": "disfumar",
    "disfumarán": "disfumar",
    "disfumarás": "disfumar",
    "disfumaré": "disfumar",
    "disfumaréis": "disfumar",
    "disfumaría": "disfumar",
    "disfumaríais": "disfumar",
    "disfumaríamos": "disfumar",
    "disfumarían": "disfumar",
    "disfumarías": "disfumar",
    "disfumas": "disfumar",
    "disfumase": "disfumar",
    "disfumaseis": "disfumar",
    "disfumasen": "disfumar",
    "disfumases": "disfumar",
    "disfumaste": "disfumar",
    "disfumasteis": "disfumar",
    "disfume": "disfumar",
    "disfumemos": "disfumar",
    "disfumen": "disfumar",
    "disfumes": "disfumar",
    "disfuminos": "disfumino",
    "disfumo": "disfumar",
    "disfumábamos": "disfumar",
    "disfumáis": "disfumar",
    "disfumáramos": "disfumar",
    "disfumáremos": "disfumar",
    "disfumásemos": "disfumar",
    "disfumé": "disfumar",
    "disfuméis": "disfumar",
    "disfumó": "disfumar",
    "disfunciones": "disfunción",
    "disgammaglobulinemias": "disgammaglobulinemia",
    "disgenesias": "disgenesia",
    "disgeusias": "disgeusia",
    "disglobulinemias": "disglobulinemia",
    "disgrega": "disgregar",
    "disgregaba": "disgregar",
    "disgregabais": "disgregar",
    "disgregaban": "disgregar",
    "disgregabas": "disgregar",
    "disgregaciones": "disgregación",
    "disgregad": "disgregar",
    "disgregada": "disgregar",
    "disgregadas": "disgregar",
    "disgregado": "disgregar",
    "disgregadora": "disgregador",
    "disgregadoras": "disgregador",
    "disgregadores": "disgregador",
    "disgregados": "disgregar",
    "disgregamos": "disgregar",
    "disgregan": "disgregar",
    "disgregando": "disgregar",
    "disgregantes": "disgregante",
    "disgregara": "disgregar",
    "disgregarais": "disgregar",
    "disgregaran": "disgregar",
    "disgregaras": "disgregar",
    "disgregare": "disgregar",
    "disgregareis": "disgregar",
    "disgregaremos": "disgregar",
    "disgregaren": "disgregar",
    "disgregares": "disgregar",
    "disgregaron": "disgregar",
    "disgregará": "disgregar",
    "disgregarán": "disgregar",
    "disgregarás": "disgregar",
    "disgregaré": "disgregar",
    "disgregaréis": "disgregar",
    "disgregaría": "disgregar",
    "disgregaríais": "disgregar",
    "disgregaríamos": "disgregar",
    "disgregarían": "disgregar",
    "disgregarías": "disgregar",
    "disgregas": "disgregar",
    "disgregase": "disgregar",
    "disgregaseis": "disgregar",
    "disgregasen": "disgregar",
    "disgregases": "disgregar",
    "disgregaste": "disgregar",
    "disgregasteis": "disgregar",
    "disgregativa": "disgregativo",
    "disgregativas": "disgregativo",
    "disgregativos": "disgregativo",
    "disgrego": "disgregar",
    "disgregue": "disgregar",
    "disgreguemos": "disgregar",
    "disgreguen": "disgregar",
    "disgregues": "disgregar",
    "disgregué": "disgregar",
    "disgreguéis": "disgregar",
    "disgregábamos": "disgregar",
    "disgregáis": "disgregar",
    "disgregáramos": "disgregar",
    "disgregáremos": "disgregar",
    "disgregásemos": "disgregar",
    "disgregó": "disgregar",
    "disgusta": "disgustar",
    "disgustaba": "disgustar",
    "disgustabais": "disgustar",
    "disgustaban": "disgustar",
    "disgustabas": "disgustar",
    "disgustad": "disgustar",
    "disgustada": "disgustar",
    "disgustadas": "disgustar",
    "disgustado": "disgustar",
    "disgustados": "disgustar",
    "disgustamos": "disgustar",
    "disgustan": "disgustar",
    "disgustando": "disgustar",
    "disgustara": "disgustar",
    "disgustarais": "disgustar",
    "disgustaran": "disgustar",
    "disgustaras": "disgustar",
    "disgustare": "disgustar",
    "disgustareis": "disgustar",
    "disgustaremos": "disgustar",
    "disgustaren": "disgustar",
    "disgustares": "disgustar",
    "disgustaron": "disgustar",
    "disgustará": "disgustar",
    "disgustarán": "disgustar",
    "disgustarás": "disgustar",
    "disgustaré": "disgustar",
    "disgustaréis": "disgustar",
    "disgustaría": "disgustar",
    "disgustaríais": "disgustar",
    "disgustaríamos": "disgustar",
    "disgustarían": "disgustar",
    "disgustarías": "disgustar",
    "disgustas": "disgustar",
    "disgustase": "disgustar",
    "disgustaseis": "disgustar",
    "disgustasen": "disgustar",
    "disgustases": "disgustar",
    "disgustaste": "disgustar",
    "disgustasteis": "disgustar",
    "disguste": "disgustar",
    "disgustemos": "disgustar",
    "disgusten": "disgustar",
    "disgustes": "disgustar",
    "disgusto": "disgustar",
    "disgustos": "disgusto",
    "disgustábamos": "disgustar",
    "disgustáis": "disgustar",
    "disgustáramos": "disgustar",
    "disgustáremos": "disgustar",
    "disgustásemos": "disgustar",
    "disgusté": "disgustar",
    "disgustéis": "disgustar",
    "disgustó": "disgustar",
    "disida": "disidir",
    "disidamos": "disidir",
    "disidan": "disidir",
    "disidas": "disidir",
    "diside": "disidir",
    "disiden": "disidir",
    "disidencias": "disidencia",
    "disidentes": "disidente",
    "disides": "disidir",
    "disidid": "disidir",
    "disidida": "disidir",
    "disididas": "disidir",
    "disidido": "disidir",
    "disididos": "disidir",
    "disidiendo": "disidir",
    "disidiera": "disidir",
    "disidierais": "disidir",
    "disidieran": "disidir",
    "disidieras": "disidir",
    "disidiere": "disidir",
    "disidiereis": "disidir",
    "disidieren": "disidir",
    "disidieres": "disidir",
    "disidieron": "disidir",
    "disidiese": "disidir",
    "disidieseis": "disidir",
    "disidiesen": "disidir",
    "disidieses": "disidir",
    "disidimos": "disidir",
    "disidiremos": "disidir",
    "disidirá": "disidir",
    "disidirán": "disidir",
    "disidirás": "disidir",
    "disidiré": "disidir",
    "disidiréis": "disidir",
    "disidiría": "disidir",
    "disidiríais": "disidir",
    "disidiríamos": "disidir",
    "disidirían": "disidir",
    "disidirías": "disidir",
    "disidiste": "disidir",
    "disidisteis": "disidir",
    "disidiéramos": "disidir",
    "disidiéremos": "disidir",
    "disidiésemos": "disidir",
    "disidió": "disidir",
    "disido": "disidir",
    "disidáis": "disidir",
    "disidí": "disidir",
    "disidía": "disidir",
    "disidíais": "disidir",
    "disidíamos": "disidir",
    "disidían": "disidir",
    "disidías": "disidir",
    "disidís": "disidir",
    "disienta": "disentir",
    "disientan": "disentir",
    "disientas": "disentir",
    "disiente": "disentir",
    "disienten": "disentir",
    "disientes": "disentir",
    "disiento": "disentir",
    "disilicatos": "disilicato",
    "disimetrías": "disimetría",
    "disimila": "disimilar",
    "disimilaba": "disimilar",
    "disimilabais": "disimilar",
    "disimilaban": "disimilar",
    "disimilabas": "disimilar",
    "disimilaciones": "disimilación",
    "disimilad": "disimilar",
    "disimilada": "disimilar",
    "disimiladas": "disimilar",
    "disimilado": "disimilar",
    "disimilados": "disimilar",
    "disimilamos": "disimilar",
    "disimilan": "disimilar",
    "disimilando": "disimilar",
    "disimilara": "disimilar",
    "disimilarais": "disimilar",
    "disimilaran": "disimilar",
    "disimilaras": "disimilar",
    "disimilare": "disimilar",
    "disimilareis": "disimilar",
    "disimilaremos": "disimilar",
    "disimilaren": "disimilar",
    "disimilares": "disimilar",
    "disimilaron": "disimilar",
    "disimilará": "disimilar",
    "disimilarán": "disimilar",
    "disimilarás": "disimilar",
    "disimilaré": "disimilar",
    "disimilaréis": "disimilar",
    "disimilaría": "disimilar",
    "disimilaríais": "disimilar",
    "disimilaríamos": "disimilar",
    "disimilarían": "disimilar",
    "disimilarías": "disimilar",
    "disimilas": "disimilar",
    "disimilase": "disimilar",
    "disimilaseis": "disimilar",
    "disimilasen": "disimilar",
    "disimilases": "disimilar",
    "disimilaste": "disimilar",
    "disimilasteis": "disimilar",
    "disimile": "disimilar",
    "disimilemos": "disimilar",
    "disimilen": "disimilar",
    "disimiles": "disimilar",
    "disimilitudes": "disimilitud",
    "disimilo": "disimilar",
    "disimilábamos": "disimilar",
    "disimiláis": "disimilar",
    "disimiláramos": "disimilar",
    "disimiláremos": "disimilar",
    "disimilásemos": "disimilar",
    "disimilé": "disimilar",
    "disimiléis": "disimilar",
    "disimiló": "disimilar",
    "disimula": "disimular",
    "disimulaba": "disimular",
    "disimulabais": "disimular",
    "disimulaban": "disimular",
    "disimulabas": "disimular",
    "disimulables": "disimulable",
    "disimulaciones": "disimulación",
    "disimulad": "disimular",
    "disimulada": "disimular",
    "disimuladas": "disimular",
    "disimulado": "disimular",
    "disimuladora": "disimulador",
    "disimuladoras": "disimulador",
    "disimuladores": "disimulador",
    "disimulados": "disimular",
    "disimulamos": "disimular",
    "disimulan": "disimular",
    "disimulando": "disimular",
    "disimulara": "disimular",
    "disimularais": "disimular",
    "disimularan": "disimular",
    "disimularas": "disimular",
    "disimulare": "disimular",
    "disimulareis": "disimular",
    "disimularemos": "disimular",
    "disimularen": "disimular",
    "disimulares": "disimular",
    "disimularon": "disimular",
    "disimulará": "disimular",
    "disimularán": "disimular",
    "disimularás": "disimular",
    "disimularé": "disimular",
    "disimularéis": "disimular",
    "disimularía": "disimular",
    "disimularíais": "disimular",
    "disimularíamos": "disimular",
    "disimularían": "disimular",
    "disimularías": "disimular",
    "disimulas": "disimular",
    "disimulase": "disimular",
    "disimulaseis": "disimular",
    "disimulasen": "disimular",
    "disimulases": "disimular",
    "disimulaste": "disimular",
    "disimulasteis": "disimular",
    "disimule": "disimular",
    "disimulemos": "disimular",
    "disimulen": "disimular",
    "disimules": "disimular",
    "disimulo": "disimular",
    "disimulos": "disimulo",
    "disimulábamos": "disimular",
    "disimuláis": "disimular",
    "disimuláramos": "disimular",
    "disimuláremos": "disimular",
    "disimulásemos": "disimular",
    "disimulé": "disimular",
    "disimuléis": "disimular",
    "disimuló": "disimular",
    "disimétrica": "disimétrico",
    "disimétricas": "disimétrico",
    "disimétricos": "disimétrico",
    "disintamos": "disentir",
    "disintiendo": "disentir",
    "disintiera": "disentir",
    "disintierais": "disentir",
    "disintieran": "disentir",
    "disintieras": "disentir",
    "disintiere": "disentir",
    "disintiereis": "disentir",
    "disintieren": "disentir",
    "disintieres": "disentir",
    "disintieron": "disentir",
    "disintiese": "disentir",
    "disintieseis": "disentir",
    "disintiesen": "disentir",
    "disintieses": "disentir",
    "disintiéramos": "disentir",
    "disintiéremos": "disentir",
    "disintiésemos": "disentir",
    "disintió": "disentir",
    "disintáis": "disentir",
    "disipa": "disipar",
    "disipaba": "disipar",
    "disipabais": "disipar",
    "disipaban": "disipar",
    "disipabas": "disipar",
    "disipables": "disipable",
    "disipaciones": "disipación",
    "disipad": "disipar",
    "disipada": "disipar",
    "disipadas": "disipar",
    "disipado": "disipar",
    "disipadora": "disipador",
    "disipadoras": "disipador",
    "disipadores": "disipador",
    "disipados": "disipar",
    "disipamos": "disipar",
    "disipan": "disipar",
    "disipando": "disipar",
    "disipara": "disipar",
    "disiparais": "disipar",
    "disiparan": "disipar",
    "disiparas": "disipar",
    "disipare": "disipar",
    "disipareis": "disipar",
    "disiparemos": "disipar",
    "disiparen": "disipar",
    "disipares": "disipar",
    "disiparon": "disipar",
    "disipará": "disipar",
    "disiparán": "disipar",
    "disiparás": "disipar",
    "disiparé": "disipar",
    "disiparéis": "disipar",
    "disiparía": "disipar",
    "disiparíais": "disipar",
    "disiparíamos": "disipar",
    "disiparían": "disipar",
    "disiparías": "disipar",
    "disipas": "disipar",
    "disipase": "disipar",
    "disipaseis": "disipar",
    "disipasen": "disipar",
    "disipases": "disipar",
    "disipaste": "disipar",
    "disipasteis": "disipar",
    "disipativa": "disipativo",
    "disipativas": "disipativo",
    "disipativos": "disipativo",
    "disipe": "disipar",
    "disipemos": "disipar",
    "disipen": "disipar",
    "disipes": "disipar",
    "disipo": "disipar",
    "disipábamos": "disipar",
    "disipáis": "disipar",
    "disipáramos": "disipar",
    "disipáremos": "disipar",
    "disipásemos": "disipar",
    "disipé": "disipar",
    "disipéis": "disipar",
    "disipó": "disipar",
    "disjunta": "disjunto",
    "disjuntas": "disjunto",
    "disjuntos": "disjunto",
    "diskettes": "diskette",
    "dislalias": "dislalia",
    "dislates": "dislate",
    "dislexias": "dislexia",
    "dislipemias": "dislipemia",
    "disloca": "dislocar",
    "dislocaba": "dislocar",
    "dislocabais": "dislocar",
    "dislocaban": "dislocar",
    "dislocabas": "dislocar",
    "dislocaciones": "dislocación",
    "dislocad": "dislocar",
    "dislocada": "dislocar",
    "dislocadas": "dislocar",
    "dislocado": "dislocar",
    "dislocados": "dislocar",
    "dislocaduras": "dislocadura",
    "dislocamos": "dislocar",
    "dislocan": "dislocar",
    "dislocando": "dislocar",
    "dislocara": "dislocar",
    "dislocarais": "dislocar",
    "dislocaran": "dislocar",
    "dislocaras": "dislocar",
    "dislocare": "dislocar",
    "dislocareis": "dislocar",
    "dislocaremos": "dislocar",
    "dislocaren": "dislocar",
    "dislocares": "dislocar",
    "dislocaron": "dislocar",
    "dislocará": "dislocar",
    "dislocarán": "dislocar",
    "dislocarás": "dislocar",
    "dislocaré": "dislocar",
    "dislocaréis": "dislocar",
    "dislocaría": "dislocar",
    "dislocaríais": "dislocar",
    "dislocaríamos": "dislocar",
    "dislocarían": "dislocar",
    "dislocarías": "dislocar",
    "dislocas": "dislocar",
    "dislocase": "dislocar",
    "dislocaseis": "dislocar",
    "dislocasen": "dislocar",
    "dislocases": "dislocar",
    "dislocaste": "dislocar",
    "dislocasteis": "dislocar",
    "disloco": "dislocar",
    "dislocábamos": "dislocar",
    "dislocáis": "dislocar",
    "dislocáramos": "dislocar",
    "dislocáremos": "dislocar",
    "dislocásemos": "dislocar",
    "dislocó": "dislocar",
    "disloque": "dislocar",
    "disloquemos": "dislocar",
    "disloquen": "dislocar",
    "disloques": "disloque",
    "disloqué": "dislocar",
    "disloquéis": "dislocar",
    "disléxica": "disléxico",
    "disléxicas": "disléxico",
    "disléxicos": "disléxico",
    "dismenorreas": "dismenorrea",
    "disminuciones": "disminución",
    "disminuciónes": "disminución",
    "disminuid": "disminuir",
    "disminuida": "disminuir",
    "disminuidas": "disminuir",
    "disminuido": "disminuir",
    "disminuidos": "disminuir",
    "disminuimos": "disminuir",
    "disminuiremos": "disminuir",
    "disminuirá": "disminuir",
    "disminuirán": "disminuir",
    "disminuirás": "disminuir",
    "disminuiré": "disminuir",
    "disminuiréis": "disminuir",
    "disminuiría": "disminuir",
    "disminuiríais": "disminuir",
    "disminuiríamos": "disminuir",
    "disminuirían": "disminuir",
    "disminuirías": "disminuir",
    "disminuiste": "disminuir",
    "disminuisteis": "disminuir",
    "disminuya": "disminuir",
    "disminuyamos": "disminuir",
    "disminuyan": "disminuir",
    "disminuyas": "disminuir",
    "disminuye": "disminuir",
    "disminuyen": "disminuir",
    "disminuyendo": "disminuir",
    "disminuyera": "disminuir",
    "disminuyerais": "disminuir",
    "disminuyeran": "disminuir",
    "disminuyeras": "disminuir",
    "disminuyere": "disminuir",
    "disminuyereis": "disminuir",
    "disminuyeren": "disminuir",
    "disminuyeres": "disminuir",
    "disminuyeron": "disminuir",
    "disminuyes": "disminuir",
    "disminuyese": "disminuir",
    "disminuyeseis": "disminuir",
    "disminuyesen": "disminuir",
    "disminuyeses": "disminuir",
    "disminuyo": "disminuir",
    "disminuyáis": "disminuir",
    "disminuyéramos": "disminuir",
    "disminuyéremos": "disminuir",
    "disminuyésemos": "disminuir",
    "disminuyó": "disminuir",
    "disminuí": "disminuir",
    "disminuía": "disminuir",
    "disminuíais": "disminuir",
    "disminuíamos": "disminuir",
    "disminuían": "disminuir",
    "disminuías": "disminuir",
    "disminuís": "disminuir",
    "dismorfias": "dismorfia",
    "disneas": "disnea",
    "disneica": "disneico",
    "disneicas": "disneico",
    "disneicos": "disneico",
    "disnoias": "disnoia",
    "disocia": "disociar",
    "disociaba": "disociar",
    "disociabais": "disociar",
    "disociaban": "disociar",
    "disociabas": "disociar",
    "disociables": "disociable",
    "disociaciones": "disociación",
    "disociad": "disociar",
    "disociada": "disociar",
    "disociadas": "disociar",
    "disociado": "disociar",
    "disociadora": "disociador",
    "disociadoras": "disociador",
    "disociadores": "disociador",
    "disociados": "disociar",
    "disociales": "disocial",
    "disociamos": "disociar",
    "disocian": "disociar",
    "disociando": "disociar",
    "disociara": "disociar",
    "disociarais": "disociar",
    "disociaran": "disociar",
    "disociaras": "disociar",
    "disociare": "disociar",
    "disociareis": "disociar",
    "disociaremos": "disociar",
    "disociaren": "disociar",
    "disociares": "disociar",
    "disociaron": "disociar",
    "disociará": "disociar",
    "disociarán": "disociar",
    "disociarás": "disociar",
    "disociaré": "disociar",
    "disociaréis": "disociar",
    "disociaría": "disociar",
    "disociaríais": "disociar",
    "disociaríamos": "disociar",
    "disociarían": "disociar",
    "disociarías": "disociar",
    "disocias": "disociar",
    "disociase": "disociar",
    "disociaseis": "disociar",
    "disociasen": "disociar",
    "disociases": "disociar",
    "disociaste": "disociar",
    "disociasteis": "disociar",
    "disociativa": "disociativo",
    "disociativas": "disociativo",
    "disociativos": "disociativo",
    "disocie": "disociar",
    "disociemos": "disociar",
    "disocien": "disociar",
    "disocies": "disociar",
    "disocio": "disociar",
    "disociábamos": "disociar",
    "disociáis": "disociar",
    "disociáramos": "disociar",
    "disociáremos": "disociar",
    "disociásemos": "disociar",
    "disocié": "disociar",
    "disociéis": "disociar",
    "disoció": "disociar",
    "disoclusiones": "disoclusión",
    "disolubilidades": "disolubilidad",
    "disolubles": "disoluble",
    "disoluciones": "disolución",
    "disoluta": "disoluto",
    "disolutas": "disoluto",
    "disolutos": "disoluto",
    "disolvamos": "disolver",
    "disolved": "disolver",
    "disolvemos": "disolver",
    "disolventes": "disolvente",
    "disolveremos": "disolver",
    "disolverá": "disolver",
    "disolverán": "disolver",
    "disolverás": "disolver",
    "disolveré": "disolver",
    "disolveréis": "disolver",
    "disolvería": "disolver",
    "disolveríais": "disolver",
    "disolveríamos": "disolver",
    "disolverían": "disolver",
    "disolverías": "disolver",
    "disolviendo": "disolver",
    "disolviera": "disolver",
    "disolvierais": "disolver",
    "disolvieran": "disolver",
    "disolvieras": "disolver",
    "disolviere": "disolver",
    "disolviereis": "disolver",
    "disolvieren": "disolver",
    "disolvieres": "disolver",
    "disolvieron": "disolver",
    "disolviese": "disolver",
    "disolvieseis": "disolver",
    "disolviesen": "disolver",
    "disolvieses": "disolver",
    "disolvimos": "disolver",
    "disolviste": "disolver",
    "disolvisteis": "disolver",
    "disolviéramos": "disolver",
    "disolviéremos": "disolver",
    "disolviésemos": "disolver",
    "disolvió": "disolver",
    "disolváis": "disolver",
    "disolvéis": "disolver",
    "disolví": "disolver",
    "disolvía": "disolver",
    "disolvíais": "disolver",
    "disolvíamos": "disolver",
    "disolvían": "disolver",
    "disolvías": "disolver",
    "disonaba": "disonar",
    "disonabais": "disonar",
    "disonaban": "disonar",
    "disonabas": "disonar",
    "disonad": "disonar",
    "disonada": "disonar",
    "disonadas": "disonar",
    "disonado": "disonar",
    "disonados": "disonar",
    "disonamos": "disonar",
    "disonancias": "disonancia",
    "disonando": "disonar",
    "disonantes": "disonante",
    "disonara": "disonar",
    "disonarais": "disonar",
    "disonaran": "disonar",
    "disonaras": "disonar",
    "disonare": "disonar",
    "disonareis": "disonar",
    "disonaremos": "disonar",
    "disonaren": "disonar",
    "disonares": "disonar",
    "disonaron": "disonar",
    "disonará": "disonar",
    "disonarán": "disonar",
    "disonarás": "disonar",
    "disonaré": "disonar",
    "disonaréis": "disonar",
    "disonaría": "disonar",
    "disonaríais": "disonar",
    "disonaríamos": "disonar",
    "disonarían": "disonar",
    "disonarías": "disonar",
    "disonase": "disonar",
    "disonaseis": "disonar",
    "disonasen": "disonar",
    "disonases": "disonar",
    "disonaste": "disonar",
    "disonasteis": "disonar",
    "disonemos": "disonar",
    "disones": "disón",
    "disonábamos": "disonar",
    "disonáis": "disonar",
    "disonáramos": "disonar",
    "disonáremos": "disonar",
    "disonásemos": "disonar",
    "disoné": "disonar",
    "disonéis": "disonar",
    "disonó": "disonar",
    "disosmias": "disosmia",
    "dispara": "disparar",
    "disparaba": "disparar",
    "disparabais": "disparar",
    "disparaban": "disparar",
    "disparabas": "disparar",
    "disparad": "disparar",
    "disparada": "disparar",
    "disparadas": "disparar",
    "disparaderos": "disparadero",
    "disparado": "disparar",
    "disparadores": "disparador",
    "disparados": "disparar",
    "disparamos": "disparar",
    "disparan": "disparar",
    "disparando": "disparar",
    "disparara": "disparar",
    "dispararais": "disparar",
    "dispararan": "disparar",
    "dispararas": "disparar",
    "disparare": "disparar",
    "disparareis": "disparar",
    "dispararemos": "disparar",
    "dispararen": "disparar",
    "disparares": "disparar",
    "dispararon": "disparar",
    "disparará": "disparar",
    "dispararán": "disparar",
    "dispararás": "disparar",
    "dispararé": "disparar",
    "dispararéis": "disparar",
    "dispararía": "disparar",
    "dispararíais": "disparar",
    "dispararíamos": "disparar",
    "dispararían": "disparar",
    "dispararías": "disparar",
    "disparas": "disparar",
    "disparase": "disparar",
    "disparaseis": "disparar",
    "disparasen": "disparar",
    "disparases": "disparar",
    "disparaste": "disparar",
    "disparasteis": "disparar",
    "disparata": "disparatar",
    "disparataba": "disparatar",
    "disparatabais": "disparatar",
    "disparataban": "disparatar",
    "disparatabas": "disparatar",
    "disparatad": "disparatar",
    "disparatada": "disparatar",
    "disparatadas": "disparatar",
    "disparatado": "disparatar",
    "disparatadora": "disparatador",
    "disparatadoras": "disparatador",
    "disparatadores": "disparatador",
    "disparatados": "disparatar",
    "disparatamos": "disparatar",
    "disparatan": "disparatar",
    "disparatando": "disparatar",
    "disparatara": "disparatar",
    "disparatarais": "disparatar",
    "disparataran": "disparatar",
    "disparataras": "disparatar",
    "disparatare": "disparatar",
    "disparatareis": "disparatar",
    "disparataremos": "disparatar",
    "disparataren": "disparatar",
    "disparatares": "disparatar",
    "disparataron": "disparatar",
    "disparatará": "disparatar",
    "disparatarán": "disparatar",
    "disparatarás": "disparatar",
    "disparataré": "disparatar",
    "disparataréis": "disparatar",
    "disparataría": "disparatar",
    "disparataríais": "disparatar",
    "disparataríamos": "disparatar",
    "disparatarían": "disparatar",
    "disparatarías": "disparatar",
    "disparatas": "disparatar",
    "disparatase": "disparatar",
    "disparataseis": "disparatar",
    "disparatasen": "disparatar",
    "disparatases": "disparatar",
    "disparataste": "disparatar",
    "disparatasteis": "disparatar",
    "disparate": "disparatar",
    "disparatemos": "disparatar",
    "disparaten": "disparatar",
    "disparates": "disparate",
    "disparato": "disparatar",
    "disparatorios": "disparatorio",
    "disparatábamos": "disparatar",
    "disparatáis": "disparatar",
    "disparatáramos": "disparatar",
    "disparatáremos": "disparatar",
    "disparatásemos": "disparatar",
    "disparaté": "disparatar",
    "disparatéis": "disparatar",
    "disparató": "disparatar",
    "dispare": "disparar",
    "dispareja": "disparejo",
    "disparejas": "disparejo",
    "disparejos": "disparejo",
    "disparemos": "disparar",
    "disparen": "disparar",
    "dispares": "disparar",
    "disparidades": "disparidad",
    "disparo": "disparar",
    "disparos": "disparo",
    "disparábamos": "disparar",
    "disparáis": "disparar",
    "disparáramos": "disparar",
    "disparáremos": "disparar",
    "disparásemos": "disparar",
    "disparé": "disparar",
    "disparéis": "disparar",
    "disparó": "disparar",
    "dispendios": "dispendio",
    "dispendiosa": "dispendioso",
    "dispendiosas": "dispendioso",
    "dispendiosos": "dispendioso",
    "dispensa": "dispensar",
    "dispensaba": "dispensar",
    "dispensabais": "dispensar",
    "dispensaban": "dispensar",
    "dispensabas": "dispensar",
    "dispensables": "dispensable",
    "dispensaciones": "dispensación",
    "dispensad": "dispensar",
    "dispensada": "dispensar",
    "dispensadas": "dispensar",
    "dispensado": "dispensar",
    "dispensadora": "dispensador",
    "dispensadoras": "dispensador",
    "dispensadores": "dispensador",
    "dispensados": "dispensar",
    "dispensamientos": "dispensamiento",
    "dispensamos": "dispensar",
    "dispensan": "dispensar",
    "dispensando": "dispensar",
    "dispensara": "dispensar",
    "dispensarais": "dispensar",
    "dispensaran": "dispensar",
    "dispensaras": "dispensar",
    "dispensare": "dispensar",
    "dispensareis": "dispensar",
    "dispensaremos": "dispensar",
    "dispensaren": "dispensar",
    "dispensares": "dispensar",
    "dispensarios": "dispensario",
    "dispensaron": "dispensar",
    "dispensará": "dispensar",
    "dispensarán": "dispensar",
    "dispensarás": "dispensar",
    "dispensaré": "dispensar",
    "dispensaréis": "dispensar",
    "dispensaría": "dispensar",
    "dispensaríais": "dispensar",
    "dispensaríamos": "dispensar",
    "dispensarían": "dispensar",
    "dispensarías": "dispensar",
    "dispensas": "dispensar",
    "dispensase": "dispensar",
    "dispensaseis": "dispensar",
    "dispensasen": "dispensar",
    "dispensases": "dispensar",
    "dispensaste": "dispensar",
    "dispensasteis": "dispensar",
    "dispense": "dispensar",
    "dispensemos": "dispensar",
    "dispensen": "dispensar",
    "dispenses": "dispensar",
    "dispensiosa": "dispensioso",
    "dispensiosas": "dispensioso",
    "dispensiosos": "dispensioso",
    "dispenso": "dispensar",
    "dispensábamos": "dispensar",
    "dispensáis": "dispensar",
    "dispensáramos": "dispensar",
    "dispensáremos": "dispensar",
    "dispensásemos": "dispensar",
    "dispensé": "dispensar",
    "dispenséis": "dispensar",
    "dispensó": "dispensar",
    "dispepsias": "dispepsia",
    "dispermias": "dispermia",
    "dispersa": "disperso",
    "dispersaba": "dispersar",
    "dispersabais": "dispersar",
    "dispersaban": "dispersar",
    "dispersabas": "dispersar",
    "dispersaciones": "dispersación",
    "dispersad": "dispersar",
    "dispersada": "dispersar",
    "dispersadas": "dispersar",
    "dispersado": "dispersar",
    "dispersadora": "dispersador",
    "dispersadoras": "dispersador",
    "dispersadores": "dispersador",
    "dispersados": "dispersar",
    "dispersamos": "dispersar",
    "dispersan": "dispersar",
    "dispersando": "dispersar",
    "dispersara": "dispersar",
    "dispersarais": "dispersar",
    "dispersaran": "dispersar",
    "dispersaras": "dispersar",
    "dispersare": "dispersar",
    "dispersareis": "dispersar",
    "dispersaremos": "dispersar",
    "dispersaren": "dispersar",
    "dispersares": "dispersar",
    "dispersaron": "dispersar",
    "dispersará": "dispersar",
    "dispersarán": "dispersar",
    "dispersarás": "dispersar",
    "dispersaré": "dispersar",
    "dispersaréis": "dispersar",
    "dispersaría": "dispersar",
    "dispersaríais": "dispersar",
    "dispersaríamos": "dispersar",
    "dispersarían": "dispersar",
    "dispersarías": "dispersar",
    "dispersas": "disperso",
    "dispersase": "dispersar",
    "dispersaseis": "dispersar",
    "dispersasen": "dispersar",
    "dispersases": "dispersar",
    "dispersaste": "dispersar",
    "dispersasteis": "dispersar",
    "disperse": "dispersar",
    "dispersemos": "dispersar",
    "dispersen": "dispersar",
    "disperses": "dispersar",
    "dispersiones": "dispersión",
    "dispersiva": "dispersivo",
    "dispersivas": "dispersivo",
    "dispersivos": "dispersivo",
    "disperso": "dispersar",
    "dispersora": "dispersor",
    "dispersoras": "dispersor",
    "dispersores": "dispersor",
    "dispersos": "disperso",
    "dispersábamos": "dispersar",
    "dispersáis": "dispersar",
    "dispersáramos": "dispersar",
    "dispersáremos": "dispersar",
    "dispersásemos": "dispersar",
    "dispersé": "dispersar",
    "disperséis": "dispersar",
    "dispersó": "dispersar",
    "displace": "displacer",
    "displaced": "displacer",
    "displacemos": "displacer",
    "displacen": "displacer",
    "displaceremos": "displacer",
    "displaceres": "displacer",
    "displacerá": "displacer",
    "displacerán": "displacer",
    "displacerás": "displacer",
    "displaceré": "displacer",
    "displaceréis": "displacer",
    "displacería": "displacer",
    "displaceríais": "displacer",
    "displaceríamos": "displacer",
    "displacerían": "displacer",
    "displacerías": "displacer",
    "displaces": "displacer",
    "displacida": "displacer",
    "displacidas": "displacer",
    "displacido": "displacer",
    "displacidos": "displacer",
    "displaciendo": "displacer",
    "displaciera": "displacer",
    "displacierais": "displacer",
    "displacieran": "displacer",
    "displacieras": "displacer",
    "displaciere": "displacer",
    "displaciereis": "displacer",
    "displacieren": "displacer",
    "displacieres": "displacer",
    "displacieron": "displacer",
    "displaciese": "displacer",
    "displacieseis": "displacer",
    "displaciesen": "displacer",
    "displacieses": "displacer",
    "displacimos": "displacer",
    "displaciste": "displacer",
    "displacisteis": "displacer",
    "displaciéramos": "displacer",
    "displaciéremos": "displacer",
    "displaciésemos": "displacer",
    "displació": "displacer",
    "displacéis": "displacer",
    "displací": "displacer",
    "displacía": "displacer",
    "displacíais": "displacer",
    "displacíamos": "displacer",
    "displacían": "displacer",
    "displacías": "displacer",
    "displasias": "displasia",
    "displays": "display",
    "displazca": "displacer",
    "displazcamos": "displacer",
    "displazcan": "displacer",
    "displazcas": "displacer",
    "displazco": "displacer",
    "displazcáis": "displacer",
    "displicencias": "displicencia",
    "displicentes": "displicente",
    "dispondeos": "dispondeo",
    "dispondremos": "disponer",
    "dispondrá": "disponer",
    "dispondrán": "disponer",
    "dispondrás": "disponer",
    "dispondré": "disponer",
    "dispondréis": "disponer",
    "dispondría": "disponer",
    "dispondríais": "disponer",
    "dispondríamos": "disponer",
    "dispondrían": "disponer",
    "dispondrías": "disponer",
    "dispone": "disponer",
    "disponed": "disponer",
    "disponedora": "disponedor",
    "disponedoras": "disponedor",
    "disponedores": "disponedor",
    "disponemos": "disponer",
    "disponen": "disponer",
    "dispones": "disponer",
    "disponga": "disponer",
    "dispongamos": "disponer",
    "dispongan": "disponer",
    "dispongas": "disponer",
    "dispongo": "disponer",
    "dispongáis": "disponer",
    "disponibilidades": "disponibilidad",
    "disponibles": "disponible",
    "disponiendo": "disponer",
    "disponéis": "disponer",
    "disponía": "disponer",
    "disponíais": "disponer",
    "disponíamos": "disponer",
    "disponían": "disponer",
    "disponías": "disponer",
    "disposiciones": "disposición",
    "dispositiva": "dispositivo",
    "dispositivas": "dispositivo",
    "dispositivos": "dispositivo",
    "disprosios": "disprosio",
    "dispuesta": "disponer",
    "dispuestas": "disponer",
    "dispuesto": "disponer",
    "dispuestos": "disponer",
    "dispuse": "disponer",
    "dispusiera": "disponer",
    "dispusierais": "disponer",
    "dispusieran": "disponer",
    "dispusieras": "disponer",
    "dispusiere": "disponer",
    "dispusiereis": "disponer",
    "dispusieren": "disponer",
    "dispusieres": "disponer",
    "dispusieron": "disponer",
    "dispusiese": "disponer",
    "dispusieseis": "disponer",
    "dispusiesen": "disponer",
    "dispusieses": "disponer",
    "dispusimos": "disponer",
    "dispusiste": "disponer",
    "dispusisteis": "disponer",
    "dispusiéramos": "disponer",
    "dispusiéremos": "disponer",
    "dispusiésemos": "disponer",
    "dispuso": "disponer",
    "disputa": "disputar",
    "disputaba": "disputar",
    "disputabais": "disputar",
    "disputaban": "disputar",
    "disputabas": "disputar",
    "disputables": "disputable",
    "disputad": "disputar",
    "disputada": "disputar",
    "disputadas": "disputar",
    "disputado": "disputar",
    "disputadora": "disputador",
    "disputadoras": "disputador",
    "disputadores": "disputador",
    "disputados": "disputar",
    "disputamos": "disputar",
    "disputan": "disputar",
    "disputando": "disputar",
    "disputara": "disputar",
    "disputarais": "disputar",
    "disputaran": "disputar",
    "disputaras": "disputar",
    "disputare": "disputar",
    "disputareis": "disputar",
    "disputaremos": "disputar",
    "disputaren": "disputar",
    "disputares": "disputar",
    "disputaron": "disputar",
    "disputará": "disputar",
    "disputarán": "disputar",
    "disputarás": "disputar",
    "disputaré": "disputar",
    "disputaréis": "disputar",
    "disputaría": "disputar",
    "disputaríais": "disputar",
    "disputaríamos": "disputar",
    "disputarían": "disputar",
    "disputarías": "disputar",
    "disputas": "disputar",
    "disputase": "disputar",
    "disputaseis": "disputar",
    "disputasen": "disputar",
    "disputases": "disputar",
    "disputaste": "disputar",
    "disputasteis": "disputar",
    "dispute": "disputar",
    "disputemos": "disputar",
    "disputen": "disputar",
    "disputes": "disputar",
    "disputo": "disputar",
    "disputábamos": "disputar",
    "disputáis": "disputar",
    "disputáramos": "disputar",
    "disputáremos": "disputar",
    "disputásemos": "disputar",
    "disputé": "disputar",
    "disputéis": "disputar",
    "disputó": "disputar",
    "dispéptica": "dispéptico",
    "dispépticas": "dispéptico",
    "dispépticos": "dispéptico",
    "dispón": "disponer",
    "disqueteras": "disquetera",
    "disquetes": "disquete",
    "disquets": "disquet",
    "disquisiciones": "disquisición",
    "disritmias": "disritmia",
    "disruptiva": "disruptivo",
    "disruptivas": "disruptivo",
    "disruptivos": "disruptivo",
    "disruptores": "disruptor",
    "dista": "distar",
    "distaba": "distar",
    "distabais": "distar",
    "distaban": "distar",
    "distabas": "distar",
    "distad": "distar",
    "distada": "distar",
    "distadas": "distar",
    "distado": "distar",
    "distados": "distar",
    "distales": "distal",
    "distalizaciones": "distalización",
    "distamos": "distar",
    "distan": "distar",
    "distancia": "distanciar",
    "distanciaba": "distanciar",
    "distanciabais": "distanciar",
    "distanciaban": "distanciar",
    "distanciabas": "distanciar",
    "distanciaciones": "distanciación",
    "distanciad": "distanciar",
    "distanciada": "distanciar",
    "distanciadas": "distanciar",
    "distanciado": "distanciar",
    "distanciadora": "distanciador",
    "distanciadoras": "distanciador",
    "distanciadores": "distanciador",
    "distanciados": "distanciar",
    "distanciamientos": "distanciamiento",
    "distanciamos": "distanciar",
    "distancian": "distanciar",
    "distanciando": "distanciar",
    "distanciara": "distanciar",
    "distanciarais": "distanciar",
    "distanciaran": "distanciar",
    "distanciaras": "distanciar",
    "distanciare": "distanciar",
    "distanciareis": "distanciar",
    "distanciaremos": "distanciar",
    "distanciaren": "distanciar",
    "distanciares": "distanciar",
    "distanciaron": "distanciar",
    "distanciará": "distanciar",
    "distanciarán": "distanciar",
    "distanciarás": "distanciar",
    "distanciaré": "distanciar",
    "distanciaréis": "distanciar",
    "distanciaría": "distanciar",
    "distanciaríais": "distanciar",
    "distanciaríamos": "distanciar",
    "distanciarían": "distanciar",
    "distanciarías": "distanciar",
    "distancias": "distanciar",
    "distanciase": "distanciar",
    "distanciaseis": "distanciar",
    "distanciasen": "distanciar",
    "distanciases": "distanciar",
    "distanciaste": "distanciar",
    "distanciasteis": "distanciar",
    "distancie": "distanciar",
    "distanciemos": "distanciar",
    "distancien": "distanciar",
    "distancies": "distanciar",
    "distancio": "distanciar",
    "distanciábamos": "distanciar",
    "distanciáis": "distanciar",
    "distanciáramos": "distanciar",
    "distanciáremos": "distanciar",
    "distanciásemos": "distanciar",
    "distancié": "distanciar",
    "distanciéis": "distanciar",
    "distanció": "distanciar",
    "distanciómetrica": "distanciómetrico",
    "distanciómetricas": "distanciómetrico",
    "distanciómetricos": "distanciómetrico",
    "distanciómetros": "distanciómetro",
    "distando": "distar",
    "distantes": "distante",
    "distara": "distar",
    "distarais": "distar",
    "distaran": "distar",
    "distaras": "distar",
    "distare": "distar",
    "distareis": "distar",
    "distaremos": "distar",
    "distaren": "distar",
    "distares": "distar",
    "distaron": "distar",
    "distará": "distar",
    "distarán": "distar",
    "distarás": "distar",
    "distaré": "distar",
    "distaréis": "distar",
    "distaría": "distar",
    "distaríais": "distar",
    "distaríamos": "distar",
    "distarían": "distar",
    "distarías": "distar",
    "distas": "distar",
    "distase": "distar",
    "distaseis": "distar",
    "distasen": "distar",
    "distases": "distar",
    "distasias": "distasia",
    "distaste": "distar",
    "distasteis": "distar",
    "diste": "distar",
    "distearatos": "distearato",
    "disteis": "dar",
    "distemos": "distar",
    "disten": "distar",
    "distenas": "distena",
    "distendamos": "distender",
    "distended": "distender",
    "distendemos": "distender",
    "distenderemos": "distender",
    "distenderá": "distender",
    "distenderán": "distender",
    "distenderás": "distender",
    "distenderé": "distender",
    "distenderéis": "distender",
    "distendería": "distender",
    "distenderíais": "distender",
    "distenderíamos": "distender",
    "distenderían": "distender",
    "distenderías": "distender",
    "distendida": "distender",
    "distendidas": "distender",
    "distendido": "distender",
    "distendidos": "distender",
    "distendiendo": "distender",
    "distendiera": "distender",
    "distendierais": "distender",
    "distendieran": "distender",
    "distendieras": "distender",
    "distendiere": "distender",
    "distendiereis": "distender",
    "distendieren": "distender",
    "distendieres": "distender",
    "distendieron": "distender",
    "distendiese": "distender",
    "distendieseis": "distender",
    "distendiesen": "distender",
    "distendieses": "distender",
    "distendimientos": "distendimiento",
    "distendimos": "distender",
    "distendiste": "distender",
    "distendisteis": "distender",
    "distendiéramos": "distender",
    "distendiéremos": "distender",
    "distendiésemos": "distender",
    "distendió": "distender",
    "distendáis": "distender",
    "distendéis": "distender",
    "distendí": "distender",
    "distendía": "distender",
    "distendíais": "distender",
    "distendíamos": "distender",
    "distendían": "distender",
    "distendías": "distender",
    "distensibles": "distensible",
    "distensiones": "distensión",
    "distermias": "distermia",
    "distes": "distar",
    "distienda": "distender",
    "distiendan": "distender",
    "distiendas": "distender",
    "distiende": "distender",
    "distienden": "distender",
    "distiendes": "distender",
    "distiendo": "distender",
    "distimias": "distimia",
    "distinciones": "distinción",
    "distinga": "distinguir",
    "distingamos": "distinguir",
    "distingan": "distinguir",
    "distingas": "distinguir",
    "distingo": "distinguir",
    "distingos": "distingo",
    "distingue": "distinguir",
    "distinguen": "distinguir",
    "distingues": "distinguir",
    "distinguibles": "distinguible",
    "distinguid": "distinguir",
    "distinguida": "distinguir",
    "distinguidas": "distinguir",
    "distinguido": "distinguir",
    "distinguidora": "distinguidor",
    "distinguidoras": "distinguidor",
    "distinguidores": "distinguidor",
    "distinguidos": "distinguir",
    "distinguiendo": "distinguir",
    "distinguiera": "distinguir",
    "distinguierais": "distinguir",
    "distinguieran": "distinguir",
    "distinguieras": "distinguir",
    "distinguiere": "distinguir",
    "distinguiereis": "distinguir",
    "distinguieren": "distinguir",
    "distinguieres": "distinguir",
    "distinguieron": "distinguir",
    "distinguiese": "distinguir",
    "distinguieseis": "distinguir",
    "distinguiesen": "distinguir",
    "distinguieses": "distinguir",
    "distinguimos": "distinguir",
    "distinguiremos": "distinguir",
    "distinguirá": "distinguir",
    "distinguirán": "distinguir",
    "distinguirás": "distinguir",
    "distinguiré": "distinguir",
    "distinguiréis": "distinguir",
    "distinguiría": "distinguir",
    "distinguiríais": "distinguir",
    "distinguiríamos": "distinguir",
    "distinguirían": "distinguir",
    "distinguirías": "distinguir",
    "distinguiste": "distinguir",
    "distinguisteis": "distinguir",
    "distinguiéramos": "distinguir",
    "distinguiéremos": "distinguir",
    "distinguiésemos": "distinguir",
    "distinguió": "distinguir",
    "distinguí": "distinguir",
    "distinguía": "distinguir",
    "distinguíais": "distinguir",
    "distinguíamos": "distinguir",
    "distinguían": "distinguir",
    "distinguías": "distinguir",
    "distinguís": "distinguir",
    "distingáis": "distinguir",
    "distinta": "distinto",
    "distintas": "distinto",
    "distintiva": "distintivo",
    "distintivas": "distintivo",
    "distintivos": "distintivo",
    "distintos": "distinto",
    "disto": "distar",
    "distoangulares": "distoangular",
    "distobucales": "distobucal",
    "distocias": "distocia",
    "distoclusales": "distoclusal",
    "distoclusiones": "distoclusión",
    "distolinguales": "distolingual",
    "distomolares": "distomolar",
    "distonías": "distonía",
    "distooclusiones": "distooclusión",
    "distopias": "distopia",
    "distorsiona": "distorsionar",
    "distorsionaba": "distorsionar",
    "distorsionabais": "distorsionar",
    "distorsionaban": "distorsionar",
    "distorsionabas": "distorsionar",
    "distorsionad": "distorsionar",
    "distorsionada": "distorsionar",
    "distorsionadas": "distorsionar",
    "distorsionado": "distorsionar",
    "distorsionadora": "distorsionador",
    "distorsionadoras": "distorsionador",
    "distorsionadores": "distorsionador",
    "distorsionados": "distorsionar",
    "distorsionamos": "distorsionar",
    "distorsionan": "distorsionar",
    "distorsionando": "distorsionar",
    "distorsionara": "distorsionar",
    "distorsionarais": "distorsionar",
    "distorsionaran": "distorsionar",
    "distorsionaras": "distorsionar",
    "distorsionare": "distorsionar",
    "distorsionareis": "distorsionar",
    "distorsionaremos": "distorsionar",
    "distorsionaren": "distorsionar",
    "distorsionares": "distorsionar",
    "distorsionaron": "distorsionar",
    "distorsionará": "distorsionar",
    "distorsionarán": "distorsionar",
    "distorsionarás": "distorsionar",
    "distorsionaré": "distorsionar",
    "distorsionaréis": "distorsionar",
    "distorsionaría": "distorsionar",
    "distorsionaríais": "distorsionar",
    "distorsionaríamos": "distorsionar",
    "distorsionarían": "distorsionar",
    "distorsionarías": "distorsionar",
    "distorsionas": "distorsionar",
    "distorsionase": "distorsionar",
    "distorsionaseis": "distorsionar",
    "distorsionasen": "distorsionar",
    "distorsionases": "distorsionar",
    "distorsionaste": "distorsionar",
    "distorsionasteis": "distorsionar",
    "distorsione": "distorsionar",
    "distorsionemos": "distorsionar",
    "distorsionen": "distorsionar",
    "distorsiones": "distorsionar",
    "distorsiono": "distorsionar",
    "distorsionábamos": "distorsionar",
    "distorsionáis": "distorsionar",
    "distorsionáramos": "distorsionar",
    "distorsionáremos": "distorsionar",
    "distorsionásemos": "distorsionar",
    "distorsioné": "distorsionar",
    "distorsionéis": "distorsionar",
    "distorsionó": "distorsionar",
    "distoversiones": "distoversión",
    "distracciones": "distracción",
    "distrae": "distraer",
    "distraed": "distraer",
    "distraemos": "distraer",
    "distraen": "distraer",
    "distraeremos": "distraer",
    "distraerá": "distraer",
    "distraerán": "distraer",
    "distraerás": "distraer",
    "distraeré": "distraer",
    "distraeréis": "distraer",
    "distraería": "distraer",
    "distraeríais": "distraer",
    "distraeríamos": "distraer",
    "distraerían": "distraer",
    "distraerías": "distraer",
    "distraes": "distraer",
    "distraiga": "distraer",
    "distraigamos": "distraer",
    "distraigan": "distraer",
    "distraigas": "distraer",
    "distraigo": "distraer",
    "distraigáis": "distraer",
    "distraje": "distraer",
    "distrajera": "distraer",
    "distrajerais": "distraer",
    "distrajeran": "distraer",
    "distrajeras": "distraer",
    "distrajere": "distraer",
    "distrajereis": "distraer",
    "distrajeren": "distraer",
    "distrajeres": "distraer",
    "distrajeron": "distraer",
    "distrajese": "distraer",
    "distrajeseis": "distraer",
    "distrajesen": "distraer",
    "distrajeses": "distraer",
    "distrajimos": "distraer",
    "distrajiste": "distraer",
    "distrajisteis": "distraer",
    "distrajo": "distraer",
    "distrajéramos": "distraer",
    "distrajéremos": "distraer",
    "distrajésemos": "distraer",
    "distrayendo": "distraer",
    "distraéis": "distraer",
    "distraía": "distraer",
    "distraíais": "distraer",
    "distraíamos": "distraer",
    "distraían": "distraer",
    "distraías": "distraer",
    "distraída": "distraído",
    "distraídas": "distraído",
    "distraído": "distraer",
    "distraídos": "distraído",
    "distribucionales": "distribucional",
    "distribucionalismos": "distribucionalismo",
    "distribuciones": "distribución",
    "distribuibles": "distribuible",
    "distribuid": "distribuir",
    "distribuida": "distribuir",
    "distribuidas": "distribuir",
    "distribuido": "distribuir",
    "distribuidora": "distribuidor",
    "distribuidoras": "distribuidora",
    "distribuidores": "distribuidor",
    "distribuidos": "distribuir",
    "distribuimos": "distribuir",
    "distribuiremos": "distribuir",
    "distribuirá": "distribuir",
    "distribuirán": "distribuir",
    "distribuirás": "distribuir",
    "distribuiré": "distribuir",
    "distribuiréis": "distribuir",
    "distribuiría": "distribuir",
    "distribuiríais": "distribuir",
    "distribuiríamos": "distribuir",
    "distribuirían": "distribuir",
    "distribuirías": "distribuir",
    "distribuiste": "distribuir",
    "distribuisteis": "distribuir",
    "distributiva": "distributivo",
    "distributivas": "distributivo",
    "distributividades": "distributividad",
    "distributivos": "distributivo",
    "distribuya": "distribuir",
    "distribuyamos": "distribuir",
    "distribuyan": "distribuir",
    "distribuyas": "distribuir",
    "distribuye": "distribuir",
    "distribuyen": "distribuir",
    "distribuyendo": "distribuir",
    "distribuyera": "distribuir",
    "distribuyerais": "distribuir",
    "distribuyeran": "distribuir",
    "distribuyeras": "distribuir",
    "distribuyere": "distribuir",
    "distribuyereis": "distribuir",
    "distribuyeren": "distribuir",
    "distribuyeres": "distribuir",
    "distribuyeron": "distribuir",
    "distribuyes": "distribuir",
    "distribuyese": "distribuir",
    "distribuyeseis": "distribuir",
    "distribuyesen": "distribuir",
    "distribuyeses": "distribuir",
    "distribuyo": "distribuir",
    "distribuyáis": "distribuir",
    "distribuyéramos": "distribuir",
    "distribuyéremos": "distribuir",
    "distribuyésemos": "distribuir",
    "distribuyó": "distribuir",
    "distribuí": "distribuir",
    "distribuía": "distribuir",
    "distribuíais": "distribuir",
    "distribuíamos": "distribuir",
    "distribuían": "distribuir",
    "distribuías": "distribuir",
    "distribuís": "distribuir",
    "distritos": "distrito",
    "distrofias": "distrofia",
    "distrófica": "distrófico",
    "distróficas": "distrófico",
    "distróficos": "distrófico",
    "disturba": "disturbar",
    "disturbaba": "disturbar",
    "disturbabais": "disturbar",
    "disturbaban": "disturbar",
    "disturbabas": "disturbar",
    "disturbad": "disturbar",
    "disturbada": "disturbar",
    "disturbadas": "disturbar",
    "disturbado": "disturbar",
    "disturbados": "disturbar",
    "disturbamos": "disturbar",
    "disturban": "disturbar",
    "disturbando": "disturbar",
    "disturbara": "disturbar",
    "disturbarais": "disturbar",
    "disturbaran": "disturbar",
    "disturbaras": "disturbar",
    "disturbare": "disturbar",
    "disturbareis": "disturbar",
    "disturbaremos": "disturbar",
    "disturbaren": "disturbar",
    "disturbares": "disturbar",
    "disturbaron": "disturbar",
    "disturbará": "disturbar",
    "disturbarán": "disturbar",
    "disturbarás": "disturbar",
    "disturbaré": "disturbar",
    "disturbaréis": "disturbar",
    "disturbaría": "disturbar",
    "disturbaríais": "disturbar",
    "disturbaríamos": "disturbar",
    "disturbarían": "disturbar",
    "disturbarías": "disturbar",
    "disturbas": "disturbar",
    "disturbase": "disturbar",
    "disturbaseis": "disturbar",
    "disturbasen": "disturbar",
    "disturbases": "disturbar",
    "disturbaste": "disturbar",
    "disturbasteis": "disturbar",
    "disturbe": "disturbar",
    "disturbemos": "disturbar",
    "disturben": "disturbar",
    "disturbes": "disturbar",
    "disturbios": "disturbio",
    "disturbo": "disturbar",
    "disturbábamos": "disturbar",
    "disturbáis": "disturbar",
    "disturbáramos": "disturbar",
    "disturbáremos": "disturbar",
    "disturbásemos": "disturbar",
    "disturbé": "disturbar",
    "disturbéis": "disturbar",
    "disturbó": "disturbar",
    "distábamos": "distar",
    "distáis": "distar",
    "distáramos": "distar",
    "distáremos": "distar",
    "distásemos": "distar",
    "disté": "distar",
    "distéis": "distar",
    "distó": "distar",
    "distócica": "distócico",
    "distócicas": "distócico",
    "distócicos": "distócico",
    "distómidos": "distómido",
    "disuada": "disuadir",
    "disuadamos": "disuadir",
    "disuadan": "disuadir",
    "disuadas": "disuadir",
    "disuade": "disuadir",
    "disuaden": "disuadir",
    "disuades": "disuadir",
    "disuadid": "disuadir",
    "disuadida": "disuadir",
    "disuadidas": "disuadir",
    "disuadido": "disuadir",
    "disuadidos": "disuadir",
    "disuadiendo": "disuadir",
    "disuadiera": "disuadir",
    "disuadierais": "disuadir",
    "disuadieran": "disuadir",
    "disuadieras": "disuadir",
    "disuadiere": "disuadir",
    "disuadiereis": "disuadir",
    "disuadieren": "disuadir",
    "disuadieres": "disuadir",
    "disuadieron": "disuadir",
    "disuadiese": "disuadir",
    "disuadieseis": "disuadir",
    "disuadiesen": "disuadir",
    "disuadieses": "disuadir",
    "disuadimos": "disuadir",
    "disuadiremos": "disuadir",
    "disuadirá": "disuadir",
    "disuadirán": "disuadir",
    "disuadirás": "disuadir",
    "disuadiré": "disuadir",
    "disuadiréis": "disuadir",
    "disuadiría": "disuadir",
    "disuadiríais": "disuadir",
    "disuadiríamos": "disuadir",
    "disuadirían": "disuadir",
    "disuadirías": "disuadir",
    "disuadiste": "disuadir",
    "disuadisteis": "disuadir",
    "disuadiéramos": "disuadir",
    "disuadiéremos": "disuadir",
    "disuadiésemos": "disuadir",
    "disuadió": "disuadir",
    "disuado": "disuadir",
    "disuadáis": "disuadir",
    "disuadí": "disuadir",
    "disuadía": "disuadir",
    "disuadíais": "disuadir",
    "disuadíamos": "disuadir",
    "disuadían": "disuadir",
    "disuadías": "disuadir",
    "disuadís": "disuadir",
    "disuasiones": "disuasión",
    "disuasiva": "disuasivo",
    "disuasivas": "disuasivo",
    "disuasivos": "disuasivo",
    "disuasora": "disuasor",
    "disuasoras": "disuasor",
    "disuasores": "disuasor",
    "disuasoria": "disuasorio",
    "disuasorias": "disuasorio",
    "disuasorios": "disuasorio",
    "disuelta": "disolver",
    "disueltas": "disolver",
    "disuelto": "disolver",
    "disueltos": "disolver",
    "disuelva": "disolver",
    "disuelvan": "disolver",
    "disuelvas": "disolver",
    "disuelve": "disolver",
    "disuelven": "disolver",
    "disuelves": "disolver",
    "disuelvo": "disolver",
    "disuena": "disonar",
    "disuenan": "disonar",
    "disuenas": "disonar",
    "disuene": "disonar",
    "disuenen": "disonar",
    "disuenes": "disonar",
    "disueno": "disonar",
    "disulfirams": "disulfiram",
    "disulfuros": "disulfuro",
    "disulfúrica": "disulfúrico",
    "disulfúricas": "disulfúrico",
    "disulfúricos": "disulfúrico",
    "disurias": "disuria",
    "disyunciones": "disyunción",
    "disyuntiva": "disyuntivo",
    "disyuntivas": "disyuntivo",
    "disyuntivos": "disyuntivo",
    "disyuntora": "disyuntor",
    "disyuntoras": "disyuntor",
    "disyuntores": "disyuntor",
    "disámaras": "disámara",
    "disépala": "disépalo",
    "disépalas": "disépalo",
    "disépalos": "disépalo",
    "disílabos": "disílabo",
    "disímiles": "disímil",
    "disúrica": "disúrico",
    "disúricas": "disúrico",
    "disúricos": "disúrico",
    "ditas": "dita",
    "diterpenos": "diterpeno",
    "diteísmos": "diteísmo",
    "diteístas": "diteísta",
    "ditiocarbámica": "ditiocarbámico",
    "ditiocarbámicas": "ditiocarbámico",
    "ditiocarbámicos": "ditiocarbámico",
    "ditiooxamidas": "ditiooxamida",
    "ditiopirofosfatos": "ditiopirofosfato",
    "ditirambos": "ditirambo",
    "ditirámbica": "ditirámbico",
    "ditirámbicas": "ditirámbico",
    "ditirámbicos": "ditirámbico",
    "ditizonas": "ditizona",
    "ditografías": "ditografía",
    "ditologías": "ditología",
    "ditroqueos": "ditroqueo",
    "diuranatos": "diuranato",
    "diurna": "diurno",
    "diurnas": "diurno",
    "diurnos": "diurno",
    "diurética": "diurético",
    "diuréticas": "diurético",
    "diuréticos": "diurético",
    "diuturna": "diuturno",
    "diuturnas": "diuturno",
    "diuturnidades": "diuturnidad",
    "diuturnos": "diuturno",
    "diva": "divo",
    "divaga": "divagar",
    "divagaba": "divagar",
    "divagabais": "divagar",
    "divagaban": "divagar",
    "divagabas": "divagar",
    "divagaciones": "divagación",
    "divagad": "divagar",
    "divagada": "divagar",
    "divagadas": "divagar",
    "divagado": "divagar",
    "divagadora": "divagador",
    "divagadoras": "divagador",
    "divagadores": "divagador",
    "divagados": "divagar",
    "divagamos": "divagar",
    "divagan": "divagar",
    "divagando": "divagar",
    "divagantes": "divagante",
    "divagara": "divagar",
    "divagarais": "divagar",
    "divagaran": "divagar",
    "divagaras": "divagar",
    "divagare": "divagar",
    "divagareis": "divagar",
    "divagaremos": "divagar",
    "divagaren": "divagar",
    "divagares": "divagar",
    "divagaron": "divagar",
    "divagará": "divagar",
    "divagarán": "divagar",
    "divagarás": "divagar",
    "divagaré": "divagar",
    "divagaréis": "divagar",
    "divagaría": "divagar",
    "divagaríais": "divagar",
    "divagaríamos": "divagar",
    "divagarían": "divagar",
    "divagarías": "divagar",
    "divagas": "divagar",
    "divagase": "divagar",
    "divagaseis": "divagar",
    "divagasen": "divagar",
    "divagases": "divagar",
    "divagaste": "divagar",
    "divagasteis": "divagar",
    "divago": "divagar",
    "divague": "divagar",
    "divaguemos": "divagar",
    "divaguen": "divagar",
    "divagues": "divagar",
    "divagué": "divagar",
    "divaguéis": "divagar",
    "divagábamos": "divagar",
    "divagáis": "divagar",
    "divagáramos": "divagar",
    "divagáremos": "divagar",
    "divagásemos": "divagar",
    "divagó": "divagar",
    "divalentes": "divalente",
    "divanes": "diván",
    "divas": "divo",
    "diverge": "divergir",
    "divergen": "divergir",
    "divergencias": "divergencia",
    "divergentes": "divergente",
    "diverges": "divergir",
    "divergid": "divergir",
    "divergida": "divergir",
    "divergidas": "divergir",
    "divergido": "divergir",
    "divergidos": "divergir",
    "divergiendo": "divergir",
    "divergiera": "divergir",
    "divergierais": "divergir",
    "divergieran": "divergir",
    "divergieras": "divergir",
    "divergiere": "divergir",
    "divergiereis": "divergir",
    "divergieren": "divergir",
    "divergieres": "divergir",
    "divergieron": "divergir",
    "divergiese": "divergir",
    "divergieseis": "divergir",
    "divergiesen": "divergir",
    "divergieses": "divergir",
    "divergimos": "divergir",
    "divergiremos": "divergir",
    "divergirá": "divergir",
    "divergirán": "divergir",
    "divergirás": "divergir",
    "divergiré": "divergir",
    "divergiréis": "divergir",
    "divergiría": "divergir",
    "divergiríais": "divergir",
    "divergiríamos": "divergir",
    "divergirían": "divergir",
    "divergirías": "divergir",
    "divergiste": "divergir",
    "divergisteis": "divergir",
    "divergiéramos": "divergir",
    "divergiéremos": "divergir",
    "divergiésemos": "divergir",
    "divergió": "divergir",
    "divergí": "divergir",
    "divergía": "divergir",
    "divergíais": "divergir",
    "divergíamos": "divergir",
    "divergían": "divergir",
    "divergías": "divergir",
    "divergís": "divergir",
    "diverja": "divergir",
    "diverjamos": "divergir",
    "diverjan": "divergir",
    "diverjas": "divergir",
    "diverjo": "divergir",
    "diverjáis": "divergir",
    "diversa": "diverso",
    "diversas": "diverso",
    "diversidades": "diversidad",
    "diversifica": "diversificar",
    "diversificaba": "diversificar",
    "diversificabais": "diversificar",
    "diversificaban": "diversificar",
    "diversificabas": "diversificar",
    "diversificaciones": "diversificación",
    "diversificad": "diversificar",
    "diversificada": "diversificar",
    "diversificadas": "diversificar",
    "diversificado": "diversificar",
    "diversificados": "diversificar",
    "diversificamos": "diversificar",
    "diversifican": "diversificar",
    "diversificando": "diversificar",
    "diversificara": "diversificar",
    "diversificarais": "diversificar",
    "diversificaran": "diversificar",
    "diversificaras": "diversificar",
    "diversificare": "diversificar",
    "diversificareis": "diversificar",
    "diversificaremos": "diversificar",
    "diversificaren": "diversificar",
    "diversificares": "diversificar",
    "diversificaron": "diversificar",
    "diversificará": "diversificar",
    "diversificarán": "diversificar",
    "diversificarás": "diversificar",
    "diversificaré": "diversificar",
    "diversificaréis": "diversificar",
    "diversificaría": "diversificar",
    "diversificaríais": "diversificar",
    "diversificaríamos": "diversificar",
    "diversificarían": "diversificar",
    "diversificarías": "diversificar",
    "diversificas": "diversificar",
    "diversificase": "diversificar",
    "diversificaseis": "diversificar",
    "diversificasen": "diversificar",
    "diversificases": "diversificar",
    "diversificaste": "diversificar",
    "diversificasteis": "diversificar",
    "diversifico": "diversificar",
    "diversificábamos": "diversificar",
    "diversificáis": "diversificar",
    "diversificáramos": "diversificar",
    "diversificáremos": "diversificar",
    "diversificásemos": "diversificar",
    "diversificó": "diversificar",
    "diversifique": "diversificar",
    "diversifiquemos": "diversificar",
    "diversifiquen": "diversificar",
    "diversifiques": "diversificar",
    "diversifiqué": "diversificar",
    "diversifiquéis": "diversificar",
    "diversiflora": "diversifloro",
    "diversifloras": "diversifloro",
    "diversifloros": "diversifloro",
    "diversifoliada": "diversifoliado",
    "diversifoliadas": "diversifoliado",
    "diversifoliados": "diversifoliado",
    "diversiformes": "diversiforme",
    "diversiones": "diversión",
    "diversiva": "diversivo",
    "diversivas": "diversivo",
    "diversivos": "diversivo",
    "diversos": "diverso",
    "divertid": "divertir",
    "divertida": "divertir",
    "divertidas": "divertir",
    "divertido": "divertir",
    "divertidos": "divertir",
    "divertimentos": "divertimento",
    "divertimientos": "divertimiento",
    "divertimos": "divertir",
    "divertiremos": "divertir",
    "divertirá": "divertir",
    "divertirán": "divertir",
    "divertirás": "divertir",
    "divertiré": "divertir",
    "divertiréis": "divertir",
    "divertiría": "divertir",
    "divertiríais": "divertir",
    "divertiríamos": "divertir",
    "divertirían": "divertir",
    "divertirías": "divertir",
    "divertiste": "divertir",
    "divertisteis": "divertir",
    "divertí": "divertir",
    "divertía": "divertir",
    "divertíais": "divertir",
    "divertíamos": "divertir",
    "divertían": "divertir",
    "divertías": "divertir",
    "divertículos": "divertículo",
    "divertís": "divertir",
    "divida": "dividir",
    "dividamos": "dividir",
    "dividan": "dividir",
    "dividas": "dividir",
    "divide": "dividir",
    "dividen": "dividir",
    "dividendos": "dividendo",
    "divides": "dividir",
    "dividid": "dividir",
    "dividida": "dividir",
    "divididas": "dividir",
    "dividido": "dividir",
    "divididos": "dividir",
    "dividiendo": "dividir",
    "dividiera": "dividir",
    "dividierais": "dividir",
    "dividieran": "dividir",
    "dividieras": "dividir",
    "dividiere": "dividir",
    "dividiereis": "dividir",
    "dividieren": "dividir",
    "dividieres": "dividir",
    "dividieron": "dividir",
    "dividiese": "dividir",
    "dividieseis": "dividir",
    "dividiesen": "dividir",
    "dividieses": "dividir",
    "dividimos": "dividir",
    "dividiremos": "dividir",
    "dividirá": "dividir",
    "dividirán": "dividir",
    "dividirás": "dividir",
    "dividiré": "dividir",
    "dividiréis": "dividir",
    "dividiría": "dividir",
    "dividiríais": "dividir",
    "dividiríamos": "dividir",
    "dividirían": "dividir",
    "dividirías": "dividir",
    "dividiste": "dividir",
    "dividisteis": "dividir",
    "dividiéramos": "dividir",
    "dividiéremos": "dividir",
    "dividiésemos": "dividir",
    "dividió": "dividir",
    "divido": "dividir",
    "dividáis": "dividir",
    "dividí": "dividir",
    "dividía": "dividir",
    "dividíais": "dividir",
    "dividíamos": "dividir",
    "dividían": "dividir",
    "dividías": "dividir",
    "dividís": "dividir",
    "divierta": "divertir",
    "diviertan": "divertir",
    "diviertas": "divertir",
    "divierte": "divertir",
    "divierten": "divertir",
    "diviertes": "divertir",
    "divierto": "divertir",
    "diviesos": "divieso",
    "divina": "divino",
    "divinas": "divino",
    "divinativa": "divinativo",
    "divinativas": "divinativo",
    "divinativos": "divinativo",
    "divinatoria": "divinatorio",
    "divinatorias": "divinatorio",
    "divinatorios": "divinatorio",
    "divinice": "divinizar",
    "divinicemos": "divinizar",
    "divinicen": "divinizar",
    "divinices": "divinizar",
    "divinicé": "divinizar",
    "divinicéis": "divinizar",
    "divinidades": "divinidad",
    "divinilbencenos": "divinilbenceno",
    "diviniza": "divinizar",
    "divinizaba": "divinizar",
    "divinizabais": "divinizar",
    "divinizaban": "divinizar",
    "divinizabas": "divinizar",
    "divinizaciones": "divinización",
    "divinizad": "divinizar",
    "divinizada": "divinizar",
    "divinizadas": "divinizar",
    "divinizado": "divinizar",
    "divinizados": "divinizar",
    "divinizamos": "divinizar",
    "divinizan": "divinizar",
    "divinizando": "divinizar",
    "divinizara": "divinizar",
    "divinizarais": "divinizar",
    "divinizaran": "divinizar",
    "divinizaras": "divinizar",
    "divinizare": "divinizar",
    "divinizareis": "divinizar",
    "divinizaremos": "divinizar",
    "divinizaren": "divinizar",
    "divinizares": "divinizar",
    "divinizaron": "divinizar",
    "divinizará": "divinizar",
    "divinizarán": "divinizar",
    "divinizarás": "divinizar",
    "divinizaré": "divinizar",
    "divinizaréis": "divinizar",
    "divinizaría": "divinizar",
    "divinizaríais": "divinizar",
    "divinizaríamos": "divinizar",
    "divinizarían": "divinizar",
    "divinizarías": "divinizar",
    "divinizas": "divinizar",
    "divinizase": "divinizar",
    "divinizaseis": "divinizar",
    "divinizasen": "divinizar",
    "divinizases": "divinizar",
    "divinizaste": "divinizar",
    "divinizasteis": "divinizar",
    "divinizo": "divinizar",
    "divinizábamos": "divinizar",
    "divinizáis": "divinizar",
    "divinizáramos": "divinizar",
    "divinizáremos": "divinizar",
    "divinizásemos": "divinizar",
    "divinizó": "divinizar",
    "divinos": "divino",
    "divirtamos": "divertir",
    "divirtiendo": "divertir",
    "divirtiera": "divertir",
    "divirtierais": "divertir",
    "divirtieran": "divertir",
    "divirtieras": "divertir",
    "divirtiere": "divertir",
    "divirtiereis": "divertir",
    "divirtieren": "divertir",
    "divirtieres": "divertir",
    "divirtieron": "divertir",
    "divirtiese": "divertir",
    "divirtieseis": "divertir",
    "divirtiesen": "divertir",
    "divirtieses": "divertir",
    "divirtiéramos": "divertir",
    "divirtiéremos": "divertir",
    "divirtiésemos": "divertir",
    "divirtió": "divertir",
    "divirtáis": "divertir",
    "divisa": "divisar",
    "divisaba": "divisar",
    "divisabais": "divisar",
    "divisaban": "divisar",
    "divisabas": "divisar",
    "divisad": "divisar",
    "divisada": "divisar",
    "divisadas": "divisar",
    "divisado": "divisar",
    "divisados": "divisar",
    "divisamos": "divisar",
    "divisan": "divisar",
    "divisando": "divisar",
    "divisara": "divisar",
    "divisarais": "divisar",
    "divisaran": "divisar",
    "divisaras": "divisar",
    "divisare": "divisar",
    "divisareis": "divisar",
    "divisaremos": "divisar",
    "divisaren": "divisar",
    "divisares": "divisar",
    "divisaron": "divisar",
    "divisará": "divisar",
    "divisarán": "divisar",
    "divisarás": "divisar",
    "divisaré": "divisar",
    "divisaréis": "divisar",
    "divisaría": "divisar",
    "divisaríais": "divisar",
    "divisaríamos": "divisar",
    "divisarían": "divisar",
    "divisarías": "divisar",
    "divisas": "divisar",
    "divisase": "divisar",
    "divisaseis": "divisar",
    "divisasen": "divisar",
    "divisases": "divisar",
    "divisaste": "divisar",
    "divisasteis": "divisar",
    "divise": "divisar",
    "divisemos": "divisar",
    "divisen": "divisar",
    "divises": "divisar",
    "divisibilidades": "divisibilidad",
    "divisibles": "divisible",
    "divisionales": "divisional",
    "divisionaria": "divisionario",
    "divisionarias": "divisionario",
    "divisionarios": "divisionario",
    "divisiones": "división",
    "divisionismos": "divisionismo",
    "divismos": "divismo",
    "diviso": "divisar",
    "divisora": "divisor",
    "divisoras": "divisor",
    "divisores": "divisor",
    "divisoria": "divisorio",
    "divisorias": "divisorio",
    "divisorios": "divisorio",
    "divisábamos": "divisar",
    "divisáis": "divisar",
    "divisáramos": "divisar",
    "divisáremos": "divisar",
    "divisásemos": "divisar",
    "divisé": "divisar",
    "diviséis": "divisar",
    "divisó": "divisar",
    "divorcia": "divorciar",
    "divorciaba": "divorciar",
    "divorciabais": "divorciar",
    "divorciaban": "divorciar",
    "divorciabas": "divorciar",
    "divorciad": "divorciar",
    "divorciada": "divorciar",
    "divorciadas": "divorciar",
    "divorciado": "divorciar",
    "divorciados": "divorciar",
    "divorciamos": "divorciar",
    "divorcian": "divorciar",
    "divorciando": "divorciar",
    "divorciara": "divorciar",
    "divorciarais": "divorciar",
    "divorciaran": "divorciar",
    "divorciaras": "divorciar",
    "divorciare": "divorciar",
    "divorciareis": "divorciar",
    "divorciaremos": "divorciar",
    "divorciaren": "divorciar",
    "divorciares": "divorciar",
    "divorciaron": "divorciar",
    "divorciará": "divorciar",
    "divorciarán": "divorciar",
    "divorciarás": "divorciar",
    "divorciaré": "divorciar",
    "divorciaréis": "divorciar",
    "divorciaría": "divorciar",
    "divorciaríais": "divorciar",
    "divorciaríamos": "divorciar",
    "divorciarían": "divorciar",
    "divorciarías": "divorciar",
    "divorcias": "divorciar",
    "divorciase": "divorciar",
    "divorciaseis": "divorciar",
    "divorciasen": "divorciar",
    "divorciases": "divorciar",
    "divorciaste": "divorciar",
    "divorciasteis": "divorciar",
    "divorcie": "divorciar",
    "divorciemos": "divorciar",
    "divorcien": "divorciar",
    "divorcies": "divorciar",
    "divorcio": "divorciar",
    "divorcios": "divorcio",
    "divorciábamos": "divorciar",
    "divorciáis": "divorciar",
    "divorciáramos": "divorciar",
    "divorciáremos": "divorciar",
    "divorciásemos": "divorciar",
    "divorcié": "divorciar",
    "divorciéis": "divorciar",
    "divorció": "divorciar",
    "divos": "divo",
    "divulga": "divulgar",
    "divulgaba": "divulgar",
    "divulgabais": "divulgar",
    "divulgaban": "divulgar",
    "divulgabas": "divulgar",
    "divulgables": "divulgable",
    "divulgaciones": "divulgación",
    "divulgad": "divulgar",
    "divulgada": "divulgar",
    "divulgadas": "divulgar",
    "divulgado": "divulgar",
    "divulgadora": "divulgador",
    "divulgadoras": "divulgador",
    "divulgadores": "divulgador",
    "divulgados": "divulgar",
    "divulgamos": "divulgar",
    "divulgan": "divulgar",
    "divulgando": "divulgar",
    "divulgara": "divulgar",
    "divulgarais": "divulgar",
    "divulgaran": "divulgar",
    "divulgaras": "divulgar",
    "divulgare": "divulgar",
    "divulgareis": "divulgar",
    "divulgaremos": "divulgar",
    "divulgaren": "divulgar",
    "divulgares": "divulgar",
    "divulgaron": "divulgar",
    "divulgará": "divulgar",
    "divulgarán": "divulgar",
    "divulgarás": "divulgar",
    "divulgaré": "divulgar",
    "divulgaréis": "divulgar",
    "divulgaría": "divulgar",
    "divulgaríais": "divulgar",
    "divulgaríamos": "divulgar",
    "divulgarían": "divulgar",
    "divulgarías": "divulgar",
    "divulgas": "divulgar",
    "divulgase": "divulgar",
    "divulgaseis": "divulgar",
    "divulgasen": "divulgar",
    "divulgases": "divulgar",
    "divulgaste": "divulgar",
    "divulgasteis": "divulgar",
    "divulgativa": "divulgativo",
    "divulgativas": "divulgativo",
    "divulgativos": "divulgativo",
    "divulgo": "divulgar",
    "divulgue": "divulgar",
    "divulguemos": "divulgar",
    "divulguen": "divulgar",
    "divulgues": "divulgar",
    "divulgué": "divulgar",
    "divulguéis": "divulgar",
    "divulgábamos": "divulgar",
    "divulgáis": "divulgar",
    "divulgáramos": "divulgar",
    "divulgáremos": "divulgar",
    "divulgásemos": "divulgar",
    "divulgó": "divulgar",
    "diwanes": "diwan",
    "diwans": "diwan",
    "dixantogenatos": "dixantogenato",
    "diyambos": "diyambo",
    "diyodohidroxiquinolinas": "diyodohidroxiquinolina",
    "diyodotironinas": "diyodotironina",
    "diyámbica": "diyámbico",
    "diyámbicas": "diyámbico",
    "diyámbicos": "diyámbico",
    "dizques": "dizque",
    "diábolos": "diábolo",
    "diáconos": "diácono",
    "diádica": "diádico",
    "diádicas": "diádico",
    "diádicos": "diádico",
    "diádocos": "diádoco",
    "diáfana": "diáfano",
    "diáfanas": "diáfano",
    "diáfanos": "diáfano",
    "diágrafos": "diágrafo",
    "diálagas": "diálaga",
    "diálogos": "diálogo",
    "diámetros": "diámetro",
    "diápsidos": "diápsido",
    "diásporas": "diáspora",
    "diásporos": "diásporo",
    "diástila": "diástilo",
    "diástilas": "diástilo",
    "diástilos": "diástilo",
    "diástoles": "diástole",
    "diédrica": "diédrico",
    "diédricas": "diédrico",
    "diédricos": "diédrico",
    "diéramos": "dar",
    "diéremos": "dar",
    "diésemos": "dar",
    "diña": "diñar",
    "diñaba": "diñar",
    "diñabais": "diñar",
    "diñaban": "diñar",
    "diñabas": "diñar",
    "diñad": "diñar",
    "diñada": "diñar",
    "diñadas": "diñar",
    "diñado": "diñar",
    "diñados": "diñar",
    "diñamos": "diñar",
    "diñan": "diñar",
    "diñando": "diñar",
    "diñara": "diñar",
    "diñarais": "diñar",
    "diñaran": "diñar",
    "diñaras": "diñar",
    "diñare": "diñar",
    "diñareis": "diñar",
    "diñaremos": "diñar",
    "diñaren": "diñar",
    "diñares": "diñar",
    "diñaron": "diñar",
    "diñará": "diñar",
    "diñarán": "diñar",
    "diñarás": "diñar",
    "diñaré": "diñar",
    "diñaréis": "diñar",
    "diñaría": "diñar",
    "diñaríais": "diñar",
    "diñaríamos": "diñar",
    "diñarían": "diñar",
    "diñarías": "diñar",
    "diñas": "diñar",
    "diñase": "diñar",
    "diñaseis": "diñar",
    "diñasen": "diñar",
    "diñases": "diñar",
    "diñaste": "diñar",
    "diñasteis": "diñar",
    "diñe": "diñar",
    "diñemos": "diñar",
    "diñen": "diñar",
    "diñes": "diñar",
    "diño": "diñar",
    "diñábamos": "diñar",
    "diñáis": "diñar",
    "diñáramos": "diñar",
    "diñáremos": "diñar",
    "diñásemos": "diñar",
    "diñé": "diñar",
    "diñéis": "diñar",
    "diñó": "diñar",
    "diódica": "diódico",
    "diódicas": "diódico",
    "diódicos": "diódico",
    "diópsidos": "diópsido",
    "dióptrica": "dióptrico",
    "dióptricas": "dióptrico",
    "dióptricos": "dióptrico",
    "dióxidos": "dióxido",
    "dl": "decilitro",
    "dm": "decímetro",
    "dobla": "doblar",
    "doblaba": "doblar",
    "doblabais": "doblar",
    "doblaban": "doblar",
    "doblabas": "doblar",
    "doblad": "doblar",
    "doblada": "doblar",
    "dobladas": "doblar",
    "dobladilla": "dobladillar",
    "dobladillaba": "dobladillar",
    "dobladillabais": "dobladillar",
    "dobladillaban": "dobladillar",
    "dobladillabas": "dobladillar",
    "dobladillad": "dobladillar",
    "dobladillada": "dobladillar",
    "dobladilladas": "dobladillar",
    "dobladillado": "dobladillar",
    "dobladillados": "dobladillar",
    "dobladillamos": "dobladillar",
    "dobladillan": "dobladillar",
    "dobladillando": "dobladillar",
    "dobladillara": "dobladillar",
    "dobladillarais": "dobladillar",
    "dobladillaran": "dobladillar",
    "dobladillaras": "dobladillar",
    "dobladillare": "dobladillar",
    "dobladillareis": "dobladillar",
    "dobladillaremos": "dobladillar",
    "dobladillaren": "dobladillar",
    "dobladillares": "dobladillar",
    "dobladillaron": "dobladillar",
    "dobladillará": "dobladillar",
    "dobladillarán": "dobladillar",
    "dobladillarás": "dobladillar",
    "dobladillaré": "dobladillar",
    "dobladillaréis": "dobladillar",
    "dobladillaría": "dobladillar",
    "dobladillaríais": "dobladillar",
    "dobladillaríamos": "dobladillar",
    "dobladillarían": "dobladillar",
    "dobladillarías": "dobladillar",
    "dobladillas": "dobladillar",
    "dobladillase": "dobladillar",
    "dobladillaseis": "dobladillar",
    "dobladillasen": "dobladillar",
    "dobladillases": "dobladillar",
    "dobladillaste": "dobladillar",
    "dobladillasteis": "dobladillar",
    "dobladille": "dobladillar",
    "dobladillemos": "dobladillar",
    "dobladillen": "dobladillar",
    "dobladilles": "dobladillar",
    "dobladillo": "dobladillar",
    "dobladillos": "dobladillo",
    "dobladillábamos": "dobladillar",
    "dobladilláis": "dobladillar",
    "dobladilláramos": "dobladillar",
    "dobladilláremos": "dobladillar",
    "dobladillásemos": "dobladillar",
    "dobladillé": "dobladillar",
    "dobladilléis": "dobladillar",
    "dobladilló": "dobladillar",
    "doblado": "doblar",
    "doblados": "doblar",
    "dobladuras": "dobladura",
    "doblajes": "doblaje",
    "doblamientos": "doblamiento",
    "doblamos": "doblar",
    "doblan": "doblar",
    "doblando": "doblar",
    "doblara": "doblar",
    "doblarais": "doblar",
    "doblaran": "doblar",
    "doblaras": "doblar",
    "doblare": "doblar",
    "doblareis": "doblar",
    "doblaremos": "doblar",
    "doblaren": "doblar",
    "doblares": "doblar",
    "doblaron": "doblar",
    "doblará": "doblar",
    "doblarán": "doblar",
    "doblarás": "doblar",
    "doblaré": "doblar",
    "doblaréis": "doblar",
    "doblaría": "doblar",
    "doblaríais": "doblar",
    "doblaríamos": "doblar",
    "doblarían": "doblar",
    "doblarías": "doblar",
    "doblas": "doblar",
    "doblase": "doblar",
    "doblaseis": "doblar",
    "doblasen": "doblar",
    "doblases": "doblar",
    "doblaste": "doblar",
    "doblasteis": "doblar",
    "doble": "doblar",
    "dobleces": "doblez",
    "doblega": "doblegar",
    "doblegaba": "doblegar",
    "doblegabais": "doblegar",
    "doblegaban": "doblegar",
    "doblegabas": "doblegar",
    "doblegables": "doblegable",
    "doblegad": "doblegar",
    "doblegada": "doblegar",
    "doblegadas": "doblegar",
    "doblegadiza": "doblegadizo",
    "doblegadizas": "doblegadizo",
    "doblegadizos": "doblegadizo",
    "doblegado": "doblegar",
    "doblegados": "doblegar",
    "doblegamientos": "doblegamiento",
    "doblegamos": "doblegar",
    "doblegan": "doblegar",
    "doblegando": "doblegar",
    "doblegara": "doblegar",
    "doblegarais": "doblegar",
    "doblegaran": "doblegar",
    "doblegaras": "doblegar",
    "doblegare": "doblegar",
    "doblegareis": "doblegar",
    "doblegaremos": "doblegar",
    "doblegaren": "doblegar",
    "doblegares": "doblegar",
    "doblegaron": "doblegar",
    "doblegará": "doblegar",
    "doblegarán": "doblegar",
    "doblegarás": "doblegar",
    "doblegaré": "doblegar",
    "doblegaréis": "doblegar",
    "doblegaría": "doblegar",
    "doblegaríais": "doblegar",
    "doblegaríamos": "doblegar",
    "doblegarían": "doblegar",
    "doblegarías": "doblegar",
    "doblegas": "doblegar",
    "doblegase": "doblegar",
    "doblegaseis": "doblegar",
    "doblegasen": "doblegar",
    "doblegases": "doblegar",
    "doblegaste": "doblegar",
    "doblegasteis": "doblegar",
    "doblego": "doblegar",
    "doblegue": "doblegar",
    "dobleguemos": "doblegar",
    "dobleguen": "doblegar",
    "doblegues": "doblegar",
    "doblegué": "doblegar",
    "dobleguéis": "doblegar",
    "doblegábamos": "doblegar",
    "doblegáis": "doblegar",
    "doblegáramos": "doblegar",
    "doblegáremos": "doblegar",
    "doblegásemos": "doblegar",
    "doblegó": "doblegar",
    "doblemos": "doblar",
    "doblen": "doblar",
    "dobles": "doble",
    "doblescudos": "doblescudo",
    "dobletes": "doblete",
    "doblistas": "doblista",
    "doblo": "doblar",
    "doblones": "doblón",
    "doblábamos": "doblar",
    "dobláis": "doblar",
    "dobláramos": "doblar",
    "dobláremos": "doblar",
    "doblásemos": "doblar",
    "doblé": "doblar",
    "dobléis": "doblar",
    "dobló": "doblar",
    "doboks": "dobok",
    "doceava": "doceavo",
    "doceavas": "doceavo",
    "doceavo": "12",
    "doceavos": "doceavo",
    "doceañistas": "doceañista",
    "docemesina": "docemesino",
    "docemesinas": "docemesino",
    "docemesinos": "docemesino",
    "docena": "doceno",
    "docenales": "docenal",
    "docenaria": "docenario",
    "docenarias": "docenario",
    "docenarios": "docenario",
    "docenas": "doceno",
    "docencias": "docencia",
    "docenos": "doceno",
    "docentes": "docente",
    "docetas": "doceta",
    "docetismos": "docetismo",
    "docilidades": "docilidad",
    "docilita": "docilitar",
    "docilitaba": "docilitar",
    "docilitabais": "docilitar",
    "docilitaban": "docilitar",
    "docilitabas": "docilitar",
    "docilitad": "docilitar",
    "docilitada": "docilitar",
    "docilitadas": "docilitar",
    "docilitado": "docilitar",
    "docilitados": "docilitar",
    "docilitamos": "docilitar",
    "docilitan": "docilitar",
    "docilitando": "docilitar",
    "docilitara": "docilitar",
    "docilitarais": "docilitar",
    "docilitaran": "docilitar",
    "docilitaras": "docilitar",
    "docilitare": "docilitar",
    "docilitareis": "docilitar",
    "docilitaremos": "docilitar",
    "docilitaren": "docilitar",
    "docilitares": "docilitar",
    "docilitaron": "docilitar",
    "docilitará": "docilitar",
    "docilitarán": "docilitar",
    "docilitarás": "docilitar",
    "docilitaré": "docilitar",
    "docilitaréis": "docilitar",
    "docilitaría": "docilitar",
    "docilitaríais": "docilitar",
    "docilitaríamos": "docilitar",
    "docilitarían": "docilitar",
    "docilitarías": "docilitar",
    "docilitas": "docilitar",
    "docilitase": "docilitar",
    "docilitaseis": "docilitar",
    "docilitasen": "docilitar",
    "docilitases": "docilitar",
    "docilitaste": "docilitar",
    "docilitasteis": "docilitar",
    "docilite": "docilitar",
    "docilitemos": "docilitar",
    "dociliten": "docilitar",
    "docilites": "docilitar",
    "docilito": "docilitar",
    "docilitábamos": "docilitar",
    "docilitáis": "docilitar",
    "docilitáramos": "docilitar",
    "docilitáremos": "docilitar",
    "docilitásemos": "docilitar",
    "docilité": "docilitar",
    "docilitéis": "docilitar",
    "docilitó": "docilitar",
    "docimasias": "docimasia",
    "docimologías": "docimología",
    "docimástica": "docimástico",
    "docimásticas": "docimástico",
    "docimásticos": "docimástico",
    "docks": "dock",
    "docta": "docto",
    "doctas": "docto",
    "doctora": "doctorar",
    "doctoraba": "doctorar",
    "doctorabais": "doctorar",
    "doctoraban": "doctorar",
    "doctorabas": "doctorar",
    "doctorad": "doctorar",
    "doctorada": "doctorar",
    "doctoradas": "doctorar",
    "doctorado": "doctorar",
    "doctorados": "doctorar",
    "doctorales": "doctoral",
    "doctoramientos": "doctoramiento",
    "doctoramos": "doctorar",
    "doctoran": "doctorar",
    "doctoranda": "doctorando",
    "doctorandas": "doctorando",
    "doctorando": "doctorar",
    "doctorandos": "doctorando",
    "doctorara": "doctorar",
    "doctorarais": "doctorar",
    "doctoraran": "doctorar",
    "doctoraras": "doctorar",
    "doctorare": "doctorar",
    "doctorareis": "doctorar",
    "doctoraremos": "doctorar",
    "doctoraren": "doctorar",
    "doctorares": "doctorar",
    "doctoraron": "doctorar",
    "doctorará": "doctorar",
    "doctorarán": "doctorar",
    "doctorarás": "doctorar",
    "doctoraré": "doctorar",
    "doctoraréis": "doctorar",
    "doctoraría": "doctorar",
    "doctoraríais": "doctorar",
    "doctoraríamos": "doctorar",
    "doctorarían": "doctorar",
    "doctorarías": "doctorar",
    "doctoras": "doctorar",
    "doctorase": "doctorar",
    "doctoraseis": "doctorar",
    "doctorasen": "doctorar",
    "doctorases": "doctorar",
    "doctoraste": "doctorar",
    "doctorasteis": "doctorar",
    "doctore": "doctorar",
    "doctorea": "doctorear",
    "doctoreaba": "doctorear",
    "doctoreabais": "doctorear",
    "doctoreaban": "doctorear",
    "doctoreabas": "doctorear",
    "doctoread": "doctorear",
    "doctoreada": "doctorear",
    "doctoreadas": "doctorear",
    "doctoreado": "doctorear",
    "doctoreados": "doctorear",
    "doctoreamos": "doctorear",
    "doctorean": "doctorear",
    "doctoreando": "doctorear",
    "doctoreara": "doctorear",
    "doctorearais": "doctorear",
    "doctorearan": "doctorear",
    "doctorearas": "doctorear",
    "doctoreare": "doctorear",
    "doctoreareis": "doctorear",
    "doctorearemos": "doctorear",
    "doctorearen": "doctorear",
    "doctoreares": "doctorear",
    "doctorearon": "doctorear",
    "doctoreará": "doctorear",
    "doctorearán": "doctorear",
    "doctorearás": "doctorear",
    "doctorearé": "doctorear",
    "doctorearéis": "doctorear",
    "doctorearía": "doctorear",
    "doctorearíais": "doctorear",
    "doctorearíamos": "doctorear",
    "doctorearían": "doctorear",
    "doctorearías": "doctorear",
    "doctoreas": "doctorear",
    "doctorease": "doctorear",
    "doctoreaseis": "doctorear",
    "doctoreasen": "doctorear",
    "doctoreases": "doctorear",
    "doctoreaste": "doctorear",
    "doctoreasteis": "doctorear",
    "doctoree": "doctorear",
    "doctoreemos": "doctorear",
    "doctoreen": "doctorear",
    "doctorees": "doctorear",
    "doctoremos": "doctorar",
    "doctoren": "doctorar",
    "doctoreo": "doctorear",
    "doctores": "doctorar",
    "doctoreábamos": "doctorear",
    "doctoreáis": "doctorear",
    "doctoreáramos": "doctorear",
    "doctoreáremos": "doctorear",
    "doctoreásemos": "doctorear",
    "doctoreé": "doctorear",
    "doctoreéis": "doctorear",
    "doctoreó": "doctorear",
    "doctoro": "doctorar",
    "doctorábamos": "doctorar",
    "doctoráis": "doctorar",
    "doctoráramos": "doctorar",
    "doctoráremos": "doctorar",
    "doctorásemos": "doctorar",
    "doctoré": "doctorar",
    "doctoréis": "doctorar",
    "doctoró": "doctorar",
    "doctos": "docto",
    "doctrina": "doctrinar",
    "doctrinaba": "doctrinar",
    "doctrinabais": "doctrinar",
    "doctrinaban": "doctrinar",
    "doctrinabas": "doctrinar",
    "doctrinad": "doctrinar",
    "doctrinada": "doctrinar",
    "doctrinadas": "doctrinar",
    "doctrinado": "doctrinar",
    "doctrinados": "doctrinar",
    "doctrinales": "doctrinal",
    "doctrinamos": "doctrinar",
    "doctrinan": "doctrinar",
    "doctrinando": "doctrinar",
    "doctrinara": "doctrinar",
    "doctrinarais": "doctrinar",
    "doctrinaran": "doctrinar",
    "doctrinaras": "doctrinar",
    "doctrinare": "doctrinar",
    "doctrinareis": "doctrinar",
    "doctrinaremos": "doctrinar",
    "doctrinaren": "doctrinar",
    "doctrinares": "doctrinar",
    "doctrinaria": "doctrinario",
    "doctrinarias": "doctrinario",
    "doctrinarios": "doctrinario",
    "doctrinarismos": "doctrinarismo",
    "doctrinaron": "doctrinar",
    "doctrinará": "doctrinar",
    "doctrinarán": "doctrinar",
    "doctrinarás": "doctrinar",
    "doctrinaré": "doctrinar",
    "doctrinaréis": "doctrinar",
    "doctrinaría": "doctrinar",
    "doctrinaríais": "doctrinar",
    "doctrinaríamos": "doctrinar",
    "doctrinarían": "doctrinar",
    "doctrinarías": "doctrinar",
    "doctrinas": "doctrinar",
    "doctrinase": "doctrinar",
    "doctrinaseis": "doctrinar",
    "doctrinasen": "doctrinar",
    "doctrinases": "doctrinar",
    "doctrinaste": "doctrinar",
    "doctrinasteis": "doctrinar",
    "doctrine": "doctrinar",
    "doctrinemos": "doctrinar",
    "doctrinen": "doctrinar",
    "doctrineros": "doctrinero",
    "doctrines": "doctrinar",
    "doctrino": "doctrinar",
    "doctrinos": "doctrino",
    "doctrinábamos": "doctrinar",
    "doctrináis": "doctrinar",
    "doctrináramos": "doctrinar",
    "doctrináremos": "doctrinar",
    "doctrinásemos": "doctrinar",
    "doctriné": "doctrinar",
    "doctrinéis": "doctrinar",
    "doctrinó": "doctrinar",
    "docudramas": "docudrama",
    "documenta": "documentar",
    "documentaba": "documentar",
    "documentabais": "documentar",
    "documentaban": "documentar",
    "documentabas": "documentar",
    "documentaciones": "documentación",
    "documentad": "documentar",
    "documentada": "documentar",
    "documentadas": "documentar",
    "documentado": "documentar",
    "documentados": "documentar",
    "documentales": "documental",
    "documentamos": "documentar",
    "documentan": "documentar",
    "documentando": "documentar",
    "documentara": "documentar",
    "documentarais": "documentar",
    "documentaran": "documentar",
    "documentaras": "documentar",
    "documentare": "documentar",
    "documentareis": "documentar",
    "documentaremos": "documentar",
    "documentaren": "documentar",
    "documentares": "documentar",
    "documentaria": "documentario",
    "documentarias": "documentario",
    "documentarios": "documentario",
    "documentaron": "documentar",
    "documentará": "documentar",
    "documentarán": "documentar",
    "documentarás": "documentar",
    "documentaré": "documentar",
    "documentaréis": "documentar",
    "documentaría": "documentar",
    "documentaríais": "documentar",
    "documentaríamos": "documentar",
    "documentarían": "documentar",
    "documentarías": "documentar",
    "documentas": "documentar",
    "documentase": "documentar",
    "documentaseis": "documentar",
    "documentasen": "documentar",
    "documentases": "documentar",
    "documentaste": "documentar",
    "documentasteis": "documentar",
    "documente": "documentar",
    "documentemos": "documentar",
    "documenten": "documentar",
    "documentes": "documentar",
    "documento": "documentar",
    "documentos": "documento",
    "documentoscopias": "documentoscopia",
    "documentábamos": "documentar",
    "documentáis": "documentar",
    "documentáramos": "documentar",
    "documentáremos": "documentar",
    "documentásemos": "documentar",
    "documenté": "documentar",
    "documentéis": "documentar",
    "documentó": "documentar",
    "docética": "docético",
    "docéticas": "docético",
    "docéticos": "docético",
    "dodecaedros": "dodecaedro",
    "dodecafonismos": "dodecafonismo",
    "dodecafonías": "dodecafonía",
    "dodecasílabos": "dodecasílabo",
    "dodecenilsuccínica": "dodecenilsuccínico",
    "dodecenilsuccínicas": "dodecenilsuccínico",
    "dodecenilsuccínicos": "dodecenilsuccínico",
    "dodecenos": "dodeceno",
    "dodecilbencenos": "dodecilbenceno",
    "dodeciles": "dodecil",
    "dodecilfenoles": "dodecilfenol",
    "dodecágonos": "dodecágono",
    "dodecástila": "dodecástilo",
    "dodecástilas": "dodecástilo",
    "dodecástilos": "dodecástilo",
    "dogales": "dogal",
    "dogaresas": "dogaresa",
    "doggeres": "dogger",
    "dogmas": "dogma",
    "dogmatice": "dogmatizar",
    "dogmaticemos": "dogmatizar",
    "dogmaticen": "dogmatizar",
    "dogmatices": "dogmatizar",
    "dogmaticé": "dogmatizar",
    "dogmaticéis": "dogmatizar",
    "dogmatismos": "dogmatismo",
    "dogmatistas": "dogmatista",
    "dogmatiza": "dogmatizar",
    "dogmatizaba": "dogmatizar",
    "dogmatizabais": "dogmatizar",
    "dogmatizaban": "dogmatizar",
    "dogmatizabas": "dogmatizar",
    "dogmatizad": "dogmatizar",
    "dogmatizada": "dogmatizar",
    "dogmatizadas": "dogmatizar",
    "dogmatizado": "dogmatizar",
    "dogmatizadora": "dogmatizador",
    "dogmatizadoras": "dogmatizador",
    "dogmatizadores": "dogmatizador",
    "dogmatizados": "dogmatizar",
    "dogmatizamos": "dogmatizar",
    "dogmatizan": "dogmatizar",
    "dogmatizando": "dogmatizar",
    "dogmatizara": "dogmatizar",
    "dogmatizarais": "dogmatizar",
    "dogmatizaran": "dogmatizar",
    "dogmatizaras": "dogmatizar",
    "dogmatizare": "dogmatizar",
    "dogmatizareis": "dogmatizar",
    "dogmatizaremos": "dogmatizar",
    "dogmatizaren": "dogmatizar",
    "dogmatizares": "dogmatizar",
    "dogmatizaron": "dogmatizar",
    "dogmatizará": "dogmatizar",
    "dogmatizarán": "dogmatizar",
    "dogmatizarás": "dogmatizar",
    "dogmatizaré": "dogmatizar",
    "dogmatizaréis": "dogmatizar",
    "dogmatizaría": "dogmatizar",
    "dogmatizaríais": "dogmatizar",
    "dogmatizaríamos": "dogmatizar",
    "dogmatizarían": "dogmatizar",
    "dogmatizarías": "dogmatizar",
    "dogmatizas": "dogmatizar",
    "dogmatizase": "dogmatizar",
    "dogmatizaseis": "dogmatizar",
    "dogmatizasen": "dogmatizar",
    "dogmatizases": "dogmatizar",
    "dogmatizaste": "dogmatizar",
    "dogmatizasteis": "dogmatizar",
    "dogmatizo": "dogmatizar",
    "dogmatizábamos": "dogmatizar",
    "dogmatizáis": "dogmatizar",
    "dogmatizáramos": "dogmatizar",
    "dogmatizáremos": "dogmatizar",
    "dogmatizásemos": "dogmatizar",
    "dogmatizó": "dogmatizar",
    "dogmática": "dogmático",
    "dogmáticas": "dogmático",
    "dogmáticos": "dogmático",
    "dogos": "dogo",
    "doguillas": "doguilla",
    "doguillos": "doguillo",
    "dojos": "dojo",
    "dolaba": "dolar",
    "dolabais": "dolar",
    "dolaban": "dolar",
    "dolabas": "dolar",
    "dolad": "dolar",
    "dolada": "dolar",
    "doladas": "dolar",
    "dolado": "dolar",
    "doladores": "dolador",
    "dolados": "dolar",
    "doladuras": "doladura",
    "dolajes": "dolaje",
    "dolamas": "dolama",
    "dolames": "dolame",
    "dolamos": "doler",
    "dolando": "dolar",
    "dolara": "dolar",
    "dolarais": "dolar",
    "dolaran": "dolar",
    "dolaras": "dolar",
    "dolare": "dolar",
    "dolareis": "dolar",
    "dolaremos": "dolar",
    "dolaren": "dolar",
    "dolares": "dolar",
    "dolarizaciones": "dolarización",
    "dolaron": "dolar",
    "dolará": "dolar",
    "dolarán": "dolar",
    "dolarás": "dolar",
    "dolaré": "dolar",
    "dolaréis": "dolar",
    "dolaría": "dolar",
    "dolaríais": "dolar",
    "dolaríamos": "dolar",
    "dolarían": "dolar",
    "dolarías": "dolar",
    "dolase": "dolar",
    "dolaseis": "dolar",
    "dolasen": "dolar",
    "dolases": "dolar",
    "dolaste": "dolar",
    "dolasteis": "dolar",
    "dolbys": "dolby",
    "doled": "doler",
    "dolemos": "doler",
    "dolencias": "dolencia",
    "doleremos": "doler",
    "doleritas": "dolerita",
    "dolerá": "doler",
    "dolerán": "doler",
    "dolerás": "doler",
    "doleré": "doler",
    "doleréis": "doler",
    "dolería": "doler",
    "doleríais": "doler",
    "doleríamos": "doler",
    "dolerían": "doler",
    "dolerías": "doler",
    "dolicocefalias": "dolicocefalia",
    "dolicocéfala": "dolicocéfalo",
    "dolicocéfalas": "dolicocéfalo",
    "dolicocéfalos": "dolicocéfalo",
    "dolicouránica": "dolicouránico",
    "dolicouránicas": "dolicouránico",
    "dolicouránicos": "dolicouránico",
    "dolida": "doler",
    "dolidas": "doler",
    "dolido": "doler",
    "dolidos": "doler",
    "doliendo": "doler",
    "dolientes": "doliente",
    "doliera": "doler",
    "dolierais": "doler",
    "dolieran": "doler",
    "dolieras": "doler",
    "doliere": "doler",
    "doliereis": "doler",
    "dolieren": "doler",
    "dolieres": "doler",
    "dolieron": "doler",
    "doliese": "doler",
    "dolieseis": "doler",
    "doliesen": "doler",
    "dolieses": "doler",
    "dolimos": "doler",
    "dolinas": "dolina",
    "doliste": "doler",
    "dolisteis": "doler",
    "doliéramos": "doler",
    "doliéremos": "doler",
    "doliésemos": "doler",
    "dolió": "doler",
    "dolmanes": "dolmán",
    "dolménica": "dolménico",
    "dolménicas": "dolménico",
    "dolménicos": "dolménico",
    "dolobres": "dolobre",
    "dolomitas": "dolomita",
    "dolomitizaciones": "dolomitización",
    "dolomías": "dolomía",
    "dolomítica": "dolomítico",
    "dolomíticas": "dolomítico",
    "dolomíticos": "dolomítico",
    "doloras": "dolora",
    "dolores": "dolor",
    "dolorida": "dolorido",
    "doloridas": "dolorido",
    "doloridos": "dolorido",
    "dolorosa": "doloroso",
    "dolorosas": "doloroso",
    "dolorosos": "doloroso",
    "dolos": "dolo",
    "dolosa": "doloso",
    "dolosas": "doloso",
    "dolosos": "doloso",
    "dolábamos": "dolar",
    "doláis": "doler",
    "doláramos": "dolar",
    "doláremos": "dolar",
    "dolásemos": "dolar",
    "dolé": "dolar",
    "doléis": "doler",
    "dolí": "doler",
    "dolía": "doler",
    "dolíais": "doler",
    "dolíamos": "doler",
    "dolían": "doler",
    "dolías": "doler",
    "doló": "dolar",
    "doma": "domar",
    "domaba": "domar",
    "domabais": "domar",
    "domaban": "domar",
    "domabas": "domar",
    "domables": "domable",
    "domad": "domar",
    "domada": "domar",
    "domadas": "domar",
    "domado": "domar",
    "domadora": "domador",
    "domadoras": "domador",
    "domadores": "domador",
    "domados": "domar",
    "domaduras": "domadura",
    "domamos": "domar",
    "doman": "domar",
    "domando": "domar",
    "domaniales": "domanial",
    "domara": "domar",
    "domarais": "domar",
    "domaran": "domar",
    "domaras": "domar",
    "domare": "domar",
    "domareis": "domar",
    "domaremos": "domar",
    "domaren": "domar",
    "domares": "domar",
    "domaron": "domar",
    "domará": "domar",
    "domarán": "domar",
    "domarás": "domar",
    "domaré": "domar",
    "domaréis": "domar",
    "domaría": "domar",
    "domaríais": "domar",
    "domaríamos": "domar",
    "domarían": "domar",
    "domarías": "domar",
    "domas": "domar",
    "domase": "domar",
    "domaseis": "domar",
    "domasen": "domar",
    "domases": "domar",
    "domaste": "domar",
    "domasteis": "domar",
    "dome": "domar",
    "domemos": "domar",
    "domen": "domar",
    "domes": "domar",
    "domestica": "domesticar",
    "domesticaba": "domesticar",
    "domesticabais": "domesticar",
    "domesticaban": "domesticar",
    "domesticabas": "domesticar",
    "domesticables": "domesticable",
    "domesticaciones": "domesticación",
    "domesticad": "domesticar",
    "domesticada": "domesticar",
    "domesticadas": "domesticar",
    "domesticado": "domesticar",
    "domesticados": "domesticar",
    "domesticamos": "domesticar",
    "domestican": "domesticar",
    "domesticando": "domesticar",
    "domesticara": "domesticar",
    "domesticarais": "domesticar",
    "domesticaran": "domesticar",
    "domesticaras": "domesticar",
    "domesticare": "domesticar",
    "domesticareis": "domesticar",
    "domesticaremos": "domesticar",
    "domesticaren": "domesticar",
    "domesticares": "domesticar",
    "domesticaron": "domesticar",
    "domesticará": "domesticar",
    "domesticarán": "domesticar",
    "domesticarás": "domesticar",
    "domesticaré": "domesticar",
    "domesticaréis": "domesticar",
    "domesticaría": "domesticar",
    "domesticaríais": "domesticar",
    "domesticaríamos": "domesticar",
    "domesticarían": "domesticar",
    "domesticarías": "domesticar",
    "domesticas": "domesticar",
    "domesticase": "domesticar",
    "domesticaseis": "domesticar",
    "domesticasen": "domesticar",
    "domesticases": "domesticar",
    "domesticaste": "domesticar",
    "domesticasteis": "domesticar",
    "domesticidades": "domesticidad",
    "domestico": "domesticar",
    "domesticábamos": "domesticar",
    "domesticáis": "domesticar",
    "domesticáramos": "domesticar",
    "domesticáremos": "domesticar",
    "domesticásemos": "domesticar",
    "domesticó": "domesticar",
    "domestique": "domesticar",
    "domestiquemos": "domesticar",
    "domestiquen": "domesticar",
    "domestiques": "domesticar",
    "domestiqué": "domesticar",
    "domestiquéis": "domesticar",
    "domeña": "domeñar",
    "domeñaba": "domeñar",
    "domeñabais": "domeñar",
    "domeñaban": "domeñar",
    "domeñabas": "domeñar",
    "domeñables": "domeñable",
    "domeñad": "domeñar",
    "domeñada": "domeñar",
    "domeñadas": "domeñar",
    "domeñado": "domeñar",
    "domeñados": "domeñar",
    "domeñamos": "domeñar",
    "domeñan": "domeñar",
    "domeñando": "domeñar",
    "domeñara": "domeñar",
    "domeñarais": "domeñar",
    "domeñaran": "domeñar",
    "domeñaras": "domeñar",
    "domeñare": "domeñar",
    "domeñareis": "domeñar",
    "domeñaremos": "domeñar",
    "domeñaren": "domeñar",
    "domeñares": "domeñar",
    "domeñaron": "domeñar",
    "domeñará": "domeñar",
    "domeñarán": "domeñar",
    "domeñarás": "domeñar",
    "domeñaré": "domeñar",
    "domeñaréis": "domeñar",
    "domeñaría": "domeñar",
    "domeñaríais": "domeñar",
    "domeñaríamos": "domeñar",
    "domeñarían": "domeñar",
    "domeñarías": "domeñar",
    "domeñas": "domeñar",
    "domeñase": "domeñar",
    "domeñaseis": "domeñar",
    "domeñasen": "domeñar",
    "domeñases": "domeñar",
    "domeñaste": "domeñar",
    "domeñasteis": "domeñar",
    "domeñe": "domeñar",
    "domeñemos": "domeñar",
    "domeñen": "domeñar",
    "domeñes": "domeñar",
    "domeño": "domeñar",
    "domeñábamos": "domeñar",
    "domeñáis": "domeñar",
    "domeñáramos": "domeñar",
    "domeñáremos": "domeñar",
    "domeñásemos": "domeñar",
    "domeñé": "domeñar",
    "domeñéis": "domeñar",
    "domeñó": "domeñar",
    "domicilia": "domiciliar",
    "domiciliaba": "domiciliar",
    "domiciliabais": "domiciliar",
    "domiciliaban": "domiciliar",
    "domiciliabas": "domiciliar",
    "domiciliaciones": "domiciliación",
    "domiciliad": "domiciliar",
    "domiciliada": "domiciliar",
    "domiciliadas": "domiciliar",
    "domiciliado": "domiciliar",
    "domiciliados": "domiciliar",
    "domiciliamos": "domiciliar",
    "domicilian": "domiciliar",
    "domiciliando": "domiciliar",
    "domiciliara": "domiciliar",
    "domiciliarais": "domiciliar",
    "domiciliaran": "domiciliar",
    "domiciliaras": "domiciliar",
    "domiciliare": "domiciliar",
    "domiciliareis": "domiciliar",
    "domiciliaremos": "domiciliar",
    "domiciliaren": "domiciliar",
    "domiciliares": "domiciliar",
    "domiciliaria": "domiciliario",
    "domiciliarias": "domiciliario",
    "domiciliarios": "domiciliario",
    "domiciliaron": "domiciliar",
    "domiciliará": "domiciliar",
    "domiciliarán": "domiciliar",
    "domiciliarás": "domiciliar",
    "domiciliaré": "domiciliar",
    "domiciliaréis": "domiciliar",
    "domiciliaría": "domiciliar",
    "domiciliaríais": "domiciliar",
    "domiciliaríamos": "domiciliar",
    "domiciliarían": "domiciliar",
    "domiciliarías": "domiciliar",
    "domicilias": "domiciliar",
    "domiciliase": "domiciliar",
    "domiciliaseis": "domiciliar",
    "domiciliasen": "domiciliar",
    "domiciliases": "domiciliar",
    "domiciliaste": "domiciliar",
    "domiciliasteis": "domiciliar",
    "domicilie": "domiciliar",
    "domiciliemos": "domiciliar",
    "domicilien": "domiciliar",
    "domicilies": "domiciliar",
    "domicilio": "domiciliar",
    "domicilios": "domicilio",
    "domiciliábamos": "domiciliar",
    "domiciliáis": "domiciliar",
    "domiciliáramos": "domiciliar",
    "domiciliáremos": "domiciliar",
    "domiciliásemos": "domiciliar",
    "domicilié": "domiciliar",
    "domiciliéis": "domiciliar",
    "domicilió": "domiciliar",
    "domina": "dominar",
    "dominaba": "dominar",
    "dominabais": "dominar",
    "dominaban": "dominar",
    "dominabas": "dominar",
    "dominaciones": "dominación",
    "dominad": "dominar",
    "dominada": "dominar",
    "dominadas": "dominar",
    "dominado": "dominar",
    "dominadora": "dominador",
    "dominadoras": "dominador",
    "dominadores": "dominador",
    "dominados": "dominar",
    "dominamos": "dominar",
    "dominan": "dominar",
    "dominancias": "dominancia",
    "dominando": "dominar",
    "dominantes": "dominante",
    "dominara": "dominar",
    "dominarais": "dominar",
    "dominaran": "dominar",
    "dominaras": "dominar",
    "dominare": "dominar",
    "dominareis": "dominar",
    "dominaremos": "dominar",
    "dominaren": "dominar",
    "dominares": "dominar",
    "dominaron": "dominar",
    "dominará": "dominar",
    "dominarán": "dominar",
    "dominarás": "dominar",
    "dominaré": "dominar",
    "dominaréis": "dominar",
    "dominaría": "dominar",
    "dominaríais": "dominar",
    "dominaríamos": "dominar",
    "dominarían": "dominar",
    "dominarías": "dominar",
    "dominas": "dominar",
    "dominase": "dominar",
    "dominaseis": "dominar",
    "dominasen": "dominar",
    "dominases": "dominar",
    "dominaste": "dominar",
    "dominasteis": "dominar",
    "dominativa": "dominativo",
    "dominativas": "dominativo",
    "dominativos": "dominativo",
    "domine": "dominar",
    "dominemos": "dominar",
    "dominen": "dominar",
    "domines": "dominar",
    "domingadas": "domingada",
    "domingos": "domingo",
    "dominguejos": "dominguejo",
    "dominguera": "dominguero",
    "domingueras": "dominguero",
    "domingueros": "dominguero",
    "dominguillos": "dominguillo",
    "dominica": "dominico",
    "dominicales": "dominical",
    "dominicana": "dominicano",
    "dominicanas": "dominicano",
    "dominicanos": "dominicano",
    "dominicas": "dominico",
    "dominicos": "dominico",
    "dominios": "dominio",
    "domino": "dominar",
    "dominábamos": "dominar",
    "domináis": "dominar",
    "domináramos": "dominar",
    "domináremos": "dominar",
    "dominásemos": "dominar",
    "dominé": "dominar",
    "dominéis": "dominar",
    "dominó": "dominar",
    "dominós": "dominó",
    "domo": "domar",
    "domos": "domo",
    "dompedros": "dompedro",
    "doms": "dom",
    "domábamos": "domar",
    "domáis": "domar",
    "domáramos": "domar",
    "domáremos": "domar",
    "domásemos": "domar",
    "domé": "domar",
    "doméis": "domar",
    "doméstica": "doméstico",
    "domésticas": "doméstico",
    "domésticos": "doméstico",
    "domínicas": "domínica",
    "domó": "domar",
    "domótica": "domótico",
    "domóticas": "domótico",
    "domóticos": "domótico",
    "dona": "donar",
    "donaba": "donar",
    "donabais": "donar",
    "donaban": "donar",
    "donabas": "donar",
    "donaciones": "donación",
    "donad": "donar",
    "donada": "donar",
    "donadas": "donar",
    "donado": "donar",
    "donadora": "donador",
    "donadoras": "donador",
    "donadores": "donador",
    "donados": "donar",
    "donaires": "donaire",
    "donairosa": "donairoso",
    "donairosas": "donairoso",
    "donairosos": "donairoso",
    "donamos": "donar",
    "donan": "donar",
    "donando": "donar",
    "donantes": "donante",
    "donara": "donar",
    "donarais": "donar",
    "donaran": "donar",
    "donaras": "donar",
    "donare": "donar",
    "donareis": "donar",
    "donaremos": "donar",
    "donaren": "donar",
    "donares": "donar",
    "donaron": "donar",
    "donará": "donar",
    "donarán": "donar",
    "donarás": "donar",
    "donaré": "donar",
    "donaréis": "donar",
    "donaría": "donar",
    "donaríais": "donar",
    "donaríamos": "donar",
    "donarían": "donar",
    "donarías": "donar",
    "donas": "donar",
    "donase": "donar",
    "donaseis": "donar",
    "donasen": "donar",
    "donases": "donar",
    "donaste": "donar",
    "donasteis": "donar",
    "donatarios": "donatario",
    "donatismos": "donatismo",
    "donatistas": "donatista",
    "donativos": "donativo",
    "donceles": "doncel",
    "doncellas": "doncella",
    "doncelleces": "doncellez",
    "doncellerías": "doncellería",
    "dondiegos": "dondiego",
    "done": "donar",
    "donemos": "donar",
    "donen": "donar",
    "dones": "donar",
    "donges": "dong",
    "dongs": "dong",
    "donilleros": "donillero",
    "donjuanes": "donjuán",
    "donjuanesca": "donjuanesco",
    "donjuanescas": "donjuanesco",
    "donjuanescos": "donjuanesco",
    "donjuanismos": "donjuanismo",
    "dono": "donar",
    "donosa": "donoso",
    "donosas": "donoso",
    "donosidades": "donosidad",
    "donosos": "donoso",
    "donostiarras": "donostiarra",
    "donosuras": "donosura",
    "donuts": "donut",
    "donábamos": "donar",
    "donáis": "donar",
    "donáramos": "donar",
    "donáremos": "donar",
    "donásemos": "donar",
    "doné": "donar",
    "donéis": "donar",
    "donó": "donar",
    "dopa": "dopar",
    "dopaba": "dopar",
    "dopabais": "dopar",
    "dopaban": "dopar",
    "dopabas": "dopar",
    "dopad": "dopar",
    "dopada": "dopar",
    "dopadas": "dopar",
    "dopado": "dopar",
    "dopados": "dopar",
    "dopajes": "dopaje",
    "dopamos": "dopar",
    "dopan": "dopar",
    "dopando": "dopar",
    "dopara": "dopar",
    "doparais": "dopar",
    "doparan": "dopar",
    "doparas": "dopar",
    "dopare": "dopar",
    "dopareis": "dopar",
    "doparemos": "dopar",
    "doparen": "dopar",
    "dopares": "dopar",
    "doparon": "dopar",
    "dopará": "dopar",
    "doparán": "dopar",
    "doparás": "dopar",
    "doparé": "dopar",
    "doparéis": "dopar",
    "doparía": "dopar",
    "doparíais": "dopar",
    "doparíamos": "dopar",
    "doparían": "dopar",
    "doparías": "dopar",
    "dopas": "dopar",
    "dopase": "dopar",
    "dopaseis": "dopar",
    "dopasen": "dopar",
    "dopases": "dopar",
    "dopaste": "dopar",
    "dopasteis": "dopar",
    "dope": "dopar",
    "dopemos": "dopar",
    "dopen": "dopar",
    "dopes": "dopar",
    "dopo": "dopar",
    "dopábamos": "dopar",
    "dopáis": "dopar",
    "dopáramos": "dopar",
    "dopáremos": "dopar",
    "dopásemos": "dopar",
    "dopé": "dopar",
    "dopéis": "dopar",
    "dopó": "dopar",
    "doquiera": "doquier",
    "dora": "dorar",
    "doraba": "dorar",
    "dorabais": "dorar",
    "doraban": "dorar",
    "dorabas": "dorar",
    "dorad": "dorar",
    "dorada": "dorar",
    "doradas": "dorar",
    "doradillas": "doradilla",
    "doradillos": "doradillo",
    "dorado": "dorar",
    "doradora": "dorador",
    "doradoras": "dorador",
    "doradores": "dorador",
    "dorados": "dorar",
    "doraduras": "doradura",
    "doramos": "dorar",
    "doran": "dorar",
    "dorando": "dorar",
    "dorara": "dorar",
    "dorarais": "dorar",
    "doraran": "dorar",
    "doraras": "dorar",
    "dorare": "dorar",
    "dorareis": "dorar",
    "doraremos": "dorar",
    "doraren": "dorar",
    "dorares": "dorar",
    "doraron": "dorar",
    "dorará": "dorar",
    "dorarán": "dorar",
    "dorarás": "dorar",
    "doraré": "dorar",
    "doraréis": "dorar",
    "doraría": "dorar",
    "doraríais": "dorar",
    "doraríamos": "dorar",
    "dorarían": "dorar",
    "dorarías": "dorar",
    "doras": "dorar",
    "dorase": "dorar",
    "doraseis": "dorar",
    "dorasen": "dorar",
    "dorases": "dorar",
    "doraste": "dorar",
    "dorasteis": "dorar",
    "dore": "dorar",
    "doremos": "dorar",
    "doren": "dorar",
    "dores": "dorar",
    "doria": "dorio",
    "dorias": "dorio",
    "dorios": "dorio",
    "dormanes": "dormán",
    "dormiciones": "dormición",
    "dormid": "dormir",
    "dormida": "dormir",
    "dormidas": "dormir",
    "dormidera": "dormidero",
    "dormideras": "dormidero",
    "dormideros": "dormidero",
    "dormido": "dormir",
    "dormidos": "dormir",
    "dormilona": "dormilón",
    "dormilonas": "dormilón",
    "dormilones": "dormilón",
    "dormimos": "dormir",
    "dormiremos": "dormir",
    "dormirá": "dormir",
    "dormirán": "dormir",
    "dormirás": "dormir",
    "dormiré": "dormir",
    "dormiréis": "dormir",
    "dormiría": "dormir",
    "dormiríais": "dormir",
    "dormiríamos": "dormir",
    "dormirían": "dormir",
    "dormirías": "dormir",
    "dormiste": "dormir",
    "dormisteis": "dormir",
    "dormita": "dormitar",
    "dormitaba": "dormitar",
    "dormitabais": "dormitar",
    "dormitaban": "dormitar",
    "dormitabas": "dormitar",
    "dormitad": "dormitar",
    "dormitada": "dormitar",
    "dormitadas": "dormitar",
    "dormitado": "dormitar",
    "dormitados": "dormitar",
    "dormitamos": "dormitar",
    "dormitan": "dormitar",
    "dormitando": "dormitar",
    "dormitara": "dormitar",
    "dormitarais": "dormitar",
    "dormitaran": "dormitar",
    "dormitaras": "dormitar",
    "dormitare": "dormitar",
    "dormitareis": "dormitar",
    "dormitaremos": "dormitar",
    "dormitaren": "dormitar",
    "dormitares": "dormitar",
    "dormitaron": "dormitar",
    "dormitará": "dormitar",
    "dormitarán": "dormitar",
    "dormitarás": "dormitar",
    "dormitaré": "dormitar",
    "dormitaréis": "dormitar",
    "dormitaría": "dormitar",
    "dormitaríais": "dormitar",
    "dormitaríamos": "dormitar",
    "dormitarían": "dormitar",
    "dormitarías": "dormitar",
    "dormitas": "dormitar",
    "dormitase": "dormitar",
    "dormitaseis": "dormitar",
    "dormitasen": "dormitar",
    "dormitases": "dormitar",
    "dormitaste": "dormitar",
    "dormitasteis": "dormitar",
    "dormite": "dormitar",
    "dormitemos": "dormitar",
    "dormiten": "dormitar",
    "dormites": "dormitar",
    "dormito": "dormitar",
    "dormitorios": "dormitorio",
    "dormitábamos": "dormitar",
    "dormitáis": "dormitar",
    "dormitáramos": "dormitar",
    "dormitáremos": "dormitar",
    "dormitásemos": "dormitar",
    "dormité": "dormitar",
    "dormitéis": "dormitar",
    "dormitó": "dormitar",
    "dormí": "dormir",
    "dormía": "dormir",
    "dormíais": "dormir",
    "dormíamos": "dormir",
    "dormían": "dormir",
    "dormías": "dormir",
    "dormís": "dormir",
    "dornajos": "dornajo",
    "dornillos": "dornillo",
    "doro": "dorar",
    "dorsales": "dorsal",
    "dorsolaterales": "dorsolateral",
    "dorsos": "dorso",
    "dorábamos": "dorar",
    "doráis": "dorar",
    "doráramos": "dorar",
    "doráremos": "dorar",
    "dorásemos": "dorar",
    "doré": "dorar",
    "doréis": "dorar",
    "doríforas": "dorífora",
    "doró": "dorar",
    "dos": "do",
    "dosalba": "dosalbo",
    "dosalbas": "dosalbo",
    "dosalbos": "dosalbo",
    "dosañales": "dosañal",
    "doscientosava": "doscientosavo",
    "doscientosavas": "doscientosavo",
    "doscientosavo": "200",
    "doscientosavos": "doscientosavo",
    "doseleras": "doselera",
    "doseles": "dosel",
    "doseletes": "doselete",
    "doses": "dos",
    "dosifica": "dosificar",
    "dosificaba": "dosificar",
    "dosificabais": "dosificar",
    "dosificaban": "dosificar",
    "dosificabas": "dosificar",
    "dosificables": "dosificable",
    "dosificaciones": "dosificación",
    "dosificad": "dosificar",
    "dosificada": "dosificar",
    "dosificadas": "dosificar",
    "dosificado": "dosificar",
    "dosificadora": "dosificador",
    "dosificadoras": "dosificador",
    "dosificadores": "dosificador",
    "dosificados": "dosificar",
    "dosificamos": "dosificar",
    "dosifican": "dosificar",
    "dosificando": "dosificar",
    "dosificara": "dosificar",
    "dosificarais": "dosificar",
    "dosificaran": "dosificar",
    "dosificaras": "dosificar",
    "dosificare": "dosificar",
    "dosificareis": "dosificar",
    "dosificaremos": "dosificar",
    "dosificaren": "dosificar",
    "dosificares": "dosificar",
    "dosificaron": "dosificar",
    "dosificará": "dosificar",
    "dosificarán": "dosificar",
    "dosificarás": "dosificar",
    "dosificaré": "dosificar",
    "dosificaréis": "dosificar",
    "dosificaría": "dosificar",
    "dosificaríais": "dosificar",
    "dosificaríamos": "dosificar",
    "dosificarían": "dosificar",
    "dosificarías": "dosificar",
    "dosificas": "dosificar",
    "dosificase": "dosificar",
    "dosificaseis": "dosificar",
    "dosificasen": "dosificar",
    "dosificases": "dosificar",
    "dosificaste": "dosificar",
    "dosificasteis": "dosificar",
    "dosifico": "dosificar",
    "dosificábamos": "dosificar",
    "dosificáis": "dosificar",
    "dosificáramos": "dosificar",
    "dosificáremos": "dosificar",
    "dosificásemos": "dosificar",
    "dosificó": "dosificar",
    "dosifique": "dosificar",
    "dosifiquemos": "dosificar",
    "dosifiquen": "dosificar",
    "dosifiques": "dosificar",
    "dosifiqué": "dosificar",
    "dosifiquéis": "dosificar",
    "dosillos": "dosillo",
    "dosimetrías": "dosimetría",
    "dosimétrica": "dosimétrico",
    "dosimétricas": "dosimétrico",
    "dosimétricos": "dosimétrico",
    "dossieres": "dossier",
    "dossiers": "dossier",
    "dosímetros": "dosímetro",
    "dota": "dotar",
    "dotaba": "dotar",
    "dotabais": "dotar",
    "dotaban": "dotar",
    "dotabas": "dotar",
    "dotacionales": "dotacional",
    "dotaciones": "dotación",
    "dotad": "dotar",
    "dotada": "dotar",
    "dotadas": "dotar",
    "dotado": "dotar",
    "dotadora": "dotador",
    "dotadoras": "dotador",
    "dotadores": "dotador",
    "dotados": "dotar",
    "dotales": "dotal",
    "dotamos": "dotar",
    "dotan": "dotar",
    "dotando": "dotar",
    "dotara": "dotar",
    "dotarais": "dotar",
    "dotaran": "dotar",
    "dotaras": "dotar",
    "dotare": "dotar",
    "dotareis": "dotar",
    "dotaremos": "dotar",
    "dotaren": "dotar",
    "dotares": "dotar",
    "dotaron": "dotar",
    "dotará": "dotar",
    "dotarán": "dotar",
    "dotarás": "dotar",
    "dotaré": "dotar",
    "dotaréis": "dotar",
    "dotaría": "dotar",
    "dotaríais": "dotar",
    "dotaríamos": "dotar",
    "dotarían": "dotar",
    "dotarías": "dotar",
    "dotas": "dotar",
    "dotase": "dotar",
    "dotaseis": "dotar",
    "dotasen": "dotar",
    "dotases": "dotar",
    "dotaste": "dotar",
    "dotasteis": "dotar",
    "dote": "dotar",
    "dotemos": "dotar",
    "doten": "dotar",
    "dotes": "dote",
    "doto": "dotar",
    "dotábamos": "dotar",
    "dotáis": "dotar",
    "dotáramos": "dotar",
    "dotáremos": "dotar",
    "dotásemos": "dotar",
    "doté": "dotar",
    "dotéis": "dotar",
    "dotó": "dotar",
    "dovela": "dovelar",
    "dovelaba": "dovelar",
    "dovelabais": "dovelar",
    "dovelaban": "dovelar",
    "dovelabas": "dovelar",
    "dovelad": "dovelar",
    "dovelada": "dovelar",
    "doveladas": "dovelar",
    "dovelado": "dovelar",
    "dovelados": "dovelar",
    "dovelajes": "dovelaje",
    "dovelamos": "dovelar",
    "dovelan": "dovelar",
    "dovelando": "dovelar",
    "dovelara": "dovelar",
    "dovelarais": "dovelar",
    "dovelaran": "dovelar",
    "dovelaras": "dovelar",
    "dovelare": "dovelar",
    "dovelareis": "dovelar",
    "dovelaremos": "dovelar",
    "dovelaren": "dovelar",
    "dovelares": "dovelar",
    "dovelaron": "dovelar",
    "dovelará": "dovelar",
    "dovelarán": "dovelar",
    "dovelarás": "dovelar",
    "dovelaré": "dovelar",
    "dovelaréis": "dovelar",
    "dovelaría": "dovelar",
    "dovelaríais": "dovelar",
    "dovelaríamos": "dovelar",
    "dovelarían": "dovelar",
    "dovelarías": "dovelar",
    "dovelas": "dovelar",
    "dovelase": "dovelar",
    "dovelaseis": "dovelar",
    "dovelasen": "dovelar",
    "dovelases": "dovelar",
    "dovelaste": "dovelar",
    "dovelasteis": "dovelar",
    "dovele": "dovelar",
    "dovelemos": "dovelar",
    "dovelen": "dovelar",
    "doveles": "dovelar",
    "dovelo": "dovelar",
    "dovelábamos": "dovelar",
    "doveláis": "dovelar",
    "doveláramos": "dovelar",
    "doveláremos": "dovelar",
    "dovelásemos": "dovelar",
    "dovelé": "dovelar",
    "doveléis": "dovelar",
    "doveló": "dovelar",
    "doxas": "doxa",
    "doxologías": "doxología",
    "doxógrafos": "doxógrafo",
    "doy": "dar",
    "dozava": "dozavo",
    "dozavada": "dozavado",
    "dozavadas": "dozavado",
    "dozavados": "dozavado",
    "dozavas": "dozavo",
    "dozavos": "dozavo",
    "doña": "don",
    "doñas": "don",
    "drabas": "draba",
    "dracmas": "dracma",
    "draconiana": "draconiano",
    "draconianas": "draconiano",
    "draconianos": "draconiano",
    "dracónica": "dracónico",
    "dracónicas": "dracónico",
    "dracónicos": "dracónico",
    "draga": "dragar",
    "dragaba": "dragar",
    "dragabais": "dragar",
    "dragaban": "dragar",
    "dragabas": "dragar",
    "dragad": "dragar",
    "dragada": "dragar",
    "dragadas": "dragar",
    "dragado": "dragar",
    "dragadora": "dragador",
    "dragadoras": "dragador",
    "dragadores": "dragador",
    "dragados": "dragar",
    "dragajes": "dragaje",
    "dragalinas": "dragalina",
    "dragamos": "dragar",
    "dragan": "dragar",
    "dragando": "dragar",
    "dragara": "dragar",
    "dragarais": "dragar",
    "dragaran": "dragar",
    "dragaras": "dragar",
    "dragare": "dragar",
    "dragareis": "dragar",
    "dragaremos": "dragar",
    "dragaren": "dragar",
    "dragares": "dragar",
    "dragaron": "dragar",
    "dragará": "dragar",
    "dragarán": "dragar",
    "dragarás": "dragar",
    "dragaré": "dragar",
    "dragaréis": "dragar",
    "dragaría": "dragar",
    "dragaríais": "dragar",
    "dragaríamos": "dragar",
    "dragarían": "dragar",
    "dragarías": "dragar",
    "dragas": "dragar",
    "dragase": "dragar",
    "dragaseis": "dragar",
    "dragasen": "dragar",
    "dragases": "dragar",
    "dragaste": "dragar",
    "dragasteis": "dragar",
    "drago": "dragar",
    "dragomanes": "dragomán",
    "dragonas": "dragona",
    "dragoncillos": "dragoncillo",
    "dragonea": "dragonear",
    "dragoneaba": "dragonear",
    "dragoneabais": "dragonear",
    "dragoneaban": "dragonear",
    "dragoneabas": "dragonear",
    "dragonead": "dragonear",
    "dragoneada": "dragonear",
    "dragoneadas": "dragonear",
    "dragoneado": "dragonear",
    "dragoneados": "dragonear",
    "dragoneamos": "dragonear",
    "dragonean": "dragonear",
    "dragoneando": "dragonear",
    "dragoneara": "dragonear",
    "dragonearais": "dragonear",
    "dragonearan": "dragonear",
    "dragonearas": "dragonear",
    "dragoneare": "dragonear",
    "dragoneareis": "dragonear",
    "dragonearemos": "dragonear",
    "dragonearen": "dragonear",
    "dragoneares": "dragonear",
    "dragonearon": "dragonear",
    "dragoneará": "dragonear",
    "dragonearán": "dragonear",
    "dragonearás": "dragonear",
    "dragonearé": "dragonear",
    "dragonearéis": "dragonear",
    "dragonearía": "dragonear",
    "dragonearíais": "dragonear",
    "dragonearíamos": "dragonear",
    "dragonearían": "dragonear",
    "dragonearías": "dragonear",
    "dragoneas": "dragonear",
    "dragonease": "dragonear",
    "dragoneaseis": "dragonear",
    "dragoneasen": "dragonear",
    "dragoneases": "dragonear",
    "dragoneaste": "dragonear",
    "dragoneasteis": "dragonear",
    "dragonee": "dragonear",
    "dragoneemos": "dragonear",
    "dragoneen": "dragonear",
    "dragonees": "dragonear",
    "dragoneo": "dragonear",
    "dragones": "dragón",
    "dragoneábamos": "dragonear",
    "dragoneáis": "dragonear",
    "dragoneáramos": "dragonear",
    "dragoneáremos": "dragonear",
    "dragoneásemos": "dragonear",
    "dragoneé": "dragonear",
    "dragoneéis": "dragonear",
    "dragoneó": "dragonear",
    "dragonteas": "dragontea",
    "dragontina": "dragontino",
    "dragontinas": "dragontino",
    "dragontinos": "dragontino",
    "dragos": "drago",
    "drague": "dragar",
    "draguemos": "dragar",
    "draguen": "dragar",
    "dragues": "dragar",
    "dragué": "dragar",
    "draguéis": "dragar",
    "dragábamos": "dragar",
    "dragáis": "dragar",
    "dragáramos": "dragar",
    "dragáremos": "dragar",
    "dragásemos": "dragar",
    "dragó": "dragar",
    "dramas": "drama",
    "dramatice": "dramatizar",
    "dramaticemos": "dramatizar",
    "dramaticen": "dramatizar",
    "dramatices": "dramatizar",
    "dramaticé": "dramatizar",
    "dramaticéis": "dramatizar",
    "dramatismos": "dramatismo",
    "dramatiza": "dramatizar",
    "dramatizaba": "dramatizar",
    "dramatizabais": "dramatizar",
    "dramatizaban": "dramatizar",
    "dramatizabas": "dramatizar",
    "dramatizables": "dramatizable",
    "dramatizaciones": "dramatización",
    "dramatizad": "dramatizar",
    "dramatizada": "dramatizar",
    "dramatizadas": "dramatizar",
    "dramatizado": "dramatizar",
    "dramatizados": "dramatizar",
    "dramatizamos": "dramatizar",
    "dramatizan": "dramatizar",
    "dramatizando": "dramatizar",
    "dramatizara": "dramatizar",
    "dramatizarais": "dramatizar",
    "dramatizaran": "dramatizar",
    "dramatizaras": "dramatizar",
    "dramatizare": "dramatizar",
    "dramatizareis": "dramatizar",
    "dramatizaremos": "dramatizar",
    "dramatizaren": "dramatizar",
    "dramatizares": "dramatizar",
    "dramatizaron": "dramatizar",
    "dramatizará": "dramatizar",
    "dramatizarán": "dramatizar",
    "dramatizarás": "dramatizar",
    "dramatizaré": "dramatizar",
    "dramatizaréis": "dramatizar",
    "dramatizaría": "dramatizar",
    "dramatizaríais": "dramatizar",
    "dramatizaríamos": "dramatizar",
    "dramatizarían": "dramatizar",
    "dramatizarías": "dramatizar",
    "dramatizas": "dramatizar",
    "dramatizase": "dramatizar",
    "dramatizaseis": "dramatizar",
    "dramatizasen": "dramatizar",
    "dramatizases": "dramatizar",
    "dramatizaste": "dramatizar",
    "dramatizasteis": "dramatizar",
    "dramatizo": "dramatizar",
    "dramatizábamos": "dramatizar",
    "dramatizáis": "dramatizar",
    "dramatizáramos": "dramatizar",
    "dramatizáremos": "dramatizar",
    "dramatizásemos": "dramatizar",
    "dramatizó": "dramatizar",
    "dramaturga": "dramaturgo",
    "dramaturgas": "dramaturgo",
    "dramaturgias": "dramaturgia",
    "dramaturgos": "dramaturgo",
    "dramones": "dramón",
    "drams": "dram",
    "dramática": "dramático",
    "dramáticas": "dramático",
    "dramáticos": "dramático",
    "drapea": "drapear",
    "drapeaba": "drapear",
    "drapeabais": "drapear",
    "drapeaban": "drapear",
    "drapeabas": "drapear",
    "drapead": "drapear",
    "drapeada": "drapear",
    "drapeadas": "drapear",
    "drapeado": "drapear",
    "drapeados": "drapear",
    "drapeamos": "drapear",
    "drapean": "drapear",
    "drapeando": "drapear",
    "drapeara": "drapear",
    "drapearais": "drapear",
    "drapearan": "drapear",
    "drapearas": "drapear",
    "drapeare": "drapear",
    "drapeareis": "drapear",
    "drapearemos": "drapear",
    "drapearen": "drapear",
    "drapeares": "drapear",
    "drapearon": "drapear",
    "drapeará": "drapear",
    "drapearán": "drapear",
    "drapearás": "drapear",
    "drapearé": "drapear",
    "drapearéis": "drapear",
    "drapearía": "drapear",
    "drapearíais": "drapear",
    "drapearíamos": "drapear",
    "drapearían": "drapear",
    "drapearías": "drapear",
    "drapeas": "drapear",
    "drapease": "drapear",
    "drapeaseis": "drapear",
    "drapeasen": "drapear",
    "drapeases": "drapear",
    "drapeaste": "drapear",
    "drapeasteis": "drapear",
    "drapee": "drapear",
    "drapeemos": "drapear",
    "drapeen": "drapear",
    "drapees": "drapear",
    "drapeo": "drapear",
    "drapeábamos": "drapear",
    "drapeáis": "drapear",
    "drapeáramos": "drapear",
    "drapeáremos": "drapear",
    "drapeásemos": "drapear",
    "drapeé": "drapear",
    "drapeéis": "drapear",
    "drapeó": "drapear",
    "draques": "draque",
    "dravitas": "dravita",
    "dravídica": "dravídico",
    "dravídicas": "dravídico",
    "dravídicos": "dravídico",
    "drena": "drenar",
    "drenaba": "drenar",
    "drenabais": "drenar",
    "drenaban": "drenar",
    "drenabas": "drenar",
    "drenad": "drenar",
    "drenada": "drenar",
    "drenadas": "drenar",
    "drenado": "drenar",
    "drenadores": "drenador",
    "drenados": "drenar",
    "drenajes": "drenaje",
    "drenamos": "drenar",
    "drenan": "drenar",
    "drenando": "drenar",
    "drenantes": "drenante",
    "drenara": "drenar",
    "drenarais": "drenar",
    "drenaran": "drenar",
    "drenaras": "drenar",
    "drenare": "drenar",
    "drenareis": "drenar",
    "drenaremos": "drenar",
    "drenaren": "drenar",
    "drenares": "drenar",
    "drenaron": "drenar",
    "drenará": "drenar",
    "drenarán": "drenar",
    "drenarás": "drenar",
    "drenaré": "drenar",
    "drenaréis": "drenar",
    "drenaría": "drenar",
    "drenaríais": "drenar",
    "drenaríamos": "drenar",
    "drenarían": "drenar",
    "drenarías": "drenar",
    "drenas": "drenar",
    "drenase": "drenar",
    "drenaseis": "drenar",
    "drenasen": "drenar",
    "drenases": "drenar",
    "drenaste": "drenar",
    "drenasteis": "drenar",
    "drene": "drenar",
    "drenemos": "drenar",
    "drenen": "drenar",
    "drenes": "drenar",
    "dreno": "drenar",
    "drens": "dren",
    "drenábamos": "drenar",
    "drenáis": "drenar",
    "drenáramos": "drenar",
    "drenáremos": "drenar",
    "drenásemos": "drenar",
    "drené": "drenar",
    "drenéis": "drenar",
    "drenó": "drenar",
    "drepanocitemias": "drepanocitemia",
    "drepanocitos": "drepanocito",
    "drepanocítica": "drepanocítico",
    "drepanocíticas": "drepanocítico",
    "drepanocíticos": "drepanocítico",
    "dresinas": "dresina",
    "dribla": "driblar",
    "driblaba": "driblar",
    "driblabais": "driblar",
    "driblaban": "driblar",
    "driblabas": "driblar",
    "driblad": "driblar",
    "driblada": "driblar",
    "dribladas": "driblar",
    "driblado": "driblar",
    "dribladora": "driblador",
    "dribladoras": "driblador",
    "dribladores": "driblador",
    "driblados": "driblar",
    "driblamos": "driblar",
    "driblan": "driblar",
    "driblando": "driblar",
    "driblara": "driblar",
    "driblarais": "driblar",
    "driblaran": "driblar",
    "driblaras": "driblar",
    "driblare": "driblar",
    "driblareis": "driblar",
    "driblaremos": "driblar",
    "driblaren": "driblar",
    "driblares": "driblar",
    "driblaron": "driblar",
    "driblará": "driblar",
    "driblarán": "driblar",
    "driblarás": "driblar",
    "driblaré": "driblar",
    "driblaréis": "driblar",
    "driblaría": "driblar",
    "driblaríais": "driblar",
    "driblaríamos": "driblar",
    "driblarían": "driblar",
    "driblarías": "driblar",
    "driblas": "driblar",
    "driblase": "driblar",
    "driblaseis": "driblar",
    "driblasen": "driblar",
    "driblases": "driblar",
    "driblaste": "driblar",
    "driblasteis": "driblar",
    "drible": "driblar",
    "driblemos": "driblar",
    "driblen": "driblar",
    "dribles": "driblar",
    "driblings": "dribling",
    "driblo": "driblar",
    "driblábamos": "driblar",
    "dribláis": "driblar",
    "dribláramos": "driblar",
    "dribláremos": "driblar",
    "driblásemos": "driblar",
    "driblé": "driblar",
    "dribléis": "driblar",
    "dribló": "driblar",
    "drice": "drizar",
    "dricemos": "drizar",
    "dricen": "drizar",
    "drices": "drizar",
    "dricé": "drizar",
    "dricéis": "drizar",
    "drifts": "drift",
    "driles": "dril",
    "drils": "dril",
    "drings": "dring",
    "drivers": "driver",
    "drives": "drive",
    "driza": "drizar",
    "drizaba": "drizar",
    "drizabais": "drizar",
    "drizaban": "drizar",
    "drizabas": "drizar",
    "drizad": "drizar",
    "drizada": "drizar",
    "drizadas": "drizar",
    "drizado": "drizar",
    "drizados": "drizar",
    "drizamos": "drizar",
    "drizan": "drizar",
    "drizando": "drizar",
    "drizara": "drizar",
    "drizarais": "drizar",
    "drizaran": "drizar",
    "drizaras": "drizar",
    "drizare": "drizar",
    "drizareis": "drizar",
    "drizaremos": "drizar",
    "drizaren": "drizar",
    "drizares": "drizar",
    "drizaron": "drizar",
    "drizará": "drizar",
    "drizarán": "drizar",
    "drizarás": "drizar",
    "drizaré": "drizar",
    "drizaréis": "drizar",
    "drizaría": "drizar",
    "drizaríais": "drizar",
    "drizaríamos": "drizar",
    "drizarían": "drizar",
    "drizarías": "drizar",
    "drizas": "drizar",
    "drizase": "drizar",
    "drizaseis": "drizar",
    "drizasen": "drizar",
    "drizases": "drizar",
    "drizaste": "drizar",
    "drizasteis": "drizar",
    "drizo": "drizar",
    "drizábamos": "drizar",
    "drizáis": "drizar",
    "drizáramos": "drizar",
    "drizáremos": "drizar",
    "drizásemos": "drizar",
    "drizó": "drizar",
    "droga": "drogar",
    "drogaba": "drogar",
    "drogabais": "drogar",
    "drogaban": "drogar",
    "drogabas": "drogar",
    "drogad": "drogar",
    "drogada": "drogar",
    "drogadas": "drogar",
    "drogadicciones": "drogadicción",
    "drogadicta": "drogadicto",
    "drogadictas": "drogadicto",
    "drogadictos": "drogadicto",
    "drogado": "drogar",
    "drogados": "drogar",
    "drogamos": "drogar",
    "drogan": "drogar",
    "drogando": "drogar",
    "drogara": "drogar",
    "drogarais": "drogar",
    "drogaran": "drogar",
    "drogaras": "drogar",
    "drogare": "drogar",
    "drogareis": "drogar",
    "drogaremos": "drogar",
    "drogaren": "drogar",
    "drogares": "drogar",
    "drogaron": "drogar",
    "drogará": "drogar",
    "drogarán": "drogar",
    "drogarás": "drogar",
    "drogaré": "drogar",
    "drogaréis": "drogar",
    "drogaría": "drogar",
    "drogaríais": "drogar",
    "drogaríamos": "drogar",
    "drogarían": "drogar",
    "drogarías": "drogar",
    "drogas": "drogar",
    "drogase": "drogar",
    "drogaseis": "drogar",
    "drogasen": "drogar",
    "drogases": "drogar",
    "drogaste": "drogar",
    "drogasteis": "drogar",
    "drogatas": "drogata",
    "drogo": "drogar",
    "drogoadicción": "drogadicción",
    "drogoadiccoónes": "drogadicción",
    "drogodependencias": "drogodependencia",
    "drogodependientes": "drogodependiente",
    "drogue": "drogar",
    "droguemos": "drogar",
    "droguen": "drogar",
    "droguerías": "droguería",
    "drogues": "drogar",
    "droguetes": "droguete",
    "drogué": "drogar",
    "droguéis": "drogar",
    "drogábamos": "drogar",
    "drogáis": "drogar",
    "drogáramos": "drogar",
    "drogáremos": "drogar",
    "drogásemos": "drogar",
    "drogó": "drogar",
    "dromas": "droma",
    "dromedarios": "dromedario",
    "dromiceidos": "dromiceido",
    "drongos": "drongo",
    "drops": "drop",
    "droseras": "drosera",
    "droseráceas": "droserácea",
    "druidas": "druida",
    "druidismos": "druidismo",
    "drumlins": "drumlin",
    "drupas": "drupa",
    "drupácea": "drupáceo",
    "drupáceas": "drupáceo",
    "drupáceos": "drupáceo",
    "drusa": "druso",
    "drusas": "druso",
    "drusiformes": "drusiforme",
    "drusos": "druso",
    "druídica": "druídico",
    "druídicas": "druídico",
    "druídicos": "druídico",
    "drástica": "drástico",
    "drásticas": "drástico",
    "drásticos": "drástico",
    "drávidas": "drávida",
    "dríadas": "dríada",
    "dríades": "dríade",
    "drías": "dría",
    "drúmlineses": "drumlin",
    "dsetas": "dseta",
    "duales": "dual",
    "dualidades": "dualidad",
    "dualismos": "dualismo",
    "dualistas": "dualista",
    "dualística": "dualístico",
    "dualísticas": "dualístico",
    "dualísticos": "dualístico",
    "dubas": "duba",
    "dubios": "dubio",
    "dubitaciones": "dubitación",
    "dubitativa": "dubitativo",
    "dubitativas": "dubitativo",
    "dubitativos": "dubitativo",
    "dublés": "dublé",
    "ducados": "ducado",
    "ducales": "ducal",
    "ducciones": "ducción",
    "ducentésima": "ducentésimo",
    "ducentésimas": "ducentésimo",
    "ducentésimo": "200",
    "ducentésimos": "ducentésimo",
    "duces": "duce",
    "ducha": "duchar",
    "duchaba": "duchar",
    "duchabais": "duchar",
    "duchaban": "duchar",
    "duchabas": "duchar",
    "duchad": "duchar",
    "duchada": "duchar",
    "duchadas": "duchar",
    "duchado": "duchar",
    "duchados": "duchar",
    "duchamos": "duchar",
    "duchan": "duchar",
    "duchando": "duchar",
    "duchara": "duchar",
    "ducharais": "duchar",
    "ducharan": "duchar",
    "ducharas": "duchar",
    "duchare": "duchar",
    "duchareis": "duchar",
    "ducharemos": "duchar",
    "ducharen": "duchar",
    "duchares": "duchar",
    "ducharon": "duchar",
    "duchará": "duchar",
    "ducharán": "duchar",
    "ducharás": "duchar",
    "ducharé": "duchar",
    "ducharéis": "duchar",
    "ducharía": "duchar",
    "ducharíais": "duchar",
    "ducharíamos": "duchar",
    "ducharían": "duchar",
    "ducharías": "duchar",
    "duchas": "duchar",
    "duchase": "duchar",
    "duchaseis": "duchar",
    "duchasen": "duchar",
    "duchases": "duchar",
    "duchaste": "duchar",
    "duchasteis": "duchar",
    "duche": "duchar",
    "duchemos": "duchar",
    "duchen": "duchar",
    "duches": "duchar",
    "ducho": "duchar",
    "duchábamos": "duchar",
    "ducháis": "duchar",
    "ducháramos": "duchar",
    "ducháremos": "duchar",
    "duchásemos": "duchar",
    "duché": "duchar",
    "duchéis": "duchar",
    "duchó": "duchar",
    "ductibilidades": "ductibilidad",
    "ductilidades": "ductilidad",
    "duda": "dudar",
    "dudaba": "dudar",
    "dudabais": "dudar",
    "dudaban": "dudar",
    "dudabas": "dudar",
    "dudad": "dudar",
    "dudada": "dudar",
    "dudadas": "dudar",
    "dudado": "dudar",
    "dudados": "dudar",
    "dudamos": "dudar",
    "dudan": "dudar",
    "dudando": "dudar",
    "dudara": "dudar",
    "dudarais": "dudar",
    "dudaran": "dudar",
    "dudaras": "dudar",
    "dudare": "dudar",
    "dudareis": "dudar",
    "dudaremos": "dudar",
    "dudaren": "dudar",
    "dudares": "dudar",
    "dudaron": "dudar",
    "dudará": "dudar",
    "dudarán": "dudar",
    "dudarás": "dudar",
    "dudaré": "dudar",
    "dudaréis": "dudar",
    "dudaría": "dudar",
    "dudaríais": "dudar",
    "dudaríamos": "dudar",
    "dudarían": "dudar",
    "dudarías": "dudar",
    "dudas": "dudar",
    "dudase": "dudar",
    "dudaseis": "dudar",
    "dudasen": "dudar",
    "dudases": "dudar",
    "dudaste": "dudar",
    "dudasteis": "dudar",
    "dude": "dudar",
    "dudemos": "dudar",
    "duden": "dudar",
    "dudes": "dudar",
    "dudo": "dudar",
    "dudosa": "dudoso",
    "dudosas": "dudoso",
    "dudosos": "dudoso",
    "dudábamos": "dudar",
    "dudáis": "dudar",
    "dudáramos": "dudar",
    "dudáremos": "dudar",
    "dudásemos": "dudar",
    "dudé": "dudar",
    "dudéis": "dudar",
    "dudó": "dudar",
    "duela": "doler",
    "duelajes": "duelaje",
    "duelan": "doler",
    "duelas": "duela",
    "duele": "doler",
    "duelen": "doler",
    "dueles": "doler",
    "duelistas": "duelista",
    "duelo": "doler",
    "duelos": "duelo",
    "duenarios": "duenario",
    "duendes": "duende",
    "duerma": "dormir",
    "duerman": "dormir",
    "duermas": "dormir",
    "duerme": "dormir",
    "duermen": "dormir",
    "duermes": "dormir",
    "duermevelas": "duermevela",
    "duermo": "dormir",
    "duernas": "duerna",
    "duernos": "duerno",
    "duetistas": "duetista",
    "duetos": "dueto",
    "dueña": "dueño",
    "dueñas": "dueño",
    "dueñesca": "dueñesco",
    "dueñescas": "dueñesco",
    "dueñescos": "dueñesco",
    "dueños": "dueño",
    "dufrenitas": "dufrenita",
    "dulas": "dula",
    "dulcamaras": "dulcamara",
    "dulceacuícolas": "dulceacuícola",
    "dulcedumbres": "dulcedumbre",
    "dulcera": "dulcero",
    "dulceras": "dulcero",
    "dulceros": "dulcero",
    "dulcerías": "dulcería",
    "dulces": "dulce",
    "dulcifica": "dulcificar",
    "dulcificaba": "dulcificar",
    "dulcificabais": "dulcificar",
    "dulcificaban": "dulcificar",
    "dulcificabas": "dulcificar",
    "dulcificaciones": "dulcificación",
    "dulcificad": "dulcificar",
    "dulcificada": "dulcificar",
    "dulcificadas": "dulcificar",
    "dulcificado": "dulcificar",
    "dulcificados": "dulcificar",
    "dulcificamos": "dulcificar",
    "dulcifican": "dulcificar",
    "dulcificando": "dulcificar",
    "dulcificara": "dulcificar",
    "dulcificarais": "dulcificar",
    "dulcificaran": "dulcificar",
    "dulcificaras": "dulcificar",
    "dulcificare": "dulcificar",
    "dulcificareis": "dulcificar",
    "dulcificaremos": "dulcificar",
    "dulcificaren": "dulcificar",
    "dulcificares": "dulcificar",
    "dulcificaron": "dulcificar",
    "dulcificará": "dulcificar",
    "dulcificarán": "dulcificar",
    "dulcificarás": "dulcificar",
    "dulcificaré": "dulcificar",
    "dulcificaréis": "dulcificar",
    "dulcificaría": "dulcificar",
    "dulcificaríais": "dulcificar",
    "dulcificaríamos": "dulcificar",
    "dulcificarían": "dulcificar",
    "dulcificarías": "dulcificar",
    "dulcificas": "dulcificar",
    "dulcificase": "dulcificar",
    "dulcificaseis": "dulcificar",
    "dulcificasen": "dulcificar",
    "dulcificases": "dulcificar",
    "dulcificaste": "dulcificar",
    "dulcificasteis": "dulcificar",
    "dulcifico": "dulcificar",
    "dulcificábamos": "dulcificar",
    "dulcificáis": "dulcificar",
    "dulcificáramos": "dulcificar",
    "dulcificáremos": "dulcificar",
    "dulcificásemos": "dulcificar",
    "dulcificó": "dulcificar",
    "dulcifique": "dulcificar",
    "dulcifiquemos": "dulcificar",
    "dulcifiquen": "dulcificar",
    "dulcifiques": "dulcificar",
    "dulcifiqué": "dulcificar",
    "dulcifiquéis": "dulcificar",
    "dulcinas": "dulcina",
    "dulcineas": "dulcinea",
    "dulcémeles": "dulcémele",
    "dulcísona": "dulcísono",
    "dulcísonas": "dulcísono",
    "dulcísonos": "dulcísono",
    "dulimanes": "dulimán",
    "dulzaina": "dulzaino",
    "dulzainas": "dulzaino",
    "dulzainos": "dulzaino",
    "dulzarrona": "dulzarrón",
    "dulzarronas": "dulzarrón",
    "dulzarrones": "dulzarrón",
    "dulzona": "dulzón",
    "dulzonas": "dulzón",
    "dulzones": "dulzón",
    "dulzuras": "dulzura",
    "dulías": "dulía",
    "dumas": "duma",
    "dumortieritas": "dumortierita",
    "dumperes": "dumper",
    "dumpers": "dúmper",
    "dumpings": "dumping",
    "dunas": "duna",
    "duodecimales": "duodecimal",
    "duodena": "duodeno",
    "duodenales": "duodenal",
    "duodenaria": "duodenario",
    "duodenarias": "duodenario",
    "duodenarios": "duodenario",
    "duodenas": "duodeno",
    "duodenomesocólica": "duodenomesocólico",
    "duodenomesocólicas": "duodenomesocólico",
    "duodenomesocólicos": "duodenomesocólico",
    "duodenos": "duodeno",
    "duodenoyeyunales": "duodenoyeyunal",
    "duodécima": "12",
    "duodécimas": "12",
    "duodécimo": "12",
    "duodécimos": "12",
    "duodécuplos": "duodécuplo",
    "duolineales": "duolineal",
    "duomesina": "duomesino",
    "duomesinas": "duomesino",
    "duomesinos": "duomesino",
    "duopolios": "duopolio",
    "duopsonios": "duopsonio",
    "duos": "duo",
    "dupla": "duplo",
    "duplas": "duplo",
    "duplica": "duplicar",
    "duplicaba": "duplicar",
    "duplicabais": "duplicar",
    "duplicaban": "duplicar",
    "duplicabas": "duplicar",
    "duplicaciones": "duplicación",
    "duplicad": "duplicar",
    "duplicada": "duplicar",
    "duplicadas": "duplicar",
    "duplicado": "duplicar",
    "duplicadora": "duplicador",
    "duplicadoras": "duplicador",
    "duplicadores": "duplicador",
    "duplicados": "duplicar",
    "duplicamos": "duplicar",
    "duplican": "duplicar",
    "duplicando": "duplicar",
    "duplicara": "duplicar",
    "duplicarais": "duplicar",
    "duplicaran": "duplicar",
    "duplicaras": "duplicar",
    "duplicare": "duplicar",
    "duplicareis": "duplicar",
    "duplicaremos": "duplicar",
    "duplicaren": "duplicar",
    "duplicares": "duplicar",
    "duplicaron": "duplicar",
    "duplicará": "duplicar",
    "duplicarán": "duplicar",
    "duplicarás": "duplicar",
    "duplicaré": "duplicar",
    "duplicaréis": "duplicar",
    "duplicaría": "duplicar",
    "duplicaríais": "duplicar",
    "duplicaríamos": "duplicar",
    "duplicarían": "duplicar",
    "duplicarías": "duplicar",
    "duplicas": "duplicar",
    "duplicase": "duplicar",
    "duplicaseis": "duplicar",
    "duplicasen": "duplicar",
    "duplicases": "duplicar",
    "duplicaste": "duplicar",
    "duplicasteis": "duplicar",
    "duplicativa": "duplicativo",
    "duplicativas": "duplicativo",
    "duplicativos": "duplicativo",
    "duplicaturas": "duplicatura",
    "duplicidades": "duplicidad",
    "duplico": "duplicar",
    "duplicábamos": "duplicar",
    "duplicáis": "duplicar",
    "duplicáramos": "duplicar",
    "duplicáremos": "duplicar",
    "duplicásemos": "duplicar",
    "duplicó": "duplicar",
    "duplique": "duplicar",
    "dupliquemos": "duplicar",
    "dupliquen": "duplicar",
    "dupliques": "duplicar",
    "dupliqué": "duplicar",
    "dupliquéis": "duplicar",
    "duplos": "duplo",
    "dupondios": "dupondio",
    "duques": "duque",
    "duquesa": "duque",
    "duquesas": "duque",
    "dura": "duro",
    "duraba": "durar",
    "durabais": "durar",
    "duraban": "durar",
    "durabas": "durar",
    "durabilidades": "durabilidad",
    "durables": "durable",
    "duraciones": "duración",
    "durad": "durar",
    "durada": "durar",
    "duradas": "durar",
    "duradera": "duradero",
    "duraderas": "duradero",
    "duraderos": "duradero",
    "durado": "durar",
    "durados": "durar",
    "duraluminios": "duraluminio",
    "duramadres": "duramadre",
    "duramenes": "duramen",
    "duramos": "durar",
    "duran": "durar",
    "durando": "durar",
    "durara": "durar",
    "durarais": "durar",
    "duraran": "durar",
    "duraras": "durar",
    "durare": "durar",
    "durareis": "durar",
    "duraremos": "durar",
    "duraren": "durar",
    "durares": "durar",
    "duraron": "durar",
    "durará": "durar",
    "durarán": "durar",
    "durarás": "durar",
    "duraré": "durar",
    "duraréis": "durar",
    "duraría": "durar",
    "duraríais": "durar",
    "duraríamos": "durar",
    "durarían": "durar",
    "durarías": "durar",
    "duras": "duro",
    "durase": "durar",
    "duraseis": "durar",
    "durasen": "durar",
    "durases": "durar",
    "duraste": "durar",
    "durasteis": "durar",
    "durativa": "durativo",
    "durativas": "durativo",
    "durativos": "durativo",
    "duraznillos": "duraznillo",
    "duraznos": "durazno",
    "dure": "durar",
    "duremos": "durar",
    "duren": "durar",
    "durenos": "dureno",
    "dures": "durar",
    "durezas": "dureza",
    "durillos": "durillo",
    "durinas": "durina",
    "durinodos": "durinodo",
    "durmamos": "dormir",
    "durmiendo": "dormir",
    "durmientes": "durmiente",
    "durmiera": "dormir",
    "durmierais": "dormir",
    "durmieran": "dormir",
    "durmieras": "dormir",
    "durmiere": "dormir",
    "durmiereis": "dormir",
    "durmieren": "dormir",
    "durmieres": "dormir",
    "durmieron": "dormir",
    "durmiese": "dormir",
    "durmieseis": "dormir",
    "durmiesen": "dormir",
    "durmieses": "dormir",
    "durmiéramos": "dormir",
    "durmiéremos": "dormir",
    "durmiésemos": "dormir",
    "durmió": "dormir",
    "durmáis": "dormir",
    "duro": "durar",
    "duros": "duro",
    "durábamos": "durar",
    "duráis": "durar",
    "durámenes": "duramen",
    "durámeneses": "duramen",
    "duráramos": "durar",
    "duráremos": "durar",
    "durásemos": "durar",
    "duré": "durar",
    "duréis": "durar",
    "duró": "durar",
    "durómetros": "durómetro",
    "duunvirales": "duunviral",
    "duunviratos": "duunvirato",
    "duunviros": "duunviro",
    "duxes": "dux",
    "dábamos": "dar",
    "dáctilos": "dáctilo",
    "dádivas": "dádiva",
    "dálmatas": "dálmata",
    "dámaras": "dámara",
    "dánica": "dánico",
    "dánicas": "dánico",
    "dánicos": "dánico",
    "dárdana": "dárdano",
    "dárdanas": "dárdano",
    "dárdanos": "dárdano",
    "dársenas": "dársena",
    "dátiles": "dátil",
    "dé": "dar",
    "débiles": "débil",
    "débitos": "débito",
    "décadas": "década",
    "décima": "10",
    "décimas": "décima",
    "décimo": "10",
    "décimos": "décimo",
    "décupla": "décuplo",
    "décuplas": "décuplo",
    "décuples": "décuple",
    "décuplos": "décuplo",
    "dédalos": "dédalo",
    "déficits": "déficit",
    "délfica": "délfico",
    "délficas": "délfico",
    "délficos": "délfico",
    "dérmica": "dérmico",
    "dérmicas": "dérmico",
    "dérmicos": "dérmico",
    "déspotas": "déspota",
    "díadas": "díada",
    "días": "día",
    "díctamos": "díctamo",
    "dídima": "dídimo",
    "dídimas": "dídimo",
    "dídimos": "dídimo",
    "díftera": "díftero",
    "dífteras": "díftero",
    "dífteros": "díftero",
    "dígitos": "dígito",
    "dígrafos": "dígrafo",
    "dímera": "dímero",
    "dímeras": "dímero",
    "dímeros": "dímero",
    "dímetros": "dímetro",
    "dínamos": "dínamo",
    "díploes": "díploe",
    "díptera": "díptero",
    "dípteras": "díptero",
    "dípteros": "díptero",
    "dípticas": "díptica",
    "dípticos": "díptico",
    "díscola": "díscolo",
    "díscolas": "díscolo",
    "díscolos": "díscolo",
    "dísona": "dísono",
    "dísonas": "dísono",
    "dísonos": "dísono",
    "dística": "dístico",
    "dísticas": "dístico",
    "dísticos": "dístico",
    "dístoma": "dístomo",
    "dístomas": "dístomo",
    "dístomos": "dístomo",
    "dístrica": "dístrico",
    "dístricas": "dístrico",
    "dístricos": "dístrico",
    "dítonos": "dítono",
    "dóciles": "dócil",
    "dólares": "dólar",
    "dólicos": "dólico",
    "dólmenes": "dolmen",
    "dómines": "dómine",
    "dóminos": "dómino",
    "dórica": "dórico",
    "dóricas": "dórico",
    "dóricos": "dórico",
    "dúctiles": "dúctil",
    "dúmperes": "dúmper",
    "dúmpereses": "dúmper",
    "dúmpers": "dúmper",
    "dúos": "dúo",
    "dúplicas": "dúplica",
    "dúplices": "dúplice",
    "e-mails": "e-mail",
    "eV": "electronvoltio",
    "eagles": "eagle",
    "ebanistas": "ebanista",
    "ebanisterías": "ebanistería",
    "ebenáceas": "ebenácea",
    "ebionitas": "ebionita",
    "ebonitas": "ebonita",
    "eboraria": "eborario",
    "eborarias": "eborario",
    "eborarios": "eborario",
    "ebria": "ebrio",
    "ebrias": "ebrio",
    "ebriedades": "ebriedad",
    "ebrios": "ebrio",
    "ebriosa": "ebrioso",
    "ebriosas": "ebrioso",
    "ebriosos": "ebrioso",
    "ebulliciones": "ebullición",
    "ebullometrías": "ebullometría",
    "ebulloscopias": "ebulloscopia",
    "ebulloscopios": "ebulloscopio",
    "eburnaciones": "eburnación",
    "ebúrnea": "ebúrneo",
    "ebúrneas": "ebúrneo",
    "ebúrneos": "ebúrneo",
    "ecartés": "ecarté",
    "eccehomos": "eccehomo",
    "eccemas": "eccema",
    "ecclesias": "ecclesia",
    "echa": "echar",
    "echaba": "echar",
    "echabais": "echar",
    "echaban": "echar",
    "echabas": "echar",
    "echad": "echar",
    "echada": "echar",
    "echadas": "echar",
    "echaderos": "echadero",
    "echadiza": "echadizo",
    "echadizas": "echadizo",
    "echadizos": "echadizo",
    "echado": "echar",
    "echados": "echar",
    "echaduras": "echadura",
    "echamos": "echar",
    "echan": "echar",
    "echando": "echar",
    "echara": "echar",
    "echarais": "echar",
    "echaran": "echar",
    "echaras": "echar",
    "echare": "echar",
    "echareis": "echar",
    "echaremos": "echar",
    "echaren": "echar",
    "echares": "echar",
    "echaron": "echar",
    "echarpes": "echarpe",
    "echará": "echar",
    "echarán": "echar",
    "echarás": "echar",
    "echaré": "echar",
    "echaréis": "echar",
    "echaría": "echar",
    "echaríais": "echar",
    "echaríamos": "echar",
    "echarían": "echar",
    "echarías": "echar",
    "echas": "echar",
    "echase": "echar",
    "echaseis": "echar",
    "echasen": "echar",
    "echases": "echar",
    "echaste": "echar",
    "echasteis": "echar",
    "echazones": "echazón",
    "eche": "echar",
    "echemos": "echar",
    "echen": "echar",
    "eches": "echar",
    "echo": "echar",
    "echábamos": "echar",
    "echáis": "echar",
    "echáramos": "echar",
    "echáremos": "echar",
    "echásemos": "echar",
    "eché": "echar",
    "echéis": "echar",
    "echó": "echar",
    "eclampsias": "eclampsia",
    "eclecticismos": "eclecticismo",
    "eclesiales": "eclesial",
    "eclesiologías": "eclesiología",
    "eclesiástica": "eclesiástico",
    "eclesiásticas": "eclesiástico",
    "eclesiásticos": "eclesiástico",
    "eclipsa": "eclipsar",
    "eclipsaba": "eclipsar",
    "eclipsabais": "eclipsar",
    "eclipsaban": "eclipsar",
    "eclipsabas": "eclipsar",
    "eclipsad": "eclipsar",
    "eclipsada": "eclipsar",
    "eclipsadas": "eclipsar",
    "eclipsado": "eclipsar",
    "eclipsados": "eclipsar",
    "eclipsamos": "eclipsar",
    "eclipsan": "eclipsar",
    "eclipsando": "eclipsar",
    "eclipsara": "eclipsar",
    "eclipsarais": "eclipsar",
    "eclipsaran": "eclipsar",
    "eclipsaras": "eclipsar",
    "eclipsare": "eclipsar",
    "eclipsareis": "eclipsar",
    "eclipsaremos": "eclipsar",
    "eclipsaren": "eclipsar",
    "eclipsares": "eclipsar",
    "eclipsaron": "eclipsar",
    "eclipsará": "eclipsar",
    "eclipsarán": "eclipsar",
    "eclipsarás": "eclipsar",
    "eclipsaré": "eclipsar",
    "eclipsaréis": "eclipsar",
    "eclipsaría": "eclipsar",
    "eclipsaríais": "eclipsar",
    "eclipsaríamos": "eclipsar",
    "eclipsarían": "eclipsar",
    "eclipsarías": "eclipsar",
    "eclipsas": "eclipsar",
    "eclipsase": "eclipsar",
    "eclipsaseis": "eclipsar",
    "eclipsasen": "eclipsar",
    "eclipsases": "eclipsar",
    "eclipsaste": "eclipsar",
    "eclipsasteis": "eclipsar",
    "eclipse": "eclipsar",
    "eclipsemos": "eclipsar",
    "eclipsen": "eclipsar",
    "eclipses": "eclipse",
    "eclipso": "eclipsar",
    "eclipsábamos": "eclipsar",
    "eclipsáis": "eclipsar",
    "eclipsáramos": "eclipsar",
    "eclipsáremos": "eclipsar",
    "eclipsásemos": "eclipsar",
    "eclipsé": "eclipsar",
    "eclipséis": "eclipsar",
    "eclipsó": "eclipsar",
    "eclisas": "eclisa",
    "eclogitas": "eclogita",
    "eclosiones": "eclosión",
    "ecléctica": "ecléctico",
    "eclécticas": "ecléctico",
    "eclécticos": "ecléctico",
    "eclímetros": "eclímetro",
    "eclíptica": "eclíptico",
    "eclípticas": "eclíptico",
    "eclípticos": "eclíptico",
    "ecocardiografías": "ecocardiografía",
    "ecodesarrollos": "ecodesarrollo",
    "ecografías": "ecografía",
    "ecolalias": "ecolalia",
    "ecologistas": "ecologista",
    "ecologías": "ecología",
    "ecológica": "ecológico",
    "ecológicas": "ecológico",
    "ecológicos": "ecológico",
    "economatos": "economato",
    "econometrías": "econometría",
    "economice": "economizar",
    "economicemos": "economizar",
    "economicen": "economizar",
    "economices": "economizar",
    "economicismos": "economicismo",
    "economicistas": "economicista",
    "economicoadministrativa": "economicoadministrativ",
    "economicoadministrativas": "economicoadministrativ",
    "economicoadministrativo": "economicoadministrativ",
    "economicoadministrativos": "economicoadministrativ",
    "economicé": "economizar",
    "economicéis": "economizar",
    "economistas": "economista",
    "economiza": "economizar",
    "economizaba": "economizar",
    "economizabais": "economizar",
    "economizaban": "economizar",
    "economizabas": "economizar",
    "economizad": "economizar",
    "economizada": "economizar",
    "economizadas": "economizar",
    "economizado": "economizar",
    "economizadora": "economizador",
    "economizadoras": "economizador",
    "economizadores": "economizador",
    "economizados": "economizar",
    "economizamos": "economizar",
    "economizan": "economizar",
    "economizando": "economizar",
    "economizara": "economizar",
    "economizarais": "economizar",
    "economizaran": "economizar",
    "economizaras": "economizar",
    "economizare": "economizar",
    "economizareis": "economizar",
    "economizaremos": "economizar",
    "economizaren": "economizar",
    "economizares": "economizar",
    "economizaron": "economizar",
    "economizará": "economizar",
    "economizarán": "economizar",
    "economizarás": "economizar",
    "economizaré": "economizar",
    "economizaréis": "economizar",
    "economizaría": "economizar",
    "economizaríais": "economizar",
    "economizaríamos": "economizar",
    "economizarían": "economizar",
    "economizarías": "economizar",
    "economizas": "economizar",
    "economizase": "economizar",
    "economizaseis": "economizar",
    "economizasen": "economizar",
    "economizases": "economizar",
    "economizaste": "economizar",
    "economizasteis": "economizar",
    "economizo": "economizar",
    "economizábamos": "economizar",
    "economizáis": "economizar",
    "economizáramos": "economizar",
    "economizáremos": "economizar",
    "economizásemos": "economizar",
    "economizó": "economizar",
    "econométrica": "econométrico",
    "econométricas": "econométrico",
    "econométricos": "econométrico",
    "economías": "economía",
    "económica": "económico",
    "económicas": "económico",
    "económicoadministrativa": "económicoadministrativo",
    "económicoadministrativas": "económicoadministrativo",
    "económicoadministrativos": "económicoadministrativo",
    "económicos": "económico",
    "ecos": "eco",
    "ecosferas": "ecosfera",
    "ecosistemas": "ecosistema",
    "ecosondas": "ecosonda",
    "ecospecies": "ecospecie",
    "ecotada": "ecotado",
    "ecotadas": "ecotado",
    "ecotados": "ecotado",
    "ecotasas": "ecotasa",
    "ecotipos": "ecotipo",
    "ecotonos": "ecotono",
    "ecotoxicologías": "ecotoxicología",
    "ecotoxicológica": "ecotoxicológico",
    "ecotoxicológicas": "ecotoxicológico",
    "ecotoxicológicos": "ecotoxicológico",
    "ectasias": "ectasia",
    "ectoblastos": "ectoblasto",
    "ectocarpáceas": "ectocarpácea",
    "ectocotíleos": "ectocotíleo",
    "ectocótilos": "ectocótilo",
    "ectodermos": "ectodermo",
    "ectodinamomorfa": "ectodinamomorfo",
    "ectodinamomorfas": "ectodinamomorfo",
    "ectodinamomorfos": "ectodinamomorfo",
    "ectodinamógena": "ectodinamógeno",
    "ectodinamógenas": "ectodinamógeno",
    "ectodinamógenos": "ectodinamógeno",
    "ectodérmica": "ectodérmico",
    "ectodérmicas": "ectodérmico",
    "ectodérmicos": "ectodérmico",
    "ectoparásita": "ectoparásito",
    "ectoparásitas": "ectoparásito",
    "ectoparásitos": "ectoparásito",
    "ectopias": "ectopia",
    "ectoplasmas": "ectoplasma",
    "ectoplásmica": "ectoplásmico",
    "ectoplásmicas": "ectoplásmico",
    "ectoplásmicos": "ectoplásmico",
    "ectoprocta": "ectoprocto",
    "ectoproctas": "ectoprocto",
    "ectoproctos": "ectoprocto",
    "ectosferas": "ectosfera",
    "ectosomas": "ectosoma",
    "ectropiones": "ectropión",
    "ectófilos": "ectófilo",
    "ectópica": "ectópico",
    "ectópicas": "ectópico",
    "ectópicos": "ectópico",
    "ectótrofa": "ectótrofo",
    "ectótrofas": "ectótrofo",
    "ectótrofos": "ectótrofo",
    "ecuables": "ecuable",
    "ecuaciones": "ecuación",
    "ecuaciónes": "ecuación",
    "ecualice": "ecualizar",
    "ecualicemos": "ecualizar",
    "ecualicen": "ecualizar",
    "ecualices": "ecualizar",
    "ecualicé": "ecualizar",
    "ecualicéis": "ecualizar",
    "ecualiza": "ecualizar",
    "ecualizaba": "ecualizar",
    "ecualizabais": "ecualizar",
    "ecualizaban": "ecualizar",
    "ecualizabas": "ecualizar",
    "ecualizaciones": "ecualización",
    "ecualizad": "ecualizar",
    "ecualizada": "ecualizar",
    "ecualizadas": "ecualizar",
    "ecualizado": "ecualizar",
    "ecualizadores": "ecualizador",
    "ecualizados": "ecualizar",
    "ecualizamos": "ecualizar",
    "ecualizan": "ecualizar",
    "ecualizando": "ecualizar",
    "ecualizara": "ecualizar",
    "ecualizarais": "ecualizar",
    "ecualizaran": "ecualizar",
    "ecualizaras": "ecualizar",
    "ecualizare": "ecualizar",
    "ecualizareis": "ecualizar",
    "ecualizaremos": "ecualizar",
    "ecualizaren": "ecualizar",
    "ecualizares": "ecualizar",
    "ecualizaron": "ecualizar",
    "ecualizará": "ecualizar",
    "ecualizarán": "ecualizar",
    "ecualizarás": "ecualizar",
    "ecualizaré": "ecualizar",
    "ecualizaréis": "ecualizar",
    "ecualizaría": "ecualizar",
    "ecualizaríais": "ecualizar",
    "ecualizaríamos": "ecualizar",
    "ecualizarían": "ecualizar",
    "ecualizarías": "ecualizar",
    "ecualizas": "ecualizar",
    "ecualizase": "ecualizar",
    "ecualizaseis": "ecualizar",
    "ecualizasen": "ecualizar",
    "ecualizases": "ecualizar",
    "ecualizaste": "ecualizar",
    "ecualizasteis": "ecualizar",
    "ecualizo": "ecualizar",
    "ecualizábamos": "ecualizar",
    "ecualizáis": "ecualizar",
    "ecualizáramos": "ecualizar",
    "ecualizáremos": "ecualizar",
    "ecualizásemos": "ecualizar",
    "ecualizó": "ecualizar",
    "ecuanimidades": "ecuanimidad",
    "ecuatoguineana": "ecuatoguineano",
    "ecuatoguineanas": "ecuatoguineano",
    "ecuatoguineanos": "ecuatoguineano",
    "ecuatoriales": "ecuatorial",
    "ecuatoriana": "ecuatoriano",
    "ecuatorianas": "ecuatoriano",
    "ecuatorianismos": "ecuatorianismo",
    "ecuatorianos": "ecuatoriano",
    "ecuestres": "ecuestre",
    "ecumenicidades": "ecumenicidad",
    "ecumenismos": "ecumenismo",
    "ecuménica": "ecuménico",
    "ecuménicas": "ecuménico",
    "ecuménicos": "ecuménico",
    "ecus": "ecu",
    "ecuánimes": "ecuánime",
    "eczemas": "eczema",
    "ecóloga": "ecólogo",
    "ecólogas": "ecólogo",
    "ecólogos": "ecólogo",
    "ecónomos": "ecónomo",
    "ecúmenes": "ecúmene",
    "ecúmenos": "ecúmeno",
    "edades": "edad",
    "edafologías": "edafología",
    "edafológica": "edafológico",
    "edafológicas": "edafológico",
    "edafológicos": "edafológico",
    "edafones": "edafón",
    "edaforrasgos": "edaforrasgo",
    "edafosferas": "edafosfera",
    "edafoturbaciones": "edafoturbación",
    "edafóloga": "edafólogo",
    "edafólogas": "edafólogo",
    "edafólogos": "edafólogo",
    "edecanes": "edecán",
    "edemas": "edema",
    "edematosa": "edematoso",
    "edematosas": "edematoso",
    "edematosos": "edematoso",
    "edenes": "edén",
    "edentaciones": "edentación",
    "edetanos": "edetano",
    "ediciones": "edición",
    "edictales": "edictal",
    "edictos": "edicto",
    "edifica": "edificar",
    "edificaba": "edificar",
    "edificabais": "edificar",
    "edificaban": "edificar",
    "edificabas": "edificar",
    "edificables": "edificable",
    "edificaciones": "edificación",
    "edificad": "edificar",
    "edificada": "edificar",
    "edificadas": "edificar",
    "edificado": "edificar",
    "edificadora": "edificador",
    "edificadoras": "edificador",
    "edificadores": "edificador",
    "edificados": "edificar",
    "edificamos": "edificar",
    "edifican": "edificar",
    "edificando": "edificar",
    "edificantes": "edificante",
    "edificara": "edificar",
    "edificarais": "edificar",
    "edificaran": "edificar",
    "edificaras": "edificar",
    "edificare": "edificar",
    "edificareis": "edificar",
    "edificaremos": "edificar",
    "edificaren": "edificar",
    "edificares": "edificar",
    "edificaron": "edificar",
    "edificará": "edificar",
    "edificarán": "edificar",
    "edificarás": "edificar",
    "edificaré": "edificar",
    "edificaréis": "edificar",
    "edificaría": "edificar",
    "edificaríais": "edificar",
    "edificaríamos": "edificar",
    "edificarían": "edificar",
    "edificarías": "edificar",
    "edificas": "edificar",
    "edificase": "edificar",
    "edificaseis": "edificar",
    "edificasen": "edificar",
    "edificases": "edificar",
    "edificaste": "edificar",
    "edificasteis": "edificar",
    "edificativa": "edificativo",
    "edificativas": "edificativo",
    "edificativos": "edificativo",
    "edificatoria": "edificatorio",
    "edificatorias": "edificatorio",
    "edificatorios": "edificatorio",
    "edificios": "edificio",
    "edifico": "edificar",
    "edificábamos": "edificar",
    "edificáis": "edificar",
    "edificáramos": "edificar",
    "edificáremos": "edificar",
    "edificásemos": "edificar",
    "edificó": "edificar",
    "edifique": "edificar",
    "edifiquemos": "edificar",
    "edifiquen": "edificar",
    "edifiques": "edificar",
    "edifiqué": "edificar",
    "edifiquéis": "edificar",
    "ediles": "edil",
    "edilicia": "edilicio",
    "edilicias": "edilicio",
    "edilicios": "edilicio",
    "edilidades": "edilidad",
    "edita": "editar",
    "editaba": "editar",
    "editabais": "editar",
    "editaban": "editar",
    "editabas": "editar",
    "editables": "editable",
    "editad": "editar",
    "editada": "editar",
    "editadas": "editar",
    "editado": "editar",
    "editados": "editar",
    "editajes": "editaje",
    "editamos": "editar",
    "editan": "editar",
    "editando": "editar",
    "editara": "editar",
    "editarais": "editar",
    "editaran": "editar",
    "editaras": "editar",
    "editare": "editar",
    "editareis": "editar",
    "editaremos": "editar",
    "editaren": "editar",
    "editares": "editar",
    "editaron": "editar",
    "editará": "editar",
    "editarán": "editar",
    "editarás": "editar",
    "editaré": "editar",
    "editaréis": "editar",
    "editaría": "editar",
    "editaríais": "editar",
    "editaríamos": "editar",
    "editarían": "editar",
    "editarías": "editar",
    "editas": "editar",
    "editase": "editar",
    "editaseis": "editar",
    "editasen": "editar",
    "editases": "editar",
    "editaste": "editar",
    "editasteis": "editar",
    "edite": "editar",
    "editemos": "editar",
    "editen": "editar",
    "edites": "editar",
    "edito": "editar",
    "editora": "editor",
    "editoras": "editor",
    "editores": "editor",
    "editoriales": "editorial",
    "editorialice": "editorializar",
    "editorialicemos": "editorializar",
    "editorialicen": "editorializar",
    "editorialices": "editorializar",
    "editorialicé": "editorializar",
    "editorialicéis": "editorializar",
    "editorialistas": "editorialista",
    "editorializa": "editorializar",
    "editorializaba": "editorializar",
    "editorializabais": "editorializar",
    "editorializaban": "editorializar",
    "editorializabas": "editorializar",
    "editorializad": "editorializar",
    "editorializada": "editorializar",
    "editorializadas": "editorializar",
    "editorializado": "editorializar",
    "editorializados": "editorializar",
    "editorializamos": "editorializar",
    "editorializan": "editorializar",
    "editorializando": "editorializar",
    "editorializara": "editorializar",
    "editorializarais": "editorializar",
    "editorializaran": "editorializar",
    "editorializaras": "editorializar",
    "editorializare": "editorializar",
    "editorializareis": "editorializar",
    "editorializaremos": "editorializar",
    "editorializaren": "editorializar",
    "editorializares": "editorializar",
    "editorializaron": "editorializar",
    "editorializará": "editorializar",
    "editorializarán": "editorializar",
    "editorializarás": "editorializar",
    "editorializaré": "editorializar",
    "editorializaréis": "editorializar",
    "editorializaría": "editorializar",
    "editorializaríais": "editorializar",
    "editorializaríamos": "editorializar",
    "editorializarían": "editorializar",
    "editorializarías": "editorializar",
    "editorializas": "editorializar",
    "editorializase": "editorializar",
    "editorializaseis": "editorializar",
    "editorializasen": "editorializar",
    "editorializases": "editorializar",
    "editorializaste": "editorializar",
    "editorializasteis": "editorializar",
    "editorializo": "editorializar",
    "editorializábamos": "editorializar",
    "editorializáis": "editorializar",
    "editorializáramos": "editorializar",
    "editorializáremos": "editorializar",
    "editorializásemos": "editorializar",
    "editorializó": "editorializar",
    "editábamos": "editar",
    "editáis": "editar",
    "editáramos": "editar",
    "editáremos": "editar",
    "editásemos": "editar",
    "edité": "editar",
    "editéis": "editar",
    "editó": "editar",
    "edogoniáceas": "edogoniácea",
    "edomitas": "edomita",
    "edredones": "edredón",
    "educa": "educar",
    "educaba": "educar",
    "educabais": "educar",
    "educaban": "educar",
    "educabas": "educar",
    "educables": "educable",
    "educacionales": "educacional",
    "educaciones": "educación",
    "educad": "educar",
    "educada": "educar",
    "educadas": "educar",
    "educado": "educar",
    "educadora": "educador",
    "educadoras": "educador",
    "educadores": "educador",
    "educados": "educar",
    "educamos": "educar",
    "educan": "educar",
    "educanda": "educando",
    "educandas": "educando",
    "educando": "educar",
    "educandos": "educando",
    "educara": "educar",
    "educarais": "educar",
    "educaran": "educar",
    "educaras": "educar",
    "educare": "educar",
    "educareis": "educar",
    "educaremos": "educar",
    "educaren": "educar",
    "educares": "educar",
    "educaron": "educar",
    "educará": "educar",
    "educarán": "educar",
    "educarás": "educar",
    "educaré": "educar",
    "educaréis": "educar",
    "educaría": "educar",
    "educaríais": "educar",
    "educaríamos": "educar",
    "educarían": "educar",
    "educarías": "educar",
    "educas": "educar",
    "educase": "educar",
    "educaseis": "educar",
    "educasen": "educar",
    "educases": "educar",
    "educaste": "educar",
    "educasteis": "educar",
    "educativa": "educativo",
    "educativas": "educativo",
    "educativos": "educativo",
    "educciones": "educción",
    "educo": "educar",
    "educábamos": "educar",
    "educáis": "educar",
    "educáramos": "educar",
    "educáremos": "educar",
    "educásemos": "educar",
    "educó": "educar",
    "edulcora": "edulcorar",
    "edulcoraba": "edulcorar",
    "edulcorabais": "edulcorar",
    "edulcoraban": "edulcorar",
    "edulcorabas": "edulcorar",
    "edulcoraciones": "edulcoración",
    "edulcorad": "edulcorar",
    "edulcorada": "edulcorar",
    "edulcoradas": "edulcorar",
    "edulcorado": "edulcorar",
    "edulcorados": "edulcorar",
    "edulcoramos": "edulcorar",
    "edulcoran": "edulcorar",
    "edulcorando": "edulcorar",
    "edulcorantes": "edulcorante",
    "edulcorara": "edulcorar",
    "edulcorarais": "edulcorar",
    "edulcoraran": "edulcorar",
    "edulcoraras": "edulcorar",
    "edulcorare": "edulcorar",
    "edulcorareis": "edulcorar",
    "edulcoraremos": "edulcorar",
    "edulcoraren": "edulcorar",
    "edulcorares": "edulcorar",
    "edulcoraron": "edulcorar",
    "edulcorará": "edulcorar",
    "edulcorarán": "edulcorar",
    "edulcorarás": "edulcorar",
    "edulcoraré": "edulcorar",
    "edulcoraréis": "edulcorar",
    "edulcoraría": "edulcorar",
    "edulcoraríais": "edulcorar",
    "edulcoraríamos": "edulcorar",
    "edulcorarían": "edulcorar",
    "edulcorarías": "edulcorar",
    "edulcoras": "edulcorar",
    "edulcorase": "edulcorar",
    "edulcoraseis": "edulcorar",
    "edulcorasen": "edulcorar",
    "edulcorases": "edulcorar",
    "edulcoraste": "edulcorar",
    "edulcorasteis": "edulcorar",
    "edulcore": "edulcorar",
    "edulcoremos": "edulcorar",
    "edulcoren": "edulcorar",
    "edulcores": "edulcorar",
    "edulcoro": "edulcorar",
    "edulcorábamos": "edulcorar",
    "edulcoráis": "edulcorar",
    "edulcoráramos": "edulcorar",
    "edulcoráremos": "edulcorar",
    "edulcorásemos": "edulcorar",
    "edulcoré": "edulcorar",
    "edulcoréis": "edulcorar",
    "edulcoró": "edulcorar",
    "eduque": "educar",
    "eduquemos": "educar",
    "eduquen": "educar",
    "eduques": "educar",
    "eduqué": "educar",
    "eduquéis": "educar",
    "edáfica": "edáfico",
    "edáficas": "edáfico",
    "edáficos": "edáfico",
    "edénica": "edénico",
    "edénicas": "edénico",
    "edénicos": "edénico",
    "edículos": "edículo",
    "edómetros": "edómetro",
    "efebos": "efebo",
    "efectismos": "efectismo",
    "efectistas": "efectista",
    "efectiva": "efectivo",
    "efectivas": "efectivo",
    "efectividades": "efectividad",
    "efectivos": "efectivo",
    "efectora": "efector",
    "efectoras": "efector",
    "efectores": "efector",
    "efectos": "efecto",
    "efectuaba": "efectuar",
    "efectuabais": "efectuar",
    "efectuaban": "efectuar",
    "efectuabas": "efectuar",
    "efectuaciones": "efectuación",
    "efectuad": "efectuar",
    "efectuada": "efectuar",
    "efectuadas": "efectuar",
    "efectuado": "efectuar",
    "efectuados": "efectuar",
    "efectuamos": "efectuar",
    "efectuando": "efectuar",
    "efectuara": "efectuar",
    "efectuarais": "efectuar",
    "efectuaran": "efectuar",
    "efectuaras": "efectuar",
    "efectuare": "efectuar",
    "efectuareis": "efectuar",
    "efectuaremos": "efectuar",
    "efectuaren": "efectuar",
    "efectuares": "efectuar",
    "efectuaron": "efectuar",
    "efectuará": "efectuar",
    "efectuarán": "efectuar",
    "efectuarás": "efectuar",
    "efectuaré": "efectuar",
    "efectuaréis": "efectuar",
    "efectuaría": "efectuar",
    "efectuaríais": "efectuar",
    "efectuaríamos": "efectuar",
    "efectuarían": "efectuar",
    "efectuarías": "efectuar",
    "efectuase": "efectuar",
    "efectuaseis": "efectuar",
    "efectuasen": "efectuar",
    "efectuases": "efectuar",
    "efectuaste": "efectuar",
    "efectuasteis": "efectuar",
    "efectuemos": "efectuar",
    "efectuábamos": "efectuar",
    "efectuáis": "efectuar",
    "efectuáramos": "efectuar",
    "efectuáremos": "efectuar",
    "efectuásemos": "efectuar",
    "efectué": "efectuar",
    "efectuéis": "efectuar",
    "efectuó": "efectuar",
    "efectúa": "efectuar",
    "efectúan": "efectuar",
    "efectúas": "efectuar",
    "efectúe": "efectuar",
    "efectúen": "efectuar",
    "efectúes": "efectuar",
    "efectúo": "efectuar",
    "efedráceas": "efedrácea",
    "efendis": "efendi",
    "eferentes": "eferente",
    "efervescencias": "efervescencia",
    "efervescentes": "efervescente",
    "efes": "efe",
    "efesia": "efesio",
    "efesias": "efesio",
    "efesina": "efesino",
    "efesinas": "efesino",
    "efesinos": "efesino",
    "efesios": "efesio",
    "eficaces": "eficaz",
    "eficacias": "eficacia",
    "eficazes": "eficaz",
    "eficiencias": "eficiencia",
    "eficientes": "eficiente",
    "efigies": "efigie",
    "eflorece": "eflorecer",
    "efloreced": "eflorecer",
    "eflorecemos": "eflorecer",
    "eflorecen": "eflorecer",
    "efloreceremos": "eflorecer",
    "eflorecerá": "eflorecer",
    "eflorecerán": "eflorecer",
    "eflorecerás": "eflorecer",
    "efloreceré": "eflorecer",
    "efloreceréis": "eflorecer",
    "eflorecería": "eflorecer",
    "efloreceríais": "eflorecer",
    "efloreceríamos": "eflorecer",
    "eflorecerían": "eflorecer",
    "eflorecerías": "eflorecer",
    "efloreces": "eflorecer",
    "eflorecida": "eflorecer",
    "eflorecidas": "eflorecer",
    "eflorecido": "eflorecer",
    "eflorecidos": "eflorecer",
    "efloreciendo": "eflorecer",
    "efloreciera": "eflorecer",
    "eflorecierais": "eflorecer",
    "eflorecieran": "eflorecer",
    "eflorecieras": "eflorecer",
    "efloreciere": "eflorecer",
    "efloreciereis": "eflorecer",
    "eflorecieren": "eflorecer",
    "eflorecieres": "eflorecer",
    "eflorecieron": "eflorecer",
    "efloreciese": "eflorecer",
    "eflorecieseis": "eflorecer",
    "efloreciesen": "eflorecer",
    "eflorecieses": "eflorecer",
    "eflorecimos": "eflorecer",
    "efloreciste": "eflorecer",
    "eflorecisteis": "eflorecer",
    "efloreciéramos": "eflorecer",
    "efloreciéremos": "eflorecer",
    "efloreciésemos": "eflorecer",
    "efloreció": "eflorecer",
    "eflorecéis": "eflorecer",
    "eflorecí": "eflorecer",
    "eflorecía": "eflorecer",
    "eflorecíais": "eflorecer",
    "eflorecíamos": "eflorecer",
    "eflorecían": "eflorecer",
    "eflorecías": "eflorecer",
    "eflorescencias": "eflorescencia",
    "eflorescentes": "eflorescente",
    "eflorezca": "eflorecer",
    "eflorezcamos": "eflorecer",
    "eflorezcan": "eflorecer",
    "eflorezcas": "eflorecer",
    "eflorezco": "eflorecer",
    "eflorezcáis": "eflorecer",
    "efluentes": "efluente",
    "efluvios": "efluvio",
    "efracciones": "efracción",
    "efractora": "efractor",
    "efractoras": "efractor",
    "efractores": "efractor",
    "efusiones": "efusión",
    "efusiva": "efusivo",
    "efusivas": "efusivo",
    "efusivos": "efusivo",
    "efusiómetros": "efusiómetro",
    "efusores": "efusor",
    "efélides": "efélide",
    "efímera": "efímero",
    "efímeras": "efímero",
    "efímeros": "efímero",
    "egarenses": "egarense",
    "egea": "egeo",
    "egeas": "egeo",
    "egeos": "egeo",
    "egidas": "egida",
    "egipanes": "egipán",
    "egipcia": "egipcio",
    "egipciaca": "egipcíaco",
    "egipciacas": "egipcíaco",
    "egipciacos": "egipcíaco",
    "egipcias": "egipcio",
    "egipcios": "egipcio",
    "egipcíaca": "egipcíaco",
    "egipcíacas": "egipcíaco",
    "egipcíacos": "egipcíaco",
    "egiptologías": "egiptología",
    "egiptológica": "egiptológico",
    "egiptológicas": "egiptológico",
    "egiptológicos": "egiptológico",
    "egiptóloga": "egiptólogo",
    "egiptólogas": "egiptólogo",
    "egiptólogos": "egiptólogo",
    "egirinas": "egirina",
    "egocentrismos": "egocentrismo",
    "egocéntrica": "egocéntrico",
    "egocéntricas": "egocéntrico",
    "egocéntricos": "egocéntrico",
    "egolatrías": "egolatría",
    "egolátrica": "egolátrico",
    "egolátricas": "egolátrico",
    "egolátricos": "egolátrico",
    "egos": "ego",
    "egotismos": "egotismo",
    "egotistas": "egotista",
    "egoísmos": "egoísmo",
    "egoístas": "egoísta",
    "egregia": "egregio",
    "egregias": "egregio",
    "egregios": "egregio",
    "egresa": "egresar",
    "egresaba": "egresar",
    "egresabais": "egresar",
    "egresaban": "egresar",
    "egresabas": "egresar",
    "egresad": "egresar",
    "egresada": "egresar",
    "egresadas": "egresar",
    "egresado": "egresar",
    "egresados": "egresar",
    "egresamos": "egresar",
    "egresan": "egresar",
    "egresando": "egresar",
    "egresara": "egresar",
    "egresarais": "egresar",
    "egresaran": "egresar",
    "egresaras": "egresar",
    "egresare": "egresar",
    "egresareis": "egresar",
    "egresaremos": "egresar",
    "egresaren": "egresar",
    "egresares": "egresar",
    "egresaron": "egresar",
    "egresará": "egresar",
    "egresarán": "egresar",
    "egresarás": "egresar",
    "egresaré": "egresar",
    "egresaréis": "egresar",
    "egresaría": "egresar",
    "egresaríais": "egresar",
    "egresaríamos": "egresar",
    "egresarían": "egresar",
    "egresarías": "egresar",
    "egresas": "egresar",
    "egresase": "egresar",
    "egresaseis": "egresar",
    "egresasen": "egresar",
    "egresases": "egresar",
    "egresaste": "egresar",
    "egresasteis": "egresar",
    "egrese": "egresar",
    "egresemos": "egresar",
    "egresen": "egresar",
    "egreses": "egresar",
    "egresiva": "egresivo",
    "egresivas": "egresivo",
    "egresivos": "egresivo",
    "egreso": "egresar",
    "egresos": "egreso",
    "egresábamos": "egresar",
    "egresáis": "egresar",
    "egresáramos": "egresar",
    "egresáremos": "egresar",
    "egresásemos": "egresar",
    "egresé": "egresar",
    "egreséis": "egresar",
    "egresó": "egresar",
    "ególatras": "ególatra",
    "eideres": "eider",
    "eidética": "eidético",
    "eidéticas": "eidético",
    "eidéticos": "eidético",
    "einsteinios": "einsteinio",
    "einstenios": "einstenio",
    "ej.": "ejemplo",
    "ej.s": "ejemplo",
    "ejecuciones": "ejecución",
    "ejecuta": "ejecutar",
    "ejecutaba": "ejecutar",
    "ejecutabais": "ejecutar",
    "ejecutaban": "ejecutar",
    "ejecutabas": "ejecutar",
    "ejecutables": "ejecutable",
    "ejecutad": "ejecutar",
    "ejecutada": "ejecutar",
    "ejecutadas": "ejecutar",
    "ejecutado": "ejecutar",
    "ejecutados": "ejecutar",
    "ejecutamos": "ejecutar",
    "ejecutan": "ejecutar",
    "ejecutando": "ejecutar",
    "ejecutantes": "ejecutante",
    "ejecutara": "ejecutar",
    "ejecutarais": "ejecutar",
    "ejecutaran": "ejecutar",
    "ejecutaras": "ejecutar",
    "ejecutare": "ejecutar",
    "ejecutareis": "ejecutar",
    "ejecutaremos": "ejecutar",
    "ejecutaren": "ejecutar",
    "ejecutares": "ejecutar",
    "ejecutaron": "ejecutar",
    "ejecutará": "ejecutar",
    "ejecutarán": "ejecutar",
    "ejecutarás": "ejecutar",
    "ejecutaré": "ejecutar",
    "ejecutaréis": "ejecutar",
    "ejecutaría": "ejecutar",
    "ejecutaríais": "ejecutar",
    "ejecutaríamos": "ejecutar",
    "ejecutarían": "ejecutar",
    "ejecutarías": "ejecutar",
    "ejecutas": "ejecutar",
    "ejecutase": "ejecutar",
    "ejecutaseis": "ejecutar",
    "ejecutasen": "ejecutar",
    "ejecutases": "ejecutar",
    "ejecutaste": "ejecutar",
    "ejecutasteis": "ejecutar",
    "ejecute": "ejecutar",
    "ejecutemos": "ejecutar",
    "ejecuten": "ejecutar",
    "ejecutes": "ejecutar",
    "ejecutiva": "ejecutivo",
    "ejecutivas": "ejecutivo",
    "ejecutividades": "ejecutividad",
    "ejecutivos": "ejecutivo",
    "ejecuto": "ejecutar",
    "ejecutora": "ejecutor",
    "ejecutoras": "ejecutor",
    "ejecutores": "ejecutor",
    "ejecutoria": "ejecutorio",
    "ejecutoriaba": "ejecutoriar",
    "ejecutoriabais": "ejecutoriar",
    "ejecutoriaban": "ejecutoriar",
    "ejecutoriabas": "ejecutoriar",
    "ejecutoriad": "ejecutoriar",
    "ejecutoriada": "ejecutoriar",
    "ejecutoriadas": "ejecutoriar",
    "ejecutoriado": "ejecutoriar",
    "ejecutoriados": "ejecutoriar",
    "ejecutoriamos": "ejecutoriar",
    "ejecutoriando": "ejecutoriar",
    "ejecutoriara": "ejecutoriar",
    "ejecutoriarais": "ejecutoriar",
    "ejecutoriaran": "ejecutoriar",
    "ejecutoriaras": "ejecutoriar",
    "ejecutoriare": "ejecutoriar",
    "ejecutoriareis": "ejecutoriar",
    "ejecutoriaremos": "ejecutoriar",
    "ejecutoriaren": "ejecutoriar",
    "ejecutoriares": "ejecutoriar",
    "ejecutoriaron": "ejecutoriar",
    "ejecutoriará": "ejecutoriar",
    "ejecutoriarán": "ejecutoriar",
    "ejecutoriarás": "ejecutoriar",
    "ejecutoriaré": "ejecutoriar",
    "ejecutoriaréis": "ejecutoriar",
    "ejecutoriaría": "ejecutoriar",
    "ejecutoriaríais": "ejecutoriar",
    "ejecutoriaríamos": "ejecutoriar",
    "ejecutoriarían": "ejecutoriar",
    "ejecutoriarías": "ejecutoriar",
    "ejecutorias": "ejecutorio",
    "ejecutoriase": "ejecutoriar",
    "ejecutoriaseis": "ejecutoriar",
    "ejecutoriasen": "ejecutoriar",
    "ejecutoriases": "ejecutoriar",
    "ejecutoriaste": "ejecutoriar",
    "ejecutoriasteis": "ejecutoriar",
    "ejecutoriedades": "ejecutoriedad",
    "ejecutoriemos": "ejecutoriar",
    "ejecutorios": "ejecutorio",
    "ejecutoriábamos": "ejecutoriar",
    "ejecutoriáis": "ejecutoriar",
    "ejecutoriáramos": "ejecutoriar",
    "ejecutoriáremos": "ejecutoriar",
    "ejecutoriásemos": "ejecutoriar",
    "ejecutorié": "ejecutoriar",
    "ejecutoriéis": "ejecutoriar",
    "ejecutorió": "ejecutoriar",
    "ejecutoría": "ejecutoriar",
    "ejecutorían": "ejecutoriar",
    "ejecutorías": "ejecutoriar",
    "ejecutoríe": "ejecutoriar",
    "ejecutoríen": "ejecutoriar",
    "ejecutoríes": "ejecutoriar",
    "ejecutorío": "ejecutoriar",
    "ejecutábamos": "ejecutar",
    "ejecutáis": "ejecutar",
    "ejecutáramos": "ejecutar",
    "ejecutáremos": "ejecutar",
    "ejecutásemos": "ejecutar",
    "ejecuté": "ejecutar",
    "ejecutéis": "ejecutar",
    "ejecutó": "ejecutar",
    "ejemplares": "ejemplar",
    "ejemplarice": "ejemplarizar",
    "ejemplaricemos": "ejemplarizar",
    "ejemplaricen": "ejemplarizar",
    "ejemplarices": "ejemplarizar",
    "ejemplaricé": "ejemplarizar",
    "ejemplaricéis": "ejemplarizar",
    "ejemplaridades": "ejemplaridad",
    "ejemplariza": "ejemplarizar",
    "ejemplarizaba": "ejemplarizar",
    "ejemplarizabais": "ejemplarizar",
    "ejemplarizaban": "ejemplarizar",
    "ejemplarizabas": "ejemplarizar",
    "ejemplarizaciones": "ejemplarización",
    "ejemplarizad": "ejemplarizar",
    "ejemplarizada": "ejemplarizar",
    "ejemplarizadas": "ejemplarizar",
    "ejemplarizado": "ejemplarizar",
    "ejemplarizados": "ejemplarizar",
    "ejemplarizamos": "ejemplarizar",
    "ejemplarizan": "ejemplarizar",
    "ejemplarizando": "ejemplarizar",
    "ejemplarizantes": "ejemplarizante",
    "ejemplarizara": "ejemplarizar",
    "ejemplarizarais": "ejemplarizar",
    "ejemplarizaran": "ejemplarizar",
    "ejemplarizaras": "ejemplarizar",
    "ejemplarizare": "ejemplarizar",
    "ejemplarizareis": "ejemplarizar",
    "ejemplarizaremos": "ejemplarizar",
    "ejemplarizaren": "ejemplarizar",
    "ejemplarizares": "ejemplarizar",
    "ejemplarizaron": "ejemplarizar",
    "ejemplarizará": "ejemplarizar",
    "ejemplarizarán": "ejemplarizar",
    "ejemplarizarás": "ejemplarizar",
    "ejemplarizaré": "ejemplarizar",
    "ejemplarizaréis": "ejemplarizar",
    "ejemplarizaría": "ejemplarizar",
    "ejemplarizaríais": "ejemplarizar",
    "ejemplarizaríamos": "ejemplarizar",
    "ejemplarizarían": "ejemplarizar",
    "ejemplarizarías": "ejemplarizar",
    "ejemplarizas": "ejemplarizar",
    "ejemplarizase": "ejemplarizar",
    "ejemplarizaseis": "ejemplarizar",
    "ejemplarizasen": "ejemplarizar",
    "ejemplarizases": "ejemplarizar",
    "ejemplarizaste": "ejemplarizar",
    "ejemplarizasteis": "ejemplarizar",
    "ejemplarizo": "ejemplarizar",
    "ejemplarizábamos": "ejemplarizar",
    "ejemplarizáis": "ejemplarizar",
    "ejemplarizáramos": "ejemplarizar",
    "ejemplarizáremos": "ejemplarizar",
    "ejemplarizásemos": "ejemplarizar",
    "ejemplarizó": "ejemplarizar",
    "ejemplifica": "ejemplificar",
    "ejemplificaba": "ejemplificar",
    "ejemplificabais": "ejemplificar",
    "ejemplificaban": "ejemplificar",
    "ejemplificabas": "ejemplificar",
    "ejemplificaciones": "ejemplificación",
    "ejemplificad": "ejemplificar",
    "ejemplificada": "ejemplificar",
    "ejemplificadas": "ejemplificar",
    "ejemplificado": "ejemplificar",
    "ejemplificados": "ejemplificar",
    "ejemplificamos": "ejemplificar",
    "ejemplifican": "ejemplificar",
    "ejemplificando": "ejemplificar",
    "ejemplificara": "ejemplificar",
    "ejemplificarais": "ejemplificar",
    "ejemplificaran": "ejemplificar",
    "ejemplificaras": "ejemplificar",
    "ejemplificare": "ejemplificar",
    "ejemplificareis": "ejemplificar",
    "ejemplificaremos": "ejemplificar",
    "ejemplificaren": "ejemplificar",
    "ejemplificares": "ejemplificar",
    "ejemplificaron": "ejemplificar",
    "ejemplificará": "ejemplificar",
    "ejemplificarán": "ejemplificar",
    "ejemplificarás": "ejemplificar",
    "ejemplificaré": "ejemplificar",
    "ejemplificaréis": "ejemplificar",
    "ejemplificaría": "ejemplificar",
    "ejemplificaríais": "ejemplificar",
    "ejemplificaríamos": "ejemplificar",
    "ejemplificarían": "ejemplificar",
    "ejemplificarías": "ejemplificar",
    "ejemplificas": "ejemplificar",
    "ejemplificase": "ejemplificar",
    "ejemplificaseis": "ejemplificar",
    "ejemplificasen": "ejemplificar",
    "ejemplificases": "ejemplificar",
    "ejemplificaste": "ejemplificar",
    "ejemplificasteis": "ejemplificar",
    "ejemplifico": "ejemplificar",
    "ejemplificábamos": "ejemplificar",
    "ejemplificáis": "ejemplificar",
    "ejemplificáramos": "ejemplificar",
    "ejemplificáremos": "ejemplificar",
    "ejemplificásemos": "ejemplificar",
    "ejemplificó": "ejemplificar",
    "ejemplifique": "ejemplificar",
    "ejemplifiquemos": "ejemplificar",
    "ejemplifiquen": "ejemplificar",
    "ejemplifiques": "ejemplificar",
    "ejemplifiqué": "ejemplificar",
    "ejemplifiquéis": "ejemplificar",
    "ejemplos": "ejemplo",
    "ejerce": "ejercer",
    "ejerced": "ejercer",
    "ejercemos": "ejercer",
    "ejercen": "ejercer",
    "ejerceremos": "ejercer",
    "ejercerá": "ejercer",
    "ejercerán": "ejercer",
    "ejercerás": "ejercer",
    "ejerceré": "ejercer",
    "ejerceréis": "ejercer",
    "ejercería": "ejercer",
    "ejerceríais": "ejercer",
    "ejerceríamos": "ejercer",
    "ejercerían": "ejercer",
    "ejercerías": "ejercer",
    "ejerces": "ejercer",
    "ejercibles": "ejercible",
    "ejercicios": "ejercicio",
    "ejercida": "ejercer",
    "ejercidas": "ejercer",
    "ejercido": "ejercer",
    "ejercidos": "ejercer",
    "ejerciendo": "ejercer",
    "ejercientes": "ejerciente",
    "ejerciera": "ejercer",
    "ejercierais": "ejercer",
    "ejercieran": "ejercer",
    "ejercieras": "ejercer",
    "ejerciere": "ejercer",
    "ejerciereis": "ejercer",
    "ejercieren": "ejercer",
    "ejercieres": "ejercer",
    "ejercieron": "ejercer",
    "ejerciese": "ejercer",
    "ejercieseis": "ejercer",
    "ejerciesen": "ejercer",
    "ejercieses": "ejercer",
    "ejercimos": "ejercer",
    "ejerciste": "ejercer",
    "ejercisteis": "ejercer",
    "ejercita": "ejercitar",
    "ejercitaba": "ejercitar",
    "ejercitabais": "ejercitar",
    "ejercitaban": "ejercitar",
    "ejercitabas": "ejercitar",
    "ejercitables": "ejercitable",
    "ejercitaciones": "ejercitación",
    "ejercitad": "ejercitar",
    "ejercitada": "ejercitar",
    "ejercitadas": "ejercitar",
    "ejercitado": "ejercitar",
    "ejercitados": "ejercitar",
    "ejercitamos": "ejercitar",
    "ejercitan": "ejercitar",
    "ejercitando": "ejercitar",
    "ejercitantes": "ejercitante",
    "ejercitara": "ejercitar",
    "ejercitarais": "ejercitar",
    "ejercitaran": "ejercitar",
    "ejercitaras": "ejercitar",
    "ejercitare": "ejercitar",
    "ejercitareis": "ejercitar",
    "ejercitaremos": "ejercitar",
    "ejercitaren": "ejercitar",
    "ejercitares": "ejercitar",
    "ejercitaron": "ejercitar",
    "ejercitará": "ejercitar",
    "ejercitarán": "ejercitar",
    "ejercitarás": "ejercitar",
    "ejercitaré": "ejercitar",
    "ejercitaréis": "ejercitar",
    "ejercitaría": "ejercitar",
    "ejercitaríais": "ejercitar",
    "ejercitaríamos": "ejercitar",
    "ejercitarían": "ejercitar",
    "ejercitarías": "ejercitar",
    "ejercitas": "ejercitar",
    "ejercitase": "ejercitar",
    "ejercitaseis": "ejercitar",
    "ejercitasen": "ejercitar",
    "ejercitases": "ejercitar",
    "ejercitaste": "ejercitar",
    "ejercitasteis": "ejercitar",
    "ejercite": "ejercitar",
    "ejercitemos": "ejercitar",
    "ejerciten": "ejercitar",
    "ejercites": "ejercitar",
    "ejercito": "ejercitar",
    "ejercitábamos": "ejercitar",
    "ejercitáis": "ejercitar",
    "ejercitáramos": "ejercitar",
    "ejercitáremos": "ejercitar",
    "ejercitásemos": "ejercitar",
    "ejercité": "ejercitar",
    "ejercitéis": "ejercitar",
    "ejercitó": "ejercitar",
    "ejerciéramos": "ejercer",
    "ejerciéremos": "ejercer",
    "ejerciésemos": "ejercer",
    "ejerció": "ejercer",
    "ejercéis": "ejercer",
    "ejercí": "ejercer",
    "ejercía": "ejercer",
    "ejercíais": "ejercer",
    "ejercíamos": "ejercer",
    "ejercían": "ejercer",
    "ejercías": "ejercer",
    "ejerza": "ejercer",
    "ejerzamos": "ejercer",
    "ejerzan": "ejercer",
    "ejerzas": "ejercer",
    "ejerzo": "ejercer",
    "ejerzáis": "ejercer",
    "ejes": "eje",
    "ejidales": "ejidal",
    "ejidos": "ejido",
    "ejotes": "ejote",
    "ejércitos": "ejército",
    "elabora": "elaborar",
    "elaboraba": "elaborar",
    "elaborabais": "elaborar",
    "elaboraban": "elaborar",
    "elaborabas": "elaborar",
    "elaborabilidades": "elaborabilidad",
    "elaborables": "elaborable",
    "elaboraciones": "elaboración",
    "elaborad": "elaborar",
    "elaborada": "elaborar",
    "elaboradas": "elaborar",
    "elaborado": "elaborar",
    "elaboradora": "elaborador",
    "elaboradoras": "elaborador",
    "elaboradores": "elaborador",
    "elaborados": "elaborar",
    "elaboramos": "elaborar",
    "elaboran": "elaborar",
    "elaborando": "elaborar",
    "elaborara": "elaborar",
    "elaborarais": "elaborar",
    "elaboraran": "elaborar",
    "elaboraras": "elaborar",
    "elaborare": "elaborar",
    "elaborareis": "elaborar",
    "elaboraremos": "elaborar",
    "elaboraren": "elaborar",
    "elaborares": "elaborar",
    "elaboraron": "elaborar",
    "elaborará": "elaborar",
    "elaborarán": "elaborar",
    "elaborarás": "elaborar",
    "elaboraré": "elaborar",
    "elaboraréis": "elaborar",
    "elaboraría": "elaborar",
    "elaboraríais": "elaborar",
    "elaboraríamos": "elaborar",
    "elaborarían": "elaborar",
    "elaborarías": "elaborar",
    "elaboras": "elaborar",
    "elaborase": "elaborar",
    "elaboraseis": "elaborar",
    "elaborasen": "elaborar",
    "elaborases": "elaborar",
    "elaboraste": "elaborar",
    "elaborasteis": "elaborar",
    "elabore": "elaborar",
    "elaboremos": "elaborar",
    "elaboren": "elaborar",
    "elabores": "elaborar",
    "elaboro": "elaborar",
    "elaborábamos": "elaborar",
    "elaboráis": "elaborar",
    "elaboráramos": "elaborar",
    "elaboráremos": "elaborar",
    "elaborásemos": "elaborar",
    "elaboré": "elaborar",
    "elaboréis": "elaborar",
    "elaboró": "elaborar",
    "elanioazules": "elanioazul",
    "elastancias": "elastancia",
    "elastasas": "elastasa",
    "elasticidades": "elasticidad",
    "elastinas": "elastina",
    "elastomérica": "elastomérico",
    "elastoméricas": "elastomérico",
    "elastoméricos": "elastomérico",
    "elastómera": "elastómero",
    "elastómeras": "elastómero",
    "elastómeros": "elastómero",
    "elastómetros": "elastómetro",
    "elaterios": "elaterio",
    "elateritas": "elaterita",
    "elayotecnias": "elayotecnia",
    "eleagnáceas": "eleagnácea",
    "eleatas": "eleata",
    "eleatismos": "eleatismo",
    "eleboreínas": "eleboreína",
    "eleccionaria": "eleccionario",
    "eleccionarias": "eleccionario",
    "eleccionarios": "eleccionario",
    "elecciones": "elección",
    "electa": "electo",
    "electas": "electo",
    "electiva": "electivo",
    "electivas": "electivo",
    "electivos": "electivo",
    "electora": "elector",
    "electorados": "electorado",
    "electorales": "electoral",
    "electoralismos": "electoralismo",
    "electoralistas": "electoralista",
    "electoras": "elector",
    "electores": "elector",
    "electos": "electo",
    "electretes": "electrete",
    "electrice": "electrizar",
    "electricemos": "electrizar",
    "electricen": "electrizar",
    "electrices": "electrizar",
    "electricidades": "electricidad",
    "electricistas": "electricista",
    "electricé": "electrizar",
    "electricéis": "electrizar",
    "electrifica": "electrificar",
    "electrificaba": "electrificar",
    "electrificabais": "electrificar",
    "electrificaban": "electrificar",
    "electrificabas": "electrificar",
    "electrificaciones": "electrificación",
    "electrificad": "electrificar",
    "electrificada": "electrificar",
    "electrificadas": "electrificar",
    "electrificado": "electrificar",
    "electrificadora": "electrificador",
    "electrificadoras": "electrificador",
    "electrificadores": "electrificador",
    "electrificados": "electrificar",
    "electrificamos": "electrificar",
    "electrifican": "electrificar",
    "electrificando": "electrificar",
    "electrificara": "electrificar",
    "electrificarais": "electrificar",
    "electrificaran": "electrificar",
    "electrificaras": "electrificar",
    "electrificare": "electrificar",
    "electrificareis": "electrificar",
    "electrificaremos": "electrificar",
    "electrificaren": "electrificar",
    "electrificares": "electrificar",
    "electrificaron": "electrificar",
    "electrificará": "electrificar",
    "electrificarán": "electrificar",
    "electrificarás": "electrificar",
    "electrificaré": "electrificar",
    "electrificaréis": "electrificar",
    "electrificaría": "electrificar",
    "electrificaríais": "electrificar",
    "electrificaríamos": "electrificar",
    "electrificarían": "electrificar",
    "electrificarías": "electrificar",
    "electrificas": "electrificar",
    "electrificase": "electrificar",
    "electrificaseis": "electrificar",
    "electrificasen": "electrificar",
    "electrificases": "electrificar",
    "electrificaste": "electrificar",
    "electrificasteis": "electrificar",
    "electrifico": "electrificar",
    "electrificábamos": "electrificar",
    "electrificáis": "electrificar",
    "electrificáramos": "electrificar",
    "electrificáremos": "electrificar",
    "electrificásemos": "electrificar",
    "electrificó": "electrificar",
    "electrifique": "electrificar",
    "electrifiquemos": "electrificar",
    "electrifiquen": "electrificar",
    "electrifiques": "electrificar",
    "electrifiqué": "electrificar",
    "electrifiquéis": "electrificar",
    "electriza": "electrizar",
    "electrizaba": "electrizar",
    "electrizabais": "electrizar",
    "electrizaban": "electrizar",
    "electrizabas": "electrizar",
    "electrizables": "electrizable",
    "electrizaciones": "electrización",
    "electrizad": "electrizar",
    "electrizada": "electrizar",
    "electrizadas": "electrizar",
    "electrizado": "electrizar",
    "electrizadora": "electrizador",
    "electrizadoras": "electrizador",
    "electrizadores": "electrizador",
    "electrizados": "electrizar",
    "electrizamos": "electrizar",
    "electrizan": "electrizar",
    "electrizando": "electrizar",
    "electrizantes": "electrizante",
    "electrizara": "electrizar",
    "electrizarais": "electrizar",
    "electrizaran": "electrizar",
    "electrizaras": "electrizar",
    "electrizare": "electrizar",
    "electrizareis": "electrizar",
    "electrizaremos": "electrizar",
    "electrizaren": "electrizar",
    "electrizares": "electrizar",
    "electrizaron": "electrizar",
    "electrizará": "electrizar",
    "electrizarán": "electrizar",
    "electrizarás": "electrizar",
    "electrizaré": "electrizar",
    "electrizaréis": "electrizar",
    "electrizaría": "electrizar",
    "electrizaríais": "electrizar",
    "electrizaríamos": "electrizar",
    "electrizarían": "electrizar",
    "electrizarías": "electrizar",
    "electrizas": "electrizar",
    "electrizase": "electrizar",
    "electrizaseis": "electrizar",
    "electrizasen": "electrizar",
    "electrizases": "electrizar",
    "electrizaste": "electrizar",
    "electrizasteis": "electrizar",
    "electrizo": "electrizar",
    "electrizábamos": "electrizar",
    "electrizáis": "electrizar",
    "electrizáramos": "electrizar",
    "electrizáremos": "electrizar",
    "electrizásemos": "electrizar",
    "electrizó": "electrizar",
    "electroacústica": "electroacústico",
    "electroacústicas": "electroacústico",
    "electroacústicos": "electroacústico",
    "electroafinidades": "electroafinidad",
    "electrobombas": "electrobomba",
    "electrocapilares": "electrocapilar",
    "electrocapilaridades": "electrocapilaridad",
    "electrocardiografías": "electrocardiografía",
    "electrocardiogramas": "electrocardiograma",
    "electrocardiógrafos": "electrocardiógrafo",
    "electrochoques": "electrochoque",
    "electrocincados": "electrocincado",
    "electrocinética": "electrocinético",
    "electrocinéticas": "electrocinético",
    "electrocinéticos": "electrocinético",
    "electrocoagulaciones": "electrocoagulación",
    "electrococleografías": "electrococleografía",
    "electroconformaciones": "electroconformación",
    "electroconformados": "electroconformado",
    "electroconvecciones": "electroconvección",
    "electroconvulsiva": "electroconvulsivo",
    "electroconvulsivas": "electroconvulsivo",
    "electroconvulsivos": "electroconvulsivo",
    "electrocromismos": "electrocromismo",
    "electrocuciones": "electrocución",
    "electrocuta": "electrocutar",
    "electrocutaba": "electrocutar",
    "electrocutabais": "electrocutar",
    "electrocutaban": "electrocutar",
    "electrocutabas": "electrocutar",
    "electrocutaciones": "electrocutación",
    "electrocutad": "electrocutar",
    "electrocutada": "electrocutar",
    "electrocutadas": "electrocutar",
    "electrocutado": "electrocutar",
    "electrocutados": "electrocutar",
    "electrocutamos": "electrocutar",
    "electrocutan": "electrocutar",
    "electrocutando": "electrocutar",
    "electrocutara": "electrocutar",
    "electrocutarais": "electrocutar",
    "electrocutaran": "electrocutar",
    "electrocutaras": "electrocutar",
    "electrocutare": "electrocutar",
    "electrocutareis": "electrocutar",
    "electrocutaremos": "electrocutar",
    "electrocutaren": "electrocutar",
    "electrocutares": "electrocutar",
    "electrocutaron": "electrocutar",
    "electrocutará": "electrocutar",
    "electrocutarán": "electrocutar",
    "electrocutarás": "electrocutar",
    "electrocutaré": "electrocutar",
    "electrocutaréis": "electrocutar",
    "electrocutaría": "electrocutar",
    "electrocutaríais": "electrocutar",
    "electrocutaríamos": "electrocutar",
    "electrocutarían": "electrocutar",
    "electrocutarías": "electrocutar",
    "electrocutas": "electrocutar",
    "electrocutase": "electrocutar",
    "electrocutaseis": "electrocutar",
    "electrocutasen": "electrocutar",
    "electrocutases": "electrocutar",
    "electrocutaste": "electrocutar",
    "electrocutasteis": "electrocutar",
    "electrocute": "electrocutar",
    "electrocutemos": "electrocutar",
    "electrocuten": "electrocutar",
    "electrocutes": "electrocutar",
    "electrocuto": "electrocutar",
    "electrocutábamos": "electrocutar",
    "electrocutáis": "electrocutar",
    "electrocutáramos": "electrocutar",
    "electrocutáremos": "electrocutar",
    "electrocutásemos": "electrocutar",
    "electrocuté": "electrocutar",
    "electrocutéis": "electrocutar",
    "electrocutó": "electrocutar",
    "electrodecantaciones": "electrodecantación",
    "electrodeposiciones": "electrodeposición",
    "electrodifusiones": "electrodifusión",
    "electrodinamómetros": "electrodinamómetro",
    "electrodinámica": "electrodinámico",
    "electrodinámicas": "electrodinámico",
    "electrodinámicos": "electrodinámico",
    "electrodisoluciones": "electrodisolución",
    "electrodoméstica": "electrodoméstico",
    "electrodomésticas": "electrodoméstico",
    "electrodomésticos": "electrodoméstico",
    "electrodos": "electrodo",
    "electroencefalogramas": "electroencefalograma",
    "electroencefalógrafos": "electroencefalógrafo",
    "electroerosiones": "electroerosión",
    "electroespectrogramas": "electroespectrograma",
    "electroestática": "electroestático",
    "electroestáticas": "electroestático",
    "electroestáticos": "electroestático",
    "electrofisiologías": "electrofisiología",
    "electroflotaciones": "electroflotación",
    "electroformaciones": "electroformación",
    "electroforética": "electroforético",
    "electroforéticas": "electroforético",
    "electroforéticos": "electroforético",
    "electrofrenos": "electrofreno",
    "electrogalvanismos": "electrogalvanismo",
    "electrogalvánica": "electrogalvánico",
    "electrogalvánicas": "electrogalvánico",
    "electrogalvánicos": "electrogalvánico",
    "electrográfica": "electrográfico",
    "electrográficas": "electrográfico",
    "electrográficos": "electrográfico",
    "electrohidráulica": "electrohidráulico",
    "electrohidráulicas": "electrohidráulico",
    "electrohidráulicos": "electrohidráulico",
    "electroimanes": "electroimán",
    "electrolice": "electrolizar",
    "electrolicemos": "electrolizar",
    "electrolicen": "electrolizar",
    "electrolices": "electrolizar",
    "electrolicé": "electrolizar",
    "electrolicéis": "electrolizar",
    "electrolitos": "electrolito",
    "electroliza": "electrolizar",
    "electrolizaba": "electrolizar",
    "electrolizabais": "electrolizar",
    "electrolizaban": "electrolizar",
    "electrolizabas": "electrolizar",
    "electrolizaciones": "electrolización",
    "electrolizad": "electrolizar",
    "electrolizada": "electrolizar",
    "electrolizadas": "electrolizar",
    "electrolizado": "electrolizar",
    "electrolizadora": "electrolizador",
    "electrolizadoras": "electrolizador",
    "electrolizadores": "electrolizador",
    "electrolizados": "electrolizar",
    "electrolizamos": "electrolizar",
    "electrolizan": "electrolizar",
    "electrolizando": "electrolizar",
    "electrolizara": "electrolizar",
    "electrolizarais": "electrolizar",
    "electrolizaran": "electrolizar",
    "electrolizaras": "electrolizar",
    "electrolizare": "electrolizar",
    "electrolizareis": "electrolizar",
    "electrolizaremos": "electrolizar",
    "electrolizaren": "electrolizar",
    "electrolizares": "electrolizar",
    "electrolizaron": "electrolizar",
    "electrolizará": "electrolizar",
    "electrolizarán": "electrolizar",
    "electrolizarás": "electrolizar",
    "electrolizaré": "electrolizar",
    "electrolizaréis": "electrolizar",
    "electrolizaría": "electrolizar",
    "electrolizaríais": "electrolizar",
    "electrolizaríamos": "electrolizar",
    "electrolizarían": "electrolizar",
    "electrolizarías": "electrolizar",
    "electrolizas": "electrolizar",
    "electrolizase": "electrolizar",
    "electrolizaseis": "electrolizar",
    "electrolizasen": "electrolizar",
    "electrolizases": "electrolizar",
    "electrolizaste": "electrolizar",
    "electrolizasteis": "electrolizar",
    "electrolizo": "electrolizar",
    "electrolizábamos": "electrolizar",
    "electrolizáis": "electrolizar",
    "electrolizáramos": "electrolizar",
    "electrolizáremos": "electrolizar",
    "electrolizásemos": "electrolizar",
    "electrolizó": "electrolizar",
    "electroluminescentes": "electroluminescente",
    "electroluminiscencias": "electroluminiscencia",
    "electroluminiscentes": "electroluminiscente",
    "electrolítica": "electrolítico",
    "electrolíticas": "electrolítico",
    "electrolíticos": "electrolítico",
    "electromagnetismos": "electromagnetismo",
    "electromagnética": "electromagnético",
    "electromagnéticas": "electromagnético",
    "electromagnéticos": "electromagnético",
    "electromecánica": "electromecánico",
    "electromecánicas": "electromecánico",
    "electromecánicos": "electromecánico",
    "electromedicinas": "electromedicina",
    "electrometalurgias": "electrometalurgia",
    "electrometalúrgica": "electrometalúrgico",
    "electrometalúrgicas": "electrometalúrgico",
    "electrometalúrgicos": "electrometalúrgico",
    "electrometeoros": "electrometeoro",
    "electrometrías": "electrometría",
    "electromicrometrías": "electromicrometría",
    "electromicrómetros": "electromicrómetro",
    "electromigraciones": "electromigración",
    "electromiografías": "electromiografía",
    "electromotores": "electromotor",
    "electromotrices": "electromotriz",
    "electromotriza": "electromotriz",
    "electromotrizas": "electromotriz",
    "electrométrica": "electrométrico",
    "electrométricas": "electrométrico",
    "electrométricos": "electrométrico",
    "electromóviles": "electromóvil",
    "electronegativa": "electronegativo",
    "electronegativas": "electronegativo",
    "electronegatividades": "electronegatividad",
    "electronegativos": "electronegativo",
    "electrones": "electrón",
    "electroneumática": "electroneumático",
    "electroneumáticas": "electroneumático",
    "electroneumáticos": "electroneumático",
    "electronistagmografías": "electronistagmografía",
    "electronistagmogramas": "electronistagmograma",
    "electronistagmógrafos": "electronistagmógrafo",
    "electronucleares": "electronuclear",
    "electronvolt": "electronvoltio",
    "electronvoltios": "electronvoltio",
    "electronvolts": "electronvolt",
    "electrooculografías": "electrooculografía",
    "electrooculogramas": "electrooculograma",
    "electroosmótica": "electroosmótico",
    "electroosmóticas": "electroosmótico",
    "electroosmóticos": "electroosmótico",
    "electropolares": "electropolar",
    "electroporaciones": "electroporación",
    "electropositiva": "electropositivo",
    "electropositivas": "electropositivo",
    "electropositivos": "electropositivo",
    "electroquímica": "electroquímico",
    "electroquímicas": "electroquímico",
    "electroquímicos": "electroquímico",
    "electrorreceptiva": "electrorreceptivo",
    "electrorreceptivas": "electrorreceptivo",
    "electrorreceptivos": "electrorreceptivo",
    "electrorretinografías": "electrorretinografía",
    "electrorretinogramas": "electrorretinograma",
    "electrorretinógrafos": "electrorretinógrafo",
    "electros": "electro",
    "electroscopios": "electroscopio",
    "electroscópica": "electroscópico",
    "electroscópicas": "electroscópico",
    "electroscópicos": "electroscópico",
    "electroshocks": "electroshock",
    "electrosiderurgias": "electrosiderurgia",
    "electrosoldaduras": "electrosoldadura",
    "electrostricciones": "electrostricción",
    "electrostática": "electrostático",
    "electrostáticas": "electrostático",
    "electrostáticos": "electrostático",
    "electrotecnias": "electrotecnia",
    "electroterapias": "electroterapia",
    "electroterapéutica": "electroterapéutico",
    "electroterapéuticas": "electroterapéutico",
    "electroterapéuticos": "electroterapéutico",
    "electrotermias": "electrotermia",
    "electroterápica": "electroterápico",
    "electroterápicas": "electroterápico",
    "electroterápicos": "electroterápico",
    "electrotipias": "electrotipia",
    "electrotonos": "electrotono",
    "electrotécnica": "electrotécnico",
    "electrotécnicas": "electrotécnico",
    "electrotécnicos": "electrotécnico",
    "electrotérmica": "electrotérmico",
    "electrotérmicas": "electrotérmico",
    "electrotérmicos": "electrotérmico",
    "electrotípica": "electrotípico",
    "electrotípicas": "electrotípico",
    "electrotípicos": "electrotípico",
    "electrotónica": "electrotónico",
    "electrotónicas": "electrotónico",
    "electrotónicos": "electrotónico",
    "electroviscosidades": "electroviscosidad",
    "electroválvulas": "electroválvula",
    "electroóptica": "electroóptico",
    "electroópticas": "electroóptico",
    "electroópticos": "electroóptico",
    "electrófila": "electrófilo",
    "electrófilas": "electrófilo",
    "electrófilos": "electrófilo",
    "electrófonos": "electrófono",
    "electróforos": "electróforo",
    "electrógena": "electrógeno",
    "electrógenas": "electrógeno",
    "electrógenos": "electrógeno",
    "electrógrafos": "electrógrafo",
    "electrólitos": "electrólito",
    "electrómetros": "electrómetro",
    "electrónica": "electrónico",
    "electrónicas": "electrónico",
    "electrónicos": "electrónico",
    "electuarios": "electuario",
    "elefanciaca": "elefancíaco",
    "elefanciacas": "elefancíaco",
    "elefanciacos": "elefancíaco",
    "elefancíaca": "elefancíaco",
    "elefancíacas": "elefancíaco",
    "elefancíacos": "elefancíaco",
    "elefancías": "elefancía",
    "elefanta": "elefante",
    "elefantas": "elefante",
    "elefantes": "elefante",
    "elefantina": "elefantino",
    "elefantinas": "elefantino",
    "elefantinos": "elefantino",
    "elefantiásica": "elefantiásico",
    "elefantiásicas": "elefantiásico",
    "elefantiásicos": "elefantiásico",
    "elefantos": "elefante",
    "elegancias": "elegancia",
    "elegantes": "elegante",
    "elegantice": "elegantizar",
    "eleganticemos": "elegantizar",
    "eleganticen": "elegantizar",
    "elegantices": "elegantizar",
    "eleganticé": "elegantizar",
    "eleganticéis": "elegantizar",
    "elegantiza": "elegantizar",
    "elegantizaba": "elegantizar",
    "elegantizabais": "elegantizar",
    "elegantizaban": "elegantizar",
    "elegantizabas": "elegantizar",
    "elegantizad": "elegantizar",
    "elegantizada": "elegantizar",
    "elegantizadas": "elegantizar",
    "elegantizado": "elegantizar",
    "elegantizados": "elegantizar",
    "elegantizamos": "elegantizar",
    "elegantizan": "elegantizar",
    "elegantizando": "elegantizar",
    "elegantizara": "elegantizar",
    "elegantizarais": "elegantizar",
    "elegantizaran": "elegantizar",
    "elegantizaras": "elegantizar",
    "elegantizare": "elegantizar",
    "elegantizareis": "elegantizar",
    "elegantizaremos": "elegantizar",
    "elegantizaren": "elegantizar",
    "elegantizares": "elegantizar",
    "elegantizaron": "elegantizar",
    "elegantizará": "elegantizar",
    "elegantizarán": "elegantizar",
    "elegantizarás": "elegantizar",
    "elegantizaré": "elegantizar",
    "elegantizaréis": "elegantizar",
    "elegantizaría": "elegantizar",
    "elegantizaríais": "elegantizar",
    "elegantizaríamos": "elegantizar",
    "elegantizarían": "elegantizar",
    "elegantizarías": "elegantizar",
    "elegantizas": "elegantizar",
    "elegantizase": "elegantizar",
    "elegantizaseis": "elegantizar",
    "elegantizasen": "elegantizar",
    "elegantizases": "elegantizar",
    "elegantizaste": "elegantizar",
    "elegantizasteis": "elegantizar",
    "elegantizo": "elegantizar",
    "elegantizábamos": "elegantizar",
    "elegantizáis": "elegantizar",
    "elegantizáramos": "elegantizar",
    "elegantizáremos": "elegantizar",
    "elegantizásemos": "elegantizar",
    "elegantizó": "elegantizar",
    "elegiaca": "elegíaco",
    "elegiacas": "elegíaco",
    "elegiacos": "elegíaco",
    "elegibilidades": "elegibilidad",
    "elegibles": "elegible",
    "elegid": "elegir",
    "elegida": "elegir",
    "elegidas": "elegir",
    "elegido": "elegir",
    "elegidos": "elegir",
    "elegimos": "elegir",
    "elegiremos": "elegir",
    "elegirá": "elegir",
    "elegirán": "elegir",
    "elegirás": "elegir",
    "elegiré": "elegir",
    "elegiréis": "elegir",
    "elegiría": "elegir",
    "elegiríais": "elegir",
    "elegiríamos": "elegir",
    "elegirían": "elegir",
    "elegirías": "elegir",
    "elegiste": "elegir",
    "elegisteis": "elegir",
    "elegí": "elegir",
    "elegía": "elegir",
    "elegíaca": "elegíaco",
    "elegíacas": "elegíaco",
    "elegíacos": "elegíaco",
    "elegíais": "elegir",
    "elegíamos": "elegir",
    "elegían": "elegir",
    "elegías": "elegir",
    "elegís": "elegir",
    "elementa": "elementar",
    "elementaba": "elementar",
    "elementabais": "elementar",
    "elementaban": "elementar",
    "elementabas": "elementar",
    "elementad": "elementar",
    "elementada": "elementar",
    "elementadas": "elementar",
    "elementado": "elementar",
    "elementados": "elementar",
    "elementales": "elemental",
    "elementalidades": "elementalidad",
    "elementamos": "elementar",
    "elementan": "elementar",
    "elementando": "elementar",
    "elementara": "elementar",
    "elementarais": "elementar",
    "elementaran": "elementar",
    "elementaras": "elementar",
    "elementare": "elementar",
    "elementareis": "elementar",
    "elementaremos": "elementar",
    "elementaren": "elementar",
    "elementares": "elementar",
    "elementaron": "elementar",
    "elementará": "elementar",
    "elementarán": "elementar",
    "elementarás": "elementar",
    "elementaré": "elementar",
    "elementaréis": "elementar",
    "elementaría": "elementar",
    "elementaríais": "elementar",
    "elementaríamos": "elementar",
    "elementarían": "elementar",
    "elementarías": "elementar",
    "elementas": "elementar",
    "elementase": "elementar",
    "elementaseis": "elementar",
    "elementasen": "elementar",
    "elementases": "elementar",
    "elementaste": "elementar",
    "elementasteis": "elementar",
    "elemente": "elementar",
    "elementemos": "elementar",
    "elementen": "elementar",
    "elementes": "elementar",
    "elemento": "elementar",
    "elementos": "elemento",
    "elementábamos": "elementar",
    "elementáis": "elementar",
    "elementáramos": "elementar",
    "elementáremos": "elementar",
    "elementásemos": "elementar",
    "elementé": "elementar",
    "elementéis": "elementar",
    "elementó": "elementar",
    "elemíes": "elemí",
    "elemís": "elemí",
    "elencos": "elenco",
    "eleolitas": "eleolita",
    "eleotecnias": "eleotecnia",
    "elepés": "elepé",
    "eles": "ele",
    "eleusina": "eleusino",
    "eleusinas": "eleusino",
    "eleusinos": "eleusino",
    "eleva": "elevar",
    "elevaba": "elevar",
    "elevabais": "elevar",
    "elevaban": "elevar",
    "elevabas": "elevar",
    "elevaciones": "elevación",
    "elevad": "elevar",
    "elevada": "elevar",
    "elevadas": "elevar",
    "elevado": "elevar",
    "elevadora": "elevador",
    "elevadoras": "elevadora",
    "elevadores": "elevador",
    "elevados": "elevar",
    "elevamos": "elevar",
    "elevan": "elevar",
    "elevando": "elevar",
    "elevara": "elevar",
    "elevarais": "elevar",
    "elevaran": "elevar",
    "elevaras": "elevar",
    "elevare": "elevar",
    "elevareis": "elevar",
    "elevaremos": "elevar",
    "elevaren": "elevar",
    "elevares": "elevar",
    "elevaron": "elevar",
    "elevará": "elevar",
    "elevarán": "elevar",
    "elevarás": "elevar",
    "elevaré": "elevar",
    "elevaréis": "elevar",
    "elevaría": "elevar",
    "elevaríais": "elevar",
    "elevaríamos": "elevar",
    "elevarían": "elevar",
    "elevarías": "elevar",
    "elevas": "elevar",
    "elevase": "elevar",
    "elevaseis": "elevar",
    "elevasen": "elevar",
    "elevases": "elevar",
    "elevaste": "elevar",
    "elevasteis": "elevar",
    "eleve": "elevar",
    "elevemos": "elevar",
    "eleven": "elevar",
    "eleves": "elevar",
    "elevo": "elevar",
    "elevábamos": "elevar",
    "eleváis": "elevar",
    "eleváramos": "elevar",
    "eleváremos": "elevar",
    "elevásemos": "elevar",
    "elevé": "elevar",
    "elevéis": "elevar",
    "elevó": "elevar",
    "eleática": "eleático",
    "eleáticas": "eleático",
    "eleáticos": "eleático",
    "elfos": "elfo",
    "elida": "elidir",
    "elidamos": "elidir",
    "elidan": "elidir",
    "elidas": "elidir",
    "elide": "elidir",
    "eliden": "elidir",
    "elides": "elidir",
    "elidid": "elidir",
    "elidida": "elidir",
    "elididas": "elidir",
    "elidido": "elidir",
    "elididos": "elidir",
    "elidiendo": "elidir",
    "elidiera": "elidir",
    "elidierais": "elidir",
    "elidieran": "elidir",
    "elidieras": "elidir",
    "elidiere": "elidir",
    "elidiereis": "elidir",
    "elidieren": "elidir",
    "elidieres": "elidir",
    "elidieron": "elidir",
    "elidiese": "elidir",
    "elidieseis": "elidir",
    "elidiesen": "elidir",
    "elidieses": "elidir",
    "elidimos": "elidir",
    "elidiremos": "elidir",
    "elidirá": "elidir",
    "elidirán": "elidir",
    "elidirás": "elidir",
    "elidiré": "elidir",
    "elidiréis": "elidir",
    "elidiría": "elidir",
    "elidiríais": "elidir",
    "elidiríamos": "elidir",
    "elidirían": "elidir",
    "elidirías": "elidir",
    "elidiste": "elidir",
    "elidisteis": "elidir",
    "elidiéramos": "elidir",
    "elidiéremos": "elidir",
    "elidiésemos": "elidir",
    "elidió": "elidir",
    "elido": "elidir",
    "elidáis": "elidir",
    "elidí": "elidir",
    "elidía": "elidir",
    "elidíais": "elidir",
    "elidíamos": "elidir",
    "elidían": "elidir",
    "elidías": "elidir",
    "elidís": "elidir",
    "elige": "elegir",
    "eligen": "elegir",
    "eliges": "elegir",
    "eligiendo": "elegir",
    "eligiera": "elegir",
    "eligierais": "elegir",
    "eligieran": "elegir",
    "eligieras": "elegir",
    "eligiere": "elegir",
    "eligiereis": "elegir",
    "eligieren": "elegir",
    "eligieres": "elegir",
    "eligieron": "elegir",
    "eligiese": "elegir",
    "eligieseis": "elegir",
    "eligiesen": "elegir",
    "eligieses": "elegir",
    "eligiéramos": "elegir",
    "eligiéremos": "elegir",
    "eligiésemos": "elegir",
    "eligió": "elegir",
    "elija": "elegir",
    "elijamos": "elegir",
    "elijan": "elegir",
    "elijas": "elegir",
    "elijo": "elegir",
    "elijáis": "elegir",
    "elimina": "eliminar",
    "eliminaba": "eliminar",
    "eliminabais": "eliminar",
    "eliminaban": "eliminar",
    "eliminabas": "eliminar",
    "eliminables": "eliminable",
    "eliminaciones": "eliminación",
    "eliminad": "eliminar",
    "eliminada": "eliminar",
    "eliminadas": "eliminar",
    "eliminado": "eliminar",
    "eliminadora": "eliminador",
    "eliminadoras": "eliminador",
    "eliminadores": "eliminador",
    "eliminados": "eliminar",
    "eliminamos": "eliminar",
    "eliminan": "eliminar",
    "eliminando": "eliminar",
    "eliminantes": "eliminante",
    "eliminara": "eliminar",
    "eliminarais": "eliminar",
    "eliminaran": "eliminar",
    "eliminaras": "eliminar",
    "eliminare": "eliminar",
    "eliminareis": "eliminar",
    "eliminaremos": "eliminar",
    "eliminaren": "eliminar",
    "eliminares": "eliminar",
    "eliminaron": "eliminar",
    "eliminará": "eliminar",
    "eliminarán": "eliminar",
    "eliminarás": "eliminar",
    "eliminaré": "eliminar",
    "eliminaréis": "eliminar",
    "eliminaría": "eliminar",
    "eliminaríais": "eliminar",
    "eliminaríamos": "eliminar",
    "eliminarían": "eliminar",
    "eliminarías": "eliminar",
    "eliminas": "eliminar",
    "eliminase": "eliminar",
    "eliminaseis": "eliminar",
    "eliminasen": "eliminar",
    "eliminases": "eliminar",
    "eliminaste": "eliminar",
    "eliminasteis": "eliminar",
    "eliminatoria": "eliminatorio",
    "eliminatorias": "eliminatorio",
    "eliminatorios": "eliminatorio",
    "elimine": "eliminar",
    "eliminemos": "eliminar",
    "eliminen": "eliminar",
    "elimines": "eliminar",
    "elimino": "eliminar",
    "eliminábamos": "eliminar",
    "elimináis": "eliminar",
    "elimináramos": "eliminar",
    "elimináremos": "eliminar",
    "eliminásemos": "eliminar",
    "eliminé": "eliminar",
    "eliminéis": "eliminar",
    "eliminó": "eliminar",
    "elipses": "elipse",
    "elipsoidales": "elipsoidal",
    "elipsoidea": "elipsoideo",
    "elipsoideas": "elipsoideo",
    "elipsoideos": "elipsoideo",
    "elipsoides": "elipsoide",
    "elipsometrías": "elipsometría",
    "elipsógrafos": "elipsógrafo",
    "elipticidades": "elipticidad",
    "eliptocitos": "eliptocito",
    "elisabetiana": "elisabetiano",
    "elisabetianas": "elisabetiano",
    "elisabetianos": "elisabetiano",
    "elisiones": "elisión",
    "elites": "elite",
    "elitismos": "elitismo",
    "elitistas": "elitista",
    "elixires": "elixir",
    "ella": "él",
    "ellas": "ellos",
    "elles": "elle",
    "ello": "él",
    "elocuciones": "elocución",
    "elocuencias": "elocuencia",
    "elocuentes": "elocuente",
    "elogia": "elogiar",
    "elogiaba": "elogiar",
    "elogiabais": "elogiar",
    "elogiaban": "elogiar",
    "elogiabas": "elogiar",
    "elogiables": "elogiable",
    "elogiad": "elogiar",
    "elogiada": "elogiar",
    "elogiadas": "elogiar",
    "elogiado": "elogiar",
    "elogiadora": "elogiador",
    "elogiadoras": "elogiador",
    "elogiadores": "elogiador",
    "elogiados": "elogiar",
    "elogiamos": "elogiar",
    "elogian": "elogiar",
    "elogiando": "elogiar",
    "elogiara": "elogiar",
    "elogiarais": "elogiar",
    "elogiaran": "elogiar",
    "elogiaras": "elogiar",
    "elogiare": "elogiar",
    "elogiareis": "elogiar",
    "elogiaremos": "elogiar",
    "elogiaren": "elogiar",
    "elogiares": "elogiar",
    "elogiaron": "elogiar",
    "elogiará": "elogiar",
    "elogiarán": "elogiar",
    "elogiarás": "elogiar",
    "elogiaré": "elogiar",
    "elogiaréis": "elogiar",
    "elogiaría": "elogiar",
    "elogiaríais": "elogiar",
    "elogiaríamos": "elogiar",
    "elogiarían": "elogiar",
    "elogiarías": "elogiar",
    "elogias": "elogiar",
    "elogiase": "elogiar",
    "elogiaseis": "elogiar",
    "elogiasen": "elogiar",
    "elogiases": "elogiar",
    "elogiaste": "elogiar",
    "elogiasteis": "elogiar",
    "elogie": "elogiar",
    "elogiemos": "elogiar",
    "elogien": "elogiar",
    "elogies": "elogiar",
    "elogio": "elogiar",
    "elogios": "elogio",
    "elogiosa": "elogioso",
    "elogiosas": "elogioso",
    "elogiosos": "elogioso",
    "elogiábamos": "elogiar",
    "elogiáis": "elogiar",
    "elogiáramos": "elogiar",
    "elogiáremos": "elogiar",
    "elogiásemos": "elogiar",
    "elogié": "elogiar",
    "elogiéis": "elogiar",
    "elogió": "elogiar",
    "elongaciones": "elongación",
    "elotes": "elote",
    "elucida": "elucidar",
    "elucidaba": "elucidar",
    "elucidabais": "elucidar",
    "elucidaban": "elucidar",
    "elucidabas": "elucidar",
    "elucidaciones": "elucidación",
    "elucidad": "elucidar",
    "elucidada": "elucidar",
    "elucidadas": "elucidar",
    "elucidado": "elucidar",
    "elucidados": "elucidar",
    "elucidamos": "elucidar",
    "elucidan": "elucidar",
    "elucidando": "elucidar",
    "elucidara": "elucidar",
    "elucidarais": "elucidar",
    "elucidaran": "elucidar",
    "elucidaras": "elucidar",
    "elucidare": "elucidar",
    "elucidareis": "elucidar",
    "elucidaremos": "elucidar",
    "elucidaren": "elucidar",
    "elucidares": "elucidar",
    "elucidaron": "elucidar",
    "elucidará": "elucidar",
    "elucidarán": "elucidar",
    "elucidarás": "elucidar",
    "elucidaré": "elucidar",
    "elucidaréis": "elucidar",
    "elucidaría": "elucidar",
    "elucidaríais": "elucidar",
    "elucidaríamos": "elucidar",
    "elucidarían": "elucidar",
    "elucidarías": "elucidar",
    "elucidas": "elucidar",
    "elucidase": "elucidar",
    "elucidaseis": "elucidar",
    "elucidasen": "elucidar",
    "elucidases": "elucidar",
    "elucidaste": "elucidar",
    "elucidasteis": "elucidar",
    "elucide": "elucidar",
    "elucidemos": "elucidar",
    "eluciden": "elucidar",
    "elucides": "elucidar",
    "elucido": "elucidar",
    "elucidábamos": "elucidar",
    "elucidáis": "elucidar",
    "elucidáramos": "elucidar",
    "elucidáremos": "elucidar",
    "elucidásemos": "elucidar",
    "elucidé": "elucidar",
    "elucidéis": "elucidar",
    "elucidó": "elucidar",
    "eluciones": "elución",
    "eluctables": "eluctable",
    "elucubra": "elucubrar",
    "elucubraba": "elucubrar",
    "elucubrabais": "elucubrar",
    "elucubraban": "elucubrar",
    "elucubrabas": "elucubrar",
    "elucubraciones": "elucubración",
    "elucubrad": "elucubrar",
    "elucubrada": "elucubrar",
    "elucubradas": "elucubrar",
    "elucubrado": "elucubrar",
    "elucubradora": "elucubrador",
    "elucubradoras": "elucubrador",
    "elucubradores": "elucubrador",
    "elucubrados": "elucubrar",
    "elucubramos": "elucubrar",
    "elucubran": "elucubrar",
    "elucubrando": "elucubrar",
    "elucubrara": "elucubrar",
    "elucubrarais": "elucubrar",
    "elucubraran": "elucubrar",
    "elucubraras": "elucubrar",
    "elucubrare": "elucubrar",
    "elucubrareis": "elucubrar",
    "elucubraremos": "elucubrar",
    "elucubraren": "elucubrar",
    "elucubrares": "elucubrar",
    "elucubraron": "elucubrar",
    "elucubrará": "elucubrar",
    "elucubrarán": "elucubrar",
    "elucubrarás": "elucubrar",
    "elucubraré": "elucubrar",
    "elucubraréis": "elucubrar",
    "elucubraría": "elucubrar",
    "elucubraríais": "elucubrar",
    "elucubraríamos": "elucubrar",
    "elucubrarían": "elucubrar",
    "elucubrarías": "elucubrar",
    "elucubras": "elucubrar",
    "elucubrase": "elucubrar",
    "elucubraseis": "elucubrar",
    "elucubrasen": "elucubrar",
    "elucubrases": "elucubrar",
    "elucubraste": "elucubrar",
    "elucubrasteis": "elucubrar",
    "elucubre": "elucubrar",
    "elucubremos": "elucubrar",
    "elucubren": "elucubrar",
    "elucubres": "elucubrar",
    "elucubro": "elucubrar",
    "elucubrábamos": "elucubrar",
    "elucubráis": "elucubrar",
    "elucubráramos": "elucubrar",
    "elucubráremos": "elucubrar",
    "elucubrásemos": "elucubrar",
    "elucubré": "elucubrar",
    "elucubréis": "elucubrar",
    "elucubró": "elucubrar",
    "eluda": "eludir",
    "eludamos": "eludir",
    "eludan": "eludir",
    "eludas": "eludir",
    "elude": "eludir",
    "eluden": "eludir",
    "eludes": "eludir",
    "eludibles": "eludible",
    "eludid": "eludir",
    "eludida": "eludir",
    "eludidas": "eludir",
    "eludido": "eludir",
    "eludidos": "eludir",
    "eludiendo": "eludir",
    "eludiera": "eludir",
    "eludierais": "eludir",
    "eludieran": "eludir",
    "eludieras": "eludir",
    "eludiere": "eludir",
    "eludiereis": "eludir",
    "eludieren": "eludir",
    "eludieres": "eludir",
    "eludieron": "eludir",
    "eludiese": "eludir",
    "eludieseis": "eludir",
    "eludiesen": "eludir",
    "eludieses": "eludir",
    "eludimos": "eludir",
    "eludiremos": "eludir",
    "eludirá": "eludir",
    "eludirán": "eludir",
    "eludirás": "eludir",
    "eludiré": "eludir",
    "eludiréis": "eludir",
    "eludiría": "eludir",
    "eludiríais": "eludir",
    "eludiríamos": "eludir",
    "eludirían": "eludir",
    "eludirías": "eludir",
    "eludiste": "eludir",
    "eludisteis": "eludir",
    "eludiéramos": "eludir",
    "eludiéremos": "eludir",
    "eludiésemos": "eludir",
    "eludió": "eludir",
    "eludo": "eludir",
    "eludáis": "eludir",
    "eludí": "eludir",
    "eludía": "eludir",
    "eludíais": "eludir",
    "eludíamos": "eludir",
    "eludían": "eludir",
    "eludías": "eludir",
    "eludís": "eludir",
    "elusiones": "elusión",
    "elusiva": "elusivo",
    "elusivas": "elusivo",
    "elusivos": "elusivo",
    "elutriaciones": "elutriación",
    "eluviaciones": "eluviación",
    "eluviales": "eluvial",
    "eluviones": "eluvión",
    "eluídos": "eluído",
    "elzeviriana": "elzeviriano",
    "elzevirianas": "elzeviriano",
    "elzevirianos": "elzeviriano",
    "elástica": "elástico",
    "elásticas": "elástico",
    "elásticos": "elástico",
    "eléboros": "eléboro",
    "eléctrica": "eléctrico",
    "eléctricas": "eléctrico",
    "eléctricos": "eléctrico",
    "elíptica": "elíptico",
    "elípticas": "elíptico",
    "elípticos": "elíptico",
    "elísea": "elíseo",
    "elíseas": "elíseo",
    "elíseos": "elíseo",
    "elíxires": "elíxir",
    "emagramas": "emagrama",
    "emana": "emanar",
    "emanaba": "emanar",
    "emanabais": "emanar",
    "emanaban": "emanar",
    "emanabas": "emanar",
    "emanaciones": "emanación",
    "emanad": "emanar",
    "emanada": "emanar",
    "emanadas": "emanar",
    "emanado": "emanar",
    "emanados": "emanar",
    "emanamos": "emanar",
    "emanan": "emanar",
    "emanando": "emanar",
    "emanantismos": "emanantismo",
    "emanantistas": "emanantista",
    "emanara": "emanar",
    "emanarais": "emanar",
    "emanaran": "emanar",
    "emanaras": "emanar",
    "emanare": "emanar",
    "emanareis": "emanar",
    "emanaremos": "emanar",
    "emanaren": "emanar",
    "emanares": "emanar",
    "emanaron": "emanar",
    "emanará": "emanar",
    "emanarán": "emanar",
    "emanarás": "emanar",
    "emanaré": "emanar",
    "emanaréis": "emanar",
    "emanaría": "emanar",
    "emanaríais": "emanar",
    "emanaríamos": "emanar",
    "emanarían": "emanar",
    "emanarías": "emanar",
    "emanas": "emanar",
    "emanase": "emanar",
    "emanaseis": "emanar",
    "emanasen": "emanar",
    "emanases": "emanar",
    "emanaste": "emanar",
    "emanasteis": "emanar",
    "emancipa": "emancipar",
    "emancipaba": "emancipar",
    "emancipabais": "emancipar",
    "emancipaban": "emancipar",
    "emancipabas": "emancipar",
    "emancipaciones": "emancipación",
    "emancipad": "emancipar",
    "emancipada": "emancipar",
    "emancipadas": "emancipar",
    "emancipado": "emancipar",
    "emancipadora": "emancipador",
    "emancipadoras": "emancipador",
    "emancipadores": "emancipador",
    "emancipados": "emancipar",
    "emancipamos": "emancipar",
    "emancipan": "emancipar",
    "emancipando": "emancipar",
    "emancipara": "emancipar",
    "emanciparais": "emancipar",
    "emanciparan": "emancipar",
    "emanciparas": "emancipar",
    "emancipare": "emancipar",
    "emancipareis": "emancipar",
    "emanciparemos": "emancipar",
    "emanciparen": "emancipar",
    "emancipares": "emancipar",
    "emanciparon": "emancipar",
    "emancipará": "emancipar",
    "emanciparán": "emancipar",
    "emanciparás": "emancipar",
    "emanciparé": "emancipar",
    "emanciparéis": "emancipar",
    "emanciparía": "emancipar",
    "emanciparíais": "emancipar",
    "emanciparíamos": "emancipar",
    "emanciparían": "emancipar",
    "emanciparías": "emancipar",
    "emancipas": "emancipar",
    "emancipase": "emancipar",
    "emancipaseis": "emancipar",
    "emancipasen": "emancipar",
    "emancipases": "emancipar",
    "emancipaste": "emancipar",
    "emancipasteis": "emancipar",
    "emancipe": "emancipar",
    "emancipemos": "emancipar",
    "emancipen": "emancipar",
    "emancipes": "emancipar",
    "emancipo": "emancipar",
    "emancipábamos": "emancipar",
    "emancipáis": "emancipar",
    "emancipáramos": "emancipar",
    "emancipáremos": "emancipar",
    "emancipásemos": "emancipar",
    "emancipé": "emancipar",
    "emancipéis": "emancipar",
    "emancipó": "emancipar",
    "emane": "emanar",
    "emanemos": "emanar",
    "emanen": "emanar",
    "emanes": "emanar",
    "emano": "emanar",
    "emanábamos": "emanar",
    "emanáis": "emanar",
    "emanáramos": "emanar",
    "emanáremos": "emanar",
    "emanásemos": "emanar",
    "emané": "emanar",
    "emanéis": "emanar",
    "emanó": "emanar",
    "emascula": "emascular",
    "emasculaba": "emascular",
    "emasculabais": "emascular",
    "emasculaban": "emascular",
    "emasculabas": "emascular",
    "emasculaciones": "emasculación",
    "emasculad": "emascular",
    "emasculada": "emascular",
    "emasculadas": "emascular",
    "emasculado": "emascular",
    "emasculados": "emascular",
    "emasculamos": "emascular",
    "emasculan": "emascular",
    "emasculando": "emascular",
    "emasculara": "emascular",
    "emascularais": "emascular",
    "emascularan": "emascular",
    "emascularas": "emascular",
    "emasculare": "emascular",
    "emasculareis": "emascular",
    "emascularemos": "emascular",
    "emascularen": "emascular",
    "emasculares": "emascular",
    "emascularon": "emascular",
    "emasculará": "emascular",
    "emascularán": "emascular",
    "emascularás": "emascular",
    "emascularé": "emascular",
    "emascularéis": "emascular",
    "emascularía": "emascular",
    "emascularíais": "emascular",
    "emascularíamos": "emascular",
    "emascularían": "emascular",
    "emascularías": "emascular",
    "emasculas": "emascular",
    "emasculase": "emascular",
    "emasculaseis": "emascular",
    "emasculasen": "emascular",
    "emasculases": "emascular",
    "emasculaste": "emascular",
    "emasculasteis": "emascular",
    "emascule": "emascular",
    "emasculemos": "emascular",
    "emasculen": "emascular",
    "emascules": "emascular",
    "emasculo": "emascular",
    "emasculábamos": "emascular",
    "emasculáis": "emascular",
    "emasculáramos": "emascular",
    "emasculáremos": "emascular",
    "emasculásemos": "emascular",
    "emasculé": "emascular",
    "emasculéis": "emascular",
    "emasculó": "emascular",
    "embabiamientos": "embabiamiento",
    "embabuca": "embabucar",
    "embabucaba": "embabucar",
    "embabucabais": "embabucar",
    "embabucaban": "embabucar",
    "embabucabas": "embabucar",
    "embabucad": "embabucar",
    "embabucada": "embabucar",
    "embabucadas": "embabucar",
    "embabucado": "embabucar",
    "embabucados": "embabucar",
    "embabucamos": "embabucar",
    "embabucan": "embabucar",
    "embabucando": "embabucar",
    "embabucara": "embabucar",
    "embabucarais": "embabucar",
    "embabucaran": "embabucar",
    "embabucaras": "embabucar",
    "embabucare": "embabucar",
    "embabucareis": "embabucar",
    "embabucaremos": "embabucar",
    "embabucaren": "embabucar",
    "embabucares": "embabucar",
    "embabucaron": "embabucar",
    "embabucará": "embabucar",
    "embabucarán": "embabucar",
    "embabucarás": "embabucar",
    "embabucaré": "embabucar",
    "embabucaréis": "embabucar",
    "embabucaría": "embabucar",
    "embabucaríais": "embabucar",
    "embabucaríamos": "embabucar",
    "embabucarían": "embabucar",
    "embabucarías": "embabucar",
    "embabucas": "embabucar",
    "embabucase": "embabucar",
    "embabucaseis": "embabucar",
    "embabucasen": "embabucar",
    "embabucases": "embabucar",
    "embabucaste": "embabucar",
    "embabucasteis": "embabucar",
    "embabuco": "embabucar",
    "embabucábamos": "embabucar",
    "embabucáis": "embabucar",
    "embabucáramos": "embabucar",
    "embabucáremos": "embabucar",
    "embabucásemos": "embabucar",
    "embabucó": "embabucar",
    "embabuque": "embabucar",
    "embabuquemos": "embabucar",
    "embabuquen": "embabucar",
    "embabuques": "embabucar",
    "embabuqué": "embabucar",
    "embabuquéis": "embabucar",
    "embace": "embazar",
    "embacemos": "embazar",
    "embacen": "embazar",
    "embaces": "embazar",
    "embacé": "embazar",
    "embacéis": "embazar",
    "embadurna": "embadurnar",
    "embadurnaba": "embadurnar",
    "embadurnabais": "embadurnar",
    "embadurnaban": "embadurnar",
    "embadurnabas": "embadurnar",
    "embadurnad": "embadurnar",
    "embadurnada": "embadurnar",
    "embadurnadas": "embadurnar",
    "embadurnado": "embadurnar",
    "embadurnadora": "embadurnador",
    "embadurnadoras": "embadurnador",
    "embadurnadores": "embadurnador",
    "embadurnados": "embadurnar",
    "embadurnamientos": "embadurnamiento",
    "embadurnamos": "embadurnar",
    "embadurnan": "embadurnar",
    "embadurnando": "embadurnar",
    "embadurnara": "embadurnar",
    "embadurnarais": "embadurnar",
    "embadurnaran": "embadurnar",
    "embadurnaras": "embadurnar",
    "embadurnare": "embadurnar",
    "embadurnareis": "embadurnar",
    "embadurnaremos": "embadurnar",
    "embadurnaren": "embadurnar",
    "embadurnares": "embadurnar",
    "embadurnaron": "embadurnar",
    "embadurnará": "embadurnar",
    "embadurnarán": "embadurnar",
    "embadurnarás": "embadurnar",
    "embadurnaré": "embadurnar",
    "embadurnaréis": "embadurnar",
    "embadurnaría": "embadurnar",
    "embadurnaríais": "embadurnar",
    "embadurnaríamos": "embadurnar",
    "embadurnarían": "embadurnar",
    "embadurnarías": "embadurnar",
    "embadurnas": "embadurnar",
    "embadurnase": "embadurnar",
    "embadurnaseis": "embadurnar",
    "embadurnasen": "embadurnar",
    "embadurnases": "embadurnar",
    "embadurnaste": "embadurnar",
    "embadurnasteis": "embadurnar",
    "embadurne": "embadurnar",
    "embadurnemos": "embadurnar",
    "embadurnen": "embadurnar",
    "embadurnes": "embadurnar",
    "embadurno": "embadurnar",
    "embadurnábamos": "embadurnar",
    "embadurnáis": "embadurnar",
    "embadurnáramos": "embadurnar",
    "embadurnáremos": "embadurnar",
    "embadurnásemos": "embadurnar",
    "embadurné": "embadurnar",
    "embadurnéis": "embadurnar",
    "embadurnó": "embadurnar",
    "embaida": "embair",
    "embaidas": "embair",
    "embairemos": "embair",
    "embairá": "embair",
    "embairán": "embair",
    "embairás": "embair",
    "embairé": "embair",
    "embairéis": "embair",
    "embairía": "embair",
    "embairíais": "embair",
    "embairíamos": "embair",
    "embairían": "embair",
    "embairías": "embair",
    "embajadas": "embajada",
    "embajadora": "embajador",
    "embajadoras": "embajador",
    "embajadores": "embajador",
    "embala": "embalar",
    "embalaba": "embalar",
    "embalabais": "embalar",
    "embalaban": "embalar",
    "embalabas": "embalar",
    "embalad": "embalar",
    "embalada": "embalar",
    "embaladas": "embalar",
    "embalado": "embalar",
    "embaladora": "embalador",
    "embaladoras": "embaladora",
    "embaladores": "embalador",
    "embalados": "embalar",
    "embalajes": "embalaje",
    "embalamientos": "embalamiento",
    "embalamos": "embalar",
    "embalan": "embalar",
    "embalando": "embalar",
    "embalara": "embalar",
    "embalarais": "embalar",
    "embalaran": "embalar",
    "embalaras": "embalar",
    "embalare": "embalar",
    "embalareis": "embalar",
    "embalaremos": "embalar",
    "embalaren": "embalar",
    "embalares": "embalar",
    "embalaron": "embalar",
    "embalará": "embalar",
    "embalarán": "embalar",
    "embalarás": "embalar",
    "embalaré": "embalar",
    "embalaréis": "embalar",
    "embalaría": "embalar",
    "embalaríais": "embalar",
    "embalaríamos": "embalar",
    "embalarían": "embalar",
    "embalarías": "embalar",
    "embalas": "embalar",
    "embalase": "embalar",
    "embalaseis": "embalar",
    "embalasen": "embalar",
    "embalases": "embalar",
    "embalaste": "embalar",
    "embalasteis": "embalar",
    "embaldosa": "embaldosar",
    "embaldosaba": "embaldosar",
    "embaldosabais": "embaldosar",
    "embaldosaban": "embaldosar",
    "embaldosabas": "embaldosar",
    "embaldosad": "embaldosar",
    "embaldosada": "embaldosar",
    "embaldosadas": "embaldosar",
    "embaldosado": "embaldosar",
    "embaldosadores": "embaldosador",
    "embaldosados": "embaldosar",
    "embaldosaduras": "embaldosadura",
    "embaldosamos": "embaldosar",
    "embaldosan": "embaldosar",
    "embaldosando": "embaldosar",
    "embaldosara": "embaldosar",
    "embaldosarais": "embaldosar",
    "embaldosaran": "embaldosar",
    "embaldosaras": "embaldosar",
    "embaldosare": "embaldosar",
    "embaldosareis": "embaldosar",
    "embaldosaremos": "embaldosar",
    "embaldosaren": "embaldosar",
    "embaldosares": "embaldosar",
    "embaldosaron": "embaldosar",
    "embaldosará": "embaldosar",
    "embaldosarán": "embaldosar",
    "embaldosarás": "embaldosar",
    "embaldosaré": "embaldosar",
    "embaldosaréis": "embaldosar",
    "embaldosaría": "embaldosar",
    "embaldosaríais": "embaldosar",
    "embaldosaríamos": "embaldosar",
    "embaldosarían": "embaldosar",
    "embaldosarías": "embaldosar",
    "embaldosas": "embaldosar",
    "embaldosase": "embaldosar",
    "embaldosaseis": "embaldosar",
    "embaldosasen": "embaldosar",
    "embaldosases": "embaldosar",
    "embaldosaste": "embaldosar",
    "embaldosasteis": "embaldosar",
    "embaldose": "embaldosar",
    "embaldosemos": "embaldosar",
    "embaldosen": "embaldosar",
    "embaldoses": "embaldosar",
    "embaldoso": "embaldosar",
    "embaldosábamos": "embaldosar",
    "embaldosáis": "embaldosar",
    "embaldosáramos": "embaldosar",
    "embaldosáremos": "embaldosar",
    "embaldosásemos": "embaldosar",
    "embaldosé": "embaldosar",
    "embaldoséis": "embaldosar",
    "embaldosó": "embaldosar",
    "embale": "embalar",
    "embalemos": "embalar",
    "embalen": "embalar",
    "embales": "embalar",
    "emballena": "emballenar",
    "emballenaba": "emballenar",
    "emballenabais": "emballenar",
    "emballenaban": "emballenar",
    "emballenabas": "emballenar",
    "emballenad": "emballenar",
    "emballenada": "emballenar",
    "emballenadas": "emballenar",
    "emballenado": "emballenar",
    "emballenados": "emballenar",
    "emballenamos": "emballenar",
    "emballenan": "emballenar",
    "emballenando": "emballenar",
    "emballenara": "emballenar",
    "emballenarais": "emballenar",
    "emballenaran": "emballenar",
    "emballenaras": "emballenar",
    "emballenare": "emballenar",
    "emballenareis": "emballenar",
    "emballenaremos": "emballenar",
    "emballenaren": "emballenar",
    "emballenares": "emballenar",
    "emballenaron": "emballenar",
    "emballenará": "emballenar",
    "emballenarán": "emballenar",
    "emballenarás": "emballenar",
    "emballenaré": "emballenar",
    "emballenaréis": "emballenar",
    "emballenaría": "emballenar",
    "emballenaríais": "emballenar",
    "emballenaríamos": "emballenar",
    "emballenarían": "emballenar",
    "emballenarías": "emballenar",
    "emballenas": "emballenar",
    "emballenase": "emballenar",
    "emballenaseis": "emballenar",
    "emballenasen": "emballenar",
    "emballenases": "emballenar",
    "emballenaste": "emballenar",
    "emballenasteis": "emballenar",
    "emballene": "emballenar",
    "emballenemos": "emballenar",
    "emballenen": "emballenar",
    "emballenes": "emballenar",
    "emballeno": "emballenar",
    "emballenábamos": "emballenar",
    "emballenáis": "emballenar",
    "emballenáramos": "emballenar",
    "emballenáremos": "emballenar",
    "emballenásemos": "emballenar",
    "emballené": "emballenar",
    "emballenéis": "emballenar",
    "emballenó": "emballenar",
    "emballestada": "emballestado",
    "emballestadas": "emballestado",
    "emballestados": "emballestado",
    "embalo": "embalar",
    "embalos": "embalo",
    "embalsa": "embalsar",
    "embalsaba": "embalsar",
    "embalsabais": "embalsar",
    "embalsaban": "embalsar",
    "embalsabas": "embalsar",
    "embalsad": "embalsar",
    "embalsada": "embalsar",
    "embalsadas": "embalsar",
    "embalsaderos": "embalsadero",
    "embalsado": "embalsar",
    "embalsados": "embalsar",
    "embalsama": "embalsamar",
    "embalsamaba": "embalsamar",
    "embalsamabais": "embalsamar",
    "embalsamaban": "embalsamar",
    "embalsamabas": "embalsamar",
    "embalsamad": "embalsamar",
    "embalsamada": "embalsamar",
    "embalsamadas": "embalsamar",
    "embalsamado": "embalsamar",
    "embalsamadora": "embalsamador",
    "embalsamadoras": "embalsamador",
    "embalsamadores": "embalsamador",
    "embalsamados": "embalsamar",
    "embalsamamientos": "embalsamamiento",
    "embalsamamos": "embalsamar",
    "embalsaman": "embalsamar",
    "embalsamando": "embalsamar",
    "embalsamara": "embalsamar",
    "embalsamarais": "embalsamar",
    "embalsamaran": "embalsamar",
    "embalsamaras": "embalsamar",
    "embalsamare": "embalsamar",
    "embalsamareis": "embalsamar",
    "embalsamaremos": "embalsamar",
    "embalsamaren": "embalsamar",
    "embalsamares": "embalsamar",
    "embalsamaron": "embalsamar",
    "embalsamará": "embalsamar",
    "embalsamarán": "embalsamar",
    "embalsamarás": "embalsamar",
    "embalsamaré": "embalsamar",
    "embalsamaréis": "embalsamar",
    "embalsamaría": "embalsamar",
    "embalsamaríais": "embalsamar",
    "embalsamaríamos": "embalsamar",
    "embalsamarían": "embalsamar",
    "embalsamarías": "embalsamar",
    "embalsamas": "embalsamar",
    "embalsamase": "embalsamar",
    "embalsamaseis": "embalsamar",
    "embalsamasen": "embalsamar",
    "embalsamases": "embalsamar",
    "embalsamaste": "embalsamar",
    "embalsamasteis": "embalsamar",
    "embalsame": "embalsamar",
    "embalsamemos": "embalsamar",
    "embalsamen": "embalsamar",
    "embalsames": "embalsamar",
    "embalsamientos": "embalsamiento",
    "embalsamo": "embalsamar",
    "embalsamos": "embalsar",
    "embalsamábamos": "embalsamar",
    "embalsamáis": "embalsamar",
    "embalsamáramos": "embalsamar",
    "embalsamáremos": "embalsamar",
    "embalsamásemos": "embalsamar",
    "embalsamé": "embalsamar",
    "embalsaméis": "embalsamar",
    "embalsamó": "embalsamar",
    "embalsan": "embalsar",
    "embalsando": "embalsar",
    "embalsara": "embalsar",
    "embalsarais": "embalsar",
    "embalsaran": "embalsar",
    "embalsaras": "embalsar",
    "embalsare": "embalsar",
    "embalsareis": "embalsar",
    "embalsaremos": "embalsar",
    "embalsaren": "embalsar",
    "embalsares": "embalsar",
    "embalsaron": "embalsar",
    "embalsará": "embalsar",
    "embalsarán": "embalsar",
    "embalsarás": "embalsar",
    "embalsaré": "embalsar",
    "embalsaréis": "embalsar",
    "embalsaría": "embalsar",
    "embalsaríais": "embalsar",
    "embalsaríamos": "embalsar",
    "embalsarían": "embalsar",
    "embalsarías": "embalsar",
    "embalsas": "embalsar",
    "embalsase": "embalsar",
    "embalsaseis": "embalsar",
    "embalsasen": "embalsar",
    "embalsases": "embalsar",
    "embalsaste": "embalsar",
    "embalsasteis": "embalsar",
    "embalse": "embalsar",
    "embalsemos": "embalsar",
    "embalsen": "embalsar",
    "embalses": "embalse",
    "embalso": "embalsar",
    "embalsábamos": "embalsar",
    "embalsáis": "embalsar",
    "embalsáramos": "embalsar",
    "embalsáremos": "embalsar",
    "embalsásemos": "embalsar",
    "embalsé": "embalsar",
    "embalséis": "embalsar",
    "embalsó": "embalsar",
    "embalábamos": "embalar",
    "embaláis": "embalar",
    "embaláramos": "embalar",
    "embaláremos": "embalar",
    "embalásemos": "embalar",
    "embalé": "embalar",
    "embaléis": "embalar",
    "embaló": "embalar",
    "embanasta": "embanastar",
    "embanastaba": "embanastar",
    "embanastabais": "embanastar",
    "embanastaban": "embanastar",
    "embanastabas": "embanastar",
    "embanastad": "embanastar",
    "embanastada": "embanastar",
    "embanastadas": "embanastar",
    "embanastado": "embanastar",
    "embanastados": "embanastar",
    "embanastamos": "embanastar",
    "embanastan": "embanastar",
    "embanastando": "embanastar",
    "embanastara": "embanastar",
    "embanastarais": "embanastar",
    "embanastaran": "embanastar",
    "embanastaras": "embanastar",
    "embanastare": "embanastar",
    "embanastareis": "embanastar",
    "embanastaremos": "embanastar",
    "embanastaren": "embanastar",
    "embanastares": "embanastar",
    "embanastaron": "embanastar",
    "embanastará": "embanastar",
    "embanastarán": "embanastar",
    "embanastarás": "embanastar",
    "embanastaré": "embanastar",
    "embanastaréis": "embanastar",
    "embanastaría": "embanastar",
    "embanastaríais": "embanastar",
    "embanastaríamos": "embanastar",
    "embanastarían": "embanastar",
    "embanastarías": "embanastar",
    "embanastas": "embanastar",
    "embanastase": "embanastar",
    "embanastaseis": "embanastar",
    "embanastasen": "embanastar",
    "embanastases": "embanastar",
    "embanastaste": "embanastar",
    "embanastasteis": "embanastar",
    "embanaste": "embanastar",
    "embanastemos": "embanastar",
    "embanasten": "embanastar",
    "embanastes": "embanastar",
    "embanasto": "embanastar",
    "embanastábamos": "embanastar",
    "embanastáis": "embanastar",
    "embanastáramos": "embanastar",
    "embanastáremos": "embanastar",
    "embanastásemos": "embanastar",
    "embanasté": "embanastar",
    "embanastéis": "embanastar",
    "embanastó": "embanastar",
    "embanca": "embancar",
    "embancaba": "embancar",
    "embancabais": "embancar",
    "embancaban": "embancar",
    "embancabas": "embancar",
    "embancad": "embancar",
    "embancada": "embancar",
    "embancadas": "embancar",
    "embancado": "embancar",
    "embancados": "embancar",
    "embancamos": "embancar",
    "embancan": "embancar",
    "embancando": "embancar",
    "embancara": "embancar",
    "embancarais": "embancar",
    "embancaran": "embancar",
    "embancaras": "embancar",
    "embancare": "embancar",
    "embancareis": "embancar",
    "embancaremos": "embancar",
    "embancaren": "embancar",
    "embancares": "embancar",
    "embancaron": "embancar",
    "embancará": "embancar",
    "embancarán": "embancar",
    "embancarás": "embancar",
    "embancaré": "embancar",
    "embancaréis": "embancar",
    "embancaría": "embancar",
    "embancaríais": "embancar",
    "embancaríamos": "embancar",
    "embancarían": "embancar",
    "embancarías": "embancar",
    "embancas": "embancar",
    "embancase": "embancar",
    "embancaseis": "embancar",
    "embancasen": "embancar",
    "embancases": "embancar",
    "embancaste": "embancar",
    "embancasteis": "embancar",
    "embanco": "embancar",
    "embancábamos": "embancar",
    "embancáis": "embancar",
    "embancáramos": "embancar",
    "embancáremos": "embancar",
    "embancásemos": "embancar",
    "embancó": "embancar",
    "embandera": "embanderar",
    "embanderaba": "embanderar",
    "embanderabais": "embanderar",
    "embanderaban": "embanderar",
    "embanderabas": "embanderar",
    "embanderad": "embanderar",
    "embanderada": "embanderar",
    "embanderadas": "embanderar",
    "embanderado": "embanderar",
    "embanderados": "embanderar",
    "embanderamos": "embanderar",
    "embanderan": "embanderar",
    "embanderando": "embanderar",
    "embanderara": "embanderar",
    "embanderarais": "embanderar",
    "embanderaran": "embanderar",
    "embanderaras": "embanderar",
    "embanderare": "embanderar",
    "embanderareis": "embanderar",
    "embanderaremos": "embanderar",
    "embanderaren": "embanderar",
    "embanderares": "embanderar",
    "embanderaron": "embanderar",
    "embanderará": "embanderar",
    "embanderarán": "embanderar",
    "embanderarás": "embanderar",
    "embanderaré": "embanderar",
    "embanderaréis": "embanderar",
    "embanderaría": "embanderar",
    "embanderaríais": "embanderar",
    "embanderaríamos": "embanderar",
    "embanderarían": "embanderar",
    "embanderarías": "embanderar",
    "embanderas": "embanderar",
    "embanderase": "embanderar",
    "embanderaseis": "embanderar",
    "embanderasen": "embanderar",
    "embanderases": "embanderar",
    "embanderaste": "embanderar",
    "embanderasteis": "embanderar",
    "embandere": "embanderar",
    "embanderemos": "embanderar",
    "embanderen": "embanderar",
    "embanderes": "embanderar",
    "embandero": "embanderar",
    "embanderábamos": "embanderar",
    "embanderáis": "embanderar",
    "embanderáramos": "embanderar",
    "embanderáremos": "embanderar",
    "embanderásemos": "embanderar",
    "embanderé": "embanderar",
    "embanderéis": "embanderar",
    "embanderó": "embanderar",
    "embanque": "embancar",
    "embanquemos": "embancar",
    "embanquen": "embancar",
    "embanques": "embancar",
    "embanqué": "embancar",
    "embanquéis": "embancar",
    "embarace": "embarazar",
    "embaracemos": "embarazar",
    "embaracen": "embarazar",
    "embaraces": "embarazar",
    "embaracé": "embarazar",
    "embaracéis": "embarazar",
    "embaraza": "embarazar",
    "embarazaba": "embarazar",
    "embarazabais": "embarazar",
    "embarazaban": "embarazar",
    "embarazabas": "embarazar",
    "embarazad": "embarazar",
    "embarazada": "embarazar",
    "embarazadas": "embarazar",
    "embarazado": "embarazar",
    "embarazadora": "embarazador",
    "embarazadoras": "embarazador",
    "embarazadores": "embarazador",
    "embarazados": "embarazar",
    "embarazamos": "embarazar",
    "embarazan": "embarazar",
    "embarazando": "embarazar",
    "embarazara": "embarazar",
    "embarazarais": "embarazar",
    "embarazaran": "embarazar",
    "embarazaras": "embarazar",
    "embarazare": "embarazar",
    "embarazareis": "embarazar",
    "embarazaremos": "embarazar",
    "embarazaren": "embarazar",
    "embarazares": "embarazar",
    "embarazaron": "embarazar",
    "embarazará": "embarazar",
    "embarazarán": "embarazar",
    "embarazarás": "embarazar",
    "embarazaré": "embarazar",
    "embarazaréis": "embarazar",
    "embarazaría": "embarazar",
    "embarazaríais": "embarazar",
    "embarazaríamos": "embarazar",
    "embarazarían": "embarazar",
    "embarazarías": "embarazar",
    "embarazas": "embarazar",
    "embarazase": "embarazar",
    "embarazaseis": "embarazar",
    "embarazasen": "embarazar",
    "embarazases": "embarazar",
    "embarazaste": "embarazar",
    "embarazasteis": "embarazar",
    "embarazo": "embarazar",
    "embarazos": "embarazo",
    "embarazosa": "embarazoso",
    "embarazosas": "embarazoso",
    "embarazosos": "embarazoso",
    "embarazábamos": "embarazar",
    "embarazáis": "embarazar",
    "embarazáramos": "embarazar",
    "embarazáremos": "embarazar",
    "embarazásemos": "embarazar",
    "embarazó": "embarazar",
    "embarbasca": "embarbascar",
    "embarbascaba": "embarbascar",
    "embarbascabais": "embarbascar",
    "embarbascaban": "embarbascar",
    "embarbascabas": "embarbascar",
    "embarbascad": "embarbascar",
    "embarbascada": "embarbascar",
    "embarbascadas": "embarbascar",
    "embarbascado": "embarbascar",
    "embarbascados": "embarbascar",
    "embarbascamos": "embarbascar",
    "embarbascan": "embarbascar",
    "embarbascando": "embarbascar",
    "embarbascara": "embarbascar",
    "embarbascarais": "embarbascar",
    "embarbascaran": "embarbascar",
    "embarbascaras": "embarbascar",
    "embarbascare": "embarbascar",
    "embarbascareis": "embarbascar",
    "embarbascaremos": "embarbascar",
    "embarbascaren": "embarbascar",
    "embarbascares": "embarbascar",
    "embarbascaron": "embarbascar",
    "embarbascará": "embarbascar",
    "embarbascarán": "embarbascar",
    "embarbascarás": "embarbascar",
    "embarbascaré": "embarbascar",
    "embarbascaréis": "embarbascar",
    "embarbascaría": "embarbascar",
    "embarbascaríais": "embarbascar",
    "embarbascaríamos": "embarbascar",
    "embarbascarían": "embarbascar",
    "embarbascarías": "embarbascar",
    "embarbascas": "embarbascar",
    "embarbascase": "embarbascar",
    "embarbascaseis": "embarbascar",
    "embarbascasen": "embarbascar",
    "embarbascases": "embarbascar",
    "embarbascaste": "embarbascar",
    "embarbascasteis": "embarbascar",
    "embarbasco": "embarbascar",
    "embarbascábamos": "embarbascar",
    "embarbascáis": "embarbascar",
    "embarbascáramos": "embarbascar",
    "embarbascáremos": "embarbascar",
    "embarbascásemos": "embarbascar",
    "embarbascó": "embarbascar",
    "embarbasque": "embarbascar",
    "embarbasquemos": "embarbascar",
    "embarbasquen": "embarbascar",
    "embarbasques": "embarbascar",
    "embarbasqué": "embarbascar",
    "embarbasquéis": "embarbascar",
    "embarbece": "embarbecer",
    "embarbeced": "embarbecer",
    "embarbecemos": "embarbecer",
    "embarbecen": "embarbecer",
    "embarbeceremos": "embarbecer",
    "embarbecerá": "embarbecer",
    "embarbecerán": "embarbecer",
    "embarbecerás": "embarbecer",
    "embarbeceré": "embarbecer",
    "embarbeceréis": "embarbecer",
    "embarbecería": "embarbecer",
    "embarbeceríais": "embarbecer",
    "embarbeceríamos": "embarbecer",
    "embarbecerían": "embarbecer",
    "embarbecerías": "embarbecer",
    "embarbeces": "embarbecer",
    "embarbecida": "embarbecer",
    "embarbecidas": "embarbecer",
    "embarbecido": "embarbecer",
    "embarbecidos": "embarbecer",
    "embarbeciendo": "embarbecer",
    "embarbeciera": "embarbecer",
    "embarbecierais": "embarbecer",
    "embarbecieran": "embarbecer",
    "embarbecieras": "embarbecer",
    "embarbeciere": "embarbecer",
    "embarbeciereis": "embarbecer",
    "embarbecieren": "embarbecer",
    "embarbecieres": "embarbecer",
    "embarbecieron": "embarbecer",
    "embarbeciese": "embarbecer",
    "embarbecieseis": "embarbecer",
    "embarbeciesen": "embarbecer",
    "embarbecieses": "embarbecer",
    "embarbecimos": "embarbecer",
    "embarbeciste": "embarbecer",
    "embarbecisteis": "embarbecer",
    "embarbeciéramos": "embarbecer",
    "embarbeciéremos": "embarbecer",
    "embarbeciésemos": "embarbecer",
    "embarbeció": "embarbecer",
    "embarbecéis": "embarbecer",
    "embarbecí": "embarbecer",
    "embarbecía": "embarbecer",
    "embarbecíais": "embarbecer",
    "embarbecíamos": "embarbecer",
    "embarbecían": "embarbecer",
    "embarbecías": "embarbecer",
    "embarbezca": "embarbecer",
    "embarbezcamos": "embarbecer",
    "embarbezcan": "embarbecer",
    "embarbezcas": "embarbecer",
    "embarbezco": "embarbecer",
    "embarbezcáis": "embarbecer",
    "embarbilla": "embarbillar",
    "embarbillaba": "embarbillar",
    "embarbillabais": "embarbillar",
    "embarbillaban": "embarbillar",
    "embarbillabas": "embarbillar",
    "embarbillad": "embarbillar",
    "embarbillada": "embarbillar",
    "embarbilladas": "embarbillar",
    "embarbillado": "embarbillar",
    "embarbillados": "embarbillar",
    "embarbillamos": "embarbillar",
    "embarbillan": "embarbillar",
    "embarbillando": "embarbillar",
    "embarbillara": "embarbillar",
    "embarbillarais": "embarbillar",
    "embarbillaran": "embarbillar",
    "embarbillaras": "embarbillar",
    "embarbillare": "embarbillar",
    "embarbillareis": "embarbillar",
    "embarbillaremos": "embarbillar",
    "embarbillaren": "embarbillar",
    "embarbillares": "embarbillar",
    "embarbillaron": "embarbillar",
    "embarbillará": "embarbillar",
    "embarbillarán": "embarbillar",
    "embarbillarás": "embarbillar",
    "embarbillaré": "embarbillar",
    "embarbillaréis": "embarbillar",
    "embarbillaría": "embarbillar",
    "embarbillaríais": "embarbillar",
    "embarbillaríamos": "embarbillar",
    "embarbillarían": "embarbillar",
    "embarbillarías": "embarbillar",
    "embarbillas": "embarbillar",
    "embarbillase": "embarbillar",
    "embarbillaseis": "embarbillar",
    "embarbillasen": "embarbillar",
    "embarbillases": "embarbillar",
    "embarbillaste": "embarbillar",
    "embarbillasteis": "embarbillar",
    "embarbille": "embarbillar",
    "embarbillemos": "embarbillar",
    "embarbillen": "embarbillar",
    "embarbilles": "embarbillar",
    "embarbillo": "embarbillar",
    "embarbillábamos": "embarbillar",
    "embarbilláis": "embarbillar",
    "embarbilláramos": "embarbillar",
    "embarbilláremos": "embarbillar",
    "embarbillásemos": "embarbillar",
    "embarbillé": "embarbillar",
    "embarbilléis": "embarbillar",
    "embarbilló": "embarbillar",
    "embarca": "embarcar",
    "embarcaba": "embarcar",
    "embarcabais": "embarcar",
    "embarcaban": "embarcar",
    "embarcabas": "embarcar",
    "embarcaciones": "embarcación",
    "embarcaciones-guía": "embarcación-guía",
    "embarcad": "embarcar",
    "embarcada": "embarcar",
    "embarcadas": "embarcar",
    "embarcaderos": "embarcadero",
    "embarcado": "embarcar",
    "embarcadora": "embarcador",
    "embarcadoras": "embarcador",
    "embarcadores": "embarcador",
    "embarcados": "embarcar",
    "embarcamos": "embarcar",
    "embarcan": "embarcar",
    "embarcando": "embarcar",
    "embarcara": "embarcar",
    "embarcarais": "embarcar",
    "embarcaran": "embarcar",
    "embarcaras": "embarcar",
    "embarcare": "embarcar",
    "embarcareis": "embarcar",
    "embarcaremos": "embarcar",
    "embarcaren": "embarcar",
    "embarcares": "embarcar",
    "embarcaron": "embarcar",
    "embarcará": "embarcar",
    "embarcarán": "embarcar",
    "embarcarás": "embarcar",
    "embarcaré": "embarcar",
    "embarcaréis": "embarcar",
    "embarcaría": "embarcar",
    "embarcaríais": "embarcar",
    "embarcaríamos": "embarcar",
    "embarcarían": "embarcar",
    "embarcarías": "embarcar",
    "embarcas": "embarcar",
    "embarcase": "embarcar",
    "embarcaseis": "embarcar",
    "embarcasen": "embarcar",
    "embarcases": "embarcar",
    "embarcaste": "embarcar",
    "embarcasteis": "embarcar",
    "embarco": "embarcar",
    "embarcos": "embarco",
    "embarcábamos": "embarcar",
    "embarcáis": "embarcar",
    "embarcáramos": "embarcar",
    "embarcáremos": "embarcar",
    "embarcásemos": "embarcar",
    "embarcó": "embarcar",
    "embarga": "embargar",
    "embargaba": "embargar",
    "embargabais": "embargar",
    "embargaban": "embargar",
    "embargabas": "embargar",
    "embargables": "embargable",
    "embargad": "embargar",
    "embargada": "embargar",
    "embargadas": "embargar",
    "embargado": "embargar",
    "embargadora": "embargador",
    "embargadoras": "embargador",
    "embargadores": "embargador",
    "embargados": "embargar",
    "embargamos": "embargar",
    "embargan": "embargar",
    "embargando": "embargar",
    "embargantes": "embargante",
    "embargara": "embargar",
    "embargarais": "embargar",
    "embargaran": "embargar",
    "embargaras": "embargar",
    "embargare": "embargar",
    "embargareis": "embargar",
    "embargaremos": "embargar",
    "embargaren": "embargar",
    "embargares": "embargar",
    "embargaron": "embargar",
    "embargará": "embargar",
    "embargarán": "embargar",
    "embargarás": "embargar",
    "embargaré": "embargar",
    "embargaréis": "embargar",
    "embargaría": "embargar",
    "embargaríais": "embargar",
    "embargaríamos": "embargar",
    "embargarían": "embargar",
    "embargarías": "embargar",
    "embargas": "embargar",
    "embargase": "embargar",
    "embargaseis": "embargar",
    "embargasen": "embargar",
    "embargases": "embargar",
    "embargaste": "embargar",
    "embargasteis": "embargar",
    "embargo": "embargar",
    "embargos": "embargo",
    "embargue": "embargar",
    "embarguemos": "embargar",
    "embarguen": "embargar",
    "embargues": "embargar",
    "embargué": "embargar",
    "embarguéis": "embargar",
    "embargábamos": "embargar",
    "embargáis": "embargar",
    "embargáramos": "embargar",
    "embargáremos": "embargar",
    "embargásemos": "embargar",
    "embargó": "embargar",
    "embarnice": "embarnizar",
    "embarnicemos": "embarnizar",
    "embarnicen": "embarnizar",
    "embarnices": "embarnizar",
    "embarnicé": "embarnizar",
    "embarnicéis": "embarnizar",
    "embarniza": "embarnizar",
    "embarnizaba": "embarnizar",
    "embarnizabais": "embarnizar",
    "embarnizaban": "embarnizar",
    "embarnizabas": "embarnizar",
    "embarnizad": "embarnizar",
    "embarnizada": "embarnizar",
    "embarnizadas": "embarnizar",
    "embarnizado": "embarnizar",
    "embarnizados": "embarnizar",
    "embarnizaduras": "embarnizadura",
    "embarnizamos": "embarnizar",
    "embarnizan": "embarnizar",
    "embarnizando": "embarnizar",
    "embarnizara": "embarnizar",
    "embarnizarais": "embarnizar",
    "embarnizaran": "embarnizar",
    "embarnizaras": "embarnizar",
    "embarnizare": "embarnizar",
    "embarnizareis": "embarnizar",
    "embarnizaremos": "embarnizar",
    "embarnizaren": "embarnizar",
    "embarnizares": "embarnizar",
    "embarnizaron": "embarnizar",
    "embarnizará": "embarnizar",
    "embarnizarán": "embarnizar",
    "embarnizarás": "embarnizar",
    "embarnizaré": "embarnizar",
    "embarnizaréis": "embarnizar",
    "embarnizaría": "embarnizar",
    "embarnizaríais": "embarnizar",
    "embarnizaríamos": "embarnizar",
    "embarnizarían": "embarnizar",
    "embarnizarías": "embarnizar",
    "embarnizas": "embarnizar",
    "embarnizase": "embarnizar",
    "embarnizaseis": "embarnizar",
    "embarnizasen": "embarnizar",
    "embarnizases": "embarnizar",
    "embarnizaste": "embarnizar",
    "embarnizasteis": "embarnizar",
    "embarnizo": "embarnizar",
    "embarnizábamos": "embarnizar",
    "embarnizáis": "embarnizar",
    "embarnizáramos": "embarnizar",
    "embarnizáremos": "embarnizar",
    "embarnizásemos": "embarnizar",
    "embarnizó": "embarnizar",
    "embarque": "embarcar",
    "embarquemos": "embarcar",
    "embarquen": "embarcar",
    "embarques": "embarque",
    "embarqué": "embarcar",
    "embarquéis": "embarcar",
    "embarra": "embarrar",
    "embarraba": "embarrar",
    "embarrabais": "embarrar",
    "embarraban": "embarrar",
    "embarrabas": "embarrar",
    "embarrad": "embarrar",
    "embarrada": "embarrar",
    "embarradas": "embarrar",
    "embarrado": "embarrar",
    "embarradora": "embarrador",
    "embarradoras": "embarrador",
    "embarradores": "embarrador",
    "embarrados": "embarrar",
    "embarraduras": "embarradura",
    "embarramos": "embarrar",
    "embarran": "embarrar",
    "embarranca": "embarrancar",
    "embarrancaba": "embarrancar",
    "embarrancabais": "embarrancar",
    "embarrancaban": "embarrancar",
    "embarrancabas": "embarrancar",
    "embarrancad": "embarrancar",
    "embarrancada": "embarrancar",
    "embarrancadas": "embarrancar",
    "embarrancado": "embarrancar",
    "embarrancados": "embarrancar",
    "embarrancamientos": "embarrancamiento",
    "embarrancamos": "embarrancar",
    "embarrancan": "embarrancar",
    "embarrancando": "embarrancar",
    "embarrancara": "embarrancar",
    "embarrancarais": "embarrancar",
    "embarrancaran": "embarrancar",
    "embarrancaras": "embarrancar",
    "embarrancare": "embarrancar",
    "embarrancareis": "embarrancar",
    "embarrancaremos": "embarrancar",
    "embarrancaren": "embarrancar",
    "embarrancares": "embarrancar",
    "embarrancaron": "embarrancar",
    "embarrancará": "embarrancar",
    "embarrancarán": "embarrancar",
    "embarrancarás": "embarrancar",
    "embarrancaré": "embarrancar",
    "embarrancaréis": "embarrancar",
    "embarrancaría": "embarrancar",
    "embarrancaríais": "embarrancar",
    "embarrancaríamos": "embarrancar",
    "embarrancarían": "embarrancar",
    "embarrancarías": "embarrancar",
    "embarrancas": "embarrancar",
    "embarrancase": "embarrancar",
    "embarrancaseis": "embarrancar",
    "embarrancasen": "embarrancar",
    "embarrancases": "embarrancar",
    "embarrancaste": "embarrancar",
    "embarrancasteis": "embarrancar",
    "embarranco": "embarrancar",
    "embarrancábamos": "embarrancar",
    "embarrancáis": "embarrancar",
    "embarrancáramos": "embarrancar",
    "embarrancáremos": "embarrancar",
    "embarrancásemos": "embarrancar",
    "embarrancó": "embarrancar",
    "embarrando": "embarrar",
    "embarranque": "embarrancar",
    "embarranquemos": "embarrancar",
    "embarranquen": "embarrancar",
    "embarranques": "embarrancar",
    "embarranqué": "embarrancar",
    "embarranquéis": "embarrancar",
    "embarrara": "embarrar",
    "embarrarais": "embarrar",
    "embarraran": "embarrar",
    "embarraras": "embarrar",
    "embarrare": "embarrar",
    "embarrareis": "embarrar",
    "embarraremos": "embarrar",
    "embarraren": "embarrar",
    "embarrares": "embarrar",
    "embarraron": "embarrar",
    "embarrará": "embarrar",
    "embarrarán": "embarrar",
    "embarrarás": "embarrar",
    "embarraré": "embarrar",
    "embarraréis": "embarrar",
    "embarraría": "embarrar",
    "embarraríais": "embarrar",
    "embarraríamos": "embarrar",
    "embarrarían": "embarrar",
    "embarrarías": "embarrar",
    "embarras": "embarrar",
    "embarrase": "embarrar",
    "embarraseis": "embarrar",
    "embarrasen": "embarrar",
    "embarrases": "embarrar",
    "embarraste": "embarrar",
    "embarrasteis": "embarrar",
    "embarre": "embarrar",
    "embarremos": "embarrar",
    "embarren": "embarrar",
    "embarres": "embarrar",
    "embarrila": "embarrilar",
    "embarrilaba": "embarrilar",
    "embarrilabais": "embarrilar",
    "embarrilaban": "embarrilar",
    "embarrilabas": "embarrilar",
    "embarrilad": "embarrilar",
    "embarrilada": "embarrilar",
    "embarriladas": "embarrilar",
    "embarrilado": "embarrilar",
    "embarriladores": "embarrilador",
    "embarrilados": "embarrilar",
    "embarrilamos": "embarrilar",
    "embarrilan": "embarrilar",
    "embarrilando": "embarrilar",
    "embarrilara": "embarrilar",
    "embarrilarais": "embarrilar",
    "embarrilaran": "embarrilar",
    "embarrilaras": "embarrilar",
    "embarrilare": "embarrilar",
    "embarrilareis": "embarrilar",
    "embarrilaremos": "embarrilar",
    "embarrilaren": "embarrilar",
    "embarrilares": "embarrilar",
    "embarrilaron": "embarrilar",
    "embarrilará": "embarrilar",
    "embarrilarán": "embarrilar",
    "embarrilarás": "embarrilar",
    "embarrilaré": "embarrilar",
    "embarrilaréis": "embarrilar",
    "embarrilaría": "embarrilar",
    "embarrilaríais": "embarrilar",
    "embarrilaríamos": "embarrilar",
    "embarrilarían": "embarrilar",
    "embarrilarías": "embarrilar",
    "embarrilas": "embarrilar",
    "embarrilase": "embarrilar",
    "embarrilaseis": "embarrilar",
    "embarrilasen": "embarrilar",
    "embarrilases": "embarrilar",
    "embarrilaste": "embarrilar",
    "embarrilasteis": "embarrilar",
    "embarrile": "embarrilar",
    "embarrilemos": "embarrilar",
    "embarrilen": "embarrilar",
    "embarriles": "embarrilar",
    "embarrilo": "embarrilar",
    "embarrilábamos": "embarrilar",
    "embarriláis": "embarrilar",
    "embarriláramos": "embarrilar",
    "embarriláremos": "embarrilar",
    "embarrilásemos": "embarrilar",
    "embarrilé": "embarrilar",
    "embarriléis": "embarrilar",
    "embarriló": "embarrilar",
    "embarro": "embarrar",
    "embarrábamos": "embarrar",
    "embarráis": "embarrar",
    "embarráramos": "embarrar",
    "embarráremos": "embarrar",
    "embarrásemos": "embarrar",
    "embarré": "embarrar",
    "embarréis": "embarrar",
    "embarró": "embarrar",
    "embarulla": "embarullar",
    "embarullaba": "embarullar",
    "embarullabais": "embarullar",
    "embarullaban": "embarullar",
    "embarullabas": "embarullar",
    "embarullad": "embarullar",
    "embarullada": "embarullar",
    "embarulladas": "embarullar",
    "embarullado": "embarullar",
    "embarulladora": "embarullador",
    "embarulladoras": "embarullador",
    "embarulladores": "embarullador",
    "embarullados": "embarullar",
    "embarullamientos": "embarullamiento",
    "embarullamos": "embarullar",
    "embarullan": "embarullar",
    "embarullando": "embarullar",
    "embarullara": "embarullar",
    "embarullarais": "embarullar",
    "embarullaran": "embarullar",
    "embarullaras": "embarullar",
    "embarullare": "embarullar",
    "embarullareis": "embarullar",
    "embarullaremos": "embarullar",
    "embarullaren": "embarullar",
    "embarullares": "embarullar",
    "embarullaron": "embarullar",
    "embarullará": "embarullar",
    "embarullarán": "embarullar",
    "embarullarás": "embarullar",
    "embarullaré": "embarullar",
    "embarullaréis": "embarullar",
    "embarullaría": "embarullar",
    "embarullaríais": "embarullar",
    "embarullaríamos": "embarullar",
    "embarullarían": "embarullar",
    "embarullarías": "embarullar",
    "embarullas": "embarullar",
    "embarullase": "embarullar",
    "embarullaseis": "embarullar",
    "embarullasen": "embarullar",
    "embarullases": "embarullar",
    "embarullaste": "embarullar",
    "embarullasteis": "embarullar",
    "embarulle": "embarullar",
    "embarullemos": "embarullar",
    "embarullen": "embarullar",
    "embarulles": "embarullar",
    "embarullo": "embarullar",
    "embarullábamos": "embarullar",
    "embarulláis": "embarullar",
    "embarulláramos": "embarullar",
    "embarulláremos": "embarullar",
    "embarullásemos": "embarullar",
    "embarullé": "embarullar",
    "embarulléis": "embarullar",
    "embarulló": "embarullar",
    "embasaduras": "embasadura",
    "embasamientos": "embasamiento",
    "embasta": "embastar",
    "embastaba": "embastar",
    "embastabais": "embastar",
    "embastaban": "embastar",
    "embastabas": "embastar",
    "embastad": "embastar",
    "embastada": "embastar",
    "embastadas": "embastar",
    "embastado": "embastar",
    "embastados": "embastar",
    "embastamos": "embastar",
    "embastan": "embastar",
    "embastando": "embastar",
    "embastara": "embastar",
    "embastarais": "embastar",
    "embastaran": "embastar",
    "embastaras": "embastar",
    "embastare": "embastar",
    "embastareis": "embastar",
    "embastaremos": "embastar",
    "embastaren": "embastar",
    "embastares": "embastar",
    "embastaron": "embastar",
    "embastará": "embastar",
    "embastarán": "embastar",
    "embastarás": "embastar",
    "embastaré": "embastar",
    "embastaréis": "embastar",
    "embastaría": "embastar",
    "embastaríais": "embastar",
    "embastaríamos": "embastar",
    "embastarían": "embastar",
    "embastarías": "embastar",
    "embastas": "embastar",
    "embastase": "embastar",
    "embastaseis": "embastar",
    "embastasen": "embastar",
    "embastases": "embastar",
    "embastaste": "embastar",
    "embastasteis": "embastar",
    "embaste": "embastar",
    "embastece": "embastecer",
    "embasteced": "embastecer",
    "embastecemos": "embastecer",
    "embastecen": "embastecer",
    "embasteceremos": "embastecer",
    "embastecerá": "embastecer",
    "embastecerán": "embastecer",
    "embastecerás": "embastecer",
    "embasteceré": "embastecer",
    "embasteceréis": "embastecer",
    "embastecería": "embastecer",
    "embasteceríais": "embastecer",
    "embasteceríamos": "embastecer",
    "embastecerían": "embastecer",
    "embastecerías": "embastecer",
    "embasteces": "embastecer",
    "embastecida": "embastecer",
    "embastecidas": "embastecer",
    "embastecido": "embastecer",
    "embastecidos": "embastecer",
    "embasteciendo": "embastecer",
    "embasteciera": "embastecer",
    "embastecierais": "embastecer",
    "embastecieran": "embastecer",
    "embastecieras": "embastecer",
    "embasteciere": "embastecer",
    "embasteciereis": "embastecer",
    "embastecieren": "embastecer",
    "embastecieres": "embastecer",
    "embastecieron": "embastecer",
    "embasteciese": "embastecer",
    "embastecieseis": "embastecer",
    "embasteciesen": "embastecer",
    "embastecieses": "embastecer",
    "embastecimos": "embastecer",
    "embasteciste": "embastecer",
    "embastecisteis": "embastecer",
    "embasteciéramos": "embastecer",
    "embasteciéremos": "embastecer",
    "embasteciésemos": "embastecer",
    "embasteció": "embastecer",
    "embastecéis": "embastecer",
    "embastecí": "embastecer",
    "embastecía": "embastecer",
    "embastecíais": "embastecer",
    "embastecíamos": "embastecer",
    "embastecían": "embastecer",
    "embastecías": "embastecer",
    "embastemos": "embastar",
    "embasten": "embastar",
    "embastes": "embaste",
    "embastezca": "embastecer",
    "embastezcamos": "embastecer",
    "embastezcan": "embastecer",
    "embastezcas": "embastecer",
    "embastezco": "embastecer",
    "embastezcáis": "embastecer",
    "embasto": "embastar",
    "embastábamos": "embastar",
    "embastáis": "embastar",
    "embastáramos": "embastar",
    "embastáremos": "embastar",
    "embastásemos": "embastar",
    "embasté": "embastar",
    "embastéis": "embastar",
    "embastó": "embastar",
    "embates": "embate",
    "embauca": "embaucar",
    "embaucaba": "embaucar",
    "embaucabais": "embaucar",
    "embaucaban": "embaucar",
    "embaucabas": "embaucar",
    "embaucad": "embaucar",
    "embaucada": "embaucar",
    "embaucadas": "embaucar",
    "embaucado": "embaucar",
    "embaucadora": "embaucador",
    "embaucadoras": "embaucador",
    "embaucadores": "embaucador",
    "embaucados": "embaucar",
    "embaucamientos": "embaucamiento",
    "embaucamos": "embaucar",
    "embaucan": "embaucar",
    "embaucando": "embaucar",
    "embaucara": "embaucar",
    "embaucarais": "embaucar",
    "embaucaran": "embaucar",
    "embaucaras": "embaucar",
    "embaucare": "embaucar",
    "embaucareis": "embaucar",
    "embaucaremos": "embaucar",
    "embaucaren": "embaucar",
    "embaucares": "embaucar",
    "embaucaron": "embaucar",
    "embaucará": "embaucar",
    "embaucarán": "embaucar",
    "embaucarás": "embaucar",
    "embaucaré": "embaucar",
    "embaucaréis": "embaucar",
    "embaucaría": "embaucar",
    "embaucaríais": "embaucar",
    "embaucaríamos": "embaucar",
    "embaucarían": "embaucar",
    "embaucarías": "embaucar",
    "embaucas": "embaucar",
    "embaucase": "embaucar",
    "embaucaseis": "embaucar",
    "embaucasen": "embaucar",
    "embaucases": "embaucar",
    "embaucaste": "embaucar",
    "embaucasteis": "embaucar",
    "embauco": "embaucar",
    "embaucábamos": "embaucar",
    "embaucáis": "embaucar",
    "embaucáramos": "embaucar",
    "embaucáremos": "embaucar",
    "embaucásemos": "embaucar",
    "embaucó": "embaucar",
    "embaula": "embaular",
    "embaulaba": "embaular",
    "embaulabais": "embaular",
    "embaulaban": "embaular",
    "embaulabas": "embaular",
    "embaulad": "embaular",
    "embaulada": "embaular",
    "embauladas": "embaular",
    "embaulado": "embaular",
    "embaulados": "embaular",
    "embaulamos": "embaular",
    "embaulan": "embaular",
    "embaulando": "embaular",
    "embaulara": "embaular",
    "embaularais": "embaular",
    "embaularan": "embaular",
    "embaularas": "embaular",
    "embaulare": "embaular",
    "embaulareis": "embaular",
    "embaularemos": "embaular",
    "embaularen": "embaular",
    "embaulares": "embaular",
    "embaularon": "embaular",
    "embaulará": "embaular",
    "embaularán": "embaular",
    "embaularás": "embaular",
    "embaularé": "embaular",
    "embaularéis": "embaular",
    "embaularía": "embaular",
    "embaularíais": "embaular",
    "embaularíamos": "embaular",
    "embaularían": "embaular",
    "embaularías": "embaular",
    "embaulas": "embaular",
    "embaulase": "embaular",
    "embaulaseis": "embaular",
    "embaulasen": "embaular",
    "embaulases": "embaular",
    "embaulaste": "embaular",
    "embaulasteis": "embaular",
    "embaule": "embaular",
    "embaulemos": "embaular",
    "embaulen": "embaular",
    "embaules": "embaular",
    "embaulo": "embaular",
    "embaulábamos": "embaular",
    "embauláis": "embaular",
    "embauláramos": "embaular",
    "embauláremos": "embaular",
    "embaulásemos": "embaular",
    "embaulé": "embaular",
    "embauléis": "embaular",
    "embauló": "embaular",
    "embauque": "embaucar",
    "embauquemos": "embaucar",
    "embauquen": "embaucar",
    "embauques": "embaucar",
    "embauqué": "embaucar",
    "embauquéis": "embaucar",
    "embayendo": "embair",
    "embayera": "embair",
    "embayerais": "embair",
    "embayeran": "embair",
    "embayeras": "embair",
    "embayere": "embair",
    "embayereis": "embair",
    "embayeren": "embair",
    "embayeres": "embair",
    "embayeron": "embair",
    "embayese": "embair",
    "embayeseis": "embair",
    "embayesen": "embair",
    "embayeses": "embair",
    "embayéramos": "embair",
    "embayéremos": "embair",
    "embayésemos": "embair",
    "embayó": "embair",
    "embaza": "embazar",
    "embazaba": "embazar",
    "embazabais": "embazar",
    "embazaban": "embazar",
    "embazabas": "embazar",
    "embazad": "embazar",
    "embazada": "embazar",
    "embazadas": "embazar",
    "embazado": "embazar",
    "embazadora": "embazador",
    "embazadoras": "embazador",
    "embazadores": "embazador",
    "embazados": "embazar",
    "embazaduras": "embazadura",
    "embazamos": "embazar",
    "embazan": "embazar",
    "embazando": "embazar",
    "embazara": "embazar",
    "embazarais": "embazar",
    "embazaran": "embazar",
    "embazaras": "embazar",
    "embazare": "embazar",
    "embazareis": "embazar",
    "embazaremos": "embazar",
    "embazaren": "embazar",
    "embazares": "embazar",
    "embazaron": "embazar",
    "embazará": "embazar",
    "embazarán": "embazar",
    "embazarás": "embazar",
    "embazaré": "embazar",
    "embazaréis": "embazar",
    "embazaría": "embazar",
    "embazaríais": "embazar",
    "embazaríamos": "embazar",
    "embazarían": "embazar",
    "embazarías": "embazar",
    "embazas": "embazar",
    "embazase": "embazar",
    "embazaseis": "embazar",
    "embazasen": "embazar",
    "embazases": "embazar",
    "embazaste": "embazar",
    "embazasteis": "embazar",
    "embazo": "embazar",
    "embazábamos": "embazar",
    "embazáis": "embazar",
    "embazáramos": "embazar",
    "embazáremos": "embazar",
    "embazásemos": "embazar",
    "embazó": "embazar",
    "embaí": "embair",
    "embaía": "embair",
    "embaíais": "embair",
    "embaíamos": "embair",
    "embaían": "embair",
    "embaías": "embair",
    "embaíd": "embair",
    "embaído": "embair",
    "embaídos": "embair",
    "embaímos": "embair",
    "embaír": "embair",
    "embaís": "embair",
    "embaíste": "embair",
    "embaísteis": "embair",
    "embeba": "embeber",
    "embebamos": "embeber",
    "embeban": "embeber",
    "embebas": "embeber",
    "embebe": "embeber",
    "embebece": "embebecer",
    "embebeced": "embebecer",
    "embebecemos": "embebecer",
    "embebecen": "embebecer",
    "embebeceremos": "embebecer",
    "embebecerá": "embebecer",
    "embebecerán": "embebecer",
    "embebecerás": "embebecer",
    "embebeceré": "embebecer",
    "embebeceréis": "embebecer",
    "embebecería": "embebecer",
    "embebeceríais": "embebecer",
    "embebeceríamos": "embebecer",
    "embebecerían": "embebecer",
    "embebecerías": "embebecer",
    "embebeces": "embebecer",
    "embebecida": "embebecer",
    "embebecidas": "embebecer",
    "embebecido": "embebecer",
    "embebecidos": "embebecer",
    "embebeciendo": "embebecer",
    "embebeciera": "embebecer",
    "embebecierais": "embebecer",
    "embebecieran": "embebecer",
    "embebecieras": "embebecer",
    "embebeciere": "embebecer",
    "embebeciereis": "embebecer",
    "embebecieren": "embebecer",
    "embebecieres": "embebecer",
    "embebecieron": "embebecer",
    "embebeciese": "embebecer",
    "embebecieseis": "embebecer",
    "embebeciesen": "embebecer",
    "embebecieses": "embebecer",
    "embebecimientos": "embebecimiento",
    "embebecimos": "embebecer",
    "embebeciste": "embebecer",
    "embebecisteis": "embebecer",
    "embebeciéramos": "embebecer",
    "embebeciéremos": "embebecer",
    "embebeciésemos": "embebecer",
    "embebeció": "embebecer",
    "embebecéis": "embebecer",
    "embebecí": "embebecer",
    "embebecía": "embebecer",
    "embebecíais": "embebecer",
    "embebecíamos": "embebecer",
    "embebecían": "embebecer",
    "embebecías": "embebecer",
    "embebed": "embeber",
    "embebedora": "embebedor",
    "embebedoras": "embebedor",
    "embebedores": "embebedor",
    "embebemos": "embeber",
    "embeben": "embeber",
    "embeberemos": "embeber",
    "embeberá": "embeber",
    "embeberán": "embeber",
    "embeberás": "embeber",
    "embeberé": "embeber",
    "embeberéis": "embeber",
    "embebería": "embeber",
    "embeberíais": "embeber",
    "embeberíamos": "embeber",
    "embeberían": "embeber",
    "embeberías": "embeber",
    "embebes": "embeber",
    "embebezca": "embebecer",
    "embebezcamos": "embebecer",
    "embebezcan": "embebecer",
    "embebezcas": "embebecer",
    "embebezco": "embebecer",
    "embebezcáis": "embebecer",
    "embebida": "embeber",
    "embebidas": "embeber",
    "embebido": "embeber",
    "embebidos": "embeber",
    "embebiendo": "embeber",
    "embebiera": "embeber",
    "embebierais": "embeber",
    "embebieran": "embeber",
    "embebieras": "embeber",
    "embebiere": "embeber",
    "embebiereis": "embeber",
    "embebieren": "embeber",
    "embebieres": "embeber",
    "embebieron": "embeber",
    "embebiese": "embeber",
    "embebieseis": "embeber",
    "embebiesen": "embeber",
    "embebieses": "embeber",
    "embebimientos": "embebimiento",
    "embebimos": "embeber",
    "embebiste": "embeber",
    "embebisteis": "embeber",
    "embebiéramos": "embeber",
    "embebiéremos": "embeber",
    "embebiésemos": "embeber",
    "embebió": "embeber",
    "embebo": "embeber",
    "embebáis": "embeber",
    "embebéis": "embeber",
    "embebí": "embeber",
    "embebía": "embeber",
    "embebíais": "embeber",
    "embebíamos": "embeber",
    "embebían": "embeber",
    "embebías": "embeber",
    "embecaduras": "embecadura",
    "embeleca": "embelecar",
    "embelecaba": "embelecar",
    "embelecabais": "embelecar",
    "embelecaban": "embelecar",
    "embelecabas": "embelecar",
    "embelecad": "embelecar",
    "embelecada": "embelecar",
    "embelecadas": "embelecar",
    "embelecado": "embelecar",
    "embelecadora": "embelecador",
    "embelecadoras": "embelecador",
    "embelecadores": "embelecador",
    "embelecados": "embelecar",
    "embelecamientos": "embelecamiento",
    "embelecamos": "embelecar",
    "embelecan": "embelecar",
    "embelecando": "embelecar",
    "embelecara": "embelecar",
    "embelecarais": "embelecar",
    "embelecaran": "embelecar",
    "embelecaras": "embelecar",
    "embelecare": "embelecar",
    "embelecareis": "embelecar",
    "embelecaremos": "embelecar",
    "embelecaren": "embelecar",
    "embelecares": "embelecar",
    "embelecaron": "embelecar",
    "embelecará": "embelecar",
    "embelecarán": "embelecar",
    "embelecarás": "embelecar",
    "embelecaré": "embelecar",
    "embelecaréis": "embelecar",
    "embelecaría": "embelecar",
    "embelecaríais": "embelecar",
    "embelecaríamos": "embelecar",
    "embelecarían": "embelecar",
    "embelecarías": "embelecar",
    "embelecas": "embelecar",
    "embelecase": "embelecar",
    "embelecaseis": "embelecar",
    "embelecasen": "embelecar",
    "embelecases": "embelecar",
    "embelecaste": "embelecar",
    "embelecasteis": "embelecar",
    "embeleco": "embelecar",
    "embelecos": "embeleco",
    "embelecábamos": "embelecar",
    "embelecáis": "embelecar",
    "embelecáramos": "embelecar",
    "embelecáremos": "embelecar",
    "embelecásemos": "embelecar",
    "embelecó": "embelecar",
    "embeleque": "embelecar",
    "embelequemos": "embelecar",
    "embelequen": "embelecar",
    "embelequera": "embelequero",
    "embelequeras": "embelequero",
    "embelequeros": "embelequero",
    "embeleques": "embelecar",
    "embelequé": "embelecar",
    "embelequéis": "embelecar",
    "embelesa": "embelesar",
    "embelesaba": "embelesar",
    "embelesabais": "embelesar",
    "embelesaban": "embelesar",
    "embelesabas": "embelesar",
    "embelesad": "embelesar",
    "embelesada": "embelesar",
    "embelesadas": "embelesar",
    "embelesado": "embelesar",
    "embelesados": "embelesar",
    "embelesamientos": "embelesamiento",
    "embelesamos": "embelesar",
    "embelesan": "embelesar",
    "embelesando": "embelesar",
    "embelesara": "embelesar",
    "embelesarais": "embelesar",
    "embelesaran": "embelesar",
    "embelesaras": "embelesar",
    "embelesare": "embelesar",
    "embelesareis": "embelesar",
    "embelesaremos": "embelesar",
    "embelesaren": "embelesar",
    "embelesares": "embelesar",
    "embelesaron": "embelesar",
    "embelesará": "embelesar",
    "embelesarán": "embelesar",
    "embelesarás": "embelesar",
    "embelesaré": "embelesar",
    "embelesaréis": "embelesar",
    "embelesaría": "embelesar",
    "embelesaríais": "embelesar",
    "embelesaríamos": "embelesar",
    "embelesarían": "embelesar",
    "embelesarías": "embelesar",
    "embelesas": "embelesar",
    "embelesase": "embelesar",
    "embelesaseis": "embelesar",
    "embelesasen": "embelesar",
    "embelesases": "embelesar",
    "embelesaste": "embelesar",
    "embelesasteis": "embelesar",
    "embelese": "embelesar",
    "embelesemos": "embelesar",
    "embelesen": "embelesar",
    "embeleses": "embelesar",
    "embeleso": "embelesar",
    "embelesos": "embeleso",
    "embelesábamos": "embelesar",
    "embelesáis": "embelesar",
    "embelesáramos": "embelesar",
    "embelesáremos": "embelesar",
    "embelesásemos": "embelesar",
    "embelesé": "embelesar",
    "embeleséis": "embelesar",
    "embelesó": "embelesar",
    "embellaquece": "embellaquecer",
    "embellaqueced": "embellaquecer",
    "embellaquecemos": "embellaquecer",
    "embellaquecen": "embellaquecer",
    "embellaqueceremos": "embellaquecer",
    "embellaquecerá": "embellaquecer",
    "embellaquecerán": "embellaquecer",
    "embellaquecerás": "embellaquecer",
    "embellaqueceré": "embellaquecer",
    "embellaqueceréis": "embellaquecer",
    "embellaquecería": "embellaquecer",
    "embellaqueceríais": "embellaquecer",
    "embellaqueceríamos": "embellaquecer",
    "embellaquecerían": "embellaquecer",
    "embellaquecerías": "embellaquecer",
    "embellaqueces": "embellaquecer",
    "embellaquecida": "embellaquecer",
    "embellaquecidas": "embellaquecer",
    "embellaquecido": "embellaquecer",
    "embellaquecidos": "embellaquecer",
    "embellaqueciendo": "embellaquecer",
    "embellaqueciera": "embellaquecer",
    "embellaquecierais": "embellaquecer",
    "embellaquecieran": "embellaquecer",
    "embellaquecieras": "embellaquecer",
    "embellaqueciere": "embellaquecer",
    "embellaqueciereis": "embellaquecer",
    "embellaquecieren": "embellaquecer",
    "embellaquecieres": "embellaquecer",
    "embellaquecieron": "embellaquecer",
    "embellaqueciese": "embellaquecer",
    "embellaquecieseis": "embellaquecer",
    "embellaqueciesen": "embellaquecer",
    "embellaquecieses": "embellaquecer",
    "embellaquecimos": "embellaquecer",
    "embellaqueciste": "embellaquecer",
    "embellaquecisteis": "embellaquecer",
    "embellaqueciéramos": "embellaquecer",
    "embellaqueciéremos": "embellaquecer",
    "embellaqueciésemos": "embellaquecer",
    "embellaqueció": "embellaquecer",
    "embellaquecéis": "embellaquecer",
    "embellaquecí": "embellaquecer",
    "embellaquecía": "embellaquecer",
    "embellaquecíais": "embellaquecer",
    "embellaquecíamos": "embellaquecer",
    "embellaquecían": "embellaquecer",
    "embellaquecías": "embellaquecer",
    "embellaquezca": "embellaquecer",
    "embellaquezcamos": "embellaquecer",
    "embellaquezcan": "embellaquecer",
    "embellaquezcas": "embellaquecer",
    "embellaquezco": "embellaquecer",
    "embellaquezcáis": "embellaquecer",
    "embellece": "embellecer",
    "embelleced": "embellecer",
    "embellecedora": "embellecedor",
    "embellecedoras": "embellecedor",
    "embellecedores": "embellecedor",
    "embellecemos": "embellecer",
    "embellecen": "embellecer",
    "embelleceremos": "embellecer",
    "embellecerá": "embellecer",
    "embellecerán": "embellecer",
    "embellecerás": "embellecer",
    "embelleceré": "embellecer",
    "embelleceréis": "embellecer",
    "embellecería": "embellecer",
    "embelleceríais": "embellecer",
    "embelleceríamos": "embellecer",
    "embellecerían": "embellecer",
    "embellecerías": "embellecer",
    "embelleces": "embellecer",
    "embellecida": "embellecer",
    "embellecidas": "embellecer",
    "embellecido": "embellecer",
    "embellecidos": "embellecer",
    "embelleciendo": "embellecer",
    "embelleciera": "embellecer",
    "embellecierais": "embellecer",
    "embellecieran": "embellecer",
    "embellecieras": "embellecer",
    "embelleciere": "embellecer",
    "embelleciereis": "embellecer",
    "embellecieren": "embellecer",
    "embellecieres": "embellecer",
    "embellecieron": "embellecer",
    "embelleciese": "embellecer",
    "embellecieseis": "embellecer",
    "embelleciesen": "embellecer",
    "embellecieses": "embellecer",
    "embellecimientos": "embellecimiento",
    "embellecimos": "embellecer",
    "embelleciste": "embellecer",
    "embellecisteis": "embellecer",
    "embelleciéramos": "embellecer",
    "embelleciéremos": "embellecer",
    "embelleciésemos": "embellecer",
    "embelleció": "embellecer",
    "embellecéis": "embellecer",
    "embellecí": "embellecer",
    "embellecía": "embellecer",
    "embellecíais": "embellecer",
    "embellecíamos": "embellecer",
    "embellecían": "embellecer",
    "embellecías": "embellecer",
    "embellezca": "embellecer",
    "embellezcamos": "embellecer",
    "embellezcan": "embellecer",
    "embellezcas": "embellecer",
    "embellezco": "embellecer",
    "embellezcáis": "embellecer",
    "embermeja": "embermejar",
    "embermejaba": "embermejar",
    "embermejabais": "embermejar",
    "embermejaban": "embermejar",
    "embermejabas": "embermejar",
    "embermejad": "embermejar",
    "embermejada": "embermejar",
    "embermejadas": "embermejar",
    "embermejado": "embermejar",
    "embermejados": "embermejar",
    "embermejamos": "embermejar",
    "embermejan": "embermejar",
    "embermejando": "embermejar",
    "embermejara": "embermejar",
    "embermejarais": "embermejar",
    "embermejaran": "embermejar",
    "embermejaras": "embermejar",
    "embermejare": "embermejar",
    "embermejareis": "embermejar",
    "embermejaremos": "embermejar",
    "embermejaren": "embermejar",
    "embermejares": "embermejar",
    "embermejaron": "embermejar",
    "embermejará": "embermejar",
    "embermejarán": "embermejar",
    "embermejarás": "embermejar",
    "embermejaré": "embermejar",
    "embermejaréis": "embermejar",
    "embermejaría": "embermejar",
    "embermejaríais": "embermejar",
    "embermejaríamos": "embermejar",
    "embermejarían": "embermejar",
    "embermejarías": "embermejar",
    "embermejas": "embermejar",
    "embermejase": "embermejar",
    "embermejaseis": "embermejar",
    "embermejasen": "embermejar",
    "embermejases": "embermejar",
    "embermejaste": "embermejar",
    "embermejasteis": "embermejar",
    "embermeje": "embermejar",
    "embermejece": "embermejecer",
    "embermejeced": "embermejecer",
    "embermejecemos": "embermejecer",
    "embermejecen": "embermejecer",
    "embermejeceremos": "embermejecer",
    "embermejecerá": "embermejecer",
    "embermejecerán": "embermejecer",
    "embermejecerás": "embermejecer",
    "embermejeceré": "embermejecer",
    "embermejeceréis": "embermejecer",
    "embermejecería": "embermejecer",
    "embermejeceríais": "embermejecer",
    "embermejeceríamos": "embermejecer",
    "embermejecerían": "embermejecer",
    "embermejecerías": "embermejecer",
    "embermejeces": "embermejecer",
    "embermejecida": "embermejecer",
    "embermejecidas": "embermejecer",
    "embermejecido": "embermejecer",
    "embermejecidos": "embermejecer",
    "embermejeciendo": "embermejecer",
    "embermejeciera": "embermejecer",
    "embermejecierais": "embermejecer",
    "embermejecieran": "embermejecer",
    "embermejecieras": "embermejecer",
    "embermejeciere": "embermejecer",
    "embermejeciereis": "embermejecer",
    "embermejecieren": "embermejecer",
    "embermejecieres": "embermejecer",
    "embermejecieron": "embermejecer",
    "embermejeciese": "embermejecer",
    "embermejecieseis": "embermejecer",
    "embermejeciesen": "embermejecer",
    "embermejecieses": "embermejecer",
    "embermejecimos": "embermejecer",
    "embermejeciste": "embermejecer",
    "embermejecisteis": "embermejecer",
    "embermejeciéramos": "embermejecer",
    "embermejeciéremos": "embermejecer",
    "embermejeciésemos": "embermejecer",
    "embermejeció": "embermejecer",
    "embermejecéis": "embermejecer",
    "embermejecí": "embermejecer",
    "embermejecía": "embermejecer",
    "embermejecíais": "embermejecer",
    "embermejecíamos": "embermejecer",
    "embermejecían": "embermejecer",
    "embermejecías": "embermejecer",
    "embermejemos": "embermejar",
    "embermejen": "embermejar",
    "embermejes": "embermejar",
    "embermejezca": "embermejecer",
    "embermejezcamos": "embermejecer",
    "embermejezcan": "embermejecer",
    "embermejezcas": "embermejecer",
    "embermejezco": "embermejecer",
    "embermejezcáis": "embermejecer",
    "embermejo": "embermejar",
    "embermejábamos": "embermejar",
    "embermejáis": "embermejar",
    "embermejáramos": "embermejar",
    "embermejáremos": "embermejar",
    "embermejásemos": "embermejar",
    "embermejé": "embermejar",
    "embermejéis": "embermejar",
    "embermejó": "embermejar",
    "emberrenchina": "emberrenchinar",
    "emberrenchinaba": "emberrenchinar",
    "emberrenchinabais": "emberrenchinar",
    "emberrenchinaban": "emberrenchinar",
    "emberrenchinabas": "emberrenchinar",
    "emberrenchinad": "emberrenchinar",
    "emberrenchinada": "emberrenchinar",
    "emberrenchinadas": "emberrenchinar",
    "emberrenchinado": "emberrenchinar",
    "emberrenchinados": "emberrenchinar",
    "emberrenchinamos": "emberrenchinar",
    "emberrenchinan": "emberrenchinar",
    "emberrenchinando": "emberrenchinar",
    "emberrenchinara": "emberrenchinar",
    "emberrenchinarais": "emberrenchinar",
    "emberrenchinaran": "emberrenchinar",
    "emberrenchinaras": "emberrenchinar",
    "emberrenchinare": "emberrenchinar",
    "emberrenchinareis": "emberrenchinar",
    "emberrenchinaremos": "emberrenchinar",
    "emberrenchinaren": "emberrenchinar",
    "emberrenchinares": "emberrenchinar",
    "emberrenchinaron": "emberrenchinar",
    "emberrenchinará": "emberrenchinar",
    "emberrenchinarán": "emberrenchinar",
    "emberrenchinarás": "emberrenchinar",
    "emberrenchinaré": "emberrenchinar",
    "emberrenchinaréis": "emberrenchinar",
    "emberrenchinaría": "emberrenchinar",
    "emberrenchinaríais": "emberrenchinar",
    "emberrenchinaríamos": "emberrenchinar",
    "emberrenchinarían": "emberrenchinar",
    "emberrenchinarías": "emberrenchinar",
    "emberrenchinas": "emberrenchinar",
    "emberrenchinase": "emberrenchinar",
    "emberrenchinaseis": "emberrenchinar",
    "emberrenchinasen": "emberrenchinar",
    "emberrenchinases": "emberrenchinar",
    "emberrenchinaste": "emberrenchinar",
    "emberrenchinasteis": "emberrenchinar",
    "emberrenchine": "emberrenchinar",
    "emberrenchinemos": "emberrenchinar",
    "emberrenchinen": "emberrenchinar",
    "emberrenchines": "emberrenchinar",
    "emberrenchino": "emberrenchinar",
    "emberrenchinábamos": "emberrenchinar",
    "emberrenchináis": "emberrenchinar",
    "emberrenchináramos": "emberrenchinar",
    "emberrenchináremos": "emberrenchinar",
    "emberrenchinásemos": "emberrenchinar",
    "emberrenchiné": "emberrenchinar",
    "emberrenchinéis": "emberrenchinar",
    "emberrenchinó": "emberrenchinar",
    "emberrincha": "emberrinchar",
    "emberrinchaba": "emberrinchar",
    "emberrinchabais": "emberrinchar",
    "emberrinchaban": "emberrinchar",
    "emberrinchabas": "emberrinchar",
    "emberrinchad": "emberrinchar",
    "emberrinchada": "emberrinchar",
    "emberrinchadas": "emberrinchar",
    "emberrinchado": "emberrinchar",
    "emberrinchados": "emberrinchar",
    "emberrinchamos": "emberrinchar",
    "emberrinchan": "emberrinchar",
    "emberrinchando": "emberrinchar",
    "emberrinchara": "emberrinchar",
    "emberrincharais": "emberrinchar",
    "emberrincharan": "emberrinchar",
    "emberrincharas": "emberrinchar",
    "emberrinchare": "emberrinchar",
    "emberrinchareis": "emberrinchar",
    "emberrincharemos": "emberrinchar",
    "emberrincharen": "emberrinchar",
    "emberrinchares": "emberrinchar",
    "emberrincharon": "emberrinchar",
    "emberrinchará": "emberrinchar",
    "emberrincharán": "emberrinchar",
    "emberrincharás": "emberrinchar",
    "emberrincharé": "emberrinchar",
    "emberrincharéis": "emberrinchar",
    "emberrincharía": "emberrinchar",
    "emberrincharíais": "emberrinchar",
    "emberrincharíamos": "emberrinchar",
    "emberrincharían": "emberrinchar",
    "emberrincharías": "emberrinchar",
    "emberrinchas": "emberrinchar",
    "emberrinchase": "emberrinchar",
    "emberrinchaseis": "emberrinchar",
    "emberrinchasen": "emberrinchar",
    "emberrinchases": "emberrinchar",
    "emberrinchaste": "emberrinchar",
    "emberrinchasteis": "emberrinchar",
    "emberrinche": "emberrinchar",
    "emberrinchemos": "emberrinchar",
    "emberrinchen": "emberrinchar",
    "emberrinches": "emberrinchar",
    "emberrincho": "emberrinchar",
    "emberrinchábamos": "emberrinchar",
    "emberrincháis": "emberrinchar",
    "emberrincháramos": "emberrinchar",
    "emberrincháremos": "emberrinchar",
    "emberrinchásemos": "emberrinchar",
    "emberrinché": "emberrinchar",
    "emberrinchéis": "emberrinchar",
    "emberrinchó": "emberrinchar",
    "embestid": "embestir",
    "embestida": "embestir",
    "embestidas": "embestir",
    "embestido": "embestir",
    "embestidora": "embestidor",
    "embestidoras": "embestidor",
    "embestidores": "embestidor",
    "embestidos": "embestir",
    "embestiduras": "embestidura",
    "embestimos": "embestir",
    "embestiremos": "embestir",
    "embestirá": "embestir",
    "embestirán": "embestir",
    "embestirás": "embestir",
    "embestiré": "embestir",
    "embestiréis": "embestir",
    "embestiría": "embestir",
    "embestiríais": "embestir",
    "embestiríamos": "embestir",
    "embestirían": "embestir",
    "embestirías": "embestir",
    "embestiste": "embestir",
    "embestisteis": "embestir",
    "embestí": "embestir",
    "embestía": "embestir",
    "embestíais": "embestir",
    "embestíamos": "embestir",
    "embestían": "embestir",
    "embestías": "embestir",
    "embestís": "embestir",
    "embetuna": "embetunar",
    "embetunaba": "embetunar",
    "embetunabais": "embetunar",
    "embetunaban": "embetunar",
    "embetunabas": "embetunar",
    "embetunad": "embetunar",
    "embetunada": "embetunar",
    "embetunadas": "embetunar",
    "embetunado": "embetunar",
    "embetunados": "embetunar",
    "embetunamos": "embetunar",
    "embetunan": "embetunar",
    "embetunando": "embetunar",
    "embetunara": "embetunar",
    "embetunarais": "embetunar",
    "embetunaran": "embetunar",
    "embetunaras": "embetunar",
    "embetunare": "embetunar",
    "embetunareis": "embetunar",
    "embetunaremos": "embetunar",
    "embetunaren": "embetunar",
    "embetunares": "embetunar",
    "embetunaron": "embetunar",
    "embetunará": "embetunar",
    "embetunarán": "embetunar",
    "embetunarás": "embetunar",
    "embetunaré": "embetunar",
    "embetunaréis": "embetunar",
    "embetunaría": "embetunar",
    "embetunaríais": "embetunar",
    "embetunaríamos": "embetunar",
    "embetunarían": "embetunar",
    "embetunarías": "embetunar",
    "embetunas": "embetunar",
    "embetunase": "embetunar",
    "embetunaseis": "embetunar",
    "embetunasen": "embetunar",
    "embetunases": "embetunar",
    "embetunaste": "embetunar",
    "embetunasteis": "embetunar",
    "embetune": "embetunar",
    "embetunemos": "embetunar",
    "embetunen": "embetunar",
    "embetunes": "embetunar",
    "embetuno": "embetunar",
    "embetunábamos": "embetunar",
    "embetunáis": "embetunar",
    "embetunáramos": "embetunar",
    "embetunáremos": "embetunar",
    "embetunásemos": "embetunar",
    "embetuné": "embetunar",
    "embetunéis": "embetunar",
    "embetunó": "embetunar",
    "embica": "embicar",
    "embicaba": "embicar",
    "embicabais": "embicar",
    "embicaban": "embicar",
    "embicabas": "embicar",
    "embicad": "embicar",
    "embicada": "embicar",
    "embicadas": "embicar",
    "embicado": "embicar",
    "embicados": "embicar",
    "embicaduras": "embicadura",
    "embicamos": "embicar",
    "embican": "embicar",
    "embicando": "embicar",
    "embicara": "embicar",
    "embicarais": "embicar",
    "embicaran": "embicar",
    "embicaras": "embicar",
    "embicare": "embicar",
    "embicareis": "embicar",
    "embicaremos": "embicar",
    "embicaren": "embicar",
    "embicares": "embicar",
    "embicaron": "embicar",
    "embicará": "embicar",
    "embicarán": "embicar",
    "embicarás": "embicar",
    "embicaré": "embicar",
    "embicaréis": "embicar",
    "embicaría": "embicar",
    "embicaríais": "embicar",
    "embicaríamos": "embicar",
    "embicarían": "embicar",
    "embicarías": "embicar",
    "embicas": "embicar",
    "embicase": "embicar",
    "embicaseis": "embicar",
    "embicasen": "embicar",
    "embicases": "embicar",
    "embicaste": "embicar",
    "embicasteis": "embicar",
    "embico": "embicar",
    "embicábamos": "embicar",
    "embicáis": "embicar",
    "embicáramos": "embicar",
    "embicáremos": "embicar",
    "embicásemos": "embicar",
    "embicó": "embicar",
    "embija": "embijar",
    "embijaba": "embijar",
    "embijabais": "embijar",
    "embijaban": "embijar",
    "embijabas": "embijar",
    "embijad": "embijar",
    "embijada": "embijar",
    "embijadas": "embijar",
    "embijado": "embijar",
    "embijados": "embijar",
    "embijamos": "embijar",
    "embijan": "embijar",
    "embijando": "embijar",
    "embijara": "embijar",
    "embijarais": "embijar",
    "embijaran": "embijar",
    "embijaras": "embijar",
    "embijare": "embijar",
    "embijareis": "embijar",
    "embijaremos": "embijar",
    "embijaren": "embijar",
    "embijares": "embijar",
    "embijaron": "embijar",
    "embijará": "embijar",
    "embijarán": "embijar",
    "embijarás": "embijar",
    "embijaré": "embijar",
    "embijaréis": "embijar",
    "embijaría": "embijar",
    "embijaríais": "embijar",
    "embijaríamos": "embijar",
    "embijarían": "embijar",
    "embijarías": "embijar",
    "embijas": "embijar",
    "embijase": "embijar",
    "embijaseis": "embijar",
    "embijasen": "embijar",
    "embijases": "embijar",
    "embijaste": "embijar",
    "embijasteis": "embijar",
    "embije": "embijar",
    "embijemos": "embijar",
    "embijen": "embijar",
    "embijes": "embije",
    "embijo": "embijar",
    "embijábamos": "embijar",
    "embijáis": "embijar",
    "embijáramos": "embijar",
    "embijáremos": "embijar",
    "embijásemos": "embijar",
    "embijé": "embijar",
    "embijéis": "embijar",
    "embijó": "embijar",
    "embique": "embicar",
    "embiquemos": "embicar",
    "embiquen": "embicar",
    "embiques": "embicar",
    "embiqué": "embicar",
    "embiquéis": "embicar",
    "embista": "embestir",
    "embistamos": "embestir",
    "embistan": "embestir",
    "embistas": "embestir",
    "embiste": "embestir",
    "embisten": "embestir",
    "embistes": "embestir",
    "embistiendo": "embestir",
    "embistiera": "embestir",
    "embistierais": "embestir",
    "embistieran": "embestir",
    "embistieras": "embestir",
    "embistiere": "embestir",
    "embistiereis": "embestir",
    "embistieren": "embestir",
    "embistieres": "embestir",
    "embistieron": "embestir",
    "embistiese": "embestir",
    "embistieseis": "embestir",
    "embistiesen": "embestir",
    "embistieses": "embestir",
    "embistiéramos": "embestir",
    "embistiéremos": "embestir",
    "embistiésemos": "embestir",
    "embistió": "embestir",
    "embisto": "embestir",
    "embistáis": "embestir",
    "embizca": "embizcar",
    "embizcaba": "embizcar",
    "embizcabais": "embizcar",
    "embizcaban": "embizcar",
    "embizcabas": "embizcar",
    "embizcad": "embizcar",
    "embizcada": "embizcar",
    "embizcadas": "embizcar",
    "embizcado": "embizcar",
    "embizcados": "embizcar",
    "embizcamos": "embizcar",
    "embizcan": "embizcar",
    "embizcando": "embizcar",
    "embizcara": "embizcar",
    "embizcarais": "embizcar",
    "embizcaran": "embizcar",
    "embizcaras": "embizcar",
    "embizcare": "embizcar",
    "embizcareis": "embizcar",
    "embizcaremos": "embizcar",
    "embizcaren": "embizcar",
    "embizcares": "embizcar",
    "embizcaron": "embizcar",
    "embizcará": "embizcar",
    "embizcarán": "embizcar",
    "embizcarás": "embizcar",
    "embizcaré": "embizcar",
    "embizcaréis": "embizcar",
    "embizcaría": "embizcar",
    "embizcaríais": "embizcar",
    "embizcaríamos": "embizcar",
    "embizcarían": "embizcar",
    "embizcarías": "embizcar",
    "embizcas": "embizcar",
    "embizcase": "embizcar",
    "embizcaseis": "embizcar",
    "embizcasen": "embizcar",
    "embizcases": "embizcar",
    "embizcaste": "embizcar",
    "embizcasteis": "embizcar",
    "embizco": "embizcar",
    "embizcábamos": "embizcar",
    "embizcáis": "embizcar",
    "embizcáramos": "embizcar",
    "embizcáremos": "embizcar",
    "embizcásemos": "embizcar",
    "embizcó": "embizcar",
    "embizque": "embizcar",
    "embizquemos": "embizcar",
    "embizquen": "embizcar",
    "embizques": "embizcar",
    "embizqué": "embizcar",
    "embizquéis": "embizcar",
    "emblandece": "emblandecer",
    "emblandeced": "emblandecer",
    "emblandecemos": "emblandecer",
    "emblandecen": "emblandecer",
    "emblandeceremos": "emblandecer",
    "emblandecerá": "emblandecer",
    "emblandecerán": "emblandecer",
    "emblandecerás": "emblandecer",
    "emblandeceré": "emblandecer",
    "emblandeceréis": "emblandecer",
    "emblandecería": "emblandecer",
    "emblandeceríais": "emblandecer",
    "emblandeceríamos": "emblandecer",
    "emblandecerían": "emblandecer",
    "emblandecerías": "emblandecer",
    "emblandeces": "emblandecer",
    "emblandecida": "emblandecer",
    "emblandecidas": "emblandecer",
    "emblandecido": "emblandecer",
    "emblandecidos": "emblandecer",
    "emblandeciendo": "emblandecer",
    "emblandeciera": "emblandecer",
    "emblandecierais": "emblandecer",
    "emblandecieran": "emblandecer",
    "emblandecieras": "emblandecer",
    "emblandeciere": "emblandecer",
    "emblandeciereis": "emblandecer",
    "emblandecieren": "emblandecer",
    "emblandecieres": "emblandecer",
    "emblandecieron": "emblandecer",
    "emblandeciese": "emblandecer",
    "emblandecieseis": "emblandecer",
    "emblandeciesen": "emblandecer",
    "emblandecieses": "emblandecer",
    "emblandecimientos": "emblandecimiento",
    "emblandecimos": "emblandecer",
    "emblandeciste": "emblandecer",
    "emblandecisteis": "emblandecer",
    "emblandeciéramos": "emblandecer",
    "emblandeciéremos": "emblandecer",
    "emblandeciésemos": "emblandecer",
    "emblandeció": "emblandecer",
    "emblandecéis": "emblandecer",
    "emblandecí": "emblandecer",
    "emblandecía": "emblandecer",
    "emblandecíais": "emblandecer",
    "emblandecíamos": "emblandecer",
    "emblandecían": "emblandecer",
    "emblandecías": "emblandecer",
    "emblandezca": "emblandecer",
    "emblandezcamos": "emblandecer",
    "emblandezcan": "emblandecer",
    "emblandezcas": "emblandecer",
    "emblandezco": "emblandecer",
    "emblandezcáis": "emblandecer",
    "emblanquece": "emblanquecer",
    "emblanqueced": "emblanquecer",
    "emblanquecemos": "emblanquecer",
    "emblanquecen": "emblanquecer",
    "emblanqueceremos": "emblanquecer",
    "emblanquecerá": "emblanquecer",
    "emblanquecerán": "emblanquecer",
    "emblanquecerás": "emblanquecer",
    "emblanqueceré": "emblanquecer",
    "emblanqueceréis": "emblanquecer",
    "emblanquecería": "emblanquecer",
    "emblanqueceríais": "emblanquecer",
    "emblanqueceríamos": "emblanquecer",
    "emblanquecerían": "emblanquecer",
    "emblanquecerías": "emblanquecer",
    "emblanqueces": "emblanquecer",
    "emblanquecida": "emblanquecer",
    "emblanquecidas": "emblanquecer",
    "emblanquecido": "emblanquecer",
    "emblanquecidos": "emblanquecer",
    "emblanqueciendo": "emblanquecer",
    "emblanqueciera": "emblanquecer",
    "emblanquecierais": "emblanquecer",
    "emblanquecieran": "emblanquecer",
    "emblanquecieras": "emblanquecer",
    "emblanqueciere": "emblanquecer",
    "emblanqueciereis": "emblanquecer",
    "emblanquecieren": "emblanquecer",
    "emblanquecieres": "emblanquecer",
    "emblanquecieron": "emblanquecer",
    "emblanqueciese": "emblanquecer",
    "emblanquecieseis": "emblanquecer",
    "emblanqueciesen": "emblanquecer",
    "emblanquecieses": "emblanquecer",
    "emblanquecimientos": "emblanquecimiento",
    "emblanquecimos": "emblanquecer",
    "emblanqueciste": "emblanquecer",
    "emblanquecisteis": "emblanquecer",
    "emblanqueciéramos": "emblanquecer",
    "emblanqueciéremos": "emblanquecer",
    "emblanqueciésemos": "emblanquecer",
    "emblanqueció": "emblanquecer",
    "emblanquecéis": "emblanquecer",
    "emblanquecí": "emblanquecer",
    "emblanquecía": "emblanquecer",
    "emblanquecíais": "emblanquecer",
    "emblanquecíamos": "emblanquecer",
    "emblanquecían": "emblanquecer",
    "emblanquecías": "emblanquecer",
    "emblanquezca": "emblanquecer",
    "emblanquezcamos": "emblanquecer",
    "emblanquezcan": "emblanquecer",
    "emblanquezcas": "emblanquecer",
    "emblanquezco": "emblanquecer",
    "emblanquezcáis": "emblanquecer",
    "emblemas": "emblema",
    "emblemática": "emblemático",
    "emblemáticas": "emblemático",
    "emblemáticos": "emblemático",
    "emboba": "embobar",
    "embobaba": "embobar",
    "embobabais": "embobar",
    "embobaban": "embobar",
    "embobabas": "embobar",
    "embobad": "embobar",
    "embobada": "embobar",
    "embobadas": "embobar",
    "embobado": "embobar",
    "embobados": "embobar",
    "embobamientos": "embobamiento",
    "embobamos": "embobar",
    "emboban": "embobar",
    "embobando": "embobar",
    "embobara": "embobar",
    "embobarais": "embobar",
    "embobaran": "embobar",
    "embobaras": "embobar",
    "embobare": "embobar",
    "embobareis": "embobar",
    "embobaremos": "embobar",
    "embobaren": "embobar",
    "embobares": "embobar",
    "embobaron": "embobar",
    "embobará": "embobar",
    "embobarán": "embobar",
    "embobarás": "embobar",
    "embobaré": "embobar",
    "embobaréis": "embobar",
    "embobaría": "embobar",
    "embobaríais": "embobar",
    "embobaríamos": "embobar",
    "embobarían": "embobar",
    "embobarías": "embobar",
    "embobas": "embobar",
    "embobase": "embobar",
    "embobaseis": "embobar",
    "embobasen": "embobar",
    "embobases": "embobar",
    "embobaste": "embobar",
    "embobasteis": "embobar",
    "embobe": "embobar",
    "embobece": "embobecer",
    "embobeced": "embobecer",
    "embobecemos": "embobecer",
    "embobecen": "embobecer",
    "embobeceremos": "embobecer",
    "embobecerá": "embobecer",
    "embobecerán": "embobecer",
    "embobecerás": "embobecer",
    "embobeceré": "embobecer",
    "embobeceréis": "embobecer",
    "embobecería": "embobecer",
    "embobeceríais": "embobecer",
    "embobeceríamos": "embobecer",
    "embobecerían": "embobecer",
    "embobecerías": "embobecer",
    "embobeces": "embobecer",
    "embobecida": "embobecer",
    "embobecidas": "embobecer",
    "embobecido": "embobecer",
    "embobecidos": "embobecer",
    "embobeciendo": "embobecer",
    "embobeciera": "embobecer",
    "embobecierais": "embobecer",
    "embobecieran": "embobecer",
    "embobecieras": "embobecer",
    "embobeciere": "embobecer",
    "embobeciereis": "embobecer",
    "embobecieren": "embobecer",
    "embobecieres": "embobecer",
    "embobecieron": "embobecer",
    "embobeciese": "embobecer",
    "embobecieseis": "embobecer",
    "embobeciesen": "embobecer",
    "embobecieses": "embobecer",
    "embobecimientos": "embobecimiento",
    "embobecimos": "embobecer",
    "embobeciste": "embobecer",
    "embobecisteis": "embobecer",
    "embobeciéramos": "embobecer",
    "embobeciéremos": "embobecer",
    "embobeciésemos": "embobecer",
    "embobeció": "embobecer",
    "embobecéis": "embobecer",
    "embobecí": "embobecer",
    "embobecía": "embobecer",
    "embobecíais": "embobecer",
    "embobecíamos": "embobecer",
    "embobecían": "embobecer",
    "embobecías": "embobecer",
    "embobemos": "embobar",
    "emboben": "embobar",
    "embobes": "embobar",
    "embobezca": "embobecer",
    "embobezcamos": "embobecer",
    "embobezcan": "embobecer",
    "embobezcas": "embobecer",
    "embobezco": "embobecer",
    "embobezcáis": "embobecer",
    "embobo": "embobar",
    "embobábamos": "embobar",
    "embobáis": "embobar",
    "embobáramos": "embobar",
    "embobáremos": "embobar",
    "embobásemos": "embobar",
    "embobé": "embobar",
    "embobéis": "embobar",
    "embobó": "embobar",
    "emboca": "embocar",
    "embocaba": "embocar",
    "embocabais": "embocar",
    "embocaban": "embocar",
    "embocabas": "embocar",
    "embocad": "embocar",
    "embocada": "embocar",
    "embocadas": "embocar",
    "embocaderos": "embocadero",
    "embocado": "embocar",
    "embocados": "embocar",
    "embocaduras": "embocadura",
    "embocamos": "embocar",
    "embocan": "embocar",
    "embocando": "embocar",
    "embocara": "embocar",
    "embocarais": "embocar",
    "embocaran": "embocar",
    "embocaras": "embocar",
    "embocare": "embocar",
    "embocareis": "embocar",
    "embocaremos": "embocar",
    "embocaren": "embocar",
    "embocares": "embocar",
    "embocaron": "embocar",
    "embocará": "embocar",
    "embocarán": "embocar",
    "embocarás": "embocar",
    "embocaré": "embocar",
    "embocaréis": "embocar",
    "embocaría": "embocar",
    "embocaríais": "embocar",
    "embocaríamos": "embocar",
    "embocarían": "embocar",
    "embocarías": "embocar",
    "embocas": "embocar",
    "embocase": "embocar",
    "embocaseis": "embocar",
    "embocasen": "embocar",
    "embocases": "embocar",
    "embocaste": "embocar",
    "embocasteis": "embocar",
    "emboce": "embozar",
    "embocemos": "embozar",
    "embocen": "embozar",
    "emboces": "embozar",
    "emboco": "embocar",
    "embocábamos": "embocar",
    "embocáis": "embocar",
    "embocáramos": "embocar",
    "embocáremos": "embocar",
    "embocásemos": "embocar",
    "embocé": "embozar",
    "embocéis": "embozar",
    "embocó": "embocar",
    "embodega": "embodegar",
    "embodegaba": "embodegar",
    "embodegabais": "embodegar",
    "embodegaban": "embodegar",
    "embodegabas": "embodegar",
    "embodegad": "embodegar",
    "embodegada": "embodegar",
    "embodegadas": "embodegar",
    "embodegado": "embodegar",
    "embodegados": "embodegar",
    "embodegamos": "embodegar",
    "embodegan": "embodegar",
    "embodegando": "embodegar",
    "embodegara": "embodegar",
    "embodegarais": "embodegar",
    "embodegaran": "embodegar",
    "embodegaras": "embodegar",
    "embodegare": "embodegar",
    "embodegareis": "embodegar",
    "embodegaremos": "embodegar",
    "embodegaren": "embodegar",
    "embodegares": "embodegar",
    "embodegaron": "embodegar",
    "embodegará": "embodegar",
    "embodegarán": "embodegar",
    "embodegarás": "embodegar",
    "embodegaré": "embodegar",
    "embodegaréis": "embodegar",
    "embodegaría": "embodegar",
    "embodegaríais": "embodegar",
    "embodegaríamos": "embodegar",
    "embodegarían": "embodegar",
    "embodegarías": "embodegar",
    "embodegas": "embodegar",
    "embodegase": "embodegar",
    "embodegaseis": "embodegar",
    "embodegasen": "embodegar",
    "embodegases": "embodegar",
    "embodegaste": "embodegar",
    "embodegasteis": "embodegar",
    "embodego": "embodegar",
    "embodegue": "embodegar",
    "embodeguemos": "embodegar",
    "embodeguen": "embodegar",
    "embodegues": "embodegar",
    "embodegué": "embodegar",
    "embodeguéis": "embodegar",
    "embodegábamos": "embodegar",
    "embodegáis": "embodegar",
    "embodegáramos": "embodegar",
    "embodegáremos": "embodegar",
    "embodegásemos": "embodegar",
    "embodegó": "embodegar",
    "emboja": "embojar",
    "embojaba": "embojar",
    "embojabais": "embojar",
    "embojaban": "embojar",
    "embojabas": "embojar",
    "embojad": "embojar",
    "embojada": "embojar",
    "embojadas": "embojar",
    "embojado": "embojar",
    "embojados": "embojar",
    "embojamos": "embojar",
    "embojan": "embojar",
    "embojando": "embojar",
    "embojara": "embojar",
    "embojarais": "embojar",
    "embojaran": "embojar",
    "embojaras": "embojar",
    "embojare": "embojar",
    "embojareis": "embojar",
    "embojaremos": "embojar",
    "embojaren": "embojar",
    "embojares": "embojar",
    "embojaron": "embojar",
    "embojará": "embojar",
    "embojarán": "embojar",
    "embojarás": "embojar",
    "embojaré": "embojar",
    "embojaréis": "embojar",
    "embojaría": "embojar",
    "embojaríais": "embojar",
    "embojaríamos": "embojar",
    "embojarían": "embojar",
    "embojarías": "embojar",
    "embojas": "embojar",
    "embojase": "embojar",
    "embojaseis": "embojar",
    "embojasen": "embojar",
    "embojases": "embojar",
    "embojaste": "embojar",
    "embojasteis": "embojar",
    "emboje": "embojar",
    "embojemos": "embojar",
    "embojen": "embojar",
    "embojes": "embojar",
    "embojo": "embojar",
    "embojos": "embojo",
    "embojábamos": "embojar",
    "embojáis": "embojar",
    "embojáramos": "embojar",
    "embojáremos": "embojar",
    "embojásemos": "embojar",
    "embojé": "embojar",
    "embojéis": "embojar",
    "embojó": "embojar",
    "embola": "embolar",
    "embolaba": "embolar",
    "embolabais": "embolar",
    "embolaban": "embolar",
    "embolabas": "embolar",
    "embolad": "embolar",
    "embolada": "embolar",
    "emboladas": "embolar",
    "embolado": "embolar",
    "embolados": "embolar",
    "embolamos": "embolar",
    "embolan": "embolar",
    "embolando": "embolar",
    "embolara": "embolar",
    "embolarais": "embolar",
    "embolaran": "embolar",
    "embolaras": "embolar",
    "embolare": "embolar",
    "embolareis": "embolar",
    "embolaremos": "embolar",
    "embolaren": "embolar",
    "embolares": "embolar",
    "embolaron": "embolar",
    "embolará": "embolar",
    "embolarán": "embolar",
    "embolarás": "embolar",
    "embolaré": "embolar",
    "embolaréis": "embolar",
    "embolaría": "embolar",
    "embolaríais": "embolar",
    "embolaríamos": "embolar",
    "embolarían": "embolar",
    "embolarías": "embolar",
    "embolas": "embolar",
    "embolase": "embolar",
    "embolaseis": "embolar",
    "embolasen": "embolar",
    "embolases": "embolar",
    "embolaste": "embolar",
    "embolasteis": "embolar",
    "embole": "embolar",
    "embolectomías": "embolectomía",
    "embolemos": "embolar",
    "embolen": "embolar",
    "emboles": "embolar",
    "embolias": "embolia",
    "emboliformes": "emboliforme",
    "embolisma": "embolismar",
    "embolismaba": "embolismar",
    "embolismabais": "embolismar",
    "embolismaban": "embolismar",
    "embolismabas": "embolismar",
    "embolismad": "embolismar",
    "embolismada": "embolismar",
    "embolismadas": "embolismar",
    "embolismado": "embolismar",
    "embolismados": "embolismar",
    "embolismamos": "embolismar",
    "embolisman": "embolismar",
    "embolismando": "embolismar",
    "embolismara": "embolismar",
    "embolismarais": "embolismar",
    "embolismaran": "embolismar",
    "embolismaras": "embolismar",
    "embolismare": "embolismar",
    "embolismareis": "embolismar",
    "embolismaremos": "embolismar",
    "embolismaren": "embolismar",
    "embolismares": "embolismar",
    "embolismaron": "embolismar",
    "embolismará": "embolismar",
    "embolismarán": "embolismar",
    "embolismarás": "embolismar",
    "embolismaré": "embolismar",
    "embolismaréis": "embolismar",
    "embolismaría": "embolismar",
    "embolismaríais": "embolismar",
    "embolismaríamos": "embolismar",
    "embolismarían": "embolismar",
    "embolismarías": "embolismar",
    "embolismas": "embolismar",
    "embolismase": "embolismar",
    "embolismaseis": "embolismar",
    "embolismasen": "embolismar",
    "embolismases": "embolismar",
    "embolismaste": "embolismar",
    "embolismasteis": "embolismar",
    "embolisme": "embolismar",
    "embolismemos": "embolismar",
    "embolismen": "embolismar",
    "embolismes": "embolismar",
    "embolismo": "embolismar",
    "embolismos": "embolismo",
    "embolismábamos": "embolismar",
    "embolismáis": "embolismar",
    "embolismáramos": "embolismar",
    "embolismáremos": "embolismar",
    "embolismásemos": "embolismar",
    "embolismática": "embolismático",
    "embolismáticas": "embolismático",
    "embolismáticos": "embolismático",
    "embolismé": "embolismar",
    "embolisméis": "embolismar",
    "embolismó": "embolismar",
    "embolo": "embolar",
    "embolsa": "embolsar",
    "embolsaba": "embolsar",
    "embolsabais": "embolsar",
    "embolsaban": "embolsar",
    "embolsabas": "embolsar",
    "embolsad": "embolsar",
    "embolsada": "embolsar",
    "embolsadas": "embolsar",
    "embolsado": "embolsar",
    "embolsados": "embolsar",
    "embolsamos": "embolsar",
    "embolsan": "embolsar",
    "embolsando": "embolsar",
    "embolsara": "embolsar",
    "embolsarais": "embolsar",
    "embolsaran": "embolsar",
    "embolsaras": "embolsar",
    "embolsare": "embolsar",
    "embolsareis": "embolsar",
    "embolsaremos": "embolsar",
    "embolsaren": "embolsar",
    "embolsares": "embolsar",
    "embolsaron": "embolsar",
    "embolsará": "embolsar",
    "embolsarán": "embolsar",
    "embolsarás": "embolsar",
    "embolsaré": "embolsar",
    "embolsaréis": "embolsar",
    "embolsaría": "embolsar",
    "embolsaríais": "embolsar",
    "embolsaríamos": "embolsar",
    "embolsarían": "embolsar",
    "embolsarías": "embolsar",
    "embolsas": "embolsar",
    "embolsase": "embolsar",
    "embolsaseis": "embolsar",
    "embolsasen": "embolsar",
    "embolsases": "embolsar",
    "embolsaste": "embolsar",
    "embolsasteis": "embolsar",
    "embolse": "embolsar",
    "embolsemos": "embolsar",
    "embolsen": "embolsar",
    "embolses": "embolsar",
    "embolso": "embolsar",
    "embolsos": "embolso",
    "embolsábamos": "embolsar",
    "embolsáis": "embolsar",
    "embolsáramos": "embolsar",
    "embolsáremos": "embolsar",
    "embolsásemos": "embolsar",
    "embolsé": "embolsar",
    "embolséis": "embolsar",
    "embolsó": "embolsar",
    "embolábamos": "embolar",
    "emboláis": "embolar",
    "emboláramos": "embolar",
    "emboláremos": "embolar",
    "embolásemos": "embolar",
    "embolé": "embolar",
    "emboléis": "embolar",
    "emboló": "embolar",
    "embona": "embonar",
    "embonaba": "embonar",
    "embonabais": "embonar",
    "embonaban": "embonar",
    "embonabas": "embonar",
    "embonad": "embonar",
    "embonada": "embonar",
    "embonadas": "embonar",
    "embonado": "embonar",
    "embonados": "embonar",
    "embonamos": "embonar",
    "embonan": "embonar",
    "embonando": "embonar",
    "embonara": "embonar",
    "embonarais": "embonar",
    "embonaran": "embonar",
    "embonaras": "embonar",
    "embonare": "embonar",
    "embonareis": "embonar",
    "embonaremos": "embonar",
    "embonaren": "embonar",
    "embonares": "embonar",
    "embonaron": "embonar",
    "embonará": "embonar",
    "embonarán": "embonar",
    "embonarás": "embonar",
    "embonaré": "embonar",
    "embonaréis": "embonar",
    "embonaría": "embonar",
    "embonaríais": "embonar",
    "embonaríamos": "embonar",
    "embonarían": "embonar",
    "embonarías": "embonar",
    "embonas": "embonar",
    "embonase": "embonar",
    "embonaseis": "embonar",
    "embonasen": "embonar",
    "embonases": "embonar",
    "embonaste": "embonar",
    "embonasteis": "embonar",
    "embone": "embonar",
    "embonemos": "embonar",
    "embonen": "embonar",
    "embones": "embonar",
    "embono": "embonar",
    "embonos": "embono",
    "embonábamos": "embonar",
    "embonáis": "embonar",
    "embonáramos": "embonar",
    "embonáremos": "embonar",
    "embonásemos": "embonar",
    "emboné": "embonar",
    "embonéis": "embonar",
    "embonó": "embonar",
    "emboque": "embocar",
    "emboquemos": "embocar",
    "emboquen": "embocar",
    "emboques": "emboque",
    "emboquilla": "emboquillar",
    "emboquillaba": "emboquillar",
    "emboquillabais": "emboquillar",
    "emboquillaban": "emboquillar",
    "emboquillabas": "emboquillar",
    "emboquillad": "emboquillar",
    "emboquillada": "emboquillar",
    "emboquilladas": "emboquillar",
    "emboquillado": "emboquillar",
    "emboquillados": "emboquillar",
    "emboquillamos": "emboquillar",
    "emboquillan": "emboquillar",
    "emboquillando": "emboquillar",
    "emboquillara": "emboquillar",
    "emboquillarais": "emboquillar",
    "emboquillaran": "emboquillar",
    "emboquillaras": "emboquillar",
    "emboquillare": "emboquillar",
    "emboquillareis": "emboquillar",
    "emboquillaremos": "emboquillar",
    "emboquillaren": "emboquillar",
    "emboquillares": "emboquillar",
    "emboquillaron": "emboquillar",
    "emboquillará": "emboquillar",
    "emboquillarán": "emboquillar",
    "emboquillarás": "emboquillar",
    "emboquillaré": "emboquillar",
    "emboquillaréis": "emboquillar",
    "emboquillaría": "emboquillar",
    "emboquillaríais": "emboquillar",
    "emboquillaríamos": "emboquillar",
    "emboquillarían": "emboquillar",
    "emboquillarías": "emboquillar",
    "emboquillas": "emboquillar",
    "emboquillase": "emboquillar",
    "emboquillaseis": "emboquillar",
    "emboquillasen": "emboquillar",
    "emboquillases": "emboquillar",
    "emboquillaste": "emboquillar",
    "emboquillasteis": "emboquillar",
    "emboquille": "emboquillar",
    "emboquillemos": "emboquillar",
    "emboquillen": "emboquillar",
    "emboquilles": "emboquillar",
    "emboquillo": "emboquillar",
    "emboquillábamos": "emboquillar",
    "emboquilláis": "emboquillar",
    "emboquilláramos": "emboquillar",
    "emboquilláremos": "emboquillar",
    "emboquillásemos": "emboquillar",
    "emboquillé": "emboquillar",
    "emboquilléis": "emboquillar",
    "emboquilló": "emboquillar",
    "emboqué": "embocar",
    "emboquéis": "embocar",
    "embornales": "embornal",
    "emborra": "emborrar",
    "emborraba": "emborrar",
    "emborrabais": "emborrar",
    "emborraban": "emborrar",
    "emborrabas": "emborrar",
    "emborrace": "emborrazar",
    "emborracemos": "emborrazar",
    "emborracen": "emborrazar",
    "emborraces": "emborrazar",
    "emborracha": "emborrachar",
    "emborrachaba": "emborrachar",
    "emborrachabais": "emborrachar",
    "emborrachaban": "emborrachar",
    "emborrachabas": "emborrachar",
    "emborrachad": "emborrachar",
    "emborrachada": "emborrachar",
    "emborrachadas": "emborrachar",
    "emborrachado": "emborrachar",
    "emborrachadora": "emborrachador",
    "emborrachadoras": "emborrachador",
    "emborrachadores": "emborrachador",
    "emborrachados": "emborrachar",
    "emborrachamientos": "emborrachamiento",
    "emborrachamos": "emborrachar",
    "emborrachan": "emborrachar",
    "emborrachando": "emborrachar",
    "emborrachara": "emborrachar",
    "emborracharais": "emborrachar",
    "emborracharan": "emborrachar",
    "emborracharas": "emborrachar",
    "emborrachare": "emborrachar",
    "emborrachareis": "emborrachar",
    "emborracharemos": "emborrachar",
    "emborracharen": "emborrachar",
    "emborrachares": "emborrachar",
    "emborracharon": "emborrachar",
    "emborrachará": "emborrachar",
    "emborracharán": "emborrachar",
    "emborracharás": "emborrachar",
    "emborracharé": "emborrachar",
    "emborracharéis": "emborrachar",
    "emborracharía": "emborrachar",
    "emborracharíais": "emborrachar",
    "emborracharíamos": "emborrachar",
    "emborracharían": "emborrachar",
    "emborracharías": "emborrachar",
    "emborrachas": "emborrachar",
    "emborrachase": "emborrachar",
    "emborrachaseis": "emborrachar",
    "emborrachasen": "emborrachar",
    "emborrachases": "emborrachar",
    "emborrachaste": "emborrachar",
    "emborrachasteis": "emborrachar",
    "emborrache": "emborrachar",
    "emborrachemos": "emborrachar",
    "emborrachen": "emborrachar",
    "emborraches": "emborrachar",
    "emborracho": "emborrachar",
    "emborrachábamos": "emborrachar",
    "emborracháis": "emborrachar",
    "emborracháramos": "emborrachar",
    "emborracháremos": "emborrachar",
    "emborrachásemos": "emborrachar",
    "emborraché": "emborrachar",
    "emborrachéis": "emborrachar",
    "emborrachó": "emborrachar",
    "emborracé": "emborrazar",
    "emborracéis": "emborrazar",
    "emborrad": "emborrar",
    "emborrada": "emborrar",
    "emborradas": "emborrar",
    "emborrado": "emborrar",
    "emborradora": "emborrador",
    "emborradoras": "emborradora",
    "emborradores": "emborrador",
    "emborrados": "emborrar",
    "emborraduras": "emborradura",
    "emborramos": "emborrar",
    "emborran": "emborrar",
    "emborrando": "emborrar",
    "emborrara": "emborrar",
    "emborrarais": "emborrar",
    "emborraran": "emborrar",
    "emborraras": "emborrar",
    "emborrare": "emborrar",
    "emborrareis": "emborrar",
    "emborraremos": "emborrar",
    "emborraren": "emborrar",
    "emborrares": "emborrar",
    "emborraron": "emborrar",
    "emborrará": "emborrar",
    "emborrarán": "emborrar",
    "emborrarás": "emborrar",
    "emborraré": "emborrar",
    "emborraréis": "emborrar",
    "emborraría": "emborrar",
    "emborraríais": "emborrar",
    "emborraríamos": "emborrar",
    "emborrarían": "emborrar",
    "emborrarías": "emborrar",
    "emborras": "emborrar",
    "emborrasca": "emborrascar",
    "emborrascaba": "emborrascar",
    "emborrascabais": "emborrascar",
    "emborrascaban": "emborrascar",
    "emborrascabas": "emborrascar",
    "emborrascad": "emborrascar",
    "emborrascada": "emborrascar",
    "emborrascadas": "emborrascar",
    "emborrascado": "emborrascar",
    "emborrascados": "emborrascar",
    "emborrascamos": "emborrascar",
    "emborrascan": "emborrascar",
    "emborrascando": "emborrascar",
    "emborrascara": "emborrascar",
    "emborrascarais": "emborrascar",
    "emborrascaran": "emborrascar",
    "emborrascaras": "emborrascar",
    "emborrascare": "emborrascar",
    "emborrascareis": "emborrascar",
    "emborrascaremos": "emborrascar",
    "emborrascaren": "emborrascar",
    "emborrascares": "emborrascar",
    "emborrascaron": "emborrascar",
    "emborrascará": "emborrascar",
    "emborrascarán": "emborrascar",
    "emborrascarás": "emborrascar",
    "emborrascaré": "emborrascar",
    "emborrascaréis": "emborrascar",
    "emborrascaría": "emborrascar",
    "emborrascaríais": "emborrascar",
    "emborrascaríamos": "emborrascar",
    "emborrascarían": "emborrascar",
    "emborrascarías": "emborrascar",
    "emborrascas": "emborrascar",
    "emborrascase": "emborrascar",
    "emborrascaseis": "emborrascar",
    "emborrascasen": "emborrascar",
    "emborrascases": "emborrascar",
    "emborrascaste": "emborrascar",
    "emborrascasteis": "emborrascar",
    "emborrasco": "emborrascar",
    "emborrascábamos": "emborrascar",
    "emborrascáis": "emborrascar",
    "emborrascáramos": "emborrascar",
    "emborrascáremos": "emborrascar",
    "emborrascásemos": "emborrascar",
    "emborrascó": "emborrascar",
    "emborrase": "emborrar",
    "emborraseis": "emborrar",
    "emborrasen": "emborrar",
    "emborrases": "emborrar",
    "emborrasque": "emborrascar",
    "emborrasquemos": "emborrascar",
    "emborrasquen": "emborrascar",
    "emborrasques": "emborrascar",
    "emborrasqué": "emborrascar",
    "emborrasquéis": "emborrascar",
    "emborraste": "emborrar",
    "emborrasteis": "emborrar",
    "emborraza": "emborrazar",
    "emborrazaba": "emborrazar",
    "emborrazabais": "emborrazar",
    "emborrazaban": "emborrazar",
    "emborrazabas": "emborrazar",
    "emborrazad": "emborrazar",
    "emborrazada": "emborrazar",
    "emborrazadas": "emborrazar",
    "emborrazado": "emborrazar",
    "emborrazados": "emborrazar",
    "emborrazamientos": "emborrazamiento",
    "emborrazamos": "emborrazar",
    "emborrazan": "emborrazar",
    "emborrazando": "emborrazar",
    "emborrazara": "emborrazar",
    "emborrazarais": "emborrazar",
    "emborrazaran": "emborrazar",
    "emborrazaras": "emborrazar",
    "emborrazare": "emborrazar",
    "emborrazareis": "emborrazar",
    "emborrazaremos": "emborrazar",
    "emborrazaren": "emborrazar",
    "emborrazares": "emborrazar",
    "emborrazaron": "emborrazar",
    "emborrazará": "emborrazar",
    "emborrazarán": "emborrazar",
    "emborrazarás": "emborrazar",
    "emborrazaré": "emborrazar",
    "emborrazaréis": "emborrazar",
    "emborrazaría": "emborrazar",
    "emborrazaríais": "emborrazar",
    "emborrazaríamos": "emborrazar",
    "emborrazarían": "emborrazar",
    "emborrazarías": "emborrazar",
    "emborrazas": "emborrazar",
    "emborrazase": "emborrazar",
    "emborrazaseis": "emborrazar",
    "emborrazasen": "emborrazar",
    "emborrazases": "emborrazar",
    "emborrazaste": "emborrazar",
    "emborrazasteis": "emborrazar",
    "emborrazo": "emborrazar",
    "emborrazábamos": "emborrazar",
    "emborrazáis": "emborrazar",
    "emborrazáramos": "emborrazar",
    "emborrazáremos": "emborrazar",
    "emborrazásemos": "emborrazar",
    "emborrazó": "emborrazar",
    "emborre": "emborrar",
    "emborremos": "emborrar",
    "emborren": "emborrar",
    "emborres": "emborrar",
    "emborrice": "emborrizar",
    "emborricemos": "emborrizar",
    "emborricen": "emborrizar",
    "emborrices": "emborrizar",
    "emborricé": "emborrizar",
    "emborricéis": "emborrizar",
    "emborriza": "emborrizar",
    "emborrizaba": "emborrizar",
    "emborrizabais": "emborrizar",
    "emborrizaban": "emborrizar",
    "emborrizabas": "emborrizar",
    "emborrizad": "emborrizar",
    "emborrizada": "emborrizar",
    "emborrizadas": "emborrizar",
    "emborrizado": "emborrizar",
    "emborrizados": "emborrizar",
    "emborrizamos": "emborrizar",
    "emborrizan": "emborrizar",
    "emborrizando": "emborrizar",
    "emborrizara": "emborrizar",
    "emborrizarais": "emborrizar",
    "emborrizaran": "emborrizar",
    "emborrizaras": "emborrizar",
    "emborrizare": "emborrizar",
    "emborrizareis": "emborrizar",
    "emborrizaremos": "emborrizar",
    "emborrizaren": "emborrizar",
    "emborrizares": "emborrizar",
    "emborrizaron": "emborrizar",
    "emborrizará": "emborrizar",
    "emborrizarán": "emborrizar",
    "emborrizarás": "emborrizar",
    "emborrizaré": "emborrizar",
    "emborrizaréis": "emborrizar",
    "emborrizaría": "emborrizar",
    "emborrizaríais": "emborrizar",
    "emborrizaríamos": "emborrizar",
    "emborrizarían": "emborrizar",
    "emborrizarías": "emborrizar",
    "emborrizas": "emborrizar",
    "emborrizase": "emborrizar",
    "emborrizaseis": "emborrizar",
    "emborrizasen": "emborrizar",
    "emborrizases": "emborrizar",
    "emborrizaste": "emborrizar",
    "emborrizasteis": "emborrizar",
    "emborrizo": "emborrizar",
    "emborrizábamos": "emborrizar",
    "emborrizáis": "emborrizar",
    "emborrizáramos": "emborrizar",
    "emborrizáremos": "emborrizar",
    "emborrizásemos": "emborrizar",
    "emborrizó": "emborrizar",
    "emborro": "emborrar",
    "emborrona": "emborronar",
    "emborronaba": "emborronar",
    "emborronabais": "emborronar",
    "emborronaban": "emborronar",
    "emborronabas": "emborronar",
    "emborronad": "emborronar",
    "emborronada": "emborronar",
    "emborronadas": "emborronar",
    "emborronado": "emborronar",
    "emborronadora": "emborronador",
    "emborronadoras": "emborronador",
    "emborronadores": "emborronador",
    "emborronados": "emborronar",
    "emborronamos": "emborronar",
    "emborronan": "emborronar",
    "emborronando": "emborronar",
    "emborronara": "emborronar",
    "emborronarais": "emborronar",
    "emborronaran": "emborronar",
    "emborronaras": "emborronar",
    "emborronare": "emborronar",
    "emborronareis": "emborronar",
    "emborronaremos": "emborronar",
    "emborronaren": "emborronar",
    "emborronares": "emborronar",
    "emborronaron": "emborronar",
    "emborronará": "emborronar",
    "emborronarán": "emborronar",
    "emborronarás": "emborronar",
    "emborronaré": "emborronar",
    "emborronaréis": "emborronar",
    "emborronaría": "emborronar",
    "emborronaríais": "emborronar",
    "emborronaríamos": "emborronar",
    "emborronarían": "emborronar",
    "emborronarías": "emborronar",
    "emborronas": "emborronar",
    "emborronase": "emborronar",
    "emborronaseis": "emborronar",
    "emborronasen": "emborronar",
    "emborronases": "emborronar",
    "emborronaste": "emborronar",
    "emborronasteis": "emborronar",
    "emborrone": "emborronar",
    "emborronemos": "emborronar",
    "emborronen": "emborronar",
    "emborrones": "emborronar",
    "emborrono": "emborronar",
    "emborronábamos": "emborronar",
    "emborronáis": "emborronar",
    "emborronáramos": "emborronar",
    "emborronáremos": "emborronar",
    "emborronásemos": "emborronar",
    "emborroné": "emborronar",
    "emborronéis": "emborronar",
    "emborronó": "emborronar",
    "emborrábamos": "emborrar",
    "emborráis": "emborrar",
    "emborráramos": "emborrar",
    "emborráremos": "emborrar",
    "emborrásemos": "emborrar",
    "emborré": "emborrar",
    "emborréis": "emborrar",
    "emborró": "emborrar",
    "embosca": "emboscar",
    "emboscaba": "emboscar",
    "emboscabais": "emboscar",
    "emboscaban": "emboscar",
    "emboscabas": "emboscar",
    "emboscad": "emboscar",
    "emboscada": "emboscar",
    "emboscadas": "emboscar",
    "emboscado": "emboscar",
    "emboscados": "emboscar",
    "emboscaduras": "emboscadura",
    "emboscamos": "emboscar",
    "emboscan": "emboscar",
    "emboscando": "emboscar",
    "emboscara": "emboscar",
    "emboscarais": "emboscar",
    "emboscaran": "emboscar",
    "emboscaras": "emboscar",
    "emboscare": "emboscar",
    "emboscareis": "emboscar",
    "emboscaremos": "emboscar",
    "emboscaren": "emboscar",
    "emboscares": "emboscar",
    "emboscaron": "emboscar",
    "emboscará": "emboscar",
    "emboscarán": "emboscar",
    "emboscarás": "emboscar",
    "emboscaré": "emboscar",
    "emboscaréis": "emboscar",
    "emboscaría": "emboscar",
    "emboscaríais": "emboscar",
    "emboscaríamos": "emboscar",
    "emboscarían": "emboscar",
    "emboscarías": "emboscar",
    "emboscas": "emboscar",
    "emboscase": "emboscar",
    "emboscaseis": "emboscar",
    "emboscasen": "emboscar",
    "emboscases": "emboscar",
    "emboscaste": "emboscar",
    "emboscasteis": "emboscar",
    "embosco": "emboscar",
    "emboscábamos": "emboscar",
    "emboscáis": "emboscar",
    "emboscáramos": "emboscar",
    "emboscáremos": "emboscar",
    "emboscásemos": "emboscar",
    "emboscó": "emboscar",
    "embosque": "emboscar",
    "embosquece": "embosquecer",
    "embosqueced": "embosquecer",
    "embosquecemos": "embosquecer",
    "embosquecen": "embosquecer",
    "embosqueceremos": "embosquecer",
    "embosquecerá": "embosquecer",
    "embosquecerán": "embosquecer",
    "embosquecerás": "embosquecer",
    "embosqueceré": "embosquecer",
    "embosqueceréis": "embosquecer",
    "embosquecería": "embosquecer",
    "embosqueceríais": "embosquecer",
    "embosqueceríamos": "embosquecer",
    "embosquecerían": "embosquecer",
    "embosquecerías": "embosquecer",
    "embosqueces": "embosquecer",
    "embosquecida": "embosquecer",
    "embosquecidas": "embosquecer",
    "embosquecido": "embosquecer",
    "embosquecidos": "embosquecer",
    "embosqueciendo": "embosquecer",
    "embosqueciera": "embosquecer",
    "embosquecierais": "embosquecer",
    "embosquecieran": "embosquecer",
    "embosquecieras": "embosquecer",
    "embosqueciere": "embosquecer",
    "embosqueciereis": "embosquecer",
    "embosquecieren": "embosquecer",
    "embosquecieres": "embosquecer",
    "embosquecieron": "embosquecer",
    "embosqueciese": "embosquecer",
    "embosquecieseis": "embosquecer",
    "embosqueciesen": "embosquecer",
    "embosquecieses": "embosquecer",
    "embosquecimos": "embosquecer",
    "embosqueciste": "embosquecer",
    "embosquecisteis": "embosquecer",
    "embosqueciéramos": "embosquecer",
    "embosqueciéremos": "embosquecer",
    "embosqueciésemos": "embosquecer",
    "embosqueció": "embosquecer",
    "embosquecéis": "embosquecer",
    "embosquecí": "embosquecer",
    "embosquecía": "embosquecer",
    "embosquecíais": "embosquecer",
    "embosquecíamos": "embosquecer",
    "embosquecían": "embosquecer",
    "embosquecías": "embosquecer",
    "embosquemos": "emboscar",
    "embosquen": "emboscar",
    "embosques": "emboscar",
    "embosquezca": "embosquecer",
    "embosquezcamos": "embosquecer",
    "embosquezcan": "embosquecer",
    "embosquezcas": "embosquecer",
    "embosquezco": "embosquecer",
    "embosquezcáis": "embosquecer",
    "embosqué": "emboscar",
    "embosquéis": "emboscar",
    "embosta": "embostar",
    "embostaba": "embostar",
    "embostabais": "embostar",
    "embostaban": "embostar",
    "embostabas": "embostar",
    "embostad": "embostar",
    "embostada": "embostar",
    "embostadas": "embostar",
    "embostado": "embostar",
    "embostados": "embostar",
    "embostamos": "embostar",
    "embostan": "embostar",
    "embostando": "embostar",
    "embostara": "embostar",
    "embostarais": "embostar",
    "embostaran": "embostar",
    "embostaras": "embostar",
    "embostare": "embostar",
    "embostareis": "embostar",
    "embostaremos": "embostar",
    "embostaren": "embostar",
    "embostares": "embostar",
    "embostaron": "embostar",
    "embostará": "embostar",
    "embostarán": "embostar",
    "embostarás": "embostar",
    "embostaré": "embostar",
    "embostaréis": "embostar",
    "embostaría": "embostar",
    "embostaríais": "embostar",
    "embostaríamos": "embostar",
    "embostarían": "embostar",
    "embostarías": "embostar",
    "embostas": "embostar",
    "embostase": "embostar",
    "embostaseis": "embostar",
    "embostasen": "embostar",
    "embostases": "embostar",
    "embostaste": "embostar",
    "embostasteis": "embostar",
    "emboste": "embostar",
    "embostemos": "embostar",
    "embosten": "embostar",
    "embostes": "embostar",
    "embosto": "embostar",
    "embostábamos": "embostar",
    "embostáis": "embostar",
    "embostáramos": "embostar",
    "embostáremos": "embostar",
    "embostásemos": "embostar",
    "embosté": "embostar",
    "embostéis": "embostar",
    "embostó": "embostar",
    "embota": "embotar",
    "embotaba": "embotar",
    "embotabais": "embotar",
    "embotaban": "embotar",
    "embotabas": "embotar",
    "embotad": "embotar",
    "embotada": "embotar",
    "embotadas": "embotar",
    "embotado": "embotar",
    "embotadora": "embotador",
    "embotadoras": "embotador",
    "embotadores": "embotador",
    "embotados": "embotar",
    "embotaduras": "embotadura",
    "embotamientos": "embotamiento",
    "embotamos": "embotar",
    "embotan": "embotar",
    "embotando": "embotar",
    "embotara": "embotar",
    "embotarais": "embotar",
    "embotaran": "embotar",
    "embotaras": "embotar",
    "embotare": "embotar",
    "embotareis": "embotar",
    "embotaremos": "embotar",
    "embotaren": "embotar",
    "embotares": "embotar",
    "embotaron": "embotar",
    "embotará": "embotar",
    "embotarán": "embotar",
    "embotarás": "embotar",
    "embotaré": "embotar",
    "embotaréis": "embotar",
    "embotaría": "embotar",
    "embotaríais": "embotar",
    "embotaríamos": "embotar",
    "embotarían": "embotar",
    "embotarías": "embotar",
    "embotas": "embotar",
    "embotase": "embotar",
    "embotaseis": "embotar",
    "embotasen": "embotar",
    "embotases": "embotar",
    "embotaste": "embotar",
    "embotasteis": "embotar",
    "embote": "embotar",
    "embotella": "embotellar",
    "embotellaba": "embotellar",
    "embotellabais": "embotellar",
    "embotellaban": "embotellar",
    "embotellabas": "embotellar",
    "embotellad": "embotellar",
    "embotellada": "embotellar",
    "embotelladas": "embotellar",
    "embotellado": "embotellar",
    "embotelladora": "embotellador",
    "embotelladoras": "embotelladora",
    "embotelladores": "embotellador",
    "embotellados": "embotellar",
    "embotellamientos": "embotellamiento",
    "embotellamos": "embotellar",
    "embotellan": "embotellar",
    "embotellando": "embotellar",
    "embotellara": "embotellar",
    "embotellarais": "embotellar",
    "embotellaran": "embotellar",
    "embotellaras": "embotellar",
    "embotellare": "embotellar",
    "embotellareis": "embotellar",
    "embotellaremos": "embotellar",
    "embotellaren": "embotellar",
    "embotellares": "embotellar",
    "embotellaron": "embotellar",
    "embotellará": "embotellar",
    "embotellarán": "embotellar",
    "embotellarás": "embotellar",
    "embotellaré": "embotellar",
    "embotellaréis": "embotellar",
    "embotellaría": "embotellar",
    "embotellaríais": "embotellar",
    "embotellaríamos": "embotellar",
    "embotellarían": "embotellar",
    "embotellarías": "embotellar",
    "embotellas": "embotellar",
    "embotellase": "embotellar",
    "embotellaseis": "embotellar",
    "embotellasen": "embotellar",
    "embotellases": "embotellar",
    "embotellaste": "embotellar",
    "embotellasteis": "embotellar",
    "embotelle": "embotellar",
    "embotellemos": "embotellar",
    "embotellen": "embotellar",
    "embotelles": "embotellar",
    "embotello": "embotellar",
    "embotellábamos": "embotellar",
    "embotelláis": "embotellar",
    "embotelláramos": "embotellar",
    "embotelláremos": "embotellar",
    "embotellásemos": "embotellar",
    "embotellé": "embotellar",
    "embotelléis": "embotellar",
    "embotelló": "embotellar",
    "embotemos": "embotar",
    "emboten": "embotar",
    "embotes": "embotar",
    "embotija": "embotijar",
    "embotijaba": "embotijar",
    "embotijabais": "embotijar",
    "embotijaban": "embotijar",
    "embotijabas": "embotijar",
    "embotijad": "embotijar",
    "embotijada": "embotijar",
    "embotijadas": "embotijar",
    "embotijado": "embotijar",
    "embotijados": "embotijar",
    "embotijamos": "embotijar",
    "embotijan": "embotijar",
    "embotijando": "embotijar",
    "embotijara": "embotijar",
    "embotijarais": "embotijar",
    "embotijaran": "embotijar",
    "embotijaras": "embotijar",
    "embotijare": "embotijar",
    "embotijareis": "embotijar",
    "embotijaremos": "embotijar",
    "embotijaren": "embotijar",
    "embotijares": "embotijar",
    "embotijaron": "embotijar",
    "embotijará": "embotijar",
    "embotijarán": "embotijar",
    "embotijarás": "embotijar",
    "embotijaré": "embotijar",
    "embotijaréis": "embotijar",
    "embotijaría": "embotijar",
    "embotijaríais": "embotijar",
    "embotijaríamos": "embotijar",
    "embotijarían": "embotijar",
    "embotijarías": "embotijar",
    "embotijas": "embotijar",
    "embotijase": "embotijar",
    "embotijaseis": "embotijar",
    "embotijasen": "embotijar",
    "embotijases": "embotijar",
    "embotijaste": "embotijar",
    "embotijasteis": "embotijar",
    "embotije": "embotijar",
    "embotijemos": "embotijar",
    "embotijen": "embotijar",
    "embotijes": "embotijar",
    "embotijo": "embotijar",
    "embotijábamos": "embotijar",
    "embotijáis": "embotijar",
    "embotijáramos": "embotijar",
    "embotijáremos": "embotijar",
    "embotijásemos": "embotijar",
    "embotijé": "embotijar",
    "embotijéis": "embotijar",
    "embotijó": "embotijar",
    "emboto": "embotar",
    "embotábamos": "embotar",
    "embotáis": "embotar",
    "embotáramos": "embotar",
    "embotáremos": "embotar",
    "embotásemos": "embotar",
    "emboté": "embotar",
    "embotéis": "embotar",
    "embotó": "embotar",
    "emboveda": "embovedar",
    "embovedaba": "embovedar",
    "embovedabais": "embovedar",
    "embovedaban": "embovedar",
    "embovedabas": "embovedar",
    "embovedad": "embovedar",
    "embovedada": "embovedar",
    "embovedadas": "embovedar",
    "embovedado": "embovedar",
    "embovedados": "embovedar",
    "embovedamos": "embovedar",
    "embovedan": "embovedar",
    "embovedando": "embovedar",
    "embovedara": "embovedar",
    "embovedarais": "embovedar",
    "embovedaran": "embovedar",
    "embovedaras": "embovedar",
    "embovedare": "embovedar",
    "embovedareis": "embovedar",
    "embovedaremos": "embovedar",
    "embovedaren": "embovedar",
    "embovedares": "embovedar",
    "embovedaron": "embovedar",
    "embovedará": "embovedar",
    "embovedarán": "embovedar",
    "embovedarás": "embovedar",
    "embovedaré": "embovedar",
    "embovedaréis": "embovedar",
    "embovedaría": "embovedar",
    "embovedaríais": "embovedar",
    "embovedaríamos": "embovedar",
    "embovedarían": "embovedar",
    "embovedarías": "embovedar",
    "embovedas": "embovedar",
    "embovedase": "embovedar",
    "embovedaseis": "embovedar",
    "embovedasen": "embovedar",
    "embovedases": "embovedar",
    "embovedaste": "embovedar",
    "embovedasteis": "embovedar",
    "embovede": "embovedar",
    "embovedemos": "embovedar",
    "emboveden": "embovedar",
    "embovedes": "embovedar",
    "embovedo": "embovedar",
    "embovedábamos": "embovedar",
    "embovedáis": "embovedar",
    "embovedáramos": "embovedar",
    "embovedáremos": "embovedar",
    "embovedásemos": "embovedar",
    "embovedé": "embovedar",
    "embovedéis": "embovedar",
    "embovedó": "embovedar",
    "emboza": "embozar",
    "embozaba": "embozar",
    "embozabais": "embozar",
    "embozaban": "embozar",
    "embozabas": "embozar",
    "embozad": "embozar",
    "embozada": "embozar",
    "embozadas": "embozar",
    "embozado": "embozar",
    "embozados": "embozar",
    "embozala": "embozalar",
    "embozalaba": "embozalar",
    "embozalabais": "embozalar",
    "embozalaban": "embozalar",
    "embozalabas": "embozalar",
    "embozalad": "embozalar",
    "embozalada": "embozalar",
    "embozaladas": "embozalar",
    "embozalado": "embozalar",
    "embozalados": "embozalar",
    "embozalamos": "embozalar",
    "embozalan": "embozalar",
    "embozalando": "embozalar",
    "embozalara": "embozalar",
    "embozalarais": "embozalar",
    "embozalaran": "embozalar",
    "embozalaras": "embozalar",
    "embozalare": "embozalar",
    "embozalareis": "embozalar",
    "embozalaremos": "embozalar",
    "embozalaren": "embozalar",
    "embozalares": "embozalar",
    "embozalaron": "embozalar",
    "embozalará": "embozalar",
    "embozalarán": "embozalar",
    "embozalarás": "embozalar",
    "embozalaré": "embozalar",
    "embozalaréis": "embozalar",
    "embozalaría": "embozalar",
    "embozalaríais": "embozalar",
    "embozalaríamos": "embozalar",
    "embozalarían": "embozalar",
    "embozalarías": "embozalar",
    "embozalas": "embozalar",
    "embozalase": "embozalar",
    "embozalaseis": "embozalar",
    "embozalasen": "embozalar",
    "embozalases": "embozalar",
    "embozalaste": "embozalar",
    "embozalasteis": "embozalar",
    "embozale": "embozalar",
    "embozalemos": "embozalar",
    "embozalen": "embozalar",
    "embozales": "embozalar",
    "embozalo": "embozalar",
    "embozalábamos": "embozalar",
    "embozaláis": "embozalar",
    "embozaláramos": "embozalar",
    "embozaláremos": "embozalar",
    "embozalásemos": "embozalar",
    "embozalé": "embozalar",
    "embozaléis": "embozalar",
    "embozaló": "embozalar",
    "embozamos": "embozar",
    "embozan": "embozar",
    "embozando": "embozar",
    "embozara": "embozar",
    "embozarais": "embozar",
    "embozaran": "embozar",
    "embozaras": "embozar",
    "embozare": "embozar",
    "embozareis": "embozar",
    "embozaremos": "embozar",
    "embozaren": "embozar",
    "embozares": "embozar",
    "embozaron": "embozar",
    "embozará": "embozar",
    "embozarán": "embozar",
    "embozarás": "embozar",
    "embozaré": "embozar",
    "embozaréis": "embozar",
    "embozaría": "embozar",
    "embozaríais": "embozar",
    "embozaríamos": "embozar",
    "embozarían": "embozar",
    "embozarías": "embozar",
    "embozas": "embozar",
    "embozase": "embozar",
    "embozaseis": "embozar",
    "embozasen": "embozar",
    "embozases": "embozar",
    "embozaste": "embozar",
    "embozasteis": "embozar",
    "embozo": "embozar",
    "embozos": "embozo",
    "embozábamos": "embozar",
    "embozáis": "embozar",
    "embozáramos": "embozar",
    "embozáremos": "embozar",
    "embozásemos": "embozar",
    "embozó": "embozar",
    "emboñiga": "emboñigar",
    "emboñigaba": "emboñigar",
    "emboñigabais": "emboñigar",
    "emboñigaban": "emboñigar",
    "emboñigabas": "emboñigar",
    "emboñigad": "emboñigar",
    "emboñigada": "emboñigar",
    "emboñigadas": "emboñigar",
    "emboñigado": "emboñigar",
    "emboñigados": "emboñigar",
    "emboñigamos": "emboñigar",
    "emboñigan": "emboñigar",
    "emboñigando": "emboñigar",
    "emboñigara": "emboñigar",
    "emboñigarais": "emboñigar",
    "emboñigaran": "emboñigar",
    "emboñigaras": "emboñigar",
    "emboñigare": "emboñigar",
    "emboñigareis": "emboñigar",
    "emboñigaremos": "emboñigar",
    "emboñigaren": "emboñigar",
    "emboñigares": "emboñigar",
    "emboñigaron": "emboñigar",
    "emboñigará": "emboñigar",
    "emboñigarán": "emboñigar",
    "emboñigarás": "emboñigar",
    "emboñigaré": "emboñigar",
    "emboñigaréis": "emboñigar",
    "emboñigaría": "emboñigar",
    "emboñigaríais": "emboñigar",
    "emboñigaríamos": "emboñigar",
    "emboñigarían": "emboñigar",
    "emboñigarías": "emboñigar",
    "emboñigas": "emboñigar",
    "emboñigase": "emboñigar",
    "emboñigaseis": "emboñigar",
    "emboñigasen": "emboñigar",
    "emboñigases": "emboñigar",
    "emboñigaste": "emboñigar",
    "emboñigasteis": "emboñigar",
    "emboñigo": "emboñigar",
    "emboñigue": "emboñigar",
    "emboñiguemos": "emboñigar",
    "emboñiguen": "emboñigar",
    "emboñigues": "emboñigar",
    "emboñigué": "emboñigar",
    "emboñiguéis": "emboñigar",
    "emboñigábamos": "emboñigar",
    "emboñigáis": "emboñigar",
    "emboñigáramos": "emboñigar",
    "emboñigáremos": "emboñigar",
    "emboñigásemos": "emboñigar",
    "emboñigó": "emboñigar",
    "embrace": "embrazar",
    "embracemos": "embrazar",
    "embracen": "embrazar",
    "embraces": "embrazar",
    "embracilada": "embracilado",
    "embraciladas": "embracilado",
    "embracilados": "embracilado",
    "embracé": "embrazar",
    "embracéis": "embrazar",
    "embraga": "embragar",
    "embragaba": "embragar",
    "embragabais": "embragar",
    "embragaban": "embragar",
    "embragabas": "embragar",
    "embragad": "embragar",
    "embragada": "embragar",
    "embragadas": "embragar",
    "embragado": "embragar",
    "embragados": "embragar",
    "embragamos": "embragar",
    "embragan": "embragar",
    "embragando": "embragar",
    "embragara": "embragar",
    "embragarais": "embragar",
    "embragaran": "embragar",
    "embragaras": "embragar",
    "embragare": "embragar",
    "embragareis": "embragar",
    "embragaremos": "embragar",
    "embragaren": "embragar",
    "embragares": "embragar",
    "embragaron": "embragar",
    "embragará": "embragar",
    "embragarán": "embragar",
    "embragarás": "embragar",
    "embragaré": "embragar",
    "embragaréis": "embragar",
    "embragaría": "embragar",
    "embragaríais": "embragar",
    "embragaríamos": "embragar",
    "embragarían": "embragar",
    "embragarías": "embragar",
    "embragas": "embragar",
    "embragase": "embragar",
    "embragaseis": "embragar",
    "embragasen": "embragar",
    "embragases": "embragar",
    "embragaste": "embragar",
    "embragasteis": "embragar",
    "embrago": "embragar",
    "embrague": "embragar",
    "embraguemos": "embragar",
    "embraguen": "embragar",
    "embragues": "embrague",
    "embragué": "embragar",
    "embraguéis": "embragar",
    "embragábamos": "embragar",
    "embragáis": "embragar",
    "embragáramos": "embragar",
    "embragáremos": "embragar",
    "embragásemos": "embragar",
    "embragó": "embragar",
    "embravece": "embravecer",
    "embraveced": "embravecer",
    "embravecemos": "embravecer",
    "embravecen": "embravecer",
    "embraveceremos": "embravecer",
    "embravecerá": "embravecer",
    "embravecerán": "embravecer",
    "embravecerás": "embravecer",
    "embraveceré": "embravecer",
    "embraveceréis": "embravecer",
    "embravecería": "embravecer",
    "embraveceríais": "embravecer",
    "embraveceríamos": "embravecer",
    "embravecerían": "embravecer",
    "embravecerías": "embravecer",
    "embraveces": "embravecer",
    "embravecida": "embravecer",
    "embravecidas": "embravecer",
    "embravecido": "embravecer",
    "embravecidos": "embravecer",
    "embraveciendo": "embravecer",
    "embraveciera": "embravecer",
    "embravecierais": "embravecer",
    "embravecieran": "embravecer",
    "embravecieras": "embravecer",
    "embraveciere": "embravecer",
    "embraveciereis": "embravecer",
    "embravecieren": "embravecer",
    "embravecieres": "embravecer",
    "embravecieron": "embravecer",
    "embraveciese": "embravecer",
    "embravecieseis": "embravecer",
    "embraveciesen": "embravecer",
    "embravecieses": "embravecer",
    "embravecimientos": "embravecimiento",
    "embravecimos": "embravecer",
    "embraveciste": "embravecer",
    "embravecisteis": "embravecer",
    "embraveciéramos": "embravecer",
    "embraveciéremos": "embravecer",
    "embraveciésemos": "embravecer",
    "embraveció": "embravecer",
    "embravecéis": "embravecer",
    "embravecí": "embravecer",
    "embravecía": "embravecer",
    "embravecíais": "embravecer",
    "embravecíamos": "embravecer",
    "embravecían": "embravecer",
    "embravecías": "embravecer",
    "embravezca": "embravecer",
    "embravezcamos": "embravecer",
    "embravezcan": "embravecer",
    "embravezcas": "embravecer",
    "embravezco": "embravecer",
    "embravezcáis": "embravecer",
    "embraza": "embrazar",
    "embrazaba": "embrazar",
    "embrazabais": "embrazar",
    "embrazaban": "embrazar",
    "embrazabas": "embrazar",
    "embrazad": "embrazar",
    "embrazada": "embrazar",
    "embrazadas": "embrazar",
    "embrazado": "embrazar",
    "embrazados": "embrazar",
    "embrazaduras": "embrazadura",
    "embrazamos": "embrazar",
    "embrazan": "embrazar",
    "embrazando": "embrazar",
    "embrazara": "embrazar",
    "embrazarais": "embrazar",
    "embrazaran": "embrazar",
    "embrazaras": "embrazar",
    "embrazare": "embrazar",
    "embrazareis": "embrazar",
    "embrazaremos": "embrazar",
    "embrazaren": "embrazar",
    "embrazares": "embrazar",
    "embrazaron": "embrazar",
    "embrazará": "embrazar",
    "embrazarán": "embrazar",
    "embrazarás": "embrazar",
    "embrazaré": "embrazar",
    "embrazaréis": "embrazar",
    "embrazaría": "embrazar",
    "embrazaríais": "embrazar",
    "embrazaríamos": "embrazar",
    "embrazarían": "embrazar",
    "embrazarías": "embrazar",
    "embrazas": "embrazar",
    "embrazase": "embrazar",
    "embrazaseis": "embrazar",
    "embrazasen": "embrazar",
    "embrazases": "embrazar",
    "embrazaste": "embrazar",
    "embrazasteis": "embrazar",
    "embrazo": "embrazar",
    "embrazábamos": "embrazar",
    "embrazáis": "embrazar",
    "embrazáramos": "embrazar",
    "embrazáremos": "embrazar",
    "embrazásemos": "embrazar",
    "embrazó": "embrazar",
    "embrea": "embrear",
    "embreaba": "embrear",
    "embreabais": "embrear",
    "embreaban": "embrear",
    "embreabas": "embrear",
    "embread": "embrear",
    "embreada": "embrear",
    "embreadas": "embrear",
    "embreado": "embrear",
    "embreados": "embrear",
    "embreaduras": "embreadura",
    "embreamos": "embrear",
    "embrean": "embrear",
    "embreando": "embrear",
    "embreara": "embrear",
    "embrearais": "embrear",
    "embrearan": "embrear",
    "embrearas": "embrear",
    "embreare": "embrear",
    "embreareis": "embrear",
    "embrearemos": "embrear",
    "embrearen": "embrear",
    "embreares": "embrear",
    "embrearon": "embrear",
    "embreará": "embrear",
    "embrearán": "embrear",
    "embrearás": "embrear",
    "embrearé": "embrear",
    "embrearéis": "embrear",
    "embrearía": "embrear",
    "embrearíais": "embrear",
    "embrearíamos": "embrear",
    "embrearían": "embrear",
    "embrearías": "embrear",
    "embreas": "embrear",
    "embrease": "embrear",
    "embreaseis": "embrear",
    "embreasen": "embrear",
    "embreases": "embrear",
    "embreaste": "embrear",
    "embreasteis": "embrear",
    "embree": "embrear",
    "embreemos": "embrear",
    "embreen": "embrear",
    "embrees": "embrear",
    "embreo": "embrear",
    "embreábamos": "embrear",
    "embreáis": "embrear",
    "embreáramos": "embrear",
    "embreáremos": "embrear",
    "embreásemos": "embrear",
    "embreé": "embrear",
    "embreéis": "embrear",
    "embreó": "embrear",
    "embriaga": "embriagar",
    "embriagaba": "embriagar",
    "embriagabais": "embriagar",
    "embriagaban": "embriagar",
    "embriagabas": "embriagar",
    "embriagad": "embriagar",
    "embriagada": "embriagar",
    "embriagadas": "embriagar",
    "embriagado": "embriagar",
    "embriagadora": "embriagador",
    "embriagadoras": "embriagador",
    "embriagadores": "embriagador",
    "embriagados": "embriagar",
    "embriagamientos": "embriagamiento",
    "embriagamos": "embriagar",
    "embriagan": "embriagar",
    "embriagando": "embriagar",
    "embriagantes": "embriagante",
    "embriagara": "embriagar",
    "embriagarais": "embriagar",
    "embriagaran": "embriagar",
    "embriagaras": "embriagar",
    "embriagare": "embriagar",
    "embriagareis": "embriagar",
    "embriagaremos": "embriagar",
    "embriagaren": "embriagar",
    "embriagares": "embriagar",
    "embriagaron": "embriagar",
    "embriagará": "embriagar",
    "embriagarán": "embriagar",
    "embriagarás": "embriagar",
    "embriagaré": "embriagar",
    "embriagaréis": "embriagar",
    "embriagaría": "embriagar",
    "embriagaríais": "embriagar",
    "embriagaríamos": "embriagar",
    "embriagarían": "embriagar",
    "embriagarías": "embriagar",
    "embriagas": "embriagar",
    "embriagase": "embriagar",
    "embriagaseis": "embriagar",
    "embriagasen": "embriagar",
    "embriagases": "embriagar",
    "embriagaste": "embriagar",
    "embriagasteis": "embriagar",
    "embriago": "embriagar",
    "embriague": "embriagar",
    "embriagueces": "embriaguez",
    "embriaguemos": "embriagar",
    "embriaguen": "embriagar",
    "embriagues": "embriagar",
    "embriagué": "embriagar",
    "embriaguéis": "embriagar",
    "embriagábamos": "embriagar",
    "embriagáis": "embriagar",
    "embriagáramos": "embriagar",
    "embriagáremos": "embriagar",
    "embriagásemos": "embriagar",
    "embriagó": "embriagar",
    "embrida": "embridar",
    "embridaba": "embridar",
    "embridabais": "embridar",
    "embridaban": "embridar",
    "embridabas": "embridar",
    "embridad": "embridar",
    "embridada": "embridar",
    "embridadas": "embridar",
    "embridado": "embridar",
    "embridados": "embridar",
    "embridamos": "embridar",
    "embridan": "embridar",
    "embridando": "embridar",
    "embridara": "embridar",
    "embridarais": "embridar",
    "embridaran": "embridar",
    "embridaras": "embridar",
    "embridare": "embridar",
    "embridareis": "embridar",
    "embridaremos": "embridar",
    "embridaren": "embridar",
    "embridares": "embridar",
    "embridaron": "embridar",
    "embridará": "embridar",
    "embridarán": "embridar",
    "embridarás": "embridar",
    "embridaré": "embridar",
    "embridaréis": "embridar",
    "embridaría": "embridar",
    "embridaríais": "embridar",
    "embridaríamos": "embridar",
    "embridarían": "embridar",
    "embridarías": "embridar",
    "embridas": "embridar",
    "embridase": "embridar",
    "embridaseis": "embridar",
    "embridasen": "embridar",
    "embridases": "embridar",
    "embridaste": "embridar",
    "embridasteis": "embridar",
    "embride": "embridar",
    "embridemos": "embridar",
    "embriden": "embridar",
    "embrides": "embridar",
    "embrido": "embridar",
    "embridábamos": "embridar",
    "embridáis": "embridar",
    "embridáramos": "embridar",
    "embridáremos": "embridar",
    "embridásemos": "embridar",
    "embridé": "embridar",
    "embridéis": "embridar",
    "embridó": "embridar",
    "embriogenias": "embriogenia",
    "embriogenésica": "embriogenésico",
    "embriogenésicas": "embriogenésico",
    "embriogenésicos": "embriogenésico",
    "embriologías": "embriología",
    "embriológica": "embriológico",
    "embriológicas": "embriológico",
    "embriológicos": "embriológico",
    "embrionales": "embrional",
    "embrionaria": "embrionario",
    "embrionarias": "embrionario",
    "embrionarios": "embrionario",
    "embriones": "embrión",
    "embrióloga": "embriólogo",
    "embriólogas": "embriólogo",
    "embriólogos": "embriólogo",
    "embroca": "embrocar",
    "embrocaba": "embrocar",
    "embrocabais": "embrocar",
    "embrocaban": "embrocar",
    "embrocabas": "embrocar",
    "embrocaciones": "embrocación",
    "embrocad": "embrocar",
    "embrocada": "embrocar",
    "embrocadas": "embrocar",
    "embrocado": "embrocar",
    "embrocados": "embrocar",
    "embrocamos": "embrocar",
    "embrocan": "embrocar",
    "embrocando": "embrocar",
    "embrocara": "embrocar",
    "embrocarais": "embrocar",
    "embrocaran": "embrocar",
    "embrocaras": "embrocar",
    "embrocare": "embrocar",
    "embrocareis": "embrocar",
    "embrocaremos": "embrocar",
    "embrocaren": "embrocar",
    "embrocares": "embrocar",
    "embrocaron": "embrocar",
    "embrocará": "embrocar",
    "embrocarán": "embrocar",
    "embrocarás": "embrocar",
    "embrocaré": "embrocar",
    "embrocaréis": "embrocar",
    "embrocaría": "embrocar",
    "embrocaríais": "embrocar",
    "embrocaríamos": "embrocar",
    "embrocarían": "embrocar",
    "embrocarías": "embrocar",
    "embrocas": "embrocar",
    "embrocase": "embrocar",
    "embrocaseis": "embrocar",
    "embrocasen": "embrocar",
    "embrocases": "embrocar",
    "embrocaste": "embrocar",
    "embrocasteis": "embrocar",
    "embrochala": "embrochalar",
    "embrochalaba": "embrochalar",
    "embrochalabais": "embrochalar",
    "embrochalaban": "embrochalar",
    "embrochalabas": "embrochalar",
    "embrochalad": "embrochalar",
    "embrochalada": "embrochalar",
    "embrochaladas": "embrochalar",
    "embrochalado": "embrochalar",
    "embrochalados": "embrochalar",
    "embrochalamos": "embrochalar",
    "embrochalan": "embrochalar",
    "embrochalando": "embrochalar",
    "embrochalara": "embrochalar",
    "embrochalarais": "embrochalar",
    "embrochalaran": "embrochalar",
    "embrochalaras": "embrochalar",
    "embrochalare": "embrochalar",
    "embrochalareis": "embrochalar",
    "embrochalaremos": "embrochalar",
    "embrochalaren": "embrochalar",
    "embrochalares": "embrochalar",
    "embrochalaron": "embrochalar",
    "embrochalará": "embrochalar",
    "embrochalarán": "embrochalar",
    "embrochalarás": "embrochalar",
    "embrochalaré": "embrochalar",
    "embrochalaréis": "embrochalar",
    "embrochalaría": "embrochalar",
    "embrochalaríais": "embrochalar",
    "embrochalaríamos": "embrochalar",
    "embrochalarían": "embrochalar",
    "embrochalarías": "embrochalar",
    "embrochalas": "embrochalar",
    "embrochalase": "embrochalar",
    "embrochalaseis": "embrochalar",
    "embrochalasen": "embrochalar",
    "embrochalases": "embrochalar",
    "embrochalaste": "embrochalar",
    "embrochalasteis": "embrochalar",
    "embrochale": "embrochalar",
    "embrochalemos": "embrochalar",
    "embrochalen": "embrochalar",
    "embrochales": "embrochalar",
    "embrochalo": "embrochalar",
    "embrochalábamos": "embrochalar",
    "embrochaláis": "embrochalar",
    "embrochaláramos": "embrochalar",
    "embrochaláremos": "embrochalar",
    "embrochalásemos": "embrochalar",
    "embrochalé": "embrochalar",
    "embrochaléis": "embrochalar",
    "embrochaló": "embrochalar",
    "embroco": "embrocar",
    "embrocábamos": "embrocar",
    "embrocáis": "embrocar",
    "embrocáramos": "embrocar",
    "embrocáremos": "embrocar",
    "embrocásemos": "embrocar",
    "embrocó": "embrocar",
    "embrolla": "embrollar",
    "embrollaba": "embrollar",
    "embrollabais": "embrollar",
    "embrollaban": "embrollar",
    "embrollabas": "embrollar",
    "embrollad": "embrollar",
    "embrollada": "embrollar",
    "embrolladas": "embrollar",
    "embrollado": "embrollar",
    "embrolladora": "embrollador",
    "embrolladoras": "embrollador",
    "embrolladores": "embrollador",
    "embrollados": "embrollar",
    "embrollamos": "embrollar",
    "embrollan": "embrollar",
    "embrollando": "embrollar",
    "embrollara": "embrollar",
    "embrollarais": "embrollar",
    "embrollaran": "embrollar",
    "embrollaras": "embrollar",
    "embrollare": "embrollar",
    "embrollareis": "embrollar",
    "embrollaremos": "embrollar",
    "embrollaren": "embrollar",
    "embrollares": "embrollar",
    "embrollaron": "embrollar",
    "embrollará": "embrollar",
    "embrollarán": "embrollar",
    "embrollarás": "embrollar",
    "embrollaré": "embrollar",
    "embrollaréis": "embrollar",
    "embrollaría": "embrollar",
    "embrollaríais": "embrollar",
    "embrollaríamos": "embrollar",
    "embrollarían": "embrollar",
    "embrollarías": "embrollar",
    "embrollas": "embrollar",
    "embrollase": "embrollar",
    "embrollaseis": "embrollar",
    "embrollasen": "embrollar",
    "embrollases": "embrollar",
    "embrollaste": "embrollar",
    "embrollasteis": "embrollar",
    "embrolle": "embrollar",
    "embrollemos": "embrollar",
    "embrollen": "embrollar",
    "embrolles": "embrollar",
    "embrollo": "embrollar",
    "embrollona": "embrollón",
    "embrollonas": "embrollón",
    "embrollones": "embrollón",
    "embrollos": "embrollo",
    "embrollosa": "embrolloso",
    "embrollosas": "embrolloso",
    "embrollosos": "embrolloso",
    "embrollábamos": "embrollar",
    "embrolláis": "embrollar",
    "embrolláramos": "embrollar",
    "embrolláremos": "embrollar",
    "embrollásemos": "embrollar",
    "embrollé": "embrollar",
    "embrolléis": "embrollar",
    "embrolló": "embrollar",
    "embroma": "embromar",
    "embromaba": "embromar",
    "embromabais": "embromar",
    "embromaban": "embromar",
    "embromabas": "embromar",
    "embromad": "embromar",
    "embromada": "embromar",
    "embromadas": "embromar",
    "embromado": "embromar",
    "embromadora": "embromador",
    "embromadoras": "embromador",
    "embromadores": "embromador",
    "embromados": "embromar",
    "embromamos": "embromar",
    "embroman": "embromar",
    "embromando": "embromar",
    "embromara": "embromar",
    "embromarais": "embromar",
    "embromaran": "embromar",
    "embromaras": "embromar",
    "embromare": "embromar",
    "embromareis": "embromar",
    "embromaremos": "embromar",
    "embromaren": "embromar",
    "embromares": "embromar",
    "embromaron": "embromar",
    "embromará": "embromar",
    "embromarán": "embromar",
    "embromarás": "embromar",
    "embromaré": "embromar",
    "embromaréis": "embromar",
    "embromaría": "embromar",
    "embromaríais": "embromar",
    "embromaríamos": "embromar",
    "embromarían": "embromar",
    "embromarías": "embromar",
    "embromas": "embromar",
    "embromase": "embromar",
    "embromaseis": "embromar",
    "embromasen": "embromar",
    "embromases": "embromar",
    "embromaste": "embromar",
    "embromasteis": "embromar",
    "embrome": "embromar",
    "embromemos": "embromar",
    "embromen": "embromar",
    "embromes": "embromar",
    "embromo": "embromar",
    "embromábamos": "embromar",
    "embromáis": "embromar",
    "embromáramos": "embromar",
    "embromáremos": "embromar",
    "embromásemos": "embromar",
    "embromé": "embromar",
    "embroméis": "embromar",
    "embromó": "embromar",
    "embroque": "embrocar",
    "embroquela": "embroquelar",
    "embroquelaba": "embroquelar",
    "embroquelabais": "embroquelar",
    "embroquelaban": "embroquelar",
    "embroquelabas": "embroquelar",
    "embroquelad": "embroquelar",
    "embroquelada": "embroquelar",
    "embroqueladas": "embroquelar",
    "embroquelado": "embroquelar",
    "embroquelados": "embroquelar",
    "embroquelamos": "embroquelar",
    "embroquelan": "embroquelar",
    "embroquelando": "embroquelar",
    "embroquelara": "embroquelar",
    "embroquelarais": "embroquelar",
    "embroquelaran": "embroquelar",
    "embroquelaras": "embroquelar",
    "embroquelare": "embroquelar",
    "embroquelareis": "embroquelar",
    "embroquelaremos": "embroquelar",
    "embroquelaren": "embroquelar",
    "embroquelares": "embroquelar",
    "embroquelaron": "embroquelar",
    "embroquelará": "embroquelar",
    "embroquelarán": "embroquelar",
    "embroquelarás": "embroquelar",
    "embroquelaré": "embroquelar",
    "embroquelaréis": "embroquelar",
    "embroquelaría": "embroquelar",
    "embroquelaríais": "embroquelar",
    "embroquelaríamos": "embroquelar",
    "embroquelarían": "embroquelar",
    "embroquelarías": "embroquelar",
    "embroquelas": "embroquelar",
    "embroquelase": "embroquelar",
    "embroquelaseis": "embroquelar",
    "embroquelasen": "embroquelar",
    "embroquelases": "embroquelar",
    "embroquelaste": "embroquelar",
    "embroquelasteis": "embroquelar",
    "embroquele": "embroquelar",
    "embroquelemos": "embroquelar",
    "embroquelen": "embroquelar",
    "embroqueles": "embroquelar",
    "embroquelo": "embroquelar",
    "embroquelábamos": "embroquelar",
    "embroqueláis": "embroquelar",
    "embroqueláramos": "embroquelar",
    "embroqueláremos": "embroquelar",
    "embroquelásemos": "embroquelar",
    "embroquelé": "embroquelar",
    "embroqueléis": "embroquelar",
    "embroqueló": "embroquelar",
    "embroquemos": "embrocar",
    "embroquen": "embrocar",
    "embroques": "embrocar",
    "embroqueta": "embroquetar",
    "embroquetaba": "embroquetar",
    "embroquetabais": "embroquetar",
    "embroquetaban": "embroquetar",
    "embroquetabas": "embroquetar",
    "embroquetad": "embroquetar",
    "embroquetada": "embroquetar",
    "embroquetadas": "embroquetar",
    "embroquetado": "embroquetar",
    "embroquetados": "embroquetar",
    "embroquetamos": "embroquetar",
    "embroquetan": "embroquetar",
    "embroquetando": "embroquetar",
    "embroquetara": "embroquetar",
    "embroquetarais": "embroquetar",
    "embroquetaran": "embroquetar",
    "embroquetaras": "embroquetar",
    "embroquetare": "embroquetar",
    "embroquetareis": "embroquetar",
    "embroquetaremos": "embroquetar",
    "embroquetaren": "embroquetar",
    "embroquetares": "embroquetar",
    "embroquetaron": "embroquetar",
    "embroquetará": "embroquetar",
    "embroquetarán": "embroquetar",
    "embroquetarás": "embroquetar",
    "embroquetaré": "embroquetar",
    "embroquetaréis": "embroquetar",
    "embroquetaría": "embroquetar",
    "embroquetaríais": "embroquetar",
    "embroquetaríamos": "embroquetar",
    "embroquetarían": "embroquetar",
    "embroquetarías": "embroquetar",
    "embroquetas": "embroquetar",
    "embroquetase": "embroquetar",
    "embroquetaseis": "embroquetar",
    "embroquetasen": "embroquetar",
    "embroquetases": "embroquetar",
    "embroquetaste": "embroquetar",
    "embroquetasteis": "embroquetar",
    "embroquete": "embroquetar",
    "embroquetemos": "embroquetar",
    "embroqueten": "embroquetar",
    "embroquetes": "embroquetar",
    "embroqueto": "embroquetar",
    "embroquetábamos": "embroquetar",
    "embroquetáis": "embroquetar",
    "embroquetáramos": "embroquetar",
    "embroquetáremos": "embroquetar",
    "embroquetásemos": "embroquetar",
    "embroqueté": "embroquetar",
    "embroquetéis": "embroquetar",
    "embroquetó": "embroquetar",
    "embroqué": "embrocar",
    "embroquéis": "embrocar",
    "embruja": "embrujar",
    "embrujaba": "embrujar",
    "embrujabais": "embrujar",
    "embrujaban": "embrujar",
    "embrujabas": "embrujar",
    "embrujad": "embrujar",
    "embrujada": "embrujar",
    "embrujadas": "embrujar",
    "embrujado": "embrujar",
    "embrujadora": "embrujador",
    "embrujadoras": "embrujador",
    "embrujadores": "embrujador",
    "embrujados": "embrujar",
    "embrujamientos": "embrujamiento",
    "embrujamos": "embrujar",
    "embrujan": "embrujar",
    "embrujando": "embrujar",
    "embrujara": "embrujar",
    "embrujarais": "embrujar",
    "embrujaran": "embrujar",
    "embrujaras": "embrujar",
    "embrujare": "embrujar",
    "embrujareis": "embrujar",
    "embrujaremos": "embrujar",
    "embrujaren": "embrujar",
    "embrujares": "embrujar",
    "embrujaron": "embrujar",
    "embrujará": "embrujar",
    "embrujarán": "embrujar",
    "embrujarás": "embrujar",
    "embrujaré": "embrujar",
    "embrujaréis": "embrujar",
    "embrujaría": "embrujar",
    "embrujaríais": "embrujar",
    "embrujaríamos": "embrujar",
    "embrujarían": "embrujar",
    "embrujarías": "embrujar",
    "embrujas": "embrujar",
    "embrujase": "embrujar",
    "embrujaseis": "embrujar",
    "embrujasen": "embrujar",
    "embrujases": "embrujar",
    "embrujaste": "embrujar",
    "embrujasteis": "embrujar",
    "embruje": "embrujar",
    "embrujemos": "embrujar",
    "embrujen": "embrujar",
    "embrujes": "embrujar",
    "embrujo": "embrujar",
    "embrujos": "embrujo",
    "embrujábamos": "embrujar",
    "embrujáis": "embrujar",
    "embrujáramos": "embrujar",
    "embrujáremos": "embrujar",
    "embrujásemos": "embrujar",
    "embrujé": "embrujar",
    "embrujéis": "embrujar",
    "embrujó": "embrujar",
    "embrutece": "embrutecer",
    "embruteced": "embrutecer",
    "embrutecedora": "embrutecedor",
    "embrutecedoras": "embrutecedor",
    "embrutecedores": "embrutecedor",
    "embrutecemos": "embrutecer",
    "embrutecen": "embrutecer",
    "embruteceremos": "embrutecer",
    "embrutecerá": "embrutecer",
    "embrutecerán": "embrutecer",
    "embrutecerás": "embrutecer",
    "embruteceré": "embrutecer",
    "embruteceréis": "embrutecer",
    "embrutecería": "embrutecer",
    "embruteceríais": "embrutecer",
    "embruteceríamos": "embrutecer",
    "embrutecerían": "embrutecer",
    "embrutecerías": "embrutecer",
    "embruteces": "embrutecer",
    "embrutecida": "embrutecer",
    "embrutecidas": "embrutecer",
    "embrutecido": "embrutecer",
    "embrutecidos": "embrutecer",
    "embruteciendo": "embrutecer",
    "embruteciera": "embrutecer",
    "embrutecierais": "embrutecer",
    "embrutecieran": "embrutecer",
    "embrutecieras": "embrutecer",
    "embruteciere": "embrutecer",
    "embruteciereis": "embrutecer",
    "embrutecieren": "embrutecer",
    "embrutecieres": "embrutecer",
    "embrutecieron": "embrutecer",
    "embruteciese": "embrutecer",
    "embrutecieseis": "embrutecer",
    "embruteciesen": "embrutecer",
    "embrutecieses": "embrutecer",
    "embrutecimientos": "embrutecimiento",
    "embrutecimos": "embrutecer",
    "embruteciste": "embrutecer",
    "embrutecisteis": "embrutecer",
    "embruteciéramos": "embrutecer",
    "embruteciéremos": "embrutecer",
    "embruteciésemos": "embrutecer",
    "embruteció": "embrutecer",
    "embrutecéis": "embrutecer",
    "embrutecí": "embrutecer",
    "embrutecía": "embrutecer",
    "embrutecíais": "embrutecer",
    "embrutecíamos": "embrutecer",
    "embrutecían": "embrutecer",
    "embrutecías": "embrutecer",
    "embrutezca": "embrutecer",
    "embrutezcamos": "embrutecer",
    "embrutezcan": "embrutecer",
    "embrutezcas": "embrutecer",
    "embrutezco": "embrutecer",
    "embrutezcáis": "embrutecer",
    "embucha": "embuchar",
    "embuchaba": "embuchar",
    "embuchabais": "embuchar",
    "embuchaban": "embuchar",
    "embuchabas": "embuchar",
    "embuchad": "embuchar",
    "embuchada": "embuchar",
    "embuchadas": "embuchar",
    "embuchado": "embuchar",
    "embuchados": "embuchar",
    "embuchamos": "embuchar",
    "embuchan": "embuchar",
    "embuchando": "embuchar",
    "embuchara": "embuchar",
    "embucharais": "embuchar",
    "embucharan": "embuchar",
    "embucharas": "embuchar",
    "embuchare": "embuchar",
    "embuchareis": "embuchar",
    "embucharemos": "embuchar",
    "embucharen": "embuchar",
    "embuchares": "embuchar",
    "embucharon": "embuchar",
    "embuchará": "embuchar",
    "embucharán": "embuchar",
    "embucharás": "embuchar",
    "embucharé": "embuchar",
    "embucharéis": "embuchar",
    "embucharía": "embuchar",
    "embucharíais": "embuchar",
    "embucharíamos": "embuchar",
    "embucharían": "embuchar",
    "embucharías": "embuchar",
    "embuchas": "embuchar",
    "embuchase": "embuchar",
    "embuchaseis": "embuchar",
    "embuchasen": "embuchar",
    "embuchases": "embuchar",
    "embuchaste": "embuchar",
    "embuchasteis": "embuchar",
    "embuche": "embuchar",
    "embuchemos": "embuchar",
    "embuchen": "embuchar",
    "embuches": "embuchar",
    "embucho": "embuchar",
    "embuchábamos": "embuchar",
    "embucháis": "embuchar",
    "embucháramos": "embuchar",
    "embucháremos": "embuchar",
    "embuchásemos": "embuchar",
    "embuché": "embuchar",
    "embuchéis": "embuchar",
    "embuchó": "embuchar",
    "embuda": "embudar",
    "embudaba": "embudar",
    "embudabais": "embudar",
    "embudaban": "embudar",
    "embudabas": "embudar",
    "embudad": "embudar",
    "embudada": "embudar",
    "embudadas": "embudar",
    "embudado": "embudar",
    "embudados": "embudar",
    "embudamos": "embudar",
    "embudan": "embudar",
    "embudando": "embudar",
    "embudara": "embudar",
    "embudarais": "embudar",
    "embudaran": "embudar",
    "embudaras": "embudar",
    "embudare": "embudar",
    "embudareis": "embudar",
    "embudaremos": "embudar",
    "embudaren": "embudar",
    "embudares": "embudar",
    "embudaron": "embudar",
    "embudará": "embudar",
    "embudarán": "embudar",
    "embudarás": "embudar",
    "embudaré": "embudar",
    "embudaréis": "embudar",
    "embudaría": "embudar",
    "embudaríais": "embudar",
    "embudaríamos": "embudar",
    "embudarían": "embudar",
    "embudarías": "embudar",
    "embudas": "embudar",
    "embudase": "embudar",
    "embudaseis": "embudar",
    "embudasen": "embudar",
    "embudases": "embudar",
    "embudaste": "embudar",
    "embudasteis": "embudar",
    "embude": "embudar",
    "embudemos": "embudar",
    "embuden": "embudar",
    "embudes": "embudar",
    "embudo": "embudar",
    "embudos": "embudo",
    "embudábamos": "embudar",
    "embudáis": "embudar",
    "embudáramos": "embudar",
    "embudáremos": "embudar",
    "embudásemos": "embudar",
    "embudé": "embudar",
    "embudéis": "embudar",
    "embudó": "embudar",
    "embulla": "embullar",
    "embullaba": "embullar",
    "embullabais": "embullar",
    "embullaban": "embullar",
    "embullabas": "embullar",
    "embullad": "embullar",
    "embullada": "embullar",
    "embulladas": "embullar",
    "embullado": "embullar",
    "embullados": "embullar",
    "embullamientos": "embullamiento",
    "embullamos": "embullar",
    "embullan": "embullar",
    "embullando": "embullar",
    "embullara": "embullar",
    "embullarais": "embullar",
    "embullaran": "embullar",
    "embullaras": "embullar",
    "embullare": "embullar",
    "embullareis": "embullar",
    "embullaremos": "embullar",
    "embullaren": "embullar",
    "embullares": "embullar",
    "embullaron": "embullar",
    "embullará": "embullar",
    "embullarán": "embullar",
    "embullarás": "embullar",
    "embullaré": "embullar",
    "embullaréis": "embullar",
    "embullaría": "embullar",
    "embullaríais": "embullar",
    "embullaríamos": "embullar",
    "embullarían": "embullar",
    "embullarías": "embullar",
    "embullas": "embullar",
    "embullase": "embullar",
    "embullaseis": "embullar",
    "embullasen": "embullar",
    "embullases": "embullar",
    "embullaste": "embullar",
    "embullasteis": "embullar",
    "embulle": "embullar",
    "embullemos": "embullar",
    "embullen": "embullar",
    "embulles": "embullar",
    "embullo": "embullar",
    "embullábamos": "embullar",
    "embulláis": "embullar",
    "embulláramos": "embullar",
    "embulláremos": "embullar",
    "embullásemos": "embullar",
    "embullé": "embullar",
    "embulléis": "embullar",
    "embulló": "embullar",
    "emburuja": "emburujar",
    "emburujaba": "emburujar",
    "emburujabais": "emburujar",
    "emburujaban": "emburujar",
    "emburujabas": "emburujar",
    "emburujad": "emburujar",
    "emburujada": "emburujar",
    "emburujadas": "emburujar",
    "emburujado": "emburujar",
    "emburujados": "emburujar",
    "emburujamos": "emburujar",
    "emburujan": "emburujar",
    "emburujando": "emburujar",
    "emburujara": "emburujar",
    "emburujarais": "emburujar",
    "emburujaran": "emburujar",
    "emburujaras": "emburujar",
    "emburujare": "emburujar",
    "emburujareis": "emburujar",
    "emburujaremos": "emburujar",
    "emburujaren": "emburujar",
    "emburujares": "emburujar",
    "emburujaron": "emburujar",
    "emburujará": "emburujar",
    "emburujarán": "emburujar",
    "emburujarás": "emburujar",
    "emburujaré": "emburujar",
    "emburujaréis": "emburujar",
    "emburujaría": "emburujar",
    "emburujaríais": "emburujar",
    "emburujaríamos": "emburujar",
    "emburujarían": "emburujar",
    "emburujarías": "emburujar",
    "emburujas": "emburujar",
    "emburujase": "emburujar",
    "emburujaseis": "emburujar",
    "emburujasen": "emburujar",
    "emburujases": "emburujar",
    "emburujaste": "emburujar",
    "emburujasteis": "emburujar",
    "emburuje": "emburujar",
    "emburujemos": "emburujar",
    "emburujen": "emburujar",
    "emburujes": "emburujar",
    "emburujo": "emburujar",
    "emburujábamos": "emburujar",
    "emburujáis": "emburujar",
    "emburujáramos": "emburujar",
    "emburujáremos": "emburujar",
    "emburujásemos": "emburujar",
    "emburujé": "emburujar",
    "emburujéis": "emburujar",
    "emburujó": "emburujar",
    "embustea": "embustear",
    "embusteaba": "embustear",
    "embusteabais": "embustear",
    "embusteaban": "embustear",
    "embusteabas": "embustear",
    "embustead": "embustear",
    "embusteada": "embustear",
    "embusteadas": "embustear",
    "embusteado": "embustear",
    "embusteados": "embustear",
    "embusteamos": "embustear",
    "embustean": "embustear",
    "embusteando": "embustear",
    "embusteara": "embustear",
    "embustearais": "embustear",
    "embustearan": "embustear",
    "embustearas": "embustear",
    "embusteare": "embustear",
    "embusteareis": "embustear",
    "embustearemos": "embustear",
    "embustearen": "embustear",
    "embusteares": "embustear",
    "embustearon": "embustear",
    "embusteará": "embustear",
    "embustearán": "embustear",
    "embustearás": "embustear",
    "embustearé": "embustear",
    "embustearéis": "embustear",
    "embustearía": "embustear",
    "embustearíais": "embustear",
    "embustearíamos": "embustear",
    "embustearían": "embustear",
    "embustearías": "embustear",
    "embusteas": "embustear",
    "embustease": "embustear",
    "embusteaseis": "embustear",
    "embusteasen": "embustear",
    "embusteases": "embustear",
    "embusteaste": "embustear",
    "embusteasteis": "embustear",
    "embustee": "embustear",
    "embusteemos": "embustear",
    "embusteen": "embustear",
    "embustees": "embustear",
    "embusteo": "embustear",
    "embustera": "embustero",
    "embusteras": "embustero",
    "embusteros": "embustero",
    "embusterías": "embustería",
    "embustes": "embuste",
    "embusteábamos": "embustear",
    "embusteáis": "embustear",
    "embusteáramos": "embustear",
    "embusteáremos": "embustear",
    "embusteásemos": "embustear",
    "embusteé": "embustear",
    "embusteéis": "embustear",
    "embusteó": "embustear",
    "embuta": "embutir",
    "embutamos": "embutir",
    "embutan": "embutir",
    "embutas": "embutir",
    "embute": "embutir",
    "embuten": "embutir",
    "embutes": "embutir",
    "embuticiones": "embutición",
    "embutid": "embutir",
    "embutida": "embutir",
    "embutidas": "embutir",
    "embutideras": "embutidera",
    "embutido": "embutir",
    "embutidora": "embutidor",
    "embutidoras": "embutidor",
    "embutidores": "embutidor",
    "embutidos": "embutir",
    "embutiendo": "embutir",
    "embutiera": "embutir",
    "embutierais": "embutir",
    "embutieran": "embutir",
    "embutieras": "embutir",
    "embutiere": "embutir",
    "embutiereis": "embutir",
    "embutieren": "embutir",
    "embutieres": "embutir",
    "embutieron": "embutir",
    "embutiese": "embutir",
    "embutieseis": "embutir",
    "embutiesen": "embutir",
    "embutieses": "embutir",
    "embutimos": "embutir",
    "embutiremos": "embutir",
    "embutirá": "embutir",
    "embutirán": "embutir",
    "embutirás": "embutir",
    "embutiré": "embutir",
    "embutiréis": "embutir",
    "embutiría": "embutir",
    "embutiríais": "embutir",
    "embutiríamos": "embutir",
    "embutirían": "embutir",
    "embutirías": "embutir",
    "embutiste": "embutir",
    "embutisteis": "embutir",
    "embutiéramos": "embutir",
    "embutiéremos": "embutir",
    "embutiésemos": "embutir",
    "embutió": "embutir",
    "embuto": "embutir",
    "embutáis": "embutir",
    "embutí": "embutir",
    "embutía": "embutir",
    "embutíais": "embutir",
    "embutíamos": "embutir",
    "embutían": "embutir",
    "embutías": "embutir",
    "embutís": "embutir",
    "emelgas": "emelga",
    "emenagogos": "emenagogo",
    "emerge": "emerger",
    "emerged": "emerger",
    "emergemos": "emerger",
    "emergen": "emerger",
    "emergencias": "emergencia",
    "emergentes": "emergente",
    "emergeremos": "emerger",
    "emergerá": "emerger",
    "emergerán": "emerger",
    "emergerás": "emerger",
    "emergeré": "emerger",
    "emergeréis": "emerger",
    "emergería": "emerger",
    "emergeríais": "emerger",
    "emergeríamos": "emerger",
    "emergerían": "emerger",
    "emergerías": "emerger",
    "emerges": "emerger",
    "emergida": "emerger",
    "emergidas": "emerger",
    "emergido": "emerger",
    "emergidos": "emerger",
    "emergiendo": "emerger",
    "emergiera": "emerger",
    "emergierais": "emerger",
    "emergieran": "emerger",
    "emergieras": "emerger",
    "emergiere": "emerger",
    "emergiereis": "emerger",
    "emergieren": "emerger",
    "emergieres": "emerger",
    "emergieron": "emerger",
    "emergiese": "emerger",
    "emergieseis": "emerger",
    "emergiesen": "emerger",
    "emergieses": "emerger",
    "emergimos": "emerger",
    "emergiste": "emerger",
    "emergisteis": "emerger",
    "emergiéramos": "emerger",
    "emergiéremos": "emerger",
    "emergiésemos": "emerger",
    "emergió": "emerger",
    "emergéis": "emerger",
    "emergí": "emerger",
    "emergía": "emerger",
    "emergíais": "emerger",
    "emergíamos": "emerger",
    "emergían": "emerger",
    "emergías": "emerger",
    "emeritenses": "emeritense",
    "emerja": "emerger",
    "emerjamos": "emerger",
    "emerjan": "emerger",
    "emerjas": "emerger",
    "emerjo": "emerger",
    "emerjáis": "emerger",
    "emersiones": "emersión",
    "emes": "eme",
    "emetropías": "emetropía",
    "emetrópica": "emetrópico",
    "emetrópicas": "emetrópico",
    "emetrópicos": "emetrópico",
    "emidosaurios": "emidosaurio",
    "emigra": "emigrar",
    "emigraba": "emigrar",
    "emigrabais": "emigrar",
    "emigraban": "emigrar",
    "emigrabas": "emigrar",
    "emigraciones": "emigración",
    "emigrad": "emigrar",
    "emigrada": "emigrar",
    "emigradas": "emigrar",
    "emigrado": "emigrar",
    "emigrados": "emigrar",
    "emigramos": "emigrar",
    "emigran": "emigrar",
    "emigrando": "emigrar",
    "emigrantes": "emigrante",
    "emigrara": "emigrar",
    "emigrarais": "emigrar",
    "emigraran": "emigrar",
    "emigraras": "emigrar",
    "emigrare": "emigrar",
    "emigrareis": "emigrar",
    "emigraremos": "emigrar",
    "emigraren": "emigrar",
    "emigrares": "emigrar",
    "emigraron": "emigrar",
    "emigrará": "emigrar",
    "emigrarán": "emigrar",
    "emigrarás": "emigrar",
    "emigraré": "emigrar",
    "emigraréis": "emigrar",
    "emigraría": "emigrar",
    "emigraríais": "emigrar",
    "emigraríamos": "emigrar",
    "emigrarían": "emigrar",
    "emigrarías": "emigrar",
    "emigras": "emigrar",
    "emigrase": "emigrar",
    "emigraseis": "emigrar",
    "emigrasen": "emigrar",
    "emigrases": "emigrar",
    "emigraste": "emigrar",
    "emigrasteis": "emigrar",
    "emigratoria": "emigratorio",
    "emigratorias": "emigratorio",
    "emigratorios": "emigratorio",
    "emigre": "emigrar",
    "emigremos": "emigrar",
    "emigren": "emigrar",
    "emigres": "emigrar",
    "emigro": "emigrar",
    "emigrábamos": "emigrar",
    "emigráis": "emigrar",
    "emigráramos": "emigrar",
    "emigráremos": "emigrar",
    "emigrásemos": "emigrar",
    "emigré": "emigrar",
    "emigréis": "emigrar",
    "emigró": "emigrar",
    "eminencias": "eminencia",
    "eminentes": "eminente",
    "emiratos": "emirato",
    "emires": "emir",
    "emisaria": "emisario",
    "emisarias": "emisario",
    "emisarios": "emisario",
    "emisiones": "emisión",
    "emisiva": "emisivo",
    "emisivas": "emisivo",
    "emisividades": "emisividad",
    "emisivos": "emisivo",
    "emisora": "emisor",
    "emisoras": "emisora",
    "emisores": "emisor",
    "emita": "emitir",
    "emitamos": "emitir",
    "emitan": "emitir",
    "emitancias": "emitancia",
    "emitas": "emitir",
    "emite": "emitir",
    "emiten": "emitir",
    "emites": "emitir",
    "emitid": "emitir",
    "emitida": "emitir",
    "emitidas": "emitir",
    "emitido": "emitir",
    "emitidos": "emitir",
    "emitiendo": "emitir",
    "emitiera": "emitir",
    "emitierais": "emitir",
    "emitieran": "emitir",
    "emitieras": "emitir",
    "emitiere": "emitir",
    "emitiereis": "emitir",
    "emitieren": "emitir",
    "emitieres": "emitir",
    "emitieron": "emitir",
    "emitiese": "emitir",
    "emitieseis": "emitir",
    "emitiesen": "emitir",
    "emitieses": "emitir",
    "emitimos": "emitir",
    "emitiremos": "emitir",
    "emitirá": "emitir",
    "emitirán": "emitir",
    "emitirás": "emitir",
    "emitiré": "emitir",
    "emitiréis": "emitir",
    "emitiría": "emitir",
    "emitiríais": "emitir",
    "emitiríamos": "emitir",
    "emitirían": "emitir",
    "emitirías": "emitir",
    "emitiste": "emitir",
    "emitisteis": "emitir",
    "emitiéramos": "emitir",
    "emitiéremos": "emitir",
    "emitiésemos": "emitir",
    "emitió": "emitir",
    "emito": "emitir",
    "emitáis": "emitir",
    "emití": "emitir",
    "emitía": "emitir",
    "emitíais": "emitir",
    "emitíamos": "emitir",
    "emitían": "emitir",
    "emitías": "emitir",
    "emitís": "emitir",
    "emociona": "emocionar",
    "emocionaba": "emocionar",
    "emocionabais": "emocionar",
    "emocionaban": "emocionar",
    "emocionabas": "emocionar",
    "emocionabilidades": "emocionabilidad",
    "emocionables": "emocionable",
    "emocionad": "emocionar",
    "emocionada": "emocionar",
    "emocionadas": "emocionar",
    "emocionado": "emocionar",
    "emocionados": "emocionar",
    "emocionales": "emocional",
    "emocionalidades": "emocionalidad",
    "emocionamos": "emocionar",
    "emocionan": "emocionar",
    "emocionando": "emocionar",
    "emocionantes": "emocionante",
    "emocionara": "emocionar",
    "emocionarais": "emocionar",
    "emocionaran": "emocionar",
    "emocionaras": "emocionar",
    "emocionare": "emocionar",
    "emocionareis": "emocionar",
    "emocionaremos": "emocionar",
    "emocionaren": "emocionar",
    "emocionares": "emocionar",
    "emocionaron": "emocionar",
    "emocionará": "emocionar",
    "emocionarán": "emocionar",
    "emocionarás": "emocionar",
    "emocionaré": "emocionar",
    "emocionaréis": "emocionar",
    "emocionaría": "emocionar",
    "emocionaríais": "emocionar",
    "emocionaríamos": "emocionar",
    "emocionarían": "emocionar",
    "emocionarías": "emocionar",
    "emocionas": "emocionar",
    "emocionase": "emocionar",
    "emocionaseis": "emocionar",
    "emocionasen": "emocionar",
    "emocionases": "emocionar",
    "emocionaste": "emocionar",
    "emocionasteis": "emocionar",
    "emocione": "emocionar",
    "emocionemos": "emocionar",
    "emocionen": "emocionar",
    "emociones": "emocionar",
    "emociono": "emocionar",
    "emocionábamos": "emocionar",
    "emocionáis": "emocionar",
    "emocionáramos": "emocionar",
    "emocionáremos": "emocionar",
    "emocionásemos": "emocionar",
    "emocioné": "emocionar",
    "emocionéis": "emocionar",
    "emocionó": "emocionar",
    "emolientes": "emoliente",
    "emolumentos": "emolumento",
    "emotiva": "emotivo",
    "emotivas": "emotivo",
    "emotividades": "emotividad",
    "emotivos": "emotivo",
    "empaca": "empacar",
    "empacaba": "empacar",
    "empacabais": "empacar",
    "empacaban": "empacar",
    "empacabas": "empacar",
    "empacad": "empacar",
    "empacada": "empacar",
    "empacadas": "empacar",
    "empacado": "empacar",
    "empacadora": "empacador",
    "empacadoras": "empacadora",
    "empacadores": "empacador",
    "empacados": "empacar",
    "empacamos": "empacar",
    "empacan": "empacar",
    "empacando": "empacar",
    "empacara": "empacar",
    "empacarais": "empacar",
    "empacaran": "empacar",
    "empacaras": "empacar",
    "empacare": "empacar",
    "empacareis": "empacar",
    "empacaremos": "empacar",
    "empacaren": "empacar",
    "empacares": "empacar",
    "empacaron": "empacar",
    "empacará": "empacar",
    "empacarán": "empacar",
    "empacarás": "empacar",
    "empacaré": "empacar",
    "empacaréis": "empacar",
    "empacaría": "empacar",
    "empacaríais": "empacar",
    "empacaríamos": "empacar",
    "empacarían": "empacar",
    "empacarías": "empacar",
    "empacas": "empacar",
    "empacase": "empacar",
    "empacaseis": "empacar",
    "empacasen": "empacar",
    "empacases": "empacar",
    "empacaste": "empacar",
    "empacasteis": "empacar",
    "empacha": "empachar",
    "empachaba": "empachar",
    "empachabais": "empachar",
    "empachaban": "empachar",
    "empachabas": "empachar",
    "empachad": "empachar",
    "empachada": "empachar",
    "empachadas": "empachar",
    "empachado": "empachar",
    "empachados": "empachar",
    "empachamos": "empachar",
    "empachan": "empachar",
    "empachando": "empachar",
    "empachara": "empachar",
    "empacharais": "empachar",
    "empacharan": "empachar",
    "empacharas": "empachar",
    "empachare": "empachar",
    "empachareis": "empachar",
    "empacharemos": "empachar",
    "empacharen": "empachar",
    "empachares": "empachar",
    "empacharon": "empachar",
    "empachará": "empachar",
    "empacharán": "empachar",
    "empacharás": "empachar",
    "empacharé": "empachar",
    "empacharéis": "empachar",
    "empacharía": "empachar",
    "empacharíais": "empachar",
    "empacharíamos": "empachar",
    "empacharían": "empachar",
    "empacharías": "empachar",
    "empachas": "empachar",
    "empachase": "empachar",
    "empachaseis": "empachar",
    "empachasen": "empachar",
    "empachases": "empachar",
    "empachaste": "empachar",
    "empachasteis": "empachar",
    "empache": "empachar",
    "empachemos": "empachar",
    "empachen": "empachar",
    "empaches": "empachar",
    "empacho": "empachar",
    "empachos": "empacho",
    "empachosa": "empachoso",
    "empachosas": "empachoso",
    "empachosos": "empachoso",
    "empachábamos": "empachar",
    "empacháis": "empachar",
    "empacháramos": "empachar",
    "empacháremos": "empachar",
    "empachásemos": "empachar",
    "empaché": "empachar",
    "empachéis": "empachar",
    "empachó": "empachar",
    "empaco": "empacar",
    "empacábamos": "empacar",
    "empacáis": "empacar",
    "empacáramos": "empacar",
    "empacáremos": "empacar",
    "empacásemos": "empacar",
    "empacó": "empacar",
    "empadra": "empadrar",
    "empadraba": "empadrar",
    "empadrabais": "empadrar",
    "empadraban": "empadrar",
    "empadrabas": "empadrar",
    "empadrad": "empadrar",
    "empadrada": "empadrar",
    "empadradas": "empadrar",
    "empadrado": "empadrar",
    "empadrados": "empadrar",
    "empadramos": "empadrar",
    "empadran": "empadrar",
    "empadrando": "empadrar",
    "empadrara": "empadrar",
    "empadrarais": "empadrar",
    "empadraran": "empadrar",
    "empadraras": "empadrar",
    "empadrare": "empadrar",
    "empadrareis": "empadrar",
    "empadraremos": "empadrar",
    "empadraren": "empadrar",
    "empadrares": "empadrar",
    "empadraron": "empadrar",
    "empadrará": "empadrar",
    "empadrarán": "empadrar",
    "empadrarás": "empadrar",
    "empadraré": "empadrar",
    "empadraréis": "empadrar",
    "empadraría": "empadrar",
    "empadraríais": "empadrar",
    "empadraríamos": "empadrar",
    "empadrarían": "empadrar",
    "empadrarías": "empadrar",
    "empadras": "empadrar",
    "empadrase": "empadrar",
    "empadraseis": "empadrar",
    "empadrasen": "empadrar",
    "empadrases": "empadrar",
    "empadraste": "empadrar",
    "empadrasteis": "empadrar",
    "empadre": "empadrar",
    "empadremos": "empadrar",
    "empadren": "empadrar",
    "empadres": "empadrar",
    "empadro": "empadrar",
    "empadrona": "empadronar",
    "empadronaba": "empadronar",
    "empadronabais": "empadronar",
    "empadronaban": "empadronar",
    "empadronabas": "empadronar",
    "empadronad": "empadronar",
    "empadronada": "empadronar",
    "empadronadas": "empadronar",
    "empadronado": "empadronar",
    "empadronadora": "empadronador",
    "empadronadoras": "empadronador",
    "empadronadores": "empadronador",
    "empadronados": "empadronar",
    "empadronamientos": "empadronamiento",
    "empadronamos": "empadronar",
    "empadronan": "empadronar",
    "empadronando": "empadronar",
    "empadronara": "empadronar",
    "empadronarais": "empadronar",
    "empadronaran": "empadronar",
    "empadronaras": "empadronar",
    "empadronare": "empadronar",
    "empadronareis": "empadronar",
    "empadronaremos": "empadronar",
    "empadronaren": "empadronar",
    "empadronares": "empadronar",
    "empadronaron": "empadronar",
    "empadronará": "empadronar",
    "empadronarán": "empadronar",
    "empadronarás": "empadronar",
    "empadronaré": "empadronar",
    "empadronaréis": "empadronar",
    "empadronaría": "empadronar",
    "empadronaríais": "empadronar",
    "empadronaríamos": "empadronar",
    "empadronarían": "empadronar",
    "empadronarías": "empadronar",
    "empadronas": "empadronar",
    "empadronase": "empadronar",
    "empadronaseis": "empadronar",
    "empadronasen": "empadronar",
    "empadronases": "empadronar",
    "empadronaste": "empadronar",
    "empadronasteis": "empadronar",
    "empadrone": "empadronar",
    "empadronemos": "empadronar",
    "empadronen": "empadronar",
    "empadrones": "empadronar",
    "empadrono": "empadronar",
    "empadronábamos": "empadronar",
    "empadronáis": "empadronar",
    "empadronáramos": "empadronar",
    "empadronáremos": "empadronar",
    "empadronásemos": "empadronar",
    "empadroné": "empadronar",
    "empadronéis": "empadronar",
    "empadronó": "empadronar",
    "empadrábamos": "empadrar",
    "empadráis": "empadrar",
    "empadráramos": "empadrar",
    "empadráremos": "empadrar",
    "empadrásemos": "empadrar",
    "empadré": "empadrar",
    "empadréis": "empadrar",
    "empadró": "empadrar",
    "empaja": "empajar",
    "empajaba": "empajar",
    "empajabais": "empajar",
    "empajaban": "empajar",
    "empajabas": "empajar",
    "empajad": "empajar",
    "empajada": "empajar",
    "empajadas": "empajar",
    "empajado": "empajar",
    "empajados": "empajar",
    "empajamos": "empajar",
    "empajan": "empajar",
    "empajando": "empajar",
    "empajara": "empajar",
    "empajarais": "empajar",
    "empajaran": "empajar",
    "empajaras": "empajar",
    "empajare": "empajar",
    "empajareis": "empajar",
    "empajaremos": "empajar",
    "empajaren": "empajar",
    "empajares": "empajar",
    "empajaron": "empajar",
    "empajará": "empajar",
    "empajarán": "empajar",
    "empajarás": "empajar",
    "empajaré": "empajar",
    "empajaréis": "empajar",
    "empajaría": "empajar",
    "empajaríais": "empajar",
    "empajaríamos": "empajar",
    "empajarían": "empajar",
    "empajarías": "empajar",
    "empajas": "empajar",
    "empajase": "empajar",
    "empajaseis": "empajar",
    "empajasen": "empajar",
    "empajases": "empajar",
    "empajaste": "empajar",
    "empajasteis": "empajar",
    "empaje": "empajar",
    "empajemos": "empajar",
    "empajen": "empajar",
    "empajes": "empajar",
    "empajo": "empajar",
    "empajábamos": "empajar",
    "empajáis": "empajar",
    "empajáramos": "empajar",
    "empajáremos": "empajar",
    "empajásemos": "empajar",
    "empajé": "empajar",
    "empajéis": "empajar",
    "empajó": "empajar",
    "empala": "empalar",
    "empalaba": "empalar",
    "empalabais": "empalar",
    "empalaban": "empalar",
    "empalabas": "empalar",
    "empalad": "empalar",
    "empalada": "empalar",
    "empaladas": "empalar",
    "empalado": "empalar",
    "empalados": "empalar",
    "empalaga": "empalagar",
    "empalagaba": "empalagar",
    "empalagabais": "empalagar",
    "empalagaban": "empalagar",
    "empalagabas": "empalagar",
    "empalagad": "empalagar",
    "empalagada": "empalagar",
    "empalagadas": "empalagar",
    "empalagado": "empalagar",
    "empalagados": "empalagar",
    "empalagamientos": "empalagamiento",
    "empalagamos": "empalagar",
    "empalagan": "empalagar",
    "empalagando": "empalagar",
    "empalagara": "empalagar",
    "empalagarais": "empalagar",
    "empalagaran": "empalagar",
    "empalagaras": "empalagar",
    "empalagare": "empalagar",
    "empalagareis": "empalagar",
    "empalagaremos": "empalagar",
    "empalagaren": "empalagar",
    "empalagares": "empalagar",
    "empalagaron": "empalagar",
    "empalagará": "empalagar",
    "empalagarán": "empalagar",
    "empalagarás": "empalagar",
    "empalagaré": "empalagar",
    "empalagaréis": "empalagar",
    "empalagaría": "empalagar",
    "empalagaríais": "empalagar",
    "empalagaríamos": "empalagar",
    "empalagarían": "empalagar",
    "empalagarías": "empalagar",
    "empalagas": "empalagar",
    "empalagase": "empalagar",
    "empalagaseis": "empalagar",
    "empalagasen": "empalagar",
    "empalagases": "empalagar",
    "empalagaste": "empalagar",
    "empalagasteis": "empalagar",
    "empalago": "empalagar",
    "empalagos": "empalago",
    "empalagosa": "empalagoso",
    "empalagosas": "empalagoso",
    "empalagosos": "empalagoso",
    "empalague": "empalagar",
    "empalaguemos": "empalagar",
    "empalaguen": "empalagar",
    "empalagues": "empalagar",
    "empalagué": "empalagar",
    "empalaguéis": "empalagar",
    "empalagábamos": "empalagar",
    "empalagáis": "empalagar",
    "empalagáramos": "empalagar",
    "empalagáremos": "empalagar",
    "empalagásemos": "empalagar",
    "empalagó": "empalagar",
    "empalamientos": "empalamiento",
    "empalamos": "empalar",
    "empalan": "empalar",
    "empalando": "empalar",
    "empalara": "empalar",
    "empalarais": "empalar",
    "empalaran": "empalar",
    "empalaras": "empalar",
    "empalare": "empalar",
    "empalareis": "empalar",
    "empalaremos": "empalar",
    "empalaren": "empalar",
    "empalares": "empalar",
    "empalaron": "empalar",
    "empalará": "empalar",
    "empalarán": "empalar",
    "empalarás": "empalar",
    "empalaré": "empalar",
    "empalaréis": "empalar",
    "empalaría": "empalar",
    "empalaríais": "empalar",
    "empalaríamos": "empalar",
    "empalarían": "empalar",
    "empalarías": "empalar",
    "empalas": "empalar",
    "empalase": "empalar",
    "empalaseis": "empalar",
    "empalasen": "empalar",
    "empalases": "empalar",
    "empalaste": "empalar",
    "empalasteis": "empalar",
    "empale": "empalar",
    "empalemos": "empalar",
    "empalen": "empalar",
    "empales": "empale",
    "empalia": "empaliar",
    "empaliaba": "empaliar",
    "empaliabais": "empaliar",
    "empaliaban": "empaliar",
    "empaliabas": "empaliar",
    "empaliad": "empaliar",
    "empaliada": "empaliar",
    "empaliadas": "empaliar",
    "empaliado": "empaliar",
    "empaliados": "empaliar",
    "empaliamos": "empaliar",
    "empalian": "empaliar",
    "empaliando": "empaliar",
    "empaliara": "empaliar",
    "empaliarais": "empaliar",
    "empaliaran": "empaliar",
    "empaliaras": "empaliar",
    "empaliare": "empaliar",
    "empaliareis": "empaliar",
    "empaliaremos": "empaliar",
    "empaliaren": "empaliar",
    "empaliares": "empaliar",
    "empaliaron": "empaliar",
    "empaliará": "empaliar",
    "empaliarán": "empaliar",
    "empaliarás": "empaliar",
    "empaliaré": "empaliar",
    "empaliaréis": "empaliar",
    "empaliaría": "empaliar",
    "empaliaríais": "empaliar",
    "empaliaríamos": "empaliar",
    "empaliarían": "empaliar",
    "empaliarías": "empaliar",
    "empalias": "empaliar",
    "empaliase": "empaliar",
    "empaliaseis": "empaliar",
    "empaliasen": "empaliar",
    "empaliases": "empaliar",
    "empaliaste": "empaliar",
    "empaliasteis": "empaliar",
    "empalice": "empalizar",
    "empalicemos": "empalizar",
    "empalicen": "empalizar",
    "empalices": "empalizar",
    "empalicé": "empalizar",
    "empalicéis": "empalizar",
    "empalidecimientos": "empalidecimiento",
    "empalie": "empaliar",
    "empaliemos": "empaliar",
    "empalien": "empaliar",
    "empalies": "empaliar",
    "empalio": "empaliar",
    "empaliza": "empalizar",
    "empalizaba": "empalizar",
    "empalizabais": "empalizar",
    "empalizaban": "empalizar",
    "empalizabas": "empalizar",
    "empalizad": "empalizar",
    "empalizada": "empalizar",
    "empalizadas": "empalizar",
    "empalizado": "empalizar",
    "empalizados": "empalizar",
    "empalizamos": "empalizar",
    "empalizan": "empalizar",
    "empalizando": "empalizar",
    "empalizara": "empalizar",
    "empalizarais": "empalizar",
    "empalizaran": "empalizar",
    "empalizaras": "empalizar",
    "empalizare": "empalizar",
    "empalizareis": "empalizar",
    "empalizaremos": "empalizar",
    "empalizaren": "empalizar",
    "empalizares": "empalizar",
    "empalizaron": "empalizar",
    "empalizará": "empalizar",
    "empalizarán": "empalizar",
    "empalizarás": "empalizar",
    "empalizaré": "empalizar",
    "empalizaréis": "empalizar",
    "empalizaría": "empalizar",
    "empalizaríais": "empalizar",
    "empalizaríamos": "empalizar",
    "empalizarían": "empalizar",
    "empalizarías": "empalizar",
    "empalizas": "empalizar",
    "empalizase": "empalizar",
    "empalizaseis": "empalizar",
    "empalizasen": "empalizar",
    "empalizases": "empalizar",
    "empalizaste": "empalizar",
    "empalizasteis": "empalizar",
    "empalizo": "empalizar",
    "empalizábamos": "empalizar",
    "empalizáis": "empalizar",
    "empalizáramos": "empalizar",
    "empalizáremos": "empalizar",
    "empalizásemos": "empalizar",
    "empalizó": "empalizar",
    "empaliábamos": "empaliar",
    "empaliáis": "empaliar",
    "empaliáramos": "empaliar",
    "empaliáremos": "empaliar",
    "empaliásemos": "empaliar",
    "empalié": "empaliar",
    "empaliéis": "empaliar",
    "empalió": "empaliar",
    "empalletados": "empalletado",
    "empalma": "empalmar",
    "empalmaba": "empalmar",
    "empalmabais": "empalmar",
    "empalmaban": "empalmar",
    "empalmabas": "empalmar",
    "empalmad": "empalmar",
    "empalmada": "empalmar",
    "empalmadas": "empalmar",
    "empalmado": "empalmar",
    "empalmadoras": "empalmadora",
    "empalmados": "empalmar",
    "empalmaduras": "empalmadura",
    "empalmamos": "empalmar",
    "empalman": "empalmar",
    "empalmando": "empalmar",
    "empalmara": "empalmar",
    "empalmarais": "empalmar",
    "empalmaran": "empalmar",
    "empalmaras": "empalmar",
    "empalmare": "empalmar",
    "empalmareis": "empalmar",
    "empalmaremos": "empalmar",
    "empalmaren": "empalmar",
    "empalmares": "empalmar",
    "empalmaron": "empalmar",
    "empalmará": "empalmar",
    "empalmarán": "empalmar",
    "empalmarás": "empalmar",
    "empalmaré": "empalmar",
    "empalmaréis": "empalmar",
    "empalmaría": "empalmar",
    "empalmaríais": "empalmar",
    "empalmaríamos": "empalmar",
    "empalmarían": "empalmar",
    "empalmarías": "empalmar",
    "empalmas": "empalmar",
    "empalmase": "empalmar",
    "empalmaseis": "empalmar",
    "empalmasen": "empalmar",
    "empalmases": "empalmar",
    "empalmaste": "empalmar",
    "empalmasteis": "empalmar",
    "empalme": "empalmar",
    "empalmemos": "empalmar",
    "empalmen": "empalmar",
    "empalmes": "empalme",
    "empalmo": "empalmar",
    "empalmábamos": "empalmar",
    "empalmáis": "empalmar",
    "empalmáramos": "empalmar",
    "empalmáremos": "empalmar",
    "empalmásemos": "empalmar",
    "empalmé": "empalmar",
    "empalméis": "empalmar",
    "empalmó": "empalmar",
    "empalo": "empalar",
    "empaloma": "empalomar",
    "empalomaba": "empalomar",
    "empalomabais": "empalomar",
    "empalomaban": "empalomar",
    "empalomabas": "empalomar",
    "empalomad": "empalomar",
    "empalomada": "empalomar",
    "empalomadas": "empalomar",
    "empalomado": "empalomar",
    "empalomados": "empalomar",
    "empalomaduras": "empalomadura",
    "empalomamos": "empalomar",
    "empaloman": "empalomar",
    "empalomando": "empalomar",
    "empalomara": "empalomar",
    "empalomarais": "empalomar",
    "empalomaran": "empalomar",
    "empalomaras": "empalomar",
    "empalomare": "empalomar",
    "empalomareis": "empalomar",
    "empalomaremos": "empalomar",
    "empalomaren": "empalomar",
    "empalomares": "empalomar",
    "empalomaron": "empalomar",
    "empalomará": "empalomar",
    "empalomarán": "empalomar",
    "empalomarás": "empalomar",
    "empalomaré": "empalomar",
    "empalomaréis": "empalomar",
    "empalomaría": "empalomar",
    "empalomaríais": "empalomar",
    "empalomaríamos": "empalomar",
    "empalomarían": "empalomar",
    "empalomarías": "empalomar",
    "empalomas": "empalomar",
    "empalomase": "empalomar",
    "empalomaseis": "empalomar",
    "empalomasen": "empalomar",
    "empalomases": "empalomar",
    "empalomaste": "empalomar",
    "empalomasteis": "empalomar",
    "empalome": "empalomar",
    "empalomemos": "empalomar",
    "empalomen": "empalomar",
    "empalomes": "empalomar",
    "empalomo": "empalomar",
    "empalomábamos": "empalomar",
    "empalomáis": "empalomar",
    "empalomáramos": "empalomar",
    "empalomáremos": "empalomar",
    "empalomásemos": "empalomar",
    "empalomé": "empalomar",
    "empaloméis": "empalomar",
    "empalomó": "empalomar",
    "empalábamos": "empalar",
    "empaláis": "empalar",
    "empaláramos": "empalar",
    "empaláremos": "empalar",
    "empalásemos": "empalar",
    "empalé": "empalar",
    "empaléis": "empalar",
    "empaló": "empalar",
    "empampa": "empampar",
    "empampaba": "empampar",
    "empampabais": "empampar",
    "empampaban": "empampar",
    "empampabas": "empampar",
    "empampad": "empampar",
    "empampada": "empampar",
    "empampadas": "empampar",
    "empampado": "empampar",
    "empampados": "empampar",
    "empampamos": "empampar",
    "empampan": "empampar",
    "empampando": "empampar",
    "empampara": "empampar",
    "empamparais": "empampar",
    "empamparan": "empampar",
    "empamparas": "empampar",
    "empampare": "empampar",
    "empampareis": "empampar",
    "empamparemos": "empampar",
    "empamparen": "empampar",
    "empampares": "empampar",
    "empamparon": "empampar",
    "empampará": "empampar",
    "empamparán": "empampar",
    "empamparás": "empampar",
    "empamparé": "empampar",
    "empamparéis": "empampar",
    "empamparía": "empampar",
    "empamparíais": "empampar",
    "empamparíamos": "empampar",
    "empamparían": "empampar",
    "empamparías": "empampar",
    "empampas": "empampar",
    "empampase": "empampar",
    "empampaseis": "empampar",
    "empampasen": "empampar",
    "empampases": "empampar",
    "empampaste": "empampar",
    "empampasteis": "empampar",
    "empampe": "empampar",
    "empampemos": "empampar",
    "empampen": "empampar",
    "empampes": "empampar",
    "empampo": "empampar",
    "empampábamos": "empampar",
    "empampáis": "empampar",
    "empampáramos": "empampar",
    "empampáremos": "empampar",
    "empampásemos": "empampar",
    "empampé": "empampar",
    "empampéis": "empampar",
    "empampó": "empampar",
    "empana": "empanar",
    "empanaba": "empanar",
    "empanabais": "empanar",
    "empanaban": "empanar",
    "empanabas": "empanar",
    "empanad": "empanar",
    "empanada": "empanar",
    "empanadas": "empanar",
    "empanadillas": "empanadilla",
    "empanado": "empanar",
    "empanados": "empanar",
    "empanamos": "empanar",
    "empanan": "empanar",
    "empanando": "empanar",
    "empanara": "empanar",
    "empanarais": "empanar",
    "empanaran": "empanar",
    "empanaras": "empanar",
    "empanare": "empanar",
    "empanareis": "empanar",
    "empanaremos": "empanar",
    "empanaren": "empanar",
    "empanares": "empanar",
    "empanaron": "empanar",
    "empanará": "empanar",
    "empanarán": "empanar",
    "empanarás": "empanar",
    "empanaré": "empanar",
    "empanaréis": "empanar",
    "empanaría": "empanar",
    "empanaríais": "empanar",
    "empanaríamos": "empanar",
    "empanarían": "empanar",
    "empanarías": "empanar",
    "empanas": "empanar",
    "empanase": "empanar",
    "empanaseis": "empanar",
    "empanasen": "empanar",
    "empanases": "empanar",
    "empanaste": "empanar",
    "empanasteis": "empanar",
    "empance": "empanzar",
    "empancemos": "empanzar",
    "empancen": "empanzar",
    "empances": "empanzar",
    "empancé": "empanzar",
    "empancéis": "empanzar",
    "empane": "empanar",
    "empanemos": "empanar",
    "empanen": "empanar",
    "empanes": "empanar",
    "empano": "empanar",
    "empantana": "empantanar",
    "empantanaba": "empantanar",
    "empantanabais": "empantanar",
    "empantanaban": "empantanar",
    "empantanabas": "empantanar",
    "empantanad": "empantanar",
    "empantanada": "empantanar",
    "empantanadas": "empantanar",
    "empantanado": "empantanar",
    "empantanados": "empantanar",
    "empantanamos": "empantanar",
    "empantanan": "empantanar",
    "empantanando": "empantanar",
    "empantanara": "empantanar",
    "empantanarais": "empantanar",
    "empantanaran": "empantanar",
    "empantanaras": "empantanar",
    "empantanare": "empantanar",
    "empantanareis": "empantanar",
    "empantanaremos": "empantanar",
    "empantanaren": "empantanar",
    "empantanares": "empantanar",
    "empantanaron": "empantanar",
    "empantanará": "empantanar",
    "empantanarán": "empantanar",
    "empantanarás": "empantanar",
    "empantanaré": "empantanar",
    "empantanaréis": "empantanar",
    "empantanaría": "empantanar",
    "empantanaríais": "empantanar",
    "empantanaríamos": "empantanar",
    "empantanarían": "empantanar",
    "empantanarías": "empantanar",
    "empantanas": "empantanar",
    "empantanase": "empantanar",
    "empantanaseis": "empantanar",
    "empantanasen": "empantanar",
    "empantanases": "empantanar",
    "empantanaste": "empantanar",
    "empantanasteis": "empantanar",
    "empantane": "empantanar",
    "empantanemos": "empantanar",
    "empantanen": "empantanar",
    "empantanes": "empantanar",
    "empantano": "empantanar",
    "empantanábamos": "empantanar",
    "empantanáis": "empantanar",
    "empantanáramos": "empantanar",
    "empantanáremos": "empantanar",
    "empantanásemos": "empantanar",
    "empantané": "empantanar",
    "empantanéis": "empantanar",
    "empantanó": "empantanar",
    "empanza": "empanzar",
    "empanzaba": "empanzar",
    "empanzabais": "empanzar",
    "empanzaban": "empanzar",
    "empanzabas": "empanzar",
    "empanzad": "empanzar",
    "empanzada": "empanzar",
    "empanzadas": "empanzar",
    "empanzado": "empanzar",
    "empanzados": "empanzar",
    "empanzamos": "empanzar",
    "empanzan": "empanzar",
    "empanzando": "empanzar",
    "empanzara": "empanzar",
    "empanzarais": "empanzar",
    "empanzaran": "empanzar",
    "empanzaras": "empanzar",
    "empanzare": "empanzar",
    "empanzareis": "empanzar",
    "empanzaremos": "empanzar",
    "empanzaren": "empanzar",
    "empanzares": "empanzar",
    "empanzaron": "empanzar",
    "empanzará": "empanzar",
    "empanzarán": "empanzar",
    "empanzarás": "empanzar",
    "empanzaré": "empanzar",
    "empanzaréis": "empanzar",
    "empanzaría": "empanzar",
    "empanzaríais": "empanzar",
    "empanzaríamos": "empanzar",
    "empanzarían": "empanzar",
    "empanzarías": "empanzar",
    "empanzas": "empanzar",
    "empanzase": "empanzar",
    "empanzaseis": "empanzar",
    "empanzasen": "empanzar",
    "empanzases": "empanzar",
    "empanzaste": "empanzar",
    "empanzasteis": "empanzar",
    "empanzo": "empanzar",
    "empanzábamos": "empanzar",
    "empanzáis": "empanzar",
    "empanzáramos": "empanzar",
    "empanzáremos": "empanzar",
    "empanzásemos": "empanzar",
    "empanzó": "empanzar",
    "empanábamos": "empanar",
    "empanáis": "empanar",
    "empanáramos": "empanar",
    "empanáremos": "empanar",
    "empanásemos": "empanar",
    "empané": "empanar",
    "empanéis": "empanar",
    "empanó": "empanar",
    "empapa": "empapar",
    "empapaba": "empapar",
    "empapabais": "empapar",
    "empapaban": "empapar",
    "empapabas": "empapar",
    "empapad": "empapar",
    "empapada": "empapar",
    "empapadas": "empapar",
    "empapado": "empapar",
    "empapados": "empapar",
    "empapamos": "empapar",
    "empapan": "empapar",
    "empapando": "empapar",
    "empapara": "empapar",
    "empaparais": "empapar",
    "empaparan": "empapar",
    "empaparas": "empapar",
    "empapare": "empapar",
    "empapareis": "empapar",
    "empaparemos": "empapar",
    "empaparen": "empapar",
    "empapares": "empapar",
    "empaparon": "empapar",
    "empapará": "empapar",
    "empaparán": "empapar",
    "empaparás": "empapar",
    "empaparé": "empapar",
    "empaparéis": "empapar",
    "empaparía": "empapar",
    "empaparíais": "empapar",
    "empaparíamos": "empapar",
    "empaparían": "empapar",
    "empaparías": "empapar",
    "empapas": "empapar",
    "empapase": "empapar",
    "empapaseis": "empapar",
    "empapasen": "empapar",
    "empapases": "empapar",
    "empapaste": "empapar",
    "empapasteis": "empapar",
    "empape": "empapar",
    "empapela": "empapelar",
    "empapelaba": "empapelar",
    "empapelabais": "empapelar",
    "empapelaban": "empapelar",
    "empapelabas": "empapelar",
    "empapelad": "empapelar",
    "empapelada": "empapelar",
    "empapeladas": "empapelar",
    "empapelado": "empapelar",
    "empapeladora": "empapelador",
    "empapeladoras": "empapelador",
    "empapeladores": "empapelador",
    "empapelados": "empapelar",
    "empapelamos": "empapelar",
    "empapelan": "empapelar",
    "empapelando": "empapelar",
    "empapelara": "empapelar",
    "empapelarais": "empapelar",
    "empapelaran": "empapelar",
    "empapelaras": "empapelar",
    "empapelare": "empapelar",
    "empapelareis": "empapelar",
    "empapelaremos": "empapelar",
    "empapelaren": "empapelar",
    "empapelares": "empapelar",
    "empapelaron": "empapelar",
    "empapelará": "empapelar",
    "empapelarán": "empapelar",
    "empapelarás": "empapelar",
    "empapelaré": "empapelar",
    "empapelaréis": "empapelar",
    "empapelaría": "empapelar",
    "empapelaríais": "empapelar",
    "empapelaríamos": "empapelar",
    "empapelarían": "empapelar",
    "empapelarías": "empapelar",
    "empapelas": "empapelar",
    "empapelase": "empapelar",
    "empapelaseis": "empapelar",
    "empapelasen": "empapelar",
    "empapelases": "empapelar",
    "empapelaste": "empapelar",
    "empapelasteis": "empapelar",
    "empapele": "empapelar",
    "empapelemos": "empapelar",
    "empapelen": "empapelar",
    "empapeles": "empapelar",
    "empapelo": "empapelar",
    "empapelábamos": "empapelar",
    "empapeláis": "empapelar",
    "empapeláramos": "empapelar",
    "empapeláremos": "empapelar",
    "empapelásemos": "empapelar",
    "empapelé": "empapelar",
    "empapeléis": "empapelar",
    "empapeló": "empapelar",
    "empapemos": "empapar",
    "empapen": "empapar",
    "empapes": "empapar",
    "empapirota": "empapirotar",
    "empapirotaba": "empapirotar",
    "empapirotabais": "empapirotar",
    "empapirotaban": "empapirotar",
    "empapirotabas": "empapirotar",
    "empapirotad": "empapirotar",
    "empapirotada": "empapirotar",
    "empapirotadas": "empapirotar",
    "empapirotado": "empapirotar",
    "empapirotados": "empapirotar",
    "empapirotamos": "empapirotar",
    "empapirotan": "empapirotar",
    "empapirotando": "empapirotar",
    "empapirotara": "empapirotar",
    "empapirotarais": "empapirotar",
    "empapirotaran": "empapirotar",
    "empapirotaras": "empapirotar",
    "empapirotare": "empapirotar",
    "empapirotareis": "empapirotar",
    "empapirotaremos": "empapirotar",
    "empapirotaren": "empapirotar",
    "empapirotares": "empapirotar",
    "empapirotaron": "empapirotar",
    "empapirotará": "empapirotar",
    "empapirotarán": "empapirotar",
    "empapirotarás": "empapirotar",
    "empapirotaré": "empapirotar",
    "empapirotaréis": "empapirotar",
    "empapirotaría": "empapirotar",
    "empapirotaríais": "empapirotar",
    "empapirotaríamos": "empapirotar",
    "empapirotarían": "empapirotar",
    "empapirotarías": "empapirotar",
    "empapirotas": "empapirotar",
    "empapirotase": "empapirotar",
    "empapirotaseis": "empapirotar",
    "empapirotasen": "empapirotar",
    "empapirotases": "empapirotar",
    "empapirotaste": "empapirotar",
    "empapirotasteis": "empapirotar",
    "empapirote": "empapirotar",
    "empapirotemos": "empapirotar",
    "empapiroten": "empapirotar",
    "empapirotes": "empapirotar",
    "empapiroto": "empapirotar",
    "empapirotábamos": "empapirotar",
    "empapirotáis": "empapirotar",
    "empapirotáramos": "empapirotar",
    "empapirotáremos": "empapirotar",
    "empapirotásemos": "empapirotar",
    "empapiroté": "empapirotar",
    "empapirotéis": "empapirotar",
    "empapirotó": "empapirotar",
    "empapo": "empapar",
    "empapuzamientos": "empapuzamiento",
    "empapábamos": "empapar",
    "empapáis": "empapar",
    "empapáramos": "empapar",
    "empapáremos": "empapar",
    "empapásemos": "empapar",
    "empapé": "empapar",
    "empapéis": "empapar",
    "empapó": "empapar",
    "empaque": "empacar",
    "empaquemos": "empacar",
    "empaquen": "empacar",
    "empaques": "empaque",
    "empaqueta": "empaquetar",
    "empaquetaba": "empaquetar",
    "empaquetabais": "empaquetar",
    "empaquetaban": "empaquetar",
    "empaquetabas": "empaquetar",
    "empaquetad": "empaquetar",
    "empaquetada": "empaquetar",
    "empaquetadas": "empaquetar",
    "empaquetado": "empaquetar",
    "empaquetadora": "empaquetador",
    "empaquetadoras": "empaquetadora",
    "empaquetadores": "empaquetador",
    "empaquetados": "empaquetar",
    "empaquetaduras": "empaquetadura",
    "empaquetamientos": "empaquetamiento",
    "empaquetamos": "empaquetar",
    "empaquetan": "empaquetar",
    "empaquetando": "empaquetar",
    "empaquetara": "empaquetar",
    "empaquetarais": "empaquetar",
    "empaquetaran": "empaquetar",
    "empaquetaras": "empaquetar",
    "empaquetare": "empaquetar",
    "empaquetareis": "empaquetar",
    "empaquetaremos": "empaquetar",
    "empaquetaren": "empaquetar",
    "empaquetares": "empaquetar",
    "empaquetaron": "empaquetar",
    "empaquetará": "empaquetar",
    "empaquetarán": "empaquetar",
    "empaquetarás": "empaquetar",
    "empaquetaré": "empaquetar",
    "empaquetaréis": "empaquetar",
    "empaquetaría": "empaquetar",
    "empaquetaríais": "empaquetar",
    "empaquetaríamos": "empaquetar",
    "empaquetarían": "empaquetar",
    "empaquetarías": "empaquetar",
    "empaquetas": "empaquetar",
    "empaquetase": "empaquetar",
    "empaquetaseis": "empaquetar",
    "empaquetasen": "empaquetar",
    "empaquetases": "empaquetar",
    "empaquetaste": "empaquetar",
    "empaquetasteis": "empaquetar",
    "empaquete": "empaquetar",
    "empaquetemos": "empaquetar",
    "empaqueten": "empaquetar",
    "empaquetes": "empaquetar",
    "empaqueto": "empaquetar",
    "empaquetábamos": "empaquetar",
    "empaquetáis": "empaquetar",
    "empaquetáramos": "empaquetar",
    "empaquetáremos": "empaquetar",
    "empaquetásemos": "empaquetar",
    "empaqueté": "empaquetar",
    "empaquetéis": "empaquetar",
    "empaquetó": "empaquetar",
    "empaqué": "empacar",
    "empaquéis": "empacar",
    "emparama": "emparamar",
    "emparamaba": "emparamar",
    "emparamabais": "emparamar",
    "emparamaban": "emparamar",
    "emparamabas": "emparamar",
    "emparamad": "emparamar",
    "emparamada": "emparamar",
    "emparamadas": "emparamar",
    "emparamado": "emparamar",
    "emparamados": "emparamar",
    "emparamamos": "emparamar",
    "emparaman": "emparamar",
    "emparamando": "emparamar",
    "emparamara": "emparamar",
    "emparamarais": "emparamar",
    "emparamaran": "emparamar",
    "emparamaras": "emparamar",
    "emparamare": "emparamar",
    "emparamareis": "emparamar",
    "emparamaremos": "emparamar",
    "emparamaren": "emparamar",
    "emparamares": "emparamar",
    "emparamaron": "emparamar",
    "emparamará": "emparamar",
    "emparamarán": "emparamar",
    "emparamarás": "emparamar",
    "emparamaré": "emparamar",
    "emparamaréis": "emparamar",
    "emparamaría": "emparamar",
    "emparamaríais": "emparamar",
    "emparamaríamos": "emparamar",
    "emparamarían": "emparamar",
    "emparamarías": "emparamar",
    "emparamas": "emparamar",
    "emparamase": "emparamar",
    "emparamaseis": "emparamar",
    "emparamasen": "emparamar",
    "emparamases": "emparamar",
    "emparamaste": "emparamar",
    "emparamasteis": "emparamar",
    "emparame": "emparamar",
    "emparamemos": "emparamar",
    "emparamen": "emparamar",
    "emparamenta": "emparamentar",
    "emparamentaba": "emparamentar",
    "emparamentabais": "emparamentar",
    "emparamentaban": "emparamentar",
    "emparamentabas": "emparamentar",
    "emparamentad": "emparamentar",
    "emparamentada": "emparamentar",
    "emparamentadas": "emparamentar",
    "emparamentado": "emparamentar",
    "emparamentados": "emparamentar",
    "emparamentamos": "emparamentar",
    "emparamentan": "emparamentar",
    "emparamentando": "emparamentar",
    "emparamentara": "emparamentar",
    "emparamentarais": "emparamentar",
    "emparamentaran": "emparamentar",
    "emparamentaras": "emparamentar",
    "emparamentare": "emparamentar",
    "emparamentareis": "emparamentar",
    "emparamentaremos": "emparamentar",
    "emparamentaren": "emparamentar",
    "emparamentares": "emparamentar",
    "emparamentaron": "emparamentar",
    "emparamentará": "emparamentar",
    "emparamentarán": "emparamentar",
    "emparamentarás": "emparamentar",
    "emparamentaré": "emparamentar",
    "emparamentaréis": "emparamentar",
    "emparamentaría": "emparamentar",
    "emparamentaríais": "emparamentar",
    "emparamentaríamos": "emparamentar",
    "emparamentarían": "emparamentar",
    "emparamentarías": "emparamentar",
    "emparamentas": "emparamentar",
    "emparamentase": "emparamentar",
    "emparamentaseis": "emparamentar",
    "emparamentasen": "emparamentar",
    "emparamentases": "emparamentar",
    "emparamentaste": "emparamentar",
    "emparamentasteis": "emparamentar",
    "emparamente": "emparamentar",
    "emparamentemos": "emparamentar",
    "emparamenten": "emparamentar",
    "emparamentes": "emparamentar",
    "emparamento": "emparamentar",
    "emparamentábamos": "emparamentar",
    "emparamentáis": "emparamentar",
    "emparamentáramos": "emparamentar",
    "emparamentáremos": "emparamentar",
    "emparamentásemos": "emparamentar",
    "emparamenté": "emparamentar",
    "emparamentéis": "emparamentar",
    "emparamentó": "emparamentar",
    "emparames": "emparamar",
    "emparamo": "emparamar",
    "emparamábamos": "emparamar",
    "emparamáis": "emparamar",
    "emparamáramos": "emparamar",
    "emparamáremos": "emparamar",
    "emparamásemos": "emparamar",
    "emparamé": "emparamar",
    "emparaméis": "emparamar",
    "emparamó": "emparamar",
    "emparas": "empara",
    "emparcha": "emparchar",
    "emparchaba": "emparchar",
    "emparchabais": "emparchar",
    "emparchaban": "emparchar",
    "emparchabas": "emparchar",
    "emparchad": "emparchar",
    "emparchada": "emparchar",
    "emparchadas": "emparchar",
    "emparchado": "emparchar",
    "emparchados": "emparchar",
    "emparchamos": "emparchar",
    "emparchan": "emparchar",
    "emparchando": "emparchar",
    "emparchara": "emparchar",
    "emparcharais": "emparchar",
    "emparcharan": "emparchar",
    "emparcharas": "emparchar",
    "emparchare": "emparchar",
    "emparchareis": "emparchar",
    "emparcharemos": "emparchar",
    "emparcharen": "emparchar",
    "emparchares": "emparchar",
    "emparcharon": "emparchar",
    "emparchará": "emparchar",
    "emparcharán": "emparchar",
    "emparcharás": "emparchar",
    "emparcharé": "emparchar",
    "emparcharéis": "emparchar",
    "emparcharía": "emparchar",
    "emparcharíais": "emparchar",
    "emparcharíamos": "emparchar",
    "emparcharían": "emparchar",
    "emparcharías": "emparchar",
    "emparchas": "emparchar",
    "emparchase": "emparchar",
    "emparchaseis": "emparchar",
    "emparchasen": "emparchar",
    "emparchases": "emparchar",
    "emparchaste": "emparchar",
    "emparchasteis": "emparchar",
    "emparche": "emparchar",
    "emparchemos": "emparchar",
    "emparchen": "emparchar",
    "emparches": "emparchar",
    "emparcho": "emparchar",
    "emparchábamos": "emparchar",
    "emparcháis": "emparchar",
    "emparcháramos": "emparchar",
    "emparcháremos": "emparchar",
    "emparchásemos": "emparchar",
    "emparché": "emparchar",
    "emparchéis": "emparchar",
    "emparchó": "emparchar",
    "empardecida": "empardecido",
    "empardecidas": "empardecido",
    "empardecidos": "empardecido",
    "empareda": "emparedar",
    "emparedaba": "emparedar",
    "emparedabais": "emparedar",
    "emparedaban": "emparedar",
    "emparedabas": "emparedar",
    "emparedad": "emparedar",
    "emparedada": "emparedar",
    "emparedadas": "emparedar",
    "emparedado": "emparedar",
    "emparedados": "emparedar",
    "emparedamientos": "emparedamiento",
    "emparedamos": "emparedar",
    "emparedan": "emparedar",
    "emparedando": "emparedar",
    "emparedara": "emparedar",
    "emparedarais": "emparedar",
    "emparedaran": "emparedar",
    "emparedaras": "emparedar",
    "emparedare": "emparedar",
    "emparedareis": "emparedar",
    "emparedaremos": "emparedar",
    "emparedaren": "emparedar",
    "emparedares": "emparedar",
    "emparedaron": "emparedar",
    "emparedará": "emparedar",
    "emparedarán": "emparedar",
    "emparedarás": "emparedar",
    "emparedaré": "emparedar",
    "emparedaréis": "emparedar",
    "emparedaría": "emparedar",
    "emparedaríais": "emparedar",
    "emparedaríamos": "emparedar",
    "emparedarían": "emparedar",
    "emparedarías": "emparedar",
    "emparedas": "emparedar",
    "emparedase": "emparedar",
    "emparedaseis": "emparedar",
    "emparedasen": "emparedar",
    "emparedases": "emparedar",
    "emparedaste": "emparedar",
    "emparedasteis": "emparedar",
    "emparede": "emparedar",
    "emparedemos": "emparedar",
    "empareden": "emparedar",
    "emparedes": "emparedar",
    "emparedo": "emparedar",
    "emparedábamos": "emparedar",
    "emparedáis": "emparedar",
    "emparedáramos": "emparedar",
    "emparedáremos": "emparedar",
    "emparedásemos": "emparedar",
    "emparedé": "emparedar",
    "emparedéis": "emparedar",
    "emparedó": "emparedar",
    "empareja": "emparejar",
    "emparejaba": "emparejar",
    "emparejabais": "emparejar",
    "emparejaban": "emparejar",
    "emparejabas": "emparejar",
    "emparejad": "emparejar",
    "emparejada": "emparejar",
    "emparejadas": "emparejar",
    "emparejado": "emparejar",
    "emparejadora": "emparejador",
    "emparejadoras": "emparejador",
    "emparejadores": "emparejador",
    "emparejados": "emparejar",
    "emparejaduras": "emparejadura",
    "emparejamientos": "emparejamiento",
    "emparejamos": "emparejar",
    "emparejan": "emparejar",
    "emparejando": "emparejar",
    "emparejara": "emparejar",
    "emparejarais": "emparejar",
    "emparejaran": "emparejar",
    "emparejaras": "emparejar",
    "emparejare": "emparejar",
    "emparejareis": "emparejar",
    "emparejaremos": "emparejar",
    "emparejaren": "emparejar",
    "emparejares": "emparejar",
    "emparejaron": "emparejar",
    "emparejará": "emparejar",
    "emparejarán": "emparejar",
    "emparejarás": "emparejar",
    "emparejaré": "emparejar",
    "emparejaréis": "emparejar",
    "emparejaría": "emparejar",
    "emparejaríais": "emparejar",
    "emparejaríamos": "emparejar",
    "emparejarían": "emparejar",
    "emparejarías": "emparejar",
    "emparejas": "emparejar",
    "emparejase": "emparejar",
    "emparejaseis": "emparejar",
    "emparejasen": "emparejar",
    "emparejases": "emparejar",
    "emparejaste": "emparejar",
    "emparejasteis": "emparejar",
    "empareje": "emparejar",
    "emparejemos": "emparejar",
    "emparejen": "emparejar",
    "emparejes": "emparejar",
    "emparejo": "emparejar",
    "emparejábamos": "emparejar",
    "emparejáis": "emparejar",
    "emparejáramos": "emparejar",
    "emparejáremos": "emparejar",
    "emparejásemos": "emparejar",
    "emparejé": "emparejar",
    "emparejéis": "emparejar",
    "emparejó": "emparejar",
    "emparenta": "emparentar",
    "emparentaba": "emparentar",
    "emparentabais": "emparentar",
    "emparentaban": "emparentar",
    "emparentabas": "emparentar",
    "emparentad": "emparentar",
    "emparentada": "emparentar",
    "emparentadas": "emparentar",
    "emparentado": "emparentar",
    "emparentados": "emparentar",
    "emparentamos": "emparentar",
    "emparentan": "emparentar",
    "emparentando": "emparentar",
    "emparentara": "emparentar",
    "emparentarais": "emparentar",
    "emparentaran": "emparentar",
    "emparentaras": "emparentar",
    "emparentare": "emparentar",
    "emparentareis": "emparentar",
    "emparentaremos": "emparentar",
    "emparentaren": "emparentar",
    "emparentares": "emparentar",
    "emparentaron": "emparentar",
    "emparentará": "emparentar",
    "emparentarán": "emparentar",
    "emparentarás": "emparentar",
    "emparentaré": "emparentar",
    "emparentaréis": "emparentar",
    "emparentaría": "emparentar",
    "emparentaríais": "emparentar",
    "emparentaríamos": "emparentar",
    "emparentarían": "emparentar",
    "emparentarías": "emparentar",
    "emparentas": "emparentar",
    "emparentase": "emparentar",
    "emparentaseis": "emparentar",
    "emparentasen": "emparentar",
    "emparentases": "emparentar",
    "emparentaste": "emparentar",
    "emparentasteis": "emparentar",
    "emparente": "emparentar",
    "emparentemos": "emparentar",
    "emparenten": "emparentar",
    "emparentes": "emparentar",
    "emparento": "emparentar",
    "emparentábamos": "emparentar",
    "emparentáis": "emparentar",
    "emparentáramos": "emparentar",
    "emparentáremos": "emparentar",
    "emparentásemos": "emparentar",
    "emparenté": "emparentar",
    "emparentéis": "emparentar",
    "emparentó": "emparentar",
    "emparienta": "emparentar",
    "emparientan": "emparentar",
    "emparientas": "emparentar",
    "empariente": "emparentar",
    "emparienten": "emparentar",
    "emparientes": "emparentar",
    "empariento": "emparentar",
    "emparra": "emparrar",
    "emparraba": "emparrar",
    "emparrabais": "emparrar",
    "emparraban": "emparrar",
    "emparrabas": "emparrar",
    "emparrad": "emparrar",
    "emparrada": "emparrar",
    "emparradas": "emparrar",
    "emparrado": "emparrar",
    "emparrados": "emparrar",
    "emparramos": "emparrar",
    "emparran": "emparrar",
    "emparrando": "emparrar",
    "emparrara": "emparrar",
    "emparrarais": "emparrar",
    "emparraran": "emparrar",
    "emparraras": "emparrar",
    "emparrare": "emparrar",
    "emparrareis": "emparrar",
    "emparraremos": "emparrar",
    "emparraren": "emparrar",
    "emparrares": "emparrar",
    "emparraron": "emparrar",
    "emparrará": "emparrar",
    "emparrarán": "emparrar",
    "emparrarás": "emparrar",
    "emparraré": "emparrar",
    "emparraréis": "emparrar",
    "emparraría": "emparrar",
    "emparraríais": "emparrar",
    "emparraríamos": "emparrar",
    "emparrarían": "emparrar",
    "emparrarías": "emparrar",
    "emparras": "emparrar",
    "emparrase": "emparrar",
    "emparraseis": "emparrar",
    "emparrasen": "emparrar",
    "emparrases": "emparrar",
    "emparraste": "emparrar",
    "emparrasteis": "emparrar",
    "emparre": "emparrar",
    "emparremos": "emparrar",
    "emparren": "emparrar",
    "emparres": "emparrar",
    "emparrilla": "emparrillar",
    "emparrillaba": "emparrillar",
    "emparrillabais": "emparrillar",
    "emparrillaban": "emparrillar",
    "emparrillabas": "emparrillar",
    "emparrillad": "emparrillar",
    "emparrillada": "emparrillar",
    "emparrilladas": "emparrillar",
    "emparrillado": "emparrillar",
    "emparrillados": "emparrillar",
    "emparrillamos": "emparrillar",
    "emparrillan": "emparrillar",
    "emparrillando": "emparrillar",
    "emparrillara": "emparrillar",
    "emparrillarais": "emparrillar",
    "emparrillaran": "emparrillar",
    "emparrillaras": "emparrillar",
    "emparrillare": "emparrillar",
    "emparrillareis": "emparrillar",
    "emparrillaremos": "emparrillar",
    "emparrillaren": "emparrillar",
    "emparrillares": "emparrillar",
    "emparrillaron": "emparrillar",
    "emparrillará": "emparrillar",
    "emparrillarán": "emparrillar",
    "emparrillarás": "emparrillar",
    "emparrillaré": "emparrillar",
    "emparrillaréis": "emparrillar",
    "emparrillaría": "emparrillar",
    "emparrillaríais": "emparrillar",
    "emparrillaríamos": "emparrillar",
    "emparrillarían": "emparrillar",
    "emparrillarías": "emparrillar",
    "emparrillas": "emparrillar",
    "emparrillase": "emparrillar",
    "emparrillaseis": "emparrillar",
    "emparrillasen": "emparrillar",
    "emparrillases": "emparrillar",
    "emparrillaste": "emparrillar",
    "emparrillasteis": "emparrillar",
    "emparrille": "emparrillar",
    "emparrillemos": "emparrillar",
    "emparrillen": "emparrillar",
    "emparrilles": "emparrillar",
    "emparrillo": "emparrillar",
    "emparrillábamos": "emparrillar",
    "emparrilláis": "emparrillar",
    "emparrilláramos": "emparrillar",
    "emparrilláremos": "emparrillar",
    "emparrillásemos": "emparrillar",
    "emparrillé": "emparrillar",
    "emparrilléis": "emparrillar",
    "emparrilló": "emparrillar",
    "emparro": "emparrar",
    "emparrábamos": "emparrar",
    "emparráis": "emparrar",
    "emparráramos": "emparrar",
    "emparráremos": "emparrar",
    "emparrásemos": "emparrar",
    "emparré": "emparrar",
    "emparréis": "emparrar",
    "emparró": "emparrar",
    "emparva": "emparvar",
    "emparvaba": "emparvar",
    "emparvabais": "emparvar",
    "emparvaban": "emparvar",
    "emparvabas": "emparvar",
    "emparvad": "emparvar",
    "emparvada": "emparvar",
    "emparvadas": "emparvar",
    "emparvado": "emparvar",
    "emparvados": "emparvar",
    "emparvamos": "emparvar",
    "emparvan": "emparvar",
    "emparvando": "emparvar",
    "emparvara": "emparvar",
    "emparvarais": "emparvar",
    "emparvaran": "emparvar",
    "emparvaras": "emparvar",
    "emparvare": "emparvar",
    "emparvareis": "emparvar",
    "emparvaremos": "emparvar",
    "emparvaren": "emparvar",
    "emparvares": "emparvar",
    "emparvaron": "emparvar",
    "emparvará": "emparvar",
    "emparvarán": "emparvar",
    "emparvarás": "emparvar",
    "emparvaré": "emparvar",
    "emparvaréis": "emparvar",
    "emparvaría": "emparvar",
    "emparvaríais": "emparvar",
    "emparvaríamos": "emparvar",
    "emparvarían": "emparvar",
    "emparvarías": "emparvar",
    "emparvas": "emparvar",
    "emparvase": "emparvar",
    "emparvaseis": "emparvar",
    "emparvasen": "emparvar",
    "emparvases": "emparvar",
    "emparvaste": "emparvar",
    "emparvasteis": "emparvar",
    "emparve": "emparvar",
    "emparvemos": "emparvar",
    "emparven": "emparvar",
    "emparves": "emparvar",
    "emparvo": "emparvar",
    "emparvábamos": "emparvar",
    "emparváis": "emparvar",
    "emparváramos": "emparvar",
    "emparváremos": "emparvar",
    "emparvásemos": "emparvar",
    "emparvé": "emparvar",
    "emparvéis": "emparvar",
    "emparvó": "emparvar",
    "empasta": "empastar",
    "empastaba": "empastar",
    "empastabais": "empastar",
    "empastaban": "empastar",
    "empastabas": "empastar",
    "empastad": "empastar",
    "empastada": "empastar",
    "empastadas": "empastar",
    "empastado": "empastar",
    "empastadora": "empastador",
    "empastadoras": "empastador",
    "empastadores": "empastador",
    "empastados": "empastar",
    "empastamos": "empastar",
    "empastan": "empastar",
    "empastando": "empastar",
    "empastara": "empastar",
    "empastarais": "empastar",
    "empastaran": "empastar",
    "empastaras": "empastar",
    "empastare": "empastar",
    "empastareis": "empastar",
    "empastaremos": "empastar",
    "empastaren": "empastar",
    "empastares": "empastar",
    "empastaron": "empastar",
    "empastará": "empastar",
    "empastarán": "empastar",
    "empastarás": "empastar",
    "empastaré": "empastar",
    "empastaréis": "empastar",
    "empastaría": "empastar",
    "empastaríais": "empastar",
    "empastaríamos": "empastar",
    "empastarían": "empastar",
    "empastarías": "empastar",
    "empastas": "empastar",
    "empastase": "empastar",
    "empastaseis": "empastar",
    "empastasen": "empastar",
    "empastases": "empastar",
    "empastaste": "empastar",
    "empastasteis": "empastar",
    "empaste": "empastar",
    "empastela": "empastelar",
    "empastelaba": "empastelar",
    "empastelabais": "empastelar",
    "empastelaban": "empastelar",
    "empastelabas": "empastelar",
    "empastelad": "empastelar",
    "empastelada": "empastelar",
    "empasteladas": "empastelar",
    "empastelado": "empastelar",
    "empastelados": "empastelar",
    "empastelamientos": "empastelamiento",
    "empastelamos": "empastelar",
    "empastelan": "empastelar",
    "empastelando": "empastelar",
    "empastelara": "empastelar",
    "empastelarais": "empastelar",
    "empastelaran": "empastelar",
    "empastelaras": "empastelar",
    "empastelare": "empastelar",
    "empastelareis": "empastelar",
    "empastelaremos": "empastelar",
    "empastelaren": "empastelar",
    "empastelares": "empastelar",
    "empastelaron": "empastelar",
    "empastelará": "empastelar",
    "empastelarán": "empastelar",
    "empastelarás": "empastelar",
    "empastelaré": "empastelar",
    "empastelaréis": "empastelar",
    "empastelaría": "empastelar",
    "empastelaríais": "empastelar",
    "empastelaríamos": "empastelar",
    "empastelarían": "empastelar",
    "empastelarías": "empastelar",
    "empastelas": "empastelar",
    "empastelase": "empastelar",
    "empastelaseis": "empastelar",
    "empastelasen": "empastelar",
    "empastelases": "empastelar",
    "empastelaste": "empastelar",
    "empastelasteis": "empastelar",
    "empastele": "empastelar",
    "empastelemos": "empastelar",
    "empastelen": "empastelar",
    "empasteles": "empastelar",
    "empastelo": "empastelar",
    "empastelábamos": "empastelar",
    "empasteláis": "empastelar",
    "empasteláramos": "empastelar",
    "empasteláremos": "empastelar",
    "empastelásemos": "empastelar",
    "empastelé": "empastelar",
    "empasteléis": "empastelar",
    "empasteló": "empastelar",
    "empastemos": "empastar",
    "empasten": "empastar",
    "empastes": "empaste",
    "empastilladoras": "empastilladora",
    "empasto": "empastar",
    "empastábamos": "empastar",
    "empastáis": "empastar",
    "empastáramos": "empastar",
    "empastáremos": "empastar",
    "empastásemos": "empastar",
    "empasté": "empastar",
    "empastéis": "empastar",
    "empastó": "empastar",
    "empata": "empatar",
    "empataba": "empatar",
    "empatabais": "empatar",
    "empataban": "empatar",
    "empatabas": "empatar",
    "empatad": "empatar",
    "empatada": "empatar",
    "empatadas": "empatar",
    "empataderas": "empatadera",
    "empatado": "empatar",
    "empatados": "empatar",
    "empatamos": "empatar",
    "empatan": "empatar",
    "empatando": "empatar",
    "empatara": "empatar",
    "empatarais": "empatar",
    "empataran": "empatar",
    "empataras": "empatar",
    "empatare": "empatar",
    "empatareis": "empatar",
    "empataremos": "empatar",
    "empataren": "empatar",
    "empatares": "empatar",
    "empataron": "empatar",
    "empatará": "empatar",
    "empatarán": "empatar",
    "empatarás": "empatar",
    "empataré": "empatar",
    "empataréis": "empatar",
    "empataría": "empatar",
    "empataríais": "empatar",
    "empataríamos": "empatar",
    "empatarían": "empatar",
    "empatarías": "empatar",
    "empatas": "empatar",
    "empatase": "empatar",
    "empataseis": "empatar",
    "empatasen": "empatar",
    "empatases": "empatar",
    "empataste": "empatar",
    "empatasteis": "empatar",
    "empate": "empatar",
    "empatemos": "empatar",
    "empaten": "empatar",
    "empates": "empate",
    "empato": "empatar",
    "empatábamos": "empatar",
    "empatáis": "empatar",
    "empatáramos": "empatar",
    "empatáremos": "empatar",
    "empatásemos": "empatar",
    "empaté": "empatar",
    "empatéis": "empatar",
    "empatías": "empatía",
    "empató": "empatar",
    "empavesa": "empavesar",
    "empavesaba": "empavesar",
    "empavesabais": "empavesar",
    "empavesaban": "empavesar",
    "empavesabas": "empavesar",
    "empavesad": "empavesar",
    "empavesada": "empavesar",
    "empavesadas": "empavesar",
    "empavesado": "empavesar",
    "empavesados": "empavesar",
    "empavesamos": "empavesar",
    "empavesan": "empavesar",
    "empavesando": "empavesar",
    "empavesara": "empavesar",
    "empavesarais": "empavesar",
    "empavesaran": "empavesar",
    "empavesaras": "empavesar",
    "empavesare": "empavesar",
    "empavesareis": "empavesar",
    "empavesaremos": "empavesar",
    "empavesaren": "empavesar",
    "empavesares": "empavesar",
    "empavesaron": "empavesar",
    "empavesará": "empavesar",
    "empavesarán": "empavesar",
    "empavesarás": "empavesar",
    "empavesaré": "empavesar",
    "empavesaréis": "empavesar",
    "empavesaría": "empavesar",
    "empavesaríais": "empavesar",
    "empavesaríamos": "empavesar",
    "empavesarían": "empavesar",
    "empavesarías": "empavesar",
    "empavesas": "empavesar",
    "empavesase": "empavesar",
    "empavesaseis": "empavesar",
    "empavesasen": "empavesar",
    "empavesases": "empavesar",
    "empavesaste": "empavesar",
    "empavesasteis": "empavesar",
    "empavese": "empavesar",
    "empavesemos": "empavesar",
    "empavesen": "empavesar",
    "empaveses": "empavesar",
    "empaveso": "empavesar",
    "empavesábamos": "empavesar",
    "empavesáis": "empavesar",
    "empavesáramos": "empavesar",
    "empavesáremos": "empavesar",
    "empavesásemos": "empavesar",
    "empavesé": "empavesar",
    "empaveséis": "empavesar",
    "empavesó": "empavesar",
    "empavona": "empavonar",
    "empavonaba": "empavonar",
    "empavonabais": "empavonar",
    "empavonaban": "empavonar",
    "empavonabas": "empavonar",
    "empavonad": "empavonar",
    "empavonada": "empavonar",
    "empavonadas": "empavonar",
    "empavonado": "empavonar",
    "empavonados": "empavonar",
    "empavonamientos": "empavonamiento",
    "empavonamos": "empavonar",
    "empavonan": "empavonar",
    "empavonando": "empavonar",
    "empavonara": "empavonar",
    "empavonarais": "empavonar",
    "empavonaran": "empavonar",
    "empavonaras": "empavonar",
    "empavonare": "empavonar",
    "empavonareis": "empavonar",
    "empavonaremos": "empavonar",
    "empavonaren": "empavonar",
    "empavonares": "empavonar",
    "empavonaron": "empavonar",
    "empavonará": "empavonar",
    "empavonarán": "empavonar",
    "empavonarás": "empavonar",
    "empavonaré": "empavonar",
    "empavonaréis": "empavonar",
    "empavonaría": "empavonar",
    "empavonaríais": "empavonar",
    "empavonaríamos": "empavonar",
    "empavonarían": "empavonar",
    "empavonarías": "empavonar",
    "empavonas": "empavonar",
    "empavonase": "empavonar",
    "empavonaseis": "empavonar",
    "empavonasen": "empavonar",
    "empavonases": "empavonar",
    "empavonaste": "empavonar",
    "empavonasteis": "empavonar",
    "empavone": "empavonar",
    "empavonemos": "empavonar",
    "empavonen": "empavonar",
    "empavones": "empavonar",
    "empavono": "empavonar",
    "empavonábamos": "empavonar",
    "empavonáis": "empavonar",
    "empavonáramos": "empavonar",
    "empavonáremos": "empavonar",
    "empavonásemos": "empavonar",
    "empavoné": "empavonar",
    "empavonéis": "empavonar",
    "empavonó": "empavonar",
    "empaña": "empañar",
    "empañaba": "empañar",
    "empañabais": "empañar",
    "empañaban": "empañar",
    "empañabas": "empañar",
    "empañad": "empañar",
    "empañada": "empañar",
    "empañadas": "empañar",
    "empañado": "empañar",
    "empañados": "empañar",
    "empañaduras": "empañadura",
    "empañamientos": "empañamiento",
    "empañamos": "empañar",
    "empañan": "empañar",
    "empañando": "empañar",
    "empañara": "empañar",
    "empañarais": "empañar",
    "empañaran": "empañar",
    "empañaras": "empañar",
    "empañare": "empañar",
    "empañareis": "empañar",
    "empañaremos": "empañar",
    "empañaren": "empañar",
    "empañares": "empañar",
    "empañaron": "empañar",
    "empañará": "empañar",
    "empañarán": "empañar",
    "empañarás": "empañar",
    "empañaré": "empañar",
    "empañaréis": "empañar",
    "empañaría": "empañar",
    "empañaríais": "empañar",
    "empañaríamos": "empañar",
    "empañarían": "empañar",
    "empañarías": "empañar",
    "empañas": "empañar",
    "empañase": "empañar",
    "empañaseis": "empañar",
    "empañasen": "empañar",
    "empañases": "empañar",
    "empañaste": "empañar",
    "empañasteis": "empañar",
    "empañe": "empañar",
    "empañemos": "empañar",
    "empañen": "empañar",
    "empañes": "empañar",
    "empañeta": "empañetar",
    "empañetaba": "empañetar",
    "empañetabais": "empañetar",
    "empañetaban": "empañetar",
    "empañetabas": "empañetar",
    "empañetad": "empañetar",
    "empañetada": "empañetar",
    "empañetadas": "empañetar",
    "empañetado": "empañetar",
    "empañetados": "empañetar",
    "empañetamos": "empañetar",
    "empañetan": "empañetar",
    "empañetando": "empañetar",
    "empañetara": "empañetar",
    "empañetarais": "empañetar",
    "empañetaran": "empañetar",
    "empañetaras": "empañetar",
    "empañetare": "empañetar",
    "empañetareis": "empañetar",
    "empañetaremos": "empañetar",
    "empañetaren": "empañetar",
    "empañetares": "empañetar",
    "empañetaron": "empañetar",
    "empañetará": "empañetar",
    "empañetarán": "empañetar",
    "empañetarás": "empañetar",
    "empañetaré": "empañetar",
    "empañetaréis": "empañetar",
    "empañetaría": "empañetar",
    "empañetaríais": "empañetar",
    "empañetaríamos": "empañetar",
    "empañetarían": "empañetar",
    "empañetarías": "empañetar",
    "empañetas": "empañetar",
    "empañetase": "empañetar",
    "empañetaseis": "empañetar",
    "empañetasen": "empañetar",
    "empañetases": "empañetar",
    "empañetaste": "empañetar",
    "empañetasteis": "empañetar",
    "empañete": "empañetar",
    "empañetemos": "empañetar",
    "empañeten": "empañetar",
    "empañetes": "empañetar",
    "empañeto": "empañetar",
    "empañetábamos": "empañetar",
    "empañetáis": "empañetar",
    "empañetáramos": "empañetar",
    "empañetáremos": "empañetar",
    "empañetásemos": "empañetar",
    "empañeté": "empañetar",
    "empañetéis": "empañetar",
    "empañetó": "empañetar",
    "empaño": "empañar",
    "empañábamos": "empañar",
    "empañáis": "empañar",
    "empañáramos": "empañar",
    "empañáremos": "empañar",
    "empañásemos": "empañar",
    "empañé": "empañar",
    "empañéis": "empañar",
    "empañó": "empañar",
    "empecatada": "empecatado",
    "empecatadas": "empecatado",
    "empecatados": "empecatado",
    "empece": "empecer",
    "empeced": "empecer",
    "empecedera": "empecedero",
    "empecederas": "empecedero",
    "empecederos": "empecedero",
    "empecemos": "empezar",
    "empecen": "empecer",
    "empeceremos": "empecer",
    "empecerá": "empecer",
    "empecerán": "empecer",
    "empecerás": "empecer",
    "empeceré": "empecer",
    "empeceréis": "empecer",
    "empecería": "empecer",
    "empeceríais": "empecer",
    "empeceríamos": "empecer",
    "empecerían": "empecer",
    "empecerías": "empecer",
    "empeces": "empecer",
    "empecibles": "empecible",
    "empecida": "empecer",
    "empecidas": "empecer",
    "empecido": "empecer",
    "empecidos": "empecer",
    "empeciendo": "empecer",
    "empeciera": "empecer",
    "empecierais": "empecer",
    "empecieran": "empecer",
    "empecieras": "empecer",
    "empeciere": "empecer",
    "empeciereis": "empecer",
    "empecieren": "empecer",
    "empecieres": "empecer",
    "empecieron": "empecer",
    "empeciese": "empecer",
    "empecieseis": "empecer",
    "empeciesen": "empecer",
    "empecieses": "empecer",
    "empecimientos": "empecimiento",
    "empecimos": "empecer",
    "empecina": "empecinar",
    "empecinaba": "empecinar",
    "empecinabais": "empecinar",
    "empecinaban": "empecinar",
    "empecinabas": "empecinar",
    "empecinad": "empecinar",
    "empecinada": "empecinar",
    "empecinadas": "empecinar",
    "empecinado": "empecinar",
    "empecinados": "empecinar",
    "empecinamientos": "empecinamiento",
    "empecinamos": "empecinar",
    "empecinan": "empecinar",
    "empecinando": "empecinar",
    "empecinara": "empecinar",
    "empecinarais": "empecinar",
    "empecinaran": "empecinar",
    "empecinaras": "empecinar",
    "empecinare": "empecinar",
    "empecinareis": "empecinar",
    "empecinaremos": "empecinar",
    "empecinaren": "empecinar",
    "empecinares": "empecinar",
    "empecinaron": "empecinar",
    "empecinará": "empecinar",
    "empecinarán": "empecinar",
    "empecinarás": "empecinar",
    "empecinaré": "empecinar",
    "empecinaréis": "empecinar",
    "empecinaría": "empecinar",
    "empecinaríais": "empecinar",
    "empecinaríamos": "empecinar",
    "empecinarían": "empecinar",
    "empecinarías": "empecinar",
    "empecinas": "empecinar",
    "empecinase": "empecinar",
    "empecinaseis": "empecinar",
    "empecinasen": "empecinar",
    "empecinases": "empecinar",
    "empecinaste": "empecinar",
    "empecinasteis": "empecinar",
    "empecine": "empecinar",
    "empecinemos": "empecinar",
    "empecinen": "empecinar",
    "empecines": "empecinar",
    "empecino": "empecinar",
    "empecinábamos": "empecinar",
    "empecináis": "empecinar",
    "empecináramos": "empecinar",
    "empecináremos": "empecinar",
    "empecinásemos": "empecinar",
    "empeciné": "empecinar",
    "empecinéis": "empecinar",
    "empecinó": "empecinar",
    "empeciste": "empecer",
    "empecisteis": "empecer",
    "empeciéramos": "empecer",
    "empeciéremos": "empecer",
    "empeciésemos": "empecer",
    "empeció": "empecer",
    "empecé": "empezar",
    "empecéis": "empezar",
    "empecí": "empecer",
    "empecía": "empecer",
    "empecíais": "empecer",
    "empecíamos": "empecer",
    "empecían": "empecer",
    "empecías": "empecer",
    "empederna": "empedernir",
    "empedernamos": "empedernir",
    "empedernan": "empedernir",
    "empedernas": "empedernir",
    "empederne": "empedernir",
    "empedernen": "empedernir",
    "empedernes": "empedernir",
    "empedernid": "empedernir",
    "empedernida": "empedernir",
    "empedernidas": "empedernir",
    "empedernido": "empedernir",
    "empedernidos": "empedernir",
    "empederniendo": "empedernir",
    "empederniera": "empedernir",
    "empedernierais": "empedernir",
    "empedernieran": "empedernir",
    "empedernieras": "empedernir",
    "empederniere": "empedernir",
    "empederniereis": "empedernir",
    "empedernieren": "empedernir",
    "empedernieres": "empedernir",
    "empedernieron": "empedernir",
    "empederniese": "empedernir",
    "empedernieseis": "empedernir",
    "empederniesen": "empedernir",
    "empedernieses": "empedernir",
    "empedernimos": "empedernir",
    "empederniremos": "empedernir",
    "empedernirá": "empedernir",
    "empedernirán": "empedernir",
    "empedernirás": "empedernir",
    "empederniré": "empedernir",
    "empederniréis": "empedernir",
    "empederniría": "empedernir",
    "empederniríais": "empedernir",
    "empederniríamos": "empedernir",
    "empedernirían": "empedernir",
    "empedernirías": "empedernir",
    "empederniste": "empedernir",
    "empedernisteis": "empedernir",
    "empederniéramos": "empedernir",
    "empederniéremos": "empedernir",
    "empederniésemos": "empedernir",
    "empedernió": "empedernir",
    "empederno": "empedernir",
    "empedernáis": "empedernir",
    "empederní": "empedernir",
    "empedernía": "empedernir",
    "empederníais": "empedernir",
    "empederníamos": "empedernir",
    "empedernían": "empedernir",
    "empedernías": "empedernir",
    "empedernís": "empedernir",
    "empedraba": "empedrar",
    "empedrabais": "empedrar",
    "empedraban": "empedrar",
    "empedrabas": "empedrar",
    "empedrad": "empedrar",
    "empedrada": "empedrar",
    "empedradas": "empedrar",
    "empedrado": "empedrar",
    "empedradora": "empedrador",
    "empedradoras": "empedrador",
    "empedradores": "empedrador",
    "empedrados": "empedrar",
    "empedramientos": "empedramiento",
    "empedramos": "empedrar",
    "empedrando": "empedrar",
    "empedrara": "empedrar",
    "empedrarais": "empedrar",
    "empedraran": "empedrar",
    "empedraras": "empedrar",
    "empedrare": "empedrar",
    "empedrareis": "empedrar",
    "empedraremos": "empedrar",
    "empedraren": "empedrar",
    "empedrares": "empedrar",
    "empedraron": "empedrar",
    "empedrará": "empedrar",
    "empedrarán": "empedrar",
    "empedrarás": "empedrar",
    "empedraré": "empedrar",
    "empedraréis": "empedrar",
    "empedraría": "empedrar",
    "empedraríais": "empedrar",
    "empedraríamos": "empedrar",
    "empedrarían": "empedrar",
    "empedrarías": "empedrar",
    "empedrase": "empedrar",
    "empedraseis": "empedrar",
    "empedrasen": "empedrar",
    "empedrases": "empedrar",
    "empedraste": "empedrar",
    "empedrasteis": "empedrar",
    "empedremos": "empedrar",
    "empedrábamos": "empedrar",
    "empedráis": "empedrar",
    "empedráramos": "empedrar",
    "empedráremos": "empedrar",
    "empedrásemos": "empedrar",
    "empedré": "empedrar",
    "empedréis": "empedrar",
    "empedró": "empedrar",
    "empega": "empegar",
    "empegaba": "empegar",
    "empegabais": "empegar",
    "empegaban": "empegar",
    "empegabas": "empegar",
    "empegad": "empegar",
    "empegada": "empegar",
    "empegadas": "empegar",
    "empegado": "empegar",
    "empegados": "empegar",
    "empegaduras": "empegadura",
    "empegamos": "empegar",
    "empegan": "empegar",
    "empegando": "empegar",
    "empegara": "empegar",
    "empegarais": "empegar",
    "empegaran": "empegar",
    "empegaras": "empegar",
    "empegare": "empegar",
    "empegareis": "empegar",
    "empegaremos": "empegar",
    "empegaren": "empegar",
    "empegares": "empegar",
    "empegaron": "empegar",
    "empegará": "empegar",
    "empegarán": "empegar",
    "empegarás": "empegar",
    "empegaré": "empegar",
    "empegaréis": "empegar",
    "empegaría": "empegar",
    "empegaríais": "empegar",
    "empegaríamos": "empegar",
    "empegarían": "empegar",
    "empegarías": "empegar",
    "empegas": "empegar",
    "empegase": "empegar",
    "empegaseis": "empegar",
    "empegasen": "empegar",
    "empegases": "empegar",
    "empegaste": "empegar",
    "empegasteis": "empegar",
    "empego": "empegar",
    "empegos": "empego",
    "empegue": "empegar",
    "empeguemos": "empegar",
    "empeguen": "empegar",
    "empegues": "empegar",
    "empegué": "empegar",
    "empeguéis": "empegar",
    "empegábamos": "empegar",
    "empegáis": "empegar",
    "empegáramos": "empegar",
    "empegáremos": "empegar",
    "empegásemos": "empegar",
    "empegó": "empegar",
    "empeines": "empeine",
    "empelecha": "empelechar",
    "empelechaba": "empelechar",
    "empelechabais": "empelechar",
    "empelechaban": "empelechar",
    "empelechabas": "empelechar",
    "empelechad": "empelechar",
    "empelechada": "empelechar",
    "empelechadas": "empelechar",
    "empelechado": "empelechar",
    "empelechados": "empelechar",
    "empelechamos": "empelechar",
    "empelechan": "empelechar",
    "empelechando": "empelechar",
    "empelechara": "empelechar",
    "empelecharais": "empelechar",
    "empelecharan": "empelechar",
    "empelecharas": "empelechar",
    "empelechare": "empelechar",
    "empelechareis": "empelechar",
    "empelecharemos": "empelechar",
    "empelecharen": "empelechar",
    "empelechares": "empelechar",
    "empelecharon": "empelechar",
    "empelechará": "empelechar",
    "empelecharán": "empelechar",
    "empelecharás": "empelechar",
    "empelecharé": "empelechar",
    "empelecharéis": "empelechar",
    "empelecharía": "empelechar",
    "empelecharíais": "empelechar",
    "empelecharíamos": "empelechar",
    "empelecharían": "empelechar",
    "empelecharías": "empelechar",
    "empelechas": "empelechar",
    "empelechase": "empelechar",
    "empelechaseis": "empelechar",
    "empelechasen": "empelechar",
    "empelechases": "empelechar",
    "empelechaste": "empelechar",
    "empelechasteis": "empelechar",
    "empeleche": "empelechar",
    "empelechemos": "empelechar",
    "empelechen": "empelechar",
    "empeleches": "empelechar",
    "empelecho": "empelechar",
    "empelechábamos": "empelechar",
    "empelecháis": "empelechar",
    "empelecháramos": "empelechar",
    "empelecháremos": "empelechar",
    "empelechásemos": "empelechar",
    "empeleché": "empelechar",
    "empelechéis": "empelechar",
    "empelechó": "empelechar",
    "empella": "empeller",
    "empellaba": "empellar",
    "empellabais": "empellar",
    "empellaban": "empellar",
    "empellabas": "empellar",
    "empellad": "empellar",
    "empellada": "empellar",
    "empelladas": "empellar",
    "empellado": "empellar",
    "empellados": "empellar",
    "empellamos": "empeller",
    "empellan": "empeller",
    "empellando": "empellar",
    "empellara": "empellar",
    "empellarais": "empellar",
    "empellaran": "empellar",
    "empellaras": "empellar",
    "empellare": "empellar",
    "empellareis": "empellar",
    "empellaremos": "empellar",
    "empellaren": "empellar",
    "empellares": "empellar",
    "empellaron": "empellar",
    "empellará": "empellar",
    "empellarán": "empellar",
    "empellarás": "empellar",
    "empellaré": "empellar",
    "empellaréis": "empellar",
    "empellaría": "empellar",
    "empellaríais": "empellar",
    "empellaríamos": "empellar",
    "empellarían": "empellar",
    "empellarías": "empellar",
    "empellas": "empeller",
    "empellase": "empellar",
    "empellaseis": "empellar",
    "empellasen": "empellar",
    "empellases": "empellar",
    "empellaste": "empellar",
    "empellasteis": "empellar",
    "empelle": "empeller",
    "empelled": "empeller",
    "empelleja": "empellejar",
    "empellejaba": "empellejar",
    "empellejabais": "empellejar",
    "empellejaban": "empellejar",
    "empellejabas": "empellejar",
    "empellejad": "empellejar",
    "empellejada": "empellejar",
    "empellejadas": "empellejar",
    "empellejado": "empellejar",
    "empellejados": "empellejar",
    "empellejamos": "empellejar",
    "empellejan": "empellejar",
    "empellejando": "empellejar",
    "empellejara": "empellejar",
    "empellejarais": "empellejar",
    "empellejaran": "empellejar",
    "empellejaras": "empellejar",
    "empellejare": "empellejar",
    "empellejareis": "empellejar",
    "empellejaremos": "empellejar",
    "empellejaren": "empellejar",
    "empellejares": "empellejar",
    "empellejaron": "empellejar",
    "empellejará": "empellejar",
    "empellejarán": "empellejar",
    "empellejarás": "empellejar",
    "empellejaré": "empellejar",
    "empellejaréis": "empellejar",
    "empellejaría": "empellejar",
    "empellejaríais": "empellejar",
    "empellejaríamos": "empellejar",
    "empellejarían": "empellejar",
    "empellejarías": "empellejar",
    "empellejas": "empellejar",
    "empellejase": "empellejar",
    "empellejaseis": "empellejar",
    "empellejasen": "empellejar",
    "empellejases": "empellejar",
    "empellejaste": "empellejar",
    "empellejasteis": "empellejar",
    "empelleje": "empellejar",
    "empellejemos": "empellejar",
    "empellejen": "empellejar",
    "empellejes": "empellejar",
    "empellejo": "empellejar",
    "empellejábamos": "empellejar",
    "empellejáis": "empellejar",
    "empellejáramos": "empellejar",
    "empellejáremos": "empellejar",
    "empellejásemos": "empellejar",
    "empellejé": "empellejar",
    "empellejéis": "empellejar",
    "empellejó": "empellejar",
    "empellemos": "empeller",
    "empellen": "empeller",
    "empellendo": "empeller",
    "empellera": "empeller",
    "empellerais": "empeller",
    "empelleran": "empeller",
    "empelleras": "empeller",
    "empellere": "empeller",
    "empellereis": "empeller",
    "empelleremos": "empeller",
    "empelleren": "empeller",
    "empelleres": "empeller",
    "empelleron": "empeller",
    "empellerá": "empeller",
    "empellerán": "empeller",
    "empellerás": "empeller",
    "empelleré": "empeller",
    "empelleréis": "empeller",
    "empellería": "empeller",
    "empelleríais": "empeller",
    "empelleríamos": "empeller",
    "empellerían": "empeller",
    "empellerías": "empeller",
    "empelles": "empeller",
    "empellese": "empeller",
    "empelleseis": "empeller",
    "empellesen": "empeller",
    "empelleses": "empeller",
    "empellida": "empeller",
    "empellidas": "empeller",
    "empellido": "empeller",
    "empellidos": "empeller",
    "empellimos": "empeller",
    "empelliste": "empeller",
    "empellisteis": "empeller",
    "empello": "empeller",
    "empellones": "empellón",
    "empellábamos": "empellar",
    "empelláis": "empeller",
    "empelláramos": "empellar",
    "empelláremos": "empellar",
    "empellásemos": "empellar",
    "empellé": "empellar",
    "empelléis": "empeller",
    "empelléramos": "empeller",
    "empelléremos": "empeller",
    "empellésemos": "empeller",
    "empellí": "empeller",
    "empellía": "empeller",
    "empellíais": "empeller",
    "empellíamos": "empeller",
    "empellían": "empeller",
    "empellías": "empeller",
    "empelló": "empeller",
    "empelota": "empelotar",
    "empelotaba": "empelotar",
    "empelotabais": "empelotar",
    "empelotaban": "empelotar",
    "empelotabas": "empelotar",
    "empelotad": "empelotar",
    "empelotada": "empelotar",
    "empelotadas": "empelotar",
    "empelotado": "empelotar",
    "empelotados": "empelotar",
    "empelotamos": "empelotar",
    "empelotan": "empelotar",
    "empelotando": "empelotar",
    "empelotara": "empelotar",
    "empelotarais": "empelotar",
    "empelotaran": "empelotar",
    "empelotaras": "empelotar",
    "empelotare": "empelotar",
    "empelotareis": "empelotar",
    "empelotaremos": "empelotar",
    "empelotaren": "empelotar",
    "empelotares": "empelotar",
    "empelotaron": "empelotar",
    "empelotará": "empelotar",
    "empelotarán": "empelotar",
    "empelotarás": "empelotar",
    "empelotaré": "empelotar",
    "empelotaréis": "empelotar",
    "empelotaría": "empelotar",
    "empelotaríais": "empelotar",
    "empelotaríamos": "empelotar",
    "empelotarían": "empelotar",
    "empelotarías": "empelotar",
    "empelotas": "empelotar",
    "empelotase": "empelotar",
    "empelotaseis": "empelotar",
    "empelotasen": "empelotar",
    "empelotases": "empelotar",
    "empelotaste": "empelotar",
    "empelotasteis": "empelotar",
    "empelote": "empelotar",
    "empelotemos": "empelotar",
    "empeloten": "empelotar",
    "empelotes": "empelotar",
    "empeloto": "empelotar",
    "empelotábamos": "empelotar",
    "empelotáis": "empelotar",
    "empelotáramos": "empelotar",
    "empelotáremos": "empelotar",
    "empelotásemos": "empelotar",
    "empeloté": "empelotar",
    "empelotéis": "empelotar",
    "empelotó": "empelotar",
    "empeltres": "empeltre",
    "empenacha": "empenachar",
    "empenachaba": "empenachar",
    "empenachabais": "empenachar",
    "empenachaban": "empenachar",
    "empenachabas": "empenachar",
    "empenachad": "empenachar",
    "empenachada": "empenachar",
    "empenachadas": "empenachar",
    "empenachado": "empenachar",
    "empenachados": "empenachar",
    "empenachamos": "empenachar",
    "empenachan": "empenachar",
    "empenachando": "empenachar",
    "empenachara": "empenachar",
    "empenacharais": "empenachar",
    "empenacharan": "empenachar",
    "empenacharas": "empenachar",
    "empenachare": "empenachar",
    "empenachareis": "empenachar",
    "empenacharemos": "empenachar",
    "empenacharen": "empenachar",
    "empenachares": "empenachar",
    "empenacharon": "empenachar",
    "empenachará": "empenachar",
    "empenacharán": "empenachar",
    "empenacharás": "empenachar",
    "empenacharé": "empenachar",
    "empenacharéis": "empenachar",
    "empenacharía": "empenachar",
    "empenacharíais": "empenachar",
    "empenacharíamos": "empenachar",
    "empenacharían": "empenachar",
    "empenacharías": "empenachar",
    "empenachas": "empenachar",
    "empenachase": "empenachar",
    "empenachaseis": "empenachar",
    "empenachasen": "empenachar",
    "empenachases": "empenachar",
    "empenachaste": "empenachar",
    "empenachasteis": "empenachar",
    "empenache": "empenachar",
    "empenachemos": "empenachar",
    "empenachen": "empenachar",
    "empenaches": "empenachar",
    "empenacho": "empenachar",
    "empenachábamos": "empenachar",
    "empenacháis": "empenachar",
    "empenacháramos": "empenachar",
    "empenacháremos": "empenachar",
    "empenachásemos": "empenachar",
    "empenaché": "empenachar",
    "empenachéis": "empenachar",
    "empenachó": "empenachar",
    "empenta": "empentar",
    "empentaba": "empentar",
    "empentabais": "empentar",
    "empentaban": "empentar",
    "empentabas": "empentar",
    "empentad": "empentar",
    "empentada": "empentar",
    "empentadas": "empentar",
    "empentado": "empentar",
    "empentados": "empentar",
    "empentamos": "empentar",
    "empentan": "empentar",
    "empentando": "empentar",
    "empentara": "empentar",
    "empentarais": "empentar",
    "empentaran": "empentar",
    "empentaras": "empentar",
    "empentare": "empentar",
    "empentareis": "empentar",
    "empentaremos": "empentar",
    "empentaren": "empentar",
    "empentares": "empentar",
    "empentaron": "empentar",
    "empentará": "empentar",
    "empentarán": "empentar",
    "empentarás": "empentar",
    "empentaré": "empentar",
    "empentaréis": "empentar",
    "empentaría": "empentar",
    "empentaríais": "empentar",
    "empentaríamos": "empentar",
    "empentarían": "empentar",
    "empentarías": "empentar",
    "empentas": "empentar",
    "empentase": "empentar",
    "empentaseis": "empentar",
    "empentasen": "empentar",
    "empentases": "empentar",
    "empentaste": "empentar",
    "empentasteis": "empentar",
    "empente": "empentar",
    "empentemos": "empentar",
    "empenten": "empentar",
    "empentes": "empentar",
    "empento": "empentar",
    "empentábamos": "empentar",
    "empentáis": "empentar",
    "empentáramos": "empentar",
    "empentáremos": "empentar",
    "empentásemos": "empentar",
    "empenté": "empentar",
    "empentéis": "empentar",
    "empentó": "empentar",
    "empeora": "empeorar",
    "empeoraba": "empeorar",
    "empeorabais": "empeorar",
    "empeoraban": "empeorar",
    "empeorabas": "empeorar",
    "empeorad": "empeorar",
    "empeorada": "empeorar",
    "empeoradas": "empeorar",
    "empeorado": "empeorar",
    "empeorados": "empeorar",
    "empeoramientos": "empeoramiento",
    "empeoramos": "empeorar",
    "empeoran": "empeorar",
    "empeorando": "empeorar",
    "empeorara": "empeorar",
    "empeorarais": "empeorar",
    "empeoraran": "empeorar",
    "empeoraras": "empeorar",
    "empeorare": "empeorar",
    "empeorareis": "empeorar",
    "empeoraremos": "empeorar",
    "empeoraren": "empeorar",
    "empeorares": "empeorar",
    "empeoraron": "empeorar",
    "empeorará": "empeorar",
    "empeorarán": "empeorar",
    "empeorarás": "empeorar",
    "empeoraré": "empeorar",
    "empeoraréis": "empeorar",
    "empeoraría": "empeorar",
    "empeoraríais": "empeorar",
    "empeoraríamos": "empeorar",
    "empeorarían": "empeorar",
    "empeorarías": "empeorar",
    "empeoras": "empeorar",
    "empeorase": "empeorar",
    "empeoraseis": "empeorar",
    "empeorasen": "empeorar",
    "empeorases": "empeorar",
    "empeoraste": "empeorar",
    "empeorasteis": "empeorar",
    "empeore": "empeorar",
    "empeoremos": "empeorar",
    "empeoren": "empeorar",
    "empeores": "empeorar",
    "empeoro": "empeorar",
    "empeorábamos": "empeorar",
    "empeoráis": "empeorar",
    "empeoráramos": "empeorar",
    "empeoráremos": "empeorar",
    "empeorásemos": "empeorar",
    "empeoré": "empeorar",
    "empeoréis": "empeorar",
    "empeoró": "empeorar",
    "empequeñece": "empequeñecer",
    "empequeñeced": "empequeñecer",
    "empequeñecemos": "empequeñecer",
    "empequeñecen": "empequeñecer",
    "empequeñeceremos": "empequeñecer",
    "empequeñecerá": "empequeñecer",
    "empequeñecerán": "empequeñecer",
    "empequeñecerás": "empequeñecer",
    "empequeñeceré": "empequeñecer",
    "empequeñeceréis": "empequeñecer",
    "empequeñecería": "empequeñecer",
    "empequeñeceríais": "empequeñecer",
    "empequeñeceríamos": "empequeñecer",
    "empequeñecerían": "empequeñecer",
    "empequeñecerías": "empequeñecer",
    "empequeñeces": "empequeñecer",
    "empequeñecida": "empequeñecer",
    "empequeñecidas": "empequeñecer",
    "empequeñecido": "empequeñecer",
    "empequeñecidos": "empequeñecer",
    "empequeñeciendo": "empequeñecer",
    "empequeñeciera": "empequeñecer",
    "empequeñecierais": "empequeñecer",
    "empequeñecieran": "empequeñecer",
    "empequeñecieras": "empequeñecer",
    "empequeñeciere": "empequeñecer",
    "empequeñeciereis": "empequeñecer",
    "empequeñecieren": "empequeñecer",
    "empequeñecieres": "empequeñecer",
    "empequeñecieron": "empequeñecer",
    "empequeñeciese": "empequeñecer",
    "empequeñecieseis": "empequeñecer",
    "empequeñeciesen": "empequeñecer",
    "empequeñecieses": "empequeñecer",
    "empequeñecimientos": "empequeñecimiento",
    "empequeñecimos": "empequeñecer",
    "empequeñeciste": "empequeñecer",
    "empequeñecisteis": "empequeñecer",
    "empequeñeciéramos": "empequeñecer",
    "empequeñeciéremos": "empequeñecer",
    "empequeñeciésemos": "empequeñecer",
    "empequeñeció": "empequeñecer",
    "empequeñecéis": "empequeñecer",
    "empequeñecí": "empequeñecer",
    "empequeñecía": "empequeñecer",
    "empequeñecíais": "empequeñecer",
    "empequeñecíamos": "empequeñecer",
    "empequeñecían": "empequeñecer",
    "empequeñecías": "empequeñecer",
    "empequeñezca": "empequeñecer",
    "empequeñezcamos": "empequeñecer",
    "empequeñezcan": "empequeñecer",
    "empequeñezcas": "empequeñecer",
    "empequeñezco": "empequeñecer",
    "empequeñezcáis": "empequeñecer",
    "emperadores": "emperador",
    "emperatrices": "emperatriz",
    "empercha": "emperchar",
    "emperchaba": "emperchar",
    "emperchabais": "emperchar",
    "emperchaban": "emperchar",
    "emperchabas": "emperchar",
    "emperchad": "emperchar",
    "emperchada": "emperchar",
    "emperchadas": "emperchar",
    "emperchado": "emperchar",
    "emperchados": "emperchar",
    "emperchamos": "emperchar",
    "emperchan": "emperchar",
    "emperchando": "emperchar",
    "emperchara": "emperchar",
    "empercharais": "emperchar",
    "empercharan": "emperchar",
    "empercharas": "emperchar",
    "emperchare": "emperchar",
    "emperchareis": "emperchar",
    "empercharemos": "emperchar",
    "empercharen": "emperchar",
    "emperchares": "emperchar",
    "empercharon": "emperchar",
    "emperchará": "emperchar",
    "empercharán": "emperchar",
    "empercharás": "emperchar",
    "empercharé": "emperchar",
    "empercharéis": "emperchar",
    "empercharía": "emperchar",
    "empercharíais": "emperchar",
    "empercharíamos": "emperchar",
    "empercharían": "emperchar",
    "empercharías": "emperchar",
    "emperchas": "emperchar",
    "emperchase": "emperchar",
    "emperchaseis": "emperchar",
    "emperchasen": "emperchar",
    "emperchases": "emperchar",
    "emperchaste": "emperchar",
    "emperchasteis": "emperchar",
    "emperche": "emperchar",
    "emperchemos": "emperchar",
    "emperchen": "emperchar",
    "emperches": "emperchar",
    "empercho": "emperchar",
    "emperchábamos": "emperchar",
    "empercháis": "emperchar",
    "empercháramos": "emperchar",
    "empercháremos": "emperchar",
    "emperchásemos": "emperchar",
    "emperché": "emperchar",
    "emperchéis": "emperchar",
    "emperchó": "emperchar",
    "emperdiga": "emperdigar",
    "emperdigaba": "emperdigar",
    "emperdigabais": "emperdigar",
    "emperdigaban": "emperdigar",
    "emperdigabas": "emperdigar",
    "emperdigad": "emperdigar",
    "emperdigada": "emperdigar",
    "emperdigadas": "emperdigar",
    "emperdigado": "emperdigar",
    "emperdigados": "emperdigar",
    "emperdigamos": "emperdigar",
    "emperdigan": "emperdigar",
    "emperdigando": "emperdigar",
    "emperdigara": "emperdigar",
    "emperdigarais": "emperdigar",
    "emperdigaran": "emperdigar",
    "emperdigaras": "emperdigar",
    "emperdigare": "emperdigar",
    "emperdigareis": "emperdigar",
    "emperdigaremos": "emperdigar",
    "emperdigaren": "emperdigar",
    "emperdigares": "emperdigar",
    "emperdigaron": "emperdigar",
    "emperdigará": "emperdigar",
    "emperdigarán": "emperdigar",
    "emperdigarás": "emperdigar",
    "emperdigaré": "emperdigar",
    "emperdigaréis": "emperdigar",
    "emperdigaría": "emperdigar",
    "emperdigaríais": "emperdigar",
    "emperdigaríamos": "emperdigar",
    "emperdigarían": "emperdigar",
    "emperdigarías": "emperdigar",
    "emperdigas": "emperdigar",
    "emperdigase": "emperdigar",
    "emperdigaseis": "emperdigar",
    "emperdigasen": "emperdigar",
    "emperdigases": "emperdigar",
    "emperdigaste": "emperdigar",
    "emperdigasteis": "emperdigar",
    "emperdigo": "emperdigar",
    "emperdigue": "emperdigar",
    "emperdiguemos": "emperdigar",
    "emperdiguen": "emperdigar",
    "emperdigues": "emperdigar",
    "emperdigué": "emperdigar",
    "emperdiguéis": "emperdigar",
    "emperdigábamos": "emperdigar",
    "emperdigáis": "emperdigar",
    "emperdigáramos": "emperdigar",
    "emperdigáremos": "emperdigar",
    "emperdigásemos": "emperdigar",
    "emperdigó": "emperdigar",
    "emperece": "emperezar",
    "emperecemos": "emperezar",
    "emperecen": "emperezar",
    "empereces": "emperezar",
    "emperecé": "emperezar",
    "emperecéis": "emperezar",
    "emperejila": "emperejilar",
    "emperejilaba": "emperejilar",
    "emperejilabais": "emperejilar",
    "emperejilaban": "emperejilar",
    "emperejilabas": "emperejilar",
    "emperejilad": "emperejilar",
    "emperejilada": "emperejilar",
    "emperejiladas": "emperejilar",
    "emperejilado": "emperejilar",
    "emperejilados": "emperejilar",
    "emperejilamos": "emperejilar",
    "emperejilan": "emperejilar",
    "emperejilando": "emperejilar",
    "emperejilara": "emperejilar",
    "emperejilarais": "emperejilar",
    "emperejilaran": "emperejilar",
    "emperejilaras": "emperejilar",
    "emperejilare": "emperejilar",
    "emperejilareis": "emperejilar",
    "emperejilaremos": "emperejilar",
    "emperejilaren": "emperejilar",
    "emperejilares": "emperejilar",
    "emperejilaron": "emperejilar",
    "emperejilará": "emperejilar",
    "emperejilarán": "emperejilar",
    "emperejilarás": "emperejilar",
    "emperejilaré": "emperejilar",
    "emperejilaréis": "emperejilar",
    "emperejilaría": "emperejilar",
    "emperejilaríais": "emperejilar",
    "emperejilaríamos": "emperejilar",
    "emperejilarían": "emperejilar",
    "emperejilarías": "emperejilar",
    "emperejilas": "emperejilar",
    "emperejilase": "emperejilar",
    "emperejilaseis": "emperejilar",
    "emperejilasen": "emperejilar",
    "emperejilases": "emperejilar",
    "emperejilaste": "emperejilar",
    "emperejilasteis": "emperejilar",
    "emperejile": "emperejilar",
    "emperejilemos": "emperejilar",
    "emperejilen": "emperejilar",
    "emperejiles": "emperejilar",
    "emperejilo": "emperejilar",
    "emperejilábamos": "emperejilar",
    "emperejiláis": "emperejilar",
    "emperejiláramos": "emperejilar",
    "emperejiláremos": "emperejilar",
    "emperejilásemos": "emperejilar",
    "emperejilé": "emperejilar",
    "emperejiléis": "emperejilar",
    "emperejiló": "emperejilar",
    "empereza": "emperezar",
    "emperezaba": "emperezar",
    "emperezabais": "emperezar",
    "emperezaban": "emperezar",
    "emperezabas": "emperezar",
    "emperezad": "emperezar",
    "emperezada": "emperezar",
    "emperezadas": "emperezar",
    "emperezado": "emperezar",
    "emperezados": "emperezar",
    "emperezamos": "emperezar",
    "emperezan": "emperezar",
    "emperezando": "emperezar",
    "emperezara": "emperezar",
    "emperezarais": "emperezar",
    "emperezaran": "emperezar",
    "emperezaras": "emperezar",
    "emperezare": "emperezar",
    "emperezareis": "emperezar",
    "emperezaremos": "emperezar",
    "emperezaren": "emperezar",
    "emperezares": "emperezar",
    "emperezaron": "emperezar",
    "emperezará": "emperezar",
    "emperezarán": "emperezar",
    "emperezarás": "emperezar",
    "emperezaré": "emperezar",
    "emperezaréis": "emperezar",
    "emperezaría": "emperezar",
    "emperezaríais": "emperezar",
    "emperezaríamos": "emperezar",
    "emperezarían": "emperezar",
    "emperezarías": "emperezar",
    "emperezas": "emperezar",
    "emperezase": "emperezar",
    "emperezaseis": "emperezar",
    "emperezasen": "emperezar",
    "emperezases": "emperezar",
    "emperezaste": "emperezar",
    "emperezasteis": "emperezar",
    "emperezo": "emperezar",
    "emperezábamos": "emperezar",
    "emperezáis": "emperezar",
    "emperezáramos": "emperezar",
    "emperezáremos": "emperezar",
    "emperezásemos": "emperezar",
    "emperezó": "emperezar",
    "empergamina": "empergaminar",
    "empergaminaba": "empergaminar",
    "empergaminabais": "empergaminar",
    "empergaminaban": "empergaminar",
    "empergaminabas": "empergaminar",
    "empergaminad": "empergaminar",
    "empergaminada": "empergaminar",
    "empergaminadas": "empergaminar",
    "empergaminado": "empergaminar",
    "empergaminados": "empergaminar",
    "empergaminamos": "empergaminar",
    "empergaminan": "empergaminar",
    "empergaminando": "empergaminar",
    "empergaminara": "empergaminar",
    "empergaminarais": "empergaminar",
    "empergaminaran": "empergaminar",
    "empergaminaras": "empergaminar",
    "empergaminare": "empergaminar",
    "empergaminareis": "empergaminar",
    "empergaminaremos": "empergaminar",
    "empergaminaren": "empergaminar",
    "empergaminares": "empergaminar",
    "empergaminaron": "empergaminar",
    "empergaminará": "empergaminar",
    "empergaminarán": "empergaminar",
    "empergaminarás": "empergaminar",
    "empergaminaré": "empergaminar",
    "empergaminaréis": "empergaminar",
    "empergaminaría": "empergaminar",
    "empergaminaríais": "empergaminar",
    "empergaminaríamos": "empergaminar",
    "empergaminarían": "empergaminar",
    "empergaminarías": "empergaminar",
    "empergaminas": "empergaminar",
    "empergaminase": "empergaminar",
    "empergaminaseis": "empergaminar",
    "empergaminasen": "empergaminar",
    "empergaminases": "empergaminar",
    "empergaminaste": "empergaminar",
    "empergaminasteis": "empergaminar",
    "empergamine": "empergaminar",
    "empergaminemos": "empergaminar",
    "empergaminen": "empergaminar",
    "empergamines": "empergaminar",
    "empergamino": "empergaminar",
    "empergaminábamos": "empergaminar",
    "empergamináis": "empergaminar",
    "empergamináramos": "empergaminar",
    "empergamináremos": "empergaminar",
    "empergaminásemos": "empergaminar",
    "empergaminé": "empergaminar",
    "empergaminéis": "empergaminar",
    "empergaminó": "empergaminar",
    "emperifolla": "emperifollar",
    "emperifollaba": "emperifollar",
    "emperifollabais": "emperifollar",
    "emperifollaban": "emperifollar",
    "emperifollabas": "emperifollar",
    "emperifollad": "emperifollar",
    "emperifollada": "emperifollar",
    "emperifolladas": "emperifollar",
    "emperifollado": "emperifollar",
    "emperifollados": "emperifollar",
    "emperifollamientos": "emperifollamiento",
    "emperifollamos": "emperifollar",
    "emperifollan": "emperifollar",
    "emperifollando": "emperifollar",
    "emperifollara": "emperifollar",
    "emperifollarais": "emperifollar",
    "emperifollaran": "emperifollar",
    "emperifollaras": "emperifollar",
    "emperifollare": "emperifollar",
    "emperifollareis": "emperifollar",
    "emperifollaremos": "emperifollar",
    "emperifollaren": "emperifollar",
    "emperifollares": "emperifollar",
    "emperifollaron": "emperifollar",
    "emperifollará": "emperifollar",
    "emperifollarán": "emperifollar",
    "emperifollarás": "emperifollar",
    "emperifollaré": "emperifollar",
    "emperifollaréis": "emperifollar",
    "emperifollaría": "emperifollar",
    "emperifollaríais": "emperifollar",
    "emperifollaríamos": "emperifollar",
    "emperifollarían": "emperifollar",
    "emperifollarías": "emperifollar",
    "emperifollas": "emperifollar",
    "emperifollase": "emperifollar",
    "emperifollaseis": "emperifollar",
    "emperifollasen": "emperifollar",
    "emperifollases": "emperifollar",
    "emperifollaste": "emperifollar",
    "emperifollasteis": "emperifollar",
    "emperifolle": "emperifollar",
    "emperifollemos": "emperifollar",
    "emperifollen": "emperifollar",
    "emperifolles": "emperifollar",
    "emperifollo": "emperifollar",
    "emperifollábamos": "emperifollar",
    "emperifolláis": "emperifollar",
    "emperifolláramos": "emperifollar",
    "emperifolláremos": "emperifollar",
    "emperifollásemos": "emperifollar",
    "emperifollé": "emperifollar",
    "emperifolléis": "emperifollar",
    "emperifolló": "emperifollar",
    "emperna": "empernar",
    "empernaba": "empernar",
    "empernabais": "empernar",
    "empernaban": "empernar",
    "empernabas": "empernar",
    "empernad": "empernar",
    "empernada": "empernar",
    "empernadas": "empernar",
    "empernado": "empernar",
    "empernados": "empernar",
    "empernamientos": "empernamiento",
    "empernamos": "empernar",
    "empernan": "empernar",
    "empernando": "empernar",
    "empernara": "empernar",
    "empernarais": "empernar",
    "empernaran": "empernar",
    "empernaras": "empernar",
    "empernare": "empernar",
    "empernareis": "empernar",
    "empernaremos": "empernar",
    "empernaren": "empernar",
    "empernares": "empernar",
    "empernaron": "empernar",
    "empernará": "empernar",
    "empernarán": "empernar",
    "empernarás": "empernar",
    "empernaré": "empernar",
    "empernaréis": "empernar",
    "empernaría": "empernar",
    "empernaríais": "empernar",
    "empernaríamos": "empernar",
    "empernarían": "empernar",
    "empernarías": "empernar",
    "empernas": "empernar",
    "empernase": "empernar",
    "empernaseis": "empernar",
    "empernasen": "empernar",
    "empernases": "empernar",
    "empernaste": "empernar",
    "empernasteis": "empernar",
    "emperne": "empernar",
    "empernemos": "empernar",
    "empernen": "empernar",
    "empernes": "empernar",
    "emperno": "empernar",
    "empernábamos": "empernar",
    "empernáis": "empernar",
    "empernáramos": "empernar",
    "empernáremos": "empernar",
    "empernásemos": "empernar",
    "emperné": "empernar",
    "empernéis": "empernar",
    "empernó": "empernar",
    "emperra": "emperrar",
    "emperraba": "emperrar",
    "emperrabais": "emperrar",
    "emperraban": "emperrar",
    "emperrabas": "emperrar",
    "emperrad": "emperrar",
    "emperrada": "emperrar",
    "emperradas": "emperrar",
    "emperrado": "emperrar",
    "emperrados": "emperrar",
    "emperramientos": "emperramiento",
    "emperramos": "emperrar",
    "emperran": "emperrar",
    "emperrando": "emperrar",
    "emperrara": "emperrar",
    "emperrarais": "emperrar",
    "emperraran": "emperrar",
    "emperraras": "emperrar",
    "emperrare": "emperrar",
    "emperrareis": "emperrar",
    "emperraremos": "emperrar",
    "emperraren": "emperrar",
    "emperrares": "emperrar",
    "emperraron": "emperrar",
    "emperrará": "emperrar",
    "emperrarán": "emperrar",
    "emperrarás": "emperrar",
    "emperraré": "emperrar",
    "emperraréis": "emperrar",
    "emperraría": "emperrar",
    "emperraríais": "emperrar",
    "emperraríamos": "emperrar",
    "emperrarían": "emperrar",
    "emperrarías": "emperrar",
    "emperras": "emperrar",
    "emperrase": "emperrar",
    "emperraseis": "emperrar",
    "emperrasen": "emperrar",
    "emperrases": "emperrar",
    "emperraste": "emperrar",
    "emperrasteis": "emperrar",
    "emperre": "emperrar",
    "emperremos": "emperrar",
    "emperren": "emperrar",
    "emperres": "emperrar",
    "emperro": "emperrar",
    "emperrábamos": "emperrar",
    "emperráis": "emperrar",
    "emperráramos": "emperrar",
    "emperráremos": "emperrar",
    "emperrásemos": "emperrar",
    "emperré": "emperrar",
    "emperréis": "emperrar",
    "emperró": "emperrar",
    "empesgues": "empesgue",
    "empezaba": "empezar",
    "empezabais": "empezar",
    "empezaban": "empezar",
    "empezabas": "empezar",
    "empezad": "empezar",
    "empezada": "empezar",
    "empezadas": "empezar",
    "empezado": "empezar",
    "empezados": "empezar",
    "empezamos": "empezar",
    "empezando": "empezar",
    "empezara": "empezar",
    "empezarais": "empezar",
    "empezaran": "empezar",
    "empezaras": "empezar",
    "empezare": "empezar",
    "empezareis": "empezar",
    "empezaremos": "empezar",
    "empezaren": "empezar",
    "empezares": "empezar",
    "empezaron": "empezar",
    "empezará": "empezar",
    "empezarán": "empezar",
    "empezarás": "empezar",
    "empezaré": "empezar",
    "empezaréis": "empezar",
    "empezaría": "empezar",
    "empezaríais": "empezar",
    "empezaríamos": "empezar",
    "empezarían": "empezar",
    "empezarías": "empezar",
    "empezase": "empezar",
    "empezaseis": "empezar",
    "empezasen": "empezar",
    "empezases": "empezar",
    "empezaste": "empezar",
    "empezasteis": "empezar",
    "empezca": "empecer",
    "empezcamos": "empecer",
    "empezcan": "empecer",
    "empezcas": "empecer",
    "empezco": "empecer",
    "empezcáis": "empecer",
    "empezábamos": "empezar",
    "empezáis": "empezar",
    "empezáramos": "empezar",
    "empezáremos": "empezar",
    "empezásemos": "empezar",
    "empezó": "empezar",
    "empeña": "empeñar",
    "empeñaba": "empeñar",
    "empeñabais": "empeñar",
    "empeñaban": "empeñar",
    "empeñabas": "empeñar",
    "empeñad": "empeñar",
    "empeñada": "empeñar",
    "empeñadas": "empeñar",
    "empeñado": "empeñar",
    "empeñados": "empeñar",
    "empeñamos": "empeñar",
    "empeñan": "empeñar",
    "empeñando": "empeñar",
    "empeñara": "empeñar",
    "empeñarais": "empeñar",
    "empeñaran": "empeñar",
    "empeñaras": "empeñar",
    "empeñare": "empeñar",
    "empeñareis": "empeñar",
    "empeñaremos": "empeñar",
    "empeñaren": "empeñar",
    "empeñares": "empeñar",
    "empeñaron": "empeñar",
    "empeñará": "empeñar",
    "empeñarán": "empeñar",
    "empeñarás": "empeñar",
    "empeñaré": "empeñar",
    "empeñaréis": "empeñar",
    "empeñaría": "empeñar",
    "empeñaríais": "empeñar",
    "empeñaríamos": "empeñar",
    "empeñarían": "empeñar",
    "empeñarías": "empeñar",
    "empeñas": "empeñar",
    "empeñase": "empeñar",
    "empeñaseis": "empeñar",
    "empeñasen": "empeñar",
    "empeñases": "empeñar",
    "empeñaste": "empeñar",
    "empeñasteis": "empeñar",
    "empeñe": "empeñar",
    "empeñemos": "empeñar",
    "empeñen": "empeñar",
    "empeñes": "empeñar",
    "empeño": "empeñar",
    "empeños": "empeño",
    "empeñábamos": "empeñar",
    "empeñáis": "empeñar",
    "empeñáramos": "empeñar",
    "empeñáremos": "empeñar",
    "empeñásemos": "empeñar",
    "empeñé": "empeñar",
    "empeñéis": "empeñar",
    "empeñó": "empeñar",
    "empica": "empicar",
    "empicaba": "empicar",
    "empicabais": "empicar",
    "empicaban": "empicar",
    "empicabas": "empicar",
    "empicad": "empicar",
    "empicada": "empicar",
    "empicadas": "empicar",
    "empicado": "empicar",
    "empicados": "empicar",
    "empicamos": "empicar",
    "empican": "empicar",
    "empicando": "empicar",
    "empicara": "empicar",
    "empicarais": "empicar",
    "empicaran": "empicar",
    "empicaras": "empicar",
    "empicare": "empicar",
    "empicareis": "empicar",
    "empicaremos": "empicar",
    "empicaren": "empicar",
    "empicares": "empicar",
    "empicaron": "empicar",
    "empicará": "empicar",
    "empicarán": "empicar",
    "empicarás": "empicar",
    "empicaré": "empicar",
    "empicaréis": "empicar",
    "empicaría": "empicar",
    "empicaríais": "empicar",
    "empicaríamos": "empicar",
    "empicarían": "empicar",
    "empicarías": "empicar",
    "empicas": "empicar",
    "empicase": "empicar",
    "empicaseis": "empicar",
    "empicasen": "empicar",
    "empicases": "empicar",
    "empicaste": "empicar",
    "empicasteis": "empicar",
    "empico": "empicar",
    "empicota": "empicotar",
    "empicotaba": "empicotar",
    "empicotabais": "empicotar",
    "empicotaban": "empicotar",
    "empicotabas": "empicotar",
    "empicotad": "empicotar",
    "empicotada": "empicotar",
    "empicotadas": "empicotar",
    "empicotado": "empicotar",
    "empicotados": "empicotar",
    "empicotaduras": "empicotadura",
    "empicotamos": "empicotar",
    "empicotan": "empicotar",
    "empicotando": "empicotar",
    "empicotara": "empicotar",
    "empicotarais": "empicotar",
    "empicotaran": "empicotar",
    "empicotaras": "empicotar",
    "empicotare": "empicotar",
    "empicotareis": "empicotar",
    "empicotaremos": "empicotar",
    "empicotaren": "empicotar",
    "empicotares": "empicotar",
    "empicotaron": "empicotar",
    "empicotará": "empicotar",
    "empicotarán": "empicotar",
    "empicotarás": "empicotar",
    "empicotaré": "empicotar",
    "empicotaréis": "empicotar",
    "empicotaría": "empicotar",
    "empicotaríais": "empicotar",
    "empicotaríamos": "empicotar",
    "empicotarían": "empicotar",
    "empicotarías": "empicotar",
    "empicotas": "empicotar",
    "empicotase": "empicotar",
    "empicotaseis": "empicotar",
    "empicotasen": "empicotar",
    "empicotases": "empicotar",
    "empicotaste": "empicotar",
    "empicotasteis": "empicotar",
    "empicote": "empicotar",
    "empicotemos": "empicotar",
    "empicoten": "empicotar",
    "empicotes": "empicotar",
    "empicoto": "empicotar",
    "empicotábamos": "empicotar",
    "empicotáis": "empicotar",
    "empicotáramos": "empicotar",
    "empicotáremos": "empicotar",
    "empicotásemos": "empicotar",
    "empicoté": "empicotar",
    "empicotéis": "empicotar",
    "empicotó": "empicotar",
    "empicábamos": "empicar",
    "empicáis": "empicar",
    "empicáramos": "empicar",
    "empicáremos": "empicar",
    "empicásemos": "empicar",
    "empicó": "empicar",
    "empiece": "empezar",
    "empiecen": "empezar",
    "empieces": "empiece",
    "empiedra": "empedrar",
    "empiedran": "empedrar",
    "empiedras": "empedrar",
    "empiedre": "empedrar",
    "empiedren": "empedrar",
    "empiedres": "empedrar",
    "empiedro": "empedrar",
    "empiemas": "empiema",
    "empieza": "empezar",
    "empiezan": "empezar",
    "empiezas": "empezar",
    "empiezo": "empezar",
    "empila": "empilar",
    "empilaba": "empilar",
    "empilabais": "empilar",
    "empilaban": "empilar",
    "empilabas": "empilar",
    "empilad": "empilar",
    "empilada": "empilar",
    "empiladas": "empilar",
    "empilado": "empilar",
    "empilados": "empilar",
    "empilamos": "empilar",
    "empilan": "empilar",
    "empilando": "empilar",
    "empilara": "empilar",
    "empilarais": "empilar",
    "empilaran": "empilar",
    "empilaras": "empilar",
    "empilare": "empilar",
    "empilareis": "empilar",
    "empilaremos": "empilar",
    "empilaren": "empilar",
    "empilares": "empilar",
    "empilaron": "empilar",
    "empilará": "empilar",
    "empilarán": "empilar",
    "empilarás": "empilar",
    "empilaré": "empilar",
    "empilaréis": "empilar",
    "empilaría": "empilar",
    "empilaríais": "empilar",
    "empilaríamos": "empilar",
    "empilarían": "empilar",
    "empilarías": "empilar",
    "empilas": "empilar",
    "empilase": "empilar",
    "empilaseis": "empilar",
    "empilasen": "empilar",
    "empilases": "empilar",
    "empilaste": "empilar",
    "empilasteis": "empilar",
    "empile": "empilar",
    "empilemos": "empilar",
    "empilen": "empilar",
    "empiles": "empilar",
    "empilo": "empilar",
    "empilábamos": "empilar",
    "empiláis": "empilar",
    "empiláramos": "empilar",
    "empiláremos": "empilar",
    "empilásemos": "empilar",
    "empilé": "empilar",
    "empiléis": "empilar",
    "empiló": "empilar",
    "empina": "empinar",
    "empinaba": "empinar",
    "empinabais": "empinar",
    "empinaban": "empinar",
    "empinabas": "empinar",
    "empinaciones": "empinación",
    "empinad": "empinar",
    "empinada": "empinar",
    "empinadas": "empinar",
    "empinado": "empinar",
    "empinados": "empinar",
    "empinaduras": "empinadura",
    "empinamientos": "empinamiento",
    "empinamos": "empinar",
    "empinan": "empinar",
    "empinando": "empinar",
    "empinara": "empinar",
    "empinarais": "empinar",
    "empinaran": "empinar",
    "empinaras": "empinar",
    "empinare": "empinar",
    "empinareis": "empinar",
    "empinaremos": "empinar",
    "empinaren": "empinar",
    "empinares": "empinar",
    "empinaron": "empinar",
    "empinará": "empinar",
    "empinarán": "empinar",
    "empinarás": "empinar",
    "empinaré": "empinar",
    "empinaréis": "empinar",
    "empinaría": "empinar",
    "empinaríais": "empinar",
    "empinaríamos": "empinar",
    "empinarían": "empinar",
    "empinarías": "empinar",
    "empinas": "empinar",
    "empinase": "empinar",
    "empinaseis": "empinar",
    "empinasen": "empinar",
    "empinases": "empinar",
    "empinaste": "empinar",
    "empinasteis": "empinar",
    "empine": "empinar",
    "empinemos": "empinar",
    "empinen": "empinar",
    "empines": "empinar",
    "empingorota": "empingorotar",
    "empingorotaba": "empingorotar",
    "empingorotabais": "empingorotar",
    "empingorotaban": "empingorotar",
    "empingorotabas": "empingorotar",
    "empingorotad": "empingorotar",
    "empingorotada": "empingorotar",
    "empingorotadas": "empingorotar",
    "empingorotado": "empingorotar",
    "empingorotados": "empingorotar",
    "empingorotamos": "empingorotar",
    "empingorotan": "empingorotar",
    "empingorotando": "empingorotar",
    "empingorotara": "empingorotar",
    "empingorotarais": "empingorotar",
    "empingorotaran": "empingorotar",
    "empingorotaras": "empingorotar",
    "empingorotare": "empingorotar",
    "empingorotareis": "empingorotar",
    "empingorotaremos": "empingorotar",
    "empingorotaren": "empingorotar",
    "empingorotares": "empingorotar",
    "empingorotaron": "empingorotar",
    "empingorotará": "empingorotar",
    "empingorotarán": "empingorotar",
    "empingorotarás": "empingorotar",
    "empingorotaré": "empingorotar",
    "empingorotaréis": "empingorotar",
    "empingorotaría": "empingorotar",
    "empingorotaríais": "empingorotar",
    "empingorotaríamos": "empingorotar",
    "empingorotarían": "empingorotar",
    "empingorotarías": "empingorotar",
    "empingorotas": "empingorotar",
    "empingorotase": "empingorotar",
    "empingorotaseis": "empingorotar",
    "empingorotasen": "empingorotar",
    "empingorotases": "empingorotar",
    "empingorotaste": "empingorotar",
    "empingorotasteis": "empingorotar",
    "empingorote": "empingorotar",
    "empingorotemos": "empingorotar",
    "empingoroten": "empingorotar",
    "empingorotes": "empingorotar",
    "empingoroto": "empingorotar",
    "empingorotábamos": "empingorotar",
    "empingorotáis": "empingorotar",
    "empingorotáramos": "empingorotar",
    "empingorotáremos": "empingorotar",
    "empingorotásemos": "empingorotar",
    "empingoroté": "empingorotar",
    "empingorotéis": "empingorotar",
    "empingorotó": "empingorotar",
    "empino": "empinar",
    "empinos": "empino",
    "empinábamos": "empinar",
    "empináis": "empinar",
    "empináramos": "empinar",
    "empináremos": "empinar",
    "empinásemos": "empinar",
    "empiné": "empinar",
    "empinéis": "empinar",
    "empinó": "empinar",
    "empipa": "empipar",
    "empipaba": "empipar",
    "empipabais": "empipar",
    "empipaban": "empipar",
    "empipabas": "empipar",
    "empipad": "empipar",
    "empipada": "empipar",
    "empipadas": "empipar",
    "empipado": "empipar",
    "empipados": "empipar",
    "empipamos": "empipar",
    "empipan": "empipar",
    "empipando": "empipar",
    "empipara": "empipar",
    "empiparais": "empipar",
    "empiparan": "empipar",
    "empiparas": "empipar",
    "empipare": "empipar",
    "empipareis": "empipar",
    "empiparemos": "empipar",
    "empiparen": "empipar",
    "empipares": "empipar",
    "empiparon": "empipar",
    "empipará": "empipar",
    "empiparán": "empipar",
    "empiparás": "empipar",
    "empiparé": "empipar",
    "empiparéis": "empipar",
    "empiparía": "empipar",
    "empiparíais": "empipar",
    "empiparíamos": "empipar",
    "empiparían": "empipar",
    "empiparías": "empipar",
    "empipas": "empipar",
    "empipase": "empipar",
    "empipaseis": "empipar",
    "empipasen": "empipar",
    "empipases": "empipar",
    "empipaste": "empipar",
    "empipasteis": "empipar",
    "empipe": "empipar",
    "empipemos": "empipar",
    "empipen": "empipar",
    "empipes": "empipar",
    "empipo": "empipar",
    "empipábamos": "empipar",
    "empipáis": "empipar",
    "empipáramos": "empipar",
    "empipáremos": "empipar",
    "empipásemos": "empipar",
    "empipé": "empipar",
    "empipéis": "empipar",
    "empipó": "empipar",
    "empique": "empicar",
    "empiquemos": "empicar",
    "empiquen": "empicar",
    "empiques": "empicar",
    "empiqué": "empicar",
    "empiquéis": "empicar",
    "empireumas": "empireuma",
    "empireumática": "empireumático",
    "empireumáticas": "empireumático",
    "empireumáticos": "empireumático",
    "empiriocriticismos": "empiriocriticismo",
    "empirismos": "empirismo",
    "empiristas": "empirista",
    "empitona": "empitonar",
    "empitonaba": "empitonar",
    "empitonabais": "empitonar",
    "empitonaban": "empitonar",
    "empitonabas": "empitonar",
    "empitonad": "empitonar",
    "empitonada": "empitonar",
    "empitonadas": "empitonar",
    "empitonado": "empitonar",
    "empitonados": "empitonar",
    "empitonamos": "empitonar",
    "empitonan": "empitonar",
    "empitonando": "empitonar",
    "empitonara": "empitonar",
    "empitonarais": "empitonar",
    "empitonaran": "empitonar",
    "empitonaras": "empitonar",
    "empitonare": "empitonar",
    "empitonareis": "empitonar",
    "empitonaremos": "empitonar",
    "empitonaren": "empitonar",
    "empitonares": "empitonar",
    "empitonaron": "empitonar",
    "empitonará": "empitonar",
    "empitonarán": "empitonar",
    "empitonarás": "empitonar",
    "empitonaré": "empitonar",
    "empitonaréis": "empitonar",
    "empitonaría": "empitonar",
    "empitonaríais": "empitonar",
    "empitonaríamos": "empitonar",
    "empitonarían": "empitonar",
    "empitonarías": "empitonar",
    "empitonas": "empitonar",
    "empitonase": "empitonar",
    "empitonaseis": "empitonar",
    "empitonasen": "empitonar",
    "empitonases": "empitonar",
    "empitonaste": "empitonar",
    "empitonasteis": "empitonar",
    "empitone": "empitonar",
    "empitonemos": "empitonar",
    "empitonen": "empitonar",
    "empitones": "empitonar",
    "empitono": "empitonar",
    "empitonábamos": "empitonar",
    "empitonáis": "empitonar",
    "empitonáramos": "empitonar",
    "empitonáremos": "empitonar",
    "empitonásemos": "empitonar",
    "empitoné": "empitonar",
    "empitonéis": "empitonar",
    "empitonó": "empitonar",
    "empizarra": "empizarrar",
    "empizarraba": "empizarrar",
    "empizarrabais": "empizarrar",
    "empizarraban": "empizarrar",
    "empizarrabas": "empizarrar",
    "empizarrad": "empizarrar",
    "empizarrada": "empizarrar",
    "empizarradas": "empizarrar",
    "empizarrado": "empizarrar",
    "empizarrados": "empizarrar",
    "empizarramos": "empizarrar",
    "empizarran": "empizarrar",
    "empizarrando": "empizarrar",
    "empizarrara": "empizarrar",
    "empizarrarais": "empizarrar",
    "empizarraran": "empizarrar",
    "empizarraras": "empizarrar",
    "empizarrare": "empizarrar",
    "empizarrareis": "empizarrar",
    "empizarraremos": "empizarrar",
    "empizarraren": "empizarrar",
    "empizarrares": "empizarrar",
    "empizarraron": "empizarrar",
    "empizarrará": "empizarrar",
    "empizarrarán": "empizarrar",
    "empizarrarás": "empizarrar",
    "empizarraré": "empizarrar",
    "empizarraréis": "empizarrar",
    "empizarraría": "empizarrar",
    "empizarraríais": "empizarrar",
    "empizarraríamos": "empizarrar",
    "empizarrarían": "empizarrar",
    "empizarrarías": "empizarrar",
    "empizarras": "empizarrar",
    "empizarrase": "empizarrar",
    "empizarraseis": "empizarrar",
    "empizarrasen": "empizarrar",
    "empizarrases": "empizarrar",
    "empizarraste": "empizarrar",
    "empizarrasteis": "empizarrar",
    "empizarre": "empizarrar",
    "empizarremos": "empizarrar",
    "empizarren": "empizarrar",
    "empizarres": "empizarrar",
    "empizarro": "empizarrar",
    "empizarrábamos": "empizarrar",
    "empizarráis": "empizarrar",
    "empizarráramos": "empizarrar",
    "empizarráremos": "empizarrar",
    "empizarrásemos": "empizarrar",
    "empizarré": "empizarrar",
    "empizarréis": "empizarrar",
    "empizarró": "empizarrar",
    "empiñonados": "empiñonado",
    "emplace": "emplazar",
    "emplacemos": "emplazar",
    "emplacen": "emplazar",
    "emplaces": "emplazar",
    "emplacé": "emplazar",
    "emplacéis": "emplazar",
    "emplasta": "emplastar",
    "emplastaba": "emplastar",
    "emplastabais": "emplastar",
    "emplastaban": "emplastar",
    "emplastabas": "emplastar",
    "emplastad": "emplastar",
    "emplastada": "emplastar",
    "emplastadas": "emplastar",
    "emplastado": "emplastar",
    "emplastados": "emplastar",
    "emplastaduras": "emplastadura",
    "emplastamientos": "emplastamiento",
    "emplastamos": "emplastar",
    "emplastan": "emplastar",
    "emplastando": "emplastar",
    "emplastara": "emplastar",
    "emplastarais": "emplastar",
    "emplastaran": "emplastar",
    "emplastaras": "emplastar",
    "emplastare": "emplastar",
    "emplastareis": "emplastar",
    "emplastaremos": "emplastar",
    "emplastaren": "emplastar",
    "emplastares": "emplastar",
    "emplastaron": "emplastar",
    "emplastará": "emplastar",
    "emplastarán": "emplastar",
    "emplastarás": "emplastar",
    "emplastaré": "emplastar",
    "emplastaréis": "emplastar",
    "emplastaría": "emplastar",
    "emplastaríais": "emplastar",
    "emplastaríamos": "emplastar",
    "emplastarían": "emplastar",
    "emplastarías": "emplastar",
    "emplastas": "emplastar",
    "emplastase": "emplastar",
    "emplastaseis": "emplastar",
    "emplastasen": "emplastar",
    "emplastases": "emplastar",
    "emplastaste": "emplastar",
    "emplastasteis": "emplastar",
    "emplaste": "emplastar",
    "emplastece": "emplastecer",
    "emplasteced": "emplastecer",
    "emplastecemos": "emplastecer",
    "emplastecen": "emplastecer",
    "emplasteceremos": "emplastecer",
    "emplastecerá": "emplastecer",
    "emplastecerán": "emplastecer",
    "emplastecerás": "emplastecer",
    "emplasteceré": "emplastecer",
    "emplasteceréis": "emplastecer",
    "emplastecería": "emplastecer",
    "emplasteceríais": "emplastecer",
    "emplasteceríamos": "emplastecer",
    "emplastecerían": "emplastecer",
    "emplastecerías": "emplastecer",
    "emplasteces": "emplastecer",
    "emplastecida": "emplastecer",
    "emplastecidas": "emplastecer",
    "emplastecido": "emplastecer",
    "emplastecidos": "emplastecer",
    "emplasteciendo": "emplastecer",
    "emplasteciera": "emplastecer",
    "emplastecierais": "emplastecer",
    "emplastecieran": "emplastecer",
    "emplastecieras": "emplastecer",
    "emplasteciere": "emplastecer",
    "emplasteciereis": "emplastecer",
    "emplastecieren": "emplastecer",
    "emplastecieres": "emplastecer",
    "emplastecieron": "emplastecer",
    "emplasteciese": "emplastecer",
    "emplastecieseis": "emplastecer",
    "emplasteciesen": "emplastecer",
    "emplastecieses": "emplastecer",
    "emplastecimos": "emplastecer",
    "emplasteciste": "emplastecer",
    "emplastecisteis": "emplastecer",
    "emplasteciéramos": "emplastecer",
    "emplasteciéremos": "emplastecer",
    "emplasteciésemos": "emplastecer",
    "emplasteció": "emplastecer",
    "emplastecéis": "emplastecer",
    "emplastecí": "emplastecer",
    "emplastecía": "emplastecer",
    "emplastecíais": "emplastecer",
    "emplastecíamos": "emplastecer",
    "emplastecían": "emplastecer",
    "emplastecías": "emplastecer",
    "emplastemos": "emplastar",
    "emplasten": "emplastar",
    "emplastes": "emplaste",
    "emplastezca": "emplastecer",
    "emplastezcamos": "emplastecer",
    "emplastezcan": "emplastecer",
    "emplastezcas": "emplastecer",
    "emplastezco": "emplastecer",
    "emplastezcáis": "emplastecer",
    "emplasto": "emplastar",
    "emplastos": "emplasto",
    "emplastábamos": "emplastar",
    "emplastáis": "emplastar",
    "emplastáramos": "emplastar",
    "emplastáremos": "emplastar",
    "emplastásemos": "emplastar",
    "emplasté": "emplastar",
    "emplastéis": "emplastar",
    "emplastó": "emplastar",
    "emplaza": "emplazar",
    "emplazaba": "emplazar",
    "emplazabais": "emplazar",
    "emplazaban": "emplazar",
    "emplazabas": "emplazar",
    "emplazad": "emplazar",
    "emplazada": "emplazar",
    "emplazadas": "emplazar",
    "emplazado": "emplazar",
    "emplazadora": "emplazador",
    "emplazadoras": "emplazador",
    "emplazadores": "emplazador",
    "emplazados": "emplazar",
    "emplazamientos": "emplazamiento",
    "emplazamos": "emplazar",
    "emplazan": "emplazar",
    "emplazando": "emplazar",
    "emplazara": "emplazar",
    "emplazarais": "emplazar",
    "emplazaran": "emplazar",
    "emplazaras": "emplazar",
    "emplazare": "emplazar",
    "emplazareis": "emplazar",
    "emplazaremos": "emplazar",
    "emplazaren": "emplazar",
    "emplazares": "emplazar",
    "emplazaron": "emplazar",
    "emplazará": "emplazar",
    "emplazarán": "emplazar",
    "emplazarás": "emplazar",
    "emplazaré": "emplazar",
    "emplazaréis": "emplazar",
    "emplazaría": "emplazar",
    "emplazaríais": "emplazar",
    "emplazaríamos": "emplazar",
    "emplazarían": "emplazar",
    "emplazarías": "emplazar",
    "emplazas": "emplazar",
    "emplazase": "emplazar",
    "emplazaseis": "emplazar",
    "emplazasen": "emplazar",
    "emplazases": "emplazar",
    "emplazaste": "emplazar",
    "emplazasteis": "emplazar",
    "emplazo": "emplazar",
    "emplazábamos": "emplazar",
    "emplazáis": "emplazar",
    "emplazáramos": "emplazar",
    "emplazáremos": "emplazar",
    "emplazásemos": "emplazar",
    "emplazó": "emplazar",
    "emplea": "emplear",
    "empleaba": "emplear",
    "empleabais": "emplear",
    "empleaban": "emplear",
    "empleabas": "emplear",
    "emplead": "emplear",
    "empleada": "emplear",
    "empleadas": "emplear",
    "empleado": "emplear",
    "empleadora": "empleador",
    "empleadoras": "empleador",
    "empleadores": "empleador",
    "empleados": "emplear",
    "empleamos": "emplear",
    "emplean": "emplear",
    "empleando": "emplear",
    "empleara": "emplear",
    "emplearais": "emplear",
    "emplearan": "emplear",
    "emplearas": "emplear",
    "empleare": "emplear",
    "empleareis": "emplear",
    "emplearemos": "emplear",
    "emplearen": "emplear",
    "empleares": "emplear",
    "emplearon": "emplear",
    "empleará": "emplear",
    "emplearán": "emplear",
    "emplearás": "emplear",
    "emplearé": "emplear",
    "emplearéis": "emplear",
    "emplearía": "emplear",
    "emplearíais": "emplear",
    "emplearíamos": "emplear",
    "emplearían": "emplear",
    "emplearías": "emplear",
    "empleas": "emplear",
    "emplease": "emplear",
    "empleaseis": "emplear",
    "empleasen": "emplear",
    "empleases": "emplear",
    "empleaste": "emplear",
    "empleasteis": "emplear",
    "emplee": "emplear",
    "empleemos": "emplear",
    "empleen": "emplear",
    "emplees": "emplear",
    "empleitas": "empleita",
    "empleitera": "empleitero",
    "empleiteras": "empleitero",
    "empleiteros": "empleitero",
    "emplentas": "emplenta",
    "empleo": "emplear",
    "empleomanías": "empleomanía",
    "empleos": "empleo",
    "empleábamos": "emplear",
    "empleáis": "emplear",
    "empleáramos": "emplear",
    "empleáremos": "emplear",
    "empleásemos": "emplear",
    "empleé": "emplear",
    "empleéis": "emplear",
    "empleó": "emplear",
    "emploma": "emplomar",
    "emplomaba": "emplomar",
    "emplomabais": "emplomar",
    "emplomaban": "emplomar",
    "emplomabas": "emplomar",
    "emplomad": "emplomar",
    "emplomada": "emplomar",
    "emplomadas": "emplomar",
    "emplomado": "emplomar",
    "emplomadora": "emplomador",
    "emplomadoras": "emplomador",
    "emplomadores": "emplomador",
    "emplomados": "emplomar",
    "emplomaduras": "emplomadura",
    "emplomamos": "emplomar",
    "emploman": "emplomar",
    "emplomando": "emplomar",
    "emplomara": "emplomar",
    "emplomarais": "emplomar",
    "emplomaran": "emplomar",
    "emplomaras": "emplomar",
    "emplomare": "emplomar",
    "emplomareis": "emplomar",
    "emplomaremos": "emplomar",
    "emplomaren": "emplomar",
    "emplomares": "emplomar",
    "emplomaron": "emplomar",
    "emplomará": "emplomar",
    "emplomarán": "emplomar",
    "emplomarás": "emplomar",
    "emplomaré": "emplomar",
    "emplomaréis": "emplomar",
    "emplomaría": "emplomar",
    "emplomaríais": "emplomar",
    "emplomaríamos": "emplomar",
    "emplomarían": "emplomar",
    "emplomarías": "emplomar",
    "emplomas": "emplomar",
    "emplomase": "emplomar",
    "emplomaseis": "emplomar",
    "emplomasen": "emplomar",
    "emplomases": "emplomar",
    "emplomaste": "emplomar",
    "emplomasteis": "emplomar",
    "emplome": "emplomar",
    "emplomemos": "emplomar",
    "emplomen": "emplomar",
    "emplomes": "emplomar",
    "emplomo": "emplomar",
    "emplomábamos": "emplomar",
    "emplomáis": "emplomar",
    "emplomáramos": "emplomar",
    "emplomáremos": "emplomar",
    "emplomásemos": "emplomar",
    "emplomé": "emplomar",
    "emploméis": "emplomar",
    "emplomó": "emplomar",
    "empluma": "emplumar",
    "emplumaba": "emplumar",
    "emplumabais": "emplumar",
    "emplumaban": "emplumar",
    "emplumabas": "emplumar",
    "emplumad": "emplumar",
    "emplumada": "emplumar",
    "emplumadas": "emplumar",
    "emplumado": "emplumar",
    "emplumados": "emplumar",
    "emplumamos": "emplumar",
    "empluman": "emplumar",
    "emplumando": "emplumar",
    "emplumara": "emplumar",
    "emplumarais": "emplumar",
    "emplumaran": "emplumar",
    "emplumaras": "emplumar",
    "emplumare": "emplumar",
    "emplumareis": "emplumar",
    "emplumaremos": "emplumar",
    "emplumaren": "emplumar",
    "emplumares": "emplumar",
    "emplumaron": "emplumar",
    "emplumará": "emplumar",
    "emplumarán": "emplumar",
    "emplumarás": "emplumar",
    "emplumaré": "emplumar",
    "emplumaréis": "emplumar",
    "emplumaría": "emplumar",
    "emplumaríais": "emplumar",
    "emplumaríamos": "emplumar",
    "emplumarían": "emplumar",
    "emplumarías": "emplumar",
    "emplumas": "emplumar",
    "emplumase": "emplumar",
    "emplumaseis": "emplumar",
    "emplumasen": "emplumar",
    "emplumases": "emplumar",
    "emplumaste": "emplumar",
    "emplumasteis": "emplumar",
    "emplume": "emplumar",
    "emplumece": "emplumecer",
    "emplumeced": "emplumecer",
    "emplumecemos": "emplumecer",
    "emplumecen": "emplumecer",
    "emplumeceremos": "emplumecer",
    "emplumecerá": "emplumecer",
    "emplumecerán": "emplumecer",
    "emplumecerás": "emplumecer",
    "emplumeceré": "emplumecer",
    "emplumeceréis": "emplumecer",
    "emplumecería": "emplumecer",
    "emplumeceríais": "emplumecer",
    "emplumeceríamos": "emplumecer",
    "emplumecerían": "emplumecer",
    "emplumecerías": "emplumecer",
    "emplumeces": "emplumecer",
    "emplumecida": "emplumecer",
    "emplumecidas": "emplumecer",
    "emplumecido": "emplumecer",
    "emplumecidos": "emplumecer",
    "emplumeciendo": "emplumecer",
    "emplumeciera": "emplumecer",
    "emplumecierais": "emplumecer",
    "emplumecieran": "emplumecer",
    "emplumecieras": "emplumecer",
    "emplumeciere": "emplumecer",
    "emplumeciereis": "emplumecer",
    "emplumecieren": "emplumecer",
    "emplumecieres": "emplumecer",
    "emplumecieron": "emplumecer",
    "emplumeciese": "emplumecer",
    "emplumecieseis": "emplumecer",
    "emplumeciesen": "emplumecer",
    "emplumecieses": "emplumecer",
    "emplumecimos": "emplumecer",
    "emplumeciste": "emplumecer",
    "emplumecisteis": "emplumecer",
    "emplumeciéramos": "emplumecer",
    "emplumeciéremos": "emplumecer",
    "emplumeciésemos": "emplumecer",
    "emplumeció": "emplumecer",
    "emplumecéis": "emplumecer",
    "emplumecí": "emplumecer",
    "emplumecía": "emplumecer",
    "emplumecíais": "emplumecer",
    "emplumecíamos": "emplumecer",
    "emplumecían": "emplumecer",
    "emplumecías": "emplumecer",
    "emplumemos": "emplumar",
    "emplumen": "emplumar",
    "emplumes": "emplumar",
    "emplumezca": "emplumecer",
    "emplumezcamos": "emplumecer",
    "emplumezcan": "emplumecer",
    "emplumezcas": "emplumecer",
    "emplumezco": "emplumecer",
    "emplumezcáis": "emplumecer",
    "emplumo": "emplumar",
    "emplumábamos": "emplumar",
    "emplumáis": "emplumar",
    "emplumáramos": "emplumar",
    "emplumáremos": "emplumar",
    "emplumásemos": "emplumar",
    "emplumé": "emplumar",
    "empluméis": "emplumar",
    "emplumó": "emplumar",
    "emplástica": "emplástico",
    "emplásticas": "emplástico",
    "emplásticos": "emplástico",
    "empobrece": "empobrecer",
    "empobreced": "empobrecer",
    "empobrecedora": "empobrecedor",
    "empobrecedoras": "empobrecedor",
    "empobrecedores": "empobrecedor",
    "empobrecemos": "empobrecer",
    "empobrecen": "empobrecer",
    "empobreceremos": "empobrecer",
    "empobrecerá": "empobrecer",
    "empobrecerán": "empobrecer",
    "empobrecerás": "empobrecer",
    "empobreceré": "empobrecer",
    "empobreceréis": "empobrecer",
    "empobrecería": "empobrecer",
    "empobreceríais": "empobrecer",
    "empobreceríamos": "empobrecer",
    "empobrecerían": "empobrecer",
    "empobrecerías": "empobrecer",
    "empobreces": "empobrecer",
    "empobrecida": "empobrecer",
    "empobrecidas": "empobrecer",
    "empobrecido": "empobrecer",
    "empobrecidos": "empobrecer",
    "empobreciendo": "empobrecer",
    "empobreciera": "empobrecer",
    "empobrecierais": "empobrecer",
    "empobrecieran": "empobrecer",
    "empobrecieras": "empobrecer",
    "empobreciere": "empobrecer",
    "empobreciereis": "empobrecer",
    "empobrecieren": "empobrecer",
    "empobrecieres": "empobrecer",
    "empobrecieron": "empobrecer",
    "empobreciese": "empobrecer",
    "empobrecieseis": "empobrecer",
    "empobreciesen": "empobrecer",
    "empobrecieses": "empobrecer",
    "empobrecimientos": "empobrecimiento",
    "empobrecimos": "empobrecer",
    "empobreciste": "empobrecer",
    "empobrecisteis": "empobrecer",
    "empobreciéramos": "empobrecer",
    "empobreciéremos": "empobrecer",
    "empobreciésemos": "empobrecer",
    "empobreció": "empobrecer",
    "empobrecéis": "empobrecer",
    "empobrecí": "empobrecer",
    "empobrecía": "empobrecer",
    "empobrecíais": "empobrecer",
    "empobrecíamos": "empobrecer",
    "empobrecían": "empobrecer",
    "empobrecías": "empobrecer",
    "empobrezca": "empobrecer",
    "empobrezcamos": "empobrecer",
    "empobrezcan": "empobrecer",
    "empobrezcas": "empobrecer",
    "empobrezco": "empobrecer",
    "empobrezcáis": "empobrecer",
    "empoce": "empozar",
    "empocemos": "empozar",
    "empocen": "empozar",
    "empoces": "empozar",
    "empocé": "empozar",
    "empocéis": "empozar",
    "empodrecimientos": "empodrecimiento",
    "empolla": "empollar",
    "empollaba": "empollar",
    "empollabais": "empollar",
    "empollaban": "empollar",
    "empollabas": "empollar",
    "empollad": "empollar",
    "empollada": "empollar",
    "empolladas": "empollar",
    "empollado": "empollar",
    "empollados": "empollar",
    "empolladuras": "empolladura",
    "empollamos": "empollar",
    "empollan": "empollar",
    "empollando": "empollar",
    "empollara": "empollar",
    "empollarais": "empollar",
    "empollaran": "empollar",
    "empollaras": "empollar",
    "empollare": "empollar",
    "empollareis": "empollar",
    "empollaremos": "empollar",
    "empollaren": "empollar",
    "empollares": "empollar",
    "empollaron": "empollar",
    "empollará": "empollar",
    "empollarán": "empollar",
    "empollarás": "empollar",
    "empollaré": "empollar",
    "empollaréis": "empollar",
    "empollaría": "empollar",
    "empollaríais": "empollar",
    "empollaríamos": "empollar",
    "empollarían": "empollar",
    "empollarías": "empollar",
    "empollas": "empollar",
    "empollase": "empollar",
    "empollaseis": "empollar",
    "empollasen": "empollar",
    "empollases": "empollar",
    "empollaste": "empollar",
    "empollasteis": "empollar",
    "empolle": "empollar",
    "empollemos": "empollar",
    "empollen": "empollar",
    "empolles": "empollar",
    "empollo": "empollar",
    "empollona": "empollón",
    "empollonas": "empollón",
    "empollones": "empollón",
    "empollábamos": "empollar",
    "empolláis": "empollar",
    "empolláramos": "empollar",
    "empolláremos": "empollar",
    "empollásemos": "empollar",
    "empollé": "empollar",
    "empolléis": "empollar",
    "empolló": "empollar",
    "empolva": "empolvar",
    "empolvaba": "empolvar",
    "empolvabais": "empolvar",
    "empolvaban": "empolvar",
    "empolvabas": "empolvar",
    "empolvad": "empolvar",
    "empolvada": "empolvar",
    "empolvadas": "empolvar",
    "empolvado": "empolvar",
    "empolvados": "empolvar",
    "empolvamos": "empolvar",
    "empolvan": "empolvar",
    "empolvando": "empolvar",
    "empolvara": "empolvar",
    "empolvarais": "empolvar",
    "empolvaran": "empolvar",
    "empolvaras": "empolvar",
    "empolvare": "empolvar",
    "empolvareis": "empolvar",
    "empolvaremos": "empolvar",
    "empolvaren": "empolvar",
    "empolvares": "empolvar",
    "empolvaron": "empolvar",
    "empolvará": "empolvar",
    "empolvarán": "empolvar",
    "empolvarás": "empolvar",
    "empolvaré": "empolvar",
    "empolvaréis": "empolvar",
    "empolvaría": "empolvar",
    "empolvaríais": "empolvar",
    "empolvaríamos": "empolvar",
    "empolvarían": "empolvar",
    "empolvarías": "empolvar",
    "empolvas": "empolvar",
    "empolvase": "empolvar",
    "empolvaseis": "empolvar",
    "empolvasen": "empolvar",
    "empolvases": "empolvar",
    "empolvaste": "empolvar",
    "empolvasteis": "empolvar",
    "empolve": "empolvar",
    "empolvemos": "empolvar",
    "empolven": "empolvar",
    "empolves": "empolvar",
    "empolvo": "empolvar",
    "empolvora": "empolvorar",
    "empolvoraba": "empolvorar",
    "empolvorabais": "empolvorar",
    "empolvoraban": "empolvorar",
    "empolvorabas": "empolvorar",
    "empolvorad": "empolvorar",
    "empolvorada": "empolvorar",
    "empolvoradas": "empolvorar",
    "empolvorado": "empolvorar",
    "empolvorados": "empolvorar",
    "empolvoramos": "empolvorar",
    "empolvoran": "empolvorar",
    "empolvorando": "empolvorar",
    "empolvorara": "empolvorar",
    "empolvorarais": "empolvorar",
    "empolvoraran": "empolvorar",
    "empolvoraras": "empolvorar",
    "empolvorare": "empolvorar",
    "empolvorareis": "empolvorar",
    "empolvoraremos": "empolvorar",
    "empolvoraren": "empolvorar",
    "empolvorares": "empolvorar",
    "empolvoraron": "empolvorar",
    "empolvorará": "empolvorar",
    "empolvorarán": "empolvorar",
    "empolvorarás": "empolvorar",
    "empolvoraré": "empolvorar",
    "empolvoraréis": "empolvorar",
    "empolvoraría": "empolvorar",
    "empolvoraríais": "empolvorar",
    "empolvoraríamos": "empolvorar",
    "empolvorarían": "empolvorar",
    "empolvorarías": "empolvorar",
    "empolvoras": "empolvorar",
    "empolvorase": "empolvorar",
    "empolvoraseis": "empolvorar",
    "empolvorasen": "empolvorar",
    "empolvorases": "empolvorar",
    "empolvoraste": "empolvorar",
    "empolvorasteis": "empolvorar",
    "empolvore": "empolvorar",
    "empolvoremos": "empolvorar",
    "empolvoren": "empolvorar",
    "empolvores": "empolvorar",
    "empolvorice": "empolvorizar",
    "empolvoricemos": "empolvorizar",
    "empolvoricen": "empolvorizar",
    "empolvorices": "empolvorizar",
    "empolvoricé": "empolvorizar",
    "empolvoricéis": "empolvorizar",
    "empolvoriza": "empolvorizar",
    "empolvorizaba": "empolvorizar",
    "empolvorizabais": "empolvorizar",
    "empolvorizaban": "empolvorizar",
    "empolvorizabas": "empolvorizar",
    "empolvorizad": "empolvorizar",
    "empolvorizada": "empolvorizar",
    "empolvorizadas": "empolvorizar",
    "empolvorizado": "empolvorizar",
    "empolvorizados": "empolvorizar",
    "empolvorizamos": "empolvorizar",
    "empolvorizan": "empolvorizar",
    "empolvorizando": "empolvorizar",
    "empolvorizara": "empolvorizar",
    "empolvorizarais": "empolvorizar",
    "empolvorizaran": "empolvorizar",
    "empolvorizaras": "empolvorizar",
    "empolvorizare": "empolvorizar",
    "empolvorizareis": "empolvorizar",
    "empolvorizaremos": "empolvorizar",
    "empolvorizaren": "empolvorizar",
    "empolvorizares": "empolvorizar",
    "empolvorizaron": "empolvorizar",
    "empolvorizará": "empolvorizar",
    "empolvorizarán": "empolvorizar",
    "empolvorizarás": "empolvorizar",
    "empolvorizaré": "empolvorizar",
    "empolvorizaréis": "empolvorizar",
    "empolvorizaría": "empolvorizar",
    "empolvorizaríais": "empolvorizar",
    "empolvorizaríamos": "empolvorizar",
    "empolvorizarían": "empolvorizar",
    "empolvorizarías": "empolvorizar",
    "empolvorizas": "empolvorizar",
    "empolvorizase": "empolvorizar",
    "empolvorizaseis": "empolvorizar",
    "empolvorizasen": "empolvorizar",
    "empolvorizases": "empolvorizar",
    "empolvorizaste": "empolvorizar",
    "empolvorizasteis": "empolvorizar",
    "empolvorizo": "empolvorizar",
    "empolvorizábamos": "empolvorizar",
    "empolvorizáis": "empolvorizar",
    "empolvorizáramos": "empolvorizar",
    "empolvorizáremos": "empolvorizar",
    "empolvorizásemos": "empolvorizar",
    "empolvorizó": "empolvorizar",
    "empolvoro": "empolvorar",
    "empolvorábamos": "empolvorar",
    "empolvoráis": "empolvorar",
    "empolvoráramos": "empolvorar",
    "empolvoráremos": "empolvorar",
    "empolvorásemos": "empolvorar",
    "empolvoré": "empolvorar",
    "empolvoréis": "empolvorar",
    "empolvoró": "empolvorar",
    "empolvábamos": "empolvar",
    "empolváis": "empolvar",
    "empolváramos": "empolvar",
    "empolváremos": "empolvar",
    "empolvásemos": "empolvar",
    "empolvé": "empolvar",
    "empolvéis": "empolvar",
    "empolvó": "empolvar",
    "emponcha": "emponchar",
    "emponchaba": "emponchar",
    "emponchabais": "emponchar",
    "emponchaban": "emponchar",
    "emponchabas": "emponchar",
    "emponchad": "emponchar",
    "emponchada": "emponchar",
    "emponchadas": "emponchar",
    "emponchado": "emponchar",
    "emponchados": "emponchar",
    "emponchamos": "emponchar",
    "emponchan": "emponchar",
    "emponchando": "emponchar",
    "emponchara": "emponchar",
    "emponcharais": "emponchar",
    "emponcharan": "emponchar",
    "emponcharas": "emponchar",
    "emponchare": "emponchar",
    "emponchareis": "emponchar",
    "emponcharemos": "emponchar",
    "emponcharen": "emponchar",
    "emponchares": "emponchar",
    "emponcharon": "emponchar",
    "emponchará": "emponchar",
    "emponcharán": "emponchar",
    "emponcharás": "emponchar",
    "emponcharé": "emponchar",
    "emponcharéis": "emponchar",
    "emponcharía": "emponchar",
    "emponcharíais": "emponchar",
    "emponcharíamos": "emponchar",
    "emponcharían": "emponchar",
    "emponcharías": "emponchar",
    "emponchas": "emponchar",
    "emponchase": "emponchar",
    "emponchaseis": "emponchar",
    "emponchasen": "emponchar",
    "emponchases": "emponchar",
    "emponchaste": "emponchar",
    "emponchasteis": "emponchar",
    "emponche": "emponchar",
    "emponchemos": "emponchar",
    "emponchen": "emponchar",
    "emponches": "emponchar",
    "emponcho": "emponchar",
    "emponchábamos": "emponchar",
    "emponcháis": "emponchar",
    "emponcháramos": "emponchar",
    "emponcháremos": "emponchar",
    "emponchásemos": "emponchar",
    "emponché": "emponchar",
    "emponchéis": "emponchar",
    "emponchó": "emponchar",
    "emponzoña": "emponzoñar",
    "emponzoñaba": "emponzoñar",
    "emponzoñabais": "emponzoñar",
    "emponzoñaban": "emponzoñar",
    "emponzoñabas": "emponzoñar",
    "emponzoñad": "emponzoñar",
    "emponzoñada": "emponzoñar",
    "emponzoñadas": "emponzoñar",
    "emponzoñado": "emponzoñar",
    "emponzoñadora": "emponzoñador",
    "emponzoñadoras": "emponzoñador",
    "emponzoñadores": "emponzoñador",
    "emponzoñados": "emponzoñar",
    "emponzoñamientos": "emponzoñamiento",
    "emponzoñamos": "emponzoñar",
    "emponzoñan": "emponzoñar",
    "emponzoñando": "emponzoñar",
    "emponzoñara": "emponzoñar",
    "emponzoñarais": "emponzoñar",
    "emponzoñaran": "emponzoñar",
    "emponzoñaras": "emponzoñar",
    "emponzoñare": "emponzoñar",
    "emponzoñareis": "emponzoñar",
    "emponzoñaremos": "emponzoñar",
    "emponzoñaren": "emponzoñar",
    "emponzoñares": "emponzoñar",
    "emponzoñaron": "emponzoñar",
    "emponzoñará": "emponzoñar",
    "emponzoñarán": "emponzoñar",
    "emponzoñarás": "emponzoñar",
    "emponzoñaré": "emponzoñar",
    "emponzoñaréis": "emponzoñar",
    "emponzoñaría": "emponzoñar",
    "emponzoñaríais": "emponzoñar",
    "emponzoñaríamos": "emponzoñar",
    "emponzoñarían": "emponzoñar",
    "emponzoñarías": "emponzoñar",
    "emponzoñas": "emponzoñar",
    "emponzoñase": "emponzoñar",
    "emponzoñaseis": "emponzoñar",
    "emponzoñasen": "emponzoñar",
    "emponzoñases": "emponzoñar",
    "emponzoñaste": "emponzoñar",
    "emponzoñasteis": "emponzoñar",
    "emponzoñe": "emponzoñar",
    "emponzoñemos": "emponzoñar",
    "emponzoñen": "emponzoñar",
    "emponzoñes": "emponzoñar",
    "emponzoño": "emponzoñar",
    "emponzoñábamos": "emponzoñar",
    "emponzoñáis": "emponzoñar",
    "emponzoñáramos": "emponzoñar",
    "emponzoñáremos": "emponzoñar",
    "emponzoñásemos": "emponzoñar",
    "emponzoñé": "emponzoñar",
    "emponzoñéis": "emponzoñar",
    "emponzoñó": "emponzoñar",
    "empopa": "empopar",
    "empopaba": "empopar",
    "empopabais": "empopar",
    "empopaban": "empopar",
    "empopabas": "empopar",
    "empopad": "empopar",
    "empopada": "empopar",
    "empopadas": "empopar",
    "empopado": "empopar",
    "empopados": "empopar",
    "empopamos": "empopar",
    "empopan": "empopar",
    "empopando": "empopar",
    "empopara": "empopar",
    "empoparais": "empopar",
    "empoparan": "empopar",
    "empoparas": "empopar",
    "empopare": "empopar",
    "empopareis": "empopar",
    "empoparemos": "empopar",
    "empoparen": "empopar",
    "empopares": "empopar",
    "empoparon": "empopar",
    "empopará": "empopar",
    "empoparán": "empopar",
    "empoparás": "empopar",
    "empoparé": "empopar",
    "empoparéis": "empopar",
    "empoparía": "empopar",
    "empoparíais": "empopar",
    "empoparíamos": "empopar",
    "empoparían": "empopar",
    "empoparías": "empopar",
    "empopas": "empopar",
    "empopase": "empopar",
    "empopaseis": "empopar",
    "empopasen": "empopar",
    "empopases": "empopar",
    "empopaste": "empopar",
    "empopasteis": "empopar",
    "empope": "empopar",
    "empopemos": "empopar",
    "empopen": "empopar",
    "empopes": "empopar",
    "empopo": "empopar",
    "empopábamos": "empopar",
    "empopáis": "empopar",
    "empopáramos": "empopar",
    "empopáremos": "empopar",
    "empopásemos": "empopar",
    "empopé": "empopar",
    "empopéis": "empopar",
    "empopó": "empopar",
    "emporcaba": "emporcar",
    "emporcabais": "emporcar",
    "emporcaban": "emporcar",
    "emporcabas": "emporcar",
    "emporcad": "emporcar",
    "emporcada": "emporcar",
    "emporcadas": "emporcar",
    "emporcado": "emporcar",
    "emporcados": "emporcar",
    "emporcamos": "emporcar",
    "emporcando": "emporcar",
    "emporcara": "emporcar",
    "emporcarais": "emporcar",
    "emporcaran": "emporcar",
    "emporcaras": "emporcar",
    "emporcare": "emporcar",
    "emporcareis": "emporcar",
    "emporcaremos": "emporcar",
    "emporcaren": "emporcar",
    "emporcares": "emporcar",
    "emporcaron": "emporcar",
    "emporcará": "emporcar",
    "emporcarán": "emporcar",
    "emporcarás": "emporcar",
    "emporcaré": "emporcar",
    "emporcaréis": "emporcar",
    "emporcaría": "emporcar",
    "emporcaríais": "emporcar",
    "emporcaríamos": "emporcar",
    "emporcarían": "emporcar",
    "emporcarías": "emporcar",
    "emporcase": "emporcar",
    "emporcaseis": "emporcar",
    "emporcasen": "emporcar",
    "emporcases": "emporcar",
    "emporcaste": "emporcar",
    "emporcasteis": "emporcar",
    "emporcábamos": "emporcar",
    "emporcáis": "emporcar",
    "emporcáramos": "emporcar",
    "emporcáremos": "emporcar",
    "emporcásemos": "emporcar",
    "emporcó": "emporcar",
    "emporios": "emporio",
    "emporquemos": "emporcar",
    "emporqué": "emporcar",
    "emporquéis": "emporcar",
    "empotra": "empotrar",
    "empotraba": "empotrar",
    "empotrabais": "empotrar",
    "empotraban": "empotrar",
    "empotrabas": "empotrar",
    "empotrables": "empotrable",
    "empotrad": "empotrar",
    "empotrada": "empotrar",
    "empotradas": "empotrar",
    "empotrado": "empotrar",
    "empotrados": "empotrar",
    "empotramientos": "empotramiento",
    "empotramos": "empotrar",
    "empotran": "empotrar",
    "empotrando": "empotrar",
    "empotrara": "empotrar",
    "empotrarais": "empotrar",
    "empotraran": "empotrar",
    "empotraras": "empotrar",
    "empotrare": "empotrar",
    "empotrareis": "empotrar",
    "empotraremos": "empotrar",
    "empotraren": "empotrar",
    "empotrares": "empotrar",
    "empotraron": "empotrar",
    "empotrará": "empotrar",
    "empotrarán": "empotrar",
    "empotrarás": "empotrar",
    "empotraré": "empotrar",
    "empotraréis": "empotrar",
    "empotraría": "empotrar",
    "empotraríais": "empotrar",
    "empotraríamos": "empotrar",
    "empotrarían": "empotrar",
    "empotrarías": "empotrar",
    "empotras": "empotrar",
    "empotrase": "empotrar",
    "empotraseis": "empotrar",
    "empotrasen": "empotrar",
    "empotrases": "empotrar",
    "empotraste": "empotrar",
    "empotrasteis": "empotrar",
    "empotre": "empotrar",
    "empotremos": "empotrar",
    "empotren": "empotrar",
    "empotrera": "empotrerar",
    "empotreraba": "empotrerar",
    "empotrerabais": "empotrerar",
    "empotreraban": "empotrerar",
    "empotrerabas": "empotrerar",
    "empotrerad": "empotrerar",
    "empotrerada": "empotrerar",
    "empotreradas": "empotrerar",
    "empotrerado": "empotrerar",
    "empotrerados": "empotrerar",
    "empotreramos": "empotrerar",
    "empotreran": "empotrerar",
    "empotrerando": "empotrerar",
    "empotrerara": "empotrerar",
    "empotrerarais": "empotrerar",
    "empotreraran": "empotrerar",
    "empotreraras": "empotrerar",
    "empotrerare": "empotrerar",
    "empotrerareis": "empotrerar",
    "empotreraremos": "empotrerar",
    "empotreraren": "empotrerar",
    "empotrerares": "empotrerar",
    "empotreraron": "empotrerar",
    "empotrerará": "empotrerar",
    "empotrerarán": "empotrerar",
    "empotrerarás": "empotrerar",
    "empotreraré": "empotrerar",
    "empotreraréis": "empotrerar",
    "empotreraría": "empotrerar",
    "empotreraríais": "empotrerar",
    "empotreraríamos": "empotrerar",
    "empotrerarían": "empotrerar",
    "empotrerarías": "empotrerar",
    "empotreras": "empotrerar",
    "empotrerase": "empotrerar",
    "empotreraseis": "empotrerar",
    "empotrerasen": "empotrerar",
    "empotrerases": "empotrerar",
    "empotreraste": "empotrerar",
    "empotrerasteis": "empotrerar",
    "empotrere": "empotrerar",
    "empotreremos": "empotrerar",
    "empotreren": "empotrerar",
    "empotreres": "empotrerar",
    "empotrero": "empotrerar",
    "empotrerábamos": "empotrerar",
    "empotreráis": "empotrerar",
    "empotreráramos": "empotrerar",
    "empotreráremos": "empotrerar",
    "empotrerásemos": "empotrerar",
    "empotreré": "empotrerar",
    "empotreréis": "empotrerar",
    "empotreró": "empotrerar",
    "empotres": "empotrar",
    "empotro": "empotrar",
    "empotrábamos": "empotrar",
    "empotráis": "empotrar",
    "empotráramos": "empotrar",
    "empotráremos": "empotrar",
    "empotrásemos": "empotrar",
    "empotré": "empotrar",
    "empotréis": "empotrar",
    "empotró": "empotrar",
    "empoza": "empozar",
    "empozaba": "empozar",
    "empozabais": "empozar",
    "empozaban": "empozar",
    "empozabas": "empozar",
    "empozad": "empozar",
    "empozada": "empozar",
    "empozadas": "empozar",
    "empozado": "empozar",
    "empozados": "empozar",
    "empozamientos": "empozamiento",
    "empozamos": "empozar",
    "empozan": "empozar",
    "empozando": "empozar",
    "empozara": "empozar",
    "empozarais": "empozar",
    "empozaran": "empozar",
    "empozaras": "empozar",
    "empozare": "empozar",
    "empozareis": "empozar",
    "empozaremos": "empozar",
    "empozaren": "empozar",
    "empozares": "empozar",
    "empozaron": "empozar",
    "empozará": "empozar",
    "empozarán": "empozar",
    "empozarás": "empozar",
    "empozaré": "empozar",
    "empozaréis": "empozar",
    "empozaría": "empozar",
    "empozaríais": "empozar",
    "empozaríamos": "empozar",
    "empozarían": "empozar",
    "empozarías": "empozar",
    "empozas": "empozar",
    "empozase": "empozar",
    "empozaseis": "empozar",
    "empozasen": "empozar",
    "empozases": "empozar",
    "empozaste": "empozar",
    "empozasteis": "empozar",
    "empozo": "empozar",
    "empozábamos": "empozar",
    "empozáis": "empozar",
    "empozáramos": "empozar",
    "empozáremos": "empozar",
    "empozásemos": "empozar",
    "empozó": "empozar",
    "empradice": "empradizar",
    "empradicemos": "empradizar",
    "empradicen": "empradizar",
    "empradices": "empradizar",
    "empradicé": "empradizar",
    "empradicéis": "empradizar",
    "empradiza": "empradizar",
    "empradizaba": "empradizar",
    "empradizabais": "empradizar",
    "empradizaban": "empradizar",
    "empradizabas": "empradizar",
    "empradizad": "empradizar",
    "empradizada": "empradizar",
    "empradizadas": "empradizar",
    "empradizado": "empradizar",
    "empradizados": "empradizar",
    "empradizamos": "empradizar",
    "empradizan": "empradizar",
    "empradizando": "empradizar",
    "empradizara": "empradizar",
    "empradizarais": "empradizar",
    "empradizaran": "empradizar",
    "empradizaras": "empradizar",
    "empradizare": "empradizar",
    "empradizareis": "empradizar",
    "empradizaremos": "empradizar",
    "empradizaren": "empradizar",
    "empradizares": "empradizar",
    "empradizaron": "empradizar",
    "empradizará": "empradizar",
    "empradizarán": "empradizar",
    "empradizarás": "empradizar",
    "empradizaré": "empradizar",
    "empradizaréis": "empradizar",
    "empradizaría": "empradizar",
    "empradizaríais": "empradizar",
    "empradizaríamos": "empradizar",
    "empradizarían": "empradizar",
    "empradizarías": "empradizar",
    "empradizas": "empradizar",
    "empradizase": "empradizar",
    "empradizaseis": "empradizar",
    "empradizasen": "empradizar",
    "empradizases": "empradizar",
    "empradizaste": "empradizar",
    "empradizasteis": "empradizar",
    "empradizo": "empradizar",
    "empradizábamos": "empradizar",
    "empradizáis": "empradizar",
    "empradizáramos": "empradizar",
    "empradizáremos": "empradizar",
    "empradizásemos": "empradizar",
    "empradizó": "empradizar",
    "emprenda": "emprender",
    "emprendamos": "emprender",
    "emprendan": "emprender",
    "emprendas": "emprender",
    "emprende": "emprender",
    "emprended": "emprender",
    "emprendedora": "emprendedor",
    "emprendedoras": "emprendedor",
    "emprendedores": "emprendedor",
    "emprendemos": "emprender",
    "emprenden": "emprender",
    "emprenderemos": "emprender",
    "emprenderá": "emprender",
    "emprenderán": "emprender",
    "emprenderás": "emprender",
    "emprenderé": "emprender",
    "emprenderéis": "emprender",
    "emprendería": "emprender",
    "emprenderíais": "emprender",
    "emprenderíamos": "emprender",
    "emprenderían": "emprender",
    "emprenderías": "emprender",
    "emprendes": "emprender",
    "emprendida": "emprender",
    "emprendidas": "emprender",
    "emprendido": "emprender",
    "emprendidos": "emprender",
    "emprendiendo": "emprender",
    "emprendiera": "emprender",
    "emprendierais": "emprender",
    "emprendieran": "emprender",
    "emprendieras": "emprender",
    "emprendiere": "emprender",
    "emprendiereis": "emprender",
    "emprendieren": "emprender",
    "emprendieres": "emprender",
    "emprendieron": "emprender",
    "emprendiese": "emprender",
    "emprendieseis": "emprender",
    "emprendiesen": "emprender",
    "emprendieses": "emprender",
    "emprendimientos": "emprendimiento",
    "emprendimos": "emprender",
    "emprendiste": "emprender",
    "emprendisteis": "emprender",
    "emprendiéramos": "emprender",
    "emprendiéremos": "emprender",
    "emprendiésemos": "emprender",
    "emprendió": "emprender",
    "emprendo": "emprender",
    "emprendáis": "emprender",
    "emprendéis": "emprender",
    "emprendí": "emprender",
    "emprendía": "emprender",
    "emprendíais": "emprender",
    "emprendíamos": "emprender",
    "emprendían": "emprender",
    "emprendías": "emprender",
    "empresaria": "empresario",
    "empresariados": "empresariado",
    "empresariales": "empresarial",
    "empresarias": "empresario",
    "empresarios": "empresario",
    "empresas": "empresa",
    "empreña": "empreñar",
    "empreñaba": "empreñar",
    "empreñabais": "empreñar",
    "empreñaban": "empreñar",
    "empreñabas": "empreñar",
    "empreñad": "empreñar",
    "empreñada": "empreñar",
    "empreñadas": "empreñar",
    "empreñado": "empreñar",
    "empreñadora": "empreñador",
    "empreñadoras": "empreñador",
    "empreñadores": "empreñador",
    "empreñados": "empreñar",
    "empreñamos": "empreñar",
    "empreñan": "empreñar",
    "empreñando": "empreñar",
    "empreñara": "empreñar",
    "empreñarais": "empreñar",
    "empreñaran": "empreñar",
    "empreñaras": "empreñar",
    "empreñare": "empreñar",
    "empreñareis": "empreñar",
    "empreñaremos": "empreñar",
    "empreñaren": "empreñar",
    "empreñares": "empreñar",
    "empreñaron": "empreñar",
    "empreñará": "empreñar",
    "empreñarán": "empreñar",
    "empreñarás": "empreñar",
    "empreñaré": "empreñar",
    "empreñaréis": "empreñar",
    "empreñaría": "empreñar",
    "empreñaríais": "empreñar",
    "empreñaríamos": "empreñar",
    "empreñarían": "empreñar",
    "empreñarías": "empreñar",
    "empreñas": "empreñar",
    "empreñase": "empreñar",
    "empreñaseis": "empreñar",
    "empreñasen": "empreñar",
    "empreñases": "empreñar",
    "empreñaste": "empreñar",
    "empreñasteis": "empreñar",
    "empreñe": "empreñar",
    "empreñemos": "empreñar",
    "empreñen": "empreñar",
    "empreñes": "empreñar",
    "empreño": "empreñar",
    "empreñábamos": "empreñar",
    "empreñáis": "empreñar",
    "empreñáramos": "empreñar",
    "empreñáremos": "empreñar",
    "empreñásemos": "empreñar",
    "empreñé": "empreñar",
    "empreñéis": "empreñar",
    "empreñó": "empreñar",
    "emprima": "emprimar",
    "emprimaba": "emprimar",
    "emprimabais": "emprimar",
    "emprimaban": "emprimar",
    "emprimabas": "emprimar",
    "emprimad": "emprimar",
    "emprimada": "emprimar",
    "emprimadas": "emprimar",
    "emprimado": "emprimar",
    "emprimados": "emprimar",
    "emprimamos": "emprimar",
    "empriman": "emprimar",
    "emprimando": "emprimar",
    "emprimara": "emprimar",
    "emprimarais": "emprimar",
    "emprimaran": "emprimar",
    "emprimaras": "emprimar",
    "emprimare": "emprimar",
    "emprimareis": "emprimar",
    "emprimaremos": "emprimar",
    "emprimaren": "emprimar",
    "emprimares": "emprimar",
    "emprimaron": "emprimar",
    "emprimará": "emprimar",
    "emprimarán": "emprimar",
    "emprimarás": "emprimar",
    "emprimaré": "emprimar",
    "emprimaréis": "emprimar",
    "emprimaría": "emprimar",
    "emprimaríais": "emprimar",
    "emprimaríamos": "emprimar",
    "emprimarían": "emprimar",
    "emprimarías": "emprimar",
    "emprimas": "emprimar",
    "emprimase": "emprimar",
    "emprimaseis": "emprimar",
    "emprimasen": "emprimar",
    "emprimases": "emprimar",
    "emprimaste": "emprimar",
    "emprimasteis": "emprimar",
    "emprime": "emprimar",
    "emprimemos": "emprimar",
    "emprimen": "emprimar",
    "emprimes": "emprimar",
    "emprimo": "emprimar",
    "emprimábamos": "emprimar",
    "emprimáis": "emprimar",
    "emprimáramos": "emprimar",
    "emprimáremos": "emprimar",
    "emprimásemos": "emprimar",
    "emprimé": "emprimar",
    "empriméis": "emprimar",
    "emprimó": "emprimar",
    "empringa": "empringar",
    "empringaba": "empringar",
    "empringabais": "empringar",
    "empringaban": "empringar",
    "empringabas": "empringar",
    "empringad": "empringar",
    "empringada": "empringar",
    "empringadas": "empringar",
    "empringado": "empringar",
    "empringados": "empringar",
    "empringamos": "empringar",
    "empringan": "empringar",
    "empringando": "empringar",
    "empringara": "empringar",
    "empringarais": "empringar",
    "empringaran": "empringar",
    "empringaras": "empringar",
    "empringare": "empringar",
    "empringareis": "empringar",
    "empringaremos": "empringar",
    "empringaren": "empringar",
    "empringares": "empringar",
    "empringaron": "empringar",
    "empringará": "empringar",
    "empringarán": "empringar",
    "empringarás": "empringar",
    "empringaré": "empringar",
    "empringaréis": "empringar",
    "empringaría": "empringar",
    "empringaríais": "empringar",
    "empringaríamos": "empringar",
    "empringarían": "empringar",
    "empringarías": "empringar",
    "empringas": "empringar",
    "empringase": "empringar",
    "empringaseis": "empringar",
    "empringasen": "empringar",
    "empringases": "empringar",
    "empringaste": "empringar",
    "empringasteis": "empringar",
    "empringo": "empringar",
    "empringue": "empringar",
    "empringuemos": "empringar",
    "empringuen": "empringar",
    "empringues": "empringar",
    "empringué": "empringar",
    "empringuéis": "empringar",
    "empringábamos": "empringar",
    "empringáis": "empringar",
    "empringáramos": "empringar",
    "empringáremos": "empringar",
    "empringásemos": "empringar",
    "empringó": "empringar",
    "empréstitos": "empréstito",
    "empuerca": "emporcar",
    "empuercan": "emporcar",
    "empuercas": "emporcar",
    "empuerco": "emporcar",
    "empuerque": "emporcar",
    "empuerquen": "emporcar",
    "empuerques": "emporcar",
    "empuja": "empujar",
    "empujaba": "empujar",
    "empujabais": "empujar",
    "empujaban": "empujar",
    "empujabas": "empujar",
    "empujad": "empujar",
    "empujada": "empujar",
    "empujadas": "empujar",
    "empujado": "empujar",
    "empujadora": "empujador",
    "empujadoras": "empujador",
    "empujadores": "empujador",
    "empujados": "empujar",
    "empujamientos": "empujamiento",
    "empujamos": "empujar",
    "empujan": "empujar",
    "empujando": "empujar",
    "empujara": "empujar",
    "empujarais": "empujar",
    "empujaran": "empujar",
    "empujaras": "empujar",
    "empujare": "empujar",
    "empujareis": "empujar",
    "empujaremos": "empujar",
    "empujaren": "empujar",
    "empujares": "empujar",
    "empujaron": "empujar",
    "empujará": "empujar",
    "empujarán": "empujar",
    "empujarás": "empujar",
    "empujaré": "empujar",
    "empujaréis": "empujar",
    "empujaría": "empujar",
    "empujaríais": "empujar",
    "empujaríamos": "empujar",
    "empujarían": "empujar",
    "empujarías": "empujar",
    "empujas": "empujar",
    "empujase": "empujar",
    "empujaseis": "empujar",
    "empujasen": "empujar",
    "empujases": "empujar",
    "empujaste": "empujar",
    "empujasteis": "empujar",
    "empuje": "empujar",
    "empujemos": "empujar",
    "empujen": "empujar",
    "empujes": "empuje",
    "empujo": "empujar",
    "empujones": "empujón",
    "empujábamos": "empujar",
    "empujáis": "empujar",
    "empujáramos": "empujar",
    "empujáremos": "empujar",
    "empujásemos": "empujar",
    "empujé": "empujar",
    "empujéis": "empujar",
    "empujó": "empujar",
    "empulga": "empulgar",
    "empulgaba": "empulgar",
    "empulgabais": "empulgar",
    "empulgaban": "empulgar",
    "empulgabas": "empulgar",
    "empulgad": "empulgar",
    "empulgada": "empulgar",
    "empulgadas": "empulgar",
    "empulgado": "empulgar",
    "empulgados": "empulgar",
    "empulgaduras": "empulgadura",
    "empulgamos": "empulgar",
    "empulgan": "empulgar",
    "empulgando": "empulgar",
    "empulgara": "empulgar",
    "empulgarais": "empulgar",
    "empulgaran": "empulgar",
    "empulgaras": "empulgar",
    "empulgare": "empulgar",
    "empulgareis": "empulgar",
    "empulgaremos": "empulgar",
    "empulgaren": "empulgar",
    "empulgares": "empulgar",
    "empulgaron": "empulgar",
    "empulgará": "empulgar",
    "empulgarán": "empulgar",
    "empulgarás": "empulgar",
    "empulgaré": "empulgar",
    "empulgaréis": "empulgar",
    "empulgaría": "empulgar",
    "empulgaríais": "empulgar",
    "empulgaríamos": "empulgar",
    "empulgarían": "empulgar",
    "empulgarías": "empulgar",
    "empulgas": "empulgar",
    "empulgase": "empulgar",
    "empulgaseis": "empulgar",
    "empulgasen": "empulgar",
    "empulgases": "empulgar",
    "empulgaste": "empulgar",
    "empulgasteis": "empulgar",
    "empulgo": "empulgar",
    "empulgue": "empulgar",
    "empulguemos": "empulgar",
    "empulguen": "empulgar",
    "empulgueras": "empulguera",
    "empulgues": "empulgar",
    "empulgué": "empulgar",
    "empulguéis": "empulgar",
    "empulgábamos": "empulgar",
    "empulgáis": "empulgar",
    "empulgáramos": "empulgar",
    "empulgáremos": "empulgar",
    "empulgásemos": "empulgar",
    "empulgó": "empulgar",
    "empurpurada": "empurpurado",
    "empurpuradas": "empurpurado",
    "empurpurados": "empurpurado",
    "empuña": "empuñar",
    "empuñaba": "empuñar",
    "empuñabais": "empuñar",
    "empuñaban": "empuñar",
    "empuñabas": "empuñar",
    "empuñad": "empuñar",
    "empuñada": "empuñar",
    "empuñadas": "empuñar",
    "empuñado": "empuñar",
    "empuñadora": "empuñador",
    "empuñadoras": "empuñador",
    "empuñadores": "empuñador",
    "empuñados": "empuñar",
    "empuñaduras": "empuñadura",
    "empuñamientos": "empuñamiento",
    "empuñamos": "empuñar",
    "empuñan": "empuñar",
    "empuñando": "empuñar",
    "empuñara": "empuñar",
    "empuñarais": "empuñar",
    "empuñaran": "empuñar",
    "empuñaras": "empuñar",
    "empuñare": "empuñar",
    "empuñareis": "empuñar",
    "empuñaremos": "empuñar",
    "empuñaren": "empuñar",
    "empuñares": "empuñar",
    "empuñaron": "empuñar",
    "empuñará": "empuñar",
    "empuñarán": "empuñar",
    "empuñarás": "empuñar",
    "empuñaré": "empuñar",
    "empuñaréis": "empuñar",
    "empuñaría": "empuñar",
    "empuñaríais": "empuñar",
    "empuñaríamos": "empuñar",
    "empuñarían": "empuñar",
    "empuñarías": "empuñar",
    "empuñas": "empuñar",
    "empuñase": "empuñar",
    "empuñaseis": "empuñar",
    "empuñasen": "empuñar",
    "empuñases": "empuñar",
    "empuñaste": "empuñar",
    "empuñasteis": "empuñar",
    "empuñe": "empuñar",
    "empuñemos": "empuñar",
    "empuñen": "empuñar",
    "empuñes": "empuñar",
    "empuñiduras": "empuñidura",
    "empuño": "empuñar",
    "empuñábamos": "empuñar",
    "empuñáis": "empuñar",
    "empuñáramos": "empuñar",
    "empuñáremos": "empuñar",
    "empuñásemos": "empuñar",
    "empuñé": "empuñar",
    "empuñéis": "empuñar",
    "empuñó": "empuñar",
    "empática": "empático",
    "empáticas": "empático",
    "empáticos": "empático",
    "empírea": "empíreo",
    "empíreas": "empíreo",
    "empíreos": "empíreo",
    "empírica": "empírico",
    "empíricas": "empírico",
    "empíricos": "empírico",
    "emula": "emular",
    "emulaba": "emular",
    "emulabais": "emular",
    "emulaban": "emular",
    "emulabas": "emular",
    "emulaciones": "emulación",
    "emulad": "emular",
    "emulada": "emular",
    "emuladas": "emular",
    "emulado": "emular",
    "emuladora": "emulador",
    "emuladoras": "emulador",
    "emuladores": "emulador",
    "emulados": "emular",
    "emulamos": "emular",
    "emulan": "emular",
    "emulando": "emular",
    "emulara": "emular",
    "emularais": "emular",
    "emularan": "emular",
    "emularas": "emular",
    "emulare": "emular",
    "emulareis": "emular",
    "emularemos": "emular",
    "emularen": "emular",
    "emulares": "emular",
    "emularon": "emular",
    "emulará": "emular",
    "emularán": "emular",
    "emularás": "emular",
    "emularé": "emular",
    "emularéis": "emular",
    "emularía": "emular",
    "emularíais": "emular",
    "emularíamos": "emular",
    "emularían": "emular",
    "emularías": "emular",
    "emulas": "emular",
    "emulase": "emular",
    "emulaseis": "emular",
    "emulasen": "emular",
    "emulases": "emular",
    "emulaste": "emular",
    "emulasteis": "emular",
    "emule": "emular",
    "emulemos": "emular",
    "emulen": "emular",
    "emules": "emular",
    "emulgentes": "emulgente",
    "emulo": "emular",
    "emulsiona": "emulsionar",
    "emulsionaba": "emulsionar",
    "emulsionabais": "emulsionar",
    "emulsionaban": "emulsionar",
    "emulsionabas": "emulsionar",
    "emulsionables": "emulsionable",
    "emulsionad": "emulsionar",
    "emulsionada": "emulsionar",
    "emulsionadas": "emulsionar",
    "emulsionado": "emulsionar",
    "emulsionados": "emulsionar",
    "emulsionamientos": "emulsionamiento",
    "emulsionamos": "emulsionar",
    "emulsionan": "emulsionar",
    "emulsionando": "emulsionar",
    "emulsionantes": "emulsionante",
    "emulsionara": "emulsionar",
    "emulsionarais": "emulsionar",
    "emulsionaran": "emulsionar",
    "emulsionaras": "emulsionar",
    "emulsionare": "emulsionar",
    "emulsionareis": "emulsionar",
    "emulsionaremos": "emulsionar",
    "emulsionaren": "emulsionar",
    "emulsionares": "emulsionar",
    "emulsionaron": "emulsionar",
    "emulsionará": "emulsionar",
    "emulsionarán": "emulsionar",
    "emulsionarás": "emulsionar",
    "emulsionaré": "emulsionar",
    "emulsionaréis": "emulsionar",
    "emulsionaría": "emulsionar",
    "emulsionaríais": "emulsionar",
    "emulsionaríamos": "emulsionar",
    "emulsionarían": "emulsionar",
    "emulsionarías": "emulsionar",
    "emulsionas": "emulsionar",
    "emulsionase": "emulsionar",
    "emulsionaseis": "emulsionar",
    "emulsionasen": "emulsionar",
    "emulsionases": "emulsionar",
    "emulsionaste": "emulsionar",
    "emulsionasteis": "emulsionar",
    "emulsione": "emulsionar",
    "emulsionemos": "emulsionar",
    "emulsionen": "emulsionar",
    "emulsiones": "emulsionar",
    "emulsiono": "emulsionar",
    "emulsionábamos": "emulsionar",
    "emulsionáis": "emulsionar",
    "emulsionáramos": "emulsionar",
    "emulsionáremos": "emulsionar",
    "emulsionásemos": "emulsionar",
    "emulsioné": "emulsionar",
    "emulsionéis": "emulsionar",
    "emulsionó": "emulsionar",
    "emulsiva": "emulsivo",
    "emulsivas": "emulsivo",
    "emulsivos": "emulsivo",
    "emulsores": "emulsor",
    "emulábamos": "emular",
    "emuláis": "emular",
    "emuláramos": "emular",
    "emuláremos": "emular",
    "emulásemos": "emular",
    "emulé": "emular",
    "emuléis": "emular",
    "emuló": "emular",
    "emunciones": "emunción",
    "emuntorios": "emuntorio",
    "emérita": "emérito",
    "eméritas": "emérito",
    "eméritos": "emérito",
    "emética": "emético",
    "eméticas": "emético",
    "eméticos": "emético",
    "emétropes": "emétrope",
    "emídidos": "emídido",
    "emúes": "emú",
    "emús": "emú",
    "enaceita": "enaceitar",
    "enaceitaba": "enaceitar",
    "enaceitabais": "enaceitar",
    "enaceitaban": "enaceitar",
    "enaceitabas": "enaceitar",
    "enaceitad": "enaceitar",
    "enaceitada": "enaceitar",
    "enaceitadas": "enaceitar",
    "enaceitado": "enaceitar",
    "enaceitados": "enaceitar",
    "enaceitamos": "enaceitar",
    "enaceitan": "enaceitar",
    "enaceitando": "enaceitar",
    "enaceitara": "enaceitar",
    "enaceitarais": "enaceitar",
    "enaceitaran": "enaceitar",
    "enaceitaras": "enaceitar",
    "enaceitare": "enaceitar",
    "enaceitareis": "enaceitar",
    "enaceitaremos": "enaceitar",
    "enaceitaren": "enaceitar",
    "enaceitares": "enaceitar",
    "enaceitaron": "enaceitar",
    "enaceitará": "enaceitar",
    "enaceitarán": "enaceitar",
    "enaceitarás": "enaceitar",
    "enaceitaré": "enaceitar",
    "enaceitaréis": "enaceitar",
    "enaceitaría": "enaceitar",
    "enaceitaríais": "enaceitar",
    "enaceitaríamos": "enaceitar",
    "enaceitarían": "enaceitar",
    "enaceitarías": "enaceitar",
    "enaceitas": "enaceitar",
    "enaceitase": "enaceitar",
    "enaceitaseis": "enaceitar",
    "enaceitasen": "enaceitar",
    "enaceitases": "enaceitar",
    "enaceitaste": "enaceitar",
    "enaceitasteis": "enaceitar",
    "enaceite": "enaceitar",
    "enaceitemos": "enaceitar",
    "enaceiten": "enaceitar",
    "enaceites": "enaceitar",
    "enaceito": "enaceitar",
    "enaceitábamos": "enaceitar",
    "enaceitáis": "enaceitar",
    "enaceitáramos": "enaceitar",
    "enaceitáremos": "enaceitar",
    "enaceitásemos": "enaceitar",
    "enaceité": "enaceitar",
    "enaceitéis": "enaceitar",
    "enaceitó": "enaceitar",
    "enagua": "enaguar",
    "enaguaba": "enaguar",
    "enaguabais": "enaguar",
    "enaguaban": "enaguar",
    "enaguabas": "enaguar",
    "enaguace": "enaguazar",
    "enaguacemos": "enaguazar",
    "enaguacen": "enaguazar",
    "enaguaces": "enaguazar",
    "enaguacha": "enaguachar",
    "enaguachaba": "enaguachar",
    "enaguachabais": "enaguachar",
    "enaguachaban": "enaguachar",
    "enaguachabas": "enaguachar",
    "enaguachad": "enaguachar",
    "enaguachada": "enaguachar",
    "enaguachadas": "enaguachar",
    "enaguachado": "enaguachar",
    "enaguachados": "enaguachar",
    "enaguachamos": "enaguachar",
    "enaguachan": "enaguachar",
    "enaguachando": "enaguachar",
    "enaguachara": "enaguachar",
    "enaguacharais": "enaguachar",
    "enaguacharan": "enaguachar",
    "enaguacharas": "enaguachar",
    "enaguachare": "enaguachar",
    "enaguachareis": "enaguachar",
    "enaguacharemos": "enaguachar",
    "enaguacharen": "enaguachar",
    "enaguachares": "enaguachar",
    "enaguacharon": "enaguachar",
    "enaguachará": "enaguachar",
    "enaguacharán": "enaguachar",
    "enaguacharás": "enaguachar",
    "enaguacharé": "enaguachar",
    "enaguacharéis": "enaguachar",
    "enaguacharía": "enaguachar",
    "enaguacharíais": "enaguachar",
    "enaguacharíamos": "enaguachar",
    "enaguacharían": "enaguachar",
    "enaguacharías": "enaguachar",
    "enaguachas": "enaguachar",
    "enaguachase": "enaguachar",
    "enaguachaseis": "enaguachar",
    "enaguachasen": "enaguachar",
    "enaguachases": "enaguachar",
    "enaguachaste": "enaguachar",
    "enaguachasteis": "enaguachar",
    "enaguache": "enaguachar",
    "enaguachemos": "enaguachar",
    "enaguachen": "enaguachar",
    "enaguaches": "enaguachar",
    "enaguacho": "enaguachar",
    "enaguachábamos": "enaguachar",
    "enaguacháis": "enaguachar",
    "enaguacháramos": "enaguachar",
    "enaguacháremos": "enaguachar",
    "enaguachásemos": "enaguachar",
    "enaguaché": "enaguachar",
    "enaguachéis": "enaguachar",
    "enaguachó": "enaguachar",
    "enaguacé": "enaguazar",
    "enaguacéis": "enaguazar",
    "enaguad": "enaguar",
    "enaguada": "enaguar",
    "enaguadas": "enaguar",
    "enaguado": "enaguar",
    "enaguados": "enaguar",
    "enaguamos": "enaguar",
    "enaguan": "enaguar",
    "enaguando": "enaguar",
    "enaguara": "enaguar",
    "enaguarais": "enaguar",
    "enaguaran": "enaguar",
    "enaguaras": "enaguar",
    "enaguare": "enaguar",
    "enaguareis": "enaguar",
    "enaguaremos": "enaguar",
    "enaguaren": "enaguar",
    "enaguares": "enaguar",
    "enaguaron": "enaguar",
    "enaguará": "enaguar",
    "enaguarán": "enaguar",
    "enaguarás": "enaguar",
    "enaguaré": "enaguar",
    "enaguaréis": "enaguar",
    "enaguaría": "enaguar",
    "enaguaríais": "enaguar",
    "enaguaríamos": "enaguar",
    "enaguarían": "enaguar",
    "enaguarías": "enaguar",
    "enaguas": "enaguar",
    "enaguase": "enaguar",
    "enaguaseis": "enaguar",
    "enaguasen": "enaguar",
    "enaguases": "enaguar",
    "enaguaste": "enaguar",
    "enaguasteis": "enaguar",
    "enaguaza": "enaguazar",
    "enaguazaba": "enaguazar",
    "enaguazabais": "enaguazar",
    "enaguazaban": "enaguazar",
    "enaguazabas": "enaguazar",
    "enaguazad": "enaguazar",
    "enaguazada": "enaguazar",
    "enaguazadas": "enaguazar",
    "enaguazado": "enaguazar",
    "enaguazados": "enaguazar",
    "enaguazamos": "enaguazar",
    "enaguazan": "enaguazar",
    "enaguazando": "enaguazar",
    "enaguazara": "enaguazar",
    "enaguazarais": "enaguazar",
    "enaguazaran": "enaguazar",
    "enaguazaras": "enaguazar",
    "enaguazare": "enaguazar",
    "enaguazareis": "enaguazar",
    "enaguazaremos": "enaguazar",
    "enaguazaren": "enaguazar",
    "enaguazares": "enaguazar",
    "enaguazaron": "enaguazar",
    "enaguazará": "enaguazar",
    "enaguazarán": "enaguazar",
    "enaguazarás": "enaguazar",
    "enaguazaré": "enaguazar",
    "enaguazaréis": "enaguazar",
    "enaguazaría": "enaguazar",
    "enaguazaríais": "enaguazar",
    "enaguazaríamos": "enaguazar",
    "enaguazarían": "enaguazar",
    "enaguazarías": "enaguazar",
    "enaguazas": "enaguazar",
    "enaguazase": "enaguazar",
    "enaguazaseis": "enaguazar",
    "enaguazasen": "enaguazar",
    "enaguazases": "enaguazar",
    "enaguazaste": "enaguazar",
    "enaguazasteis": "enaguazar",
    "enaguazo": "enaguazar",
    "enaguazábamos": "enaguazar",
    "enaguazáis": "enaguazar",
    "enaguazáramos": "enaguazar",
    "enaguazáremos": "enaguazar",
    "enaguazásemos": "enaguazar",
    "enaguazó": "enaguazar",
    "enaguo": "enaguar",
    "enaguábamos": "enaguar",
    "enaguáis": "enaguar",
    "enaguáramos": "enaguar",
    "enaguáremos": "enaguar",
    "enaguásemos": "enaguar",
    "enaguó": "enaguar",
    "enagüe": "enaguar",
    "enagüemos": "enaguar",
    "enagüen": "enaguar",
    "enagües": "enaguar",
    "enagüé": "enaguar",
    "enagüéis": "enaguar",
    "enajena": "enajenar",
    "enajenaba": "enajenar",
    "enajenabais": "enajenar",
    "enajenaban": "enajenar",
    "enajenabas": "enajenar",
    "enajenables": "enajenable",
    "enajenaciones": "enajenación",
    "enajenad": "enajenar",
    "enajenada": "enajenar",
    "enajenadas": "enajenar",
    "enajenado": "enajenar",
    "enajenadora": "enajenador",
    "enajenadoras": "enajenador",
    "enajenadores": "enajenador",
    "enajenados": "enajenar",
    "enajenamientos": "enajenamiento",
    "enajenamos": "enajenar",
    "enajenan": "enajenar",
    "enajenando": "enajenar",
    "enajenantes": "enajenante",
    "enajenara": "enajenar",
    "enajenarais": "enajenar",
    "enajenaran": "enajenar",
    "enajenaras": "enajenar",
    "enajenare": "enajenar",
    "enajenareis": "enajenar",
    "enajenaremos": "enajenar",
    "enajenaren": "enajenar",
    "enajenares": "enajenar",
    "enajenaron": "enajenar",
    "enajenará": "enajenar",
    "enajenarán": "enajenar",
    "enajenarás": "enajenar",
    "enajenaré": "enajenar",
    "enajenaréis": "enajenar",
    "enajenaría": "enajenar",
    "enajenaríais": "enajenar",
    "enajenaríamos": "enajenar",
    "enajenarían": "enajenar",
    "enajenarías": "enajenar",
    "enajenas": "enajenar",
    "enajenase": "enajenar",
    "enajenaseis": "enajenar",
    "enajenasen": "enajenar",
    "enajenases": "enajenar",
    "enajenaste": "enajenar",
    "enajenasteis": "enajenar",
    "enajene": "enajenar",
    "enajenemos": "enajenar",
    "enajenen": "enajenar",
    "enajenes": "enajenar",
    "enajeno": "enajenar",
    "enajenábamos": "enajenar",
    "enajenáis": "enajenar",
    "enajenáramos": "enajenar",
    "enajenáremos": "enajenar",
    "enajenásemos": "enajenar",
    "enajené": "enajenar",
    "enajenéis": "enajenar",
    "enajenó": "enajenar",
    "enalba": "enalbar",
    "enalbaba": "enalbar",
    "enalbabais": "enalbar",
    "enalbaban": "enalbar",
    "enalbabas": "enalbar",
    "enalbad": "enalbar",
    "enalbada": "enalbar",
    "enalbadas": "enalbar",
    "enalbado": "enalbar",
    "enalbados": "enalbar",
    "enalbamos": "enalbar",
    "enalban": "enalbar",
    "enalbando": "enalbar",
    "enalbara": "enalbar",
    "enalbarais": "enalbar",
    "enalbaran": "enalbar",
    "enalbaras": "enalbar",
    "enalbarda": "enalbardar",
    "enalbardaba": "enalbardar",
    "enalbardabais": "enalbardar",
    "enalbardaban": "enalbardar",
    "enalbardabas": "enalbardar",
    "enalbardad": "enalbardar",
    "enalbardada": "enalbardar",
    "enalbardadas": "enalbardar",
    "enalbardado": "enalbardar",
    "enalbardados": "enalbardar",
    "enalbardamos": "enalbardar",
    "enalbardan": "enalbardar",
    "enalbardando": "enalbardar",
    "enalbardara": "enalbardar",
    "enalbardarais": "enalbardar",
    "enalbardaran": "enalbardar",
    "enalbardaras": "enalbardar",
    "enalbardare": "enalbardar",
    "enalbardareis": "enalbardar",
    "enalbardaremos": "enalbardar",
    "enalbardaren": "enalbardar",
    "enalbardares": "enalbardar",
    "enalbardaron": "enalbardar",
    "enalbardará": "enalbardar",
    "enalbardarán": "enalbardar",
    "enalbardarás": "enalbardar",
    "enalbardaré": "enalbardar",
    "enalbardaréis": "enalbardar",
    "enalbardaría": "enalbardar",
    "enalbardaríais": "enalbardar",
    "enalbardaríamos": "enalbardar",
    "enalbardarían": "enalbardar",
    "enalbardarías": "enalbardar",
    "enalbardas": "enalbardar",
    "enalbardase": "enalbardar",
    "enalbardaseis": "enalbardar",
    "enalbardasen": "enalbardar",
    "enalbardases": "enalbardar",
    "enalbardaste": "enalbardar",
    "enalbardasteis": "enalbardar",
    "enalbarde": "enalbardar",
    "enalbardemos": "enalbardar",
    "enalbarden": "enalbardar",
    "enalbardes": "enalbardar",
    "enalbardo": "enalbardar",
    "enalbardábamos": "enalbardar",
    "enalbardáis": "enalbardar",
    "enalbardáramos": "enalbardar",
    "enalbardáremos": "enalbardar",
    "enalbardásemos": "enalbardar",
    "enalbardé": "enalbardar",
    "enalbardéis": "enalbardar",
    "enalbardó": "enalbardar",
    "enalbare": "enalbar",
    "enalbareis": "enalbar",
    "enalbaremos": "enalbar",
    "enalbaren": "enalbar",
    "enalbares": "enalbar",
    "enalbaron": "enalbar",
    "enalbará": "enalbar",
    "enalbarán": "enalbar",
    "enalbarás": "enalbar",
    "enalbaré": "enalbar",
    "enalbaréis": "enalbar",
    "enalbaría": "enalbar",
    "enalbaríais": "enalbar",
    "enalbaríamos": "enalbar",
    "enalbarían": "enalbar",
    "enalbarías": "enalbar",
    "enalbas": "enalbar",
    "enalbase": "enalbar",
    "enalbaseis": "enalbar",
    "enalbasen": "enalbar",
    "enalbases": "enalbar",
    "enalbaste": "enalbar",
    "enalbasteis": "enalbar",
    "enalbe": "enalbar",
    "enalbemos": "enalbar",
    "enalben": "enalbar",
    "enalbes": "enalbar",
    "enalbo": "enalbar",
    "enalbábamos": "enalbar",
    "enalbáis": "enalbar",
    "enalbáramos": "enalbar",
    "enalbáremos": "enalbar",
    "enalbásemos": "enalbar",
    "enalbé": "enalbar",
    "enalbéis": "enalbar",
    "enalbó": "enalbar",
    "enalmagrada": "enalmagrado",
    "enalmagradas": "enalmagrado",
    "enalmagrados": "enalmagrado",
    "enaltece": "enaltecer",
    "enalteced": "enaltecer",
    "enaltecedora": "enaltecedor",
    "enaltecedoras": "enaltecedor",
    "enaltecedores": "enaltecedor",
    "enaltecemos": "enaltecer",
    "enaltecen": "enaltecer",
    "enalteceremos": "enaltecer",
    "enaltecerá": "enaltecer",
    "enaltecerán": "enaltecer",
    "enaltecerás": "enaltecer",
    "enalteceré": "enaltecer",
    "enalteceréis": "enaltecer",
    "enaltecería": "enaltecer",
    "enalteceríais": "enaltecer",
    "enalteceríamos": "enaltecer",
    "enaltecerían": "enaltecer",
    "enaltecerías": "enaltecer",
    "enalteces": "enaltecer",
    "enaltecida": "enaltecer",
    "enaltecidas": "enaltecer",
    "enaltecido": "enaltecer",
    "enaltecidos": "enaltecer",
    "enalteciendo": "enaltecer",
    "enalteciera": "enaltecer",
    "enaltecierais": "enaltecer",
    "enaltecieran": "enaltecer",
    "enaltecieras": "enaltecer",
    "enalteciere": "enaltecer",
    "enalteciereis": "enaltecer",
    "enaltecieren": "enaltecer",
    "enaltecieres": "enaltecer",
    "enaltecieron": "enaltecer",
    "enalteciese": "enaltecer",
    "enaltecieseis": "enaltecer",
    "enalteciesen": "enaltecer",
    "enaltecieses": "enaltecer",
    "enaltecimientos": "enaltecimiento",
    "enaltecimos": "enaltecer",
    "enalteciste": "enaltecer",
    "enaltecisteis": "enaltecer",
    "enalteciéramos": "enaltecer",
    "enalteciéremos": "enaltecer",
    "enalteciésemos": "enaltecer",
    "enalteció": "enaltecer",
    "enaltecéis": "enaltecer",
    "enaltecí": "enaltecer",
    "enaltecía": "enaltecer",
    "enaltecíais": "enaltecer",
    "enaltecíamos": "enaltecer",
    "enaltecían": "enaltecer",
    "enaltecías": "enaltecer",
    "enaltezca": "enaltecer",
    "enaltezcamos": "enaltecer",
    "enaltezcan": "enaltecer",
    "enaltezcas": "enaltecer",
    "enaltezco": "enaltecer",
    "enaltezcáis": "enaltecer",
    "enamelomas": "enameloma",
    "enamora": "enamorar",
    "enamoraba": "enamorar",
    "enamorabais": "enamorar",
    "enamoraban": "enamorar",
    "enamorabas": "enamorar",
    "enamorad": "enamorar",
    "enamorada": "enamorar",
    "enamoradas": "enamorar",
    "enamoradiza": "enamoradizo",
    "enamoradizas": "enamoradizo",
    "enamoradizos": "enamoradizo",
    "enamorado": "enamorar",
    "enamoradora": "enamorador",
    "enamoradoras": "enamorador",
    "enamoradores": "enamorador",
    "enamorados": "enamorar",
    "enamoramientos": "enamoramiento",
    "enamoramos": "enamorar",
    "enamoran": "enamorar",
    "enamorando": "enamorar",
    "enamorara": "enamorar",
    "enamorarais": "enamorar",
    "enamoraran": "enamorar",
    "enamoraras": "enamorar",
    "enamorare": "enamorar",
    "enamorareis": "enamorar",
    "enamoraremos": "enamorar",
    "enamoraren": "enamorar",
    "enamorares": "enamorar",
    "enamoraron": "enamorar",
    "enamorará": "enamorar",
    "enamorarán": "enamorar",
    "enamorarás": "enamorar",
    "enamoraré": "enamorar",
    "enamoraréis": "enamorar",
    "enamoraría": "enamorar",
    "enamoraríais": "enamorar",
    "enamoraríamos": "enamorar",
    "enamorarían": "enamorar",
    "enamorarías": "enamorar",
    "enamoras": "enamorar",
    "enamorase": "enamorar",
    "enamoraseis": "enamorar",
    "enamorasen": "enamorar",
    "enamorases": "enamorar",
    "enamoraste": "enamorar",
    "enamorasteis": "enamorar",
    "enamore": "enamorar",
    "enamoremos": "enamorar",
    "enamoren": "enamorar",
    "enamores": "enamorar",
    "enamorica": "enamoricar",
    "enamoricaba": "enamoricar",
    "enamoricabais": "enamoricar",
    "enamoricaban": "enamoricar",
    "enamoricabas": "enamoricar",
    "enamoricad": "enamoricar",
    "enamoricada": "enamoricar",
    "enamoricadas": "enamoricar",
    "enamoricado": "enamoricar",
    "enamoricados": "enamoricar",
    "enamoricamos": "enamoricar",
    "enamorican": "enamoricar",
    "enamoricando": "enamoricar",
    "enamoricara": "enamoricar",
    "enamoricarais": "enamoricar",
    "enamoricaran": "enamoricar",
    "enamoricaras": "enamoricar",
    "enamoricare": "enamoricar",
    "enamoricareis": "enamoricar",
    "enamoricaremos": "enamoricar",
    "enamoricaren": "enamoricar",
    "enamoricares": "enamoricar",
    "enamoricaron": "enamoricar",
    "enamoricará": "enamoricar",
    "enamoricarán": "enamoricar",
    "enamoricarás": "enamoricar",
    "enamoricaré": "enamoricar",
    "enamoricaréis": "enamoricar",
    "enamoricaría": "enamoricar",
    "enamoricaríais": "enamoricar",
    "enamoricaríamos": "enamoricar",
    "enamoricarían": "enamoricar",
    "enamoricarías": "enamoricar",
    "enamoricas": "enamoricar",
    "enamoricase": "enamoricar",
    "enamoricaseis": "enamoricar",
    "enamoricasen": "enamoricar",
    "enamoricases": "enamoricar",
    "enamoricaste": "enamoricar",
    "enamoricasteis": "enamoricar",
    "enamorico": "enamoricar",
    "enamoricábamos": "enamoricar",
    "enamoricáis": "enamoricar",
    "enamoricáramos": "enamoricar",
    "enamoricáremos": "enamoricar",
    "enamoricásemos": "enamoricar",
    "enamoricó": "enamoricar",
    "enamorique": "enamoricar",
    "enamoriquemos": "enamoricar",
    "enamoriquen": "enamoricar",
    "enamoriques": "enamoricar",
    "enamoriqué": "enamoricar",
    "enamoriquéis": "enamoricar",
    "enamorisca": "enamoriscar",
    "enamoriscaba": "enamoriscar",
    "enamoriscabais": "enamoriscar",
    "enamoriscaban": "enamoriscar",
    "enamoriscabas": "enamoriscar",
    "enamoriscad": "enamoriscar",
    "enamoriscada": "enamoriscar",
    "enamoriscadas": "enamoriscar",
    "enamoriscado": "enamoriscar",
    "enamoriscados": "enamoriscar",
    "enamoriscamientos": "enamoriscamiento",
    "enamoriscamos": "enamoriscar",
    "enamoriscan": "enamoriscar",
    "enamoriscando": "enamoriscar",
    "enamoriscara": "enamoriscar",
    "enamoriscarais": "enamoriscar",
    "enamoriscaran": "enamoriscar",
    "enamoriscaras": "enamoriscar",
    "enamoriscare": "enamoriscar",
    "enamoriscareis": "enamoriscar",
    "enamoriscaremos": "enamoriscar",
    "enamoriscaren": "enamoriscar",
    "enamoriscares": "enamoriscar",
    "enamoriscaron": "enamoriscar",
    "enamoriscará": "enamoriscar",
    "enamoriscarán": "enamoriscar",
    "enamoriscarás": "enamoriscar",
    "enamoriscaré": "enamoriscar",
    "enamoriscaréis": "enamoriscar",
    "enamoriscaría": "enamoriscar",
    "enamoriscaríais": "enamoriscar",
    "enamoriscaríamos": "enamoriscar",
    "enamoriscarían": "enamoriscar",
    "enamoriscarías": "enamoriscar",
    "enamoriscas": "enamoriscar",
    "enamoriscase": "enamoriscar",
    "enamoriscaseis": "enamoriscar",
    "enamoriscasen": "enamoriscar",
    "enamoriscases": "enamoriscar",
    "enamoriscaste": "enamoriscar",
    "enamoriscasteis": "enamoriscar",
    "enamorisco": "enamoriscar",
    "enamoriscábamos": "enamoriscar",
    "enamoriscáis": "enamoriscar",
    "enamoriscáramos": "enamoriscar",
    "enamoriscáremos": "enamoriscar",
    "enamoriscásemos": "enamoriscar",
    "enamoriscó": "enamoriscar",
    "enamorisque": "enamoriscar",
    "enamorisquemos": "enamoriscar",
    "enamorisquen": "enamoriscar",
    "enamorisques": "enamoriscar",
    "enamorisqué": "enamoriscar",
    "enamorisquéis": "enamoriscar",
    "enamoro": "enamorar",
    "enamorábamos": "enamorar",
    "enamoráis": "enamorar",
    "enamoráramos": "enamorar",
    "enamoráremos": "enamorar",
    "enamorásemos": "enamorar",
    "enamoré": "enamorar",
    "enamoréis": "enamorar",
    "enamoró": "enamorar",
    "enana": "enano",
    "enanas": "enano",
    "enancha": "enanchar",
    "enanchaba": "enanchar",
    "enanchabais": "enanchar",
    "enanchaban": "enanchar",
    "enanchabas": "enanchar",
    "enanchad": "enanchar",
    "enanchada": "enanchar",
    "enanchadas": "enanchar",
    "enanchado": "enanchar",
    "enanchados": "enanchar",
    "enanchamos": "enanchar",
    "enanchan": "enanchar",
    "enanchando": "enanchar",
    "enanchara": "enanchar",
    "enancharais": "enanchar",
    "enancharan": "enanchar",
    "enancharas": "enanchar",
    "enanchare": "enanchar",
    "enanchareis": "enanchar",
    "enancharemos": "enanchar",
    "enancharen": "enanchar",
    "enanchares": "enanchar",
    "enancharon": "enanchar",
    "enanchará": "enanchar",
    "enancharán": "enanchar",
    "enancharás": "enanchar",
    "enancharé": "enanchar",
    "enancharéis": "enanchar",
    "enancharía": "enanchar",
    "enancharíais": "enanchar",
    "enancharíamos": "enanchar",
    "enancharían": "enanchar",
    "enancharías": "enanchar",
    "enanchas": "enanchar",
    "enanchase": "enanchar",
    "enanchaseis": "enanchar",
    "enanchasen": "enanchar",
    "enanchases": "enanchar",
    "enanchaste": "enanchar",
    "enanchasteis": "enanchar",
    "enanche": "enanchar",
    "enanchemos": "enanchar",
    "enanchen": "enanchar",
    "enanches": "enanchar",
    "enancho": "enanchar",
    "enanchábamos": "enanchar",
    "enancháis": "enanchar",
    "enancháramos": "enanchar",
    "enancháremos": "enanchar",
    "enanchásemos": "enanchar",
    "enanché": "enanchar",
    "enanchéis": "enanchar",
    "enanchó": "enanchar",
    "enangosta": "enangostar",
    "enangostaba": "enangostar",
    "enangostabais": "enangostar",
    "enangostaban": "enangostar",
    "enangostabas": "enangostar",
    "enangostad": "enangostar",
    "enangostada": "enangostar",
    "enangostadas": "enangostar",
    "enangostado": "enangostar",
    "enangostados": "enangostar",
    "enangostamos": "enangostar",
    "enangostan": "enangostar",
    "enangostando": "enangostar",
    "enangostara": "enangostar",
    "enangostarais": "enangostar",
    "enangostaran": "enangostar",
    "enangostaras": "enangostar",
    "enangostare": "enangostar",
    "enangostareis": "enangostar",
    "enangostaremos": "enangostar",
    "enangostaren": "enangostar",
    "enangostares": "enangostar",
    "enangostaron": "enangostar",
    "enangostará": "enangostar",
    "enangostarán": "enangostar",
    "enangostarás": "enangostar",
    "enangostaré": "enangostar",
    "enangostaréis": "enangostar",
    "enangostaría": "enangostar",
    "enangostaríais": "enangostar",
    "enangostaríamos": "enangostar",
    "enangostarían": "enangostar",
    "enangostarías": "enangostar",
    "enangostas": "enangostar",
    "enangostase": "enangostar",
    "enangostaseis": "enangostar",
    "enangostasen": "enangostar",
    "enangostases": "enangostar",
    "enangostaste": "enangostar",
    "enangostasteis": "enangostar",
    "enangoste": "enangostar",
    "enangostemos": "enangostar",
    "enangosten": "enangostar",
    "enangostes": "enangostar",
    "enangosto": "enangostar",
    "enangostábamos": "enangostar",
    "enangostáis": "enangostar",
    "enangostáramos": "enangostar",
    "enangostáremos": "enangostar",
    "enangostásemos": "enangostar",
    "enangosté": "enangostar",
    "enangostéis": "enangostar",
    "enangostó": "enangostar",
    "enanismos": "enanismo",
    "enanos": "enano",
    "enantatos": "enantato",
    "enantemas": "enantema",
    "enantes": "enante",
    "enantiomorfa": "enantiomorfo",
    "enantiomorfas": "enantiomorfo",
    "enantiomorfismos": "enantiomorfismo",
    "enantiomorfos": "enantiomorfo",
    "enantiotrópica": "enantiotrópico",
    "enantiotrópicas": "enantiotrópico",
    "enantiotrópicos": "enantiotrópico",
    "enantiómeros": "enantiómero",
    "enantiótropa": "enantiótropo",
    "enantiótropas": "enantiótropo",
    "enantiótropos": "enantiótropo",
    "enarbola": "enarbolar",
    "enarbolaba": "enarbolar",
    "enarbolabais": "enarbolar",
    "enarbolaban": "enarbolar",
    "enarbolabas": "enarbolar",
    "enarbolad": "enarbolar",
    "enarbolada": "enarbolar",
    "enarboladas": "enarbolar",
    "enarbolado": "enarbolar",
    "enarbolados": "enarbolar",
    "enarbolamos": "enarbolar",
    "enarbolan": "enarbolar",
    "enarbolando": "enarbolar",
    "enarbolara": "enarbolar",
    "enarbolarais": "enarbolar",
    "enarbolaran": "enarbolar",
    "enarbolaras": "enarbolar",
    "enarbolare": "enarbolar",
    "enarbolareis": "enarbolar",
    "enarbolaremos": "enarbolar",
    "enarbolaren": "enarbolar",
    "enarbolares": "enarbolar",
    "enarbolaron": "enarbolar",
    "enarbolará": "enarbolar",
    "enarbolarán": "enarbolar",
    "enarbolarás": "enarbolar",
    "enarbolaré": "enarbolar",
    "enarbolaréis": "enarbolar",
    "enarbolaría": "enarbolar",
    "enarbolaríais": "enarbolar",
    "enarbolaríamos": "enarbolar",
    "enarbolarían": "enarbolar",
    "enarbolarías": "enarbolar",
    "enarbolas": "enarbolar",
    "enarbolase": "enarbolar",
    "enarbolaseis": "enarbolar",
    "enarbolasen": "enarbolar",
    "enarbolases": "enarbolar",
    "enarbolaste": "enarbolar",
    "enarbolasteis": "enarbolar",
    "enarbole": "enarbolar",
    "enarbolemos": "enarbolar",
    "enarbolen": "enarbolar",
    "enarboles": "enarbolar",
    "enarbolo": "enarbolar",
    "enarbolábamos": "enarbolar",
    "enarboláis": "enarbolar",
    "enarboláramos": "enarbolar",
    "enarboláremos": "enarbolar",
    "enarbolásemos": "enarbolar",
    "enarbolé": "enarbolar",
    "enarboléis": "enarbolar",
    "enarboló": "enarbolar",
    "enarca": "enarcar",
    "enarcaba": "enarcar",
    "enarcabais": "enarcar",
    "enarcaban": "enarcar",
    "enarcabas": "enarcar",
    "enarcad": "enarcar",
    "enarcada": "enarcar",
    "enarcadas": "enarcar",
    "enarcado": "enarcar",
    "enarcados": "enarcar",
    "enarcamientos": "enarcamiento",
    "enarcamos": "enarcar",
    "enarcan": "enarcar",
    "enarcando": "enarcar",
    "enarcara": "enarcar",
    "enarcarais": "enarcar",
    "enarcaran": "enarcar",
    "enarcaras": "enarcar",
    "enarcare": "enarcar",
    "enarcareis": "enarcar",
    "enarcaremos": "enarcar",
    "enarcaren": "enarcar",
    "enarcares": "enarcar",
    "enarcaron": "enarcar",
    "enarcará": "enarcar",
    "enarcarán": "enarcar",
    "enarcarás": "enarcar",
    "enarcaré": "enarcar",
    "enarcaréis": "enarcar",
    "enarcaría": "enarcar",
    "enarcaríais": "enarcar",
    "enarcaríamos": "enarcar",
    "enarcarían": "enarcar",
    "enarcarías": "enarcar",
    "enarcas": "enarcar",
    "enarcase": "enarcar",
    "enarcaseis": "enarcar",
    "enarcasen": "enarcar",
    "enarcases": "enarcar",
    "enarcaste": "enarcar",
    "enarcasteis": "enarcar",
    "enarco": "enarcar",
    "enarcábamos": "enarcar",
    "enarcáis": "enarcar",
    "enarcáramos": "enarcar",
    "enarcáremos": "enarcar",
    "enarcásemos": "enarcar",
    "enarcó": "enarcar",
    "enardece": "enardecer",
    "enardeced": "enardecer",
    "enardecedora": "enardecedor",
    "enardecedoras": "enardecedor",
    "enardecedores": "enardecedor",
    "enardecemos": "enardecer",
    "enardecen": "enardecer",
    "enardeceremos": "enardecer",
    "enardecerá": "enardecer",
    "enardecerán": "enardecer",
    "enardecerás": "enardecer",
    "enardeceré": "enardecer",
    "enardeceréis": "enardecer",
    "enardecería": "enardecer",
    "enardeceríais": "enardecer",
    "enardeceríamos": "enardecer",
    "enardecerían": "enardecer",
    "enardecerías": "enardecer",
    "enardeces": "enardecer",
    "enardecida": "enardecer",
    "enardecidas": "enardecer",
    "enardecido": "enardecer",
    "enardecidos": "enardecer",
    "enardeciendo": "enardecer",
    "enardeciera": "enardecer",
    "enardecierais": "enardecer",
    "enardecieran": "enardecer",
    "enardecieras": "enardecer",
    "enardeciere": "enardecer",
    "enardeciereis": "enardecer",
    "enardecieren": "enardecer",
    "enardecieres": "enardecer",
    "enardecieron": "enardecer",
    "enardeciese": "enardecer",
    "enardecieseis": "enardecer",
    "enardeciesen": "enardecer",
    "enardecieses": "enardecer",
    "enardecimientos": "enardecimiento",
    "enardecimos": "enardecer",
    "enardeciste": "enardecer",
    "enardecisteis": "enardecer",
    "enardeciéramos": "enardecer",
    "enardeciéremos": "enardecer",
    "enardeciésemos": "enardecer",
    "enardeció": "enardecer",
    "enardecéis": "enardecer",
    "enardecí": "enardecer",
    "enardecía": "enardecer",
    "enardecíais": "enardecer",
    "enardecíamos": "enardecer",
    "enardecían": "enardecer",
    "enardecías": "enardecer",
    "enardezca": "enardecer",
    "enardezcamos": "enardecer",
    "enardezcan": "enardecer",
    "enardezcas": "enardecer",
    "enardezco": "enardecer",
    "enardezcáis": "enardecer",
    "enarena": "enarenar",
    "enarenaba": "enarenar",
    "enarenabais": "enarenar",
    "enarenaban": "enarenar",
    "enarenabas": "enarenar",
    "enarenaciones": "enarenación",
    "enarenad": "enarenar",
    "enarenada": "enarenar",
    "enarenadas": "enarenar",
    "enarenado": "enarenar",
    "enarenados": "enarenar",
    "enarenamos": "enarenar",
    "enarenan": "enarenar",
    "enarenando": "enarenar",
    "enarenara": "enarenar",
    "enarenarais": "enarenar",
    "enarenaran": "enarenar",
    "enarenaras": "enarenar",
    "enarenare": "enarenar",
    "enarenareis": "enarenar",
    "enarenaremos": "enarenar",
    "enarenaren": "enarenar",
    "enarenares": "enarenar",
    "enarenaron": "enarenar",
    "enarenará": "enarenar",
    "enarenarán": "enarenar",
    "enarenarás": "enarenar",
    "enarenaré": "enarenar",
    "enarenaréis": "enarenar",
    "enarenaría": "enarenar",
    "enarenaríais": "enarenar",
    "enarenaríamos": "enarenar",
    "enarenarían": "enarenar",
    "enarenarías": "enarenar",
    "enarenas": "enarenar",
    "enarenase": "enarenar",
    "enarenaseis": "enarenar",
    "enarenasen": "enarenar",
    "enarenases": "enarenar",
    "enarenaste": "enarenar",
    "enarenasteis": "enarenar",
    "enarene": "enarenar",
    "enarenemos": "enarenar",
    "enarenen": "enarenar",
    "enarenes": "enarenar",
    "enareno": "enarenar",
    "enarenábamos": "enarenar",
    "enarenáis": "enarenar",
    "enarenáramos": "enarenar",
    "enarenáremos": "enarenar",
    "enarenásemos": "enarenar",
    "enarené": "enarenar",
    "enarenéis": "enarenar",
    "enarenó": "enarenar",
    "enargitas": "enargita",
    "enarmas": "enarma",
    "enarmonías": "enarmonía",
    "enarmónica": "enarmónico",
    "enarmónicas": "enarmónico",
    "enarmónicos": "enarmónico",
    "enarque": "enarcar",
    "enarquemos": "enarcar",
    "enarquen": "enarcar",
    "enarques": "enarcar",
    "enarqué": "enarcar",
    "enarquéis": "enarcar",
    "enartrodiales": "enartrodial",
    "enasta": "enastar",
    "enastaba": "enastar",
    "enastabais": "enastar",
    "enastaban": "enastar",
    "enastabas": "enastar",
    "enastad": "enastar",
    "enastada": "enastar",
    "enastadas": "enastar",
    "enastado": "enastar",
    "enastados": "enastar",
    "enastamos": "enastar",
    "enastan": "enastar",
    "enastando": "enastar",
    "enastara": "enastar",
    "enastarais": "enastar",
    "enastaran": "enastar",
    "enastaras": "enastar",
    "enastare": "enastar",
    "enastareis": "enastar",
    "enastaremos": "enastar",
    "enastaren": "enastar",
    "enastares": "enastar",
    "enastaron": "enastar",
    "enastará": "enastar",
    "enastarán": "enastar",
    "enastarás": "enastar",
    "enastaré": "enastar",
    "enastaréis": "enastar",
    "enastaría": "enastar",
    "enastaríais": "enastar",
    "enastaríamos": "enastar",
    "enastarían": "enastar",
    "enastarías": "enastar",
    "enastas": "enastar",
    "enastase": "enastar",
    "enastaseis": "enastar",
    "enastasen": "enastar",
    "enastases": "enastar",
    "enastaste": "enastar",
    "enastasteis": "enastar",
    "enaste": "enastar",
    "enastemos": "enastar",
    "enasten": "enastar",
    "enastes": "enastar",
    "enasto": "enastar",
    "enastábamos": "enastar",
    "enastáis": "enastar",
    "enastáramos": "enastar",
    "enastáremos": "enastar",
    "enastásemos": "enastar",
    "enasté": "enastar",
    "enastéis": "enastar",
    "enastó": "enastar",
    "enbetunamientos": "enbetunamiento",
    "encaba": "encabar",
    "encababa": "encabar",
    "encababais": "encabar",
    "encababan": "encabar",
    "encababas": "encabar",
    "encabad": "encabar",
    "encabada": "encabar",
    "encabadas": "encabar",
    "encabado": "encabar",
    "encabados": "encabar",
    "encabalga": "encabalgar",
    "encabalgaba": "encabalgar",
    "encabalgabais": "encabalgar",
    "encabalgaban": "encabalgar",
    "encabalgabas": "encabalgar",
    "encabalgad": "encabalgar",
    "encabalgada": "encabalgar",
    "encabalgadas": "encabalgar",
    "encabalgado": "encabalgar",
    "encabalgados": "encabalgar",
    "encabalgamientos": "encabalgamiento",
    "encabalgamos": "encabalgar",
    "encabalgan": "encabalgar",
    "encabalgando": "encabalgar",
    "encabalgara": "encabalgar",
    "encabalgarais": "encabalgar",
    "encabalgaran": "encabalgar",
    "encabalgaras": "encabalgar",
    "encabalgare": "encabalgar",
    "encabalgareis": "encabalgar",
    "encabalgaremos": "encabalgar",
    "encabalgaren": "encabalgar",
    "encabalgares": "encabalgar",
    "encabalgaron": "encabalgar",
    "encabalgará": "encabalgar",
    "encabalgarán": "encabalgar",
    "encabalgarás": "encabalgar",
    "encabalgaré": "encabalgar",
    "encabalgaréis": "encabalgar",
    "encabalgaría": "encabalgar",
    "encabalgaríais": "encabalgar",
    "encabalgaríamos": "encabalgar",
    "encabalgarían": "encabalgar",
    "encabalgarías": "encabalgar",
    "encabalgas": "encabalgar",
    "encabalgase": "encabalgar",
    "encabalgaseis": "encabalgar",
    "encabalgasen": "encabalgar",
    "encabalgases": "encabalgar",
    "encabalgaste": "encabalgar",
    "encabalgasteis": "encabalgar",
    "encabalgo": "encabalgar",
    "encabalgue": "encabalgar",
    "encabalguemos": "encabalgar",
    "encabalguen": "encabalgar",
    "encabalgues": "encabalgar",
    "encabalgué": "encabalgar",
    "encabalguéis": "encabalgar",
    "encabalgábamos": "encabalgar",
    "encabalgáis": "encabalgar",
    "encabalgáramos": "encabalgar",
    "encabalgáremos": "encabalgar",
    "encabalgásemos": "encabalgar",
    "encabalgó": "encabalgar",
    "encaballa": "encaballar",
    "encaballaba": "encaballar",
    "encaballabais": "encaballar",
    "encaballaban": "encaballar",
    "encaballabas": "encaballar",
    "encaballad": "encaballar",
    "encaballada": "encaballar",
    "encaballadas": "encaballar",
    "encaballado": "encaballar",
    "encaballados": "encaballar",
    "encaballamos": "encaballar",
    "encaballan": "encaballar",
    "encaballando": "encaballar",
    "encaballara": "encaballar",
    "encaballarais": "encaballar",
    "encaballaran": "encaballar",
    "encaballaras": "encaballar",
    "encaballare": "encaballar",
    "encaballareis": "encaballar",
    "encaballaremos": "encaballar",
    "encaballaren": "encaballar",
    "encaballares": "encaballar",
    "encaballaron": "encaballar",
    "encaballará": "encaballar",
    "encaballarán": "encaballar",
    "encaballarás": "encaballar",
    "encaballaré": "encaballar",
    "encaballaréis": "encaballar",
    "encaballaría": "encaballar",
    "encaballaríais": "encaballar",
    "encaballaríamos": "encaballar",
    "encaballarían": "encaballar",
    "encaballarías": "encaballar",
    "encaballas": "encaballar",
    "encaballase": "encaballar",
    "encaballaseis": "encaballar",
    "encaballasen": "encaballar",
    "encaballases": "encaballar",
    "encaballaste": "encaballar",
    "encaballasteis": "encaballar",
    "encaballe": "encaballar",
    "encaballemos": "encaballar",
    "encaballen": "encaballar",
    "encaballes": "encaballar",
    "encaballo": "encaballar",
    "encaballábamos": "encaballar",
    "encaballáis": "encaballar",
    "encaballáramos": "encaballar",
    "encaballáremos": "encaballar",
    "encaballásemos": "encaballar",
    "encaballé": "encaballar",
    "encaballéis": "encaballar",
    "encaballó": "encaballar",
    "encabamos": "encabar",
    "encaban": "encabar",
    "encabando": "encabar",
    "encabara": "encabar",
    "encabarais": "encabar",
    "encabaran": "encabar",
    "encabaras": "encabar",
    "encabare": "encabar",
    "encabareis": "encabar",
    "encabaremos": "encabar",
    "encabaren": "encabar",
    "encabares": "encabar",
    "encabaron": "encabar",
    "encabará": "encabar",
    "encabarán": "encabar",
    "encabarás": "encabar",
    "encabaré": "encabar",
    "encabaréis": "encabar",
    "encabaría": "encabar",
    "encabaríais": "encabar",
    "encabaríamos": "encabar",
    "encabarían": "encabar",
    "encabarías": "encabar",
    "encabas": "encabar",
    "encabase": "encabar",
    "encabaseis": "encabar",
    "encabasen": "encabar",
    "encabases": "encabar",
    "encabaste": "encabar",
    "encabasteis": "encabar",
    "encabe": "encabar",
    "encabece": "encabezar",
    "encabecemos": "encabezar",
    "encabecen": "encabezar",
    "encabeces": "encabezar",
    "encabecé": "encabezar",
    "encabecéis": "encabezar",
    "encabemos": "encabar",
    "encaben": "encabar",
    "encabes": "encabar",
    "encabestra": "encabestrar",
    "encabestraba": "encabestrar",
    "encabestrabais": "encabestrar",
    "encabestraban": "encabestrar",
    "encabestrabas": "encabestrar",
    "encabestrad": "encabestrar",
    "encabestrada": "encabestrar",
    "encabestradas": "encabestrar",
    "encabestrado": "encabestrar",
    "encabestrados": "encabestrar",
    "encabestraduras": "encabestradura",
    "encabestramos": "encabestrar",
    "encabestran": "encabestrar",
    "encabestrando": "encabestrar",
    "encabestrara": "encabestrar",
    "encabestrarais": "encabestrar",
    "encabestraran": "encabestrar",
    "encabestraras": "encabestrar",
    "encabestrare": "encabestrar",
    "encabestrareis": "encabestrar",
    "encabestraremos": "encabestrar",
    "encabestraren": "encabestrar",
    "encabestrares": "encabestrar",
    "encabestraron": "encabestrar",
    "encabestrará": "encabestrar",
    "encabestrarán": "encabestrar",
    "encabestrarás": "encabestrar",
    "encabestraré": "encabestrar",
    "encabestraréis": "encabestrar",
    "encabestraría": "encabestrar",
    "encabestraríais": "encabestrar",
    "encabestraríamos": "encabestrar",
    "encabestrarían": "encabestrar",
    "encabestrarías": "encabestrar",
    "encabestras": "encabestrar",
    "encabestrase": "encabestrar",
    "encabestraseis": "encabestrar",
    "encabestrasen": "encabestrar",
    "encabestrases": "encabestrar",
    "encabestraste": "encabestrar",
    "encabestrasteis": "encabestrar",
    "encabestre": "encabestrar",
    "encabestremos": "encabestrar",
    "encabestren": "encabestrar",
    "encabestres": "encabestrar",
    "encabestro": "encabestrar",
    "encabestrábamos": "encabestrar",
    "encabestráis": "encabestrar",
    "encabestráramos": "encabestrar",
    "encabestráremos": "encabestrar",
    "encabestrásemos": "encabestrar",
    "encabestré": "encabestrar",
    "encabestréis": "encabestrar",
    "encabestró": "encabestrar",
    "encabeza": "encabezar",
    "encabezaba": "encabezar",
    "encabezabais": "encabezar",
    "encabezaban": "encabezar",
    "encabezabas": "encabezar",
    "encabezad": "encabezar",
    "encabezada": "encabezar",
    "encabezadas": "encabezar",
    "encabezado": "encabezar",
    "encabezados": "encabezar",
    "encabezamientos": "encabezamiento",
    "encabezamos": "encabezar",
    "encabezan": "encabezar",
    "encabezando": "encabezar",
    "encabezara": "encabezar",
    "encabezarais": "encabezar",
    "encabezaran": "encabezar",
    "encabezaras": "encabezar",
    "encabezare": "encabezar",
    "encabezareis": "encabezar",
    "encabezaremos": "encabezar",
    "encabezaren": "encabezar",
    "encabezares": "encabezar",
    "encabezaron": "encabezar",
    "encabezará": "encabezar",
    "encabezarán": "encabezar",
    "encabezarás": "encabezar",
    "encabezaré": "encabezar",
    "encabezaréis": "encabezar",
    "encabezaría": "encabezar",
    "encabezaríais": "encabezar",
    "encabezaríamos": "encabezar",
    "encabezarían": "encabezar",
    "encabezarías": "encabezar",
    "encabezas": "encabezar",
    "encabezase": "encabezar",
    "encabezaseis": "encabezar",
    "encabezasen": "encabezar",
    "encabezases": "encabezar",
    "encabezaste": "encabezar",
    "encabezasteis": "encabezar",
    "encabezo": "encabezar",
    "encabezábamos": "encabezar",
    "encabezáis": "encabezar",
    "encabezáramos": "encabezar",
    "encabezáremos": "encabezar",
    "encabezásemos": "encabezar",
    "encabezó": "encabezar",
    "encabo": "encabar",
    "encabriados": "encabriado",
    "encabrita": "encabritar",
    "encabritaba": "encabritar",
    "encabritabais": "encabritar",
    "encabritaban": "encabritar",
    "encabritabas": "encabritar",
    "encabritad": "encabritar",
    "encabritada": "encabritar",
    "encabritadas": "encabritar",
    "encabritado": "encabritar",
    "encabritados": "encabritar",
    "encabritamientos": "encabritamiento",
    "encabritamos": "encabritar",
    "encabritan": "encabritar",
    "encabritando": "encabritar",
    "encabritara": "encabritar",
    "encabritarais": "encabritar",
    "encabritaran": "encabritar",
    "encabritaras": "encabritar",
    "encabritare": "encabritar",
    "encabritareis": "encabritar",
    "encabritaremos": "encabritar",
    "encabritaren": "encabritar",
    "encabritares": "encabritar",
    "encabritaron": "encabritar",
    "encabritará": "encabritar",
    "encabritarán": "encabritar",
    "encabritarás": "encabritar",
    "encabritaré": "encabritar",
    "encabritaréis": "encabritar",
    "encabritaría": "encabritar",
    "encabritaríais": "encabritar",
    "encabritaríamos": "encabritar",
    "encabritarían": "encabritar",
    "encabritarías": "encabritar",
    "encabritas": "encabritar",
    "encabritase": "encabritar",
    "encabritaseis": "encabritar",
    "encabritasen": "encabritar",
    "encabritases": "encabritar",
    "encabritaste": "encabritar",
    "encabritasteis": "encabritar",
    "encabrite": "encabritar",
    "encabritemos": "encabritar",
    "encabriten": "encabritar",
    "encabrites": "encabritar",
    "encabrito": "encabritar",
    "encabritábamos": "encabritar",
    "encabritáis": "encabritar",
    "encabritáramos": "encabritar",
    "encabritáremos": "encabritar",
    "encabritásemos": "encabritar",
    "encabrité": "encabritar",
    "encabritéis": "encabritar",
    "encabritó": "encabritar",
    "encabábamos": "encabar",
    "encabáis": "encabar",
    "encabáramos": "encabar",
    "encabáremos": "encabar",
    "encabásemos": "encabar",
    "encabé": "encabar",
    "encabéis": "encabar",
    "encabó": "encabar",
    "encacha": "encachar",
    "encachaba": "encachar",
    "encachabais": "encachar",
    "encachaban": "encachar",
    "encachabas": "encachar",
    "encachad": "encachar",
    "encachada": "encachar",
    "encachadas": "encachar",
    "encachado": "encachar",
    "encachados": "encachar",
    "encachamos": "encachar",
    "encachan": "encachar",
    "encachando": "encachar",
    "encachara": "encachar",
    "encacharais": "encachar",
    "encacharan": "encachar",
    "encacharas": "encachar",
    "encachare": "encachar",
    "encachareis": "encachar",
    "encacharemos": "encachar",
    "encacharen": "encachar",
    "encachares": "encachar",
    "encacharon": "encachar",
    "encachará": "encachar",
    "encacharán": "encachar",
    "encacharás": "encachar",
    "encacharé": "encachar",
    "encacharéis": "encachar",
    "encacharía": "encachar",
    "encacharíais": "encachar",
    "encacharíamos": "encachar",
    "encacharían": "encachar",
    "encacharías": "encachar",
    "encachas": "encachar",
    "encachase": "encachar",
    "encachaseis": "encachar",
    "encachasen": "encachar",
    "encachases": "encachar",
    "encachaste": "encachar",
    "encachasteis": "encachar",
    "encache": "encachar",
    "encachemos": "encachar",
    "encachen": "encachar",
    "encaches": "encachar",
    "encacho": "encachar",
    "encachábamos": "encachar",
    "encacháis": "encachar",
    "encacháramos": "encachar",
    "encacháremos": "encachar",
    "encachásemos": "encachar",
    "encaché": "encachar",
    "encachéis": "encachar",
    "encachó": "encachar",
    "encada": "encadar",
    "encadaba": "encadar",
    "encadabais": "encadar",
    "encadaban": "encadar",
    "encadabas": "encadar",
    "encadad": "encadar",
    "encadada": "encadar",
    "encadadas": "encadar",
    "encadado": "encadar",
    "encadados": "encadar",
    "encadamos": "encadar",
    "encadan": "encadar",
    "encadando": "encadar",
    "encadara": "encadar",
    "encadarais": "encadar",
    "encadaran": "encadar",
    "encadaras": "encadar",
    "encadare": "encadar",
    "encadareis": "encadar",
    "encadaremos": "encadar",
    "encadaren": "encadar",
    "encadares": "encadar",
    "encadaron": "encadar",
    "encadará": "encadar",
    "encadarán": "encadar",
    "encadarás": "encadar",
    "encadaré": "encadar",
    "encadaréis": "encadar",
    "encadaría": "encadar",
    "encadaríais": "encadar",
    "encadaríamos": "encadar",
    "encadarían": "encadar",
    "encadarías": "encadar",
    "encadas": "encadar",
    "encadase": "encadar",
    "encadaseis": "encadar",
    "encadasen": "encadar",
    "encadases": "encadar",
    "encadaste": "encadar",
    "encadasteis": "encadar",
    "encade": "encadar",
    "encademos": "encadar",
    "encaden": "encadar",
    "encadena": "encadenar",
    "encadenaba": "encadenar",
    "encadenabais": "encadenar",
    "encadenaban": "encadenar",
    "encadenabas": "encadenar",
    "encadenad": "encadenar",
    "encadenada": "encadenar",
    "encadenadas": "encadenar",
    "encadenado": "encadenar",
    "encadenados": "encadenar",
    "encadenamientos": "encadenamiento",
    "encadenamos": "encadenar",
    "encadenan": "encadenar",
    "encadenando": "encadenar",
    "encadenara": "encadenar",
    "encadenarais": "encadenar",
    "encadenaran": "encadenar",
    "encadenaras": "encadenar",
    "encadenare": "encadenar",
    "encadenareis": "encadenar",
    "encadenaremos": "encadenar",
    "encadenaren": "encadenar",
    "encadenares": "encadenar",
    "encadenaron": "encadenar",
    "encadenará": "encadenar",
    "encadenarán": "encadenar",
    "encadenarás": "encadenar",
    "encadenaré": "encadenar",
    "encadenaréis": "encadenar",
    "encadenaría": "encadenar",
    "encadenaríais": "encadenar",
    "encadenaríamos": "encadenar",
    "encadenarían": "encadenar",
    "encadenarías": "encadenar",
    "encadenas": "encadenar",
    "encadenase": "encadenar",
    "encadenaseis": "encadenar",
    "encadenasen": "encadenar",
    "encadenases": "encadenar",
    "encadenaste": "encadenar",
    "encadenasteis": "encadenar",
    "encadene": "encadenar",
    "encadenemos": "encadenar",
    "encadenen": "encadenar",
    "encadenes": "encadenar",
    "encadeno": "encadenar",
    "encadenábamos": "encadenar",
    "encadenáis": "encadenar",
    "encadenáramos": "encadenar",
    "encadenáremos": "encadenar",
    "encadenásemos": "encadenar",
    "encadené": "encadenar",
    "encadenéis": "encadenar",
    "encadenó": "encadenar",
    "encades": "encadar",
    "encado": "encadar",
    "encadábamos": "encadar",
    "encadáis": "encadar",
    "encadáramos": "encadar",
    "encadáremos": "encadar",
    "encadásemos": "encadar",
    "encadé": "encadar",
    "encadéis": "encadar",
    "encadó": "encadar",
    "encaja": "encajar",
    "encajaba": "encajar",
    "encajabais": "encajar",
    "encajaban": "encajar",
    "encajabas": "encajar",
    "encajables": "encajable",
    "encajad": "encajar",
    "encajada": "encajar",
    "encajadas": "encajar",
    "encajado": "encajar",
    "encajadora": "encajador",
    "encajadoras": "encajador",
    "encajadores": "encajador",
    "encajados": "encajar",
    "encajaduras": "encajadura",
    "encajamos": "encajar",
    "encajan": "encajar",
    "encajando": "encajar",
    "encajara": "encajar",
    "encajarais": "encajar",
    "encajaran": "encajar",
    "encajaras": "encajar",
    "encajare": "encajar",
    "encajareis": "encajar",
    "encajaremos": "encajar",
    "encajaren": "encajar",
    "encajares": "encajar",
    "encajaron": "encajar",
    "encajará": "encajar",
    "encajarán": "encajar",
    "encajarás": "encajar",
    "encajaré": "encajar",
    "encajaréis": "encajar",
    "encajaría": "encajar",
    "encajaríais": "encajar",
    "encajaríamos": "encajar",
    "encajarían": "encajar",
    "encajarías": "encajar",
    "encajas": "encajar",
    "encajase": "encajar",
    "encajaseis": "encajar",
    "encajasen": "encajar",
    "encajases": "encajar",
    "encajaste": "encajar",
    "encajasteis": "encajar",
    "encaje": "encajar",
    "encajemos": "encajar",
    "encajen": "encajar",
    "encajera": "encajero",
    "encajeras": "encajero",
    "encajeros": "encajero",
    "encajes": "encaje",
    "encajo": "encajar",
    "encajona": "encajonar",
    "encajonaba": "encajonar",
    "encajonabais": "encajonar",
    "encajonaban": "encajonar",
    "encajonabas": "encajonar",
    "encajonad": "encajonar",
    "encajonada": "encajonar",
    "encajonadas": "encajonar",
    "encajonado": "encajonar",
    "encajonados": "encajonar",
    "encajonamientos": "encajonamiento",
    "encajonamos": "encajonar",
    "encajonan": "encajonar",
    "encajonando": "encajonar",
    "encajonara": "encajonar",
    "encajonarais": "encajonar",
    "encajonaran": "encajonar",
    "encajonaras": "encajonar",
    "encajonare": "encajonar",
    "encajonareis": "encajonar",
    "encajonaremos": "encajonar",
    "encajonaren": "encajonar",
    "encajonares": "encajonar",
    "encajonaron": "encajonar",
    "encajonará": "encajonar",
    "encajonarán": "encajonar",
    "encajonarás": "encajonar",
    "encajonaré": "encajonar",
    "encajonaréis": "encajonar",
    "encajonaría": "encajonar",
    "encajonaríais": "encajonar",
    "encajonaríamos": "encajonar",
    "encajonarían": "encajonar",
    "encajonarías": "encajonar",
    "encajonas": "encajonar",
    "encajonase": "encajonar",
    "encajonaseis": "encajonar",
    "encajonasen": "encajonar",
    "encajonases": "encajonar",
    "encajonaste": "encajonar",
    "encajonasteis": "encajonar",
    "encajone": "encajonar",
    "encajonemos": "encajonar",
    "encajonen": "encajonar",
    "encajones": "encajonar",
    "encajono": "encajonar",
    "encajonábamos": "encajonar",
    "encajonáis": "encajonar",
    "encajonáramos": "encajonar",
    "encajonáremos": "encajonar",
    "encajonásemos": "encajonar",
    "encajoné": "encajonar",
    "encajonéis": "encajonar",
    "encajonó": "encajonar",
    "encajábamos": "encajar",
    "encajáis": "encajar",
    "encajáramos": "encajar",
    "encajáremos": "encajar",
    "encajásemos": "encajar",
    "encajé": "encajar",
    "encajéis": "encajar",
    "encajó": "encajar",
    "encala": "encalar",
    "encalaba": "encalar",
    "encalabais": "encalar",
    "encalaban": "encalar",
    "encalabas": "encalar",
    "encalaboce": "encalabozar",
    "encalabocemos": "encalabozar",
    "encalabocen": "encalabozar",
    "encalaboces": "encalabozar",
    "encalabocé": "encalabozar",
    "encalabocéis": "encalabozar",
    "encalaboza": "encalabozar",
    "encalabozaba": "encalabozar",
    "encalabozabais": "encalabozar",
    "encalabozaban": "encalabozar",
    "encalabozabas": "encalabozar",
    "encalabozad": "encalabozar",
    "encalabozada": "encalabozar",
    "encalabozadas": "encalabozar",
    "encalabozado": "encalabozar",
    "encalabozados": "encalabozar",
    "encalabozamos": "encalabozar",
    "encalabozan": "encalabozar",
    "encalabozando": "encalabozar",
    "encalabozara": "encalabozar",
    "encalabozarais": "encalabozar",
    "encalabozaran": "encalabozar",
    "encalabozaras": "encalabozar",
    "encalabozare": "encalabozar",
    "encalabozareis": "encalabozar",
    "encalabozaremos": "encalabozar",
    "encalabozaren": "encalabozar",
    "encalabozares": "encalabozar",
    "encalabozaron": "encalabozar",
    "encalabozará": "encalabozar",
    "encalabozarán": "encalabozar",
    "encalabozarás": "encalabozar",
    "encalabozaré": "encalabozar",
    "encalabozaréis": "encalabozar",
    "encalabozaría": "encalabozar",
    "encalabozaríais": "encalabozar",
    "encalabozaríamos": "encalabozar",
    "encalabozarían": "encalabozar",
    "encalabozarías": "encalabozar",
    "encalabozas": "encalabozar",
    "encalabozase": "encalabozar",
    "encalabozaseis": "encalabozar",
    "encalabozasen": "encalabozar",
    "encalabozases": "encalabozar",
    "encalabozaste": "encalabozar",
    "encalabozasteis": "encalabozar",
    "encalabozo": "encalabozar",
    "encalabozábamos": "encalabozar",
    "encalabozáis": "encalabozar",
    "encalabozáramos": "encalabozar",
    "encalabozáremos": "encalabozar",
    "encalabozásemos": "encalabozar",
    "encalabozó": "encalabozar",
    "encalabrina": "encalabrinar",
    "encalabrinaba": "encalabrinar",
    "encalabrinabais": "encalabrinar",
    "encalabrinaban": "encalabrinar",
    "encalabrinabas": "encalabrinar",
    "encalabrinad": "encalabrinar",
    "encalabrinada": "encalabrinar",
    "encalabrinadas": "encalabrinar",
    "encalabrinado": "encalabrinar",
    "encalabrinados": "encalabrinar",
    "encalabrinamientos": "encalabrinamiento",
    "encalabrinamos": "encalabrinar",
    "encalabrinan": "encalabrinar",
    "encalabrinando": "encalabrinar",
    "encalabrinara": "encalabrinar",
    "encalabrinarais": "encalabrinar",
    "encalabrinaran": "encalabrinar",
    "encalabrinaras": "encalabrinar",
    "encalabrinare": "encalabrinar",
    "encalabrinareis": "encalabrinar",
    "encalabrinaremos": "encalabrinar",
    "encalabrinaren": "encalabrinar",
    "encalabrinares": "encalabrinar",
    "encalabrinaron": "encalabrinar",
    "encalabrinará": "encalabrinar",
    "encalabrinarán": "encalabrinar",
    "encalabrinarás": "encalabrinar",
    "encalabrinaré": "encalabrinar",
    "encalabrinaréis": "encalabrinar",
    "encalabrinaría": "encalabrinar",
    "encalabrinaríais": "encalabrinar",
    "encalabrinaríamos": "encalabrinar",
    "encalabrinarían": "encalabrinar",
    "encalabrinarías": "encalabrinar",
    "encalabrinas": "encalabrinar",
    "encalabrinase": "encalabrinar",
    "encalabrinaseis": "encalabrinar",
    "encalabrinasen": "encalabrinar",
    "encalabrinases": "encalabrinar",
    "encalabrinaste": "encalabrinar",
    "encalabrinasteis": "encalabrinar",
    "encalabrine": "encalabrinar",
    "encalabrinemos": "encalabrinar",
    "encalabrinen": "encalabrinar",
    "encalabrines": "encalabrinar",
    "encalabrino": "encalabrinar",
    "encalabrinábamos": "encalabrinar",
    "encalabrináis": "encalabrinar",
    "encalabrináramos": "encalabrinar",
    "encalabrináremos": "encalabrinar",
    "encalabrinásemos": "encalabrinar",
    "encalabriné": "encalabrinar",
    "encalabrinéis": "encalabrinar",
    "encalabrinó": "encalabrinar",
    "encalad": "encalar",
    "encalada": "encalar",
    "encaladas": "encalar",
    "encalado": "encalar",
    "encaladora": "encalador",
    "encaladoras": "encalador",
    "encaladores": "encalador",
    "encalados": "encalar",
    "encaladuras": "encaladura",
    "encalamoca": "encalamocar",
    "encalamocaba": "encalamocar",
    "encalamocabais": "encalamocar",
    "encalamocaban": "encalamocar",
    "encalamocabas": "encalamocar",
    "encalamocad": "encalamocar",
    "encalamocada": "encalamocar",
    "encalamocadas": "encalamocar",
    "encalamocado": "encalamocar",
    "encalamocados": "encalamocar",
    "encalamocamos": "encalamocar",
    "encalamocan": "encalamocar",
    "encalamocando": "encalamocar",
    "encalamocara": "encalamocar",
    "encalamocarais": "encalamocar",
    "encalamocaran": "encalamocar",
    "encalamocaras": "encalamocar",
    "encalamocare": "encalamocar",
    "encalamocareis": "encalamocar",
    "encalamocaremos": "encalamocar",
    "encalamocaren": "encalamocar",
    "encalamocares": "encalamocar",
    "encalamocaron": "encalamocar",
    "encalamocará": "encalamocar",
    "encalamocarán": "encalamocar",
    "encalamocarás": "encalamocar",
    "encalamocaré": "encalamocar",
    "encalamocaréis": "encalamocar",
    "encalamocaría": "encalamocar",
    "encalamocaríais": "encalamocar",
    "encalamocaríamos": "encalamocar",
    "encalamocarían": "encalamocar",
    "encalamocarías": "encalamocar",
    "encalamocas": "encalamocar",
    "encalamocase": "encalamocar",
    "encalamocaseis": "encalamocar",
    "encalamocasen": "encalamocar",
    "encalamocases": "encalamocar",
    "encalamocaste": "encalamocar",
    "encalamocasteis": "encalamocar",
    "encalamoco": "encalamocar",
    "encalamocábamos": "encalamocar",
    "encalamocáis": "encalamocar",
    "encalamocáramos": "encalamocar",
    "encalamocáremos": "encalamocar",
    "encalamocásemos": "encalamocar",
    "encalamocó": "encalamocar",
    "encalamoque": "encalamocar",
    "encalamoquemos": "encalamocar",
    "encalamoquen": "encalamocar",
    "encalamoques": "encalamocar",
    "encalamoqué": "encalamocar",
    "encalamoquéis": "encalamocar",
    "encalamos": "encalar",
    "encalan": "encalar",
    "encalando": "encalar",
    "encalara": "encalar",
    "encalarais": "encalar",
    "encalaran": "encalar",
    "encalaras": "encalar",
    "encalare": "encalar",
    "encalareis": "encalar",
    "encalaremos": "encalar",
    "encalaren": "encalar",
    "encalares": "encalar",
    "encalaron": "encalar",
    "encalará": "encalar",
    "encalarán": "encalar",
    "encalarás": "encalar",
    "encalaré": "encalar",
    "encalaréis": "encalar",
    "encalaría": "encalar",
    "encalaríais": "encalar",
    "encalaríamos": "encalar",
    "encalarían": "encalar",
    "encalarías": "encalar",
    "encalas": "encalar",
    "encalase": "encalar",
    "encalaseis": "encalar",
    "encalasen": "encalar",
    "encalases": "encalar",
    "encalaste": "encalar",
    "encalasteis": "encalar",
    "encale": "encalar",
    "encalemos": "encalar",
    "encalen": "encalar",
    "encales": "encalar",
    "encalla": "encallar",
    "encallaba": "encallar",
    "encallabais": "encallar",
    "encallaban": "encallar",
    "encallabas": "encallar",
    "encallad": "encallar",
    "encallada": "encallar",
    "encalladas": "encallar",
    "encalladeros": "encalladero",
    "encallado": "encallar",
    "encallados": "encallar",
    "encalladuras": "encalladura",
    "encallamientos": "encallamiento",
    "encallamos": "encallar",
    "encallan": "encallar",
    "encallando": "encallar",
    "encallara": "encallar",
    "encallarais": "encallar",
    "encallaran": "encallar",
    "encallaras": "encallar",
    "encallare": "encallar",
    "encallareis": "encallar",
    "encallaremos": "encallar",
    "encallaren": "encallar",
    "encallares": "encallar",
    "encallaron": "encallar",
    "encallará": "encallar",
    "encallarán": "encallar",
    "encallarás": "encallar",
    "encallaré": "encallar",
    "encallaréis": "encallar",
    "encallaría": "encallar",
    "encallaríais": "encallar",
    "encallaríamos": "encallar",
    "encallarían": "encallar",
    "encallarías": "encallar",
    "encallas": "encallar",
    "encallase": "encallar",
    "encallaseis": "encallar",
    "encallasen": "encallar",
    "encallases": "encallar",
    "encallaste": "encallar",
    "encallasteis": "encallar",
    "encalle": "encallar",
    "encallece": "encallecer",
    "encalleced": "encallecer",
    "encallecemos": "encallecer",
    "encallecen": "encallecer",
    "encalleceremos": "encallecer",
    "encallecerá": "encallecer",
    "encallecerán": "encallecer",
    "encallecerás": "encallecer",
    "encalleceré": "encallecer",
    "encalleceréis": "encallecer",
    "encallecería": "encallecer",
    "encalleceríais": "encallecer",
    "encalleceríamos": "encallecer",
    "encallecerían": "encallecer",
    "encallecerías": "encallecer",
    "encalleces": "encallecer",
    "encallecida": "encallecer",
    "encallecidas": "encallecer",
    "encallecido": "encallecer",
    "encallecidos": "encallecer",
    "encalleciendo": "encallecer",
    "encalleciera": "encallecer",
    "encallecierais": "encallecer",
    "encallecieran": "encallecer",
    "encallecieras": "encallecer",
    "encalleciere": "encallecer",
    "encalleciereis": "encallecer",
    "encallecieren": "encallecer",
    "encallecieres": "encallecer",
    "encallecieron": "encallecer",
    "encalleciese": "encallecer",
    "encallecieseis": "encallecer",
    "encalleciesen": "encallecer",
    "encallecieses": "encallecer",
    "encallecimientos": "encallecimiento",
    "encallecimos": "encallecer",
    "encalleciste": "encallecer",
    "encallecisteis": "encallecer",
    "encalleciéramos": "encallecer",
    "encalleciéremos": "encallecer",
    "encalleciésemos": "encallecer",
    "encalleció": "encallecer",
    "encallecéis": "encallecer",
    "encallecí": "encallecer",
    "encallecía": "encallecer",
    "encallecíais": "encallecer",
    "encallecíamos": "encallecer",
    "encallecían": "encallecer",
    "encallecías": "encallecer",
    "encallemos": "encallar",
    "encallen": "encallar",
    "encalles": "encallar",
    "encallezca": "encallecer",
    "encallezcamos": "encallecer",
    "encallezcan": "encallecer",
    "encallezcas": "encallecer",
    "encallezco": "encallecer",
    "encallezcáis": "encallecer",
    "encallo": "encallar",
    "encallábamos": "encallar",
    "encalláis": "encallar",
    "encalláramos": "encallar",
    "encalláremos": "encallar",
    "encallásemos": "encallar",
    "encallé": "encallar",
    "encalléis": "encallar",
    "encalló": "encallar",
    "encalma": "encalmar",
    "encalmaba": "encalmar",
    "encalmabais": "encalmar",
    "encalmaban": "encalmar",
    "encalmabas": "encalmar",
    "encalmad": "encalmar",
    "encalmada": "encalmar",
    "encalmadas": "encalmar",
    "encalmado": "encalmar",
    "encalmados": "encalmar",
    "encalmaduras": "encalmadura",
    "encalmamos": "encalmar",
    "encalman": "encalmar",
    "encalmando": "encalmar",
    "encalmara": "encalmar",
    "encalmarais": "encalmar",
    "encalmaran": "encalmar",
    "encalmaras": "encalmar",
    "encalmare": "encalmar",
    "encalmareis": "encalmar",
    "encalmaremos": "encalmar",
    "encalmaren": "encalmar",
    "encalmares": "encalmar",
    "encalmaron": "encalmar",
    "encalmará": "encalmar",
    "encalmarán": "encalmar",
    "encalmarás": "encalmar",
    "encalmaré": "encalmar",
    "encalmaréis": "encalmar",
    "encalmaría": "encalmar",
    "encalmaríais": "encalmar",
    "encalmaríamos": "encalmar",
    "encalmarían": "encalmar",
    "encalmarías": "encalmar",
    "encalmas": "encalmar",
    "encalmase": "encalmar",
    "encalmaseis": "encalmar",
    "encalmasen": "encalmar",
    "encalmases": "encalmar",
    "encalmaste": "encalmar",
    "encalmasteis": "encalmar",
    "encalme": "encalmar",
    "encalmemos": "encalmar",
    "encalmen": "encalmar",
    "encalmes": "encalmar",
    "encalmo": "encalmar",
    "encalmábamos": "encalmar",
    "encalmáis": "encalmar",
    "encalmáramos": "encalmar",
    "encalmáremos": "encalmar",
    "encalmásemos": "encalmar",
    "encalmé": "encalmar",
    "encalméis": "encalmar",
    "encalmó": "encalmar",
    "encalo": "encalar",
    "encalvece": "encalvecer",
    "encalveced": "encalvecer",
    "encalvecemos": "encalvecer",
    "encalvecen": "encalvecer",
    "encalveceremos": "encalvecer",
    "encalvecerá": "encalvecer",
    "encalvecerán": "encalvecer",
    "encalvecerás": "encalvecer",
    "encalveceré": "encalvecer",
    "encalveceréis": "encalvecer",
    "encalvecería": "encalvecer",
    "encalveceríais": "encalvecer",
    "encalveceríamos": "encalvecer",
    "encalvecerían": "encalvecer",
    "encalvecerías": "encalvecer",
    "encalveces": "encalvecer",
    "encalvecida": "encalvecer",
    "encalvecidas": "encalvecer",
    "encalvecido": "encalvecer",
    "encalvecidos": "encalvecer",
    "encalveciendo": "encalvecer",
    "encalveciera": "encalvecer",
    "encalvecierais": "encalvecer",
    "encalvecieran": "encalvecer",
    "encalvecieras": "encalvecer",
    "encalveciere": "encalvecer",
    "encalveciereis": "encalvecer",
    "encalvecieren": "encalvecer",
    "encalvecieres": "encalvecer",
    "encalvecieron": "encalvecer",
    "encalveciese": "encalvecer",
    "encalvecieseis": "encalvecer",
    "encalveciesen": "encalvecer",
    "encalvecieses": "encalvecer",
    "encalvecimientos": "encalvecimiento",
    "encalvecimos": "encalvecer",
    "encalveciste": "encalvecer",
    "encalvecisteis": "encalvecer",
    "encalveciéramos": "encalvecer",
    "encalveciéremos": "encalvecer",
    "encalveciésemos": "encalvecer",
    "encalveció": "encalvecer",
    "encalvecéis": "encalvecer",
    "encalvecí": "encalvecer",
    "encalvecía": "encalvecer",
    "encalvecíais": "encalvecer",
    "encalvecíamos": "encalvecer",
    "encalvecían": "encalvecer",
    "encalvecías": "encalvecer",
    "encalvezca": "encalvecer",
    "encalvezcamos": "encalvecer",
    "encalvezcan": "encalvecer",
    "encalvezcas": "encalvecer",
    "encalvezco": "encalvecer",
    "encalvezcáis": "encalvecer",
    "encalábamos": "encalar",
    "encaláis": "encalar",
    "encaláramos": "encalar",
    "encaláremos": "encalar",
    "encalásemos": "encalar",
    "encalé": "encalar",
    "encaléis": "encalar",
    "encaló": "encalar",
    "encama": "encamar",
    "encamaba": "encamar",
    "encamabais": "encamar",
    "encamaban": "encamar",
    "encamabas": "encamar",
    "encamaciones": "encamación",
    "encamad": "encamar",
    "encamada": "encamar",
    "encamadas": "encamar",
    "encamado": "encamar",
    "encamados": "encamar",
    "encamamos": "encamar",
    "encaman": "encamar",
    "encamando": "encamar",
    "encamara": "encamar",
    "encamarais": "encamar",
    "encamaran": "encamar",
    "encamaras": "encamar",
    "encamare": "encamar",
    "encamareis": "encamar",
    "encamaremos": "encamar",
    "encamaren": "encamar",
    "encamares": "encamar",
    "encamaron": "encamar",
    "encamará": "encamar",
    "encamarán": "encamar",
    "encamarás": "encamar",
    "encamaré": "encamar",
    "encamaréis": "encamar",
    "encamaría": "encamar",
    "encamaríais": "encamar",
    "encamaríamos": "encamar",
    "encamarían": "encamar",
    "encamarías": "encamar",
    "encamas": "encamar",
    "encamase": "encamar",
    "encamaseis": "encamar",
    "encamasen": "encamar",
    "encamases": "encamar",
    "encamaste": "encamar",
    "encamasteis": "encamar",
    "encambija": "encambijar",
    "encambijaba": "encambijar",
    "encambijabais": "encambijar",
    "encambijaban": "encambijar",
    "encambijabas": "encambijar",
    "encambijad": "encambijar",
    "encambijada": "encambijar",
    "encambijadas": "encambijar",
    "encambijado": "encambijar",
    "encambijados": "encambijar",
    "encambijamos": "encambijar",
    "encambijan": "encambijar",
    "encambijando": "encambijar",
    "encambijara": "encambijar",
    "encambijarais": "encambijar",
    "encambijaran": "encambijar",
    "encambijaras": "encambijar",
    "encambijare": "encambijar",
    "encambijareis": "encambijar",
    "encambijaremos": "encambijar",
    "encambijaren": "encambijar",
    "encambijares": "encambijar",
    "encambijaron": "encambijar",
    "encambijará": "encambijar",
    "encambijarán": "encambijar",
    "encambijarás": "encambijar",
    "encambijaré": "encambijar",
    "encambijaréis": "encambijar",
    "encambijaría": "encambijar",
    "encambijaríais": "encambijar",
    "encambijaríamos": "encambijar",
    "encambijarían": "encambijar",
    "encambijarías": "encambijar",
    "encambijas": "encambijar",
    "encambijase": "encambijar",
    "encambijaseis": "encambijar",
    "encambijasen": "encambijar",
    "encambijases": "encambijar",
    "encambijaste": "encambijar",
    "encambijasteis": "encambijar",
    "encambije": "encambijar",
    "encambijemos": "encambijar",
    "encambijen": "encambijar",
    "encambijes": "encambijar",
    "encambijo": "encambijar",
    "encambijábamos": "encambijar",
    "encambijáis": "encambijar",
    "encambijáramos": "encambijar",
    "encambijáremos": "encambijar",
    "encambijásemos": "encambijar",
    "encambijé": "encambijar",
    "encambijéis": "encambijar",
    "encambijó": "encambijar",
    "encambronamientos": "encambronamiento",
    "encame": "encamar",
    "encamemos": "encamar",
    "encamen": "encamar",
    "encames": "encamar",
    "encamina": "encaminar",
    "encaminaba": "encaminar",
    "encaminabais": "encaminar",
    "encaminaban": "encaminar",
    "encaminabas": "encaminar",
    "encaminad": "encaminar",
    "encaminada": "encaminar",
    "encaminadas": "encaminar",
    "encaminado": "encaminar",
    "encaminadores": "encaminador",
    "encaminados": "encaminar",
    "encaminaduras": "encaminadura",
    "encaminamientos": "encaminamiento",
    "encaminamos": "encaminar",
    "encaminan": "encaminar",
    "encaminando": "encaminar",
    "encaminara": "encaminar",
    "encaminarais": "encaminar",
    "encaminaran": "encaminar",
    "encaminaras": "encaminar",
    "encaminare": "encaminar",
    "encaminareis": "encaminar",
    "encaminaremos": "encaminar",
    "encaminaren": "encaminar",
    "encaminares": "encaminar",
    "encaminaron": "encaminar",
    "encaminará": "encaminar",
    "encaminarán": "encaminar",
    "encaminarás": "encaminar",
    "encaminaré": "encaminar",
    "encaminaréis": "encaminar",
    "encaminaría": "encaminar",
    "encaminaríais": "encaminar",
    "encaminaríamos": "encaminar",
    "encaminarían": "encaminar",
    "encaminarías": "encaminar",
    "encaminas": "encaminar",
    "encaminase": "encaminar",
    "encaminaseis": "encaminar",
    "encaminasen": "encaminar",
    "encaminases": "encaminar",
    "encaminaste": "encaminar",
    "encaminasteis": "encaminar",
    "encamine": "encaminar",
    "encaminemos": "encaminar",
    "encaminen": "encaminar",
    "encamines": "encaminar",
    "encamino": "encaminar",
    "encaminábamos": "encaminar",
    "encamináis": "encaminar",
    "encamináramos": "encaminar",
    "encamináremos": "encaminar",
    "encaminásemos": "encaminar",
    "encaminé": "encaminar",
    "encaminéis": "encaminar",
    "encaminó": "encaminar",
    "encamisa": "encamisar",
    "encamisaba": "encamisar",
    "encamisabais": "encamisar",
    "encamisaban": "encamisar",
    "encamisabas": "encamisar",
    "encamisad": "encamisar",
    "encamisada": "encamisar",
    "encamisadas": "encamisar",
    "encamisado": "encamisar",
    "encamisados": "encamisar",
    "encamisamos": "encamisar",
    "encamisan": "encamisar",
    "encamisando": "encamisar",
    "encamisara": "encamisar",
    "encamisarais": "encamisar",
    "encamisaran": "encamisar",
    "encamisaras": "encamisar",
    "encamisare": "encamisar",
    "encamisareis": "encamisar",
    "encamisaremos": "encamisar",
    "encamisaren": "encamisar",
    "encamisares": "encamisar",
    "encamisaron": "encamisar",
    "encamisará": "encamisar",
    "encamisarán": "encamisar",
    "encamisarás": "encamisar",
    "encamisaré": "encamisar",
    "encamisaréis": "encamisar",
    "encamisaría": "encamisar",
    "encamisaríais": "encamisar",
    "encamisaríamos": "encamisar",
    "encamisarían": "encamisar",
    "encamisarías": "encamisar",
    "encamisas": "encamisar",
    "encamisase": "encamisar",
    "encamisaseis": "encamisar",
    "encamisasen": "encamisar",
    "encamisases": "encamisar",
    "encamisaste": "encamisar",
    "encamisasteis": "encamisar",
    "encamise": "encamisar",
    "encamisemos": "encamisar",
    "encamisen": "encamisar",
    "encamises": "encamisar",
    "encamiso": "encamisar",
    "encamisábamos": "encamisar",
    "encamisáis": "encamisar",
    "encamisáramos": "encamisar",
    "encamisáremos": "encamisar",
    "encamisásemos": "encamisar",
    "encamisé": "encamisar",
    "encamiséis": "encamisar",
    "encamisó": "encamisar",
    "encamo": "encamar",
    "encamonada": "encamonado",
    "encamonadas": "encamonado",
    "encamonados": "encamonado",
    "encamota": "encamotar",
    "encamotaba": "encamotar",
    "encamotabais": "encamotar",
    "encamotaban": "encamotar",
    "encamotabas": "encamotar",
    "encamotad": "encamotar",
    "encamotada": "encamotar",
    "encamotadas": "encamotar",
    "encamotado": "encamotar",
    "encamotados": "encamotar",
    "encamotamos": "encamotar",
    "encamotan": "encamotar",
    "encamotando": "encamotar",
    "encamotara": "encamotar",
    "encamotarais": "encamotar",
    "encamotaran": "encamotar",
    "encamotaras": "encamotar",
    "encamotare": "encamotar",
    "encamotareis": "encamotar",
    "encamotaremos": "encamotar",
    "encamotaren": "encamotar",
    "encamotares": "encamotar",
    "encamotaron": "encamotar",
    "encamotará": "encamotar",
    "encamotarán": "encamotar",
    "encamotarás": "encamotar",
    "encamotaré": "encamotar",
    "encamotaréis": "encamotar",
    "encamotaría": "encamotar",
    "encamotaríais": "encamotar",
    "encamotaríamos": "encamotar",
    "encamotarían": "encamotar",
    "encamotarías": "encamotar",
    "encamotas": "encamotar",
    "encamotase": "encamotar",
    "encamotaseis": "encamotar",
    "encamotasen": "encamotar",
    "encamotases": "encamotar",
    "encamotaste": "encamotar",
    "encamotasteis": "encamotar",
    "encamote": "encamotar",
    "encamotemos": "encamotar",
    "encamoten": "encamotar",
    "encamotes": "encamotar",
    "encamoto": "encamotar",
    "encamotábamos": "encamotar",
    "encamotáis": "encamotar",
    "encamotáramos": "encamotar",
    "encamotáremos": "encamotar",
    "encamotásemos": "encamotar",
    "encamoté": "encamotar",
    "encamotéis": "encamotar",
    "encamotó": "encamotar",
    "encampana": "encampanar",
    "encampanaba": "encampanar",
    "encampanabais": "encampanar",
    "encampanaban": "encampanar",
    "encampanabas": "encampanar",
    "encampanad": "encampanar",
    "encampanada": "encampanar",
    "encampanadas": "encampanar",
    "encampanado": "encampanar",
    "encampanados": "encampanar",
    "encampanamos": "encampanar",
    "encampanan": "encampanar",
    "encampanando": "encampanar",
    "encampanara": "encampanar",
    "encampanarais": "encampanar",
    "encampanaran": "encampanar",
    "encampanaras": "encampanar",
    "encampanare": "encampanar",
    "encampanareis": "encampanar",
    "encampanaremos": "encampanar",
    "encampanaren": "encampanar",
    "encampanares": "encampanar",
    "encampanaron": "encampanar",
    "encampanará": "encampanar",
    "encampanarán": "encampanar",
    "encampanarás": "encampanar",
    "encampanaré": "encampanar",
    "encampanaréis": "encampanar",
    "encampanaría": "encampanar",
    "encampanaríais": "encampanar",
    "encampanaríamos": "encampanar",
    "encampanarían": "encampanar",
    "encampanarías": "encampanar",
    "encampanas": "encampanar",
    "encampanase": "encampanar",
    "encampanaseis": "encampanar",
    "encampanasen": "encampanar",
    "encampanases": "encampanar",
    "encampanaste": "encampanar",
    "encampanasteis": "encampanar",
    "encampane": "encampanar",
    "encampanemos": "encampanar",
    "encampanen": "encampanar",
    "encampanes": "encampanar",
    "encampano": "encampanar",
    "encampanábamos": "encampanar",
    "encampanáis": "encampanar",
    "encampanáramos": "encampanar",
    "encampanáremos": "encampanar",
    "encampanásemos": "encampanar",
    "encampané": "encampanar",
    "encampanéis": "encampanar",
    "encampanó": "encampanar",
    "encamábamos": "encamar",
    "encamáis": "encamar",
    "encamáramos": "encamar",
    "encamáremos": "encamar",
    "encamásemos": "encamar",
    "encamé": "encamar",
    "encaméis": "encamar",
    "encamó": "encamar",
    "encanalizamientos": "encanalizamiento",
    "encanalla": "encanallar",
    "encanallaba": "encanallar",
    "encanallabais": "encanallar",
    "encanallaban": "encanallar",
    "encanallabas": "encanallar",
    "encanallad": "encanallar",
    "encanallada": "encanallar",
    "encanalladas": "encanallar",
    "encanallado": "encanallar",
    "encanallados": "encanallar",
    "encanallamientos": "encanallamiento",
    "encanallamos": "encanallar",
    "encanallan": "encanallar",
    "encanallando": "encanallar",
    "encanallara": "encanallar",
    "encanallarais": "encanallar",
    "encanallaran": "encanallar",
    "encanallaras": "encanallar",
    "encanallare": "encanallar",
    "encanallareis": "encanallar",
    "encanallaremos": "encanallar",
    "encanallaren": "encanallar",
    "encanallares": "encanallar",
    "encanallaron": "encanallar",
    "encanallará": "encanallar",
    "encanallarán": "encanallar",
    "encanallarás": "encanallar",
    "encanallaré": "encanallar",
    "encanallaréis": "encanallar",
    "encanallaría": "encanallar",
    "encanallaríais": "encanallar",
    "encanallaríamos": "encanallar",
    "encanallarían": "encanallar",
    "encanallarías": "encanallar",
    "encanallas": "encanallar",
    "encanallase": "encanallar",
    "encanallaseis": "encanallar",
    "encanallasen": "encanallar",
    "encanallases": "encanallar",
    "encanallaste": "encanallar",
    "encanallasteis": "encanallar",
    "encanalle": "encanallar",
    "encanallemos": "encanallar",
    "encanallen": "encanallar",
    "encanalles": "encanallar",
    "encanallo": "encanallar",
    "encanallábamos": "encanallar",
    "encanalláis": "encanallar",
    "encanalláramos": "encanallar",
    "encanalláremos": "encanallar",
    "encanallásemos": "encanallar",
    "encanallé": "encanallar",
    "encanalléis": "encanallar",
    "encanalló": "encanallar",
    "encanasta": "encanastar",
    "encanastaba": "encanastar",
    "encanastabais": "encanastar",
    "encanastaban": "encanastar",
    "encanastabas": "encanastar",
    "encanastad": "encanastar",
    "encanastada": "encanastar",
    "encanastadas": "encanastar",
    "encanastado": "encanastar",
    "encanastados": "encanastar",
    "encanastamos": "encanastar",
    "encanastan": "encanastar",
    "encanastando": "encanastar",
    "encanastara": "encanastar",
    "encanastarais": "encanastar",
    "encanastaran": "encanastar",
    "encanastaras": "encanastar",
    "encanastare": "encanastar",
    "encanastareis": "encanastar",
    "encanastaremos": "encanastar",
    "encanastaren": "encanastar",
    "encanastares": "encanastar",
    "encanastaron": "encanastar",
    "encanastará": "encanastar",
    "encanastarán": "encanastar",
    "encanastarás": "encanastar",
    "encanastaré": "encanastar",
    "encanastaréis": "encanastar",
    "encanastaría": "encanastar",
    "encanastaríais": "encanastar",
    "encanastaríamos": "encanastar",
    "encanastarían": "encanastar",
    "encanastarías": "encanastar",
    "encanastas": "encanastar",
    "encanastase": "encanastar",
    "encanastaseis": "encanastar",
    "encanastasen": "encanastar",
    "encanastases": "encanastar",
    "encanastaste": "encanastar",
    "encanastasteis": "encanastar",
    "encanaste": "encanastar",
    "encanastemos": "encanastar",
    "encanasten": "encanastar",
    "encanastes": "encanastar",
    "encanasto": "encanastar",
    "encanastábamos": "encanastar",
    "encanastáis": "encanastar",
    "encanastáramos": "encanastar",
    "encanastáremos": "encanastar",
    "encanastásemos": "encanastar",
    "encanasté": "encanastar",
    "encanastéis": "encanastar",
    "encanastó": "encanastar",
    "encancera": "encancerar",
    "encanceraba": "encancerar",
    "encancerabais": "encancerar",
    "encanceraban": "encancerar",
    "encancerabas": "encancerar",
    "encancerad": "encancerar",
    "encancerada": "encancerar",
    "encanceradas": "encancerar",
    "encancerado": "encancerar",
    "encancerados": "encancerar",
    "encanceramos": "encancerar",
    "encanceran": "encancerar",
    "encancerando": "encancerar",
    "encancerara": "encancerar",
    "encancerarais": "encancerar",
    "encanceraran": "encancerar",
    "encanceraras": "encancerar",
    "encancerare": "encancerar",
    "encancerareis": "encancerar",
    "encanceraremos": "encancerar",
    "encanceraren": "encancerar",
    "encancerares": "encancerar",
    "encanceraron": "encancerar",
    "encancerará": "encancerar",
    "encancerarán": "encancerar",
    "encancerarás": "encancerar",
    "encanceraré": "encancerar",
    "encanceraréis": "encancerar",
    "encanceraría": "encancerar",
    "encanceraríais": "encancerar",
    "encanceraríamos": "encancerar",
    "encancerarían": "encancerar",
    "encancerarías": "encancerar",
    "encanceras": "encancerar",
    "encancerase": "encancerar",
    "encanceraseis": "encancerar",
    "encancerasen": "encancerar",
    "encancerases": "encancerar",
    "encanceraste": "encancerar",
    "encancerasteis": "encancerar",
    "encancere": "encancerar",
    "encanceremos": "encancerar",
    "encanceren": "encancerar",
    "encanceres": "encancerar",
    "encancero": "encancerar",
    "encancerábamos": "encancerar",
    "encanceráis": "encancerar",
    "encanceráramos": "encancerar",
    "encanceráremos": "encancerar",
    "encancerásemos": "encancerar",
    "encanceré": "encancerar",
    "encanceréis": "encancerar",
    "encanceró": "encancerar",
    "encandece": "encandecer",
    "encandeced": "encandecer",
    "encandecemos": "encandecer",
    "encandecen": "encandecer",
    "encandeceremos": "encandecer",
    "encandecerá": "encandecer",
    "encandecerán": "encandecer",
    "encandecerás": "encandecer",
    "encandeceré": "encandecer",
    "encandeceréis": "encandecer",
    "encandecería": "encandecer",
    "encandeceríais": "encandecer",
    "encandeceríamos": "encandecer",
    "encandecerían": "encandecer",
    "encandecerías": "encandecer",
    "encandeces": "encandecer",
    "encandecida": "encandecer",
    "encandecidas": "encandecer",
    "encandecido": "encandecer",
    "encandecidos": "encandecer",
    "encandeciendo": "encandecer",
    "encandeciera": "encandecer",
    "encandecierais": "encandecer",
    "encandecieran": "encandecer",
    "encandecieras": "encandecer",
    "encandeciere": "encandecer",
    "encandeciereis": "encandecer",
    "encandecieren": "encandecer",
    "encandecieres": "encandecer",
    "encandecieron": "encandecer",
    "encandeciese": "encandecer",
    "encandecieseis": "encandecer",
    "encandeciesen": "encandecer",
    "encandecieses": "encandecer",
    "encandecimos": "encandecer",
    "encandeciste": "encandecer",
    "encandecisteis": "encandecer",
    "encandeciéramos": "encandecer",
    "encandeciéremos": "encandecer",
    "encandeciésemos": "encandecer",
    "encandeció": "encandecer",
    "encandecéis": "encandecer",
    "encandecí": "encandecer",
    "encandecía": "encandecer",
    "encandecíais": "encandecer",
    "encandecíamos": "encandecer",
    "encandecían": "encandecer",
    "encandecías": "encandecer",
    "encandela": "encandelar",
    "encandelaba": "encandelar",
    "encandelabais": "encandelar",
    "encandelaban": "encandelar",
    "encandelabas": "encandelar",
    "encandelad": "encandelar",
    "encandelada": "encandelar",
    "encandeladas": "encandelar",
    "encandelado": "encandelar",
    "encandelados": "encandelar",
    "encandelamos": "encandelar",
    "encandelan": "encandelar",
    "encandelando": "encandelar",
    "encandelara": "encandelar",
    "encandelarais": "encandelar",
    "encandelaran": "encandelar",
    "encandelaras": "encandelar",
    "encandelare": "encandelar",
    "encandelareis": "encandelar",
    "encandelaremos": "encandelar",
    "encandelaren": "encandelar",
    "encandelares": "encandelar",
    "encandelaron": "encandelar",
    "encandelará": "encandelar",
    "encandelarán": "encandelar",
    "encandelarás": "encandelar",
    "encandelaré": "encandelar",
    "encandelaréis": "encandelar",
    "encandelaría": "encandelar",
    "encandelaríais": "encandelar",
    "encandelaríamos": "encandelar",
    "encandelarían": "encandelar",
    "encandelarías": "encandelar",
    "encandelas": "encandelar",
    "encandelase": "encandelar",
    "encandelaseis": "encandelar",
    "encandelasen": "encandelar",
    "encandelases": "encandelar",
    "encandelaste": "encandelar",
    "encandelasteis": "encandelar",
    "encandele": "encandelar",
    "encandelemos": "encandelar",
    "encandelen": "encandelar",
    "encandeles": "encandelar",
    "encandelo": "encandelar",
    "encandelábamos": "encandelar",
    "encandeláis": "encandelar",
    "encandeláramos": "encandelar",
    "encandeláremos": "encandelar",
    "encandelásemos": "encandelar",
    "encandelé": "encandelar",
    "encandeléis": "encandelar",
    "encandeló": "encandelar",
    "encandezca": "encandecer",
    "encandezcamos": "encandecer",
    "encandezcan": "encandecer",
    "encandezcas": "encandecer",
    "encandezco": "encandecer",
    "encandezcáis": "encandecer",
    "encandila": "encandilar",
    "encandilaba": "encandilar",
    "encandilabais": "encandilar",
    "encandilaban": "encandilar",
    "encandilabas": "encandilar",
    "encandilad": "encandilar",
    "encandilada": "encandilar",
    "encandiladas": "encandilar",
    "encandiladeras": "encandiladera",
    "encandilado": "encandilar",
    "encandiladora": "encandilador",
    "encandiladoras": "encandiladora",
    "encandiladores": "encandilador",
    "encandilados": "encandilar",
    "encandilamientos": "encandilamiento",
    "encandilamos": "encandilar",
    "encandilan": "encandilar",
    "encandilando": "encandilar",
    "encandilara": "encandilar",
    "encandilarais": "encandilar",
    "encandilaran": "encandilar",
    "encandilaras": "encandilar",
    "encandilare": "encandilar",
    "encandilareis": "encandilar",
    "encandilaremos": "encandilar",
    "encandilaren": "encandilar",
    "encandilares": "encandilar",
    "encandilaron": "encandilar",
    "encandilará": "encandilar",
    "encandilarán": "encandilar",
    "encandilarás": "encandilar",
    "encandilaré": "encandilar",
    "encandilaréis": "encandilar",
    "encandilaría": "encandilar",
    "encandilaríais": "encandilar",
    "encandilaríamos": "encandilar",
    "encandilarían": "encandilar",
    "encandilarías": "encandilar",
    "encandilas": "encandilar",
    "encandilase": "encandilar",
    "encandilaseis": "encandilar",
    "encandilasen": "encandilar",
    "encandilases": "encandilar",
    "encandilaste": "encandilar",
    "encandilasteis": "encandilar",
    "encandile": "encandilar",
    "encandilemos": "encandilar",
    "encandilen": "encandilar",
    "encandiles": "encandilar",
    "encandilo": "encandilar",
    "encandilábamos": "encandilar",
    "encandiláis": "encandilar",
    "encandiláramos": "encandilar",
    "encandiláremos": "encandilar",
    "encandilásemos": "encandilar",
    "encandilé": "encandilar",
    "encandiléis": "encandilar",
    "encandiló": "encandilar",
    "encanece": "encanecer",
    "encaneced": "encanecer",
    "encanecemos": "encanecer",
    "encanecen": "encanecer",
    "encaneceremos": "encanecer",
    "encanecerá": "encanecer",
    "encanecerán": "encanecer",
    "encanecerás": "encanecer",
    "encaneceré": "encanecer",
    "encaneceréis": "encanecer",
    "encanecería": "encanecer",
    "encaneceríais": "encanecer",
    "encaneceríamos": "encanecer",
    "encanecerían": "encanecer",
    "encanecerías": "encanecer",
    "encaneces": "encanecer",
    "encanecida": "encanecer",
    "encanecidas": "encanecer",
    "encanecido": "encanecer",
    "encanecidos": "encanecer",
    "encaneciendo": "encanecer",
    "encaneciera": "encanecer",
    "encanecierais": "encanecer",
    "encanecieran": "encanecer",
    "encanecieras": "encanecer",
    "encaneciere": "encanecer",
    "encaneciereis": "encanecer",
    "encanecieren": "encanecer",
    "encanecieres": "encanecer",
    "encanecieron": "encanecer",
    "encaneciese": "encanecer",
    "encanecieseis": "encanecer",
    "encaneciesen": "encanecer",
    "encanecieses": "encanecer",
    "encanecimientos": "encanecimiento",
    "encanecimos": "encanecer",
    "encaneciste": "encanecer",
    "encanecisteis": "encanecer",
    "encaneciéramos": "encanecer",
    "encaneciéremos": "encanecer",
    "encaneciésemos": "encanecer",
    "encaneció": "encanecer",
    "encanecéis": "encanecer",
    "encanecí": "encanecer",
    "encanecía": "encanecer",
    "encanecíais": "encanecer",
    "encanecíamos": "encanecer",
    "encanecían": "encanecer",
    "encanecías": "encanecer",
    "encanezca": "encanecer",
    "encanezcamos": "encanecer",
    "encanezcan": "encanecer",
    "encanezcas": "encanecer",
    "encanezco": "encanecer",
    "encanezcáis": "encanecer",
    "encanija": "encanijar",
    "encanijaba": "encanijar",
    "encanijabais": "encanijar",
    "encanijaban": "encanijar",
    "encanijabas": "encanijar",
    "encanijad": "encanijar",
    "encanijada": "encanijar",
    "encanijadas": "encanijar",
    "encanijado": "encanijar",
    "encanijados": "encanijar",
    "encanijamientos": "encanijamiento",
    "encanijamos": "encanijar",
    "encanijan": "encanijar",
    "encanijando": "encanijar",
    "encanijara": "encanijar",
    "encanijarais": "encanijar",
    "encanijaran": "encanijar",
    "encanijaras": "encanijar",
    "encanijare": "encanijar",
    "encanijareis": "encanijar",
    "encanijaremos": "encanijar",
    "encanijaren": "encanijar",
    "encanijares": "encanijar",
    "encanijaron": "encanijar",
    "encanijará": "encanijar",
    "encanijarán": "encanijar",
    "encanijarás": "encanijar",
    "encanijaré": "encanijar",
    "encanijaréis": "encanijar",
    "encanijaría": "encanijar",
    "encanijaríais": "encanijar",
    "encanijaríamos": "encanijar",
    "encanijarían": "encanijar",
    "encanijarías": "encanijar",
    "encanijas": "encanijar",
    "encanijase": "encanijar",
    "encanijaseis": "encanijar",
    "encanijasen": "encanijar",
    "encanijases": "encanijar",
    "encanijaste": "encanijar",
    "encanijasteis": "encanijar",
    "encanije": "encanijar",
    "encanijemos": "encanijar",
    "encanijen": "encanijar",
    "encanijes": "encanijar",
    "encanijo": "encanijar",
    "encanijábamos": "encanijar",
    "encanijáis": "encanijar",
    "encanijáramos": "encanijar",
    "encanijáremos": "encanijar",
    "encanijásemos": "encanijar",
    "encanijé": "encanijar",
    "encanijéis": "encanijar",
    "encanijó": "encanijar",
    "encanilla": "encanillar",
    "encanillaba": "encanillar",
    "encanillabais": "encanillar",
    "encanillaban": "encanillar",
    "encanillabas": "encanillar",
    "encanillad": "encanillar",
    "encanillada": "encanillar",
    "encanilladas": "encanillar",
    "encanillado": "encanillar",
    "encanillados": "encanillar",
    "encanillamos": "encanillar",
    "encanillan": "encanillar",
    "encanillando": "encanillar",
    "encanillara": "encanillar",
    "encanillarais": "encanillar",
    "encanillaran": "encanillar",
    "encanillaras": "encanillar",
    "encanillare": "encanillar",
    "encanillareis": "encanillar",
    "encanillaremos": "encanillar",
    "encanillaren": "encanillar",
    "encanillares": "encanillar",
    "encanillaron": "encanillar",
    "encanillará": "encanillar",
    "encanillarán": "encanillar",
    "encanillarás": "encanillar",
    "encanillaré": "encanillar",
    "encanillaréis": "encanillar",
    "encanillaría": "encanillar",
    "encanillaríais": "encanillar",
    "encanillaríamos": "encanillar",
    "encanillarían": "encanillar",
    "encanillarías": "encanillar",
    "encanillas": "encanillar",
    "encanillase": "encanillar",
    "encanillaseis": "encanillar",
    "encanillasen": "encanillar",
    "encanillases": "encanillar",
    "encanillaste": "encanillar",
    "encanillasteis": "encanillar",
    "encanille": "encanillar",
    "encanillemos": "encanillar",
    "encanillen": "encanillar",
    "encanilles": "encanillar",
    "encanillo": "encanillar",
    "encanillábamos": "encanillar",
    "encanilláis": "encanillar",
    "encanilláramos": "encanillar",
    "encanilláremos": "encanillar",
    "encanillásemos": "encanillar",
    "encanillé": "encanillar",
    "encanilléis": "encanillar",
    "encanilló": "encanillar",
    "encanta": "encantar",
    "encantaba": "encantar",
    "encantabais": "encantar",
    "encantaban": "encantar",
    "encantabas": "encantar",
    "encantad": "encantar",
    "encantada": "encantar",
    "encantadas": "encantar",
    "encantado": "encantar",
    "encantadora": "encantador",
    "encantadoras": "encantador",
    "encantadores": "encantador",
    "encantados": "encantar",
    "encantamientos": "encantamiento",
    "encantamos": "encantar",
    "encantan": "encantar",
    "encantando": "encantar",
    "encantara": "encantarar",
    "encantaraba": "encantarar",
    "encantarabais": "encantarar",
    "encantaraban": "encantarar",
    "encantarabas": "encantarar",
    "encantarad": "encantarar",
    "encantarada": "encantarar",
    "encantaradas": "encantarar",
    "encantarado": "encantarar",
    "encantarados": "encantarar",
    "encantarais": "encantar",
    "encantaramos": "encantarar",
    "encantaran": "encantarar",
    "encantarando": "encantarar",
    "encantarara": "encantarar",
    "encantararais": "encantarar",
    "encantararan": "encantarar",
    "encantararas": "encantarar",
    "encantarare": "encantarar",
    "encantarareis": "encantarar",
    "encantararemos": "encantarar",
    "encantararen": "encantarar",
    "encantarares": "encantarar",
    "encantararon": "encantarar",
    "encantarará": "encantarar",
    "encantararán": "encantarar",
    "encantararás": "encantarar",
    "encantararé": "encantarar",
    "encantararéis": "encantarar",
    "encantararía": "encantarar",
    "encantararíais": "encantarar",
    "encantararíamos": "encantarar",
    "encantararían": "encantarar",
    "encantararías": "encantarar",
    "encantaras": "encantarar",
    "encantarase": "encantarar",
    "encantaraseis": "encantarar",
    "encantarasen": "encantarar",
    "encantarases": "encantarar",
    "encantaraste": "encantarar",
    "encantarasteis": "encantarar",
    "encantare": "encantarar",
    "encantareis": "encantar",
    "encantaremos": "encantarar",
    "encantaren": "encantarar",
    "encantares": "encantarar",
    "encantaro": "encantarar",
    "encantaron": "encantar",
    "encantará": "encantar",
    "encantarábamos": "encantarar",
    "encantaráis": "encantarar",
    "encantarán": "encantar",
    "encantaráramos": "encantarar",
    "encantaráremos": "encantarar",
    "encantarás": "encantar",
    "encantarásemos": "encantarar",
    "encantaré": "encantarar",
    "encantaréis": "encantarar",
    "encantaría": "encantar",
    "encantaríais": "encantar",
    "encantaríamos": "encantar",
    "encantarían": "encantar",
    "encantarías": "encantar",
    "encantaró": "encantarar",
    "encantas": "encantar",
    "encantase": "encantar",
    "encantaseis": "encantar",
    "encantasen": "encantar",
    "encantases": "encantar",
    "encantaste": "encantar",
    "encantasteis": "encantar",
    "encante": "encantar",
    "encantemos": "encantar",
    "encanten": "encantar",
    "encantes": "encante",
    "encanto": "encantar",
    "encantos": "encanto",
    "encantábamos": "encantar",
    "encantáis": "encantar",
    "encantáramos": "encantar",
    "encantáremos": "encantar",
    "encantásemos": "encantar",
    "encanté": "encantar",
    "encantéis": "encantar",
    "encantó": "encantar",
    "encanuta": "encanutar",
    "encanutaba": "encanutar",
    "encanutabais": "encanutar",
    "encanutaban": "encanutar",
    "encanutabas": "encanutar",
    "encanutad": "encanutar",
    "encanutada": "encanutar",
    "encanutadas": "encanutar",
    "encanutado": "encanutar",
    "encanutados": "encanutar",
    "encanutamos": "encanutar",
    "encanutan": "encanutar",
    "encanutando": "encanutar",
    "encanutara": "encanutar",
    "encanutarais": "encanutar",
    "encanutaran": "encanutar",
    "encanutaras": "encanutar",
    "encanutare": "encanutar",
    "encanutareis": "encanutar",
    "encanutaremos": "encanutar",
    "encanutaren": "encanutar",
    "encanutares": "encanutar",
    "encanutaron": "encanutar",
    "encanutará": "encanutar",
    "encanutarán": "encanutar",
    "encanutarás": "encanutar",
    "encanutaré": "encanutar",
    "encanutaréis": "encanutar",
    "encanutaría": "encanutar",
    "encanutaríais": "encanutar",
    "encanutaríamos": "encanutar",
    "encanutarían": "encanutar",
    "encanutarías": "encanutar",
    "encanutas": "encanutar",
    "encanutase": "encanutar",
    "encanutaseis": "encanutar",
    "encanutasen": "encanutar",
    "encanutases": "encanutar",
    "encanutaste": "encanutar",
    "encanutasteis": "encanutar",
    "encanute": "encanutar",
    "encanutemos": "encanutar",
    "encanuten": "encanutar",
    "encanutes": "encanutar",
    "encanuto": "encanutar",
    "encanutábamos": "encanutar",
    "encanutáis": "encanutar",
    "encanutáramos": "encanutar",
    "encanutáremos": "encanutar",
    "encanutásemos": "encanutar",
    "encanuté": "encanutar",
    "encanutéis": "encanutar",
    "encanutó": "encanutar",
    "encapacetada": "encapacetado",
    "encapacetadas": "encapacetado",
    "encapacetados": "encapacetado",
    "encapacha": "encapachar",
    "encapachaba": "encapachar",
    "encapachabais": "encapachar",
    "encapachaban": "encapachar",
    "encapachabas": "encapachar",
    "encapachad": "encapachar",
    "encapachada": "encapachar",
    "encapachadas": "encapachar",
    "encapachado": "encapachar",
    "encapachados": "encapachar",
    "encapachaduras": "encapachadura",
    "encapachamos": "encapachar",
    "encapachan": "encapachar",
    "encapachando": "encapachar",
    "encapachara": "encapachar",
    "encapacharais": "encapachar",
    "encapacharan": "encapachar",
    "encapacharas": "encapachar",
    "encapachare": "encapachar",
    "encapachareis": "encapachar",
    "encapacharemos": "encapachar",
    "encapacharen": "encapachar",
    "encapachares": "encapachar",
    "encapacharon": "encapachar",
    "encapachará": "encapachar",
    "encapacharán": "encapachar",
    "encapacharás": "encapachar",
    "encapacharé": "encapachar",
    "encapacharéis": "encapachar",
    "encapacharía": "encapachar",
    "encapacharíais": "encapachar",
    "encapacharíamos": "encapachar",
    "encapacharían": "encapachar",
    "encapacharías": "encapachar",
    "encapachas": "encapachar",
    "encapachase": "encapachar",
    "encapachaseis": "encapachar",
    "encapachasen": "encapachar",
    "encapachases": "encapachar",
    "encapachaste": "encapachar",
    "encapachasteis": "encapachar",
    "encapache": "encapachar",
    "encapachemos": "encapachar",
    "encapachen": "encapachar",
    "encapaches": "encapachar",
    "encapacho": "encapachar",
    "encapachábamos": "encapachar",
    "encapacháis": "encapachar",
    "encapacháramos": "encapachar",
    "encapacháremos": "encapachar",
    "encapachásemos": "encapachar",
    "encapaché": "encapachar",
    "encapachéis": "encapachar",
    "encapachó": "encapachar",
    "encaparazona": "encaparazonar",
    "encaparazonaba": "encaparazonar",
    "encaparazonabais": "encaparazonar",
    "encaparazonaban": "encaparazonar",
    "encaparazonabas": "encaparazonar",
    "encaparazonad": "encaparazonar",
    "encaparazonada": "encaparazonar",
    "encaparazonadas": "encaparazonar",
    "encaparazonado": "encaparazonar",
    "encaparazonados": "encaparazonar",
    "encaparazonamos": "encaparazonar",
    "encaparazonan": "encaparazonar",
    "encaparazonando": "encaparazonar",
    "encaparazonara": "encaparazonar",
    "encaparazonarais": "encaparazonar",
    "encaparazonaran": "encaparazonar",
    "encaparazonaras": "encaparazonar",
    "encaparazonare": "encaparazonar",
    "encaparazonareis": "encaparazonar",
    "encaparazonaremos": "encaparazonar",
    "encaparazonaren": "encaparazonar",
    "encaparazonares": "encaparazonar",
    "encaparazonaron": "encaparazonar",
    "encaparazonará": "encaparazonar",
    "encaparazonarán": "encaparazonar",
    "encaparazonarás": "encaparazonar",
    "encaparazonaré": "encaparazonar",
    "encaparazonaréis": "encaparazonar",
    "encaparazonaría": "encaparazonar",
    "encaparazonaríais": "encaparazonar",
    "encaparazonaríamos": "encaparazonar",
    "encaparazonarían": "encaparazonar",
    "encaparazonarías": "encaparazonar",
    "encaparazonas": "encaparazonar",
    "encaparazonase": "encaparazonar",
    "encaparazonaseis": "encaparazonar",
    "encaparazonasen": "encaparazonar",
    "encaparazonases": "encaparazonar",
    "encaparazonaste": "encaparazonar",
    "encaparazonasteis": "encaparazonar",
    "encaparazone": "encaparazonar",
    "encaparazonemos": "encaparazonar",
    "encaparazonen": "encaparazonar",
    "encaparazones": "encaparazonar",
    "encaparazono": "encaparazonar",
    "encaparazonábamos": "encaparazonar",
    "encaparazonáis": "encaparazonar",
    "encaparazonáramos": "encaparazonar",
    "encaparazonáremos": "encaparazonar",
    "encaparazonásemos": "encaparazonar",
    "encaparazoné": "encaparazonar",
    "encaparazonéis": "encaparazonar",
    "encaparazonó": "encaparazonar",
    "encaperuzamientos": "encaperuzamiento",
    "encapilla": "encapillar",
    "encapillaba": "encapillar",
    "encapillabais": "encapillar",
    "encapillaban": "encapillar",
    "encapillabas": "encapillar",
    "encapillad": "encapillar",
    "encapillada": "encapillar",
    "encapilladas": "encapillar",
    "encapillado": "encapillar",
    "encapillados": "encapillar",
    "encapilladuras": "encapilladura",
    "encapillamos": "encapillar",
    "encapillan": "encapillar",
    "encapillando": "encapillar",
    "encapillara": "encapillar",
    "encapillarais": "encapillar",
    "encapillaran": "encapillar",
    "encapillaras": "encapillar",
    "encapillare": "encapillar",
    "encapillareis": "encapillar",
    "encapillaremos": "encapillar",
    "encapillaren": "encapillar",
    "encapillares": "encapillar",
    "encapillaron": "encapillar",
    "encapillará": "encapillar",
    "encapillarán": "encapillar",
    "encapillarás": "encapillar",
    "encapillaré": "encapillar",
    "encapillaréis": "encapillar",
    "encapillaría": "encapillar",
    "encapillaríais": "encapillar",
    "encapillaríamos": "encapillar",
    "encapillarían": "encapillar",
    "encapillarías": "encapillar",
    "encapillas": "encapillar",
    "encapillase": "encapillar",
    "encapillaseis": "encapillar",
    "encapillasen": "encapillar",
    "encapillases": "encapillar",
    "encapillaste": "encapillar",
    "encapillasteis": "encapillar",
    "encapille": "encapillar",
    "encapillemos": "encapillar",
    "encapillen": "encapillar",
    "encapilles": "encapillar",
    "encapillo": "encapillar",
    "encapillábamos": "encapillar",
    "encapilláis": "encapillar",
    "encapilláramos": "encapillar",
    "encapilláremos": "encapillar",
    "encapillásemos": "encapillar",
    "encapillé": "encapillar",
    "encapilléis": "encapillar",
    "encapilló": "encapillar",
    "encapirota": "encapirotar",
    "encapirotaba": "encapirotar",
    "encapirotabais": "encapirotar",
    "encapirotaban": "encapirotar",
    "encapirotabas": "encapirotar",
    "encapirotad": "encapirotar",
    "encapirotada": "encapirotar",
    "encapirotadas": "encapirotar",
    "encapirotado": "encapirotar",
    "encapirotados": "encapirotar",
    "encapirotamos": "encapirotar",
    "encapirotan": "encapirotar",
    "encapirotando": "encapirotar",
    "encapirotara": "encapirotar",
    "encapirotarais": "encapirotar",
    "encapirotaran": "encapirotar",
    "encapirotaras": "encapirotar",
    "encapirotare": "encapirotar",
    "encapirotareis": "encapirotar",
    "encapirotaremos": "encapirotar",
    "encapirotaren": "encapirotar",
    "encapirotares": "encapirotar",
    "encapirotaron": "encapirotar",
    "encapirotará": "encapirotar",
    "encapirotarán": "encapirotar",
    "encapirotarás": "encapirotar",
    "encapirotaré": "encapirotar",
    "encapirotaréis": "encapirotar",
    "encapirotaría": "encapirotar",
    "encapirotaríais": "encapirotar",
    "encapirotaríamos": "encapirotar",
    "encapirotarían": "encapirotar",
    "encapirotarías": "encapirotar",
    "encapirotas": "encapirotar",
    "encapirotase": "encapirotar",
    "encapirotaseis": "encapirotar",
    "encapirotasen": "encapirotar",
    "encapirotases": "encapirotar",
    "encapirotaste": "encapirotar",
    "encapirotasteis": "encapirotar",
    "encapirote": "encapirotar",
    "encapirotemos": "encapirotar",
    "encapiroten": "encapirotar",
    "encapirotes": "encapirotar",
    "encapiroto": "encapirotar",
    "encapirotábamos": "encapirotar",
    "encapirotáis": "encapirotar",
    "encapirotáramos": "encapirotar",
    "encapirotáremos": "encapirotar",
    "encapirotásemos": "encapirotar",
    "encapiroté": "encapirotar",
    "encapirotéis": "encapirotar",
    "encapirotó": "encapirotar",
    "encapota": "encapotar",
    "encapotaba": "encapotar",
    "encapotabais": "encapotar",
    "encapotaban": "encapotar",
    "encapotabas": "encapotar",
    "encapotad": "encapotar",
    "encapotada": "encapotar",
    "encapotadas": "encapotar",
    "encapotado": "encapotar",
    "encapotados": "encapotar",
    "encapotaduras": "encapotadura",
    "encapotamientos": "encapotamiento",
    "encapotamos": "encapotar",
    "encapotan": "encapotar",
    "encapotando": "encapotar",
    "encapotara": "encapotar",
    "encapotarais": "encapotar",
    "encapotaran": "encapotar",
    "encapotaras": "encapotar",
    "encapotare": "encapotar",
    "encapotareis": "encapotar",
    "encapotaremos": "encapotar",
    "encapotaren": "encapotar",
    "encapotares": "encapotar",
    "encapotaron": "encapotar",
    "encapotará": "encapotar",
    "encapotarán": "encapotar",
    "encapotarás": "encapotar",
    "encapotaré": "encapotar",
    "encapotaréis": "encapotar",
    "encapotaría": "encapotar",
    "encapotaríais": "encapotar",
    "encapotaríamos": "encapotar",
    "encapotarían": "encapotar",
    "encapotarías": "encapotar",
    "encapotas": "encapotar",
    "encapotase": "encapotar",
    "encapotaseis": "encapotar",
    "encapotasen": "encapotar",
    "encapotases": "encapotar",
    "encapotaste": "encapotar",
    "encapotasteis": "encapotar",
    "encapote": "encapotar",
    "encapotemos": "encapotar",
    "encapoten": "encapotar",
    "encapotes": "encapotar",
    "encapoto": "encapotar",
    "encapotábamos": "encapotar",
    "encapotáis": "encapotar",
    "encapotáramos": "encapotar",
    "encapotáremos": "encapotar",
    "encapotásemos": "encapotar",
    "encapoté": "encapotar",
    "encapotéis": "encapotar",
    "encapotó": "encapotar",
    "encapricha": "encaprichar",
    "encaprichaba": "encaprichar",
    "encaprichabais": "encaprichar",
    "encaprichaban": "encaprichar",
    "encaprichabas": "encaprichar",
    "encaprichad": "encaprichar",
    "encaprichada": "encaprichar",
    "encaprichadas": "encaprichar",
    "encaprichado": "encaprichar",
    "encaprichados": "encaprichar",
    "encaprichamientos": "encaprichamiento",
    "encaprichamos": "encaprichar",
    "encaprichan": "encaprichar",
    "encaprichando": "encaprichar",
    "encaprichara": "encaprichar",
    "encapricharais": "encaprichar",
    "encapricharan": "encaprichar",
    "encapricharas": "encaprichar",
    "encaprichare": "encaprichar",
    "encaprichareis": "encaprichar",
    "encapricharemos": "encaprichar",
    "encapricharen": "encaprichar",
    "encaprichares": "encaprichar",
    "encapricharon": "encaprichar",
    "encaprichará": "encaprichar",
    "encapricharán": "encaprichar",
    "encapricharás": "encaprichar",
    "encapricharé": "encaprichar",
    "encapricharéis": "encaprichar",
    "encapricharía": "encaprichar",
    "encapricharíais": "encaprichar",
    "encapricharíamos": "encaprichar",
    "encapricharían": "encaprichar",
    "encapricharías": "encaprichar",
    "encaprichas": "encaprichar",
    "encaprichase": "encaprichar",
    "encaprichaseis": "encaprichar",
    "encaprichasen": "encaprichar",
    "encaprichases": "encaprichar",
    "encaprichaste": "encaprichar",
    "encaprichasteis": "encaprichar",
    "encapriche": "encaprichar",
    "encaprichemos": "encaprichar",
    "encaprichen": "encaprichar",
    "encapriches": "encaprichar",
    "encapricho": "encaprichar",
    "encaprichábamos": "encaprichar",
    "encapricháis": "encaprichar",
    "encapricháramos": "encaprichar",
    "encapricháremos": "encaprichar",
    "encaprichásemos": "encaprichar",
    "encapriché": "encaprichar",
    "encaprichéis": "encaprichar",
    "encaprichó": "encaprichar",
    "encapsula": "encapsular",
    "encapsulaba": "encapsular",
    "encapsulabais": "encapsular",
    "encapsulaban": "encapsular",
    "encapsulabas": "encapsular",
    "encapsulaciones": "encapsulación",
    "encapsulad": "encapsular",
    "encapsulada": "encapsular",
    "encapsuladas": "encapsular",
    "encapsulado": "encapsular",
    "encapsulados": "encapsular",
    "encapsulamientos": "encapsulamiento",
    "encapsulamos": "encapsular",
    "encapsulan": "encapsular",
    "encapsulando": "encapsular",
    "encapsulara": "encapsular",
    "encapsularais": "encapsular",
    "encapsularan": "encapsular",
    "encapsularas": "encapsular",
    "encapsulare": "encapsular",
    "encapsulareis": "encapsular",
    "encapsularemos": "encapsular",
    "encapsularen": "encapsular",
    "encapsulares": "encapsular",
    "encapsularon": "encapsular",
    "encapsulará": "encapsular",
    "encapsularán": "encapsular",
    "encapsularás": "encapsular",
    "encapsularé": "encapsular",
    "encapsularéis": "encapsular",
    "encapsularía": "encapsular",
    "encapsularíais": "encapsular",
    "encapsularíamos": "encapsular",
    "encapsularían": "encapsular",
    "encapsularías": "encapsular",
    "encapsulas": "encapsular",
    "encapsulase": "encapsular",
    "encapsulaseis": "encapsular",
    "encapsulasen": "encapsular",
    "encapsulases": "encapsular",
    "encapsulaste": "encapsular",
    "encapsulasteis": "encapsular",
    "encapsule": "encapsular",
    "encapsulemos": "encapsular",
    "encapsulen": "encapsular",
    "encapsules": "encapsular",
    "encapsulo": "encapsular",
    "encapsulábamos": "encapsular",
    "encapsuláis": "encapsular",
    "encapsuláramos": "encapsular",
    "encapsuláremos": "encapsular",
    "encapsulásemos": "encapsular",
    "encapsulé": "encapsular",
    "encapsuléis": "encapsular",
    "encapsuló": "encapsular",
    "encapuce": "encapuzar",
    "encapucemos": "encapuzar",
    "encapucen": "encapuzar",
    "encapuces": "encapuzar",
    "encapucha": "encapuchar",
    "encapuchaba": "encapuchar",
    "encapuchabais": "encapuchar",
    "encapuchaban": "encapuchar",
    "encapuchabas": "encapuchar",
    "encapuchad": "encapuchar",
    "encapuchada": "encapuchar",
    "encapuchadas": "encapuchar",
    "encapuchado": "encapuchar",
    "encapuchados": "encapuchar",
    "encapuchamos": "encapuchar",
    "encapuchan": "encapuchar",
    "encapuchando": "encapuchar",
    "encapuchara": "encapuchar",
    "encapucharais": "encapuchar",
    "encapucharan": "encapuchar",
    "encapucharas": "encapuchar",
    "encapuchare": "encapuchar",
    "encapuchareis": "encapuchar",
    "encapucharemos": "encapuchar",
    "encapucharen": "encapuchar",
    "encapuchares": "encapuchar",
    "encapucharon": "encapuchar",
    "encapuchará": "encapuchar",
    "encapucharán": "encapuchar",
    "encapucharás": "encapuchar",
    "encapucharé": "encapuchar",
    "encapucharéis": "encapuchar",
    "encapucharía": "encapuchar",
    "encapucharíais": "encapuchar",
    "encapucharíamos": "encapuchar",
    "encapucharían": "encapuchar",
    "encapucharías": "encapuchar",
    "encapuchas": "encapuchar",
    "encapuchase": "encapuchar",
    "encapuchaseis": "encapuchar",
    "encapuchasen": "encapuchar",
    "encapuchases": "encapuchar",
    "encapuchaste": "encapuchar",
    "encapuchasteis": "encapuchar",
    "encapuche": "encapuchar",
    "encapuchemos": "encapuchar",
    "encapuchen": "encapuchar",
    "encapuches": "encapuchar",
    "encapucho": "encapuchar",
    "encapuchábamos": "encapuchar",
    "encapucháis": "encapuchar",
    "encapucháramos": "encapuchar",
    "encapucháremos": "encapuchar",
    "encapuchásemos": "encapuchar",
    "encapuché": "encapuchar",
    "encapuchéis": "encapuchar",
    "encapuchó": "encapuchar",
    "encapucé": "encapuzar",
    "encapucéis": "encapuzar",
    "encapullada": "encapullado",
    "encapulladas": "encapullado",
    "encapullados": "encapullado",
    "encapuza": "encapuzar",
    "encapuzaba": "encapuzar",
    "encapuzabais": "encapuzar",
    "encapuzaban": "encapuzar",
    "encapuzabas": "encapuzar",
    "encapuzad": "encapuzar",
    "encapuzada": "encapuzar",
    "encapuzadas": "encapuzar",
    "encapuzado": "encapuzar",
    "encapuzados": "encapuzar",
    "encapuzamos": "encapuzar",
    "encapuzan": "encapuzar",
    "encapuzando": "encapuzar",
    "encapuzara": "encapuzar",
    "encapuzarais": "encapuzar",
    "encapuzaran": "encapuzar",
    "encapuzaras": "encapuzar",
    "encapuzare": "encapuzar",
    "encapuzareis": "encapuzar",
    "encapuzaremos": "encapuzar",
    "encapuzaren": "encapuzar",
    "encapuzares": "encapuzar",
    "encapuzaron": "encapuzar",
    "encapuzará": "encapuzar",
    "encapuzarán": "encapuzar",
    "encapuzarás": "encapuzar",
    "encapuzaré": "encapuzar",
    "encapuzaréis": "encapuzar",
    "encapuzaría": "encapuzar",
    "encapuzaríais": "encapuzar",
    "encapuzaríamos": "encapuzar",
    "encapuzarían": "encapuzar",
    "encapuzarías": "encapuzar",
    "encapuzas": "encapuzar",
    "encapuzase": "encapuzar",
    "encapuzaseis": "encapuzar",
    "encapuzasen": "encapuzar",
    "encapuzases": "encapuzar",
    "encapuzaste": "encapuzar",
    "encapuzasteis": "encapuzar",
    "encapuzo": "encapuzar",
    "encapuzábamos": "encapuzar",
    "encapuzáis": "encapuzar",
    "encapuzáramos": "encapuzar",
    "encapuzáremos": "encapuzar",
    "encapuzásemos": "encapuzar",
    "encapuzó": "encapuzar",
    "encara": "encarar",
    "encaraba": "encarar",
    "encarabais": "encarar",
    "encaraban": "encarar",
    "encarabas": "encarar",
    "encarad": "encarar",
    "encarada": "encarar",
    "encaradas": "encarar",
    "encarado": "encarar",
    "encarados": "encarar",
    "encarama": "encaramar",
    "encaramaba": "encaramar",
    "encaramabais": "encaramar",
    "encaramaban": "encaramar",
    "encaramabas": "encaramar",
    "encaramad": "encaramar",
    "encaramada": "encaramar",
    "encaramadas": "encaramar",
    "encaramado": "encaramar",
    "encaramados": "encaramar",
    "encaramamos": "encaramar",
    "encaraman": "encaramar",
    "encaramando": "encaramar",
    "encaramara": "encaramar",
    "encaramarais": "encaramar",
    "encaramaran": "encaramar",
    "encaramaras": "encaramar",
    "encaramare": "encaramar",
    "encaramareis": "encaramar",
    "encaramaremos": "encaramar",
    "encaramaren": "encaramar",
    "encaramares": "encaramar",
    "encaramaron": "encaramar",
    "encaramará": "encaramar",
    "encaramarán": "encaramar",
    "encaramarás": "encaramar",
    "encaramaré": "encaramar",
    "encaramaréis": "encaramar",
    "encaramaría": "encaramar",
    "encaramaríais": "encaramar",
    "encaramaríamos": "encaramar",
    "encaramarían": "encaramar",
    "encaramarías": "encaramar",
    "encaramas": "encaramar",
    "encaramase": "encaramar",
    "encaramaseis": "encaramar",
    "encaramasen": "encaramar",
    "encaramases": "encaramar",
    "encaramaste": "encaramar",
    "encaramasteis": "encaramar",
    "encarame": "encaramar",
    "encaramemos": "encaramar",
    "encaramen": "encaramar",
    "encarames": "encaramar",
    "encaramientos": "encaramiento",
    "encaramo": "encaramar",
    "encaramos": "encarar",
    "encaramábamos": "encaramar",
    "encaramáis": "encaramar",
    "encaramáramos": "encaramar",
    "encaramáremos": "encaramar",
    "encaramásemos": "encaramar",
    "encaramé": "encaramar",
    "encaraméis": "encaramar",
    "encaramó": "encaramar",
    "encaran": "encarar",
    "encarando": "encarar",
    "encarara": "encarar",
    "encararais": "encarar",
    "encararan": "encarar",
    "encararas": "encarar",
    "encarare": "encarar",
    "encarareis": "encarar",
    "encararemos": "encarar",
    "encararen": "encarar",
    "encarares": "encarar",
    "encararon": "encarar",
    "encarará": "encarar",
    "encararán": "encarar",
    "encararás": "encarar",
    "encararé": "encarar",
    "encararéis": "encarar",
    "encararía": "encarar",
    "encararíais": "encarar",
    "encararíamos": "encarar",
    "encararían": "encarar",
    "encararías": "encarar",
    "encaras": "encarar",
    "encarase": "encarar",
    "encaraseis": "encarar",
    "encarasen": "encarar",
    "encarases": "encarar",
    "encaraste": "encarar",
    "encarasteis": "encarar",
    "encarcela": "encarcelar",
    "encarcelaba": "encarcelar",
    "encarcelabais": "encarcelar",
    "encarcelaban": "encarcelar",
    "encarcelabas": "encarcelar",
    "encarcelaciones": "encarcelación",
    "encarcelad": "encarcelar",
    "encarcelada": "encarcelar",
    "encarceladas": "encarcelar",
    "encarcelado": "encarcelar",
    "encarceladora": "encarcelador",
    "encarceladoras": "encarcelador",
    "encarceladores": "encarcelador",
    "encarcelados": "encarcelar",
    "encarcelamientos": "encarcelamiento",
    "encarcelamos": "encarcelar",
    "encarcelan": "encarcelar",
    "encarcelando": "encarcelar",
    "encarcelara": "encarcelar",
    "encarcelarais": "encarcelar",
    "encarcelaran": "encarcelar",
    "encarcelaras": "encarcelar",
    "encarcelare": "encarcelar",
    "encarcelareis": "encarcelar",
    "encarcelaremos": "encarcelar",
    "encarcelaren": "encarcelar",
    "encarcelares": "encarcelar",
    "encarcelaron": "encarcelar",
    "encarcelará": "encarcelar",
    "encarcelarán": "encarcelar",
    "encarcelarás": "encarcelar",
    "encarcelaré": "encarcelar",
    "encarcelaréis": "encarcelar",
    "encarcelaría": "encarcelar",
    "encarcelaríais": "encarcelar",
    "encarcelaríamos": "encarcelar",
    "encarcelarían": "encarcelar",
    "encarcelarías": "encarcelar",
    "encarcelas": "encarcelar",
    "encarcelase": "encarcelar",
    "encarcelaseis": "encarcelar",
    "encarcelasen": "encarcelar",
    "encarcelases": "encarcelar",
    "encarcelaste": "encarcelar",
    "encarcelasteis": "encarcelar",
    "encarcele": "encarcelar",
    "encarcelemos": "encarcelar",
    "encarcelen": "encarcelar",
    "encarceles": "encarcelar",
    "encarcelo": "encarcelar",
    "encarcelábamos": "encarcelar",
    "encarceláis": "encarcelar",
    "encarceláramos": "encarcelar",
    "encarceláremos": "encarcelar",
    "encarcelásemos": "encarcelar",
    "encarcelé": "encarcelar",
    "encarceléis": "encarcelar",
    "encarceló": "encarcelar",
    "encare": "encarar",
    "encarece": "encarecer",
    "encareced": "encarecer",
    "encarecedora": "encarecedor",
    "encarecedoras": "encarecedor",
    "encarecedores": "encarecedor",
    "encarecemos": "encarecer",
    "encarecen": "encarecer",
    "encareceremos": "encarecer",
    "encarecerá": "encarecer",
    "encarecerán": "encarecer",
    "encarecerás": "encarecer",
    "encareceré": "encarecer",
    "encareceréis": "encarecer",
    "encarecería": "encarecer",
    "encareceríais": "encarecer",
    "encareceríamos": "encarecer",
    "encarecerían": "encarecer",
    "encarecerías": "encarecer",
    "encareces": "encarecer",
    "encarecida": "encarecer",
    "encarecidas": "encarecer",
    "encarecido": "encarecer",
    "encarecidos": "encarecer",
    "encareciendo": "encarecer",
    "encareciera": "encarecer",
    "encarecierais": "encarecer",
    "encarecieran": "encarecer",
    "encarecieras": "encarecer",
    "encareciere": "encarecer",
    "encareciereis": "encarecer",
    "encarecieren": "encarecer",
    "encarecieres": "encarecer",
    "encarecieron": "encarecer",
    "encareciese": "encarecer",
    "encarecieseis": "encarecer",
    "encareciesen": "encarecer",
    "encarecieses": "encarecer",
    "encarecimientos": "encarecimiento",
    "encarecimos": "encarecer",
    "encareciste": "encarecer",
    "encarecisteis": "encarecer",
    "encareciéramos": "encarecer",
    "encareciéremos": "encarecer",
    "encareciésemos": "encarecer",
    "encareció": "encarecer",
    "encarecéis": "encarecer",
    "encarecí": "encarecer",
    "encarecía": "encarecer",
    "encarecíais": "encarecer",
    "encarecíamos": "encarecer",
    "encarecían": "encarecer",
    "encarecías": "encarecer",
    "encaremos": "encarar",
    "encaren": "encarar",
    "encares": "encarar",
    "encarezca": "encarecer",
    "encarezcamos": "encarecer",
    "encarezcan": "encarecer",
    "encarezcas": "encarecer",
    "encarezco": "encarecer",
    "encarezcáis": "encarecer",
    "encarga": "encargar",
    "encargaba": "encargar",
    "encargabais": "encargar",
    "encargaban": "encargar",
    "encargabas": "encargar",
    "encargad": "encargar",
    "encargada": "encargar",
    "encargadas": "encargar",
    "encargado": "encargar",
    "encargados": "encargar",
    "encargamos": "encargar",
    "encargan": "encargar",
    "encargando": "encargar",
    "encargara": "encargar",
    "encargarais": "encargar",
    "encargaran": "encargar",
    "encargaras": "encargar",
    "encargare": "encargar",
    "encargareis": "encargar",
    "encargaremos": "encargar",
    "encargaren": "encargar",
    "encargares": "encargar",
    "encargaron": "encargar",
    "encargará": "encargar",
    "encargarán": "encargar",
    "encargarás": "encargar",
    "encargaré": "encargar",
    "encargaréis": "encargar",
    "encargaría": "encargar",
    "encargaríais": "encargar",
    "encargaríamos": "encargar",
    "encargarían": "encargar",
    "encargarías": "encargar",
    "encargas": "encargar",
    "encargase": "encargar",
    "encargaseis": "encargar",
    "encargasen": "encargar",
    "encargases": "encargar",
    "encargaste": "encargar",
    "encargasteis": "encargar",
    "encargo": "encargar",
    "encargos": "encargo",
    "encargue": "encargar",
    "encarguemos": "encargar",
    "encarguen": "encargar",
    "encargues": "encargar",
    "encargué": "encargar",
    "encarguéis": "encargar",
    "encargábamos": "encargar",
    "encargáis": "encargar",
    "encargáramos": "encargar",
    "encargáremos": "encargar",
    "encargásemos": "encargar",
    "encargó": "encargar",
    "encariña": "encariñar",
    "encariñaba": "encariñar",
    "encariñabais": "encariñar",
    "encariñaban": "encariñar",
    "encariñabas": "encariñar",
    "encariñad": "encariñar",
    "encariñada": "encariñar",
    "encariñadas": "encariñar",
    "encariñado": "encariñar",
    "encariñados": "encariñar",
    "encariñamientos": "encariñamiento",
    "encariñamos": "encariñar",
    "encariñan": "encariñar",
    "encariñando": "encariñar",
    "encariñara": "encariñar",
    "encariñarais": "encariñar",
    "encariñaran": "encariñar",
    "encariñaras": "encariñar",
    "encariñare": "encariñar",
    "encariñareis": "encariñar",
    "encariñaremos": "encariñar",
    "encariñaren": "encariñar",
    "encariñares": "encariñar",
    "encariñaron": "encariñar",
    "encariñará": "encariñar",
    "encariñarán": "encariñar",
    "encariñarás": "encariñar",
    "encariñaré": "encariñar",
    "encariñaréis": "encariñar",
    "encariñaría": "encariñar",
    "encariñaríais": "encariñar",
    "encariñaríamos": "encariñar",
    "encariñarían": "encariñar",
    "encariñarías": "encariñar",
    "encariñas": "encariñar",
    "encariñase": "encariñar",
    "encariñaseis": "encariñar",
    "encariñasen": "encariñar",
    "encariñases": "encariñar",
    "encariñaste": "encariñar",
    "encariñasteis": "encariñar",
    "encariñe": "encariñar",
    "encariñemos": "encariñar",
    "encariñen": "encariñar",
    "encariñes": "encariñar",
    "encariño": "encariñar",
    "encariñábamos": "encariñar",
    "encariñáis": "encariñar",
    "encariñáramos": "encariñar",
    "encariñáremos": "encariñar",
    "encariñásemos": "encariñar",
    "encariñé": "encariñar",
    "encariñéis": "encariñar",
    "encariñó": "encariñar",
    "encarna": "encarnar",
    "encarnaba": "encarnar",
    "encarnabais": "encarnar",
    "encarnaban": "encarnar",
    "encarnabas": "encarnar",
    "encarnaciones": "encarnación",
    "encarnad": "encarnar",
    "encarnada": "encarnar",
    "encarnadas": "encarnar",
    "encarnadina": "encarnadino",
    "encarnadinas": "encarnadino",
    "encarnadinos": "encarnadino",
    "encarnado": "encarnar",
    "encarnados": "encarnar",
    "encarnaduras": "encarnadura",
    "encarnamientos": "encarnamiento",
    "encarnamos": "encarnar",
    "encarnan": "encarnar",
    "encarnando": "encarnar",
    "encarnara": "encarnar",
    "encarnarais": "encarnar",
    "encarnaran": "encarnar",
    "encarnaras": "encarnar",
    "encarnare": "encarnar",
    "encarnareis": "encarnar",
    "encarnaremos": "encarnar",
    "encarnaren": "encarnar",
    "encarnares": "encarnar",
    "encarnaron": "encarnar",
    "encarnará": "encarnar",
    "encarnarán": "encarnar",
    "encarnarás": "encarnar",
    "encarnaré": "encarnar",
    "encarnaréis": "encarnar",
    "encarnaría": "encarnar",
    "encarnaríais": "encarnar",
    "encarnaríamos": "encarnar",
    "encarnarían": "encarnar",
    "encarnarías": "encarnar",
    "encarnas": "encarnar",
    "encarnase": "encarnar",
    "encarnaseis": "encarnar",
    "encarnasen": "encarnar",
    "encarnases": "encarnar",
    "encarnaste": "encarnar",
    "encarnasteis": "encarnar",
    "encarne": "encarnar",
    "encarnece": "encarnecer",
    "encarneced": "encarnecer",
    "encarnecemos": "encarnecer",
    "encarnecen": "encarnecer",
    "encarneceremos": "encarnecer",
    "encarnecerá": "encarnecer",
    "encarnecerán": "encarnecer",
    "encarnecerás": "encarnecer",
    "encarneceré": "encarnecer",
    "encarneceréis": "encarnecer",
    "encarnecería": "encarnecer",
    "encarneceríais": "encarnecer",
    "encarneceríamos": "encarnecer",
    "encarnecerían": "encarnecer",
    "encarnecerías": "encarnecer",
    "encarneces": "encarnecer",
    "encarnecida": "encarnecer",
    "encarnecidas": "encarnecer",
    "encarnecido": "encarnecer",
    "encarnecidos": "encarnecer",
    "encarneciendo": "encarnecer",
    "encarneciera": "encarnecer",
    "encarnecierais": "encarnecer",
    "encarnecieran": "encarnecer",
    "encarnecieras": "encarnecer",
    "encarneciere": "encarnecer",
    "encarneciereis": "encarnecer",
    "encarnecieren": "encarnecer",
    "encarnecieres": "encarnecer",
    "encarnecieron": "encarnecer",
    "encarneciese": "encarnecer",
    "encarnecieseis": "encarnecer",
    "encarneciesen": "encarnecer",
    "encarnecieses": "encarnecer",
    "encarnecimos": "encarnecer",
    "encarneciste": "encarnecer",
    "encarnecisteis": "encarnecer",
    "encarneciéramos": "encarnecer",
    "encarneciéremos": "encarnecer",
    "encarneciésemos": "encarnecer",
    "encarneció": "encarnecer",
    "encarnecéis": "encarnecer",
    "encarnecí": "encarnecer",
    "encarnecía": "encarnecer",
    "encarnecíais": "encarnecer",
    "encarnecíamos": "encarnecer",
    "encarnecían": "encarnecer",
    "encarnecías": "encarnecer",
    "encarnemos": "encarnar",
    "encarnen": "encarnar",
    "encarnes": "encarne",
    "encarnezca": "encarnecer",
    "encarnezcamos": "encarnecer",
    "encarnezcan": "encarnecer",
    "encarnezcas": "encarnecer",
    "encarnezco": "encarnecer",
    "encarnezcáis": "encarnecer",
    "encarnice": "encarnizar",
    "encarnicemos": "encarnizar",
    "encarnicen": "encarnizar",
    "encarnices": "encarnizar",
    "encarnicé": "encarnizar",
    "encarnicéis": "encarnizar",
    "encarniza": "encarnizar",
    "encarnizaba": "encarnizar",
    "encarnizabais": "encarnizar",
    "encarnizaban": "encarnizar",
    "encarnizabas": "encarnizar",
    "encarnizad": "encarnizar",
    "encarnizada": "encarnizar",
    "encarnizadas": "encarnizar",
    "encarnizado": "encarnizar",
    "encarnizados": "encarnizar",
    "encarnizamientos": "encarnizamiento",
    "encarnizamos": "encarnizar",
    "encarnizan": "encarnizar",
    "encarnizando": "encarnizar",
    "encarnizara": "encarnizar",
    "encarnizarais": "encarnizar",
    "encarnizaran": "encarnizar",
    "encarnizaras": "encarnizar",
    "encarnizare": "encarnizar",
    "encarnizareis": "encarnizar",
    "encarnizaremos": "encarnizar",
    "encarnizaren": "encarnizar",
    "encarnizares": "encarnizar",
    "encarnizaron": "encarnizar",
    "encarnizará": "encarnizar",
    "encarnizarán": "encarnizar",
    "encarnizarás": "encarnizar",
    "encarnizaré": "encarnizar",
    "encarnizaréis": "encarnizar",
    "encarnizaría": "encarnizar",
    "encarnizaríais": "encarnizar",
    "encarnizaríamos": "encarnizar",
    "encarnizarían": "encarnizar",
    "encarnizarías": "encarnizar",
    "encarnizas": "encarnizar",
    "encarnizase": "encarnizar",
    "encarnizaseis": "encarnizar",
    "encarnizasen": "encarnizar",
    "encarnizases": "encarnizar",
    "encarnizaste": "encarnizar",
    "encarnizasteis": "encarnizar",
    "encarnizo": "encarnizar",
    "encarnizábamos": "encarnizar",
    "encarnizáis": "encarnizar",
    "encarnizáramos": "encarnizar",
    "encarnizáremos": "encarnizar",
    "encarnizásemos": "encarnizar",
    "encarnizó": "encarnizar",
    "encarno": "encarnar",
    "encarnábamos": "encarnar",
    "encarnáis": "encarnar",
    "encarnáramos": "encarnar",
    "encarnáremos": "encarnar",
    "encarnásemos": "encarnar",
    "encarné": "encarnar",
    "encarnéis": "encarnar",
    "encarnó": "encarnar",
    "encaro": "encarar",
    "encaros": "encaro",
    "encarpeta": "encarpetar",
    "encarpetaba": "encarpetar",
    "encarpetabais": "encarpetar",
    "encarpetaban": "encarpetar",
    "encarpetabas": "encarpetar",
    "encarpetad": "encarpetar",
    "encarpetada": "encarpetar",
    "encarpetadas": "encarpetar",
    "encarpetado": "encarpetar",
    "encarpetados": "encarpetar",
    "encarpetamos": "encarpetar",
    "encarpetan": "encarpetar",
    "encarpetando": "encarpetar",
    "encarpetara": "encarpetar",
    "encarpetarais": "encarpetar",
    "encarpetaran": "encarpetar",
    "encarpetaras": "encarpetar",
    "encarpetare": "encarpetar",
    "encarpetareis": "encarpetar",
    "encarpetaremos": "encarpetar",
    "encarpetaren": "encarpetar",
    "encarpetares": "encarpetar",
    "encarpetaron": "encarpetar",
    "encarpetará": "encarpetar",
    "encarpetarán": "encarpetar",
    "encarpetarás": "encarpetar",
    "encarpetaré": "encarpetar",
    "encarpetaréis": "encarpetar",
    "encarpetaría": "encarpetar",
    "encarpetaríais": "encarpetar",
    "encarpetaríamos": "encarpetar",
    "encarpetarían": "encarpetar",
    "encarpetarías": "encarpetar",
    "encarpetas": "encarpetar",
    "encarpetase": "encarpetar",
    "encarpetaseis": "encarpetar",
    "encarpetasen": "encarpetar",
    "encarpetases": "encarpetar",
    "encarpetaste": "encarpetar",
    "encarpetasteis": "encarpetar",
    "encarpete": "encarpetar",
    "encarpetemos": "encarpetar",
    "encarpeten": "encarpetar",
    "encarpetes": "encarpetar",
    "encarpeto": "encarpetar",
    "encarpetábamos": "encarpetar",
    "encarpetáis": "encarpetar",
    "encarpetáramos": "encarpetar",
    "encarpetáremos": "encarpetar",
    "encarpetásemos": "encarpetar",
    "encarpeté": "encarpetar",
    "encarpetéis": "encarpetar",
    "encarpetó": "encarpetar",
    "encarrila": "encarrilar",
    "encarrilaba": "encarrilar",
    "encarrilabais": "encarrilar",
    "encarrilaban": "encarrilar",
    "encarrilabas": "encarrilar",
    "encarrilad": "encarrilar",
    "encarrilada": "encarrilar",
    "encarriladas": "encarrilar",
    "encarriladeras": "encarriladera",
    "encarrilado": "encarrilar",
    "encarriladora": "encarrilador",
    "encarriladoras": "encarrilador",
    "encarriladores": "encarrilador",
    "encarrilados": "encarrilar",
    "encarrilamientos": "encarrilamiento",
    "encarrilamos": "encarrilar",
    "encarrilan": "encarrilar",
    "encarrilando": "encarrilar",
    "encarrilara": "encarrilar",
    "encarrilarais": "encarrilar",
    "encarrilaran": "encarrilar",
    "encarrilaras": "encarrilar",
    "encarrilare": "encarrilar",
    "encarrilareis": "encarrilar",
    "encarrilaremos": "encarrilar",
    "encarrilaren": "encarrilar",
    "encarrilares": "encarrilar",
    "encarrilaron": "encarrilar",
    "encarrilará": "encarrilar",
    "encarrilarán": "encarrilar",
    "encarrilarás": "encarrilar",
    "encarrilaré": "encarrilar",
    "encarrilaréis": "encarrilar",
    "encarrilaría": "encarrilar",
    "encarrilaríais": "encarrilar",
    "encarrilaríamos": "encarrilar",
    "encarrilarían": "encarrilar",
    "encarrilarías": "encarrilar",
    "encarrilas": "encarrilar",
    "encarrilase": "encarrilar",
    "encarrilaseis": "encarrilar",
    "encarrilasen": "encarrilar",
    "encarrilases": "encarrilar",
    "encarrilaste": "encarrilar",
    "encarrilasteis": "encarrilar",
    "encarrile": "encarrilar",
    "encarrilemos": "encarrilar",
    "encarrilen": "encarrilar",
    "encarriles": "encarrilar",
    "encarrilla": "encarrillar",
    "encarrillaba": "encarrillar",
    "encarrillabais": "encarrillar",
    "encarrillaban": "encarrillar",
    "encarrillabas": "encarrillar",
    "encarrillad": "encarrillar",
    "encarrillada": "encarrillar",
    "encarrilladas": "encarrillar",
    "encarrillado": "encarrillar",
    "encarrillados": "encarrillar",
    "encarrillamos": "encarrillar",
    "encarrillan": "encarrillar",
    "encarrillando": "encarrillar",
    "encarrillara": "encarrillar",
    "encarrillarais": "encarrillar",
    "encarrillaran": "encarrillar",
    "encarrillaras": "encarrillar",
    "encarrillare": "encarrillar",
    "encarrillareis": "encarrillar",
    "encarrillaremos": "encarrillar",
    "encarrillaren": "encarrillar",
    "encarrillares": "encarrillar",
    "encarrillaron": "encarrillar",
    "encarrillará": "encarrillar",
    "encarrillarán": "encarrillar",
    "encarrillarás": "encarrillar",
    "encarrillaré": "encarrillar",
    "encarrillaréis": "encarrillar",
    "encarrillaría": "encarrillar",
    "encarrillaríais": "encarrillar",
    "encarrillaríamos": "encarrillar",
    "encarrillarían": "encarrillar",
    "encarrillarías": "encarrillar",
    "encarrillas": "encarrillar",
    "encarrillase": "encarrillar",
    "encarrillaseis": "encarrillar",
    "encarrillasen": "encarrillar",
    "encarrillases": "encarrillar",
    "encarrillaste": "encarrillar",
    "encarrillasteis": "encarrillar",
    "encarrille": "encarrillar",
    "encarrillemos": "encarrillar",
    "encarrillen": "encarrillar",
    "encarrilles": "encarrillar",
    "encarrillo": "encarrillar",
    "encarrillábamos": "encarrillar",
    "encarrilláis": "encarrillar",
    "encarrilláramos": "encarrillar",
    "encarrilláremos": "encarrillar",
    "encarrillásemos": "encarrillar",
    "encarrillé": "encarrillar",
    "encarrilléis": "encarrillar",
    "encarrilló": "encarrillar",
    "encarrilo": "encarrilar",
    "encarrilábamos": "encarrilar",
    "encarriláis": "encarrilar",
    "encarriláramos": "encarrilar",
    "encarriláremos": "encarrilar",
    "encarrilásemos": "encarrilar",
    "encarrilé": "encarrilar",
    "encarriléis": "encarrilar",
    "encarriló": "encarrilar",
    "encarroña": "encarroñar",
    "encarroñaba": "encarroñar",
    "encarroñabais": "encarroñar",
    "encarroñaban": "encarroñar",
    "encarroñabas": "encarroñar",
    "encarroñad": "encarroñar",
    "encarroñada": "encarroñar",
    "encarroñadas": "encarroñar",
    "encarroñado": "encarroñar",
    "encarroñados": "encarroñar",
    "encarroñamos": "encarroñar",
    "encarroñan": "encarroñar",
    "encarroñando": "encarroñar",
    "encarroñara": "encarroñar",
    "encarroñarais": "encarroñar",
    "encarroñaran": "encarroñar",
    "encarroñaras": "encarroñar",
    "encarroñare": "encarroñar",
    "encarroñareis": "encarroñar",
    "encarroñaremos": "encarroñar",
    "encarroñaren": "encarroñar",
    "encarroñares": "encarroñar",
    "encarroñaron": "encarroñar",
    "encarroñará": "encarroñar",
    "encarroñarán": "encarroñar",
    "encarroñarás": "encarroñar",
    "encarroñaré": "encarroñar",
    "encarroñaréis": "encarroñar",
    "encarroñaría": "encarroñar",
    "encarroñaríais": "encarroñar",
    "encarroñaríamos": "encarroñar",
    "encarroñarían": "encarroñar",
    "encarroñarías": "encarroñar",
    "encarroñas": "encarroñar",
    "encarroñase": "encarroñar",
    "encarroñaseis": "encarroñar",
    "encarroñasen": "encarroñar",
    "encarroñases": "encarroñar",
    "encarroñaste": "encarroñar",
    "encarroñasteis": "encarroñar",
    "encarroñe": "encarroñar",
    "encarroñemos": "encarroñar",
    "encarroñen": "encarroñar",
    "encarroñes": "encarroñar",
    "encarroño": "encarroñar",
    "encarroñábamos": "encarroñar",
    "encarroñáis": "encarroñar",
    "encarroñáramos": "encarroñar",
    "encarroñáremos": "encarroñar",
    "encarroñásemos": "encarroñar",
    "encarroñé": "encarroñar",
    "encarroñéis": "encarroñar",
    "encarroñó": "encarroñar",
    "encarruja": "encarrujar",
    "encarrujaba": "encarrujar",
    "encarrujabais": "encarrujar",
    "encarrujaban": "encarrujar",
    "encarrujabas": "encarrujar",
    "encarrujad": "encarrujar",
    "encarrujada": "encarrujar",
    "encarrujadas": "encarrujar",
    "encarrujado": "encarrujar",
    "encarrujados": "encarrujar",
    "encarrujamos": "encarrujar",
    "encarrujan": "encarrujar",
    "encarrujando": "encarrujar",
    "encarrujara": "encarrujar",
    "encarrujarais": "encarrujar",
    "encarrujaran": "encarrujar",
    "encarrujaras": "encarrujar",
    "encarrujare": "encarrujar",
    "encarrujareis": "encarrujar",
    "encarrujaremos": "encarrujar",
    "encarrujaren": "encarrujar",
    "encarrujares": "encarrujar",
    "encarrujaron": "encarrujar",
    "encarrujará": "encarrujar",
    "encarrujarán": "encarrujar",
    "encarrujarás": "encarrujar",
    "encarrujaré": "encarrujar",
    "encarrujaréis": "encarrujar",
    "encarrujaría": "encarrujar",
    "encarrujaríais": "encarrujar",
    "encarrujaríamos": "encarrujar",
    "encarrujarían": "encarrujar",
    "encarrujarías": "encarrujar",
    "encarrujas": "encarrujar",
    "encarrujase": "encarrujar",
    "encarrujaseis": "encarrujar",
    "encarrujasen": "encarrujar",
    "encarrujases": "encarrujar",
    "encarrujaste": "encarrujar",
    "encarrujasteis": "encarrujar",
    "encarruje": "encarrujar",
    "encarrujemos": "encarrujar",
    "encarrujen": "encarrujar",
    "encarrujes": "encarrujar",
    "encarrujo": "encarrujar",
    "encarrujábamos": "encarrujar",
    "encarrujáis": "encarrujar",
    "encarrujáramos": "encarrujar",
    "encarrujáremos": "encarrujar",
    "encarrujásemos": "encarrujar",
    "encarrujé": "encarrujar",
    "encarrujéis": "encarrujar",
    "encarrujó": "encarrujar",
    "encarta": "encartar",
    "encartaba": "encartar",
    "encartabais": "encartar",
    "encartaban": "encartar",
    "encartabas": "encartar",
    "encartaciones": "encartación",
    "encartad": "encartar",
    "encartada": "encartar",
    "encartadas": "encartar",
    "encartado": "encartar",
    "encartados": "encartar",
    "encartamientos": "encartamiento",
    "encartamos": "encartar",
    "encartan": "encartar",
    "encartando": "encartar",
    "encartara": "encartar",
    "encartarais": "encartar",
    "encartaran": "encartar",
    "encartaras": "encartar",
    "encartare": "encartar",
    "encartareis": "encartar",
    "encartaremos": "encartar",
    "encartaren": "encartar",
    "encartares": "encartar",
    "encartaron": "encartar",
    "encartará": "encartar",
    "encartarán": "encartar",
    "encartarás": "encartar",
    "encartaré": "encartar",
    "encartaréis": "encartar",
    "encartaría": "encartar",
    "encartaríais": "encartar",
    "encartaríamos": "encartar",
    "encartarían": "encartar",
    "encartarías": "encartar",
    "encartas": "encartar",
    "encartase": "encartar",
    "encartaseis": "encartar",
    "encartasen": "encartar",
    "encartases": "encartar",
    "encartaste": "encartar",
    "encartasteis": "encartar",
    "encarte": "encartar",
    "encartemos": "encartar",
    "encarten": "encartar",
    "encartes": "encarte",
    "encarto": "encartar",
    "encartona": "encartonar",
    "encartonaba": "encartonar",
    "encartonabais": "encartonar",
    "encartonaban": "encartonar",
    "encartonabas": "encartonar",
    "encartonad": "encartonar",
    "encartonada": "encartonar",
    "encartonadas": "encartonar",
    "encartonado": "encartonar",
    "encartonadora": "encartonador",
    "encartonadoras": "encartonador",
    "encartonadores": "encartonador",
    "encartonados": "encartonar",
    "encartonamos": "encartonar",
    "encartonan": "encartonar",
    "encartonando": "encartonar",
    "encartonara": "encartonar",
    "encartonarais": "encartonar",
    "encartonaran": "encartonar",
    "encartonaras": "encartonar",
    "encartonare": "encartonar",
    "encartonareis": "encartonar",
    "encartonaremos": "encartonar",
    "encartonaren": "encartonar",
    "encartonares": "encartonar",
    "encartonaron": "encartonar",
    "encartonará": "encartonar",
    "encartonarán": "encartonar",
    "encartonarás": "encartonar",
    "encartonaré": "encartonar",
    "encartonaréis": "encartonar",
    "encartonaría": "encartonar",
    "encartonaríais": "encartonar",
    "encartonaríamos": "encartonar",
    "encartonarían": "encartonar",
    "encartonarías": "encartonar",
    "encartonas": "encartonar",
    "encartonase": "encartonar",
    "encartonaseis": "encartonar",
    "encartonasen": "encartonar",
    "encartonases": "encartonar",
    "encartonaste": "encartonar",
    "encartonasteis": "encartonar",
    "encartone": "encartonar",
    "encartonemos": "encartonar",
    "encartonen": "encartonar",
    "encartones": "encartonar",
    "encartono": "encartonar",
    "encartonábamos": "encartonar",
    "encartonáis": "encartonar",
    "encartonáramos": "encartonar",
    "encartonáremos": "encartonar",
    "encartonásemos": "encartonar",
    "encartoné": "encartonar",
    "encartonéis": "encartonar",
    "encartonó": "encartonar",
    "encartábamos": "encartar",
    "encartáis": "encartar",
    "encartáramos": "encartar",
    "encartáremos": "encartar",
    "encartásemos": "encartar",
    "encarté": "encartar",
    "encartéis": "encartar",
    "encartó": "encartar",
    "encarábamos": "encarar",
    "encaráis": "encarar",
    "encaráramos": "encarar",
    "encaráremos": "encarar",
    "encarásemos": "encarar",
    "encaré": "encarar",
    "encaréis": "encarar",
    "encaró": "encarar",
    "encascabela": "encascabelar",
    "encascabelaba": "encascabelar",
    "encascabelabais": "encascabelar",
    "encascabelaban": "encascabelar",
    "encascabelabas": "encascabelar",
    "encascabelad": "encascabelar",
    "encascabelada": "encascabelar",
    "encascabeladas": "encascabelar",
    "encascabelado": "encascabelar",
    "encascabelados": "encascabelar",
    "encascabelamos": "encascabelar",
    "encascabelan": "encascabelar",
    "encascabelando": "encascabelar",
    "encascabelara": "encascabelar",
    "encascabelarais": "encascabelar",
    "encascabelaran": "encascabelar",
    "encascabelaras": "encascabelar",
    "encascabelare": "encascabelar",
    "encascabelareis": "encascabelar",
    "encascabelaremos": "encascabelar",
    "encascabelaren": "encascabelar",
    "encascabelares": "encascabelar",
    "encascabelaron": "encascabelar",
    "encascabelará": "encascabelar",
    "encascabelarán": "encascabelar",
    "encascabelarás": "encascabelar",
    "encascabelaré": "encascabelar",
    "encascabelaréis": "encascabelar",
    "encascabelaría": "encascabelar",
    "encascabelaríais": "encascabelar",
    "encascabelaríamos": "encascabelar",
    "encascabelarían": "encascabelar",
    "encascabelarías": "encascabelar",
    "encascabelas": "encascabelar",
    "encascabelase": "encascabelar",
    "encascabelaseis": "encascabelar",
    "encascabelasen": "encascabelar",
    "encascabelases": "encascabelar",
    "encascabelaste": "encascabelar",
    "encascabelasteis": "encascabelar",
    "encascabele": "encascabelar",
    "encascabelemos": "encascabelar",
    "encascabelen": "encascabelar",
    "encascabeles": "encascabelar",
    "encascabelo": "encascabelar",
    "encascabelábamos": "encascabelar",
    "encascabeláis": "encascabelar",
    "encascabeláramos": "encascabelar",
    "encascabeláremos": "encascabelar",
    "encascabelásemos": "encascabelar",
    "encascabelé": "encascabelar",
    "encascabeléis": "encascabelar",
    "encascabeló": "encascabelar",
    "encascota": "encascotar",
    "encascotaba": "encascotar",
    "encascotabais": "encascotar",
    "encascotaban": "encascotar",
    "encascotabas": "encascotar",
    "encascotad": "encascotar",
    "encascotada": "encascotar",
    "encascotadas": "encascotar",
    "encascotado": "encascotar",
    "encascotados": "encascotar",
    "encascotamos": "encascotar",
    "encascotan": "encascotar",
    "encascotando": "encascotar",
    "encascotara": "encascotar",
    "encascotarais": "encascotar",
    "encascotaran": "encascotar",
    "encascotaras": "encascotar",
    "encascotare": "encascotar",
    "encascotareis": "encascotar",
    "encascotaremos": "encascotar",
    "encascotaren": "encascotar",
    "encascotares": "encascotar",
    "encascotaron": "encascotar",
    "encascotará": "encascotar",
    "encascotarán": "encascotar",
    "encascotarás": "encascotar",
    "encascotaré": "encascotar",
    "encascotaréis": "encascotar",
    "encascotaría": "encascotar",
    "encascotaríais": "encascotar",
    "encascotaríamos": "encascotar",
    "encascotarían": "encascotar",
    "encascotarías": "encascotar",
    "encascotas": "encascotar",
    "encascotase": "encascotar",
    "encascotaseis": "encascotar",
    "encascotasen": "encascotar",
    "encascotases": "encascotar",
    "encascotaste": "encascotar",
    "encascotasteis": "encascotar",
    "encascote": "encascotar",
    "encascotemos": "encascotar",
    "encascoten": "encascotar",
    "encascotes": "encascotar",
    "encascoto": "encascotar",
    "encascotábamos": "encascotar",
    "encascotáis": "encascotar",
    "encascotáramos": "encascotar",
    "encascotáremos": "encascotar",
    "encascotásemos": "encascotar",
    "encascoté": "encascotar",
    "encascotéis": "encascotar",
    "encascotó": "encascotar",
    "encasilla": "encasillar",
    "encasillaba": "encasillar",
    "encasillabais": "encasillar",
    "encasillaban": "encasillar",
    "encasillabas": "encasillar",
    "encasillables": "encasillable",
    "encasillad": "encasillar",
    "encasillada": "encasillar",
    "encasilladas": "encasillar",
    "encasillado": "encasillar",
    "encasillados": "encasillar",
    "encasillamientos": "encasillamiento",
    "encasillamos": "encasillar",
    "encasillan": "encasillar",
    "encasillando": "encasillar",
    "encasillara": "encasillar",
    "encasillarais": "encasillar",
    "encasillaran": "encasillar",
    "encasillaras": "encasillar",
    "encasillare": "encasillar",
    "encasillareis": "encasillar",
    "encasillaremos": "encasillar",
    "encasillaren": "encasillar",
    "encasillares": "encasillar",
    "encasillaron": "encasillar",
    "encasillará": "encasillar",
    "encasillarán": "encasillar",
    "encasillarás": "encasillar",
    "encasillaré": "encasillar",
    "encasillaréis": "encasillar",
    "encasillaría": "encasillar",
    "encasillaríais": "encasillar",
    "encasillaríamos": "encasillar",
    "encasillarían": "encasillar",
    "encasillarías": "encasillar",
    "encasillas": "encasillar",
    "encasillase": "encasillar",
    "encasillaseis": "encasillar",
    "encasillasen": "encasillar",
    "encasillases": "encasillar",
    "encasillaste": "encasillar",
    "encasillasteis": "encasillar",
    "encasille": "encasillar",
    "encasillemos": "encasillar",
    "encasillen": "encasillar",
    "encasilles": "encasillar",
    "encasillo": "encasillar",
    "encasillábamos": "encasillar",
    "encasilláis": "encasillar",
    "encasilláramos": "encasillar",
    "encasilláremos": "encasillar",
    "encasillásemos": "encasillar",
    "encasillé": "encasillar",
    "encasilléis": "encasillar",
    "encasilló": "encasillar",
    "encasqueta": "encasquetar",
    "encasquetaba": "encasquetar",
    "encasquetabais": "encasquetar",
    "encasquetaban": "encasquetar",
    "encasquetabas": "encasquetar",
    "encasquetad": "encasquetar",
    "encasquetada": "encasquetar",
    "encasquetadas": "encasquetar",
    "encasquetado": "encasquetar",
    "encasquetados": "encasquetar",
    "encasquetamientos": "encasquetamiento",
    "encasquetamos": "encasquetar",
    "encasquetan": "encasquetar",
    "encasquetando": "encasquetar",
    "encasquetara": "encasquetar",
    "encasquetarais": "encasquetar",
    "encasquetaran": "encasquetar",
    "encasquetaras": "encasquetar",
    "encasquetare": "encasquetar",
    "encasquetareis": "encasquetar",
    "encasquetaremos": "encasquetar",
    "encasquetaren": "encasquetar",
    "encasquetares": "encasquetar",
    "encasquetaron": "encasquetar",
    "encasquetará": "encasquetar",
    "encasquetarán": "encasquetar",
    "encasquetarás": "encasquetar",
    "encasquetaré": "encasquetar",
    "encasquetaréis": "encasquetar",
    "encasquetaría": "encasquetar",
    "encasquetaríais": "encasquetar",
    "encasquetaríamos": "encasquetar",
    "encasquetarían": "encasquetar",
    "encasquetarías": "encasquetar",
    "encasquetas": "encasquetar",
    "encasquetase": "encasquetar",
    "encasquetaseis": "encasquetar",
    "encasquetasen": "encasquetar",
    "encasquetases": "encasquetar",
    "encasquetaste": "encasquetar",
    "encasquetasteis": "encasquetar",
    "encasquete": "encasquetar",
    "encasquetemos": "encasquetar",
    "encasqueten": "encasquetar",
    "encasquetes": "encasquetar",
    "encasqueto": "encasquetar",
    "encasquetábamos": "encasquetar",
    "encasquetáis": "encasquetar",
    "encasquetáramos": "encasquetar",
    "encasquetáremos": "encasquetar",
    "encasquetásemos": "encasquetar",
    "encasqueté": "encasquetar",
    "encasquetéis": "encasquetar",
    "encasquetó": "encasquetar",
    "encasquilla": "encasquillar",
    "encasquillaba": "encasquillar",
    "encasquillabais": "encasquillar",
    "encasquillaban": "encasquillar",
    "encasquillabas": "encasquillar",
    "encasquillad": "encasquillar",
    "encasquillada": "encasquillar",
    "encasquilladas": "encasquillar",
    "encasquillado": "encasquillar",
    "encasquillados": "encasquillar",
    "encasquillamientos": "encasquillamiento",
    "encasquillamos": "encasquillar",
    "encasquillan": "encasquillar",
    "encasquillando": "encasquillar",
    "encasquillara": "encasquillar",
    "encasquillarais": "encasquillar",
    "encasquillaran": "encasquillar",
    "encasquillaras": "encasquillar",
    "encasquillare": "encasquillar",
    "encasquillareis": "encasquillar",
    "encasquillaremos": "encasquillar",
    "encasquillaren": "encasquillar",
    "encasquillares": "encasquillar",
    "encasquillaron": "encasquillar",
    "encasquillará": "encasquillar",
    "encasquillarán": "encasquillar",
    "encasquillarás": "encasquillar",
    "encasquillaré": "encasquillar",
    "encasquillaréis": "encasquillar",
    "encasquillaría": "encasquillar",
    "encasquillaríais": "encasquillar",
    "encasquillaríamos": "encasquillar",
    "encasquillarían": "encasquillar",
    "encasquillarías": "encasquillar",
    "encasquillas": "encasquillar",
    "encasquillase": "encasquillar",
    "encasquillaseis": "encasquillar",
    "encasquillasen": "encasquillar",
    "encasquillases": "encasquillar",
    "encasquillaste": "encasquillar",
    "encasquillasteis": "encasquillar",
    "encasquille": "encasquillar",
    "encasquillemos": "encasquillar",
    "encasquillen": "encasquillar",
    "encasquilles": "encasquillar",
    "encasquillo": "encasquillar",
    "encasquillábamos": "encasquillar",
    "encasquilláis": "encasquillar",
    "encasquilláramos": "encasquillar",
    "encasquilláremos": "encasquillar",
    "encasquillásemos": "encasquillar",
    "encasquillé": "encasquillar",
    "encasquilléis": "encasquillar",
    "encasquilló": "encasquillar",
    "encastes": "encaste",
    "encastilla": "encastillar",
    "encastillaba": "encastillar",
    "encastillabais": "encastillar",
    "encastillaban": "encastillar",
    "encastillabas": "encastillar",
    "encastillad": "encastillar",
    "encastillada": "encastillar",
    "encastilladas": "encastillar",
    "encastillado": "encastillar",
    "encastillados": "encastillar",
    "encastillamos": "encastillar",
    "encastillan": "encastillar",
    "encastillando": "encastillar",
    "encastillara": "encastillar",
    "encastillarais": "encastillar",
    "encastillaran": "encastillar",
    "encastillaras": "encastillar",
    "encastillare": "encastillar",
    "encastillareis": "encastillar",
    "encastillaremos": "encastillar",
    "encastillaren": "encastillar",
    "encastillares": "encastillar",
    "encastillaron": "encastillar",
    "encastillará": "encastillar",
    "encastillarán": "encastillar",
    "encastillarás": "encastillar",
    "encastillaré": "encastillar",
    "encastillaréis": "encastillar",
    "encastillaría": "encastillar",
    "encastillaríais": "encastillar",
    "encastillaríamos": "encastillar",
    "encastillarían": "encastillar",
    "encastillarías": "encastillar",
    "encastillas": "encastillar",
    "encastillase": "encastillar",
    "encastillaseis": "encastillar",
    "encastillasen": "encastillar",
    "encastillases": "encastillar",
    "encastillaste": "encastillar",
    "encastillasteis": "encastillar",
    "encastille": "encastillar",
    "encastillemos": "encastillar",
    "encastillen": "encastillar",
    "encastilles": "encastillar",
    "encastillo": "encastillar",
    "encastillábamos": "encastillar",
    "encastilláis": "encastillar",
    "encastilláramos": "encastillar",
    "encastilláremos": "encastillar",
    "encastillásemos": "encastillar",
    "encastillé": "encastillar",
    "encastilléis": "encastillar",
    "encastilló": "encastillar",
    "encastra": "encastrar",
    "encastraba": "encastrar",
    "encastrabais": "encastrar",
    "encastraban": "encastrar",
    "encastrabas": "encastrar",
    "encastrad": "encastrar",
    "encastrada": "encastrar",
    "encastradas": "encastrar",
    "encastrado": "encastrar",
    "encastrados": "encastrar",
    "encastramos": "encastrar",
    "encastran": "encastrar",
    "encastrando": "encastrar",
    "encastrara": "encastrar",
    "encastrarais": "encastrar",
    "encastraran": "encastrar",
    "encastraras": "encastrar",
    "encastrare": "encastrar",
    "encastrareis": "encastrar",
    "encastraremos": "encastrar",
    "encastraren": "encastrar",
    "encastrares": "encastrar",
    "encastraron": "encastrar",
    "encastrará": "encastrar",
    "encastrarán": "encastrar",
    "encastrarás": "encastrar",
    "encastraré": "encastrar",
    "encastraréis": "encastrar",
    "encastraría": "encastrar",
    "encastraríais": "encastrar",
    "encastraríamos": "encastrar",
    "encastrarían": "encastrar",
    "encastrarías": "encastrar",
    "encastras": "encastrar",
    "encastrase": "encastrar",
    "encastraseis": "encastrar",
    "encastrasen": "encastrar",
    "encastrases": "encastrar",
    "encastraste": "encastrar",
    "encastrasteis": "encastrar",
    "encastre": "encastrar",
    "encastremos": "encastrar",
    "encastren": "encastrar",
    "encastres": "encastrar",
    "encastro": "encastrar",
    "encastrábamos": "encastrar",
    "encastráis": "encastrar",
    "encastráramos": "encastrar",
    "encastráremos": "encastrar",
    "encastrásemos": "encastrar",
    "encastré": "encastrar",
    "encastréis": "encastrar",
    "encastró": "encastrar",
    "encatusa": "encatusar",
    "encatusaba": "encatusar",
    "encatusabais": "encatusar",
    "encatusaban": "encatusar",
    "encatusabas": "encatusar",
    "encatusad": "encatusar",
    "encatusada": "encatusar",
    "encatusadas": "encatusar",
    "encatusado": "encatusar",
    "encatusados": "encatusar",
    "encatusamos": "encatusar",
    "encatusan": "encatusar",
    "encatusando": "encatusar",
    "encatusara": "encatusar",
    "encatusarais": "encatusar",
    "encatusaran": "encatusar",
    "encatusaras": "encatusar",
    "encatusare": "encatusar",
    "encatusareis": "encatusar",
    "encatusaremos": "encatusar",
    "encatusaren": "encatusar",
    "encatusares": "encatusar",
    "encatusaron": "encatusar",
    "encatusará": "encatusar",
    "encatusarán": "encatusar",
    "encatusarás": "encatusar",
    "encatusaré": "encatusar",
    "encatusaréis": "encatusar",
    "encatusaría": "encatusar",
    "encatusaríais": "encatusar",
    "encatusaríamos": "encatusar",
    "encatusarían": "encatusar",
    "encatusarías": "encatusar",
    "encatusas": "encatusar",
    "encatusase": "encatusar",
    "encatusaseis": "encatusar",
    "encatusasen": "encatusar",
    "encatusases": "encatusar",
    "encatusaste": "encatusar",
    "encatusasteis": "encatusar",
    "encatuse": "encatusar",
    "encatusemos": "encatusar",
    "encatusen": "encatusar",
    "encatuses": "encatusar",
    "encatuso": "encatusar",
    "encatusábamos": "encatusar",
    "encatusáis": "encatusar",
    "encatusáramos": "encatusar",
    "encatusáremos": "encatusar",
    "encatusásemos": "encatusar",
    "encatusé": "encatusar",
    "encatuséis": "encatusar",
    "encatusó": "encatusar",
    "encauce": "encauzar",
    "encaucemos": "encauzar",
    "encaucen": "encauzar",
    "encauces": "encauzar",
    "encaucha": "encauchar",
    "encauchaba": "encauchar",
    "encauchabais": "encauchar",
    "encauchaban": "encauchar",
    "encauchabas": "encauchar",
    "encauchad": "encauchar",
    "encauchada": "encauchar",
    "encauchadas": "encauchar",
    "encauchado": "encauchar",
    "encauchados": "encauchar",
    "encauchamos": "encauchar",
    "encauchan": "encauchar",
    "encauchando": "encauchar",
    "encauchara": "encauchar",
    "encaucharais": "encauchar",
    "encaucharan": "encauchar",
    "encaucharas": "encauchar",
    "encauchare": "encauchar",
    "encauchareis": "encauchar",
    "encaucharemos": "encauchar",
    "encaucharen": "encauchar",
    "encauchares": "encauchar",
    "encaucharon": "encauchar",
    "encauchará": "encauchar",
    "encaucharán": "encauchar",
    "encaucharás": "encauchar",
    "encaucharé": "encauchar",
    "encaucharéis": "encauchar",
    "encaucharía": "encauchar",
    "encaucharíais": "encauchar",
    "encaucharíamos": "encauchar",
    "encaucharían": "encauchar",
    "encaucharías": "encauchar",
    "encauchas": "encauchar",
    "encauchase": "encauchar",
    "encauchaseis": "encauchar",
    "encauchasen": "encauchar",
    "encauchases": "encauchar",
    "encauchaste": "encauchar",
    "encauchasteis": "encauchar",
    "encauche": "encauchar",
    "encauchemos": "encauchar",
    "encauchen": "encauchar",
    "encauches": "encauchar",
    "encaucho": "encauchar",
    "encauchábamos": "encauchar",
    "encaucháis": "encauchar",
    "encaucháramos": "encauchar",
    "encaucháremos": "encauchar",
    "encauchásemos": "encauchar",
    "encauché": "encauchar",
    "encauchéis": "encauchar",
    "encauchó": "encauchar",
    "encaucé": "encauzar",
    "encaucéis": "encauzar",
    "encausa": "encausar",
    "encausaba": "encausar",
    "encausabais": "encausar",
    "encausaban": "encausar",
    "encausabas": "encausar",
    "encausad": "encausar",
    "encausada": "encausar",
    "encausadas": "encausar",
    "encausado": "encausar",
    "encausados": "encausar",
    "encausamos": "encausar",
    "encausan": "encausar",
    "encausando": "encausar",
    "encausara": "encausar",
    "encausarais": "encausar",
    "encausaran": "encausar",
    "encausaras": "encausar",
    "encausare": "encausar",
    "encausareis": "encausar",
    "encausaremos": "encausar",
    "encausaren": "encausar",
    "encausares": "encausar",
    "encausaron": "encausar",
    "encausará": "encausar",
    "encausarán": "encausar",
    "encausarás": "encausar",
    "encausaré": "encausar",
    "encausaréis": "encausar",
    "encausaría": "encausar",
    "encausaríais": "encausar",
    "encausaríamos": "encausar",
    "encausarían": "encausar",
    "encausarías": "encausar",
    "encausas": "encausar",
    "encausase": "encausar",
    "encausaseis": "encausar",
    "encausasen": "encausar",
    "encausases": "encausar",
    "encausaste": "encausar",
    "encausasteis": "encausar",
    "encause": "encausar",
    "encausemos": "encausar",
    "encausen": "encausar",
    "encauses": "encausar",
    "encauso": "encausar",
    "encaustes": "encauste",
    "encaustos": "encausto",
    "encausábamos": "encausar",
    "encausáis": "encausar",
    "encausáramos": "encausar",
    "encausáremos": "encausar",
    "encausásemos": "encausar",
    "encausé": "encausar",
    "encauséis": "encausar",
    "encausó": "encausar",
    "encauza": "encauzar",
    "encauzaba": "encauzar",
    "encauzabais": "encauzar",
    "encauzaban": "encauzar",
    "encauzabas": "encauzar",
    "encauzad": "encauzar",
    "encauzada": "encauzar",
    "encauzadas": "encauzar",
    "encauzado": "encauzar",
    "encauzados": "encauzar",
    "encauzamientos": "encauzamiento",
    "encauzamos": "encauzar",
    "encauzan": "encauzar",
    "encauzando": "encauzar",
    "encauzara": "encauzar",
    "encauzarais": "encauzar",
    "encauzaran": "encauzar",
    "encauzaras": "encauzar",
    "encauzare": "encauzar",
    "encauzareis": "encauzar",
    "encauzaremos": "encauzar",
    "encauzaren": "encauzar",
    "encauzares": "encauzar",
    "encauzaron": "encauzar",
    "encauzará": "encauzar",
    "encauzarán": "encauzar",
    "encauzarás": "encauzar",
    "encauzaré": "encauzar",
    "encauzaréis": "encauzar",
    "encauzaría": "encauzar",
    "encauzaríais": "encauzar",
    "encauzaríamos": "encauzar",
    "encauzarían": "encauzar",
    "encauzarías": "encauzar",
    "encauzas": "encauzar",
    "encauzase": "encauzar",
    "encauzaseis": "encauzar",
    "encauzasen": "encauzar",
    "encauzases": "encauzar",
    "encauzaste": "encauzar",
    "encauzasteis": "encauzar",
    "encauzo": "encauzar",
    "encauzábamos": "encauzar",
    "encauzáis": "encauzar",
    "encauzáramos": "encauzar",
    "encauzáremos": "encauzar",
    "encauzásemos": "encauzar",
    "encauzó": "encauzar",
    "encaña": "encañar",
    "encañaba": "encañar",
    "encañabais": "encañar",
    "encañaban": "encañar",
    "encañabas": "encañar",
    "encañad": "encañar",
    "encañada": "encañar",
    "encañadas": "encañar",
    "encañado": "encañar",
    "encañados": "encañar",
    "encañamos": "encañar",
    "encañan": "encañar",
    "encañando": "encañar",
    "encañara": "encañar",
    "encañarais": "encañar",
    "encañaran": "encañar",
    "encañaras": "encañar",
    "encañare": "encañar",
    "encañareis": "encañar",
    "encañaremos": "encañar",
    "encañaren": "encañar",
    "encañares": "encañar",
    "encañaron": "encañar",
    "encañará": "encañar",
    "encañarán": "encañar",
    "encañarás": "encañar",
    "encañaré": "encañar",
    "encañaréis": "encañar",
    "encañaría": "encañar",
    "encañaríais": "encañar",
    "encañaríamos": "encañar",
    "encañarían": "encañar",
    "encañarías": "encañar",
    "encañas": "encañar",
    "encañase": "encañar",
    "encañaseis": "encañar",
    "encañasen": "encañar",
    "encañases": "encañar",
    "encañaste": "encañar",
    "encañasteis": "encañar",
    "encañe": "encañar",
    "encañemos": "encañar",
    "encañen": "encañar",
    "encañes": "encañar",
    "encañizadas": "encañizada",
    "encaño": "encañar",
    "encañona": "encañonar",
    "encañonaba": "encañonar",
    "encañonabais": "encañonar",
    "encañonaban": "encañonar",
    "encañonabas": "encañonar",
    "encañonad": "encañonar",
    "encañonada": "encañonar",
    "encañonadas": "encañonar",
    "encañonado": "encañonar",
    "encañonados": "encañonar",
    "encañonamientos": "encañonamiento",
    "encañonamos": "encañonar",
    "encañonan": "encañonar",
    "encañonando": "encañonar",
    "encañonara": "encañonar",
    "encañonarais": "encañonar",
    "encañonaran": "encañonar",
    "encañonaras": "encañonar",
    "encañonare": "encañonar",
    "encañonareis": "encañonar",
    "encañonaremos": "encañonar",
    "encañonaren": "encañonar",
    "encañonares": "encañonar",
    "encañonaron": "encañonar",
    "encañonará": "encañonar",
    "encañonarán": "encañonar",
    "encañonarás": "encañonar",
    "encañonaré": "encañonar",
    "encañonaréis": "encañonar",
    "encañonaría": "encañonar",
    "encañonaríais": "encañonar",
    "encañonaríamos": "encañonar",
    "encañonarían": "encañonar",
    "encañonarías": "encañonar",
    "encañonas": "encañonar",
    "encañonase": "encañonar",
    "encañonaseis": "encañonar",
    "encañonasen": "encañonar",
    "encañonases": "encañonar",
    "encañonaste": "encañonar",
    "encañonasteis": "encañonar",
    "encañone": "encañonar",
    "encañonemos": "encañonar",
    "encañonen": "encañonar",
    "encañones": "encañonar",
    "encañono": "encañonar",
    "encañonábamos": "encañonar",
    "encañonáis": "encañonar",
    "encañonáramos": "encañonar",
    "encañonáremos": "encañonar",
    "encañonásemos": "encañonar",
    "encañoné": "encañonar",
    "encañonéis": "encañonar",
    "encañonó": "encañonar",
    "encañuta": "encañutar",
    "encañutaba": "encañutar",
    "encañutabais": "encañutar",
    "encañutaban": "encañutar",
    "encañutabas": "encañutar",
    "encañutad": "encañutar",
    "encañutada": "encañutar",
    "encañutadas": "encañutar",
    "encañutado": "encañutar",
    "encañutados": "encañutar",
    "encañutamos": "encañutar",
    "encañutan": "encañutar",
    "encañutando": "encañutar",
    "encañutara": "encañutar",
    "encañutarais": "encañutar",
    "encañutaran": "encañutar",
    "encañutaras": "encañutar",
    "encañutare": "encañutar",
    "encañutareis": "encañutar",
    "encañutaremos": "encañutar",
    "encañutaren": "encañutar",
    "encañutares": "encañutar",
    "encañutaron": "encañutar",
    "encañutará": "encañutar",
    "encañutarán": "encañutar",
    "encañutarás": "encañutar",
    "encañutaré": "encañutar",
    "encañutaréis": "encañutar",
    "encañutaría": "encañutar",
    "encañutaríais": "encañutar",
    "encañutaríamos": "encañutar",
    "encañutarían": "encañutar",
    "encañutarías": "encañutar",
    "encañutas": "encañutar",
    "encañutase": "encañutar",
    "encañutaseis": "encañutar",
    "encañutasen": "encañutar",
    "encañutases": "encañutar",
    "encañutaste": "encañutar",
    "encañutasteis": "encañutar",
    "encañute": "encañutar",
    "encañutemos": "encañutar",
    "encañuten": "encañutar",
    "encañutes": "encañutar",
    "encañuto": "encañutar",
    "encañutábamos": "encañutar",
    "encañutáis": "encañutar",
    "encañutáramos": "encañutar",
    "encañutáremos": "encañutar",
    "encañutásemos": "encañutar",
    "encañuté": "encañutar",
    "encañutéis": "encañutar",
    "encañutó": "encañutar",
    "encañábamos": "encañar",
    "encañáis": "encañar",
    "encañáramos": "encañar",
    "encañáremos": "encañar",
    "encañásemos": "encañar",
    "encañé": "encañar",
    "encañéis": "encañar",
    "encañó": "encañar",
    "encebada": "encebadar",
    "encebadaba": "encebadar",
    "encebadabais": "encebadar",
    "encebadaban": "encebadar",
    "encebadabas": "encebadar",
    "encebadad": "encebadar",
    "encebadada": "encebadar",
    "encebadadas": "encebadar",
    "encebadado": "encebadar",
    "encebadados": "encebadar",
    "encebadamientos": "encebadamiento",
    "encebadamos": "encebadar",
    "encebadan": "encebadar",
    "encebadando": "encebadar",
    "encebadara": "encebadar",
    "encebadarais": "encebadar",
    "encebadaran": "encebadar",
    "encebadaras": "encebadar",
    "encebadare": "encebadar",
    "encebadareis": "encebadar",
    "encebadaremos": "encebadar",
    "encebadaren": "encebadar",
    "encebadares": "encebadar",
    "encebadaron": "encebadar",
    "encebadará": "encebadar",
    "encebadarán": "encebadar",
    "encebadarás": "encebadar",
    "encebadaré": "encebadar",
    "encebadaréis": "encebadar",
    "encebadaría": "encebadar",
    "encebadaríais": "encebadar",
    "encebadaríamos": "encebadar",
    "encebadarían": "encebadar",
    "encebadarías": "encebadar",
    "encebadas": "encebadar",
    "encebadase": "encebadar",
    "encebadaseis": "encebadar",
    "encebadasen": "encebadar",
    "encebadases": "encebadar",
    "encebadaste": "encebadar",
    "encebadasteis": "encebadar",
    "encebade": "encebadar",
    "encebademos": "encebadar",
    "encebaden": "encebadar",
    "encebades": "encebadar",
    "encebado": "encebadar",
    "encebadábamos": "encebadar",
    "encebadáis": "encebadar",
    "encebadáramos": "encebadar",
    "encebadáremos": "encebadar",
    "encebadásemos": "encebadar",
    "encebadé": "encebadar",
    "encebadéis": "encebadar",
    "encebadó": "encebadar",
    "encebollados": "encebollado",
    "encefalografías": "encefalografía",
    "encefalogramas": "encefalograma",
    "encefalopatías": "encefalopatía",
    "encefalítica": "encefalítico",
    "encefalíticas": "encefalítico",
    "encefalíticos": "encefalítico",
    "encefálica": "encefálico",
    "encefálicas": "encefálico",
    "encefálicos": "encefálico",
    "encela": "encelar",
    "encelaba": "encelar",
    "encelabais": "encelar",
    "encelaban": "encelar",
    "encelabas": "encelar",
    "encelad": "encelar",
    "encelada": "encelar",
    "enceladas": "encelar",
    "encelado": "encelar",
    "encelados": "encelar",
    "encelamientos": "encelamiento",
    "encelamos": "encelar",
    "encelan": "encelar",
    "encelando": "encelar",
    "encelara": "encelar",
    "encelarais": "encelar",
    "encelaran": "encelar",
    "encelaras": "encelar",
    "encelare": "encelar",
    "encelareis": "encelar",
    "encelaremos": "encelar",
    "encelaren": "encelar",
    "encelares": "encelar",
    "encelaron": "encelar",
    "encelará": "encelar",
    "encelarán": "encelar",
    "encelarás": "encelar",
    "encelaré": "encelar",
    "encelaréis": "encelar",
    "encelaría": "encelar",
    "encelaríais": "encelar",
    "encelaríamos": "encelar",
    "encelarían": "encelar",
    "encelarías": "encelar",
    "encelas": "encelar",
    "encelase": "encelar",
    "encelaseis": "encelar",
    "encelasen": "encelar",
    "encelases": "encelar",
    "encelaste": "encelar",
    "encelasteis": "encelar",
    "enceldamientos": "enceldamiento",
    "encele": "encelar",
    "encelemos": "encelar",
    "encelen": "encelar",
    "enceles": "encelar",
    "encellas": "encella",
    "encelo": "encelar",
    "encelábamos": "encelar",
    "enceláis": "encelar",
    "enceláramos": "encelar",
    "enceláremos": "encelar",
    "encelásemos": "encelar",
    "encelé": "encelar",
    "enceléis": "encelar",
    "enceló": "encelar",
    "encenaga": "encenagar",
    "encenagaba": "encenagar",
    "encenagabais": "encenagar",
    "encenagaban": "encenagar",
    "encenagabas": "encenagar",
    "encenagad": "encenagar",
    "encenagada": "encenagar",
    "encenagadas": "encenagar",
    "encenagado": "encenagar",
    "encenagados": "encenagar",
    "encenagamientos": "encenagamiento",
    "encenagamos": "encenagar",
    "encenagan": "encenagar",
    "encenagando": "encenagar",
    "encenagara": "encenagar",
    "encenagarais": "encenagar",
    "encenagaran": "encenagar",
    "encenagaras": "encenagar",
    "encenagare": "encenagar",
    "encenagareis": "encenagar",
    "encenagaremos": "encenagar",
    "encenagaren": "encenagar",
    "encenagares": "encenagar",
    "encenagaron": "encenagar",
    "encenagará": "encenagar",
    "encenagarán": "encenagar",
    "encenagarás": "encenagar",
    "encenagaré": "encenagar",
    "encenagaréis": "encenagar",
    "encenagaría": "encenagar",
    "encenagaríais": "encenagar",
    "encenagaríamos": "encenagar",
    "encenagarían": "encenagar",
    "encenagarías": "encenagar",
    "encenagas": "encenagar",
    "encenagase": "encenagar",
    "encenagaseis": "encenagar",
    "encenagasen": "encenagar",
    "encenagases": "encenagar",
    "encenagaste": "encenagar",
    "encenagasteis": "encenagar",
    "encenago": "encenagar",
    "encenague": "encenagar",
    "encenaguemos": "encenagar",
    "encenaguen": "encenagar",
    "encenagues": "encenagar",
    "encenagué": "encenagar",
    "encenaguéis": "encenagar",
    "encenagábamos": "encenagar",
    "encenagáis": "encenagar",
    "encenagáramos": "encenagar",
    "encenagáremos": "encenagar",
    "encenagásemos": "encenagar",
    "encenagó": "encenagar",
    "encencerrada": "encencerrado",
    "encencerradas": "encencerrado",
    "encencerrados": "encencerrado",
    "encendajas": "encendaja",
    "encendamos": "encender",
    "encended": "encender",
    "encendedora": "encendedor",
    "encendedoras": "encendedor",
    "encendedores": "encendedor",
    "encendemos": "encender",
    "encenderemos": "encender",
    "encenderá": "encender",
    "encenderán": "encender",
    "encenderás": "encender",
    "encenderé": "encender",
    "encenderéis": "encender",
    "encendería": "encender",
    "encenderíais": "encender",
    "encenderíamos": "encender",
    "encenderían": "encender",
    "encenderías": "encender",
    "encendida": "encender",
    "encendidas": "encender",
    "encendido": "encender",
    "encendidos": "encendido",
    "encendiendo": "encender",
    "encendiera": "encender",
    "encendierais": "encender",
    "encendieran": "encender",
    "encendieras": "encender",
    "encendiere": "encender",
    "encendiereis": "encender",
    "encendieren": "encender",
    "encendieres": "encender",
    "encendieron": "encender",
    "encendiese": "encender",
    "encendieseis": "encender",
    "encendiesen": "encender",
    "encendieses": "encender",
    "encendimos": "encender",
    "encendiste": "encender",
    "encendisteis": "encender",
    "encendiéramos": "encender",
    "encendiéremos": "encender",
    "encendiésemos": "encender",
    "encendió": "encender",
    "encendáis": "encender",
    "encendéis": "encender",
    "encendí": "encender",
    "encendía": "encender",
    "encendíais": "encender",
    "encendíamos": "encender",
    "encendían": "encender",
    "encendías": "encender",
    "encenice": "encenizar",
    "encenicemos": "encenizar",
    "encenicen": "encenizar",
    "encenices": "encenizar",
    "encenicé": "encenizar",
    "encenicéis": "encenizar",
    "enceniza": "encenizar",
    "encenizaba": "encenizar",
    "encenizabais": "encenizar",
    "encenizaban": "encenizar",
    "encenizabas": "encenizar",
    "encenizad": "encenizar",
    "encenizada": "encenizar",
    "encenizadas": "encenizar",
    "encenizado": "encenizar",
    "encenizados": "encenizar",
    "encenizamos": "encenizar",
    "encenizan": "encenizar",
    "encenizando": "encenizar",
    "encenizara": "encenizar",
    "encenizarais": "encenizar",
    "encenizaran": "encenizar",
    "encenizaras": "encenizar",
    "encenizare": "encenizar",
    "encenizareis": "encenizar",
    "encenizaremos": "encenizar",
    "encenizaren": "encenizar",
    "encenizares": "encenizar",
    "encenizaron": "encenizar",
    "encenizará": "encenizar",
    "encenizarán": "encenizar",
    "encenizarás": "encenizar",
    "encenizaré": "encenizar",
    "encenizaréis": "encenizar",
    "encenizaría": "encenizar",
    "encenizaríais": "encenizar",
    "encenizaríamos": "encenizar",
    "encenizarían": "encenizar",
    "encenizarías": "encenizar",
    "encenizas": "encenizar",
    "encenizase": "encenizar",
    "encenizaseis": "encenizar",
    "encenizasen": "encenizar",
    "encenizases": "encenizar",
    "encenizaste": "encenizar",
    "encenizasteis": "encenizar",
    "encenizo": "encenizar",
    "encenizábamos": "encenizar",
    "encenizáis": "encenizar",
    "encenizáramos": "encenizar",
    "encenizáremos": "encenizar",
    "encenizásemos": "encenizar",
    "encenizó": "encenizar",
    "encenta": "encentar",
    "encentaba": "encentar",
    "encentabais": "encentar",
    "encentaban": "encentar",
    "encentabas": "encentar",
    "encentad": "encentar",
    "encentada": "encentar",
    "encentadas": "encentar",
    "encentado": "encentar",
    "encentados": "encentar",
    "encentamos": "encentar",
    "encentan": "encentar",
    "encentando": "encentar",
    "encentara": "encentar",
    "encentarais": "encentar",
    "encentaran": "encentar",
    "encentaras": "encentar",
    "encentare": "encentar",
    "encentareis": "encentar",
    "encentaremos": "encentar",
    "encentaren": "encentar",
    "encentares": "encentar",
    "encentaron": "encentar",
    "encentará": "encentar",
    "encentarán": "encentar",
    "encentarás": "encentar",
    "encentaré": "encentar",
    "encentaréis": "encentar",
    "encentaría": "encentar",
    "encentaríais": "encentar",
    "encentaríamos": "encentar",
    "encentarían": "encentar",
    "encentarías": "encentar",
    "encentas": "encentar",
    "encentase": "encentar",
    "encentaseis": "encentar",
    "encentasen": "encentar",
    "encentases": "encentar",
    "encentaste": "encentar",
    "encentasteis": "encentar",
    "encente": "encentar",
    "encentemos": "encentar",
    "encenten": "encentar",
    "encentes": "encentar",
    "encento": "encentar",
    "encentábamos": "encentar",
    "encentáis": "encentar",
    "encentáramos": "encentar",
    "encentáremos": "encentar",
    "encentásemos": "encentar",
    "encenté": "encentar",
    "encentéis": "encentar",
    "encentó": "encentar",
    "encepa": "encepar",
    "encepaba": "encepar",
    "encepabais": "encepar",
    "encepaban": "encepar",
    "encepabas": "encepar",
    "encepad": "encepar",
    "encepada": "encepar",
    "encepadas": "encepar",
    "encepado": "encepar",
    "encepadora": "encepador",
    "encepadoras": "encepador",
    "encepadores": "encepador",
    "encepados": "encepar",
    "encepaduras": "encepadura",
    "encepamos": "encepar",
    "encepan": "encepar",
    "encepando": "encepar",
    "encepara": "encepar",
    "enceparais": "encepar",
    "enceparan": "encepar",
    "enceparas": "encepar",
    "encepare": "encepar",
    "encepareis": "encepar",
    "enceparemos": "encepar",
    "enceparen": "encepar",
    "encepares": "encepar",
    "enceparon": "encepar",
    "encepará": "encepar",
    "enceparán": "encepar",
    "enceparás": "encepar",
    "enceparé": "encepar",
    "enceparéis": "encepar",
    "enceparía": "encepar",
    "enceparíais": "encepar",
    "enceparíamos": "encepar",
    "enceparían": "encepar",
    "enceparías": "encepar",
    "encepas": "encepar",
    "encepase": "encepar",
    "encepaseis": "encepar",
    "encepasen": "encepar",
    "encepases": "encepar",
    "encepaste": "encepar",
    "encepasteis": "encepar",
    "encepe": "encepar",
    "encepemos": "encepar",
    "encepen": "encepar",
    "encepes": "encepe",
    "encepo": "encepar",
    "encepábamos": "encepar",
    "encepáis": "encepar",
    "encepáramos": "encepar",
    "encepáremos": "encepar",
    "encepásemos": "encepar",
    "encepé": "encepar",
    "encepéis": "encepar",
    "encepó": "encepar",
    "encera": "encerar",
    "enceraba": "encerar",
    "encerabais": "encerar",
    "enceraban": "encerar",
    "encerabas": "encerar",
    "encerad": "encerar",
    "encerada": "encerar",
    "enceradas": "encerar",
    "encerado": "encerar",
    "enceradoras": "enceradora",
    "enceradores": "encerador",
    "encerados": "encerar",
    "enceramos": "encerar",
    "enceran": "encerar",
    "encerando": "encerar",
    "encerara": "encerar",
    "encerarais": "encerar",
    "enceraran": "encerar",
    "enceraras": "encerar",
    "encerare": "encerar",
    "encerareis": "encerar",
    "enceraremos": "encerar",
    "enceraren": "encerar",
    "encerares": "encerar",
    "enceraron": "encerar",
    "encerará": "encerar",
    "encerarán": "encerar",
    "encerarás": "encerar",
    "enceraré": "encerar",
    "enceraréis": "encerar",
    "enceraría": "encerar",
    "enceraríais": "encerar",
    "enceraríamos": "encerar",
    "encerarían": "encerar",
    "encerarías": "encerar",
    "enceras": "encerar",
    "encerase": "encerar",
    "enceraseis": "encerar",
    "encerasen": "encerar",
    "encerases": "encerar",
    "enceraste": "encerar",
    "encerasteis": "encerar",
    "encere": "encerar",
    "enceremos": "encerar",
    "enceren": "encerar",
    "enceres": "encerar",
    "encero": "encerar",
    "encerota": "encerotar",
    "encerotaba": "encerotar",
    "encerotabais": "encerotar",
    "encerotaban": "encerotar",
    "encerotabas": "encerotar",
    "encerotad": "encerotar",
    "encerotada": "encerotar",
    "encerotadas": "encerotar",
    "encerotado": "encerotar",
    "encerotados": "encerotar",
    "encerotamos": "encerotar",
    "encerotan": "encerotar",
    "encerotando": "encerotar",
    "encerotara": "encerotar",
    "encerotarais": "encerotar",
    "encerotaran": "encerotar",
    "encerotaras": "encerotar",
    "encerotare": "encerotar",
    "encerotareis": "encerotar",
    "encerotaremos": "encerotar",
    "encerotaren": "encerotar",
    "encerotares": "encerotar",
    "encerotaron": "encerotar",
    "encerotará": "encerotar",
    "encerotarán": "encerotar",
    "encerotarás": "encerotar",
    "encerotaré": "encerotar",
    "encerotaréis": "encerotar",
    "encerotaría": "encerotar",
    "encerotaríais": "encerotar",
    "encerotaríamos": "encerotar",
    "encerotarían": "encerotar",
    "encerotarías": "encerotar",
    "encerotas": "encerotar",
    "encerotase": "encerotar",
    "encerotaseis": "encerotar",
    "encerotasen": "encerotar",
    "encerotases": "encerotar",
    "encerotaste": "encerotar",
    "encerotasteis": "encerotar",
    "encerote": "encerotar",
    "encerotemos": "encerotar",
    "enceroten": "encerotar",
    "encerotes": "encerotar",
    "enceroto": "encerotar",
    "encerotábamos": "encerotar",
    "encerotáis": "encerotar",
    "encerotáramos": "encerotar",
    "encerotáremos": "encerotar",
    "encerotásemos": "encerotar",
    "enceroté": "encerotar",
    "encerotéis": "encerotar",
    "encerotó": "encerotar",
    "encerraba": "encerrar",
    "encerrabais": "encerrar",
    "encerraban": "encerrar",
    "encerrabas": "encerrar",
    "encerrables": "encerrable",
    "encerrad": "encerrar",
    "encerrada": "encerrar",
    "encerradas": "encerrar",
    "encerraderos": "encerradero",
    "encerrado": "encerrar",
    "encerrados": "encerrar",
    "encerramientos": "encerramiento",
    "encerramos": "encerrar",
    "encerrando": "encerrar",
    "encerrara": "encerrar",
    "encerrarais": "encerrar",
    "encerraran": "encerrar",
    "encerraras": "encerrar",
    "encerrare": "encerrar",
    "encerrareis": "encerrar",
    "encerraremos": "encerrar",
    "encerraren": "encerrar",
    "encerrares": "encerrar",
    "encerraron": "encerrar",
    "encerrará": "encerrar",
    "encerrarán": "encerrar",
    "encerrarás": "encerrar",
    "encerraré": "encerrar",
    "encerraréis": "encerrar",
    "encerraría": "encerrar",
    "encerraríais": "encerrar",
    "encerraríamos": "encerrar",
    "encerrarían": "encerrar",
    "encerrarías": "encerrar",
    "encerrase": "encerrar",
    "encerraseis": "encerrar",
    "encerrasen": "encerrar",
    "encerrases": "encerrar",
    "encerraste": "encerrar",
    "encerrasteis": "encerrar",
    "encerremos": "encerrar",
    "encerronas": "encerrona",
    "encerrábamos": "encerrar",
    "encerráis": "encerrar",
    "encerráramos": "encerrar",
    "encerráremos": "encerrar",
    "encerrásemos": "encerrar",
    "encerré": "encerrar",
    "encerréis": "encerrar",
    "encerró": "encerrar",
    "encerábamos": "encerar",
    "enceráis": "encerar",
    "enceráramos": "encerar",
    "enceráremos": "encerar",
    "encerásemos": "encerar",
    "enceré": "encerar",
    "enceréis": "encerar",
    "enceró": "encerar",
    "encespeda": "encespedar",
    "encespedaba": "encespedar",
    "encespedabais": "encespedar",
    "encespedaban": "encespedar",
    "encespedabas": "encespedar",
    "encespedad": "encespedar",
    "encespedada": "encespedar",
    "encespedadas": "encespedar",
    "encespedado": "encespedar",
    "encespedados": "encespedar",
    "encespedamos": "encespedar",
    "encespedan": "encespedar",
    "encespedando": "encespedar",
    "encespedara": "encespedar",
    "encespedarais": "encespedar",
    "encespedaran": "encespedar",
    "encespedaras": "encespedar",
    "encespedare": "encespedar",
    "encespedareis": "encespedar",
    "encespedaremos": "encespedar",
    "encespedaren": "encespedar",
    "encespedares": "encespedar",
    "encespedaron": "encespedar",
    "encespedará": "encespedar",
    "encespedarán": "encespedar",
    "encespedarás": "encespedar",
    "encespedaré": "encespedar",
    "encespedaréis": "encespedar",
    "encespedaría": "encespedar",
    "encespedaríais": "encespedar",
    "encespedaríamos": "encespedar",
    "encespedarían": "encespedar",
    "encespedarías": "encespedar",
    "encespedas": "encespedar",
    "encespedase": "encespedar",
    "encespedaseis": "encespedar",
    "encespedasen": "encespedar",
    "encespedases": "encespedar",
    "encespedaste": "encespedar",
    "encespedasteis": "encespedar",
    "encespede": "encespedar",
    "encespedemos": "encespedar",
    "encespeden": "encespedar",
    "encespedes": "encespedar",
    "encespedo": "encespedar",
    "encespedábamos": "encespedar",
    "encespedáis": "encespedar",
    "encespedáramos": "encespedar",
    "encespedáremos": "encespedar",
    "encespedásemos": "encespedar",
    "encespedé": "encespedar",
    "encespedéis": "encespedar",
    "encespedó": "encespedar",
    "encesta": "encestar",
    "encestaba": "encestar",
    "encestabais": "encestar",
    "encestaban": "encestar",
    "encestabas": "encestar",
    "encestad": "encestar",
    "encestada": "encestar",
    "encestadas": "encestar",
    "encestado": "encestar",
    "encestadora": "encestador",
    "encestadoras": "encestador",
    "encestadores": "encestador",
    "encestados": "encestar",
    "encestamos": "encestar",
    "encestan": "encestar",
    "encestando": "encestar",
    "encestara": "encestar",
    "encestarais": "encestar",
    "encestaran": "encestar",
    "encestaras": "encestar",
    "encestare": "encestar",
    "encestareis": "encestar",
    "encestaremos": "encestar",
    "encestaren": "encestar",
    "encestares": "encestar",
    "encestaron": "encestar",
    "encestará": "encestar",
    "encestarán": "encestar",
    "encestarás": "encestar",
    "encestaré": "encestar",
    "encestaréis": "encestar",
    "encestaría": "encestar",
    "encestaríais": "encestar",
    "encestaríamos": "encestar",
    "encestarían": "encestar",
    "encestarías": "encestar",
    "encestas": "encestar",
    "encestase": "encestar",
    "encestaseis": "encestar",
    "encestasen": "encestar",
    "encestases": "encestar",
    "encestaste": "encestar",
    "encestasteis": "encestar",
    "enceste": "encestar",
    "encestemos": "encestar",
    "encesten": "encestar",
    "encestes": "enceste",
    "encesto": "encestar",
    "encestábamos": "encestar",
    "encestáis": "encestar",
    "encestáramos": "encestar",
    "encestáremos": "encestar",
    "encestásemos": "encestar",
    "encesté": "encestar",
    "encestéis": "encestar",
    "encestó": "encestar",
    "enchaleca": "enchalecar",
    "enchalecaba": "enchalecar",
    "enchalecabais": "enchalecar",
    "enchalecaban": "enchalecar",
    "enchalecabas": "enchalecar",
    "enchalecad": "enchalecar",
    "enchalecada": "enchalecar",
    "enchalecadas": "enchalecar",
    "enchalecado": "enchalecar",
    "enchalecados": "enchalecar",
    "enchalecamos": "enchalecar",
    "enchalecan": "enchalecar",
    "enchalecando": "enchalecar",
    "enchalecara": "enchalecar",
    "enchalecarais": "enchalecar",
    "enchalecaran": "enchalecar",
    "enchalecaras": "enchalecar",
    "enchalecare": "enchalecar",
    "enchalecareis": "enchalecar",
    "enchalecaremos": "enchalecar",
    "enchalecaren": "enchalecar",
    "enchalecares": "enchalecar",
    "enchalecaron": "enchalecar",
    "enchalecará": "enchalecar",
    "enchalecarán": "enchalecar",
    "enchalecarás": "enchalecar",
    "enchalecaré": "enchalecar",
    "enchalecaréis": "enchalecar",
    "enchalecaría": "enchalecar",
    "enchalecaríais": "enchalecar",
    "enchalecaríamos": "enchalecar",
    "enchalecarían": "enchalecar",
    "enchalecarías": "enchalecar",
    "enchalecas": "enchalecar",
    "enchalecase": "enchalecar",
    "enchalecaseis": "enchalecar",
    "enchalecasen": "enchalecar",
    "enchalecases": "enchalecar",
    "enchalecaste": "enchalecar",
    "enchalecasteis": "enchalecar",
    "enchaleco": "enchalecar",
    "enchalecábamos": "enchalecar",
    "enchalecáis": "enchalecar",
    "enchalecáramos": "enchalecar",
    "enchalecáremos": "enchalecar",
    "enchalecásemos": "enchalecar",
    "enchalecó": "enchalecar",
    "enchaleque": "enchalecar",
    "enchalequemos": "enchalecar",
    "enchalequen": "enchalecar",
    "enchaleques": "enchalecar",
    "enchalequé": "enchalecar",
    "enchalequéis": "enchalecar",
    "enchancleta": "enchancletar",
    "enchancletaba": "enchancletar",
    "enchancletabais": "enchancletar",
    "enchancletaban": "enchancletar",
    "enchancletabas": "enchancletar",
    "enchancletad": "enchancletar",
    "enchancletada": "enchancletar",
    "enchancletadas": "enchancletar",
    "enchancletado": "enchancletar",
    "enchancletados": "enchancletar",
    "enchancletamos": "enchancletar",
    "enchancletan": "enchancletar",
    "enchancletando": "enchancletar",
    "enchancletara": "enchancletar",
    "enchancletarais": "enchancletar",
    "enchancletaran": "enchancletar",
    "enchancletaras": "enchancletar",
    "enchancletare": "enchancletar",
    "enchancletareis": "enchancletar",
    "enchancletaremos": "enchancletar",
    "enchancletaren": "enchancletar",
    "enchancletares": "enchancletar",
    "enchancletaron": "enchancletar",
    "enchancletará": "enchancletar",
    "enchancletarán": "enchancletar",
    "enchancletarás": "enchancletar",
    "enchancletaré": "enchancletar",
    "enchancletaréis": "enchancletar",
    "enchancletaría": "enchancletar",
    "enchancletaríais": "enchancletar",
    "enchancletaríamos": "enchancletar",
    "enchancletarían": "enchancletar",
    "enchancletarías": "enchancletar",
    "enchancletas": "enchancletar",
    "enchancletase": "enchancletar",
    "enchancletaseis": "enchancletar",
    "enchancletasen": "enchancletar",
    "enchancletases": "enchancletar",
    "enchancletaste": "enchancletar",
    "enchancletasteis": "enchancletar",
    "enchanclete": "enchancletar",
    "enchancletemos": "enchancletar",
    "enchancleten": "enchancletar",
    "enchancletes": "enchancletar",
    "enchancleto": "enchancletar",
    "enchancletábamos": "enchancletar",
    "enchancletáis": "enchancletar",
    "enchancletáramos": "enchancletar",
    "enchancletáremos": "enchancletar",
    "enchancletásemos": "enchancletar",
    "enchancleté": "enchancletar",
    "enchancletéis": "enchancletar",
    "enchancletó": "enchancletar",
    "enchapa": "enchapar",
    "enchapaba": "enchapar",
    "enchapabais": "enchapar",
    "enchapaban": "enchapar",
    "enchapabas": "enchapar",
    "enchapad": "enchapar",
    "enchapada": "enchapar",
    "enchapadas": "enchapar",
    "enchapado": "enchapar",
    "enchapados": "enchapar",
    "enchapamos": "enchapar",
    "enchapan": "enchapar",
    "enchapando": "enchapar",
    "enchapara": "enchapar",
    "enchaparais": "enchapar",
    "enchaparan": "enchapar",
    "enchaparas": "enchapar",
    "enchapare": "enchapar",
    "enchapareis": "enchapar",
    "enchaparemos": "enchapar",
    "enchaparen": "enchapar",
    "enchapares": "enchapar",
    "enchaparon": "enchapar",
    "enchapará": "enchapar",
    "enchaparán": "enchapar",
    "enchaparás": "enchapar",
    "enchaparé": "enchapar",
    "enchaparéis": "enchapar",
    "enchaparía": "enchapar",
    "enchaparíais": "enchapar",
    "enchaparíamos": "enchapar",
    "enchaparían": "enchapar",
    "enchaparías": "enchapar",
    "enchapas": "enchapar",
    "enchapase": "enchapar",
    "enchapaseis": "enchapar",
    "enchapasen": "enchapar",
    "enchapases": "enchapar",
    "enchapaste": "enchapar",
    "enchapasteis": "enchapar",
    "enchape": "enchapar",
    "enchapemos": "enchapar",
    "enchapen": "enchapar",
    "enchapes": "enchapar",
    "enchapinada": "enchapinado",
    "enchapinadas": "enchapinado",
    "enchapinados": "enchapinado",
    "enchapo": "enchapar",
    "enchapábamos": "enchapar",
    "enchapáis": "enchapar",
    "enchapáramos": "enchapar",
    "enchapáremos": "enchapar",
    "enchapásemos": "enchapar",
    "enchapé": "enchapar",
    "enchapéis": "enchapar",
    "enchapó": "enchapar",
    "encharca": "encharcar",
    "encharcaba": "encharcar",
    "encharcabais": "encharcar",
    "encharcaban": "encharcar",
    "encharcabas": "encharcar",
    "encharcad": "encharcar",
    "encharcada": "encharcar",
    "encharcadas": "encharcar",
    "encharcado": "encharcar",
    "encharcados": "encharcar",
    "encharcamientos": "encharcamiento",
    "encharcamos": "encharcar",
    "encharcan": "encharcar",
    "encharcando": "encharcar",
    "encharcara": "encharcar",
    "encharcarais": "encharcar",
    "encharcaran": "encharcar",
    "encharcaras": "encharcar",
    "encharcare": "encharcar",
    "encharcareis": "encharcar",
    "encharcaremos": "encharcar",
    "encharcaren": "encharcar",
    "encharcares": "encharcar",
    "encharcaron": "encharcar",
    "encharcará": "encharcar",
    "encharcarán": "encharcar",
    "encharcarás": "encharcar",
    "encharcaré": "encharcar",
    "encharcaréis": "encharcar",
    "encharcaría": "encharcar",
    "encharcaríais": "encharcar",
    "encharcaríamos": "encharcar",
    "encharcarían": "encharcar",
    "encharcarías": "encharcar",
    "encharcas": "encharcar",
    "encharcase": "encharcar",
    "encharcaseis": "encharcar",
    "encharcasen": "encharcar",
    "encharcases": "encharcar",
    "encharcaste": "encharcar",
    "encharcasteis": "encharcar",
    "encharco": "encharcar",
    "encharcábamos": "encharcar",
    "encharcáis": "encharcar",
    "encharcáramos": "encharcar",
    "encharcáremos": "encharcar",
    "encharcásemos": "encharcar",
    "encharcó": "encharcar",
    "encharque": "encharcar",
    "encharquemos": "encharcar",
    "encharquen": "encharcar",
    "encharques": "encharcar",
    "encharqué": "encharcar",
    "encharquéis": "encharcar",
    "enchaveta": "enchavetar",
    "enchavetaba": "enchavetar",
    "enchavetabais": "enchavetar",
    "enchavetaban": "enchavetar",
    "enchavetabas": "enchavetar",
    "enchavetad": "enchavetar",
    "enchavetada": "enchavetar",
    "enchavetadas": "enchavetar",
    "enchavetado": "enchavetar",
    "enchavetados": "enchavetar",
    "enchavetamos": "enchavetar",
    "enchavetan": "enchavetar",
    "enchavetando": "enchavetar",
    "enchavetara": "enchavetar",
    "enchavetarais": "enchavetar",
    "enchavetaran": "enchavetar",
    "enchavetaras": "enchavetar",
    "enchavetare": "enchavetar",
    "enchavetareis": "enchavetar",
    "enchavetaremos": "enchavetar",
    "enchavetaren": "enchavetar",
    "enchavetares": "enchavetar",
    "enchavetaron": "enchavetar",
    "enchavetará": "enchavetar",
    "enchavetarán": "enchavetar",
    "enchavetarás": "enchavetar",
    "enchavetaré": "enchavetar",
    "enchavetaréis": "enchavetar",
    "enchavetaría": "enchavetar",
    "enchavetaríais": "enchavetar",
    "enchavetaríamos": "enchavetar",
    "enchavetarían": "enchavetar",
    "enchavetarías": "enchavetar",
    "enchavetas": "enchavetar",
    "enchavetase": "enchavetar",
    "enchavetaseis": "enchavetar",
    "enchavetasen": "enchavetar",
    "enchavetases": "enchavetar",
    "enchavetaste": "enchavetar",
    "enchavetasteis": "enchavetar",
    "enchavete": "enchavetar",
    "enchavetemos": "enchavetar",
    "enchaveten": "enchavetar",
    "enchavetes": "enchavetar",
    "enchaveto": "enchavetar",
    "enchavetábamos": "enchavetar",
    "enchavetáis": "enchavetar",
    "enchavetáramos": "enchavetar",
    "enchavetáremos": "enchavetar",
    "enchavetásemos": "enchavetar",
    "enchaveté": "enchavetar",
    "enchavetéis": "enchavetar",
    "enchavetó": "enchavetar",
    "enchila": "enchilar",
    "enchilaba": "enchilar",
    "enchilabais": "enchilar",
    "enchilaban": "enchilar",
    "enchilabas": "enchilar",
    "enchilad": "enchilar",
    "enchilada": "enchilar",
    "enchiladas": "enchilar",
    "enchilado": "enchilar",
    "enchilados": "enchilar",
    "enchilamos": "enchilar",
    "enchilan": "enchilar",
    "enchilando": "enchilar",
    "enchilara": "enchilar",
    "enchilarais": "enchilar",
    "enchilaran": "enchilar",
    "enchilaras": "enchilar",
    "enchilare": "enchilar",
    "enchilareis": "enchilar",
    "enchilaremos": "enchilar",
    "enchilaren": "enchilar",
    "enchilares": "enchilar",
    "enchilaron": "enchilar",
    "enchilará": "enchilar",
    "enchilarán": "enchilar",
    "enchilarás": "enchilar",
    "enchilaré": "enchilar",
    "enchilaréis": "enchilar",
    "enchilaría": "enchilar",
    "enchilaríais": "enchilar",
    "enchilaríamos": "enchilar",
    "enchilarían": "enchilar",
    "enchilarías": "enchilar",
    "enchilas": "enchilar",
    "enchilase": "enchilar",
    "enchilaseis": "enchilar",
    "enchilasen": "enchilar",
    "enchilases": "enchilar",
    "enchilaste": "enchilar",
    "enchilasteis": "enchilar",
    "enchile": "enchilar",
    "enchilemos": "enchilar",
    "enchilen": "enchilar",
    "enchiles": "enchilar",
    "enchilo": "enchilar",
    "enchilábamos": "enchilar",
    "enchiláis": "enchilar",
    "enchiláramos": "enchilar",
    "enchiláremos": "enchilar",
    "enchilásemos": "enchilar",
    "enchilé": "enchilar",
    "enchiléis": "enchilar",
    "enchiló": "enchilar",
    "enchina": "enchinar",
    "enchinaba": "enchinar",
    "enchinabais": "enchinar",
    "enchinaban": "enchinar",
    "enchinabas": "enchinar",
    "enchinad": "enchinar",
    "enchinada": "enchinar",
    "enchinadas": "enchinar",
    "enchinado": "enchinar",
    "enchinados": "enchinar",
    "enchinamos": "enchinar",
    "enchinan": "enchinar",
    "enchinando": "enchinar",
    "enchinara": "enchinar",
    "enchinarais": "enchinar",
    "enchinaran": "enchinar",
    "enchinaras": "enchinar",
    "enchinare": "enchinar",
    "enchinareis": "enchinar",
    "enchinaremos": "enchinar",
    "enchinaren": "enchinar",
    "enchinares": "enchinar",
    "enchinaron": "enchinar",
    "enchinará": "enchinar",
    "enchinarán": "enchinar",
    "enchinarás": "enchinar",
    "enchinaré": "enchinar",
    "enchinaréis": "enchinar",
    "enchinaría": "enchinar",
    "enchinaríais": "enchinar",
    "enchinaríamos": "enchinar",
    "enchinarían": "enchinar",
    "enchinarías": "enchinar",
    "enchinas": "enchinar",
    "enchinase": "enchinar",
    "enchinaseis": "enchinar",
    "enchinasen": "enchinar",
    "enchinases": "enchinar",
    "enchinaste": "enchinar",
    "enchinasteis": "enchinar",
    "enchine": "enchinar",
    "enchinemos": "enchinar",
    "enchinen": "enchinar",
    "enchines": "enchinar",
    "enchino": "enchinar",
    "enchinábamos": "enchinar",
    "enchináis": "enchinar",
    "enchináramos": "enchinar",
    "enchináremos": "enchinar",
    "enchinásemos": "enchinar",
    "enchiné": "enchinar",
    "enchinéis": "enchinar",
    "enchinó": "enchinar",
    "enchiqueramientos": "enchiqueramiento",
    "enchirona": "enchironar",
    "enchironaba": "enchironar",
    "enchironabais": "enchironar",
    "enchironaban": "enchironar",
    "enchironabas": "enchironar",
    "enchironad": "enchironar",
    "enchironada": "enchironar",
    "enchironadas": "enchironar",
    "enchironado": "enchironar",
    "enchironados": "enchironar",
    "enchironamos": "enchironar",
    "enchironan": "enchironar",
    "enchironando": "enchironar",
    "enchironara": "enchironar",
    "enchironarais": "enchironar",
    "enchironaran": "enchironar",
    "enchironaras": "enchironar",
    "enchironare": "enchironar",
    "enchironareis": "enchironar",
    "enchironaremos": "enchironar",
    "enchironaren": "enchironar",
    "enchironares": "enchironar",
    "enchironaron": "enchironar",
    "enchironará": "enchironar",
    "enchironarán": "enchironar",
    "enchironarás": "enchironar",
    "enchironaré": "enchironar",
    "enchironaréis": "enchironar",
    "enchironaría": "enchironar",
    "enchironaríais": "enchironar",
    "enchironaríamos": "enchironar",
    "enchironarían": "enchironar",
    "enchironarías": "enchironar",
    "enchironas": "enchironar",
    "enchironase": "enchironar",
    "enchironaseis": "enchironar",
    "enchironasen": "enchironar",
    "enchironases": "enchironar",
    "enchironaste": "enchironar",
    "enchironasteis": "enchironar",
    "enchirone": "enchironar",
    "enchironemos": "enchironar",
    "enchironen": "enchironar",
    "enchirones": "enchironar",
    "enchirono": "enchironar",
    "enchironábamos": "enchironar",
    "enchironáis": "enchironar",
    "enchironáramos": "enchironar",
    "enchironáremos": "enchironar",
    "enchironásemos": "enchironar",
    "enchironé": "enchironar",
    "enchironéis": "enchironar",
    "enchironó": "enchironar",
    "enchueca": "enchuecar",
    "enchuecaba": "enchuecar",
    "enchuecabais": "enchuecar",
    "enchuecaban": "enchuecar",
    "enchuecabas": "enchuecar",
    "enchuecad": "enchuecar",
    "enchuecada": "enchuecar",
    "enchuecadas": "enchuecar",
    "enchuecado": "enchuecar",
    "enchuecados": "enchuecar",
    "enchuecamos": "enchuecar",
    "enchuecan": "enchuecar",
    "enchuecando": "enchuecar",
    "enchuecara": "enchuecar",
    "enchuecarais": "enchuecar",
    "enchuecaran": "enchuecar",
    "enchuecaras": "enchuecar",
    "enchuecare": "enchuecar",
    "enchuecareis": "enchuecar",
    "enchuecaremos": "enchuecar",
    "enchuecaren": "enchuecar",
    "enchuecares": "enchuecar",
    "enchuecaron": "enchuecar",
    "enchuecará": "enchuecar",
    "enchuecarán": "enchuecar",
    "enchuecarás": "enchuecar",
    "enchuecaré": "enchuecar",
    "enchuecaréis": "enchuecar",
    "enchuecaría": "enchuecar",
    "enchuecaríais": "enchuecar",
    "enchuecaríamos": "enchuecar",
    "enchuecarían": "enchuecar",
    "enchuecarías": "enchuecar",
    "enchuecas": "enchuecar",
    "enchuecase": "enchuecar",
    "enchuecaseis": "enchuecar",
    "enchuecasen": "enchuecar",
    "enchuecases": "enchuecar",
    "enchuecaste": "enchuecar",
    "enchuecasteis": "enchuecar",
    "enchueco": "enchuecar",
    "enchuecábamos": "enchuecar",
    "enchuecáis": "enchuecar",
    "enchuecáramos": "enchuecar",
    "enchuecáremos": "enchuecar",
    "enchuecásemos": "enchuecar",
    "enchuecó": "enchuecar",
    "enchueque": "enchuecar",
    "enchuequemos": "enchuecar",
    "enchuequen": "enchuecar",
    "enchueques": "enchuecar",
    "enchuequé": "enchuecar",
    "enchuequéis": "enchuecar",
    "enchufa": "enchufar",
    "enchufaba": "enchufar",
    "enchufabais": "enchufar",
    "enchufaban": "enchufar",
    "enchufabas": "enchufar",
    "enchufables": "enchufable",
    "enchufad": "enchufar",
    "enchufada": "enchufar",
    "enchufadas": "enchufar",
    "enchufado": "enchufar",
    "enchufados": "enchufar",
    "enchufamos": "enchufar",
    "enchufan": "enchufar",
    "enchufando": "enchufar",
    "enchufara": "enchufar",
    "enchufarais": "enchufar",
    "enchufaran": "enchufar",
    "enchufaras": "enchufar",
    "enchufare": "enchufar",
    "enchufareis": "enchufar",
    "enchufaremos": "enchufar",
    "enchufaren": "enchufar",
    "enchufares": "enchufar",
    "enchufaron": "enchufar",
    "enchufará": "enchufar",
    "enchufarán": "enchufar",
    "enchufarás": "enchufar",
    "enchufaré": "enchufar",
    "enchufaréis": "enchufar",
    "enchufaría": "enchufar",
    "enchufaríais": "enchufar",
    "enchufaríamos": "enchufar",
    "enchufarían": "enchufar",
    "enchufarías": "enchufar",
    "enchufas": "enchufar",
    "enchufase": "enchufar",
    "enchufaseis": "enchufar",
    "enchufasen": "enchufar",
    "enchufases": "enchufar",
    "enchufaste": "enchufar",
    "enchufasteis": "enchufar",
    "enchufe": "enchufar",
    "enchufemos": "enchufar",
    "enchufen": "enchufar",
    "enchufes": "enchufe",
    "enchufo": "enchufar",
    "enchufábamos": "enchufar",
    "enchufáis": "enchufar",
    "enchufáramos": "enchufar",
    "enchufáremos": "enchufar",
    "enchufásemos": "enchufar",
    "enchufé": "enchufar",
    "enchuféis": "enchufar",
    "enchufó": "enchufar",
    "enciclopedias": "enciclopedia",
    "enciclopedismos": "enciclopedismo",
    "enciclopedistas": "enciclopedista",
    "enciclopédica": "enciclopédico",
    "enciclopédicas": "enciclopédico",
    "enciclopédicos": "enciclopédico",
    "encienda": "encender",
    "enciendan": "encender",
    "enciendas": "encender",
    "enciende": "encender",
    "encienden": "encender",
    "enciendes": "encender",
    "enciendo": "encender",
    "encierra": "encerrar",
    "encierran": "encerrar",
    "encierras": "encerrar",
    "encierre": "encerrar",
    "encierren": "encerrar",
    "encierres": "encerrar",
    "encierro": "encerrar",
    "encierros": "encierro",
    "encima": "encimar",
    "encimaba": "encimar",
    "encimabais": "encimar",
    "encimaban": "encimar",
    "encimabas": "encimar",
    "encimad": "encimar",
    "encimada": "encimar",
    "encimadas": "encimar",
    "encimado": "encimar",
    "encimados": "encimar",
    "encimamos": "encimar",
    "enciman": "encimar",
    "encimando": "encimar",
    "encimara": "encimar",
    "encimarais": "encimar",
    "encimaran": "encimar",
    "encimaras": "encimar",
    "encimare": "encimar",
    "encimareis": "encimar",
    "encimaremos": "encimar",
    "encimaren": "encimar",
    "encimares": "encimar",
    "encimaron": "encimar",
    "encimará": "encimar",
    "encimarán": "encimar",
    "encimarás": "encimar",
    "encimaré": "encimar",
    "encimaréis": "encimar",
    "encimaría": "encimar",
    "encimaríais": "encimar",
    "encimaríamos": "encimar",
    "encimarían": "encimar",
    "encimarías": "encimar",
    "encimas": "encimar",
    "encimase": "encimar",
    "encimaseis": "encimar",
    "encimasen": "encimar",
    "encimases": "encimar",
    "encimaste": "encimar",
    "encimasteis": "encimar",
    "encime": "encimar",
    "encimemos": "encimar",
    "encimen": "encimar",
    "encimera": "encimero",
    "encimeras": "encimero",
    "encimeros": "encimero",
    "encimes": "encimar",
    "encimo": "encimar",
    "encimábamos": "encimar",
    "encimáis": "encimar",
    "encimáramos": "encimar",
    "encimáremos": "encimar",
    "encimásemos": "encimar",
    "encimé": "encimar",
    "enciméis": "encimar",
    "encimó": "encimar",
    "encinales": "encinal",
    "encinares": "encinar",
    "encinas": "encina",
    "encinos": "encino",
    "encinta": "encintar",
    "encintaba": "encintar",
    "encintabais": "encintar",
    "encintaban": "encintar",
    "encintabas": "encintar",
    "encintad": "encintar",
    "encintada": "encintar",
    "encintadas": "encintar",
    "encintado": "encintar",
    "encintados": "encintar",
    "encintamos": "encintar",
    "encintan": "encintar",
    "encintando": "encintar",
    "encintara": "encintar",
    "encintarais": "encintar",
    "encintaran": "encintar",
    "encintaras": "encintar",
    "encintare": "encintar",
    "encintareis": "encintar",
    "encintaremos": "encintar",
    "encintaren": "encintar",
    "encintares": "encintar",
    "encintaron": "encintar",
    "encintará": "encintar",
    "encintarán": "encintar",
    "encintarás": "encintar",
    "encintaré": "encintar",
    "encintaréis": "encintar",
    "encintaría": "encintar",
    "encintaríais": "encintar",
    "encintaríamos": "encintar",
    "encintarían": "encintar",
    "encintarías": "encintar",
    "encintas": "encintar",
    "encintase": "encintar",
    "encintaseis": "encintar",
    "encintasen": "encintar",
    "encintases": "encintar",
    "encintaste": "encintar",
    "encintasteis": "encintar",
    "encinte": "encintar",
    "encintemos": "encintar",
    "encinten": "encintar",
    "encintes": "encintar",
    "encinto": "encintar",
    "encintábamos": "encintar",
    "encintáis": "encintar",
    "encintáramos": "encintar",
    "encintáremos": "encintar",
    "encintásemos": "encintar",
    "encinté": "encintar",
    "encintéis": "encintar",
    "encintó": "encintar",
    "encisma": "encismar",
    "encismaba": "encismar",
    "encismabais": "encismar",
    "encismaban": "encismar",
    "encismabas": "encismar",
    "encismad": "encismar",
    "encismada": "encismar",
    "encismadas": "encismar",
    "encismado": "encismar",
    "encismados": "encismar",
    "encismamos": "encismar",
    "encisman": "encismar",
    "encismando": "encismar",
    "encismara": "encismar",
    "encismarais": "encismar",
    "encismaran": "encismar",
    "encismaras": "encismar",
    "encismare": "encismar",
    "encismareis": "encismar",
    "encismaremos": "encismar",
    "encismaren": "encismar",
    "encismares": "encismar",
    "encismaron": "encismar",
    "encismará": "encismar",
    "encismarán": "encismar",
    "encismarás": "encismar",
    "encismaré": "encismar",
    "encismaréis": "encismar",
    "encismaría": "encismar",
    "encismaríais": "encismar",
    "encismaríamos": "encismar",
    "encismarían": "encismar",
    "encismarías": "encismar",
    "encismas": "encismar",
    "encismase": "encismar",
    "encismaseis": "encismar",
    "encismasen": "encismar",
    "encismases": "encismar",
    "encismaste": "encismar",
    "encismasteis": "encismar",
    "encisme": "encismar",
    "encismemos": "encismar",
    "encismen": "encismar",
    "encismes": "encismar",
    "encismo": "encismar",
    "encismábamos": "encismar",
    "encismáis": "encismar",
    "encismáramos": "encismar",
    "encismáremos": "encismar",
    "encismásemos": "encismar",
    "encismé": "encismar",
    "encisméis": "encismar",
    "encismó": "encismar",
    "encizaña": "encizañar",
    "encizañaba": "encizañar",
    "encizañabais": "encizañar",
    "encizañaban": "encizañar",
    "encizañabas": "encizañar",
    "encizañad": "encizañar",
    "encizañada": "encizañar",
    "encizañadas": "encizañar",
    "encizañado": "encizañar",
    "encizañadora": "encizañador",
    "encizañadoras": "encizañador",
    "encizañadores": "encizañador",
    "encizañados": "encizañar",
    "encizañamos": "encizañar",
    "encizañan": "encizañar",
    "encizañando": "encizañar",
    "encizañara": "encizañar",
    "encizañarais": "encizañar",
    "encizañaran": "encizañar",
    "encizañaras": "encizañar",
    "encizañare": "encizañar",
    "encizañareis": "encizañar",
    "encizañaremos": "encizañar",
    "encizañaren": "encizañar",
    "encizañares": "encizañar",
    "encizañaron": "encizañar",
    "encizañará": "encizañar",
    "encizañarán": "encizañar",
    "encizañarás": "encizañar",
    "encizañaré": "encizañar",
    "encizañaréis": "encizañar",
    "encizañaría": "encizañar",
    "encizañaríais": "encizañar",
    "encizañaríamos": "encizañar",
    "encizañarían": "encizañar",
    "encizañarías": "encizañar",
    "encizañas": "encizañar",
    "encizañase": "encizañar",
    "encizañaseis": "encizañar",
    "encizañasen": "encizañar",
    "encizañases": "encizañar",
    "encizañaste": "encizañar",
    "encizañasteis": "encizañar",
    "encizañe": "encizañar",
    "encizañemos": "encizañar",
    "encizañen": "encizañar",
    "encizañes": "encizañar",
    "encizaño": "encizañar",
    "encizañábamos": "encizañar",
    "encizañáis": "encizañar",
    "encizañáramos": "encizañar",
    "encizañáremos": "encizañar",
    "encizañásemos": "encizañar",
    "encizañé": "encizañar",
    "encizañéis": "encizañar",
    "encizañó": "encizañar",
    "enclaustra": "enclaustrar",
    "enclaustraba": "enclaustrar",
    "enclaustrabais": "enclaustrar",
    "enclaustraban": "enclaustrar",
    "enclaustrabas": "enclaustrar",
    "enclaustrad": "enclaustrar",
    "enclaustrada": "enclaustrar",
    "enclaustradas": "enclaustrar",
    "enclaustrado": "enclaustrar",
    "enclaustrados": "enclaustrar",
    "enclaustramientos": "enclaustramiento",
    "enclaustramos": "enclaustrar",
    "enclaustran": "enclaustrar",
    "enclaustrando": "enclaustrar",
    "enclaustrara": "enclaustrar",
    "enclaustrarais": "enclaustrar",
    "enclaustraran": "enclaustrar",
    "enclaustraras": "enclaustrar",
    "enclaustrare": "enclaustrar",
    "enclaustrareis": "enclaustrar",
    "enclaustraremos": "enclaustrar",
    "enclaustraren": "enclaustrar",
    "enclaustrares": "enclaustrar",
    "enclaustraron": "enclaustrar",
    "enclaustrará": "enclaustrar",
    "enclaustrarán": "enclaustrar",
    "enclaustrarás": "enclaustrar",
    "enclaustraré": "enclaustrar",
    "enclaustraréis": "enclaustrar",
    "enclaustraría": "enclaustrar",
    "enclaustraríais": "enclaustrar",
    "enclaustraríamos": "enclaustrar",
    "enclaustrarían": "enclaustrar",
    "enclaustrarías": "enclaustrar",
    "enclaustras": "enclaustrar",
    "enclaustrase": "enclaustrar",
    "enclaustraseis": "enclaustrar",
    "enclaustrasen": "enclaustrar",
    "enclaustrases": "enclaustrar",
    "enclaustraste": "enclaustrar",
    "enclaustrasteis": "enclaustrar",
    "enclaustre": "enclaustrar",
    "enclaustremos": "enclaustrar",
    "enclaustren": "enclaustrar",
    "enclaustres": "enclaustrar",
    "enclaustro": "enclaustrar",
    "enclaustrábamos": "enclaustrar",
    "enclaustráis": "enclaustrar",
    "enclaustráramos": "enclaustrar",
    "enclaustráremos": "enclaustrar",
    "enclaustrásemos": "enclaustrar",
    "enclaustré": "enclaustrar",
    "enclaustréis": "enclaustrar",
    "enclaustró": "enclaustrar",
    "enclava": "enclavar",
    "enclavaba": "enclavar",
    "enclavabais": "enclavar",
    "enclavaban": "enclavar",
    "enclavabas": "enclavar",
    "enclavaciones": "enclavación",
    "enclavad": "enclavar",
    "enclavada": "enclavar",
    "enclavadas": "enclavar",
    "enclavado": "enclavar",
    "enclavados": "enclavar",
    "enclavaduras": "enclavadura",
    "enclavamientos": "enclavamiento",
    "enclavamos": "enclavar",
    "enclavan": "enclavar",
    "enclavando": "enclavar",
    "enclavara": "enclavar",
    "enclavarais": "enclavar",
    "enclavaran": "enclavar",
    "enclavaras": "enclavar",
    "enclavare": "enclavar",
    "enclavareis": "enclavar",
    "enclavaremos": "enclavar",
    "enclavaren": "enclavar",
    "enclavares": "enclavar",
    "enclavaron": "enclavar",
    "enclavará": "enclavar",
    "enclavarán": "enclavar",
    "enclavarás": "enclavar",
    "enclavaré": "enclavar",
    "enclavaréis": "enclavar",
    "enclavaría": "enclavar",
    "enclavaríais": "enclavar",
    "enclavaríamos": "enclavar",
    "enclavarían": "enclavar",
    "enclavarías": "enclavar",
    "enclavas": "enclavar",
    "enclavase": "enclavar",
    "enclavaseis": "enclavar",
    "enclavasen": "enclavar",
    "enclavases": "enclavar",
    "enclavaste": "enclavar",
    "enclavasteis": "enclavar",
    "enclave": "enclavar",
    "enclavemos": "enclavar",
    "enclaven": "enclavar",
    "enclaves": "enclave",
    "enclavija": "enclavijar",
    "enclavijaba": "enclavijar",
    "enclavijabais": "enclavijar",
    "enclavijaban": "enclavijar",
    "enclavijabas": "enclavijar",
    "enclavijad": "enclavijar",
    "enclavijada": "enclavijar",
    "enclavijadas": "enclavijar",
    "enclavijado": "enclavijar",
    "enclavijados": "enclavijar",
    "enclavijamientos": "enclavijamiento",
    "enclavijamos": "enclavijar",
    "enclavijan": "enclavijar",
    "enclavijando": "enclavijar",
    "enclavijara": "enclavijar",
    "enclavijarais": "enclavijar",
    "enclavijaran": "enclavijar",
    "enclavijaras": "enclavijar",
    "enclavijare": "enclavijar",
    "enclavijareis": "enclavijar",
    "enclavijaremos": "enclavijar",
    "enclavijaren": "enclavijar",
    "enclavijares": "enclavijar",
    "enclavijaron": "enclavijar",
    "enclavijará": "enclavijar",
    "enclavijarán": "enclavijar",
    "enclavijarás": "enclavijar",
    "enclavijaré": "enclavijar",
    "enclavijaréis": "enclavijar",
    "enclavijaría": "enclavijar",
    "enclavijaríais": "enclavijar",
    "enclavijaríamos": "enclavijar",
    "enclavijarían": "enclavijar",
    "enclavijarías": "enclavijar",
    "enclavijas": "enclavijar",
    "enclavijase": "enclavijar",
    "enclavijaseis": "enclavijar",
    "enclavijasen": "enclavijar",
    "enclavijases": "enclavijar",
    "enclavijaste": "enclavijar",
    "enclavijasteis": "enclavijar",
    "enclavije": "enclavijar",
    "enclavijemos": "enclavijar",
    "enclavijen": "enclavijar",
    "enclavijes": "enclavijar",
    "enclavijo": "enclavijar",
    "enclavijábamos": "enclavijar",
    "enclavijáis": "enclavijar",
    "enclavijáramos": "enclavijar",
    "enclavijáremos": "enclavijar",
    "enclavijásemos": "enclavijar",
    "enclavijé": "enclavijar",
    "enclavijéis": "enclavijar",
    "enclavijó": "enclavijar",
    "enclavo": "enclavar",
    "enclavábamos": "enclavar",
    "enclaváis": "enclavar",
    "enclaváramos": "enclavar",
    "enclaváremos": "enclavar",
    "enclavásemos": "enclavar",
    "enclavé": "enclavar",
    "enclavéis": "enclavar",
    "enclavó": "enclavar",
    "enclenques": "enclenque",
    "enclocaba": "enclocar",
    "enclocabais": "enclocar",
    "enclocaban": "enclocar",
    "enclocabas": "enclocar",
    "enclocad": "enclocar",
    "enclocada": "enclocar",
    "enclocadas": "enclocar",
    "enclocado": "enclocar",
    "enclocados": "enclocar",
    "enclocamos": "enclocar",
    "enclocando": "enclocar",
    "enclocara": "enclocar",
    "enclocarais": "enclocar",
    "enclocaran": "enclocar",
    "enclocaras": "enclocar",
    "enclocare": "enclocar",
    "enclocareis": "enclocar",
    "enclocaremos": "enclocar",
    "enclocaren": "enclocar",
    "enclocares": "enclocar",
    "enclocaron": "enclocar",
    "enclocará": "enclocar",
    "enclocarán": "enclocar",
    "enclocarás": "enclocar",
    "enclocaré": "enclocar",
    "enclocaréis": "enclocar",
    "enclocaría": "enclocar",
    "enclocaríais": "enclocar",
    "enclocaríamos": "enclocar",
    "enclocarían": "enclocar",
    "enclocarías": "enclocar",
    "enclocase": "enclocar",
    "enclocaseis": "enclocar",
    "enclocasen": "enclocar",
    "enclocases": "enclocar",
    "enclocaste": "enclocar",
    "enclocasteis": "enclocar",
    "enclocábamos": "enclocar",
    "enclocáis": "enclocar",
    "enclocáramos": "enclocar",
    "enclocáremos": "enclocar",
    "enclocásemos": "enclocar",
    "enclocó": "enclocar",
    "encloquece": "encloquecer",
    "encloqueced": "encloquecer",
    "encloquecemos": "encloquecer",
    "encloquecen": "encloquecer",
    "encloqueceremos": "encloquecer",
    "encloquecerá": "encloquecer",
    "encloquecerán": "encloquecer",
    "encloquecerás": "encloquecer",
    "encloqueceré": "encloquecer",
    "encloqueceréis": "encloquecer",
    "encloquecería": "encloquecer",
    "encloqueceríais": "encloquecer",
    "encloqueceríamos": "encloquecer",
    "encloquecerían": "encloquecer",
    "encloquecerías": "encloquecer",
    "encloqueces": "encloquecer",
    "encloquecida": "encloquecer",
    "encloquecidas": "encloquecer",
    "encloquecido": "encloquecer",
    "encloquecidos": "encloquecer",
    "encloqueciendo": "encloquecer",
    "encloqueciera": "encloquecer",
    "encloquecierais": "encloquecer",
    "encloquecieran": "encloquecer",
    "encloquecieras": "encloquecer",
    "encloqueciere": "encloquecer",
    "encloqueciereis": "encloquecer",
    "encloquecieren": "encloquecer",
    "encloquecieres": "encloquecer",
    "encloquecieron": "encloquecer",
    "encloqueciese": "encloquecer",
    "encloquecieseis": "encloquecer",
    "encloqueciesen": "encloquecer",
    "encloquecieses": "encloquecer",
    "encloquecimos": "encloquecer",
    "encloqueciste": "encloquecer",
    "encloquecisteis": "encloquecer",
    "encloqueciéramos": "encloquecer",
    "encloqueciéremos": "encloquecer",
    "encloqueciésemos": "encloquecer",
    "encloqueció": "encloquecer",
    "encloquecéis": "encloquecer",
    "encloquecí": "encloquecer",
    "encloquecía": "encloquecer",
    "encloquecíais": "encloquecer",
    "encloquecíamos": "encloquecer",
    "encloquecían": "encloquecer",
    "encloquecías": "encloquecer",
    "encloquemos": "enclocar",
    "encloquezca": "encloquecer",
    "encloquezcamos": "encloquecer",
    "encloquezcan": "encloquecer",
    "encloquezcas": "encloquecer",
    "encloquezco": "encloquecer",
    "encloquezcáis": "encloquecer",
    "encloqué": "enclocar",
    "encloquéis": "enclocar",
    "enclosures": "enclosure",
    "enclueca": "enclocar",
    "encluecan": "enclocar",
    "encluecas": "enclocar",
    "enclueco": "enclocar",
    "enclueque": "enclocar",
    "encluequen": "enclocar",
    "enclueques": "enclocar",
    "enclítica": "enclítico",
    "enclíticas": "enclítico",
    "enclíticos": "enclítico",
    "encoba": "encobar",
    "encobaba": "encobar",
    "encobabais": "encobar",
    "encobaban": "encobar",
    "encobabas": "encobar",
    "encobad": "encobar",
    "encobada": "encobar",
    "encobadas": "encobar",
    "encobado": "encobar",
    "encobados": "encobar",
    "encobamos": "encobar",
    "encoban": "encobar",
    "encobando": "encobar",
    "encobara": "encobar",
    "encobarais": "encobar",
    "encobaran": "encobar",
    "encobaras": "encobar",
    "encobare": "encobar",
    "encobareis": "encobar",
    "encobaremos": "encobar",
    "encobaren": "encobar",
    "encobares": "encobar",
    "encobaron": "encobar",
    "encobará": "encobar",
    "encobarán": "encobar",
    "encobarás": "encobar",
    "encobaré": "encobar",
    "encobaréis": "encobar",
    "encobaría": "encobar",
    "encobaríais": "encobar",
    "encobaríamos": "encobar",
    "encobarían": "encobar",
    "encobarías": "encobar",
    "encobas": "encobar",
    "encobase": "encobar",
    "encobaseis": "encobar",
    "encobasen": "encobar",
    "encobases": "encobar",
    "encobaste": "encobar",
    "encobasteis": "encobar",
    "encobe": "encobar",
    "encobemos": "encobar",
    "encoben": "encobar",
    "encobertada": "encobertado",
    "encobertadas": "encobertado",
    "encobertados": "encobertado",
    "encobes": "encobar",
    "encobo": "encobar",
    "encobra": "encobrar",
    "encobraba": "encobrar",
    "encobrabais": "encobrar",
    "encobraban": "encobrar",
    "encobrabas": "encobrar",
    "encobrad": "encobrar",
    "encobrada": "encobrar",
    "encobradas": "encobrar",
    "encobrado": "encobrar",
    "encobrados": "encobrar",
    "encobramos": "encobrar",
    "encobran": "encobrar",
    "encobrando": "encobrar",
    "encobrara": "encobrar",
    "encobrarais": "encobrar",
    "encobraran": "encobrar",
    "encobraras": "encobrar",
    "encobrare": "encobrar",
    "encobrareis": "encobrar",
    "encobraremos": "encobrar",
    "encobraren": "encobrar",
    "encobrares": "encobrar",
    "encobraron": "encobrar",
    "encobrará": "encobrar",
    "encobrarán": "encobrar",
    "encobrarás": "encobrar",
    "encobraré": "encobrar",
    "encobraréis": "encobrar",
    "encobraría": "encobrar",
    "encobraríais": "encobrar",
    "encobraríamos": "encobrar",
    "encobrarían": "encobrar",
    "encobrarías": "encobrar",
    "encobras": "encobrar",
    "encobrase": "encobrar",
    "encobraseis": "encobrar",
    "encobrasen": "encobrar",
    "encobrases": "encobrar",
    "encobraste": "encobrar",
    "encobrasteis": "encobrar",
    "encobre": "encobrar",
    "encobremos": "encobrar",
    "encobren": "encobrar",
    "encobres": "encobrar",
    "encobrizada": "encobrizado",
    "encobrizadas": "encobrizado",
    "encobrizados": "encobrizado",
    "encobro": "encobrar",
    "encobrábamos": "encobrar",
    "encobráis": "encobrar",
    "encobráramos": "encobrar",
    "encobráremos": "encobrar",
    "encobrásemos": "encobrar",
    "encobré": "encobrar",
    "encobréis": "encobrar",
    "encobró": "encobrar",
    "encobábamos": "encobar",
    "encobáis": "encobar",
    "encobáramos": "encobar",
    "encobáremos": "encobar",
    "encobásemos": "encobar",
    "encobé": "encobar",
    "encobéis": "encobar",
    "encobó": "encobar",
    "encochada": "encochado",
    "encochadas": "encochado",
    "encochados": "encochado",
    "encocora": "encocorar",
    "encocoraba": "encocorar",
    "encocorabais": "encocorar",
    "encocoraban": "encocorar",
    "encocorabas": "encocorar",
    "encocorad": "encocorar",
    "encocorada": "encocorar",
    "encocoradas": "encocorar",
    "encocorado": "encocorar",
    "encocorados": "encocorar",
    "encocoramos": "encocorar",
    "encocoran": "encocorar",
    "encocorando": "encocorar",
    "encocorara": "encocorar",
    "encocorarais": "encocorar",
    "encocoraran": "encocorar",
    "encocoraras": "encocorar",
    "encocorare": "encocorar",
    "encocorareis": "encocorar",
    "encocoraremos": "encocorar",
    "encocoraren": "encocorar",
    "encocorares": "encocorar",
    "encocoraron": "encocorar",
    "encocorará": "encocorar",
    "encocorarán": "encocorar",
    "encocorarás": "encocorar",
    "encocoraré": "encocorar",
    "encocoraréis": "encocorar",
    "encocoraría": "encocorar",
    "encocoraríais": "encocorar",
    "encocoraríamos": "encocorar",
    "encocorarían": "encocorar",
    "encocorarías": "encocorar",
    "encocoras": "encocorar",
    "encocorase": "encocorar",
    "encocoraseis": "encocorar",
    "encocorasen": "encocorar",
    "encocorases": "encocorar",
    "encocoraste": "encocorar",
    "encocorasteis": "encocorar",
    "encocore": "encocorar",
    "encocoremos": "encocorar",
    "encocoren": "encocorar",
    "encocores": "encocorar",
    "encocoro": "encocorar",
    "encocorábamos": "encocorar",
    "encocoráis": "encocorar",
    "encocoráramos": "encocorar",
    "encocoráremos": "encocorar",
    "encocorásemos": "encocorar",
    "encocoré": "encocorar",
    "encocoréis": "encocorar",
    "encocoró": "encocorar",
    "encofra": "encofrar",
    "encofraba": "encofrar",
    "encofrabais": "encofrar",
    "encofraban": "encofrar",
    "encofrabas": "encofrar",
    "encofrad": "encofrar",
    "encofrada": "encofrar",
    "encofradas": "encofrar",
    "encofrado": "encofrar",
    "encofrados": "encofrar",
    "encoframos": "encofrar",
    "encofran": "encofrar",
    "encofrando": "encofrar",
    "encofrara": "encofrar",
    "encofrarais": "encofrar",
    "encofraran": "encofrar",
    "encofraras": "encofrar",
    "encofrare": "encofrar",
    "encofrareis": "encofrar",
    "encofraremos": "encofrar",
    "encofraren": "encofrar",
    "encofrares": "encofrar",
    "encofraron": "encofrar",
    "encofrará": "encofrar",
    "encofrarán": "encofrar",
    "encofrarás": "encofrar",
    "encofraré": "encofrar",
    "encofraréis": "encofrar",
    "encofraría": "encofrar",
    "encofraríais": "encofrar",
    "encofraríamos": "encofrar",
    "encofrarían": "encofrar",
    "encofrarías": "encofrar",
    "encofras": "encofrar",
    "encofrase": "encofrar",
    "encofraseis": "encofrar",
    "encofrasen": "encofrar",
    "encofrases": "encofrar",
    "encofraste": "encofrar",
    "encofrasteis": "encofrar",
    "encofre": "encofrar",
    "encofremos": "encofrar",
    "encofren": "encofrar",
    "encofres": "encofrar",
    "encofro": "encofrar",
    "encofrábamos": "encofrar",
    "encofráis": "encofrar",
    "encofráramos": "encofrar",
    "encofráremos": "encofrar",
    "encofrásemos": "encofrar",
    "encofré": "encofrar",
    "encofréis": "encofrar",
    "encofró": "encofrar",
    "encoge": "encoger",
    "encoged": "encoger",
    "encogemos": "encoger",
    "encogen": "encoger",
    "encogeremos": "encoger",
    "encogerá": "encoger",
    "encogerán": "encoger",
    "encogerás": "encoger",
    "encogeré": "encoger",
    "encogeréis": "encoger",
    "encogería": "encoger",
    "encogeríais": "encoger",
    "encogeríamos": "encoger",
    "encogerían": "encoger",
    "encogerías": "encoger",
    "encoges": "encoger",
    "encogibles": "encogible",
    "encogida": "encoger",
    "encogidas": "encoger",
    "encogido": "encoger",
    "encogidos": "encoger",
    "encogiendo": "encoger",
    "encogiera": "encoger",
    "encogierais": "encoger",
    "encogieran": "encoger",
    "encogieras": "encoger",
    "encogiere": "encoger",
    "encogiereis": "encoger",
    "encogieren": "encoger",
    "encogieres": "encoger",
    "encogieron": "encoger",
    "encogiese": "encoger",
    "encogieseis": "encoger",
    "encogiesen": "encoger",
    "encogieses": "encoger",
    "encogimientos": "encogimiento",
    "encogimos": "encoger",
    "encogiste": "encoger",
    "encogisteis": "encoger",
    "encogiéramos": "encoger",
    "encogiéremos": "encoger",
    "encogiésemos": "encoger",
    "encogió": "encoger",
    "encogéis": "encoger",
    "encogí": "encoger",
    "encogía": "encoger",
    "encogíais": "encoger",
    "encogíamos": "encoger",
    "encogían": "encoger",
    "encogías": "encoger",
    "encoja": "encoger",
    "encojamos": "encoger",
    "encojan": "encoger",
    "encojas": "encoger",
    "encojo": "encoger",
    "encojáis": "encoger",
    "encola": "encolar",
    "encolaba": "encolar",
    "encolabais": "encolar",
    "encolaban": "encolar",
    "encolabas": "encolar",
    "encolad": "encolar",
    "encolada": "encolar",
    "encoladas": "encolar",
    "encolado": "encolar",
    "encoladora": "encolador",
    "encoladoras": "encolador",
    "encoladores": "encolador",
    "encolados": "encolar",
    "encoladuras": "encoladura",
    "encolamientos": "encolamiento",
    "encolamos": "encolar",
    "encolan": "encolar",
    "encolando": "encolar",
    "encolara": "encolar",
    "encolarais": "encolar",
    "encolaran": "encolar",
    "encolaras": "encolar",
    "encolare": "encolar",
    "encolareis": "encolar",
    "encolaremos": "encolar",
    "encolaren": "encolar",
    "encolares": "encolar",
    "encolaron": "encolar",
    "encolará": "encolar",
    "encolarán": "encolar",
    "encolarás": "encolar",
    "encolaré": "encolar",
    "encolaréis": "encolar",
    "encolaría": "encolar",
    "encolaríais": "encolar",
    "encolaríamos": "encolar",
    "encolarían": "encolar",
    "encolarías": "encolar",
    "encolas": "encolar",
    "encolase": "encolar",
    "encolaseis": "encolar",
    "encolasen": "encolar",
    "encolases": "encolar",
    "encolaste": "encolar",
    "encolasteis": "encolar",
    "encole": "encolar",
    "encolemos": "encolar",
    "encolen": "encolar",
    "encolerice": "encolerizar",
    "encolericemos": "encolerizar",
    "encolericen": "encolerizar",
    "encolerices": "encolerizar",
    "encolericé": "encolerizar",
    "encolericéis": "encolerizar",
    "encoleriza": "encolerizar",
    "encolerizaba": "encolerizar",
    "encolerizabais": "encolerizar",
    "encolerizaban": "encolerizar",
    "encolerizabas": "encolerizar",
    "encolerizad": "encolerizar",
    "encolerizada": "encolerizar",
    "encolerizadas": "encolerizar",
    "encolerizado": "encolerizar",
    "encolerizados": "encolerizar",
    "encolerizamientos": "encolerizamiento",
    "encolerizamos": "encolerizar",
    "encolerizan": "encolerizar",
    "encolerizando": "encolerizar",
    "encolerizara": "encolerizar",
    "encolerizarais": "encolerizar",
    "encolerizaran": "encolerizar",
    "encolerizaras": "encolerizar",
    "encolerizare": "encolerizar",
    "encolerizareis": "encolerizar",
    "encolerizaremos": "encolerizar",
    "encolerizaren": "encolerizar",
    "encolerizares": "encolerizar",
    "encolerizaron": "encolerizar",
    "encolerizará": "encolerizar",
    "encolerizarán": "encolerizar",
    "encolerizarás": "encolerizar",
    "encolerizaré": "encolerizar",
    "encolerizaréis": "encolerizar",
    "encolerizaría": "encolerizar",
    "encolerizaríais": "encolerizar",
    "encolerizaríamos": "encolerizar",
    "encolerizarían": "encolerizar",
    "encolerizarías": "encolerizar",
    "encolerizas": "encolerizar",
    "encolerizase": "encolerizar",
    "encolerizaseis": "encolerizar",
    "encolerizasen": "encolerizar",
    "encolerizases": "encolerizar",
    "encolerizaste": "encolerizar",
    "encolerizasteis": "encolerizar",
    "encolerizo": "encolerizar",
    "encolerizábamos": "encolerizar",
    "encolerizáis": "encolerizar",
    "encolerizáramos": "encolerizar",
    "encolerizáremos": "encolerizar",
    "encolerizásemos": "encolerizar",
    "encolerizó": "encolerizar",
    "encoles": "encolar",
    "encolo": "encolar",
    "encolábamos": "encolar",
    "encoláis": "encolar",
    "encoláramos": "encolar",
    "encoláremos": "encolar",
    "encolásemos": "encolar",
    "encolé": "encolar",
    "encoléis": "encolar",
    "encoló": "encolar",
    "encomendaba": "encomendar",
    "encomendabais": "encomendar",
    "encomendaban": "encomendar",
    "encomendabas": "encomendar",
    "encomendad": "encomendar",
    "encomendada": "encomendar",
    "encomendadas": "encomendar",
    "encomendado": "encomendar",
    "encomendados": "encomendar",
    "encomendamos": "encomendar",
    "encomendando": "encomendar",
    "encomendara": "encomendar",
    "encomendarais": "encomendar",
    "encomendaran": "encomendar",
    "encomendaras": "encomendar",
    "encomendare": "encomendar",
    "encomendareis": "encomendar",
    "encomendaremos": "encomendar",
    "encomendaren": "encomendar",
    "encomendares": "encomendar",
    "encomendaron": "encomendar",
    "encomendará": "encomendar",
    "encomendarán": "encomendar",
    "encomendarás": "encomendar",
    "encomendaré": "encomendar",
    "encomendaréis": "encomendar",
    "encomendaría": "encomendar",
    "encomendaríais": "encomendar",
    "encomendaríamos": "encomendar",
    "encomendarían": "encomendar",
    "encomendarías": "encomendar",
    "encomendase": "encomendar",
    "encomendaseis": "encomendar",
    "encomendasen": "encomendar",
    "encomendases": "encomendar",
    "encomendaste": "encomendar",
    "encomendasteis": "encomendar",
    "encomendemos": "encomendar",
    "encomenderos": "encomendero",
    "encomendábamos": "encomendar",
    "encomendáis": "encomendar",
    "encomendáramos": "encomendar",
    "encomendáremos": "encomendar",
    "encomendásemos": "encomendar",
    "encomendé": "encomendar",
    "encomendéis": "encomendar",
    "encomendó": "encomendar",
    "encomia": "encomiar",
    "encomiaba": "encomiar",
    "encomiabais": "encomiar",
    "encomiaban": "encomiar",
    "encomiabas": "encomiar",
    "encomiables": "encomiable",
    "encomiad": "encomiar",
    "encomiada": "encomiar",
    "encomiadas": "encomiar",
    "encomiado": "encomiar",
    "encomiadora": "encomiador",
    "encomiadoras": "encomiador",
    "encomiadores": "encomiador",
    "encomiados": "encomiar",
    "encomiamos": "encomiar",
    "encomian": "encomiar",
    "encomiando": "encomiar",
    "encomiara": "encomiar",
    "encomiarais": "encomiar",
    "encomiaran": "encomiar",
    "encomiaras": "encomiar",
    "encomiare": "encomiar",
    "encomiareis": "encomiar",
    "encomiaremos": "encomiar",
    "encomiaren": "encomiar",
    "encomiares": "encomiar",
    "encomiaron": "encomiar",
    "encomiará": "encomiar",
    "encomiarán": "encomiar",
    "encomiarás": "encomiar",
    "encomiaré": "encomiar",
    "encomiaréis": "encomiar",
    "encomiaría": "encomiar",
    "encomiaríais": "encomiar",
    "encomiaríamos": "encomiar",
    "encomiarían": "encomiar",
    "encomiarías": "encomiar",
    "encomias": "encomiar",
    "encomiase": "encomiar",
    "encomiaseis": "encomiar",
    "encomiasen": "encomiar",
    "encomiases": "encomiar",
    "encomiastas": "encomiasta",
    "encomiaste": "encomiar",
    "encomiasteis": "encomiar",
    "encomie": "encomiar",
    "encomiemos": "encomiar",
    "encomien": "encomiar",
    "encomienda": "encomendar",
    "encomiendan": "encomendar",
    "encomiendas": "encomienda",
    "encomiende": "encomendar",
    "encomienden": "encomendar",
    "encomiendes": "encomendar",
    "encomiendo": "encomendar",
    "encomies": "encomiar",
    "encomio": "encomiar",
    "encomios": "encomio",
    "encomiábamos": "encomiar",
    "encomiáis": "encomiar",
    "encomiáramos": "encomiar",
    "encomiáremos": "encomiar",
    "encomiásemos": "encomiar",
    "encomiástica": "encomiástico",
    "encomiásticas": "encomiástico",
    "encomiásticos": "encomiástico",
    "encomié": "encomiar",
    "encomiéis": "encomiar",
    "encomió": "encomiar",
    "encompadra": "encompadrar",
    "encompadraba": "encompadrar",
    "encompadrabais": "encompadrar",
    "encompadraban": "encompadrar",
    "encompadrabas": "encompadrar",
    "encompadrad": "encompadrar",
    "encompadrada": "encompadrar",
    "encompadradas": "encompadrar",
    "encompadrado": "encompadrar",
    "encompadrados": "encompadrar",
    "encompadramos": "encompadrar",
    "encompadran": "encompadrar",
    "encompadrando": "encompadrar",
    "encompadrara": "encompadrar",
    "encompadrarais": "encompadrar",
    "encompadraran": "encompadrar",
    "encompadraras": "encompadrar",
    "encompadrare": "encompadrar",
    "encompadrareis": "encompadrar",
    "encompadraremos": "encompadrar",
    "encompadraren": "encompadrar",
    "encompadrares": "encompadrar",
    "encompadraron": "encompadrar",
    "encompadrará": "encompadrar",
    "encompadrarán": "encompadrar",
    "encompadrarás": "encompadrar",
    "encompadraré": "encompadrar",
    "encompadraréis": "encompadrar",
    "encompadraría": "encompadrar",
    "encompadraríais": "encompadrar",
    "encompadraríamos": "encompadrar",
    "encompadrarían": "encompadrar",
    "encompadrarías": "encompadrar",
    "encompadras": "encompadrar",
    "encompadrase": "encompadrar",
    "encompadraseis": "encompadrar",
    "encompadrasen": "encompadrar",
    "encompadrases": "encompadrar",
    "encompadraste": "encompadrar",
    "encompadrasteis": "encompadrar",
    "encompadre": "encompadrar",
    "encompadremos": "encompadrar",
    "encompadren": "encompadrar",
    "encompadres": "encompadrar",
    "encompadro": "encompadrar",
    "encompadrábamos": "encompadrar",
    "encompadráis": "encompadrar",
    "encompadráramos": "encompadrar",
    "encompadráremos": "encompadrar",
    "encompadrásemos": "encompadrar",
    "encompadré": "encompadrar",
    "encompadréis": "encompadrar",
    "encompadró": "encompadrar",
    "encona": "enconar",
    "enconaba": "enconar",
    "enconabais": "enconar",
    "enconaban": "enconar",
    "enconabas": "enconar",
    "enconad": "enconar",
    "enconada": "enconar",
    "enconadas": "enconar",
    "enconado": "enconar",
    "enconados": "enconar",
    "enconamientos": "enconamiento",
    "enconamos": "enconar",
    "enconan": "enconar",
    "enconando": "enconar",
    "enconara": "enconar",
    "enconarais": "enconar",
    "enconaran": "enconar",
    "enconaras": "enconar",
    "enconare": "enconar",
    "enconareis": "enconar",
    "enconaremos": "enconar",
    "enconaren": "enconar",
    "enconares": "enconar",
    "enconaron": "enconar",
    "enconará": "enconar",
    "enconarán": "enconar",
    "enconarás": "enconar",
    "enconaré": "enconar",
    "enconaréis": "enconar",
    "enconaría": "enconar",
    "enconaríais": "enconar",
    "enconaríamos": "enconar",
    "enconarían": "enconar",
    "enconarías": "enconar",
    "enconas": "enconar",
    "enconase": "enconar",
    "enconaseis": "enconar",
    "enconasen": "enconar",
    "enconases": "enconar",
    "enconaste": "enconar",
    "enconasteis": "enconar",
    "encone": "enconar",
    "enconemos": "enconar",
    "enconen": "enconar",
    "encones": "enconar",
    "encono": "enconar",
    "enconos": "encono",
    "enconosa": "enconoso",
    "enconosas": "enconoso",
    "enconosos": "enconoso",
    "encontraba": "encontrar",
    "encontrabais": "encontrar",
    "encontraban": "encontrar",
    "encontrabas": "encontrar",
    "encontrad": "encontrar",
    "encontrada": "encontrar",
    "encontradas": "encontrar",
    "encontradiza": "encontradizo",
    "encontradizas": "encontradizo",
    "encontradizos": "encontradizo",
    "encontrado": "encontrar",
    "encontrados": "encontrar",
    "encontramos": "encontrar",
    "encontrando": "encontrar",
    "encontrara": "encontrar",
    "encontrarais": "encontrar",
    "encontraran": "encontrar",
    "encontraras": "encontrar",
    "encontrare": "encontrar",
    "encontrareis": "encontrar",
    "encontraremos": "encontrar",
    "encontraren": "encontrar",
    "encontrares": "encontrar",
    "encontraron": "encontrar",
    "encontrará": "encontrar",
    "encontrarán": "encontrar",
    "encontrarás": "encontrar",
    "encontraré": "encontrar",
    "encontraréis": "encontrar",
    "encontraría": "encontrar",
    "encontraríais": "encontrar",
    "encontraríamos": "encontrar",
    "encontrarían": "encontrar",
    "encontrarías": "encontrar",
    "encontrase": "encontrar",
    "encontraseis": "encontrar",
    "encontrasen": "encontrar",
    "encontrases": "encontrar",
    "encontraste": "encontrar",
    "encontrasteis": "encontrar",
    "encontremos": "encontrar",
    "encontronazos": "encontronazo",
    "encontrones": "encontrón",
    "encontrábamos": "encontrar",
    "encontráis": "encontrar",
    "encontráramos": "encontrar",
    "encontráremos": "encontrar",
    "encontrásemos": "encontrar",
    "encontré": "encontrar",
    "encontréis": "encontrar",
    "encontró": "encontrar",
    "enconábamos": "enconar",
    "enconáis": "enconar",
    "enconáramos": "enconar",
    "enconáremos": "enconar",
    "enconásemos": "enconar",
    "enconé": "enconar",
    "enconéis": "enconar",
    "enconó": "enconar",
    "encopeta": "encopetar",
    "encopetaba": "encopetar",
    "encopetabais": "encopetar",
    "encopetaban": "encopetar",
    "encopetabas": "encopetar",
    "encopetad": "encopetar",
    "encopetada": "encopetar",
    "encopetadas": "encopetar",
    "encopetado": "encopetar",
    "encopetados": "encopetar",
    "encopetamientos": "encopetamiento",
    "encopetamos": "encopetar",
    "encopetan": "encopetar",
    "encopetando": "encopetar",
    "encopetara": "encopetar",
    "encopetarais": "encopetar",
    "encopetaran": "encopetar",
    "encopetaras": "encopetar",
    "encopetare": "encopetar",
    "encopetareis": "encopetar",
    "encopetaremos": "encopetar",
    "encopetaren": "encopetar",
    "encopetares": "encopetar",
    "encopetaron": "encopetar",
    "encopetará": "encopetar",
    "encopetarán": "encopetar",
    "encopetarás": "encopetar",
    "encopetaré": "encopetar",
    "encopetaréis": "encopetar",
    "encopetaría": "encopetar",
    "encopetaríais": "encopetar",
    "encopetaríamos": "encopetar",
    "encopetarían": "encopetar",
    "encopetarías": "encopetar",
    "encopetas": "encopetar",
    "encopetase": "encopetar",
    "encopetaseis": "encopetar",
    "encopetasen": "encopetar",
    "encopetases": "encopetar",
    "encopetaste": "encopetar",
    "encopetasteis": "encopetar",
    "encopete": "encopetar",
    "encopetemos": "encopetar",
    "encopeten": "encopetar",
    "encopetes": "encopetar",
    "encopeto": "encopetar",
    "encopetábamos": "encopetar",
    "encopetáis": "encopetar",
    "encopetáramos": "encopetar",
    "encopetáremos": "encopetar",
    "encopetásemos": "encopetar",
    "encopeté": "encopetar",
    "encopetéis": "encopetar",
    "encopetó": "encopetar",
    "encoraba": "encorar",
    "encorabais": "encorar",
    "encoraban": "encorar",
    "encorabas": "encorar",
    "encorad": "encorar",
    "encorada": "encorar",
    "encoradas": "encorar",
    "encorado": "encorar",
    "encorados": "encorar",
    "encoraja": "encorajar",
    "encorajaba": "encorajar",
    "encorajabais": "encorajar",
    "encorajaban": "encorajar",
    "encorajabas": "encorajar",
    "encorajad": "encorajar",
    "encorajada": "encorajar",
    "encorajadas": "encorajar",
    "encorajado": "encorajar",
    "encorajados": "encorajar",
    "encorajamos": "encorajar",
    "encorajan": "encorajar",
    "encorajando": "encorajar",
    "encorajara": "encorajar",
    "encorajarais": "encorajar",
    "encorajaran": "encorajar",
    "encorajaras": "encorajar",
    "encorajare": "encorajar",
    "encorajareis": "encorajar",
    "encorajaremos": "encorajar",
    "encorajaren": "encorajar",
    "encorajares": "encorajar",
    "encorajaron": "encorajar",
    "encorajará": "encorajar",
    "encorajarán": "encorajar",
    "encorajarás": "encorajar",
    "encorajaré": "encorajar",
    "encorajaréis": "encorajar",
    "encorajaría": "encorajar",
    "encorajaríais": "encorajar",
    "encorajaríamos": "encorajar",
    "encorajarían": "encorajar",
    "encorajarías": "encorajar",
    "encorajas": "encorajar",
    "encorajase": "encorajar",
    "encorajaseis": "encorajar",
    "encorajasen": "encorajar",
    "encorajases": "encorajar",
    "encorajaste": "encorajar",
    "encorajasteis": "encorajar",
    "encoraje": "encorajar",
    "encorajemos": "encorajar",
    "encorajen": "encorajar",
    "encorajes": "encorajar",
    "encorajina": "encorajinar",
    "encorajinaba": "encorajinar",
    "encorajinabais": "encorajinar",
    "encorajinaban": "encorajinar",
    "encorajinabas": "encorajinar",
    "encorajinad": "encorajinar",
    "encorajinada": "encorajinar",
    "encorajinadas": "encorajinar",
    "encorajinado": "encorajinar",
    "encorajinados": "encorajinar",
    "encorajinamos": "encorajinar",
    "encorajinan": "encorajinar",
    "encorajinando": "encorajinar",
    "encorajinara": "encorajinar",
    "encorajinarais": "encorajinar",
    "encorajinaran": "encorajinar",
    "encorajinaras": "encorajinar",
    "encorajinare": "encorajinar",
    "encorajinareis": "encorajinar",
    "encorajinaremos": "encorajinar",
    "encorajinaren": "encorajinar",
    "encorajinares": "encorajinar",
    "encorajinaron": "encorajinar",
    "encorajinará": "encorajinar",
    "encorajinarán": "encorajinar",
    "encorajinarás": "encorajinar",
    "encorajinaré": "encorajinar",
    "encorajinaréis": "encorajinar",
    "encorajinaría": "encorajinar",
    "encorajinaríais": "encorajinar",
    "encorajinaríamos": "encorajinar",
    "encorajinarían": "encorajinar",
    "encorajinarías": "encorajinar",
    "encorajinas": "encorajinar",
    "encorajinase": "encorajinar",
    "encorajinaseis": "encorajinar",
    "encorajinasen": "encorajinar",
    "encorajinases": "encorajinar",
    "encorajinaste": "encorajinar",
    "encorajinasteis": "encorajinar",
    "encorajine": "encorajinar",
    "encorajinemos": "encorajinar",
    "encorajinen": "encorajinar",
    "encorajines": "encorajinar",
    "encorajino": "encorajinar",
    "encorajinábamos": "encorajinar",
    "encorajináis": "encorajinar",
    "encorajináramos": "encorajinar",
    "encorajináremos": "encorajinar",
    "encorajinásemos": "encorajinar",
    "encorajiné": "encorajinar",
    "encorajinéis": "encorajinar",
    "encorajinó": "encorajinar",
    "encorajo": "encorajar",
    "encorajábamos": "encorajar",
    "encorajáis": "encorajar",
    "encorajáramos": "encorajar",
    "encorajáremos": "encorajar",
    "encorajásemos": "encorajar",
    "encorajé": "encorajar",
    "encorajéis": "encorajar",
    "encorajó": "encorajar",
    "encoramos": "encorar",
    "encorando": "encorar",
    "encorara": "encorar",
    "encorarais": "encorar",
    "encoraran": "encorar",
    "encoraras": "encorar",
    "encorare": "encorar",
    "encorareis": "encorar",
    "encoraremos": "encorar",
    "encoraren": "encorar",
    "encorares": "encorar",
    "encoraron": "encorar",
    "encorará": "encorar",
    "encorarán": "encorar",
    "encorarás": "encorar",
    "encoraré": "encorar",
    "encoraréis": "encorar",
    "encoraría": "encorar",
    "encoraríais": "encorar",
    "encoraríamos": "encorar",
    "encorarían": "encorar",
    "encorarías": "encorar",
    "encorase": "encorar",
    "encoraseis": "encorar",
    "encorasen": "encorar",
    "encorases": "encorar",
    "encoraste": "encorar",
    "encorasteis": "encorar",
    "encorazada": "encorazado",
    "encorazadas": "encorazado",
    "encorazados": "encorazado",
    "encorcha": "encorchar",
    "encorchaba": "encorchar",
    "encorchabais": "encorchar",
    "encorchaban": "encorchar",
    "encorchabas": "encorchar",
    "encorchad": "encorchar",
    "encorchada": "encorchar",
    "encorchadas": "encorchar",
    "encorchado": "encorchar",
    "encorchadora": "encorchador",
    "encorchadoras": "encorchador",
    "encorchadores": "encorchador",
    "encorchados": "encorchar",
    "encorchamos": "encorchar",
    "encorchan": "encorchar",
    "encorchando": "encorchar",
    "encorchara": "encorchar",
    "encorcharais": "encorchar",
    "encorcharan": "encorchar",
    "encorcharas": "encorchar",
    "encorchare": "encorchar",
    "encorchareis": "encorchar",
    "encorcharemos": "encorchar",
    "encorcharen": "encorchar",
    "encorchares": "encorchar",
    "encorcharon": "encorchar",
    "encorchará": "encorchar",
    "encorcharán": "encorchar",
    "encorcharás": "encorchar",
    "encorcharé": "encorchar",
    "encorcharéis": "encorchar",
    "encorcharía": "encorchar",
    "encorcharíais": "encorchar",
    "encorcharíamos": "encorchar",
    "encorcharían": "encorchar",
    "encorcharías": "encorchar",
    "encorchas": "encorchar",
    "encorchase": "encorchar",
    "encorchaseis": "encorchar",
    "encorchasen": "encorchar",
    "encorchases": "encorchar",
    "encorchaste": "encorchar",
    "encorchasteis": "encorchar",
    "encorche": "encorchar",
    "encorchemos": "encorchar",
    "encorchen": "encorchar",
    "encorches": "encorchar",
    "encorcheta": "encorchetar",
    "encorchetaba": "encorchetar",
    "encorchetabais": "encorchetar",
    "encorchetaban": "encorchetar",
    "encorchetabas": "encorchetar",
    "encorchetad": "encorchetar",
    "encorchetada": "encorchetar",
    "encorchetadas": "encorchetar",
    "encorchetado": "encorchetar",
    "encorchetados": "encorchetar",
    "encorchetamos": "encorchetar",
    "encorchetan": "encorchetar",
    "encorchetando": "encorchetar",
    "encorchetara": "encorchetar",
    "encorchetarais": "encorchetar",
    "encorchetaran": "encorchetar",
    "encorchetaras": "encorchetar",
    "encorchetare": "encorchetar",
    "encorchetareis": "encorchetar",
    "encorchetaremos": "encorchetar",
    "encorchetaren": "encorchetar",
    "encorchetares": "encorchetar",
    "encorchetaron": "encorchetar",
    "encorchetará": "encorchetar",
    "encorchetarán": "encorchetar",
    "encorchetarás": "encorchetar",
    "encorchetaré": "encorchetar",
    "encorchetaréis": "encorchetar",
    "encorchetaría": "encorchetar",
    "encorchetaríais": "encorchetar",
    "encorchetaríamos": "encorchetar",
    "encorchetarían": "encorchetar",
    "encorchetarías": "encorchetar",
    "encorchetas": "encorchetar",
    "encorchetase": "encorchetar",
    "encorchetaseis": "encorchetar",
    "encorchetasen": "encorchetar",
    "encorchetases": "encorchetar",
    "encorchetaste": "encorchetar",
    "encorchetasteis": "encorchetar",
    "encorchete": "encorchetar",
    "encorchetemos": "encorchetar",
    "encorcheten": "encorchetar",
    "encorchetes": "encorchetar",
    "encorcheto": "encorchetar",
    "encorchetábamos": "encorchetar",
    "encorchetáis": "encorchetar",
    "encorchetáramos": "encorchetar",
    "encorchetáremos": "encorchetar",
    "encorchetásemos": "encorchetar",
    "encorcheté": "encorchetar",
    "encorchetéis": "encorchetar",
    "encorchetó": "encorchetar",
    "encorcho": "encorchar",
    "encorchábamos": "encorchar",
    "encorcháis": "encorchar",
    "encorcháramos": "encorchar",
    "encorcháremos": "encorchar",
    "encorchásemos": "encorchar",
    "encorché": "encorchar",
    "encorchéis": "encorchar",
    "encorchó": "encorchar",
    "encordaba": "encordar",
    "encordabais": "encordar",
    "encordaban": "encordar",
    "encordabas": "encordar",
    "encordad": "encordar",
    "encordada": "encordar",
    "encordadas": "encordar",
    "encordado": "encordar",
    "encordados": "encordar",
    "encordaduras": "encordadura",
    "encordamos": "encordar",
    "encordando": "encordar",
    "encordara": "encordar",
    "encordarais": "encordar",
    "encordaran": "encordar",
    "encordaras": "encordar",
    "encordare": "encordar",
    "encordareis": "encordar",
    "encordaremos": "encordar",
    "encordaren": "encordar",
    "encordares": "encordar",
    "encordaron": "encordar",
    "encordará": "encordar",
    "encordarán": "encordar",
    "encordarás": "encordar",
    "encordaré": "encordar",
    "encordaréis": "encordar",
    "encordaría": "encordar",
    "encordaríais": "encordar",
    "encordaríamos": "encordar",
    "encordarían": "encordar",
    "encordarías": "encordar",
    "encordase": "encordar",
    "encordaseis": "encordar",
    "encordasen": "encordar",
    "encordases": "encordar",
    "encordaste": "encordar",
    "encordasteis": "encordar",
    "encordela": "encordelar",
    "encordelaba": "encordelar",
    "encordelabais": "encordelar",
    "encordelaban": "encordelar",
    "encordelabas": "encordelar",
    "encordelad": "encordelar",
    "encordelada": "encordelar",
    "encordeladas": "encordelar",
    "encordelado": "encordelar",
    "encordelados": "encordelar",
    "encordelamos": "encordelar",
    "encordelan": "encordelar",
    "encordelando": "encordelar",
    "encordelara": "encordelar",
    "encordelarais": "encordelar",
    "encordelaran": "encordelar",
    "encordelaras": "encordelar",
    "encordelare": "encordelar",
    "encordelareis": "encordelar",
    "encordelaremos": "encordelar",
    "encordelaren": "encordelar",
    "encordelares": "encordelar",
    "encordelaron": "encordelar",
    "encordelará": "encordelar",
    "encordelarán": "encordelar",
    "encordelarás": "encordelar",
    "encordelaré": "encordelar",
    "encordelaréis": "encordelar",
    "encordelaría": "encordelar",
    "encordelaríais": "encordelar",
    "encordelaríamos": "encordelar",
    "encordelarían": "encordelar",
    "encordelarías": "encordelar",
    "encordelas": "encordelar",
    "encordelase": "encordelar",
    "encordelaseis": "encordelar",
    "encordelasen": "encordelar",
    "encordelases": "encordelar",
    "encordelaste": "encordelar",
    "encordelasteis": "encordelar",
    "encordele": "encordelar",
    "encordelemos": "encordelar",
    "encordelen": "encordelar",
    "encordeles": "encordelar",
    "encordelo": "encordelar",
    "encordelábamos": "encordelar",
    "encordeláis": "encordelar",
    "encordeláramos": "encordelar",
    "encordeláremos": "encordelar",
    "encordelásemos": "encordelar",
    "encordelé": "encordelar",
    "encordeléis": "encordelar",
    "encordeló": "encordelar",
    "encordemos": "encordar",
    "encordona": "encordonar",
    "encordonaba": "encordonar",
    "encordonabais": "encordonar",
    "encordonaban": "encordonar",
    "encordonabas": "encordonar",
    "encordonad": "encordonar",
    "encordonada": "encordonar",
    "encordonadas": "encordonar",
    "encordonado": "encordonar",
    "encordonados": "encordonar",
    "encordonamos": "encordonar",
    "encordonan": "encordonar",
    "encordonando": "encordonar",
    "encordonara": "encordonar",
    "encordonarais": "encordonar",
    "encordonaran": "encordonar",
    "encordonaras": "encordonar",
    "encordonare": "encordonar",
    "encordonareis": "encordonar",
    "encordonaremos": "encordonar",
    "encordonaren": "encordonar",
    "encordonares": "encordonar",
    "encordonaron": "encordonar",
    "encordonará": "encordonar",
    "encordonarán": "encordonar",
    "encordonarás": "encordonar",
    "encordonaré": "encordonar",
    "encordonaréis": "encordonar",
    "encordonaría": "encordonar",
    "encordonaríais": "encordonar",
    "encordonaríamos": "encordonar",
    "encordonarían": "encordonar",
    "encordonarías": "encordonar",
    "encordonas": "encordonar",
    "encordonase": "encordonar",
    "encordonaseis": "encordonar",
    "encordonasen": "encordonar",
    "encordonases": "encordonar",
    "encordonaste": "encordonar",
    "encordonasteis": "encordonar",
    "encordone": "encordonar",
    "encordonemos": "encordonar",
    "encordonen": "encordonar",
    "encordones": "encordonar",
    "encordono": "encordonar",
    "encordonábamos": "encordonar",
    "encordonáis": "encordonar",
    "encordonáramos": "encordonar",
    "encordonáremos": "encordonar",
    "encordonásemos": "encordonar",
    "encordoné": "encordonar",
    "encordonéis": "encordonar",
    "encordonó": "encordonar",
    "encordábamos": "encordar",
    "encordáis": "encordar",
    "encordáramos": "encordar",
    "encordáremos": "encordar",
    "encordásemos": "encordar",
    "encordé": "encordar",
    "encordéis": "encordar",
    "encordó": "encordar",
    "encoremos": "encorar",
    "encoriaciones": "encoriación",
    "encornada": "encornado",
    "encornadas": "encornado",
    "encornados": "encornado",
    "encornaduras": "encornadura",
    "encoroce": "encorozar",
    "encorocemos": "encorozar",
    "encorocen": "encorozar",
    "encoroces": "encorozar",
    "encorocé": "encorozar",
    "encorocéis": "encorozar",
    "encoroza": "encorozar",
    "encorozaba": "encorozar",
    "encorozabais": "encorozar",
    "encorozaban": "encorozar",
    "encorozabas": "encorozar",
    "encorozad": "encorozar",
    "encorozada": "encorozar",
    "encorozadas": "encorozar",
    "encorozado": "encorozar",
    "encorozados": "encorozar",
    "encorozamos": "encorozar",
    "encorozan": "encorozar",
    "encorozando": "encorozar",
    "encorozara": "encorozar",
    "encorozarais": "encorozar",
    "encorozaran": "encorozar",
    "encorozaras": "encorozar",
    "encorozare": "encorozar",
    "encorozareis": "encorozar",
    "encorozaremos": "encorozar",
    "encorozaren": "encorozar",
    "encorozares": "encorozar",
    "encorozaron": "encorozar",
    "encorozará": "encorozar",
    "encorozarán": "encorozar",
    "encorozarás": "encorozar",
    "encorozaré": "encorozar",
    "encorozaréis": "encorozar",
    "encorozaría": "encorozar",
    "encorozaríais": "encorozar",
    "encorozaríamos": "encorozar",
    "encorozarían": "encorozar",
    "encorozarías": "encorozar",
    "encorozas": "encorozar",
    "encorozase": "encorozar",
    "encorozaseis": "encorozar",
    "encorozasen": "encorozar",
    "encorozases": "encorozar",
    "encorozaste": "encorozar",
    "encorozasteis": "encorozar",
    "encorozo": "encorozar",
    "encorozábamos": "encorozar",
    "encorozáis": "encorozar",
    "encorozáramos": "encorozar",
    "encorozáremos": "encorozar",
    "encorozásemos": "encorozar",
    "encorozó": "encorozar",
    "encorrala": "encorralar",
    "encorralaba": "encorralar",
    "encorralabais": "encorralar",
    "encorralaban": "encorralar",
    "encorralabas": "encorralar",
    "encorralad": "encorralar",
    "encorralada": "encorralar",
    "encorraladas": "encorralar",
    "encorralado": "encorralar",
    "encorralados": "encorralar",
    "encorralamos": "encorralar",
    "encorralan": "encorralar",
    "encorralando": "encorralar",
    "encorralara": "encorralar",
    "encorralarais": "encorralar",
    "encorralaran": "encorralar",
    "encorralaras": "encorralar",
    "encorralare": "encorralar",
    "encorralareis": "encorralar",
    "encorralaremos": "encorralar",
    "encorralaren": "encorralar",
    "encorralares": "encorralar",
    "encorralaron": "encorralar",
    "encorralará": "encorralar",
    "encorralarán": "encorralar",
    "encorralarás": "encorralar",
    "encorralaré": "encorralar",
    "encorralaréis": "encorralar",
    "encorralaría": "encorralar",
    "encorralaríais": "encorralar",
    "encorralaríamos": "encorralar",
    "encorralarían": "encorralar",
    "encorralarías": "encorralar",
    "encorralas": "encorralar",
    "encorralase": "encorralar",
    "encorralaseis": "encorralar",
    "encorralasen": "encorralar",
    "encorralases": "encorralar",
    "encorralaste": "encorralar",
    "encorralasteis": "encorralar",
    "encorrale": "encorralar",
    "encorralemos": "encorralar",
    "encorralen": "encorralar",
    "encorrales": "encorralar",
    "encorralo": "encorralar",
    "encorralábamos": "encorralar",
    "encorraláis": "encorralar",
    "encorraláramos": "encorralar",
    "encorraláremos": "encorralar",
    "encorralásemos": "encorralar",
    "encorralé": "encorralar",
    "encorraléis": "encorralar",
    "encorraló": "encorralar",
    "encorseta": "encorsetar",
    "encorsetaba": "encorsetar",
    "encorsetabais": "encorsetar",
    "encorsetaban": "encorsetar",
    "encorsetabas": "encorsetar",
    "encorsetad": "encorsetar",
    "encorsetada": "encorsetar",
    "encorsetadas": "encorsetar",
    "encorsetado": "encorsetar",
    "encorsetados": "encorsetar",
    "encorsetamientos": "encorsetamiento",
    "encorsetamos": "encorsetar",
    "encorsetan": "encorsetar",
    "encorsetando": "encorsetar",
    "encorsetara": "encorsetar",
    "encorsetarais": "encorsetar",
    "encorsetaran": "encorsetar",
    "encorsetaras": "encorsetar",
    "encorsetare": "encorsetar",
    "encorsetareis": "encorsetar",
    "encorsetaremos": "encorsetar",
    "encorsetaren": "encorsetar",
    "encorsetares": "encorsetar",
    "encorsetaron": "encorsetar",
    "encorsetará": "encorsetar",
    "encorsetarán": "encorsetar",
    "encorsetarás": "encorsetar",
    "encorsetaré": "encorsetar",
    "encorsetaréis": "encorsetar",
    "encorsetaría": "encorsetar",
    "encorsetaríais": "encorsetar",
    "encorsetaríamos": "encorsetar",
    "encorsetarían": "encorsetar",
    "encorsetarías": "encorsetar",
    "encorsetas": "encorsetar",
    "encorsetase": "encorsetar",
    "encorsetaseis": "encorsetar",
    "encorsetasen": "encorsetar",
    "encorsetases": "encorsetar",
    "encorsetaste": "encorsetar",
    "encorsetasteis": "encorsetar",
    "encorsete": "encorsetar",
    "encorsetemos": "encorsetar",
    "encorseten": "encorsetar",
    "encorsetes": "encorsetar",
    "encorseto": "encorsetar",
    "encorsetábamos": "encorsetar",
    "encorsetáis": "encorsetar",
    "encorsetáramos": "encorsetar",
    "encorsetáremos": "encorsetar",
    "encorsetásemos": "encorsetar",
    "encorseté": "encorsetar",
    "encorsetéis": "encorsetar",
    "encorsetó": "encorsetar",
    "encorva": "encorvar",
    "encorvaba": "encorvar",
    "encorvabais": "encorvar",
    "encorvaban": "encorvar",
    "encorvabas": "encorvar",
    "encorvad": "encorvar",
    "encorvada": "encorvar",
    "encorvadas": "encorvar",
    "encorvado": "encorvar",
    "encorvados": "encorvar",
    "encorvaduras": "encorvadura",
    "encorvamientos": "encorvamiento",
    "encorvamos": "encorvar",
    "encorvan": "encorvar",
    "encorvando": "encorvar",
    "encorvara": "encorvar",
    "encorvarais": "encorvar",
    "encorvaran": "encorvar",
    "encorvaras": "encorvar",
    "encorvare": "encorvar",
    "encorvareis": "encorvar",
    "encorvaremos": "encorvar",
    "encorvaren": "encorvar",
    "encorvares": "encorvar",
    "encorvaron": "encorvar",
    "encorvará": "encorvar",
    "encorvarán": "encorvar",
    "encorvarás": "encorvar",
    "encorvaré": "encorvar",
    "encorvaréis": "encorvar",
    "encorvaría": "encorvar",
    "encorvaríais": "encorvar",
    "encorvaríamos": "encorvar",
    "encorvarían": "encorvar",
    "encorvarías": "encorvar",
    "encorvas": "encorvar",
    "encorvase": "encorvar",
    "encorvaseis": "encorvar",
    "encorvasen": "encorvar",
    "encorvases": "encorvar",
    "encorvaste": "encorvar",
    "encorvasteis": "encorvar",
    "encorve": "encorvar",
    "encorvemos": "encorvar",
    "encorven": "encorvar",
    "encorves": "encorvar",
    "encorvo": "encorvar",
    "encorvábamos": "encorvar",
    "encorváis": "encorvar",
    "encorváramos": "encorvar",
    "encorváremos": "encorvar",
    "encorvásemos": "encorvar",
    "encorvé": "encorvar",
    "encorvéis": "encorvar",
    "encorvó": "encorvar",
    "encorábamos": "encorar",
    "encoráis": "encorar",
    "encoráramos": "encorar",
    "encoráremos": "encorar",
    "encorásemos": "encorar",
    "encoré": "encorar",
    "encoréis": "encorar",
    "encoró": "encorar",
    "encostala": "encostalar",
    "encostalaba": "encostalar",
    "encostalabais": "encostalar",
    "encostalaban": "encostalar",
    "encostalabas": "encostalar",
    "encostalad": "encostalar",
    "encostalada": "encostalar",
    "encostaladas": "encostalar",
    "encostalado": "encostalar",
    "encostalados": "encostalar",
    "encostalamos": "encostalar",
    "encostalan": "encostalar",
    "encostalando": "encostalar",
    "encostalara": "encostalar",
    "encostalarais": "encostalar",
    "encostalaran": "encostalar",
    "encostalaras": "encostalar",
    "encostalare": "encostalar",
    "encostalareis": "encostalar",
    "encostalaremos": "encostalar",
    "encostalaren": "encostalar",
    "encostalares": "encostalar",
    "encostalaron": "encostalar",
    "encostalará": "encostalar",
    "encostalarán": "encostalar",
    "encostalarás": "encostalar",
    "encostalaré": "encostalar",
    "encostalaréis": "encostalar",
    "encostalaría": "encostalar",
    "encostalaríais": "encostalar",
    "encostalaríamos": "encostalar",
    "encostalarían": "encostalar",
    "encostalarías": "encostalar",
    "encostalas": "encostalar",
    "encostalase": "encostalar",
    "encostalaseis": "encostalar",
    "encostalasen": "encostalar",
    "encostalases": "encostalar",
    "encostalaste": "encostalar",
    "encostalasteis": "encostalar",
    "encostale": "encostalar",
    "encostalemos": "encostalar",
    "encostalen": "encostalar",
    "encostales": "encostalar",
    "encostalo": "encostalar",
    "encostalábamos": "encostalar",
    "encostaláis": "encostalar",
    "encostaláramos": "encostalar",
    "encostaláremos": "encostalar",
    "encostalásemos": "encostalar",
    "encostalé": "encostalar",
    "encostaléis": "encostalar",
    "encostaló": "encostalar",
    "encostra": "encostrar",
    "encostraba": "encostrar",
    "encostrabais": "encostrar",
    "encostraban": "encostrar",
    "encostrabas": "encostrar",
    "encostrad": "encostrar",
    "encostrada": "encostrar",
    "encostradas": "encostrar",
    "encostrado": "encostrar",
    "encostrados": "encostrar",
    "encostraduras": "encostradura",
    "encostramientos": "encostramiento",
    "encostramos": "encostrar",
    "encostran": "encostrar",
    "encostrando": "encostrar",
    "encostrara": "encostrar",
    "encostrarais": "encostrar",
    "encostraran": "encostrar",
    "encostraras": "encostrar",
    "encostrare": "encostrar",
    "encostrareis": "encostrar",
    "encostraremos": "encostrar",
    "encostraren": "encostrar",
    "encostrares": "encostrar",
    "encostraron": "encostrar",
    "encostrará": "encostrar",
    "encostrarán": "encostrar",
    "encostrarás": "encostrar",
    "encostraré": "encostrar",
    "encostraréis": "encostrar",
    "encostraría": "encostrar",
    "encostraríais": "encostrar",
    "encostraríamos": "encostrar",
    "encostrarían": "encostrar",
    "encostrarías": "encostrar",
    "encostras": "encostrar",
    "encostrase": "encostrar",
    "encostraseis": "encostrar",
    "encostrasen": "encostrar",
    "encostrases": "encostrar",
    "encostraste": "encostrar",
    "encostrasteis": "encostrar",
    "encostre": "encostrar",
    "encostremos": "encostrar",
    "encostren": "encostrar",
    "encostres": "encostrar",
    "encostro": "encostrar",
    "encostrábamos": "encostrar",
    "encostráis": "encostrar",
    "encostráramos": "encostrar",
    "encostráremos": "encostrar",
    "encostrásemos": "encostrar",
    "encostré": "encostrar",
    "encostréis": "encostrar",
    "encostró": "encostrar",
    "encovaba": "encovar",
    "encovabais": "encovar",
    "encovaban": "encovar",
    "encovabas": "encovar",
    "encovad": "encovar",
    "encovada": "encovar",
    "encovadas": "encovar",
    "encovado": "encovar",
    "encovados": "encovar",
    "encovaduras": "encovadura",
    "encovamos": "encovar",
    "encovando": "encovar",
    "encovara": "encovar",
    "encovarais": "encovar",
    "encovaran": "encovar",
    "encovaras": "encovar",
    "encovare": "encovar",
    "encovareis": "encovar",
    "encovaremos": "encovar",
    "encovaren": "encovar",
    "encovares": "encovar",
    "encovaron": "encovar",
    "encovará": "encovar",
    "encovarán": "encovar",
    "encovarás": "encovar",
    "encovaré": "encovar",
    "encovaréis": "encovar",
    "encovaría": "encovar",
    "encovaríais": "encovar",
    "encovaríamos": "encovar",
    "encovarían": "encovar",
    "encovarías": "encovar",
    "encovase": "encovar",
    "encovaseis": "encovar",
    "encovasen": "encovar",
    "encovases": "encovar",
    "encovaste": "encovar",
    "encovasteis": "encovar",
    "encovemos": "encovar",
    "encovábamos": "encovar",
    "encováis": "encovar",
    "encováramos": "encovar",
    "encováremos": "encovar",
    "encovásemos": "encovar",
    "encové": "encovar",
    "encovéis": "encovar",
    "encovó": "encovar",
    "encrasa": "encrasar",
    "encrasaba": "encrasar",
    "encrasabais": "encrasar",
    "encrasaban": "encrasar",
    "encrasabas": "encrasar",
    "encrasad": "encrasar",
    "encrasada": "encrasar",
    "encrasadas": "encrasar",
    "encrasado": "encrasar",
    "encrasados": "encrasar",
    "encrasamos": "encrasar",
    "encrasan": "encrasar",
    "encrasando": "encrasar",
    "encrasara": "encrasar",
    "encrasarais": "encrasar",
    "encrasaran": "encrasar",
    "encrasaras": "encrasar",
    "encrasare": "encrasar",
    "encrasareis": "encrasar",
    "encrasaremos": "encrasar",
    "encrasaren": "encrasar",
    "encrasares": "encrasar",
    "encrasaron": "encrasar",
    "encrasará": "encrasar",
    "encrasarán": "encrasar",
    "encrasarás": "encrasar",
    "encrasaré": "encrasar",
    "encrasaréis": "encrasar",
    "encrasaría": "encrasar",
    "encrasaríais": "encrasar",
    "encrasaríamos": "encrasar",
    "encrasarían": "encrasar",
    "encrasarías": "encrasar",
    "encrasas": "encrasar",
    "encrasase": "encrasar",
    "encrasaseis": "encrasar",
    "encrasasen": "encrasar",
    "encrasases": "encrasar",
    "encrasaste": "encrasar",
    "encrasasteis": "encrasar",
    "encrase": "encrasar",
    "encrasemos": "encrasar",
    "encrasen": "encrasar",
    "encrases": "encrasar",
    "encraso": "encrasar",
    "encrasábamos": "encrasar",
    "encrasáis": "encrasar",
    "encrasáramos": "encrasar",
    "encrasáremos": "encrasar",
    "encrasásemos": "encrasar",
    "encrasé": "encrasar",
    "encraséis": "encrasar",
    "encrasó": "encrasar",
    "encrespa": "encrespar",
    "encrespaba": "encrespar",
    "encrespabais": "encrespar",
    "encrespaban": "encrespar",
    "encrespabas": "encrespar",
    "encrespad": "encrespar",
    "encrespada": "encrespar",
    "encrespadas": "encrespar",
    "encrespado": "encrespar",
    "encrespadora": "encrespador",
    "encrespadoras": "encrespador",
    "encrespadores": "encrespador",
    "encrespados": "encrespar",
    "encrespaduras": "encrespadura",
    "encrespamientos": "encrespamiento",
    "encrespamos": "encrespar",
    "encrespan": "encrespar",
    "encrespando": "encrespar",
    "encrespara": "encrespar",
    "encresparais": "encrespar",
    "encresparan": "encrespar",
    "encresparas": "encrespar",
    "encrespare": "encrespar",
    "encrespareis": "encrespar",
    "encresparemos": "encrespar",
    "encresparen": "encrespar",
    "encrespares": "encrespar",
    "encresparon": "encrespar",
    "encrespará": "encrespar",
    "encresparán": "encrespar",
    "encresparás": "encrespar",
    "encresparé": "encrespar",
    "encresparéis": "encrespar",
    "encresparía": "encrespar",
    "encresparíais": "encrespar",
    "encresparíamos": "encrespar",
    "encresparían": "encrespar",
    "encresparías": "encrespar",
    "encrespas": "encrespar",
    "encrespase": "encrespar",
    "encrespaseis": "encrespar",
    "encrespasen": "encrespar",
    "encrespases": "encrespar",
    "encrespaste": "encrespar",
    "encrespasteis": "encrespar",
    "encrespe": "encrespar",
    "encrespemos": "encrespar",
    "encrespen": "encrespar",
    "encrespes": "encrespar",
    "encrespo": "encrespar",
    "encrespábamos": "encrespar",
    "encrespáis": "encrespar",
    "encrespáramos": "encrespar",
    "encrespáremos": "encrespar",
    "encrespásemos": "encrespar",
    "encrespé": "encrespar",
    "encrespéis": "encrespar",
    "encrespó": "encrespar",
    "encrestada": "encrestado",
    "encrestadas": "encrestado",
    "encrestados": "encrestado",
    "encrestamientos": "encrestamiento",
    "encripta": "encriptar",
    "encriptaba": "encriptar",
    "encriptabais": "encriptar",
    "encriptaban": "encriptar",
    "encriptabas": "encriptar",
    "encriptaciones": "encriptación",
    "encriptad": "encriptar",
    "encriptada": "encriptar",
    "encriptadas": "encriptar",
    "encriptado": "encriptar",
    "encriptados": "encriptar",
    "encriptamos": "encriptar",
    "encriptan": "encriptar",
    "encriptando": "encriptar",
    "encriptara": "encriptar",
    "encriptarais": "encriptar",
    "encriptaran": "encriptar",
    "encriptaras": "encriptar",
    "encriptare": "encriptar",
    "encriptareis": "encriptar",
    "encriptaremos": "encriptar",
    "encriptaren": "encriptar",
    "encriptares": "encriptar",
    "encriptaron": "encriptar",
    "encriptará": "encriptar",
    "encriptarán": "encriptar",
    "encriptarás": "encriptar",
    "encriptaré": "encriptar",
    "encriptaréis": "encriptar",
    "encriptaría": "encriptar",
    "encriptaríais": "encriptar",
    "encriptaríamos": "encriptar",
    "encriptarían": "encriptar",
    "encriptarías": "encriptar",
    "encriptas": "encriptar",
    "encriptase": "encriptar",
    "encriptaseis": "encriptar",
    "encriptasen": "encriptar",
    "encriptases": "encriptar",
    "encriptaste": "encriptar",
    "encriptasteis": "encriptar",
    "encripte": "encriptar",
    "encriptemos": "encriptar",
    "encripten": "encriptar",
    "encriptes": "encriptar",
    "encripto": "encriptar",
    "encriptábamos": "encriptar",
    "encriptáis": "encriptar",
    "encriptáramos": "encriptar",
    "encriptáremos": "encriptar",
    "encriptásemos": "encriptar",
    "encripté": "encriptar",
    "encriptéis": "encriptar",
    "encriptó": "encriptar",
    "encristala": "encristalar",
    "encristalaba": "encristalar",
    "encristalabais": "encristalar",
    "encristalaban": "encristalar",
    "encristalabas": "encristalar",
    "encristalad": "encristalar",
    "encristalada": "encristalar",
    "encristaladas": "encristalar",
    "encristalado": "encristalar",
    "encristalados": "encristalar",
    "encristalamientos": "encristalamiento",
    "encristalamos": "encristalar",
    "encristalan": "encristalar",
    "encristalando": "encristalar",
    "encristalara": "encristalar",
    "encristalarais": "encristalar",
    "encristalaran": "encristalar",
    "encristalaras": "encristalar",
    "encristalare": "encristalar",
    "encristalareis": "encristalar",
    "encristalaremos": "encristalar",
    "encristalaren": "encristalar",
    "encristalares": "encristalar",
    "encristalaron": "encristalar",
    "encristalará": "encristalar",
    "encristalarán": "encristalar",
    "encristalarás": "encristalar",
    "encristalaré": "encristalar",
    "encristalaréis": "encristalar",
    "encristalaría": "encristalar",
    "encristalaríais": "encristalar",
    "encristalaríamos": "encristalar",
    "encristalarían": "encristalar",
    "encristalarías": "encristalar",
    "encristalas": "encristalar",
    "encristalase": "encristalar",
    "encristalaseis": "encristalar",
    "encristalasen": "encristalar",
    "encristalases": "encristalar",
    "encristalaste": "encristalar",
    "encristalasteis": "encristalar",
    "encristale": "encristalar",
    "encristalemos": "encristalar",
    "encristalen": "encristalar",
    "encristales": "encristalar",
    "encristalo": "encristalar",
    "encristalábamos": "encristalar",
    "encristaláis": "encristalar",
    "encristaláramos": "encristalar",
    "encristaláremos": "encristalar",
    "encristalásemos": "encristalar",
    "encristalé": "encristalar",
    "encristaléis": "encristalar",
    "encristaló": "encristalar",
    "encrucijadas": "encrucijada",
    "encrudece": "encrudecer",
    "encrudeced": "encrudecer",
    "encrudecemos": "encrudecer",
    "encrudecen": "encrudecer",
    "encrudeceremos": "encrudecer",
    "encrudecerá": "encrudecer",
    "encrudecerán": "encrudecer",
    "encrudecerás": "encrudecer",
    "encrudeceré": "encrudecer",
    "encrudeceréis": "encrudecer",
    "encrudecería": "encrudecer",
    "encrudeceríais": "encrudecer",
    "encrudeceríamos": "encrudecer",
    "encrudecerían": "encrudecer",
    "encrudecerías": "encrudecer",
    "encrudeces": "encrudecer",
    "encrudecida": "encrudecer",
    "encrudecidas": "encrudecer",
    "encrudecido": "encrudecer",
    "encrudecidos": "encrudecer",
    "encrudeciendo": "encrudecer",
    "encrudeciera": "encrudecer",
    "encrudecierais": "encrudecer",
    "encrudecieran": "encrudecer",
    "encrudecieras": "encrudecer",
    "encrudeciere": "encrudecer",
    "encrudeciereis": "encrudecer",
    "encrudecieren": "encrudecer",
    "encrudecieres": "encrudecer",
    "encrudecieron": "encrudecer",
    "encrudeciese": "encrudecer",
    "encrudecieseis": "encrudecer",
    "encrudeciesen": "encrudecer",
    "encrudecieses": "encrudecer",
    "encrudecimientos": "encrudecimiento",
    "encrudecimos": "encrudecer",
    "encrudeciste": "encrudecer",
    "encrudecisteis": "encrudecer",
    "encrudeciéramos": "encrudecer",
    "encrudeciéremos": "encrudecer",
    "encrudeciésemos": "encrudecer",
    "encrudeció": "encrudecer",
    "encrudecéis": "encrudecer",
    "encrudecí": "encrudecer",
    "encrudecía": "encrudecer",
    "encrudecíais": "encrudecer",
    "encrudecíamos": "encrudecer",
    "encrudecían": "encrudecer",
    "encrudecías": "encrudecer",
    "encrudezca": "encrudecer",
    "encrudezcamos": "encrudecer",
    "encrudezcan": "encrudecer",
    "encrudezcas": "encrudecer",
    "encrudezco": "encrudecer",
    "encrudezcáis": "encrudecer",
    "encruelece": "encruelecer",
    "encrueleced": "encruelecer",
    "encruelecemos": "encruelecer",
    "encruelecen": "encruelecer",
    "encrueleceremos": "encruelecer",
    "encruelecerá": "encruelecer",
    "encruelecerán": "encruelecer",
    "encruelecerás": "encruelecer",
    "encrueleceré": "encruelecer",
    "encrueleceréis": "encruelecer",
    "encruelecería": "encruelecer",
    "encrueleceríais": "encruelecer",
    "encrueleceríamos": "encruelecer",
    "encruelecerían": "encruelecer",
    "encruelecerías": "encruelecer",
    "encrueleces": "encruelecer",
    "encruelecida": "encruelecer",
    "encruelecidas": "encruelecer",
    "encruelecido": "encruelecer",
    "encruelecidos": "encruelecer",
    "encrueleciendo": "encruelecer",
    "encrueleciera": "encruelecer",
    "encruelecierais": "encruelecer",
    "encruelecieran": "encruelecer",
    "encruelecieras": "encruelecer",
    "encrueleciere": "encruelecer",
    "encrueleciereis": "encruelecer",
    "encruelecieren": "encruelecer",
    "encruelecieres": "encruelecer",
    "encruelecieron": "encruelecer",
    "encrueleciese": "encruelecer",
    "encruelecieseis": "encruelecer",
    "encrueleciesen": "encruelecer",
    "encruelecieses": "encruelecer",
    "encruelecimientos": "encruelecimiento",
    "encruelecimos": "encruelecer",
    "encrueleciste": "encruelecer",
    "encruelecisteis": "encruelecer",
    "encrueleciéramos": "encruelecer",
    "encrueleciéremos": "encruelecer",
    "encrueleciésemos": "encruelecer",
    "encrueleció": "encruelecer",
    "encruelecéis": "encruelecer",
    "encruelecí": "encruelecer",
    "encruelecía": "encruelecer",
    "encruelecíais": "encruelecer",
    "encruelecíamos": "encruelecer",
    "encruelecían": "encruelecer",
    "encruelecías": "encruelecer",
    "encruelezca": "encruelecer",
    "encruelezcamos": "encruelecer",
    "encruelezcan": "encruelecer",
    "encruelezcas": "encruelecer",
    "encruelezco": "encruelecer",
    "encruelezcáis": "encruelecer",
    "encuaderna": "encuadernar",
    "encuadernaba": "encuadernar",
    "encuadernabais": "encuadernar",
    "encuadernaban": "encuadernar",
    "encuadernabas": "encuadernar",
    "encuadernables": "encuadernable",
    "encuadernaciones": "encuadernación",
    "encuadernad": "encuadernar",
    "encuadernada": "encuadernar",
    "encuadernadas": "encuadernar",
    "encuadernado": "encuadernar",
    "encuadernadora": "encuadernador",
    "encuadernadoras": "encuadernador",
    "encuadernadores": "encuadernador",
    "encuadernados": "encuadernar",
    "encuadernamos": "encuadernar",
    "encuadernan": "encuadernar",
    "encuadernando": "encuadernar",
    "encuadernara": "encuadernar",
    "encuadernarais": "encuadernar",
    "encuadernaran": "encuadernar",
    "encuadernaras": "encuadernar",
    "encuadernare": "encuadernar",
    "encuadernareis": "encuadernar",
    "encuadernaremos": "encuadernar",
    "encuadernaren": "encuadernar",
    "encuadernares": "encuadernar",
    "encuadernaron": "encuadernar",
    "encuadernará": "encuadernar",
    "encuadernarán": "encuadernar",
    "encuadernarás": "encuadernar",
    "encuadernaré": "encuadernar",
    "encuadernaréis": "encuadernar",
    "encuadernaría": "encuadernar",
    "encuadernaríais": "encuadernar",
    "encuadernaríamos": "encuadernar",
    "encuadernarían": "encuadernar",
    "encuadernarías": "encuadernar",
    "encuadernas": "encuadernar",
    "encuadernase": "encuadernar",
    "encuadernaseis": "encuadernar",
    "encuadernasen": "encuadernar",
    "encuadernases": "encuadernar",
    "encuadernaste": "encuadernar",
    "encuadernasteis": "encuadernar",
    "encuaderne": "encuadernar",
    "encuadernemos": "encuadernar",
    "encuadernen": "encuadernar",
    "encuadernes": "encuadernar",
    "encuaderno": "encuadernar",
    "encuadernábamos": "encuadernar",
    "encuadernáis": "encuadernar",
    "encuadernáramos": "encuadernar",
    "encuadernáremos": "encuadernar",
    "encuadernásemos": "encuadernar",
    "encuaderné": "encuadernar",
    "encuadernéis": "encuadernar",
    "encuadernó": "encuadernar",
    "encuadra": "encuadrar",
    "encuadraba": "encuadrar",
    "encuadrabais": "encuadrar",
    "encuadraban": "encuadrar",
    "encuadrabas": "encuadrar",
    "encuadrad": "encuadrar",
    "encuadrada": "encuadrar",
    "encuadradas": "encuadrar",
    "encuadrado": "encuadrar",
    "encuadrados": "encuadrar",
    "encuadramientos": "encuadramiento",
    "encuadramos": "encuadrar",
    "encuadran": "encuadrar",
    "encuadrando": "encuadrar",
    "encuadrara": "encuadrar",
    "encuadrarais": "encuadrar",
    "encuadraran": "encuadrar",
    "encuadraras": "encuadrar",
    "encuadrare": "encuadrar",
    "encuadrareis": "encuadrar",
    "encuadraremos": "encuadrar",
    "encuadraren": "encuadrar",
    "encuadrares": "encuadrar",
    "encuadraron": "encuadrar",
    "encuadrará": "encuadrar",
    "encuadrarán": "encuadrar",
    "encuadrarás": "encuadrar",
    "encuadraré": "encuadrar",
    "encuadraréis": "encuadrar",
    "encuadraría": "encuadrar",
    "encuadraríais": "encuadrar",
    "encuadraríamos": "encuadrar",
    "encuadrarían": "encuadrar",
    "encuadrarías": "encuadrar",
    "encuadras": "encuadrar",
    "encuadrase": "encuadrar",
    "encuadraseis": "encuadrar",
    "encuadrasen": "encuadrar",
    "encuadrases": "encuadrar",
    "encuadraste": "encuadrar",
    "encuadrasteis": "encuadrar",
    "encuadre": "encuadrar",
    "encuadremos": "encuadrar",
    "encuadren": "encuadrar",
    "encuadres": "encuadre",
    "encuadro": "encuadrar",
    "encuadrábamos": "encuadrar",
    "encuadráis": "encuadrar",
    "encuadráramos": "encuadrar",
    "encuadráremos": "encuadrar",
    "encuadrásemos": "encuadrar",
    "encuadré": "encuadrar",
    "encuadréis": "encuadrar",
    "encuadró": "encuadrar",
    "encuarteros": "encuartero",
    "encuartes": "encuarte",
    "encuba": "encubar",
    "encubaba": "encubar",
    "encubabais": "encubar",
    "encubaban": "encubar",
    "encubabas": "encubar",
    "encubad": "encubar",
    "encubada": "encubar",
    "encubadas": "encubar",
    "encubado": "encubar",
    "encubados": "encubar",
    "encubamos": "encubar",
    "encuban": "encubar",
    "encubando": "encubar",
    "encubara": "encubar",
    "encubarais": "encubar",
    "encubaran": "encubar",
    "encubaras": "encubar",
    "encubare": "encubar",
    "encubareis": "encubar",
    "encubaremos": "encubar",
    "encubaren": "encubar",
    "encubares": "encubar",
    "encubaron": "encubar",
    "encubará": "encubar",
    "encubarán": "encubar",
    "encubarás": "encubar",
    "encubaré": "encubar",
    "encubaréis": "encubar",
    "encubaría": "encubar",
    "encubaríais": "encubar",
    "encubaríamos": "encubar",
    "encubarían": "encubar",
    "encubarías": "encubar",
    "encubas": "encubar",
    "encubase": "encubar",
    "encubaseis": "encubar",
    "encubasen": "encubar",
    "encubases": "encubar",
    "encubaste": "encubar",
    "encubasteis": "encubar",
    "encube": "encubar",
    "encubemos": "encubar",
    "encuben": "encubar",
    "encubes": "encubar",
    "encubierta": "encubrir",
    "encubiertas": "encubrir",
    "encubierto": "encubrir",
    "encubiertos": "encubrir",
    "encubo": "encubar",
    "encubra": "encubrir",
    "encubramos": "encubrir",
    "encubran": "encubrir",
    "encubras": "encubrir",
    "encubre": "encubrir",
    "encubren": "encubrir",
    "encubres": "encubrir",
    "encubrid": "encubrir",
    "encubridiza": "encubridizo",
    "encubridizas": "encubridizo",
    "encubridizos": "encubridizo",
    "encubridora": "encubridor",
    "encubridoras": "encubridor",
    "encubridores": "encubridor",
    "encubriendo": "encubrir",
    "encubriera": "encubrir",
    "encubrierais": "encubrir",
    "encubrieran": "encubrir",
    "encubrieras": "encubrir",
    "encubriere": "encubrir",
    "encubriereis": "encubrir",
    "encubrieren": "encubrir",
    "encubrieres": "encubrir",
    "encubrieron": "encubrir",
    "encubriese": "encubrir",
    "encubrieseis": "encubrir",
    "encubriesen": "encubrir",
    "encubrieses": "encubrir",
    "encubrimientos": "encubrimiento",
    "encubrimos": "encubrir",
    "encubriremos": "encubrir",
    "encubrirá": "encubrir",
    "encubrirán": "encubrir",
    "encubrirás": "encubrir",
    "encubriré": "encubrir",
    "encubriréis": "encubrir",
    "encubriría": "encubrir",
    "encubriríais": "encubrir",
    "encubriríamos": "encubrir",
    "encubrirían": "encubrir",
    "encubrirías": "encubrir",
    "encubriste": "encubrir",
    "encubristeis": "encubrir",
    "encubriéramos": "encubrir",
    "encubriéremos": "encubrir",
    "encubriésemos": "encubrir",
    "encubrió": "encubrir",
    "encubro": "encubrir",
    "encubráis": "encubrir",
    "encubrí": "encubrir",
    "encubría": "encubrir",
    "encubríais": "encubrir",
    "encubríamos": "encubrir",
    "encubrían": "encubrir",
    "encubrías": "encubrir",
    "encubrís": "encubrir",
    "encubábamos": "encubar",
    "encubáis": "encubar",
    "encubáramos": "encubar",
    "encubáremos": "encubar",
    "encubásemos": "encubar",
    "encubé": "encubar",
    "encubéis": "encubar",
    "encubó": "encubar",
    "encuentra": "encontrar",
    "encuentran": "encontrar",
    "encuentras": "encontrar",
    "encuentre": "encontrar",
    "encuentren": "encontrar",
    "encuentres": "encontrar",
    "encuentro": "encontrar",
    "encuentros": "encuentro",
    "encuera": "encuerar",
    "encueraba": "encuerar",
    "encuerabais": "encuerar",
    "encueraban": "encuerar",
    "encuerabas": "encuerar",
    "encuerad": "encuerar",
    "encuerada": "encuerar",
    "encueradas": "encuerar",
    "encuerado": "encuerar",
    "encuerados": "encuerar",
    "encueramos": "encuerar",
    "encueran": "encuerar",
    "encuerando": "encuerar",
    "encuerara": "encuerar",
    "encuerarais": "encuerar",
    "encueraran": "encuerar",
    "encueraras": "encuerar",
    "encuerare": "encuerar",
    "encuerareis": "encuerar",
    "encueraremos": "encuerar",
    "encueraren": "encuerar",
    "encuerares": "encuerar",
    "encueraron": "encuerar",
    "encuerará": "encuerar",
    "encuerarán": "encuerar",
    "encuerarás": "encuerar",
    "encueraré": "encuerar",
    "encueraréis": "encuerar",
    "encueraría": "encuerar",
    "encueraríais": "encuerar",
    "encueraríamos": "encuerar",
    "encuerarían": "encuerar",
    "encuerarías": "encuerar",
    "encueras": "encuerar",
    "encuerase": "encuerar",
    "encueraseis": "encuerar",
    "encuerasen": "encuerar",
    "encuerases": "encuerar",
    "encueraste": "encuerar",
    "encuerasteis": "encuerar",
    "encuerda": "encordar",
    "encuerdan": "encordar",
    "encuerdas": "encordar",
    "encuerde": "encordar",
    "encuerden": "encordar",
    "encuerdes": "encordar",
    "encuerdo": "encordar",
    "encuere": "encuerar",
    "encueremos": "encuerar",
    "encueren": "encuerar",
    "encueres": "encuerar",
    "encuero": "encuerar",
    "encuerábamos": "encuerar",
    "encueráis": "encuerar",
    "encueráramos": "encuerar",
    "encueráremos": "encuerar",
    "encuerásemos": "encuerar",
    "encueré": "encuerar",
    "encueréis": "encuerar",
    "encueró": "encuerar",
    "encuesta": "encuestar",
    "encuestaba": "encuestar",
    "encuestabais": "encuestar",
    "encuestaban": "encuestar",
    "encuestabas": "encuestar",
    "encuestad": "encuestar",
    "encuestada": "encuestar",
    "encuestadas": "encuestar",
    "encuestado": "encuestar",
    "encuestadora": "encuestador",
    "encuestadoras": "encuestador",
    "encuestadores": "encuestador",
    "encuestados": "encuestar",
    "encuestamos": "encuestar",
    "encuestan": "encuestar",
    "encuestando": "encuestar",
    "encuestara": "encuestar",
    "encuestarais": "encuestar",
    "encuestaran": "encuestar",
    "encuestaras": "encuestar",
    "encuestare": "encuestar",
    "encuestareis": "encuestar",
    "encuestaremos": "encuestar",
    "encuestaren": "encuestar",
    "encuestares": "encuestar",
    "encuestaron": "encuestar",
    "encuestará": "encuestar",
    "encuestarán": "encuestar",
    "encuestarás": "encuestar",
    "encuestaré": "encuestar",
    "encuestaréis": "encuestar",
    "encuestaría": "encuestar",
    "encuestaríais": "encuestar",
    "encuestaríamos": "encuestar",
    "encuestarían": "encuestar",
    "encuestarías": "encuestar",
    "encuestas": "encuestar",
    "encuestase": "encuestar",
    "encuestaseis": "encuestar",
    "encuestasen": "encuestar",
    "encuestases": "encuestar",
    "encuestaste": "encuestar",
    "encuestasteis": "encuestar",
    "encueste": "encuestar",
    "encuestemos": "encuestar",
    "encuesten": "encuestar",
    "encuestes": "encuestar",
    "encuesto": "encuestar",
    "encuestábamos": "encuestar",
    "encuestáis": "encuestar",
    "encuestáramos": "encuestar",
    "encuestáremos": "encuestar",
    "encuestásemos": "encuestar",
    "encuesté": "encuestar",
    "encuestéis": "encuestar",
    "encuestó": "encuestar",
    "encueva": "encovar",
    "encuevan": "encovar",
    "encuevas": "encovar",
    "encueve": "encovar",
    "encueven": "encovar",
    "encueves": "encovar",
    "encuevo": "encovar",
    "encuita": "encuitar",
    "encuitaba": "encuitar",
    "encuitabais": "encuitar",
    "encuitaban": "encuitar",
    "encuitabas": "encuitar",
    "encuitad": "encuitar",
    "encuitada": "encuitar",
    "encuitadas": "encuitar",
    "encuitado": "encuitar",
    "encuitados": "encuitar",
    "encuitamos": "encuitar",
    "encuitan": "encuitar",
    "encuitando": "encuitar",
    "encuitara": "encuitar",
    "encuitarais": "encuitar",
    "encuitaran": "encuitar",
    "encuitaras": "encuitar",
    "encuitare": "encuitar",
    "encuitareis": "encuitar",
    "encuitaremos": "encuitar",
    "encuitaren": "encuitar",
    "encuitares": "encuitar",
    "encuitaron": "encuitar",
    "encuitará": "encuitar",
    "encuitarán": "encuitar",
    "encuitarás": "encuitar",
    "encuitaré": "encuitar",
    "encuitaréis": "encuitar",
    "encuitaría": "encuitar",
    "encuitaríais": "encuitar",
    "encuitaríamos": "encuitar",
    "encuitarían": "encuitar",
    "encuitarías": "encuitar",
    "encuitas": "encuitar",
    "encuitase": "encuitar",
    "encuitaseis": "encuitar",
    "encuitasen": "encuitar",
    "encuitases": "encuitar",
    "encuitaste": "encuitar",
    "encuitasteis": "encuitar",
    "encuite": "encuitar",
    "encuitemos": "encuitar",
    "encuiten": "encuitar",
    "encuites": "encuitar",
    "encuito": "encuitar",
    "encuitábamos": "encuitar",
    "encuitáis": "encuitar",
    "encuitáramos": "encuitar",
    "encuitáremos": "encuitar",
    "encuitásemos": "encuitar",
    "encuité": "encuitar",
    "encuitéis": "encuitar",
    "encuitó": "encuitar",
    "encumbra": "encumbrar",
    "encumbraba": "encumbrar",
    "encumbrabais": "encumbrar",
    "encumbraban": "encumbrar",
    "encumbrabas": "encumbrar",
    "encumbrad": "encumbrar",
    "encumbrada": "encumbrar",
    "encumbradas": "encumbrar",
    "encumbrado": "encumbrar",
    "encumbrados": "encumbrar",
    "encumbramientos": "encumbramiento",
    "encumbramos": "encumbrar",
    "encumbran": "encumbrar",
    "encumbrando": "encumbrar",
    "encumbrara": "encumbrar",
    "encumbrarais": "encumbrar",
    "encumbraran": "encumbrar",
    "encumbraras": "encumbrar",
    "encumbrare": "encumbrar",
    "encumbrareis": "encumbrar",
    "encumbraremos": "encumbrar",
    "encumbraren": "encumbrar",
    "encumbrares": "encumbrar",
    "encumbraron": "encumbrar",
    "encumbrará": "encumbrar",
    "encumbrarán": "encumbrar",
    "encumbrarás": "encumbrar",
    "encumbraré": "encumbrar",
    "encumbraréis": "encumbrar",
    "encumbraría": "encumbrar",
    "encumbraríais": "encumbrar",
    "encumbraríamos": "encumbrar",
    "encumbrarían": "encumbrar",
    "encumbrarías": "encumbrar",
    "encumbras": "encumbrar",
    "encumbrase": "encumbrar",
    "encumbraseis": "encumbrar",
    "encumbrasen": "encumbrar",
    "encumbrases": "encumbrar",
    "encumbraste": "encumbrar",
    "encumbrasteis": "encumbrar",
    "encumbre": "encumbrar",
    "encumbremos": "encumbrar",
    "encumbren": "encumbrar",
    "encumbres": "encumbrar",
    "encumbro": "encumbrar",
    "encumbrábamos": "encumbrar",
    "encumbráis": "encumbrar",
    "encumbráramos": "encumbrar",
    "encumbráremos": "encumbrar",
    "encumbrásemos": "encumbrar",
    "encumbré": "encumbrar",
    "encumbréis": "encumbrar",
    "encumbró": "encumbrar",
    "encuna": "encunar",
    "encunaba": "encunar",
    "encunabais": "encunar",
    "encunaban": "encunar",
    "encunabas": "encunar",
    "encunad": "encunar",
    "encunada": "encunar",
    "encunadas": "encunar",
    "encunado": "encunar",
    "encunados": "encunar",
    "encunamos": "encunar",
    "encunan": "encunar",
    "encunando": "encunar",
    "encunara": "encunar",
    "encunarais": "encunar",
    "encunaran": "encunar",
    "encunaras": "encunar",
    "encunare": "encunar",
    "encunareis": "encunar",
    "encunaremos": "encunar",
    "encunaren": "encunar",
    "encunares": "encunar",
    "encunaron": "encunar",
    "encunará": "encunar",
    "encunarán": "encunar",
    "encunarás": "encunar",
    "encunaré": "encunar",
    "encunaréis": "encunar",
    "encunaría": "encunar",
    "encunaríais": "encunar",
    "encunaríamos": "encunar",
    "encunarían": "encunar",
    "encunarías": "encunar",
    "encunas": "encunar",
    "encunase": "encunar",
    "encunaseis": "encunar",
    "encunasen": "encunar",
    "encunases": "encunar",
    "encunaste": "encunar",
    "encunasteis": "encunar",
    "encune": "encunar",
    "encunemos": "encunar",
    "encunen": "encunar",
    "encunes": "encunar",
    "encuno": "encunar",
    "encunábamos": "encunar",
    "encunáis": "encunar",
    "encunáramos": "encunar",
    "encunáremos": "encunar",
    "encunásemos": "encunar",
    "encuné": "encunar",
    "encunéis": "encunar",
    "encunó": "encunar",
    "encureña": "encureñar",
    "encureñaba": "encureñar",
    "encureñabais": "encureñar",
    "encureñaban": "encureñar",
    "encureñabas": "encureñar",
    "encureñad": "encureñar",
    "encureñada": "encureñar",
    "encureñadas": "encureñar",
    "encureñado": "encureñar",
    "encureñados": "encureñar",
    "encureñamos": "encureñar",
    "encureñan": "encureñar",
    "encureñando": "encureñar",
    "encureñara": "encureñar",
    "encureñarais": "encureñar",
    "encureñaran": "encureñar",
    "encureñaras": "encureñar",
    "encureñare": "encureñar",
    "encureñareis": "encureñar",
    "encureñaremos": "encureñar",
    "encureñaren": "encureñar",
    "encureñares": "encureñar",
    "encureñaron": "encureñar",
    "encureñará": "encureñar",
    "encureñarán": "encureñar",
    "encureñarás": "encureñar",
    "encureñaré": "encureñar",
    "encureñaréis": "encureñar",
    "encureñaría": "encureñar",
    "encureñaríais": "encureñar",
    "encureñaríamos": "encureñar",
    "encureñarían": "encureñar",
    "encureñarías": "encureñar",
    "encureñas": "encureñar",
    "encureñase": "encureñar",
    "encureñaseis": "encureñar",
    "encureñasen": "encureñar",
    "encureñases": "encureñar",
    "encureñaste": "encureñar",
    "encureñasteis": "encureñar",
    "encureñe": "encureñar",
    "encureñemos": "encureñar",
    "encureñen": "encureñar",
    "encureñes": "encureñar",
    "encureño": "encureñar",
    "encureñábamos": "encureñar",
    "encureñáis": "encureñar",
    "encureñáramos": "encureñar",
    "encureñáremos": "encureñar",
    "encureñásemos": "encureñar",
    "encureñé": "encureñar",
    "encureñéis": "encureñar",
    "encureñó": "encureñar",
    "encurta": "encurtir",
    "encurtamos": "encurtir",
    "encurtan": "encurtir",
    "encurtas": "encurtir",
    "encurte": "encurtir",
    "encurten": "encurtir",
    "encurtes": "encurtir",
    "encurtid": "encurtir",
    "encurtida": "encurtir",
    "encurtidas": "encurtir",
    "encurtido": "encurtir",
    "encurtidos": "encurtir",
    "encurtiendo": "encurtir",
    "encurtiera": "encurtir",
    "encurtierais": "encurtir",
    "encurtieran": "encurtir",
    "encurtieras": "encurtir",
    "encurtiere": "encurtir",
    "encurtiereis": "encurtir",
    "encurtieren": "encurtir",
    "encurtieres": "encurtir",
    "encurtieron": "encurtir",
    "encurtiese": "encurtir",
    "encurtieseis": "encurtir",
    "encurtiesen": "encurtir",
    "encurtieses": "encurtir",
    "encurtimos": "encurtir",
    "encurtiremos": "encurtir",
    "encurtirá": "encurtir",
    "encurtirán": "encurtir",
    "encurtirás": "encurtir",
    "encurtiré": "encurtir",
    "encurtiréis": "encurtir",
    "encurtiría": "encurtir",
    "encurtiríais": "encurtir",
    "encurtiríamos": "encurtir",
    "encurtirían": "encurtir",
    "encurtirías": "encurtir",
    "encurtiste": "encurtir",
    "encurtisteis": "encurtir",
    "encurtiéramos": "encurtir",
    "encurtiéremos": "encurtir",
    "encurtiésemos": "encurtir",
    "encurtió": "encurtir",
    "encurto": "encurtir",
    "encurtáis": "encurtir",
    "encurtí": "encurtir",
    "encurtía": "encurtir",
    "encurtíais": "encurtir",
    "encurtíamos": "encurtir",
    "encurtían": "encurtir",
    "encurtías": "encurtir",
    "encurtís": "encurtir",
    "encáustica": "encáustico",
    "encáusticas": "encáustico",
    "encáusticos": "encáustico",
    "encéfalos": "encéfalo",
    "encías": "encía",
    "encíclica": "encíclico",
    "encíclicas": "encíclico",
    "encíclicos": "encíclico",
    "endebleces": "endeblez",
    "endebles": "endeble",
    "endecasilábica": "endecasilábico",
    "endecasilábicas": "endecasilábico",
    "endecasilábicos": "endecasilábico",
    "endecasílaba": "endecasílabo",
    "endecasílabas": "endecasílabo",
    "endecasílabos": "endecasílabo",
    "endecha": "endechar",
    "endechaba": "endechar",
    "endechabais": "endechar",
    "endechaban": "endechar",
    "endechabas": "endechar",
    "endechad": "endechar",
    "endechada": "endechar",
    "endechadas": "endechar",
    "endechado": "endechar",
    "endechados": "endechar",
    "endechamos": "endechar",
    "endechan": "endechar",
    "endechando": "endechar",
    "endechara": "endechar",
    "endecharais": "endechar",
    "endecharan": "endechar",
    "endecharas": "endechar",
    "endechare": "endechar",
    "endechareis": "endechar",
    "endecharemos": "endechar",
    "endecharen": "endechar",
    "endechares": "endechar",
    "endecharon": "endechar",
    "endechará": "endechar",
    "endecharán": "endechar",
    "endecharás": "endechar",
    "endecharé": "endechar",
    "endecharéis": "endechar",
    "endecharía": "endechar",
    "endecharíais": "endechar",
    "endecharíamos": "endechar",
    "endecharían": "endechar",
    "endecharías": "endechar",
    "endechas": "endechar",
    "endechase": "endechar",
    "endechaseis": "endechar",
    "endechasen": "endechar",
    "endechases": "endechar",
    "endechaste": "endechar",
    "endechasteis": "endechar",
    "endeche": "endechar",
    "endechemos": "endechar",
    "endechen": "endechar",
    "endeches": "endechar",
    "endecho": "endechar",
    "endechábamos": "endechar",
    "endecháis": "endechar",
    "endecháramos": "endechar",
    "endecháremos": "endechar",
    "endechásemos": "endechar",
    "endeché": "endechar",
    "endechéis": "endechar",
    "endechó": "endechar",
    "endecágonos": "endecágono",
    "endejas": "endeja",
    "endelionitas": "endelionita",
    "endemias": "endemia",
    "endemicidades": "endemicidad",
    "endemismos": "endemismo",
    "endemonia": "endemoniar",
    "endemoniaba": "endemoniar",
    "endemoniabais": "endemoniar",
    "endemoniaban": "endemoniar",
    "endemoniabas": "endemoniar",
    "endemoniad": "endemoniar",
    "endemoniada": "endemoniar",
    "endemoniadas": "endemoniar",
    "endemoniado": "endemoniar",
    "endemoniados": "endemoniar",
    "endemoniamos": "endemoniar",
    "endemonian": "endemoniar",
    "endemoniando": "endemoniar",
    "endemoniara": "endemoniar",
    "endemoniarais": "endemoniar",
    "endemoniaran": "endemoniar",
    "endemoniaras": "endemoniar",
    "endemoniare": "endemoniar",
    "endemoniareis": "endemoniar",
    "endemoniaremos": "endemoniar",
    "endemoniaren": "endemoniar",
    "endemoniares": "endemoniar",
    "endemoniaron": "endemoniar",
    "endemoniará": "endemoniar",
    "endemoniarán": "endemoniar",
    "endemoniarás": "endemoniar",
    "endemoniaré": "endemoniar",
    "endemoniaréis": "endemoniar",
    "endemoniaría": "endemoniar",
    "endemoniaríais": "endemoniar",
    "endemoniaríamos": "endemoniar",
    "endemoniarían": "endemoniar",
    "endemoniarías": "endemoniar",
    "endemonias": "endemoniar",
    "endemoniase": "endemoniar",
    "endemoniaseis": "endemoniar",
    "endemoniasen": "endemoniar",
    "endemoniases": "endemoniar",
    "endemoniaste": "endemoniar",
    "endemoniasteis": "endemoniar",
    "endemonie": "endemoniar",
    "endemoniemos": "endemoniar",
    "endemonien": "endemoniar",
    "endemonies": "endemoniar",
    "endemonio": "endemoniar",
    "endemoniábamos": "endemoniar",
    "endemoniáis": "endemoniar",
    "endemoniáramos": "endemoniar",
    "endemoniáremos": "endemoniar",
    "endemoniásemos": "endemoniar",
    "endemonié": "endemoniar",
    "endemoniéis": "endemoniar",
    "endemonió": "endemoniar",
    "endentaba": "endentar",
    "endentabais": "endentar",
    "endentaban": "endentar",
    "endentabas": "endentar",
    "endentad": "endentar",
    "endentada": "endentar",
    "endentadas": "endentar",
    "endentado": "endentar",
    "endentados": "endentar",
    "endentamos": "endentar",
    "endentando": "endentar",
    "endentara": "endentar",
    "endentarais": "endentar",
    "endentaran": "endentar",
    "endentaras": "endentar",
    "endentare": "endentar",
    "endentareis": "endentar",
    "endentaremos": "endentar",
    "endentaren": "endentar",
    "endentares": "endentar",
    "endentaron": "endentar",
    "endentará": "endentar",
    "endentarán": "endentar",
    "endentarás": "endentar",
    "endentaré": "endentar",
    "endentaréis": "endentar",
    "endentaría": "endentar",
    "endentaríais": "endentar",
    "endentaríamos": "endentar",
    "endentarían": "endentar",
    "endentarías": "endentar",
    "endentase": "endentar",
    "endentaseis": "endentar",
    "endentasen": "endentar",
    "endentases": "endentar",
    "endentaste": "endentar",
    "endentasteis": "endentar",
    "endentece": "endentecer",
    "endenteced": "endentecer",
    "endentecemos": "endentecer",
    "endentecen": "endentecer",
    "endenteceremos": "endentecer",
    "endentecerá": "endentecer",
    "endentecerán": "endentecer",
    "endentecerás": "endentecer",
    "endenteceré": "endentecer",
    "endenteceréis": "endentecer",
    "endentecería": "endentecer",
    "endenteceríais": "endentecer",
    "endenteceríamos": "endentecer",
    "endentecerían": "endentecer",
    "endentecerías": "endentecer",
    "endenteces": "endentecer",
    "endentecida": "endentecer",
    "endentecidas": "endentecer",
    "endentecido": "endentecer",
    "endentecidos": "endentecer",
    "endenteciendo": "endentecer",
    "endenteciera": "endentecer",
    "endentecierais": "endentecer",
    "endentecieran": "endentecer",
    "endentecieras": "endentecer",
    "endenteciere": "endentecer",
    "endenteciereis": "endentecer",
    "endentecieren": "endentecer",
    "endentecieres": "endentecer",
    "endentecieron": "endentecer",
    "endenteciese": "endentecer",
    "endentecieseis": "endentecer",
    "endenteciesen": "endentecer",
    "endentecieses": "endentecer",
    "endentecimos": "endentecer",
    "endenteciste": "endentecer",
    "endentecisteis": "endentecer",
    "endenteciéramos": "endentecer",
    "endenteciéremos": "endentecer",
    "endenteciésemos": "endentecer",
    "endenteció": "endentecer",
    "endentecéis": "endentecer",
    "endentecí": "endentecer",
    "endentecía": "endentecer",
    "endentecíais": "endentecer",
    "endentecíamos": "endentecer",
    "endentecían": "endentecer",
    "endentecías": "endentecer",
    "endentemos": "endentar",
    "endentezca": "endentecer",
    "endentezcamos": "endentecer",
    "endentezcan": "endentecer",
    "endentezcas": "endentecer",
    "endentezco": "endentecer",
    "endentezcáis": "endentecer",
    "endentábamos": "endentar",
    "endentáis": "endentar",
    "endentáramos": "endentar",
    "endentáremos": "endentar",
    "endentásemos": "endentar",
    "endenté": "endentar",
    "endentéis": "endentar",
    "endentó": "endentar",
    "enderece": "enderezar",
    "enderecemos": "enderezar",
    "enderecen": "enderezar",
    "endereces": "enderezar",
    "enderecé": "enderezar",
    "enderecéis": "enderezar",
    "endereza": "enderezar",
    "enderezaba": "enderezar",
    "enderezabais": "enderezar",
    "enderezaban": "enderezar",
    "enderezabas": "enderezar",
    "enderezad": "enderezar",
    "enderezada": "enderezar",
    "enderezadas": "enderezar",
    "enderezado": "enderezar",
    "enderezadora": "enderezador",
    "enderezadoras": "enderezadora",
    "enderezadores": "enderezador",
    "enderezados": "enderezar",
    "enderezamientos": "enderezamiento",
    "enderezamos": "enderezar",
    "enderezan": "enderezar",
    "enderezando": "enderezar",
    "enderezara": "enderezar",
    "enderezarais": "enderezar",
    "enderezaran": "enderezar",
    "enderezaras": "enderezar",
    "enderezare": "enderezar",
    "enderezareis": "enderezar",
    "enderezaremos": "enderezar",
    "enderezaren": "enderezar",
    "enderezares": "enderezar",
    "enderezaron": "enderezar",
    "enderezará": "enderezar",
    "enderezarán": "enderezar",
    "enderezarás": "enderezar",
    "enderezaré": "enderezar",
    "enderezaréis": "enderezar",
    "enderezaría": "enderezar",
    "enderezaríais": "enderezar",
    "enderezaríamos": "enderezar",
    "enderezarían": "enderezar",
    "enderezarías": "enderezar",
    "enderezas": "enderezar",
    "enderezase": "enderezar",
    "enderezaseis": "enderezar",
    "enderezasen": "enderezar",
    "enderezases": "enderezar",
    "enderezaste": "enderezar",
    "enderezasteis": "enderezar",
    "enderezo": "enderezar",
    "enderezábamos": "enderezar",
    "enderezáis": "enderezar",
    "enderezáramos": "enderezar",
    "enderezáremos": "enderezar",
    "enderezásemos": "enderezar",
    "enderezó": "enderezar",
    "endeuda": "endeudar",
    "endeudaba": "endeudar",
    "endeudabais": "endeudar",
    "endeudaban": "endeudar",
    "endeudabas": "endeudar",
    "endeudad": "endeudar",
    "endeudada": "endeudar",
    "endeudadas": "endeudar",
    "endeudado": "endeudar",
    "endeudados": "endeudar",
    "endeudamientos": "endeudamiento",
    "endeudamos": "endeudar",
    "endeudan": "endeudar",
    "endeudando": "endeudar",
    "endeudara": "endeudar",
    "endeudarais": "endeudar",
    "endeudaran": "endeudar",
    "endeudaras": "endeudar",
    "endeudare": "endeudar",
    "endeudareis": "endeudar",
    "endeudaremos": "endeudar",
    "endeudaren": "endeudar",
    "endeudares": "endeudar",
    "endeudaron": "endeudar",
    "endeudará": "endeudar",
    "endeudarán": "endeudar",
    "endeudarás": "endeudar",
    "endeudaré": "endeudar",
    "endeudaréis": "endeudar",
    "endeudaría": "endeudar",
    "endeudaríais": "endeudar",
    "endeudaríamos": "endeudar",
    "endeudarían": "endeudar",
    "endeudarías": "endeudar",
    "endeudas": "endeudar",
    "endeudase": "endeudar",
    "endeudaseis": "endeudar",
    "endeudasen": "endeudar",
    "endeudases": "endeudar",
    "endeudaste": "endeudar",
    "endeudasteis": "endeudar",
    "endeude": "endeudar",
    "endeudemos": "endeudar",
    "endeuden": "endeudar",
    "endeudes": "endeudar",
    "endeudo": "endeudar",
    "endeudábamos": "endeudar",
    "endeudáis": "endeudar",
    "endeudáramos": "endeudar",
    "endeudáremos": "endeudar",
    "endeudásemos": "endeudar",
    "endeudé": "endeudar",
    "endeudéis": "endeudar",
    "endeudó": "endeudar",
    "endevotada": "endevotado",
    "endevotadas": "endevotado",
    "endevotados": "endevotado",
    "endiabla": "endiablar",
    "endiablaba": "endiablar",
    "endiablabais": "endiablar",
    "endiablaban": "endiablar",
    "endiablabas": "endiablar",
    "endiablad": "endiablar",
    "endiablada": "endiablar",
    "endiabladas": "endiablar",
    "endiablado": "endiablar",
    "endiablados": "endiablar",
    "endiablamos": "endiablar",
    "endiablan": "endiablar",
    "endiablando": "endiablar",
    "endiablara": "endiablar",
    "endiablarais": "endiablar",
    "endiablaran": "endiablar",
    "endiablaras": "endiablar",
    "endiablare": "endiablar",
    "endiablareis": "endiablar",
    "endiablaremos": "endiablar",
    "endiablaren": "endiablar",
    "endiablares": "endiablar",
    "endiablaron": "endiablar",
    "endiablará": "endiablar",
    "endiablarán": "endiablar",
    "endiablarás": "endiablar",
    "endiablaré": "endiablar",
    "endiablaréis": "endiablar",
    "endiablaría": "endiablar",
    "endiablaríais": "endiablar",
    "endiablaríamos": "endiablar",
    "endiablarían": "endiablar",
    "endiablarías": "endiablar",
    "endiablas": "endiablar",
    "endiablase": "endiablar",
    "endiablaseis": "endiablar",
    "endiablasen": "endiablar",
    "endiablases": "endiablar",
    "endiablaste": "endiablar",
    "endiablasteis": "endiablar",
    "endiable": "endiablar",
    "endiablemos": "endiablar",
    "endiablen": "endiablar",
    "endiables": "endiablar",
    "endiablo": "endiablar",
    "endiablábamos": "endiablar",
    "endiabláis": "endiablar",
    "endiabláramos": "endiablar",
    "endiabláremos": "endiablar",
    "endiablásemos": "endiablar",
    "endiablé": "endiablar",
    "endiabléis": "endiablar",
    "endiabló": "endiablar",
    "endibias": "endibia",
    "endienta": "endentar",
    "endientan": "endentar",
    "endientas": "endentar",
    "endiente": "endentar",
    "endienten": "endentar",
    "endientes": "endentar",
    "endiento": "endentar",
    "endilga": "endilgar",
    "endilgaba": "endilgar",
    "endilgabais": "endilgar",
    "endilgaban": "endilgar",
    "endilgabas": "endilgar",
    "endilgad": "endilgar",
    "endilgada": "endilgar",
    "endilgadas": "endilgar",
    "endilgado": "endilgar",
    "endilgados": "endilgar",
    "endilgamientos": "endilgamiento",
    "endilgamos": "endilgar",
    "endilgan": "endilgar",
    "endilgando": "endilgar",
    "endilgara": "endilgar",
    "endilgarais": "endilgar",
    "endilgaran": "endilgar",
    "endilgaras": "endilgar",
    "endilgare": "endilgar",
    "endilgareis": "endilgar",
    "endilgaremos": "endilgar",
    "endilgaren": "endilgar",
    "endilgares": "endilgar",
    "endilgaron": "endilgar",
    "endilgará": "endilgar",
    "endilgarán": "endilgar",
    "endilgarás": "endilgar",
    "endilgaré": "endilgar",
    "endilgaréis": "endilgar",
    "endilgaría": "endilgar",
    "endilgaríais": "endilgar",
    "endilgaríamos": "endilgar",
    "endilgarían": "endilgar",
    "endilgarías": "endilgar",
    "endilgas": "endilgar",
    "endilgase": "endilgar",
    "endilgaseis": "endilgar",
    "endilgasen": "endilgar",
    "endilgases": "endilgar",
    "endilgaste": "endilgar",
    "endilgasteis": "endilgar",
    "endilgo": "endilgar",
    "endilgue": "endilgar",
    "endilguemos": "endilgar",
    "endilguen": "endilgar",
    "endilgues": "endilgar",
    "endilgué": "endilgar",
    "endilguéis": "endilgar",
    "endilgábamos": "endilgar",
    "endilgáis": "endilgar",
    "endilgáramos": "endilgar",
    "endilgáremos": "endilgar",
    "endilgásemos": "endilgar",
    "endilgó": "endilgar",
    "endina": "endino",
    "endinas": "endino",
    "endinos": "endino",
    "endiosa": "endiosar",
    "endiosaba": "endiosar",
    "endiosabais": "endiosar",
    "endiosaban": "endiosar",
    "endiosabas": "endiosar",
    "endiosad": "endiosar",
    "endiosada": "endiosar",
    "endiosadas": "endiosar",
    "endiosado": "endiosar",
    "endiosados": "endiosar",
    "endiosamientos": "endiosamiento",
    "endiosamos": "endiosar",
    "endiosan": "endiosar",
    "endiosando": "endiosar",
    "endiosara": "endiosar",
    "endiosarais": "endiosar",
    "endiosaran": "endiosar",
    "endiosaras": "endiosar",
    "endiosare": "endiosar",
    "endiosareis": "endiosar",
    "endiosaremos": "endiosar",
    "endiosaren": "endiosar",
    "endiosares": "endiosar",
    "endiosaron": "endiosar",
    "endiosará": "endiosar",
    "endiosarán": "endiosar",
    "endiosarás": "endiosar",
    "endiosaré": "endiosar",
    "endiosaréis": "endiosar",
    "endiosaría": "endiosar",
    "endiosaríais": "endiosar",
    "endiosaríamos": "endiosar",
    "endiosarían": "endiosar",
    "endiosarías": "endiosar",
    "endiosas": "endiosar",
    "endiosase": "endiosar",
    "endiosaseis": "endiosar",
    "endiosasen": "endiosar",
    "endiosases": "endiosar",
    "endiosaste": "endiosar",
    "endiosasteis": "endiosar",
    "endiose": "endiosar",
    "endiosemos": "endiosar",
    "endiosen": "endiosar",
    "endioses": "endiosar",
    "endioso": "endiosar",
    "endiosábamos": "endiosar",
    "endiosáis": "endiosar",
    "endiosáramos": "endiosar",
    "endiosáremos": "endiosar",
    "endiosásemos": "endiosar",
    "endiosé": "endiosar",
    "endioséis": "endiosar",
    "endiosó": "endiosar",
    "endita": "enditar",
    "enditaba": "enditar",
    "enditabais": "enditar",
    "enditaban": "enditar",
    "enditabas": "enditar",
    "enditad": "enditar",
    "enditada": "enditar",
    "enditadas": "enditar",
    "enditado": "enditar",
    "enditados": "enditar",
    "enditamos": "enditar",
    "enditan": "enditar",
    "enditando": "enditar",
    "enditara": "enditar",
    "enditarais": "enditar",
    "enditaran": "enditar",
    "enditaras": "enditar",
    "enditare": "enditar",
    "enditareis": "enditar",
    "enditaremos": "enditar",
    "enditaren": "enditar",
    "enditares": "enditar",
    "enditaron": "enditar",
    "enditará": "enditar",
    "enditarán": "enditar",
    "enditarás": "enditar",
    "enditaré": "enditar",
    "enditaréis": "enditar",
    "enditaría": "enditar",
    "enditaríais": "enditar",
    "enditaríamos": "enditar",
    "enditarían": "enditar",
    "enditarías": "enditar",
    "enditas": "enditar",
    "enditase": "enditar",
    "enditaseis": "enditar",
    "enditasen": "enditar",
    "enditases": "enditar",
    "enditaste": "enditar",
    "enditasteis": "enditar",
    "endite": "enditar",
    "enditemos": "enditar",
    "enditen": "enditar",
    "endites": "enditar",
    "endito": "enditar",
    "enditábamos": "enditar",
    "enditáis": "enditar",
    "enditáramos": "enditar",
    "enditáremos": "enditar",
    "enditásemos": "enditar",
    "endité": "enditar",
    "enditéis": "enditar",
    "enditó": "enditar",
    "endivias": "endivia",
    "endiña": "endiñar",
    "endiñaba": "endiñar",
    "endiñabais": "endiñar",
    "endiñaban": "endiñar",
    "endiñabas": "endiñar",
    "endiñad": "endiñar",
    "endiñada": "endiñar",
    "endiñadas": "endiñar",
    "endiñado": "endiñar",
    "endiñados": "endiñar",
    "endiñamos": "endiñar",
    "endiñan": "endiñar",
    "endiñando": "endiñar",
    "endiñara": "endiñar",
    "endiñarais": "endiñar",
    "endiñaran": "endiñar",
    "endiñaras": "endiñar",
    "endiñare": "endiñar",
    "endiñareis": "endiñar",
    "endiñaremos": "endiñar",
    "endiñaren": "endiñar",
    "endiñares": "endiñar",
    "endiñaron": "endiñar",
    "endiñará": "endiñar",
    "endiñarán": "endiñar",
    "endiñarás": "endiñar",
    "endiñaré": "endiñar",
    "endiñaréis": "endiñar",
    "endiñaría": "endiñar",
    "endiñaríais": "endiñar",
    "endiñaríamos": "endiñar",
    "endiñarían": "endiñar",
    "endiñarías": "endiñar",
    "endiñas": "endiñar",
    "endiñase": "endiñar",
    "endiñaseis": "endiñar",
    "endiñasen": "endiñar",
    "endiñases": "endiñar",
    "endiñaste": "endiñar",
    "endiñasteis": "endiñar",
    "endiñe": "endiñar",
    "endiñemos": "endiñar",
    "endiñen": "endiñar",
    "endiñes": "endiñar",
    "endiño": "endiñar",
    "endiñábamos": "endiñar",
    "endiñáis": "endiñar",
    "endiñáramos": "endiñar",
    "endiñáremos": "endiñar",
    "endiñásemos": "endiñar",
    "endiñé": "endiñar",
    "endiñéis": "endiñar",
    "endiñó": "endiñar",
    "endoblastos": "endoblasto",
    "endobles": "endoble",
    "endocardios": "endocardio",
    "endocarpos": "endocarpo",
    "endocinética": "endocinético",
    "endocinéticas": "endocinético",
    "endocinéticos": "endocinético",
    "endoclinales": "endoclinal",
    "endocondrales": "endocondral",
    "endocrina": "endocrino",
    "endocrinas": "endocrino",
    "endocrinologías": "endocrinología",
    "endocrinológica": "endocrinológico",
    "endocrinológicas": "endocrinológico",
    "endocrinológicos": "endocrinológico",
    "endocrinopatías": "endocrinopatía",
    "endocrinos": "endocrino",
    "endocrinóloga": "endocrinólogo",
    "endocrinólogas": "endocrinólogo",
    "endocrinólogos": "endocrinólogo",
    "endocráneos": "endocráneo",
    "endodermos": "endodermo",
    "endodina": "endodino",
    "endodinamomorfa": "endodinamomorfo",
    "endodinamomorfas": "endodinamomorfo",
    "endodinamomorfos": "endodinamomorfo",
    "endodinamógena": "endodinamógeno",
    "endodinamógenas": "endodinamógeno",
    "endodinamógenos": "endodinamógeno",
    "endodinas": "endodino",
    "endodinos": "endodino",
    "endodoncias": "endodoncia",
    "endodontologías": "endodontología",
    "endodérmica": "endodérmico",
    "endodérmicas": "endodérmico",
    "endodérmicos": "endodérmico",
    "endodóncica": "endodóncico",
    "endodóncicas": "endodóncico",
    "endodóncicos": "endodóncico",
    "endoesqueletos": "endoesqueleto",
    "endogamias": "endogamia",
    "endognacias": "endognacia",
    "endognática": "endognático",
    "endognáticas": "endognático",
    "endognáticos": "endognático",
    "endogámica": "endogámico",
    "endogámicas": "endogámico",
    "endogámicos": "endogámico",
    "endolinfas": "endolinfa",
    "endolinfática": "endolinfático",
    "endolinfáticas": "endolinfático",
    "endolinfáticos": "endolinfático",
    "endometrios": "endometrio",
    "endometrías": "endometría",
    "endominga": "endomingar",
    "endomingaba": "endomingar",
    "endomingabais": "endomingar",
    "endomingaban": "endomingar",
    "endomingabas": "endomingar",
    "endomingad": "endomingar",
    "endomingada": "endomingar",
    "endomingadas": "endomingar",
    "endomingado": "endomingar",
    "endomingados": "endomingar",
    "endomingamos": "endomingar",
    "endomingan": "endomingar",
    "endomingando": "endomingar",
    "endomingara": "endomingar",
    "endomingarais": "endomingar",
    "endomingaran": "endomingar",
    "endomingaras": "endomingar",
    "endomingare": "endomingar",
    "endomingareis": "endomingar",
    "endomingaremos": "endomingar",
    "endomingaren": "endomingar",
    "endomingares": "endomingar",
    "endomingaron": "endomingar",
    "endomingará": "endomingar",
    "endomingarán": "endomingar",
    "endomingarás": "endomingar",
    "endomingaré": "endomingar",
    "endomingaréis": "endomingar",
    "endomingaría": "endomingar",
    "endomingaríais": "endomingar",
    "endomingaríamos": "endomingar",
    "endomingarían": "endomingar",
    "endomingarías": "endomingar",
    "endomingas": "endomingar",
    "endomingase": "endomingar",
    "endomingaseis": "endomingar",
    "endomingasen": "endomingar",
    "endomingases": "endomingar",
    "endomingaste": "endomingar",
    "endomingasteis": "endomingar",
    "endomingo": "endomingar",
    "endomingue": "endomingar",
    "endominguemos": "endomingar",
    "endominguen": "endomingar",
    "endomingues": "endomingar",
    "endomingué": "endomingar",
    "endominguéis": "endomingar",
    "endomingábamos": "endomingar",
    "endomingáis": "endomingar",
    "endomingáramos": "endomingar",
    "endomingáremos": "endomingar",
    "endomingásemos": "endomingar",
    "endomingó": "endomingar",
    "endomorfismos": "endomorfismo",
    "endonucleasas": "endonucleasa",
    "endoparásitos": "endoparásito",
    "endopelviana": "endopelviano",
    "endopelvianas": "endopelviano",
    "endopelvianos": "endopelviano",
    "endoplasmas": "endoplasma",
    "endoplasmática": "endoplasmático",
    "endoplasmáticas": "endoplasmático",
    "endoplasmáticos": "endoplasmático",
    "endoplásmica": "endoplásmico",
    "endoplásmicas": "endoplásmico",
    "endoplásmicos": "endoplásmico",
    "endoproctos": "endoprocto",
    "endorreica": "endorreico",
    "endorreicas": "endorreico",
    "endorreicos": "endorreico",
    "endorreísmos": "endorreísmo",
    "endorsos": "endorso",
    "endosa": "endosar",
    "endosaba": "endosar",
    "endosabais": "endosar",
    "endosaban": "endosar",
    "endosabas": "endosar",
    "endosables": "endosable",
    "endosad": "endosar",
    "endosada": "endosar",
    "endosadas": "endosar",
    "endosado": "endosar",
    "endosadora": "endosador",
    "endosadoras": "endosador",
    "endosadores": "endosador",
    "endosados": "endosar",
    "endosamos": "endosar",
    "endosan": "endosar",
    "endosando": "endosar",
    "endosantes": "endosante",
    "endosara": "endosar",
    "endosarais": "endosar",
    "endosaran": "endosar",
    "endosaras": "endosar",
    "endosare": "endosar",
    "endosareis": "endosar",
    "endosaremos": "endosar",
    "endosaren": "endosar",
    "endosares": "endosar",
    "endosaron": "endosar",
    "endosará": "endosar",
    "endosarán": "endosar",
    "endosarás": "endosar",
    "endosaré": "endosar",
    "endosaréis": "endosar",
    "endosaría": "endosar",
    "endosaríais": "endosar",
    "endosaríamos": "endosar",
    "endosarían": "endosar",
    "endosarías": "endosar",
    "endosas": "endosar",
    "endosase": "endosar",
    "endosaseis": "endosar",
    "endosasen": "endosar",
    "endosases": "endosar",
    "endosaste": "endosar",
    "endosasteis": "endosar",
    "endosataria": "endosatario",
    "endosatarias": "endosatario",
    "endosatarios": "endosatario",
    "endoscopias": "endoscopia",
    "endoscopios": "endoscopio",
    "endoscópica": "endoscópico",
    "endoscópicas": "endoscópico",
    "endoscópicos": "endoscópico",
    "endose": "endosar",
    "endosela": "endoselar",
    "endoselaba": "endoselar",
    "endoselabais": "endoselar",
    "endoselaban": "endoselar",
    "endoselabas": "endoselar",
    "endoselad": "endoselar",
    "endoselada": "endoselar",
    "endoseladas": "endoselar",
    "endoselado": "endoselar",
    "endoselados": "endoselar",
    "endoselamos": "endoselar",
    "endoselan": "endoselar",
    "endoselando": "endoselar",
    "endoselara": "endoselar",
    "endoselarais": "endoselar",
    "endoselaran": "endoselar",
    "endoselaras": "endoselar",
    "endoselare": "endoselar",
    "endoselareis": "endoselar",
    "endoselaremos": "endoselar",
    "endoselaren": "endoselar",
    "endoselares": "endoselar",
    "endoselaron": "endoselar",
    "endoselará": "endoselar",
    "endoselarán": "endoselar",
    "endoselarás": "endoselar",
    "endoselaré": "endoselar",
    "endoselaréis": "endoselar",
    "endoselaría": "endoselar",
    "endoselaríais": "endoselar",
    "endoselaríamos": "endoselar",
    "endoselarían": "endoselar",
    "endoselarías": "endoselar",
    "endoselas": "endoselar",
    "endoselase": "endoselar",
    "endoselaseis": "endoselar",
    "endoselasen": "endoselar",
    "endoselases": "endoselar",
    "endoselaste": "endoselar",
    "endoselasteis": "endoselar",
    "endosele": "endoselar",
    "endoselemos": "endoselar",
    "endoselen": "endoselar",
    "endoseles": "endoselar",
    "endoselo": "endoselar",
    "endoselábamos": "endoselar",
    "endoseláis": "endoselar",
    "endoseláramos": "endoselar",
    "endoseláremos": "endoselar",
    "endoselásemos": "endoselar",
    "endoselé": "endoselar",
    "endoseléis": "endoselar",
    "endoseló": "endoselar",
    "endosemos": "endosar",
    "endosen": "endosar",
    "endoses": "endose",
    "endosferas": "endosfera",
    "endoso": "endosar",
    "endosomas": "endosoma",
    "endosos": "endoso",
    "endospermas": "endosperma",
    "endosqueletos": "endosqueleto",
    "endostales": "endostal",
    "endostios": "endostio",
    "endosábamos": "endosar",
    "endosáis": "endosar",
    "endosáramos": "endosar",
    "endosáremos": "endosar",
    "endosásemos": "endosar",
    "endosé": "endosar",
    "endoséis": "endosar",
    "endosó": "endosar",
    "endoteliales": "endotelial",
    "endotelios": "endotelio",
    "endotorácica": "endotorácico",
    "endotorácicas": "endotorácico",
    "endotorácicos": "endotorácico",
    "endotoxinas": "endotoxina",
    "endotraqueales": "endotraqueal",
    "endotérmica": "endotérmico",
    "endotérmicas": "endotérmico",
    "endotérmicos": "endotérmico",
    "endotóxica": "endotóxico",
    "endotóxicas": "endotóxico",
    "endotóxicos": "endotóxico",
    "endovenosa": "endovenoso",
    "endovenosas": "endovenoso",
    "endovenosos": "endovenoso",
    "endriagos": "endriago",
    "endrina": "endrino",
    "endrinas": "endrino",
    "endrinos": "endrino",
    "endroga": "endrogar",
    "endrogaba": "endrogar",
    "endrogabais": "endrogar",
    "endrogaban": "endrogar",
    "endrogabas": "endrogar",
    "endrogad": "endrogar",
    "endrogada": "endrogar",
    "endrogadas": "endrogar",
    "endrogado": "endrogar",
    "endrogados": "endrogar",
    "endrogamos": "endrogar",
    "endrogan": "endrogar",
    "endrogando": "endrogar",
    "endrogara": "endrogar",
    "endrogarais": "endrogar",
    "endrogaran": "endrogar",
    "endrogaras": "endrogar",
    "endrogare": "endrogar",
    "endrogareis": "endrogar",
    "endrogaremos": "endrogar",
    "endrogaren": "endrogar",
    "endrogares": "endrogar",
    "endrogaron": "endrogar",
    "endrogará": "endrogar",
    "endrogarán": "endrogar",
    "endrogarás": "endrogar",
    "endrogaré": "endrogar",
    "endrogaréis": "endrogar",
    "endrogaría": "endrogar",
    "endrogaríais": "endrogar",
    "endrogaríamos": "endrogar",
    "endrogarían": "endrogar",
    "endrogarías": "endrogar",
    "endrogas": "endrogar",
    "endrogase": "endrogar",
    "endrogaseis": "endrogar",
    "endrogasen": "endrogar",
    "endrogases": "endrogar",
    "endrogaste": "endrogar",
    "endrogasteis": "endrogar",
    "endrogo": "endrogar",
    "endrogue": "endrogar",
    "endroguemos": "endrogar",
    "endroguen": "endrogar",
    "endrogues": "endrogar",
    "endrogué": "endrogar",
    "endroguéis": "endrogar",
    "endrogábamos": "endrogar",
    "endrogáis": "endrogar",
    "endrogáramos": "endrogar",
    "endrogáremos": "endrogar",
    "endrogásemos": "endrogar",
    "endrogó": "endrogar",
    "endulce": "endulzar",
    "endulcemos": "endulzar",
    "endulcen": "endulzar",
    "endulces": "endulzar",
    "endulcé": "endulzar",
    "endulcéis": "endulzar",
    "endulza": "endulzar",
    "endulzaba": "endulzar",
    "endulzabais": "endulzar",
    "endulzaban": "endulzar",
    "endulzabas": "endulzar",
    "endulzad": "endulzar",
    "endulzada": "endulzar",
    "endulzadas": "endulzar",
    "endulzado": "endulzar",
    "endulzados": "endulzar",
    "endulzaduras": "endulzadura",
    "endulzamientos": "endulzamiento",
    "endulzamos": "endulzar",
    "endulzan": "endulzar",
    "endulzando": "endulzar",
    "endulzantes": "endulzante",
    "endulzara": "endulzar",
    "endulzarais": "endulzar",
    "endulzaran": "endulzar",
    "endulzaras": "endulzar",
    "endulzare": "endulzar",
    "endulzareis": "endulzar",
    "endulzaremos": "endulzar",
    "endulzaren": "endulzar",
    "endulzares": "endulzar",
    "endulzaron": "endulzar",
    "endulzará": "endulzar",
    "endulzarán": "endulzar",
    "endulzarás": "endulzar",
    "endulzaré": "endulzar",
    "endulzaréis": "endulzar",
    "endulzaría": "endulzar",
    "endulzaríais": "endulzar",
    "endulzaríamos": "endulzar",
    "endulzarían": "endulzar",
    "endulzarías": "endulzar",
    "endulzas": "endulzar",
    "endulzase": "endulzar",
    "endulzaseis": "endulzar",
    "endulzasen": "endulzar",
    "endulzases": "endulzar",
    "endulzaste": "endulzar",
    "endulzasteis": "endulzar",
    "endulzo": "endulzar",
    "endulzábamos": "endulzar",
    "endulzáis": "endulzar",
    "endulzáramos": "endulzar",
    "endulzáremos": "endulzar",
    "endulzásemos": "endulzar",
    "endulzó": "endulzar",
    "endura": "endurar",
    "enduraba": "endurar",
    "endurabais": "endurar",
    "enduraban": "endurar",
    "endurabas": "endurar",
    "endurad": "endurar",
    "endurada": "endurar",
    "enduradas": "endurar",
    "endurado": "endurar",
    "endurados": "endurar",
    "enduramos": "endurar",
    "enduran": "endurar",
    "endurando": "endurar",
    "endurara": "endurar",
    "endurarais": "endurar",
    "enduraran": "endurar",
    "enduraras": "endurar",
    "endurare": "endurar",
    "endurareis": "endurar",
    "enduraremos": "endurar",
    "enduraren": "endurar",
    "endurares": "endurar",
    "enduraron": "endurar",
    "endurará": "endurar",
    "endurarán": "endurar",
    "endurarás": "endurar",
    "enduraré": "endurar",
    "enduraréis": "endurar",
    "enduraría": "endurar",
    "enduraríais": "endurar",
    "enduraríamos": "endurar",
    "endurarían": "endurar",
    "endurarías": "endurar",
    "enduras": "endurar",
    "endurase": "endurar",
    "enduraseis": "endurar",
    "endurasen": "endurar",
    "endurases": "endurar",
    "enduraste": "endurar",
    "endurasteis": "endurar",
    "endure": "endurar",
    "endurece": "endurecer",
    "endureced": "endurecer",
    "endurecedora": "endurecedor",
    "endurecedoras": "endurecedor",
    "endurecedores": "endurecedor",
    "endurecemos": "endurecer",
    "endurecen": "endurecer",
    "endureceremos": "endurecer",
    "endurecerá": "endurecer",
    "endurecerán": "endurecer",
    "endurecerás": "endurecer",
    "endureceré": "endurecer",
    "endureceréis": "endurecer",
    "endurecería": "endurecer",
    "endureceríais": "endurecer",
    "endureceríamos": "endurecer",
    "endurecerían": "endurecer",
    "endurecerías": "endurecer",
    "endureces": "endurecer",
    "endurecida": "endurecer",
    "endurecidas": "endurecer",
    "endurecido": "endurecer",
    "endurecidos": "endurecer",
    "endureciendo": "endurecer",
    "endureciera": "endurecer",
    "endurecierais": "endurecer",
    "endurecieran": "endurecer",
    "endurecieras": "endurecer",
    "endureciere": "endurecer",
    "endureciereis": "endurecer",
    "endurecieren": "endurecer",
    "endurecieres": "endurecer",
    "endurecieron": "endurecer",
    "endureciese": "endurecer",
    "endurecieseis": "endurecer",
    "endureciesen": "endurecer",
    "endurecieses": "endurecer",
    "endurecimientos": "endurecimiento",
    "endurecimos": "endurecer",
    "endureciste": "endurecer",
    "endurecisteis": "endurecer",
    "endureciéramos": "endurecer",
    "endureciéremos": "endurecer",
    "endureciésemos": "endurecer",
    "endureció": "endurecer",
    "endurecéis": "endurecer",
    "endurecí": "endurecer",
    "endurecía": "endurecer",
    "endurecíais": "endurecer",
    "endurecíamos": "endurecer",
    "endurecían": "endurecer",
    "endurecías": "endurecer",
    "enduremos": "endurar",
    "enduren": "endurar",
    "endures": "endurar",
    "endurezca": "endurecer",
    "endurezcamos": "endurecer",
    "endurezcan": "endurecer",
    "endurezcas": "endurecer",
    "endurezco": "endurecer",
    "endurezcáis": "endurecer",
    "enduro": "endurar",
    "endurábamos": "endurar",
    "enduráis": "endurar",
    "enduráramos": "endurar",
    "enduráremos": "endurar",
    "endurásemos": "endurar",
    "enduré": "endurar",
    "enduréis": "endurar",
    "enduró": "endurar",
    "endécadas": "endécada",
    "endémica": "endémico",
    "endémicas": "endémico",
    "endémicos": "endémico",
    "endófitos": "endófito",
    "endógena": "endógeno",
    "endógenas": "endógeno",
    "endógenos": "endógeno",
    "endómetros": "endómetro",
    "endósporas": "endóspora",
    "endótrofa": "endótrofo",
    "endótrofas": "endótrofo",
    "endótrofos": "endótrofo",
    "ene.": "enero",
    "eneas": "enea",
    "eneasílaba": "eneasílabo",
    "eneasílabas": "eneasílabo",
    "eneasílabos": "eneasílabo",
    "enebrales": "enebral",
    "enebros": "enebro",
    "enechada": "enechado",
    "enechadas": "enechado",
    "enechados": "enechado",
    "eneldos": "eneldo",
    "enemas": "enema",
    "enemiga": "enemigo",
    "enemigas": "enemigo",
    "enemigos": "enemigo",
    "enemista": "enemistar",
    "enemistaba": "enemistar",
    "enemistabais": "enemistar",
    "enemistaban": "enemistar",
    "enemistabas": "enemistar",
    "enemistad": "enemistar",
    "enemistada": "enemistar",
    "enemistadas": "enemistar",
    "enemistades": "enemistad",
    "enemistado": "enemistar",
    "enemistados": "enemistar",
    "enemistamos": "enemistar",
    "enemistan": "enemistar",
    "enemistando": "enemistar",
    "enemistara": "enemistar",
    "enemistarais": "enemistar",
    "enemistaran": "enemistar",
    "enemistaras": "enemistar",
    "enemistare": "enemistar",
    "enemistareis": "enemistar",
    "enemistaremos": "enemistar",
    "enemistaren": "enemistar",
    "enemistares": "enemistar",
    "enemistaron": "enemistar",
    "enemistará": "enemistar",
    "enemistarán": "enemistar",
    "enemistarás": "enemistar",
    "enemistaré": "enemistar",
    "enemistaréis": "enemistar",
    "enemistaría": "enemistar",
    "enemistaríais": "enemistar",
    "enemistaríamos": "enemistar",
    "enemistarían": "enemistar",
    "enemistarías": "enemistar",
    "enemistas": "enemistar",
    "enemistase": "enemistar",
    "enemistaseis": "enemistar",
    "enemistasen": "enemistar",
    "enemistases": "enemistar",
    "enemistaste": "enemistar",
    "enemistasteis": "enemistar",
    "enemiste": "enemistar",
    "enemistemos": "enemistar",
    "enemisten": "enemistar",
    "enemistes": "enemistar",
    "enemisto": "enemistar",
    "enemistábamos": "enemistar",
    "enemistáis": "enemistar",
    "enemistáramos": "enemistar",
    "enemistáremos": "enemistar",
    "enemistásemos": "enemistar",
    "enemisté": "enemistar",
    "enemistéis": "enemistar",
    "enemistó": "enemistar",
    "eneolíticos": "eneolítico",
    "energética": "energético",
    "energéticas": "energético",
    "energéticos": "energético",
    "energías": "energía",
    "energúmena": "energúmeno",
    "energúmenas": "energúmeno",
    "energúmenos": "energúmeno",
    "eneros": "enero",
    "enerva": "enervar",
    "enervaba": "enervar",
    "enervabais": "enervar",
    "enervaban": "enervar",
    "enervabas": "enervar",
    "enervaciones": "enervación",
    "enervad": "enervar",
    "enervada": "enervar",
    "enervadas": "enervar",
    "enervado": "enervar",
    "enervadora": "enervador",
    "enervadoras": "enervador",
    "enervadores": "enervador",
    "enervados": "enervar",
    "enervamientos": "enervamiento",
    "enervamos": "enervar",
    "enervan": "enervar",
    "enervando": "enervar",
    "enervara": "enervar",
    "enervarais": "enervar",
    "enervaran": "enervar",
    "enervaras": "enervar",
    "enervare": "enervar",
    "enervareis": "enervar",
    "enervaremos": "enervar",
    "enervaren": "enervar",
    "enervares": "enervar",
    "enervaron": "enervar",
    "enervará": "enervar",
    "enervarán": "enervar",
    "enervarás": "enervar",
    "enervaré": "enervar",
    "enervaréis": "enervar",
    "enervaría": "enervar",
    "enervaríais": "enervar",
    "enervaríamos": "enervar",
    "enervarían": "enervar",
    "enervarías": "enervar",
    "enervas": "enervar",
    "enervase": "enervar",
    "enervaseis": "enervar",
    "enervasen": "enervar",
    "enervases": "enervar",
    "enervaste": "enervar",
    "enervasteis": "enervar",
    "enerve": "enervar",
    "enervemos": "enervar",
    "enerven": "enervar",
    "enerves": "enervar",
    "enervo": "enervar",
    "enervábamos": "enervar",
    "enerváis": "enervar",
    "enerváramos": "enervar",
    "enerváremos": "enervar",
    "enervásemos": "enervar",
    "enervé": "enervar",
    "enervéis": "enervar",
    "enervó": "enervar",
    "enes": "ene",
    "eneágonos": "eneágono",
    "enfada": "enfadar",
    "enfadaba": "enfadar",
    "enfadabais": "enfadar",
    "enfadaban": "enfadar",
    "enfadabas": "enfadar",
    "enfadad": "enfadar",
    "enfadada": "enfadar",
    "enfadadas": "enfadar",
    "enfadadiza": "enfadadizo",
    "enfadadizas": "enfadadizo",
    "enfadadizos": "enfadadizo",
    "enfadado": "enfadar",
    "enfadados": "enfadar",
    "enfadamientos": "enfadamiento",
    "enfadamos": "enfadar",
    "enfadan": "enfadar",
    "enfadando": "enfadar",
    "enfadara": "enfadar",
    "enfadarais": "enfadar",
    "enfadaran": "enfadar",
    "enfadaras": "enfadar",
    "enfadare": "enfadar",
    "enfadareis": "enfadar",
    "enfadaremos": "enfadar",
    "enfadaren": "enfadar",
    "enfadares": "enfadar",
    "enfadaron": "enfadar",
    "enfadará": "enfadar",
    "enfadarán": "enfadar",
    "enfadarás": "enfadar",
    "enfadaré": "enfadar",
    "enfadaréis": "enfadar",
    "enfadaría": "enfadar",
    "enfadaríais": "enfadar",
    "enfadaríamos": "enfadar",
    "enfadarían": "enfadar",
    "enfadarías": "enfadar",
    "enfadas": "enfadar",
    "enfadase": "enfadar",
    "enfadaseis": "enfadar",
    "enfadasen": "enfadar",
    "enfadases": "enfadar",
    "enfadaste": "enfadar",
    "enfadasteis": "enfadar",
    "enfade": "enfadar",
    "enfademos": "enfadar",
    "enfaden": "enfadar",
    "enfades": "enfadar",
    "enfado": "enfadar",
    "enfados": "enfado",
    "enfadosa": "enfadoso",
    "enfadosas": "enfadoso",
    "enfadosos": "enfadoso",
    "enfadábamos": "enfadar",
    "enfadáis": "enfadar",
    "enfadáramos": "enfadar",
    "enfadáremos": "enfadar",
    "enfadásemos": "enfadar",
    "enfadé": "enfadar",
    "enfadéis": "enfadar",
    "enfadó": "enfadar",
    "enfaenada": "enfaenado",
    "enfaenadas": "enfaenado",
    "enfaenados": "enfaenado",
    "enfalda": "enfaldar",
    "enfaldaba": "enfaldar",
    "enfaldabais": "enfaldar",
    "enfaldaban": "enfaldar",
    "enfaldabas": "enfaldar",
    "enfaldad": "enfaldar",
    "enfaldada": "enfaldar",
    "enfaldadas": "enfaldar",
    "enfaldado": "enfaldar",
    "enfaldadores": "enfaldador",
    "enfaldados": "enfaldar",
    "enfaldamos": "enfaldar",
    "enfaldan": "enfaldar",
    "enfaldando": "enfaldar",
    "enfaldara": "enfaldar",
    "enfaldarais": "enfaldar",
    "enfaldaran": "enfaldar",
    "enfaldaras": "enfaldar",
    "enfaldare": "enfaldar",
    "enfaldareis": "enfaldar",
    "enfaldaremos": "enfaldar",
    "enfaldaren": "enfaldar",
    "enfaldares": "enfaldar",
    "enfaldaron": "enfaldar",
    "enfaldará": "enfaldar",
    "enfaldarán": "enfaldar",
    "enfaldarás": "enfaldar",
    "enfaldaré": "enfaldar",
    "enfaldaréis": "enfaldar",
    "enfaldaría": "enfaldar",
    "enfaldaríais": "enfaldar",
    "enfaldaríamos": "enfaldar",
    "enfaldarían": "enfaldar",
    "enfaldarías": "enfaldar",
    "enfaldas": "enfaldar",
    "enfaldase": "enfaldar",
    "enfaldaseis": "enfaldar",
    "enfaldasen": "enfaldar",
    "enfaldases": "enfaldar",
    "enfaldaste": "enfaldar",
    "enfaldasteis": "enfaldar",
    "enfalde": "enfaldar",
    "enfaldemos": "enfaldar",
    "enfalden": "enfaldar",
    "enfaldes": "enfaldar",
    "enfaldo": "enfaldar",
    "enfaldos": "enfaldo",
    "enfaldábamos": "enfaldar",
    "enfaldáis": "enfaldar",
    "enfaldáramos": "enfaldar",
    "enfaldáremos": "enfaldar",
    "enfaldásemos": "enfaldar",
    "enfaldé": "enfaldar",
    "enfaldéis": "enfaldar",
    "enfaldó": "enfaldar",
    "enfanga": "enfangar",
    "enfangaba": "enfangar",
    "enfangabais": "enfangar",
    "enfangaban": "enfangar",
    "enfangabas": "enfangar",
    "enfangad": "enfangar",
    "enfangada": "enfangar",
    "enfangadas": "enfangar",
    "enfangado": "enfangar",
    "enfangados": "enfangar",
    "enfangamientos": "enfangamiento",
    "enfangamos": "enfangar",
    "enfangan": "enfangar",
    "enfangando": "enfangar",
    "enfangara": "enfangar",
    "enfangarais": "enfangar",
    "enfangaran": "enfangar",
    "enfangaras": "enfangar",
    "enfangare": "enfangar",
    "enfangareis": "enfangar",
    "enfangaremos": "enfangar",
    "enfangaren": "enfangar",
    "enfangares": "enfangar",
    "enfangaron": "enfangar",
    "enfangará": "enfangar",
    "enfangarán": "enfangar",
    "enfangarás": "enfangar",
    "enfangaré": "enfangar",
    "enfangaréis": "enfangar",
    "enfangaría": "enfangar",
    "enfangaríais": "enfangar",
    "enfangaríamos": "enfangar",
    "enfangarían": "enfangar",
    "enfangarías": "enfangar",
    "enfangas": "enfangar",
    "enfangase": "enfangar",
    "enfangaseis": "enfangar",
    "enfangasen": "enfangar",
    "enfangases": "enfangar",
    "enfangaste": "enfangar",
    "enfangasteis": "enfangar",
    "enfango": "enfangar",
    "enfangue": "enfangar",
    "enfanguemos": "enfangar",
    "enfanguen": "enfangar",
    "enfangues": "enfangar",
    "enfangué": "enfangar",
    "enfanguéis": "enfangar",
    "enfangábamos": "enfangar",
    "enfangáis": "enfangar",
    "enfangáramos": "enfangar",
    "enfangáremos": "enfangar",
    "enfangásemos": "enfangar",
    "enfangó": "enfangar",
    "enfarda": "enfardar",
    "enfardaba": "enfardar",
    "enfardabais": "enfardar",
    "enfardaban": "enfardar",
    "enfardabas": "enfardar",
    "enfardad": "enfardar",
    "enfardada": "enfardar",
    "enfardadas": "enfardar",
    "enfardado": "enfardar",
    "enfardados": "enfardar",
    "enfardamos": "enfardar",
    "enfardan": "enfardar",
    "enfardando": "enfardar",
    "enfardara": "enfardar",
    "enfardarais": "enfardar",
    "enfardaran": "enfardar",
    "enfardaras": "enfardar",
    "enfardare": "enfardar",
    "enfardareis": "enfardar",
    "enfardaremos": "enfardar",
    "enfardaren": "enfardar",
    "enfardares": "enfardar",
    "enfardaron": "enfardar",
    "enfardará": "enfardar",
    "enfardarán": "enfardar",
    "enfardarás": "enfardar",
    "enfardaré": "enfardar",
    "enfardaréis": "enfardar",
    "enfardaría": "enfardar",
    "enfardaríais": "enfardar",
    "enfardaríamos": "enfardar",
    "enfardarían": "enfardar",
    "enfardarías": "enfardar",
    "enfardas": "enfardar",
    "enfardase": "enfardar",
    "enfardaseis": "enfardar",
    "enfardasen": "enfardar",
    "enfardases": "enfardar",
    "enfardaste": "enfardar",
    "enfardasteis": "enfardar",
    "enfarde": "enfardar",
    "enfardela": "enfardelar",
    "enfardelaba": "enfardelar",
    "enfardelabais": "enfardelar",
    "enfardelaban": "enfardelar",
    "enfardelabas": "enfardelar",
    "enfardelad": "enfardelar",
    "enfardelada": "enfardelar",
    "enfardeladas": "enfardelar",
    "enfardelado": "enfardelar",
    "enfardelados": "enfardelar",
    "enfardelamos": "enfardelar",
    "enfardelan": "enfardelar",
    "enfardelando": "enfardelar",
    "enfardelara": "enfardelar",
    "enfardelarais": "enfardelar",
    "enfardelaran": "enfardelar",
    "enfardelaras": "enfardelar",
    "enfardelare": "enfardelar",
    "enfardelareis": "enfardelar",
    "enfardelaremos": "enfardelar",
    "enfardelaren": "enfardelar",
    "enfardelares": "enfardelar",
    "enfardelaron": "enfardelar",
    "enfardelará": "enfardelar",
    "enfardelarán": "enfardelar",
    "enfardelarás": "enfardelar",
    "enfardelaré": "enfardelar",
    "enfardelaréis": "enfardelar",
    "enfardelaría": "enfardelar",
    "enfardelaríais": "enfardelar",
    "enfardelaríamos": "enfardelar",
    "enfardelarían": "enfardelar",
    "enfardelarías": "enfardelar",
    "enfardelas": "enfardelar",
    "enfardelase": "enfardelar",
    "enfardelaseis": "enfardelar",
    "enfardelasen": "enfardelar",
    "enfardelases": "enfardelar",
    "enfardelaste": "enfardelar",
    "enfardelasteis": "enfardelar",
    "enfardele": "enfardelar",
    "enfardelemos": "enfardelar",
    "enfardelen": "enfardelar",
    "enfardeles": "enfardelar",
    "enfardelo": "enfardelar",
    "enfardelábamos": "enfardelar",
    "enfardeláis": "enfardelar",
    "enfardeláramos": "enfardelar",
    "enfardeláremos": "enfardelar",
    "enfardelásemos": "enfardelar",
    "enfardelé": "enfardelar",
    "enfardeléis": "enfardelar",
    "enfardeló": "enfardelar",
    "enfardemos": "enfardar",
    "enfarden": "enfardar",
    "enfardes": "enfardar",
    "enfardo": "enfardar",
    "enfardábamos": "enfardar",
    "enfardáis": "enfardar",
    "enfardáramos": "enfardar",
    "enfardáremos": "enfardar",
    "enfardásemos": "enfardar",
    "enfardé": "enfardar",
    "enfardéis": "enfardar",
    "enfardó": "enfardar",
    "enfatice": "enfatizar",
    "enfaticemos": "enfatizar",
    "enfaticen": "enfatizar",
    "enfatices": "enfatizar",
    "enfaticé": "enfatizar",
    "enfaticéis": "enfatizar",
    "enfatiza": "enfatizar",
    "enfatizaba": "enfatizar",
    "enfatizabais": "enfatizar",
    "enfatizaban": "enfatizar",
    "enfatizabas": "enfatizar",
    "enfatizaciones": "enfatización",
    "enfatizad": "enfatizar",
    "enfatizada": "enfatizar",
    "enfatizadas": "enfatizar",
    "enfatizado": "enfatizar",
    "enfatizados": "enfatizar",
    "enfatizamos": "enfatizar",
    "enfatizan": "enfatizar",
    "enfatizando": "enfatizar",
    "enfatizara": "enfatizar",
    "enfatizarais": "enfatizar",
    "enfatizaran": "enfatizar",
    "enfatizaras": "enfatizar",
    "enfatizare": "enfatizar",
    "enfatizareis": "enfatizar",
    "enfatizaremos": "enfatizar",
    "enfatizaren": "enfatizar",
    "enfatizares": "enfatizar",
    "enfatizaron": "enfatizar",
    "enfatizará": "enfatizar",
    "enfatizarán": "enfatizar",
    "enfatizarás": "enfatizar",
    "enfatizaré": "enfatizar",
    "enfatizaréis": "enfatizar",
    "enfatizaría": "enfatizar",
    "enfatizaríais": "enfatizar",
    "enfatizaríamos": "enfatizar",
    "enfatizarían": "enfatizar",
    "enfatizarías": "enfatizar",
    "enfatizas": "enfatizar",
    "enfatizase": "enfatizar",
    "enfatizaseis": "enfatizar",
    "enfatizasen": "enfatizar",
    "enfatizases": "enfatizar",
    "enfatizaste": "enfatizar",
    "enfatizasteis": "enfatizar",
    "enfatizo": "enfatizar",
    "enfatizábamos": "enfatizar",
    "enfatizáis": "enfatizar",
    "enfatizáramos": "enfatizar",
    "enfatizáremos": "enfatizar",
    "enfatizásemos": "enfatizar",
    "enfatizó": "enfatizar",
    "enfatuaba": "enfatuar",
    "enfatuabais": "enfatuar",
    "enfatuaban": "enfatuar",
    "enfatuabas": "enfatuar",
    "enfatuad": "enfatuar",
    "enfatuada": "enfatuar",
    "enfatuadas": "enfatuar",
    "enfatuado": "enfatuar",
    "enfatuados": "enfatuar",
    "enfatuamos": "enfatuar",
    "enfatuando": "enfatuar",
    "enfatuara": "enfatuar",
    "enfatuarais": "enfatuar",
    "enfatuaran": "enfatuar",
    "enfatuaras": "enfatuar",
    "enfatuare": "enfatuar",
    "enfatuareis": "enfatuar",
    "enfatuaremos": "enfatuar",
    "enfatuaren": "enfatuar",
    "enfatuares": "enfatuar",
    "enfatuaron": "enfatuar",
    "enfatuará": "enfatuar",
    "enfatuarán": "enfatuar",
    "enfatuarás": "enfatuar",
    "enfatuaré": "enfatuar",
    "enfatuaréis": "enfatuar",
    "enfatuaría": "enfatuar",
    "enfatuaríais": "enfatuar",
    "enfatuaríamos": "enfatuar",
    "enfatuarían": "enfatuar",
    "enfatuarías": "enfatuar",
    "enfatuase": "enfatuar",
    "enfatuaseis": "enfatuar",
    "enfatuasen": "enfatuar",
    "enfatuases": "enfatuar",
    "enfatuaste": "enfatuar",
    "enfatuasteis": "enfatuar",
    "enfatuemos": "enfatuar",
    "enfatuábamos": "enfatuar",
    "enfatuáis": "enfatuar",
    "enfatuáramos": "enfatuar",
    "enfatuáremos": "enfatuar",
    "enfatuásemos": "enfatuar",
    "enfatué": "enfatuar",
    "enfatuéis": "enfatuar",
    "enfatuó": "enfatuar",
    "enfatúa": "enfatuar",
    "enfatúan": "enfatuar",
    "enfatúas": "enfatuar",
    "enfatúe": "enfatuar",
    "enfatúen": "enfatuar",
    "enfatúes": "enfatuar",
    "enfatúo": "enfatuar",
    "enferma": "enfermo",
    "enfermaba": "enfermar",
    "enfermabais": "enfermar",
    "enfermaban": "enfermar",
    "enfermabas": "enfermar",
    "enfermad": "enfermar",
    "enfermada": "enfermar",
    "enfermadas": "enfermar",
    "enfermado": "enfermar",
    "enfermados": "enfermar",
    "enfermamos": "enfermar",
    "enferman": "enfermar",
    "enfermando": "enfermar",
    "enfermara": "enfermar",
    "enfermarais": "enfermar",
    "enfermaran": "enfermar",
    "enfermaras": "enfermar",
    "enfermare": "enfermar",
    "enfermareis": "enfermar",
    "enfermaremos": "enfermar",
    "enfermaren": "enfermar",
    "enfermares": "enfermar",
    "enfermaron": "enfermar",
    "enfermará": "enfermar",
    "enfermarán": "enfermar",
    "enfermarás": "enfermar",
    "enfermaré": "enfermar",
    "enfermaréis": "enfermar",
    "enfermaría": "enfermar",
    "enfermaríais": "enfermar",
    "enfermaríamos": "enfermar",
    "enfermarían": "enfermar",
    "enfermarías": "enfermar",
    "enfermas": "enfermo",
    "enfermase": "enfermar",
    "enfermaseis": "enfermar",
    "enfermasen": "enfermar",
    "enfermases": "enfermar",
    "enfermaste": "enfermar",
    "enfermasteis": "enfermar",
    "enferme": "enfermar",
    "enfermedades": "enfermedad",
    "enfermemos": "enfermar",
    "enfermen": "enfermar",
    "enfermera": "enfermero",
    "enfermeras": "enfermero",
    "enfermeros": "enfermero",
    "enfermerías": "enfermería",
    "enfermes": "enfermar",
    "enfermiza": "enfermizo",
    "enfermizas": "enfermizo",
    "enfermizos": "enfermizo",
    "enfermo": "enfermar",
    "enfermos": "enfermo",
    "enfermábamos": "enfermar",
    "enfermáis": "enfermar",
    "enfermáramos": "enfermar",
    "enfermáremos": "enfermar",
    "enfermásemos": "enfermar",
    "enfermé": "enfermar",
    "enferméis": "enfermar",
    "enfermó": "enfermar",
    "enfervorice": "enfervorizar",
    "enfervoricemos": "enfervorizar",
    "enfervoricen": "enfervorizar",
    "enfervorices": "enfervorizar",
    "enfervoricé": "enfervorizar",
    "enfervoricéis": "enfervorizar",
    "enfervoriza": "enfervorizar",
    "enfervorizaba": "enfervorizar",
    "enfervorizabais": "enfervorizar",
    "enfervorizaban": "enfervorizar",
    "enfervorizabas": "enfervorizar",
    "enfervorizaciones": "enfervorización",
    "enfervorizad": "enfervorizar",
    "enfervorizada": "enfervorizar",
    "enfervorizadas": "enfervorizar",
    "enfervorizado": "enfervorizar",
    "enfervorizadora": "enfervorizador",
    "enfervorizadoras": "enfervorizador",
    "enfervorizadores": "enfervorizador",
    "enfervorizados": "enfervorizar",
    "enfervorizamos": "enfervorizar",
    "enfervorizan": "enfervorizar",
    "enfervorizando": "enfervorizar",
    "enfervorizara": "enfervorizar",
    "enfervorizarais": "enfervorizar",
    "enfervorizaran": "enfervorizar",
    "enfervorizaras": "enfervorizar",
    "enfervorizare": "enfervorizar",
    "enfervorizareis": "enfervorizar",
    "enfervorizaremos": "enfervorizar",
    "enfervorizaren": "enfervorizar",
    "enfervorizares": "enfervorizar",
    "enfervorizaron": "enfervorizar",
    "enfervorizará": "enfervorizar",
    "enfervorizarán": "enfervorizar",
    "enfervorizarás": "enfervorizar",
    "enfervorizaré": "enfervorizar",
    "enfervorizaréis": "enfervorizar",
    "enfervorizaría": "enfervorizar",
    "enfervorizaríais": "enfervorizar",
    "enfervorizaríamos": "enfervorizar",
    "enfervorizarían": "enfervorizar",
    "enfervorizarías": "enfervorizar",
    "enfervorizas": "enfervorizar",
    "enfervorizase": "enfervorizar",
    "enfervorizaseis": "enfervorizar",
    "enfervorizasen": "enfervorizar",
    "enfervorizases": "enfervorizar",
    "enfervorizaste": "enfervorizar",
    "enfervorizasteis": "enfervorizar",
    "enfervorizo": "enfervorizar",
    "enfervorizábamos": "enfervorizar",
    "enfervorizáis": "enfervorizar",
    "enfervorizáramos": "enfervorizar",
    "enfervorizáremos": "enfervorizar",
    "enfervorizásemos": "enfervorizar",
    "enfervorizó": "enfervorizar",
    "enfeuda": "enfeudar",
    "enfeudaba": "enfeudar",
    "enfeudabais": "enfeudar",
    "enfeudaban": "enfeudar",
    "enfeudabas": "enfeudar",
    "enfeudaciones": "enfeudación",
    "enfeudad": "enfeudar",
    "enfeudada": "enfeudar",
    "enfeudadas": "enfeudar",
    "enfeudado": "enfeudar",
    "enfeudados": "enfeudar",
    "enfeudamos": "enfeudar",
    "enfeudan": "enfeudar",
    "enfeudando": "enfeudar",
    "enfeudara": "enfeudar",
    "enfeudarais": "enfeudar",
    "enfeudaran": "enfeudar",
    "enfeudaras": "enfeudar",
    "enfeudare": "enfeudar",
    "enfeudareis": "enfeudar",
    "enfeudaremos": "enfeudar",
    "enfeudaren": "enfeudar",
    "enfeudares": "enfeudar",
    "enfeudaron": "enfeudar",
    "enfeudará": "enfeudar",
    "enfeudarán": "enfeudar",
    "enfeudarás": "enfeudar",
    "enfeudaré": "enfeudar",
    "enfeudaréis": "enfeudar",
    "enfeudaría": "enfeudar",
    "enfeudaríais": "enfeudar",
    "enfeudaríamos": "enfeudar",
    "enfeudarían": "enfeudar",
    "enfeudarías": "enfeudar",
    "enfeudas": "enfeudar",
    "enfeudase": "enfeudar",
    "enfeudaseis": "enfeudar",
    "enfeudasen": "enfeudar",
    "enfeudases": "enfeudar",
    "enfeudaste": "enfeudar",
    "enfeudasteis": "enfeudar",
    "enfeude": "enfeudar",
    "enfeudemos": "enfeudar",
    "enfeuden": "enfeudar",
    "enfeudes": "enfeudar",
    "enfeudo": "enfeudar",
    "enfeudábamos": "enfeudar",
    "enfeudáis": "enfeudar",
    "enfeudáramos": "enfeudar",
    "enfeudáremos": "enfeudar",
    "enfeudásemos": "enfeudar",
    "enfeudé": "enfeudar",
    "enfeudéis": "enfeudar",
    "enfeudó": "enfeudar",
    "enfila": "enfilar",
    "enfilaba": "enfilar",
    "enfilabais": "enfilar",
    "enfilaban": "enfilar",
    "enfilabas": "enfilar",
    "enfilaciones": "enfilación",
    "enfilad": "enfilar",
    "enfilada": "enfilar",
    "enfiladas": "enfilar",
    "enfilado": "enfilar",
    "enfilados": "enfilar",
    "enfilamos": "enfilar",
    "enfilan": "enfilar",
    "enfilando": "enfilar",
    "enfilara": "enfilar",
    "enfilarais": "enfilar",
    "enfilaran": "enfilar",
    "enfilaras": "enfilar",
    "enfilare": "enfilar",
    "enfilareis": "enfilar",
    "enfilaremos": "enfilar",
    "enfilaren": "enfilar",
    "enfilares": "enfilar",
    "enfilaron": "enfilar",
    "enfilará": "enfilar",
    "enfilarán": "enfilar",
    "enfilarás": "enfilar",
    "enfilaré": "enfilar",
    "enfilaréis": "enfilar",
    "enfilaría": "enfilar",
    "enfilaríais": "enfilar",
    "enfilaríamos": "enfilar",
    "enfilarían": "enfilar",
    "enfilarías": "enfilar",
    "enfilas": "enfilar",
    "enfilase": "enfilar",
    "enfilaseis": "enfilar",
    "enfilasen": "enfilar",
    "enfilases": "enfilar",
    "enfilaste": "enfilar",
    "enfilasteis": "enfilar",
    "enfile": "enfilar",
    "enfilemos": "enfilar",
    "enfilen": "enfilar",
    "enfiles": "enfilar",
    "enfilo": "enfilar",
    "enfilábamos": "enfilar",
    "enfiláis": "enfilar",
    "enfiláramos": "enfilar",
    "enfiláremos": "enfilar",
    "enfilásemos": "enfilar",
    "enfilé": "enfilar",
    "enfiléis": "enfilar",
    "enfiló": "enfilar",
    "enfisemas": "enfisema",
    "enfisematosa": "enfisematoso",
    "enfisematosas": "enfisematoso",
    "enfisematosos": "enfisematoso",
    "enfistola": "enfistolar",
    "enfistolaba": "enfistolar",
    "enfistolabais": "enfistolar",
    "enfistolaban": "enfistolar",
    "enfistolabas": "enfistolar",
    "enfistolad": "enfistolar",
    "enfistolada": "enfistolar",
    "enfistoladas": "enfistolar",
    "enfistolado": "enfistolar",
    "enfistolados": "enfistolar",
    "enfistolamos": "enfistolar",
    "enfistolan": "enfistolar",
    "enfistolando": "enfistolar",
    "enfistolara": "enfistolar",
    "enfistolarais": "enfistolar",
    "enfistolaran": "enfistolar",
    "enfistolaras": "enfistolar",
    "enfistolare": "enfistolar",
    "enfistolareis": "enfistolar",
    "enfistolaremos": "enfistolar",
    "enfistolaren": "enfistolar",
    "enfistolares": "enfistolar",
    "enfistolaron": "enfistolar",
    "enfistolará": "enfistolar",
    "enfistolarán": "enfistolar",
    "enfistolarás": "enfistolar",
    "enfistolaré": "enfistolar",
    "enfistolaréis": "enfistolar",
    "enfistolaría": "enfistolar",
    "enfistolaríais": "enfistolar",
    "enfistolaríamos": "enfistolar",
    "enfistolarían": "enfistolar",
    "enfistolarías": "enfistolar",
    "enfistolas": "enfistolar",
    "enfistolase": "enfistolar",
    "enfistolaseis": "enfistolar",
    "enfistolasen": "enfistolar",
    "enfistolases": "enfistolar",
    "enfistolaste": "enfistolar",
    "enfistolasteis": "enfistolar",
    "enfistole": "enfistolar",
    "enfistolemos": "enfistolar",
    "enfistolen": "enfistolar",
    "enfistoles": "enfistolar",
    "enfistolo": "enfistolar",
    "enfistolábamos": "enfistolar",
    "enfistoláis": "enfistolar",
    "enfistoláramos": "enfistolar",
    "enfistoláremos": "enfistolar",
    "enfistolásemos": "enfistolar",
    "enfistolé": "enfistolar",
    "enfistoléis": "enfistolar",
    "enfistoló": "enfistolar",
    "enfiteutas": "enfiteuta",
    "enfitéutica": "enfitéutico",
    "enfitéuticas": "enfitéutico",
    "enfitéuticos": "enfitéutico",
    "enflaquece": "enflaquecer",
    "enflaqueced": "enflaquecer",
    "enflaquecemos": "enflaquecer",
    "enflaquecen": "enflaquecer",
    "enflaqueceremos": "enflaquecer",
    "enflaquecerá": "enflaquecer",
    "enflaquecerán": "enflaquecer",
    "enflaquecerás": "enflaquecer",
    "enflaqueceré": "enflaquecer",
    "enflaqueceréis": "enflaquecer",
    "enflaquecería": "enflaquecer",
    "enflaqueceríais": "enflaquecer",
    "enflaqueceríamos": "enflaquecer",
    "enflaquecerían": "enflaquecer",
    "enflaquecerías": "enflaquecer",
    "enflaqueces": "enflaquecer",
    "enflaquecida": "enflaquecer",
    "enflaquecidas": "enflaquecer",
    "enflaquecido": "enflaquecer",
    "enflaquecidos": "enflaquecer",
    "enflaqueciendo": "enflaquecer",
    "enflaqueciera": "enflaquecer",
    "enflaquecierais": "enflaquecer",
    "enflaquecieran": "enflaquecer",
    "enflaquecieras": "enflaquecer",
    "enflaqueciere": "enflaquecer",
    "enflaqueciereis": "enflaquecer",
    "enflaquecieren": "enflaquecer",
    "enflaquecieres": "enflaquecer",
    "enflaquecieron": "enflaquecer",
    "enflaqueciese": "enflaquecer",
    "enflaquecieseis": "enflaquecer",
    "enflaqueciesen": "enflaquecer",
    "enflaquecieses": "enflaquecer",
    "enflaquecimientos": "enflaquecimiento",
    "enflaquecimos": "enflaquecer",
    "enflaqueciste": "enflaquecer",
    "enflaquecisteis": "enflaquecer",
    "enflaqueciéramos": "enflaquecer",
    "enflaqueciéremos": "enflaquecer",
    "enflaqueciésemos": "enflaquecer",
    "enflaqueció": "enflaquecer",
    "enflaquecéis": "enflaquecer",
    "enflaquecí": "enflaquecer",
    "enflaquecía": "enflaquecer",
    "enflaquecíais": "enflaquecer",
    "enflaquecíamos": "enflaquecer",
    "enflaquecían": "enflaquecer",
    "enflaquecías": "enflaquecer",
    "enflaquezca": "enflaquecer",
    "enflaquezcamos": "enflaquecer",
    "enflaquezcan": "enflaquecer",
    "enflaquezcas": "enflaquecer",
    "enflaquezco": "enflaquecer",
    "enflaquezcáis": "enflaquecer",
    "enflauta": "enflautar",
    "enflautaba": "enflautar",
    "enflautabais": "enflautar",
    "enflautaban": "enflautar",
    "enflautabas": "enflautar",
    "enflautad": "enflautar",
    "enflautada": "enflautar",
    "enflautadas": "enflautar",
    "enflautado": "enflautar",
    "enflautados": "enflautar",
    "enflautamos": "enflautar",
    "enflautan": "enflautar",
    "enflautando": "enflautar",
    "enflautara": "enflautar",
    "enflautarais": "enflautar",
    "enflautaran": "enflautar",
    "enflautaras": "enflautar",
    "enflautare": "enflautar",
    "enflautareis": "enflautar",
    "enflautaremos": "enflautar",
    "enflautaren": "enflautar",
    "enflautares": "enflautar",
    "enflautaron": "enflautar",
    "enflautará": "enflautar",
    "enflautarán": "enflautar",
    "enflautarás": "enflautar",
    "enflautaré": "enflautar",
    "enflautaréis": "enflautar",
    "enflautaría": "enflautar",
    "enflautaríais": "enflautar",
    "enflautaríamos": "enflautar",
    "enflautarían": "enflautar",
    "enflautarías": "enflautar",
    "enflautas": "enflautar",
    "enflautase": "enflautar",
    "enflautaseis": "enflautar",
    "enflautasen": "enflautar",
    "enflautases": "enflautar",
    "enflautaste": "enflautar",
    "enflautasteis": "enflautar",
    "enflaute": "enflautar",
    "enflautemos": "enflautar",
    "enflauten": "enflautar",
    "enflautes": "enflautar",
    "enflauto": "enflautar",
    "enflautábamos": "enflautar",
    "enflautáis": "enflautar",
    "enflautáramos": "enflautar",
    "enflautáremos": "enflautar",
    "enflautásemos": "enflautar",
    "enflauté": "enflautar",
    "enflautéis": "enflautar",
    "enflautó": "enflautar",
    "enflechada": "enflechado",
    "enflechadas": "enflechado",
    "enflechados": "enflechado",
    "enfoca": "enfocar",
    "enfocaba": "enfocar",
    "enfocabais": "enfocar",
    "enfocaban": "enfocar",
    "enfocabas": "enfocar",
    "enfocad": "enfocar",
    "enfocada": "enfocar",
    "enfocadas": "enfocar",
    "enfocado": "enfocar",
    "enfocados": "enfocar",
    "enfocamos": "enfocar",
    "enfocan": "enfocar",
    "enfocando": "enfocar",
    "enfocara": "enfocar",
    "enfocarais": "enfocar",
    "enfocaran": "enfocar",
    "enfocaras": "enfocar",
    "enfocare": "enfocar",
    "enfocareis": "enfocar",
    "enfocaremos": "enfocar",
    "enfocaren": "enfocar",
    "enfocares": "enfocar",
    "enfocaron": "enfocar",
    "enfocará": "enfocar",
    "enfocarán": "enfocar",
    "enfocarás": "enfocar",
    "enfocaré": "enfocar",
    "enfocaréis": "enfocar",
    "enfocaría": "enfocar",
    "enfocaríais": "enfocar",
    "enfocaríamos": "enfocar",
    "enfocarían": "enfocar",
    "enfocarías": "enfocar",
    "enfocas": "enfocar",
    "enfocase": "enfocar",
    "enfocaseis": "enfocar",
    "enfocasen": "enfocar",
    "enfocases": "enfocar",
    "enfocaste": "enfocar",
    "enfocasteis": "enfocar",
    "enfoco": "enfocar",
    "enfocábamos": "enfocar",
    "enfocáis": "enfocar",
    "enfocáramos": "enfocar",
    "enfocáremos": "enfocar",
    "enfocásemos": "enfocar",
    "enfocó": "enfocar",
    "enfoque": "enfocar",
    "enfoquemos": "enfocar",
    "enfoquen": "enfocar",
    "enfoques": "enfoque",
    "enfoqué": "enfocar",
    "enfoquéis": "enfocar",
    "enfosca": "enfoscar",
    "enfoscaba": "enfoscar",
    "enfoscabais": "enfoscar",
    "enfoscaban": "enfoscar",
    "enfoscabas": "enfoscar",
    "enfoscad": "enfoscar",
    "enfoscada": "enfoscar",
    "enfoscadas": "enfoscar",
    "enfoscado": "enfoscar",
    "enfoscados": "enfoscar",
    "enfoscamos": "enfoscar",
    "enfoscan": "enfoscar",
    "enfoscando": "enfoscar",
    "enfoscara": "enfoscar",
    "enfoscarais": "enfoscar",
    "enfoscaran": "enfoscar",
    "enfoscaras": "enfoscar",
    "enfoscare": "enfoscar",
    "enfoscareis": "enfoscar",
    "enfoscaremos": "enfoscar",
    "enfoscaren": "enfoscar",
    "enfoscares": "enfoscar",
    "enfoscaron": "enfoscar",
    "enfoscará": "enfoscar",
    "enfoscarán": "enfoscar",
    "enfoscarás": "enfoscar",
    "enfoscaré": "enfoscar",
    "enfoscaréis": "enfoscar",
    "enfoscaría": "enfoscar",
    "enfoscaríais": "enfoscar",
    "enfoscaríamos": "enfoscar",
    "enfoscarían": "enfoscar",
    "enfoscarías": "enfoscar",
    "enfoscas": "enfoscar",
    "enfoscase": "enfoscar",
    "enfoscaseis": "enfoscar",
    "enfoscasen": "enfoscar",
    "enfoscases": "enfoscar",
    "enfoscaste": "enfoscar",
    "enfoscasteis": "enfoscar",
    "enfosco": "enfoscar",
    "enfoscábamos": "enfoscar",
    "enfoscáis": "enfoscar",
    "enfoscáramos": "enfoscar",
    "enfoscáremos": "enfoscar",
    "enfoscásemos": "enfoscar",
    "enfoscó": "enfoscar",
    "enfosque": "enfoscar",
    "enfosquemos": "enfoscar",
    "enfosquen": "enfoscar",
    "enfosques": "enfoscar",
    "enfosqué": "enfoscar",
    "enfosquéis": "enfoscar",
    "enfraila": "enfrailar",
    "enfrailaba": "enfrailar",
    "enfrailabais": "enfrailar",
    "enfrailaban": "enfrailar",
    "enfrailabas": "enfrailar",
    "enfrailad": "enfrailar",
    "enfrailada": "enfrailar",
    "enfrailadas": "enfrailar",
    "enfrailado": "enfrailar",
    "enfrailados": "enfrailar",
    "enfrailamos": "enfrailar",
    "enfrailan": "enfrailar",
    "enfrailando": "enfrailar",
    "enfrailara": "enfrailar",
    "enfrailarais": "enfrailar",
    "enfrailaran": "enfrailar",
    "enfrailaras": "enfrailar",
    "enfrailare": "enfrailar",
    "enfrailareis": "enfrailar",
    "enfrailaremos": "enfrailar",
    "enfrailaren": "enfrailar",
    "enfrailares": "enfrailar",
    "enfrailaron": "enfrailar",
    "enfrailará": "enfrailar",
    "enfrailarán": "enfrailar",
    "enfrailarás": "enfrailar",
    "enfrailaré": "enfrailar",
    "enfrailaréis": "enfrailar",
    "enfrailaría": "enfrailar",
    "enfrailaríais": "enfrailar",
    "enfrailaríamos": "enfrailar",
    "enfrailarían": "enfrailar",
    "enfrailarías": "enfrailar",
    "enfrailas": "enfrailar",
    "enfrailase": "enfrailar",
    "enfrailaseis": "enfrailar",
    "enfrailasen": "enfrailar",
    "enfrailases": "enfrailar",
    "enfrailaste": "enfrailar",
    "enfrailasteis": "enfrailar",
    "enfraile": "enfrailar",
    "enfrailemos": "enfrailar",
    "enfrailen": "enfrailar",
    "enfrailes": "enfrailar",
    "enfrailo": "enfrailar",
    "enfrailábamos": "enfrailar",
    "enfrailáis": "enfrailar",
    "enfrailáramos": "enfrailar",
    "enfrailáremos": "enfrailar",
    "enfrailásemos": "enfrailar",
    "enfrailé": "enfrailar",
    "enfrailéis": "enfrailar",
    "enfrailó": "enfrailar",
    "enfranques": "enfranque",
    "enfrasca": "enfrascar",
    "enfrascaba": "enfrascar",
    "enfrascabais": "enfrascar",
    "enfrascaban": "enfrascar",
    "enfrascabas": "enfrascar",
    "enfrascad": "enfrascar",
    "enfrascada": "enfrascar",
    "enfrascadas": "enfrascar",
    "enfrascado": "enfrascar",
    "enfrascados": "enfrascar",
    "enfrascamientos": "enfrascamiento",
    "enfrascamos": "enfrascar",
    "enfrascan": "enfrascar",
    "enfrascando": "enfrascar",
    "enfrascara": "enfrascar",
    "enfrascarais": "enfrascar",
    "enfrascaran": "enfrascar",
    "enfrascaras": "enfrascar",
    "enfrascare": "enfrascar",
    "enfrascareis": "enfrascar",
    "enfrascaremos": "enfrascar",
    "enfrascaren": "enfrascar",
    "enfrascares": "enfrascar",
    "enfrascaron": "enfrascar",
    "enfrascará": "enfrascar",
    "enfrascarán": "enfrascar",
    "enfrascarás": "enfrascar",
    "enfrascaré": "enfrascar",
    "enfrascaréis": "enfrascar",
    "enfrascaría": "enfrascar",
    "enfrascaríais": "enfrascar",
    "enfrascaríamos": "enfrascar",
    "enfrascarían": "enfrascar",
    "enfrascarías": "enfrascar",
    "enfrascas": "enfrascar",
    "enfrascase": "enfrascar",
    "enfrascaseis": "enfrascar",
    "enfrascasen": "enfrascar",
    "enfrascases": "enfrascar",
    "enfrascaste": "enfrascar",
    "enfrascasteis": "enfrascar",
    "enfrasco": "enfrascar",
    "enfrascábamos": "enfrascar",
    "enfrascáis": "enfrascar",
    "enfrascáramos": "enfrascar",
    "enfrascáremos": "enfrascar",
    "enfrascásemos": "enfrascar",
    "enfrascó": "enfrascar",
    "enfrasque": "enfrascar",
    "enfrasquemos": "enfrascar",
    "enfrasquen": "enfrascar",
    "enfrasques": "enfrascar",
    "enfrasqué": "enfrascar",
    "enfrasquéis": "enfrascar",
    "enfrena": "enfrenar",
    "enfrenaba": "enfrenar",
    "enfrenabais": "enfrenar",
    "enfrenaban": "enfrenar",
    "enfrenabas": "enfrenar",
    "enfrenad": "enfrenar",
    "enfrenada": "enfrenar",
    "enfrenadas": "enfrenar",
    "enfrenado": "enfrenar",
    "enfrenados": "enfrenar",
    "enfrenamos": "enfrenar",
    "enfrenan": "enfrenar",
    "enfrenando": "enfrenar",
    "enfrenara": "enfrenar",
    "enfrenarais": "enfrenar",
    "enfrenaran": "enfrenar",
    "enfrenaras": "enfrenar",
    "enfrenare": "enfrenar",
    "enfrenareis": "enfrenar",
    "enfrenaremos": "enfrenar",
    "enfrenaren": "enfrenar",
    "enfrenares": "enfrenar",
    "enfrenaron": "enfrenar",
    "enfrenará": "enfrenar",
    "enfrenarán": "enfrenar",
    "enfrenarás": "enfrenar",
    "enfrenaré": "enfrenar",
    "enfrenaréis": "enfrenar",
    "enfrenaría": "enfrenar",
    "enfrenaríais": "enfrenar",
    "enfrenaríamos": "enfrenar",
    "enfrenarían": "enfrenar",
    "enfrenarías": "enfrenar",
    "enfrenas": "enfrenar",
    "enfrenase": "enfrenar",
    "enfrenaseis": "enfrenar",
    "enfrenasen": "enfrenar",
    "enfrenases": "enfrenar",
    "enfrenaste": "enfrenar",
    "enfrenasteis": "enfrenar",
    "enfrene": "enfrenar",
    "enfrenemos": "enfrenar",
    "enfrenen": "enfrenar",
    "enfrenes": "enfrenar",
    "enfreno": "enfrenar",
    "enfrenta": "enfrentar",
    "enfrentaba": "enfrentar",
    "enfrentabais": "enfrentar",
    "enfrentaban": "enfrentar",
    "enfrentabas": "enfrentar",
    "enfrentad": "enfrentar",
    "enfrentada": "enfrentar",
    "enfrentadas": "enfrentar",
    "enfrentado": "enfrentar",
    "enfrentados": "enfrentar",
    "enfrentamientos": "enfrentamiento",
    "enfrentamos": "enfrentar",
    "enfrentan": "enfrentar",
    "enfrentando": "enfrentar",
    "enfrentara": "enfrentar",
    "enfrentarais": "enfrentar",
    "enfrentaran": "enfrentar",
    "enfrentaras": "enfrentar",
    "enfrentare": "enfrentar",
    "enfrentareis": "enfrentar",
    "enfrentaremos": "enfrentar",
    "enfrentaren": "enfrentar",
    "enfrentares": "enfrentar",
    "enfrentaron": "enfrentar",
    "enfrentará": "enfrentar",
    "enfrentarán": "enfrentar",
    "enfrentarás": "enfrentar",
    "enfrentaré": "enfrentar",
    "enfrentaréis": "enfrentar",
    "enfrentaría": "enfrentar",
    "enfrentaríais": "enfrentar",
    "enfrentaríamos": "enfrentar",
    "enfrentarían": "enfrentar",
    "enfrentarías": "enfrentar",
    "enfrentas": "enfrentar",
    "enfrentase": "enfrentar",
    "enfrentaseis": "enfrentar",
    "enfrentasen": "enfrentar",
    "enfrentases": "enfrentar",
    "enfrentaste": "enfrentar",
    "enfrentasteis": "enfrentar",
    "enfrente": "enfrentar",
    "enfrentemos": "enfrentar",
    "enfrenten": "enfrentar",
    "enfrentes": "enfrentar",
    "enfrento": "enfrentar",
    "enfrentábamos": "enfrentar",
    "enfrentáis": "enfrentar",
    "enfrentáramos": "enfrentar",
    "enfrentáremos": "enfrentar",
    "enfrentásemos": "enfrentar",
    "enfrenté": "enfrentar",
    "enfrentéis": "enfrentar",
    "enfrentó": "enfrentar",
    "enfrenábamos": "enfrenar",
    "enfrenáis": "enfrenar",
    "enfrenáramos": "enfrenar",
    "enfrenáremos": "enfrenar",
    "enfrenásemos": "enfrenar",
    "enfrené": "enfrenar",
    "enfrenéis": "enfrenar",
    "enfrenó": "enfrenar",
    "enfriaba": "enfriar",
    "enfriabais": "enfriar",
    "enfriaban": "enfriar",
    "enfriabas": "enfriar",
    "enfriables": "enfriable",
    "enfriad": "enfriar",
    "enfriada": "enfriar",
    "enfriadas": "enfriar",
    "enfriaderas": "enfriadera",
    "enfriaderos": "enfriadero",
    "enfriado": "enfriar",
    "enfriadora": "enfriador",
    "enfriadoras": "enfriador",
    "enfriadores": "enfriador",
    "enfriados": "enfriar",
    "enfriamientos": "enfriamiento",
    "enfriamos": "enfriar",
    "enfriando": "enfriar",
    "enfriara": "enfriar",
    "enfriarais": "enfriar",
    "enfriaran": "enfriar",
    "enfriaras": "enfriar",
    "enfriare": "enfriar",
    "enfriareis": "enfriar",
    "enfriaremos": "enfriar",
    "enfriaren": "enfriar",
    "enfriares": "enfriar",
    "enfriaron": "enfriar",
    "enfriará": "enfriar",
    "enfriarán": "enfriar",
    "enfriarás": "enfriar",
    "enfriaré": "enfriar",
    "enfriaréis": "enfriar",
    "enfriaría": "enfriar",
    "enfriaríais": "enfriar",
    "enfriaríamos": "enfriar",
    "enfriarían": "enfriar",
    "enfriarías": "enfriar",
    "enfriase": "enfriar",
    "enfriaseis": "enfriar",
    "enfriasen": "enfriar",
    "enfriases": "enfriar",
    "enfriaste": "enfriar",
    "enfriasteis": "enfriar",
    "enfriemos": "enfriar",
    "enfriábamos": "enfriar",
    "enfriáis": "enfriar",
    "enfriáramos": "enfriar",
    "enfriáremos": "enfriar",
    "enfriásemos": "enfriar",
    "enfrié": "enfriar",
    "enfriéis": "enfriar",
    "enfrió": "enfriar",
    "enfronta": "enfrontar",
    "enfrontaba": "enfrontar",
    "enfrontabais": "enfrontar",
    "enfrontaban": "enfrontar",
    "enfrontabas": "enfrontar",
    "enfrontad": "enfrontar",
    "enfrontada": "enfrontar",
    "enfrontadas": "enfrontar",
    "enfrontado": "enfrontar",
    "enfrontados": "enfrontar",
    "enfrontamientos": "enfrontamiento",
    "enfrontamos": "enfrontar",
    "enfrontan": "enfrontar",
    "enfrontando": "enfrontar",
    "enfrontara": "enfrontar",
    "enfrontarais": "enfrontar",
    "enfrontaran": "enfrontar",
    "enfrontaras": "enfrontar",
    "enfrontare": "enfrontar",
    "enfrontareis": "enfrontar",
    "enfrontaremos": "enfrontar",
    "enfrontaren": "enfrontar",
    "enfrontares": "enfrontar",
    "enfrontaron": "enfrontar",
    "enfrontará": "enfrontar",
    "enfrontarán": "enfrontar",
    "enfrontarás": "enfrontar",
    "enfrontaré": "enfrontar",
    "enfrontaréis": "enfrontar",
    "enfrontaría": "enfrontar",
    "enfrontaríais": "enfrontar",
    "enfrontaríamos": "enfrontar",
    "enfrontarían": "enfrontar",
    "enfrontarías": "enfrontar",
    "enfrontas": "enfrontar",
    "enfrontase": "enfrontar",
    "enfrontaseis": "enfrontar",
    "enfrontasen": "enfrontar",
    "enfrontases": "enfrontar",
    "enfrontaste": "enfrontar",
    "enfrontasteis": "enfrontar",
    "enfronte": "enfrontar",
    "enfrontemos": "enfrontar",
    "enfronten": "enfrontar",
    "enfrontes": "enfrontar",
    "enfronto": "enfrontar",
    "enfrontábamos": "enfrontar",
    "enfrontáis": "enfrontar",
    "enfrontáramos": "enfrontar",
    "enfrontáremos": "enfrontar",
    "enfrontásemos": "enfrontar",
    "enfronté": "enfrontar",
    "enfrontéis": "enfrontar",
    "enfrontó": "enfrontar",
    "enfría": "enfriar",
    "enfrían": "enfriar",
    "enfrías": "enfriar",
    "enfríe": "enfriar",
    "enfríen": "enfriar",
    "enfríes": "enfriar",
    "enfrío": "enfriar",
    "enfulla": "enfullar",
    "enfullaba": "enfullar",
    "enfullabais": "enfullar",
    "enfullaban": "enfullar",
    "enfullabas": "enfullar",
    "enfullad": "enfullar",
    "enfullada": "enfullar",
    "enfulladas": "enfullar",
    "enfullado": "enfullar",
    "enfullados": "enfullar",
    "enfullamos": "enfullar",
    "enfullan": "enfullar",
    "enfullando": "enfullar",
    "enfullara": "enfullar",
    "enfullarais": "enfullar",
    "enfullaran": "enfullar",
    "enfullaras": "enfullar",
    "enfullare": "enfullar",
    "enfullareis": "enfullar",
    "enfullaremos": "enfullar",
    "enfullaren": "enfullar",
    "enfullares": "enfullar",
    "enfullaron": "enfullar",
    "enfullará": "enfullar",
    "enfullarán": "enfullar",
    "enfullarás": "enfullar",
    "enfullaré": "enfullar",
    "enfullaréis": "enfullar",
    "enfullaría": "enfullar",
    "enfullaríais": "enfullar",
    "enfullaríamos": "enfullar",
    "enfullarían": "enfullar",
    "enfullarías": "enfullar",
    "enfullas": "enfullar",
    "enfullase": "enfullar",
    "enfullaseis": "enfullar",
    "enfullasen": "enfullar",
    "enfullases": "enfullar",
    "enfullaste": "enfullar",
    "enfullasteis": "enfullar",
    "enfulle": "enfullar",
    "enfullemos": "enfullar",
    "enfullen": "enfullar",
    "enfulles": "enfullar",
    "enfullo": "enfullar",
    "enfullábamos": "enfullar",
    "enfulláis": "enfullar",
    "enfulláramos": "enfullar",
    "enfulláremos": "enfullar",
    "enfullásemos": "enfullar",
    "enfullé": "enfullar",
    "enfulléis": "enfullar",
    "enfulló": "enfullar",
    "enfunda": "enfundar",
    "enfundaba": "enfundar",
    "enfundabais": "enfundar",
    "enfundaban": "enfundar",
    "enfundabas": "enfundar",
    "enfundad": "enfundar",
    "enfundada": "enfundar",
    "enfundadas": "enfundar",
    "enfundado": "enfundar",
    "enfundados": "enfundar",
    "enfundaduras": "enfundadura",
    "enfundamos": "enfundar",
    "enfundan": "enfundar",
    "enfundando": "enfundar",
    "enfundara": "enfundar",
    "enfundarais": "enfundar",
    "enfundaran": "enfundar",
    "enfundaras": "enfundar",
    "enfundare": "enfundar",
    "enfundareis": "enfundar",
    "enfundaremos": "enfundar",
    "enfundaren": "enfundar",
    "enfundares": "enfundar",
    "enfundaron": "enfundar",
    "enfundará": "enfundar",
    "enfundarán": "enfundar",
    "enfundarás": "enfundar",
    "enfundaré": "enfundar",
    "enfundaréis": "enfundar",
    "enfundaría": "enfundar",
    "enfundaríais": "enfundar",
    "enfundaríamos": "enfundar",
    "enfundarían": "enfundar",
    "enfundarías": "enfundar",
    "enfundas": "enfundar",
    "enfundase": "enfundar",
    "enfundaseis": "enfundar",
    "enfundasen": "enfundar",
    "enfundases": "enfundar",
    "enfundaste": "enfundar",
    "enfundasteis": "enfundar",
    "enfunde": "enfundar",
    "enfundemos": "enfundar",
    "enfunden": "enfundar",
    "enfundes": "enfundar",
    "enfundo": "enfundar",
    "enfundábamos": "enfundar",
    "enfundáis": "enfundar",
    "enfundáramos": "enfundar",
    "enfundáremos": "enfundar",
    "enfundásemos": "enfundar",
    "enfundé": "enfundar",
    "enfundéis": "enfundar",
    "enfundó": "enfundar",
    "enfurece": "enfurecer",
    "enfureced": "enfurecer",
    "enfurecemos": "enfurecer",
    "enfurecen": "enfurecer",
    "enfureceremos": "enfurecer",
    "enfurecerá": "enfurecer",
    "enfurecerán": "enfurecer",
    "enfurecerás": "enfurecer",
    "enfureceré": "enfurecer",
    "enfureceréis": "enfurecer",
    "enfurecería": "enfurecer",
    "enfureceríais": "enfurecer",
    "enfureceríamos": "enfurecer",
    "enfurecerían": "enfurecer",
    "enfurecerías": "enfurecer",
    "enfureces": "enfurecer",
    "enfurecida": "enfurecer",
    "enfurecidas": "enfurecer",
    "enfurecido": "enfurecer",
    "enfurecidos": "enfurecer",
    "enfureciendo": "enfurecer",
    "enfureciera": "enfurecer",
    "enfurecierais": "enfurecer",
    "enfurecieran": "enfurecer",
    "enfurecieras": "enfurecer",
    "enfureciere": "enfurecer",
    "enfureciereis": "enfurecer",
    "enfurecieren": "enfurecer",
    "enfurecieres": "enfurecer",
    "enfurecieron": "enfurecer",
    "enfureciese": "enfurecer",
    "enfurecieseis": "enfurecer",
    "enfureciesen": "enfurecer",
    "enfurecieses": "enfurecer",
    "enfurecimientos": "enfurecimiento",
    "enfurecimos": "enfurecer",
    "enfureciste": "enfurecer",
    "enfurecisteis": "enfurecer",
    "enfureciéramos": "enfurecer",
    "enfureciéremos": "enfurecer",
    "enfureciésemos": "enfurecer",
    "enfureció": "enfurecer",
    "enfurecéis": "enfurecer",
    "enfurecí": "enfurecer",
    "enfurecía": "enfurecer",
    "enfurecíais": "enfurecer",
    "enfurecíamos": "enfurecer",
    "enfurecían": "enfurecer",
    "enfurecías": "enfurecer",
    "enfurezca": "enfurecer",
    "enfurezcamos": "enfurecer",
    "enfurezcan": "enfurecer",
    "enfurezcas": "enfurecer",
    "enfurezco": "enfurecer",
    "enfurezcáis": "enfurecer",
    "enfurruña": "enfurruñar",
    "enfurruñaba": "enfurruñar",
    "enfurruñabais": "enfurruñar",
    "enfurruñaban": "enfurruñar",
    "enfurruñabas": "enfurruñar",
    "enfurruñad": "enfurruñar",
    "enfurruñada": "enfurruñar",
    "enfurruñadas": "enfurruñar",
    "enfurruñado": "enfurruñar",
    "enfurruñados": "enfurruñar",
    "enfurruñamientos": "enfurruñamiento",
    "enfurruñamos": "enfurruñar",
    "enfurruñan": "enfurruñar",
    "enfurruñando": "enfurruñar",
    "enfurruñara": "enfurruñar",
    "enfurruñarais": "enfurruñar",
    "enfurruñaran": "enfurruñar",
    "enfurruñaras": "enfurruñar",
    "enfurruñare": "enfurruñar",
    "enfurruñareis": "enfurruñar",
    "enfurruñaremos": "enfurruñar",
    "enfurruñaren": "enfurruñar",
    "enfurruñares": "enfurruñar",
    "enfurruñaron": "enfurruñar",
    "enfurruñará": "enfurruñar",
    "enfurruñarán": "enfurruñar",
    "enfurruñarás": "enfurruñar",
    "enfurruñaré": "enfurruñar",
    "enfurruñaréis": "enfurruñar",
    "enfurruñaría": "enfurruñar",
    "enfurruñaríais": "enfurruñar",
    "enfurruñaríamos": "enfurruñar",
    "enfurruñarían": "enfurruñar",
    "enfurruñarías": "enfurruñar",
    "enfurruñas": "enfurruñar",
    "enfurruñase": "enfurruñar",
    "enfurruñaseis": "enfurruñar",
    "enfurruñasen": "enfurruñar",
    "enfurruñases": "enfurruñar",
    "enfurruñaste": "enfurruñar",
    "enfurruñasteis": "enfurruñar",
    "enfurruñe": "enfurruñar",
    "enfurruñemos": "enfurruñar",
    "enfurruñen": "enfurruñar",
    "enfurruñes": "enfurruñar",
    "enfurruño": "enfurruñar",
    "enfurruñábamos": "enfurruñar",
    "enfurruñáis": "enfurruñar",
    "enfurruñáramos": "enfurruñar",
    "enfurruñáremos": "enfurruñar",
    "enfurruñásemos": "enfurruñar",
    "enfurruñé": "enfurruñar",
    "enfurruñéis": "enfurruñar",
    "enfurruñó": "enfurruñar",
    "enfurta": "enfurtir",
    "enfurtamos": "enfurtir",
    "enfurtan": "enfurtir",
    "enfurtas": "enfurtir",
    "enfurte": "enfurtir",
    "enfurten": "enfurtir",
    "enfurtes": "enfurtir",
    "enfurtid": "enfurtir",
    "enfurtida": "enfurtir",
    "enfurtidas": "enfurtir",
    "enfurtido": "enfurtir",
    "enfurtidos": "enfurtir",
    "enfurtiendo": "enfurtir",
    "enfurtiera": "enfurtir",
    "enfurtierais": "enfurtir",
    "enfurtieran": "enfurtir",
    "enfurtieras": "enfurtir",
    "enfurtiere": "enfurtir",
    "enfurtiereis": "enfurtir",
    "enfurtieren": "enfurtir",
    "enfurtieres": "enfurtir",
    "enfurtieron": "enfurtir",
    "enfurtiese": "enfurtir",
    "enfurtieseis": "enfurtir",
    "enfurtiesen": "enfurtir",
    "enfurtieses": "enfurtir",
    "enfurtimos": "enfurtir",
    "enfurtiremos": "enfurtir",
    "enfurtirá": "enfurtir",
    "enfurtirán": "enfurtir",
    "enfurtirás": "enfurtir",
    "enfurtiré": "enfurtir",
    "enfurtiréis": "enfurtir",
    "enfurtiría": "enfurtir",
    "enfurtiríais": "enfurtir",
    "enfurtiríamos": "enfurtir",
    "enfurtirían": "enfurtir",
    "enfurtirías": "enfurtir",
    "enfurtiste": "enfurtir",
    "enfurtisteis": "enfurtir",
    "enfurtiéramos": "enfurtir",
    "enfurtiéremos": "enfurtir",
    "enfurtiésemos": "enfurtir",
    "enfurtió": "enfurtir",
    "enfurto": "enfurtir",
    "enfurtáis": "enfurtir",
    "enfurtí": "enfurtir",
    "enfurtía": "enfurtir",
    "enfurtíais": "enfurtir",
    "enfurtíamos": "enfurtir",
    "enfurtían": "enfurtir",
    "enfurtías": "enfurtir",
    "enfurtís": "enfurtir",
    "enfática": "enfático",
    "enfáticas": "enfático",
    "enfáticos": "enfático",
    "engace": "engazar",
    "engacemos": "engazar",
    "engacen": "engazar",
    "engaces": "engazar",
    "engacé": "engazar",
    "engacéis": "engazar",
    "engafa": "engafar",
    "engafaba": "engafar",
    "engafabais": "engafar",
    "engafaban": "engafar",
    "engafabas": "engafar",
    "engafad": "engafar",
    "engafada": "engafar",
    "engafadas": "engafar",
    "engafado": "engafar",
    "engafados": "engafar",
    "engafamientos": "engafamiento",
    "engafamos": "engafar",
    "engafan": "engafar",
    "engafando": "engafar",
    "engafara": "engafar",
    "engafarais": "engafar",
    "engafaran": "engafar",
    "engafaras": "engafar",
    "engafare": "engafar",
    "engafareis": "engafar",
    "engafaremos": "engafar",
    "engafaren": "engafar",
    "engafares": "engafar",
    "engafaron": "engafar",
    "engafará": "engafar",
    "engafarán": "engafar",
    "engafarás": "engafar",
    "engafaré": "engafar",
    "engafaréis": "engafar",
    "engafaría": "engafar",
    "engafaríais": "engafar",
    "engafaríamos": "engafar",
    "engafarían": "engafar",
    "engafarías": "engafar",
    "engafas": "engafar",
    "engafase": "engafar",
    "engafaseis": "engafar",
    "engafasen": "engafar",
    "engafases": "engafar",
    "engafaste": "engafar",
    "engafasteis": "engafar",
    "engafe": "engafar",
    "engafemos": "engafar",
    "engafen": "engafar",
    "engafes": "engafar",
    "engafo": "engafar",
    "engafábamos": "engafar",
    "engafáis": "engafar",
    "engafáramos": "engafar",
    "engafáremos": "engafar",
    "engafásemos": "engafar",
    "engafé": "engafar",
    "engaféis": "engafar",
    "engafó": "engafar",
    "engaita": "engaitar",
    "engaitaba": "engaitar",
    "engaitabais": "engaitar",
    "engaitaban": "engaitar",
    "engaitabas": "engaitar",
    "engaitad": "engaitar",
    "engaitada": "engaitar",
    "engaitadas": "engaitar",
    "engaitado": "engaitar",
    "engaitados": "engaitar",
    "engaitamos": "engaitar",
    "engaitan": "engaitar",
    "engaitando": "engaitar",
    "engaitara": "engaitar",
    "engaitarais": "engaitar",
    "engaitaran": "engaitar",
    "engaitaras": "engaitar",
    "engaitare": "engaitar",
    "engaitareis": "engaitar",
    "engaitaremos": "engaitar",
    "engaitaren": "engaitar",
    "engaitares": "engaitar",
    "engaitaron": "engaitar",
    "engaitará": "engaitar",
    "engaitarán": "engaitar",
    "engaitarás": "engaitar",
    "engaitaré": "engaitar",
    "engaitaréis": "engaitar",
    "engaitaría": "engaitar",
    "engaitaríais": "engaitar",
    "engaitaríamos": "engaitar",
    "engaitarían": "engaitar",
    "engaitarías": "engaitar",
    "engaitas": "engaitar",
    "engaitase": "engaitar",
    "engaitaseis": "engaitar",
    "engaitasen": "engaitar",
    "engaitases": "engaitar",
    "engaitaste": "engaitar",
    "engaitasteis": "engaitar",
    "engaite": "engaitar",
    "engaitemos": "engaitar",
    "engaiten": "engaitar",
    "engaites": "engaitar",
    "engaito": "engaitar",
    "engaitábamos": "engaitar",
    "engaitáis": "engaitar",
    "engaitáramos": "engaitar",
    "engaitáremos": "engaitar",
    "engaitásemos": "engaitar",
    "engaité": "engaitar",
    "engaitéis": "engaitar",
    "engaitó": "engaitar",
    "engalana": "engalanar",
    "engalanaba": "engalanar",
    "engalanabais": "engalanar",
    "engalanaban": "engalanar",
    "engalanabas": "engalanar",
    "engalanad": "engalanar",
    "engalanada": "engalanar",
    "engalanadas": "engalanar",
    "engalanado": "engalanar",
    "engalanados": "engalanar",
    "engalanamientos": "engalanamiento",
    "engalanamos": "engalanar",
    "engalanan": "engalanar",
    "engalanando": "engalanar",
    "engalanara": "engalanar",
    "engalanarais": "engalanar",
    "engalanaran": "engalanar",
    "engalanaras": "engalanar",
    "engalanare": "engalanar",
    "engalanareis": "engalanar",
    "engalanaremos": "engalanar",
    "engalanaren": "engalanar",
    "engalanares": "engalanar",
    "engalanaron": "engalanar",
    "engalanará": "engalanar",
    "engalanarán": "engalanar",
    "engalanarás": "engalanar",
    "engalanaré": "engalanar",
    "engalanaréis": "engalanar",
    "engalanaría": "engalanar",
    "engalanaríais": "engalanar",
    "engalanaríamos": "engalanar",
    "engalanarían": "engalanar",
    "engalanarías": "engalanar",
    "engalanas": "engalanar",
    "engalanase": "engalanar",
    "engalanaseis": "engalanar",
    "engalanasen": "engalanar",
    "engalanases": "engalanar",
    "engalanaste": "engalanar",
    "engalanasteis": "engalanar",
    "engalane": "engalanar",
    "engalanemos": "engalanar",
    "engalanen": "engalanar",
    "engalanes": "engalanar",
    "engalano": "engalanar",
    "engalanábamos": "engalanar",
    "engalanáis": "engalanar",
    "engalanáramos": "engalanar",
    "engalanáremos": "engalanar",
    "engalanásemos": "engalanar",
    "engalané": "engalanar",
    "engalanéis": "engalanar",
    "engalanó": "engalanar",
    "engalga": "engalgar",
    "engalgaba": "engalgar",
    "engalgabais": "engalgar",
    "engalgaban": "engalgar",
    "engalgabas": "engalgar",
    "engalgad": "engalgar",
    "engalgada": "engalgar",
    "engalgadas": "engalgar",
    "engalgado": "engalgar",
    "engalgados": "engalgar",
    "engalgamos": "engalgar",
    "engalgan": "engalgar",
    "engalgando": "engalgar",
    "engalgara": "engalgar",
    "engalgarais": "engalgar",
    "engalgaran": "engalgar",
    "engalgaras": "engalgar",
    "engalgare": "engalgar",
    "engalgareis": "engalgar",
    "engalgaremos": "engalgar",
    "engalgaren": "engalgar",
    "engalgares": "engalgar",
    "engalgaron": "engalgar",
    "engalgará": "engalgar",
    "engalgarán": "engalgar",
    "engalgarás": "engalgar",
    "engalgaré": "engalgar",
    "engalgaréis": "engalgar",
    "engalgaría": "engalgar",
    "engalgaríais": "engalgar",
    "engalgaríamos": "engalgar",
    "engalgarían": "engalgar",
    "engalgarías": "engalgar",
    "engalgas": "engalgar",
    "engalgase": "engalgar",
    "engalgaseis": "engalgar",
    "engalgasen": "engalgar",
    "engalgases": "engalgar",
    "engalgaste": "engalgar",
    "engalgasteis": "engalgar",
    "engalgo": "engalgar",
    "engalgue": "engalgar",
    "engalguemos": "engalgar",
    "engalguen": "engalgar",
    "engalgues": "engalgar",
    "engalgué": "engalgar",
    "engalguéis": "engalgar",
    "engalgábamos": "engalgar",
    "engalgáis": "engalgar",
    "engalgáramos": "engalgar",
    "engalgáremos": "engalgar",
    "engalgásemos": "engalgar",
    "engalgó": "engalgar",
    "engalla": "engallar",
    "engallaba": "engallar",
    "engallabais": "engallar",
    "engallaban": "engallar",
    "engallabas": "engallar",
    "engallad": "engallar",
    "engallada": "engallar",
    "engalladas": "engallar",
    "engallado": "engallar",
    "engalladores": "engallador",
    "engallados": "engallar",
    "engalladuras": "engalladura",
    "engallamos": "engallar",
    "engallan": "engallar",
    "engallando": "engallar",
    "engallara": "engallar",
    "engallarais": "engallar",
    "engallaran": "engallar",
    "engallaras": "engallar",
    "engallare": "engallar",
    "engallareis": "engallar",
    "engallaremos": "engallar",
    "engallaren": "engallar",
    "engallares": "engallar",
    "engallaron": "engallar",
    "engallará": "engallar",
    "engallarán": "engallar",
    "engallarás": "engallar",
    "engallaré": "engallar",
    "engallaréis": "engallar",
    "engallaría": "engallar",
    "engallaríais": "engallar",
    "engallaríamos": "engallar",
    "engallarían": "engallar",
    "engallarías": "engallar",
    "engallas": "engallar",
    "engallase": "engallar",
    "engallaseis": "engallar",
    "engallasen": "engallar",
    "engallases": "engallar",
    "engallaste": "engallar",
    "engallasteis": "engallar",
    "engalle": "engallar",
    "engallemos": "engallar",
    "engallen": "engallar",
    "engalles": "engalle",
    "engallo": "engallar",
    "engallábamos": "engallar",
    "engalláis": "engallar",
    "engalláramos": "engallar",
    "engalláremos": "engallar",
    "engallásemos": "engallar",
    "engallé": "engallar",
    "engalléis": "engallar",
    "engalló": "engallar",
    "engancha": "enganchar",
    "enganchaba": "enganchar",
    "enganchabais": "enganchar",
    "enganchaban": "enganchar",
    "enganchabas": "enganchar",
    "enganchad": "enganchar",
    "enganchada": "enganchar",
    "enganchadas": "enganchar",
    "enganchado": "enganchar",
    "enganchados": "enganchar",
    "enganchamos": "enganchar",
    "enganchan": "enganchar",
    "enganchando": "enganchar",
    "enganchara": "enganchar",
    "engancharais": "enganchar",
    "engancharan": "enganchar",
    "engancharas": "enganchar",
    "enganchare": "enganchar",
    "enganchareis": "enganchar",
    "engancharemos": "enganchar",
    "engancharen": "enganchar",
    "enganchares": "enganchar",
    "engancharon": "enganchar",
    "enganchará": "enganchar",
    "engancharán": "enganchar",
    "engancharás": "enganchar",
    "engancharé": "enganchar",
    "engancharéis": "enganchar",
    "engancharía": "enganchar",
    "engancharíais": "enganchar",
    "engancharíamos": "enganchar",
    "engancharían": "enganchar",
    "engancharías": "enganchar",
    "enganchas": "enganchar",
    "enganchase": "enganchar",
    "enganchaseis": "enganchar",
    "enganchasen": "enganchar",
    "enganchases": "enganchar",
    "enganchaste": "enganchar",
    "enganchasteis": "enganchar",
    "enganche": "enganchar",
    "enganchemos": "enganchar",
    "enganchen": "enganchar",
    "enganches": "enganche",
    "engancho": "enganchar",
    "enganchones": "enganchón",
    "enganchábamos": "enganchar",
    "engancháis": "enganchar",
    "engancháramos": "enganchar",
    "engancháremos": "enganchar",
    "enganchásemos": "enganchar",
    "enganché": "enganchar",
    "enganchéis": "enganchar",
    "enganchó": "enganchar",
    "engarabata": "engarabatar",
    "engarabataba": "engarabatar",
    "engarabatabais": "engarabatar",
    "engarabataban": "engarabatar",
    "engarabatabas": "engarabatar",
    "engarabatad": "engarabatar",
    "engarabatada": "engarabatar",
    "engarabatadas": "engarabatar",
    "engarabatado": "engarabatar",
    "engarabatados": "engarabatar",
    "engarabatamos": "engarabatar",
    "engarabatan": "engarabatar",
    "engarabatando": "engarabatar",
    "engarabatara": "engarabatar",
    "engarabatarais": "engarabatar",
    "engarabataran": "engarabatar",
    "engarabataras": "engarabatar",
    "engarabatare": "engarabatar",
    "engarabatareis": "engarabatar",
    "engarabataremos": "engarabatar",
    "engarabataren": "engarabatar",
    "engarabatares": "engarabatar",
    "engarabataron": "engarabatar",
    "engarabatará": "engarabatar",
    "engarabatarán": "engarabatar",
    "engarabatarás": "engarabatar",
    "engarabataré": "engarabatar",
    "engarabataréis": "engarabatar",
    "engarabataría": "engarabatar",
    "engarabataríais": "engarabatar",
    "engarabataríamos": "engarabatar",
    "engarabatarían": "engarabatar",
    "engarabatarías": "engarabatar",
    "engarabatas": "engarabatar",
    "engarabatase": "engarabatar",
    "engarabataseis": "engarabatar",
    "engarabatasen": "engarabatar",
    "engarabatases": "engarabatar",
    "engarabataste": "engarabatar",
    "engarabatasteis": "engarabatar",
    "engarabate": "engarabatar",
    "engarabatemos": "engarabatar",
    "engarabaten": "engarabatar",
    "engarabates": "engarabatar",
    "engarabato": "engarabatar",
    "engarabatábamos": "engarabatar",
    "engarabatáis": "engarabatar",
    "engarabatáramos": "engarabatar",
    "engarabatáremos": "engarabatar",
    "engarabatásemos": "engarabatar",
    "engarabaté": "engarabatar",
    "engarabatéis": "engarabatar",
    "engarabató": "engarabatar",
    "engarabita": "engarabitar",
    "engarabitaba": "engarabitar",
    "engarabitabais": "engarabitar",
    "engarabitaban": "engarabitar",
    "engarabitabas": "engarabitar",
    "engarabitad": "engarabitar",
    "engarabitada": "engarabitar",
    "engarabitadas": "engarabitar",
    "engarabitado": "engarabitar",
    "engarabitados": "engarabitar",
    "engarabitamos": "engarabitar",
    "engarabitan": "engarabitar",
    "engarabitando": "engarabitar",
    "engarabitara": "engarabitar",
    "engarabitarais": "engarabitar",
    "engarabitaran": "engarabitar",
    "engarabitaras": "engarabitar",
    "engarabitare": "engarabitar",
    "engarabitareis": "engarabitar",
    "engarabitaremos": "engarabitar",
    "engarabitaren": "engarabitar",
    "engarabitares": "engarabitar",
    "engarabitaron": "engarabitar",
    "engarabitará": "engarabitar",
    "engarabitarán": "engarabitar",
    "engarabitarás": "engarabitar",
    "engarabitaré": "engarabitar",
    "engarabitaréis": "engarabitar",
    "engarabitaría": "engarabitar",
    "engarabitaríais": "engarabitar",
    "engarabitaríamos": "engarabitar",
    "engarabitarían": "engarabitar",
    "engarabitarías": "engarabitar",
    "engarabitas": "engarabitar",
    "engarabitase": "engarabitar",
    "engarabitaseis": "engarabitar",
    "engarabitasen": "engarabitar",
    "engarabitases": "engarabitar",
    "engarabitaste": "engarabitar",
    "engarabitasteis": "engarabitar",
    "engarabite": "engarabitar",
    "engarabitemos": "engarabitar",
    "engarabiten": "engarabitar",
    "engarabites": "engarabitar",
    "engarabito": "engarabitar",
    "engarabitábamos": "engarabitar",
    "engarabitáis": "engarabitar",
    "engarabitáramos": "engarabitar",
    "engarabitáremos": "engarabitar",
    "engarabitásemos": "engarabitar",
    "engarabité": "engarabitar",
    "engarabitéis": "engarabitar",
    "engarabitó": "engarabitar",
    "engarba": "engarbar",
    "engarbaba": "engarbar",
    "engarbabais": "engarbar",
    "engarbaban": "engarbar",
    "engarbabas": "engarbar",
    "engarbad": "engarbar",
    "engarbada": "engarbar",
    "engarbadas": "engarbar",
    "engarbado": "engarbar",
    "engarbados": "engarbar",
    "engarbamos": "engarbar",
    "engarban": "engarbar",
    "engarbando": "engarbar",
    "engarbara": "engarbar",
    "engarbarais": "engarbar",
    "engarbaran": "engarbar",
    "engarbaras": "engarbar",
    "engarbare": "engarbar",
    "engarbareis": "engarbar",
    "engarbaremos": "engarbar",
    "engarbaren": "engarbar",
    "engarbares": "engarbar",
    "engarbaron": "engarbar",
    "engarbará": "engarbar",
    "engarbarán": "engarbar",
    "engarbarás": "engarbar",
    "engarbaré": "engarbar",
    "engarbaréis": "engarbar",
    "engarbaría": "engarbar",
    "engarbaríais": "engarbar",
    "engarbaríamos": "engarbar",
    "engarbarían": "engarbar",
    "engarbarías": "engarbar",
    "engarbas": "engarbar",
    "engarbase": "engarbar",
    "engarbaseis": "engarbar",
    "engarbasen": "engarbar",
    "engarbases": "engarbar",
    "engarbaste": "engarbar",
    "engarbasteis": "engarbar",
    "engarbe": "engarbar",
    "engarbemos": "engarbar",
    "engarben": "engarbar",
    "engarbes": "engarbar",
    "engarbo": "engarbar",
    "engarbábamos": "engarbar",
    "engarbáis": "engarbar",
    "engarbáramos": "engarbar",
    "engarbáremos": "engarbar",
    "engarbásemos": "engarbar",
    "engarbé": "engarbar",
    "engarbéis": "engarbar",
    "engarbó": "engarbar",
    "engarce": "engarzar",
    "engarcemos": "engarzar",
    "engarcen": "engarzar",
    "engarces": "engarzar",
    "engarcé": "engarzar",
    "engarcéis": "engarzar",
    "engarganta": "engargantar",
    "engargantaba": "engargantar",
    "engargantabais": "engargantar",
    "engargantaban": "engargantar",
    "engargantabas": "engargantar",
    "engargantad": "engargantar",
    "engargantada": "engargantar",
    "engargantadas": "engargantar",
    "engargantado": "engargantar",
    "engargantados": "engargantar",
    "engargantaduras": "engargantadura",
    "engargantamos": "engargantar",
    "engargantan": "engargantar",
    "engargantando": "engargantar",
    "engargantara": "engargantar",
    "engargantarais": "engargantar",
    "engargantaran": "engargantar",
    "engargantaras": "engargantar",
    "engargantare": "engargantar",
    "engargantareis": "engargantar",
    "engargantaremos": "engargantar",
    "engargantaren": "engargantar",
    "engargantares": "engargantar",
    "engargantaron": "engargantar",
    "engargantará": "engargantar",
    "engargantarán": "engargantar",
    "engargantarás": "engargantar",
    "engargantaré": "engargantar",
    "engargantaréis": "engargantar",
    "engargantaría": "engargantar",
    "engargantaríais": "engargantar",
    "engargantaríamos": "engargantar",
    "engargantarían": "engargantar",
    "engargantarías": "engargantar",
    "engargantas": "engargantar",
    "engargantase": "engargantar",
    "engargantaseis": "engargantar",
    "engargantasen": "engargantar",
    "engargantases": "engargantar",
    "engargantaste": "engargantar",
    "engargantasteis": "engargantar",
    "engargante": "engargantar",
    "engargantemos": "engargantar",
    "engarganten": "engargantar",
    "engargantes": "engargante",
    "engarganto": "engargantar",
    "engargantábamos": "engargantar",
    "engargantáis": "engargantar",
    "engargantáramos": "engargantar",
    "engargantáremos": "engargantar",
    "engargantásemos": "engargantar",
    "engarganté": "engargantar",
    "engargantéis": "engargantar",
    "engargantó": "engargantar",
    "engargola": "engargolar",
    "engargolaba": "engargolar",
    "engargolabais": "engargolar",
    "engargolaban": "engargolar",
    "engargolabas": "engargolar",
    "engargolad": "engargolar",
    "engargolada": "engargolar",
    "engargoladas": "engargolar",
    "engargolado": "engargolar",
    "engargolados": "engargolar",
    "engargolamos": "engargolar",
    "engargolan": "engargolar",
    "engargolando": "engargolar",
    "engargolara": "engargolar",
    "engargolarais": "engargolar",
    "engargolaran": "engargolar",
    "engargolaras": "engargolar",
    "engargolare": "engargolar",
    "engargolareis": "engargolar",
    "engargolaremos": "engargolar",
    "engargolaren": "engargolar",
    "engargolares": "engargolar",
    "engargolaron": "engargolar",
    "engargolará": "engargolar",
    "engargolarán": "engargolar",
    "engargolarás": "engargolar",
    "engargolaré": "engargolar",
    "engargolaréis": "engargolar",
    "engargolaría": "engargolar",
    "engargolaríais": "engargolar",
    "engargolaríamos": "engargolar",
    "engargolarían": "engargolar",
    "engargolarías": "engargolar",
    "engargolas": "engargolar",
    "engargolase": "engargolar",
    "engargolaseis": "engargolar",
    "engargolasen": "engargolar",
    "engargolases": "engargolar",
    "engargolaste": "engargolar",
    "engargolasteis": "engargolar",
    "engargole": "engargolar",
    "engargolemos": "engargolar",
    "engargolen": "engargolar",
    "engargoles": "engargolar",
    "engargolo": "engargolar",
    "engargolábamos": "engargolar",
    "engargoláis": "engargolar",
    "engargoláramos": "engargolar",
    "engargoláremos": "engargolar",
    "engargolásemos": "engargolar",
    "engargolé": "engargolar",
    "engargoléis": "engargolar",
    "engargoló": "engargolar",
    "engarnios": "engarnio",
    "engarrafa": "engarrafar",
    "engarrafaba": "engarrafar",
    "engarrafabais": "engarrafar",
    "engarrafaban": "engarrafar",
    "engarrafabas": "engarrafar",
    "engarrafad": "engarrafar",
    "engarrafada": "engarrafar",
    "engarrafadas": "engarrafar",
    "engarrafado": "engarrafar",
    "engarrafados": "engarrafar",
    "engarrafamos": "engarrafar",
    "engarrafan": "engarrafar",
    "engarrafando": "engarrafar",
    "engarrafara": "engarrafar",
    "engarrafarais": "engarrafar",
    "engarrafaran": "engarrafar",
    "engarrafaras": "engarrafar",
    "engarrafare": "engarrafar",
    "engarrafareis": "engarrafar",
    "engarrafaremos": "engarrafar",
    "engarrafaren": "engarrafar",
    "engarrafares": "engarrafar",
    "engarrafaron": "engarrafar",
    "engarrafará": "engarrafar",
    "engarrafarán": "engarrafar",
    "engarrafarás": "engarrafar",
    "engarrafaré": "engarrafar",
    "engarrafaréis": "engarrafar",
    "engarrafaría": "engarrafar",
    "engarrafaríais": "engarrafar",
    "engarrafaríamos": "engarrafar",
    "engarrafarían": "engarrafar",
    "engarrafarías": "engarrafar",
    "engarrafas": "engarrafar",
    "engarrafase": "engarrafar",
    "engarrafaseis": "engarrafar",
    "engarrafasen": "engarrafar",
    "engarrafases": "engarrafar",
    "engarrafaste": "engarrafar",
    "engarrafasteis": "engarrafar",
    "engarrafe": "engarrafar",
    "engarrafemos": "engarrafar",
    "engarrafen": "engarrafar",
    "engarrafes": "engarrafar",
    "engarrafo": "engarrafar",
    "engarrafábamos": "engarrafar",
    "engarrafáis": "engarrafar",
    "engarrafáramos": "engarrafar",
    "engarrafáremos": "engarrafar",
    "engarrafásemos": "engarrafar",
    "engarrafé": "engarrafar",
    "engarraféis": "engarrafar",
    "engarrafó": "engarrafar",
    "engarros": "engarro",
    "engarrota": "engarrotar",
    "engarrotaba": "engarrotar",
    "engarrotabais": "engarrotar",
    "engarrotaban": "engarrotar",
    "engarrotabas": "engarrotar",
    "engarrotad": "engarrotar",
    "engarrotada": "engarrotar",
    "engarrotadas": "engarrotar",
    "engarrotado": "engarrotar",
    "engarrotados": "engarrotar",
    "engarrotamos": "engarrotar",
    "engarrotan": "engarrotar",
    "engarrotando": "engarrotar",
    "engarrotara": "engarrotar",
    "engarrotarais": "engarrotar",
    "engarrotaran": "engarrotar",
    "engarrotaras": "engarrotar",
    "engarrotare": "engarrotar",
    "engarrotareis": "engarrotar",
    "engarrotaremos": "engarrotar",
    "engarrotaren": "engarrotar",
    "engarrotares": "engarrotar",
    "engarrotaron": "engarrotar",
    "engarrotará": "engarrotar",
    "engarrotarán": "engarrotar",
    "engarrotarás": "engarrotar",
    "engarrotaré": "engarrotar",
    "engarrotaréis": "engarrotar",
    "engarrotaría": "engarrotar",
    "engarrotaríais": "engarrotar",
    "engarrotaríamos": "engarrotar",
    "engarrotarían": "engarrotar",
    "engarrotarías": "engarrotar",
    "engarrotas": "engarrotar",
    "engarrotase": "engarrotar",
    "engarrotaseis": "engarrotar",
    "engarrotasen": "engarrotar",
    "engarrotases": "engarrotar",
    "engarrotaste": "engarrotar",
    "engarrotasteis": "engarrotar",
    "engarrote": "engarrotar",
    "engarrotemos": "engarrotar",
    "engarroten": "engarrotar",
    "engarrotes": "engarrotar",
    "engarroto": "engarrotar",
    "engarrotábamos": "engarrotar",
    "engarrotáis": "engarrotar",
    "engarrotáramos": "engarrotar",
    "engarrotáremos": "engarrotar",
    "engarrotásemos": "engarrotar",
    "engarroté": "engarrotar",
    "engarrotéis": "engarrotar",
    "engarrotó": "engarrotar",
    "engarza": "engarzar",
    "engarzaba": "engarzar",
    "engarzabais": "engarzar",
    "engarzaban": "engarzar",
    "engarzabas": "engarzar",
    "engarzad": "engarzar",
    "engarzada": "engarzar",
    "engarzadas": "engarzar",
    "engarzado": "engarzar",
    "engarzadora": "engarzador",
    "engarzadoras": "engarzador",
    "engarzadores": "engarzador",
    "engarzados": "engarzar",
    "engarzaduras": "engarzadura",
    "engarzamos": "engarzar",
    "engarzan": "engarzar",
    "engarzando": "engarzar",
    "engarzara": "engarzar",
    "engarzarais": "engarzar",
    "engarzaran": "engarzar",
    "engarzaras": "engarzar",
    "engarzare": "engarzar",
    "engarzareis": "engarzar",
    "engarzaremos": "engarzar",
    "engarzaren": "engarzar",
    "engarzares": "engarzar",
    "engarzaron": "engarzar",
    "engarzará": "engarzar",
    "engarzarán": "engarzar",
    "engarzarás": "engarzar",
    "engarzaré": "engarzar",
    "engarzaréis": "engarzar",
    "engarzaría": "engarzar",
    "engarzaríais": "engarzar",
    "engarzaríamos": "engarzar",
    "engarzarían": "engarzar",
    "engarzarías": "engarzar",
    "engarzas": "engarzar",
    "engarzase": "engarzar",
    "engarzaseis": "engarzar",
    "engarzasen": "engarzar",
    "engarzases": "engarzar",
    "engarzaste": "engarzar",
    "engarzasteis": "engarzar",
    "engarzo": "engarzar",
    "engarzábamos": "engarzar",
    "engarzáis": "engarzar",
    "engarzáramos": "engarzar",
    "engarzáremos": "engarzar",
    "engarzásemos": "engarzar",
    "engarzó": "engarzar",
    "engasta": "engastar",
    "engastaba": "engastar",
    "engastabais": "engastar",
    "engastaban": "engastar",
    "engastabas": "engastar",
    "engastad": "engastar",
    "engastada": "engastar",
    "engastadas": "engastar",
    "engastado": "engastar",
    "engastadora": "engastador",
    "engastadoras": "engastador",
    "engastadores": "engastador",
    "engastados": "engastar",
    "engastaduras": "engastadura",
    "engastamos": "engastar",
    "engastan": "engastar",
    "engastando": "engastar",
    "engastara": "engastar",
    "engastarais": "engastar",
    "engastaran": "engastar",
    "engastaras": "engastar",
    "engastare": "engastar",
    "engastareis": "engastar",
    "engastaremos": "engastar",
    "engastaren": "engastar",
    "engastares": "engastar",
    "engastaron": "engastar",
    "engastará": "engastar",
    "engastarán": "engastar",
    "engastarás": "engastar",
    "engastaré": "engastar",
    "engastaréis": "engastar",
    "engastaría": "engastar",
    "engastaríais": "engastar",
    "engastaríamos": "engastar",
    "engastarían": "engastar",
    "engastarías": "engastar",
    "engastas": "engastar",
    "engastase": "engastar",
    "engastaseis": "engastar",
    "engastasen": "engastar",
    "engastases": "engastar",
    "engastaste": "engastar",
    "engastasteis": "engastar",
    "engaste": "engastar",
    "engastemos": "engastar",
    "engasten": "engastar",
    "engastes": "engaste",
    "engasto": "engastar",
    "engastábamos": "engastar",
    "engastáis": "engastar",
    "engastáramos": "engastar",
    "engastáremos": "engastar",
    "engastásemos": "engastar",
    "engasté": "engastar",
    "engastéis": "engastar",
    "engastó": "engastar",
    "engatada": "engatado",
    "engatadas": "engatado",
    "engatados": "engatado",
    "engatilla": "engatillar",
    "engatillaba": "engatillar",
    "engatillabais": "engatillar",
    "engatillaban": "engatillar",
    "engatillabas": "engatillar",
    "engatillad": "engatillar",
    "engatillada": "engatillar",
    "engatilladas": "engatillar",
    "engatillado": "engatillar",
    "engatillados": "engatillar",
    "engatillamos": "engatillar",
    "engatillan": "engatillar",
    "engatillando": "engatillar",
    "engatillara": "engatillar",
    "engatillarais": "engatillar",
    "engatillaran": "engatillar",
    "engatillaras": "engatillar",
    "engatillare": "engatillar",
    "engatillareis": "engatillar",
    "engatillaremos": "engatillar",
    "engatillaren": "engatillar",
    "engatillares": "engatillar",
    "engatillaron": "engatillar",
    "engatillará": "engatillar",
    "engatillarán": "engatillar",
    "engatillarás": "engatillar",
    "engatillaré": "engatillar",
    "engatillaréis": "engatillar",
    "engatillaría": "engatillar",
    "engatillaríais": "engatillar",
    "engatillaríamos": "engatillar",
    "engatillarían": "engatillar",
    "engatillarías": "engatillar",
    "engatillas": "engatillar",
    "engatillase": "engatillar",
    "engatillaseis": "engatillar",
    "engatillasen": "engatillar",
    "engatillases": "engatillar",
    "engatillaste": "engatillar",
    "engatillasteis": "engatillar",
    "engatille": "engatillar",
    "engatillemos": "engatillar",
    "engatillen": "engatillar",
    "engatilles": "engatillar",
    "engatillo": "engatillar",
    "engatillábamos": "engatillar",
    "engatilláis": "engatillar",
    "engatilláramos": "engatillar",
    "engatilláremos": "engatillar",
    "engatillásemos": "engatillar",
    "engatillé": "engatillar",
    "engatilléis": "engatillar",
    "engatilló": "engatillar",
    "engatusa": "engatusar",
    "engatusaba": "engatusar",
    "engatusabais": "engatusar",
    "engatusaban": "engatusar",
    "engatusabas": "engatusar",
    "engatusad": "engatusar",
    "engatusada": "engatusar",
    "engatusadas": "engatusar",
    "engatusado": "engatusar",
    "engatusadora": "engatusador",
    "engatusadoras": "engatusador",
    "engatusadores": "engatusador",
    "engatusados": "engatusar",
    "engatusamientos": "engatusamiento",
    "engatusamos": "engatusar",
    "engatusan": "engatusar",
    "engatusando": "engatusar",
    "engatusara": "engatusar",
    "engatusarais": "engatusar",
    "engatusaran": "engatusar",
    "engatusaras": "engatusar",
    "engatusare": "engatusar",
    "engatusareis": "engatusar",
    "engatusaremos": "engatusar",
    "engatusaren": "engatusar",
    "engatusares": "engatusar",
    "engatusaron": "engatusar",
    "engatusará": "engatusar",
    "engatusarán": "engatusar",
    "engatusarás": "engatusar",
    "engatusaré": "engatusar",
    "engatusaréis": "engatusar",
    "engatusaría": "engatusar",
    "engatusaríais": "engatusar",
    "engatusaríamos": "engatusar",
    "engatusarían": "engatusar",
    "engatusarías": "engatusar",
    "engatusas": "engatusar",
    "engatusase": "engatusar",
    "engatusaseis": "engatusar",
    "engatusasen": "engatusar",
    "engatusases": "engatusar",
    "engatusaste": "engatusar",
    "engatusasteis": "engatusar",
    "engatuse": "engatusar",
    "engatusemos": "engatusar",
    "engatusen": "engatusar",
    "engatuses": "engatusar",
    "engatuso": "engatusar",
    "engatusábamos": "engatusar",
    "engatusáis": "engatusar",
    "engatusáramos": "engatusar",
    "engatusáremos": "engatusar",
    "engatusásemos": "engatusar",
    "engatusé": "engatusar",
    "engatuséis": "engatusar",
    "engatusó": "engatusar",
    "engavilla": "engavillar",
    "engavillaba": "engavillar",
    "engavillabais": "engavillar",
    "engavillaban": "engavillar",
    "engavillabas": "engavillar",
    "engavillad": "engavillar",
    "engavillada": "engavillar",
    "engavilladas": "engavillar",
    "engavillado": "engavillar",
    "engavillados": "engavillar",
    "engavillamos": "engavillar",
    "engavillan": "engavillar",
    "engavillando": "engavillar",
    "engavillara": "engavillar",
    "engavillarais": "engavillar",
    "engavillaran": "engavillar",
    "engavillaras": "engavillar",
    "engavillare": "engavillar",
    "engavillareis": "engavillar",
    "engavillaremos": "engavillar",
    "engavillaren": "engavillar",
    "engavillares": "engavillar",
    "engavillaron": "engavillar",
    "engavillará": "engavillar",
    "engavillarán": "engavillar",
    "engavillarás": "engavillar",
    "engavillaré": "engavillar",
    "engavillaréis": "engavillar",
    "engavillaría": "engavillar",
    "engavillaríais": "engavillar",
    "engavillaríamos": "engavillar",
    "engavillarían": "engavillar",
    "engavillarías": "engavillar",
    "engavillas": "engavillar",
    "engavillase": "engavillar",
    "engavillaseis": "engavillar",
    "engavillasen": "engavillar",
    "engavillases": "engavillar",
    "engavillaste": "engavillar",
    "engavillasteis": "engavillar",
    "engaville": "engavillar",
    "engavillemos": "engavillar",
    "engavillen": "engavillar",
    "engavilles": "engavillar",
    "engavillo": "engavillar",
    "engavillábamos": "engavillar",
    "engavilláis": "engavillar",
    "engavilláramos": "engavillar",
    "engavilláremos": "engavillar",
    "engavillásemos": "engavillar",
    "engavillé": "engavillar",
    "engavilléis": "engavillar",
    "engavilló": "engavillar",
    "engaza": "engazar",
    "engazaba": "engazar",
    "engazabais": "engazar",
    "engazaban": "engazar",
    "engazabas": "engazar",
    "engazad": "engazar",
    "engazada": "engazar",
    "engazadas": "engazar",
    "engazado": "engazar",
    "engazados": "engazar",
    "engazamos": "engazar",
    "engazan": "engazar",
    "engazando": "engazar",
    "engazara": "engazar",
    "engazarais": "engazar",
    "engazaran": "engazar",
    "engazaras": "engazar",
    "engazare": "engazar",
    "engazareis": "engazar",
    "engazaremos": "engazar",
    "engazaren": "engazar",
    "engazares": "engazar",
    "engazaron": "engazar",
    "engazará": "engazar",
    "engazarán": "engazar",
    "engazarás": "engazar",
    "engazaré": "engazar",
    "engazaréis": "engazar",
    "engazaría": "engazar",
    "engazaríais": "engazar",
    "engazaríamos": "engazar",
    "engazarían": "engazar",
    "engazarías": "engazar",
    "engazas": "engazar",
    "engazase": "engazar",
    "engazaseis": "engazar",
    "engazasen": "engazar",
    "engazases": "engazar",
    "engazaste": "engazar",
    "engazasteis": "engazar",
    "engazo": "engazar",
    "engazábamos": "engazar",
    "engazáis": "engazar",
    "engazáramos": "engazar",
    "engazáremos": "engazar",
    "engazásemos": "engazar",
    "engazó": "engazar",
    "engaña": "engañar",
    "engañaba": "engañar",
    "engañabais": "engañar",
    "engañaban": "engañar",
    "engañabas": "engañar",
    "engañad": "engañar",
    "engañada": "engañar",
    "engañadas": "engañar",
    "engañadiza": "engañadizo",
    "engañadizas": "engañadizo",
    "engañadizos": "engañadizo",
    "engañado": "engañar",
    "engañadora": "engañador",
    "engañadoras": "engañador",
    "engañadores": "engañador",
    "engañados": "engañar",
    "engañamos": "engañar",
    "engañan": "engañar",
    "engañando": "engañar",
    "engañara": "engañar",
    "engañarais": "engañar",
    "engañaran": "engañar",
    "engañaras": "engañar",
    "engañare": "engañar",
    "engañareis": "engañar",
    "engañaremos": "engañar",
    "engañaren": "engañar",
    "engañares": "engañar",
    "engañaron": "engañar",
    "engañará": "engañar",
    "engañarán": "engañar",
    "engañarás": "engañar",
    "engañaré": "engañar",
    "engañaréis": "engañar",
    "engañaría": "engañar",
    "engañaríais": "engañar",
    "engañaríamos": "engañar",
    "engañarían": "engañar",
    "engañarías": "engañar",
    "engañas": "engañar",
    "engañase": "engañar",
    "engañaseis": "engañar",
    "engañasen": "engañar",
    "engañases": "engañar",
    "engañaste": "engañar",
    "engañasteis": "engañar",
    "engañe": "engañar",
    "engañemos": "engañar",
    "engañen": "engañar",
    "engañes": "engañar",
    "engañifas": "engañifa",
    "engaño": "engañar",
    "engaños": "engaño",
    "engañosa": "engañoso",
    "engañosas": "engañoso",
    "engañosos": "engañoso",
    "engañábamos": "engañar",
    "engañáis": "engañar",
    "engañáramos": "engañar",
    "engañáremos": "engañar",
    "engañásemos": "engañar",
    "engañé": "engañar",
    "engañéis": "engañar",
    "engañó": "engañar",
    "engendra": "engendrar",
    "engendraba": "engendrar",
    "engendrabais": "engendrar",
    "engendraban": "engendrar",
    "engendrabas": "engendrar",
    "engendrad": "engendrar",
    "engendrada": "engendrar",
    "engendradas": "engendrar",
    "engendrado": "engendrar",
    "engendradora": "engendrador",
    "engendradoras": "engendrador",
    "engendradores": "engendrador",
    "engendrados": "engendrar",
    "engendramientos": "engendramiento",
    "engendramos": "engendrar",
    "engendran": "engendrar",
    "engendrando": "engendrar",
    "engendrara": "engendrar",
    "engendrarais": "engendrar",
    "engendraran": "engendrar",
    "engendraras": "engendrar",
    "engendrare": "engendrar",
    "engendrareis": "engendrar",
    "engendraremos": "engendrar",
    "engendraren": "engendrar",
    "engendrares": "engendrar",
    "engendraron": "engendrar",
    "engendrará": "engendrar",
    "engendrarán": "engendrar",
    "engendrarás": "engendrar",
    "engendraré": "engendrar",
    "engendraréis": "engendrar",
    "engendraría": "engendrar",
    "engendraríais": "engendrar",
    "engendraríamos": "engendrar",
    "engendrarían": "engendrar",
    "engendrarías": "engendrar",
    "engendras": "engendrar",
    "engendrase": "engendrar",
    "engendraseis": "engendrar",
    "engendrasen": "engendrar",
    "engendrases": "engendrar",
    "engendraste": "engendrar",
    "engendrasteis": "engendrar",
    "engendre": "engendrar",
    "engendremos": "engendrar",
    "engendren": "engendrar",
    "engendres": "engendrar",
    "engendro": "engendrar",
    "engendros": "engendro",
    "engendrábamos": "engendrar",
    "engendráis": "engendrar",
    "engendráramos": "engendrar",
    "engendráremos": "engendrar",
    "engendrásemos": "engendrar",
    "engendré": "engendrar",
    "engendréis": "engendrar",
    "engendró": "engendrar",
    "engerid": "engerir",
    "engerida": "engerir",
    "engeridas": "engerir",
    "engerido": "engerir",
    "engeridos": "engerir",
    "engerimos": "engerir",
    "engeriremos": "engerir",
    "engerirá": "engerir",
    "engerirán": "engerir",
    "engerirás": "engerir",
    "engeriré": "engerir",
    "engeriréis": "engerir",
    "engeriría": "engerir",
    "engeriríais": "engerir",
    "engeriríamos": "engerir",
    "engerirían": "engerir",
    "engerirías": "engerir",
    "engeriste": "engerir",
    "engeristeis": "engerir",
    "engerí": "engerir",
    "engería": "engerir",
    "engeríais": "engerir",
    "engeríamos": "engerir",
    "engerían": "engerir",
    "engerías": "engerir",
    "engerís": "engerir",
    "engestada": "engestado",
    "engestadas": "engestado",
    "engestados": "engestado",
    "engiba": "engibar",
    "engibaba": "engibar",
    "engibabais": "engibar",
    "engibaban": "engibar",
    "engibabas": "engibar",
    "engibad": "engibar",
    "engibada": "engibar",
    "engibadas": "engibar",
    "engibado": "engibar",
    "engibados": "engibar",
    "engibamos": "engibar",
    "engiban": "engibar",
    "engibando": "engibar",
    "engibara": "engibar",
    "engibarais": "engibar",
    "engibaran": "engibar",
    "engibaras": "engibar",
    "engibare": "engibar",
    "engibareis": "engibar",
    "engibaremos": "engibar",
    "engibaren": "engibar",
    "engibares": "engibar",
    "engibaron": "engibar",
    "engibará": "engibar",
    "engibarán": "engibar",
    "engibarás": "engibar",
    "engibaré": "engibar",
    "engibaréis": "engibar",
    "engibaría": "engibar",
    "engibaríais": "engibar",
    "engibaríamos": "engibar",
    "engibarían": "engibar",
    "engibarías": "engibar",
    "engibas": "engibar",
    "engibase": "engibar",
    "engibaseis": "engibar",
    "engibasen": "engibar",
    "engibases": "engibar",
    "engibaste": "engibar",
    "engibasteis": "engibar",
    "engibe": "engibar",
    "engibemos": "engibar",
    "engiben": "engibar",
    "engibes": "engibar",
    "engibo": "engibar",
    "engibábamos": "engibar",
    "engibáis": "engibar",
    "engibáramos": "engibar",
    "engibáremos": "engibar",
    "engibásemos": "engibar",
    "engibé": "engibar",
    "engibéis": "engibar",
    "engibó": "engibar",
    "engiera": "engerir",
    "engieran": "engerir",
    "engieras": "engerir",
    "engiere": "engerir",
    "engieren": "engerir",
    "engieres": "engerir",
    "engiero": "engerir",
    "engiramos": "engerir",
    "engiriendo": "engerir",
    "engiriera": "engerir",
    "engirierais": "engerir",
    "engirieran": "engerir",
    "engirieras": "engerir",
    "engiriere": "engerir",
    "engiriereis": "engerir",
    "engirieren": "engerir",
    "engirieres": "engerir",
    "engirieron": "engerir",
    "engiriese": "engerir",
    "engirieseis": "engerir",
    "engiriesen": "engerir",
    "engirieses": "engerir",
    "engiriéramos": "engerir",
    "engiriéremos": "engerir",
    "engiriésemos": "engerir",
    "engirió": "engerir",
    "engiráis": "engerir",
    "englandada": "englandado",
    "englandadas": "englandado",
    "englandados": "englandado",
    "englantada": "englantado",
    "englantadas": "englantado",
    "englantados": "englantado",
    "engloba": "englobar",
    "englobaba": "englobar",
    "englobabais": "englobar",
    "englobaban": "englobar",
    "englobabas": "englobar",
    "englobad": "englobar",
    "englobada": "englobar",
    "englobadas": "englobar",
    "englobado": "englobar",
    "englobados": "englobar",
    "englobamientos": "englobamiento",
    "englobamos": "englobar",
    "engloban": "englobar",
    "englobando": "englobar",
    "englobara": "englobar",
    "englobarais": "englobar",
    "englobaran": "englobar",
    "englobaras": "englobar",
    "englobare": "englobar",
    "englobareis": "englobar",
    "englobaremos": "englobar",
    "englobaren": "englobar",
    "englobares": "englobar",
    "englobaron": "englobar",
    "englobará": "englobar",
    "englobarán": "englobar",
    "englobarás": "englobar",
    "englobaré": "englobar",
    "englobaréis": "englobar",
    "englobaría": "englobar",
    "englobaríais": "englobar",
    "englobaríamos": "englobar",
    "englobarían": "englobar",
    "englobarías": "englobar",
    "englobas": "englobar",
    "englobase": "englobar",
    "englobaseis": "englobar",
    "englobasen": "englobar",
    "englobases": "englobar",
    "englobaste": "englobar",
    "englobasteis": "englobar",
    "englobe": "englobar",
    "englobemos": "englobar",
    "engloben": "englobar",
    "englobes": "englobar",
    "englobo": "englobar",
    "englobábamos": "englobar",
    "englobáis": "englobar",
    "englobáramos": "englobar",
    "englobáremos": "englobar",
    "englobásemos": "englobar",
    "englobé": "englobar",
    "englobéis": "englobar",
    "englobó": "englobar",
    "engolada": "engolado",
    "engoladas": "engolado",
    "engolados": "engolado",
    "engolamientos": "engolamiento",
    "engolfa": "engolfar",
    "engolfaba": "engolfar",
    "engolfabais": "engolfar",
    "engolfaban": "engolfar",
    "engolfabas": "engolfar",
    "engolfad": "engolfar",
    "engolfada": "engolfar",
    "engolfadas": "engolfar",
    "engolfado": "engolfar",
    "engolfados": "engolfar",
    "engolfamos": "engolfar",
    "engolfan": "engolfar",
    "engolfando": "engolfar",
    "engolfara": "engolfar",
    "engolfarais": "engolfar",
    "engolfaran": "engolfar",
    "engolfaras": "engolfar",
    "engolfare": "engolfar",
    "engolfareis": "engolfar",
    "engolfaremos": "engolfar",
    "engolfaren": "engolfar",
    "engolfares": "engolfar",
    "engolfaron": "engolfar",
    "engolfará": "engolfar",
    "engolfarán": "engolfar",
    "engolfarás": "engolfar",
    "engolfaré": "engolfar",
    "engolfaréis": "engolfar",
    "engolfaría": "engolfar",
    "engolfaríais": "engolfar",
    "engolfaríamos": "engolfar",
    "engolfarían": "engolfar",
    "engolfarías": "engolfar",
    "engolfas": "engolfar",
    "engolfase": "engolfar",
    "engolfaseis": "engolfar",
    "engolfasen": "engolfar",
    "engolfases": "engolfar",
    "engolfaste": "engolfar",
    "engolfasteis": "engolfar",
    "engolfe": "engolfar",
    "engolfemos": "engolfar",
    "engolfen": "engolfar",
    "engolfes": "engolfar",
    "engolfo": "engolfar",
    "engolfábamos": "engolfar",
    "engolfáis": "engolfar",
    "engolfáramos": "engolfar",
    "engolfáremos": "engolfar",
    "engolfásemos": "engolfar",
    "engolfé": "engolfar",
    "engolféis": "engolfar",
    "engolfó": "engolfar",
    "engolillada": "engolillado",
    "engolilladas": "engolillado",
    "engolillados": "engolillado",
    "engollamientos": "engollamiento",
    "engolosina": "engolosinar",
    "engolosinaba": "engolosinar",
    "engolosinabais": "engolosinar",
    "engolosinaban": "engolosinar",
    "engolosinabas": "engolosinar",
    "engolosinad": "engolosinar",
    "engolosinada": "engolosinar",
    "engolosinadas": "engolosinar",
    "engolosinado": "engolosinar",
    "engolosinadora": "engolosinador",
    "engolosinadoras": "engolosinador",
    "engolosinadores": "engolosinador",
    "engolosinados": "engolosinar",
    "engolosinamientos": "engolosinamiento",
    "engolosinamos": "engolosinar",
    "engolosinan": "engolosinar",
    "engolosinando": "engolosinar",
    "engolosinara": "engolosinar",
    "engolosinarais": "engolosinar",
    "engolosinaran": "engolosinar",
    "engolosinaras": "engolosinar",
    "engolosinare": "engolosinar",
    "engolosinareis": "engolosinar",
    "engolosinaremos": "engolosinar",
    "engolosinaren": "engolosinar",
    "engolosinares": "engolosinar",
    "engolosinaron": "engolosinar",
    "engolosinará": "engolosinar",
    "engolosinarán": "engolosinar",
    "engolosinarás": "engolosinar",
    "engolosinaré": "engolosinar",
    "engolosinaréis": "engolosinar",
    "engolosinaría": "engolosinar",
    "engolosinaríais": "engolosinar",
    "engolosinaríamos": "engolosinar",
    "engolosinarían": "engolosinar",
    "engolosinarías": "engolosinar",
    "engolosinas": "engolosinar",
    "engolosinase": "engolosinar",
    "engolosinaseis": "engolosinar",
    "engolosinasen": "engolosinar",
    "engolosinases": "engolosinar",
    "engolosinaste": "engolosinar",
    "engolosinasteis": "engolosinar",
    "engolosine": "engolosinar",
    "engolosinemos": "engolosinar",
    "engolosinen": "engolosinar",
    "engolosines": "engolosinar",
    "engolosino": "engolosinar",
    "engolosinábamos": "engolosinar",
    "engolosináis": "engolosinar",
    "engolosináramos": "engolosinar",
    "engolosináremos": "engolosinar",
    "engolosinásemos": "engolosinar",
    "engolosiné": "engolosinar",
    "engolosinéis": "engolosinar",
    "engolosinó": "engolosinar",
    "engoma": "engomar",
    "engomaba": "engomar",
    "engomabais": "engomar",
    "engomaban": "engomar",
    "engomabas": "engomar",
    "engomad": "engomar",
    "engomada": "engomar",
    "engomadas": "engomar",
    "engomado": "engomar",
    "engomadora": "engomador",
    "engomadoras": "engomador",
    "engomadores": "engomador",
    "engomados": "engomar",
    "engomaduras": "engomadura",
    "engomamos": "engomar",
    "engoman": "engomar",
    "engomando": "engomar",
    "engomara": "engomar",
    "engomarais": "engomar",
    "engomaran": "engomar",
    "engomaras": "engomar",
    "engomare": "engomar",
    "engomareis": "engomar",
    "engomaremos": "engomar",
    "engomaren": "engomar",
    "engomares": "engomar",
    "engomaron": "engomar",
    "engomará": "engomar",
    "engomarán": "engomar",
    "engomarás": "engomar",
    "engomaré": "engomar",
    "engomaréis": "engomar",
    "engomaría": "engomar",
    "engomaríais": "engomar",
    "engomaríamos": "engomar",
    "engomarían": "engomar",
    "engomarías": "engomar",
    "engomas": "engomar",
    "engomase": "engomar",
    "engomaseis": "engomar",
    "engomasen": "engomar",
    "engomases": "engomar",
    "engomaste": "engomar",
    "engomasteis": "engomar",
    "engome": "engomar",
    "engomemos": "engomar",
    "engomen": "engomar",
    "engomes": "engomar",
    "engomo": "engomar",
    "engomábamos": "engomar",
    "engomáis": "engomar",
    "engomáramos": "engomar",
    "engomáremos": "engomar",
    "engomásemos": "engomar",
    "engomé": "engomar",
    "engoméis": "engomar",
    "engomó": "engomar",
    "engorda": "engordar",
    "engordaba": "engordar",
    "engordabais": "engordar",
    "engordaban": "engordar",
    "engordabas": "engordar",
    "engordad": "engordar",
    "engordada": "engordar",
    "engordadas": "engordar",
    "engordado": "engordar",
    "engordados": "engordar",
    "engordamientos": "engordamiento",
    "engordamos": "engordar",
    "engordan": "engordar",
    "engordando": "engordar",
    "engordara": "engordar",
    "engordarais": "engordar",
    "engordaran": "engordar",
    "engordaras": "engordar",
    "engordare": "engordar",
    "engordareis": "engordar",
    "engordaremos": "engordar",
    "engordaren": "engordar",
    "engordares": "engordar",
    "engordaron": "engordar",
    "engordará": "engordar",
    "engordarán": "engordar",
    "engordarás": "engordar",
    "engordaré": "engordar",
    "engordaréis": "engordar",
    "engordaría": "engordar",
    "engordaríais": "engordar",
    "engordaríamos": "engordar",
    "engordarían": "engordar",
    "engordarías": "engordar",
    "engordas": "engordar",
    "engordase": "engordar",
    "engordaseis": "engordar",
    "engordasen": "engordar",
    "engordases": "engordar",
    "engordaste": "engordar",
    "engordasteis": "engordar",
    "engorde": "engordar",
    "engordemos": "engordar",
    "engorden": "engordar",
    "engordes": "engorde",
    "engordo": "engordar",
    "engordábamos": "engordar",
    "engordáis": "engordar",
    "engordáramos": "engordar",
    "engordáremos": "engordar",
    "engordásemos": "engordar",
    "engordé": "engordar",
    "engordéis": "engordar",
    "engordó": "engordar",
    "engorros": "engorro",
    "engorrosa": "engorroso",
    "engorrosas": "engorroso",
    "engorrosos": "engorroso",
    "engozna": "engoznar",
    "engoznaba": "engoznar",
    "engoznabais": "engoznar",
    "engoznaban": "engoznar",
    "engoznabas": "engoznar",
    "engoznad": "engoznar",
    "engoznada": "engoznar",
    "engoznadas": "engoznar",
    "engoznado": "engoznar",
    "engoznados": "engoznar",
    "engoznamos": "engoznar",
    "engoznan": "engoznar",
    "engoznando": "engoznar",
    "engoznara": "engoznar",
    "engoznarais": "engoznar",
    "engoznaran": "engoznar",
    "engoznaras": "engoznar",
    "engoznare": "engoznar",
    "engoznareis": "engoznar",
    "engoznaremos": "engoznar",
    "engoznaren": "engoznar",
    "engoznares": "engoznar",
    "engoznaron": "engoznar",
    "engoznará": "engoznar",
    "engoznarán": "engoznar",
    "engoznarás": "engoznar",
    "engoznaré": "engoznar",
    "engoznaréis": "engoznar",
    "engoznaría": "engoznar",
    "engoznaríais": "engoznar",
    "engoznaríamos": "engoznar",
    "engoznarían": "engoznar",
    "engoznarías": "engoznar",
    "engoznas": "engoznar",
    "engoznase": "engoznar",
    "engoznaseis": "engoznar",
    "engoznasen": "engoznar",
    "engoznases": "engoznar",
    "engoznaste": "engoznar",
    "engoznasteis": "engoznar",
    "engozne": "engoznar",
    "engoznemos": "engoznar",
    "engoznen": "engoznar",
    "engoznes": "engoznar",
    "engozno": "engoznar",
    "engoznábamos": "engoznar",
    "engoznáis": "engoznar",
    "engoznáramos": "engoznar",
    "engoznáremos": "engoznar",
    "engoznásemos": "engoznar",
    "engozné": "engoznar",
    "engoznéis": "engoznar",
    "engoznó": "engoznar",
    "engrana": "engranar",
    "engranaba": "engranar",
    "engranabais": "engranar",
    "engranaban": "engranar",
    "engranabas": "engranar",
    "engranad": "engranar",
    "engranada": "engranar",
    "engranadas": "engranar",
    "engranado": "engranar",
    "engranados": "engranar",
    "engranajes": "engranaje",
    "engranamos": "engranar",
    "engranan": "engranar",
    "engranando": "engranar",
    "engranara": "engranar",
    "engranarais": "engranar",
    "engranaran": "engranar",
    "engranaras": "engranar",
    "engranare": "engranar",
    "engranareis": "engranar",
    "engranaremos": "engranar",
    "engranaren": "engranar",
    "engranares": "engranar",
    "engranaron": "engranar",
    "engranará": "engranar",
    "engranarán": "engranar",
    "engranarás": "engranar",
    "engranaré": "engranar",
    "engranaréis": "engranar",
    "engranaría": "engranar",
    "engranaríais": "engranar",
    "engranaríamos": "engranar",
    "engranarían": "engranar",
    "engranarías": "engranar",
    "engranas": "engranar",
    "engranase": "engranar",
    "engranaseis": "engranar",
    "engranasen": "engranar",
    "engranases": "engranar",
    "engranaste": "engranar",
    "engranasteis": "engranar",
    "engrandece": "engrandecer",
    "engrandeced": "engrandecer",
    "engrandecemos": "engrandecer",
    "engrandecen": "engrandecer",
    "engrandeceremos": "engrandecer",
    "engrandecerá": "engrandecer",
    "engrandecerán": "engrandecer",
    "engrandecerás": "engrandecer",
    "engrandeceré": "engrandecer",
    "engrandeceréis": "engrandecer",
    "engrandecería": "engrandecer",
    "engrandeceríais": "engrandecer",
    "engrandeceríamos": "engrandecer",
    "engrandecerían": "engrandecer",
    "engrandecerías": "engrandecer",
    "engrandeces": "engrandecer",
    "engrandecida": "engrandecer",
    "engrandecidas": "engrandecer",
    "engrandecido": "engrandecer",
    "engrandecidos": "engrandecer",
    "engrandeciendo": "engrandecer",
    "engrandeciera": "engrandecer",
    "engrandecierais": "engrandecer",
    "engrandecieran": "engrandecer",
    "engrandecieras": "engrandecer",
    "engrandeciere": "engrandecer",
    "engrandeciereis": "engrandecer",
    "engrandecieren": "engrandecer",
    "engrandecieres": "engrandecer",
    "engrandecieron": "engrandecer",
    "engrandeciese": "engrandecer",
    "engrandecieseis": "engrandecer",
    "engrandeciesen": "engrandecer",
    "engrandecieses": "engrandecer",
    "engrandecimientos": "engrandecimiento",
    "engrandecimos": "engrandecer",
    "engrandeciste": "engrandecer",
    "engrandecisteis": "engrandecer",
    "engrandeciéramos": "engrandecer",
    "engrandeciéremos": "engrandecer",
    "engrandeciésemos": "engrandecer",
    "engrandeció": "engrandecer",
    "engrandecéis": "engrandecer",
    "engrandecí": "engrandecer",
    "engrandecía": "engrandecer",
    "engrandecíais": "engrandecer",
    "engrandecíamos": "engrandecer",
    "engrandecían": "engrandecer",
    "engrandecías": "engrandecer",
    "engrandezca": "engrandecer",
    "engrandezcamos": "engrandecer",
    "engrandezcan": "engrandecer",
    "engrandezcas": "engrandecer",
    "engrandezco": "engrandecer",
    "engrandezcáis": "engrandecer",
    "engrane": "engranar",
    "engranemos": "engranar",
    "engranen": "engranar",
    "engranera": "engranerar",
    "engraneraba": "engranerar",
    "engranerabais": "engranerar",
    "engraneraban": "engranerar",
    "engranerabas": "engranerar",
    "engranerad": "engranerar",
    "engranerada": "engranerar",
    "engraneradas": "engranerar",
    "engranerado": "engranerar",
    "engranerados": "engranerar",
    "engraneramos": "engranerar",
    "engraneran": "engranerar",
    "engranerando": "engranerar",
    "engranerara": "engranerar",
    "engranerarais": "engranerar",
    "engraneraran": "engranerar",
    "engraneraras": "engranerar",
    "engranerare": "engranerar",
    "engranerareis": "engranerar",
    "engraneraremos": "engranerar",
    "engraneraren": "engranerar",
    "engranerares": "engranerar",
    "engraneraron": "engranerar",
    "engranerará": "engranerar",
    "engranerarán": "engranerar",
    "engranerarás": "engranerar",
    "engraneraré": "engranerar",
    "engraneraréis": "engranerar",
    "engraneraría": "engranerar",
    "engraneraríais": "engranerar",
    "engraneraríamos": "engranerar",
    "engranerarían": "engranerar",
    "engranerarías": "engranerar",
    "engraneras": "engranerar",
    "engranerase": "engranerar",
    "engraneraseis": "engranerar",
    "engranerasen": "engranerar",
    "engranerases": "engranerar",
    "engraneraste": "engranerar",
    "engranerasteis": "engranerar",
    "engranere": "engranerar",
    "engraneremos": "engranerar",
    "engraneren": "engranerar",
    "engraneres": "engranerar",
    "engranero": "engranerar",
    "engranerábamos": "engranerar",
    "engraneráis": "engranerar",
    "engraneráramos": "engranerar",
    "engraneráremos": "engranerar",
    "engranerásemos": "engranerar",
    "engraneré": "engranerar",
    "engraneréis": "engranerar",
    "engraneró": "engranerar",
    "engranes": "engranar",
    "engrano": "engranar",
    "engranuja": "engranujar",
    "engranujaba": "engranujar",
    "engranujabais": "engranujar",
    "engranujaban": "engranujar",
    "engranujabas": "engranujar",
    "engranujad": "engranujar",
    "engranujada": "engranujar",
    "engranujadas": "engranujar",
    "engranujado": "engranujar",
    "engranujados": "engranujar",
    "engranujamos": "engranujar",
    "engranujan": "engranujar",
    "engranujando": "engranujar",
    "engranujara": "engranujar",
    "engranujarais": "engranujar",
    "engranujaran": "engranujar",
    "engranujaras": "engranujar",
    "engranujare": "engranujar",
    "engranujareis": "engranujar",
    "engranujaremos": "engranujar",
    "engranujaren": "engranujar",
    "engranujares": "engranujar",
    "engranujaron": "engranujar",
    "engranujará": "engranujar",
    "engranujarán": "engranujar",
    "engranujarás": "engranujar",
    "engranujaré": "engranujar",
    "engranujaréis": "engranujar",
    "engranujaría": "engranujar",
    "engranujaríais": "engranujar",
    "engranujaríamos": "engranujar",
    "engranujarían": "engranujar",
    "engranujarías": "engranujar",
    "engranujas": "engranujar",
    "engranujase": "engranujar",
    "engranujaseis": "engranujar",
    "engranujasen": "engranujar",
    "engranujases": "engranujar",
    "engranujaste": "engranujar",
    "engranujasteis": "engranujar",
    "engranuje": "engranujar",
    "engranujemos": "engranujar",
    "engranujen": "engranujar",
    "engranujes": "engranujar",
    "engranujo": "engranujar",
    "engranujábamos": "engranujar",
    "engranujáis": "engranujar",
    "engranujáramos": "engranujar",
    "engranujáremos": "engranujar",
    "engranujásemos": "engranujar",
    "engranujé": "engranujar",
    "engranujéis": "engranujar",
    "engranujó": "engranujar",
    "engranábamos": "engranar",
    "engranáis": "engranar",
    "engranáramos": "engranar",
    "engranáremos": "engranar",
    "engranásemos": "engranar",
    "engrané": "engranar",
    "engranéis": "engranar",
    "engranó": "engranar",
    "engrapa": "engrapar",
    "engrapaba": "engrapar",
    "engrapabais": "engrapar",
    "engrapaban": "engrapar",
    "engrapabas": "engrapar",
    "engrapad": "engrapar",
    "engrapada": "engrapar",
    "engrapadas": "engrapar",
    "engrapado": "engrapar",
    "engrapadoras": "engrapadora",
    "engrapados": "engrapar",
    "engrapamos": "engrapar",
    "engrapan": "engrapar",
    "engrapando": "engrapar",
    "engrapara": "engrapar",
    "engraparais": "engrapar",
    "engraparan": "engrapar",
    "engraparas": "engrapar",
    "engrapare": "engrapar",
    "engrapareis": "engrapar",
    "engraparemos": "engrapar",
    "engraparen": "engrapar",
    "engrapares": "engrapar",
    "engraparon": "engrapar",
    "engrapará": "engrapar",
    "engraparán": "engrapar",
    "engraparás": "engrapar",
    "engraparé": "engrapar",
    "engraparéis": "engrapar",
    "engraparía": "engrapar",
    "engraparíais": "engrapar",
    "engraparíamos": "engrapar",
    "engraparían": "engrapar",
    "engraparías": "engrapar",
    "engrapas": "engrapar",
    "engrapase": "engrapar",
    "engrapaseis": "engrapar",
    "engrapasen": "engrapar",
    "engrapases": "engrapar",
    "engrapaste": "engrapar",
    "engrapasteis": "engrapar",
    "engrape": "engrapar",
    "engrapemos": "engrapar",
    "engrapen": "engrapar",
    "engrapes": "engrapar",
    "engrapo": "engrapar",
    "engrapábamos": "engrapar",
    "engrapáis": "engrapar",
    "engrapáramos": "engrapar",
    "engrapáremos": "engrapar",
    "engrapásemos": "engrapar",
    "engrapé": "engrapar",
    "engrapéis": "engrapar",
    "engrapó": "engrapar",
    "engrasa": "engrasar",
    "engrasaba": "engrasar",
    "engrasabais": "engrasar",
    "engrasaban": "engrasar",
    "engrasabas": "engrasar",
    "engrasaciones": "engrasación",
    "engrasad": "engrasar",
    "engrasada": "engrasar",
    "engrasadas": "engrasar",
    "engrasado": "engrasar",
    "engrasadora": "engrasador",
    "engrasadoras": "engrasador",
    "engrasadores": "engrasador",
    "engrasados": "engrasar",
    "engrasamos": "engrasar",
    "engrasan": "engrasar",
    "engrasando": "engrasar",
    "engrasara": "engrasar",
    "engrasarais": "engrasar",
    "engrasaran": "engrasar",
    "engrasaras": "engrasar",
    "engrasare": "engrasar",
    "engrasareis": "engrasar",
    "engrasaremos": "engrasar",
    "engrasaren": "engrasar",
    "engrasares": "engrasar",
    "engrasaron": "engrasar",
    "engrasará": "engrasar",
    "engrasarán": "engrasar",
    "engrasarás": "engrasar",
    "engrasaré": "engrasar",
    "engrasaréis": "engrasar",
    "engrasaría": "engrasar",
    "engrasaríais": "engrasar",
    "engrasaríamos": "engrasar",
    "engrasarían": "engrasar",
    "engrasarías": "engrasar",
    "engrasas": "engrasar",
    "engrasase": "engrasar",
    "engrasaseis": "engrasar",
    "engrasasen": "engrasar",
    "engrasases": "engrasar",
    "engrasaste": "engrasar",
    "engrasasteis": "engrasar",
    "engrase": "engrasar",
    "engrasemos": "engrasar",
    "engrasen": "engrasar",
    "engrases": "engrase",
    "engraso": "engrasar",
    "engrasábamos": "engrasar",
    "engrasáis": "engrasar",
    "engrasáramos": "engrasar",
    "engrasáremos": "engrasar",
    "engrasásemos": "engrasar",
    "engrasé": "engrasar",
    "engraséis": "engrasar",
    "engrasó": "engrasar",
    "engreimientos": "engreimiento",
    "engreiremos": "engreír",
    "engreirá": "engreír",
    "engreirán": "engreír",
    "engreirás": "engreír",
    "engreiré": "engreír",
    "engreiréis": "engreír",
    "engreiría": "engreír",
    "engreiríais": "engreír",
    "engreiríamos": "engreír",
    "engreirían": "engreír",
    "engreirías": "engreír",
    "engresca": "engrescar",
    "engrescaba": "engrescar",
    "engrescabais": "engrescar",
    "engrescaban": "engrescar",
    "engrescabas": "engrescar",
    "engrescad": "engrescar",
    "engrescada": "engrescar",
    "engrescadas": "engrescar",
    "engrescado": "engrescar",
    "engrescados": "engrescar",
    "engrescamientos": "engrescamiento",
    "engrescamos": "engrescar",
    "engrescan": "engrescar",
    "engrescando": "engrescar",
    "engrescara": "engrescar",
    "engrescarais": "engrescar",
    "engrescaran": "engrescar",
    "engrescaras": "engrescar",
    "engrescare": "engrescar",
    "engrescareis": "engrescar",
    "engrescaremos": "engrescar",
    "engrescaren": "engrescar",
    "engrescares": "engrescar",
    "engrescaron": "engrescar",
    "engrescará": "engrescar",
    "engrescarán": "engrescar",
    "engrescarás": "engrescar",
    "engrescaré": "engrescar",
    "engrescaréis": "engrescar",
    "engrescaría": "engrescar",
    "engrescaríais": "engrescar",
    "engrescaríamos": "engrescar",
    "engrescarían": "engrescar",
    "engrescarías": "engrescar",
    "engrescas": "engrescar",
    "engrescase": "engrescar",
    "engrescaseis": "engrescar",
    "engrescasen": "engrescar",
    "engrescases": "engrescar",
    "engrescaste": "engrescar",
    "engrescasteis": "engrescar",
    "engresco": "engrescar",
    "engrescábamos": "engrescar",
    "engrescáis": "engrescar",
    "engrescáramos": "engrescar",
    "engrescáremos": "engrescar",
    "engrescásemos": "engrescar",
    "engrescó": "engrescar",
    "engresque": "engrescar",
    "engresquemos": "engrescar",
    "engresquen": "engrescar",
    "engresques": "engrescar",
    "engresqué": "engrescar",
    "engresquéis": "engrescar",
    "engreí": "engreír",
    "engreía": "engreír",
    "engreíais": "engreír",
    "engreíamos": "engreír",
    "engreían": "engreír",
    "engreías": "engreír",
    "engreíd": "engreír",
    "engreída": "engreír",
    "engreídas": "engreír",
    "engreído": "engreír",
    "engreídos": "engreír",
    "engreímos": "engreír",
    "engreís": "engreír",
    "engreíste": "engreír",
    "engreísteis": "engreír",
    "engreñada": "engreñado",
    "engreñadas": "engreñado",
    "engreñados": "engreñado",
    "engriamos": "engreír",
    "engriendo": "engreír",
    "engriera": "engreír",
    "engrierais": "engreír",
    "engrieran": "engreír",
    "engrieras": "engreír",
    "engriere": "engreír",
    "engriereis": "engreír",
    "engrieren": "engreír",
    "engrieres": "engreír",
    "engrieron": "engreír",
    "engriese": "engreír",
    "engrieseis": "engreír",
    "engriesen": "engreír",
    "engrieses": "engreír",
    "engrifa": "engrifar",
    "engrifaba": "engrifar",
    "engrifabais": "engrifar",
    "engrifaban": "engrifar",
    "engrifabas": "engrifar",
    "engrifad": "engrifar",
    "engrifada": "engrifar",
    "engrifadas": "engrifar",
    "engrifado": "engrifar",
    "engrifados": "engrifar",
    "engrifamos": "engrifar",
    "engrifan": "engrifar",
    "engrifando": "engrifar",
    "engrifara": "engrifar",
    "engrifarais": "engrifar",
    "engrifaran": "engrifar",
    "engrifaras": "engrifar",
    "engrifare": "engrifar",
    "engrifareis": "engrifar",
    "engrifaremos": "engrifar",
    "engrifaren": "engrifar",
    "engrifares": "engrifar",
    "engrifaron": "engrifar",
    "engrifará": "engrifar",
    "engrifarán": "engrifar",
    "engrifarás": "engrifar",
    "engrifaré": "engrifar",
    "engrifaréis": "engrifar",
    "engrifaría": "engrifar",
    "engrifaríais": "engrifar",
    "engrifaríamos": "engrifar",
    "engrifarían": "engrifar",
    "engrifarías": "engrifar",
    "engrifas": "engrifar",
    "engrifase": "engrifar",
    "engrifaseis": "engrifar",
    "engrifasen": "engrifar",
    "engrifases": "engrifar",
    "engrifaste": "engrifar",
    "engrifasteis": "engrifar",
    "engrife": "engrifar",
    "engrifemos": "engrifar",
    "engrifen": "engrifar",
    "engrifes": "engrifar",
    "engrifo": "engrifar",
    "engrifábamos": "engrifar",
    "engrifáis": "engrifar",
    "engrifáramos": "engrifar",
    "engrifáremos": "engrifar",
    "engrifásemos": "engrifar",
    "engrifé": "engrifar",
    "engriféis": "engrifar",
    "engrifó": "engrifar",
    "engrilla": "engrillar",
    "engrillaba": "engrillar",
    "engrillabais": "engrillar",
    "engrillaban": "engrillar",
    "engrillabas": "engrillar",
    "engrillad": "engrillar",
    "engrillada": "engrillar",
    "engrilladas": "engrillar",
    "engrillado": "engrillar",
    "engrillados": "engrillar",
    "engrillamos": "engrillar",
    "engrillan": "engrillar",
    "engrillando": "engrillar",
    "engrillara": "engrillar",
    "engrillarais": "engrillar",
    "engrillaran": "engrillar",
    "engrillaras": "engrillar",
    "engrillare": "engrillar",
    "engrillareis": "engrillar",
    "engrillaremos": "engrillar",
    "engrillaren": "engrillar",
    "engrillares": "engrillar",
    "engrillaron": "engrillar",
    "engrillará": "engrillar",
    "engrillarán": "engrillar",
    "engrillarás": "engrillar",
    "engrillaré": "engrillar",
    "engrillaréis": "engrillar",
    "engrillaría": "engrillar",
    "engrillaríais": "engrillar",
    "engrillaríamos": "engrillar",
    "engrillarían": "engrillar",
    "engrillarías": "engrillar",
    "engrillas": "engrillar",
    "engrillase": "engrillar",
    "engrillaseis": "engrillar",
    "engrillasen": "engrillar",
    "engrillases": "engrillar",
    "engrillaste": "engrillar",
    "engrillasteis": "engrillar",
    "engrille": "engrillar",
    "engrillemos": "engrillar",
    "engrillen": "engrillar",
    "engrilles": "engrillar",
    "engrillo": "engrillar",
    "engrillábamos": "engrillar",
    "engrilláis": "engrillar",
    "engrilláramos": "engrillar",
    "engrilláremos": "engrillar",
    "engrillásemos": "engrillar",
    "engrillé": "engrillar",
    "engrilléis": "engrillar",
    "engrilló": "engrillar",
    "engringa": "engringar",
    "engringaba": "engringar",
    "engringabais": "engringar",
    "engringaban": "engringar",
    "engringabas": "engringar",
    "engringad": "engringar",
    "engringada": "engringar",
    "engringadas": "engringar",
    "engringado": "engringar",
    "engringados": "engringar",
    "engringamos": "engringar",
    "engringan": "engringar",
    "engringando": "engringar",
    "engringara": "engringar",
    "engringarais": "engringar",
    "engringaran": "engringar",
    "engringaras": "engringar",
    "engringare": "engringar",
    "engringareis": "engringar",
    "engringaremos": "engringar",
    "engringaren": "engringar",
    "engringares": "engringar",
    "engringaron": "engringar",
    "engringará": "engringar",
    "engringarán": "engringar",
    "engringarás": "engringar",
    "engringaré": "engringar",
    "engringaréis": "engringar",
    "engringaría": "engringar",
    "engringaríais": "engringar",
    "engringaríamos": "engringar",
    "engringarían": "engringar",
    "engringarías": "engringar",
    "engringas": "engringar",
    "engringase": "engringar",
    "engringaseis": "engringar",
    "engringasen": "engringar",
    "engringases": "engringar",
    "engringaste": "engringar",
    "engringasteis": "engringar",
    "engringo": "engringar",
    "engringue": "engringar",
    "engringuemos": "engringar",
    "engringuen": "engringar",
    "engringues": "engringar",
    "engringué": "engringar",
    "engringuéis": "engringar",
    "engringábamos": "engringar",
    "engringáis": "engringar",
    "engringáramos": "engringar",
    "engringáremos": "engringar",
    "engringásemos": "engringar",
    "engringó": "engringar",
    "engriáis": "engreír",
    "engriéramos": "engreír",
    "engriéremos": "engreír",
    "engriésemos": "engreír",
    "engrió": "engreír",
    "engrosa": "engrosar",
    "engrosaba": "engrosar",
    "engrosabais": "engrosar",
    "engrosaban": "engrosar",
    "engrosabas": "engrosar",
    "engrosad": "engrosar",
    "engrosada": "engrosar",
    "engrosadas": "engrosar",
    "engrosado": "engrosar",
    "engrosados": "engrosar",
    "engrosamientos": "engrosamiento",
    "engrosamos": "engrosar",
    "engrosan": "engrosar",
    "engrosando": "engrosar",
    "engrosara": "engrosar",
    "engrosarais": "engrosar",
    "engrosaran": "engrosar",
    "engrosaras": "engrosar",
    "engrosare": "engrosar",
    "engrosareis": "engrosar",
    "engrosaremos": "engrosar",
    "engrosaren": "engrosar",
    "engrosares": "engrosar",
    "engrosaron": "engrosar",
    "engrosará": "engrosar",
    "engrosarán": "engrosar",
    "engrosarás": "engrosar",
    "engrosaré": "engrosar",
    "engrosaréis": "engrosar",
    "engrosaría": "engrosar",
    "engrosaríais": "engrosar",
    "engrosaríamos": "engrosar",
    "engrosarían": "engrosar",
    "engrosarías": "engrosar",
    "engrosas": "engrosar",
    "engrosase": "engrosar",
    "engrosaseis": "engrosar",
    "engrosasen": "engrosar",
    "engrosases": "engrosar",
    "engrosaste": "engrosar",
    "engrosasteis": "engrosar",
    "engrose": "engrosar",
    "engrosemos": "engrosar",
    "engrosen": "engrosar",
    "engroses": "engrosar",
    "engroso": "engrosar",
    "engrosábamos": "engrosar",
    "engrosáis": "engrosar",
    "engrosáramos": "engrosar",
    "engrosáremos": "engrosar",
    "engrosásemos": "engrosar",
    "engrosé": "engrosar",
    "engroséis": "engrosar",
    "engrosó": "engrosar",
    "engruda": "engrudar",
    "engrudaba": "engrudar",
    "engrudabais": "engrudar",
    "engrudaban": "engrudar",
    "engrudabas": "engrudar",
    "engrudad": "engrudar",
    "engrudada": "engrudar",
    "engrudadas": "engrudar",
    "engrudado": "engrudar",
    "engrudadora": "engrudador",
    "engrudadoras": "engrudador",
    "engrudadores": "engrudador",
    "engrudados": "engrudar",
    "engrudamientos": "engrudamiento",
    "engrudamos": "engrudar",
    "engrudan": "engrudar",
    "engrudando": "engrudar",
    "engrudara": "engrudar",
    "engrudarais": "engrudar",
    "engrudaran": "engrudar",
    "engrudaras": "engrudar",
    "engrudare": "engrudar",
    "engrudareis": "engrudar",
    "engrudaremos": "engrudar",
    "engrudaren": "engrudar",
    "engrudares": "engrudar",
    "engrudaron": "engrudar",
    "engrudará": "engrudar",
    "engrudarán": "engrudar",
    "engrudarás": "engrudar",
    "engrudaré": "engrudar",
    "engrudaréis": "engrudar",
    "engrudaría": "engrudar",
    "engrudaríais": "engrudar",
    "engrudaríamos": "engrudar",
    "engrudarían": "engrudar",
    "engrudarías": "engrudar",
    "engrudas": "engrudar",
    "engrudase": "engrudar",
    "engrudaseis": "engrudar",
    "engrudasen": "engrudar",
    "engrudases": "engrudar",
    "engrudaste": "engrudar",
    "engrudasteis": "engrudar",
    "engrude": "engrudar",
    "engrudemos": "engrudar",
    "engruden": "engrudar",
    "engrudes": "engrudar",
    "engrudo": "engrudar",
    "engrudos": "engrudo",
    "engrudábamos": "engrudar",
    "engrudáis": "engrudar",
    "engrudáramos": "engrudar",
    "engrudáremos": "engrudar",
    "engrudásemos": "engrudar",
    "engrudé": "engrudar",
    "engrudéis": "engrudar",
    "engrudó": "engrudar",
    "engruesa": "engruesar",
    "engruesaba": "engruesar",
    "engruesabais": "engruesar",
    "engruesaban": "engruesar",
    "engruesabas": "engruesar",
    "engruesad": "engruesar",
    "engruesada": "engruesar",
    "engruesadas": "engruesar",
    "engruesado": "engruesar",
    "engruesados": "engruesar",
    "engruesamos": "engruesar",
    "engruesan": "engruesar",
    "engruesando": "engruesar",
    "engruesara": "engruesar",
    "engruesarais": "engruesar",
    "engruesaran": "engruesar",
    "engruesaras": "engruesar",
    "engruesare": "engruesar",
    "engruesareis": "engruesar",
    "engruesaremos": "engruesar",
    "engruesaren": "engruesar",
    "engruesares": "engruesar",
    "engruesaron": "engruesar",
    "engruesará": "engruesar",
    "engruesarán": "engruesar",
    "engruesarás": "engruesar",
    "engruesaré": "engruesar",
    "engruesaréis": "engruesar",
    "engruesaría": "engruesar",
    "engruesaríais": "engruesar",
    "engruesaríamos": "engruesar",
    "engruesarían": "engruesar",
    "engruesarías": "engruesar",
    "engruesas": "engruesar",
    "engruesase": "engruesar",
    "engruesaseis": "engruesar",
    "engruesasen": "engruesar",
    "engruesases": "engruesar",
    "engruesaste": "engruesar",
    "engruesasteis": "engruesar",
    "engruese": "engruesar",
    "engruesemos": "engruesar",
    "engruesen": "engruesar",
    "engrueses": "engruesar",
    "engrueso": "engruesar",
    "engruesábamos": "engruesar",
    "engruesáis": "engruesar",
    "engruesáramos": "engruesar",
    "engruesáremos": "engruesar",
    "engruesásemos": "engruesar",
    "engruesé": "engruesar",
    "engrueséis": "engruesar",
    "engruesó": "engruesar",
    "engrumece": "engrumecer",
    "engrumeced": "engrumecer",
    "engrumecemos": "engrumecer",
    "engrumecen": "engrumecer",
    "engrumeceremos": "engrumecer",
    "engrumecerá": "engrumecer",
    "engrumecerán": "engrumecer",
    "engrumecerás": "engrumecer",
    "engrumeceré": "engrumecer",
    "engrumeceréis": "engrumecer",
    "engrumecería": "engrumecer",
    "engrumeceríais": "engrumecer",
    "engrumeceríamos": "engrumecer",
    "engrumecerían": "engrumecer",
    "engrumecerías": "engrumecer",
    "engrumeces": "engrumecer",
    "engrumecida": "engrumecer",
    "engrumecidas": "engrumecer",
    "engrumecido": "engrumecer",
    "engrumecidos": "engrumecer",
    "engrumeciendo": "engrumecer",
    "engrumeciera": "engrumecer",
    "engrumecierais": "engrumecer",
    "engrumecieran": "engrumecer",
    "engrumecieras": "engrumecer",
    "engrumeciere": "engrumecer",
    "engrumeciereis": "engrumecer",
    "engrumecieren": "engrumecer",
    "engrumecieres": "engrumecer",
    "engrumecieron": "engrumecer",
    "engrumeciese": "engrumecer",
    "engrumecieseis": "engrumecer",
    "engrumeciesen": "engrumecer",
    "engrumecieses": "engrumecer",
    "engrumecimos": "engrumecer",
    "engrumeciste": "engrumecer",
    "engrumecisteis": "engrumecer",
    "engrumeciéramos": "engrumecer",
    "engrumeciéremos": "engrumecer",
    "engrumeciésemos": "engrumecer",
    "engrumeció": "engrumecer",
    "engrumecéis": "engrumecer",
    "engrumecí": "engrumecer",
    "engrumecía": "engrumecer",
    "engrumecíais": "engrumecer",
    "engrumecíamos": "engrumecer",
    "engrumecían": "engrumecer",
    "engrumecías": "engrumecer",
    "engrumezca": "engrumecer",
    "engrumezcamos": "engrumecer",
    "engrumezcan": "engrumecer",
    "engrumezcas": "engrumecer",
    "engrumezco": "engrumecer",
    "engrumezcáis": "engrumecer",
    "engráulidos": "engráulido",
    "engría": "engreír",
    "engrían": "engreír",
    "engrías": "engreír",
    "engríe": "engreír",
    "engríen": "engreír",
    "engríes": "engreír",
    "engrío": "engreír",
    "enguachina": "enguachinar",
    "enguachinaba": "enguachinar",
    "enguachinabais": "enguachinar",
    "enguachinaban": "enguachinar",
    "enguachinabas": "enguachinar",
    "enguachinad": "enguachinar",
    "enguachinada": "enguachinar",
    "enguachinadas": "enguachinar",
    "enguachinado": "enguachinar",
    "enguachinados": "enguachinar",
    "enguachinamos": "enguachinar",
    "enguachinan": "enguachinar",
    "enguachinando": "enguachinar",
    "enguachinara": "enguachinar",
    "enguachinarais": "enguachinar",
    "enguachinaran": "enguachinar",
    "enguachinaras": "enguachinar",
    "enguachinare": "enguachinar",
    "enguachinareis": "enguachinar",
    "enguachinaremos": "enguachinar",
    "enguachinaren": "enguachinar",
    "enguachinares": "enguachinar",
    "enguachinaron": "enguachinar",
    "enguachinará": "enguachinar",
    "enguachinarán": "enguachinar",
    "enguachinarás": "enguachinar",
    "enguachinaré": "enguachinar",
    "enguachinaréis": "enguachinar",
    "enguachinaría": "enguachinar",
    "enguachinaríais": "enguachinar",
    "enguachinaríamos": "enguachinar",
    "enguachinarían": "enguachinar",
    "enguachinarías": "enguachinar",
    "enguachinas": "enguachinar",
    "enguachinase": "enguachinar",
    "enguachinaseis": "enguachinar",
    "enguachinasen": "enguachinar",
    "enguachinases": "enguachinar",
    "enguachinaste": "enguachinar",
    "enguachinasteis": "enguachinar",
    "enguachine": "enguachinar",
    "enguachinemos": "enguachinar",
    "enguachinen": "enguachinar",
    "enguachines": "enguachinar",
    "enguachino": "enguachinar",
    "enguachinábamos": "enguachinar",
    "enguachináis": "enguachinar",
    "enguachináramos": "enguachinar",
    "enguachináremos": "enguachinar",
    "enguachinásemos": "enguachinar",
    "enguachiné": "enguachinar",
    "enguachinéis": "enguachinar",
    "enguachinó": "enguachinar",
    "engualdrapa": "engualdrapar",
    "engualdrapaba": "engualdrapar",
    "engualdrapabais": "engualdrapar",
    "engualdrapaban": "engualdrapar",
    "engualdrapabas": "engualdrapar",
    "engualdrapad": "engualdrapar",
    "engualdrapada": "engualdrapar",
    "engualdrapadas": "engualdrapar",
    "engualdrapado": "engualdrapar",
    "engualdrapados": "engualdrapar",
    "engualdrapamos": "engualdrapar",
    "engualdrapan": "engualdrapar",
    "engualdrapando": "engualdrapar",
    "engualdrapara": "engualdrapar",
    "engualdraparais": "engualdrapar",
    "engualdraparan": "engualdrapar",
    "engualdraparas": "engualdrapar",
    "engualdrapare": "engualdrapar",
    "engualdrapareis": "engualdrapar",
    "engualdraparemos": "engualdrapar",
    "engualdraparen": "engualdrapar",
    "engualdrapares": "engualdrapar",
    "engualdraparon": "engualdrapar",
    "engualdrapará": "engualdrapar",
    "engualdraparán": "engualdrapar",
    "engualdraparás": "engualdrapar",
    "engualdraparé": "engualdrapar",
    "engualdraparéis": "engualdrapar",
    "engualdraparía": "engualdrapar",
    "engualdraparíais": "engualdrapar",
    "engualdraparíamos": "engualdrapar",
    "engualdraparían": "engualdrapar",
    "engualdraparías": "engualdrapar",
    "engualdrapas": "engualdrapar",
    "engualdrapase": "engualdrapar",
    "engualdrapaseis": "engualdrapar",
    "engualdrapasen": "engualdrapar",
    "engualdrapases": "engualdrapar",
    "engualdrapaste": "engualdrapar",
    "engualdrapasteis": "engualdrapar",
    "engualdrape": "engualdrapar",
    "engualdrapemos": "engualdrapar",
    "engualdrapen": "engualdrapar",
    "engualdrapes": "engualdrapar",
    "engualdrapo": "engualdrapar",
    "engualdrapábamos": "engualdrapar",
    "engualdrapáis": "engualdrapar",
    "engualdrapáramos": "engualdrapar",
    "engualdrapáremos": "engualdrapar",
    "engualdrapásemos": "engualdrapar",
    "engualdrapé": "engualdrapar",
    "engualdrapéis": "engualdrapar",
    "engualdrapó": "engualdrapar",
    "enguanta": "enguantar",
    "enguantaba": "enguantar",
    "enguantabais": "enguantar",
    "enguantaban": "enguantar",
    "enguantabas": "enguantar",
    "enguantad": "enguantar",
    "enguantada": "enguantar",
    "enguantadas": "enguantar",
    "enguantado": "enguantar",
    "enguantados": "enguantar",
    "enguantamos": "enguantar",
    "enguantan": "enguantar",
    "enguantando": "enguantar",
    "enguantara": "enguantar",
    "enguantarais": "enguantar",
    "enguantaran": "enguantar",
    "enguantaras": "enguantar",
    "enguantare": "enguantar",
    "enguantareis": "enguantar",
    "enguantaremos": "enguantar",
    "enguantaren": "enguantar",
    "enguantares": "enguantar",
    "enguantaron": "enguantar",
    "enguantará": "enguantar",
    "enguantarán": "enguantar",
    "enguantarás": "enguantar",
    "enguantaré": "enguantar",
    "enguantaréis": "enguantar",
    "enguantaría": "enguantar",
    "enguantaríais": "enguantar",
    "enguantaríamos": "enguantar",
    "enguantarían": "enguantar",
    "enguantarías": "enguantar",
    "enguantas": "enguantar",
    "enguantase": "enguantar",
    "enguantaseis": "enguantar",
    "enguantasen": "enguantar",
    "enguantases": "enguantar",
    "enguantaste": "enguantar",
    "enguantasteis": "enguantar",
    "enguante": "enguantar",
    "enguantemos": "enguantar",
    "enguanten": "enguantar",
    "enguantes": "enguantar",
    "enguanto": "enguantar",
    "enguantábamos": "enguantar",
    "enguantáis": "enguantar",
    "enguantáramos": "enguantar",
    "enguantáremos": "enguantar",
    "enguantásemos": "enguantar",
    "enguanté": "enguantar",
    "enguantéis": "enguantar",
    "enguantó": "enguantar",
    "enguata": "enguatar",
    "enguataba": "enguatar",
    "enguatabais": "enguatar",
    "enguataban": "enguatar",
    "enguatabas": "enguatar",
    "enguatad": "enguatar",
    "enguatada": "enguatar",
    "enguatadas": "enguatar",
    "enguatado": "enguatar",
    "enguatados": "enguatar",
    "enguatamos": "enguatar",
    "enguatan": "enguatar",
    "enguatando": "enguatar",
    "enguatara": "enguatar",
    "enguatarais": "enguatar",
    "enguataran": "enguatar",
    "enguataras": "enguatar",
    "enguatare": "enguatar",
    "enguatareis": "enguatar",
    "enguataremos": "enguatar",
    "enguataren": "enguatar",
    "enguatares": "enguatar",
    "enguataron": "enguatar",
    "enguatará": "enguatar",
    "enguatarán": "enguatar",
    "enguatarás": "enguatar",
    "enguataré": "enguatar",
    "enguataréis": "enguatar",
    "enguataría": "enguatar",
    "enguataríais": "enguatar",
    "enguataríamos": "enguatar",
    "enguatarían": "enguatar",
    "enguatarías": "enguatar",
    "enguatas": "enguatar",
    "enguatase": "enguatar",
    "enguataseis": "enguatar",
    "enguatasen": "enguatar",
    "enguatases": "enguatar",
    "enguataste": "enguatar",
    "enguatasteis": "enguatar",
    "enguate": "enguatar",
    "enguatemos": "enguatar",
    "enguaten": "enguatar",
    "enguates": "enguatar",
    "enguato": "enguatar",
    "enguatábamos": "enguatar",
    "enguatáis": "enguatar",
    "enguatáramos": "enguatar",
    "enguatáremos": "enguatar",
    "enguatásemos": "enguatar",
    "enguaté": "enguatar",
    "enguatéis": "enguatar",
    "enguató": "enguatar",
    "enguedejada": "enguedejado",
    "enguedejadas": "enguedejado",
    "enguedejados": "enguedejado",
    "enguichada": "enguichado",
    "enguichadas": "enguichado",
    "enguichados": "enguichado",
    "enguijarra": "enguijarrar",
    "enguijarraba": "enguijarrar",
    "enguijarrabais": "enguijarrar",
    "enguijarraban": "enguijarrar",
    "enguijarrabas": "enguijarrar",
    "enguijarrad": "enguijarrar",
    "enguijarrada": "enguijarrar",
    "enguijarradas": "enguijarrar",
    "enguijarrado": "enguijarrar",
    "enguijarrados": "enguijarrar",
    "enguijarramos": "enguijarrar",
    "enguijarran": "enguijarrar",
    "enguijarrando": "enguijarrar",
    "enguijarrara": "enguijarrar",
    "enguijarrarais": "enguijarrar",
    "enguijarraran": "enguijarrar",
    "enguijarraras": "enguijarrar",
    "enguijarrare": "enguijarrar",
    "enguijarrareis": "enguijarrar",
    "enguijarraremos": "enguijarrar",
    "enguijarraren": "enguijarrar",
    "enguijarrares": "enguijarrar",
    "enguijarraron": "enguijarrar",
    "enguijarrará": "enguijarrar",
    "enguijarrarán": "enguijarrar",
    "enguijarrarás": "enguijarrar",
    "enguijarraré": "enguijarrar",
    "enguijarraréis": "enguijarrar",
    "enguijarraría": "enguijarrar",
    "enguijarraríais": "enguijarrar",
    "enguijarraríamos": "enguijarrar",
    "enguijarrarían": "enguijarrar",
    "enguijarrarías": "enguijarrar",
    "enguijarras": "enguijarrar",
    "enguijarrase": "enguijarrar",
    "enguijarraseis": "enguijarrar",
    "enguijarrasen": "enguijarrar",
    "enguijarrases": "enguijarrar",
    "enguijarraste": "enguijarrar",
    "enguijarrasteis": "enguijarrar",
    "enguijarre": "enguijarrar",
    "enguijarremos": "enguijarrar",
    "enguijarren": "enguijarrar",
    "enguijarres": "enguijarrar",
    "enguijarro": "enguijarrar",
    "enguijarrábamos": "enguijarrar",
    "enguijarráis": "enguijarrar",
    "enguijarráramos": "enguijarrar",
    "enguijarráremos": "enguijarrar",
    "enguijarrásemos": "enguijarrar",
    "enguijarré": "enguijarrar",
    "enguijarréis": "enguijarrar",
    "enguijarró": "enguijarrar",
    "enguillota": "enguillotar",
    "enguillotaba": "enguillotar",
    "enguillotabais": "enguillotar",
    "enguillotaban": "enguillotar",
    "enguillotabas": "enguillotar",
    "enguillotad": "enguillotar",
    "enguillotada": "enguillotar",
    "enguillotadas": "enguillotar",
    "enguillotado": "enguillotar",
    "enguillotados": "enguillotar",
    "enguillotamos": "enguillotar",
    "enguillotan": "enguillotar",
    "enguillotando": "enguillotar",
    "enguillotara": "enguillotar",
    "enguillotarais": "enguillotar",
    "enguillotaran": "enguillotar",
    "enguillotaras": "enguillotar",
    "enguillotare": "enguillotar",
    "enguillotareis": "enguillotar",
    "enguillotaremos": "enguillotar",
    "enguillotaren": "enguillotar",
    "enguillotares": "enguillotar",
    "enguillotaron": "enguillotar",
    "enguillotará": "enguillotar",
    "enguillotarán": "enguillotar",
    "enguillotarás": "enguillotar",
    "enguillotaré": "enguillotar",
    "enguillotaréis": "enguillotar",
    "enguillotaría": "enguillotar",
    "enguillotaríais": "enguillotar",
    "enguillotaríamos": "enguillotar",
    "enguillotarían": "enguillotar",
    "enguillotarías": "enguillotar",
    "enguillotas": "enguillotar",
    "enguillotase": "enguillotar",
    "enguillotaseis": "enguillotar",
    "enguillotasen": "enguillotar",
    "enguillotases": "enguillotar",
    "enguillotaste": "enguillotar",
    "enguillotasteis": "enguillotar",
    "enguillote": "enguillotar",
    "enguillotemos": "enguillotar",
    "enguilloten": "enguillotar",
    "enguillotes": "enguillotar",
    "enguilloto": "enguillotar",
    "enguillotábamos": "enguillotar",
    "enguillotáis": "enguillotar",
    "enguillotáramos": "enguillotar",
    "enguillotáremos": "enguillotar",
    "enguillotásemos": "enguillotar",
    "enguilloté": "enguillotar",
    "enguillotéis": "enguillotar",
    "enguillotó": "enguillotar",
    "enguirnalda": "enguirnaldar",
    "enguirnaldaba": "enguirnaldar",
    "enguirnaldabais": "enguirnaldar",
    "enguirnaldaban": "enguirnaldar",
    "enguirnaldabas": "enguirnaldar",
    "enguirnaldad": "enguirnaldar",
    "enguirnaldada": "enguirnaldar",
    "enguirnaldadas": "enguirnaldar",
    "enguirnaldado": "enguirnaldar",
    "enguirnaldados": "enguirnaldar",
    "enguirnaldamos": "enguirnaldar",
    "enguirnaldan": "enguirnaldar",
    "enguirnaldando": "enguirnaldar",
    "enguirnaldara": "enguirnaldar",
    "enguirnaldarais": "enguirnaldar",
    "enguirnaldaran": "enguirnaldar",
    "enguirnaldaras": "enguirnaldar",
    "enguirnaldare": "enguirnaldar",
    "enguirnaldareis": "enguirnaldar",
    "enguirnaldaremos": "enguirnaldar",
    "enguirnaldaren": "enguirnaldar",
    "enguirnaldares": "enguirnaldar",
    "enguirnaldaron": "enguirnaldar",
    "enguirnaldará": "enguirnaldar",
    "enguirnaldarán": "enguirnaldar",
    "enguirnaldarás": "enguirnaldar",
    "enguirnaldaré": "enguirnaldar",
    "enguirnaldaréis": "enguirnaldar",
    "enguirnaldaría": "enguirnaldar",
    "enguirnaldaríais": "enguirnaldar",
    "enguirnaldaríamos": "enguirnaldar",
    "enguirnaldarían": "enguirnaldar",
    "enguirnaldarías": "enguirnaldar",
    "enguirnaldas": "enguirnaldar",
    "enguirnaldase": "enguirnaldar",
    "enguirnaldaseis": "enguirnaldar",
    "enguirnaldasen": "enguirnaldar",
    "enguirnaldases": "enguirnaldar",
    "enguirnaldaste": "enguirnaldar",
    "enguirnaldasteis": "enguirnaldar",
    "enguirnalde": "enguirnaldar",
    "enguirnaldemos": "enguirnaldar",
    "enguirnalden": "enguirnaldar",
    "enguirnaldes": "enguirnaldar",
    "enguirnaldo": "enguirnaldar",
    "enguirnaldábamos": "enguirnaldar",
    "enguirnaldáis": "enguirnaldar",
    "enguirnaldáramos": "enguirnaldar",
    "enguirnaldáremos": "enguirnaldar",
    "enguirnaldásemos": "enguirnaldar",
    "enguirnaldé": "enguirnaldar",
    "enguirnaldéis": "enguirnaldar",
    "enguirnaldó": "enguirnaldar",
    "enguizga": "enguizgar",
    "enguizgaba": "enguizgar",
    "enguizgabais": "enguizgar",
    "enguizgaban": "enguizgar",
    "enguizgabas": "enguizgar",
    "enguizgad": "enguizgar",
    "enguizgada": "enguizgar",
    "enguizgadas": "enguizgar",
    "enguizgado": "enguizgar",
    "enguizgados": "enguizgar",
    "enguizgamos": "enguizgar",
    "enguizgan": "enguizgar",
    "enguizgando": "enguizgar",
    "enguizgara": "enguizgar",
    "enguizgarais": "enguizgar",
    "enguizgaran": "enguizgar",
    "enguizgaras": "enguizgar",
    "enguizgare": "enguizgar",
    "enguizgareis": "enguizgar",
    "enguizgaremos": "enguizgar",
    "enguizgaren": "enguizgar",
    "enguizgares": "enguizgar",
    "enguizgaron": "enguizgar",
    "enguizgará": "enguizgar",
    "enguizgarán": "enguizgar",
    "enguizgarás": "enguizgar",
    "enguizgaré": "enguizgar",
    "enguizgaréis": "enguizgar",
    "enguizgaría": "enguizgar",
    "enguizgaríais": "enguizgar",
    "enguizgaríamos": "enguizgar",
    "enguizgarían": "enguizgar",
    "enguizgarías": "enguizgar",
    "enguizgas": "enguizgar",
    "enguizgase": "enguizgar",
    "enguizgaseis": "enguizgar",
    "enguizgasen": "enguizgar",
    "enguizgases": "enguizgar",
    "enguizgaste": "enguizgar",
    "enguizgasteis": "enguizgar",
    "enguizgo": "enguizgar",
    "enguizgue": "enguizgar",
    "enguizguemos": "enguizgar",
    "enguizguen": "enguizgar",
    "enguizgues": "enguizgar",
    "enguizgué": "enguizgar",
    "enguizguéis": "enguizgar",
    "enguizgábamos": "enguizgar",
    "enguizgáis": "enguizgar",
    "enguizgáramos": "enguizgar",
    "enguizgáremos": "enguizgar",
    "enguizgásemos": "enguizgar",
    "enguizgó": "enguizgar",
    "engulla": "engullir",
    "engullamos": "engullir",
    "engullan": "engullir",
    "engullas": "engullir",
    "engulle": "engullir",
    "engullen": "engullir",
    "engullendo": "engullir",
    "engullera": "engullir",
    "engullerais": "engullir",
    "engulleran": "engullir",
    "engulleras": "engullir",
    "engullere": "engullir",
    "engullereis": "engullir",
    "engulleren": "engullir",
    "engulleres": "engullir",
    "engulleron": "engullir",
    "engulles": "engullir",
    "engullese": "engullir",
    "engulleseis": "engullir",
    "engullesen": "engullir",
    "engulleses": "engullir",
    "engullid": "engullir",
    "engullida": "engullir",
    "engullidas": "engullir",
    "engullido": "engullir",
    "engullidos": "engullir",
    "engullimos": "engullir",
    "engulliremos": "engullir",
    "engullirá": "engullir",
    "engullirán": "engullir",
    "engullirás": "engullir",
    "engulliré": "engullir",
    "engulliréis": "engullir",
    "engulliría": "engullir",
    "engulliríais": "engullir",
    "engulliríamos": "engullir",
    "engullirían": "engullir",
    "engullirías": "engullir",
    "engulliste": "engullir",
    "engullisteis": "engullir",
    "engullo": "engullir",
    "engulláis": "engullir",
    "engulléramos": "engullir",
    "engulléremos": "engullir",
    "engullésemos": "engullir",
    "engullí": "engullir",
    "engullía": "engullir",
    "engullíais": "engullir",
    "engullíamos": "engullir",
    "engullían": "engullir",
    "engullías": "engullir",
    "engullís": "engullir",
    "engulló": "engullir",
    "engurras": "engurra",
    "engurrios": "engurrio",
    "engurruña": "engurruñar",
    "engurruñaba": "engurruñar",
    "engurruñabais": "engurruñar",
    "engurruñaban": "engurruñar",
    "engurruñabas": "engurruñar",
    "engurruñad": "engurruñar",
    "engurruñada": "engurruñar",
    "engurruñadas": "engurruñar",
    "engurruñado": "engurruñar",
    "engurruñados": "engurruñar",
    "engurruñamos": "engurruñar",
    "engurruñan": "engurruñar",
    "engurruñando": "engurruñar",
    "engurruñara": "engurruñar",
    "engurruñarais": "engurruñar",
    "engurruñaran": "engurruñar",
    "engurruñaras": "engurruñar",
    "engurruñare": "engurruñar",
    "engurruñareis": "engurruñar",
    "engurruñaremos": "engurruñar",
    "engurruñaren": "engurruñar",
    "engurruñares": "engurruñar",
    "engurruñaron": "engurruñar",
    "engurruñará": "engurruñar",
    "engurruñarán": "engurruñar",
    "engurruñarás": "engurruñar",
    "engurruñaré": "engurruñar",
    "engurruñaréis": "engurruñar",
    "engurruñaría": "engurruñar",
    "engurruñaríais": "engurruñar",
    "engurruñaríamos": "engurruñar",
    "engurruñarían": "engurruñar",
    "engurruñarías": "engurruñar",
    "engurruñas": "engurruñar",
    "engurruñase": "engurruñar",
    "engurruñaseis": "engurruñar",
    "engurruñasen": "engurruñar",
    "engurruñases": "engurruñar",
    "engurruñaste": "engurruñar",
    "engurruñasteis": "engurruñar",
    "engurruñe": "engurruñar",
    "engurruñemos": "engurruñar",
    "engurruñen": "engurruñar",
    "engurruñes": "engurruñar",
    "engurruño": "engurruñar",
    "engurruñábamos": "engurruñar",
    "engurruñáis": "engurruñar",
    "engurruñáramos": "engurruñar",
    "engurruñáremos": "engurruñar",
    "engurruñásemos": "engurruñar",
    "engurruñé": "engurruñar",
    "engurruñéis": "engurruñar",
    "engurruñó": "engurruñar",
    "enhacina": "enhacinar",
    "enhacinaba": "enhacinar",
    "enhacinabais": "enhacinar",
    "enhacinaban": "enhacinar",
    "enhacinabas": "enhacinar",
    "enhacinad": "enhacinar",
    "enhacinada": "enhacinar",
    "enhacinadas": "enhacinar",
    "enhacinado": "enhacinar",
    "enhacinados": "enhacinar",
    "enhacinamos": "enhacinar",
    "enhacinan": "enhacinar",
    "enhacinando": "enhacinar",
    "enhacinara": "enhacinar",
    "enhacinarais": "enhacinar",
    "enhacinaran": "enhacinar",
    "enhacinaras": "enhacinar",
    "enhacinare": "enhacinar",
    "enhacinareis": "enhacinar",
    "enhacinaremos": "enhacinar",
    "enhacinaren": "enhacinar",
    "enhacinares": "enhacinar",
    "enhacinaron": "enhacinar",
    "enhacinará": "enhacinar",
    "enhacinarán": "enhacinar",
    "enhacinarás": "enhacinar",
    "enhacinaré": "enhacinar",
    "enhacinaréis": "enhacinar",
    "enhacinaría": "enhacinar",
    "enhacinaríais": "enhacinar",
    "enhacinaríamos": "enhacinar",
    "enhacinarían": "enhacinar",
    "enhacinarías": "enhacinar",
    "enhacinas": "enhacinar",
    "enhacinase": "enhacinar",
    "enhacinaseis": "enhacinar",
    "enhacinasen": "enhacinar",
    "enhacinases": "enhacinar",
    "enhacinaste": "enhacinar",
    "enhacinasteis": "enhacinar",
    "enhacine": "enhacinar",
    "enhacinemos": "enhacinar",
    "enhacinen": "enhacinar",
    "enhacines": "enhacinar",
    "enhacino": "enhacinar",
    "enhacinábamos": "enhacinar",
    "enhacináis": "enhacinar",
    "enhacináramos": "enhacinar",
    "enhacináremos": "enhacinar",
    "enhacinásemos": "enhacinar",
    "enhaciné": "enhacinar",
    "enhacinéis": "enhacinar",
    "enhacinó": "enhacinar",
    "enharina": "enharinar",
    "enharinaba": "enharinar",
    "enharinabais": "enharinar",
    "enharinaban": "enharinar",
    "enharinabas": "enharinar",
    "enharinad": "enharinar",
    "enharinada": "enharinar",
    "enharinadas": "enharinar",
    "enharinado": "enharinar",
    "enharinados": "enharinar",
    "enharinamos": "enharinar",
    "enharinan": "enharinar",
    "enharinando": "enharinar",
    "enharinara": "enharinar",
    "enharinarais": "enharinar",
    "enharinaran": "enharinar",
    "enharinaras": "enharinar",
    "enharinare": "enharinar",
    "enharinareis": "enharinar",
    "enharinaremos": "enharinar",
    "enharinaren": "enharinar",
    "enharinares": "enharinar",
    "enharinaron": "enharinar",
    "enharinará": "enharinar",
    "enharinarán": "enharinar",
    "enharinarás": "enharinar",
    "enharinaré": "enharinar",
    "enharinaréis": "enharinar",
    "enharinaría": "enharinar",
    "enharinaríais": "enharinar",
    "enharinaríamos": "enharinar",
    "enharinarían": "enharinar",
    "enharinarías": "enharinar",
    "enharinas": "enharinar",
    "enharinase": "enharinar",
    "enharinaseis": "enharinar",
    "enharinasen": "enharinar",
    "enharinases": "enharinar",
    "enharinaste": "enharinar",
    "enharinasteis": "enharinar",
    "enharine": "enharinar",
    "enharinemos": "enharinar",
    "enharinen": "enharinar",
    "enharines": "enharinar",
    "enharino": "enharinar",
    "enharinábamos": "enharinar",
    "enharináis": "enharinar",
    "enharináramos": "enharinar",
    "enharináremos": "enharinar",
    "enharinásemos": "enharinar",
    "enhariné": "enharinar",
    "enharinéis": "enharinar",
    "enharinó": "enharinar",
    "enhastiaba": "enhastiar",
    "enhastiabais": "enhastiar",
    "enhastiaban": "enhastiar",
    "enhastiabas": "enhastiar",
    "enhastiad": "enhastiar",
    "enhastiada": "enhastiar",
    "enhastiadas": "enhastiar",
    "enhastiado": "enhastiar",
    "enhastiados": "enhastiar",
    "enhastiamos": "enhastiar",
    "enhastiando": "enhastiar",
    "enhastiara": "enhastiar",
    "enhastiarais": "enhastiar",
    "enhastiaran": "enhastiar",
    "enhastiaras": "enhastiar",
    "enhastiare": "enhastiar",
    "enhastiareis": "enhastiar",
    "enhastiaremos": "enhastiar",
    "enhastiaren": "enhastiar",
    "enhastiares": "enhastiar",
    "enhastiaron": "enhastiar",
    "enhastiará": "enhastiar",
    "enhastiarán": "enhastiar",
    "enhastiarás": "enhastiar",
    "enhastiaré": "enhastiar",
    "enhastiaréis": "enhastiar",
    "enhastiaría": "enhastiar",
    "enhastiaríais": "enhastiar",
    "enhastiaríamos": "enhastiar",
    "enhastiarían": "enhastiar",
    "enhastiarías": "enhastiar",
    "enhastiase": "enhastiar",
    "enhastiaseis": "enhastiar",
    "enhastiasen": "enhastiar",
    "enhastiases": "enhastiar",
    "enhastiaste": "enhastiar",
    "enhastiasteis": "enhastiar",
    "enhastiemos": "enhastiar",
    "enhastiábamos": "enhastiar",
    "enhastiáis": "enhastiar",
    "enhastiáramos": "enhastiar",
    "enhastiáremos": "enhastiar",
    "enhastiásemos": "enhastiar",
    "enhastié": "enhastiar",
    "enhastiéis": "enhastiar",
    "enhastió": "enhastiar",
    "enhastía": "enhastiar",
    "enhastían": "enhastiar",
    "enhastías": "enhastiar",
    "enhastíe": "enhastiar",
    "enhastíen": "enhastiar",
    "enhastíes": "enhastiar",
    "enhastío": "enhastiar",
    "enhebilla": "enhebillar",
    "enhebillaba": "enhebillar",
    "enhebillabais": "enhebillar",
    "enhebillaban": "enhebillar",
    "enhebillabas": "enhebillar",
    "enhebillad": "enhebillar",
    "enhebillada": "enhebillar",
    "enhebilladas": "enhebillar",
    "enhebillado": "enhebillar",
    "enhebillados": "enhebillar",
    "enhebillamos": "enhebillar",
    "enhebillan": "enhebillar",
    "enhebillando": "enhebillar",
    "enhebillara": "enhebillar",
    "enhebillarais": "enhebillar",
    "enhebillaran": "enhebillar",
    "enhebillaras": "enhebillar",
    "enhebillare": "enhebillar",
    "enhebillareis": "enhebillar",
    "enhebillaremos": "enhebillar",
    "enhebillaren": "enhebillar",
    "enhebillares": "enhebillar",
    "enhebillaron": "enhebillar",
    "enhebillará": "enhebillar",
    "enhebillarán": "enhebillar",
    "enhebillarás": "enhebillar",
    "enhebillaré": "enhebillar",
    "enhebillaréis": "enhebillar",
    "enhebillaría": "enhebillar",
    "enhebillaríais": "enhebillar",
    "enhebillaríamos": "enhebillar",
    "enhebillarían": "enhebillar",
    "enhebillarías": "enhebillar",
    "enhebillas": "enhebillar",
    "enhebillase": "enhebillar",
    "enhebillaseis": "enhebillar",
    "enhebillasen": "enhebillar",
    "enhebillases": "enhebillar",
    "enhebillaste": "enhebillar",
    "enhebillasteis": "enhebillar",
    "enhebille": "enhebillar",
    "enhebillemos": "enhebillar",
    "enhebillen": "enhebillar",
    "enhebilles": "enhebillar",
    "enhebillo": "enhebillar",
    "enhebillábamos": "enhebillar",
    "enhebilláis": "enhebillar",
    "enhebilláramos": "enhebillar",
    "enhebilláremos": "enhebillar",
    "enhebillásemos": "enhebillar",
    "enhebillé": "enhebillar",
    "enhebilléis": "enhebillar",
    "enhebilló": "enhebillar",
    "enhebra": "enhebrar",
    "enhebraba": "enhebrar",
    "enhebrabais": "enhebrar",
    "enhebraban": "enhebrar",
    "enhebrabas": "enhebrar",
    "enhebrad": "enhebrar",
    "enhebrada": "enhebrar",
    "enhebradas": "enhebrar",
    "enhebrado": "enhebrar",
    "enhebrados": "enhebrar",
    "enhebramos": "enhebrar",
    "enhebran": "enhebrar",
    "enhebrando": "enhebrar",
    "enhebrara": "enhebrar",
    "enhebrarais": "enhebrar",
    "enhebraran": "enhebrar",
    "enhebraras": "enhebrar",
    "enhebrare": "enhebrar",
    "enhebrareis": "enhebrar",
    "enhebraremos": "enhebrar",
    "enhebraren": "enhebrar",
    "enhebrares": "enhebrar",
    "enhebraron": "enhebrar",
    "enhebrará": "enhebrar",
    "enhebrarán": "enhebrar",
    "enhebrarás": "enhebrar",
    "enhebraré": "enhebrar",
    "enhebraréis": "enhebrar",
    "enhebraría": "enhebrar",
    "enhebraríais": "enhebrar",
    "enhebraríamos": "enhebrar",
    "enhebrarían": "enhebrar",
    "enhebrarías": "enhebrar",
    "enhebras": "enhebrar",
    "enhebrase": "enhebrar",
    "enhebraseis": "enhebrar",
    "enhebrasen": "enhebrar",
    "enhebrases": "enhebrar",
    "enhebraste": "enhebrar",
    "enhebrasteis": "enhebrar",
    "enhebre": "enhebrar",
    "enhebremos": "enhebrar",
    "enhebren": "enhebrar",
    "enhebres": "enhebrar",
    "enhebro": "enhebrar",
    "enhebrábamos": "enhebrar",
    "enhebráis": "enhebrar",
    "enhebráramos": "enhebrar",
    "enhebráremos": "enhebrar",
    "enhebrásemos": "enhebrar",
    "enhebré": "enhebrar",
    "enhebréis": "enhebrar",
    "enhebró": "enhebrar",
    "enhena": "enhenar",
    "enhenaba": "enhenar",
    "enhenabais": "enhenar",
    "enhenaban": "enhenar",
    "enhenabas": "enhenar",
    "enhenad": "enhenar",
    "enhenada": "enhenar",
    "enhenadas": "enhenar",
    "enhenado": "enhenar",
    "enhenados": "enhenar",
    "enhenamos": "enhenar",
    "enhenan": "enhenar",
    "enhenando": "enhenar",
    "enhenara": "enhenar",
    "enhenarais": "enhenar",
    "enhenaran": "enhenar",
    "enhenaras": "enhenar",
    "enhenare": "enhenar",
    "enhenareis": "enhenar",
    "enhenaremos": "enhenar",
    "enhenaren": "enhenar",
    "enhenares": "enhenar",
    "enhenaron": "enhenar",
    "enhenará": "enhenar",
    "enhenarán": "enhenar",
    "enhenarás": "enhenar",
    "enhenaré": "enhenar",
    "enhenaréis": "enhenar",
    "enhenaría": "enhenar",
    "enhenaríais": "enhenar",
    "enhenaríamos": "enhenar",
    "enhenarían": "enhenar",
    "enhenarías": "enhenar",
    "enhenas": "enhenar",
    "enhenase": "enhenar",
    "enhenaseis": "enhenar",
    "enhenasen": "enhenar",
    "enhenases": "enhenar",
    "enhenaste": "enhenar",
    "enhenasteis": "enhenar",
    "enhene": "enhenar",
    "enhenemos": "enhenar",
    "enhenen": "enhenar",
    "enhenes": "enhenar",
    "enheno": "enhenar",
    "enhenábamos": "enhenar",
    "enhenáis": "enhenar",
    "enhenáramos": "enhenar",
    "enhenáremos": "enhenar",
    "enhenásemos": "enhenar",
    "enhené": "enhenar",
    "enhenéis": "enhenar",
    "enhenó": "enhenar",
    "enherbola": "enherbolar",
    "enherbolaba": "enherbolar",
    "enherbolabais": "enherbolar",
    "enherbolaban": "enherbolar",
    "enherbolabas": "enherbolar",
    "enherbolad": "enherbolar",
    "enherbolada": "enherbolar",
    "enherboladas": "enherbolar",
    "enherbolado": "enherbolar",
    "enherbolados": "enherbolar",
    "enherbolamos": "enherbolar",
    "enherbolan": "enherbolar",
    "enherbolando": "enherbolar",
    "enherbolara": "enherbolar",
    "enherbolarais": "enherbolar",
    "enherbolaran": "enherbolar",
    "enherbolaras": "enherbolar",
    "enherbolare": "enherbolar",
    "enherbolareis": "enherbolar",
    "enherbolaremos": "enherbolar",
    "enherbolaren": "enherbolar",
    "enherbolares": "enherbolar",
    "enherbolaron": "enherbolar",
    "enherbolará": "enherbolar",
    "enherbolarán": "enherbolar",
    "enherbolarás": "enherbolar",
    "enherbolaré": "enherbolar",
    "enherbolaréis": "enherbolar",
    "enherbolaría": "enherbolar",
    "enherbolaríais": "enherbolar",
    "enherbolaríamos": "enherbolar",
    "enherbolarían": "enherbolar",
    "enherbolarías": "enherbolar",
    "enherbolas": "enherbolar",
    "enherbolase": "enherbolar",
    "enherbolaseis": "enherbolar",
    "enherbolasen": "enherbolar",
    "enherbolases": "enherbolar",
    "enherbolaste": "enherbolar",
    "enherbolasteis": "enherbolar",
    "enherbole": "enherbolar",
    "enherbolemos": "enherbolar",
    "enherbolen": "enherbolar",
    "enherboles": "enherbolar",
    "enherbolo": "enherbolar",
    "enherbolábamos": "enherbolar",
    "enherboláis": "enherbolar",
    "enherboláramos": "enherbolar",
    "enherboláremos": "enherbolar",
    "enherbolásemos": "enherbolar",
    "enherbolé": "enherbolar",
    "enherboléis": "enherbolar",
    "enherboló": "enherbolar",
    "enhestaduras": "enhestadura",
    "enhestamientos": "enhestamiento",
    "enhiesta": "enhiesto",
    "enhiestas": "enhiesto",
    "enhiestos": "enhiesto",
    "enhila": "enhilar",
    "enhilaba": "enhilar",
    "enhilabais": "enhilar",
    "enhilaban": "enhilar",
    "enhilabas": "enhilar",
    "enhilad": "enhilar",
    "enhilada": "enhilar",
    "enhiladas": "enhilar",
    "enhilado": "enhilar",
    "enhilados": "enhilar",
    "enhilamos": "enhilar",
    "enhilan": "enhilar",
    "enhilando": "enhilar",
    "enhilara": "enhilar",
    "enhilarais": "enhilar",
    "enhilaran": "enhilar",
    "enhilaras": "enhilar",
    "enhilare": "enhilar",
    "enhilareis": "enhilar",
    "enhilaremos": "enhilar",
    "enhilaren": "enhilar",
    "enhilares": "enhilar",
    "enhilaron": "enhilar",
    "enhilará": "enhilar",
    "enhilarán": "enhilar",
    "enhilarás": "enhilar",
    "enhilaré": "enhilar",
    "enhilaréis": "enhilar",
    "enhilaría": "enhilar",
    "enhilaríais": "enhilar",
    "enhilaríamos": "enhilar",
    "enhilarían": "enhilar",
    "enhilarías": "enhilar",
    "enhilas": "enhilar",
    "enhilase": "enhilar",
    "enhilaseis": "enhilar",
    "enhilasen": "enhilar",
    "enhilases": "enhilar",
    "enhilaste": "enhilar",
    "enhilasteis": "enhilar",
    "enhile": "enhilar",
    "enhilemos": "enhilar",
    "enhilen": "enhilar",
    "enhiles": "enhilar",
    "enhilo": "enhilar",
    "enhilábamos": "enhilar",
    "enhiláis": "enhilar",
    "enhiláramos": "enhilar",
    "enhiláremos": "enhilar",
    "enhilásemos": "enhilar",
    "enhilé": "enhilar",
    "enhiléis": "enhilar",
    "enhiló": "enhilar",
    "enhorabuenas": "enhorabuena",
    "enhorna": "enhornar",
    "enhornaba": "enhornar",
    "enhornabais": "enhornar",
    "enhornaban": "enhornar",
    "enhornabas": "enhornar",
    "enhornad": "enhornar",
    "enhornada": "enhornar",
    "enhornadas": "enhornar",
    "enhornado": "enhornar",
    "enhornados": "enhornar",
    "enhornamos": "enhornar",
    "enhornan": "enhornar",
    "enhornando": "enhornar",
    "enhornara": "enhornar",
    "enhornarais": "enhornar",
    "enhornaran": "enhornar",
    "enhornaras": "enhornar",
    "enhornare": "enhornar",
    "enhornareis": "enhornar",
    "enhornaremos": "enhornar",
    "enhornaren": "enhornar",
    "enhornares": "enhornar",
    "enhornaron": "enhornar",
    "enhornará": "enhornar",
    "enhornarán": "enhornar",
    "enhornarás": "enhornar",
    "enhornaré": "enhornar",
    "enhornaréis": "enhornar",
    "enhornaría": "enhornar",
    "enhornaríais": "enhornar",
    "enhornaríamos": "enhornar",
    "enhornarían": "enhornar",
    "enhornarías": "enhornar",
    "enhornas": "enhornar",
    "enhornase": "enhornar",
    "enhornaseis": "enhornar",
    "enhornasen": "enhornar",
    "enhornases": "enhornar",
    "enhornaste": "enhornar",
    "enhornasteis": "enhornar",
    "enhorne": "enhornar",
    "enhornemos": "enhornar",
    "enhornen": "enhornar",
    "enhornes": "enhornar",
    "enhorno": "enhornar",
    "enhornábamos": "enhornar",
    "enhornáis": "enhornar",
    "enhornáramos": "enhornar",
    "enhornáremos": "enhornar",
    "enhornásemos": "enhornar",
    "enhorné": "enhornar",
    "enhornéis": "enhornar",
    "enhornó": "enhornar",
    "enhueca": "enhuecar",
    "enhuecaba": "enhuecar",
    "enhuecabais": "enhuecar",
    "enhuecaban": "enhuecar",
    "enhuecabas": "enhuecar",
    "enhuecad": "enhuecar",
    "enhuecada": "enhuecar",
    "enhuecadas": "enhuecar",
    "enhuecado": "enhuecar",
    "enhuecados": "enhuecar",
    "enhuecamos": "enhuecar",
    "enhuecan": "enhuecar",
    "enhuecando": "enhuecar",
    "enhuecara": "enhuecar",
    "enhuecarais": "enhuecar",
    "enhuecaran": "enhuecar",
    "enhuecaras": "enhuecar",
    "enhuecare": "enhuecar",
    "enhuecareis": "enhuecar",
    "enhuecaremos": "enhuecar",
    "enhuecaren": "enhuecar",
    "enhuecares": "enhuecar",
    "enhuecaron": "enhuecar",
    "enhuecará": "enhuecar",
    "enhuecarán": "enhuecar",
    "enhuecarás": "enhuecar",
    "enhuecaré": "enhuecar",
    "enhuecaréis": "enhuecar",
    "enhuecaría": "enhuecar",
    "enhuecaríais": "enhuecar",
    "enhuecaríamos": "enhuecar",
    "enhuecarían": "enhuecar",
    "enhuecarías": "enhuecar",
    "enhuecas": "enhuecar",
    "enhuecase": "enhuecar",
    "enhuecaseis": "enhuecar",
    "enhuecasen": "enhuecar",
    "enhuecases": "enhuecar",
    "enhuecaste": "enhuecar",
    "enhuecasteis": "enhuecar",
    "enhueco": "enhuecar",
    "enhuecábamos": "enhuecar",
    "enhuecáis": "enhuecar",
    "enhuecáramos": "enhuecar",
    "enhuecáremos": "enhuecar",
    "enhuecásemos": "enhuecar",
    "enhuecó": "enhuecar",
    "enhueque": "enhuecar",
    "enhuequemos": "enhuecar",
    "enhuequen": "enhuecar",
    "enhueques": "enhuecar",
    "enhuequé": "enhuecar",
    "enhuequéis": "enhuecar",
    "enigmas": "enigma",
    "enigmática": "enigmático",
    "enigmáticas": "enigmático",
    "enigmáticos": "enigmático",
    "enjabona": "enjabonar",
    "enjabonaba": "enjabonar",
    "enjabonabais": "enjabonar",
    "enjabonaban": "enjabonar",
    "enjabonabas": "enjabonar",
    "enjabonad": "enjabonar",
    "enjabonada": "enjabonar",
    "enjabonadas": "enjabonar",
    "enjabonado": "enjabonar",
    "enjabonados": "enjabonar",
    "enjabonaduras": "enjabonadura",
    "enjabonamos": "enjabonar",
    "enjabonan": "enjabonar",
    "enjabonando": "enjabonar",
    "enjabonara": "enjabonar",
    "enjabonarais": "enjabonar",
    "enjabonaran": "enjabonar",
    "enjabonaras": "enjabonar",
    "enjabonare": "enjabonar",
    "enjabonareis": "enjabonar",
    "enjabonaremos": "enjabonar",
    "enjabonaren": "enjabonar",
    "enjabonares": "enjabonar",
    "enjabonaron": "enjabonar",
    "enjabonará": "enjabonar",
    "enjabonarán": "enjabonar",
    "enjabonarás": "enjabonar",
    "enjabonaré": "enjabonar",
    "enjabonaréis": "enjabonar",
    "enjabonaría": "enjabonar",
    "enjabonaríais": "enjabonar",
    "enjabonaríamos": "enjabonar",
    "enjabonarían": "enjabonar",
    "enjabonarías": "enjabonar",
    "enjabonas": "enjabonar",
    "enjabonase": "enjabonar",
    "enjabonaseis": "enjabonar",
    "enjabonasen": "enjabonar",
    "enjabonases": "enjabonar",
    "enjabonaste": "enjabonar",
    "enjabonasteis": "enjabonar",
    "enjabone": "enjabonar",
    "enjabonemos": "enjabonar",
    "enjabonen": "enjabonar",
    "enjabones": "enjabonar",
    "enjabono": "enjabonar",
    "enjabonábamos": "enjabonar",
    "enjabonáis": "enjabonar",
    "enjabonáramos": "enjabonar",
    "enjabonáremos": "enjabonar",
    "enjabonásemos": "enjabonar",
    "enjaboné": "enjabonar",
    "enjabonéis": "enjabonar",
    "enjabonó": "enjabonar",
    "enjaece": "enjaezar",
    "enjaecemos": "enjaezar",
    "enjaecen": "enjaezar",
    "enjaeces": "enjaezar",
    "enjaecé": "enjaezar",
    "enjaecéis": "enjaezar",
    "enjaeza": "enjaezar",
    "enjaezaba": "enjaezar",
    "enjaezabais": "enjaezar",
    "enjaezaban": "enjaezar",
    "enjaezabas": "enjaezar",
    "enjaezad": "enjaezar",
    "enjaezada": "enjaezar",
    "enjaezadas": "enjaezar",
    "enjaezado": "enjaezar",
    "enjaezados": "enjaezar",
    "enjaezamientos": "enjaezamiento",
    "enjaezamos": "enjaezar",
    "enjaezan": "enjaezar",
    "enjaezando": "enjaezar",
    "enjaezara": "enjaezar",
    "enjaezarais": "enjaezar",
    "enjaezaran": "enjaezar",
    "enjaezaras": "enjaezar",
    "enjaezare": "enjaezar",
    "enjaezareis": "enjaezar",
    "enjaezaremos": "enjaezar",
    "enjaezaren": "enjaezar",
    "enjaezares": "enjaezar",
    "enjaezaron": "enjaezar",
    "enjaezará": "enjaezar",
    "enjaezarán": "enjaezar",
    "enjaezarás": "enjaezar",
    "enjaezaré": "enjaezar",
    "enjaezaréis": "enjaezar",
    "enjaezaría": "enjaezar",
    "enjaezaríais": "enjaezar",
    "enjaezaríamos": "enjaezar",
    "enjaezarían": "enjaezar",
    "enjaezarías": "enjaezar",
    "enjaezas": "enjaezar",
    "enjaezase": "enjaezar",
    "enjaezaseis": "enjaezar",
    "enjaezasen": "enjaezar",
    "enjaezases": "enjaezar",
    "enjaezaste": "enjaezar",
    "enjaezasteis": "enjaezar",
    "enjaezo": "enjaezar",
    "enjaezábamos": "enjaezar",
    "enjaezáis": "enjaezar",
    "enjaezáramos": "enjaezar",
    "enjaezáremos": "enjaezar",
    "enjaezásemos": "enjaezar",
    "enjaezó": "enjaezar",
    "enjalbega": "enjalbegar",
    "enjalbegaba": "enjalbegar",
    "enjalbegabais": "enjalbegar",
    "enjalbegaban": "enjalbegar",
    "enjalbegabas": "enjalbegar",
    "enjalbegad": "enjalbegar",
    "enjalbegada": "enjalbegar",
    "enjalbegadas": "enjalbegar",
    "enjalbegado": "enjalbegar",
    "enjalbegadora": "enjalbegador",
    "enjalbegadoras": "enjalbegador",
    "enjalbegadores": "enjalbegador",
    "enjalbegados": "enjalbegar",
    "enjalbegaduras": "enjalbegadura",
    "enjalbegamos": "enjalbegar",
    "enjalbegan": "enjalbegar",
    "enjalbegando": "enjalbegar",
    "enjalbegara": "enjalbegar",
    "enjalbegarais": "enjalbegar",
    "enjalbegaran": "enjalbegar",
    "enjalbegaras": "enjalbegar",
    "enjalbegare": "enjalbegar",
    "enjalbegareis": "enjalbegar",
    "enjalbegaremos": "enjalbegar",
    "enjalbegaren": "enjalbegar",
    "enjalbegares": "enjalbegar",
    "enjalbegaron": "enjalbegar",
    "enjalbegará": "enjalbegar",
    "enjalbegarán": "enjalbegar",
    "enjalbegarás": "enjalbegar",
    "enjalbegaré": "enjalbegar",
    "enjalbegaréis": "enjalbegar",
    "enjalbegaría": "enjalbegar",
    "enjalbegaríais": "enjalbegar",
    "enjalbegaríamos": "enjalbegar",
    "enjalbegarían": "enjalbegar",
    "enjalbegarías": "enjalbegar",
    "enjalbegas": "enjalbegar",
    "enjalbegase": "enjalbegar",
    "enjalbegaseis": "enjalbegar",
    "enjalbegasen": "enjalbegar",
    "enjalbegases": "enjalbegar",
    "enjalbegaste": "enjalbegar",
    "enjalbegasteis": "enjalbegar",
    "enjalbego": "enjalbegar",
    "enjalbegue": "enjalbegar",
    "enjalbeguemos": "enjalbegar",
    "enjalbeguen": "enjalbegar",
    "enjalbegues": "enjalbegar",
    "enjalbegué": "enjalbegar",
    "enjalbeguéis": "enjalbegar",
    "enjalbegábamos": "enjalbegar",
    "enjalbegáis": "enjalbegar",
    "enjalbegáramos": "enjalbegar",
    "enjalbegáremos": "enjalbegar",
    "enjalbegásemos": "enjalbegar",
    "enjalbegó": "enjalbegar",
    "enjalbiegos": "enjalbiego",
    "enjalma": "enjalmar",
    "enjalmaba": "enjalmar",
    "enjalmabais": "enjalmar",
    "enjalmaban": "enjalmar",
    "enjalmabas": "enjalmar",
    "enjalmad": "enjalmar",
    "enjalmada": "enjalmar",
    "enjalmadas": "enjalmar",
    "enjalmado": "enjalmar",
    "enjalmados": "enjalmar",
    "enjalmamos": "enjalmar",
    "enjalman": "enjalmar",
    "enjalmando": "enjalmar",
    "enjalmara": "enjalmar",
    "enjalmarais": "enjalmar",
    "enjalmaran": "enjalmar",
    "enjalmaras": "enjalmar",
    "enjalmare": "enjalmar",
    "enjalmareis": "enjalmar",
    "enjalmaremos": "enjalmar",
    "enjalmaren": "enjalmar",
    "enjalmares": "enjalmar",
    "enjalmaron": "enjalmar",
    "enjalmará": "enjalmar",
    "enjalmarán": "enjalmar",
    "enjalmarás": "enjalmar",
    "enjalmaré": "enjalmar",
    "enjalmaréis": "enjalmar",
    "enjalmaría": "enjalmar",
    "enjalmaríais": "enjalmar",
    "enjalmaríamos": "enjalmar",
    "enjalmarían": "enjalmar",
    "enjalmarías": "enjalmar",
    "enjalmas": "enjalmar",
    "enjalmase": "enjalmar",
    "enjalmaseis": "enjalmar",
    "enjalmasen": "enjalmar",
    "enjalmases": "enjalmar",
    "enjalmaste": "enjalmar",
    "enjalmasteis": "enjalmar",
    "enjalme": "enjalmar",
    "enjalmemos": "enjalmar",
    "enjalmen": "enjalmar",
    "enjalmes": "enjalmar",
    "enjalmo": "enjalmar",
    "enjalmábamos": "enjalmar",
    "enjalmáis": "enjalmar",
    "enjalmáramos": "enjalmar",
    "enjalmáremos": "enjalmar",
    "enjalmásemos": "enjalmar",
    "enjalmé": "enjalmar",
    "enjalméis": "enjalmar",
    "enjalmó": "enjalmar",
    "enjambra": "enjambrar",
    "enjambraba": "enjambrar",
    "enjambrabais": "enjambrar",
    "enjambraban": "enjambrar",
    "enjambrabas": "enjambrar",
    "enjambrad": "enjambrar",
    "enjambrada": "enjambrar",
    "enjambradas": "enjambrar",
    "enjambraderas": "enjambradera",
    "enjambraderos": "enjambradero",
    "enjambrado": "enjambrar",
    "enjambrados": "enjambrar",
    "enjambramos": "enjambrar",
    "enjambran": "enjambrar",
    "enjambrando": "enjambrar",
    "enjambrara": "enjambrar",
    "enjambrarais": "enjambrar",
    "enjambraran": "enjambrar",
    "enjambraras": "enjambrar",
    "enjambrare": "enjambrar",
    "enjambrareis": "enjambrar",
    "enjambraremos": "enjambrar",
    "enjambraren": "enjambrar",
    "enjambrares": "enjambrar",
    "enjambraron": "enjambrar",
    "enjambrará": "enjambrar",
    "enjambrarán": "enjambrar",
    "enjambrarás": "enjambrar",
    "enjambraré": "enjambrar",
    "enjambraréis": "enjambrar",
    "enjambraría": "enjambrar",
    "enjambraríais": "enjambrar",
    "enjambraríamos": "enjambrar",
    "enjambrarían": "enjambrar",
    "enjambrarías": "enjambrar",
    "enjambras": "enjambrar",
    "enjambrase": "enjambrar",
    "enjambraseis": "enjambrar",
    "enjambrasen": "enjambrar",
    "enjambrases": "enjambrar",
    "enjambraste": "enjambrar",
    "enjambrasteis": "enjambrar",
    "enjambrazones": "enjambrazón",
    "enjambre": "enjambrar",
    "enjambremos": "enjambrar",
    "enjambren": "enjambrar",
    "enjambres": "enjambre",
    "enjambro": "enjambrar",
    "enjambrábamos": "enjambrar",
    "enjambráis": "enjambrar",
    "enjambráramos": "enjambrar",
    "enjambráremos": "enjambrar",
    "enjambrásemos": "enjambrar",
    "enjambré": "enjambrar",
    "enjambréis": "enjambrar",
    "enjambró": "enjambrar",
    "enjaquima": "enjaquimar",
    "enjaquimaba": "enjaquimar",
    "enjaquimabais": "enjaquimar",
    "enjaquimaban": "enjaquimar",
    "enjaquimabas": "enjaquimar",
    "enjaquimad": "enjaquimar",
    "enjaquimada": "enjaquimar",
    "enjaquimadas": "enjaquimar",
    "enjaquimado": "enjaquimar",
    "enjaquimados": "enjaquimar",
    "enjaquimamos": "enjaquimar",
    "enjaquiman": "enjaquimar",
    "enjaquimando": "enjaquimar",
    "enjaquimara": "enjaquimar",
    "enjaquimarais": "enjaquimar",
    "enjaquimaran": "enjaquimar",
    "enjaquimaras": "enjaquimar",
    "enjaquimare": "enjaquimar",
    "enjaquimareis": "enjaquimar",
    "enjaquimaremos": "enjaquimar",
    "enjaquimaren": "enjaquimar",
    "enjaquimares": "enjaquimar",
    "enjaquimaron": "enjaquimar",
    "enjaquimará": "enjaquimar",
    "enjaquimarán": "enjaquimar",
    "enjaquimarás": "enjaquimar",
    "enjaquimaré": "enjaquimar",
    "enjaquimaréis": "enjaquimar",
    "enjaquimaría": "enjaquimar",
    "enjaquimaríais": "enjaquimar",
    "enjaquimaríamos": "enjaquimar",
    "enjaquimarían": "enjaquimar",
    "enjaquimarías": "enjaquimar",
    "enjaquimas": "enjaquimar",
    "enjaquimase": "enjaquimar",
    "enjaquimaseis": "enjaquimar",
    "enjaquimasen": "enjaquimar",
    "enjaquimases": "enjaquimar",
    "enjaquimaste": "enjaquimar",
    "enjaquimasteis": "enjaquimar",
    "enjaquime": "enjaquimar",
    "enjaquimemos": "enjaquimar",
    "enjaquimen": "enjaquimar",
    "enjaquimes": "enjaquimar",
    "enjaquimo": "enjaquimar",
    "enjaquimábamos": "enjaquimar",
    "enjaquimáis": "enjaquimar",
    "enjaquimáramos": "enjaquimar",
    "enjaquimáremos": "enjaquimar",
    "enjaquimásemos": "enjaquimar",
    "enjaquimé": "enjaquimar",
    "enjaquiméis": "enjaquimar",
    "enjaquimó": "enjaquimar",
    "enjarcia": "enjarciar",
    "enjarciaba": "enjarciar",
    "enjarciabais": "enjarciar",
    "enjarciaban": "enjarciar",
    "enjarciabas": "enjarciar",
    "enjarciad": "enjarciar",
    "enjarciada": "enjarciar",
    "enjarciadas": "enjarciar",
    "enjarciado": "enjarciar",
    "enjarciados": "enjarciar",
    "enjarciamos": "enjarciar",
    "enjarcian": "enjarciar",
    "enjarciando": "enjarciar",
    "enjarciara": "enjarciar",
    "enjarciarais": "enjarciar",
    "enjarciaran": "enjarciar",
    "enjarciaras": "enjarciar",
    "enjarciare": "enjarciar",
    "enjarciareis": "enjarciar",
    "enjarciaremos": "enjarciar",
    "enjarciaren": "enjarciar",
    "enjarciares": "enjarciar",
    "enjarciaron": "enjarciar",
    "enjarciará": "enjarciar",
    "enjarciarán": "enjarciar",
    "enjarciarás": "enjarciar",
    "enjarciaré": "enjarciar",
    "enjarciaréis": "enjarciar",
    "enjarciaría": "enjarciar",
    "enjarciaríais": "enjarciar",
    "enjarciaríamos": "enjarciar",
    "enjarciarían": "enjarciar",
    "enjarciarías": "enjarciar",
    "enjarcias": "enjarciar",
    "enjarciase": "enjarciar",
    "enjarciaseis": "enjarciar",
    "enjarciasen": "enjarciar",
    "enjarciases": "enjarciar",
    "enjarciaste": "enjarciar",
    "enjarciasteis": "enjarciar",
    "enjarcie": "enjarciar",
    "enjarciemos": "enjarciar",
    "enjarcien": "enjarciar",
    "enjarcies": "enjarciar",
    "enjarcio": "enjarciar",
    "enjarciábamos": "enjarciar",
    "enjarciáis": "enjarciar",
    "enjarciáramos": "enjarciar",
    "enjarciáremos": "enjarciar",
    "enjarciásemos": "enjarciar",
    "enjarcié": "enjarciar",
    "enjarciéis": "enjarciar",
    "enjarció": "enjarciar",
    "enjareta": "enjaretar",
    "enjaretaba": "enjaretar",
    "enjaretabais": "enjaretar",
    "enjaretaban": "enjaretar",
    "enjaretabas": "enjaretar",
    "enjaretad": "enjaretar",
    "enjaretada": "enjaretar",
    "enjaretadas": "enjaretar",
    "enjaretado": "enjaretar",
    "enjaretados": "enjaretar",
    "enjaretamos": "enjaretar",
    "enjaretan": "enjaretar",
    "enjaretando": "enjaretar",
    "enjaretara": "enjaretar",
    "enjaretarais": "enjaretar",
    "enjaretaran": "enjaretar",
    "enjaretaras": "enjaretar",
    "enjaretare": "enjaretar",
    "enjaretareis": "enjaretar",
    "enjaretaremos": "enjaretar",
    "enjaretaren": "enjaretar",
    "enjaretares": "enjaretar",
    "enjaretaron": "enjaretar",
    "enjaretará": "enjaretar",
    "enjaretarán": "enjaretar",
    "enjaretarás": "enjaretar",
    "enjaretaré": "enjaretar",
    "enjaretaréis": "enjaretar",
    "enjaretaría": "enjaretar",
    "enjaretaríais": "enjaretar",
    "enjaretaríamos": "enjaretar",
    "enjaretarían": "enjaretar",
    "enjaretarías": "enjaretar",
    "enjaretas": "enjaretar",
    "enjaretase": "enjaretar",
    "enjaretaseis": "enjaretar",
    "enjaretasen": "enjaretar",
    "enjaretases": "enjaretar",
    "enjaretaste": "enjaretar",
    "enjaretasteis": "enjaretar",
    "enjarete": "enjaretar",
    "enjaretemos": "enjaretar",
    "enjareten": "enjaretar",
    "enjaretes": "enjaretar",
    "enjareto": "enjaretar",
    "enjaretábamos": "enjaretar",
    "enjaretáis": "enjaretar",
    "enjaretáramos": "enjaretar",
    "enjaretáremos": "enjaretar",
    "enjaretásemos": "enjaretar",
    "enjareté": "enjaretar",
    "enjaretéis": "enjaretar",
    "enjaretó": "enjaretar",
    "enjarjes": "enjarje",
    "enjaula": "enjaular",
    "enjaulaba": "enjaular",
    "enjaulabais": "enjaular",
    "enjaulaban": "enjaular",
    "enjaulabas": "enjaular",
    "enjaulad": "enjaular",
    "enjaulada": "enjaular",
    "enjauladas": "enjaular",
    "enjaulado": "enjaular",
    "enjaulados": "enjaular",
    "enjaulamientos": "enjaulamiento",
    "enjaulamos": "enjaular",
    "enjaulan": "enjaular",
    "enjaulando": "enjaular",
    "enjaulara": "enjaular",
    "enjaularais": "enjaular",
    "enjaularan": "enjaular",
    "enjaularas": "enjaular",
    "enjaulare": "enjaular",
    "enjaulareis": "enjaular",
    "enjaularemos": "enjaular",
    "enjaularen": "enjaular",
    "enjaulares": "enjaular",
    "enjaularon": "enjaular",
    "enjaulará": "enjaular",
    "enjaularán": "enjaular",
    "enjaularás": "enjaular",
    "enjaularé": "enjaular",
    "enjaularéis": "enjaular",
    "enjaularía": "enjaular",
    "enjaularíais": "enjaular",
    "enjaularíamos": "enjaular",
    "enjaularían": "enjaular",
    "enjaularías": "enjaular",
    "enjaulas": "enjaular",
    "enjaulase": "enjaular",
    "enjaulaseis": "enjaular",
    "enjaulasen": "enjaular",
    "enjaulases": "enjaular",
    "enjaulaste": "enjaular",
    "enjaulasteis": "enjaular",
    "enjaule": "enjaular",
    "enjaulemos": "enjaular",
    "enjaulen": "enjaular",
    "enjaules": "enjaular",
    "enjaulo": "enjaular",
    "enjaulábamos": "enjaular",
    "enjauláis": "enjaular",
    "enjauláramos": "enjaular",
    "enjauláremos": "enjaular",
    "enjaulásemos": "enjaular",
    "enjaulé": "enjaular",
    "enjauléis": "enjaular",
    "enjauló": "enjaular",
    "enjeba": "enjebar",
    "enjebaba": "enjebar",
    "enjebabais": "enjebar",
    "enjebaban": "enjebar",
    "enjebabas": "enjebar",
    "enjebad": "enjebar",
    "enjebada": "enjebar",
    "enjebadas": "enjebar",
    "enjebado": "enjebar",
    "enjebados": "enjebar",
    "enjebamos": "enjebar",
    "enjeban": "enjebar",
    "enjebando": "enjebar",
    "enjebara": "enjebar",
    "enjebarais": "enjebar",
    "enjebaran": "enjebar",
    "enjebaras": "enjebar",
    "enjebare": "enjebar",
    "enjebareis": "enjebar",
    "enjebaremos": "enjebar",
    "enjebaren": "enjebar",
    "enjebares": "enjebar",
    "enjebaron": "enjebar",
    "enjebará": "enjebar",
    "enjebarán": "enjebar",
    "enjebarás": "enjebar",
    "enjebaré": "enjebar",
    "enjebaréis": "enjebar",
    "enjebaría": "enjebar",
    "enjebaríais": "enjebar",
    "enjebaríamos": "enjebar",
    "enjebarían": "enjebar",
    "enjebarías": "enjebar",
    "enjebas": "enjebar",
    "enjebase": "enjebar",
    "enjebaseis": "enjebar",
    "enjebasen": "enjebar",
    "enjebases": "enjebar",
    "enjebaste": "enjebar",
    "enjebasteis": "enjebar",
    "enjebe": "enjebar",
    "enjebemos": "enjebar",
    "enjeben": "enjebar",
    "enjebes": "enjebe",
    "enjebo": "enjebar",
    "enjebábamos": "enjebar",
    "enjebáis": "enjebar",
    "enjebáramos": "enjebar",
    "enjebáremos": "enjebar",
    "enjebásemos": "enjebar",
    "enjebé": "enjebar",
    "enjebéis": "enjebar",
    "enjebó": "enjebar",
    "enjerta": "enjertar",
    "enjertaba": "enjertar",
    "enjertabais": "enjertar",
    "enjertaban": "enjertar",
    "enjertabas": "enjertar",
    "enjertaciones": "enjertación",
    "enjertad": "enjertar",
    "enjertada": "enjertar",
    "enjertadas": "enjertar",
    "enjertado": "enjertar",
    "enjertados": "enjertar",
    "enjertamos": "enjertar",
    "enjertan": "enjertar",
    "enjertando": "enjertar",
    "enjertara": "enjertar",
    "enjertarais": "enjertar",
    "enjertaran": "enjertar",
    "enjertaras": "enjertar",
    "enjertare": "enjertar",
    "enjertareis": "enjertar",
    "enjertaremos": "enjertar",
    "enjertaren": "enjertar",
    "enjertares": "enjertar",
    "enjertaron": "enjertar",
    "enjertará": "enjertar",
    "enjertarán": "enjertar",
    "enjertarás": "enjertar",
    "enjertaré": "enjertar",
    "enjertaréis": "enjertar",
    "enjertaría": "enjertar",
    "enjertaríais": "enjertar",
    "enjertaríamos": "enjertar",
    "enjertarían": "enjertar",
    "enjertarías": "enjertar",
    "enjertas": "enjertar",
    "enjertase": "enjertar",
    "enjertaseis": "enjertar",
    "enjertasen": "enjertar",
    "enjertases": "enjertar",
    "enjertaste": "enjertar",
    "enjertasteis": "enjertar",
    "enjerte": "enjertar",
    "enjertemos": "enjertar",
    "enjerten": "enjertar",
    "enjertes": "enjertar",
    "enjerto": "enjertar",
    "enjertos": "enjerto",
    "enjertábamos": "enjertar",
    "enjertáis": "enjertar",
    "enjertáramos": "enjertar",
    "enjertáremos": "enjertar",
    "enjertásemos": "enjertar",
    "enjerté": "enjertar",
    "enjertéis": "enjertar",
    "enjertó": "enjertar",
    "enjoya": "enjoyar",
    "enjoyaba": "enjoyar",
    "enjoyabais": "enjoyar",
    "enjoyaban": "enjoyar",
    "enjoyabas": "enjoyar",
    "enjoyad": "enjoyar",
    "enjoyada": "enjoyar",
    "enjoyadas": "enjoyar",
    "enjoyado": "enjoyar",
    "enjoyados": "enjoyar",
    "enjoyamos": "enjoyar",
    "enjoyan": "enjoyar",
    "enjoyando": "enjoyar",
    "enjoyara": "enjoyar",
    "enjoyarais": "enjoyar",
    "enjoyaran": "enjoyar",
    "enjoyaras": "enjoyar",
    "enjoyare": "enjoyar",
    "enjoyareis": "enjoyar",
    "enjoyaremos": "enjoyar",
    "enjoyaren": "enjoyar",
    "enjoyares": "enjoyar",
    "enjoyaron": "enjoyar",
    "enjoyará": "enjoyar",
    "enjoyarán": "enjoyar",
    "enjoyarás": "enjoyar",
    "enjoyaré": "enjoyar",
    "enjoyaréis": "enjoyar",
    "enjoyaría": "enjoyar",
    "enjoyaríais": "enjoyar",
    "enjoyaríamos": "enjoyar",
    "enjoyarían": "enjoyar",
    "enjoyarías": "enjoyar",
    "enjoyas": "enjoyar",
    "enjoyase": "enjoyar",
    "enjoyaseis": "enjoyar",
    "enjoyasen": "enjoyar",
    "enjoyases": "enjoyar",
    "enjoyaste": "enjoyar",
    "enjoyasteis": "enjoyar",
    "enjoye": "enjoyar",
    "enjoyemos": "enjoyar",
    "enjoyen": "enjoyar",
    "enjoyes": "enjoyar",
    "enjoyo": "enjoyar",
    "enjoyábamos": "enjoyar",
    "enjoyáis": "enjoyar",
    "enjoyáramos": "enjoyar",
    "enjoyáremos": "enjoyar",
    "enjoyásemos": "enjoyar",
    "enjoyé": "enjoyar",
    "enjoyéis": "enjoyar",
    "enjoyó": "enjoyar",
    "enjuaga": "enjuagar",
    "enjuagaba": "enjuagar",
    "enjuagabais": "enjuagar",
    "enjuagaban": "enjuagar",
    "enjuagabas": "enjuagar",
    "enjuagad": "enjuagar",
    "enjuagada": "enjuagar",
    "enjuagadas": "enjuagar",
    "enjuagado": "enjuagar",
    "enjuagadoras": "enjuagadora",
    "enjuagados": "enjuagar",
    "enjuagaduras": "enjuagadura",
    "enjuagamos": "enjuagar",
    "enjuagan": "enjuagar",
    "enjuagando": "enjuagar",
    "enjuagara": "enjuagar",
    "enjuagarais": "enjuagar",
    "enjuagaran": "enjuagar",
    "enjuagaras": "enjuagar",
    "enjuagare": "enjuagar",
    "enjuagareis": "enjuagar",
    "enjuagaremos": "enjuagar",
    "enjuagaren": "enjuagar",
    "enjuagares": "enjuagar",
    "enjuagaron": "enjuagar",
    "enjuagará": "enjuagar",
    "enjuagarán": "enjuagar",
    "enjuagarás": "enjuagar",
    "enjuagaré": "enjuagar",
    "enjuagaréis": "enjuagar",
    "enjuagaría": "enjuagar",
    "enjuagaríais": "enjuagar",
    "enjuagaríamos": "enjuagar",
    "enjuagarían": "enjuagar",
    "enjuagarías": "enjuagar",
    "enjuagas": "enjuagar",
    "enjuagase": "enjuagar",
    "enjuagaseis": "enjuagar",
    "enjuagasen": "enjuagar",
    "enjuagases": "enjuagar",
    "enjuagaste": "enjuagar",
    "enjuagasteis": "enjuagar",
    "enjuagatorios": "enjuagatorio",
    "enjuago": "enjuagar",
    "enjuague": "enjuagar",
    "enjuaguemos": "enjuagar",
    "enjuaguen": "enjuagar",
    "enjuagues": "enjuague",
    "enjuagué": "enjuagar",
    "enjuaguéis": "enjuagar",
    "enjuagábamos": "enjuagar",
    "enjuagáis": "enjuagar",
    "enjuagáramos": "enjuagar",
    "enjuagáremos": "enjuagar",
    "enjuagásemos": "enjuagar",
    "enjuagó": "enjuagar",
    "enjuga": "enjugar",
    "enjugaba": "enjugar",
    "enjugabais": "enjugar",
    "enjugaban": "enjugar",
    "enjugabas": "enjugar",
    "enjugad": "enjugar",
    "enjugada": "enjugar",
    "enjugadas": "enjugar",
    "enjugado": "enjugar",
    "enjugadora": "enjugador",
    "enjugadoras": "enjugador",
    "enjugadores": "enjugador",
    "enjugados": "enjugar",
    "enjugamos": "enjugar",
    "enjugan": "enjugar",
    "enjugando": "enjugar",
    "enjugara": "enjugar",
    "enjugarais": "enjugar",
    "enjugaran": "enjugar",
    "enjugaras": "enjugar",
    "enjugare": "enjugar",
    "enjugareis": "enjugar",
    "enjugaremos": "enjugar",
    "enjugaren": "enjugar",
    "enjugares": "enjugar",
    "enjugaron": "enjugar",
    "enjugará": "enjugar",
    "enjugarán": "enjugar",
    "enjugarás": "enjugar",
    "enjugaré": "enjugar",
    "enjugaréis": "enjugar",
    "enjugaría": "enjugar",
    "enjugaríais": "enjugar",
    "enjugaríamos": "enjugar",
    "enjugarían": "enjugar",
    "enjugarías": "enjugar",
    "enjugas": "enjugar",
    "enjugase": "enjugar",
    "enjugaseis": "enjugar",
    "enjugasen": "enjugar",
    "enjugases": "enjugar",
    "enjugaste": "enjugar",
    "enjugasteis": "enjugar",
    "enjugo": "enjugar",
    "enjugue": "enjugar",
    "enjuguemos": "enjugar",
    "enjuguen": "enjugar",
    "enjugues": "enjugar",
    "enjugué": "enjugar",
    "enjuguéis": "enjugar",
    "enjugábamos": "enjugar",
    "enjugáis": "enjugar",
    "enjugáramos": "enjugar",
    "enjugáremos": "enjugar",
    "enjugásemos": "enjugar",
    "enjugó": "enjugar",
    "enjuicia": "enjuiciar",
    "enjuiciaba": "enjuiciar",
    "enjuiciabais": "enjuiciar",
    "enjuiciaban": "enjuiciar",
    "enjuiciabas": "enjuiciar",
    "enjuiciables": "enjuiciable",
    "enjuiciad": "enjuiciar",
    "enjuiciada": "enjuiciar",
    "enjuiciadas": "enjuiciar",
    "enjuiciado": "enjuiciar",
    "enjuiciados": "enjuiciar",
    "enjuiciamientos": "enjuiciamiento",
    "enjuiciamos": "enjuiciar",
    "enjuician": "enjuiciar",
    "enjuiciando": "enjuiciar",
    "enjuiciara": "enjuiciar",
    "enjuiciarais": "enjuiciar",
    "enjuiciaran": "enjuiciar",
    "enjuiciaras": "enjuiciar",
    "enjuiciare": "enjuiciar",
    "enjuiciareis": "enjuiciar",
    "enjuiciaremos": "enjuiciar",
    "enjuiciaren": "enjuiciar",
    "enjuiciares": "enjuiciar",
    "enjuiciaron": "enjuiciar",
    "enjuiciará": "enjuiciar",
    "enjuiciarán": "enjuiciar",
    "enjuiciarás": "enjuiciar",
    "enjuiciaré": "enjuiciar",
    "enjuiciaréis": "enjuiciar",
    "enjuiciaría": "enjuiciar",
    "enjuiciaríais": "enjuiciar",
    "enjuiciaríamos": "enjuiciar",
    "enjuiciarían": "enjuiciar",
    "enjuiciarías": "enjuiciar",
    "enjuicias": "enjuiciar",
    "enjuiciase": "enjuiciar",
    "enjuiciaseis": "enjuiciar",
    "enjuiciasen": "enjuiciar",
    "enjuiciases": "enjuiciar",
    "enjuiciaste": "enjuiciar",
    "enjuiciasteis": "enjuiciar",
    "enjuicie": "enjuiciar",
    "enjuiciemos": "enjuiciar",
    "enjuicien": "enjuiciar",
    "enjuicies": "enjuiciar",
    "enjuicio": "enjuiciar",
    "enjuiciábamos": "enjuiciar",
    "enjuiciáis": "enjuiciar",
    "enjuiciáramos": "enjuiciar",
    "enjuiciáremos": "enjuiciar",
    "enjuiciásemos": "enjuiciar",
    "enjuicié": "enjuiciar",
    "enjuiciéis": "enjuiciar",
    "enjuició": "enjuiciar",
    "enjulios": "enjulio",
    "enjullos": "enjullo",
    "enjundias": "enjundia",
    "enjundiosa": "enjundioso",
    "enjundiosas": "enjundioso",
    "enjundiosos": "enjundioso",
    "enjunques": "enjunque",
    "enjuta": "enjuto",
    "enjutaba": "enjutar",
    "enjutabais": "enjutar",
    "enjutaban": "enjutar",
    "enjutabas": "enjutar",
    "enjutad": "enjutar",
    "enjutada": "enjutar",
    "enjutadas": "enjutar",
    "enjutado": "enjutar",
    "enjutados": "enjutar",
    "enjutamos": "enjutar",
    "enjutan": "enjutar",
    "enjutando": "enjutar",
    "enjutara": "enjutar",
    "enjutarais": "enjutar",
    "enjutaran": "enjutar",
    "enjutaras": "enjutar",
    "enjutare": "enjutar",
    "enjutareis": "enjutar",
    "enjutaremos": "enjutar",
    "enjutaren": "enjutar",
    "enjutares": "enjutar",
    "enjutaron": "enjutar",
    "enjutará": "enjutar",
    "enjutarán": "enjutar",
    "enjutarás": "enjutar",
    "enjutaré": "enjutar",
    "enjutaréis": "enjutar",
    "enjutaría": "enjutar",
    "enjutaríais": "enjutar",
    "enjutaríamos": "enjutar",
    "enjutarían": "enjutar",
    "enjutarías": "enjutar",
    "enjutas": "enjuto",
    "enjutase": "enjutar",
    "enjutaseis": "enjutar",
    "enjutasen": "enjutar",
    "enjutases": "enjutar",
    "enjutaste": "enjutar",
    "enjutasteis": "enjutar",
    "enjute": "enjutar",
    "enjuteces": "enjutez",
    "enjutemos": "enjutar",
    "enjuten": "enjutar",
    "enjutes": "enjutar",
    "enjuto": "enjutar",
    "enjutos": "enjuto",
    "enjutábamos": "enjutar",
    "enjutáis": "enjutar",
    "enjutáramos": "enjutar",
    "enjutáremos": "enjutar",
    "enjutásemos": "enjutar",
    "enjuté": "enjutar",
    "enjutéis": "enjutar",
    "enjutó": "enjutar",
    "enlabia": "enlabiar",
    "enlabiaba": "enlabiar",
    "enlabiabais": "enlabiar",
    "enlabiaban": "enlabiar",
    "enlabiabas": "enlabiar",
    "enlabiad": "enlabiar",
    "enlabiada": "enlabiar",
    "enlabiadas": "enlabiar",
    "enlabiado": "enlabiar",
    "enlabiados": "enlabiar",
    "enlabiamos": "enlabiar",
    "enlabian": "enlabiar",
    "enlabiando": "enlabiar",
    "enlabiara": "enlabiar",
    "enlabiarais": "enlabiar",
    "enlabiaran": "enlabiar",
    "enlabiaras": "enlabiar",
    "enlabiare": "enlabiar",
    "enlabiareis": "enlabiar",
    "enlabiaremos": "enlabiar",
    "enlabiaren": "enlabiar",
    "enlabiares": "enlabiar",
    "enlabiaron": "enlabiar",
    "enlabiará": "enlabiar",
    "enlabiarán": "enlabiar",
    "enlabiarás": "enlabiar",
    "enlabiaré": "enlabiar",
    "enlabiaréis": "enlabiar",
    "enlabiaría": "enlabiar",
    "enlabiaríais": "enlabiar",
    "enlabiaríamos": "enlabiar",
    "enlabiarían": "enlabiar",
    "enlabiarías": "enlabiar",
    "enlabias": "enlabiar",
    "enlabiase": "enlabiar",
    "enlabiaseis": "enlabiar",
    "enlabiasen": "enlabiar",
    "enlabiases": "enlabiar",
    "enlabiaste": "enlabiar",
    "enlabiasteis": "enlabiar",
    "enlabie": "enlabiar",
    "enlabiemos": "enlabiar",
    "enlabien": "enlabiar",
    "enlabies": "enlabiar",
    "enlabio": "enlabiar",
    "enlabios": "enlabio",
    "enlabiábamos": "enlabiar",
    "enlabiáis": "enlabiar",
    "enlabiáramos": "enlabiar",
    "enlabiáremos": "enlabiar",
    "enlabiásemos": "enlabiar",
    "enlabié": "enlabiar",
    "enlabiéis": "enlabiar",
    "enlabió": "enlabiar",
    "enlace": "enlazar",
    "enlacemos": "enlazar",
    "enlacen": "enlazar",
    "enlaces": "enlazar",
    "enlacé": "enlazar",
    "enlacéis": "enlazar",
    "enladrilla": "enladrillar",
    "enladrillaba": "enladrillar",
    "enladrillabais": "enladrillar",
    "enladrillaban": "enladrillar",
    "enladrillabas": "enladrillar",
    "enladrillad": "enladrillar",
    "enladrillada": "enladrillar",
    "enladrilladas": "enladrillar",
    "enladrillado": "enladrillar",
    "enladrilladora": "enladrillador",
    "enladrilladoras": "enladrillador",
    "enladrilladores": "enladrillador",
    "enladrillados": "enladrillar",
    "enladrilladuras": "enladrilladura",
    "enladrillamos": "enladrillar",
    "enladrillan": "enladrillar",
    "enladrillando": "enladrillar",
    "enladrillara": "enladrillar",
    "enladrillarais": "enladrillar",
    "enladrillaran": "enladrillar",
    "enladrillaras": "enladrillar",
    "enladrillare": "enladrillar",
    "enladrillareis": "enladrillar",
    "enladrillaremos": "enladrillar",
    "enladrillaren": "enladrillar",
    "enladrillares": "enladrillar",
    "enladrillaron": "enladrillar",
    "enladrillará": "enladrillar",
    "enladrillarán": "enladrillar",
    "enladrillarás": "enladrillar",
    "enladrillaré": "enladrillar",
    "enladrillaréis": "enladrillar",
    "enladrillaría": "enladrillar",
    "enladrillaríais": "enladrillar",
    "enladrillaríamos": "enladrillar",
    "enladrillarían": "enladrillar",
    "enladrillarías": "enladrillar",
    "enladrillas": "enladrillar",
    "enladrillase": "enladrillar",
    "enladrillaseis": "enladrillar",
    "enladrillasen": "enladrillar",
    "enladrillases": "enladrillar",
    "enladrillaste": "enladrillar",
    "enladrillasteis": "enladrillar",
    "enladrille": "enladrillar",
    "enladrillemos": "enladrillar",
    "enladrillen": "enladrillar",
    "enladrilles": "enladrillar",
    "enladrillo": "enladrillar",
    "enladrillábamos": "enladrillar",
    "enladrilláis": "enladrillar",
    "enladrilláramos": "enladrillar",
    "enladrilláremos": "enladrillar",
    "enladrillásemos": "enladrillar",
    "enladrillé": "enladrillar",
    "enladrilléis": "enladrillar",
    "enladrilló": "enladrillar",
    "enlaguna": "enlagunar",
    "enlagunaba": "enlagunar",
    "enlagunabais": "enlagunar",
    "enlagunaban": "enlagunar",
    "enlagunabas": "enlagunar",
    "enlagunad": "enlagunar",
    "enlagunada": "enlagunar",
    "enlagunadas": "enlagunar",
    "enlagunado": "enlagunar",
    "enlagunados": "enlagunar",
    "enlagunamos": "enlagunar",
    "enlagunan": "enlagunar",
    "enlagunando": "enlagunar",
    "enlagunara": "enlagunar",
    "enlagunarais": "enlagunar",
    "enlagunaran": "enlagunar",
    "enlagunaras": "enlagunar",
    "enlagunare": "enlagunar",
    "enlagunareis": "enlagunar",
    "enlagunaremos": "enlagunar",
    "enlagunaren": "enlagunar",
    "enlagunares": "enlagunar",
    "enlagunaron": "enlagunar",
    "enlagunará": "enlagunar",
    "enlagunarán": "enlagunar",
    "enlagunarás": "enlagunar",
    "enlagunaré": "enlagunar",
    "enlagunaréis": "enlagunar",
    "enlagunaría": "enlagunar",
    "enlagunaríais": "enlagunar",
    "enlagunaríamos": "enlagunar",
    "enlagunarían": "enlagunar",
    "enlagunarías": "enlagunar",
    "enlagunas": "enlagunar",
    "enlagunase": "enlagunar",
    "enlagunaseis": "enlagunar",
    "enlagunasen": "enlagunar",
    "enlagunases": "enlagunar",
    "enlagunaste": "enlagunar",
    "enlagunasteis": "enlagunar",
    "enlagune": "enlagunar",
    "enlagunemos": "enlagunar",
    "enlagunen": "enlagunar",
    "enlagunes": "enlagunar",
    "enlaguno": "enlagunar",
    "enlagunábamos": "enlagunar",
    "enlagunáis": "enlagunar",
    "enlagunáramos": "enlagunar",
    "enlagunáremos": "enlagunar",
    "enlagunásemos": "enlagunar",
    "enlaguné": "enlagunar",
    "enlagunéis": "enlagunar",
    "enlagunó": "enlagunar",
    "enlama": "enlamar",
    "enlamaba": "enlamar",
    "enlamabais": "enlamar",
    "enlamaban": "enlamar",
    "enlamabas": "enlamar",
    "enlamad": "enlamar",
    "enlamada": "enlamar",
    "enlamadas": "enlamar",
    "enlamado": "enlamar",
    "enlamados": "enlamar",
    "enlamamos": "enlamar",
    "enlaman": "enlamar",
    "enlamando": "enlamar",
    "enlamara": "enlamar",
    "enlamarais": "enlamar",
    "enlamaran": "enlamar",
    "enlamaras": "enlamar",
    "enlamare": "enlamar",
    "enlamareis": "enlamar",
    "enlamaremos": "enlamar",
    "enlamaren": "enlamar",
    "enlamares": "enlamar",
    "enlamaron": "enlamar",
    "enlamará": "enlamar",
    "enlamarán": "enlamar",
    "enlamarás": "enlamar",
    "enlamaré": "enlamar",
    "enlamaréis": "enlamar",
    "enlamaría": "enlamar",
    "enlamaríais": "enlamar",
    "enlamaríamos": "enlamar",
    "enlamarían": "enlamar",
    "enlamarías": "enlamar",
    "enlamas": "enlamar",
    "enlamase": "enlamar",
    "enlamaseis": "enlamar",
    "enlamasen": "enlamar",
    "enlamases": "enlamar",
    "enlamaste": "enlamar",
    "enlamasteis": "enlamar",
    "enlame": "enlamar",
    "enlamemos": "enlamar",
    "enlamen": "enlamar",
    "enlames": "enlamar",
    "enlamo": "enlamar",
    "enlamábamos": "enlamar",
    "enlamáis": "enlamar",
    "enlamáramos": "enlamar",
    "enlamáremos": "enlamar",
    "enlamásemos": "enlamar",
    "enlamé": "enlamar",
    "enlaméis": "enlamar",
    "enlamó": "enlamar",
    "enlanada": "enlanado",
    "enlanadas": "enlanado",
    "enlanados": "enlanado",
    "enlata": "enlatar",
    "enlataba": "enlatar",
    "enlatabais": "enlatar",
    "enlataban": "enlatar",
    "enlatabas": "enlatar",
    "enlatad": "enlatar",
    "enlatada": "enlatar",
    "enlatadas": "enlatar",
    "enlatado": "enlatar",
    "enlatados": "enlatar",
    "enlatamos": "enlatar",
    "enlatan": "enlatar",
    "enlatando": "enlatar",
    "enlatara": "enlatar",
    "enlatarais": "enlatar",
    "enlataran": "enlatar",
    "enlataras": "enlatar",
    "enlatare": "enlatar",
    "enlatareis": "enlatar",
    "enlataremos": "enlatar",
    "enlataren": "enlatar",
    "enlatares": "enlatar",
    "enlataron": "enlatar",
    "enlatará": "enlatar",
    "enlatarán": "enlatar",
    "enlatarás": "enlatar",
    "enlataré": "enlatar",
    "enlataréis": "enlatar",
    "enlataría": "enlatar",
    "enlataríais": "enlatar",
    "enlataríamos": "enlatar",
    "enlatarían": "enlatar",
    "enlatarías": "enlatar",
    "enlatas": "enlatar",
    "enlatase": "enlatar",
    "enlataseis": "enlatar",
    "enlatasen": "enlatar",
    "enlatases": "enlatar",
    "enlataste": "enlatar",
    "enlatasteis": "enlatar",
    "enlate": "enlatar",
    "enlatemos": "enlatar",
    "enlaten": "enlatar",
    "enlates": "enlatar",
    "enlato": "enlatar",
    "enlatábamos": "enlatar",
    "enlatáis": "enlatar",
    "enlatáramos": "enlatar",
    "enlatáremos": "enlatar",
    "enlatásemos": "enlatar",
    "enlaté": "enlatar",
    "enlatéis": "enlatar",
    "enlató": "enlatar",
    "enlaza": "enlazar",
    "enlazaba": "enlazar",
    "enlazabais": "enlazar",
    "enlazaban": "enlazar",
    "enlazabas": "enlazar",
    "enlazables": "enlazable",
    "enlazad": "enlazar",
    "enlazada": "enlazar",
    "enlazadas": "enlazar",
    "enlazado": "enlazar",
    "enlazadores": "enlazador",
    "enlazados": "enlazar",
    "enlazamientos": "enlazamiento",
    "enlazamos": "enlazar",
    "enlazan": "enlazar",
    "enlazando": "enlazar",
    "enlazara": "enlazar",
    "enlazarais": "enlazar",
    "enlazaran": "enlazar",
    "enlazaras": "enlazar",
    "enlazare": "enlazar",
    "enlazareis": "enlazar",
    "enlazaremos": "enlazar",
    "enlazaren": "enlazar",
    "enlazares": "enlazar",
    "enlazaron": "enlazar",
    "enlazará": "enlazar",
    "enlazarán": "enlazar",
    "enlazarás": "enlazar",
    "enlazaré": "enlazar",
    "enlazaréis": "enlazar",
    "enlazaría": "enlazar",
    "enlazaríais": "enlazar",
    "enlazaríamos": "enlazar",
    "enlazarían": "enlazar",
    "enlazarías": "enlazar",
    "enlazas": "enlazar",
    "enlazase": "enlazar",
    "enlazaseis": "enlazar",
    "enlazasen": "enlazar",
    "enlazases": "enlazar",
    "enlazaste": "enlazar",
    "enlazasteis": "enlazar",
    "enlazo": "enlazar",
    "enlazábamos": "enlazar",
    "enlazáis": "enlazar",
    "enlazáramos": "enlazar",
    "enlazáremos": "enlazar",
    "enlazásemos": "enlazar",
    "enlazó": "enlazar",
    "enlecha": "enlechar",
    "enlechaba": "enlechar",
    "enlechabais": "enlechar",
    "enlechaban": "enlechar",
    "enlechabas": "enlechar",
    "enlechad": "enlechar",
    "enlechada": "enlechar",
    "enlechadas": "enlechar",
    "enlechado": "enlechar",
    "enlechados": "enlechar",
    "enlechamos": "enlechar",
    "enlechan": "enlechar",
    "enlechando": "enlechar",
    "enlechara": "enlechar",
    "enlecharais": "enlechar",
    "enlecharan": "enlechar",
    "enlecharas": "enlechar",
    "enlechare": "enlechar",
    "enlechareis": "enlechar",
    "enlecharemos": "enlechar",
    "enlecharen": "enlechar",
    "enlechares": "enlechar",
    "enlecharon": "enlechar",
    "enlechará": "enlechar",
    "enlecharán": "enlechar",
    "enlecharás": "enlechar",
    "enlecharé": "enlechar",
    "enlecharéis": "enlechar",
    "enlecharía": "enlechar",
    "enlecharíais": "enlechar",
    "enlecharíamos": "enlechar",
    "enlecharían": "enlechar",
    "enlecharías": "enlechar",
    "enlechas": "enlechar",
    "enlechase": "enlechar",
    "enlechaseis": "enlechar",
    "enlechasen": "enlechar",
    "enlechases": "enlechar",
    "enlechaste": "enlechar",
    "enlechasteis": "enlechar",
    "enleche": "enlechar",
    "enlechemos": "enlechar",
    "enlechen": "enlechar",
    "enleches": "enlechar",
    "enlecho": "enlechar",
    "enlechábamos": "enlechar",
    "enlecháis": "enlechar",
    "enlecháramos": "enlechar",
    "enlecháremos": "enlechar",
    "enlechásemos": "enlechar",
    "enleché": "enlechar",
    "enlechéis": "enlechar",
    "enlechó": "enlechar",
    "enlegaja": "enlegajar",
    "enlegajaba": "enlegajar",
    "enlegajabais": "enlegajar",
    "enlegajaban": "enlegajar",
    "enlegajabas": "enlegajar",
    "enlegajad": "enlegajar",
    "enlegajada": "enlegajar",
    "enlegajadas": "enlegajar",
    "enlegajado": "enlegajar",
    "enlegajados": "enlegajar",
    "enlegajamos": "enlegajar",
    "enlegajan": "enlegajar",
    "enlegajando": "enlegajar",
    "enlegajara": "enlegajar",
    "enlegajarais": "enlegajar",
    "enlegajaran": "enlegajar",
    "enlegajaras": "enlegajar",
    "enlegajare": "enlegajar",
    "enlegajareis": "enlegajar",
    "enlegajaremos": "enlegajar",
    "enlegajaren": "enlegajar",
    "enlegajares": "enlegajar",
    "enlegajaron": "enlegajar",
    "enlegajará": "enlegajar",
    "enlegajarán": "enlegajar",
    "enlegajarás": "enlegajar",
    "enlegajaré": "enlegajar",
    "enlegajaréis": "enlegajar",
    "enlegajaría": "enlegajar",
    "enlegajaríais": "enlegajar",
    "enlegajaríamos": "enlegajar",
    "enlegajarían": "enlegajar",
    "enlegajarías": "enlegajar",
    "enlegajas": "enlegajar",
    "enlegajase": "enlegajar",
    "enlegajaseis": "enlegajar",
    "enlegajasen": "enlegajar",
    "enlegajases": "enlegajar",
    "enlegajaste": "enlegajar",
    "enlegajasteis": "enlegajar",
    "enlegaje": "enlegajar",
    "enlegajemos": "enlegajar",
    "enlegajen": "enlegajar",
    "enlegajes": "enlegajar",
    "enlegajo": "enlegajar",
    "enlegajábamos": "enlegajar",
    "enlegajáis": "enlegajar",
    "enlegajáramos": "enlegajar",
    "enlegajáremos": "enlegajar",
    "enlegajásemos": "enlegajar",
    "enlegajé": "enlegajar",
    "enlegajéis": "enlegajar",
    "enlegajó": "enlegajar",
    "enlegama": "enlegamar",
    "enlegamaba": "enlegamar",
    "enlegamabais": "enlegamar",
    "enlegamaban": "enlegamar",
    "enlegamabas": "enlegamar",
    "enlegamad": "enlegamar",
    "enlegamada": "enlegamar",
    "enlegamadas": "enlegamar",
    "enlegamado": "enlegamar",
    "enlegamados": "enlegamar",
    "enlegamamos": "enlegamar",
    "enlegaman": "enlegamar",
    "enlegamando": "enlegamar",
    "enlegamara": "enlegamar",
    "enlegamarais": "enlegamar",
    "enlegamaran": "enlegamar",
    "enlegamaras": "enlegamar",
    "enlegamare": "enlegamar",
    "enlegamareis": "enlegamar",
    "enlegamaremos": "enlegamar",
    "enlegamaren": "enlegamar",
    "enlegamares": "enlegamar",
    "enlegamaron": "enlegamar",
    "enlegamará": "enlegamar",
    "enlegamarán": "enlegamar",
    "enlegamarás": "enlegamar",
    "enlegamaré": "enlegamar",
    "enlegamaréis": "enlegamar",
    "enlegamaría": "enlegamar",
    "enlegamaríais": "enlegamar",
    "enlegamaríamos": "enlegamar",
    "enlegamarían": "enlegamar",
    "enlegamarías": "enlegamar",
    "enlegamas": "enlegamar",
    "enlegamase": "enlegamar",
    "enlegamaseis": "enlegamar",
    "enlegamasen": "enlegamar",
    "enlegamases": "enlegamar",
    "enlegamaste": "enlegamar",
    "enlegamasteis": "enlegamar",
    "enlegame": "enlegamar",
    "enlegamemos": "enlegamar",
    "enlegamen": "enlegamar",
    "enlegames": "enlegamar",
    "enlegamo": "enlegamar",
    "enlegamábamos": "enlegamar",
    "enlegamáis": "enlegamar",
    "enlegamáramos": "enlegamar",
    "enlegamáremos": "enlegamar",
    "enlegamásemos": "enlegamar",
    "enlegamé": "enlegamar",
    "enlegaméis": "enlegamar",
    "enlegamó": "enlegamar",
    "enlejiaba": "enlejiar",
    "enlejiabais": "enlejiar",
    "enlejiaban": "enlejiar",
    "enlejiabas": "enlejiar",
    "enlejiad": "enlejiar",
    "enlejiada": "enlejiar",
    "enlejiadas": "enlejiar",
    "enlejiado": "enlejiar",
    "enlejiados": "enlejiar",
    "enlejiamos": "enlejiar",
    "enlejiando": "enlejiar",
    "enlejiara": "enlejiar",
    "enlejiarais": "enlejiar",
    "enlejiaran": "enlejiar",
    "enlejiaras": "enlejiar",
    "enlejiare": "enlejiar",
    "enlejiareis": "enlejiar",
    "enlejiaremos": "enlejiar",
    "enlejiaren": "enlejiar",
    "enlejiares": "enlejiar",
    "enlejiaron": "enlejiar",
    "enlejiará": "enlejiar",
    "enlejiarán": "enlejiar",
    "enlejiarás": "enlejiar",
    "enlejiaré": "enlejiar",
    "enlejiaréis": "enlejiar",
    "enlejiaría": "enlejiar",
    "enlejiaríais": "enlejiar",
    "enlejiaríamos": "enlejiar",
    "enlejiarían": "enlejiar",
    "enlejiarías": "enlejiar",
    "enlejiase": "enlejiar",
    "enlejiaseis": "enlejiar",
    "enlejiasen": "enlejiar",
    "enlejiases": "enlejiar",
    "enlejiaste": "enlejiar",
    "enlejiasteis": "enlejiar",
    "enlejiemos": "enlejiar",
    "enlejiábamos": "enlejiar",
    "enlejiáis": "enlejiar",
    "enlejiáramos": "enlejiar",
    "enlejiáremos": "enlejiar",
    "enlejiásemos": "enlejiar",
    "enlejié": "enlejiar",
    "enlejiéis": "enlejiar",
    "enlejió": "enlejiar",
    "enlejía": "enlejiar",
    "enlejían": "enlejiar",
    "enlejías": "enlejiar",
    "enlejíe": "enlejiar",
    "enlejíen": "enlejiar",
    "enlejíes": "enlejiar",
    "enlejío": "enlejiar",
    "enlentecimientos": "enlentecimiento",
    "enlevitada": "enlevitado",
    "enlevitadas": "enlevitado",
    "enlevitados": "enlevitado",
    "enliga": "enligar",
    "enligaba": "enligar",
    "enligabais": "enligar",
    "enligaban": "enligar",
    "enligabas": "enligar",
    "enligad": "enligar",
    "enligada": "enligar",
    "enligadas": "enligar",
    "enligado": "enligar",
    "enligados": "enligar",
    "enligamos": "enligar",
    "enligan": "enligar",
    "enligando": "enligar",
    "enligara": "enligar",
    "enligarais": "enligar",
    "enligaran": "enligar",
    "enligaras": "enligar",
    "enligare": "enligar",
    "enligareis": "enligar",
    "enligaremos": "enligar",
    "enligaren": "enligar",
    "enligares": "enligar",
    "enligaron": "enligar",
    "enligará": "enligar",
    "enligarán": "enligar",
    "enligarás": "enligar",
    "enligaré": "enligar",
    "enligaréis": "enligar",
    "enligaría": "enligar",
    "enligaríais": "enligar",
    "enligaríamos": "enligar",
    "enligarían": "enligar",
    "enligarías": "enligar",
    "enligas": "enligar",
    "enligase": "enligar",
    "enligaseis": "enligar",
    "enligasen": "enligar",
    "enligases": "enligar",
    "enligaste": "enligar",
    "enligasteis": "enligar",
    "enligo": "enligar",
    "enligue": "enligar",
    "enliguemos": "enligar",
    "enliguen": "enligar",
    "enligues": "enligar",
    "enligué": "enligar",
    "enliguéis": "enligar",
    "enligábamos": "enligar",
    "enligáis": "enligar",
    "enligáramos": "enligar",
    "enligáremos": "enligar",
    "enligásemos": "enligar",
    "enligó": "enligar",
    "enlistona": "enlistonar",
    "enlistonaba": "enlistonar",
    "enlistonabais": "enlistonar",
    "enlistonaban": "enlistonar",
    "enlistonabas": "enlistonar",
    "enlistonad": "enlistonar",
    "enlistonada": "enlistonar",
    "enlistonadas": "enlistonar",
    "enlistonado": "enlistonar",
    "enlistonados": "enlistonar",
    "enlistonamos": "enlistonar",
    "enlistonan": "enlistonar",
    "enlistonando": "enlistonar",
    "enlistonara": "enlistonar",
    "enlistonarais": "enlistonar",
    "enlistonaran": "enlistonar",
    "enlistonaras": "enlistonar",
    "enlistonare": "enlistonar",
    "enlistonareis": "enlistonar",
    "enlistonaremos": "enlistonar",
    "enlistonaren": "enlistonar",
    "enlistonares": "enlistonar",
    "enlistonaron": "enlistonar",
    "enlistonará": "enlistonar",
    "enlistonarán": "enlistonar",
    "enlistonarás": "enlistonar",
    "enlistonaré": "enlistonar",
    "enlistonaréis": "enlistonar",
    "enlistonaría": "enlistonar",
    "enlistonaríais": "enlistonar",
    "enlistonaríamos": "enlistonar",
    "enlistonarían": "enlistonar",
    "enlistonarías": "enlistonar",
    "enlistonas": "enlistonar",
    "enlistonase": "enlistonar",
    "enlistonaseis": "enlistonar",
    "enlistonasen": "enlistonar",
    "enlistonases": "enlistonar",
    "enlistonaste": "enlistonar",
    "enlistonasteis": "enlistonar",
    "enlistone": "enlistonar",
    "enlistonemos": "enlistonar",
    "enlistonen": "enlistonar",
    "enlistones": "enlistonar",
    "enlistono": "enlistonar",
    "enlistonábamos": "enlistonar",
    "enlistonáis": "enlistonar",
    "enlistonáramos": "enlistonar",
    "enlistonáremos": "enlistonar",
    "enlistonásemos": "enlistonar",
    "enlistoné": "enlistonar",
    "enlistonéis": "enlistonar",
    "enlistonó": "enlistonar",
    "enllanta": "enllantar",
    "enllantaba": "enllantar",
    "enllantabais": "enllantar",
    "enllantaban": "enllantar",
    "enllantabas": "enllantar",
    "enllantad": "enllantar",
    "enllantada": "enllantar",
    "enllantadas": "enllantar",
    "enllantado": "enllantar",
    "enllantados": "enllantar",
    "enllantamos": "enllantar",
    "enllantan": "enllantar",
    "enllantando": "enllantar",
    "enllantara": "enllantar",
    "enllantarais": "enllantar",
    "enllantaran": "enllantar",
    "enllantaras": "enllantar",
    "enllantare": "enllantar",
    "enllantareis": "enllantar",
    "enllantaremos": "enllantar",
    "enllantaren": "enllantar",
    "enllantares": "enllantar",
    "enllantaron": "enllantar",
    "enllantará": "enllantar",
    "enllantarán": "enllantar",
    "enllantarás": "enllantar",
    "enllantaré": "enllantar",
    "enllantaréis": "enllantar",
    "enllantaría": "enllantar",
    "enllantaríais": "enllantar",
    "enllantaríamos": "enllantar",
    "enllantarían": "enllantar",
    "enllantarías": "enllantar",
    "enllantas": "enllantar",
    "enllantase": "enllantar",
    "enllantaseis": "enllantar",
    "enllantasen": "enllantar",
    "enllantases": "enllantar",
    "enllantaste": "enllantar",
    "enllantasteis": "enllantar",
    "enllante": "enllantar",
    "enllantemos": "enllantar",
    "enllanten": "enllantar",
    "enllantes": "enllante",
    "enllanto": "enllantar",
    "enllantábamos": "enllantar",
    "enllantáis": "enllantar",
    "enllantáramos": "enllantar",
    "enllantáremos": "enllantar",
    "enllantásemos": "enllantar",
    "enllanté": "enllantar",
    "enllantéis": "enllantar",
    "enllantó": "enllantar",
    "enlobreguece": "enlobreguecer",
    "enlobregueced": "enlobreguecer",
    "enlobreguecemos": "enlobreguecer",
    "enlobreguecen": "enlobreguecer",
    "enlobregueceremos": "enlobreguecer",
    "enlobreguecerá": "enlobreguecer",
    "enlobreguecerán": "enlobreguecer",
    "enlobreguecerás": "enlobreguecer",
    "enlobregueceré": "enlobreguecer",
    "enlobregueceréis": "enlobreguecer",
    "enlobreguecería": "enlobreguecer",
    "enlobregueceríais": "enlobreguecer",
    "enlobregueceríamos": "enlobreguecer",
    "enlobreguecerían": "enlobreguecer",
    "enlobreguecerías": "enlobreguecer",
    "enlobregueces": "enlobreguecer",
    "enlobreguecida": "enlobreguecer",
    "enlobreguecidas": "enlobreguecer",
    "enlobreguecido": "enlobreguecer",
    "enlobreguecidos": "enlobreguecer",
    "enlobregueciendo": "enlobreguecer",
    "enlobregueciera": "enlobreguecer",
    "enlobreguecierais": "enlobreguecer",
    "enlobreguecieran": "enlobreguecer",
    "enlobreguecieras": "enlobreguecer",
    "enlobregueciere": "enlobreguecer",
    "enlobregueciereis": "enlobreguecer",
    "enlobreguecieren": "enlobreguecer",
    "enlobreguecieres": "enlobreguecer",
    "enlobreguecieron": "enlobreguecer",
    "enlobregueciese": "enlobreguecer",
    "enlobreguecieseis": "enlobreguecer",
    "enlobregueciesen": "enlobreguecer",
    "enlobreguecieses": "enlobreguecer",
    "enlobreguecimos": "enlobreguecer",
    "enlobregueciste": "enlobreguecer",
    "enlobreguecisteis": "enlobreguecer",
    "enlobregueciéramos": "enlobreguecer",
    "enlobregueciéremos": "enlobreguecer",
    "enlobregueciésemos": "enlobreguecer",
    "enlobregueció": "enlobreguecer",
    "enlobreguecéis": "enlobreguecer",
    "enlobreguecí": "enlobreguecer",
    "enlobreguecía": "enlobreguecer",
    "enlobreguecíais": "enlobreguecer",
    "enlobreguecíamos": "enlobreguecer",
    "enlobreguecían": "enlobreguecer",
    "enlobreguecías": "enlobreguecer",
    "enlobreguezca": "enlobreguecer",
    "enlobreguezcamos": "enlobreguecer",
    "enlobreguezcan": "enlobreguecer",
    "enlobreguezcas": "enlobreguecer",
    "enlobreguezco": "enlobreguecer",
    "enlobreguezcáis": "enlobreguecer",
    "enloda": "enlodar",
    "enlodaba": "enlodar",
    "enlodabais": "enlodar",
    "enlodaban": "enlodar",
    "enlodabas": "enlodar",
    "enlodace": "enlodazar",
    "enlodacemos": "enlodazar",
    "enlodacen": "enlodazar",
    "enlodaces": "enlodazar",
    "enlodacé": "enlodazar",
    "enlodacéis": "enlodazar",
    "enlodad": "enlodar",
    "enlodada": "enlodar",
    "enlodadas": "enlodar",
    "enlodado": "enlodar",
    "enlodados": "enlodar",
    "enlodaduras": "enlodadura",
    "enlodamientos": "enlodamiento",
    "enlodamos": "enlodar",
    "enlodan": "enlodar",
    "enlodando": "enlodar",
    "enlodara": "enlodar",
    "enlodarais": "enlodar",
    "enlodaran": "enlodar",
    "enlodaras": "enlodar",
    "enlodare": "enlodar",
    "enlodareis": "enlodar",
    "enlodaremos": "enlodar",
    "enlodaren": "enlodar",
    "enlodares": "enlodar",
    "enlodaron": "enlodar",
    "enlodará": "enlodar",
    "enlodarán": "enlodar",
    "enlodarás": "enlodar",
    "enlodaré": "enlodar",
    "enlodaréis": "enlodar",
    "enlodaría": "enlodar",
    "enlodaríais": "enlodar",
    "enlodaríamos": "enlodar",
    "enlodarían": "enlodar",
    "enlodarías": "enlodar",
    "enlodas": "enlodar",
    "enlodase": "enlodar",
    "enlodaseis": "enlodar",
    "enlodasen": "enlodar",
    "enlodases": "enlodar",
    "enlodaste": "enlodar",
    "enlodasteis": "enlodar",
    "enlodaza": "enlodazar",
    "enlodazaba": "enlodazar",
    "enlodazabais": "enlodazar",
    "enlodazaban": "enlodazar",
    "enlodazabas": "enlodazar",
    "enlodazad": "enlodazar",
    "enlodazada": "enlodazar",
    "enlodazadas": "enlodazar",
    "enlodazado": "enlodazar",
    "enlodazados": "enlodazar",
    "enlodazamos": "enlodazar",
    "enlodazan": "enlodazar",
    "enlodazando": "enlodazar",
    "enlodazara": "enlodazar",
    "enlodazarais": "enlodazar",
    "enlodazaran": "enlodazar",
    "enlodazaras": "enlodazar",
    "enlodazare": "enlodazar",
    "enlodazareis": "enlodazar",
    "enlodazaremos": "enlodazar",
    "enlodazaren": "enlodazar",
    "enlodazares": "enlodazar",
    "enlodazaron": "enlodazar",
    "enlodazará": "enlodazar",
    "enlodazarán": "enlodazar",
    "enlodazarás": "enlodazar",
    "enlodazaré": "enlodazar",
    "enlodazaréis": "enlodazar",
    "enlodazaría": "enlodazar",
    "enlodazaríais": "enlodazar",
    "enlodazaríamos": "enlodazar",
    "enlodazarían": "enlodazar",
    "enlodazarías": "enlodazar",
    "enlodazas": "enlodazar",
    "enlodazase": "enlodazar",
    "enlodazaseis": "enlodazar",
    "enlodazasen": "enlodazar",
    "enlodazases": "enlodazar",
    "enlodazaste": "enlodazar",
    "enlodazasteis": "enlodazar",
    "enlodazo": "enlodazar",
    "enlodazábamos": "enlodazar",
    "enlodazáis": "enlodazar",
    "enlodazáramos": "enlodazar",
    "enlodazáremos": "enlodazar",
    "enlodazásemos": "enlodazar",
    "enlodazó": "enlodazar",
    "enlode": "enlodar",
    "enlodemos": "enlodar",
    "enloden": "enlodar",
    "enlodes": "enlodar",
    "enlodo": "enlodar",
    "enlodábamos": "enlodar",
    "enlodáis": "enlodar",
    "enlodáramos": "enlodar",
    "enlodáremos": "enlodar",
    "enlodásemos": "enlodar",
    "enlodé": "enlodar",
    "enlodéis": "enlodar",
    "enlodó": "enlodar",
    "enloma": "enlomar",
    "enlomaba": "enlomar",
    "enlomabais": "enlomar",
    "enlomaban": "enlomar",
    "enlomabas": "enlomar",
    "enlomad": "enlomar",
    "enlomada": "enlomar",
    "enlomadas": "enlomar",
    "enlomado": "enlomar",
    "enlomados": "enlomar",
    "enlomamos": "enlomar",
    "enloman": "enlomar",
    "enlomando": "enlomar",
    "enlomara": "enlomar",
    "enlomarais": "enlomar",
    "enlomaran": "enlomar",
    "enlomaras": "enlomar",
    "enlomare": "enlomar",
    "enlomareis": "enlomar",
    "enlomaremos": "enlomar",
    "enlomaren": "enlomar",
    "enlomares": "enlomar",
    "enlomaron": "enlomar",
    "enlomará": "enlomar",
    "enlomarán": "enlomar",
    "enlomarás": "enlomar",
    "enlomaré": "enlomar",
    "enlomaréis": "enlomar",
    "enlomaría": "enlomar",
    "enlomaríais": "enlomar",
    "enlomaríamos": "enlomar",
    "enlomarían": "enlomar",
    "enlomarías": "enlomar",
    "enlomas": "enlomar",
    "enlomase": "enlomar",
    "enlomaseis": "enlomar",
    "enlomasen": "enlomar",
    "enlomases": "enlomar",
    "enlomaste": "enlomar",
    "enlomasteis": "enlomar",
    "enlome": "enlomar",
    "enlomemos": "enlomar",
    "enlomen": "enlomar",
    "enlomes": "enlomar",
    "enlomo": "enlomar",
    "enlomábamos": "enlomar",
    "enlomáis": "enlomar",
    "enlomáramos": "enlomar",
    "enlomáremos": "enlomar",
    "enlomásemos": "enlomar",
    "enlomé": "enlomar",
    "enloméis": "enlomar",
    "enlomó": "enlomar",
    "enloquece": "enloquecer",
    "enloqueced": "enloquecer",
    "enloquecedora": "enloquecedor",
    "enloquecedoras": "enloquecedor",
    "enloquecedores": "enloquecedor",
    "enloquecemos": "enloquecer",
    "enloquecen": "enloquecer",
    "enloqueceremos": "enloquecer",
    "enloquecerá": "enloquecer",
    "enloquecerán": "enloquecer",
    "enloquecerás": "enloquecer",
    "enloqueceré": "enloquecer",
    "enloqueceréis": "enloquecer",
    "enloquecería": "enloquecer",
    "enloqueceríais": "enloquecer",
    "enloqueceríamos": "enloquecer",
    "enloquecerían": "enloquecer",
    "enloquecerías": "enloquecer",
    "enloqueces": "enloquecer",
    "enloquecida": "enloquecer",
    "enloquecidas": "enloquecer",
    "enloquecido": "enloquecer",
    "enloquecidos": "enloquecer",
    "enloqueciendo": "enloquecer",
    "enloqueciera": "enloquecer",
    "enloquecierais": "enloquecer",
    "enloquecieran": "enloquecer",
    "enloquecieras": "enloquecer",
    "enloqueciere": "enloquecer",
    "enloqueciereis": "enloquecer",
    "enloquecieren": "enloquecer",
    "enloquecieres": "enloquecer",
    "enloquecieron": "enloquecer",
    "enloqueciese": "enloquecer",
    "enloquecieseis": "enloquecer",
    "enloqueciesen": "enloquecer",
    "enloquecieses": "enloquecer",
    "enloquecimientos": "enloquecimiento",
    "enloquecimos": "enloquecer",
    "enloqueciste": "enloquecer",
    "enloquecisteis": "enloquecer",
    "enloqueciéramos": "enloquecer",
    "enloqueciéremos": "enloquecer",
    "enloqueciésemos": "enloquecer",
    "enloqueció": "enloquecer",
    "enloquecéis": "enloquecer",
    "enloquecí": "enloquecer",
    "enloquecía": "enloquecer",
    "enloquecíais": "enloquecer",
    "enloquecíamos": "enloquecer",
    "enloquecían": "enloquecer",
    "enloquecías": "enloquecer",
    "enloquezca": "enloquecer",
    "enloquezcamos": "enloquecer",
    "enloquezcan": "enloquecer",
    "enloquezcas": "enloquecer",
    "enloquezco": "enloquecer",
    "enloquezcáis": "enloquecer",
    "enlosa": "enlosar",
    "enlosaba": "enlosar",
    "enlosabais": "enlosar",
    "enlosaban": "enlosar",
    "enlosabas": "enlosar",
    "enlosad": "enlosar",
    "enlosada": "enlosar",
    "enlosadas": "enlosar",
    "enlosado": "enlosar",
    "enlosadora": "enlosador",
    "enlosadoras": "enlosador",
    "enlosadores": "enlosador",
    "enlosados": "enlosar",
    "enlosamos": "enlosar",
    "enlosan": "enlosar",
    "enlosando": "enlosar",
    "enlosara": "enlosar",
    "enlosarais": "enlosar",
    "enlosaran": "enlosar",
    "enlosaras": "enlosar",
    "enlosare": "enlosar",
    "enlosareis": "enlosar",
    "enlosaremos": "enlosar",
    "enlosaren": "enlosar",
    "enlosares": "enlosar",
    "enlosaron": "enlosar",
    "enlosará": "enlosar",
    "enlosarán": "enlosar",
    "enlosarás": "enlosar",
    "enlosaré": "enlosar",
    "enlosaréis": "enlosar",
    "enlosaría": "enlosar",
    "enlosaríais": "enlosar",
    "enlosaríamos": "enlosar",
    "enlosarían": "enlosar",
    "enlosarías": "enlosar",
    "enlosas": "enlosar",
    "enlosase": "enlosar",
    "enlosaseis": "enlosar",
    "enlosasen": "enlosar",
    "enlosases": "enlosar",
    "enlosaste": "enlosar",
    "enlosasteis": "enlosar",
    "enlose": "enlosar",
    "enlosemos": "enlosar",
    "enlosen": "enlosar",
    "enloses": "enlosar",
    "enloso": "enlosar",
    "enlosábamos": "enlosar",
    "enlosáis": "enlosar",
    "enlosáramos": "enlosar",
    "enlosáremos": "enlosar",
    "enlosásemos": "enlosar",
    "enlosé": "enlosar",
    "enloséis": "enlosar",
    "enlosó": "enlosar",
    "enluce": "enlucir",
    "enlucen": "enlucir",
    "enluces": "enlucir",
    "enlucid": "enlucir",
    "enlucida": "enlucir",
    "enlucidas": "enlucir",
    "enlucido": "enlucir",
    "enlucidora": "enlucidor",
    "enlucidoras": "enlucidor",
    "enlucidores": "enlucidor",
    "enlucidos": "enlucir",
    "enluciendo": "enlucir",
    "enluciera": "enlucir",
    "enlucierais": "enlucir",
    "enlucieran": "enlucir",
    "enlucieras": "enlucir",
    "enluciere": "enlucir",
    "enluciereis": "enlucir",
    "enlucieren": "enlucir",
    "enlucieres": "enlucir",
    "enlucieron": "enlucir",
    "enluciese": "enlucir",
    "enlucieseis": "enlucir",
    "enluciesen": "enlucir",
    "enlucieses": "enlucir",
    "enlucimos": "enlucir",
    "enluciremos": "enlucir",
    "enlucirá": "enlucir",
    "enlucirán": "enlucir",
    "enlucirás": "enlucir",
    "enluciré": "enlucir",
    "enluciréis": "enlucir",
    "enluciría": "enlucir",
    "enluciríais": "enlucir",
    "enluciríamos": "enlucir",
    "enlucirían": "enlucir",
    "enlucirías": "enlucir",
    "enluciste": "enlucir",
    "enlucisteis": "enlucir",
    "enluciéramos": "enlucir",
    "enluciéremos": "enlucir",
    "enluciésemos": "enlucir",
    "enlució": "enlucir",
    "enlucí": "enlucir",
    "enlucía": "enlucir",
    "enlucíais": "enlucir",
    "enlucíamos": "enlucir",
    "enlucían": "enlucir",
    "enlucías": "enlucir",
    "enlucís": "enlucir",
    "enlustrece": "enlustrecer",
    "enlustreced": "enlustrecer",
    "enlustrecemos": "enlustrecer",
    "enlustrecen": "enlustrecer",
    "enlustreceremos": "enlustrecer",
    "enlustrecerá": "enlustrecer",
    "enlustrecerán": "enlustrecer",
    "enlustrecerás": "enlustrecer",
    "enlustreceré": "enlustrecer",
    "enlustreceréis": "enlustrecer",
    "enlustrecería": "enlustrecer",
    "enlustreceríais": "enlustrecer",
    "enlustreceríamos": "enlustrecer",
    "enlustrecerían": "enlustrecer",
    "enlustrecerías": "enlustrecer",
    "enlustreces": "enlustrecer",
    "enlustrecida": "enlustrecer",
    "enlustrecidas": "enlustrecer",
    "enlustrecido": "enlustrecer",
    "enlustrecidos": "enlustrecer",
    "enlustreciendo": "enlustrecer",
    "enlustreciera": "enlustrecer",
    "enlustrecierais": "enlustrecer",
    "enlustrecieran": "enlustrecer",
    "enlustrecieras": "enlustrecer",
    "enlustreciere": "enlustrecer",
    "enlustreciereis": "enlustrecer",
    "enlustrecieren": "enlustrecer",
    "enlustrecieres": "enlustrecer",
    "enlustrecieron": "enlustrecer",
    "enlustreciese": "enlustrecer",
    "enlustrecieseis": "enlustrecer",
    "enlustreciesen": "enlustrecer",
    "enlustrecieses": "enlustrecer",
    "enlustrecimos": "enlustrecer",
    "enlustreciste": "enlustrecer",
    "enlustrecisteis": "enlustrecer",
    "enlustreciéramos": "enlustrecer",
    "enlustreciéremos": "enlustrecer",
    "enlustreciésemos": "enlustrecer",
    "enlustreció": "enlustrecer",
    "enlustrecéis": "enlustrecer",
    "enlustrecí": "enlustrecer",
    "enlustrecía": "enlustrecer",
    "enlustrecíais": "enlustrecer",
    "enlustrecíamos": "enlustrecer",
    "enlustrecían": "enlustrecer",
    "enlustrecías": "enlustrecer",
    "enlustrezca": "enlustrecer",
    "enlustrezcamos": "enlustrecer",
    "enlustrezcan": "enlustrecer",
    "enlustrezcas": "enlustrecer",
    "enlustrezco": "enlustrecer",
    "enlustrezcáis": "enlustrecer",
    "enluta": "enlutar",
    "enlutaba": "enlutar",
    "enlutabais": "enlutar",
    "enlutaban": "enlutar",
    "enlutabas": "enlutar",
    "enlutad": "enlutar",
    "enlutada": "enlutar",
    "enlutadas": "enlutar",
    "enlutado": "enlutar",
    "enlutados": "enlutar",
    "enlutamientos": "enlutamiento",
    "enlutamos": "enlutar",
    "enlutan": "enlutar",
    "enlutando": "enlutar",
    "enlutara": "enlutar",
    "enlutarais": "enlutar",
    "enlutaran": "enlutar",
    "enlutaras": "enlutar",
    "enlutare": "enlutar",
    "enlutareis": "enlutar",
    "enlutaremos": "enlutar",
    "enlutaren": "enlutar",
    "enlutares": "enlutar",
    "enlutaron": "enlutar",
    "enlutará": "enlutar",
    "enlutarán": "enlutar",
    "enlutarás": "enlutar",
    "enlutaré": "enlutar",
    "enlutaréis": "enlutar",
    "enlutaría": "enlutar",
    "enlutaríais": "enlutar",
    "enlutaríamos": "enlutar",
    "enlutarían": "enlutar",
    "enlutarías": "enlutar",
    "enlutas": "enlutar",
    "enlutase": "enlutar",
    "enlutaseis": "enlutar",
    "enlutasen": "enlutar",
    "enlutases": "enlutar",
    "enlutaste": "enlutar",
    "enlutasteis": "enlutar",
    "enlute": "enlutar",
    "enlutemos": "enlutar",
    "enluten": "enlutar",
    "enlutes": "enlutar",
    "enluto": "enlutar",
    "enlutábamos": "enlutar",
    "enlutáis": "enlutar",
    "enlutáramos": "enlutar",
    "enlutáremos": "enlutar",
    "enlutásemos": "enlutar",
    "enluté": "enlutar",
    "enlutéis": "enlutar",
    "enlutó": "enlutar",
    "enluzca": "enlucir",
    "enluzcamos": "enlucir",
    "enluzcan": "enlucir",
    "enluzcas": "enlucir",
    "enluzco": "enlucir",
    "enluzcáis": "enlucir",
    "enmadera": "enmaderar",
    "enmaderaba": "enmaderar",
    "enmaderabais": "enmaderar",
    "enmaderaban": "enmaderar",
    "enmaderabas": "enmaderar",
    "enmaderad": "enmaderar",
    "enmaderada": "enmaderar",
    "enmaderadas": "enmaderar",
    "enmaderado": "enmaderar",
    "enmaderados": "enmaderar",
    "enmaderamientos": "enmaderamiento",
    "enmaderamos": "enmaderar",
    "enmaderan": "enmaderar",
    "enmaderando": "enmaderar",
    "enmaderara": "enmaderar",
    "enmaderarais": "enmaderar",
    "enmaderaran": "enmaderar",
    "enmaderaras": "enmaderar",
    "enmaderare": "enmaderar",
    "enmaderareis": "enmaderar",
    "enmaderaremos": "enmaderar",
    "enmaderaren": "enmaderar",
    "enmaderares": "enmaderar",
    "enmaderaron": "enmaderar",
    "enmaderará": "enmaderar",
    "enmaderarán": "enmaderar",
    "enmaderarás": "enmaderar",
    "enmaderaré": "enmaderar",
    "enmaderaréis": "enmaderar",
    "enmaderaría": "enmaderar",
    "enmaderaríais": "enmaderar",
    "enmaderaríamos": "enmaderar",
    "enmaderarían": "enmaderar",
    "enmaderarías": "enmaderar",
    "enmaderas": "enmaderar",
    "enmaderase": "enmaderar",
    "enmaderaseis": "enmaderar",
    "enmaderasen": "enmaderar",
    "enmaderases": "enmaderar",
    "enmaderaste": "enmaderar",
    "enmaderasteis": "enmaderar",
    "enmadere": "enmaderar",
    "enmaderemos": "enmaderar",
    "enmaderen": "enmaderar",
    "enmaderes": "enmaderar",
    "enmadero": "enmaderar",
    "enmaderábamos": "enmaderar",
    "enmaderáis": "enmaderar",
    "enmaderáramos": "enmaderar",
    "enmaderáremos": "enmaderar",
    "enmaderásemos": "enmaderar",
    "enmaderé": "enmaderar",
    "enmaderéis": "enmaderar",
    "enmaderó": "enmaderar",
    "enmadra": "enmadrar",
    "enmadraba": "enmadrar",
    "enmadrabais": "enmadrar",
    "enmadraban": "enmadrar",
    "enmadrabas": "enmadrar",
    "enmadrad": "enmadrar",
    "enmadrada": "enmadrar",
    "enmadradas": "enmadrar",
    "enmadrado": "enmadrar",
    "enmadrados": "enmadrar",
    "enmadramos": "enmadrar",
    "enmadran": "enmadrar",
    "enmadrando": "enmadrar",
    "enmadrara": "enmadrar",
    "enmadrarais": "enmadrar",
    "enmadraran": "enmadrar",
    "enmadraras": "enmadrar",
    "enmadrare": "enmadrar",
    "enmadrareis": "enmadrar",
    "enmadraremos": "enmadrar",
    "enmadraren": "enmadrar",
    "enmadrares": "enmadrar",
    "enmadraron": "enmadrar",
    "enmadrará": "enmadrar",
    "enmadrarán": "enmadrar",
    "enmadrarás": "enmadrar",
    "enmadraré": "enmadrar",
    "enmadraréis": "enmadrar",
    "enmadraría": "enmadrar",
    "enmadraríais": "enmadrar",
    "enmadraríamos": "enmadrar",
    "enmadrarían": "enmadrar",
    "enmadrarías": "enmadrar",
    "enmadras": "enmadrar",
    "enmadrase": "enmadrar",
    "enmadraseis": "enmadrar",
    "enmadrasen": "enmadrar",
    "enmadrases": "enmadrar",
    "enmadraste": "enmadrar",
    "enmadrasteis": "enmadrar",
    "enmadre": "enmadrar",
    "enmadremos": "enmadrar",
    "enmadren": "enmadrar",
    "enmadres": "enmadrar",
    "enmadro": "enmadrar",
    "enmadrábamos": "enmadrar",
    "enmadráis": "enmadrar",
    "enmadráramos": "enmadrar",
    "enmadráremos": "enmadrar",
    "enmadrásemos": "enmadrar",
    "enmadré": "enmadrar",
    "enmadréis": "enmadrar",
    "enmadró": "enmadrar",
    "enmagrece": "enmagrecer",
    "enmagreced": "enmagrecer",
    "enmagrecemos": "enmagrecer",
    "enmagrecen": "enmagrecer",
    "enmagreceremos": "enmagrecer",
    "enmagrecerá": "enmagrecer",
    "enmagrecerán": "enmagrecer",
    "enmagrecerás": "enmagrecer",
    "enmagreceré": "enmagrecer",
    "enmagreceréis": "enmagrecer",
    "enmagrecería": "enmagrecer",
    "enmagreceríais": "enmagrecer",
    "enmagreceríamos": "enmagrecer",
    "enmagrecerían": "enmagrecer",
    "enmagrecerías": "enmagrecer",
    "enmagreces": "enmagrecer",
    "enmagrecida": "enmagrecer",
    "enmagrecidas": "enmagrecer",
    "enmagrecido": "enmagrecer",
    "enmagrecidos": "enmagrecer",
    "enmagreciendo": "enmagrecer",
    "enmagreciera": "enmagrecer",
    "enmagrecierais": "enmagrecer",
    "enmagrecieran": "enmagrecer",
    "enmagrecieras": "enmagrecer",
    "enmagreciere": "enmagrecer",
    "enmagreciereis": "enmagrecer",
    "enmagrecieren": "enmagrecer",
    "enmagrecieres": "enmagrecer",
    "enmagrecieron": "enmagrecer",
    "enmagreciese": "enmagrecer",
    "enmagrecieseis": "enmagrecer",
    "enmagreciesen": "enmagrecer",
    "enmagrecieses": "enmagrecer",
    "enmagrecimos": "enmagrecer",
    "enmagreciste": "enmagrecer",
    "enmagrecisteis": "enmagrecer",
    "enmagreciéramos": "enmagrecer",
    "enmagreciéremos": "enmagrecer",
    "enmagreciésemos": "enmagrecer",
    "enmagreció": "enmagrecer",
    "enmagrecéis": "enmagrecer",
    "enmagrecí": "enmagrecer",
    "enmagrecía": "enmagrecer",
    "enmagrecíais": "enmagrecer",
    "enmagrecíamos": "enmagrecer",
    "enmagrecían": "enmagrecer",
    "enmagrecías": "enmagrecer",
    "enmagrezca": "enmagrecer",
    "enmagrezcamos": "enmagrecer",
    "enmagrezcan": "enmagrecer",
    "enmagrezcas": "enmagrecer",
    "enmagrezco": "enmagrecer",
    "enmagrezcáis": "enmagrecer",
    "enmalece": "enmalecer",
    "enmaleced": "enmalecer",
    "enmalecemos": "enmalecer",
    "enmalecen": "enmalecer",
    "enmaleceremos": "enmalecer",
    "enmalecerá": "enmalecer",
    "enmalecerán": "enmalecer",
    "enmalecerás": "enmalecer",
    "enmaleceré": "enmalecer",
    "enmaleceréis": "enmalecer",
    "enmalecería": "enmalecer",
    "enmaleceríais": "enmalecer",
    "enmaleceríamos": "enmalecer",
    "enmalecerían": "enmalecer",
    "enmalecerías": "enmalecer",
    "enmaleces": "enmalecer",
    "enmalecida": "enmalecer",
    "enmalecidas": "enmalecer",
    "enmalecido": "enmalecer",
    "enmalecidos": "enmalecer",
    "enmaleciendo": "enmalecer",
    "enmaleciera": "enmalecer",
    "enmalecierais": "enmalecer",
    "enmalecieran": "enmalecer",
    "enmalecieras": "enmalecer",
    "enmaleciere": "enmalecer",
    "enmaleciereis": "enmalecer",
    "enmalecieren": "enmalecer",
    "enmalecieres": "enmalecer",
    "enmalecieron": "enmalecer",
    "enmaleciese": "enmalecer",
    "enmalecieseis": "enmalecer",
    "enmaleciesen": "enmalecer",
    "enmalecieses": "enmalecer",
    "enmalecimos": "enmalecer",
    "enmaleciste": "enmalecer",
    "enmalecisteis": "enmalecer",
    "enmaleciéramos": "enmalecer",
    "enmaleciéremos": "enmalecer",
    "enmaleciésemos": "enmalecer",
    "enmaleció": "enmalecer",
    "enmalecéis": "enmalecer",
    "enmalecí": "enmalecer",
    "enmalecía": "enmalecer",
    "enmalecíais": "enmalecer",
    "enmalecíamos": "enmalecer",
    "enmalecían": "enmalecer",
    "enmalecías": "enmalecer",
    "enmalezca": "enmalecer",
    "enmalezcamos": "enmalecer",
    "enmalezcan": "enmalecer",
    "enmalezcas": "enmalecer",
    "enmalezco": "enmalecer",
    "enmalezcáis": "enmalecer",
    "enmalla": "enmallar",
    "enmallaba": "enmallar",
    "enmallabais": "enmallar",
    "enmallaban": "enmallar",
    "enmallabas": "enmallar",
    "enmallad": "enmallar",
    "enmallada": "enmallar",
    "enmalladas": "enmallar",
    "enmallado": "enmallar",
    "enmallados": "enmallar",
    "enmallamos": "enmallar",
    "enmallan": "enmallar",
    "enmallando": "enmallar",
    "enmallara": "enmallar",
    "enmallarais": "enmallar",
    "enmallaran": "enmallar",
    "enmallaras": "enmallar",
    "enmallare": "enmallar",
    "enmallareis": "enmallar",
    "enmallaremos": "enmallar",
    "enmallaren": "enmallar",
    "enmallares": "enmallar",
    "enmallaron": "enmallar",
    "enmallará": "enmallar",
    "enmallarán": "enmallar",
    "enmallarás": "enmallar",
    "enmallaré": "enmallar",
    "enmallaréis": "enmallar",
    "enmallaría": "enmallar",
    "enmallaríais": "enmallar",
    "enmallaríamos": "enmallar",
    "enmallarían": "enmallar",
    "enmallarías": "enmallar",
    "enmallas": "enmallar",
    "enmallase": "enmallar",
    "enmallaseis": "enmallar",
    "enmallasen": "enmallar",
    "enmallases": "enmallar",
    "enmallaste": "enmallar",
    "enmallasteis": "enmallar",
    "enmalle": "enmallar",
    "enmallemos": "enmallar",
    "enmallen": "enmallar",
    "enmalles": "enmalle",
    "enmallo": "enmallar",
    "enmallábamos": "enmallar",
    "enmalláis": "enmallar",
    "enmalláramos": "enmallar",
    "enmalláremos": "enmallar",
    "enmallásemos": "enmallar",
    "enmallé": "enmallar",
    "enmalléis": "enmallar",
    "enmalló": "enmallar",
    "enmanga": "enmangar",
    "enmangaba": "enmangar",
    "enmangabais": "enmangar",
    "enmangaban": "enmangar",
    "enmangabas": "enmangar",
    "enmangad": "enmangar",
    "enmangada": "enmangar",
    "enmangadas": "enmangar",
    "enmangado": "enmangar",
    "enmangados": "enmangar",
    "enmangamos": "enmangar",
    "enmangan": "enmangar",
    "enmangando": "enmangar",
    "enmangara": "enmangar",
    "enmangarais": "enmangar",
    "enmangaran": "enmangar",
    "enmangaras": "enmangar",
    "enmangare": "enmangar",
    "enmangareis": "enmangar",
    "enmangaremos": "enmangar",
    "enmangaren": "enmangar",
    "enmangares": "enmangar",
    "enmangaron": "enmangar",
    "enmangará": "enmangar",
    "enmangarán": "enmangar",
    "enmangarás": "enmangar",
    "enmangaré": "enmangar",
    "enmangaréis": "enmangar",
    "enmangaría": "enmangar",
    "enmangaríais": "enmangar",
    "enmangaríamos": "enmangar",
    "enmangarían": "enmangar",
    "enmangarías": "enmangar",
    "enmangas": "enmangar",
    "enmangase": "enmangar",
    "enmangaseis": "enmangar",
    "enmangasen": "enmangar",
    "enmangases": "enmangar",
    "enmangaste": "enmangar",
    "enmangasteis": "enmangar",
    "enmango": "enmangar",
    "enmangue": "enmangar",
    "enmanguemos": "enmangar",
    "enmanguen": "enmangar",
    "enmangues": "enmangar",
    "enmangué": "enmangar",
    "enmanguéis": "enmangar",
    "enmangábamos": "enmangar",
    "enmangáis": "enmangar",
    "enmangáramos": "enmangar",
    "enmangáremos": "enmangar",
    "enmangásemos": "enmangar",
    "enmangó": "enmangar",
    "enmanta": "enmantar",
    "enmantaba": "enmantar",
    "enmantabais": "enmantar",
    "enmantaban": "enmantar",
    "enmantabas": "enmantar",
    "enmantad": "enmantar",
    "enmantada": "enmantar",
    "enmantadas": "enmantar",
    "enmantado": "enmantar",
    "enmantados": "enmantar",
    "enmantamos": "enmantar",
    "enmantan": "enmantar",
    "enmantando": "enmantar",
    "enmantara": "enmantar",
    "enmantarais": "enmantar",
    "enmantaran": "enmantar",
    "enmantaras": "enmantar",
    "enmantare": "enmantar",
    "enmantareis": "enmantar",
    "enmantaremos": "enmantar",
    "enmantaren": "enmantar",
    "enmantares": "enmantar",
    "enmantaron": "enmantar",
    "enmantará": "enmantar",
    "enmantarán": "enmantar",
    "enmantarás": "enmantar",
    "enmantaré": "enmantar",
    "enmantaréis": "enmantar",
    "enmantaría": "enmantar",
    "enmantaríais": "enmantar",
    "enmantaríamos": "enmantar",
    "enmantarían": "enmantar",
    "enmantarías": "enmantar",
    "enmantas": "enmantar",
    "enmantase": "enmantar",
    "enmantaseis": "enmantar",
    "enmantasen": "enmantar",
    "enmantases": "enmantar",
    "enmantaste": "enmantar",
    "enmantasteis": "enmantar",
    "enmante": "enmantar",
    "enmantemos": "enmantar",
    "enmanten": "enmantar",
    "enmantes": "enmantar",
    "enmanto": "enmantar",
    "enmantábamos": "enmantar",
    "enmantáis": "enmantar",
    "enmantáramos": "enmantar",
    "enmantáremos": "enmantar",
    "enmantásemos": "enmantar",
    "enmanté": "enmantar",
    "enmantéis": "enmantar",
    "enmantó": "enmantar",
    "enmara": "enmarar",
    "enmaraba": "enmarar",
    "enmarabais": "enmarar",
    "enmaraban": "enmarar",
    "enmarabas": "enmarar",
    "enmarad": "enmarar",
    "enmarada": "enmarar",
    "enmaradas": "enmarar",
    "enmarado": "enmarar",
    "enmarados": "enmarar",
    "enmaramos": "enmarar",
    "enmaran": "enmarar",
    "enmarando": "enmarar",
    "enmarara": "enmarar",
    "enmararais": "enmarar",
    "enmararan": "enmarar",
    "enmararas": "enmarar",
    "enmarare": "enmarar",
    "enmarareis": "enmarar",
    "enmararemos": "enmarar",
    "enmararen": "enmarar",
    "enmarares": "enmarar",
    "enmararon": "enmarar",
    "enmarará": "enmarar",
    "enmararán": "enmarar",
    "enmararás": "enmarar",
    "enmararé": "enmarar",
    "enmararéis": "enmarar",
    "enmararía": "enmarar",
    "enmararíais": "enmarar",
    "enmararíamos": "enmarar",
    "enmararían": "enmarar",
    "enmararías": "enmarar",
    "enmaras": "enmarar",
    "enmarase": "enmarar",
    "enmaraseis": "enmarar",
    "enmarasen": "enmarar",
    "enmarases": "enmarar",
    "enmaraste": "enmarar",
    "enmarasteis": "enmarar",
    "enmaraña": "enmarañar",
    "enmarañaba": "enmarañar",
    "enmarañabais": "enmarañar",
    "enmarañaban": "enmarañar",
    "enmarañabas": "enmarañar",
    "enmarañad": "enmarañar",
    "enmarañada": "enmarañar",
    "enmarañadas": "enmarañar",
    "enmarañado": "enmarañar",
    "enmarañadora": "enmarañador",
    "enmarañadoras": "enmarañador",
    "enmarañadores": "enmarañador",
    "enmarañados": "enmarañar",
    "enmarañamientos": "enmarañamiento",
    "enmarañamos": "enmarañar",
    "enmarañan": "enmarañar",
    "enmarañando": "enmarañar",
    "enmarañara": "enmarañar",
    "enmarañarais": "enmarañar",
    "enmarañaran": "enmarañar",
    "enmarañaras": "enmarañar",
    "enmarañare": "enmarañar",
    "enmarañareis": "enmarañar",
    "enmarañaremos": "enmarañar",
    "enmarañaren": "enmarañar",
    "enmarañares": "enmarañar",
    "enmarañaron": "enmarañar",
    "enmarañará": "enmarañar",
    "enmarañarán": "enmarañar",
    "enmarañarás": "enmarañar",
    "enmarañaré": "enmarañar",
    "enmarañaréis": "enmarañar",
    "enmarañaría": "enmarañar",
    "enmarañaríais": "enmarañar",
    "enmarañaríamos": "enmarañar",
    "enmarañarían": "enmarañar",
    "enmarañarías": "enmarañar",
    "enmarañas": "enmarañar",
    "enmarañase": "enmarañar",
    "enmarañaseis": "enmarañar",
    "enmarañasen": "enmarañar",
    "enmarañases": "enmarañar",
    "enmarañaste": "enmarañar",
    "enmarañasteis": "enmarañar",
    "enmarañe": "enmarañar",
    "enmarañemos": "enmarañar",
    "enmarañen": "enmarañar",
    "enmarañes": "enmarañar",
    "enmaraño": "enmarañar",
    "enmarañábamos": "enmarañar",
    "enmarañáis": "enmarañar",
    "enmarañáramos": "enmarañar",
    "enmarañáremos": "enmarañar",
    "enmarañásemos": "enmarañar",
    "enmarañé": "enmarañar",
    "enmarañéis": "enmarañar",
    "enmarañó": "enmarañar",
    "enmarca": "enmarcar",
    "enmarcaba": "enmarcar",
    "enmarcabais": "enmarcar",
    "enmarcaban": "enmarcar",
    "enmarcabas": "enmarcar",
    "enmarcad": "enmarcar",
    "enmarcada": "enmarcar",
    "enmarcadas": "enmarcar",
    "enmarcado": "enmarcar",
    "enmarcados": "enmarcar",
    "enmarcamos": "enmarcar",
    "enmarcan": "enmarcar",
    "enmarcando": "enmarcar",
    "enmarcantes": "enmarcante",
    "enmarcara": "enmarcar",
    "enmarcarais": "enmarcar",
    "enmarcaran": "enmarcar",
    "enmarcaras": "enmarcar",
    "enmarcare": "enmarcar",
    "enmarcareis": "enmarcar",
    "enmarcaremos": "enmarcar",
    "enmarcaren": "enmarcar",
    "enmarcares": "enmarcar",
    "enmarcaron": "enmarcar",
    "enmarcará": "enmarcar",
    "enmarcarán": "enmarcar",
    "enmarcarás": "enmarcar",
    "enmarcaré": "enmarcar",
    "enmarcaréis": "enmarcar",
    "enmarcaría": "enmarcar",
    "enmarcaríais": "enmarcar",
    "enmarcaríamos": "enmarcar",
    "enmarcarían": "enmarcar",
    "enmarcarías": "enmarcar",
    "enmarcas": "enmarcar",
    "enmarcase": "enmarcar",
    "enmarcaseis": "enmarcar",
    "enmarcasen": "enmarcar",
    "enmarcases": "enmarcar",
    "enmarcaste": "enmarcar",
    "enmarcasteis": "enmarcar",
    "enmarco": "enmarcar",
    "enmarcábamos": "enmarcar",
    "enmarcáis": "enmarcar",
    "enmarcáramos": "enmarcar",
    "enmarcáremos": "enmarcar",
    "enmarcásemos": "enmarcar",
    "enmarcó": "enmarcar",
    "enmare": "enmarar",
    "enmaremos": "enmarar",
    "enmaren": "enmarar",
    "enmares": "enmarar",
    "enmarida": "enmaridar",
    "enmaridaba": "enmaridar",
    "enmaridabais": "enmaridar",
    "enmaridaban": "enmaridar",
    "enmaridabas": "enmaridar",
    "enmaridad": "enmaridar",
    "enmaridada": "enmaridar",
    "enmaridadas": "enmaridar",
    "enmaridado": "enmaridar",
    "enmaridados": "enmaridar",
    "enmaridamos": "enmaridar",
    "enmaridan": "enmaridar",
    "enmaridando": "enmaridar",
    "enmaridara": "enmaridar",
    "enmaridarais": "enmaridar",
    "enmaridaran": "enmaridar",
    "enmaridaras": "enmaridar",
    "enmaridare": "enmaridar",
    "enmaridareis": "enmaridar",
    "enmaridaremos": "enmaridar",
    "enmaridaren": "enmaridar",
    "enmaridares": "enmaridar",
    "enmaridaron": "enmaridar",
    "enmaridará": "enmaridar",
    "enmaridarán": "enmaridar",
    "enmaridarás": "enmaridar",
    "enmaridaré": "enmaridar",
    "enmaridaréis": "enmaridar",
    "enmaridaría": "enmaridar",
    "enmaridaríais": "enmaridar",
    "enmaridaríamos": "enmaridar",
    "enmaridarían": "enmaridar",
    "enmaridarías": "enmaridar",
    "enmaridas": "enmaridar",
    "enmaridase": "enmaridar",
    "enmaridaseis": "enmaridar",
    "enmaridasen": "enmaridar",
    "enmaridases": "enmaridar",
    "enmaridaste": "enmaridar",
    "enmaridasteis": "enmaridar",
    "enmaride": "enmaridar",
    "enmaridemos": "enmaridar",
    "enmariden": "enmaridar",
    "enmarides": "enmaridar",
    "enmarido": "enmaridar",
    "enmaridábamos": "enmaridar",
    "enmaridáis": "enmaridar",
    "enmaridáramos": "enmaridar",
    "enmaridáremos": "enmaridar",
    "enmaridásemos": "enmaridar",
    "enmaridé": "enmaridar",
    "enmaridéis": "enmaridar",
    "enmaridó": "enmaridar",
    "enmarillece": "enmarillecer",
    "enmarilleced": "enmarillecer",
    "enmarillecemos": "enmarillecer",
    "enmarillecen": "enmarillecer",
    "enmarilleceremos": "enmarillecer",
    "enmarillecerá": "enmarillecer",
    "enmarillecerán": "enmarillecer",
    "enmarillecerás": "enmarillecer",
    "enmarilleceré": "enmarillecer",
    "enmarilleceréis": "enmarillecer",
    "enmarillecería": "enmarillecer",
    "enmarilleceríais": "enmarillecer",
    "enmarilleceríamos": "enmarillecer",
    "enmarillecerían": "enmarillecer",
    "enmarillecerías": "enmarillecer",
    "enmarilleces": "enmarillecer",
    "enmarillecida": "enmarillecer",
    "enmarillecidas": "enmarillecer",
    "enmarillecido": "enmarillecer",
    "enmarillecidos": "enmarillecer",
    "enmarilleciendo": "enmarillecer",
    "enmarilleciera": "enmarillecer",
    "enmarillecierais": "enmarillecer",
    "enmarillecieran": "enmarillecer",
    "enmarillecieras": "enmarillecer",
    "enmarilleciere": "enmarillecer",
    "enmarilleciereis": "enmarillecer",
    "enmarillecieren": "enmarillecer",
    "enmarillecieres": "enmarillecer",
    "enmarillecieron": "enmarillecer",
    "enmarilleciese": "enmarillecer",
    "enmarillecieseis": "enmarillecer",
    "enmarilleciesen": "enmarillecer",
    "enmarillecieses": "enmarillecer",
    "enmarillecimos": "enmarillecer",
    "enmarilleciste": "enmarillecer",
    "enmarillecisteis": "enmarillecer",
    "enmarilleciéramos": "enmarillecer",
    "enmarilleciéremos": "enmarillecer",
    "enmarilleciésemos": "enmarillecer",
    "enmarilleció": "enmarillecer",
    "enmarillecéis": "enmarillecer",
    "enmarillecí": "enmarillecer",
    "enmarillecía": "enmarillecer",
    "enmarillecíais": "enmarillecer",
    "enmarillecíamos": "enmarillecer",
    "enmarillecían": "enmarillecer",
    "enmarillecías": "enmarillecer",
    "enmarillezca": "enmarillecer",
    "enmarillezcamos": "enmarillecer",
    "enmarillezcan": "enmarillecer",
    "enmarillezcas": "enmarillecer",
    "enmarillezco": "enmarillecer",
    "enmarillezcáis": "enmarillecer",
    "enmaro": "enmarar",
    "enmarque": "enmarcar",
    "enmarquemos": "enmarcar",
    "enmarquen": "enmarcar",
    "enmarques": "enmarcar",
    "enmarqué": "enmarcar",
    "enmarquéis": "enmarcar",
    "enmarábamos": "enmarar",
    "enmaráis": "enmarar",
    "enmaráramos": "enmarar",
    "enmaráremos": "enmarar",
    "enmarásemos": "enmarar",
    "enmaré": "enmarar",
    "enmaréis": "enmarar",
    "enmaró": "enmarar",
    "enmascara": "enmascarar",
    "enmascaraba": "enmascarar",
    "enmascarabais": "enmascarar",
    "enmascaraban": "enmascarar",
    "enmascarabas": "enmascarar",
    "enmascarad": "enmascarar",
    "enmascarada": "enmascarar",
    "enmascaradas": "enmascarar",
    "enmascarado": "enmascarar",
    "enmascarados": "enmascarar",
    "enmascaramientos": "enmascaramiento",
    "enmascaramos": "enmascarar",
    "enmascaran": "enmascarar",
    "enmascarando": "enmascarar",
    "enmascarara": "enmascarar",
    "enmascararais": "enmascarar",
    "enmascararan": "enmascarar",
    "enmascararas": "enmascarar",
    "enmascarare": "enmascarar",
    "enmascarareis": "enmascarar",
    "enmascararemos": "enmascarar",
    "enmascararen": "enmascarar",
    "enmascarares": "enmascarar",
    "enmascararon": "enmascarar",
    "enmascarará": "enmascarar",
    "enmascararán": "enmascarar",
    "enmascararás": "enmascarar",
    "enmascararé": "enmascarar",
    "enmascararéis": "enmascarar",
    "enmascararía": "enmascarar",
    "enmascararíais": "enmascarar",
    "enmascararíamos": "enmascarar",
    "enmascararían": "enmascarar",
    "enmascararías": "enmascarar",
    "enmascaras": "enmascarar",
    "enmascarase": "enmascarar",
    "enmascaraseis": "enmascarar",
    "enmascarasen": "enmascarar",
    "enmascarases": "enmascarar",
    "enmascaraste": "enmascarar",
    "enmascarasteis": "enmascarar",
    "enmascare": "enmascarar",
    "enmascaremos": "enmascarar",
    "enmascaren": "enmascarar",
    "enmascares": "enmascarar",
    "enmascaro": "enmascarar",
    "enmascarábamos": "enmascarar",
    "enmascaráis": "enmascarar",
    "enmascaráramos": "enmascarar",
    "enmascaráremos": "enmascarar",
    "enmascarásemos": "enmascarar",
    "enmascaré": "enmascarar",
    "enmascaréis": "enmascarar",
    "enmascaró": "enmascarar",
    "enmasilla": "enmasillar",
    "enmasillaba": "enmasillar",
    "enmasillabais": "enmasillar",
    "enmasillaban": "enmasillar",
    "enmasillabas": "enmasillar",
    "enmasillad": "enmasillar",
    "enmasillada": "enmasillar",
    "enmasilladas": "enmasillar",
    "enmasillado": "enmasillar",
    "enmasillados": "enmasillar",
    "enmasillamos": "enmasillar",
    "enmasillan": "enmasillar",
    "enmasillando": "enmasillar",
    "enmasillara": "enmasillar",
    "enmasillarais": "enmasillar",
    "enmasillaran": "enmasillar",
    "enmasillaras": "enmasillar",
    "enmasillare": "enmasillar",
    "enmasillareis": "enmasillar",
    "enmasillaremos": "enmasillar",
    "enmasillaren": "enmasillar",
    "enmasillares": "enmasillar",
    "enmasillaron": "enmasillar",
    "enmasillará": "enmasillar",
    "enmasillarán": "enmasillar",
    "enmasillarás": "enmasillar",
    "enmasillaré": "enmasillar",
    "enmasillaréis": "enmasillar",
    "enmasillaría": "enmasillar",
    "enmasillaríais": "enmasillar",
    "enmasillaríamos": "enmasillar",
    "enmasillarían": "enmasillar",
    "enmasillarías": "enmasillar",
    "enmasillas": "enmasillar",
    "enmasillase": "enmasillar",
    "enmasillaseis": "enmasillar",
    "enmasillasen": "enmasillar",
    "enmasillases": "enmasillar",
    "enmasillaste": "enmasillar",
    "enmasillasteis": "enmasillar",
    "enmasille": "enmasillar",
    "enmasillemos": "enmasillar",
    "enmasillen": "enmasillar",
    "enmasilles": "enmasillar",
    "enmasillo": "enmasillar",
    "enmasillábamos": "enmasillar",
    "enmasilláis": "enmasillar",
    "enmasilláramos": "enmasillar",
    "enmasilláremos": "enmasillar",
    "enmasillásemos": "enmasillar",
    "enmasillé": "enmasillar",
    "enmasilléis": "enmasillar",
    "enmasilló": "enmasillar",
    "enmelaba": "enmelar",
    "enmelabais": "enmelar",
    "enmelaban": "enmelar",
    "enmelabas": "enmelar",
    "enmelad": "enmelar",
    "enmelada": "enmelar",
    "enmeladas": "enmelar",
    "enmelado": "enmelar",
    "enmelados": "enmelar",
    "enmelamos": "enmelar",
    "enmelando": "enmelar",
    "enmelara": "enmelar",
    "enmelarais": "enmelar",
    "enmelaran": "enmelar",
    "enmelaras": "enmelar",
    "enmelare": "enmelar",
    "enmelareis": "enmelar",
    "enmelaremos": "enmelar",
    "enmelaren": "enmelar",
    "enmelares": "enmelar",
    "enmelaron": "enmelar",
    "enmelará": "enmelar",
    "enmelarán": "enmelar",
    "enmelarás": "enmelar",
    "enmelaré": "enmelar",
    "enmelaréis": "enmelar",
    "enmelaría": "enmelar",
    "enmelaríais": "enmelar",
    "enmelaríamos": "enmelar",
    "enmelarían": "enmelar",
    "enmelarías": "enmelar",
    "enmelase": "enmelar",
    "enmelaseis": "enmelar",
    "enmelasen": "enmelar",
    "enmelases": "enmelar",
    "enmelaste": "enmelar",
    "enmelasteis": "enmelar",
    "enmelemos": "enmelar",
    "enmelábamos": "enmelar",
    "enmeláis": "enmelar",
    "enmeláramos": "enmelar",
    "enmeláremos": "enmelar",
    "enmelásemos": "enmelar",
    "enmelé": "enmelar",
    "enmeléis": "enmelar",
    "enmeló": "enmelar",
    "enmendaba": "enmendar",
    "enmendabais": "enmendar",
    "enmendaban": "enmendar",
    "enmendabas": "enmendar",
    "enmendaciones": "enmendación",
    "enmendad": "enmendar",
    "enmendada": "enmendar",
    "enmendadas": "enmendar",
    "enmendado": "enmendar",
    "enmendadora": "enmendador",
    "enmendadoras": "enmendador",
    "enmendadores": "enmendador",
    "enmendados": "enmendar",
    "enmendaduras": "enmendadura",
    "enmendamos": "enmendar",
    "enmendando": "enmendar",
    "enmendara": "enmendar",
    "enmendarais": "enmendar",
    "enmendaran": "enmendar",
    "enmendaras": "enmendar",
    "enmendare": "enmendar",
    "enmendareis": "enmendar",
    "enmendaremos": "enmendar",
    "enmendaren": "enmendar",
    "enmendares": "enmendar",
    "enmendaron": "enmendar",
    "enmendará": "enmendar",
    "enmendarán": "enmendar",
    "enmendarás": "enmendar",
    "enmendaré": "enmendar",
    "enmendaréis": "enmendar",
    "enmendaría": "enmendar",
    "enmendaríais": "enmendar",
    "enmendaríamos": "enmendar",
    "enmendarían": "enmendar",
    "enmendarías": "enmendar",
    "enmendase": "enmendar",
    "enmendaseis": "enmendar",
    "enmendasen": "enmendar",
    "enmendases": "enmendar",
    "enmendaste": "enmendar",
    "enmendasteis": "enmendar",
    "enmendemos": "enmendar",
    "enmendábamos": "enmendar",
    "enmendáis": "enmendar",
    "enmendáramos": "enmendar",
    "enmendáremos": "enmendar",
    "enmendásemos": "enmendar",
    "enmendé": "enmendar",
    "enmendéis": "enmendar",
    "enmendó": "enmendar",
    "enmiela": "enmelar",
    "enmielan": "enmelar",
    "enmielas": "enmelar",
    "enmiele": "enmelar",
    "enmielen": "enmelar",
    "enmieles": "enmelar",
    "enmielo": "enmelar",
    "enmienda": "enmendar",
    "enmiendan": "enmendar",
    "enmiendas": "enmienda",
    "enmiende": "enmendar",
    "enmienden": "enmendar",
    "enmiendes": "enmendar",
    "enmiendo": "enmendar",
    "enmohece": "enmohecer",
    "enmoheced": "enmohecer",
    "enmohecemos": "enmohecer",
    "enmohecen": "enmohecer",
    "enmoheceremos": "enmohecer",
    "enmohecerá": "enmohecer",
    "enmohecerán": "enmohecer",
    "enmohecerás": "enmohecer",
    "enmoheceré": "enmohecer",
    "enmoheceréis": "enmohecer",
    "enmohecería": "enmohecer",
    "enmoheceríais": "enmohecer",
    "enmoheceríamos": "enmohecer",
    "enmohecerían": "enmohecer",
    "enmohecerías": "enmohecer",
    "enmoheces": "enmohecer",
    "enmohecida": "enmohecer",
    "enmohecidas": "enmohecer",
    "enmohecido": "enmohecer",
    "enmohecidos": "enmohecer",
    "enmoheciendo": "enmohecer",
    "enmoheciera": "enmohecer",
    "enmohecierais": "enmohecer",
    "enmohecieran": "enmohecer",
    "enmohecieras": "enmohecer",
    "enmoheciere": "enmohecer",
    "enmoheciereis": "enmohecer",
    "enmohecieren": "enmohecer",
    "enmohecieres": "enmohecer",
    "enmohecieron": "enmohecer",
    "enmoheciese": "enmohecer",
    "enmohecieseis": "enmohecer",
    "enmoheciesen": "enmohecer",
    "enmohecieses": "enmohecer",
    "enmohecimientos": "enmohecimiento",
    "enmohecimos": "enmohecer",
    "enmoheciste": "enmohecer",
    "enmohecisteis": "enmohecer",
    "enmoheciéramos": "enmohecer",
    "enmoheciéremos": "enmohecer",
    "enmoheciésemos": "enmohecer",
    "enmoheció": "enmohecer",
    "enmohecéis": "enmohecer",
    "enmohecí": "enmohecer",
    "enmohecía": "enmohecer",
    "enmohecíais": "enmohecer",
    "enmohecíamos": "enmohecer",
    "enmohecían": "enmohecer",
    "enmohecías": "enmohecer",
    "enmohezca": "enmohecer",
    "enmohezcamos": "enmohecer",
    "enmohezcan": "enmohecer",
    "enmohezcas": "enmohecer",
    "enmohezco": "enmohecer",
    "enmohezcáis": "enmohecer",
    "enmoldadores": "enmoldador",
    "enmollece": "enmollecer",
    "enmolleced": "enmollecer",
    "enmollecemos": "enmollecer",
    "enmollecen": "enmollecer",
    "enmolleceremos": "enmollecer",
    "enmollecerá": "enmollecer",
    "enmollecerán": "enmollecer",
    "enmollecerás": "enmollecer",
    "enmolleceré": "enmollecer",
    "enmolleceréis": "enmollecer",
    "enmollecería": "enmollecer",
    "enmolleceríais": "enmollecer",
    "enmolleceríamos": "enmollecer",
    "enmollecerían": "enmollecer",
    "enmollecerías": "enmollecer",
    "enmolleces": "enmollecer",
    "enmollecida": "enmollecer",
    "enmollecidas": "enmollecer",
    "enmollecido": "enmollecer",
    "enmollecidos": "enmollecer",
    "enmolleciendo": "enmollecer",
    "enmolleciera": "enmollecer",
    "enmollecierais": "enmollecer",
    "enmollecieran": "enmollecer",
    "enmollecieras": "enmollecer",
    "enmolleciere": "enmollecer",
    "enmolleciereis": "enmollecer",
    "enmollecieren": "enmollecer",
    "enmollecieres": "enmollecer",
    "enmollecieron": "enmollecer",
    "enmolleciese": "enmollecer",
    "enmollecieseis": "enmollecer",
    "enmolleciesen": "enmollecer",
    "enmollecieses": "enmollecer",
    "enmollecimos": "enmollecer",
    "enmolleciste": "enmollecer",
    "enmollecisteis": "enmollecer",
    "enmolleciéramos": "enmollecer",
    "enmolleciéremos": "enmollecer",
    "enmolleciésemos": "enmollecer",
    "enmolleció": "enmollecer",
    "enmollecéis": "enmollecer",
    "enmollecí": "enmollecer",
    "enmollecía": "enmollecer",
    "enmollecíais": "enmollecer",
    "enmollecíamos": "enmollecer",
    "enmollecían": "enmollecer",
    "enmollecías": "enmollecer",
    "enmollezca": "enmollecer",
    "enmollezcamos": "enmollecer",
    "enmollezcan": "enmollecer",
    "enmollezcas": "enmollecer",
    "enmollezco": "enmollecer",
    "enmollezcáis": "enmollecer",
    "enmonda": "enmondar",
    "enmondaba": "enmondar",
    "enmondabais": "enmondar",
    "enmondaban": "enmondar",
    "enmondabas": "enmondar",
    "enmondad": "enmondar",
    "enmondada": "enmondar",
    "enmondadas": "enmondar",
    "enmondado": "enmondar",
    "enmondados": "enmondar",
    "enmondamos": "enmondar",
    "enmondan": "enmondar",
    "enmondando": "enmondar",
    "enmondara": "enmondar",
    "enmondarais": "enmondar",
    "enmondaran": "enmondar",
    "enmondaras": "enmondar",
    "enmondare": "enmondar",
    "enmondareis": "enmondar",
    "enmondaremos": "enmondar",
    "enmondaren": "enmondar",
    "enmondares": "enmondar",
    "enmondaron": "enmondar",
    "enmondará": "enmondar",
    "enmondarán": "enmondar",
    "enmondarás": "enmondar",
    "enmondaré": "enmondar",
    "enmondaréis": "enmondar",
    "enmondaría": "enmondar",
    "enmondaríais": "enmondar",
    "enmondaríamos": "enmondar",
    "enmondarían": "enmondar",
    "enmondarías": "enmondar",
    "enmondas": "enmondar",
    "enmondase": "enmondar",
    "enmondaseis": "enmondar",
    "enmondasen": "enmondar",
    "enmondases": "enmondar",
    "enmondaste": "enmondar",
    "enmondasteis": "enmondar",
    "enmonde": "enmondar",
    "enmondemos": "enmondar",
    "enmonden": "enmondar",
    "enmondes": "enmondar",
    "enmondo": "enmondar",
    "enmondábamos": "enmondar",
    "enmondáis": "enmondar",
    "enmondáramos": "enmondar",
    "enmondáremos": "enmondar",
    "enmondásemos": "enmondar",
    "enmondé": "enmondar",
    "enmondéis": "enmondar",
    "enmondó": "enmondar",
    "enmudece": "enmudecer",
    "enmudeced": "enmudecer",
    "enmudecemos": "enmudecer",
    "enmudecen": "enmudecer",
    "enmudeceremos": "enmudecer",
    "enmudecerá": "enmudecer",
    "enmudecerán": "enmudecer",
    "enmudecerás": "enmudecer",
    "enmudeceré": "enmudecer",
    "enmudeceréis": "enmudecer",
    "enmudecería": "enmudecer",
    "enmudeceríais": "enmudecer",
    "enmudeceríamos": "enmudecer",
    "enmudecerían": "enmudecer",
    "enmudecerías": "enmudecer",
    "enmudeces": "enmudecer",
    "enmudecida": "enmudecer",
    "enmudecidas": "enmudecer",
    "enmudecido": "enmudecer",
    "enmudecidos": "enmudecer",
    "enmudeciendo": "enmudecer",
    "enmudeciera": "enmudecer",
    "enmudecierais": "enmudecer",
    "enmudecieran": "enmudecer",
    "enmudecieras": "enmudecer",
    "enmudeciere": "enmudecer",
    "enmudeciereis": "enmudecer",
    "enmudecieren": "enmudecer",
    "enmudecieres": "enmudecer",
    "enmudecieron": "enmudecer",
    "enmudeciese": "enmudecer",
    "enmudecieseis": "enmudecer",
    "enmudeciesen": "enmudecer",
    "enmudecieses": "enmudecer",
    "enmudecimientos": "enmudecimiento",
    "enmudecimos": "enmudecer",
    "enmudeciste": "enmudecer",
    "enmudecisteis": "enmudecer",
    "enmudeciéramos": "enmudecer",
    "enmudeciéremos": "enmudecer",
    "enmudeciésemos": "enmudecer",
    "enmudeció": "enmudecer",
    "enmudecéis": "enmudecer",
    "enmudecí": "enmudecer",
    "enmudecía": "enmudecer",
    "enmudecíais": "enmudecer",
    "enmudecíamos": "enmudecer",
    "enmudecían": "enmudecer",
    "enmudecías": "enmudecer",
    "enmudezca": "enmudecer",
    "enmudezcamos": "enmudecer",
    "enmudezcan": "enmudecer",
    "enmudezcas": "enmudecer",
    "enmudezco": "enmudecer",
    "enmudezcáis": "enmudecer",
    "enmugrece": "enmugrecer",
    "enmugreced": "enmugrecer",
    "enmugrecemos": "enmugrecer",
    "enmugrecen": "enmugrecer",
    "enmugreceremos": "enmugrecer",
    "enmugrecerá": "enmugrecer",
    "enmugrecerán": "enmugrecer",
    "enmugrecerás": "enmugrecer",
    "enmugreceré": "enmugrecer",
    "enmugreceréis": "enmugrecer",
    "enmugrecería": "enmugrecer",
    "enmugreceríais": "enmugrecer",
    "enmugreceríamos": "enmugrecer",
    "enmugrecerían": "enmugrecer",
    "enmugrecerías": "enmugrecer",
    "enmugreces": "enmugrecer",
    "enmugrecida": "enmugrecer",
    "enmugrecidas": "enmugrecer",
    "enmugrecido": "enmugrecer",
    "enmugrecidos": "enmugrecer",
    "enmugreciendo": "enmugrecer",
    "enmugreciera": "enmugrecer",
    "enmugrecierais": "enmugrecer",
    "enmugrecieran": "enmugrecer",
    "enmugrecieras": "enmugrecer",
    "enmugreciere": "enmugrecer",
    "enmugreciereis": "enmugrecer",
    "enmugrecieren": "enmugrecer",
    "enmugrecieres": "enmugrecer",
    "enmugrecieron": "enmugrecer",
    "enmugreciese": "enmugrecer",
    "enmugrecieseis": "enmugrecer",
    "enmugreciesen": "enmugrecer",
    "enmugrecieses": "enmugrecer",
    "enmugrecimos": "enmugrecer",
    "enmugreciste": "enmugrecer",
    "enmugrecisteis": "enmugrecer",
    "enmugreciéramos": "enmugrecer",
    "enmugreciéremos": "enmugrecer",
    "enmugreciésemos": "enmugrecer",
    "enmugreció": "enmugrecer",
    "enmugrecéis": "enmugrecer",
    "enmugrecí": "enmugrecer",
    "enmugrecía": "enmugrecer",
    "enmugrecíais": "enmugrecer",
    "enmugrecíamos": "enmugrecer",
    "enmugrecían": "enmugrecer",
    "enmugrecías": "enmugrecer",
    "enmugrezca": "enmugrecer",
    "enmugrezcamos": "enmugrecer",
    "enmugrezcan": "enmugrecer",
    "enmugrezcas": "enmugrecer",
    "enmugrezco": "enmugrecer",
    "enmugrezcáis": "enmugrecer",
    "ennegrece": "ennegrecer",
    "ennegreced": "ennegrecer",
    "ennegrecemos": "ennegrecer",
    "ennegrecen": "ennegrecer",
    "ennegreceremos": "ennegrecer",
    "ennegrecerá": "ennegrecer",
    "ennegrecerán": "ennegrecer",
    "ennegrecerás": "ennegrecer",
    "ennegreceré": "ennegrecer",
    "ennegreceréis": "ennegrecer",
    "ennegrecería": "ennegrecer",
    "ennegreceríais": "ennegrecer",
    "ennegreceríamos": "ennegrecer",
    "ennegrecerían": "ennegrecer",
    "ennegrecerías": "ennegrecer",
    "ennegreces": "ennegrecer",
    "ennegrecida": "ennegrecer",
    "ennegrecidas": "ennegrecer",
    "ennegrecido": "ennegrecer",
    "ennegrecidos": "ennegrecer",
    "ennegreciendo": "ennegrecer",
    "ennegreciera": "ennegrecer",
    "ennegrecierais": "ennegrecer",
    "ennegrecieran": "ennegrecer",
    "ennegrecieras": "ennegrecer",
    "ennegreciere": "ennegrecer",
    "ennegreciereis": "ennegrecer",
    "ennegrecieren": "ennegrecer",
    "ennegrecieres": "ennegrecer",
    "ennegrecieron": "ennegrecer",
    "ennegreciese": "ennegrecer",
    "ennegrecieseis": "ennegrecer",
    "ennegreciesen": "ennegrecer",
    "ennegrecieses": "ennegrecer",
    "ennegrecimientos": "ennegrecimiento",
    "ennegrecimos": "ennegrecer",
    "ennegreciste": "ennegrecer",
    "ennegrecisteis": "ennegrecer",
    "ennegreciéramos": "ennegrecer",
    "ennegreciéremos": "ennegrecer",
    "ennegreciésemos": "ennegrecer",
    "ennegreció": "ennegrecer",
    "ennegrecéis": "ennegrecer",
    "ennegrecí": "ennegrecer",
    "ennegrecía": "ennegrecer",
    "ennegrecíais": "ennegrecer",
    "ennegrecíamos": "ennegrecer",
    "ennegrecían": "ennegrecer",
    "ennegrecías": "ennegrecer",
    "ennegrezca": "ennegrecer",
    "ennegrezcamos": "ennegrecer",
    "ennegrezcan": "ennegrecer",
    "ennegrezcas": "ennegrecer",
    "ennegrezco": "ennegrecer",
    "ennegrezcáis": "ennegrecer",
    "ennoblece": "ennoblecer",
    "ennobleced": "ennoblecer",
    "ennoblecemos": "ennoblecer",
    "ennoblecen": "ennoblecer",
    "ennobleceremos": "ennoblecer",
    "ennoblecerá": "ennoblecer",
    "ennoblecerán": "ennoblecer",
    "ennoblecerás": "ennoblecer",
    "ennobleceré": "ennoblecer",
    "ennobleceréis": "ennoblecer",
    "ennoblecería": "ennoblecer",
    "ennobleceríais": "ennoblecer",
    "ennobleceríamos": "ennoblecer",
    "ennoblecerían": "ennoblecer",
    "ennoblecerías": "ennoblecer",
    "ennobleces": "ennoblecer",
    "ennoblecida": "ennoblecer",
    "ennoblecidas": "ennoblecer",
    "ennoblecido": "ennoblecer",
    "ennoblecidos": "ennoblecer",
    "ennobleciendo": "ennoblecer",
    "ennobleciera": "ennoblecer",
    "ennoblecierais": "ennoblecer",
    "ennoblecieran": "ennoblecer",
    "ennoblecieras": "ennoblecer",
    "ennobleciere": "ennoblecer",
    "ennobleciereis": "ennoblecer",
    "ennoblecieren": "ennoblecer",
    "ennoblecieres": "ennoblecer",
    "ennoblecieron": "ennoblecer",
    "ennobleciese": "ennoblecer",
    "ennoblecieseis": "ennoblecer",
    "ennobleciesen": "ennoblecer",
    "ennoblecieses": "ennoblecer",
    "ennoblecimientos": "ennoblecimiento",
    "ennoblecimos": "ennoblecer",
    "ennobleciste": "ennoblecer",
    "ennoblecisteis": "ennoblecer",
    "ennobleciéramos": "ennoblecer",
    "ennobleciéremos": "ennoblecer",
    "ennobleciésemos": "ennoblecer",
    "ennobleció": "ennoblecer",
    "ennoblecéis": "ennoblecer",
    "ennoblecí": "ennoblecer",
    "ennoblecía": "ennoblecer",
    "ennoblecíais": "ennoblecer",
    "ennoblecíamos": "ennoblecer",
    "ennoblecían": "ennoblecer",
    "ennoblecías": "ennoblecer",
    "ennoblezca": "ennoblecer",
    "ennoblezcamos": "ennoblecer",
    "ennoblezcan": "ennoblecer",
    "ennoblezcas": "ennoblecer",
    "ennoblezco": "ennoblecer",
    "ennoblezcáis": "ennoblecer",
    "enodios": "enodio",
    "enodrida": "enodrido",
    "enodridas": "enodrido",
    "enodridos": "enodrido",
    "enoftalmías": "enoftalmía",
    "enografías": "enografía",
    "enográfica": "enográfico",
    "enográficas": "enográfico",
    "enográficos": "enográfico",
    "enoja": "enojar",
    "enojaba": "enojar",
    "enojabais": "enojar",
    "enojaban": "enojar",
    "enojabas": "enojar",
    "enojad": "enojar",
    "enojada": "enojar",
    "enojadas": "enojar",
    "enojadiza": "enojadizo",
    "enojadizas": "enojadizo",
    "enojadizos": "enojadizo",
    "enojado": "enojar",
    "enojados": "enojar",
    "enojamos": "enojar",
    "enojan": "enojar",
    "enojando": "enojar",
    "enojara": "enojar",
    "enojarais": "enojar",
    "enojaran": "enojar",
    "enojaras": "enojar",
    "enojare": "enojar",
    "enojareis": "enojar",
    "enojaremos": "enojar",
    "enojaren": "enojar",
    "enojares": "enojar",
    "enojaron": "enojar",
    "enojará": "enojar",
    "enojarán": "enojar",
    "enojarás": "enojar",
    "enojaré": "enojar",
    "enojaréis": "enojar",
    "enojaría": "enojar",
    "enojaríais": "enojar",
    "enojaríamos": "enojar",
    "enojarían": "enojar",
    "enojarías": "enojar",
    "enojas": "enojar",
    "enojase": "enojar",
    "enojaseis": "enojar",
    "enojasen": "enojar",
    "enojases": "enojar",
    "enojaste": "enojar",
    "enojasteis": "enojar",
    "enoje": "enojar",
    "enojemos": "enojar",
    "enojen": "enojar",
    "enojes": "enojar",
    "enojo": "enojar",
    "enojos": "enojo",
    "enojosa": "enojoso",
    "enojosas": "enojoso",
    "enojosos": "enojoso",
    "enojábamos": "enojar",
    "enojáis": "enojar",
    "enojáramos": "enojar",
    "enojáremos": "enojar",
    "enojásemos": "enojar",
    "enojé": "enojar",
    "enojéis": "enojar",
    "enojó": "enojar",
    "enoles": "enol",
    "enologías": "enología",
    "enológica": "enológico",
    "enológicas": "enológico",
    "enológicos": "enológico",
    "enorgullece": "enorgullecer",
    "enorgulleced": "enorgullecer",
    "enorgullecedora": "enorgullecedor",
    "enorgullecedoras": "enorgullecedor",
    "enorgullecedores": "enorgullecedor",
    "enorgullecemos": "enorgullecer",
    "enorgullecen": "enorgullecer",
    "enorgulleceremos": "enorgullecer",
    "enorgullecerá": "enorgullecer",
    "enorgullecerán": "enorgullecer",
    "enorgullecerás": "enorgullecer",
    "enorgulleceré": "enorgullecer",
    "enorgulleceréis": "enorgullecer",
    "enorgullecería": "enorgullecer",
    "enorgulleceríais": "enorgullecer",
    "enorgulleceríamos": "enorgullecer",
    "enorgullecerían": "enorgullecer",
    "enorgullecerías": "enorgullecer",
    "enorgulleces": "enorgullecer",
    "enorgullecida": "enorgullecer",
    "enorgullecidas": "enorgullecer",
    "enorgullecido": "enorgullecer",
    "enorgullecidos": "enorgullecer",
    "enorgulleciendo": "enorgullecer",
    "enorgulleciera": "enorgullecer",
    "enorgullecierais": "enorgullecer",
    "enorgullecieran": "enorgullecer",
    "enorgullecieras": "enorgullecer",
    "enorgulleciere": "enorgullecer",
    "enorgulleciereis": "enorgullecer",
    "enorgullecieren": "enorgullecer",
    "enorgullecieres": "enorgullecer",
    "enorgullecieron": "enorgullecer",
    "enorgulleciese": "enorgullecer",
    "enorgullecieseis": "enorgullecer",
    "enorgulleciesen": "enorgullecer",
    "enorgullecieses": "enorgullecer",
    "enorgullecimientos": "enorgullecimiento",
    "enorgullecimos": "enorgullecer",
    "enorgulleciste": "enorgullecer",
    "enorgullecisteis": "enorgullecer",
    "enorgulleciéramos": "enorgullecer",
    "enorgulleciéremos": "enorgullecer",
    "enorgulleciésemos": "enorgullecer",
    "enorgulleció": "enorgullecer",
    "enorgullecéis": "enorgullecer",
    "enorgullecí": "enorgullecer",
    "enorgullecía": "enorgullecer",
    "enorgullecíais": "enorgullecer",
    "enorgullecíamos": "enorgullecer",
    "enorgullecían": "enorgullecer",
    "enorgullecías": "enorgullecer",
    "enorgullezca": "enorgullecer",
    "enorgullezcamos": "enorgullecer",
    "enorgullezcan": "enorgullecer",
    "enorgullezcas": "enorgullecer",
    "enorgullezco": "enorgullecer",
    "enorgullezcáis": "enorgullecer",
    "enormes": "enorme",
    "enormidades": "enormidad",
    "enotecnias": "enotecnia",
    "enoteráceas": "enoterácea",
    "enotécnica": "enotécnico",
    "enotécnicas": "enotécnico",
    "enotécnicos": "enotécnico",
    "enquicia": "enquiciar",
    "enquiciaba": "enquiciar",
    "enquiciabais": "enquiciar",
    "enquiciaban": "enquiciar",
    "enquiciabas": "enquiciar",
    "enquiciad": "enquiciar",
    "enquiciada": "enquiciar",
    "enquiciadas": "enquiciar",
    "enquiciado": "enquiciar",
    "enquiciados": "enquiciar",
    "enquiciamientos": "enquiciamiento",
    "enquiciamos": "enquiciar",
    "enquician": "enquiciar",
    "enquiciando": "enquiciar",
    "enquiciara": "enquiciar",
    "enquiciarais": "enquiciar",
    "enquiciaran": "enquiciar",
    "enquiciaras": "enquiciar",
    "enquiciare": "enquiciar",
    "enquiciareis": "enquiciar",
    "enquiciaremos": "enquiciar",
    "enquiciaren": "enquiciar",
    "enquiciares": "enquiciar",
    "enquiciaron": "enquiciar",
    "enquiciará": "enquiciar",
    "enquiciarán": "enquiciar",
    "enquiciarás": "enquiciar",
    "enquiciaré": "enquiciar",
    "enquiciaréis": "enquiciar",
    "enquiciaría": "enquiciar",
    "enquiciaríais": "enquiciar",
    "enquiciaríamos": "enquiciar",
    "enquiciarían": "enquiciar",
    "enquiciarías": "enquiciar",
    "enquicias": "enquiciar",
    "enquiciase": "enquiciar",
    "enquiciaseis": "enquiciar",
    "enquiciasen": "enquiciar",
    "enquiciases": "enquiciar",
    "enquiciaste": "enquiciar",
    "enquiciasteis": "enquiciar",
    "enquicie": "enquiciar",
    "enquiciemos": "enquiciar",
    "enquicien": "enquiciar",
    "enquicies": "enquiciar",
    "enquicio": "enquiciar",
    "enquiciábamos": "enquiciar",
    "enquiciáis": "enquiciar",
    "enquiciáramos": "enquiciar",
    "enquiciáremos": "enquiciar",
    "enquiciásemos": "enquiciar",
    "enquicié": "enquiciar",
    "enquiciéis": "enquiciar",
    "enquició": "enquiciar",
    "enquillotra": "enquillotrar",
    "enquillotraba": "enquillotrar",
    "enquillotrabais": "enquillotrar",
    "enquillotraban": "enquillotrar",
    "enquillotrabas": "enquillotrar",
    "enquillotrad": "enquillotrar",
    "enquillotrada": "enquillotrar",
    "enquillotradas": "enquillotrar",
    "enquillotrado": "enquillotrar",
    "enquillotrados": "enquillotrar",
    "enquillotramos": "enquillotrar",
    "enquillotran": "enquillotrar",
    "enquillotrando": "enquillotrar",
    "enquillotrara": "enquillotrar",
    "enquillotrarais": "enquillotrar",
    "enquillotraran": "enquillotrar",
    "enquillotraras": "enquillotrar",
    "enquillotrare": "enquillotrar",
    "enquillotrareis": "enquillotrar",
    "enquillotraremos": "enquillotrar",
    "enquillotraren": "enquillotrar",
    "enquillotrares": "enquillotrar",
    "enquillotraron": "enquillotrar",
    "enquillotrará": "enquillotrar",
    "enquillotrarán": "enquillotrar",
    "enquillotrarás": "enquillotrar",
    "enquillotraré": "enquillotrar",
    "enquillotraréis": "enquillotrar",
    "enquillotraría": "enquillotrar",
    "enquillotraríais": "enquillotrar",
    "enquillotraríamos": "enquillotrar",
    "enquillotrarían": "enquillotrar",
    "enquillotrarías": "enquillotrar",
    "enquillotras": "enquillotrar",
    "enquillotrase": "enquillotrar",
    "enquillotraseis": "enquillotrar",
    "enquillotrasen": "enquillotrar",
    "enquillotrases": "enquillotrar",
    "enquillotraste": "enquillotrar",
    "enquillotrasteis": "enquillotrar",
    "enquillotre": "enquillotrar",
    "enquillotremos": "enquillotrar",
    "enquillotren": "enquillotrar",
    "enquillotres": "enquillotrar",
    "enquillotro": "enquillotrar",
    "enquillotrábamos": "enquillotrar",
    "enquillotráis": "enquillotrar",
    "enquillotráramos": "enquillotrar",
    "enquillotráremos": "enquillotrar",
    "enquillotrásemos": "enquillotrar",
    "enquillotré": "enquillotrar",
    "enquillotréis": "enquillotrar",
    "enquillotró": "enquillotrar",
    "enquiridiones": "enquiridión",
    "enquista": "enquistar",
    "enquistaba": "enquistar",
    "enquistabais": "enquistar",
    "enquistaban": "enquistar",
    "enquistabas": "enquistar",
    "enquistad": "enquistar",
    "enquistada": "enquistar",
    "enquistadas": "enquistar",
    "enquistado": "enquistar",
    "enquistados": "enquistar",
    "enquistamientos": "enquistamiento",
    "enquistamos": "enquistar",
    "enquistan": "enquistar",
    "enquistando": "enquistar",
    "enquistara": "enquistar",
    "enquistarais": "enquistar",
    "enquistaran": "enquistar",
    "enquistaras": "enquistar",
    "enquistare": "enquistar",
    "enquistareis": "enquistar",
    "enquistaremos": "enquistar",
    "enquistaren": "enquistar",
    "enquistares": "enquistar",
    "enquistaron": "enquistar",
    "enquistará": "enquistar",
    "enquistarán": "enquistar",
    "enquistarás": "enquistar",
    "enquistaré": "enquistar",
    "enquistaréis": "enquistar",
    "enquistaría": "enquistar",
    "enquistaríais": "enquistar",
    "enquistaríamos": "enquistar",
    "enquistarían": "enquistar",
    "enquistarías": "enquistar",
    "enquistas": "enquistar",
    "enquistase": "enquistar",
    "enquistaseis": "enquistar",
    "enquistasen": "enquistar",
    "enquistases": "enquistar",
    "enquistaste": "enquistar",
    "enquistasteis": "enquistar",
    "enquiste": "enquistar",
    "enquistemos": "enquistar",
    "enquisten": "enquistar",
    "enquistes": "enquistar",
    "enquisto": "enquistar",
    "enquistábamos": "enquistar",
    "enquistáis": "enquistar",
    "enquistáramos": "enquistar",
    "enquistáremos": "enquistar",
    "enquistásemos": "enquistar",
    "enquisté": "enquistar",
    "enquistéis": "enquistar",
    "enquistó": "enquistar",
    "enrabia": "enrabiar",
    "enrabiaba": "enrabiar",
    "enrabiabais": "enrabiar",
    "enrabiaban": "enrabiar",
    "enrabiabas": "enrabiar",
    "enrabiad": "enrabiar",
    "enrabiada": "enrabiar",
    "enrabiadas": "enrabiar",
    "enrabiado": "enrabiar",
    "enrabiados": "enrabiar",
    "enrabiamos": "enrabiar",
    "enrabian": "enrabiar",
    "enrabiando": "enrabiar",
    "enrabiara": "enrabiar",
    "enrabiarais": "enrabiar",
    "enrabiaran": "enrabiar",
    "enrabiaras": "enrabiar",
    "enrabiare": "enrabiar",
    "enrabiareis": "enrabiar",
    "enrabiaremos": "enrabiar",
    "enrabiaren": "enrabiar",
    "enrabiares": "enrabiar",
    "enrabiaron": "enrabiar",
    "enrabiará": "enrabiar",
    "enrabiarán": "enrabiar",
    "enrabiarás": "enrabiar",
    "enrabiaré": "enrabiar",
    "enrabiaréis": "enrabiar",
    "enrabiaría": "enrabiar",
    "enrabiaríais": "enrabiar",
    "enrabiaríamos": "enrabiar",
    "enrabiarían": "enrabiar",
    "enrabiarías": "enrabiar",
    "enrabias": "enrabiar",
    "enrabiase": "enrabiar",
    "enrabiaseis": "enrabiar",
    "enrabiasen": "enrabiar",
    "enrabiases": "enrabiar",
    "enrabiaste": "enrabiar",
    "enrabiasteis": "enrabiar",
    "enrabie": "enrabiar",
    "enrabiemos": "enrabiar",
    "enrabien": "enrabiar",
    "enrabies": "enrabiar",
    "enrabieta": "enrabietar",
    "enrabietaba": "enrabietar",
    "enrabietabais": "enrabietar",
    "enrabietaban": "enrabietar",
    "enrabietabas": "enrabietar",
    "enrabietad": "enrabietar",
    "enrabietada": "enrabietar",
    "enrabietadas": "enrabietar",
    "enrabietado": "enrabietar",
    "enrabietados": "enrabietar",
    "enrabietamos": "enrabietar",
    "enrabietan": "enrabietar",
    "enrabietando": "enrabietar",
    "enrabietara": "enrabietar",
    "enrabietarais": "enrabietar",
    "enrabietaran": "enrabietar",
    "enrabietaras": "enrabietar",
    "enrabietare": "enrabietar",
    "enrabietareis": "enrabietar",
    "enrabietaremos": "enrabietar",
    "enrabietaren": "enrabietar",
    "enrabietares": "enrabietar",
    "enrabietaron": "enrabietar",
    "enrabietará": "enrabietar",
    "enrabietarán": "enrabietar",
    "enrabietarás": "enrabietar",
    "enrabietaré": "enrabietar",
    "enrabietaréis": "enrabietar",
    "enrabietaría": "enrabietar",
    "enrabietaríais": "enrabietar",
    "enrabietaríamos": "enrabietar",
    "enrabietarían": "enrabietar",
    "enrabietarías": "enrabietar",
    "enrabietas": "enrabietar",
    "enrabietase": "enrabietar",
    "enrabietaseis": "enrabietar",
    "enrabietasen": "enrabietar",
    "enrabietases": "enrabietar",
    "enrabietaste": "enrabietar",
    "enrabietasteis": "enrabietar",
    "enrabiete": "enrabietar",
    "enrabietemos": "enrabietar",
    "enrabieten": "enrabietar",
    "enrabietes": "enrabietar",
    "enrabieto": "enrabietar",
    "enrabietábamos": "enrabietar",
    "enrabietáis": "enrabietar",
    "enrabietáramos": "enrabietar",
    "enrabietáremos": "enrabietar",
    "enrabietásemos": "enrabietar",
    "enrabieté": "enrabietar",
    "enrabietéis": "enrabietar",
    "enrabietó": "enrabietar",
    "enrabio": "enrabiar",
    "enrabiábamos": "enrabiar",
    "enrabiáis": "enrabiar",
    "enrabiáramos": "enrabiar",
    "enrabiáremos": "enrabiar",
    "enrabiásemos": "enrabiar",
    "enrabié": "enrabiar",
    "enrabiéis": "enrabiar",
    "enrabió": "enrabiar",
    "enraicemos": "enraizar",
    "enraicé": "enraizar",
    "enraicéis": "enraizar",
    "enraizaba": "enraizar",
    "enraizabais": "enraizar",
    "enraizaban": "enraizar",
    "enraizabas": "enraizar",
    "enraizad": "enraizar",
    "enraizada": "enraizar",
    "enraizadas": "enraizar",
    "enraizado": "enraizar",
    "enraizados": "enraizar",
    "enraizamientos": "enraizamiento",
    "enraizamos": "enraizar",
    "enraizando": "enraizar",
    "enraizara": "enraizar",
    "enraizarais": "enraizar",
    "enraizaran": "enraizar",
    "enraizaras": "enraizar",
    "enraizare": "enraizar",
    "enraizareis": "enraizar",
    "enraizaremos": "enraizar",
    "enraizaren": "enraizar",
    "enraizares": "enraizar",
    "enraizaron": "enraizar",
    "enraizará": "enraizar",
    "enraizarán": "enraizar",
    "enraizarás": "enraizar",
    "enraizaré": "enraizar",
    "enraizaréis": "enraizar",
    "enraizaría": "enraizar",
    "enraizaríais": "enraizar",
    "enraizaríamos": "enraizar",
    "enraizarían": "enraizar",
    "enraizarías": "enraizar",
    "enraizase": "enraizar",
    "enraizaseis": "enraizar",
    "enraizasen": "enraizar",
    "enraizases": "enraizar",
    "enraizaste": "enraizar",
    "enraizasteis": "enraizar",
    "enraizábamos": "enraizar",
    "enraizáis": "enraizar",
    "enraizáramos": "enraizar",
    "enraizáremos": "enraizar",
    "enraizásemos": "enraizar",
    "enraizó": "enraizar",
    "enrama": "enramar",
    "enramaba": "enramar",
    "enramabais": "enramar",
    "enramaban": "enramar",
    "enramabas": "enramar",
    "enramad": "enramar",
    "enramada": "enramar",
    "enramadas": "enramar",
    "enramado": "enramar",
    "enramados": "enramar",
    "enramamos": "enramar",
    "enraman": "enramar",
    "enramando": "enramar",
    "enramara": "enramar",
    "enramarais": "enramar",
    "enramaran": "enramar",
    "enramaras": "enramar",
    "enramare": "enramar",
    "enramareis": "enramar",
    "enramaremos": "enramar",
    "enramaren": "enramar",
    "enramares": "enramar",
    "enramaron": "enramar",
    "enramará": "enramar",
    "enramarán": "enramar",
    "enramarás": "enramar",
    "enramaré": "enramar",
    "enramaréis": "enramar",
    "enramaría": "enramar",
    "enramaríais": "enramar",
    "enramaríamos": "enramar",
    "enramarían": "enramar",
    "enramarías": "enramar",
    "enramas": "enramar",
    "enramase": "enramar",
    "enramaseis": "enramar",
    "enramasen": "enramar",
    "enramases": "enramar",
    "enramaste": "enramar",
    "enramasteis": "enramar",
    "enrame": "enramar",
    "enramemos": "enramar",
    "enramen": "enramar",
    "enrames": "enrame",
    "enramo": "enramar",
    "enramábamos": "enramar",
    "enramáis": "enramar",
    "enramáramos": "enramar",
    "enramáremos": "enramar",
    "enramásemos": "enramar",
    "enramé": "enramar",
    "enraméis": "enramar",
    "enramó": "enramar",
    "enrancia": "enranciar",
    "enranciaba": "enranciar",
    "enranciabais": "enranciar",
    "enranciaban": "enranciar",
    "enranciabas": "enranciar",
    "enranciad": "enranciar",
    "enranciada": "enranciar",
    "enranciadas": "enranciar",
    "enranciado": "enranciar",
    "enranciados": "enranciar",
    "enranciamos": "enranciar",
    "enrancian": "enranciar",
    "enranciando": "enranciar",
    "enranciara": "enranciar",
    "enranciarais": "enranciar",
    "enranciaran": "enranciar",
    "enranciaras": "enranciar",
    "enranciare": "enranciar",
    "enranciareis": "enranciar",
    "enranciaremos": "enranciar",
    "enranciaren": "enranciar",
    "enranciares": "enranciar",
    "enranciaron": "enranciar",
    "enranciará": "enranciar",
    "enranciarán": "enranciar",
    "enranciarás": "enranciar",
    "enranciaré": "enranciar",
    "enranciaréis": "enranciar",
    "enranciaría": "enranciar",
    "enranciaríais": "enranciar",
    "enranciaríamos": "enranciar",
    "enranciarían": "enranciar",
    "enranciarías": "enranciar",
    "enrancias": "enranciar",
    "enranciase": "enranciar",
    "enranciaseis": "enranciar",
    "enranciasen": "enranciar",
    "enranciases": "enranciar",
    "enranciaste": "enranciar",
    "enranciasteis": "enranciar",
    "enrancie": "enranciar",
    "enranciemos": "enranciar",
    "enrancien": "enranciar",
    "enrancies": "enranciar",
    "enrancio": "enranciar",
    "enranciábamos": "enranciar",
    "enranciáis": "enranciar",
    "enranciáramos": "enranciar",
    "enranciáremos": "enranciar",
    "enranciásemos": "enranciar",
    "enrancié": "enranciar",
    "enranciéis": "enranciar",
    "enranció": "enranciar",
    "enrarece": "enrarecer",
    "enrareced": "enrarecer",
    "enrarecemos": "enrarecer",
    "enrarecen": "enrarecer",
    "enrareceremos": "enrarecer",
    "enrarecerá": "enrarecer",
    "enrarecerán": "enrarecer",
    "enrarecerás": "enrarecer",
    "enrareceré": "enrarecer",
    "enrareceréis": "enrarecer",
    "enrarecería": "enrarecer",
    "enrareceríais": "enrarecer",
    "enrareceríamos": "enrarecer",
    "enrarecerían": "enrarecer",
    "enrarecerías": "enrarecer",
    "enrareces": "enrarecer",
    "enrarecida": "enrarecer",
    "enrarecidas": "enrarecer",
    "enrarecido": "enrarecer",
    "enrarecidos": "enrarecer",
    "enrareciendo": "enrarecer",
    "enrareciera": "enrarecer",
    "enrarecierais": "enrarecer",
    "enrarecieran": "enrarecer",
    "enrarecieras": "enrarecer",
    "enrareciere": "enrarecer",
    "enrareciereis": "enrarecer",
    "enrarecieren": "enrarecer",
    "enrarecieres": "enrarecer",
    "enrarecieron": "enrarecer",
    "enrareciese": "enrarecer",
    "enrarecieseis": "enrarecer",
    "enrareciesen": "enrarecer",
    "enrarecieses": "enrarecer",
    "enrarecimientos": "enrarecimiento",
    "enrarecimos": "enrarecer",
    "enrareciste": "enrarecer",
    "enrarecisteis": "enrarecer",
    "enrareciéramos": "enrarecer",
    "enrareciéremos": "enrarecer",
    "enrareciésemos": "enrarecer",
    "enrareció": "enrarecer",
    "enrarecéis": "enrarecer",
    "enrarecí": "enrarecer",
    "enrarecía": "enrarecer",
    "enrarecíais": "enrarecer",
    "enrarecíamos": "enrarecer",
    "enrarecían": "enrarecer",
    "enrarecías": "enrarecer",
    "enrarezca": "enrarecer",
    "enrarezcamos": "enrarecer",
    "enrarezcan": "enrarecer",
    "enrarezcas": "enrarecer",
    "enrarezco": "enrarecer",
    "enrarezcáis": "enrarecer",
    "enrasa": "enrasar",
    "enrasaba": "enrasar",
    "enrasabais": "enrasar",
    "enrasaban": "enrasar",
    "enrasabas": "enrasar",
    "enrasad": "enrasar",
    "enrasada": "enrasar",
    "enrasadas": "enrasar",
    "enrasado": "enrasar",
    "enrasados": "enrasar",
    "enrasamos": "enrasar",
    "enrasan": "enrasar",
    "enrasando": "enrasar",
    "enrasara": "enrasar",
    "enrasarais": "enrasar",
    "enrasaran": "enrasar",
    "enrasaras": "enrasar",
    "enrasare": "enrasar",
    "enrasareis": "enrasar",
    "enrasaremos": "enrasar",
    "enrasaren": "enrasar",
    "enrasares": "enrasar",
    "enrasaron": "enrasar",
    "enrasará": "enrasar",
    "enrasarán": "enrasar",
    "enrasarás": "enrasar",
    "enrasaré": "enrasar",
    "enrasaréis": "enrasar",
    "enrasaría": "enrasar",
    "enrasaríais": "enrasar",
    "enrasaríamos": "enrasar",
    "enrasarían": "enrasar",
    "enrasarías": "enrasar",
    "enrasas": "enrasar",
    "enrasase": "enrasar",
    "enrasaseis": "enrasar",
    "enrasasen": "enrasar",
    "enrasases": "enrasar",
    "enrasaste": "enrasar",
    "enrasasteis": "enrasar",
    "enrase": "enrasar",
    "enrasemos": "enrasar",
    "enrasen": "enrasar",
    "enrases": "enrase",
    "enraso": "enrasar",
    "enrasábamos": "enrasar",
    "enrasáis": "enrasar",
    "enrasáramos": "enrasar",
    "enrasáremos": "enrasar",
    "enrasásemos": "enrasar",
    "enrasé": "enrasar",
    "enraséis": "enrasar",
    "enrasó": "enrasar",
    "enraya": "enrayar",
    "enrayaba": "enrayar",
    "enrayabais": "enrayar",
    "enrayaban": "enrayar",
    "enrayabas": "enrayar",
    "enrayad": "enrayar",
    "enrayada": "enrayar",
    "enrayadas": "enrayar",
    "enrayado": "enrayar",
    "enrayados": "enrayar",
    "enrayamos": "enrayar",
    "enrayan": "enrayar",
    "enrayando": "enrayar",
    "enrayara": "enrayar",
    "enrayarais": "enrayar",
    "enrayaran": "enrayar",
    "enrayaras": "enrayar",
    "enrayare": "enrayar",
    "enrayareis": "enrayar",
    "enrayaremos": "enrayar",
    "enrayaren": "enrayar",
    "enrayares": "enrayar",
    "enrayaron": "enrayar",
    "enrayará": "enrayar",
    "enrayarán": "enrayar",
    "enrayarás": "enrayar",
    "enrayaré": "enrayar",
    "enrayaréis": "enrayar",
    "enrayaría": "enrayar",
    "enrayaríais": "enrayar",
    "enrayaríamos": "enrayar",
    "enrayarían": "enrayar",
    "enrayarías": "enrayar",
    "enrayas": "enrayar",
    "enrayase": "enrayar",
    "enrayaseis": "enrayar",
    "enrayasen": "enrayar",
    "enrayases": "enrayar",
    "enrayaste": "enrayar",
    "enrayasteis": "enrayar",
    "enraye": "enrayar",
    "enrayemos": "enrayar",
    "enrayen": "enrayar",
    "enrayes": "enrayar",
    "enrayo": "enrayar",
    "enrayábamos": "enrayar",
    "enrayáis": "enrayar",
    "enrayáramos": "enrayar",
    "enrayáremos": "enrayar",
    "enrayásemos": "enrayar",
    "enrayé": "enrayar",
    "enrayéis": "enrayar",
    "enrayó": "enrayar",
    "enraíce": "enraizar",
    "enraícen": "enraizar",
    "enraíces": "enraizar",
    "enraíza": "enraizar",
    "enraízan": "enraizar",
    "enraízas": "enraizar",
    "enraízo": "enraizar",
    "enreda": "enredar",
    "enredaba": "enredar",
    "enredabais": "enredar",
    "enredaban": "enredar",
    "enredabas": "enredar",
    "enredad": "enredar",
    "enredada": "enredar",
    "enredadas": "enredar",
    "enredaderas": "enredadera",
    "enredado": "enredar",
    "enredadora": "enredador",
    "enredadoras": "enredador",
    "enredadores": "enredador",
    "enredados": "enredar",
    "enredamos": "enredar",
    "enredan": "enredar",
    "enredando": "enredar",
    "enredara": "enredar",
    "enredarais": "enredar",
    "enredaran": "enredar",
    "enredaras": "enredar",
    "enredare": "enredar",
    "enredareis": "enredar",
    "enredaremos": "enredar",
    "enredaren": "enredar",
    "enredares": "enredar",
    "enredaron": "enredar",
    "enredará": "enredar",
    "enredarán": "enredar",
    "enredarás": "enredar",
    "enredaré": "enredar",
    "enredaréis": "enredar",
    "enredaría": "enredar",
    "enredaríais": "enredar",
    "enredaríamos": "enredar",
    "enredarían": "enredar",
    "enredarías": "enredar",
    "enredas": "enredar",
    "enredase": "enredar",
    "enredaseis": "enredar",
    "enredasen": "enredar",
    "enredases": "enredar",
    "enredaste": "enredar",
    "enredasteis": "enredar",
    "enrede": "enredar",
    "enredemos": "enredar",
    "enreden": "enredar",
    "enredes": "enredar",
    "enredijos": "enredijo",
    "enredo": "enredar",
    "enredona": "enredón",
    "enredonas": "enredón",
    "enredones": "enredón",
    "enredos": "enredo",
    "enredosa": "enredoso",
    "enredosas": "enredoso",
    "enredosos": "enredoso",
    "enredábamos": "enredar",
    "enredáis": "enredar",
    "enredáramos": "enredar",
    "enredáremos": "enredar",
    "enredásemos": "enredar",
    "enredé": "enredar",
    "enredéis": "enredar",
    "enredó": "enredar",
    "enreja": "enrejar",
    "enrejaba": "enrejar",
    "enrejabais": "enrejar",
    "enrejaban": "enrejar",
    "enrejabas": "enrejar",
    "enrejad": "enrejar",
    "enrejada": "enrejar",
    "enrejadas": "enrejar",
    "enrejado": "enrejar",
    "enrejados": "enrejar",
    "enrejaduras": "enrejadura",
    "enrejamos": "enrejar",
    "enrejan": "enrejar",
    "enrejando": "enrejar",
    "enrejara": "enrejar",
    "enrejarais": "enrejar",
    "enrejaran": "enrejar",
    "enrejaras": "enrejar",
    "enrejare": "enrejar",
    "enrejareis": "enrejar",
    "enrejaremos": "enrejar",
    "enrejaren": "enrejar",
    "enrejares": "enrejar",
    "enrejaron": "enrejar",
    "enrejará": "enrejar",
    "enrejarán": "enrejar",
    "enrejarás": "enrejar",
    "enrejaré": "enrejar",
    "enrejaréis": "enrejar",
    "enrejaría": "enrejar",
    "enrejaríais": "enrejar",
    "enrejaríamos": "enrejar",
    "enrejarían": "enrejar",
    "enrejarías": "enrejar",
    "enrejas": "enrejar",
    "enrejase": "enrejar",
    "enrejaseis": "enrejar",
    "enrejasen": "enrejar",
    "enrejases": "enrejar",
    "enrejaste": "enrejar",
    "enrejasteis": "enrejar",
    "enreje": "enrejar",
    "enrejemos": "enrejar",
    "enrejen": "enrejar",
    "enrejes": "enrejar",
    "enrejillada": "enrejillado",
    "enrejilladas": "enrejillado",
    "enrejillados": "enrejillado",
    "enrejo": "enrejar",
    "enrejábamos": "enrejar",
    "enrejáis": "enrejar",
    "enrejáramos": "enrejar",
    "enrejáremos": "enrejar",
    "enrejásemos": "enrejar",
    "enrejé": "enrejar",
    "enrejéis": "enrejar",
    "enrejó": "enrejar",
    "enrevesada": "enrevesado",
    "enrevesadas": "enrevesado",
    "enrevesados": "enrevesado",
    "enriaba": "enriar",
    "enriabais": "enriar",
    "enriaban": "enriar",
    "enriabas": "enriar",
    "enriad": "enriar",
    "enriada": "enriar",
    "enriadas": "enriar",
    "enriado": "enriar",
    "enriadora": "enriador",
    "enriadoras": "enriador",
    "enriadores": "enriador",
    "enriados": "enriar",
    "enriamientos": "enriamiento",
    "enriamos": "enriar",
    "enriando": "enriar",
    "enriara": "enriar",
    "enriarais": "enriar",
    "enriaran": "enriar",
    "enriaras": "enriar",
    "enriare": "enriar",
    "enriareis": "enriar",
    "enriaremos": "enriar",
    "enriaren": "enriar",
    "enriares": "enriar",
    "enriaron": "enriar",
    "enriará": "enriar",
    "enriarán": "enriar",
    "enriarás": "enriar",
    "enriaré": "enriar",
    "enriaréis": "enriar",
    "enriaría": "enriar",
    "enriaríais": "enriar",
    "enriaríamos": "enriar",
    "enriarían": "enriar",
    "enriarías": "enriar",
    "enriase": "enriar",
    "enriaseis": "enriar",
    "enriasen": "enriar",
    "enriases": "enriar",
    "enriaste": "enriar",
    "enriasteis": "enriar",
    "enriela": "enrielar",
    "enrielaba": "enrielar",
    "enrielabais": "enrielar",
    "enrielaban": "enrielar",
    "enrielabas": "enrielar",
    "enrielad": "enrielar",
    "enrielada": "enrielar",
    "enrieladas": "enrielar",
    "enrielado": "enrielar",
    "enrielados": "enrielar",
    "enrielamos": "enrielar",
    "enrielan": "enrielar",
    "enrielando": "enrielar",
    "enrielara": "enrielar",
    "enrielarais": "enrielar",
    "enrielaran": "enrielar",
    "enrielaras": "enrielar",
    "enrielare": "enrielar",
    "enrielareis": "enrielar",
    "enrielaremos": "enrielar",
    "enrielaren": "enrielar",
    "enrielares": "enrielar",
    "enrielaron": "enrielar",
    "enrielará": "enrielar",
    "enrielarán": "enrielar",
    "enrielarás": "enrielar",
    "enrielaré": "enrielar",
    "enrielaréis": "enrielar",
    "enrielaría": "enrielar",
    "enrielaríais": "enrielar",
    "enrielaríamos": "enrielar",
    "enrielarían": "enrielar",
    "enrielarías": "enrielar",
    "enrielas": "enrielar",
    "enrielase": "enrielar",
    "enrielaseis": "enrielar",
    "enrielasen": "enrielar",
    "enrielases": "enrielar",
    "enrielaste": "enrielar",
    "enrielasteis": "enrielar",
    "enriele": "enrielar",
    "enrielemos": "enrielar",
    "enrielen": "enrielar",
    "enrieles": "enrielar",
    "enrielo": "enrielar",
    "enrielábamos": "enrielar",
    "enrieláis": "enrielar",
    "enrieláramos": "enrielar",
    "enrieláremos": "enrielar",
    "enrielásemos": "enrielar",
    "enrielé": "enrielar",
    "enrieléis": "enrielar",
    "enrieló": "enrielar",
    "enriemos": "enriar",
    "enrigidecimientos": "enrigidecimiento",
    "enripia": "enripiar",
    "enripiaba": "enripiar",
    "enripiabais": "enripiar",
    "enripiaban": "enripiar",
    "enripiabas": "enripiar",
    "enripiad": "enripiar",
    "enripiada": "enripiar",
    "enripiadas": "enripiar",
    "enripiado": "enripiar",
    "enripiados": "enripiar",
    "enripiamos": "enripiar",
    "enripian": "enripiar",
    "enripiando": "enripiar",
    "enripiara": "enripiar",
    "enripiarais": "enripiar",
    "enripiaran": "enripiar",
    "enripiaras": "enripiar",
    "enripiare": "enripiar",
    "enripiareis": "enripiar",
    "enripiaremos": "enripiar",
    "enripiaren": "enripiar",
    "enripiares": "enripiar",
    "enripiaron": "enripiar",
    "enripiará": "enripiar",
    "enripiarán": "enripiar",
    "enripiarás": "enripiar",
    "enripiaré": "enripiar",
    "enripiaréis": "enripiar",
    "enripiaría": "enripiar",
    "enripiaríais": "enripiar",
    "enripiaríamos": "enripiar",
    "enripiarían": "enripiar",
    "enripiarías": "enripiar",
    "enripias": "enripiar",
    "enripiase": "enripiar",
    "enripiaseis": "enripiar",
    "enripiasen": "enripiar",
    "enripiases": "enripiar",
    "enripiaste": "enripiar",
    "enripiasteis": "enripiar",
    "enripie": "enripiar",
    "enripiemos": "enripiar",
    "enripien": "enripiar",
    "enripies": "enripiar",
    "enripio": "enripiar",
    "enripiábamos": "enripiar",
    "enripiáis": "enripiar",
    "enripiáramos": "enripiar",
    "enripiáremos": "enripiar",
    "enripiásemos": "enripiar",
    "enripié": "enripiar",
    "enripiéis": "enripiar",
    "enripió": "enripiar",
    "enriquece": "enriquecer",
    "enriqueced": "enriquecer",
    "enriquecedora": "enriquecedor",
    "enriquecedoras": "enriquecedor",
    "enriquecedores": "enriquecedor",
    "enriquecemos": "enriquecer",
    "enriquecen": "enriquecer",
    "enriqueceremos": "enriquecer",
    "enriquecerá": "enriquecer",
    "enriquecerán": "enriquecer",
    "enriquecerás": "enriquecer",
    "enriqueceré": "enriquecer",
    "enriqueceréis": "enriquecer",
    "enriquecería": "enriquecer",
    "enriqueceríais": "enriquecer",
    "enriqueceríamos": "enriquecer",
    "enriquecerían": "enriquecer",
    "enriquecerías": "enriquecer",
    "enriqueces": "enriquecer",
    "enriquecida": "enriquecer",
    "enriquecidas": "enriquecer",
    "enriquecido": "enriquecer",
    "enriquecidos": "enriquecer",
    "enriqueciendo": "enriquecer",
    "enriqueciera": "enriquecer",
    "enriquecierais": "enriquecer",
    "enriquecieran": "enriquecer",
    "enriquecieras": "enriquecer",
    "enriqueciere": "enriquecer",
    "enriqueciereis": "enriquecer",
    "enriquecieren": "enriquecer",
    "enriquecieres": "enriquecer",
    "enriquecieron": "enriquecer",
    "enriqueciese": "enriquecer",
    "enriquecieseis": "enriquecer",
    "enriqueciesen": "enriquecer",
    "enriquecieses": "enriquecer",
    "enriquecimientos": "enriquecimiento",
    "enriquecimos": "enriquecer",
    "enriqueciste": "enriquecer",
    "enriquecisteis": "enriquecer",
    "enriqueciéramos": "enriquecer",
    "enriqueciéremos": "enriquecer",
    "enriqueciésemos": "enriquecer",
    "enriqueció": "enriquecer",
    "enriquecéis": "enriquecer",
    "enriquecí": "enriquecer",
    "enriquecía": "enriquecer",
    "enriquecíais": "enriquecer",
    "enriquecíamos": "enriquecer",
    "enriquecían": "enriquecer",
    "enriquecías": "enriquecer",
    "enriquezca": "enriquecer",
    "enriquezcamos": "enriquecer",
    "enriquezcan": "enriquecer",
    "enriquezcas": "enriquecer",
    "enriquezco": "enriquecer",
    "enriquezcáis": "enriquecer",
    "enriqueña": "enriqueño",
    "enriqueñas": "enriqueño",
    "enriqueños": "enriqueño",
    "enrisca": "enriscar",
    "enriscaba": "enriscar",
    "enriscabais": "enriscar",
    "enriscaban": "enriscar",
    "enriscabas": "enriscar",
    "enriscad": "enriscar",
    "enriscada": "enriscar",
    "enriscadas": "enriscar",
    "enriscado": "enriscar",
    "enriscados": "enriscar",
    "enriscamientos": "enriscamiento",
    "enriscamos": "enriscar",
    "enriscan": "enriscar",
    "enriscando": "enriscar",
    "enriscara": "enriscar",
    "enriscarais": "enriscar",
    "enriscaran": "enriscar",
    "enriscaras": "enriscar",
    "enriscare": "enriscar",
    "enriscareis": "enriscar",
    "enriscaremos": "enriscar",
    "enriscaren": "enriscar",
    "enriscares": "enriscar",
    "enriscaron": "enriscar",
    "enriscará": "enriscar",
    "enriscarán": "enriscar",
    "enriscarás": "enriscar",
    "enriscaré": "enriscar",
    "enriscaréis": "enriscar",
    "enriscaría": "enriscar",
    "enriscaríais": "enriscar",
    "enriscaríamos": "enriscar",
    "enriscarían": "enriscar",
    "enriscarías": "enriscar",
    "enriscas": "enriscar",
    "enriscase": "enriscar",
    "enriscaseis": "enriscar",
    "enriscasen": "enriscar",
    "enriscases": "enriscar",
    "enriscaste": "enriscar",
    "enriscasteis": "enriscar",
    "enrisco": "enriscar",
    "enriscábamos": "enriscar",
    "enriscáis": "enriscar",
    "enriscáramos": "enriscar",
    "enriscáremos": "enriscar",
    "enriscásemos": "enriscar",
    "enriscó": "enriscar",
    "enrisque": "enriscar",
    "enrisquemos": "enriscar",
    "enrisquen": "enriscar",
    "enrisques": "enriscar",
    "enrisqué": "enriscar",
    "enrisquéis": "enriscar",
    "enristra": "enristrar",
    "enristraba": "enristrar",
    "enristrabais": "enristrar",
    "enristraban": "enristrar",
    "enristrabas": "enristrar",
    "enristrad": "enristrar",
    "enristrada": "enristrar",
    "enristradas": "enristrar",
    "enristrado": "enristrar",
    "enristrados": "enristrar",
    "enristramos": "enristrar",
    "enristran": "enristrar",
    "enristrando": "enristrar",
    "enristrara": "enristrar",
    "enristrarais": "enristrar",
    "enristraran": "enristrar",
    "enristraras": "enristrar",
    "enristrare": "enristrar",
    "enristrareis": "enristrar",
    "enristraremos": "enristrar",
    "enristraren": "enristrar",
    "enristrares": "enristrar",
    "enristraron": "enristrar",
    "enristrará": "enristrar",
    "enristrarán": "enristrar",
    "enristrarás": "enristrar",
    "enristraré": "enristrar",
    "enristraréis": "enristrar",
    "enristraría": "enristrar",
    "enristraríais": "enristrar",
    "enristraríamos": "enristrar",
    "enristrarían": "enristrar",
    "enristrarías": "enristrar",
    "enristras": "enristrar",
    "enristrase": "enristrar",
    "enristraseis": "enristrar",
    "enristrasen": "enristrar",
    "enristrases": "enristrar",
    "enristraste": "enristrar",
    "enristrasteis": "enristrar",
    "enristre": "enristrar",
    "enristremos": "enristrar",
    "enristren": "enristrar",
    "enristres": "enristre",
    "enristro": "enristrar",
    "enristrábamos": "enristrar",
    "enristráis": "enristrar",
    "enristráramos": "enristrar",
    "enristráremos": "enristrar",
    "enristrásemos": "enristrar",
    "enristré": "enristrar",
    "enristréis": "enristrar",
    "enristró": "enristrar",
    "enriábamos": "enriar",
    "enriáis": "enriar",
    "enriáramos": "enriar",
    "enriáremos": "enriar",
    "enriásemos": "enriar",
    "enrié": "enriar",
    "enriéis": "enriar",
    "enrió": "enriar",
    "enroca": "enrocar",
    "enrocaba": "enrocar",
    "enrocabais": "enrocar",
    "enrocaban": "enrocar",
    "enrocabas": "enrocar",
    "enrocad": "enrocar",
    "enrocada": "enrocar",
    "enrocadas": "enrocar",
    "enrocado": "enrocar",
    "enrocados": "enrocar",
    "enrocamos": "enrocar",
    "enrocan": "enrocar",
    "enrocando": "enrocar",
    "enrocara": "enrocar",
    "enrocarais": "enrocar",
    "enrocaran": "enrocar",
    "enrocaras": "enrocar",
    "enrocare": "enrocar",
    "enrocareis": "enrocar",
    "enrocaremos": "enrocar",
    "enrocaren": "enrocar",
    "enrocares": "enrocar",
    "enrocaron": "enrocar",
    "enrocará": "enrocar",
    "enrocarán": "enrocar",
    "enrocarás": "enrocar",
    "enrocaré": "enrocar",
    "enrocaréis": "enrocar",
    "enrocaría": "enrocar",
    "enrocaríais": "enrocar",
    "enrocaríamos": "enrocar",
    "enrocarían": "enrocar",
    "enrocarías": "enrocar",
    "enrocas": "enrocar",
    "enrocase": "enrocar",
    "enrocaseis": "enrocar",
    "enrocasen": "enrocar",
    "enrocases": "enrocar",
    "enrocaste": "enrocar",
    "enrocasteis": "enrocar",
    "enroco": "enrocar",
    "enrocábamos": "enrocar",
    "enrocáis": "enrocar",
    "enrocáramos": "enrocar",
    "enrocáremos": "enrocar",
    "enrocásemos": "enrocar",
    "enrocó": "enrocar",
    "enrodaba": "enrodar",
    "enrodabais": "enrodar",
    "enrodaban": "enrodar",
    "enrodabas": "enrodar",
    "enrodad": "enrodar",
    "enrodada": "enrodar",
    "enrodadas": "enrodar",
    "enrodado": "enrodar",
    "enrodados": "enrodar",
    "enrodamos": "enrodar",
    "enrodando": "enrodar",
    "enrodara": "enrodar",
    "enrodarais": "enrodar",
    "enrodaran": "enrodar",
    "enrodaras": "enrodar",
    "enrodare": "enrodar",
    "enrodareis": "enrodar",
    "enrodaremos": "enrodar",
    "enrodaren": "enrodar",
    "enrodares": "enrodar",
    "enrodaron": "enrodar",
    "enrodará": "enrodar",
    "enrodarán": "enrodar",
    "enrodarás": "enrodar",
    "enrodaré": "enrodar",
    "enrodaréis": "enrodar",
    "enrodaría": "enrodar",
    "enrodaríais": "enrodar",
    "enrodaríamos": "enrodar",
    "enrodarían": "enrodar",
    "enrodarías": "enrodar",
    "enrodase": "enrodar",
    "enrodaseis": "enrodar",
    "enrodasen": "enrodar",
    "enrodases": "enrodar",
    "enrodaste": "enrodar",
    "enrodasteis": "enrodar",
    "enrodelada": "enrodelado",
    "enrodeladas": "enrodelado",
    "enrodelados": "enrodelado",
    "enrodemos": "enrodar",
    "enrodriga": "enrodrigar",
    "enrodrigaba": "enrodrigar",
    "enrodrigabais": "enrodrigar",
    "enrodrigaban": "enrodrigar",
    "enrodrigabas": "enrodrigar",
    "enrodrigad": "enrodrigar",
    "enrodrigada": "enrodrigar",
    "enrodrigadas": "enrodrigar",
    "enrodrigado": "enrodrigar",
    "enrodrigados": "enrodrigar",
    "enrodrigamos": "enrodrigar",
    "enrodrigan": "enrodrigar",
    "enrodrigando": "enrodrigar",
    "enrodrigara": "enrodrigar",
    "enrodrigarais": "enrodrigar",
    "enrodrigaran": "enrodrigar",
    "enrodrigaras": "enrodrigar",
    "enrodrigare": "enrodrigar",
    "enrodrigareis": "enrodrigar",
    "enrodrigaremos": "enrodrigar",
    "enrodrigaren": "enrodrigar",
    "enrodrigares": "enrodrigar",
    "enrodrigaron": "enrodrigar",
    "enrodrigará": "enrodrigar",
    "enrodrigarán": "enrodrigar",
    "enrodrigarás": "enrodrigar",
    "enrodrigaré": "enrodrigar",
    "enrodrigaréis": "enrodrigar",
    "enrodrigaría": "enrodrigar",
    "enrodrigaríais": "enrodrigar",
    "enrodrigaríamos": "enrodrigar",
    "enrodrigarían": "enrodrigar",
    "enrodrigarías": "enrodrigar",
    "enrodrigas": "enrodrigar",
    "enrodrigase": "enrodrigar",
    "enrodrigaseis": "enrodrigar",
    "enrodrigasen": "enrodrigar",
    "enrodrigases": "enrodrigar",
    "enrodrigaste": "enrodrigar",
    "enrodrigasteis": "enrodrigar",
    "enrodrigo": "enrodrigar",
    "enrodrigona": "enrodrigonar",
    "enrodrigonaba": "enrodrigonar",
    "enrodrigonabais": "enrodrigonar",
    "enrodrigonaban": "enrodrigonar",
    "enrodrigonabas": "enrodrigonar",
    "enrodrigonad": "enrodrigonar",
    "enrodrigonada": "enrodrigonar",
    "enrodrigonadas": "enrodrigonar",
    "enrodrigonado": "enrodrigonar",
    "enrodrigonados": "enrodrigonar",
    "enrodrigonamos": "enrodrigonar",
    "enrodrigonan": "enrodrigonar",
    "enrodrigonando": "enrodrigonar",
    "enrodrigonara": "enrodrigonar",
    "enrodrigonarais": "enrodrigonar",
    "enrodrigonaran": "enrodrigonar",
    "enrodrigonaras": "enrodrigonar",
    "enrodrigonare": "enrodrigonar",
    "enrodrigonareis": "enrodrigonar",
    "enrodrigonaremos": "enrodrigonar",
    "enrodrigonaren": "enrodrigonar",
    "enrodrigonares": "enrodrigonar",
    "enrodrigonaron": "enrodrigonar",
    "enrodrigonará": "enrodrigonar",
    "enrodrigonarán": "enrodrigonar",
    "enrodrigonarás": "enrodrigonar",
    "enrodrigonaré": "enrodrigonar",
    "enrodrigonaréis": "enrodrigonar",
    "enrodrigonaría": "enrodrigonar",
    "enrodrigonaríais": "enrodrigonar",
    "enrodrigonaríamos": "enrodrigonar",
    "enrodrigonarían": "enrodrigonar",
    "enrodrigonarías": "enrodrigonar",
    "enrodrigonas": "enrodrigonar",
    "enrodrigonase": "enrodrigonar",
    "enrodrigonaseis": "enrodrigonar",
    "enrodrigonasen": "enrodrigonar",
    "enrodrigonases": "enrodrigonar",
    "enrodrigonaste": "enrodrigonar",
    "enrodrigonasteis": "enrodrigonar",
    "enrodrigone": "enrodrigonar",
    "enrodrigonemos": "enrodrigonar",
    "enrodrigonen": "enrodrigonar",
    "enrodrigones": "enrodrigonar",
    "enrodrigono": "enrodrigonar",
    "enrodrigonábamos": "enrodrigonar",
    "enrodrigonáis": "enrodrigonar",
    "enrodrigonáramos": "enrodrigonar",
    "enrodrigonáremos": "enrodrigonar",
    "enrodrigonásemos": "enrodrigonar",
    "enrodrigoné": "enrodrigonar",
    "enrodrigonéis": "enrodrigonar",
    "enrodrigonó": "enrodrigonar",
    "enrodrigue": "enrodrigar",
    "enrodriguemos": "enrodrigar",
    "enrodriguen": "enrodrigar",
    "enrodrigues": "enrodrigar",
    "enrodrigué": "enrodrigar",
    "enrodriguéis": "enrodrigar",
    "enrodrigábamos": "enrodrigar",
    "enrodrigáis": "enrodrigar",
    "enrodrigáramos": "enrodrigar",
    "enrodrigáremos": "enrodrigar",
    "enrodrigásemos": "enrodrigar",
    "enrodrigó": "enrodrigar",
    "enrodábamos": "enrodar",
    "enrodáis": "enrodar",
    "enrodáramos": "enrodar",
    "enrodáremos": "enrodar",
    "enrodásemos": "enrodar",
    "enrodé": "enrodar",
    "enrodéis": "enrodar",
    "enrodó": "enrodar",
    "enrojece": "enrojecer",
    "enrojeced": "enrojecer",
    "enrojecemos": "enrojecer",
    "enrojecen": "enrojecer",
    "enrojeceremos": "enrojecer",
    "enrojecerá": "enrojecer",
    "enrojecerán": "enrojecer",
    "enrojecerás": "enrojecer",
    "enrojeceré": "enrojecer",
    "enrojeceréis": "enrojecer",
    "enrojecería": "enrojecer",
    "enrojeceríais": "enrojecer",
    "enrojeceríamos": "enrojecer",
    "enrojecerían": "enrojecer",
    "enrojecerías": "enrojecer",
    "enrojeces": "enrojecer",
    "enrojecida": "enrojecer",
    "enrojecidas": "enrojecer",
    "enrojecido": "enrojecer",
    "enrojecidos": "enrojecer",
    "enrojeciendo": "enrojecer",
    "enrojeciera": "enrojecer",
    "enrojecierais": "enrojecer",
    "enrojecieran": "enrojecer",
    "enrojecieras": "enrojecer",
    "enrojeciere": "enrojecer",
    "enrojeciereis": "enrojecer",
    "enrojecieren": "enrojecer",
    "enrojecieres": "enrojecer",
    "enrojecieron": "enrojecer",
    "enrojeciese": "enrojecer",
    "enrojecieseis": "enrojecer",
    "enrojeciesen": "enrojecer",
    "enrojecieses": "enrojecer",
    "enrojecimientos": "enrojecimiento",
    "enrojecimos": "enrojecer",
    "enrojeciste": "enrojecer",
    "enrojecisteis": "enrojecer",
    "enrojeciéramos": "enrojecer",
    "enrojeciéremos": "enrojecer",
    "enrojeciésemos": "enrojecer",
    "enrojeció": "enrojecer",
    "enrojecéis": "enrojecer",
    "enrojecí": "enrojecer",
    "enrojecía": "enrojecer",
    "enrojecíais": "enrojecer",
    "enrojecíamos": "enrojecer",
    "enrojecían": "enrojecer",
    "enrojecías": "enrojecer",
    "enrojezca": "enrojecer",
    "enrojezcamos": "enrojecer",
    "enrojezcan": "enrojecer",
    "enrojezcas": "enrojecer",
    "enrojezco": "enrojecer",
    "enrojezcáis": "enrojecer",
    "enrola": "enrolar",
    "enrolaba": "enrolar",
    "enrolabais": "enrolar",
    "enrolaban": "enrolar",
    "enrolabas": "enrolar",
    "enrolad": "enrolar",
    "enrolada": "enrolar",
    "enroladas": "enrolar",
    "enrolado": "enrolar",
    "enrolados": "enrolar",
    "enrolamientos": "enrolamiento",
    "enrolamos": "enrolar",
    "enrolan": "enrolar",
    "enrolando": "enrolar",
    "enrolara": "enrolar",
    "enrolarais": "enrolar",
    "enrolaran": "enrolar",
    "enrolaras": "enrolar",
    "enrolare": "enrolar",
    "enrolareis": "enrolar",
    "enrolaremos": "enrolar",
    "enrolaren": "enrolar",
    "enrolares": "enrolar",
    "enrolaron": "enrolar",
    "enrolará": "enrolar",
    "enrolarán": "enrolar",
    "enrolarás": "enrolar",
    "enrolaré": "enrolar",
    "enrolaréis": "enrolar",
    "enrolaría": "enrolar",
    "enrolaríais": "enrolar",
    "enrolaríamos": "enrolar",
    "enrolarían": "enrolar",
    "enrolarías": "enrolar",
    "enrolas": "enrolar",
    "enrolase": "enrolar",
    "enrolaseis": "enrolar",
    "enrolasen": "enrolar",
    "enrolases": "enrolar",
    "enrolaste": "enrolar",
    "enrolasteis": "enrolar",
    "enrole": "enrolar",
    "enrolemos": "enrolar",
    "enrolen": "enrolar",
    "enroles": "enrolar",
    "enrolla": "enrollar",
    "enrollaba": "enrollar",
    "enrollabais": "enrollar",
    "enrollaban": "enrollar",
    "enrollabas": "enrollar",
    "enrollables": "enrollable",
    "enrollad": "enrollar",
    "enrollada": "enrollar",
    "enrolladas": "enrollar",
    "enrollado": "enrollar",
    "enrolladora": "enrollador",
    "enrolladoras": "enrollador",
    "enrolladores": "enrollador",
    "enrollados": "enrollar",
    "enrollamientos": "enrollamiento",
    "enrollamos": "enrollar",
    "enrollan": "enrollar",
    "enrollando": "enrollar",
    "enrollara": "enrollar",
    "enrollarais": "enrollar",
    "enrollaran": "enrollar",
    "enrollaras": "enrollar",
    "enrollare": "enrollar",
    "enrollareis": "enrollar",
    "enrollaremos": "enrollar",
    "enrollaren": "enrollar",
    "enrollares": "enrollar",
    "enrollaron": "enrollar",
    "enrollará": "enrollar",
    "enrollarán": "enrollar",
    "enrollarás": "enrollar",
    "enrollaré": "enrollar",
    "enrollaréis": "enrollar",
    "enrollaría": "enrollar",
    "enrollaríais": "enrollar",
    "enrollaríamos": "enrollar",
    "enrollarían": "enrollar",
    "enrollarías": "enrollar",
    "enrollas": "enrollar",
    "enrollase": "enrollar",
    "enrollaseis": "enrollar",
    "enrollasen": "enrollar",
    "enrollases": "enrollar",
    "enrollaste": "enrollar",
    "enrollasteis": "enrollar",
    "enrolle": "enrollar",
    "enrollemos": "enrollar",
    "enrollen": "enrollar",
    "enrolles": "enrollar",
    "enrollo": "enrollar",
    "enrollábamos": "enrollar",
    "enrolláis": "enrollar",
    "enrolláramos": "enrollar",
    "enrolláremos": "enrollar",
    "enrollásemos": "enrollar",
    "enrollé": "enrollar",
    "enrolléis": "enrollar",
    "enrolló": "enrollar",
    "enrolo": "enrolar",
    "enrolábamos": "enrolar",
    "enroláis": "enrolar",
    "enroláramos": "enrolar",
    "enroláremos": "enrolar",
    "enrolásemos": "enrolar",
    "enrolé": "enrolar",
    "enroléis": "enrolar",
    "enroló": "enrolar",
    "enroma": "enromar",
    "enromaba": "enromar",
    "enromabais": "enromar",
    "enromaban": "enromar",
    "enromabas": "enromar",
    "enromad": "enromar",
    "enromada": "enromar",
    "enromadas": "enromar",
    "enromado": "enromar",
    "enromados": "enromar",
    "enromamos": "enromar",
    "enroman": "enromar",
    "enromando": "enromar",
    "enromara": "enromar",
    "enromarais": "enromar",
    "enromaran": "enromar",
    "enromaras": "enromar",
    "enromare": "enromar",
    "enromareis": "enromar",
    "enromaremos": "enromar",
    "enromaren": "enromar",
    "enromares": "enromar",
    "enromaron": "enromar",
    "enromará": "enromar",
    "enromarán": "enromar",
    "enromarás": "enromar",
    "enromaré": "enromar",
    "enromaréis": "enromar",
    "enromaría": "enromar",
    "enromaríais": "enromar",
    "enromaríamos": "enromar",
    "enromarían": "enromar",
    "enromarías": "enromar",
    "enromas": "enromar",
    "enromase": "enromar",
    "enromaseis": "enromar",
    "enromasen": "enromar",
    "enromases": "enromar",
    "enromaste": "enromar",
    "enromasteis": "enromar",
    "enrome": "enromar",
    "enromemos": "enromar",
    "enromen": "enromar",
    "enromes": "enromar",
    "enromo": "enromar",
    "enromábamos": "enromar",
    "enromáis": "enromar",
    "enromáramos": "enromar",
    "enromáremos": "enromar",
    "enromásemos": "enromar",
    "enromé": "enromar",
    "enroméis": "enromar",
    "enromó": "enromar",
    "enronquece": "enronquecer",
    "enronqueced": "enronquecer",
    "enronquecemos": "enronquecer",
    "enronquecen": "enronquecer",
    "enronqueceremos": "enronquecer",
    "enronquecerá": "enronquecer",
    "enronquecerán": "enronquecer",
    "enronquecerás": "enronquecer",
    "enronqueceré": "enronquecer",
    "enronqueceréis": "enronquecer",
    "enronquecería": "enronquecer",
    "enronqueceríais": "enronquecer",
    "enronqueceríamos": "enronquecer",
    "enronquecerían": "enronquecer",
    "enronquecerías": "enronquecer",
    "enronqueces": "enronquecer",
    "enronquecida": "enronquecer",
    "enronquecidas": "enronquecer",
    "enronquecido": "enronquecer",
    "enronquecidos": "enronquecer",
    "enronqueciendo": "enronquecer",
    "enronqueciera": "enronquecer",
    "enronquecierais": "enronquecer",
    "enronquecieran": "enronquecer",
    "enronquecieras": "enronquecer",
    "enronqueciere": "enronquecer",
    "enronqueciereis": "enronquecer",
    "enronquecieren": "enronquecer",
    "enronquecieres": "enronquecer",
    "enronquecieron": "enronquecer",
    "enronqueciese": "enronquecer",
    "enronquecieseis": "enronquecer",
    "enronqueciesen": "enronquecer",
    "enronquecieses": "enronquecer",
    "enronquecimientos": "enronquecimiento",
    "enronquecimos": "enronquecer",
    "enronqueciste": "enronquecer",
    "enronquecisteis": "enronquecer",
    "enronqueciéramos": "enronquecer",
    "enronqueciéremos": "enronquecer",
    "enronqueciésemos": "enronquecer",
    "enronqueció": "enronquecer",
    "enronquecéis": "enronquecer",
    "enronquecí": "enronquecer",
    "enronquecía": "enronquecer",
    "enronquecíais": "enronquecer",
    "enronquecíamos": "enronquecer",
    "enronquecían": "enronquecer",
    "enronquecías": "enronquecer",
    "enronquezca": "enronquecer",
    "enronquezcamos": "enronquecer",
    "enronquezcan": "enronquecer",
    "enronquezcas": "enronquecer",
    "enronquezco": "enronquecer",
    "enronquezcáis": "enronquecer",
    "enroque": "enrocar",
    "enroquemos": "enrocar",
    "enroquen": "enrocar",
    "enroques": "enroque",
    "enroqué": "enrocar",
    "enroquéis": "enrocar",
    "enrosca": "enroscar",
    "enroscaba": "enroscar",
    "enroscabais": "enroscar",
    "enroscaban": "enroscar",
    "enroscabas": "enroscar",
    "enroscad": "enroscar",
    "enroscada": "enroscar",
    "enroscadas": "enroscar",
    "enroscado": "enroscar",
    "enroscados": "enroscar",
    "enroscaduras": "enroscadura",
    "enroscamientos": "enroscamiento",
    "enroscamos": "enroscar",
    "enroscan": "enroscar",
    "enroscando": "enroscar",
    "enroscara": "enroscar",
    "enroscarais": "enroscar",
    "enroscaran": "enroscar",
    "enroscaras": "enroscar",
    "enroscare": "enroscar",
    "enroscareis": "enroscar",
    "enroscaremos": "enroscar",
    "enroscaren": "enroscar",
    "enroscares": "enroscar",
    "enroscaron": "enroscar",
    "enroscará": "enroscar",
    "enroscarán": "enroscar",
    "enroscarás": "enroscar",
    "enroscaré": "enroscar",
    "enroscaréis": "enroscar",
    "enroscaría": "enroscar",
    "enroscaríais": "enroscar",
    "enroscaríamos": "enroscar",
    "enroscarían": "enroscar",
    "enroscarías": "enroscar",
    "enroscas": "enroscar",
    "enroscase": "enroscar",
    "enroscaseis": "enroscar",
    "enroscasen": "enroscar",
    "enroscases": "enroscar",
    "enroscaste": "enroscar",
    "enroscasteis": "enroscar",
    "enrosco": "enroscar",
    "enroscábamos": "enroscar",
    "enroscáis": "enroscar",
    "enroscáramos": "enroscar",
    "enroscáremos": "enroscar",
    "enroscásemos": "enroscar",
    "enroscó": "enroscar",
    "enrosque": "enroscar",
    "enrosquemos": "enroscar",
    "enrosquen": "enroscar",
    "enrosques": "enroscar",
    "enrosqué": "enroscar",
    "enrosquéis": "enroscar",
    "enrostra": "enrostrar",
    "enrostraba": "enrostrar",
    "enrostrabais": "enrostrar",
    "enrostraban": "enrostrar",
    "enrostrabas": "enrostrar",
    "enrostrad": "enrostrar",
    "enrostrada": "enrostrar",
    "enrostradas": "enrostrar",
    "enrostrado": "enrostrar",
    "enrostrados": "enrostrar",
    "enrostramos": "enrostrar",
    "enrostran": "enrostrar",
    "enrostrando": "enrostrar",
    "enrostrara": "enrostrar",
    "enrostrarais": "enrostrar",
    "enrostraran": "enrostrar",
    "enrostraras": "enrostrar",
    "enrostrare": "enrostrar",
    "enrostrareis": "enrostrar",
    "enrostraremos": "enrostrar",
    "enrostraren": "enrostrar",
    "enrostrares": "enrostrar",
    "enrostraron": "enrostrar",
    "enrostrará": "enrostrar",
    "enrostrarán": "enrostrar",
    "enrostrarás": "enrostrar",
    "enrostraré": "enrostrar",
    "enrostraréis": "enrostrar",
    "enrostraría": "enrostrar",
    "enrostraríais": "enrostrar",
    "enrostraríamos": "enrostrar",
    "enrostrarían": "enrostrar",
    "enrostrarías": "enrostrar",
    "enrostras": "enrostrar",
    "enrostrase": "enrostrar",
    "enrostraseis": "enrostrar",
    "enrostrasen": "enrostrar",
    "enrostrases": "enrostrar",
    "enrostraste": "enrostrar",
    "enrostrasteis": "enrostrar",
    "enrostre": "enrostrar",
    "enrostremos": "enrostrar",
    "enrostren": "enrostrar",
    "enrostres": "enrostrar",
    "enrostro": "enrostrar",
    "enrostrábamos": "enrostrar",
    "enrostráis": "enrostrar",
    "enrostráramos": "enrostrar",
    "enrostráremos": "enrostrar",
    "enrostrásemos": "enrostrar",
    "enrostré": "enrostrar",
    "enrostréis": "enrostrar",
    "enrostró": "enrostrar",
    "enrubia": "enrubiar",
    "enrubiaba": "enrubiar",
    "enrubiabais": "enrubiar",
    "enrubiaban": "enrubiar",
    "enrubiabas": "enrubiar",
    "enrubiad": "enrubiar",
    "enrubiada": "enrubiar",
    "enrubiadas": "enrubiar",
    "enrubiado": "enrubiar",
    "enrubiados": "enrubiar",
    "enrubiamos": "enrubiar",
    "enrubian": "enrubiar",
    "enrubiando": "enrubiar",
    "enrubiara": "enrubiar",
    "enrubiarais": "enrubiar",
    "enrubiaran": "enrubiar",
    "enrubiaras": "enrubiar",
    "enrubiare": "enrubiar",
    "enrubiareis": "enrubiar",
    "enrubiaremos": "enrubiar",
    "enrubiaren": "enrubiar",
    "enrubiares": "enrubiar",
    "enrubiaron": "enrubiar",
    "enrubiará": "enrubiar",
    "enrubiarán": "enrubiar",
    "enrubiarás": "enrubiar",
    "enrubiaré": "enrubiar",
    "enrubiaréis": "enrubiar",
    "enrubiaría": "enrubiar",
    "enrubiaríais": "enrubiar",
    "enrubiaríamos": "enrubiar",
    "enrubiarían": "enrubiar",
    "enrubiarías": "enrubiar",
    "enrubias": "enrubiar",
    "enrubiase": "enrubiar",
    "enrubiaseis": "enrubiar",
    "enrubiasen": "enrubiar",
    "enrubiases": "enrubiar",
    "enrubiaste": "enrubiar",
    "enrubiasteis": "enrubiar",
    "enrubie": "enrubiar",
    "enrubiemos": "enrubiar",
    "enrubien": "enrubiar",
    "enrubies": "enrubiar",
    "enrubio": "enrubiar",
    "enrubios": "enrubio",
    "enrubiábamos": "enrubiar",
    "enrubiáis": "enrubiar",
    "enrubiáramos": "enrubiar",
    "enrubiáremos": "enrubiar",
    "enrubiásemos": "enrubiar",
    "enrubié": "enrubiar",
    "enrubiéis": "enrubiar",
    "enrubió": "enrubiar",
    "enrudecimientos": "enrudecimiento",
    "enrueca": "enrocar",
    "enruecan": "enrocar",
    "enruecas": "enrocar",
    "enrueco": "enrocar",
    "enrueda": "enrodar",
    "enruedan": "enrodar",
    "enruedas": "enrodar",
    "enruede": "enrodar",
    "enrueden": "enrodar",
    "enruedes": "enrodar",
    "enruedo": "enrodar",
    "enrueque": "enrocar",
    "enruequen": "enrocar",
    "enrueques": "enrocar",
    "enrugas": "enruga",
    "enría": "enriar",
    "enrían": "enriar",
    "enrías": "enriar",
    "enríe": "enriar",
    "enríen": "enriar",
    "enríes": "enriar",
    "enrío": "enriar",
    "ensabana": "ensabanar",
    "ensabanaba": "ensabanar",
    "ensabanabais": "ensabanar",
    "ensabanaban": "ensabanar",
    "ensabanabas": "ensabanar",
    "ensabanad": "ensabanar",
    "ensabanada": "ensabanar",
    "ensabanadas": "ensabanar",
    "ensabanado": "ensabanar",
    "ensabanados": "ensabanar",
    "ensabanamos": "ensabanar",
    "ensabanan": "ensabanar",
    "ensabanando": "ensabanar",
    "ensabanara": "ensabanar",
    "ensabanarais": "ensabanar",
    "ensabanaran": "ensabanar",
    "ensabanaras": "ensabanar",
    "ensabanare": "ensabanar",
    "ensabanareis": "ensabanar",
    "ensabanaremos": "ensabanar",
    "ensabanaren": "ensabanar",
    "ensabanares": "ensabanar",
    "ensabanaron": "ensabanar",
    "ensabanará": "ensabanar",
    "ensabanarán": "ensabanar",
    "ensabanarás": "ensabanar",
    "ensabanaré": "ensabanar",
    "ensabanaréis": "ensabanar",
    "ensabanaría": "ensabanar",
    "ensabanaríais": "ensabanar",
    "ensabanaríamos": "ensabanar",
    "ensabanarían": "ensabanar",
    "ensabanarías": "ensabanar",
    "ensabanas": "ensabanar",
    "ensabanase": "ensabanar",
    "ensabanaseis": "ensabanar",
    "ensabanasen": "ensabanar",
    "ensabanases": "ensabanar",
    "ensabanaste": "ensabanar",
    "ensabanasteis": "ensabanar",
    "ensabane": "ensabanar",
    "ensabanemos": "ensabanar",
    "ensabanen": "ensabanar",
    "ensabanes": "ensabanar",
    "ensabano": "ensabanar",
    "ensabanábamos": "ensabanar",
    "ensabanáis": "ensabanar",
    "ensabanáramos": "ensabanar",
    "ensabanáremos": "ensabanar",
    "ensabanásemos": "ensabanar",
    "ensabané": "ensabanar",
    "ensabanéis": "ensabanar",
    "ensabanó": "ensabanar",
    "ensaca": "ensacar",
    "ensacaba": "ensacar",
    "ensacabais": "ensacar",
    "ensacaban": "ensacar",
    "ensacabas": "ensacar",
    "ensacad": "ensacar",
    "ensacada": "ensacar",
    "ensacadas": "ensacar",
    "ensacado": "ensacar",
    "ensacadora": "ensacador",
    "ensacadoras": "ensacadora",
    "ensacadores": "ensacador",
    "ensacados": "ensacar",
    "ensacamos": "ensacar",
    "ensacan": "ensacar",
    "ensacando": "ensacar",
    "ensacara": "ensacar",
    "ensacarais": "ensacar",
    "ensacaran": "ensacar",
    "ensacaras": "ensacar",
    "ensacare": "ensacar",
    "ensacareis": "ensacar",
    "ensacaremos": "ensacar",
    "ensacaren": "ensacar",
    "ensacares": "ensacar",
    "ensacaron": "ensacar",
    "ensacará": "ensacar",
    "ensacarán": "ensacar",
    "ensacarás": "ensacar",
    "ensacaré": "ensacar",
    "ensacaréis": "ensacar",
    "ensacaría": "ensacar",
    "ensacaríais": "ensacar",
    "ensacaríamos": "ensacar",
    "ensacarían": "ensacar",
    "ensacarías": "ensacar",
    "ensacas": "ensacar",
    "ensacase": "ensacar",
    "ensacaseis": "ensacar",
    "ensacasen": "ensacar",
    "ensacases": "ensacar",
    "ensacaste": "ensacar",
    "ensacasteis": "ensacar",
    "ensaco": "ensacar",
    "ensacábamos": "ensacar",
    "ensacáis": "ensacar",
    "ensacáramos": "ensacar",
    "ensacáremos": "ensacar",
    "ensacásemos": "ensacar",
    "ensacó": "ensacar",
    "ensaimadas": "ensaimada",
    "ensaladas": "ensalada",
    "ensaladeras": "ensaladera",
    "ensaladillas": "ensaladilla",
    "ensalce": "ensalzar",
    "ensalcemos": "ensalzar",
    "ensalcen": "ensalzar",
    "ensalces": "ensalzar",
    "ensalcé": "ensalzar",
    "ensalcéis": "ensalzar",
    "ensaliva": "ensalivar",
    "ensalivaba": "ensalivar",
    "ensalivabais": "ensalivar",
    "ensalivaban": "ensalivar",
    "ensalivabas": "ensalivar",
    "ensalivad": "ensalivar",
    "ensalivada": "ensalivar",
    "ensalivadas": "ensalivar",
    "ensalivado": "ensalivar",
    "ensalivados": "ensalivar",
    "ensalivamos": "ensalivar",
    "ensalivan": "ensalivar",
    "ensalivando": "ensalivar",
    "ensalivara": "ensalivar",
    "ensalivarais": "ensalivar",
    "ensalivaran": "ensalivar",
    "ensalivaras": "ensalivar",
    "ensalivare": "ensalivar",
    "ensalivareis": "ensalivar",
    "ensalivaremos": "ensalivar",
    "ensalivaren": "ensalivar",
    "ensalivares": "ensalivar",
    "ensalivaron": "ensalivar",
    "ensalivará": "ensalivar",
    "ensalivarán": "ensalivar",
    "ensalivarás": "ensalivar",
    "ensalivaré": "ensalivar",
    "ensalivaréis": "ensalivar",
    "ensalivaría": "ensalivar",
    "ensalivaríais": "ensalivar",
    "ensalivaríamos": "ensalivar",
    "ensalivarían": "ensalivar",
    "ensalivarías": "ensalivar",
    "ensalivas": "ensalivar",
    "ensalivase": "ensalivar",
    "ensalivaseis": "ensalivar",
    "ensalivasen": "ensalivar",
    "ensalivases": "ensalivar",
    "ensalivaste": "ensalivar",
    "ensalivasteis": "ensalivar",
    "ensalive": "ensalivar",
    "ensalivemos": "ensalivar",
    "ensaliven": "ensalivar",
    "ensalives": "ensalivar",
    "ensalivo": "ensalivar",
    "ensalivábamos": "ensalivar",
    "ensaliváis": "ensalivar",
    "ensaliváramos": "ensalivar",
    "ensaliváremos": "ensalivar",
    "ensalivásemos": "ensalivar",
    "ensalivé": "ensalivar",
    "ensalivéis": "ensalivar",
    "ensalivó": "ensalivar",
    "ensalma": "ensalmar",
    "ensalmaba": "ensalmar",
    "ensalmabais": "ensalmar",
    "ensalmaban": "ensalmar",
    "ensalmabas": "ensalmar",
    "ensalmad": "ensalmar",
    "ensalmada": "ensalmar",
    "ensalmadas": "ensalmar",
    "ensalmado": "ensalmar",
    "ensalmados": "ensalmar",
    "ensalmamos": "ensalmar",
    "ensalman": "ensalmar",
    "ensalmando": "ensalmar",
    "ensalmara": "ensalmar",
    "ensalmarais": "ensalmar",
    "ensalmaran": "ensalmar",
    "ensalmaras": "ensalmar",
    "ensalmare": "ensalmar",
    "ensalmareis": "ensalmar",
    "ensalmaremos": "ensalmar",
    "ensalmaren": "ensalmar",
    "ensalmares": "ensalmar",
    "ensalmaron": "ensalmar",
    "ensalmará": "ensalmar",
    "ensalmarán": "ensalmar",
    "ensalmarás": "ensalmar",
    "ensalmaré": "ensalmar",
    "ensalmaréis": "ensalmar",
    "ensalmaría": "ensalmar",
    "ensalmaríais": "ensalmar",
    "ensalmaríamos": "ensalmar",
    "ensalmarían": "ensalmar",
    "ensalmarías": "ensalmar",
    "ensalmas": "ensalmar",
    "ensalmase": "ensalmar",
    "ensalmaseis": "ensalmar",
    "ensalmasen": "ensalmar",
    "ensalmases": "ensalmar",
    "ensalmaste": "ensalmar",
    "ensalmasteis": "ensalmar",
    "ensalme": "ensalmar",
    "ensalmemos": "ensalmar",
    "ensalmen": "ensalmar",
    "ensalmes": "ensalmar",
    "ensalmo": "ensalmar",
    "ensalmos": "ensalmo",
    "ensalmábamos": "ensalmar",
    "ensalmáis": "ensalmar",
    "ensalmáramos": "ensalmar",
    "ensalmáremos": "ensalmar",
    "ensalmásemos": "ensalmar",
    "ensalmé": "ensalmar",
    "ensalméis": "ensalmar",
    "ensalmó": "ensalmar",
    "ensalza": "ensalzar",
    "ensalzaba": "ensalzar",
    "ensalzabais": "ensalzar",
    "ensalzaban": "ensalzar",
    "ensalzabas": "ensalzar",
    "ensalzad": "ensalzar",
    "ensalzada": "ensalzar",
    "ensalzadas": "ensalzar",
    "ensalzado": "ensalzar",
    "ensalzadora": "ensalzador",
    "ensalzadoras": "ensalzador",
    "ensalzadores": "ensalzador",
    "ensalzados": "ensalzar",
    "ensalzamientos": "ensalzamiento",
    "ensalzamos": "ensalzar",
    "ensalzan": "ensalzar",
    "ensalzando": "ensalzar",
    "ensalzara": "ensalzar",
    "ensalzarais": "ensalzar",
    "ensalzaran": "ensalzar",
    "ensalzaras": "ensalzar",
    "ensalzare": "ensalzar",
    "ensalzareis": "ensalzar",
    "ensalzaremos": "ensalzar",
    "ensalzaren": "ensalzar",
    "ensalzares": "ensalzar",
    "ensalzaron": "ensalzar",
    "ensalzará": "ensalzar",
    "ensalzarán": "ensalzar",
    "ensalzarás": "ensalzar",
    "ensalzaré": "ensalzar",
    "ensalzaréis": "ensalzar",
    "ensalzaría": "ensalzar",
    "ensalzaríais": "ensalzar",
    "ensalzaríamos": "ensalzar",
    "ensalzarían": "ensalzar",
    "ensalzarías": "ensalzar",
    "ensalzas": "ensalzar",
    "ensalzase": "ensalzar",
    "ensalzaseis": "ensalzar",
    "ensalzasen": "ensalzar",
    "ensalzases": "ensalzar",
    "ensalzaste": "ensalzar",
    "ensalzasteis": "ensalzar",
    "ensalzo": "ensalzar",
    "ensalzábamos": "ensalzar",
    "ensalzáis": "ensalzar",
    "ensalzáramos": "ensalzar",
    "ensalzáremos": "ensalzar",
    "ensalzásemos": "ensalzar",
    "ensalzó": "ensalzar",
    "ensambenita": "ensambenitar",
    "ensambenitaba": "ensambenitar",
    "ensambenitabais": "ensambenitar",
    "ensambenitaban": "ensambenitar",
    "ensambenitabas": "ensambenitar",
    "ensambenitad": "ensambenitar",
    "ensambenitada": "ensambenitar",
    "ensambenitadas": "ensambenitar",
    "ensambenitado": "ensambenitar",
    "ensambenitados": "ensambenitar",
    "ensambenitamos": "ensambenitar",
    "ensambenitan": "ensambenitar",
    "ensambenitando": "ensambenitar",
    "ensambenitara": "ensambenitar",
    "ensambenitarais": "ensambenitar",
    "ensambenitaran": "ensambenitar",
    "ensambenitaras": "ensambenitar",
    "ensambenitare": "ensambenitar",
    "ensambenitareis": "ensambenitar",
    "ensambenitaremos": "ensambenitar",
    "ensambenitaren": "ensambenitar",
    "ensambenitares": "ensambenitar",
    "ensambenitaron": "ensambenitar",
    "ensambenitará": "ensambenitar",
    "ensambenitarán": "ensambenitar",
    "ensambenitarás": "ensambenitar",
    "ensambenitaré": "ensambenitar",
    "ensambenitaréis": "ensambenitar",
    "ensambenitaría": "ensambenitar",
    "ensambenitaríais": "ensambenitar",
    "ensambenitaríamos": "ensambenitar",
    "ensambenitarían": "ensambenitar",
    "ensambenitarías": "ensambenitar",
    "ensambenitas": "ensambenitar",
    "ensambenitase": "ensambenitar",
    "ensambenitaseis": "ensambenitar",
    "ensambenitasen": "ensambenitar",
    "ensambenitases": "ensambenitar",
    "ensambenitaste": "ensambenitar",
    "ensambenitasteis": "ensambenitar",
    "ensambenite": "ensambenitar",
    "ensambenitemos": "ensambenitar",
    "ensambeniten": "ensambenitar",
    "ensambenites": "ensambenitar",
    "ensambenito": "ensambenitar",
    "ensambenitábamos": "ensambenitar",
    "ensambenitáis": "ensambenitar",
    "ensambenitáramos": "ensambenitar",
    "ensambenitáremos": "ensambenitar",
    "ensambenitásemos": "ensambenitar",
    "ensambenité": "ensambenitar",
    "ensambenitéis": "ensambenitar",
    "ensambenitó": "ensambenitar",
    "ensambla": "ensamblar",
    "ensamblaba": "ensamblar",
    "ensamblabais": "ensamblar",
    "ensamblaban": "ensamblar",
    "ensamblabas": "ensamblar",
    "ensamblad": "ensamblar",
    "ensamblada": "ensamblar",
    "ensambladas": "ensamblar",
    "ensamblado": "ensamblar",
    "ensambladora": "ensamblador",
    "ensambladoras": "ensamblador",
    "ensambladores": "ensamblador",
    "ensamblados": "ensamblar",
    "ensambladuras": "ensambladura",
    "ensamblajes": "ensamblaje",
    "ensamblamientos": "ensamblamiento",
    "ensamblamos": "ensamblar",
    "ensamblan": "ensamblar",
    "ensamblando": "ensamblar",
    "ensamblara": "ensamblar",
    "ensamblarais": "ensamblar",
    "ensamblaran": "ensamblar",
    "ensamblaras": "ensamblar",
    "ensamblare": "ensamblar",
    "ensamblareis": "ensamblar",
    "ensamblaremos": "ensamblar",
    "ensamblaren": "ensamblar",
    "ensamblares": "ensamblar",
    "ensamblaron": "ensamblar",
    "ensamblará": "ensamblar",
    "ensamblarán": "ensamblar",
    "ensamblarás": "ensamblar",
    "ensamblaré": "ensamblar",
    "ensamblaréis": "ensamblar",
    "ensamblaría": "ensamblar",
    "ensamblaríais": "ensamblar",
    "ensamblaríamos": "ensamblar",
    "ensamblarían": "ensamblar",
    "ensamblarías": "ensamblar",
    "ensamblas": "ensamblar",
    "ensamblase": "ensamblar",
    "ensamblaseis": "ensamblar",
    "ensamblasen": "ensamblar",
    "ensamblases": "ensamblar",
    "ensamblaste": "ensamblar",
    "ensamblasteis": "ensamblar",
    "ensamble": "ensamblar",
    "ensamblemos": "ensamblar",
    "ensamblen": "ensamblar",
    "ensambles": "ensamble",
    "ensamblo": "ensamblar",
    "ensamblábamos": "ensamblar",
    "ensambláis": "ensamblar",
    "ensambláramos": "ensamblar",
    "ensambláremos": "ensamblar",
    "ensamblásemos": "ensamblar",
    "ensamblé": "ensamblar",
    "ensambléis": "ensamblar",
    "ensambló": "ensamblar",
    "ensancha": "ensanchar",
    "ensanchaba": "ensanchar",
    "ensanchabais": "ensanchar",
    "ensanchaban": "ensanchar",
    "ensanchabas": "ensanchar",
    "ensanchad": "ensanchar",
    "ensanchada": "ensanchar",
    "ensanchadas": "ensanchar",
    "ensanchado": "ensanchar",
    "ensanchadora": "ensanchador",
    "ensanchadoras": "ensanchador",
    "ensanchadores": "ensanchador",
    "ensanchados": "ensanchar",
    "ensanchamientos": "ensanchamiento",
    "ensanchamos": "ensanchar",
    "ensanchan": "ensanchar",
    "ensanchando": "ensanchar",
    "ensanchara": "ensanchar",
    "ensancharais": "ensanchar",
    "ensancharan": "ensanchar",
    "ensancharas": "ensanchar",
    "ensanchare": "ensanchar",
    "ensanchareis": "ensanchar",
    "ensancharemos": "ensanchar",
    "ensancharen": "ensanchar",
    "ensanchares": "ensanchar",
    "ensancharon": "ensanchar",
    "ensanchará": "ensanchar",
    "ensancharán": "ensanchar",
    "ensancharás": "ensanchar",
    "ensancharé": "ensanchar",
    "ensancharéis": "ensanchar",
    "ensancharía": "ensanchar",
    "ensancharíais": "ensanchar",
    "ensancharíamos": "ensanchar",
    "ensancharían": "ensanchar",
    "ensancharías": "ensanchar",
    "ensanchas": "ensanchar",
    "ensanchase": "ensanchar",
    "ensanchaseis": "ensanchar",
    "ensanchasen": "ensanchar",
    "ensanchases": "ensanchar",
    "ensanchaste": "ensanchar",
    "ensanchasteis": "ensanchar",
    "ensanche": "ensanchar",
    "ensanchemos": "ensanchar",
    "ensanchen": "ensanchar",
    "ensanches": "ensanche",
    "ensancho": "ensanchar",
    "ensanchábamos": "ensanchar",
    "ensancháis": "ensanchar",
    "ensancháramos": "ensanchar",
    "ensancháremos": "ensanchar",
    "ensanchásemos": "ensanchar",
    "ensanché": "ensanchar",
    "ensanchéis": "ensanchar",
    "ensanchó": "ensanchar",
    "ensandece": "ensandecer",
    "ensandeced": "ensandecer",
    "ensandecemos": "ensandecer",
    "ensandecen": "ensandecer",
    "ensandeceremos": "ensandecer",
    "ensandecerá": "ensandecer",
    "ensandecerán": "ensandecer",
    "ensandecerás": "ensandecer",
    "ensandeceré": "ensandecer",
    "ensandeceréis": "ensandecer",
    "ensandecería": "ensandecer",
    "ensandeceríais": "ensandecer",
    "ensandeceríamos": "ensandecer",
    "ensandecerían": "ensandecer",
    "ensandecerías": "ensandecer",
    "ensandeces": "ensandecer",
    "ensandecida": "ensandecer",
    "ensandecidas": "ensandecer",
    "ensandecido": "ensandecer",
    "ensandecidos": "ensandecer",
    "ensandeciendo": "ensandecer",
    "ensandeciera": "ensandecer",
    "ensandecierais": "ensandecer",
    "ensandecieran": "ensandecer",
    "ensandecieras": "ensandecer",
    "ensandeciere": "ensandecer",
    "ensandeciereis": "ensandecer",
    "ensandecieren": "ensandecer",
    "ensandecieres": "ensandecer",
    "ensandecieron": "ensandecer",
    "ensandeciese": "ensandecer",
    "ensandecieseis": "ensandecer",
    "ensandeciesen": "ensandecer",
    "ensandecieses": "ensandecer",
    "ensandecimos": "ensandecer",
    "ensandeciste": "ensandecer",
    "ensandecisteis": "ensandecer",
    "ensandeciéramos": "ensandecer",
    "ensandeciéremos": "ensandecer",
    "ensandeciésemos": "ensandecer",
    "ensandeció": "ensandecer",
    "ensandecéis": "ensandecer",
    "ensandecí": "ensandecer",
    "ensandecía": "ensandecer",
    "ensandecíais": "ensandecer",
    "ensandecíamos": "ensandecer",
    "ensandecían": "ensandecer",
    "ensandecías": "ensandecer",
    "ensandezca": "ensandecer",
    "ensandezcamos": "ensandecer",
    "ensandezcan": "ensandecer",
    "ensandezcas": "ensandecer",
    "ensandezco": "ensandecer",
    "ensandezcáis": "ensandecer",
    "ensangrentaba": "ensangrentar",
    "ensangrentabais": "ensangrentar",
    "ensangrentaban": "ensangrentar",
    "ensangrentabas": "ensangrentar",
    "ensangrentad": "ensangrentar",
    "ensangrentada": "ensangrentar",
    "ensangrentadas": "ensangrentar",
    "ensangrentado": "ensangrentar",
    "ensangrentados": "ensangrentar",
    "ensangrentamientos": "ensangrentamiento",
    "ensangrentamos": "ensangrentar",
    "ensangrentando": "ensangrentar",
    "ensangrentara": "ensangrentar",
    "ensangrentarais": "ensangrentar",
    "ensangrentaran": "ensangrentar",
    "ensangrentaras": "ensangrentar",
    "ensangrentare": "ensangrentar",
    "ensangrentareis": "ensangrentar",
    "ensangrentaremos": "ensangrentar",
    "ensangrentaren": "ensangrentar",
    "ensangrentares": "ensangrentar",
    "ensangrentaron": "ensangrentar",
    "ensangrentará": "ensangrentar",
    "ensangrentarán": "ensangrentar",
    "ensangrentarás": "ensangrentar",
    "ensangrentaré": "ensangrentar",
    "ensangrentaréis": "ensangrentar",
    "ensangrentaría": "ensangrentar",
    "ensangrentaríais": "ensangrentar",
    "ensangrentaríamos": "ensangrentar",
    "ensangrentarían": "ensangrentar",
    "ensangrentarías": "ensangrentar",
    "ensangrentase": "ensangrentar",
    "ensangrentaseis": "ensangrentar",
    "ensangrentasen": "ensangrentar",
    "ensangrentases": "ensangrentar",
    "ensangrentaste": "ensangrentar",
    "ensangrentasteis": "ensangrentar",
    "ensangrentemos": "ensangrentar",
    "ensangrentábamos": "ensangrentar",
    "ensangrentáis": "ensangrentar",
    "ensangrentáramos": "ensangrentar",
    "ensangrentáremos": "ensangrentar",
    "ensangrentásemos": "ensangrentar",
    "ensangrenté": "ensangrentar",
    "ensangrentéis": "ensangrentar",
    "ensangrentó": "ensangrentar",
    "ensangrienta": "ensangrentar",
    "ensangrientan": "ensangrentar",
    "ensangrientas": "ensangrentar",
    "ensangriente": "ensangrentar",
    "ensangrienten": "ensangrentar",
    "ensangrientes": "ensangrentar",
    "ensangriento": "ensangrentar",
    "ensaque": "ensacar",
    "ensaquemos": "ensacar",
    "ensaquen": "ensacar",
    "ensaques": "ensacar",
    "ensaqué": "ensacar",
    "ensaquéis": "ensacar",
    "ensarmentaba": "ensarmentar",
    "ensarmentabais": "ensarmentar",
    "ensarmentaban": "ensarmentar",
    "ensarmentabas": "ensarmentar",
    "ensarmentad": "ensarmentar",
    "ensarmentada": "ensarmentar",
    "ensarmentadas": "ensarmentar",
    "ensarmentado": "ensarmentar",
    "ensarmentados": "ensarmentar",
    "ensarmentamos": "ensarmentar",
    "ensarmentando": "ensarmentar",
    "ensarmentara": "ensarmentar",
    "ensarmentarais": "ensarmentar",
    "ensarmentaran": "ensarmentar",
    "ensarmentaras": "ensarmentar",
    "ensarmentare": "ensarmentar",
    "ensarmentareis": "ensarmentar",
    "ensarmentaremos": "ensarmentar",
    "ensarmentaren": "ensarmentar",
    "ensarmentares": "ensarmentar",
    "ensarmentaron": "ensarmentar",
    "ensarmentará": "ensarmentar",
    "ensarmentarán": "ensarmentar",
    "ensarmentarás": "ensarmentar",
    "ensarmentaré": "ensarmentar",
    "ensarmentaréis": "ensarmentar",
    "ensarmentaría": "ensarmentar",
    "ensarmentaríais": "ensarmentar",
    "ensarmentaríamos": "ensarmentar",
    "ensarmentarían": "ensarmentar",
    "ensarmentarías": "ensarmentar",
    "ensarmentase": "ensarmentar",
    "ensarmentaseis": "ensarmentar",
    "ensarmentasen": "ensarmentar",
    "ensarmentases": "ensarmentar",
    "ensarmentaste": "ensarmentar",
    "ensarmentasteis": "ensarmentar",
    "ensarmentemos": "ensarmentar",
    "ensarmentábamos": "ensarmentar",
    "ensarmentáis": "ensarmentar",
    "ensarmentáramos": "ensarmentar",
    "ensarmentáremos": "ensarmentar",
    "ensarmentásemos": "ensarmentar",
    "ensarmenté": "ensarmentar",
    "ensarmentéis": "ensarmentar",
    "ensarmentó": "ensarmentar",
    "ensarmienta": "ensarmentar",
    "ensarmientan": "ensarmentar",
    "ensarmientas": "ensarmentar",
    "ensarmiente": "ensarmentar",
    "ensarmienten": "ensarmentar",
    "ensarmientes": "ensarmentar",
    "ensarmiento": "ensarmentar",
    "ensarta": "ensartar",
    "ensartaba": "ensartar",
    "ensartabais": "ensartar",
    "ensartaban": "ensartar",
    "ensartabas": "ensartar",
    "ensartad": "ensartar",
    "ensartada": "ensartar",
    "ensartadas": "ensartar",
    "ensartado": "ensartar",
    "ensartados": "ensartar",
    "ensartamos": "ensartar",
    "ensartan": "ensartar",
    "ensartando": "ensartar",
    "ensartara": "ensartar",
    "ensartarais": "ensartar",
    "ensartaran": "ensartar",
    "ensartaras": "ensartar",
    "ensartare": "ensartar",
    "ensartareis": "ensartar",
    "ensartaremos": "ensartar",
    "ensartaren": "ensartar",
    "ensartares": "ensartar",
    "ensartaron": "ensartar",
    "ensartará": "ensartar",
    "ensartarán": "ensartar",
    "ensartarás": "ensartar",
    "ensartaré": "ensartar",
    "ensartaréis": "ensartar",
    "ensartaría": "ensartar",
    "ensartaríais": "ensartar",
    "ensartaríamos": "ensartar",
    "ensartarían": "ensartar",
    "ensartarías": "ensartar",
    "ensartas": "ensartar",
    "ensartase": "ensartar",
    "ensartaseis": "ensartar",
    "ensartasen": "ensartar",
    "ensartases": "ensartar",
    "ensartaste": "ensartar",
    "ensartasteis": "ensartar",
    "ensarte": "ensartar",
    "ensartemos": "ensartar",
    "ensarten": "ensartar",
    "ensartes": "ensartar",
    "ensarto": "ensartar",
    "ensartábamos": "ensartar",
    "ensartáis": "ensartar",
    "ensartáramos": "ensartar",
    "ensartáremos": "ensartar",
    "ensartásemos": "ensartar",
    "ensarté": "ensartar",
    "ensartéis": "ensartar",
    "ensartó": "ensartar",
    "ensaya": "ensayar",
    "ensayaba": "ensayar",
    "ensayabais": "ensayar",
    "ensayaban": "ensayar",
    "ensayabas": "ensayar",
    "ensayad": "ensayar",
    "ensayada": "ensayar",
    "ensayadas": "ensayar",
    "ensayado": "ensayar",
    "ensayadores": "ensayador",
    "ensayados": "ensayar",
    "ensayamos": "ensayar",
    "ensayan": "ensayar",
    "ensayando": "ensayar",
    "ensayara": "ensayar",
    "ensayarais": "ensayar",
    "ensayaran": "ensayar",
    "ensayaras": "ensayar",
    "ensayare": "ensayar",
    "ensayareis": "ensayar",
    "ensayaremos": "ensayar",
    "ensayaren": "ensayar",
    "ensayares": "ensayar",
    "ensayaron": "ensayar",
    "ensayará": "ensayar",
    "ensayarán": "ensayar",
    "ensayarás": "ensayar",
    "ensayaré": "ensayar",
    "ensayaréis": "ensayar",
    "ensayaría": "ensayar",
    "ensayaríais": "ensayar",
    "ensayaríamos": "ensayar",
    "ensayarían": "ensayar",
    "ensayarías": "ensayar",
    "ensayas": "ensayar",
    "ensayase": "ensayar",
    "ensayaseis": "ensayar",
    "ensayasen": "ensayar",
    "ensayases": "ensayar",
    "ensayaste": "ensayar",
    "ensayasteis": "ensayar",
    "ensaye": "ensayar",
    "ensayemos": "ensayar",
    "ensayen": "ensayar",
    "ensayes": "ensaye",
    "ensayismos": "ensayismo",
    "ensayistas": "ensayista",
    "ensayo": "ensayar",
    "ensayos": "ensayo",
    "ensayábamos": "ensayar",
    "ensayáis": "ensayar",
    "ensayáramos": "ensayar",
    "ensayáremos": "ensayar",
    "ensayásemos": "ensayar",
    "ensayé": "ensayar",
    "ensayéis": "ensayar",
    "ensayística": "ensayístico",
    "ensayísticas": "ensayístico",
    "ensayísticos": "ensayístico",
    "ensayó": "ensayar",
    "ensaña": "ensañar",
    "ensañaba": "ensañar",
    "ensañabais": "ensañar",
    "ensañaban": "ensañar",
    "ensañabas": "ensañar",
    "ensañad": "ensañar",
    "ensañada": "ensañar",
    "ensañadas": "ensañar",
    "ensañado": "ensañar",
    "ensañados": "ensañar",
    "ensañamientos": "ensañamiento",
    "ensañamos": "ensañar",
    "ensañan": "ensañar",
    "ensañando": "ensañar",
    "ensañara": "ensañar",
    "ensañarais": "ensañar",
    "ensañaran": "ensañar",
    "ensañaras": "ensañar",
    "ensañare": "ensañar",
    "ensañareis": "ensañar",
    "ensañaremos": "ensañar",
    "ensañaren": "ensañar",
    "ensañares": "ensañar",
    "ensañaron": "ensañar",
    "ensañará": "ensañar",
    "ensañarán": "ensañar",
    "ensañarás": "ensañar",
    "ensañaré": "ensañar",
    "ensañaréis": "ensañar",
    "ensañaría": "ensañar",
    "ensañaríais": "ensañar",
    "ensañaríamos": "ensañar",
    "ensañarían": "ensañar",
    "ensañarías": "ensañar",
    "ensañas": "ensañar",
    "ensañase": "ensañar",
    "ensañaseis": "ensañar",
    "ensañasen": "ensañar",
    "ensañases": "ensañar",
    "ensañaste": "ensañar",
    "ensañasteis": "ensañar",
    "ensañe": "ensañar",
    "ensañemos": "ensañar",
    "ensañen": "ensañar",
    "ensañes": "ensañar",
    "ensaño": "ensañar",
    "ensañábamos": "ensañar",
    "ensañáis": "ensañar",
    "ensañáramos": "ensañar",
    "ensañáremos": "ensañar",
    "ensañásemos": "ensañar",
    "ensañé": "ensañar",
    "ensañéis": "ensañar",
    "ensañó": "ensañar",
    "enseba": "ensebar",
    "ensebaba": "ensebar",
    "ensebabais": "ensebar",
    "ensebaban": "ensebar",
    "ensebabas": "ensebar",
    "ensebad": "ensebar",
    "ensebada": "ensebar",
    "ensebadas": "ensebar",
    "ensebado": "ensebar",
    "ensebados": "ensebar",
    "ensebamos": "ensebar",
    "enseban": "ensebar",
    "ensebando": "ensebar",
    "ensebara": "ensebar",
    "ensebarais": "ensebar",
    "ensebaran": "ensebar",
    "ensebaras": "ensebar",
    "ensebare": "ensebar",
    "ensebareis": "ensebar",
    "ensebaremos": "ensebar",
    "ensebaren": "ensebar",
    "ensebares": "ensebar",
    "ensebaron": "ensebar",
    "ensebará": "ensebar",
    "ensebarán": "ensebar",
    "ensebarás": "ensebar",
    "ensebaré": "ensebar",
    "ensebaréis": "ensebar",
    "ensebaría": "ensebar",
    "ensebaríais": "ensebar",
    "ensebaríamos": "ensebar",
    "ensebarían": "ensebar",
    "ensebarías": "ensebar",
    "ensebas": "ensebar",
    "ensebase": "ensebar",
    "ensebaseis": "ensebar",
    "ensebasen": "ensebar",
    "ensebases": "ensebar",
    "ensebaste": "ensebar",
    "ensebasteis": "ensebar",
    "ensebe": "ensebar",
    "ensebemos": "ensebar",
    "enseben": "ensebar",
    "ensebes": "ensebar",
    "ensebo": "ensebar",
    "ensebábamos": "ensebar",
    "ensebáis": "ensebar",
    "ensebáramos": "ensebar",
    "ensebáremos": "ensebar",
    "ensebásemos": "ensebar",
    "ensebé": "ensebar",
    "ensebéis": "ensebar",
    "ensebó": "ensebar",
    "enselva": "enselvar",
    "enselvaba": "enselvar",
    "enselvabais": "enselvar",
    "enselvaban": "enselvar",
    "enselvabas": "enselvar",
    "enselvad": "enselvar",
    "enselvada": "enselvar",
    "enselvadas": "enselvar",
    "enselvado": "enselvar",
    "enselvados": "enselvar",
    "enselvamos": "enselvar",
    "enselvan": "enselvar",
    "enselvando": "enselvar",
    "enselvara": "enselvar",
    "enselvarais": "enselvar",
    "enselvaran": "enselvar",
    "enselvaras": "enselvar",
    "enselvare": "enselvar",
    "enselvareis": "enselvar",
    "enselvaremos": "enselvar",
    "enselvaren": "enselvar",
    "enselvares": "enselvar",
    "enselvaron": "enselvar",
    "enselvará": "enselvar",
    "enselvarán": "enselvar",
    "enselvarás": "enselvar",
    "enselvaré": "enselvar",
    "enselvaréis": "enselvar",
    "enselvaría": "enselvar",
    "enselvaríais": "enselvar",
    "enselvaríamos": "enselvar",
    "enselvarían": "enselvar",
    "enselvarías": "enselvar",
    "enselvas": "enselvar",
    "enselvase": "enselvar",
    "enselvaseis": "enselvar",
    "enselvasen": "enselvar",
    "enselvases": "enselvar",
    "enselvaste": "enselvar",
    "enselvasteis": "enselvar",
    "enselve": "enselvar",
    "enselvemos": "enselvar",
    "enselven": "enselvar",
    "enselves": "enselvar",
    "enselvo": "enselvar",
    "enselvábamos": "enselvar",
    "enselváis": "enselvar",
    "enselváramos": "enselvar",
    "enselváremos": "enselvar",
    "enselvásemos": "enselvar",
    "enselvé": "enselvar",
    "enselvéis": "enselvar",
    "enselvó": "enselvar",
    "ensena": "ensenar",
    "ensenaba": "ensenar",
    "ensenabais": "ensenar",
    "ensenaban": "ensenar",
    "ensenabas": "ensenar",
    "ensenad": "ensenar",
    "ensenada": "ensenar",
    "ensenadas": "ensenar",
    "ensenado": "ensenar",
    "ensenados": "ensenar",
    "ensenamos": "ensenar",
    "ensenan": "ensenar",
    "ensenando": "ensenar",
    "ensenara": "ensenar",
    "ensenarais": "ensenar",
    "ensenaran": "ensenar",
    "ensenaras": "ensenar",
    "ensenare": "ensenar",
    "ensenareis": "ensenar",
    "ensenaremos": "ensenar",
    "ensenaren": "ensenar",
    "ensenares": "ensenar",
    "ensenaron": "ensenar",
    "ensenará": "ensenar",
    "ensenarán": "ensenar",
    "ensenarás": "ensenar",
    "ensenaré": "ensenar",
    "ensenaréis": "ensenar",
    "ensenaría": "ensenar",
    "ensenaríais": "ensenar",
    "ensenaríamos": "ensenar",
    "ensenarían": "ensenar",
    "ensenarías": "ensenar",
    "ensenas": "ensenar",
    "ensenase": "ensenar",
    "ensenaseis": "ensenar",
    "ensenasen": "ensenar",
    "ensenases": "ensenar",
    "ensenaste": "ensenar",
    "ensenasteis": "ensenar",
    "ensene": "ensenar",
    "ensenemos": "ensenar",
    "ensenen": "ensenar",
    "ensenes": "ensenar",
    "enseno": "ensenar",
    "ensenábamos": "ensenar",
    "ensenáis": "ensenar",
    "ensenáramos": "ensenar",
    "ensenáremos": "ensenar",
    "ensenásemos": "ensenar",
    "ensené": "ensenar",
    "ensenéis": "ensenar",
    "ensenó": "ensenar",
    "ensera": "enserar",
    "enseraba": "enserar",
    "enserabais": "enserar",
    "enseraban": "enserar",
    "enserabas": "enserar",
    "enserad": "enserar",
    "enserada": "enserar",
    "enseradas": "enserar",
    "enserado": "enserar",
    "enserados": "enserar",
    "enseramos": "enserar",
    "enseran": "enserar",
    "enserando": "enserar",
    "enserara": "enserar",
    "enserarais": "enserar",
    "enseraran": "enserar",
    "enseraras": "enserar",
    "enserare": "enserar",
    "enserareis": "enserar",
    "enseraremos": "enserar",
    "enseraren": "enserar",
    "enserares": "enserar",
    "enseraron": "enserar",
    "enserará": "enserar",
    "enserarán": "enserar",
    "enserarás": "enserar",
    "enseraré": "enserar",
    "enseraréis": "enserar",
    "enseraría": "enserar",
    "enseraríais": "enserar",
    "enseraríamos": "enserar",
    "enserarían": "enserar",
    "enserarías": "enserar",
    "enseras": "enserar",
    "enserase": "enserar",
    "enseraseis": "enserar",
    "enserasen": "enserar",
    "enserases": "enserar",
    "enseraste": "enserar",
    "enserasteis": "enserar",
    "ensere": "enserar",
    "enseremos": "enserar",
    "enseren": "enserar",
    "enseres": "enserar",
    "enseria": "enseriar",
    "enseriaba": "enseriar",
    "enseriabais": "enseriar",
    "enseriaban": "enseriar",
    "enseriabas": "enseriar",
    "enseriad": "enseriar",
    "enseriada": "enseriar",
    "enseriadas": "enseriar",
    "enseriado": "enseriar",
    "enseriados": "enseriar",
    "enseriamos": "enseriar",
    "enserian": "enseriar",
    "enseriando": "enseriar",
    "enseriara": "enseriar",
    "enseriarais": "enseriar",
    "enseriaran": "enseriar",
    "enseriaras": "enseriar",
    "enseriare": "enseriar",
    "enseriareis": "enseriar",
    "enseriaremos": "enseriar",
    "enseriaren": "enseriar",
    "enseriares": "enseriar",
    "enseriaron": "enseriar",
    "enseriará": "enseriar",
    "enseriarán": "enseriar",
    "enseriarás": "enseriar",
    "enseriaré": "enseriar",
    "enseriaréis": "enseriar",
    "enseriaría": "enseriar",
    "enseriaríais": "enseriar",
    "enseriaríamos": "enseriar",
    "enseriarían": "enseriar",
    "enseriarías": "enseriar",
    "enserias": "enseriar",
    "enseriase": "enseriar",
    "enseriaseis": "enseriar",
    "enseriasen": "enseriar",
    "enseriases": "enseriar",
    "enseriaste": "enseriar",
    "enseriasteis": "enseriar",
    "enserie": "enseriar",
    "enseriemos": "enseriar",
    "enserien": "enseriar",
    "enseries": "enseriar",
    "enserio": "enseriar",
    "enseriábamos": "enseriar",
    "enseriáis": "enseriar",
    "enseriáramos": "enseriar",
    "enseriáremos": "enseriar",
    "enseriásemos": "enseriar",
    "enserié": "enseriar",
    "enseriéis": "enseriar",
    "enserió": "enseriar",
    "ensero": "enserar",
    "enserábamos": "enserar",
    "enseráis": "enserar",
    "enseráramos": "enserar",
    "enseráremos": "enserar",
    "enserásemos": "enserar",
    "enseré": "enserar",
    "enseréis": "enserar",
    "enseró": "enserar",
    "enseña": "enseñar",
    "enseñaba": "enseñar",
    "enseñabais": "enseñar",
    "enseñaban": "enseñar",
    "enseñabas": "enseñar",
    "enseñad": "enseñar",
    "enseñada": "enseñar",
    "enseñadas": "enseñar",
    "enseñado": "enseñar",
    "enseñados": "enseñar",
    "enseñamos": "enseñar",
    "enseñan": "enseñar",
    "enseñando": "enseñar",
    "enseñantes": "enseñante",
    "enseñanzas": "enseñanza",
    "enseñara": "enseñar",
    "enseñarais": "enseñar",
    "enseñaran": "enseñar",
    "enseñaras": "enseñar",
    "enseñare": "enseñar",
    "enseñareis": "enseñar",
    "enseñaremos": "enseñar",
    "enseñaren": "enseñar",
    "enseñares": "enseñar",
    "enseñaron": "enseñar",
    "enseñará": "enseñar",
    "enseñarán": "enseñar",
    "enseñarás": "enseñar",
    "enseñaré": "enseñar",
    "enseñaréis": "enseñar",
    "enseñaría": "enseñar",
    "enseñaríais": "enseñar",
    "enseñaríamos": "enseñar",
    "enseñarían": "enseñar",
    "enseñarías": "enseñar",
    "enseñas": "enseñar",
    "enseñase": "enseñar",
    "enseñaseis": "enseñar",
    "enseñasen": "enseñar",
    "enseñases": "enseñar",
    "enseñaste": "enseñar",
    "enseñasteis": "enseñar",
    "enseñe": "enseñar",
    "enseñemos": "enseñar",
    "enseñen": "enseñar",
    "enseñes": "enseñar",
    "enseño": "enseñar",
    "enseñorea": "enseñorear",
    "enseñoreaba": "enseñorear",
    "enseñoreabais": "enseñorear",
    "enseñoreaban": "enseñorear",
    "enseñoreabas": "enseñorear",
    "enseñoread": "enseñorear",
    "enseñoreada": "enseñorear",
    "enseñoreadas": "enseñorear",
    "enseñoreado": "enseñorear",
    "enseñoreados": "enseñorear",
    "enseñoreamientos": "enseñoreamiento",
    "enseñoreamos": "enseñorear",
    "enseñorean": "enseñorear",
    "enseñoreando": "enseñorear",
    "enseñoreara": "enseñorear",
    "enseñorearais": "enseñorear",
    "enseñorearan": "enseñorear",
    "enseñorearas": "enseñorear",
    "enseñoreare": "enseñorear",
    "enseñoreareis": "enseñorear",
    "enseñorearemos": "enseñorear",
    "enseñorearen": "enseñorear",
    "enseñoreares": "enseñorear",
    "enseñorearon": "enseñorear",
    "enseñoreará": "enseñorear",
    "enseñorearán": "enseñorear",
    "enseñorearás": "enseñorear",
    "enseñorearé": "enseñorear",
    "enseñorearéis": "enseñorear",
    "enseñorearía": "enseñorear",
    "enseñorearíais": "enseñorear",
    "enseñorearíamos": "enseñorear",
    "enseñorearían": "enseñorear",
    "enseñorearías": "enseñorear",
    "enseñoreas": "enseñorear",
    "enseñorease": "enseñorear",
    "enseñoreaseis": "enseñorear",
    "enseñoreasen": "enseñorear",
    "enseñoreases": "enseñorear",
    "enseñoreaste": "enseñorear",
    "enseñoreasteis": "enseñorear",
    "enseñoree": "enseñorear",
    "enseñoreemos": "enseñorear",
    "enseñoreen": "enseñorear",
    "enseñorees": "enseñorear",
    "enseñoreo": "enseñorear",
    "enseñoreábamos": "enseñorear",
    "enseñoreáis": "enseñorear",
    "enseñoreáramos": "enseñorear",
    "enseñoreáremos": "enseñorear",
    "enseñoreásemos": "enseñorear",
    "enseñoreé": "enseñorear",
    "enseñoreéis": "enseñorear",
    "enseñoreó": "enseñorear",
    "enseñábamos": "enseñar",
    "enseñáis": "enseñar",
    "enseñáramos": "enseñar",
    "enseñáremos": "enseñar",
    "enseñásemos": "enseñar",
    "enseñé": "enseñar",
    "enseñéis": "enseñar",
    "enseñó": "enseñar",
    "ensiformes": "ensiforme",
    "ensila": "ensilar",
    "ensilaba": "ensilar",
    "ensilabais": "ensilar",
    "ensilaban": "ensilar",
    "ensilabas": "ensilar",
    "ensilad": "ensilar",
    "ensilada": "ensilar",
    "ensiladas": "ensilar",
    "ensilado": "ensilar",
    "ensiladoras": "ensiladora",
    "ensilados": "ensilar",
    "ensilajes": "ensilaje",
    "ensilamos": "ensilar",
    "ensilan": "ensilar",
    "ensilando": "ensilar",
    "ensilara": "ensilar",
    "ensilarais": "ensilar",
    "ensilaran": "ensilar",
    "ensilaras": "ensilar",
    "ensilare": "ensilar",
    "ensilareis": "ensilar",
    "ensilaremos": "ensilar",
    "ensilaren": "ensilar",
    "ensilares": "ensilar",
    "ensilaron": "ensilar",
    "ensilará": "ensilar",
    "ensilarán": "ensilar",
    "ensilarás": "ensilar",
    "ensilaré": "ensilar",
    "ensilaréis": "ensilar",
    "ensilaría": "ensilar",
    "ensilaríais": "ensilar",
    "ensilaríamos": "ensilar",
    "ensilarían": "ensilar",
    "ensilarías": "ensilar",
    "ensilas": "ensilar",
    "ensilase": "ensilar",
    "ensilaseis": "ensilar",
    "ensilasen": "ensilar",
    "ensilases": "ensilar",
    "ensilaste": "ensilar",
    "ensilasteis": "ensilar",
    "ensile": "ensilar",
    "ensilemos": "ensilar",
    "ensilen": "ensilar",
    "ensiles": "ensilar",
    "ensilla": "ensillar",
    "ensillaba": "ensillar",
    "ensillabais": "ensillar",
    "ensillaban": "ensillar",
    "ensillabas": "ensillar",
    "ensillad": "ensillar",
    "ensillada": "ensillar",
    "ensilladas": "ensillar",
    "ensillado": "ensillar",
    "ensillados": "ensillar",
    "ensilladuras": "ensilladura",
    "ensillamientos": "ensillamiento",
    "ensillamos": "ensillar",
    "ensillan": "ensillar",
    "ensillando": "ensillar",
    "ensillara": "ensillar",
    "ensillarais": "ensillar",
    "ensillaran": "ensillar",
    "ensillaras": "ensillar",
    "ensillare": "ensillar",
    "ensillareis": "ensillar",
    "ensillaremos": "ensillar",
    "ensillaren": "ensillar",
    "ensillares": "ensillar",
    "ensillaron": "ensillar",
    "ensillará": "ensillar",
    "ensillarán": "ensillar",
    "ensillarás": "ensillar",
    "ensillaré": "ensillar",
    "ensillaréis": "ensillar",
    "ensillaría": "ensillar",
    "ensillaríais": "ensillar",
    "ensillaríamos": "ensillar",
    "ensillarían": "ensillar",
    "ensillarías": "ensillar",
    "ensillas": "ensillar",
    "ensillase": "ensillar",
    "ensillaseis": "ensillar",
    "ensillasen": "ensillar",
    "ensillases": "ensillar",
    "ensillaste": "ensillar",
    "ensillasteis": "ensillar",
    "ensille": "ensillar",
    "ensillemos": "ensillar",
    "ensillen": "ensillar",
    "ensilles": "ensillar",
    "ensillo": "ensillar",
    "ensillábamos": "ensillar",
    "ensilláis": "ensillar",
    "ensilláramos": "ensillar",
    "ensilláremos": "ensillar",
    "ensillásemos": "ensillar",
    "ensillé": "ensillar",
    "ensilléis": "ensillar",
    "ensilló": "ensillar",
    "ensilo": "ensilar",
    "ensilvece": "ensilvecer",
    "ensilveced": "ensilvecer",
    "ensilvecemos": "ensilvecer",
    "ensilvecen": "ensilvecer",
    "ensilveceremos": "ensilvecer",
    "ensilvecerá": "ensilvecer",
    "ensilvecerán": "ensilvecer",
    "ensilvecerás": "ensilvecer",
    "ensilveceré": "ensilvecer",
    "ensilveceréis": "ensilvecer",
    "ensilvecería": "ensilvecer",
    "ensilveceríais": "ensilvecer",
    "ensilveceríamos": "ensilvecer",
    "ensilvecerían": "ensilvecer",
    "ensilvecerías": "ensilvecer",
    "ensilveces": "ensilvecer",
    "ensilvecida": "ensilvecer",
    "ensilvecidas": "ensilvecer",
    "ensilvecido": "ensilvecer",
    "ensilvecidos": "ensilvecer",
    "ensilveciendo": "ensilvecer",
    "ensilveciera": "ensilvecer",
    "ensilvecierais": "ensilvecer",
    "ensilvecieran": "ensilvecer",
    "ensilvecieras": "ensilvecer",
    "ensilveciere": "ensilvecer",
    "ensilveciereis": "ensilvecer",
    "ensilvecieren": "ensilvecer",
    "ensilvecieres": "ensilvecer",
    "ensilvecieron": "ensilvecer",
    "ensilveciese": "ensilvecer",
    "ensilvecieseis": "ensilvecer",
    "ensilveciesen": "ensilvecer",
    "ensilvecieses": "ensilvecer",
    "ensilvecimos": "ensilvecer",
    "ensilveciste": "ensilvecer",
    "ensilvecisteis": "ensilvecer",
    "ensilveciéramos": "ensilvecer",
    "ensilveciéremos": "ensilvecer",
    "ensilveciésemos": "ensilvecer",
    "ensilveció": "ensilvecer",
    "ensilvecéis": "ensilvecer",
    "ensilvecí": "ensilvecer",
    "ensilvecía": "ensilvecer",
    "ensilvecíais": "ensilvecer",
    "ensilvecíamos": "ensilvecer",
    "ensilvecían": "ensilvecer",
    "ensilvecías": "ensilvecer",
    "ensilvezca": "ensilvecer",
    "ensilvezcamos": "ensilvecer",
    "ensilvezcan": "ensilvecer",
    "ensilvezcas": "ensilvecer",
    "ensilvezco": "ensilvecer",
    "ensilvezcáis": "ensilvecer",
    "ensilábamos": "ensilar",
    "ensiláis": "ensilar",
    "ensiláramos": "ensilar",
    "ensiláremos": "ensilar",
    "ensilásemos": "ensilar",
    "ensilé": "ensilar",
    "ensiléis": "ensilar",
    "ensiló": "ensilar",
    "ensimajes": "ensimaje",
    "ensimisma": "ensimismar",
    "ensimismaba": "ensimismar",
    "ensimismabais": "ensimismar",
    "ensimismaban": "ensimismar",
    "ensimismabas": "ensimismar",
    "ensimismad": "ensimismar",
    "ensimismada": "ensimismar",
    "ensimismadas": "ensimismar",
    "ensimismado": "ensimismar",
    "ensimismados": "ensimismar",
    "ensimismamientos": "ensimismamiento",
    "ensimismamos": "ensimismar",
    "ensimisman": "ensimismar",
    "ensimismando": "ensimismar",
    "ensimismara": "ensimismar",
    "ensimismarais": "ensimismar",
    "ensimismaran": "ensimismar",
    "ensimismaras": "ensimismar",
    "ensimismare": "ensimismar",
    "ensimismareis": "ensimismar",
    "ensimismaremos": "ensimismar",
    "ensimismaren": "ensimismar",
    "ensimismares": "ensimismar",
    "ensimismaron": "ensimismar",
    "ensimismará": "ensimismar",
    "ensimismarán": "ensimismar",
    "ensimismarás": "ensimismar",
    "ensimismaré": "ensimismar",
    "ensimismaréis": "ensimismar",
    "ensimismaría": "ensimismar",
    "ensimismaríais": "ensimismar",
    "ensimismaríamos": "ensimismar",
    "ensimismarían": "ensimismar",
    "ensimismarías": "ensimismar",
    "ensimismas": "ensimismar",
    "ensimismase": "ensimismar",
    "ensimismaseis": "ensimismar",
    "ensimismasen": "ensimismar",
    "ensimismases": "ensimismar",
    "ensimismaste": "ensimismar",
    "ensimismasteis": "ensimismar",
    "ensimisme": "ensimismar",
    "ensimismemos": "ensimismar",
    "ensimismen": "ensimismar",
    "ensimismes": "ensimismar",
    "ensimismo": "ensimismar",
    "ensimismábamos": "ensimismar",
    "ensimismáis": "ensimismar",
    "ensimismáramos": "ensimismar",
    "ensimismáremos": "ensimismar",
    "ensimismásemos": "ensimismar",
    "ensimismé": "ensimismar",
    "ensimisméis": "ensimismar",
    "ensimismó": "ensimismar",
    "ensoberbece": "ensoberbecer",
    "ensoberbeced": "ensoberbecer",
    "ensoberbecemos": "ensoberbecer",
    "ensoberbecen": "ensoberbecer",
    "ensoberbeceremos": "ensoberbecer",
    "ensoberbecerá": "ensoberbecer",
    "ensoberbecerán": "ensoberbecer",
    "ensoberbecerás": "ensoberbecer",
    "ensoberbeceré": "ensoberbecer",
    "ensoberbeceréis": "ensoberbecer",
    "ensoberbecería": "ensoberbecer",
    "ensoberbeceríais": "ensoberbecer",
    "ensoberbeceríamos": "ensoberbecer",
    "ensoberbecerían": "ensoberbecer",
    "ensoberbecerías": "ensoberbecer",
    "ensoberbeces": "ensoberbecer",
    "ensoberbecida": "ensoberbecer",
    "ensoberbecidas": "ensoberbecer",
    "ensoberbecido": "ensoberbecer",
    "ensoberbecidos": "ensoberbecer",
    "ensoberbeciendo": "ensoberbecer",
    "ensoberbeciera": "ensoberbecer",
    "ensoberbecierais": "ensoberbecer",
    "ensoberbecieran": "ensoberbecer",
    "ensoberbecieras": "ensoberbecer",
    "ensoberbeciere": "ensoberbecer",
    "ensoberbeciereis": "ensoberbecer",
    "ensoberbecieren": "ensoberbecer",
    "ensoberbecieres": "ensoberbecer",
    "ensoberbecieron": "ensoberbecer",
    "ensoberbeciese": "ensoberbecer",
    "ensoberbecieseis": "ensoberbecer",
    "ensoberbeciesen": "ensoberbecer",
    "ensoberbecieses": "ensoberbecer",
    "ensoberbecimientos": "ensoberbecimiento",
    "ensoberbecimos": "ensoberbecer",
    "ensoberbeciste": "ensoberbecer",
    "ensoberbecisteis": "ensoberbecer",
    "ensoberbeciéramos": "ensoberbecer",
    "ensoberbeciéremos": "ensoberbecer",
    "ensoberbeciésemos": "ensoberbecer",
    "ensoberbeció": "ensoberbecer",
    "ensoberbecéis": "ensoberbecer",
    "ensoberbecí": "ensoberbecer",
    "ensoberbecía": "ensoberbecer",
    "ensoberbecíais": "ensoberbecer",
    "ensoberbecíamos": "ensoberbecer",
    "ensoberbecían": "ensoberbecer",
    "ensoberbecías": "ensoberbecer",
    "ensoberbezca": "ensoberbecer",
    "ensoberbezcamos": "ensoberbecer",
    "ensoberbezcan": "ensoberbecer",
    "ensoberbezcas": "ensoberbecer",
    "ensoberbezco": "ensoberbecer",
    "ensoberbezcáis": "ensoberbecer",
    "ensoga": "ensogar",
    "ensogaba": "ensogar",
    "ensogabais": "ensogar",
    "ensogaban": "ensogar",
    "ensogabas": "ensogar",
    "ensogad": "ensogar",
    "ensogada": "ensogar",
    "ensogadas": "ensogar",
    "ensogado": "ensogar",
    "ensogados": "ensogar",
    "ensogamos": "ensogar",
    "ensogan": "ensogar",
    "ensogando": "ensogar",
    "ensogara": "ensogar",
    "ensogarais": "ensogar",
    "ensogaran": "ensogar",
    "ensogaras": "ensogar",
    "ensogare": "ensogar",
    "ensogareis": "ensogar",
    "ensogaremos": "ensogar",
    "ensogaren": "ensogar",
    "ensogares": "ensogar",
    "ensogaron": "ensogar",
    "ensogará": "ensogar",
    "ensogarán": "ensogar",
    "ensogarás": "ensogar",
    "ensogaré": "ensogar",
    "ensogaréis": "ensogar",
    "ensogaría": "ensogar",
    "ensogaríais": "ensogar",
    "ensogaríamos": "ensogar",
    "ensogarían": "ensogar",
    "ensogarías": "ensogar",
    "ensogas": "ensogar",
    "ensogase": "ensogar",
    "ensogaseis": "ensogar",
    "ensogasen": "ensogar",
    "ensogases": "ensogar",
    "ensogaste": "ensogar",
    "ensogasteis": "ensogar",
    "ensogo": "ensogar",
    "ensogue": "ensogar",
    "ensoguemos": "ensogar",
    "ensoguen": "ensogar",
    "ensogues": "ensogar",
    "ensogué": "ensogar",
    "ensoguéis": "ensogar",
    "ensogábamos": "ensogar",
    "ensogáis": "ensogar",
    "ensogáramos": "ensogar",
    "ensogáremos": "ensogar",
    "ensogásemos": "ensogar",
    "ensogó": "ensogar",
    "ensombrece": "ensombrecer",
    "ensombreced": "ensombrecer",
    "ensombrecemos": "ensombrecer",
    "ensombrecen": "ensombrecer",
    "ensombreceremos": "ensombrecer",
    "ensombrecerá": "ensombrecer",
    "ensombrecerán": "ensombrecer",
    "ensombrecerás": "ensombrecer",
    "ensombreceré": "ensombrecer",
    "ensombreceréis": "ensombrecer",
    "ensombrecería": "ensombrecer",
    "ensombreceríais": "ensombrecer",
    "ensombreceríamos": "ensombrecer",
    "ensombrecerían": "ensombrecer",
    "ensombrecerías": "ensombrecer",
    "ensombreces": "ensombrecer",
    "ensombrecida": "ensombrecer",
    "ensombrecidas": "ensombrecer",
    "ensombrecido": "ensombrecer",
    "ensombrecidos": "ensombrecer",
    "ensombreciendo": "ensombrecer",
    "ensombreciera": "ensombrecer",
    "ensombrecierais": "ensombrecer",
    "ensombrecieran": "ensombrecer",
    "ensombrecieras": "ensombrecer",
    "ensombreciere": "ensombrecer",
    "ensombreciereis": "ensombrecer",
    "ensombrecieren": "ensombrecer",
    "ensombrecieres": "ensombrecer",
    "ensombrecieron": "ensombrecer",
    "ensombreciese": "ensombrecer",
    "ensombrecieseis": "ensombrecer",
    "ensombreciesen": "ensombrecer",
    "ensombrecieses": "ensombrecer",
    "ensombrecimos": "ensombrecer",
    "ensombreciste": "ensombrecer",
    "ensombrecisteis": "ensombrecer",
    "ensombreciéramos": "ensombrecer",
    "ensombreciéremos": "ensombrecer",
    "ensombreciésemos": "ensombrecer",
    "ensombreció": "ensombrecer",
    "ensombrecéis": "ensombrecer",
    "ensombrecí": "ensombrecer",
    "ensombrecía": "ensombrecer",
    "ensombrecíais": "ensombrecer",
    "ensombrecíamos": "ensombrecer",
    "ensombrecían": "ensombrecer",
    "ensombrecías": "ensombrecer",
    "ensombrerada": "ensombrerado",
    "ensombreradas": "ensombrerado",
    "ensombrerados": "ensombrerado",
    "ensombrezca": "ensombrecer",
    "ensombrezcamos": "ensombrecer",
    "ensombrezcan": "ensombrecer",
    "ensombrezcas": "ensombrecer",
    "ensombrezco": "ensombrecer",
    "ensombrezcáis": "ensombrecer",
    "ensopa": "ensopar",
    "ensopaba": "ensopar",
    "ensopabais": "ensopar",
    "ensopaban": "ensopar",
    "ensopabas": "ensopar",
    "ensopad": "ensopar",
    "ensopada": "ensopar",
    "ensopadas": "ensopar",
    "ensopado": "ensopar",
    "ensopados": "ensopar",
    "ensopamos": "ensopar",
    "ensopan": "ensopar",
    "ensopando": "ensopar",
    "ensopara": "ensopar",
    "ensoparais": "ensopar",
    "ensoparan": "ensopar",
    "ensoparas": "ensopar",
    "ensopare": "ensopar",
    "ensopareis": "ensopar",
    "ensoparemos": "ensopar",
    "ensoparen": "ensopar",
    "ensopares": "ensopar",
    "ensoparon": "ensopar",
    "ensopará": "ensopar",
    "ensoparán": "ensopar",
    "ensoparás": "ensopar",
    "ensoparé": "ensopar",
    "ensoparéis": "ensopar",
    "ensoparía": "ensopar",
    "ensoparíais": "ensopar",
    "ensoparíamos": "ensopar",
    "ensoparían": "ensopar",
    "ensoparías": "ensopar",
    "ensopas": "ensopar",
    "ensopase": "ensopar",
    "ensopaseis": "ensopar",
    "ensopasen": "ensopar",
    "ensopases": "ensopar",
    "ensopaste": "ensopar",
    "ensopasteis": "ensopar",
    "ensope": "ensopar",
    "ensopemos": "ensopar",
    "ensopen": "ensopar",
    "ensopes": "ensopar",
    "ensopo": "ensopar",
    "ensopábamos": "ensopar",
    "ensopáis": "ensopar",
    "ensopáramos": "ensopar",
    "ensopáremos": "ensopar",
    "ensopásemos": "ensopar",
    "ensopé": "ensopar",
    "ensopéis": "ensopar",
    "ensopó": "ensopar",
    "ensordece": "ensordecer",
    "ensordeced": "ensordecer",
    "ensordecedora": "ensordecedor",
    "ensordecedoras": "ensordecedor",
    "ensordecedores": "ensordecedor",
    "ensordecemos": "ensordecer",
    "ensordecen": "ensordecer",
    "ensordeceremos": "ensordecer",
    "ensordecerá": "ensordecer",
    "ensordecerán": "ensordecer",
    "ensordecerás": "ensordecer",
    "ensordeceré": "ensordecer",
    "ensordeceréis": "ensordecer",
    "ensordecería": "ensordecer",
    "ensordeceríais": "ensordecer",
    "ensordeceríamos": "ensordecer",
    "ensordecerían": "ensordecer",
    "ensordecerías": "ensordecer",
    "ensordeces": "ensordecer",
    "ensordecida": "ensordecer",
    "ensordecidas": "ensordecer",
    "ensordecido": "ensordecer",
    "ensordecidos": "ensordecer",
    "ensordeciendo": "ensordecer",
    "ensordeciera": "ensordecer",
    "ensordecierais": "ensordecer",
    "ensordecieran": "ensordecer",
    "ensordecieras": "ensordecer",
    "ensordeciere": "ensordecer",
    "ensordeciereis": "ensordecer",
    "ensordecieren": "ensordecer",
    "ensordecieres": "ensordecer",
    "ensordecieron": "ensordecer",
    "ensordeciese": "ensordecer",
    "ensordecieseis": "ensordecer",
    "ensordeciesen": "ensordecer",
    "ensordecieses": "ensordecer",
    "ensordecimientos": "ensordecimiento",
    "ensordecimos": "ensordecer",
    "ensordeciste": "ensordecer",
    "ensordecisteis": "ensordecer",
    "ensordeciéramos": "ensordecer",
    "ensordeciéremos": "ensordecer",
    "ensordeciésemos": "ensordecer",
    "ensordeció": "ensordecer",
    "ensordecéis": "ensordecer",
    "ensordecí": "ensordecer",
    "ensordecía": "ensordecer",
    "ensordecíais": "ensordecer",
    "ensordecíamos": "ensordecer",
    "ensordecían": "ensordecer",
    "ensordecías": "ensordecer",
    "ensordezca": "ensordecer",
    "ensordezcamos": "ensordecer",
    "ensordezcan": "ensordecer",
    "ensordezcas": "ensordecer",
    "ensordezco": "ensordecer",
    "ensordezcáis": "ensordecer",
    "ensortija": "ensortijar",
    "ensortijaba": "ensortijar",
    "ensortijabais": "ensortijar",
    "ensortijaban": "ensortijar",
    "ensortijabas": "ensortijar",
    "ensortijad": "ensortijar",
    "ensortijada": "ensortijar",
    "ensortijadas": "ensortijar",
    "ensortijado": "ensortijar",
    "ensortijados": "ensortijar",
    "ensortijamientos": "ensortijamiento",
    "ensortijamos": "ensortijar",
    "ensortijan": "ensortijar",
    "ensortijando": "ensortijar",
    "ensortijara": "ensortijar",
    "ensortijarais": "ensortijar",
    "ensortijaran": "ensortijar",
    "ensortijaras": "ensortijar",
    "ensortijare": "ensortijar",
    "ensortijareis": "ensortijar",
    "ensortijaremos": "ensortijar",
    "ensortijaren": "ensortijar",
    "ensortijares": "ensortijar",
    "ensortijaron": "ensortijar",
    "ensortijará": "ensortijar",
    "ensortijarán": "ensortijar",
    "ensortijarás": "ensortijar",
    "ensortijaré": "ensortijar",
    "ensortijaréis": "ensortijar",
    "ensortijaría": "ensortijar",
    "ensortijaríais": "ensortijar",
    "ensortijaríamos": "ensortijar",
    "ensortijarían": "ensortijar",
    "ensortijarías": "ensortijar",
    "ensortijas": "ensortijar",
    "ensortijase": "ensortijar",
    "ensortijaseis": "ensortijar",
    "ensortijasen": "ensortijar",
    "ensortijases": "ensortijar",
    "ensortijaste": "ensortijar",
    "ensortijasteis": "ensortijar",
    "ensortije": "ensortijar",
    "ensortijemos": "ensortijar",
    "ensortijen": "ensortijar",
    "ensortijes": "ensortijar",
    "ensortijo": "ensortijar",
    "ensortijábamos": "ensortijar",
    "ensortijáis": "ensortijar",
    "ensortijáramos": "ensortijar",
    "ensortijáremos": "ensortijar",
    "ensortijásemos": "ensortijar",
    "ensortijé": "ensortijar",
    "ensortijéis": "ensortijar",
    "ensortijó": "ensortijar",
    "ensoñaba": "ensoñar",
    "ensoñabais": "ensoñar",
    "ensoñaban": "ensoñar",
    "ensoñabas": "ensoñar",
    "ensoñaciones": "ensoñación",
    "ensoñad": "ensoñar",
    "ensoñada": "ensoñar",
    "ensoñadas": "ensoñar",
    "ensoñado": "ensoñar",
    "ensoñadora": "ensoñador",
    "ensoñadoras": "ensoñador",
    "ensoñadores": "ensoñador",
    "ensoñados": "ensoñar",
    "ensoñamos": "ensoñar",
    "ensoñando": "ensoñar",
    "ensoñara": "ensoñar",
    "ensoñarais": "ensoñar",
    "ensoñaran": "ensoñar",
    "ensoñaras": "ensoñar",
    "ensoñare": "ensoñar",
    "ensoñareis": "ensoñar",
    "ensoñaremos": "ensoñar",
    "ensoñaren": "ensoñar",
    "ensoñares": "ensoñar",
    "ensoñaron": "ensoñar",
    "ensoñará": "ensoñar",
    "ensoñarán": "ensoñar",
    "ensoñarás": "ensoñar",
    "ensoñaré": "ensoñar",
    "ensoñaréis": "ensoñar",
    "ensoñaría": "ensoñar",
    "ensoñaríais": "ensoñar",
    "ensoñaríamos": "ensoñar",
    "ensoñarían": "ensoñar",
    "ensoñarías": "ensoñar",
    "ensoñase": "ensoñar",
    "ensoñaseis": "ensoñar",
    "ensoñasen": "ensoñar",
    "ensoñases": "ensoñar",
    "ensoñaste": "ensoñar",
    "ensoñasteis": "ensoñar",
    "ensoñemos": "ensoñar",
    "ensoñábamos": "ensoñar",
    "ensoñáis": "ensoñar",
    "ensoñáramos": "ensoñar",
    "ensoñáremos": "ensoñar",
    "ensoñásemos": "ensoñar",
    "ensoñé": "ensoñar",
    "ensoñéis": "ensoñar",
    "ensoñó": "ensoñar",
    "enstatitas": "enstatita",
    "ensucia": "ensuciar",
    "ensuciaba": "ensuciar",
    "ensuciabais": "ensuciar",
    "ensuciaban": "ensuciar",
    "ensuciabas": "ensuciar",
    "ensuciad": "ensuciar",
    "ensuciada": "ensuciar",
    "ensuciadas": "ensuciar",
    "ensuciado": "ensuciar",
    "ensuciadora": "ensuciador",
    "ensuciadoras": "ensuciador",
    "ensuciadores": "ensuciador",
    "ensuciados": "ensuciar",
    "ensuciamientos": "ensuciamiento",
    "ensuciamos": "ensuciar",
    "ensucian": "ensuciar",
    "ensuciando": "ensuciar",
    "ensuciara": "ensuciar",
    "ensuciarais": "ensuciar",
    "ensuciaran": "ensuciar",
    "ensuciaras": "ensuciar",
    "ensuciare": "ensuciar",
    "ensuciareis": "ensuciar",
    "ensuciaremos": "ensuciar",
    "ensuciaren": "ensuciar",
    "ensuciares": "ensuciar",
    "ensuciaron": "ensuciar",
    "ensuciará": "ensuciar",
    "ensuciarán": "ensuciar",
    "ensuciarás": "ensuciar",
    "ensuciaré": "ensuciar",
    "ensuciaréis": "ensuciar",
    "ensuciaría": "ensuciar",
    "ensuciaríais": "ensuciar",
    "ensuciaríamos": "ensuciar",
    "ensuciarían": "ensuciar",
    "ensuciarías": "ensuciar",
    "ensucias": "ensuciar",
    "ensuciase": "ensuciar",
    "ensuciaseis": "ensuciar",
    "ensuciasen": "ensuciar",
    "ensuciases": "ensuciar",
    "ensuciaste": "ensuciar",
    "ensuciasteis": "ensuciar",
    "ensucie": "ensuciar",
    "ensuciemos": "ensuciar",
    "ensucien": "ensuciar",
    "ensucies": "ensuciar",
    "ensucio": "ensuciar",
    "ensuciábamos": "ensuciar",
    "ensuciáis": "ensuciar",
    "ensuciáramos": "ensuciar",
    "ensuciáremos": "ensuciar",
    "ensuciásemos": "ensuciar",
    "ensucié": "ensuciar",
    "ensuciéis": "ensuciar",
    "ensució": "ensuciar",
    "ensueña": "ensoñar",
    "ensueñan": "ensoñar",
    "ensueñas": "ensoñar",
    "ensueñe": "ensoñar",
    "ensueñen": "ensoñar",
    "ensueñes": "ensoñar",
    "ensueño": "ensoñar",
    "ensueños": "ensueño",
    "ensullos": "ensullo",
    "entabicados": "entabicado",
    "entabla": "entablar",
    "entablaba": "entablar",
    "entablabais": "entablar",
    "entablaban": "entablar",
    "entablabas": "entablar",
    "entablaciones": "entablación",
    "entablad": "entablar",
    "entablada": "entablar",
    "entabladas": "entablar",
    "entablado": "entablar",
    "entablados": "entablar",
    "entabladuras": "entabladura",
    "entablamentos": "entablamento",
    "entablamientos": "entablamiento",
    "entablamos": "entablar",
    "entablan": "entablar",
    "entablando": "entablar",
    "entablara": "entablar",
    "entablarais": "entablar",
    "entablaran": "entablar",
    "entablaras": "entablar",
    "entablare": "entablar",
    "entablareis": "entablar",
    "entablaremos": "entablar",
    "entablaren": "entablar",
    "entablares": "entablar",
    "entablaron": "entablar",
    "entablará": "entablar",
    "entablarán": "entablar",
    "entablarás": "entablar",
    "entablaré": "entablar",
    "entablaréis": "entablar",
    "entablaría": "entablar",
    "entablaríais": "entablar",
    "entablaríamos": "entablar",
    "entablarían": "entablar",
    "entablarías": "entablar",
    "entablas": "entablar",
    "entablase": "entablar",
    "entablaseis": "entablar",
    "entablasen": "entablar",
    "entablases": "entablar",
    "entablaste": "entablar",
    "entablasteis": "entablar",
    "entable": "entablar",
    "entablemos": "entablar",
    "entablen": "entablar",
    "entablera": "entablerar",
    "entableraba": "entablerar",
    "entablerabais": "entablerar",
    "entableraban": "entablerar",
    "entablerabas": "entablerar",
    "entablerad": "entablerar",
    "entablerada": "entablerar",
    "entableradas": "entablerar",
    "entablerado": "entablerar",
    "entablerados": "entablerar",
    "entableramos": "entablerar",
    "entableran": "entablerar",
    "entablerando": "entablerar",
    "entablerara": "entablerar",
    "entablerarais": "entablerar",
    "entableraran": "entablerar",
    "entableraras": "entablerar",
    "entablerare": "entablerar",
    "entablerareis": "entablerar",
    "entableraremos": "entablerar",
    "entableraren": "entablerar",
    "entablerares": "entablerar",
    "entableraron": "entablerar",
    "entablerará": "entablerar",
    "entablerarán": "entablerar",
    "entablerarás": "entablerar",
    "entableraré": "entablerar",
    "entableraréis": "entablerar",
    "entableraría": "entablerar",
    "entableraríais": "entablerar",
    "entableraríamos": "entablerar",
    "entablerarían": "entablerar",
    "entablerarías": "entablerar",
    "entableras": "entablerar",
    "entablerase": "entablerar",
    "entableraseis": "entablerar",
    "entablerasen": "entablerar",
    "entablerases": "entablerar",
    "entableraste": "entablerar",
    "entablerasteis": "entablerar",
    "entablere": "entablerar",
    "entableremos": "entablerar",
    "entableren": "entablerar",
    "entableres": "entablerar",
    "entablero": "entablerar",
    "entablerábamos": "entablerar",
    "entableráis": "entablerar",
    "entableráramos": "entablerar",
    "entableráremos": "entablerar",
    "entablerásemos": "entablerar",
    "entableré": "entablerar",
    "entableréis": "entablerar",
    "entableró": "entablerar",
    "entables": "entable",
    "entablilla": "entablillar",
    "entablillaba": "entablillar",
    "entablillabais": "entablillar",
    "entablillaban": "entablillar",
    "entablillabas": "entablillar",
    "entablillad": "entablillar",
    "entablillada": "entablillar",
    "entablilladas": "entablillar",
    "entablillado": "entablillar",
    "entablillados": "entablillar",
    "entablillamientos": "entablillamiento",
    "entablillamos": "entablillar",
    "entablillan": "entablillar",
    "entablillando": "entablillar",
    "entablillara": "entablillar",
    "entablillarais": "entablillar",
    "entablillaran": "entablillar",
    "entablillaras": "entablillar",
    "entablillare": "entablillar",
    "entablillareis": "entablillar",
    "entablillaremos": "entablillar",
    "entablillaren": "entablillar",
    "entablillares": "entablillar",
    "entablillaron": "entablillar",
    "entablillará": "entablillar",
    "entablillarán": "entablillar",
    "entablillarás": "entablillar",
    "entablillaré": "entablillar",
    "entablillaréis": "entablillar",
    "entablillaría": "entablillar",
    "entablillaríais": "entablillar",
    "entablillaríamos": "entablillar",
    "entablillarían": "entablillar",
    "entablillarías": "entablillar",
    "entablillas": "entablillar",
    "entablillase": "entablillar",
    "entablillaseis": "entablillar",
    "entablillasen": "entablillar",
    "entablillases": "entablillar",
    "entablillaste": "entablillar",
    "entablillasteis": "entablillar",
    "entablille": "entablillar",
    "entablillemos": "entablillar",
    "entablillen": "entablillar",
    "entablilles": "entablillar",
    "entablillo": "entablillar",
    "entablillábamos": "entablillar",
    "entablilláis": "entablillar",
    "entablilláramos": "entablillar",
    "entablilláremos": "entablillar",
    "entablillásemos": "entablillar",
    "entablillé": "entablillar",
    "entablilléis": "entablillar",
    "entablilló": "entablillar",
    "entablo": "entablar",
    "entablábamos": "entablar",
    "entabláis": "entablar",
    "entabláramos": "entablar",
    "entabláremos": "entablar",
    "entablásemos": "entablar",
    "entablé": "entablar",
    "entabléis": "entablar",
    "entabló": "entablar",
    "entada": "entado",
    "entadas": "entado",
    "entados": "entado",
    "entalama": "entalamar",
    "entalamaba": "entalamar",
    "entalamabais": "entalamar",
    "entalamaban": "entalamar",
    "entalamabas": "entalamar",
    "entalamad": "entalamar",
    "entalamada": "entalamar",
    "entalamadas": "entalamar",
    "entalamado": "entalamar",
    "entalamados": "entalamar",
    "entalamaduras": "entalamadura",
    "entalamamos": "entalamar",
    "entalaman": "entalamar",
    "entalamando": "entalamar",
    "entalamara": "entalamar",
    "entalamarais": "entalamar",
    "entalamaran": "entalamar",
    "entalamaras": "entalamar",
    "entalamare": "entalamar",
    "entalamareis": "entalamar",
    "entalamaremos": "entalamar",
    "entalamaren": "entalamar",
    "entalamares": "entalamar",
    "entalamaron": "entalamar",
    "entalamará": "entalamar",
    "entalamarán": "entalamar",
    "entalamarás": "entalamar",
    "entalamaré": "entalamar",
    "entalamaréis": "entalamar",
    "entalamaría": "entalamar",
    "entalamaríais": "entalamar",
    "entalamaríamos": "entalamar",
    "entalamarían": "entalamar",
    "entalamarías": "entalamar",
    "entalamas": "entalamar",
    "entalamase": "entalamar",
    "entalamaseis": "entalamar",
    "entalamasen": "entalamar",
    "entalamases": "entalamar",
    "entalamaste": "entalamar",
    "entalamasteis": "entalamar",
    "entalame": "entalamar",
    "entalamemos": "entalamar",
    "entalamen": "entalamar",
    "entalames": "entalamar",
    "entalamo": "entalamar",
    "entalamábamos": "entalamar",
    "entalamáis": "entalamar",
    "entalamáramos": "entalamar",
    "entalamáremos": "entalamar",
    "entalamásemos": "entalamar",
    "entalamé": "entalamar",
    "entalaméis": "entalamar",
    "entalamó": "entalamar",
    "entalega": "entalegar",
    "entalegaba": "entalegar",
    "entalegabais": "entalegar",
    "entalegaban": "entalegar",
    "entalegabas": "entalegar",
    "entalegad": "entalegar",
    "entalegada": "entalegar",
    "entalegadas": "entalegar",
    "entalegado": "entalegar",
    "entalegados": "entalegar",
    "entalegamos": "entalegar",
    "entalegan": "entalegar",
    "entalegando": "entalegar",
    "entalegara": "entalegar",
    "entalegarais": "entalegar",
    "entalegaran": "entalegar",
    "entalegaras": "entalegar",
    "entalegare": "entalegar",
    "entalegareis": "entalegar",
    "entalegaremos": "entalegar",
    "entalegaren": "entalegar",
    "entalegares": "entalegar",
    "entalegaron": "entalegar",
    "entalegará": "entalegar",
    "entalegarán": "entalegar",
    "entalegarás": "entalegar",
    "entalegaré": "entalegar",
    "entalegaréis": "entalegar",
    "entalegaría": "entalegar",
    "entalegaríais": "entalegar",
    "entalegaríamos": "entalegar",
    "entalegarían": "entalegar",
    "entalegarías": "entalegar",
    "entalegas": "entalegar",
    "entalegase": "entalegar",
    "entalegaseis": "entalegar",
    "entalegasen": "entalegar",
    "entalegases": "entalegar",
    "entalegaste": "entalegar",
    "entalegasteis": "entalegar",
    "entalego": "entalegar",
    "entalegue": "entalegar",
    "entaleguemos": "entalegar",
    "entaleguen": "entalegar",
    "entalegues": "entalegar",
    "entalegué": "entalegar",
    "entaleguéis": "entalegar",
    "entalegábamos": "entalegar",
    "entalegáis": "entalegar",
    "entalegáramos": "entalegar",
    "entalegáremos": "entalegar",
    "entalegásemos": "entalegar",
    "entalegó": "entalegar",
    "entalinga": "entalingar",
    "entalingaba": "entalingar",
    "entalingabais": "entalingar",
    "entalingaban": "entalingar",
    "entalingabas": "entalingar",
    "entalingad": "entalingar",
    "entalingada": "entalingar",
    "entalingadas": "entalingar",
    "entalingado": "entalingar",
    "entalingados": "entalingar",
    "entalingamos": "entalingar",
    "entalingan": "entalingar",
    "entalingando": "entalingar",
    "entalingara": "entalingar",
    "entalingarais": "entalingar",
    "entalingaran": "entalingar",
    "entalingaras": "entalingar",
    "entalingare": "entalingar",
    "entalingareis": "entalingar",
    "entalingaremos": "entalingar",
    "entalingaren": "entalingar",
    "entalingares": "entalingar",
    "entalingaron": "entalingar",
    "entalingará": "entalingar",
    "entalingarán": "entalingar",
    "entalingarás": "entalingar",
    "entalingaré": "entalingar",
    "entalingaréis": "entalingar",
    "entalingaría": "entalingar",
    "entalingaríais": "entalingar",
    "entalingaríamos": "entalingar",
    "entalingarían": "entalingar",
    "entalingarías": "entalingar",
    "entalingas": "entalingar",
    "entalingase": "entalingar",
    "entalingaseis": "entalingar",
    "entalingasen": "entalingar",
    "entalingases": "entalingar",
    "entalingaste": "entalingar",
    "entalingasteis": "entalingar",
    "entalingo": "entalingar",
    "entalingue": "entalingar",
    "entalinguemos": "entalingar",
    "entalinguen": "entalingar",
    "entalingues": "entalingar",
    "entalingué": "entalingar",
    "entalinguéis": "entalingar",
    "entalingábamos": "entalingar",
    "entalingáis": "entalingar",
    "entalingáramos": "entalingar",
    "entalingáremos": "entalingar",
    "entalingásemos": "entalingar",
    "entalingó": "entalingar",
    "entalla": "entallar",
    "entallaba": "entallar",
    "entallabais": "entallar",
    "entallaban": "entallar",
    "entallabas": "entallar",
    "entallad": "entallar",
    "entallada": "entallar",
    "entalladas": "entallar",
    "entallado": "entallar",
    "entalladora": "entallador",
    "entalladoras": "entallador",
    "entalladores": "entallador",
    "entallados": "entallar",
    "entalladuras": "entalladura",
    "entallamientos": "entallamiento",
    "entallamos": "entallar",
    "entallan": "entallar",
    "entallando": "entallar",
    "entallara": "entallar",
    "entallarais": "entallar",
    "entallaran": "entallar",
    "entallaras": "entallar",
    "entallare": "entallar",
    "entallareis": "entallar",
    "entallaremos": "entallar",
    "entallaren": "entallar",
    "entallares": "entallar",
    "entallaron": "entallar",
    "entallará": "entallar",
    "entallarán": "entallar",
    "entallarás": "entallar",
    "entallaré": "entallar",
    "entallaréis": "entallar",
    "entallaría": "entallar",
    "entallaríais": "entallar",
    "entallaríamos": "entallar",
    "entallarían": "entallar",
    "entallarías": "entallar",
    "entallas": "entallar",
    "entallase": "entallar",
    "entallaseis": "entallar",
    "entallasen": "entallar",
    "entallases": "entallar",
    "entallaste": "entallar",
    "entallasteis": "entallar",
    "entalle": "entallar",
    "entallece": "entallecer",
    "entalleced": "entallecer",
    "entallecemos": "entallecer",
    "entallecen": "entallecer",
    "entalleceremos": "entallecer",
    "entallecerá": "entallecer",
    "entallecerán": "entallecer",
    "entallecerás": "entallecer",
    "entalleceré": "entallecer",
    "entalleceréis": "entallecer",
    "entallecería": "entallecer",
    "entalleceríais": "entallecer",
    "entalleceríamos": "entallecer",
    "entallecerían": "entallecer",
    "entallecerías": "entallecer",
    "entalleces": "entallecer",
    "entallecida": "entallecer",
    "entallecidas": "entallecer",
    "entallecido": "entallecer",
    "entallecidos": "entallecer",
    "entalleciendo": "entallecer",
    "entalleciera": "entallecer",
    "entallecierais": "entallecer",
    "entallecieran": "entallecer",
    "entallecieras": "entallecer",
    "entalleciere": "entallecer",
    "entalleciereis": "entallecer",
    "entallecieren": "entallecer",
    "entallecieres": "entallecer",
    "entallecieron": "entallecer",
    "entalleciese": "entallecer",
    "entallecieseis": "entallecer",
    "entalleciesen": "entallecer",
    "entallecieses": "entallecer",
    "entallecimos": "entallecer",
    "entalleciste": "entallecer",
    "entallecisteis": "entallecer",
    "entalleciéramos": "entallecer",
    "entalleciéremos": "entallecer",
    "entalleciésemos": "entallecer",
    "entalleció": "entallecer",
    "entallecéis": "entallecer",
    "entallecí": "entallecer",
    "entallecía": "entallecer",
    "entallecíais": "entallecer",
    "entallecíamos": "entallecer",
    "entallecían": "entallecer",
    "entallecías": "entallecer",
    "entallemos": "entallar",
    "entallen": "entallar",
    "entalles": "entalle",
    "entallezca": "entallecer",
    "entallezcamos": "entallecer",
    "entallezcan": "entallecer",
    "entallezcas": "entallecer",
    "entallezco": "entallecer",
    "entallezcáis": "entallecer",
    "entallo": "entallar",
    "entallábamos": "entallar",
    "entalláis": "entallar",
    "entalláramos": "entallar",
    "entalláremos": "entallar",
    "entallásemos": "entallar",
    "entallé": "entallar",
    "entalléis": "entallar",
    "entalló": "entallar",
    "entalpías": "entalpía",
    "entapetada": "entapetado",
    "entapetadas": "entapetado",
    "entapetados": "entapetado",
    "entapice": "entapizar",
    "entapicemos": "entapizar",
    "entapicen": "entapizar",
    "entapices": "entapizar",
    "entapicé": "entapizar",
    "entapicéis": "entapizar",
    "entapiza": "entapizar",
    "entapizaba": "entapizar",
    "entapizabais": "entapizar",
    "entapizaban": "entapizar",
    "entapizabas": "entapizar",
    "entapizad": "entapizar",
    "entapizada": "entapizar",
    "entapizadas": "entapizar",
    "entapizado": "entapizar",
    "entapizados": "entapizar",
    "entapizamos": "entapizar",
    "entapizan": "entapizar",
    "entapizando": "entapizar",
    "entapizara": "entapizar",
    "entapizarais": "entapizar",
    "entapizaran": "entapizar",
    "entapizaras": "entapizar",
    "entapizare": "entapizar",
    "entapizareis": "entapizar",
    "entapizaremos": "entapizar",
    "entapizaren": "entapizar",
    "entapizares": "entapizar",
    "entapizaron": "entapizar",
    "entapizará": "entapizar",
    "entapizarán": "entapizar",
    "entapizarás": "entapizar",
    "entapizaré": "entapizar",
    "entapizaréis": "entapizar",
    "entapizaría": "entapizar",
    "entapizaríais": "entapizar",
    "entapizaríamos": "entapizar",
    "entapizarían": "entapizar",
    "entapizarías": "entapizar",
    "entapizas": "entapizar",
    "entapizase": "entapizar",
    "entapizaseis": "entapizar",
    "entapizasen": "entapizar",
    "entapizases": "entapizar",
    "entapizaste": "entapizar",
    "entapizasteis": "entapizar",
    "entapizo": "entapizar",
    "entapizábamos": "entapizar",
    "entapizáis": "entapizar",
    "entapizáramos": "entapizar",
    "entapizáremos": "entapizar",
    "entapizásemos": "entapizar",
    "entapizó": "entapizar",
    "entapuja": "entapujar",
    "entapujaba": "entapujar",
    "entapujabais": "entapujar",
    "entapujaban": "entapujar",
    "entapujabas": "entapujar",
    "entapujad": "entapujar",
    "entapujada": "entapujar",
    "entapujadas": "entapujar",
    "entapujado": "entapujar",
    "entapujados": "entapujar",
    "entapujamos": "entapujar",
    "entapujan": "entapujar",
    "entapujando": "entapujar",
    "entapujara": "entapujar",
    "entapujarais": "entapujar",
    "entapujaran": "entapujar",
    "entapujaras": "entapujar",
    "entapujare": "entapujar",
    "entapujareis": "entapujar",
    "entapujaremos": "entapujar",
    "entapujaren": "entapujar",
    "entapujares": "entapujar",
    "entapujaron": "entapujar",
    "entapujará": "entapujar",
    "entapujarán": "entapujar",
    "entapujarás": "entapujar",
    "entapujaré": "entapujar",
    "entapujaréis": "entapujar",
    "entapujaría": "entapujar",
    "entapujaríais": "entapujar",
    "entapujaríamos": "entapujar",
    "entapujarían": "entapujar",
    "entapujarías": "entapujar",
    "entapujas": "entapujar",
    "entapujase": "entapujar",
    "entapujaseis": "entapujar",
    "entapujasen": "entapujar",
    "entapujases": "entapujar",
    "entapujaste": "entapujar",
    "entapujasteis": "entapujar",
    "entapuje": "entapujar",
    "entapujemos": "entapujar",
    "entapujen": "entapujar",
    "entapujes": "entapujar",
    "entapujo": "entapujar",
    "entapujábamos": "entapujar",
    "entapujáis": "entapujar",
    "entapujáramos": "entapujar",
    "entapujáremos": "entapujar",
    "entapujásemos": "entapujar",
    "entapujé": "entapujar",
    "entapujéis": "entapujar",
    "entapujó": "entapujar",
    "entarima": "entarimar",
    "entarimaba": "entarimar",
    "entarimabais": "entarimar",
    "entarimaban": "entarimar",
    "entarimabas": "entarimar",
    "entarimad": "entarimar",
    "entarimada": "entarimar",
    "entarimadas": "entarimar",
    "entarimado": "entarimar",
    "entarimadora": "entarimador",
    "entarimadoras": "entarimador",
    "entarimadores": "entarimador",
    "entarimados": "entarimar",
    "entarimamos": "entarimar",
    "entariman": "entarimar",
    "entarimando": "entarimar",
    "entarimara": "entarimar",
    "entarimarais": "entarimar",
    "entarimaran": "entarimar",
    "entarimaras": "entarimar",
    "entarimare": "entarimar",
    "entarimareis": "entarimar",
    "entarimaremos": "entarimar",
    "entarimaren": "entarimar",
    "entarimares": "entarimar",
    "entarimaron": "entarimar",
    "entarimará": "entarimar",
    "entarimarán": "entarimar",
    "entarimarás": "entarimar",
    "entarimaré": "entarimar",
    "entarimaréis": "entarimar",
    "entarimaría": "entarimar",
    "entarimaríais": "entarimar",
    "entarimaríamos": "entarimar",
    "entarimarían": "entarimar",
    "entarimarías": "entarimar",
    "entarimas": "entarimar",
    "entarimase": "entarimar",
    "entarimaseis": "entarimar",
    "entarimasen": "entarimar",
    "entarimases": "entarimar",
    "entarimaste": "entarimar",
    "entarimasteis": "entarimar",
    "entarime": "entarimar",
    "entarimemos": "entarimar",
    "entarimen": "entarimar",
    "entarimes": "entarimar",
    "entarimo": "entarimar",
    "entarimábamos": "entarimar",
    "entarimáis": "entarimar",
    "entarimáramos": "entarimar",
    "entarimáremos": "entarimar",
    "entarimásemos": "entarimar",
    "entarimé": "entarimar",
    "entariméis": "entarimar",
    "entarimó": "entarimar",
    "entarquina": "entarquinar",
    "entarquinaba": "entarquinar",
    "entarquinabais": "entarquinar",
    "entarquinaban": "entarquinar",
    "entarquinabas": "entarquinar",
    "entarquinad": "entarquinar",
    "entarquinada": "entarquinar",
    "entarquinadas": "entarquinar",
    "entarquinado": "entarquinar",
    "entarquinados": "entarquinar",
    "entarquinamientos": "entarquinamiento",
    "entarquinamos": "entarquinar",
    "entarquinan": "entarquinar",
    "entarquinando": "entarquinar",
    "entarquinara": "entarquinar",
    "entarquinarais": "entarquinar",
    "entarquinaran": "entarquinar",
    "entarquinaras": "entarquinar",
    "entarquinare": "entarquinar",
    "entarquinareis": "entarquinar",
    "entarquinaremos": "entarquinar",
    "entarquinaren": "entarquinar",
    "entarquinares": "entarquinar",
    "entarquinaron": "entarquinar",
    "entarquinará": "entarquinar",
    "entarquinarán": "entarquinar",
    "entarquinarás": "entarquinar",
    "entarquinaré": "entarquinar",
    "entarquinaréis": "entarquinar",
    "entarquinaría": "entarquinar",
    "entarquinaríais": "entarquinar",
    "entarquinaríamos": "entarquinar",
    "entarquinarían": "entarquinar",
    "entarquinarías": "entarquinar",
    "entarquinas": "entarquinar",
    "entarquinase": "entarquinar",
    "entarquinaseis": "entarquinar",
    "entarquinasen": "entarquinar",
    "entarquinases": "entarquinar",
    "entarquinaste": "entarquinar",
    "entarquinasteis": "entarquinar",
    "entarquine": "entarquinar",
    "entarquinemos": "entarquinar",
    "entarquinen": "entarquinar",
    "entarquines": "entarquinar",
    "entarquino": "entarquinar",
    "entarquinábamos": "entarquinar",
    "entarquináis": "entarquinar",
    "entarquináramos": "entarquinar",
    "entarquináremos": "entarquinar",
    "entarquinásemos": "entarquinar",
    "entarquiné": "entarquinar",
    "entarquinéis": "entarquinar",
    "entarquinó": "entarquinar",
    "entaruga": "entarugar",
    "entarugaba": "entarugar",
    "entarugabais": "entarugar",
    "entarugaban": "entarugar",
    "entarugabas": "entarugar",
    "entarugad": "entarugar",
    "entarugada": "entarugar",
    "entarugadas": "entarugar",
    "entarugado": "entarugar",
    "entarugados": "entarugar",
    "entarugamos": "entarugar",
    "entarugan": "entarugar",
    "entarugando": "entarugar",
    "entarugara": "entarugar",
    "entarugarais": "entarugar",
    "entarugaran": "entarugar",
    "entarugaras": "entarugar",
    "entarugare": "entarugar",
    "entarugareis": "entarugar",
    "entarugaremos": "entarugar",
    "entarugaren": "entarugar",
    "entarugares": "entarugar",
    "entarugaron": "entarugar",
    "entarugará": "entarugar",
    "entarugarán": "entarugar",
    "entarugarás": "entarugar",
    "entarugaré": "entarugar",
    "entarugaréis": "entarugar",
    "entarugaría": "entarugar",
    "entarugaríais": "entarugar",
    "entarugaríamos": "entarugar",
    "entarugarían": "entarugar",
    "entarugarías": "entarugar",
    "entarugas": "entarugar",
    "entarugase": "entarugar",
    "entarugaseis": "entarugar",
    "entarugasen": "entarugar",
    "entarugases": "entarugar",
    "entarugaste": "entarugar",
    "entarugasteis": "entarugar",
    "entarugo": "entarugar",
    "entarugue": "entarugar",
    "entaruguemos": "entarugar",
    "entaruguen": "entarugar",
    "entarugues": "entarugar",
    "entarugué": "entarugar",
    "entaruguéis": "entarugar",
    "entarugábamos": "entarugar",
    "entarugáis": "entarugar",
    "entarugáramos": "entarugar",
    "entarugáremos": "entarugar",
    "entarugásemos": "entarugar",
    "entarugó": "entarugar",
    "enteca": "enteco",
    "entecaba": "entecar",
    "entecabais": "entecar",
    "entecaban": "entecar",
    "entecabas": "entecar",
    "entecad": "entecar",
    "entecada": "entecar",
    "entecadas": "entecar",
    "entecado": "entecar",
    "entecados": "entecar",
    "entecamos": "entecar",
    "entecan": "entecar",
    "entecando": "entecar",
    "entecara": "entecar",
    "entecarais": "entecar",
    "entecaran": "entecar",
    "entecaras": "entecar",
    "entecare": "entecar",
    "entecareis": "entecar",
    "entecaremos": "entecar",
    "entecaren": "entecar",
    "entecares": "entecar",
    "entecaron": "entecar",
    "entecará": "entecar",
    "entecarán": "entecar",
    "entecarás": "entecar",
    "entecaré": "entecar",
    "entecaréis": "entecar",
    "entecaría": "entecar",
    "entecaríais": "entecar",
    "entecaríamos": "entecar",
    "entecarían": "entecar",
    "entecarías": "entecar",
    "entecas": "enteco",
    "entecase": "entecar",
    "entecaseis": "entecar",
    "entecasen": "entecar",
    "entecases": "entecar",
    "entecaste": "entecar",
    "entecasteis": "entecar",
    "entecha": "entechar",
    "entechaba": "entechar",
    "entechabais": "entechar",
    "entechaban": "entechar",
    "entechabas": "entechar",
    "entechad": "entechar",
    "entechada": "entechar",
    "entechadas": "entechar",
    "entechado": "entechar",
    "entechados": "entechar",
    "entechamos": "entechar",
    "entechan": "entechar",
    "entechando": "entechar",
    "entechara": "entechar",
    "entecharais": "entechar",
    "entecharan": "entechar",
    "entecharas": "entechar",
    "entechare": "entechar",
    "entechareis": "entechar",
    "entecharemos": "entechar",
    "entecharen": "entechar",
    "entechares": "entechar",
    "entecharon": "entechar",
    "entechará": "entechar",
    "entecharán": "entechar",
    "entecharás": "entechar",
    "entecharé": "entechar",
    "entecharéis": "entechar",
    "entecharía": "entechar",
    "entecharíais": "entechar",
    "entecharíamos": "entechar",
    "entecharían": "entechar",
    "entecharías": "entechar",
    "entechas": "entechar",
    "entechase": "entechar",
    "entechaseis": "entechar",
    "entechasen": "entechar",
    "entechases": "entechar",
    "entechaste": "entechar",
    "entechasteis": "entechar",
    "enteche": "entechar",
    "entechemos": "entechar",
    "entechen": "entechar",
    "enteches": "entechar",
    "entecho": "entechar",
    "entechábamos": "entechar",
    "entecháis": "entechar",
    "entecháramos": "entechar",
    "entecháremos": "entechar",
    "entechásemos": "entechar",
    "enteché": "entechar",
    "entechéis": "entechar",
    "entechó": "entechar",
    "enteco": "entecar",
    "entecos": "enteco",
    "entecábamos": "entecar",
    "entecáis": "entecar",
    "entecáramos": "entecar",
    "entecáremos": "entecar",
    "entecásemos": "entecar",
    "entecó": "entecar",
    "enteja": "entejar",
    "entejaba": "entejar",
    "entejabais": "entejar",
    "entejaban": "entejar",
    "entejabas": "entejar",
    "entejad": "entejar",
    "entejada": "entejar",
    "entejadas": "entejar",
    "entejado": "entejar",
    "entejados": "entejar",
    "entejamos": "entejar",
    "entejan": "entejar",
    "entejando": "entejar",
    "entejara": "entejar",
    "entejarais": "entejar",
    "entejaran": "entejar",
    "entejaras": "entejar",
    "entejare": "entejar",
    "entejareis": "entejar",
    "entejaremos": "entejar",
    "entejaren": "entejar",
    "entejares": "entejar",
    "entejaron": "entejar",
    "entejará": "entejar",
    "entejarán": "entejar",
    "entejarás": "entejar",
    "entejaré": "entejar",
    "entejaréis": "entejar",
    "entejaría": "entejar",
    "entejaríais": "entejar",
    "entejaríamos": "entejar",
    "entejarían": "entejar",
    "entejarías": "entejar",
    "entejas": "entejar",
    "entejase": "entejar",
    "entejaseis": "entejar",
    "entejasen": "entejar",
    "entejases": "entejar",
    "entejaste": "entejar",
    "entejasteis": "entejar",
    "enteje": "entejar",
    "entejemos": "entejar",
    "entejen": "entejar",
    "entejes": "entejar",
    "entejo": "entejar",
    "entejábamos": "entejar",
    "entejáis": "entejar",
    "entejáramos": "entejar",
    "entejáremos": "entejar",
    "entejásemos": "entejar",
    "entejé": "entejar",
    "entejéis": "entejar",
    "entejó": "entejar",
    "entelequias": "entelequia",
    "entenada": "entenado",
    "entenadas": "entenado",
    "entenados": "entenado",
    "entenas": "entena",
    "entendamos": "entender",
    "entended": "entender",
    "entendedora": "entendedor",
    "entendedoras": "entendedor",
    "entendedores": "entendedor",
    "entendemos": "entender",
    "entenderemos": "entender",
    "entenderes": "entender",
    "entenderá": "entender",
    "entenderán": "entender",
    "entenderás": "entender",
    "entenderé": "entender",
    "entenderéis": "entender",
    "entendería": "entender",
    "entenderíais": "entender",
    "entenderíamos": "entender",
    "entenderían": "entender",
    "entenderías": "entender",
    "entendibles": "entendible",
    "entendida": "entendido",
    "entendidas": "entendido",
    "entendido": "entender",
    "entendidos": "entendido",
    "entendiendo": "entender",
    "entendiera": "entender",
    "entendierais": "entender",
    "entendieran": "entender",
    "entendieras": "entender",
    "entendiere": "entender",
    "entendiereis": "entender",
    "entendieren": "entender",
    "entendieres": "entender",
    "entendieron": "entender",
    "entendiese": "entender",
    "entendieseis": "entender",
    "entendiesen": "entender",
    "entendieses": "entender",
    "entendimientos": "entendimiento",
    "entendimos": "entender",
    "entendiste": "entender",
    "entendisteis": "entender",
    "entendiéramos": "entender",
    "entendiéremos": "entender",
    "entendiésemos": "entender",
    "entendió": "entender",
    "entendáis": "entender",
    "entendéis": "entender",
    "entendí": "entender",
    "entendía": "entender",
    "entendíais": "entender",
    "entendíamos": "entender",
    "entendían": "entender",
    "entendías": "entender",
    "entenebrece": "entenebrecer",
    "entenebreced": "entenebrecer",
    "entenebrecemos": "entenebrecer",
    "entenebrecen": "entenebrecer",
    "entenebreceremos": "entenebrecer",
    "entenebrecerá": "entenebrecer",
    "entenebrecerán": "entenebrecer",
    "entenebrecerás": "entenebrecer",
    "entenebreceré": "entenebrecer",
    "entenebreceréis": "entenebrecer",
    "entenebrecería": "entenebrecer",
    "entenebreceríais": "entenebrecer",
    "entenebreceríamos": "entenebrecer",
    "entenebrecerían": "entenebrecer",
    "entenebrecerías": "entenebrecer",
    "entenebreces": "entenebrecer",
    "entenebrecida": "entenebrecer",
    "entenebrecidas": "entenebrecer",
    "entenebrecido": "entenebrecer",
    "entenebrecidos": "entenebrecer",
    "entenebreciendo": "entenebrecer",
    "entenebreciera": "entenebrecer",
    "entenebrecierais": "entenebrecer",
    "entenebrecieran": "entenebrecer",
    "entenebrecieras": "entenebrecer",
    "entenebreciere": "entenebrecer",
    "entenebreciereis": "entenebrecer",
    "entenebrecieren": "entenebrecer",
    "entenebrecieres": "entenebrecer",
    "entenebrecieron": "entenebrecer",
    "entenebreciese": "entenebrecer",
    "entenebrecieseis": "entenebrecer",
    "entenebreciesen": "entenebrecer",
    "entenebrecieses": "entenebrecer",
    "entenebrecimos": "entenebrecer",
    "entenebreciste": "entenebrecer",
    "entenebrecisteis": "entenebrecer",
    "entenebreciéramos": "entenebrecer",
    "entenebreciéremos": "entenebrecer",
    "entenebreciésemos": "entenebrecer",
    "entenebreció": "entenebrecer",
    "entenebrecéis": "entenebrecer",
    "entenebrecí": "entenebrecer",
    "entenebrecía": "entenebrecer",
    "entenebrecíais": "entenebrecer",
    "entenebrecíamos": "entenebrecer",
    "entenebrecían": "entenebrecer",
    "entenebrecías": "entenebrecer",
    "entenebrezca": "entenebrecer",
    "entenebrezcamos": "entenebrecer",
    "entenebrezcan": "entenebrecer",
    "entenebrezcas": "entenebrecer",
    "entenebrezco": "entenebrecer",
    "entenebrezcáis": "entenebrecer",
    "ententes": "entente",
    "enteque": "entecar",
    "entequemos": "entecar",
    "entequen": "entecar",
    "enteques": "entecar",
    "entequé": "entecar",
    "entequéis": "entecar",
    "entera": "entero",
    "enteraba": "enterar",
    "enterabais": "enterar",
    "enteraban": "enterar",
    "enterabas": "enterar",
    "enterad": "enterar",
    "enterada": "enterar",
    "enteradas": "enterar",
    "enterado": "enterar",
    "enterados": "enterar",
    "enteralgias": "enteralgia",
    "enteramos": "enterar",
    "enteran": "enterar",
    "enterando": "enterar",
    "enterara": "enterar",
    "enterarais": "enterar",
    "enteraran": "enterar",
    "enteraras": "enterar",
    "enterare": "enterar",
    "enterareis": "enterar",
    "enteraremos": "enterar",
    "enteraren": "enterar",
    "enterares": "enterar",
    "enteraron": "enterar",
    "enterará": "enterar",
    "enterarán": "enterar",
    "enterarás": "enterar",
    "enteraré": "enterar",
    "enteraréis": "enterar",
    "enteraría": "enterar",
    "enteraríais": "enterar",
    "enteraríamos": "enterar",
    "enterarían": "enterar",
    "enterarías": "enterar",
    "enteras": "entero",
    "enterase": "enterar",
    "enteraseis": "enterar",
    "enterasen": "enterar",
    "enterases": "enterar",
    "enteraste": "enterar",
    "enterasteis": "enterar",
    "enterca": "entercar",
    "entercaba": "entercar",
    "entercabais": "entercar",
    "entercaban": "entercar",
    "entercabas": "entercar",
    "entercad": "entercar",
    "entercada": "entercar",
    "entercadas": "entercar",
    "entercado": "entercar",
    "entercados": "entercar",
    "entercamos": "entercar",
    "entercan": "entercar",
    "entercando": "entercar",
    "entercara": "entercar",
    "entercarais": "entercar",
    "entercaran": "entercar",
    "entercaras": "entercar",
    "entercare": "entercar",
    "entercareis": "entercar",
    "entercaremos": "entercar",
    "entercaren": "entercar",
    "entercares": "entercar",
    "entercaron": "entercar",
    "entercará": "entercar",
    "entercarán": "entercar",
    "entercarás": "entercar",
    "entercaré": "entercar",
    "entercaréis": "entercar",
    "entercaría": "entercar",
    "entercaríais": "entercar",
    "entercaríamos": "entercar",
    "entercarían": "entercar",
    "entercarías": "entercar",
    "entercas": "entercar",
    "entercase": "entercar",
    "entercaseis": "entercar",
    "entercasen": "entercar",
    "entercases": "entercar",
    "entercaste": "entercar",
    "entercasteis": "entercar",
    "enterco": "entercar",
    "entercábamos": "entercar",
    "entercáis": "entercar",
    "entercáramos": "entercar",
    "entercáremos": "entercar",
    "entercásemos": "entercar",
    "entercó": "entercar",
    "entere": "enterar",
    "enteremos": "enterar",
    "enteren": "enterar",
    "enteres": "enterar",
    "enterezas": "entereza",
    "enteriza": "enterizo",
    "enterizas": "enterizo",
    "enterizos": "enterizo",
    "enternece": "enternecer",
    "enterneced": "enternecer",
    "enternecedora": "enternecedor",
    "enternecedoras": "enternecedor",
    "enternecedores": "enternecedor",
    "enternecemos": "enternecer",
    "enternecen": "enternecer",
    "enterneceremos": "enternecer",
    "enternecerá": "enternecer",
    "enternecerán": "enternecer",
    "enternecerás": "enternecer",
    "enterneceré": "enternecer",
    "enterneceréis": "enternecer",
    "enternecería": "enternecer",
    "enterneceríais": "enternecer",
    "enterneceríamos": "enternecer",
    "enternecerían": "enternecer",
    "enternecerías": "enternecer",
    "enterneces": "enternecer",
    "enternecida": "enternecer",
    "enternecidas": "enternecer",
    "enternecido": "enternecer",
    "enternecidos": "enternecer",
    "enterneciendo": "enternecer",
    "enterneciera": "enternecer",
    "enternecierais": "enternecer",
    "enternecieran": "enternecer",
    "enternecieras": "enternecer",
    "enterneciere": "enternecer",
    "enterneciereis": "enternecer",
    "enternecieren": "enternecer",
    "enternecieres": "enternecer",
    "enternecieron": "enternecer",
    "enterneciese": "enternecer",
    "enternecieseis": "enternecer",
    "enterneciesen": "enternecer",
    "enternecieses": "enternecer",
    "enternecimientos": "enternecimiento",
    "enternecimos": "enternecer",
    "enterneciste": "enternecer",
    "enternecisteis": "enternecer",
    "enterneciéramos": "enternecer",
    "enterneciéremos": "enternecer",
    "enterneciésemos": "enternecer",
    "enterneció": "enternecer",
    "enternecéis": "enternecer",
    "enternecí": "enternecer",
    "enternecía": "enternecer",
    "enternecíais": "enternecer",
    "enternecíamos": "enternecer",
    "enternecían": "enternecer",
    "enternecías": "enternecer",
    "enternezca": "enternecer",
    "enternezcamos": "enternecer",
    "enternezcan": "enternecer",
    "enternezcas": "enternecer",
    "enternezco": "enternecer",
    "enternezcáis": "enternecer",
    "entero": "enterar",
    "enterobacterias": "enterobacteria",
    "enterobacteriáceas": "enterobacteriácea",
    "enteroceles": "enterocele",
    "enterocitos": "enterocito",
    "enterocélica": "enterocélico",
    "enterocélicas": "enterocélico",
    "enterocélicos": "enterocélico",
    "enterolitos": "enterolito",
    "enterologías": "enterología",
    "enteropatías": "enteropatía",
    "enteros": "entero",
    "enterotomías": "enterotomía",
    "enterotoxinas": "enterotoxina",
    "enterque": "entercar",
    "enterquemos": "entercar",
    "enterquen": "entercar",
    "enterques": "entercar",
    "enterqué": "entercar",
    "enterquéis": "entercar",
    "enterraba": "enterrar",
    "enterrabais": "enterrar",
    "enterraban": "enterrar",
    "enterrabas": "enterrar",
    "enterrad": "enterrar",
    "enterrada": "enterrar",
    "enterradas": "enterrar",
    "enterrado": "enterrar",
    "enterradora": "enterrador",
    "enterradoras": "enterrador",
    "enterradores": "enterrador",
    "enterrados": "enterrar",
    "enterramientos": "enterramiento",
    "enterramos": "enterrar",
    "enterrando": "enterrar",
    "enterrara": "enterrar",
    "enterrarais": "enterrar",
    "enterraran": "enterrar",
    "enterraras": "enterrar",
    "enterrare": "enterrar",
    "enterrareis": "enterrar",
    "enterraremos": "enterrar",
    "enterraren": "enterrar",
    "enterrares": "enterrar",
    "enterraron": "enterrar",
    "enterrará": "enterrar",
    "enterrarán": "enterrar",
    "enterrarás": "enterrar",
    "enterraré": "enterrar",
    "enterraréis": "enterrar",
    "enterraría": "enterrar",
    "enterraríais": "enterrar",
    "enterraríamos": "enterrar",
    "enterrarían": "enterrar",
    "enterrarías": "enterrar",
    "enterrase": "enterrar",
    "enterraseis": "enterrar",
    "enterrasen": "enterrar",
    "enterrases": "enterrar",
    "enterraste": "enterrar",
    "enterrasteis": "enterrar",
    "enterremos": "enterrar",
    "enterrábamos": "enterrar",
    "enterráis": "enterrar",
    "enterráramos": "enterrar",
    "enterráremos": "enterrar",
    "enterrásemos": "enterrar",
    "enterré": "enterrar",
    "enterréis": "enterrar",
    "enterró": "enterrar",
    "enterábamos": "enterar",
    "enteráis": "enterar",
    "enteráramos": "enterar",
    "enteráremos": "enterar",
    "enterásemos": "enterar",
    "enteré": "enterar",
    "enteréis": "enterar",
    "enteró": "enterar",
    "entes": "ente",
    "entesa": "entesar",
    "entesaba": "entesar",
    "entesabais": "entesar",
    "entesaban": "entesar",
    "entesabas": "entesar",
    "entesad": "entesar",
    "entesada": "entesar",
    "entesadas": "entesar",
    "entesado": "entesar",
    "entesados": "entesar",
    "entesamos": "entesar",
    "entesan": "entesar",
    "entesando": "entesar",
    "entesara": "entesar",
    "entesarais": "entesar",
    "entesaran": "entesar",
    "entesaras": "entesar",
    "entesare": "entesar",
    "entesareis": "entesar",
    "entesaremos": "entesar",
    "entesaren": "entesar",
    "entesares": "entesar",
    "entesaron": "entesar",
    "entesará": "entesar",
    "entesarán": "entesar",
    "entesarás": "entesar",
    "entesaré": "entesar",
    "entesaréis": "entesar",
    "entesaría": "entesar",
    "entesaríais": "entesar",
    "entesaríamos": "entesar",
    "entesarían": "entesar",
    "entesarías": "entesar",
    "entesas": "entesar",
    "entesase": "entesar",
    "entesaseis": "entesar",
    "entesasen": "entesar",
    "entesases": "entesar",
    "entesaste": "entesar",
    "entesasteis": "entesar",
    "entese": "entesar",
    "entesemos": "entesar",
    "entesen": "entesar",
    "enteses": "entesar",
    "enteso": "entesar",
    "entesábamos": "entesar",
    "entesáis": "entesar",
    "entesáramos": "entesar",
    "entesáremos": "entesar",
    "entesásemos": "entesar",
    "entesé": "entesar",
    "enteséis": "entesar",
    "entesó": "entesar",
    "entiba": "entibar",
    "entibaba": "entibar",
    "entibabais": "entibar",
    "entibaban": "entibar",
    "entibabas": "entibar",
    "entibaciones": "entibación",
    "entibad": "entibar",
    "entibada": "entibar",
    "entibadas": "entibar",
    "entibado": "entibar",
    "entibadora": "entibador",
    "entibadoras": "entibador",
    "entibadores": "entibador",
    "entibados": "entibar",
    "entibamos": "entibar",
    "entiban": "entibar",
    "entibando": "entibar",
    "entibara": "entibar",
    "entibarais": "entibar",
    "entibaran": "entibar",
    "entibaras": "entibar",
    "entibare": "entibar",
    "entibareis": "entibar",
    "entibaremos": "entibar",
    "entibaren": "entibar",
    "entibares": "entibar",
    "entibaron": "entibar",
    "entibará": "entibar",
    "entibarán": "entibar",
    "entibarás": "entibar",
    "entibaré": "entibar",
    "entibaréis": "entibar",
    "entibaría": "entibar",
    "entibaríais": "entibar",
    "entibaríamos": "entibar",
    "entibarían": "entibar",
    "entibarías": "entibar",
    "entibas": "entibar",
    "entibase": "entibar",
    "entibaseis": "entibar",
    "entibasen": "entibar",
    "entibases": "entibar",
    "entibaste": "entibar",
    "entibasteis": "entibar",
    "entibe": "entibar",
    "entibemos": "entibar",
    "entiben": "entibar",
    "entibes": "entibar",
    "entibia": "entibiar",
    "entibiaba": "entibiar",
    "entibiabais": "entibiar",
    "entibiaban": "entibiar",
    "entibiabas": "entibiar",
    "entibiad": "entibiar",
    "entibiada": "entibiar",
    "entibiadas": "entibiar",
    "entibiado": "entibiar",
    "entibiados": "entibiar",
    "entibiamos": "entibiar",
    "entibian": "entibiar",
    "entibiando": "entibiar",
    "entibiara": "entibiar",
    "entibiarais": "entibiar",
    "entibiaran": "entibiar",
    "entibiaras": "entibiar",
    "entibiare": "entibiar",
    "entibiareis": "entibiar",
    "entibiaremos": "entibiar",
    "entibiaren": "entibiar",
    "entibiares": "entibiar",
    "entibiaron": "entibiar",
    "entibiará": "entibiar",
    "entibiarán": "entibiar",
    "entibiarás": "entibiar",
    "entibiaré": "entibiar",
    "entibiaréis": "entibiar",
    "entibiaría": "entibiar",
    "entibiaríais": "entibiar",
    "entibiaríamos": "entibiar",
    "entibiarían": "entibiar",
    "entibiarías": "entibiar",
    "entibias": "entibiar",
    "entibiase": "entibiar",
    "entibiaseis": "entibiar",
    "entibiasen": "entibiar",
    "entibiases": "entibiar",
    "entibiaste": "entibiar",
    "entibiasteis": "entibiar",
    "entibie": "entibiar",
    "entibiemos": "entibiar",
    "entibien": "entibiar",
    "entibies": "entibiar",
    "entibio": "entibiar",
    "entibiábamos": "entibiar",
    "entibiáis": "entibiar",
    "entibiáramos": "entibiar",
    "entibiáremos": "entibiar",
    "entibiásemos": "entibiar",
    "entibié": "entibiar",
    "entibiéis": "entibiar",
    "entibió": "entibiar",
    "entibo": "entibar",
    "entibos": "entibo",
    "entibábamos": "entibar",
    "entibáis": "entibar",
    "entibáramos": "entibar",
    "entibáremos": "entibar",
    "entibásemos": "entibar",
    "entibé": "entibar",
    "entibéis": "entibar",
    "entibó": "entibar",
    "entice": "entizar",
    "enticemos": "entizar",
    "enticen": "entizar",
    "entices": "entizar",
    "enticé": "entizar",
    "enticéis": "entizar",
    "entidades": "entidad",
    "entienda": "entender",
    "entiendan": "entender",
    "entiendas": "entender",
    "entiende": "entender",
    "entienden": "entender",
    "entiendes": "entender",
    "entiendo": "entender",
    "entierra": "enterrar",
    "entierran": "enterrar",
    "entierras": "enterrar",
    "entierre": "enterrar",
    "entierren": "enterrar",
    "entierres": "enterrar",
    "entierro": "enterrar",
    "entierros": "entierro",
    "entiesa": "entiesar",
    "entiesaba": "entiesar",
    "entiesabais": "entiesar",
    "entiesaban": "entiesar",
    "entiesabas": "entiesar",
    "entiesad": "entiesar",
    "entiesada": "entiesar",
    "entiesadas": "entiesar",
    "entiesado": "entiesar",
    "entiesados": "entiesar",
    "entiesamos": "entiesar",
    "entiesan": "entiesar",
    "entiesando": "entiesar",
    "entiesara": "entiesar",
    "entiesarais": "entiesar",
    "entiesaran": "entiesar",
    "entiesaras": "entiesar",
    "entiesare": "entiesar",
    "entiesareis": "entiesar",
    "entiesaremos": "entiesar",
    "entiesaren": "entiesar",
    "entiesares": "entiesar",
    "entiesaron": "entiesar",
    "entiesará": "entiesar",
    "entiesarán": "entiesar",
    "entiesarás": "entiesar",
    "entiesaré": "entiesar",
    "entiesaréis": "entiesar",
    "entiesaría": "entiesar",
    "entiesaríais": "entiesar",
    "entiesaríamos": "entiesar",
    "entiesarían": "entiesar",
    "entiesarías": "entiesar",
    "entiesas": "entiesar",
    "entiesase": "entiesar",
    "entiesaseis": "entiesar",
    "entiesasen": "entiesar",
    "entiesases": "entiesar",
    "entiesaste": "entiesar",
    "entiesasteis": "entiesar",
    "entiese": "entiesar",
    "entiesemos": "entiesar",
    "entiesen": "entiesar",
    "entieses": "entiesar",
    "entieso": "entiesar",
    "entiesábamos": "entiesar",
    "entiesáis": "entiesar",
    "entiesáramos": "entiesar",
    "entiesáremos": "entiesar",
    "entiesásemos": "entiesar",
    "entiesé": "entiesar",
    "entieséis": "entiesar",
    "entiesó": "entiesar",
    "entimemas": "entimema",
    "entimemática": "entimemático",
    "entimemáticas": "entimemático",
    "entimemáticos": "entimemático",
    "entina": "entinar",
    "entinaba": "entinar",
    "entinabais": "entinar",
    "entinaban": "entinar",
    "entinabas": "entinar",
    "entinad": "entinar",
    "entinada": "entinar",
    "entinadas": "entinar",
    "entinado": "entinar",
    "entinados": "entinar",
    "entinamos": "entinar",
    "entinan": "entinar",
    "entinando": "entinar",
    "entinara": "entinar",
    "entinarais": "entinar",
    "entinaran": "entinar",
    "entinaras": "entinar",
    "entinare": "entinar",
    "entinareis": "entinar",
    "entinaremos": "entinar",
    "entinaren": "entinar",
    "entinares": "entinar",
    "entinaron": "entinar",
    "entinará": "entinar",
    "entinarán": "entinar",
    "entinarás": "entinar",
    "entinaré": "entinar",
    "entinaréis": "entinar",
    "entinaría": "entinar",
    "entinaríais": "entinar",
    "entinaríamos": "entinar",
    "entinarían": "entinar",
    "entinarías": "entinar",
    "entinas": "entinar",
    "entinase": "entinar",
    "entinaseis": "entinar",
    "entinasen": "entinar",
    "entinases": "entinar",
    "entinaste": "entinar",
    "entinasteis": "entinar",
    "entine": "entinar",
    "entinemos": "entinar",
    "entinen": "entinar",
    "entines": "entinar",
    "entino": "entinar",
    "entinta": "entintar",
    "entintaba": "entintar",
    "entintabais": "entintar",
    "entintaban": "entintar",
    "entintabas": "entintar",
    "entintad": "entintar",
    "entintada": "entintar",
    "entintadas": "entintar",
    "entintado": "entintar",
    "entintados": "entintar",
    "entintajes": "entintaje",
    "entintamos": "entintar",
    "entintan": "entintar",
    "entintando": "entintar",
    "entintara": "entintar",
    "entintarais": "entintar",
    "entintaran": "entintar",
    "entintaras": "entintar",
    "entintare": "entintar",
    "entintareis": "entintar",
    "entintaremos": "entintar",
    "entintaren": "entintar",
    "entintares": "entintar",
    "entintaron": "entintar",
    "entintará": "entintar",
    "entintarán": "entintar",
    "entintarás": "entintar",
    "entintaré": "entintar",
    "entintaréis": "entintar",
    "entintaría": "entintar",
    "entintaríais": "entintar",
    "entintaríamos": "entintar",
    "entintarían": "entintar",
    "entintarías": "entintar",
    "entintas": "entintar",
    "entintase": "entintar",
    "entintaseis": "entintar",
    "entintasen": "entintar",
    "entintases": "entintar",
    "entintaste": "entintar",
    "entintasteis": "entintar",
    "entinte": "entintar",
    "entintemos": "entintar",
    "entinten": "entintar",
    "entintes": "entinte",
    "entinto": "entintar",
    "entintábamos": "entintar",
    "entintáis": "entintar",
    "entintáramos": "entintar",
    "entintáremos": "entintar",
    "entintásemos": "entintar",
    "entinté": "entintar",
    "entintéis": "entintar",
    "entintó": "entintar",
    "entinábamos": "entinar",
    "entináis": "entinar",
    "entináramos": "entinar",
    "entináremos": "entinar",
    "entinásemos": "entinar",
    "entiné": "entinar",
    "entinéis": "entinar",
    "entinó": "entinar",
    "entitativa": "entitativo",
    "entitativas": "entitativo",
    "entitativos": "entitativo",
    "entiza": "entizar",
    "entizaba": "entizar",
    "entizabais": "entizar",
    "entizaban": "entizar",
    "entizabas": "entizar",
    "entizad": "entizar",
    "entizada": "entizar",
    "entizadas": "entizar",
    "entizado": "entizar",
    "entizados": "entizar",
    "entizamos": "entizar",
    "entizan": "entizar",
    "entizando": "entizar",
    "entizara": "entizar",
    "entizarais": "entizar",
    "entizaran": "entizar",
    "entizaras": "entizar",
    "entizare": "entizar",
    "entizareis": "entizar",
    "entizaremos": "entizar",
    "entizaren": "entizar",
    "entizares": "entizar",
    "entizaron": "entizar",
    "entizará": "entizar",
    "entizarán": "entizar",
    "entizarás": "entizar",
    "entizaré": "entizar",
    "entizaréis": "entizar",
    "entizaría": "entizar",
    "entizaríais": "entizar",
    "entizaríamos": "entizar",
    "entizarían": "entizar",
    "entizarías": "entizar",
    "entizas": "entizar",
    "entizase": "entizar",
    "entizaseis": "entizar",
    "entizasen": "entizar",
    "entizases": "entizar",
    "entizaste": "entizar",
    "entizasteis": "entizar",
    "entizna": "entiznar",
    "entiznaba": "entiznar",
    "entiznabais": "entiznar",
    "entiznaban": "entiznar",
    "entiznabas": "entiznar",
    "entiznad": "entiznar",
    "entiznada": "entiznar",
    "entiznadas": "entiznar",
    "entiznado": "entiznar",
    "entiznados": "entiznar",
    "entiznamos": "entiznar",
    "entiznan": "entiznar",
    "entiznando": "entiznar",
    "entiznara": "entiznar",
    "entiznarais": "entiznar",
    "entiznaran": "entiznar",
    "entiznaras": "entiznar",
    "entiznare": "entiznar",
    "entiznareis": "entiznar",
    "entiznaremos": "entiznar",
    "entiznaren": "entiznar",
    "entiznares": "entiznar",
    "entiznaron": "entiznar",
    "entiznará": "entiznar",
    "entiznarán": "entiznar",
    "entiznarás": "entiznar",
    "entiznaré": "entiznar",
    "entiznaréis": "entiznar",
    "entiznaría": "entiznar",
    "entiznaríais": "entiznar",
    "entiznaríamos": "entiznar",
    "entiznarían": "entiznar",
    "entiznarías": "entiznar",
    "entiznas": "entiznar",
    "entiznase": "entiznar",
    "entiznaseis": "entiznar",
    "entiznasen": "entiznar",
    "entiznases": "entiznar",
    "entiznaste": "entiznar",
    "entiznasteis": "entiznar",
    "entizne": "entiznar",
    "entiznemos": "entiznar",
    "entiznen": "entiznar",
    "entiznes": "entiznar",
    "entizno": "entiznar",
    "entiznábamos": "entiznar",
    "entiznáis": "entiznar",
    "entiznáramos": "entiznar",
    "entiznáremos": "entiznar",
    "entiznásemos": "entiznar",
    "entizné": "entiznar",
    "entiznéis": "entiznar",
    "entiznó": "entiznar",
    "entizo": "entizar",
    "entizábamos": "entizar",
    "entizáis": "entizar",
    "entizáramos": "entizar",
    "entizáremos": "entizar",
    "entizásemos": "entizar",
    "entizó": "entizar",
    "entlo": "entresuelo",
    "entoftalmías": "entoftalmía",
    "entolda": "entoldar",
    "entoldaba": "entoldar",
    "entoldabais": "entoldar",
    "entoldaban": "entoldar",
    "entoldabas": "entoldar",
    "entoldad": "entoldar",
    "entoldada": "entoldar",
    "entoldadas": "entoldar",
    "entoldado": "entoldar",
    "entoldados": "entoldar",
    "entoldamos": "entoldar",
    "entoldan": "entoldar",
    "entoldando": "entoldar",
    "entoldara": "entoldar",
    "entoldarais": "entoldar",
    "entoldaran": "entoldar",
    "entoldaras": "entoldar",
    "entoldare": "entoldar",
    "entoldareis": "entoldar",
    "entoldaremos": "entoldar",
    "entoldaren": "entoldar",
    "entoldares": "entoldar",
    "entoldaron": "entoldar",
    "entoldará": "entoldar",
    "entoldarán": "entoldar",
    "entoldarás": "entoldar",
    "entoldaré": "entoldar",
    "entoldaréis": "entoldar",
    "entoldaría": "entoldar",
    "entoldaríais": "entoldar",
    "entoldaríamos": "entoldar",
    "entoldarían": "entoldar",
    "entoldarías": "entoldar",
    "entoldas": "entoldar",
    "entoldase": "entoldar",
    "entoldaseis": "entoldar",
    "entoldasen": "entoldar",
    "entoldases": "entoldar",
    "entoldaste": "entoldar",
    "entoldasteis": "entoldar",
    "entolde": "entoldar",
    "entoldemos": "entoldar",
    "entolden": "entoldar",
    "entoldes": "entoldar",
    "entoldo": "entoldar",
    "entoldábamos": "entoldar",
    "entoldáis": "entoldar",
    "entoldáramos": "entoldar",
    "entoldáremos": "entoldar",
    "entoldásemos": "entoldar",
    "entoldé": "entoldar",
    "entoldéis": "entoldar",
    "entoldó": "entoldar",
    "entomofagias": "entomofagia",
    "entomoftoráceas": "entomoftorácea",
    "entomologías": "entomología",
    "entomológica": "entomológico",
    "entomológicas": "entomológico",
    "entomológicos": "entomológico",
    "entomófaga": "entomófago",
    "entomófagas": "entomófago",
    "entomófagos": "entomófago",
    "entomófila": "entomófilo",
    "entomófilas": "entomófilo",
    "entomófilos": "entomófilo",
    "entomóloga": "entomólogo",
    "entomólogas": "entomólogo",
    "entomólogos": "entomólogo",
    "entona": "entonar",
    "entonaba": "entonar",
    "entonabais": "entonar",
    "entonaban": "entonar",
    "entonabas": "entonar",
    "entonaciones": "entonación",
    "entonad": "entonar",
    "entonada": "entonar",
    "entonadas": "entonar",
    "entonaderas": "entonadera",
    "entonado": "entonar",
    "entonadora": "entonador",
    "entonadoras": "entonador",
    "entonadores": "entonador",
    "entonados": "entonar",
    "entonamientos": "entonamiento",
    "entonamos": "entonar",
    "entonan": "entonar",
    "entonando": "entonar",
    "entonara": "entonar",
    "entonarais": "entonar",
    "entonaran": "entonar",
    "entonaras": "entonar",
    "entonare": "entonar",
    "entonareis": "entonar",
    "entonaremos": "entonar",
    "entonaren": "entonar",
    "entonares": "entonar",
    "entonaron": "entonar",
    "entonará": "entonar",
    "entonarán": "entonar",
    "entonarás": "entonar",
    "entonaré": "entonar",
    "entonaréis": "entonar",
    "entonaría": "entonar",
    "entonaríais": "entonar",
    "entonaríamos": "entonar",
    "entonarían": "entonar",
    "entonarías": "entonar",
    "entonas": "entonar",
    "entonase": "entonar",
    "entonaseis": "entonar",
    "entonasen": "entonar",
    "entonases": "entonar",
    "entonaste": "entonar",
    "entonasteis": "entonar",
    "entone": "entonar",
    "entonela": "entonelar",
    "entonelaba": "entonelar",
    "entonelabais": "entonelar",
    "entonelaban": "entonelar",
    "entonelabas": "entonelar",
    "entonelad": "entonelar",
    "entonelada": "entonelar",
    "entoneladas": "entonelar",
    "entonelado": "entonelar",
    "entonelados": "entonelar",
    "entonelamos": "entonelar",
    "entonelan": "entonelar",
    "entonelando": "entonelar",
    "entonelara": "entonelar",
    "entonelarais": "entonelar",
    "entonelaran": "entonelar",
    "entonelaras": "entonelar",
    "entonelare": "entonelar",
    "entonelareis": "entonelar",
    "entonelaremos": "entonelar",
    "entonelaren": "entonelar",
    "entonelares": "entonelar",
    "entonelaron": "entonelar",
    "entonelará": "entonelar",
    "entonelarán": "entonelar",
    "entonelarás": "entonelar",
    "entonelaré": "entonelar",
    "entonelaréis": "entonelar",
    "entonelaría": "entonelar",
    "entonelaríais": "entonelar",
    "entonelaríamos": "entonelar",
    "entonelarían": "entonelar",
    "entonelarías": "entonelar",
    "entonelas": "entonelar",
    "entonelase": "entonelar",
    "entonelaseis": "entonelar",
    "entonelasen": "entonelar",
    "entonelases": "entonelar",
    "entonelaste": "entonelar",
    "entonelasteis": "entonelar",
    "entonele": "entonelar",
    "entonelemos": "entonelar",
    "entonelen": "entonelar",
    "entoneles": "entonelar",
    "entonelo": "entonelar",
    "entonelábamos": "entonelar",
    "entoneláis": "entonelar",
    "entoneláramos": "entonelar",
    "entoneláremos": "entonelar",
    "entonelásemos": "entonelar",
    "entonelé": "entonelar",
    "entoneléis": "entonelar",
    "entoneló": "entonelar",
    "entonemos": "entonar",
    "entonen": "entonar",
    "entones": "entonar",
    "entonga": "entongar",
    "entongaba": "entongar",
    "entongabais": "entongar",
    "entongaban": "entongar",
    "entongabas": "entongar",
    "entongad": "entongar",
    "entongada": "entongar",
    "entongadas": "entongar",
    "entongado": "entongar",
    "entongados": "entongar",
    "entongamos": "entongar",
    "entongan": "entongar",
    "entongando": "entongar",
    "entongara": "entongar",
    "entongarais": "entongar",
    "entongaran": "entongar",
    "entongaras": "entongar",
    "entongare": "entongar",
    "entongareis": "entongar",
    "entongaremos": "entongar",
    "entongaren": "entongar",
    "entongares": "entongar",
    "entongaron": "entongar",
    "entongará": "entongar",
    "entongarán": "entongar",
    "entongarás": "entongar",
    "entongaré": "entongar",
    "entongaréis": "entongar",
    "entongaría": "entongar",
    "entongaríais": "entongar",
    "entongaríamos": "entongar",
    "entongarían": "entongar",
    "entongarías": "entongar",
    "entongas": "entongar",
    "entongase": "entongar",
    "entongaseis": "entongar",
    "entongasen": "entongar",
    "entongases": "entongar",
    "entongaste": "entongar",
    "entongasteis": "entongar",
    "entongo": "entongar",
    "entongue": "entongar",
    "entonguemos": "entongar",
    "entonguen": "entongar",
    "entongues": "entongar",
    "entongué": "entongar",
    "entonguéis": "entongar",
    "entongábamos": "entongar",
    "entongáis": "entongar",
    "entongáramos": "entongar",
    "entongáremos": "entongar",
    "entongásemos": "entongar",
    "entongó": "entongar",
    "entono": "entonar",
    "entonos": "entono",
    "entontece": "entontecer",
    "entonteced": "entontecer",
    "entontecemos": "entontecer",
    "entontecen": "entontecer",
    "entonteceremos": "entontecer",
    "entontecerá": "entontecer",
    "entontecerán": "entontecer",
    "entontecerás": "entontecer",
    "entonteceré": "entontecer",
    "entonteceréis": "entontecer",
    "entontecería": "entontecer",
    "entonteceríais": "entontecer",
    "entonteceríamos": "entontecer",
    "entontecerían": "entontecer",
    "entontecerías": "entontecer",
    "entonteces": "entontecer",
    "entontecida": "entontecer",
    "entontecidas": "entontecer",
    "entontecido": "entontecer",
    "entontecidos": "entontecer",
    "entonteciendo": "entontecer",
    "entonteciera": "entontecer",
    "entontecierais": "entontecer",
    "entontecieran": "entontecer",
    "entontecieras": "entontecer",
    "entonteciere": "entontecer",
    "entonteciereis": "entontecer",
    "entontecieren": "entontecer",
    "entontecieres": "entontecer",
    "entontecieron": "entontecer",
    "entonteciese": "entontecer",
    "entontecieseis": "entontecer",
    "entonteciesen": "entontecer",
    "entontecieses": "entontecer",
    "entontecimientos": "entontecimiento",
    "entontecimos": "entontecer",
    "entonteciste": "entontecer",
    "entontecisteis": "entontecer",
    "entonteciéramos": "entontecer",
    "entonteciéremos": "entontecer",
    "entonteciésemos": "entontecer",
    "entonteció": "entontecer",
    "entontecéis": "entontecer",
    "entontecí": "entontecer",
    "entontecía": "entontecer",
    "entontecíais": "entontecer",
    "entontecíamos": "entontecer",
    "entontecían": "entontecer",
    "entontecías": "entontecer",
    "entontezca": "entontecer",
    "entontezcamos": "entontecer",
    "entontezcan": "entontecer",
    "entontezcas": "entontecer",
    "entontezco": "entontecer",
    "entontezcáis": "entontecer",
    "entonábamos": "entonar",
    "entonáis": "entonar",
    "entonáramos": "entonar",
    "entonáremos": "entonar",
    "entonásemos": "entonar",
    "entoné": "entonar",
    "entonéis": "entonar",
    "entonó": "entonar",
    "entorcha": "entorchar",
    "entorchaba": "entorchar",
    "entorchabais": "entorchar",
    "entorchaban": "entorchar",
    "entorchabas": "entorchar",
    "entorchad": "entorchar",
    "entorchada": "entorchar",
    "entorchadas": "entorchar",
    "entorchado": "entorchar",
    "entorchados": "entorchar",
    "entorchamos": "entorchar",
    "entorchan": "entorchar",
    "entorchando": "entorchar",
    "entorchara": "entorchar",
    "entorcharais": "entorchar",
    "entorcharan": "entorchar",
    "entorcharas": "entorchar",
    "entorchare": "entorchar",
    "entorchareis": "entorchar",
    "entorcharemos": "entorchar",
    "entorcharen": "entorchar",
    "entorchares": "entorchar",
    "entorcharon": "entorchar",
    "entorchará": "entorchar",
    "entorcharán": "entorchar",
    "entorcharás": "entorchar",
    "entorcharé": "entorchar",
    "entorcharéis": "entorchar",
    "entorcharía": "entorchar",
    "entorcharíais": "entorchar",
    "entorcharíamos": "entorchar",
    "entorcharían": "entorchar",
    "entorcharías": "entorchar",
    "entorchas": "entorchar",
    "entorchase": "entorchar",
    "entorchaseis": "entorchar",
    "entorchasen": "entorchar",
    "entorchases": "entorchar",
    "entorchaste": "entorchar",
    "entorchasteis": "entorchar",
    "entorche": "entorchar",
    "entorchemos": "entorchar",
    "entorchen": "entorchar",
    "entorches": "entorchar",
    "entorcho": "entorchar",
    "entorchábamos": "entorchar",
    "entorcháis": "entorchar",
    "entorcháramos": "entorchar",
    "entorcháremos": "entorchar",
    "entorchásemos": "entorchar",
    "entorché": "entorchar",
    "entorchéis": "entorchar",
    "entorchó": "entorchar",
    "entorila": "entorilar",
    "entorilaba": "entorilar",
    "entorilabais": "entorilar",
    "entorilaban": "entorilar",
    "entorilabas": "entorilar",
    "entorilad": "entorilar",
    "entorilada": "entorilar",
    "entoriladas": "entorilar",
    "entorilado": "entorilar",
    "entorilados": "entorilar",
    "entorilamos": "entorilar",
    "entorilan": "entorilar",
    "entorilando": "entorilar",
    "entorilara": "entorilar",
    "entorilarais": "entorilar",
    "entorilaran": "entorilar",
    "entorilaras": "entorilar",
    "entorilare": "entorilar",
    "entorilareis": "entorilar",
    "entorilaremos": "entorilar",
    "entorilaren": "entorilar",
    "entorilares": "entorilar",
    "entorilaron": "entorilar",
    "entorilará": "entorilar",
    "entorilarán": "entorilar",
    "entorilarás": "entorilar",
    "entorilaré": "entorilar",
    "entorilaréis": "entorilar",
    "entorilaría": "entorilar",
    "entorilaríais": "entorilar",
    "entorilaríamos": "entorilar",
    "entorilarían": "entorilar",
    "entorilarías": "entorilar",
    "entorilas": "entorilar",
    "entorilase": "entorilar",
    "entorilaseis": "entorilar",
    "entorilasen": "entorilar",
    "entorilases": "entorilar",
    "entorilaste": "entorilar",
    "entorilasteis": "entorilar",
    "entorile": "entorilar",
    "entorilemos": "entorilar",
    "entorilen": "entorilar",
    "entoriles": "entorilar",
    "entorilo": "entorilar",
    "entorilábamos": "entorilar",
    "entoriláis": "entorilar",
    "entoriláramos": "entorilar",
    "entoriláremos": "entorilar",
    "entorilásemos": "entorilar",
    "entorilé": "entorilar",
    "entoriléis": "entorilar",
    "entoriló": "entorilar",
    "entorna": "entornar",
    "entornaba": "entornar",
    "entornabais": "entornar",
    "entornaban": "entornar",
    "entornabas": "entornar",
    "entornad": "entornar",
    "entornada": "entornar",
    "entornadas": "entornar",
    "entornado": "entornar",
    "entornados": "entornar",
    "entornamos": "entornar",
    "entornan": "entornar",
    "entornando": "entornar",
    "entornara": "entornar",
    "entornarais": "entornar",
    "entornaran": "entornar",
    "entornaras": "entornar",
    "entornare": "entornar",
    "entornareis": "entornar",
    "entornaremos": "entornar",
    "entornaren": "entornar",
    "entornares": "entornar",
    "entornaron": "entornar",
    "entornará": "entornar",
    "entornarán": "entornar",
    "entornarás": "entornar",
    "entornaré": "entornar",
    "entornaréis": "entornar",
    "entornaría": "entornar",
    "entornaríais": "entornar",
    "entornaríamos": "entornar",
    "entornarían": "entornar",
    "entornarías": "entornar",
    "entornas": "entornar",
    "entornase": "entornar",
    "entornaseis": "entornar",
    "entornasen": "entornar",
    "entornases": "entornar",
    "entornaste": "entornar",
    "entornasteis": "entornar",
    "entorne": "entornar",
    "entornemos": "entornar",
    "entornen": "entornar",
    "entornes": "entornar",
    "entorno": "entornar",
    "entornos": "entorno",
    "entornábamos": "entornar",
    "entornáis": "entornar",
    "entornáramos": "entornar",
    "entornáremos": "entornar",
    "entornásemos": "entornar",
    "entorné": "entornar",
    "entornéis": "entornar",
    "entornó": "entornar",
    "entorpece": "entorpecer",
    "entorpeced": "entorpecer",
    "entorpecemos": "entorpecer",
    "entorpecen": "entorpecer",
    "entorpeceremos": "entorpecer",
    "entorpecerá": "entorpecer",
    "entorpecerán": "entorpecer",
    "entorpecerás": "entorpecer",
    "entorpeceré": "entorpecer",
    "entorpeceréis": "entorpecer",
    "entorpecería": "entorpecer",
    "entorpeceríais": "entorpecer",
    "entorpeceríamos": "entorpecer",
    "entorpecerían": "entorpecer",
    "entorpecerías": "entorpecer",
    "entorpeces": "entorpecer",
    "entorpecida": "entorpecer",
    "entorpecidas": "entorpecer",
    "entorpecido": "entorpecer",
    "entorpecidos": "entorpecer",
    "entorpeciendo": "entorpecer",
    "entorpeciera": "entorpecer",
    "entorpecierais": "entorpecer",
    "entorpecieran": "entorpecer",
    "entorpecieras": "entorpecer",
    "entorpeciere": "entorpecer",
    "entorpeciereis": "entorpecer",
    "entorpecieren": "entorpecer",
    "entorpecieres": "entorpecer",
    "entorpecieron": "entorpecer",
    "entorpeciese": "entorpecer",
    "entorpecieseis": "entorpecer",
    "entorpeciesen": "entorpecer",
    "entorpecieses": "entorpecer",
    "entorpecimientos": "entorpecimiento",
    "entorpecimos": "entorpecer",
    "entorpeciste": "entorpecer",
    "entorpecisteis": "entorpecer",
    "entorpeciéramos": "entorpecer",
    "entorpeciéremos": "entorpecer",
    "entorpeciésemos": "entorpecer",
    "entorpeció": "entorpecer",
    "entorpecéis": "entorpecer",
    "entorpecí": "entorpecer",
    "entorpecía": "entorpecer",
    "entorpecíais": "entorpecer",
    "entorpecíamos": "entorpecer",
    "entorpecían": "entorpecer",
    "entorpecías": "entorpecer",
    "entorpezca": "entorpecer",
    "entorpezcamos": "entorpecer",
    "entorpezcan": "entorpecer",
    "entorpezcas": "entorpecer",
    "entorpezco": "entorpecer",
    "entorpezcáis": "entorpecer",
    "entortaba": "entortar",
    "entortabais": "entortar",
    "entortaban": "entortar",
    "entortabas": "entortar",
    "entortad": "entortar",
    "entortada": "entortar",
    "entortadas": "entortar",
    "entortado": "entortar",
    "entortados": "entortar",
    "entortamos": "entortar",
    "entortando": "entortar",
    "entortara": "entortar",
    "entortarais": "entortar",
    "entortaran": "entortar",
    "entortaras": "entortar",
    "entortare": "entortar",
    "entortareis": "entortar",
    "entortaremos": "entortar",
    "entortaren": "entortar",
    "entortares": "entortar",
    "entortaron": "entortar",
    "entortará": "entortar",
    "entortarán": "entortar",
    "entortarás": "entortar",
    "entortaré": "entortar",
    "entortaréis": "entortar",
    "entortaría": "entortar",
    "entortaríais": "entortar",
    "entortaríamos": "entortar",
    "entortarían": "entortar",
    "entortarías": "entortar",
    "entortase": "entortar",
    "entortaseis": "entortar",
    "entortasen": "entortar",
    "entortases": "entortar",
    "entortaste": "entortar",
    "entortasteis": "entortar",
    "entortemos": "entortar",
    "entortábamos": "entortar",
    "entortáis": "entortar",
    "entortáramos": "entortar",
    "entortáremos": "entortar",
    "entortásemos": "entortar",
    "entorté": "entortar",
    "entortéis": "entortar",
    "entortó": "entortar",
    "entosiga": "entosigar",
    "entosigaba": "entosigar",
    "entosigabais": "entosigar",
    "entosigaban": "entosigar",
    "entosigabas": "entosigar",
    "entosigad": "entosigar",
    "entosigada": "entosigar",
    "entosigadas": "entosigar",
    "entosigado": "entosigar",
    "entosigados": "entosigar",
    "entosigamos": "entosigar",
    "entosigan": "entosigar",
    "entosigando": "entosigar",
    "entosigara": "entosigar",
    "entosigarais": "entosigar",
    "entosigaran": "entosigar",
    "entosigaras": "entosigar",
    "entosigare": "entosigar",
    "entosigareis": "entosigar",
    "entosigaremos": "entosigar",
    "entosigaren": "entosigar",
    "entosigares": "entosigar",
    "entosigaron": "entosigar",
    "entosigará": "entosigar",
    "entosigarán": "entosigar",
    "entosigarás": "entosigar",
    "entosigaré": "entosigar",
    "entosigaréis": "entosigar",
    "entosigaría": "entosigar",
    "entosigaríais": "entosigar",
    "entosigaríamos": "entosigar",
    "entosigarían": "entosigar",
    "entosigarías": "entosigar",
    "entosigas": "entosigar",
    "entosigase": "entosigar",
    "entosigaseis": "entosigar",
    "entosigasen": "entosigar",
    "entosigases": "entosigar",
    "entosigaste": "entosigar",
    "entosigasteis": "entosigar",
    "entosigo": "entosigar",
    "entosigue": "entosigar",
    "entosiguemos": "entosigar",
    "entosiguen": "entosigar",
    "entosigues": "entosigar",
    "entosigué": "entosigar",
    "entosiguéis": "entosigar",
    "entosigábamos": "entosigar",
    "entosigáis": "entosigar",
    "entosigáramos": "entosigar",
    "entosigáremos": "entosigar",
    "entosigásemos": "entosigar",
    "entosigó": "entosigar",
    "entra": "entrar",
    "entraba": "entrar",
    "entrababa": "entrabar",
    "entrababais": "entrabar",
    "entrababan": "entrabar",
    "entrababas": "entrabar",
    "entrabad": "entrabar",
    "entrabada": "entrabar",
    "entrabadas": "entrabar",
    "entrabado": "entrabar",
    "entrabados": "entrabar",
    "entrabais": "entrar",
    "entrabamos": "entrabar",
    "entraban": "entrar",
    "entrabando": "entrabar",
    "entrabara": "entrabar",
    "entrabarais": "entrabar",
    "entrabaran": "entrabar",
    "entrabaras": "entrabar",
    "entrabare": "entrabar",
    "entrabareis": "entrabar",
    "entrabaremos": "entrabar",
    "entrabaren": "entrabar",
    "entrabares": "entrabar",
    "entrabaron": "entrabar",
    "entrabará": "entrabar",
    "entrabarán": "entrabar",
    "entrabarás": "entrabar",
    "entrabaré": "entrabar",
    "entrabaréis": "entrabar",
    "entrabaría": "entrabar",
    "entrabaríais": "entrabar",
    "entrabaríamos": "entrabar",
    "entrabarían": "entrabar",
    "entrabarías": "entrabar",
    "entrabas": "entrar",
    "entrabase": "entrabar",
    "entrabaseis": "entrabar",
    "entrabasen": "entrabar",
    "entrabases": "entrabar",
    "entrabaste": "entrabar",
    "entrabasteis": "entrabar",
    "entrabe": "entrabar",
    "entrabemos": "entrabar",
    "entraben": "entrabar",
    "entrabes": "entrabar",
    "entrabierta": "entreabrir",
    "entrabiertas": "entreabrir",
    "entrabierto": "entreabrir",
    "entrabiertos": "entreabrir",
    "entrabo": "entrabar",
    "entrabábamos": "entrabar",
    "entrabáis": "entrabar",
    "entrabáramos": "entrabar",
    "entrabáremos": "entrabar",
    "entrabásemos": "entrabar",
    "entrabé": "entrabar",
    "entrabéis": "entrabar",
    "entrabó": "entrabar",
    "entrad": "entrar",
    "entrada": "entrar",
    "entradas": "entrar",
    "entrado": "entrar",
    "entradora": "entrador",
    "entradoras": "entrador",
    "entradores": "entrador",
    "entrados": "entrar",
    "entrama": "entramar",
    "entramaba": "entramar",
    "entramabais": "entramar",
    "entramaban": "entramar",
    "entramabas": "entramar",
    "entramad": "entramar",
    "entramada": "entramar",
    "entramadas": "entramar",
    "entramado": "entramar",
    "entramados": "entramar",
    "entramamos": "entramar",
    "entraman": "entramar",
    "entramando": "entramar",
    "entramara": "entramar",
    "entramarais": "entramar",
    "entramaran": "entramar",
    "entramaras": "entramar",
    "entramare": "entramar",
    "entramareis": "entramar",
    "entramaremos": "entramar",
    "entramaren": "entramar",
    "entramares": "entramar",
    "entramaron": "entramar",
    "entramará": "entramar",
    "entramarán": "entramar",
    "entramarás": "entramar",
    "entramaré": "entramar",
    "entramaréis": "entramar",
    "entramaría": "entramar",
    "entramaríais": "entramar",
    "entramaríamos": "entramar",
    "entramarían": "entramar",
    "entramarías": "entramar",
    "entramas": "entramar",
    "entramase": "entramar",
    "entramaseis": "entramar",
    "entramasen": "entramar",
    "entramases": "entramar",
    "entramaste": "entramar",
    "entramasteis": "entramar",
    "entrame": "entramar",
    "entramemos": "entramar",
    "entramen": "entramar",
    "entrames": "entramar",
    "entramo": "entramar",
    "entramos": "entrar",
    "entrampa": "entrampar",
    "entrampaba": "entrampar",
    "entrampabais": "entrampar",
    "entrampaban": "entrampar",
    "entrampabas": "entrampar",
    "entrampad": "entrampar",
    "entrampada": "entrampar",
    "entrampadas": "entrampar",
    "entrampado": "entrampar",
    "entrampados": "entrampar",
    "entrampamientos": "entrampamiento",
    "entrampamos": "entrampar",
    "entrampan": "entrampar",
    "entrampando": "entrampar",
    "entrampara": "entrampar",
    "entramparais": "entrampar",
    "entramparan": "entrampar",
    "entramparas": "entrampar",
    "entrampare": "entrampar",
    "entrampareis": "entrampar",
    "entramparemos": "entrampar",
    "entramparen": "entrampar",
    "entrampares": "entrampar",
    "entramparon": "entrampar",
    "entrampará": "entrampar",
    "entramparán": "entrampar",
    "entramparás": "entrampar",
    "entramparé": "entrampar",
    "entramparéis": "entrampar",
    "entramparía": "entrampar",
    "entramparíais": "entrampar",
    "entramparíamos": "entrampar",
    "entramparían": "entrampar",
    "entramparías": "entrampar",
    "entrampas": "entrampar",
    "entrampase": "entrampar",
    "entrampaseis": "entrampar",
    "entrampasen": "entrampar",
    "entrampases": "entrampar",
    "entrampaste": "entrampar",
    "entrampasteis": "entrampar",
    "entrampe": "entrampar",
    "entrampemos": "entrampar",
    "entrampen": "entrampar",
    "entrampes": "entrampar",
    "entrampo": "entrampar",
    "entrampábamos": "entrampar",
    "entrampáis": "entrampar",
    "entrampáramos": "entrampar",
    "entrampáremos": "entrampar",
    "entrampásemos": "entrampar",
    "entrampé": "entrampar",
    "entrampéis": "entrampar",
    "entrampó": "entrampar",
    "entramábamos": "entramar",
    "entramáis": "entramar",
    "entramáramos": "entramar",
    "entramáremos": "entramar",
    "entramásemos": "entramar",
    "entramé": "entramar",
    "entraméis": "entramar",
    "entramó": "entramar",
    "entran": "entrar",
    "entrando": "entrar",
    "entrantes": "entrante",
    "entrara": "entrar",
    "entrarais": "entrar",
    "entraran": "entrar",
    "entraras": "entrar",
    "entrare": "entrar",
    "entrareis": "entrar",
    "entraremos": "entrar",
    "entraren": "entrar",
    "entrares": "entrar",
    "entraron": "entrar",
    "entrará": "entrar",
    "entrarán": "entrar",
    "entrarás": "entrar",
    "entraré": "entrar",
    "entraréis": "entrar",
    "entraría": "entrar",
    "entraríais": "entrar",
    "entraríamos": "entrar",
    "entrarían": "entrar",
    "entrarías": "entrar",
    "entras": "entrar",
    "entrase": "entrar",
    "entraseis": "entrar",
    "entrasen": "entrar",
    "entrases": "entrar",
    "entraste": "entrar",
    "entrasteis": "entrar",
    "entraña": "entrañar",
    "entrañaba": "entrañar",
    "entrañabais": "entrañar",
    "entrañaban": "entrañar",
    "entrañabas": "entrañar",
    "entrañables": "entrañable",
    "entrañad": "entrañar",
    "entrañada": "entrañar",
    "entrañadas": "entrañar",
    "entrañado": "entrañar",
    "entrañados": "entrañar",
    "entrañamos": "entrañar",
    "entrañan": "entrañar",
    "entrañando": "entrañar",
    "entrañara": "entrañar",
    "entrañarais": "entrañar",
    "entrañaran": "entrañar",
    "entrañaras": "entrañar",
    "entrañare": "entrañar",
    "entrañareis": "entrañar",
    "entrañaremos": "entrañar",
    "entrañaren": "entrañar",
    "entrañares": "entrañar",
    "entrañaron": "entrañar",
    "entrañará": "entrañar",
    "entrañarán": "entrañar",
    "entrañarás": "entrañar",
    "entrañaré": "entrañar",
    "entrañaréis": "entrañar",
    "entrañaría": "entrañar",
    "entrañaríais": "entrañar",
    "entrañaríamos": "entrañar",
    "entrañarían": "entrañar",
    "entrañarías": "entrañar",
    "entrañas": "entrañar",
    "entrañase": "entrañar",
    "entrañaseis": "entrañar",
    "entrañasen": "entrañar",
    "entrañases": "entrañar",
    "entrañaste": "entrañar",
    "entrañasteis": "entrañar",
    "entrañe": "entrañar",
    "entrañemos": "entrañar",
    "entrañen": "entrañar",
    "entrañes": "entrañar",
    "entraño": "entrañar",
    "entrañábamos": "entrañar",
    "entrañáis": "entrañar",
    "entrañáramos": "entrañar",
    "entrañáremos": "entrañar",
    "entrañásemos": "entrañar",
    "entrañé": "entrañar",
    "entrañéis": "entrañar",
    "entrañó": "entrañar",
    "entre": "entrar",
    "entreabra": "entreabrir",
    "entreabramos": "entreabrir",
    "entreabran": "entreabrir",
    "entreabras": "entreabrir",
    "entreabre": "entreabrir",
    "entreabren": "entreabrir",
    "entreabres": "entreabrir",
    "entreabrid": "entreabrir",
    "entreabriendo": "entreabrir",
    "entreabriera": "entreabrir",
    "entreabrierais": "entreabrir",
    "entreabrieran": "entreabrir",
    "entreabrieras": "entreabrir",
    "entreabriere": "entreabrir",
    "entreabriereis": "entreabrir",
    "entreabrieren": "entreabrir",
    "entreabrieres": "entreabrir",
    "entreabrieron": "entreabrir",
    "entreabriese": "entreabrir",
    "entreabrieseis": "entreabrir",
    "entreabriesen": "entreabrir",
    "entreabrieses": "entreabrir",
    "entreabrimos": "entreabrir",
    "entreabriremos": "entreabrir",
    "entreabrirá": "entreabrir",
    "entreabrirán": "entreabrir",
    "entreabrirás": "entreabrir",
    "entreabriré": "entreabrir",
    "entreabriréis": "entreabrir",
    "entreabriría": "entreabrir",
    "entreabriríais": "entreabrir",
    "entreabriríamos": "entreabrir",
    "entreabrirían": "entreabrir",
    "entreabrirías": "entreabrir",
    "entreabriste": "entreabrir",
    "entreabristeis": "entreabrir",
    "entreabriéramos": "entreabrir",
    "entreabriéremos": "entreabrir",
    "entreabriésemos": "entreabrir",
    "entreabrió": "entreabrir",
    "entreabro": "entreabrir",
    "entreabráis": "entreabrir",
    "entreabrí": "entreabrir",
    "entreabría": "entreabrir",
    "entreabríais": "entreabrir",
    "entreabríamos": "entreabrir",
    "entreabrían": "entreabrir",
    "entreabrías": "entreabrir",
    "entreabrís": "entreabrir",
    "entreactos": "entreacto",
    "entreancha": "entreancho",
    "entreanchas": "entreancho",
    "entreanchos": "entreancho",
    "entreayuda": "entreayudar",
    "entreayudaba": "entreayudar",
    "entreayudabais": "entreayudar",
    "entreayudaban": "entreayudar",
    "entreayudabas": "entreayudar",
    "entreayudad": "entreayudar",
    "entreayudada": "entreayudar",
    "entreayudadas": "entreayudar",
    "entreayudado": "entreayudar",
    "entreayudados": "entreayudar",
    "entreayudamos": "entreayudar",
    "entreayudan": "entreayudar",
    "entreayudando": "entreayudar",
    "entreayudara": "entreayudar",
    "entreayudarais": "entreayudar",
    "entreayudaran": "entreayudar",
    "entreayudaras": "entreayudar",
    "entreayudare": "entreayudar",
    "entreayudareis": "entreayudar",
    "entreayudaremos": "entreayudar",
    "entreayudaren": "entreayudar",
    "entreayudares": "entreayudar",
    "entreayudaron": "entreayudar",
    "entreayudará": "entreayudar",
    "entreayudarán": "entreayudar",
    "entreayudarás": "entreayudar",
    "entreayudaré": "entreayudar",
    "entreayudaréis": "entreayudar",
    "entreayudaría": "entreayudar",
    "entreayudaríais": "entreayudar",
    "entreayudaríamos": "entreayudar",
    "entreayudarían": "entreayudar",
    "entreayudarías": "entreayudar",
    "entreayudas": "entreayudar",
    "entreayudase": "entreayudar",
    "entreayudaseis": "entreayudar",
    "entreayudasen": "entreayudar",
    "entreayudases": "entreayudar",
    "entreayudaste": "entreayudar",
    "entreayudasteis": "entreayudar",
    "entreayude": "entreayudar",
    "entreayudemos": "entreayudar",
    "entreayuden": "entreayudar",
    "entreayudes": "entreayudar",
    "entreayudo": "entreayudar",
    "entreayudábamos": "entreayudar",
    "entreayudáis": "entreayudar",
    "entreayudáramos": "entreayudar",
    "entreayudáremos": "entreayudar",
    "entreayudásemos": "entreayudar",
    "entreayudé": "entreayudar",
    "entreayudéis": "entreayudar",
    "entreayudó": "entreayudar",
    "entrebarreras": "entrebarrera",
    "entrecalles": "entrecalle",
    "entrecana": "entrecano",
    "entrecanales": "entrecanal",
    "entrecanas": "entrecano",
    "entrecanos": "entrecano",
    "entrecava": "entrecavar",
    "entrecavaba": "entrecavar",
    "entrecavabais": "entrecavar",
    "entrecavaban": "entrecavar",
    "entrecavabas": "entrecavar",
    "entrecavad": "entrecavar",
    "entrecavada": "entrecavar",
    "entrecavadas": "entrecavar",
    "entrecavado": "entrecavar",
    "entrecavados": "entrecavar",
    "entrecavamos": "entrecavar",
    "entrecavan": "entrecavar",
    "entrecavando": "entrecavar",
    "entrecavara": "entrecavar",
    "entrecavarais": "entrecavar",
    "entrecavaran": "entrecavar",
    "entrecavaras": "entrecavar",
    "entrecavare": "entrecavar",
    "entrecavareis": "entrecavar",
    "entrecavaremos": "entrecavar",
    "entrecavaren": "entrecavar",
    "entrecavares": "entrecavar",
    "entrecavaron": "entrecavar",
    "entrecavará": "entrecavar",
    "entrecavarán": "entrecavar",
    "entrecavarás": "entrecavar",
    "entrecavaré": "entrecavar",
    "entrecavaréis": "entrecavar",
    "entrecavaría": "entrecavar",
    "entrecavaríais": "entrecavar",
    "entrecavaríamos": "entrecavar",
    "entrecavarían": "entrecavar",
    "entrecavarías": "entrecavar",
    "entrecavas": "entrecavar",
    "entrecavase": "entrecavar",
    "entrecavaseis": "entrecavar",
    "entrecavasen": "entrecavar",
    "entrecavases": "entrecavar",
    "entrecavaste": "entrecavar",
    "entrecavasteis": "entrecavar",
    "entrecave": "entrecavar",
    "entrecavemos": "entrecavar",
    "entrecaven": "entrecavar",
    "entrecaves": "entrecavar",
    "entrecavo": "entrecavar",
    "entrecavábamos": "entrecavar",
    "entrecaváis": "entrecavar",
    "entrecaváramos": "entrecavar",
    "entrecaváremos": "entrecavar",
    "entrecavásemos": "entrecavar",
    "entrecavé": "entrecavar",
    "entrecavéis": "entrecavar",
    "entrecavó": "entrecavar",
    "entrecejos": "entrecejo",
    "entrecercas": "entrecerca",
    "entrecerraba": "entrecerrar",
    "entrecerrabais": "entrecerrar",
    "entrecerraban": "entrecerrar",
    "entrecerrabas": "entrecerrar",
    "entrecerrad": "entrecerrar",
    "entrecerrada": "entrecerrar",
    "entrecerradas": "entrecerrar",
    "entrecerrado": "entrecerrar",
    "entrecerrados": "entrecerrar",
    "entrecerramos": "entrecerrar",
    "entrecerrando": "entrecerrar",
    "entrecerrara": "entrecerrar",
    "entrecerrarais": "entrecerrar",
    "entrecerraran": "entrecerrar",
    "entrecerraras": "entrecerrar",
    "entrecerrare": "entrecerrar",
    "entrecerrareis": "entrecerrar",
    "entrecerraremos": "entrecerrar",
    "entrecerraren": "entrecerrar",
    "entrecerrares": "entrecerrar",
    "entrecerraron": "entrecerrar",
    "entrecerrará": "entrecerrar",
    "entrecerrarán": "entrecerrar",
    "entrecerrarás": "entrecerrar",
    "entrecerraré": "entrecerrar",
    "entrecerraréis": "entrecerrar",
    "entrecerraría": "entrecerrar",
    "entrecerraríais": "entrecerrar",
    "entrecerraríamos": "entrecerrar",
    "entrecerrarían": "entrecerrar",
    "entrecerrarías": "entrecerrar",
    "entrecerrase": "entrecerrar",
    "entrecerraseis": "entrecerrar",
    "entrecerrasen": "entrecerrar",
    "entrecerrases": "entrecerrar",
    "entrecerraste": "entrecerrar",
    "entrecerrasteis": "entrecerrar",
    "entrecerremos": "entrecerrar",
    "entrecerrábamos": "entrecerrar",
    "entrecerráis": "entrecerrar",
    "entrecerráramos": "entrecerrar",
    "entrecerráremos": "entrecerrar",
    "entrecerrásemos": "entrecerrar",
    "entrecerré": "entrecerrar",
    "entrecerréis": "entrecerrar",
    "entrecerró": "entrecerrar",
    "entrechates": "entrechat",
    "entrechats": "entrechat",
    "entrechoca": "entrechocar",
    "entrechocaba": "entrechocar",
    "entrechocabais": "entrechocar",
    "entrechocaban": "entrechocar",
    "entrechocabas": "entrechocar",
    "entrechocad": "entrechocar",
    "entrechocada": "entrechocar",
    "entrechocadas": "entrechocar",
    "entrechocado": "entrechocar",
    "entrechocados": "entrechocar",
    "entrechocamos": "entrechocar",
    "entrechocan": "entrechocar",
    "entrechocando": "entrechocar",
    "entrechocara": "entrechocar",
    "entrechocarais": "entrechocar",
    "entrechocaran": "entrechocar",
    "entrechocaras": "entrechocar",
    "entrechocare": "entrechocar",
    "entrechocareis": "entrechocar",
    "entrechocaremos": "entrechocar",
    "entrechocaren": "entrechocar",
    "entrechocares": "entrechocar",
    "entrechocaron": "entrechocar",
    "entrechocará": "entrechocar",
    "entrechocarán": "entrechocar",
    "entrechocarás": "entrechocar",
    "entrechocaré": "entrechocar",
    "entrechocaréis": "entrechocar",
    "entrechocaría": "entrechocar",
    "entrechocaríais": "entrechocar",
    "entrechocaríamos": "entrechocar",
    "entrechocarían": "entrechocar",
    "entrechocarías": "entrechocar",
    "entrechocas": "entrechocar",
    "entrechocase": "entrechocar",
    "entrechocaseis": "entrechocar",
    "entrechocasen": "entrechocar",
    "entrechocases": "entrechocar",
    "entrechocaste": "entrechocar",
    "entrechocasteis": "entrechocar",
    "entrechoco": "entrechocar",
    "entrechocábamos": "entrechocar",
    "entrechocáis": "entrechocar",
    "entrechocáramos": "entrechocar",
    "entrechocáremos": "entrechocar",
    "entrechocásemos": "entrechocar",
    "entrechocó": "entrechocar",
    "entrechoque": "entrechocar",
    "entrechoquemos": "entrechocar",
    "entrechoquen": "entrechocar",
    "entrechoques": "entrechocar",
    "entrechoqué": "entrechocar",
    "entrechoquéis": "entrechocar",
    "entrecierra": "entrecerrar",
    "entrecierran": "entrecerrar",
    "entrecierras": "entrecerrar",
    "entrecierre": "entrecerrar",
    "entrecierren": "entrecerrar",
    "entrecierres": "entrecerrar",
    "entrecierro": "entrecerrar",
    "entrecintas": "entrecinta",
    "entreclara": "entreclaro",
    "entreclaras": "entreclaro",
    "entreclaros": "entreclaro",
    "entrecogeduras": "entrecogedura",
    "entrecoma": "entrecomar",
    "entrecomaba": "entrecomar",
    "entrecomabais": "entrecomar",
    "entrecomaban": "entrecomar",
    "entrecomabas": "entrecomar",
    "entrecomad": "entrecomar",
    "entrecomada": "entrecomar",
    "entrecomadas": "entrecomar",
    "entrecomado": "entrecomar",
    "entrecomados": "entrecomar",
    "entrecomamos": "entrecomar",
    "entrecoman": "entrecomar",
    "entrecomando": "entrecomar",
    "entrecomara": "entrecomar",
    "entrecomarais": "entrecomar",
    "entrecomaran": "entrecomar",
    "entrecomaras": "entrecomar",
    "entrecomare": "entrecomar",
    "entrecomareis": "entrecomar",
    "entrecomaremos": "entrecomar",
    "entrecomaren": "entrecomar",
    "entrecomares": "entrecomar",
    "entrecomaron": "entrecomar",
    "entrecomará": "entrecomar",
    "entrecomarán": "entrecomar",
    "entrecomarás": "entrecomar",
    "entrecomaré": "entrecomar",
    "entrecomaréis": "entrecomar",
    "entrecomaría": "entrecomar",
    "entrecomaríais": "entrecomar",
    "entrecomaríamos": "entrecomar",
    "entrecomarían": "entrecomar",
    "entrecomarías": "entrecomar",
    "entrecomas": "entrecomar",
    "entrecomase": "entrecomar",
    "entrecomaseis": "entrecomar",
    "entrecomasen": "entrecomar",
    "entrecomases": "entrecomar",
    "entrecomaste": "entrecomar",
    "entrecomasteis": "entrecomar",
    "entrecome": "entrecomar",
    "entrecomemos": "entrecomar",
    "entrecomen": "entrecomar",
    "entrecomes": "entrecomar",
    "entrecomilla": "entrecomillar",
    "entrecomillaba": "entrecomillar",
    "entrecomillabais": "entrecomillar",
    "entrecomillaban": "entrecomillar",
    "entrecomillabas": "entrecomillar",
    "entrecomillad": "entrecomillar",
    "entrecomillada": "entrecomillar",
    "entrecomilladas": "entrecomillar",
    "entrecomillado": "entrecomillar",
    "entrecomillados": "entrecomillar",
    "entrecomillamos": "entrecomillar",
    "entrecomillan": "entrecomillar",
    "entrecomillando": "entrecomillar",
    "entrecomillara": "entrecomillar",
    "entrecomillarais": "entrecomillar",
    "entrecomillaran": "entrecomillar",
    "entrecomillaras": "entrecomillar",
    "entrecomillare": "entrecomillar",
    "entrecomillareis": "entrecomillar",
    "entrecomillaremos": "entrecomillar",
    "entrecomillaren": "entrecomillar",
    "entrecomillares": "entrecomillar",
    "entrecomillaron": "entrecomillar",
    "entrecomillará": "entrecomillar",
    "entrecomillarán": "entrecomillar",
    "entrecomillarás": "entrecomillar",
    "entrecomillaré": "entrecomillar",
    "entrecomillaréis": "entrecomillar",
    "entrecomillaría": "entrecomillar",
    "entrecomillaríais": "entrecomillar",
    "entrecomillaríamos": "entrecomillar",
    "entrecomillarían": "entrecomillar",
    "entrecomillarías": "entrecomillar",
    "entrecomillas": "entrecomillar",
    "entrecomillase": "entrecomillar",
    "entrecomillaseis": "entrecomillar",
    "entrecomillasen": "entrecomillar",
    "entrecomillases": "entrecomillar",
    "entrecomillaste": "entrecomillar",
    "entrecomillasteis": "entrecomillar",
    "entrecomille": "entrecomillar",
    "entrecomillemos": "entrecomillar",
    "entrecomillen": "entrecomillar",
    "entrecomilles": "entrecomillar",
    "entrecomillo": "entrecomillar",
    "entrecomillábamos": "entrecomillar",
    "entrecomilláis": "entrecomillar",
    "entrecomilláramos": "entrecomillar",
    "entrecomilláremos": "entrecomillar",
    "entrecomillásemos": "entrecomillar",
    "entrecomillé": "entrecomillar",
    "entrecomilléis": "entrecomillar",
    "entrecomilló": "entrecomillar",
    "entrecomo": "entrecomar",
    "entrecomábamos": "entrecomar",
    "entrecomáis": "entrecomar",
    "entrecomáramos": "entrecomar",
    "entrecomáremos": "entrecomar",
    "entrecomásemos": "entrecomar",
    "entrecomé": "entrecomar",
    "entrecoméis": "entrecomar",
    "entrecomó": "entrecomar",
    "entrecoros": "entrecoro",
    "entrecorta": "entrecortar",
    "entrecortaba": "entrecortar",
    "entrecortabais": "entrecortar",
    "entrecortaban": "entrecortar",
    "entrecortabas": "entrecortar",
    "entrecortad": "entrecortar",
    "entrecortada": "entrecortar",
    "entrecortadas": "entrecortar",
    "entrecortado": "entrecortar",
    "entrecortados": "entrecortar",
    "entrecortaduras": "entrecortadura",
    "entrecortamos": "entrecortar",
    "entrecortan": "entrecortar",
    "entrecortando": "entrecortar",
    "entrecortara": "entrecortar",
    "entrecortarais": "entrecortar",
    "entrecortaran": "entrecortar",
    "entrecortaras": "entrecortar",
    "entrecortare": "entrecortar",
    "entrecortareis": "entrecortar",
    "entrecortaremos": "entrecortar",
    "entrecortaren": "entrecortar",
    "entrecortares": "entrecortar",
    "entrecortaron": "entrecortar",
    "entrecortará": "entrecortar",
    "entrecortarán": "entrecortar",
    "entrecortarás": "entrecortar",
    "entrecortaré": "entrecortar",
    "entrecortaréis": "entrecortar",
    "entrecortaría": "entrecortar",
    "entrecortaríais": "entrecortar",
    "entrecortaríamos": "entrecortar",
    "entrecortarían": "entrecortar",
    "entrecortarías": "entrecortar",
    "entrecortas": "entrecortar",
    "entrecortase": "entrecortar",
    "entrecortaseis": "entrecortar",
    "entrecortasen": "entrecortar",
    "entrecortases": "entrecortar",
    "entrecortaste": "entrecortar",
    "entrecortasteis": "entrecortar",
    "entrecorte": "entrecortar",
    "entrecortemos": "entrecortar",
    "entrecorten": "entrecortar",
    "entrecortes": "entrecortar",
    "entrecortezas": "entrecorteza",
    "entrecorto": "entrecortar",
    "entrecortábamos": "entrecortar",
    "entrecortáis": "entrecortar",
    "entrecortáramos": "entrecortar",
    "entrecortáremos": "entrecortar",
    "entrecortásemos": "entrecortar",
    "entrecorté": "entrecortar",
    "entrecortéis": "entrecortar",
    "entrecortó": "entrecortar",
    "entrecotes": "entrecot",
    "entrecots": "entrecot",
    "entrecruce": "entrecruzar",
    "entrecrucemos": "entrecruzar",
    "entrecrucen": "entrecruzar",
    "entrecruces": "entrecruzar",
    "entrecrucé": "entrecruzar",
    "entrecrucéis": "entrecruzar",
    "entrecruza": "entrecruzar",
    "entrecruzaba": "entrecruzar",
    "entrecruzabais": "entrecruzar",
    "entrecruzaban": "entrecruzar",
    "entrecruzabas": "entrecruzar",
    "entrecruzad": "entrecruzar",
    "entrecruzada": "entrecruzar",
    "entrecruzadas": "entrecruzar",
    "entrecruzado": "entrecruzar",
    "entrecruzados": "entrecruzar",
    "entrecruzamientos": "entrecruzamiento",
    "entrecruzamos": "entrecruzar",
    "entrecruzan": "entrecruzar",
    "entrecruzando": "entrecruzar",
    "entrecruzara": "entrecruzar",
    "entrecruzarais": "entrecruzar",
    "entrecruzaran": "entrecruzar",
    "entrecruzaras": "entrecruzar",
    "entrecruzare": "entrecruzar",
    "entrecruzareis": "entrecruzar",
    "entrecruzaremos": "entrecruzar",
    "entrecruzaren": "entrecruzar",
    "entrecruzares": "entrecruzar",
    "entrecruzaron": "entrecruzar",
    "entrecruzará": "entrecruzar",
    "entrecruzarán": "entrecruzar",
    "entrecruzarás": "entrecruzar",
    "entrecruzaré": "entrecruzar",
    "entrecruzaréis": "entrecruzar",
    "entrecruzaría": "entrecruzar",
    "entrecruzaríais": "entrecruzar",
    "entrecruzaríamos": "entrecruzar",
    "entrecruzarían": "entrecruzar",
    "entrecruzarías": "entrecruzar",
    "entrecruzas": "entrecruzar",
    "entrecruzase": "entrecruzar",
    "entrecruzaseis": "entrecruzar",
    "entrecruzasen": "entrecruzar",
    "entrecruzases": "entrecruzar",
    "entrecruzaste": "entrecruzar",
    "entrecruzasteis": "entrecruzar",
    "entrecruzo": "entrecruzar",
    "entrecruzábamos": "entrecruzar",
    "entrecruzáis": "entrecruzar",
    "entrecruzáramos": "entrecruzar",
    "entrecruzáremos": "entrecruzar",
    "entrecruzásemos": "entrecruzar",
    "entrecruzó": "entrecruzar",
    "entrecubiertas": "entrecubierta",
    "entrecuestos": "entrecuesto",
    "entredecid": "entredecir",
    "entredecimos": "entredecir",
    "entredeciremos": "entredecir",
    "entredecirá": "entredecir",
    "entredecirán": "entredecir",
    "entredecirás": "entredecir",
    "entredeciré": "entredecir",
    "entredeciréis": "entredecir",
    "entredeciría": "entredecir",
    "entredeciríais": "entredecir",
    "entredeciríamos": "entredecir",
    "entredecirían": "entredecir",
    "entredecirías": "entredecir",
    "entredecía": "entredecir",
    "entredecíais": "entredecir",
    "entredecíamos": "entredecir",
    "entredecían": "entredecir",
    "entredecías": "entredecir",
    "entredecís": "entredecir",
    "entredice": "entredecir",
    "entredicen": "entredecir",
    "entredices": "entredecir",
    "entredicha": "entredecir",
    "entredichas": "entredecir",
    "entredicho": "entredecir",
    "entredichos": "entredicho",
    "entrediciendo": "entredecir",
    "entrediga": "entredecir",
    "entredigamos": "entredecir",
    "entredigan": "entredecir",
    "entredigas": "entredecir",
    "entredigo": "entredecir",
    "entredigáis": "entredecir",
    "entredije": "entredecir",
    "entredijera": "entredecir",
    "entredijerais": "entredecir",
    "entredijeran": "entredecir",
    "entredijeras": "entredecir",
    "entredijere": "entredecir",
    "entredijereis": "entredecir",
    "entredijeren": "entredecir",
    "entredijeres": "entredecir",
    "entredijeron": "entredecir",
    "entredijese": "entredecir",
    "entredijeseis": "entredecir",
    "entredijesen": "entredecir",
    "entredijeses": "entredecir",
    "entredijimos": "entredecir",
    "entredijiste": "entredecir",
    "entredijisteis": "entredecir",
    "entredijo": "entredecir",
    "entredijéramos": "entredecir",
    "entredijéremos": "entredecir",
    "entredijésemos": "entredecir",
    "entredobles": "entredoble",
    "entredoses": "entredós",
    "entreejes": "entreeje",
    "entrefiletes": "entrefilete",
    "entrefina": "entrefino",
    "entrefinas": "entrefino",
    "entrefinos": "entrefino",
    "entreforros": "entreforro",
    "entrega": "entregar",
    "entregaba": "entregar",
    "entregabais": "entregar",
    "entregaban": "entregar",
    "entregabas": "entregar",
    "entregables": "entregable",
    "entregad": "entregar",
    "entregada": "entregar",
    "entregadas": "entregar",
    "entregado": "entregar",
    "entregados": "entregar",
    "entregamos": "entregar",
    "entregan": "entregar",
    "entregando": "entregar",
    "entregara": "entregar",
    "entregarais": "entregar",
    "entregaran": "entregar",
    "entregaras": "entregar",
    "entregare": "entregar",
    "entregareis": "entregar",
    "entregaremos": "entregar",
    "entregaren": "entregar",
    "entregares": "entregar",
    "entregaron": "entregar",
    "entregará": "entregar",
    "entregarán": "entregar",
    "entregarás": "entregar",
    "entregaré": "entregar",
    "entregaréis": "entregar",
    "entregaría": "entregar",
    "entregaríais": "entregar",
    "entregaríamos": "entregar",
    "entregarían": "entregar",
    "entregarías": "entregar",
    "entregas": "entregar",
    "entregase": "entregar",
    "entregaseis": "entregar",
    "entregasen": "entregar",
    "entregases": "entregar",
    "entregaste": "entregar",
    "entregasteis": "entregar",
    "entrego": "entregar",
    "entregue": "entregar",
    "entreguemos": "entregar",
    "entreguen": "entregar",
    "entregues": "entregar",
    "entregué": "entregar",
    "entreguéis": "entregar",
    "entregábamos": "entregar",
    "entregáis": "entregar",
    "entregáramos": "entregar",
    "entregáremos": "entregar",
    "entregásemos": "entregar",
    "entregó": "entregar",
    "entrehierros": "entrehierro",
    "entrejunta": "entrejuntar",
    "entrejuntaba": "entrejuntar",
    "entrejuntabais": "entrejuntar",
    "entrejuntaban": "entrejuntar",
    "entrejuntabas": "entrejuntar",
    "entrejuntad": "entrejuntar",
    "entrejuntada": "entrejuntar",
    "entrejuntadas": "entrejuntar",
    "entrejuntado": "entrejuntar",
    "entrejuntados": "entrejuntar",
    "entrejuntamos": "entrejuntar",
    "entrejuntan": "entrejuntar",
    "entrejuntando": "entrejuntar",
    "entrejuntara": "entrejuntar",
    "entrejuntarais": "entrejuntar",
    "entrejuntaran": "entrejuntar",
    "entrejuntaras": "entrejuntar",
    "entrejuntare": "entrejuntar",
    "entrejuntareis": "entrejuntar",
    "entrejuntaremos": "entrejuntar",
    "entrejuntaren": "entrejuntar",
    "entrejuntares": "entrejuntar",
    "entrejuntaron": "entrejuntar",
    "entrejuntará": "entrejuntar",
    "entrejuntarán": "entrejuntar",
    "entrejuntarás": "entrejuntar",
    "entrejuntaré": "entrejuntar",
    "entrejuntaréis": "entrejuntar",
    "entrejuntaría": "entrejuntar",
    "entrejuntaríais": "entrejuntar",
    "entrejuntaríamos": "entrejuntar",
    "entrejuntarían": "entrejuntar",
    "entrejuntarías": "entrejuntar",
    "entrejuntas": "entrejuntar",
    "entrejuntase": "entrejuntar",
    "entrejuntaseis": "entrejuntar",
    "entrejuntasen": "entrejuntar",
    "entrejuntases": "entrejuntar",
    "entrejuntaste": "entrejuntar",
    "entrejuntasteis": "entrejuntar",
    "entrejunte": "entrejuntar",
    "entrejuntemos": "entrejuntar",
    "entrejunten": "entrejuntar",
    "entrejuntes": "entrejuntar",
    "entrejunto": "entrejuntar",
    "entrejuntábamos": "entrejuntar",
    "entrejuntáis": "entrejuntar",
    "entrejuntáramos": "entrejuntar",
    "entrejuntáremos": "entrejuntar",
    "entrejuntásemos": "entrejuntar",
    "entrejunté": "entrejuntar",
    "entrejuntéis": "entrejuntar",
    "entrejuntó": "entrejuntar",
    "entrelace": "entrelazar",
    "entrelacemos": "entrelazar",
    "entrelacen": "entrelazar",
    "entrelaces": "entrelazar",
    "entrelacé": "entrelazar",
    "entrelacéis": "entrelazar",
    "entrelarga": "entrelargo",
    "entrelargas": "entrelargo",
    "entrelargos": "entrelargo",
    "entrelaza": "entrelazar",
    "entrelazaba": "entrelazar",
    "entrelazabais": "entrelazar",
    "entrelazaban": "entrelazar",
    "entrelazabas": "entrelazar",
    "entrelazad": "entrelazar",
    "entrelazada": "entrelazar",
    "entrelazadas": "entrelazar",
    "entrelazado": "entrelazar",
    "entrelazados": "entrelazar",
    "entrelazamientos": "entrelazamiento",
    "entrelazamos": "entrelazar",
    "entrelazan": "entrelazar",
    "entrelazando": "entrelazar",
    "entrelazara": "entrelazar",
    "entrelazarais": "entrelazar",
    "entrelazaran": "entrelazar",
    "entrelazaras": "entrelazar",
    "entrelazare": "entrelazar",
    "entrelazareis": "entrelazar",
    "entrelazaremos": "entrelazar",
    "entrelazaren": "entrelazar",
    "entrelazares": "entrelazar",
    "entrelazaron": "entrelazar",
    "entrelazará": "entrelazar",
    "entrelazarán": "entrelazar",
    "entrelazarás": "entrelazar",
    "entrelazaré": "entrelazar",
    "entrelazaréis": "entrelazar",
    "entrelazaría": "entrelazar",
    "entrelazaríais": "entrelazar",
    "entrelazaríamos": "entrelazar",
    "entrelazarían": "entrelazar",
    "entrelazarías": "entrelazar",
    "entrelazas": "entrelazar",
    "entrelazase": "entrelazar",
    "entrelazaseis": "entrelazar",
    "entrelazasen": "entrelazar",
    "entrelazases": "entrelazar",
    "entrelazaste": "entrelazar",
    "entrelazasteis": "entrelazar",
    "entrelazo": "entrelazar",
    "entrelazos": "entrelazo",
    "entrelazábamos": "entrelazar",
    "entrelazáis": "entrelazar",
    "entrelazáramos": "entrelazar",
    "entrelazáremos": "entrelazar",
    "entrelazásemos": "entrelazar",
    "entrelazó": "entrelazar",
    "entrelinea": "entrelinear",
    "entrelineaba": "entrelinear",
    "entrelineabais": "entrelinear",
    "entrelineaban": "entrelinear",
    "entrelineabas": "entrelinear",
    "entrelinead": "entrelinear",
    "entrelineada": "entrelinear",
    "entrelineadas": "entrelinear",
    "entrelineado": "entrelinear",
    "entrelineados": "entrelinear",
    "entrelineamos": "entrelinear",
    "entrelinean": "entrelinear",
    "entrelineando": "entrelinear",
    "entrelineara": "entrelinear",
    "entrelinearais": "entrelinear",
    "entrelinearan": "entrelinear",
    "entrelinearas": "entrelinear",
    "entrelineare": "entrelinear",
    "entrelineareis": "entrelinear",
    "entrelinearemos": "entrelinear",
    "entrelinearen": "entrelinear",
    "entrelineares": "entrelinear",
    "entrelinearon": "entrelinear",
    "entrelineará": "entrelinear",
    "entrelinearán": "entrelinear",
    "entrelinearás": "entrelinear",
    "entrelinearé": "entrelinear",
    "entrelinearéis": "entrelinear",
    "entrelinearía": "entrelinear",
    "entrelinearíais": "entrelinear",
    "entrelinearíamos": "entrelinear",
    "entrelinearían": "entrelinear",
    "entrelinearías": "entrelinear",
    "entrelineas": "entrelinear",
    "entrelinease": "entrelinear",
    "entrelineaseis": "entrelinear",
    "entrelineasen": "entrelinear",
    "entrelineases": "entrelinear",
    "entrelineaste": "entrelinear",
    "entrelineasteis": "entrelinear",
    "entrelinee": "entrelinear",
    "entrelineemos": "entrelinear",
    "entrelineen": "entrelinear",
    "entrelinees": "entrelinear",
    "entrelineo": "entrelinear",
    "entrelineábamos": "entrelinear",
    "entrelineáis": "entrelinear",
    "entrelineáramos": "entrelinear",
    "entrelineáremos": "entrelinear",
    "entrelineásemos": "entrelinear",
    "entrelineé": "entrelinear",
    "entrelineéis": "entrelinear",
    "entrelineó": "entrelinear",
    "entrelistada": "entrelistado",
    "entrelistadas": "entrelistado",
    "entrelistados": "entrelistado",
    "entreliños": "entreliño",
    "entreluce": "entrelucir",
    "entrelucen": "entrelucir",
    "entreluces": "entrelucir",
    "entrelucid": "entrelucir",
    "entrelucida": "entrelucir",
    "entrelucidas": "entrelucir",
    "entrelucido": "entrelucir",
    "entrelucidos": "entrelucir",
    "entreluciendo": "entrelucir",
    "entreluciera": "entrelucir",
    "entrelucierais": "entrelucir",
    "entrelucieran": "entrelucir",
    "entrelucieras": "entrelucir",
    "entreluciere": "entrelucir",
    "entreluciereis": "entrelucir",
    "entrelucieren": "entrelucir",
    "entrelucieres": "entrelucir",
    "entrelucieron": "entrelucir",
    "entreluciese": "entrelucir",
    "entrelucieseis": "entrelucir",
    "entreluciesen": "entrelucir",
    "entrelucieses": "entrelucir",
    "entrelucimos": "entrelucir",
    "entreluciremos": "entrelucir",
    "entrelucirá": "entrelucir",
    "entrelucirán": "entrelucir",
    "entrelucirás": "entrelucir",
    "entreluciré": "entrelucir",
    "entreluciréis": "entrelucir",
    "entreluciría": "entrelucir",
    "entreluciríais": "entrelucir",
    "entreluciríamos": "entrelucir",
    "entrelucirían": "entrelucir",
    "entrelucirías": "entrelucir",
    "entreluciste": "entrelucir",
    "entrelucisteis": "entrelucir",
    "entreluciéramos": "entrelucir",
    "entreluciéremos": "entrelucir",
    "entreluciésemos": "entrelucir",
    "entrelució": "entrelucir",
    "entrelucí": "entrelucir",
    "entrelucía": "entrelucir",
    "entrelucíais": "entrelucir",
    "entrelucíamos": "entrelucir",
    "entrelucían": "entrelucir",
    "entrelucías": "entrelucir",
    "entrelucís": "entrelucir",
    "entreluzca": "entrelucir",
    "entreluzcamos": "entrelucir",
    "entreluzcan": "entrelucir",
    "entreluzcas": "entrelucir",
    "entreluzco": "entrelucir",
    "entreluzcáis": "entrelucir",
    "entrelíneas": "entrelínea",
    "entremedia": "entremedio",
    "entremedias": "entremedio",
    "entremedios": "entremedio",
    "entremeses": "entremés",
    "entremesiles": "entremesil",
    "entremesistas": "entremesista",
    "entremeta": "entremeter",
    "entremetamos": "entremeter",
    "entremetan": "entremeter",
    "entremetas": "entremeter",
    "entremete": "entremeter",
    "entremeted": "entremeter",
    "entremetemos": "entremeter",
    "entremeten": "entremeter",
    "entremeteremos": "entremeter",
    "entremeterá": "entremeter",
    "entremeterán": "entremeter",
    "entremeterás": "entremeter",
    "entremeteré": "entremeter",
    "entremeteréis": "entremeter",
    "entremetería": "entremeter",
    "entremeteríais": "entremeter",
    "entremeteríamos": "entremeter",
    "entremeterían": "entremeter",
    "entremeterías": "entremeter",
    "entremetes": "entremeter",
    "entremetida": "entremetido",
    "entremetidas": "entremetido",
    "entremetido": "entremeter",
    "entremetidos": "entremetido",
    "entremetiendo": "entremeter",
    "entremetiera": "entremeter",
    "entremetierais": "entremeter",
    "entremetieran": "entremeter",
    "entremetieras": "entremeter",
    "entremetiere": "entremeter",
    "entremetiereis": "entremeter",
    "entremetieren": "entremeter",
    "entremetieres": "entremeter",
    "entremetieron": "entremeter",
    "entremetiese": "entremeter",
    "entremetieseis": "entremeter",
    "entremetiesen": "entremeter",
    "entremetieses": "entremeter",
    "entremetimientos": "entremetimiento",
    "entremetimos": "entremeter",
    "entremetiste": "entremeter",
    "entremetisteis": "entremeter",
    "entremetiéramos": "entremeter",
    "entremetiéremos": "entremeter",
    "entremetiésemos": "entremeter",
    "entremetió": "entremeter",
    "entremeto": "entremeter",
    "entremetáis": "entremeter",
    "entremetéis": "entremeter",
    "entremetí": "entremeter",
    "entremetía": "entremeter",
    "entremetíais": "entremeter",
    "entremetíamos": "entremeter",
    "entremetían": "entremeter",
    "entremetías": "entremeter",
    "entremezcla": "entremezclar",
    "entremezclaba": "entremezclar",
    "entremezclabais": "entremezclar",
    "entremezclaban": "entremezclar",
    "entremezclabas": "entremezclar",
    "entremezclad": "entremezclar",
    "entremezclada": "entremezclar",
    "entremezcladas": "entremezclar",
    "entremezclado": "entremezclar",
    "entremezclados": "entremezclar",
    "entremezcladuras": "entremezcladura",
    "entremezclamos": "entremezclar",
    "entremezclan": "entremezclar",
    "entremezclando": "entremezclar",
    "entremezclara": "entremezclar",
    "entremezclarais": "entremezclar",
    "entremezclaran": "entremezclar",
    "entremezclaras": "entremezclar",
    "entremezclare": "entremezclar",
    "entremezclareis": "entremezclar",
    "entremezclaremos": "entremezclar",
    "entremezclaren": "entremezclar",
    "entremezclares": "entremezclar",
    "entremezclaron": "entremezclar",
    "entremezclará": "entremezclar",
    "entremezclarán": "entremezclar",
    "entremezclarás": "entremezclar",
    "entremezclaré": "entremezclar",
    "entremezclaréis": "entremezclar",
    "entremezclaría": "entremezclar",
    "entremezclaríais": "entremezclar",
    "entremezclaríamos": "entremezclar",
    "entremezclarían": "entremezclar",
    "entremezclarías": "entremezclar",
    "entremezclas": "entremezclar",
    "entremezclase": "entremezclar",
    "entremezclaseis": "entremezclar",
    "entremezclasen": "entremezclar",
    "entremezclases": "entremezclar",
    "entremezclaste": "entremezclar",
    "entremezclasteis": "entremezclar",
    "entremezcle": "entremezclar",
    "entremezclemos": "entremezclar",
    "entremezclen": "entremezclar",
    "entremezcles": "entremezclar",
    "entremezclo": "entremezclar",
    "entremezclábamos": "entremezclar",
    "entremezcláis": "entremezclar",
    "entremezcláramos": "entremezclar",
    "entremezcláremos": "entremezclar",
    "entremezclásemos": "entremezclar",
    "entremezclé": "entremezclar",
    "entremezcléis": "entremezclar",
    "entremezcló": "entremezclar",
    "entremiches": "entremiche",
    "entremos": "entrar",
    "entren": "entrar",
    "entrena": "entrenar",
    "entrenaba": "entrenar",
    "entrenabais": "entrenar",
    "entrenaban": "entrenar",
    "entrenabas": "entrenar",
    "entrenad": "entrenar",
    "entrenada": "entrenar",
    "entrenadas": "entrenar",
    "entrenado": "entrenar",
    "entrenadora": "entrenador",
    "entrenadoras": "entrenador",
    "entrenadores": "entrenador",
    "entrenados": "entrenar",
    "entrenamientos": "entrenamiento",
    "entrenamos": "entrenar",
    "entrenan": "entrenar",
    "entrenando": "entrenar",
    "entrenara": "entrenar",
    "entrenarais": "entrenar",
    "entrenaran": "entrenar",
    "entrenaras": "entrenar",
    "entrenare": "entrenar",
    "entrenareis": "entrenar",
    "entrenaremos": "entrenar",
    "entrenaren": "entrenar",
    "entrenares": "entrenar",
    "entrenaron": "entrenar",
    "entrenará": "entrenar",
    "entrenarán": "entrenar",
    "entrenarás": "entrenar",
    "entrenaré": "entrenar",
    "entrenaréis": "entrenar",
    "entrenaría": "entrenar",
    "entrenaríais": "entrenar",
    "entrenaríamos": "entrenar",
    "entrenarían": "entrenar",
    "entrenarías": "entrenar",
    "entrenas": "entrenar",
    "entrenase": "entrenar",
    "entrenaseis": "entrenar",
    "entrenasen": "entrenar",
    "entrenases": "entrenar",
    "entrenaste": "entrenar",
    "entrenasteis": "entrenar",
    "entrene": "entrenar",
    "entrenemos": "entrenar",
    "entrenen": "entrenar",
    "entrenes": "entrenar",
    "entreno": "entrenar",
    "entrenudos": "entrenudo",
    "entrenábamos": "entrenar",
    "entrenáis": "entrenar",
    "entrenáramos": "entrenar",
    "entrenáremos": "entrenar",
    "entrenásemos": "entrenar",
    "entrené": "entrenar",
    "entrenéis": "entrenar",
    "entrenó": "entrenar",
    "entreoiga": "entreoír",
    "entreoigamos": "entreoír",
    "entreoigan": "entreoír",
    "entreoigas": "entreoír",
    "entreoigo": "entreoír",
    "entreoigáis": "entreoír",
    "entreoiremos": "entreoír",
    "entreoirá": "entreoír",
    "entreoirán": "entreoír",
    "entreoirás": "entreoír",
    "entreoiré": "entreoír",
    "entreoiréis": "entreoír",
    "entreoiría": "entreoír",
    "entreoiríais": "entreoír",
    "entreoiríamos": "entreoír",
    "entreoirían": "entreoír",
    "entreoirías": "entreoír",
    "entreoscura": "entreoscuro",
    "entreoscuras": "entreoscuro",
    "entreoscuros": "entreoscuro",
    "entreoye": "entreoír",
    "entreoyen": "entreoír",
    "entreoyendo": "entreoír",
    "entreoyera": "entreoír",
    "entreoyerais": "entreoír",
    "entreoyeran": "entreoír",
    "entreoyeras": "entreoír",
    "entreoyere": "entreoír",
    "entreoyereis": "entreoír",
    "entreoyeren": "entreoír",
    "entreoyeres": "entreoír",
    "entreoyeron": "entreoír",
    "entreoyes": "entreoír",
    "entreoyese": "entreoír",
    "entreoyeseis": "entreoír",
    "entreoyesen": "entreoír",
    "entreoyeses": "entreoír",
    "entreoyéramos": "entreoír",
    "entreoyéremos": "entreoír",
    "entreoyésemos": "entreoír",
    "entreoyó": "entreoír",
    "entreoí": "entreoír",
    "entreoía": "entreoír",
    "entreoíais": "entreoír",
    "entreoíamos": "entreoír",
    "entreoían": "entreoír",
    "entreoías": "entreoír",
    "entreoíd": "entreoír",
    "entreoída": "entreoír",
    "entreoídas": "entreoír",
    "entreoído": "entreoír",
    "entreoídos": "entreoír",
    "entreoímos": "entreoír",
    "entreoís": "entreoír",
    "entreoíste": "entreoír",
    "entreoísteis": "entreoír",
    "entrepalmaduras": "entrepalmadura",
    "entrepasos": "entrepaso",
    "entrepaños": "entrepaño",
    "entrepechugas": "entrepechuga",
    "entrepela": "entrepelar",
    "entrepelaba": "entrepelar",
    "entrepelabais": "entrepelar",
    "entrepelaban": "entrepelar",
    "entrepelabas": "entrepelar",
    "entrepelad": "entrepelar",
    "entrepelada": "entrepelar",
    "entrepeladas": "entrepelar",
    "entrepelado": "entrepelar",
    "entrepelados": "entrepelar",
    "entrepelamos": "entrepelar",
    "entrepelan": "entrepelar",
    "entrepelando": "entrepelar",
    "entrepelara": "entrepelar",
    "entrepelarais": "entrepelar",
    "entrepelaran": "entrepelar",
    "entrepelaras": "entrepelar",
    "entrepelare": "entrepelar",
    "entrepelareis": "entrepelar",
    "entrepelaremos": "entrepelar",
    "entrepelaren": "entrepelar",
    "entrepelares": "entrepelar",
    "entrepelaron": "entrepelar",
    "entrepelará": "entrepelar",
    "entrepelarán": "entrepelar",
    "entrepelarás": "entrepelar",
    "entrepelaré": "entrepelar",
    "entrepelaréis": "entrepelar",
    "entrepelaría": "entrepelar",
    "entrepelaríais": "entrepelar",
    "entrepelaríamos": "entrepelar",
    "entrepelarían": "entrepelar",
    "entrepelarías": "entrepelar",
    "entrepelas": "entrepelar",
    "entrepelase": "entrepelar",
    "entrepelaseis": "entrepelar",
    "entrepelasen": "entrepelar",
    "entrepelases": "entrepelar",
    "entrepelaste": "entrepelar",
    "entrepelasteis": "entrepelar",
    "entrepele": "entrepelar",
    "entrepelemos": "entrepelar",
    "entrepelen": "entrepelar",
    "entrepeles": "entrepelar",
    "entrepelo": "entrepelar",
    "entrepelábamos": "entrepelar",
    "entrepeláis": "entrepelar",
    "entrepeláramos": "entrepelar",
    "entrepeláremos": "entrepelar",
    "entrepelásemos": "entrepelar",
    "entrepelé": "entrepelar",
    "entrepeléis": "entrepelar",
    "entrepeló": "entrepelar",
    "entrepernaba": "entrepernar",
    "entrepernabais": "entrepernar",
    "entrepernaban": "entrepernar",
    "entrepernabas": "entrepernar",
    "entrepernad": "entrepernar",
    "entrepernada": "entrepernar",
    "entrepernadas": "entrepernar",
    "entrepernado": "entrepernar",
    "entrepernados": "entrepernar",
    "entrepernamos": "entrepernar",
    "entrepernando": "entrepernar",
    "entrepernara": "entrepernar",
    "entrepernarais": "entrepernar",
    "entrepernaran": "entrepernar",
    "entrepernaras": "entrepernar",
    "entrepernare": "entrepernar",
    "entrepernareis": "entrepernar",
    "entrepernaremos": "entrepernar",
    "entrepernaren": "entrepernar",
    "entrepernares": "entrepernar",
    "entrepernaron": "entrepernar",
    "entrepernará": "entrepernar",
    "entrepernarán": "entrepernar",
    "entrepernarás": "entrepernar",
    "entrepernaré": "entrepernar",
    "entrepernaréis": "entrepernar",
    "entrepernaría": "entrepernar",
    "entrepernaríais": "entrepernar",
    "entrepernaríamos": "entrepernar",
    "entrepernarían": "entrepernar",
    "entrepernarías": "entrepernar",
    "entrepernase": "entrepernar",
    "entrepernaseis": "entrepernar",
    "entrepernasen": "entrepernar",
    "entrepernases": "entrepernar",
    "entrepernaste": "entrepernar",
    "entrepernasteis": "entrepernar",
    "entrepernemos": "entrepernar",
    "entrepernábamos": "entrepernar",
    "entrepernáis": "entrepernar",
    "entrepernáramos": "entrepernar",
    "entrepernáremos": "entrepernar",
    "entrepernásemos": "entrepernar",
    "entreperné": "entrepernar",
    "entrepernéis": "entrepernar",
    "entrepernó": "entrepernar",
    "entrepetada": "entrepetado",
    "entrepetadas": "entrepetado",
    "entrepetados": "entrepetado",
    "entrepierna": "entrepernar",
    "entrepiernan": "entrepernar",
    "entrepiernas": "entrepierna",
    "entrepierne": "entrepernar",
    "entrepiernen": "entrepernar",
    "entrepiernes": "entrepernar",
    "entrepierno": "entrepernar",
    "entrepisos": "entrepiso",
    "entreplantas": "entreplanta",
    "entrepuentes": "entrepuente",
    "entrepunzaduras": "entrepunzadura",
    "entrerraída": "entrerraído",
    "entrerraídas": "entrerraído",
    "entrerraídos": "entrerraído",
    "entrerrenglona": "entrerrenglonar",
    "entrerrenglonaba": "entrerrenglonar",
    "entrerrenglonabais": "entrerrenglonar",
    "entrerrenglonaban": "entrerrenglonar",
    "entrerrenglonabas": "entrerrenglonar",
    "entrerrenglonad": "entrerrenglonar",
    "entrerrenglonada": "entrerrenglonar",
    "entrerrenglonadas": "entrerrenglonar",
    "entrerrenglonado": "entrerrenglonar",
    "entrerrenglonados": "entrerrenglonar",
    "entrerrenglonaduras": "entrerrenglonadura",
    "entrerrenglonamos": "entrerrenglonar",
    "entrerrenglonan": "entrerrenglonar",
    "entrerrenglonando": "entrerrenglonar",
    "entrerrenglonara": "entrerrenglonar",
    "entrerrenglonarais": "entrerrenglonar",
    "entrerrenglonaran": "entrerrenglonar",
    "entrerrenglonaras": "entrerrenglonar",
    "entrerrenglonare": "entrerrenglonar",
    "entrerrenglonareis": "entrerrenglonar",
    "entrerrenglonaremos": "entrerrenglonar",
    "entrerrenglonaren": "entrerrenglonar",
    "entrerrenglonares": "entrerrenglonar",
    "entrerrenglonaron": "entrerrenglonar",
    "entrerrenglonará": "entrerrenglonar",
    "entrerrenglonarán": "entrerrenglonar",
    "entrerrenglonarás": "entrerrenglonar",
    "entrerrenglonaré": "entrerrenglonar",
    "entrerrenglonaréis": "entrerrenglonar",
    "entrerrenglonaría": "entrerrenglonar",
    "entrerrenglonaríais": "entrerrenglonar",
    "entrerrenglonaríamos": "entrerrenglonar",
    "entrerrenglonarían": "entrerrenglonar",
    "entrerrenglonarías": "entrerrenglonar",
    "entrerrenglonas": "entrerrenglonar",
    "entrerrenglonase": "entrerrenglonar",
    "entrerrenglonaseis": "entrerrenglonar",
    "entrerrenglonasen": "entrerrenglonar",
    "entrerrenglonases": "entrerrenglonar",
    "entrerrenglonaste": "entrerrenglonar",
    "entrerrenglonasteis": "entrerrenglonar",
    "entrerrenglone": "entrerrenglonar",
    "entrerrenglonemos": "entrerrenglonar",
    "entrerrenglonen": "entrerrenglonar",
    "entrerrenglones": "entrerrenglonar",
    "entrerrenglono": "entrerrenglonar",
    "entrerrenglonábamos": "entrerrenglonar",
    "entrerrenglonáis": "entrerrenglonar",
    "entrerrenglonáramos": "entrerrenglonar",
    "entrerrenglonáremos": "entrerrenglonar",
    "entrerrenglonásemos": "entrerrenglonar",
    "entrerrengloné": "entrerrenglonar",
    "entrerrenglonéis": "entrerrenglonar",
    "entrerrenglonó": "entrerrenglonar",
    "entres": "entrar",
    "entresaca": "entresacar",
    "entresacaba": "entresacar",
    "entresacabais": "entresacar",
    "entresacaban": "entresacar",
    "entresacabas": "entresacar",
    "entresacad": "entresacar",
    "entresacada": "entresacar",
    "entresacadas": "entresacar",
    "entresacado": "entresacar",
    "entresacados": "entresacar",
    "entresacaduras": "entresacadura",
    "entresacamos": "entresacar",
    "entresacan": "entresacar",
    "entresacando": "entresacar",
    "entresacara": "entresacar",
    "entresacarais": "entresacar",
    "entresacaran": "entresacar",
    "entresacaras": "entresacar",
    "entresacare": "entresacar",
    "entresacareis": "entresacar",
    "entresacaremos": "entresacar",
    "entresacaren": "entresacar",
    "entresacares": "entresacar",
    "entresacaron": "entresacar",
    "entresacará": "entresacar",
    "entresacarán": "entresacar",
    "entresacarás": "entresacar",
    "entresacaré": "entresacar",
    "entresacaréis": "entresacar",
    "entresacaría": "entresacar",
    "entresacaríais": "entresacar",
    "entresacaríamos": "entresacar",
    "entresacarían": "entresacar",
    "entresacarías": "entresacar",
    "entresacas": "entresacar",
    "entresacase": "entresacar",
    "entresacaseis": "entresacar",
    "entresacasen": "entresacar",
    "entresacases": "entresacar",
    "entresacaste": "entresacar",
    "entresacasteis": "entresacar",
    "entresaco": "entresacar",
    "entresacábamos": "entresacar",
    "entresacáis": "entresacar",
    "entresacáramos": "entresacar",
    "entresacáremos": "entresacar",
    "entresacásemos": "entresacar",
    "entresacó": "entresacar",
    "entresaque": "entresacar",
    "entresaquemos": "entresacar",
    "entresaquen": "entresacar",
    "entresaques": "entresacar",
    "entresaqué": "entresacar",
    "entresaquéis": "entresacar",
    "entresijos": "entresijo",
    "entresuelos": "entresuelo",
    "entretalla": "entretallar",
    "entretallaba": "entretallar",
    "entretallabais": "entretallar",
    "entretallaban": "entretallar",
    "entretallabas": "entretallar",
    "entretallad": "entretallar",
    "entretallada": "entretallar",
    "entretalladas": "entretallar",
    "entretallado": "entretallar",
    "entretallados": "entretallar",
    "entretalladuras": "entretalladura",
    "entretallamos": "entretallar",
    "entretallan": "entretallar",
    "entretallando": "entretallar",
    "entretallara": "entretallar",
    "entretallarais": "entretallar",
    "entretallaran": "entretallar",
    "entretallaras": "entretallar",
    "entretallare": "entretallar",
    "entretallareis": "entretallar",
    "entretallaremos": "entretallar",
    "entretallaren": "entretallar",
    "entretallares": "entretallar",
    "entretallaron": "entretallar",
    "entretallará": "entretallar",
    "entretallarán": "entretallar",
    "entretallarás": "entretallar",
    "entretallaré": "entretallar",
    "entretallaréis": "entretallar",
    "entretallaría": "entretallar",
    "entretallaríais": "entretallar",
    "entretallaríamos": "entretallar",
    "entretallarían": "entretallar",
    "entretallarías": "entretallar",
    "entretallas": "entretallar",
    "entretallase": "entretallar",
    "entretallaseis": "entretallar",
    "entretallasen": "entretallar",
    "entretallases": "entretallar",
    "entretallaste": "entretallar",
    "entretallasteis": "entretallar",
    "entretalle": "entretallar",
    "entretallemos": "entretallar",
    "entretallen": "entretallar",
    "entretalles": "entretallar",
    "entretallo": "entretallar",
    "entretallábamos": "entretallar",
    "entretalláis": "entretallar",
    "entretalláramos": "entretallar",
    "entretalláremos": "entretallar",
    "entretallásemos": "entretallar",
    "entretallé": "entretallar",
    "entretalléis": "entretallar",
    "entretalló": "entretallar",
    "entretantos": "entretanto",
    "entreteja": "entretejer",
    "entretejamos": "entretejer",
    "entretejan": "entretejer",
    "entretejas": "entretejer",
    "entreteje": "entretejer",
    "entretejed": "entretejer",
    "entretejedora": "entretejedor",
    "entretejedoras": "entretejedor",
    "entretejedores": "entretejedor",
    "entretejeduras": "entretejedura",
    "entretejemos": "entretejer",
    "entretejen": "entretejer",
    "entretejeremos": "entretejer",
    "entretejerá": "entretejer",
    "entretejerán": "entretejer",
    "entretejerás": "entretejer",
    "entretejeré": "entretejer",
    "entretejeréis": "entretejer",
    "entretejería": "entretejer",
    "entretejeríais": "entretejer",
    "entretejeríamos": "entretejer",
    "entretejerían": "entretejer",
    "entretejerías": "entretejer",
    "entretejes": "entretejer",
    "entretejida": "entretejer",
    "entretejidas": "entretejer",
    "entretejido": "entretejer",
    "entretejidos": "entretejer",
    "entretejiendo": "entretejer",
    "entretejiera": "entretejer",
    "entretejierais": "entretejer",
    "entretejieran": "entretejer",
    "entretejieras": "entretejer",
    "entretejiere": "entretejer",
    "entretejiereis": "entretejer",
    "entretejieren": "entretejer",
    "entretejieres": "entretejer",
    "entretejieron": "entretejer",
    "entretejiese": "entretejer",
    "entretejieseis": "entretejer",
    "entretejiesen": "entretejer",
    "entretejieses": "entretejer",
    "entretejimientos": "entretejimiento",
    "entretejimos": "entretejer",
    "entretejiste": "entretejer",
    "entretejisteis": "entretejer",
    "entretejiéramos": "entretejer",
    "entretejiéremos": "entretejer",
    "entretejiésemos": "entretejer",
    "entretejió": "entretejer",
    "entretejo": "entretejer",
    "entretejáis": "entretejer",
    "entretejéis": "entretejer",
    "entretejí": "entretejer",
    "entretejía": "entretejer",
    "entretejíais": "entretejer",
    "entretejíamos": "entretejer",
    "entretejían": "entretejer",
    "entretejías": "entretejer",
    "entretela": "entretelar",
    "entretelaba": "entretelar",
    "entretelabais": "entretelar",
    "entretelaban": "entretelar",
    "entretelabas": "entretelar",
    "entretelad": "entretelar",
    "entretelada": "entretelar",
    "entreteladas": "entretelar",
    "entretelado": "entretelar",
    "entretelados": "entretelar",
    "entretelamos": "entretelar",
    "entretelan": "entretelar",
    "entretelando": "entretelar",
    "entretelara": "entretelar",
    "entretelarais": "entretelar",
    "entretelaran": "entretelar",
    "entretelaras": "entretelar",
    "entretelare": "entretelar",
    "entretelareis": "entretelar",
    "entretelaremos": "entretelar",
    "entretelaren": "entretelar",
    "entretelares": "entretelar",
    "entretelaron": "entretelar",
    "entretelará": "entretelar",
    "entretelarán": "entretelar",
    "entretelarás": "entretelar",
    "entretelaré": "entretelar",
    "entretelaréis": "entretelar",
    "entretelaría": "entretelar",
    "entretelaríais": "entretelar",
    "entretelaríamos": "entretelar",
    "entretelarían": "entretelar",
    "entretelarías": "entretelar",
    "entretelas": "entretelar",
    "entretelase": "entretelar",
    "entretelaseis": "entretelar",
    "entretelasen": "entretelar",
    "entretelases": "entretelar",
    "entretelaste": "entretelar",
    "entretelasteis": "entretelar",
    "entretele": "entretelar",
    "entretelemos": "entretelar",
    "entretelen": "entretelar",
    "entreteles": "entretelar",
    "entretelo": "entretelar",
    "entretelábamos": "entretelar",
    "entreteláis": "entretelar",
    "entreteláramos": "entretelar",
    "entreteláremos": "entretelar",
    "entretelásemos": "entretelar",
    "entretelé": "entretelar",
    "entreteléis": "entretelar",
    "entreteló": "entretelar",
    "entretenciones": "entretención",
    "entretendremos": "entretener",
    "entretendrá": "entretener",
    "entretendrán": "entretener",
    "entretendrás": "entretener",
    "entretendré": "entretener",
    "entretendréis": "entretener",
    "entretendría": "entretener",
    "entretendríais": "entretener",
    "entretendríamos": "entretener",
    "entretendrían": "entretener",
    "entretendrías": "entretener",
    "entretened": "entretener",
    "entretenedora": "entretenedor",
    "entretenedoras": "entretenedor",
    "entretenedores": "entretenedor",
    "entretenemos": "entretener",
    "entretenga": "entretener",
    "entretengamos": "entretener",
    "entretengan": "entretener",
    "entretengas": "entretener",
    "entretengo": "entretener",
    "entretengáis": "entretener",
    "entretenibilidades": "entretenibilidad",
    "entretenida": "entretenido",
    "entretenidas": "entretenido",
    "entretenido": "entretener",
    "entretenidos": "entretenido",
    "entreteniendo": "entretener",
    "entretenimientos": "entretenimiento",
    "entretenéis": "entretener",
    "entretenía": "entretener",
    "entreteníais": "entretener",
    "entreteníamos": "entretener",
    "entretenían": "entretener",
    "entretenías": "entretener",
    "entretiempos": "entretiempo",
    "entretiene": "entretener",
    "entretienen": "entretener",
    "entretienes": "entretener",
    "entretuve": "entretener",
    "entretuviera": "entretener",
    "entretuvierais": "entretener",
    "entretuvieran": "entretener",
    "entretuvieras": "entretener",
    "entretuviere": "entretener",
    "entretuviereis": "entretener",
    "entretuvieren": "entretener",
    "entretuvieres": "entretener",
    "entretuvieron": "entretener",
    "entretuviese": "entretener",
    "entretuvieseis": "entretener",
    "entretuviesen": "entretener",
    "entretuvieses": "entretener",
    "entretuvimos": "entretener",
    "entretuviste": "entretener",
    "entretuvisteis": "entretener",
    "entretuviéramos": "entretener",
    "entretuviéremos": "entretener",
    "entretuviésemos": "entretener",
    "entretuvo": "entretener",
    "entretén": "entretener",
    "entrevea": "entrever",
    "entreveamos": "entrever",
    "entrevean": "entrever",
    "entreveas": "entrever",
    "entreved": "entrever",
    "entrevemos": "entrever",
    "entreventanas": "entreventana",
    "entreveo": "entrever",
    "entrevera": "entreverar",
    "entreveraba": "entreverar",
    "entreverabais": "entreverar",
    "entreveraban": "entreverar",
    "entreverabas": "entreverar",
    "entreverad": "entreverar",
    "entreverada": "entreverar",
    "entreveradas": "entreverar",
    "entreverado": "entreverar",
    "entreverados": "entreverar",
    "entreveramos": "entreverar",
    "entreveran": "entreverar",
    "entreverando": "entreverar",
    "entreverara": "entreverar",
    "entreverarais": "entreverar",
    "entreveraran": "entreverar",
    "entreveraras": "entreverar",
    "entreverare": "entreverar",
    "entreverareis": "entreverar",
    "entreveraremos": "entreverar",
    "entreveraren": "entreverar",
    "entreverares": "entreverar",
    "entreveraron": "entreverar",
    "entreverará": "entreverar",
    "entreverarán": "entreverar",
    "entreverarás": "entreverar",
    "entreveraré": "entreverar",
    "entreveraréis": "entreverar",
    "entreveraría": "entreverar",
    "entreveraríais": "entreverar",
    "entreveraríamos": "entreverar",
    "entreverarían": "entreverar",
    "entreverarías": "entreverar",
    "entreveras": "entreverar",
    "entreverase": "entreverar",
    "entreveraseis": "entreverar",
    "entreverasen": "entreverar",
    "entreverases": "entreverar",
    "entreveraste": "entreverar",
    "entreverasteis": "entreverar",
    "entrevere": "entreverar",
    "entreveremos": "entreverar",
    "entreveren": "entreverar",
    "entreveres": "entreverar",
    "entrevero": "entreverar",
    "entreverá": "entrever",
    "entreverábamos": "entreverar",
    "entreveráis": "entreverar",
    "entreverán": "entrever",
    "entreveráramos": "entreverar",
    "entreveráremos": "entreverar",
    "entreverás": "entrever",
    "entreverásemos": "entreverar",
    "entreveré": "entreverar",
    "entreveréis": "entreverar",
    "entrevería": "entrever",
    "entreveríais": "entrever",
    "entreveríamos": "entrever",
    "entreverían": "entrever",
    "entreverías": "entrever",
    "entreveró": "entreverar",
    "entreveáis": "entrever",
    "entreveía": "entrever",
    "entreveíais": "entrever",
    "entreveíamos": "entrever",
    "entreveían": "entrever",
    "entreveías": "entrever",
    "entreviendo": "entrever",
    "entreviera": "entrever",
    "entrevierais": "entrever",
    "entrevieran": "entrever",
    "entrevieras": "entrever",
    "entreviere": "entrever",
    "entreviereis": "entrever",
    "entrevieren": "entrever",
    "entrevieres": "entrever",
    "entrevieron": "entrever",
    "entreviese": "entrever",
    "entrevieseis": "entrever",
    "entreviesen": "entrever",
    "entrevieses": "entrever",
    "entreviga": "entrevigar",
    "entrevigaba": "entrevigar",
    "entrevigabais": "entrevigar",
    "entrevigaban": "entrevigar",
    "entrevigabas": "entrevigar",
    "entrevigad": "entrevigar",
    "entrevigada": "entrevigar",
    "entrevigadas": "entrevigar",
    "entrevigado": "entrevigar",
    "entrevigados": "entrevigar",
    "entrevigamos": "entrevigar",
    "entrevigan": "entrevigar",
    "entrevigando": "entrevigar",
    "entrevigara": "entrevigar",
    "entrevigarais": "entrevigar",
    "entrevigaran": "entrevigar",
    "entrevigaras": "entrevigar",
    "entrevigare": "entrevigar",
    "entrevigareis": "entrevigar",
    "entrevigaremos": "entrevigar",
    "entrevigaren": "entrevigar",
    "entrevigares": "entrevigar",
    "entrevigaron": "entrevigar",
    "entrevigará": "entrevigar",
    "entrevigarán": "entrevigar",
    "entrevigarás": "entrevigar",
    "entrevigaré": "entrevigar",
    "entrevigaréis": "entrevigar",
    "entrevigaría": "entrevigar",
    "entrevigaríais": "entrevigar",
    "entrevigaríamos": "entrevigar",
    "entrevigarían": "entrevigar",
    "entrevigarías": "entrevigar",
    "entrevigas": "entrevigar",
    "entrevigase": "entrevigar",
    "entrevigaseis": "entrevigar",
    "entrevigasen": "entrevigar",
    "entrevigases": "entrevigar",
    "entrevigaste": "entrevigar",
    "entrevigasteis": "entrevigar",
    "entrevigo": "entrevigar",
    "entrevigue": "entrevigar",
    "entreviguemos": "entrevigar",
    "entreviguen": "entrevigar",
    "entrevigues": "entrevigar",
    "entrevigué": "entrevigar",
    "entreviguéis": "entrevigar",
    "entrevigábamos": "entrevigar",
    "entrevigáis": "entrevigar",
    "entrevigáramos": "entrevigar",
    "entrevigáremos": "entrevigar",
    "entrevigásemos": "entrevigar",
    "entrevigó": "entrevigar",
    "entrevimos": "entrever",
    "entrevista": "entrevistar",
    "entrevistaba": "entrevistar",
    "entrevistabais": "entrevistar",
    "entrevistaban": "entrevistar",
    "entrevistabas": "entrevistar",
    "entrevistad": "entrevistar",
    "entrevistada": "entrevistar",
    "entrevistadas": "entrevistar",
    "entrevistado": "entrevistar",
    "entrevistadora": "entrevistador",
    "entrevistadoras": "entrevistador",
    "entrevistadores": "entrevistador",
    "entrevistados": "entrevistar",
    "entrevistamos": "entrevistar",
    "entrevistan": "entrevistar",
    "entrevistando": "entrevistar",
    "entrevistara": "entrevistar",
    "entrevistarais": "entrevistar",
    "entrevistaran": "entrevistar",
    "entrevistaras": "entrevistar",
    "entrevistare": "entrevistar",
    "entrevistareis": "entrevistar",
    "entrevistaremos": "entrevistar",
    "entrevistaren": "entrevistar",
    "entrevistares": "entrevistar",
    "entrevistaron": "entrevistar",
    "entrevistará": "entrevistar",
    "entrevistarán": "entrevistar",
    "entrevistarás": "entrevistar",
    "entrevistaré": "entrevistar",
    "entrevistaréis": "entrevistar",
    "entrevistaría": "entrevistar",
    "entrevistaríais": "entrevistar",
    "entrevistaríamos": "entrevistar",
    "entrevistarían": "entrevistar",
    "entrevistarías": "entrevistar",
    "entrevistas": "entrevistar",
    "entrevistase": "entrevistar",
    "entrevistaseis": "entrevistar",
    "entrevistasen": "entrevistar",
    "entrevistases": "entrevistar",
    "entrevistaste": "entrevistar",
    "entrevistasteis": "entrevistar",
    "entreviste": "entrevistar",
    "entrevisteis": "entrever",
    "entrevistemos": "entrevistar",
    "entrevisten": "entrevistar",
    "entrevistes": "entrevistar",
    "entrevisto": "entrevistar",
    "entrevistos": "entrever",
    "entrevistábamos": "entrevistar",
    "entrevistáis": "entrevistar",
    "entrevistáramos": "entrevistar",
    "entrevistáremos": "entrevistar",
    "entrevistásemos": "entrevistar",
    "entrevisté": "entrevistar",
    "entrevistéis": "entrevistar",
    "entrevistó": "entrevistar",
    "entreviéramos": "entrever",
    "entreviéremos": "entrever",
    "entreviésemos": "entrever",
    "entrevió": "entrever",
    "entrevueltas": "entrevuelta",
    "entrevé": "entrever",
    "entrevéis": "entrever",
    "entrevén": "entrever",
    "entrevés": "entrever",
    "entreví": "entrever",
    "entrevías": "entrevía",
    "entripa": "entripar",
    "entripaba": "entripar",
    "entripabais": "entripar",
    "entripaban": "entripar",
    "entripabas": "entripar",
    "entripad": "entripar",
    "entripada": "entripar",
    "entripadas": "entripar",
    "entripado": "entripar",
    "entripados": "entripar",
    "entripamos": "entripar",
    "entripan": "entripar",
    "entripando": "entripar",
    "entripara": "entripar",
    "entriparais": "entripar",
    "entriparan": "entripar",
    "entriparas": "entripar",
    "entripare": "entripar",
    "entripareis": "entripar",
    "entriparemos": "entripar",
    "entriparen": "entripar",
    "entripares": "entripar",
    "entriparon": "entripar",
    "entripará": "entripar",
    "entriparán": "entripar",
    "entriparás": "entripar",
    "entriparé": "entripar",
    "entriparéis": "entripar",
    "entriparía": "entripar",
    "entriparíais": "entripar",
    "entriparíamos": "entripar",
    "entriparían": "entripar",
    "entriparías": "entripar",
    "entripas": "entripar",
    "entripase": "entripar",
    "entripaseis": "entripar",
    "entripasen": "entripar",
    "entripases": "entripar",
    "entripaste": "entripar",
    "entripasteis": "entripar",
    "entripe": "entripar",
    "entripemos": "entripar",
    "entripen": "entripar",
    "entripes": "entripar",
    "entripo": "entripar",
    "entripábamos": "entripar",
    "entripáis": "entripar",
    "entripáramos": "entripar",
    "entripáremos": "entripar",
    "entripásemos": "entripar",
    "entripé": "entripar",
    "entripéis": "entripar",
    "entripó": "entripar",
    "entristece": "entristecer",
    "entristeced": "entristecer",
    "entristecemos": "entristecer",
    "entristecen": "entristecer",
    "entristeceremos": "entristecer",
    "entristecerá": "entristecer",
    "entristecerán": "entristecer",
    "entristecerás": "entristecer",
    "entristeceré": "entristecer",
    "entristeceréis": "entristecer",
    "entristecería": "entristecer",
    "entristeceríais": "entristecer",
    "entristeceríamos": "entristecer",
    "entristecerían": "entristecer",
    "entristecerías": "entristecer",
    "entristeces": "entristecer",
    "entristecida": "entristecer",
    "entristecidas": "entristecer",
    "entristecido": "entristecer",
    "entristecidos": "entristecer",
    "entristeciendo": "entristecer",
    "entristeciera": "entristecer",
    "entristecierais": "entristecer",
    "entristecieran": "entristecer",
    "entristecieras": "entristecer",
    "entristeciere": "entristecer",
    "entristeciereis": "entristecer",
    "entristecieren": "entristecer",
    "entristecieres": "entristecer",
    "entristecieron": "entristecer",
    "entristeciese": "entristecer",
    "entristecieseis": "entristecer",
    "entristeciesen": "entristecer",
    "entristecieses": "entristecer",
    "entristecimientos": "entristecimiento",
    "entristecimos": "entristecer",
    "entristeciste": "entristecer",
    "entristecisteis": "entristecer",
    "entristeciéramos": "entristecer",
    "entristeciéremos": "entristecer",
    "entristeciésemos": "entristecer",
    "entristeció": "entristecer",
    "entristecéis": "entristecer",
    "entristecí": "entristecer",
    "entristecía": "entristecer",
    "entristecíais": "entristecer",
    "entristecíamos": "entristecer",
    "entristecían": "entristecer",
    "entristecías": "entristecer",
    "entristezca": "entristecer",
    "entristezcamos": "entristecer",
    "entristezcan": "entristecer",
    "entristezcas": "entristecer",
    "entristezco": "entristecer",
    "entristezcáis": "entristecer",
    "entro": "entrar",
    "entroja": "entrojar",
    "entrojaba": "entrojar",
    "entrojabais": "entrojar",
    "entrojaban": "entrojar",
    "entrojabas": "entrojar",
    "entrojad": "entrojar",
    "entrojada": "entrojar",
    "entrojadas": "entrojar",
    "entrojado": "entrojar",
    "entrojados": "entrojar",
    "entrojamos": "entrojar",
    "entrojan": "entrojar",
    "entrojando": "entrojar",
    "entrojara": "entrojar",
    "entrojarais": "entrojar",
    "entrojaran": "entrojar",
    "entrojaras": "entrojar",
    "entrojare": "entrojar",
    "entrojareis": "entrojar",
    "entrojaremos": "entrojar",
    "entrojaren": "entrojar",
    "entrojares": "entrojar",
    "entrojaron": "entrojar",
    "entrojará": "entrojar",
    "entrojarán": "entrojar",
    "entrojarás": "entrojar",
    "entrojaré": "entrojar",
    "entrojaréis": "entrojar",
    "entrojaría": "entrojar",
    "entrojaríais": "entrojar",
    "entrojaríamos": "entrojar",
    "entrojarían": "entrojar",
    "entrojarías": "entrojar",
    "entrojas": "entrojar",
    "entrojase": "entrojar",
    "entrojaseis": "entrojar",
    "entrojasen": "entrojar",
    "entrojases": "entrojar",
    "entrojaste": "entrojar",
    "entrojasteis": "entrojar",
    "entroje": "entrojar",
    "entrojemos": "entrojar",
    "entrojen": "entrojar",
    "entrojes": "entrojar",
    "entrojo": "entrojar",
    "entrojábamos": "entrojar",
    "entrojáis": "entrojar",
    "entrojáramos": "entrojar",
    "entrojáremos": "entrojar",
    "entrojásemos": "entrojar",
    "entrojé": "entrojar",
    "entrojéis": "entrojar",
    "entrojó": "entrojar",
    "entrometa": "entrometer",
    "entrometamos": "entrometer",
    "entrometan": "entrometer",
    "entrometas": "entrometer",
    "entromete": "entrometer",
    "entrometed": "entrometer",
    "entrometemos": "entrometer",
    "entrometen": "entrometer",
    "entrometeremos": "entrometer",
    "entrometerá": "entrometer",
    "entrometerán": "entrometer",
    "entrometerás": "entrometer",
    "entrometeré": "entrometer",
    "entrometeréis": "entrometer",
    "entrometería": "entrometer",
    "entrometeríais": "entrometer",
    "entrometeríamos": "entrometer",
    "entrometerían": "entrometer",
    "entrometerías": "entrometer",
    "entrometes": "entrometer",
    "entrometida": "entrometer",
    "entrometidas": "entrometer",
    "entrometido": "entrometer",
    "entrometidos": "entrometer",
    "entrometiendo": "entrometer",
    "entrometiera": "entrometer",
    "entrometierais": "entrometer",
    "entrometieran": "entrometer",
    "entrometieras": "entrometer",
    "entrometiere": "entrometer",
    "entrometiereis": "entrometer",
    "entrometieren": "entrometer",
    "entrometieres": "entrometer",
    "entrometieron": "entrometer",
    "entrometiese": "entrometer",
    "entrometieseis": "entrometer",
    "entrometiesen": "entrometer",
    "entrometieses": "entrometer",
    "entrometimientos": "entrometimiento",
    "entrometimos": "entrometer",
    "entrometiste": "entrometer",
    "entrometisteis": "entrometer",
    "entrometiéramos": "entrometer",
    "entrometiéremos": "entrometer",
    "entrometiésemos": "entrometer",
    "entrometió": "entrometer",
    "entrometo": "entrometer",
    "entrometáis": "entrometer",
    "entrometéis": "entrometer",
    "entrometí": "entrometer",
    "entrometía": "entrometer",
    "entrometíais": "entrometer",
    "entrometíamos": "entrometer",
    "entrometían": "entrometer",
    "entrometías": "entrometer",
    "entrona": "entronar",
    "entronaba": "entronar",
    "entronabais": "entronar",
    "entronaban": "entronar",
    "entronabas": "entronar",
    "entronaciones": "entronación",
    "entronad": "entronar",
    "entronada": "entronar",
    "entronadas": "entronar",
    "entronado": "entronar",
    "entronados": "entronar",
    "entronamientos": "entronamiento",
    "entronamos": "entronar",
    "entronan": "entronar",
    "entronando": "entronar",
    "entronara": "entronar",
    "entronarais": "entronar",
    "entronaran": "entronar",
    "entronaras": "entronar",
    "entronare": "entronar",
    "entronareis": "entronar",
    "entronaremos": "entronar",
    "entronaren": "entronar",
    "entronares": "entronar",
    "entronaron": "entronar",
    "entronará": "entronar",
    "entronarán": "entronar",
    "entronarás": "entronar",
    "entronaré": "entronar",
    "entronaréis": "entronar",
    "entronaría": "entronar",
    "entronaríais": "entronar",
    "entronaríamos": "entronar",
    "entronarían": "entronar",
    "entronarías": "entronar",
    "entronas": "entronar",
    "entronase": "entronar",
    "entronaseis": "entronar",
    "entronasen": "entronar",
    "entronases": "entronar",
    "entronaste": "entronar",
    "entronasteis": "entronar",
    "entronca": "entroncar",
    "entroncaba": "entroncar",
    "entroncabais": "entroncar",
    "entroncaban": "entroncar",
    "entroncabas": "entroncar",
    "entroncad": "entroncar",
    "entroncada": "entroncar",
    "entroncadas": "entroncar",
    "entroncado": "entroncar",
    "entroncados": "entroncar",
    "entroncamientos": "entroncamiento",
    "entroncamos": "entroncar",
    "entroncan": "entroncar",
    "entroncando": "entroncar",
    "entroncara": "entroncar",
    "entroncarais": "entroncar",
    "entroncaran": "entroncar",
    "entroncaras": "entroncar",
    "entroncare": "entroncar",
    "entroncareis": "entroncar",
    "entroncaremos": "entroncar",
    "entroncaren": "entroncar",
    "entroncares": "entroncar",
    "entroncaron": "entroncar",
    "entroncará": "entroncar",
    "entroncarán": "entroncar",
    "entroncarás": "entroncar",
    "entroncaré": "entroncar",
    "entroncaréis": "entroncar",
    "entroncaría": "entroncar",
    "entroncaríais": "entroncar",
    "entroncaríamos": "entroncar",
    "entroncarían": "entroncar",
    "entroncarías": "entroncar",
    "entroncas": "entroncar",
    "entroncase": "entroncar",
    "entroncaseis": "entroncar",
    "entroncasen": "entroncar",
    "entroncases": "entroncar",
    "entroncaste": "entroncar",
    "entroncasteis": "entroncar",
    "entronco": "entroncar",
    "entroncábamos": "entroncar",
    "entroncáis": "entroncar",
    "entroncáramos": "entroncar",
    "entroncáremos": "entroncar",
    "entroncásemos": "entroncar",
    "entroncó": "entroncar",
    "entrone": "entronar",
    "entronemos": "entronar",
    "entronen": "entronar",
    "entrones": "entronar",
    "entronice": "entronizar",
    "entronicemos": "entronizar",
    "entronicen": "entronizar",
    "entronices": "entronizar",
    "entronicé": "entronizar",
    "entronicéis": "entronizar",
    "entroniza": "entronizar",
    "entronizaba": "entronizar",
    "entronizabais": "entronizar",
    "entronizaban": "entronizar",
    "entronizabas": "entronizar",
    "entronizaciones": "entronización",
    "entronizad": "entronizar",
    "entronizada": "entronizar",
    "entronizadas": "entronizar",
    "entronizado": "entronizar",
    "entronizados": "entronizar",
    "entronizamientos": "entronizamiento",
    "entronizamos": "entronizar",
    "entronizan": "entronizar",
    "entronizando": "entronizar",
    "entronizara": "entronizar",
    "entronizarais": "entronizar",
    "entronizaran": "entronizar",
    "entronizaras": "entronizar",
    "entronizare": "entronizar",
    "entronizareis": "entronizar",
    "entronizaremos": "entronizar",
    "entronizaren": "entronizar",
    "entronizares": "entronizar",
    "entronizaron": "entronizar",
    "entronizará": "entronizar",
    "entronizarán": "entronizar",
    "entronizarás": "entronizar",
    "entronizaré": "entronizar",
    "entronizaréis": "entronizar",
    "entronizaría": "entronizar",
    "entronizaríais": "entronizar",
    "entronizaríamos": "entronizar",
    "entronizarían": "entronizar",
    "entronizarías": "entronizar",
    "entronizas": "entronizar",
    "entronizase": "entronizar",
    "entronizaseis": "entronizar",
    "entronizasen": "entronizar",
    "entronizases": "entronizar",
    "entronizaste": "entronizar",
    "entronizasteis": "entronizar",
    "entronizo": "entronizar",
    "entronizábamos": "entronizar",
    "entronizáis": "entronizar",
    "entronizáramos": "entronizar",
    "entronizáremos": "entronizar",
    "entronizásemos": "entronizar",
    "entronizó": "entronizar",
    "entrono": "entronar",
    "entronque": "entroncar",
    "entronquemos": "entroncar",
    "entronquen": "entroncar",
    "entronques": "entronque",
    "entronqué": "entroncar",
    "entronquéis": "entroncar",
    "entronábamos": "entronar",
    "entronáis": "entronar",
    "entronáramos": "entronar",
    "entronáremos": "entronar",
    "entronásemos": "entronar",
    "entroné": "entronar",
    "entronéis": "entronar",
    "entronó": "entronar",
    "entropiones": "entropión",
    "entropías": "entropía",
    "entrucha": "entruchar",
    "entruchaba": "entruchar",
    "entruchabais": "entruchar",
    "entruchaban": "entruchar",
    "entruchabas": "entruchar",
    "entruchad": "entruchar",
    "entruchada": "entruchar",
    "entruchadas": "entruchar",
    "entruchado": "entruchar",
    "entruchados": "entruchar",
    "entruchamos": "entruchar",
    "entruchan": "entruchar",
    "entruchando": "entruchar",
    "entruchara": "entruchar",
    "entrucharais": "entruchar",
    "entrucharan": "entruchar",
    "entrucharas": "entruchar",
    "entruchare": "entruchar",
    "entruchareis": "entruchar",
    "entrucharemos": "entruchar",
    "entrucharen": "entruchar",
    "entruchares": "entruchar",
    "entrucharon": "entruchar",
    "entruchará": "entruchar",
    "entrucharán": "entruchar",
    "entrucharás": "entruchar",
    "entrucharé": "entruchar",
    "entrucharéis": "entruchar",
    "entrucharía": "entruchar",
    "entrucharíais": "entruchar",
    "entrucharíamos": "entruchar",
    "entrucharían": "entruchar",
    "entrucharías": "entruchar",
    "entruchas": "entruchar",
    "entruchase": "entruchar",
    "entruchaseis": "entruchar",
    "entruchasen": "entruchar",
    "entruchases": "entruchar",
    "entruchaste": "entruchar",
    "entruchasteis": "entruchar",
    "entruche": "entruchar",
    "entruchemos": "entruchar",
    "entruchen": "entruchar",
    "entruches": "entruchar",
    "entrucho": "entruchar",
    "entruchona": "entruchón",
    "entruchonas": "entruchón",
    "entruchones": "entruchón",
    "entruchábamos": "entruchar",
    "entrucháis": "entruchar",
    "entrucháramos": "entruchar",
    "entrucháremos": "entruchar",
    "entruchásemos": "entruchar",
    "entruché": "entruchar",
    "entruchéis": "entruchar",
    "entruchó": "entruchar",
    "entruja": "entrujar",
    "entrujaba": "entrujar",
    "entrujabais": "entrujar",
    "entrujaban": "entrujar",
    "entrujabas": "entrujar",
    "entrujad": "entrujar",
    "entrujada": "entrujar",
    "entrujadas": "entrujar",
    "entrujado": "entrujar",
    "entrujados": "entrujar",
    "entrujamos": "entrujar",
    "entrujan": "entrujar",
    "entrujando": "entrujar",
    "entrujara": "entrujar",
    "entrujarais": "entrujar",
    "entrujaran": "entrujar",
    "entrujaras": "entrujar",
    "entrujare": "entrujar",
    "entrujareis": "entrujar",
    "entrujaremos": "entrujar",
    "entrujaren": "entrujar",
    "entrujares": "entrujar",
    "entrujaron": "entrujar",
    "entrujará": "entrujar",
    "entrujarán": "entrujar",
    "entrujarás": "entrujar",
    "entrujaré": "entrujar",
    "entrujaréis": "entrujar",
    "entrujaría": "entrujar",
    "entrujaríais": "entrujar",
    "entrujaríamos": "entrujar",
    "entrujarían": "entrujar",
    "entrujarías": "entrujar",
    "entrujas": "entrujar",
    "entrujase": "entrujar",
    "entrujaseis": "entrujar",
    "entrujasen": "entrujar",
    "entrujases": "entrujar",
    "entrujaste": "entrujar",
    "entrujasteis": "entrujar",
    "entruje": "entrujar",
    "entrujemos": "entrujar",
    "entrujen": "entrujar",
    "entrujes": "entrujar",
    "entrujo": "entrujar",
    "entrujábamos": "entrujar",
    "entrujáis": "entrujar",
    "entrujáramos": "entrujar",
    "entrujáremos": "entrujar",
    "entrujásemos": "entrujar",
    "entrujé": "entrujar",
    "entrujéis": "entrujar",
    "entrujó": "entrujar",
    "entrábamos": "entrar",
    "entráis": "entrar",
    "entráramos": "entrar",
    "entráremos": "entrar",
    "entrásemos": "entrar",
    "entré": "entrar",
    "entréis": "entrar",
    "entró": "entrar",
    "entrópica": "entrópico",
    "entrópicas": "entrópico",
    "entrópicos": "entrópico",
    "entuba": "entubar",
    "entubaba": "entubar",
    "entubabais": "entubar",
    "entubaban": "entubar",
    "entubabas": "entubar",
    "entubaciones": "entubación",
    "entubad": "entubar",
    "entubada": "entubar",
    "entubadas": "entubar",
    "entubado": "entubar",
    "entubados": "entubar",
    "entubamos": "entubar",
    "entuban": "entubar",
    "entubando": "entubar",
    "entubara": "entubar",
    "entubarais": "entubar",
    "entubaran": "entubar",
    "entubaras": "entubar",
    "entubare": "entubar",
    "entubareis": "entubar",
    "entubaremos": "entubar",
    "entubaren": "entubar",
    "entubares": "entubar",
    "entubaron": "entubar",
    "entubará": "entubar",
    "entubarán": "entubar",
    "entubarás": "entubar",
    "entubaré": "entubar",
    "entubaréis": "entubar",
    "entubaría": "entubar",
    "entubaríais": "entubar",
    "entubaríamos": "entubar",
    "entubarían": "entubar",
    "entubarías": "entubar",
    "entubas": "entubar",
    "entubase": "entubar",
    "entubaseis": "entubar",
    "entubasen": "entubar",
    "entubases": "entubar",
    "entubaste": "entubar",
    "entubasteis": "entubar",
    "entube": "entubar",
    "entubemos": "entubar",
    "entuben": "entubar",
    "entubes": "entubar",
    "entubo": "entubar",
    "entubábamos": "entubar",
    "entubáis": "entubar",
    "entubáramos": "entubar",
    "entubáremos": "entubar",
    "entubásemos": "entubar",
    "entubé": "entubar",
    "entubéis": "entubar",
    "entubó": "entubar",
    "entuerta": "entortar",
    "entuertan": "entortar",
    "entuertas": "entortar",
    "entuerte": "entortar",
    "entuerten": "entortar",
    "entuertes": "entortar",
    "entuerto": "entortar",
    "entuertos": "entuerto",
    "entullece": "entullecer",
    "entulleced": "entullecer",
    "entullecemos": "entullecer",
    "entullecen": "entullecer",
    "entulleceremos": "entullecer",
    "entullecerá": "entullecer",
    "entullecerán": "entullecer",
    "entullecerás": "entullecer",
    "entulleceré": "entullecer",
    "entulleceréis": "entullecer",
    "entullecería": "entullecer",
    "entulleceríais": "entullecer",
    "entulleceríamos": "entullecer",
    "entullecerían": "entullecer",
    "entullecerías": "entullecer",
    "entulleces": "entullecer",
    "entullecida": "entullecer",
    "entullecidas": "entullecer",
    "entullecido": "entullecer",
    "entullecidos": "entullecer",
    "entulleciendo": "entullecer",
    "entulleciera": "entullecer",
    "entullecierais": "entullecer",
    "entullecieran": "entullecer",
    "entullecieras": "entullecer",
    "entulleciere": "entullecer",
    "entulleciereis": "entullecer",
    "entullecieren": "entullecer",
    "entullecieres": "entullecer",
    "entullecieron": "entullecer",
    "entulleciese": "entullecer",
    "entullecieseis": "entullecer",
    "entulleciesen": "entullecer",
    "entullecieses": "entullecer",
    "entullecimos": "entullecer",
    "entulleciste": "entullecer",
    "entullecisteis": "entullecer",
    "entulleciéramos": "entullecer",
    "entulleciéremos": "entullecer",
    "entulleciésemos": "entullecer",
    "entulleció": "entullecer",
    "entullecéis": "entullecer",
    "entullecí": "entullecer",
    "entullecía": "entullecer",
    "entullecíais": "entullecer",
    "entullecíamos": "entullecer",
    "entullecían": "entullecer",
    "entullecías": "entullecer",
    "entullezca": "entullecer",
    "entullezcamos": "entullecer",
    "entullezcan": "entullecer",
    "entullezcas": "entullecer",
    "entullezco": "entullecer",
    "entullezcáis": "entullecer",
    "entuma": "entumir",
    "entumamos": "entumir",
    "entuman": "entumir",
    "entumas": "entumir",
    "entume": "entumir",
    "entumece": "entumecer",
    "entumeced": "entumecer",
    "entumecemos": "entumecer",
    "entumecen": "entumecer",
    "entumeceremos": "entumecer",
    "entumecerá": "entumecer",
    "entumecerán": "entumecer",
    "entumecerás": "entumecer",
    "entumeceré": "entumecer",
    "entumeceréis": "entumecer",
    "entumecería": "entumecer",
    "entumeceríais": "entumecer",
    "entumeceríamos": "entumecer",
    "entumecerían": "entumecer",
    "entumecerías": "entumecer",
    "entumeces": "entumecer",
    "entumecida": "entumecer",
    "entumecidas": "entumecer",
    "entumecido": "entumecer",
    "entumecidos": "entumecer",
    "entumeciendo": "entumecer",
    "entumeciera": "entumecer",
    "entumecierais": "entumecer",
    "entumecieran": "entumecer",
    "entumecieras": "entumecer",
    "entumeciere": "entumecer",
    "entumeciereis": "entumecer",
    "entumecieren": "entumecer",
    "entumecieres": "entumecer",
    "entumecieron": "entumecer",
    "entumeciese": "entumecer",
    "entumecieseis": "entumecer",
    "entumeciesen": "entumecer",
    "entumecieses": "entumecer",
    "entumecimientos": "entumecimiento",
    "entumecimos": "entumecer",
    "entumeciste": "entumecer",
    "entumecisteis": "entumecer",
    "entumeciéramos": "entumecer",
    "entumeciéremos": "entumecer",
    "entumeciésemos": "entumecer",
    "entumeció": "entumecer",
    "entumecéis": "entumecer",
    "entumecí": "entumecer",
    "entumecía": "entumecer",
    "entumecíais": "entumecer",
    "entumecíamos": "entumecer",
    "entumecían": "entumecer",
    "entumecías": "entumecer",
    "entumen": "entumir",
    "entumes": "entumir",
    "entumezca": "entumecer",
    "entumezcamos": "entumecer",
    "entumezcan": "entumecer",
    "entumezcas": "entumecer",
    "entumezco": "entumecer",
    "entumezcáis": "entumecer",
    "entumid": "entumir",
    "entumida": "entumir",
    "entumidas": "entumir",
    "entumido": "entumir",
    "entumidos": "entumir",
    "entumiendo": "entumir",
    "entumiera": "entumir",
    "entumierais": "entumir",
    "entumieran": "entumir",
    "entumieras": "entumir",
    "entumiere": "entumir",
    "entumiereis": "entumir",
    "entumieren": "entumir",
    "entumieres": "entumir",
    "entumieron": "entumir",
    "entumiese": "entumir",
    "entumieseis": "entumir",
    "entumiesen": "entumir",
    "entumieses": "entumir",
    "entumimos": "entumir",
    "entumiremos": "entumir",
    "entumirá": "entumir",
    "entumirán": "entumir",
    "entumirás": "entumir",
    "entumiré": "entumir",
    "entumiréis": "entumir",
    "entumiría": "entumir",
    "entumiríais": "entumir",
    "entumiríamos": "entumir",
    "entumirían": "entumir",
    "entumirías": "entumir",
    "entumiste": "entumir",
    "entumisteis": "entumir",
    "entumiéramos": "entumir",
    "entumiéremos": "entumir",
    "entumiésemos": "entumir",
    "entumió": "entumir",
    "entumo": "entumir",
    "entumáis": "entumir",
    "entumí": "entumir",
    "entumía": "entumir",
    "entumíais": "entumir",
    "entumíamos": "entumir",
    "entumían": "entumir",
    "entumías": "entumir",
    "entumís": "entumir",
    "entunica": "entunicar",
    "entunicaba": "entunicar",
    "entunicabais": "entunicar",
    "entunicaban": "entunicar",
    "entunicabas": "entunicar",
    "entunicad": "entunicar",
    "entunicada": "entunicar",
    "entunicadas": "entunicar",
    "entunicado": "entunicar",
    "entunicados": "entunicar",
    "entunicamos": "entunicar",
    "entunican": "entunicar",
    "entunicando": "entunicar",
    "entunicara": "entunicar",
    "entunicarais": "entunicar",
    "entunicaran": "entunicar",
    "entunicaras": "entunicar",
    "entunicare": "entunicar",
    "entunicareis": "entunicar",
    "entunicaremos": "entunicar",
    "entunicaren": "entunicar",
    "entunicares": "entunicar",
    "entunicaron": "entunicar",
    "entunicará": "entunicar",
    "entunicarán": "entunicar",
    "entunicarás": "entunicar",
    "entunicaré": "entunicar",
    "entunicaréis": "entunicar",
    "entunicaría": "entunicar",
    "entunicaríais": "entunicar",
    "entunicaríamos": "entunicar",
    "entunicarían": "entunicar",
    "entunicarías": "entunicar",
    "entunicas": "entunicar",
    "entunicase": "entunicar",
    "entunicaseis": "entunicar",
    "entunicasen": "entunicar",
    "entunicases": "entunicar",
    "entunicaste": "entunicar",
    "entunicasteis": "entunicar",
    "entunico": "entunicar",
    "entunicábamos": "entunicar",
    "entunicáis": "entunicar",
    "entunicáramos": "entunicar",
    "entunicáremos": "entunicar",
    "entunicásemos": "entunicar",
    "entunicó": "entunicar",
    "entunique": "entunicar",
    "entuniquemos": "entunicar",
    "entuniquen": "entunicar",
    "entuniques": "entunicar",
    "entuniqué": "entunicar",
    "entuniquéis": "entunicar",
    "entupa": "entupir",
    "entupamos": "entupir",
    "entupan": "entupir",
    "entupas": "entupir",
    "entupe": "entupir",
    "entupen": "entupir",
    "entupes": "entupir",
    "entupid": "entupir",
    "entupida": "entupir",
    "entupidas": "entupir",
    "entupido": "entupir",
    "entupidos": "entupir",
    "entupiendo": "entupir",
    "entupiera": "entupir",
    "entupierais": "entupir",
    "entupieran": "entupir",
    "entupieras": "entupir",
    "entupiere": "entupir",
    "entupiereis": "entupir",
    "entupieren": "entupir",
    "entupieres": "entupir",
    "entupieron": "entupir",
    "entupiese": "entupir",
    "entupieseis": "entupir",
    "entupiesen": "entupir",
    "entupieses": "entupir",
    "entupimos": "entupir",
    "entupiremos": "entupir",
    "entupirá": "entupir",
    "entupirán": "entupir",
    "entupirás": "entupir",
    "entupiré": "entupir",
    "entupiréis": "entupir",
    "entupiría": "entupir",
    "entupiríais": "entupir",
    "entupiríamos": "entupir",
    "entupirían": "entupir",
    "entupirías": "entupir",
    "entupiste": "entupir",
    "entupisteis": "entupir",
    "entupiéramos": "entupir",
    "entupiéremos": "entupir",
    "entupiésemos": "entupir",
    "entupió": "entupir",
    "entupo": "entupir",
    "entupáis": "entupir",
    "entupí": "entupir",
    "entupía": "entupir",
    "entupíais": "entupir",
    "entupíamos": "entupir",
    "entupían": "entupir",
    "entupías": "entupir",
    "entupís": "entupir",
    "enturbia": "enturbiar",
    "enturbiaba": "enturbiar",
    "enturbiabais": "enturbiar",
    "enturbiaban": "enturbiar",
    "enturbiabas": "enturbiar",
    "enturbiad": "enturbiar",
    "enturbiada": "enturbiar",
    "enturbiadas": "enturbiar",
    "enturbiado": "enturbiar",
    "enturbiados": "enturbiar",
    "enturbiamientos": "enturbiamiento",
    "enturbiamos": "enturbiar",
    "enturbian": "enturbiar",
    "enturbiando": "enturbiar",
    "enturbiara": "enturbiar",
    "enturbiarais": "enturbiar",
    "enturbiaran": "enturbiar",
    "enturbiaras": "enturbiar",
    "enturbiare": "enturbiar",
    "enturbiareis": "enturbiar",
    "enturbiaremos": "enturbiar",
    "enturbiaren": "enturbiar",
    "enturbiares": "enturbiar",
    "enturbiaron": "enturbiar",
    "enturbiará": "enturbiar",
    "enturbiarán": "enturbiar",
    "enturbiarás": "enturbiar",
    "enturbiaré": "enturbiar",
    "enturbiaréis": "enturbiar",
    "enturbiaría": "enturbiar",
    "enturbiaríais": "enturbiar",
    "enturbiaríamos": "enturbiar",
    "enturbiarían": "enturbiar",
    "enturbiarías": "enturbiar",
    "enturbias": "enturbiar",
    "enturbiase": "enturbiar",
    "enturbiaseis": "enturbiar",
    "enturbiasen": "enturbiar",
    "enturbiases": "enturbiar",
    "enturbiaste": "enturbiar",
    "enturbiasteis": "enturbiar",
    "enturbie": "enturbiar",
    "enturbiemos": "enturbiar",
    "enturbien": "enturbiar",
    "enturbies": "enturbiar",
    "enturbio": "enturbiar",
    "enturbiábamos": "enturbiar",
    "enturbiáis": "enturbiar",
    "enturbiáramos": "enturbiar",
    "enturbiáremos": "enturbiar",
    "enturbiásemos": "enturbiar",
    "enturbié": "enturbiar",
    "enturbiéis": "enturbiar",
    "enturbió": "enturbiar",
    "entusiasma": "entusiasmar",
    "entusiasmaba": "entusiasmar",
    "entusiasmabais": "entusiasmar",
    "entusiasmaban": "entusiasmar",
    "entusiasmabas": "entusiasmar",
    "entusiasmad": "entusiasmar",
    "entusiasmada": "entusiasmar",
    "entusiasmadas": "entusiasmar",
    "entusiasmado": "entusiasmar",
    "entusiasmados": "entusiasmar",
    "entusiasmamos": "entusiasmar",
    "entusiasman": "entusiasmar",
    "entusiasmando": "entusiasmar",
    "entusiasmara": "entusiasmar",
    "entusiasmarais": "entusiasmar",
    "entusiasmaran": "entusiasmar",
    "entusiasmaras": "entusiasmar",
    "entusiasmare": "entusiasmar",
    "entusiasmareis": "entusiasmar",
    "entusiasmaremos": "entusiasmar",
    "entusiasmaren": "entusiasmar",
    "entusiasmares": "entusiasmar",
    "entusiasmaron": "entusiasmar",
    "entusiasmará": "entusiasmar",
    "entusiasmarán": "entusiasmar",
    "entusiasmarás": "entusiasmar",
    "entusiasmaré": "entusiasmar",
    "entusiasmaréis": "entusiasmar",
    "entusiasmaría": "entusiasmar",
    "entusiasmaríais": "entusiasmar",
    "entusiasmaríamos": "entusiasmar",
    "entusiasmarían": "entusiasmar",
    "entusiasmarías": "entusiasmar",
    "entusiasmas": "entusiasmar",
    "entusiasmase": "entusiasmar",
    "entusiasmaseis": "entusiasmar",
    "entusiasmasen": "entusiasmar",
    "entusiasmases": "entusiasmar",
    "entusiasmaste": "entusiasmar",
    "entusiasmasteis": "entusiasmar",
    "entusiasme": "entusiasmar",
    "entusiasmemos": "entusiasmar",
    "entusiasmen": "entusiasmar",
    "entusiasmes": "entusiasmar",
    "entusiasmo": "entusiasmar",
    "entusiasmos": "entusiasmo",
    "entusiasmábamos": "entusiasmar",
    "entusiasmáis": "entusiasmar",
    "entusiasmáramos": "entusiasmar",
    "entusiasmáremos": "entusiasmar",
    "entusiasmásemos": "entusiasmar",
    "entusiasmé": "entusiasmar",
    "entusiasméis": "entusiasmar",
    "entusiasmó": "entusiasmar",
    "entusiastas": "entusiasta",
    "entusiástica": "entusiástico",
    "entusiásticas": "entusiástico",
    "entusiásticos": "entusiástico",
    "entálpica": "entálpico",
    "entálpicas": "entálpico",
    "entálpicos": "entálpico",
    "entérica": "entérico",
    "entéricas": "entérico",
    "entéricos": "entérico",
    "enuclea": "enuclear",
    "enucleaba": "enuclear",
    "enucleabais": "enuclear",
    "enucleaban": "enuclear",
    "enucleabas": "enuclear",
    "enucleaciones": "enucleación",
    "enuclead": "enuclear",
    "enucleada": "enuclear",
    "enucleadas": "enuclear",
    "enucleado": "enuclear",
    "enucleados": "enuclear",
    "enucleamos": "enuclear",
    "enuclean": "enuclear",
    "enucleando": "enuclear",
    "enucleara": "enuclear",
    "enuclearais": "enuclear",
    "enuclearan": "enuclear",
    "enuclearas": "enuclear",
    "enucleare": "enuclear",
    "enucleareis": "enuclear",
    "enuclearemos": "enuclear",
    "enuclearen": "enuclear",
    "enucleares": "enuclear",
    "enuclearon": "enuclear",
    "enucleará": "enuclear",
    "enuclearán": "enuclear",
    "enuclearás": "enuclear",
    "enuclearé": "enuclear",
    "enuclearéis": "enuclear",
    "enuclearía": "enuclear",
    "enuclearíais": "enuclear",
    "enuclearíamos": "enuclear",
    "enuclearían": "enuclear",
    "enuclearías": "enuclear",
    "enucleas": "enuclear",
    "enuclease": "enuclear",
    "enucleaseis": "enuclear",
    "enucleasen": "enuclear",
    "enucleases": "enuclear",
    "enucleaste": "enuclear",
    "enucleasteis": "enuclear",
    "enuclee": "enuclear",
    "enucleemos": "enuclear",
    "enucleen": "enuclear",
    "enuclees": "enuclear",
    "enucleo": "enuclear",
    "enucleábamos": "enuclear",
    "enucleáis": "enuclear",
    "enucleáramos": "enuclear",
    "enucleáremos": "enuclear",
    "enucleásemos": "enuclear",
    "enucleé": "enuclear",
    "enucleéis": "enuclear",
    "enucleó": "enuclear",
    "enumera": "enumerar",
    "enumeraba": "enumerar",
    "enumerabais": "enumerar",
    "enumeraban": "enumerar",
    "enumerabas": "enumerar",
    "enumerables": "enumerable",
    "enumeraciones": "enumeración",
    "enumerad": "enumerar",
    "enumerada": "enumerar",
    "enumeradas": "enumerar",
    "enumerado": "enumerar",
    "enumerados": "enumerar",
    "enumeramos": "enumerar",
    "enumeran": "enumerar",
    "enumerando": "enumerar",
    "enumerara": "enumerar",
    "enumerarais": "enumerar",
    "enumeraran": "enumerar",
    "enumeraras": "enumerar",
    "enumerare": "enumerar",
    "enumerareis": "enumerar",
    "enumeraremos": "enumerar",
    "enumeraren": "enumerar",
    "enumerares": "enumerar",
    "enumeraron": "enumerar",
    "enumerará": "enumerar",
    "enumerarán": "enumerar",
    "enumerarás": "enumerar",
    "enumeraré": "enumerar",
    "enumeraréis": "enumerar",
    "enumeraría": "enumerar",
    "enumeraríais": "enumerar",
    "enumeraríamos": "enumerar",
    "enumerarían": "enumerar",
    "enumerarías": "enumerar",
    "enumeras": "enumerar",
    "enumerase": "enumerar",
    "enumeraseis": "enumerar",
    "enumerasen": "enumerar",
    "enumerases": "enumerar",
    "enumeraste": "enumerar",
    "enumerasteis": "enumerar",
    "enumerativa": "enumerativo",
    "enumerativas": "enumerativo",
    "enumerativos": "enumerativo",
    "enumere": "enumerar",
    "enumeremos": "enumerar",
    "enumeren": "enumerar",
    "enumeres": "enumerar",
    "enumero": "enumerar",
    "enumerábamos": "enumerar",
    "enumeráis": "enumerar",
    "enumeráramos": "enumerar",
    "enumeráremos": "enumerar",
    "enumerásemos": "enumerar",
    "enumeré": "enumerar",
    "enumeréis": "enumerar",
    "enumeró": "enumerar",
    "enuncia": "enunciar",
    "enunciaba": "enunciar",
    "enunciabais": "enunciar",
    "enunciaban": "enunciar",
    "enunciabas": "enunciar",
    "enunciaciones": "enunciación",
    "enunciad": "enunciar",
    "enunciada": "enunciar",
    "enunciadas": "enunciar",
    "enunciado": "enunciar",
    "enunciados": "enunciar",
    "enunciamos": "enunciar",
    "enuncian": "enunciar",
    "enunciando": "enunciar",
    "enunciara": "enunciar",
    "enunciarais": "enunciar",
    "enunciaran": "enunciar",
    "enunciaras": "enunciar",
    "enunciare": "enunciar",
    "enunciareis": "enunciar",
    "enunciaremos": "enunciar",
    "enunciaren": "enunciar",
    "enunciares": "enunciar",
    "enunciaron": "enunciar",
    "enunciará": "enunciar",
    "enunciarán": "enunciar",
    "enunciarás": "enunciar",
    "enunciaré": "enunciar",
    "enunciaréis": "enunciar",
    "enunciaría": "enunciar",
    "enunciaríais": "enunciar",
    "enunciaríamos": "enunciar",
    "enunciarían": "enunciar",
    "enunciarías": "enunciar",
    "enuncias": "enunciar",
    "enunciase": "enunciar",
    "enunciaseis": "enunciar",
    "enunciasen": "enunciar",
    "enunciases": "enunciar",
    "enunciaste": "enunciar",
    "enunciasteis": "enunciar",
    "enunciativa": "enunciativo",
    "enunciativas": "enunciativo",
    "enunciativos": "enunciativo",
    "enuncie": "enunciar",
    "enunciemos": "enunciar",
    "enuncien": "enunciar",
    "enuncies": "enunciar",
    "enuncio": "enunciar",
    "enunciábamos": "enunciar",
    "enunciáis": "enunciar",
    "enunciáramos": "enunciar",
    "enunciáremos": "enunciar",
    "enunciásemos": "enunciar",
    "enuncié": "enunciar",
    "enunciéis": "enunciar",
    "enunció": "enunciar",
    "envaina": "envainar",
    "envainaba": "envainar",
    "envainabais": "envainar",
    "envainaban": "envainar",
    "envainabas": "envainar",
    "envainad": "envainar",
    "envainada": "envainar",
    "envainadas": "envainar",
    "envainado": "envainar",
    "envainadora": "envainador",
    "envainadoras": "envainador",
    "envainadores": "envainador",
    "envainados": "envainar",
    "envainamos": "envainar",
    "envainan": "envainar",
    "envainando": "envainar",
    "envainara": "envainar",
    "envainarais": "envainar",
    "envainaran": "envainar",
    "envainaras": "envainar",
    "envainare": "envainar",
    "envainareis": "envainar",
    "envainaremos": "envainar",
    "envainaren": "envainar",
    "envainares": "envainar",
    "envainaron": "envainar",
    "envainará": "envainar",
    "envainarán": "envainar",
    "envainarás": "envainar",
    "envainaré": "envainar",
    "envainaréis": "envainar",
    "envainaría": "envainar",
    "envainaríais": "envainar",
    "envainaríamos": "envainar",
    "envainarían": "envainar",
    "envainarías": "envainar",
    "envainas": "envainar",
    "envainase": "envainar",
    "envainaseis": "envainar",
    "envainasen": "envainar",
    "envainases": "envainar",
    "envainaste": "envainar",
    "envainasteis": "envainar",
    "envaine": "envainar",
    "envainemos": "envainar",
    "envainen": "envainar",
    "envaines": "envainar",
    "envaino": "envainar",
    "envainábamos": "envainar",
    "envaináis": "envainar",
    "envaináramos": "envainar",
    "envaináremos": "envainar",
    "envainásemos": "envainar",
    "envainé": "envainar",
    "envainéis": "envainar",
    "envainó": "envainar",
    "envalentona": "envalentonar",
    "envalentonaba": "envalentonar",
    "envalentonabais": "envalentonar",
    "envalentonaban": "envalentonar",
    "envalentonabas": "envalentonar",
    "envalentonad": "envalentonar",
    "envalentonada": "envalentonar",
    "envalentonadas": "envalentonar",
    "envalentonado": "envalentonar",
    "envalentonados": "envalentonar",
    "envalentonamientos": "envalentonamiento",
    "envalentonamos": "envalentonar",
    "envalentonan": "envalentonar",
    "envalentonando": "envalentonar",
    "envalentonara": "envalentonar",
    "envalentonarais": "envalentonar",
    "envalentonaran": "envalentonar",
    "envalentonaras": "envalentonar",
    "envalentonare": "envalentonar",
    "envalentonareis": "envalentonar",
    "envalentonaremos": "envalentonar",
    "envalentonaren": "envalentonar",
    "envalentonares": "envalentonar",
    "envalentonaron": "envalentonar",
    "envalentonará": "envalentonar",
    "envalentonarán": "envalentonar",
    "envalentonarás": "envalentonar",
    "envalentonaré": "envalentonar",
    "envalentonaréis": "envalentonar",
    "envalentonaría": "envalentonar",
    "envalentonaríais": "envalentonar",
    "envalentonaríamos": "envalentonar",
    "envalentonarían": "envalentonar",
    "envalentonarías": "envalentonar",
    "envalentonas": "envalentonar",
    "envalentonase": "envalentonar",
    "envalentonaseis": "envalentonar",
    "envalentonasen": "envalentonar",
    "envalentonases": "envalentonar",
    "envalentonaste": "envalentonar",
    "envalentonasteis": "envalentonar",
    "envalentone": "envalentonar",
    "envalentonemos": "envalentonar",
    "envalentonen": "envalentonar",
    "envalentones": "envalentonar",
    "envalentono": "envalentonar",
    "envalentonábamos": "envalentonar",
    "envalentonáis": "envalentonar",
    "envalentonáramos": "envalentonar",
    "envalentonáremos": "envalentonar",
    "envalentonásemos": "envalentonar",
    "envalentoné": "envalentonar",
    "envalentonéis": "envalentonar",
    "envalentonó": "envalentonar",
    "envalija": "envalijar",
    "envalijaba": "envalijar",
    "envalijabais": "envalijar",
    "envalijaban": "envalijar",
    "envalijabas": "envalijar",
    "envalijad": "envalijar",
    "envalijada": "envalijar",
    "envalijadas": "envalijar",
    "envalijado": "envalijar",
    "envalijados": "envalijar",
    "envalijamos": "envalijar",
    "envalijan": "envalijar",
    "envalijando": "envalijar",
    "envalijara": "envalijar",
    "envalijarais": "envalijar",
    "envalijaran": "envalijar",
    "envalijaras": "envalijar",
    "envalijare": "envalijar",
    "envalijareis": "envalijar",
    "envalijaremos": "envalijar",
    "envalijaren": "envalijar",
    "envalijares": "envalijar",
    "envalijaron": "envalijar",
    "envalijará": "envalijar",
    "envalijarán": "envalijar",
    "envalijarás": "envalijar",
    "envalijaré": "envalijar",
    "envalijaréis": "envalijar",
    "envalijaría": "envalijar",
    "envalijaríais": "envalijar",
    "envalijaríamos": "envalijar",
    "envalijarían": "envalijar",
    "envalijarías": "envalijar",
    "envalijas": "envalijar",
    "envalijase": "envalijar",
    "envalijaseis": "envalijar",
    "envalijasen": "envalijar",
    "envalijases": "envalijar",
    "envalijaste": "envalijar",
    "envalijasteis": "envalijar",
    "envalije": "envalijar",
    "envalijemos": "envalijar",
    "envalijen": "envalijar",
    "envalijes": "envalijar",
    "envalijo": "envalijar",
    "envalijábamos": "envalijar",
    "envalijáis": "envalijar",
    "envalijáramos": "envalijar",
    "envalijáremos": "envalijar",
    "envalijásemos": "envalijar",
    "envalijé": "envalijar",
    "envalijéis": "envalijar",
    "envalijó": "envalijar",
    "envanece": "envanecer",
    "envaneced": "envanecer",
    "envanecemos": "envanecer",
    "envanecen": "envanecer",
    "envaneceremos": "envanecer",
    "envanecerá": "envanecer",
    "envanecerán": "envanecer",
    "envanecerás": "envanecer",
    "envaneceré": "envanecer",
    "envaneceréis": "envanecer",
    "envanecería": "envanecer",
    "envaneceríais": "envanecer",
    "envaneceríamos": "envanecer",
    "envanecerían": "envanecer",
    "envanecerías": "envanecer",
    "envaneces": "envanecer",
    "envanecida": "envanecer",
    "envanecidas": "envanecer",
    "envanecido": "envanecer",
    "envanecidos": "envanecer",
    "envaneciendo": "envanecer",
    "envaneciera": "envanecer",
    "envanecierais": "envanecer",
    "envanecieran": "envanecer",
    "envanecieras": "envanecer",
    "envaneciere": "envanecer",
    "envaneciereis": "envanecer",
    "envanecieren": "envanecer",
    "envanecieres": "envanecer",
    "envanecieron": "envanecer",
    "envaneciese": "envanecer",
    "envanecieseis": "envanecer",
    "envaneciesen": "envanecer",
    "envanecieses": "envanecer",
    "envanecimientos": "envanecimiento",
    "envanecimos": "envanecer",
    "envaneciste": "envanecer",
    "envanecisteis": "envanecer",
    "envaneciéramos": "envanecer",
    "envaneciéremos": "envanecer",
    "envaneciésemos": "envanecer",
    "envaneció": "envanecer",
    "envanecéis": "envanecer",
    "envanecí": "envanecer",
    "envanecía": "envanecer",
    "envanecíais": "envanecer",
    "envanecíamos": "envanecer",
    "envanecían": "envanecer",
    "envanecías": "envanecer",
    "envanezca": "envanecer",
    "envanezcamos": "envanecer",
    "envanezcan": "envanecer",
    "envanezcas": "envanecer",
    "envanezco": "envanecer",
    "envanezcáis": "envanecer",
    "envara": "envarar",
    "envaraba": "envarar",
    "envarabais": "envarar",
    "envaraban": "envarar",
    "envarabas": "envarar",
    "envarad": "envarar",
    "envarada": "envarar",
    "envaradas": "envarar",
    "envarado": "envarar",
    "envarados": "envarar",
    "envaramientos": "envaramiento",
    "envaramos": "envarar",
    "envaran": "envarar",
    "envarando": "envarar",
    "envarara": "envarar",
    "envararais": "envarar",
    "envararan": "envarar",
    "envararas": "envarar",
    "envarare": "envarar",
    "envarareis": "envarar",
    "envararemos": "envarar",
    "envararen": "envarar",
    "envarares": "envarar",
    "envararon": "envarar",
    "envarará": "envarar",
    "envararán": "envarar",
    "envararás": "envarar",
    "envararé": "envarar",
    "envararéis": "envarar",
    "envararía": "envarar",
    "envararíais": "envarar",
    "envararíamos": "envarar",
    "envararían": "envarar",
    "envararías": "envarar",
    "envaras": "envarar",
    "envarase": "envarar",
    "envaraseis": "envarar",
    "envarasen": "envarar",
    "envarases": "envarar",
    "envaraste": "envarar",
    "envarasteis": "envarar",
    "envare": "envarar",
    "envaremos": "envarar",
    "envaren": "envarar",
    "envares": "envarar",
    "envaro": "envarar",
    "envarábamos": "envarar",
    "envaráis": "envarar",
    "envaráramos": "envarar",
    "envaráremos": "envarar",
    "envarásemos": "envarar",
    "envaré": "envarar",
    "envaréis": "envarar",
    "envaró": "envarar",
    "envasa": "envasar",
    "envasaba": "envasar",
    "envasabais": "envasar",
    "envasaban": "envasar",
    "envasabas": "envasar",
    "envasad": "envasar",
    "envasada": "envasar",
    "envasadas": "envasar",
    "envasado": "envasar",
    "envasadora": "envasador",
    "envasadoras": "envasador",
    "envasadores": "envasador",
    "envasados": "envasar",
    "envasamos": "envasar",
    "envasan": "envasar",
    "envasando": "envasar",
    "envasara": "envasar",
    "envasarais": "envasar",
    "envasaran": "envasar",
    "envasaras": "envasar",
    "envasare": "envasar",
    "envasareis": "envasar",
    "envasaremos": "envasar",
    "envasaren": "envasar",
    "envasares": "envasar",
    "envasaron": "envasar",
    "envasará": "envasar",
    "envasarán": "envasar",
    "envasarás": "envasar",
    "envasaré": "envasar",
    "envasaréis": "envasar",
    "envasaría": "envasar",
    "envasaríais": "envasar",
    "envasaríamos": "envasar",
    "envasarían": "envasar",
    "envasarías": "envasar",
    "envasas": "envasar",
    "envasase": "envasar",
    "envasaseis": "envasar",
    "envasasen": "envasar",
    "envasases": "envasar",
    "envasaste": "envasar",
    "envasasteis": "envasar",
    "envase": "envasar",
    "envasemos": "envasar",
    "envasen": "envasar",
    "envases": "envase",
    "envaso": "envasar",
    "envasábamos": "envasar",
    "envasáis": "envasar",
    "envasáramos": "envasar",
    "envasáremos": "envasar",
    "envasásemos": "envasar",
    "envasé": "envasar",
    "envaséis": "envasar",
    "envasó": "envasar",
    "envedija": "envedijar",
    "envedijaba": "envedijar",
    "envedijabais": "envedijar",
    "envedijaban": "envedijar",
    "envedijabas": "envedijar",
    "envedijad": "envedijar",
    "envedijada": "envedijar",
    "envedijadas": "envedijar",
    "envedijado": "envedijar",
    "envedijados": "envedijar",
    "envedijamos": "envedijar",
    "envedijan": "envedijar",
    "envedijando": "envedijar",
    "envedijara": "envedijar",
    "envedijarais": "envedijar",
    "envedijaran": "envedijar",
    "envedijaras": "envedijar",
    "envedijare": "envedijar",
    "envedijareis": "envedijar",
    "envedijaremos": "envedijar",
    "envedijaren": "envedijar",
    "envedijares": "envedijar",
    "envedijaron": "envedijar",
    "envedijará": "envedijar",
    "envedijarán": "envedijar",
    "envedijarás": "envedijar",
    "envedijaré": "envedijar",
    "envedijaréis": "envedijar",
    "envedijaría": "envedijar",
    "envedijaríais": "envedijar",
    "envedijaríamos": "envedijar",
    "envedijarían": "envedijar",
    "envedijarías": "envedijar",
    "envedijas": "envedijar",
    "envedijase": "envedijar",
    "envedijaseis": "envedijar",
    "envedijasen": "envedijar",
    "envedijases": "envedijar",
    "envedijaste": "envedijar",
    "envedijasteis": "envedijar",
    "envedije": "envedijar",
    "envedijemos": "envedijar",
    "envedijen": "envedijar",
    "envedijes": "envedijar",
    "envedijo": "envedijar",
    "envedijábamos": "envedijar",
    "envedijáis": "envedijar",
    "envedijáramos": "envedijar",
    "envedijáremos": "envedijar",
    "envedijásemos": "envedijar",
    "envedijé": "envedijar",
    "envedijéis": "envedijar",
    "envedijó": "envedijar",
    "envejece": "envejecer",
    "envejeced": "envejecer",
    "envejecemos": "envejecer",
    "envejecen": "envejecer",
    "envejeceremos": "envejecer",
    "envejecerá": "envejecer",
    "envejecerán": "envejecer",
    "envejecerás": "envejecer",
    "envejeceré": "envejecer",
    "envejeceréis": "envejecer",
    "envejecería": "envejecer",
    "envejeceríais": "envejecer",
    "envejeceríamos": "envejecer",
    "envejecerían": "envejecer",
    "envejecerías": "envejecer",
    "envejeces": "envejecer",
    "envejecida": "envejecer",
    "envejecidas": "envejecer",
    "envejecido": "envejecer",
    "envejecidos": "envejecer",
    "envejeciendo": "envejecer",
    "envejeciera": "envejecer",
    "envejecierais": "envejecer",
    "envejecieran": "envejecer",
    "envejecieras": "envejecer",
    "envejeciere": "envejecer",
    "envejeciereis": "envejecer",
    "envejecieren": "envejecer",
    "envejecieres": "envejecer",
    "envejecieron": "envejecer",
    "envejeciese": "envejecer",
    "envejecieseis": "envejecer",
    "envejeciesen": "envejecer",
    "envejecieses": "envejecer",
    "envejecimientos": "envejecimiento",
    "envejecimos": "envejecer",
    "envejeciste": "envejecer",
    "envejecisteis": "envejecer",
    "envejeciéramos": "envejecer",
    "envejeciéremos": "envejecer",
    "envejeciésemos": "envejecer",
    "envejeció": "envejecer",
    "envejecéis": "envejecer",
    "envejecí": "envejecer",
    "envejecía": "envejecer",
    "envejecíais": "envejecer",
    "envejecíamos": "envejecer",
    "envejecían": "envejecer",
    "envejecías": "envejecer",
    "envejezca": "envejecer",
    "envejezcamos": "envejecer",
    "envejezcan": "envejecer",
    "envejezcas": "envejecer",
    "envejezco": "envejecer",
    "envejezcáis": "envejecer",
    "envelopes": "envelope",
    "envenena": "envenenar",
    "envenenaba": "envenenar",
    "envenenabais": "envenenar",
    "envenenaban": "envenenar",
    "envenenabas": "envenenar",
    "envenenad": "envenenar",
    "envenenada": "envenenar",
    "envenenadas": "envenenar",
    "envenenado": "envenenar",
    "envenenadora": "envenenador",
    "envenenadoras": "envenenador",
    "envenenadores": "envenenador",
    "envenenados": "envenenar",
    "envenenamientos": "envenenamiento",
    "envenenamos": "envenenar",
    "envenenan": "envenenar",
    "envenenando": "envenenar",
    "envenenara": "envenenar",
    "envenenarais": "envenenar",
    "envenenaran": "envenenar",
    "envenenaras": "envenenar",
    "envenenare": "envenenar",
    "envenenareis": "envenenar",
    "envenenaremos": "envenenar",
    "envenenaren": "envenenar",
    "envenenares": "envenenar",
    "envenenaron": "envenenar",
    "envenenará": "envenenar",
    "envenenarán": "envenenar",
    "envenenarás": "envenenar",
    "envenenaré": "envenenar",
    "envenenaréis": "envenenar",
    "envenenaría": "envenenar",
    "envenenaríais": "envenenar",
    "envenenaríamos": "envenenar",
    "envenenarían": "envenenar",
    "envenenarías": "envenenar",
    "envenenas": "envenenar",
    "envenenase": "envenenar",
    "envenenaseis": "envenenar",
    "envenenasen": "envenenar",
    "envenenases": "envenenar",
    "envenenaste": "envenenar",
    "envenenasteis": "envenenar",
    "envenene": "envenenar",
    "envenenemos": "envenenar",
    "envenenen": "envenenar",
    "envenenes": "envenenar",
    "enveneno": "envenenar",
    "envenenábamos": "envenenar",
    "envenenáis": "envenenar",
    "envenenáramos": "envenenar",
    "envenenáremos": "envenenar",
    "envenenásemos": "envenenar",
    "envenené": "envenenar",
    "envenenéis": "envenenar",
    "envenenó": "envenenar",
    "envera": "enverar",
    "enveraba": "enverar",
    "enverabais": "enverar",
    "enveraban": "enverar",
    "enverabas": "enverar",
    "enverad": "enverar",
    "enverada": "enverar",
    "enveradas": "enverar",
    "enverado": "enverar",
    "enverados": "enverar",
    "enveramos": "enverar",
    "enveran": "enverar",
    "enverando": "enverar",
    "enverara": "enverar",
    "enverarais": "enverar",
    "enveraran": "enverar",
    "enveraras": "enverar",
    "enverare": "enverar",
    "enverareis": "enverar",
    "enveraremos": "enverar",
    "enveraren": "enverar",
    "enverares": "enverar",
    "enveraron": "enverar",
    "enverará": "enverar",
    "enverarán": "enverar",
    "enverarás": "enverar",
    "enveraré": "enverar",
    "enveraréis": "enverar",
    "enveraría": "enverar",
    "enveraríais": "enverar",
    "enveraríamos": "enverar",
    "enverarían": "enverar",
    "enverarías": "enverar",
    "enveras": "enverar",
    "enverase": "enverar",
    "enveraseis": "enverar",
    "enverasen": "enverar",
    "enverases": "enverar",
    "enveraste": "enverar",
    "enverasteis": "enverar",
    "enverdece": "enverdecer",
    "enverdeced": "enverdecer",
    "enverdecemos": "enverdecer",
    "enverdecen": "enverdecer",
    "enverdeceremos": "enverdecer",
    "enverdecerá": "enverdecer",
    "enverdecerán": "enverdecer",
    "enverdecerás": "enverdecer",
    "enverdeceré": "enverdecer",
    "enverdeceréis": "enverdecer",
    "enverdecería": "enverdecer",
    "enverdeceríais": "enverdecer",
    "enverdeceríamos": "enverdecer",
    "enverdecerían": "enverdecer",
    "enverdecerías": "enverdecer",
    "enverdeces": "enverdecer",
    "enverdecida": "enverdecer",
    "enverdecidas": "enverdecer",
    "enverdecido": "enverdecer",
    "enverdecidos": "enverdecer",
    "enverdeciendo": "enverdecer",
    "enverdeciera": "enverdecer",
    "enverdecierais": "enverdecer",
    "enverdecieran": "enverdecer",
    "enverdecieras": "enverdecer",
    "enverdeciere": "enverdecer",
    "enverdeciereis": "enverdecer",
    "enverdecieren": "enverdecer",
    "enverdecieres": "enverdecer",
    "enverdecieron": "enverdecer",
    "enverdeciese": "enverdecer",
    "enverdecieseis": "enverdecer",
    "enverdeciesen": "enverdecer",
    "enverdecieses": "enverdecer",
    "enverdecimientos": "enverdecimiento",
    "enverdecimos": "enverdecer",
    "enverdeciste": "enverdecer",
    "enverdecisteis": "enverdecer",
    "enverdeciéramos": "enverdecer",
    "enverdeciéremos": "enverdecer",
    "enverdeciésemos": "enverdecer",
    "enverdeció": "enverdecer",
    "enverdecéis": "enverdecer",
    "enverdecí": "enverdecer",
    "enverdecía": "enverdecer",
    "enverdecíais": "enverdecer",
    "enverdecíamos": "enverdecer",
    "enverdecían": "enverdecer",
    "enverdecías": "enverdecer",
    "enverdezca": "enverdecer",
    "enverdezcamos": "enverdecer",
    "enverdezcan": "enverdecer",
    "enverdezcas": "enverdecer",
    "enverdezco": "enverdecer",
    "enverdezcáis": "enverdecer",
    "envere": "enverar",
    "enveremos": "enverar",
    "enveren": "enverar",
    "enveres": "enverar",
    "enverga": "envergar",
    "envergaba": "envergar",
    "envergabais": "envergar",
    "envergaban": "envergar",
    "envergabas": "envergar",
    "envergad": "envergar",
    "envergada": "envergar",
    "envergadas": "envergar",
    "envergado": "envergar",
    "envergados": "envergar",
    "envergaduras": "envergadura",
    "envergamos": "envergar",
    "envergan": "envergar",
    "envergando": "envergar",
    "envergara": "envergar",
    "envergarais": "envergar",
    "envergaran": "envergar",
    "envergaras": "envergar",
    "envergare": "envergar",
    "envergareis": "envergar",
    "envergaremos": "envergar",
    "envergaren": "envergar",
    "envergares": "envergar",
    "envergaron": "envergar",
    "envergará": "envergar",
    "envergarán": "envergar",
    "envergarás": "envergar",
    "envergaré": "envergar",
    "envergaréis": "envergar",
    "envergaría": "envergar",
    "envergaríais": "envergar",
    "envergaríamos": "envergar",
    "envergarían": "envergar",
    "envergarías": "envergar",
    "envergas": "envergar",
    "envergase": "envergar",
    "envergaseis": "envergar",
    "envergasen": "envergar",
    "envergases": "envergar",
    "envergaste": "envergar",
    "envergasteis": "envergar",
    "envergo": "envergar",
    "envergue": "envergar",
    "enverguemos": "envergar",
    "enverguen": "envergar",
    "envergues": "envergue",
    "envergué": "envergar",
    "enverguéis": "envergar",
    "envergábamos": "envergar",
    "envergáis": "envergar",
    "envergáramos": "envergar",
    "envergáremos": "envergar",
    "envergásemos": "envergar",
    "envergó": "envergar",
    "enverjados": "enverjado",
    "envero": "enverar",
    "enveros": "envero",
    "enverábamos": "enverar",
    "enveráis": "enverar",
    "enveráramos": "enverar",
    "enveráremos": "enverar",
    "enverásemos": "enverar",
    "enveré": "enverar",
    "enveréis": "enverar",
    "enveró": "enverar",
    "envesada": "envesado",
    "envesadas": "envesado",
    "envesados": "envesado",
    "enveses": "envés",
    "enviaba": "enviar",
    "enviabais": "enviar",
    "enviaban": "enviar",
    "enviabas": "enviar",
    "enviad": "enviar",
    "enviada": "enviar",
    "enviadas": "enviar",
    "enviado": "enviar",
    "enviados": "enviar",
    "enviajada": "enviajado",
    "enviajadas": "enviajado",
    "enviajados": "enviajado",
    "enviamos": "enviar",
    "enviando": "enviar",
    "enviara": "enviar",
    "enviarais": "enviar",
    "enviaran": "enviar",
    "enviaras": "enviar",
    "enviare": "enviar",
    "enviareis": "enviar",
    "enviaremos": "enviar",
    "enviaren": "enviar",
    "enviares": "enviar",
    "enviaron": "enviar",
    "enviará": "enviar",
    "enviarán": "enviar",
    "enviarás": "enviar",
    "enviaré": "enviar",
    "enviaréis": "enviar",
    "enviaría": "enviar",
    "enviaríais": "enviar",
    "enviaríamos": "enviar",
    "enviarían": "enviar",
    "enviarías": "enviar",
    "enviase": "enviar",
    "enviaseis": "enviar",
    "enviasen": "enviar",
    "enviases": "enviar",
    "enviaste": "enviar",
    "enviasteis": "enviar",
    "envicia": "enviciar",
    "enviciaba": "enviciar",
    "enviciabais": "enviciar",
    "enviciaban": "enviciar",
    "enviciabas": "enviciar",
    "enviciad": "enviciar",
    "enviciada": "enviciar",
    "enviciadas": "enviciar",
    "enviciado": "enviciar",
    "enviciados": "enviciar",
    "enviciamientos": "enviciamiento",
    "enviciamos": "enviciar",
    "envician": "enviciar",
    "enviciando": "enviciar",
    "enviciara": "enviciar",
    "enviciarais": "enviciar",
    "enviciaran": "enviciar",
    "enviciaras": "enviciar",
    "enviciare": "enviciar",
    "enviciareis": "enviciar",
    "enviciaremos": "enviciar",
    "enviciaren": "enviciar",
    "enviciares": "enviciar",
    "enviciaron": "enviciar",
    "enviciará": "enviciar",
    "enviciarán": "enviciar",
    "enviciarás": "enviciar",
    "enviciaré": "enviciar",
    "enviciaréis": "enviciar",
    "enviciaría": "enviciar",
    "enviciaríais": "enviciar",
    "enviciaríamos": "enviciar",
    "enviciarían": "enviciar",
    "enviciarías": "enviciar",
    "envicias": "enviciar",
    "enviciase": "enviciar",
    "enviciaseis": "enviciar",
    "enviciasen": "enviciar",
    "enviciases": "enviciar",
    "enviciaste": "enviciar",
    "enviciasteis": "enviciar",
    "envicie": "enviciar",
    "enviciemos": "enviciar",
    "envicien": "enviciar",
    "envicies": "enviciar",
    "envicio": "enviciar",
    "enviciábamos": "enviciar",
    "enviciáis": "enviciar",
    "enviciáramos": "enviciar",
    "enviciáremos": "enviciar",
    "enviciásemos": "enviciar",
    "envicié": "enviciar",
    "enviciéis": "enviciar",
    "envició": "enviciar",
    "envida": "envidar",
    "envidaba": "envidar",
    "envidabais": "envidar",
    "envidaban": "envidar",
    "envidabas": "envidar",
    "envidad": "envidar",
    "envidada": "envidar",
    "envidadas": "envidar",
    "envidado": "envidar",
    "envidadora": "envidador",
    "envidadoras": "envidador",
    "envidadores": "envidador",
    "envidados": "envidar",
    "envidamos": "envidar",
    "envidan": "envidar",
    "envidando": "envidar",
    "envidara": "envidar",
    "envidarais": "envidar",
    "envidaran": "envidar",
    "envidaras": "envidar",
    "envidare": "envidar",
    "envidareis": "envidar",
    "envidaremos": "envidar",
    "envidaren": "envidar",
    "envidares": "envidar",
    "envidaron": "envidar",
    "envidará": "envidar",
    "envidarán": "envidar",
    "envidarás": "envidar",
    "envidaré": "envidar",
    "envidaréis": "envidar",
    "envidaría": "envidar",
    "envidaríais": "envidar",
    "envidaríamos": "envidar",
    "envidarían": "envidar",
    "envidarías": "envidar",
    "envidas": "envidar",
    "envidase": "envidar",
    "envidaseis": "envidar",
    "envidasen": "envidar",
    "envidases": "envidar",
    "envidaste": "envidar",
    "envidasteis": "envidar",
    "envide": "envidar",
    "envidemos": "envidar",
    "enviden": "envidar",
    "envides": "envidar",
    "envidia": "envidiar",
    "envidiaba": "envidiar",
    "envidiabais": "envidiar",
    "envidiaban": "envidiar",
    "envidiabas": "envidiar",
    "envidiables": "envidiable",
    "envidiad": "envidiar",
    "envidiada": "envidiar",
    "envidiadas": "envidiar",
    "envidiado": "envidiar",
    "envidiados": "envidiar",
    "envidiamos": "envidiar",
    "envidian": "envidiar",
    "envidiando": "envidiar",
    "envidiara": "envidiar",
    "envidiarais": "envidiar",
    "envidiaran": "envidiar",
    "envidiaras": "envidiar",
    "envidiare": "envidiar",
    "envidiareis": "envidiar",
    "envidiaremos": "envidiar",
    "envidiaren": "envidiar",
    "envidiares": "envidiar",
    "envidiaron": "envidiar",
    "envidiará": "envidiar",
    "envidiarán": "envidiar",
    "envidiarás": "envidiar",
    "envidiaré": "envidiar",
    "envidiaréis": "envidiar",
    "envidiaría": "envidiar",
    "envidiaríais": "envidiar",
    "envidiaríamos": "envidiar",
    "envidiarían": "envidiar",
    "envidiarías": "envidiar",
    "envidias": "envidiar",
    "envidiase": "envidiar",
    "envidiaseis": "envidiar",
    "envidiasen": "envidiar",
    "envidiases": "envidiar",
    "envidiaste": "envidiar",
    "envidiasteis": "envidiar",
    "envidie": "envidiar",
    "envidiemos": "envidiar",
    "envidien": "envidiar",
    "envidies": "envidiar",
    "envidio": "envidiar",
    "envidiosa": "envidioso",
    "envidiosas": "envidioso",
    "envidiosos": "envidioso",
    "envidiábamos": "envidiar",
    "envidiáis": "envidiar",
    "envidiáramos": "envidiar",
    "envidiáremos": "envidiar",
    "envidiásemos": "envidiar",
    "envidié": "envidiar",
    "envidiéis": "envidiar",
    "envidió": "envidiar",
    "envido": "envidar",
    "envidábamos": "envidar",
    "envidáis": "envidar",
    "envidáramos": "envidar",
    "envidáremos": "envidar",
    "envidásemos": "envidar",
    "envidé": "envidar",
    "envidéis": "envidar",
    "envidó": "envidar",
    "enviemos": "enviar",
    "envigados": "envigado",
    "envilece": "envilecer",
    "envileced": "envilecer",
    "envilecedora": "envilecedor",
    "envilecedoras": "envilecedor",
    "envilecedores": "envilecedor",
    "envilecemos": "envilecer",
    "envilecen": "envilecer",
    "envileceremos": "envilecer",
    "envilecerá": "envilecer",
    "envilecerán": "envilecer",
    "envilecerás": "envilecer",
    "envileceré": "envilecer",
    "envileceréis": "envilecer",
    "envilecería": "envilecer",
    "envileceríais": "envilecer",
    "envileceríamos": "envilecer",
    "envilecerían": "envilecer",
    "envilecerías": "envilecer",
    "envileces": "envilecer",
    "envilecida": "envilecer",
    "envilecidas": "envilecer",
    "envilecido": "envilecer",
    "envilecidos": "envilecer",
    "envileciendo": "envilecer",
    "envileciera": "envilecer",
    "envilecierais": "envilecer",
    "envilecieran": "envilecer",
    "envilecieras": "envilecer",
    "envileciere": "envilecer",
    "envileciereis": "envilecer",
    "envilecieren": "envilecer",
    "envilecieres": "envilecer",
    "envilecieron": "envilecer",
    "envileciese": "envilecer",
    "envilecieseis": "envilecer",
    "envileciesen": "envilecer",
    "envilecieses": "envilecer",
    "envilecimientos": "envilecimiento",
    "envilecimos": "envilecer",
    "envileciste": "envilecer",
    "envilecisteis": "envilecer",
    "envileciéramos": "envilecer",
    "envileciéremos": "envilecer",
    "envileciésemos": "envilecer",
    "envileció": "envilecer",
    "envilecéis": "envilecer",
    "envilecí": "envilecer",
    "envilecía": "envilecer",
    "envilecíais": "envilecer",
    "envilecíamos": "envilecer",
    "envilecían": "envilecer",
    "envilecías": "envilecer",
    "envilezca": "envilecer",
    "envilezcamos": "envilecer",
    "envilezcan": "envilecer",
    "envilezcas": "envilecer",
    "envilezco": "envilecer",
    "envilezcáis": "envilecer",
    "envina": "envinar",
    "envinaba": "envinar",
    "envinabais": "envinar",
    "envinaban": "envinar",
    "envinabas": "envinar",
    "envinad": "envinar",
    "envinada": "envinar",
    "envinadas": "envinar",
    "envinado": "envinar",
    "envinados": "envinar",
    "envinagra": "envinagrar",
    "envinagraba": "envinagrar",
    "envinagrabais": "envinagrar",
    "envinagraban": "envinagrar",
    "envinagrabas": "envinagrar",
    "envinagrad": "envinagrar",
    "envinagrada": "envinagrar",
    "envinagradas": "envinagrar",
    "envinagrado": "envinagrar",
    "envinagrados": "envinagrar",
    "envinagramos": "envinagrar",
    "envinagran": "envinagrar",
    "envinagrando": "envinagrar",
    "envinagrara": "envinagrar",
    "envinagrarais": "envinagrar",
    "envinagraran": "envinagrar",
    "envinagraras": "envinagrar",
    "envinagrare": "envinagrar",
    "envinagrareis": "envinagrar",
    "envinagraremos": "envinagrar",
    "envinagraren": "envinagrar",
    "envinagrares": "envinagrar",
    "envinagraron": "envinagrar",
    "envinagrará": "envinagrar",
    "envinagrarán": "envinagrar",
    "envinagrarás": "envinagrar",
    "envinagraré": "envinagrar",
    "envinagraréis": "envinagrar",
    "envinagraría": "envinagrar",
    "envinagraríais": "envinagrar",
    "envinagraríamos": "envinagrar",
    "envinagrarían": "envinagrar",
    "envinagrarías": "envinagrar",
    "envinagras": "envinagrar",
    "envinagrase": "envinagrar",
    "envinagraseis": "envinagrar",
    "envinagrasen": "envinagrar",
    "envinagrases": "envinagrar",
    "envinagraste": "envinagrar",
    "envinagrasteis": "envinagrar",
    "envinagre": "envinagrar",
    "envinagremos": "envinagrar",
    "envinagren": "envinagrar",
    "envinagres": "envinagrar",
    "envinagro": "envinagrar",
    "envinagrábamos": "envinagrar",
    "envinagráis": "envinagrar",
    "envinagráramos": "envinagrar",
    "envinagráremos": "envinagrar",
    "envinagrásemos": "envinagrar",
    "envinagré": "envinagrar",
    "envinagréis": "envinagrar",
    "envinagró": "envinagrar",
    "envinamos": "envinar",
    "envinan": "envinar",
    "envinando": "envinar",
    "envinara": "envinar",
    "envinarais": "envinar",
    "envinaran": "envinar",
    "envinaras": "envinar",
    "envinare": "envinar",
    "envinareis": "envinar",
    "envinaremos": "envinar",
    "envinaren": "envinar",
    "envinares": "envinar",
    "envinaron": "envinar",
    "envinará": "envinar",
    "envinarán": "envinar",
    "envinarás": "envinar",
    "envinaré": "envinar",
    "envinaréis": "envinar",
    "envinaría": "envinar",
    "envinaríais": "envinar",
    "envinaríamos": "envinar",
    "envinarían": "envinar",
    "envinarías": "envinar",
    "envinas": "envinar",
    "envinase": "envinar",
    "envinaseis": "envinar",
    "envinasen": "envinar",
    "envinases": "envinar",
    "envinaste": "envinar",
    "envinasteis": "envinar",
    "envine": "envinar",
    "envinemos": "envinar",
    "envinen": "envinar",
    "envines": "envinar",
    "envino": "envinar",
    "envinábamos": "envinar",
    "envináis": "envinar",
    "envináramos": "envinar",
    "envináremos": "envinar",
    "envinásemos": "envinar",
    "enviné": "envinar",
    "envinéis": "envinar",
    "envinó": "envinar",
    "enviones": "envión",
    "envisca": "enviscar",
    "enviscaba": "enviscar",
    "enviscabais": "enviscar",
    "enviscaban": "enviscar",
    "enviscabas": "enviscar",
    "enviscad": "enviscar",
    "enviscada": "enviscar",
    "enviscadas": "enviscar",
    "enviscado": "enviscar",
    "enviscados": "enviscar",
    "enviscamos": "enviscar",
    "enviscan": "enviscar",
    "enviscando": "enviscar",
    "enviscara": "enviscar",
    "enviscarais": "enviscar",
    "enviscaran": "enviscar",
    "enviscaras": "enviscar",
    "enviscare": "enviscar",
    "enviscareis": "enviscar",
    "enviscaremos": "enviscar",
    "enviscaren": "enviscar",
    "enviscares": "enviscar",
    "enviscaron": "enviscar",
    "enviscará": "enviscar",
    "enviscarán": "enviscar",
    "enviscarás": "enviscar",
    "enviscaré": "enviscar",
    "enviscaréis": "enviscar",
    "enviscaría": "enviscar",
    "enviscaríais": "enviscar",
    "enviscaríamos": "enviscar",
    "enviscarían": "enviscar",
    "enviscarías": "enviscar",
    "enviscas": "enviscar",
    "enviscase": "enviscar",
    "enviscaseis": "enviscar",
    "enviscasen": "enviscar",
    "enviscases": "enviscar",
    "enviscaste": "enviscar",
    "enviscasteis": "enviscar",
    "envisco": "enviscar",
    "enviscábamos": "enviscar",
    "enviscáis": "enviscar",
    "enviscáramos": "enviscar",
    "enviscáremos": "enviscar",
    "enviscásemos": "enviscar",
    "enviscó": "enviscar",
    "envisque": "enviscar",
    "envisquemos": "enviscar",
    "envisquen": "enviscar",
    "envisques": "enviscar",
    "envisqué": "enviscar",
    "envisquéis": "enviscar",
    "envites": "envite",
    "enviuda": "enviudar",
    "enviudaba": "enviudar",
    "enviudabais": "enviudar",
    "enviudaban": "enviudar",
    "enviudabas": "enviudar",
    "enviudad": "enviudar",
    "enviudada": "enviudar",
    "enviudadas": "enviudar",
    "enviudado": "enviudar",
    "enviudados": "enviudar",
    "enviudamientos": "enviudamiento",
    "enviudamos": "enviudar",
    "enviudan": "enviudar",
    "enviudando": "enviudar",
    "enviudara": "enviudar",
    "enviudarais": "enviudar",
    "enviudaran": "enviudar",
    "enviudaras": "enviudar",
    "enviudare": "enviudar",
    "enviudareis": "enviudar",
    "enviudaremos": "enviudar",
    "enviudaren": "enviudar",
    "enviudares": "enviudar",
    "enviudaron": "enviudar",
    "enviudará": "enviudar",
    "enviudarán": "enviudar",
    "enviudarás": "enviudar",
    "enviudaré": "enviudar",
    "enviudaréis": "enviudar",
    "enviudaría": "enviudar",
    "enviudaríais": "enviudar",
    "enviudaríamos": "enviudar",
    "enviudarían": "enviudar",
    "enviudarías": "enviudar",
    "enviudas": "enviudar",
    "enviudase": "enviudar",
    "enviudaseis": "enviudar",
    "enviudasen": "enviudar",
    "enviudases": "enviudar",
    "enviudaste": "enviudar",
    "enviudasteis": "enviudar",
    "enviude": "enviudar",
    "enviudemos": "enviudar",
    "enviuden": "enviudar",
    "enviudes": "enviudar",
    "enviudo": "enviudar",
    "enviudábamos": "enviudar",
    "enviudáis": "enviudar",
    "enviudáramos": "enviudar",
    "enviudáremos": "enviudar",
    "enviudásemos": "enviudar",
    "enviudé": "enviudar",
    "enviudéis": "enviudar",
    "enviudó": "enviudar",
    "enviábamos": "enviar",
    "enviáis": "enviar",
    "enviáramos": "enviar",
    "enviáremos": "enviar",
    "enviásemos": "enviar",
    "envié": "enviar",
    "enviéis": "enviar",
    "envió": "enviar",
    "envoltijos": "envoltijo",
    "envoltorios": "envoltorio",
    "envolturas": "envoltura",
    "envolvamos": "envolver",
    "envolved": "envolver",
    "envolvederos": "envolvedero",
    "envolvedores": "envolvedor",
    "envolvemos": "envolver",
    "envolventes": "envolvente",
    "envolveremos": "envolver",
    "envolverá": "envolver",
    "envolverán": "envolver",
    "envolverás": "envolver",
    "envolveré": "envolver",
    "envolveréis": "envolver",
    "envolvería": "envolver",
    "envolveríais": "envolver",
    "envolveríamos": "envolver",
    "envolverían": "envolver",
    "envolverías": "envolver",
    "envolviendo": "envolver",
    "envolviera": "envolver",
    "envolvierais": "envolver",
    "envolvieran": "envolver",
    "envolvieras": "envolver",
    "envolviere": "envolver",
    "envolviereis": "envolver",
    "envolvieren": "envolver",
    "envolvieres": "envolver",
    "envolvieron": "envolver",
    "envolviese": "envolver",
    "envolvieseis": "envolver",
    "envolviesen": "envolver",
    "envolvieses": "envolver",
    "envolvimientos": "envolvimiento",
    "envolvimos": "envolver",
    "envolviste": "envolver",
    "envolvisteis": "envolver",
    "envolviéramos": "envolver",
    "envolviéremos": "envolver",
    "envolviésemos": "envolver",
    "envolvió": "envolver",
    "envolváis": "envolver",
    "envolvéis": "envolver",
    "envolví": "envolver",
    "envolvía": "envolver",
    "envolvíais": "envolver",
    "envolvíamos": "envolver",
    "envolvían": "envolver",
    "envolvías": "envolver",
    "envuelta": "envolver",
    "envueltas": "envolver",
    "envuelto": "envolver",
    "envueltos": "envolver",
    "envuelva": "envolver",
    "envuelvan": "envolver",
    "envuelvas": "envolver",
    "envuelve": "envolver",
    "envuelven": "envolver",
    "envuelves": "envolver",
    "envuelvo": "envolver",
    "envía": "enviar",
    "envían": "enviar",
    "envías": "enviar",
    "envíe": "enviar",
    "envíen": "enviar",
    "envíes": "enviar",
    "envío": "enviar",
    "envíos": "envío",
    "enyerba": "enyerbar",
    "enyerbaba": "enyerbar",
    "enyerbabais": "enyerbar",
    "enyerbaban": "enyerbar",
    "enyerbabas": "enyerbar",
    "enyerbad": "enyerbar",
    "enyerbada": "enyerbar",
    "enyerbadas": "enyerbar",
    "enyerbado": "enyerbar",
    "enyerbados": "enyerbar",
    "enyerbamos": "enyerbar",
    "enyerban": "enyerbar",
    "enyerbando": "enyerbar",
    "enyerbara": "enyerbar",
    "enyerbarais": "enyerbar",
    "enyerbaran": "enyerbar",
    "enyerbaras": "enyerbar",
    "enyerbare": "enyerbar",
    "enyerbareis": "enyerbar",
    "enyerbaremos": "enyerbar",
    "enyerbaren": "enyerbar",
    "enyerbares": "enyerbar",
    "enyerbaron": "enyerbar",
    "enyerbará": "enyerbar",
    "enyerbarán": "enyerbar",
    "enyerbarás": "enyerbar",
    "enyerbaré": "enyerbar",
    "enyerbaréis": "enyerbar",
    "enyerbaría": "enyerbar",
    "enyerbaríais": "enyerbar",
    "enyerbaríamos": "enyerbar",
    "enyerbarían": "enyerbar",
    "enyerbarías": "enyerbar",
    "enyerbas": "enyerbar",
    "enyerbase": "enyerbar",
    "enyerbaseis": "enyerbar",
    "enyerbasen": "enyerbar",
    "enyerbases": "enyerbar",
    "enyerbaste": "enyerbar",
    "enyerbasteis": "enyerbar",
    "enyerbe": "enyerbar",
    "enyerbemos": "enyerbar",
    "enyerben": "enyerbar",
    "enyerbes": "enyerbar",
    "enyerbo": "enyerbar",
    "enyerbábamos": "enyerbar",
    "enyerbáis": "enyerbar",
    "enyerbáramos": "enyerbar",
    "enyerbáremos": "enyerbar",
    "enyerbásemos": "enyerbar",
    "enyerbé": "enyerbar",
    "enyerbéis": "enyerbar",
    "enyerbó": "enyerbar",
    "enyesa": "enyesar",
    "enyesaba": "enyesar",
    "enyesabais": "enyesar",
    "enyesaban": "enyesar",
    "enyesabas": "enyesar",
    "enyesad": "enyesar",
    "enyesada": "enyesar",
    "enyesadas": "enyesar",
    "enyesado": "enyesar",
    "enyesados": "enyesar",
    "enyesaduras": "enyesadura",
    "enyesamos": "enyesar",
    "enyesan": "enyesar",
    "enyesando": "enyesar",
    "enyesara": "enyesar",
    "enyesarais": "enyesar",
    "enyesaran": "enyesar",
    "enyesaras": "enyesar",
    "enyesare": "enyesar",
    "enyesareis": "enyesar",
    "enyesaremos": "enyesar",
    "enyesaren": "enyesar",
    "enyesares": "enyesar",
    "enyesaron": "enyesar",
    "enyesará": "enyesar",
    "enyesarán": "enyesar",
    "enyesarás": "enyesar",
    "enyesaré": "enyesar",
    "enyesaréis": "enyesar",
    "enyesaría": "enyesar",
    "enyesaríais": "enyesar",
    "enyesaríamos": "enyesar",
    "enyesarían": "enyesar",
    "enyesarías": "enyesar",
    "enyesas": "enyesar",
    "enyesase": "enyesar",
    "enyesaseis": "enyesar",
    "enyesasen": "enyesar",
    "enyesases": "enyesar",
    "enyesaste": "enyesar",
    "enyesasteis": "enyesar",
    "enyese": "enyesar",
    "enyesemos": "enyesar",
    "enyesen": "enyesar",
    "enyeses": "enyesar",
    "enyeso": "enyesar",
    "enyesábamos": "enyesar",
    "enyesáis": "enyesar",
    "enyesáramos": "enyesar",
    "enyesáremos": "enyesar",
    "enyesásemos": "enyesar",
    "enyesé": "enyesar",
    "enyeséis": "enyesar",
    "enyesó": "enyesar",
    "enyuga": "enyugar",
    "enyugaba": "enyugar",
    "enyugabais": "enyugar",
    "enyugaban": "enyugar",
    "enyugabas": "enyugar",
    "enyugad": "enyugar",
    "enyugada": "enyugar",
    "enyugadas": "enyugar",
    "enyugado": "enyugar",
    "enyugados": "enyugar",
    "enyugamos": "enyugar",
    "enyugan": "enyugar",
    "enyugando": "enyugar",
    "enyugara": "enyugar",
    "enyugarais": "enyugar",
    "enyugaran": "enyugar",
    "enyugaras": "enyugar",
    "enyugare": "enyugar",
    "enyugareis": "enyugar",
    "enyugaremos": "enyugar",
    "enyugaren": "enyugar",
    "enyugares": "enyugar",
    "enyugaron": "enyugar",
    "enyugará": "enyugar",
    "enyugarán": "enyugar",
    "enyugarás": "enyugar",
    "enyugaré": "enyugar",
    "enyugaréis": "enyugar",
    "enyugaría": "enyugar",
    "enyugaríais": "enyugar",
    "enyugaríamos": "enyugar",
    "enyugarían": "enyugar",
    "enyugarías": "enyugar",
    "enyugas": "enyugar",
    "enyugase": "enyugar",
    "enyugaseis": "enyugar",
    "enyugasen": "enyugar",
    "enyugases": "enyugar",
    "enyugaste": "enyugar",
    "enyugasteis": "enyugar",
    "enyugo": "enyugar",
    "enyugue": "enyugar",
    "enyuguemos": "enyugar",
    "enyuguen": "enyugar",
    "enyugues": "enyugar",
    "enyugué": "enyugar",
    "enyuguéis": "enyugar",
    "enyugábamos": "enyugar",
    "enyugáis": "enyugar",
    "enyugáramos": "enyugar",
    "enyugáremos": "enyugar",
    "enyugásemos": "enyugar",
    "enyugó": "enyugar",
    "enzainaba": "enzainar",
    "enzainabais": "enzainar",
    "enzainaban": "enzainar",
    "enzainabas": "enzainar",
    "enzainad": "enzainar",
    "enzainada": "enzainar",
    "enzainadas": "enzainar",
    "enzainado": "enzainar",
    "enzainados": "enzainar",
    "enzainamos": "enzainar",
    "enzainando": "enzainar",
    "enzainara": "enzainar",
    "enzainarais": "enzainar",
    "enzainaran": "enzainar",
    "enzainaras": "enzainar",
    "enzainare": "enzainar",
    "enzainareis": "enzainar",
    "enzainaremos": "enzainar",
    "enzainaren": "enzainar",
    "enzainares": "enzainar",
    "enzainaron": "enzainar",
    "enzainará": "enzainar",
    "enzainarán": "enzainar",
    "enzainarás": "enzainar",
    "enzainaré": "enzainar",
    "enzainaréis": "enzainar",
    "enzainaría": "enzainar",
    "enzainaríais": "enzainar",
    "enzainaríamos": "enzainar",
    "enzainarían": "enzainar",
    "enzainarías": "enzainar",
    "enzainase": "enzainar",
    "enzainaseis": "enzainar",
    "enzainasen": "enzainar",
    "enzainases": "enzainar",
    "enzainaste": "enzainar",
    "enzainasteis": "enzainar",
    "enzainemos": "enzainar",
    "enzainábamos": "enzainar",
    "enzaináis": "enzainar",
    "enzaináramos": "enzainar",
    "enzaináremos": "enzainar",
    "enzainásemos": "enzainar",
    "enzainé": "enzainar",
    "enzainéis": "enzainar",
    "enzainó": "enzainar",
    "enzamarrada": "enzamarrado",
    "enzamarradas": "enzamarrado",
    "enzamarrados": "enzamarrado",
    "enzarce": "enzarzar",
    "enzarcemos": "enzarzar",
    "enzarcen": "enzarzar",
    "enzarces": "enzarzar",
    "enzarcé": "enzarzar",
    "enzarcéis": "enzarzar",
    "enzarza": "enzarzar",
    "enzarzaba": "enzarzar",
    "enzarzabais": "enzarzar",
    "enzarzaban": "enzarzar",
    "enzarzabas": "enzarzar",
    "enzarzad": "enzarzar",
    "enzarzada": "enzarzar",
    "enzarzadas": "enzarzar",
    "enzarzado": "enzarzar",
    "enzarzados": "enzarzar",
    "enzarzamientos": "enzarzamiento",
    "enzarzamos": "enzarzar",
    "enzarzan": "enzarzar",
    "enzarzando": "enzarzar",
    "enzarzara": "enzarzar",
    "enzarzarais": "enzarzar",
    "enzarzaran": "enzarzar",
    "enzarzaras": "enzarzar",
    "enzarzare": "enzarzar",
    "enzarzareis": "enzarzar",
    "enzarzaremos": "enzarzar",
    "enzarzaren": "enzarzar",
    "enzarzares": "enzarzar",
    "enzarzaron": "enzarzar",
    "enzarzará": "enzarzar",
    "enzarzarán": "enzarzar",
    "enzarzarás": "enzarzar",
    "enzarzaré": "enzarzar",
    "enzarzaréis": "enzarzar",
    "enzarzaría": "enzarzar",
    "enzarzaríais": "enzarzar",
    "enzarzaríamos": "enzarzar",
    "enzarzarían": "enzarzar",
    "enzarzarías": "enzarzar",
    "enzarzas": "enzarzar",
    "enzarzase": "enzarzar",
    "enzarzaseis": "enzarzar",
    "enzarzasen": "enzarzar",
    "enzarzases": "enzarzar",
    "enzarzaste": "enzarzar",
    "enzarzasteis": "enzarzar",
    "enzarzo": "enzarzar",
    "enzarzábamos": "enzarzar",
    "enzarzáis": "enzarzar",
    "enzarzáramos": "enzarzar",
    "enzarzáremos": "enzarzar",
    "enzarzásemos": "enzarzar",
    "enzarzó": "enzarzar",
    "enzaína": "enzainar",
    "enzaínan": "enzainar",
    "enzaínas": "enzainar",
    "enzaíne": "enzainar",
    "enzaínen": "enzainar",
    "enzaínes": "enzainar",
    "enzaíno": "enzainar",
    "enzimas": "enzima",
    "enzimologías": "enzimología",
    "enzimática": "enzimático",
    "enzimáticas": "enzimático",
    "enzimáticos": "enzimático",
    "enzootias": "enzootia",
    "enzuncha": "enzunchar",
    "enzunchaba": "enzunchar",
    "enzunchabais": "enzunchar",
    "enzunchaban": "enzunchar",
    "enzunchabas": "enzunchar",
    "enzunchad": "enzunchar",
    "enzunchada": "enzunchar",
    "enzunchadas": "enzunchar",
    "enzunchado": "enzunchar",
    "enzunchados": "enzunchar",
    "enzunchamos": "enzunchar",
    "enzunchan": "enzunchar",
    "enzunchando": "enzunchar",
    "enzunchara": "enzunchar",
    "enzuncharais": "enzunchar",
    "enzuncharan": "enzunchar",
    "enzuncharas": "enzunchar",
    "enzunchare": "enzunchar",
    "enzunchareis": "enzunchar",
    "enzuncharemos": "enzunchar",
    "enzuncharen": "enzunchar",
    "enzunchares": "enzunchar",
    "enzuncharon": "enzunchar",
    "enzunchará": "enzunchar",
    "enzuncharán": "enzunchar",
    "enzuncharás": "enzunchar",
    "enzuncharé": "enzunchar",
    "enzuncharéis": "enzunchar",
    "enzuncharía": "enzunchar",
    "enzuncharíais": "enzunchar",
    "enzuncharíamos": "enzunchar",
    "enzuncharían": "enzunchar",
    "enzuncharías": "enzunchar",
    "enzunchas": "enzunchar",
    "enzunchase": "enzunchar",
    "enzunchaseis": "enzunchar",
    "enzunchasen": "enzunchar",
    "enzunchases": "enzunchar",
    "enzunchaste": "enzunchar",
    "enzunchasteis": "enzunchar",
    "enzunche": "enzunchar",
    "enzunchemos": "enzunchar",
    "enzunchen": "enzunchar",
    "enzunches": "enzunchar",
    "enzuncho": "enzunchar",
    "enzunchábamos": "enzunchar",
    "enzuncháis": "enzunchar",
    "enzuncháramos": "enzunchar",
    "enzuncháremos": "enzunchar",
    "enzunchásemos": "enzunchar",
    "enzunché": "enzunchar",
    "enzunchéis": "enzunchar",
    "enzunchó": "enzunchar",
    "enzurdece": "enzurdecer",
    "enzurdeced": "enzurdecer",
    "enzurdecemos": "enzurdecer",
    "enzurdecen": "enzurdecer",
    "enzurdeceremos": "enzurdecer",
    "enzurdecerá": "enzurdecer",
    "enzurdecerán": "enzurdecer",
    "enzurdecerás": "enzurdecer",
    "enzurdeceré": "enzurdecer",
    "enzurdeceréis": "enzurdecer",
    "enzurdecería": "enzurdecer",
    "enzurdeceríais": "enzurdecer",
    "enzurdeceríamos": "enzurdecer",
    "enzurdecerían": "enzurdecer",
    "enzurdecerías": "enzurdecer",
    "enzurdeces": "enzurdecer",
    "enzurdecida": "enzurdecer",
    "enzurdecidas": "enzurdecer",
    "enzurdecido": "enzurdecer",
    "enzurdecidos": "enzurdecer",
    "enzurdeciendo": "enzurdecer",
    "enzurdeciera": "enzurdecer",
    "enzurdecierais": "enzurdecer",
    "enzurdecieran": "enzurdecer",
    "enzurdecieras": "enzurdecer",
    "enzurdeciere": "enzurdecer",
    "enzurdeciereis": "enzurdecer",
    "enzurdecieren": "enzurdecer",
    "enzurdecieres": "enzurdecer",
    "enzurdecieron": "enzurdecer",
    "enzurdeciese": "enzurdecer",
    "enzurdecieseis": "enzurdecer",
    "enzurdeciesen": "enzurdecer",
    "enzurdecieses": "enzurdecer",
    "enzurdecimos": "enzurdecer",
    "enzurdeciste": "enzurdecer",
    "enzurdecisteis": "enzurdecer",
    "enzurdeciéramos": "enzurdecer",
    "enzurdeciéremos": "enzurdecer",
    "enzurdeciésemos": "enzurdecer",
    "enzurdeció": "enzurdecer",
    "enzurdecéis": "enzurdecer",
    "enzurdecí": "enzurdecer",
    "enzurdecía": "enzurdecer",
    "enzurdecíais": "enzurdecer",
    "enzurdecíamos": "enzurdecer",
    "enzurdecían": "enzurdecer",
    "enzurdecías": "enzurdecer",
    "enzurdezca": "enzurdecer",
    "enzurdezcamos": "enzurdecer",
    "enzurdezcan": "enzurdecer",
    "enzurdezcas": "enzurdecer",
    "enzurdezco": "enzurdecer",
    "enzurdezcáis": "enzurdecer",
    "enzurice": "enzurizar",
    "enzuricemos": "enzurizar",
    "enzuricen": "enzurizar",
    "enzurices": "enzurizar",
    "enzuricé": "enzurizar",
    "enzuricéis": "enzurizar",
    "enzuriza": "enzurizar",
    "enzurizaba": "enzurizar",
    "enzurizabais": "enzurizar",
    "enzurizaban": "enzurizar",
    "enzurizabas": "enzurizar",
    "enzurizad": "enzurizar",
    "enzurizada": "enzurizar",
    "enzurizadas": "enzurizar",
    "enzurizado": "enzurizar",
    "enzurizados": "enzurizar",
    "enzurizamos": "enzurizar",
    "enzurizan": "enzurizar",
    "enzurizando": "enzurizar",
    "enzurizara": "enzurizar",
    "enzurizarais": "enzurizar",
    "enzurizaran": "enzurizar",
    "enzurizaras": "enzurizar",
    "enzurizare": "enzurizar",
    "enzurizareis": "enzurizar",
    "enzurizaremos": "enzurizar",
    "enzurizaren": "enzurizar",
    "enzurizares": "enzurizar",
    "enzurizaron": "enzurizar",
    "enzurizará": "enzurizar",
    "enzurizarán": "enzurizar",
    "enzurizarás": "enzurizar",
    "enzurizaré": "enzurizar",
    "enzurizaréis": "enzurizar",
    "enzurizaría": "enzurizar",
    "enzurizaríais": "enzurizar",
    "enzurizaríamos": "enzurizar",
    "enzurizarían": "enzurizar",
    "enzurizarías": "enzurizar",
    "enzurizas": "enzurizar",
    "enzurizase": "enzurizar",
    "enzurizaseis": "enzurizar",
    "enzurizasen": "enzurizar",
    "enzurizases": "enzurizar",
    "enzurizaste": "enzurizar",
    "enzurizasteis": "enzurizar",
    "enzurizo": "enzurizar",
    "enzurizábamos": "enzurizar",
    "enzurizáis": "enzurizar",
    "enzurizáramos": "enzurizar",
    "enzurizáremos": "enzurizar",
    "enzurizásemos": "enzurizar",
    "enzurizó": "enzurizar",
    "enzurrona": "enzurronar",
    "enzurronaba": "enzurronar",
    "enzurronabais": "enzurronar",
    "enzurronaban": "enzurronar",
    "enzurronabas": "enzurronar",
    "enzurronad": "enzurronar",
    "enzurronada": "enzurronar",
    "enzurronadas": "enzurronar",
    "enzurronado": "enzurronar",
    "enzurronados": "enzurronar",
    "enzurronamos": "enzurronar",
    "enzurronan": "enzurronar",
    "enzurronando": "enzurronar",
    "enzurronara": "enzurronar",
    "enzurronarais": "enzurronar",
    "enzurronaran": "enzurronar",
    "enzurronaras": "enzurronar",
    "enzurronare": "enzurronar",
    "enzurronareis": "enzurronar",
    "enzurronaremos": "enzurronar",
    "enzurronaren": "enzurronar",
    "enzurronares": "enzurronar",
    "enzurronaron": "enzurronar",
    "enzurronará": "enzurronar",
    "enzurronarán": "enzurronar",
    "enzurronarás": "enzurronar",
    "enzurronaré": "enzurronar",
    "enzurronaréis": "enzurronar",
    "enzurronaría": "enzurronar",
    "enzurronaríais": "enzurronar",
    "enzurronaríamos": "enzurronar",
    "enzurronarían": "enzurronar",
    "enzurronarías": "enzurronar",
    "enzurronas": "enzurronar",
    "enzurronase": "enzurronar",
    "enzurronaseis": "enzurronar",
    "enzurronasen": "enzurronar",
    "enzurronases": "enzurronar",
    "enzurronaste": "enzurronar",
    "enzurronasteis": "enzurronar",
    "enzurrone": "enzurronar",
    "enzurronemos": "enzurronar",
    "enzurronen": "enzurronar",
    "enzurrones": "enzurronar",
    "enzurrono": "enzurronar",
    "enzurronábamos": "enzurronar",
    "enzurronáis": "enzurronar",
    "enzurronáramos": "enzurronar",
    "enzurronáremos": "enzurronar",
    "enzurronásemos": "enzurronar",
    "enzurroné": "enzurronar",
    "enzurronéis": "enzurronar",
    "enzurronó": "enzurronar",
    "enálages": "enálage",
    "enérgica": "enérgico",
    "enérgicas": "enérgico",
    "enérgicos": "enérgico",
    "enésima": "enésimo",
    "enésimas": "enésimo",
    "enésimos": "enésimo",
    "enógrafa": "enógrafo",
    "enógrafas": "enógrafo",
    "enógrafos": "enógrafo",
    "enóloga": "enólogo",
    "enólogas": "enólogo",
    "enólogos": "enólogo",
    "enómetros": "enómetro",
    "eocenos": "eoceno",
    "eocámbricos": "eocámbrico",
    "eocénica": "eocénico",
    "eocénicas": "eocénico",
    "eocénicos": "eocénico",
    "eolia": "eolio",
    "eolias": "eolio",
    "eolios": "eolio",
    "eolitos": "eolito",
    "eolípilas": "eolípila",
    "eones": "eón",
    "eosinas": "eosina",
    "eosinofilias": "eosinofilia",
    "eosinopenias": "eosinopenia",
    "eosinófila": "eosinófilo",
    "eosinófilas": "eosinófilo",
    "eosinófilos": "eosinófilo",
    "epactas": "epacta",
    "epanáforas": "epanáfora",
    "epanástrofes": "epanástrofe",
    "eparcas": "eparca",
    "ependimales": "ependimal",
    "ependimaria": "ependimario",
    "ependimarias": "ependimario",
    "ependimarios": "ependimario",
    "epentética": "epentético",
    "epentéticas": "epentético",
    "epentéticos": "epentético",
    "eperlanos": "eperlano",
    "epibléfarones": "epibléfaron",
    "epicantos": "epicanto",
    "epicardios": "epicardio",
    "epicarpios": "epicarpio",
    "epicarpos": "epicarpo",
    "epicedios": "epicedio",
    "epicena": "epiceno",
    "epicenas": "epiceno",
    "epicenos": "epiceno",
    "epicentrales": "epicentral",
    "epicentros": "epicentro",
    "epiceyos": "epiceyo",
    "epicicloidales": "epicicloidal",
    "epicicloides": "epicicloide",
    "epiciclos": "epiciclo",
    "epiclorhidrinas": "epiclorhidrina",
    "epicraneales": "epicraneal",
    "epicráneos": "epicráneo",
    "epicureísmos": "epicureísmo",
    "epicéntrica": "epicéntrico",
    "epicéntricas": "epicéntrico",
    "epicéntricos": "epicéntrico",
    "epicíclica": "epicíclico",
    "epicíclicas": "epicíclico",
    "epicíclicos": "epicíclico",
    "epicóndilos": "epicóndilo",
    "epicúrea": "epicúreo",
    "epicúreas": "epicúreo",
    "epicúreos": "epicúreo",
    "epidemiales": "epidemial",
    "epidemias": "epidemia",
    "epidemicidades": "epidemicidad",
    "epidemiologías": "epidemiología",
    "epidemiológica": "epidemiológico",
    "epidemiológicas": "epidemiológico",
    "epidemiológicos": "epidemiológico",
    "epidemióloga": "epidemiólogo",
    "epidemiólogas": "epidemiólogo",
    "epidemiólogos": "epidemiólogo",
    "epidiascopios": "epidiascopio",
    "epididimaria": "epididimario",
    "epididimarias": "epididimario",
    "epididimarios": "epididimario",
    "epidiáscopos": "epidiáscopo",
    "epidotas": "epidota",
    "epidurales": "epidural",
    "epidémica": "epidémico",
    "epidémicas": "epidémico",
    "epidémicos": "epidémico",
    "epidérmica": "epidérmico",
    "epidérmicas": "epidérmico",
    "epidérmicos": "epidérmico",
    "epidídimos": "epidídimo",
    "epiesclerales": "epiescleral",
    "epifanías": "epifanía",
    "epifenómenos": "epifenómeno",
    "epifila": "epifilo",
    "epifilas": "epifilo",
    "epifilos": "epifilo",
    "epifisaria": "epifisario",
    "epifisarias": "epifisario",
    "epifisarios": "epifisario",
    "epifita": "epifito",
    "epifitas": "epifito",
    "epifitos": "epifito",
    "epifonemas": "epifonema",
    "epiforas": "epifora",
    "epigastrios": "epigastrio",
    "epigenias": "epigenia",
    "epigenética": "epigenético",
    "epigenéticas": "epigenético",
    "epigenéticos": "epigenético",
    "epiglótica": "epiglótico",
    "epiglóticas": "epiglótico",
    "epiglóticos": "epiglótico",
    "epigonismos": "epigonismo",
    "epigrafistas": "epigrafista",
    "epigrafías": "epigrafía",
    "epigramas": "epigrama",
    "epigramataria": "epigramatario",
    "epigramatarias": "epigramatario",
    "epigramatarios": "epigramatario",
    "epigramistas": "epigramista",
    "epigramática": "epigramático",
    "epigramáticas": "epigramático",
    "epigramáticos": "epigramático",
    "epigráfica": "epigráfico",
    "epigráficas": "epigráfico",
    "epigráficos": "epigráfico",
    "epigástrica": "epigástrico",
    "epigástricas": "epigástrico",
    "epigástricos": "epigástrico",
    "epigénica": "epigénico",
    "epigénicas": "epigénico",
    "epigénicos": "epigénico",
    "epigónica": "epigónico",
    "epigónicas": "epigónico",
    "epigónicos": "epigónico",
    "epilepsias": "epilepsia",
    "epilimniones": "epilimnion",
    "epilimnios": "epilimnio",
    "epiloga": "epilogar",
    "epilogaba": "epilogar",
    "epilogabais": "epilogar",
    "epilogaban": "epilogar",
    "epilogabas": "epilogar",
    "epilogad": "epilogar",
    "epilogada": "epilogar",
    "epilogadas": "epilogar",
    "epilogado": "epilogar",
    "epilogados": "epilogar",
    "epilogamos": "epilogar",
    "epilogan": "epilogar",
    "epilogando": "epilogar",
    "epilogara": "epilogar",
    "epilogarais": "epilogar",
    "epilogaran": "epilogar",
    "epilogaras": "epilogar",
    "epilogare": "epilogar",
    "epilogareis": "epilogar",
    "epilogaremos": "epilogar",
    "epilogaren": "epilogar",
    "epilogares": "epilogar",
    "epilogaron": "epilogar",
    "epilogará": "epilogar",
    "epilogarán": "epilogar",
    "epilogarás": "epilogar",
    "epilogaré": "epilogar",
    "epilogaréis": "epilogar",
    "epilogaría": "epilogar",
    "epilogaríais": "epilogar",
    "epilogaríamos": "epilogar",
    "epilogarían": "epilogar",
    "epilogarías": "epilogar",
    "epilogas": "epilogar",
    "epilogase": "epilogar",
    "epilogaseis": "epilogar",
    "epilogasen": "epilogar",
    "epilogases": "epilogar",
    "epilogaste": "epilogar",
    "epilogasteis": "epilogar",
    "epilogismos": "epilogismo",
    "epilogo": "epilogar",
    "epilogue": "epilogar",
    "epiloguemos": "epilogar",
    "epiloguen": "epilogar",
    "epilogues": "epilogar",
    "epilogué": "epilogar",
    "epiloguéis": "epilogar",
    "epilogábamos": "epilogar",
    "epilogáis": "epilogar",
    "epilogáramos": "epilogar",
    "epilogáremos": "epilogar",
    "epilogásemos": "epilogar",
    "epilogó": "epilogar",
    "epiléptica": "epiléptico",
    "epilépticas": "epiléptico",
    "epilépticos": "epiléptico",
    "epimorfismos": "epimorfismo",
    "epinastias": "epinastia",
    "epinefrinas": "epinefrina",
    "epineuros": "epineuro",
    "epinicios": "epinicio",
    "epipediones": "epipedión",
    "epiploica": "epiploico",
    "epiploicas": "epiploico",
    "epiploicos": "epiploico",
    "epiplones": "epiplón",
    "epiqueratofaquias": "epiqueratofaquia",
    "epiqueremas": "epiquerema",
    "epiqueyas": "epiqueya",
    "epirogénica": "epirogénico",
    "epirogénicas": "epirogénico",
    "epirogénicos": "epirogénico",
    "epirotas": "epirota",
    "epirótica": "epirótico",
    "epiróticas": "epirótico",
    "epiróticos": "epirótico",
    "episclerales": "episcleral",
    "episcopados": "episcopado",
    "episcopales": "episcopal",
    "episcopaliana": "episcopaliano",
    "episcopalianas": "episcopaliano",
    "episcopalianos": "episcopaliano",
    "episcopalismos": "episcopalismo",
    "episcopios": "episcopio",
    "episcopologios": "episcopologio",
    "episodios": "episodio",
    "episomas": "episoma",
    "epispermas": "episperma",
    "epispásticos": "epispástico",
    "epistasias": "epistasia",
    "epistemologías": "epistemología",
    "epistemológica": "epistemológico",
    "epistemológicas": "epistemológico",
    "epistemológicos": "epistemológico",
    "epistilos": "epistilo",
    "epistolares": "epistolar",
    "epistolarios": "epistolario",
    "epistoleros": "epistolero",
    "epistomas": "epistoma",
    "epistémica": "epistémico",
    "epistémicas": "epistémico",
    "epistémicos": "epistémico",
    "episódica": "episódico",
    "episódicas": "episódico",
    "episódicos": "episódico",
    "epitafios": "epitafio",
    "epitalamios": "epitalamio",
    "epitalámica": "epitalámico",
    "epitalámicas": "epitalámico",
    "epitalámicos": "epitalámico",
    "epitaxias": "epitaxia",
    "epiteliales": "epitelial",
    "epitelioides": "epitelioide",
    "epiteliomas": "epitelioma",
    "epitelios": "epitelio",
    "epitimpánica": "epitimpánico",
    "epitimpánicas": "epitimpánico",
    "epitimpánicos": "epitimpánico",
    "epitoma": "epitomar",
    "epitomaba": "epitomar",
    "epitomabais": "epitomar",
    "epitomaban": "epitomar",
    "epitomabas": "epitomar",
    "epitomad": "epitomar",
    "epitomada": "epitomar",
    "epitomadas": "epitomar",
    "epitomado": "epitomar",
    "epitomadora": "epitomador",
    "epitomadoras": "epitomador",
    "epitomadores": "epitomador",
    "epitomados": "epitomar",
    "epitomamos": "epitomar",
    "epitoman": "epitomar",
    "epitomando": "epitomar",
    "epitomara": "epitomar",
    "epitomarais": "epitomar",
    "epitomaran": "epitomar",
    "epitomaras": "epitomar",
    "epitomare": "epitomar",
    "epitomareis": "epitomar",
    "epitomaremos": "epitomar",
    "epitomaren": "epitomar",
    "epitomares": "epitomar",
    "epitomaron": "epitomar",
    "epitomará": "epitomar",
    "epitomarán": "epitomar",
    "epitomarás": "epitomar",
    "epitomaré": "epitomar",
    "epitomaréis": "epitomar",
    "epitomaría": "epitomar",
    "epitomaríais": "epitomar",
    "epitomaríamos": "epitomar",
    "epitomarían": "epitomar",
    "epitomarías": "epitomar",
    "epitomas": "epitomar",
    "epitomase": "epitomar",
    "epitomaseis": "epitomar",
    "epitomasen": "epitomar",
    "epitomases": "epitomar",
    "epitomaste": "epitomar",
    "epitomasteis": "epitomar",
    "epitome": "epitomar",
    "epitomemos": "epitomar",
    "epitomen": "epitomar",
    "epitomes": "epitomar",
    "epitomo": "epitomar",
    "epitomábamos": "epitomar",
    "epitomáis": "epitomar",
    "epitomáramos": "epitomar",
    "epitomáremos": "epitomar",
    "epitomásemos": "epitomar",
    "epitomé": "epitomar",
    "epitoméis": "epitomar",
    "epitomó": "epitomar",
    "epitrópica": "epitrópico",
    "epitrópicas": "epitrópico",
    "epitrópicos": "epitrópico",
    "epitáctica": "epitáctico",
    "epitácticas": "epitáctico",
    "epitácticos": "epitáctico",
    "epitérmica": "epitérmico",
    "epitérmicas": "epitérmico",
    "epitérmicos": "epitérmico",
    "epitímpanos": "epitímpano",
    "epiyectiva": "epiyectivo",
    "epiyectivas": "epiyectivo",
    "epiyectivos": "epiyectivo",
    "epizoarios": "epizoario",
    "epizonas": "epizona",
    "epizootias": "epizootia",
    "epizoótica": "epizoótico",
    "epizoóticas": "epizoótico",
    "epizoóticos": "epizoótico",
    "epiácuica": "epiácuico",
    "epiácuicas": "epiácuico",
    "epiácuicos": "epiácuico",
    "epodas": "epoda",
    "epodos": "epodo",
    "epopeyas": "epopeya",
    "epoxidaciones": "epoxidación",
    "epsomitas": "epsomita",
    "epulones": "epulón",
    "epéndimos": "epéndimo",
    "epífita": "epífito",
    "epífitas": "epífito",
    "epífitos": "epífito",
    "epígonos": "epígono",
    "epígrafes": "epígrafe",
    "epílogos": "epílogo",
    "epímacos": "epímaco",
    "epímones": "epímone",
    "epístolas": "epístola",
    "epístrofes": "epístrofe",
    "epítemas": "epítema",
    "epítetos": "epíteto",
    "epítimas": "epítima",
    "epítimos": "epítimo",
    "epítomes": "epítome",
    "epítritos": "epítrito",
    "epítropes": "epítrope",
    "epónima": "epónimo",
    "epónimas": "epónimo",
    "epónimos": "epónimo",
    "epóxidos": "epóxido",
    "equiaxiales": "equiaxial",
    "equidades": "equidad",
    "equidiferencias": "equidiferencia",
    "equidireccionales": "equidireccional",
    "equidista": "equidistar",
    "equidistaba": "equidistar",
    "equidistabais": "equidistar",
    "equidistaban": "equidistar",
    "equidistabas": "equidistar",
    "equidistad": "equidistar",
    "equidistada": "equidistar",
    "equidistadas": "equidistar",
    "equidistado": "equidistar",
    "equidistados": "equidistar",
    "equidistamos": "equidistar",
    "equidistan": "equidistar",
    "equidistancias": "equidistancia",
    "equidistando": "equidistar",
    "equidistantes": "equidistante",
    "equidistara": "equidistar",
    "equidistarais": "equidistar",
    "equidistaran": "equidistar",
    "equidistaras": "equidistar",
    "equidistare": "equidistar",
    "equidistareis": "equidistar",
    "equidistaremos": "equidistar",
    "equidistaren": "equidistar",
    "equidistares": "equidistar",
    "equidistaron": "equidistar",
    "equidistará": "equidistar",
    "equidistarán": "equidistar",
    "equidistarás": "equidistar",
    "equidistaré": "equidistar",
    "equidistaréis": "equidistar",
    "equidistaría": "equidistar",
    "equidistaríais": "equidistar",
    "equidistaríamos": "equidistar",
    "equidistarían": "equidistar",
    "equidistarías": "equidistar",
    "equidistas": "equidistar",
    "equidistase": "equidistar",
    "equidistaseis": "equidistar",
    "equidistasen": "equidistar",
    "equidistases": "equidistar",
    "equidistaste": "equidistar",
    "equidistasteis": "equidistar",
    "equidiste": "equidistar",
    "equidistemos": "equidistar",
    "equidisten": "equidistar",
    "equidistes": "equidistar",
    "equidisto": "equidistar",
    "equidistábamos": "equidistar",
    "equidistáis": "equidistar",
    "equidistáramos": "equidistar",
    "equidistáremos": "equidistar",
    "equidistásemos": "equidistar",
    "equidisté": "equidistar",
    "equidistéis": "equidistar",
    "equidistó": "equidistar",
    "equidnas": "equidna",
    "equilaterales": "equilateral",
    "equilibra": "equilibrar",
    "equilibraba": "equilibrar",
    "equilibrabais": "equilibrar",
    "equilibraban": "equilibrar",
    "equilibrabas": "equilibrar",
    "equilibraciones": "equilibración",
    "equilibrad": "equilibrar",
    "equilibrada": "equilibrar",
    "equilibradas": "equilibrar",
    "equilibrado": "equilibrar",
    "equilibradora": "equilibrador",
    "equilibradoras": "equilibradora",
    "equilibradores": "equilibrador",
    "equilibrados": "equilibrar",
    "equilibramos": "equilibrar",
    "equilibran": "equilibrar",
    "equilibrando": "equilibrar",
    "equilibrantes": "equilibrante",
    "equilibrara": "equilibrar",
    "equilibrarais": "equilibrar",
    "equilibraran": "equilibrar",
    "equilibraras": "equilibrar",
    "equilibrare": "equilibrar",
    "equilibrareis": "equilibrar",
    "equilibraremos": "equilibrar",
    "equilibraren": "equilibrar",
    "equilibrares": "equilibrar",
    "equilibraron": "equilibrar",
    "equilibrará": "equilibrar",
    "equilibrarán": "equilibrar",
    "equilibrarás": "equilibrar",
    "equilibraré": "equilibrar",
    "equilibraréis": "equilibrar",
    "equilibraría": "equilibrar",
    "equilibraríais": "equilibrar",
    "equilibraríamos": "equilibrar",
    "equilibrarían": "equilibrar",
    "equilibrarías": "equilibrar",
    "equilibras": "equilibrar",
    "equilibrase": "equilibrar",
    "equilibraseis": "equilibrar",
    "equilibrasen": "equilibrar",
    "equilibrases": "equilibrar",
    "equilibraste": "equilibrar",
    "equilibrasteis": "equilibrar",
    "equilibre": "equilibrar",
    "equilibremos": "equilibrar",
    "equilibren": "equilibrar",
    "equilibres": "equilibrar",
    "equilibrios": "equilibrio",
    "equilibrismos": "equilibrismo",
    "equilibristas": "equilibrista",
    "equilibro": "equilibrar",
    "equilibrábamos": "equilibrar",
    "equilibráis": "equilibrar",
    "equilibráramos": "equilibrar",
    "equilibráremos": "equilibrar",
    "equilibrásemos": "equilibrar",
    "equilibré": "equilibrar",
    "equilibréis": "equilibrar",
    "equilibró": "equilibrar",
    "equilátera": "equilátero",
    "equiláteras": "equilátero",
    "equiláteros": "equilátero",
    "equimoleculares": "equimolecular",
    "equimótica": "equimótico",
    "equimóticas": "equimótico",
    "equimóticos": "equimótico",
    "equina": "equino",
    "equinada": "equinado",
    "equinadas": "equinado",
    "equinados": "equinado",
    "equinas": "equino",
    "equinocciales": "equinoccial",
    "equinoccios": "equinoccio",
    "equinococos": "equinococo",
    "equinodermos": "equinodermo",
    "equinoideos": "equinoideo",
    "equinos": "equino",
    "equipa": "equipar",
    "equipaba": "equipar",
    "equipabais": "equipar",
    "equipaban": "equipar",
    "equipabas": "equipar",
    "equipad": "equipar",
    "equipada": "equipar",
    "equipadas": "equipar",
    "equipado": "equipar",
    "equipados": "equipar",
    "equipajes": "equipaje",
    "equipamientos": "equipamiento",
    "equipamos": "equipar",
    "equipan": "equipar",
    "equipando": "equipar",
    "equipara": "equiparar",
    "equiparaba": "equiparar",
    "equiparabais": "equiparar",
    "equiparaban": "equiparar",
    "equiparabas": "equiparar",
    "equiparables": "equiparable",
    "equiparaciones": "equiparación",
    "equiparad": "equiparar",
    "equiparada": "equiparar",
    "equiparadas": "equiparar",
    "equiparado": "equiparar",
    "equiparados": "equiparar",
    "equiparais": "equipar",
    "equiparamos": "equiparar",
    "equiparan": "equiparar",
    "equiparando": "equiparar",
    "equiparara": "equiparar",
    "equipararais": "equiparar",
    "equipararan": "equiparar",
    "equipararas": "equiparar",
    "equiparare": "equiparar",
    "equiparareis": "equiparar",
    "equipararemos": "equiparar",
    "equipararen": "equiparar",
    "equiparares": "equiparar",
    "equipararon": "equiparar",
    "equiparará": "equiparar",
    "equipararán": "equiparar",
    "equipararás": "equiparar",
    "equipararé": "equiparar",
    "equipararéis": "equiparar",
    "equipararía": "equiparar",
    "equipararíais": "equiparar",
    "equipararíamos": "equiparar",
    "equipararían": "equiparar",
    "equipararías": "equiparar",
    "equiparas": "equiparar",
    "equiparase": "equiparar",
    "equiparaseis": "equiparar",
    "equiparasen": "equiparar",
    "equiparases": "equiparar",
    "equiparaste": "equiparar",
    "equiparasteis": "equiparar",
    "equipare": "equiparar",
    "equipareis": "equipar",
    "equiparemos": "equiparar",
    "equiparen": "equiparar",
    "equipares": "equiparar",
    "equiparo": "equiparar",
    "equiparon": "equipar",
    "equiparticiones": "equipartición",
    "equipará": "equipar",
    "equiparábamos": "equiparar",
    "equiparáis": "equiparar",
    "equiparán": "equipar",
    "equiparáramos": "equiparar",
    "equiparáremos": "equiparar",
    "equiparás": "equipar",
    "equiparásemos": "equiparar",
    "equiparé": "equiparar",
    "equiparéis": "equiparar",
    "equiparía": "equipar",
    "equiparíais": "equipar",
    "equiparíamos": "equipar",
    "equiparían": "equipar",
    "equiparías": "equipar",
    "equiparó": "equiparar",
    "equipas": "equipar",
    "equipase": "equipar",
    "equipaseis": "equipar",
    "equipasen": "equipar",
    "equipases": "equipar",
    "equipaste": "equipar",
    "equipasteis": "equipar",
    "equipe": "equipar",
    "equipemos": "equipar",
    "equipen": "equipar",
    "equipes": "equipar",
    "equipo": "equipar",
    "equipolada": "equipolado",
    "equipoladas": "equipolado",
    "equipolados": "equipolado",
    "equipolencias": "equipolencia",
    "equipolentes": "equipolente",
    "equipondera": "equiponderar",
    "equiponderaba": "equiponderar",
    "equiponderabais": "equiponderar",
    "equiponderaban": "equiponderar",
    "equiponderabas": "equiponderar",
    "equiponderad": "equiponderar",
    "equiponderada": "equiponderar",
    "equiponderadas": "equiponderar",
    "equiponderado": "equiponderar",
    "equiponderados": "equiponderar",
    "equiponderamos": "equiponderar",
    "equiponderan": "equiponderar",
    "equiponderancias": "equiponderancia",
    "equiponderando": "equiponderar",
    "equiponderara": "equiponderar",
    "equiponderarais": "equiponderar",
    "equiponderaran": "equiponderar",
    "equiponderaras": "equiponderar",
    "equiponderare": "equiponderar",
    "equiponderareis": "equiponderar",
    "equiponderaremos": "equiponderar",
    "equiponderaren": "equiponderar",
    "equiponderares": "equiponderar",
    "equiponderaron": "equiponderar",
    "equiponderará": "equiponderar",
    "equiponderarán": "equiponderar",
    "equiponderarás": "equiponderar",
    "equiponderaré": "equiponderar",
    "equiponderaréis": "equiponderar",
    "equiponderaría": "equiponderar",
    "equiponderaríais": "equiponderar",
    "equiponderaríamos": "equiponderar",
    "equiponderarían": "equiponderar",
    "equiponderarías": "equiponderar",
    "equiponderas": "equiponderar",
    "equiponderase": "equiponderar",
    "equiponderaseis": "equiponderar",
    "equiponderasen": "equiponderar",
    "equiponderases": "equiponderar",
    "equiponderaste": "equiponderar",
    "equiponderasteis": "equiponderar",
    "equipondere": "equiponderar",
    "equiponderemos": "equiponderar",
    "equiponderen": "equiponderar",
    "equiponderes": "equiponderar",
    "equipondero": "equiponderar",
    "equiponderábamos": "equiponderar",
    "equiponderáis": "equiponderar",
    "equiponderáramos": "equiponderar",
    "equiponderáremos": "equiponderar",
    "equiponderásemos": "equiponderar",
    "equiponderé": "equiponderar",
    "equiponderéis": "equiponderar",
    "equiponderó": "equiponderar",
    "equipos": "equipo",
    "equipotenciales": "equipotencial",
    "equipotentes": "equipotente",
    "equiprobables": "equiprobable",
    "equipábamos": "equipar",
    "equipáis": "equipar",
    "equipáramos": "equipar",
    "equipáremos": "equipar",
    "equipásemos": "equipar",
    "equipé": "equipar",
    "equipéis": "equipar",
    "equipó": "equipar",
    "equirrectangulares": "equirrectangular",
    "equisetos": "equiseto",
    "equisetáceas": "equisetácea",
    "equitaciones": "equitación",
    "equitativa": "equitativo",
    "equitativas": "equitativo",
    "equitativos": "equitativo",
    "equivaldremos": "equivaler",
    "equivaldrá": "equivaler",
    "equivaldrán": "equivaler",
    "equivaldrás": "equivaler",
    "equivaldré": "equivaler",
    "equivaldréis": "equivaler",
    "equivaldría": "equivaler",
    "equivaldríais": "equivaler",
    "equivaldríamos": "equivaler",
    "equivaldrían": "equivaler",
    "equivaldrías": "equivaler",
    "equivale": "equivaler",
    "equivaled": "equivaler",
    "equivalemos": "equivaler",
    "equivalen": "equivaler",
    "equivalencias": "equivalencia",
    "equivalentes": "equivalente",
    "equivales": "equivaler",
    "equivalga": "equivaler",
    "equivalgamos": "equivaler",
    "equivalgan": "equivaler",
    "equivalgas": "equivaler",
    "equivalgo": "equivaler",
    "equivalgáis": "equivaler",
    "equivalida": "equivaler",
    "equivalidas": "equivaler",
    "equivalido": "equivaler",
    "equivalidos": "equivaler",
    "equivaliendo": "equivaler",
    "equivaliera": "equivaler",
    "equivalierais": "equivaler",
    "equivalieran": "equivaler",
    "equivalieras": "equivaler",
    "equivaliere": "equivaler",
    "equivaliereis": "equivaler",
    "equivalieren": "equivaler",
    "equivalieres": "equivaler",
    "equivalieron": "equivaler",
    "equivaliese": "equivaler",
    "equivalieseis": "equivaler",
    "equivaliesen": "equivaler",
    "equivalieses": "equivaler",
    "equivalimos": "equivaler",
    "equivaliste": "equivaler",
    "equivalisteis": "equivaler",
    "equivaliéramos": "equivaler",
    "equivaliéremos": "equivaler",
    "equivaliésemos": "equivaler",
    "equivalió": "equivaler",
    "equivaléis": "equivaler",
    "equivalí": "equivaler",
    "equivalía": "equivaler",
    "equivalíais": "equivaler",
    "equivalíamos": "equivaler",
    "equivalían": "equivaler",
    "equivalías": "equivaler",
    "equivoca": "equivocar",
    "equivocaba": "equivocar",
    "equivocabais": "equivocar",
    "equivocaban": "equivocar",
    "equivocabas": "equivocar",
    "equivocaciones": "equivocación",
    "equivocad": "equivocar",
    "equivocada": "equivocar",
    "equivocadas": "equivocar",
    "equivocado": "equivocar",
    "equivocados": "equivocar",
    "equivocamos": "equivocar",
    "equivocan": "equivocar",
    "equivocando": "equivocar",
    "equivocara": "equivocar",
    "equivocarais": "equivocar",
    "equivocaran": "equivocar",
    "equivocaras": "equivocar",
    "equivocare": "equivocar",
    "equivocareis": "equivocar",
    "equivocaremos": "equivocar",
    "equivocaren": "equivocar",
    "equivocares": "equivocar",
    "equivocaron": "equivocar",
    "equivocará": "equivocar",
    "equivocarán": "equivocar",
    "equivocarás": "equivocar",
    "equivocaré": "equivocar",
    "equivocaréis": "equivocar",
    "equivocaría": "equivocar",
    "equivocaríais": "equivocar",
    "equivocaríamos": "equivocar",
    "equivocarían": "equivocar",
    "equivocarías": "equivocar",
    "equivocas": "equivocar",
    "equivocase": "equivocar",
    "equivocaseis": "equivocar",
    "equivocasen": "equivocar",
    "equivocases": "equivocar",
    "equivocaste": "equivocar",
    "equivocasteis": "equivocar",
    "equivocidades": "equivocidad",
    "equivoco": "equivocar",
    "equivocábamos": "equivocar",
    "equivocáis": "equivocar",
    "equivocáramos": "equivocar",
    "equivocáremos": "equivocar",
    "equivocásemos": "equivocar",
    "equivocó": "equivocar",
    "equivoque": "equivocar",
    "equivoquemos": "equivocar",
    "equivoquen": "equivocar",
    "equivoques": "equivocar",
    "equivoqué": "equivocar",
    "equivoquéis": "equivocar",
    "equiángula": "equiángulo",
    "equiángulas": "equiángulo",
    "equiángulos": "equiángulo",
    "equívoca": "equívoco",
    "equívocas": "equívoco",
    "equívocos": "equívoco",
    "era": "ser",
    "eraba": "erar",
    "erabais": "erar",
    "eraban": "erar",
    "erabas": "erar",
    "erad": "erar",
    "erada": "erar",
    "eradas": "erar",
    "erado": "erar",
    "erados": "erar",
    "erais": "ser",
    "erala": "eral",
    "eralas": "eral",
    "erales": "eral",
    "eramos": "erar",
    "eran": "ser",
    "erando": "erar",
    "erara": "erar",
    "erarais": "erar",
    "eraran": "erar",
    "eraras": "erar",
    "erare": "erar",
    "erareis": "erar",
    "eraremos": "erar",
    "eraren": "erar",
    "erares": "erar",
    "erarios": "erario",
    "eraron": "erar",
    "erará": "erar",
    "erarán": "erar",
    "erarás": "erar",
    "eraré": "erar",
    "eraréis": "erar",
    "eraría": "erar",
    "eraríais": "erar",
    "eraríamos": "erar",
    "erarían": "erar",
    "erarías": "erar",
    "eras": "ser",
    "erase": "erar",
    "eraseis": "erar",
    "erasen": "erar",
    "erases": "erar",
    "erasmiana": "erasmiano",
    "erasmianas": "erasmiano",
    "erasmianos": "erasmiano",
    "erasmismos": "erasmismo",
    "erasmistas": "erasmista",
    "eraste": "erar",
    "erasteis": "erar",
    "erbios": "erbio",
    "ere": "erar",
    "erebos": "erebo",
    "erecciones": "erección",
    "erecta": "erecto",
    "erectas": "erecto",
    "erectilidades": "erectilidad",
    "erectora": "erector",
    "erectoras": "erector",
    "erectores": "erector",
    "erectos": "erecto",
    "eremitas": "eremita",
    "eremitismos": "eremitismo",
    "eremitorios": "eremitorio",
    "eremos": "erar",
    "eremítica": "eremítico",
    "eremíticas": "eremítico",
    "eremíticos": "eremítico",
    "eren": "erar",
    "eres": "ser",
    "eretismos": "eretismo",
    "ergastoplasmas": "ergastoplasma",
    "ergativa": "ergativo",
    "ergativas": "ergativo",
    "ergatividades": "ergatividad",
    "ergativos": "ergativo",
    "erges": "erg",
    "ergios": "ergio",
    "ergogramas": "ergograma",
    "ergonomías": "ergonomía",
    "ergonómica": "ergonómico",
    "ergonómicas": "ergonómico",
    "ergonómicos": "ergonómico",
    "ergosteroles": "ergosterol",
    "ergotaminas": "ergotamina",
    "ergotice": "ergotizar",
    "ergoticemos": "ergotizar",
    "ergoticen": "ergotizar",
    "ergotices": "ergotizar",
    "ergoticé": "ergotizar",
    "ergoticéis": "ergotizar",
    "ergotinas": "ergotina",
    "ergotismos": "ergotismo",
    "ergotistas": "ergotista",
    "ergotiza": "ergotizar",
    "ergotizaba": "ergotizar",
    "ergotizabais": "ergotizar",
    "ergotizaban": "ergotizar",
    "ergotizabas": "ergotizar",
    "ergotizad": "ergotizar",
    "ergotizada": "ergotizar",
    "ergotizadas": "ergotizar",
    "ergotizado": "ergotizar",
    "ergotizados": "ergotizar",
    "ergotizamos": "ergotizar",
    "ergotizan": "ergotizar",
    "ergotizando": "ergotizar",
    "ergotizantes": "ergotizante",
    "ergotizara": "ergotizar",
    "ergotizarais": "ergotizar",
    "ergotizaran": "ergotizar",
    "ergotizaras": "ergotizar",
    "ergotizare": "ergotizar",
    "ergotizareis": "ergotizar",
    "ergotizaremos": "ergotizar",
    "ergotizaren": "ergotizar",
    "ergotizares": "ergotizar",
    "ergotizaron": "ergotizar",
    "ergotizará": "ergotizar",
    "ergotizarán": "ergotizar",
    "ergotizarás": "ergotizar",
    "ergotizaré": "ergotizar",
    "ergotizaréis": "ergotizar",
    "ergotizaría": "ergotizar",
    "ergotizaríais": "ergotizar",
    "ergotizaríamos": "ergotizar",
    "ergotizarían": "ergotizar",
    "ergotizarías": "ergotizar",
    "ergotizas": "ergotizar",
    "ergotizase": "ergotizar",
    "ergotizaseis": "ergotizar",
    "ergotizasen": "ergotizar",
    "ergotizases": "ergotizar",
    "ergotizaste": "ergotizar",
    "ergotizasteis": "ergotizar",
    "ergotizo": "ergotizar",
    "ergotizábamos": "ergotizar",
    "ergotizáis": "ergotizar",
    "ergotizáramos": "ergotizar",
    "ergotizáremos": "ergotizar",
    "ergotizásemos": "ergotizar",
    "ergotizó": "ergotizar",
    "ergs": "erg",
    "erguid": "erguir",
    "erguida": "erguir",
    "erguidas": "erguir",
    "erguido": "erguir",
    "erguidos": "erguir",
    "erguimientos": "erguimiento",
    "erguimos": "erguir",
    "erguiremos": "erguir",
    "erguirá": "erguir",
    "erguirán": "erguir",
    "erguirás": "erguir",
    "erguiré": "erguir",
    "erguiréis": "erguir",
    "erguiría": "erguir",
    "erguiríais": "erguir",
    "erguiríamos": "erguir",
    "erguirían": "erguir",
    "erguirías": "erguir",
    "erguiste": "erguir",
    "erguisteis": "erguir",
    "erguí": "erguir",
    "erguía": "erguir",
    "erguíais": "erguir",
    "erguíamos": "erguir",
    "erguían": "erguir",
    "erguías": "erguir",
    "erguís": "erguir",
    "ergástulas": "ergástula",
    "ergástulos": "ergástulo",
    "ergódica": "ergódico",
    "ergódicas": "ergódico",
    "ergódicos": "ergódico",
    "eriales": "erial",
    "erice": "erizar",
    "ericemos": "erizar",
    "ericen": "erizar",
    "erices": "erizar",
    "ericáceas": "ericácea",
    "ericé": "erizar",
    "ericéis": "erizar",
    "erige": "erigir",
    "erigen": "erigir",
    "eriges": "erigir",
    "erigid": "erigir",
    "erigida": "erigir",
    "erigidas": "erigir",
    "erigido": "erigir",
    "erigidos": "erigir",
    "erigiendo": "erigir",
    "erigiera": "erigir",
    "erigierais": "erigir",
    "erigieran": "erigir",
    "erigieras": "erigir",
    "erigiere": "erigir",
    "erigiereis": "erigir",
    "erigieren": "erigir",
    "erigieres": "erigir",
    "erigieron": "erigir",
    "erigiese": "erigir",
    "erigieseis": "erigir",
    "erigiesen": "erigir",
    "erigieses": "erigir",
    "erigimos": "erigir",
    "erigiremos": "erigir",
    "erigirá": "erigir",
    "erigirán": "erigir",
    "erigirás": "erigir",
    "erigiré": "erigir",
    "erigiréis": "erigir",
    "erigiría": "erigir",
    "erigiríais": "erigir",
    "erigiríamos": "erigir",
    "erigirían": "erigir",
    "erigirías": "erigir",
    "erigiste": "erigir",
    "erigisteis": "erigir",
    "erigiéramos": "erigir",
    "erigiéremos": "erigir",
    "erigiésemos": "erigir",
    "erigió": "erigir",
    "erigmofonías": "erigmofonía",
    "erigí": "erigir",
    "erigía": "erigir",
    "erigíais": "erigir",
    "erigíamos": "erigir",
    "erigían": "erigir",
    "erigías": "erigir",
    "erigís": "erigir",
    "erija": "erigir",
    "erijamos": "erigir",
    "erijan": "erigir",
    "erijas": "erigir",
    "erijo": "erigir",
    "erijáis": "erigir",
    "eriotecnias": "eriotecnia",
    "erisipelas": "erisipela",
    "erisipelatosa": "erisipelatoso",
    "erisipelatosas": "erisipelatoso",
    "erisipelatosos": "erisipelatoso",
    "eritemas": "eritema",
    "eritematosupurativa": "eritematosupurativo",
    "eritematosupurativas": "eritematosupurativo",
    "eritematosupurativos": "eritematosupurativo",
    "eritrea": "eritreo",
    "eritreas": "eritreo",
    "eritremias": "eritremia",
    "eritrenos": "eritreno",
    "eritreos": "eritreo",
    "eritrinas": "eritrina",
    "eritritoles": "eritritol",
    "eritroblastopenias": "eritroblastopenia",
    "eritroblastos": "eritroblasto",
    "eritrocitoblastos": "eritrocitoblasto",
    "eritrocitopenias": "eritrocitopenia",
    "eritrocitos": "eritrocito",
    "eritrocítica": "eritrocítico",
    "eritrocíticas": "eritrocítico",
    "eritrocíticos": "eritrocítico",
    "eritrodextrinas": "eritrodextrina",
    "eritrodoncias": "eritrodoncia",
    "eritrofobias": "eritrofobia",
    "eritrogonios": "eritrogonio",
    "eritroleucemias": "eritroleucemia",
    "eritropenias": "eritropenia",
    "eritropoyetinas": "eritropoyetina",
    "eritropoyética": "eritropoyético",
    "eritropoyéticas": "eritropoyético",
    "eritropoyéticos": "eritropoyético",
    "eritropsias": "eritropsia",
    "eritrosinas": "eritrosina",
    "eritrosomas": "eritrosoma",
    "eritórbica": "eritórbico",
    "eritórbicas": "eritórbico",
    "eritórbicos": "eritórbico",
    "eriza": "erizar",
    "erizaba": "erizar",
    "erizabais": "erizar",
    "erizaban": "erizar",
    "erizabas": "erizar",
    "erizad": "erizar",
    "erizada": "erizar",
    "erizadas": "erizar",
    "erizado": "erizar",
    "erizados": "erizar",
    "erizamientos": "erizamiento",
    "erizamos": "erizar",
    "erizan": "erizar",
    "erizando": "erizar",
    "erizara": "erizar",
    "erizarais": "erizar",
    "erizaran": "erizar",
    "erizaras": "erizar",
    "erizare": "erizar",
    "erizareis": "erizar",
    "erizaremos": "erizar",
    "erizaren": "erizar",
    "erizares": "erizar",
    "erizaron": "erizar",
    "erizará": "erizar",
    "erizarán": "erizar",
    "erizarás": "erizar",
    "erizaré": "erizar",
    "erizaréis": "erizar",
    "erizaría": "erizar",
    "erizaríais": "erizar",
    "erizaríamos": "erizar",
    "erizarían": "erizar",
    "erizarías": "erizar",
    "erizas": "erizar",
    "erizase": "erizar",
    "erizaseis": "erizar",
    "erizasen": "erizar",
    "erizases": "erizar",
    "erizaste": "erizar",
    "erizasteis": "erizar",
    "erizo": "erizar",
    "erizos": "erizo",
    "erizábamos": "erizar",
    "erizáis": "erizar",
    "erizáramos": "erizar",
    "erizáremos": "erizar",
    "erizásemos": "erizar",
    "erizó": "erizar",
    "erl": "erlang",
    "erlangs": "erlang",
    "ermitas": "ermita",
    "ermitaña": "ermitaño",
    "ermitañas": "ermitaño",
    "ermitaños": "ermitaño",
    "ero": "erar",
    "eroga": "erogar",
    "erogaba": "erogar",
    "erogabais": "erogar",
    "erogaban": "erogar",
    "erogabas": "erogar",
    "erogaciones": "erogación",
    "erogad": "erogar",
    "erogada": "erogar",
    "erogadas": "erogar",
    "erogado": "erogar",
    "erogados": "erogar",
    "erogamos": "erogar",
    "erogan": "erogar",
    "erogando": "erogar",
    "erogara": "erogar",
    "erogarais": "erogar",
    "erogaran": "erogar",
    "erogaras": "erogar",
    "erogare": "erogar",
    "erogareis": "erogar",
    "erogaremos": "erogar",
    "erogaren": "erogar",
    "erogares": "erogar",
    "erogaron": "erogar",
    "erogará": "erogar",
    "erogarán": "erogar",
    "erogarás": "erogar",
    "erogaré": "erogar",
    "erogaréis": "erogar",
    "erogaría": "erogar",
    "erogaríais": "erogar",
    "erogaríamos": "erogar",
    "erogarían": "erogar",
    "erogarías": "erogar",
    "erogas": "erogar",
    "erogase": "erogar",
    "erogaseis": "erogar",
    "erogasen": "erogar",
    "erogases": "erogar",
    "erogaste": "erogar",
    "erogasteis": "erogar",
    "erogo": "erogar",
    "erogue": "erogar",
    "eroguemos": "erogar",
    "eroguen": "erogar",
    "erogues": "erogar",
    "erogué": "erogar",
    "eroguéis": "erogar",
    "erogábamos": "erogar",
    "erogáis": "erogar",
    "erogáramos": "erogar",
    "erogáremos": "erogar",
    "erogásemos": "erogar",
    "erogó": "erogar",
    "erosiona": "erosionar",
    "erosionaba": "erosionar",
    "erosionabais": "erosionar",
    "erosionaban": "erosionar",
    "erosionabas": "erosionar",
    "erosionabilidades": "erosionabilidad",
    "erosionables": "erosionable",
    "erosionad": "erosionar",
    "erosionada": "erosionar",
    "erosionadas": "erosionar",
    "erosionado": "erosionar",
    "erosionados": "erosionar",
    "erosionales": "erosional",
    "erosionamos": "erosionar",
    "erosionan": "erosionar",
    "erosionando": "erosionar",
    "erosionara": "erosionar",
    "erosionarais": "erosionar",
    "erosionaran": "erosionar",
    "erosionaras": "erosionar",
    "erosionare": "erosionar",
    "erosionareis": "erosionar",
    "erosionaremos": "erosionar",
    "erosionaren": "erosionar",
    "erosionares": "erosionar",
    "erosionaron": "erosionar",
    "erosionará": "erosionar",
    "erosionarán": "erosionar",
    "erosionarás": "erosionar",
    "erosionaré": "erosionar",
    "erosionaréis": "erosionar",
    "erosionaría": "erosionar",
    "erosionaríais": "erosionar",
    "erosionaríamos": "erosionar",
    "erosionarían": "erosionar",
    "erosionarías": "erosionar",
    "erosionas": "erosionar",
    "erosionase": "erosionar",
    "erosionaseis": "erosionar",
    "erosionasen": "erosionar",
    "erosionases": "erosionar",
    "erosionaste": "erosionar",
    "erosionasteis": "erosionar",
    "erosione": "erosionar",
    "erosionemos": "erosionar",
    "erosionen": "erosionar",
    "erosiones": "erosionar",
    "erosiono": "erosionar",
    "erosionábamos": "erosionar",
    "erosionáis": "erosionar",
    "erosionáramos": "erosionar",
    "erosionáremos": "erosionar",
    "erosionásemos": "erosionar",
    "erosioné": "erosionar",
    "erosionéis": "erosionar",
    "erosionó": "erosionar",
    "erosiva": "erosivo",
    "erosivas": "erosivo",
    "erosivos": "erosivo",
    "erostratismos": "erostratismo",
    "erotemas": "erotema",
    "erotismos": "erotismo",
    "erotomanías": "erotomanía",
    "erotómana": "erotómano",
    "erotómanas": "erotómano",
    "erotómanos": "erotómano",
    "erraba": "errar",
    "errabais": "errar",
    "erraban": "errar",
    "errabas": "errar",
    "errabunda": "errabundo",
    "errabundas": "errabundo",
    "errabundos": "errabundo",
    "errad": "errar",
    "errada": "errar",
    "erradas": "errar",
    "erradica": "erradicar",
    "erradicaba": "erradicar",
    "erradicabais": "erradicar",
    "erradicaban": "erradicar",
    "erradicabas": "erradicar",
    "erradicaciones": "erradicación",
    "erradicad": "erradicar",
    "erradicada": "erradicar",
    "erradicadas": "erradicar",
    "erradicado": "erradicar",
    "erradicados": "erradicar",
    "erradicamos": "erradicar",
    "erradican": "erradicar",
    "erradicando": "erradicar",
    "erradicara": "erradicar",
    "erradicarais": "erradicar",
    "erradicaran": "erradicar",
    "erradicaras": "erradicar",
    "erradicare": "erradicar",
    "erradicareis": "erradicar",
    "erradicaremos": "erradicar",
    "erradicaren": "erradicar",
    "erradicares": "erradicar",
    "erradicaron": "erradicar",
    "erradicará": "erradicar",
    "erradicarán": "erradicar",
    "erradicarás": "erradicar",
    "erradicaré": "erradicar",
    "erradicaréis": "erradicar",
    "erradicaría": "erradicar",
    "erradicaríais": "erradicar",
    "erradicaríamos": "erradicar",
    "erradicarían": "erradicar",
    "erradicarías": "erradicar",
    "erradicas": "erradicar",
    "erradicase": "erradicar",
    "erradicaseis": "erradicar",
    "erradicasen": "erradicar",
    "erradicases": "erradicar",
    "erradicaste": "erradicar",
    "erradicasteis": "erradicar",
    "erradico": "erradicar",
    "erradicábamos": "erradicar",
    "erradicáis": "erradicar",
    "erradicáramos": "erradicar",
    "erradicáremos": "erradicar",
    "erradicásemos": "erradicar",
    "erradicó": "erradicar",
    "erradique": "erradicar",
    "erradiquemos": "erradicar",
    "erradiquen": "erradicar",
    "erradiques": "erradicar",
    "erradiqué": "erradicar",
    "erradiquéis": "erradicar",
    "errado": "errar",
    "errados": "errar",
    "erramos": "errar",
    "errando": "errar",
    "errantes": "errante",
    "errara": "errar",
    "errarais": "errar",
    "erraran": "errar",
    "erraras": "errar",
    "errare": "errar",
    "errareis": "errar",
    "erraremos": "errar",
    "erraren": "errar",
    "errares": "errar",
    "erraron": "errar",
    "errará": "errar",
    "errarán": "errar",
    "errarás": "errar",
    "erraré": "errar",
    "erraréis": "errar",
    "erraría": "errar",
    "erraríais": "errar",
    "erraríamos": "errar",
    "errarían": "errar",
    "errarías": "errar",
    "errase": "errar",
    "erraseis": "errar",
    "errasen": "errar",
    "errases": "errar",
    "erraste": "errar",
    "errasteis": "errar",
    "erratas": "errata",
    "erremos": "errar",
    "erres": "erre",
    "errores": "error",
    "errábamos": "errar",
    "erráis": "errar",
    "erráramos": "errar",
    "erráremos": "errar",
    "errásemos": "errar",
    "errática": "errático",
    "erráticas": "errático",
    "erráticos": "errático",
    "erré": "errar",
    "erréis": "errar",
    "erró": "errar",
    "errónea": "erróneo",
    "erróneas": "erróneo",
    "erróneos": "erróneo",
    "ertzainas": "ertzaina",
    "erubescencias": "erubescencia",
    "erubescentes": "erubescente",
    "erubescitas": "erubescita",
    "erucamidas": "erucamida",
    "eructa": "eructar",
    "eructaba": "eructar",
    "eructabais": "eructar",
    "eructaban": "eructar",
    "eructabas": "eructar",
    "eructaciones": "eructación",
    "eructad": "eructar",
    "eructada": "eructar",
    "eructadas": "eructar",
    "eructado": "eructar",
    "eructados": "eructar",
    "eructamos": "eructar",
    "eructan": "eructar",
    "eructando": "eructar",
    "eructara": "eructar",
    "eructarais": "eructar",
    "eructaran": "eructar",
    "eructaras": "eructar",
    "eructare": "eructar",
    "eructareis": "eructar",
    "eructaremos": "eructar",
    "eructaren": "eructar",
    "eructares": "eructar",
    "eructaron": "eructar",
    "eructará": "eructar",
    "eructarán": "eructar",
    "eructarás": "eructar",
    "eructaré": "eructar",
    "eructaréis": "eructar",
    "eructaría": "eructar",
    "eructaríais": "eructar",
    "eructaríamos": "eructar",
    "eructarían": "eructar",
    "eructarías": "eructar",
    "eructas": "eructar",
    "eructase": "eructar",
    "eructaseis": "eructar",
    "eructasen": "eructar",
    "eructases": "eructar",
    "eructaste": "eructar",
    "eructasteis": "eructar",
    "eructe": "eructar",
    "eructemos": "eructar",
    "eructen": "eructar",
    "eructes": "eructar",
    "eructo": "eructar",
    "eructos": "eructo",
    "eructábamos": "eructar",
    "eructáis": "eructar",
    "eructáramos": "eructar",
    "eructáremos": "eructar",
    "eructásemos": "eructar",
    "eructé": "eructar",
    "eructéis": "eructar",
    "eructó": "eructar",
    "erudiciones": "erudición",
    "erudita": "erudito",
    "eruditas": "erudito",
    "eruditos": "erudito",
    "erupciones": "erupción",
    "erupciónes": "erupción",
    "eruptiva": "eruptivo",
    "eruptivas": "eruptivo",
    "eruptivos": "eruptivo",
    "eruta": "erutar",
    "erutaba": "erutar",
    "erutabais": "erutar",
    "erutaban": "erutar",
    "erutabas": "erutar",
    "erutad": "erutar",
    "erutada": "erutar",
    "erutadas": "erutar",
    "erutado": "erutar",
    "erutados": "erutar",
    "erutamos": "erutar",
    "erutan": "erutar",
    "erutando": "erutar",
    "erutara": "erutar",
    "erutarais": "erutar",
    "erutaran": "erutar",
    "erutaras": "erutar",
    "erutare": "erutar",
    "erutareis": "erutar",
    "erutaremos": "erutar",
    "erutaren": "erutar",
    "erutares": "erutar",
    "erutaron": "erutar",
    "erutará": "erutar",
    "erutarán": "erutar",
    "erutarás": "erutar",
    "erutaré": "erutar",
    "erutaréis": "erutar",
    "erutaría": "erutar",
    "erutaríais": "erutar",
    "erutaríamos": "erutar",
    "erutarían": "erutar",
    "erutarías": "erutar",
    "erutas": "erutar",
    "erutase": "erutar",
    "erutaseis": "erutar",
    "erutasen": "erutar",
    "erutases": "erutar",
    "erutaste": "erutar",
    "erutasteis": "erutar",
    "erute": "erutar",
    "erutemos": "erutar",
    "eruten": "erutar",
    "erutes": "erutar",
    "eruto": "erutar",
    "erutábamos": "erutar",
    "erutáis": "erutar",
    "erutáramos": "erutar",
    "erutáremos": "erutar",
    "erutásemos": "erutar",
    "eruté": "erutar",
    "erutéis": "erutar",
    "erutó": "erutar",
    "erábamos": "erar",
    "eráis": "erar",
    "eráramos": "erar",
    "eráremos": "erar",
    "erásemos": "erar",
    "eré": "erar",
    "eréctiles": "eréctil",
    "eréis": "erar",
    "eríos": "erío",
    "erística": "erístico",
    "erísticas": "erístico",
    "erísticos": "erístico",
    "eró": "erar",
    "erógena": "erógeno",
    "erógenas": "erógeno",
    "erógenos": "erógeno",
    "erótica": "erótico",
    "eróticas": "erótico",
    "eróticos": "erótico",
    "es": "ser",
    "esa": "ese",
    "esas": "ese",
    "esbatimentos": "esbatimento",
    "esbelta": "esbelto",
    "esbeltas": "esbelto",
    "esbelteces": "esbeltez",
    "esbeltos": "esbelto",
    "esbirros": "esbirro",
    "esboce": "esbozar",
    "esbocemos": "esbozar",
    "esbocen": "esbozar",
    "esboces": "esbozar",
    "esbocé": "esbozar",
    "esbocéis": "esbozar",
    "esboza": "esbozar",
    "esbozaba": "esbozar",
    "esbozabais": "esbozar",
    "esbozaban": "esbozar",
    "esbozabas": "esbozar",
    "esbozad": "esbozar",
    "esbozada": "esbozar",
    "esbozadas": "esbozar",
    "esbozado": "esbozar",
    "esbozados": "esbozar",
    "esbozamos": "esbozar",
    "esbozan": "esbozar",
    "esbozando": "esbozar",
    "esbozara": "esbozar",
    "esbozarais": "esbozar",
    "esbozaran": "esbozar",
    "esbozaras": "esbozar",
    "esbozare": "esbozar",
    "esbozareis": "esbozar",
    "esbozaremos": "esbozar",
    "esbozaren": "esbozar",
    "esbozares": "esbozar",
    "esbozaron": "esbozar",
    "esbozará": "esbozar",
    "esbozarán": "esbozar",
    "esbozarás": "esbozar",
    "esbozaré": "esbozar",
    "esbozaréis": "esbozar",
    "esbozaría": "esbozar",
    "esbozaríais": "esbozar",
    "esbozaríamos": "esbozar",
    "esbozarían": "esbozar",
    "esbozarías": "esbozar",
    "esbozas": "esbozar",
    "esbozase": "esbozar",
    "esbozaseis": "esbozar",
    "esbozasen": "esbozar",
    "esbozases": "esbozar",
    "esbozaste": "esbozar",
    "esbozasteis": "esbozar",
    "esbozo": "esbozar",
    "esbozos": "esbozo",
    "esbozábamos": "esbozar",
    "esbozáis": "esbozar",
    "esbozáramos": "esbozar",
    "esbozáremos": "esbozar",
    "esbozásemos": "esbozar",
    "esbozó": "esbozar",
    "escabecha": "escabechar",
    "escabechaba": "escabechar",
    "escabechabais": "escabechar",
    "escabechaban": "escabechar",
    "escabechabas": "escabechar",
    "escabechad": "escabechar",
    "escabechada": "escabechar",
    "escabechadas": "escabechar",
    "escabechado": "escabechar",
    "escabechados": "escabechar",
    "escabechamos": "escabechar",
    "escabechan": "escabechar",
    "escabechando": "escabechar",
    "escabechara": "escabechar",
    "escabecharais": "escabechar",
    "escabecharan": "escabechar",
    "escabecharas": "escabechar",
    "escabechare": "escabechar",
    "escabechareis": "escabechar",
    "escabecharemos": "escabechar",
    "escabecharen": "escabechar",
    "escabechares": "escabechar",
    "escabecharon": "escabechar",
    "escabechará": "escabechar",
    "escabecharán": "escabechar",
    "escabecharás": "escabechar",
    "escabecharé": "escabechar",
    "escabecharéis": "escabechar",
    "escabecharía": "escabechar",
    "escabecharíais": "escabechar",
    "escabecharíamos": "escabechar",
    "escabecharían": "escabechar",
    "escabecharías": "escabechar",
    "escabechas": "escabechar",
    "escabechase": "escabechar",
    "escabechaseis": "escabechar",
    "escabechasen": "escabechar",
    "escabechases": "escabechar",
    "escabechaste": "escabechar",
    "escabechasteis": "escabechar",
    "escabeche": "escabechar",
    "escabechemos": "escabechar",
    "escabechen": "escabechar",
    "escabeches": "escabeche",
    "escabechinas": "escabechina",
    "escabecho": "escabechar",
    "escabechábamos": "escabechar",
    "escabecháis": "escabechar",
    "escabecháramos": "escabechar",
    "escabecháremos": "escabechar",
    "escabechásemos": "escabechar",
    "escabeché": "escabechar",
    "escabechéis": "escabechar",
    "escabechó": "escabechar",
    "escabeles": "escabel",
    "escabina": "escabino",
    "escabinada": "escabinado",
    "escabinadas": "escabinado",
    "escabinados": "escabinado",
    "escabinas": "escabino",
    "escabinatos": "escabinato",
    "escabinos": "escabino",
    "escabiosas": "escabiosa",
    "escabros": "escabro",
    "escabrosa": "escabroso",
    "escabrosas": "escabroso",
    "escabrosidades": "escabrosidad",
    "escabrosos": "escabroso",
    "escabuches": "escabuche",
    "escabulla": "escabullir",
    "escabullamos": "escabullir",
    "escabullan": "escabullir",
    "escabullas": "escabullir",
    "escabulle": "escabullir",
    "escabullen": "escabullir",
    "escabullendo": "escabullir",
    "escabullera": "escabullir",
    "escabullerais": "escabullir",
    "escabulleran": "escabullir",
    "escabulleras": "escabullir",
    "escabullere": "escabullir",
    "escabullereis": "escabullir",
    "escabulleren": "escabullir",
    "escabulleres": "escabullir",
    "escabulleron": "escabullir",
    "escabulles": "escabullir",
    "escabullese": "escabullir",
    "escabulleseis": "escabullir",
    "escabullesen": "escabullir",
    "escabulleses": "escabullir",
    "escabullid": "escabullir",
    "escabullida": "escabullir",
    "escabullidas": "escabullir",
    "escabullido": "escabullir",
    "escabullidos": "escabullir",
    "escabullimientos": "escabullimiento",
    "escabullimos": "escabullir",
    "escabulliremos": "escabullir",
    "escabullirá": "escabullir",
    "escabullirán": "escabullir",
    "escabullirás": "escabullir",
    "escabulliré": "escabullir",
    "escabulliréis": "escabullir",
    "escabulliría": "escabullir",
    "escabulliríais": "escabullir",
    "escabulliríamos": "escabullir",
    "escabullirían": "escabullir",
    "escabullirías": "escabullir",
    "escabulliste": "escabullir",
    "escabullisteis": "escabullir",
    "escabullo": "escabullir",
    "escabulláis": "escabullir",
    "escabulléramos": "escabullir",
    "escabulléremos": "escabullir",
    "escabullésemos": "escabullir",
    "escabullí": "escabullir",
    "escabullía": "escabullir",
    "escabullíais": "escabullir",
    "escabullíamos": "escabullir",
    "escabullían": "escabullir",
    "escabullías": "escabullir",
    "escabullís": "escabullir",
    "escabulló": "escabullir",
    "escacha": "escachar",
    "escachaba": "escachar",
    "escachabais": "escachar",
    "escachaban": "escachar",
    "escachabas": "escachar",
    "escachad": "escachar",
    "escachada": "escachar",
    "escachadas": "escachar",
    "escachado": "escachar",
    "escachados": "escachar",
    "escachamos": "escachar",
    "escachan": "escachar",
    "escachando": "escachar",
    "escachara": "escachar",
    "escacharais": "escachar",
    "escacharan": "escachar",
    "escacharas": "escachar",
    "escachare": "escachar",
    "escachareis": "escachar",
    "escacharemos": "escachar",
    "escacharen": "escachar",
    "escachares": "escachar",
    "escacharon": "escachar",
    "escacharra": "escacharrar",
    "escacharraba": "escacharrar",
    "escacharrabais": "escacharrar",
    "escacharraban": "escacharrar",
    "escacharrabas": "escacharrar",
    "escacharrad": "escacharrar",
    "escacharrada": "escacharrar",
    "escacharradas": "escacharrar",
    "escacharrado": "escacharrar",
    "escacharrados": "escacharrar",
    "escacharramientos": "escacharramiento",
    "escacharramos": "escacharrar",
    "escacharran": "escacharrar",
    "escacharrando": "escacharrar",
    "escacharrara": "escacharrar",
    "escacharrarais": "escacharrar",
    "escacharraran": "escacharrar",
    "escacharraras": "escacharrar",
    "escacharrare": "escacharrar",
    "escacharrareis": "escacharrar",
    "escacharraremos": "escacharrar",
    "escacharraren": "escacharrar",
    "escacharrares": "escacharrar",
    "escacharraron": "escacharrar",
    "escacharrará": "escacharrar",
    "escacharrarán": "escacharrar",
    "escacharrarás": "escacharrar",
    "escacharraré": "escacharrar",
    "escacharraréis": "escacharrar",
    "escacharraría": "escacharrar",
    "escacharraríais": "escacharrar",
    "escacharraríamos": "escacharrar",
    "escacharrarían": "escacharrar",
    "escacharrarías": "escacharrar",
    "escacharras": "escacharrar",
    "escacharrase": "escacharrar",
    "escacharraseis": "escacharrar",
    "escacharrasen": "escacharrar",
    "escacharrases": "escacharrar",
    "escacharraste": "escacharrar",
    "escacharrasteis": "escacharrar",
    "escacharre": "escacharrar",
    "escacharremos": "escacharrar",
    "escacharren": "escacharrar",
    "escacharres": "escacharrar",
    "escacharro": "escacharrar",
    "escacharrábamos": "escacharrar",
    "escacharráis": "escacharrar",
    "escacharráramos": "escacharrar",
    "escacharráremos": "escacharrar",
    "escacharrásemos": "escacharrar",
    "escacharré": "escacharrar",
    "escacharréis": "escacharrar",
    "escacharró": "escacharrar",
    "escachará": "escachar",
    "escacharán": "escachar",
    "escacharás": "escachar",
    "escacharé": "escachar",
    "escacharéis": "escachar",
    "escacharía": "escachar",
    "escacharíais": "escachar",
    "escacharíamos": "escachar",
    "escacharían": "escachar",
    "escacharías": "escachar",
    "escachas": "escachar",
    "escachase": "escachar",
    "escachaseis": "escachar",
    "escachasen": "escachar",
    "escachases": "escachar",
    "escachaste": "escachar",
    "escachasteis": "escachar",
    "escache": "escachar",
    "escachemos": "escachar",
    "escachen": "escachar",
    "escaches": "escachar",
    "escachifolla": "escachifollar",
    "escachifollaba": "escachifollar",
    "escachifollabais": "escachifollar",
    "escachifollaban": "escachifollar",
    "escachifollabas": "escachifollar",
    "escachifollad": "escachifollar",
    "escachifollada": "escachifollar",
    "escachifolladas": "escachifollar",
    "escachifollado": "escachifollar",
    "escachifollados": "escachifollar",
    "escachifollamos": "escachifollar",
    "escachifollan": "escachifollar",
    "escachifollando": "escachifollar",
    "escachifollara": "escachifollar",
    "escachifollarais": "escachifollar",
    "escachifollaran": "escachifollar",
    "escachifollaras": "escachifollar",
    "escachifollare": "escachifollar",
    "escachifollareis": "escachifollar",
    "escachifollaremos": "escachifollar",
    "escachifollaren": "escachifollar",
    "escachifollares": "escachifollar",
    "escachifollaron": "escachifollar",
    "escachifollará": "escachifollar",
    "escachifollarán": "escachifollar",
    "escachifollarás": "escachifollar",
    "escachifollaré": "escachifollar",
    "escachifollaréis": "escachifollar",
    "escachifollaría": "escachifollar",
    "escachifollaríais": "escachifollar",
    "escachifollaríamos": "escachifollar",
    "escachifollarían": "escachifollar",
    "escachifollarías": "escachifollar",
    "escachifollas": "escachifollar",
    "escachifollase": "escachifollar",
    "escachifollaseis": "escachifollar",
    "escachifollasen": "escachifollar",
    "escachifollases": "escachifollar",
    "escachifollaste": "escachifollar",
    "escachifollasteis": "escachifollar",
    "escachifolle": "escachifollar",
    "escachifollemos": "escachifollar",
    "escachifollen": "escachifollar",
    "escachifolles": "escachifollar",
    "escachifollo": "escachifollar",
    "escachifollábamos": "escachifollar",
    "escachifolláis": "escachifollar",
    "escachifolláramos": "escachifollar",
    "escachifolláremos": "escachifollar",
    "escachifollásemos": "escachifollar",
    "escachifollé": "escachifollar",
    "escachifolléis": "escachifollar",
    "escachifolló": "escachifollar",
    "escachifulla": "escachifullar",
    "escachifullaba": "escachifullar",
    "escachifullabais": "escachifullar",
    "escachifullaban": "escachifullar",
    "escachifullabas": "escachifullar",
    "escachifullad": "escachifullar",
    "escachifullada": "escachifullar",
    "escachifulladas": "escachifullar",
    "escachifullado": "escachifullar",
    "escachifullados": "escachifullar",
    "escachifullamos": "escachifullar",
    "escachifullan": "escachifullar",
    "escachifullando": "escachifullar",
    "escachifullara": "escachifullar",
    "escachifullarais": "escachifullar",
    "escachifullaran": "escachifullar",
    "escachifullaras": "escachifullar",
    "escachifullare": "escachifullar",
    "escachifullareis": "escachifullar",
    "escachifullaremos": "escachifullar",
    "escachifullaren": "escachifullar",
    "escachifullares": "escachifullar",
    "escachifullaron": "escachifullar",
    "escachifullará": "escachifullar",
    "escachifullarán": "escachifullar",
    "escachifullarás": "escachifullar",
    "escachifullaré": "escachifullar",
    "escachifullaréis": "escachifullar",
    "escachifullaría": "escachifullar",
    "escachifullaríais": "escachifullar",
    "escachifullaríamos": "escachifullar",
    "escachifullarían": "escachifullar",
    "escachifullarías": "escachifullar",
    "escachifullas": "escachifullar",
    "escachifullase": "escachifullar",
    "escachifullaseis": "escachifullar",
    "escachifullasen": "escachifullar",
    "escachifullases": "escachifullar",
    "escachifullaste": "escachifullar",
    "escachifullasteis": "escachifullar",
    "escachifulle": "escachifullar",
    "escachifullemos": "escachifullar",
    "escachifullen": "escachifullar",
    "escachifulles": "escachifullar",
    "escachifullo": "escachifullar",
    "escachifullábamos": "escachifullar",
    "escachifulláis": "escachifullar",
    "escachifulláramos": "escachifullar",
    "escachifulláremos": "escachifullar",
    "escachifullásemos": "escachifullar",
    "escachifullé": "escachifullar",
    "escachifulléis": "escachifullar",
    "escachifulló": "escachifullar",
    "escacho": "escachar",
    "escachábamos": "escachar",
    "escacháis": "escachar",
    "escacháramos": "escachar",
    "escacháremos": "escachar",
    "escachásemos": "escachar",
    "escaché": "escachar",
    "escachéis": "escachar",
    "escachó": "escachar",
    "escafandras": "escafandra",
    "escafandrismos": "escafandrismo",
    "escafandristas": "escafandrista",
    "escafila": "escafilar",
    "escafilaba": "escafilar",
    "escafilabais": "escafilar",
    "escafilaban": "escafilar",
    "escafilabas": "escafilar",
    "escafilad": "escafilar",
    "escafilada": "escafilar",
    "escafiladas": "escafilar",
    "escafilado": "escafilar",
    "escafilados": "escafilar",
    "escafilamos": "escafilar",
    "escafilan": "escafilar",
    "escafilando": "escafilar",
    "escafilara": "escafilar",
    "escafilarais": "escafilar",
    "escafilaran": "escafilar",
    "escafilaras": "escafilar",
    "escafilare": "escafilar",
    "escafilareis": "escafilar",
    "escafilaremos": "escafilar",
    "escafilaren": "escafilar",
    "escafilares": "escafilar",
    "escafilaron": "escafilar",
    "escafilará": "escafilar",
    "escafilarán": "escafilar",
    "escafilarás": "escafilar",
    "escafilaré": "escafilar",
    "escafilaréis": "escafilar",
    "escafilaría": "escafilar",
    "escafilaríais": "escafilar",
    "escafilaríamos": "escafilar",
    "escafilarían": "escafilar",
    "escafilarías": "escafilar",
    "escafilas": "escafilar",
    "escafilase": "escafilar",
    "escafilaseis": "escafilar",
    "escafilasen": "escafilar",
    "escafilases": "escafilar",
    "escafilaste": "escafilar",
    "escafilasteis": "escafilar",
    "escafile": "escafilar",
    "escafilemos": "escafilar",
    "escafilen": "escafilar",
    "escafiles": "escafilar",
    "escafilo": "escafilar",
    "escafilábamos": "escafilar",
    "escafiláis": "escafilar",
    "escafiláramos": "escafilar",
    "escafiláremos": "escafilar",
    "escafilásemos": "escafilar",
    "escafilé": "escafilar",
    "escafiléis": "escafilar",
    "escafiló": "escafilar",
    "escafoidea": "escafoideo",
    "escafoideas": "escafoideo",
    "escafoideos": "escafoideo",
    "escafópodos": "escafópodo",
    "escajos": "escajo",
    "escala": "escalar",
    "escalaba": "escalar",
    "escalabais": "escalar",
    "escalaban": "escalar",
    "escalabas": "escalar",
    "escalables": "escalable",
    "escalabornes": "escalaborne",
    "escalabra": "escalabrar",
    "escalabraba": "escalabrar",
    "escalabrabais": "escalabrar",
    "escalabraban": "escalabrar",
    "escalabrabas": "escalabrar",
    "escalabrad": "escalabrar",
    "escalabrada": "escalabrar",
    "escalabradas": "escalabrar",
    "escalabrado": "escalabrar",
    "escalabrados": "escalabrar",
    "escalabramos": "escalabrar",
    "escalabran": "escalabrar",
    "escalabrando": "escalabrar",
    "escalabrara": "escalabrar",
    "escalabrarais": "escalabrar",
    "escalabraran": "escalabrar",
    "escalabraras": "escalabrar",
    "escalabrare": "escalabrar",
    "escalabrareis": "escalabrar",
    "escalabraremos": "escalabrar",
    "escalabraren": "escalabrar",
    "escalabrares": "escalabrar",
    "escalabraron": "escalabrar",
    "escalabrará": "escalabrar",
    "escalabrarán": "escalabrar",
    "escalabrarás": "escalabrar",
    "escalabraré": "escalabrar",
    "escalabraréis": "escalabrar",
    "escalabraría": "escalabrar",
    "escalabraríais": "escalabrar",
    "escalabraríamos": "escalabrar",
    "escalabrarían": "escalabrar",
    "escalabrarías": "escalabrar",
    "escalabras": "escalabrar",
    "escalabrase": "escalabrar",
    "escalabraseis": "escalabrar",
    "escalabrasen": "escalabrar",
    "escalabrases": "escalabrar",
    "escalabraste": "escalabrar",
    "escalabrasteis": "escalabrar",
    "escalabre": "escalabrar",
    "escalabremos": "escalabrar",
    "escalabren": "escalabrar",
    "escalabres": "escalabrar",
    "escalabro": "escalabrar",
    "escalabrábamos": "escalabrar",
    "escalabráis": "escalabrar",
    "escalabráramos": "escalabrar",
    "escalabráremos": "escalabrar",
    "escalabrásemos": "escalabrar",
    "escalabré": "escalabrar",
    "escalabréis": "escalabrar",
    "escalabró": "escalabrar",
    "escalad": "escalar",
    "escalada": "escalar",
    "escaladas": "escalar",
    "escalado": "escalar",
    "escaladora": "escalador",
    "escaladoras": "escalador",
    "escaladores": "escalador",
    "escalados": "escalar",
    "escalafones": "escalafón",
    "escalamientos": "escalamiento",
    "escalamos": "escalar",
    "escalan": "escalar",
    "escalando": "escalar",
    "escalara": "escalar",
    "escalarais": "escalar",
    "escalaran": "escalar",
    "escalaras": "escalar",
    "escalare": "escalar",
    "escalareis": "escalar",
    "escalaremos": "escalar",
    "escalaren": "escalar",
    "escalares": "escalar",
    "escalariformes": "escalariforme",
    "escalaron": "escalar",
    "escalará": "escalar",
    "escalarán": "escalar",
    "escalarás": "escalar",
    "escalaré": "escalar",
    "escalaréis": "escalar",
    "escalaría": "escalar",
    "escalaríais": "escalar",
    "escalaríamos": "escalar",
    "escalarían": "escalar",
    "escalarías": "escalar",
    "escalas": "escalar",
    "escalase": "escalar",
    "escalaseis": "escalar",
    "escalasen": "escalar",
    "escalases": "escalar",
    "escalaste": "escalar",
    "escalasteis": "escalar",
    "escalda": "escaldar",
    "escaldaba": "escaldar",
    "escaldabais": "escaldar",
    "escaldaban": "escaldar",
    "escaldabas": "escaldar",
    "escaldad": "escaldar",
    "escaldada": "escaldar",
    "escaldadas": "escaldar",
    "escaldado": "escaldar",
    "escaldados": "escaldar",
    "escaldaduras": "escaldadura",
    "escaldamos": "escaldar",
    "escaldan": "escaldar",
    "escaldando": "escaldar",
    "escaldara": "escaldar",
    "escaldarais": "escaldar",
    "escaldaran": "escaldar",
    "escaldaras": "escaldar",
    "escaldare": "escaldar",
    "escaldareis": "escaldar",
    "escaldaremos": "escaldar",
    "escaldaren": "escaldar",
    "escaldares": "escaldar",
    "escaldaron": "escaldar",
    "escaldará": "escaldar",
    "escaldarán": "escaldar",
    "escaldarás": "escaldar",
    "escaldaré": "escaldar",
    "escaldaréis": "escaldar",
    "escaldaría": "escaldar",
    "escaldaríais": "escaldar",
    "escaldaríamos": "escaldar",
    "escaldarían": "escaldar",
    "escaldarías": "escaldar",
    "escaldas": "escaldar",
    "escaldase": "escaldar",
    "escaldaseis": "escaldar",
    "escaldasen": "escaldar",
    "escaldases": "escaldar",
    "escaldaste": "escaldar",
    "escaldasteis": "escaldar",
    "escalde": "escaldar",
    "escaldemos": "escaldar",
    "escalden": "escaldar",
    "escaldes": "escaldar",
    "escaldo": "escaldar",
    "escaldos": "escaldo",
    "escaldábamos": "escaldar",
    "escaldáis": "escaldar",
    "escaldáramos": "escaldar",
    "escaldáremos": "escaldar",
    "escaldásemos": "escaldar",
    "escaldé": "escaldar",
    "escaldéis": "escaldar",
    "escaldó": "escaldar",
    "escale": "escalar",
    "escalemos": "escalar",
    "escalen": "escalar",
    "escalena": "escaleno",
    "escalenas": "escaleno",
    "escalenoedros": "escalenoedro",
    "escalenos": "escaleno",
    "escalenoédrica": "escalenoédrico",
    "escalenoédricas": "escalenoédrico",
    "escalenoédricos": "escalenoédrico",
    "escaleras": "escalera",
    "escaleriformes": "escaleriforme",
    "escalerillas": "escalerilla",
    "escales": "escalar",
    "escaletas": "escaleta",
    "escalfa": "escalfar",
    "escalfaba": "escalfar",
    "escalfabais": "escalfar",
    "escalfaban": "escalfar",
    "escalfabas": "escalfar",
    "escalfad": "escalfar",
    "escalfada": "escalfar",
    "escalfadas": "escalfar",
    "escalfado": "escalfar",
    "escalfadores": "escalfador",
    "escalfados": "escalfar",
    "escalfamos": "escalfar",
    "escalfan": "escalfar",
    "escalfando": "escalfar",
    "escalfara": "escalfar",
    "escalfarais": "escalfar",
    "escalfaran": "escalfar",
    "escalfaras": "escalfar",
    "escalfare": "escalfar",
    "escalfareis": "escalfar",
    "escalfaremos": "escalfar",
    "escalfaren": "escalfar",
    "escalfares": "escalfar",
    "escalfaron": "escalfar",
    "escalfarotes": "escalfarote",
    "escalfará": "escalfar",
    "escalfarán": "escalfar",
    "escalfarás": "escalfar",
    "escalfaré": "escalfar",
    "escalfaréis": "escalfar",
    "escalfaría": "escalfar",
    "escalfaríais": "escalfar",
    "escalfaríamos": "escalfar",
    "escalfarían": "escalfar",
    "escalfarías": "escalfar",
    "escalfas": "escalfar",
    "escalfase": "escalfar",
    "escalfaseis": "escalfar",
    "escalfasen": "escalfar",
    "escalfases": "escalfar",
    "escalfaste": "escalfar",
    "escalfasteis": "escalfar",
    "escalfe": "escalfar",
    "escalfemos": "escalfar",
    "escalfen": "escalfar",
    "escalfes": "escalfar",
    "escalfetas": "escalfeta",
    "escalfo": "escalfar",
    "escalfábamos": "escalfar",
    "escalfáis": "escalfar",
    "escalfáramos": "escalfar",
    "escalfáremos": "escalfar",
    "escalfásemos": "escalfar",
    "escalfé": "escalfar",
    "escalféis": "escalfar",
    "escalfó": "escalfar",
    "escalinatas": "escalinata",
    "escalios": "escalio",
    "escalmos": "escalmo",
    "escalo": "escalar",
    "escalofriantes": "escalofriante",
    "escalofríos": "escalofrío",
    "escalona": "escalonar",
    "escalonaba": "escalonar",
    "escalonabais": "escalonar",
    "escalonaban": "escalonar",
    "escalonabas": "escalonar",
    "escalonad": "escalonar",
    "escalonada": "escalonar",
    "escalonadas": "escalonar",
    "escalonado": "escalonar",
    "escalonados": "escalonar",
    "escalonamientos": "escalonamiento",
    "escalonamos": "escalonar",
    "escalonan": "escalonar",
    "escalonando": "escalonar",
    "escalonara": "escalonar",
    "escalonarais": "escalonar",
    "escalonaran": "escalonar",
    "escalonaras": "escalonar",
    "escalonare": "escalonar",
    "escalonareis": "escalonar",
    "escalonaremos": "escalonar",
    "escalonaren": "escalonar",
    "escalonares": "escalonar",
    "escalonaron": "escalonar",
    "escalonará": "escalonar",
    "escalonarán": "escalonar",
    "escalonarás": "escalonar",
    "escalonaré": "escalonar",
    "escalonaréis": "escalonar",
    "escalonaría": "escalonar",
    "escalonaríais": "escalonar",
    "escalonaríamos": "escalonar",
    "escalonarían": "escalonar",
    "escalonarías": "escalonar",
    "escalonas": "escalonar",
    "escalonase": "escalonar",
    "escalonaseis": "escalonar",
    "escalonasen": "escalonar",
    "escalonases": "escalonar",
    "escalonaste": "escalonar",
    "escalonasteis": "escalonar",
    "escalone": "escalonar",
    "escalonemos": "escalonar",
    "escalonen": "escalonar",
    "escalones": "escalonar",
    "escalonias": "escalonia",
    "escalono": "escalonar",
    "escalonábamos": "escalonar",
    "escalonáis": "escalonar",
    "escalonáramos": "escalonar",
    "escalonáremos": "escalonar",
    "escalonásemos": "escalonar",
    "escaloné": "escalonar",
    "escalonéis": "escalonar",
    "escalonó": "escalonar",
    "escalopas": "escalopa",
    "escalopes": "escalope",
    "escalos": "escalo",
    "escalpaciones": "escalpación",
    "escalpelos": "escalpelo",
    "escalplos": "escalplo",
    "escalábamos": "escalar",
    "escaláis": "escalar",
    "escaláramos": "escalar",
    "escaláremos": "escalar",
    "escalásemos": "escalar",
    "escalé": "escalar",
    "escaléis": "escalar",
    "escalímetros": "escalímetro",
    "escaló": "escalar",
    "escama": "escamar",
    "escamaba": "escamar",
    "escamabais": "escamar",
    "escamaban": "escamar",
    "escamabas": "escamar",
    "escamad": "escamar",
    "escamada": "escamar",
    "escamadas": "escamar",
    "escamado": "escamar",
    "escamados": "escamar",
    "escamamos": "escamar",
    "escaman": "escamar",
    "escamando": "escamar",
    "escamara": "escamar",
    "escamarais": "escamar",
    "escamaran": "escamar",
    "escamaras": "escamar",
    "escamare": "escamar",
    "escamareis": "escamar",
    "escamaremos": "escamar",
    "escamaren": "escamar",
    "escamares": "escamar",
    "escamaron": "escamar",
    "escamará": "escamar",
    "escamarán": "escamar",
    "escamarás": "escamar",
    "escamaré": "escamar",
    "escamaréis": "escamar",
    "escamaría": "escamar",
    "escamaríais": "escamar",
    "escamaríamos": "escamar",
    "escamarían": "escamar",
    "escamarías": "escamar",
    "escamas": "escamar",
    "escamase": "escamar",
    "escamaseis": "escamar",
    "escamasen": "escamar",
    "escamases": "escamar",
    "escamaste": "escamar",
    "escamasteis": "escamar",
    "escame": "escamar",
    "escamemos": "escamar",
    "escamen": "escamar",
    "escames": "escamar",
    "escamo": "escamar",
    "escamochos": "escamocho",
    "escamomastoidea": "escamomastoideo",
    "escamomastoideas": "escamomastoideo",
    "escamomastoideos": "escamomastoideo",
    "escamona": "escamón",
    "escamonas": "escamón",
    "escamonda": "escamondar",
    "escamondaba": "escamondar",
    "escamondabais": "escamondar",
    "escamondaban": "escamondar",
    "escamondabas": "escamondar",
    "escamondad": "escamondar",
    "escamondada": "escamondar",
    "escamondadas": "escamondar",
    "escamondado": "escamondar",
    "escamondados": "escamondar",
    "escamondaduras": "escamondadura",
    "escamondamos": "escamondar",
    "escamondan": "escamondar",
    "escamondando": "escamondar",
    "escamondara": "escamondar",
    "escamondarais": "escamondar",
    "escamondaran": "escamondar",
    "escamondaras": "escamondar",
    "escamondare": "escamondar",
    "escamondareis": "escamondar",
    "escamondaremos": "escamondar",
    "escamondaren": "escamondar",
    "escamondares": "escamondar",
    "escamondaron": "escamondar",
    "escamondará": "escamondar",
    "escamondarán": "escamondar",
    "escamondarás": "escamondar",
    "escamondaré": "escamondar",
    "escamondaréis": "escamondar",
    "escamondaría": "escamondar",
    "escamondaríais": "escamondar",
    "escamondaríamos": "escamondar",
    "escamondarían": "escamondar",
    "escamondarías": "escamondar",
    "escamondas": "escamondar",
    "escamondase": "escamondar",
    "escamondaseis": "escamondar",
    "escamondasen": "escamondar",
    "escamondases": "escamondar",
    "escamondaste": "escamondar",
    "escamondasteis": "escamondar",
    "escamonde": "escamondar",
    "escamondemos": "escamondar",
    "escamonden": "escamondar",
    "escamondes": "escamondar",
    "escamondo": "escamondar",
    "escamondos": "escamondo",
    "escamondábamos": "escamondar",
    "escamondáis": "escamondar",
    "escamondáramos": "escamondar",
    "escamondáremos": "escamondar",
    "escamondásemos": "escamondar",
    "escamondé": "escamondar",
    "escamondéis": "escamondar",
    "escamondó": "escamondar",
    "escamones": "escamón",
    "escamosa": "escamoso",
    "escamosas": "escamoso",
    "escamosos": "escamoso",
    "escamota": "escamotar",
    "escamotaba": "escamotar",
    "escamotabais": "escamotar",
    "escamotaban": "escamotar",
    "escamotabas": "escamotar",
    "escamotad": "escamotar",
    "escamotada": "escamotar",
    "escamotadas": "escamotar",
    "escamotado": "escamotar",
    "escamotados": "escamotar",
    "escamotamos": "escamotar",
    "escamotan": "escamotar",
    "escamotando": "escamotar",
    "escamotara": "escamotar",
    "escamotarais": "escamotar",
    "escamotaran": "escamotar",
    "escamotaras": "escamotar",
    "escamotare": "escamotar",
    "escamotareis": "escamotar",
    "escamotaremos": "escamotar",
    "escamotaren": "escamotar",
    "escamotares": "escamotar",
    "escamotaron": "escamotar",
    "escamotará": "escamotar",
    "escamotarán": "escamotar",
    "escamotarás": "escamotar",
    "escamotaré": "escamotar",
    "escamotaréis": "escamotar",
    "escamotaría": "escamotar",
    "escamotaríais": "escamotar",
    "escamotaríamos": "escamotar",
    "escamotarían": "escamotar",
    "escamotarías": "escamotar",
    "escamotas": "escamotar",
    "escamotase": "escamotar",
    "escamotaseis": "escamotar",
    "escamotasen": "escamotar",
    "escamotases": "escamotar",
    "escamotaste": "escamotar",
    "escamotasteis": "escamotar",
    "escamote": "escamotar",
    "escamotea": "escamotear",
    "escamoteaba": "escamotear",
    "escamoteabais": "escamotear",
    "escamoteaban": "escamotear",
    "escamoteabas": "escamotear",
    "escamoteables": "escamoteable",
    "escamotead": "escamotear",
    "escamoteada": "escamotear",
    "escamoteadas": "escamotear",
    "escamoteado": "escamotear",
    "escamoteadora": "escamoteador",
    "escamoteadoras": "escamoteador",
    "escamoteadores": "escamoteador",
    "escamoteados": "escamotear",
    "escamoteamos": "escamotear",
    "escamotean": "escamotear",
    "escamoteando": "escamotear",
    "escamoteara": "escamotear",
    "escamotearais": "escamotear",
    "escamotearan": "escamotear",
    "escamotearas": "escamotear",
    "escamoteare": "escamotear",
    "escamoteareis": "escamotear",
    "escamotearemos": "escamotear",
    "escamotearen": "escamotear",
    "escamoteares": "escamotear",
    "escamotearon": "escamotear",
    "escamoteará": "escamotear",
    "escamotearán": "escamotear",
    "escamotearás": "escamotear",
    "escamotearé": "escamotear",
    "escamotearéis": "escamotear",
    "escamotearía": "escamotear",
    "escamotearíais": "escamotear",
    "escamotearíamos": "escamotear",
    "escamotearían": "escamotear",
    "escamotearías": "escamotear",
    "escamoteas": "escamotear",
    "escamotease": "escamotear",
    "escamoteaseis": "escamotear",
    "escamoteasen": "escamotear",
    "escamoteases": "escamotear",
    "escamoteaste": "escamotear",
    "escamoteasteis": "escamotear",
    "escamotee": "escamotear",
    "escamoteemos": "escamotear",
    "escamoteen": "escamotear",
    "escamotees": "escamotear",
    "escamotemos": "escamotar",
    "escamoten": "escamotar",
    "escamoteo": "escamotear",
    "escamoteos": "escamoteo",
    "escamotes": "escamotar",
    "escamoteábamos": "escamotear",
    "escamoteáis": "escamotear",
    "escamoteáramos": "escamotear",
    "escamoteáremos": "escamotear",
    "escamoteásemos": "escamotear",
    "escamoteé": "escamotear",
    "escamoteéis": "escamotear",
    "escamoteó": "escamotear",
    "escamoto": "escamotar",
    "escamotábamos": "escamotar",
    "escamotáis": "escamotar",
    "escamotáramos": "escamotar",
    "escamotáremos": "escamotar",
    "escamotásemos": "escamotar",
    "escamoté": "escamotar",
    "escamotéis": "escamotar",
    "escamotó": "escamotar",
    "escampa": "escampar",
    "escampaba": "escampar",
    "escampabais": "escampar",
    "escampaban": "escampar",
    "escampabas": "escampar",
    "escampad": "escampar",
    "escampada": "escampar",
    "escampadas": "escampar",
    "escampado": "escampar",
    "escampados": "escampar",
    "escampamos": "escampar",
    "escampan": "escampar",
    "escampando": "escampar",
    "escampara": "escampar",
    "escamparais": "escampar",
    "escamparan": "escampar",
    "escamparas": "escampar",
    "escampare": "escampar",
    "escampareis": "escampar",
    "escamparemos": "escampar",
    "escamparen": "escampar",
    "escampares": "escampar",
    "escamparon": "escampar",
    "escampará": "escampar",
    "escamparán": "escampar",
    "escamparás": "escampar",
    "escamparé": "escampar",
    "escamparéis": "escampar",
    "escamparía": "escampar",
    "escamparíais": "escampar",
    "escamparíamos": "escampar",
    "escamparían": "escampar",
    "escamparías": "escampar",
    "escampas": "escampar",
    "escampase": "escampar",
    "escampaseis": "escampar",
    "escampasen": "escampar",
    "escampases": "escampar",
    "escampaste": "escampar",
    "escampasteis": "escampar",
    "escampavías": "escampavía",
    "escampe": "escampar",
    "escampemos": "escampar",
    "escampen": "escampar",
    "escampes": "escampar",
    "escampo": "escampar",
    "escampos": "escampo",
    "escampábamos": "escampar",
    "escampáis": "escampar",
    "escampáramos": "escampar",
    "escampáremos": "escampar",
    "escampásemos": "escampar",
    "escampé": "escampar",
    "escampéis": "escampar",
    "escampó": "escampar",
    "escamuda": "escamudo",
    "escamudas": "escamudo",
    "escamudos": "escamudo",
    "escamuja": "escamujar",
    "escamujaba": "escamujar",
    "escamujabais": "escamujar",
    "escamujaban": "escamujar",
    "escamujabas": "escamujar",
    "escamujad": "escamujar",
    "escamujada": "escamujar",
    "escamujadas": "escamujar",
    "escamujado": "escamujar",
    "escamujados": "escamujar",
    "escamujamos": "escamujar",
    "escamujan": "escamujar",
    "escamujando": "escamujar",
    "escamujara": "escamujar",
    "escamujarais": "escamujar",
    "escamujaran": "escamujar",
    "escamujaras": "escamujar",
    "escamujare": "escamujar",
    "escamujareis": "escamujar",
    "escamujaremos": "escamujar",
    "escamujaren": "escamujar",
    "escamujares": "escamujar",
    "escamujaron": "escamujar",
    "escamujará": "escamujar",
    "escamujarán": "escamujar",
    "escamujarás": "escamujar",
    "escamujaré": "escamujar",
    "escamujaréis": "escamujar",
    "escamujaría": "escamujar",
    "escamujaríais": "escamujar",
    "escamujaríamos": "escamujar",
    "escamujarían": "escamujar",
    "escamujarías": "escamujar",
    "escamujas": "escamujar",
    "escamujase": "escamujar",
    "escamujaseis": "escamujar",
    "escamujasen": "escamujar",
    "escamujases": "escamujar",
    "escamujaste": "escamujar",
    "escamujasteis": "escamujar",
    "escamuje": "escamujar",
    "escamujemos": "escamujar",
    "escamujen": "escamujar",
    "escamujes": "escamujar",
    "escamujo": "escamujar",
    "escamujos": "escamujo",
    "escamujábamos": "escamujar",
    "escamujáis": "escamujar",
    "escamujáramos": "escamujar",
    "escamujáremos": "escamujar",
    "escamujásemos": "escamujar",
    "escamujé": "escamujar",
    "escamujéis": "escamujar",
    "escamujó": "escamujar",
    "escamábamos": "escamar",
    "escamáis": "escamar",
    "escamáramos": "escamar",
    "escamáremos": "escamar",
    "escamásemos": "escamar",
    "escamé": "escamar",
    "escaméis": "escamar",
    "escamó": "escamar",
    "escancia": "escanciar",
    "escanciaba": "escanciar",
    "escanciabais": "escanciar",
    "escanciaban": "escanciar",
    "escanciabas": "escanciar",
    "escanciad": "escanciar",
    "escanciada": "escanciar",
    "escanciadas": "escanciar",
    "escanciado": "escanciar",
    "escanciadora": "escanciador",
    "escanciadoras": "escanciador",
    "escanciadores": "escanciador",
    "escanciados": "escanciar",
    "escanciamos": "escanciar",
    "escancian": "escanciar",
    "escanciando": "escanciar",
    "escanciara": "escanciar",
    "escanciarais": "escanciar",
    "escanciaran": "escanciar",
    "escanciaras": "escanciar",
    "escanciare": "escanciar",
    "escanciareis": "escanciar",
    "escanciaremos": "escanciar",
    "escanciaren": "escanciar",
    "escanciares": "escanciar",
    "escanciaron": "escanciar",
    "escanciará": "escanciar",
    "escanciarán": "escanciar",
    "escanciarás": "escanciar",
    "escanciaré": "escanciar",
    "escanciaréis": "escanciar",
    "escanciaría": "escanciar",
    "escanciaríais": "escanciar",
    "escanciaríamos": "escanciar",
    "escanciarían": "escanciar",
    "escanciarías": "escanciar",
    "escancias": "escanciar",
    "escanciase": "escanciar",
    "escanciaseis": "escanciar",
    "escanciasen": "escanciar",
    "escanciases": "escanciar",
    "escanciaste": "escanciar",
    "escanciasteis": "escanciar",
    "escancie": "escanciar",
    "escanciemos": "escanciar",
    "escancien": "escanciar",
    "escancies": "escanciar",
    "escancio": "escanciar",
    "escanciábamos": "escanciar",
    "escanciáis": "escanciar",
    "escanciáramos": "escanciar",
    "escanciáremos": "escanciar",
    "escanciásemos": "escanciar",
    "escancié": "escanciar",
    "escanciéis": "escanciar",
    "escanció": "escanciar",
    "escanda": "escandir",
    "escandaleras": "escandalera",
    "escandalice": "escandalizar",
    "escandalicemos": "escandalizar",
    "escandalicen": "escandalizar",
    "escandalices": "escandalizar",
    "escandalicé": "escandalizar",
    "escandalicéis": "escandalizar",
    "escandaliza": "escandalizar",
    "escandalizaba": "escandalizar",
    "escandalizabais": "escandalizar",
    "escandalizaban": "escandalizar",
    "escandalizabas": "escandalizar",
    "escandalizad": "escandalizar",
    "escandalizada": "escandalizar",
    "escandalizadas": "escandalizar",
    "escandalizado": "escandalizar",
    "escandalizadora": "escandalizador",
    "escandalizadoras": "escandalizador",
    "escandalizadores": "escandalizador",
    "escandalizados": "escandalizar",
    "escandalizamos": "escandalizar",
    "escandalizan": "escandalizar",
    "escandalizando": "escandalizar",
    "escandalizara": "escandalizar",
    "escandalizarais": "escandalizar",
    "escandalizaran": "escandalizar",
    "escandalizaras": "escandalizar",
    "escandalizare": "escandalizar",
    "escandalizareis": "escandalizar",
    "escandalizaremos": "escandalizar",
    "escandalizaren": "escandalizar",
    "escandalizares": "escandalizar",
    "escandalizaron": "escandalizar",
    "escandalizará": "escandalizar",
    "escandalizarán": "escandalizar",
    "escandalizarás": "escandalizar",
    "escandalizaré": "escandalizar",
    "escandalizaréis": "escandalizar",
    "escandalizaría": "escandalizar",
    "escandalizaríais": "escandalizar",
    "escandalizaríamos": "escandalizar",
    "escandalizarían": "escandalizar",
    "escandalizarías": "escandalizar",
    "escandalizas": "escandalizar",
    "escandalizase": "escandalizar",
    "escandalizaseis": "escandalizar",
    "escandalizasen": "escandalizar",
    "escandalizases": "escandalizar",
    "escandalizaste": "escandalizar",
    "escandalizasteis": "escandalizar",
    "escandalizo": "escandalizar",
    "escandalizábamos": "escandalizar",
    "escandalizáis": "escandalizar",
    "escandalizáramos": "escandalizar",
    "escandalizáremos": "escandalizar",
    "escandalizásemos": "escandalizar",
    "escandalizó": "escandalizar",
    "escandalla": "escandallar",
    "escandallaba": "escandallar",
    "escandallabais": "escandallar",
    "escandallaban": "escandallar",
    "escandallabas": "escandallar",
    "escandallad": "escandallar",
    "escandallada": "escandallar",
    "escandalladas": "escandallar",
    "escandallado": "escandallar",
    "escandallados": "escandallar",
    "escandallamos": "escandallar",
    "escandallan": "escandallar",
    "escandallando": "escandallar",
    "escandallara": "escandallar",
    "escandallarais": "escandallar",
    "escandallaran": "escandallar",
    "escandallaras": "escandallar",
    "escandallare": "escandallar",
    "escandallareis": "escandallar",
    "escandallaremos": "escandallar",
    "escandallaren": "escandallar",
    "escandallares": "escandallar",
    "escandallaron": "escandallar",
    "escandallará": "escandallar",
    "escandallarán": "escandallar",
    "escandallarás": "escandallar",
    "escandallaré": "escandallar",
    "escandallaréis": "escandallar",
    "escandallaría": "escandallar",
    "escandallaríais": "escandallar",
    "escandallaríamos": "escandallar",
    "escandallarían": "escandallar",
    "escandallarías": "escandallar",
    "escandallas": "escandallar",
    "escandallase": "escandallar",
    "escandallaseis": "escandallar",
    "escandallasen": "escandallar",
    "escandallases": "escandallar",
    "escandallaste": "escandallar",
    "escandallasteis": "escandallar",
    "escandalle": "escandallar",
    "escandallemos": "escandallar",
    "escandallen": "escandallar",
    "escandalles": "escandallar",
    "escandallo": "escandallar",
    "escandallos": "escandallo",
    "escandallábamos": "escandallar",
    "escandalláis": "escandallar",
    "escandalláramos": "escandallar",
    "escandalláremos": "escandallar",
    "escandallásemos": "escandallar",
    "escandallé": "escandallar",
    "escandalléis": "escandallar",
    "escandalló": "escandallar",
    "escandalosa": "escandaloso",
    "escandalosas": "escandaloso",
    "escandalosos": "escandaloso",
    "escandamos": "escandir",
    "escandan": "escandir",
    "escandas": "escandir",
    "escande": "escandir",
    "escanden": "escandir",
    "escandentes": "escandente",
    "escandes": "escandir",
    "escandias": "escandia",
    "escandid": "escandir",
    "escandida": "escandir",
    "escandidas": "escandir",
    "escandido": "escandir",
    "escandidos": "escandir",
    "escandiendo": "escandir",
    "escandiera": "escandir",
    "escandierais": "escandir",
    "escandieran": "escandir",
    "escandieras": "escandir",
    "escandiere": "escandir",
    "escandiereis": "escandir",
    "escandieren": "escandir",
    "escandieres": "escandir",
    "escandieron": "escandir",
    "escandiese": "escandir",
    "escandieseis": "escandir",
    "escandiesen": "escandir",
    "escandieses": "escandir",
    "escandimos": "escandir",
    "escandinava": "escandinavo",
    "escandinavas": "escandinavo",
    "escandinavos": "escandinavo",
    "escandios": "escandio",
    "escandiremos": "escandir",
    "escandirá": "escandir",
    "escandirán": "escandir",
    "escandirás": "escandir",
    "escandiré": "escandir",
    "escandiréis": "escandir",
    "escandiría": "escandir",
    "escandiríais": "escandir",
    "escandiríamos": "escandir",
    "escandirían": "escandir",
    "escandirías": "escandir",
    "escandiste": "escandir",
    "escandisteis": "escandir",
    "escandiéramos": "escandir",
    "escandiéremos": "escandir",
    "escandiésemos": "escandir",
    "escandió": "escandir",
    "escando": "escandir",
    "escandáis": "escandir",
    "escandí": "escandir",
    "escandía": "escandir",
    "escandíais": "escandir",
    "escandíamos": "escandir",
    "escandían": "escandir",
    "escandías": "escandir",
    "escandís": "escandir",
    "escanea": "escanear",
    "escaneaba": "escanear",
    "escaneabais": "escanear",
    "escaneaban": "escanear",
    "escaneabas": "escanear",
    "escanead": "escanear",
    "escaneada": "escanear",
    "escaneadas": "escanear",
    "escaneado": "escanear",
    "escaneados": "escanear",
    "escaneamos": "escanear",
    "escanean": "escanear",
    "escaneando": "escanear",
    "escaneara": "escanear",
    "escanearais": "escanear",
    "escanearan": "escanear",
    "escanearas": "escanear",
    "escaneare": "escanear",
    "escaneareis": "escanear",
    "escanearemos": "escanear",
    "escanearen": "escanear",
    "escaneares": "escanear",
    "escanearon": "escanear",
    "escaneará": "escanear",
    "escanearán": "escanear",
    "escanearás": "escanear",
    "escanearé": "escanear",
    "escanearéis": "escanear",
    "escanearía": "escanear",
    "escanearíais": "escanear",
    "escanearíamos": "escanear",
    "escanearían": "escanear",
    "escanearías": "escanear",
    "escaneas": "escanear",
    "escanease": "escanear",
    "escaneaseis": "escanear",
    "escaneasen": "escanear",
    "escaneases": "escanear",
    "escaneaste": "escanear",
    "escaneasteis": "escanear",
    "escanee": "escanear",
    "escaneemos": "escanear",
    "escaneen": "escanear",
    "escanees": "escanear",
    "escaneo": "escanear",
    "escaneos": "escaneo",
    "escaneábamos": "escanear",
    "escaneáis": "escanear",
    "escaneáramos": "escanear",
    "escaneáremos": "escanear",
    "escaneásemos": "escanear",
    "escaneé": "escanear",
    "escaneéis": "escanear",
    "escaneó": "escanear",
    "escanistas": "escanista",
    "escansiones": "escansión",
    "escantilla": "escantillar",
    "escantillaba": "escantillar",
    "escantillabais": "escantillar",
    "escantillaban": "escantillar",
    "escantillabas": "escantillar",
    "escantillad": "escantillar",
    "escantillada": "escantillar",
    "escantilladas": "escantillar",
    "escantillado": "escantillar",
    "escantillados": "escantillar",
    "escantillamos": "escantillar",
    "escantillan": "escantillar",
    "escantillando": "escantillar",
    "escantillara": "escantillar",
    "escantillarais": "escantillar",
    "escantillaran": "escantillar",
    "escantillaras": "escantillar",
    "escantillare": "escantillar",
    "escantillareis": "escantillar",
    "escantillaremos": "escantillar",
    "escantillaren": "escantillar",
    "escantillares": "escantillar",
    "escantillaron": "escantillar",
    "escantillará": "escantillar",
    "escantillarán": "escantillar",
    "escantillarás": "escantillar",
    "escantillaré": "escantillar",
    "escantillaréis": "escantillar",
    "escantillaría": "escantillar",
    "escantillaríais": "escantillar",
    "escantillaríamos": "escantillar",
    "escantillarían": "escantillar",
    "escantillarías": "escantillar",
    "escantillas": "escantillar",
    "escantillase": "escantillar",
    "escantillaseis": "escantillar",
    "escantillasen": "escantillar",
    "escantillases": "escantillar",
    "escantillaste": "escantillar",
    "escantillasteis": "escantillar",
    "escantille": "escantillar",
    "escantillemos": "escantillar",
    "escantillen": "escantillar",
    "escantilles": "escantillar",
    "escantillo": "escantillar",
    "escantillones": "escantillón",
    "escantillábamos": "escantillar",
    "escantilláis": "escantillar",
    "escantilláramos": "escantillar",
    "escantilláremos": "escantillar",
    "escantillásemos": "escantillar",
    "escantillé": "escantillar",
    "escantilléis": "escantillar",
    "escantilló": "escantillar",
    "escapa": "escapar",
    "escapaba": "escapar",
    "escapabais": "escapar",
    "escapaban": "escapar",
    "escapabas": "escapar",
    "escapad": "escapar",
    "escapada": "escapar",
    "escapadas": "escapar",
    "escapado": "escapar",
    "escapados": "escapar",
    "escapamos": "escapar",
    "escapan": "escapar",
    "escapando": "escapar",
    "escapara": "escapar",
    "escaparais": "escapar",
    "escaparan": "escapar",
    "escaparas": "escapar",
    "escaparates": "escaparate",
    "escaparatismos": "escaparatismo",
    "escaparatistas": "escaparatista",
    "escapare": "escapar",
    "escapareis": "escapar",
    "escaparemos": "escapar",
    "escaparen": "escapar",
    "escapares": "escapar",
    "escaparon": "escapar",
    "escapará": "escapar",
    "escaparán": "escapar",
    "escaparás": "escapar",
    "escaparé": "escapar",
    "escaparéis": "escapar",
    "escaparía": "escapar",
    "escaparíais": "escapar",
    "escaparíamos": "escapar",
    "escaparían": "escapar",
    "escaparías": "escapar",
    "escapas": "escapar",
    "escapase": "escapar",
    "escapaseis": "escapar",
    "escapasen": "escapar",
    "escapases": "escapar",
    "escapaste": "escapar",
    "escapasteis": "escapar",
    "escapatoria": "escapatorio",
    "escapatorias": "escapatorio",
    "escapatorios": "escapatorio",
    "escape": "escapar",
    "escapemos": "escapar",
    "escapen": "escapar",
    "escapes": "escape",
    "escapismos": "escapismo",
    "escapistas": "escapista",
    "escapo": "escapar",
    "escapolitas": "escapolita",
    "escapos": "escapo",
    "escapulares": "escapular",
    "escapularios": "escapulario",
    "escapulohumerales": "escapulohumeral",
    "escapábamos": "escapar",
    "escapáis": "escapar",
    "escapáramos": "escapar",
    "escapáremos": "escapar",
    "escapásemos": "escapar",
    "escapé": "escapar",
    "escapéis": "escapar",
    "escapó": "escapar",
    "escaqueada": "escaqueado",
    "escaqueadas": "escaqueado",
    "escaqueados": "escaqueado",
    "escaques": "escaque",
    "escarabajea": "escarabajear",
    "escarabajeaba": "escarabajear",
    "escarabajeabais": "escarabajear",
    "escarabajeaban": "escarabajear",
    "escarabajeabas": "escarabajear",
    "escarabajead": "escarabajear",
    "escarabajeada": "escarabajear",
    "escarabajeadas": "escarabajear",
    "escarabajeado": "escarabajear",
    "escarabajeados": "escarabajear",
    "escarabajeamos": "escarabajear",
    "escarabajean": "escarabajear",
    "escarabajeando": "escarabajear",
    "escarabajeara": "escarabajear",
    "escarabajearais": "escarabajear",
    "escarabajearan": "escarabajear",
    "escarabajearas": "escarabajear",
    "escarabajeare": "escarabajear",
    "escarabajeareis": "escarabajear",
    "escarabajearemos": "escarabajear",
    "escarabajearen": "escarabajear",
    "escarabajeares": "escarabajear",
    "escarabajearon": "escarabajear",
    "escarabajeará": "escarabajear",
    "escarabajearán": "escarabajear",
    "escarabajearás": "escarabajear",
    "escarabajearé": "escarabajear",
    "escarabajearéis": "escarabajear",
    "escarabajearía": "escarabajear",
    "escarabajearíais": "escarabajear",
    "escarabajearíamos": "escarabajear",
    "escarabajearían": "escarabajear",
    "escarabajearías": "escarabajear",
    "escarabajeas": "escarabajear",
    "escarabajease": "escarabajear",
    "escarabajeaseis": "escarabajear",
    "escarabajeasen": "escarabajear",
    "escarabajeases": "escarabajear",
    "escarabajeaste": "escarabajear",
    "escarabajeasteis": "escarabajear",
    "escarabajee": "escarabajear",
    "escarabajeemos": "escarabajear",
    "escarabajeen": "escarabajear",
    "escarabajees": "escarabajear",
    "escarabajeo": "escarabajear",
    "escarabajeos": "escarabajeo",
    "escarabajeábamos": "escarabajear",
    "escarabajeáis": "escarabajear",
    "escarabajeáramos": "escarabajear",
    "escarabajeáremos": "escarabajear",
    "escarabajeásemos": "escarabajear",
    "escarabajeé": "escarabajear",
    "escarabajeéis": "escarabajear",
    "escarabajeó": "escarabajear",
    "escarabajos": "escarabajo",
    "escarabajuelos": "escarabajuelo",
    "escarabeidos": "escarabeido",
    "escarabeos": "escarabeo",
    "escaramuce": "escaramuzar",
    "escaramucea": "escaramucear",
    "escaramuceaba": "escaramucear",
    "escaramuceabais": "escaramucear",
    "escaramuceaban": "escaramucear",
    "escaramuceabas": "escaramucear",
    "escaramucead": "escaramucear",
    "escaramuceada": "escaramucear",
    "escaramuceadas": "escaramucear",
    "escaramuceado": "escaramucear",
    "escaramuceados": "escaramucear",
    "escaramuceamos": "escaramucear",
    "escaramucean": "escaramucear",
    "escaramuceando": "escaramucear",
    "escaramuceara": "escaramucear",
    "escaramucearais": "escaramucear",
    "escaramucearan": "escaramucear",
    "escaramucearas": "escaramucear",
    "escaramuceare": "escaramucear",
    "escaramuceareis": "escaramucear",
    "escaramucearemos": "escaramucear",
    "escaramucearen": "escaramucear",
    "escaramuceares": "escaramucear",
    "escaramucearon": "escaramucear",
    "escaramuceará": "escaramucear",
    "escaramucearán": "escaramucear",
    "escaramucearás": "escaramucear",
    "escaramucearé": "escaramucear",
    "escaramucearéis": "escaramucear",
    "escaramucearía": "escaramucear",
    "escaramucearíais": "escaramucear",
    "escaramucearíamos": "escaramucear",
    "escaramucearían": "escaramucear",
    "escaramucearías": "escaramucear",
    "escaramuceas": "escaramucear",
    "escaramucease": "escaramucear",
    "escaramuceaseis": "escaramucear",
    "escaramuceasen": "escaramucear",
    "escaramuceases": "escaramucear",
    "escaramuceaste": "escaramucear",
    "escaramuceasteis": "escaramucear",
    "escaramucee": "escaramucear",
    "escaramuceemos": "escaramucear",
    "escaramuceen": "escaramucear",
    "escaramucees": "escaramucear",
    "escaramucemos": "escaramuzar",
    "escaramucen": "escaramuzar",
    "escaramuceo": "escaramucear",
    "escaramuces": "escaramuzar",
    "escaramuceábamos": "escaramucear",
    "escaramuceáis": "escaramucear",
    "escaramuceáramos": "escaramucear",
    "escaramuceáremos": "escaramucear",
    "escaramuceásemos": "escaramucear",
    "escaramuceé": "escaramucear",
    "escaramuceéis": "escaramucear",
    "escaramuceó": "escaramucear",
    "escaramucé": "escaramuzar",
    "escaramucéis": "escaramuzar",
    "escaramujos": "escaramujo",
    "escaramuza": "escaramuzar",
    "escaramuzaba": "escaramuzar",
    "escaramuzabais": "escaramuzar",
    "escaramuzaban": "escaramuzar",
    "escaramuzabas": "escaramuzar",
    "escaramuzad": "escaramuzar",
    "escaramuzada": "escaramuzar",
    "escaramuzadas": "escaramuzar",
    "escaramuzado": "escaramuzar",
    "escaramuzadora": "escaramuzador",
    "escaramuzadoras": "escaramuzador",
    "escaramuzadores": "escaramuzador",
    "escaramuzados": "escaramuzar",
    "escaramuzamos": "escaramuzar",
    "escaramuzan": "escaramuzar",
    "escaramuzando": "escaramuzar",
    "escaramuzara": "escaramuzar",
    "escaramuzarais": "escaramuzar",
    "escaramuzaran": "escaramuzar",
    "escaramuzaras": "escaramuzar",
    "escaramuzare": "escaramuzar",
    "escaramuzareis": "escaramuzar",
    "escaramuzaremos": "escaramuzar",
    "escaramuzaren": "escaramuzar",
    "escaramuzares": "escaramuzar",
    "escaramuzaron": "escaramuzar",
    "escaramuzará": "escaramuzar",
    "escaramuzarán": "escaramuzar",
    "escaramuzarás": "escaramuzar",
    "escaramuzaré": "escaramuzar",
    "escaramuzaréis": "escaramuzar",
    "escaramuzaría": "escaramuzar",
    "escaramuzaríais": "escaramuzar",
    "escaramuzaríamos": "escaramuzar",
    "escaramuzarían": "escaramuzar",
    "escaramuzarías": "escaramuzar",
    "escaramuzas": "escaramuzar",
    "escaramuzase": "escaramuzar",
    "escaramuzaseis": "escaramuzar",
    "escaramuzasen": "escaramuzar",
    "escaramuzases": "escaramuzar",
    "escaramuzaste": "escaramuzar",
    "escaramuzasteis": "escaramuzar",
    "escaramuzo": "escaramuzar",
    "escaramuzábamos": "escaramuzar",
    "escaramuzáis": "escaramuzar",
    "escaramuzáramos": "escaramuzar",
    "escaramuzáremos": "escaramuzar",
    "escaramuzásemos": "escaramuzar",
    "escaramuzó": "escaramuzar",
    "escarapelas": "escarapela",
    "escaras": "escara",
    "escarba": "escarbar",
    "escarbaba": "escarbar",
    "escarbabais": "escarbar",
    "escarbaban": "escarbar",
    "escarbabas": "escarbar",
    "escarbad": "escarbar",
    "escarbada": "escarbar",
    "escarbadas": "escarbar",
    "escarbaderos": "escarbadero",
    "escarbado": "escarbar",
    "escarbadora": "escarbador",
    "escarbadoras": "escarbador",
    "escarbadores": "escarbador",
    "escarbados": "escarbar",
    "escarbaduras": "escarbadura",
    "escarbamos": "escarbar",
    "escarban": "escarbar",
    "escarbando": "escarbar",
    "escarbara": "escarbar",
    "escarbarais": "escarbar",
    "escarbaran": "escarbar",
    "escarbaras": "escarbar",
    "escarbare": "escarbar",
    "escarbareis": "escarbar",
    "escarbaremos": "escarbar",
    "escarbaren": "escarbar",
    "escarbares": "escarbar",
    "escarbaron": "escarbar",
    "escarbará": "escarbar",
    "escarbarán": "escarbar",
    "escarbarás": "escarbar",
    "escarbaré": "escarbar",
    "escarbaréis": "escarbar",
    "escarbaría": "escarbar",
    "escarbaríais": "escarbar",
    "escarbaríamos": "escarbar",
    "escarbarían": "escarbar",
    "escarbarías": "escarbar",
    "escarbas": "escarbar",
    "escarbase": "escarbar",
    "escarbaseis": "escarbar",
    "escarbasen": "escarbar",
    "escarbases": "escarbar",
    "escarbaste": "escarbar",
    "escarbasteis": "escarbar",
    "escarbe": "escarbar",
    "escarbemos": "escarbar",
    "escarben": "escarbar",
    "escarbes": "escarbar",
    "escarbillos": "escarbillo",
    "escarbo": "escarbar",
    "escarbos": "escarbo",
    "escarbábamos": "escarbar",
    "escarbáis": "escarbar",
    "escarbáramos": "escarbar",
    "escarbáremos": "escarbar",
    "escarbásemos": "escarbar",
    "escarbé": "escarbar",
    "escarbéis": "escarbar",
    "escarbó": "escarbar",
    "escarce": "escarzar",
    "escarcea": "escarcear",
    "escarceaba": "escarcear",
    "escarceabais": "escarcear",
    "escarceaban": "escarcear",
    "escarceabas": "escarcear",
    "escarcead": "escarcear",
    "escarceada": "escarcear",
    "escarceadas": "escarcear",
    "escarceado": "escarcear",
    "escarceados": "escarcear",
    "escarceamos": "escarcear",
    "escarcean": "escarcear",
    "escarceando": "escarcear",
    "escarceara": "escarcear",
    "escarcearais": "escarcear",
    "escarcearan": "escarcear",
    "escarcearas": "escarcear",
    "escarceare": "escarcear",
    "escarceareis": "escarcear",
    "escarcearemos": "escarcear",
    "escarcearen": "escarcear",
    "escarceares": "escarcear",
    "escarcearon": "escarcear",
    "escarceará": "escarcear",
    "escarcearán": "escarcear",
    "escarcearás": "escarcear",
    "escarcearé": "escarcear",
    "escarcearéis": "escarcear",
    "escarcearía": "escarcear",
    "escarcearíais": "escarcear",
    "escarcearíamos": "escarcear",
    "escarcearían": "escarcear",
    "escarcearías": "escarcear",
    "escarceas": "escarcear",
    "escarcease": "escarcear",
    "escarceaseis": "escarcear",
    "escarceasen": "escarcear",
    "escarceases": "escarcear",
    "escarceaste": "escarcear",
    "escarceasteis": "escarcear",
    "escarcee": "escarcear",
    "escarceemos": "escarcear",
    "escarceen": "escarcear",
    "escarcees": "escarcear",
    "escarcelas": "escarcela",
    "escarcemos": "escarzar",
    "escarcen": "escarzar",
    "escarceo": "escarcear",
    "escarceos": "escarceo",
    "escarces": "escarzar",
    "escarceábamos": "escarcear",
    "escarceáis": "escarcear",
    "escarceáramos": "escarcear",
    "escarceáremos": "escarcear",
    "escarceásemos": "escarcear",
    "escarceé": "escarcear",
    "escarceéis": "escarcear",
    "escarceó": "escarcear",
    "escarcha": "escarchar",
    "escarchaba": "escarchar",
    "escarchabais": "escarchar",
    "escarchaban": "escarchar",
    "escarchabas": "escarchar",
    "escarchad": "escarchar",
    "escarchada": "escarchar",
    "escarchadas": "escarchar",
    "escarchado": "escarchar",
    "escarchados": "escarchar",
    "escarchamos": "escarchar",
    "escarchan": "escarchar",
    "escarchando": "escarchar",
    "escarchara": "escarchar",
    "escarcharais": "escarchar",
    "escarcharan": "escarchar",
    "escarcharas": "escarchar",
    "escarchare": "escarchar",
    "escarchareis": "escarchar",
    "escarcharemos": "escarchar",
    "escarcharen": "escarchar",
    "escarchares": "escarchar",
    "escarcharon": "escarchar",
    "escarchará": "escarchar",
    "escarcharán": "escarchar",
    "escarcharás": "escarchar",
    "escarcharé": "escarchar",
    "escarcharéis": "escarchar",
    "escarcharía": "escarchar",
    "escarcharíais": "escarchar",
    "escarcharíamos": "escarchar",
    "escarcharían": "escarchar",
    "escarcharías": "escarchar",
    "escarchas": "escarchar",
    "escarchase": "escarchar",
    "escarchaseis": "escarchar",
    "escarchasen": "escarchar",
    "escarchases": "escarchar",
    "escarchaste": "escarchar",
    "escarchasteis": "escarchar",
    "escarche": "escarchar",
    "escarchemos": "escarchar",
    "escarchen": "escarchar",
    "escarches": "escarche",
    "escarcho": "escarchar",
    "escarchábamos": "escarchar",
    "escarcháis": "escarchar",
    "escarcháramos": "escarchar",
    "escarcháremos": "escarchar",
    "escarchásemos": "escarchar",
    "escarché": "escarchar",
    "escarchéis": "escarchar",
    "escarchó": "escarchar",
    "escarcinas": "escarcina",
    "escarcinazos": "escarcinazo",
    "escarcé": "escarzar",
    "escarcéis": "escarzar",
    "escarda": "escardar",
    "escardaba": "escardar",
    "escardabais": "escardar",
    "escardaban": "escardar",
    "escardabas": "escardar",
    "escardad": "escardar",
    "escardada": "escardar",
    "escardadas": "escardar",
    "escardaderas": "escardadera",
    "escardado": "escardar",
    "escardadoras": "escardadora",
    "escardadores": "escardador",
    "escardados": "escardar",
    "escardamos": "escardar",
    "escardan": "escardar",
    "escardando": "escardar",
    "escardara": "escardar",
    "escardarais": "escardar",
    "escardaran": "escardar",
    "escardaras": "escardar",
    "escardare": "escardar",
    "escardareis": "escardar",
    "escardaremos": "escardar",
    "escardaren": "escardar",
    "escardares": "escardar",
    "escardaron": "escardar",
    "escardará": "escardar",
    "escardarán": "escardar",
    "escardarás": "escardar",
    "escardaré": "escardar",
    "escardaréis": "escardar",
    "escardaría": "escardar",
    "escardaríais": "escardar",
    "escardaríamos": "escardar",
    "escardarían": "escardar",
    "escardarías": "escardar",
    "escardas": "escardar",
    "escardase": "escardar",
    "escardaseis": "escardar",
    "escardasen": "escardar",
    "escardases": "escardar",
    "escardaste": "escardar",
    "escardasteis": "escardar",
    "escarde": "escardar",
    "escardemos": "escardar",
    "escarden": "escardar",
    "escardes": "escardar",
    "escardilla": "escardillar",
    "escardillaba": "escardillar",
    "escardillabais": "escardillar",
    "escardillaban": "escardillar",
    "escardillabas": "escardillar",
    "escardillad": "escardillar",
    "escardillada": "escardillar",
    "escardilladas": "escardillar",
    "escardillado": "escardillar",
    "escardillados": "escardillar",
    "escardillamos": "escardillar",
    "escardillan": "escardillar",
    "escardillando": "escardillar",
    "escardillara": "escardillar",
    "escardillarais": "escardillar",
    "escardillaran": "escardillar",
    "escardillaras": "escardillar",
    "escardillare": "escardillar",
    "escardillareis": "escardillar",
    "escardillaremos": "escardillar",
    "escardillaren": "escardillar",
    "escardillares": "escardillar",
    "escardillaron": "escardillar",
    "escardillará": "escardillar",
    "escardillarán": "escardillar",
    "escardillarás": "escardillar",
    "escardillaré": "escardillar",
    "escardillaréis": "escardillar",
    "escardillaría": "escardillar",
    "escardillaríais": "escardillar",
    "escardillaríamos": "escardillar",
    "escardillarían": "escardillar",
    "escardillarías": "escardillar",
    "escardillas": "escardillar",
    "escardillase": "escardillar",
    "escardillaseis": "escardillar",
    "escardillasen": "escardillar",
    "escardillases": "escardillar",
    "escardillaste": "escardillar",
    "escardillasteis": "escardillar",
    "escardille": "escardillar",
    "escardillemos": "escardillar",
    "escardillen": "escardillar",
    "escardilles": "escardillar",
    "escardillo": "escardillar",
    "escardillos": "escardillo",
    "escardillábamos": "escardillar",
    "escardilláis": "escardillar",
    "escardilláramos": "escardillar",
    "escardilláremos": "escardillar",
    "escardillásemos": "escardillar",
    "escardillé": "escardillar",
    "escardilléis": "escardillar",
    "escardilló": "escardillar",
    "escardo": "escardar",
    "escardábamos": "escardar",
    "escardáis": "escardar",
    "escardáramos": "escardar",
    "escardáremos": "escardar",
    "escardásemos": "escardar",
    "escardé": "escardar",
    "escardéis": "escardar",
    "escardó": "escardar",
    "escaria": "escariar",
    "escariaba": "escariar",
    "escariabais": "escariar",
    "escariaban": "escariar",
    "escariabas": "escariar",
    "escariad": "escariar",
    "escariada": "escariar",
    "escariadas": "escariar",
    "escariado": "escariar",
    "escariadora": "escariador",
    "escariadoras": "escariador",
    "escariadores": "escariador",
    "escariados": "escariar",
    "escariamos": "escariar",
    "escarian": "escariar",
    "escariando": "escariar",
    "escariara": "escariar",
    "escariarais": "escariar",
    "escariaran": "escariar",
    "escariaras": "escariar",
    "escariare": "escariar",
    "escariareis": "escariar",
    "escariaremos": "escariar",
    "escariaren": "escariar",
    "escariares": "escariar",
    "escariaron": "escariar",
    "escariará": "escariar",
    "escariarán": "escariar",
    "escariarás": "escariar",
    "escariaré": "escariar",
    "escariaréis": "escariar",
    "escariaría": "escariar",
    "escariaríais": "escariar",
    "escariaríamos": "escariar",
    "escariarían": "escariar",
    "escariarías": "escariar",
    "escarias": "escariar",
    "escariase": "escariar",
    "escariaseis": "escariar",
    "escariasen": "escariar",
    "escariases": "escariar",
    "escariaste": "escariar",
    "escariasteis": "escariar",
    "escarice": "escarizar",
    "escaricemos": "escarizar",
    "escaricen": "escarizar",
    "escarices": "escarizar",
    "escaricé": "escarizar",
    "escaricéis": "escarizar",
    "escarie": "escariar",
    "escariemos": "escariar",
    "escarien": "escariar",
    "escaries": "escariar",
    "escarifica": "escarificar",
    "escarificaba": "escarificar",
    "escarificabais": "escarificar",
    "escarificaban": "escarificar",
    "escarificabas": "escarificar",
    "escarificaciones": "escarificación",
    "escarificad": "escarificar",
    "escarificada": "escarificar",
    "escarificadas": "escarificar",
    "escarificado": "escarificar",
    "escarificadora": "escarificador",
    "escarificadoras": "escarificadora",
    "escarificadores": "escarificador",
    "escarificados": "escarificar",
    "escarificamos": "escarificar",
    "escarifican": "escarificar",
    "escarificando": "escarificar",
    "escarificara": "escarificar",
    "escarificarais": "escarificar",
    "escarificaran": "escarificar",
    "escarificaras": "escarificar",
    "escarificare": "escarificar",
    "escarificareis": "escarificar",
    "escarificaremos": "escarificar",
    "escarificaren": "escarificar",
    "escarificares": "escarificar",
    "escarificaron": "escarificar",
    "escarificará": "escarificar",
    "escarificarán": "escarificar",
    "escarificarás": "escarificar",
    "escarificaré": "escarificar",
    "escarificaréis": "escarificar",
    "escarificaría": "escarificar",
    "escarificaríais": "escarificar",
    "escarificaríamos": "escarificar",
    "escarificarían": "escarificar",
    "escarificarías": "escarificar",
    "escarificas": "escarificar",
    "escarificase": "escarificar",
    "escarificaseis": "escarificar",
    "escarificasen": "escarificar",
    "escarificases": "escarificar",
    "escarificaste": "escarificar",
    "escarificasteis": "escarificar",
    "escarifico": "escarificar",
    "escarificábamos": "escarificar",
    "escarificáis": "escarificar",
    "escarificáramos": "escarificar",
    "escarificáremos": "escarificar",
    "escarificásemos": "escarificar",
    "escarificó": "escarificar",
    "escario": "escariar",
    "escariosa": "escarioso",
    "escariosas": "escarioso",
    "escariosos": "escarioso",
    "escarique": "escarificar",
    "escariquemos": "escarificar",
    "escariquen": "escarificar",
    "escariques": "escarificar",
    "escariqué": "escarificar",
    "escariquéis": "escarificar",
    "escariza": "escarizar",
    "escarizaba": "escarizar",
    "escarizabais": "escarizar",
    "escarizaban": "escarizar",
    "escarizabas": "escarizar",
    "escarizad": "escarizar",
    "escarizada": "escarizar",
    "escarizadas": "escarizar",
    "escarizado": "escarizar",
    "escarizados": "escarizar",
    "escarizamos": "escarizar",
    "escarizan": "escarizar",
    "escarizando": "escarizar",
    "escarizara": "escarizar",
    "escarizarais": "escarizar",
    "escarizaran": "escarizar",
    "escarizaras": "escarizar",
    "escarizare": "escarizar",
    "escarizareis": "escarizar",
    "escarizaremos": "escarizar",
    "escarizaren": "escarizar",
    "escarizares": "escarizar",
    "escarizaron": "escarizar",
    "escarizará": "escarizar",
    "escarizarán": "escarizar",
    "escarizarás": "escarizar",
    "escarizaré": "escarizar",
    "escarizaréis": "escarizar",
    "escarizaría": "escarizar",
    "escarizaríais": "escarizar",
    "escarizaríamos": "escarizar",
    "escarizarían": "escarizar",
    "escarizarías": "escarizar",
    "escarizas": "escarizar",
    "escarizase": "escarizar",
    "escarizaseis": "escarizar",
    "escarizasen": "escarizar",
    "escarizases": "escarizar",
    "escarizaste": "escarizar",
    "escarizasteis": "escarizar",
    "escarizo": "escarizar",
    "escarizábamos": "escarizar",
    "escarizáis": "escarizar",
    "escarizáramos": "escarizar",
    "escarizáremos": "escarizar",
    "escarizásemos": "escarizar",
    "escarizó": "escarizar",
    "escariábamos": "escariar",
    "escariáis": "escariar",
    "escariáramos": "escariar",
    "escariáremos": "escariar",
    "escariásemos": "escariar",
    "escarié": "escariar",
    "escariéis": "escariar",
    "escarió": "escariar",
    "escarladores": "escarlador",
    "escarlatas": "escarlata",
    "escarlatinas": "escarlatina",
    "escarmena": "escarmenar",
    "escarmenaba": "escarmenar",
    "escarmenabais": "escarmenar",
    "escarmenaban": "escarmenar",
    "escarmenabas": "escarmenar",
    "escarmenad": "escarmenar",
    "escarmenada": "escarmenar",
    "escarmenadas": "escarmenar",
    "escarmenado": "escarmenar",
    "escarmenadora": "escarmenador",
    "escarmenadoras": "escarmenador",
    "escarmenadores": "escarmenador",
    "escarmenados": "escarmenar",
    "escarmenamos": "escarmenar",
    "escarmenan": "escarmenar",
    "escarmenando": "escarmenar",
    "escarmenara": "escarmenar",
    "escarmenarais": "escarmenar",
    "escarmenaran": "escarmenar",
    "escarmenaras": "escarmenar",
    "escarmenare": "escarmenar",
    "escarmenareis": "escarmenar",
    "escarmenaremos": "escarmenar",
    "escarmenaren": "escarmenar",
    "escarmenares": "escarmenar",
    "escarmenaron": "escarmenar",
    "escarmenará": "escarmenar",
    "escarmenarán": "escarmenar",
    "escarmenarás": "escarmenar",
    "escarmenaré": "escarmenar",
    "escarmenaréis": "escarmenar",
    "escarmenaría": "escarmenar",
    "escarmenaríais": "escarmenar",
    "escarmenaríamos": "escarmenar",
    "escarmenarían": "escarmenar",
    "escarmenarías": "escarmenar",
    "escarmenas": "escarmenar",
    "escarmenase": "escarmenar",
    "escarmenaseis": "escarmenar",
    "escarmenasen": "escarmenar",
    "escarmenases": "escarmenar",
    "escarmenaste": "escarmenar",
    "escarmenasteis": "escarmenar",
    "escarmene": "escarmenar",
    "escarmenemos": "escarmenar",
    "escarmenen": "escarmenar",
    "escarmenes": "escarmenar",
    "escarmeno": "escarmenar",
    "escarmentaba": "escarmentar",
    "escarmentabais": "escarmentar",
    "escarmentaban": "escarmentar",
    "escarmentabas": "escarmentar",
    "escarmentad": "escarmentar",
    "escarmentada": "escarmentar",
    "escarmentadas": "escarmentar",
    "escarmentado": "escarmentar",
    "escarmentados": "escarmentar",
    "escarmentamos": "escarmentar",
    "escarmentando": "escarmentar",
    "escarmentara": "escarmentar",
    "escarmentarais": "escarmentar",
    "escarmentaran": "escarmentar",
    "escarmentaras": "escarmentar",
    "escarmentare": "escarmentar",
    "escarmentareis": "escarmentar",
    "escarmentaremos": "escarmentar",
    "escarmentaren": "escarmentar",
    "escarmentares": "escarmentar",
    "escarmentaron": "escarmentar",
    "escarmentará": "escarmentar",
    "escarmentarán": "escarmentar",
    "escarmentarás": "escarmentar",
    "escarmentaré": "escarmentar",
    "escarmentaréis": "escarmentar",
    "escarmentaría": "escarmentar",
    "escarmentaríais": "escarmentar",
    "escarmentaríamos": "escarmentar",
    "escarmentarían": "escarmentar",
    "escarmentarías": "escarmentar",
    "escarmentase": "escarmentar",
    "escarmentaseis": "escarmentar",
    "escarmentasen": "escarmentar",
    "escarmentases": "escarmentar",
    "escarmentaste": "escarmentar",
    "escarmentasteis": "escarmentar",
    "escarmentemos": "escarmentar",
    "escarmentábamos": "escarmentar",
    "escarmentáis": "escarmentar",
    "escarmentáramos": "escarmentar",
    "escarmentáremos": "escarmentar",
    "escarmentásemos": "escarmentar",
    "escarmenté": "escarmentar",
    "escarmentéis": "escarmentar",
    "escarmentó": "escarmentar",
    "escarmenábamos": "escarmenar",
    "escarmenáis": "escarmenar",
    "escarmenáramos": "escarmenar",
    "escarmenáremos": "escarmenar",
    "escarmenásemos": "escarmenar",
    "escarmené": "escarmenar",
    "escarmenéis": "escarmenar",
    "escarmenó": "escarmenar",
    "escarmienta": "escarmentar",
    "escarmientan": "escarmentar",
    "escarmientas": "escarmentar",
    "escarmiente": "escarmentar",
    "escarmienten": "escarmentar",
    "escarmientes": "escarmentar",
    "escarmiento": "escarmentar",
    "escarmientos": "escarmiento",
    "escarnece": "escarnecer",
    "escarneced": "escarnecer",
    "escarnecedora": "escarnecedor",
    "escarnecedoras": "escarnecedor",
    "escarnecedores": "escarnecedor",
    "escarnecemos": "escarnecer",
    "escarnecen": "escarnecer",
    "escarneceremos": "escarnecer",
    "escarnecerá": "escarnecer",
    "escarnecerán": "escarnecer",
    "escarnecerás": "escarnecer",
    "escarneceré": "escarnecer",
    "escarneceréis": "escarnecer",
    "escarnecería": "escarnecer",
    "escarneceríais": "escarnecer",
    "escarneceríamos": "escarnecer",
    "escarnecerían": "escarnecer",
    "escarnecerías": "escarnecer",
    "escarneces": "escarnecer",
    "escarnecida": "escarnecer",
    "escarnecidas": "escarnecer",
    "escarnecido": "escarnecer",
    "escarnecidos": "escarnecer",
    "escarneciendo": "escarnecer",
    "escarneciera": "escarnecer",
    "escarnecierais": "escarnecer",
    "escarnecieran": "escarnecer",
    "escarnecieras": "escarnecer",
    "escarneciere": "escarnecer",
    "escarneciereis": "escarnecer",
    "escarnecieren": "escarnecer",
    "escarnecieres": "escarnecer",
    "escarnecieron": "escarnecer",
    "escarneciese": "escarnecer",
    "escarnecieseis": "escarnecer",
    "escarneciesen": "escarnecer",
    "escarnecieses": "escarnecer",
    "escarnecimientos": "escarnecimiento",
    "escarnecimos": "escarnecer",
    "escarneciste": "escarnecer",
    "escarnecisteis": "escarnecer",
    "escarneciéramos": "escarnecer",
    "escarneciéremos": "escarnecer",
    "escarneciésemos": "escarnecer",
    "escarneció": "escarnecer",
    "escarnecéis": "escarnecer",
    "escarnecí": "escarnecer",
    "escarnecía": "escarnecer",
    "escarnecíais": "escarnecer",
    "escarnecíamos": "escarnecer",
    "escarnecían": "escarnecer",
    "escarnecías": "escarnecer",
    "escarnezca": "escarnecer",
    "escarnezcamos": "escarnecer",
    "escarnezcan": "escarnecer",
    "escarnezcas": "escarnecer",
    "escarnezco": "escarnecer",
    "escarnezcáis": "escarnecer",
    "escarnios": "escarnio",
    "escarola": "escarolar",
    "escarolaba": "escarolar",
    "escarolabais": "escarolar",
    "escarolaban": "escarolar",
    "escarolabas": "escarolar",
    "escarolad": "escarolar",
    "escarolada": "escarolar",
    "escaroladas": "escarolar",
    "escarolado": "escarolar",
    "escarolados": "escarolar",
    "escarolamos": "escarolar",
    "escarolan": "escarolar",
    "escarolando": "escarolar",
    "escarolara": "escarolar",
    "escarolarais": "escarolar",
    "escarolaran": "escarolar",
    "escarolaras": "escarolar",
    "escarolare": "escarolar",
    "escarolareis": "escarolar",
    "escarolaremos": "escarolar",
    "escarolaren": "escarolar",
    "escarolares": "escarolar",
    "escarolaron": "escarolar",
    "escarolará": "escarolar",
    "escarolarán": "escarolar",
    "escarolarás": "escarolar",
    "escarolaré": "escarolar",
    "escarolaréis": "escarolar",
    "escarolaría": "escarolar",
    "escarolaríais": "escarolar",
    "escarolaríamos": "escarolar",
    "escarolarían": "escarolar",
    "escarolarías": "escarolar",
    "escarolas": "escarolar",
    "escarolase": "escarolar",
    "escarolaseis": "escarolar",
    "escarolasen": "escarolar",
    "escarolases": "escarolar",
    "escarolaste": "escarolar",
    "escarolasteis": "escarolar",
    "escarole": "escarolar",
    "escarolemos": "escarolar",
    "escarolen": "escarolar",
    "escaroles": "escarolar",
    "escarolo": "escarolar",
    "escarolábamos": "escarolar",
    "escaroláis": "escarolar",
    "escaroláramos": "escarolar",
    "escaroláremos": "escarolar",
    "escarolásemos": "escarolar",
    "escarolé": "escarolar",
    "escaroléis": "escarolar",
    "escaroló": "escarolar",
    "escarpa": "escarpar",
    "escarpaba": "escarpar",
    "escarpabais": "escarpar",
    "escarpaban": "escarpar",
    "escarpabas": "escarpar",
    "escarpad": "escarpar",
    "escarpada": "escarpar",
    "escarpadas": "escarpar",
    "escarpado": "escarpar",
    "escarpados": "escarpar",
    "escarpaduras": "escarpadura",
    "escarpamos": "escarpar",
    "escarpan": "escarpar",
    "escarpando": "escarpar",
    "escarpara": "escarpar",
    "escarparais": "escarpar",
    "escarparan": "escarpar",
    "escarparas": "escarpar",
    "escarpare": "escarpar",
    "escarpareis": "escarpar",
    "escarparemos": "escarpar",
    "escarparen": "escarpar",
    "escarpares": "escarpar",
    "escarparon": "escarpar",
    "escarpará": "escarpar",
    "escarparán": "escarpar",
    "escarparás": "escarpar",
    "escarparé": "escarpar",
    "escarparéis": "escarpar",
    "escarparía": "escarpar",
    "escarparíais": "escarpar",
    "escarparíamos": "escarpar",
    "escarparían": "escarpar",
    "escarparías": "escarpar",
    "escarpas": "escarpar",
    "escarpase": "escarpar",
    "escarpaseis": "escarpar",
    "escarpasen": "escarpar",
    "escarpases": "escarpar",
    "escarpaste": "escarpar",
    "escarpasteis": "escarpar",
    "escarpe": "escarpar",
    "escarpelos": "escarpelo",
    "escarpemos": "escarpar",
    "escarpen": "escarpar",
    "escarpes": "escarpe",
    "escarpiadores": "escarpiador",
    "escarpias": "escarpia",
    "escarpidores": "escarpidor",
    "escarpines": "escarpín",
    "escarpo": "escarpar",
    "escarpábamos": "escarpar",
    "escarpáis": "escarpar",
    "escarpáramos": "escarpar",
    "escarpáremos": "escarpar",
    "escarpásemos": "escarpar",
    "escarpé": "escarpar",
    "escarpéis": "escarpar",
    "escarpínes": "escarpín",
    "escarpó": "escarpar",
    "escarrancha": "escarranchar",
    "escarranchaba": "escarranchar",
    "escarranchabais": "escarranchar",
    "escarranchaban": "escarranchar",
    "escarranchabas": "escarranchar",
    "escarranchad": "escarranchar",
    "escarranchada": "escarranchar",
    "escarranchadas": "escarranchar",
    "escarranchado": "escarranchar",
    "escarranchados": "escarranchar",
    "escarranchamos": "escarranchar",
    "escarranchan": "escarranchar",
    "escarranchando": "escarranchar",
    "escarranchara": "escarranchar",
    "escarrancharais": "escarranchar",
    "escarrancharan": "escarranchar",
    "escarrancharas": "escarranchar",
    "escarranchare": "escarranchar",
    "escarranchareis": "escarranchar",
    "escarrancharemos": "escarranchar",
    "escarrancharen": "escarranchar",
    "escarranchares": "escarranchar",
    "escarrancharon": "escarranchar",
    "escarranchará": "escarranchar",
    "escarrancharán": "escarranchar",
    "escarrancharás": "escarranchar",
    "escarrancharé": "escarranchar",
    "escarrancharéis": "escarranchar",
    "escarrancharía": "escarranchar",
    "escarrancharíais": "escarranchar",
    "escarrancharíamos": "escarranchar",
    "escarrancharían": "escarranchar",
    "escarrancharías": "escarranchar",
    "escarranchas": "escarranchar",
    "escarranchase": "escarranchar",
    "escarranchaseis": "escarranchar",
    "escarranchasen": "escarranchar",
    "escarranchases": "escarranchar",
    "escarranchaste": "escarranchar",
    "escarranchasteis": "escarranchar",
    "escarranche": "escarranchar",
    "escarranchemos": "escarranchar",
    "escarranchen": "escarranchar",
    "escarranches": "escarranchar",
    "escarrancho": "escarranchar",
    "escarranchábamos": "escarranchar",
    "escarrancháis": "escarranchar",
    "escarrancháramos": "escarranchar",
    "escarrancháremos": "escarranchar",
    "escarranchásemos": "escarranchar",
    "escarranché": "escarranchar",
    "escarranchéis": "escarranchar",
    "escarranchó": "escarranchar",
    "escartivanas": "escartivana",
    "escarza": "escarzar",
    "escarzaba": "escarzar",
    "escarzabais": "escarzar",
    "escarzaban": "escarzar",
    "escarzabas": "escarzar",
    "escarzad": "escarzar",
    "escarzada": "escarzar",
    "escarzadas": "escarzar",
    "escarzado": "escarzar",
    "escarzadores": "escarzador",
    "escarzados": "escarzar",
    "escarzamos": "escarzar",
    "escarzan": "escarzar",
    "escarzana": "escarzano",
    "escarzanas": "escarzano",
    "escarzando": "escarzar",
    "escarzanos": "escarzano",
    "escarzara": "escarzar",
    "escarzarais": "escarzar",
    "escarzaran": "escarzar",
    "escarzaras": "escarzar",
    "escarzare": "escarzar",
    "escarzareis": "escarzar",
    "escarzaremos": "escarzar",
    "escarzaren": "escarzar",
    "escarzares": "escarzar",
    "escarzaron": "escarzar",
    "escarzará": "escarzar",
    "escarzarán": "escarzar",
    "escarzarás": "escarzar",
    "escarzaré": "escarzar",
    "escarzaréis": "escarzar",
    "escarzaría": "escarzar",
    "escarzaríais": "escarzar",
    "escarzaríamos": "escarzar",
    "escarzarían": "escarzar",
    "escarzarías": "escarzar",
    "escarzas": "escarzar",
    "escarzase": "escarzar",
    "escarzaseis": "escarzar",
    "escarzasen": "escarzar",
    "escarzases": "escarzar",
    "escarzaste": "escarzar",
    "escarzasteis": "escarzar",
    "escarzo": "escarzar",
    "escarzos": "escarzo",
    "escarzábamos": "escarzar",
    "escarzáis": "escarzar",
    "escarzáramos": "escarzar",
    "escarzáremos": "escarzar",
    "escarzásemos": "escarzar",
    "escarzó": "escarzar",
    "escasa": "escaso",
    "escasas": "escaso",
    "escasea": "escasear",
    "escaseaba": "escasear",
    "escaseabais": "escasear",
    "escaseaban": "escasear",
    "escaseabas": "escasear",
    "escasead": "escasear",
    "escaseada": "escasear",
    "escaseadas": "escasear",
    "escaseado": "escasear",
    "escaseados": "escasear",
    "escaseamos": "escasear",
    "escasean": "escasear",
    "escaseando": "escasear",
    "escaseara": "escasear",
    "escasearais": "escasear",
    "escasearan": "escasear",
    "escasearas": "escasear",
    "escaseare": "escasear",
    "escaseareis": "escasear",
    "escasearemos": "escasear",
    "escasearen": "escasear",
    "escaseares": "escasear",
    "escasearon": "escasear",
    "escaseará": "escasear",
    "escasearán": "escasear",
    "escasearás": "escasear",
    "escasearé": "escasear",
    "escasearéis": "escasear",
    "escasearía": "escasear",
    "escasearíais": "escasear",
    "escasearíamos": "escasear",
    "escasearían": "escasear",
    "escasearías": "escasear",
    "escaseas": "escasear",
    "escasease": "escasear",
    "escaseaseis": "escasear",
    "escaseasen": "escasear",
    "escaseases": "escasear",
    "escaseaste": "escasear",
    "escaseasteis": "escasear",
    "escaseces": "escasez",
    "escasee": "escasear",
    "escaseemos": "escasear",
    "escaseen": "escasear",
    "escasees": "escasear",
    "escaseo": "escasear",
    "escasera": "escasero",
    "escaseras": "escasero",
    "escaseros": "escasero",
    "escaseábamos": "escasear",
    "escaseáis": "escasear",
    "escaseáramos": "escasear",
    "escaseáremos": "escasear",
    "escaseásemos": "escasear",
    "escaseé": "escasear",
    "escaseéis": "escasear",
    "escaseó": "escasear",
    "escasos": "escaso",
    "escatima": "escatimar",
    "escatimaba": "escatimar",
    "escatimabais": "escatimar",
    "escatimaban": "escatimar",
    "escatimabas": "escatimar",
    "escatimad": "escatimar",
    "escatimada": "escatimar",
    "escatimadas": "escatimar",
    "escatimado": "escatimar",
    "escatimados": "escatimar",
    "escatimamos": "escatimar",
    "escatiman": "escatimar",
    "escatimando": "escatimar",
    "escatimara": "escatimar",
    "escatimarais": "escatimar",
    "escatimaran": "escatimar",
    "escatimaras": "escatimar",
    "escatimare": "escatimar",
    "escatimareis": "escatimar",
    "escatimaremos": "escatimar",
    "escatimaren": "escatimar",
    "escatimares": "escatimar",
    "escatimaron": "escatimar",
    "escatimará": "escatimar",
    "escatimarán": "escatimar",
    "escatimarás": "escatimar",
    "escatimaré": "escatimar",
    "escatimaréis": "escatimar",
    "escatimaría": "escatimar",
    "escatimaríais": "escatimar",
    "escatimaríamos": "escatimar",
    "escatimarían": "escatimar",
    "escatimarías": "escatimar",
    "escatimas": "escatimar",
    "escatimase": "escatimar",
    "escatimaseis": "escatimar",
    "escatimasen": "escatimar",
    "escatimases": "escatimar",
    "escatimaste": "escatimar",
    "escatimasteis": "escatimar",
    "escatime": "escatimar",
    "escatimemos": "escatimar",
    "escatimen": "escatimar",
    "escatimes": "escatimar",
    "escatimo": "escatimar",
    "escatimábamos": "escatimar",
    "escatimáis": "escatimar",
    "escatimáramos": "escatimar",
    "escatimáremos": "escatimar",
    "escatimásemos": "escatimar",
    "escatimé": "escatimar",
    "escatiméis": "escatimar",
    "escatimó": "escatimar",
    "escatofagias": "escatofagia",
    "escatoles": "escatol",
    "escatologías": "escatología",
    "escatológica": "escatológico",
    "escatológicas": "escatológico",
    "escatológicos": "escatológico",
    "escatófaga": "escatófago",
    "escatófagas": "escatófago",
    "escatófagos": "escatófago",
    "escatófila": "escatófilo",
    "escatófilas": "escatófilo",
    "escatófilos": "escatófilo",
    "escaupiles": "escaupil",
    "escavadora": "escavador",
    "escavadoras": "escavador",
    "escavadores": "escavador",
    "escayola": "escayolar",
    "escayolaba": "escayolar",
    "escayolabais": "escayolar",
    "escayolaban": "escayolar",
    "escayolabas": "escayolar",
    "escayolad": "escayolar",
    "escayolada": "escayolar",
    "escayoladas": "escayolar",
    "escayolado": "escayolar",
    "escayolados": "escayolar",
    "escayolamos": "escayolar",
    "escayolan": "escayolar",
    "escayolando": "escayolar",
    "escayolara": "escayolar",
    "escayolarais": "escayolar",
    "escayolaran": "escayolar",
    "escayolaras": "escayolar",
    "escayolare": "escayolar",
    "escayolareis": "escayolar",
    "escayolaremos": "escayolar",
    "escayolaren": "escayolar",
    "escayolares": "escayolar",
    "escayolaron": "escayolar",
    "escayolará": "escayolar",
    "escayolarán": "escayolar",
    "escayolarás": "escayolar",
    "escayolaré": "escayolar",
    "escayolaréis": "escayolar",
    "escayolaría": "escayolar",
    "escayolaríais": "escayolar",
    "escayolaríamos": "escayolar",
    "escayolarían": "escayolar",
    "escayolarías": "escayolar",
    "escayolas": "escayolar",
    "escayolase": "escayolar",
    "escayolaseis": "escayolar",
    "escayolasen": "escayolar",
    "escayolases": "escayolar",
    "escayolaste": "escayolar",
    "escayolasteis": "escayolar",
    "escayole": "escayolar",
    "escayolemos": "escayolar",
    "escayolen": "escayolar",
    "escayoles": "escayolar",
    "escayolistas": "escayolista",
    "escayolo": "escayolar",
    "escayolábamos": "escayolar",
    "escayoláis": "escayolar",
    "escayoláramos": "escayolar",
    "escayoláremos": "escayolar",
    "escayolásemos": "escayolar",
    "escayolé": "escayolar",
    "escayoléis": "escayolar",
    "escayoló": "escayolar",
    "escañas": "escaña",
    "escaños": "escaño",
    "escañuelos": "escañuelo",
    "escenarios": "escenario",
    "escenas": "escena",
    "escenifica": "escenificar",
    "escenificaba": "escenificar",
    "escenificabais": "escenificar",
    "escenificaban": "escenificar",
    "escenificabas": "escenificar",
    "escenificables": "escenificable",
    "escenificaciones": "escenificación",
    "escenificad": "escenificar",
    "escenificada": "escenificar",
    "escenificadas": "escenificar",
    "escenificado": "escenificar",
    "escenificados": "escenificar",
    "escenificamos": "escenificar",
    "escenifican": "escenificar",
    "escenificando": "escenificar",
    "escenificara": "escenificar",
    "escenificarais": "escenificar",
    "escenificaran": "escenificar",
    "escenificaras": "escenificar",
    "escenificare": "escenificar",
    "escenificareis": "escenificar",
    "escenificaremos": "escenificar",
    "escenificaren": "escenificar",
    "escenificares": "escenificar",
    "escenificaron": "escenificar",
    "escenificará": "escenificar",
    "escenificarán": "escenificar",
    "escenificarás": "escenificar",
    "escenificaré": "escenificar",
    "escenificaréis": "escenificar",
    "escenificaría": "escenificar",
    "escenificaríais": "escenificar",
    "escenificaríamos": "escenificar",
    "escenificarían": "escenificar",
    "escenificarías": "escenificar",
    "escenificas": "escenificar",
    "escenificase": "escenificar",
    "escenificaseis": "escenificar",
    "escenificasen": "escenificar",
    "escenificases": "escenificar",
    "escenificaste": "escenificar",
    "escenificasteis": "escenificar",
    "escenifico": "escenificar",
    "escenificábamos": "escenificar",
    "escenificáis": "escenificar",
    "escenificáramos": "escenificar",
    "escenificáremos": "escenificar",
    "escenificásemos": "escenificar",
    "escenificó": "escenificar",
    "escenifique": "escenificar",
    "escenifiquemos": "escenificar",
    "escenifiquen": "escenificar",
    "escenifiques": "escenificar",
    "escenifiqué": "escenificar",
    "escenifiquéis": "escenificar",
    "escenografías": "escenografía",
    "escenográfica": "escenográfico",
    "escenográficas": "escenográfico",
    "escenográficos": "escenográfico",
    "escenotécnicas": "escenotécnica",
    "escenógrafa": "escenógrafo",
    "escenógrafas": "escenógrafo",
    "escenógrafos": "escenógrafo",
    "escepticismos": "escepticismo",
    "escifos": "escifo",
    "escifozoos": "escifozoo",
    "esciliorrínidos": "esciliorrínido",
    "escincos": "escinco",
    "escinda": "escindir",
    "escindamos": "escindir",
    "escindan": "escindir",
    "escindas": "escindir",
    "escinde": "escindir",
    "escinden": "escindir",
    "escindes": "escindir",
    "escindibles": "escindible",
    "escindid": "escindir",
    "escindida": "escindir",
    "escindidas": "escindir",
    "escindido": "escindir",
    "escindidos": "escindir",
    "escindiendo": "escindir",
    "escindiera": "escindir",
    "escindierais": "escindir",
    "escindieran": "escindir",
    "escindieras": "escindir",
    "escindiere": "escindir",
    "escindiereis": "escindir",
    "escindieren": "escindir",
    "escindieres": "escindir",
    "escindieron": "escindir",
    "escindiese": "escindir",
    "escindieseis": "escindir",
    "escindiesen": "escindir",
    "escindieses": "escindir",
    "escindimos": "escindir",
    "escindiremos": "escindir",
    "escindirá": "escindir",
    "escindirán": "escindir",
    "escindirás": "escindir",
    "escindiré": "escindir",
    "escindiréis": "escindir",
    "escindiría": "escindir",
    "escindiríais": "escindir",
    "escindiríamos": "escindir",
    "escindirían": "escindir",
    "escindirías": "escindir",
    "escindiste": "escindir",
    "escindisteis": "escindir",
    "escindiéramos": "escindir",
    "escindiéremos": "escindir",
    "escindiésemos": "escindir",
    "escindió": "escindir",
    "escindo": "escindir",
    "escindáis": "escindir",
    "escindí": "escindir",
    "escindía": "escindir",
    "escindíais": "escindir",
    "escindíamos": "escindir",
    "escindían": "escindir",
    "escindías": "escindir",
    "escindís": "escindir",
    "escintilaciones": "escintilación",
    "esciográfica": "esciográfico",
    "esciográficas": "esciográfico",
    "esciográficos": "esciográfico",
    "escirros": "escirro",
    "escisiones": "escisión",
    "escitamínea": "escitamíneo",
    "escitamíneas": "escitamíneo",
    "escitamíneos": "escitamíneo",
    "escitas": "escita",
    "esciénidos": "esciénido",
    "esciófila": "esciófilo",
    "esciófilas": "esciófilo",
    "esciófilos": "esciófilo",
    "esciúridos": "esciúrido",
    "esclarece": "esclarecer",
    "esclareced": "esclarecer",
    "esclarecedora": "esclarecedor",
    "esclarecedoras": "esclarecedor",
    "esclarecedores": "esclarecedor",
    "esclarecemos": "esclarecer",
    "esclarecen": "esclarecer",
    "esclareceremos": "esclarecer",
    "esclarecerá": "esclarecer",
    "esclarecerán": "esclarecer",
    "esclarecerás": "esclarecer",
    "esclareceré": "esclarecer",
    "esclareceréis": "esclarecer",
    "esclarecería": "esclarecer",
    "esclareceríais": "esclarecer",
    "esclareceríamos": "esclarecer",
    "esclarecerían": "esclarecer",
    "esclarecerías": "esclarecer",
    "esclareces": "esclarecer",
    "esclarecida": "esclarecer",
    "esclarecidas": "esclarecer",
    "esclarecido": "esclarecer",
    "esclarecidos": "esclarecer",
    "esclareciendo": "esclarecer",
    "esclareciera": "esclarecer",
    "esclarecierais": "esclarecer",
    "esclarecieran": "esclarecer",
    "esclarecieras": "esclarecer",
    "esclareciere": "esclarecer",
    "esclareciereis": "esclarecer",
    "esclarecieren": "esclarecer",
    "esclarecieres": "esclarecer",
    "esclarecieron": "esclarecer",
    "esclareciese": "esclarecer",
    "esclarecieseis": "esclarecer",
    "esclareciesen": "esclarecer",
    "esclarecieses": "esclarecer",
    "esclarecimientos": "esclarecimiento",
    "esclarecimos": "esclarecer",
    "esclareciste": "esclarecer",
    "esclarecisteis": "esclarecer",
    "esclareciéramos": "esclarecer",
    "esclareciéremos": "esclarecer",
    "esclareciésemos": "esclarecer",
    "esclareció": "esclarecer",
    "esclarecéis": "esclarecer",
    "esclarecí": "esclarecer",
    "esclarecía": "esclarecer",
    "esclarecíais": "esclarecer",
    "esclarecíamos": "esclarecer",
    "esclarecían": "esclarecer",
    "esclarecías": "esclarecer",
    "esclarezca": "esclarecer",
    "esclarezcamos": "esclarecer",
    "esclarezcan": "esclarecer",
    "esclarezcas": "esclarecer",
    "esclarezco": "esclarecer",
    "esclarezcáis": "esclarecer",
    "esclava": "esclavo",
    "esclavas": "esclavo",
    "esclavice": "esclavizar",
    "esclavicemos": "esclavizar",
    "esclavicen": "esclavizar",
    "esclavices": "esclavizar",
    "esclavicé": "esclavizar",
    "esclavicéis": "esclavizar",
    "esclavinas": "esclavina",
    "esclavismos": "esclavismo",
    "esclavistas": "esclavista",
    "esclavitudes": "esclavitud",
    "esclaviza": "esclavizar",
    "esclavizaba": "esclavizar",
    "esclavizabais": "esclavizar",
    "esclavizaban": "esclavizar",
    "esclavizabas": "esclavizar",
    "esclavizaciones": "esclavización",
    "esclavizad": "esclavizar",
    "esclavizada": "esclavizar",
    "esclavizadas": "esclavizar",
    "esclavizado": "esclavizar",
    "esclavizados": "esclavizar",
    "esclavizamos": "esclavizar",
    "esclavizan": "esclavizar",
    "esclavizando": "esclavizar",
    "esclavizara": "esclavizar",
    "esclavizarais": "esclavizar",
    "esclavizaran": "esclavizar",
    "esclavizaras": "esclavizar",
    "esclavizare": "esclavizar",
    "esclavizareis": "esclavizar",
    "esclavizaremos": "esclavizar",
    "esclavizaren": "esclavizar",
    "esclavizares": "esclavizar",
    "esclavizaron": "esclavizar",
    "esclavizará": "esclavizar",
    "esclavizarán": "esclavizar",
    "esclavizarás": "esclavizar",
    "esclavizaré": "esclavizar",
    "esclavizaréis": "esclavizar",
    "esclavizaría": "esclavizar",
    "esclavizaríais": "esclavizar",
    "esclavizaríamos": "esclavizar",
    "esclavizarían": "esclavizar",
    "esclavizarías": "esclavizar",
    "esclavizas": "esclavizar",
    "esclavizase": "esclavizar",
    "esclavizaseis": "esclavizar",
    "esclavizasen": "esclavizar",
    "esclavizases": "esclavizar",
    "esclavizaste": "esclavizar",
    "esclavizasteis": "esclavizar",
    "esclavizo": "esclavizar",
    "esclavizábamos": "esclavizar",
    "esclavizáis": "esclavizar",
    "esclavizáramos": "esclavizar",
    "esclavizáremos": "esclavizar",
    "esclavizásemos": "esclavizar",
    "esclavizó": "esclavizar",
    "esclavona": "esclavón",
    "esclavonas": "esclavón",
    "esclavones": "esclavón",
    "esclavos": "esclavo",
    "esclerales": "escleral",
    "escleras": "esclera",
    "esclerectasias": "esclerectasia",
    "esclerectoiridectomías": "esclerectoiridectomía",
    "esclerectomías": "esclerectomía",
    "esclereidas": "esclereida",
    "escleremas": "esclerema",
    "esclerocios": "esclerocio",
    "escleroconjuntivales": "escleroconjuntival",
    "esclerocorneales": "esclerocorneal",
    "esclerodermias": "esclerodermia",
    "escleroftalmías": "escleroftalmía",
    "escleromalacias": "escleromalacia",
    "esclerométrica": "esclerométrico",
    "esclerométricas": "esclerométrico",
    "esclerométricos": "esclerométrico",
    "esclerosa": "escleroso",
    "esclerosantes": "esclerosante",
    "esclerosas": "escleroso",
    "esclerosos": "escleroso",
    "esclerostomías": "esclerostomía",
    "esclerotomías": "esclerotomía",
    "esclerénquimas": "esclerénquima",
    "esclerófila": "esclerófilo",
    "esclerófilas": "esclerófilo",
    "esclerófilos": "esclerófilo",
    "esclerómetros": "esclerómetro",
    "esclerósica": "esclerósico",
    "esclerósicas": "esclerósico",
    "esclerósicos": "esclerósico",
    "esclerótica": "esclerótico",
    "escleróticas": "esclerótico",
    "escleróticos": "esclerótico",
    "esclusadas": "esclusada",
    "esclusas": "esclusa",
    "esclusera": "esclusero",
    "escluseras": "esclusero",
    "escluseros": "esclusero",
    "escoas": "escoa",
    "escobadas": "escobada",
    "escobajos": "escobajo",
    "escobas": "escoba",
    "escobazos": "escobazo",
    "escobenes": "escobén",
    "escobera": "escobero",
    "escoberas": "escobero",
    "escoberos": "escobero",
    "escobetas": "escobeta",
    "escobilla": "escobillar",
    "escobillaba": "escobillar",
    "escobillabais": "escobillar",
    "escobillaban": "escobillar",
    "escobillabas": "escobillar",
    "escobillad": "escobillar",
    "escobillada": "escobillar",
    "escobilladas": "escobillar",
    "escobillado": "escobillar",
    "escobillados": "escobillar",
    "escobillamos": "escobillar",
    "escobillan": "escobillar",
    "escobillando": "escobillar",
    "escobillara": "escobillar",
    "escobillarais": "escobillar",
    "escobillaran": "escobillar",
    "escobillaras": "escobillar",
    "escobillare": "escobillar",
    "escobillareis": "escobillar",
    "escobillaremos": "escobillar",
    "escobillaren": "escobillar",
    "escobillares": "escobillar",
    "escobillaron": "escobillar",
    "escobillará": "escobillar",
    "escobillarán": "escobillar",
    "escobillarás": "escobillar",
    "escobillaré": "escobillar",
    "escobillaréis": "escobillar",
    "escobillaría": "escobillar",
    "escobillaríais": "escobillar",
    "escobillaríamos": "escobillar",
    "escobillarían": "escobillar",
    "escobillarías": "escobillar",
    "escobillas": "escobillar",
    "escobillase": "escobillar",
    "escobillaseis": "escobillar",
    "escobillasen": "escobillar",
    "escobillases": "escobillar",
    "escobillaste": "escobillar",
    "escobillasteis": "escobillar",
    "escobille": "escobillar",
    "escobillemos": "escobillar",
    "escobillen": "escobillar",
    "escobilles": "escobillar",
    "escobillo": "escobillar",
    "escobillones": "escobillón",
    "escobillábamos": "escobillar",
    "escobilláis": "escobillar",
    "escobilláramos": "escobillar",
    "escobilláremos": "escobillar",
    "escobillásemos": "escobillar",
    "escobillé": "escobillar",
    "escobilléis": "escobillar",
    "escobilló": "escobillar",
    "escobinas": "escobina",
    "escobones": "escobón",
    "escobos": "escobo",
    "escoced": "escocer",
    "escoceduras": "escocedura",
    "escocemos": "escocer",
    "escoceremos": "escocer",
    "escocerá": "escocer",
    "escocerán": "escocer",
    "escocerás": "escocer",
    "escoceré": "escocer",
    "escoceréis": "escocer",
    "escocería": "escocer",
    "escoceríais": "escocer",
    "escoceríamos": "escocer",
    "escocerían": "escocer",
    "escocerías": "escocer",
    "escocesa": "escocés",
    "escocesas": "escocés",
    "escoceses": "escocés",
    "escocida": "escocer",
    "escocidas": "escocer",
    "escocido": "escocer",
    "escocidos": "escocer",
    "escociendo": "escocer",
    "escociera": "escocer",
    "escocierais": "escocer",
    "escocieran": "escocer",
    "escocieras": "escocer",
    "escociere": "escocer",
    "escociereis": "escocer",
    "escocieren": "escocer",
    "escocieres": "escocer",
    "escocieron": "escocer",
    "escociese": "escocer",
    "escocieseis": "escocer",
    "escociesen": "escocer",
    "escocieses": "escocer",
    "escocimientos": "escocimiento",
    "escocimos": "escocer",
    "escociste": "escocer",
    "escocisteis": "escocer",
    "escociéramos": "escocer",
    "escociéremos": "escocer",
    "escociésemos": "escocer",
    "escoció": "escocer",
    "escocéis": "escocer",
    "escocésa": "escocés",
    "escocésas": "escocés",
    "escocéses": "escocés",
    "escocí": "escocer",
    "escocía": "escocer",
    "escocíais": "escocer",
    "escocíamos": "escocer",
    "escocían": "escocer",
    "escocías": "escocer",
    "escoda": "escodar",
    "escodaba": "escodar",
    "escodabais": "escodar",
    "escodaban": "escodar",
    "escodabas": "escodar",
    "escodad": "escodar",
    "escodada": "escodar",
    "escodadas": "escodar",
    "escodaderos": "escodadero",
    "escodado": "escodar",
    "escodados": "escodar",
    "escodamos": "escodar",
    "escodan": "escodar",
    "escodando": "escodar",
    "escodara": "escodar",
    "escodarais": "escodar",
    "escodaran": "escodar",
    "escodaras": "escodar",
    "escodare": "escodar",
    "escodareis": "escodar",
    "escodaremos": "escodar",
    "escodaren": "escodar",
    "escodares": "escodar",
    "escodaron": "escodar",
    "escodará": "escodar",
    "escodarán": "escodar",
    "escodarás": "escodar",
    "escodaré": "escodar",
    "escodaréis": "escodar",
    "escodaría": "escodar",
    "escodaríais": "escodar",
    "escodaríamos": "escodar",
    "escodarían": "escodar",
    "escodarías": "escodar",
    "escodas": "escodar",
    "escodase": "escodar",
    "escodaseis": "escodar",
    "escodasen": "escodar",
    "escodases": "escodar",
    "escodaste": "escodar",
    "escodasteis": "escodar",
    "escode": "escodar",
    "escodemos": "escodar",
    "escoden": "escodar",
    "escodes": "escodar",
    "escodo": "escodar",
    "escodábamos": "escodar",
    "escodáis": "escodar",
    "escodáramos": "escodar",
    "escodáremos": "escodar",
    "escodásemos": "escodar",
    "escodé": "escodar",
    "escodéis": "escodar",
    "escodó": "escodar",
    "escofina": "escofinar",
    "escofinaba": "escofinar",
    "escofinabais": "escofinar",
    "escofinaban": "escofinar",
    "escofinabas": "escofinar",
    "escofinad": "escofinar",
    "escofinada": "escofinar",
    "escofinadas": "escofinar",
    "escofinado": "escofinar",
    "escofinados": "escofinar",
    "escofinamos": "escofinar",
    "escofinan": "escofinar",
    "escofinando": "escofinar",
    "escofinara": "escofinar",
    "escofinarais": "escofinar",
    "escofinaran": "escofinar",
    "escofinaras": "escofinar",
    "escofinare": "escofinar",
    "escofinareis": "escofinar",
    "escofinaremos": "escofinar",
    "escofinaren": "escofinar",
    "escofinares": "escofinar",
    "escofinaron": "escofinar",
    "escofinará": "escofinar",
    "escofinarán": "escofinar",
    "escofinarás": "escofinar",
    "escofinaré": "escofinar",
    "escofinaréis": "escofinar",
    "escofinaría": "escofinar",
    "escofinaríais": "escofinar",
    "escofinaríamos": "escofinar",
    "escofinarían": "escofinar",
    "escofinarías": "escofinar",
    "escofinas": "escofinar",
    "escofinase": "escofinar",
    "escofinaseis": "escofinar",
    "escofinasen": "escofinar",
    "escofinases": "escofinar",
    "escofinaste": "escofinar",
    "escofinasteis": "escofinar",
    "escofine": "escofinar",
    "escofinemos": "escofinar",
    "escofinen": "escofinar",
    "escofines": "escofinar",
    "escofino": "escofinar",
    "escofinábamos": "escofinar",
    "escofináis": "escofinar",
    "escofináramos": "escofinar",
    "escofináremos": "escofinar",
    "escofinásemos": "escofinar",
    "escofiné": "escofinar",
    "escofinéis": "escofinar",
    "escofinó": "escofinar",
    "escoge": "escoger",
    "escoged": "escoger",
    "escogemos": "escoger",
    "escogen": "escoger",
    "escogencias": "escogencia",
    "escogeremos": "escoger",
    "escogerá": "escoger",
    "escogerán": "escoger",
    "escogerás": "escoger",
    "escogeré": "escoger",
    "escogeréis": "escoger",
    "escogería": "escoger",
    "escogeríais": "escoger",
    "escogeríamos": "escoger",
    "escogerían": "escoger",
    "escogerías": "escoger",
    "escoges": "escoger",
    "escogida": "escoger",
    "escogidas": "escoger",
    "escogido": "escoger",
    "escogidos": "escoger",
    "escogiendo": "escoger",
    "escogiera": "escoger",
    "escogierais": "escoger",
    "escogieran": "escoger",
    "escogieras": "escoger",
    "escogiere": "escoger",
    "escogiereis": "escoger",
    "escogieren": "escoger",
    "escogieres": "escoger",
    "escogieron": "escoger",
    "escogiese": "escoger",
    "escogieseis": "escoger",
    "escogiesen": "escoger",
    "escogieses": "escoger",
    "escogimientos": "escogimiento",
    "escogimos": "escoger",
    "escogiste": "escoger",
    "escogisteis": "escoger",
    "escogiéramos": "escoger",
    "escogiéremos": "escoger",
    "escogiésemos": "escoger",
    "escogió": "escoger",
    "escogéis": "escoger",
    "escogí": "escoger",
    "escogía": "escoger",
    "escogíais": "escoger",
    "escogíamos": "escoger",
    "escogían": "escoger",
    "escogías": "escoger",
    "escoja": "escoger",
    "escojamos": "escoger",
    "escojan": "escoger",
    "escojas": "escoger",
    "escojo": "escoger",
    "escojáis": "escoger",
    "escolanos": "escolano",
    "escolanías": "escolanía",
    "escolapia": "escolapio",
    "escolapias": "escolapio",
    "escolapios": "escolapio",
    "escolares": "escolar",
    "escolarice": "escolarizar",
    "escolaricemos": "escolarizar",
    "escolaricen": "escolarizar",
    "escolarices": "escolarizar",
    "escolaricé": "escolarizar",
    "escolaricéis": "escolarizar",
    "escolaridades": "escolaridad",
    "escolariega": "escolariego",
    "escolariegas": "escolariego",
    "escolariegos": "escolariego",
    "escolariza": "escolarizar",
    "escolarizaba": "escolarizar",
    "escolarizabais": "escolarizar",
    "escolarizaban": "escolarizar",
    "escolarizabas": "escolarizar",
    "escolarizaciones": "escolarización",
    "escolarizad": "escolarizar",
    "escolarizada": "escolarizar",
    "escolarizadas": "escolarizar",
    "escolarizado": "escolarizar",
    "escolarizados": "escolarizar",
    "escolarizamos": "escolarizar",
    "escolarizan": "escolarizar",
    "escolarizando": "escolarizar",
    "escolarizara": "escolarizar",
    "escolarizarais": "escolarizar",
    "escolarizaran": "escolarizar",
    "escolarizaras": "escolarizar",
    "escolarizare": "escolarizar",
    "escolarizareis": "escolarizar",
    "escolarizaremos": "escolarizar",
    "escolarizaren": "escolarizar",
    "escolarizares": "escolarizar",
    "escolarizaron": "escolarizar",
    "escolarizará": "escolarizar",
    "escolarizarán": "escolarizar",
    "escolarizarás": "escolarizar",
    "escolarizaré": "escolarizar",
    "escolarizaréis": "escolarizar",
    "escolarizaría": "escolarizar",
    "escolarizaríais": "escolarizar",
    "escolarizaríamos": "escolarizar",
    "escolarizarían": "escolarizar",
    "escolarizarías": "escolarizar",
    "escolarizas": "escolarizar",
    "escolarizase": "escolarizar",
    "escolarizaseis": "escolarizar",
    "escolarizasen": "escolarizar",
    "escolarizases": "escolarizar",
    "escolarizaste": "escolarizar",
    "escolarizasteis": "escolarizar",
    "escolarizo": "escolarizar",
    "escolarizábamos": "escolarizar",
    "escolarizáis": "escolarizar",
    "escolarizáramos": "escolarizar",
    "escolarizáremos": "escolarizar",
    "escolarizásemos": "escolarizar",
    "escolarizó": "escolarizar",
    "escolasticismos": "escolasticismo",
    "escolecitas": "escolecita",
    "escolia": "escoliar",
    "escoliaba": "escoliar",
    "escoliabais": "escoliar",
    "escoliaban": "escoliar",
    "escoliabas": "escoliar",
    "escoliad": "escoliar",
    "escoliada": "escoliar",
    "escoliadas": "escoliar",
    "escoliado": "escoliar",
    "escoliadora": "escoliador",
    "escoliadoras": "escoliador",
    "escoliadores": "escoliador",
    "escoliados": "escoliar",
    "escoliamos": "escoliar",
    "escolian": "escoliar",
    "escoliando": "escoliar",
    "escoliara": "escoliar",
    "escoliarais": "escoliar",
    "escoliaran": "escoliar",
    "escoliaras": "escoliar",
    "escoliare": "escoliar",
    "escoliareis": "escoliar",
    "escoliaremos": "escoliar",
    "escoliaren": "escoliar",
    "escoliares": "escoliar",
    "escoliaron": "escoliar",
    "escoliará": "escoliar",
    "escoliarán": "escoliar",
    "escoliarás": "escoliar",
    "escoliaré": "escoliar",
    "escoliaréis": "escoliar",
    "escoliaría": "escoliar",
    "escoliaríais": "escoliar",
    "escoliaríamos": "escoliar",
    "escoliarían": "escoliar",
    "escoliarías": "escoliar",
    "escolias": "escoliar",
    "escoliase": "escoliar",
    "escoliaseis": "escoliar",
    "escoliasen": "escoliar",
    "escoliases": "escoliar",
    "escoliastas": "escoliasta",
    "escoliaste": "escoliar",
    "escoliasteis": "escoliar",
    "escolie": "escoliar",
    "escoliemos": "escoliar",
    "escolien": "escoliar",
    "escolies": "escoliar",
    "escolimosa": "escolimoso",
    "escolimosas": "escolimoso",
    "escolimosos": "escolimoso",
    "escolio": "escoliar",
    "escolios": "escolio",
    "escoliábamos": "escoliar",
    "escoliáis": "escoliar",
    "escoliáramos": "escoliar",
    "escoliáremos": "escoliar",
    "escoliásemos": "escoliar",
    "escolié": "escoliar",
    "escoliéis": "escoliar",
    "escolió": "escoliar",
    "escolla": "escollar",
    "escollaba": "escollar",
    "escollabais": "escollar",
    "escollaban": "escollar",
    "escollabas": "escollar",
    "escollad": "escollar",
    "escollada": "escollar",
    "escolladas": "escollar",
    "escollado": "escollar",
    "escollados": "escollar",
    "escollamos": "escollar",
    "escollan": "escollar",
    "escollando": "escollar",
    "escollara": "escollar",
    "escollarais": "escollar",
    "escollaran": "escollar",
    "escollaras": "escollar",
    "escollare": "escollar",
    "escollareis": "escollar",
    "escollaremos": "escollar",
    "escollaren": "escollar",
    "escollares": "escollar",
    "escollaron": "escollar",
    "escollará": "escollar",
    "escollarán": "escollar",
    "escollarás": "escollar",
    "escollaré": "escollar",
    "escollaréis": "escollar",
    "escollaría": "escollar",
    "escollaríais": "escollar",
    "escollaríamos": "escollar",
    "escollarían": "escollar",
    "escollarías": "escollar",
    "escollas": "escollar",
    "escollase": "escollar",
    "escollaseis": "escollar",
    "escollasen": "escollar",
    "escollases": "escollar",
    "escollaste": "escollar",
    "escollasteis": "escollar",
    "escolle": "escollar",
    "escollemos": "escollar",
    "escollen": "escollar",
    "escolleras": "escollera",
    "escolles": "escollar",
    "escollo": "escollar",
    "escollos": "escollo",
    "escollábamos": "escollar",
    "escolláis": "escollar",
    "escolláramos": "escollar",
    "escolláremos": "escollar",
    "escollásemos": "escollar",
    "escollé": "escollar",
    "escolléis": "escollar",
    "escolló": "escollar",
    "escolopendras": "escolopendra",
    "escolopácidos": "escolopácido",
    "escolta": "escoltar",
    "escoltaba": "escoltar",
    "escoltabais": "escoltar",
    "escoltaban": "escoltar",
    "escoltabas": "escoltar",
    "escoltad": "escoltar",
    "escoltada": "escoltar",
    "escoltadas": "escoltar",
    "escoltado": "escoltar",
    "escoltados": "escoltar",
    "escoltamos": "escoltar",
    "escoltan": "escoltar",
    "escoltando": "escoltar",
    "escoltara": "escoltar",
    "escoltarais": "escoltar",
    "escoltaran": "escoltar",
    "escoltaras": "escoltar",
    "escoltare": "escoltar",
    "escoltareis": "escoltar",
    "escoltaremos": "escoltar",
    "escoltaren": "escoltar",
    "escoltares": "escoltar",
    "escoltaron": "escoltar",
    "escoltará": "escoltar",
    "escoltarán": "escoltar",
    "escoltarás": "escoltar",
    "escoltaré": "escoltar",
    "escoltaréis": "escoltar",
    "escoltaría": "escoltar",
    "escoltaríais": "escoltar",
    "escoltaríamos": "escoltar",
    "escoltarían": "escoltar",
    "escoltarías": "escoltar",
    "escoltas": "escoltar",
    "escoltase": "escoltar",
    "escoltaseis": "escoltar",
    "escoltasen": "escoltar",
    "escoltases": "escoltar",
    "escoltaste": "escoltar",
    "escoltasteis": "escoltar",
    "escolte": "escoltar",
    "escoltemos": "escoltar",
    "escolten": "escoltar",
    "escoltes": "escoltar",
    "escoltistas": "escoltista",
    "escolto": "escoltar",
    "escoltábamos": "escoltar",
    "escoltáis": "escoltar",
    "escoltáramos": "escoltar",
    "escoltáremos": "escoltar",
    "escoltásemos": "escoltar",
    "escolté": "escoltar",
    "escoltéis": "escoltar",
    "escoltó": "escoltar",
    "escolástica": "escolástico",
    "escolásticas": "escolástico",
    "escolásticos": "escolástico",
    "escombra": "escombrar",
    "escombraba": "escombrar",
    "escombrabais": "escombrar",
    "escombraban": "escombrar",
    "escombrabas": "escombrar",
    "escombrad": "escombrar",
    "escombrada": "escombrar",
    "escombradas": "escombrar",
    "escombrado": "escombrar",
    "escombrados": "escombrar",
    "escombramos": "escombrar",
    "escombran": "escombrar",
    "escombrando": "escombrar",
    "escombrara": "escombrar",
    "escombrarais": "escombrar",
    "escombraran": "escombrar",
    "escombraras": "escombrar",
    "escombrare": "escombrar",
    "escombrareis": "escombrar",
    "escombraremos": "escombrar",
    "escombraren": "escombrar",
    "escombrares": "escombrar",
    "escombraron": "escombrar",
    "escombrará": "escombrar",
    "escombrarán": "escombrar",
    "escombrarás": "escombrar",
    "escombraré": "escombrar",
    "escombraréis": "escombrar",
    "escombraría": "escombrar",
    "escombraríais": "escombrar",
    "escombraríamos": "escombrar",
    "escombrarían": "escombrar",
    "escombrarías": "escombrar",
    "escombras": "escombrar",
    "escombrase": "escombrar",
    "escombraseis": "escombrar",
    "escombrasen": "escombrar",
    "escombrases": "escombrar",
    "escombraste": "escombrar",
    "escombrasteis": "escombrar",
    "escombre": "escombrar",
    "escombremos": "escombrar",
    "escombren": "escombrar",
    "escombreras": "escombrera",
    "escombres": "escombrar",
    "escombro": "escombrar",
    "escombros": "escombro",
    "escombrábamos": "escombrar",
    "escombráis": "escombrar",
    "escombráramos": "escombrar",
    "escombráremos": "escombrar",
    "escombrásemos": "escombrar",
    "escombré": "escombrar",
    "escombréis": "escombrar",
    "escombró": "escombrar",
    "escomendrijos": "escomendrijo",
    "esconces": "esconce",
    "esconda": "esconder",
    "escondamos": "esconder",
    "escondan": "esconder",
    "escondas": "esconder",
    "esconde": "esconder",
    "esconded": "esconder",
    "escondederos": "escondedero",
    "escondemos": "esconder",
    "esconden": "esconder",
    "esconderemos": "esconder",
    "esconderá": "esconder",
    "esconderán": "esconder",
    "esconderás": "esconder",
    "esconderé": "esconder",
    "esconderéis": "esconder",
    "escondería": "esconder",
    "esconderíais": "esconder",
    "esconderíamos": "esconder",
    "esconderían": "esconder",
    "esconderías": "esconder",
    "escondes": "esconder",
    "escondida": "esconder",
    "escondidas": "esconder",
    "escondido": "esconder",
    "escondidos": "esconder",
    "escondiendo": "esconder",
    "escondiera": "esconder",
    "escondierais": "esconder",
    "escondieran": "esconder",
    "escondieras": "esconder",
    "escondiere": "esconder",
    "escondiereis": "esconder",
    "escondieren": "esconder",
    "escondieres": "esconder",
    "escondieron": "esconder",
    "escondiese": "esconder",
    "escondieseis": "esconder",
    "escondiesen": "esconder",
    "escondieses": "esconder",
    "escondimientos": "escondimiento",
    "escondimos": "esconder",
    "escondiste": "esconder",
    "escondisteis": "esconder",
    "escondites": "escondite",
    "escondiéramos": "esconder",
    "escondiéremos": "esconder",
    "escondiésemos": "esconder",
    "escondió": "esconder",
    "escondo": "esconder",
    "escondrijos": "escondrijo",
    "escondáis": "esconder",
    "escondéis": "esconder",
    "escondí": "esconder",
    "escondía": "esconder",
    "escondíais": "esconder",
    "escondíamos": "esconder",
    "escondían": "esconder",
    "escondías": "esconder",
    "escopetas": "escopeta",
    "escopetazos": "escopetazo",
    "escopetea": "escopetear",
    "escopeteaba": "escopetear",
    "escopeteabais": "escopetear",
    "escopeteaban": "escopetear",
    "escopeteabas": "escopetear",
    "escopetead": "escopetear",
    "escopeteada": "escopetear",
    "escopeteadas": "escopetear",
    "escopeteado": "escopetear",
    "escopeteados": "escopetear",
    "escopeteamos": "escopetear",
    "escopetean": "escopetear",
    "escopeteando": "escopetear",
    "escopeteara": "escopetear",
    "escopetearais": "escopetear",
    "escopetearan": "escopetear",
    "escopetearas": "escopetear",
    "escopeteare": "escopetear",
    "escopeteareis": "escopetear",
    "escopetearemos": "escopetear",
    "escopetearen": "escopetear",
    "escopeteares": "escopetear",
    "escopetearon": "escopetear",
    "escopeteará": "escopetear",
    "escopetearán": "escopetear",
    "escopetearás": "escopetear",
    "escopetearé": "escopetear",
    "escopetearéis": "escopetear",
    "escopetearía": "escopetear",
    "escopetearíais": "escopetear",
    "escopetearíamos": "escopetear",
    "escopetearían": "escopetear",
    "escopetearías": "escopetear",
    "escopeteas": "escopetear",
    "escopetease": "escopetear",
    "escopeteaseis": "escopetear",
    "escopeteasen": "escopetear",
    "escopeteases": "escopetear",
    "escopeteaste": "escopetear",
    "escopeteasteis": "escopetear",
    "escopetee": "escopetear",
    "escopeteemos": "escopetear",
    "escopeteen": "escopetear",
    "escopetees": "escopetear",
    "escopeteo": "escopetear",
    "escopeteos": "escopeteo",
    "escopetera": "escopetero",
    "escopeteras": "escopetero",
    "escopeteros": "escopetero",
    "escopeterías": "escopetería",
    "escopeteábamos": "escopetear",
    "escopeteáis": "escopetear",
    "escopeteáramos": "escopetear",
    "escopeteáremos": "escopetear",
    "escopeteásemos": "escopetear",
    "escopeteé": "escopetear",
    "escopeteéis": "escopetear",
    "escopeteó": "escopetear",
    "escopladuras": "escopladura",
    "escoplea": "escoplear",
    "escopleaba": "escoplear",
    "escopleabais": "escoplear",
    "escopleaban": "escoplear",
    "escopleabas": "escoplear",
    "escoplead": "escoplear",
    "escopleada": "escoplear",
    "escopleadas": "escoplear",
    "escopleado": "escoplear",
    "escopleados": "escoplear",
    "escopleaduras": "escopleadura",
    "escopleamos": "escoplear",
    "escoplean": "escoplear",
    "escopleando": "escoplear",
    "escopleara": "escoplear",
    "escoplearais": "escoplear",
    "escoplearan": "escoplear",
    "escoplearas": "escoplear",
    "escopleare": "escoplear",
    "escopleareis": "escoplear",
    "escoplearemos": "escoplear",
    "escoplearen": "escoplear",
    "escopleares": "escoplear",
    "escoplearon": "escoplear",
    "escopleará": "escoplear",
    "escoplearán": "escoplear",
    "escoplearás": "escoplear",
    "escoplearé": "escoplear",
    "escoplearéis": "escoplear",
    "escoplearía": "escoplear",
    "escoplearíais": "escoplear",
    "escoplearíamos": "escoplear",
    "escoplearían": "escoplear",
    "escoplearías": "escoplear",
    "escopleas": "escoplear",
    "escoplease": "escoplear",
    "escopleaseis": "escoplear",
    "escopleasen": "escoplear",
    "escopleases": "escoplear",
    "escopleaste": "escoplear",
    "escopleasteis": "escoplear",
    "escoplee": "escoplear",
    "escopleemos": "escoplear",
    "escopleen": "escoplear",
    "escoplees": "escoplear",
    "escopleo": "escoplear",
    "escopleábamos": "escoplear",
    "escopleáis": "escoplear",
    "escopleáramos": "escoplear",
    "escopleáremos": "escoplear",
    "escopleásemos": "escoplear",
    "escopleé": "escoplear",
    "escopleéis": "escoplear",
    "escopleó": "escoplear",
    "escoplos": "escoplo",
    "escopofilias": "escopofilia",
    "escopolaminas": "escopolamina",
    "escopolinas": "escopolina",
    "escora": "escorar",
    "escoraba": "escorar",
    "escorabais": "escorar",
    "escoraban": "escorar",
    "escorabas": "escorar",
    "escorad": "escorar",
    "escorada": "escorar",
    "escoradas": "escorar",
    "escorado": "escorar",
    "escorados": "escorar",
    "escoramos": "escorar",
    "escoran": "escorar",
    "escorando": "escorar",
    "escorara": "escorar",
    "escorarais": "escorar",
    "escoraran": "escorar",
    "escoraras": "escorar",
    "escorare": "escorar",
    "escorareis": "escorar",
    "escoraremos": "escorar",
    "escoraren": "escorar",
    "escorares": "escorar",
    "escoraron": "escorar",
    "escorará": "escorar",
    "escorarán": "escorar",
    "escorarás": "escorar",
    "escoraré": "escorar",
    "escoraréis": "escorar",
    "escoraría": "escorar",
    "escoraríais": "escorar",
    "escoraríamos": "escorar",
    "escorarían": "escorar",
    "escorarías": "escorar",
    "escoras": "escorar",
    "escorase": "escorar",
    "escoraseis": "escorar",
    "escorasen": "escorar",
    "escorases": "escorar",
    "escoraste": "escorar",
    "escorasteis": "escorar",
    "escorbutos": "escorbuto",
    "escorbútica": "escorbútico",
    "escorbúticas": "escorbútico",
    "escorbúticos": "escorbútico",
    "escorce": "escorzar",
    "escorcemos": "escorzar",
    "escorcen": "escorzar",
    "escorces": "escorzar",
    "escorchapines": "escorchapín",
    "escorchones": "escorchón",
    "escorcé": "escorzar",
    "escorcéis": "escorzar",
    "escordios": "escordio",
    "escore": "escorar",
    "escoremos": "escorar",
    "escoren": "escorar",
    "escores": "escorar",
    "escoria": "escoriar",
    "escoriaba": "escoriar",
    "escoriabais": "escoriar",
    "escoriaban": "escoriar",
    "escoriabas": "escoriar",
    "escoriaciones": "escoriación",
    "escoriad": "escoriar",
    "escoriada": "escoriar",
    "escoriadas": "escoriar",
    "escoriado": "escoriar",
    "escoriados": "escoriar",
    "escoriales": "escorial",
    "escoriamos": "escoriar",
    "escorian": "escoriar",
    "escoriando": "escoriar",
    "escoriara": "escoriar",
    "escoriarais": "escoriar",
    "escoriaran": "escoriar",
    "escoriaras": "escoriar",
    "escoriare": "escoriar",
    "escoriareis": "escoriar",
    "escoriaremos": "escoriar",
    "escoriaren": "escoriar",
    "escoriares": "escoriar",
    "escoriaron": "escoriar",
    "escoriará": "escoriar",
    "escoriarán": "escoriar",
    "escoriarás": "escoriar",
    "escoriaré": "escoriar",
    "escoriaréis": "escoriar",
    "escoriaría": "escoriar",
    "escoriaríais": "escoriar",
    "escoriaríamos": "escoriar",
    "escoriarían": "escoriar",
    "escoriarías": "escoriar",
    "escorias": "escoriar",
    "escoriase": "escoriar",
    "escoriaseis": "escoriar",
    "escoriasen": "escoriar",
    "escoriases": "escoriar",
    "escoriaste": "escoriar",
    "escoriasteis": "escoriar",
    "escorie": "escoriar",
    "escoriemos": "escoriar",
    "escorien": "escoriar",
    "escories": "escoriar",
    "escorifica": "escorificar",
    "escorificaba": "escorificar",
    "escorificabais": "escorificar",
    "escorificaban": "escorificar",
    "escorificabas": "escorificar",
    "escorificaciones": "escorificación",
    "escorificad": "escorificar",
    "escorificada": "escorificar",
    "escorificadas": "escorificar",
    "escorificado": "escorificar",
    "escorificadores": "escorificador",
    "escorificados": "escorificar",
    "escorificamos": "escorificar",
    "escorifican": "escorificar",
    "escorificando": "escorificar",
    "escorificantes": "escorificante",
    "escorificara": "escorificar",
    "escorificarais": "escorificar",
    "escorificaran": "escorificar",
    "escorificaras": "escorificar",
    "escorificare": "escorificar",
    "escorificareis": "escorificar",
    "escorificaremos": "escorificar",
    "escorificaren": "escorificar",
    "escorificares": "escorificar",
    "escorificaron": "escorificar",
    "escorificará": "escorificar",
    "escorificarán": "escorificar",
    "escorificarás": "escorificar",
    "escorificaré": "escorificar",
    "escorificaréis": "escorificar",
    "escorificaría": "escorificar",
    "escorificaríais": "escorificar",
    "escorificaríamos": "escorificar",
    "escorificarían": "escorificar",
    "escorificarías": "escorificar",
    "escorificas": "escorificar",
    "escorificase": "escorificar",
    "escorificaseis": "escorificar",
    "escorificasen": "escorificar",
    "escorificases": "escorificar",
    "escorificaste": "escorificar",
    "escorificasteis": "escorificar",
    "escorifico": "escorificar",
    "escorificábamos": "escorificar",
    "escorificáis": "escorificar",
    "escorificáramos": "escorificar",
    "escorificáremos": "escorificar",
    "escorificásemos": "escorificar",
    "escorificó": "escorificar",
    "escorifique": "escorificar",
    "escorifiquemos": "escorificar",
    "escorifiquen": "escorificar",
    "escorifiques": "escorificar",
    "escorifiqué": "escorificar",
    "escorifiquéis": "escorificar",
    "escorio": "escoriar",
    "escoriábamos": "escoriar",
    "escoriáis": "escoriar",
    "escoriáramos": "escoriar",
    "escoriáremos": "escoriar",
    "escoriásemos": "escoriar",
    "escorié": "escoriar",
    "escoriéis": "escoriar",
    "escorió": "escoriar",
    "escoro": "escorar",
    "escoroditas": "escorodita",
    "escorpenas": "escorpena",
    "escorpinas": "escorpina",
    "escorpiones": "escorpión",
    "escorpénidos": "escorpénido",
    "escorrentías": "escorrentía",
    "escorrozos": "escorrozo",
    "escorsanas": "escorsana",
    "escorza": "escorzar",
    "escorzaba": "escorzar",
    "escorzabais": "escorzar",
    "escorzaban": "escorzar",
    "escorzabas": "escorzar",
    "escorzad": "escorzar",
    "escorzada": "escorzar",
    "escorzadas": "escorzar",
    "escorzado": "escorzar",
    "escorzados": "escorzar",
    "escorzamos": "escorzar",
    "escorzan": "escorzar",
    "escorzando": "escorzar",
    "escorzara": "escorzar",
    "escorzarais": "escorzar",
    "escorzaran": "escorzar",
    "escorzaras": "escorzar",
    "escorzare": "escorzar",
    "escorzareis": "escorzar",
    "escorzaremos": "escorzar",
    "escorzaren": "escorzar",
    "escorzares": "escorzar",
    "escorzaron": "escorzar",
    "escorzará": "escorzar",
    "escorzarán": "escorzar",
    "escorzarás": "escorzar",
    "escorzaré": "escorzar",
    "escorzaréis": "escorzar",
    "escorzaría": "escorzar",
    "escorzaríais": "escorzar",
    "escorzaríamos": "escorzar",
    "escorzarían": "escorzar",
    "escorzarías": "escorzar",
    "escorzas": "escorzar",
    "escorzase": "escorzar",
    "escorzaseis": "escorzar",
    "escorzasen": "escorzar",
    "escorzases": "escorzar",
    "escorzaste": "escorzar",
    "escorzasteis": "escorzar",
    "escorzo": "escorzar",
    "escorzoneras": "escorzonera",
    "escorzones": "escorzón",
    "escorzos": "escorzo",
    "escorzábamos": "escorzar",
    "escorzáis": "escorzar",
    "escorzáramos": "escorzar",
    "escorzáremos": "escorzar",
    "escorzásemos": "escorzar",
    "escorzó": "escorzar",
    "escorábamos": "escorar",
    "escoráis": "escorar",
    "escoráramos": "escorar",
    "escoráremos": "escorar",
    "escorásemos": "escorar",
    "escoré": "escorar",
    "escoréis": "escorar",
    "escoró": "escorar",
    "escosca": "escoscar",
    "escoscaba": "escoscar",
    "escoscabais": "escoscar",
    "escoscaban": "escoscar",
    "escoscabas": "escoscar",
    "escoscad": "escoscar",
    "escoscada": "escoscar",
    "escoscadas": "escoscar",
    "escoscado": "escoscar",
    "escoscados": "escoscar",
    "escoscamos": "escoscar",
    "escoscan": "escoscar",
    "escoscando": "escoscar",
    "escoscara": "escoscar",
    "escoscarais": "escoscar",
    "escoscaran": "escoscar",
    "escoscaras": "escoscar",
    "escoscare": "escoscar",
    "escoscareis": "escoscar",
    "escoscaremos": "escoscar",
    "escoscaren": "escoscar",
    "escoscares": "escoscar",
    "escoscaron": "escoscar",
    "escoscará": "escoscar",
    "escoscarán": "escoscar",
    "escoscarás": "escoscar",
    "escoscaré": "escoscar",
    "escoscaréis": "escoscar",
    "escoscaría": "escoscar",
    "escoscaríais": "escoscar",
    "escoscaríamos": "escoscar",
    "escoscarían": "escoscar",
    "escoscarías": "escoscar",
    "escoscas": "escoscar",
    "escoscase": "escoscar",
    "escoscaseis": "escoscar",
    "escoscasen": "escoscar",
    "escoscases": "escoscar",
    "escoscaste": "escoscar",
    "escoscasteis": "escoscar",
    "escosco": "escoscar",
    "escoscábamos": "escoscar",
    "escoscáis": "escoscar",
    "escoscáramos": "escoscar",
    "escoscáremos": "escoscar",
    "escoscásemos": "escoscar",
    "escoscó": "escoscar",
    "escosque": "escoscar",
    "escosquemos": "escoscar",
    "escosquen": "escoscar",
    "escosques": "escoscar",
    "escosqué": "escoscar",
    "escosquéis": "escoscar",
    "escota": "escotar",
    "escotaba": "escotar",
    "escotabais": "escotar",
    "escotaban": "escotar",
    "escotabas": "escotar",
    "escotad": "escotar",
    "escotada": "escotar",
    "escotadas": "escotar",
    "escotado": "escotar",
    "escotados": "escotar",
    "escotaduras": "escotadura",
    "escotamos": "escotar",
    "escotan": "escotar",
    "escotando": "escotar",
    "escotara": "escotar",
    "escotarais": "escotar",
    "escotaran": "escotar",
    "escotaras": "escotar",
    "escotare": "escotar",
    "escotareis": "escotar",
    "escotaremos": "escotar",
    "escotaren": "escotar",
    "escotares": "escotar",
    "escotaron": "escotar",
    "escotará": "escotar",
    "escotarán": "escotar",
    "escotarás": "escotar",
    "escotaré": "escotar",
    "escotaréis": "escotar",
    "escotaría": "escotar",
    "escotaríais": "escotar",
    "escotaríamos": "escotar",
    "escotarían": "escotar",
    "escotarías": "escotar",
    "escotas": "escotar",
    "escotase": "escotar",
    "escotaseis": "escotar",
    "escotasen": "escotar",
    "escotases": "escotar",
    "escotaste": "escotar",
    "escotasteis": "escotar",
    "escote": "escotar",
    "escotemos": "escotar",
    "escoten": "escotar",
    "escotera": "escotero",
    "escoteras": "escotero",
    "escoteros": "escotero",
    "escotes": "escote",
    "escotillas": "escotilla",
    "escotillones": "escotillón",
    "escotines": "escotín",
    "escotismos": "escotismo",
    "escotistas": "escotista",
    "escoto": "escotar",
    "escotomas": "escotoma",
    "escotomatosa": "escotomatoso",
    "escotomatosas": "escotomatoso",
    "escotomatosos": "escotomatoso",
    "escotometrías": "escotometría",
    "escotopias": "escotopia",
    "escotos": "escoto",
    "escotábamos": "escotar",
    "escotáis": "escotar",
    "escotáramos": "escotar",
    "escotáremos": "escotar",
    "escotásemos": "escotar",
    "escoté": "escotar",
    "escotéis": "escotar",
    "escotó": "escotar",
    "escotómetros": "escotómetro",
    "escotópica": "escotópico",
    "escotópicas": "escotópico",
    "escotópicos": "escotópico",
    "escozamos": "escocer",
    "escozores": "escozor",
    "escozáis": "escocer",
    "escriba": "escribir",
    "escribamos": "escribir",
    "escriban": "escribir",
    "escribaniles": "escribanil",
    "escribanos": "escribano",
    "escribanías": "escribanía",
    "escribas": "escribir",
    "escribe": "escribir",
    "escriben": "escribir",
    "escribes": "escribir",
    "escribibles": "escribible",
    "escribid": "escribir",
    "escribiendo": "escribir",
    "escribientes": "escribiente",
    "escribiera": "escribir",
    "escribierais": "escribir",
    "escribieran": "escribir",
    "escribieras": "escribir",
    "escribiere": "escribir",
    "escribiereis": "escribir",
    "escribieren": "escribir",
    "escribieres": "escribir",
    "escribieron": "escribir",
    "escribiese": "escribir",
    "escribieseis": "escribir",
    "escribiesen": "escribir",
    "escribieses": "escribir",
    "escribimos": "escribir",
    "escribiremos": "escribir",
    "escribirá": "escribir",
    "escribirán": "escribir",
    "escribirás": "escribir",
    "escribiré": "escribir",
    "escribiréis": "escribir",
    "escribiría": "escribir",
    "escribiríais": "escribir",
    "escribiríamos": "escribir",
    "escribirían": "escribir",
    "escribirías": "escribir",
    "escribiste": "escribir",
    "escribisteis": "escribir",
    "escribiéramos": "escribir",
    "escribiéremos": "escribir",
    "escribiésemos": "escribir",
    "escribió": "escribir",
    "escribo": "escribir",
    "escribáis": "escribir",
    "escribí": "escribir",
    "escribía": "escribir",
    "escribíais": "escribir",
    "escribíamos": "escribir",
    "escribían": "escribir",
    "escribías": "escribir",
    "escribís": "escribir",
    "escripias": "escripia",
    "escrita": "escribir",
    "escritas": "escribir",
    "escritillas": "escritilla",
    "escrito": "escribir",
    "escritora": "escritor",
    "escritoras": "escritor",
    "escritores": "escritor",
    "escritorios": "escritorio",
    "escritorzuela": "escritorzuelo",
    "escritorzuelas": "escritorzuelo",
    "escritorzuelos": "escritorzuelo",
    "escritos": "escrito",
    "escritura": "escriturar",
    "escrituraba": "escriturar",
    "escriturabais": "escriturar",
    "escrituraban": "escriturar",
    "escriturabas": "escriturar",
    "escriturad": "escriturar",
    "escriturada": "escriturar",
    "escrituradas": "escriturar",
    "escriturado": "escriturar",
    "escriturados": "escriturar",
    "escrituramos": "escriturar",
    "escrituran": "escriturar",
    "escriturando": "escriturar",
    "escriturara": "escriturar",
    "escriturarais": "escriturar",
    "escrituraran": "escriturar",
    "escrituraras": "escriturar",
    "escriturare": "escriturar",
    "escriturareis": "escriturar",
    "escrituraremos": "escriturar",
    "escrituraren": "escriturar",
    "escriturares": "escriturar",
    "escrituraria": "escriturario",
    "escriturarias": "escriturario",
    "escriturarios": "escriturario",
    "escrituraron": "escriturar",
    "escriturará": "escriturar",
    "escriturarán": "escriturar",
    "escriturarás": "escriturar",
    "escrituraré": "escriturar",
    "escrituraréis": "escriturar",
    "escrituraría": "escriturar",
    "escrituraríais": "escriturar",
    "escrituraríamos": "escriturar",
    "escriturarían": "escriturar",
    "escriturarías": "escriturar",
    "escrituras": "escriturar",
    "escriturase": "escriturar",
    "escrituraseis": "escriturar",
    "escriturasen": "escriturar",
    "escriturases": "escriturar",
    "escrituraste": "escriturar",
    "escriturasteis": "escriturar",
    "escriture": "escriturar",
    "escrituremos": "escriturar",
    "escrituren": "escriturar",
    "escritures": "escriturar",
    "escrituro": "escriturar",
    "escriturábamos": "escriturar",
    "escrituráis": "escriturar",
    "escrituráramos": "escriturar",
    "escrituráremos": "escriturar",
    "escriturásemos": "escriturar",
    "escrituré": "escriturar",
    "escrituréis": "escriturar",
    "escrituró": "escriturar",
    "escriños": "escriño",
    "escrofularias": "escrofularia",
    "escrofulariáceas": "escrofulariácea",
    "escrofulismos": "escrofulismo",
    "escrofulosa": "escrofuloso",
    "escrofulosas": "escrofuloso",
    "escrofulosos": "escrofuloso",
    "escrotales": "escrotal",
    "escrotos": "escroto",
    "escrupulillos": "escrupulillo",
    "escrupulosa": "escrupuloso",
    "escrupulosas": "escrupuloso",
    "escrupulosidades": "escrupulosidad",
    "escrupulosos": "escrupuloso",
    "escruta": "escrutar",
    "escrutaba": "escrutar",
    "escrutabais": "escrutar",
    "escrutaban": "escrutar",
    "escrutabas": "escrutar",
    "escrutad": "escrutar",
    "escrutada": "escrutar",
    "escrutadas": "escrutar",
    "escrutado": "escrutar",
    "escrutadora": "escrutador",
    "escrutadoras": "escrutador",
    "escrutadores": "escrutador",
    "escrutados": "escrutar",
    "escrutamos": "escrutar",
    "escrutan": "escrutar",
    "escrutando": "escrutar",
    "escrutara": "escrutar",
    "escrutarais": "escrutar",
    "escrutaran": "escrutar",
    "escrutaras": "escrutar",
    "escrutare": "escrutar",
    "escrutareis": "escrutar",
    "escrutaremos": "escrutar",
    "escrutaren": "escrutar",
    "escrutares": "escrutar",
    "escrutaron": "escrutar",
    "escrutará": "escrutar",
    "escrutarán": "escrutar",
    "escrutarás": "escrutar",
    "escrutaré": "escrutar",
    "escrutaréis": "escrutar",
    "escrutaría": "escrutar",
    "escrutaríais": "escrutar",
    "escrutaríamos": "escrutar",
    "escrutarían": "escrutar",
    "escrutarías": "escrutar",
    "escrutas": "escrutar",
    "escrutase": "escrutar",
    "escrutaseis": "escrutar",
    "escrutasen": "escrutar",
    "escrutases": "escrutar",
    "escrutaste": "escrutar",
    "escrutasteis": "escrutar",
    "escrute": "escrutar",
    "escrutemos": "escrutar",
    "escruten": "escrutar",
    "escrutes": "escrutar",
    "escrutinios": "escrutinio",
    "escrutiñadora": "escrutiñador",
    "escrutiñadoras": "escrutiñador",
    "escrutiñadores": "escrutiñador",
    "escruto": "escrutar",
    "escrutábamos": "escrutar",
    "escrutáis": "escrutar",
    "escrutáramos": "escrutar",
    "escrutáremos": "escrutar",
    "escrutásemos": "escrutar",
    "escruté": "escrutar",
    "escrutéis": "escrutar",
    "escrutó": "escrutar",
    "escrófulas": "escrófula",
    "escrúpulos": "escrúpulo",
    "escuadra": "escuadrar",
    "escuadraba": "escuadrar",
    "escuadrabais": "escuadrar",
    "escuadraban": "escuadrar",
    "escuadrabas": "escuadrar",
    "escuadrad": "escuadrar",
    "escuadrada": "escuadrar",
    "escuadradas": "escuadrar",
    "escuadrado": "escuadrar",
    "escuadrados": "escuadrar",
    "escuadramos": "escuadrar",
    "escuadran": "escuadrar",
    "escuadrando": "escuadrar",
    "escuadrara": "escuadrar",
    "escuadrarais": "escuadrar",
    "escuadraran": "escuadrar",
    "escuadraras": "escuadrar",
    "escuadrare": "escuadrar",
    "escuadrareis": "escuadrar",
    "escuadraremos": "escuadrar",
    "escuadraren": "escuadrar",
    "escuadrares": "escuadrar",
    "escuadraron": "escuadrar",
    "escuadrará": "escuadrar",
    "escuadrarán": "escuadrar",
    "escuadrarás": "escuadrar",
    "escuadraré": "escuadrar",
    "escuadraréis": "escuadrar",
    "escuadraría": "escuadrar",
    "escuadraríais": "escuadrar",
    "escuadraríamos": "escuadrar",
    "escuadrarían": "escuadrar",
    "escuadrarías": "escuadrar",
    "escuadras": "escuadrar",
    "escuadrase": "escuadrar",
    "escuadraseis": "escuadrar",
    "escuadrasen": "escuadrar",
    "escuadrases": "escuadrar",
    "escuadraste": "escuadrar",
    "escuadrasteis": "escuadrar",
    "escuadre": "escuadrar",
    "escuadremos": "escuadrar",
    "escuadren": "escuadrar",
    "escuadreos": "escuadreo",
    "escuadres": "escuadrar",
    "escuadrillas": "escuadrilla",
    "escuadro": "escuadrar",
    "escuadrona": "escuadronar",
    "escuadronaba": "escuadronar",
    "escuadronabais": "escuadronar",
    "escuadronaban": "escuadronar",
    "escuadronabas": "escuadronar",
    "escuadronad": "escuadronar",
    "escuadronada": "escuadronar",
    "escuadronadas": "escuadronar",
    "escuadronado": "escuadronar",
    "escuadronados": "escuadronar",
    "escuadronamos": "escuadronar",
    "escuadronan": "escuadronar",
    "escuadronando": "escuadronar",
    "escuadronara": "escuadronar",
    "escuadronarais": "escuadronar",
    "escuadronaran": "escuadronar",
    "escuadronaras": "escuadronar",
    "escuadronare": "escuadronar",
    "escuadronareis": "escuadronar",
    "escuadronaremos": "escuadronar",
    "escuadronaren": "escuadronar",
    "escuadronares": "escuadronar",
    "escuadronaron": "escuadronar",
    "escuadronará": "escuadronar",
    "escuadronarán": "escuadronar",
    "escuadronarás": "escuadronar",
    "escuadronaré": "escuadronar",
    "escuadronaréis": "escuadronar",
    "escuadronaría": "escuadronar",
    "escuadronaríais": "escuadronar",
    "escuadronaríamos": "escuadronar",
    "escuadronarían": "escuadronar",
    "escuadronarías": "escuadronar",
    "escuadronas": "escuadronar",
    "escuadronase": "escuadronar",
    "escuadronaseis": "escuadronar",
    "escuadronasen": "escuadronar",
    "escuadronases": "escuadronar",
    "escuadronaste": "escuadronar",
    "escuadronasteis": "escuadronar",
    "escuadrone": "escuadronar",
    "escuadronemos": "escuadronar",
    "escuadronen": "escuadronar",
    "escuadrones": "escuadronar",
    "escuadrono": "escuadronar",
    "escuadronábamos": "escuadronar",
    "escuadronáis": "escuadronar",
    "escuadronáramos": "escuadronar",
    "escuadronáremos": "escuadronar",
    "escuadronásemos": "escuadronar",
    "escuadroné": "escuadronar",
    "escuadronéis": "escuadronar",
    "escuadronó": "escuadronar",
    "escuadrábamos": "escuadrar",
    "escuadráis": "escuadrar",
    "escuadráramos": "escuadrar",
    "escuadráremos": "escuadrar",
    "escuadrásemos": "escuadrar",
    "escuadré": "escuadrar",
    "escuadréis": "escuadrar",
    "escuadrías": "escuadría",
    "escuadró": "escuadrar",
    "escualideces": "escualidez",
    "escualores": "escualor",
    "escualos": "escualo",
    "escuaques": "escuaque",
    "escucha": "escuchar",
    "escuchaba": "escuchar",
    "escuchabais": "escuchar",
    "escuchaban": "escuchar",
    "escuchabas": "escuchar",
    "escuchad": "escuchar",
    "escuchada": "escuchar",
    "escuchadas": "escuchar",
    "escuchado": "escuchar",
    "escuchadores": "escuchador",
    "escuchados": "escuchar",
    "escuchamos": "escuchar",
    "escuchan": "escuchar",
    "escuchando": "escuchar",
    "escuchara": "escuchar",
    "escucharais": "escuchar",
    "escucharan": "escuchar",
    "escucharas": "escuchar",
    "escuchare": "escuchar",
    "escuchareis": "escuchar",
    "escucharemos": "escuchar",
    "escucharen": "escuchar",
    "escuchares": "escuchar",
    "escucharon": "escuchar",
    "escuchará": "escuchar",
    "escucharán": "escuchar",
    "escucharás": "escuchar",
    "escucharé": "escuchar",
    "escucharéis": "escuchar",
    "escucharía": "escuchar",
    "escucharíais": "escuchar",
    "escucharíamos": "escuchar",
    "escucharían": "escuchar",
    "escucharías": "escuchar",
    "escuchas": "escuchar",
    "escuchase": "escuchar",
    "escuchaseis": "escuchar",
    "escuchasen": "escuchar",
    "escuchases": "escuchar",
    "escuchaste": "escuchar",
    "escuchasteis": "escuchar",
    "escuche": "escuchar",
    "escuchemos": "escuchar",
    "escuchen": "escuchar",
    "escuches": "escuchar",
    "escuchimizada": "escuchimizado",
    "escuchimizadas": "escuchimizado",
    "escuchimizados": "escuchimizado",
    "escucho": "escuchar",
    "escuchábamos": "escuchar",
    "escucháis": "escuchar",
    "escucháramos": "escuchar",
    "escucháremos": "escuchar",
    "escuchásemos": "escuchar",
    "escuché": "escuchar",
    "escuchéis": "escuchar",
    "escuchó": "escuchar",
    "escuda": "escudar",
    "escudaba": "escudar",
    "escudabais": "escudar",
    "escudaban": "escudar",
    "escudabas": "escudar",
    "escudad": "escudar",
    "escudada": "escudar",
    "escudadas": "escudar",
    "escudado": "escudar",
    "escudados": "escudar",
    "escudamientos": "escudamiento",
    "escudamos": "escudar",
    "escudan": "escudar",
    "escudando": "escudar",
    "escudara": "escudar",
    "escudarais": "escudar",
    "escudaran": "escudar",
    "escudaras": "escudar",
    "escudare": "escudar",
    "escudareis": "escudar",
    "escudaremos": "escudar",
    "escudaren": "escudar",
    "escudares": "escudar",
    "escudaron": "escudar",
    "escudará": "escudar",
    "escudarán": "escudar",
    "escudarás": "escudar",
    "escudaré": "escudar",
    "escudaréis": "escudar",
    "escudaría": "escudar",
    "escudaríais": "escudar",
    "escudaríamos": "escudar",
    "escudarían": "escudar",
    "escudarías": "escudar",
    "escudas": "escudar",
    "escudase": "escudar",
    "escudaseis": "escudar",
    "escudasen": "escudar",
    "escudases": "escudar",
    "escudaste": "escudar",
    "escudasteis": "escudar",
    "escude": "escudar",
    "escudemos": "escudar",
    "escuden": "escudar",
    "escudera": "escudero",
    "escuderas": "escudero",
    "escuderea": "escuderear",
    "escudereaba": "escuderear",
    "escudereabais": "escuderear",
    "escudereaban": "escuderear",
    "escudereabas": "escuderear",
    "escuderead": "escuderear",
    "escudereada": "escuderear",
    "escudereadas": "escuderear",
    "escudereado": "escuderear",
    "escudereados": "escuderear",
    "escudereamos": "escuderear",
    "escuderean": "escuderear",
    "escudereando": "escuderear",
    "escudereara": "escuderear",
    "escuderearais": "escuderear",
    "escuderearan": "escuderear",
    "escuderearas": "escuderear",
    "escudereare": "escuderear",
    "escudereareis": "escuderear",
    "escuderearemos": "escuderear",
    "escuderearen": "escuderear",
    "escudereares": "escuderear",
    "escuderearon": "escuderear",
    "escudereará": "escuderear",
    "escuderearán": "escuderear",
    "escuderearás": "escuderear",
    "escuderearé": "escuderear",
    "escuderearéis": "escuderear",
    "escuderearía": "escuderear",
    "escuderearíais": "escuderear",
    "escuderearíamos": "escuderear",
    "escuderearían": "escuderear",
    "escuderearías": "escuderear",
    "escudereas": "escuderear",
    "escuderease": "escuderear",
    "escudereaseis": "escuderear",
    "escudereasen": "escuderear",
    "escudereases": "escuderear",
    "escudereaste": "escuderear",
    "escudereasteis": "escuderear",
    "escuderee": "escuderear",
    "escudereemos": "escuderear",
    "escudereen": "escuderear",
    "escuderees": "escuderear",
    "escudereo": "escuderear",
    "escudereábamos": "escuderear",
    "escudereáis": "escuderear",
    "escudereáramos": "escuderear",
    "escudereáremos": "escuderear",
    "escudereásemos": "escuderear",
    "escudereé": "escuderear",
    "escudereéis": "escuderear",
    "escudereó": "escuderear",
    "escuderiles": "escuderil",
    "escuderones": "escuderón",
    "escuderos": "escudero",
    "escuderías": "escudería",
    "escudes": "escudar",
    "escudetes": "escudete",
    "escudilla": "escudillar",
    "escudillaba": "escudillar",
    "escudillabais": "escudillar",
    "escudillaban": "escudillar",
    "escudillabas": "escudillar",
    "escudillad": "escudillar",
    "escudillada": "escudillar",
    "escudilladas": "escudillar",
    "escudillado": "escudillar",
    "escudillados": "escudillar",
    "escudillamos": "escudillar",
    "escudillan": "escudillar",
    "escudillando": "escudillar",
    "escudillara": "escudillar",
    "escudillarais": "escudillar",
    "escudillaran": "escudillar",
    "escudillaras": "escudillar",
    "escudillare": "escudillar",
    "escudillareis": "escudillar",
    "escudillaremos": "escudillar",
    "escudillaren": "escudillar",
    "escudillares": "escudillar",
    "escudillaron": "escudillar",
    "escudillará": "escudillar",
    "escudillarán": "escudillar",
    "escudillarás": "escudillar",
    "escudillaré": "escudillar",
    "escudillaréis": "escudillar",
    "escudillaría": "escudillar",
    "escudillaríais": "escudillar",
    "escudillaríamos": "escudillar",
    "escudillarían": "escudillar",
    "escudillarías": "escudillar",
    "escudillas": "escudillar",
    "escudillase": "escudillar",
    "escudillaseis": "escudillar",
    "escudillasen": "escudillar",
    "escudillases": "escudillar",
    "escudillaste": "escudillar",
    "escudillasteis": "escudillar",
    "escudille": "escudillar",
    "escudillemos": "escudillar",
    "escudillen": "escudillar",
    "escudilles": "escudillar",
    "escudillo": "escudillar",
    "escudillábamos": "escudillar",
    "escudilláis": "escudillar",
    "escudilláramos": "escudillar",
    "escudilláremos": "escudillar",
    "escudillásemos": "escudillar",
    "escudillé": "escudillar",
    "escudilléis": "escudillar",
    "escudilló": "escudillar",
    "escudo": "escudar",
    "escudos": "escudo",
    "escudriña": "escudriñar",
    "escudriñaba": "escudriñar",
    "escudriñabais": "escudriñar",
    "escudriñaban": "escudriñar",
    "escudriñabas": "escudriñar",
    "escudriñables": "escudriñable",
    "escudriñad": "escudriñar",
    "escudriñada": "escudriñar",
    "escudriñadas": "escudriñar",
    "escudriñado": "escudriñar",
    "escudriñadora": "escudriñador",
    "escudriñadoras": "escudriñador",
    "escudriñadores": "escudriñador",
    "escudriñados": "escudriñar",
    "escudriñamientos": "escudriñamiento",
    "escudriñamos": "escudriñar",
    "escudriñan": "escudriñar",
    "escudriñando": "escudriñar",
    "escudriñara": "escudriñar",
    "escudriñarais": "escudriñar",
    "escudriñaran": "escudriñar",
    "escudriñaras": "escudriñar",
    "escudriñare": "escudriñar",
    "escudriñareis": "escudriñar",
    "escudriñaremos": "escudriñar",
    "escudriñaren": "escudriñar",
    "escudriñares": "escudriñar",
    "escudriñaron": "escudriñar",
    "escudriñará": "escudriñar",
    "escudriñarán": "escudriñar",
    "escudriñarás": "escudriñar",
    "escudriñaré": "escudriñar",
    "escudriñaréis": "escudriñar",
    "escudriñaría": "escudriñar",
    "escudriñaríais": "escudriñar",
    "escudriñaríamos": "escudriñar",
    "escudriñarían": "escudriñar",
    "escudriñarías": "escudriñar",
    "escudriñas": "escudriñar",
    "escudriñase": "escudriñar",
    "escudriñaseis": "escudriñar",
    "escudriñasen": "escudriñar",
    "escudriñases": "escudriñar",
    "escudriñaste": "escudriñar",
    "escudriñasteis": "escudriñar",
    "escudriñe": "escudriñar",
    "escudriñemos": "escudriñar",
    "escudriñen": "escudriñar",
    "escudriñes": "escudriñar",
    "escudriño": "escudriñar",
    "escudriñábamos": "escudriñar",
    "escudriñáis": "escudriñar",
    "escudriñáramos": "escudriñar",
    "escudriñáremos": "escudriñar",
    "escudriñásemos": "escudriñar",
    "escudriñé": "escudriñar",
    "escudriñéis": "escudriñar",
    "escudriñó": "escudriñar",
    "escudábamos": "escudar",
    "escudáis": "escudar",
    "escudáramos": "escudar",
    "escudáremos": "escudar",
    "escudásemos": "escudar",
    "escudé": "escudar",
    "escudéis": "escudar",
    "escudó": "escudar",
    "escuece": "escocer",
    "escuecen": "escocer",
    "escueces": "escocer",
    "escuelas": "escuela",
    "escuerzos": "escuerzo",
    "escueta": "escueto",
    "escuetas": "escueto",
    "escuetos": "escueto",
    "escueza": "escocer",
    "escuezan": "escocer",
    "escuezas": "escocer",
    "escuezo": "escocer",
    "esculca": "esculcar",
    "esculcaba": "esculcar",
    "esculcabais": "esculcar",
    "esculcaban": "esculcar",
    "esculcabas": "esculcar",
    "esculcad": "esculcar",
    "esculcada": "esculcar",
    "esculcadas": "esculcar",
    "esculcado": "esculcar",
    "esculcados": "esculcar",
    "esculcamos": "esculcar",
    "esculcan": "esculcar",
    "esculcando": "esculcar",
    "esculcara": "esculcar",
    "esculcarais": "esculcar",
    "esculcaran": "esculcar",
    "esculcaras": "esculcar",
    "esculcare": "esculcar",
    "esculcareis": "esculcar",
    "esculcaremos": "esculcar",
    "esculcaren": "esculcar",
    "esculcares": "esculcar",
    "esculcaron": "esculcar",
    "esculcará": "esculcar",
    "esculcarán": "esculcar",
    "esculcarás": "esculcar",
    "esculcaré": "esculcar",
    "esculcaréis": "esculcar",
    "esculcaría": "esculcar",
    "esculcaríais": "esculcar",
    "esculcaríamos": "esculcar",
    "esculcarían": "esculcar",
    "esculcarías": "esculcar",
    "esculcas": "esculcar",
    "esculcase": "esculcar",
    "esculcaseis": "esculcar",
    "esculcasen": "esculcar",
    "esculcases": "esculcar",
    "esculcaste": "esculcar",
    "esculcasteis": "esculcar",
    "esculco": "esculcar",
    "esculcábamos": "esculcar",
    "esculcáis": "esculcar",
    "esculcáramos": "esculcar",
    "esculcáremos": "esculcar",
    "esculcásemos": "esculcar",
    "esculcó": "esculcar",
    "esculinas": "esculina",
    "esculladores": "escullador",
    "esculpa": "esculpir",
    "esculpamos": "esculpir",
    "esculpan": "esculpir",
    "esculpas": "esculpir",
    "esculpe": "esculpir",
    "esculpen": "esculpir",
    "esculpes": "esculpir",
    "esculpid": "esculpir",
    "esculpida": "esculpir",
    "esculpidas": "esculpir",
    "esculpido": "esculpir",
    "esculpidos": "esculpir",
    "esculpiendo": "esculpir",
    "esculpiera": "esculpir",
    "esculpierais": "esculpir",
    "esculpieran": "esculpir",
    "esculpieras": "esculpir",
    "esculpiere": "esculpir",
    "esculpiereis": "esculpir",
    "esculpieren": "esculpir",
    "esculpieres": "esculpir",
    "esculpieron": "esculpir",
    "esculpiese": "esculpir",
    "esculpieseis": "esculpir",
    "esculpiesen": "esculpir",
    "esculpieses": "esculpir",
    "esculpimos": "esculpir",
    "esculpiremos": "esculpir",
    "esculpirá": "esculpir",
    "esculpirán": "esculpir",
    "esculpirás": "esculpir",
    "esculpiré": "esculpir",
    "esculpiréis": "esculpir",
    "esculpiría": "esculpir",
    "esculpiríais": "esculpir",
    "esculpiríamos": "esculpir",
    "esculpirían": "esculpir",
    "esculpirías": "esculpir",
    "esculpiste": "esculpir",
    "esculpisteis": "esculpir",
    "esculpiéramos": "esculpir",
    "esculpiéremos": "esculpir",
    "esculpiésemos": "esculpir",
    "esculpió": "esculpir",
    "esculpo": "esculpir",
    "esculpáis": "esculpir",
    "esculpí": "esculpir",
    "esculpía": "esculpir",
    "esculpíais": "esculpir",
    "esculpíamos": "esculpir",
    "esculpían": "esculpir",
    "esculpías": "esculpir",
    "esculpís": "esculpir",
    "esculque": "esculcar",
    "esculquemos": "esculcar",
    "esculquen": "esculcar",
    "esculques": "esculcar",
    "esculqué": "esculcar",
    "esculquéis": "esculcar",
    "escultismos": "escultismo",
    "escultora": "escultor",
    "escultoras": "escultor",
    "escultores": "escultor",
    "esculturales": "escultural",
    "esculturas": "escultura",
    "escultórica": "escultórico",
    "escultóricas": "escultórico",
    "escultóricos": "escultórico",
    "escunas": "escuna",
    "escupa": "escupir",
    "escupamos": "escupir",
    "escupan": "escupir",
    "escupas": "escupir",
    "escupe": "escupir",
    "escupen": "escupir",
    "escupes": "escupir",
    "escupetinas": "escupetina",
    "escupid": "escupir",
    "escupida": "escupir",
    "escupidas": "escupir",
    "escupideras": "escupidera",
    "escupideros": "escupidero",
    "escupido": "escupir",
    "escupidores": "escupidor",
    "escupidos": "escupir",
    "escupiduras": "escupidura",
    "escupiendo": "escupir",
    "escupiera": "escupir",
    "escupierais": "escupir",
    "escupieran": "escupir",
    "escupieras": "escupir",
    "escupiere": "escupir",
    "escupiereis": "escupir",
    "escupieren": "escupir",
    "escupieres": "escupir",
    "escupieron": "escupir",
    "escupiese": "escupir",
    "escupieseis": "escupir",
    "escupiesen": "escupir",
    "escupieses": "escupir",
    "escupimos": "escupir",
    "escupiremos": "escupir",
    "escupirá": "escupir",
    "escupirán": "escupir",
    "escupirás": "escupir",
    "escupiré": "escupir",
    "escupiréis": "escupir",
    "escupiría": "escupir",
    "escupiríais": "escupir",
    "escupiríamos": "escupir",
    "escupirían": "escupir",
    "escupirías": "escupir",
    "escupiste": "escupir",
    "escupisteis": "escupir",
    "escupitajos": "escupitajo",
    "escupitinajos": "escupitinajo",
    "escupitinas": "escupitina",
    "escupiéramos": "escupir",
    "escupiéremos": "escupir",
    "escupiésemos": "escupir",
    "escupiñas": "escupiña",
    "escupió": "escupir",
    "escupo": "escupir",
    "escupáis": "escupir",
    "escupí": "escupir",
    "escupía": "escupir",
    "escupíais": "escupir",
    "escupíamos": "escupir",
    "escupían": "escupir",
    "escupías": "escupir",
    "escupís": "escupir",
    "escurialenses": "escurialense",
    "escurra": "escurrir",
    "escurrajas": "escurraja",
    "escurramos": "escurrir",
    "escurran": "escurrir",
    "escurras": "escurrir",
    "escurre": "escurrir",
    "escurren": "escurrir",
    "escurres": "escurrir",
    "escurribandas": "escurribanda",
    "escurrid": "escurrir",
    "escurrida": "escurrir",
    "escurridas": "escurrir",
    "escurridera": "escurridero",
    "escurrideras": "escurridero",
    "escurrideros": "escurridero",
    "escurridiza": "escurridizo",
    "escurridizas": "escurridizo",
    "escurridizos": "escurridizo",
    "escurrido": "escurrir",
    "escurridora": "escurridor",
    "escurridoras": "escurridor",
    "escurridores": "escurridor",
    "escurridos": "escurrir",
    "escurriendo": "escurrir",
    "escurriera": "escurrir",
    "escurrierais": "escurrir",
    "escurrieran": "escurrir",
    "escurrieras": "escurrir",
    "escurriere": "escurrir",
    "escurriereis": "escurrir",
    "escurrieren": "escurrir",
    "escurrieres": "escurrir",
    "escurrieron": "escurrir",
    "escurriese": "escurrir",
    "escurrieseis": "escurrir",
    "escurriesen": "escurrir",
    "escurrieses": "escurrir",
    "escurrimientos": "escurrimiento",
    "escurrimos": "escurrir",
    "escurriremos": "escurrir",
    "escurrirá": "escurrir",
    "escurrirán": "escurrir",
    "escurrirás": "escurrir",
    "escurriré": "escurrir",
    "escurriréis": "escurrir",
    "escurriría": "escurrir",
    "escurriríais": "escurrir",
    "escurriríamos": "escurrir",
    "escurrirían": "escurrir",
    "escurrirías": "escurrir",
    "escurriste": "escurrir",
    "escurristeis": "escurrir",
    "escurriéramos": "escurrir",
    "escurriéremos": "escurrir",
    "escurriésemos": "escurrir",
    "escurrió": "escurrir",
    "escurro": "escurrir",
    "escurráis": "escurrir",
    "escurrí": "escurrir",
    "escurría": "escurrir",
    "escurríais": "escurrir",
    "escurríamos": "escurrir",
    "escurrían": "escurrir",
    "escurrías": "escurrir",
    "escurrís": "escurrir",
    "escursanas": "escursana",
    "escusabarajas": "escusabaraja",
    "escusada": "escusado",
    "escusadas": "escusado",
    "escusados": "escusado",
    "escusas": "escusa",
    "escusones": "escusón",
    "escutelos": "escutelo",
    "escutiformes": "escutiforme",
    "escuálida": "escuálido",
    "escuálidas": "escuálido",
    "escuálidos": "escuálido",
    "escálamos": "escálamo",
    "escándalos": "escándalo",
    "escáneres": "escáner",
    "escánerses": "escáner",
    "escápulas": "escápula",
    "escénica": "escénico",
    "escénicas": "escénico",
    "escénicos": "escénico",
    "escéptica": "escéptico",
    "escépticas": "escéptico",
    "escépticos": "escéptico",
    "escíbalos": "escíbalo",
    "escíncidos": "escíncido",
    "escómbridos": "escómbrido",
    "escópulas": "escópula",
    "escórporas": "escórpora",
    "esdrujulice": "esdrujulizar",
    "esdrujulicemos": "esdrujulizar",
    "esdrujulicen": "esdrujulizar",
    "esdrujulices": "esdrujulizar",
    "esdrujulicé": "esdrujulizar",
    "esdrujulicéis": "esdrujulizar",
    "esdrujuliza": "esdrujulizar",
    "esdrujulizaba": "esdrujulizar",
    "esdrujulizabais": "esdrujulizar",
    "esdrujulizaban": "esdrujulizar",
    "esdrujulizabas": "esdrujulizar",
    "esdrujulizad": "esdrujulizar",
    "esdrujulizada": "esdrujulizar",
    "esdrujulizadas": "esdrujulizar",
    "esdrujulizado": "esdrujulizar",
    "esdrujulizados": "esdrujulizar",
    "esdrujulizamos": "esdrujulizar",
    "esdrujulizan": "esdrujulizar",
    "esdrujulizando": "esdrujulizar",
    "esdrujulizara": "esdrujulizar",
    "esdrujulizarais": "esdrujulizar",
    "esdrujulizaran": "esdrujulizar",
    "esdrujulizaras": "esdrujulizar",
    "esdrujulizare": "esdrujulizar",
    "esdrujulizareis": "esdrujulizar",
    "esdrujulizaremos": "esdrujulizar",
    "esdrujulizaren": "esdrujulizar",
    "esdrujulizares": "esdrujulizar",
    "esdrujulizaron": "esdrujulizar",
    "esdrujulizará": "esdrujulizar",
    "esdrujulizarán": "esdrujulizar",
    "esdrujulizarás": "esdrujulizar",
    "esdrujulizaré": "esdrujulizar",
    "esdrujulizaréis": "esdrujulizar",
    "esdrujulizaría": "esdrujulizar",
    "esdrujulizaríais": "esdrujulizar",
    "esdrujulizaríamos": "esdrujulizar",
    "esdrujulizarían": "esdrujulizar",
    "esdrujulizarías": "esdrujulizar",
    "esdrujulizas": "esdrujulizar",
    "esdrujulizase": "esdrujulizar",
    "esdrujulizaseis": "esdrujulizar",
    "esdrujulizasen": "esdrujulizar",
    "esdrujulizases": "esdrujulizar",
    "esdrujulizaste": "esdrujulizar",
    "esdrujulizasteis": "esdrujulizar",
    "esdrujulizo": "esdrujulizar",
    "esdrujulizábamos": "esdrujulizar",
    "esdrujulizáis": "esdrujulizar",
    "esdrujulizáramos": "esdrujulizar",
    "esdrujulizáremos": "esdrujulizar",
    "esdrujulizásemos": "esdrujulizar",
    "esdrujulizó": "esdrujulizar",
    "esdrújula": "esdrújulo",
    "esdrújulas": "esdrújulo",
    "esdrújulos": "esdrújulo",
    "esenciales": "esencial",
    "esencialidades": "esencialidad",
    "esencialismos": "esencialismo",
    "esencialistas": "esencialista",
    "esencias": "esencia",
    "esencieros": "esenciero",
    "esenia": "esenio",
    "esenias": "esenio",
    "esenios": "esenio",
    "esfacela": "esfacelar",
    "esfacelaba": "esfacelar",
    "esfacelabais": "esfacelar",
    "esfacelaban": "esfacelar",
    "esfacelabas": "esfacelar",
    "esfacelad": "esfacelar",
    "esfacelada": "esfacelar",
    "esfaceladas": "esfacelar",
    "esfacelado": "esfacelar",
    "esfacelados": "esfacelar",
    "esfacelamos": "esfacelar",
    "esfacelan": "esfacelar",
    "esfacelando": "esfacelar",
    "esfacelara": "esfacelar",
    "esfacelarais": "esfacelar",
    "esfacelaran": "esfacelar",
    "esfacelaras": "esfacelar",
    "esfacelare": "esfacelar",
    "esfacelareis": "esfacelar",
    "esfacelaremos": "esfacelar",
    "esfacelaren": "esfacelar",
    "esfacelares": "esfacelar",
    "esfacelaron": "esfacelar",
    "esfacelará": "esfacelar",
    "esfacelarán": "esfacelar",
    "esfacelarás": "esfacelar",
    "esfacelaré": "esfacelar",
    "esfacelaréis": "esfacelar",
    "esfacelaría": "esfacelar",
    "esfacelaríais": "esfacelar",
    "esfacelaríamos": "esfacelar",
    "esfacelarían": "esfacelar",
    "esfacelarías": "esfacelar",
    "esfacelas": "esfacelar",
    "esfacelase": "esfacelar",
    "esfacelaseis": "esfacelar",
    "esfacelasen": "esfacelar",
    "esfacelases": "esfacelar",
    "esfacelaste": "esfacelar",
    "esfacelasteis": "esfacelar",
    "esfacele": "esfacelar",
    "esfacelemos": "esfacelar",
    "esfacelen": "esfacelar",
    "esfaceles": "esfacelar",
    "esfacelo": "esfacelar",
    "esfacelos": "esfacelo",
    "esfacelábamos": "esfacelar",
    "esfaceláis": "esfacelar",
    "esfaceláramos": "esfacelar",
    "esfaceláremos": "esfacelar",
    "esfacelásemos": "esfacelar",
    "esfacelé": "esfacelar",
    "esfaceléis": "esfacelar",
    "esfaceló": "esfacelar",
    "esfaleritas": "esfalerita",
    "esfenas": "esfena",
    "esfenocigomática": "esfenocigomático",
    "esfenocigomáticas": "esfenocigomático",
    "esfenocigomáticos": "esfenocigomático",
    "esfenoedros": "esfenoedro",
    "esfenoetmoidales": "esfenoetmoidal",
    "esfenofrontales": "esfenofrontal",
    "esfenoidales": "esfenoidal",
    "esfenoidotomías": "esfenoidotomía",
    "esfenomandibulares": "esfenomandibular",
    "esfenomaxilares": "esfenomaxilar",
    "esfenooccipitales": "esfenooccipital",
    "esfenopalatina": "esfenopalatino",
    "esfenopalatinas": "esfenopalatino",
    "esfenopalatinos": "esfenopalatino",
    "esfenoparietales": "esfenoparietal",
    "esfenopetrosa": "esfenopetroso",
    "esfenopetrosas": "esfenopetroso",
    "esfenopetrosos": "esfenopetroso",
    "esfenotemporales": "esfenotemporal",
    "esferales": "esferal",
    "esferas": "esfera",
    "esfericidades": "esfericidad",
    "esferocitos": "esferocito",
    "esferoidales": "esferoidal",
    "esferoidea": "esferoideo",
    "esferoideas": "esferoideo",
    "esferoideos": "esferoideo",
    "esferoides": "esferoide",
    "esferoidizaciones": "esferoidización",
    "esferolitos": "esferolito",
    "esferomas": "esferoma",
    "esferoplastos": "esferoplasto",
    "esferosomas": "esferosoma",
    "esferulítica": "esferulítico",
    "esferulíticas": "esferulítico",
    "esferulíticos": "esferulítico",
    "esferómetros": "esferómetro",
    "esfigmografías": "esfigmografía",
    "esfigmomanómetros": "esfigmomanómetro",
    "esfigmógrafos": "esfigmógrafo",
    "esfinges": "esfinge",
    "esfingomielinas": "esfingomielina",
    "esfingosinas": "esfingosina",
    "esfirénidos": "esfirénido",
    "esforcemos": "esforzar",
    "esforcé": "esforzar",
    "esforcéis": "esforzar",
    "esforrocinos": "esforrocino",
    "esforzaba": "esforzar",
    "esforzabais": "esforzar",
    "esforzaban": "esforzar",
    "esforzabas": "esforzar",
    "esforzad": "esforzar",
    "esforzada": "esforzar",
    "esforzadas": "esforzar",
    "esforzado": "esforzar",
    "esforzados": "esforzar",
    "esforzamos": "esforzar",
    "esforzando": "esforzar",
    "esforzara": "esforzar",
    "esforzarais": "esforzar",
    "esforzaran": "esforzar",
    "esforzaras": "esforzar",
    "esforzare": "esforzar",
    "esforzareis": "esforzar",
    "esforzaremos": "esforzar",
    "esforzaren": "esforzar",
    "esforzares": "esforzar",
    "esforzaron": "esforzar",
    "esforzará": "esforzar",
    "esforzarán": "esforzar",
    "esforzarás": "esforzar",
    "esforzaré": "esforzar",
    "esforzaréis": "esforzar",
    "esforzaría": "esforzar",
    "esforzaríais": "esforzar",
    "esforzaríamos": "esforzar",
    "esforzarían": "esforzar",
    "esforzarías": "esforzar",
    "esforzase": "esforzar",
    "esforzaseis": "esforzar",
    "esforzasen": "esforzar",
    "esforzases": "esforzar",
    "esforzaste": "esforzar",
    "esforzasteis": "esforzar",
    "esforzábamos": "esforzar",
    "esforzáis": "esforzar",
    "esforzáramos": "esforzar",
    "esforzáremos": "esforzar",
    "esforzásemos": "esforzar",
    "esforzó": "esforzar",
    "esfuerce": "esforzar",
    "esfuercen": "esforzar",
    "esfuerces": "esforzar",
    "esfuerza": "esforzar",
    "esfuerzan": "esforzar",
    "esfuerzas": "esforzar",
    "esfuerzo": "esforzar",
    "esfuerzos": "esfuerzo",
    "esfuma": "esfumar",
    "esfumaba": "esfumar",
    "esfumabais": "esfumar",
    "esfumaban": "esfumar",
    "esfumabas": "esfumar",
    "esfumaciones": "esfumación",
    "esfumad": "esfumar",
    "esfumada": "esfumar",
    "esfumadas": "esfumar",
    "esfumado": "esfumar",
    "esfumados": "esfumar",
    "esfumamos": "esfumar",
    "esfuman": "esfumar",
    "esfumando": "esfumar",
    "esfumara": "esfumar",
    "esfumarais": "esfumar",
    "esfumaran": "esfumar",
    "esfumaras": "esfumar",
    "esfumare": "esfumar",
    "esfumareis": "esfumar",
    "esfumaremos": "esfumar",
    "esfumaren": "esfumar",
    "esfumares": "esfumar",
    "esfumaron": "esfumar",
    "esfumará": "esfumar",
    "esfumarán": "esfumar",
    "esfumarás": "esfumar",
    "esfumaré": "esfumar",
    "esfumaréis": "esfumar",
    "esfumaría": "esfumar",
    "esfumaríais": "esfumar",
    "esfumaríamos": "esfumar",
    "esfumarían": "esfumar",
    "esfumarías": "esfumar",
    "esfumas": "esfumar",
    "esfumase": "esfumar",
    "esfumaseis": "esfumar",
    "esfumasen": "esfumar",
    "esfumases": "esfumar",
    "esfumaste": "esfumar",
    "esfumasteis": "esfumar",
    "esfume": "esfumar",
    "esfumemos": "esfumar",
    "esfumen": "esfumar",
    "esfumes": "esfumar",
    "esfumina": "esfuminar",
    "esfuminaba": "esfuminar",
    "esfuminabais": "esfuminar",
    "esfuminaban": "esfuminar",
    "esfuminabas": "esfuminar",
    "esfuminad": "esfuminar",
    "esfuminada": "esfuminar",
    "esfuminadas": "esfuminar",
    "esfuminado": "esfuminar",
    "esfuminados": "esfuminar",
    "esfuminamos": "esfuminar",
    "esfuminan": "esfuminar",
    "esfuminando": "esfuminar",
    "esfuminara": "esfuminar",
    "esfuminarais": "esfuminar",
    "esfuminaran": "esfuminar",
    "esfuminaras": "esfuminar",
    "esfuminare": "esfuminar",
    "esfuminareis": "esfuminar",
    "esfuminaremos": "esfuminar",
    "esfuminaren": "esfuminar",
    "esfuminares": "esfuminar",
    "esfuminaron": "esfuminar",
    "esfuminará": "esfuminar",
    "esfuminarán": "esfuminar",
    "esfuminarás": "esfuminar",
    "esfuminaré": "esfuminar",
    "esfuminaréis": "esfuminar",
    "esfuminaría": "esfuminar",
    "esfuminaríais": "esfuminar",
    "esfuminaríamos": "esfuminar",
    "esfuminarían": "esfuminar",
    "esfuminarías": "esfuminar",
    "esfuminas": "esfuminar",
    "esfuminase": "esfuminar",
    "esfuminaseis": "esfuminar",
    "esfuminasen": "esfuminar",
    "esfuminases": "esfuminar",
    "esfuminaste": "esfuminar",
    "esfuminasteis": "esfuminar",
    "esfumine": "esfuminar",
    "esfuminemos": "esfuminar",
    "esfuminen": "esfuminar",
    "esfumines": "esfuminar",
    "esfumino": "esfuminar",
    "esfuminos": "esfumino",
    "esfuminábamos": "esfuminar",
    "esfumináis": "esfuminar",
    "esfumináramos": "esfuminar",
    "esfumináremos": "esfuminar",
    "esfuminásemos": "esfuminar",
    "esfuminé": "esfuminar",
    "esfuminéis": "esfuminar",
    "esfuminó": "esfuminar",
    "esfumo": "esfumar",
    "esfumábamos": "esfumar",
    "esfumáis": "esfumar",
    "esfumáramos": "esfumar",
    "esfumáremos": "esfumar",
    "esfumásemos": "esfumar",
    "esfumé": "esfumar",
    "esfuméis": "esfumar",
    "esfumó": "esfumar",
    "esfágnica": "esfágnico",
    "esfágnicas": "esfágnico",
    "esfágnicos": "esfágnico",
    "esférica": "esférico",
    "esféricas": "esférico",
    "esféricos": "esférico",
    "esfíngidos": "esfíngido",
    "esfínteres": "esfínter",
    "esgrafiaba": "esgrafiar",
    "esgrafiabais": "esgrafiar",
    "esgrafiaban": "esgrafiar",
    "esgrafiabas": "esgrafiar",
    "esgrafiad": "esgrafiar",
    "esgrafiada": "esgrafiar",
    "esgrafiadas": "esgrafiar",
    "esgrafiado": "esgrafiar",
    "esgrafiados": "esgrafiar",
    "esgrafiamos": "esgrafiar",
    "esgrafiando": "esgrafiar",
    "esgrafiara": "esgrafiar",
    "esgrafiarais": "esgrafiar",
    "esgrafiaran": "esgrafiar",
    "esgrafiaras": "esgrafiar",
    "esgrafiare": "esgrafiar",
    "esgrafiareis": "esgrafiar",
    "esgrafiaremos": "esgrafiar",
    "esgrafiaren": "esgrafiar",
    "esgrafiares": "esgrafiar",
    "esgrafiaron": "esgrafiar",
    "esgrafiará": "esgrafiar",
    "esgrafiarán": "esgrafiar",
    "esgrafiarás": "esgrafiar",
    "esgrafiaré": "esgrafiar",
    "esgrafiaréis": "esgrafiar",
    "esgrafiaría": "esgrafiar",
    "esgrafiaríais": "esgrafiar",
    "esgrafiaríamos": "esgrafiar",
    "esgrafiarían": "esgrafiar",
    "esgrafiarías": "esgrafiar",
    "esgrafiase": "esgrafiar",
    "esgrafiaseis": "esgrafiar",
    "esgrafiasen": "esgrafiar",
    "esgrafiases": "esgrafiar",
    "esgrafiaste": "esgrafiar",
    "esgrafiasteis": "esgrafiar",
    "esgrafiemos": "esgrafiar",
    "esgrafiábamos": "esgrafiar",
    "esgrafiáis": "esgrafiar",
    "esgrafiáramos": "esgrafiar",
    "esgrafiáremos": "esgrafiar",
    "esgrafiásemos": "esgrafiar",
    "esgrafié": "esgrafiar",
    "esgrafiéis": "esgrafiar",
    "esgrafió": "esgrafiar",
    "esgrafía": "esgrafiar",
    "esgrafían": "esgrafiar",
    "esgrafías": "esgrafiar",
    "esgrafíe": "esgrafiar",
    "esgrafíen": "esgrafiar",
    "esgrafíes": "esgrafiar",
    "esgrafío": "esgrafiar",
    "esgrima": "esgrimir",
    "esgrimadora": "esgrimador",
    "esgrimadoras": "esgrimador",
    "esgrimadores": "esgrimador",
    "esgrimamos": "esgrimir",
    "esgriman": "esgrimir",
    "esgrimas": "esgrimir",
    "esgrime": "esgrimir",
    "esgrimen": "esgrimir",
    "esgrimes": "esgrimir",
    "esgrimid": "esgrimir",
    "esgrimida": "esgrimir",
    "esgrimidas": "esgrimir",
    "esgrimido": "esgrimir",
    "esgrimidora": "esgrimidor",
    "esgrimidoras": "esgrimidor",
    "esgrimidores": "esgrimidor",
    "esgrimidos": "esgrimir",
    "esgrimiduras": "esgrimidura",
    "esgrimiendo": "esgrimir",
    "esgrimiera": "esgrimir",
    "esgrimierais": "esgrimir",
    "esgrimieran": "esgrimir",
    "esgrimieras": "esgrimir",
    "esgrimiere": "esgrimir",
    "esgrimiereis": "esgrimir",
    "esgrimieren": "esgrimir",
    "esgrimieres": "esgrimir",
    "esgrimieron": "esgrimir",
    "esgrimiese": "esgrimir",
    "esgrimieseis": "esgrimir",
    "esgrimiesen": "esgrimir",
    "esgrimieses": "esgrimir",
    "esgrimimos": "esgrimir",
    "esgrimiremos": "esgrimir",
    "esgrimirá": "esgrimir",
    "esgrimirán": "esgrimir",
    "esgrimirás": "esgrimir",
    "esgrimiré": "esgrimir",
    "esgrimiréis": "esgrimir",
    "esgrimiría": "esgrimir",
    "esgrimiríais": "esgrimir",
    "esgrimiríamos": "esgrimir",
    "esgrimirían": "esgrimir",
    "esgrimirías": "esgrimir",
    "esgrimistas": "esgrimista",
    "esgrimiste": "esgrimir",
    "esgrimisteis": "esgrimir",
    "esgrimiéramos": "esgrimir",
    "esgrimiéremos": "esgrimir",
    "esgrimiésemos": "esgrimir",
    "esgrimió": "esgrimir",
    "esgrimo": "esgrimir",
    "esgrimáis": "esgrimir",
    "esgrimí": "esgrimir",
    "esgrimía": "esgrimir",
    "esgrimíais": "esgrimir",
    "esgrimíamos": "esgrimir",
    "esgrimían": "esgrimir",
    "esgrimías": "esgrimir",
    "esgrimís": "esgrimir",
    "esguace": "esguazar",
    "esguacemos": "esguazar",
    "esguacen": "esguazar",
    "esguaces": "esguazar",
    "esguacé": "esguazar",
    "esguacéis": "esguazar",
    "esguaza": "esguazar",
    "esguazaba": "esguazar",
    "esguazabais": "esguazar",
    "esguazaban": "esguazar",
    "esguazabas": "esguazar",
    "esguazables": "esguazable",
    "esguazad": "esguazar",
    "esguazada": "esguazar",
    "esguazadas": "esguazar",
    "esguazado": "esguazar",
    "esguazados": "esguazar",
    "esguazamos": "esguazar",
    "esguazan": "esguazar",
    "esguazando": "esguazar",
    "esguazara": "esguazar",
    "esguazarais": "esguazar",
    "esguazaran": "esguazar",
    "esguazaras": "esguazar",
    "esguazare": "esguazar",
    "esguazareis": "esguazar",
    "esguazaremos": "esguazar",
    "esguazaren": "esguazar",
    "esguazares": "esguazar",
    "esguazaron": "esguazar",
    "esguazará": "esguazar",
    "esguazarán": "esguazar",
    "esguazarás": "esguazar",
    "esguazaré": "esguazar",
    "esguazaréis": "esguazar",
    "esguazaría": "esguazar",
    "esguazaríais": "esguazar",
    "esguazaríamos": "esguazar",
    "esguazarían": "esguazar",
    "esguazarías": "esguazar",
    "esguazas": "esguazar",
    "esguazase": "esguazar",
    "esguazaseis": "esguazar",
    "esguazasen": "esguazar",
    "esguazases": "esguazar",
    "esguazaste": "esguazar",
    "esguazasteis": "esguazar",
    "esguazo": "esguazar",
    "esguazos": "esguazo",
    "esguazábamos": "esguazar",
    "esguazáis": "esguazar",
    "esguazáramos": "esguazar",
    "esguazáremos": "esguazar",
    "esguazásemos": "esguazar",
    "esguazó": "esguazar",
    "esgucios": "esgucio",
    "esguinces": "esguince",
    "esguines": "esguín",
    "esiva": "esivo",
    "esivas": "esivo",
    "esivos": "esivo",
    "eslabona": "eslabonar",
    "eslabonaba": "eslabonar",
    "eslabonabais": "eslabonar",
    "eslabonaban": "eslabonar",
    "eslabonabas": "eslabonar",
    "eslabonad": "eslabonar",
    "eslabonada": "eslabonar",
    "eslabonadas": "eslabonar",
    "eslabonado": "eslabonar",
    "eslabonadora": "eslabonador",
    "eslabonadoras": "eslabonador",
    "eslabonadores": "eslabonador",
    "eslabonados": "eslabonar",
    "eslabonamientos": "eslabonamiento",
    "eslabonamos": "eslabonar",
    "eslabonan": "eslabonar",
    "eslabonando": "eslabonar",
    "eslabonara": "eslabonar",
    "eslabonarais": "eslabonar",
    "eslabonaran": "eslabonar",
    "eslabonaras": "eslabonar",
    "eslabonare": "eslabonar",
    "eslabonareis": "eslabonar",
    "eslabonaremos": "eslabonar",
    "eslabonaren": "eslabonar",
    "eslabonares": "eslabonar",
    "eslabonaron": "eslabonar",
    "eslabonará": "eslabonar",
    "eslabonarán": "eslabonar",
    "eslabonarás": "eslabonar",
    "eslabonaré": "eslabonar",
    "eslabonaréis": "eslabonar",
    "eslabonaría": "eslabonar",
    "eslabonaríais": "eslabonar",
    "eslabonaríamos": "eslabonar",
    "eslabonarían": "eslabonar",
    "eslabonarías": "eslabonar",
    "eslabonas": "eslabonar",
    "eslabonase": "eslabonar",
    "eslabonaseis": "eslabonar",
    "eslabonasen": "eslabonar",
    "eslabonases": "eslabonar",
    "eslabonaste": "eslabonar",
    "eslabonasteis": "eslabonar",
    "eslabone": "eslabonar",
    "eslabonemos": "eslabonar",
    "eslabonen": "eslabonar",
    "eslabones": "eslabonar",
    "eslabono": "eslabonar",
    "eslabonábamos": "eslabonar",
    "eslabonáis": "eslabonar",
    "eslabonáramos": "eslabonar",
    "eslabonáremos": "eslabonar",
    "eslabonásemos": "eslabonar",
    "eslaboné": "eslabonar",
    "eslabonéis": "eslabonar",
    "eslabonó": "eslabonar",
    "eslalom": "eslálom",
    "eslaloms": "eslálom",
    "eslava": "eslavo",
    "eslavas": "eslavo",
    "eslavos": "eslavo",
    "eslavófilos": "eslavófilo",
    "eslinga": "eslingar",
    "eslingaba": "eslingar",
    "eslingabais": "eslingar",
    "eslingaban": "eslingar",
    "eslingabas": "eslingar",
    "eslingad": "eslingar",
    "eslingada": "eslingar",
    "eslingadas": "eslingar",
    "eslingado": "eslingar",
    "eslingados": "eslingar",
    "eslingamos": "eslingar",
    "eslingan": "eslingar",
    "eslingando": "eslingar",
    "eslingara": "eslingar",
    "eslingarais": "eslingar",
    "eslingaran": "eslingar",
    "eslingaras": "eslingar",
    "eslingare": "eslingar",
    "eslingareis": "eslingar",
    "eslingaremos": "eslingar",
    "eslingaren": "eslingar",
    "eslingares": "eslingar",
    "eslingaron": "eslingar",
    "eslingará": "eslingar",
    "eslingarán": "eslingar",
    "eslingarás": "eslingar",
    "eslingaré": "eslingar",
    "eslingaréis": "eslingar",
    "eslingaría": "eslingar",
    "eslingaríais": "eslingar",
    "eslingaríamos": "eslingar",
    "eslingarían": "eslingar",
    "eslingarías": "eslingar",
    "eslingas": "eslingar",
    "eslingase": "eslingar",
    "eslingaseis": "eslingar",
    "eslingasen": "eslingar",
    "eslingases": "eslingar",
    "eslingaste": "eslingar",
    "eslingasteis": "eslingar",
    "eslingo": "eslingar",
    "eslingue": "eslingar",
    "eslinguemos": "eslingar",
    "eslinguen": "eslingar",
    "eslingues": "eslingar",
    "eslingué": "eslingar",
    "eslinguéis": "eslingar",
    "eslingábamos": "eslingar",
    "eslingáis": "eslingar",
    "eslingáramos": "eslingar",
    "eslingáremos": "eslingar",
    "eslingásemos": "eslingar",
    "eslingó": "eslingar",
    "eslizones": "eslizón",
    "esloganes": "eslogan",
    "esloras": "eslora",
    "eslovaca": "eslovaco",
    "eslovacas": "eslovaco",
    "eslovacos": "eslovaco",
    "eslovena": "esloveno",
    "eslovenas": "esloveno",
    "eslovenos": "esloveno",
    "esláloms": "eslálom",
    "eslóganes": "eslógan",
    "esmalta": "esmaltar",
    "esmaltaba": "esmaltar",
    "esmaltabais": "esmaltar",
    "esmaltaban": "esmaltar",
    "esmaltabas": "esmaltar",
    "esmaltad": "esmaltar",
    "esmaltada": "esmaltar",
    "esmaltadas": "esmaltar",
    "esmaltado": "esmaltar",
    "esmaltados": "esmaltar",
    "esmaltamos": "esmaltar",
    "esmaltan": "esmaltar",
    "esmaltando": "esmaltar",
    "esmaltara": "esmaltar",
    "esmaltarais": "esmaltar",
    "esmaltaran": "esmaltar",
    "esmaltaras": "esmaltar",
    "esmaltare": "esmaltar",
    "esmaltareis": "esmaltar",
    "esmaltaremos": "esmaltar",
    "esmaltaren": "esmaltar",
    "esmaltares": "esmaltar",
    "esmaltaron": "esmaltar",
    "esmaltará": "esmaltar",
    "esmaltarán": "esmaltar",
    "esmaltarás": "esmaltar",
    "esmaltaré": "esmaltar",
    "esmaltaréis": "esmaltar",
    "esmaltaría": "esmaltar",
    "esmaltaríais": "esmaltar",
    "esmaltaríamos": "esmaltar",
    "esmaltarían": "esmaltar",
    "esmaltarías": "esmaltar",
    "esmaltas": "esmaltar",
    "esmaltase": "esmaltar",
    "esmaltaseis": "esmaltar",
    "esmaltasen": "esmaltar",
    "esmaltases": "esmaltar",
    "esmaltaste": "esmaltar",
    "esmaltasteis": "esmaltar",
    "esmalte": "esmaltar",
    "esmaltemos": "esmaltar",
    "esmalten": "esmaltar",
    "esmaltes": "esmalte",
    "esmaltinas": "esmaltina",
    "esmaltines": "esmaltín",
    "esmaltitas": "esmaltita",
    "esmalto": "esmaltar",
    "esmaltábamos": "esmaltar",
    "esmaltáis": "esmaltar",
    "esmaltáramos": "esmaltar",
    "esmaltáremos": "esmaltar",
    "esmaltásemos": "esmaltar",
    "esmalté": "esmaltar",
    "esmaltéis": "esmaltar",
    "esmaltó": "esmaltar",
    "esmaragditas": "esmaragdita",
    "esmectitas": "esmectita",
    "esmera": "esmerar",
    "esmeraba": "esmerar",
    "esmerabais": "esmerar",
    "esmeraban": "esmerar",
    "esmerabas": "esmerar",
    "esmerad": "esmerar",
    "esmerada": "esmerar",
    "esmeradas": "esmerar",
    "esmerado": "esmerar",
    "esmeradores": "esmerador",
    "esmerados": "esmerar",
    "esmeraldas": "esmeralda",
    "esmeramos": "esmerar",
    "esmeran": "esmerar",
    "esmerando": "esmerar",
    "esmerara": "esmerar",
    "esmerarais": "esmerar",
    "esmeraran": "esmerar",
    "esmeraras": "esmerar",
    "esmerare": "esmerar",
    "esmerareis": "esmerar",
    "esmeraremos": "esmerar",
    "esmeraren": "esmerar",
    "esmerares": "esmerar",
    "esmeraron": "esmerar",
    "esmerará": "esmerar",
    "esmerarán": "esmerar",
    "esmerarás": "esmerar",
    "esmeraré": "esmerar",
    "esmeraréis": "esmerar",
    "esmeraría": "esmerar",
    "esmeraríais": "esmerar",
    "esmeraríamos": "esmerar",
    "esmerarían": "esmerar",
    "esmerarías": "esmerar",
    "esmeras": "esmerar",
    "esmerase": "esmerar",
    "esmeraseis": "esmerar",
    "esmerasen": "esmerar",
    "esmerases": "esmerar",
    "esmeraste": "esmerar",
    "esmerasteis": "esmerar",
    "esmere": "esmerar",
    "esmerejones": "esmerejón",
    "esmeremos": "esmerar",
    "esmeren": "esmerar",
    "esmeres": "esmerar",
    "esmerila": "esmerilar",
    "esmerilaba": "esmerilar",
    "esmerilabais": "esmerilar",
    "esmerilaban": "esmerilar",
    "esmerilabas": "esmerilar",
    "esmerilad": "esmerilar",
    "esmerilada": "esmerilar",
    "esmeriladas": "esmerilar",
    "esmerilado": "esmerilar",
    "esmeriladoras": "esmeriladora",
    "esmerilados": "esmerilar",
    "esmerilamos": "esmerilar",
    "esmerilan": "esmerilar",
    "esmerilando": "esmerilar",
    "esmerilara": "esmerilar",
    "esmerilarais": "esmerilar",
    "esmerilaran": "esmerilar",
    "esmerilaras": "esmerilar",
    "esmerilare": "esmerilar",
    "esmerilareis": "esmerilar",
    "esmerilaremos": "esmerilar",
    "esmerilaren": "esmerilar",
    "esmerilares": "esmerilar",
    "esmerilaron": "esmerilar",
    "esmerilará": "esmerilar",
    "esmerilarán": "esmerilar",
    "esmerilarás": "esmerilar",
    "esmerilaré": "esmerilar",
    "esmerilaréis": "esmerilar",
    "esmerilaría": "esmerilar",
    "esmerilaríais": "esmerilar",
    "esmerilaríamos": "esmerilar",
    "esmerilarían": "esmerilar",
    "esmerilarías": "esmerilar",
    "esmerilas": "esmerilar",
    "esmerilase": "esmerilar",
    "esmerilaseis": "esmerilar",
    "esmerilasen": "esmerilar",
    "esmerilases": "esmerilar",
    "esmerilaste": "esmerilar",
    "esmerilasteis": "esmerilar",
    "esmerile": "esmerilar",
    "esmerilemos": "esmerilar",
    "esmerilen": "esmerilar",
    "esmeriles": "esmerilar",
    "esmerilo": "esmerilar",
    "esmerilábamos": "esmerilar",
    "esmeriláis": "esmerilar",
    "esmeriláramos": "esmerilar",
    "esmeriláremos": "esmerilar",
    "esmerilásemos": "esmerilar",
    "esmerilé": "esmerilar",
    "esmeriléis": "esmerilar",
    "esmeriló": "esmerilar",
    "esmero": "esmerar",
    "esmeros": "esmero",
    "esmerábamos": "esmerar",
    "esmeráis": "esmerar",
    "esmeráramos": "esmerar",
    "esmeráremos": "esmerar",
    "esmerásemos": "esmerar",
    "esmeré": "esmerar",
    "esmeréis": "esmerar",
    "esmeró": "esmerar",
    "esmirriada": "esmirriado",
    "esmirriadas": "esmirriado",
    "esmirriados": "esmirriado",
    "esmoladeras": "esmoladera",
    "esmoquins": "esmoquin",
    "esméctica": "esméctico",
    "esmécticas": "esméctico",
    "esmécticos": "esméctico",
    "esmóquines": "esmoquin",
    "esnifadas": "esnifada",
    "esnobismos": "esnobismo",
    "esnobs": "esnob",
    "eso": "ese",
    "esodesviaciones": "esodesviación",
    "esofágica": "esofágico",
    "esofágicas": "esofágico",
    "esofágicos": "esofágico",
    "esofórica": "esofórico",
    "esofóricas": "esofórico",
    "esofóricos": "esofórico",
    "esos": "ese",
    "esoterismos": "esoterismo",
    "esotra": "esotro",
    "esotras": "esotro",
    "esotropías": "esotropía",
    "esotros": "esotro",
    "esotrópica": "esotrópico",
    "esotrópicas": "esotrópico",
    "esotrópicos": "esotrópico",
    "esotérica": "esotérico",
    "esotéricas": "esotérico",
    "esotéricos": "esotérico",
    "espabila": "espabilar",
    "espabilaba": "espabilar",
    "espabilabais": "espabilar",
    "espabilaban": "espabilar",
    "espabilabas": "espabilar",
    "espabilad": "espabilar",
    "espabilada": "espabilar",
    "espabiladas": "espabilar",
    "espabilado": "espabilar",
    "espabilados": "espabilar",
    "espabilamos": "espabilar",
    "espabilan": "espabilar",
    "espabilando": "espabilar",
    "espabilara": "espabilar",
    "espabilarais": "espabilar",
    "espabilaran": "espabilar",
    "espabilaras": "espabilar",
    "espabilare": "espabilar",
    "espabilareis": "espabilar",
    "espabilaremos": "espabilar",
    "espabilaren": "espabilar",
    "espabilares": "espabilar",
    "espabilaron": "espabilar",
    "espabilará": "espabilar",
    "espabilarán": "espabilar",
    "espabilarás": "espabilar",
    "espabilaré": "espabilar",
    "espabilaréis": "espabilar",
    "espabilaría": "espabilar",
    "espabilaríais": "espabilar",
    "espabilaríamos": "espabilar",
    "espabilarían": "espabilar",
    "espabilarías": "espabilar",
    "espabilas": "espabilar",
    "espabilase": "espabilar",
    "espabilaseis": "espabilar",
    "espabilasen": "espabilar",
    "espabilases": "espabilar",
    "espabilaste": "espabilar",
    "espabilasteis": "espabilar",
    "espabile": "espabilar",
    "espabilemos": "espabilar",
    "espabilen": "espabilar",
    "espabiles": "espabilar",
    "espabilo": "espabilar",
    "espabilábamos": "espabilar",
    "espabiláis": "espabilar",
    "espabiláramos": "espabilar",
    "espabiláremos": "espabilar",
    "espabilásemos": "espabilar",
    "espabilé": "espabilar",
    "espabiléis": "espabilar",
    "espabiló": "espabilar",
    "espachurra": "espachurrar",
    "espachurraba": "espachurrar",
    "espachurrabais": "espachurrar",
    "espachurraban": "espachurrar",
    "espachurrabas": "espachurrar",
    "espachurrad": "espachurrar",
    "espachurrada": "espachurrar",
    "espachurradas": "espachurrar",
    "espachurrado": "espachurrar",
    "espachurrados": "espachurrar",
    "espachurramientos": "espachurramiento",
    "espachurramos": "espachurrar",
    "espachurran": "espachurrar",
    "espachurrando": "espachurrar",
    "espachurrara": "espachurrar",
    "espachurrarais": "espachurrar",
    "espachurraran": "espachurrar",
    "espachurraras": "espachurrar",
    "espachurrare": "espachurrar",
    "espachurrareis": "espachurrar",
    "espachurraremos": "espachurrar",
    "espachurraren": "espachurrar",
    "espachurrares": "espachurrar",
    "espachurraron": "espachurrar",
    "espachurrará": "espachurrar",
    "espachurrarán": "espachurrar",
    "espachurrarás": "espachurrar",
    "espachurraré": "espachurrar",
    "espachurraréis": "espachurrar",
    "espachurraría": "espachurrar",
    "espachurraríais": "espachurrar",
    "espachurraríamos": "espachurrar",
    "espachurrarían": "espachurrar",
    "espachurrarías": "espachurrar",
    "espachurras": "espachurrar",
    "espachurrase": "espachurrar",
    "espachurraseis": "espachurrar",
    "espachurrasen": "espachurrar",
    "espachurrases": "espachurrar",
    "espachurraste": "espachurrar",
    "espachurrasteis": "espachurrar",
    "espachurre": "espachurrar",
    "espachurremos": "espachurrar",
    "espachurren": "espachurrar",
    "espachurres": "espachurrar",
    "espachurro": "espachurrar",
    "espachurrábamos": "espachurrar",
    "espachurráis": "espachurrar",
    "espachurráramos": "espachurrar",
    "espachurráremos": "espachurrar",
    "espachurrásemos": "espachurrar",
    "espachurré": "espachurrar",
    "espachurréis": "espachurrar",
    "espachurró": "espachurrar",
    "espacia": "espaciar",
    "espaciaba": "espaciar",
    "espaciabais": "espaciar",
    "espaciaban": "espaciar",
    "espaciabas": "espaciar",
    "espaciad": "espaciar",
    "espaciada": "espaciar",
    "espaciadas": "espaciar",
    "espaciado": "espaciar",
    "espaciadora": "espaciador",
    "espaciadoras": "espaciador",
    "espaciadores": "espaciador",
    "espaciados": "espaciar",
    "espaciales": "espacial",
    "espaciamientos": "espaciamiento",
    "espaciamos": "espaciar",
    "espacian": "espaciar",
    "espaciando": "espaciar",
    "espaciara": "espaciar",
    "espaciarais": "espaciar",
    "espaciaran": "espaciar",
    "espaciaras": "espaciar",
    "espaciare": "espaciar",
    "espaciareis": "espaciar",
    "espaciaremos": "espaciar",
    "espaciaren": "espaciar",
    "espaciares": "espaciar",
    "espaciaron": "espaciar",
    "espaciará": "espaciar",
    "espaciarán": "espaciar",
    "espaciarás": "espaciar",
    "espaciaré": "espaciar",
    "espaciaréis": "espaciar",
    "espaciaría": "espaciar",
    "espaciaríais": "espaciar",
    "espaciaríamos": "espaciar",
    "espaciarían": "espaciar",
    "espaciarías": "espaciar",
    "espacias": "espaciar",
    "espaciase": "espaciar",
    "espaciaseis": "espaciar",
    "espaciasen": "espaciar",
    "espaciases": "espaciar",
    "espaciaste": "espaciar",
    "espaciasteis": "espaciar",
    "espacie": "espaciar",
    "espaciemos": "espaciar",
    "espacien": "espaciar",
    "espacies": "espaciar",
    "espacio": "espaciar",
    "espacios": "espacio",
    "espaciosa": "espacioso",
    "espaciosas": "espacioso",
    "espaciosidades": "espaciosidad",
    "espaciosos": "espacioso",
    "espaciábamos": "espaciar",
    "espaciáis": "espaciar",
    "espaciáramos": "espaciar",
    "espaciáremos": "espaciar",
    "espaciásemos": "espaciar",
    "espacié": "espaciar",
    "espaciéis": "espaciar",
    "espació": "espaciar",
    "espada": "espadar",
    "espadaba": "espadar",
    "espadabais": "espadar",
    "espadaban": "espadar",
    "espadabas": "espadar",
    "espadachines": "espadachín",
    "espadad": "espadar",
    "espadada": "espadar",
    "espadadas": "espadar",
    "espadado": "espadar",
    "espadados": "espadar",
    "espadamos": "espadar",
    "espadan": "espadar",
    "espadando": "espadar",
    "espadara": "espadar",
    "espadarais": "espadar",
    "espadaran": "espadar",
    "espadaras": "espadar",
    "espadare": "espadar",
    "espadareis": "espadar",
    "espadaremos": "espadar",
    "espadaren": "espadar",
    "espadares": "espadar",
    "espadaron": "espadar",
    "espadartes": "espadarte",
    "espadará": "espadar",
    "espadarán": "espadar",
    "espadarás": "espadar",
    "espadaré": "espadar",
    "espadaréis": "espadar",
    "espadaría": "espadar",
    "espadaríais": "espadar",
    "espadaríamos": "espadar",
    "espadarían": "espadar",
    "espadarías": "espadar",
    "espadas": "espadar",
    "espadase": "espadar",
    "espadaseis": "espadar",
    "espadasen": "espadar",
    "espadases": "espadar",
    "espadaste": "espadar",
    "espadasteis": "espadar",
    "espadañadas": "espadañada",
    "espadañas": "espadaña",
    "espade": "espadar",
    "espademos": "espadar",
    "espaden": "espadar",
    "espadera": "espadero",
    "espaderas": "espadero",
    "espaderos": "espadero",
    "espaderías": "espadería",
    "espades": "espadar",
    "espadicifloras": "espadiciflora",
    "espadilla": "espadillar",
    "espadillaba": "espadillar",
    "espadillabais": "espadillar",
    "espadillaban": "espadillar",
    "espadillabas": "espadillar",
    "espadillad": "espadillar",
    "espadillada": "espadillar",
    "espadilladas": "espadillar",
    "espadillado": "espadillar",
    "espadillados": "espadillar",
    "espadillamos": "espadillar",
    "espadillan": "espadillar",
    "espadillando": "espadillar",
    "espadillara": "espadillar",
    "espadillarais": "espadillar",
    "espadillaran": "espadillar",
    "espadillaras": "espadillar",
    "espadillare": "espadillar",
    "espadillareis": "espadillar",
    "espadillaremos": "espadillar",
    "espadillaren": "espadillar",
    "espadillares": "espadillar",
    "espadillaron": "espadillar",
    "espadillará": "espadillar",
    "espadillarán": "espadillar",
    "espadillarás": "espadillar",
    "espadillaré": "espadillar",
    "espadillaréis": "espadillar",
    "espadillaría": "espadillar",
    "espadillaríais": "espadillar",
    "espadillaríamos": "espadillar",
    "espadillarían": "espadillar",
    "espadillarías": "espadillar",
    "espadillas": "espadillar",
    "espadillase": "espadillar",
    "espadillaseis": "espadillar",
    "espadillasen": "espadillar",
    "espadillases": "espadillar",
    "espadillaste": "espadillar",
    "espadillasteis": "espadillar",
    "espadille": "espadillar",
    "espadillemos": "espadillar",
    "espadillen": "espadillar",
    "espadilles": "espadillar",
    "espadillo": "espadillar",
    "espadillábamos": "espadillar",
    "espadilláis": "espadillar",
    "espadilláramos": "espadillar",
    "espadilláremos": "espadillar",
    "espadillásemos": "espadillar",
    "espadillé": "espadillar",
    "espadilléis": "espadillar",
    "espadilló": "espadillar",
    "espadinas": "espadina",
    "espadines": "espadín",
    "espadistas": "espadista",
    "espado": "espadar",
    "espadones": "espadón",
    "espadábamos": "espadar",
    "espadáis": "espadar",
    "espadáramos": "espadar",
    "espadáremos": "espadar",
    "espadásemos": "espadar",
    "espadé": "espadar",
    "espadéis": "espadar",
    "espadó": "espadar",
    "espaguetis": "espagueti",
    "espagíricas": "espagírica",
    "espahíes": "espahí",
    "espahís": "espahí",
    "espala": "espalar",
    "espalaba": "espalar",
    "espalabais": "espalar",
    "espalaban": "espalar",
    "espalabas": "espalar",
    "espalad": "espalar",
    "espalada": "espalar",
    "espaladas": "espalar",
    "espalado": "espalar",
    "espalados": "espalar",
    "espalamos": "espalar",
    "espalan": "espalar",
    "espalando": "espalar",
    "espalara": "espalar",
    "espalarais": "espalar",
    "espalaran": "espalar",
    "espalaras": "espalar",
    "espalare": "espalar",
    "espalareis": "espalar",
    "espalaremos": "espalar",
    "espalaren": "espalar",
    "espalares": "espalar",
    "espalaron": "espalar",
    "espalará": "espalar",
    "espalarán": "espalar",
    "espalarás": "espalar",
    "espalaré": "espalar",
    "espalaréis": "espalar",
    "espalaría": "espalar",
    "espalaríais": "espalar",
    "espalaríamos": "espalar",
    "espalarían": "espalar",
    "espalarías": "espalar",
    "espalas": "espalar",
    "espalase": "espalar",
    "espalaseis": "espalar",
    "espalasen": "espalar",
    "espalases": "espalar",
    "espalaste": "espalar",
    "espalasteis": "espalar",
    "espaldarazos": "espaldarazo",
    "espaldarcetes": "espaldarcete",
    "espaldares": "espaldar",
    "espaldarones": "espaldarón",
    "espaldas": "espalda",
    "espaldea": "espaldear",
    "espaldeaba": "espaldear",
    "espaldeabais": "espaldear",
    "espaldeaban": "espaldear",
    "espaldeabas": "espaldear",
    "espaldead": "espaldear",
    "espaldeada": "espaldear",
    "espaldeadas": "espaldear",
    "espaldeado": "espaldear",
    "espaldeados": "espaldear",
    "espaldeamos": "espaldear",
    "espaldean": "espaldear",
    "espaldeando": "espaldear",
    "espaldeara": "espaldear",
    "espaldearais": "espaldear",
    "espaldearan": "espaldear",
    "espaldearas": "espaldear",
    "espaldeare": "espaldear",
    "espaldeareis": "espaldear",
    "espaldearemos": "espaldear",
    "espaldearen": "espaldear",
    "espaldeares": "espaldear",
    "espaldearon": "espaldear",
    "espaldeará": "espaldear",
    "espaldearán": "espaldear",
    "espaldearás": "espaldear",
    "espaldearé": "espaldear",
    "espaldearéis": "espaldear",
    "espaldearía": "espaldear",
    "espaldearíais": "espaldear",
    "espaldearíamos": "espaldear",
    "espaldearían": "espaldear",
    "espaldearías": "espaldear",
    "espaldeas": "espaldear",
    "espaldease": "espaldear",
    "espaldeaseis": "espaldear",
    "espaldeasen": "espaldear",
    "espaldeases": "espaldear",
    "espaldeaste": "espaldear",
    "espaldeasteis": "espaldear",
    "espaldee": "espaldear",
    "espaldeemos": "espaldear",
    "espaldeen": "espaldear",
    "espaldees": "espaldear",
    "espaldeo": "espaldear",
    "espalderas": "espaldera",
    "espalderes": "espalder",
    "espaldeábamos": "espaldear",
    "espaldeáis": "espaldear",
    "espaldeáramos": "espaldear",
    "espaldeáremos": "espaldear",
    "espaldeásemos": "espaldear",
    "espaldeé": "espaldear",
    "espaldeéis": "espaldear",
    "espaldeó": "espaldear",
    "espaldillas": "espaldilla",
    "espaldistas": "espaldista",
    "espaldona": "espaldonar",
    "espaldonaba": "espaldonar",
    "espaldonabais": "espaldonar",
    "espaldonaban": "espaldonar",
    "espaldonabas": "espaldonar",
    "espaldonad": "espaldonar",
    "espaldonada": "espaldonar",
    "espaldonadas": "espaldonar",
    "espaldonado": "espaldonar",
    "espaldonados": "espaldonar",
    "espaldonamos": "espaldonar",
    "espaldonan": "espaldonar",
    "espaldonando": "espaldonar",
    "espaldonara": "espaldonar",
    "espaldonarais": "espaldonar",
    "espaldonaran": "espaldonar",
    "espaldonaras": "espaldonar",
    "espaldonare": "espaldonar",
    "espaldonareis": "espaldonar",
    "espaldonaremos": "espaldonar",
    "espaldonaren": "espaldonar",
    "espaldonares": "espaldonar",
    "espaldonaron": "espaldonar",
    "espaldonará": "espaldonar",
    "espaldonarán": "espaldonar",
    "espaldonarás": "espaldonar",
    "espaldonaré": "espaldonar",
    "espaldonaréis": "espaldonar",
    "espaldonaría": "espaldonar",
    "espaldonaríais": "espaldonar",
    "espaldonaríamos": "espaldonar",
    "espaldonarían": "espaldonar",
    "espaldonarías": "espaldonar",
    "espaldonas": "espaldonar",
    "espaldonase": "espaldonar",
    "espaldonaseis": "espaldonar",
    "espaldonasen": "espaldonar",
    "espaldonases": "espaldonar",
    "espaldonaste": "espaldonar",
    "espaldonasteis": "espaldonar",
    "espaldone": "espaldonar",
    "espaldonemos": "espaldonar",
    "espaldonen": "espaldonar",
    "espaldones": "espaldonar",
    "espaldono": "espaldonar",
    "espaldonábamos": "espaldonar",
    "espaldonáis": "espaldonar",
    "espaldonáramos": "espaldonar",
    "espaldonáremos": "espaldonar",
    "espaldonásemos": "espaldonar",
    "espaldoné": "espaldonar",
    "espaldonéis": "espaldonar",
    "espaldonó": "espaldonar",
    "espalduda": "espaldudo",
    "espaldudas": "espaldudo",
    "espaldudos": "espaldudo",
    "espale": "espalar",
    "espalemos": "espalar",
    "espalen": "espalar",
    "espaleras": "espalera",
    "espales": "espalar",
    "espalmaduras": "espalmadura",
    "espalo": "espalar",
    "espaltos": "espalto",
    "espalábamos": "espalar",
    "espaláis": "espalar",
    "espaláramos": "espalar",
    "espaláremos": "espalar",
    "espalásemos": "espalar",
    "espalé": "espalar",
    "espaléis": "espalar",
    "espaló": "espalar",
    "espanta": "espantar",
    "espantaba": "espantar",
    "espantabais": "espantar",
    "espantaban": "espantar",
    "espantabas": "espantar",
    "espantad": "espantar",
    "espantada": "espantar",
    "espantadas": "espantar",
    "espantadiza": "espantadizo",
    "espantadizas": "espantadizo",
    "espantadizos": "espantadizo",
    "espantado": "espantar",
    "espantadora": "espantador",
    "espantadoras": "espantador",
    "espantadores": "espantador",
    "espantados": "espantar",
    "espantajos": "espantajo",
    "espantamos": "espantar",
    "espantan": "espantar",
    "espantando": "espantar",
    "espantara": "espantar",
    "espantarais": "espantar",
    "espantaran": "espantar",
    "espantaras": "espantar",
    "espantare": "espantar",
    "espantareis": "espantar",
    "espantaremos": "espantar",
    "espantaren": "espantar",
    "espantares": "espantar",
    "espantaron": "espantar",
    "espantará": "espantar",
    "espantarán": "espantar",
    "espantarás": "espantar",
    "espantaré": "espantar",
    "espantaréis": "espantar",
    "espantaría": "espantar",
    "espantaríais": "espantar",
    "espantaríamos": "espantar",
    "espantarían": "espantar",
    "espantarías": "espantar",
    "espantas": "espantar",
    "espantase": "espantar",
    "espantaseis": "espantar",
    "espantasen": "espantar",
    "espantases": "espantar",
    "espantaste": "espantar",
    "espantasteis": "espantar",
    "espante": "espantar",
    "espantemos": "espantar",
    "espanten": "espantar",
    "espantes": "espante",
    "espanto": "espantar",
    "espantos": "espanto",
    "espantosa": "espantoso",
    "espantosas": "espantoso",
    "espantosos": "espantoso",
    "espantábamos": "espantar",
    "espantáis": "espantar",
    "espantáramos": "espantar",
    "espantáremos": "espantar",
    "espantásemos": "espantar",
    "espanté": "espantar",
    "espantéis": "espantar",
    "espantó": "espantar",
    "esparadrapos": "esparadrapo",
    "esparavanes": "esparaván",
    "esparaveles": "esparavel",
    "esparce": "esparcir",
    "esparcen": "esparcir",
    "esparces": "esparcir",
    "esparciatas": "esparciata",
    "esparcid": "esparcir",
    "esparcida": "esparcir",
    "esparcidas": "esparcir",
    "esparcido": "esparcir",
    "esparcidora": "esparcidor",
    "esparcidoras": "esparcidora",
    "esparcidores": "esparcidor",
    "esparcidos": "esparcir",
    "esparciendo": "esparcir",
    "esparciera": "esparcir",
    "esparcierais": "esparcir",
    "esparcieran": "esparcir",
    "esparcieras": "esparcir",
    "esparciere": "esparcir",
    "esparciereis": "esparcir",
    "esparcieren": "esparcir",
    "esparcieres": "esparcir",
    "esparcieron": "esparcir",
    "esparciese": "esparcir",
    "esparcieseis": "esparcir",
    "esparciesen": "esparcir",
    "esparcieses": "esparcir",
    "esparcimientos": "esparcimiento",
    "esparcimos": "esparcir",
    "esparciremos": "esparcir",
    "esparcirá": "esparcir",
    "esparcirán": "esparcir",
    "esparcirás": "esparcir",
    "esparciré": "esparcir",
    "esparciréis": "esparcir",
    "esparciría": "esparcir",
    "esparciríais": "esparcir",
    "esparciríamos": "esparcir",
    "esparcirían": "esparcir",
    "esparcirías": "esparcir",
    "esparciste": "esparcir",
    "esparcisteis": "esparcir",
    "esparciéramos": "esparcir",
    "esparciéremos": "esparcir",
    "esparciésemos": "esparcir",
    "esparció": "esparcir",
    "esparcí": "esparcir",
    "esparcía": "esparcir",
    "esparcíais": "esparcir",
    "esparcíamos": "esparcir",
    "esparcían": "esparcir",
    "esparcías": "esparcir",
    "esparcís": "esparcir",
    "esparragadora": "esparragador",
    "esparragadoras": "esparragador",
    "esparragadores": "esparragador",
    "esparragados": "esparragado",
    "esparragales": "esparragal",
    "esparragueras": "esparraguera",
    "esparragueros": "esparraguero",
    "esparraguinas": "esparraguina",
    "esparranca": "esparrancar",
    "esparrancaba": "esparrancar",
    "esparrancabais": "esparrancar",
    "esparrancaban": "esparrancar",
    "esparrancabas": "esparrancar",
    "esparrancad": "esparrancar",
    "esparrancada": "esparrancar",
    "esparrancadas": "esparrancar",
    "esparrancado": "esparrancar",
    "esparrancados": "esparrancar",
    "esparrancamos": "esparrancar",
    "esparrancan": "esparrancar",
    "esparrancando": "esparrancar",
    "esparrancara": "esparrancar",
    "esparrancarais": "esparrancar",
    "esparrancaran": "esparrancar",
    "esparrancaras": "esparrancar",
    "esparrancare": "esparrancar",
    "esparrancareis": "esparrancar",
    "esparrancaremos": "esparrancar",
    "esparrancaren": "esparrancar",
    "esparrancares": "esparrancar",
    "esparrancaron": "esparrancar",
    "esparrancará": "esparrancar",
    "esparrancarán": "esparrancar",
    "esparrancarás": "esparrancar",
    "esparrancaré": "esparrancar",
    "esparrancaréis": "esparrancar",
    "esparrancaría": "esparrancar",
    "esparrancaríais": "esparrancar",
    "esparrancaríamos": "esparrancar",
    "esparrancarían": "esparrancar",
    "esparrancarías": "esparrancar",
    "esparrancas": "esparrancar",
    "esparrancase": "esparrancar",
    "esparrancaseis": "esparrancar",
    "esparrancasen": "esparrancar",
    "esparrancases": "esparrancar",
    "esparrancaste": "esparrancar",
    "esparrancasteis": "esparrancar",
    "esparranco": "esparrancar",
    "esparrancábamos": "esparrancar",
    "esparrancáis": "esparrancar",
    "esparrancáramos": "esparrancar",
    "esparrancáremos": "esparrancar",
    "esparrancásemos": "esparrancar",
    "esparrancó": "esparrancar",
    "esparranque": "esparrancar",
    "esparranquemos": "esparrancar",
    "esparranquen": "esparrancar",
    "esparranques": "esparrancar",
    "esparranqué": "esparrancar",
    "esparranquéis": "esparrancar",
    "espartana": "espartano",
    "espartanas": "espartano",
    "espartanos": "espartano",
    "espartaquismos": "espartaquismo",
    "espartaquistas": "espartaquista",
    "espartera": "espartero",
    "esparteras": "espartero",
    "esparteros": "espartero",
    "esparterías": "espartería",
    "esparteínas": "esparteína",
    "esparteñas": "esparteña",
    "espartillas": "espartilla",
    "espartizales": "espartizal",
    "espartos": "esparto",
    "esparza": "esparcir",
    "esparzamos": "esparcir",
    "esparzan": "esparcir",
    "esparzas": "esparcir",
    "esparzo": "esparcir",
    "esparzáis": "esparcir",
    "espasmolítica": "espasmolítico",
    "espasmolíticas": "espasmolítico",
    "espasmolíticos": "espasmolítico",
    "espasmos": "espasmo",
    "espasmódica": "espasmódico",
    "espasmódicas": "espasmódico",
    "espasmódicos": "espasmódico",
    "espasmógena": "espasmógeno",
    "espasmógenas": "espasmógeno",
    "espasmógenos": "espasmógeno",
    "espasticidades": "espasticidad",
    "espatarra": "espatarrar",
    "espatarraba": "espatarrar",
    "espatarrabais": "espatarrar",
    "espatarraban": "espatarrar",
    "espatarrabas": "espatarrar",
    "espatarrad": "espatarrar",
    "espatarrada": "espatarrar",
    "espatarradas": "espatarrar",
    "espatarrado": "espatarrar",
    "espatarrados": "espatarrar",
    "espatarramos": "espatarrar",
    "espatarran": "espatarrar",
    "espatarrando": "espatarrar",
    "espatarrara": "espatarrar",
    "espatarrarais": "espatarrar",
    "espatarraran": "espatarrar",
    "espatarraras": "espatarrar",
    "espatarrare": "espatarrar",
    "espatarrareis": "espatarrar",
    "espatarraremos": "espatarrar",
    "espatarraren": "espatarrar",
    "espatarrares": "espatarrar",
    "espatarraron": "espatarrar",
    "espatarrará": "espatarrar",
    "espatarrarán": "espatarrar",
    "espatarrarás": "espatarrar",
    "espatarraré": "espatarrar",
    "espatarraréis": "espatarrar",
    "espatarraría": "espatarrar",
    "espatarraríais": "espatarrar",
    "espatarraríamos": "espatarrar",
    "espatarrarían": "espatarrar",
    "espatarrarías": "espatarrar",
    "espatarras": "espatarrar",
    "espatarrase": "espatarrar",
    "espatarraseis": "espatarrar",
    "espatarrasen": "espatarrar",
    "espatarrases": "espatarrar",
    "espatarraste": "espatarrar",
    "espatarrasteis": "espatarrar",
    "espatarre": "espatarrar",
    "espatarremos": "espatarrar",
    "espatarren": "espatarrar",
    "espatarres": "espatarrar",
    "espatarro": "espatarrar",
    "espatarrábamos": "espatarrar",
    "espatarráis": "espatarrar",
    "espatarráramos": "espatarrar",
    "espatarráremos": "espatarrar",
    "espatarrásemos": "espatarrar",
    "espatarré": "espatarrar",
    "espatarréis": "espatarrar",
    "espatarró": "espatarrar",
    "espatas": "espata",
    "espatos": "espato",
    "espatulaciones": "espatulación",
    "espatulomancias": "espatulomancia",
    "espatulomancías": "espatulomancía",
    "espayes": "espay",
    "espays": "espay",
    "española": "españolar",
    "españolaba": "españolar",
    "españolabais": "españolar",
    "españolaban": "españolar",
    "españolabas": "españolar",
    "españolad": "españolar",
    "españolada": "españolar",
    "españoladas": "españolar",
    "españolado": "españolar",
    "españolados": "españolar",
    "españolamos": "españolar",
    "españolan": "españolar",
    "españolando": "españolar",
    "españolara": "españolar",
    "españolarais": "españolar",
    "españolaran": "españolar",
    "españolaras": "españolar",
    "españolare": "españolar",
    "españolareis": "españolar",
    "españolaremos": "españolar",
    "españolaren": "españolar",
    "españolares": "españolar",
    "españolaron": "españolar",
    "españolará": "españolar",
    "españolarán": "españolar",
    "españolarás": "españolar",
    "españolaré": "españolar",
    "españolaréis": "españolar",
    "españolaría": "españolar",
    "españolaríais": "españolar",
    "españolaríamos": "españolar",
    "españolarían": "españolar",
    "españolarías": "españolar",
    "españolas": "españolar",
    "españolase": "españolar",
    "españolaseis": "españolar",
    "españolasen": "españolar",
    "españolases": "españolar",
    "españolaste": "españolar",
    "españolasteis": "españolar",
    "españole": "españolar",
    "españolemos": "españolar",
    "españolen": "españolar",
    "españolerías": "españolería",
    "españoles": "españolar",
    "españolice": "españolizar",
    "españolicemos": "españolizar",
    "españolicen": "españolizar",
    "españolices": "españolizar",
    "españolicé": "españolizar",
    "españolicéis": "españolizar",
    "españolismos": "españolismo",
    "españolistas": "españolista",
    "españoliza": "españolizar",
    "españolizaba": "españolizar",
    "españolizabais": "españolizar",
    "españolizaban": "españolizar",
    "españolizabas": "españolizar",
    "españolizaciones": "españolización",
    "españolizad": "españolizar",
    "españolizada": "españolizar",
    "españolizadas": "españolizar",
    "españolizado": "españolizar",
    "españolizados": "españolizar",
    "españolizamos": "españolizar",
    "españolizan": "españolizar",
    "españolizando": "españolizar",
    "españolizara": "españolizar",
    "españolizarais": "españolizar",
    "españolizaran": "españolizar",
    "españolizaras": "españolizar",
    "españolizare": "españolizar",
    "españolizareis": "españolizar",
    "españolizaremos": "españolizar",
    "españolizaren": "españolizar",
    "españolizares": "españolizar",
    "españolizaron": "españolizar",
    "españolizará": "españolizar",
    "españolizarán": "españolizar",
    "españolizarás": "españolizar",
    "españolizaré": "españolizar",
    "españolizaréis": "españolizar",
    "españolizaría": "españolizar",
    "españolizaríais": "españolizar",
    "españolizaríamos": "españolizar",
    "españolizarían": "españolizar",
    "españolizarías": "españolizar",
    "españolizas": "españolizar",
    "españolizase": "españolizar",
    "españolizaseis": "españolizar",
    "españolizasen": "españolizar",
    "españolizases": "españolizar",
    "españolizaste": "españolizar",
    "españolizasteis": "españolizar",
    "españolizo": "españolizar",
    "españolizábamos": "españolizar",
    "españolizáis": "españolizar",
    "españolizáramos": "españolizar",
    "españolizáremos": "españolizar",
    "españolizásemos": "españolizar",
    "españolizó": "españolizar",
    "españolo": "españolar",
    "españolábamos": "españolar",
    "españoláis": "españolar",
    "españoláramos": "españolar",
    "españoláremos": "españolar",
    "españolásemos": "españolar",
    "españolé": "españolar",
    "españoléis": "españolar",
    "españoló": "españolar",
    "especiaciones": "especiación",
    "especiales": "especial",
    "especialice": "especializar",
    "especialicemos": "especializar",
    "especialicen": "especializar",
    "especialices": "especializar",
    "especialicé": "especializar",
    "especialicéis": "especializar",
    "especialidades": "especialidad",
    "especialistas": "especialista",
    "especializa": "especializar",
    "especializaba": "especializar",
    "especializabais": "especializar",
    "especializaban": "especializar",
    "especializabas": "especializar",
    "especializaciones": "especialización",
    "especializad": "especializar",
    "especializada": "especializar",
    "especializadas": "especializar",
    "especializado": "especializar",
    "especializados": "especializar",
    "especializamos": "especializar",
    "especializan": "especializar",
    "especializando": "especializar",
    "especializara": "especializar",
    "especializarais": "especializar",
    "especializaran": "especializar",
    "especializaras": "especializar",
    "especializare": "especializar",
    "especializareis": "especializar",
    "especializaremos": "especializar",
    "especializaren": "especializar",
    "especializares": "especializar",
    "especializaron": "especializar",
    "especializará": "especializar",
    "especializarán": "especializar",
    "especializarás": "especializar",
    "especializaré": "especializar",
    "especializaréis": "especializar",
    "especializaría": "especializar",
    "especializaríais": "especializar",
    "especializaríamos": "especializar",
    "especializarían": "especializar",
    "especializarías": "especializar",
    "especializas": "especializar",
    "especializase": "especializar",
    "especializaseis": "especializar",
    "especializasen": "especializar",
    "especializases": "especializar",
    "especializaste": "especializar",
    "especializasteis": "especializar",
    "especializo": "especializar",
    "especializábamos": "especializar",
    "especializáis": "especializar",
    "especializáramos": "especializar",
    "especializáremos": "especializar",
    "especializásemos": "especializar",
    "especializó": "especializar",
    "especias": "especia",
    "especiera": "especiero",
    "especieras": "especiero",
    "especieros": "especiero",
    "especierías": "especiería",
    "especies": "especie",
    "especifica": "especificar",
    "especificaba": "especificar",
    "especificabais": "especificar",
    "especificaban": "especificar",
    "especificabas": "especificar",
    "especificables": "especificable",
    "especificaciones": "especificación",
    "especificad": "especificar",
    "especificada": "especificar",
    "especificadas": "especificar",
    "especificado": "especificar",
    "especificadora": "especificador",
    "especificadoras": "especificador",
    "especificadores": "especificador",
    "especificados": "especificar",
    "especificamos": "especificar",
    "especifican": "especificar",
    "especificando": "especificar",
    "especificara": "especificar",
    "especificarais": "especificar",
    "especificaran": "especificar",
    "especificaras": "especificar",
    "especificare": "especificar",
    "especificareis": "especificar",
    "especificaremos": "especificar",
    "especificaren": "especificar",
    "especificares": "especificar",
    "especificaron": "especificar",
    "especificará": "especificar",
    "especificarán": "especificar",
    "especificarás": "especificar",
    "especificaré": "especificar",
    "especificaréis": "especificar",
    "especificaría": "especificar",
    "especificaríais": "especificar",
    "especificaríamos": "especificar",
    "especificarían": "especificar",
    "especificarías": "especificar",
    "especificas": "especificar",
    "especificase": "especificar",
    "especificaseis": "especificar",
    "especificasen": "especificar",
    "especificases": "especificar",
    "especificaste": "especificar",
    "especificasteis": "especificar",
    "especificativa": "especificativo",
    "especificativas": "especificativo",
    "especificativos": "especificativo",
    "especificidades": "especificidad",
    "especifico": "especificar",
    "especificábamos": "especificar",
    "especificáis": "especificar",
    "especificáramos": "especificar",
    "especificáremos": "especificar",
    "especificásemos": "especificar",
    "especificó": "especificar",
    "especifique": "especificar",
    "especifiquemos": "especificar",
    "especifiquen": "especificar",
    "especifiques": "especificar",
    "especifiqué": "especificar",
    "especifiquéis": "especificar",
    "especiosa": "especioso",
    "especiosas": "especioso",
    "especiosos": "especioso",
    "especiotas": "especiota",
    "espectaculares": "espectacular",
    "espectacularidades": "espectacularidad",
    "espectadora": "espectador",
    "espectadoras": "espectador",
    "espectadores": "espectador",
    "espectrales": "espectral",
    "espectrinas": "espectrina",
    "espectrocolorímetros": "espectrocolorímetro",
    "espectroeléctrica": "espectroeléctrico",
    "espectroeléctricas": "espectroeléctrico",
    "espectroeléctricos": "espectroeléctrico",
    "espectrofluorímetros": "espectrofluorímetro",
    "espectrofotometrías": "espectrofotometría",
    "espectrofotométrica": "espectrofotométrico",
    "espectrofotométricas": "espectrofotométrico",
    "espectrofotométricos": "espectrofotométrico",
    "espectrofotómetros": "espectrofotómetro",
    "espectrografías": "espectrografía",
    "espectrogramas": "espectrograma",
    "espectrográfica": "espectrográfico",
    "espectrográficas": "espectrográfico",
    "espectrográficos": "espectrográfico",
    "espectroheliógrafos": "espectroheliógrafo",
    "espectrolitas": "espectrolita",
    "espectrometrías": "espectrometría",
    "espectrométrica": "espectrométrico",
    "espectrométricas": "espectrométrico",
    "espectrométricos": "espectrométrico",
    "espectropolarímetros": "espectropolarímetro",
    "espectrorradiométrica": "espectrorradiométrico",
    "espectrorradiométricas": "espectrorradiométrico",
    "espectrorradiométricos": "espectrorradiométrico",
    "espectros": "espectro",
    "espectroscopias": "espectroscopia",
    "espectroscopios": "espectroscopio",
    "espectroscópica": "espectroscópico",
    "espectroscópicas": "espectroscópico",
    "espectroscópicos": "espectroscópico",
    "espectrotipos": "espectrotipo",
    "espectrógrafos": "espectrógrafo",
    "espectrómetros": "espectrómetro",
    "espectáculos": "espectáculo",
    "especula": "especular",
    "especulaba": "especular",
    "especulabais": "especular",
    "especulaban": "especular",
    "especulabas": "especular",
    "especulaciones": "especulación",
    "especulad": "especular",
    "especulada": "especular",
    "especuladas": "especular",
    "especulado": "especular",
    "especuladora": "especulador",
    "especuladoras": "especulador",
    "especuladores": "especulador",
    "especulados": "especular",
    "especulamos": "especular",
    "especulan": "especular",
    "especulando": "especular",
    "especulara": "especular",
    "especularais": "especular",
    "especularan": "especular",
    "especularas": "especular",
    "especulare": "especular",
    "especulareis": "especular",
    "especularemos": "especular",
    "especularen": "especular",
    "especulares": "especular",
    "especularitas": "especularita",
    "especularon": "especular",
    "especulará": "especular",
    "especularán": "especular",
    "especularás": "especular",
    "especularé": "especular",
    "especularéis": "especular",
    "especularía": "especular",
    "especularíais": "especular",
    "especularíamos": "especular",
    "especularían": "especular",
    "especularías": "especular",
    "especulas": "especular",
    "especulase": "especular",
    "especulaseis": "especular",
    "especulasen": "especular",
    "especulases": "especular",
    "especulaste": "especular",
    "especulasteis": "especular",
    "especulativa": "especulativo",
    "especulativas": "especulativo",
    "especulativos": "especulativo",
    "especule": "especular",
    "especulemos": "especular",
    "especulen": "especular",
    "especules": "especular",
    "especulo": "especular",
    "especulábamos": "especular",
    "especuláis": "especular",
    "especuláramos": "especular",
    "especuláremos": "especular",
    "especulásemos": "especular",
    "especulé": "especular",
    "especuléis": "especular",
    "especuló": "especular",
    "específica": "específico",
    "específicas": "específico",
    "específicos": "específico",
    "espeja": "espejar",
    "espejaba": "espejar",
    "espejabais": "espejar",
    "espejaban": "espejar",
    "espejabas": "espejar",
    "espejad": "espejar",
    "espejada": "espejar",
    "espejadas": "espejar",
    "espejado": "espejar",
    "espejados": "espejar",
    "espejamos": "espejar",
    "espejan": "espejar",
    "espejando": "espejar",
    "espejara": "espejar",
    "espejarais": "espejar",
    "espejaran": "espejar",
    "espejaras": "espejar",
    "espejare": "espejar",
    "espejareis": "espejar",
    "espejaremos": "espejar",
    "espejaren": "espejar",
    "espejares": "espejar",
    "espejaron": "espejar",
    "espejará": "espejar",
    "espejarán": "espejar",
    "espejarás": "espejar",
    "espejaré": "espejar",
    "espejaréis": "espejar",
    "espejaría": "espejar",
    "espejaríais": "espejar",
    "espejaríamos": "espejar",
    "espejarían": "espejar",
    "espejarías": "espejar",
    "espejas": "espejar",
    "espejase": "espejar",
    "espejaseis": "espejar",
    "espejasen": "espejar",
    "espejases": "espejar",
    "espejaste": "espejar",
    "espejasteis": "espejar",
    "espeje": "espejar",
    "espejea": "espejear",
    "espejeaba": "espejear",
    "espejeabais": "espejear",
    "espejeaban": "espejear",
    "espejeabas": "espejear",
    "espejead": "espejear",
    "espejeada": "espejear",
    "espejeadas": "espejear",
    "espejeado": "espejear",
    "espejeados": "espejear",
    "espejeamos": "espejear",
    "espejean": "espejear",
    "espejeando": "espejear",
    "espejeara": "espejear",
    "espejearais": "espejear",
    "espejearan": "espejear",
    "espejearas": "espejear",
    "espejeare": "espejear",
    "espejeareis": "espejear",
    "espejearemos": "espejear",
    "espejearen": "espejear",
    "espejeares": "espejear",
    "espejearon": "espejear",
    "espejeará": "espejear",
    "espejearán": "espejear",
    "espejearás": "espejear",
    "espejearé": "espejear",
    "espejearéis": "espejear",
    "espejearía": "espejear",
    "espejearíais": "espejear",
    "espejearíamos": "espejear",
    "espejearían": "espejear",
    "espejearías": "espejear",
    "espejeas": "espejear",
    "espejease": "espejear",
    "espejeaseis": "espejear",
    "espejeasen": "espejear",
    "espejeases": "espejear",
    "espejeaste": "espejear",
    "espejeasteis": "espejear",
    "espejee": "espejear",
    "espejeemos": "espejear",
    "espejeen": "espejear",
    "espejees": "espejear",
    "espejemos": "espejar",
    "espejen": "espejar",
    "espejeo": "espejear",
    "espejeos": "espejeo",
    "espejera": "espejero",
    "espejeras": "espejero",
    "espejeros": "espejero",
    "espejerías": "espejería",
    "espejes": "espejar",
    "espejeábamos": "espejear",
    "espejeáis": "espejear",
    "espejeáramos": "espejear",
    "espejeáremos": "espejear",
    "espejeásemos": "espejear",
    "espejeé": "espejear",
    "espejeéis": "espejear",
    "espejeó": "espejear",
    "espejillos": "espejillo",
    "espejismos": "espejismo",
    "espejo": "espejar",
    "espejos": "espejo",
    "espejuelas": "espejuela",
    "espejuelos": "espejuelo",
    "espejábamos": "espejar",
    "espejáis": "espejar",
    "espejáramos": "espejar",
    "espejáremos": "espejar",
    "espejásemos": "espejar",
    "espejé": "espejar",
    "espejéis": "espejar",
    "espejó": "espejar",
    "espeleologías": "espeleología",
    "espeleológica": "espeleológico",
    "espeleológicas": "espeleológico",
    "espeleológicos": "espeleológico",
    "espeleóloga": "espeleólogo",
    "espeleólogas": "espeleólogo",
    "espeleólogos": "espeleólogo",
    "espeltas": "espelta",
    "espeluce": "espeluzar",
    "espelucemos": "espeluzar",
    "espelucen": "espeluzar",
    "espeluces": "espeluzar",
    "espelucé": "espeluzar",
    "espelucéis": "espeluzar",
    "espeluncas": "espelunca",
    "espeluza": "espeluzar",
    "espeluzaba": "espeluzar",
    "espeluzabais": "espeluzar",
    "espeluzaban": "espeluzar",
    "espeluzabas": "espeluzar",
    "espeluzad": "espeluzar",
    "espeluzada": "espeluzar",
    "espeluzadas": "espeluzar",
    "espeluzado": "espeluzar",
    "espeluzados": "espeluzar",
    "espeluzamos": "espeluzar",
    "espeluzan": "espeluzar",
    "espeluzando": "espeluzar",
    "espeluzara": "espeluzar",
    "espeluzarais": "espeluzar",
    "espeluzaran": "espeluzar",
    "espeluzaras": "espeluzar",
    "espeluzare": "espeluzar",
    "espeluzareis": "espeluzar",
    "espeluzaremos": "espeluzar",
    "espeluzaren": "espeluzar",
    "espeluzares": "espeluzar",
    "espeluzaron": "espeluzar",
    "espeluzará": "espeluzar",
    "espeluzarán": "espeluzar",
    "espeluzarás": "espeluzar",
    "espeluzaré": "espeluzar",
    "espeluzaréis": "espeluzar",
    "espeluzaría": "espeluzar",
    "espeluzaríais": "espeluzar",
    "espeluzaríamos": "espeluzar",
    "espeluzarían": "espeluzar",
    "espeluzarías": "espeluzar",
    "espeluzas": "espeluzar",
    "espeluzase": "espeluzar",
    "espeluzaseis": "espeluzar",
    "espeluzasen": "espeluzar",
    "espeluzases": "espeluzar",
    "espeluzaste": "espeluzar",
    "espeluzasteis": "espeluzar",
    "espeluzna": "espeluznar",
    "espeluznaba": "espeluznar",
    "espeluznabais": "espeluznar",
    "espeluznaban": "espeluznar",
    "espeluznabas": "espeluznar",
    "espeluznad": "espeluznar",
    "espeluznada": "espeluznar",
    "espeluznadas": "espeluznar",
    "espeluznado": "espeluznar",
    "espeluznados": "espeluznar",
    "espeluznamientos": "espeluznamiento",
    "espeluznamos": "espeluznar",
    "espeluznan": "espeluznar",
    "espeluznando": "espeluznar",
    "espeluznantes": "espeluznante",
    "espeluznara": "espeluznar",
    "espeluznarais": "espeluznar",
    "espeluznaran": "espeluznar",
    "espeluznaras": "espeluznar",
    "espeluznare": "espeluznar",
    "espeluznareis": "espeluznar",
    "espeluznaremos": "espeluznar",
    "espeluznaren": "espeluznar",
    "espeluznares": "espeluznar",
    "espeluznaron": "espeluznar",
    "espeluznará": "espeluznar",
    "espeluznarán": "espeluznar",
    "espeluznarás": "espeluznar",
    "espeluznaré": "espeluznar",
    "espeluznaréis": "espeluznar",
    "espeluznaría": "espeluznar",
    "espeluznaríais": "espeluznar",
    "espeluznaríamos": "espeluznar",
    "espeluznarían": "espeluznar",
    "espeluznarías": "espeluznar",
    "espeluznas": "espeluznar",
    "espeluznase": "espeluznar",
    "espeluznaseis": "espeluznar",
    "espeluznasen": "espeluznar",
    "espeluznases": "espeluznar",
    "espeluznaste": "espeluznar",
    "espeluznasteis": "espeluznar",
    "espeluzne": "espeluznar",
    "espeluznemos": "espeluznar",
    "espeluznen": "espeluznar",
    "espeluznes": "espeluznar",
    "espeluzno": "espeluznar",
    "espeluznos": "espeluzno",
    "espeluznábamos": "espeluznar",
    "espeluznáis": "espeluznar",
    "espeluznáramos": "espeluznar",
    "espeluznáremos": "espeluznar",
    "espeluznásemos": "espeluznar",
    "espeluzné": "espeluznar",
    "espeluznéis": "espeluznar",
    "espeluznó": "espeluznar",
    "espeluzo": "espeluzar",
    "espeluzábamos": "espeluzar",
    "espeluzáis": "espeluzar",
    "espeluzáramos": "espeluzar",
    "espeluzáremos": "espeluzar",
    "espeluzásemos": "espeluzar",
    "espeluzó": "espeluzar",
    "espeques": "espeque",
    "espera": "esperar",
    "esperaba": "esperar",
    "esperabais": "esperar",
    "esperaban": "esperar",
    "esperabas": "esperar",
    "esperables": "esperable",
    "esperad": "esperar",
    "esperada": "esperar",
    "esperadas": "esperar",
    "esperado": "esperar",
    "esperados": "esperar",
    "esperamos": "esperar",
    "esperan": "esperar",
    "esperance": "esperanzar",
    "esperancemos": "esperanzar",
    "esperancen": "esperanzar",
    "esperances": "esperanzar",
    "esperancé": "esperanzar",
    "esperancéis": "esperanzar",
    "esperando": "esperar",
    "esperantistas": "esperantista",
    "esperantos": "esperanto",
    "esperanza": "esperanzar",
    "esperanzaba": "esperanzar",
    "esperanzabais": "esperanzar",
    "esperanzaban": "esperanzar",
    "esperanzabas": "esperanzar",
    "esperanzad": "esperanzar",
    "esperanzada": "esperanzar",
    "esperanzadas": "esperanzar",
    "esperanzado": "esperanzar",
    "esperanzadora": "esperanzador",
    "esperanzadoras": "esperanzador",
    "esperanzadores": "esperanzador",
    "esperanzados": "esperanzar",
    "esperanzamos": "esperanzar",
    "esperanzan": "esperanzar",
    "esperanzando": "esperanzar",
    "esperanzara": "esperanzar",
    "esperanzarais": "esperanzar",
    "esperanzaran": "esperanzar",
    "esperanzaras": "esperanzar",
    "esperanzare": "esperanzar",
    "esperanzareis": "esperanzar",
    "esperanzaremos": "esperanzar",
    "esperanzaren": "esperanzar",
    "esperanzares": "esperanzar",
    "esperanzaron": "esperanzar",
    "esperanzará": "esperanzar",
    "esperanzarán": "esperanzar",
    "esperanzarás": "esperanzar",
    "esperanzaré": "esperanzar",
    "esperanzaréis": "esperanzar",
    "esperanzaría": "esperanzar",
    "esperanzaríais": "esperanzar",
    "esperanzaríamos": "esperanzar",
    "esperanzarían": "esperanzar",
    "esperanzarías": "esperanzar",
    "esperanzas": "esperanzar",
    "esperanzase": "esperanzar",
    "esperanzaseis": "esperanzar",
    "esperanzasen": "esperanzar",
    "esperanzases": "esperanzar",
    "esperanzaste": "esperanzar",
    "esperanzasteis": "esperanzar",
    "esperanzo": "esperanzar",
    "esperanzábamos": "esperanzar",
    "esperanzáis": "esperanzar",
    "esperanzáramos": "esperanzar",
    "esperanzáremos": "esperanzar",
    "esperanzásemos": "esperanzar",
    "esperanzó": "esperanzar",
    "esperara": "esperar",
    "esperarais": "esperar",
    "esperaran": "esperar",
    "esperaras": "esperar",
    "esperare": "esperar",
    "esperareis": "esperar",
    "esperaremos": "esperar",
    "esperaren": "esperar",
    "esperares": "esperar",
    "esperaron": "esperar",
    "esperará": "esperar",
    "esperarán": "esperar",
    "esperarás": "esperar",
    "esperaré": "esperar",
    "esperaréis": "esperar",
    "esperaría": "esperar",
    "esperaríais": "esperar",
    "esperaríamos": "esperar",
    "esperarían": "esperar",
    "esperarías": "esperar",
    "esperas": "esperar",
    "esperase": "esperar",
    "esperaseis": "esperar",
    "esperasen": "esperar",
    "esperases": "esperar",
    "esperaste": "esperar",
    "esperasteis": "esperar",
    "espere": "esperar",
    "esperemos": "esperar",
    "esperen": "esperar",
    "esperes": "esperar",
    "esperezamientos": "esperezamiento",
    "esperezos": "esperezo",
    "espermacetis": "espermaceti",
    "espermas": "esperma",
    "espermaticidas": "espermaticida",
    "espermatidas": "espermatida",
    "espermatocitos": "espermatocito",
    "espermatogonias": "espermatogonia",
    "espermatorreas": "espermatorrea",
    "espermatozoarios": "espermatozoario",
    "espermatozoides": "espermatozoide",
    "espermatozoos": "espermatozoo",
    "espermatócitos": "espermatócito",
    "espermatófitos": "espermatófito",
    "espermatóforos": "espermatóforo",
    "esperminas": "espermina",
    "espermática": "espermático",
    "espermáticas": "espermático",
    "espermáticos": "espermático",
    "espermátidas": "espermátida",
    "espermátides": "espermátide",
    "espernadas": "espernada",
    "espernanca": "espernancar",
    "espernancaba": "espernancar",
    "espernancabais": "espernancar",
    "espernancaban": "espernancar",
    "espernancabas": "espernancar",
    "espernancad": "espernancar",
    "espernancada": "espernancar",
    "espernancadas": "espernancar",
    "espernancado": "espernancar",
    "espernancados": "espernancar",
    "espernancamos": "espernancar",
    "espernancan": "espernancar",
    "espernancando": "espernancar",
    "espernancara": "espernancar",
    "espernancarais": "espernancar",
    "espernancaran": "espernancar",
    "espernancaras": "espernancar",
    "espernancare": "espernancar",
    "espernancareis": "espernancar",
    "espernancaremos": "espernancar",
    "espernancaren": "espernancar",
    "espernancares": "espernancar",
    "espernancaron": "espernancar",
    "espernancará": "espernancar",
    "espernancarán": "espernancar",
    "espernancarás": "espernancar",
    "espernancaré": "espernancar",
    "espernancaréis": "espernancar",
    "espernancaría": "espernancar",
    "espernancaríais": "espernancar",
    "espernancaríamos": "espernancar",
    "espernancarían": "espernancar",
    "espernancarías": "espernancar",
    "espernancas": "espernancar",
    "espernancase": "espernancar",
    "espernancaseis": "espernancar",
    "espernancasen": "espernancar",
    "espernancases": "espernancar",
    "espernancaste": "espernancar",
    "espernancasteis": "espernancar",
    "espernanco": "espernancar",
    "espernancábamos": "espernancar",
    "espernancáis": "espernancar",
    "espernancáramos": "espernancar",
    "espernancáremos": "espernancar",
    "espernancásemos": "espernancar",
    "espernancó": "espernancar",
    "espernanque": "espernancar",
    "espernanquemos": "espernancar",
    "espernanquen": "espernancar",
    "espernanques": "espernancar",
    "espernanqué": "espernancar",
    "espernanquéis": "espernancar",
    "espero": "esperar",
    "esperones": "esperón",
    "esperpentos": "esperpento",
    "esperpéntica": "esperpéntico",
    "esperpénticas": "esperpéntico",
    "esperpénticos": "esperpéntico",
    "esperábamos": "esperar",
    "esperáis": "esperar",
    "esperáramos": "esperar",
    "esperáremos": "esperar",
    "esperásemos": "esperar",
    "esperé": "esperar",
    "esperéis": "esperar",
    "esperó": "esperar",
    "espesa": "espeso",
    "espesaba": "espesar",
    "espesabais": "espesar",
    "espesaban": "espesar",
    "espesabas": "espesar",
    "espesad": "espesar",
    "espesada": "espesar",
    "espesadas": "espesar",
    "espesado": "espesar",
    "espesadores": "espesador",
    "espesados": "espesar",
    "espesamientos": "espesamiento",
    "espesamos": "espesar",
    "espesan": "espesar",
    "espesando": "espesar",
    "espesantes": "espesante",
    "espesara": "espesar",
    "espesarais": "espesar",
    "espesaran": "espesar",
    "espesaras": "espesar",
    "espesare": "espesar",
    "espesareis": "espesar",
    "espesaremos": "espesar",
    "espesaren": "espesar",
    "espesares": "espesar",
    "espesaron": "espesar",
    "espesartinas": "espesartina",
    "espesartitas": "espesartita",
    "espesará": "espesar",
    "espesarán": "espesar",
    "espesarás": "espesar",
    "espesaré": "espesar",
    "espesaréis": "espesar",
    "espesaría": "espesar",
    "espesaríais": "espesar",
    "espesaríamos": "espesar",
    "espesarían": "espesar",
    "espesarías": "espesar",
    "espesas": "espeso",
    "espesase": "espesar",
    "espesaseis": "espesar",
    "espesasen": "espesar",
    "espesases": "espesar",
    "espesaste": "espesar",
    "espesasteis": "espesar",
    "espese": "espesar",
    "espesemos": "espesar",
    "espesen": "espesar",
    "espeses": "espesar",
    "espeso": "espesar",
    "espesores": "espesor",
    "espesos": "espeso",
    "espesuras": "espesura",
    "espesábamos": "espesar",
    "espesáis": "espesar",
    "espesáramos": "espesar",
    "espesáremos": "espesar",
    "espesásemos": "espesar",
    "espesé": "espesar",
    "espeséis": "espesar",
    "espesó": "espesar",
    "espeta": "espetar",
    "espetaba": "espetar",
    "espetabais": "espetar",
    "espetaban": "espetar",
    "espetabas": "espetar",
    "espetad": "espetar",
    "espetada": "espetar",
    "espetadas": "espetar",
    "espetado": "espetar",
    "espetados": "espetar",
    "espetamos": "espetar",
    "espetan": "espetar",
    "espetando": "espetar",
    "espetara": "espetar",
    "espetarais": "espetar",
    "espetaran": "espetar",
    "espetaras": "espetar",
    "espetare": "espetar",
    "espetareis": "espetar",
    "espetaremos": "espetar",
    "espetaren": "espetar",
    "espetares": "espetar",
    "espetaron": "espetar",
    "espetará": "espetar",
    "espetarán": "espetar",
    "espetarás": "espetar",
    "espetaré": "espetar",
    "espetaréis": "espetar",
    "espetaría": "espetar",
    "espetaríais": "espetar",
    "espetaríamos": "espetar",
    "espetarían": "espetar",
    "espetarías": "espetar",
    "espetas": "espetar",
    "espetase": "espetar",
    "espetaseis": "espetar",
    "espetasen": "espetar",
    "espetases": "espetar",
    "espetaste": "espetar",
    "espetasteis": "espetar",
    "espete": "espetar",
    "espetemos": "espetar",
    "espeten": "espetar",
    "espeteras": "espetera",
    "espetes": "espetar",
    "espeto": "espetar",
    "espetones": "espetón",
    "espetábamos": "espetar",
    "espetáis": "espetar",
    "espetáramos": "espetar",
    "espetáremos": "espetar",
    "espetásemos": "espetar",
    "espeté": "espetar",
    "espetéis": "espetar",
    "espetó": "espetar",
    "espiaba": "espiar",
    "espiabais": "espiar",
    "espiaban": "espiar",
    "espiabas": "espiar",
    "espiad": "espiar",
    "espiada": "espiar",
    "espiadas": "espiar",
    "espiado": "espiar",
    "espiados": "espiar",
    "espiamos": "espiar",
    "espiando": "espiar",
    "espiara": "espiar",
    "espiarais": "espiar",
    "espiaran": "espiar",
    "espiaras": "espiar",
    "espiare": "espiar",
    "espiareis": "espiar",
    "espiaremos": "espiar",
    "espiaren": "espiar",
    "espiares": "espiar",
    "espiaron": "espiar",
    "espiará": "espiar",
    "espiarán": "espiar",
    "espiarás": "espiar",
    "espiaré": "espiar",
    "espiaréis": "espiar",
    "espiaría": "espiar",
    "espiaríais": "espiar",
    "espiaríamos": "espiar",
    "espiarían": "espiar",
    "espiarías": "espiar",
    "espiase": "espiar",
    "espiaseis": "espiar",
    "espiasen": "espiar",
    "espiases": "espiar",
    "espiaste": "espiar",
    "espiasteis": "espiar",
    "espibias": "espibia",
    "espibiones": "espibión",
    "espibios": "espibio",
    "espicanardos": "espicanardo",
    "espicha": "espichar",
    "espichaba": "espichar",
    "espichabais": "espichar",
    "espichaban": "espichar",
    "espichabas": "espichar",
    "espichad": "espichar",
    "espichada": "espichar",
    "espichadas": "espichar",
    "espichado": "espichar",
    "espichados": "espichar",
    "espichamos": "espichar",
    "espichan": "espichar",
    "espichando": "espichar",
    "espichara": "espichar",
    "espicharais": "espichar",
    "espicharan": "espichar",
    "espicharas": "espichar",
    "espichare": "espichar",
    "espichareis": "espichar",
    "espicharemos": "espichar",
    "espicharen": "espichar",
    "espichares": "espichar",
    "espicharon": "espichar",
    "espichará": "espichar",
    "espicharán": "espichar",
    "espicharás": "espichar",
    "espicharé": "espichar",
    "espicharéis": "espichar",
    "espicharía": "espichar",
    "espicharíais": "espichar",
    "espicharíamos": "espichar",
    "espicharían": "espichar",
    "espicharías": "espichar",
    "espichas": "espichar",
    "espichase": "espichar",
    "espichaseis": "espichar",
    "espichasen": "espichar",
    "espichases": "espichar",
    "espichaste": "espichar",
    "espichasteis": "espichar",
    "espiche": "espichar",
    "espichemos": "espichar",
    "espichen": "espichar",
    "espiches": "espiche",
    "espicho": "espichar",
    "espichones": "espichón",
    "espichábamos": "espichar",
    "espicháis": "espichar",
    "espicháramos": "espichar",
    "espicháremos": "espichar",
    "espichásemos": "espichar",
    "espiché": "espichar",
    "espichéis": "espichar",
    "espichó": "espichar",
    "espiciformes": "espiciforme",
    "espiemos": "espiar",
    "espiga": "espigar",
    "espigaba": "espigar",
    "espigabais": "espigar",
    "espigaban": "espigar",
    "espigabas": "espigar",
    "espigad": "espigar",
    "espigada": "espigar",
    "espigadas": "espigar",
    "espigado": "espigar",
    "espigadora": "espigador",
    "espigadoras": "espigador",
    "espigadores": "espigador",
    "espigados": "espigar",
    "espigamos": "espigar",
    "espigan": "espigar",
    "espigando": "espigar",
    "espigara": "espigar",
    "espigarais": "espigar",
    "espigaran": "espigar",
    "espigaras": "espigar",
    "espigare": "espigar",
    "espigareis": "espigar",
    "espigaremos": "espigar",
    "espigaren": "espigar",
    "espigares": "espigar",
    "espigaron": "espigar",
    "espigará": "espigar",
    "espigarán": "espigar",
    "espigarás": "espigar",
    "espigaré": "espigar",
    "espigaréis": "espigar",
    "espigaría": "espigar",
    "espigaríais": "espigar",
    "espigaríamos": "espigar",
    "espigarían": "espigar",
    "espigarías": "espigar",
    "espigas": "espigar",
    "espigase": "espigar",
    "espigaseis": "espigar",
    "espigasen": "espigar",
    "espigases": "espigar",
    "espigaste": "espigar",
    "espigasteis": "espigar",
    "espigo": "espigar",
    "espigones": "espigón",
    "espigos": "espigo",
    "espigue": "espigar",
    "espiguemos": "espigar",
    "espiguen": "espigar",
    "espigueos": "espigueo",
    "espigues": "espigar",
    "espiguillas": "espiguilla",
    "espigué": "espigar",
    "espiguéis": "espigar",
    "espigábamos": "espigar",
    "espigáis": "espigar",
    "espigáramos": "espigar",
    "espigáremos": "espigar",
    "espigásemos": "espigar",
    "espigó": "espigar",
    "espina": "espinar",
    "espinaba": "espinar",
    "espinabais": "espinar",
    "espinaban": "espinar",
    "espinabas": "espinar",
    "espinacas": "espinaca",
    "espinad": "espinar",
    "espinada": "espinar",
    "espinadas": "espinar",
    "espinado": "espinar",
    "espinados": "espinar",
    "espinaduras": "espinadura",
    "espinales": "espinal",
    "espinamos": "espinar",
    "espinan": "espinar",
    "espinando": "espinar",
    "espinapeces": "espinapez",
    "espinara": "espinar",
    "espinarais": "espinar",
    "espinaran": "espinar",
    "espinaras": "espinar",
    "espinare": "espinar",
    "espinareis": "espinar",
    "espinaremos": "espinar",
    "espinaren": "espinar",
    "espinares": "espinar",
    "espinaron": "espinar",
    "espinará": "espinar",
    "espinarán": "espinar",
    "espinarás": "espinar",
    "espinaré": "espinar",
    "espinaréis": "espinar",
    "espinaría": "espinar",
    "espinaríais": "espinar",
    "espinaríamos": "espinar",
    "espinarían": "espinar",
    "espinarías": "espinar",
    "espinas": "espinar",
    "espinase": "espinar",
    "espinaseis": "espinar",
    "espinasen": "espinar",
    "espinases": "espinar",
    "espinaste": "espinar",
    "espinasteis": "espinar",
    "espinazos": "espinazo",
    "espine": "espinar",
    "espinelas": "espinela",
    "espineles": "espinel",
    "espinemos": "espinar",
    "espinen": "espinar",
    "espineros": "espinero",
    "espines": "espinar",
    "espinescentes": "espinescente",
    "espinetas": "espineta",
    "espingardas": "espingarda",
    "espinillas": "espinilla",
    "espinilleras": "espinillera",
    "espinillos": "espinillo",
    "espino": "espinar",
    "espinos": "espino",
    "espinosa": "espinoso",
    "espinosas": "espinoso",
    "espinosismos": "espinosismo",
    "espinosistas": "espinosista",
    "espinosos": "espinoso",
    "espinterómetros": "espinterómetro",
    "espinábamos": "espinar",
    "espináis": "espinar",
    "espináramos": "espinar",
    "espináremos": "espinar",
    "espinásemos": "espinar",
    "espiné": "espinar",
    "espinéis": "espinar",
    "espinó": "espinar",
    "espiochas": "espiocha",
    "espiona": "espión",
    "espionajes": "espionaje",
    "espionas": "espión",
    "espiones": "espión",
    "espira": "espirar",
    "espiraba": "espirar",
    "espirabais": "espirar",
    "espiraban": "espirar",
    "espirabas": "espirar",
    "espiraciones": "espiración",
    "espirad": "espirar",
    "espirada": "espirar",
    "espiradas": "espirar",
    "espirado": "espirar",
    "espiradora": "espirador",
    "espiradoras": "espirador",
    "espiradores": "espirador",
    "espirados": "espirar",
    "espirales": "espiral",
    "espiramos": "espirar",
    "espiran": "espirar",
    "espirando": "espirar",
    "espirantes": "espirante",
    "espirantizaciones": "espirantización",
    "espirara": "espirar",
    "espirarais": "espirar",
    "espiraran": "espirar",
    "espiraras": "espirar",
    "espirare": "espirar",
    "espirareis": "espirar",
    "espiraremos": "espirar",
    "espiraren": "espirar",
    "espirares": "espirar",
    "espiraron": "espirar",
    "espirará": "espirar",
    "espirarán": "espirar",
    "espirarás": "espirar",
    "espiraré": "espirar",
    "espiraréis": "espirar",
    "espiraría": "espirar",
    "espiraríais": "espirar",
    "espiraríamos": "espirar",
    "espirarían": "espirar",
    "espirarías": "espirar",
    "espiras": "espirar",
    "espirase": "espirar",
    "espiraseis": "espirar",
    "espirasen": "espirar",
    "espirases": "espirar",
    "espiraste": "espirar",
    "espirasteis": "espirar",
    "espire": "espirar",
    "espireas": "espirea",
    "espiremas": "espirema",
    "espiremos": "espirar",
    "espiren": "espirar",
    "espires": "espirar",
    "espirilos": "espirilo",
    "espiritada": "espiritado",
    "espiritadas": "espiritado",
    "espiritados": "espiritado",
    "espiritismos": "espiritismo",
    "espiritistas": "espiritista",
    "espiritosa": "espiritoso",
    "espiritosas": "espiritoso",
    "espiritosos": "espiritoso",
    "espiritrompas": "espiritrompa",
    "espirituales": "espiritual",
    "espiritualice": "espiritualizar",
    "espiritualicemos": "espiritualizar",
    "espiritualicen": "espiritualizar",
    "espiritualices": "espiritualizar",
    "espiritualicé": "espiritualizar",
    "espiritualicéis": "espiritualizar",
    "espiritualidades": "espiritualidad",
    "espiritualismos": "espiritualismo",
    "espiritualistas": "espiritualista",
    "espiritualiza": "espiritualizar",
    "espiritualizaba": "espiritualizar",
    "espiritualizabais": "espiritualizar",
    "espiritualizaban": "espiritualizar",
    "espiritualizabas": "espiritualizar",
    "espiritualizaciones": "espiritualización",
    "espiritualizad": "espiritualizar",
    "espiritualizada": "espiritualizar",
    "espiritualizadas": "espiritualizar",
    "espiritualizado": "espiritualizar",
    "espiritualizados": "espiritualizar",
    "espiritualizamos": "espiritualizar",
    "espiritualizan": "espiritualizar",
    "espiritualizando": "espiritualizar",
    "espiritualizara": "espiritualizar",
    "espiritualizarais": "espiritualizar",
    "espiritualizaran": "espiritualizar",
    "espiritualizaras": "espiritualizar",
    "espiritualizare": "espiritualizar",
    "espiritualizareis": "espiritualizar",
    "espiritualizaremos": "espiritualizar",
    "espiritualizaren": "espiritualizar",
    "espiritualizares": "espiritualizar",
    "espiritualizaron": "espiritualizar",
    "espiritualizará": "espiritualizar",
    "espiritualizarán": "espiritualizar",
    "espiritualizarás": "espiritualizar",
    "espiritualizaré": "espiritualizar",
    "espiritualizaréis": "espiritualizar",
    "espiritualizaría": "espiritualizar",
    "espiritualizaríais": "espiritualizar",
    "espiritualizaríamos": "espiritualizar",
    "espiritualizarían": "espiritualizar",
    "espiritualizarías": "espiritualizar",
    "espiritualizas": "espiritualizar",
    "espiritualizase": "espiritualizar",
    "espiritualizaseis": "espiritualizar",
    "espiritualizasen": "espiritualizar",
    "espiritualizases": "espiritualizar",
    "espiritualizaste": "espiritualizar",
    "espiritualizasteis": "espiritualizar",
    "espiritualizo": "espiritualizar",
    "espiritualizábamos": "espiritualizar",
    "espiritualizáis": "espiritualizar",
    "espiritualizáramos": "espiritualizar",
    "espiritualizáremos": "espiritualizar",
    "espiritualizásemos": "espiritualizar",
    "espiritualizó": "espiritualizar",
    "espirituosa": "espirituoso",
    "espirituosas": "espirituoso",
    "espirituosos": "espirituoso",
    "espiro": "espirar",
    "espiroidea": "espiroideo",
    "espiroideas": "espiroideo",
    "espiroideos": "espiroideo",
    "espirometrías": "espirometría",
    "espirometálica": "espirometálico",
    "espirometálicas": "espirometálico",
    "espirometálicos": "espirometálico",
    "espiroquetales": "espiroquetal",
    "espiroquetas": "espiroqueta",
    "espiroquetáceas": "espiroquetácea",
    "espirábamos": "espirar",
    "espiráculos": "espiráculo",
    "espiráis": "espirar",
    "espiráramos": "espirar",
    "espiráremos": "espirar",
    "espirásemos": "espirar",
    "espiré": "espirar",
    "espiréis": "espirar",
    "espiró": "espirar",
    "espirógrafos": "espirógrafo",
    "espirómetros": "espirómetro",
    "espita": "espitar",
    "espitaba": "espitar",
    "espitabais": "espitar",
    "espitaban": "espitar",
    "espitabas": "espitar",
    "espitad": "espitar",
    "espitada": "espitar",
    "espitadas": "espitar",
    "espitado": "espitar",
    "espitados": "espitar",
    "espitamos": "espitar",
    "espitan": "espitar",
    "espitando": "espitar",
    "espitara": "espitar",
    "espitarais": "espitar",
    "espitaran": "espitar",
    "espitaras": "espitar",
    "espitare": "espitar",
    "espitareis": "espitar",
    "espitaremos": "espitar",
    "espitaren": "espitar",
    "espitares": "espitar",
    "espitaron": "espitar",
    "espitará": "espitar",
    "espitarán": "espitar",
    "espitarás": "espitar",
    "espitaré": "espitar",
    "espitaréis": "espitar",
    "espitaría": "espitar",
    "espitaríais": "espitar",
    "espitaríamos": "espitar",
    "espitarían": "espitar",
    "espitarías": "espitar",
    "espitas": "espitar",
    "espitase": "espitar",
    "espitaseis": "espitar",
    "espitasen": "espitar",
    "espitases": "espitar",
    "espitaste": "espitar",
    "espitasteis": "espitar",
    "espite": "espitar",
    "espitemos": "espitar",
    "espiten": "espitar",
    "espites": "espitar",
    "espito": "espitar",
    "espitos": "espito",
    "espitábamos": "espitar",
    "espitáis": "espitar",
    "espitáramos": "espitar",
    "espitáremos": "espitar",
    "espitásemos": "espitar",
    "espité": "espitar",
    "espitéis": "espitar",
    "espitó": "espitar",
    "espiábamos": "espiar",
    "espiáis": "espiar",
    "espiáramos": "espiar",
    "espiáremos": "espiar",
    "espiásemos": "espiar",
    "espié": "espiar",
    "espiéis": "espiar",
    "espió": "espiar",
    "esplacnocráneos": "esplacnocráneo",
    "esplacnoesqueletos": "esplacnoesqueleto",
    "esplenda": "esplender",
    "esplendamos": "esplender",
    "esplendan": "esplender",
    "esplendas": "esplender",
    "esplende": "esplender",
    "esplended": "esplender",
    "esplendemos": "esplender",
    "esplenden": "esplender",
    "esplenderemos": "esplender",
    "esplenderá": "esplender",
    "esplenderán": "esplender",
    "esplenderás": "esplender",
    "esplenderé": "esplender",
    "esplenderéis": "esplender",
    "esplendería": "esplender",
    "esplenderíais": "esplender",
    "esplenderíamos": "esplender",
    "esplenderían": "esplender",
    "esplenderías": "esplender",
    "esplendes": "esplender",
    "esplendida": "esplender",
    "esplendidas": "esplender",
    "esplendideces": "esplendidez",
    "esplendido": "esplender",
    "esplendidos": "esplender",
    "esplendiendo": "esplender",
    "esplendiera": "esplender",
    "esplendierais": "esplender",
    "esplendieran": "esplender",
    "esplendieras": "esplender",
    "esplendiere": "esplender",
    "esplendiereis": "esplender",
    "esplendieren": "esplender",
    "esplendieres": "esplender",
    "esplendieron": "esplender",
    "esplendiese": "esplender",
    "esplendieseis": "esplender",
    "esplendiesen": "esplender",
    "esplendieses": "esplender",
    "esplendimos": "esplender",
    "esplendiste": "esplender",
    "esplendisteis": "esplender",
    "esplendiéramos": "esplender",
    "esplendiéremos": "esplender",
    "esplendiésemos": "esplender",
    "esplendió": "esplender",
    "esplendo": "esplender",
    "esplendores": "esplendor",
    "esplendorosa": "esplendoroso",
    "esplendorosas": "esplendoroso",
    "esplendorosos": "esplendoroso",
    "esplendáis": "esplender",
    "esplendéis": "esplender",
    "esplendí": "esplender",
    "esplendía": "esplender",
    "esplendíais": "esplender",
    "esplendíamos": "esplender",
    "esplendían": "esplender",
    "esplendías": "esplender",
    "esplenectomías": "esplenectomía",
    "esplenios": "esplenio",
    "esplenomegalias": "esplenomegalia",
    "esplenopatías": "esplenopatía",
    "esplenética": "esplenético",
    "esplenéticas": "esplenético",
    "esplenéticos": "esplenético",
    "esplenócitos": "esplenócito",
    "espliegos": "espliego",
    "esplines": "esplín",
    "espliques": "esplique",
    "esplácnica": "esplácnico",
    "esplácnicas": "esplácnico",
    "esplácnicos": "esplácnico",
    "espléndida": "espléndido",
    "espléndidas": "espléndido",
    "espléndidos": "espléndido",
    "esplénica": "esplénico",
    "esplénicas": "esplénico",
    "esplénicos": "esplénico",
    "espodumenas": "espodumena",
    "espodúmenos": "espodúmeno",
    "espoladas": "espolada",
    "espolazos": "espolazo",
    "espolea": "espolear",
    "espoleaba": "espolear",
    "espoleabais": "espolear",
    "espoleaban": "espolear",
    "espoleabas": "espolear",
    "espolead": "espolear",
    "espoleada": "espolear",
    "espoleadas": "espolear",
    "espoleado": "espolear",
    "espoleados": "espolear",
    "espoleaduras": "espoleadura",
    "espoleamos": "espolear",
    "espolean": "espolear",
    "espoleando": "espolear",
    "espoleara": "espolear",
    "espolearais": "espolear",
    "espolearan": "espolear",
    "espolearas": "espolear",
    "espoleare": "espolear",
    "espoleareis": "espolear",
    "espolearemos": "espolear",
    "espolearen": "espolear",
    "espoleares": "espolear",
    "espolearon": "espolear",
    "espoleará": "espolear",
    "espolearán": "espolear",
    "espolearás": "espolear",
    "espolearé": "espolear",
    "espolearéis": "espolear",
    "espolearía": "espolear",
    "espolearíais": "espolear",
    "espolearíamos": "espolear",
    "espolearían": "espolear",
    "espolearías": "espolear",
    "espoleas": "espolear",
    "espolease": "espolear",
    "espoleaseis": "espolear",
    "espoleasen": "espolear",
    "espoleases": "espolear",
    "espoleaste": "espolear",
    "espoleasteis": "espolear",
    "espolee": "espolear",
    "espoleemos": "espolear",
    "espoleen": "espolear",
    "espolees": "espolear",
    "espoleo": "espolear",
    "espoletas": "espoleta",
    "espoleábamos": "espolear",
    "espoleáis": "espolear",
    "espoleáramos": "espolear",
    "espoleáremos": "espolear",
    "espoleásemos": "espolear",
    "espoleé": "espolear",
    "espoleéis": "espolear",
    "espoleó": "espolear",
    "espoliaciones": "espoliación",
    "espoliadora": "espoliador",
    "espoliadoras": "espoliador",
    "espoliadores": "espoliador",
    "espolina": "espolinar",
    "espolinaba": "espolinar",
    "espolinabais": "espolinar",
    "espolinaban": "espolinar",
    "espolinabas": "espolinar",
    "espolinad": "espolinar",
    "espolinada": "espolinar",
    "espolinadas": "espolinar",
    "espolinado": "espolinar",
    "espolinados": "espolinar",
    "espolinamos": "espolinar",
    "espolinan": "espolinar",
    "espolinando": "espolinar",
    "espolinara": "espolinar",
    "espolinarais": "espolinar",
    "espolinaran": "espolinar",
    "espolinaras": "espolinar",
    "espolinare": "espolinar",
    "espolinareis": "espolinar",
    "espolinaremos": "espolinar",
    "espolinaren": "espolinar",
    "espolinares": "espolinar",
    "espolinaron": "espolinar",
    "espolinará": "espolinar",
    "espolinarán": "espolinar",
    "espolinarás": "espolinar",
    "espolinaré": "espolinar",
    "espolinaréis": "espolinar",
    "espolinaría": "espolinar",
    "espolinaríais": "espolinar",
    "espolinaríamos": "espolinar",
    "espolinarían": "espolinar",
    "espolinarías": "espolinar",
    "espolinas": "espolinar",
    "espolinase": "espolinar",
    "espolinaseis": "espolinar",
    "espolinasen": "espolinar",
    "espolinases": "espolinar",
    "espolinaste": "espolinar",
    "espolinasteis": "espolinar",
    "espoline": "espolinar",
    "espolinemos": "espolinar",
    "espolinen": "espolinar",
    "espolines": "espolinar",
    "espolino": "espolinar",
    "espolinábamos": "espolinar",
    "espolináis": "espolinar",
    "espolináramos": "espolinar",
    "espolináremos": "espolinar",
    "espolinásemos": "espolinar",
    "espoliné": "espolinar",
    "espolinéis": "espolinar",
    "espolinó": "espolinar",
    "espolios": "espolio",
    "espoliques": "espolique",
    "espolonadas": "espolonada",
    "espolonazos": "espolonazo",
    "espolones": "espolón",
    "espolvorea": "espolvorear",
    "espolvoreaba": "espolvorear",
    "espolvoreabais": "espolvorear",
    "espolvoreaban": "espolvorear",
    "espolvoreabas": "espolvorear",
    "espolvoread": "espolvorear",
    "espolvoreada": "espolvorear",
    "espolvoreadas": "espolvorear",
    "espolvoreado": "espolvorear",
    "espolvoreadoras": "espolvoreadora",
    "espolvoreados": "espolvorear",
    "espolvoreamos": "espolvorear",
    "espolvorean": "espolvorear",
    "espolvoreando": "espolvorear",
    "espolvoreara": "espolvorear",
    "espolvorearais": "espolvorear",
    "espolvorearan": "espolvorear",
    "espolvorearas": "espolvorear",
    "espolvoreare": "espolvorear",
    "espolvoreareis": "espolvorear",
    "espolvorearemos": "espolvorear",
    "espolvorearen": "espolvorear",
    "espolvoreares": "espolvorear",
    "espolvorearon": "espolvorear",
    "espolvoreará": "espolvorear",
    "espolvorearán": "espolvorear",
    "espolvorearás": "espolvorear",
    "espolvorearé": "espolvorear",
    "espolvorearéis": "espolvorear",
    "espolvorearía": "espolvorear",
    "espolvorearíais": "espolvorear",
    "espolvorearíamos": "espolvorear",
    "espolvorearían": "espolvorear",
    "espolvorearías": "espolvorear",
    "espolvoreas": "espolvorear",
    "espolvorease": "espolvorear",
    "espolvoreaseis": "espolvorear",
    "espolvoreasen": "espolvorear",
    "espolvoreases": "espolvorear",
    "espolvoreaste": "espolvorear",
    "espolvoreasteis": "espolvorear",
    "espolvoree": "espolvorear",
    "espolvoreemos": "espolvorear",
    "espolvoreen": "espolvorear",
    "espolvorees": "espolvorear",
    "espolvoreo": "espolvorear",
    "espolvoreos": "espolvoreo",
    "espolvoreábamos": "espolvorear",
    "espolvoreáis": "espolvorear",
    "espolvoreáramos": "espolvorear",
    "espolvoreáremos": "espolvorear",
    "espolvoreásemos": "espolvorear",
    "espolvoreé": "espolvorear",
    "espolvoreéis": "espolvorear",
    "espolvoreó": "espolvorear",
    "espondaica": "espondaico",
    "espondaicas": "espondaico",
    "espondaicos": "espondaico",
    "espondeos": "espondeo",
    "espongiarios": "espongiario",
    "esponja": "esponjar",
    "esponjaba": "esponjar",
    "esponjabais": "esponjar",
    "esponjaban": "esponjar",
    "esponjabas": "esponjar",
    "esponjad": "esponjar",
    "esponjada": "esponjar",
    "esponjadas": "esponjar",
    "esponjado": "esponjar",
    "esponjados": "esponjar",
    "esponjaduras": "esponjadura",
    "esponjamientos": "esponjamiento",
    "esponjamos": "esponjar",
    "esponjan": "esponjar",
    "esponjando": "esponjar",
    "esponjara": "esponjar",
    "esponjarais": "esponjar",
    "esponjaran": "esponjar",
    "esponjaras": "esponjar",
    "esponjare": "esponjar",
    "esponjareis": "esponjar",
    "esponjaremos": "esponjar",
    "esponjaren": "esponjar",
    "esponjares": "esponjar",
    "esponjaron": "esponjar",
    "esponjará": "esponjar",
    "esponjarán": "esponjar",
    "esponjarás": "esponjar",
    "esponjaré": "esponjar",
    "esponjaréis": "esponjar",
    "esponjaría": "esponjar",
    "esponjaríais": "esponjar",
    "esponjaríamos": "esponjar",
    "esponjarían": "esponjar",
    "esponjarías": "esponjar",
    "esponjas": "esponjar",
    "esponjase": "esponjar",
    "esponjaseis": "esponjar",
    "esponjasen": "esponjar",
    "esponjases": "esponjar",
    "esponjaste": "esponjar",
    "esponjasteis": "esponjar",
    "esponje": "esponjar",
    "esponjemos": "esponjar",
    "esponjen": "esponjar",
    "esponjeras": "esponjera",
    "esponjes": "esponjar",
    "esponjo": "esponjar",
    "esponjosa": "esponjoso",
    "esponjosas": "esponjoso",
    "esponjosidades": "esponjosidad",
    "esponjosos": "esponjoso",
    "esponjábamos": "esponjar",
    "esponjáis": "esponjar",
    "esponjáramos": "esponjar",
    "esponjáremos": "esponjar",
    "esponjásemos": "esponjar",
    "esponjé": "esponjar",
    "esponjéis": "esponjar",
    "esponjó": "esponjar",
    "esponsalicia": "esponsalicio",
    "esponsalicias": "esponsalicio",
    "esponsalicios": "esponsalicio",
    "esponsorizaciones": "esponsorización",
    "espontanea": "espontanear",
    "espontaneaba": "espontanear",
    "espontaneabais": "espontanear",
    "espontaneaban": "espontanear",
    "espontaneabas": "espontanear",
    "espontanead": "espontanear",
    "espontaneada": "espontanear",
    "espontaneadas": "espontanear",
    "espontaneado": "espontanear",
    "espontaneados": "espontanear",
    "espontaneamos": "espontanear",
    "espontanean": "espontanear",
    "espontaneando": "espontanear",
    "espontaneara": "espontanear",
    "espontanearais": "espontanear",
    "espontanearan": "espontanear",
    "espontanearas": "espontanear",
    "espontaneare": "espontanear",
    "espontaneareis": "espontanear",
    "espontanearemos": "espontanear",
    "espontanearen": "espontanear",
    "espontaneares": "espontanear",
    "espontanearon": "espontanear",
    "espontaneará": "espontanear",
    "espontanearán": "espontanear",
    "espontanearás": "espontanear",
    "espontanearé": "espontanear",
    "espontanearéis": "espontanear",
    "espontanearía": "espontanear",
    "espontanearíais": "espontanear",
    "espontanearíamos": "espontanear",
    "espontanearían": "espontanear",
    "espontanearías": "espontanear",
    "espontaneas": "espontanear",
    "espontanease": "espontanear",
    "espontaneaseis": "espontanear",
    "espontaneasen": "espontanear",
    "espontaneases": "espontanear",
    "espontaneaste": "espontanear",
    "espontaneasteis": "espontanear",
    "espontanee": "espontanear",
    "espontaneemos": "espontanear",
    "espontaneen": "espontanear",
    "espontanees": "espontanear",
    "espontaneidades": "espontaneidad",
    "espontaneo": "espontanear",
    "espontaneábamos": "espontanear",
    "espontaneáis": "espontanear",
    "espontaneáramos": "espontanear",
    "espontaneáremos": "espontanear",
    "espontaneásemos": "espontanear",
    "espontaneé": "espontanear",
    "espontaneéis": "espontanear",
    "espontaneó": "espontanear",
    "espontánea": "espontáneo",
    "espontáneas": "espontáneo",
    "espontáneos": "espontáneo",
    "esporangios": "esporangio",
    "esporas": "espora",
    "esporidesminas": "esporidesmina",
    "esporofilos": "esporofilo",
    "esporozoarios": "esporozoario",
    "esporozoos": "esporozoo",
    "esportadas": "esportada",
    "esportea": "esportear",
    "esporteaba": "esportear",
    "esporteabais": "esportear",
    "esporteaban": "esportear",
    "esporteabas": "esportear",
    "esportead": "esportear",
    "esporteada": "esportear",
    "esporteadas": "esportear",
    "esporteado": "esportear",
    "esporteados": "esportear",
    "esporteamos": "esportear",
    "esportean": "esportear",
    "esporteando": "esportear",
    "esporteara": "esportear",
    "esportearais": "esportear",
    "esportearan": "esportear",
    "esportearas": "esportear",
    "esporteare": "esportear",
    "esporteareis": "esportear",
    "esportearemos": "esportear",
    "esportearen": "esportear",
    "esporteares": "esportear",
    "esportearon": "esportear",
    "esporteará": "esportear",
    "esportearán": "esportear",
    "esportearás": "esportear",
    "esportearé": "esportear",
    "esportearéis": "esportear",
    "esportearía": "esportear",
    "esportearíais": "esportear",
    "esportearíamos": "esportear",
    "esportearían": "esportear",
    "esportearías": "esportear",
    "esporteas": "esportear",
    "esportease": "esportear",
    "esporteaseis": "esportear",
    "esporteasen": "esportear",
    "esporteases": "esportear",
    "esporteaste": "esportear",
    "esporteasteis": "esportear",
    "esportee": "esportear",
    "esporteemos": "esportear",
    "esporteen": "esportear",
    "esportees": "esportear",
    "esporteo": "esportear",
    "esporteábamos": "esportear",
    "esporteáis": "esportear",
    "esporteáramos": "esportear",
    "esporteáremos": "esportear",
    "esporteásemos": "esportear",
    "esporteé": "esportear",
    "esporteéis": "esportear",
    "esporteó": "esportear",
    "esportilleros": "esportillero",
    "esportillos": "esportillo",
    "esportones": "esportón",
    "esporádica": "esporádico",
    "esporádicas": "esporádico",
    "esporádicos": "esporádico",
    "esporócitos": "esporócito",
    "esporófita": "esporófito",
    "esporófitas": "esporófito",
    "esporófitos": "esporófito",
    "esposa": "esposo",
    "esposaba": "esposar",
    "esposabais": "esposar",
    "esposaban": "esposar",
    "esposabas": "esposar",
    "esposad": "esposar",
    "esposada": "esposar",
    "esposadas": "esposar",
    "esposado": "esposar",
    "esposados": "esposar",
    "esposamos": "esposar",
    "esposan": "esposar",
    "esposando": "esposar",
    "esposara": "esposar",
    "esposarais": "esposar",
    "esposaran": "esposar",
    "esposaras": "esposar",
    "esposare": "esposar",
    "esposareis": "esposar",
    "esposaremos": "esposar",
    "esposaren": "esposar",
    "esposares": "esposar",
    "esposaron": "esposar",
    "esposará": "esposar",
    "esposarán": "esposar",
    "esposarás": "esposar",
    "esposaré": "esposar",
    "esposaréis": "esposar",
    "esposaría": "esposar",
    "esposaríais": "esposar",
    "esposaríamos": "esposar",
    "esposarían": "esposar",
    "esposarías": "esposar",
    "esposas": "esposo",
    "esposase": "esposar",
    "esposaseis": "esposar",
    "esposasen": "esposar",
    "esposases": "esposar",
    "esposaste": "esposar",
    "esposasteis": "esposar",
    "espose": "esposar",
    "esposemos": "esposar",
    "esposen": "esposar",
    "esposes": "esposar",
    "esposo": "esposar",
    "esposos": "esposo",
    "esposábamos": "esposar",
    "esposáis": "esposar",
    "esposáramos": "esposar",
    "esposáremos": "esposar",
    "esposásemos": "esposar",
    "esposé": "esposar",
    "esposéis": "esposar",
    "esposó": "esposar",
    "esprays": "espray",
    "esprinta": "esprintar",
    "esprintaba": "esprintar",
    "esprintabais": "esprintar",
    "esprintaban": "esprintar",
    "esprintabas": "esprintar",
    "esprintad": "esprintar",
    "esprintada": "esprintar",
    "esprintadas": "esprintar",
    "esprintado": "esprintar",
    "esprintados": "esprintar",
    "esprintamos": "esprintar",
    "esprintan": "esprintar",
    "esprintando": "esprintar",
    "esprintara": "esprintar",
    "esprintarais": "esprintar",
    "esprintaran": "esprintar",
    "esprintaras": "esprintar",
    "esprintare": "esprintar",
    "esprintareis": "esprintar",
    "esprintaremos": "esprintar",
    "esprintaren": "esprintar",
    "esprintares": "esprintar",
    "esprintaron": "esprintar",
    "esprintará": "esprintar",
    "esprintarán": "esprintar",
    "esprintarás": "esprintar",
    "esprintaré": "esprintar",
    "esprintaréis": "esprintar",
    "esprintaría": "esprintar",
    "esprintaríais": "esprintar",
    "esprintaríamos": "esprintar",
    "esprintarían": "esprintar",
    "esprintarías": "esprintar",
    "esprintas": "esprintar",
    "esprintase": "esprintar",
    "esprintaseis": "esprintar",
    "esprintasen": "esprintar",
    "esprintases": "esprintar",
    "esprintaste": "esprintar",
    "esprintasteis": "esprintar",
    "esprinte": "esprintar",
    "esprintemos": "esprintar",
    "esprinten": "esprintar",
    "esprinters": "esprinter",
    "esprintes": "esprintar",
    "esprinto": "esprintar",
    "esprints": "esprint",
    "esprintábamos": "esprintar",
    "esprintáis": "esprintar",
    "esprintáramos": "esprintar",
    "esprintáremos": "esprintar",
    "esprintásemos": "esprintar",
    "esprinté": "esprintar",
    "esprintéis": "esprintar",
    "esprintó": "esprintar",
    "esprues": "esprue",
    "esprínteres": "esprínter",
    "esprínters": "esprínter",
    "espuelas": "espuela",
    "espuertas": "espuerta",
    "espulga": "espulgar",
    "espulgaba": "espulgar",
    "espulgabais": "espulgar",
    "espulgaban": "espulgar",
    "espulgabas": "espulgar",
    "espulgad": "espulgar",
    "espulgada": "espulgar",
    "espulgadas": "espulgar",
    "espulgaderos": "espulgadero",
    "espulgado": "espulgar",
    "espulgadora": "espulgador",
    "espulgadoras": "espulgador",
    "espulgadores": "espulgador",
    "espulgados": "espulgar",
    "espulgamos": "espulgar",
    "espulgan": "espulgar",
    "espulgando": "espulgar",
    "espulgara": "espulgar",
    "espulgarais": "espulgar",
    "espulgaran": "espulgar",
    "espulgaras": "espulgar",
    "espulgare": "espulgar",
    "espulgareis": "espulgar",
    "espulgaremos": "espulgar",
    "espulgaren": "espulgar",
    "espulgares": "espulgar",
    "espulgaron": "espulgar",
    "espulgará": "espulgar",
    "espulgarán": "espulgar",
    "espulgarás": "espulgar",
    "espulgaré": "espulgar",
    "espulgaréis": "espulgar",
    "espulgaría": "espulgar",
    "espulgaríais": "espulgar",
    "espulgaríamos": "espulgar",
    "espulgarían": "espulgar",
    "espulgarías": "espulgar",
    "espulgas": "espulgar",
    "espulgase": "espulgar",
    "espulgaseis": "espulgar",
    "espulgasen": "espulgar",
    "espulgases": "espulgar",
    "espulgaste": "espulgar",
    "espulgasteis": "espulgar",
    "espulgo": "espulgar",
    "espulgos": "espulgo",
    "espulgue": "espulgar",
    "espulguemos": "espulgar",
    "espulguen": "espulgar",
    "espulgues": "espulgar",
    "espulgué": "espulgar",
    "espulguéis": "espulgar",
    "espulgábamos": "espulgar",
    "espulgáis": "espulgar",
    "espulgáramos": "espulgar",
    "espulgáremos": "espulgar",
    "espulgásemos": "espulgar",
    "espulgó": "espulgar",
    "espuma": "espumar",
    "espumaba": "espumar",
    "espumabais": "espumar",
    "espumaban": "espumar",
    "espumabas": "espumar",
    "espumaciones": "espumación",
    "espumad": "espumar",
    "espumada": "espumar",
    "espumadas": "espumar",
    "espumaderas": "espumadera",
    "espumado": "espumar",
    "espumadora": "espumador",
    "espumadoras": "espumador",
    "espumadores": "espumador",
    "espumados": "espumar",
    "espumaja": "espumajar",
    "espumajaba": "espumajar",
    "espumajabais": "espumajar",
    "espumajaban": "espumajar",
    "espumajabas": "espumajar",
    "espumajad": "espumajar",
    "espumajada": "espumajar",
    "espumajadas": "espumajar",
    "espumajado": "espumajar",
    "espumajados": "espumajar",
    "espumajamos": "espumajar",
    "espumajan": "espumajar",
    "espumajando": "espumajar",
    "espumajara": "espumajar",
    "espumajarais": "espumajar",
    "espumajaran": "espumajar",
    "espumajaras": "espumajar",
    "espumajare": "espumajar",
    "espumajareis": "espumajar",
    "espumajaremos": "espumajar",
    "espumajaren": "espumajar",
    "espumajares": "espumajar",
    "espumajaron": "espumajar",
    "espumajará": "espumajar",
    "espumajarán": "espumajar",
    "espumajarás": "espumajar",
    "espumajaré": "espumajar",
    "espumajaréis": "espumajar",
    "espumajaría": "espumajar",
    "espumajaríais": "espumajar",
    "espumajaríamos": "espumajar",
    "espumajarían": "espumajar",
    "espumajarías": "espumajar",
    "espumajas": "espumajar",
    "espumajase": "espumajar",
    "espumajaseis": "espumajar",
    "espumajasen": "espumajar",
    "espumajases": "espumajar",
    "espumajaste": "espumajar",
    "espumajasteis": "espumajar",
    "espumaje": "espumajar",
    "espumajea": "espumajear",
    "espumajeaba": "espumajear",
    "espumajeabais": "espumajear",
    "espumajeaban": "espumajear",
    "espumajeabas": "espumajear",
    "espumajead": "espumajear",
    "espumajeada": "espumajear",
    "espumajeadas": "espumajear",
    "espumajeado": "espumajear",
    "espumajeados": "espumajear",
    "espumajeamos": "espumajear",
    "espumajean": "espumajear",
    "espumajeando": "espumajear",
    "espumajeara": "espumajear",
    "espumajearais": "espumajear",
    "espumajearan": "espumajear",
    "espumajearas": "espumajear",
    "espumajeare": "espumajear",
    "espumajeareis": "espumajear",
    "espumajearemos": "espumajear",
    "espumajearen": "espumajear",
    "espumajeares": "espumajear",
    "espumajearon": "espumajear",
    "espumajeará": "espumajear",
    "espumajearán": "espumajear",
    "espumajearás": "espumajear",
    "espumajearé": "espumajear",
    "espumajearéis": "espumajear",
    "espumajearía": "espumajear",
    "espumajearíais": "espumajear",
    "espumajearíamos": "espumajear",
    "espumajearían": "espumajear",
    "espumajearías": "espumajear",
    "espumajeas": "espumajear",
    "espumajease": "espumajear",
    "espumajeaseis": "espumajear",
    "espumajeasen": "espumajear",
    "espumajeases": "espumajear",
    "espumajeaste": "espumajear",
    "espumajeasteis": "espumajear",
    "espumajee": "espumajear",
    "espumajeemos": "espumajear",
    "espumajeen": "espumajear",
    "espumajees": "espumajear",
    "espumajemos": "espumajar",
    "espumajen": "espumajar",
    "espumajeo": "espumajear",
    "espumajes": "espumaje",
    "espumajeábamos": "espumajear",
    "espumajeáis": "espumajear",
    "espumajeáramos": "espumajear",
    "espumajeáremos": "espumajear",
    "espumajeásemos": "espumajear",
    "espumajeé": "espumajear",
    "espumajeéis": "espumajear",
    "espumajeó": "espumajear",
    "espumajo": "espumajar",
    "espumajos": "espumajo",
    "espumajábamos": "espumajar",
    "espumajáis": "espumajar",
    "espumajáramos": "espumajar",
    "espumajáremos": "espumajar",
    "espumajásemos": "espumajar",
    "espumajé": "espumajar",
    "espumajéis": "espumajar",
    "espumajó": "espumajar",
    "espumamos": "espumar",
    "espuman": "espumar",
    "espumando": "espumar",
    "espumantes": "espumante",
    "espumara": "espumar",
    "espumarais": "espumar",
    "espumarajos": "espumarajo",
    "espumaran": "espumar",
    "espumaras": "espumar",
    "espumare": "espumar",
    "espumareis": "espumar",
    "espumaremos": "espumar",
    "espumaren": "espumar",
    "espumares": "espumar",
    "espumaron": "espumar",
    "espumará": "espumar",
    "espumarán": "espumar",
    "espumarás": "espumar",
    "espumaré": "espumar",
    "espumaréis": "espumar",
    "espumaría": "espumar",
    "espumaríais": "espumar",
    "espumaríamos": "espumar",
    "espumarían": "espumar",
    "espumarías": "espumar",
    "espumas": "espumar",
    "espumase": "espumar",
    "espumaseis": "espumar",
    "espumasen": "espumar",
    "espumases": "espumar",
    "espumaste": "espumar",
    "espumasteis": "espumar",
    "espume": "espumar",
    "espumemos": "espumar",
    "espumen": "espumar",
    "espumeros": "espumero",
    "espumes": "espumar",
    "espumillas": "espumilla",
    "espumo": "espumar",
    "espumosa": "espumoso",
    "espumosas": "espumoso",
    "espumosidades": "espumosidad",
    "espumosos": "espumoso",
    "espumábamos": "espumar",
    "espumáis": "espumar",
    "espumáramos": "espumar",
    "espumáremos": "espumar",
    "espumásemos": "espumar",
    "espumé": "espumar",
    "espuméis": "espumar",
    "espumó": "espumar",
    "espundias": "espundia",
    "espuria": "espurio",
    "espurias": "espurio",
    "espurios": "espurio",
    "espurrea": "espurrear",
    "espurreaba": "espurrear",
    "espurreabais": "espurrear",
    "espurreaban": "espurrear",
    "espurreabas": "espurrear",
    "espurread": "espurrear",
    "espurreada": "espurrear",
    "espurreadas": "espurrear",
    "espurreado": "espurrear",
    "espurreados": "espurrear",
    "espurreamos": "espurrear",
    "espurrean": "espurrear",
    "espurreando": "espurrear",
    "espurreara": "espurrear",
    "espurrearais": "espurrear",
    "espurrearan": "espurrear",
    "espurrearas": "espurrear",
    "espurreare": "espurrear",
    "espurreareis": "espurrear",
    "espurrearemos": "espurrear",
    "espurrearen": "espurrear",
    "espurreares": "espurrear",
    "espurrearon": "espurrear",
    "espurreará": "espurrear",
    "espurrearán": "espurrear",
    "espurrearás": "espurrear",
    "espurrearé": "espurrear",
    "espurrearéis": "espurrear",
    "espurrearía": "espurrear",
    "espurrearíais": "espurrear",
    "espurrearíamos": "espurrear",
    "espurrearían": "espurrear",
    "espurrearías": "espurrear",
    "espurreas": "espurrear",
    "espurrease": "espurrear",
    "espurreaseis": "espurrear",
    "espurreasen": "espurrear",
    "espurreases": "espurrear",
    "espurreaste": "espurrear",
    "espurreasteis": "espurrear",
    "espurree": "espurrear",
    "espurreemos": "espurrear",
    "espurreen": "espurrear",
    "espurrees": "espurrear",
    "espurreo": "espurrear",
    "espurreábamos": "espurrear",
    "espurreáis": "espurrear",
    "espurreáramos": "espurrear",
    "espurreáremos": "espurrear",
    "espurreásemos": "espurrear",
    "espurreé": "espurrear",
    "espurreéis": "espurrear",
    "espurreó": "espurrear",
    "espurriaba": "espurriar",
    "espurriabais": "espurriar",
    "espurriaban": "espurriar",
    "espurriabas": "espurriar",
    "espurriad": "espurriar",
    "espurriada": "espurriar",
    "espurriadas": "espurriar",
    "espurriado": "espurriar",
    "espurriados": "espurriar",
    "espurriamos": "espurriar",
    "espurriando": "espurriar",
    "espurriara": "espurriar",
    "espurriarais": "espurriar",
    "espurriaran": "espurriar",
    "espurriaras": "espurriar",
    "espurriare": "espurriar",
    "espurriareis": "espurriar",
    "espurriaremos": "espurriar",
    "espurriaren": "espurriar",
    "espurriares": "espurriar",
    "espurriaron": "espurriar",
    "espurriará": "espurriar",
    "espurriarán": "espurriar",
    "espurriarás": "espurriar",
    "espurriaré": "espurriar",
    "espurriaréis": "espurriar",
    "espurriaría": "espurriar",
    "espurriaríais": "espurriar",
    "espurriaríamos": "espurriar",
    "espurriarían": "espurriar",
    "espurriarías": "espurriar",
    "espurriase": "espurriar",
    "espurriaseis": "espurriar",
    "espurriasen": "espurriar",
    "espurriases": "espurriar",
    "espurriaste": "espurriar",
    "espurriasteis": "espurriar",
    "espurriemos": "espurriar",
    "espurriábamos": "espurriar",
    "espurriáis": "espurriar",
    "espurriáramos": "espurriar",
    "espurriáremos": "espurriar",
    "espurriásemos": "espurriar",
    "espurrié": "espurriar",
    "espurriéis": "espurriar",
    "espurrió": "espurriar",
    "espurría": "espurriar",
    "espurrían": "espurriar",
    "espurrías": "espurriar",
    "espurríe": "espurriar",
    "espurríen": "espurriar",
    "espurríes": "espurriar",
    "espurrío": "espurriar",
    "esputa": "esputar",
    "esputaba": "esputar",
    "esputabais": "esputar",
    "esputaban": "esputar",
    "esputabas": "esputar",
    "esputad": "esputar",
    "esputada": "esputar",
    "esputadas": "esputar",
    "esputado": "esputar",
    "esputados": "esputar",
    "esputamos": "esputar",
    "esputan": "esputar",
    "esputando": "esputar",
    "esputara": "esputar",
    "esputarais": "esputar",
    "esputaran": "esputar",
    "esputaras": "esputar",
    "esputare": "esputar",
    "esputareis": "esputar",
    "esputaremos": "esputar",
    "esputaren": "esputar",
    "esputares": "esputar",
    "esputaron": "esputar",
    "esputará": "esputar",
    "esputarán": "esputar",
    "esputarás": "esputar",
    "esputaré": "esputar",
    "esputaréis": "esputar",
    "esputaría": "esputar",
    "esputaríais": "esputar",
    "esputaríamos": "esputar",
    "esputarían": "esputar",
    "esputarías": "esputar",
    "esputas": "esputar",
    "esputase": "esputar",
    "esputaseis": "esputar",
    "esputasen": "esputar",
    "esputases": "esputar",
    "esputaste": "esputar",
    "esputasteis": "esputar",
    "espute": "esputar",
    "esputemos": "esputar",
    "esputen": "esputar",
    "esputes": "esputar",
    "esputo": "esputar",
    "esputos": "esputo",
    "esputábamos": "esputar",
    "esputáis": "esputar",
    "esputáramos": "esputar",
    "esputáremos": "esputar",
    "esputásemos": "esputar",
    "esputé": "esputar",
    "esputéis": "esputar",
    "esputó": "esputar",
    "espádices": "espádice",
    "espáridos": "espárido",
    "espárragos": "espárrago",
    "espástica": "espástico",
    "espásticas": "espástico",
    "espásticos": "espástico",
    "espática": "espático",
    "espáticas": "espático",
    "espáticos": "espático",
    "espátulas": "espátula",
    "espéculos": "espéculo",
    "espéltea": "espélteo",
    "espélteas": "espélteo",
    "espélteos": "espélteo",
    "espía": "espiar",
    "espían": "espiar",
    "espías": "espiar",
    "espículas": "espícula",
    "espíe": "espiar",
    "espíen": "espiar",
    "espíes": "espiar",
    "espío": "espiar",
    "espíritus": "espíritu",
    "espódica": "espódico",
    "espódicas": "espódico",
    "espódicos": "espódico",
    "espóndiles": "espóndil",
    "espóndilos": "espóndilo",
    "espónsores": "espónsor",
    "espúrea": "espúreo",
    "espúreas": "espúreo",
    "espúreos": "espúreo",
    "esquejes": "esqueje",
    "esquelas": "esquela",
    "esqueletos": "esqueleto",
    "esquelética": "esquelético",
    "esqueléticas": "esquelético",
    "esqueléticos": "esquelético",
    "esquemas": "esquema",
    "esquematice": "esquematizar",
    "esquematicemos": "esquematizar",
    "esquematicen": "esquematizar",
    "esquematices": "esquematizar",
    "esquematicé": "esquematizar",
    "esquematicéis": "esquematizar",
    "esquematismos": "esquematismo",
    "esquematiza": "esquematizar",
    "esquematizaba": "esquematizar",
    "esquematizabais": "esquematizar",
    "esquematizaban": "esquematizar",
    "esquematizabas": "esquematizar",
    "esquematizaciones": "esquematización",
    "esquematizad": "esquematizar",
    "esquematizada": "esquematizar",
    "esquematizadas": "esquematizar",
    "esquematizado": "esquematizar",
    "esquematizados": "esquematizar",
    "esquematizamos": "esquematizar",
    "esquematizan": "esquematizar",
    "esquematizando": "esquematizar",
    "esquematizara": "esquematizar",
    "esquematizarais": "esquematizar",
    "esquematizaran": "esquematizar",
    "esquematizaras": "esquematizar",
    "esquematizare": "esquematizar",
    "esquematizareis": "esquematizar",
    "esquematizaremos": "esquematizar",
    "esquematizaren": "esquematizar",
    "esquematizares": "esquematizar",
    "esquematizaron": "esquematizar",
    "esquematizará": "esquematizar",
    "esquematizarán": "esquematizar",
    "esquematizarás": "esquematizar",
    "esquematizaré": "esquematizar",
    "esquematizaréis": "esquematizar",
    "esquematizaría": "esquematizar",
    "esquematizaríais": "esquematizar",
    "esquematizaríamos": "esquematizar",
    "esquematizarían": "esquematizar",
    "esquematizarías": "esquematizar",
    "esquematizas": "esquematizar",
    "esquematizase": "esquematizar",
    "esquematizaseis": "esquematizar",
    "esquematizasen": "esquematizar",
    "esquematizases": "esquematizar",
    "esquematizaste": "esquematizar",
    "esquematizasteis": "esquematizar",
    "esquematizo": "esquematizar",
    "esquematizábamos": "esquematizar",
    "esquematizáis": "esquematizar",
    "esquematizáramos": "esquematizar",
    "esquematizáremos": "esquematizar",
    "esquematizásemos": "esquematizar",
    "esquematizó": "esquematizar",
    "esquemática": "esquemático",
    "esquemáticas": "esquemático",
    "esquemáticos": "esquemático",
    "esquenas": "esquena",
    "esqueros": "esquero",
    "esquerranistas": "esquerranista",
    "esquiaba": "esquiar",
    "esquiabais": "esquiar",
    "esquiaban": "esquiar",
    "esquiabas": "esquiar",
    "esquiables": "esquiable",
    "esquiad": "esquiar",
    "esquiada": "esquiar",
    "esquiadas": "esquiar",
    "esquiado": "esquiar",
    "esquiadora": "esquiador",
    "esquiadoras": "esquiador",
    "esquiadores": "esquiador",
    "esquiados": "esquiar",
    "esquiajes": "esquiaje",
    "esquiamos": "esquiar",
    "esquiando": "esquiar",
    "esquiara": "esquiar",
    "esquiarais": "esquiar",
    "esquiaran": "esquiar",
    "esquiaras": "esquiar",
    "esquiare": "esquiar",
    "esquiareis": "esquiar",
    "esquiaremos": "esquiar",
    "esquiaren": "esquiar",
    "esquiares": "esquiar",
    "esquiaron": "esquiar",
    "esquiará": "esquiar",
    "esquiarán": "esquiar",
    "esquiarás": "esquiar",
    "esquiaré": "esquiar",
    "esquiaréis": "esquiar",
    "esquiaría": "esquiar",
    "esquiaríais": "esquiar",
    "esquiaríamos": "esquiar",
    "esquiarían": "esquiar",
    "esquiarías": "esquiar",
    "esquiascopias": "esquiascopia",
    "esquiase": "esquiar",
    "esquiaseis": "esquiar",
    "esquiasen": "esquiar",
    "esquiases": "esquiar",
    "esquiaste": "esquiar",
    "esquiasteis": "esquiar",
    "esquicios": "esquicio",
    "esquiemos": "esquiar",
    "esquifadas": "esquifada",
    "esquifazones": "esquifazón",
    "esquifes": "esquife",
    "esquifistas": "esquifista",
    "esquila": "esquilar",
    "esquilaba": "esquilar",
    "esquilabais": "esquilar",
    "esquilaban": "esquilar",
    "esquilabas": "esquilar",
    "esquilad": "esquilar",
    "esquilada": "esquilar",
    "esquiladas": "esquilar",
    "esquiladeros": "esquiladero",
    "esquilado": "esquilar",
    "esquiladora": "esquilador",
    "esquiladoras": "esquilador",
    "esquiladores": "esquilador",
    "esquilados": "esquilar",
    "esquilamos": "esquilar",
    "esquilan": "esquilar",
    "esquilando": "esquilar",
    "esquilara": "esquilar",
    "esquilarais": "esquilar",
    "esquilaran": "esquilar",
    "esquilaras": "esquilar",
    "esquilare": "esquilar",
    "esquilareis": "esquilar",
    "esquilaremos": "esquilar",
    "esquilaren": "esquilar",
    "esquilares": "esquilar",
    "esquilaron": "esquilar",
    "esquilará": "esquilar",
    "esquilarán": "esquilar",
    "esquilarás": "esquilar",
    "esquilaré": "esquilar",
    "esquilaréis": "esquilar",
    "esquilaría": "esquilar",
    "esquilaríais": "esquilar",
    "esquilaríamos": "esquilar",
    "esquilarían": "esquilar",
    "esquilarías": "esquilar",
    "esquilas": "esquilar",
    "esquilase": "esquilar",
    "esquilaseis": "esquilar",
    "esquilasen": "esquilar",
    "esquilases": "esquilar",
    "esquilaste": "esquilar",
    "esquilasteis": "esquilar",
    "esquile": "esquilar",
    "esquilemos": "esquilar",
    "esquilen": "esquilar",
    "esquileos": "esquileo",
    "esquileros": "esquilero",
    "esquiles": "esquilar",
    "esquilimosa": "esquilimoso",
    "esquilimosas": "esquilimoso",
    "esquilimosos": "esquilimoso",
    "esquilma": "esquilmar",
    "esquilmaba": "esquilmar",
    "esquilmabais": "esquilmar",
    "esquilmaban": "esquilmar",
    "esquilmabas": "esquilmar",
    "esquilmaciones": "esquilmación",
    "esquilmad": "esquilmar",
    "esquilmada": "esquilmar",
    "esquilmadas": "esquilmar",
    "esquilmado": "esquilmar",
    "esquilmados": "esquilmar",
    "esquilmamos": "esquilmar",
    "esquilman": "esquilmar",
    "esquilmando": "esquilmar",
    "esquilmara": "esquilmar",
    "esquilmarais": "esquilmar",
    "esquilmaran": "esquilmar",
    "esquilmaras": "esquilmar",
    "esquilmare": "esquilmar",
    "esquilmareis": "esquilmar",
    "esquilmaremos": "esquilmar",
    "esquilmaren": "esquilmar",
    "esquilmares": "esquilmar",
    "esquilmaron": "esquilmar",
    "esquilmará": "esquilmar",
    "esquilmarán": "esquilmar",
    "esquilmarás": "esquilmar",
    "esquilmaré": "esquilmar",
    "esquilmaréis": "esquilmar",
    "esquilmaría": "esquilmar",
    "esquilmaríais": "esquilmar",
    "esquilmaríamos": "esquilmar",
    "esquilmarían": "esquilmar",
    "esquilmarías": "esquilmar",
    "esquilmas": "esquilmar",
    "esquilmase": "esquilmar",
    "esquilmaseis": "esquilmar",
    "esquilmasen": "esquilmar",
    "esquilmases": "esquilmar",
    "esquilmaste": "esquilmar",
    "esquilmasteis": "esquilmar",
    "esquilme": "esquilmar",
    "esquilmemos": "esquilmar",
    "esquilmen": "esquilmar",
    "esquilmes": "esquilmar",
    "esquilmo": "esquilmar",
    "esquilmos": "esquilmo",
    "esquilmábamos": "esquilmar",
    "esquilmáis": "esquilmar",
    "esquilmáramos": "esquilmar",
    "esquilmáremos": "esquilmar",
    "esquilmásemos": "esquilmar",
    "esquilmé": "esquilmar",
    "esquilméis": "esquilmar",
    "esquilmó": "esquilmar",
    "esquilo": "esquilar",
    "esquilones": "esquilón",
    "esquilábamos": "esquilar",
    "esquiláis": "esquilar",
    "esquiláramos": "esquilar",
    "esquiláremos": "esquilar",
    "esquilásemos": "esquilar",
    "esquilé": "esquilar",
    "esquiléis": "esquilar",
    "esquiló": "esquilar",
    "esquimales": "esquimal",
    "esquimotajes": "esquimotaje",
    "esquina": "esquinar",
    "esquinaba": "esquinar",
    "esquinabais": "esquinar",
    "esquinaban": "esquinar",
    "esquinabas": "esquinar",
    "esquinad": "esquinar",
    "esquinada": "esquinar",
    "esquinadas": "esquinar",
    "esquinado": "esquinar",
    "esquinados": "esquinar",
    "esquinaduras": "esquinadura",
    "esquinamos": "esquinar",
    "esquinan": "esquinar",
    "esquinando": "esquinar",
    "esquinara": "esquinar",
    "esquinarais": "esquinar",
    "esquinaran": "esquinar",
    "esquinaras": "esquinar",
    "esquinare": "esquinar",
    "esquinareis": "esquinar",
    "esquinaremos": "esquinar",
    "esquinaren": "esquinar",
    "esquinares": "esquinar",
    "esquinaron": "esquinar",
    "esquinará": "esquinar",
    "esquinarán": "esquinar",
    "esquinarás": "esquinar",
    "esquinaré": "esquinar",
    "esquinaréis": "esquinar",
    "esquinaría": "esquinar",
    "esquinaríais": "esquinar",
    "esquinaríamos": "esquinar",
    "esquinarían": "esquinar",
    "esquinarías": "esquinar",
    "esquinas": "esquinar",
    "esquinase": "esquinar",
    "esquinaseis": "esquinar",
    "esquinasen": "esquinar",
    "esquinases": "esquinar",
    "esquinaste": "esquinar",
    "esquinasteis": "esquinar",
    "esquinazos": "esquinazo",
    "esquince": "esquinzar",
    "esquincemos": "esquinzar",
    "esquincen": "esquinzar",
    "esquinces": "esquinzar",
    "esquincé": "esquinzar",
    "esquincéis": "esquinzar",
    "esquine": "esquinar",
    "esquinelas": "esquinela",
    "esquinemos": "esquinar",
    "esquinen": "esquinar",
    "esquines": "esquinar",
    "esquino": "esquinar",
    "esquinza": "esquinzar",
    "esquinzaba": "esquinzar",
    "esquinzabais": "esquinzar",
    "esquinzaban": "esquinzar",
    "esquinzabas": "esquinzar",
    "esquinzad": "esquinzar",
    "esquinzada": "esquinzar",
    "esquinzadas": "esquinzar",
    "esquinzado": "esquinzar",
    "esquinzados": "esquinzar",
    "esquinzamos": "esquinzar",
    "esquinzan": "esquinzar",
    "esquinzando": "esquinzar",
    "esquinzara": "esquinzar",
    "esquinzarais": "esquinzar",
    "esquinzaran": "esquinzar",
    "esquinzaras": "esquinzar",
    "esquinzare": "esquinzar",
    "esquinzareis": "esquinzar",
    "esquinzaremos": "esquinzar",
    "esquinzaren": "esquinzar",
    "esquinzares": "esquinzar",
    "esquinzaron": "esquinzar",
    "esquinzará": "esquinzar",
    "esquinzarán": "esquinzar",
    "esquinzarás": "esquinzar",
    "esquinzaré": "esquinzar",
    "esquinzaréis": "esquinzar",
    "esquinzaría": "esquinzar",
    "esquinzaríais": "esquinzar",
    "esquinzaríamos": "esquinzar",
    "esquinzarían": "esquinzar",
    "esquinzarías": "esquinzar",
    "esquinzas": "esquinzar",
    "esquinzase": "esquinzar",
    "esquinzaseis": "esquinzar",
    "esquinzasen": "esquinzar",
    "esquinzases": "esquinzar",
    "esquinzaste": "esquinzar",
    "esquinzasteis": "esquinzar",
    "esquinzo": "esquinzar",
    "esquinzábamos": "esquinzar",
    "esquinzáis": "esquinzar",
    "esquinzáramos": "esquinzar",
    "esquinzáremos": "esquinzar",
    "esquinzásemos": "esquinzar",
    "esquinzó": "esquinzar",
    "esquinábamos": "esquinar",
    "esquináis": "esquinar",
    "esquináramos": "esquinar",
    "esquináremos": "esquinar",
    "esquinásemos": "esquinar",
    "esquiné": "esquinar",
    "esquinéis": "esquinar",
    "esquinó": "esquinar",
    "esquires": "esquire",
    "esquirlas": "esquirla",
    "esquiroles": "esquirol",
    "esquistocitos": "esquistocito",
    "esquistos": "esquisto",
    "esquistosa": "esquistoso",
    "esquistosas": "esquistoso",
    "esquistosidades": "esquistosidad",
    "esquistosos": "esquistoso",
    "esquiva": "esquivo",
    "esquivaba": "esquivar",
    "esquivabais": "esquivar",
    "esquivaban": "esquivar",
    "esquivabas": "esquivar",
    "esquivad": "esquivar",
    "esquivada": "esquivar",
    "esquivadas": "esquivar",
    "esquivado": "esquivar",
    "esquivados": "esquivar",
    "esquivamos": "esquivar",
    "esquivan": "esquivar",
    "esquivando": "esquivar",
    "esquivara": "esquivar",
    "esquivarais": "esquivar",
    "esquivaran": "esquivar",
    "esquivaras": "esquivar",
    "esquivare": "esquivar",
    "esquivareis": "esquivar",
    "esquivaremos": "esquivar",
    "esquivaren": "esquivar",
    "esquivares": "esquivar",
    "esquivaron": "esquivar",
    "esquivará": "esquivar",
    "esquivarán": "esquivar",
    "esquivarás": "esquivar",
    "esquivaré": "esquivar",
    "esquivaréis": "esquivar",
    "esquivaría": "esquivar",
    "esquivaríais": "esquivar",
    "esquivaríamos": "esquivar",
    "esquivarían": "esquivar",
    "esquivarías": "esquivar",
    "esquivas": "esquivo",
    "esquivase": "esquivar",
    "esquivaseis": "esquivar",
    "esquivasen": "esquivar",
    "esquivases": "esquivar",
    "esquivaste": "esquivar",
    "esquivasteis": "esquivar",
    "esquive": "esquivar",
    "esquiveces": "esquivez",
    "esquivemos": "esquivar",
    "esquiven": "esquivar",
    "esquives": "esquivar",
    "esquivo": "esquivar",
    "esquivos": "esquivo",
    "esquivábamos": "esquivar",
    "esquiváis": "esquivar",
    "esquiváramos": "esquivar",
    "esquiváremos": "esquivar",
    "esquivásemos": "esquivar",
    "esquivé": "esquivar",
    "esquivéis": "esquivar",
    "esquivó": "esquivar",
    "esquizada": "esquizado",
    "esquizadas": "esquizado",
    "esquizados": "esquizado",
    "esquizocarpos": "esquizocarpo",
    "esquizofasias": "esquizofasia",
    "esquizofrenias": "esquizofrenia",
    "esquizofrénica": "esquizofrénico",
    "esquizofrénicas": "esquizofrénico",
    "esquizofrénicos": "esquizofrénico",
    "esquizoides": "esquizoide",
    "esquizomicetos": "esquizomiceto",
    "esquiábamos": "esquiar",
    "esquiáis": "esquiar",
    "esquiáramos": "esquiar",
    "esquiáremos": "esquiar",
    "esquiásemos": "esquiar",
    "esquié": "esquiar",
    "esquiéis": "esquiar",
    "esquió": "esquiar",
    "esquía": "esquiar",
    "esquían": "esquiar",
    "esquías": "esquiar",
    "esquíe": "esquiar",
    "esquíen": "esquiar",
    "esquíes": "esquiar",
    "esquío": "esquiar",
    "esquís": "esquí",
    "esquíses": "esquí",
    "esta": "este",
    "estaba": "estar",
    "estabais": "estar",
    "estaban": "estar",
    "estabas": "estar",
    "estabilice": "estabilizar",
    "estabilicemos": "estabilizar",
    "estabilicen": "estabilizar",
    "estabilices": "estabilizar",
    "estabilicé": "estabilizar",
    "estabilicéis": "estabilizar",
    "estabilidades": "estabilidad",
    "estabilientes": "estabiliente",
    "estabiliza": "estabilizar",
    "estabilizaba": "estabilizar",
    "estabilizabais": "estabilizar",
    "estabilizaban": "estabilizar",
    "estabilizabas": "estabilizar",
    "estabilizaciones": "estabilización",
    "estabilizad": "estabilizar",
    "estabilizada": "estabilizar",
    "estabilizadas": "estabilizar",
    "estabilizado": "estabilizar",
    "estabilizadora": "estabilizador",
    "estabilizadoras": "estabilizador",
    "estabilizadores": "estabilizador",
    "estabilizados": "estabilizar",
    "estabilizamos": "estabilizar",
    "estabilizan": "estabilizar",
    "estabilizando": "estabilizar",
    "estabilizara": "estabilizar",
    "estabilizarais": "estabilizar",
    "estabilizaran": "estabilizar",
    "estabilizaras": "estabilizar",
    "estabilizare": "estabilizar",
    "estabilizareis": "estabilizar",
    "estabilizaremos": "estabilizar",
    "estabilizaren": "estabilizar",
    "estabilizares": "estabilizar",
    "estabilizaron": "estabilizar",
    "estabilizará": "estabilizar",
    "estabilizarán": "estabilizar",
    "estabilizarás": "estabilizar",
    "estabilizaré": "estabilizar",
    "estabilizaréis": "estabilizar",
    "estabilizaría": "estabilizar",
    "estabilizaríais": "estabilizar",
    "estabilizaríamos": "estabilizar",
    "estabilizarían": "estabilizar",
    "estabilizarías": "estabilizar",
    "estabilizas": "estabilizar",
    "estabilizase": "estabilizar",
    "estabilizaseis": "estabilizar",
    "estabilizasen": "estabilizar",
    "estabilizases": "estabilizar",
    "estabilizaste": "estabilizar",
    "estabilizasteis": "estabilizar",
    "estabilizo": "estabilizar",
    "estabilizábamos": "estabilizar",
    "estabilizáis": "estabilizar",
    "estabilizáramos": "estabilizar",
    "estabilizáremos": "estabilizar",
    "estabilizásemos": "estabilizar",
    "estabilizó": "estabilizar",
    "establece": "establecer",
    "estableced": "establecer",
    "establecedora": "establecedor",
    "establecedoras": "establecedor",
    "establecedores": "establecedor",
    "establecemos": "establecer",
    "establecen": "establecer",
    "estableceremos": "establecer",
    "establecerá": "establecer",
    "establecerán": "establecer",
    "establecerás": "establecer",
    "estableceré": "establecer",
    "estableceréis": "establecer",
    "establecería": "establecer",
    "estableceríais": "establecer",
    "estableceríamos": "establecer",
    "establecerían": "establecer",
    "establecerías": "establecer",
    "estableces": "establecer",
    "establecida": "establecer",
    "establecidas": "establecer",
    "establecido": "establecer",
    "establecidos": "establecer",
    "estableciendo": "establecer",
    "estableciera": "establecer",
    "establecierais": "establecer",
    "establecieran": "establecer",
    "establecieras": "establecer",
    "estableciere": "establecer",
    "estableciereis": "establecer",
    "establecieren": "establecer",
    "establecieres": "establecer",
    "establecieron": "establecer",
    "estableciese": "establecer",
    "establecieseis": "establecer",
    "estableciesen": "establecer",
    "establecieses": "establecer",
    "establecimientos": "establecimiento",
    "establecimos": "establecer",
    "estableciste": "establecer",
    "establecisteis": "establecer",
    "estableciéramos": "establecer",
    "estableciéremos": "establecer",
    "estableciésemos": "establecer",
    "estableció": "establecer",
    "establecéis": "establecer",
    "establecí": "establecer",
    "establecía": "establecer",
    "establecíais": "establecer",
    "establecíamos": "establecer",
    "establecían": "establecer",
    "establecías": "establecer",
    "establera": "establero",
    "estableras": "establero",
    "estableros": "establero",
    "estables": "estable",
    "establezca": "establecer",
    "establezcamos": "establecer",
    "establezcan": "establecer",
    "establezcas": "establecer",
    "establezco": "establecer",
    "establezcáis": "establecer",
    "establos": "establo",
    "estabula": "estabular",
    "estabulaba": "estabular",
    "estabulabais": "estabular",
    "estabulaban": "estabular",
    "estabulabas": "estabular",
    "estabulaciones": "estabulación",
    "estabulad": "estabular",
    "estabulada": "estabular",
    "estabuladas": "estabular",
    "estabulado": "estabular",
    "estabulados": "estabular",
    "estabulamos": "estabular",
    "estabulan": "estabular",
    "estabulando": "estabular",
    "estabulara": "estabular",
    "estabularais": "estabular",
    "estabularan": "estabular",
    "estabularas": "estabular",
    "estabulare": "estabular",
    "estabulareis": "estabular",
    "estabularemos": "estabular",
    "estabularen": "estabular",
    "estabulares": "estabular",
    "estabularon": "estabular",
    "estabulará": "estabular",
    "estabularán": "estabular",
    "estabularás": "estabular",
    "estabularé": "estabular",
    "estabularéis": "estabular",
    "estabularía": "estabular",
    "estabularíais": "estabular",
    "estabularíamos": "estabular",
    "estabularían": "estabular",
    "estabularías": "estabular",
    "estabulas": "estabular",
    "estabulase": "estabular",
    "estabulaseis": "estabular",
    "estabulasen": "estabular",
    "estabulases": "estabular",
    "estabulaste": "estabular",
    "estabulasteis": "estabular",
    "estabule": "estabular",
    "estabulemos": "estabular",
    "estabulen": "estabular",
    "estabules": "estabular",
    "estabulo": "estabular",
    "estabulábamos": "estabular",
    "estabuláis": "estabular",
    "estabuláramos": "estabular",
    "estabuláremos": "estabular",
    "estabulásemos": "estabular",
    "estabulé": "estabular",
    "estabuléis": "estabular",
    "estabuló": "estabular",
    "estaca": "estacar",
    "estacaba": "estacar",
    "estacabais": "estacar",
    "estacaban": "estacar",
    "estacabas": "estacar",
    "estacad": "estacar",
    "estacada": "estacar",
    "estacadas": "estacar",
    "estacado": "estacar",
    "estacados": "estacar",
    "estacaduras": "estacadura",
    "estacamos": "estacar",
    "estacan": "estacar",
    "estacando": "estacar",
    "estacara": "estacar",
    "estacarais": "estacar",
    "estacaran": "estacar",
    "estacaras": "estacar",
    "estacare": "estacar",
    "estacareis": "estacar",
    "estacaremos": "estacar",
    "estacaren": "estacar",
    "estacares": "estacar",
    "estacaron": "estacar",
    "estacará": "estacar",
    "estacarán": "estacar",
    "estacarás": "estacar",
    "estacaré": "estacar",
    "estacaréis": "estacar",
    "estacaría": "estacar",
    "estacaríais": "estacar",
    "estacaríamos": "estacar",
    "estacarían": "estacar",
    "estacarías": "estacar",
    "estacas": "estacar",
    "estacase": "estacar",
    "estacaseis": "estacar",
    "estacasen": "estacar",
    "estacases": "estacar",
    "estacaste": "estacar",
    "estacasteis": "estacar",
    "estacazos": "estacazo",
    "estachas": "estacha",
    "estaciona": "estacionar",
    "estacionaba": "estacionar",
    "estacionabais": "estacionar",
    "estacionaban": "estacionar",
    "estacionabas": "estacionar",
    "estacionad": "estacionar",
    "estacionada": "estacionar",
    "estacionadas": "estacionar",
    "estacionado": "estacionar",
    "estacionados": "estacionar",
    "estacionales": "estacional",
    "estacionalidades": "estacionalidad",
    "estacionamientos": "estacionamiento",
    "estacionamos": "estacionar",
    "estacionan": "estacionar",
    "estacionando": "estacionar",
    "estacionara": "estacionar",
    "estacionarais": "estacionar",
    "estacionaran": "estacionar",
    "estacionaras": "estacionar",
    "estacionare": "estacionar",
    "estacionareis": "estacionar",
    "estacionaremos": "estacionar",
    "estacionaren": "estacionar",
    "estacionares": "estacionar",
    "estacionaria": "estacionario",
    "estacionarias": "estacionario",
    "estacionarios": "estacionario",
    "estacionaron": "estacionar",
    "estacionará": "estacionar",
    "estacionarán": "estacionar",
    "estacionarás": "estacionar",
    "estacionaré": "estacionar",
    "estacionaréis": "estacionar",
    "estacionaría": "estacionar",
    "estacionaríais": "estacionar",
    "estacionaríamos": "estacionar",
    "estacionarían": "estacionar",
    "estacionarías": "estacionar",
    "estacionas": "estacionar",
    "estacionase": "estacionar",
    "estacionaseis": "estacionar",
    "estacionasen": "estacionar",
    "estacionases": "estacionar",
    "estacionaste": "estacionar",
    "estacionasteis": "estacionar",
    "estacione": "estacionar",
    "estacionemos": "estacionar",
    "estacionen": "estacionar",
    "estacionera": "estacionero",
    "estacioneras": "estacionero",
    "estacioneros": "estacionero",
    "estaciones": "estacionar",
    "estaciono": "estacionar",
    "estacionábamos": "estacionar",
    "estacionáis": "estacionar",
    "estacionáramos": "estacionar",
    "estacionáremos": "estacionar",
    "estacionásemos": "estacionar",
    "estacioné": "estacionar",
    "estacionéis": "estacionar",
    "estacionó": "estacionar",
    "estaco": "estacar",
    "estacones": "estacón",
    "estactes": "estacte",
    "estacábamos": "estacar",
    "estacáis": "estacar",
    "estacáramos": "estacar",
    "estacáremos": "estacar",
    "estacásemos": "estacar",
    "estacó": "estacar",
    "estad": "estar",
    "estada": "estar",
    "estadales": "estadal",
    "estadas": "estar",
    "estadillos": "estadillo",
    "estadimetrías": "estadimetría",
    "estadimétrica": "estadimétrico",
    "estadimétricas": "estadimétrico",
    "estadimétricos": "estadimétrico",
    "estadios": "estadio",
    "estadistas": "estadista",
    "estadiza": "estadizo",
    "estadizas": "estadizo",
    "estadizos": "estadizo",
    "estado": "estar",
    "estados": "estar",
    "estadounidenses": "estadounidense",
    "estadías": "estadía",
    "estadímetros": "estadímetro",
    "estadística": "estadístico",
    "estadísticas": "estadístico",
    "estadísticos": "estadístico",
    "estafa": "estafar",
    "estafaba": "estafar",
    "estafabais": "estafar",
    "estafaban": "estafar",
    "estafabas": "estafar",
    "estafad": "estafar",
    "estafada": "estafar",
    "estafadas": "estafar",
    "estafado": "estafar",
    "estafadora": "estafador",
    "estafadoras": "estafador",
    "estafadores": "estafador",
    "estafados": "estafar",
    "estafamos": "estafar",
    "estafan": "estafar",
    "estafando": "estafar",
    "estafara": "estafar",
    "estafarais": "estafar",
    "estafaran": "estafar",
    "estafaras": "estafar",
    "estafare": "estafar",
    "estafareis": "estafar",
    "estafaremos": "estafar",
    "estafaren": "estafar",
    "estafares": "estafar",
    "estafaron": "estafar",
    "estafará": "estafar",
    "estafarán": "estafar",
    "estafarás": "estafar",
    "estafaré": "estafar",
    "estafaréis": "estafar",
    "estafaría": "estafar",
    "estafaríais": "estafar",
    "estafaríamos": "estafar",
    "estafarían": "estafar",
    "estafarías": "estafar",
    "estafas": "estafar",
    "estafase": "estafar",
    "estafaseis": "estafar",
    "estafasen": "estafar",
    "estafases": "estafar",
    "estafaste": "estafar",
    "estafasteis": "estafar",
    "estafe": "estafar",
    "estafemos": "estafar",
    "estafen": "estafar",
    "estafermos": "estafermo",
    "estafes": "estafar",
    "estafetas": "estafeta",
    "estafetera": "estafetero",
    "estafeteras": "estafetero",
    "estafeteros": "estafetero",
    "estafetiles": "estafetil",
    "estafilococias": "estafilococia",
    "estafilococos": "estafilococo",
    "estafilomas": "estafiloma",
    "estafilínidos": "estafilínido",
    "estafo": "estafar",
    "estafábamos": "estafar",
    "estafáis": "estafar",
    "estafáramos": "estafar",
    "estafáremos": "estafar",
    "estafásemos": "estafar",
    "estafé": "estafar",
    "estaféis": "estafar",
    "estafó": "estafar",
    "estajanovismos": "estajanovismo",
    "estajanovistas": "estajanovista",
    "estajera": "estajero",
    "estajeras": "estajero",
    "estajeros": "estajero",
    "estajistas": "estajista",
    "estalaciones": "estalación",
    "estalactitas": "estalactita",
    "estalagmitas": "estalagmita",
    "estalagmometrías": "estalagmometría",
    "estalagmómetros": "estalagmómetro",
    "estalajes": "estalaje",
    "estalinismos": "estalinismo",
    "estalinistas": "estalinista",
    "estalinizaciones": "estalinización",
    "estalla": "estallar",
    "estallaba": "estallar",
    "estallabais": "estallar",
    "estallaban": "estallar",
    "estallabas": "estallar",
    "estallad": "estallar",
    "estallada": "estallar",
    "estalladas": "estallar",
    "estallado": "estallar",
    "estallados": "estallar",
    "estallamos": "estallar",
    "estallan": "estallar",
    "estallando": "estallar",
    "estallara": "estallar",
    "estallarais": "estallar",
    "estallaran": "estallar",
    "estallaras": "estallar",
    "estallare": "estallar",
    "estallareis": "estallar",
    "estallaremos": "estallar",
    "estallaren": "estallar",
    "estallares": "estallar",
    "estallaron": "estallar",
    "estallará": "estallar",
    "estallarán": "estallar",
    "estallarás": "estallar",
    "estallaré": "estallar",
    "estallaréis": "estallar",
    "estallaría": "estallar",
    "estallaríais": "estallar",
    "estallaríamos": "estallar",
    "estallarían": "estallar",
    "estallarías": "estallar",
    "estallas": "estallar",
    "estallase": "estallar",
    "estallaseis": "estallar",
    "estallasen": "estallar",
    "estallases": "estallar",
    "estallaste": "estallar",
    "estallasteis": "estallar",
    "estalle": "estallar",
    "estallemos": "estallar",
    "estallen": "estallar",
    "estalles": "estallar",
    "estallidos": "estallido",
    "estallo": "estallar",
    "estallábamos": "estallar",
    "estalláis": "estallar",
    "estalláramos": "estallar",
    "estalláremos": "estallar",
    "estallásemos": "estallar",
    "estallé": "estallar",
    "estalléis": "estallar",
    "estalló": "estallar",
    "estambra": "estambrar",
    "estambraba": "estambrar",
    "estambrabais": "estambrar",
    "estambraban": "estambrar",
    "estambrabas": "estambrar",
    "estambrad": "estambrar",
    "estambrada": "estambrar",
    "estambradas": "estambrar",
    "estambrado": "estambrar",
    "estambrados": "estambrar",
    "estambramos": "estambrar",
    "estambran": "estambrar",
    "estambrando": "estambrar",
    "estambrara": "estambrar",
    "estambrarais": "estambrar",
    "estambraran": "estambrar",
    "estambraras": "estambrar",
    "estambrare": "estambrar",
    "estambrareis": "estambrar",
    "estambraremos": "estambrar",
    "estambraren": "estambrar",
    "estambrares": "estambrar",
    "estambraron": "estambrar",
    "estambrará": "estambrar",
    "estambrarán": "estambrar",
    "estambrarás": "estambrar",
    "estambraré": "estambrar",
    "estambraréis": "estambrar",
    "estambraría": "estambrar",
    "estambraríais": "estambrar",
    "estambraríamos": "estambrar",
    "estambrarían": "estambrar",
    "estambrarías": "estambrar",
    "estambras": "estambrar",
    "estambrase": "estambrar",
    "estambraseis": "estambrar",
    "estambrasen": "estambrar",
    "estambrases": "estambrar",
    "estambraste": "estambrar",
    "estambrasteis": "estambrar",
    "estambre": "estambrar",
    "estambremos": "estambrar",
    "estambren": "estambrar",
    "estambres": "estambre",
    "estambro": "estambrar",
    "estambrábamos": "estambrar",
    "estambráis": "estambrar",
    "estambráramos": "estambrar",
    "estambráremos": "estambrar",
    "estambrásemos": "estambrar",
    "estambré": "estambrar",
    "estambréis": "estambrar",
    "estambró": "estambrar",
    "estamentales": "estamental",
    "estamentos": "estamento",
    "estameñas": "estameña",
    "estaminodios": "estaminodio",
    "estamos": "estar",
    "estampa": "estampar",
    "estampaba": "estampar",
    "estampabais": "estampar",
    "estampaban": "estampar",
    "estampabas": "estampar",
    "estampaciones": "estampación",
    "estampad": "estampar",
    "estampada": "estampar",
    "estampadas": "estampar",
    "estampado": "estampar",
    "estampadora": "estampador",
    "estampadoras": "estampador",
    "estampadores": "estampador",
    "estampados": "estampar",
    "estampamos": "estampar",
    "estampan": "estampar",
    "estampando": "estampar",
    "estampara": "estampar",
    "estamparais": "estampar",
    "estamparan": "estampar",
    "estamparas": "estampar",
    "estampare": "estampar",
    "estampareis": "estampar",
    "estamparemos": "estampar",
    "estamparen": "estampar",
    "estampares": "estampar",
    "estamparon": "estampar",
    "estampará": "estampar",
    "estamparán": "estampar",
    "estamparás": "estampar",
    "estamparé": "estampar",
    "estamparéis": "estampar",
    "estamparía": "estampar",
    "estamparíais": "estampar",
    "estamparíamos": "estampar",
    "estamparían": "estampar",
    "estamparías": "estampar",
    "estampas": "estampar",
    "estampase": "estampar",
    "estampaseis": "estampar",
    "estampasen": "estampar",
    "estampases": "estampar",
    "estampaste": "estampar",
    "estampasteis": "estampar",
    "estampe": "estampar",
    "estampemos": "estampar",
    "estampen": "estampar",
    "estampera": "estampero",
    "estamperas": "estampero",
    "estamperos": "estampero",
    "estamperías": "estampería",
    "estampes": "estampar",
    "estampidas": "estampida",
    "estampidos": "estampido",
    "estampilla": "estampillar",
    "estampillaba": "estampillar",
    "estampillabais": "estampillar",
    "estampillaban": "estampillar",
    "estampillabas": "estampillar",
    "estampillad": "estampillar",
    "estampillada": "estampillar",
    "estampilladas": "estampillar",
    "estampillado": "estampillar",
    "estampilladoras": "estampilladora",
    "estampillados": "estampillar",
    "estampillamos": "estampillar",
    "estampillan": "estampillar",
    "estampillando": "estampillar",
    "estampillara": "estampillar",
    "estampillarais": "estampillar",
    "estampillaran": "estampillar",
    "estampillaras": "estampillar",
    "estampillare": "estampillar",
    "estampillareis": "estampillar",
    "estampillaremos": "estampillar",
    "estampillaren": "estampillar",
    "estampillares": "estampillar",
    "estampillaron": "estampillar",
    "estampillará": "estampillar",
    "estampillarán": "estampillar",
    "estampillarás": "estampillar",
    "estampillaré": "estampillar",
    "estampillaréis": "estampillar",
    "estampillaría": "estampillar",
    "estampillaríais": "estampillar",
    "estampillaríamos": "estampillar",
    "estampillarían": "estampillar",
    "estampillarías": "estampillar",
    "estampillas": "estampillar",
    "estampillase": "estampillar",
    "estampillaseis": "estampillar",
    "estampillasen": "estampillar",
    "estampillases": "estampillar",
    "estampillaste": "estampillar",
    "estampillasteis": "estampillar",
    "estampille": "estampillar",
    "estampillemos": "estampillar",
    "estampillen": "estampillar",
    "estampilles": "estampillar",
    "estampillo": "estampillar",
    "estampillábamos": "estampillar",
    "estampilláis": "estampillar",
    "estampilláramos": "estampillar",
    "estampilláremos": "estampillar",
    "estampillásemos": "estampillar",
    "estampillé": "estampillar",
    "estampilléis": "estampillar",
    "estampilló": "estampillar",
    "estampo": "estampar",
    "estampábamos": "estampar",
    "estampáis": "estampar",
    "estampáramos": "estampar",
    "estampáremos": "estampar",
    "estampásemos": "estampar",
    "estampé": "estampar",
    "estampéis": "estampar",
    "estampó": "estampar",
    "estamínea": "estamíneo",
    "estamíneas": "estamíneo",
    "estamíneos": "estamíneo",
    "estanca": "estanco",
    "estancaba": "estancar",
    "estancabais": "estancar",
    "estancaban": "estancar",
    "estancabas": "estancar",
    "estancaciones": "estancación",
    "estancad": "estancar",
    "estancada": "estancar",
    "estancadas": "estancar",
    "estancado": "estancar",
    "estancados": "estancar",
    "estancamientos": "estancamiento",
    "estancamos": "estancar",
    "estancan": "estancar",
    "estancando": "estancar",
    "estancara": "estancar",
    "estancarais": "estancar",
    "estancaran": "estancar",
    "estancaras": "estancar",
    "estancare": "estancar",
    "estancareis": "estancar",
    "estancaremos": "estancar",
    "estancaren": "estancar",
    "estancares": "estancar",
    "estancaron": "estancar",
    "estancará": "estancar",
    "estancarán": "estancar",
    "estancarás": "estancar",
    "estancaré": "estancar",
    "estancaréis": "estancar",
    "estancaría": "estancar",
    "estancaríais": "estancar",
    "estancaríamos": "estancar",
    "estancarían": "estancar",
    "estancarías": "estancar",
    "estancas": "estanco",
    "estancase": "estancar",
    "estancaseis": "estancar",
    "estancasen": "estancar",
    "estancases": "estancar",
    "estancaste": "estancar",
    "estancasteis": "estancar",
    "estancias": "estancia",
    "estanciera": "estanciero",
    "estancieras": "estanciero",
    "estancieros": "estanciero",
    "estanco": "estancar",
    "estancos": "estanco",
    "estancábamos": "estancar",
    "estancáis": "estancar",
    "estancáramos": "estancar",
    "estancáremos": "estancar",
    "estancásemos": "estancar",
    "estancó": "estancar",
    "estandardice": "estandardizar",
    "estandardicemos": "estandardizar",
    "estandardicen": "estandardizar",
    "estandardices": "estandardizar",
    "estandardicé": "estandardizar",
    "estandardicéis": "estandardizar",
    "estandardiza": "estandardizar",
    "estandardizaba": "estandardizar",
    "estandardizabais": "estandardizar",
    "estandardizaban": "estandardizar",
    "estandardizabas": "estandardizar",
    "estandardizad": "estandardizar",
    "estandardizada": "estandardizar",
    "estandardizadas": "estandardizar",
    "estandardizado": "estandardizar",
    "estandardizados": "estandardizar",
    "estandardizamos": "estandardizar",
    "estandardizan": "estandardizar",
    "estandardizando": "estandardizar",
    "estandardizara": "estandardizar",
    "estandardizarais": "estandardizar",
    "estandardizaran": "estandardizar",
    "estandardizaras": "estandardizar",
    "estandardizare": "estandardizar",
    "estandardizareis": "estandardizar",
    "estandardizaremos": "estandardizar",
    "estandardizaren": "estandardizar",
    "estandardizares": "estandardizar",
    "estandardizaron": "estandardizar",
    "estandardizará": "estandardizar",
    "estandardizarán": "estandardizar",
    "estandardizarás": "estandardizar",
    "estandardizaré": "estandardizar",
    "estandardizaréis": "estandardizar",
    "estandardizaría": "estandardizar",
    "estandardizaríais": "estandardizar",
    "estandardizaríamos": "estandardizar",
    "estandardizarían": "estandardizar",
    "estandardizarías": "estandardizar",
    "estandardizas": "estandardizar",
    "estandardizase": "estandardizar",
    "estandardizaseis": "estandardizar",
    "estandardizasen": "estandardizar",
    "estandardizases": "estandardizar",
    "estandardizaste": "estandardizar",
    "estandardizasteis": "estandardizar",
    "estandardizo": "estandardizar",
    "estandardizábamos": "estandardizar",
    "estandardizáis": "estandardizar",
    "estandardizáramos": "estandardizar",
    "estandardizáremos": "estandardizar",
    "estandardizásemos": "estandardizar",
    "estandardizó": "estandardizar",
    "estandarice": "estandarizar",
    "estandaricemos": "estandarizar",
    "estandaricen": "estandarizar",
    "estandarices": "estandarizar",
    "estandaricé": "estandarizar",
    "estandaricéis": "estandarizar",
    "estandariza": "estandarizar",
    "estandarizaba": "estandarizar",
    "estandarizabais": "estandarizar",
    "estandarizaban": "estandarizar",
    "estandarizabas": "estandarizar",
    "estandarizaciones": "estandarización",
    "estandarizad": "estandarizar",
    "estandarizada": "estandarizar",
    "estandarizadas": "estandarizar",
    "estandarizado": "estandarizar",
    "estandarizados": "estandarizar",
    "estandarizamos": "estandarizar",
    "estandarizan": "estandarizar",
    "estandarizando": "estandarizar",
    "estandarizara": "estandarizar",
    "estandarizarais": "estandarizar",
    "estandarizaran": "estandarizar",
    "estandarizaras": "estandarizar",
    "estandarizare": "estandarizar",
    "estandarizareis": "estandarizar",
    "estandarizaremos": "estandarizar",
    "estandarizaren": "estandarizar",
    "estandarizares": "estandarizar",
    "estandarizaron": "estandarizar",
    "estandarizará": "estandarizar",
    "estandarizarán": "estandarizar",
    "estandarizarás": "estandarizar",
    "estandarizaré": "estandarizar",
    "estandarizaréis": "estandarizar",
    "estandarizaría": "estandarizar",
    "estandarizaríais": "estandarizar",
    "estandarizaríamos": "estandarizar",
    "estandarizarían": "estandarizar",
    "estandarizarías": "estandarizar",
    "estandarizas": "estandarizar",
    "estandarizase": "estandarizar",
    "estandarizaseis": "estandarizar",
    "estandarizasen": "estandarizar",
    "estandarizases": "estandarizar",
    "estandarizaste": "estandarizar",
    "estandarizasteis": "estandarizar",
    "estandarizo": "estandarizar",
    "estandarizábamos": "estandarizar",
    "estandarizáis": "estandarizar",
    "estandarizáramos": "estandarizar",
    "estandarizáremos": "estandarizar",
    "estandarizásemos": "estandarizar",
    "estandarizó": "estandarizar",
    "estandartes": "estandarte",
    "estando": "estar",
    "estanflaciones": "estanflación",
    "estangurrias": "estangurria",
    "estannatos": "estannato",
    "estanninas": "estannina",
    "estannitas": "estannita",
    "estannosa": "estannoso",
    "estannosas": "estannoso",
    "estannosos": "estannoso",
    "estannífera": "estannífero",
    "estanníferas": "estannífero",
    "estanníferos": "estannífero",
    "estanque": "estancar",
    "estanqueidades": "estanqueidad",
    "estanquemos": "estancar",
    "estanquen": "estancar",
    "estanquera": "estanquero",
    "estanqueras": "estanquero",
    "estanqueros": "estanquero",
    "estanques": "estanque",
    "estanquidades": "estanquidad",
    "estanqué": "estancar",
    "estanquéis": "estancar",
    "estantales": "estantal",
    "estanterías": "estantería",
    "estantes": "estante",
    "estantiguas": "estantigua",
    "estantía": "estantío",
    "estantías": "estantío",
    "estantíos": "estantío",
    "estapedectomías": "estapedectomía",
    "estapediales": "estapedial",
    "estapediotomías": "estapediotomía",
    "estapediovestibulares": "estapediovestibular",
    "estaque": "estacar",
    "estaqueadas": "estaqueada",
    "estaquemos": "estacar",
    "estaquen": "estacar",
    "estaques": "estacar",
    "estaquilla": "estaquillar",
    "estaquillaba": "estaquillar",
    "estaquillabais": "estaquillar",
    "estaquillaban": "estaquillar",
    "estaquillabas": "estaquillar",
    "estaquillad": "estaquillar",
    "estaquillada": "estaquillar",
    "estaquilladas": "estaquillar",
    "estaquillado": "estaquillar",
    "estaquilladores": "estaquillador",
    "estaquillados": "estaquillar",
    "estaquillamos": "estaquillar",
    "estaquillan": "estaquillar",
    "estaquillando": "estaquillar",
    "estaquillara": "estaquillar",
    "estaquillarais": "estaquillar",
    "estaquillaran": "estaquillar",
    "estaquillaras": "estaquillar",
    "estaquillare": "estaquillar",
    "estaquillareis": "estaquillar",
    "estaquillaremos": "estaquillar",
    "estaquillaren": "estaquillar",
    "estaquillares": "estaquillar",
    "estaquillaron": "estaquillar",
    "estaquillará": "estaquillar",
    "estaquillarán": "estaquillar",
    "estaquillarás": "estaquillar",
    "estaquillaré": "estaquillar",
    "estaquillaréis": "estaquillar",
    "estaquillaría": "estaquillar",
    "estaquillaríais": "estaquillar",
    "estaquillaríamos": "estaquillar",
    "estaquillarían": "estaquillar",
    "estaquillarías": "estaquillar",
    "estaquillas": "estaquillar",
    "estaquillase": "estaquillar",
    "estaquillaseis": "estaquillar",
    "estaquillasen": "estaquillar",
    "estaquillases": "estaquillar",
    "estaquillaste": "estaquillar",
    "estaquillasteis": "estaquillar",
    "estaquille": "estaquillar",
    "estaquillemos": "estaquillar",
    "estaquillen": "estaquillar",
    "estaquilles": "estaquillar",
    "estaquillo": "estaquillar",
    "estaquillábamos": "estaquillar",
    "estaquilláis": "estaquillar",
    "estaquilláramos": "estaquillar",
    "estaquilláremos": "estaquillar",
    "estaquillásemos": "estaquillar",
    "estaquillé": "estaquillar",
    "estaquilléis": "estaquillar",
    "estaquilló": "estaquillar",
    "estaqué": "estacar",
    "estaquéis": "estacar",
    "estarce": "estarcir",
    "estarcen": "estarcir",
    "estarces": "estarcir",
    "estarcid": "estarcir",
    "estarcida": "estarcir",
    "estarcidas": "estarcir",
    "estarcido": "estarcir",
    "estarcidora": "estarcidor",
    "estarcidoras": "estarcidor",
    "estarcidores": "estarcidor",
    "estarcidos": "estarcir",
    "estarciendo": "estarcir",
    "estarciera": "estarcir",
    "estarcierais": "estarcir",
    "estarcieran": "estarcir",
    "estarcieras": "estarcir",
    "estarciere": "estarcir",
    "estarciereis": "estarcir",
    "estarcieren": "estarcir",
    "estarcieres": "estarcir",
    "estarcieron": "estarcir",
    "estarciese": "estarcir",
    "estarcieseis": "estarcir",
    "estarciesen": "estarcir",
    "estarcieses": "estarcir",
    "estarcimos": "estarcir",
    "estarciremos": "estarcir",
    "estarcirá": "estarcir",
    "estarcirán": "estarcir",
    "estarcirás": "estarcir",
    "estarciré": "estarcir",
    "estarciréis": "estarcir",
    "estarciría": "estarcir",
    "estarciríais": "estarcir",
    "estarciríamos": "estarcir",
    "estarcirían": "estarcir",
    "estarcirías": "estarcir",
    "estarciste": "estarcir",
    "estarcisteis": "estarcir",
    "estarciéramos": "estarcir",
    "estarciéremos": "estarcir",
    "estarciésemos": "estarcir",
    "estarció": "estarcir",
    "estarcí": "estarcir",
    "estarcía": "estarcir",
    "estarcíais": "estarcir",
    "estarcíamos": "estarcir",
    "estarcían": "estarcir",
    "estarcías": "estarcir",
    "estarcís": "estarcir",
    "estaremos": "estar",
    "estarza": "estarcir",
    "estarzamos": "estarcir",
    "estarzan": "estarcir",
    "estarzas": "estarcir",
    "estarzo": "estarcir",
    "estarzáis": "estarcir",
    "estará": "estar",
    "estarán": "estar",
    "estarás": "estar",
    "estaré": "estar",
    "estaréis": "estar",
    "estaría": "estar",
    "estaríais": "estar",
    "estaríamos": "estar",
    "estarían": "estar",
    "estarías": "estar",
    "estas": "este",
    "estatales": "estatal",
    "estatalismos": "estatalismo",
    "estateras": "estatera",
    "estatificaciones": "estatificación",
    "estatismos": "estatismo",
    "estativa": "estativo",
    "estativas": "estativo",
    "estativos": "estativo",
    "estatizaciones": "estatización",
    "estatizada": "estatizado",
    "estatizadas": "estatizado",
    "estatizados": "estatizado",
    "estatoacústica": "estatoacústico",
    "estatoacústicas": "estatoacústico",
    "estatoacústicos": "estatoacústico",
    "estatocistos": "estatocisto",
    "estatolitos": "estatolito",
    "estatores": "estator",
    "estatorreactores": "estatorreactor",
    "estatoscopios": "estatoscopio",
    "estatuaria": "estatuario",
    "estatuarias": "estatuario",
    "estatuarios": "estatuario",
    "estatuas": "estatua",
    "estatuderatos": "estatuderato",
    "estatuid": "estatuir",
    "estatuida": "estatuir",
    "estatuidas": "estatuir",
    "estatuido": "estatuir",
    "estatuidos": "estatuir",
    "estatuillas": "estatuilla",
    "estatuimos": "estatuir",
    "estatuiremos": "estatuir",
    "estatuirá": "estatuir",
    "estatuirán": "estatuir",
    "estatuirás": "estatuir",
    "estatuiré": "estatuir",
    "estatuiréis": "estatuir",
    "estatuiría": "estatuir",
    "estatuiríais": "estatuir",
    "estatuiríamos": "estatuir",
    "estatuirían": "estatuir",
    "estatuirías": "estatuir",
    "estatuiste": "estatuir",
    "estatuisteis": "estatuir",
    "estaturas": "estatura",
    "estatutaria": "estatutario",
    "estatutarias": "estatutario",
    "estatutarios": "estatutario",
    "estatutos": "estatuto",
    "estatuya": "estatuir",
    "estatuyamos": "estatuir",
    "estatuyan": "estatuir",
    "estatuyas": "estatuir",
    "estatuye": "estatuir",
    "estatuyen": "estatuir",
    "estatuyendo": "estatuir",
    "estatuyera": "estatuir",
    "estatuyerais": "estatuir",
    "estatuyeran": "estatuir",
    "estatuyeras": "estatuir",
    "estatuyere": "estatuir",
    "estatuyereis": "estatuir",
    "estatuyeren": "estatuir",
    "estatuyeres": "estatuir",
    "estatuyeron": "estatuir",
    "estatuyes": "estatuir",
    "estatuyese": "estatuir",
    "estatuyeseis": "estatuir",
    "estatuyesen": "estatuir",
    "estatuyeses": "estatuir",
    "estatuyo": "estatuir",
    "estatuyáis": "estatuir",
    "estatuyéramos": "estatuir",
    "estatuyéremos": "estatuir",
    "estatuyésemos": "estatuir",
    "estatuyó": "estatuir",
    "estatuí": "estatuir",
    "estatuía": "estatuir",
    "estatuíais": "estatuir",
    "estatuíamos": "estatuir",
    "estatuían": "estatuir",
    "estatuías": "estatuir",
    "estatuís": "estatuir",
    "estatórica": "estatórico",
    "estatóricas": "estatórico",
    "estatóricos": "estatórico",
    "estatúderes": "estatúder",
    "estaurolitas": "estaurolita",
    "estauromedusas": "estauromedusa",
    "estayes": "estay",
    "estays": "estay",
    "estaña": "estañar",
    "estañaba": "estañar",
    "estañabais": "estañar",
    "estañaban": "estañar",
    "estañabas": "estañar",
    "estañad": "estañar",
    "estañada": "estañar",
    "estañadas": "estañar",
    "estañado": "estañar",
    "estañadora": "estañador",
    "estañadoras": "estañador",
    "estañadores": "estañador",
    "estañados": "estañar",
    "estañaduras": "estañadura",
    "estañamos": "estañar",
    "estañan": "estañar",
    "estañando": "estañar",
    "estañara": "estañar",
    "estañarais": "estañar",
    "estañaran": "estañar",
    "estañaras": "estañar",
    "estañare": "estañar",
    "estañareis": "estañar",
    "estañaremos": "estañar",
    "estañaren": "estañar",
    "estañares": "estañar",
    "estañaron": "estañar",
    "estañará": "estañar",
    "estañarán": "estañar",
    "estañarás": "estañar",
    "estañaré": "estañar",
    "estañaréis": "estañar",
    "estañaría": "estañar",
    "estañaríais": "estañar",
    "estañaríamos": "estañar",
    "estañarían": "estañar",
    "estañarías": "estañar",
    "estañas": "estañar",
    "estañase": "estañar",
    "estañaseis": "estañar",
    "estañasen": "estañar",
    "estañases": "estañar",
    "estañaste": "estañar",
    "estañasteis": "estañar",
    "estañe": "estañar",
    "estañemos": "estañar",
    "estañen": "estañar",
    "estañera": "estañero",
    "estañeras": "estañero",
    "estañeros": "estañero",
    "estañes": "estañar",
    "estaño": "estañar",
    "estaños": "estaño",
    "estañábamos": "estañar",
    "estañáis": "estañar",
    "estañáramos": "estañar",
    "estañáremos": "estañar",
    "estañásemos": "estañar",
    "estañé": "estañar",
    "estañéis": "estañar",
    "estañó": "estañar",
    "estearatos": "estearato",
    "estearinas": "estearina",
    "estearílica": "estearílico",
    "estearílicas": "estearílico",
    "estearílicos": "estearílico",
    "esteatitas": "esteatita",
    "esteatorreas": "esteatorrea",
    "estebas": "esteba",
    "estece": "estezar",
    "estecemos": "estezar",
    "estecen": "estezar",
    "esteces": "estezar",
    "estecé": "estezar",
    "estecéis": "estezar",
    "estefanitas": "estefanita",
    "estelada": "estelado",
    "esteladas": "estelado",
    "estelados": "estelado",
    "estelares": "estelar",
    "estelas": "estela",
    "esteliformes": "esteliforme",
    "estelionatos": "estelionato",
    "esteliones": "estelión",
    "estelitas": "estelita",
    "estelífera": "estelífero",
    "estelíferas": "estelífero",
    "estelíferos": "estelífero",
    "estelómetros": "estelómetro",
    "estemos": "estar",
    "estemples": "estemple",
    "estenoda": "estenodo",
    "estenodas": "estenodo",
    "estenodos": "estenodo",
    "estenografiaba": "estenografiar",
    "estenografiabais": "estenografiar",
    "estenografiaban": "estenografiar",
    "estenografiabas": "estenografiar",
    "estenografiad": "estenografiar",
    "estenografiada": "estenografiar",
    "estenografiadas": "estenografiar",
    "estenografiado": "estenografiar",
    "estenografiados": "estenografiar",
    "estenografiamos": "estenografiar",
    "estenografiando": "estenografiar",
    "estenografiara": "estenografiar",
    "estenografiarais": "estenografiar",
    "estenografiaran": "estenografiar",
    "estenografiaras": "estenografiar",
    "estenografiare": "estenografiar",
    "estenografiareis": "estenografiar",
    "estenografiaremos": "estenografiar",
    "estenografiaren": "estenografiar",
    "estenografiares": "estenografiar",
    "estenografiaron": "estenografiar",
    "estenografiará": "estenografiar",
    "estenografiarán": "estenografiar",
    "estenografiarás": "estenografiar",
    "estenografiaré": "estenografiar",
    "estenografiaréis": "estenografiar",
    "estenografiaría": "estenografiar",
    "estenografiaríais": "estenografiar",
    "estenografiaríamos": "estenografiar",
    "estenografiarían": "estenografiar",
    "estenografiarías": "estenografiar",
    "estenografiase": "estenografiar",
    "estenografiaseis": "estenografiar",
    "estenografiasen": "estenografiar",
    "estenografiases": "estenografiar",
    "estenografiaste": "estenografiar",
    "estenografiasteis": "estenografiar",
    "estenografiemos": "estenografiar",
    "estenografiábamos": "estenografiar",
    "estenografiáis": "estenografiar",
    "estenografiáramos": "estenografiar",
    "estenografiáremos": "estenografiar",
    "estenografiásemos": "estenografiar",
    "estenografié": "estenografiar",
    "estenografiéis": "estenografiar",
    "estenografió": "estenografiar",
    "estenografía": "estenografiar",
    "estenografían": "estenografiar",
    "estenografías": "estenografiar",
    "estenografíe": "estenografiar",
    "estenografíen": "estenografiar",
    "estenografíes": "estenografiar",
    "estenografío": "estenografiar",
    "estenogramas": "estenograma",
    "estenográfica": "estenográfico",
    "estenográficas": "estenográfico",
    "estenográficos": "estenográfico",
    "estenométopes": "estenométope",
    "estenopeica": "estenopeico",
    "estenopeicas": "estenopeico",
    "estenopeicos": "estenopeico",
    "estenordestes": "estenordeste",
    "estenorestes": "estenoreste",
    "estenotipias": "estenotipia",
    "estenotipistas": "estenotipista",
    "estenotípica": "estenotípico",
    "estenotípicas": "estenotípico",
    "estenotípicos": "estenotípico",
    "estentórea": "estentóreo",
    "estentóreas": "estentóreo",
    "estentóreos": "estentóreo",
    "estenógrafa": "estenógrafo",
    "estenógrafas": "estenógrafo",
    "estenógrafos": "estenógrafo",
    "esteorradianes": "esteorradián",
    "esteparia": "estepario",
    "esteparias": "estepario",
    "esteparios": "estepario",
    "estepas": "estepa",
    "estepillas": "estepilla",
    "esteponeros": "esteponero",
    "estequiometrías": "estequiometría",
    "estequiométrica": "estequiométrico",
    "estequiométricas": "estequiométrico",
    "estequiométricos": "estequiométrico",
    "estera": "esterar",
    "esteraba": "esterar",
    "esterabais": "esterar",
    "esteraban": "esterar",
    "esterabas": "esterar",
    "esterad": "esterar",
    "esterada": "esterar",
    "esteradas": "esterar",
    "esterado": "esterar",
    "esterados": "esterar",
    "esteramos": "esterar",
    "esteran": "esterar",
    "esterando": "esterar",
    "esterara": "esterar",
    "esterarais": "esterar",
    "esteraran": "esterar",
    "esteraras": "esterar",
    "esterare": "esterar",
    "esterareis": "esterar",
    "esteraremos": "esterar",
    "esteraren": "esterar",
    "esterares": "esterar",
    "esteraron": "esterar",
    "esterará": "esterar",
    "esterarán": "esterar",
    "esterarás": "esterar",
    "esteraré": "esterar",
    "esteraréis": "esterar",
    "esteraría": "esterar",
    "esteraríais": "esterar",
    "esteraríamos": "esterar",
    "esterarían": "esterar",
    "esterarías": "esterar",
    "esteras": "esterar",
    "esterasas": "esterasa",
    "esterase": "esterar",
    "esteraseis": "esterar",
    "esterasen": "esterar",
    "esterases": "esterar",
    "esteraste": "esterar",
    "esterasteis": "esterar",
    "estercola": "estercolar",
    "estercolaba": "estercolar",
    "estercolabais": "estercolar",
    "estercolaban": "estercolar",
    "estercolabas": "estercolar",
    "estercolad": "estercolar",
    "estercolada": "estercolar",
    "estercoladas": "estercolar",
    "estercolado": "estercolar",
    "estercolados": "estercolar",
    "estercoladuras": "estercoladura",
    "estercolamientos": "estercolamiento",
    "estercolamos": "estercolar",
    "estercolan": "estercolar",
    "estercolando": "estercolar",
    "estercolara": "estercolar",
    "estercolarais": "estercolar",
    "estercolaran": "estercolar",
    "estercolaras": "estercolar",
    "estercolare": "estercolar",
    "estercolareis": "estercolar",
    "estercolaremos": "estercolar",
    "estercolaren": "estercolar",
    "estercolares": "estercolar",
    "estercolaron": "estercolar",
    "estercolará": "estercolar",
    "estercolarán": "estercolar",
    "estercolarás": "estercolar",
    "estercolaré": "estercolar",
    "estercolaréis": "estercolar",
    "estercolaría": "estercolar",
    "estercolaríais": "estercolar",
    "estercolaríamos": "estercolar",
    "estercolarían": "estercolar",
    "estercolarías": "estercolar",
    "estercolas": "estercolar",
    "estercolase": "estercolar",
    "estercolaseis": "estercolar",
    "estercolasen": "estercolar",
    "estercolases": "estercolar",
    "estercolaste": "estercolar",
    "estercolasteis": "estercolar",
    "estercole": "estercolar",
    "estercolemos": "estercolar",
    "estercolen": "estercolar",
    "estercoleros": "estercolero",
    "estercoles": "estercolar",
    "estercolo": "estercolar",
    "estercolábamos": "estercolar",
    "estercoláis": "estercolar",
    "estercoláramos": "estercolar",
    "estercoláremos": "estercolar",
    "estercolásemos": "estercolar",
    "estercolé": "estercolar",
    "estercoléis": "estercolar",
    "estercoló": "estercolar",
    "estercuelos": "estercuelo",
    "esterculiáceas": "esterculiácea",
    "estercórea": "estercóreo",
    "estercóreas": "estercóreo",
    "estercóreos": "estercóreo",
    "estere": "esterar",
    "esteremos": "esterar",
    "esteren": "esterar",
    "estereobloques": "estereobloque",
    "estereocilios": "estereocilio",
    "estereocomparadores": "estereocomparador",
    "estereocristalina": "estereocristalino",
    "estereocristalinas": "estereocristalino",
    "estereocristalinos": "estereocristalino",
    "estereoespecífica": "estereoespecífico",
    "estereoespecíficas": "estereoespecífico",
    "estereoespecíficos": "estereoespecífico",
    "estereofonías": "estereofonía",
    "estereofónica": "estereofónico",
    "estereofónicas": "estereofónico",
    "estereofónicos": "estereofónico",
    "estereografías": "estereografía",
    "estereogramas": "estereograma",
    "estereográfica": "estereográfico",
    "estereográficas": "estereográfico",
    "estereográficos": "estereográfico",
    "estereoisomerías": "estereoisomería",
    "estereoisómera": "estereoisómero",
    "estereoisómeras": "estereoisómero",
    "estereoisómeros": "estereoisómero",
    "estereometrías": "estereometría",
    "estereomicroscopios": "estereomicroscopio",
    "estereométrica": "estereométrico",
    "estereométricas": "estereométrico",
    "estereométricos": "estereométrico",
    "estereoquímicas": "estereoquímica",
    "estereoradianes": "estereoradián",
    "estereorradianes": "estereorradián",
    "estereorregulares": "estereorregular",
    "estereoscopias": "estereoscopia",
    "estereoscopios": "estereoscopio",
    "estereoscópica": "estereoscópico",
    "estereoscópicas": "estereoscópico",
    "estereoscópicos": "estereoscópico",
    "estereoselectiva": "estereoselectivo",
    "estereoselectivas": "estereoselectivo",
    "estereoselectivos": "estereoselectivo",
    "estereotipa": "estereotipar",
    "estereotipaba": "estereotipar",
    "estereotipabais": "estereotipar",
    "estereotipaban": "estereotipar",
    "estereotipabas": "estereotipar",
    "estereotipad": "estereotipar",
    "estereotipada": "estereotipar",
    "estereotipadas": "estereotipar",
    "estereotipado": "estereotipar",
    "estereotipadora": "estereotipador",
    "estereotipadoras": "estereotipador",
    "estereotipadores": "estereotipador",
    "estereotipados": "estereotipar",
    "estereotipamos": "estereotipar",
    "estereotipan": "estereotipar",
    "estereotipando": "estereotipar",
    "estereotipara": "estereotipar",
    "estereotiparais": "estereotipar",
    "estereotiparan": "estereotipar",
    "estereotiparas": "estereotipar",
    "estereotipare": "estereotipar",
    "estereotipareis": "estereotipar",
    "estereotiparemos": "estereotipar",
    "estereotiparen": "estereotipar",
    "estereotipares": "estereotipar",
    "estereotiparon": "estereotipar",
    "estereotipará": "estereotipar",
    "estereotiparán": "estereotipar",
    "estereotiparás": "estereotipar",
    "estereotiparé": "estereotipar",
    "estereotiparéis": "estereotipar",
    "estereotiparía": "estereotipar",
    "estereotiparíais": "estereotipar",
    "estereotiparíamos": "estereotipar",
    "estereotiparían": "estereotipar",
    "estereotiparías": "estereotipar",
    "estereotipas": "estereotipar",
    "estereotipase": "estereotipar",
    "estereotipaseis": "estereotipar",
    "estereotipasen": "estereotipar",
    "estereotipases": "estereotipar",
    "estereotipaste": "estereotipar",
    "estereotipasteis": "estereotipar",
    "estereotipe": "estereotipar",
    "estereotipemos": "estereotipar",
    "estereotipen": "estereotipar",
    "estereotipes": "estereotipar",
    "estereotipias": "estereotipia",
    "estereotipo": "estereotipar",
    "estereotipos": "estereotipo",
    "estereotipábamos": "estereotipar",
    "estereotipáis": "estereotipar",
    "estereotipáramos": "estereotipar",
    "estereotipáremos": "estereotipar",
    "estereotipásemos": "estereotipar",
    "estereotipé": "estereotipar",
    "estereotipéis": "estereotipar",
    "estereotipó": "estereotipar",
    "estereotomías": "estereotomía",
    "estereotípica": "estereotípico",
    "estereotípicas": "estereotípico",
    "estereotípicos": "estereotípico",
    "estereotómica": "estereotómico",
    "estereotómicas": "estereotómico",
    "estereotómicos": "estereotómico",
    "esteres": "esterar",
    "estereóbatos": "estereóbato",
    "estereómetros": "estereómetro",
    "esterifica": "esterificar",
    "esterificaba": "esterificar",
    "esterificabais": "esterificar",
    "esterificaban": "esterificar",
    "esterificabas": "esterificar",
    "esterificaciones": "esterificación",
    "esterificad": "esterificar",
    "esterificada": "esterificar",
    "esterificadas": "esterificar",
    "esterificado": "esterificar",
    "esterificados": "esterificar",
    "esterificamos": "esterificar",
    "esterifican": "esterificar",
    "esterificando": "esterificar",
    "esterificara": "esterificar",
    "esterificarais": "esterificar",
    "esterificaran": "esterificar",
    "esterificaras": "esterificar",
    "esterificare": "esterificar",
    "esterificareis": "esterificar",
    "esterificaremos": "esterificar",
    "esterificaren": "esterificar",
    "esterificares": "esterificar",
    "esterificaron": "esterificar",
    "esterificará": "esterificar",
    "esterificarán": "esterificar",
    "esterificarás": "esterificar",
    "esterificaré": "esterificar",
    "esterificaréis": "esterificar",
    "esterificaría": "esterificar",
    "esterificaríais": "esterificar",
    "esterificaríamos": "esterificar",
    "esterificarían": "esterificar",
    "esterificarías": "esterificar",
    "esterificas": "esterificar",
    "esterificase": "esterificar",
    "esterificaseis": "esterificar",
    "esterificasen": "esterificar",
    "esterificases": "esterificar",
    "esterificaste": "esterificar",
    "esterificasteis": "esterificar",
    "esterifico": "esterificar",
    "esterificábamos": "esterificar",
    "esterificáis": "esterificar",
    "esterificáramos": "esterificar",
    "esterificáremos": "esterificar",
    "esterificásemos": "esterificar",
    "esterificó": "esterificar",
    "esterifique": "esterificar",
    "esterifiquemos": "esterificar",
    "esterifiquen": "esterificar",
    "esterifiques": "esterificar",
    "esterifiqué": "esterificar",
    "esterifiquéis": "esterificar",
    "esterigmas": "esterigma",
    "esterigmatocistinas": "esterigmatocistina",
    "esterilice": "esterilizar",
    "esterilicemos": "esterilizar",
    "esterilicen": "esterilizar",
    "esterilices": "esterilizar",
    "esterilicé": "esterilizar",
    "esterilicéis": "esterilizar",
    "esterilidades": "esterilidad",
    "esteriliza": "esterilizar",
    "esterilizaba": "esterilizar",
    "esterilizabais": "esterilizar",
    "esterilizaban": "esterilizar",
    "esterilizabas": "esterilizar",
    "esterilizaciones": "esterilización",
    "esterilizad": "esterilizar",
    "esterilizada": "esterilizar",
    "esterilizadas": "esterilizar",
    "esterilizado": "esterilizar",
    "esterilizadora": "esterilizador",
    "esterilizadoras": "esterilizador",
    "esterilizadores": "esterilizador",
    "esterilizados": "esterilizar",
    "esterilizamos": "esterilizar",
    "esterilizan": "esterilizar",
    "esterilizando": "esterilizar",
    "esterilizara": "esterilizar",
    "esterilizarais": "esterilizar",
    "esterilizaran": "esterilizar",
    "esterilizaras": "esterilizar",
    "esterilizare": "esterilizar",
    "esterilizareis": "esterilizar",
    "esterilizaremos": "esterilizar",
    "esterilizaren": "esterilizar",
    "esterilizares": "esterilizar",
    "esterilizaron": "esterilizar",
    "esterilizará": "esterilizar",
    "esterilizarán": "esterilizar",
    "esterilizarás": "esterilizar",
    "esterilizaré": "esterilizar",
    "esterilizaréis": "esterilizar",
    "esterilizaría": "esterilizar",
    "esterilizaríais": "esterilizar",
    "esterilizaríamos": "esterilizar",
    "esterilizarían": "esterilizar",
    "esterilizarías": "esterilizar",
    "esterilizas": "esterilizar",
    "esterilizase": "esterilizar",
    "esterilizaseis": "esterilizar",
    "esterilizasen": "esterilizar",
    "esterilizases": "esterilizar",
    "esterilizaste": "esterilizar",
    "esterilizasteis": "esterilizar",
    "esterilizo": "esterilizar",
    "esterilizábamos": "esterilizar",
    "esterilizáis": "esterilizar",
    "esterilizáramos": "esterilizar",
    "esterilizáremos": "esterilizar",
    "esterilizásemos": "esterilizar",
    "esterilizó": "esterilizar",
    "esterillas": "esterilla",
    "esterlinas": "esterlina",
    "esternales": "esternal",
    "esternoclaviculares": "esternoclavicular",
    "esternocleidomastoidea": "esternocleidomastoideo",
    "esternocleidomastoideas": "esternocleidomastoideo",
    "esternocleidomastoideos": "esternocleidomastoideo",
    "esternocostales": "esternocostal",
    "esternocostoclaviculares": "esternocostoclavicular",
    "esternohioidea": "esternohioideo",
    "esternohioideas": "esternohioideo",
    "esternohioideos": "esternohioideo",
    "esternones": "esternón",
    "esternopericardíaca": "esternopericardíaco",
    "esternopericardíacas": "esternopericardíaco",
    "esternopericardíacos": "esternopericardíaco",
    "esternotiroidea": "esternotiroideo",
    "esternotiroideas": "esternotiroideo",
    "esternotiroideos": "esternotiroideo",
    "estero": "esterar",
    "esteroides": "esteroide",
    "esteroles": "esterol",
    "esteros": "estero",
    "estertores": "estertor",
    "estertorosa": "estertoroso",
    "estertorosas": "estertoroso",
    "estertorosos": "estertoroso",
    "estertórea": "estertóreo",
    "estertóreas": "estertóreo",
    "estertóreos": "estertóreo",
    "esterábamos": "esterar",
    "esteráis": "esterar",
    "esteráramos": "esterar",
    "esteráremos": "esterar",
    "esterásemos": "esterar",
    "esteré": "esterar",
    "esteréis": "esterar",
    "esteró": "esterar",
    "estes": "este",
    "estesudestes": "estesudeste",
    "estetas": "esteta",
    "esteticismos": "esteticismo",
    "esteticistas": "esteticista",
    "estetoscopias": "estetoscopia",
    "estetoscopios": "estetoscopio",
    "estevada": "estevado",
    "estevadas": "estevado",
    "estevados": "estevado",
    "estevas": "esteva",
    "esteza": "estezar",
    "estezaba": "estezar",
    "estezabais": "estezar",
    "estezaban": "estezar",
    "estezabas": "estezar",
    "estezad": "estezar",
    "estezada": "estezar",
    "estezadas": "estezar",
    "estezado": "estezar",
    "estezados": "estezar",
    "estezamos": "estezar",
    "estezan": "estezar",
    "estezando": "estezar",
    "estezara": "estezar",
    "estezarais": "estezar",
    "estezaran": "estezar",
    "estezaras": "estezar",
    "estezare": "estezar",
    "estezareis": "estezar",
    "estezaremos": "estezar",
    "estezaren": "estezar",
    "estezares": "estezar",
    "estezaron": "estezar",
    "estezará": "estezar",
    "estezarán": "estezar",
    "estezarás": "estezar",
    "estezaré": "estezar",
    "estezaréis": "estezar",
    "estezaría": "estezar",
    "estezaríais": "estezar",
    "estezaríamos": "estezar",
    "estezarían": "estezar",
    "estezarías": "estezar",
    "estezas": "estezar",
    "estezase": "estezar",
    "estezaseis": "estezar",
    "estezasen": "estezar",
    "estezases": "estezar",
    "estezaste": "estezar",
    "estezasteis": "estezar",
    "estezo": "estezar",
    "estezábamos": "estezar",
    "estezáis": "estezar",
    "estezáramos": "estezar",
    "estezáremos": "estezar",
    "estezásemos": "estezar",
    "estezó": "estezar",
    "esteárica": "esteárico",
    "esteáricas": "esteárico",
    "esteáricos": "esteárico",
    "estheticiennees": "estheticienne",
    "estiajes": "estiaje",
    "estiba": "estibar",
    "estibaba": "estibar",
    "estibabais": "estibar",
    "estibaban": "estibar",
    "estibabas": "estibar",
    "estibad": "estibar",
    "estibada": "estibar",
    "estibadas": "estibar",
    "estibado": "estibar",
    "estibadora": "estibador",
    "estibadoras": "estibador",
    "estibadores": "estibador",
    "estibados": "estibar",
    "estibamos": "estibar",
    "estiban": "estibar",
    "estibando": "estibar",
    "estibara": "estibar",
    "estibarais": "estibar",
    "estibaran": "estibar",
    "estibaras": "estibar",
    "estibare": "estibar",
    "estibareis": "estibar",
    "estibaremos": "estibar",
    "estibaren": "estibar",
    "estibares": "estibar",
    "estibaron": "estibar",
    "estibará": "estibar",
    "estibarán": "estibar",
    "estibarás": "estibar",
    "estibaré": "estibar",
    "estibaréis": "estibar",
    "estibaría": "estibar",
    "estibaríais": "estibar",
    "estibaríamos": "estibar",
    "estibarían": "estibar",
    "estibarías": "estibar",
    "estibas": "estibar",
    "estibase": "estibar",
    "estibaseis": "estibar",
    "estibasen": "estibar",
    "estibases": "estibar",
    "estibaste": "estibar",
    "estibasteis": "estibar",
    "estibe": "estibar",
    "estibemos": "estibar",
    "estiben": "estibar",
    "estibes": "estibar",
    "estibinas": "estibina",
    "estibios": "estibio",
    "estibo": "estibar",
    "estibábamos": "estibar",
    "estibáis": "estibar",
    "estibáramos": "estibar",
    "estibáremos": "estibar",
    "estibásemos": "estibar",
    "estibé": "estibar",
    "estibéis": "estibar",
    "estibó": "estibar",
    "esticopos": "esticopo",
    "estiercos": "estierco",
    "estigia": "estigio",
    "estigias": "estigio",
    "estigios": "estigio",
    "estigmas": "estigma",
    "estigmasteroles": "estigmasterol",
    "estigmatice": "estigmatizar",
    "estigmaticemos": "estigmatizar",
    "estigmaticen": "estigmatizar",
    "estigmatices": "estigmatizar",
    "estigmaticé": "estigmatizar",
    "estigmaticéis": "estigmatizar",
    "estigmatismos": "estigmatismo",
    "estigmatiza": "estigmatizar",
    "estigmatizaba": "estigmatizar",
    "estigmatizabais": "estigmatizar",
    "estigmatizaban": "estigmatizar",
    "estigmatizabas": "estigmatizar",
    "estigmatizaciones": "estigmatización",
    "estigmatizad": "estigmatizar",
    "estigmatizada": "estigmatizar",
    "estigmatizadas": "estigmatizar",
    "estigmatizado": "estigmatizar",
    "estigmatizadora": "estigmatizador",
    "estigmatizadoras": "estigmatizador",
    "estigmatizadores": "estigmatizador",
    "estigmatizados": "estigmatizar",
    "estigmatizamos": "estigmatizar",
    "estigmatizan": "estigmatizar",
    "estigmatizando": "estigmatizar",
    "estigmatizara": "estigmatizar",
    "estigmatizarais": "estigmatizar",
    "estigmatizaran": "estigmatizar",
    "estigmatizaras": "estigmatizar",
    "estigmatizare": "estigmatizar",
    "estigmatizareis": "estigmatizar",
    "estigmatizaremos": "estigmatizar",
    "estigmatizaren": "estigmatizar",
    "estigmatizares": "estigmatizar",
    "estigmatizaron": "estigmatizar",
    "estigmatizará": "estigmatizar",
    "estigmatizarán": "estigmatizar",
    "estigmatizarás": "estigmatizar",
    "estigmatizaré": "estigmatizar",
    "estigmatizaréis": "estigmatizar",
    "estigmatizaría": "estigmatizar",
    "estigmatizaríais": "estigmatizar",
    "estigmatizaríamos": "estigmatizar",
    "estigmatizarían": "estigmatizar",
    "estigmatizarías": "estigmatizar",
    "estigmatizas": "estigmatizar",
    "estigmatizase": "estigmatizar",
    "estigmatizaseis": "estigmatizar",
    "estigmatizasen": "estigmatizar",
    "estigmatizases": "estigmatizar",
    "estigmatizaste": "estigmatizar",
    "estigmatizasteis": "estigmatizar",
    "estigmatizo": "estigmatizar",
    "estigmatizábamos": "estigmatizar",
    "estigmatizáis": "estigmatizar",
    "estigmatizáramos": "estigmatizar",
    "estigmatizáremos": "estigmatizar",
    "estigmatizásemos": "estigmatizar",
    "estigmatizó": "estigmatizar",
    "estigmática": "estigmático",
    "estigmáticas": "estigmático",
    "estigmáticos": "estigmático",
    "estila": "estilar",
    "estilaba": "estilar",
    "estilabais": "estilar",
    "estilaban": "estilar",
    "estilabas": "estilar",
    "estilad": "estilar",
    "estilada": "estilar",
    "estiladas": "estilar",
    "estilado": "estilar",
    "estilados": "estilar",
    "estilamos": "estilar",
    "estilan": "estilar",
    "estilando": "estilar",
    "estilara": "estilar",
    "estilarais": "estilar",
    "estilaran": "estilar",
    "estilaras": "estilar",
    "estilare": "estilar",
    "estilareis": "estilar",
    "estilaremos": "estilar",
    "estilaren": "estilar",
    "estilares": "estilar",
    "estilaron": "estilar",
    "estilará": "estilar",
    "estilarán": "estilar",
    "estilarás": "estilar",
    "estilaré": "estilar",
    "estilaréis": "estilar",
    "estilaría": "estilar",
    "estilaríais": "estilar",
    "estilaríamos": "estilar",
    "estilarían": "estilar",
    "estilarías": "estilar",
    "estilas": "estilar",
    "estilase": "estilar",
    "estilaseis": "estilar",
    "estilasen": "estilar",
    "estilases": "estilar",
    "estilaste": "estilar",
    "estilasteis": "estilar",
    "estilbenos": "estilbeno",
    "estilbitas": "estilbita",
    "estile": "estilar",
    "estilemos": "estilar",
    "estilen": "estilar",
    "estiles": "estilar",
    "estiletes": "estilete",
    "estilice": "estilizar",
    "estilicemos": "estilizar",
    "estilicen": "estilizar",
    "estilices": "estilizar",
    "estilicé": "estilizar",
    "estilicéis": "estilizar",
    "estilistas": "estilista",
    "estilitas": "estilita",
    "estiliza": "estilizar",
    "estilizaba": "estilizar",
    "estilizabais": "estilizar",
    "estilizaban": "estilizar",
    "estilizabas": "estilizar",
    "estilizaciones": "estilización",
    "estilizad": "estilizar",
    "estilizada": "estilizar",
    "estilizadas": "estilizar",
    "estilizado": "estilizar",
    "estilizados": "estilizar",
    "estilizamos": "estilizar",
    "estilizan": "estilizar",
    "estilizando": "estilizar",
    "estilizara": "estilizar",
    "estilizarais": "estilizar",
    "estilizaran": "estilizar",
    "estilizaras": "estilizar",
    "estilizare": "estilizar",
    "estilizareis": "estilizar",
    "estilizaremos": "estilizar",
    "estilizaren": "estilizar",
    "estilizares": "estilizar",
    "estilizaron": "estilizar",
    "estilizará": "estilizar",
    "estilizarán": "estilizar",
    "estilizarás": "estilizar",
    "estilizaré": "estilizar",
    "estilizaréis": "estilizar",
    "estilizaría": "estilizar",
    "estilizaríais": "estilizar",
    "estilizaríamos": "estilizar",
    "estilizarían": "estilizar",
    "estilizarías": "estilizar",
    "estilizas": "estilizar",
    "estilizase": "estilizar",
    "estilizaseis": "estilizar",
    "estilizasen": "estilizar",
    "estilizases": "estilizar",
    "estilizaste": "estilizar",
    "estilizasteis": "estilizar",
    "estilizo": "estilizar",
    "estilizábamos": "estilizar",
    "estilizáis": "estilizar",
    "estilizáramos": "estilizar",
    "estilizáremos": "estilizar",
    "estilizásemos": "estilizar",
    "estilizó": "estilizar",
    "estilo": "estilar",
    "estilobatos": "estilobato",
    "estilofaríngea": "estilofaríngeo",
    "estilofaríngeas": "estilofaríngeo",
    "estilofaríngeos": "estilofaríngeo",
    "estiloglosa": "estilogloso",
    "estiloglosas": "estilogloso",
    "estiloglosos": "estilogloso",
    "estilográfica": "estilográfico",
    "estilográficas": "estilográfico",
    "estilográficos": "estilográfico",
    "estilohioidea": "estilohioideo",
    "estilohioideas": "estilohioideo",
    "estilohioideos": "estilohioideo",
    "estiloidea": "estiloideo",
    "estiloideas": "estiloideo",
    "estiloideos": "estiloideo",
    "estilomandibulares": "estilomandibular",
    "estilomastoidea": "estilomastoideo",
    "estilomastoideas": "estilomastoideo",
    "estilomastoideos": "estilomastoideo",
    "estilomatóforos": "estilomatóforo",
    "estilos": "estilo",
    "estilábamos": "estilar",
    "estiláis": "estilar",
    "estiláramos": "estilar",
    "estiláremos": "estilar",
    "estilásemos": "estilar",
    "estilé": "estilar",
    "estiléis": "estilar",
    "estilística": "estilístico",
    "estilísticas": "estilístico",
    "estilísticos": "estilístico",
    "estiló": "estilar",
    "estilóbatos": "estilóbato",
    "estilógrafos": "estilógrafo",
    "estima": "estimar",
    "estimaba": "estimar",
    "estimabais": "estimar",
    "estimaban": "estimar",
    "estimabas": "estimar",
    "estimabilidades": "estimabilidad",
    "estimables": "estimable",
    "estimaciones": "estimación",
    "estimad": "estimar",
    "estimada": "estimar",
    "estimadas": "estimar",
    "estimado": "estimar",
    "estimadora": "estimador",
    "estimadoras": "estimador",
    "estimadores": "estimador",
    "estimados": "estimar",
    "estimamos": "estimar",
    "estiman": "estimar",
    "estimando": "estimar",
    "estimara": "estimar",
    "estimarais": "estimar",
    "estimaran": "estimar",
    "estimaras": "estimar",
    "estimare": "estimar",
    "estimareis": "estimar",
    "estimaremos": "estimar",
    "estimaren": "estimar",
    "estimares": "estimar",
    "estimaron": "estimar",
    "estimará": "estimar",
    "estimarán": "estimar",
    "estimarás": "estimar",
    "estimaré": "estimar",
    "estimaréis": "estimar",
    "estimaría": "estimar",
    "estimaríais": "estimar",
    "estimaríamos": "estimar",
    "estimarían": "estimar",
    "estimarías": "estimar",
    "estimas": "estimar",
    "estimase": "estimar",
    "estimaseis": "estimar",
    "estimasen": "estimar",
    "estimases": "estimar",
    "estimaste": "estimar",
    "estimasteis": "estimar",
    "estimativa": "estimativo",
    "estimativas": "estimativo",
    "estimativos": "estimativo",
    "estimatoria": "estimatorio",
    "estimatorias": "estimatorio",
    "estimatorios": "estimatorio",
    "estime": "estimar",
    "estimemos": "estimar",
    "estimen": "estimar",
    "estimes": "estimar",
    "estimo": "estimar",
    "estimula": "estimular",
    "estimulaba": "estimular",
    "estimulabais": "estimular",
    "estimulaban": "estimular",
    "estimulabas": "estimular",
    "estimulaciones": "estimulación",
    "estimulad": "estimular",
    "estimulada": "estimular",
    "estimuladas": "estimular",
    "estimulado": "estimular",
    "estimuladora": "estimulador",
    "estimuladoras": "estimulador",
    "estimuladores": "estimulador",
    "estimulados": "estimular",
    "estimulamos": "estimular",
    "estimulan": "estimular",
    "estimulando": "estimular",
    "estimulantes": "estimulante",
    "estimulara": "estimular",
    "estimularais": "estimular",
    "estimularan": "estimular",
    "estimularas": "estimular",
    "estimulare": "estimular",
    "estimulareis": "estimular",
    "estimularemos": "estimular",
    "estimularen": "estimular",
    "estimulares": "estimular",
    "estimularon": "estimular",
    "estimulará": "estimular",
    "estimularán": "estimular",
    "estimularás": "estimular",
    "estimularé": "estimular",
    "estimularéis": "estimular",
    "estimularía": "estimular",
    "estimularíais": "estimular",
    "estimularíamos": "estimular",
    "estimularían": "estimular",
    "estimularías": "estimular",
    "estimulas": "estimular",
    "estimulase": "estimular",
    "estimulaseis": "estimular",
    "estimulasen": "estimular",
    "estimulases": "estimular",
    "estimulaste": "estimular",
    "estimulasteis": "estimular",
    "estimule": "estimular",
    "estimulemos": "estimular",
    "estimulen": "estimular",
    "estimules": "estimular",
    "estimulo": "estimular",
    "estimulábamos": "estimular",
    "estimuláis": "estimular",
    "estimuláramos": "estimular",
    "estimuláremos": "estimular",
    "estimulásemos": "estimular",
    "estimulé": "estimular",
    "estimuléis": "estimular",
    "estimuló": "estimular",
    "estimábamos": "estimar",
    "estimáis": "estimar",
    "estimáramos": "estimar",
    "estimáremos": "estimar",
    "estimásemos": "estimar",
    "estimé": "estimar",
    "estiméis": "estimar",
    "estimó": "estimar",
    "estipendia": "estipendiar",
    "estipendiaba": "estipendiar",
    "estipendiabais": "estipendiar",
    "estipendiaban": "estipendiar",
    "estipendiabas": "estipendiar",
    "estipendiad": "estipendiar",
    "estipendiada": "estipendiar",
    "estipendiadas": "estipendiar",
    "estipendiado": "estipendiar",
    "estipendiados": "estipendiar",
    "estipendiamos": "estipendiar",
    "estipendian": "estipendiar",
    "estipendiando": "estipendiar",
    "estipendiara": "estipendiar",
    "estipendiarais": "estipendiar",
    "estipendiaran": "estipendiar",
    "estipendiaras": "estipendiar",
    "estipendiare": "estipendiar",
    "estipendiareis": "estipendiar",
    "estipendiaremos": "estipendiar",
    "estipendiaren": "estipendiar",
    "estipendiares": "estipendiar",
    "estipendiaria": "estipendiario",
    "estipendiarias": "estipendiario",
    "estipendiarios": "estipendiario",
    "estipendiaron": "estipendiar",
    "estipendiará": "estipendiar",
    "estipendiarán": "estipendiar",
    "estipendiarás": "estipendiar",
    "estipendiaré": "estipendiar",
    "estipendiaréis": "estipendiar",
    "estipendiaría": "estipendiar",
    "estipendiaríais": "estipendiar",
    "estipendiaríamos": "estipendiar",
    "estipendiarían": "estipendiar",
    "estipendiarías": "estipendiar",
    "estipendias": "estipendiar",
    "estipendiase": "estipendiar",
    "estipendiaseis": "estipendiar",
    "estipendiasen": "estipendiar",
    "estipendiases": "estipendiar",
    "estipendiaste": "estipendiar",
    "estipendiasteis": "estipendiar",
    "estipendie": "estipendiar",
    "estipendiemos": "estipendiar",
    "estipendien": "estipendiar",
    "estipendies": "estipendiar",
    "estipendio": "estipendiar",
    "estipendios": "estipendio",
    "estipendiábamos": "estipendiar",
    "estipendiáis": "estipendiar",
    "estipendiáramos": "estipendiar",
    "estipendiáremos": "estipendiar",
    "estipendiásemos": "estipendiar",
    "estipendié": "estipendiar",
    "estipendiéis": "estipendiar",
    "estipendió": "estipendiar",
    "estipula": "estipular",
    "estipulaba": "estipular",
    "estipulabais": "estipular",
    "estipulaban": "estipular",
    "estipulabas": "estipular",
    "estipulaciones": "estipulación",
    "estipulad": "estipular",
    "estipulada": "estipular",
    "estipuladas": "estipular",
    "estipulado": "estipular",
    "estipulados": "estipular",
    "estipulamos": "estipular",
    "estipulan": "estipular",
    "estipulando": "estipular",
    "estipulara": "estipular",
    "estipularais": "estipular",
    "estipularan": "estipular",
    "estipularas": "estipular",
    "estipulare": "estipular",
    "estipulareis": "estipular",
    "estipularemos": "estipular",
    "estipularen": "estipular",
    "estipulares": "estipular",
    "estipularon": "estipular",
    "estipulará": "estipular",
    "estipularán": "estipular",
    "estipularás": "estipular",
    "estipularé": "estipular",
    "estipularéis": "estipular",
    "estipularía": "estipular",
    "estipularíais": "estipular",
    "estipularíamos": "estipular",
    "estipularían": "estipular",
    "estipularías": "estipular",
    "estipulas": "estipular",
    "estipulase": "estipular",
    "estipulaseis": "estipular",
    "estipulasen": "estipular",
    "estipulases": "estipular",
    "estipulaste": "estipular",
    "estipulasteis": "estipular",
    "estipule": "estipular",
    "estipulemos": "estipular",
    "estipulen": "estipular",
    "estipules": "estipular",
    "estipulo": "estipular",
    "estipulábamos": "estipular",
    "estipuláis": "estipular",
    "estipuláramos": "estipular",
    "estipuláremos": "estipular",
    "estipulásemos": "estipular",
    "estipulé": "estipular",
    "estipuléis": "estipular",
    "estipuló": "estipular",
    "estiques": "estique",
    "estira": "estirar",
    "estiraba": "estirar",
    "estirabais": "estirar",
    "estiraban": "estirar",
    "estirabas": "estirar",
    "estirables": "estirable",
    "estiracáceas": "estiracácea",
    "estirad": "estirar",
    "estirada": "estirar",
    "estiradas": "estirar",
    "estirado": "estirar",
    "estiradora": "estirador",
    "estiradoras": "estiradora",
    "estiradores": "estirador",
    "estirados": "estirar",
    "estirajes": "estiraje",
    "estiralílica": "estiralílico",
    "estiralílicas": "estiralílico",
    "estiralílicos": "estiralílico",
    "estiramientos": "estiramiento",
    "estiramos": "estirar",
    "estiran": "estirar",
    "estirando": "estirar",
    "estirara": "estirar",
    "estirarais": "estirar",
    "estiraran": "estirar",
    "estiraras": "estirar",
    "estirare": "estirar",
    "estirareis": "estirar",
    "estiraremos": "estirar",
    "estiraren": "estirar",
    "estirares": "estirar",
    "estiraron": "estirar",
    "estirará": "estirar",
    "estirarán": "estirar",
    "estirarás": "estirar",
    "estiraré": "estirar",
    "estiraréis": "estirar",
    "estiraría": "estirar",
    "estiraríais": "estirar",
    "estiraríamos": "estirar",
    "estirarían": "estirar",
    "estirarías": "estirar",
    "estiras": "estirar",
    "estirase": "estirar",
    "estiraseis": "estirar",
    "estirasen": "estirar",
    "estirases": "estirar",
    "estiraste": "estirar",
    "estirasteis": "estirar",
    "estire": "estirar",
    "estiremos": "estirar",
    "estiren": "estirar",
    "estirenos": "estireno",
    "estires": "estirar",
    "estiro": "estirar",
    "estirones": "estirón",
    "estirpes": "estirpe",
    "estirábamos": "estirar",
    "estiráis": "estirar",
    "estiráramos": "estirar",
    "estiráremos": "estirar",
    "estirásemos": "estirar",
    "estiré": "estirar",
    "estiréis": "estirar",
    "estiró": "estirar",
    "estivaciones": "estivación",
    "estivadas": "estivada",
    "estivales": "estival",
    "estiércoles": "estiércol",
    "estiómenos": "estiómeno",
    "esto": "este",
    "estocadas": "estocada",
    "estocs": "estoc",
    "estocástica": "estocástico",
    "estocásticas": "estocástico",
    "estocásticos": "estocástico",
    "estofa": "estofar",
    "estofaba": "estofar",
    "estofabais": "estofar",
    "estofaban": "estofar",
    "estofabas": "estofar",
    "estofad": "estofar",
    "estofada": "estofar",
    "estofadas": "estofar",
    "estofado": "estofar",
    "estofados": "estofar",
    "estofamos": "estofar",
    "estofan": "estofar",
    "estofando": "estofar",
    "estofara": "estofar",
    "estofarais": "estofar",
    "estofaran": "estofar",
    "estofaras": "estofar",
    "estofare": "estofar",
    "estofareis": "estofar",
    "estofaremos": "estofar",
    "estofaren": "estofar",
    "estofares": "estofar",
    "estofaron": "estofar",
    "estofará": "estofar",
    "estofarán": "estofar",
    "estofarás": "estofar",
    "estofaré": "estofar",
    "estofaréis": "estofar",
    "estofaría": "estofar",
    "estofaríais": "estofar",
    "estofaríamos": "estofar",
    "estofarían": "estofar",
    "estofarías": "estofar",
    "estofas": "estofar",
    "estofase": "estofar",
    "estofaseis": "estofar",
    "estofasen": "estofar",
    "estofases": "estofar",
    "estofaste": "estofar",
    "estofasteis": "estofar",
    "estofe": "estofar",
    "estofemos": "estofar",
    "estofen": "estofar",
    "estofes": "estofar",
    "estofo": "estofar",
    "estofos": "estofo",
    "estofábamos": "estofar",
    "estofáis": "estofar",
    "estofáramos": "estofar",
    "estofáremos": "estofar",
    "estofásemos": "estofar",
    "estofé": "estofar",
    "estoféis": "estofar",
    "estofó": "estofar",
    "estoica": "estoico",
    "estoicas": "estoico",
    "estoicismos": "estoicismo",
    "estoicos": "estoico",
    "estolas": "estola",
    "estolideces": "estolidez",
    "estolones": "estolón",
    "estomacales": "estomacal",
    "estomaga": "estomagar",
    "estomagaba": "estomagar",
    "estomagabais": "estomagar",
    "estomagaban": "estomagar",
    "estomagabas": "estomagar",
    "estomagad": "estomagar",
    "estomagada": "estomagar",
    "estomagadas": "estomagar",
    "estomagado": "estomagar",
    "estomagados": "estomagar",
    "estomagamos": "estomagar",
    "estomagan": "estomagar",
    "estomagando": "estomagar",
    "estomagara": "estomagar",
    "estomagarais": "estomagar",
    "estomagaran": "estomagar",
    "estomagaras": "estomagar",
    "estomagare": "estomagar",
    "estomagareis": "estomagar",
    "estomagaremos": "estomagar",
    "estomagaren": "estomagar",
    "estomagares": "estomagar",
    "estomagaron": "estomagar",
    "estomagará": "estomagar",
    "estomagarán": "estomagar",
    "estomagarás": "estomagar",
    "estomagaré": "estomagar",
    "estomagaréis": "estomagar",
    "estomagaría": "estomagar",
    "estomagaríais": "estomagar",
    "estomagaríamos": "estomagar",
    "estomagarían": "estomagar",
    "estomagarías": "estomagar",
    "estomagas": "estomagar",
    "estomagase": "estomagar",
    "estomagaseis": "estomagar",
    "estomagasen": "estomagar",
    "estomagases": "estomagar",
    "estomagaste": "estomagar",
    "estomagasteis": "estomagar",
    "estomago": "estomagar",
    "estomague": "estomagar",
    "estomaguemos": "estomagar",
    "estomaguen": "estomagar",
    "estomagues": "estomagar",
    "estomagué": "estomagar",
    "estomaguéis": "estomagar",
    "estomagábamos": "estomagar",
    "estomagáis": "estomagar",
    "estomagáramos": "estomagar",
    "estomagáremos": "estomagar",
    "estomagásemos": "estomagar",
    "estomagó": "estomagar",
    "estomas": "estoma",
    "estomatalgias": "estomatalgia",
    "estomatocitos": "estomatocito",
    "estomatognática": "estomatognático",
    "estomatognáticas": "estomatognático",
    "estomatognáticos": "estomatognático",
    "estomatografías": "estomatografía",
    "estomatolalias": "estomatolalia",
    "estomatologías": "estomatología",
    "estomatológica": "estomatológico",
    "estomatológicas": "estomatológico",
    "estomatológicos": "estomatológico",
    "estomatopatías": "estomatopatía",
    "estomatoplastias": "estomatoplastia",
    "estomatorragias": "estomatorragia",
    "estomatoscopios": "estomatoscopio",
    "estomatóloga": "estomatólogo",
    "estomatólogas": "estomatólogo",
    "estomatólogos": "estomatólogo",
    "estomiátidos": "estomiátido",
    "estomática": "estomático",
    "estomáticas": "estomático",
    "estomáticos": "estomático",
    "estonia": "estonio",
    "estoniana": "estoniano",
    "estonianas": "estoniano",
    "estonianos": "estoniano",
    "estonias": "estonio",
    "estonios": "estonio",
    "estopadas": "estopada",
    "estopas": "estopa",
    "estopeña": "estopeño",
    "estopeñas": "estopeño",
    "estopeños": "estopeño",
    "estopillas": "estopilla",
    "estopines": "estopín",
    "estopones": "estopón",
    "estopores": "estopor",
    "estoposa": "estoposo",
    "estoposas": "estoposo",
    "estoposos": "estoposo",
    "estoppeles": "estoppel",
    "estoquea": "estoquear",
    "estoqueaba": "estoquear",
    "estoqueabais": "estoquear",
    "estoqueaban": "estoquear",
    "estoqueabas": "estoquear",
    "estoquead": "estoquear",
    "estoqueada": "estoquear",
    "estoqueadas": "estoquear",
    "estoqueado": "estoquear",
    "estoqueadora": "estoqueador",
    "estoqueadoras": "estoqueador",
    "estoqueadores": "estoqueador",
    "estoqueados": "estoquear",
    "estoqueamos": "estoquear",
    "estoquean": "estoquear",
    "estoqueando": "estoquear",
    "estoqueara": "estoquear",
    "estoquearais": "estoquear",
    "estoquearan": "estoquear",
    "estoquearas": "estoquear",
    "estoqueare": "estoquear",
    "estoqueareis": "estoquear",
    "estoquearemos": "estoquear",
    "estoquearen": "estoquear",
    "estoqueares": "estoquear",
    "estoquearon": "estoquear",
    "estoqueará": "estoquear",
    "estoquearán": "estoquear",
    "estoquearás": "estoquear",
    "estoquearé": "estoquear",
    "estoquearéis": "estoquear",
    "estoquearía": "estoquear",
    "estoquearíais": "estoquear",
    "estoquearíamos": "estoquear",
    "estoquearían": "estoquear",
    "estoquearías": "estoquear",
    "estoqueas": "estoquear",
    "estoquease": "estoquear",
    "estoqueaseis": "estoquear",
    "estoqueasen": "estoquear",
    "estoqueases": "estoquear",
    "estoqueaste": "estoquear",
    "estoqueasteis": "estoquear",
    "estoquee": "estoquear",
    "estoqueemos": "estoquear",
    "estoqueen": "estoquear",
    "estoquees": "estoquear",
    "estoqueo": "estoquear",
    "estoques": "estoque",
    "estoqueábamos": "estoquear",
    "estoqueáis": "estoquear",
    "estoqueáramos": "estoquear",
    "estoqueáremos": "estoquear",
    "estoqueásemos": "estoquear",
    "estoqueé": "estoquear",
    "estoqueéis": "estoquear",
    "estoqueó": "estoquear",
    "estoquillos": "estoquillo",
    "estoraques": "estoraque",
    "estorba": "estorbar",
    "estorbaba": "estorbar",
    "estorbabais": "estorbar",
    "estorbaban": "estorbar",
    "estorbabas": "estorbar",
    "estorbad": "estorbar",
    "estorbada": "estorbar",
    "estorbadas": "estorbar",
    "estorbado": "estorbar",
    "estorbados": "estorbar",
    "estorbamos": "estorbar",
    "estorban": "estorbar",
    "estorbando": "estorbar",
    "estorbara": "estorbar",
    "estorbarais": "estorbar",
    "estorbaran": "estorbar",
    "estorbaras": "estorbar",
    "estorbare": "estorbar",
    "estorbareis": "estorbar",
    "estorbaremos": "estorbar",
    "estorbaren": "estorbar",
    "estorbares": "estorbar",
    "estorbaron": "estorbar",
    "estorbará": "estorbar",
    "estorbarán": "estorbar",
    "estorbarás": "estorbar",
    "estorbaré": "estorbar",
    "estorbaréis": "estorbar",
    "estorbaría": "estorbar",
    "estorbaríais": "estorbar",
    "estorbaríamos": "estorbar",
    "estorbarían": "estorbar",
    "estorbarías": "estorbar",
    "estorbas": "estorbar",
    "estorbase": "estorbar",
    "estorbaseis": "estorbar",
    "estorbasen": "estorbar",
    "estorbases": "estorbar",
    "estorbaste": "estorbar",
    "estorbasteis": "estorbar",
    "estorbe": "estorbar",
    "estorbemos": "estorbar",
    "estorben": "estorbar",
    "estorbes": "estorbar",
    "estorbo": "estorbar",
    "estorbos": "estorbo",
    "estorbábamos": "estorbar",
    "estorbáis": "estorbar",
    "estorbáramos": "estorbar",
    "estorbáremos": "estorbar",
    "estorbásemos": "estorbar",
    "estorbé": "estorbar",
    "estorbéis": "estorbar",
    "estorbó": "estorbar",
    "estores": "estor",
    "estornijas": "estornija",
    "estorninos": "estornino",
    "estornuda": "estornudar",
    "estornudaba": "estornudar",
    "estornudabais": "estornudar",
    "estornudaban": "estornudar",
    "estornudabas": "estornudar",
    "estornudad": "estornudar",
    "estornudada": "estornudar",
    "estornudadas": "estornudar",
    "estornudado": "estornudar",
    "estornudados": "estornudar",
    "estornudamos": "estornudar",
    "estornudan": "estornudar",
    "estornudando": "estornudar",
    "estornudara": "estornudar",
    "estornudarais": "estornudar",
    "estornudaran": "estornudar",
    "estornudaras": "estornudar",
    "estornudare": "estornudar",
    "estornudareis": "estornudar",
    "estornudaremos": "estornudar",
    "estornudaren": "estornudar",
    "estornudares": "estornudar",
    "estornudaron": "estornudar",
    "estornudará": "estornudar",
    "estornudarán": "estornudar",
    "estornudarás": "estornudar",
    "estornudaré": "estornudar",
    "estornudaréis": "estornudar",
    "estornudaría": "estornudar",
    "estornudaríais": "estornudar",
    "estornudaríamos": "estornudar",
    "estornudarían": "estornudar",
    "estornudarías": "estornudar",
    "estornudas": "estornudar",
    "estornudase": "estornudar",
    "estornudaseis": "estornudar",
    "estornudasen": "estornudar",
    "estornudases": "estornudar",
    "estornudaste": "estornudar",
    "estornudasteis": "estornudar",
    "estornude": "estornudar",
    "estornudemos": "estornudar",
    "estornuden": "estornudar",
    "estornudes": "estornudar",
    "estornudo": "estornudar",
    "estornudos": "estornudo",
    "estornudábamos": "estornudar",
    "estornudáis": "estornudar",
    "estornudáramos": "estornudar",
    "estornudáremos": "estornudar",
    "estornudásemos": "estornudar",
    "estornudé": "estornudar",
    "estornudéis": "estornudar",
    "estornudó": "estornudar",
    "estos": "este",
    "estotra": "estotro",
    "estotras": "estotro",
    "estotros": "estotro",
    "estova": "estovar",
    "estovaba": "estovar",
    "estovabais": "estovar",
    "estovaban": "estovar",
    "estovabas": "estovar",
    "estovad": "estovar",
    "estovada": "estovar",
    "estovadas": "estovar",
    "estovado": "estovar",
    "estovados": "estovar",
    "estovamos": "estovar",
    "estovan": "estovar",
    "estovando": "estovar",
    "estovara": "estovar",
    "estovarais": "estovar",
    "estovaran": "estovar",
    "estovaras": "estovar",
    "estovare": "estovar",
    "estovareis": "estovar",
    "estovaremos": "estovar",
    "estovaren": "estovar",
    "estovares": "estovar",
    "estovaron": "estovar",
    "estovará": "estovar",
    "estovarán": "estovar",
    "estovarás": "estovar",
    "estovaré": "estovar",
    "estovaréis": "estovar",
    "estovaría": "estovar",
    "estovaríais": "estovar",
    "estovaríamos": "estovar",
    "estovarían": "estovar",
    "estovarías": "estovar",
    "estovas": "estovar",
    "estovase": "estovar",
    "estovaseis": "estovar",
    "estovasen": "estovar",
    "estovases": "estovar",
    "estovaste": "estovar",
    "estovasteis": "estovar",
    "estove": "estovar",
    "estovemos": "estovar",
    "estoven": "estovar",
    "estoves": "estovar",
    "estovo": "estovar",
    "estovábamos": "estovar",
    "estováis": "estovar",
    "estováramos": "estovar",
    "estováremos": "estovar",
    "estovásemos": "estovar",
    "estové": "estovar",
    "estovéis": "estovar",
    "estovó": "estovar",
    "estoy": "estar",
    "estrabismos": "estrabismo",
    "estrabismómetros": "estrabismómetro",
    "estrabometrías": "estrabometría",
    "estrabotomías": "estrabotomía",
    "estrabómetros": "estrabómetro",
    "estracillas": "estracilla",
    "estradioles": "estradiol",
    "estradiotas": "estradiota",
    "estradiotes": "estradiote",
    "estrados": "estrado",
    "estrafalaria": "estrafalario",
    "estrafalarias": "estrafalario",
    "estrafalarios": "estrafalario",
    "estraga": "estragar",
    "estragaba": "estragar",
    "estragabais": "estragar",
    "estragaban": "estragar",
    "estragabas": "estragar",
    "estragad": "estragar",
    "estragada": "estragar",
    "estragadas": "estragar",
    "estragado": "estragar",
    "estragadora": "estragador",
    "estragadoras": "estragador",
    "estragadores": "estragador",
    "estragados": "estragar",
    "estragamientos": "estragamiento",
    "estragamos": "estragar",
    "estragan": "estragar",
    "estragando": "estragar",
    "estragara": "estragar",
    "estragarais": "estragar",
    "estragaran": "estragar",
    "estragaras": "estragar",
    "estragare": "estragar",
    "estragareis": "estragar",
    "estragaremos": "estragar",
    "estragaren": "estragar",
    "estragares": "estragar",
    "estragaron": "estragar",
    "estragará": "estragar",
    "estragarán": "estragar",
    "estragarás": "estragar",
    "estragaré": "estragar",
    "estragaréis": "estragar",
    "estragaría": "estragar",
    "estragaríais": "estragar",
    "estragaríamos": "estragar",
    "estragarían": "estragar",
    "estragarías": "estragar",
    "estragas": "estragar",
    "estragase": "estragar",
    "estragaseis": "estragar",
    "estragasen": "estragar",
    "estragases": "estragar",
    "estragaste": "estragar",
    "estragasteis": "estragar",
    "estrago": "estragar",
    "estragoles": "estragol",
    "estragones": "estragón",
    "estragos": "estrago",
    "estrague": "estragar",
    "estraguemos": "estragar",
    "estraguen": "estragar",
    "estragues": "estragar",
    "estragué": "estragar",
    "estraguéis": "estragar",
    "estragábamos": "estragar",
    "estragáis": "estragar",
    "estragáramos": "estragar",
    "estragáremos": "estragar",
    "estragásemos": "estragar",
    "estragó": "estragar",
    "estrambotes": "estrambote",
    "estrambótica": "estrambótico",
    "estrambóticas": "estrambótico",
    "estrambóticos": "estrambótico",
    "estramonios": "estramonio",
    "estrangoles": "estrangol",
    "estrangula": "estrangular",
    "estrangulaba": "estrangular",
    "estrangulabais": "estrangular",
    "estrangulaban": "estrangular",
    "estrangulabas": "estrangular",
    "estrangulaciones": "estrangulación",
    "estrangulad": "estrangular",
    "estrangulada": "estrangular",
    "estranguladas": "estrangular",
    "estrangulado": "estrangular",
    "estranguladora": "estrangulador",
    "estranguladoras": "estrangulador",
    "estranguladores": "estrangulador",
    "estrangulados": "estrangular",
    "estrangulamientos": "estrangulamiento",
    "estrangulamos": "estrangular",
    "estrangulan": "estrangular",
    "estrangulando": "estrangular",
    "estrangulara": "estrangular",
    "estrangularais": "estrangular",
    "estrangularan": "estrangular",
    "estrangularas": "estrangular",
    "estrangulare": "estrangular",
    "estrangulareis": "estrangular",
    "estrangularemos": "estrangular",
    "estrangularen": "estrangular",
    "estrangulares": "estrangular",
    "estrangularon": "estrangular",
    "estrangulará": "estrangular",
    "estrangularán": "estrangular",
    "estrangularás": "estrangular",
    "estrangularé": "estrangular",
    "estrangularéis": "estrangular",
    "estrangularía": "estrangular",
    "estrangularíais": "estrangular",
    "estrangularíamos": "estrangular",
    "estrangularían": "estrangular",
    "estrangularías": "estrangular",
    "estrangulas": "estrangular",
    "estrangulase": "estrangular",
    "estrangulaseis": "estrangular",
    "estrangulasen": "estrangular",
    "estrangulases": "estrangular",
    "estrangulaste": "estrangular",
    "estrangulasteis": "estrangular",
    "estrangule": "estrangular",
    "estrangulemos": "estrangular",
    "estrangulen": "estrangular",
    "estrangules": "estrangular",
    "estrangulo": "estrangular",
    "estrangulábamos": "estrangular",
    "estranguláis": "estrangular",
    "estranguláramos": "estrangular",
    "estranguláremos": "estrangular",
    "estrangulásemos": "estrangular",
    "estrangulé": "estrangular",
    "estranguléis": "estrangular",
    "estranguló": "estrangular",
    "estrangurias": "estranguria",
    "estrapalucios": "estrapalucio",
    "estraperlea": "estraperlear",
    "estraperleaba": "estraperlear",
    "estraperleabais": "estraperlear",
    "estraperleaban": "estraperlear",
    "estraperleabas": "estraperlear",
    "estraperlead": "estraperlear",
    "estraperleada": "estraperlear",
    "estraperleadas": "estraperlear",
    "estraperleado": "estraperlear",
    "estraperleados": "estraperlear",
    "estraperleamos": "estraperlear",
    "estraperlean": "estraperlear",
    "estraperleando": "estraperlear",
    "estraperleara": "estraperlear",
    "estraperlearais": "estraperlear",
    "estraperlearan": "estraperlear",
    "estraperlearas": "estraperlear",
    "estraperleare": "estraperlear",
    "estraperleareis": "estraperlear",
    "estraperlearemos": "estraperlear",
    "estraperlearen": "estraperlear",
    "estraperleares": "estraperlear",
    "estraperlearon": "estraperlear",
    "estraperleará": "estraperlear",
    "estraperlearán": "estraperlear",
    "estraperlearás": "estraperlear",
    "estraperlearé": "estraperlear",
    "estraperlearéis": "estraperlear",
    "estraperlearía": "estraperlear",
    "estraperlearíais": "estraperlear",
    "estraperlearíamos": "estraperlear",
    "estraperlearían": "estraperlear",
    "estraperlearías": "estraperlear",
    "estraperleas": "estraperlear",
    "estraperlease": "estraperlear",
    "estraperleaseis": "estraperlear",
    "estraperleasen": "estraperlear",
    "estraperleases": "estraperlear",
    "estraperleaste": "estraperlear",
    "estraperleasteis": "estraperlear",
    "estraperlee": "estraperlear",
    "estraperleemos": "estraperlear",
    "estraperleen": "estraperlear",
    "estraperlees": "estraperlear",
    "estraperleo": "estraperlear",
    "estraperleábamos": "estraperlear",
    "estraperleáis": "estraperlear",
    "estraperleáramos": "estraperlear",
    "estraperleáremos": "estraperlear",
    "estraperleásemos": "estraperlear",
    "estraperleé": "estraperlear",
    "estraperleéis": "estraperlear",
    "estraperleó": "estraperlear",
    "estraperlistas": "estraperlista",
    "estraperlos": "estraperlo",
    "estrapontines": "estrapontín",
    "estratagemas": "estratagema",
    "estrategas": "estratega",
    "estrategias": "estrategia",
    "estratifica": "estratificar",
    "estratificaba": "estratificar",
    "estratificabais": "estratificar",
    "estratificaban": "estratificar",
    "estratificabas": "estratificar",
    "estratificaciones": "estratificación",
    "estratificad": "estratificar",
    "estratificada": "estratificar",
    "estratificadas": "estratificar",
    "estratificado": "estratificar",
    "estratificados": "estratificar",
    "estratificamos": "estratificar",
    "estratifican": "estratificar",
    "estratificando": "estratificar",
    "estratificara": "estratificar",
    "estratificarais": "estratificar",
    "estratificaran": "estratificar",
    "estratificaras": "estratificar",
    "estratificare": "estratificar",
    "estratificareis": "estratificar",
    "estratificaremos": "estratificar",
    "estratificaren": "estratificar",
    "estratificares": "estratificar",
    "estratificaron": "estratificar",
    "estratificará": "estratificar",
    "estratificarán": "estratificar",
    "estratificarás": "estratificar",
    "estratificaré": "estratificar",
    "estratificaréis": "estratificar",
    "estratificaría": "estratificar",
    "estratificaríais": "estratificar",
    "estratificaríamos": "estratificar",
    "estratificarían": "estratificar",
    "estratificarías": "estratificar",
    "estratificas": "estratificar",
    "estratificase": "estratificar",
    "estratificaseis": "estratificar",
    "estratificasen": "estratificar",
    "estratificases": "estratificar",
    "estratificaste": "estratificar",
    "estratificasteis": "estratificar",
    "estratifico": "estratificar",
    "estratificábamos": "estratificar",
    "estratificáis": "estratificar",
    "estratificáramos": "estratificar",
    "estratificáremos": "estratificar",
    "estratificásemos": "estratificar",
    "estratificó": "estratificar",
    "estratifique": "estratificar",
    "estratifiquemos": "estratificar",
    "estratifiquen": "estratificar",
    "estratifiques": "estratificar",
    "estratifiqué": "estratificar",
    "estratifiquéis": "estratificar",
    "estratigrafías": "estratigrafía",
    "estratigráfica": "estratigráfico",
    "estratigráficas": "estratigráfico",
    "estratigráficos": "estratigráfico",
    "estratocúmulos": "estratocúmulo",
    "estratopausas": "estratopausa",
    "estratos": "estrato",
    "estratosferas": "estratosfera",
    "estratosférica": "estratosférico",
    "estratosféricas": "estratosférico",
    "estratosféricos": "estratosférico",
    "estratovolcanes": "estratovolcán",
    "estratégica": "estratégico",
    "estratégicas": "estratégico",
    "estratégicos": "estratégico",
    "estratígrafa": "estratígrafo",
    "estratígrafas": "estratígrafo",
    "estratígrafos": "estratígrafo",
    "estratóstatos": "estratóstato",
    "estraves": "estrave",
    "estrazas": "estraza",
    "estrecha": "estrecho",
    "estrechaba": "estrechar",
    "estrechabais": "estrechar",
    "estrechaban": "estrechar",
    "estrechabas": "estrechar",
    "estrechad": "estrechar",
    "estrechada": "estrechar",
    "estrechadas": "estrechar",
    "estrechado": "estrechar",
    "estrechados": "estrechar",
    "estrechamientos": "estrechamiento",
    "estrechamos": "estrechar",
    "estrechan": "estrechar",
    "estrechando": "estrechar",
    "estrechara": "estrechar",
    "estrecharais": "estrechar",
    "estrecharan": "estrechar",
    "estrecharas": "estrechar",
    "estrechare": "estrechar",
    "estrechareis": "estrechar",
    "estrecharemos": "estrechar",
    "estrecharen": "estrechar",
    "estrechares": "estrechar",
    "estrecharon": "estrechar",
    "estrechará": "estrechar",
    "estrecharán": "estrechar",
    "estrecharás": "estrechar",
    "estrecharé": "estrechar",
    "estrecharéis": "estrechar",
    "estrecharía": "estrechar",
    "estrecharíais": "estrechar",
    "estrecharíamos": "estrechar",
    "estrecharían": "estrechar",
    "estrecharías": "estrechar",
    "estrechas": "estrecho",
    "estrechase": "estrechar",
    "estrechaseis": "estrechar",
    "estrechasen": "estrechar",
    "estrechases": "estrechar",
    "estrechaste": "estrechar",
    "estrechasteis": "estrechar",
    "estreche": "estrechar",
    "estrecheces": "estrechez",
    "estrechemos": "estrechar",
    "estrechen": "estrechar",
    "estreches": "estrechar",
    "estrecho": "estrechar",
    "estrechones": "estrechón",
    "estrechos": "estrecho",
    "estrechábamos": "estrechar",
    "estrecháis": "estrechar",
    "estrecháramos": "estrechar",
    "estrecháremos": "estrechar",
    "estrechásemos": "estrechar",
    "estreché": "estrechar",
    "estrechéis": "estrechar",
    "estrechó": "estrechar",
    "estregaba": "estregar",
    "estregabais": "estregar",
    "estregaban": "estregar",
    "estregabas": "estregar",
    "estregad": "estregar",
    "estregada": "estregar",
    "estregadas": "estregar",
    "estregaderas": "estregadera",
    "estregaderos": "estregadero",
    "estregado": "estregar",
    "estregados": "estregar",
    "estregaduras": "estregadura",
    "estregamientos": "estregamiento",
    "estregamos": "estregar",
    "estregando": "estregar",
    "estregara": "estregar",
    "estregarais": "estregar",
    "estregaran": "estregar",
    "estregaras": "estregar",
    "estregare": "estregar",
    "estregareis": "estregar",
    "estregaremos": "estregar",
    "estregaren": "estregar",
    "estregares": "estregar",
    "estregaron": "estregar",
    "estregará": "estregar",
    "estregarán": "estregar",
    "estregarás": "estregar",
    "estregaré": "estregar",
    "estregaréis": "estregar",
    "estregaría": "estregar",
    "estregaríais": "estregar",
    "estregaríamos": "estregar",
    "estregarían": "estregar",
    "estregarías": "estregar",
    "estregase": "estregar",
    "estregaseis": "estregar",
    "estregasen": "estregar",
    "estregases": "estregar",
    "estregaste": "estregar",
    "estregasteis": "estregar",
    "estreguemos": "estregar",
    "estregué": "estregar",
    "estreguéis": "estregar",
    "estregábamos": "estregar",
    "estregáis": "estregar",
    "estregáramos": "estregar",
    "estregáremos": "estregar",
    "estregásemos": "estregar",
    "estregó": "estregar",
    "estrella": "estrellar",
    "estrellaba": "estrellar",
    "estrellabais": "estrellar",
    "estrellaban": "estrellar",
    "estrellabas": "estrellar",
    "estrellad": "estrellar",
    "estrellada": "estrellar",
    "estrelladas": "estrellar",
    "estrelladeras": "estrelladera",
    "estrelladeros": "estrelladero",
    "estrellado": "estrellar",
    "estrellados": "estrellar",
    "estrellamares": "estrellamar",
    "estrellamos": "estrellar",
    "estrellan": "estrellar",
    "estrellando": "estrellar",
    "estrellara": "estrellar",
    "estrellarais": "estrellar",
    "estrellaran": "estrellar",
    "estrellaras": "estrellar",
    "estrellare": "estrellar",
    "estrellareis": "estrellar",
    "estrellaremos": "estrellar",
    "estrellaren": "estrellar",
    "estrellares": "estrellar",
    "estrellaron": "estrellar",
    "estrellará": "estrellar",
    "estrellarán": "estrellar",
    "estrellarás": "estrellar",
    "estrellaré": "estrellar",
    "estrellaréis": "estrellar",
    "estrellaría": "estrellar",
    "estrellaríais": "estrellar",
    "estrellaríamos": "estrellar",
    "estrellarían": "estrellar",
    "estrellarías": "estrellar",
    "estrellas": "estrellar",
    "estrellase": "estrellar",
    "estrellaseis": "estrellar",
    "estrellasen": "estrellar",
    "estrellases": "estrellar",
    "estrellaste": "estrellar",
    "estrellasteis": "estrellar",
    "estrellatos": "estrellato",
    "estrelle": "estrellar",
    "estrellemos": "estrellar",
    "estrellen": "estrellar",
    "estrellera": "estrellero",
    "estrelleras": "estrellero",
    "estrelleros": "estrellero",
    "estrelles": "estrellar",
    "estrello": "estrellar",
    "estrellones": "estrellón",
    "estrelluelas": "estrelluela",
    "estrellábamos": "estrellar",
    "estrelláis": "estrellar",
    "estrelláramos": "estrellar",
    "estrelláremos": "estrellar",
    "estrellásemos": "estrellar",
    "estrellé": "estrellar",
    "estrelléis": "estrellar",
    "estrelló": "estrellar",
    "estremece": "estremecer",
    "estremeced": "estremecer",
    "estremecedora": "estremecedor",
    "estremecedoras": "estremecedor",
    "estremecedores": "estremecedor",
    "estremecemos": "estremecer",
    "estremecen": "estremecer",
    "estremeceremos": "estremecer",
    "estremecerá": "estremecer",
    "estremecerán": "estremecer",
    "estremecerás": "estremecer",
    "estremeceré": "estremecer",
    "estremeceréis": "estremecer",
    "estremecería": "estremecer",
    "estremeceríais": "estremecer",
    "estremeceríamos": "estremecer",
    "estremecerían": "estremecer",
    "estremecerías": "estremecer",
    "estremeces": "estremecer",
    "estremecida": "estremecer",
    "estremecidas": "estremecer",
    "estremecido": "estremecer",
    "estremecidos": "estremecer",
    "estremeciendo": "estremecer",
    "estremeciera": "estremecer",
    "estremecierais": "estremecer",
    "estremecieran": "estremecer",
    "estremecieras": "estremecer",
    "estremeciere": "estremecer",
    "estremeciereis": "estremecer",
    "estremecieren": "estremecer",
    "estremecieres": "estremecer",
    "estremecieron": "estremecer",
    "estremeciese": "estremecer",
    "estremecieseis": "estremecer",
    "estremeciesen": "estremecer",
    "estremecieses": "estremecer",
    "estremecimientos": "estremecimiento",
    "estremecimos": "estremecer",
    "estremeciste": "estremecer",
    "estremecisteis": "estremecer",
    "estremeciéramos": "estremecer",
    "estremeciéremos": "estremecer",
    "estremeciésemos": "estremecer",
    "estremeció": "estremecer",
    "estremecéis": "estremecer",
    "estremecí": "estremecer",
    "estremecía": "estremecer",
    "estremecíais": "estremecer",
    "estremecíamos": "estremecer",
    "estremecían": "estremecer",
    "estremecías": "estremecer",
    "estremezca": "estremecer",
    "estremezcamos": "estremecer",
    "estremezcan": "estremecer",
    "estremezcas": "estremecer",
    "estremezco": "estremecer",
    "estremezcáis": "estremecer",
    "estrena": "estrenar",
    "estrenaba": "estrenar",
    "estrenabais": "estrenar",
    "estrenaban": "estrenar",
    "estrenabas": "estrenar",
    "estrenad": "estrenar",
    "estrenada": "estrenar",
    "estrenadas": "estrenar",
    "estrenado": "estrenar",
    "estrenados": "estrenar",
    "estrenamos": "estrenar",
    "estrenan": "estrenar",
    "estrenando": "estrenar",
    "estrenara": "estrenar",
    "estrenarais": "estrenar",
    "estrenaran": "estrenar",
    "estrenaras": "estrenar",
    "estrenare": "estrenar",
    "estrenareis": "estrenar",
    "estrenaremos": "estrenar",
    "estrenaren": "estrenar",
    "estrenares": "estrenar",
    "estrenaron": "estrenar",
    "estrenará": "estrenar",
    "estrenarán": "estrenar",
    "estrenarás": "estrenar",
    "estrenaré": "estrenar",
    "estrenaréis": "estrenar",
    "estrenaría": "estrenar",
    "estrenaríais": "estrenar",
    "estrenaríamos": "estrenar",
    "estrenarían": "estrenar",
    "estrenarías": "estrenar",
    "estrenas": "estrenar",
    "estrenase": "estrenar",
    "estrenaseis": "estrenar",
    "estrenasen": "estrenar",
    "estrenases": "estrenar",
    "estrenaste": "estrenar",
    "estrenasteis": "estrenar",
    "estrene": "estrenar",
    "estrenemos": "estrenar",
    "estrenen": "estrenar",
    "estrenes": "estrenar",
    "estrenistas": "estrenista",
    "estreno": "estrenar",
    "estrenos": "estreno",
    "estrenques": "estrenque",
    "estrenua": "estrenuo",
    "estrenuas": "estrenuo",
    "estrenuidades": "estrenuidad",
    "estrenuos": "estrenuo",
    "estrenábamos": "estrenar",
    "estrenáis": "estrenar",
    "estrenáramos": "estrenar",
    "estrenáremos": "estrenar",
    "estrenásemos": "estrenar",
    "estrené": "estrenar",
    "estrenéis": "estrenar",
    "estrenó": "estrenar",
    "estrepadas": "estrepada",
    "estrepitosa": "estrepitoso",
    "estrepitosas": "estrepitoso",
    "estrepitosos": "estrepitoso",
    "estrepsípteros": "estrepsíptero",
    "estreptococias": "estreptococia",
    "estreptococos": "estreptococo",
    "estreptocócica": "estreptocócico",
    "estreptocócicas": "estreptocócico",
    "estreptocócicos": "estreptocócico",
    "estreptodornasas": "estreptodornasa",
    "estreptoduocinas": "estreptoduocina",
    "estreptomicinas": "estreptomicina",
    "estreptoquinasas": "estreptoquinasa",
    "estreptotricinas": "estreptotricina",
    "estresantes": "estresante",
    "estreseses": "estrés",
    "estreñid": "estreñir",
    "estreñida": "estreñir",
    "estreñidas": "estreñir",
    "estreñido": "estreñir",
    "estreñidos": "estreñir",
    "estreñimientos": "estreñimiento",
    "estreñimos": "estreñir",
    "estreñiremos": "estreñir",
    "estreñirá": "estreñir",
    "estreñirán": "estreñir",
    "estreñirás": "estreñir",
    "estreñiré": "estreñir",
    "estreñiréis": "estreñir",
    "estreñiría": "estreñir",
    "estreñiríais": "estreñir",
    "estreñiríamos": "estreñir",
    "estreñirían": "estreñir",
    "estreñirías": "estreñir",
    "estreñiste": "estreñir",
    "estreñisteis": "estreñir",
    "estreñí": "estreñir",
    "estreñía": "estreñir",
    "estreñíais": "estreñir",
    "estreñíamos": "estreñir",
    "estreñían": "estreñir",
    "estreñías": "estreñir",
    "estreñís": "estreñir",
    "estriaba": "estriar",
    "estriabais": "estriar",
    "estriaban": "estriar",
    "estriabas": "estriar",
    "estriaciones": "estriación",
    "estriad": "estriar",
    "estriada": "estriar",
    "estriadas": "estriar",
    "estriado": "estriar",
    "estriados": "estriar",
    "estriamos": "estriar",
    "estriando": "estriar",
    "estriara": "estriar",
    "estriarais": "estriar",
    "estriaran": "estriar",
    "estriaras": "estriar",
    "estriare": "estriar",
    "estriareis": "estriar",
    "estriaremos": "estriar",
    "estriaren": "estriar",
    "estriares": "estriar",
    "estriaron": "estriar",
    "estriará": "estriar",
    "estriarán": "estriar",
    "estriarás": "estriar",
    "estriaré": "estriar",
    "estriaréis": "estriar",
    "estriaría": "estriar",
    "estriaríais": "estriar",
    "estriaríamos": "estriar",
    "estriarían": "estriar",
    "estriarías": "estriar",
    "estriase": "estriar",
    "estriaseis": "estriar",
    "estriasen": "estriar",
    "estriases": "estriar",
    "estriaste": "estriar",
    "estriasteis": "estriar",
    "estriba": "estribar",
    "estribaba": "estribar",
    "estribabais": "estribar",
    "estribaban": "estribar",
    "estribabas": "estribar",
    "estribaciones": "estribación",
    "estribad": "estribar",
    "estribada": "estribar",
    "estribadas": "estribar",
    "estribaderos": "estribadero",
    "estribado": "estribar",
    "estribados": "estribar",
    "estribamos": "estribar",
    "estriban": "estribar",
    "estribando": "estribar",
    "estribara": "estribar",
    "estribarais": "estribar",
    "estribaran": "estribar",
    "estribaras": "estribar",
    "estribare": "estribar",
    "estribareis": "estribar",
    "estribaremos": "estribar",
    "estribaren": "estribar",
    "estribares": "estribar",
    "estribaron": "estribar",
    "estribará": "estribar",
    "estribarán": "estribar",
    "estribarás": "estribar",
    "estribaré": "estribar",
    "estribaréis": "estribar",
    "estribaría": "estribar",
    "estribaríais": "estribar",
    "estribaríamos": "estribar",
    "estribarían": "estribar",
    "estribarías": "estribar",
    "estribas": "estribar",
    "estribase": "estribar",
    "estribaseis": "estribar",
    "estribasen": "estribar",
    "estribases": "estribar",
    "estribaste": "estribar",
    "estribasteis": "estribar",
    "estribe": "estribar",
    "estribemos": "estribar",
    "estriben": "estribar",
    "estriberas": "estribera",
    "estriberones": "estriberón",
    "estriberías": "estribería",
    "estribes": "estribar",
    "estribillos": "estribillo",
    "estribo": "estribar",
    "estribora": "estribor",
    "estriboras": "estribor",
    "estribores": "estribor",
    "estribos": "estribo",
    "estribotes": "estribote",
    "estribábamos": "estribar",
    "estribáis": "estribar",
    "estribáramos": "estribar",
    "estribáremos": "estribar",
    "estribásemos": "estribar",
    "estribé": "estribar",
    "estribéis": "estribar",
    "estribó": "estribar",
    "estricados": "estricado",
    "estricciones": "estricción",
    "estricninas": "estricnina",
    "estricta": "estricto",
    "estrictas": "estricto",
    "estrictos": "estricto",
    "estridencias": "estridencia",
    "estridentes": "estridente",
    "estridores": "estridor",
    "estridula": "estridular",
    "estridulaba": "estridular",
    "estridulabais": "estridular",
    "estridulaban": "estridular",
    "estridulabas": "estridular",
    "estridulad": "estridular",
    "estridulada": "estridular",
    "estriduladas": "estridular",
    "estridulado": "estridular",
    "estridulados": "estridular",
    "estridulamos": "estridular",
    "estridulan": "estridular",
    "estridulando": "estridular",
    "estridulara": "estridular",
    "estridularais": "estridular",
    "estridularan": "estridular",
    "estridularas": "estridular",
    "estridulare": "estridular",
    "estridulareis": "estridular",
    "estridularemos": "estridular",
    "estridularen": "estridular",
    "estridulares": "estridular",
    "estridularon": "estridular",
    "estridulará": "estridular",
    "estridularán": "estridular",
    "estridularás": "estridular",
    "estridularé": "estridular",
    "estridularéis": "estridular",
    "estridularía": "estridular",
    "estridularíais": "estridular",
    "estridularíamos": "estridular",
    "estridularían": "estridular",
    "estridularías": "estridular",
    "estridulas": "estridular",
    "estridulase": "estridular",
    "estridulaseis": "estridular",
    "estridulasen": "estridular",
    "estridulases": "estridular",
    "estridulaste": "estridular",
    "estridulasteis": "estridular",
    "estridule": "estridular",
    "estridulemos": "estridular",
    "estridulen": "estridular",
    "estridules": "estridular",
    "estridulo": "estridular",
    "estridulosa": "estriduloso",
    "estridulosas": "estriduloso",
    "estridulosos": "estriduloso",
    "estridulábamos": "estridular",
    "estriduláis": "estridular",
    "estriduláramos": "estridular",
    "estriduláremos": "estridular",
    "estridulásemos": "estridular",
    "estridulé": "estridular",
    "estriduléis": "estridular",
    "estriduló": "estridular",
    "estriega": "estregar",
    "estriegan": "estregar",
    "estriegas": "estregar",
    "estriego": "estregar",
    "estriegue": "estregar",
    "estrieguen": "estregar",
    "estriegues": "estregar",
    "estriemos": "estriar",
    "estrinques": "estrinque",
    "estrioles": "estriol",
    "estrioscopias": "estrioscopia",
    "estriábamos": "estriar",
    "estriáis": "estriar",
    "estriáramos": "estriar",
    "estriáremos": "estriar",
    "estriásemos": "estriar",
    "estrié": "estriar",
    "estriéis": "estriar",
    "estriña": "estreñir",
    "estriñamos": "estreñir",
    "estriñan": "estreñir",
    "estriñas": "estreñir",
    "estriñe": "estreñir",
    "estriñen": "estreñir",
    "estriñendo": "estreñir",
    "estriñera": "estreñir",
    "estriñerais": "estreñir",
    "estriñeran": "estreñir",
    "estriñeras": "estreñir",
    "estriñere": "estreñir",
    "estriñereis": "estreñir",
    "estriñeren": "estreñir",
    "estriñeres": "estreñir",
    "estriñeron": "estreñir",
    "estriñes": "estreñir",
    "estriñese": "estreñir",
    "estriñeseis": "estreñir",
    "estriñesen": "estreñir",
    "estriñeses": "estreñir",
    "estriño": "estreñir",
    "estriñáis": "estreñir",
    "estriñéramos": "estreñir",
    "estriñéremos": "estreñir",
    "estriñésemos": "estreñir",
    "estriñó": "estreñir",
    "estrió": "estriar",
    "estrobometrías": "estrobometría",
    "estrobos": "estrobo",
    "estroboscopias": "estroboscopia",
    "estroboscopios": "estroboscopio",
    "estroboscópica": "estroboscópico",
    "estroboscópicas": "estroboscópico",
    "estroboscópicos": "estroboscópico",
    "estrofantinas": "estrofantina",
    "estrofas": "estrofa",
    "estrofoides": "estrofoide",
    "estromas": "estroma",
    "estromatolitos": "estromatolito",
    "estromboliana": "estromboliano",
    "estrombolianas": "estromboliano",
    "estrombolianos": "estromboliano",
    "estroncianitas": "estroncianita",
    "estroncios": "estroncio",
    "estrones": "estrón",
    "estropadas": "estropada",
    "estropajeos": "estropajeo",
    "estropajos": "estropajo",
    "estropajosa": "estropajoso",
    "estropajosas": "estropajoso",
    "estropajosos": "estropajoso",
    "estropea": "estropear",
    "estropeaba": "estropear",
    "estropeabais": "estropear",
    "estropeaban": "estropear",
    "estropeabas": "estropear",
    "estropead": "estropear",
    "estropeada": "estropear",
    "estropeadas": "estropear",
    "estropeado": "estropear",
    "estropeados": "estropear",
    "estropeamientos": "estropeamiento",
    "estropeamos": "estropear",
    "estropean": "estropear",
    "estropeando": "estropear",
    "estropeara": "estropear",
    "estropearais": "estropear",
    "estropearan": "estropear",
    "estropearas": "estropear",
    "estropeare": "estropear",
    "estropeareis": "estropear",
    "estropearemos": "estropear",
    "estropearen": "estropear",
    "estropeares": "estropear",
    "estropearon": "estropear",
    "estropeará": "estropear",
    "estropearán": "estropear",
    "estropearás": "estropear",
    "estropearé": "estropear",
    "estropearéis": "estropear",
    "estropearía": "estropear",
    "estropearíais": "estropear",
    "estropearíamos": "estropear",
    "estropearían": "estropear",
    "estropearías": "estropear",
    "estropeas": "estropear",
    "estropease": "estropear",
    "estropeaseis": "estropear",
    "estropeasen": "estropear",
    "estropeases": "estropear",
    "estropeaste": "estropear",
    "estropeasteis": "estropear",
    "estropee": "estropear",
    "estropeemos": "estropear",
    "estropeen": "estropear",
    "estropees": "estropear",
    "estropeo": "estropear",
    "estropeos": "estropeo",
    "estropeábamos": "estropear",
    "estropeáis": "estropear",
    "estropeáramos": "estropear",
    "estropeáremos": "estropear",
    "estropeásemos": "estropear",
    "estropeé": "estropear",
    "estropeéis": "estropear",
    "estropeó": "estropear",
    "estropicios": "estropicio",
    "estros": "estro",
    "estrucioniformes": "estrucioniforme",
    "estructura": "estructurar",
    "estructuraba": "estructurar",
    "estructurabais": "estructurar",
    "estructuraban": "estructurar",
    "estructurabas": "estructurar",
    "estructurables": "estructurable",
    "estructuraciones": "estructuración",
    "estructurad": "estructurar",
    "estructurada": "estructurar",
    "estructuradas": "estructurar",
    "estructurado": "estructurar",
    "estructurados": "estructurar",
    "estructurales": "estructural",
    "estructuralismos": "estructuralismo",
    "estructuralistas": "estructuralista",
    "estructuramos": "estructurar",
    "estructuran": "estructurar",
    "estructurando": "estructurar",
    "estructurara": "estructurar",
    "estructurarais": "estructurar",
    "estructuraran": "estructurar",
    "estructuraras": "estructurar",
    "estructurare": "estructurar",
    "estructurareis": "estructurar",
    "estructuraremos": "estructurar",
    "estructuraren": "estructurar",
    "estructurares": "estructurar",
    "estructuraron": "estructurar",
    "estructurará": "estructurar",
    "estructurarán": "estructurar",
    "estructurarás": "estructurar",
    "estructuraré": "estructurar",
    "estructuraréis": "estructurar",
    "estructuraría": "estructurar",
    "estructuraríais": "estructurar",
    "estructuraríamos": "estructurar",
    "estructurarían": "estructurar",
    "estructurarías": "estructurar",
    "estructuras": "estructurar",
    "estructurase": "estructurar",
    "estructuraseis": "estructurar",
    "estructurasen": "estructurar",
    "estructurases": "estructurar",
    "estructuraste": "estructurar",
    "estructurasteis": "estructurar",
    "estructure": "estructurar",
    "estructuremos": "estructurar",
    "estructuren": "estructurar",
    "estructures": "estructurar",
    "estructuro": "estructurar",
    "estructurábamos": "estructurar",
    "estructuráis": "estructurar",
    "estructuráramos": "estructurar",
    "estructuráremos": "estructurar",
    "estructurásemos": "estructurar",
    "estructuré": "estructurar",
    "estructuréis": "estructurar",
    "estructuró": "estructurar",
    "estruendos": "estruendo",
    "estruendosa": "estruendoso",
    "estruendosas": "estruendoso",
    "estruendosos": "estruendoso",
    "estruja": "estrujar",
    "estrujaba": "estrujar",
    "estrujabais": "estrujar",
    "estrujaban": "estrujar",
    "estrujabas": "estrujar",
    "estrujad": "estrujar",
    "estrujada": "estrujar",
    "estrujadas": "estrujar",
    "estrujado": "estrujar",
    "estrujadora": "estrujador",
    "estrujadoras": "estrujador",
    "estrujadores": "estrujador",
    "estrujados": "estrujar",
    "estrujaduras": "estrujadura",
    "estrujamientos": "estrujamiento",
    "estrujamos": "estrujar",
    "estrujan": "estrujar",
    "estrujando": "estrujar",
    "estrujara": "estrujar",
    "estrujarais": "estrujar",
    "estrujaran": "estrujar",
    "estrujaras": "estrujar",
    "estrujare": "estrujar",
    "estrujareis": "estrujar",
    "estrujaremos": "estrujar",
    "estrujaren": "estrujar",
    "estrujares": "estrujar",
    "estrujaron": "estrujar",
    "estrujará": "estrujar",
    "estrujarán": "estrujar",
    "estrujarás": "estrujar",
    "estrujaré": "estrujar",
    "estrujaréis": "estrujar",
    "estrujaría": "estrujar",
    "estrujaríais": "estrujar",
    "estrujaríamos": "estrujar",
    "estrujarían": "estrujar",
    "estrujarías": "estrujar",
    "estrujas": "estrujar",
    "estrujase": "estrujar",
    "estrujaseis": "estrujar",
    "estrujasen": "estrujar",
    "estrujases": "estrujar",
    "estrujaste": "estrujar",
    "estrujasteis": "estrujar",
    "estruje": "estrujar",
    "estrujemos": "estrujar",
    "estrujen": "estrujar",
    "estrujes": "estrujar",
    "estrujo": "estrujar",
    "estrujones": "estrujón",
    "estrujábamos": "estrujar",
    "estrujáis": "estrujar",
    "estrujáramos": "estrujar",
    "estrujáremos": "estrujar",
    "estrujásemos": "estrujar",
    "estrujé": "estrujar",
    "estrujéis": "estrujar",
    "estrujó": "estrujar",
    "estrábica": "estrábico",
    "estrábicas": "estrábico",
    "estrábicos": "estrábico",
    "estrépitos": "estrépito",
    "estría": "estriar",
    "estrían": "estriar",
    "estrías": "estriar",
    "estríe": "estriar",
    "estríen": "estriar",
    "estríes": "estriar",
    "estrígidos": "estrígido",
    "estrío": "estriar",
    "estróbilos": "estróbilo",
    "estrófica": "estrófico",
    "estróficas": "estrófico",
    "estróficos": "estrófico",
    "estrógena": "estrógeno",
    "estrógenas": "estrógeno",
    "estrógenos": "estrógeno",
    "estuarios": "estuario",
    "estuca": "estucar",
    "estucaba": "estucar",
    "estucabais": "estucar",
    "estucaban": "estucar",
    "estucabas": "estucar",
    "estucad": "estucar",
    "estucada": "estucar",
    "estucadas": "estucar",
    "estucado": "estucar",
    "estucadora": "estucador",
    "estucadoras": "estucador",
    "estucadores": "estucador",
    "estucados": "estucar",
    "estucamos": "estucar",
    "estucan": "estucar",
    "estucando": "estucar",
    "estucara": "estucar",
    "estucarais": "estucar",
    "estucaran": "estucar",
    "estucaras": "estucar",
    "estucare": "estucar",
    "estucareis": "estucar",
    "estucaremos": "estucar",
    "estucaren": "estucar",
    "estucares": "estucar",
    "estucaron": "estucar",
    "estucará": "estucar",
    "estucarán": "estucar",
    "estucarás": "estucar",
    "estucaré": "estucar",
    "estucaréis": "estucar",
    "estucaría": "estucar",
    "estucaríais": "estucar",
    "estucaríamos": "estucar",
    "estucarían": "estucar",
    "estucarías": "estucar",
    "estucas": "estucar",
    "estucase": "estucar",
    "estucaseis": "estucar",
    "estucasen": "estucar",
    "estucases": "estucar",
    "estucaste": "estucar",
    "estucasteis": "estucar",
    "estucha": "estuchar",
    "estuchaba": "estuchar",
    "estuchabais": "estuchar",
    "estuchaban": "estuchar",
    "estuchabas": "estuchar",
    "estuchad": "estuchar",
    "estuchada": "estuchar",
    "estuchadas": "estuchar",
    "estuchado": "estuchar",
    "estuchados": "estuchar",
    "estuchamos": "estuchar",
    "estuchan": "estuchar",
    "estuchando": "estuchar",
    "estuchara": "estuchar",
    "estucharais": "estuchar",
    "estucharan": "estuchar",
    "estucharas": "estuchar",
    "estuchare": "estuchar",
    "estuchareis": "estuchar",
    "estucharemos": "estuchar",
    "estucharen": "estuchar",
    "estuchares": "estuchar",
    "estucharon": "estuchar",
    "estuchará": "estuchar",
    "estucharán": "estuchar",
    "estucharás": "estuchar",
    "estucharé": "estuchar",
    "estucharéis": "estuchar",
    "estucharía": "estuchar",
    "estucharíais": "estuchar",
    "estucharíamos": "estuchar",
    "estucharían": "estuchar",
    "estucharías": "estuchar",
    "estuchas": "estuchar",
    "estuchase": "estuchar",
    "estuchaseis": "estuchar",
    "estuchasen": "estuchar",
    "estuchases": "estuchar",
    "estuchaste": "estuchar",
    "estuchasteis": "estuchar",
    "estuche": "estuchar",
    "estuchemos": "estuchar",
    "estuchen": "estuchar",
    "estuches": "estuche",
    "estuchistas": "estuchista",
    "estucho": "estuchar",
    "estuchábamos": "estuchar",
    "estucháis": "estuchar",
    "estucháramos": "estuchar",
    "estucháremos": "estuchar",
    "estuchásemos": "estuchar",
    "estuché": "estuchar",
    "estuchéis": "estuchar",
    "estuchó": "estuchar",
    "estuco": "estucar",
    "estucos": "estuco",
    "estucábamos": "estucar",
    "estucáis": "estucar",
    "estucáramos": "estucar",
    "estucáremos": "estucar",
    "estucásemos": "estucar",
    "estucó": "estucar",
    "estudia": "estudiar",
    "estudiaba": "estudiar",
    "estudiabais": "estudiar",
    "estudiaban": "estudiar",
    "estudiabas": "estudiar",
    "estudiad": "estudiar",
    "estudiada": "estudiar",
    "estudiadas": "estudiar",
    "estudiado": "estudiar",
    "estudiados": "estudiar",
    "estudiamos": "estudiar",
    "estudian": "estudiar",
    "estudiando": "estudiar",
    "estudiantados": "estudiantado",
    "estudiantes": "estudiante",
    "estudiantiles": "estudiantil",
    "estudiantina": "estudiantino",
    "estudiantinas": "estudiantino",
    "estudiantinos": "estudiantino",
    "estudiara": "estudiar",
    "estudiarais": "estudiar",
    "estudiaran": "estudiar",
    "estudiaras": "estudiar",
    "estudiare": "estudiar",
    "estudiareis": "estudiar",
    "estudiaremos": "estudiar",
    "estudiaren": "estudiar",
    "estudiares": "estudiar",
    "estudiaron": "estudiar",
    "estudiará": "estudiar",
    "estudiarán": "estudiar",
    "estudiarás": "estudiar",
    "estudiaré": "estudiar",
    "estudiaréis": "estudiar",
    "estudiaría": "estudiar",
    "estudiaríais": "estudiar",
    "estudiaríamos": "estudiar",
    "estudiarían": "estudiar",
    "estudiarías": "estudiar",
    "estudias": "estudiar",
    "estudiase": "estudiar",
    "estudiaseis": "estudiar",
    "estudiasen": "estudiar",
    "estudiases": "estudiar",
    "estudiaste": "estudiar",
    "estudiasteis": "estudiar",
    "estudie": "estudiar",
    "estudiemos": "estudiar",
    "estudien": "estudiar",
    "estudies": "estudiar",
    "estudio": "estudiar",
    "estudios": "estudio",
    "estudiosa": "estudioso",
    "estudiosas": "estudioso",
    "estudiosidades": "estudiosidad",
    "estudiosos": "estudioso",
    "estudiábamos": "estudiar",
    "estudiáis": "estudiar",
    "estudiáramos": "estudiar",
    "estudiáremos": "estudiar",
    "estudiásemos": "estudiar",
    "estudié": "estudiar",
    "estudiéis": "estudiar",
    "estudió": "estudiar",
    "estufa": "estufar",
    "estufaba": "estufar",
    "estufabais": "estufar",
    "estufaban": "estufar",
    "estufabas": "estufar",
    "estufad": "estufar",
    "estufada": "estufar",
    "estufadas": "estufar",
    "estufado": "estufar",
    "estufados": "estufar",
    "estufamos": "estufar",
    "estufan": "estufar",
    "estufando": "estufar",
    "estufara": "estufar",
    "estufarais": "estufar",
    "estufaran": "estufar",
    "estufaras": "estufar",
    "estufare": "estufar",
    "estufareis": "estufar",
    "estufaremos": "estufar",
    "estufaren": "estufar",
    "estufares": "estufar",
    "estufaron": "estufar",
    "estufará": "estufar",
    "estufarán": "estufar",
    "estufarás": "estufar",
    "estufaré": "estufar",
    "estufaréis": "estufar",
    "estufaría": "estufar",
    "estufaríais": "estufar",
    "estufaríamos": "estufar",
    "estufarían": "estufar",
    "estufarías": "estufar",
    "estufas": "estufar",
    "estufase": "estufar",
    "estufaseis": "estufar",
    "estufasen": "estufar",
    "estufases": "estufar",
    "estufaste": "estufar",
    "estufasteis": "estufar",
    "estufe": "estufar",
    "estufemos": "estufar",
    "estufen": "estufar",
    "estufera": "estufero",
    "estuferas": "estufero",
    "estuferos": "estufero",
    "estufes": "estufar",
    "estufillas": "estufilla",
    "estufistas": "estufista",
    "estufo": "estufar",
    "estufábamos": "estufar",
    "estufáis": "estufar",
    "estufáramos": "estufar",
    "estufáremos": "estufar",
    "estufásemos": "estufar",
    "estufé": "estufar",
    "estuféis": "estufar",
    "estufó": "estufar",
    "estulta": "estulto",
    "estultas": "estulto",
    "estulticias": "estulticia",
    "estultos": "estulto",
    "estupefacciones": "estupefacción",
    "estupefacientes": "estupefaciente",
    "estupefacta": "estupefacto",
    "estupefactas": "estupefacto",
    "estupefactiva": "estupefactivo",
    "estupefactivas": "estupefactivo",
    "estupefactivos": "estupefactivo",
    "estupefactos": "estupefacto",
    "estupenda": "estupendo",
    "estupendas": "estupendo",
    "estupendos": "estupendo",
    "estupideces": "estupidez",
    "estupores": "estupor",
    "estuporosa": "estuporoso",
    "estuporosas": "estuporoso",
    "estuporosos": "estuporoso",
    "estupra": "estuprar",
    "estupraba": "estuprar",
    "estuprabais": "estuprar",
    "estupraban": "estuprar",
    "estuprabas": "estuprar",
    "estuprad": "estuprar",
    "estuprada": "estuprar",
    "estupradas": "estuprar",
    "estuprado": "estuprar",
    "estupradora": "estuprador",
    "estupradoras": "estuprador",
    "estupradores": "estuprador",
    "estuprados": "estuprar",
    "estupramos": "estuprar",
    "estupran": "estuprar",
    "estuprando": "estuprar",
    "estuprara": "estuprar",
    "estuprarais": "estuprar",
    "estupraran": "estuprar",
    "estupraras": "estuprar",
    "estuprare": "estuprar",
    "estuprareis": "estuprar",
    "estupraremos": "estuprar",
    "estupraren": "estuprar",
    "estuprares": "estuprar",
    "estupraron": "estuprar",
    "estuprará": "estuprar",
    "estuprarán": "estuprar",
    "estuprarás": "estuprar",
    "estupraré": "estuprar",
    "estupraréis": "estuprar",
    "estupraría": "estuprar",
    "estupraríais": "estuprar",
    "estupraríamos": "estuprar",
    "estuprarían": "estuprar",
    "estuprarías": "estuprar",
    "estupras": "estuprar",
    "estuprase": "estuprar",
    "estupraseis": "estuprar",
    "estuprasen": "estuprar",
    "estuprases": "estuprar",
    "estupraste": "estuprar",
    "estuprasteis": "estuprar",
    "estupre": "estuprar",
    "estupremos": "estuprar",
    "estupren": "estuprar",
    "estupres": "estuprar",
    "estupro": "estuprar",
    "estupros": "estupro",
    "estuprábamos": "estuprar",
    "estupráis": "estuprar",
    "estupráramos": "estuprar",
    "estupráremos": "estuprar",
    "estuprásemos": "estuprar",
    "estupré": "estuprar",
    "estupréis": "estuprar",
    "estupró": "estuprar",
    "estuque": "estucar",
    "estuquemos": "estucar",
    "estuquen": "estucar",
    "estuquerías": "estuquería",
    "estuques": "estucar",
    "estuquistas": "estuquista",
    "estuqué": "estucar",
    "estuquéis": "estucar",
    "esturiones": "esturión",
    "estuve": "estar",
    "estuviera": "estar",
    "estuvierais": "estar",
    "estuvieran": "estar",
    "estuvieras": "estar",
    "estuviere": "estar",
    "estuviereis": "estar",
    "estuvieren": "estar",
    "estuvieres": "estar",
    "estuvieron": "estar",
    "estuviese": "estar",
    "estuvieseis": "estar",
    "estuviesen": "estar",
    "estuvieses": "estar",
    "estuvimos": "estar",
    "estuviste": "estar",
    "estuvisteis": "estar",
    "estuviéramos": "estar",
    "estuviéremos": "estar",
    "estuviésemos": "estar",
    "estuvo": "estar",
    "está": "estar",
    "estábamos": "estar",
    "estáis": "estar",
    "están": "estar",
    "estándares": "estándar",
    "estándarses": "estándar",
    "estánnica": "estánnico",
    "estánnicas": "estánnico",
    "estánnicos": "estánnico",
    "estárteres": "estárter",
    "estás": "estar",
    "estática": "estático",
    "estáticas": "estático",
    "estáticos": "estático",
    "esté": "estar",
    "estéis": "estar",
    "estén": "estar",
    "estérea": "estéreo",
    "estéreas": "estéreo",
    "estéreos": "estéreo",
    "estérica": "estérico",
    "estéricas": "estérico",
    "estéricos": "estérico",
    "estériles": "estéril",
    "estés": "estar",
    "estética": "estético",
    "estéticas": "estético",
    "estéticos": "estético",
    "estífnica": "estífnico",
    "estífnicas": "estífnico",
    "estífnicos": "estífnico",
    "estímulos": "estímulo",
    "estíos": "estío",
    "estípites": "estípite",
    "estíptica": "estíptico",
    "estípticas": "estíptico",
    "estípticos": "estíptico",
    "estípulas": "estípula",
    "estítica": "estítico",
    "estíticas": "estítico",
    "estíticos": "estítico",
    "estólida": "estólido",
    "estólidas": "estólido",
    "estólidos": "estólido",
    "estómagos": "estómago",
    "estúpida": "estúpido",
    "estúpidas": "estúpido",
    "estúpidos": "estúpido",
    "estúrnidos": "estúrnido",
    "esvarones": "esvarón",
    "esviajes": "esviaje",
    "esvásticas": "esvástica",
    "esófagos": "esófago",
    "etalajes": "etalaje",
    "etalones": "etalón",
    "etanales": "etanal",
    "etanoica": "etanoico",
    "etanoicas": "etanoico",
    "etanoicos": "etanoico",
    "etanolaminas": "etanolamina",
    "etanoles": "etanol",
    "etanos": "etano",
    "etapas": "etapa",
    "etarras": "etarra",
    "etas": "eta",
    "etc": "etcétera",
    "etc.": "etcétera",
    "etcéteras": "etcétera",
    "etenoides": "etenoide",
    "etenos": "eteno",
    "eterice": "eterizar",
    "etericemos": "eterizar",
    "etericen": "eterizar",
    "eterices": "eterizar",
    "etericé": "eterizar",
    "etericéis": "eterizar",
    "eterifica": "eterificar",
    "eterificaba": "eterificar",
    "eterificabais": "eterificar",
    "eterificaban": "eterificar",
    "eterificabas": "eterificar",
    "eterificaciones": "eterificación",
    "eterificad": "eterificar",
    "eterificada": "eterificar",
    "eterificadas": "eterificar",
    "eterificado": "eterificar",
    "eterificados": "eterificar",
    "eterificamos": "eterificar",
    "eterifican": "eterificar",
    "eterificando": "eterificar",
    "eterificara": "eterificar",
    "eterificarais": "eterificar",
    "eterificaran": "eterificar",
    "eterificaras": "eterificar",
    "eterificare": "eterificar",
    "eterificareis": "eterificar",
    "eterificaremos": "eterificar",
    "eterificaren": "eterificar",
    "eterificares": "eterificar",
    "eterificaron": "eterificar",
    "eterificará": "eterificar",
    "eterificarán": "eterificar",
    "eterificarás": "eterificar",
    "eterificaré": "eterificar",
    "eterificaréis": "eterificar",
    "eterificaría": "eterificar",
    "eterificaríais": "eterificar",
    "eterificaríamos": "eterificar",
    "eterificarían": "eterificar",
    "eterificarías": "eterificar",
    "eterificas": "eterificar",
    "eterificase": "eterificar",
    "eterificaseis": "eterificar",
    "eterificasen": "eterificar",
    "eterificases": "eterificar",
    "eterificaste": "eterificar",
    "eterificasteis": "eterificar",
    "eterifico": "eterificar",
    "eterificábamos": "eterificar",
    "eterificáis": "eterificar",
    "eterificáramos": "eterificar",
    "eterificáremos": "eterificar",
    "eterificásemos": "eterificar",
    "eterificó": "eterificar",
    "eterifique": "eterificar",
    "eterifiquemos": "eterificar",
    "eterifiquen": "eterificar",
    "eterifiques": "eterificar",
    "eterifiqué": "eterificar",
    "eterifiquéis": "eterificar",
    "eterismos": "eterismo",
    "eteriza": "eterizar",
    "eterizaba": "eterizar",
    "eterizabais": "eterizar",
    "eterizaban": "eterizar",
    "eterizabas": "eterizar",
    "eterizaciones": "eterización",
    "eterizad": "eterizar",
    "eterizada": "eterizar",
    "eterizadas": "eterizar",
    "eterizado": "eterizar",
    "eterizados": "eterizar",
    "eterizamos": "eterizar",
    "eterizan": "eterizar",
    "eterizando": "eterizar",
    "eterizara": "eterizar",
    "eterizarais": "eterizar",
    "eterizaran": "eterizar",
    "eterizaras": "eterizar",
    "eterizare": "eterizar",
    "eterizareis": "eterizar",
    "eterizaremos": "eterizar",
    "eterizaren": "eterizar",
    "eterizares": "eterizar",
    "eterizaron": "eterizar",
    "eterizará": "eterizar",
    "eterizarán": "eterizar",
    "eterizarás": "eterizar",
    "eterizaré": "eterizar",
    "eterizaréis": "eterizar",
    "eterizaría": "eterizar",
    "eterizaríais": "eterizar",
    "eterizaríamos": "eterizar",
    "eterizarían": "eterizar",
    "eterizarías": "eterizar",
    "eterizas": "eterizar",
    "eterizase": "eterizar",
    "eterizaseis": "eterizar",
    "eterizasen": "eterizar",
    "eterizases": "eterizar",
    "eterizaste": "eterizar",
    "eterizasteis": "eterizar",
    "eterizo": "eterizar",
    "eterizábamos": "eterizar",
    "eterizáis": "eterizar",
    "eterizáramos": "eterizar",
    "eterizáremos": "eterizar",
    "eterizásemos": "eterizar",
    "eterizó": "eterizar",
    "eterna": "eterno",
    "eternas": "eterno",
    "eternice": "eternizar",
    "eternicemos": "eternizar",
    "eternicen": "eternizar",
    "eternices": "eternizar",
    "eternicé": "eternizar",
    "eternicéis": "eternizar",
    "eternidades": "eternidad",
    "eterniza": "eternizar",
    "eternizaba": "eternizar",
    "eternizabais": "eternizar",
    "eternizaban": "eternizar",
    "eternizabas": "eternizar",
    "eternizables": "eternizable",
    "eternizaciones": "eternización",
    "eternizad": "eternizar",
    "eternizada": "eternizar",
    "eternizadas": "eternizar",
    "eternizado": "eternizar",
    "eternizados": "eternizar",
    "eternizamos": "eternizar",
    "eternizan": "eternizar",
    "eternizando": "eternizar",
    "eternizara": "eternizar",
    "eternizarais": "eternizar",
    "eternizaran": "eternizar",
    "eternizaras": "eternizar",
    "eternizare": "eternizar",
    "eternizareis": "eternizar",
    "eternizaremos": "eternizar",
    "eternizaren": "eternizar",
    "eternizares": "eternizar",
    "eternizaron": "eternizar",
    "eternizará": "eternizar",
    "eternizarán": "eternizar",
    "eternizarás": "eternizar",
    "eternizaré": "eternizar",
    "eternizaréis": "eternizar",
    "eternizaría": "eternizar",
    "eternizaríais": "eternizar",
    "eternizaríamos": "eternizar",
    "eternizarían": "eternizar",
    "eternizarías": "eternizar",
    "eternizas": "eternizar",
    "eternizase": "eternizar",
    "eternizaseis": "eternizar",
    "eternizasen": "eternizar",
    "eternizases": "eternizar",
    "eternizaste": "eternizar",
    "eternizasteis": "eternizar",
    "eternizo": "eternizar",
    "eternizábamos": "eternizar",
    "eternizáis": "eternizar",
    "eternizáramos": "eternizar",
    "eternizáremos": "eternizar",
    "eternizásemos": "eternizar",
    "eternizó": "eternizar",
    "eternos": "eterno",
    "eteromanías": "eteromanía",
    "etesia": "etesio",
    "etesias": "etesio",
    "etesios": "etesio",
    "etilaciones": "etilación",
    "etilaminas": "etilamina",
    "etilarseniosa": "etilarsenioso",
    "etilarseniosas": "etilarsenioso",
    "etilarseniosos": "etilarsenioso",
    "etilatos": "etilato",
    "etilbencenos": "etilbenceno",
    "etilbencilanilinas": "etilbencilanilina",
    "etilbutiraldehídos": "etilbutiraldehído",
    "etilbutírica": "etilbutírico",
    "etilbutíricas": "etilbutírico",
    "etilbutíricos": "etilbutírico",
    "etilcarbonatos": "etilcarbonato",
    "etilcelulosas": "etilcelulosa",
    "etilciclohexanos": "etilciclohexano",
    "etildiclorosilanos": "etildiclorosilano",
    "etildietanolaminas": "etildietanolamina",
    "etilenbisditiocarbamidas": "etilenbisditiocarbamida",
    "etilendiaminas": "etilendiamina",
    "etilenglicoles": "etilenglicol",
    "etileniminas": "etilenimina",
    "etilenos": "etileno",
    "etiletanolaminas": "etiletanolamina",
    "etilfosfórica": "etilfosfórico",
    "etilfosfóricas": "etilfosfórico",
    "etilfosfóricos": "etilfosfórico",
    "etilhexilaminas": "etilhexilamina",
    "etilhexoatos": "etilhexoato",
    "etilhexoica": "etilhexoico",
    "etilhexoicas": "etilhexoico",
    "etilhexoicos": "etilhexoico",
    "etilismos": "etilismo",
    "etilmercúrica": "etilmercúrico",
    "etilmercúricas": "etilmercúrico",
    "etilmercúricos": "etilmercúrico",
    "etilos": "etilo",
    "etilsulfatos": "etilsulfato",
    "etilsulfurosa": "etilsulfuroso",
    "etilsulfurosas": "etilsulfuroso",
    "etilsulfurosos": "etilsulfuroso",
    "etilsulfúrica": "etilsulfúrico",
    "etilsulfúricas": "etilsulfúrico",
    "etilsulfúricos": "etilsulfúrico",
    "etiltriclorosilanos": "etiltriclorosilano",
    "etilénica": "etilénico",
    "etilénicas": "etilénico",
    "etilénicos": "etilénico",
    "etilómetros": "etilómetro",
    "etimologice": "etimologizar",
    "etimologicemos": "etimologizar",
    "etimologicen": "etimologizar",
    "etimologices": "etimologizar",
    "etimologicé": "etimologizar",
    "etimologicéis": "etimologizar",
    "etimologistas": "etimologista",
    "etimologiza": "etimologizar",
    "etimologizaba": "etimologizar",
    "etimologizabais": "etimologizar",
    "etimologizaban": "etimologizar",
    "etimologizabas": "etimologizar",
    "etimologizad": "etimologizar",
    "etimologizada": "etimologizar",
    "etimologizadas": "etimologizar",
    "etimologizado": "etimologizar",
    "etimologizados": "etimologizar",
    "etimologizamos": "etimologizar",
    "etimologizan": "etimologizar",
    "etimologizando": "etimologizar",
    "etimologizara": "etimologizar",
    "etimologizarais": "etimologizar",
    "etimologizaran": "etimologizar",
    "etimologizaras": "etimologizar",
    "etimologizare": "etimologizar",
    "etimologizareis": "etimologizar",
    "etimologizaremos": "etimologizar",
    "etimologizaren": "etimologizar",
    "etimologizares": "etimologizar",
    "etimologizaron": "etimologizar",
    "etimologizará": "etimologizar",
    "etimologizarán": "etimologizar",
    "etimologizarás": "etimologizar",
    "etimologizaré": "etimologizar",
    "etimologizaréis": "etimologizar",
    "etimologizaría": "etimologizar",
    "etimologizaríais": "etimologizar",
    "etimologizaríamos": "etimologizar",
    "etimologizarían": "etimologizar",
    "etimologizarías": "etimologizar",
    "etimologizas": "etimologizar",
    "etimologizase": "etimologizar",
    "etimologizaseis": "etimologizar",
    "etimologizasen": "etimologizar",
    "etimologizases": "etimologizar",
    "etimologizaste": "etimologizar",
    "etimologizasteis": "etimologizar",
    "etimologizo": "etimologizar",
    "etimologizábamos": "etimologizar",
    "etimologizáis": "etimologizar",
    "etimologizáramos": "etimologizar",
    "etimologizáremos": "etimologizar",
    "etimologizásemos": "etimologizar",
    "etimologizó": "etimologizar",
    "etimologías": "etimología",
    "etimológica": "etimológico",
    "etimológicas": "etimológico",
    "etimológicos": "etimológico",
    "etimóloga": "etimólogo",
    "etimólogas": "etimólogo",
    "etimólogos": "etimólogo",
    "etiolaciones": "etiolación",
    "etiologías": "etiología",
    "etiológica": "etiológico",
    "etiológicas": "etiológico",
    "etiológicos": "etiológico",
    "etiopes": "etiope",
    "etiopia": "etiopio",
    "etiopias": "etiopio",
    "etiopios": "etiopio",
    "etioplastos": "etioplasto",
    "etiqueta": "etiquetar",
    "etiquetaba": "etiquetar",
    "etiquetabais": "etiquetar",
    "etiquetaban": "etiquetar",
    "etiquetabas": "etiquetar",
    "etiquetaciones": "etiquetación",
    "etiquetad": "etiquetar",
    "etiquetada": "etiquetar",
    "etiquetadas": "etiquetar",
    "etiquetado": "etiquetar",
    "etiquetadora": "etiquetador",
    "etiquetadoras": "etiquetador",
    "etiquetadores": "etiquetador",
    "etiquetados": "etiquetar",
    "etiquetajes": "etiquetaje",
    "etiquetamos": "etiquetar",
    "etiquetan": "etiquetar",
    "etiquetando": "etiquetar",
    "etiquetara": "etiquetar",
    "etiquetarais": "etiquetar",
    "etiquetaran": "etiquetar",
    "etiquetaras": "etiquetar",
    "etiquetare": "etiquetar",
    "etiquetareis": "etiquetar",
    "etiquetaremos": "etiquetar",
    "etiquetaren": "etiquetar",
    "etiquetares": "etiquetar",
    "etiquetaron": "etiquetar",
    "etiquetará": "etiquetar",
    "etiquetarán": "etiquetar",
    "etiquetarás": "etiquetar",
    "etiquetaré": "etiquetar",
    "etiquetaréis": "etiquetar",
    "etiquetaría": "etiquetar",
    "etiquetaríais": "etiquetar",
    "etiquetaríamos": "etiquetar",
    "etiquetarían": "etiquetar",
    "etiquetarías": "etiquetar",
    "etiquetas": "etiquetar",
    "etiquetase": "etiquetar",
    "etiquetaseis": "etiquetar",
    "etiquetasen": "etiquetar",
    "etiquetases": "etiquetar",
    "etiquetaste": "etiquetar",
    "etiquetasteis": "etiquetar",
    "etiquete": "etiquetar",
    "etiquetemos": "etiquetar",
    "etiqueten": "etiquetar",
    "etiquetes": "etiquetar",
    "etiqueto": "etiquetar",
    "etiquetábamos": "etiquetar",
    "etiquetáis": "etiquetar",
    "etiquetáramos": "etiquetar",
    "etiquetáremos": "etiquetar",
    "etiquetásemos": "etiquetar",
    "etiqueté": "etiquetar",
    "etiquetéis": "etiquetar",
    "etiquetó": "etiquetar",
    "etiópica": "etiópico",
    "etiópicas": "etiópico",
    "etiópicos": "etiópico",
    "etiópidas": "etiópida",
    "etmoidales": "etmoidal",
    "etmoidectomías": "etmoidectomía",
    "etmoides": "etmoide",
    "etmoidomaxilares": "etmoidomaxilar",
    "etnarcas": "etnarca",
    "etnias": "etnia",
    "etnicidades": "etnicidad",
    "etnoarqueologías": "etnoarqueología",
    "etnocentrismos": "etnocentrismo",
    "etnocidios": "etnocidio",
    "etnociencias": "etnociencia",
    "etnografías": "etnografía",
    "etnográfica": "etnográfico",
    "etnográficas": "etnográfico",
    "etnográficos": "etnográfico",
    "etnohistorias": "etnohistoria",
    "etnolingüísticas": "etnolingüística",
    "etnologías": "etnología",
    "etnológica": "etnológico",
    "etnológicas": "etnológico",
    "etnológicos": "etnológico",
    "etnometodologías": "etnometodología",
    "etnopsicologías": "etnopsicología",
    "etnosemánticas": "etnosemántica",
    "etnógrafa": "etnógrafo",
    "etnógrafas": "etnógrafo",
    "etnógrafos": "etnógrafo",
    "etnóloga": "etnólogo",
    "etnólogas": "etnólogo",
    "etnólogos": "etnólogo",
    "etolia": "etolio",
    "etolias": "etolio",
    "etolios": "etolio",
    "etologistas": "etologista",
    "etologías": "etología",
    "etológica": "etológico",
    "etológicas": "etológico",
    "etológicos": "etológico",
    "etopeyas": "etopeya",
    "etoxitriglicoles": "etoxitriglicol",
    "etrusca": "etrusco",
    "etruscas": "etrusco",
    "etruscologías": "etruscología",
    "etruscos": "etrusco",
    "etérea": "etéreo",
    "etéreas": "etéreo",
    "etéreos": "etéreo",
    "etílica": "etílico",
    "etílicas": "etílico",
    "etílicos": "etílico",
    "etíopes": "etíope",
    "etóloga": "etólogo",
    "etólogas": "etólogo",
    "etólogos": "etólogo",
    "eubolias": "eubolia",
    "eucaliptoles": "eucaliptol",
    "eucaliptos": "eucalipto",
    "eucariontes": "eucarionte",
    "eucariota": "eucarioto",
    "eucariotas": "eucarioto",
    "eucariotos": "eucarioto",
    "eucaristías": "eucaristía",
    "eucariótica": "eucariótico",
    "eucarióticas": "eucariótico",
    "eucarióticos": "eucariótico",
    "eucarística": "eucarístico",
    "eucarísticas": "eucarístico",
    "eucarísticos": "eucarístico",
    "euclasas": "euclasa",
    "euclidiana": "euclidiano",
    "euclidianas": "euclidiano",
    "euclidianos": "euclidiano",
    "euclorinas": "euclorina",
    "eucologios": "eucologio",
    "eucrasias": "eucrasia",
    "eucromatinas": "eucromatina",
    "eucromosomas": "eucromosoma",
    "eucroítas": "eucroíta",
    "eucrática": "eucrático",
    "eucráticas": "eucrático",
    "eucráticos": "eucrático",
    "eudemonismos": "eudemonismo",
    "eudiómetros": "eudiómetro",
    "eufemismos": "eufemismo",
    "eufemística": "eufemístico",
    "eufemísticas": "eufemístico",
    "eufemísticos": "eufemístico",
    "eufonías": "eufonía",
    "euforbios": "euforbio",
    "euforbiácea": "euforbiáceo",
    "euforbiáceas": "euforbiáceo",
    "euforbiáceos": "euforbiáceo",
    "euforias": "euforia",
    "euforizantes": "euforizante",
    "eufuismos": "eufuismo",
    "eufónica": "eufónico",
    "eufónicas": "eufónico",
    "eufónicos": "eufónico",
    "eufórica": "eufórico",
    "eufóricas": "eufórico",
    "eufóricos": "eufórico",
    "eufótidas": "eufótida",
    "eugenatos": "eugenato",
    "eugenesias": "eugenesia",
    "eugenoles": "eugenol",
    "eugenésica": "eugenésico",
    "eugenésicas": "eugenésico",
    "eugenésicos": "eugenésico",
    "euglobulinas": "euglobulina",
    "euhedrales": "euhedral",
    "euleriana": "euleriano",
    "eulerianas": "euleriano",
    "eulerianos": "euleriano",
    "eunucos": "eunuco",
    "eupepsias": "eupepsia",
    "euploides": "euploide",
    "euploidias": "euploidia",
    "eupneas": "eupnea",
    "eupátridas": "eupátrida",
    "eupéptica": "eupéptico",
    "eupépticas": "eupéptico",
    "eupépticos": "eupéptico",
    "eurasiática": "eurasiático",
    "eurasiáticas": "eurasiático",
    "eurasiáticos": "eurasiático",
    "eurignática": "eurignático",
    "eurignáticas": "eurignático",
    "eurignáticos": "eurignático",
    "euriprosopias": "euriprosopia",
    "euritmias": "euritmia",
    "euroafricana": "euroafricano",
    "euroafricanas": "euroafricano",
    "euroafricanos": "euroafricano",
    "euroasia": "euroasio",
    "euroasias": "euroasio",
    "euroasios": "euroasio",
    "euroasiática": "euroasiático",
    "euroasiáticas": "euroasiático",
    "euroasiáticos": "euroasiático",
    "eurobonos": "eurobono",
    "eurocheques": "eurocheque",
    "eurocréditos": "eurocrédito",
    "eurodiputada": "eurodiputado",
    "eurodiputadas": "eurodiputado",
    "eurodiputados": "eurodiputado",
    "eurodivisas": "eurodivisa",
    "eurodólares": "eurodólar",
    "euroemisiones": "euroemisión",
    "euroemisiónes": "euroemisión",
    "euromagrebíes": "euromagrebí",
    "euromagrebís": "euromagrebí",
    "euromediterránea": "euromediterráneo",
    "euromediterráneas": "euromediterráneo",
    "euromediterráneos": "euromediterráneo",
    "euromercados": "euromercado",
    "euromisiles": "euromisil",
    "europea": "europeo",
    "europeas": "europeo",
    "europeice": "europeizar",
    "europeicemos": "europeizar",
    "europeicen": "europeizar",
    "europeices": "europeizar",
    "europeicé": "europeizar",
    "europeicéis": "europeizar",
    "europeiza": "europeizar",
    "europeizaba": "europeizar",
    "europeizabais": "europeizar",
    "europeizaban": "europeizar",
    "europeizabas": "europeizar",
    "europeizaciones": "europeización",
    "europeizad": "europeizar",
    "europeizada": "europeizar",
    "europeizadas": "europeizar",
    "europeizado": "europeizar",
    "europeizados": "europeizar",
    "europeizamos": "europeizar",
    "europeizan": "europeizar",
    "europeizando": "europeizar",
    "europeizara": "europeizar",
    "europeizarais": "europeizar",
    "europeizaran": "europeizar",
    "europeizaras": "europeizar",
    "europeizare": "europeizar",
    "europeizareis": "europeizar",
    "europeizaremos": "europeizar",
    "europeizaren": "europeizar",
    "europeizares": "europeizar",
    "europeizaron": "europeizar",
    "europeizará": "europeizar",
    "europeizarán": "europeizar",
    "europeizarás": "europeizar",
    "europeizaré": "europeizar",
    "europeizaréis": "europeizar",
    "europeizaría": "europeizar",
    "europeizaríais": "europeizar",
    "europeizaríamos": "europeizar",
    "europeizarían": "europeizar",
    "europeizarías": "europeizar",
    "europeizas": "europeizar",
    "europeizase": "europeizar",
    "europeizaseis": "europeizar",
    "europeizasen": "europeizar",
    "europeizases": "europeizar",
    "europeizaste": "europeizar",
    "europeizasteis": "europeizar",
    "europeizo": "europeizar",
    "europeizábamos": "europeizar",
    "europeizáis": "europeizar",
    "europeizáramos": "europeizar",
    "europeizáremos": "europeizar",
    "europeizásemos": "europeizar",
    "europeizó": "europeizar",
    "europeos": "europeo",
    "europeísmos": "europeísmo",
    "europeístas": "europeísta",
    "europios": "europio",
    "euros": "euro",
    "eurosiberiana": "eurosiberiano",
    "eurosiberianas": "eurosiberiano",
    "eurosiberianos": "eurosiberiano",
    "eurosistemas": "eurosistema",
    "euroventanillas": "euroventanilla",
    "eurovisiones": "eurovisión",
    "eurítmica": "eurítmico",
    "eurítmicas": "eurítmico",
    "eurítmicos": "eurítmico",
    "euskalduna": "euskaldún",
    "euskaldunas": "euskaldún",
    "euskaldunes": "euskaldún",
    "euskera": "euskero",
    "euskeras": "euskero",
    "euskeros": "euskero",
    "eusqueras": "eusquera",
    "eustatismos": "eustatismo",
    "eutanasias": "eutanasia",
    "eutanásica": "eutanásico",
    "eutanásicas": "eutanásico",
    "eutanásicos": "eutanásico",
    "eutectoides": "eutectoide",
    "eutexias": "eutexia",
    "eutiquiana": "eutiquiano",
    "eutiquianas": "eutiquiano",
    "eutiquianismos": "eutiquianismo",
    "eutiquianos": "eutiquiano",
    "eutiscopios": "eutiscopio",
    "eutrapelias": "eutrapelia",
    "eutrapélica": "eutrapélico",
    "eutrapélicas": "eutrapélico",
    "eutrapélicos": "eutrapélico",
    "eutrofias": "eutrofia",
    "eutrofice": "eutrofizar",
    "eutroficemos": "eutrofizar",
    "eutroficen": "eutrofizar",
    "eutrofices": "eutrofizar",
    "eutroficé": "eutrofizar",
    "eutroficéis": "eutrofizar",
    "eutrofiza": "eutrofizar",
    "eutrofizaba": "eutrofizar",
    "eutrofizabais": "eutrofizar",
    "eutrofizaban": "eutrofizar",
    "eutrofizabas": "eutrofizar",
    "eutrofizaciones": "eutrofización",
    "eutrofizad": "eutrofizar",
    "eutrofizada": "eutrofizar",
    "eutrofizadas": "eutrofizar",
    "eutrofizado": "eutrofizar",
    "eutrofizados": "eutrofizar",
    "eutrofizamos": "eutrofizar",
    "eutrofizan": "eutrofizar",
    "eutrofizando": "eutrofizar",
    "eutrofizara": "eutrofizar",
    "eutrofizarais": "eutrofizar",
    "eutrofizaran": "eutrofizar",
    "eutrofizaras": "eutrofizar",
    "eutrofizare": "eutrofizar",
    "eutrofizareis": "eutrofizar",
    "eutrofizaremos": "eutrofizar",
    "eutrofizaren": "eutrofizar",
    "eutrofizares": "eutrofizar",
    "eutrofizaron": "eutrofizar",
    "eutrofizará": "eutrofizar",
    "eutrofizarán": "eutrofizar",
    "eutrofizarás": "eutrofizar",
    "eutrofizaré": "eutrofizar",
    "eutrofizaréis": "eutrofizar",
    "eutrofizaría": "eutrofizar",
    "eutrofizaríais": "eutrofizar",
    "eutrofizaríamos": "eutrofizar",
    "eutrofizarían": "eutrofizar",
    "eutrofizarías": "eutrofizar",
    "eutrofizas": "eutrofizar",
    "eutrofizase": "eutrofizar",
    "eutrofizaseis": "eutrofizar",
    "eutrofizasen": "eutrofizar",
    "eutrofizases": "eutrofizar",
    "eutrofizaste": "eutrofizar",
    "eutrofizasteis": "eutrofizar",
    "eutrofizo": "eutrofizar",
    "eutrofizábamos": "eutrofizar",
    "eutrofizáis": "eutrofizar",
    "eutrofizáramos": "eutrofizar",
    "eutrofizáremos": "eutrofizar",
    "eutrofizásemos": "eutrofizar",
    "eutrofizó": "eutrofizar",
    "eutropelias": "eutropelia",
    "eutropías": "eutropía",
    "eutrófica": "eutrófico",
    "eutróficas": "eutrófico",
    "eutróficos": "eutrófico",
    "eutrópica": "eutrópico",
    "eutrópicas": "eutrópico",
    "eutrópicos": "eutrópico",
    "eutéctica": "eutéctico",
    "eutécticas": "eutéctico",
    "eutécticos": "eutéctico",
    "euxenitas": "euxenita",
    "evacuaba": "evacuar",
    "evacuabais": "evacuar",
    "evacuaban": "evacuar",
    "evacuabas": "evacuar",
    "evacuaciones": "evacuación",
    "evacuad": "evacuar",
    "evacuada": "evacuar",
    "evacuadas": "evacuar",
    "evacuado": "evacuar",
    "evacuados": "evacuar",
    "evacuamos": "evacuar",
    "evacuando": "evacuar",
    "evacuantes": "evacuante",
    "evacuara": "evacuar",
    "evacuarais": "evacuar",
    "evacuaran": "evacuar",
    "evacuaras": "evacuar",
    "evacuare": "evacuar",
    "evacuareis": "evacuar",
    "evacuaremos": "evacuar",
    "evacuaren": "evacuar",
    "evacuares": "evacuar",
    "evacuaron": "evacuar",
    "evacuará": "evacuar",
    "evacuarán": "evacuar",
    "evacuarás": "evacuar",
    "evacuaré": "evacuar",
    "evacuaréis": "evacuar",
    "evacuaría": "evacuar",
    "evacuaríais": "evacuar",
    "evacuaríamos": "evacuar",
    "evacuarían": "evacuar",
    "evacuarías": "evacuar",
    "evacuase": "evacuar",
    "evacuaseis": "evacuar",
    "evacuasen": "evacuar",
    "evacuases": "evacuar",
    "evacuaste": "evacuar",
    "evacuasteis": "evacuar",
    "evacuativos": "evacuativo",
    "evacuatorios": "evacuatorio",
    "evacuemos": "evacuar",
    "evacuábamos": "evacuar",
    "evacuáis": "evacuar",
    "evacuáramos": "evacuar",
    "evacuáremos": "evacuar",
    "evacuásemos": "evacuar",
    "evacué": "evacuar",
    "evacuéis": "evacuar",
    "evacuó": "evacuar",
    "evacúa": "evacuar",
    "evacúan": "evacuar",
    "evacúas": "evacuar",
    "evacúe": "evacuar",
    "evacúen": "evacuar",
    "evacúes": "evacuar",
    "evacúo": "evacuar",
    "evada": "evadir",
    "evadamos": "evadir",
    "evadan": "evadir",
    "evadas": "evadir",
    "evade": "evadir",
    "evaden": "evadir",
    "evades": "evadir",
    "evadid": "evadir",
    "evadida": "evadir",
    "evadidas": "evadir",
    "evadido": "evadir",
    "evadidos": "evadir",
    "evadiendo": "evadir",
    "evadiera": "evadir",
    "evadierais": "evadir",
    "evadieran": "evadir",
    "evadieras": "evadir",
    "evadiere": "evadir",
    "evadiereis": "evadir",
    "evadieren": "evadir",
    "evadieres": "evadir",
    "evadieron": "evadir",
    "evadiese": "evadir",
    "evadieseis": "evadir",
    "evadiesen": "evadir",
    "evadieses": "evadir",
    "evadimos": "evadir",
    "evadiremos": "evadir",
    "evadirá": "evadir",
    "evadirán": "evadir",
    "evadirás": "evadir",
    "evadiré": "evadir",
    "evadiréis": "evadir",
    "evadiría": "evadir",
    "evadiríais": "evadir",
    "evadiríamos": "evadir",
    "evadirían": "evadir",
    "evadirías": "evadir",
    "evadiste": "evadir",
    "evadisteis": "evadir",
    "evadiéramos": "evadir",
    "evadiéremos": "evadir",
    "evadiésemos": "evadir",
    "evadió": "evadir",
    "evado": "evadir",
    "evadáis": "evadir",
    "evadí": "evadir",
    "evadía": "evadir",
    "evadíais": "evadir",
    "evadíamos": "evadir",
    "evadían": "evadir",
    "evadías": "evadir",
    "evadís": "evadir",
    "evaluaba": "evaluar",
    "evaluabais": "evaluar",
    "evaluaban": "evaluar",
    "evaluabas": "evaluar",
    "evaluabilidades": "evaluabilidad",
    "evaluables": "evaluable",
    "evaluaciones": "evaluación",
    "evaluad": "evaluar",
    "evaluada": "evaluar",
    "evaluadas": "evaluar",
    "evaluado": "evaluar",
    "evaluadora": "evaluador",
    "evaluadoras": "evaluador",
    "evaluadores": "evaluador",
    "evaluados": "evaluar",
    "evaluamos": "evaluar",
    "evaluando": "evaluar",
    "evaluara": "evaluar",
    "evaluarais": "evaluar",
    "evaluaran": "evaluar",
    "evaluaras": "evaluar",
    "evaluare": "evaluar",
    "evaluareis": "evaluar",
    "evaluaremos": "evaluar",
    "evaluaren": "evaluar",
    "evaluares": "evaluar",
    "evaluaron": "evaluar",
    "evaluará": "evaluar",
    "evaluarán": "evaluar",
    "evaluarás": "evaluar",
    "evaluaré": "evaluar",
    "evaluaréis": "evaluar",
    "evaluaría": "evaluar",
    "evaluaríais": "evaluar",
    "evaluaríamos": "evaluar",
    "evaluarían": "evaluar",
    "evaluarías": "evaluar",
    "evaluase": "evaluar",
    "evaluaseis": "evaluar",
    "evaluasen": "evaluar",
    "evaluases": "evaluar",
    "evaluaste": "evaluar",
    "evaluasteis": "evaluar",
    "evaluativa": "evaluativo",
    "evaluativas": "evaluativo",
    "evaluativos": "evaluativo",
    "evaluemos": "evaluar",
    "evaluábamos": "evaluar",
    "evaluáis": "evaluar",
    "evaluáramos": "evaluar",
    "evaluáremos": "evaluar",
    "evaluásemos": "evaluar",
    "evalué": "evaluar",
    "evaluéis": "evaluar",
    "evaluó": "evaluar",
    "evalúa": "evaluar",
    "evalúan": "evaluar",
    "evalúas": "evaluar",
    "evalúe": "evaluar",
    "evalúen": "evaluar",
    "evalúes": "evaluar",
    "evalúo": "evaluar",
    "evanescentes": "evanescente",
    "evangeliarios": "evangeliario",
    "evangelice": "evangelizar",
    "evangelicemos": "evangelizar",
    "evangelicen": "evangelizar",
    "evangelices": "evangelizar",
    "evangelicé": "evangelizar",
    "evangelicéis": "evangelizar",
    "evangelios": "evangelio",
    "evangelismos": "evangelismo",
    "evangelistas": "evangelista",
    "evangelisteros": "evangelistero",
    "evangeliza": "evangelizar",
    "evangelizaba": "evangelizar",
    "evangelizabais": "evangelizar",
    "evangelizaban": "evangelizar",
    "evangelizabas": "evangelizar",
    "evangelizaciones": "evangelización",
    "evangelizad": "evangelizar",
    "evangelizada": "evangelizar",
    "evangelizadas": "evangelizar",
    "evangelizado": "evangelizar",
    "evangelizadora": "evangelizador",
    "evangelizadoras": "evangelizador",
    "evangelizadores": "evangelizador",
    "evangelizados": "evangelizar",
    "evangelizamos": "evangelizar",
    "evangelizan": "evangelizar",
    "evangelizando": "evangelizar",
    "evangelizara": "evangelizar",
    "evangelizarais": "evangelizar",
    "evangelizaran": "evangelizar",
    "evangelizaras": "evangelizar",
    "evangelizare": "evangelizar",
    "evangelizareis": "evangelizar",
    "evangelizaremos": "evangelizar",
    "evangelizaren": "evangelizar",
    "evangelizares": "evangelizar",
    "evangelizaron": "evangelizar",
    "evangelizará": "evangelizar",
    "evangelizarán": "evangelizar",
    "evangelizarás": "evangelizar",
    "evangelizaré": "evangelizar",
    "evangelizaréis": "evangelizar",
    "evangelizaría": "evangelizar",
    "evangelizaríais": "evangelizar",
    "evangelizaríamos": "evangelizar",
    "evangelizarían": "evangelizar",
    "evangelizarías": "evangelizar",
    "evangelizas": "evangelizar",
    "evangelizase": "evangelizar",
    "evangelizaseis": "evangelizar",
    "evangelizasen": "evangelizar",
    "evangelizases": "evangelizar",
    "evangelizaste": "evangelizar",
    "evangelizasteis": "evangelizar",
    "evangelizo": "evangelizar",
    "evangelizábamos": "evangelizar",
    "evangelizáis": "evangelizar",
    "evangelizáramos": "evangelizar",
    "evangelizáremos": "evangelizar",
    "evangelizásemos": "evangelizar",
    "evangelizó": "evangelizar",
    "evangélica": "evangélico",
    "evangélicas": "evangélico",
    "evangélicos": "evangélico",
    "evapora": "evaporar",
    "evaporaba": "evaporar",
    "evaporabais": "evaporar",
    "evaporaban": "evaporar",
    "evaporabas": "evaporar",
    "evaporables": "evaporable",
    "evaporaciones": "evaporación",
    "evaporad": "evaporar",
    "evaporada": "evaporar",
    "evaporadas": "evaporar",
    "evaporado": "evaporar",
    "evaporadora": "evaporador",
    "evaporadoras": "evaporador",
    "evaporadores": "evaporador",
    "evaporados": "evaporar",
    "evaporamos": "evaporar",
    "evaporan": "evaporar",
    "evaporando": "evaporar",
    "evaporara": "evaporar",
    "evaporarais": "evaporar",
    "evaporaran": "evaporar",
    "evaporaras": "evaporar",
    "evaporare": "evaporar",
    "evaporareis": "evaporar",
    "evaporaremos": "evaporar",
    "evaporaren": "evaporar",
    "evaporares": "evaporar",
    "evaporaron": "evaporar",
    "evaporará": "evaporar",
    "evaporarán": "evaporar",
    "evaporarás": "evaporar",
    "evaporaré": "evaporar",
    "evaporaréis": "evaporar",
    "evaporaría": "evaporar",
    "evaporaríais": "evaporar",
    "evaporaríamos": "evaporar",
    "evaporarían": "evaporar",
    "evaporarías": "evaporar",
    "evaporas": "evaporar",
    "evaporase": "evaporar",
    "evaporaseis": "evaporar",
    "evaporasen": "evaporar",
    "evaporases": "evaporar",
    "evaporaste": "evaporar",
    "evaporasteis": "evaporar",
    "evaporativa": "evaporativo",
    "evaporativas": "evaporativo",
    "evaporativos": "evaporativo",
    "evaporatorios": "evaporatorio",
    "evapore": "evaporar",
    "evaporemos": "evaporar",
    "evaporen": "evaporar",
    "evapores": "evaporar",
    "evaporice": "evaporizar",
    "evaporicemos": "evaporizar",
    "evaporicen": "evaporizar",
    "evaporices": "evaporizar",
    "evaporicé": "evaporizar",
    "evaporicéis": "evaporizar",
    "evaporimetrías": "evaporimetría",
    "evaporitas": "evaporita",
    "evaporiza": "evaporizar",
    "evaporizaba": "evaporizar",
    "evaporizabais": "evaporizar",
    "evaporizaban": "evaporizar",
    "evaporizabas": "evaporizar",
    "evaporizaciones": "evaporización",
    "evaporizad": "evaporizar",
    "evaporizada": "evaporizar",
    "evaporizadas": "evaporizar",
    "evaporizado": "evaporizar",
    "evaporizados": "evaporizar",
    "evaporizamos": "evaporizar",
    "evaporizan": "evaporizar",
    "evaporizando": "evaporizar",
    "evaporizara": "evaporizar",
    "evaporizarais": "evaporizar",
    "evaporizaran": "evaporizar",
    "evaporizaras": "evaporizar",
    "evaporizare": "evaporizar",
    "evaporizareis": "evaporizar",
    "evaporizaremos": "evaporizar",
    "evaporizaren": "evaporizar",
    "evaporizares": "evaporizar",
    "evaporizaron": "evaporizar",
    "evaporizará": "evaporizar",
    "evaporizarán": "evaporizar",
    "evaporizarás": "evaporizar",
    "evaporizaré": "evaporizar",
    "evaporizaréis": "evaporizar",
    "evaporizaría": "evaporizar",
    "evaporizaríais": "evaporizar",
    "evaporizaríamos": "evaporizar",
    "evaporizarían": "evaporizar",
    "evaporizarías": "evaporizar",
    "evaporizas": "evaporizar",
    "evaporizase": "evaporizar",
    "evaporizaseis": "evaporizar",
    "evaporizasen": "evaporizar",
    "evaporizases": "evaporizar",
    "evaporizaste": "evaporizar",
    "evaporizasteis": "evaporizar",
    "evaporizo": "evaporizar",
    "evaporizábamos": "evaporizar",
    "evaporizáis": "evaporizar",
    "evaporizáramos": "evaporizar",
    "evaporizáremos": "evaporizar",
    "evaporizásemos": "evaporizar",
    "evaporizó": "evaporizar",
    "evaporo": "evaporar",
    "evaporábamos": "evaporar",
    "evaporáis": "evaporar",
    "evaporáramos": "evaporar",
    "evaporáremos": "evaporar",
    "evaporásemos": "evaporar",
    "evaporé": "evaporar",
    "evaporéis": "evaporar",
    "evaporímetros": "evaporímetro",
    "evaporítica": "evaporítico",
    "evaporíticas": "evaporítico",
    "evaporíticos": "evaporítico",
    "evaporó": "evaporar",
    "evapotranspiraciones": "evapotranspiración",
    "evasiones": "evasión",
    "evasiva": "evasivo",
    "evasivas": "evasivo",
    "evasivos": "evasivo",
    "evasora": "evasor",
    "evasoras": "evasor",
    "evasores": "evasor",
    "evecciones": "evección",
    "evehentes": "evehente",
    "eventos": "evento",
    "eventraciones": "eventración",
    "eventuales": "eventual",
    "eventualidades": "eventualidad",
    "eversiones": "eversión",
    "evicciones": "evicción",
    "evidencia": "evidenciar",
    "evidenciaba": "evidenciar",
    "evidenciabais": "evidenciar",
    "evidenciaban": "evidenciar",
    "evidenciabas": "evidenciar",
    "evidenciad": "evidenciar",
    "evidenciada": "evidenciar",
    "evidenciadas": "evidenciar",
    "evidenciado": "evidenciar",
    "evidenciados": "evidenciar",
    "evidenciamos": "evidenciar",
    "evidencian": "evidenciar",
    "evidenciando": "evidenciar",
    "evidenciara": "evidenciar",
    "evidenciarais": "evidenciar",
    "evidenciaran": "evidenciar",
    "evidenciaras": "evidenciar",
    "evidenciare": "evidenciar",
    "evidenciareis": "evidenciar",
    "evidenciaremos": "evidenciar",
    "evidenciaren": "evidenciar",
    "evidenciares": "evidenciar",
    "evidenciaron": "evidenciar",
    "evidenciará": "evidenciar",
    "evidenciarán": "evidenciar",
    "evidenciarás": "evidenciar",
    "evidenciaré": "evidenciar",
    "evidenciaréis": "evidenciar",
    "evidenciaría": "evidenciar",
    "evidenciaríais": "evidenciar",
    "evidenciaríamos": "evidenciar",
    "evidenciarían": "evidenciar",
    "evidenciarías": "evidenciar",
    "evidencias": "evidenciar",
    "evidenciase": "evidenciar",
    "evidenciaseis": "evidenciar",
    "evidenciasen": "evidenciar",
    "evidenciases": "evidenciar",
    "evidenciaste": "evidenciar",
    "evidenciasteis": "evidenciar",
    "evidencie": "evidenciar",
    "evidenciemos": "evidenciar",
    "evidencien": "evidenciar",
    "evidencies": "evidenciar",
    "evidencio": "evidenciar",
    "evidenciábamos": "evidenciar",
    "evidenciáis": "evidenciar",
    "evidenciáramos": "evidenciar",
    "evidenciáremos": "evidenciar",
    "evidenciásemos": "evidenciar",
    "evidencié": "evidenciar",
    "evidenciéis": "evidenciar",
    "evidenció": "evidenciar",
    "evidentes": "evidente",
    "evisceraciones": "evisceración",
    "evita": "evitar",
    "evitaba": "evitar",
    "evitabais": "evitar",
    "evitaban": "evitar",
    "evitabas": "evitar",
    "evitabilidades": "evitabilidad",
    "evitables": "evitable",
    "evitaciones": "evitación",
    "evitad": "evitar",
    "evitada": "evitar",
    "evitadas": "evitar",
    "evitado": "evitar",
    "evitados": "evitar",
    "evitamos": "evitar",
    "evitan": "evitar",
    "evitando": "evitar",
    "evitara": "evitar",
    "evitarais": "evitar",
    "evitaran": "evitar",
    "evitaras": "evitar",
    "evitare": "evitar",
    "evitareis": "evitar",
    "evitaremos": "evitar",
    "evitaren": "evitar",
    "evitares": "evitar",
    "evitaron": "evitar",
    "evitará": "evitar",
    "evitarán": "evitar",
    "evitarás": "evitar",
    "evitaré": "evitar",
    "evitaréis": "evitar",
    "evitaría": "evitar",
    "evitaríais": "evitar",
    "evitaríamos": "evitar",
    "evitarían": "evitar",
    "evitarías": "evitar",
    "evitas": "evitar",
    "evitase": "evitar",
    "evitaseis": "evitar",
    "evitasen": "evitar",
    "evitases": "evitar",
    "evitaste": "evitar",
    "evitasteis": "evitar",
    "evite": "evitar",
    "evitemos": "evitar",
    "eviten": "evitar",
    "eviterna": "eviterno",
    "eviternas": "eviterno",
    "eviternos": "eviterno",
    "evites": "evitar",
    "evito": "evitar",
    "evitábamos": "evitar",
    "evitáis": "evitar",
    "evitáramos": "evitar",
    "evitáremos": "evitar",
    "evitásemos": "evitar",
    "evité": "evitar",
    "evitéis": "evitar",
    "evitó": "evitar",
    "evoca": "evocar",
    "evocaba": "evocar",
    "evocabais": "evocar",
    "evocaban": "evocar",
    "evocabas": "evocar",
    "evocables": "evocable",
    "evocaciones": "evocación",
    "evocad": "evocar",
    "evocada": "evocar",
    "evocadas": "evocar",
    "evocado": "evocar",
    "evocadora": "evocador",
    "evocadoras": "evocador",
    "evocadores": "evocador",
    "evocados": "evocar",
    "evocamos": "evocar",
    "evocan": "evocar",
    "evocando": "evocar",
    "evocara": "evocar",
    "evocarais": "evocar",
    "evocaran": "evocar",
    "evocaras": "evocar",
    "evocare": "evocar",
    "evocareis": "evocar",
    "evocaremos": "evocar",
    "evocaren": "evocar",
    "evocares": "evocar",
    "evocaron": "evocar",
    "evocará": "evocar",
    "evocarán": "evocar",
    "evocarás": "evocar",
    "evocaré": "evocar",
    "evocaréis": "evocar",
    "evocaría": "evocar",
    "evocaríais": "evocar",
    "evocaríamos": "evocar",
    "evocarían": "evocar",
    "evocarías": "evocar",
    "evocas": "evocar",
    "evocase": "evocar",
    "evocaseis": "evocar",
    "evocasen": "evocar",
    "evocases": "evocar",
    "evocaste": "evocar",
    "evocasteis": "evocar",
    "evocatoria": "evocatorio",
    "evocatorias": "evocatorio",
    "evocatorios": "evocatorio",
    "evoco": "evocar",
    "evocábamos": "evocar",
    "evocáis": "evocar",
    "evocáramos": "evocar",
    "evocáremos": "evocar",
    "evocásemos": "evocar",
    "evocó": "evocar",
    "evoluciona": "evolucionar",
    "evolucionaba": "evolucionar",
    "evolucionabais": "evolucionar",
    "evolucionaban": "evolucionar",
    "evolucionabas": "evolucionar",
    "evolucionad": "evolucionar",
    "evolucionada": "evolucionar",
    "evolucionadas": "evolucionar",
    "evolucionado": "evolucionar",
    "evolucionados": "evolucionar",
    "evolucionamos": "evolucionar",
    "evolucionan": "evolucionar",
    "evolucionando": "evolucionar",
    "evolucionara": "evolucionar",
    "evolucionarais": "evolucionar",
    "evolucionaran": "evolucionar",
    "evolucionaras": "evolucionar",
    "evolucionare": "evolucionar",
    "evolucionareis": "evolucionar",
    "evolucionaremos": "evolucionar",
    "evolucionaren": "evolucionar",
    "evolucionares": "evolucionar",
    "evolucionaron": "evolucionar",
    "evolucionará": "evolucionar",
    "evolucionarán": "evolucionar",
    "evolucionarás": "evolucionar",
    "evolucionaré": "evolucionar",
    "evolucionaréis": "evolucionar",
    "evolucionaría": "evolucionar",
    "evolucionaríais": "evolucionar",
    "evolucionaríamos": "evolucionar",
    "evolucionarían": "evolucionar",
    "evolucionarías": "evolucionar",
    "evolucionas": "evolucionar",
    "evolucionase": "evolucionar",
    "evolucionaseis": "evolucionar",
    "evolucionasen": "evolucionar",
    "evolucionases": "evolucionar",
    "evolucionaste": "evolucionar",
    "evolucionasteis": "evolucionar",
    "evolucione": "evolucionar",
    "evolucionemos": "evolucionar",
    "evolucionen": "evolucionar",
    "evoluciones": "evolucionar",
    "evolucionismos": "evolucionismo",
    "evolucionistas": "evolucionista",
    "evoluciono": "evolucionar",
    "evolucionábamos": "evolucionar",
    "evolucionáis": "evolucionar",
    "evolucionáramos": "evolucionar",
    "evolucionáremos": "evolucionar",
    "evolucionásemos": "evolucionar",
    "evolucioné": "evolucionar",
    "evolucionéis": "evolucionar",
    "evolucionó": "evolucionar",
    "evolutas": "evoluta",
    "evolutiva": "evolutivo",
    "evolutivas": "evolutivo",
    "evolutivos": "evolutivo",
    "evoque": "evocar",
    "evoquemos": "evocar",
    "evoquen": "evocar",
    "evoques": "evocar",
    "evoqué": "evocar",
    "evoquéis": "evocar",
    "evos": "evo",
    "evónimos": "evónimo",
    "ex-campeona": "ex-campeón",
    "ex-campeonas": "ex-campeón",
    "ex-campeones": "ex-campeón",
    "exabruptos": "exabrupto",
    "exabytes": "exabyte",
    "exacciones": "exacción",
    "exacerba": "exacerbar",
    "exacerbaba": "exacerbar",
    "exacerbabais": "exacerbar",
    "exacerbaban": "exacerbar",
    "exacerbabas": "exacerbar",
    "exacerbaciones": "exacerbación",
    "exacerbad": "exacerbar",
    "exacerbada": "exacerbar",
    "exacerbadas": "exacerbar",
    "exacerbado": "exacerbar",
    "exacerbados": "exacerbar",
    "exacerbamientos": "exacerbamiento",
    "exacerbamos": "exacerbar",
    "exacerban": "exacerbar",
    "exacerbando": "exacerbar",
    "exacerbara": "exacerbar",
    "exacerbarais": "exacerbar",
    "exacerbaran": "exacerbar",
    "exacerbaras": "exacerbar",
    "exacerbare": "exacerbar",
    "exacerbareis": "exacerbar",
    "exacerbaremos": "exacerbar",
    "exacerbaren": "exacerbar",
    "exacerbares": "exacerbar",
    "exacerbaron": "exacerbar",
    "exacerbará": "exacerbar",
    "exacerbarán": "exacerbar",
    "exacerbarás": "exacerbar",
    "exacerbaré": "exacerbar",
    "exacerbaréis": "exacerbar",
    "exacerbaría": "exacerbar",
    "exacerbaríais": "exacerbar",
    "exacerbaríamos": "exacerbar",
    "exacerbarían": "exacerbar",
    "exacerbarías": "exacerbar",
    "exacerbas": "exacerbar",
    "exacerbase": "exacerbar",
    "exacerbaseis": "exacerbar",
    "exacerbasen": "exacerbar",
    "exacerbases": "exacerbar",
    "exacerbaste": "exacerbar",
    "exacerbasteis": "exacerbar",
    "exacerbe": "exacerbar",
    "exacerbemos": "exacerbar",
    "exacerben": "exacerbar",
    "exacerbes": "exacerbar",
    "exacerbo": "exacerbar",
    "exacerbábamos": "exacerbar",
    "exacerbáis": "exacerbar",
    "exacerbáramos": "exacerbar",
    "exacerbáremos": "exacerbar",
    "exacerbásemos": "exacerbar",
    "exacerbé": "exacerbar",
    "exacerbéis": "exacerbar",
    "exacerbó": "exacerbar",
    "exacta": "exacto",
    "exactas": "exacto",
    "exactitudes": "exactitud",
    "exactores": "exactor",
    "exactos": "exacto",
    "exagera": "exagerar",
    "exageraba": "exagerar",
    "exagerabais": "exagerar",
    "exageraban": "exagerar",
    "exagerabas": "exagerar",
    "exageraciones": "exageración",
    "exagerad": "exagerar",
    "exagerada": "exagerar",
    "exageradas": "exagerar",
    "exagerado": "exagerar",
    "exageradora": "exagerador",
    "exageradoras": "exagerador",
    "exageradores": "exagerador",
    "exagerados": "exagerar",
    "exageramos": "exagerar",
    "exageran": "exagerar",
    "exagerando": "exagerar",
    "exagerara": "exagerar",
    "exagerarais": "exagerar",
    "exageraran": "exagerar",
    "exageraras": "exagerar",
    "exagerare": "exagerar",
    "exagerareis": "exagerar",
    "exageraremos": "exagerar",
    "exageraren": "exagerar",
    "exagerares": "exagerar",
    "exageraron": "exagerar",
    "exagerará": "exagerar",
    "exagerarán": "exagerar",
    "exagerarás": "exagerar",
    "exageraré": "exagerar",
    "exageraréis": "exagerar",
    "exageraría": "exagerar",
    "exageraríais": "exagerar",
    "exageraríamos": "exagerar",
    "exagerarían": "exagerar",
    "exagerarías": "exagerar",
    "exageras": "exagerar",
    "exagerase": "exagerar",
    "exageraseis": "exagerar",
    "exagerasen": "exagerar",
    "exagerases": "exagerar",
    "exageraste": "exagerar",
    "exagerasteis": "exagerar",
    "exagerativa": "exagerativo",
    "exagerativas": "exagerativo",
    "exagerativos": "exagerativo",
    "exagere": "exagerar",
    "exageremos": "exagerar",
    "exageren": "exagerar",
    "exageres": "exagerar",
    "exagero": "exagerar",
    "exagerábamos": "exagerar",
    "exageráis": "exagerar",
    "exageráramos": "exagerar",
    "exageráremos": "exagerar",
    "exagerásemos": "exagerar",
    "exageré": "exagerar",
    "exageréis": "exagerar",
    "exageró": "exagerar",
    "exalta": "exaltar",
    "exaltaba": "exaltar",
    "exaltabais": "exaltar",
    "exaltaban": "exaltar",
    "exaltabas": "exaltar",
    "exaltaciones": "exaltación",
    "exaltad": "exaltar",
    "exaltada": "exaltar",
    "exaltadas": "exaltar",
    "exaltado": "exaltar",
    "exaltadora": "exaltador",
    "exaltadoras": "exaltador",
    "exaltadores": "exaltador",
    "exaltados": "exaltar",
    "exaltamientos": "exaltamiento",
    "exaltamos": "exaltar",
    "exaltan": "exaltar",
    "exaltando": "exaltar",
    "exaltara": "exaltar",
    "exaltarais": "exaltar",
    "exaltaran": "exaltar",
    "exaltaras": "exaltar",
    "exaltare": "exaltar",
    "exaltareis": "exaltar",
    "exaltaremos": "exaltar",
    "exaltaren": "exaltar",
    "exaltares": "exaltar",
    "exaltaron": "exaltar",
    "exaltará": "exaltar",
    "exaltarán": "exaltar",
    "exaltarás": "exaltar",
    "exaltaré": "exaltar",
    "exaltaréis": "exaltar",
    "exaltaría": "exaltar",
    "exaltaríais": "exaltar",
    "exaltaríamos": "exaltar",
    "exaltarían": "exaltar",
    "exaltarías": "exaltar",
    "exaltas": "exaltar",
    "exaltase": "exaltar",
    "exaltaseis": "exaltar",
    "exaltasen": "exaltar",
    "exaltases": "exaltar",
    "exaltaste": "exaltar",
    "exaltasteis": "exaltar",
    "exalte": "exaltar",
    "exaltemos": "exaltar",
    "exalten": "exaltar",
    "exaltes": "exaltar",
    "exalto": "exaltar",
    "exaltábamos": "exaltar",
    "exaltáis": "exaltar",
    "exaltáramos": "exaltar",
    "exaltáremos": "exaltar",
    "exaltásemos": "exaltar",
    "exalté": "exaltar",
    "exaltéis": "exaltar",
    "exaltó": "exaltar",
    "examina": "examinar",
    "examinaba": "examinar",
    "examinabais": "examinar",
    "examinaban": "examinar",
    "examinabas": "examinar",
    "examinaciones": "examinación",
    "examinad": "examinar",
    "examinada": "examinar",
    "examinadas": "examinar",
    "examinado": "examinar",
    "examinadora": "examinador",
    "examinadoras": "examinador",
    "examinadores": "examinador",
    "examinados": "examinar",
    "examinamos": "examinar",
    "examinan": "examinar",
    "examinanda": "examinando",
    "examinandas": "examinando",
    "examinando": "examinar",
    "examinandos": "examinando",
    "examinara": "examinar",
    "examinarais": "examinar",
    "examinaran": "examinar",
    "examinaras": "examinar",
    "examinare": "examinar",
    "examinareis": "examinar",
    "examinaremos": "examinar",
    "examinaren": "examinar",
    "examinares": "examinar",
    "examinaron": "examinar",
    "examinará": "examinar",
    "examinarán": "examinar",
    "examinarás": "examinar",
    "examinaré": "examinar",
    "examinaréis": "examinar",
    "examinaría": "examinar",
    "examinaríais": "examinar",
    "examinaríamos": "examinar",
    "examinarían": "examinar",
    "examinarías": "examinar",
    "examinas": "examinar",
    "examinase": "examinar",
    "examinaseis": "examinar",
    "examinasen": "examinar",
    "examinases": "examinar",
    "examinaste": "examinar",
    "examinasteis": "examinar",
    "examine": "examinar",
    "examinemos": "examinar",
    "examinen": "examinar",
    "examines": "examinar",
    "examino": "examinar",
    "examinábamos": "examinar",
    "examináis": "examinar",
    "examináramos": "examinar",
    "examináremos": "examinar",
    "examinásemos": "examinar",
    "examiné": "examinar",
    "examinéis": "examinar",
    "examinó": "examinar",
    "exangües": "exangüe",
    "exanimaciones": "exanimación",
    "exantemas": "exantema",
    "exantemática": "exantemático",
    "exantemáticas": "exantemático",
    "exantemáticos": "exantemático",
    "exaptaciones": "exaptación",
    "exarcados": "exarcado",
    "exarcas": "exarca",
    "exarcos": "exarco",
    "exaspera": "exasperar",
    "exasperaba": "exasperar",
    "exasperabais": "exasperar",
    "exasperaban": "exasperar",
    "exasperabas": "exasperar",
    "exasperaciones": "exasperación",
    "exasperad": "exasperar",
    "exasperada": "exasperar",
    "exasperadas": "exasperar",
    "exasperado": "exasperar",
    "exasperados": "exasperar",
    "exasperamos": "exasperar",
    "exasperan": "exasperar",
    "exasperando": "exasperar",
    "exasperantes": "exasperante",
    "exasperara": "exasperar",
    "exasperarais": "exasperar",
    "exasperaran": "exasperar",
    "exasperaras": "exasperar",
    "exasperare": "exasperar",
    "exasperareis": "exasperar",
    "exasperaremos": "exasperar",
    "exasperaren": "exasperar",
    "exasperares": "exasperar",
    "exasperaron": "exasperar",
    "exasperará": "exasperar",
    "exasperarán": "exasperar",
    "exasperarás": "exasperar",
    "exasperaré": "exasperar",
    "exasperaréis": "exasperar",
    "exasperaría": "exasperar",
    "exasperaríais": "exasperar",
    "exasperaríamos": "exasperar",
    "exasperarían": "exasperar",
    "exasperarías": "exasperar",
    "exasperas": "exasperar",
    "exasperase": "exasperar",
    "exasperaseis": "exasperar",
    "exasperasen": "exasperar",
    "exasperases": "exasperar",
    "exasperaste": "exasperar",
    "exasperasteis": "exasperar",
    "exaspere": "exasperar",
    "exasperemos": "exasperar",
    "exasperen": "exasperar",
    "exasperes": "exasperar",
    "exaspero": "exasperar",
    "exasperábamos": "exasperar",
    "exasperáis": "exasperar",
    "exasperáramos": "exasperar",
    "exasperáremos": "exasperar",
    "exasperásemos": "exasperar",
    "exasperé": "exasperar",
    "exasperéis": "exasperar",
    "exasperó": "exasperar",
    "exazulgranas": "exazulgrana",
    "exbarcelonistas": "exbarcelonista",
    "excandece": "excandecer",
    "excandeced": "excandecer",
    "excandecemos": "excandecer",
    "excandecen": "excandecer",
    "excandecencias": "excandecencia",
    "excandeceremos": "excandecer",
    "excandecerá": "excandecer",
    "excandecerán": "excandecer",
    "excandecerás": "excandecer",
    "excandeceré": "excandecer",
    "excandeceréis": "excandecer",
    "excandecería": "excandecer",
    "excandeceríais": "excandecer",
    "excandeceríamos": "excandecer",
    "excandecerían": "excandecer",
    "excandecerías": "excandecer",
    "excandeces": "excandecer",
    "excandecida": "excandecer",
    "excandecidas": "excandecer",
    "excandecido": "excandecer",
    "excandecidos": "excandecer",
    "excandeciendo": "excandecer",
    "excandeciera": "excandecer",
    "excandecierais": "excandecer",
    "excandecieran": "excandecer",
    "excandecieras": "excandecer",
    "excandeciere": "excandecer",
    "excandeciereis": "excandecer",
    "excandecieren": "excandecer",
    "excandecieres": "excandecer",
    "excandecieron": "excandecer",
    "excandeciese": "excandecer",
    "excandecieseis": "excandecer",
    "excandeciesen": "excandecer",
    "excandecieses": "excandecer",
    "excandecimos": "excandecer",
    "excandeciste": "excandecer",
    "excandecisteis": "excandecer",
    "excandeciéramos": "excandecer",
    "excandeciéremos": "excandecer",
    "excandeciésemos": "excandecer",
    "excandeció": "excandecer",
    "excandecéis": "excandecer",
    "excandecí": "excandecer",
    "excandecía": "excandecer",
    "excandecíais": "excandecer",
    "excandecíamos": "excandecer",
    "excandecían": "excandecer",
    "excandecías": "excandecer",
    "excandezca": "excandecer",
    "excandezcamos": "excandecer",
    "excandezcan": "excandecer",
    "excandezcas": "excandecer",
    "excandezco": "excandecer",
    "excandezcáis": "excandecer",
    "excarcela": "excarcelar",
    "excarcelaba": "excarcelar",
    "excarcelabais": "excarcelar",
    "excarcelaban": "excarcelar",
    "excarcelabas": "excarcelar",
    "excarcelables": "excarcelable",
    "excarcelaciones": "excarcelación",
    "excarcelad": "excarcelar",
    "excarcelada": "excarcelar",
    "excarceladas": "excarcelar",
    "excarcelado": "excarcelar",
    "excarcelados": "excarcelar",
    "excarcelamos": "excarcelar",
    "excarcelan": "excarcelar",
    "excarcelando": "excarcelar",
    "excarcelara": "excarcelar",
    "excarcelarais": "excarcelar",
    "excarcelaran": "excarcelar",
    "excarcelaras": "excarcelar",
    "excarcelare": "excarcelar",
    "excarcelareis": "excarcelar",
    "excarcelaremos": "excarcelar",
    "excarcelaren": "excarcelar",
    "excarcelares": "excarcelar",
    "excarcelaron": "excarcelar",
    "excarcelará": "excarcelar",
    "excarcelarán": "excarcelar",
    "excarcelarás": "excarcelar",
    "excarcelaré": "excarcelar",
    "excarcelaréis": "excarcelar",
    "excarcelaría": "excarcelar",
    "excarcelaríais": "excarcelar",
    "excarcelaríamos": "excarcelar",
    "excarcelarían": "excarcelar",
    "excarcelarías": "excarcelar",
    "excarcelas": "excarcelar",
    "excarcelase": "excarcelar",
    "excarcelaseis": "excarcelar",
    "excarcelasen": "excarcelar",
    "excarcelases": "excarcelar",
    "excarcelaste": "excarcelar",
    "excarcelasteis": "excarcelar",
    "excarcele": "excarcelar",
    "excarcelemos": "excarcelar",
    "excarcelen": "excarcelar",
    "excarceles": "excarcelar",
    "excarcelo": "excarcelar",
    "excarcelábamos": "excarcelar",
    "excarceláis": "excarcelar",
    "excarceláramos": "excarcelar",
    "excarceláremos": "excarcelar",
    "excarcelásemos": "excarcelar",
    "excarcelé": "excarcelar",
    "excarceléis": "excarcelar",
    "excarceló": "excarcelar",
    "excarceraciones": "excarceración",
    "excava": "excavar",
    "excavaba": "excavar",
    "excavabais": "excavar",
    "excavaban": "excavar",
    "excavabas": "excavar",
    "excavaciones": "excavación",
    "excavad": "excavar",
    "excavada": "excavar",
    "excavadas": "excavar",
    "excavado": "excavar",
    "excavadora": "excavador",
    "excavadoras": "excavadora",
    "excavadores": "excavador",
    "excavados": "excavar",
    "excavamos": "excavar",
    "excavan": "excavar",
    "excavando": "excavar",
    "excavara": "excavar",
    "excavarais": "excavar",
    "excavaran": "excavar",
    "excavaras": "excavar",
    "excavare": "excavar",
    "excavareis": "excavar",
    "excavaremos": "excavar",
    "excavaren": "excavar",
    "excavares": "excavar",
    "excavaron": "excavar",
    "excavará": "excavar",
    "excavarán": "excavar",
    "excavarás": "excavar",
    "excavaré": "excavar",
    "excavaréis": "excavar",
    "excavaría": "excavar",
    "excavaríais": "excavar",
    "excavaríamos": "excavar",
    "excavarían": "excavar",
    "excavarías": "excavar",
    "excavas": "excavar",
    "excavase": "excavar",
    "excavaseis": "excavar",
    "excavasen": "excavar",
    "excavases": "excavar",
    "excavaste": "excavar",
    "excavasteis": "excavar",
    "excave": "excavar",
    "excavemos": "excavar",
    "excaven": "excavar",
    "excaves": "excavar",
    "excavo": "excavar",
    "excavábamos": "excavar",
    "excaváis": "excavar",
    "excaváramos": "excavar",
    "excaváremos": "excavar",
    "excavásemos": "excavar",
    "excavé": "excavar",
    "excavéis": "excavar",
    "excavó": "excavar",
    "exceda": "exceder",
    "excedamos": "exceder",
    "excedan": "exceder",
    "excedas": "exceder",
    "excede": "exceder",
    "exceded": "exceder",
    "excedemos": "exceder",
    "exceden": "exceder",
    "excedencias": "excedencia",
    "excedentaria": "excedentario",
    "excedentarias": "excedentario",
    "excedentarios": "excedentario",
    "excedentes": "excedente",
    "excederemos": "exceder",
    "excederá": "exceder",
    "excederán": "exceder",
    "excederás": "exceder",
    "excederé": "exceder",
    "excederéis": "exceder",
    "excedería": "exceder",
    "excederíais": "exceder",
    "excederíamos": "exceder",
    "excederían": "exceder",
    "excederías": "exceder",
    "excedes": "exceder",
    "excedida": "exceder",
    "excedidas": "exceder",
    "excedido": "exceder",
    "excedidos": "exceder",
    "excediendo": "exceder",
    "excediera": "exceder",
    "excedierais": "exceder",
    "excedieran": "exceder",
    "excedieras": "exceder",
    "excediere": "exceder",
    "excediereis": "exceder",
    "excedieren": "exceder",
    "excedieres": "exceder",
    "excedieron": "exceder",
    "excediese": "exceder",
    "excedieseis": "exceder",
    "excediesen": "exceder",
    "excedieses": "exceder",
    "excedimos": "exceder",
    "excediste": "exceder",
    "excedisteis": "exceder",
    "excediéramos": "exceder",
    "excediéremos": "exceder",
    "excediésemos": "exceder",
    "excedió": "exceder",
    "excedo": "exceder",
    "excedáis": "exceder",
    "excedéis": "exceder",
    "excedí": "exceder",
    "excedía": "exceder",
    "excedíais": "exceder",
    "excedíamos": "exceder",
    "excedían": "exceder",
    "excedías": "exceder",
    "excelencias": "excelencia",
    "excelentes": "excelente",
    "excelentísima": "excelentísimo",
    "excelentísimas": "excelentísimo",
    "excelentísimos": "excelentísimo",
    "excelsa": "excelso",
    "excelsas": "excelso",
    "excelsitudes": "excelsitud",
    "excelsos": "excelso",
    "excentricidades": "excentricidad",
    "excepciona": "excepcionar",
    "excepcionaba": "excepcionar",
    "excepcionabais": "excepcionar",
    "excepcionaban": "excepcionar",
    "excepcionabas": "excepcionar",
    "excepcionad": "excepcionar",
    "excepcionada": "excepcionar",
    "excepcionadas": "excepcionar",
    "excepcionado": "excepcionar",
    "excepcionados": "excepcionar",
    "excepcionales": "excepcional",
    "excepcionalidades": "excepcionalidad",
    "excepcionalismos": "excepcionalismo",
    "excepcionamos": "excepcionar",
    "excepcionan": "excepcionar",
    "excepcionando": "excepcionar",
    "excepcionara": "excepcionar",
    "excepcionarais": "excepcionar",
    "excepcionaran": "excepcionar",
    "excepcionaras": "excepcionar",
    "excepcionare": "excepcionar",
    "excepcionareis": "excepcionar",
    "excepcionaremos": "excepcionar",
    "excepcionaren": "excepcionar",
    "excepcionares": "excepcionar",
    "excepcionaron": "excepcionar",
    "excepcionará": "excepcionar",
    "excepcionarán": "excepcionar",
    "excepcionarás": "excepcionar",
    "excepcionaré": "excepcionar",
    "excepcionaréis": "excepcionar",
    "excepcionaría": "excepcionar",
    "excepcionaríais": "excepcionar",
    "excepcionaríamos": "excepcionar",
    "excepcionarían": "excepcionar",
    "excepcionarías": "excepcionar",
    "excepcionas": "excepcionar",
    "excepcionase": "excepcionar",
    "excepcionaseis": "excepcionar",
    "excepcionasen": "excepcionar",
    "excepcionases": "excepcionar",
    "excepcionaste": "excepcionar",
    "excepcionasteis": "excepcionar",
    "excepcione": "excepcionar",
    "excepcionemos": "excepcionar",
    "excepcionen": "excepcionar",
    "excepciones": "excepcionar",
    "excepciono": "excepcionar",
    "excepcionábamos": "excepcionar",
    "excepcionáis": "excepcionar",
    "excepcionáramos": "excepcionar",
    "excepcionáremos": "excepcionar",
    "excepcionásemos": "excepcionar",
    "excepcioné": "excepcionar",
    "excepcionéis": "excepcionar",
    "excepcionó": "excepcionar",
    "exceptios": "exceptio",
    "exceptiva": "exceptivo",
    "exceptivas": "exceptivo",
    "exceptivos": "exceptivo",
    "exceptuaba": "exceptuar",
    "exceptuabais": "exceptuar",
    "exceptuaban": "exceptuar",
    "exceptuabas": "exceptuar",
    "exceptuad": "exceptuar",
    "exceptuada": "exceptuar",
    "exceptuadas": "exceptuar",
    "exceptuado": "exceptuar",
    "exceptuados": "exceptuar",
    "exceptuamos": "exceptuar",
    "exceptuando": "exceptuar",
    "exceptuara": "exceptuar",
    "exceptuarais": "exceptuar",
    "exceptuaran": "exceptuar",
    "exceptuaras": "exceptuar",
    "exceptuare": "exceptuar",
    "exceptuareis": "exceptuar",
    "exceptuaremos": "exceptuar",
    "exceptuaren": "exceptuar",
    "exceptuares": "exceptuar",
    "exceptuaron": "exceptuar",
    "exceptuará": "exceptuar",
    "exceptuarán": "exceptuar",
    "exceptuarás": "exceptuar",
    "exceptuaré": "exceptuar",
    "exceptuaréis": "exceptuar",
    "exceptuaría": "exceptuar",
    "exceptuaríais": "exceptuar",
    "exceptuaríamos": "exceptuar",
    "exceptuarían": "exceptuar",
    "exceptuarías": "exceptuar",
    "exceptuase": "exceptuar",
    "exceptuaseis": "exceptuar",
    "exceptuasen": "exceptuar",
    "exceptuases": "exceptuar",
    "exceptuaste": "exceptuar",
    "exceptuasteis": "exceptuar",
    "exceptuemos": "exceptuar",
    "exceptuábamos": "exceptuar",
    "exceptuáis": "exceptuar",
    "exceptuáramos": "exceptuar",
    "exceptuáremos": "exceptuar",
    "exceptuásemos": "exceptuar",
    "exceptué": "exceptuar",
    "exceptuéis": "exceptuar",
    "exceptuó": "exceptuar",
    "exceptúa": "exceptuar",
    "exceptúan": "exceptuar",
    "exceptúas": "exceptuar",
    "exceptúe": "exceptuar",
    "exceptúen": "exceptuar",
    "exceptúes": "exceptuar",
    "exceptúo": "exceptuar",
    "excerptas": "excerpta",
    "excertas": "excerta",
    "excesiva": "excesivo",
    "excesivas": "excesivo",
    "excesivos": "excesivo",
    "excesos": "exceso",
    "excicloforias": "excicloforia",
    "excipientes": "excipiente",
    "excisa": "exciso",
    "excisas": "exciso",
    "excisiones": "excisión",
    "excisos": "exciso",
    "excita": "excitar",
    "excitaba": "excitar",
    "excitabais": "excitar",
    "excitaban": "excitar",
    "excitabas": "excitar",
    "excitabilidades": "excitabilidad",
    "excitables": "excitable",
    "excitaciones": "excitación",
    "excitad": "excitar",
    "excitada": "excitar",
    "excitadas": "excitar",
    "excitado": "excitar",
    "excitadora": "excitador",
    "excitadoras": "excitador",
    "excitadores": "excitador",
    "excitados": "excitar",
    "excitamos": "excitar",
    "excitan": "excitar",
    "excitando": "excitar",
    "excitantes": "excitante",
    "excitara": "excitar",
    "excitarais": "excitar",
    "excitaran": "excitar",
    "excitaras": "excitar",
    "excitare": "excitar",
    "excitareis": "excitar",
    "excitaremos": "excitar",
    "excitaren": "excitar",
    "excitares": "excitar",
    "excitaron": "excitar",
    "excitará": "excitar",
    "excitarán": "excitar",
    "excitarás": "excitar",
    "excitaré": "excitar",
    "excitaréis": "excitar",
    "excitaría": "excitar",
    "excitaríais": "excitar",
    "excitaríamos": "excitar",
    "excitarían": "excitar",
    "excitarías": "excitar",
    "excitas": "excitar",
    "excitase": "excitar",
    "excitaseis": "excitar",
    "excitasen": "excitar",
    "excitases": "excitar",
    "excitaste": "excitar",
    "excitasteis": "excitar",
    "excitativos": "excitativo",
    "excitatrices": "excitatriz",
    "excite": "excitar",
    "excitemos": "excitar",
    "exciten": "excitar",
    "excites": "excitar",
    "excito": "excitar",
    "excitoanabólica": "excitoanabólico",
    "excitoanabólicas": "excitoanabólico",
    "excitoanabólicos": "excitoanabólico",
    "excitocatabólica": "excitocatabólico",
    "excitocatabólicas": "excitocatabólico",
    "excitocatabólicos": "excitocatabólico",
    "excitrones": "excitrón",
    "excitábamos": "excitar",
    "excitáis": "excitar",
    "excitáramos": "excitar",
    "excitáremos": "excitar",
    "excitásemos": "excitar",
    "excité": "excitar",
    "excitéis": "excitar",
    "excitó": "excitar",
    "exclama": "exclamar",
    "exclamaba": "exclamar",
    "exclamabais": "exclamar",
    "exclamaban": "exclamar",
    "exclamabas": "exclamar",
    "exclamaciones": "exclamación",
    "exclamad": "exclamar",
    "exclamada": "exclamar",
    "exclamadas": "exclamar",
    "exclamado": "exclamar",
    "exclamados": "exclamar",
    "exclamamos": "exclamar",
    "exclaman": "exclamar",
    "exclamando": "exclamar",
    "exclamara": "exclamar",
    "exclamarais": "exclamar",
    "exclamaran": "exclamar",
    "exclamaras": "exclamar",
    "exclamare": "exclamar",
    "exclamareis": "exclamar",
    "exclamaremos": "exclamar",
    "exclamaren": "exclamar",
    "exclamares": "exclamar",
    "exclamaron": "exclamar",
    "exclamará": "exclamar",
    "exclamarán": "exclamar",
    "exclamarás": "exclamar",
    "exclamaré": "exclamar",
    "exclamaréis": "exclamar",
    "exclamaría": "exclamar",
    "exclamaríais": "exclamar",
    "exclamaríamos": "exclamar",
    "exclamarían": "exclamar",
    "exclamarías": "exclamar",
    "exclamas": "exclamar",
    "exclamase": "exclamar",
    "exclamaseis": "exclamar",
    "exclamasen": "exclamar",
    "exclamases": "exclamar",
    "exclamaste": "exclamar",
    "exclamasteis": "exclamar",
    "exclamativa": "exclamativo",
    "exclamativas": "exclamativo",
    "exclamativos": "exclamativo",
    "exclamatoria": "exclamatorio",
    "exclamatorias": "exclamatorio",
    "exclamatorios": "exclamatorio",
    "exclame": "exclamar",
    "exclamemos": "exclamar",
    "exclamen": "exclamar",
    "exclames": "exclamar",
    "exclamo": "exclamar",
    "exclamábamos": "exclamar",
    "exclamáis": "exclamar",
    "exclamáramos": "exclamar",
    "exclamáremos": "exclamar",
    "exclamásemos": "exclamar",
    "exclamé": "exclamar",
    "exclaméis": "exclamar",
    "exclamó": "exclamar",
    "exclaustra": "exclaustrar",
    "exclaustraba": "exclaustrar",
    "exclaustrabais": "exclaustrar",
    "exclaustraban": "exclaustrar",
    "exclaustrabas": "exclaustrar",
    "exclaustraciones": "exclaustración",
    "exclaustrad": "exclaustrar",
    "exclaustrada": "exclaustrar",
    "exclaustradas": "exclaustrar",
    "exclaustrado": "exclaustrar",
    "exclaustrados": "exclaustrar",
    "exclaustramos": "exclaustrar",
    "exclaustran": "exclaustrar",
    "exclaustrando": "exclaustrar",
    "exclaustrara": "exclaustrar",
    "exclaustrarais": "exclaustrar",
    "exclaustraran": "exclaustrar",
    "exclaustraras": "exclaustrar",
    "exclaustrare": "exclaustrar",
    "exclaustrareis": "exclaustrar",
    "exclaustraremos": "exclaustrar",
    "exclaustraren": "exclaustrar",
    "exclaustrares": "exclaustrar",
    "exclaustraron": "exclaustrar",
    "exclaustrará": "exclaustrar",
    "exclaustrarán": "exclaustrar",
    "exclaustrarás": "exclaustrar",
    "exclaustraré": "exclaustrar",
    "exclaustraréis": "exclaustrar",
    "exclaustraría": "exclaustrar",
    "exclaustraríais": "exclaustrar",
    "exclaustraríamos": "exclaustrar",
    "exclaustrarían": "exclaustrar",
    "exclaustrarías": "exclaustrar",
    "exclaustras": "exclaustrar",
    "exclaustrase": "exclaustrar",
    "exclaustraseis": "exclaustrar",
    "exclaustrasen": "exclaustrar",
    "exclaustrases": "exclaustrar",
    "exclaustraste": "exclaustrar",
    "exclaustrasteis": "exclaustrar",
    "exclaustre": "exclaustrar",
    "exclaustremos": "exclaustrar",
    "exclaustren": "exclaustrar",
    "exclaustres": "exclaustrar",
    "exclaustro": "exclaustrar",
    "exclaustrábamos": "exclaustrar",
    "exclaustráis": "exclaustrar",
    "exclaustráramos": "exclaustrar",
    "exclaustráremos": "exclaustrar",
    "exclaustrásemos": "exclaustrar",
    "exclaustré": "exclaustrar",
    "exclaustréis": "exclaustrar",
    "exclaustró": "exclaustrar",
    "exclaves": "exclave",
    "excluibles": "excluible",
    "excluid": "excluir",
    "excluida": "excluir",
    "excluidas": "excluir",
    "excluido": "excluir",
    "excluidora": "excluidor",
    "excluidoras": "excluidor",
    "excluidores": "excluidor",
    "excluidos": "excluir",
    "excluimos": "excluir",
    "excluiremos": "excluir",
    "excluirá": "excluir",
    "excluirán": "excluir",
    "excluirás": "excluir",
    "excluiré": "excluir",
    "excluiréis": "excluir",
    "excluiría": "excluir",
    "excluiríais": "excluir",
    "excluiríamos": "excluir",
    "excluirían": "excluir",
    "excluirías": "excluir",
    "excluiste": "excluir",
    "excluisteis": "excluir",
    "exclusiones": "exclusión",
    "exclusiva": "exclusivo",
    "exclusivas": "exclusivo",
    "exclusividades": "exclusividad",
    "exclusivismos": "exclusivismo",
    "exclusivistas": "exclusivista",
    "exclusivos": "exclusivo",
    "excluya": "excluir",
    "excluyamos": "excluir",
    "excluyan": "excluir",
    "excluyas": "excluir",
    "excluye": "excluir",
    "excluyen": "excluir",
    "excluyendo": "excluir",
    "excluyentes": "excluyente",
    "excluyera": "excluir",
    "excluyerais": "excluir",
    "excluyeran": "excluir",
    "excluyeras": "excluir",
    "excluyere": "excluir",
    "excluyereis": "excluir",
    "excluyeren": "excluir",
    "excluyeres": "excluir",
    "excluyeron": "excluir",
    "excluyes": "excluir",
    "excluyese": "excluir",
    "excluyeseis": "excluir",
    "excluyesen": "excluir",
    "excluyeses": "excluir",
    "excluyo": "excluir",
    "excluyáis": "excluir",
    "excluyéramos": "excluir",
    "excluyéremos": "excluir",
    "excluyésemos": "excluir",
    "excluyó": "excluir",
    "excluí": "excluir",
    "excluía": "excluir",
    "excluíais": "excluir",
    "excluíamos": "excluir",
    "excluían": "excluir",
    "excluías": "excluir",
    "excluís": "excluir",
    "excombatientes": "excombatiente",
    "excompañera": "excompañero",
    "excompañeras": "excompañero",
    "excompañeros": "excompañero",
    "excomulga": "excomulgar",
    "excomulgaba": "excomulgar",
    "excomulgabais": "excomulgar",
    "excomulgaban": "excomulgar",
    "excomulgabas": "excomulgar",
    "excomulgaciones": "excomulgación",
    "excomulgad": "excomulgar",
    "excomulgada": "excomulgar",
    "excomulgadas": "excomulgar",
    "excomulgado": "excomulgar",
    "excomulgadora": "excomulgador",
    "excomulgadoras": "excomulgador",
    "excomulgadores": "excomulgador",
    "excomulgados": "excomulgar",
    "excomulgamos": "excomulgar",
    "excomulgan": "excomulgar",
    "excomulgando": "excomulgar",
    "excomulgara": "excomulgar",
    "excomulgarais": "excomulgar",
    "excomulgaran": "excomulgar",
    "excomulgaras": "excomulgar",
    "excomulgare": "excomulgar",
    "excomulgareis": "excomulgar",
    "excomulgaremos": "excomulgar",
    "excomulgaren": "excomulgar",
    "excomulgares": "excomulgar",
    "excomulgaron": "excomulgar",
    "excomulgará": "excomulgar",
    "excomulgarán": "excomulgar",
    "excomulgarás": "excomulgar",
    "excomulgaré": "excomulgar",
    "excomulgaréis": "excomulgar",
    "excomulgaría": "excomulgar",
    "excomulgaríais": "excomulgar",
    "excomulgaríamos": "excomulgar",
    "excomulgarían": "excomulgar",
    "excomulgarías": "excomulgar",
    "excomulgas": "excomulgar",
    "excomulgase": "excomulgar",
    "excomulgaseis": "excomulgar",
    "excomulgasen": "excomulgar",
    "excomulgases": "excomulgar",
    "excomulgaste": "excomulgar",
    "excomulgasteis": "excomulgar",
    "excomulgo": "excomulgar",
    "excomulgue": "excomulgar",
    "excomulguemos": "excomulgar",
    "excomulguen": "excomulgar",
    "excomulgues": "excomulgar",
    "excomulgué": "excomulgar",
    "excomulguéis": "excomulgar",
    "excomulgábamos": "excomulgar",
    "excomulgáis": "excomulgar",
    "excomulgáramos": "excomulgar",
    "excomulgáremos": "excomulgar",
    "excomulgásemos": "excomulgar",
    "excomulgó": "excomulgar",
    "excomuniones": "excomunión",
    "excoria": "excoriar",
    "excoriaba": "excoriar",
    "excoriabais": "excoriar",
    "excoriaban": "excoriar",
    "excoriabas": "excoriar",
    "excoriaciones": "excoriación",
    "excoriad": "excoriar",
    "excoriada": "excoriar",
    "excoriadas": "excoriar",
    "excoriado": "excoriar",
    "excoriados": "excoriar",
    "excoriamos": "excoriar",
    "excorian": "excoriar",
    "excoriando": "excoriar",
    "excoriara": "excoriar",
    "excoriarais": "excoriar",
    "excoriaran": "excoriar",
    "excoriaras": "excoriar",
    "excoriare": "excoriar",
    "excoriareis": "excoriar",
    "excoriaremos": "excoriar",
    "excoriaren": "excoriar",
    "excoriares": "excoriar",
    "excoriaron": "excoriar",
    "excoriará": "excoriar",
    "excoriarán": "excoriar",
    "excoriarás": "excoriar",
    "excoriaré": "excoriar",
    "excoriaréis": "excoriar",
    "excoriaría": "excoriar",
    "excoriaríais": "excoriar",
    "excoriaríamos": "excoriar",
    "excoriarían": "excoriar",
    "excoriarías": "excoriar",
    "excorias": "excoriar",
    "excoriase": "excoriar",
    "excoriaseis": "excoriar",
    "excoriasen": "excoriar",
    "excoriases": "excoriar",
    "excoriaste": "excoriar",
    "excoriasteis": "excoriar",
    "excorie": "excoriar",
    "excoriemos": "excoriar",
    "excorien": "excoriar",
    "excories": "excoriar",
    "excorio": "excoriar",
    "excoriábamos": "excoriar",
    "excoriáis": "excoriar",
    "excoriáramos": "excoriar",
    "excoriáremos": "excoriar",
    "excoriásemos": "excoriar",
    "excorié": "excoriar",
    "excoriéis": "excoriar",
    "excorió": "excoriar",
    "excrecencias": "excrecencia",
    "excreciones": "excreción",
    "excrementa": "excrementar",
    "excrementaba": "excrementar",
    "excrementabais": "excrementar",
    "excrementaban": "excrementar",
    "excrementabas": "excrementar",
    "excrementad": "excrementar",
    "excrementada": "excrementar",
    "excrementadas": "excrementar",
    "excrementado": "excrementar",
    "excrementados": "excrementar",
    "excrementales": "excremental",
    "excrementamos": "excrementar",
    "excrementan": "excrementar",
    "excrementando": "excrementar",
    "excrementara": "excrementar",
    "excrementarais": "excrementar",
    "excrementaran": "excrementar",
    "excrementaras": "excrementar",
    "excrementare": "excrementar",
    "excrementareis": "excrementar",
    "excrementaremos": "excrementar",
    "excrementaren": "excrementar",
    "excrementares": "excrementar",
    "excrementaron": "excrementar",
    "excrementará": "excrementar",
    "excrementarán": "excrementar",
    "excrementarás": "excrementar",
    "excrementaré": "excrementar",
    "excrementaréis": "excrementar",
    "excrementaría": "excrementar",
    "excrementaríais": "excrementar",
    "excrementaríamos": "excrementar",
    "excrementarían": "excrementar",
    "excrementarías": "excrementar",
    "excrementas": "excrementar",
    "excrementase": "excrementar",
    "excrementaseis": "excrementar",
    "excrementasen": "excrementar",
    "excrementases": "excrementar",
    "excrementaste": "excrementar",
    "excrementasteis": "excrementar",
    "excremente": "excrementar",
    "excrementemos": "excrementar",
    "excrementen": "excrementar",
    "excrementes": "excrementar",
    "excrementicia": "excrementicio",
    "excrementicias": "excrementicio",
    "excrementicios": "excrementicio",
    "excremento": "excrementar",
    "excrementos": "excremento",
    "excrementosa": "excrementoso",
    "excrementosas": "excrementoso",
    "excrementosos": "excrementoso",
    "excrementábamos": "excrementar",
    "excrementáis": "excrementar",
    "excrementáramos": "excrementar",
    "excrementáremos": "excrementar",
    "excrementásemos": "excrementar",
    "excrementé": "excrementar",
    "excrementéis": "excrementar",
    "excrementó": "excrementar",
    "excrescencias": "excrescencia",
    "excreta": "excreto",
    "excretaba": "excretar",
    "excretabais": "excretar",
    "excretaban": "excretar",
    "excretabas": "excretar",
    "excretad": "excretar",
    "excretada": "excretar",
    "excretadas": "excretar",
    "excretado": "excretar",
    "excretados": "excretar",
    "excretamos": "excretar",
    "excretan": "excretar",
    "excretando": "excretar",
    "excretara": "excretar",
    "excretarais": "excretar",
    "excretaran": "excretar",
    "excretaras": "excretar",
    "excretare": "excretar",
    "excretareis": "excretar",
    "excretaremos": "excretar",
    "excretaren": "excretar",
    "excretares": "excretar",
    "excretaron": "excretar",
    "excretará": "excretar",
    "excretarán": "excretar",
    "excretarás": "excretar",
    "excretaré": "excretar",
    "excretaréis": "excretar",
    "excretaría": "excretar",
    "excretaríais": "excretar",
    "excretaríamos": "excretar",
    "excretarían": "excretar",
    "excretarías": "excretar",
    "excretas": "excreto",
    "excretase": "excretar",
    "excretaseis": "excretar",
    "excretasen": "excretar",
    "excretases": "excretar",
    "excretaste": "excretar",
    "excretasteis": "excretar",
    "excrete": "excretar",
    "excretemos": "excretar",
    "excreten": "excretar",
    "excretes": "excretar",
    "excreto": "excretar",
    "excretora": "excretor",
    "excretoras": "excretor",
    "excretores": "excretor",
    "excretoria": "excretorio",
    "excretorias": "excretorio",
    "excretorios": "excretorio",
    "excretos": "excreto",
    "excretábamos": "excretar",
    "excretáis": "excretar",
    "excretáramos": "excretar",
    "excretáremos": "excretar",
    "excretásemos": "excretar",
    "excreté": "excretar",
    "excretéis": "excretar",
    "excretó": "excretar",
    "exculpa": "exculpar",
    "exculpaba": "exculpar",
    "exculpabais": "exculpar",
    "exculpaban": "exculpar",
    "exculpabas": "exculpar",
    "exculpaciones": "exculpación",
    "exculpad": "exculpar",
    "exculpada": "exculpar",
    "exculpadas": "exculpar",
    "exculpado": "exculpar",
    "exculpados": "exculpar",
    "exculpamos": "exculpar",
    "exculpan": "exculpar",
    "exculpando": "exculpar",
    "exculpantes": "exculpante",
    "exculpara": "exculpar",
    "exculparais": "exculpar",
    "exculparan": "exculpar",
    "exculparas": "exculpar",
    "exculpare": "exculpar",
    "exculpareis": "exculpar",
    "exculparemos": "exculpar",
    "exculparen": "exculpar",
    "exculpares": "exculpar",
    "exculparon": "exculpar",
    "exculpará": "exculpar",
    "exculparán": "exculpar",
    "exculparás": "exculpar",
    "exculparé": "exculpar",
    "exculparéis": "exculpar",
    "exculparía": "exculpar",
    "exculparíais": "exculpar",
    "exculparíamos": "exculpar",
    "exculparían": "exculpar",
    "exculparías": "exculpar",
    "exculpas": "exculpar",
    "exculpase": "exculpar",
    "exculpaseis": "exculpar",
    "exculpasen": "exculpar",
    "exculpases": "exculpar",
    "exculpaste": "exculpar",
    "exculpasteis": "exculpar",
    "exculpatoria": "exculpatorio",
    "exculpatorias": "exculpatorio",
    "exculpatorios": "exculpatorio",
    "exculpe": "exculpar",
    "exculpemos": "exculpar",
    "exculpen": "exculpar",
    "exculpes": "exculpar",
    "exculpo": "exculpar",
    "exculpábamos": "exculpar",
    "exculpáis": "exculpar",
    "exculpáramos": "exculpar",
    "exculpáremos": "exculpar",
    "exculpásemos": "exculpar",
    "exculpé": "exculpar",
    "exculpéis": "exculpar",
    "exculpó": "exculpar",
    "excursiones": "excursión",
    "excursionismos": "excursionismo",
    "excursionistas": "excursionista",
    "excusa": "excusar",
    "excusaba": "excusar",
    "excusabais": "excusar",
    "excusaban": "excusar",
    "excusabas": "excusar",
    "excusables": "excusable",
    "excusad": "excusar",
    "excusada": "excusar",
    "excusadas": "excusar",
    "excusado": "excusar",
    "excusadora": "excusador",
    "excusadoras": "excusador",
    "excusadores": "excusador",
    "excusados": "excusar",
    "excusalíes": "excusalí",
    "excusalís": "excusalí",
    "excusamos": "excusar",
    "excusan": "excusar",
    "excusando": "excusar",
    "excusara": "excusar",
    "excusarais": "excusar",
    "excusaran": "excusar",
    "excusaras": "excusar",
    "excusare": "excusar",
    "excusareis": "excusar",
    "excusaremos": "excusar",
    "excusaren": "excusar",
    "excusares": "excusar",
    "excusaron": "excusar",
    "excusará": "excusar",
    "excusarán": "excusar",
    "excusarás": "excusar",
    "excusaré": "excusar",
    "excusaréis": "excusar",
    "excusaría": "excusar",
    "excusaríais": "excusar",
    "excusaríamos": "excusar",
    "excusarían": "excusar",
    "excusarías": "excusar",
    "excusas": "excusar",
    "excusase": "excusar",
    "excusaseis": "excusar",
    "excusasen": "excusar",
    "excusases": "excusar",
    "excusaste": "excusar",
    "excusasteis": "excusar",
    "excuse": "excusar",
    "excusemos": "excusar",
    "excusen": "excusar",
    "excuses": "excusar",
    "excusiones": "excusión",
    "excuso": "excusar",
    "excusos": "excuso",
    "excusábamos": "excusar",
    "excusáis": "excusar",
    "excusáramos": "excusar",
    "excusáremos": "excusar",
    "excusásemos": "excusar",
    "excusé": "excusar",
    "excuséis": "excusar",
    "excusó": "excusar",
    "excéntrica": "excéntrico",
    "excéntricas": "excéntrico",
    "excéntricos": "excéntrico",
    "exdeportivistas": "exdeportivista",
    "execra": "execrar",
    "execraba": "execrar",
    "execrabais": "execrar",
    "execraban": "execrar",
    "execrabas": "execrar",
    "execrables": "execrable",
    "execraciones": "execración",
    "execrad": "execrar",
    "execrada": "execrar",
    "execradas": "execrar",
    "execrado": "execrar",
    "execradora": "execrador",
    "execradoras": "execrador",
    "execradores": "execrador",
    "execrados": "execrar",
    "execramos": "execrar",
    "execran": "execrar",
    "execrando": "execrar",
    "execrara": "execrar",
    "execrarais": "execrar",
    "execraran": "execrar",
    "execraras": "execrar",
    "execrare": "execrar",
    "execrareis": "execrar",
    "execraremos": "execrar",
    "execraren": "execrar",
    "execrares": "execrar",
    "execraron": "execrar",
    "execrará": "execrar",
    "execrarán": "execrar",
    "execrarás": "execrar",
    "execraré": "execrar",
    "execraréis": "execrar",
    "execraría": "execrar",
    "execraríais": "execrar",
    "execraríamos": "execrar",
    "execrarían": "execrar",
    "execrarías": "execrar",
    "execras": "execrar",
    "execrase": "execrar",
    "execraseis": "execrar",
    "execrasen": "execrar",
    "execrases": "execrar",
    "execraste": "execrar",
    "execrasteis": "execrar",
    "execrativa": "execrativo",
    "execrativas": "execrativo",
    "execrativos": "execrativo",
    "execratoria": "execratorio",
    "execratorias": "execratorio",
    "execratorios": "execratorio",
    "execre": "execrar",
    "execremos": "execrar",
    "execren": "execrar",
    "execres": "execrar",
    "execro": "execrar",
    "execrábamos": "execrar",
    "execráis": "execrar",
    "execráramos": "execrar",
    "execráremos": "execrar",
    "execrásemos": "execrar",
    "execré": "execrar",
    "execréis": "execrar",
    "execró": "execrar",
    "exedras": "exedra",
    "exegetas": "exegeta",
    "exegética": "exegético",
    "exegéticas": "exegético",
    "exegéticos": "exegético",
    "exenciones": "exención",
    "exenta": "exento",
    "exentaba": "exentar",
    "exentabais": "exentar",
    "exentaban": "exentar",
    "exentabas": "exentar",
    "exentad": "exentar",
    "exentada": "exentar",
    "exentadas": "exentar",
    "exentado": "exentar",
    "exentados": "exentar",
    "exentamos": "exentar",
    "exentan": "exentar",
    "exentando": "exentar",
    "exentara": "exentar",
    "exentarais": "exentar",
    "exentaran": "exentar",
    "exentaras": "exentar",
    "exentare": "exentar",
    "exentareis": "exentar",
    "exentaremos": "exentar",
    "exentaren": "exentar",
    "exentares": "exentar",
    "exentaron": "exentar",
    "exentará": "exentar",
    "exentarán": "exentar",
    "exentarás": "exentar",
    "exentaré": "exentar",
    "exentaréis": "exentar",
    "exentaría": "exentar",
    "exentaríais": "exentar",
    "exentaríamos": "exentar",
    "exentarían": "exentar",
    "exentarías": "exentar",
    "exentas": "exento",
    "exentase": "exentar",
    "exentaseis": "exentar",
    "exentasen": "exentar",
    "exentases": "exentar",
    "exentaste": "exentar",
    "exentasteis": "exentar",
    "exente": "exentar",
    "exentemos": "exentar",
    "exenten": "exentar",
    "exentes": "exentar",
    "exento": "exentar",
    "exentos": "exento",
    "exentábamos": "exentar",
    "exentáis": "exentar",
    "exentáramos": "exentar",
    "exentáremos": "exentar",
    "exentásemos": "exentar",
    "exenté": "exentar",
    "exentéis": "exentar",
    "exentó": "exentar",
    "exequatures": "exequatur",
    "exequias": "exequia",
    "exequibles": "exequible",
    "exequátures": "exequátur",
    "exequáturs": "exequátur",
    "exergos": "exergo",
    "exfolia": "exfoliar",
    "exfoliaba": "exfoliar",
    "exfoliabais": "exfoliar",
    "exfoliaban": "exfoliar",
    "exfoliabas": "exfoliar",
    "exfoliaciones": "exfoliación",
    "exfoliad": "exfoliar",
    "exfoliada": "exfoliar",
    "exfoliadas": "exfoliar",
    "exfoliado": "exfoliar",
    "exfoliados": "exfoliar",
    "exfoliamos": "exfoliar",
    "exfolian": "exfoliar",
    "exfoliando": "exfoliar",
    "exfoliantes": "exfoliante",
    "exfoliara": "exfoliar",
    "exfoliarais": "exfoliar",
    "exfoliaran": "exfoliar",
    "exfoliaras": "exfoliar",
    "exfoliare": "exfoliar",
    "exfoliareis": "exfoliar",
    "exfoliaremos": "exfoliar",
    "exfoliaren": "exfoliar",
    "exfoliares": "exfoliar",
    "exfoliaron": "exfoliar",
    "exfoliará": "exfoliar",
    "exfoliarán": "exfoliar",
    "exfoliarás": "exfoliar",
    "exfoliaré": "exfoliar",
    "exfoliaréis": "exfoliar",
    "exfoliaría": "exfoliar",
    "exfoliaríais": "exfoliar",
    "exfoliaríamos": "exfoliar",
    "exfoliarían": "exfoliar",
    "exfoliarías": "exfoliar",
    "exfolias": "exfoliar",
    "exfoliase": "exfoliar",
    "exfoliaseis": "exfoliar",
    "exfoliasen": "exfoliar",
    "exfoliases": "exfoliar",
    "exfoliaste": "exfoliar",
    "exfoliasteis": "exfoliar",
    "exfoliativa": "exfoliativo",
    "exfoliativas": "exfoliativo",
    "exfoliativos": "exfoliativo",
    "exfolie": "exfoliar",
    "exfoliemos": "exfoliar",
    "exfolien": "exfoliar",
    "exfolies": "exfoliar",
    "exfolio": "exfoliar",
    "exfoliábamos": "exfoliar",
    "exfoliáis": "exfoliar",
    "exfoliáramos": "exfoliar",
    "exfoliáremos": "exfoliar",
    "exfoliásemos": "exfoliar",
    "exfolié": "exfoliar",
    "exfoliéis": "exfoliar",
    "exfolió": "exfoliar",
    "exhala": "exhalar",
    "exhalaba": "exhalar",
    "exhalabais": "exhalar",
    "exhalaban": "exhalar",
    "exhalabas": "exhalar",
    "exhalaciones": "exhalación",
    "exhalad": "exhalar",
    "exhalada": "exhalar",
    "exhaladas": "exhalar",
    "exhalado": "exhalar",
    "exhaladora": "exhalador",
    "exhaladoras": "exhalador",
    "exhaladores": "exhalador",
    "exhalados": "exhalar",
    "exhalamos": "exhalar",
    "exhalan": "exhalar",
    "exhalando": "exhalar",
    "exhalara": "exhalar",
    "exhalarais": "exhalar",
    "exhalaran": "exhalar",
    "exhalaras": "exhalar",
    "exhalare": "exhalar",
    "exhalareis": "exhalar",
    "exhalaremos": "exhalar",
    "exhalaren": "exhalar",
    "exhalares": "exhalar",
    "exhalaron": "exhalar",
    "exhalará": "exhalar",
    "exhalarán": "exhalar",
    "exhalarás": "exhalar",
    "exhalaré": "exhalar",
    "exhalaréis": "exhalar",
    "exhalaría": "exhalar",
    "exhalaríais": "exhalar",
    "exhalaríamos": "exhalar",
    "exhalarían": "exhalar",
    "exhalarías": "exhalar",
    "exhalas": "exhalar",
    "exhalase": "exhalar",
    "exhalaseis": "exhalar",
    "exhalasen": "exhalar",
    "exhalases": "exhalar",
    "exhalaste": "exhalar",
    "exhalasteis": "exhalar",
    "exhale": "exhalar",
    "exhalemos": "exhalar",
    "exhalen": "exhalar",
    "exhales": "exhalar",
    "exhalo": "exhalar",
    "exhalábamos": "exhalar",
    "exhaláis": "exhalar",
    "exhaláramos": "exhalar",
    "exhaláremos": "exhalar",
    "exhalásemos": "exhalar",
    "exhalé": "exhalar",
    "exhaléis": "exhalar",
    "exhaló": "exhalar",
    "exhausta": "exhausto",
    "exhaustas": "exhausto",
    "exhaustiva": "exhaustivo",
    "exhaustivas": "exhaustivo",
    "exhaustivos": "exhaustivo",
    "exhaustos": "exhausto",
    "exhereda": "exheredar",
    "exheredaba": "exheredar",
    "exheredabais": "exheredar",
    "exheredaban": "exheredar",
    "exheredabas": "exheredar",
    "exheredad": "exheredar",
    "exheredada": "exheredar",
    "exheredadas": "exheredar",
    "exheredado": "exheredar",
    "exheredados": "exheredar",
    "exheredamos": "exheredar",
    "exheredan": "exheredar",
    "exheredando": "exheredar",
    "exheredara": "exheredar",
    "exheredarais": "exheredar",
    "exheredaran": "exheredar",
    "exheredaras": "exheredar",
    "exheredare": "exheredar",
    "exheredareis": "exheredar",
    "exheredaremos": "exheredar",
    "exheredaren": "exheredar",
    "exheredares": "exheredar",
    "exheredaron": "exheredar",
    "exheredará": "exheredar",
    "exheredarán": "exheredar",
    "exheredarás": "exheredar",
    "exheredaré": "exheredar",
    "exheredaréis": "exheredar",
    "exheredaría": "exheredar",
    "exheredaríais": "exheredar",
    "exheredaríamos": "exheredar",
    "exheredarían": "exheredar",
    "exheredarías": "exheredar",
    "exheredas": "exheredar",
    "exheredase": "exheredar",
    "exheredaseis": "exheredar",
    "exheredasen": "exheredar",
    "exheredases": "exheredar",
    "exheredaste": "exheredar",
    "exheredasteis": "exheredar",
    "exherede": "exheredar",
    "exheredemos": "exheredar",
    "exhereden": "exheredar",
    "exheredes": "exheredar",
    "exheredo": "exheredar",
    "exheredábamos": "exheredar",
    "exheredáis": "exheredar",
    "exheredáramos": "exheredar",
    "exheredáremos": "exheredar",
    "exheredásemos": "exheredar",
    "exheredé": "exheredar",
    "exheredéis": "exheredar",
    "exheredó": "exheredar",
    "exhiba": "exhibir",
    "exhibamos": "exhibir",
    "exhiban": "exhibir",
    "exhibas": "exhibir",
    "exhibe": "exhibir",
    "exhiben": "exhibir",
    "exhibes": "exhibir",
    "exhibiciones": "exhibición",
    "exhibicionismos": "exhibicionismo",
    "exhibicionistas": "exhibicionista",
    "exhibid": "exhibir",
    "exhibida": "exhibir",
    "exhibidas": "exhibir",
    "exhibido": "exhibir",
    "exhibidos": "exhibir",
    "exhibiendo": "exhibir",
    "exhibiera": "exhibir",
    "exhibierais": "exhibir",
    "exhibieran": "exhibir",
    "exhibieras": "exhibir",
    "exhibiere": "exhibir",
    "exhibiereis": "exhibir",
    "exhibieren": "exhibir",
    "exhibieres": "exhibir",
    "exhibieron": "exhibir",
    "exhibiese": "exhibir",
    "exhibieseis": "exhibir",
    "exhibiesen": "exhibir",
    "exhibieses": "exhibir",
    "exhibimos": "exhibir",
    "exhibiremos": "exhibir",
    "exhibirá": "exhibir",
    "exhibirán": "exhibir",
    "exhibirás": "exhibir",
    "exhibiré": "exhibir",
    "exhibiréis": "exhibir",
    "exhibiría": "exhibir",
    "exhibiríais": "exhibir",
    "exhibiríamos": "exhibir",
    "exhibirían": "exhibir",
    "exhibirías": "exhibir",
    "exhibiste": "exhibir",
    "exhibisteis": "exhibir",
    "exhibitoria": "exhibitorio",
    "exhibitorias": "exhibitorio",
    "exhibitorios": "exhibitorio",
    "exhibiéramos": "exhibir",
    "exhibiéremos": "exhibir",
    "exhibiésemos": "exhibir",
    "exhibió": "exhibir",
    "exhibo": "exhibir",
    "exhibáis": "exhibir",
    "exhibí": "exhibir",
    "exhibía": "exhibir",
    "exhibíais": "exhibir",
    "exhibíamos": "exhibir",
    "exhibían": "exhibir",
    "exhibías": "exhibir",
    "exhibís": "exhibir",
    "exhorta": "exhortar",
    "exhortaba": "exhortar",
    "exhortabais": "exhortar",
    "exhortaban": "exhortar",
    "exhortabas": "exhortar",
    "exhortaciones": "exhortación",
    "exhortad": "exhortar",
    "exhortada": "exhortar",
    "exhortadas": "exhortar",
    "exhortado": "exhortar",
    "exhortadora": "exhortador",
    "exhortadoras": "exhortador",
    "exhortadores": "exhortador",
    "exhortados": "exhortar",
    "exhortamos": "exhortar",
    "exhortan": "exhortar",
    "exhortando": "exhortar",
    "exhortantes": "exhortante",
    "exhortara": "exhortar",
    "exhortarais": "exhortar",
    "exhortaran": "exhortar",
    "exhortaras": "exhortar",
    "exhortare": "exhortar",
    "exhortareis": "exhortar",
    "exhortaremos": "exhortar",
    "exhortaren": "exhortar",
    "exhortares": "exhortar",
    "exhortaron": "exhortar",
    "exhortará": "exhortar",
    "exhortarán": "exhortar",
    "exhortarás": "exhortar",
    "exhortaré": "exhortar",
    "exhortaréis": "exhortar",
    "exhortaría": "exhortar",
    "exhortaríais": "exhortar",
    "exhortaríamos": "exhortar",
    "exhortarían": "exhortar",
    "exhortarías": "exhortar",
    "exhortas": "exhortar",
    "exhortase": "exhortar",
    "exhortaseis": "exhortar",
    "exhortasen": "exhortar",
    "exhortases": "exhortar",
    "exhortaste": "exhortar",
    "exhortasteis": "exhortar",
    "exhortativa": "exhortativo",
    "exhortativas": "exhortativo",
    "exhortativos": "exhortativo",
    "exhortatoria": "exhortatorio",
    "exhortatorias": "exhortatorio",
    "exhortatorios": "exhortatorio",
    "exhorte": "exhortar",
    "exhortemos": "exhortar",
    "exhorten": "exhortar",
    "exhortes": "exhortar",
    "exhorto": "exhortar",
    "exhortos": "exhorto",
    "exhortábamos": "exhortar",
    "exhortáis": "exhortar",
    "exhortáramos": "exhortar",
    "exhortáremos": "exhortar",
    "exhortásemos": "exhortar",
    "exhorté": "exhortar",
    "exhortéis": "exhortar",
    "exhortó": "exhortar",
    "exhuberantes": "exhuberante",
    "exhuma": "exhumar",
    "exhumaba": "exhumar",
    "exhumabais": "exhumar",
    "exhumaban": "exhumar",
    "exhumabas": "exhumar",
    "exhumaciones": "exhumación",
    "exhumad": "exhumar",
    "exhumada": "exhumar",
    "exhumadas": "exhumar",
    "exhumado": "exhumar",
    "exhumadora": "exhumador",
    "exhumadoras": "exhumador",
    "exhumadores": "exhumador",
    "exhumados": "exhumar",
    "exhumamos": "exhumar",
    "exhuman": "exhumar",
    "exhumando": "exhumar",
    "exhumara": "exhumar",
    "exhumarais": "exhumar",
    "exhumaran": "exhumar",
    "exhumaras": "exhumar",
    "exhumare": "exhumar",
    "exhumareis": "exhumar",
    "exhumaremos": "exhumar",
    "exhumaren": "exhumar",
    "exhumares": "exhumar",
    "exhumaron": "exhumar",
    "exhumará": "exhumar",
    "exhumarán": "exhumar",
    "exhumarás": "exhumar",
    "exhumaré": "exhumar",
    "exhumaréis": "exhumar",
    "exhumaría": "exhumar",
    "exhumaríais": "exhumar",
    "exhumaríamos": "exhumar",
    "exhumarían": "exhumar",
    "exhumarías": "exhumar",
    "exhumas": "exhumar",
    "exhumase": "exhumar",
    "exhumaseis": "exhumar",
    "exhumasen": "exhumar",
    "exhumases": "exhumar",
    "exhumaste": "exhumar",
    "exhumasteis": "exhumar",
    "exhume": "exhumar",
    "exhumemos": "exhumar",
    "exhumen": "exhumar",
    "exhumes": "exhumar",
    "exhumo": "exhumar",
    "exhumábamos": "exhumar",
    "exhumáis": "exhumar",
    "exhumáramos": "exhumar",
    "exhumáremos": "exhumar",
    "exhumásemos": "exhumar",
    "exhumé": "exhumar",
    "exhuméis": "exhumar",
    "exhumó": "exhumar",
    "exige": "exigir",
    "exigen": "exigir",
    "exigencias": "exigencia",
    "exigentes": "exigente",
    "exiges": "exigir",
    "exigibilidades": "exigibilidad",
    "exigibles": "exigible",
    "exigid": "exigir",
    "exigida": "exigir",
    "exigidas": "exigir",
    "exigido": "exigir",
    "exigidos": "exigir",
    "exigiendo": "exigir",
    "exigiera": "exigir",
    "exigierais": "exigir",
    "exigieran": "exigir",
    "exigieras": "exigir",
    "exigiere": "exigir",
    "exigiereis": "exigir",
    "exigieren": "exigir",
    "exigieres": "exigir",
    "exigieron": "exigir",
    "exigiese": "exigir",
    "exigieseis": "exigir",
    "exigiesen": "exigir",
    "exigieses": "exigir",
    "exigimos": "exigir",
    "exigiremos": "exigir",
    "exigirá": "exigir",
    "exigirán": "exigir",
    "exigirás": "exigir",
    "exigiré": "exigir",
    "exigiréis": "exigir",
    "exigiría": "exigir",
    "exigiríais": "exigir",
    "exigiríamos": "exigir",
    "exigirían": "exigir",
    "exigirías": "exigir",
    "exigiste": "exigir",
    "exigisteis": "exigir",
    "exigiéramos": "exigir",
    "exigiéremos": "exigir",
    "exigiésemos": "exigir",
    "exigió": "exigir",
    "exigua": "exiguo",
    "exiguas": "exiguo",
    "exiguos": "exiguo",
    "exigí": "exigir",
    "exigía": "exigir",
    "exigíais": "exigir",
    "exigíamos": "exigir",
    "exigían": "exigir",
    "exigías": "exigir",
    "exigís": "exigir",
    "exigüidades": "exigüidad",
    "exija": "exigir",
    "exijamos": "exigir",
    "exijan": "exigir",
    "exijas": "exigir",
    "exijo": "exigir",
    "exijáis": "exigir",
    "exila": "exilar",
    "exilaba": "exilar",
    "exilabais": "exilar",
    "exilaban": "exilar",
    "exilabas": "exilar",
    "exilad": "exilar",
    "exilada": "exilar",
    "exiladas": "exilar",
    "exilado": "exilar",
    "exilados": "exilar",
    "exilamos": "exilar",
    "exilan": "exilar",
    "exilando": "exilar",
    "exilara": "exilar",
    "exilarais": "exilar",
    "exilaran": "exilar",
    "exilaras": "exilar",
    "exilarcas": "exilarca",
    "exilare": "exilar",
    "exilareis": "exilar",
    "exilaremos": "exilar",
    "exilaren": "exilar",
    "exilares": "exilar",
    "exilaron": "exilar",
    "exilará": "exilar",
    "exilarán": "exilar",
    "exilarás": "exilar",
    "exilaré": "exilar",
    "exilaréis": "exilar",
    "exilaría": "exilar",
    "exilaríais": "exilar",
    "exilaríamos": "exilar",
    "exilarían": "exilar",
    "exilarías": "exilar",
    "exilas": "exilar",
    "exilase": "exilar",
    "exilaseis": "exilar",
    "exilasen": "exilar",
    "exilases": "exilar",
    "exilaste": "exilar",
    "exilasteis": "exilar",
    "exile": "exilar",
    "exilemos": "exilar",
    "exilen": "exilar",
    "exiles": "exilar",
    "exilia": "exiliar",
    "exiliaba": "exiliar",
    "exiliabais": "exiliar",
    "exiliaban": "exiliar",
    "exiliabas": "exiliar",
    "exiliad": "exiliar",
    "exiliada": "exiliar",
    "exiliadas": "exiliar",
    "exiliado": "exiliar",
    "exiliados": "exiliar",
    "exiliamos": "exiliar",
    "exilian": "exiliar",
    "exiliando": "exiliar",
    "exiliara": "exiliar",
    "exiliarais": "exiliar",
    "exiliaran": "exiliar",
    "exiliaras": "exiliar",
    "exiliare": "exiliar",
    "exiliareis": "exiliar",
    "exiliaremos": "exiliar",
    "exiliaren": "exiliar",
    "exiliares": "exiliar",
    "exiliaron": "exiliar",
    "exiliará": "exiliar",
    "exiliarán": "exiliar",
    "exiliarás": "exiliar",
    "exiliaré": "exiliar",
    "exiliaréis": "exiliar",
    "exiliaría": "exiliar",
    "exiliaríais": "exiliar",
    "exiliaríamos": "exiliar",
    "exiliarían": "exiliar",
    "exiliarías": "exiliar",
    "exilias": "exiliar",
    "exiliase": "exiliar",
    "exiliaseis": "exiliar",
    "exiliasen": "exiliar",
    "exiliases": "exiliar",
    "exiliaste": "exiliar",
    "exiliasteis": "exiliar",
    "exilie": "exiliar",
    "exiliemos": "exiliar",
    "exilien": "exiliar",
    "exilies": "exiliar",
    "exilio": "exiliar",
    "exilios": "exilio",
    "exiliábamos": "exiliar",
    "exiliáis": "exiliar",
    "exiliáramos": "exiliar",
    "exiliáremos": "exiliar",
    "exiliásemos": "exiliar",
    "exilié": "exiliar",
    "exiliéis": "exiliar",
    "exilió": "exiliar",
    "exilo": "exilar",
    "exilábamos": "exilar",
    "exiláis": "exilar",
    "exiláramos": "exilar",
    "exiláremos": "exilar",
    "exilásemos": "exilar",
    "exilé": "exilar",
    "exiléis": "exilar",
    "exiló": "exilar",
    "exima": "eximir",
    "eximamos": "eximir",
    "eximan": "eximir",
    "eximas": "eximir",
    "exime": "eximir",
    "eximen": "eximir",
    "eximentes": "eximente",
    "eximes": "eximir",
    "eximia": "eximio",
    "eximias": "eximio",
    "eximid": "eximir",
    "eximida": "eximir",
    "eximidas": "eximir",
    "eximido": "eximir",
    "eximidos": "eximir",
    "eximiendo": "eximir",
    "eximiera": "eximir",
    "eximierais": "eximir",
    "eximieran": "eximir",
    "eximieras": "eximir",
    "eximiere": "eximir",
    "eximiereis": "eximir",
    "eximieren": "eximir",
    "eximieres": "eximir",
    "eximieron": "eximir",
    "eximiese": "eximir",
    "eximieseis": "eximir",
    "eximiesen": "eximir",
    "eximieses": "eximir",
    "eximimos": "eximir",
    "eximios": "eximio",
    "eximiremos": "eximir",
    "eximirá": "eximir",
    "eximirán": "eximir",
    "eximirás": "eximir",
    "eximiré": "eximir",
    "eximiréis": "eximir",
    "eximiría": "eximir",
    "eximiríais": "eximir",
    "eximiríamos": "eximir",
    "eximirían": "eximir",
    "eximirías": "eximir",
    "eximiste": "eximir",
    "eximisteis": "eximir",
    "eximiéramos": "eximir",
    "eximiéremos": "eximir",
    "eximiésemos": "eximir",
    "eximió": "eximir",
    "eximo": "eximir",
    "eximáis": "eximir",
    "eximí": "eximir",
    "eximía": "eximir",
    "eximíais": "eximir",
    "eximíamos": "eximir",
    "eximían": "eximir",
    "eximías": "eximir",
    "eximís": "eximir",
    "exinaniciones": "exinanición",
    "exinas": "exina",
    "exinternacionales": "exinternacional",
    "exista": "existir",
    "existamos": "existir",
    "existan": "existir",
    "existas": "existir",
    "existe": "existir",
    "existen": "existir",
    "existenciales": "existencial",
    "existencialismos": "existencialismo",
    "existencialistas": "existencialista",
    "existencias": "existencia",
    "existentes": "existente",
    "existes": "existir",
    "existid": "existir",
    "existida": "existir",
    "existidas": "existir",
    "existido": "existir",
    "existidos": "existir",
    "existiendo": "existir",
    "existiera": "existir",
    "existierais": "existir",
    "existieran": "existir",
    "existieras": "existir",
    "existiere": "existir",
    "existiereis": "existir",
    "existieren": "existir",
    "existieres": "existir",
    "existieron": "existir",
    "existiese": "existir",
    "existieseis": "existir",
    "existiesen": "existir",
    "existieses": "existir",
    "existimaciones": "existimación",
    "existimos": "existir",
    "existiremos": "existir",
    "existirá": "existir",
    "existirán": "existir",
    "existirás": "existir",
    "existiré": "existir",
    "existiréis": "existir",
    "existiría": "existir",
    "existiríais": "existir",
    "existiríamos": "existir",
    "existirían": "existir",
    "existirías": "existir",
    "exististe": "existir",
    "exististeis": "existir",
    "existiéramos": "existir",
    "existiéremos": "existir",
    "existiésemos": "existir",
    "existió": "existir",
    "existo": "existir",
    "existáis": "existir",
    "existí": "existir",
    "existía": "existir",
    "existíais": "existir",
    "existíamos": "existir",
    "existían": "existir",
    "existías": "existir",
    "existís": "existir",
    "exitosa": "exitoso",
    "exitosas": "exitoso",
    "exitosos": "exitoso",
    "exministra": "exministro",
    "exministras": "exministro",
    "exministros": "exministro",
    "exnúmeros": "exnúmero",
    "exobiologías": "exobiología",
    "exocinética": "exocinético",
    "exocinéticas": "exocinético",
    "exocinéticos": "exocinético",
    "exocrina": "exocrino",
    "exocrinas": "exocrino",
    "exocrinos": "exocrino",
    "exocéntrica": "exocéntrico",
    "exocéntricas": "exocéntrico",
    "exocéntricos": "exocéntrico",
    "exodoncias": "exodoncia",
    "exodoncistas": "exodoncista",
    "exoenergética": "exoenergético",
    "exoenergéticas": "exoenergético",
    "exoenergéticos": "exoenergético",
    "exoenzimas": "exoenzima",
    "exoesqueletos": "exoesqueleto",
    "exoforias": "exoforia",
    "exoftalmometrías": "exoftalmometría",
    "exoftalmías": "exoftalmía",
    "exoftalmómetros": "exoftalmómetro",
    "exoftálmica": "exoftálmico",
    "exoftálmicas": "exoftálmico",
    "exoftálmicos": "exoftálmico",
    "exogamias": "exogamia",
    "exogastrulaciones": "exogastrulación",
    "exognacias": "exognacia",
    "exogámica": "exogámico",
    "exogámicas": "exogámico",
    "exogámicos": "exogámico",
    "exogástrulas": "exogástrula",
    "exonera": "exonerar",
    "exoneraba": "exonerar",
    "exonerabais": "exonerar",
    "exoneraban": "exonerar",
    "exonerabas": "exonerar",
    "exoneraciones": "exoneración",
    "exonerad": "exonerar",
    "exonerada": "exonerar",
    "exoneradas": "exonerar",
    "exonerado": "exonerar",
    "exonerados": "exonerar",
    "exoneramos": "exonerar",
    "exoneran": "exonerar",
    "exonerando": "exonerar",
    "exonerara": "exonerar",
    "exonerarais": "exonerar",
    "exoneraran": "exonerar",
    "exoneraras": "exonerar",
    "exonerare": "exonerar",
    "exonerareis": "exonerar",
    "exoneraremos": "exonerar",
    "exoneraren": "exonerar",
    "exonerares": "exonerar",
    "exoneraron": "exonerar",
    "exonerará": "exonerar",
    "exonerarán": "exonerar",
    "exonerarás": "exonerar",
    "exoneraré": "exonerar",
    "exoneraréis": "exonerar",
    "exoneraría": "exonerar",
    "exoneraríais": "exonerar",
    "exoneraríamos": "exonerar",
    "exonerarían": "exonerar",
    "exonerarías": "exonerar",
    "exoneras": "exonerar",
    "exonerase": "exonerar",
    "exoneraseis": "exonerar",
    "exonerasen": "exonerar",
    "exonerases": "exonerar",
    "exoneraste": "exonerar",
    "exonerasteis": "exonerar",
    "exonerativa": "exonerativo",
    "exonerativas": "exonerativo",
    "exonerativos": "exonerativo",
    "exoneratoria": "exoneratorio",
    "exoneratorias": "exoneratorio",
    "exoneratorios": "exoneratorio",
    "exonere": "exonerar",
    "exoneremos": "exonerar",
    "exoneren": "exonerar",
    "exoneres": "exonerar",
    "exonero": "exonerar",
    "exonerábamos": "exonerar",
    "exoneráis": "exonerar",
    "exoneráramos": "exonerar",
    "exoneráremos": "exonerar",
    "exonerásemos": "exonerar",
    "exoneré": "exonerar",
    "exoneréis": "exonerar",
    "exoneró": "exonerar",
    "exones": "exón",
    "exorbitaciones": "exorbitación",
    "exorbitancias": "exorbitancia",
    "exorbitantes": "exorbitante",
    "exorcice": "exorcizar",
    "exorcicemos": "exorcizar",
    "exorcicen": "exorcizar",
    "exorcices": "exorcizar",
    "exorcicé": "exorcizar",
    "exorcicéis": "exorcizar",
    "exorcismos": "exorcismo",
    "exorcistados": "exorcistado",
    "exorcistas": "exorcista",
    "exorciza": "exorcizar",
    "exorcizaba": "exorcizar",
    "exorcizabais": "exorcizar",
    "exorcizaban": "exorcizar",
    "exorcizabas": "exorcizar",
    "exorcizad": "exorcizar",
    "exorcizada": "exorcizar",
    "exorcizadas": "exorcizar",
    "exorcizado": "exorcizar",
    "exorcizados": "exorcizar",
    "exorcizamos": "exorcizar",
    "exorcizan": "exorcizar",
    "exorcizando": "exorcizar",
    "exorcizara": "exorcizar",
    "exorcizarais": "exorcizar",
    "exorcizaran": "exorcizar",
    "exorcizaras": "exorcizar",
    "exorcizare": "exorcizar",
    "exorcizareis": "exorcizar",
    "exorcizaremos": "exorcizar",
    "exorcizaren": "exorcizar",
    "exorcizares": "exorcizar",
    "exorcizaron": "exorcizar",
    "exorcizará": "exorcizar",
    "exorcizarán": "exorcizar",
    "exorcizarás": "exorcizar",
    "exorcizaré": "exorcizar",
    "exorcizaréis": "exorcizar",
    "exorcizaría": "exorcizar",
    "exorcizaríais": "exorcizar",
    "exorcizaríamos": "exorcizar",
    "exorcizarían": "exorcizar",
    "exorcizarías": "exorcizar",
    "exorcizas": "exorcizar",
    "exorcizase": "exorcizar",
    "exorcizaseis": "exorcizar",
    "exorcizasen": "exorcizar",
    "exorcizases": "exorcizar",
    "exorcizaste": "exorcizar",
    "exorcizasteis": "exorcizar",
    "exorcizo": "exorcizar",
    "exorcizábamos": "exorcizar",
    "exorcizáis": "exorcizar",
    "exorcizáramos": "exorcizar",
    "exorcizáremos": "exorcizar",
    "exorcizásemos": "exorcizar",
    "exorcizó": "exorcizar",
    "exordios": "exordio",
    "exorna": "exornar",
    "exornaba": "exornar",
    "exornabais": "exornar",
    "exornaban": "exornar",
    "exornabas": "exornar",
    "exornaciones": "exornación",
    "exornad": "exornar",
    "exornada": "exornar",
    "exornadas": "exornar",
    "exornado": "exornar",
    "exornados": "exornar",
    "exornamos": "exornar",
    "exornan": "exornar",
    "exornando": "exornar",
    "exornara": "exornar",
    "exornarais": "exornar",
    "exornaran": "exornar",
    "exornaras": "exornar",
    "exornare": "exornar",
    "exornareis": "exornar",
    "exornaremos": "exornar",
    "exornaren": "exornar",
    "exornares": "exornar",
    "exornaron": "exornar",
    "exornará": "exornar",
    "exornarán": "exornar",
    "exornarás": "exornar",
    "exornaré": "exornar",
    "exornaréis": "exornar",
    "exornaría": "exornar",
    "exornaríais": "exornar",
    "exornaríamos": "exornar",
    "exornarían": "exornar",
    "exornarías": "exornar",
    "exornas": "exornar",
    "exornase": "exornar",
    "exornaseis": "exornar",
    "exornasen": "exornar",
    "exornases": "exornar",
    "exornaste": "exornar",
    "exornasteis": "exornar",
    "exorne": "exornar",
    "exornemos": "exornar",
    "exornen": "exornar",
    "exornes": "exornar",
    "exorno": "exornar",
    "exornos": "exorno",
    "exornábamos": "exornar",
    "exornáis": "exornar",
    "exornáramos": "exornar",
    "exornáremos": "exornar",
    "exornásemos": "exornar",
    "exorné": "exornar",
    "exornéis": "exornar",
    "exornó": "exornar",
    "exorreica": "exorreico",
    "exorreicas": "exorreico",
    "exorreicos": "exorreico",
    "exosferas": "exosfera",
    "exosqueletos": "exosqueleto",
    "exoticidades": "exoticidad",
    "exotiqueces": "exotiquez",
    "exotismos": "exotismo",
    "exotoxinas": "exotoxina",
    "exotropías": "exotropía",
    "exotérica": "exotérico",
    "exotéricas": "exotérico",
    "exotéricos": "exotérico",
    "exotérmica": "exotérmico",
    "exotérmicas": "exotérmico",
    "exotérmicos": "exotérmico",
    "expanda": "expandir",
    "expandamos": "expandir",
    "expandan": "expandir",
    "expandas": "expandir",
    "expande": "expandir",
    "expanden": "expandir",
    "expandes": "expandir",
    "expandibilidades": "expandibilidad",
    "expandibles": "expandible",
    "expandid": "expandir",
    "expandida": "expandir",
    "expandidas": "expandir",
    "expandido": "expandir",
    "expandidos": "expandir",
    "expandiendo": "expandir",
    "expandiera": "expandir",
    "expandierais": "expandir",
    "expandieran": "expandir",
    "expandieras": "expandir",
    "expandiere": "expandir",
    "expandiereis": "expandir",
    "expandieren": "expandir",
    "expandieres": "expandir",
    "expandieron": "expandir",
    "expandiese": "expandir",
    "expandieseis": "expandir",
    "expandiesen": "expandir",
    "expandieses": "expandir",
    "expandimos": "expandir",
    "expandiremos": "expandir",
    "expandirá": "expandir",
    "expandirán": "expandir",
    "expandirás": "expandir",
    "expandiré": "expandir",
    "expandiréis": "expandir",
    "expandiría": "expandir",
    "expandiríais": "expandir",
    "expandiríamos": "expandir",
    "expandirían": "expandir",
    "expandirías": "expandir",
    "expandiste": "expandir",
    "expandisteis": "expandir",
    "expandiéramos": "expandir",
    "expandiéremos": "expandir",
    "expandiésemos": "expandir",
    "expandió": "expandir",
    "expando": "expandir",
    "expandáis": "expandir",
    "expandí": "expandir",
    "expandía": "expandir",
    "expandíais": "expandir",
    "expandíamos": "expandir",
    "expandían": "expandir",
    "expandías": "expandir",
    "expandís": "expandir",
    "expansibilidades": "expansibilidad",
    "expansibles": "expansible",
    "expansiona": "expansionar",
    "expansionaba": "expansionar",
    "expansionabais": "expansionar",
    "expansionaban": "expansionar",
    "expansionabas": "expansionar",
    "expansionables": "expansionable",
    "expansionad": "expansionar",
    "expansionada": "expansionar",
    "expansionadas": "expansionar",
    "expansionado": "expansionar",
    "expansionados": "expansionar",
    "expansionamos": "expansionar",
    "expansionan": "expansionar",
    "expansionando": "expansionar",
    "expansionara": "expansionar",
    "expansionarais": "expansionar",
    "expansionaran": "expansionar",
    "expansionaras": "expansionar",
    "expansionare": "expansionar",
    "expansionareis": "expansionar",
    "expansionaremos": "expansionar",
    "expansionaren": "expansionar",
    "expansionares": "expansionar",
    "expansionaron": "expansionar",
    "expansionará": "expansionar",
    "expansionarán": "expansionar",
    "expansionarás": "expansionar",
    "expansionaré": "expansionar",
    "expansionaréis": "expansionar",
    "expansionaría": "expansionar",
    "expansionaríais": "expansionar",
    "expansionaríamos": "expansionar",
    "expansionarían": "expansionar",
    "expansionarías": "expansionar",
    "expansionas": "expansionar",
    "expansionase": "expansionar",
    "expansionaseis": "expansionar",
    "expansionasen": "expansionar",
    "expansionases": "expansionar",
    "expansionaste": "expansionar",
    "expansionasteis": "expansionar",
    "expansione": "expansionar",
    "expansionemos": "expansionar",
    "expansionen": "expansionar",
    "expansiones": "expansionar",
    "expansionismos": "expansionismo",
    "expansionistas": "expansionista",
    "expansiono": "expansionar",
    "expansionábamos": "expansionar",
    "expansionáis": "expansionar",
    "expansionáramos": "expansionar",
    "expansionáremos": "expansionar",
    "expansionásemos": "expansionar",
    "expansioné": "expansionar",
    "expansionéis": "expansionar",
    "expansionó": "expansionar",
    "expansiva": "expansivo",
    "expansivas": "expansivo",
    "expansivos": "expansivo",
    "expansora": "expansor",
    "expansoras": "expansor",
    "expansores": "expansor",
    "expatriaba": "expatriar",
    "expatriabais": "expatriar",
    "expatriaban": "expatriar",
    "expatriabas": "expatriar",
    "expatriaciones": "expatriación",
    "expatriad": "expatriar",
    "expatriada": "expatriar",
    "expatriadas": "expatriar",
    "expatriado": "expatriar",
    "expatriados": "expatriar",
    "expatriamos": "expatriar",
    "expatriando": "expatriar",
    "expatriara": "expatriar",
    "expatriarais": "expatriar",
    "expatriaran": "expatriar",
    "expatriaras": "expatriar",
    "expatriare": "expatriar",
    "expatriareis": "expatriar",
    "expatriaremos": "expatriar",
    "expatriaren": "expatriar",
    "expatriares": "expatriar",
    "expatriaron": "expatriar",
    "expatriará": "expatriar",
    "expatriarán": "expatriar",
    "expatriarás": "expatriar",
    "expatriaré": "expatriar",
    "expatriaréis": "expatriar",
    "expatriaría": "expatriar",
    "expatriaríais": "expatriar",
    "expatriaríamos": "expatriar",
    "expatriarían": "expatriar",
    "expatriarías": "expatriar",
    "expatriase": "expatriar",
    "expatriaseis": "expatriar",
    "expatriasen": "expatriar",
    "expatriases": "expatriar",
    "expatriaste": "expatriar",
    "expatriasteis": "expatriar",
    "expatriemos": "expatriar",
    "expatriábamos": "expatriar",
    "expatriáis": "expatriar",
    "expatriáramos": "expatriar",
    "expatriáremos": "expatriar",
    "expatriásemos": "expatriar",
    "expatrié": "expatriar",
    "expatriéis": "expatriar",
    "expatrió": "expatriar",
    "expatría": "expatriar",
    "expatrían": "expatriar",
    "expatrías": "expatriar",
    "expatríe": "expatriar",
    "expatríen": "expatriar",
    "expatríes": "expatriar",
    "expatrío": "expatriar",
    "expectaciones": "expectación",
    "expectantes": "expectante",
    "expectativas": "expectativa",
    "expectora": "expectorar",
    "expectoraba": "expectorar",
    "expectorabais": "expectorar",
    "expectoraban": "expectorar",
    "expectorabas": "expectorar",
    "expectoraciones": "expectoración",
    "expectorad": "expectorar",
    "expectorada": "expectorar",
    "expectoradas": "expectorar",
    "expectorado": "expectorar",
    "expectorados": "expectorar",
    "expectoramos": "expectorar",
    "expectoran": "expectorar",
    "expectorando": "expectorar",
    "expectorantes": "expectorante",
    "expectorara": "expectorar",
    "expectorarais": "expectorar",
    "expectoraran": "expectorar",
    "expectoraras": "expectorar",
    "expectorare": "expectorar",
    "expectorareis": "expectorar",
    "expectoraremos": "expectorar",
    "expectoraren": "expectorar",
    "expectorares": "expectorar",
    "expectoraron": "expectorar",
    "expectorará": "expectorar",
    "expectorarán": "expectorar",
    "expectorarás": "expectorar",
    "expectoraré": "expectorar",
    "expectoraréis": "expectorar",
    "expectoraría": "expectorar",
    "expectoraríais": "expectorar",
    "expectoraríamos": "expectorar",
    "expectorarían": "expectorar",
    "expectorarías": "expectorar",
    "expectoras": "expectorar",
    "expectorase": "expectorar",
    "expectoraseis": "expectorar",
    "expectorasen": "expectorar",
    "expectorases": "expectorar",
    "expectoraste": "expectorar",
    "expectorasteis": "expectorar",
    "expectore": "expectorar",
    "expectoremos": "expectorar",
    "expectoren": "expectorar",
    "expectores": "expectorar",
    "expectoro": "expectorar",
    "expectorábamos": "expectorar",
    "expectoráis": "expectorar",
    "expectoráramos": "expectorar",
    "expectoráremos": "expectorar",
    "expectorásemos": "expectorar",
    "expectoré": "expectorar",
    "expectoréis": "expectorar",
    "expectoró": "expectorar",
    "expedicionaria": "expedicionario",
    "expedicionarias": "expedicionario",
    "expedicionarios": "expedicionario",
    "expedicionera": "expedicionero",
    "expedicioneras": "expedicionero",
    "expedicioneros": "expedicionero",
    "expediciones": "expedición",
    "expedid": "expedir",
    "expedida": "expedir",
    "expedidas": "expedir",
    "expedido": "expedir",
    "expedidora": "expedidor",
    "expedidoras": "expedidor",
    "expedidores": "expedidor",
    "expedidos": "expedir",
    "expedienta": "expedientar",
    "expedientaba": "expedientar",
    "expedientabais": "expedientar",
    "expedientaban": "expedientar",
    "expedientabas": "expedientar",
    "expedientaciones": "expedientación",
    "expedientad": "expedientar",
    "expedientada": "expedientar",
    "expedientadas": "expedientar",
    "expedientado": "expedientar",
    "expedientados": "expedientar",
    "expedientamos": "expedientar",
    "expedientan": "expedientar",
    "expedientando": "expedientar",
    "expedientara": "expedientar",
    "expedientarais": "expedientar",
    "expedientaran": "expedientar",
    "expedientaras": "expedientar",
    "expedientare": "expedientar",
    "expedientareis": "expedientar",
    "expedientaremos": "expedientar",
    "expedientaren": "expedientar",
    "expedientares": "expedientar",
    "expedientaron": "expedientar",
    "expedientará": "expedientar",
    "expedientarán": "expedientar",
    "expedientarás": "expedientar",
    "expedientaré": "expedientar",
    "expedientaréis": "expedientar",
    "expedientaría": "expedientar",
    "expedientaríais": "expedientar",
    "expedientaríamos": "expedientar",
    "expedientarían": "expedientar",
    "expedientarías": "expedientar",
    "expedientas": "expedientar",
    "expedientase": "expedientar",
    "expedientaseis": "expedientar",
    "expedientasen": "expedientar",
    "expedientases": "expedientar",
    "expedientaste": "expedientar",
    "expedientasteis": "expedientar",
    "expediente": "expedientar",
    "expedientemos": "expedientar",
    "expedienten": "expedientar",
    "expedienteos": "expedienteo",
    "expedientes": "expediente",
    "expediento": "expedientar",
    "expedientábamos": "expedientar",
    "expedientáis": "expedientar",
    "expedientáramos": "expedientar",
    "expedientáremos": "expedientar",
    "expedientásemos": "expedientar",
    "expedienté": "expedientar",
    "expedientéis": "expedientar",
    "expedientó": "expedientar",
    "expedimos": "expedir",
    "expediremos": "expedir",
    "expedirá": "expedir",
    "expedirán": "expedir",
    "expedirás": "expedir",
    "expediré": "expedir",
    "expediréis": "expedir",
    "expediría": "expedir",
    "expediríais": "expedir",
    "expediríamos": "expedir",
    "expedirían": "expedir",
    "expedirías": "expedir",
    "expediste": "expedir",
    "expedisteis": "expedir",
    "expedita": "expedito",
    "expeditas": "expedito",
    "expeditiva": "expeditivo",
    "expeditivas": "expeditivo",
    "expeditivos": "expeditivo",
    "expeditora": "expeditor",
    "expeditoras": "expeditor",
    "expeditores": "expeditor",
    "expeditos": "expedito",
    "expedí": "expedir",
    "expedía": "expedir",
    "expedíais": "expedir",
    "expedíamos": "expedir",
    "expedían": "expedir",
    "expedías": "expedir",
    "expedís": "expedir",
    "expela": "expeler",
    "expelamos": "expeler",
    "expelan": "expeler",
    "expelas": "expeler",
    "expele": "expeler",
    "expeled": "expeler",
    "expelemos": "expeler",
    "expelen": "expeler",
    "expeleremos": "expeler",
    "expelerá": "expeler",
    "expelerán": "expeler",
    "expelerás": "expeler",
    "expeleré": "expeler",
    "expeleréis": "expeler",
    "expelería": "expeler",
    "expeleríais": "expeler",
    "expeleríamos": "expeler",
    "expelerían": "expeler",
    "expelerías": "expeler",
    "expeles": "expeler",
    "expelida": "expeler",
    "expelidas": "expeler",
    "expelido": "expeler",
    "expelidos": "expeler",
    "expeliendo": "expeler",
    "expeliera": "expeler",
    "expelierais": "expeler",
    "expelieran": "expeler",
    "expelieras": "expeler",
    "expeliere": "expeler",
    "expeliereis": "expeler",
    "expelieren": "expeler",
    "expelieres": "expeler",
    "expelieron": "expeler",
    "expeliese": "expeler",
    "expelieseis": "expeler",
    "expeliesen": "expeler",
    "expelieses": "expeler",
    "expelimos": "expeler",
    "expeliste": "expeler",
    "expelisteis": "expeler",
    "expeliéramos": "expeler",
    "expeliéremos": "expeler",
    "expeliésemos": "expeler",
    "expelió": "expeler",
    "expelo": "expeler",
    "expeláis": "expeler",
    "expeléis": "expeler",
    "expelí": "expeler",
    "expelía": "expeler",
    "expelíais": "expeler",
    "expelíamos": "expeler",
    "expelían": "expeler",
    "expelías": "expeler",
    "expenda": "expender",
    "expendamos": "expender",
    "expendan": "expender",
    "expendas": "expender",
    "expende": "expender",
    "expended": "expender",
    "expendedora": "expendedor",
    "expendedoras": "expendedor",
    "expendedores": "expendedor",
    "expendedurías": "expendeduría",
    "expendemos": "expender",
    "expenden": "expender",
    "expenderemos": "expender",
    "expenderá": "expender",
    "expenderán": "expender",
    "expenderás": "expender",
    "expenderé": "expender",
    "expenderéis": "expender",
    "expendería": "expender",
    "expenderíais": "expender",
    "expenderíamos": "expender",
    "expenderían": "expender",
    "expenderías": "expender",
    "expendes": "expender",
    "expendiciones": "expendición",
    "expendida": "expender",
    "expendidas": "expender",
    "expendido": "expender",
    "expendidos": "expender",
    "expendiendo": "expender",
    "expendiera": "expender",
    "expendierais": "expender",
    "expendieran": "expender",
    "expendieras": "expender",
    "expendiere": "expender",
    "expendiereis": "expender",
    "expendieren": "expender",
    "expendieres": "expender",
    "expendieron": "expender",
    "expendiese": "expender",
    "expendieseis": "expender",
    "expendiesen": "expender",
    "expendieses": "expender",
    "expendimos": "expender",
    "expendios": "expendio",
    "expendiste": "expender",
    "expendisteis": "expender",
    "expendiéramos": "expender",
    "expendiéremos": "expender",
    "expendiésemos": "expender",
    "expendió": "expender",
    "expendo": "expender",
    "expendáis": "expender",
    "expendéis": "expender",
    "expendí": "expender",
    "expendía": "expender",
    "expendíais": "expender",
    "expendíamos": "expender",
    "expendían": "expender",
    "expendías": "expender",
    "expensa": "expensar",
    "expensaba": "expensar",
    "expensabais": "expensar",
    "expensaban": "expensar",
    "expensabas": "expensar",
    "expensad": "expensar",
    "expensada": "expensar",
    "expensadas": "expensar",
    "expensado": "expensar",
    "expensados": "expensar",
    "expensamos": "expensar",
    "expensan": "expensar",
    "expensando": "expensar",
    "expensara": "expensar",
    "expensarais": "expensar",
    "expensaran": "expensar",
    "expensaras": "expensar",
    "expensare": "expensar",
    "expensareis": "expensar",
    "expensaremos": "expensar",
    "expensaren": "expensar",
    "expensares": "expensar",
    "expensaron": "expensar",
    "expensará": "expensar",
    "expensarán": "expensar",
    "expensarás": "expensar",
    "expensaré": "expensar",
    "expensaréis": "expensar",
    "expensaría": "expensar",
    "expensaríais": "expensar",
    "expensaríamos": "expensar",
    "expensarían": "expensar",
    "expensarías": "expensar",
    "expensas": "expensar",
    "expensase": "expensar",
    "expensaseis": "expensar",
    "expensasen": "expensar",
    "expensases": "expensar",
    "expensaste": "expensar",
    "expensasteis": "expensar",
    "expense": "expensar",
    "expensemos": "expensar",
    "expensen": "expensar",
    "expenses": "expensar",
    "expenso": "expensar",
    "expensábamos": "expensar",
    "expensáis": "expensar",
    "expensáramos": "expensar",
    "expensáremos": "expensar",
    "expensásemos": "expensar",
    "expensé": "expensar",
    "expenséis": "expensar",
    "expensó": "expensar",
    "experiencias": "experiencia",
    "experimenta": "experimentar",
    "experimentaba": "experimentar",
    "experimentabais": "experimentar",
    "experimentaban": "experimentar",
    "experimentabas": "experimentar",
    "experimentaciones": "experimentación",
    "experimentad": "experimentar",
    "experimentada": "experimentar",
    "experimentadas": "experimentar",
    "experimentado": "experimentar",
    "experimentadora": "experimentador",
    "experimentadoras": "experimentador",
    "experimentadores": "experimentador",
    "experimentados": "experimentar",
    "experimentales": "experimental",
    "experimentalismos": "experimentalismo",
    "experimentamos": "experimentar",
    "experimentan": "experimentar",
    "experimentando": "experimentar",
    "experimentara": "experimentar",
    "experimentarais": "experimentar",
    "experimentaran": "experimentar",
    "experimentaras": "experimentar",
    "experimentare": "experimentar",
    "experimentareis": "experimentar",
    "experimentaremos": "experimentar",
    "experimentaren": "experimentar",
    "experimentares": "experimentar",
    "experimentaron": "experimentar",
    "experimentará": "experimentar",
    "experimentarán": "experimentar",
    "experimentarás": "experimentar",
    "experimentaré": "experimentar",
    "experimentaréis": "experimentar",
    "experimentaría": "experimentar",
    "experimentaríais": "experimentar",
    "experimentaríamos": "experimentar",
    "experimentarían": "experimentar",
    "experimentarías": "experimentar",
    "experimentas": "experimentar",
    "experimentase": "experimentar",
    "experimentaseis": "experimentar",
    "experimentasen": "experimentar",
    "experimentases": "experimentar",
    "experimentaste": "experimentar",
    "experimentasteis": "experimentar",
    "experimente": "experimentar",
    "experimentemos": "experimentar",
    "experimenten": "experimentar",
    "experimentes": "experimentar",
    "experimento": "experimentar",
    "experimentos": "experimento",
    "experimentábamos": "experimentar",
    "experimentáis": "experimentar",
    "experimentáramos": "experimentar",
    "experimentáremos": "experimentar",
    "experimentásemos": "experimentar",
    "experimenté": "experimentar",
    "experimentéis": "experimentar",
    "experimentó": "experimentar",
    "experta": "experto",
    "expertas": "experto",
    "expertos": "experto",
    "expiaba": "expiar",
    "expiabais": "expiar",
    "expiaban": "expiar",
    "expiabas": "expiar",
    "expiaciones": "expiación",
    "expiad": "expiar",
    "expiada": "expiar",
    "expiadas": "expiar",
    "expiado": "expiar",
    "expiados": "expiar",
    "expiamos": "expiar",
    "expiando": "expiar",
    "expiara": "expiar",
    "expiarais": "expiar",
    "expiaran": "expiar",
    "expiaras": "expiar",
    "expiare": "expiar",
    "expiareis": "expiar",
    "expiaremos": "expiar",
    "expiaren": "expiar",
    "expiares": "expiar",
    "expiaron": "expiar",
    "expiará": "expiar",
    "expiarán": "expiar",
    "expiarás": "expiar",
    "expiaré": "expiar",
    "expiaréis": "expiar",
    "expiaría": "expiar",
    "expiaríais": "expiar",
    "expiaríamos": "expiar",
    "expiarían": "expiar",
    "expiarías": "expiar",
    "expiase": "expiar",
    "expiaseis": "expiar",
    "expiasen": "expiar",
    "expiases": "expiar",
    "expiaste": "expiar",
    "expiasteis": "expiar",
    "expiativa": "expiativo",
    "expiativas": "expiativo",
    "expiativos": "expiativo",
    "expiatoria": "expiatorio",
    "expiatorias": "expiatorio",
    "expiatorios": "expiatorio",
    "expida": "expedir",
    "expidamos": "expedir",
    "expidan": "expedir",
    "expidas": "expedir",
    "expide": "expedir",
    "expiden": "expedir",
    "expides": "expedir",
    "expidiendo": "expedir",
    "expidiera": "expedir",
    "expidierais": "expedir",
    "expidieran": "expedir",
    "expidieras": "expedir",
    "expidiere": "expedir",
    "expidiereis": "expedir",
    "expidieren": "expedir",
    "expidieres": "expedir",
    "expidieron": "expedir",
    "expidiese": "expedir",
    "expidieseis": "expedir",
    "expidiesen": "expedir",
    "expidieses": "expedir",
    "expidiéramos": "expedir",
    "expidiéremos": "expedir",
    "expidiésemos": "expedir",
    "expidió": "expedir",
    "expido": "expedir",
    "expidáis": "expedir",
    "expiemos": "expiar",
    "expila": "expilar",
    "expilaba": "expilar",
    "expilabais": "expilar",
    "expilaban": "expilar",
    "expilabas": "expilar",
    "expilad": "expilar",
    "expilada": "expilar",
    "expiladas": "expilar",
    "expilado": "expilar",
    "expilados": "expilar",
    "expilamos": "expilar",
    "expilan": "expilar",
    "expilando": "expilar",
    "expilara": "expilar",
    "expilarais": "expilar",
    "expilaran": "expilar",
    "expilaras": "expilar",
    "expilare": "expilar",
    "expilareis": "expilar",
    "expilaremos": "expilar",
    "expilaren": "expilar",
    "expilares": "expilar",
    "expilaron": "expilar",
    "expilará": "expilar",
    "expilarán": "expilar",
    "expilarás": "expilar",
    "expilaré": "expilar",
    "expilaréis": "expilar",
    "expilaría": "expilar",
    "expilaríais": "expilar",
    "expilaríamos": "expilar",
    "expilarían": "expilar",
    "expilarías": "expilar",
    "expilas": "expilar",
    "expilase": "expilar",
    "expilaseis": "expilar",
    "expilasen": "expilar",
    "expilases": "expilar",
    "expilaste": "expilar",
    "expilasteis": "expilar",
    "expile": "expilar",
    "expilemos": "expilar",
    "expilen": "expilar",
    "expiles": "expilar",
    "expilo": "expilar",
    "expilábamos": "expilar",
    "expiláis": "expilar",
    "expiláramos": "expilar",
    "expiláremos": "expilar",
    "expilásemos": "expilar",
    "expilé": "expilar",
    "expiléis": "expilar",
    "expiló": "expilar",
    "expira": "expirar",
    "expiraba": "expirar",
    "expirabais": "expirar",
    "expiraban": "expirar",
    "expirabas": "expirar",
    "expiraciones": "expiración",
    "expirad": "expirar",
    "expirada": "expirar",
    "expiradas": "expirar",
    "expirado": "expirar",
    "expirados": "expirar",
    "expiramos": "expirar",
    "expiran": "expirar",
    "expirando": "expirar",
    "expirara": "expirar",
    "expirarais": "expirar",
    "expiraran": "expirar",
    "expiraras": "expirar",
    "expirare": "expirar",
    "expirareis": "expirar",
    "expiraremos": "expirar",
    "expiraren": "expirar",
    "expirares": "expirar",
    "expiraron": "expirar",
    "expirará": "expirar",
    "expirarán": "expirar",
    "expirarás": "expirar",
    "expiraré": "expirar",
    "expiraréis": "expirar",
    "expiraría": "expirar",
    "expiraríais": "expirar",
    "expiraríamos": "expirar",
    "expirarían": "expirar",
    "expirarías": "expirar",
    "expiras": "expirar",
    "expirase": "expirar",
    "expiraseis": "expirar",
    "expirasen": "expirar",
    "expirases": "expirar",
    "expiraste": "expirar",
    "expirasteis": "expirar",
    "expire": "expirar",
    "expiremos": "expirar",
    "expiren": "expirar",
    "expires": "expirar",
    "expiro": "expirar",
    "expirábamos": "expirar",
    "expiráis": "expirar",
    "expiráramos": "expirar",
    "expiráremos": "expirar",
    "expirásemos": "expirar",
    "expiré": "expirar",
    "expiréis": "expirar",
    "expiró": "expirar",
    "expiábamos": "expiar",
    "expiáis": "expiar",
    "expiáramos": "expiar",
    "expiáremos": "expiar",
    "expiásemos": "expiar",
    "expié": "expiar",
    "expiéis": "expiar",
    "expió": "expiar",
    "explana": "explanar",
    "explanaba": "explanar",
    "explanabais": "explanar",
    "explanaban": "explanar",
    "explanabas": "explanar",
    "explanaciones": "explanación",
    "explanad": "explanar",
    "explanada": "explanar",
    "explanadas": "explanar",
    "explanado": "explanar",
    "explanadoras": "explanadora",
    "explanados": "explanar",
    "explanamos": "explanar",
    "explanan": "explanar",
    "explanando": "explanar",
    "explanara": "explanar",
    "explanarais": "explanar",
    "explanaran": "explanar",
    "explanaras": "explanar",
    "explanare": "explanar",
    "explanareis": "explanar",
    "explanaremos": "explanar",
    "explanaren": "explanar",
    "explanares": "explanar",
    "explanaron": "explanar",
    "explanará": "explanar",
    "explanarán": "explanar",
    "explanarás": "explanar",
    "explanaré": "explanar",
    "explanaréis": "explanar",
    "explanaría": "explanar",
    "explanaríais": "explanar",
    "explanaríamos": "explanar",
    "explanarían": "explanar",
    "explanarías": "explanar",
    "explanas": "explanar",
    "explanase": "explanar",
    "explanaseis": "explanar",
    "explanasen": "explanar",
    "explanases": "explanar",
    "explanaste": "explanar",
    "explanasteis": "explanar",
    "explane": "explanar",
    "explanemos": "explanar",
    "explanen": "explanar",
    "explanes": "explanar",
    "explano": "explanar",
    "explantaciones": "explantación",
    "explantes": "explante",
    "explanábamos": "explanar",
    "explanáis": "explanar",
    "explanáramos": "explanar",
    "explanáremos": "explanar",
    "explanásemos": "explanar",
    "explané": "explanar",
    "explanéis": "explanar",
    "explanó": "explanar",
    "explaya": "explayar",
    "explayaba": "explayar",
    "explayabais": "explayar",
    "explayaban": "explayar",
    "explayabas": "explayar",
    "explayaciones": "explayación",
    "explayad": "explayar",
    "explayada": "explayar",
    "explayadas": "explayar",
    "explayado": "explayar",
    "explayados": "explayar",
    "explayamos": "explayar",
    "explayan": "explayar",
    "explayando": "explayar",
    "explayara": "explayar",
    "explayarais": "explayar",
    "explayaran": "explayar",
    "explayaras": "explayar",
    "explayare": "explayar",
    "explayareis": "explayar",
    "explayaremos": "explayar",
    "explayaren": "explayar",
    "explayares": "explayar",
    "explayaron": "explayar",
    "explayará": "explayar",
    "explayarán": "explayar",
    "explayarás": "explayar",
    "explayaré": "explayar",
    "explayaréis": "explayar",
    "explayaría": "explayar",
    "explayaríais": "explayar",
    "explayaríamos": "explayar",
    "explayarían": "explayar",
    "explayarías": "explayar",
    "explayas": "explayar",
    "explayase": "explayar",
    "explayaseis": "explayar",
    "explayasen": "explayar",
    "explayases": "explayar",
    "explayaste": "explayar",
    "explayasteis": "explayar",
    "explaye": "explayar",
    "explayemos": "explayar",
    "explayen": "explayar",
    "explayes": "explayar",
    "explayo": "explayar",
    "explayábamos": "explayar",
    "explayáis": "explayar",
    "explayáramos": "explayar",
    "explayáremos": "explayar",
    "explayásemos": "explayar",
    "explayé": "explayar",
    "explayéis": "explayar",
    "explayó": "explayar",
    "expletiva": "expletivo",
    "expletivas": "expletivo",
    "expletivos": "expletivo",
    "explica": "explicar",
    "explicaba": "explicar",
    "explicabais": "explicar",
    "explicaban": "explicar",
    "explicabas": "explicar",
    "explicables": "explicable",
    "explicaciones": "explicación",
    "explicad": "explicar",
    "explicada": "explicar",
    "explicadas": "explicar",
    "explicado": "explicar",
    "explicadora": "explicador",
    "explicadoras": "explicador",
    "explicadores": "explicador",
    "explicados": "explicar",
    "explicamos": "explicar",
    "explican": "explicar",
    "explicando": "explicar",
    "explicara": "explicar",
    "explicarais": "explicar",
    "explicaran": "explicar",
    "explicaras": "explicar",
    "explicare": "explicar",
    "explicareis": "explicar",
    "explicaremos": "explicar",
    "explicaren": "explicar",
    "explicares": "explicar",
    "explicaron": "explicar",
    "explicará": "explicar",
    "explicarán": "explicar",
    "explicarás": "explicar",
    "explicaré": "explicar",
    "explicaréis": "explicar",
    "explicaría": "explicar",
    "explicaríais": "explicar",
    "explicaríamos": "explicar",
    "explicarían": "explicar",
    "explicarías": "explicar",
    "explicas": "explicar",
    "explicase": "explicar",
    "explicaseis": "explicar",
    "explicasen": "explicar",
    "explicases": "explicar",
    "explicaste": "explicar",
    "explicasteis": "explicar",
    "explicativa": "explicativo",
    "explicativas": "explicativo",
    "explicativos": "explicativo",
    "explicita": "explicitar",
    "explicitaba": "explicitar",
    "explicitabais": "explicitar",
    "explicitaban": "explicitar",
    "explicitabas": "explicitar",
    "explicitaciones": "explicitación",
    "explicitad": "explicitar",
    "explicitada": "explicitar",
    "explicitadas": "explicitar",
    "explicitado": "explicitar",
    "explicitados": "explicitar",
    "explicitamos": "explicitar",
    "explicitan": "explicitar",
    "explicitando": "explicitar",
    "explicitara": "explicitar",
    "explicitarais": "explicitar",
    "explicitaran": "explicitar",
    "explicitaras": "explicitar",
    "explicitare": "explicitar",
    "explicitareis": "explicitar",
    "explicitaremos": "explicitar",
    "explicitaren": "explicitar",
    "explicitares": "explicitar",
    "explicitaron": "explicitar",
    "explicitará": "explicitar",
    "explicitarán": "explicitar",
    "explicitarás": "explicitar",
    "explicitaré": "explicitar",
    "explicitaréis": "explicitar",
    "explicitaría": "explicitar",
    "explicitaríais": "explicitar",
    "explicitaríamos": "explicitar",
    "explicitarían": "explicitar",
    "explicitarías": "explicitar",
    "explicitas": "explicitar",
    "explicitase": "explicitar",
    "explicitaseis": "explicitar",
    "explicitasen": "explicitar",
    "explicitases": "explicitar",
    "explicitaste": "explicitar",
    "explicitasteis": "explicitar",
    "explicite": "explicitar",
    "explicitemos": "explicitar",
    "expliciten": "explicitar",
    "explicites": "explicitar",
    "explicito": "explicitar",
    "explicitábamos": "explicitar",
    "explicitáis": "explicitar",
    "explicitáramos": "explicitar",
    "explicitáremos": "explicitar",
    "explicitásemos": "explicitar",
    "explicité": "explicitar",
    "explicitéis": "explicitar",
    "explicitó": "explicitar",
    "explico": "explicar",
    "explicotea": "explicotear",
    "explicoteaba": "explicotear",
    "explicoteabais": "explicotear",
    "explicoteaban": "explicotear",
    "explicoteabas": "explicotear",
    "explicotead": "explicotear",
    "explicoteada": "explicotear",
    "explicoteadas": "explicotear",
    "explicoteado": "explicotear",
    "explicoteados": "explicotear",
    "explicoteamos": "explicotear",
    "explicotean": "explicotear",
    "explicoteando": "explicotear",
    "explicoteara": "explicotear",
    "explicotearais": "explicotear",
    "explicotearan": "explicotear",
    "explicotearas": "explicotear",
    "explicoteare": "explicotear",
    "explicoteareis": "explicotear",
    "explicotearemos": "explicotear",
    "explicotearen": "explicotear",
    "explicoteares": "explicotear",
    "explicotearon": "explicotear",
    "explicoteará": "explicotear",
    "explicotearán": "explicotear",
    "explicotearás": "explicotear",
    "explicotearé": "explicotear",
    "explicotearéis": "explicotear",
    "explicotearía": "explicotear",
    "explicotearíais": "explicotear",
    "explicotearíamos": "explicotear",
    "explicotearían": "explicotear",
    "explicotearías": "explicotear",
    "explicoteas": "explicotear",
    "explicotease": "explicotear",
    "explicoteaseis": "explicotear",
    "explicoteasen": "explicotear",
    "explicoteases": "explicotear",
    "explicoteaste": "explicotear",
    "explicoteasteis": "explicotear",
    "explicotee": "explicotear",
    "explicoteemos": "explicotear",
    "explicoteen": "explicotear",
    "explicotees": "explicotear",
    "explicoteo": "explicotear",
    "explicoteábamos": "explicotear",
    "explicoteáis": "explicotear",
    "explicoteáramos": "explicotear",
    "explicoteáremos": "explicotear",
    "explicoteásemos": "explicotear",
    "explicoteé": "explicotear",
    "explicoteéis": "explicotear",
    "explicoteó": "explicotear",
    "explicábamos": "explicar",
    "explicáis": "explicar",
    "explicáramos": "explicar",
    "explicáremos": "explicar",
    "explicásemos": "explicar",
    "explicó": "explicar",
    "explique": "explicar",
    "expliquemos": "explicar",
    "expliquen": "explicar",
    "expliques": "explicar",
    "expliqué": "explicar",
    "expliquéis": "explicar",
    "explora": "explorar",
    "exploraba": "explorar",
    "explorabais": "explorar",
    "exploraban": "explorar",
    "explorabas": "explorar",
    "explorables": "explorable",
    "exploraciones": "exploración",
    "explorad": "explorar",
    "explorada": "explorar",
    "exploradas": "explorar",
    "explorado": "explorar",
    "exploradora": "explorador",
    "exploradoras": "explorador",
    "exploradores": "explorador",
    "explorados": "explorar",
    "exploramos": "explorar",
    "exploran": "explorar",
    "explorando": "explorar",
    "explorara": "explorar",
    "explorarais": "explorar",
    "exploraran": "explorar",
    "exploraras": "explorar",
    "explorare": "explorar",
    "explorareis": "explorar",
    "exploraremos": "explorar",
    "exploraren": "explorar",
    "explorares": "explorar",
    "exploraron": "explorar",
    "explorará": "explorar",
    "explorarán": "explorar",
    "explorarás": "explorar",
    "exploraré": "explorar",
    "exploraréis": "explorar",
    "exploraría": "explorar",
    "exploraríais": "explorar",
    "exploraríamos": "explorar",
    "explorarían": "explorar",
    "explorarías": "explorar",
    "exploras": "explorar",
    "explorase": "explorar",
    "exploraseis": "explorar",
    "explorasen": "explorar",
    "explorases": "explorar",
    "exploraste": "explorar",
    "explorasteis": "explorar",
    "exploratoria": "exploratorio",
    "exploratorias": "exploratorio",
    "exploratorios": "exploratorio",
    "explore": "explorar",
    "exploremos": "explorar",
    "exploren": "explorar",
    "explores": "explorar",
    "exploro": "explorar",
    "explorábamos": "explorar",
    "exploráis": "explorar",
    "exploráramos": "explorar",
    "exploráremos": "explorar",
    "explorásemos": "explorar",
    "exploré": "explorar",
    "exploréis": "explorar",
    "exploró": "explorar",
    "explosiona": "explosionar",
    "explosionaba": "explosionar",
    "explosionabais": "explosionar",
    "explosionaban": "explosionar",
    "explosionabas": "explosionar",
    "explosionad": "explosionar",
    "explosionada": "explosionar",
    "explosionadas": "explosionar",
    "explosionado": "explosionar",
    "explosionados": "explosionar",
    "explosionamos": "explosionar",
    "explosionan": "explosionar",
    "explosionando": "explosionar",
    "explosionara": "explosionar",
    "explosionarais": "explosionar",
    "explosionaran": "explosionar",
    "explosionaras": "explosionar",
    "explosionare": "explosionar",
    "explosionareis": "explosionar",
    "explosionaremos": "explosionar",
    "explosionaren": "explosionar",
    "explosionares": "explosionar",
    "explosionaron": "explosionar",
    "explosionará": "explosionar",
    "explosionarán": "explosionar",
    "explosionarás": "explosionar",
    "explosionaré": "explosionar",
    "explosionaréis": "explosionar",
    "explosionaría": "explosionar",
    "explosionaríais": "explosionar",
    "explosionaríamos": "explosionar",
    "explosionarían": "explosionar",
    "explosionarías": "explosionar",
    "explosionas": "explosionar",
    "explosionase": "explosionar",
    "explosionaseis": "explosionar",
    "explosionasen": "explosionar",
    "explosionases": "explosionar",
    "explosionaste": "explosionar",
    "explosionasteis": "explosionar",
    "explosione": "explosionar",
    "explosionemos": "explosionar",
    "explosionen": "explosionar",
    "explosiones": "explosionar",
    "explosiono": "explosionar",
    "explosionábamos": "explosionar",
    "explosionáis": "explosionar",
    "explosionáramos": "explosionar",
    "explosionáremos": "explosionar",
    "explosionásemos": "explosionar",
    "explosioné": "explosionar",
    "explosionéis": "explosionar",
    "explosionó": "explosionar",
    "explosiva": "explosivo",
    "explosivas": "explosivo",
    "explosividades": "explosividad",
    "explosivos": "explosivo",
    "explosímetros": "explosímetro",
    "explota": "explotar",
    "explotaba": "explotar",
    "explotabais": "explotar",
    "explotaban": "explotar",
    "explotabas": "explotar",
    "explotables": "explotable",
    "explotaciones": "explotación",
    "explotad": "explotar",
    "explotada": "explotar",
    "explotadas": "explotar",
    "explotado": "explotar",
    "explotadora": "explotador",
    "explotadoras": "explotador",
    "explotadores": "explotador",
    "explotados": "explotar",
    "explotamos": "explotar",
    "explotan": "explotar",
    "explotando": "explotar",
    "explotara": "explotar",
    "explotarais": "explotar",
    "explotaran": "explotar",
    "explotaras": "explotar",
    "explotare": "explotar",
    "explotareis": "explotar",
    "explotaremos": "explotar",
    "explotaren": "explotar",
    "explotares": "explotar",
    "explotaron": "explotar",
    "explotará": "explotar",
    "explotarán": "explotar",
    "explotarás": "explotar",
    "explotaré": "explotar",
    "explotaréis": "explotar",
    "explotaría": "explotar",
    "explotaríais": "explotar",
    "explotaríamos": "explotar",
    "explotarían": "explotar",
    "explotarías": "explotar",
    "explotas": "explotar",
    "explotase": "explotar",
    "explotaseis": "explotar",
    "explotasen": "explotar",
    "explotases": "explotar",
    "explotaste": "explotar",
    "explotasteis": "explotar",
    "explote": "explotar",
    "explotemos": "explotar",
    "exploten": "explotar",
    "explotes": "explotar",
    "exploto": "explotar",
    "explotábamos": "explotar",
    "explotáis": "explotar",
    "explotáramos": "explotar",
    "explotáremos": "explotar",
    "explotásemos": "explotar",
    "exploté": "explotar",
    "explotéis": "explotar",
    "explotó": "explotar",
    "explénica": "explénico",
    "explénicas": "explénico",
    "explénicos": "explénico",
    "explícita": "explícito",
    "explícitas": "explícito",
    "explícitos": "explícito",
    "expolia": "expoliar",
    "expoliaba": "expoliar",
    "expoliabais": "expoliar",
    "expoliaban": "expoliar",
    "expoliabas": "expoliar",
    "expoliaciones": "expoliación",
    "expoliad": "expoliar",
    "expoliada": "expoliar",
    "expoliadas": "expoliar",
    "expoliado": "expoliar",
    "expoliadora": "expoliador",
    "expoliadoras": "expoliador",
    "expoliadores": "expoliador",
    "expoliados": "expoliar",
    "expoliamos": "expoliar",
    "expolian": "expoliar",
    "expoliando": "expoliar",
    "expoliara": "expoliar",
    "expoliarais": "expoliar",
    "expoliaran": "expoliar",
    "expoliaras": "expoliar",
    "expoliare": "expoliar",
    "expoliareis": "expoliar",
    "expoliaremos": "expoliar",
    "expoliaren": "expoliar",
    "expoliares": "expoliar",
    "expoliaron": "expoliar",
    "expoliará": "expoliar",
    "expoliarán": "expoliar",
    "expoliarás": "expoliar",
    "expoliaré": "expoliar",
    "expoliaréis": "expoliar",
    "expoliaría": "expoliar",
    "expoliaríais": "expoliar",
    "expoliaríamos": "expoliar",
    "expoliarían": "expoliar",
    "expoliarías": "expoliar",
    "expolias": "expoliar",
    "expoliase": "expoliar",
    "expoliaseis": "expoliar",
    "expoliasen": "expoliar",
    "expoliases": "expoliar",
    "expoliaste": "expoliar",
    "expoliasteis": "expoliar",
    "expoliciones": "expolición",
    "expolie": "expoliar",
    "expoliemos": "expoliar",
    "expolien": "expoliar",
    "expolies": "expoliar",
    "expolio": "expoliar",
    "expolios": "expolio",
    "expoliábamos": "expoliar",
    "expoliáis": "expoliar",
    "expoliáramos": "expoliar",
    "expoliáremos": "expoliar",
    "expoliásemos": "expoliar",
    "expolié": "expoliar",
    "expoliéis": "expoliar",
    "expolió": "expoliar",
    "expondremos": "exponer",
    "expondrá": "exponer",
    "expondrán": "exponer",
    "expondrás": "exponer",
    "expondré": "exponer",
    "expondréis": "exponer",
    "expondría": "exponer",
    "expondríais": "exponer",
    "expondríamos": "exponer",
    "expondrían": "exponer",
    "expondrías": "exponer",
    "expone": "exponer",
    "exponed": "exponer",
    "exponemos": "exponer",
    "exponen": "exponer",
    "exponenciales": "exponencial",
    "exponentes": "exponente",
    "expones": "exponer",
    "exponga": "exponer",
    "expongamos": "exponer",
    "expongan": "exponer",
    "expongas": "exponer",
    "expongo": "exponer",
    "expongáis": "exponer",
    "exponiendo": "exponer",
    "exponéis": "exponer",
    "exponía": "exponer",
    "exponíais": "exponer",
    "exponíamos": "exponer",
    "exponían": "exponer",
    "exponías": "exponer",
    "exporta": "exportar",
    "exportaba": "exportar",
    "exportabais": "exportar",
    "exportaban": "exportar",
    "exportabas": "exportar",
    "exportabilidades": "exportabilidad",
    "exportables": "exportable",
    "exportaciones": "exportación",
    "exportad": "exportar",
    "exportada": "exportar",
    "exportadas": "exportar",
    "exportado": "exportar",
    "exportadora": "exportador",
    "exportadoras": "exportador",
    "exportadores": "exportador",
    "exportados": "exportar",
    "exportamos": "exportar",
    "exportan": "exportar",
    "exportando": "exportar",
    "exportara": "exportar",
    "exportarais": "exportar",
    "exportaran": "exportar",
    "exportaras": "exportar",
    "exportare": "exportar",
    "exportareis": "exportar",
    "exportaremos": "exportar",
    "exportaren": "exportar",
    "exportares": "exportar",
    "exportaron": "exportar",
    "exportará": "exportar",
    "exportarán": "exportar",
    "exportarás": "exportar",
    "exportaré": "exportar",
    "exportaréis": "exportar",
    "exportaría": "exportar",
    "exportaríais": "exportar",
    "exportaríamos": "exportar",
    "exportarían": "exportar",
    "exportarías": "exportar",
    "exportas": "exportar",
    "exportase": "exportar",
    "exportaseis": "exportar",
    "exportasen": "exportar",
    "exportases": "exportar",
    "exportaste": "exportar",
    "exportasteis": "exportar",
    "exporte": "exportar",
    "exportemos": "exportar",
    "exporten": "exportar",
    "exportes": "exportar",
    "exporto": "exportar",
    "exportábamos": "exportar",
    "exportáis": "exportar",
    "exportáramos": "exportar",
    "exportáremos": "exportar",
    "exportásemos": "exportar",
    "exporté": "exportar",
    "exportéis": "exportar",
    "exportó": "exportar",
    "exposiciones": "exposición",
    "expositiva": "expositivo",
    "expositivas": "expositivo",
    "expositivos": "expositivo",
    "expositora": "expositor",
    "expositoras": "expositor",
    "expositores": "expositor",
    "exposímetros": "exposímetro",
    "expremijos": "expremijo",
    "expresa": "expreso",
    "expresaba": "expresar",
    "expresabais": "expresar",
    "expresaban": "expresar",
    "expresabas": "expresar",
    "expresad": "expresar",
    "expresada": "expresar",
    "expresadas": "expresar",
    "expresado": "expresar",
    "expresados": "expresar",
    "expresamos": "expresar",
    "expresan": "expresar",
    "expresando": "expresar",
    "expresara": "expresar",
    "expresarais": "expresar",
    "expresaran": "expresar",
    "expresaras": "expresar",
    "expresare": "expresar",
    "expresareis": "expresar",
    "expresaremos": "expresar",
    "expresaren": "expresar",
    "expresares": "expresar",
    "expresaron": "expresar",
    "expresará": "expresar",
    "expresarán": "expresar",
    "expresarás": "expresar",
    "expresaré": "expresar",
    "expresaréis": "expresar",
    "expresaría": "expresar",
    "expresaríais": "expresar",
    "expresaríamos": "expresar",
    "expresarían": "expresar",
    "expresarías": "expresar",
    "expresas": "expreso",
    "expresase": "expresar",
    "expresaseis": "expresar",
    "expresasen": "expresar",
    "expresases": "expresar",
    "expresaste": "expresar",
    "expresasteis": "expresar",
    "exprese": "expresar",
    "expresemos": "expresar",
    "expresen": "expresar",
    "expreses": "expresar",
    "expresiones": "expresión",
    "expresionismos": "expresionismo",
    "expresionistas": "expresionista",
    "expresiva": "expresivo",
    "expresivas": "expresivo",
    "expresividades": "expresividad",
    "expresivos": "expresivo",
    "expreso": "expresar",
    "expresos": "expreso",
    "expresábamos": "expresar",
    "expresáis": "expresar",
    "expresáramos": "expresar",
    "expresáremos": "expresar",
    "expresásemos": "expresar",
    "expresé": "expresar",
    "expreséis": "expresar",
    "expresó": "expresar",
    "exprima": "exprimir",
    "exprimamos": "exprimir",
    "expriman": "exprimir",
    "exprimas": "exprimir",
    "exprime": "exprimir",
    "exprimen": "exprimir",
    "exprimes": "exprimir",
    "exprimid": "exprimir",
    "exprimida": "exprimir",
    "exprimidas": "exprimir",
    "exprimidera": "exprimidero",
    "exprimideras": "exprimidero",
    "exprimideros": "exprimidero",
    "exprimido": "exprimir",
    "exprimidora": "exprimidor",
    "exprimidoras": "exprimidor",
    "exprimidores": "exprimidor",
    "exprimidos": "exprimir",
    "exprimiendo": "exprimir",
    "exprimiera": "exprimir",
    "exprimierais": "exprimir",
    "exprimieran": "exprimir",
    "exprimieras": "exprimir",
    "exprimiere": "exprimir",
    "exprimiereis": "exprimir",
    "exprimieren": "exprimir",
    "exprimieres": "exprimir",
    "exprimieron": "exprimir",
    "exprimiese": "exprimir",
    "exprimieseis": "exprimir",
    "exprimiesen": "exprimir",
    "exprimieses": "exprimir",
    "exprimimos": "exprimir",
    "exprimiremos": "exprimir",
    "exprimirá": "exprimir",
    "exprimirán": "exprimir",
    "exprimirás": "exprimir",
    "exprimiré": "exprimir",
    "exprimiréis": "exprimir",
    "exprimiría": "exprimir",
    "exprimiríais": "exprimir",
    "exprimiríamos": "exprimir",
    "exprimirían": "exprimir",
    "exprimirías": "exprimir",
    "exprimiste": "exprimir",
    "exprimisteis": "exprimir",
    "exprimiéramos": "exprimir",
    "exprimiéremos": "exprimir",
    "exprimiésemos": "exprimir",
    "exprimió": "exprimir",
    "exprimo": "exprimir",
    "exprimáis": "exprimir",
    "exprimí": "exprimir",
    "exprimía": "exprimir",
    "exprimíais": "exprimir",
    "exprimíamos": "exprimir",
    "exprimían": "exprimir",
    "exprimías": "exprimir",
    "exprimís": "exprimir",
    "expromisiones": "expromisión",
    "expropia": "expropiar",
    "expropiaba": "expropiar",
    "expropiabais": "expropiar",
    "expropiaban": "expropiar",
    "expropiabas": "expropiar",
    "expropiaciones": "expropiación",
    "expropiad": "expropiar",
    "expropiada": "expropiar",
    "expropiadas": "expropiar",
    "expropiado": "expropiar",
    "expropiadora": "expropiador",
    "expropiadoras": "expropiador",
    "expropiadores": "expropiador",
    "expropiados": "expropiar",
    "expropiamos": "expropiar",
    "expropian": "expropiar",
    "expropiando": "expropiar",
    "expropiantes": "expropiante",
    "expropiara": "expropiar",
    "expropiarais": "expropiar",
    "expropiaran": "expropiar",
    "expropiaras": "expropiar",
    "expropiare": "expropiar",
    "expropiareis": "expropiar",
    "expropiaremos": "expropiar",
    "expropiaren": "expropiar",
    "expropiares": "expropiar",
    "expropiaron": "expropiar",
    "expropiará": "expropiar",
    "expropiarán": "expropiar",
    "expropiarás": "expropiar",
    "expropiaré": "expropiar",
    "expropiaréis": "expropiar",
    "expropiaría": "expropiar",
    "expropiaríais": "expropiar",
    "expropiaríamos": "expropiar",
    "expropiarían": "expropiar",
    "expropiarías": "expropiar",
    "expropias": "expropiar",
    "expropiase": "expropiar",
    "expropiaseis": "expropiar",
    "expropiasen": "expropiar",
    "expropiases": "expropiar",
    "expropiaste": "expropiar",
    "expropiasteis": "expropiar",
    "expropiatoria": "expropiatorio",
    "expropiatorias": "expropiatorio",
    "expropiatorios": "expropiatorio",
    "expropie": "expropiar",
    "expropiemos": "expropiar",
    "expropien": "expropiar",
    "expropies": "expropiar",
    "expropio": "expropiar",
    "expropiábamos": "expropiar",
    "expropiáis": "expropiar",
    "expropiáramos": "expropiar",
    "expropiáremos": "expropiar",
    "expropiásemos": "expropiar",
    "expropié": "expropiar",
    "expropiéis": "expropiar",
    "expropió": "expropiar",
    "expuesta": "exponer",
    "expuestas": "exponer",
    "expuesto": "exponer",
    "expuestos": "exponer",
    "expugna": "expugnar",
    "expugnaba": "expugnar",
    "expugnabais": "expugnar",
    "expugnaban": "expugnar",
    "expugnabas": "expugnar",
    "expugnables": "expugnable",
    "expugnaciones": "expugnación",
    "expugnad": "expugnar",
    "expugnada": "expugnar",
    "expugnadas": "expugnar",
    "expugnado": "expugnar",
    "expugnadora": "expugnador",
    "expugnadoras": "expugnador",
    "expugnadores": "expugnador",
    "expugnados": "expugnar",
    "expugnamos": "expugnar",
    "expugnan": "expugnar",
    "expugnando": "expugnar",
    "expugnara": "expugnar",
    "expugnarais": "expugnar",
    "expugnaran": "expugnar",
    "expugnaras": "expugnar",
    "expugnare": "expugnar",
    "expugnareis": "expugnar",
    "expugnaremos": "expugnar",
    "expugnaren": "expugnar",
    "expugnares": "expugnar",
    "expugnaron": "expugnar",
    "expugnará": "expugnar",
    "expugnarán": "expugnar",
    "expugnarás": "expugnar",
    "expugnaré": "expugnar",
    "expugnaréis": "expugnar",
    "expugnaría": "expugnar",
    "expugnaríais": "expugnar",
    "expugnaríamos": "expugnar",
    "expugnarían": "expugnar",
    "expugnarías": "expugnar",
    "expugnas": "expugnar",
    "expugnase": "expugnar",
    "expugnaseis": "expugnar",
    "expugnasen": "expugnar",
    "expugnases": "expugnar",
    "expugnaste": "expugnar",
    "expugnasteis": "expugnar",
    "expugne": "expugnar",
    "expugnemos": "expugnar",
    "expugnen": "expugnar",
    "expugnes": "expugnar",
    "expugno": "expugnar",
    "expugnábamos": "expugnar",
    "expugnáis": "expugnar",
    "expugnáramos": "expugnar",
    "expugnáremos": "expugnar",
    "expugnásemos": "expugnar",
    "expugné": "expugnar",
    "expugnéis": "expugnar",
    "expugnó": "expugnar",
    "expulsa": "expulsar",
    "expulsaba": "expulsar",
    "expulsabais": "expulsar",
    "expulsaban": "expulsar",
    "expulsabas": "expulsar",
    "expulsad": "expulsar",
    "expulsada": "expulsar",
    "expulsadas": "expulsar",
    "expulsado": "expulsar",
    "expulsados": "expulsar",
    "expulsamos": "expulsar",
    "expulsan": "expulsar",
    "expulsando": "expulsar",
    "expulsara": "expulsar",
    "expulsarais": "expulsar",
    "expulsaran": "expulsar",
    "expulsaras": "expulsar",
    "expulsare": "expulsar",
    "expulsareis": "expulsar",
    "expulsaremos": "expulsar",
    "expulsaren": "expulsar",
    "expulsares": "expulsar",
    "expulsaron": "expulsar",
    "expulsará": "expulsar",
    "expulsarán": "expulsar",
    "expulsarás": "expulsar",
    "expulsaré": "expulsar",
    "expulsaréis": "expulsar",
    "expulsaría": "expulsar",
    "expulsaríais": "expulsar",
    "expulsaríamos": "expulsar",
    "expulsarían": "expulsar",
    "expulsarías": "expulsar",
    "expulsas": "expulsar",
    "expulsase": "expulsar",
    "expulsaseis": "expulsar",
    "expulsasen": "expulsar",
    "expulsases": "expulsar",
    "expulsaste": "expulsar",
    "expulsasteis": "expulsar",
    "expulse": "expulsar",
    "expulsemos": "expulsar",
    "expulsen": "expulsar",
    "expulses": "expulsar",
    "expulsiones": "expulsión",
    "expulsivos": "expulsivo",
    "expulso": "expulsar",
    "expulsora": "expulsor",
    "expulsoras": "expulsor",
    "expulsores": "expulsor",
    "expulsábamos": "expulsar",
    "expulsáis": "expulsar",
    "expulsáramos": "expulsar",
    "expulsáremos": "expulsar",
    "expulsásemos": "expulsar",
    "expulsé": "expulsar",
    "expulséis": "expulsar",
    "expulsó": "expulsar",
    "expurga": "expurgar",
    "expurgaba": "expurgar",
    "expurgabais": "expurgar",
    "expurgaban": "expurgar",
    "expurgabas": "expurgar",
    "expurgaciones": "expurgación",
    "expurgad": "expurgar",
    "expurgada": "expurgar",
    "expurgadas": "expurgar",
    "expurgado": "expurgar",
    "expurgadora": "expurgador",
    "expurgadoras": "expurgador",
    "expurgadores": "expurgador",
    "expurgados": "expurgar",
    "expurgamos": "expurgar",
    "expurgan": "expurgar",
    "expurgando": "expurgar",
    "expurgara": "expurgar",
    "expurgarais": "expurgar",
    "expurgaran": "expurgar",
    "expurgaras": "expurgar",
    "expurgare": "expurgar",
    "expurgareis": "expurgar",
    "expurgaremos": "expurgar",
    "expurgaren": "expurgar",
    "expurgares": "expurgar",
    "expurgaron": "expurgar",
    "expurgará": "expurgar",
    "expurgarán": "expurgar",
    "expurgarás": "expurgar",
    "expurgaré": "expurgar",
    "expurgaréis": "expurgar",
    "expurgaría": "expurgar",
    "expurgaríais": "expurgar",
    "expurgaríamos": "expurgar",
    "expurgarían": "expurgar",
    "expurgarías": "expurgar",
    "expurgas": "expurgar",
    "expurgase": "expurgar",
    "expurgaseis": "expurgar",
    "expurgasen": "expurgar",
    "expurgases": "expurgar",
    "expurgaste": "expurgar",
    "expurgasteis": "expurgar",
    "expurgatoria": "expurgatorio",
    "expurgatorias": "expurgatorio",
    "expurgatorios": "expurgatorio",
    "expurgo": "expurgar",
    "expurgos": "expurgo",
    "expurgue": "expurgar",
    "expurguemos": "expurgar",
    "expurguen": "expurgar",
    "expurgues": "expurgar",
    "expurgué": "expurgar",
    "expurguéis": "expurgar",
    "expurgábamos": "expurgar",
    "expurgáis": "expurgar",
    "expurgáramos": "expurgar",
    "expurgáremos": "expurgar",
    "expurgásemos": "expurgar",
    "expurgó": "expurgar",
    "expuse": "exponer",
    "expusiera": "exponer",
    "expusierais": "exponer",
    "expusieran": "exponer",
    "expusieras": "exponer",
    "expusiere": "exponer",
    "expusiereis": "exponer",
    "expusieren": "exponer",
    "expusieres": "exponer",
    "expusieron": "exponer",
    "expusiese": "exponer",
    "expusieseis": "exponer",
    "expusiesen": "exponer",
    "expusieses": "exponer",
    "expusimos": "exponer",
    "expusiste": "exponer",
    "expusisteis": "exponer",
    "expusiéramos": "exponer",
    "expusiéremos": "exponer",
    "expusiésemos": "exponer",
    "expuso": "exponer",
    "expía": "expiar",
    "expían": "expiar",
    "expías": "expiar",
    "expíe": "expiar",
    "expíen": "expiar",
    "expíes": "expiar",
    "expío": "expiar",
    "expón": "exponer",
    "expósita": "expósito",
    "expósitas": "expósito",
    "expósitos": "expósito",
    "exquisita": "exquisito",
    "exquisitas": "exquisito",
    "exquisiteces": "exquisitez",
    "exquisitos": "exquisito",
    "exrepresora": "exrepresor",
    "exrepresoras": "exrepresor",
    "exrepresores": "exrepresor",
    "exsanguinotransfusiones": "exsanguinotransfusión",
    "exsoluciones": "exsolución",
    "extasiaba": "extasiar",
    "extasiabais": "extasiar",
    "extasiaban": "extasiar",
    "extasiabas": "extasiar",
    "extasiad": "extasiar",
    "extasiada": "extasiar",
    "extasiadas": "extasiar",
    "extasiado": "extasiar",
    "extasiados": "extasiar",
    "extasiamientos": "extasiamiento",
    "extasiamos": "extasiar",
    "extasiando": "extasiar",
    "extasiara": "extasiar",
    "extasiarais": "extasiar",
    "extasiaran": "extasiar",
    "extasiaras": "extasiar",
    "extasiare": "extasiar",
    "extasiareis": "extasiar",
    "extasiaremos": "extasiar",
    "extasiaren": "extasiar",
    "extasiares": "extasiar",
    "extasiaron": "extasiar",
    "extasiará": "extasiar",
    "extasiarán": "extasiar",
    "extasiarás": "extasiar",
    "extasiaré": "extasiar",
    "extasiaréis": "extasiar",
    "extasiaría": "extasiar",
    "extasiaríais": "extasiar",
    "extasiaríamos": "extasiar",
    "extasiarían": "extasiar",
    "extasiarías": "extasiar",
    "extasiase": "extasiar",
    "extasiaseis": "extasiar",
    "extasiasen": "extasiar",
    "extasiases": "extasiar",
    "extasiaste": "extasiar",
    "extasiasteis": "extasiar",
    "extasiemos": "extasiar",
    "extasiábamos": "extasiar",
    "extasiáis": "extasiar",
    "extasiáramos": "extasiar",
    "extasiáremos": "extasiar",
    "extasiásemos": "extasiar",
    "extasié": "extasiar",
    "extasiéis": "extasiar",
    "extasió": "extasiar",
    "extasía": "extasiar",
    "extasían": "extasiar",
    "extasías": "extasiar",
    "extasíe": "extasiar",
    "extasíen": "extasiar",
    "extasíes": "extasiar",
    "extasío": "extasiar",
    "extemporales": "extemporal",
    "extemporaneidades": "extemporaneidad",
    "extemporánea": "extemporáneo",
    "extemporáneas": "extemporáneo",
    "extemporáneos": "extemporáneo",
    "extendamos": "extender",
    "extended": "extender",
    "extendedoras": "extendedora",
    "extendedores": "extendedor",
    "extendemos": "extender",
    "extenderemos": "extender",
    "extenderá": "extender",
    "extenderán": "extender",
    "extenderás": "extender",
    "extenderé": "extender",
    "extenderéis": "extender",
    "extendería": "extender",
    "extenderíais": "extender",
    "extenderíamos": "extender",
    "extenderían": "extender",
    "extenderías": "extender",
    "extendibilidades": "extendibilidad",
    "extendida": "extender",
    "extendidas": "extender",
    "extendido": "extender",
    "extendidos": "extender",
    "extendiendo": "extender",
    "extendiera": "extender",
    "extendierais": "extender",
    "extendieran": "extender",
    "extendieras": "extender",
    "extendiere": "extender",
    "extendiereis": "extender",
    "extendieren": "extender",
    "extendieres": "extender",
    "extendieron": "extender",
    "extendiese": "extender",
    "extendieseis": "extender",
    "extendiesen": "extender",
    "extendieses": "extender",
    "extendimos": "extender",
    "extendiste": "extender",
    "extendisteis": "extender",
    "extendiéramos": "extender",
    "extendiéremos": "extender",
    "extendiésemos": "extender",
    "extendió": "extender",
    "extendáis": "extender",
    "extendéis": "extender",
    "extendí": "extender",
    "extendía": "extender",
    "extendíais": "extender",
    "extendíamos": "extender",
    "extendían": "extender",
    "extendías": "extender",
    "extensa": "extenso",
    "extensas": "extenso",
    "extensibilidades": "extensibilidad",
    "extensibles": "extensible",
    "extensiones": "extensión",
    "extensiva": "extensivo",
    "extensivas": "extensivo",
    "extensivos": "extensivo",
    "extensométrica": "extensométrico",
    "extensométricas": "extensométrico",
    "extensométricos": "extensométrico",
    "extensora": "extensor",
    "extensoras": "extensor",
    "extensores": "extensor",
    "extensos": "extenso",
    "extensógrafos": "extensógrafo",
    "extensómetros": "extensómetro",
    "extenuaba": "extenuar",
    "extenuabais": "extenuar",
    "extenuaban": "extenuar",
    "extenuabas": "extenuar",
    "extenuaciones": "extenuación",
    "extenuad": "extenuar",
    "extenuada": "extenuar",
    "extenuadas": "extenuar",
    "extenuado": "extenuar",
    "extenuados": "extenuar",
    "extenuamos": "extenuar",
    "extenuando": "extenuar",
    "extenuara": "extenuar",
    "extenuarais": "extenuar",
    "extenuaran": "extenuar",
    "extenuaras": "extenuar",
    "extenuare": "extenuar",
    "extenuareis": "extenuar",
    "extenuaremos": "extenuar",
    "extenuaren": "extenuar",
    "extenuares": "extenuar",
    "extenuaron": "extenuar",
    "extenuará": "extenuar",
    "extenuarán": "extenuar",
    "extenuarás": "extenuar",
    "extenuaré": "extenuar",
    "extenuaréis": "extenuar",
    "extenuaría": "extenuar",
    "extenuaríais": "extenuar",
    "extenuaríamos": "extenuar",
    "extenuarían": "extenuar",
    "extenuarías": "extenuar",
    "extenuase": "extenuar",
    "extenuaseis": "extenuar",
    "extenuasen": "extenuar",
    "extenuases": "extenuar",
    "extenuaste": "extenuar",
    "extenuasteis": "extenuar",
    "extenuativa": "extenuativo",
    "extenuativas": "extenuativo",
    "extenuativos": "extenuativo",
    "extenuemos": "extenuar",
    "extenuábamos": "extenuar",
    "extenuáis": "extenuar",
    "extenuáramos": "extenuar",
    "extenuáremos": "extenuar",
    "extenuásemos": "extenuar",
    "extenué": "extenuar",
    "extenuéis": "extenuar",
    "extenuó": "extenuar",
    "extenúa": "extenuar",
    "extenúan": "extenuar",
    "extenúas": "extenuar",
    "extenúe": "extenuar",
    "extenúen": "extenuar",
    "extenúes": "extenuar",
    "extenúo": "extenuar",
    "exteriores": "exterior",
    "exteriorice": "exteriorizar",
    "exterioricemos": "exteriorizar",
    "exterioricen": "exteriorizar",
    "exteriorices": "exteriorizar",
    "exterioricé": "exteriorizar",
    "exterioricéis": "exteriorizar",
    "exterioridades": "exterioridad",
    "exterioriza": "exteriorizar",
    "exteriorizaba": "exteriorizar",
    "exteriorizabais": "exteriorizar",
    "exteriorizaban": "exteriorizar",
    "exteriorizabas": "exteriorizar",
    "exteriorizaciones": "exteriorización",
    "exteriorizad": "exteriorizar",
    "exteriorizada": "exteriorizar",
    "exteriorizadas": "exteriorizar",
    "exteriorizado": "exteriorizar",
    "exteriorizados": "exteriorizar",
    "exteriorizamos": "exteriorizar",
    "exteriorizan": "exteriorizar",
    "exteriorizando": "exteriorizar",
    "exteriorizara": "exteriorizar",
    "exteriorizarais": "exteriorizar",
    "exteriorizaran": "exteriorizar",
    "exteriorizaras": "exteriorizar",
    "exteriorizare": "exteriorizar",
    "exteriorizareis": "exteriorizar",
    "exteriorizaremos": "exteriorizar",
    "exteriorizaren": "exteriorizar",
    "exteriorizares": "exteriorizar",
    "exteriorizaron": "exteriorizar",
    "exteriorizará": "exteriorizar",
    "exteriorizarán": "exteriorizar",
    "exteriorizarás": "exteriorizar",
    "exteriorizaré": "exteriorizar",
    "exteriorizaréis": "exteriorizar",
    "exteriorizaría": "exteriorizar",
    "exteriorizaríais": "exteriorizar",
    "exteriorizaríamos": "exteriorizar",
    "exteriorizarían": "exteriorizar",
    "exteriorizarías": "exteriorizar",
    "exteriorizas": "exteriorizar",
    "exteriorizase": "exteriorizar",
    "exteriorizaseis": "exteriorizar",
    "exteriorizasen": "exteriorizar",
    "exteriorizases": "exteriorizar",
    "exteriorizaste": "exteriorizar",
    "exteriorizasteis": "exteriorizar",
    "exteriorizo": "exteriorizar",
    "exteriorizábamos": "exteriorizar",
    "exteriorizáis": "exteriorizar",
    "exteriorizáramos": "exteriorizar",
    "exteriorizáremos": "exteriorizar",
    "exteriorizásemos": "exteriorizar",
    "exteriorizó": "exteriorizar",
    "extermina": "exterminar",
    "exterminaba": "exterminar",
    "exterminabais": "exterminar",
    "exterminaban": "exterminar",
    "exterminabas": "exterminar",
    "exterminables": "exterminable",
    "exterminaciones": "exterminación",
    "exterminad": "exterminar",
    "exterminada": "exterminar",
    "exterminadas": "exterminar",
    "exterminado": "exterminar",
    "exterminadora": "exterminador",
    "exterminadoras": "exterminador",
    "exterminadores": "exterminador",
    "exterminados": "exterminar",
    "exterminamos": "exterminar",
    "exterminan": "exterminar",
    "exterminando": "exterminar",
    "exterminara": "exterminar",
    "exterminarais": "exterminar",
    "exterminaran": "exterminar",
    "exterminaras": "exterminar",
    "exterminare": "exterminar",
    "exterminareis": "exterminar",
    "exterminaremos": "exterminar",
    "exterminaren": "exterminar",
    "exterminares": "exterminar",
    "exterminaron": "exterminar",
    "exterminará": "exterminar",
    "exterminarán": "exterminar",
    "exterminarás": "exterminar",
    "exterminaré": "exterminar",
    "exterminaréis": "exterminar",
    "exterminaría": "exterminar",
    "exterminaríais": "exterminar",
    "exterminaríamos": "exterminar",
    "exterminarían": "exterminar",
    "exterminarías": "exterminar",
    "exterminas": "exterminar",
    "exterminase": "exterminar",
    "exterminaseis": "exterminar",
    "exterminasen": "exterminar",
    "exterminases": "exterminar",
    "exterminaste": "exterminar",
    "exterminasteis": "exterminar",
    "extermine": "exterminar",
    "exterminemos": "exterminar",
    "exterminen": "exterminar",
    "extermines": "exterminar",
    "exterminios": "exterminio",
    "extermino": "exterminar",
    "exterminábamos": "exterminar",
    "extermináis": "exterminar",
    "extermináramos": "exterminar",
    "extermináremos": "exterminar",
    "exterminásemos": "exterminar",
    "exterminé": "exterminar",
    "exterminéis": "exterminar",
    "exterminó": "exterminar",
    "externa": "externo",
    "externados": "externado",
    "externalidades": "externalidad",
    "externalismos": "externalismo",
    "externalizaciones": "externalización",
    "externas": "externo",
    "externos": "externo",
    "exteroceptiva": "exteroceptivo",
    "exteroceptivas": "exteroceptivo",
    "exteroceptivos": "exteroceptivo",
    "extienda": "extender",
    "extiendan": "extender",
    "extiendas": "extender",
    "extiende": "extender",
    "extienden": "extender",
    "extiendes": "extender",
    "extiendo": "extender",
    "extinciones": "extinción",
    "extinga": "extinguir",
    "extingamos": "extinguir",
    "extingan": "extinguir",
    "extingas": "extinguir",
    "extingo": "extinguir",
    "extingue": "extinguir",
    "extinguen": "extinguir",
    "extingues": "extinguir",
    "extinguibles": "extinguible",
    "extinguid": "extinguir",
    "extinguida": "extinguir",
    "extinguidas": "extinguir",
    "extinguido": "extinguir",
    "extinguidos": "extinguir",
    "extinguiendo": "extinguir",
    "extinguiera": "extinguir",
    "extinguierais": "extinguir",
    "extinguieran": "extinguir",
    "extinguieras": "extinguir",
    "extinguiere": "extinguir",
    "extinguiereis": "extinguir",
    "extinguieren": "extinguir",
    "extinguieres": "extinguir",
    "extinguieron": "extinguir",
    "extinguiese": "extinguir",
    "extinguieseis": "extinguir",
    "extinguiesen": "extinguir",
    "extinguieses": "extinguir",
    "extinguimos": "extinguir",
    "extinguiremos": "extinguir",
    "extinguirá": "extinguir",
    "extinguirán": "extinguir",
    "extinguirás": "extinguir",
    "extinguiré": "extinguir",
    "extinguiréis": "extinguir",
    "extinguiría": "extinguir",
    "extinguiríais": "extinguir",
    "extinguiríamos": "extinguir",
    "extinguirían": "extinguir",
    "extinguirías": "extinguir",
    "extinguiste": "extinguir",
    "extinguisteis": "extinguir",
    "extinguiéramos": "extinguir",
    "extinguiéremos": "extinguir",
    "extinguiésemos": "extinguir",
    "extinguió": "extinguir",
    "extinguí": "extinguir",
    "extinguía": "extinguir",
    "extinguíais": "extinguir",
    "extinguíamos": "extinguir",
    "extinguían": "extinguir",
    "extinguías": "extinguir",
    "extinguís": "extinguir",
    "extingáis": "extinguir",
    "extinta": "extinto",
    "extintas": "extinto",
    "extintiva": "extintivo",
    "extintivas": "extintivo",
    "extintivos": "extintivo",
    "extintora": "extintor",
    "extintoras": "extintor",
    "extintores": "extintor",
    "extintos": "extinto",
    "extirpa": "extirpar",
    "extirpaba": "extirpar",
    "extirpabais": "extirpar",
    "extirpaban": "extirpar",
    "extirpabas": "extirpar",
    "extirpables": "extirpable",
    "extirpaciones": "extirpación",
    "extirpad": "extirpar",
    "extirpada": "extirpar",
    "extirpadas": "extirpar",
    "extirpado": "extirpar",
    "extirpadora": "extirpador",
    "extirpadoras": "extirpador",
    "extirpadores": "extirpador",
    "extirpados": "extirpar",
    "extirpamos": "extirpar",
    "extirpan": "extirpar",
    "extirpando": "extirpar",
    "extirpara": "extirpar",
    "extirparais": "extirpar",
    "extirparan": "extirpar",
    "extirparas": "extirpar",
    "extirpare": "extirpar",
    "extirpareis": "extirpar",
    "extirparemos": "extirpar",
    "extirparen": "extirpar",
    "extirpares": "extirpar",
    "extirparon": "extirpar",
    "extirpará": "extirpar",
    "extirparán": "extirpar",
    "extirparás": "extirpar",
    "extirparé": "extirpar",
    "extirparéis": "extirpar",
    "extirparía": "extirpar",
    "extirparíais": "extirpar",
    "extirparíamos": "extirpar",
    "extirparían": "extirpar",
    "extirparías": "extirpar",
    "extirpas": "extirpar",
    "extirpase": "extirpar",
    "extirpaseis": "extirpar",
    "extirpasen": "extirpar",
    "extirpases": "extirpar",
    "extirpaste": "extirpar",
    "extirpasteis": "extirpar",
    "extirpe": "extirpar",
    "extirpemos": "extirpar",
    "extirpen": "extirpar",
    "extirpes": "extirpar",
    "extirpo": "extirpar",
    "extirpábamos": "extirpar",
    "extirpáis": "extirpar",
    "extirpáramos": "extirpar",
    "extirpáremos": "extirpar",
    "extirpásemos": "extirpar",
    "extirpé": "extirpar",
    "extirpéis": "extirpar",
    "extirpó": "extirpar",
    "extornos": "extorno",
    "extorsiona": "extorsionar",
    "extorsionaba": "extorsionar",
    "extorsionabais": "extorsionar",
    "extorsionaban": "extorsionar",
    "extorsionabas": "extorsionar",
    "extorsionad": "extorsionar",
    "extorsionada": "extorsionar",
    "extorsionadas": "extorsionar",
    "extorsionado": "extorsionar",
    "extorsionadora": "extorsionador",
    "extorsionadoras": "extorsionador",
    "extorsionadores": "extorsionador",
    "extorsionados": "extorsionar",
    "extorsionamos": "extorsionar",
    "extorsionan": "extorsionar",
    "extorsionando": "extorsionar",
    "extorsionara": "extorsionar",
    "extorsionarais": "extorsionar",
    "extorsionaran": "extorsionar",
    "extorsionaras": "extorsionar",
    "extorsionare": "extorsionar",
    "extorsionareis": "extorsionar",
    "extorsionaremos": "extorsionar",
    "extorsionaren": "extorsionar",
    "extorsionares": "extorsionar",
    "extorsionaron": "extorsionar",
    "extorsionará": "extorsionar",
    "extorsionarán": "extorsionar",
    "extorsionarás": "extorsionar",
    "extorsionaré": "extorsionar",
    "extorsionaréis": "extorsionar",
    "extorsionaría": "extorsionar",
    "extorsionaríais": "extorsionar",
    "extorsionaríamos": "extorsionar",
    "extorsionarían": "extorsionar",
    "extorsionarías": "extorsionar",
    "extorsionas": "extorsionar",
    "extorsionase": "extorsionar",
    "extorsionaseis": "extorsionar",
    "extorsionasen": "extorsionar",
    "extorsionases": "extorsionar",
    "extorsionaste": "extorsionar",
    "extorsionasteis": "extorsionar",
    "extorsione": "extorsionar",
    "extorsionemos": "extorsionar",
    "extorsionen": "extorsionar",
    "extorsiones": "extorsionar",
    "extorsionistas": "extorsionista",
    "extorsiono": "extorsionar",
    "extorsionábamos": "extorsionar",
    "extorsionáis": "extorsionar",
    "extorsionáramos": "extorsionar",
    "extorsionáremos": "extorsionar",
    "extorsionásemos": "extorsionar",
    "extorsioné": "extorsionar",
    "extorsionéis": "extorsionar",
    "extorsionó": "extorsionar",
    "extraatmosférica": "extraatmosférico",
    "extraatmosféricas": "extraatmosférico",
    "extraatmosféricos": "extraatmosférico",
    "extrabrillantes": "extrabrillante",
    "extrabucales": "extrabucal",
    "extracciones": "extracción",
    "extracelulares": "extracelular",
    "extracomunitaria": "extracomunitario",
    "extracomunitarias": "extracomunitario",
    "extracomunitarios": "extracomunitario",
    "extracontables": "extracontable",
    "extracontractuales": "extracontractual",
    "extracoronales": "extracoronal",
    "extracorpórea": "extracorpóreo",
    "extracorpóreas": "extracorpóreo",
    "extracorpóreos": "extracorpóreo",
    "extracorrientes": "extracorriente",
    "extracraneales": "extracraneal",
    "extracta": "extractar",
    "extractaba": "extractar",
    "extractabais": "extractar",
    "extractaban": "extractar",
    "extractabas": "extractar",
    "extractad": "extractar",
    "extractada": "extractar",
    "extractadas": "extractar",
    "extractado": "extractar",
    "extractadora": "extractador",
    "extractadoras": "extractador",
    "extractadores": "extractador",
    "extractados": "extractar",
    "extractamos": "extractar",
    "extractan": "extractar",
    "extractando": "extractar",
    "extractara": "extractar",
    "extractarais": "extractar",
    "extractaran": "extractar",
    "extractaras": "extractar",
    "extractare": "extractar",
    "extractareis": "extractar",
    "extractaremos": "extractar",
    "extractaren": "extractar",
    "extractares": "extractar",
    "extractaron": "extractar",
    "extractará": "extractar",
    "extractarán": "extractar",
    "extractarás": "extractar",
    "extractaré": "extractar",
    "extractaréis": "extractar",
    "extractaría": "extractar",
    "extractaríais": "extractar",
    "extractaríamos": "extractar",
    "extractarían": "extractar",
    "extractarías": "extractar",
    "extractas": "extractar",
    "extractase": "extractar",
    "extractaseis": "extractar",
    "extractasen": "extractar",
    "extractases": "extractar",
    "extractaste": "extractar",
    "extractasteis": "extractar",
    "extracte": "extractar",
    "extractemos": "extractar",
    "extracten": "extractar",
    "extractes": "extractar",
    "extractiva": "extractivo",
    "extractivas": "extractivo",
    "extractivos": "extractivo",
    "extracto": "extractar",
    "extractora": "extractor",
    "extractoras": "extractor",
    "extractores": "extractor",
    "extractos": "extracto",
    "extractábamos": "extractar",
    "extractáis": "extractar",
    "extractáramos": "extractar",
    "extractáremos": "extractar",
    "extractásemos": "extractar",
    "extracté": "extractar",
    "extractéis": "extractar",
    "extractó": "extractar",
    "extracurriculares": "extracurricular",
    "extradiciones": "extradición",
    "extradita": "extraditar",
    "extraditaba": "extraditar",
    "extraditabais": "extraditar",
    "extraditaban": "extraditar",
    "extraditabas": "extraditar",
    "extraditables": "extraditable",
    "extraditaciones": "extraditación",
    "extraditad": "extraditar",
    "extraditada": "extraditar",
    "extraditadas": "extraditar",
    "extraditado": "extraditar",
    "extraditados": "extraditar",
    "extraditamos": "extraditar",
    "extraditan": "extraditar",
    "extraditando": "extraditar",
    "extraditara": "extraditar",
    "extraditarais": "extraditar",
    "extraditaran": "extraditar",
    "extraditaras": "extraditar",
    "extraditare": "extraditar",
    "extraditareis": "extraditar",
    "extraditaremos": "extraditar",
    "extraditaren": "extraditar",
    "extraditares": "extraditar",
    "extraditaron": "extraditar",
    "extraditará": "extraditar",
    "extraditarán": "extraditar",
    "extraditarás": "extraditar",
    "extraditaré": "extraditar",
    "extraditaréis": "extraditar",
    "extraditaría": "extraditar",
    "extraditaríais": "extraditar",
    "extraditaríamos": "extraditar",
    "extraditarían": "extraditar",
    "extraditarías": "extraditar",
    "extraditas": "extraditar",
    "extraditase": "extraditar",
    "extraditaseis": "extraditar",
    "extraditasen": "extraditar",
    "extraditases": "extraditar",
    "extraditaste": "extraditar",
    "extraditasteis": "extraditar",
    "extradite": "extraditar",
    "extraditemos": "extraditar",
    "extraditen": "extraditar",
    "extradites": "extraditar",
    "extradito": "extraditar",
    "extraditábamos": "extraditar",
    "extraditáis": "extraditar",
    "extraditáramos": "extraditar",
    "extraditáremos": "extraditar",
    "extraditásemos": "extraditar",
    "extradité": "extraditar",
    "extraditéis": "extraditar",
    "extraditó": "extraditar",
    "extradoses": "extradós",
    "extrae": "extraer",
    "extraed": "extraer",
    "extraemos": "extraer",
    "extraen": "extraer",
    "extraeremos": "extraer",
    "extraerá": "extraer",
    "extraerán": "extraer",
    "extraerás": "extraer",
    "extraeré": "extraer",
    "extraeréis": "extraer",
    "extraería": "extraer",
    "extraeríais": "extraer",
    "extraeríamos": "extraer",
    "extraerían": "extraer",
    "extraerías": "extraer",
    "extraes": "extraer",
    "extraescolares": "extraescolar",
    "extrafina": "extrafino",
    "extrafinas": "extrafino",
    "extrafinos": "extrafino",
    "extrafuncionales": "extrafuncional",
    "extrahospitalaria": "extrahospitalario",
    "extrahospitalarias": "extrahospitalario",
    "extrahospitalarios": "extrahospitalario",
    "extraiga": "extraer",
    "extraigamos": "extraer",
    "extraigan": "extraer",
    "extraigas": "extraer",
    "extraigo": "extraer",
    "extraigáis": "extraer",
    "extraje": "extraer",
    "extrajera": "extraer",
    "extrajerais": "extraer",
    "extrajeran": "extraer",
    "extrajeras": "extraer",
    "extrajere": "extraer",
    "extrajereis": "extraer",
    "extrajeren": "extraer",
    "extrajeres": "extraer",
    "extrajeron": "extraer",
    "extrajese": "extraer",
    "extrajeseis": "extraer",
    "extrajesen": "extraer",
    "extrajeses": "extraer",
    "extrajimos": "extraer",
    "extrajiste": "extraer",
    "extrajisteis": "extraer",
    "extrajo": "extraer",
    "extrajudiciales": "extrajudicial",
    "extrajéramos": "extraer",
    "extrajéremos": "extraer",
    "extrajésemos": "extraer",
    "extralarga": "extralargo",
    "extralargas": "extralargo",
    "extralargos": "extralargo",
    "extralegales": "extralegal",
    "extralimita": "extralimitar",
    "extralimitaba": "extralimitar",
    "extralimitabais": "extralimitar",
    "extralimitaban": "extralimitar",
    "extralimitabas": "extralimitar",
    "extralimitaciones": "extralimitación",
    "extralimitad": "extralimitar",
    "extralimitada": "extralimitar",
    "extralimitadas": "extralimitar",
    "extralimitado": "extralimitar",
    "extralimitados": "extralimitar",
    "extralimitamos": "extralimitar",
    "extralimitan": "extralimitar",
    "extralimitando": "extralimitar",
    "extralimitara": "extralimitar",
    "extralimitarais": "extralimitar",
    "extralimitaran": "extralimitar",
    "extralimitaras": "extralimitar",
    "extralimitare": "extralimitar",
    "extralimitareis": "extralimitar",
    "extralimitaremos": "extralimitar",
    "extralimitaren": "extralimitar",
    "extralimitares": "extralimitar",
    "extralimitaron": "extralimitar",
    "extralimitará": "extralimitar",
    "extralimitarán": "extralimitar",
    "extralimitarás": "extralimitar",
    "extralimitaré": "extralimitar",
    "extralimitaréis": "extralimitar",
    "extralimitaría": "extralimitar",
    "extralimitaríais": "extralimitar",
    "extralimitaríamos": "extralimitar",
    "extralimitarían": "extralimitar",
    "extralimitarías": "extralimitar",
    "extralimitas": "extralimitar",
    "extralimitase": "extralimitar",
    "extralimitaseis": "extralimitar",
    "extralimitasen": "extralimitar",
    "extralimitases": "extralimitar",
    "extralimitaste": "extralimitar",
    "extralimitasteis": "extralimitar",
    "extralimite": "extralimitar",
    "extralimitemos": "extralimitar",
    "extralimiten": "extralimitar",
    "extralimites": "extralimitar",
    "extralimito": "extralimitar",
    "extralimitábamos": "extralimitar",
    "extralimitáis": "extralimitar",
    "extralimitáramos": "extralimitar",
    "extralimitáremos": "extralimitar",
    "extralimitásemos": "extralimitar",
    "extralimité": "extralimitar",
    "extralimitéis": "extralimitar",
    "extralimitó": "extralimitar",
    "extralingüística": "extralingüístico",
    "extralingüísticas": "extralingüístico",
    "extralingüísticos": "extralingüístico",
    "extramaritales": "extramarital",
    "extranjera": "extranjero",
    "extranjeras": "extranjero",
    "extranjerice": "extranjerizar",
    "extranjericemos": "extranjerizar",
    "extranjericen": "extranjerizar",
    "extranjerices": "extranjerizar",
    "extranjericé": "extranjerizar",
    "extranjericéis": "extranjerizar",
    "extranjerismos": "extranjerismo",
    "extranjeriza": "extranjerizar",
    "extranjerizaba": "extranjerizar",
    "extranjerizabais": "extranjerizar",
    "extranjerizaban": "extranjerizar",
    "extranjerizabas": "extranjerizar",
    "extranjerizad": "extranjerizar",
    "extranjerizada": "extranjerizar",
    "extranjerizadas": "extranjerizar",
    "extranjerizado": "extranjerizar",
    "extranjerizados": "extranjerizar",
    "extranjerizamientos": "extranjerizamiento",
    "extranjerizamos": "extranjerizar",
    "extranjerizan": "extranjerizar",
    "extranjerizando": "extranjerizar",
    "extranjerizara": "extranjerizar",
    "extranjerizarais": "extranjerizar",
    "extranjerizaran": "extranjerizar",
    "extranjerizaras": "extranjerizar",
    "extranjerizare": "extranjerizar",
    "extranjerizareis": "extranjerizar",
    "extranjerizaremos": "extranjerizar",
    "extranjerizaren": "extranjerizar",
    "extranjerizares": "extranjerizar",
    "extranjerizaron": "extranjerizar",
    "extranjerizará": "extranjerizar",
    "extranjerizarán": "extranjerizar",
    "extranjerizarás": "extranjerizar",
    "extranjerizaré": "extranjerizar",
    "extranjerizaréis": "extranjerizar",
    "extranjerizaría": "extranjerizar",
    "extranjerizaríais": "extranjerizar",
    "extranjerizaríamos": "extranjerizar",
    "extranjerizarían": "extranjerizar",
    "extranjerizarías": "extranjerizar",
    "extranjerizas": "extranjerizar",
    "extranjerizase": "extranjerizar",
    "extranjerizaseis": "extranjerizar",
    "extranjerizasen": "extranjerizar",
    "extranjerizases": "extranjerizar",
    "extranjerizaste": "extranjerizar",
    "extranjerizasteis": "extranjerizar",
    "extranjerizo": "extranjerizar",
    "extranjerizábamos": "extranjerizar",
    "extranjerizáis": "extranjerizar",
    "extranjerizáramos": "extranjerizar",
    "extranjerizáremos": "extranjerizar",
    "extranjerizásemos": "extranjerizar",
    "extranjerizó": "extranjerizar",
    "extranjeros": "extranjero",
    "extranjerías": "extranjería",
    "extranjías": "extranjía",
    "extraoculares": "extraocular",
    "extraoficiales": "extraoficial",
    "extraorales": "extraoral",
    "extraordinaria": "extraordinario",
    "extraordinarias": "extraordinario",
    "extraordinarios": "extraordinario",
    "extraparlamentaria": "extraparlamentario",
    "extraparlamentarias": "extraparlamentario",
    "extraparlamentarios": "extraparlamentario",
    "extraplomos": "extraplomo",
    "extrapola": "extrapolar",
    "extrapolaba": "extrapolar",
    "extrapolabais": "extrapolar",
    "extrapolaban": "extrapolar",
    "extrapolabas": "extrapolar",
    "extrapolables": "extrapolable",
    "extrapolaciones": "extrapolación",
    "extrapolad": "extrapolar",
    "extrapolada": "extrapolar",
    "extrapoladas": "extrapolar",
    "extrapolado": "extrapolar",
    "extrapolados": "extrapolar",
    "extrapolamos": "extrapolar",
    "extrapolan": "extrapolar",
    "extrapolando": "extrapolar",
    "extrapolara": "extrapolar",
    "extrapolarais": "extrapolar",
    "extrapolaran": "extrapolar",
    "extrapolaras": "extrapolar",
    "extrapolare": "extrapolar",
    "extrapolareis": "extrapolar",
    "extrapolaremos": "extrapolar",
    "extrapolaren": "extrapolar",
    "extrapolares": "extrapolar",
    "extrapolaron": "extrapolar",
    "extrapolará": "extrapolar",
    "extrapolarán": "extrapolar",
    "extrapolarás": "extrapolar",
    "extrapolaré": "extrapolar",
    "extrapolaréis": "extrapolar",
    "extrapolaría": "extrapolar",
    "extrapolaríais": "extrapolar",
    "extrapolaríamos": "extrapolar",
    "extrapolarían": "extrapolar",
    "extrapolarías": "extrapolar",
    "extrapolas": "extrapolar",
    "extrapolase": "extrapolar",
    "extrapolaseis": "extrapolar",
    "extrapolasen": "extrapolar",
    "extrapolases": "extrapolar",
    "extrapolaste": "extrapolar",
    "extrapolasteis": "extrapolar",
    "extrapole": "extrapolar",
    "extrapolemos": "extrapolar",
    "extrapolen": "extrapolar",
    "extrapoles": "extrapolar",
    "extrapolo": "extrapolar",
    "extrapolábamos": "extrapolar",
    "extrapoláis": "extrapolar",
    "extrapoláramos": "extrapolar",
    "extrapoláremos": "extrapolar",
    "extrapolásemos": "extrapolar",
    "extrapolé": "extrapolar",
    "extrapoléis": "extrapolar",
    "extrapoló": "extrapolar",
    "extrapresupuestaria": "extrapresupuestario",
    "extrapresupuestarias": "extrapresupuestario",
    "extrapresupuestarios": "extrapresupuestario",
    "extraprofundidades": "extraprofundidad",
    "extrarradios": "extrarradio",
    "extras": "extra",
    "extrasalariales": "extrasalarial",
    "extrasensoriales": "extrasensorial",
    "extrasístoles": "extrasístole",
    "extraterrestres": "extraterrestre",
    "extraterritoriales": "extraterritorial",
    "extraterritorialidades": "extraterritorialidad",
    "extratémporas": "extratémpora",
    "extrauterina": "extrauterino",
    "extrauterinas": "extrauterino",
    "extrauterinos": "extrauterino",
    "extravagancias": "extravagancia",
    "extravagantes": "extravagante",
    "extravasa": "extravasar",
    "extravasaba": "extravasar",
    "extravasabais": "extravasar",
    "extravasaban": "extravasar",
    "extravasabas": "extravasar",
    "extravasaciones": "extravasación",
    "extravasad": "extravasar",
    "extravasada": "extravasar",
    "extravasadas": "extravasar",
    "extravasado": "extravasar",
    "extravasados": "extravasar",
    "extravasamos": "extravasar",
    "extravasan": "extravasar",
    "extravasando": "extravasar",
    "extravasara": "extravasar",
    "extravasarais": "extravasar",
    "extravasaran": "extravasar",
    "extravasaras": "extravasar",
    "extravasare": "extravasar",
    "extravasareis": "extravasar",
    "extravasaremos": "extravasar",
    "extravasaren": "extravasar",
    "extravasares": "extravasar",
    "extravasaron": "extravasar",
    "extravasará": "extravasar",
    "extravasarán": "extravasar",
    "extravasarás": "extravasar",
    "extravasaré": "extravasar",
    "extravasaréis": "extravasar",
    "extravasaría": "extravasar",
    "extravasaríais": "extravasar",
    "extravasaríamos": "extravasar",
    "extravasarían": "extravasar",
    "extravasarías": "extravasar",
    "extravasas": "extravasar",
    "extravasase": "extravasar",
    "extravasaseis": "extravasar",
    "extravasasen": "extravasar",
    "extravasases": "extravasar",
    "extravasaste": "extravasar",
    "extravasasteis": "extravasar",
    "extravase": "extravasar",
    "extravasemos": "extravasar",
    "extravasen": "extravasar",
    "extravases": "extravasar",
    "extravaso": "extravasar",
    "extravasábamos": "extravasar",
    "extravasáis": "extravasar",
    "extravasáramos": "extravasar",
    "extravasáremos": "extravasar",
    "extravasásemos": "extravasar",
    "extravasé": "extravasar",
    "extravaséis": "extravasar",
    "extravasó": "extravasar",
    "extravena": "extravenar",
    "extravenaba": "extravenar",
    "extravenabais": "extravenar",
    "extravenaban": "extravenar",
    "extravenabas": "extravenar",
    "extravenad": "extravenar",
    "extravenada": "extravenar",
    "extravenadas": "extravenar",
    "extravenado": "extravenar",
    "extravenados": "extravenar",
    "extravenamos": "extravenar",
    "extravenan": "extravenar",
    "extravenando": "extravenar",
    "extravenara": "extravenar",
    "extravenarais": "extravenar",
    "extravenaran": "extravenar",
    "extravenaras": "extravenar",
    "extravenare": "extravenar",
    "extravenareis": "extravenar",
    "extravenaremos": "extravenar",
    "extravenaren": "extravenar",
    "extravenares": "extravenar",
    "extravenaron": "extravenar",
    "extravenará": "extravenar",
    "extravenarán": "extravenar",
    "extravenarás": "extravenar",
    "extravenaré": "extravenar",
    "extravenaréis": "extravenar",
    "extravenaría": "extravenar",
    "extravenaríais": "extravenar",
    "extravenaríamos": "extravenar",
    "extravenarían": "extravenar",
    "extravenarías": "extravenar",
    "extravenas": "extravenar",
    "extravenase": "extravenar",
    "extravenaseis": "extravenar",
    "extravenasen": "extravenar",
    "extravenases": "extravenar",
    "extravenaste": "extravenar",
    "extravenasteis": "extravenar",
    "extravene": "extravenar",
    "extravenemos": "extravenar",
    "extravenen": "extravenar",
    "extravenes": "extravenar",
    "extraveno": "extravenar",
    "extravenábamos": "extravenar",
    "extravenáis": "extravenar",
    "extravenáramos": "extravenar",
    "extravenáremos": "extravenar",
    "extravenásemos": "extravenar",
    "extravené": "extravenar",
    "extravenéis": "extravenar",
    "extravenó": "extravenar",
    "extraversiones": "extraversión",
    "extravertida": "extravertido",
    "extravertidas": "extravertido",
    "extravertidos": "extravertido",
    "extraviaba": "extraviar",
    "extraviabais": "extraviar",
    "extraviaban": "extraviar",
    "extraviabas": "extraviar",
    "extraviad": "extraviar",
    "extraviada": "extraviar",
    "extraviadas": "extraviar",
    "extraviado": "extraviar",
    "extraviados": "extraviar",
    "extraviamos": "extraviar",
    "extraviando": "extraviar",
    "extraviara": "extraviar",
    "extraviarais": "extraviar",
    "extraviaran": "extraviar",
    "extraviaras": "extraviar",
    "extraviare": "extraviar",
    "extraviareis": "extraviar",
    "extraviaremos": "extraviar",
    "extraviaren": "extraviar",
    "extraviares": "extraviar",
    "extraviaron": "extraviar",
    "extraviará": "extraviar",
    "extraviarán": "extraviar",
    "extraviarás": "extraviar",
    "extraviaré": "extraviar",
    "extraviaréis": "extraviar",
    "extraviaría": "extraviar",
    "extraviaríais": "extraviar",
    "extraviaríamos": "extraviar",
    "extraviarían": "extraviar",
    "extraviarías": "extraviar",
    "extraviase": "extraviar",
    "extraviaseis": "extraviar",
    "extraviasen": "extraviar",
    "extraviases": "extraviar",
    "extraviaste": "extraviar",
    "extraviasteis": "extraviar",
    "extraviemos": "extraviar",
    "extraviábamos": "extraviar",
    "extraviáis": "extraviar",
    "extraviáramos": "extraviar",
    "extraviáremos": "extraviar",
    "extraviásemos": "extraviar",
    "extravié": "extraviar",
    "extraviéis": "extraviar",
    "extravió": "extraviar",
    "extravía": "extraviar",
    "extravían": "extraviar",
    "extravías": "extraviar",
    "extravíe": "extraviar",
    "extravíen": "extraviar",
    "extravíes": "extraviar",
    "extravío": "extraviar",
    "extravíos": "extravío",
    "extrayendo": "extraer",
    "extraéis": "extraer",
    "extraía": "extraer",
    "extraíais": "extraer",
    "extraíamos": "extraer",
    "extraían": "extraer",
    "extraías": "extraer",
    "extraíbles": "extraíble",
    "extraída": "extraer",
    "extraídas": "extraer",
    "extraído": "extraer",
    "extraídos": "extraer",
    "extraña": "extraño",
    "extrañaba": "extrañar",
    "extrañabais": "extrañar",
    "extrañaban": "extrañar",
    "extrañabas": "extrañar",
    "extrañad": "extrañar",
    "extrañada": "extrañar",
    "extrañadas": "extrañar",
    "extrañado": "extrañar",
    "extrañados": "extrañar",
    "extrañamientos": "extrañamiento",
    "extrañamos": "extrañar",
    "extrañan": "extrañar",
    "extrañando": "extrañar",
    "extrañara": "extrañar",
    "extrañarais": "extrañar",
    "extrañaran": "extrañar",
    "extrañaras": "extrañar",
    "extrañare": "extrañar",
    "extrañareis": "extrañar",
    "extrañaremos": "extrañar",
    "extrañaren": "extrañar",
    "extrañares": "extrañar",
    "extrañaron": "extrañar",
    "extrañará": "extrañar",
    "extrañarán": "extrañar",
    "extrañarás": "extrañar",
    "extrañaré": "extrañar",
    "extrañaréis": "extrañar",
    "extrañaría": "extrañar",
    "extrañaríais": "extrañar",
    "extrañaríamos": "extrañar",
    "extrañarían": "extrañar",
    "extrañarías": "extrañar",
    "extrañas": "extraño",
    "extrañase": "extrañar",
    "extrañaseis": "extrañar",
    "extrañasen": "extrañar",
    "extrañases": "extrañar",
    "extrañaste": "extrañar",
    "extrañasteis": "extrañar",
    "extrañe": "extrañar",
    "extrañeces": "extrañez",
    "extrañemos": "extrañar",
    "extrañen": "extrañar",
    "extrañes": "extrañar",
    "extrañezas": "extrañeza",
    "extraño": "extrañar",
    "extraños": "extraño",
    "extrañábamos": "extrañar",
    "extrañáis": "extrañar",
    "extrañáramos": "extrañar",
    "extrañáremos": "extrañar",
    "extrañásemos": "extrañar",
    "extrañé": "extrañar",
    "extrañéis": "extrañar",
    "extrañó": "extrañar",
    "extrema": "extremo",
    "extremaba": "extremar",
    "extremabais": "extremar",
    "extremaban": "extremar",
    "extremabas": "extremar",
    "extremad": "extremar",
    "extremada": "extremar",
    "extremadas": "extremar",
    "extremado": "extremar",
    "extremados": "extremar",
    "extremamos": "extremar",
    "extreman": "extremar",
    "extremando": "extremar",
    "extremara": "extremar",
    "extremarais": "extremar",
    "extremaran": "extremar",
    "extremaras": "extremar",
    "extremare": "extremar",
    "extremareis": "extremar",
    "extremaremos": "extremar",
    "extremaren": "extremar",
    "extremares": "extremar",
    "extremaron": "extremar",
    "extremará": "extremar",
    "extremarán": "extremar",
    "extremarás": "extremar",
    "extremaré": "extremar",
    "extremaréis": "extremar",
    "extremaría": "extremar",
    "extremaríais": "extremar",
    "extremaríamos": "extremar",
    "extremarían": "extremar",
    "extremarías": "extremar",
    "extremas": "extremo",
    "extremase": "extremar",
    "extremaseis": "extremar",
    "extremasen": "extremar",
    "extremases": "extremar",
    "extremaste": "extremar",
    "extremasteis": "extremar",
    "extremaunciones": "extremaunción",
    "extreme": "extremar",
    "extrememos": "extremar",
    "extremen": "extremar",
    "extremes": "extremar",
    "extremeña": "extremeño",
    "extremeñas": "extremeño",
    "extremeños": "extremeño",
    "extremidades": "extremidad",
    "extremismos": "extremismo",
    "extremistas": "extremista",
    "extremo": "extremar",
    "extremos": "extremo",
    "extremosa": "extremoso",
    "extremosas": "extremoso",
    "extremosidades": "extremosidad",
    "extremosos": "extremoso",
    "extremábamos": "extremar",
    "extremáis": "extremar",
    "extremáramos": "extremar",
    "extremáremos": "extremar",
    "extremásemos": "extremar",
    "extremé": "extremar",
    "extreméis": "extremar",
    "extremó": "extremar",
    "extromisiones": "extromisión",
    "extroversiones": "extroversión",
    "extrovertida": "extrovertido",
    "extrovertidas": "extrovertido",
    "extrovertidos": "extrovertido",
    "extruda": "extrudir",
    "extrudamos": "extrudir",
    "extrudan": "extrudir",
    "extrudas": "extrudir",
    "extrude": "extrudir",
    "extruden": "extrudir",
    "extrudes": "extrudir",
    "extrudid": "extrudir",
    "extrudida": "extrudir",
    "extrudidas": "extrudir",
    "extrudido": "extrudir",
    "extrudidos": "extrudir",
    "extrudiendo": "extrudir",
    "extrudiera": "extrudir",
    "extrudierais": "extrudir",
    "extrudieran": "extrudir",
    "extrudieras": "extrudir",
    "extrudiere": "extrudir",
    "extrudiereis": "extrudir",
    "extrudieren": "extrudir",
    "extrudieres": "extrudir",
    "extrudieron": "extrudir",
    "extrudiese": "extrudir",
    "extrudieseis": "extrudir",
    "extrudiesen": "extrudir",
    "extrudieses": "extrudir",
    "extrudimos": "extrudir",
    "extrudiremos": "extrudir",
    "extrudirá": "extrudir",
    "extrudirán": "extrudir",
    "extrudirás": "extrudir",
    "extrudiré": "extrudir",
    "extrudiréis": "extrudir",
    "extrudiría": "extrudir",
    "extrudiríais": "extrudir",
    "extrudiríamos": "extrudir",
    "extrudirían": "extrudir",
    "extrudirías": "extrudir",
    "extrudiste": "extrudir",
    "extrudisteis": "extrudir",
    "extrudiéramos": "extrudir",
    "extrudiéremos": "extrudir",
    "extrudiésemos": "extrudir",
    "extrudió": "extrudir",
    "extrudo": "extrudir",
    "extrudáis": "extrudir",
    "extrudí": "extrudir",
    "extrudía": "extrudir",
    "extrudíais": "extrudir",
    "extrudíamos": "extrudir",
    "extrudían": "extrudir",
    "extrudías": "extrudir",
    "extrudís": "extrudir",
    "extruidores": "extruidor",
    "extrusionadores": "extrusionador",
    "extrusiones": "extrusión",
    "extrusiva": "extrusivo",
    "extrusivas": "extrusivo",
    "extrusivos": "extrusivo",
    "extrusora": "extrusor",
    "extrusoras": "extrusora",
    "extrusores": "extrusor",
    "extrínseca": "extrínseco",
    "extrínsecas": "extrínseco",
    "extrínsecos": "extrínseco",
    "extática": "extático",
    "extáticas": "extático",
    "extáticos": "extático",
    "exuberancias": "exuberancia",
    "exuberantes": "exuberante",
    "exuda": "exudar",
    "exudaba": "exudar",
    "exudabais": "exudar",
    "exudaban": "exudar",
    "exudabas": "exudar",
    "exudaciones": "exudación",
    "exudad": "exudar",
    "exudada": "exudar",
    "exudadas": "exudar",
    "exudado": "exudar",
    "exudados": "exudar",
    "exudamos": "exudar",
    "exudan": "exudar",
    "exudando": "exudar",
    "exudara": "exudar",
    "exudarais": "exudar",
    "exudaran": "exudar",
    "exudaras": "exudar",
    "exudare": "exudar",
    "exudareis": "exudar",
    "exudaremos": "exudar",
    "exudaren": "exudar",
    "exudares": "exudar",
    "exudaron": "exudar",
    "exudará": "exudar",
    "exudarán": "exudar",
    "exudarás": "exudar",
    "exudaré": "exudar",
    "exudaréis": "exudar",
    "exudaría": "exudar",
    "exudaríais": "exudar",
    "exudaríamos": "exudar",
    "exudarían": "exudar",
    "exudarías": "exudar",
    "exudas": "exudar",
    "exudase": "exudar",
    "exudaseis": "exudar",
    "exudasen": "exudar",
    "exudases": "exudar",
    "exudaste": "exudar",
    "exudasteis": "exudar",
    "exudativa": "exudativo",
    "exudativas": "exudativo",
    "exudativos": "exudativo",
    "exude": "exudar",
    "exudemos": "exudar",
    "exuden": "exudar",
    "exudes": "exudar",
    "exudo": "exudar",
    "exudábamos": "exudar",
    "exudáis": "exudar",
    "exudáramos": "exudar",
    "exudáremos": "exudar",
    "exudásemos": "exudar",
    "exudé": "exudar",
    "exudéis": "exudar",
    "exudó": "exudar",
    "exulcera": "exulcerar",
    "exulceraba": "exulcerar",
    "exulcerabais": "exulcerar",
    "exulceraban": "exulcerar",
    "exulcerabas": "exulcerar",
    "exulceraciones": "exulceración",
    "exulcerad": "exulcerar",
    "exulcerada": "exulcerar",
    "exulceradas": "exulcerar",
    "exulcerado": "exulcerar",
    "exulcerados": "exulcerar",
    "exulceramos": "exulcerar",
    "exulceran": "exulcerar",
    "exulcerando": "exulcerar",
    "exulcerara": "exulcerar",
    "exulcerarais": "exulcerar",
    "exulceraran": "exulcerar",
    "exulceraras": "exulcerar",
    "exulcerare": "exulcerar",
    "exulcerareis": "exulcerar",
    "exulceraremos": "exulcerar",
    "exulceraren": "exulcerar",
    "exulcerares": "exulcerar",
    "exulceraron": "exulcerar",
    "exulcerará": "exulcerar",
    "exulcerarán": "exulcerar",
    "exulcerarás": "exulcerar",
    "exulceraré": "exulcerar",
    "exulceraréis": "exulcerar",
    "exulceraría": "exulcerar",
    "exulceraríais": "exulcerar",
    "exulceraríamos": "exulcerar",
    "exulcerarían": "exulcerar",
    "exulcerarías": "exulcerar",
    "exulceras": "exulcerar",
    "exulcerase": "exulcerar",
    "exulceraseis": "exulcerar",
    "exulcerasen": "exulcerar",
    "exulcerases": "exulcerar",
    "exulceraste": "exulcerar",
    "exulcerasteis": "exulcerar",
    "exulcere": "exulcerar",
    "exulceremos": "exulcerar",
    "exulceren": "exulcerar",
    "exulceres": "exulcerar",
    "exulcero": "exulcerar",
    "exulcerábamos": "exulcerar",
    "exulceráis": "exulcerar",
    "exulceráramos": "exulcerar",
    "exulceráremos": "exulcerar",
    "exulcerásemos": "exulcerar",
    "exulceré": "exulcerar",
    "exulceréis": "exulcerar",
    "exulceró": "exulcerar",
    "exulta": "exultar",
    "exultaba": "exultar",
    "exultabais": "exultar",
    "exultaban": "exultar",
    "exultabas": "exultar",
    "exultaciones": "exultación",
    "exultad": "exultar",
    "exultada": "exultar",
    "exultadas": "exultar",
    "exultado": "exultar",
    "exultados": "exultar",
    "exultamos": "exultar",
    "exultan": "exultar",
    "exultando": "exultar",
    "exultantes": "exultante",
    "exultara": "exultar",
    "exultarais": "exultar",
    "exultaran": "exultar",
    "exultaras": "exultar",
    "exultare": "exultar",
    "exultareis": "exultar",
    "exultaremos": "exultar",
    "exultaren": "exultar",
    "exultares": "exultar",
    "exultaron": "exultar",
    "exultará": "exultar",
    "exultarán": "exultar",
    "exultarás": "exultar",
    "exultaré": "exultar",
    "exultaréis": "exultar",
    "exultaría": "exultar",
    "exultaríais": "exultar",
    "exultaríamos": "exultar",
    "exultarían": "exultar",
    "exultarías": "exultar",
    "exultas": "exultar",
    "exultase": "exultar",
    "exultaseis": "exultar",
    "exultasen": "exultar",
    "exultases": "exultar",
    "exultaste": "exultar",
    "exultasteis": "exultar",
    "exulte": "exultar",
    "exultemos": "exultar",
    "exulten": "exultar",
    "exultes": "exultar",
    "exulto": "exultar",
    "exultábamos": "exultar",
    "exultáis": "exultar",
    "exultáramos": "exultar",
    "exultáremos": "exultar",
    "exultásemos": "exultar",
    "exulté": "exultar",
    "exultéis": "exultar",
    "exultó": "exultar",
    "exutorios": "exutorio",
    "exvicesecretaria": "exvicesecretario",
    "exvicesecretarias": "exvicesecretario",
    "exvicesecretarios": "exvicesecretario",
    "exvotos": "exvoto",
    "exámenes": "examen",
    "exánimes": "exánime",
    "exáricos": "exárico",
    "exégetas": "exégeta",
    "exógena": "exógeno",
    "exógenas": "exógeno",
    "exógenos": "exógeno",
    "exótica": "exótico",
    "exóticas": "exótico",
    "exóticos": "exótico",
    "eyacula": "eyacular",
    "eyaculaba": "eyacular",
    "eyaculabais": "eyacular",
    "eyaculaban": "eyacular",
    "eyaculabas": "eyacular",
    "eyaculaciones": "eyaculación",
    "eyaculad": "eyacular",
    "eyaculada": "eyacular",
    "eyaculadas": "eyacular",
    "eyaculado": "eyacular",
    "eyaculadora": "eyaculador",
    "eyaculadoras": "eyaculador",
    "eyaculadores": "eyaculador",
    "eyaculados": "eyacular",
    "eyaculamos": "eyacular",
    "eyaculan": "eyacular",
    "eyaculando": "eyacular",
    "eyaculara": "eyacular",
    "eyacularais": "eyacular",
    "eyacularan": "eyacular",
    "eyacularas": "eyacular",
    "eyaculare": "eyacular",
    "eyaculareis": "eyacular",
    "eyacularemos": "eyacular",
    "eyacularen": "eyacular",
    "eyaculares": "eyacular",
    "eyacularon": "eyacular",
    "eyaculará": "eyacular",
    "eyacularán": "eyacular",
    "eyacularás": "eyacular",
    "eyacularé": "eyacular",
    "eyacularéis": "eyacular",
    "eyacularía": "eyacular",
    "eyacularíais": "eyacular",
    "eyacularíamos": "eyacular",
    "eyacularían": "eyacular",
    "eyacularías": "eyacular",
    "eyaculas": "eyacular",
    "eyaculase": "eyacular",
    "eyaculaseis": "eyacular",
    "eyaculasen": "eyacular",
    "eyaculases": "eyacular",
    "eyaculaste": "eyacular",
    "eyaculasteis": "eyacular",
    "eyaculatoria": "eyaculatorio",
    "eyaculatorias": "eyaculatorio",
    "eyaculatorios": "eyaculatorio",
    "eyacule": "eyacular",
    "eyaculemos": "eyacular",
    "eyaculen": "eyacular",
    "eyacules": "eyacular",
    "eyaculo": "eyacular",
    "eyaculábamos": "eyacular",
    "eyaculáis": "eyacular",
    "eyaculáramos": "eyacular",
    "eyaculáremos": "eyacular",
    "eyaculásemos": "eyacular",
    "eyaculé": "eyacular",
    "eyaculéis": "eyacular",
    "eyaculó": "eyacular",
    "eyecciones": "eyección",
    "eyectables": "eyectable",
    "eyectiva": "eyectivo",
    "eyectivas": "eyectivo",
    "eyectivos": "eyectivo",
    "eyectores": "eyector",
    "eñes": "eñe",
    "eólica": "eólico",
    "eólicas": "eólico",
    "eólicos": "eólico",
    "fabadas": "fabada",
    "fabiana": "fabiano",
    "fabianas": "fabiano",
    "fabianismos": "fabianismo",
    "fabianos": "fabiano",
    "fabismos": "fabismo",
    "fablas": "fabla",
    "fabliaues": "fabliau",
    "fabliaus": "fabliau",
    "fabordones": "fabordón",
    "fabriana": "fabriano",
    "fabrianas": "fabriano",
    "fabrianos": "fabriano",
    "fabrica": "fabricar",
    "fabricaba": "fabricar",
    "fabricabais": "fabricar",
    "fabricaban": "fabricar",
    "fabricabas": "fabricar",
    "fabricaciones": "fabricación",
    "fabricad": "fabricar",
    "fabricada": "fabricar",
    "fabricadas": "fabricar",
    "fabricado": "fabricar",
    "fabricados": "fabricar",
    "fabricamos": "fabricar",
    "fabrican": "fabricar",
    "fabricando": "fabricar",
    "fabricantes": "fabricante",
    "fabricara": "fabricar",
    "fabricarais": "fabricar",
    "fabricaran": "fabricar",
    "fabricaras": "fabricar",
    "fabricare": "fabricar",
    "fabricareis": "fabricar",
    "fabricaremos": "fabricar",
    "fabricaren": "fabricar",
    "fabricares": "fabricar",
    "fabricaron": "fabricar",
    "fabricará": "fabricar",
    "fabricarán": "fabricar",
    "fabricarás": "fabricar",
    "fabricaré": "fabricar",
    "fabricaréis": "fabricar",
    "fabricaría": "fabricar",
    "fabricaríais": "fabricar",
    "fabricaríamos": "fabricar",
    "fabricarían": "fabricar",
    "fabricarías": "fabricar",
    "fabricas": "fabricar",
    "fabricase": "fabricar",
    "fabricaseis": "fabricar",
    "fabricasen": "fabricar",
    "fabricases": "fabricar",
    "fabricaste": "fabricar",
    "fabricasteis": "fabricar",
    "fabrico": "fabricar",
    "fabricábamos": "fabricar",
    "fabricáis": "fabricar",
    "fabricáramos": "fabricar",
    "fabricáremos": "fabricar",
    "fabricásemos": "fabricar",
    "fabricó": "fabricar",
    "fabriles": "fabril",
    "fabrique": "fabricar",
    "fabriquemos": "fabricar",
    "fabriquen": "fabricar",
    "fabriquera": "fabriquero",
    "fabriqueras": "fabriquero",
    "fabriqueros": "fabriquero",
    "fabriques": "fabricar",
    "fabriqué": "fabricar",
    "fabriquéis": "fabricar",
    "fabulaciones": "fabulación",
    "fabuladora": "fabulador",
    "fabuladoras": "fabulador",
    "fabuladores": "fabulador",
    "fabularios": "fabulario",
    "fabulesca": "fabulesco",
    "fabulescas": "fabulesco",
    "fabulescos": "fabulesco",
    "fabulistas": "fabulista",
    "fabulosa": "fabuloso",
    "fabulosas": "fabuloso",
    "fabulosidades": "fabulosidad",
    "fabulosos": "fabuloso",
    "facas": "faca",
    "facazos": "facazo",
    "faccionaria": "faccionario",
    "faccionarias": "faccionario",
    "faccionarios": "faccionario",
    "facciones": "facción",
    "facciosa": "faccioso",
    "facciosas": "faccioso",
    "facciosos": "faccioso",
    "facecias": "facecia",
    "faces": "faz",
    "facetas": "faceta",
    "fachada": "fachado",
    "fachadas": "fachado",
    "fachados": "fachado",
    "fachas": "facha",
    "fachea": "fachear",
    "facheaba": "fachear",
    "facheabais": "fachear",
    "facheaban": "fachear",
    "facheabas": "fachear",
    "fachead": "fachear",
    "facheada": "fachear",
    "facheadas": "fachear",
    "facheado": "fachear",
    "facheados": "fachear",
    "facheamos": "fachear",
    "fachean": "fachear",
    "facheando": "fachear",
    "facheara": "fachear",
    "fachearais": "fachear",
    "fachearan": "fachear",
    "fachearas": "fachear",
    "facheare": "fachear",
    "facheareis": "fachear",
    "fachearemos": "fachear",
    "fachearen": "fachear",
    "facheares": "fachear",
    "fachearon": "fachear",
    "facheará": "fachear",
    "fachearán": "fachear",
    "fachearás": "fachear",
    "fachearé": "fachear",
    "fachearéis": "fachear",
    "fachearía": "fachear",
    "fachearíais": "fachear",
    "fachearíamos": "fachear",
    "fachearían": "fachear",
    "fachearías": "fachear",
    "facheas": "fachear",
    "fachease": "fachear",
    "facheaseis": "fachear",
    "facheasen": "fachear",
    "facheases": "fachear",
    "facheaste": "fachear",
    "facheasteis": "fachear",
    "fachee": "fachear",
    "facheemos": "fachear",
    "facheen": "fachear",
    "fachees": "fachear",
    "fachendas": "fachenda",
    "fachendea": "fachendear",
    "fachendeaba": "fachendear",
    "fachendeabais": "fachendear",
    "fachendeaban": "fachendear",
    "fachendeabas": "fachendear",
    "fachendead": "fachendear",
    "fachendeada": "fachendear",
    "fachendeadas": "fachendear",
    "fachendeado": "fachendear",
    "fachendeados": "fachendear",
    "fachendeamos": "fachendear",
    "fachendean": "fachendear",
    "fachendeando": "fachendear",
    "fachendeara": "fachendear",
    "fachendearais": "fachendear",
    "fachendearan": "fachendear",
    "fachendearas": "fachendear",
    "fachendeare": "fachendear",
    "fachendeareis": "fachendear",
    "fachendearemos": "fachendear",
    "fachendearen": "fachendear",
    "fachendeares": "fachendear",
    "fachendearon": "fachendear",
    "fachendeará": "fachendear",
    "fachendearán": "fachendear",
    "fachendearás": "fachendear",
    "fachendearé": "fachendear",
    "fachendearéis": "fachendear",
    "fachendearía": "fachendear",
    "fachendearíais": "fachendear",
    "fachendearíamos": "fachendear",
    "fachendearían": "fachendear",
    "fachendearías": "fachendear",
    "fachendeas": "fachendear",
    "fachendease": "fachendear",
    "fachendeaseis": "fachendear",
    "fachendeasen": "fachendear",
    "fachendeases": "fachendear",
    "fachendeaste": "fachendear",
    "fachendeasteis": "fachendear",
    "fachendee": "fachendear",
    "fachendeemos": "fachendear",
    "fachendeen": "fachendear",
    "fachendees": "fachendear",
    "fachendeo": "fachendear",
    "fachendeábamos": "fachendear",
    "fachendeáis": "fachendear",
    "fachendeáramos": "fachendear",
    "fachendeáremos": "fachendear",
    "fachendeásemos": "fachendear",
    "fachendeé": "fachendear",
    "fachendeéis": "fachendear",
    "fachendeó": "fachendear",
    "fachendistas": "fachendista",
    "fachendona": "fachendón",
    "fachendonas": "fachendón",
    "fachendones": "fachendón",
    "fachendosa": "fachendoso",
    "fachendosas": "fachendoso",
    "fachendosos": "fachendoso",
    "facheo": "fachear",
    "facheábamos": "fachear",
    "facheáis": "fachear",
    "facheáramos": "fachear",
    "facheáremos": "fachear",
    "facheásemos": "fachear",
    "facheé": "fachear",
    "facheéis": "fachear",
    "facheó": "fachear",
    "fachosa": "fachoso",
    "fachosas": "fachoso",
    "fachosos": "fachoso",
    "fachuda": "fachudo",
    "fachudas": "fachudo",
    "fachudos": "fachudo",
    "faciales": "facial",
    "facilidades": "facilidad",
    "facilita": "facilitar",
    "facilitaba": "facilitar",
    "facilitabais": "facilitar",
    "facilitaban": "facilitar",
    "facilitabas": "facilitar",
    "facilitaciones": "facilitación",
    "facilitad": "facilitar",
    "facilitada": "facilitar",
    "facilitadas": "facilitar",
    "facilitado": "facilitar",
    "facilitadora": "facilitador",
    "facilitadoras": "facilitador",
    "facilitadores": "facilitador",
    "facilitados": "facilitar",
    "facilitamos": "facilitar",
    "facilitan": "facilitar",
    "facilitando": "facilitar",
    "facilitara": "facilitar",
    "facilitarais": "facilitar",
    "facilitaran": "facilitar",
    "facilitaras": "facilitar",
    "facilitare": "facilitar",
    "facilitareis": "facilitar",
    "facilitaremos": "facilitar",
    "facilitaren": "facilitar",
    "facilitares": "facilitar",
    "facilitaron": "facilitar",
    "facilitará": "facilitar",
    "facilitarán": "facilitar",
    "facilitarás": "facilitar",
    "facilitaré": "facilitar",
    "facilitaréis": "facilitar",
    "facilitaría": "facilitar",
    "facilitaríais": "facilitar",
    "facilitaríamos": "facilitar",
    "facilitarían": "facilitar",
    "facilitarías": "facilitar",
    "facilitas": "facilitar",
    "facilitase": "facilitar",
    "facilitaseis": "facilitar",
    "facilitasen": "facilitar",
    "facilitases": "facilitar",
    "facilitaste": "facilitar",
    "facilitasteis": "facilitar",
    "facilite": "facilitar",
    "facilitemos": "facilitar",
    "faciliten": "facilitar",
    "facilites": "facilitar",
    "facilito": "facilitar",
    "facilitona": "facilitón",
    "facilitonas": "facilitón",
    "facilitones": "facilitón",
    "facilitábamos": "facilitar",
    "facilitáis": "facilitar",
    "facilitáramos": "facilitar",
    "facilitáremos": "facilitar",
    "facilitásemos": "facilitar",
    "facilité": "facilitar",
    "facilitéis": "facilitar",
    "facilitó": "facilitar",
    "facinerosa": "facineroso",
    "facinerosas": "facineroso",
    "facinerosos": "facineroso",
    "facistoles": "facistol",
    "facocistectomías": "facocistectomía",
    "facoemulsificaciones": "facoemulsificación",
    "facoglaucomas": "facoglaucoma",
    "facomalacias": "facomalacia",
    "facoscopias": "facoscopia",
    "facsimilares": "facsimilar",
    "facsímiles": "facsímil",
    "factibilidades": "factibilidad",
    "factibles": "factible",
    "facticia": "facticio",
    "facticias": "facticio",
    "facticios": "facticio",
    "factitiva": "factitivo",
    "factitivas": "factitivo",
    "factitivos": "factitivo",
    "factorajes": "factoraje",
    "factores": "factor",
    "factoriales": "factorial",
    "factorings": "factoring",
    "factorizables": "factorizable",
    "factorizaciones": "factorización",
    "factorías": "factoría",
    "factos": "facto",
    "factuales": "factual",
    "factura": "facturar",
    "facturaba": "facturar",
    "facturabais": "facturar",
    "facturaban": "facturar",
    "facturabas": "facturar",
    "facturables": "facturable",
    "facturaciones": "facturación",
    "facturad": "facturar",
    "facturada": "facturar",
    "facturadas": "facturar",
    "facturado": "facturar",
    "facturados": "facturar",
    "facturamos": "facturar",
    "facturan": "facturar",
    "facturando": "facturar",
    "facturara": "facturar",
    "facturarais": "facturar",
    "facturaran": "facturar",
    "facturaras": "facturar",
    "facturare": "facturar",
    "facturareis": "facturar",
    "facturaremos": "facturar",
    "facturaren": "facturar",
    "facturares": "facturar",
    "facturaron": "facturar",
    "facturará": "facturar",
    "facturarán": "facturar",
    "facturarás": "facturar",
    "facturaré": "facturar",
    "facturaréis": "facturar",
    "facturaría": "facturar",
    "facturaríais": "facturar",
    "facturaríamos": "facturar",
    "facturarían": "facturar",
    "facturarías": "facturar",
    "facturas": "facturar",
    "facturase": "facturar",
    "facturaseis": "facturar",
    "facturasen": "facturar",
    "facturases": "facturar",
    "facturaste": "facturar",
    "facturasteis": "facturar",
    "facture": "facturar",
    "facturemos": "facturar",
    "facturen": "facturar",
    "factures": "facturar",
    "facturo": "facturar",
    "facturábamos": "facturar",
    "facturáis": "facturar",
    "facturáramos": "facturar",
    "facturáremos": "facturar",
    "facturásemos": "facturar",
    "facturé": "facturar",
    "facturéis": "facturar",
    "facturó": "facturar",
    "factótumes": "factótum",
    "factótums": "factótum",
    "faculta": "facultar",
    "facultaba": "facultar",
    "facultabais": "facultar",
    "facultaban": "facultar",
    "facultabas": "facultar",
    "facultad": "facultar",
    "facultada": "facultar",
    "facultadas": "facultar",
    "facultades": "facultad",
    "facultado": "facultar",
    "facultados": "facultar",
    "facultamos": "facultar",
    "facultan": "facultar",
    "facultando": "facultar",
    "facultara": "facultar",
    "facultarais": "facultar",
    "facultaran": "facultar",
    "facultaras": "facultar",
    "facultare": "facultar",
    "facultareis": "facultar",
    "facultaremos": "facultar",
    "facultaren": "facultar",
    "facultares": "facultar",
    "facultaron": "facultar",
    "facultará": "facultar",
    "facultarán": "facultar",
    "facultarás": "facultar",
    "facultaré": "facultar",
    "facultaréis": "facultar",
    "facultaría": "facultar",
    "facultaríais": "facultar",
    "facultaríamos": "facultar",
    "facultarían": "facultar",
    "facultarías": "facultar",
    "facultas": "facultar",
    "facultase": "facultar",
    "facultaseis": "facultar",
    "facultasen": "facultar",
    "facultases": "facultar",
    "facultaste": "facultar",
    "facultasteis": "facultar",
    "facultativa": "facultativo",
    "facultativas": "facultativo",
    "facultativos": "facultativo",
    "faculte": "facultar",
    "facultemos": "facultar",
    "faculten": "facultar",
    "facultes": "facultar",
    "faculto": "facultar",
    "facultábamos": "facultar",
    "facultáis": "facultar",
    "facultáramos": "facultar",
    "facultáremos": "facultar",
    "facultásemos": "facultar",
    "faculté": "facultar",
    "facultéis": "facultar",
    "facultó": "facultar",
    "facunda": "facundo",
    "facundas": "facundo",
    "facundias": "facundia",
    "facundos": "facundo",
    "fados": "fado",
    "faena": "faenar",
    "faenaba": "faenar",
    "faenabais": "faenar",
    "faenaban": "faenar",
    "faenabas": "faenar",
    "faenad": "faenar",
    "faenada": "faenar",
    "faenadas": "faenar",
    "faenado": "faenar",
    "faenados": "faenar",
    "faenamos": "faenar",
    "faenan": "faenar",
    "faenando": "faenar",
    "faenara": "faenar",
    "faenarais": "faenar",
    "faenaran": "faenar",
    "faenaras": "faenar",
    "faenare": "faenar",
    "faenareis": "faenar",
    "faenaremos": "faenar",
    "faenaren": "faenar",
    "faenares": "faenar",
    "faenaron": "faenar",
    "faenará": "faenar",
    "faenarán": "faenar",
    "faenarás": "faenar",
    "faenaré": "faenar",
    "faenaréis": "faenar",
    "faenaría": "faenar",
    "faenaríais": "faenar",
    "faenaríamos": "faenar",
    "faenarían": "faenar",
    "faenarías": "faenar",
    "faenas": "faenar",
    "faenase": "faenar",
    "faenaseis": "faenar",
    "faenasen": "faenar",
    "faenases": "faenar",
    "faenaste": "faenar",
    "faenasteis": "faenar",
    "faene": "faenar",
    "faenemos": "faenar",
    "faenen": "faenar",
    "faenes": "faenar",
    "faeno": "faenar",
    "faenábamos": "faenar",
    "faenáis": "faenar",
    "faenáramos": "faenar",
    "faenáremos": "faenar",
    "faenásemos": "faenar",
    "faené": "faenar",
    "faenéis": "faenar",
    "faenó": "faenar",
    "faetones": "faetón",
    "fagocitaria": "fagocitario",
    "fagocitarias": "fagocitario",
    "fagocitarios": "fagocitario",
    "fagocitos": "fagocito",
    "fagocítica": "fagocítico",
    "fagocíticas": "fagocítico",
    "fagocíticos": "fagocítico",
    "fagotes": "fagote",
    "fagotistas": "fagotista",
    "fagots": "fagot",
    "fagáceas": "fagácea",
    "faisana": "faisán",
    "faisanas": "faisán",
    "faisanera": "faisanero",
    "faisaneras": "faisanero",
    "faisaneros": "faisanero",
    "faisanerías": "faisanería",
    "faisanes": "faisán",
    "faja": "fajar",
    "fajaba": "fajar",
    "fajabais": "fajar",
    "fajaban": "fajar",
    "fajabas": "fajar",
    "fajad": "fajar",
    "fajada": "fajar",
    "fajadas": "fajar",
    "fajado": "fajar",
    "fajados": "fajar",
    "fajaduras": "fajadura",
    "fajamientos": "fajamiento",
    "fajamos": "fajar",
    "fajan": "fajar",
    "fajando": "fajar",
    "fajara": "fajar",
    "fajarais": "fajar",
    "fajaran": "fajar",
    "fajaras": "fajar",
    "fajardos": "fajardo",
    "fajare": "fajar",
    "fajareis": "fajar",
    "fajaremos": "fajar",
    "fajaren": "fajar",
    "fajares": "fajar",
    "fajaron": "fajar",
    "fajará": "fajar",
    "fajarán": "fajar",
    "fajarás": "fajar",
    "fajaré": "fajar",
    "fajaréis": "fajar",
    "fajaría": "fajar",
    "fajaríais": "fajar",
    "fajaríamos": "fajar",
    "fajarían": "fajar",
    "fajarías": "fajar",
    "fajas": "fajar",
    "fajase": "fajar",
    "fajaseis": "fajar",
    "fajasen": "fajar",
    "fajases": "fajar",
    "fajaste": "fajar",
    "fajasteis": "fajar",
    "faje": "fajar",
    "fajeada": "fajeado",
    "fajeadas": "fajeado",
    "fajeados": "fajeado",
    "fajemos": "fajar",
    "fajen": "fajar",
    "fajera": "fajero",
    "fajeras": "fajero",
    "fajeros": "fajero",
    "fajes": "fajar",
    "fajillas": "fajilla",
    "fajinadas": "fajinada",
    "fajinas": "fajina",
    "fajines": "fajín",
    "fajo": "fajar",
    "fajona": "fajón",
    "fajonas": "fajón",
    "fajones": "fajón",
    "fajos": "fajo",
    "fajábamos": "fajar",
    "fajáis": "fajar",
    "fajáramos": "fajar",
    "fajáremos": "fajar",
    "fajásemos": "fajar",
    "fajé": "fajar",
    "fajéis": "fajar",
    "fajó": "fajar",
    "falaces": "falaz",
    "falacias": "falacia",
    "falangeros": "falangero",
    "falanges": "falange",
    "falangetas": "falangeta",
    "falanginas": "falangina",
    "falangismos": "falangismo",
    "falangistas": "falangista",
    "falangéridos": "falangérido",
    "falansterios": "falansterio",
    "falaropos": "falaropo",
    "falazes": "falaz",
    "falcada": "falcado",
    "falcadas": "falcado",
    "falcados": "falcado",
    "falcarios": "falcario",
    "falcas": "falca",
    "falces": "falce",
    "falcidias": "falcidia",
    "falciformes": "falciforme",
    "falcirrostra": "falcirrostro",
    "falcirrostras": "falcirrostro",
    "falcirrostros": "falcirrostro",
    "falcones": "falcón",
    "falconetes": "falconete",
    "falconiformes": "falconiforme",
    "falcónida": "falcónido",
    "falcónidas": "falcónido",
    "falcónidos": "falcónido",
    "faldamentas": "faldamenta",
    "faldamentos": "faldamento",
    "faldares": "faldar",
    "faldas": "falda",
    "faldea": "faldear",
    "faldeaba": "faldear",
    "faldeabais": "faldear",
    "faldeaban": "faldear",
    "faldeabas": "faldear",
    "faldead": "faldear",
    "faldeada": "faldear",
    "faldeadas": "faldear",
    "faldeado": "faldear",
    "faldeados": "faldear",
    "faldeamos": "faldear",
    "faldean": "faldear",
    "faldeando": "faldear",
    "faldeara": "faldear",
    "faldearais": "faldear",
    "faldearan": "faldear",
    "faldearas": "faldear",
    "faldeare": "faldear",
    "faldeareis": "faldear",
    "faldearemos": "faldear",
    "faldearen": "faldear",
    "faldeares": "faldear",
    "faldearon": "faldear",
    "faldeará": "faldear",
    "faldearán": "faldear",
    "faldearás": "faldear",
    "faldearé": "faldear",
    "faldearéis": "faldear",
    "faldearía": "faldear",
    "faldearíais": "faldear",
    "faldearíamos": "faldear",
    "faldearían": "faldear",
    "faldearías": "faldear",
    "faldeas": "faldear",
    "faldease": "faldear",
    "faldeaseis": "faldear",
    "faldeasen": "faldear",
    "faldeases": "faldear",
    "faldeaste": "faldear",
    "faldeasteis": "faldear",
    "faldee": "faldear",
    "faldeemos": "faldear",
    "faldeen": "faldear",
    "faldees": "faldear",
    "faldellines": "faldellín",
    "faldeo": "faldear",
    "faldera": "faldero",
    "falderas": "faldero",
    "falderos": "faldero",
    "faldetas": "faldeta",
    "faldeábamos": "faldear",
    "faldeáis": "faldear",
    "faldeáramos": "faldear",
    "faldeáremos": "faldear",
    "faldeásemos": "faldear",
    "faldeé": "faldear",
    "faldeéis": "faldear",
    "faldeó": "faldear",
    "faldinegra": "faldinegro",
    "faldinegras": "faldinegro",
    "faldinegros": "faldinegro",
    "faldistorios": "faldistorio",
    "faldoncillos": "faldoncillo",
    "faldones": "faldón",
    "faldriqueras": "faldriquera",
    "falduda": "faldudo",
    "faldudas": "faldudo",
    "faldudos": "faldudo",
    "faldularios": "faldulario",
    "falernos": "falerno",
    "falibilidades": "falibilidad",
    "falibles": "falible",
    "faliscos": "falisco",
    "falismos": "falismo",
    "falla": "fallo",
    "fallaba": "fallar",
    "fallabais": "fallar",
    "fallaban": "fallar",
    "fallabas": "fallar",
    "fallad": "fallar",
    "fallada": "fallar",
    "falladas": "fallar",
    "fallado": "fallar",
    "falladora": "fallador",
    "falladoras": "fallador",
    "falladores": "fallador",
    "fallados": "fallar",
    "fallamos": "fallir",
    "fallan": "fallir",
    "fallancas": "fallanca",
    "fallando": "fallar",
    "fallara": "fallar",
    "fallarais": "fallar",
    "fallaran": "fallar",
    "fallaras": "fallar",
    "fallare": "fallar",
    "fallareis": "fallar",
    "fallaremos": "fallar",
    "fallaren": "fallar",
    "fallares": "fallar",
    "fallaron": "fallar",
    "fallará": "fallar",
    "fallarán": "fallar",
    "fallarás": "fallar",
    "fallaré": "fallar",
    "fallaréis": "fallar",
    "fallaría": "fallar",
    "fallaríais": "fallar",
    "fallaríamos": "fallar",
    "fallarían": "fallar",
    "fallarías": "fallar",
    "fallas": "fallo",
    "fallase": "fallar",
    "fallaseis": "fallar",
    "fallasen": "fallar",
    "fallases": "fallar",
    "fallaste": "fallar",
    "fallasteis": "fallar",
    "falle": "fallir",
    "fallebas": "falleba",
    "fallece": "fallecer",
    "falleced": "fallecer",
    "fallecedera": "fallecedero",
    "fallecederas": "fallecedero",
    "fallecederos": "fallecedero",
    "fallecemos": "fallecer",
    "fallecen": "fallecer",
    "falleceremos": "fallecer",
    "fallecerá": "fallecer",
    "fallecerán": "fallecer",
    "fallecerás": "fallecer",
    "falleceré": "fallecer",
    "falleceréis": "fallecer",
    "fallecería": "fallecer",
    "falleceríais": "fallecer",
    "falleceríamos": "fallecer",
    "fallecerían": "fallecer",
    "fallecerías": "fallecer",
    "falleces": "fallecer",
    "fallecida": "fallecido",
    "fallecidas": "fallecido",
    "fallecido": "fallecer",
    "fallecidos": "fallecido",
    "falleciendo": "fallecer",
    "falleciera": "fallecer",
    "fallecierais": "fallecer",
    "fallecieran": "fallecer",
    "fallecieras": "fallecer",
    "falleciere": "fallecer",
    "falleciereis": "fallecer",
    "fallecieren": "fallecer",
    "fallecieres": "fallecer",
    "fallecieron": "fallecer",
    "falleciese": "fallecer",
    "fallecieseis": "fallecer",
    "falleciesen": "fallecer",
    "fallecieses": "fallecer",
    "fallecimientos": "fallecimiento",
    "fallecimos": "fallecer",
    "falleciste": "fallecer",
    "fallecisteis": "fallecer",
    "falleciéramos": "fallecer",
    "falleciéremos": "fallecer",
    "falleciésemos": "fallecer",
    "falleció": "fallecer",
    "fallecéis": "fallecer",
    "fallecí": "fallecer",
    "fallecía": "fallecer",
    "fallecíais": "fallecer",
    "fallecíamos": "fallecer",
    "fallecían": "fallecer",
    "fallecías": "fallecer",
    "fallemos": "fallar",
    "fallen": "fallir",
    "fallendo": "fallir",
    "fallera": "fallir",
    "fallerais": "fallir",
    "falleran": "fallir",
    "falleras": "fallir",
    "fallere": "fallir",
    "fallereis": "fallir",
    "falleren": "fallir",
    "falleres": "fallir",
    "falleron": "fallir",
    "falleros": "fallero",
    "falles": "fallir",
    "fallese": "fallir",
    "falleseis": "fallir",
    "fallesen": "fallir",
    "falleses": "fallir",
    "fallezca": "fallecer",
    "fallezcamos": "fallecer",
    "fallezcan": "fallecer",
    "fallezcas": "fallecer",
    "fallezco": "fallecer",
    "fallezcáis": "fallecer",
    "fallid": "fallir",
    "fallida": "fallir",
    "fallidas": "fallir",
    "fallido": "fallir",
    "fallidos": "fallir",
    "fallimos": "fallir",
    "falliremos": "fallir",
    "fallirá": "fallir",
    "fallirán": "fallir",
    "fallirás": "fallir",
    "falliré": "fallir",
    "falliréis": "fallir",
    "falliría": "fallir",
    "falliríais": "fallir",
    "falliríamos": "fallir",
    "fallirían": "fallir",
    "fallirías": "fallir",
    "falliste": "fallir",
    "fallisteis": "fallir",
    "fallo": "fallir",
    "fallos": "fallo",
    "fallábamos": "fallar",
    "falláis": "fallir",
    "falláramos": "fallar",
    "falláremos": "fallar",
    "fallásemos": "fallar",
    "fallé": "fallar",
    "falléis": "fallar",
    "falléramos": "fallir",
    "falléremos": "fallir",
    "fallésemos": "fallir",
    "fallí": "fallir",
    "fallía": "fallir",
    "fallíais": "fallir",
    "fallíamos": "fallir",
    "fallían": "fallir",
    "fallías": "fallir",
    "fallís": "fallir",
    "falló": "fallir",
    "faloidinas": "faloidina",
    "falos": "falo",
    "falsa": "falso",
    "falsaamarras": "falsaamarra",
    "falsaarmaduras": "falsaarmadura",
    "falsabragas": "falsabraga",
    "falsadas": "falsada",
    "falsaria": "falsario",
    "falsarias": "falsario",
    "falsarios": "falsario",
    "falsas": "falso",
    "falsea": "falsear",
    "falseaba": "falsear",
    "falseabais": "falsear",
    "falseaban": "falsear",
    "falseabas": "falsear",
    "falsead": "falsear",
    "falseada": "falsear",
    "falseadas": "falsear",
    "falseado": "falsear",
    "falseadora": "falseador",
    "falseadoras": "falseador",
    "falseadores": "falseador",
    "falseados": "falsear",
    "falseamientos": "falseamiento",
    "falseamos": "falsear",
    "falsean": "falsear",
    "falseando": "falsear",
    "falseara": "falsear",
    "falsearais": "falsear",
    "falsearan": "falsear",
    "falsearas": "falsear",
    "falseare": "falsear",
    "falseareis": "falsear",
    "falsearemos": "falsear",
    "falsearen": "falsear",
    "falseares": "falsear",
    "falsearon": "falsear",
    "falseará": "falsear",
    "falsearán": "falsear",
    "falsearás": "falsear",
    "falsearé": "falsear",
    "falsearéis": "falsear",
    "falsearía": "falsear",
    "falsearíais": "falsear",
    "falsearíamos": "falsear",
    "falsearían": "falsear",
    "falsearías": "falsear",
    "falseas": "falsear",
    "falsease": "falsear",
    "falseaseis": "falsear",
    "falseasen": "falsear",
    "falseases": "falsear",
    "falseaste": "falsear",
    "falseasteis": "falsear",
    "falsedades": "falsedad",
    "falsee": "falsear",
    "falseemos": "falsear",
    "falseen": "falsear",
    "falsees": "falsear",
    "falseo": "falsear",
    "falseos": "falseo",
    "falsetas": "falseta",
    "falsetes": "falsete",
    "falseábamos": "falsear",
    "falseáis": "falsear",
    "falseáramos": "falsear",
    "falseáremos": "falsear",
    "falseásemos": "falsear",
    "falseé": "falsear",
    "falseéis": "falsear",
    "falseó": "falsear",
    "falsifica": "falsificar",
    "falsificaba": "falsificar",
    "falsificabais": "falsificar",
    "falsificaban": "falsificar",
    "falsificabas": "falsificar",
    "falsificaciones": "falsificación",
    "falsificad": "falsificar",
    "falsificada": "falsificar",
    "falsificadas": "falsificar",
    "falsificado": "falsificar",
    "falsificadora": "falsificador",
    "falsificadoras": "falsificador",
    "falsificadores": "falsificador",
    "falsificados": "falsificar",
    "falsificamos": "falsificar",
    "falsifican": "falsificar",
    "falsificando": "falsificar",
    "falsificara": "falsificar",
    "falsificarais": "falsificar",
    "falsificaran": "falsificar",
    "falsificaras": "falsificar",
    "falsificare": "falsificar",
    "falsificareis": "falsificar",
    "falsificaremos": "falsificar",
    "falsificaren": "falsificar",
    "falsificares": "falsificar",
    "falsificaron": "falsificar",
    "falsificará": "falsificar",
    "falsificarán": "falsificar",
    "falsificarás": "falsificar",
    "falsificaré": "falsificar",
    "falsificaréis": "falsificar",
    "falsificaría": "falsificar",
    "falsificaríais": "falsificar",
    "falsificaríamos": "falsificar",
    "falsificarían": "falsificar",
    "falsificarías": "falsificar",
    "falsificas": "falsificar",
    "falsificase": "falsificar",
    "falsificaseis": "falsificar",
    "falsificasen": "falsificar",
    "falsificases": "falsificar",
    "falsificaste": "falsificar",
    "falsificasteis": "falsificar",
    "falsifico": "falsificar",
    "falsificábamos": "falsificar",
    "falsificáis": "falsificar",
    "falsificáramos": "falsificar",
    "falsificáremos": "falsificar",
    "falsificásemos": "falsificar",
    "falsificó": "falsificar",
    "falsifique": "falsificar",
    "falsifiquemos": "falsificar",
    "falsifiquen": "falsificar",
    "falsifiques": "falsificar",
    "falsifiqué": "falsificar",
    "falsifiquéis": "falsificar",
    "falsillas": "falsilla",
    "falsos": "falso",
    "falsías": "falsía",
    "falta": "falto",
    "faltaba": "faltar",
    "faltabais": "faltar",
    "faltaban": "faltar",
    "faltabas": "faltar",
    "faltad": "faltar",
    "faltada": "faltar",
    "faltadas": "faltar",
    "faltado": "faltar",
    "faltados": "faltar",
    "faltamos": "faltar",
    "faltan": "faltar",
    "faltando": "faltar",
    "faltantes": "faltante",
    "faltara": "faltar",
    "faltarais": "faltar",
    "faltaran": "faltar",
    "faltaras": "faltar",
    "faltare": "faltar",
    "faltareis": "faltar",
    "faltaremos": "faltar",
    "faltaren": "faltar",
    "faltares": "faltar",
    "faltaron": "faltar",
    "faltará": "faltar",
    "faltarán": "faltar",
    "faltarás": "faltar",
    "faltaré": "faltar",
    "faltaréis": "faltar",
    "faltaría": "faltar",
    "faltaríais": "faltar",
    "faltaríamos": "faltar",
    "faltarían": "faltar",
    "faltarías": "faltar",
    "faltas": "falto",
    "faltase": "faltar",
    "faltaseis": "faltar",
    "faltasen": "faltar",
    "faltases": "faltar",
    "faltaste": "faltar",
    "faltasteis": "faltar",
    "falte": "faltar",
    "faltemos": "faltar",
    "falten": "faltar",
    "faltes": "faltar",
    "falto": "faltar",
    "faltona": "faltón",
    "faltonas": "faltón",
    "faltones": "faltón",
    "faltos": "falto",
    "faltosa": "faltoso",
    "faltosas": "faltoso",
    "faltosos": "faltoso",
    "faltriqueras": "faltriquera",
    "faltábamos": "faltar",
    "faltáis": "faltar",
    "faltáramos": "faltar",
    "faltáremos": "faltar",
    "faltásemos": "faltar",
    "falté": "faltar",
    "faltéis": "faltar",
    "faltó": "faltar",
    "faluchos": "falucho",
    "faláricas": "falárica",
    "falúas": "falúa",
    "famas": "fama",
    "famatinitas": "famatinita",
    "familiares": "familiar",
    "familiarice": "familiarizar",
    "familiaricemos": "familiarizar",
    "familiaricen": "familiarizar",
    "familiarices": "familiarizar",
    "familiaricé": "familiarizar",
    "familiaricéis": "familiarizar",
    "familiaridades": "familiaridad",
    "familiariza": "familiarizar",
    "familiarizaba": "familiarizar",
    "familiarizabais": "familiarizar",
    "familiarizaban": "familiarizar",
    "familiarizabas": "familiarizar",
    "familiarizaciones": "familiarización",
    "familiarizad": "familiarizar",
    "familiarizada": "familiarizar",
    "familiarizadas": "familiarizar",
    "familiarizado": "familiarizar",
    "familiarizados": "familiarizar",
    "familiarizamos": "familiarizar",
    "familiarizan": "familiarizar",
    "familiarizando": "familiarizar",
    "familiarizara": "familiarizar",
    "familiarizarais": "familiarizar",
    "familiarizaran": "familiarizar",
    "familiarizaras": "familiarizar",
    "familiarizare": "familiarizar",
    "familiarizareis": "familiarizar",
    "familiarizaremos": "familiarizar",
    "familiarizaren": "familiarizar",
    "familiarizares": "familiarizar",
    "familiarizaron": "familiarizar",
    "familiarizará": "familiarizar",
    "familiarizarán": "familiarizar",
    "familiarizarás": "familiarizar",
    "familiarizaré": "familiarizar",
    "familiarizaréis": "familiarizar",
    "familiarizaría": "familiarizar",
    "familiarizaríais": "familiarizar",
    "familiarizaríamos": "familiarizar",
    "familiarizarían": "familiarizar",
    "familiarizarías": "familiarizar",
    "familiarizas": "familiarizar",
    "familiarizase": "familiarizar",
    "familiarizaseis": "familiarizar",
    "familiarizasen": "familiarizar",
    "familiarizases": "familiarizar",
    "familiarizaste": "familiarizar",
    "familiarizasteis": "familiarizar",
    "familiarizo": "familiarizar",
    "familiarizábamos": "familiarizar",
    "familiarizáis": "familiarizar",
    "familiarizáramos": "familiarizar",
    "familiarizáremos": "familiarizar",
    "familiarizásemos": "familiarizar",
    "familiarizó": "familiarizar",
    "familias": "familia",
    "familiaturas": "familiatura",
    "familiones": "familión",
    "familismos": "familismo",
    "famosa": "famoso",
    "famosas": "famoso",
    "famosos": "famoso",
    "famulares": "famular",
    "famulatos": "famulato",
    "famélica": "famélico",
    "famélicas": "famélico",
    "famélicos": "famélico",
    "fanales": "fanal",
    "fanas": "fana",
    "fanatice": "fanatizar",
    "fanaticemos": "fanatizar",
    "fanaticen": "fanatizar",
    "fanatices": "fanatizar",
    "fanaticé": "fanatizar",
    "fanaticéis": "fanatizar",
    "fanatismos": "fanatismo",
    "fanatiza": "fanatizar",
    "fanatizaba": "fanatizar",
    "fanatizabais": "fanatizar",
    "fanatizaban": "fanatizar",
    "fanatizabas": "fanatizar",
    "fanatizaciones": "fanatización",
    "fanatizad": "fanatizar",
    "fanatizada": "fanatizar",
    "fanatizadas": "fanatizar",
    "fanatizado": "fanatizar",
    "fanatizadora": "fanatizador",
    "fanatizadoras": "fanatizador",
    "fanatizadores": "fanatizador",
    "fanatizados": "fanatizar",
    "fanatizamos": "fanatizar",
    "fanatizan": "fanatizar",
    "fanatizando": "fanatizar",
    "fanatizara": "fanatizar",
    "fanatizarais": "fanatizar",
    "fanatizaran": "fanatizar",
    "fanatizaras": "fanatizar",
    "fanatizare": "fanatizar",
    "fanatizareis": "fanatizar",
    "fanatizaremos": "fanatizar",
    "fanatizaren": "fanatizar",
    "fanatizares": "fanatizar",
    "fanatizaron": "fanatizar",
    "fanatizará": "fanatizar",
    "fanatizarán": "fanatizar",
    "fanatizarás": "fanatizar",
    "fanatizaré": "fanatizar",
    "fanatizaréis": "fanatizar",
    "fanatizaría": "fanatizar",
    "fanatizaríais": "fanatizar",
    "fanatizaríamos": "fanatizar",
    "fanatizarían": "fanatizar",
    "fanatizarías": "fanatizar",
    "fanatizas": "fanatizar",
    "fanatizase": "fanatizar",
    "fanatizaseis": "fanatizar",
    "fanatizasen": "fanatizar",
    "fanatizases": "fanatizar",
    "fanatizaste": "fanatizar",
    "fanatizasteis": "fanatizar",
    "fanatizo": "fanatizar",
    "fanatizábamos": "fanatizar",
    "fanatizáis": "fanatizar",
    "fanatizáramos": "fanatizar",
    "fanatizáremos": "fanatizar",
    "fanatizásemos": "fanatizar",
    "fanatizó": "fanatizar",
    "fandangos": "fandango",
    "fandanguera": "fandanguero",
    "fandangueras": "fandanguero",
    "fandangueros": "fandanguero",
    "fandanguillos": "fandanguillo",
    "fanecas": "faneca",
    "fanegadas": "fanegada",
    "fanegas": "fanega",
    "faneritas": "fanerita",
    "fanerocristalina": "fanerocristalino",
    "fanerocristalinas": "fanerocristalino",
    "fanerocristalinos": "fanerocristalino",
    "fanerítica": "fanerítico",
    "faneríticas": "fanerítico",
    "faneríticos": "fanerítico",
    "fanerófitos": "fanerófito",
    "fanerógama": "fanerógamo",
    "fanerógamas": "fanerógamo",
    "fanerógamos": "fanerógamo",
    "fanfarrea": "fanfarrear",
    "fanfarreaba": "fanfarrear",
    "fanfarreabais": "fanfarrear",
    "fanfarreaban": "fanfarrear",
    "fanfarreabas": "fanfarrear",
    "fanfarread": "fanfarrear",
    "fanfarreada": "fanfarrear",
    "fanfarreadas": "fanfarrear",
    "fanfarreado": "fanfarrear",
    "fanfarreados": "fanfarrear",
    "fanfarreamos": "fanfarrear",
    "fanfarrean": "fanfarrear",
    "fanfarreando": "fanfarrear",
    "fanfarreara": "fanfarrear",
    "fanfarrearais": "fanfarrear",
    "fanfarrearan": "fanfarrear",
    "fanfarrearas": "fanfarrear",
    "fanfarreare": "fanfarrear",
    "fanfarreareis": "fanfarrear",
    "fanfarrearemos": "fanfarrear",
    "fanfarrearen": "fanfarrear",
    "fanfarreares": "fanfarrear",
    "fanfarrearon": "fanfarrear",
    "fanfarreará": "fanfarrear",
    "fanfarrearán": "fanfarrear",
    "fanfarrearás": "fanfarrear",
    "fanfarrearé": "fanfarrear",
    "fanfarrearéis": "fanfarrear",
    "fanfarrearía": "fanfarrear",
    "fanfarrearíais": "fanfarrear",
    "fanfarrearíamos": "fanfarrear",
    "fanfarrearían": "fanfarrear",
    "fanfarrearías": "fanfarrear",
    "fanfarreas": "fanfarrear",
    "fanfarrease": "fanfarrear",
    "fanfarreaseis": "fanfarrear",
    "fanfarreasen": "fanfarrear",
    "fanfarreases": "fanfarrear",
    "fanfarreaste": "fanfarrear",
    "fanfarreasteis": "fanfarrear",
    "fanfarree": "fanfarrear",
    "fanfarreemos": "fanfarrear",
    "fanfarreen": "fanfarrear",
    "fanfarrees": "fanfarrear",
    "fanfarreo": "fanfarrear",
    "fanfarreábamos": "fanfarrear",
    "fanfarreáis": "fanfarrear",
    "fanfarreáramos": "fanfarrear",
    "fanfarreáremos": "fanfarrear",
    "fanfarreásemos": "fanfarrear",
    "fanfarreé": "fanfarrear",
    "fanfarreéis": "fanfarrear",
    "fanfarreó": "fanfarrear",
    "fanfarrias": "fanfarria",
    "fanfarrona": "fanfarrón",
    "fanfarronadas": "fanfarronada",
    "fanfarronas": "fanfarrón",
    "fanfarronea": "fanfarronear",
    "fanfarroneaba": "fanfarronear",
    "fanfarroneabais": "fanfarronear",
    "fanfarroneaban": "fanfarronear",
    "fanfarroneabas": "fanfarronear",
    "fanfarronead": "fanfarronear",
    "fanfarroneada": "fanfarronear",
    "fanfarroneadas": "fanfarronear",
    "fanfarroneado": "fanfarronear",
    "fanfarroneados": "fanfarronear",
    "fanfarroneamos": "fanfarronear",
    "fanfarronean": "fanfarronear",
    "fanfarroneando": "fanfarronear",
    "fanfarroneara": "fanfarronear",
    "fanfarronearais": "fanfarronear",
    "fanfarronearan": "fanfarronear",
    "fanfarronearas": "fanfarronear",
    "fanfarroneare": "fanfarronear",
    "fanfarroneareis": "fanfarronear",
    "fanfarronearemos": "fanfarronear",
    "fanfarronearen": "fanfarronear",
    "fanfarroneares": "fanfarronear",
    "fanfarronearon": "fanfarronear",
    "fanfarroneará": "fanfarronear",
    "fanfarronearán": "fanfarronear",
    "fanfarronearás": "fanfarronear",
    "fanfarronearé": "fanfarronear",
    "fanfarronearéis": "fanfarronear",
    "fanfarronearía": "fanfarronear",
    "fanfarronearíais": "fanfarronear",
    "fanfarronearíamos": "fanfarronear",
    "fanfarronearían": "fanfarronear",
    "fanfarronearías": "fanfarronear",
    "fanfarroneas": "fanfarronear",
    "fanfarronease": "fanfarronear",
    "fanfarroneaseis": "fanfarronear",
    "fanfarroneasen": "fanfarronear",
    "fanfarroneases": "fanfarronear",
    "fanfarroneaste": "fanfarronear",
    "fanfarroneasteis": "fanfarronear",
    "fanfarronee": "fanfarronear",
    "fanfarroneemos": "fanfarronear",
    "fanfarroneen": "fanfarronear",
    "fanfarronees": "fanfarronear",
    "fanfarroneo": "fanfarronear",
    "fanfarronerías": "fanfarronería",
    "fanfarrones": "fanfarrón",
    "fanfarronescas": "fanfarronesca",
    "fanfarroneábamos": "fanfarronear",
    "fanfarroneáis": "fanfarronear",
    "fanfarroneáramos": "fanfarronear",
    "fanfarroneáremos": "fanfarronear",
    "fanfarroneásemos": "fanfarronear",
    "fanfarroneé": "fanfarronear",
    "fanfarroneéis": "fanfarronear",
    "fanfarroneó": "fanfarronear",
    "fanfurriñas": "fanfurriña",
    "fangales": "fangal",
    "fangares": "fangar",
    "fangos": "fango",
    "fangosa": "fangoso",
    "fangosas": "fangoso",
    "fangosos": "fangoso",
    "fanotrones": "fanotrón",
    "fans": "fan",
    "fantasea": "fantasear",
    "fantaseaba": "fantasear",
    "fantaseabais": "fantasear",
    "fantaseaban": "fantasear",
    "fantaseabas": "fantasear",
    "fantasead": "fantasear",
    "fantaseada": "fantasear",
    "fantaseadas": "fantasear",
    "fantaseado": "fantasear",
    "fantaseadora": "fantaseador",
    "fantaseadoras": "fantaseador",
    "fantaseadores": "fantaseador",
    "fantaseados": "fantasear",
    "fantaseamos": "fantasear",
    "fantasean": "fantasear",
    "fantaseando": "fantasear",
    "fantaseara": "fantasear",
    "fantasearais": "fantasear",
    "fantasearan": "fantasear",
    "fantasearas": "fantasear",
    "fantaseare": "fantasear",
    "fantaseareis": "fantasear",
    "fantasearemos": "fantasear",
    "fantasearen": "fantasear",
    "fantaseares": "fantasear",
    "fantasearon": "fantasear",
    "fantaseará": "fantasear",
    "fantasearán": "fantasear",
    "fantasearás": "fantasear",
    "fantasearé": "fantasear",
    "fantasearéis": "fantasear",
    "fantasearía": "fantasear",
    "fantasearíais": "fantasear",
    "fantasearíamos": "fantasear",
    "fantasearían": "fantasear",
    "fantasearías": "fantasear",
    "fantaseas": "fantasear",
    "fantasease": "fantasear",
    "fantaseaseis": "fantasear",
    "fantaseasen": "fantasear",
    "fantaseases": "fantasear",
    "fantaseaste": "fantasear",
    "fantaseasteis": "fantasear",
    "fantasee": "fantasear",
    "fantaseemos": "fantasear",
    "fantaseen": "fantasear",
    "fantasees": "fantasear",
    "fantaseo": "fantasear",
    "fantaseábamos": "fantasear",
    "fantaseáis": "fantasear",
    "fantaseáramos": "fantasear",
    "fantaseáremos": "fantasear",
    "fantaseásemos": "fantasear",
    "fantaseé": "fantasear",
    "fantaseéis": "fantasear",
    "fantaseó": "fantasear",
    "fantasiosa": "fantasioso",
    "fantasiosas": "fantasioso",
    "fantasiosos": "fantasioso",
    "fantasmagorías": "fantasmagoría",
    "fantasmagórica": "fantasmagórico",
    "fantasmagóricas": "fantasmagórico",
    "fantasmagóricos": "fantasmagórico",
    "fantasmales": "fantasmal",
    "fantasmas": "fantasma",
    "fantasmona": "fantasmón",
    "fantasmonas": "fantasmón",
    "fantasmones": "fantasmón",
    "fantasías": "fantasía",
    "fantochadas": "fantochada",
    "fantoches": "fantoche",
    "fantástica": "fantástico",
    "fantásticas": "fantástico",
    "fantásticos": "fantástico",
    "fanática": "fanático",
    "fanáticas": "fanático",
    "fanáticos": "fanático",
    "fanés": "fané",
    "faquines": "faquín",
    "faquires": "faquir",
    "faquirismos": "faquirismo",
    "faquíes": "faquí",
    "faquís": "faquí",
    "faraday": "faradio",
    "faradayes": "faraday",
    "faradays": "faradio",
    "faradios": "faradio",
    "farads": "farad",
    "faradímetros": "faradímetro",
    "faralaes": "faralá",
    "farallones": "farallón",
    "faramallas": "faramalla",
    "faramallea": "faramallear",
    "faramalleaba": "faramallear",
    "faramalleabais": "faramallear",
    "faramalleaban": "faramallear",
    "faramalleabas": "faramallear",
    "faramallead": "faramallear",
    "faramalleada": "faramallear",
    "faramalleadas": "faramallear",
    "faramalleado": "faramallear",
    "faramalleados": "faramallear",
    "faramalleamos": "faramallear",
    "faramallean": "faramallear",
    "faramalleando": "faramallear",
    "faramalleara": "faramallear",
    "faramallearais": "faramallear",
    "faramallearan": "faramallear",
    "faramallearas": "faramallear",
    "faramalleare": "faramallear",
    "faramalleareis": "faramallear",
    "faramallearemos": "faramallear",
    "faramallearen": "faramallear",
    "faramalleares": "faramallear",
    "faramallearon": "faramallear",
    "faramalleará": "faramallear",
    "faramallearán": "faramallear",
    "faramallearás": "faramallear",
    "faramallearé": "faramallear",
    "faramallearéis": "faramallear",
    "faramallearía": "faramallear",
    "faramallearíais": "faramallear",
    "faramallearíamos": "faramallear",
    "faramallearían": "faramallear",
    "faramallearías": "faramallear",
    "faramalleas": "faramallear",
    "faramallease": "faramallear",
    "faramalleaseis": "faramallear",
    "faramalleasen": "faramallear",
    "faramalleases": "faramallear",
    "faramalleaste": "faramallear",
    "faramalleasteis": "faramallear",
    "faramallee": "faramallear",
    "faramalleemos": "faramallear",
    "faramalleen": "faramallear",
    "faramallees": "faramallear",
    "faramalleo": "faramallear",
    "faramallera": "faramallero",
    "faramalleras": "faramallero",
    "faramalleros": "faramallero",
    "faramalleábamos": "faramallear",
    "faramalleáis": "faramallear",
    "faramalleáramos": "faramallear",
    "faramalleáremos": "faramallear",
    "faramalleásemos": "faramallear",
    "faramalleé": "faramallear",
    "faramalleéis": "faramallear",
    "faramalleó": "faramallear",
    "faramallona": "faramallón",
    "faramallonas": "faramallón",
    "faramallones": "faramallón",
    "farandulea": "farandulear",
    "faranduleaba": "farandulear",
    "faranduleabais": "farandulear",
    "faranduleaban": "farandulear",
    "faranduleabas": "farandulear",
    "farandulead": "farandulear",
    "faranduleada": "farandulear",
    "faranduleadas": "farandulear",
    "faranduleado": "farandulear",
    "faranduleados": "farandulear",
    "faranduleamos": "farandulear",
    "farandulean": "farandulear",
    "faranduleando": "farandulear",
    "faranduleara": "farandulear",
    "farandulearais": "farandulear",
    "farandulearan": "farandulear",
    "farandulearas": "farandulear",
    "faranduleare": "farandulear",
    "faranduleareis": "farandulear",
    "farandulearemos": "farandulear",
    "farandulearen": "farandulear",
    "faranduleares": "farandulear",
    "farandulearon": "farandulear",
    "faranduleará": "farandulear",
    "farandulearán": "farandulear",
    "farandulearás": "farandulear",
    "farandulearé": "farandulear",
    "farandulearéis": "farandulear",
    "farandulearía": "farandulear",
    "farandulearíais": "farandulear",
    "farandulearíamos": "farandulear",
    "farandulearían": "farandulear",
    "farandulearías": "farandulear",
    "faranduleas": "farandulear",
    "farandulease": "farandulear",
    "faranduleaseis": "farandulear",
    "faranduleasen": "farandulear",
    "faranduleases": "farandulear",
    "faranduleaste": "farandulear",
    "faranduleasteis": "farandulear",
    "farandulee": "farandulear",
    "faranduleemos": "farandulear",
    "faranduleen": "farandulear",
    "farandulees": "farandulear",
    "faranduleo": "farandulear",
    "farandulera": "farandulero",
    "faranduleras": "farandulero",
    "faranduleros": "farandulero",
    "faranduleábamos": "farandulear",
    "faranduleáis": "farandulear",
    "faranduleáramos": "farandulear",
    "faranduleáremos": "farandulear",
    "faranduleásemos": "farandulear",
    "faranduleé": "farandulear",
    "faranduleéis": "farandulear",
    "faranduleó": "farandulear",
    "farandúlica": "farandúlico",
    "farandúlicas": "farandúlico",
    "farandúlicos": "farandúlico",
    "faraona": "faraón",
    "faraonas": "faraón",
    "faraones": "faraón",
    "faras": "fara",
    "farautes": "faraute",
    "faraónica": "faraónico",
    "faraónicas": "faraónico",
    "faraónicos": "faraónico",
    "farda": "fardar",
    "fardaba": "fardar",
    "fardabais": "fardar",
    "fardaban": "fardar",
    "fardabas": "fardar",
    "fardad": "fardar",
    "fardada": "fardar",
    "fardadas": "fardar",
    "fardado": "fardar",
    "fardados": "fardar",
    "fardajes": "fardaje",
    "fardamos": "fardar",
    "fardan": "fardar",
    "fardando": "fardar",
    "fardara": "fardar",
    "fardarais": "fardar",
    "fardaran": "fardar",
    "fardaras": "fardar",
    "fardare": "fardar",
    "fardareis": "fardar",
    "fardaremos": "fardar",
    "fardaren": "fardar",
    "fardares": "fardar",
    "fardaron": "fardar",
    "fardará": "fardar",
    "fardarán": "fardar",
    "fardarás": "fardar",
    "fardaré": "fardar",
    "fardaréis": "fardar",
    "fardaría": "fardar",
    "fardaríais": "fardar",
    "fardaríamos": "fardar",
    "fardarían": "fardar",
    "fardarías": "fardar",
    "fardas": "fardar",
    "fardase": "fardar",
    "fardaseis": "fardar",
    "fardasen": "fardar",
    "fardases": "fardar",
    "fardaste": "fardar",
    "fardasteis": "fardar",
    "farde": "fardar",
    "fardeles": "fardel",
    "fardemos": "fardar",
    "farden": "fardar",
    "farderías": "fardería",
    "fardes": "farde",
    "fardo": "fardar",
    "fardona": "fardón",
    "fardonas": "fardón",
    "fardones": "fardón",
    "fardos": "fardo",
    "fardábamos": "fardar",
    "fardáis": "fardar",
    "fardáramos": "fardar",
    "fardáremos": "fardar",
    "fardásemos": "fardar",
    "fardé": "fardar",
    "fardéis": "fardar",
    "fardó": "fardar",
    "farellones": "farellón",
    "farera": "farero",
    "fareras": "farero",
    "fareros": "farero",
    "farfalaes": "farfalá",
    "farfallona": "farfallón",
    "farfallonas": "farfallón",
    "farfallones": "farfallón",
    "farfanes": "farfán",
    "farfantes": "farfante",
    "farfantonadas": "farfantonada",
    "farfantonerías": "farfantonería",
    "farfantones": "farfantón",
    "farfollas": "farfolla",
    "farfulla": "farfullar",
    "farfullaba": "farfullar",
    "farfullabais": "farfullar",
    "farfullaban": "farfullar",
    "farfullabas": "farfullar",
    "farfullad": "farfullar",
    "farfullada": "farfullar",
    "farfulladas": "farfullar",
    "farfullado": "farfullar",
    "farfulladora": "farfullador",
    "farfulladoras": "farfullador",
    "farfulladores": "farfullador",
    "farfullados": "farfullar",
    "farfullamos": "farfullar",
    "farfullan": "farfullar",
    "farfullando": "farfullar",
    "farfullara": "farfullar",
    "farfullarais": "farfullar",
    "farfullaran": "farfullar",
    "farfullaras": "farfullar",
    "farfullare": "farfullar",
    "farfullareis": "farfullar",
    "farfullaremos": "farfullar",
    "farfullaren": "farfullar",
    "farfullares": "farfullar",
    "farfullaron": "farfullar",
    "farfullará": "farfullar",
    "farfullarán": "farfullar",
    "farfullarás": "farfullar",
    "farfullaré": "farfullar",
    "farfullaréis": "farfullar",
    "farfullaría": "farfullar",
    "farfullaríais": "farfullar",
    "farfullaríamos": "farfullar",
    "farfullarían": "farfullar",
    "farfullarías": "farfullar",
    "farfullas": "farfullar",
    "farfullase": "farfullar",
    "farfullaseis": "farfullar",
    "farfullasen": "farfullar",
    "farfullases": "farfullar",
    "farfullaste": "farfullar",
    "farfullasteis": "farfullar",
    "farfulle": "farfullar",
    "farfullemos": "farfullar",
    "farfullen": "farfullar",
    "farfullera": "farfullero",
    "farfulleras": "farfullero",
    "farfulleros": "farfullero",
    "farfulles": "farfullar",
    "farfullo": "farfullar",
    "farfullábamos": "farfullar",
    "farfulláis": "farfullar",
    "farfulláramos": "farfullar",
    "farfulláremos": "farfullar",
    "farfullásemos": "farfullar",
    "farfullé": "farfullar",
    "farfulléis": "farfullar",
    "farfulló": "farfullar",
    "fargallona": "fargallón",
    "fargallonas": "fargallón",
    "fargallones": "fargallón",
    "farillones": "farillón",
    "faringalgias": "faringalgia",
    "faringectomías": "faringectomía",
    "faringes": "faringe",
    "faringobasilares": "faringobasilar",
    "faringologías": "faringología",
    "faringopalatina": "faringopalatino",
    "faringopalatinas": "faringopalatino",
    "faringopalatinos": "faringopalatino",
    "faringopatías": "faringopatía",
    "faringoperístoles": "faringoperístole",
    "faringoplastias": "faringoplastia",
    "faringorragias": "faringorragia",
    "faringotomías": "faringotomía",
    "faringítica": "faringítico",
    "faringíticas": "faringítico",
    "faringíticos": "faringítico",
    "farinácea": "farináceo",
    "farináceas": "farináceo",
    "farináceos": "farináceo",
    "farisaica": "farisaico",
    "farisaicas": "farisaico",
    "farisaicos": "farisaico",
    "farisea": "fariseo",
    "fariseas": "fariseo",
    "fariseos": "fariseo",
    "fariseísmos": "fariseísmo",
    "farmacias": "farmacia",
    "farmacodinámicas": "farmacodinámica",
    "farmacolitas": "farmacolita",
    "farmacologías": "farmacología",
    "farmacológica": "farmacológico",
    "farmacológicas": "farmacológico",
    "farmacológicos": "farmacológico",
    "farmacopeas": "farmacopea",
    "farmacotecnias": "farmacotecnia",
    "farmacoterapias": "farmacoterapia",
    "farmacéutica": "farmacéutico",
    "farmacéuticas": "farmacéutico",
    "farmacéuticos": "farmacéutico",
    "farmacóloga": "farmacólogo",
    "farmacólogas": "farmacólogo",
    "farmacólogos": "farmacólogo",
    "farnesoles": "farnesol",
    "farolas": "farola",
    "farolazos": "farolazo",
    "farolea": "farolear",
    "faroleaba": "farolear",
    "faroleabais": "farolear",
    "faroleaban": "farolear",
    "faroleabas": "farolear",
    "farolead": "farolear",
    "faroleada": "farolear",
    "faroleadas": "farolear",
    "faroleado": "farolear",
    "faroleados": "farolear",
    "faroleamos": "farolear",
    "farolean": "farolear",
    "faroleando": "farolear",
    "faroleara": "farolear",
    "farolearais": "farolear",
    "farolearan": "farolear",
    "farolearas": "farolear",
    "faroleare": "farolear",
    "faroleareis": "farolear",
    "farolearemos": "farolear",
    "farolearen": "farolear",
    "faroleares": "farolear",
    "farolearon": "farolear",
    "faroleará": "farolear",
    "farolearán": "farolear",
    "farolearás": "farolear",
    "farolearé": "farolear",
    "farolearéis": "farolear",
    "farolearía": "farolear",
    "farolearíais": "farolear",
    "farolearíamos": "farolear",
    "farolearían": "farolear",
    "farolearías": "farolear",
    "faroleas": "farolear",
    "farolease": "farolear",
    "faroleaseis": "farolear",
    "faroleasen": "farolear",
    "faroleases": "farolear",
    "faroleaste": "farolear",
    "faroleasteis": "farolear",
    "farolee": "farolear",
    "faroleemos": "farolear",
    "faroleen": "farolear",
    "farolees": "farolear",
    "faroleo": "farolear",
    "faroleos": "faroleo",
    "farolera": "farolero",
    "faroleras": "farolero",
    "faroleros": "farolero",
    "farolerías": "farolería",
    "faroles": "farol",
    "faroleábamos": "farolear",
    "faroleáis": "farolear",
    "faroleáramos": "farolear",
    "faroleáremos": "farolear",
    "faroleásemos": "farolear",
    "faroleé": "farolear",
    "faroleéis": "farolear",
    "faroleó": "farolear",
    "farolillos": "farolillo",
    "farolona": "farolón",
    "farolonas": "farolón",
    "farolones": "farolón",
    "faros": "faro",
    "farotona": "farotón",
    "farotonas": "farotón",
    "farotones": "farotón",
    "farpada": "farpado",
    "farpadas": "farpado",
    "farpados": "farpado",
    "farpas": "farpa",
    "farragosa": "farragoso",
    "farragosas": "farragoso",
    "farragosos": "farragoso",
    "farras": "farra",
    "farrea": "farrear",
    "farreaba": "farrear",
    "farreabais": "farrear",
    "farreaban": "farrear",
    "farreabas": "farrear",
    "farread": "farrear",
    "farreada": "farrear",
    "farreadas": "farrear",
    "farreado": "farrear",
    "farreados": "farrear",
    "farreamos": "farrear",
    "farrean": "farrear",
    "farreando": "farrear",
    "farreara": "farrear",
    "farrearais": "farrear",
    "farrearan": "farrear",
    "farrearas": "farrear",
    "farreare": "farrear",
    "farreareis": "farrear",
    "farrearemos": "farrear",
    "farrearen": "farrear",
    "farreares": "farrear",
    "farrearon": "farrear",
    "farreará": "farrear",
    "farrearán": "farrear",
    "farrearás": "farrear",
    "farrearé": "farrear",
    "farrearéis": "farrear",
    "farrearía": "farrear",
    "farrearíais": "farrear",
    "farrearíamos": "farrear",
    "farrearían": "farrear",
    "farrearías": "farrear",
    "farreas": "farrear",
    "farrease": "farrear",
    "farreaseis": "farrear",
    "farreasen": "farrear",
    "farreases": "farrear",
    "farreaste": "farrear",
    "farreasteis": "farrear",
    "farree": "farrear",
    "farreemos": "farrear",
    "farreen": "farrear",
    "farrees": "farrear",
    "farreo": "farrear",
    "farreábamos": "farrear",
    "farreáis": "farrear",
    "farreáramos": "farrear",
    "farreáremos": "farrear",
    "farreásemos": "farrear",
    "farreé": "farrear",
    "farreéis": "farrear",
    "farreó": "farrear",
    "farristas": "farrista",
    "farros": "farro",
    "farruca": "farruco",
    "farrucas": "farruco",
    "farrucos": "farruco",
    "farsanta": "farsante",
    "farsantas": "farsante",
    "farsanterías": "farsantería",
    "farsantes": "farsante",
    "farsas": "farsa",
    "farsetos": "farseto",
    "farádica": "farádico",
    "farádicas": "farádico",
    "farádicos": "farádico",
    "farándulas": "farándula",
    "faríngea": "faríngeo",
    "faríngeas": "faríngeo",
    "faríngeos": "faríngeo",
    "fas": "fa",
    "fascias": "fascia",
    "fasciculada": "fasciculado",
    "fasciculadas": "fasciculado",
    "fasciculados": "fasciculado",
    "fascina": "fascinar",
    "fascinaba": "fascinar",
    "fascinabais": "fascinar",
    "fascinaban": "fascinar",
    "fascinabas": "fascinar",
    "fascinaciones": "fascinación",
    "fascinad": "fascinar",
    "fascinada": "fascinar",
    "fascinadas": "fascinar",
    "fascinado": "fascinar",
    "fascinadora": "fascinador",
    "fascinadoras": "fascinador",
    "fascinadores": "fascinador",
    "fascinados": "fascinar",
    "fascinamos": "fascinar",
    "fascinan": "fascinar",
    "fascinando": "fascinar",
    "fascinantes": "fascinante",
    "fascinara": "fascinar",
    "fascinarais": "fascinar",
    "fascinaran": "fascinar",
    "fascinaras": "fascinar",
    "fascinare": "fascinar",
    "fascinareis": "fascinar",
    "fascinaremos": "fascinar",
    "fascinaren": "fascinar",
    "fascinares": "fascinar",
    "fascinaron": "fascinar",
    "fascinará": "fascinar",
    "fascinarán": "fascinar",
    "fascinarás": "fascinar",
    "fascinaré": "fascinar",
    "fascinaréis": "fascinar",
    "fascinaría": "fascinar",
    "fascinaríais": "fascinar",
    "fascinaríamos": "fascinar",
    "fascinarían": "fascinar",
    "fascinarías": "fascinar",
    "fascinas": "fascinar",
    "fascinase": "fascinar",
    "fascinaseis": "fascinar",
    "fascinasen": "fascinar",
    "fascinases": "fascinar",
    "fascinaste": "fascinar",
    "fascinasteis": "fascinar",
    "fascine": "fascinar",
    "fascinemos": "fascinar",
    "fascinen": "fascinar",
    "fascines": "fascinar",
    "fascino": "fascinar",
    "fascinábamos": "fascinar",
    "fascináis": "fascinar",
    "fascináramos": "fascinar",
    "fascináremos": "fascinar",
    "fascinásemos": "fascinar",
    "fasciné": "fascinar",
    "fascinéis": "fascinar",
    "fascinó": "fascinar",
    "fascismos": "fascismo",
    "fascistas": "fascista",
    "fascistoides": "fascistoide",
    "fascogalos": "fascogalo",
    "fascículos": "fascículo",
    "fases": "fase",
    "fasitrones": "fasitrón",
    "fasiánidos": "fasiánido",
    "fasta": "fasto",
    "fastas": "fasto",
    "fastiales": "fastial",
    "fastidia": "fastidiar",
    "fastidiaba": "fastidiar",
    "fastidiabais": "fastidiar",
    "fastidiaban": "fastidiar",
    "fastidiabas": "fastidiar",
    "fastidiad": "fastidiar",
    "fastidiada": "fastidiar",
    "fastidiadas": "fastidiar",
    "fastidiado": "fastidiar",
    "fastidiados": "fastidiar",
    "fastidiamos": "fastidiar",
    "fastidian": "fastidiar",
    "fastidiando": "fastidiar",
    "fastidiara": "fastidiar",
    "fastidiarais": "fastidiar",
    "fastidiaran": "fastidiar",
    "fastidiaras": "fastidiar",
    "fastidiare": "fastidiar",
    "fastidiareis": "fastidiar",
    "fastidiaremos": "fastidiar",
    "fastidiaren": "fastidiar",
    "fastidiares": "fastidiar",
    "fastidiaron": "fastidiar",
    "fastidiará": "fastidiar",
    "fastidiarán": "fastidiar",
    "fastidiarás": "fastidiar",
    "fastidiaré": "fastidiar",
    "fastidiaréis": "fastidiar",
    "fastidiaría": "fastidiar",
    "fastidiaríais": "fastidiar",
    "fastidiaríamos": "fastidiar",
    "fastidiarían": "fastidiar",
    "fastidiarías": "fastidiar",
    "fastidias": "fastidiar",
    "fastidiase": "fastidiar",
    "fastidiaseis": "fastidiar",
    "fastidiasen": "fastidiar",
    "fastidiases": "fastidiar",
    "fastidiaste": "fastidiar",
    "fastidiasteis": "fastidiar",
    "fastidie": "fastidiar",
    "fastidiemos": "fastidiar",
    "fastidien": "fastidiar",
    "fastidies": "fastidiar",
    "fastidio": "fastidiar",
    "fastidios": "fastidio",
    "fastidiosa": "fastidioso",
    "fastidiosas": "fastidioso",
    "fastidiosos": "fastidioso",
    "fastidiábamos": "fastidiar",
    "fastidiáis": "fastidiar",
    "fastidiáramos": "fastidiar",
    "fastidiáremos": "fastidiar",
    "fastidiásemos": "fastidiar",
    "fastidié": "fastidiar",
    "fastidiéis": "fastidiar",
    "fastidió": "fastidiar",
    "fastigios": "fastigio",
    "fastos": "fasto",
    "fastuosa": "fastuoso",
    "fastuosas": "fastuoso",
    "fastuosidades": "fastuosidad",
    "fastuosos": "fastuoso",
    "fasímetros": "fasímetro",
    "fatales": "fatal",
    "fatalidades": "fatalidad",
    "fatalismos": "fatalismo",
    "fatalistas": "fatalista",
    "fatiga": "fatigar",
    "fatigaba": "fatigar",
    "fatigabais": "fatigar",
    "fatigaban": "fatigar",
    "fatigabas": "fatigar",
    "fatigad": "fatigar",
    "fatigada": "fatigar",
    "fatigadas": "fatigar",
    "fatigado": "fatigar",
    "fatigados": "fatigar",
    "fatigamos": "fatigar",
    "fatigan": "fatigar",
    "fatigando": "fatigar",
    "fatigara": "fatigar",
    "fatigarais": "fatigar",
    "fatigaran": "fatigar",
    "fatigaras": "fatigar",
    "fatigare": "fatigar",
    "fatigareis": "fatigar",
    "fatigaremos": "fatigar",
    "fatigaren": "fatigar",
    "fatigares": "fatigar",
    "fatigaron": "fatigar",
    "fatigará": "fatigar",
    "fatigarán": "fatigar",
    "fatigarás": "fatigar",
    "fatigaré": "fatigar",
    "fatigaréis": "fatigar",
    "fatigaría": "fatigar",
    "fatigaríais": "fatigar",
    "fatigaríamos": "fatigar",
    "fatigarían": "fatigar",
    "fatigarías": "fatigar",
    "fatigas": "fatigar",
    "fatigase": "fatigar",
    "fatigaseis": "fatigar",
    "fatigasen": "fatigar",
    "fatigases": "fatigar",
    "fatigaste": "fatigar",
    "fatigasteis": "fatigar",
    "fatigo": "fatigar",
    "fatigosa": "fatigoso",
    "fatigosas": "fatigoso",
    "fatigosos": "fatigoso",
    "fatigue": "fatigar",
    "fatiguemos": "fatigar",
    "fatiguen": "fatigar",
    "fatigues": "fatigar",
    "fatigué": "fatigar",
    "fatiguéis": "fatigar",
    "fatigábamos": "fatigar",
    "fatigáis": "fatigar",
    "fatigáramos": "fatigar",
    "fatigáremos": "fatigar",
    "fatigásemos": "fatigar",
    "fatigó": "fatigar",
    "fatimíes": "fatimí",
    "fatimís": "fatimí",
    "fatua": "fatuo",
    "fatuas": "fatuo",
    "fatuidades": "fatuidad",
    "fatuos": "fatuo",
    "fatídica": "fatídico",
    "fatídicas": "fatídico",
    "fatídicos": "fatídico",
    "faucales": "faucal",
    "faunas": "fauna",
    "faunesca": "faunesco",
    "faunescas": "faunesco",
    "faunescos": "faunesco",
    "faunos": "fauno",
    "faunística": "faunístico",
    "faunísticas": "faunístico",
    "faunísticos": "faunístico",
    "fausta": "fausto",
    "faustas": "fausto",
    "faustos": "fausto",
    "fautora": "fautor",
    "fautoras": "fautor",
    "fautores": "fautor",
    "fautorías": "fautoría",
    "fauves": "fauve",
    "fauvismos": "fauvismo",
    "fauvistas": "fauvista",
    "favelas": "favela",
    "favismos": "favismo",
    "favorables": "favorable",
    "favorece": "favorecer",
    "favoreced": "favorecer",
    "favorecedora": "favorecedor",
    "favorecedoras": "favorecedor",
    "favorecedores": "favorecedor",
    "favorecemos": "favorecer",
    "favorecen": "favorecer",
    "favoreceremos": "favorecer",
    "favorecerá": "favorecer",
    "favorecerán": "favorecer",
    "favorecerás": "favorecer",
    "favoreceré": "favorecer",
    "favoreceréis": "favorecer",
    "favorecería": "favorecer",
    "favoreceríais": "favorecer",
    "favoreceríamos": "favorecer",
    "favorecerían": "favorecer",
    "favorecerías": "favorecer",
    "favoreces": "favorecer",
    "favorecida": "favorecer",
    "favorecidas": "favorecer",
    "favorecido": "favorecer",
    "favorecidos": "favorecer",
    "favoreciendo": "favorecer",
    "favoreciera": "favorecer",
    "favorecierais": "favorecer",
    "favorecieran": "favorecer",
    "favorecieras": "favorecer",
    "favoreciere": "favorecer",
    "favoreciereis": "favorecer",
    "favorecieren": "favorecer",
    "favorecieres": "favorecer",
    "favorecieron": "favorecer",
    "favoreciese": "favorecer",
    "favorecieseis": "favorecer",
    "favoreciesen": "favorecer",
    "favorecieses": "favorecer",
    "favorecimientos": "favorecimiento",
    "favorecimos": "favorecer",
    "favoreciste": "favorecer",
    "favorecisteis": "favorecer",
    "favoreciéramos": "favorecer",
    "favoreciéremos": "favorecer",
    "favoreciésemos": "favorecer",
    "favoreció": "favorecer",
    "favorecéis": "favorecer",
    "favorecí": "favorecer",
    "favorecía": "favorecer",
    "favorecíais": "favorecer",
    "favorecíamos": "favorecer",
    "favorecían": "favorecer",
    "favorecías": "favorecer",
    "favores": "favor",
    "favorezca": "favorecer",
    "favorezcamos": "favorecer",
    "favorezcan": "favorecer",
    "favorezcas": "favorecer",
    "favorezco": "favorecer",
    "favorezcáis": "favorecer",
    "favorita": "favorito",
    "favoritas": "favorito",
    "favoritismos": "favoritismo",
    "favoritos": "favorito",
    "favos": "favo",
    "faxes": "fax",
    "fayalitas": "fayalita",
    "fayancas": "fayanca",
    "fayas": "faya",
    "fayenzas": "fayenza",
    "fazendas": "fazenda",
    "fañada": "fañado",
    "fañadas": "fañado",
    "fañados": "fañado",
    "fea": "feo",
    "feacia": "feacio",
    "feacias": "feacio",
    "feacios": "feacio",
    "fealdades": "fealdad",
    "feas": "feo",
    "feb.": "febrero",
    "febea": "febeo",
    "febeas": "febeo",
    "febeos": "febeo",
    "febles": "feble",
    "febreras": "febrera",
    "febreros": "febrero",
    "febricitantes": "febricitante",
    "febriles": "febril",
    "febroniana": "febroniano",
    "febronianas": "febroniano",
    "febronianismos": "febronianismo",
    "febronianos": "febroniano",
    "febrículas": "febrícula",
    "febrífuga": "febrífugo",
    "febrífugas": "febrífugo",
    "febrífugos": "febrífugo",
    "fecales": "fecal",
    "fecha": "fecho",
    "fechaba": "fechar",
    "fechabais": "fechar",
    "fechaban": "fechar",
    "fechabas": "fechar",
    "fechad": "fechar",
    "fechada": "fechar",
    "fechadas": "fechar",
    "fechado": "fechar",
    "fechadores": "fechador",
    "fechados": "fechar",
    "fechamos": "fechar",
    "fechan": "fechar",
    "fechando": "fechar",
    "fechara": "fechar",
    "fecharais": "fechar",
    "fecharan": "fechar",
    "fecharas": "fechar",
    "fechare": "fechar",
    "fechareis": "fechar",
    "fecharemos": "fechar",
    "fecharen": "fechar",
    "fechares": "fechar",
    "fecharon": "fechar",
    "fechará": "fechar",
    "fecharán": "fechar",
    "fecharás": "fechar",
    "fecharé": "fechar",
    "fecharéis": "fechar",
    "fecharía": "fechar",
    "fecharíais": "fechar",
    "fecharíamos": "fechar",
    "fecharían": "fechar",
    "fecharías": "fechar",
    "fechas": "fecho",
    "fechase": "fechar",
    "fechaseis": "fechar",
    "fechasen": "fechar",
    "fechases": "fechar",
    "fechaste": "fechar",
    "fechasteis": "fechar",
    "feche": "fechar",
    "fechemos": "fechar",
    "fechen": "fechar",
    "feches": "fechar",
    "fecho": "fechar",
    "fechorías": "fechoría",
    "fechos": "fecho",
    "fechábamos": "fechar",
    "fecháis": "fechar",
    "fecháramos": "fechar",
    "fecháremos": "fechar",
    "fechásemos": "fechar",
    "feché": "fechar",
    "fechéis": "fechar",
    "fechó": "fechar",
    "feciales": "fecial",
    "feculenta": "feculento",
    "feculentas": "feculento",
    "feculentos": "feculento",
    "fecunda": "fecundo",
    "fecundaba": "fecundar",
    "fecundabais": "fecundar",
    "fecundaban": "fecundar",
    "fecundabas": "fecundar",
    "fecundables": "fecundable",
    "fecundaciones": "fecundación",
    "fecundad": "fecundar",
    "fecundada": "fecundar",
    "fecundadas": "fecundar",
    "fecundado": "fecundar",
    "fecundadora": "fecundador",
    "fecundadoras": "fecundador",
    "fecundadores": "fecundador",
    "fecundados": "fecundar",
    "fecundamos": "fecundar",
    "fecundan": "fecundar",
    "fecundando": "fecundar",
    "fecundara": "fecundar",
    "fecundarais": "fecundar",
    "fecundaran": "fecundar",
    "fecundaras": "fecundar",
    "fecundare": "fecundar",
    "fecundareis": "fecundar",
    "fecundaremos": "fecundar",
    "fecundaren": "fecundar",
    "fecundares": "fecundar",
    "fecundaron": "fecundar",
    "fecundará": "fecundar",
    "fecundarán": "fecundar",
    "fecundarás": "fecundar",
    "fecundaré": "fecundar",
    "fecundaréis": "fecundar",
    "fecundaría": "fecundar",
    "fecundaríais": "fecundar",
    "fecundaríamos": "fecundar",
    "fecundarían": "fecundar",
    "fecundarías": "fecundar",
    "fecundas": "fecundo",
    "fecundase": "fecundar",
    "fecundaseis": "fecundar",
    "fecundasen": "fecundar",
    "fecundases": "fecundar",
    "fecundaste": "fecundar",
    "fecundasteis": "fecundar",
    "fecundativa": "fecundativo",
    "fecundativas": "fecundativo",
    "fecundativos": "fecundativo",
    "fecunde": "fecundar",
    "fecundemos": "fecundar",
    "fecunden": "fecundar",
    "fecundes": "fecundar",
    "fecundice": "fecundizar",
    "fecundicemos": "fecundizar",
    "fecundicen": "fecundizar",
    "fecundices": "fecundizar",
    "fecundicé": "fecundizar",
    "fecundicéis": "fecundizar",
    "fecundidades": "fecundidad",
    "fecundiza": "fecundizar",
    "fecundizaba": "fecundizar",
    "fecundizabais": "fecundizar",
    "fecundizaban": "fecundizar",
    "fecundizabas": "fecundizar",
    "fecundizaciones": "fecundización",
    "fecundizad": "fecundizar",
    "fecundizada": "fecundizar",
    "fecundizadas": "fecundizar",
    "fecundizado": "fecundizar",
    "fecundizados": "fecundizar",
    "fecundizamos": "fecundizar",
    "fecundizan": "fecundizar",
    "fecundizando": "fecundizar",
    "fecundizara": "fecundizar",
    "fecundizarais": "fecundizar",
    "fecundizaran": "fecundizar",
    "fecundizaras": "fecundizar",
    "fecundizare": "fecundizar",
    "fecundizareis": "fecundizar",
    "fecundizaremos": "fecundizar",
    "fecundizaren": "fecundizar",
    "fecundizares": "fecundizar",
    "fecundizaron": "fecundizar",
    "fecundizará": "fecundizar",
    "fecundizarán": "fecundizar",
    "fecundizarás": "fecundizar",
    "fecundizaré": "fecundizar",
    "fecundizaréis": "fecundizar",
    "fecundizaría": "fecundizar",
    "fecundizaríais": "fecundizar",
    "fecundizaríamos": "fecundizar",
    "fecundizarían": "fecundizar",
    "fecundizarías": "fecundizar",
    "fecundizas": "fecundizar",
    "fecundizase": "fecundizar",
    "fecundizaseis": "fecundizar",
    "fecundizasen": "fecundizar",
    "fecundizases": "fecundizar",
    "fecundizaste": "fecundizar",
    "fecundizasteis": "fecundizar",
    "fecundizo": "fecundizar",
    "fecundizábamos": "fecundizar",
    "fecundizáis": "fecundizar",
    "fecundizáramos": "fecundizar",
    "fecundizáremos": "fecundizar",
    "fecundizásemos": "fecundizar",
    "fecundizó": "fecundizar",
    "fecundo": "fecundar",
    "fecundos": "fecundo",
    "fecundábamos": "fecundar",
    "fecundáis": "fecundar",
    "fecundáramos": "fecundar",
    "fecundáremos": "fecundar",
    "fecundásemos": "fecundar",
    "fecundé": "fecundar",
    "fecundéis": "fecundar",
    "fecundó": "fecundar",
    "fedatarios": "fedatario",
    "federa": "federar",
    "federaba": "federar",
    "federabais": "federar",
    "federaban": "federar",
    "federabas": "federar",
    "federaciones": "federación",
    "federad": "federar",
    "federada": "federar",
    "federadas": "federar",
    "federado": "federar",
    "federados": "federar",
    "federales": "federal",
    "federalidades": "federalidad",
    "federalismos": "federalismo",
    "federalistas": "federalista",
    "federamos": "federar",
    "federan": "federar",
    "federando": "federar",
    "federara": "federar",
    "federarais": "federar",
    "federaran": "federar",
    "federaras": "federar",
    "federare": "federar",
    "federareis": "federar",
    "federaremos": "federar",
    "federaren": "federar",
    "federares": "federar",
    "federaron": "federar",
    "federará": "federar",
    "federarán": "federar",
    "federarás": "federar",
    "federaré": "federar",
    "federaréis": "federar",
    "federaría": "federar",
    "federaríais": "federar",
    "federaríamos": "federar",
    "federarían": "federar",
    "federarías": "federar",
    "federas": "federar",
    "federase": "federar",
    "federaseis": "federar",
    "federasen": "federar",
    "federases": "federar",
    "federaste": "federar",
    "federasteis": "federar",
    "federativa": "federativo",
    "federativas": "federativo",
    "federativos": "federativo",
    "federe": "federar",
    "federemos": "federar",
    "federen": "federar",
    "federes": "federar",
    "federo": "federar",
    "federábamos": "federar",
    "federáis": "federar",
    "federáramos": "federar",
    "federáremos": "federar",
    "federásemos": "federar",
    "federé": "federar",
    "federéis": "federar",
    "federó": "federar",
    "fedrises": "fedrí",
    "feed-backs": "feed-back",
    "feedbacks": "feedback",
    "feelings": "feeling",
    "fehacientes": "fehaciente",
    "felaciones": "felación",
    "feladices": "feladiz",
    "feldespatoides": "feldespatoide",
    "feldespatos": "feldespato",
    "feldespática": "feldespático",
    "feldespáticas": "feldespático",
    "feldespáticos": "feldespático",
    "felibres": "felibre",
    "felibrismos": "felibrismo",
    "felice": "feliz",
    "felices": "feliz",
    "felicidades": "felicidad",
    "felicita": "felicitar",
    "felicitaba": "felicitar",
    "felicitabais": "felicitar",
    "felicitaban": "felicitar",
    "felicitabas": "felicitar",
    "felicitaciones": "felicitación",
    "felicitad": "felicitar",
    "felicitada": "felicitar",
    "felicitadas": "felicitar",
    "felicitado": "felicitar",
    "felicitados": "felicitar",
    "felicitamos": "felicitar",
    "felicitan": "felicitar",
    "felicitando": "felicitar",
    "felicitara": "felicitar",
    "felicitarais": "felicitar",
    "felicitaran": "felicitar",
    "felicitaras": "felicitar",
    "felicitare": "felicitar",
    "felicitareis": "felicitar",
    "felicitaremos": "felicitar",
    "felicitaren": "felicitar",
    "felicitares": "felicitar",
    "felicitaron": "felicitar",
    "felicitará": "felicitar",
    "felicitarán": "felicitar",
    "felicitarás": "felicitar",
    "felicitaré": "felicitar",
    "felicitaréis": "felicitar",
    "felicitaría": "felicitar",
    "felicitaríais": "felicitar",
    "felicitaríamos": "felicitar",
    "felicitarían": "felicitar",
    "felicitarías": "felicitar",
    "felicitas": "felicitar",
    "felicitase": "felicitar",
    "felicitaseis": "felicitar",
    "felicitasen": "felicitar",
    "felicitases": "felicitar",
    "felicitaste": "felicitar",
    "felicitasteis": "felicitar",
    "felicite": "felicitar",
    "felicitemos": "felicitar",
    "feliciten": "felicitar",
    "felicites": "felicitar",
    "felicito": "felicitar",
    "felicitábamos": "felicitar",
    "felicitáis": "felicitar",
    "felicitáramos": "felicitar",
    "felicitáremos": "felicitar",
    "felicitásemos": "felicitar",
    "felicité": "felicitar",
    "felicitéis": "felicitar",
    "felicitó": "felicitar",
    "feligresa": "feligrés",
    "feligresas": "feligrés",
    "feligreses": "feligrés",
    "feligresías": "feligresía",
    "felina": "felino",
    "felinas": "felino",
    "felinos": "felino",
    "felipismos": "felipismo",
    "felipistas": "felipista",
    "felona": "felón",
    "felonas": "felón",
    "felones": "felón",
    "felonías": "felonía",
    "felpa": "felpar",
    "felpaba": "felpar",
    "felpabais": "felpar",
    "felpaban": "felpar",
    "felpabas": "felpar",
    "felpad": "felpar",
    "felpada": "felpar",
    "felpadas": "felpar",
    "felpado": "felpar",
    "felpados": "felpar",
    "felpamos": "felpar",
    "felpan": "felpar",
    "felpando": "felpar",
    "felpara": "felpar",
    "felparais": "felpar",
    "felparan": "felpar",
    "felparas": "felpar",
    "felpare": "felpar",
    "felpareis": "felpar",
    "felparemos": "felpar",
    "felparen": "felpar",
    "felpares": "felpar",
    "felparon": "felpar",
    "felpará": "felpar",
    "felparán": "felpar",
    "felparás": "felpar",
    "felparé": "felpar",
    "felparéis": "felpar",
    "felparía": "felpar",
    "felparíais": "felpar",
    "felparíamos": "felpar",
    "felparían": "felpar",
    "felparías": "felpar",
    "felpas": "felpar",
    "felpase": "felpar",
    "felpaseis": "felpar",
    "felpasen": "felpar",
    "felpases": "felpar",
    "felpaste": "felpar",
    "felpasteis": "felpar",
    "felpe": "felpar",
    "felpemos": "felpar",
    "felpen": "felpar",
    "felpes": "felpar",
    "felpillas": "felpilla",
    "felpo": "felpar",
    "felpos": "felpo",
    "felposa": "felposo",
    "felposas": "felposo",
    "felposos": "felposo",
    "felpuda": "felpudo",
    "felpudas": "felpudo",
    "felpudos": "felpudo",
    "felpábamos": "felpar",
    "felpáis": "felpar",
    "felpáramos": "felpar",
    "felpáremos": "felpar",
    "felpásemos": "felpar",
    "felpé": "felpar",
    "felpéis": "felpar",
    "felpó": "felpar",
    "felsitas": "felsita",
    "felógenos": "felógeno",
    "femeniles": "femenil",
    "femenina": "femenino",
    "femeninas": "femenino",
    "femeninos": "femenino",
    "fementida": "fementido",
    "fementidas": "fementido",
    "fementidos": "fementido",
    "femineidades": "femineidad",
    "feminidades": "feminidad",
    "feminismos": "feminismo",
    "feministas": "feminista",
    "feminizaciones": "feminización",
    "feminoides": "feminoide",
    "femorales": "femoral",
    "femorocutánea": "femorocutáneo",
    "femorocutáneas": "femorocutáneo",
    "femorocutáneos": "femorocutáneo",
    "femorotibiales": "femorotibial",
    "fenacemidas": "fenacemida",
    "fenacetinas": "fenacetina",
    "fenacinas": "fenacina",
    "fenacitas": "fenacita",
    "fenantrenoquinonas": "fenantrenoquinona",
    "fenantrenos": "fenantreno",
    "fenantrénica": "fenantrénico",
    "fenantrénicas": "fenantrénico",
    "fenantrénicos": "fenantrénico",
    "fenaquitas": "fenaquita",
    "fenatos": "fenato",
    "fenece": "fenecer",
    "feneced": "fenecer",
    "fenecemos": "fenecer",
    "fenecen": "fenecer",
    "feneceremos": "fenecer",
    "fenecerá": "fenecer",
    "fenecerán": "fenecer",
    "fenecerás": "fenecer",
    "feneceré": "fenecer",
    "feneceréis": "fenecer",
    "fenecería": "fenecer",
    "feneceríais": "fenecer",
    "feneceríamos": "fenecer",
    "fenecerían": "fenecer",
    "fenecerías": "fenecer",
    "feneces": "fenecer",
    "fenecida": "fenecer",
    "fenecidas": "fenecer",
    "fenecido": "fenecer",
    "fenecidos": "fenecer",
    "feneciendo": "fenecer",
    "feneciera": "fenecer",
    "fenecierais": "fenecer",
    "fenecieran": "fenecer",
    "fenecieras": "fenecer",
    "feneciere": "fenecer",
    "feneciereis": "fenecer",
    "fenecieren": "fenecer",
    "fenecieres": "fenecer",
    "fenecieron": "fenecer",
    "feneciese": "fenecer",
    "fenecieseis": "fenecer",
    "feneciesen": "fenecer",
    "fenecieses": "fenecer",
    "fenecimientos": "fenecimiento",
    "fenecimos": "fenecer",
    "feneciste": "fenecer",
    "fenecisteis": "fenecer",
    "feneciéramos": "fenecer",
    "feneciéremos": "fenecer",
    "feneciésemos": "fenecer",
    "feneció": "fenecer",
    "fenecs": "fenec",
    "fenecéis": "fenecer",
    "fenecí": "fenecer",
    "fenecía": "fenecer",
    "fenecíais": "fenecer",
    "fenecíamos": "fenecer",
    "fenecían": "fenecer",
    "fenecías": "fenecer",
    "fenes": "fen",
    "fenestraciones": "fenestración",
    "fenestrada": "fenestrado",
    "fenestradas": "fenestrado",
    "fenestrados": "fenestrado",
    "fenetidinas": "fenetidina",
    "fenezca": "fenecer",
    "fenezcamos": "fenecer",
    "fenezcan": "fenecer",
    "fenezcas": "fenecer",
    "fenezco": "fenecer",
    "fenezcáis": "fenecer",
    "feniana": "feniano",
    "fenianas": "feniano",
    "fenianismos": "fenianismo",
    "fenianos": "feniano",
    "fenicia": "fenicio",
    "fenicias": "fenicio",
    "fenicios": "fenicio",
    "fenicoptéridos": "fenicoptérido",
    "fenilacetaldehídos": "fenilacetaldehído",
    "fenilacetamidas": "fenilacetamida",
    "fenilacetatos": "fenilacetato",
    "fenilacética": "fenilacético",
    "fenilacéticas": "fenilacético",
    "fenilacéticos": "fenilacético",
    "fenilalaninas": "fenilalanina",
    "fenilaminas": "fenilamina",
    "fenilarsónica": "fenilarsónico",
    "fenilarsónicas": "fenilarsónico",
    "fenilarsónicos": "fenilarsónico",
    "fenilbutazonas": "fenilbutazona",
    "fenilbutinoles": "fenilbutinol",
    "fenilcarbamatos": "fenilcarbamato",
    "fenilcarbetoxipirazolonas": "fenilcarbetoxipirazolona",
    "fenilciclohexanoles": "fenilciclohexanol",
    "fenilciclohexanos": "fenilciclohexano",
    "fenilendiaminas": "fenilendiamina",
    "feniletanolaminas": "feniletanolamina",
    "feniletilacética": "feniletilacético",
    "feniletilacéticas": "feniletilacético",
    "feniletilacéticos": "feniletilacético",
    "feniletiletanolaminas": "feniletiletanolamina",
    "fenilfenoles": "fenilfenol",
    "fenilfosfinatos": "fenilfosfinato",
    "fenilhidracinas": "fenilhidracina",
    "fenilmercúrica": "fenilmercúrico",
    "fenilmercúricas": "fenilmercúrico",
    "fenilmercúricos": "fenilmercúrico",
    "fenilmetiletanolaminas": "fenilmetiletanolamina",
    "fenilmorfolinas": "fenilmorfolina",
    "fenilnaftilaminas": "fenilnaftilamina",
    "fenilos": "fenilo",
    "fenilpiperacinas": "fenilpiperacina",
    "fenilpropilmetilaminas": "fenilpropilmetilamina",
    "feniltriclorosilanos": "feniltriclorosilano",
    "fenobarbitales": "fenobarbital",
    "fenoclinos": "fenoclino",
    "fenocristales": "fenocristal",
    "fenogrecos": "fenogreco",
    "fenoldisulfónica": "fenoldisulfónico",
    "fenoldisulfónicas": "fenoldisulfónico",
    "fenoldisulfónicos": "fenoldisulfónico",
    "fenoles": "fenol",
    "fenolftaleínas": "fenolftaleína",
    "fenologías": "fenología",
    "fenolsulfonatos": "fenolsulfonato",
    "fenolsulfónica": "fenolsulfónico",
    "fenolsulfónicas": "fenolsulfónico",
    "fenolsulfónicos": "fenolsulfónico",
    "fenológica": "fenológico",
    "fenológicas": "fenológico",
    "fenológicos": "fenológico",
    "fenomenales": "fenomenal",
    "fenomenismos": "fenomenismo",
    "fenomenologías": "fenomenología",
    "fenomenológica": "fenomenológico",
    "fenomenológicas": "fenomenológico",
    "fenomenológicos": "fenomenológico",
    "fenoménica": "fenoménico",
    "fenoménicas": "fenoménico",
    "fenoménicos": "fenoménico",
    "fenotiacinas": "fenotiacina",
    "fenotipos": "fenotipo",
    "fenotípica": "fenotípico",
    "fenotípicas": "fenotípico",
    "fenotípicos": "fenotípico",
    "fenoxiacética": "fenoxiacético",
    "fenoxiacéticas": "fenoxiacético",
    "fenoxiacéticos": "fenoxiacético",
    "fenoxibenzaminas": "fenoxibenzamina",
    "fenoxipropanodioles": "fenoxipropanodiol",
    "fens": "fen",
    "fenílica": "fenílico",
    "fenílicas": "fenílico",
    "fenílicos": "fenílico",
    "fenólica": "fenólico",
    "fenólicas": "fenólico",
    "fenólicos": "fenólico",
    "fenómenos": "fenómeno",
    "fenóxidos": "fenóxido",
    "feofíceas": "feofícea",
    "feos": "feo",
    "feraces": "feraz",
    "feracidades": "feracidad",
    "ferales": "feral",
    "ferbames": "ferbam",
    "ferberitas": "ferberita",
    "ferbisialsoles": "ferbisialsol",
    "fergusonitas": "fergusonita",
    "feria": "feriar",
    "feriaba": "feriar",
    "feriabais": "feriar",
    "feriaban": "feriar",
    "feriabas": "feriar",
    "feriad": "feriar",
    "feriada": "feriar",
    "feriadas": "feriar",
    "feriado": "feriar",
    "feriados": "feriar",
    "feriales": "ferial",
    "feriamos": "feriar",
    "ferian": "feriar",
    "feriando": "feriar",
    "feriantes": "feriante",
    "feriara": "feriar",
    "feriarais": "feriar",
    "feriaran": "feriar",
    "feriaras": "feriar",
    "feriare": "feriar",
    "feriareis": "feriar",
    "feriaremos": "feriar",
    "feriaren": "feriar",
    "feriares": "feriar",
    "feriaron": "feriar",
    "feriará": "feriar",
    "feriarán": "feriar",
    "feriarás": "feriar",
    "feriaré": "feriar",
    "feriaréis": "feriar",
    "feriaría": "feriar",
    "feriaríais": "feriar",
    "feriaríamos": "feriar",
    "feriarían": "feriar",
    "feriarías": "feriar",
    "ferias": "feriar",
    "feriase": "feriar",
    "feriaseis": "feriar",
    "feriasen": "feriar",
    "feriases": "feriar",
    "feriaste": "feriar",
    "feriasteis": "feriar",
    "ferie": "feriar",
    "feriemos": "feriar",
    "ferien": "feriar",
    "feries": "feriar",
    "ferina": "ferino",
    "ferinas": "ferino",
    "ferinos": "ferino",
    "ferio": "feriar",
    "feriábamos": "feriar",
    "feriáis": "feriar",
    "feriáramos": "feriar",
    "feriáremos": "feriar",
    "feriásemos": "feriar",
    "ferié": "feriar",
    "feriéis": "feriar",
    "ferió": "feriar",
    "fermatas": "fermata",
    "fermenta": "fermentar",
    "fermentaba": "fermentar",
    "fermentabais": "fermentar",
    "fermentaban": "fermentar",
    "fermentabas": "fermentar",
    "fermentables": "fermentable",
    "fermentaciones": "fermentación",
    "fermentad": "fermentar",
    "fermentada": "fermentar",
    "fermentadas": "fermentar",
    "fermentado": "fermentar",
    "fermentadora": "fermentador",
    "fermentadoras": "fermentador",
    "fermentadores": "fermentador",
    "fermentados": "fermentar",
    "fermentamos": "fermentar",
    "fermentan": "fermentar",
    "fermentando": "fermentar",
    "fermentara": "fermentar",
    "fermentarais": "fermentar",
    "fermentaran": "fermentar",
    "fermentaras": "fermentar",
    "fermentare": "fermentar",
    "fermentareis": "fermentar",
    "fermentaremos": "fermentar",
    "fermentaren": "fermentar",
    "fermentares": "fermentar",
    "fermentaron": "fermentar",
    "fermentará": "fermentar",
    "fermentarán": "fermentar",
    "fermentarás": "fermentar",
    "fermentaré": "fermentar",
    "fermentaréis": "fermentar",
    "fermentaría": "fermentar",
    "fermentaríais": "fermentar",
    "fermentaríamos": "fermentar",
    "fermentarían": "fermentar",
    "fermentarías": "fermentar",
    "fermentas": "fermentar",
    "fermentase": "fermentar",
    "fermentaseis": "fermentar",
    "fermentasen": "fermentar",
    "fermentases": "fermentar",
    "fermentaste": "fermentar",
    "fermentasteis": "fermentar",
    "fermentativa": "fermentativo",
    "fermentativas": "fermentativo",
    "fermentativos": "fermentativo",
    "fermente": "fermentar",
    "fermentemos": "fermentar",
    "fermenten": "fermentar",
    "fermentes": "fermentar",
    "fermento": "fermentar",
    "fermentos": "fermento",
    "fermentábamos": "fermentar",
    "fermentáis": "fermentar",
    "fermentáramos": "fermentar",
    "fermentáremos": "fermentar",
    "fermentásemos": "fermentar",
    "fermenté": "fermentar",
    "fermentéis": "fermentar",
    "fermentó": "fermentar",
    "fermies": "fermi",
    "fermiones": "fermión",
    "fermios": "fermio",
    "fermis": "fermi",
    "fernandina": "fernandino",
    "fernandinas": "fernandino",
    "fernandinos": "fernandino",
    "feroces": "feroz",
    "ferocidades": "ferocidad",
    "ferodos": "ferodo",
    "ferozes": "feroz",
    "ferraba": "ferrar",
    "ferrabais": "ferrar",
    "ferraban": "ferrar",
    "ferrabas": "ferrar",
    "ferrad": "ferrar",
    "ferrada": "ferrar",
    "ferradas": "ferrar",
    "ferrado": "ferrar",
    "ferrados": "ferrar",
    "ferrallistas": "ferrallista",
    "ferralítica": "ferralítico",
    "ferralíticas": "ferralítico",
    "ferralíticos": "ferralítico",
    "ferramos": "ferrar",
    "ferrando": "ferrar",
    "ferrara": "ferrar",
    "ferrarais": "ferrar",
    "ferraran": "ferrar",
    "ferraras": "ferrar",
    "ferrare": "ferrar",
    "ferrareis": "ferrar",
    "ferraremos": "ferrar",
    "ferraren": "ferrar",
    "ferrares": "ferrar",
    "ferraron": "ferrar",
    "ferrará": "ferrar",
    "ferrarán": "ferrar",
    "ferrarás": "ferrar",
    "ferraré": "ferrar",
    "ferraréis": "ferrar",
    "ferraría": "ferrar",
    "ferraríais": "ferrar",
    "ferraríamos": "ferrar",
    "ferrarían": "ferrar",
    "ferrarías": "ferrar",
    "ferrase": "ferrar",
    "ferraseis": "ferrar",
    "ferrasen": "ferrar",
    "ferrases": "ferrar",
    "ferraste": "ferrar",
    "ferrasteis": "ferrar",
    "ferremos": "ferrar",
    "ferreruelos": "ferreruelo",
    "ferrerías": "ferrería",
    "ferretera": "ferretero",
    "ferreteras": "ferretero",
    "ferreteros": "ferretero",
    "ferreterías": "ferretería",
    "ferretes": "ferrete",
    "ferricianuros": "ferricianuro",
    "ferrieléctrica": "ferrieléctrico",
    "ferrieléctricas": "ferrieléctrico",
    "ferrieléctricos": "ferrieléctrico",
    "ferrificaciones": "ferrificación",
    "ferrimagnetismos": "ferrimagnetismo",
    "ferrimagnética": "ferrimagnético",
    "ferrimagnéticas": "ferrimagnético",
    "ferrimagnéticos": "ferrimagnético",
    "ferris": "ferry",
    "ferritas": "ferrita",
    "ferritinas": "ferritina",
    "ferroaleaciones": "ferroaleación",
    "ferroaluminios": "ferroaluminio",
    "ferroboros": "ferroboro",
    "ferrocarrilera": "ferrocarrilero",
    "ferrocarrileras": "ferrocarrilero",
    "ferrocarrileros": "ferrocarrilero",
    "ferrocarriles": "ferrocarril",
    "ferrocenos": "ferroceno",
    "ferrocianuros": "ferrocianuro",
    "ferrocinos": "ferrocino",
    "ferrocromos": "ferrocromo",
    "ferrodinámica": "ferrodinámico",
    "ferrodinámicas": "ferrodinámico",
    "ferrodinámicos": "ferrodinámico",
    "ferroelectricidades": "ferroelectricidad",
    "ferroeléctrica": "ferroeléctrico",
    "ferroeléctricas": "ferroeléctrico",
    "ferroeléctricos": "ferroeléctrico",
    "ferrofósforos": "ferrofósforo",
    "ferroinductiva": "ferroinductivo",
    "ferroinductivas": "ferroinductivo",
    "ferroinductivos": "ferroinductivo",
    "ferromagnetismos": "ferromagnetismo",
    "ferromagnética": "ferromagnético",
    "ferromagnéticas": "ferromagnético",
    "ferromagnéticos": "ferromagnético",
    "ferromanganesos": "ferromanganeso",
    "ferromolibdenos": "ferromolibdeno",
    "ferrones": "ferrón",
    "ferroníqueles": "ferroníquel",
    "ferroprusiatos": "ferroprusiato",
    "ferropénica": "ferropénico",
    "ferropénicas": "ferropénico",
    "ferropénicos": "ferropénico",
    "ferrorresonancias": "ferrorresonancia",
    "ferrorresonantes": "ferrorresonante",
    "ferrosa": "ferroso",
    "ferrosas": "ferroso",
    "ferrosilicios": "ferrosilicio",
    "ferrosos": "ferroso",
    "ferrotitanios": "ferrotitanio",
    "ferrotungstenos": "ferrotungsteno",
    "ferrovanadios": "ferrovanadio",
    "ferroviales": "ferrovial",
    "ferroviaria": "ferroviario",
    "ferroviarias": "ferroviario",
    "ferroviarios": "ferroviario",
    "ferruginizaciones": "ferruginización",
    "ferruginosa": "ferruginoso",
    "ferruginosas": "ferruginoso",
    "ferruginosos": "ferruginoso",
    "ferrugínea": "ferrugíneo",
    "ferrugíneas": "ferrugíneo",
    "ferrugíneos": "ferrugíneo",
    "ferrábamos": "ferrar",
    "ferráis": "ferrar",
    "ferráramos": "ferrar",
    "ferráremos": "ferrar",
    "ferrásemos": "ferrar",
    "ferré": "ferrar",
    "ferréis": "ferrar",
    "ferrífera": "ferrífero",
    "ferríferas": "ferrífero",
    "ferríferos": "ferrífero",
    "ferrítica": "ferrítico",
    "ferríticas": "ferrítico",
    "ferríticos": "ferrítico",
    "ferró": "ferrar",
    "fersialítica": "fersialítico",
    "fersialíticas": "fersialítico",
    "fersialíticos": "fersialítico",
    "fertilice": "fertilizar",
    "fertilicemos": "fertilizar",
    "fertilicen": "fertilizar",
    "fertilices": "fertilizar",
    "fertilicé": "fertilizar",
    "fertilicéis": "fertilizar",
    "fertilidades": "fertilidad",
    "fertiliza": "fertilizar",
    "fertilizaba": "fertilizar",
    "fertilizabais": "fertilizar",
    "fertilizaban": "fertilizar",
    "fertilizabas": "fertilizar",
    "fertilizables": "fertilizable",
    "fertilizaciones": "fertilización",
    "fertilizad": "fertilizar",
    "fertilizada": "fertilizar",
    "fertilizadas": "fertilizar",
    "fertilizado": "fertilizar",
    "fertilizadora": "fertilizador",
    "fertilizadoras": "fertilizador",
    "fertilizadores": "fertilizador",
    "fertilizados": "fertilizar",
    "fertilizamos": "fertilizar",
    "fertilizan": "fertilizar",
    "fertilizando": "fertilizar",
    "fertilizantes": "fertilizante",
    "fertilizara": "fertilizar",
    "fertilizarais": "fertilizar",
    "fertilizaran": "fertilizar",
    "fertilizaras": "fertilizar",
    "fertilizare": "fertilizar",
    "fertilizareis": "fertilizar",
    "fertilizaremos": "fertilizar",
    "fertilizaren": "fertilizar",
    "fertilizares": "fertilizar",
    "fertilizaron": "fertilizar",
    "fertilizará": "fertilizar",
    "fertilizarán": "fertilizar",
    "fertilizarás": "fertilizar",
    "fertilizaré": "fertilizar",
    "fertilizaréis": "fertilizar",
    "fertilizaría": "fertilizar",
    "fertilizaríais": "fertilizar",
    "fertilizaríamos": "fertilizar",
    "fertilizarían": "fertilizar",
    "fertilizarías": "fertilizar",
    "fertilizas": "fertilizar",
    "fertilizase": "fertilizar",
    "fertilizaseis": "fertilizar",
    "fertilizasen": "fertilizar",
    "fertilizases": "fertilizar",
    "fertilizaste": "fertilizar",
    "fertilizasteis": "fertilizar",
    "fertilizo": "fertilizar",
    "fertilizábamos": "fertilizar",
    "fertilizáis": "fertilizar",
    "fertilizáramos": "fertilizar",
    "fertilizáremos": "fertilizar",
    "fertilizásemos": "fertilizar",
    "fertilizó": "fertilizar",
    "fertirrigaciones": "fertirrigación",
    "ferulizaciones": "ferulización",
    "ferulácea": "feruláceo",
    "feruláceas": "feruláceo",
    "feruláceos": "feruláceo",
    "fervientes": "ferviente",
    "fervores": "fervor",
    "fervorice": "fervorizar",
    "fervoricemos": "fervorizar",
    "fervoricen": "fervorizar",
    "fervorices": "fervorizar",
    "fervoricé": "fervorizar",
    "fervoricéis": "fervorizar",
    "fervorines": "fervorín",
    "fervoriza": "fervorizar",
    "fervorizaba": "fervorizar",
    "fervorizabais": "fervorizar",
    "fervorizaban": "fervorizar",
    "fervorizabas": "fervorizar",
    "fervorizad": "fervorizar",
    "fervorizada": "fervorizar",
    "fervorizadas": "fervorizar",
    "fervorizado": "fervorizar",
    "fervorizados": "fervorizar",
    "fervorizamos": "fervorizar",
    "fervorizan": "fervorizar",
    "fervorizando": "fervorizar",
    "fervorizara": "fervorizar",
    "fervorizarais": "fervorizar",
    "fervorizaran": "fervorizar",
    "fervorizaras": "fervorizar",
    "fervorizare": "fervorizar",
    "fervorizareis": "fervorizar",
    "fervorizaremos": "fervorizar",
    "fervorizaren": "fervorizar",
    "fervorizares": "fervorizar",
    "fervorizaron": "fervorizar",
    "fervorizará": "fervorizar",
    "fervorizarán": "fervorizar",
    "fervorizarás": "fervorizar",
    "fervorizaré": "fervorizar",
    "fervorizaréis": "fervorizar",
    "fervorizaría": "fervorizar",
    "fervorizaríais": "fervorizar",
    "fervorizaríamos": "fervorizar",
    "fervorizarían": "fervorizar",
    "fervorizarías": "fervorizar",
    "fervorizas": "fervorizar",
    "fervorizase": "fervorizar",
    "fervorizaseis": "fervorizar",
    "fervorizasen": "fervorizar",
    "fervorizases": "fervorizar",
    "fervorizaste": "fervorizar",
    "fervorizasteis": "fervorizar",
    "fervorizo": "fervorizar",
    "fervorizábamos": "fervorizar",
    "fervorizáis": "fervorizar",
    "fervorizáramos": "fervorizar",
    "fervorizáremos": "fervorizar",
    "fervorizásemos": "fervorizar",
    "fervorizó": "fervorizar",
    "fervorosa": "fervoroso",
    "fervorosas": "fervoroso",
    "fervorosos": "fervoroso",
    "fes": "fe",
    "festeja": "festejar",
    "festejaba": "festejar",
    "festejabais": "festejar",
    "festejaban": "festejar",
    "festejabas": "festejar",
    "festejad": "festejar",
    "festejada": "festejar",
    "festejadas": "festejar",
    "festejado": "festejar",
    "festejadora": "festejador",
    "festejadoras": "festejador",
    "festejadores": "festejador",
    "festejados": "festejar",
    "festejamos": "festejar",
    "festejan": "festejar",
    "festejando": "festejar",
    "festejara": "festejar",
    "festejarais": "festejar",
    "festejaran": "festejar",
    "festejaras": "festejar",
    "festejare": "festejar",
    "festejareis": "festejar",
    "festejaremos": "festejar",
    "festejaren": "festejar",
    "festejares": "festejar",
    "festejaron": "festejar",
    "festejará": "festejar",
    "festejarán": "festejar",
    "festejarás": "festejar",
    "festejaré": "festejar",
    "festejaréis": "festejar",
    "festejaría": "festejar",
    "festejaríais": "festejar",
    "festejaríamos": "festejar",
    "festejarían": "festejar",
    "festejarías": "festejar",
    "festejas": "festejar",
    "festejase": "festejar",
    "festejaseis": "festejar",
    "festejasen": "festejar",
    "festejases": "festejar",
    "festejaste": "festejar",
    "festejasteis": "festejar",
    "festeje": "festejar",
    "festejemos": "festejar",
    "festejen": "festejar",
    "festejes": "festejar",
    "festejo": "festejar",
    "festejos": "festejo",
    "festejábamos": "festejar",
    "festejáis": "festejar",
    "festejáramos": "festejar",
    "festejáremos": "festejar",
    "festejásemos": "festejar",
    "festejé": "festejar",
    "festejéis": "festejar",
    "festejó": "festejar",
    "festinaciones": "festinación",
    "festines": "festín",
    "festiva": "festivo",
    "festivales": "festival",
    "festivas": "festivo",
    "festividades": "festividad",
    "festivos": "festivo",
    "festonea": "festonear",
    "festoneaba": "festonear",
    "festoneabais": "festonear",
    "festoneaban": "festonear",
    "festoneabas": "festonear",
    "festonead": "festonear",
    "festoneada": "festonear",
    "festoneadas": "festonear",
    "festoneado": "festonear",
    "festoneados": "festonear",
    "festoneamos": "festonear",
    "festonean": "festonear",
    "festoneando": "festonear",
    "festoneara": "festonear",
    "festonearais": "festonear",
    "festonearan": "festonear",
    "festonearas": "festonear",
    "festoneare": "festonear",
    "festoneareis": "festonear",
    "festonearemos": "festonear",
    "festonearen": "festonear",
    "festoneares": "festonear",
    "festonearon": "festonear",
    "festoneará": "festonear",
    "festonearán": "festonear",
    "festonearás": "festonear",
    "festonearé": "festonear",
    "festonearéis": "festonear",
    "festonearía": "festonear",
    "festonearíais": "festonear",
    "festonearíamos": "festonear",
    "festonearían": "festonear",
    "festonearías": "festonear",
    "festoneas": "festonear",
    "festonease": "festonear",
    "festoneaseis": "festonear",
    "festoneasen": "festonear",
    "festoneases": "festonear",
    "festoneaste": "festonear",
    "festoneasteis": "festonear",
    "festonee": "festonear",
    "festoneemos": "festonear",
    "festoneen": "festonear",
    "festonees": "festonear",
    "festoneo": "festonear",
    "festones": "festón",
    "festoneábamos": "festonear",
    "festoneáis": "festonear",
    "festoneáramos": "festonear",
    "festoneáremos": "festonear",
    "festoneásemos": "festonear",
    "festoneé": "festonear",
    "festoneéis": "festonear",
    "festoneó": "festonear",
    "festucas": "festuca",
    "fetaciones": "fetación",
    "fetales": "fetal",
    "fetiches": "fetiche",
    "fetichismos": "fetichismo",
    "fetichistas": "fetichista",
    "feticidas": "feticida",
    "feticidios": "feticidio",
    "fetideces": "fetidez",
    "fetoproteínas": "fetoproteína",
    "fetos": "feto",
    "feudales": "feudal",
    "feudalidades": "feudalidad",
    "feudalismos": "feudalismo",
    "feudataria": "feudatario",
    "feudatarias": "feudatario",
    "feudatarios": "feudatario",
    "feudos": "feudo",
    "feísmos": "feísmo",
    "feófitos": "feófito",
    "feúca": "feúco",
    "feúcas": "feúco",
    "feúcha": "feúcho",
    "feúchas": "feúcho",
    "feúchos": "feúcho",
    "feúcos": "feúco",
    "fiaba": "fiar",
    "fiabais": "fiar",
    "fiaban": "fiar",
    "fiabas": "fiar",
    "fiabilidades": "fiabilidad",
    "fiables": "fiable",
    "fiacres": "fiacre",
    "fiad": "fiar",
    "fiada": "fiar",
    "fiadas": "fiar",
    "fiado": "fiar",
    "fiadora": "fiador",
    "fiadoras": "fiador",
    "fiadores": "fiador",
    "fiados": "fiar",
    "fiales": "fiale",
    "fiambreras": "fiambrera",
    "fiambres": "fiambre",
    "fiamos": "fiar",
    "fiando": "fiar",
    "fianitas": "fianita",
    "fianzas": "fianza",
    "fiara": "fiar",
    "fiarais": "fiar",
    "fiaran": "fiar",
    "fiaras": "fiar",
    "fiare": "fiar",
    "fiareis": "fiar",
    "fiaremos": "fiar",
    "fiaren": "fiar",
    "fiares": "fiar",
    "fiaron": "fiar",
    "fiará": "fiar",
    "fiarán": "fiar",
    "fiarás": "fiar",
    "fiaré": "fiar",
    "fiaréis": "fiar",
    "fiaría": "fiar",
    "fiaríais": "fiar",
    "fiaríamos": "fiar",
    "fiarían": "fiar",
    "fiarías": "fiar",
    "fiascos": "fiasco",
    "fiase": "fiar",
    "fiaseis": "fiar",
    "fiasen": "fiar",
    "fiases": "fiar",
    "fiaste": "fiar",
    "fiasteis": "fiar",
    "fibrada": "fibrado",
    "fibradas": "fibrado",
    "fibrados": "fibrado",
    "fibras": "fibra",
    "fibrilaciones": "fibrilación",
    "fibriladora": "fibrilador",
    "fibriladoras": "fibrilador",
    "fibriladores": "fibrilador",
    "fibrilares": "fibrilar",
    "fibrillas": "fibrilla",
    "fibrinas": "fibrina",
    "fibrinasas": "fibrinasa",
    "fibrinemias": "fibrinemia",
    "fibrinoformaciones": "fibrinoformación",
    "fibrinoides": "fibrinoide",
    "fibrinolisinas": "fibrinolisina",
    "fibrinógenos": "fibrinógeno",
    "fibroblastos": "fibroblasto",
    "fibrocartilaginosa": "fibrocartilaginoso",
    "fibrocartilaginosas": "fibrocartilaginoso",
    "fibrocartilaginosos": "fibrocartilaginoso",
    "fibrocartílagos": "fibrocartílago",
    "fibrocementos": "fibrocemento",
    "fibrocitos": "fibrocito",
    "fibrodentinas": "fibrodentina",
    "fibrolitas": "fibrolita",
    "fibromas": "fibroma",
    "fibromucosas": "fibromucosa",
    "fibronectinas": "fibronectina",
    "fibroplasias": "fibroplasia",
    "fibrosa": "fibroso",
    "fibrosarcomas": "fibrosarcoma",
    "fibrosas": "fibroso",
    "fibroscopios": "fibroscopio",
    "fibrosos": "fibroso",
    "fibrógrafos": "fibrógrafo",
    "fibulaciones": "fibulación",
    "fibulares": "fibular",
    "ficciones": "ficción",
    "ficha": "fichar",
    "fichaba": "fichar",
    "fichabais": "fichar",
    "fichaban": "fichar",
    "fichabas": "fichar",
    "fichad": "fichar",
    "fichada": "fichar",
    "fichadas": "fichar",
    "fichado": "fichar",
    "fichados": "fichar",
    "fichajes": "fichaje",
    "fichamos": "fichar",
    "fichan": "fichar",
    "fichando": "fichar",
    "fichara": "fichar",
    "ficharais": "fichar",
    "ficharan": "fichar",
    "ficharas": "fichar",
    "fichare": "fichar",
    "fichareis": "fichar",
    "ficharemos": "fichar",
    "ficharen": "fichar",
    "fichares": "fichar",
    "ficharon": "fichar",
    "fichará": "fichar",
    "ficharán": "fichar",
    "ficharás": "fichar",
    "ficharé": "fichar",
    "ficharéis": "fichar",
    "ficharía": "fichar",
    "ficharíais": "fichar",
    "ficharíamos": "fichar",
    "ficharían": "fichar",
    "ficharías": "fichar",
    "fichas": "fichar",
    "fichase": "fichar",
    "fichaseis": "fichar",
    "fichasen": "fichar",
    "fichases": "fichar",
    "fichaste": "fichar",
    "fichasteis": "fichar",
    "fiche": "fichar",
    "fichemos": "fichar",
    "fichen": "fichar",
    "ficheros": "fichero",
    "fiches": "fichar",
    "ficho": "fichar",
    "fichábamos": "fichar",
    "ficháis": "fichar",
    "ficháramos": "fichar",
    "ficháremos": "fichar",
    "fichásemos": "fichar",
    "fiché": "fichar",
    "fichéis": "fichar",
    "fichó": "fichar",
    "ficinas": "ficina",
    "ficoidea": "ficoideo",
    "ficoideas": "ficoideo",
    "ficoideos": "ficoideo",
    "ficologías": "ficología",
    "ficomicetes": "ficomicete",
    "ficticia": "ficticio",
    "ficticias": "ficticio",
    "ficticios": "ficticio",
    "ficófaga": "ficófago",
    "ficófagas": "ficófago",
    "ficófagos": "ficófago",
    "fidedigna": "fidedigno",
    "fidedignas": "fidedigno",
    "fidedignos": "fidedigno",
    "fideera": "fideero",
    "fideeras": "fideero",
    "fideeros": "fideero",
    "fideicomisa": "fideicomiso",
    "fideicomisaria": "fideicomisario",
    "fideicomisarias": "fideicomisario",
    "fideicomisarios": "fideicomisario",
    "fideicomisas": "fideicomiso",
    "fideicomisos": "fideicomiso",
    "fideicomitentes": "fideicomitente",
    "fideicomitida": "fideicomitido",
    "fideicomitidas": "fideicomitido",
    "fideicomitidos": "fideicomitido",
    "fidelidades": "fidelidad",
    "fideos": "fideo",
    "fideísmos": "fideísmo",
    "fiduciales": "fiducial",
    "fiduciaria": "fiduciario",
    "fiduciarias": "fiduciario",
    "fiduciarios": "fiduciario",
    "fiducias": "fiducia",
    "fiducidaria": "fiducidario",
    "fiducidarias": "fiducidario",
    "fiducidarios": "fiducidario",
    "fiebres": "fiebre",
    "fielatos": "fielato",
    "fieldades": "fieldad",
    "fieles": "fiel",
    "fieltros": "fieltro",
    "fiemos": "fiar",
    "fiera": "fiero",
    "fierabrases": "fierabrás",
    "fieras": "fiero",
    "fierezas": "fiereza",
    "fieros": "fiero",
    "fierra": "ferrar",
    "fierran": "ferrar",
    "fierras": "ferrar",
    "fierre": "ferrar",
    "fierren": "ferrar",
    "fierres": "ferrar",
    "fierro": "ferrar",
    "fierros": "fierro",
    "fiestas": "fiesta",
    "fiestera": "fiestero",
    "fiesteras": "fiestero",
    "fiesteros": "fiestero",
    "fig": "figura",
    "figles": "figle",
    "figonera": "figonero",
    "figoneras": "figonero",
    "figoneros": "figonero",
    "figones": "figón",
    "figs": "figura",
    "figuerenses": "figuerense",
    "figulina": "figulino",
    "figulinas": "figulino",
    "figulinos": "figulino",
    "figura": "figurar",
    "figuraba": "figurar",
    "figurabais": "figurar",
    "figuraban": "figurar",
    "figurabas": "figurar",
    "figurables": "figurable",
    "figuraciones": "figuración",
    "figurad": "figurar",
    "figurada": "figurar",
    "figuradas": "figurar",
    "figurado": "figurar",
    "figurados": "figurar",
    "figuramos": "figurar",
    "figuran": "figurar",
    "figurando": "figurar",
    "figuranta": "figurante",
    "figurantas": "figurante",
    "figurantes": "figurante",
    "figurara": "figurar",
    "figurarais": "figurar",
    "figuraran": "figurar",
    "figuraras": "figurar",
    "figurare": "figurar",
    "figurareis": "figurar",
    "figuraremos": "figurar",
    "figuraren": "figurar",
    "figurares": "figurar",
    "figuraron": "figurar",
    "figurará": "figurar",
    "figurarán": "figurar",
    "figurarás": "figurar",
    "figuraré": "figurar",
    "figuraréis": "figurar",
    "figuraría": "figurar",
    "figuraríais": "figurar",
    "figuraríamos": "figurar",
    "figurarían": "figurar",
    "figurarías": "figurar",
    "figuras": "figurar",
    "figurase": "figurar",
    "figuraseis": "figurar",
    "figurasen": "figurar",
    "figurases": "figurar",
    "figuraste": "figurar",
    "figurasteis": "figurar",
    "figurativa": "figurativo",
    "figurativas": "figurativo",
    "figurativos": "figurativo",
    "figure": "figurar",
    "figuremos": "figurar",
    "figuren": "figurar",
    "figurera": "figurero",
    "figureras": "figurero",
    "figureros": "figurero",
    "figurerías": "figurería",
    "figures": "figurar",
    "figurillas": "figurilla",
    "figurines": "figurín",
    "figurinistas": "figurinista",
    "figuritas": "figurita",
    "figuro": "figurar",
    "figurones": "figurón",
    "figurábamos": "figurar",
    "figuráis": "figurar",
    "figuráramos": "figurar",
    "figuráremos": "figurar",
    "figurásemos": "figurar",
    "figuré": "figurar",
    "figuréis": "figurar",
    "figuró": "figurar",
    "fija": "fijo",
    "fijaba": "fijar",
    "fijabais": "fijar",
    "fijaban": "fijar",
    "fijabas": "fijar",
    "fijaciones": "fijación",
    "fijad": "fijar",
    "fijada": "fijar",
    "fijadas": "fijar",
    "fijado": "fijar",
    "fijadora": "fijador",
    "fijadoras": "fijador",
    "fijadores": "fijador",
    "fijados": "fijar",
    "fijamos": "fijar",
    "fijan": "fijar",
    "fijando": "fijar",
    "fijapelos": "fijapelo",
    "fijara": "fijar",
    "fijarais": "fijar",
    "fijaran": "fijar",
    "fijaras": "fijar",
    "fijare": "fijar",
    "fijareis": "fijar",
    "fijaremos": "fijar",
    "fijaren": "fijar",
    "fijares": "fijar",
    "fijaron": "fijar",
    "fijará": "fijar",
    "fijarán": "fijar",
    "fijarás": "fijar",
    "fijaré": "fijar",
    "fijaréis": "fijar",
    "fijaría": "fijar",
    "fijaríais": "fijar",
    "fijaríamos": "fijar",
    "fijarían": "fijar",
    "fijarías": "fijar",
    "fijas": "fijo",
    "fijase": "fijar",
    "fijaseis": "fijar",
    "fijasen": "fijar",
    "fijases": "fijar",
    "fijaste": "fijar",
    "fijasteis": "fijar",
    "fije": "fijar",
    "fijemos": "fijar",
    "fijen": "fijar",
    "fijes": "fijar",
    "fijezas": "fijeza",
    "fijo": "fijar",
    "fijos": "fijo",
    "fijábamos": "fijar",
    "fijáis": "fijar",
    "fijáramos": "fijar",
    "fijáremos": "fijar",
    "fijásemos": "fijar",
    "fijé": "fijar",
    "fijéis": "fijar",
    "fijó": "fijar",
    "filacterias": "filacteria",
    "filadices": "filadiz",
    "filadios": "filadio",
    "filamentos": "filamento",
    "filamentosa": "filamentoso",
    "filamentosas": "filamentoso",
    "filamentosos": "filamentoso",
    "filandrias": "filandria",
    "filandros": "filandro",
    "filantropismos": "filantropismo",
    "filantropías": "filantropía",
    "filantrópica": "filantrópico",
    "filantrópicas": "filantrópico",
    "filantrópicos": "filantrópico",
    "filarias": "filaria",
    "filarmonías": "filarmonía",
    "filarmónica": "filarmónico",
    "filarmónicas": "filarmónico",
    "filarmónicos": "filarmónico",
    "filas": "fila",
    "filatelias": "filatelia",
    "filatelistas": "filatelista",
    "filatera": "filatero",
    "filateras": "filatero",
    "filateros": "filatero",
    "filaterías": "filatería",
    "filatélica": "filatélico",
    "filatélicas": "filatélico",
    "filatélicos": "filatélico",
    "filemones": "filemón",
    "filena": "fileno",
    "filenas": "fileno",
    "filenos": "fileno",
    "fileras": "filera",
    "filetas": "fileta",
    "filetea": "filetear",
    "fileteaba": "filetear",
    "fileteabais": "filetear",
    "fileteaban": "filetear",
    "fileteabas": "filetear",
    "filetead": "filetear",
    "fileteada": "filetear",
    "fileteadas": "filetear",
    "fileteado": "filetear",
    "fileteados": "filetear",
    "fileteamos": "filetear",
    "filetean": "filetear",
    "fileteando": "filetear",
    "fileteara": "filetear",
    "filetearais": "filetear",
    "filetearan": "filetear",
    "filetearas": "filetear",
    "fileteare": "filetear",
    "fileteareis": "filetear",
    "filetearemos": "filetear",
    "filetearen": "filetear",
    "fileteares": "filetear",
    "filetearon": "filetear",
    "fileteará": "filetear",
    "filetearán": "filetear",
    "filetearás": "filetear",
    "filetearé": "filetear",
    "filetearéis": "filetear",
    "filetearía": "filetear",
    "filetearíais": "filetear",
    "filetearíamos": "filetear",
    "filetearían": "filetear",
    "filetearías": "filetear",
    "fileteas": "filetear",
    "filetease": "filetear",
    "fileteaseis": "filetear",
    "fileteasen": "filetear",
    "fileteases": "filetear",
    "fileteaste": "filetear",
    "fileteasteis": "filetear",
    "filetee": "filetear",
    "fileteemos": "filetear",
    "fileteen": "filetear",
    "filetees": "filetear",
    "fileteo": "filetear",
    "filetes": "filete",
    "fileteábamos": "filetear",
    "fileteáis": "filetear",
    "fileteáramos": "filetear",
    "fileteáremos": "filetear",
    "fileteásemos": "filetear",
    "fileteé": "filetear",
    "fileteéis": "filetear",
    "fileteó": "filetear",
    "filetones": "filetón",
    "filfas": "filfa",
    "filhelenismos": "filhelenismo",
    "filiaba": "filiar",
    "filiabais": "filiar",
    "filiaban": "filiar",
    "filiabas": "filiar",
    "filiaciones": "filiación",
    "filiad": "filiar",
    "filiada": "filiar",
    "filiadas": "filiar",
    "filiado": "filiar",
    "filiados": "filiar",
    "filiales": "filial",
    "filiamos": "filiar",
    "filiando": "filiar",
    "filiara": "filiar",
    "filiarais": "filiar",
    "filiaran": "filiar",
    "filiaras": "filiar",
    "filiare": "filiar",
    "filiareis": "filiar",
    "filiaremos": "filiar",
    "filiaren": "filiar",
    "filiares": "filiar",
    "filiaron": "filiar",
    "filiará": "filiar",
    "filiarán": "filiar",
    "filiarás": "filiar",
    "filiaré": "filiar",
    "filiaréis": "filiar",
    "filiaría": "filiar",
    "filiaríais": "filiar",
    "filiaríamos": "filiar",
    "filiarían": "filiar",
    "filiarías": "filiar",
    "filias": "filia",
    "filiase": "filiar",
    "filiaseis": "filiar",
    "filiasen": "filiar",
    "filiases": "filiar",
    "filiaste": "filiar",
    "filiasteis": "filiar",
    "filibranquios": "filibranquio",
    "filibusterismos": "filibusterismo",
    "filibusteros": "filibustero",
    "filicópsidos": "filicópsido",
    "filiemos": "filiar",
    "filieras": "filiera",
    "filiformes": "filiforme",
    "filigranas": "filigrana",
    "fililíes": "fililí",
    "fililís": "fililí",
    "filipichines": "filipichín",
    "filipina": "filipino",
    "filipinas": "filipino",
    "filipinismos": "filipinismo",
    "filipinos": "filipino",
    "filipéndulas": "filipéndula",
    "filistea": "filisteo",
    "filisteas": "filisteo",
    "filisteos": "filisteo",
    "filitas": "filita",
    "filiábamos": "filiar",
    "filiáis": "filiar",
    "filiáramos": "filiar",
    "filiáremos": "filiar",
    "filiásemos": "filiar",
    "filié": "filiar",
    "filiéis": "filiar",
    "filió": "filiar",
    "fillós": "filló",
    "filma": "filmar",
    "filmaba": "filmar",
    "filmabais": "filmar",
    "filmaban": "filmar",
    "filmabas": "filmar",
    "filmaciones": "filmación",
    "filmad": "filmar",
    "filmada": "filmar",
    "filmadas": "filmar",
    "filmado": "filmar",
    "filmadora": "filmador",
    "filmadoras": "filmador",
    "filmadores": "filmador",
    "filmados": "filmar",
    "filmamos": "filmar",
    "filman": "filmar",
    "filmando": "filmar",
    "filmara": "filmar",
    "filmarais": "filmar",
    "filmaran": "filmar",
    "filmaras": "filmar",
    "filmare": "filmar",
    "filmareis": "filmar",
    "filmaremos": "filmar",
    "filmaren": "filmar",
    "filmares": "filmar",
    "filmaron": "filmar",
    "filmará": "filmar",
    "filmarán": "filmar",
    "filmarás": "filmar",
    "filmaré": "filmar",
    "filmaréis": "filmar",
    "filmaría": "filmar",
    "filmaríais": "filmar",
    "filmaríamos": "filmar",
    "filmarían": "filmar",
    "filmarías": "filmar",
    "filmas": "filmar",
    "filmase": "filmar",
    "filmaseis": "filmar",
    "filmasen": "filmar",
    "filmases": "filmar",
    "filmaste": "filmar",
    "filmasteis": "filmar",
    "filme": "filmar",
    "filmemos": "filmar",
    "filmen": "filmar",
    "filmes": "filme",
    "filminas": "filmina",
    "filmletes": "filmlet",
    "filmlets": "filmlet",
    "filmo": "filmar",
    "filmografías": "filmografía",
    "filmotecas": "filmoteca",
    "films": "film",
    "filmábamos": "filmar",
    "filmáis": "filmar",
    "filmáramos": "filmar",
    "filmáremos": "filmar",
    "filmásemos": "filmar",
    "filmé": "filmar",
    "filméis": "filmar",
    "filmó": "filmar",
    "filobranquias": "filobranquia",
    "filodendros": "filodendro",
    "filodios": "filodio",
    "filogenias": "filogenia",
    "filogenética": "filogenético",
    "filogenéticas": "filogenético",
    "filogenéticos": "filogenético",
    "filogénica": "filogénico",
    "filogénicas": "filogénico",
    "filogénicos": "filogénico",
    "filologías": "filología",
    "filológica": "filológico",
    "filológicas": "filológico",
    "filológicos": "filológico",
    "filomanías": "filomanía",
    "filomas": "filoma",
    "filomelas": "filomela",
    "filomenas": "filomena",
    "filones": "filón",
    "filoniana": "filoniano",
    "filonianas": "filoniano",
    "filonianos": "filoniano",
    "filonios": "filonio",
    "filonitas": "filonita",
    "filos": "filo",
    "filosedas": "filoseda",
    "filosilicatos": "filosilicato",
    "filosofa": "filosofar",
    "filosofaba": "filosofar",
    "filosofabais": "filosofar",
    "filosofaban": "filosofar",
    "filosofabas": "filosofar",
    "filosofad": "filosofar",
    "filosofada": "filosofar",
    "filosofadas": "filosofar",
    "filosofado": "filosofar",
    "filosofadora": "filosofador",
    "filosofadoras": "filosofador",
    "filosofadores": "filosofador",
    "filosofados": "filosofar",
    "filosofales": "filosofal",
    "filosofamos": "filosofar",
    "filosofan": "filosofar",
    "filosofando": "filosofar",
    "filosofara": "filosofar",
    "filosofarais": "filosofar",
    "filosofaran": "filosofar",
    "filosofaras": "filosofar",
    "filosofare": "filosofar",
    "filosofareis": "filosofar",
    "filosofaremos": "filosofar",
    "filosofaren": "filosofar",
    "filosofares": "filosofar",
    "filosofaron": "filosofar",
    "filosofará": "filosofar",
    "filosofarán": "filosofar",
    "filosofarás": "filosofar",
    "filosofaré": "filosofar",
    "filosofaréis": "filosofar",
    "filosofaría": "filosofar",
    "filosofaríais": "filosofar",
    "filosofaríamos": "filosofar",
    "filosofarían": "filosofar",
    "filosofarías": "filosofar",
    "filosofas": "filosofar",
    "filosofase": "filosofar",
    "filosofaseis": "filosofar",
    "filosofasen": "filosofar",
    "filosofases": "filosofar",
    "filosofaste": "filosofar",
    "filosofasteis": "filosofar",
    "filosofastra": "filosofastro",
    "filosofastras": "filosofastro",
    "filosofastros": "filosofastro",
    "filosofe": "filosofar",
    "filosofemas": "filosofema",
    "filosofemos": "filosofar",
    "filosofen": "filosofar",
    "filosofes": "filosofar",
    "filosofismos": "filosofismo",
    "filosofo": "filosofar",
    "filosofábamos": "filosofar",
    "filosofáis": "filosofar",
    "filosofáramos": "filosofar",
    "filosofáremos": "filosofar",
    "filosofásemos": "filosofar",
    "filosofé": "filosofar",
    "filosoféis": "filosofar",
    "filosofías": "filosofía",
    "filosofó": "filosofar",
    "filosoviética": "filosoviético",
    "filosoviéticas": "filosoviético",
    "filosoviéticos": "filosoviético",
    "filosófica": "filosófico",
    "filosóficas": "filosófico",
    "filosóficos": "filosófico",
    "filoxeras": "filoxera",
    "filoxérica": "filoxérico",
    "filoxéricas": "filoxérico",
    "filoxéricos": "filoxérico",
    "filtra": "filtrar",
    "filtraba": "filtrar",
    "filtrabais": "filtrar",
    "filtraban": "filtrar",
    "filtrabas": "filtrar",
    "filtrables": "filtrable",
    "filtraciones": "filtración",
    "filtrad": "filtrar",
    "filtrada": "filtrar",
    "filtradas": "filtrar",
    "filtrado": "filtrar",
    "filtradora": "filtrador",
    "filtradoras": "filtrador",
    "filtradores": "filtrador",
    "filtrados": "filtrar",
    "filtrajes": "filtraje",
    "filtramos": "filtrar",
    "filtran": "filtrar",
    "filtrando": "filtrar",
    "filtrantes": "filtrante",
    "filtrara": "filtrar",
    "filtrarais": "filtrar",
    "filtraran": "filtrar",
    "filtraras": "filtrar",
    "filtrare": "filtrar",
    "filtrareis": "filtrar",
    "filtraremos": "filtrar",
    "filtraren": "filtrar",
    "filtrares": "filtrar",
    "filtraron": "filtrar",
    "filtrará": "filtrar",
    "filtrarán": "filtrar",
    "filtrarás": "filtrar",
    "filtraré": "filtrar",
    "filtraréis": "filtrar",
    "filtraría": "filtrar",
    "filtraríais": "filtrar",
    "filtraríamos": "filtrar",
    "filtrarían": "filtrar",
    "filtrarías": "filtrar",
    "filtras": "filtrar",
    "filtrase": "filtrar",
    "filtraseis": "filtrar",
    "filtrasen": "filtrar",
    "filtrases": "filtrar",
    "filtraste": "filtrar",
    "filtrasteis": "filtrar",
    "filtre": "filtrar",
    "filtremos": "filtrar",
    "filtren": "filtrar",
    "filtres": "filtrar",
    "filtro": "filtrar",
    "filtros": "filtro",
    "filtrábamos": "filtrar",
    "filtráis": "filtrar",
    "filtráramos": "filtrar",
    "filtráremos": "filtrar",
    "filtrásemos": "filtrar",
    "filtré": "filtrar",
    "filtréis": "filtrar",
    "filtró": "filtrar",
    "filums": "filum",
    "filustres": "filustre",
    "filvanes": "filván",
    "filántropa": "filántropo",
    "filántropas": "filántropo",
    "filántropos": "filántropo",
    "filásticas": "filástica",
    "filía": "filiar",
    "filían": "filiar",
    "filías": "filiar",
    "filíe": "filiar",
    "filíen": "filiar",
    "filíes": "filiar",
    "filío": "filiar",
    "filípicas": "filípica",
    "filófaga": "filófago",
    "filófagas": "filófago",
    "filófagos": "filófago",
    "filóloga": "filólogo",
    "filólogas": "filólogo",
    "filólogos": "filólogo",
    "filósofa": "filósofo",
    "filósofas": "filósofo",
    "filósofos": "filósofo",
    "fimbriada": "fimbriado",
    "fimbriadas": "fimbriado",
    "fimbriados": "fimbriado",
    "fimbrias": "fimbria",
    "fimbrinas": "fimbrina",
    "fimias": "fimia",
    "fimos": "fimo",
    "fimícolas": "fimícola",
    "fina": "fino",
    "finaba": "finar",
    "finabais": "finar",
    "finaban": "finar",
    "finabas": "finar",
    "finad": "finar",
    "finada": "finar",
    "finadas": "finar",
    "finado": "finar",
    "finados": "finar",
    "finales": "final",
    "finalice": "finalizar",
    "finalicemos": "finalizar",
    "finalicen": "finalizar",
    "finalices": "finalizar",
    "finalicé": "finalizar",
    "finalicéis": "finalizar",
    "finalidades": "finalidad",
    "finalismos": "finalismo",
    "finalistas": "finalista",
    "finaliza": "finalizar",
    "finalizaba": "finalizar",
    "finalizabais": "finalizar",
    "finalizaban": "finalizar",
    "finalizabas": "finalizar",
    "finalizaciones": "finalización",
    "finalizad": "finalizar",
    "finalizada": "finalizar",
    "finalizadas": "finalizar",
    "finalizado": "finalizar",
    "finalizadores": "finalizador",
    "finalizados": "finalizar",
    "finalizamos": "finalizar",
    "finalizan": "finalizar",
    "finalizando": "finalizar",
    "finalizara": "finalizar",
    "finalizarais": "finalizar",
    "finalizaran": "finalizar",
    "finalizaras": "finalizar",
    "finalizare": "finalizar",
    "finalizareis": "finalizar",
    "finalizaremos": "finalizar",
    "finalizaren": "finalizar",
    "finalizares": "finalizar",
    "finalizaron": "finalizar",
    "finalizará": "finalizar",
    "finalizarán": "finalizar",
    "finalizarás": "finalizar",
    "finalizaré": "finalizar",
    "finalizaréis": "finalizar",
    "finalizaría": "finalizar",
    "finalizaríais": "finalizar",
    "finalizaríamos": "finalizar",
    "finalizarían": "finalizar",
    "finalizarías": "finalizar",
    "finalizas": "finalizar",
    "finalizase": "finalizar",
    "finalizaseis": "finalizar",
    "finalizasen": "finalizar",
    "finalizases": "finalizar",
    "finalizaste": "finalizar",
    "finalizasteis": "finalizar",
    "finalizo": "finalizar",
    "finalizábamos": "finalizar",
    "finalizáis": "finalizar",
    "finalizáramos": "finalizar",
    "finalizáremos": "finalizar",
    "finalizásemos": "finalizar",
    "finalizó": "finalizar",
    "finamos": "finar",
    "finan": "finar",
    "financia": "financiar",
    "financiaba": "financiar",
    "financiabais": "financiar",
    "financiaban": "financiar",
    "financiabas": "financiar",
    "financiables": "financiable",
    "financiaciones": "financiación",
    "financiad": "financiar",
    "financiada": "financiar",
    "financiadas": "financiar",
    "financiado": "financiar",
    "financiadora": "financiador",
    "financiadoras": "financiador",
    "financiadores": "financiador",
    "financiados": "financiar",
    "financiales": "financial",
    "financiamientos": "financiamiento",
    "financiamos": "financiar",
    "financian": "financiar",
    "financiando": "financiar",
    "financiara": "financiar",
    "financiarais": "financiar",
    "financiaran": "financiar",
    "financiaras": "financiar",
    "financiare": "financiar",
    "financiareis": "financiar",
    "financiaremos": "financiar",
    "financiaren": "financiar",
    "financiares": "financiar",
    "financiaria": "financiario",
    "financiarias": "financiario",
    "financiarios": "financiario",
    "financiaron": "financiar",
    "financiará": "financiar",
    "financiarán": "financiar",
    "financiarás": "financiar",
    "financiaré": "financiar",
    "financiaréis": "financiar",
    "financiaría": "financiar",
    "financiaríais": "financiar",
    "financiaríamos": "financiar",
    "financiarían": "financiar",
    "financiarías": "financiar",
    "financias": "financiar",
    "financiase": "financiar",
    "financiaseis": "financiar",
    "financiasen": "financiar",
    "financiases": "financiar",
    "financiaste": "financiar",
    "financiasteis": "financiar",
    "financie": "financiar",
    "financiemos": "financiar",
    "financien": "financiar",
    "financiera": "financiero",
    "financieras": "financiero",
    "financieros": "financiero",
    "financies": "financiar",
    "financio": "financiar",
    "financistas": "financista",
    "financiábamos": "financiar",
    "financiáis": "financiar",
    "financiáramos": "financiar",
    "financiáremos": "financiar",
    "financiásemos": "financiar",
    "financié": "financiar",
    "financiéis": "financiar",
    "financió": "financiar",
    "finando": "finar",
    "finara": "finar",
    "finarais": "finar",
    "finaran": "finar",
    "finaras": "finar",
    "finare": "finar",
    "finareis": "finar",
    "finaremos": "finar",
    "finaren": "finar",
    "finares": "finar",
    "finaron": "finar",
    "finará": "finar",
    "finarán": "finar",
    "finarás": "finar",
    "finaré": "finar",
    "finaréis": "finar",
    "finaría": "finar",
    "finaríais": "finar",
    "finaríamos": "finar",
    "finarían": "finar",
    "finarías": "finar",
    "finas": "fino",
    "finase": "finar",
    "finaseis": "finar",
    "finasen": "finar",
    "finases": "finar",
    "finaste": "finar",
    "finasteis": "finar",
    "fincas": "finca",
    "finchada": "finchado",
    "finchadas": "finchado",
    "finchados": "finchado",
    "fine": "finar",
    "finemos": "finar",
    "finen": "finar",
    "fines": "finar",
    "finesa": "finés",
    "finesas": "finés",
    "fineses": "finés",
    "finetas": "fineta",
    "finezas": "fineza",
    "finge": "fingir",
    "fingen": "fingir",
    "finges": "fingir",
    "fingid": "fingir",
    "fingida": "fingir",
    "fingidas": "fingir",
    "fingido": "fingir",
    "fingidora": "fingidor",
    "fingidoras": "fingidor",
    "fingidores": "fingidor",
    "fingidos": "fingir",
    "fingiendo": "fingir",
    "fingiera": "fingir",
    "fingierais": "fingir",
    "fingieran": "fingir",
    "fingieras": "fingir",
    "fingiere": "fingir",
    "fingiereis": "fingir",
    "fingieren": "fingir",
    "fingieres": "fingir",
    "fingieron": "fingir",
    "fingiese": "fingir",
    "fingieseis": "fingir",
    "fingiesen": "fingir",
    "fingieses": "fingir",
    "fingimientos": "fingimiento",
    "fingimos": "fingir",
    "fingiremos": "fingir",
    "fingirá": "fingir",
    "fingirán": "fingir",
    "fingirás": "fingir",
    "fingiré": "fingir",
    "fingiréis": "fingir",
    "fingiría": "fingir",
    "fingiríais": "fingir",
    "fingiríamos": "fingir",
    "fingirían": "fingir",
    "fingirías": "fingir",
    "fingiste": "fingir",
    "fingisteis": "fingir",
    "fingiéramos": "fingir",
    "fingiéremos": "fingir",
    "fingiésemos": "fingir",
    "fingió": "fingir",
    "fingí": "fingir",
    "fingía": "fingir",
    "fingíais": "fingir",
    "fingíamos": "fingir",
    "fingían": "fingir",
    "fingías": "fingir",
    "fingís": "fingir",
    "finibusterres": "finibusterre",
    "finiquita": "finiquitar",
    "finiquitaba": "finiquitar",
    "finiquitabais": "finiquitar",
    "finiquitaban": "finiquitar",
    "finiquitabas": "finiquitar",
    "finiquitad": "finiquitar",
    "finiquitada": "finiquitar",
    "finiquitadas": "finiquitar",
    "finiquitado": "finiquitar",
    "finiquitados": "finiquitar",
    "finiquitamos": "finiquitar",
    "finiquitan": "finiquitar",
    "finiquitando": "finiquitar",
    "finiquitara": "finiquitar",
    "finiquitarais": "finiquitar",
    "finiquitaran": "finiquitar",
    "finiquitaras": "finiquitar",
    "finiquitare": "finiquitar",
    "finiquitareis": "finiquitar",
    "finiquitaremos": "finiquitar",
    "finiquitaren": "finiquitar",
    "finiquitares": "finiquitar",
    "finiquitaron": "finiquitar",
    "finiquitará": "finiquitar",
    "finiquitarán": "finiquitar",
    "finiquitarás": "finiquitar",
    "finiquitaré": "finiquitar",
    "finiquitaréis": "finiquitar",
    "finiquitaría": "finiquitar",
    "finiquitaríais": "finiquitar",
    "finiquitaríamos": "finiquitar",
    "finiquitarían": "finiquitar",
    "finiquitarías": "finiquitar",
    "finiquitas": "finiquitar",
    "finiquitase": "finiquitar",
    "finiquitaseis": "finiquitar",
    "finiquitasen": "finiquitar",
    "finiquitases": "finiquitar",
    "finiquitaste": "finiquitar",
    "finiquitasteis": "finiquitar",
    "finiquite": "finiquitar",
    "finiquitemos": "finiquitar",
    "finiquiten": "finiquitar",
    "finiquites": "finiquitar",
    "finiquito": "finiquitar",
    "finiquitos": "finiquito",
    "finiquitábamos": "finiquitar",
    "finiquitáis": "finiquitar",
    "finiquitáramos": "finiquitar",
    "finiquitáremos": "finiquitar",
    "finiquitásemos": "finiquitar",
    "finiquité": "finiquitar",
    "finiquitéis": "finiquitar",
    "finiquitó": "finiquitar",
    "finiseculares": "finisecular",
    "finita": "finito",
    "finitas": "finito",
    "finitos": "finito",
    "finitudes": "finitud",
    "finja": "fingir",
    "finjamos": "fingir",
    "finjan": "fingir",
    "finjas": "fingir",
    "finjo": "fingir",
    "finjáis": "fingir",
    "finlandesa": "finlandés",
    "finlandesas": "finlandés",
    "finlandeses": "finlandés",
    "finlandésa": "finlandés",
    "finlandésas": "finlandés",
    "finlandéses": "finlandés",
    "fino": "finar",
    "finos": "fino",
    "finougria": "finougrio",
    "finougrias": "finougrio",
    "finougrios": "finougrio",
    "finta": "fintar",
    "fintaba": "fintar",
    "fintabais": "fintar",
    "fintaban": "fintar",
    "fintabas": "fintar",
    "fintad": "fintar",
    "fintada": "fintar",
    "fintadas": "fintar",
    "fintado": "fintar",
    "fintadora": "fintador",
    "fintadoras": "fintador",
    "fintadores": "fintador",
    "fintados": "fintar",
    "fintamos": "fintar",
    "fintan": "fintar",
    "fintando": "fintar",
    "fintara": "fintar",
    "fintarais": "fintar",
    "fintaran": "fintar",
    "fintaras": "fintar",
    "fintare": "fintar",
    "fintareis": "fintar",
    "fintaremos": "fintar",
    "fintaren": "fintar",
    "fintares": "fintar",
    "fintaron": "fintar",
    "fintará": "fintar",
    "fintarán": "fintar",
    "fintarás": "fintar",
    "fintaré": "fintar",
    "fintaréis": "fintar",
    "fintaría": "fintar",
    "fintaríais": "fintar",
    "fintaríamos": "fintar",
    "fintarían": "fintar",
    "fintarías": "fintar",
    "fintas": "fintar",
    "fintase": "fintar",
    "fintaseis": "fintar",
    "fintasen": "fintar",
    "fintases": "fintar",
    "fintaste": "fintar",
    "fintasteis": "fintar",
    "finte": "fintar",
    "fintea": "fintear",
    "finteaba": "fintear",
    "finteabais": "fintear",
    "finteaban": "fintear",
    "finteabas": "fintear",
    "fintead": "fintear",
    "finteada": "fintear",
    "finteadas": "fintear",
    "finteado": "fintear",
    "finteados": "fintear",
    "finteamos": "fintear",
    "fintean": "fintear",
    "finteando": "fintear",
    "finteara": "fintear",
    "fintearais": "fintear",
    "fintearan": "fintear",
    "fintearas": "fintear",
    "finteare": "fintear",
    "finteareis": "fintear",
    "fintearemos": "fintear",
    "fintearen": "fintear",
    "finteares": "fintear",
    "fintearon": "fintear",
    "finteará": "fintear",
    "fintearán": "fintear",
    "fintearás": "fintear",
    "fintearé": "fintear",
    "fintearéis": "fintear",
    "fintearía": "fintear",
    "fintearíais": "fintear",
    "fintearíamos": "fintear",
    "fintearían": "fintear",
    "fintearías": "fintear",
    "finteas": "fintear",
    "fintease": "fintear",
    "finteaseis": "fintear",
    "finteasen": "fintear",
    "finteases": "fintear",
    "finteaste": "fintear",
    "finteasteis": "fintear",
    "fintee": "fintear",
    "finteemos": "fintear",
    "finteen": "fintear",
    "fintees": "fintear",
    "fintemos": "fintar",
    "finten": "fintar",
    "finteo": "fintear",
    "fintes": "fintar",
    "finteábamos": "fintear",
    "finteáis": "fintear",
    "finteáramos": "fintear",
    "finteáremos": "fintear",
    "finteásemos": "fintear",
    "finteé": "fintear",
    "finteéis": "fintear",
    "finteó": "fintear",
    "finto": "fintar",
    "fintábamos": "fintar",
    "fintáis": "fintar",
    "fintáramos": "fintar",
    "fintáremos": "fintar",
    "fintásemos": "fintar",
    "finté": "fintar",
    "fintéis": "fintar",
    "fintó": "fintar",
    "finuras": "finura",
    "finustiquerías": "finustiquería",
    "finábamos": "finar",
    "fináis": "finar",
    "fináramos": "finar",
    "fináremos": "finar",
    "finásemos": "finar",
    "finé": "finar",
    "finéis": "finar",
    "finésa": "finés",
    "finésas": "finés",
    "finéses": "finés",
    "finítima": "finítimo",
    "finítimas": "finítimo",
    "finítimos": "finítimo",
    "finó": "finar",
    "finústica": "finústico",
    "finústicas": "finústico",
    "finústicos": "finústico",
    "fiocos": "fioco",
    "fiordos": "fiordo",
    "firma": "firmar",
    "firmaba": "firmar",
    "firmabais": "firmar",
    "firmaban": "firmar",
    "firmabas": "firmar",
    "firmad": "firmar",
    "firmada": "firmar",
    "firmadas": "firmar",
    "firmado": "firmar",
    "firmados": "firmar",
    "firmales": "firmal",
    "firmamentos": "firmamento",
    "firmamos": "firmar",
    "firman": "firmar",
    "firmando": "firmar",
    "firmanes": "firmán",
    "firmantes": "firmante",
    "firmara": "firmar",
    "firmarais": "firmar",
    "firmaran": "firmar",
    "firmaras": "firmar",
    "firmare": "firmar",
    "firmareis": "firmar",
    "firmaremos": "firmar",
    "firmaren": "firmar",
    "firmares": "firmar",
    "firmaron": "firmar",
    "firmará": "firmar",
    "firmarán": "firmar",
    "firmarás": "firmar",
    "firmaré": "firmar",
    "firmaréis": "firmar",
    "firmaría": "firmar",
    "firmaríais": "firmar",
    "firmaríamos": "firmar",
    "firmarían": "firmar",
    "firmarías": "firmar",
    "firmas": "firmar",
    "firmase": "firmar",
    "firmaseis": "firmar",
    "firmasen": "firmar",
    "firmases": "firmar",
    "firmaste": "firmar",
    "firmasteis": "firmar",
    "firme": "firmar",
    "firmemos": "firmar",
    "firmen": "firmar",
    "firmes": "firme",
    "firmezas": "firmeza",
    "firmo": "firmar",
    "firmones": "firmón",
    "firmábamos": "firmar",
    "firmáis": "firmar",
    "firmáramos": "firmar",
    "firmáremos": "firmar",
    "firmásemos": "firmar",
    "firmé": "firmar",
    "firméis": "firmar",
    "firmó": "firmar",
    "fis": "fi",
    "fiscales": "fiscal",
    "fiscalice": "fiscalizar",
    "fiscalicemos": "fiscalizar",
    "fiscalicen": "fiscalizar",
    "fiscalices": "fiscalizar",
    "fiscalicé": "fiscalizar",
    "fiscalicéis": "fiscalizar",
    "fiscalidades": "fiscalidad",
    "fiscalistas": "fiscalista",
    "fiscaliza": "fiscalizar",
    "fiscalizaba": "fiscalizar",
    "fiscalizabais": "fiscalizar",
    "fiscalizaban": "fiscalizar",
    "fiscalizabas": "fiscalizar",
    "fiscalizaciones": "fiscalización",
    "fiscalizad": "fiscalizar",
    "fiscalizada": "fiscalizar",
    "fiscalizadas": "fiscalizar",
    "fiscalizado": "fiscalizar",
    "fiscalizadora": "fiscalizador",
    "fiscalizadoras": "fiscalizador",
    "fiscalizadores": "fiscalizador",
    "fiscalizados": "fiscalizar",
    "fiscalizamos": "fiscalizar",
    "fiscalizan": "fiscalizar",
    "fiscalizando": "fiscalizar",
    "fiscalizara": "fiscalizar",
    "fiscalizarais": "fiscalizar",
    "fiscalizaran": "fiscalizar",
    "fiscalizaras": "fiscalizar",
    "fiscalizare": "fiscalizar",
    "fiscalizareis": "fiscalizar",
    "fiscalizaremos": "fiscalizar",
    "fiscalizaren": "fiscalizar",
    "fiscalizares": "fiscalizar",
    "fiscalizaron": "fiscalizar",
    "fiscalizará": "fiscalizar",
    "fiscalizarán": "fiscalizar",
    "fiscalizarás": "fiscalizar",
    "fiscalizaré": "fiscalizar",
    "fiscalizaréis": "fiscalizar",
    "fiscalizaría": "fiscalizar",
    "fiscalizaríais": "fiscalizar",
    "fiscalizaríamos": "fiscalizar",
    "fiscalizarían": "fiscalizar",
    "fiscalizarías": "fiscalizar",
    "fiscalizas": "fiscalizar",
    "fiscalizase": "fiscalizar",
    "fiscalizaseis": "fiscalizar",
    "fiscalizasen": "fiscalizar",
    "fiscalizases": "fiscalizar",
    "fiscalizaste": "fiscalizar",
    "fiscalizasteis": "fiscalizar",
    "fiscalizo": "fiscalizar",
    "fiscalizábamos": "fiscalizar",
    "fiscalizáis": "fiscalizar",
    "fiscalizáramos": "fiscalizar",
    "fiscalizáremos": "fiscalizar",
    "fiscalizásemos": "fiscalizar",
    "fiscalizó": "fiscalizar",
    "fiscalías": "fiscalía",
    "fiscornos": "fiscorno",
    "fiscos": "fisco",
    "fisga": "fisgar",
    "fisgaba": "fisgar",
    "fisgabais": "fisgar",
    "fisgaban": "fisgar",
    "fisgabas": "fisgar",
    "fisgad": "fisgar",
    "fisgada": "fisgar",
    "fisgadas": "fisgar",
    "fisgado": "fisgar",
    "fisgados": "fisgar",
    "fisgamos": "fisgar",
    "fisgan": "fisgar",
    "fisgando": "fisgar",
    "fisgara": "fisgar",
    "fisgarais": "fisgar",
    "fisgaran": "fisgar",
    "fisgaras": "fisgar",
    "fisgare": "fisgar",
    "fisgareis": "fisgar",
    "fisgaremos": "fisgar",
    "fisgaren": "fisgar",
    "fisgares": "fisgar",
    "fisgaron": "fisgar",
    "fisgará": "fisgar",
    "fisgarán": "fisgar",
    "fisgarás": "fisgar",
    "fisgaré": "fisgar",
    "fisgaréis": "fisgar",
    "fisgaría": "fisgar",
    "fisgaríais": "fisgar",
    "fisgaríamos": "fisgar",
    "fisgarían": "fisgar",
    "fisgarías": "fisgar",
    "fisgas": "fisgar",
    "fisgase": "fisgar",
    "fisgaseis": "fisgar",
    "fisgasen": "fisgar",
    "fisgases": "fisgar",
    "fisgaste": "fisgar",
    "fisgasteis": "fisgar",
    "fisgo": "fisgar",
    "fisgona": "fisgón",
    "fisgonas": "fisgón",
    "fisgonea": "fisgonear",
    "fisgoneaba": "fisgonear",
    "fisgoneabais": "fisgonear",
    "fisgoneaban": "fisgonear",
    "fisgoneabas": "fisgonear",
    "fisgonead": "fisgonear",
    "fisgoneada": "fisgonear",
    "fisgoneadas": "fisgonear",
    "fisgoneado": "fisgonear",
    "fisgoneados": "fisgonear",
    "fisgoneamos": "fisgonear",
    "fisgonean": "fisgonear",
    "fisgoneando": "fisgonear",
    "fisgoneara": "fisgonear",
    "fisgonearais": "fisgonear",
    "fisgonearan": "fisgonear",
    "fisgonearas": "fisgonear",
    "fisgoneare": "fisgonear",
    "fisgoneareis": "fisgonear",
    "fisgonearemos": "fisgonear",
    "fisgonearen": "fisgonear",
    "fisgoneares": "fisgonear",
    "fisgonearon": "fisgonear",
    "fisgoneará": "fisgonear",
    "fisgonearán": "fisgonear",
    "fisgonearás": "fisgonear",
    "fisgonearé": "fisgonear",
    "fisgonearéis": "fisgonear",
    "fisgonearía": "fisgonear",
    "fisgonearíais": "fisgonear",
    "fisgonearíamos": "fisgonear",
    "fisgonearían": "fisgonear",
    "fisgonearías": "fisgonear",
    "fisgoneas": "fisgonear",
    "fisgonease": "fisgonear",
    "fisgoneaseis": "fisgonear",
    "fisgoneasen": "fisgonear",
    "fisgoneases": "fisgonear",
    "fisgoneaste": "fisgonear",
    "fisgoneasteis": "fisgonear",
    "fisgonee": "fisgonear",
    "fisgoneemos": "fisgonear",
    "fisgoneen": "fisgonear",
    "fisgonees": "fisgonear",
    "fisgoneo": "fisgonear",
    "fisgoneos": "fisgoneo",
    "fisgones": "fisgón",
    "fisgoneábamos": "fisgonear",
    "fisgoneáis": "fisgonear",
    "fisgoneáramos": "fisgonear",
    "fisgoneáremos": "fisgonear",
    "fisgoneásemos": "fisgonear",
    "fisgoneé": "fisgonear",
    "fisgoneéis": "fisgonear",
    "fisgoneó": "fisgonear",
    "fisgue": "fisgar",
    "fisguemos": "fisgar",
    "fisguen": "fisgar",
    "fisgues": "fisgar",
    "fisgué": "fisgar",
    "fisguéis": "fisgar",
    "fisgábamos": "fisgar",
    "fisgáis": "fisgar",
    "fisgáramos": "fisgar",
    "fisgáremos": "fisgar",
    "fisgásemos": "fisgar",
    "fisgó": "fisgar",
    "fisiatrías": "fisiatría",
    "fisibles": "fisible",
    "fisicalismos": "fisicalismo",
    "fisicoquímica": "fisicoquímico",
    "fisicoquímicas": "fisicoquímico",
    "fisicoquímicos": "fisicoquímico",
    "fisiocracias": "fisiocracia",
    "fisiocromatinas": "fisiocromatina",
    "fisiocrática": "fisiocrático",
    "fisiocráticas": "fisiocrático",
    "fisiocráticos": "fisiocrático",
    "fisiognomías": "fisiognomía",
    "fisiografías": "fisiografía",
    "fisiográfica": "fisiográfico",
    "fisiográficas": "fisiográfico",
    "fisiográficos": "fisiográfico",
    "fisiologías": "fisiología",
    "fisiológica": "fisiológico",
    "fisiológicas": "fisiológico",
    "fisiológicos": "fisiológico",
    "fisiona": "fisionar",
    "fisionaba": "fisionar",
    "fisionabais": "fisionar",
    "fisionaban": "fisionar",
    "fisionabas": "fisionar",
    "fisionables": "fisionable",
    "fisionad": "fisionar",
    "fisionada": "fisionar",
    "fisionadas": "fisionar",
    "fisionado": "fisionar",
    "fisionados": "fisionar",
    "fisionamos": "fisionar",
    "fisionan": "fisionar",
    "fisionando": "fisionar",
    "fisionara": "fisionar",
    "fisionarais": "fisionar",
    "fisionaran": "fisionar",
    "fisionaras": "fisionar",
    "fisionare": "fisionar",
    "fisionareis": "fisionar",
    "fisionaremos": "fisionar",
    "fisionaren": "fisionar",
    "fisionares": "fisionar",
    "fisionaron": "fisionar",
    "fisionará": "fisionar",
    "fisionarán": "fisionar",
    "fisionarás": "fisionar",
    "fisionaré": "fisionar",
    "fisionaréis": "fisionar",
    "fisionaría": "fisionar",
    "fisionaríais": "fisionar",
    "fisionaríamos": "fisionar",
    "fisionarían": "fisionar",
    "fisionarías": "fisionar",
    "fisionas": "fisionar",
    "fisionase": "fisionar",
    "fisionaseis": "fisionar",
    "fisionasen": "fisionar",
    "fisionases": "fisionar",
    "fisionaste": "fisionar",
    "fisionasteis": "fisionar",
    "fisione": "fisionar",
    "fisionemos": "fisionar",
    "fisionen": "fisionar",
    "fisiones": "fisionar",
    "fisiono": "fisionar",
    "fisionomías": "fisionomía",
    "fisionábamos": "fisionar",
    "fisionáis": "fisionar",
    "fisionáramos": "fisionar",
    "fisionáremos": "fisionar",
    "fisionásemos": "fisionar",
    "fisioné": "fisionar",
    "fisionéis": "fisionar",
    "fisionó": "fisionar",
    "fisionómica": "fisionómico",
    "fisionómicas": "fisionómico",
    "fisionómicos": "fisionómico",
    "fisiopatologías": "fisiopatología",
    "fisiopatológica": "fisiopatológico",
    "fisiopatológicas": "fisiopatológico",
    "fisiopatológicos": "fisiopatológico",
    "fisioquímicas": "fisioquímica",
    "fisioterapeutas": "fisioterapeuta",
    "fisioterapias": "fisioterapia",
    "fisioterapistas": "fisioterapista",
    "fisioterápica": "fisioterápico",
    "fisioterápicas": "fisioterápico",
    "fisioterápicos": "fisioterápico",
    "fisirrostra": "fisirrostro",
    "fisirrostras": "fisirrostro",
    "fisirrostros": "fisirrostro",
    "fisisorciones": "fisisorción",
    "fisiátrica": "fisiátrico",
    "fisiátricas": "fisiátrico",
    "fisiátricos": "fisiátrico",
    "fisiócratas": "fisiócrata",
    "fisióloga": "fisiólogo",
    "fisiólogas": "fisiólogo",
    "fisiólogos": "fisiólogo",
    "fisonomistas": "fisonomista",
    "fisonomías": "fisonomía",
    "fisostigminas": "fisostigmina",
    "fistulares": "fistular",
    "fistuloplastias": "fistuloplastia",
    "fistulosa": "fistuloso",
    "fistulosas": "fistuloso",
    "fistulosos": "fistuloso",
    "fisurabilidades": "fisurabilidad",
    "fisuraciones": "fisuración",
    "fisurantes": "fisurante",
    "fisuras": "fisura",
    "fisípedos": "fisípedo",
    "fisónoma": "fisónomo",
    "fisónomas": "fisónomo",
    "fisónomos": "fisónomo",
    "fisóstomos": "fisóstomo",
    "fitoaglutininas": "fitoaglutinina",
    "fitofenologías": "fitofenología",
    "fitofármacos": "fitofármaco",
    "fitogenética": "fitogenético",
    "fitogenéticas": "fitogenético",
    "fitogenéticos": "fitogenético",
    "fitogeografías": "fitogeografía",
    "fitografías": "fitografía",
    "fitográfica": "fitográfico",
    "fitográficas": "fitográfico",
    "fitográficos": "fitográfico",
    "fitohemaglutininas": "fitohemaglutinina",
    "fitohormonas": "fitohormona",
    "fitolacáceas": "fitolacácea",
    "fitoles": "fitol",
    "fitologías": "fitología",
    "fitonadionas": "fitonadiona",
    "fitoparásitos": "fitoparásito",
    "fitopatologías": "fitopatología",
    "fitopatológica": "fitopatológico",
    "fitopatológicas": "fitopatológico",
    "fitopatológicos": "fitopatológico",
    "fitopatógena": "fitopatógeno",
    "fitopatógenas": "fitopatógeno",
    "fitopatógenos": "fitopatógeno",
    "fitopatóloga": "fitopatólogo",
    "fitopatólogas": "fitopatólogo",
    "fitopatólogos": "fitopatólogo",
    "fitoplanctónica": "fitoplanctónico",
    "fitoplanctónicas": "fitoplanctónico",
    "fitoplanctónicos": "fitoplanctónico",
    "fitoplánctones": "fitoplancton",
    "fitoquímica": "fitoquímico",
    "fitoquímicas": "fitoquímico",
    "fitoquímicos": "fitoquímico",
    "fitorreguladora": "fitorregulador",
    "fitorreguladoras": "fitorregulador",
    "fitorreguladores": "fitorregulador",
    "fitosanitaria": "fitosanitario",
    "fitosanitarias": "fitosanitario",
    "fitosanitarios": "fitosanitario",
    "fitosociologías": "fitosociología",
    "fitosteroles": "fitosterol",
    "fitotomías": "fitotomía",
    "fitotoxicidades": "fitotoxicidad",
    "fitotrones": "fitotrón",
    "fitófaga": "fitófago",
    "fitófagas": "fitófago",
    "fitófagos": "fitófago",
    "fitógrafa": "fitógrafo",
    "fitógrafas": "fitógrafo",
    "fitógrafos": "fitógrafo",
    "fiábamos": "fiar",
    "fiáis": "fiar",
    "fiáramos": "fiar",
    "fiáremos": "fiar",
    "fiásemos": "fiar",
    "fié": "fiar",
    "fiéis": "fiar",
    "fiñanas": "fiñana",
    "fió": "fiar",
    "flabelicornia": "flabelicornio",
    "flabelicornias": "flabelicornio",
    "flabelicornios": "flabelicornio",
    "flabeliformes": "flabeliforme",
    "flabelos": "flabelo",
    "flaberífera": "flaberífero",
    "flaberíferas": "flaberífero",
    "flaberíferos": "flaberífero",
    "flaca": "flaco",
    "flacas": "flaco",
    "flaccideces": "flaccidez",
    "flacideces": "flacidez",
    "flacos": "flaco",
    "flacucha": "flacucho",
    "flacuchas": "flacucho",
    "flacuchos": "flacucho",
    "flacuras": "flacura",
    "flacurtiáceas": "flacurtiácea",
    "flagela": "flagelar",
    "flagelaba": "flagelar",
    "flagelabais": "flagelar",
    "flagelaban": "flagelar",
    "flagelabas": "flagelar",
    "flagelaciones": "flagelación",
    "flagelad": "flagelar",
    "flagelada": "flagelar",
    "flageladas": "flagelar",
    "flagelado": "flagelar",
    "flageladora": "flagelador",
    "flageladoras": "flagelador",
    "flageladores": "flagelador",
    "flagelados": "flagelar",
    "flagelamos": "flagelar",
    "flagelan": "flagelar",
    "flagelando": "flagelar",
    "flagelantes": "flagelante",
    "flagelara": "flagelar",
    "flagelarais": "flagelar",
    "flagelaran": "flagelar",
    "flagelaras": "flagelar",
    "flagelare": "flagelar",
    "flagelareis": "flagelar",
    "flagelaremos": "flagelar",
    "flagelaren": "flagelar",
    "flagelares": "flagelar",
    "flagelaron": "flagelar",
    "flagelará": "flagelar",
    "flagelarán": "flagelar",
    "flagelarás": "flagelar",
    "flagelaré": "flagelar",
    "flagelaréis": "flagelar",
    "flagelaría": "flagelar",
    "flagelaríais": "flagelar",
    "flagelaríamos": "flagelar",
    "flagelarían": "flagelar",
    "flagelarías": "flagelar",
    "flagelas": "flagelar",
    "flagelase": "flagelar",
    "flagelaseis": "flagelar",
    "flagelasen": "flagelar",
    "flagelases": "flagelar",
    "flagelaste": "flagelar",
    "flagelasteis": "flagelar",
    "flagele": "flagelar",
    "flagelemos": "flagelar",
    "flagelen": "flagelar",
    "flageles": "flagelar",
    "flagelinas": "flagelina",
    "flagelo": "flagelar",
    "flagelos": "flagelo",
    "flagelábamos": "flagelar",
    "flageláis": "flagelar",
    "flageláramos": "flagelar",
    "flageláremos": "flagelar",
    "flagelásemos": "flagelar",
    "flagelé": "flagelar",
    "flageléis": "flagelar",
    "flageló": "flagelar",
    "flagrancias": "flagrancia",
    "flagrantes": "flagrante",
    "flags": "flag",
    "flamantes": "flamante",
    "flamas": "flama",
    "flamboyanes": "flamboyán",
    "flamea": "flamear",
    "flameaba": "flamear",
    "flameabais": "flamear",
    "flameaban": "flamear",
    "flameabas": "flamear",
    "flamead": "flamear",
    "flameada": "flamear",
    "flameadas": "flamear",
    "flameado": "flamear",
    "flameados": "flamear",
    "flameamos": "flamear",
    "flamean": "flamear",
    "flameando": "flamear",
    "flameara": "flamear",
    "flamearais": "flamear",
    "flamearan": "flamear",
    "flamearas": "flamear",
    "flameare": "flamear",
    "flameareis": "flamear",
    "flamearemos": "flamear",
    "flamearen": "flamear",
    "flameares": "flamear",
    "flamearon": "flamear",
    "flameará": "flamear",
    "flamearán": "flamear",
    "flamearás": "flamear",
    "flamearé": "flamear",
    "flamearéis": "flamear",
    "flamearía": "flamear",
    "flamearíais": "flamear",
    "flamearíamos": "flamear",
    "flamearían": "flamear",
    "flamearías": "flamear",
    "flameas": "flamear",
    "flamease": "flamear",
    "flameaseis": "flamear",
    "flameasen": "flamear",
    "flameases": "flamear",
    "flameaste": "flamear",
    "flameasteis": "flamear",
    "flamee": "flamear",
    "flameemos": "flamear",
    "flameen": "flamear",
    "flamees": "flamear",
    "flamenca": "flamenco",
    "flamencas": "flamenco",
    "flamencos": "flamenco",
    "flamenquerías": "flamenquería",
    "flamenquismos": "flamenquismo",
    "flameo": "flamear",
    "flameos": "flameo",
    "flameros": "flamero",
    "flameábamos": "flamear",
    "flameáis": "flamear",
    "flameáramos": "flamear",
    "flameáremos": "flamear",
    "flameásemos": "flamear",
    "flameé": "flamear",
    "flameéis": "flamear",
    "flameó": "flamear",
    "flamígera": "flamígero",
    "flamígeras": "flamígero",
    "flamígeros": "flamígero",
    "flancos": "flanco",
    "flanera": "flanero",
    "flaneras": "flanero",
    "flaneros": "flanero",
    "flanes": "flan",
    "flanquea": "flanquear",
    "flanqueaba": "flanquear",
    "flanqueabais": "flanquear",
    "flanqueaban": "flanquear",
    "flanqueabas": "flanquear",
    "flanquead": "flanquear",
    "flanqueada": "flanquear",
    "flanqueadas": "flanquear",
    "flanqueado": "flanquear",
    "flanqueados": "flanquear",
    "flanqueamos": "flanquear",
    "flanquean": "flanquear",
    "flanqueando": "flanquear",
    "flanqueara": "flanquear",
    "flanquearais": "flanquear",
    "flanquearan": "flanquear",
    "flanquearas": "flanquear",
    "flanqueare": "flanquear",
    "flanqueareis": "flanquear",
    "flanquearemos": "flanquear",
    "flanquearen": "flanquear",
    "flanqueares": "flanquear",
    "flanquearon": "flanquear",
    "flanqueará": "flanquear",
    "flanquearán": "flanquear",
    "flanquearás": "flanquear",
    "flanquearé": "flanquear",
    "flanquearéis": "flanquear",
    "flanquearía": "flanquear",
    "flanquearíais": "flanquear",
    "flanquearíamos": "flanquear",
    "flanquearían": "flanquear",
    "flanquearías": "flanquear",
    "flanqueas": "flanquear",
    "flanquease": "flanquear",
    "flanqueaseis": "flanquear",
    "flanqueasen": "flanquear",
    "flanqueases": "flanquear",
    "flanqueaste": "flanquear",
    "flanqueasteis": "flanquear",
    "flanquee": "flanquear",
    "flanqueemos": "flanquear",
    "flanqueen": "flanquear",
    "flanquees": "flanquear",
    "flanqueo": "flanquear",
    "flanqueos": "flanqueo",
    "flanqueábamos": "flanquear",
    "flanqueáis": "flanquear",
    "flanqueáramos": "flanquear",
    "flanqueáremos": "flanquear",
    "flanqueásemos": "flanquear",
    "flanqueé": "flanquear",
    "flanqueéis": "flanquear",
    "flanqueó": "flanquear",
    "flaps": "flap",
    "flaquea": "flaquear",
    "flaqueaba": "flaquear",
    "flaqueabais": "flaquear",
    "flaqueaban": "flaquear",
    "flaqueabas": "flaquear",
    "flaquead": "flaquear",
    "flaqueada": "flaquear",
    "flaqueadas": "flaquear",
    "flaqueado": "flaquear",
    "flaqueados": "flaquear",
    "flaqueamos": "flaquear",
    "flaquean": "flaquear",
    "flaqueando": "flaquear",
    "flaqueara": "flaquear",
    "flaquearais": "flaquear",
    "flaquearan": "flaquear",
    "flaquearas": "flaquear",
    "flaqueare": "flaquear",
    "flaqueareis": "flaquear",
    "flaquearemos": "flaquear",
    "flaquearen": "flaquear",
    "flaqueares": "flaquear",
    "flaquearon": "flaquear",
    "flaqueará": "flaquear",
    "flaquearán": "flaquear",
    "flaquearás": "flaquear",
    "flaquearé": "flaquear",
    "flaquearéis": "flaquear",
    "flaquearía": "flaquear",
    "flaquearíais": "flaquear",
    "flaquearíamos": "flaquear",
    "flaquearían": "flaquear",
    "flaquearías": "flaquear",
    "flaqueas": "flaquear",
    "flaquease": "flaquear",
    "flaqueaseis": "flaquear",
    "flaqueasen": "flaquear",
    "flaqueases": "flaquear",
    "flaqueaste": "flaquear",
    "flaqueasteis": "flaquear",
    "flaquee": "flaquear",
    "flaqueemos": "flaquear",
    "flaqueen": "flaquear",
    "flaquees": "flaquear",
    "flaqueo": "flaquear",
    "flaqueos": "flaqueo",
    "flaquezas": "flaqueza",
    "flaqueábamos": "flaquear",
    "flaqueáis": "flaquear",
    "flaqueáramos": "flaquear",
    "flaqueáremos": "flaquear",
    "flaqueásemos": "flaquear",
    "flaqueé": "flaquear",
    "flaqueéis": "flaquear",
    "flaqueó": "flaquear",
    "flash-backs": "flash-back",
    "flashbacks": "flashback",
    "flashes": "flash",
    "flatos": "flato",
    "flatosa": "flatoso",
    "flatosas": "flatoso",
    "flatosos": "flatoso",
    "flatulencias": "flatulencia",
    "flatulenta": "flatulento",
    "flatulentas": "flatulento",
    "flatulentos": "flatulento",
    "flautada": "flautado",
    "flautadas": "flautado",
    "flautados": "flautado",
    "flautas": "flauta",
    "flauteada": "flauteado",
    "flauteadas": "flauteado",
    "flauteados": "flauteado",
    "flautera": "flautero",
    "flauteras": "flautero",
    "flauteros": "flautero",
    "flautillos": "flautillo",
    "flautines": "flautín",
    "flautistas": "flautista",
    "flava": "flavo",
    "flavantrenos": "flavantreno",
    "flavas": "flavo",
    "flavinas": "flavina",
    "flavonas": "flavona",
    "flavoproteínas": "flavoproteína",
    "flavos": "flavo",
    "flebectomías": "flebectomía",
    "flebogramas": "flebograma",
    "flebotomiana": "flebotomiano",
    "flebotomianas": "flebotomiano",
    "flebotomianos": "flebotomiano",
    "flebotomías": "flebotomía",
    "flecha": "flechar",
    "flechaba": "flechar",
    "flechabais": "flechar",
    "flechaban": "flechar",
    "flechabas": "flechar",
    "flechad": "flechar",
    "flechada": "flechar",
    "flechadas": "flechar",
    "flechado": "flechar",
    "flechados": "flechar",
    "flechaduras": "flechadura",
    "flechamos": "flechar",
    "flechan": "flechar",
    "flechando": "flechar",
    "flechara": "flechar",
    "flecharais": "flechar",
    "flecharan": "flechar",
    "flecharas": "flechar",
    "flechare": "flechar",
    "flechareis": "flechar",
    "flecharemos": "flechar",
    "flecharen": "flechar",
    "flechares": "flechar",
    "flecharon": "flechar",
    "flechará": "flechar",
    "flecharán": "flechar",
    "flecharás": "flechar",
    "flecharé": "flechar",
    "flecharéis": "flechar",
    "flecharía": "flechar",
    "flecharíais": "flechar",
    "flecharíamos": "flechar",
    "flecharían": "flechar",
    "flecharías": "flechar",
    "flechas": "flechar",
    "flechase": "flechar",
    "flechaseis": "flechar",
    "flechasen": "flechar",
    "flechases": "flechar",
    "flechaste": "flechar",
    "flechasteis": "flechar",
    "flechastes": "flechaste",
    "flechazos": "flechazo",
    "fleche": "flechar",
    "flechemos": "flechar",
    "flechen": "flechar",
    "flechera": "flechero",
    "flecheras": "flechero",
    "flecheros": "flechero",
    "flecherías": "flechería",
    "fleches": "flechar",
    "flecho": "flechar",
    "flechábamos": "flechar",
    "flecháis": "flechar",
    "flecháramos": "flechar",
    "flecháremos": "flechar",
    "flechásemos": "flechar",
    "fleché": "flechar",
    "flechéis": "flechar",
    "flechó": "flechar",
    "flecos": "fleco",
    "flectora": "flector",
    "flectoras": "flector",
    "flectores": "flector",
    "flegmasías": "flegmasía",
    "flejes": "fleje",
    "flemas": "flema",
    "flemes": "fleme",
    "flemones": "flemón",
    "flemosa": "flemoso",
    "flemosas": "flemoso",
    "flemosos": "flemoso",
    "flemuda": "flemudo",
    "flemudas": "flemudo",
    "flemudos": "flemudo",
    "flemática": "flemático",
    "flemáticas": "flemático",
    "flemáticos": "flemático",
    "fleos": "fleo",
    "flequillos": "flequillo",
    "fleta": "fletar",
    "fletaba": "fletar",
    "fletabais": "fletar",
    "fletaban": "fletar",
    "fletabas": "fletar",
    "fletad": "fletar",
    "fletada": "fletar",
    "fletadas": "fletar",
    "fletado": "fletar",
    "fletadora": "fletador",
    "fletadoras": "fletador",
    "fletadores": "fletador",
    "fletados": "fletar",
    "fletamentos": "fletamento",
    "fletamientos": "fletamiento",
    "fletamos": "fletar",
    "fletan": "fletar",
    "fletando": "fletar",
    "fletantes": "fletante",
    "fletara": "fletar",
    "fletarais": "fletar",
    "fletaran": "fletar",
    "fletaras": "fletar",
    "fletare": "fletar",
    "fletareis": "fletar",
    "fletaremos": "fletar",
    "fletaren": "fletar",
    "fletares": "fletar",
    "fletaron": "fletar",
    "fletará": "fletar",
    "fletarán": "fletar",
    "fletarás": "fletar",
    "fletaré": "fletar",
    "fletaréis": "fletar",
    "fletaría": "fletar",
    "fletaríais": "fletar",
    "fletaríamos": "fletar",
    "fletarían": "fletar",
    "fletarías": "fletar",
    "fletas": "fletar",
    "fletase": "fletar",
    "fletaseis": "fletar",
    "fletasen": "fletar",
    "fletases": "fletar",
    "fletaste": "fletar",
    "fletasteis": "fletar",
    "flete": "fletar",
    "fletemos": "fletar",
    "fleten": "fletar",
    "fletes": "flete",
    "fleto": "fletar",
    "fletábamos": "fletar",
    "fletáis": "fletar",
    "fletáramos": "fletar",
    "fletáremos": "fletar",
    "fletásemos": "fletar",
    "fleté": "fletar",
    "fletéis": "fletar",
    "fletó": "fletar",
    "flexatons": "flexaton",
    "flexibilice": "flexibilizar",
    "flexibilicemos": "flexibilizar",
    "flexibilicen": "flexibilizar",
    "flexibilices": "flexibilizar",
    "flexibilicé": "flexibilizar",
    "flexibilicéis": "flexibilizar",
    "flexibilidades": "flexibilidad",
    "flexibiliza": "flexibilizar",
    "flexibilizaba": "flexibilizar",
    "flexibilizabais": "flexibilizar",
    "flexibilizaban": "flexibilizar",
    "flexibilizabas": "flexibilizar",
    "flexibilizaciones": "flexibilización",
    "flexibilizad": "flexibilizar",
    "flexibilizada": "flexibilizar",
    "flexibilizadas": "flexibilizar",
    "flexibilizado": "flexibilizar",
    "flexibilizados": "flexibilizar",
    "flexibilizamos": "flexibilizar",
    "flexibilizan": "flexibilizar",
    "flexibilizando": "flexibilizar",
    "flexibilizara": "flexibilizar",
    "flexibilizarais": "flexibilizar",
    "flexibilizaran": "flexibilizar",
    "flexibilizaras": "flexibilizar",
    "flexibilizare": "flexibilizar",
    "flexibilizareis": "flexibilizar",
    "flexibilizaremos": "flexibilizar",
    "flexibilizaren": "flexibilizar",
    "flexibilizares": "flexibilizar",
    "flexibilizaron": "flexibilizar",
    "flexibilizará": "flexibilizar",
    "flexibilizarán": "flexibilizar",
    "flexibilizarás": "flexibilizar",
    "flexibilizaré": "flexibilizar",
    "flexibilizaréis": "flexibilizar",
    "flexibilizaría": "flexibilizar",
    "flexibilizaríais": "flexibilizar",
    "flexibilizaríamos": "flexibilizar",
    "flexibilizarían": "flexibilizar",
    "flexibilizarías": "flexibilizar",
    "flexibilizas": "flexibilizar",
    "flexibilizase": "flexibilizar",
    "flexibilizaseis": "flexibilizar",
    "flexibilizasen": "flexibilizar",
    "flexibilizases": "flexibilizar",
    "flexibilizaste": "flexibilizar",
    "flexibilizasteis": "flexibilizar",
    "flexibilizo": "flexibilizar",
    "flexibilizábamos": "flexibilizar",
    "flexibilizáis": "flexibilizar",
    "flexibilizáramos": "flexibilizar",
    "flexibilizáremos": "flexibilizar",
    "flexibilizásemos": "flexibilizar",
    "flexibilizó": "flexibilizar",
    "flexibles": "flexible",
    "flexiona": "flexionar",
    "flexionaba": "flexionar",
    "flexionabais": "flexionar",
    "flexionaban": "flexionar",
    "flexionabas": "flexionar",
    "flexionables": "flexionable",
    "flexionad": "flexionar",
    "flexionada": "flexionar",
    "flexionadas": "flexionar",
    "flexionado": "flexionar",
    "flexionados": "flexionar",
    "flexionales": "flexional",
    "flexionamos": "flexionar",
    "flexionan": "flexionar",
    "flexionando": "flexionar",
    "flexionara": "flexionar",
    "flexionarais": "flexionar",
    "flexionaran": "flexionar",
    "flexionaras": "flexionar",
    "flexionare": "flexionar",
    "flexionareis": "flexionar",
    "flexionaremos": "flexionar",
    "flexionaren": "flexionar",
    "flexionares": "flexionar",
    "flexionaron": "flexionar",
    "flexionará": "flexionar",
    "flexionarán": "flexionar",
    "flexionarás": "flexionar",
    "flexionaré": "flexionar",
    "flexionaréis": "flexionar",
    "flexionaría": "flexionar",
    "flexionaríais": "flexionar",
    "flexionaríamos": "flexionar",
    "flexionarían": "flexionar",
    "flexionarías": "flexionar",
    "flexionas": "flexionar",
    "flexionase": "flexionar",
    "flexionaseis": "flexionar",
    "flexionasen": "flexionar",
    "flexionases": "flexionar",
    "flexionaste": "flexionar",
    "flexionasteis": "flexionar",
    "flexione": "flexionar",
    "flexionemos": "flexionar",
    "flexionen": "flexionar",
    "flexiones": "flexionar",
    "flexiono": "flexionar",
    "flexionábamos": "flexionar",
    "flexionáis": "flexionar",
    "flexionáramos": "flexionar",
    "flexionáremos": "flexionar",
    "flexionásemos": "flexionar",
    "flexioné": "flexionar",
    "flexionéis": "flexionar",
    "flexionó": "flexionar",
    "flexiva": "flexivo",
    "flexivas": "flexivo",
    "flexivos": "flexivo",
    "flexora": "flexor",
    "flexoras": "flexor",
    "flexores": "flexor",
    "flexos": "flexo",
    "flexuosa": "flexuoso",
    "flexuosas": "flexuoso",
    "flexuosos": "flexuoso",
    "flexuras": "flexura",
    "flexátones": "flexaton",
    "flexómetros": "flexómetro",
    "flic-flacs": "flic-flac",
    "flicks": "flick",
    "flictenas": "flictena",
    "flictenulares": "flictenular",
    "flintes": "flint",
    "flints": "flint",
    "flips": "flip",
    "flirtea": "flirtear",
    "flirteaba": "flirtear",
    "flirteabais": "flirtear",
    "flirteaban": "flirtear",
    "flirteabas": "flirtear",
    "flirtead": "flirtear",
    "flirteada": "flirtear",
    "flirteadas": "flirtear",
    "flirteado": "flirtear",
    "flirteados": "flirtear",
    "flirteamos": "flirtear",
    "flirtean": "flirtear",
    "flirteando": "flirtear",
    "flirteara": "flirtear",
    "flirtearais": "flirtear",
    "flirtearan": "flirtear",
    "flirtearas": "flirtear",
    "flirteare": "flirtear",
    "flirteareis": "flirtear",
    "flirtearemos": "flirtear",
    "flirtearen": "flirtear",
    "flirteares": "flirtear",
    "flirtearon": "flirtear",
    "flirteará": "flirtear",
    "flirtearán": "flirtear",
    "flirtearás": "flirtear",
    "flirtearé": "flirtear",
    "flirtearéis": "flirtear",
    "flirtearía": "flirtear",
    "flirtearíais": "flirtear",
    "flirtearíamos": "flirtear",
    "flirtearían": "flirtear",
    "flirtearías": "flirtear",
    "flirteas": "flirtear",
    "flirtease": "flirtear",
    "flirteaseis": "flirtear",
    "flirteasen": "flirtear",
    "flirteases": "flirtear",
    "flirteaste": "flirtear",
    "flirteasteis": "flirtear",
    "flirtee": "flirtear",
    "flirteemos": "flirtear",
    "flirteen": "flirtear",
    "flirtees": "flirtear",
    "flirteo": "flirtear",
    "flirteos": "flirteo",
    "flirtes": "flirt",
    "flirteábamos": "flirtear",
    "flirteáis": "flirtear",
    "flirteáramos": "flirtear",
    "flirteáremos": "flirtear",
    "flirteásemos": "flirtear",
    "flirteé": "flirtear",
    "flirteéis": "flirtear",
    "flirteó": "flirtear",
    "flirts": "flirt",
    "flocada": "flocado",
    "flocadas": "flocado",
    "flocados": "flocado",
    "flocaduras": "flocadura",
    "flocajes": "flocaje",
    "flocas": "floca",
    "flocula": "flocular",
    "floculaba": "flocular",
    "floculabais": "flocular",
    "floculaban": "flocular",
    "floculabas": "flocular",
    "floculaciones": "floculación",
    "floculad": "flocular",
    "floculada": "flocular",
    "floculadas": "flocular",
    "floculado": "flocular",
    "floculadores": "floculador",
    "floculados": "flocular",
    "floculamos": "flocular",
    "floculan": "flocular",
    "floculando": "flocular",
    "floculantes": "floculante",
    "floculara": "flocular",
    "flocularais": "flocular",
    "flocularan": "flocular",
    "flocularas": "flocular",
    "floculare": "flocular",
    "floculareis": "flocular",
    "flocularemos": "flocular",
    "flocularen": "flocular",
    "floculares": "flocular",
    "flocularon": "flocular",
    "floculará": "flocular",
    "flocularán": "flocular",
    "flocularás": "flocular",
    "flocularé": "flocular",
    "flocularéis": "flocular",
    "flocularía": "flocular",
    "flocularíais": "flocular",
    "flocularíamos": "flocular",
    "flocularían": "flocular",
    "flocularías": "flocular",
    "floculas": "flocular",
    "floculase": "flocular",
    "floculaseis": "flocular",
    "floculasen": "flocular",
    "floculases": "flocular",
    "floculaste": "flocular",
    "floculasteis": "flocular",
    "flocule": "flocular",
    "floculemos": "flocular",
    "floculen": "flocular",
    "floculenta": "floculento",
    "floculentas": "floculento",
    "floculentes": "floculente",
    "floculentos": "floculento",
    "flocules": "flocular",
    "floculo": "flocular",
    "floculonodulares": "floculonodular",
    "floculábamos": "flocular",
    "floculáis": "flocular",
    "floculáramos": "flocular",
    "floculáremos": "flocular",
    "floculásemos": "flocular",
    "floculé": "flocular",
    "floculéis": "flocular",
    "floculó": "flocular",
    "floemas": "floema",
    "flogistos": "flogisto",
    "flogopitas": "flogopita",
    "flogística": "flogístico",
    "flogísticas": "flogístico",
    "flogísticos": "flogístico",
    "floja": "flojo",
    "flojas": "flojo",
    "flojea": "flojear",
    "flojeaba": "flojear",
    "flojeabais": "flojear",
    "flojeaban": "flojear",
    "flojeabas": "flojear",
    "flojead": "flojear",
    "flojeada": "flojear",
    "flojeadas": "flojear",
    "flojeado": "flojear",
    "flojeados": "flojear",
    "flojeamos": "flojear",
    "flojean": "flojear",
    "flojeando": "flojear",
    "flojeara": "flojear",
    "flojearais": "flojear",
    "flojearan": "flojear",
    "flojearas": "flojear",
    "flojeare": "flojear",
    "flojeareis": "flojear",
    "flojearemos": "flojear",
    "flojearen": "flojear",
    "flojeares": "flojear",
    "flojearon": "flojear",
    "flojeará": "flojear",
    "flojearán": "flojear",
    "flojearás": "flojear",
    "flojearé": "flojear",
    "flojearéis": "flojear",
    "flojearía": "flojear",
    "flojearíais": "flojear",
    "flojearíamos": "flojear",
    "flojearían": "flojear",
    "flojearías": "flojear",
    "flojeas": "flojear",
    "flojease": "flojear",
    "flojeaseis": "flojear",
    "flojeasen": "flojear",
    "flojeases": "flojear",
    "flojeaste": "flojear",
    "flojeasteis": "flojear",
    "flojedades": "flojedad",
    "flojee": "flojear",
    "flojeemos": "flojear",
    "flojeen": "flojear",
    "flojees": "flojear",
    "flojeles": "flojel",
    "flojeo": "flojear",
    "flojeos": "flojeo",
    "flojeras": "flojera",
    "flojeábamos": "flojear",
    "flojeáis": "flojear",
    "flojeáramos": "flojear",
    "flojeáremos": "flojear",
    "flojeásemos": "flojear",
    "flojeé": "flojear",
    "flojeéis": "flojear",
    "flojeó": "flojear",
    "flojita": "flojito",
    "flojitas": "flojito",
    "flojitos": "flojito",
    "flojos": "flojo",
    "flojuras": "flojura",
    "floqueada": "floqueado",
    "floqueadas": "floqueado",
    "floqueados": "floqueado",
    "floques": "floque",
    "floraciones": "floración",
    "florales": "floral",
    "floras": "flora",
    "flordelisa": "flordelisar",
    "flordelisaba": "flordelisar",
    "flordelisabais": "flordelisar",
    "flordelisaban": "flordelisar",
    "flordelisabas": "flordelisar",
    "flordelisad": "flordelisar",
    "flordelisada": "flordelisar",
    "flordelisadas": "flordelisar",
    "flordelisado": "flordelisar",
    "flordelisados": "flordelisar",
    "flordelisamos": "flordelisar",
    "flordelisan": "flordelisar",
    "flordelisando": "flordelisar",
    "flordelisara": "flordelisar",
    "flordelisarais": "flordelisar",
    "flordelisaran": "flordelisar",
    "flordelisaras": "flordelisar",
    "flordelisare": "flordelisar",
    "flordelisareis": "flordelisar",
    "flordelisaremos": "flordelisar",
    "flordelisaren": "flordelisar",
    "flordelisares": "flordelisar",
    "flordelisaron": "flordelisar",
    "flordelisará": "flordelisar",
    "flordelisarán": "flordelisar",
    "flordelisarás": "flordelisar",
    "flordelisaré": "flordelisar",
    "flordelisaréis": "flordelisar",
    "flordelisaría": "flordelisar",
    "flordelisaríais": "flordelisar",
    "flordelisaríamos": "flordelisar",
    "flordelisarían": "flordelisar",
    "flordelisarías": "flordelisar",
    "flordelisas": "flordelisar",
    "flordelisase": "flordelisar",
    "flordelisaseis": "flordelisar",
    "flordelisasen": "flordelisar",
    "flordelisases": "flordelisar",
    "flordelisaste": "flordelisar",
    "flordelisasteis": "flordelisar",
    "flordelise": "flordelisar",
    "flordelisemos": "flordelisar",
    "flordelisen": "flordelisar",
    "flordelises": "flordelisar",
    "flordeliso": "flordelisar",
    "flordelisábamos": "flordelisar",
    "flordelisáis": "flordelisar",
    "flordelisáramos": "flordelisar",
    "flordelisáremos": "flordelisar",
    "flordelisásemos": "flordelisar",
    "flordelisé": "flordelisar",
    "flordeliséis": "flordelisar",
    "flordelisó": "flordelisar",
    "florea": "florear",
    "floreaba": "florear",
    "floreabais": "florear",
    "floreaban": "florear",
    "floreabas": "florear",
    "floread": "florear",
    "floreada": "florear",
    "floreadas": "florear",
    "floreado": "florear",
    "floreados": "florear",
    "floreales": "floreal",
    "floreamos": "florear",
    "florean": "florear",
    "floreando": "florear",
    "floreara": "florear",
    "florearais": "florear",
    "florearan": "florear",
    "florearas": "florear",
    "floreare": "florear",
    "floreareis": "florear",
    "florearemos": "florear",
    "florearen": "florear",
    "floreares": "florear",
    "florearon": "florear",
    "floreará": "florear",
    "florearán": "florear",
    "florearás": "florear",
    "florearé": "florear",
    "florearéis": "florear",
    "florearía": "florear",
    "florearíais": "florear",
    "florearíamos": "florear",
    "florearían": "florear",
    "florearías": "florear",
    "floreas": "florear",
    "florease": "florear",
    "floreaseis": "florear",
    "floreasen": "florear",
    "floreases": "florear",
    "floreaste": "florear",
    "floreasteis": "florear",
    "florece": "florecer",
    "floreced": "florecer",
    "florecedora": "florecedor",
    "florecedoras": "florecedor",
    "florecedores": "florecedor",
    "florecemos": "florecer",
    "florecen": "florecer",
    "floreceremos": "florecer",
    "florecerá": "florecer",
    "florecerán": "florecer",
    "florecerás": "florecer",
    "floreceré": "florecer",
    "floreceréis": "florecer",
    "florecería": "florecer",
    "floreceríais": "florecer",
    "floreceríamos": "florecer",
    "florecerían": "florecer",
    "florecerías": "florecer",
    "floreces": "florecer",
    "florecida": "florecer",
    "florecidas": "florecer",
    "florecido": "florecer",
    "florecidos": "florecer",
    "floreciendo": "florecer",
    "florecientes": "floreciente",
    "floreciera": "florecer",
    "florecierais": "florecer",
    "florecieran": "florecer",
    "florecieras": "florecer",
    "floreciere": "florecer",
    "floreciereis": "florecer",
    "florecieren": "florecer",
    "florecieres": "florecer",
    "florecieron": "florecer",
    "floreciese": "florecer",
    "florecieseis": "florecer",
    "floreciesen": "florecer",
    "florecieses": "florecer",
    "florecimientos": "florecimiento",
    "florecimos": "florecer",
    "floreciste": "florecer",
    "florecisteis": "florecer",
    "floreciéramos": "florecer",
    "floreciéremos": "florecer",
    "floreciésemos": "florecer",
    "floreció": "florecer",
    "florecéis": "florecer",
    "florecí": "florecer",
    "florecía": "florecer",
    "florecíais": "florecer",
    "florecíamos": "florecer",
    "florecían": "florecer",
    "florecías": "florecer",
    "floree": "florear",
    "floreemos": "florear",
    "floreen": "florear",
    "florees": "florear",
    "florentina": "florentino",
    "florentinas": "florentino",
    "florentinos": "florentino",
    "floreo": "florear",
    "floreos": "floreo",
    "floreros": "florero",
    "flores": "flor",
    "florescencias": "florescencia",
    "florestas": "floresta",
    "florestera": "florestero",
    "floresteras": "florestero",
    "floresteros": "florestero",
    "floretazos": "floretazo",
    "floretea": "floretear",
    "floreteaba": "floretear",
    "floreteabais": "floretear",
    "floreteaban": "floretear",
    "floreteabas": "floretear",
    "floretead": "floretear",
    "floreteada": "floretear",
    "floreteadas": "floretear",
    "floreteado": "floretear",
    "floreteados": "floretear",
    "floreteamos": "floretear",
    "floretean": "floretear",
    "floreteando": "floretear",
    "floreteara": "floretear",
    "floretearais": "floretear",
    "floretearan": "floretear",
    "floretearas": "floretear",
    "floreteare": "floretear",
    "floreteareis": "floretear",
    "floretearemos": "floretear",
    "floretearen": "floretear",
    "floreteares": "floretear",
    "floretearon": "floretear",
    "floreteará": "floretear",
    "floretearán": "floretear",
    "floretearás": "floretear",
    "floretearé": "floretear",
    "floretearéis": "floretear",
    "floretearía": "floretear",
    "floretearíais": "floretear",
    "floretearíamos": "floretear",
    "floretearían": "floretear",
    "floretearías": "floretear",
    "floreteas": "floretear",
    "floretease": "floretear",
    "floreteaseis": "floretear",
    "floreteasen": "floretear",
    "floreteases": "floretear",
    "floreteaste": "floretear",
    "floreteasteis": "floretear",
    "floretee": "floretear",
    "floreteemos": "floretear",
    "floreteen": "floretear",
    "floretees": "floretear",
    "floreteo": "floretear",
    "floreteos": "floreteo",
    "floretes": "florete",
    "floreteábamos": "floretear",
    "floreteáis": "floretear",
    "floreteáramos": "floretear",
    "floreteáremos": "floretear",
    "floreteásemos": "floretear",
    "floreteé": "floretear",
    "floreteéis": "floretear",
    "floreteó": "floretear",
    "floretistas": "floretista",
    "florezca": "florecer",
    "florezcamos": "florecer",
    "florezcan": "florecer",
    "florezcas": "florecer",
    "florezco": "florecer",
    "florezcáis": "florecer",
    "floreábamos": "florear",
    "floreáis": "florear",
    "floreáramos": "florear",
    "floreáremos": "florear",
    "floreásemos": "florear",
    "floreé": "florear",
    "floreéis": "florear",
    "floreó": "florear",
    "floricultora": "floricultor",
    "floricultoras": "floricultor",
    "floricultores": "floricultor",
    "floriculturas": "floricultura",
    "florida": "florido",
    "floridas": "florido",
    "florideces": "floridez",
    "floridos": "florido",
    "florilegios": "florilegio",
    "florines": "florín",
    "floripondios": "floripondio",
    "floristas": "florista",
    "floristerías": "floristería",
    "florituras": "floritura",
    "floroglucinoles": "floroglucinol",
    "florones": "florón",
    "florídeas": "florídea",
    "florífera": "florífero",
    "floríferas": "florífero",
    "floríferos": "florífero",
    "flosculosa": "flosculoso",
    "flosculosas": "flosculoso",
    "flosculosos": "flosculoso",
    "flota": "flotar",
    "flotaba": "flotar",
    "flotabais": "flotar",
    "flotaban": "flotar",
    "flotabas": "flotar",
    "flotabilidades": "flotabilidad",
    "flotables": "flotable",
    "flotaciones": "flotación",
    "flotad": "flotar",
    "flotada": "flotar",
    "flotadas": "flotar",
    "flotado": "flotar",
    "flotadora": "flotador",
    "flotadoras": "flotador",
    "flotadores": "flotador",
    "flotados": "flotar",
    "flotaduras": "flotadura",
    "flotamientos": "flotamiento",
    "flotamos": "flotar",
    "flotan": "flotar",
    "flotando": "flotar",
    "flotantes": "flotante",
    "flotara": "flotar",
    "flotarais": "flotar",
    "flotaran": "flotar",
    "flotaras": "flotar",
    "flotare": "flotar",
    "flotareis": "flotar",
    "flotaremos": "flotar",
    "flotaren": "flotar",
    "flotares": "flotar",
    "flotaron": "flotar",
    "flotará": "flotar",
    "flotarán": "flotar",
    "flotarás": "flotar",
    "flotaré": "flotar",
    "flotaréis": "flotar",
    "flotaría": "flotar",
    "flotaríais": "flotar",
    "flotaríamos": "flotar",
    "flotarían": "flotar",
    "flotarías": "flotar",
    "flotas": "flotar",
    "flotase": "flotar",
    "flotaseis": "flotar",
    "flotasen": "flotar",
    "flotases": "flotar",
    "flotaste": "flotar",
    "flotasteis": "flotar",
    "flote": "flotar",
    "flotemos": "flotar",
    "floten": "flotar",
    "flotes": "flote",
    "flotillas": "flotilla",
    "floto": "flotar",
    "flotábamos": "flotar",
    "flotáis": "flotar",
    "flotáramos": "flotar",
    "flotáremos": "flotar",
    "flotásemos": "flotar",
    "floté": "flotar",
    "flotéis": "flotar",
    "flotó": "flotar",
    "fluctuaba": "fluctuar",
    "fluctuabais": "fluctuar",
    "fluctuaban": "fluctuar",
    "fluctuabas": "fluctuar",
    "fluctuaciones": "fluctuación",
    "fluctuad": "fluctuar",
    "fluctuada": "fluctuar",
    "fluctuadas": "fluctuar",
    "fluctuado": "fluctuar",
    "fluctuados": "fluctuar",
    "fluctuamos": "fluctuar",
    "fluctuando": "fluctuar",
    "fluctuantes": "fluctuante",
    "fluctuara": "fluctuar",
    "fluctuarais": "fluctuar",
    "fluctuaran": "fluctuar",
    "fluctuaras": "fluctuar",
    "fluctuare": "fluctuar",
    "fluctuareis": "fluctuar",
    "fluctuaremos": "fluctuar",
    "fluctuaren": "fluctuar",
    "fluctuares": "fluctuar",
    "fluctuaron": "fluctuar",
    "fluctuará": "fluctuar",
    "fluctuarán": "fluctuar",
    "fluctuarás": "fluctuar",
    "fluctuaré": "fluctuar",
    "fluctuaréis": "fluctuar",
    "fluctuaría": "fluctuar",
    "fluctuaríais": "fluctuar",
    "fluctuaríamos": "fluctuar",
    "fluctuarían": "fluctuar",
    "fluctuarías": "fluctuar",
    "fluctuase": "fluctuar",
    "fluctuaseis": "fluctuar",
    "fluctuasen": "fluctuar",
    "fluctuases": "fluctuar",
    "fluctuaste": "fluctuar",
    "fluctuasteis": "fluctuar",
    "fluctuemos": "fluctuar",
    "fluctuosa": "fluctuoso",
    "fluctuosas": "fluctuoso",
    "fluctuosos": "fluctuoso",
    "fluctuábamos": "fluctuar",
    "fluctuáis": "fluctuar",
    "fluctuáramos": "fluctuar",
    "fluctuáremos": "fluctuar",
    "fluctuásemos": "fluctuar",
    "fluctué": "fluctuar",
    "fluctuéis": "fluctuar",
    "fluctuó": "fluctuar",
    "fluctúa": "fluctuar",
    "fluctúan": "fluctuar",
    "fluctúas": "fluctuar",
    "fluctúe": "fluctuar",
    "fluctúen": "fluctuar",
    "fluctúes": "fluctuar",
    "fluctúo": "fluctuar",
    "fluencias": "fluencia",
    "fluentes": "fluente",
    "fluid": "fluir",
    "fluida": "fluir",
    "fluidales": "fluidal",
    "fluidas": "fluir",
    "fluideces": "fluidez",
    "fluidifica": "fluidificar",
    "fluidificaba": "fluidificar",
    "fluidificabais": "fluidificar",
    "fluidificaban": "fluidificar",
    "fluidificabas": "fluidificar",
    "fluidificaciones": "fluidificación",
    "fluidificad": "fluidificar",
    "fluidificada": "fluidificar",
    "fluidificadas": "fluidificar",
    "fluidificado": "fluidificar",
    "fluidificados": "fluidificar",
    "fluidificamos": "fluidificar",
    "fluidifican": "fluidificar",
    "fluidificando": "fluidificar",
    "fluidificantes": "fluidificante",
    "fluidificara": "fluidificar",
    "fluidificarais": "fluidificar",
    "fluidificaran": "fluidificar",
    "fluidificaras": "fluidificar",
    "fluidificare": "fluidificar",
    "fluidificareis": "fluidificar",
    "fluidificaremos": "fluidificar",
    "fluidificaren": "fluidificar",
    "fluidificares": "fluidificar",
    "fluidificaron": "fluidificar",
    "fluidificará": "fluidificar",
    "fluidificarán": "fluidificar",
    "fluidificarás": "fluidificar",
    "fluidificaré": "fluidificar",
    "fluidificaréis": "fluidificar",
    "fluidificaría": "fluidificar",
    "fluidificaríais": "fluidificar",
    "fluidificaríamos": "fluidificar",
    "fluidificarían": "fluidificar",
    "fluidificarías": "fluidificar",
    "fluidificas": "fluidificar",
    "fluidificase": "fluidificar",
    "fluidificaseis": "fluidificar",
    "fluidificasen": "fluidificar",
    "fluidificases": "fluidificar",
    "fluidificaste": "fluidificar",
    "fluidificasteis": "fluidificar",
    "fluidifico": "fluidificar",
    "fluidificábamos": "fluidificar",
    "fluidificáis": "fluidificar",
    "fluidificáramos": "fluidificar",
    "fluidificáremos": "fluidificar",
    "fluidificásemos": "fluidificar",
    "fluidificó": "fluidificar",
    "fluidifique": "fluidificar",
    "fluidifiquemos": "fluidificar",
    "fluidifiquen": "fluidificar",
    "fluidifiques": "fluidificar",
    "fluidifiqué": "fluidificar",
    "fluidifiquéis": "fluidificar",
    "fluidizaciones": "fluidización",
    "fluido": "fluir",
    "fluidos": "fluir",
    "fluimos": "fluir",
    "fluiremos": "fluir",
    "fluirá": "fluir",
    "fluirán": "fluir",
    "fluirás": "fluir",
    "fluiré": "fluir",
    "fluiréis": "fluir",
    "fluiría": "fluir",
    "fluiríais": "fluir",
    "fluiríamos": "fluir",
    "fluirían": "fluir",
    "fluirías": "fluir",
    "fluiste": "fluir",
    "fluisteis": "fluir",
    "flujos": "flujo",
    "fluminenses": "fluminense",
    "fluoacetatos": "fluoacetato",
    "fluoacetofenonas": "fluoacetofenona",
    "fluoanilinas": "fluoanilina",
    "fluobencenos": "fluobenceno",
    "fluobórica": "fluobórico",
    "fluobóricas": "fluobórico",
    "fluobóricos": "fluobórico",
    "fluocarbonos": "fluocarbono",
    "fluofosfatos": "fluofosfato",
    "fluofosfórica": "fluofosfórico",
    "fluofosfóricas": "fluofosfórico",
    "fluofosfóricos": "fluofosfórico",
    "fluoraciones": "fluoración",
    "fluorada": "fluorado",
    "fluoradas": "fluorado",
    "fluorados": "fluorado",
    "fluorapatitas": "fluorapatita",
    "fluorescencias": "fluorescencia",
    "fluorescentes": "fluorescente",
    "fluoresceínas": "fluoresceína",
    "fluorescimetrías": "fluorescimetría",
    "fluoretilenos": "fluoretileno",
    "fluorhídrica": "fluorhídrico",
    "fluorhídricas": "fluorhídrico",
    "fluorhídricos": "fluorhídrico",
    "fluorimetrías": "fluorimetría",
    "fluoritas": "fluorita",
    "fluorizaciones": "fluorización",
    "fluorocromos": "fluorocromo",
    "fluorofosfatos": "fluorofosfato",
    "fluorografías": "fluorografía",
    "fluorometrías": "fluorometría",
    "fluoroscópica": "fluoroscópico",
    "fluoroscópicas": "fluoroscópico",
    "fluoroscópicos": "fluoroscópico",
    "fluoruros": "fluoruro",
    "fluorímetros": "fluorímetro",
    "fluorítica": "fluorítico",
    "fluoríticas": "fluorítico",
    "fluoríticos": "fluorítico",
    "fluosilicatos": "fluosilicato",
    "fluosilícica": "fluosilícico",
    "fluosilícicas": "fluosilícico",
    "fluosilícicos": "fluosilícico",
    "fluosulfónica": "fluosulfónico",
    "fluosulfónicas": "fluosulfónico",
    "fluosulfónicos": "fluosulfónico",
    "fluotenos": "fluoteno",
    "fluoxetinas": "fluoxetina",
    "fluviales": "fluvial",
    "fluvioglaciares": "fluvioglaciar",
    "fluvisoles": "fluvisol",
    "fluxiones": "fluxión",
    "fluxímetros": "fluxímetro",
    "fluxómetros": "fluxómetro",
    "fluya": "fluir",
    "fluyamos": "fluir",
    "fluyan": "fluir",
    "fluyas": "fluir",
    "fluye": "fluir",
    "fluyen": "fluir",
    "fluyendo": "fluir",
    "fluyentes": "fluyente",
    "fluyera": "fluir",
    "fluyerais": "fluir",
    "fluyeran": "fluir",
    "fluyeras": "fluir",
    "fluyere": "fluir",
    "fluyereis": "fluir",
    "fluyeren": "fluir",
    "fluyeres": "fluir",
    "fluyeron": "fluir",
    "fluyes": "fluir",
    "fluyese": "fluir",
    "fluyeseis": "fluir",
    "fluyesen": "fluir",
    "fluyeses": "fluir",
    "fluyo": "fluir",
    "fluyáis": "fluir",
    "fluyéramos": "fluir",
    "fluyéremos": "fluir",
    "fluyésemos": "fluir",
    "fluyó": "fluir",
    "fluí": "fluir",
    "fluía": "fluir",
    "fluíais": "fluir",
    "fluíamos": "fluir",
    "fluían": "fluir",
    "fluías": "fluir",
    "fluídicas": "fluídica",
    "fluís": "fluir",
    "fluórica": "fluórico",
    "fluóricas": "fluórico",
    "fluóricos": "fluórico",
    "flysches": "flysch",
    "flyschs": "flysch",
    "fláccida": "fláccido",
    "fláccidas": "fláccido",
    "fláccidos": "fláccido",
    "flácida": "flácido",
    "flácidas": "flácido",
    "flácidos": "flácido",
    "flámea": "flámeo",
    "flámeas": "flámeo",
    "flámeos": "flámeo",
    "flámines": "flamen",
    "flámulas": "flámula",
    "flébiles": "flébil",
    "flóculos": "flóculo",
    "flósculos": "flósculo",
    "flúores": "flúor",
    "flúteres": "flúter",
    "fm": "femtómetro",
    "fobias": "fobia",
    "focales": "focal",
    "focalice": "focalizar",
    "focalicemos": "focalizar",
    "focalicen": "focalizar",
    "focalices": "focalizar",
    "focalicé": "focalizar",
    "focalicéis": "focalizar",
    "focaliza": "focalizar",
    "focalizaba": "focalizar",
    "focalizabais": "focalizar",
    "focalizaban": "focalizar",
    "focalizabas": "focalizar",
    "focalizaciones": "focalización",
    "focalizad": "focalizar",
    "focalizada": "focalizar",
    "focalizadas": "focalizar",
    "focalizado": "focalizar",
    "focalizados": "focalizar",
    "focalizamos": "focalizar",
    "focalizan": "focalizar",
    "focalizando": "focalizar",
    "focalizara": "focalizar",
    "focalizarais": "focalizar",
    "focalizaran": "focalizar",
    "focalizaras": "focalizar",
    "focalizare": "focalizar",
    "focalizareis": "focalizar",
    "focalizaremos": "focalizar",
    "focalizaren": "focalizar",
    "focalizares": "focalizar",
    "focalizaron": "focalizar",
    "focalizará": "focalizar",
    "focalizarán": "focalizar",
    "focalizarás": "focalizar",
    "focalizaré": "focalizar",
    "focalizaréis": "focalizar",
    "focalizaría": "focalizar",
    "focalizaríais": "focalizar",
    "focalizaríamos": "focalizar",
    "focalizarían": "focalizar",
    "focalizarías": "focalizar",
    "focalizas": "focalizar",
    "focalizase": "focalizar",
    "focalizaseis": "focalizar",
    "focalizasen": "focalizar",
    "focalizases": "focalizar",
    "focalizaste": "focalizar",
    "focalizasteis": "focalizar",
    "focalizo": "focalizar",
    "focalizábamos": "focalizar",
    "focalizáis": "focalizar",
    "focalizáramos": "focalizar",
    "focalizáremos": "focalizar",
    "focalizásemos": "focalizar",
    "focalizó": "focalizar",
    "focas": "foca",
    "foceifizas": "foceifiza",
    "focenses": "focense",
    "fochas": "focha",
    "focinos": "focino",
    "focometrías": "focometría",
    "focos": "foco",
    "focómetros": "focómetro",
    "fofa": "fofo",
    "fofas": "fofo",
    "fofos": "fofo",
    "fogajes": "fogaje",
    "fogaradas": "fogarada",
    "fogaratas": "fogarata",
    "fogariles": "fogaril",
    "fogatas": "fogata",
    "fogonaduras": "fogonadura",
    "fogonazos": "fogonazo",
    "fogonera": "fogonero",
    "fogoneras": "fogonero",
    "fogoneros": "fogonero",
    "fogones": "fogón",
    "fogosa": "fogoso",
    "fogosas": "fogoso",
    "fogosidades": "fogosidad",
    "fogosos": "fogoso",
    "foguea": "foguear",
    "fogueaba": "foguear",
    "fogueabais": "foguear",
    "fogueaban": "foguear",
    "fogueabas": "foguear",
    "fogueaciones": "fogueación",
    "foguead": "foguear",
    "fogueada": "foguear",
    "fogueadas": "foguear",
    "fogueado": "foguear",
    "fogueados": "foguear",
    "fogueamos": "foguear",
    "foguean": "foguear",
    "fogueando": "foguear",
    "fogueara": "foguear",
    "foguearais": "foguear",
    "foguearan": "foguear",
    "foguearas": "foguear",
    "fogueare": "foguear",
    "fogueareis": "foguear",
    "foguearemos": "foguear",
    "foguearen": "foguear",
    "fogueares": "foguear",
    "foguearon": "foguear",
    "fogueará": "foguear",
    "foguearán": "foguear",
    "foguearás": "foguear",
    "foguearé": "foguear",
    "foguearéis": "foguear",
    "foguearía": "foguear",
    "foguearíais": "foguear",
    "foguearíamos": "foguear",
    "foguearían": "foguear",
    "foguearías": "foguear",
    "fogueas": "foguear",
    "foguease": "foguear",
    "fogueaseis": "foguear",
    "fogueasen": "foguear",
    "fogueases": "foguear",
    "fogueaste": "foguear",
    "fogueasteis": "foguear",
    "foguee": "foguear",
    "fogueemos": "foguear",
    "fogueen": "foguear",
    "foguees": "foguear",
    "fogueo": "foguear",
    "fogueos": "fogueo",
    "fogueábamos": "foguear",
    "fogueáis": "foguear",
    "fogueáramos": "foguear",
    "fogueáremos": "foguear",
    "fogueásemos": "foguear",
    "fogueé": "foguear",
    "fogueéis": "foguear",
    "fogueó": "foguear",
    "foidítica": "foidítico",
    "foidíticas": "foidítico",
    "foidíticos": "foidítico",
    "folclor": "folclore",
    "folclores": "folclore",
    "folcloristas": "folclorista",
    "folclórica": "folclórico",
    "folclóricas": "folclórico",
    "folclóricos": "folclórico",
    "folders": "fólder",
    "folgos": "folgo",
    "folia": "foliar",
    "foliaba": "foliar",
    "foliabais": "foliar",
    "foliaban": "foliar",
    "foliabas": "foliar",
    "foliaciones": "foliación",
    "foliad": "foliar",
    "foliada": "foliar",
    "foliadas": "foliar",
    "foliado": "foliar",
    "foliadora": "foliador",
    "foliadoras": "foliador",
    "foliadores": "foliador",
    "foliados": "foliar",
    "foliamos": "foliar",
    "folian": "foliar",
    "foliando": "foliar",
    "foliara": "foliar",
    "foliarais": "foliar",
    "foliaran": "foliar",
    "foliaras": "foliar",
    "foliare": "foliar",
    "foliareis": "foliar",
    "foliaremos": "foliar",
    "foliaren": "foliar",
    "foliares": "foliar",
    "foliaron": "foliar",
    "foliará": "foliar",
    "foliarán": "foliar",
    "foliarás": "foliar",
    "foliaré": "foliar",
    "foliaréis": "foliar",
    "foliaría": "foliar",
    "foliaríais": "foliar",
    "foliaríamos": "foliar",
    "foliarían": "foliar",
    "foliarías": "foliar",
    "folias": "foliar",
    "foliase": "foliar",
    "foliaseis": "foliar",
    "foliasen": "foliar",
    "foliases": "foliar",
    "foliaste": "foliar",
    "foliasteis": "foliar",
    "foliaturas": "foliatura",
    "foliculares": "folicular",
    "folidotos": "folidoto",
    "folie": "foliar",
    "foliemos": "foliar",
    "folien": "foliar",
    "folies": "foliar",
    "folio": "foliar",
    "folios": "folio",
    "foliábamos": "foliar",
    "foliácea": "foliáceo",
    "foliáceas": "foliáceo",
    "foliáceos": "foliáceo",
    "foliáis": "foliar",
    "foliáramos": "foliar",
    "foliáremos": "foliar",
    "foliásemos": "foliar",
    "folié": "foliar",
    "foliéis": "foliar",
    "folió": "foliar",
    "folklores": "folklore",
    "folkloristas": "folklorista",
    "folklórica": "folklórico",
    "folklóricas": "folklórico",
    "folklóricos": "folklórico",
    "folla": "follar",
    "follaba": "follar",
    "follabais": "follar",
    "follaban": "follar",
    "follabas": "follar",
    "follad": "follar",
    "follada": "follar",
    "folladas": "follar",
    "follado": "follar",
    "follados": "follar",
    "follajes": "follaje",
    "follamos": "follar",
    "follan": "follar",
    "follando": "follar",
    "follara": "follar",
    "follarais": "follar",
    "follaran": "follar",
    "follaras": "follar",
    "follare": "follar",
    "follareis": "follar",
    "follaremos": "follar",
    "follaren": "follar",
    "follares": "follar",
    "follaron": "follar",
    "follará": "follar",
    "follarán": "follar",
    "follarás": "follar",
    "follaré": "follar",
    "follaréis": "follar",
    "follaría": "follar",
    "follaríais": "follar",
    "follaríamos": "follar",
    "follarían": "follar",
    "follarías": "follar",
    "follas": "follar",
    "follase": "follar",
    "follaseis": "follar",
    "follasen": "follar",
    "follases": "follar",
    "follaste": "follar",
    "follasteis": "follar",
    "folle": "follar",
    "follemos": "follar",
    "follen": "follar",
    "follera": "follero",
    "folleras": "follero",
    "folleros": "follero",
    "folles": "follar",
    "folletera": "folletero",
    "folleteras": "folletero",
    "folleteros": "folletero",
    "folletines": "folletín",
    "folletinesca": "folletinesco",
    "folletinescas": "folletinesco",
    "folletinescos": "folletinesco",
    "folletones": "folletón",
    "folletos": "folleto",
    "follo": "follar",
    "follones": "follón",
    "follábamos": "follar",
    "folláis": "follar",
    "folláramos": "follar",
    "folláremos": "follar",
    "follásemos": "follar",
    "follé": "follar",
    "folléis": "follar",
    "folló": "follar",
    "folías": "folía",
    "folículos": "folículo",
    "fomenta": "fomentar",
    "fomentaba": "fomentar",
    "fomentabais": "fomentar",
    "fomentaban": "fomentar",
    "fomentabas": "fomentar",
    "fomentaciones": "fomentación",
    "fomentad": "fomentar",
    "fomentada": "fomentar",
    "fomentadas": "fomentar",
    "fomentado": "fomentar",
    "fomentadora": "fomentador",
    "fomentadoras": "fomentador",
    "fomentadores": "fomentador",
    "fomentados": "fomentar",
    "fomentamos": "fomentar",
    "fomentan": "fomentar",
    "fomentando": "fomentar",
    "fomentara": "fomentar",
    "fomentarais": "fomentar",
    "fomentaran": "fomentar",
    "fomentaras": "fomentar",
    "fomentare": "fomentar",
    "fomentareis": "fomentar",
    "fomentaremos": "fomentar",
    "fomentaren": "fomentar",
    "fomentares": "fomentar",
    "fomentaron": "fomentar",
    "fomentará": "fomentar",
    "fomentarán": "fomentar",
    "fomentarás": "fomentar",
    "fomentaré": "fomentar",
    "fomentaréis": "fomentar",
    "fomentaría": "fomentar",
    "fomentaríais": "fomentar",
    "fomentaríamos": "fomentar",
    "fomentarían": "fomentar",
    "fomentarías": "fomentar",
    "fomentas": "fomentar",
    "fomentase": "fomentar",
    "fomentaseis": "fomentar",
    "fomentasen": "fomentar",
    "fomentases": "fomentar",
    "fomentaste": "fomentar",
    "fomentasteis": "fomentar",
    "fomente": "fomentar",
    "fomentemos": "fomentar",
    "fomenten": "fomentar",
    "fomentes": "fomentar",
    "fomento": "fomentar",
    "fomentos": "fomento",
    "fomentábamos": "fomentar",
    "fomentáis": "fomentar",
    "fomentáramos": "fomentar",
    "fomentáremos": "fomentar",
    "fomentásemos": "fomentar",
    "fomenté": "fomentar",
    "fomentéis": "fomentar",
    "fomentó": "fomentar",
    "fonaciones": "fonación",
    "fonadora": "fonador",
    "fonadoras": "fonador",
    "fonadores": "fonador",
    "fonatoria": "fonatorio",
    "fonatorias": "fonatorio",
    "fonatorios": "fonatorio",
    "fondada": "fondado",
    "fondadas": "fondado",
    "fondados": "fondado",
    "fondas": "fonda",
    "fondea": "fondear",
    "fondeaba": "fondear",
    "fondeabais": "fondear",
    "fondeaban": "fondear",
    "fondeabas": "fondear",
    "fondead": "fondear",
    "fondeada": "fondear",
    "fondeadas": "fondear",
    "fondeaderos": "fondeadero",
    "fondeado": "fondear",
    "fondeados": "fondear",
    "fondeamos": "fondear",
    "fondean": "fondear",
    "fondeando": "fondear",
    "fondeara": "fondear",
    "fondearais": "fondear",
    "fondearan": "fondear",
    "fondearas": "fondear",
    "fondeare": "fondear",
    "fondeareis": "fondear",
    "fondearemos": "fondear",
    "fondearen": "fondear",
    "fondeares": "fondear",
    "fondearon": "fondear",
    "fondeará": "fondear",
    "fondearán": "fondear",
    "fondearás": "fondear",
    "fondearé": "fondear",
    "fondearéis": "fondear",
    "fondearía": "fondear",
    "fondearíais": "fondear",
    "fondearíamos": "fondear",
    "fondearían": "fondear",
    "fondearías": "fondear",
    "fondeas": "fondear",
    "fondease": "fondear",
    "fondeaseis": "fondear",
    "fondeasen": "fondear",
    "fondeases": "fondear",
    "fondeaste": "fondear",
    "fondeasteis": "fondear",
    "fondee": "fondear",
    "fondeemos": "fondear",
    "fondeen": "fondear",
    "fondees": "fondear",
    "fondeo": "fondear",
    "fondeos": "fondeo",
    "fondeábamos": "fondear",
    "fondeáis": "fondear",
    "fondeáramos": "fondear",
    "fondeáremos": "fondear",
    "fondeásemos": "fondear",
    "fondeé": "fondear",
    "fondeéis": "fondear",
    "fondeó": "fondear",
    "fondillones": "fondillón",
    "fondistas": "fondista",
    "fondona": "fondón",
    "fondonas": "fondón",
    "fondones": "fondón",
    "fondos": "fondo",
    "fondues": "fondue",
    "fonemas": "fonema",
    "fonemática": "fonemático",
    "fonemáticas": "fonemático",
    "fonemáticos": "fonemático",
    "fonendoscopios": "fonendoscopio",
    "fones": "fon",
    "fonetismos": "fonetismo",
    "fonetistas": "fonetista",
    "foniatras": "foniatra",
    "foniatrías": "foniatría",
    "foniles": "fonil",
    "fonios": "fonio",
    "fonocaptores": "fonocaptor",
    "fonocardiografías": "fonocardiografía",
    "fonocardiogramas": "fonocardiograma",
    "fonocirugías": "fonocirugía",
    "fonoestilística": "fonoestilístico",
    "fonoestilísticas": "fonoestilístico",
    "fonoestilísticos": "fonoestilístico",
    "fonografías": "fonografía",
    "fonogramas": "fonograma",
    "fonográfica": "fonográfico",
    "fonográficas": "fonográfico",
    "fonográficos": "fonográfico",
    "fonolitas": "fonolita",
    "fonologistas": "fonologista",
    "fonologizaciones": "fonologización",
    "fonologías": "fonología",
    "fonológica": "fonológico",
    "fonológicas": "fonológico",
    "fonológicos": "fonológico",
    "fonometrías": "fonometría",
    "fonométrica": "fonométrico",
    "fonométricas": "fonométrico",
    "fonométricos": "fonométrico",
    "fonones": "fonón",
    "fonoquímicas": "fonoquímica",
    "fonotecas": "fonoteca",
    "fonotecnias": "fonotecnia",
    "fons": "fon",
    "fonsados": "fonsado",
    "fontales": "fontal",
    "fontanales": "fontanal",
    "fontanares": "fontanar",
    "fontanas": "fontana",
    "fontanelas": "fontanela",
    "fontanera": "fontanero",
    "fontaneras": "fontanero",
    "fontaneros": "fontanero",
    "fontanerías": "fontanería",
    "fontículos": "fontículo",
    "fonémica": "fonémico",
    "fonémicas": "fonémico",
    "fonémicos": "fonémico",
    "fonética": "fonético",
    "fonéticas": "fonético",
    "fonéticos": "fonético",
    "fonógrafos": "fonógrafo",
    "fonóloga": "fonólogo",
    "fonólogas": "fonólogo",
    "fonólogos": "fonólogo",
    "fonómetros": "fonómetro",
    "foques": "foque",
    "forajida": "forajido",
    "forajidas": "forajido",
    "forajidos": "forajido",
    "forales": "foral",
    "foralismos": "foralismo",
    "foramens": "foramen",
    "foraminales": "foraminal",
    "foraminas": "foramina",
    "foraminífera": "foraminífero",
    "foraminíferas": "foraminífero",
    "foraminíferos": "foraminífero",
    "forastera": "forastero",
    "forasteras": "forastero",
    "forasteros": "forastero",
    "forbólica": "forbólico",
    "forbólicas": "forbólico",
    "forbólicos": "forbólico",
    "forceja": "forcejar",
    "forcejaba": "forcejar",
    "forcejabais": "forcejar",
    "forcejaban": "forcejar",
    "forcejabas": "forcejar",
    "forcejad": "forcejar",
    "forcejada": "forcejar",
    "forcejadas": "forcejar",
    "forcejado": "forcejar",
    "forcejados": "forcejar",
    "forcejamos": "forcejar",
    "forcejan": "forcejar",
    "forcejando": "forcejar",
    "forcejara": "forcejar",
    "forcejarais": "forcejar",
    "forcejaran": "forcejar",
    "forcejaras": "forcejar",
    "forcejare": "forcejar",
    "forcejareis": "forcejar",
    "forcejaremos": "forcejar",
    "forcejaren": "forcejar",
    "forcejares": "forcejar",
    "forcejaron": "forcejar",
    "forcejará": "forcejar",
    "forcejarán": "forcejar",
    "forcejarás": "forcejar",
    "forcejaré": "forcejar",
    "forcejaréis": "forcejar",
    "forcejaría": "forcejar",
    "forcejaríais": "forcejar",
    "forcejaríamos": "forcejar",
    "forcejarían": "forcejar",
    "forcejarías": "forcejar",
    "forcejas": "forcejar",
    "forcejase": "forcejar",
    "forcejaseis": "forcejar",
    "forcejasen": "forcejar",
    "forcejases": "forcejar",
    "forcejaste": "forcejar",
    "forcejasteis": "forcejar",
    "forceje": "forcejar",
    "forcejea": "forcejear",
    "forcejeaba": "forcejear",
    "forcejeabais": "forcejear",
    "forcejeaban": "forcejear",
    "forcejeabas": "forcejear",
    "forcejead": "forcejear",
    "forcejeada": "forcejear",
    "forcejeadas": "forcejear",
    "forcejeado": "forcejear",
    "forcejeados": "forcejear",
    "forcejeamos": "forcejear",
    "forcejean": "forcejear",
    "forcejeando": "forcejear",
    "forcejeara": "forcejear",
    "forcejearais": "forcejear",
    "forcejearan": "forcejear",
    "forcejearas": "forcejear",
    "forcejeare": "forcejear",
    "forcejeareis": "forcejear",
    "forcejearemos": "forcejear",
    "forcejearen": "forcejear",
    "forcejeares": "forcejear",
    "forcejearon": "forcejear",
    "forcejeará": "forcejear",
    "forcejearán": "forcejear",
    "forcejearás": "forcejear",
    "forcejearé": "forcejear",
    "forcejearéis": "forcejear",
    "forcejearía": "forcejear",
    "forcejearíais": "forcejear",
    "forcejearíamos": "forcejear",
    "forcejearían": "forcejear",
    "forcejearías": "forcejear",
    "forcejeas": "forcejear",
    "forcejease": "forcejear",
    "forcejeaseis": "forcejear",
    "forcejeasen": "forcejear",
    "forcejeases": "forcejear",
    "forcejeaste": "forcejear",
    "forcejeasteis": "forcejear",
    "forcejee": "forcejear",
    "forcejeemos": "forcejear",
    "forcejeen": "forcejear",
    "forcejees": "forcejear",
    "forcejemos": "forcejar",
    "forcejen": "forcejar",
    "forcejeo": "forcejear",
    "forcejeos": "forcejeo",
    "forcejes": "forcejar",
    "forcejeábamos": "forcejear",
    "forcejeáis": "forcejear",
    "forcejeáramos": "forcejear",
    "forcejeáremos": "forcejear",
    "forcejeásemos": "forcejear",
    "forcejeé": "forcejear",
    "forcejeéis": "forcejear",
    "forcejeó": "forcejear",
    "forcejo": "forcejar",
    "forcejones": "forcejón",
    "forcejos": "forcejo",
    "forcejuda": "forcejudo",
    "forcejudas": "forcejudo",
    "forcejudos": "forcejudo",
    "forcejábamos": "forcejar",
    "forcejáis": "forcejar",
    "forcejáramos": "forcejar",
    "forcejáremos": "forcejar",
    "forcejásemos": "forcejar",
    "forcejé": "forcejar",
    "forcejéis": "forcejar",
    "forcejó": "forcejar",
    "forcemos": "forzar",
    "forcé": "forzar",
    "forcéis": "forzar",
    "forcículas": "forcícula",
    "forcípulas": "forcípula",
    "forenses": "forense",
    "forera": "forero",
    "foreras": "forero",
    "foreros": "forero",
    "foresta": "forestar",
    "forestaba": "forestar",
    "forestabais": "forestar",
    "forestaban": "forestar",
    "forestabas": "forestar",
    "forestaciones": "forestación",
    "forestad": "forestar",
    "forestada": "forestar",
    "forestadas": "forestar",
    "forestado": "forestar",
    "forestados": "forestar",
    "forestales": "forestal",
    "forestamos": "forestar",
    "forestan": "forestar",
    "forestando": "forestar",
    "forestara": "forestar",
    "forestarais": "forestar",
    "forestaran": "forestar",
    "forestaras": "forestar",
    "forestare": "forestar",
    "forestareis": "forestar",
    "forestaremos": "forestar",
    "forestaren": "forestar",
    "forestares": "forestar",
    "forestaron": "forestar",
    "forestará": "forestar",
    "forestarán": "forestar",
    "forestarás": "forestar",
    "forestaré": "forestar",
    "forestaréis": "forestar",
    "forestaría": "forestar",
    "forestaríais": "forestar",
    "forestaríamos": "forestar",
    "forestarían": "forestar",
    "forestarías": "forestar",
    "forestas": "forestar",
    "forestase": "forestar",
    "forestaseis": "forestar",
    "forestasen": "forestar",
    "forestases": "forestar",
    "forestaste": "forestar",
    "forestasteis": "forestar",
    "foreste": "forestar",
    "forestemos": "forestar",
    "foresten": "forestar",
    "forestes": "forestar",
    "foresto": "forestar",
    "forestábamos": "forestar",
    "forestáis": "forestar",
    "forestáramos": "forestar",
    "forestáremos": "forestar",
    "forestásemos": "forestar",
    "foresté": "forestar",
    "forestéis": "forestar",
    "forestó": "forestar",
    "forfaites": "forfait",
    "forfaits": "forfait",
    "forfetizaciones": "forfetización",
    "foriascopios": "foriascopio",
    "forillos": "forillo",
    "forints": "forint",
    "forja": "forjar",
    "forjaba": "forjar",
    "forjabais": "forjar",
    "forjaban": "forjar",
    "forjabas": "forjar",
    "forjabilidades": "forjabilidad",
    "forjad": "forjar",
    "forjada": "forjar",
    "forjadas": "forjar",
    "forjado": "forjar",
    "forjadora": "forjador",
    "forjadoras": "forjador",
    "forjadores": "forjador",
    "forjados": "forjar",
    "forjaduras": "forjadura",
    "forjamos": "forjar",
    "forjan": "forjar",
    "forjando": "forjar",
    "forjara": "forjar",
    "forjarais": "forjar",
    "forjaran": "forjar",
    "forjaras": "forjar",
    "forjare": "forjar",
    "forjareis": "forjar",
    "forjaremos": "forjar",
    "forjaren": "forjar",
    "forjares": "forjar",
    "forjaron": "forjar",
    "forjará": "forjar",
    "forjarán": "forjar",
    "forjarás": "forjar",
    "forjaré": "forjar",
    "forjaréis": "forjar",
    "forjaría": "forjar",
    "forjaríais": "forjar",
    "forjaríamos": "forjar",
    "forjarían": "forjar",
    "forjarías": "forjar",
    "forjas": "forjar",
    "forjase": "forjar",
    "forjaseis": "forjar",
    "forjasen": "forjar",
    "forjases": "forjar",
    "forjaste": "forjar",
    "forjasteis": "forjar",
    "forje": "forjar",
    "forjemos": "forjar",
    "forjen": "forjar",
    "forjes": "forjar",
    "forjo": "forjar",
    "forjábamos": "forjar",
    "forjáis": "forjar",
    "forjáramos": "forjar",
    "forjáremos": "forjar",
    "forjásemos": "forjar",
    "forjé": "forjar",
    "forjéis": "forjar",
    "forjó": "forjar",
    "forma": "formar",
    "formaba": "formar",
    "formabais": "formar",
    "formaban": "formar",
    "formabas": "formar",
    "formabilidades": "formabilidad",
    "formables": "formable",
    "formaciones": "formación",
    "formad": "formar",
    "formada": "formar",
    "formadas": "formar",
    "formado": "formar",
    "formadora": "formador",
    "formadoras": "formadora",
    "formadores": "formador",
    "formados": "formar",
    "formaldehídos": "formaldehído",
    "formales": "formal",
    "formalice": "formalizar",
    "formalicemos": "formalizar",
    "formalicen": "formalizar",
    "formalices": "formalizar",
    "formalicé": "formalizar",
    "formalicéis": "formalizar",
    "formalidades": "formalidad",
    "formalinas": "formalina",
    "formalismos": "formalismo",
    "formalistas": "formalista",
    "formaliza": "formalizar",
    "formalizaba": "formalizar",
    "formalizabais": "formalizar",
    "formalizaban": "formalizar",
    "formalizabas": "formalizar",
    "formalizaciones": "formalización",
    "formalizad": "formalizar",
    "formalizada": "formalizar",
    "formalizadas": "formalizar",
    "formalizado": "formalizar",
    "formalizados": "formalizar",
    "formalizamos": "formalizar",
    "formalizan": "formalizar",
    "formalizando": "formalizar",
    "formalizara": "formalizar",
    "formalizarais": "formalizar",
    "formalizaran": "formalizar",
    "formalizaras": "formalizar",
    "formalizare": "formalizar",
    "formalizareis": "formalizar",
    "formalizaremos": "formalizar",
    "formalizaren": "formalizar",
    "formalizares": "formalizar",
    "formalizaron": "formalizar",
    "formalizará": "formalizar",
    "formalizarán": "formalizar",
    "formalizarás": "formalizar",
    "formalizaré": "formalizar",
    "formalizaréis": "formalizar",
    "formalizaría": "formalizar",
    "formalizaríais": "formalizar",
    "formalizaríamos": "formalizar",
    "formalizarían": "formalizar",
    "formalizarías": "formalizar",
    "formalizas": "formalizar",
    "formalizase": "formalizar",
    "formalizaseis": "formalizar",
    "formalizasen": "formalizar",
    "formalizases": "formalizar",
    "formalizaste": "formalizar",
    "formalizasteis": "formalizar",
    "formalizo": "formalizar",
    "formalizábamos": "formalizar",
    "formalizáis": "formalizar",
    "formalizáramos": "formalizar",
    "formalizáremos": "formalizar",
    "formalizásemos": "formalizar",
    "formalizó": "formalizar",
    "formamidas": "formamida",
    "formamos": "formar",
    "forman": "formar",
    "formando": "formar",
    "formantes": "formante",
    "formara": "formar",
    "formarais": "formar",
    "formaran": "formar",
    "formaras": "formar",
    "formare": "formar",
    "formareis": "formar",
    "formaremos": "formar",
    "formaren": "formar",
    "formares": "formar",
    "formaron": "formar",
    "formará": "formar",
    "formarán": "formar",
    "formarás": "formar",
    "formaré": "formar",
    "formaréis": "formar",
    "formaría": "formar",
    "formaríais": "formar",
    "formaríamos": "formar",
    "formarían": "formar",
    "formarías": "formar",
    "formas": "formar",
    "formase": "formar",
    "formaseis": "formar",
    "formasen": "formar",
    "formases": "formar",
    "formaste": "formar",
    "formasteis": "formar",
    "formatea": "formatear",
    "formateaba": "formatear",
    "formateabais": "formatear",
    "formateaban": "formatear",
    "formateabas": "formatear",
    "formatead": "formatear",
    "formateada": "formatear",
    "formateadas": "formatear",
    "formateado": "formatear",
    "formateadora": "formateador",
    "formateadoras": "formateador",
    "formateadores": "formateador",
    "formateados": "formatear",
    "formateamos": "formatear",
    "formatean": "formatear",
    "formateando": "formatear",
    "formateara": "formatear",
    "formatearais": "formatear",
    "formatearan": "formatear",
    "formatearas": "formatear",
    "formateare": "formatear",
    "formateareis": "formatear",
    "formatearemos": "formatear",
    "formatearen": "formatear",
    "formateares": "formatear",
    "formatearon": "formatear",
    "formateará": "formatear",
    "formatearán": "formatear",
    "formatearás": "formatear",
    "formatearé": "formatear",
    "formatearéis": "formatear",
    "formatearía": "formatear",
    "formatearíais": "formatear",
    "formatearíamos": "formatear",
    "formatearían": "formatear",
    "formatearías": "formatear",
    "formateas": "formatear",
    "formatease": "formatear",
    "formateaseis": "formatear",
    "formateasen": "formatear",
    "formateases": "formatear",
    "formateaste": "formatear",
    "formateasteis": "formatear",
    "formatee": "formatear",
    "formateemos": "formatear",
    "formateen": "formatear",
    "formatees": "formatear",
    "formateo": "formatear",
    "formateos": "formateo",
    "formateábamos": "formatear",
    "formateáis": "formatear",
    "formateáramos": "formatear",
    "formateáremos": "formatear",
    "formateásemos": "formatear",
    "formateé": "formatear",
    "formateéis": "formatear",
    "formateó": "formatear",
    "formativa": "formativo",
    "formativas": "formativo",
    "formativos": "formativo",
    "formatos": "formato",
    "forme": "formar",
    "formemos": "formar",
    "formen": "formar",
    "formeros": "formero",
    "formes": "formar",
    "formiatos": "formiato",
    "formicaciones": "formicación",
    "formicantes": "formicante",
    "formicinas": "formicina",
    "formidables": "formidable",
    "formilaciones": "formilación",
    "formo": "formar",
    "formocresoles": "formocresol",
    "formoles": "formol",
    "formones": "formón",
    "formula": "formular",
    "formulaba": "formular",
    "formulabais": "formular",
    "formulaban": "formular",
    "formulabas": "formular",
    "formulaciones": "formulación",
    "formulad": "formular",
    "formulada": "formular",
    "formuladas": "formular",
    "formulado": "formular",
    "formuladora": "formulador",
    "formuladoras": "formulador",
    "formuladores": "formulador",
    "formulados": "formular",
    "formulamos": "formular",
    "formulan": "formular",
    "formulando": "formular",
    "formulara": "formular",
    "formularais": "formular",
    "formularan": "formular",
    "formularas": "formular",
    "formulare": "formular",
    "formulareis": "formular",
    "formularemos": "formular",
    "formularen": "formular",
    "formulares": "formular",
    "formularia": "formulario",
    "formularias": "formulario",
    "formularios": "formulario",
    "formularon": "formular",
    "formulará": "formular",
    "formularán": "formular",
    "formularás": "formular",
    "formularé": "formular",
    "formularéis": "formular",
    "formularía": "formular",
    "formularíais": "formular",
    "formularíamos": "formular",
    "formularían": "formular",
    "formularías": "formular",
    "formulas": "formular",
    "formulase": "formular",
    "formulaseis": "formular",
    "formulasen": "formular",
    "formulases": "formular",
    "formulaste": "formular",
    "formulasteis": "formular",
    "formule": "formular",
    "formulemos": "formular",
    "formulen": "formular",
    "formules": "formular",
    "formulismos": "formulismo",
    "formulistas": "formulista",
    "formulo": "formular",
    "formulábamos": "formular",
    "formuláis": "formular",
    "formuláramos": "formular",
    "formuláremos": "formular",
    "formulásemos": "formular",
    "formulé": "formular",
    "formuléis": "formular",
    "formuló": "formular",
    "formábamos": "formar",
    "formáis": "formar",
    "formáramos": "formar",
    "formáremos": "formar",
    "formásemos": "formar",
    "formé": "formar",
    "forméis": "formar",
    "formó": "formar",
    "formónes": "formón",
    "fornelos": "fornelo",
    "fornica": "fornicar",
    "fornicaba": "fornicar",
    "fornicabais": "fornicar",
    "fornicaban": "fornicar",
    "fornicabas": "fornicar",
    "fornicaciones": "fornicación",
    "fornicad": "fornicar",
    "fornicada": "fornicar",
    "fornicadas": "fornicar",
    "fornicado": "fornicar",
    "fornicadora": "fornicador",
    "fornicadoras": "fornicador",
    "fornicadores": "fornicador",
    "fornicados": "fornicar",
    "fornicamos": "fornicar",
    "fornican": "fornicar",
    "fornicando": "fornicar",
    "fornicara": "fornicar",
    "fornicarais": "fornicar",
    "fornicaran": "fornicar",
    "fornicaras": "fornicar",
    "fornicare": "fornicar",
    "fornicareis": "fornicar",
    "fornicaremos": "fornicar",
    "fornicaren": "fornicar",
    "fornicares": "fornicar",
    "fornicaria": "fornicario",
    "fornicarias": "fornicario",
    "fornicarios": "fornicario",
    "fornicaron": "fornicar",
    "fornicará": "fornicar",
    "fornicarán": "fornicar",
    "fornicarás": "fornicar",
    "fornicaré": "fornicar",
    "fornicaréis": "fornicar",
    "fornicaría": "fornicar",
    "fornicaríais": "fornicar",
    "fornicaríamos": "fornicar",
    "fornicarían": "fornicar",
    "fornicarías": "fornicar",
    "fornicas": "fornicar",
    "fornicase": "fornicar",
    "fornicaseis": "fornicar",
    "fornicasen": "fornicar",
    "fornicases": "fornicar",
    "fornicaste": "fornicar",
    "fornicasteis": "fornicar",
    "fornico": "fornicar",
    "fornicábamos": "fornicar",
    "fornicáis": "fornicar",
    "fornicáramos": "fornicar",
    "fornicáremos": "fornicar",
    "fornicásemos": "fornicar",
    "fornicó": "fornicar",
    "fornida": "fornido",
    "fornidas": "fornido",
    "fornidos": "fornido",
    "fornique": "fornicar",
    "forniquemos": "fornicar",
    "forniquen": "fornicar",
    "forniques": "fornicar",
    "forniqué": "fornicar",
    "forniquéis": "fornicar",
    "fornituras": "fornitura",
    "forofa": "forofo",
    "forofas": "forofo",
    "forofismos": "forofismo",
    "forofos": "forofo",
    "foronas": "forona",
    "foros": "foro",
    "forra": "forrar",
    "forraba": "forrar",
    "forrabais": "forrar",
    "forraban": "forrar",
    "forrabas": "forrar",
    "forraciones": "forración",
    "forrad": "forrar",
    "forrada": "forrar",
    "forradas": "forrar",
    "forrado": "forrar",
    "forrados": "forrar",
    "forrajea": "forrajear",
    "forrajeaba": "forrajear",
    "forrajeabais": "forrajear",
    "forrajeaban": "forrajear",
    "forrajeabas": "forrajear",
    "forrajead": "forrajear",
    "forrajeada": "forrajear",
    "forrajeadas": "forrajear",
    "forrajeado": "forrajear",
    "forrajeadores": "forrajeador",
    "forrajeados": "forrajear",
    "forrajeamos": "forrajear",
    "forrajean": "forrajear",
    "forrajeando": "forrajear",
    "forrajeara": "forrajear",
    "forrajearais": "forrajear",
    "forrajearan": "forrajear",
    "forrajearas": "forrajear",
    "forrajeare": "forrajear",
    "forrajeareis": "forrajear",
    "forrajearemos": "forrajear",
    "forrajearen": "forrajear",
    "forrajeares": "forrajear",
    "forrajearon": "forrajear",
    "forrajeará": "forrajear",
    "forrajearán": "forrajear",
    "forrajearás": "forrajear",
    "forrajearé": "forrajear",
    "forrajearéis": "forrajear",
    "forrajearía": "forrajear",
    "forrajearíais": "forrajear",
    "forrajearíamos": "forrajear",
    "forrajearían": "forrajear",
    "forrajearías": "forrajear",
    "forrajeas": "forrajear",
    "forrajease": "forrajear",
    "forrajeaseis": "forrajear",
    "forrajeasen": "forrajear",
    "forrajeases": "forrajear",
    "forrajeaste": "forrajear",
    "forrajeasteis": "forrajear",
    "forrajee": "forrajear",
    "forrajeemos": "forrajear",
    "forrajeen": "forrajear",
    "forrajees": "forrajear",
    "forrajeo": "forrajear",
    "forrajera": "forrajero",
    "forrajeras": "forrajero",
    "forrajeros": "forrajero",
    "forrajes": "forraje",
    "forrajeábamos": "forrajear",
    "forrajeáis": "forrajear",
    "forrajeáramos": "forrajear",
    "forrajeáremos": "forrajear",
    "forrajeásemos": "forrajear",
    "forrajeé": "forrajear",
    "forrajeéis": "forrajear",
    "forrajeó": "forrajear",
    "forramos": "forrar",
    "forran": "forrar",
    "forrando": "forrar",
    "forrara": "forrar",
    "forrarais": "forrar",
    "forraran": "forrar",
    "forraras": "forrar",
    "forrare": "forrar",
    "forrareis": "forrar",
    "forraremos": "forrar",
    "forraren": "forrar",
    "forrares": "forrar",
    "forraron": "forrar",
    "forrará": "forrar",
    "forrarán": "forrar",
    "forrarás": "forrar",
    "forraré": "forrar",
    "forraréis": "forrar",
    "forraría": "forrar",
    "forraríais": "forrar",
    "forraríamos": "forrar",
    "forrarían": "forrar",
    "forrarías": "forrar",
    "forras": "forrar",
    "forrase": "forrar",
    "forraseis": "forrar",
    "forrasen": "forrar",
    "forrases": "forrar",
    "forraste": "forrar",
    "forrasteis": "forrar",
    "forre": "forrar",
    "forremos": "forrar",
    "forren": "forrar",
    "forres": "forrar",
    "forro": "forrar",
    "forrocinos": "forrocino",
    "forros": "forro",
    "forruncha": "forruncho",
    "forrunchas": "forruncho",
    "forrunchos": "forruncho",
    "forrábamos": "forrar",
    "forráis": "forrar",
    "forráramos": "forrar",
    "forráremos": "forrar",
    "forrásemos": "forrar",
    "forré": "forrar",
    "forréis": "forrar",
    "forró": "forrar",
    "forsitias": "forsitia",
    "forsteritas": "forsterita",
    "fortachona": "fortachón",
    "fortachonas": "fortachón",
    "fortachones": "fortachón",
    "fortalece": "fortalecer",
    "fortaleced": "fortalecer",
    "fortalecedora": "fortalecedor",
    "fortalecedoras": "fortalecedor",
    "fortalecedores": "fortalecedor",
    "fortalecemos": "fortalecer",
    "fortalecen": "fortalecer",
    "fortaleceremos": "fortalecer",
    "fortalecerá": "fortalecer",
    "fortalecerán": "fortalecer",
    "fortalecerás": "fortalecer",
    "fortaleceré": "fortalecer",
    "fortaleceréis": "fortalecer",
    "fortalecería": "fortalecer",
    "fortaleceríais": "fortalecer",
    "fortaleceríamos": "fortalecer",
    "fortalecerían": "fortalecer",
    "fortalecerías": "fortalecer",
    "fortaleces": "fortalecer",
    "fortalecida": "fortalecer",
    "fortalecidas": "fortalecer",
    "fortalecido": "fortalecer",
    "fortalecidos": "fortalecer",
    "fortaleciendo": "fortalecer",
    "fortaleciera": "fortalecer",
    "fortalecierais": "fortalecer",
    "fortalecieran": "fortalecer",
    "fortalecieras": "fortalecer",
    "fortaleciere": "fortalecer",
    "fortaleciereis": "fortalecer",
    "fortalecieren": "fortalecer",
    "fortalecieres": "fortalecer",
    "fortalecieron": "fortalecer",
    "fortaleciese": "fortalecer",
    "fortalecieseis": "fortalecer",
    "fortaleciesen": "fortalecer",
    "fortalecieses": "fortalecer",
    "fortalecimientos": "fortalecimiento",
    "fortalecimos": "fortalecer",
    "fortaleciste": "fortalecer",
    "fortalecisteis": "fortalecer",
    "fortaleciéramos": "fortalecer",
    "fortaleciéremos": "fortalecer",
    "fortaleciésemos": "fortalecer",
    "fortaleció": "fortalecer",
    "fortalecéis": "fortalecer",
    "fortalecí": "fortalecer",
    "fortalecía": "fortalecer",
    "fortalecíais": "fortalecer",
    "fortalecíamos": "fortalecer",
    "fortalecían": "fortalecer",
    "fortalecías": "fortalecer",
    "fortalezas": "fortaleza",
    "fortalezca": "fortalecer",
    "fortalezcamos": "fortalecer",
    "fortalezcan": "fortalecer",
    "fortalezcas": "fortalecer",
    "fortalezco": "fortalecer",
    "fortalezcáis": "fortalecer",
    "fortepianos": "fortepiano",
    "fortifica": "fortificar",
    "fortificaba": "fortificar",
    "fortificabais": "fortificar",
    "fortificaban": "fortificar",
    "fortificabas": "fortificar",
    "fortificaciones": "fortificación",
    "fortificad": "fortificar",
    "fortificada": "fortificar",
    "fortificadas": "fortificar",
    "fortificado": "fortificar",
    "fortificadora": "fortificador",
    "fortificadoras": "fortificador",
    "fortificadores": "fortificador",
    "fortificados": "fortificar",
    "fortificamos": "fortificar",
    "fortifican": "fortificar",
    "fortificando": "fortificar",
    "fortificara": "fortificar",
    "fortificarais": "fortificar",
    "fortificaran": "fortificar",
    "fortificaras": "fortificar",
    "fortificare": "fortificar",
    "fortificareis": "fortificar",
    "fortificaremos": "fortificar",
    "fortificaren": "fortificar",
    "fortificares": "fortificar",
    "fortificaron": "fortificar",
    "fortificará": "fortificar",
    "fortificarán": "fortificar",
    "fortificarás": "fortificar",
    "fortificaré": "fortificar",
    "fortificaréis": "fortificar",
    "fortificaría": "fortificar",
    "fortificaríais": "fortificar",
    "fortificaríamos": "fortificar",
    "fortificarían": "fortificar",
    "fortificarías": "fortificar",
    "fortificas": "fortificar",
    "fortificase": "fortificar",
    "fortificaseis": "fortificar",
    "fortificasen": "fortificar",
    "fortificases": "fortificar",
    "fortificaste": "fortificar",
    "fortificasteis": "fortificar",
    "fortifico": "fortificar",
    "fortificábamos": "fortificar",
    "fortificáis": "fortificar",
    "fortificáramos": "fortificar",
    "fortificáremos": "fortificar",
    "fortificásemos": "fortificar",
    "fortificó": "fortificar",
    "fortifique": "fortificar",
    "fortifiquemos": "fortificar",
    "fortifiquen": "fortificar",
    "fortifiques": "fortificar",
    "fortifiqué": "fortificar",
    "fortifiquéis": "fortificar",
    "fortines": "fortín",
    "fortuita": "fortuito",
    "fortuitas": "fortuito",
    "fortuitos": "fortuito",
    "fortunas": "fortuna",
    "fortínes": "fortín",
    "forums": "fórum",
    "forzaba": "forzar",
    "forzabais": "forzar",
    "forzaban": "forzar",
    "forzabas": "forzar",
    "forzad": "forzar",
    "forzada": "forzar",
    "forzadas": "forzar",
    "forzado": "forzar",
    "forzados": "forzar",
    "forzales": "forzal",
    "forzamientos": "forzamiento",
    "forzamos": "forzar",
    "forzando": "forzar",
    "forzara": "forzar",
    "forzarais": "forzar",
    "forzaran": "forzar",
    "forzaras": "forzar",
    "forzare": "forzar",
    "forzareis": "forzar",
    "forzaremos": "forzar",
    "forzaren": "forzar",
    "forzares": "forzar",
    "forzaron": "forzar",
    "forzará": "forzar",
    "forzarán": "forzar",
    "forzarás": "forzar",
    "forzaré": "forzar",
    "forzaréis": "forzar",
    "forzaría": "forzar",
    "forzaríais": "forzar",
    "forzaríamos": "forzar",
    "forzarían": "forzar",
    "forzarías": "forzar",
    "forzase": "forzar",
    "forzaseis": "forzar",
    "forzasen": "forzar",
    "forzases": "forzar",
    "forzaste": "forzar",
    "forzasteis": "forzar",
    "forzosa": "forzoso",
    "forzosas": "forzoso",
    "forzosos": "forzoso",
    "forzuda": "forzudo",
    "forzudas": "forzudo",
    "forzudos": "forzudo",
    "forzábamos": "forzar",
    "forzáis": "forzar",
    "forzáramos": "forzar",
    "forzáremos": "forzar",
    "forzásemos": "forzar",
    "forzó": "forzar",
    "forámenes": "foramen",
    "foránea": "foráneo",
    "foráneas": "foráneo",
    "foráneos": "foráneo",
    "forúnculos": "forúnculo",
    "fosa": "fosar",
    "fosaba": "fosar",
    "fosabais": "fosar",
    "fosaban": "fosar",
    "fosabas": "fosar",
    "fosad": "fosar",
    "fosada": "fosar",
    "fosadas": "fosar",
    "fosado": "fosar",
    "fosados": "fosar",
    "fosamos": "fosar",
    "fosan": "fosar",
    "fosando": "fosar",
    "fosara": "fosar",
    "fosarais": "fosar",
    "fosaran": "fosar",
    "fosaras": "fosar",
    "fosare": "fosar",
    "fosareis": "fosar",
    "fosaremos": "fosar",
    "fosaren": "fosar",
    "fosares": "fosar",
    "fosaron": "fosar",
    "fosará": "fosar",
    "fosarán": "fosar",
    "fosarás": "fosar",
    "fosaré": "fosar",
    "fosaréis": "fosar",
    "fosaría": "fosar",
    "fosaríais": "fosar",
    "fosaríamos": "fosar",
    "fosarían": "fosar",
    "fosarías": "fosar",
    "fosas": "fosar",
    "fosase": "fosar",
    "fosaseis": "fosar",
    "fosasen": "fosar",
    "fosases": "fosar",
    "fosaste": "fosar",
    "fosasteis": "fosar",
    "fosca": "fosco",
    "foscas": "fosco",
    "foscos": "fosco",
    "fose": "fosar",
    "fosemos": "fosar",
    "fosen": "fosar",
    "foses": "fosar",
    "fosfaminas": "fosfamina",
    "fosfata": "fosfatar",
    "fosfataba": "fosfatar",
    "fosfatabais": "fosfatar",
    "fosfataban": "fosfatar",
    "fosfatabas": "fosfatar",
    "fosfataciones": "fosfatación",
    "fosfatad": "fosfatar",
    "fosfatada": "fosfatar",
    "fosfatadas": "fosfatar",
    "fosfatado": "fosfatar",
    "fosfatados": "fosfatar",
    "fosfatamos": "fosfatar",
    "fosfatan": "fosfatar",
    "fosfatando": "fosfatar",
    "fosfatara": "fosfatar",
    "fosfatarais": "fosfatar",
    "fosfataran": "fosfatar",
    "fosfataras": "fosfatar",
    "fosfatare": "fosfatar",
    "fosfatareis": "fosfatar",
    "fosfataremos": "fosfatar",
    "fosfataren": "fosfatar",
    "fosfatares": "fosfatar",
    "fosfataron": "fosfatar",
    "fosfatará": "fosfatar",
    "fosfatarán": "fosfatar",
    "fosfatarás": "fosfatar",
    "fosfataré": "fosfatar",
    "fosfataréis": "fosfatar",
    "fosfataría": "fosfatar",
    "fosfataríais": "fosfatar",
    "fosfataríamos": "fosfatar",
    "fosfatarían": "fosfatar",
    "fosfatarías": "fosfatar",
    "fosfatas": "fosfatar",
    "fosfatasas": "fosfatasa",
    "fosfatase": "fosfatar",
    "fosfataseis": "fosfatar",
    "fosfatasen": "fosfatar",
    "fosfatases": "fosfatar",
    "fosfataste": "fosfatar",
    "fosfatasteis": "fosfatar",
    "fosfate": "fosfatar",
    "fosfatemos": "fosfatar",
    "fosfaten": "fosfatar",
    "fosfates": "fosfatar",
    "fosfatidilcolinas": "fosfatidilcolina",
    "fosfato": "fosfatar",
    "fosfatos": "fosfato",
    "fosfaturias": "fosfaturia",
    "fosfatábamos": "fosfatar",
    "fosfatáis": "fosfatar",
    "fosfatáramos": "fosfatar",
    "fosfatáremos": "fosfatar",
    "fosfatásemos": "fosfatar",
    "fosfaté": "fosfatar",
    "fosfatéis": "fosfatar",
    "fosfató": "fosfatar",
    "fosfenos": "fosfeno",
    "fosfinas": "fosfina",
    "fosfitos": "fosfito",
    "fosfolipasas": "fosfolipasa",
    "fosfolípidos": "fosfolípido",
    "fosfomolibdatos": "fosfomolibdato",
    "fosfomolíbdica": "fosfomolíbdico",
    "fosfomolíbdicas": "fosfomolíbdico",
    "fosfomolíbdicos": "fosfomolíbdico",
    "fosfonatos": "fosfonato",
    "fosfopenias": "fosfopenia",
    "fosforece": "fosforecer",
    "fosforeced": "fosforecer",
    "fosforecemos": "fosforecer",
    "fosforecen": "fosforecer",
    "fosforeceremos": "fosforecer",
    "fosforecerá": "fosforecer",
    "fosforecerán": "fosforecer",
    "fosforecerás": "fosforecer",
    "fosforeceré": "fosforecer",
    "fosforeceréis": "fosforecer",
    "fosforecería": "fosforecer",
    "fosforeceríais": "fosforecer",
    "fosforeceríamos": "fosforecer",
    "fosforecerían": "fosforecer",
    "fosforecerías": "fosforecer",
    "fosforeces": "fosforecer",
    "fosforecida": "fosforecer",
    "fosforecidas": "fosforecer",
    "fosforecido": "fosforecer",
    "fosforecidos": "fosforecer",
    "fosforeciendo": "fosforecer",
    "fosforeciera": "fosforecer",
    "fosforecierais": "fosforecer",
    "fosforecieran": "fosforecer",
    "fosforecieras": "fosforecer",
    "fosforeciere": "fosforecer",
    "fosforeciereis": "fosforecer",
    "fosforecieren": "fosforecer",
    "fosforecieres": "fosforecer",
    "fosforecieron": "fosforecer",
    "fosforeciese": "fosforecer",
    "fosforecieseis": "fosforecer",
    "fosforeciesen": "fosforecer",
    "fosforecieses": "fosforecer",
    "fosforecimos": "fosforecer",
    "fosforeciste": "fosforecer",
    "fosforecisteis": "fosforecer",
    "fosforeciéramos": "fosforecer",
    "fosforeciéremos": "fosforecer",
    "fosforeciésemos": "fosforecer",
    "fosforeció": "fosforecer",
    "fosforecéis": "fosforecer",
    "fosforecí": "fosforecer",
    "fosforecía": "fosforecer",
    "fosforecíais": "fosforecer",
    "fosforecíamos": "fosforecer",
    "fosforecían": "fosforecer",
    "fosforecías": "fosforecer",
    "fosforera": "fosforero",
    "fosforeras": "fosforero",
    "fosforeros": "fosforero",
    "fosforesce": "fosforescer",
    "fosforesced": "fosforescer",
    "fosforescemos": "fosforescer",
    "fosforescen": "fosforescer",
    "fosforescencias": "fosforescencia",
    "fosforescentes": "fosforescente",
    "fosforesceremos": "fosforescer",
    "fosforescerá": "fosforescer",
    "fosforescerán": "fosforescer",
    "fosforescerás": "fosforescer",
    "fosforesceré": "fosforescer",
    "fosforesceréis": "fosforescer",
    "fosforescería": "fosforescer",
    "fosforesceríais": "fosforescer",
    "fosforesceríamos": "fosforescer",
    "fosforescerían": "fosforescer",
    "fosforescerías": "fosforescer",
    "fosforesces": "fosforescer",
    "fosforescida": "fosforescer",
    "fosforescidas": "fosforescer",
    "fosforescido": "fosforescer",
    "fosforescidos": "fosforescer",
    "fosforesciendo": "fosforescer",
    "fosforesciera": "fosforescer",
    "fosforescierais": "fosforescer",
    "fosforescieran": "fosforescer",
    "fosforescieras": "fosforescer",
    "fosforesciere": "fosforescer",
    "fosforesciereis": "fosforescer",
    "fosforescieren": "fosforescer",
    "fosforescieres": "fosforescer",
    "fosforescieron": "fosforescer",
    "fosforesciese": "fosforescer",
    "fosforescieseis": "fosforescer",
    "fosforesciesen": "fosforescer",
    "fosforescieses": "fosforescer",
    "fosforescimos": "fosforescer",
    "fosforesciste": "fosforescer",
    "fosforescisteis": "fosforescer",
    "fosforesciéramos": "fosforescer",
    "fosforesciéremos": "fosforescer",
    "fosforesciésemos": "fosforescer",
    "fosforesció": "fosforescer",
    "fosforescéis": "fosforescer",
    "fosforescí": "fosforescer",
    "fosforescía": "fosforescer",
    "fosforescíais": "fosforescer",
    "fosforescíamos": "fosforescer",
    "fosforescían": "fosforescer",
    "fosforescías": "fosforescer",
    "fosforezca": "fosforescer",
    "fosforezcamos": "fosforescer",
    "fosforezcan": "fosforescer",
    "fosforezcas": "fosforescer",
    "fosforezco": "fosforescer",
    "fosforezcáis": "fosforescer",
    "fosforilaciones": "fosforilación",
    "fosforismos": "fosforismo",
    "fosforitas": "fosforita",
    "fosforosa": "fosforoso",
    "fosforosas": "fosforoso",
    "fosforoscopios": "fosforoscopio",
    "fosforosos": "fosforoso",
    "fosfotioatos": "fosfotioato",
    "fosfotungstatos": "fosfotungstato",
    "fosfotúngstica": "fosfotúngstico",
    "fosfotúngsticas": "fosfotúngstico",
    "fosfotúngsticos": "fosfotúngstico",
    "fosfuranilitas": "fosfuranilita",
    "fosfuros": "fosfuro",
    "fosfática": "fosfático",
    "fosfáticas": "fosfático",
    "fosfáticos": "fosfático",
    "fosfínica": "fosfínico",
    "fosfínicas": "fosfínico",
    "fosfínicos": "fosfínico",
    "fosfónica": "fosfónico",
    "fosfónicas": "fosfónico",
    "fosfónicos": "fosfónico",
    "fosfórica": "fosfórico",
    "fosfóricas": "fosfórico",
    "fosfóricos": "fosfórico",
    "fosgenaciones": "fosgenación",
    "fosgenitas": "fosgenita",
    "fosgenos": "fosgeno",
    "fosilice": "fosilizar",
    "fosilicemos": "fosilizar",
    "fosilicen": "fosilizar",
    "fosilices": "fosilizar",
    "fosilicé": "fosilizar",
    "fosilicéis": "fosilizar",
    "fosiliza": "fosilizar",
    "fosilizaba": "fosilizar",
    "fosilizabais": "fosilizar",
    "fosilizaban": "fosilizar",
    "fosilizabas": "fosilizar",
    "fosilizaciones": "fosilización",
    "fosilizad": "fosilizar",
    "fosilizada": "fosilizar",
    "fosilizadas": "fosilizar",
    "fosilizado": "fosilizar",
    "fosilizadora": "fosilizador",
    "fosilizadoras": "fosilizador",
    "fosilizadores": "fosilizador",
    "fosilizados": "fosilizar",
    "fosilizamos": "fosilizar",
    "fosilizan": "fosilizar",
    "fosilizando": "fosilizar",
    "fosilizara": "fosilizar",
    "fosilizarais": "fosilizar",
    "fosilizaran": "fosilizar",
    "fosilizaras": "fosilizar",
    "fosilizare": "fosilizar",
    "fosilizareis": "fosilizar",
    "fosilizaremos": "fosilizar",
    "fosilizaren": "fosilizar",
    "fosilizares": "fosilizar",
    "fosilizaron": "fosilizar",
    "fosilizará": "fosilizar",
    "fosilizarán": "fosilizar",
    "fosilizarás": "fosilizar",
    "fosilizaré": "fosilizar",
    "fosilizaréis": "fosilizar",
    "fosilizaría": "fosilizar",
    "fosilizaríais": "fosilizar",
    "fosilizaríamos": "fosilizar",
    "fosilizarían": "fosilizar",
    "fosilizarías": "fosilizar",
    "fosilizas": "fosilizar",
    "fosilizase": "fosilizar",
    "fosilizaseis": "fosilizar",
    "fosilizasen": "fosilizar",
    "fosilizases": "fosilizar",
    "fosilizaste": "fosilizar",
    "fosilizasteis": "fosilizar",
    "fosilizo": "fosilizar",
    "fosilizábamos": "fosilizar",
    "fosilizáis": "fosilizar",
    "fosilizáramos": "fosilizar",
    "fosilizáremos": "fosilizar",
    "fosilizásemos": "fosilizar",
    "fosilizó": "fosilizar",
    "fosilífera": "fosilífero",
    "fosilíferas": "fosilífero",
    "fosilíferos": "fosilífero",
    "fositas": "fosita",
    "foso": "fosar",
    "fosos": "foso",
    "fosqueras": "fosquera",
    "fosvitinas": "fosvitina",
    "fosábamos": "fosar",
    "fosáis": "fosar",
    "fosáramos": "fosar",
    "fosáremos": "fosar",
    "fosásemos": "fosar",
    "fosé": "fosar",
    "foséis": "fosar",
    "fosó": "fosar",
    "fotios": "fotio",
    "fotoactiva": "fotoactivo",
    "fotoactivas": "fotoactivo",
    "fotoactivos": "fotoactivo",
    "fotocinética": "fotocinético",
    "fotocinéticas": "fotocinético",
    "fotocinéticos": "fotocinético",
    "fotocoagulaciones": "fotocoagulación",
    "fotocomponedoras": "fotocomponedora",
    "fotocomposiciones": "fotocomposición",
    "fotoconductibilidades": "fotoconductibilidad",
    "fotoconductiva": "fotoconductivo",
    "fotoconductivas": "fotoconductivo",
    "fotoconductividades": "fotoconductividad",
    "fotoconductivos": "fotoconductivo",
    "fotoconductora": "fotoconductor",
    "fotoconductoras": "fotoconductor",
    "fotoconductores": "fotoconductor",
    "fotocontroles": "fotocontrol",
    "fotocopia": "fotocopiar",
    "fotocopiaba": "fotocopiar",
    "fotocopiabais": "fotocopiar",
    "fotocopiaban": "fotocopiar",
    "fotocopiabas": "fotocopiar",
    "fotocopiad": "fotocopiar",
    "fotocopiada": "fotocopiar",
    "fotocopiadas": "fotocopiar",
    "fotocopiado": "fotocopiar",
    "fotocopiadora": "fotocopiador",
    "fotocopiadoras": "fotocopiadora",
    "fotocopiadores": "fotocopiador",
    "fotocopiados": "fotocopiar",
    "fotocopiamos": "fotocopiar",
    "fotocopian": "fotocopiar",
    "fotocopiando": "fotocopiar",
    "fotocopiara": "fotocopiar",
    "fotocopiarais": "fotocopiar",
    "fotocopiaran": "fotocopiar",
    "fotocopiaras": "fotocopiar",
    "fotocopiare": "fotocopiar",
    "fotocopiareis": "fotocopiar",
    "fotocopiaremos": "fotocopiar",
    "fotocopiaren": "fotocopiar",
    "fotocopiares": "fotocopiar",
    "fotocopiaron": "fotocopiar",
    "fotocopiará": "fotocopiar",
    "fotocopiarán": "fotocopiar",
    "fotocopiarás": "fotocopiar",
    "fotocopiaré": "fotocopiar",
    "fotocopiaréis": "fotocopiar",
    "fotocopiaría": "fotocopiar",
    "fotocopiaríais": "fotocopiar",
    "fotocopiaríamos": "fotocopiar",
    "fotocopiarían": "fotocopiar",
    "fotocopiarías": "fotocopiar",
    "fotocopias": "fotocopiar",
    "fotocopiase": "fotocopiar",
    "fotocopiaseis": "fotocopiar",
    "fotocopiasen": "fotocopiar",
    "fotocopiases": "fotocopiar",
    "fotocopiaste": "fotocopiar",
    "fotocopiasteis": "fotocopiar",
    "fotocopie": "fotocopiar",
    "fotocopiemos": "fotocopiar",
    "fotocopien": "fotocopiar",
    "fotocopies": "fotocopiar",
    "fotocopio": "fotocopiar",
    "fotocopiábamos": "fotocopiar",
    "fotocopiáis": "fotocopiar",
    "fotocopiáramos": "fotocopiar",
    "fotocopiáremos": "fotocopiar",
    "fotocopiásemos": "fotocopiar",
    "fotocopié": "fotocopiar",
    "fotocopiéis": "fotocopiar",
    "fotocopió": "fotocopiar",
    "fotocroísmos": "fotocroísmo",
    "fotocátodos": "fotocátodo",
    "fotocélulas": "fotocélula",
    "fotodesintegraciones": "fotodesintegración",
    "fotodetectores": "fotodetector",
    "fotodiodos": "fotodiodo",
    "fotodisociaciones": "fotodisociación",
    "fotoelasticidades": "fotoelasticidad",
    "fotoelectricidades": "fotoelectricidad",
    "fotoelectrones": "fotoelectrón",
    "fotoelástica": "fotoelástico",
    "fotoelásticas": "fotoelástico",
    "fotoelásticos": "fotoelástico",
    "fotoeléctrica": "fotoeléctrico",
    "fotoeléctricas": "fotoeléctrico",
    "fotoeléctricos": "fotoeléctrico",
    "fotoemisiones": "fotoemisión",
    "fotoemisiva": "fotoemisivo",
    "fotoemisivas": "fotoemisivo",
    "fotoemisivos": "fotoemisivo",
    "fotoemisora": "fotoemisor",
    "fotoemisoras": "fotoemisor",
    "fotoemisores": "fotoemisor",
    "fotoestática": "fotoestático",
    "fotoestáticas": "fotoestático",
    "fotoestáticos": "fotoestático",
    "fotofobias": "fotofobia",
    "fotogenias": "fotogenia",
    "fotogeologías": "fotogeología",
    "fotograba": "fotograbar",
    "fotogrababa": "fotograbar",
    "fotogrababais": "fotograbar",
    "fotogrababan": "fotograbar",
    "fotogrababas": "fotograbar",
    "fotograbaciones": "fotograbación",
    "fotograbad": "fotograbar",
    "fotograbada": "fotograbar",
    "fotograbadas": "fotograbar",
    "fotograbado": "fotograbar",
    "fotograbadora": "fotograbador",
    "fotograbadoras": "fotograbador",
    "fotograbadores": "fotograbador",
    "fotograbados": "fotograbar",
    "fotograbamos": "fotograbar",
    "fotograban": "fotograbar",
    "fotograbando": "fotograbar",
    "fotograbara": "fotograbar",
    "fotograbarais": "fotograbar",
    "fotograbaran": "fotograbar",
    "fotograbaras": "fotograbar",
    "fotograbare": "fotograbar",
    "fotograbareis": "fotograbar",
    "fotograbaremos": "fotograbar",
    "fotograbaren": "fotograbar",
    "fotograbares": "fotograbar",
    "fotograbaron": "fotograbar",
    "fotograbará": "fotograbar",
    "fotograbarán": "fotograbar",
    "fotograbarás": "fotograbar",
    "fotograbaré": "fotograbar",
    "fotograbaréis": "fotograbar",
    "fotograbaría": "fotograbar",
    "fotograbaríais": "fotograbar",
    "fotograbaríamos": "fotograbar",
    "fotograbarían": "fotograbar",
    "fotograbarías": "fotograbar",
    "fotograbas": "fotograbar",
    "fotograbase": "fotograbar",
    "fotograbaseis": "fotograbar",
    "fotograbasen": "fotograbar",
    "fotograbases": "fotograbar",
    "fotograbaste": "fotograbar",
    "fotograbasteis": "fotograbar",
    "fotograbe": "fotograbar",
    "fotograbemos": "fotograbar",
    "fotograben": "fotograbar",
    "fotograbes": "fotograbar",
    "fotograbo": "fotograbar",
    "fotograbábamos": "fotograbar",
    "fotograbáis": "fotograbar",
    "fotograbáramos": "fotograbar",
    "fotograbáremos": "fotograbar",
    "fotograbásemos": "fotograbar",
    "fotograbé": "fotograbar",
    "fotograbéis": "fotograbar",
    "fotograbó": "fotograbar",
    "fotografiaba": "fotografiar",
    "fotografiabais": "fotografiar",
    "fotografiaban": "fotografiar",
    "fotografiabas": "fotografiar",
    "fotografiad": "fotografiar",
    "fotografiada": "fotografiar",
    "fotografiadas": "fotografiar",
    "fotografiado": "fotografiar",
    "fotografiados": "fotografiar",
    "fotografiamos": "fotografiar",
    "fotografiando": "fotografiar",
    "fotografiara": "fotografiar",
    "fotografiarais": "fotografiar",
    "fotografiaran": "fotografiar",
    "fotografiaras": "fotografiar",
    "fotografiare": "fotografiar",
    "fotografiareis": "fotografiar",
    "fotografiaremos": "fotografiar",
    "fotografiaren": "fotografiar",
    "fotografiares": "fotografiar",
    "fotografiaron": "fotografiar",
    "fotografiará": "fotografiar",
    "fotografiarán": "fotografiar",
    "fotografiarás": "fotografiar",
    "fotografiaré": "fotografiar",
    "fotografiaréis": "fotografiar",
    "fotografiaría": "fotografiar",
    "fotografiaríais": "fotografiar",
    "fotografiaríamos": "fotografiar",
    "fotografiarían": "fotografiar",
    "fotografiarías": "fotografiar",
    "fotografiase": "fotografiar",
    "fotografiaseis": "fotografiar",
    "fotografiasen": "fotografiar",
    "fotografiases": "fotografiar",
    "fotografiaste": "fotografiar",
    "fotografiasteis": "fotografiar",
    "fotografiemos": "fotografiar",
    "fotografiábamos": "fotografiar",
    "fotografiáis": "fotografiar",
    "fotografiáramos": "fotografiar",
    "fotografiáremos": "fotografiar",
    "fotografiásemos": "fotografiar",
    "fotografié": "fotografiar",
    "fotografiéis": "fotografiar",
    "fotografió": "fotografiar",
    "fotografía": "fotografiar",
    "fotografían": "fotografiar",
    "fotografías": "fotografiar",
    "fotografíe": "fotografiar",
    "fotografíen": "fotografiar",
    "fotografíes": "fotografiar",
    "fotografío": "fotografiar",
    "fotogramas": "fotograma",
    "fotogrametrías": "fotogrametría",
    "fotogramétrica": "fotogramétrico",
    "fotogramétricas": "fotogramétrico",
    "fotogramétricos": "fotogramétrico",
    "fotográfica": "fotográfico",
    "fotográficas": "fotográfico",
    "fotográficos": "fotográfico",
    "fotográmetros": "fotográmetro",
    "fotogénica": "fotogénico",
    "fotogénicas": "fotogénico",
    "fotogénicos": "fotogénico",
    "fotointerpretaciones": "fotointerpretación",
    "fotolitografiaba": "fotolitografiar",
    "fotolitografiabais": "fotolitografiar",
    "fotolitografiaban": "fotolitografiar",
    "fotolitografiabas": "fotolitografiar",
    "fotolitografiad": "fotolitografiar",
    "fotolitografiada": "fotolitografiar",
    "fotolitografiadas": "fotolitografiar",
    "fotolitografiado": "fotolitografiar",
    "fotolitografiados": "fotolitografiar",
    "fotolitografiamos": "fotolitografiar",
    "fotolitografiando": "fotolitografiar",
    "fotolitografiara": "fotolitografiar",
    "fotolitografiarais": "fotolitografiar",
    "fotolitografiaran": "fotolitografiar",
    "fotolitografiaras": "fotolitografiar",
    "fotolitografiare": "fotolitografiar",
    "fotolitografiareis": "fotolitografiar",
    "fotolitografiaremos": "fotolitografiar",
    "fotolitografiaren": "fotolitografiar",
    "fotolitografiares": "fotolitografiar",
    "fotolitografiaron": "fotolitografiar",
    "fotolitografiará": "fotolitografiar",
    "fotolitografiarán": "fotolitografiar",
    "fotolitografiarás": "fotolitografiar",
    "fotolitografiaré": "fotolitografiar",
    "fotolitografiaréis": "fotolitografiar",
    "fotolitografiaría": "fotolitografiar",
    "fotolitografiaríais": "fotolitografiar",
    "fotolitografiaríamos": "fotolitografiar",
    "fotolitografiarían": "fotolitografiar",
    "fotolitografiarías": "fotolitografiar",
    "fotolitografiase": "fotolitografiar",
    "fotolitografiaseis": "fotolitografiar",
    "fotolitografiasen": "fotolitografiar",
    "fotolitografiases": "fotolitografiar",
    "fotolitografiaste": "fotolitografiar",
    "fotolitografiasteis": "fotolitografiar",
    "fotolitografiemos": "fotolitografiar",
    "fotolitografiábamos": "fotolitografiar",
    "fotolitografiáis": "fotolitografiar",
    "fotolitografiáramos": "fotolitografiar",
    "fotolitografiáremos": "fotolitografiar",
    "fotolitografiásemos": "fotolitografiar",
    "fotolitografié": "fotolitografiar",
    "fotolitografiéis": "fotolitografiar",
    "fotolitografió": "fotolitografiar",
    "fotolitografía": "fotolitografiar",
    "fotolitografían": "fotolitografiar",
    "fotolitografías": "fotolitografiar",
    "fotolitografíe": "fotolitografiar",
    "fotolitografíen": "fotolitografiar",
    "fotolitografíes": "fotolitografiar",
    "fotolitografío": "fotolitografiar",
    "fotolitográfica": "fotolitográfico",
    "fotolitográficas": "fotolitográfico",
    "fotolitográficos": "fotolitográfico",
    "fotolitos": "fotolito",
    "fotoluminiscencias": "fotoluminiscencia",
    "fotomapas": "fotomapa",
    "fotomatones": "fotomatón",
    "fotomecánica": "fotomecánico",
    "fotomecánicas": "fotomecánico",
    "fotomecánicos": "fotomecánico",
    "fotometeoros": "fotometeoro",
    "fotometrías": "fotometría",
    "fotomontajes": "fotomontaje",
    "fotomosaicos": "fotomosaico",
    "fotomultiplicadora": "fotomultiplicador",
    "fotomultiplicadoras": "fotomultiplicador",
    "fotomultiplicadores": "fotomultiplicador",
    "fotométrica": "fotométrico",
    "fotométricas": "fotométrico",
    "fotométricos": "fotométrico",
    "fotonegativa": "fotonegativo",
    "fotonegativas": "fotonegativo",
    "fotonegativos": "fotonegativo",
    "fotones": "fotón",
    "fotonitrosaciones": "fotonitrosación",
    "fotonovelas": "fotonovela",
    "fotoperiodicidades": "fotoperiodicidad",
    "fotoperiodistas": "fotoperiodista",
    "fotopias": "fotopia",
    "fotopilas": "fotopila",
    "fotoplanos": "fotoplano",
    "fotopolímeros": "fotopolímero",
    "fotopositiva": "fotopositivo",
    "fotopositivas": "fotopositivo",
    "fotopositivos": "fotopositivo",
    "fotopsias": "fotopsia",
    "fotoquímica": "fotoquímico",
    "fotoquímicas": "fotoquímico",
    "fotoquímicos": "fotoquímico",
    "fotorreactivacoónes": "fotorreactivación",
    "fotorreceptora": "fotorreceptor",
    "fotorreceptoras": "fotorreceptor",
    "fotorreceptores": "fotorreceptor",
    "fotorresinas": "fotorresina",
    "fotorresistencias": "fotorresistencia",
    "fotos": "foto",
    "fotosensibles": "fotosensible",
    "fotosensores": "fotosensor",
    "fotosferas": "fotosfera",
    "fotosférica": "fotosférico",
    "fotosféricas": "fotosférico",
    "fotosféricos": "fotosférico",
    "fotosintetizadora": "fotosintetizador",
    "fotosintetizadoras": "fotosintetizador",
    "fotosintetizadores": "fotosintetizador",
    "fotosintética": "fotosintético",
    "fotosintéticas": "fotosintético",
    "fotosintéticos": "fotosintético",
    "fotostatos": "fotostato",
    "fototactismos": "fototactismo",
    "fototaxias": "fototaxia",
    "fototecas": "fototeca",
    "fototelegrafías": "fototelegrafía",
    "fototelégrafos": "fototelégrafo",
    "fototerapias": "fototerapia",
    "fototerápica": "fototerápico",
    "fototerápicas": "fototerápico",
    "fototerápicos": "fototerápico",
    "fototipias": "fototipia",
    "fototipografías": "fototipografía",
    "fototipográfica": "fototipográfico",
    "fototipográficas": "fototipográfico",
    "fototipográficos": "fototipográfico",
    "fototiristores": "fototiristor",
    "fototopografías": "fototopografía",
    "fototransistores": "fototransistor",
    "fototriangulaciones": "fototriangulación",
    "fototropismos": "fototropismo",
    "fototubos": "fototubo",
    "fototípica": "fototípico",
    "fototípicas": "fototípico",
    "fototípicos": "fototípico",
    "fotovoltaica": "fotovoltaico",
    "fotovoltaicas": "fotovoltaico",
    "fotovoltaicos": "fotovoltaico",
    "fotutos": "fotuto",
    "fotófila": "fotófilo",
    "fotófilas": "fotófilo",
    "fotófilos": "fotófilo",
    "fotófoba": "fotófobo",
    "fotófobas": "fotófobo",
    "fotófobos": "fotófobo",
    "fotófonos": "fotófono",
    "fotógrafa": "fotógrafo",
    "fotógrafas": "fotógrafo",
    "fotógrafos": "fotógrafo",
    "fotómetros": "fotómetro",
    "fotónica": "fotónico",
    "fotónicas": "fotónico",
    "fotónicos": "fotónico",
    "fotópica": "fotópico",
    "fotópicas": "fotópico",
    "fotópicos": "fotópico",
    "fouettés": "fouetté",
    "foulards": "foulard",
    "fourierismos": "fourierismo",
    "foveales": "foveal",
    "foveolares": "foveolar",
    "foveolas": "foveola",
    "fox-terriers": "fox-terrier",
    "foxterriers": "foxterrier",
    "fracasa": "fracasar",
    "fracasaba": "fracasar",
    "fracasabais": "fracasar",
    "fracasaban": "fracasar",
    "fracasabas": "fracasar",
    "fracasad": "fracasar",
    "fracasada": "fracasar",
    "fracasadas": "fracasar",
    "fracasado": "fracasar",
    "fracasados": "fracasar",
    "fracasamos": "fracasar",
    "fracasan": "fracasar",
    "fracasando": "fracasar",
    "fracasara": "fracasar",
    "fracasarais": "fracasar",
    "fracasaran": "fracasar",
    "fracasaras": "fracasar",
    "fracasare": "fracasar",
    "fracasareis": "fracasar",
    "fracasaremos": "fracasar",
    "fracasaren": "fracasar",
    "fracasares": "fracasar",
    "fracasaron": "fracasar",
    "fracasará": "fracasar",
    "fracasarán": "fracasar",
    "fracasarás": "fracasar",
    "fracasaré": "fracasar",
    "fracasaréis": "fracasar",
    "fracasaría": "fracasar",
    "fracasaríais": "fracasar",
    "fracasaríamos": "fracasar",
    "fracasarían": "fracasar",
    "fracasarías": "fracasar",
    "fracasas": "fracasar",
    "fracasase": "fracasar",
    "fracasaseis": "fracasar",
    "fracasasen": "fracasar",
    "fracasases": "fracasar",
    "fracasaste": "fracasar",
    "fracasasteis": "fracasar",
    "fracase": "fracasar",
    "fracasemos": "fracasar",
    "fracasen": "fracasar",
    "fracases": "fracasar",
    "fracaso": "fracasar",
    "fracasos": "fracaso",
    "fracasábamos": "fracasar",
    "fracasáis": "fracasar",
    "fracasáramos": "fracasar",
    "fracasáremos": "fracasar",
    "fracasásemos": "fracasar",
    "fracasé": "fracasar",
    "fracaséis": "fracasar",
    "fracasó": "fracasar",
    "fracciona": "fraccionar",
    "fraccionaba": "fraccionar",
    "fraccionabais": "fraccionar",
    "fraccionaban": "fraccionar",
    "fraccionabas": "fraccionar",
    "fraccionables": "fraccionable",
    "fraccionad": "fraccionar",
    "fraccionada": "fraccionar",
    "fraccionadas": "fraccionar",
    "fraccionado": "fraccionar",
    "fraccionados": "fraccionar",
    "fraccionales": "fraccional",
    "fraccionamientos": "fraccionamiento",
    "fraccionamos": "fraccionar",
    "fraccionan": "fraccionar",
    "fraccionando": "fraccionar",
    "fraccionara": "fraccionar",
    "fraccionarais": "fraccionar",
    "fraccionaran": "fraccionar",
    "fraccionaras": "fraccionar",
    "fraccionare": "fraccionar",
    "fraccionareis": "fraccionar",
    "fraccionaremos": "fraccionar",
    "fraccionaren": "fraccionar",
    "fraccionares": "fraccionar",
    "fraccionaria": "fraccionario",
    "fraccionarias": "fraccionario",
    "fraccionarios": "fraccionario",
    "fraccionaron": "fraccionar",
    "fraccionará": "fraccionar",
    "fraccionarán": "fraccionar",
    "fraccionarás": "fraccionar",
    "fraccionaré": "fraccionar",
    "fraccionaréis": "fraccionar",
    "fraccionaría": "fraccionar",
    "fraccionaríais": "fraccionar",
    "fraccionaríamos": "fraccionar",
    "fraccionarían": "fraccionar",
    "fraccionarías": "fraccionar",
    "fraccionas": "fraccionar",
    "fraccionase": "fraccionar",
    "fraccionaseis": "fraccionar",
    "fraccionasen": "fraccionar",
    "fraccionases": "fraccionar",
    "fraccionaste": "fraccionar",
    "fraccionasteis": "fraccionar",
    "fraccione": "fraccionar",
    "fraccionemos": "fraccionar",
    "fraccionen": "fraccionar",
    "fracciones": "fraccionar",
    "fracciono": "fraccionar",
    "fraccionábamos": "fraccionar",
    "fraccionáis": "fraccionar",
    "fraccionáramos": "fraccionar",
    "fraccionáremos": "fraccionar",
    "fraccionásemos": "fraccionar",
    "fraccioné": "fraccionar",
    "fraccionéis": "fraccionar",
    "fraccionó": "fraccionar",
    "fracs": "frac",
    "fractocúmulos": "fractocúmulo",
    "fractoestratos": "fractoestrato",
    "fractografías": "fractografía",
    "fractura": "fracturar",
    "fracturaba": "fracturar",
    "fracturabais": "fracturar",
    "fracturaban": "fracturar",
    "fracturabas": "fracturar",
    "fracturad": "fracturar",
    "fracturada": "fracturar",
    "fracturadas": "fracturar",
    "fracturado": "fracturar",
    "fracturados": "fracturar",
    "fracturamos": "fracturar",
    "fracturan": "fracturar",
    "fracturando": "fracturar",
    "fracturara": "fracturar",
    "fracturarais": "fracturar",
    "fracturaran": "fracturar",
    "fracturaras": "fracturar",
    "fracturare": "fracturar",
    "fracturareis": "fracturar",
    "fracturaremos": "fracturar",
    "fracturaren": "fracturar",
    "fracturares": "fracturar",
    "fracturaron": "fracturar",
    "fracturará": "fracturar",
    "fracturarán": "fracturar",
    "fracturarás": "fracturar",
    "fracturaré": "fracturar",
    "fracturaréis": "fracturar",
    "fracturaría": "fracturar",
    "fracturaríais": "fracturar",
    "fracturaríamos": "fracturar",
    "fracturarían": "fracturar",
    "fracturarías": "fracturar",
    "fracturas": "fracturar",
    "fracturase": "fracturar",
    "fracturaseis": "fracturar",
    "fracturasen": "fracturar",
    "fracturases": "fracturar",
    "fracturaste": "fracturar",
    "fracturasteis": "fracturar",
    "fracture": "fracturar",
    "fracturemos": "fracturar",
    "fracturen": "fracturar",
    "fractures": "fracturar",
    "fracturo": "fracturar",
    "fracturábamos": "fracturar",
    "fracturáis": "fracturar",
    "fracturáramos": "fracturar",
    "fracturáremos": "fracturar",
    "fracturásemos": "fracturar",
    "fracturé": "fracturar",
    "fracturéis": "fracturar",
    "fracturó": "fracturar",
    "fragancias": "fragancia",
    "fragantes": "fragante",
    "fragas": "fraga",
    "fragatas": "fragata",
    "fragilidades": "fragilidad",
    "fragilizaciones": "fragilización",
    "fragiácuica": "fragiácuico",
    "fragiácuicas": "fragiácuico",
    "fragiácuicos": "fragiácuico",
    "fragmenta": "fragmentar",
    "fragmentaba": "fragmentar",
    "fragmentabais": "fragmentar",
    "fragmentaban": "fragmentar",
    "fragmentabas": "fragmentar",
    "fragmentaciones": "fragmentación",
    "fragmentad": "fragmentar",
    "fragmentada": "fragmentar",
    "fragmentadas": "fragmentar",
    "fragmentado": "fragmentar",
    "fragmentados": "fragmentar",
    "fragmentamos": "fragmentar",
    "fragmentan": "fragmentar",
    "fragmentando": "fragmentar",
    "fragmentara": "fragmentar",
    "fragmentarais": "fragmentar",
    "fragmentaran": "fragmentar",
    "fragmentaras": "fragmentar",
    "fragmentare": "fragmentar",
    "fragmentareis": "fragmentar",
    "fragmentaremos": "fragmentar",
    "fragmentaren": "fragmentar",
    "fragmentares": "fragmentar",
    "fragmentaria": "fragmentario",
    "fragmentarias": "fragmentario",
    "fragmentariedades": "fragmentariedad",
    "fragmentarios": "fragmentario",
    "fragmentaron": "fragmentar",
    "fragmentará": "fragmentar",
    "fragmentarán": "fragmentar",
    "fragmentarás": "fragmentar",
    "fragmentaré": "fragmentar",
    "fragmentaréis": "fragmentar",
    "fragmentaría": "fragmentar",
    "fragmentaríais": "fragmentar",
    "fragmentaríamos": "fragmentar",
    "fragmentarían": "fragmentar",
    "fragmentarías": "fragmentar",
    "fragmentas": "fragmentar",
    "fragmentase": "fragmentar",
    "fragmentaseis": "fragmentar",
    "fragmentasen": "fragmentar",
    "fragmentases": "fragmentar",
    "fragmentaste": "fragmentar",
    "fragmentasteis": "fragmentar",
    "fragmente": "fragmentar",
    "fragmentemos": "fragmentar",
    "fragmenten": "fragmentar",
    "fragmentes": "fragmentar",
    "fragmentinas": "fragmentina",
    "fragmento": "fragmentar",
    "fragmentos": "fragmento",
    "fragmentábamos": "fragmentar",
    "fragmentáis": "fragmentar",
    "fragmentáramos": "fragmentar",
    "fragmentáremos": "fragmentar",
    "fragmentásemos": "fragmentar",
    "fragmenté": "fragmentar",
    "fragmentéis": "fragmentar",
    "fragmentó": "fragmentar",
    "fragmoplastos": "fragmoplasto",
    "fragmosomas": "fragmosoma",
    "fragores": "fragor",
    "fragorosa": "fragoroso",
    "fragorosas": "fragoroso",
    "fragorosos": "fragoroso",
    "fragosa": "fragoso",
    "fragosas": "fragoso",
    "fragosidades": "fragosidad",
    "fragosos": "fragoso",
    "fragua": "fraguar",
    "fraguaba": "fraguar",
    "fraguabais": "fraguar",
    "fraguaban": "fraguar",
    "fraguabas": "fraguar",
    "fraguables": "fraguable",
    "fraguad": "fraguar",
    "fraguada": "fraguar",
    "fraguadas": "fraguar",
    "fraguado": "fraguar",
    "fraguadora": "fraguador",
    "fraguadoras": "fraguador",
    "fraguadores": "fraguador",
    "fraguados": "fraguar",
    "fraguamos": "fraguar",
    "fraguan": "fraguar",
    "fraguando": "fraguar",
    "fraguara": "fraguar",
    "fraguarais": "fraguar",
    "fraguaran": "fraguar",
    "fraguaras": "fraguar",
    "fraguare": "fraguar",
    "fraguareis": "fraguar",
    "fraguaremos": "fraguar",
    "fraguaren": "fraguar",
    "fraguares": "fraguar",
    "fraguaron": "fraguar",
    "fraguará": "fraguar",
    "fraguarán": "fraguar",
    "fraguarás": "fraguar",
    "fraguaré": "fraguar",
    "fraguaréis": "fraguar",
    "fraguaría": "fraguar",
    "fraguaríais": "fraguar",
    "fraguaríamos": "fraguar",
    "fraguarían": "fraguar",
    "fraguarías": "fraguar",
    "fraguas": "fraguar",
    "fraguase": "fraguar",
    "fraguaseis": "fraguar",
    "fraguasen": "fraguar",
    "fraguases": "fraguar",
    "fraguaste": "fraguar",
    "fraguasteis": "fraguar",
    "fraguo": "fraguar",
    "fraguábamos": "fraguar",
    "fraguáis": "fraguar",
    "fraguáramos": "fraguar",
    "fraguáremos": "fraguar",
    "fraguásemos": "fraguar",
    "fraguó": "fraguar",
    "fragüe": "fraguar",
    "fragüemos": "fraguar",
    "fragüen": "fraguar",
    "fragües": "fraguar",
    "fragüé": "fraguar",
    "fragüéis": "fraguar",
    "frailecillos": "frailecillo",
    "frailejones": "frailejón",
    "frailenga": "frailengo",
    "frailengas": "frailengo",
    "frailengos": "frailengo",
    "fraileros": "frailero",
    "frailerías": "frailería",
    "frailes": "fraile",
    "frailesca": "frailesco",
    "frailescas": "frailesco",
    "frailescos": "frailesco",
    "fraileña": "fraileño",
    "fraileñas": "fraileño",
    "fraileños": "fraileño",
    "frailuna": "frailuno",
    "frailunas": "frailuno",
    "frailunos": "frailuno",
    "frailías": "frailía",
    "frambuesas": "frambuesa",
    "frambuesos": "frambueso",
    "franca": "franco",
    "francachelas": "francachela",
    "francaletes": "francalete",
    "francas": "franco",
    "francesa": "francés",
    "francesadas": "francesada",
    "francesas": "francés",
    "franceses": "francés",
    "francesillas": "francesilla",
    "franchuta": "franchute",
    "franchutas": "franchute",
    "franchutes": "franchute",
    "francios": "francio",
    "franciscana": "franciscano",
    "franciscanas": "franciscano",
    "franciscanos": "franciscano",
    "franciscos": "francisco",
    "francmasona": "francmasón",
    "francmasonas": "francmasón",
    "francmasonerías": "francmasonería",
    "francmasones": "francmasón",
    "francmasónica": "francmasónico",
    "francmasónicas": "francmasónico",
    "francmasónicos": "francmasónico",
    "franco-alemana": "franco-alemán",
    "franco-alemanas": "franco-alemán",
    "franco-alemanes": "franco-alemán",
    "francobordos": "francobordo",
    "francolines": "francolín",
    "francoprovenzales": "francoprovenzal",
    "francos": "franco",
    "francota": "francote",
    "francotas": "francote",
    "francotes": "francote",
    "francotiradora": "francotirador",
    "francotiradoras": "francotirador",
    "francotiradores": "francotirador",
    "francófila": "francófilo",
    "francófilas": "francófilo",
    "francófilos": "francófilo",
    "francófoba": "francófobo",
    "francófobas": "francófobo",
    "francófobos": "francófobo",
    "francófona": "francófono",
    "francófonas": "francófono",
    "francófonos": "francófono",
    "franelas": "franela",
    "frangentes": "frangente",
    "frangibles": "frangible",
    "frangolla": "frangollar",
    "frangollaba": "frangollar",
    "frangollabais": "frangollar",
    "frangollaban": "frangollar",
    "frangollabas": "frangollar",
    "frangollad": "frangollar",
    "frangollada": "frangollar",
    "frangolladas": "frangollar",
    "frangollado": "frangollar",
    "frangollados": "frangollar",
    "frangollamos": "frangollar",
    "frangollan": "frangollar",
    "frangollando": "frangollar",
    "frangollara": "frangollar",
    "frangollarais": "frangollar",
    "frangollaran": "frangollar",
    "frangollaras": "frangollar",
    "frangollare": "frangollar",
    "frangollareis": "frangollar",
    "frangollaremos": "frangollar",
    "frangollaren": "frangollar",
    "frangollares": "frangollar",
    "frangollaron": "frangollar",
    "frangollará": "frangollar",
    "frangollarán": "frangollar",
    "frangollarás": "frangollar",
    "frangollaré": "frangollar",
    "frangollaréis": "frangollar",
    "frangollaría": "frangollar",
    "frangollaríais": "frangollar",
    "frangollaríamos": "frangollar",
    "frangollarían": "frangollar",
    "frangollarías": "frangollar",
    "frangollas": "frangollar",
    "frangollase": "frangollar",
    "frangollaseis": "frangollar",
    "frangollasen": "frangollar",
    "frangollases": "frangollar",
    "frangollaste": "frangollar",
    "frangollasteis": "frangollar",
    "frangolle": "frangollar",
    "frangollemos": "frangollar",
    "frangollen": "frangollar",
    "frangollera": "frangollero",
    "frangolleras": "frangollero",
    "frangolleros": "frangollero",
    "frangolles": "frangollar",
    "frangollo": "frangollar",
    "frangollos": "frangollo",
    "frangollábamos": "frangollar",
    "frangolláis": "frangollar",
    "frangolláramos": "frangollar",
    "frangolláremos": "frangollar",
    "frangollásemos": "frangollar",
    "frangollé": "frangollar",
    "frangolléis": "frangollar",
    "frangolló": "frangollar",
    "frangotes": "frangote",
    "franjas": "franja",
    "franjea": "franjear",
    "franjeaba": "franjear",
    "franjeabais": "franjear",
    "franjeaban": "franjear",
    "franjeabas": "franjear",
    "franjead": "franjear",
    "franjeada": "franjear",
    "franjeadas": "franjear",
    "franjeado": "franjear",
    "franjeados": "franjear",
    "franjeamos": "franjear",
    "franjean": "franjear",
    "franjeando": "franjear",
    "franjeara": "franjear",
    "franjearais": "franjear",
    "franjearan": "franjear",
    "franjearas": "franjear",
    "franjeare": "franjear",
    "franjeareis": "franjear",
    "franjearemos": "franjear",
    "franjearen": "franjear",
    "franjeares": "franjear",
    "franjearon": "franjear",
    "franjeará": "franjear",
    "franjearán": "franjear",
    "franjearás": "franjear",
    "franjearé": "franjear",
    "franjearéis": "franjear",
    "franjearía": "franjear",
    "franjearíais": "franjear",
    "franjearíamos": "franjear",
    "franjearían": "franjear",
    "franjearías": "franjear",
    "franjeas": "franjear",
    "franjease": "franjear",
    "franjeaseis": "franjear",
    "franjeasen": "franjear",
    "franjeases": "franjear",
    "franjeaste": "franjear",
    "franjeasteis": "franjear",
    "franjee": "franjear",
    "franjeemos": "franjear",
    "franjeen": "franjear",
    "franjees": "franjear",
    "franjeo": "franjear",
    "franjeábamos": "franjear",
    "franjeáis": "franjear",
    "franjeáramos": "franjear",
    "franjeáremos": "franjear",
    "franjeásemos": "franjear",
    "franjeé": "franjear",
    "franjeéis": "franjear",
    "franjeó": "franjear",
    "franklinitas": "franklinita",
    "franklins": "franklin",
    "franquea": "franquear",
    "franqueaba": "franquear",
    "franqueabais": "franquear",
    "franqueaban": "franquear",
    "franqueabas": "franquear",
    "franqueables": "franqueable",
    "franquead": "franquear",
    "franqueada": "franquear",
    "franqueadas": "franquear",
    "franqueado": "franquear",
    "franqueadora": "franqueador",
    "franqueadoras": "franqueador",
    "franqueadores": "franqueador",
    "franqueados": "franquear",
    "franqueamientos": "franqueamiento",
    "franqueamos": "franquear",
    "franquean": "franquear",
    "franqueando": "franquear",
    "franqueara": "franquear",
    "franquearais": "franquear",
    "franquearan": "franquear",
    "franquearas": "franquear",
    "franqueare": "franquear",
    "franqueareis": "franquear",
    "franquearemos": "franquear",
    "franquearen": "franquear",
    "franqueares": "franquear",
    "franquearon": "franquear",
    "franqueará": "franquear",
    "franquearán": "franquear",
    "franquearás": "franquear",
    "franquearé": "franquear",
    "franquearéis": "franquear",
    "franquearía": "franquear",
    "franquearíais": "franquear",
    "franquearíamos": "franquear",
    "franquearían": "franquear",
    "franquearías": "franquear",
    "franqueas": "franquear",
    "franquease": "franquear",
    "franqueaseis": "franquear",
    "franqueasen": "franquear",
    "franqueases": "franquear",
    "franqueaste": "franquear",
    "franqueasteis": "franquear",
    "franquee": "franquear",
    "franqueemos": "franquear",
    "franqueen": "franquear",
    "franquees": "franquear",
    "franqueo": "franquear",
    "franqueos": "franqueo",
    "franquezas": "franqueza",
    "franqueábamos": "franquear",
    "franqueáis": "franquear",
    "franqueáramos": "franquear",
    "franqueáremos": "franquear",
    "franqueásemos": "franquear",
    "franqueé": "franquear",
    "franqueéis": "franquear",
    "franqueó": "franquear",
    "franquiciadora": "franquiciador",
    "franquiciadoras": "franquiciador",
    "franquiciadores": "franquiciador",
    "franquicias": "franquicia",
    "franquismos": "franquismo",
    "franquistas": "franquista",
    "frascas": "frasca",
    "frascos": "frasco",
    "frasea": "frasear",
    "fraseaba": "frasear",
    "fraseabais": "frasear",
    "fraseaban": "frasear",
    "fraseabas": "frasear",
    "frasead": "frasear",
    "fraseada": "frasear",
    "fraseadas": "frasear",
    "fraseado": "frasear",
    "fraseados": "frasear",
    "fraseamos": "frasear",
    "frasean": "frasear",
    "fraseando": "frasear",
    "fraseara": "frasear",
    "frasearais": "frasear",
    "frasearan": "frasear",
    "frasearas": "frasear",
    "fraseare": "frasear",
    "fraseareis": "frasear",
    "frasearemos": "frasear",
    "frasearen": "frasear",
    "fraseares": "frasear",
    "frasearon": "frasear",
    "fraseará": "frasear",
    "frasearán": "frasear",
    "frasearás": "frasear",
    "frasearé": "frasear",
    "frasearéis": "frasear",
    "frasearía": "frasear",
    "frasearíais": "frasear",
    "frasearíamos": "frasear",
    "frasearían": "frasear",
    "frasearías": "frasear",
    "fraseas": "frasear",
    "frasease": "frasear",
    "fraseaseis": "frasear",
    "fraseasen": "frasear",
    "fraseases": "frasear",
    "fraseaste": "frasear",
    "fraseasteis": "frasear",
    "frasee": "frasear",
    "fraseemos": "frasear",
    "fraseen": "frasear",
    "frasees": "frasear",
    "fraseo": "frasear",
    "fraseologías": "fraseología",
    "fraseológica": "fraseológico",
    "fraseológicas": "fraseológico",
    "fraseológicos": "fraseológico",
    "fraseos": "fraseo",
    "frases": "frase",
    "fraseábamos": "frasear",
    "fraseáis": "frasear",
    "fraseáramos": "frasear",
    "fraseáremos": "frasear",
    "fraseásemos": "frasear",
    "fraseé": "frasear",
    "fraseéis": "frasear",
    "fraseó": "frasear",
    "frasqueras": "frasquera",
    "frasquetas": "frasqueta",
    "frasquitos": "frasquito",
    "fratases": "fratás",
    "fraterna": "fraterno",
    "fraternales": "fraternal",
    "fraternas": "fraterno",
    "fraternice": "fraternizar",
    "fraternicemos": "fraternizar",
    "fraternicen": "fraternizar",
    "fraternices": "fraternizar",
    "fraternicé": "fraternizar",
    "fraternicéis": "fraternizar",
    "fraternidades": "fraternidad",
    "fraterniza": "fraternizar",
    "fraternizaba": "fraternizar",
    "fraternizabais": "fraternizar",
    "fraternizaban": "fraternizar",
    "fraternizabas": "fraternizar",
    "fraternizaciones": "fraternización",
    "fraternizad": "fraternizar",
    "fraternizada": "fraternizar",
    "fraternizadas": "fraternizar",
    "fraternizado": "fraternizar",
    "fraternizados": "fraternizar",
    "fraternizamos": "fraternizar",
    "fraternizan": "fraternizar",
    "fraternizando": "fraternizar",
    "fraternizara": "fraternizar",
    "fraternizarais": "fraternizar",
    "fraternizaran": "fraternizar",
    "fraternizaras": "fraternizar",
    "fraternizare": "fraternizar",
    "fraternizareis": "fraternizar",
    "fraternizaremos": "fraternizar",
    "fraternizaren": "fraternizar",
    "fraternizares": "fraternizar",
    "fraternizaron": "fraternizar",
    "fraternizará": "fraternizar",
    "fraternizarán": "fraternizar",
    "fraternizarás": "fraternizar",
    "fraternizaré": "fraternizar",
    "fraternizaréis": "fraternizar",
    "fraternizaría": "fraternizar",
    "fraternizaríais": "fraternizar",
    "fraternizaríamos": "fraternizar",
    "fraternizarían": "fraternizar",
    "fraternizarías": "fraternizar",
    "fraternizas": "fraternizar",
    "fraternizase": "fraternizar",
    "fraternizaseis": "fraternizar",
    "fraternizasen": "fraternizar",
    "fraternizases": "fraternizar",
    "fraternizaste": "fraternizar",
    "fraternizasteis": "fraternizar",
    "fraternizo": "fraternizar",
    "fraternizábamos": "fraternizar",
    "fraternizáis": "fraternizar",
    "fraternizáramos": "fraternizar",
    "fraternizáremos": "fraternizar",
    "fraternizásemos": "fraternizar",
    "fraternizó": "fraternizar",
    "fraternos": "fraterno",
    "fratricidas": "fratricida",
    "fratricidios": "fratricidio",
    "fratrías": "fratría",
    "fraudes": "fraude",
    "fraudulencias": "fraudulencia",
    "fraudulenta": "fraudulento",
    "fraudulentas": "fraudulento",
    "fraudulentos": "fraudulento",
    "fraustinas": "fraustina",
    "frayes": "fray",
    "frazadas": "frazada",
    "frece": "frezar",
    "frecemos": "frezar",
    "frecen": "frezar",
    "freces": "frezar",
    "frecuenciales": "frecuencial",
    "frecuencias": "frecuencia",
    "frecuencímetros": "frecuencímetro",
    "frecuenta": "frecuentar",
    "frecuentaba": "frecuentar",
    "frecuentabais": "frecuentar",
    "frecuentaban": "frecuentar",
    "frecuentabas": "frecuentar",
    "frecuentaciones": "frecuentación",
    "frecuentad": "frecuentar",
    "frecuentada": "frecuentar",
    "frecuentadas": "frecuentar",
    "frecuentado": "frecuentar",
    "frecuentadora": "frecuentador",
    "frecuentadoras": "frecuentador",
    "frecuentadores": "frecuentador",
    "frecuentados": "frecuentar",
    "frecuentamos": "frecuentar",
    "frecuentan": "frecuentar",
    "frecuentando": "frecuentar",
    "frecuentara": "frecuentar",
    "frecuentarais": "frecuentar",
    "frecuentaran": "frecuentar",
    "frecuentaras": "frecuentar",
    "frecuentare": "frecuentar",
    "frecuentareis": "frecuentar",
    "frecuentaremos": "frecuentar",
    "frecuentaren": "frecuentar",
    "frecuentares": "frecuentar",
    "frecuentaron": "frecuentar",
    "frecuentará": "frecuentar",
    "frecuentarán": "frecuentar",
    "frecuentarás": "frecuentar",
    "frecuentaré": "frecuentar",
    "frecuentaréis": "frecuentar",
    "frecuentaría": "frecuentar",
    "frecuentaríais": "frecuentar",
    "frecuentaríamos": "frecuentar",
    "frecuentarían": "frecuentar",
    "frecuentarías": "frecuentar",
    "frecuentas": "frecuentar",
    "frecuentase": "frecuentar",
    "frecuentaseis": "frecuentar",
    "frecuentasen": "frecuentar",
    "frecuentases": "frecuentar",
    "frecuentaste": "frecuentar",
    "frecuentasteis": "frecuentar",
    "frecuentativa": "frecuentativo",
    "frecuentativas": "frecuentativo",
    "frecuentativos": "frecuentativo",
    "frecuente": "frecuentar",
    "frecuentemos": "frecuentar",
    "frecuenten": "frecuentar",
    "frecuentes": "frecuente",
    "frecuento": "frecuentar",
    "frecuentábamos": "frecuentar",
    "frecuentáis": "frecuentar",
    "frecuentáramos": "frecuentar",
    "frecuentáremos": "frecuentar",
    "frecuentásemos": "frecuentar",
    "frecuenté": "frecuentar",
    "frecuentéis": "frecuentar",
    "frecuentó": "frecuentar",
    "frecé": "frezar",
    "frecéis": "frezar",
    "freecinemas": "freecinema",
    "fregaba": "fregar",
    "fregabais": "fregar",
    "fregaban": "fregar",
    "fregabas": "fregar",
    "fregad": "fregar",
    "fregada": "fregar",
    "fregadas": "fregar",
    "fregaderas": "fregadera",
    "fregaderos": "fregadero",
    "fregado": "fregar",
    "fregadora": "fregador",
    "fregadoras": "fregador",
    "fregadores": "fregador",
    "fregados": "fregar",
    "fregaduras": "fregadura",
    "fregamientos": "fregamiento",
    "fregamos": "fregar",
    "fregando": "fregar",
    "fregara": "fregar",
    "fregarais": "fregar",
    "fregaran": "fregar",
    "fregaras": "fregar",
    "fregare": "fregar",
    "fregareis": "fregar",
    "fregaremos": "fregar",
    "fregaren": "fregar",
    "fregares": "fregar",
    "fregaron": "fregar",
    "fregará": "fregar",
    "fregarán": "fregar",
    "fregarás": "fregar",
    "fregaré": "fregar",
    "fregaréis": "fregar",
    "fregaría": "fregar",
    "fregaríais": "fregar",
    "fregaríamos": "fregar",
    "fregarían": "fregar",
    "fregarías": "fregar",
    "fregase": "fregar",
    "fregaseis": "fregar",
    "fregasen": "fregar",
    "fregases": "fregar",
    "fregaste": "fregar",
    "fregasteis": "fregar",
    "fregona": "fregón",
    "fregonas": "fregona",
    "fregones": "fregón",
    "fregoniles": "fregonil",
    "fregotea": "fregotear",
    "fregoteaba": "fregotear",
    "fregoteabais": "fregotear",
    "fregoteaban": "fregotear",
    "fregoteabas": "fregotear",
    "fregotead": "fregotear",
    "fregoteada": "fregotear",
    "fregoteadas": "fregotear",
    "fregoteado": "fregotear",
    "fregoteados": "fregotear",
    "fregoteamos": "fregotear",
    "fregotean": "fregotear",
    "fregoteando": "fregotear",
    "fregoteara": "fregotear",
    "fregotearais": "fregotear",
    "fregotearan": "fregotear",
    "fregotearas": "fregotear",
    "fregoteare": "fregotear",
    "fregoteareis": "fregotear",
    "fregotearemos": "fregotear",
    "fregotearen": "fregotear",
    "fregoteares": "fregotear",
    "fregotearon": "fregotear",
    "fregoteará": "fregotear",
    "fregotearán": "fregotear",
    "fregotearás": "fregotear",
    "fregotearé": "fregotear",
    "fregotearéis": "fregotear",
    "fregotearía": "fregotear",
    "fregotearíais": "fregotear",
    "fregotearíamos": "fregotear",
    "fregotearían": "fregotear",
    "fregotearías": "fregotear",
    "fregoteas": "fregotear",
    "fregotease": "fregotear",
    "fregoteaseis": "fregotear",
    "fregoteasen": "fregotear",
    "fregoteases": "fregotear",
    "fregoteaste": "fregotear",
    "fregoteasteis": "fregotear",
    "fregotee": "fregotear",
    "fregoteemos": "fregotear",
    "fregoteen": "fregotear",
    "fregotees": "fregotear",
    "fregoteo": "fregotear",
    "fregoteos": "fregoteo",
    "fregoteábamos": "fregotear",
    "fregoteáis": "fregotear",
    "fregoteáramos": "fregotear",
    "fregoteáremos": "fregotear",
    "fregoteásemos": "fregotear",
    "fregoteé": "fregotear",
    "fregoteéis": "fregotear",
    "fregoteó": "fregotear",
    "freguemos": "fregar",
    "fregué": "fregar",
    "freguéis": "fregar",
    "fregábamos": "fregar",
    "fregáis": "fregar",
    "fregáramos": "fregar",
    "fregáremos": "fregar",
    "fregásemos": "fregar",
    "fregó": "fregar",
    "freibergitas": "freibergita",
    "freidoras": "freidora",
    "freidores": "freidor",
    "freiduras": "freidura",
    "freidurías": "freiduría",
    "freilas": "freila",
    "freiles": "freile",
    "freiras": "freira",
    "freiremos": "freír",
    "freires": "freire",
    "freirá": "freír",
    "freirán": "freír",
    "freirás": "freír",
    "freiré": "freír",
    "freiréis": "freír",
    "freiría": "freír",
    "freiríais": "freír",
    "freiríamos": "freír",
    "freirían": "freír",
    "freirías": "freír",
    "frena": "frenar",
    "frenaba": "frenar",
    "frenabais": "frenar",
    "frenaban": "frenar",
    "frenabas": "frenar",
    "frenad": "frenar",
    "frenada": "frenar",
    "frenadas": "frenar",
    "frenado": "frenar",
    "frenados": "frenar",
    "frenales": "frenal",
    "frenamos": "frenar",
    "frenan": "frenar",
    "frenando": "frenar",
    "frenantes": "frenante",
    "frenara": "frenar",
    "frenarais": "frenar",
    "frenaran": "frenar",
    "frenaras": "frenar",
    "frenare": "frenar",
    "frenareis": "frenar",
    "frenaremos": "frenar",
    "frenaren": "frenar",
    "frenares": "frenar",
    "frenaron": "frenar",
    "frenará": "frenar",
    "frenarán": "frenar",
    "frenarás": "frenar",
    "frenaré": "frenar",
    "frenaréis": "frenar",
    "frenaría": "frenar",
    "frenaríais": "frenar",
    "frenaríamos": "frenar",
    "frenarían": "frenar",
    "frenarías": "frenar",
    "frenas": "frenar",
    "frenase": "frenar",
    "frenaseis": "frenar",
    "frenasen": "frenar",
    "frenases": "frenar",
    "frenaste": "frenar",
    "frenasteis": "frenar",
    "frenazos": "frenazo",
    "frene": "frenar",
    "frenectomías": "frenectomía",
    "frenemos": "frenar",
    "frenen": "frenar",
    "frenera": "frenero",
    "freneras": "frenero",
    "freneros": "frenero",
    "frenerías": "frenería",
    "frenes": "frenar",
    "frenesíes": "frenesí",
    "frenesís": "frenesí",
    "frenicoesplénica": "frenicoesplénico",
    "frenicoesplénicas": "frenicoesplénico",
    "frenicoesplénicos": "frenicoesplénico",
    "frenicopleurales": "frenicopleural",
    "frenillos": "frenillo",
    "freno": "frenar",
    "frenocólica": "frenocólico",
    "frenocólicas": "frenocólico",
    "frenocólicos": "frenocólico",
    "frenologías": "frenología",
    "frenológica": "frenológico",
    "frenológicas": "frenológico",
    "frenológicos": "frenológico",
    "frenopatías": "frenopatía",
    "frenoplastias": "frenoplastia",
    "frenos": "freno",
    "frenotomías": "frenotomía",
    "frentes": "frente",
    "frentistas": "frentista",
    "frenábamos": "frenar",
    "frenáis": "frenar",
    "frenáramos": "frenar",
    "frenáremos": "frenar",
    "frenásemos": "frenar",
    "frené": "frenar",
    "frenéis": "frenar",
    "frenética": "frenético",
    "frenéticas": "frenético",
    "frenéticos": "frenético",
    "frenó": "frenar",
    "frenóloga": "frenólogo",
    "frenólogas": "frenólogo",
    "frenólogos": "frenólogo",
    "frenópatas": "frenópata",
    "freones": "freón",
    "freos": "freo",
    "fresa": "fresar",
    "fresaba": "fresar",
    "fresabais": "fresar",
    "fresaban": "fresar",
    "fresabas": "fresar",
    "fresad": "fresar",
    "fresada": "fresar",
    "fresadas": "fresar",
    "fresado": "fresar",
    "fresadora": "fresador",
    "fresadoras": "fresadora",
    "fresadores": "fresador",
    "fresados": "fresar",
    "fresajes": "fresaje",
    "fresales": "fresal",
    "fresamos": "fresar",
    "fresan": "fresar",
    "fresando": "fresar",
    "fresara": "fresar",
    "fresarais": "fresar",
    "fresaran": "fresar",
    "fresaras": "fresar",
    "fresare": "fresar",
    "fresareis": "fresar",
    "fresaremos": "fresar",
    "fresaren": "fresar",
    "fresares": "fresar",
    "fresaron": "fresar",
    "fresará": "fresar",
    "fresarán": "fresar",
    "fresarás": "fresar",
    "fresaré": "fresar",
    "fresaréis": "fresar",
    "fresaría": "fresar",
    "fresaríais": "fresar",
    "fresaríamos": "fresar",
    "fresarían": "fresar",
    "fresarías": "fresar",
    "fresas": "fresar",
    "fresase": "fresar",
    "fresaseis": "fresar",
    "fresasen": "fresar",
    "fresases": "fresar",
    "fresaste": "fresar",
    "fresasteis": "fresar",
    "fresca": "fresco",
    "frescas": "fresco",
    "frescores": "frescor",
    "frescos": "fresco",
    "frescota": "frescote",
    "frescotas": "frescote",
    "frescotes": "frescote",
    "frescuras": "frescura",
    "frese": "fresar",
    "fresemos": "fresar",
    "fresen": "fresar",
    "fresera": "fresero",
    "freseras": "fresero",
    "freseros": "fresero",
    "freses": "fresar",
    "fresias": "fresia",
    "fresnales": "fresnal",
    "fresnedas": "fresneda",
    "fresnos": "fresno",
    "freso": "fresar",
    "fresones": "fresón",
    "fresquedales": "fresquedal",
    "fresquera": "fresquero",
    "fresqueras": "fresquero",
    "fresqueros": "fresquero",
    "fresábamos": "fresar",
    "fresáis": "fresar",
    "fresáramos": "fresar",
    "fresáremos": "fresar",
    "fresásemos": "fresar",
    "fresé": "fresar",
    "freséis": "fresar",
    "fresó": "fresar",
    "freudiana": "freudiano",
    "freudianas": "freudiano",
    "freudianos": "freudiano",
    "freudismos": "freudismo",
    "freza": "frezar",
    "frezaba": "frezar",
    "frezabais": "frezar",
    "frezaban": "frezar",
    "frezabas": "frezar",
    "frezad": "frezar",
    "frezada": "frezar",
    "frezadas": "frezar",
    "frezado": "frezar",
    "frezados": "frezar",
    "frezamos": "frezar",
    "frezan": "frezar",
    "frezando": "frezar",
    "frezara": "frezar",
    "frezarais": "frezar",
    "frezaran": "frezar",
    "frezaras": "frezar",
    "frezare": "frezar",
    "frezareis": "frezar",
    "frezaremos": "frezar",
    "frezaren": "frezar",
    "frezares": "frezar",
    "frezaron": "frezar",
    "frezará": "frezar",
    "frezarán": "frezar",
    "frezarás": "frezar",
    "frezaré": "frezar",
    "frezaréis": "frezar",
    "frezaría": "frezar",
    "frezaríais": "frezar",
    "frezaríamos": "frezar",
    "frezarían": "frezar",
    "frezarías": "frezar",
    "frezas": "frezar",
    "frezase": "frezar",
    "frezaseis": "frezar",
    "frezasen": "frezar",
    "frezases": "frezar",
    "frezaste": "frezar",
    "frezasteis": "frezar",
    "frezo": "frezar",
    "frezábamos": "frezar",
    "frezáis": "frezar",
    "frezáramos": "frezar",
    "frezáremos": "frezar",
    "frezásemos": "frezar",
    "frezó": "frezar",
    "freática": "freático",
    "freáticas": "freático",
    "freáticos": "freático",
    "freí": "freír",
    "freía": "freír",
    "freíais": "freír",
    "freíamos": "freír",
    "freían": "freír",
    "freías": "freír",
    "freíd": "freír",
    "freída": "freír",
    "freídas": "freír",
    "freído": "freír",
    "freídos": "freír",
    "freímos": "freír",
    "freís": "freír",
    "freíste": "freír",
    "freísteis": "freír",
    "friabilidades": "friabilidad",
    "friables": "friable",
    "frialdades": "frialdad",
    "friamos": "freír",
    "frica": "fricar",
    "fricaba": "fricar",
    "fricabais": "fricar",
    "fricaban": "fricar",
    "fricabas": "fricar",
    "fricaciones": "fricación",
    "fricad": "fricar",
    "fricada": "fricar",
    "fricadas": "fricar",
    "fricado": "fricar",
    "fricados": "fricar",
    "fricamos": "fricar",
    "frican": "fricar",
    "fricando": "fricar",
    "fricandos": "fricandó",
    "fricandós": "fricandó",
    "fricara": "fricar",
    "fricarais": "fricar",
    "fricaran": "fricar",
    "fricaras": "fricar",
    "fricare": "fricar",
    "fricareis": "fricar",
    "fricaremos": "fricar",
    "fricaren": "fricar",
    "fricares": "fricar",
    "fricaron": "fricar",
    "fricará": "fricar",
    "fricarán": "fricar",
    "fricarás": "fricar",
    "fricaré": "fricar",
    "fricaréis": "fricar",
    "fricaría": "fricar",
    "fricaríais": "fricar",
    "fricaríamos": "fricar",
    "fricarían": "fricar",
    "fricarías": "fricar",
    "fricas": "fricar",
    "fricase": "fricar",
    "fricaseis": "fricar",
    "fricasen": "fricar",
    "fricases": "fricar",
    "fricaste": "fricar",
    "fricasteis": "fricar",
    "fricasés": "fricasé",
    "fricativa": "fricativo",
    "fricativas": "fricativo",
    "fricativos": "fricativo",
    "fricatizaciones": "fricatización",
    "fricciona": "friccionar",
    "friccionaba": "friccionar",
    "friccionabais": "friccionar",
    "friccionaban": "friccionar",
    "friccionabas": "friccionar",
    "friccionad": "friccionar",
    "friccionada": "friccionar",
    "friccionadas": "friccionar",
    "friccionado": "friccionar",
    "friccionados": "friccionar",
    "friccionales": "friccional",
    "friccionamos": "friccionar",
    "friccionan": "friccionar",
    "friccionando": "friccionar",
    "friccionara": "friccionar",
    "friccionarais": "friccionar",
    "friccionaran": "friccionar",
    "friccionaras": "friccionar",
    "friccionare": "friccionar",
    "friccionareis": "friccionar",
    "friccionaremos": "friccionar",
    "friccionaren": "friccionar",
    "friccionares": "friccionar",
    "friccionaron": "friccionar",
    "friccionará": "friccionar",
    "friccionarán": "friccionar",
    "friccionarás": "friccionar",
    "friccionaré": "friccionar",
    "friccionaréis": "friccionar",
    "friccionaría": "friccionar",
    "friccionaríais": "friccionar",
    "friccionaríamos": "friccionar",
    "friccionarían": "friccionar",
    "friccionarías": "friccionar",
    "friccionas": "friccionar",
    "friccionase": "friccionar",
    "friccionaseis": "friccionar",
    "friccionasen": "friccionar",
    "friccionases": "friccionar",
    "friccionaste": "friccionar",
    "friccionasteis": "friccionar",
    "friccione": "friccionar",
    "friccionemos": "friccionar",
    "friccionen": "friccionar",
    "fricciones": "friccionar",
    "fricciono": "friccionar",
    "friccionábamos": "friccionar",
    "friccionáis": "friccionar",
    "friccionáramos": "friccionar",
    "friccionáremos": "friccionar",
    "friccionásemos": "friccionar",
    "friccioné": "friccionar",
    "friccionéis": "friccionar",
    "friccionó": "friccionar",
    "frico": "fricar",
    "fricábamos": "fricar",
    "fricáis": "fricar",
    "fricáramos": "fricar",
    "fricáremos": "fricar",
    "fricásemos": "fricar",
    "fricó": "fricar",
    "friega": "fregar",
    "friegan": "fregar",
    "friegas": "friega",
    "friego": "fregar",
    "friegue": "fregar",
    "frieguen": "fregar",
    "friegues": "fregar",
    "friendo": "freír",
    "friera": "freír",
    "frierais": "freír",
    "frieran": "freír",
    "frieras": "friera",
    "friere": "freír",
    "friereis": "freír",
    "frieren": "freír",
    "frieres": "freír",
    "frieron": "freír",
    "friese": "freír",
    "frieseis": "freír",
    "friesen": "freír",
    "frieses": "freír",
    "frigia": "frigio",
    "frigias": "frigio",
    "frigideces": "frigidez",
    "frigios": "frigio",
    "frigorías": "frigoría",
    "frigorífica": "frigorífico",
    "frigoríficas": "frigorífico",
    "frigoríficos": "frigorífico",
    "frigorígen": "frigorígeno",
    "frigorígena": "frigorígeno",
    "frigorígenas": "frigorígeno",
    "frigorígenes": "frigorígeno",
    "frijol": "fríjol",
    "frijolares": "frijolar",
    "frijoles": "fríjol",
    "frijolillos": "frijolillo",
    "frimarios": "frimario",
    "fringílidos": "fringílido",
    "friolenta": "friolento",
    "friolentas": "friolento",
    "friolentos": "friolento",
    "friolera": "friolero",
    "frioleras": "friolero",
    "frioleros": "friolero",
    "frique": "fricar",
    "friquemos": "fricar",
    "friquen": "fricar",
    "friques": "fricar",
    "friqué": "fricar",
    "friquéis": "fricar",
    "frisa": "frisar",
    "frisaba": "frisar",
    "frisabais": "frisar",
    "frisaban": "frisar",
    "frisabas": "frisar",
    "frisad": "frisar",
    "frisada": "frisar",
    "frisadas": "frisar",
    "frisado": "frisar",
    "frisadora": "frisador",
    "frisadoras": "frisador",
    "frisadores": "frisador",
    "frisados": "frisar",
    "frisaduras": "frisadura",
    "frisamos": "frisar",
    "frisan": "frisar",
    "frisando": "frisar",
    "frisara": "frisar",
    "frisarais": "frisar",
    "frisaran": "frisar",
    "frisaras": "frisar",
    "frisare": "frisar",
    "frisareis": "frisar",
    "frisaremos": "frisar",
    "frisaren": "frisar",
    "frisares": "frisar",
    "frisaron": "frisar",
    "frisará": "frisar",
    "frisarán": "frisar",
    "frisarás": "frisar",
    "frisaré": "frisar",
    "frisaréis": "frisar",
    "frisaría": "frisar",
    "frisaríais": "frisar",
    "frisaríamos": "frisar",
    "frisarían": "frisar",
    "frisarías": "frisar",
    "frisas": "frisar",
    "frisase": "frisar",
    "frisaseis": "frisar",
    "frisasen": "frisar",
    "frisases": "frisar",
    "frisaste": "frisar",
    "frisasteis": "frisar",
    "frise": "frisar",
    "frisemos": "frisar",
    "frisen": "frisar",
    "frises": "frisar",
    "frisia": "frisio",
    "frisias": "frisio",
    "frisios": "frisio",
    "friso": "frisar",
    "frisona": "frisón",
    "frisonas": "frisón",
    "frisones": "frisón",
    "frisos": "friso",
    "frisábamos": "frisar",
    "frisáis": "frisar",
    "frisáramos": "frisar",
    "frisáremos": "frisar",
    "frisásemos": "frisar",
    "frisé": "frisar",
    "friséis": "frisar",
    "frisó": "frisar",
    "frita": "freír",
    "fritadas": "fritada",
    "fritados": "fritado",
    "fritangas": "fritanga",
    "fritas": "frita",
    "frito": "freír",
    "fritos": "freír",
    "frituras": "fritura",
    "friulana": "friulano",
    "friulanas": "friulano",
    "friulanos": "friulano",
    "frivolice": "frivolizar",
    "frivolicemos": "frivolizar",
    "frivolicen": "frivolizar",
    "frivolices": "frivolizar",
    "frivolicé": "frivolizar",
    "frivolicéis": "frivolizar",
    "frivolidades": "frivolidad",
    "frivoliza": "frivolizar",
    "frivolizaba": "frivolizar",
    "frivolizabais": "frivolizar",
    "frivolizaban": "frivolizar",
    "frivolizabas": "frivolizar",
    "frivolizaciones": "frivolización",
    "frivolizad": "frivolizar",
    "frivolizada": "frivolizar",
    "frivolizadas": "frivolizar",
    "frivolizado": "frivolizar",
    "frivolizados": "frivolizar",
    "frivolizamos": "frivolizar",
    "frivolizan": "frivolizar",
    "frivolizando": "frivolizar",
    "frivolizara": "frivolizar",
    "frivolizarais": "frivolizar",
    "frivolizaran": "frivolizar",
    "frivolizaras": "frivolizar",
    "frivolizare": "frivolizar",
    "frivolizareis": "frivolizar",
    "frivolizaremos": "frivolizar",
    "frivolizaren": "frivolizar",
    "frivolizares": "frivolizar",
    "frivolizaron": "frivolizar",
    "frivolizará": "frivolizar",
    "frivolizarán": "frivolizar",
    "frivolizarás": "frivolizar",
    "frivolizaré": "frivolizar",
    "frivolizaréis": "frivolizar",
    "frivolizaría": "frivolizar",
    "frivolizaríais": "frivolizar",
    "frivolizaríamos": "frivolizar",
    "frivolizarían": "frivolizar",
    "frivolizarías": "frivolizar",
    "frivolizas": "frivolizar",
    "frivolizase": "frivolizar",
    "frivolizaseis": "frivolizar",
    "frivolizasen": "frivolizar",
    "frivolizases": "frivolizar",
    "frivolizaste": "frivolizar",
    "frivolizasteis": "frivolizar",
    "frivolizo": "frivolizar",
    "frivolizábamos": "frivolizar",
    "frivolizáis": "frivolizar",
    "frivolizáramos": "frivolizar",
    "frivolizáremos": "frivolizar",
    "frivolizásemos": "frivolizar",
    "frivolizó": "frivolizar",
    "friáis": "freír",
    "friéramos": "freír",
    "friéremos": "freír",
    "friésemos": "freír",
    "frió": "freír",
    "frondas": "fronda",
    "frondes": "fronde",
    "frondosa": "frondoso",
    "frondosas": "frondoso",
    "frondosidades": "frondosidad",
    "frondosos": "frondoso",
    "frontaleras": "frontalera",
    "frontales": "frontal",
    "frontalidades": "frontalidad",
    "frontenistas": "frontenista",
    "frontera": "frontero",
    "fronteras": "frontero",
    "fronteriza": "fronterizo",
    "fronterizas": "fronterizo",
    "fronterizos": "fronterizo",
    "fronteros": "frontero",
    "frontiles": "frontil",
    "frontina": "frontino",
    "frontinas": "frontino",
    "frontings": "fronting",
    "frontinos": "frontino",
    "frontispicios": "frontispicio",
    "frontocigomática": "frontocigomático",
    "frontocigomáticas": "frontocigomático",
    "frontocigomáticos": "frontocigomático",
    "frontoetmoidales": "frontoetmoidal",
    "frontolacrimales": "frontolacrimal",
    "frontomalares": "frontomalar",
    "frontomaxilares": "frontomaxilar",
    "frontonasales": "frontonasal",
    "frontones": "frontón",
    "frontooccipitales": "frontooccipital",
    "frontoparietales": "frontoparietal",
    "frontotemporales": "frontotemporal",
    "frontuda": "frontudo",
    "frontudas": "frontudo",
    "frontudos": "frontudo",
    "frota": "frotar",
    "frotaba": "frotar",
    "frotabais": "frotar",
    "frotaban": "frotar",
    "frotabas": "frotar",
    "frotaciones": "frotación",
    "frotad": "frotar",
    "frotada": "frotar",
    "frotadas": "frotar",
    "frotado": "frotar",
    "frotadora": "frotador",
    "frotadoras": "frotador",
    "frotadores": "frotador",
    "frotados": "frotar",
    "frotaduras": "frotadura",
    "frotamientos": "frotamiento",
    "frotamos": "frotar",
    "frotan": "frotar",
    "frotando": "frotar",
    "frotantes": "frotante",
    "frotara": "frotar",
    "frotarais": "frotar",
    "frotaran": "frotar",
    "frotaras": "frotar",
    "frotare": "frotar",
    "frotareis": "frotar",
    "frotaremos": "frotar",
    "frotaren": "frotar",
    "frotares": "frotar",
    "frotaron": "frotar",
    "frotará": "frotar",
    "frotarán": "frotar",
    "frotarás": "frotar",
    "frotaré": "frotar",
    "frotaréis": "frotar",
    "frotaría": "frotar",
    "frotaríais": "frotar",
    "frotaríamos": "frotar",
    "frotarían": "frotar",
    "frotarías": "frotar",
    "frotas": "frotar",
    "frotase": "frotar",
    "frotaseis": "frotar",
    "frotasen": "frotar",
    "frotases": "frotar",
    "frotaste": "frotar",
    "frotasteis": "frotar",
    "frote": "frotar",
    "frotemos": "frotar",
    "froten": "frotar",
    "frotes": "frote",
    "froto": "frotar",
    "frotábamos": "frotar",
    "frotáis": "frotar",
    "frotáramos": "frotar",
    "frotáremos": "frotar",
    "frotásemos": "frotar",
    "froté": "frotar",
    "frotéis": "frotar",
    "frotó": "frotar",
    "fructidores": "fructidor",
    "fructifica": "fructificar",
    "fructificaba": "fructificar",
    "fructificabais": "fructificar",
    "fructificaban": "fructificar",
    "fructificabas": "fructificar",
    "fructificables": "fructificable",
    "fructificaciones": "fructificación",
    "fructificad": "fructificar",
    "fructificada": "fructificar",
    "fructificadas": "fructificar",
    "fructificado": "fructificar",
    "fructificadora": "fructificador",
    "fructificadoras": "fructificador",
    "fructificadores": "fructificador",
    "fructificados": "fructificar",
    "fructificamos": "fructificar",
    "fructifican": "fructificar",
    "fructificando": "fructificar",
    "fructificara": "fructificar",
    "fructificarais": "fructificar",
    "fructificaran": "fructificar",
    "fructificaras": "fructificar",
    "fructificare": "fructificar",
    "fructificareis": "fructificar",
    "fructificaremos": "fructificar",
    "fructificaren": "fructificar",
    "fructificares": "fructificar",
    "fructificaron": "fructificar",
    "fructificará": "fructificar",
    "fructificarán": "fructificar",
    "fructificarás": "fructificar",
    "fructificaré": "fructificar",
    "fructificaréis": "fructificar",
    "fructificaría": "fructificar",
    "fructificaríais": "fructificar",
    "fructificaríamos": "fructificar",
    "fructificarían": "fructificar",
    "fructificarías": "fructificar",
    "fructificas": "fructificar",
    "fructificase": "fructificar",
    "fructificaseis": "fructificar",
    "fructificasen": "fructificar",
    "fructificases": "fructificar",
    "fructificaste": "fructificar",
    "fructificasteis": "fructificar",
    "fructifico": "fructificar",
    "fructificábamos": "fructificar",
    "fructificáis": "fructificar",
    "fructificáramos": "fructificar",
    "fructificáremos": "fructificar",
    "fructificásemos": "fructificar",
    "fructificó": "fructificar",
    "fructifique": "fructificar",
    "fructifiquemos": "fructificar",
    "fructifiquen": "fructificar",
    "fructifiques": "fructificar",
    "fructifiqué": "fructificar",
    "fructifiquéis": "fructificar",
    "fructosas": "fructosa",
    "fructuaria": "fructuario",
    "fructuarias": "fructuario",
    "fructuarios": "fructuario",
    "fructuosa": "fructuoso",
    "fructuosas": "fructuoso",
    "fructuosos": "fructuoso",
    "fructuraria": "fructurario",
    "fructurarias": "fructurario",
    "fructurarios": "fructurario",
    "fructífera": "fructífero",
    "fructíferas": "fructífero",
    "fructíferos": "fructífero",
    "frugales": "frugal",
    "frugalidades": "frugalidad",
    "frugífera": "frugífero",
    "frugíferas": "frugífero",
    "frugíferos": "frugífero",
    "frugívora": "frugívoro",
    "frugívoras": "frugívoro",
    "frugívoros": "frugívoro",
    "fruiciones": "fruición",
    "fruid": "fruir",
    "fruida": "fruir",
    "fruidas": "fruir",
    "fruido": "fruir",
    "fruidos": "fruir",
    "fruimos": "fruir",
    "fruiremos": "fruir",
    "fruirá": "fruir",
    "fruirán": "fruir",
    "fruirás": "fruir",
    "fruiré": "fruir",
    "fruiréis": "fruir",
    "fruiría": "fruir",
    "fruiríais": "fruir",
    "fruiríamos": "fruir",
    "fruirían": "fruir",
    "fruirías": "fruir",
    "fruiste": "fruir",
    "fruisteis": "fruir",
    "frumanticia": "frumanticio",
    "frumanticias": "frumanticio",
    "frumanticios": "frumanticio",
    "frumentaria": "frumentario",
    "frumentarias": "frumentario",
    "frumentarios": "frumentario",
    "frunce": "fruncir",
    "fruncen": "fruncir",
    "frunces": "fruncir",
    "fruncid": "fruncir",
    "fruncida": "fruncir",
    "fruncidas": "fruncir",
    "fruncido": "fruncir",
    "fruncidora": "fruncidor",
    "fruncidoras": "fruncidor",
    "fruncidores": "fruncidor",
    "fruncidos": "fruncir",
    "frunciendo": "fruncir",
    "frunciera": "fruncir",
    "fruncierais": "fruncir",
    "fruncieran": "fruncir",
    "fruncieras": "fruncir",
    "frunciere": "fruncir",
    "frunciereis": "fruncir",
    "fruncieren": "fruncir",
    "fruncieres": "fruncir",
    "fruncieron": "fruncir",
    "frunciese": "fruncir",
    "fruncieseis": "fruncir",
    "frunciesen": "fruncir",
    "fruncieses": "fruncir",
    "fruncimientos": "fruncimiento",
    "fruncimos": "fruncir",
    "frunciremos": "fruncir",
    "fruncirá": "fruncir",
    "fruncirán": "fruncir",
    "fruncirás": "fruncir",
    "frunciré": "fruncir",
    "frunciréis": "fruncir",
    "frunciría": "fruncir",
    "frunciríais": "fruncir",
    "frunciríamos": "fruncir",
    "fruncirían": "fruncir",
    "fruncirías": "fruncir",
    "frunciste": "fruncir",
    "fruncisteis": "fruncir",
    "frunciéramos": "fruncir",
    "frunciéremos": "fruncir",
    "frunciésemos": "fruncir",
    "frunció": "fruncir",
    "fruncí": "fruncir",
    "fruncía": "fruncir",
    "fruncíais": "fruncir",
    "fruncíamos": "fruncir",
    "fruncían": "fruncir",
    "fruncías": "fruncir",
    "fruncís": "fruncir",
    "frunza": "fruncir",
    "frunzamos": "fruncir",
    "frunzan": "fruncir",
    "frunzas": "fruncir",
    "frunzo": "fruncir",
    "frunzáis": "fruncir",
    "fruslera": "fruslero",
    "frusleras": "fruslero",
    "frusleros": "fruslero",
    "fruslerías": "fruslería",
    "frustra": "frustrar",
    "frustraba": "frustrar",
    "frustrabais": "frustrar",
    "frustraban": "frustrar",
    "frustrabas": "frustrar",
    "frustraciones": "frustración",
    "frustrad": "frustrar",
    "frustrada": "frustrar",
    "frustradas": "frustrar",
    "frustrado": "frustrar",
    "frustradora": "frustrador",
    "frustradoras": "frustrador",
    "frustradores": "frustrador",
    "frustrados": "frustrar",
    "frustramos": "frustrar",
    "frustran": "frustrar",
    "frustrando": "frustrar",
    "frustrantes": "frustrante",
    "frustrara": "frustrar",
    "frustrarais": "frustrar",
    "frustraran": "frustrar",
    "frustraras": "frustrar",
    "frustrare": "frustrar",
    "frustrareis": "frustrar",
    "frustraremos": "frustrar",
    "frustraren": "frustrar",
    "frustrares": "frustrar",
    "frustraron": "frustrar",
    "frustrará": "frustrar",
    "frustrarán": "frustrar",
    "frustrarás": "frustrar",
    "frustraré": "frustrar",
    "frustraréis": "frustrar",
    "frustraría": "frustrar",
    "frustraríais": "frustrar",
    "frustraríamos": "frustrar",
    "frustrarían": "frustrar",
    "frustrarías": "frustrar",
    "frustras": "frustrar",
    "frustrase": "frustrar",
    "frustraseis": "frustrar",
    "frustrasen": "frustrar",
    "frustrases": "frustrar",
    "frustraste": "frustrar",
    "frustrasteis": "frustrar",
    "frustratoria": "frustratorio",
    "frustratorias": "frustratorio",
    "frustratorios": "frustratorio",
    "frustre": "frustrar",
    "frustremos": "frustrar",
    "frustren": "frustrar",
    "frustres": "frustrar",
    "frustro": "frustrar",
    "frustrábamos": "frustrar",
    "frustráis": "frustrar",
    "frustránea": "frustráneo",
    "frustráneas": "frustráneo",
    "frustráneos": "frustráneo",
    "frustráramos": "frustrar",
    "frustráremos": "frustrar",
    "frustrásemos": "frustrar",
    "frustré": "frustrar",
    "frustréis": "frustrar",
    "frustró": "frustrar",
    "fruta": "fruto",
    "frutajes": "frutaje",
    "frutales": "frutal",
    "frutas": "fruto",
    "frutera": "frutero",
    "fruteras": "frutero",
    "fruteros": "frutero",
    "fruterías": "frutería",
    "fruticosa": "fruticoso",
    "fruticosas": "fruticoso",
    "fruticosos": "fruticoso",
    "fruticulturas": "fruticultura",
    "frutos": "fruto",
    "fruya": "fruir",
    "fruyamos": "fruir",
    "fruyan": "fruir",
    "fruyas": "fruir",
    "fruye": "fruir",
    "fruyen": "fruir",
    "fruyendo": "fruir",
    "fruyera": "fruir",
    "fruyerais": "fruir",
    "fruyeran": "fruir",
    "fruyeras": "fruir",
    "fruyere": "fruir",
    "fruyereis": "fruir",
    "fruyeren": "fruir",
    "fruyeres": "fruir",
    "fruyeron": "fruir",
    "fruyes": "fruir",
    "fruyese": "fruir",
    "fruyeseis": "fruir",
    "fruyesen": "fruir",
    "fruyeses": "fruir",
    "fruyo": "fruir",
    "fruyáis": "fruir",
    "fruyéramos": "fruir",
    "fruyéremos": "fruir",
    "fruyésemos": "fruir",
    "fruyó": "fruir",
    "fruí": "fruir",
    "fruía": "fruir",
    "fruíais": "fruir",
    "fruíamos": "fruir",
    "fruían": "fruir",
    "fruías": "fruir",
    "fruís": "fruir",
    "frágica": "frágico",
    "frágicas": "frágico",
    "frágicos": "frágico",
    "frágiles": "frágil",
    "frámeas": "frámea",
    "fráncicos": "fráncico",
    "fréjoles": "fréjol",
    "frémitos": "frémito",
    "frénica": "frénico",
    "frénicas": "frénico",
    "frénicos": "frénico",
    "fría": "frío",
    "frían": "freír",
    "frías": "frío",
    "fríe": "freír",
    "fríen": "freír",
    "fríes": "freír",
    "frígida": "frígido",
    "frígidas": "frígido",
    "frígidos": "frígido",
    "fríjoles": "fríjol",
    "frío": "freír",
    "fríos": "frío",
    "frívola": "frívolo",
    "frívolas": "frívolo",
    "frívolos": "frívolo",
    "frútices": "frútice",
    "ftalamidas": "ftalamida",
    "ftalatos": "ftalato",
    "ftaleínas": "ftaleína",
    "ftalilsulfacetamidas": "ftalilsulfacetamida",
    "ftalilsulfatiazoles": "ftalilsulfatiazol",
    "ftalimidas": "ftalimida",
    "ftalonitrilos": "ftalonitrilo",
    "ftps": "ftp",
    "ftálica": "ftálico",
    "ftálicas": "ftálico",
    "ftálicos": "ftálico",
    "fucila": "fucilar",
    "fucilaba": "fucilar",
    "fucilabais": "fucilar",
    "fucilaban": "fucilar",
    "fucilabas": "fucilar",
    "fucilad": "fucilar",
    "fucilada": "fucilar",
    "fuciladas": "fucilar",
    "fucilado": "fucilar",
    "fucilados": "fucilar",
    "fucilamos": "fucilar",
    "fucilan": "fucilar",
    "fucilando": "fucilar",
    "fucilara": "fucilar",
    "fucilarais": "fucilar",
    "fucilaran": "fucilar",
    "fucilaras": "fucilar",
    "fucilare": "fucilar",
    "fucilareis": "fucilar",
    "fucilaremos": "fucilar",
    "fucilaren": "fucilar",
    "fucilares": "fucilar",
    "fucilaron": "fucilar",
    "fucilará": "fucilar",
    "fucilarán": "fucilar",
    "fucilarás": "fucilar",
    "fucilaré": "fucilar",
    "fucilaréis": "fucilar",
    "fucilaría": "fucilar",
    "fucilaríais": "fucilar",
    "fucilaríamos": "fucilar",
    "fucilarían": "fucilar",
    "fucilarías": "fucilar",
    "fucilas": "fucilar",
    "fucilase": "fucilar",
    "fucilaseis": "fucilar",
    "fucilasen": "fucilar",
    "fucilases": "fucilar",
    "fucilaste": "fucilar",
    "fucilasteis": "fucilar",
    "fucilazos": "fucilazo",
    "fucile": "fucilar",
    "fucilemos": "fucilar",
    "fucilen": "fucilar",
    "fuciles": "fucilar",
    "fucilo": "fucilar",
    "fucilábamos": "fucilar",
    "fuciláis": "fucilar",
    "fuciláramos": "fucilar",
    "fuciláremos": "fucilar",
    "fucilásemos": "fucilar",
    "fucilé": "fucilar",
    "fuciléis": "fucilar",
    "fuciló": "fucilar",
    "fucsias": "fucsia",
    "fucsinas": "fucsina",
    "fucsitas": "fucsita",
    "fucáceas": "fucácea",
    "fue": "ser",
    "fuegos": "fuego",
    "fuel-oils": "fuel-oil",
    "fuella": "follar",
    "fuellan": "follar",
    "fuellas": "follar",
    "fuelle": "follar",
    "fuellen": "follar",
    "fuelles": "fuelle",
    "fuello": "follar",
    "fuengiroleña": "fuengiroleño",
    "fuengiroleñas": "fuengiroleño",
    "fuengiroleños": "fuengiroleño",
    "fuentes": "fuente",
    "fuera": "ser",
    "fuerais": "ser",
    "fueran": "ser",
    "fueras": "ser",
    "fuerce": "forzar",
    "fuercen": "forzar",
    "fuerces": "forzar",
    "fuere": "ser",
    "fuereis": "ser",
    "fueren": "ser",
    "fueres": "ser",
    "fueristas": "fuerista",
    "fueron": "ser",
    "fueros": "fuero",
    "fuertes": "fuerte",
    "fuerza": "forzar",
    "fuerzan": "forzar",
    "fuerzas": "fuerza",
    "fuerzo": "forzar",
    "fuese": "ser",
    "fueseis": "ser",
    "fuesen": "ser",
    "fueses": "ser",
    "fuetazos": "fuetazo",
    "fuetes": "fuete",
    "fuets": "fuet",
    "fufos": "fufo",
    "fuga": "fugar",
    "fugaba": "fugar",
    "fugabais": "fugar",
    "fugaban": "fugar",
    "fugabas": "fugar",
    "fugaces": "fugaz",
    "fugacidades": "fugacidad",
    "fugad": "fugar",
    "fugada": "fugar",
    "fugadas": "fugar",
    "fugado": "fugar",
    "fugados": "fugar",
    "fugamos": "fugar",
    "fugan": "fugar",
    "fugando": "fugar",
    "fugara": "fugar",
    "fugarais": "fugar",
    "fugaran": "fugar",
    "fugaras": "fugar",
    "fugare": "fugar",
    "fugareis": "fugar",
    "fugaremos": "fugar",
    "fugaren": "fugar",
    "fugares": "fugar",
    "fugaron": "fugar",
    "fugará": "fugar",
    "fugarán": "fugar",
    "fugarás": "fugar",
    "fugaré": "fugar",
    "fugaréis": "fugar",
    "fugaría": "fugar",
    "fugaríais": "fugar",
    "fugaríamos": "fugar",
    "fugarían": "fugar",
    "fugarías": "fugar",
    "fugas": "fugar",
    "fugase": "fugar",
    "fugaseis": "fugar",
    "fugasen": "fugar",
    "fugases": "fugar",
    "fugaste": "fugar",
    "fugasteis": "fugar",
    "fugazes": "fugaz",
    "fugitiva": "fugitivo",
    "fugitivas": "fugitivo",
    "fugitivos": "fugitivo",
    "fugitómetros": "fugitómetro",
    "fugo": "fugar",
    "fugue": "fugar",
    "fuguemos": "fugar",
    "fuguen": "fugar",
    "fugues": "fugar",
    "fugué": "fugar",
    "fuguéis": "fugar",
    "fugábamos": "fugar",
    "fugáis": "fugar",
    "fugáramos": "fugar",
    "fugáremos": "fugar",
    "fugásemos": "fugar",
    "fugó": "fugar",
    "fui": "ser",
    "fuimos": "ser",
    "fuiste": "ser",
    "fuisteis": "ser",
    "fulana": "fulano",
    "fulanas": "fulano",
    "fulanos": "fulano",
    "fulares": "fular",
    "fulastres": "fulastre",
    "fulbes": "fulbe",
    "fulbitos": "fulbito",
    "fulcros": "fulcro",
    "fulera": "fulero",
    "fuleras": "fulero",
    "fuleros": "fulero",
    "fulge": "fulgir",
    "fulgen": "fulgir",
    "fulgencias": "fulgencia",
    "fulgentes": "fulgente",
    "fulges": "fulgir",
    "fulgid": "fulgir",
    "fulgida": "fulgir",
    "fulgidas": "fulgir",
    "fulgido": "fulgir",
    "fulgidos": "fulgir",
    "fulgiendo": "fulgir",
    "fulgiera": "fulgir",
    "fulgierais": "fulgir",
    "fulgieran": "fulgir",
    "fulgieras": "fulgir",
    "fulgiere": "fulgir",
    "fulgiereis": "fulgir",
    "fulgieren": "fulgir",
    "fulgieres": "fulgir",
    "fulgieron": "fulgir",
    "fulgiese": "fulgir",
    "fulgieseis": "fulgir",
    "fulgiesen": "fulgir",
    "fulgieses": "fulgir",
    "fulgimos": "fulgir",
    "fulgiremos": "fulgir",
    "fulgirá": "fulgir",
    "fulgirán": "fulgir",
    "fulgirás": "fulgir",
    "fulgiré": "fulgir",
    "fulgiréis": "fulgir",
    "fulgiría": "fulgir",
    "fulgiríais": "fulgir",
    "fulgiríamos": "fulgir",
    "fulgirían": "fulgir",
    "fulgirías": "fulgir",
    "fulgiste": "fulgir",
    "fulgisteis": "fulgir",
    "fulgiéramos": "fulgir",
    "fulgiéremos": "fulgir",
    "fulgiésemos": "fulgir",
    "fulgió": "fulgir",
    "fulgores": "fulgor",
    "fulgura": "fulgurar",
    "fulguraba": "fulgurar",
    "fulgurabais": "fulgurar",
    "fulguraban": "fulgurar",
    "fulgurabas": "fulgurar",
    "fulguraciones": "fulguración",
    "fulgurad": "fulgurar",
    "fulgurada": "fulgurar",
    "fulguradas": "fulgurar",
    "fulgurado": "fulgurar",
    "fulgurados": "fulgurar",
    "fulguramos": "fulgurar",
    "fulguran": "fulgurar",
    "fulgurando": "fulgurar",
    "fulgurantes": "fulgurante",
    "fulgurara": "fulgurar",
    "fulgurarais": "fulgurar",
    "fulguraran": "fulgurar",
    "fulguraras": "fulgurar",
    "fulgurare": "fulgurar",
    "fulgurareis": "fulgurar",
    "fulguraremos": "fulgurar",
    "fulguraren": "fulgurar",
    "fulgurares": "fulgurar",
    "fulguraron": "fulgurar",
    "fulgurará": "fulgurar",
    "fulgurarán": "fulgurar",
    "fulgurarás": "fulgurar",
    "fulguraré": "fulgurar",
    "fulguraréis": "fulgurar",
    "fulguraría": "fulgurar",
    "fulguraríais": "fulgurar",
    "fulguraríamos": "fulgurar",
    "fulgurarían": "fulgurar",
    "fulgurarías": "fulgurar",
    "fulguras": "fulgurar",
    "fulgurase": "fulgurar",
    "fulguraseis": "fulgurar",
    "fulgurasen": "fulgurar",
    "fulgurases": "fulgurar",
    "fulguraste": "fulgurar",
    "fulgurasteis": "fulgurar",
    "fulgure": "fulgurar",
    "fulguremos": "fulgurar",
    "fulguren": "fulgurar",
    "fulgures": "fulgurar",
    "fulguritas": "fulgurita",
    "fulguro": "fulgurar",
    "fulgurosa": "fulguroso",
    "fulgurosas": "fulguroso",
    "fulgurosos": "fulguroso",
    "fulgurábamos": "fulgurar",
    "fulguráis": "fulgurar",
    "fulguráramos": "fulgurar",
    "fulguráremos": "fulgurar",
    "fulgurásemos": "fulgurar",
    "fulguré": "fulgurar",
    "fulguréis": "fulgurar",
    "fulguró": "fulgurar",
    "fulgí": "fulgir",
    "fulgía": "fulgir",
    "fulgíais": "fulgir",
    "fulgíamos": "fulgir",
    "fulgían": "fulgir",
    "fulgías": "fulgir",
    "fulgís": "fulgir",
    "fulgúrea": "fulgúreo",
    "fulgúreas": "fulgúreo",
    "fulgúreos": "fulgúreo",
    "fuliginosa": "fuliginoso",
    "fuliginosas": "fuliginoso",
    "fuliginosidades": "fuliginosidad",
    "fuliginosos": "fuliginoso",
    "fuligos": "fuligo",
    "fulja": "fulgir",
    "fuljamos": "fulgir",
    "fuljan": "fulgir",
    "fuljas": "fulgir",
    "fuljo": "fulgir",
    "fuljáis": "fulgir",
    "fullera": "fullero",
    "fulleras": "fullero",
    "fulleresca": "fulleresco",
    "fullerescas": "fulleresco",
    "fullerescos": "fulleresco",
    "fulleros": "fullero",
    "fullerías": "fullería",
    "fullonas": "fullona",
    "fulmares": "fulmar",
    "fulmicotones": "fulmicotón",
    "fulmina": "fulminar",
    "fulminaba": "fulminar",
    "fulminabais": "fulminar",
    "fulminaban": "fulminar",
    "fulminabas": "fulminar",
    "fulminaciones": "fulminación",
    "fulminad": "fulminar",
    "fulminada": "fulminar",
    "fulminadas": "fulminar",
    "fulminado": "fulminar",
    "fulminadora": "fulminador",
    "fulminadoras": "fulminador",
    "fulminadores": "fulminador",
    "fulminados": "fulminar",
    "fulminamos": "fulminar",
    "fulminan": "fulminar",
    "fulminando": "fulminar",
    "fulminantes": "fulminante",
    "fulminara": "fulminar",
    "fulminarais": "fulminar",
    "fulminaran": "fulminar",
    "fulminaras": "fulminar",
    "fulminare": "fulminar",
    "fulminareis": "fulminar",
    "fulminaremos": "fulminar",
    "fulminaren": "fulminar",
    "fulminares": "fulminar",
    "fulminaron": "fulminar",
    "fulminará": "fulminar",
    "fulminarán": "fulminar",
    "fulminarás": "fulminar",
    "fulminaré": "fulminar",
    "fulminaréis": "fulminar",
    "fulminaría": "fulminar",
    "fulminaríais": "fulminar",
    "fulminaríamos": "fulminar",
    "fulminarían": "fulminar",
    "fulminarías": "fulminar",
    "fulminas": "fulminar",
    "fulminase": "fulminar",
    "fulminaseis": "fulminar",
    "fulminasen": "fulminar",
    "fulminases": "fulminar",
    "fulminaste": "fulminar",
    "fulminasteis": "fulminar",
    "fulminatos": "fulminato",
    "fulminatrices": "fulminatriz",
    "fulmine": "fulminar",
    "fulminemos": "fulminar",
    "fulminen": "fulminar",
    "fulmines": "fulminar",
    "fulmino": "fulminar",
    "fulminosa": "fulminoso",
    "fulminosas": "fulminoso",
    "fulminosos": "fulminoso",
    "fulminábamos": "fulminar",
    "fulmináis": "fulminar",
    "fulmináramos": "fulminar",
    "fulmináremos": "fulminar",
    "fulminásemos": "fulminar",
    "fulminé": "fulminar",
    "fulminéis": "fulminar",
    "fulminó": "fulminar",
    "fulmínea": "fulmíneo",
    "fulmíneas": "fulmíneo",
    "fulmíneos": "fulmíneo",
    "fuma": "fumar",
    "fumaba": "fumar",
    "fumabais": "fumar",
    "fumaban": "fumar",
    "fumabas": "fumar",
    "fumad": "fumar",
    "fumada": "fumar",
    "fumadas": "fumar",
    "fumaderos": "fumadero",
    "fumado": "fumar",
    "fumadora": "fumador",
    "fumadoras": "fumador",
    "fumadores": "fumador",
    "fumados": "fumar",
    "fumamos": "fumar",
    "fuman": "fumar",
    "fumando": "fumar",
    "fumara": "fumar",
    "fumaradas": "fumarada",
    "fumarais": "fumar",
    "fumaran": "fumar",
    "fumaras": "fumar",
    "fumaratos": "fumarato",
    "fumare": "fumar",
    "fumareis": "fumar",
    "fumareles": "fumarel",
    "fumaremos": "fumar",
    "fumaren": "fumar",
    "fumares": "fumar",
    "fumarias": "fumaria",
    "fumarolas": "fumarola",
    "fumaron": "fumar",
    "fumará": "fumar",
    "fumarán": "fumar",
    "fumarás": "fumar",
    "fumaré": "fumar",
    "fumaréis": "fumar",
    "fumaría": "fumar",
    "fumaríais": "fumar",
    "fumaríamos": "fumar",
    "fumarían": "fumar",
    "fumarías": "fumar",
    "fumas": "fumar",
    "fumase": "fumar",
    "fumaseis": "fumar",
    "fumasen": "fumar",
    "fumases": "fumar",
    "fumaste": "fumar",
    "fumasteis": "fumar",
    "fumatas": "fumata",
    "fume": "fumar",
    "fumemos": "fumar",
    "fumen": "fumar",
    "fumes": "fumar",
    "fumiga": "fumigar",
    "fumigaba": "fumigar",
    "fumigabais": "fumigar",
    "fumigaban": "fumigar",
    "fumigabas": "fumigar",
    "fumigaciones": "fumigación",
    "fumigad": "fumigar",
    "fumigada": "fumigar",
    "fumigadas": "fumigar",
    "fumigado": "fumigar",
    "fumigadora": "fumigador",
    "fumigadoras": "fumigador",
    "fumigadores": "fumigador",
    "fumigados": "fumigar",
    "fumigamos": "fumigar",
    "fumigan": "fumigar",
    "fumigando": "fumigar",
    "fumigantes": "fumigante",
    "fumigara": "fumigar",
    "fumigarais": "fumigar",
    "fumigaran": "fumigar",
    "fumigaras": "fumigar",
    "fumigare": "fumigar",
    "fumigareis": "fumigar",
    "fumigaremos": "fumigar",
    "fumigaren": "fumigar",
    "fumigares": "fumigar",
    "fumigaron": "fumigar",
    "fumigará": "fumigar",
    "fumigarán": "fumigar",
    "fumigarás": "fumigar",
    "fumigaré": "fumigar",
    "fumigaréis": "fumigar",
    "fumigaría": "fumigar",
    "fumigaríais": "fumigar",
    "fumigaríamos": "fumigar",
    "fumigarían": "fumigar",
    "fumigarías": "fumigar",
    "fumigas": "fumigar",
    "fumigase": "fumigar",
    "fumigaseis": "fumigar",
    "fumigasen": "fumigar",
    "fumigases": "fumigar",
    "fumigaste": "fumigar",
    "fumigasteis": "fumigar",
    "fumigatoria": "fumigatorio",
    "fumigatorias": "fumigatorio",
    "fumigatorios": "fumigatorio",
    "fumigo": "fumigar",
    "fumigue": "fumigar",
    "fumiguemos": "fumigar",
    "fumiguen": "fumigar",
    "fumigues": "fumigar",
    "fumigué": "fumigar",
    "fumiguéis": "fumigar",
    "fumigábamos": "fumigar",
    "fumigáis": "fumigar",
    "fumigáramos": "fumigar",
    "fumigáremos": "fumigar",
    "fumigásemos": "fumigar",
    "fumigó": "fumigar",
    "fumistas": "fumista",
    "fumisterías": "fumistería",
    "fumo": "fumar",
    "fumosa": "fumoso",
    "fumosas": "fumoso",
    "fumosidades": "fumosidad",
    "fumosos": "fumoso",
    "fumábamos": "fumar",
    "fumáis": "fumar",
    "fumáramos": "fumar",
    "fumáremos": "fumar",
    "fumárica": "fumárico",
    "fumáricas": "fumárico",
    "fumáricos": "fumárico",
    "fumásemos": "fumar",
    "fumé": "fumar",
    "fuméis": "fumar",
    "fumífera": "fumífero",
    "fumíferas": "fumífero",
    "fumíferos": "fumífero",
    "fumígena": "fumígeno",
    "fumígenas": "fumígeno",
    "fumígenos": "fumígeno",
    "fumívora": "fumívoro",
    "fumívoras": "fumívoro",
    "fumívoros": "fumívoro",
    "fumó": "fumar",
    "funambulistas": "funambulista",
    "funciona": "funcionar",
    "funcionaba": "funcionar",
    "funcionabais": "funcionar",
    "funcionaban": "funcionar",
    "funcionabas": "funcionar",
    "funcionad": "funcionar",
    "funcionada": "funcionar",
    "funcionadas": "funcionar",
    "funcionado": "funcionar",
    "funcionados": "funcionar",
    "funcionales": "funcional",
    "funcionalidades": "funcionalidad",
    "funcionalismos": "funcionalismo",
    "funcionalistas": "funcionalista",
    "funcionamientos": "funcionamiento",
    "funcionamos": "funcionar",
    "funcionan": "funcionar",
    "funcionando": "funcionar",
    "funcionara": "funcionar",
    "funcionarais": "funcionar",
    "funcionaran": "funcionar",
    "funcionaras": "funcionar",
    "funcionare": "funcionar",
    "funcionareis": "funcionar",
    "funcionaremos": "funcionar",
    "funcionaren": "funcionar",
    "funcionares": "funcionar",
    "funcionaria": "funcionario",
    "funcionariados": "funcionariado",
    "funcionarias": "funcionario",
    "funcionarios": "funcionario",
    "funcionarismos": "funcionarismo",
    "funcionaron": "funcionar",
    "funcionará": "funcionar",
    "funcionarán": "funcionar",
    "funcionarás": "funcionar",
    "funcionaré": "funcionar",
    "funcionaréis": "funcionar",
    "funcionaría": "funcionar",
    "funcionaríais": "funcionar",
    "funcionaríamos": "funcionar",
    "funcionarían": "funcionar",
    "funcionarías": "funcionar",
    "funcionas": "funcionar",
    "funcionase": "funcionar",
    "funcionaseis": "funcionar",
    "funcionasen": "funcionar",
    "funcionases": "funcionar",
    "funcionaste": "funcionar",
    "funcionasteis": "funcionar",
    "funcione": "funcionar",
    "funcionemos": "funcionar",
    "funcionen": "funcionar",
    "funciones": "funcionar",
    "funciono": "funcionar",
    "funcionábamos": "funcionar",
    "funcionáis": "funcionar",
    "funcionáramos": "funcionar",
    "funcionáremos": "funcionar",
    "funcionásemos": "funcionar",
    "funcioné": "funcionar",
    "funcionéis": "funcionar",
    "funcionó": "funcionar",
    "funda": "fundir",
    "fundaba": "fundar",
    "fundabais": "fundar",
    "fundaban": "fundar",
    "fundabas": "fundar",
    "fundacionales": "fundacional",
    "fundacionalismos": "fundacionalismo",
    "fundaciones": "fundación",
    "fundad": "fundar",
    "fundada": "fundar",
    "fundadas": "fundar",
    "fundado": "fundar",
    "fundadora": "fundador",
    "fundadoras": "fundador",
    "fundadores": "fundador",
    "fundados": "fundar",
    "fundamenta": "fundamentar",
    "fundamentaba": "fundamentar",
    "fundamentabais": "fundamentar",
    "fundamentaban": "fundamentar",
    "fundamentabas": "fundamentar",
    "fundamentaciones": "fundamentación",
    "fundamentad": "fundamentar",
    "fundamentada": "fundamentar",
    "fundamentadas": "fundamentar",
    "fundamentado": "fundamentar",
    "fundamentados": "fundamentar",
    "fundamentales": "fundamental",
    "fundamentalismos": "fundamentalismo",
    "fundamentalistas": "fundamentalista",
    "fundamentamos": "fundamentar",
    "fundamentan": "fundamentar",
    "fundamentando": "fundamentar",
    "fundamentara": "fundamentar",
    "fundamentarais": "fundamentar",
    "fundamentaran": "fundamentar",
    "fundamentaras": "fundamentar",
    "fundamentare": "fundamentar",
    "fundamentareis": "fundamentar",
    "fundamentaremos": "fundamentar",
    "fundamentaren": "fundamentar",
    "fundamentares": "fundamentar",
    "fundamentaron": "fundamentar",
    "fundamentará": "fundamentar",
    "fundamentarán": "fundamentar",
    "fundamentarás": "fundamentar",
    "fundamentaré": "fundamentar",
    "fundamentaréis": "fundamentar",
    "fundamentaría": "fundamentar",
    "fundamentaríais": "fundamentar",
    "fundamentaríamos": "fundamentar",
    "fundamentarían": "fundamentar",
    "fundamentarías": "fundamentar",
    "fundamentas": "fundamentar",
    "fundamentase": "fundamentar",
    "fundamentaseis": "fundamentar",
    "fundamentasen": "fundamentar",
    "fundamentases": "fundamentar",
    "fundamentaste": "fundamentar",
    "fundamentasteis": "fundamentar",
    "fundamente": "fundamentar",
    "fundamentemos": "fundamentar",
    "fundamenten": "fundamentar",
    "fundamentes": "fundamentar",
    "fundamento": "fundamentar",
    "fundamentos": "fundamento",
    "fundamentábamos": "fundamentar",
    "fundamentáis": "fundamentar",
    "fundamentáramos": "fundamentar",
    "fundamentáremos": "fundamentar",
    "fundamentásemos": "fundamentar",
    "fundamenté": "fundamentar",
    "fundamentéis": "fundamentar",
    "fundamentó": "fundamentar",
    "fundamos": "fundir",
    "fundan": "fundir",
    "fundando": "fundar",
    "fundara": "fundar",
    "fundarais": "fundar",
    "fundaran": "fundar",
    "fundaras": "fundar",
    "fundare": "fundar",
    "fundareis": "fundar",
    "fundaremos": "fundar",
    "fundaren": "fundar",
    "fundares": "fundar",
    "fundaria": "fundario",
    "fundarias": "fundario",
    "fundarios": "fundario",
    "fundaron": "fundar",
    "fundará": "fundar",
    "fundarán": "fundar",
    "fundarás": "fundar",
    "fundaré": "fundar",
    "fundaréis": "fundar",
    "fundaría": "fundar",
    "fundaríais": "fundar",
    "fundaríamos": "fundar",
    "fundarían": "fundar",
    "fundarías": "fundar",
    "fundas": "fundir",
    "fundase": "fundar",
    "fundaseis": "fundar",
    "fundasen": "fundar",
    "fundases": "fundar",
    "fundaste": "fundar",
    "fundasteis": "fundar",
    "funde": "fundir",
    "fundemos": "fundar",
    "funden": "fundir",
    "fundentes": "fundente",
    "fundes": "fundir",
    "fundibles": "fundible",
    "fundibularios": "fundibulario",
    "fundiciones": "fundición",
    "fundid": "fundir",
    "fundida": "fundir",
    "fundidas": "fundir",
    "fundido": "fundir",
    "fundidora": "fundidor",
    "fundidoras": "fundidora",
    "fundidores": "fundidor",
    "fundidos": "fundir",
    "fundiendo": "fundir",
    "fundiera": "fundir",
    "fundierais": "fundir",
    "fundieran": "fundir",
    "fundieras": "fundir",
    "fundiere": "fundir",
    "fundiereis": "fundir",
    "fundieren": "fundir",
    "fundieres": "fundir",
    "fundieron": "fundir",
    "fundiese": "fundir",
    "fundieseis": "fundir",
    "fundiesen": "fundir",
    "fundieses": "fundir",
    "fundiformes": "fundiforme",
    "fundimos": "fundir",
    "fundiremos": "fundir",
    "fundirá": "fundir",
    "fundirán": "fundir",
    "fundirás": "fundir",
    "fundiré": "fundir",
    "fundiréis": "fundir",
    "fundiría": "fundir",
    "fundiríais": "fundir",
    "fundiríamos": "fundir",
    "fundirían": "fundir",
    "fundirías": "fundir",
    "fundiste": "fundir",
    "fundisteis": "fundir",
    "fundiéramos": "fundir",
    "fundiéremos": "fundir",
    "fundiésemos": "fundir",
    "fundió": "fundir",
    "fundo": "fundir",
    "fundos": "fundo",
    "fundábamos": "fundar",
    "fundáis": "fundir",
    "fundáramos": "fundar",
    "fundáremos": "fundar",
    "fundásemos": "fundar",
    "fundé": "fundar",
    "fundéis": "fundar",
    "fundí": "fundir",
    "fundía": "fundir",
    "fundíais": "fundir",
    "fundíamos": "fundir",
    "fundían": "fundir",
    "fundías": "fundir",
    "fundís": "fundir",
    "fundó": "fundar",
    "funerales": "funeral",
    "funeraria": "funerario",
    "funerarias": "funerario",
    "funerarios": "funerario",
    "funesta": "funesto",
    "funestas": "funesto",
    "funestos": "funesto",
    "fungales": "fungal",
    "funge": "fungir",
    "fungen": "fungir",
    "funges": "fungir",
    "fungibles": "fungible",
    "fungicidas": "fungicida",
    "fungid": "fungir",
    "fungida": "fungir",
    "fungidas": "fungir",
    "fungido": "fungir",
    "fungidos": "fungir",
    "fungiendo": "fungir",
    "fungiera": "fungir",
    "fungierais": "fungir",
    "fungieran": "fungir",
    "fungieras": "fungir",
    "fungiere": "fungir",
    "fungiereis": "fungir",
    "fungieren": "fungir",
    "fungieres": "fungir",
    "fungieron": "fungir",
    "fungiese": "fungir",
    "fungieseis": "fungir",
    "fungiesen": "fungir",
    "fungieses": "fungir",
    "fungiformes": "fungiforme",
    "fungimientos": "fungimiento",
    "fungimos": "fungir",
    "fungiremos": "fungir",
    "fungirá": "fungir",
    "fungirán": "fungir",
    "fungirás": "fungir",
    "fungiré": "fungir",
    "fungiréis": "fungir",
    "fungiría": "fungir",
    "fungiríais": "fungir",
    "fungiríamos": "fungir",
    "fungirían": "fungir",
    "fungirías": "fungir",
    "fungistatos": "fungistato",
    "fungiste": "fungir",
    "fungisteis": "fungir",
    "fungistática": "fungistático",
    "fungistáticas": "fungistático",
    "fungistáticos": "fungistático",
    "fungiéramos": "fungir",
    "fungiéremos": "fungir",
    "fungiésemos": "fungir",
    "fungió": "fungir",
    "fungosa": "fungoso",
    "fungosas": "fungoso",
    "fungosidades": "fungosidad",
    "fungosos": "fungoso",
    "funguicidas": "funguicida",
    "fungí": "fungir",
    "fungía": "fungir",
    "fungíais": "fungir",
    "fungíamos": "fungir",
    "fungían": "fungir",
    "fungías": "fungir",
    "fungís": "fungir",
    "funiculares": "funicular",
    "funja": "fungir",
    "funjamos": "fungir",
    "funjan": "fungir",
    "funjas": "fungir",
    "funjo": "fungir",
    "funjáis": "fungir",
    "funámbula": "funámbulo",
    "funámbulas": "funámbulo",
    "funámbulos": "funámbulo",
    "funérea": "funéreo",
    "funéreas": "funéreo",
    "funéreos": "funéreo",
    "funículos": "funículo",
    "fura": "furo",
    "furanos": "furano",
    "furas": "furo",
    "furcaciones": "furcación",
    "furcias": "furcia",
    "furfurales": "furfural",
    "furfurácea": "furfuráceo",
    "furfuráceas": "furfuráceo",
    "furfuráceos": "furfuráceo",
    "furgones": "furgón",
    "furgonetas": "furgoneta",
    "furias": "furia",
    "furibunda": "furibundo",
    "furibundas": "furibundo",
    "furibundos": "furibundo",
    "furierismos": "furierismo",
    "furieristas": "furierista",
    "furilacrílica": "furilacrílico",
    "furilacrílicas": "furilacrílico",
    "furilacrílicos": "furilacrílico",
    "furiosa": "furioso",
    "furiosas": "furioso",
    "furiosos": "furioso",
    "furoatos": "furoato",
    "furores": "furor",
    "furos": "furo",
    "furrieles": "furriel",
    "furrieres": "furrier",
    "furtiva": "furtivo",
    "furtivas": "furtivo",
    "furtivismos": "furtivismo",
    "furtivos": "furtivo",
    "furánica": "furánico",
    "furánicas": "furánico",
    "furánicos": "furánico",
    "furúnculos": "furúnculo",
    "fusada": "fusado",
    "fusadas": "fusado",
    "fusados": "fusado",
    "fusarenonas": "fusarenona",
    "fusas": "fusa",
    "fuselada": "fuselado",
    "fuseladas": "fuselado",
    "fuselados": "fuselado",
    "fuselajes": "fuselaje",
    "fusenos": "fuseno",
    "fusibilidades": "fusibilidad",
    "fusibles": "fusible",
    "fusiformes": "fusiforme",
    "fusila": "fusilar",
    "fusilaba": "fusilar",
    "fusilabais": "fusilar",
    "fusilaban": "fusilar",
    "fusilabas": "fusilar",
    "fusilad": "fusilar",
    "fusilada": "fusilar",
    "fusiladas": "fusilar",
    "fusilado": "fusilar",
    "fusilados": "fusilar",
    "fusilamientos": "fusilamiento",
    "fusilamos": "fusilar",
    "fusilan": "fusilar",
    "fusilando": "fusilar",
    "fusilara": "fusilar",
    "fusilarais": "fusilar",
    "fusilaran": "fusilar",
    "fusilaras": "fusilar",
    "fusilare": "fusilar",
    "fusilareis": "fusilar",
    "fusilaremos": "fusilar",
    "fusilaren": "fusilar",
    "fusilares": "fusilar",
    "fusilaron": "fusilar",
    "fusilará": "fusilar",
    "fusilarán": "fusilar",
    "fusilarás": "fusilar",
    "fusilaré": "fusilar",
    "fusilaréis": "fusilar",
    "fusilaría": "fusilar",
    "fusilaríais": "fusilar",
    "fusilaríamos": "fusilar",
    "fusilarían": "fusilar",
    "fusilarías": "fusilar",
    "fusilas": "fusilar",
    "fusilase": "fusilar",
    "fusilaseis": "fusilar",
    "fusilasen": "fusilar",
    "fusilases": "fusilar",
    "fusilaste": "fusilar",
    "fusilasteis": "fusilar",
    "fusile": "fusilar",
    "fusilemos": "fusilar",
    "fusilen": "fusilar",
    "fusilera": "fusilero",
    "fusileras": "fusilero",
    "fusileros": "fusilero",
    "fusilerías": "fusilería",
    "fusiles": "fusilar",
    "fusilo": "fusilar",
    "fusilábamos": "fusilar",
    "fusiláis": "fusilar",
    "fusiláramos": "fusilar",
    "fusiláremos": "fusilar",
    "fusilásemos": "fusilar",
    "fusilé": "fusilar",
    "fusiléis": "fusilar",
    "fusiló": "fusilar",
    "fusiona": "fusionar",
    "fusionaba": "fusionar",
    "fusionabais": "fusionar",
    "fusionaban": "fusionar",
    "fusionabas": "fusionar",
    "fusionad": "fusionar",
    "fusionada": "fusionar",
    "fusionadas": "fusionar",
    "fusionado": "fusionar",
    "fusionados": "fusionar",
    "fusionamos": "fusionar",
    "fusionan": "fusionar",
    "fusionando": "fusionar",
    "fusionara": "fusionar",
    "fusionarais": "fusionar",
    "fusionaran": "fusionar",
    "fusionaras": "fusionar",
    "fusionare": "fusionar",
    "fusionareis": "fusionar",
    "fusionaremos": "fusionar",
    "fusionaren": "fusionar",
    "fusionares": "fusionar",
    "fusionaron": "fusionar",
    "fusionará": "fusionar",
    "fusionarán": "fusionar",
    "fusionarás": "fusionar",
    "fusionaré": "fusionar",
    "fusionaréis": "fusionar",
    "fusionaría": "fusionar",
    "fusionaríais": "fusionar",
    "fusionaríamos": "fusionar",
    "fusionarían": "fusionar",
    "fusionarías": "fusionar",
    "fusionas": "fusionar",
    "fusionase": "fusionar",
    "fusionaseis": "fusionar",
    "fusionasen": "fusionar",
    "fusionases": "fusionar",
    "fusionaste": "fusionar",
    "fusionasteis": "fusionar",
    "fusione": "fusionar",
    "fusionemos": "fusionar",
    "fusionen": "fusionar",
    "fusiones": "fusionar",
    "fusionistas": "fusionista",
    "fusiono": "fusionar",
    "fusionábamos": "fusionar",
    "fusionáis": "fusionar",
    "fusionáramos": "fusionar",
    "fusionáremos": "fusionar",
    "fusionásemos": "fusionar",
    "fusioné": "fusionar",
    "fusionéis": "fusionar",
    "fusionó": "fusionar",
    "fusitas": "fusita",
    "fuslinas": "fuslina",
    "fusobacterias": "fusobacteria",
    "fusoespirilares": "fusoespirilar",
    "fusores": "fusor",
    "fusos": "fuso",
    "fustada": "fustado",
    "fustadas": "fustado",
    "fustados": "fustado",
    "fustales": "fustal",
    "fustanelas": "fustanela",
    "fustanes": "fustán",
    "fustas": "fusta",
    "fustazos": "fustazo",
    "fustaños": "fustaño",
    "fustera": "fustero",
    "fusteras": "fustero",
    "fusteros": "fustero",
    "fustes": "fuste",
    "fustetes": "fustete",
    "fustiga": "fustigar",
    "fustigaba": "fustigar",
    "fustigabais": "fustigar",
    "fustigaban": "fustigar",
    "fustigabas": "fustigar",
    "fustigaciones": "fustigación",
    "fustigad": "fustigar",
    "fustigada": "fustigar",
    "fustigadas": "fustigar",
    "fustigado": "fustigar",
    "fustigadora": "fustigador",
    "fustigadoras": "fustigador",
    "fustigadores": "fustigador",
    "fustigados": "fustigar",
    "fustigamos": "fustigar",
    "fustigan": "fustigar",
    "fustigando": "fustigar",
    "fustigara": "fustigar",
    "fustigarais": "fustigar",
    "fustigaran": "fustigar",
    "fustigaras": "fustigar",
    "fustigare": "fustigar",
    "fustigareis": "fustigar",
    "fustigaremos": "fustigar",
    "fustigaren": "fustigar",
    "fustigares": "fustigar",
    "fustigaron": "fustigar",
    "fustigará": "fustigar",
    "fustigarán": "fustigar",
    "fustigarás": "fustigar",
    "fustigaré": "fustigar",
    "fustigaréis": "fustigar",
    "fustigaría": "fustigar",
    "fustigaríais": "fustigar",
    "fustigaríamos": "fustigar",
    "fustigarían": "fustigar",
    "fustigarías": "fustigar",
    "fustigas": "fustigar",
    "fustigase": "fustigar",
    "fustigaseis": "fustigar",
    "fustigasen": "fustigar",
    "fustigases": "fustigar",
    "fustigaste": "fustigar",
    "fustigasteis": "fustigar",
    "fustigo": "fustigar",
    "fustigue": "fustigar",
    "fustiguemos": "fustigar",
    "fustiguen": "fustigar",
    "fustigues": "fustigar",
    "fustigué": "fustigar",
    "fustiguéis": "fustigar",
    "fustigábamos": "fustigar",
    "fustigáis": "fustigar",
    "fustigáramos": "fustigar",
    "fustigáremos": "fustigar",
    "fustigásemos": "fustigar",
    "fustigó": "fustigar",
    "futbitos": "futbito",
    "futbolera": "futbolero",
    "futboleras": "futbolero",
    "futboleros": "futbolero",
    "futboles": "futbol",
    "futbolines": "futbolín",
    "futbolistas": "futbolista",
    "futbolística": "futbolístico",
    "futbolísticas": "futbolístico",
    "futbolísticos": "futbolístico",
    "futesas": "futesa",
    "futilidades": "futilidad",
    "futones": "futón",
    "futura": "futuro",
    "futuraria": "futurario",
    "futurarias": "futurario",
    "futurarios": "futurario",
    "futuras": "futuro",
    "futuribles": "futurible",
    "futuriciones": "futurición",
    "futuridades": "futuridad",
    "futurismos": "futurismo",
    "futuristas": "futurista",
    "futurologías": "futurología",
    "futuros": "futuro",
    "futurística": "futurístico",
    "futurísticas": "futurístico",
    "futurísticos": "futurístico",
    "futuróloga": "futurólogo",
    "futurólogas": "futurólogo",
    "futurólogos": "futurólogo",
    "fuéramos": "ser",
    "fuéremos": "ser",
    "fuésemos": "ser",
    "fábricas": "fábrica",
    "fábulas": "fábula",
    "fáciles": "fácil",
    "fáctica": "fáctico",
    "fácticas": "fáctico",
    "fácticos": "fáctico",
    "fáculas": "fácula",
    "fágica": "fágico",
    "fágicas": "fágico",
    "fágicos": "fágico",
    "fálica": "fálico",
    "fálicas": "fálico",
    "fálicos": "fálico",
    "fámula": "fámulo",
    "fámulas": "fámulo",
    "fámulos": "fámulo",
    "fárfaras": "fárfara",
    "fármacos": "fármaco",
    "fárragos": "fárrago",
    "fásoles": "fásol",
    "fática": "fático",
    "fáticas": "fático",
    "fáticos": "fático",
    "fáunica": "fáunico",
    "fáunicas": "fáunico",
    "fáunicos": "fáunico",
    "féculas": "fécula",
    "félibriges": "félibrige",
    "félida": "félido",
    "félidas": "félido",
    "félidos": "félido",
    "félsica": "félsico",
    "félsicas": "félsico",
    "félsicos": "félsico",
    "fémica": "fémico",
    "fémicas": "fémico",
    "fémicos": "fémico",
    "féminas": "fémina",
    "fémures": "fémur",
    "fénixs": "fénix",
    "féretros": "féretro",
    "férrea": "férreo",
    "férreas": "férreo",
    "férreos": "férreo",
    "férrica": "férrico",
    "férricas": "férrico",
    "férricos": "férrico",
    "fértiles": "fértil",
    "férulas": "férula",
    "férvida": "férvido",
    "férvidas": "férvido",
    "férvidos": "férvido",
    "fétida": "fétido",
    "fétidas": "fétido",
    "fétidos": "fétido",
    "fía": "fiar",
    "fían": "fiar",
    "fías": "fiar",
    "fíats": "fíat",
    "fíbrica": "fíbrico",
    "fíbricas": "fíbrico",
    "fíbricos": "fíbrico",
    "fíbulas": "fíbula",
    "fíe": "fiar",
    "fíen": "fiar",
    "fíes": "fiar",
    "fígaros": "fígaro",
    "fílmica": "fílmico",
    "fílmicas": "fílmico",
    "fílmicos": "fílmico",
    "fímica": "fímico",
    "fímicas": "fímico",
    "fímicos": "fímico",
    "fío": "fiar",
    "física": "físico",
    "físicas": "físico",
    "físicos": "físico",
    "físiles": "físil",
    "fístulas": "fístula",
    "fítica": "fítico",
    "fíticas": "fítico",
    "fíticos": "fítico",
    "fócidos": "fócido",
    "fóculos": "fóculo",
    "fólderes": "fólder",
    "fólica": "fólico",
    "fólicas": "fólico",
    "fólicos": "fólico",
    "fónica": "fónico",
    "fónicas": "fónico",
    "fónicos": "fónico",
    "fórmica": "fórmico",
    "fórmicas": "fórmico",
    "fórmicos": "fórmico",
    "fórmulas": "fórmula",
    "fórnixes": "fórnix",
    "fórums": "fórum",
    "fósforos": "fósforo",
    "fósiles": "fósil",
    "fótica": "fótico",
    "fóticas": "fótico",
    "fóticos": "fótico",
    "fóveas": "fóvea",
    "fúcares": "fúcar",
    "fúlgida": "fúlgido",
    "fúlgidas": "fúlgido",
    "fúlgidos": "fúlgido",
    "fúlvica": "fúlvico",
    "fúlvicas": "fúlvico",
    "fúlvicos": "fúlvico",
    "fúndica": "fúndico",
    "fúndicas": "fúndico",
    "fúndicos": "fúndico",
    "fúnebres": "fúnebre",
    "fútboles": "fútbol",
    "fútbols": "fútbol",
    "fútiles": "fútil",
    "g": "gramo",
    "gabacha": "gabacho",
    "gabachadas": "gabachada",
    "gabachas": "gabacho",
    "gabachos": "gabacho",
    "gabanes": "gabán",
    "gabardinas": "gabardina",
    "gabarras": "gabarra",
    "gabarrera": "gabarrero",
    "gabarreras": "gabarrero",
    "gabarreros": "gabarrero",
    "gabarros": "gabarro",
    "gabazos": "gabazo",
    "gabejos": "gabejo",
    "gabelas": "gabela",
    "gabinetes": "gabinete",
    "gabletes": "gablete",
    "gabonesa": "gabonés",
    "gabonesas": "gabonés",
    "gaboneses": "gabonés",
    "gabra": "gabro",
    "gabras": "gabro",
    "gabros": "gabro",
    "gacelas": "gacela",
    "gaceles": "gacel",
    "gacetas": "gaceta",
    "gacetera": "gacetero",
    "gaceteras": "gacetero",
    "gaceteros": "gacetero",
    "gacetillas": "gacetilla",
    "gacetillera": "gacetillero",
    "gacetilleras": "gacetillero",
    "gacetilleros": "gacetillero",
    "gacetistas": "gacetista",
    "gacha": "gachó",
    "gachapazos": "gachapazo",
    "gachas": "gachó",
    "gachetas": "gacheta",
    "gachona": "gachón",
    "gachonadas": "gachonada",
    "gachonas": "gachón",
    "gachonerías": "gachonería",
    "gachones": "gachón",
    "gachos": "gachó",
    "gachumbos": "gachumbo",
    "gachupina": "gachupín",
    "gachupinas": "gachupín",
    "gachupines": "gachupín",
    "gachés": "gaché",
    "gachís": "gachí",
    "gachós": "gachó",
    "gaditana": "gaditano",
    "gaditanas": "gaditano",
    "gaditanes": "gaditano",
    "gaditanos": "gaditano",
    "gadolinios": "gadolinio",
    "gadolinitas": "gadolinita",
    "gafa": "gafo",
    "gafaba": "gafar",
    "gafabais": "gafar",
    "gafaban": "gafar",
    "gafabas": "gafar",
    "gafad": "gafar",
    "gafada": "gafar",
    "gafadas": "gafar",
    "gafado": "gafar",
    "gafados": "gafar",
    "gafamos": "gafar",
    "gafan": "gafar",
    "gafando": "gafar",
    "gafara": "gafar",
    "gafarais": "gafar",
    "gafaran": "gafar",
    "gafaras": "gafar",
    "gafare": "gafar",
    "gafareis": "gafar",
    "gafaremos": "gafar",
    "gafaren": "gafar",
    "gafares": "gafar",
    "gafaron": "gafar",
    "gafará": "gafar",
    "gafarán": "gafar",
    "gafarás": "gafar",
    "gafaré": "gafar",
    "gafaréis": "gafar",
    "gafaría": "gafar",
    "gafaríais": "gafar",
    "gafaríamos": "gafar",
    "gafarían": "gafar",
    "gafarías": "gafar",
    "gafas": "gafo",
    "gafase": "gafar",
    "gafaseis": "gafar",
    "gafasen": "gafar",
    "gafases": "gafar",
    "gafaste": "gafar",
    "gafasteis": "gafar",
    "gafe": "gafar",
    "gafedades": "gafedad",
    "gafemos": "gafar",
    "gafen": "gafar",
    "gafes": "gafe",
    "gafetes": "gafete",
    "gafo": "gafar",
    "gafos": "gafo",
    "gafábamos": "gafar",
    "gafáis": "gafar",
    "gafáramos": "gafar",
    "gafáremos": "gafar",
    "gafásemos": "gafar",
    "gafé": "gafar",
    "gaféis": "gafar",
    "gafó": "gafar",
    "gags": "gag",
    "gaguea": "gaguear",
    "gagueaba": "gaguear",
    "gagueabais": "gaguear",
    "gagueaban": "gaguear",
    "gagueabas": "gaguear",
    "gaguead": "gaguear",
    "gagueada": "gaguear",
    "gagueadas": "gaguear",
    "gagueado": "gaguear",
    "gagueados": "gaguear",
    "gagueamos": "gaguear",
    "gaguean": "gaguear",
    "gagueando": "gaguear",
    "gagueara": "gaguear",
    "gaguearais": "gaguear",
    "gaguearan": "gaguear",
    "gaguearas": "gaguear",
    "gagueare": "gaguear",
    "gagueareis": "gaguear",
    "gaguearemos": "gaguear",
    "gaguearen": "gaguear",
    "gagueares": "gaguear",
    "gaguearon": "gaguear",
    "gagueará": "gaguear",
    "gaguearán": "gaguear",
    "gaguearás": "gaguear",
    "gaguearé": "gaguear",
    "gaguearéis": "gaguear",
    "gaguearía": "gaguear",
    "gaguearíais": "gaguear",
    "gaguearíamos": "gaguear",
    "gaguearían": "gaguear",
    "gaguearías": "gaguear",
    "gagueas": "gaguear",
    "gaguease": "gaguear",
    "gagueaseis": "gaguear",
    "gagueasen": "gaguear",
    "gagueases": "gaguear",
    "gagueaste": "gaguear",
    "gagueasteis": "gaguear",
    "gaguee": "gaguear",
    "gagueemos": "gaguear",
    "gagueen": "gaguear",
    "gaguees": "gaguear",
    "gagueo": "gaguear",
    "gagueábamos": "gaguear",
    "gagueáis": "gaguear",
    "gagueáramos": "gaguear",
    "gagueáremos": "gaguear",
    "gagueásemos": "gaguear",
    "gagueé": "gaguear",
    "gagueéis": "gaguear",
    "gagueó": "gaguear",
    "gahnitas": "gahnita",
    "gahnoespinelas": "gahnoespinela",
    "gaitas": "gaita",
    "gaitera": "gaitero",
    "gaiteras": "gaitero",
    "gaiteros": "gaitero",
    "gaiterías": "gaitería",
    "gajes": "gaje",
    "gajos": "gajo",
    "gala": "galo",
    "galactitas": "galactita",
    "galactofagias": "galactofagia",
    "galactosas": "galactosa",
    "galactosemias": "galactosemia",
    "galactosidasas": "galactosidasa",
    "galactosurias": "galactosuria",
    "galactófaga": "galactófago",
    "galactófagas": "galactófago",
    "galactófagos": "galactófago",
    "galactófora": "galactóforo",
    "galactóforas": "galactóforo",
    "galactóforos": "galactóforo",
    "galactómetros": "galactómetro",
    "galafates": "galafate",
    "galaica": "galaico",
    "galaicas": "galaico",
    "galaicoportuguesa": "galaicoportugués",
    "galaicoportuguesas": "galaicoportugués",
    "galaicoportugueses": "galaicoportugués",
    "galaicos": "galaico",
    "galalitas": "galalita",
    "galana": "galano",
    "galanas": "galano",
    "galanes": "galán",
    "galanos": "galano",
    "galantea": "galantear",
    "galanteaba": "galantear",
    "galanteabais": "galantear",
    "galanteaban": "galantear",
    "galanteabas": "galantear",
    "galantead": "galantear",
    "galanteada": "galantear",
    "galanteadas": "galantear",
    "galanteado": "galantear",
    "galanteadora": "galanteador",
    "galanteadoras": "galanteador",
    "galanteadores": "galanteador",
    "galanteados": "galantear",
    "galanteamos": "galantear",
    "galantean": "galantear",
    "galanteando": "galantear",
    "galanteara": "galantear",
    "galantearais": "galantear",
    "galantearan": "galantear",
    "galantearas": "galantear",
    "galanteare": "galantear",
    "galanteareis": "galantear",
    "galantearemos": "galantear",
    "galantearen": "galantear",
    "galanteares": "galantear",
    "galantearon": "galantear",
    "galanteará": "galantear",
    "galantearán": "galantear",
    "galantearás": "galantear",
    "galantearé": "galantear",
    "galantearéis": "galantear",
    "galantearía": "galantear",
    "galantearíais": "galantear",
    "galantearíamos": "galantear",
    "galantearían": "galantear",
    "galantearías": "galantear",
    "galanteas": "galantear",
    "galantease": "galantear",
    "galanteaseis": "galantear",
    "galanteasen": "galantear",
    "galanteases": "galantear",
    "galanteaste": "galantear",
    "galanteasteis": "galantear",
    "galantee": "galantear",
    "galanteemos": "galantear",
    "galanteen": "galantear",
    "galantees": "galantear",
    "galanteo": "galantear",
    "galanteos": "galanteo",
    "galanterías": "galantería",
    "galantes": "galante",
    "galanteábamos": "galantear",
    "galanteáis": "galantear",
    "galanteáramos": "galantear",
    "galanteáremos": "galantear",
    "galanteásemos": "galantear",
    "galanteé": "galantear",
    "galanteéis": "galantear",
    "galanteó": "galantear",
    "galanuras": "galanura",
    "galapagares": "galapagar",
    "galapagueras": "galapaguera",
    "galapos": "galapo",
    "galardona": "galardonar",
    "galardonaba": "galardonar",
    "galardonabais": "galardonar",
    "galardonaban": "galardonar",
    "galardonabas": "galardonar",
    "galardonad": "galardonar",
    "galardonada": "galardonar",
    "galardonadas": "galardonar",
    "galardonado": "galardonar",
    "galardonadora": "galardonador",
    "galardonadoras": "galardonador",
    "galardonadores": "galardonador",
    "galardonados": "galardonar",
    "galardonamos": "galardonar",
    "galardonan": "galardonar",
    "galardonando": "galardonar",
    "galardonara": "galardonar",
    "galardonarais": "galardonar",
    "galardonaran": "galardonar",
    "galardonaras": "galardonar",
    "galardonare": "galardonar",
    "galardonareis": "galardonar",
    "galardonaremos": "galardonar",
    "galardonaren": "galardonar",
    "galardonares": "galardonar",
    "galardonaron": "galardonar",
    "galardonará": "galardonar",
    "galardonarán": "galardonar",
    "galardonarás": "galardonar",
    "galardonaré": "galardonar",
    "galardonaréis": "galardonar",
    "galardonaría": "galardonar",
    "galardonaríais": "galardonar",
    "galardonaríamos": "galardonar",
    "galardonarían": "galardonar",
    "galardonarías": "galardonar",
    "galardonas": "galardonar",
    "galardonase": "galardonar",
    "galardonaseis": "galardonar",
    "galardonasen": "galardonar",
    "galardonases": "galardonar",
    "galardonaste": "galardonar",
    "galardonasteis": "galardonar",
    "galardone": "galardonar",
    "galardonemos": "galardonar",
    "galardonen": "galardonar",
    "galardones": "galardonar",
    "galardono": "galardonar",
    "galardonábamos": "galardonar",
    "galardonáis": "galardonar",
    "galardonáramos": "galardonar",
    "galardonáremos": "galardonar",
    "galardonásemos": "galardonar",
    "galardoné": "galardonar",
    "galardonéis": "galardonar",
    "galardonó": "galardonar",
    "galas": "galo",
    "galaxias": "galaxia",
    "galayos": "galayo",
    "galbanada": "galbanado",
    "galbanadas": "galbanado",
    "galbanados": "galbanado",
    "galbanas": "galbana",
    "galbanera": "galbanero",
    "galbaneras": "galbanero",
    "galbaneros": "galbanero",
    "galbanosa": "galbanoso",
    "galbanosas": "galbanoso",
    "galbanosos": "galbanoso",
    "galdosiana": "galdosiano",
    "galdosianas": "galdosiano",
    "galdosianos": "galdosiano",
    "galeatos": "galeato",
    "galeazas": "galeaza",
    "galena": "galeno",
    "galenas": "galeno",
    "galenismos": "galenismo",
    "galenistas": "galenista",
    "galenos": "galeno",
    "galeones": "galeón",
    "galeotas": "galeota",
    "galeotes": "galeote",
    "galeotos": "galeoto",
    "galeradas": "galerada",
    "galeras": "galera",
    "galerera": "galerero",
    "galereras": "galerero",
    "galereros": "galerero",
    "galerines": "galerín",
    "galeristas": "galerista",
    "galernas": "galerna",
    "galernos": "galerno",
    "galerones": "galerón",
    "galerías": "galería",
    "galesa": "galés",
    "galesas": "galés",
    "galeses": "galés",
    "galga": "galgo",
    "galgas": "galgo",
    "galgos": "galgo",
    "galguera": "galguero",
    "galgueras": "galguero",
    "galgueros": "galguero",
    "galguesca": "galguesco",
    "galguescas": "galguesco",
    "galguescos": "galguesco",
    "galgueña": "galgueño",
    "galgueñas": "galgueño",
    "galgueños": "galgueño",
    "galianas": "galiana",
    "galiba": "galibar",
    "galibaba": "galibar",
    "galibabais": "galibar",
    "galibaban": "galibar",
    "galibabas": "galibar",
    "galibad": "galibar",
    "galibada": "galibar",
    "galibadas": "galibar",
    "galibado": "galibar",
    "galibados": "galibar",
    "galibamos": "galibar",
    "galiban": "galibar",
    "galibando": "galibar",
    "galibara": "galibar",
    "galibarais": "galibar",
    "galibaran": "galibar",
    "galibaras": "galibar",
    "galibare": "galibar",
    "galibareis": "galibar",
    "galibaremos": "galibar",
    "galibaren": "galibar",
    "galibares": "galibar",
    "galibaron": "galibar",
    "galibará": "galibar",
    "galibarán": "galibar",
    "galibarás": "galibar",
    "galibaré": "galibar",
    "galibaréis": "galibar",
    "galibaría": "galibar",
    "galibaríais": "galibar",
    "galibaríamos": "galibar",
    "galibarían": "galibar",
    "galibarías": "galibar",
    "galibas": "galibar",
    "galibase": "galibar",
    "galibaseis": "galibar",
    "galibasen": "galibar",
    "galibases": "galibar",
    "galibaste": "galibar",
    "galibasteis": "galibar",
    "galibe": "galibar",
    "galibemos": "galibar",
    "galiben": "galibar",
    "galibes": "galibar",
    "galibo": "galibar",
    "galibábamos": "galibar",
    "galibáis": "galibar",
    "galibáramos": "galibar",
    "galibáremos": "galibar",
    "galibásemos": "galibar",
    "galibé": "galibar",
    "galibéis": "galibar",
    "galibó": "galibar",
    "galicada": "galicado",
    "galicadas": "galicado",
    "galicados": "galicado",
    "galicana": "galicano",
    "galicanas": "galicano",
    "galicanismos": "galicanismo",
    "galicanos": "galicano",
    "galicismos": "galicismo",
    "galicistas": "galicista",
    "galilea": "galileo",
    "galileas": "galileo",
    "galileos": "galileo",
    "galios": "galio",
    "galiparlantes": "galiparlante",
    "galiparlas": "galiparla",
    "galiparlistas": "galiparlista",
    "galipodios": "galipodio",
    "galipotes": "galipote",
    "galizabras": "galizabra",
    "galla": "gallar",
    "gallaba": "gallar",
    "gallabais": "gallar",
    "gallaban": "gallar",
    "gallabas": "gallar",
    "gallad": "gallar",
    "gallada": "gallar",
    "galladas": "gallar",
    "gallado": "gallar",
    "gallados": "gallar",
    "galladuras": "galladura",
    "gallamos": "gallar",
    "gallan": "gallar",
    "gallando": "gallar",
    "gallara": "gallar",
    "gallarais": "gallar",
    "gallaran": "gallar",
    "gallaras": "gallar",
    "gallarda": "gallardo",
    "gallardas": "gallardo",
    "gallardea": "gallardear",
    "gallardeaba": "gallardear",
    "gallardeabais": "gallardear",
    "gallardeaban": "gallardear",
    "gallardeabas": "gallardear",
    "gallardead": "gallardear",
    "gallardeada": "gallardear",
    "gallardeadas": "gallardear",
    "gallardeado": "gallardear",
    "gallardeados": "gallardear",
    "gallardeamos": "gallardear",
    "gallardean": "gallardear",
    "gallardeando": "gallardear",
    "gallardeara": "gallardear",
    "gallardearais": "gallardear",
    "gallardearan": "gallardear",
    "gallardearas": "gallardear",
    "gallardeare": "gallardear",
    "gallardeareis": "gallardear",
    "gallardearemos": "gallardear",
    "gallardearen": "gallardear",
    "gallardeares": "gallardear",
    "gallardearon": "gallardear",
    "gallardeará": "gallardear",
    "gallardearán": "gallardear",
    "gallardearás": "gallardear",
    "gallardearé": "gallardear",
    "gallardearéis": "gallardear",
    "gallardearía": "gallardear",
    "gallardearíais": "gallardear",
    "gallardearíamos": "gallardear",
    "gallardearían": "gallardear",
    "gallardearías": "gallardear",
    "gallardeas": "gallardear",
    "gallardease": "gallardear",
    "gallardeaseis": "gallardear",
    "gallardeasen": "gallardear",
    "gallardeases": "gallardear",
    "gallardeaste": "gallardear",
    "gallardeasteis": "gallardear",
    "gallardee": "gallardear",
    "gallardeemos": "gallardear",
    "gallardeen": "gallardear",
    "gallardees": "gallardear",
    "gallardeo": "gallardear",
    "gallardetes": "gallardete",
    "gallardeábamos": "gallardear",
    "gallardeáis": "gallardear",
    "gallardeáramos": "gallardear",
    "gallardeáremos": "gallardear",
    "gallardeásemos": "gallardear",
    "gallardeé": "gallardear",
    "gallardeéis": "gallardear",
    "gallardeó": "gallardear",
    "gallardos": "gallardo",
    "gallardías": "gallardía",
    "gallare": "gallar",
    "gallareis": "gallar",
    "gallaremos": "gallar",
    "gallaren": "gallar",
    "gallares": "gallar",
    "gallaretas": "gallareta",
    "gallaron": "gallar",
    "gallaruzas": "gallaruza",
    "gallará": "gallar",
    "gallarán": "gallar",
    "gallarás": "gallar",
    "gallaré": "gallar",
    "gallaréis": "gallar",
    "gallaría": "gallar",
    "gallaríais": "gallar",
    "gallaríamos": "gallar",
    "gallarían": "gallar",
    "gallarías": "gallar",
    "gallas": "gallar",
    "gallase": "gallar",
    "gallaseis": "gallar",
    "gallasen": "gallar",
    "gallases": "gallar",
    "gallaste": "gallar",
    "gallasteis": "gallar",
    "galle": "gallar",
    "gallea": "gallear",
    "galleaba": "gallear",
    "galleabais": "gallear",
    "galleaban": "gallear",
    "galleabas": "gallear",
    "gallead": "gallear",
    "galleada": "gallear",
    "galleadas": "gallear",
    "galleado": "gallear",
    "galleados": "gallear",
    "galleamos": "gallear",
    "gallean": "gallear",
    "galleando": "gallear",
    "galleara": "gallear",
    "gallearais": "gallear",
    "gallearan": "gallear",
    "gallearas": "gallear",
    "galleare": "gallear",
    "galleareis": "gallear",
    "gallearemos": "gallear",
    "gallearen": "gallear",
    "galleares": "gallear",
    "gallearon": "gallear",
    "galleará": "gallear",
    "gallearán": "gallear",
    "gallearás": "gallear",
    "gallearé": "gallear",
    "gallearéis": "gallear",
    "gallearía": "gallear",
    "gallearíais": "gallear",
    "gallearíamos": "gallear",
    "gallearían": "gallear",
    "gallearías": "gallear",
    "galleas": "gallear",
    "gallease": "gallear",
    "galleaseis": "gallear",
    "galleasen": "gallear",
    "galleases": "gallear",
    "galleaste": "gallear",
    "galleasteis": "gallear",
    "gallee": "gallear",
    "galleemos": "gallear",
    "galleen": "gallear",
    "gallees": "gallear",
    "gallega": "gallego",
    "gallegadas": "gallegada",
    "gallegas": "gallego",
    "gallegos": "gallego",
    "galleguismos": "galleguismo",
    "gallemos": "gallar",
    "gallen": "gallar",
    "galleo": "gallear",
    "galleos": "galleo",
    "gallera": "gallero",
    "galleras": "gallero",
    "galleros": "gallero",
    "galles": "gallar",
    "galletas": "galleta",
    "galletera": "galletero",
    "galleteras": "galletero",
    "galleteros": "galletero",
    "galleterías": "galletería",
    "galleábamos": "gallear",
    "galleáis": "gallear",
    "galleáramos": "gallear",
    "galleáremos": "gallear",
    "galleásemos": "gallear",
    "galleé": "gallear",
    "galleéis": "gallear",
    "galleó": "gallear",
    "galliformes": "galliforme",
    "gallinas": "gallina",
    "gallinazas": "gallinaza",
    "gallinera": "gallinero",
    "gallineras": "gallinero",
    "gallineros": "gallinero",
    "gallinerías": "gallinería",
    "gallinácea": "gallináceo",
    "gallináceas": "gallináceo",
    "gallináceos": "gallináceo",
    "gallipatos": "gallipato",
    "gallipavas": "gallipava",
    "gallipavos": "gallipavo",
    "gallipuentes": "gallipuente",
    "gallita": "gallito",
    "gallitas": "gallito",
    "gallitos": "gallito",
    "gallo": "gallar",
    "gallocrestas": "gallocresta",
    "gallofa": "gallofo",
    "gallofas": "gallofo",
    "gallofea": "gallofear",
    "gallofeaba": "gallofear",
    "gallofeabais": "gallofear",
    "gallofeaban": "gallofear",
    "gallofeabas": "gallofear",
    "gallofead": "gallofear",
    "gallofeada": "gallofear",
    "gallofeadas": "gallofear",
    "gallofeado": "gallofear",
    "gallofeados": "gallofear",
    "gallofeamos": "gallofear",
    "gallofean": "gallofear",
    "gallofeando": "gallofear",
    "gallofeara": "gallofear",
    "gallofearais": "gallofear",
    "gallofearan": "gallofear",
    "gallofearas": "gallofear",
    "gallofeare": "gallofear",
    "gallofeareis": "gallofear",
    "gallofearemos": "gallofear",
    "gallofearen": "gallofear",
    "gallofeares": "gallofear",
    "gallofearon": "gallofear",
    "gallofeará": "gallofear",
    "gallofearán": "gallofear",
    "gallofearás": "gallofear",
    "gallofearé": "gallofear",
    "gallofearéis": "gallofear",
    "gallofearía": "gallofear",
    "gallofearíais": "gallofear",
    "gallofearíamos": "gallofear",
    "gallofearían": "gallofear",
    "gallofearías": "gallofear",
    "gallofeas": "gallofear",
    "gallofease": "gallofear",
    "gallofeaseis": "gallofear",
    "gallofeasen": "gallofear",
    "gallofeases": "gallofear",
    "gallofeaste": "gallofear",
    "gallofeasteis": "gallofear",
    "gallofee": "gallofear",
    "gallofeemos": "gallofear",
    "gallofeen": "gallofear",
    "gallofees": "gallofear",
    "gallofeo": "gallofear",
    "gallofera": "gallofero",
    "galloferas": "gallofero",
    "galloferos": "gallofero",
    "gallofeábamos": "gallofear",
    "gallofeáis": "gallofear",
    "gallofeáramos": "gallofear",
    "gallofeáremos": "gallofear",
    "gallofeásemos": "gallofear",
    "gallofeé": "gallofear",
    "gallofeéis": "gallofear",
    "gallofeó": "gallofear",
    "gallofos": "gallofo",
    "gallones": "gallón",
    "gallos": "gallo",
    "gallábamos": "gallar",
    "galláis": "gallar",
    "galláramos": "gallar",
    "galláremos": "gallar",
    "gallásemos": "gallar",
    "gallé": "gallar",
    "galléis": "gallar",
    "gallística": "gallístico",
    "gallísticas": "gallístico",
    "gallísticos": "gallístico",
    "galló": "gallar",
    "galocha": "galocho",
    "galochas": "galocho",
    "galochera": "galochero",
    "galocheras": "galochero",
    "galocheros": "galochero",
    "galochos": "galocho",
    "galocianinas": "galocianina",
    "galonea": "galonear",
    "galoneaba": "galonear",
    "galoneabais": "galonear",
    "galoneaban": "galonear",
    "galoneabas": "galonear",
    "galonead": "galonear",
    "galoneada": "galonear",
    "galoneadas": "galonear",
    "galoneado": "galonear",
    "galoneadora": "galoneador",
    "galoneadoras": "galoneador",
    "galoneadores": "galoneador",
    "galoneados": "galonear",
    "galoneaduras": "galoneadura",
    "galoneamos": "galonear",
    "galonean": "galonear",
    "galoneando": "galonear",
    "galoneara": "galonear",
    "galonearais": "galonear",
    "galonearan": "galonear",
    "galonearas": "galonear",
    "galoneare": "galonear",
    "galoneareis": "galonear",
    "galonearemos": "galonear",
    "galonearen": "galonear",
    "galoneares": "galonear",
    "galonearon": "galonear",
    "galoneará": "galonear",
    "galonearán": "galonear",
    "galonearás": "galonear",
    "galonearé": "galonear",
    "galonearéis": "galonear",
    "galonearía": "galonear",
    "galonearíais": "galonear",
    "galonearíamos": "galonear",
    "galonearían": "galonear",
    "galonearías": "galonear",
    "galoneas": "galonear",
    "galonease": "galonear",
    "galoneaseis": "galonear",
    "galoneasen": "galonear",
    "galoneases": "galonear",
    "galoneaste": "galonear",
    "galoneasteis": "galonear",
    "galonee": "galonear",
    "galoneemos": "galonear",
    "galoneen": "galonear",
    "galonees": "galonear",
    "galoneo": "galonear",
    "galones": "galón",
    "galoneábamos": "galonear",
    "galoneáis": "galonear",
    "galoneáramos": "galonear",
    "galoneáremos": "galonear",
    "galoneásemos": "galonear",
    "galoneé": "galonear",
    "galoneéis": "galonear",
    "galoneó": "galonear",
    "galonistas": "galonista",
    "galopa": "galopar",
    "galopaba": "galopar",
    "galopabais": "galopar",
    "galopaban": "galopar",
    "galopabas": "galopar",
    "galopad": "galopar",
    "galopada": "galopar",
    "galopadas": "galopar",
    "galopado": "galopar",
    "galopados": "galopar",
    "galopamos": "galopar",
    "galopan": "galopar",
    "galopando": "galopar",
    "galopantes": "galopante",
    "galopara": "galopar",
    "galoparais": "galopar",
    "galoparan": "galopar",
    "galoparas": "galopar",
    "galopare": "galopar",
    "galopareis": "galopar",
    "galoparemos": "galopar",
    "galoparen": "galopar",
    "galopares": "galopar",
    "galoparon": "galopar",
    "galopará": "galopar",
    "galoparán": "galopar",
    "galoparás": "galopar",
    "galoparé": "galopar",
    "galoparéis": "galopar",
    "galoparía": "galopar",
    "galoparíais": "galopar",
    "galoparíamos": "galopar",
    "galoparían": "galopar",
    "galoparías": "galopar",
    "galopas": "galopar",
    "galopase": "galopar",
    "galopaseis": "galopar",
    "galopasen": "galopar",
    "galopases": "galopar",
    "galopaste": "galopar",
    "galopasteis": "galopar",
    "galope": "galopar",
    "galopea": "galopear",
    "galopeaba": "galopear",
    "galopeabais": "galopear",
    "galopeaban": "galopear",
    "galopeabas": "galopear",
    "galopead": "galopear",
    "galopeada": "galopear",
    "galopeadas": "galopear",
    "galopeado": "galopear",
    "galopeados": "galopear",
    "galopeamos": "galopear",
    "galopean": "galopear",
    "galopeando": "galopear",
    "galopeara": "galopear",
    "galopearais": "galopear",
    "galopearan": "galopear",
    "galopearas": "galopear",
    "galopeare": "galopear",
    "galopeareis": "galopear",
    "galopearemos": "galopear",
    "galopearen": "galopear",
    "galopeares": "galopear",
    "galopearon": "galopear",
    "galopeará": "galopear",
    "galopearán": "galopear",
    "galopearás": "galopear",
    "galopearé": "galopear",
    "galopearéis": "galopear",
    "galopearía": "galopear",
    "galopearíais": "galopear",
    "galopearíamos": "galopear",
    "galopearían": "galopear",
    "galopearías": "galopear",
    "galopeas": "galopear",
    "galopease": "galopear",
    "galopeaseis": "galopear",
    "galopeasen": "galopear",
    "galopeases": "galopear",
    "galopeaste": "galopear",
    "galopeasteis": "galopear",
    "galopee": "galopear",
    "galopeemos": "galopear",
    "galopeen": "galopear",
    "galopees": "galopear",
    "galopemos": "galopar",
    "galopen": "galopar",
    "galopeo": "galopear",
    "galopes": "galope",
    "galopeábamos": "galopear",
    "galopeáis": "galopear",
    "galopeáramos": "galopear",
    "galopeáremos": "galopear",
    "galopeásemos": "galopear",
    "galopeé": "galopear",
    "galopeéis": "galopear",
    "galopeó": "galopear",
    "galopina": "galopín",
    "galopinadas": "galopinada",
    "galopinas": "galopín",
    "galopines": "galopín",
    "galopo": "galopar",
    "galops": "galop",
    "galopábamos": "galopar",
    "galopáis": "galopar",
    "galopáramos": "galopar",
    "galopáremos": "galopar",
    "galopásemos": "galopar",
    "galopé": "galopar",
    "galopéis": "galopar",
    "galopó": "galopar",
    "galorromana": "galorromano",
    "galorromanas": "galorromano",
    "galorromanos": "galorromano",
    "galos": "galo",
    "galpitos": "galpito",
    "galpones": "galpón",
    "galucha": "galuchar",
    "galuchaba": "galuchar",
    "galuchabais": "galuchar",
    "galuchaban": "galuchar",
    "galuchabas": "galuchar",
    "galuchad": "galuchar",
    "galuchada": "galuchar",
    "galuchadas": "galuchar",
    "galuchado": "galuchar",
    "galuchados": "galuchar",
    "galuchamos": "galuchar",
    "galuchan": "galuchar",
    "galuchando": "galuchar",
    "galuchara": "galuchar",
    "galucharais": "galuchar",
    "galucharan": "galuchar",
    "galucharas": "galuchar",
    "galuchare": "galuchar",
    "galuchareis": "galuchar",
    "galucharemos": "galuchar",
    "galucharen": "galuchar",
    "galuchares": "galuchar",
    "galucharon": "galuchar",
    "galuchará": "galuchar",
    "galucharán": "galuchar",
    "galucharás": "galuchar",
    "galucharé": "galuchar",
    "galucharéis": "galuchar",
    "galucharía": "galuchar",
    "galucharíais": "galuchar",
    "galucharíamos": "galuchar",
    "galucharían": "galuchar",
    "galucharías": "galuchar",
    "galuchas": "galuchar",
    "galuchase": "galuchar",
    "galuchaseis": "galuchar",
    "galuchasen": "galuchar",
    "galuchases": "galuchar",
    "galuchaste": "galuchar",
    "galuchasteis": "galuchar",
    "galuche": "galuchar",
    "galuchemos": "galuchar",
    "galuchen": "galuchar",
    "galuches": "galuchar",
    "galucho": "galuchar",
    "galuchábamos": "galuchar",
    "galucháis": "galuchar",
    "galucháramos": "galuchar",
    "galucháremos": "galuchar",
    "galuchásemos": "galuchar",
    "galuché": "galuchar",
    "galuchéis": "galuchar",
    "galuchó": "galuchar",
    "galvanice": "galvanizar",
    "galvanicemos": "galvanizar",
    "galvanicen": "galvanizar",
    "galvanices": "galvanizar",
    "galvanicé": "galvanizar",
    "galvanicéis": "galvanizar",
    "galvanismos": "galvanismo",
    "galvaniza": "galvanizar",
    "galvanizaba": "galvanizar",
    "galvanizabais": "galvanizar",
    "galvanizaban": "galvanizar",
    "galvanizabas": "galvanizar",
    "galvanizaciones": "galvanización",
    "galvanizad": "galvanizar",
    "galvanizada": "galvanizar",
    "galvanizadas": "galvanizar",
    "galvanizado": "galvanizar",
    "galvanizadora": "galvanizador",
    "galvanizadoras": "galvanizador",
    "galvanizadores": "galvanizador",
    "galvanizados": "galvanizar",
    "galvanizamos": "galvanizar",
    "galvanizan": "galvanizar",
    "galvanizando": "galvanizar",
    "galvanizara": "galvanizar",
    "galvanizarais": "galvanizar",
    "galvanizaran": "galvanizar",
    "galvanizaras": "galvanizar",
    "galvanizare": "galvanizar",
    "galvanizareis": "galvanizar",
    "galvanizaremos": "galvanizar",
    "galvanizaren": "galvanizar",
    "galvanizares": "galvanizar",
    "galvanizaron": "galvanizar",
    "galvanizará": "galvanizar",
    "galvanizarán": "galvanizar",
    "galvanizarás": "galvanizar",
    "galvanizaré": "galvanizar",
    "galvanizaréis": "galvanizar",
    "galvanizaría": "galvanizar",
    "galvanizaríais": "galvanizar",
    "galvanizaríamos": "galvanizar",
    "galvanizarían": "galvanizar",
    "galvanizarías": "galvanizar",
    "galvanizas": "galvanizar",
    "galvanizase": "galvanizar",
    "galvanizaseis": "galvanizar",
    "galvanizasen": "galvanizar",
    "galvanizases": "galvanizar",
    "galvanizaste": "galvanizar",
    "galvanizasteis": "galvanizar",
    "galvanizo": "galvanizar",
    "galvanizábamos": "galvanizar",
    "galvanizáis": "galvanizar",
    "galvanizáramos": "galvanizar",
    "galvanizáremos": "galvanizar",
    "galvanizásemos": "galvanizar",
    "galvanizó": "galvanizar",
    "galvanometrías": "galvanometría",
    "galvanoplastias": "galvanoplastia",
    "galvanoplástica": "galvanoplástico",
    "galvanoplásticas": "galvanoplástico",
    "galvanoplásticos": "galvanoplástico",
    "galvanos": "galvano",
    "galvanoscopios": "galvanoscopio",
    "galvanostegias": "galvanostegia",
    "galvanotecnias": "galvanotecnia",
    "galvanómetros": "galvanómetro",
    "galvánica": "galvánico",
    "galvánicas": "galvánico",
    "galvánicos": "galvánico",
    "galáctica": "galáctico",
    "galácticas": "galáctico",
    "galácticos": "galáctico",
    "galápagos": "galápago",
    "galénica": "galénico",
    "galénicas": "galénico",
    "galénicos": "galénico",
    "gamada": "gamado",
    "gamadas": "gamado",
    "gamados": "gamado",
    "gamarras": "gamarra",
    "gamas": "gama",
    "gambas": "gamba",
    "gamberra": "gamberro",
    "gamberradas": "gamberrada",
    "gamberras": "gamberro",
    "gamberrismos": "gamberrismo",
    "gamberros": "gamberro",
    "gambetas": "gambeta",
    "gambetea": "gambetear",
    "gambeteaba": "gambetear",
    "gambeteabais": "gambetear",
    "gambeteaban": "gambetear",
    "gambeteabas": "gambetear",
    "gambetead": "gambetear",
    "gambeteada": "gambetear",
    "gambeteadas": "gambetear",
    "gambeteado": "gambetear",
    "gambeteados": "gambetear",
    "gambeteamos": "gambetear",
    "gambetean": "gambetear",
    "gambeteando": "gambetear",
    "gambeteara": "gambetear",
    "gambetearais": "gambetear",
    "gambetearan": "gambetear",
    "gambetearas": "gambetear",
    "gambeteare": "gambetear",
    "gambeteareis": "gambetear",
    "gambetearemos": "gambetear",
    "gambetearen": "gambetear",
    "gambeteares": "gambetear",
    "gambetearon": "gambetear",
    "gambeteará": "gambetear",
    "gambetearán": "gambetear",
    "gambetearás": "gambetear",
    "gambetearé": "gambetear",
    "gambetearéis": "gambetear",
    "gambetearía": "gambetear",
    "gambetearíais": "gambetear",
    "gambetearíamos": "gambetear",
    "gambetearían": "gambetear",
    "gambetearías": "gambetear",
    "gambeteas": "gambetear",
    "gambetease": "gambetear",
    "gambeteaseis": "gambetear",
    "gambeteasen": "gambetear",
    "gambeteases": "gambetear",
    "gambeteaste": "gambetear",
    "gambeteasteis": "gambetear",
    "gambetee": "gambetear",
    "gambeteemos": "gambetear",
    "gambeteen": "gambetear",
    "gambetees": "gambetear",
    "gambeteo": "gambetear",
    "gambeteábamos": "gambetear",
    "gambeteáis": "gambetear",
    "gambeteáramos": "gambetear",
    "gambeteáremos": "gambetear",
    "gambeteásemos": "gambetear",
    "gambeteé": "gambetear",
    "gambeteéis": "gambetear",
    "gambeteó": "gambetear",
    "gambetos": "gambeto",
    "gambiana": "gambiano",
    "gambianas": "gambiano",
    "gambianos": "gambiano",
    "gambistas": "gambista",
    "gambitos": "gambito",
    "gamboas": "gamboa",
    "gambotas": "gambota",
    "gambuceros": "gambucero",
    "gambusinas": "gambusina",
    "gambusinos": "gambusino",
    "gambuzas": "gambuza",
    "gamechogos": "gamechogo",
    "gamelladas": "gamellada",
    "gamellas": "gamella",
    "gamellones": "gamellón",
    "gametocitos": "gametocito",
    "gametos": "gameto",
    "gametófitos": "gametófito",
    "gameznos": "gamezno",
    "gamillones": "gamillón",
    "gamitidos": "gamitido",
    "gammaglobulinas": "gammaglobulina",
    "gammagrafías": "gammagrafía",
    "gammapatías": "gammapatía",
    "gammarradiografías": "gammarradiografía",
    "gammas": "gamma",
    "gammopatías": "gammopatía",
    "gamodemos": "gamodemo",
    "gamonales": "gamonal",
    "gamonalismos": "gamonalismo",
    "gamones": "gamón",
    "gamonitos": "gamonito",
    "gamopétala": "gamopétalo",
    "gamopétalas": "gamopétalo",
    "gamopétalos": "gamopétalo",
    "gamos": "gamo",
    "gamosépala": "gamosépalo",
    "gamosépalas": "gamosépalo",
    "gamosépalos": "gamosépalo",
    "gamuzada": "gamuzado",
    "gamuzadas": "gamuzado",
    "gamuzados": "gamuzado",
    "gamuzas": "gamuza",
    "gamética": "gamético",
    "gaméticas": "gamético",
    "gaméticos": "gamético",
    "gana": "ganar",
    "ganaba": "ganar",
    "ganabais": "ganar",
    "ganaban": "ganar",
    "ganabas": "ganar",
    "ganad": "ganar",
    "ganada": "ganar",
    "ganadas": "ganar",
    "ganadera": "ganadero",
    "ganaderas": "ganadero",
    "ganaderos": "ganadero",
    "ganaderías": "ganadería",
    "ganado": "ganar",
    "ganadora": "ganador",
    "ganadoras": "ganador",
    "ganadores": "ganador",
    "ganados": "ganar",
    "ganamos": "ganar",
    "ganan": "ganar",
    "gananciales": "ganancial",
    "ganancias": "ganancia",
    "gananciosa": "ganancioso",
    "gananciosas": "ganancioso",
    "gananciosos": "ganancioso",
    "ganando": "ganar",
    "ganapanes": "ganapán",
    "ganara": "ganar",
    "ganarais": "ganar",
    "ganaran": "ganar",
    "ganaras": "ganar",
    "ganare": "ganar",
    "ganareis": "ganar",
    "ganaremos": "ganar",
    "ganaren": "ganar",
    "ganares": "ganar",
    "ganaron": "ganar",
    "ganará": "ganar",
    "ganarán": "ganar",
    "ganarás": "ganar",
    "ganaré": "ganar",
    "ganaréis": "ganar",
    "ganaría": "ganar",
    "ganaríais": "ganar",
    "ganaríamos": "ganar",
    "ganarían": "ganar",
    "ganarías": "ganar",
    "ganas": "ganar",
    "ganase": "ganar",
    "ganaseis": "ganar",
    "ganasen": "ganar",
    "ganases": "ganar",
    "ganaste": "ganar",
    "ganasteis": "ganar",
    "ganchera": "ganchero",
    "gancheras": "ganchero",
    "gancheros": "ganchero",
    "ganchillos": "ganchillo",
    "ganchos": "gancho",
    "ganchosa": "ganchoso",
    "ganchosas": "ganchoso",
    "ganchosos": "ganchoso",
    "ganchuda": "ganchudo",
    "ganchudas": "ganchudo",
    "ganchudos": "ganchudo",
    "gandingas": "gandinga",
    "gandula": "gandul",
    "gandulas": "gandul",
    "gandulea": "gandulear",
    "ganduleaba": "gandulear",
    "ganduleabais": "gandulear",
    "ganduleaban": "gandulear",
    "ganduleabas": "gandulear",
    "gandulead": "gandulear",
    "ganduleada": "gandulear",
    "ganduleadas": "gandulear",
    "ganduleado": "gandulear",
    "ganduleados": "gandulear",
    "ganduleamos": "gandulear",
    "gandulean": "gandulear",
    "ganduleando": "gandulear",
    "ganduleara": "gandulear",
    "gandulearais": "gandulear",
    "gandulearan": "gandulear",
    "gandulearas": "gandulear",
    "ganduleare": "gandulear",
    "ganduleareis": "gandulear",
    "gandulearemos": "gandulear",
    "gandulearen": "gandulear",
    "ganduleares": "gandulear",
    "gandulearon": "gandulear",
    "ganduleará": "gandulear",
    "gandulearán": "gandulear",
    "gandulearás": "gandulear",
    "gandulearé": "gandulear",
    "gandulearéis": "gandulear",
    "gandulearía": "gandulear",
    "gandulearíais": "gandulear",
    "gandulearíamos": "gandulear",
    "gandulearían": "gandulear",
    "gandulearías": "gandulear",
    "ganduleas": "gandulear",
    "gandulease": "gandulear",
    "ganduleaseis": "gandulear",
    "ganduleasen": "gandulear",
    "ganduleases": "gandulear",
    "ganduleaste": "gandulear",
    "ganduleasteis": "gandulear",
    "gandulee": "gandulear",
    "ganduleemos": "gandulear",
    "ganduleen": "gandulear",
    "gandulees": "gandulear",
    "ganduleo": "gandulear",
    "gandulerías": "gandulería",
    "gandules": "gandul",
    "ganduleábamos": "gandulear",
    "ganduleáis": "gandulear",
    "ganduleáramos": "gandulear",
    "ganduleáremos": "gandulear",
    "ganduleásemos": "gandulear",
    "ganduleé": "gandulear",
    "ganduleéis": "gandulear",
    "ganduleó": "gandulear",
    "gane": "ganar",
    "ganemos": "ganar",
    "ganen": "ganar",
    "ganes": "ganar",
    "gangas": "ganga",
    "gangliocitos": "gangliocito",
    "ganglionares": "ganglionar",
    "ganglios": "ganglio",
    "gangliósidos": "gangliósido",
    "gangosa": "gangoso",
    "gangosas": "gangoso",
    "gangosidades": "gangosidad",
    "gangosos": "gangoso",
    "gangrena": "gangrenar",
    "gangrenaba": "gangrenar",
    "gangrenabais": "gangrenar",
    "gangrenaban": "gangrenar",
    "gangrenabas": "gangrenar",
    "gangrenad": "gangrenar",
    "gangrenada": "gangrenar",
    "gangrenadas": "gangrenar",
    "gangrenado": "gangrenar",
    "gangrenados": "gangrenar",
    "gangrenamos": "gangrenar",
    "gangrenan": "gangrenar",
    "gangrenando": "gangrenar",
    "gangrenara": "gangrenar",
    "gangrenarais": "gangrenar",
    "gangrenaran": "gangrenar",
    "gangrenaras": "gangrenar",
    "gangrenare": "gangrenar",
    "gangrenareis": "gangrenar",
    "gangrenaremos": "gangrenar",
    "gangrenaren": "gangrenar",
    "gangrenares": "gangrenar",
    "gangrenaron": "gangrenar",
    "gangrenará": "gangrenar",
    "gangrenarán": "gangrenar",
    "gangrenarás": "gangrenar",
    "gangrenaré": "gangrenar",
    "gangrenaréis": "gangrenar",
    "gangrenaría": "gangrenar",
    "gangrenaríais": "gangrenar",
    "gangrenaríamos": "gangrenar",
    "gangrenarían": "gangrenar",
    "gangrenarías": "gangrenar",
    "gangrenas": "gangrenar",
    "gangrenase": "gangrenar",
    "gangrenaseis": "gangrenar",
    "gangrenasen": "gangrenar",
    "gangrenases": "gangrenar",
    "gangrenaste": "gangrenar",
    "gangrenasteis": "gangrenar",
    "gangrene": "gangrenar",
    "gangrenemos": "gangrenar",
    "gangrenen": "gangrenar",
    "gangrenes": "gangrenar",
    "gangreno": "gangrenar",
    "gangrenosa": "gangrenoso",
    "gangrenosas": "gangrenoso",
    "gangrenosos": "gangrenoso",
    "gangrenábamos": "gangrenar",
    "gangrenáis": "gangrenar",
    "gangrenáramos": "gangrenar",
    "gangrenáremos": "gangrenar",
    "gangrenásemos": "gangrenar",
    "gangrené": "gangrenar",
    "gangrenéis": "gangrenar",
    "gangrenó": "gangrenar",
    "gangsterismos": "gangsterismo",
    "gangsters": "gángster",
    "ganguea": "ganguear",
    "gangueaba": "ganguear",
    "gangueabais": "ganguear",
    "gangueaban": "ganguear",
    "gangueabas": "ganguear",
    "ganguead": "ganguear",
    "gangueada": "ganguear",
    "gangueadas": "ganguear",
    "gangueado": "ganguear",
    "gangueados": "ganguear",
    "gangueamos": "ganguear",
    "ganguean": "ganguear",
    "gangueando": "ganguear",
    "gangueara": "ganguear",
    "ganguearais": "ganguear",
    "ganguearan": "ganguear",
    "ganguearas": "ganguear",
    "gangueare": "ganguear",
    "gangueareis": "ganguear",
    "ganguearemos": "ganguear",
    "ganguearen": "ganguear",
    "gangueares": "ganguear",
    "ganguearon": "ganguear",
    "gangueará": "ganguear",
    "ganguearán": "ganguear",
    "ganguearás": "ganguear",
    "ganguearé": "ganguear",
    "ganguearéis": "ganguear",
    "ganguearía": "ganguear",
    "ganguearíais": "ganguear",
    "ganguearíamos": "ganguear",
    "ganguearían": "ganguear",
    "ganguearías": "ganguear",
    "gangueas": "ganguear",
    "ganguease": "ganguear",
    "gangueaseis": "ganguear",
    "gangueasen": "ganguear",
    "gangueases": "ganguear",
    "gangueaste": "ganguear",
    "gangueasteis": "ganguear",
    "ganguee": "ganguear",
    "gangueemos": "ganguear",
    "gangueen": "ganguear",
    "ganguees": "ganguear",
    "gangueo": "ganguear",
    "gangueos": "gangueo",
    "ganguera": "ganguero",
    "gangueras": "ganguero",
    "gangueros": "ganguero",
    "gangueábamos": "ganguear",
    "gangueáis": "ganguear",
    "gangueáramos": "ganguear",
    "gangueáremos": "ganguear",
    "gangueásemos": "ganguear",
    "gangueé": "ganguear",
    "gangueéis": "ganguear",
    "gangueó": "ganguear",
    "gano": "ganar",
    "ganosa": "ganoso",
    "ganosas": "ganoso",
    "ganosos": "ganoso",
    "gansa": "ganso",
    "gansadas": "gansada",
    "gansarones": "gansarón",
    "gansas": "ganso",
    "gansea": "gansear",
    "ganseaba": "gansear",
    "ganseabais": "gansear",
    "ganseaban": "gansear",
    "ganseabas": "gansear",
    "gansead": "gansear",
    "ganseada": "gansear",
    "ganseadas": "gansear",
    "ganseado": "gansear",
    "ganseados": "gansear",
    "ganseamos": "gansear",
    "gansean": "gansear",
    "ganseando": "gansear",
    "ganseara": "gansear",
    "gansearais": "gansear",
    "gansearan": "gansear",
    "gansearas": "gansear",
    "ganseare": "gansear",
    "ganseareis": "gansear",
    "gansearemos": "gansear",
    "gansearen": "gansear",
    "ganseares": "gansear",
    "gansearon": "gansear",
    "ganseará": "gansear",
    "gansearán": "gansear",
    "gansearás": "gansear",
    "gansearé": "gansear",
    "gansearéis": "gansear",
    "gansearía": "gansear",
    "gansearíais": "gansear",
    "gansearíamos": "gansear",
    "gansearían": "gansear",
    "gansearías": "gansear",
    "ganseas": "gansear",
    "gansease": "gansear",
    "ganseaseis": "gansear",
    "ganseasen": "gansear",
    "ganseases": "gansear",
    "ganseaste": "gansear",
    "ganseasteis": "gansear",
    "gansee": "gansear",
    "ganseemos": "gansear",
    "ganseen": "gansear",
    "gansees": "gansear",
    "ganseo": "gansear",
    "ganseábamos": "gansear",
    "ganseáis": "gansear",
    "ganseáramos": "gansear",
    "ganseáremos": "gansear",
    "ganseásemos": "gansear",
    "ganseé": "gansear",
    "ganseéis": "gansear",
    "ganseó": "gansear",
    "gansos": "ganso",
    "gantesa": "gantés",
    "gantesas": "gantés",
    "ganteses": "gantés",
    "ganzúas": "ganzúa",
    "ganábamos": "ganar",
    "ganáis": "ganar",
    "ganáramos": "ganar",
    "ganáremos": "ganar",
    "ganásemos": "ganar",
    "gané": "ganar",
    "ganéis": "ganar",
    "ganísteres": "ganíster",
    "ganó": "ganar",
    "gaps": "gap",
    "garabatea": "garabatear",
    "garabateaba": "garabatear",
    "garabateabais": "garabatear",
    "garabateaban": "garabatear",
    "garabateabas": "garabatear",
    "garabatead": "garabatear",
    "garabateada": "garabatear",
    "garabateadas": "garabatear",
    "garabateado": "garabatear",
    "garabateados": "garabatear",
    "garabateamos": "garabatear",
    "garabatean": "garabatear",
    "garabateando": "garabatear",
    "garabateara": "garabatear",
    "garabatearais": "garabatear",
    "garabatearan": "garabatear",
    "garabatearas": "garabatear",
    "garabateare": "garabatear",
    "garabateareis": "garabatear",
    "garabatearemos": "garabatear",
    "garabatearen": "garabatear",
    "garabateares": "garabatear",
    "garabatearon": "garabatear",
    "garabateará": "garabatear",
    "garabatearán": "garabatear",
    "garabatearás": "garabatear",
    "garabatearé": "garabatear",
    "garabatearéis": "garabatear",
    "garabatearía": "garabatear",
    "garabatearíais": "garabatear",
    "garabatearíamos": "garabatear",
    "garabatearían": "garabatear",
    "garabatearías": "garabatear",
    "garabateas": "garabatear",
    "garabatease": "garabatear",
    "garabateaseis": "garabatear",
    "garabateasen": "garabatear",
    "garabateases": "garabatear",
    "garabateaste": "garabatear",
    "garabateasteis": "garabatear",
    "garabatee": "garabatear",
    "garabateemos": "garabatear",
    "garabateen": "garabatear",
    "garabatees": "garabatear",
    "garabateo": "garabatear",
    "garabateos": "garabateo",
    "garabateábamos": "garabatear",
    "garabateáis": "garabatear",
    "garabateáramos": "garabatear",
    "garabateáremos": "garabatear",
    "garabateásemos": "garabatear",
    "garabateé": "garabatear",
    "garabateéis": "garabatear",
    "garabateó": "garabatear",
    "garabatos": "garabato",
    "garabatosa": "garabatoso",
    "garabatosas": "garabatoso",
    "garabatosos": "garabatoso",
    "garabitos": "garabito",
    "garages": "garage",
    "garajes": "garaje",
    "garambainas": "garambaina",
    "garandumbas": "garandumba",
    "garantes": "garante",
    "garantice": "garantizar",
    "garanticemos": "garantizar",
    "garanticen": "garantizar",
    "garantices": "garantizar",
    "garanticé": "garantizar",
    "garanticéis": "garantizar",
    "garantid": "garantir",
    "garantida": "garantir",
    "garantidas": "garantir",
    "garantido": "garantir",
    "garantidos": "garantir",
    "garantiendo": "garantir",
    "garantiera": "garantir",
    "garantierais": "garantir",
    "garantieran": "garantir",
    "garantieras": "garantir",
    "garantiere": "garantir",
    "garantiereis": "garantir",
    "garantieren": "garantir",
    "garantieres": "garantir",
    "garantieron": "garantir",
    "garantiese": "garantir",
    "garantieseis": "garantir",
    "garantiesen": "garantir",
    "garantieses": "garantir",
    "garantimos": "garantir",
    "garantiremos": "garantir",
    "garantirá": "garantir",
    "garantirán": "garantir",
    "garantirás": "garantir",
    "garantiré": "garantir",
    "garantiréis": "garantir",
    "garantiría": "garantir",
    "garantiríais": "garantir",
    "garantiríamos": "garantir",
    "garantirían": "garantir",
    "garantirías": "garantir",
    "garantiste": "garantir",
    "garantisteis": "garantir",
    "garantiza": "garantizar",
    "garantizaba": "garantizar",
    "garantizabais": "garantizar",
    "garantizaban": "garantizar",
    "garantizabas": "garantizar",
    "garantizad": "garantizar",
    "garantizada": "garantizar",
    "garantizadas": "garantizar",
    "garantizado": "garantizar",
    "garantizadora": "garantizador",
    "garantizadoras": "garantizador",
    "garantizadores": "garantizador",
    "garantizados": "garantizar",
    "garantizamos": "garantizar",
    "garantizan": "garantizar",
    "garantizando": "garantizar",
    "garantizara": "garantizar",
    "garantizarais": "garantizar",
    "garantizaran": "garantizar",
    "garantizaras": "garantizar",
    "garantizare": "garantizar",
    "garantizareis": "garantizar",
    "garantizaremos": "garantizar",
    "garantizaren": "garantizar",
    "garantizares": "garantizar",
    "garantizaron": "garantizar",
    "garantizará": "garantizar",
    "garantizarán": "garantizar",
    "garantizarás": "garantizar",
    "garantizaré": "garantizar",
    "garantizaréis": "garantizar",
    "garantizaría": "garantizar",
    "garantizaríais": "garantizar",
    "garantizaríamos": "garantizar",
    "garantizarían": "garantizar",
    "garantizarías": "garantizar",
    "garantizas": "garantizar",
    "garantizase": "garantizar",
    "garantizaseis": "garantizar",
    "garantizasen": "garantizar",
    "garantizases": "garantizar",
    "garantizaste": "garantizar",
    "garantizasteis": "garantizar",
    "garantizo": "garantizar",
    "garantizábamos": "garantizar",
    "garantizáis": "garantizar",
    "garantizáramos": "garantizar",
    "garantizáremos": "garantizar",
    "garantizásemos": "garantizar",
    "garantizó": "garantizar",
    "garantiéramos": "garantir",
    "garantiéremos": "garantir",
    "garantiésemos": "garantir",
    "garantió": "garantir",
    "garantí": "garantir",
    "garantía": "garantir",
    "garantíais": "garantir",
    "garantíamos": "garantir",
    "garantían": "garantir",
    "garantías": "garantir",
    "garantís": "garantir",
    "garapachos": "garapacho",
    "garapitas": "garapita",
    "garapiña": "garapiñar",
    "garapiñaba": "garapiñar",
    "garapiñabais": "garapiñar",
    "garapiñaban": "garapiñar",
    "garapiñabas": "garapiñar",
    "garapiñad": "garapiñar",
    "garapiñada": "garapiñar",
    "garapiñadas": "garapiñar",
    "garapiñado": "garapiñar",
    "garapiñados": "garapiñar",
    "garapiñamos": "garapiñar",
    "garapiñan": "garapiñar",
    "garapiñando": "garapiñar",
    "garapiñara": "garapiñar",
    "garapiñarais": "garapiñar",
    "garapiñaran": "garapiñar",
    "garapiñaras": "garapiñar",
    "garapiñare": "garapiñar",
    "garapiñareis": "garapiñar",
    "garapiñaremos": "garapiñar",
    "garapiñaren": "garapiñar",
    "garapiñares": "garapiñar",
    "garapiñaron": "garapiñar",
    "garapiñará": "garapiñar",
    "garapiñarán": "garapiñar",
    "garapiñarás": "garapiñar",
    "garapiñaré": "garapiñar",
    "garapiñaréis": "garapiñar",
    "garapiñaría": "garapiñar",
    "garapiñaríais": "garapiñar",
    "garapiñaríamos": "garapiñar",
    "garapiñarían": "garapiñar",
    "garapiñarías": "garapiñar",
    "garapiñas": "garapiñar",
    "garapiñase": "garapiñar",
    "garapiñaseis": "garapiñar",
    "garapiñasen": "garapiñar",
    "garapiñases": "garapiñar",
    "garapiñaste": "garapiñar",
    "garapiñasteis": "garapiñar",
    "garapiñe": "garapiñar",
    "garapiñemos": "garapiñar",
    "garapiñen": "garapiñar",
    "garapiñeras": "garapiñera",
    "garapiñes": "garapiñar",
    "garapiño": "garapiñar",
    "garapiñábamos": "garapiñar",
    "garapiñáis": "garapiñar",
    "garapiñáramos": "garapiñar",
    "garapiñáremos": "garapiñar",
    "garapiñásemos": "garapiñar",
    "garapiñé": "garapiñar",
    "garapiñéis": "garapiñar",
    "garapiñó": "garapiñar",
    "garapullos": "garapullo",
    "garaturas": "garatura",
    "garatusas": "garatusa",
    "garañones": "garañón",
    "garbancera": "garbancero",
    "garbanceras": "garbancero",
    "garbanceros": "garbancero",
    "garbanzera": "garbanzero",
    "garbanzeras": "garbanzero",
    "garbanzeros": "garbanzero",
    "garbanzos": "garbanzo",
    "garbea": "garbear",
    "garbeaba": "garbear",
    "garbeabais": "garbear",
    "garbeaban": "garbear",
    "garbeabas": "garbear",
    "garbead": "garbear",
    "garbeada": "garbear",
    "garbeadas": "garbear",
    "garbeado": "garbear",
    "garbeados": "garbear",
    "garbeamos": "garbear",
    "garbean": "garbear",
    "garbeando": "garbear",
    "garbeara": "garbear",
    "garbearais": "garbear",
    "garbearan": "garbear",
    "garbearas": "garbear",
    "garbeare": "garbear",
    "garbeareis": "garbear",
    "garbearemos": "garbear",
    "garbearen": "garbear",
    "garbeares": "garbear",
    "garbearon": "garbear",
    "garbeará": "garbear",
    "garbearán": "garbear",
    "garbearás": "garbear",
    "garbearé": "garbear",
    "garbearéis": "garbear",
    "garbearía": "garbear",
    "garbearíais": "garbear",
    "garbearíamos": "garbear",
    "garbearían": "garbear",
    "garbearías": "garbear",
    "garbeas": "garbear",
    "garbease": "garbear",
    "garbeaseis": "garbear",
    "garbeasen": "garbear",
    "garbeases": "garbear",
    "garbeaste": "garbear",
    "garbeasteis": "garbear",
    "garbee": "garbear",
    "garbeemos": "garbear",
    "garbeen": "garbear",
    "garbees": "garbear",
    "garbeo": "garbear",
    "garbeos": "garbeo",
    "garbeábamos": "garbear",
    "garbeáis": "garbear",
    "garbeáramos": "garbear",
    "garbeáremos": "garbear",
    "garbeásemos": "garbear",
    "garbeé": "garbear",
    "garbeéis": "garbear",
    "garbeó": "garbear",
    "garbilla": "garbillar",
    "garbillaba": "garbillar",
    "garbillabais": "garbillar",
    "garbillaban": "garbillar",
    "garbillabas": "garbillar",
    "garbillad": "garbillar",
    "garbillada": "garbillar",
    "garbilladas": "garbillar",
    "garbillado": "garbillar",
    "garbilladora": "garbillador",
    "garbilladoras": "garbillador",
    "garbilladores": "garbillador",
    "garbillados": "garbillar",
    "garbillamos": "garbillar",
    "garbillan": "garbillar",
    "garbillando": "garbillar",
    "garbillara": "garbillar",
    "garbillarais": "garbillar",
    "garbillaran": "garbillar",
    "garbillaras": "garbillar",
    "garbillare": "garbillar",
    "garbillareis": "garbillar",
    "garbillaremos": "garbillar",
    "garbillaren": "garbillar",
    "garbillares": "garbillar",
    "garbillaron": "garbillar",
    "garbillará": "garbillar",
    "garbillarán": "garbillar",
    "garbillarás": "garbillar",
    "garbillaré": "garbillar",
    "garbillaréis": "garbillar",
    "garbillaría": "garbillar",
    "garbillaríais": "garbillar",
    "garbillaríamos": "garbillar",
    "garbillarían": "garbillar",
    "garbillarías": "garbillar",
    "garbillas": "garbillar",
    "garbillase": "garbillar",
    "garbillaseis": "garbillar",
    "garbillasen": "garbillar",
    "garbillases": "garbillar",
    "garbillaste": "garbillar",
    "garbillasteis": "garbillar",
    "garbille": "garbillar",
    "garbillemos": "garbillar",
    "garbillen": "garbillar",
    "garbilles": "garbillar",
    "garbillo": "garbillar",
    "garbillos": "garbillo",
    "garbillábamos": "garbillar",
    "garbilláis": "garbillar",
    "garbilláramos": "garbillar",
    "garbilláremos": "garbillar",
    "garbillásemos": "garbillar",
    "garbillé": "garbillar",
    "garbilléis": "garbillar",
    "garbilló": "garbillar",
    "garbinos": "garbino",
    "garbones": "garbón",
    "garbos": "garbo",
    "garbosa": "garboso",
    "garbosas": "garboso",
    "garbosos": "garboso",
    "garbullos": "garbullo",
    "garcetas": "garceta",
    "garcillas": "garcilla",
    "garcías": "garcía",
    "gardenias": "gardenia",
    "gardingos": "gardingo",
    "gardones": "gardón",
    "garduñas": "garduña",
    "garduños": "garduño",
    "garfadas": "garfada",
    "garfas": "garfa",
    "garfea": "garfear",
    "garfeaba": "garfear",
    "garfeabais": "garfear",
    "garfeaban": "garfear",
    "garfeabas": "garfear",
    "garfead": "garfear",
    "garfeada": "garfear",
    "garfeadas": "garfear",
    "garfeado": "garfear",
    "garfeados": "garfear",
    "garfeamos": "garfear",
    "garfean": "garfear",
    "garfeando": "garfear",
    "garfeara": "garfear",
    "garfearais": "garfear",
    "garfearan": "garfear",
    "garfearas": "garfear",
    "garfeare": "garfear",
    "garfeareis": "garfear",
    "garfearemos": "garfear",
    "garfearen": "garfear",
    "garfeares": "garfear",
    "garfearon": "garfear",
    "garfeará": "garfear",
    "garfearán": "garfear",
    "garfearás": "garfear",
    "garfearé": "garfear",
    "garfearéis": "garfear",
    "garfearía": "garfear",
    "garfearíais": "garfear",
    "garfearíamos": "garfear",
    "garfearían": "garfear",
    "garfearías": "garfear",
    "garfeas": "garfear",
    "garfease": "garfear",
    "garfeaseis": "garfear",
    "garfeasen": "garfear",
    "garfeases": "garfear",
    "garfeaste": "garfear",
    "garfeasteis": "garfear",
    "garfee": "garfear",
    "garfeemos": "garfear",
    "garfeen": "garfear",
    "garfees": "garfear",
    "garfeo": "garfear",
    "garfeábamos": "garfear",
    "garfeáis": "garfear",
    "garfeáramos": "garfear",
    "garfeáremos": "garfear",
    "garfeásemos": "garfear",
    "garfeé": "garfear",
    "garfeéis": "garfear",
    "garfeó": "garfear",
    "garfiadas": "garfiada",
    "garfios": "garfio",
    "gargajea": "gargajear",
    "gargajeaba": "gargajear",
    "gargajeabais": "gargajear",
    "gargajeaban": "gargajear",
    "gargajeabas": "gargajear",
    "gargajead": "gargajear",
    "gargajeada": "gargajear",
    "gargajeadas": "gargajear",
    "gargajeado": "gargajear",
    "gargajeados": "gargajear",
    "gargajeamos": "gargajear",
    "gargajean": "gargajear",
    "gargajeando": "gargajear",
    "gargajeara": "gargajear",
    "gargajearais": "gargajear",
    "gargajearan": "gargajear",
    "gargajearas": "gargajear",
    "gargajeare": "gargajear",
    "gargajeareis": "gargajear",
    "gargajearemos": "gargajear",
    "gargajearen": "gargajear",
    "gargajeares": "gargajear",
    "gargajearon": "gargajear",
    "gargajeará": "gargajear",
    "gargajearán": "gargajear",
    "gargajearás": "gargajear",
    "gargajearé": "gargajear",
    "gargajearéis": "gargajear",
    "gargajearía": "gargajear",
    "gargajearíais": "gargajear",
    "gargajearíamos": "gargajear",
    "gargajearían": "gargajear",
    "gargajearías": "gargajear",
    "gargajeas": "gargajear",
    "gargajease": "gargajear",
    "gargajeaseis": "gargajear",
    "gargajeasen": "gargajear",
    "gargajeases": "gargajear",
    "gargajeaste": "gargajear",
    "gargajeasteis": "gargajear",
    "gargajee": "gargajear",
    "gargajeemos": "gargajear",
    "gargajeen": "gargajear",
    "gargajees": "gargajear",
    "gargajeo": "gargajear",
    "gargajeos": "gargajeo",
    "gargajeábamos": "gargajear",
    "gargajeáis": "gargajear",
    "gargajeáramos": "gargajear",
    "gargajeáremos": "gargajear",
    "gargajeásemos": "gargajear",
    "gargajeé": "gargajear",
    "gargajeéis": "gargajear",
    "gargajeó": "gargajear",
    "gargajos": "gargajo",
    "gargajosa": "gargajoso",
    "gargajosas": "gargajoso",
    "gargajosos": "gargajoso",
    "gargantadas": "gargantada",
    "gargantas": "garganta",
    "gargantea": "gargantear",
    "garganteaba": "gargantear",
    "garganteabais": "gargantear",
    "garganteaban": "gargantear",
    "garganteabas": "gargantear",
    "gargantead": "gargantear",
    "garganteada": "gargantear",
    "garganteadas": "gargantear",
    "garganteado": "gargantear",
    "garganteados": "gargantear",
    "garganteamos": "gargantear",
    "gargantean": "gargantear",
    "garganteando": "gargantear",
    "garganteara": "gargantear",
    "gargantearais": "gargantear",
    "gargantearan": "gargantear",
    "gargantearas": "gargantear",
    "garganteare": "gargantear",
    "garganteareis": "gargantear",
    "gargantearemos": "gargantear",
    "gargantearen": "gargantear",
    "garganteares": "gargantear",
    "gargantearon": "gargantear",
    "garganteará": "gargantear",
    "gargantearán": "gargantear",
    "gargantearás": "gargantear",
    "gargantearé": "gargantear",
    "gargantearéis": "gargantear",
    "gargantearía": "gargantear",
    "gargantearíais": "gargantear",
    "gargantearíamos": "gargantear",
    "gargantearían": "gargantear",
    "gargantearías": "gargantear",
    "garganteas": "gargantear",
    "gargantease": "gargantear",
    "garganteaseis": "gargantear",
    "garganteasen": "gargantear",
    "garganteases": "gargantear",
    "garganteaste": "gargantear",
    "garganteasteis": "gargantear",
    "gargantee": "gargantear",
    "garganteemos": "gargantear",
    "garganteen": "gargantear",
    "gargantees": "gargantear",
    "garganteo": "gargantear",
    "garganteos": "garganteo",
    "garganteábamos": "gargantear",
    "garganteáis": "gargantear",
    "garganteáramos": "gargantear",
    "garganteáremos": "gargantear",
    "garganteásemos": "gargantear",
    "garganteé": "gargantear",
    "garganteéis": "gargantear",
    "garganteó": "gargantear",
    "gargantiles": "gargantil",
    "gargantillas": "gargantilla",
    "gargarice": "gargarizar",
    "gargaricemos": "gargarizar",
    "gargaricen": "gargarizar",
    "gargarices": "gargarizar",
    "gargaricé": "gargarizar",
    "gargaricéis": "gargarizar",
    "gargarismos": "gargarismo",
    "gargariza": "gargarizar",
    "gargarizaba": "gargarizar",
    "gargarizabais": "gargarizar",
    "gargarizaban": "gargarizar",
    "gargarizabas": "gargarizar",
    "gargarizad": "gargarizar",
    "gargarizada": "gargarizar",
    "gargarizadas": "gargarizar",
    "gargarizado": "gargarizar",
    "gargarizados": "gargarizar",
    "gargarizamos": "gargarizar",
    "gargarizan": "gargarizar",
    "gargarizando": "gargarizar",
    "gargarizara": "gargarizar",
    "gargarizarais": "gargarizar",
    "gargarizaran": "gargarizar",
    "gargarizaras": "gargarizar",
    "gargarizare": "gargarizar",
    "gargarizareis": "gargarizar",
    "gargarizaremos": "gargarizar",
    "gargarizaren": "gargarizar",
    "gargarizares": "gargarizar",
    "gargarizaron": "gargarizar",
    "gargarizará": "gargarizar",
    "gargarizarán": "gargarizar",
    "gargarizarás": "gargarizar",
    "gargarizaré": "gargarizar",
    "gargarizaréis": "gargarizar",
    "gargarizaría": "gargarizar",
    "gargarizaríais": "gargarizar",
    "gargarizaríamos": "gargarizar",
    "gargarizarían": "gargarizar",
    "gargarizarías": "gargarizar",
    "gargarizas": "gargarizar",
    "gargarizase": "gargarizar",
    "gargarizaseis": "gargarizar",
    "gargarizasen": "gargarizar",
    "gargarizases": "gargarizar",
    "gargarizaste": "gargarizar",
    "gargarizasteis": "gargarizar",
    "gargarizo": "gargarizar",
    "gargarizábamos": "gargarizar",
    "gargarizáis": "gargarizar",
    "gargarizáramos": "gargarizar",
    "gargarizáremos": "gargarizar",
    "gargarizásemos": "gargarizar",
    "gargarizó": "gargarizar",
    "gargueros": "garguero",
    "gargüeros": "gargüero",
    "garibaldina": "garibaldino",
    "garibaldinas": "garibaldino",
    "garibaldinos": "garibaldino",
    "garibas": "gariba",
    "garifa": "garifo",
    "garifas": "garifo",
    "garifos": "garifo",
    "garitas": "garita",
    "garitera": "garitero",
    "gariteras": "garitero",
    "gariteros": "garitero",
    "garitos": "garito",
    "garla": "garlar",
    "garlaba": "garlar",
    "garlabais": "garlar",
    "garlaban": "garlar",
    "garlabas": "garlar",
    "garlad": "garlar",
    "garlada": "garlar",
    "garladas": "garlar",
    "garlado": "garlar",
    "garladora": "garlador",
    "garladoras": "garlador",
    "garladores": "garlador",
    "garlados": "garlar",
    "garlamos": "garlar",
    "garlan": "garlar",
    "garlando": "garlar",
    "garlara": "garlar",
    "garlarais": "garlar",
    "garlaran": "garlar",
    "garlaras": "garlar",
    "garlare": "garlar",
    "garlareis": "garlar",
    "garlaremos": "garlar",
    "garlaren": "garlar",
    "garlares": "garlar",
    "garlaron": "garlar",
    "garlará": "garlar",
    "garlarán": "garlar",
    "garlarás": "garlar",
    "garlaré": "garlar",
    "garlaréis": "garlar",
    "garlaría": "garlar",
    "garlaríais": "garlar",
    "garlaríamos": "garlar",
    "garlarían": "garlar",
    "garlarías": "garlar",
    "garlas": "garlar",
    "garlase": "garlar",
    "garlaseis": "garlar",
    "garlasen": "garlar",
    "garlases": "garlar",
    "garlaste": "garlar",
    "garlasteis": "garlar",
    "garle": "garlar",
    "garlemos": "garlar",
    "garlen": "garlar",
    "garles": "garlar",
    "garlitos": "garlito",
    "garlo": "garlar",
    "garlochas": "garlocha",
    "garlopas": "garlopa",
    "garlábamos": "garlar",
    "garláis": "garlar",
    "garláramos": "garlar",
    "garláremos": "garlar",
    "garlásemos": "garlar",
    "garlé": "garlar",
    "garléis": "garlar",
    "garló": "garlar",
    "garnachas": "garnacha",
    "garnieles": "garniel",
    "garnieritas": "garnierita",
    "garos": "garo",
    "garra": "garrar",
    "garraba": "garrar",
    "garrabais": "garrar",
    "garraban": "garrar",
    "garrabas": "garrar",
    "garrad": "garrar",
    "garrada": "garrar",
    "garradas": "garrar",
    "garrado": "garrar",
    "garrados": "garrar",
    "garrafales": "garrafal",
    "garrafas": "garrafa",
    "garrafinas": "garrafina",
    "garrafiña": "garrafiñar",
    "garrafiñaba": "garrafiñar",
    "garrafiñabais": "garrafiñar",
    "garrafiñaban": "garrafiñar",
    "garrafiñabas": "garrafiñar",
    "garrafiñad": "garrafiñar",
    "garrafiñada": "garrafiñar",
    "garrafiñadas": "garrafiñar",
    "garrafiñado": "garrafiñar",
    "garrafiñados": "garrafiñar",
    "garrafiñamos": "garrafiñar",
    "garrafiñan": "garrafiñar",
    "garrafiñando": "garrafiñar",
    "garrafiñara": "garrafiñar",
    "garrafiñarais": "garrafiñar",
    "garrafiñaran": "garrafiñar",
    "garrafiñaras": "garrafiñar",
    "garrafiñare": "garrafiñar",
    "garrafiñareis": "garrafiñar",
    "garrafiñaremos": "garrafiñar",
    "garrafiñaren": "garrafiñar",
    "garrafiñares": "garrafiñar",
    "garrafiñaron": "garrafiñar",
    "garrafiñará": "garrafiñar",
    "garrafiñarán": "garrafiñar",
    "garrafiñarás": "garrafiñar",
    "garrafiñaré": "garrafiñar",
    "garrafiñaréis": "garrafiñar",
    "garrafiñaría": "garrafiñar",
    "garrafiñaríais": "garrafiñar",
    "garrafiñaríamos": "garrafiñar",
    "garrafiñarían": "garrafiñar",
    "garrafiñarías": "garrafiñar",
    "garrafiñas": "garrafiñar",
    "garrafiñase": "garrafiñar",
    "garrafiñaseis": "garrafiñar",
    "garrafiñasen": "garrafiñar",
    "garrafiñases": "garrafiñar",
    "garrafiñaste": "garrafiñar",
    "garrafiñasteis": "garrafiñar",
    "garrafiñe": "garrafiñar",
    "garrafiñemos": "garrafiñar",
    "garrafiñen": "garrafiñar",
    "garrafiñes": "garrafiñar",
    "garrafiño": "garrafiñar",
    "garrafiñábamos": "garrafiñar",
    "garrafiñáis": "garrafiñar",
    "garrafiñáramos": "garrafiñar",
    "garrafiñáremos": "garrafiñar",
    "garrafiñásemos": "garrafiñar",
    "garrafiñé": "garrafiñar",
    "garrafiñéis": "garrafiñar",
    "garrafiñó": "garrafiñar",
    "garramas": "garrama",
    "garramos": "garrar",
    "garran": "garrar",
    "garranchadas": "garranchada",
    "garranchas": "garrancha",
    "garranchazos": "garranchazo",
    "garranchos": "garrancho",
    "garrando": "garrar",
    "garrapa": "garrapo",
    "garrapas": "garrapo",
    "garrapatas": "garrapata",
    "garrapatea": "garrapatear",
    "garrapateaba": "garrapatear",
    "garrapateabais": "garrapatear",
    "garrapateaban": "garrapatear",
    "garrapateabas": "garrapatear",
    "garrapatead": "garrapatear",
    "garrapateada": "garrapatear",
    "garrapateadas": "garrapatear",
    "garrapateado": "garrapatear",
    "garrapateados": "garrapatear",
    "garrapateamos": "garrapatear",
    "garrapatean": "garrapatear",
    "garrapateando": "garrapatear",
    "garrapateara": "garrapatear",
    "garrapatearais": "garrapatear",
    "garrapatearan": "garrapatear",
    "garrapatearas": "garrapatear",
    "garrapateare": "garrapatear",
    "garrapateareis": "garrapatear",
    "garrapatearemos": "garrapatear",
    "garrapatearen": "garrapatear",
    "garrapateares": "garrapatear",
    "garrapatearon": "garrapatear",
    "garrapateará": "garrapatear",
    "garrapatearán": "garrapatear",
    "garrapatearás": "garrapatear",
    "garrapatearé": "garrapatear",
    "garrapatearéis": "garrapatear",
    "garrapatearía": "garrapatear",
    "garrapatearíais": "garrapatear",
    "garrapatearíamos": "garrapatear",
    "garrapatearían": "garrapatear",
    "garrapatearías": "garrapatear",
    "garrapateas": "garrapatear",
    "garrapatease": "garrapatear",
    "garrapateaseis": "garrapatear",
    "garrapateasen": "garrapatear",
    "garrapateases": "garrapatear",
    "garrapateaste": "garrapatear",
    "garrapateasteis": "garrapatear",
    "garrapatee": "garrapatear",
    "garrapateemos": "garrapatear",
    "garrapateen": "garrapatear",
    "garrapatees": "garrapatear",
    "garrapateo": "garrapatear",
    "garrapateábamos": "garrapatear",
    "garrapateáis": "garrapatear",
    "garrapateáramos": "garrapatear",
    "garrapateáremos": "garrapatear",
    "garrapateásemos": "garrapatear",
    "garrapateé": "garrapatear",
    "garrapateéis": "garrapatear",
    "garrapateó": "garrapatear",
    "garrapatillos": "garrapatillo",
    "garrapatos": "garrapato",
    "garrapatosa": "garrapatoso",
    "garrapatosas": "garrapatoso",
    "garrapatosos": "garrapatoso",
    "garrapiña": "garrapiñar",
    "garrapiñaba": "garrapiñar",
    "garrapiñabais": "garrapiñar",
    "garrapiñaban": "garrapiñar",
    "garrapiñabas": "garrapiñar",
    "garrapiñad": "garrapiñar",
    "garrapiñada": "garrapiñar",
    "garrapiñadas": "garrapiñar",
    "garrapiñado": "garrapiñar",
    "garrapiñados": "garrapiñar",
    "garrapiñamos": "garrapiñar",
    "garrapiñan": "garrapiñar",
    "garrapiñando": "garrapiñar",
    "garrapiñara": "garrapiñar",
    "garrapiñarais": "garrapiñar",
    "garrapiñaran": "garrapiñar",
    "garrapiñaras": "garrapiñar",
    "garrapiñare": "garrapiñar",
    "garrapiñareis": "garrapiñar",
    "garrapiñaremos": "garrapiñar",
    "garrapiñaren": "garrapiñar",
    "garrapiñares": "garrapiñar",
    "garrapiñaron": "garrapiñar",
    "garrapiñará": "garrapiñar",
    "garrapiñarán": "garrapiñar",
    "garrapiñarás": "garrapiñar",
    "garrapiñaré": "garrapiñar",
    "garrapiñaréis": "garrapiñar",
    "garrapiñaría": "garrapiñar",
    "garrapiñaríais": "garrapiñar",
    "garrapiñaríamos": "garrapiñar",
    "garrapiñarían": "garrapiñar",
    "garrapiñarías": "garrapiñar",
    "garrapiñas": "garrapiñar",
    "garrapiñase": "garrapiñar",
    "garrapiñaseis": "garrapiñar",
    "garrapiñasen": "garrapiñar",
    "garrapiñases": "garrapiñar",
    "garrapiñaste": "garrapiñar",
    "garrapiñasteis": "garrapiñar",
    "garrapiñe": "garrapiñar",
    "garrapiñemos": "garrapiñar",
    "garrapiñen": "garrapiñar",
    "garrapiñes": "garrapiñar",
    "garrapiño": "garrapiñar",
    "garrapiñábamos": "garrapiñar",
    "garrapiñáis": "garrapiñar",
    "garrapiñáramos": "garrapiñar",
    "garrapiñáremos": "garrapiñar",
    "garrapiñásemos": "garrapiñar",
    "garrapiñé": "garrapiñar",
    "garrapiñéis": "garrapiñar",
    "garrapiñó": "garrapiñar",
    "garrapos": "garrapo",
    "garrara": "garrar",
    "garrarais": "garrar",
    "garraran": "garrar",
    "garraras": "garrar",
    "garrare": "garrar",
    "garrareis": "garrar",
    "garraremos": "garrar",
    "garraren": "garrar",
    "garrares": "garrar",
    "garraron": "garrar",
    "garrará": "garrar",
    "garrarán": "garrar",
    "garrarás": "garrar",
    "garraré": "garrar",
    "garraréis": "garrar",
    "garraría": "garrar",
    "garraríais": "garrar",
    "garraríamos": "garrar",
    "garrarían": "garrar",
    "garrarías": "garrar",
    "garras": "garrar",
    "garrase": "garrar",
    "garraseis": "garrar",
    "garrasen": "garrar",
    "garrases": "garrar",
    "garraste": "garrar",
    "garrasteis": "garrar",
    "garre": "garrar",
    "garrea": "garrear",
    "garreaba": "garrear",
    "garreabais": "garrear",
    "garreaban": "garrear",
    "garreabas": "garrear",
    "garread": "garrear",
    "garreada": "garrear",
    "garreadas": "garrear",
    "garreado": "garrear",
    "garreados": "garrear",
    "garreamos": "garrear",
    "garrean": "garrear",
    "garreando": "garrear",
    "garreara": "garrear",
    "garrearais": "garrear",
    "garrearan": "garrear",
    "garrearas": "garrear",
    "garreare": "garrear",
    "garreareis": "garrear",
    "garrearemos": "garrear",
    "garrearen": "garrear",
    "garreares": "garrear",
    "garrearon": "garrear",
    "garreará": "garrear",
    "garrearán": "garrear",
    "garrearás": "garrear",
    "garrearé": "garrear",
    "garrearéis": "garrear",
    "garrearía": "garrear",
    "garrearíais": "garrear",
    "garrearíamos": "garrear",
    "garrearían": "garrear",
    "garrearías": "garrear",
    "garreas": "garrear",
    "garrease": "garrear",
    "garreaseis": "garrear",
    "garreasen": "garrear",
    "garreases": "garrear",
    "garreaste": "garrear",
    "garreasteis": "garrear",
    "garree": "garrear",
    "garreemos": "garrear",
    "garreen": "garrear",
    "garrees": "garrear",
    "garremos": "garrar",
    "garren": "garrar",
    "garreo": "garrear",
    "garres": "garrar",
    "garreábamos": "garrear",
    "garreáis": "garrear",
    "garreáramos": "garrear",
    "garreáremos": "garrear",
    "garreásemos": "garrear",
    "garreé": "garrear",
    "garreéis": "garrear",
    "garreó": "garrear",
    "garrida": "garrido",
    "garridas": "garrido",
    "garridos": "garrido",
    "garro": "garrar",
    "garrobales": "garrobal",
    "garrobas": "garroba",
    "garrobillas": "garrobilla",
    "garrocha": "garrochar",
    "garrochaba": "garrochar",
    "garrochabais": "garrochar",
    "garrochaban": "garrochar",
    "garrochabas": "garrochar",
    "garrochad": "garrochar",
    "garrochada": "garrochar",
    "garrochadas": "garrochar",
    "garrochado": "garrochar",
    "garrochados": "garrochar",
    "garrochamos": "garrochar",
    "garrochan": "garrochar",
    "garrochando": "garrochar",
    "garrochara": "garrochar",
    "garrocharais": "garrochar",
    "garrocharan": "garrochar",
    "garrocharas": "garrochar",
    "garrochare": "garrochar",
    "garrochareis": "garrochar",
    "garrocharemos": "garrochar",
    "garrocharen": "garrochar",
    "garrochares": "garrochar",
    "garrocharon": "garrochar",
    "garrochará": "garrochar",
    "garrocharán": "garrochar",
    "garrocharás": "garrochar",
    "garrocharé": "garrochar",
    "garrocharéis": "garrochar",
    "garrocharía": "garrochar",
    "garrocharíais": "garrochar",
    "garrocharíamos": "garrochar",
    "garrocharían": "garrochar",
    "garrocharías": "garrochar",
    "garrochas": "garrochar",
    "garrochase": "garrochar",
    "garrochaseis": "garrochar",
    "garrochasen": "garrochar",
    "garrochases": "garrochar",
    "garrochaste": "garrochar",
    "garrochasteis": "garrochar",
    "garroche": "garrochar",
    "garrochea": "garrochear",
    "garrocheaba": "garrochear",
    "garrocheabais": "garrochear",
    "garrocheaban": "garrochear",
    "garrocheabas": "garrochear",
    "garrochead": "garrochear",
    "garrocheada": "garrochear",
    "garrocheadas": "garrochear",
    "garrocheado": "garrochear",
    "garrocheados": "garrochear",
    "garrocheamos": "garrochear",
    "garrochean": "garrochear",
    "garrocheando": "garrochear",
    "garrocheara": "garrochear",
    "garrochearais": "garrochear",
    "garrochearan": "garrochear",
    "garrochearas": "garrochear",
    "garrocheare": "garrochear",
    "garrocheareis": "garrochear",
    "garrochearemos": "garrochear",
    "garrochearen": "garrochear",
    "garrocheares": "garrochear",
    "garrochearon": "garrochear",
    "garrocheará": "garrochear",
    "garrochearán": "garrochear",
    "garrochearás": "garrochear",
    "garrochearé": "garrochear",
    "garrochearéis": "garrochear",
    "garrochearía": "garrochear",
    "garrochearíais": "garrochear",
    "garrochearíamos": "garrochear",
    "garrochearían": "garrochear",
    "garrochearías": "garrochear",
    "garrocheas": "garrochear",
    "garrochease": "garrochear",
    "garrocheaseis": "garrochear",
    "garrocheasen": "garrochear",
    "garrocheases": "garrochear",
    "garrocheaste": "garrochear",
    "garrocheasteis": "garrochear",
    "garrochee": "garrochear",
    "garrocheemos": "garrochear",
    "garrocheen": "garrochear",
    "garrochees": "garrochear",
    "garrochemos": "garrochar",
    "garrochen": "garrochar",
    "garrocheo": "garrochear",
    "garroches": "garrochar",
    "garrocheábamos": "garrochear",
    "garrocheáis": "garrochear",
    "garrocheáramos": "garrochear",
    "garrocheáremos": "garrochear",
    "garrocheásemos": "garrochear",
    "garrocheé": "garrochear",
    "garrocheéis": "garrochear",
    "garrocheó": "garrochear",
    "garrocho": "garrochar",
    "garrochones": "garrochón",
    "garrochábamos": "garrochar",
    "garrocháis": "garrochar",
    "garrocháramos": "garrochar",
    "garrocháremos": "garrochar",
    "garrochásemos": "garrochar",
    "garroché": "garrochar",
    "garrochéis": "garrochar",
    "garrochó": "garrochar",
    "garrofales": "garrofal",
    "garrones": "garrón",
    "garrotazos": "garrotazo",
    "garrotea": "garrotear",
    "garroteaba": "garrotear",
    "garroteabais": "garrotear",
    "garroteaban": "garrotear",
    "garroteabas": "garrotear",
    "garrotead": "garrotear",
    "garroteada": "garrotear",
    "garroteadas": "garrotear",
    "garroteado": "garrotear",
    "garroteados": "garrotear",
    "garroteamos": "garrotear",
    "garrotean": "garrotear",
    "garroteando": "garrotear",
    "garroteara": "garrotear",
    "garrotearais": "garrotear",
    "garrotearan": "garrotear",
    "garrotearas": "garrotear",
    "garroteare": "garrotear",
    "garroteareis": "garrotear",
    "garrotearemos": "garrotear",
    "garrotearen": "garrotear",
    "garroteares": "garrotear",
    "garrotearon": "garrotear",
    "garroteará": "garrotear",
    "garrotearán": "garrotear",
    "garrotearás": "garrotear",
    "garrotearé": "garrotear",
    "garrotearéis": "garrotear",
    "garrotearía": "garrotear",
    "garrotearíais": "garrotear",
    "garrotearíamos": "garrotear",
    "garrotearían": "garrotear",
    "garrotearías": "garrotear",
    "garroteas": "garrotear",
    "garrotease": "garrotear",
    "garroteaseis": "garrotear",
    "garroteasen": "garrotear",
    "garroteases": "garrotear",
    "garroteaste": "garrotear",
    "garroteasteis": "garrotear",
    "garrotee": "garrotear",
    "garroteemos": "garrotear",
    "garroteen": "garrotear",
    "garrotees": "garrotear",
    "garroteo": "garrotear",
    "garrotes": "garrote",
    "garroteábamos": "garrotear",
    "garroteáis": "garrotear",
    "garroteáramos": "garrotear",
    "garroteáremos": "garrotear",
    "garroteásemos": "garrotear",
    "garroteé": "garrotear",
    "garroteéis": "garrotear",
    "garroteó": "garrotear",
    "garrotillos": "garrotillo",
    "garrotines": "garrotín",
    "garruchas": "garrucha",
    "garruchos": "garrucho",
    "garruda": "garrudo",
    "garrudas": "garrudo",
    "garrudos": "garrudo",
    "garrulerías": "garrulería",
    "garrulidades": "garrulidad",
    "garrábamos": "garrar",
    "garráis": "garrar",
    "garráramos": "garrar",
    "garráremos": "garrar",
    "garrásemos": "garrar",
    "garré": "garrar",
    "garréis": "garrar",
    "garró": "garrar",
    "garua": "garuar",
    "garuaba": "garuar",
    "garuabais": "garuar",
    "garuaban": "garuar",
    "garuabas": "garuar",
    "garuad": "garuar",
    "garuada": "garuar",
    "garuadas": "garuar",
    "garuado": "garuar",
    "garuados": "garuar",
    "garuamos": "garuar",
    "garuan": "garuar",
    "garuando": "garuar",
    "garuara": "garuar",
    "garuarais": "garuar",
    "garuaran": "garuar",
    "garuaras": "garuar",
    "garuare": "garuar",
    "garuareis": "garuar",
    "garuaremos": "garuar",
    "garuaren": "garuar",
    "garuares": "garuar",
    "garuaron": "garuar",
    "garuará": "garuar",
    "garuarán": "garuar",
    "garuarás": "garuar",
    "garuaré": "garuar",
    "garuaréis": "garuar",
    "garuaría": "garuar",
    "garuaríais": "garuar",
    "garuaríamos": "garuar",
    "garuarían": "garuar",
    "garuarías": "garuar",
    "garuas": "garuar",
    "garuase": "garuar",
    "garuaseis": "garuar",
    "garuasen": "garuar",
    "garuases": "garuar",
    "garuaste": "garuar",
    "garuasteis": "garuar",
    "garue": "garuar",
    "garuemos": "garuar",
    "garuen": "garuar",
    "garues": "garuar",
    "garulladas": "garullada",
    "garullas": "garulla",
    "garuo": "garuar",
    "garuábamos": "garuar",
    "garuáis": "garuar",
    "garuáramos": "garuar",
    "garuáremos": "garuar",
    "garuásemos": "garuar",
    "garué": "garuar",
    "garuéis": "garuar",
    "garuó": "garuar",
    "garza": "garzo",
    "garzas": "garzo",
    "garzonea": "garzonear",
    "garzoneaba": "garzonear",
    "garzoneabais": "garzonear",
    "garzoneaban": "garzonear",
    "garzoneabas": "garzonear",
    "garzonead": "garzonear",
    "garzoneada": "garzonear",
    "garzoneadas": "garzonear",
    "garzoneado": "garzonear",
    "garzoneados": "garzonear",
    "garzoneamos": "garzonear",
    "garzonean": "garzonear",
    "garzoneando": "garzonear",
    "garzoneara": "garzonear",
    "garzonearais": "garzonear",
    "garzonearan": "garzonear",
    "garzonearas": "garzonear",
    "garzoneare": "garzonear",
    "garzoneareis": "garzonear",
    "garzonearemos": "garzonear",
    "garzonearen": "garzonear",
    "garzoneares": "garzonear",
    "garzonearon": "garzonear",
    "garzoneará": "garzonear",
    "garzonearán": "garzonear",
    "garzonearás": "garzonear",
    "garzonearé": "garzonear",
    "garzonearéis": "garzonear",
    "garzonearía": "garzonear",
    "garzonearíais": "garzonear",
    "garzonearíamos": "garzonear",
    "garzonearían": "garzonear",
    "garzonearías": "garzonear",
    "garzoneas": "garzonear",
    "garzonease": "garzonear",
    "garzoneaseis": "garzonear",
    "garzoneasen": "garzonear",
    "garzoneases": "garzonear",
    "garzoneaste": "garzonear",
    "garzoneasteis": "garzonear",
    "garzonee": "garzonear",
    "garzoneemos": "garzonear",
    "garzoneen": "garzonear",
    "garzonees": "garzonear",
    "garzoneo": "garzonear",
    "garzones": "garzón",
    "garzoneábamos": "garzonear",
    "garzoneáis": "garzonear",
    "garzoneáramos": "garzonear",
    "garzoneáremos": "garzonear",
    "garzoneásemos": "garzonear",
    "garzoneé": "garzonear",
    "garzoneéis": "garzonear",
    "garzoneó": "garzonear",
    "garzos": "garzo",
    "garzotas": "garzota",
    "garúas": "garúa",
    "gas-oils": "gas-oil",
    "gasas": "gasa",
    "gascona": "gascón",
    "gasconas": "gascón",
    "gascones": "gascón",
    "gascóna": "gascón",
    "gascónas": "gascón",
    "gascónes": "gascón",
    "gasea": "gasear",
    "gaseaba": "gasear",
    "gaseabais": "gasear",
    "gaseaban": "gasear",
    "gaseabas": "gasear",
    "gasead": "gasear",
    "gaseada": "gasear",
    "gaseadas": "gasear",
    "gaseado": "gasear",
    "gaseados": "gasear",
    "gaseamos": "gasear",
    "gasean": "gasear",
    "gaseando": "gasear",
    "gaseara": "gasear",
    "gasearais": "gasear",
    "gasearan": "gasear",
    "gasearas": "gasear",
    "gaseare": "gasear",
    "gaseareis": "gasear",
    "gasearemos": "gasear",
    "gasearen": "gasear",
    "gaseares": "gasear",
    "gasearon": "gasear",
    "gaseará": "gasear",
    "gasearán": "gasear",
    "gasearás": "gasear",
    "gasearé": "gasear",
    "gasearéis": "gasear",
    "gasearía": "gasear",
    "gasearíais": "gasear",
    "gasearíamos": "gasear",
    "gasearían": "gasear",
    "gasearías": "gasear",
    "gaseas": "gasear",
    "gasease": "gasear",
    "gaseaseis": "gasear",
    "gaseasen": "gasear",
    "gaseases": "gasear",
    "gaseaste": "gasear",
    "gaseasteis": "gasear",
    "gasee": "gasear",
    "gaseemos": "gasear",
    "gaseen": "gasear",
    "gasees": "gasear",
    "gaseiformes": "gaseiforme",
    "gaseo": "gasear",
    "gaseoductos": "gaseoducto",
    "gaseosa": "gaseoso",
    "gaseosas": "gaseoso",
    "gaseosos": "gaseoso",
    "gases": "gas",
    "gaseábamos": "gasear",
    "gaseáis": "gasear",
    "gaseáramos": "gasear",
    "gaseáremos": "gasear",
    "gaseásemos": "gasear",
    "gaseé": "gasear",
    "gaseéis": "gasear",
    "gaseó": "gasear",
    "gasifica": "gasificar",
    "gasificaba": "gasificar",
    "gasificabais": "gasificar",
    "gasificaban": "gasificar",
    "gasificabas": "gasificar",
    "gasificables": "gasificable",
    "gasificaciones": "gasificación",
    "gasificad": "gasificar",
    "gasificada": "gasificar",
    "gasificadas": "gasificar",
    "gasificado": "gasificar",
    "gasificadores": "gasificador",
    "gasificados": "gasificar",
    "gasificamos": "gasificar",
    "gasifican": "gasificar",
    "gasificando": "gasificar",
    "gasificara": "gasificar",
    "gasificarais": "gasificar",
    "gasificaran": "gasificar",
    "gasificaras": "gasificar",
    "gasificare": "gasificar",
    "gasificareis": "gasificar",
    "gasificaremos": "gasificar",
    "gasificaren": "gasificar",
    "gasificares": "gasificar",
    "gasificaron": "gasificar",
    "gasificará": "gasificar",
    "gasificarán": "gasificar",
    "gasificarás": "gasificar",
    "gasificaré": "gasificar",
    "gasificaréis": "gasificar",
    "gasificaría": "gasificar",
    "gasificaríais": "gasificar",
    "gasificaríamos": "gasificar",
    "gasificarían": "gasificar",
    "gasificarías": "gasificar",
    "gasificas": "gasificar",
    "gasificase": "gasificar",
    "gasificaseis": "gasificar",
    "gasificasen": "gasificar",
    "gasificases": "gasificar",
    "gasificaste": "gasificar",
    "gasificasteis": "gasificar",
    "gasifico": "gasificar",
    "gasificábamos": "gasificar",
    "gasificáis": "gasificar",
    "gasificáramos": "gasificar",
    "gasificáremos": "gasificar",
    "gasificásemos": "gasificar",
    "gasificó": "gasificar",
    "gasifique": "gasificar",
    "gasifiquemos": "gasificar",
    "gasifiquen": "gasificar",
    "gasifiques": "gasificar",
    "gasifiqué": "gasificar",
    "gasifiquéis": "gasificar",
    "gasiformes": "gasiforme",
    "gasistas": "gasista",
    "gasoductos": "gasoducto",
    "gasoholes": "gasohol",
    "gasoils": "gasoil",
    "gasolinas": "gasolina",
    "gasolinera": "gasolinero",
    "gasolineras": "gasolinero",
    "gasolineros": "gasolinero",
    "gasométrica": "gasométrico",
    "gasométricas": "gasométrico",
    "gasométricos": "gasométrico",
    "gasta": "gastar",
    "gastaba": "gastar",
    "gastabais": "gastar",
    "gastaban": "gastar",
    "gastabas": "gastar",
    "gastad": "gastar",
    "gastada": "gastar",
    "gastadas": "gastar",
    "gastaderos": "gastadero",
    "gastado": "gastar",
    "gastadora": "gastador",
    "gastadoras": "gastador",
    "gastadores": "gastador",
    "gastados": "gastar",
    "gastamos": "gastar",
    "gastan": "gastar",
    "gastando": "gastar",
    "gastara": "gastar",
    "gastarais": "gastar",
    "gastaran": "gastar",
    "gastaras": "gastar",
    "gastare": "gastar",
    "gastareis": "gastar",
    "gastaremos": "gastar",
    "gastaren": "gastar",
    "gastares": "gastar",
    "gastaron": "gastar",
    "gastará": "gastar",
    "gastarán": "gastar",
    "gastarás": "gastar",
    "gastaré": "gastar",
    "gastaréis": "gastar",
    "gastaría": "gastar",
    "gastaríais": "gastar",
    "gastaríamos": "gastar",
    "gastarían": "gastar",
    "gastarías": "gastar",
    "gastas": "gastar",
    "gastase": "gastar",
    "gastaseis": "gastar",
    "gastasen": "gastar",
    "gastases": "gastar",
    "gastaste": "gastar",
    "gastasteis": "gastar",
    "gaste": "gastar",
    "gastemos": "gastar",
    "gasten": "gastar",
    "gasterópodos": "gasterópodo",
    "gastes": "gastar",
    "gasto": "gastar",
    "gastos": "gasto",
    "gastralgias": "gastralgia",
    "gastrectasias": "gastrectasia",
    "gastrectomías": "gastrectomía",
    "gastrocelos": "gastrocelo",
    "gastrocnemia": "gastrocnemio",
    "gastrocnemias": "gastrocnemio",
    "gastrocnemios": "gastrocnemio",
    "gastrocólica": "gastrocólico",
    "gastrocólicas": "gastrocólico",
    "gastrocólicos": "gastrocólico",
    "gastroduodenales": "gastroduodenal",
    "gastroenterologías": "gastroenterología",
    "gastroenteróloga": "gastroenterólogo",
    "gastroenterólogas": "gastroenterólogo",
    "gastroenterólogos": "gastroenterólogo",
    "gastroepiploica": "gastroepiploico",
    "gastroepiploicas": "gastroepiploico",
    "gastroepiploicos": "gastroepiploico",
    "gastroesplénica": "gastroesplénico",
    "gastroesplénicas": "gastroesplénico",
    "gastroesplénicos": "gastroesplénico",
    "gastrofrénica": "gastrofrénico",
    "gastrofrénicas": "gastrofrénico",
    "gastrofrénicos": "gastrofrénico",
    "gastrointestinales": "gastrointestinal",
    "gastrologías": "gastrología",
    "gastronomías": "gastronomía",
    "gastronómica": "gastronómico",
    "gastronómicas": "gastronómico",
    "gastronómicos": "gastronómico",
    "gastropancreática": "gastropancreático",
    "gastropancreáticas": "gastropancreático",
    "gastropancreáticos": "gastropancreático",
    "gastropatías": "gastropatía",
    "gastrorragias": "gastrorragia",
    "gastroscopias": "gastroscopia",
    "gastrotomías": "gastrotomía",
    "gastrotricos": "gastrotrico",
    "gastrovasculares": "gastrovascular",
    "gastrulaciones": "gastrulación",
    "gastrálgica": "gastrálgico",
    "gastrálgicas": "gastrálgico",
    "gastrálgicos": "gastrálgico",
    "gastróloga": "gastrólogo",
    "gastrólogas": "gastrólogo",
    "gastrólogos": "gastrólogo",
    "gastrónoma": "gastrónomo",
    "gastrónomas": "gastrónomo",
    "gastrónomos": "gastrónomo",
    "gastrópodos": "gastrópodo",
    "gastábamos": "gastar",
    "gastáis": "gastar",
    "gastáramos": "gastar",
    "gastáremos": "gastar",
    "gastásemos": "gastar",
    "gasté": "gastar",
    "gastéis": "gastar",
    "gastó": "gastar",
    "gasógenos": "gasógeno",
    "gasóleos": "gasóleo",
    "gasómetros": "gasómetro",
    "gata": "gato",
    "gatadas": "gatada",
    "gatallona": "gatallón",
    "gatallonas": "gatallón",
    "gatallones": "gatallón",
    "gatas": "gato",
    "gatazos": "gatazo",
    "gatea": "gatear",
    "gateaba": "gatear",
    "gateabais": "gatear",
    "gateaban": "gatear",
    "gateabas": "gatear",
    "gatead": "gatear",
    "gateada": "gatear",
    "gateadas": "gatear",
    "gateado": "gatear",
    "gateados": "gatear",
    "gateamos": "gatear",
    "gatean": "gatear",
    "gateando": "gatear",
    "gateara": "gatear",
    "gatearais": "gatear",
    "gatearan": "gatear",
    "gatearas": "gatear",
    "gateare": "gatear",
    "gateareis": "gatear",
    "gatearemos": "gatear",
    "gatearen": "gatear",
    "gateares": "gatear",
    "gatearon": "gatear",
    "gateará": "gatear",
    "gatearán": "gatear",
    "gatearás": "gatear",
    "gatearé": "gatear",
    "gatearéis": "gatear",
    "gatearía": "gatear",
    "gatearíais": "gatear",
    "gatearíamos": "gatear",
    "gatearían": "gatear",
    "gatearías": "gatear",
    "gateas": "gatear",
    "gatease": "gatear",
    "gateaseis": "gatear",
    "gateasen": "gatear",
    "gateases": "gatear",
    "gateaste": "gatear",
    "gateasteis": "gatear",
    "gatee": "gatear",
    "gateemos": "gatear",
    "gateen": "gatear",
    "gatees": "gatear",
    "gatenas": "gatena",
    "gateo": "gatear",
    "gatera": "gatero",
    "gateras": "gatero",
    "gateros": "gatero",
    "gaterías": "gatería",
    "gatesca": "gatesco",
    "gatescas": "gatesco",
    "gatescos": "gatesco",
    "gateábamos": "gatear",
    "gateáis": "gatear",
    "gateáramos": "gatear",
    "gateáremos": "gatear",
    "gateásemos": "gatear",
    "gateé": "gatear",
    "gateéis": "gatear",
    "gateó": "gatear",
    "gatillazos": "gatillazo",
    "gatillos": "gatillo",
    "gatos": "gato",
    "gatuna": "gatuno",
    "gatunas": "gatuno",
    "gatunos": "gatuno",
    "gatuperios": "gatuperio",
    "gatuñas": "gatuña",
    "gaucha": "gaucho",
    "gauchadas": "gauchada",
    "gauchas": "gaucho",
    "gauchesca": "gauchesco",
    "gauchescas": "gauchesco",
    "gauchescos": "gauchesco",
    "gauchos": "gaucho",
    "gaullistas": "gaullista",
    "gausiana": "gausiano",
    "gausianas": "gausiano",
    "gausianos": "gausiano",
    "gausios": "gausio",
    "gausiómetros": "gausiómetro",
    "gaussiana": "gaussiano",
    "gaussianas": "gaussiano",
    "gaussianos": "gaussiano",
    "gavetas": "gaveta",
    "gavetera": "gavetero",
    "gaveteras": "gavetero",
    "gaveteros": "gavetero",
    "gaviales": "gavial",
    "gavias": "gavia",
    "gaviera": "gaviero",
    "gavieras": "gaviero",
    "gavieros": "gaviero",
    "gavietas": "gavieta",
    "gavietes": "gaviete",
    "gavilancillos": "gavilancillo",
    "gavilanes": "gavilán",
    "gavilla": "gavillar",
    "gavillaba": "gavillar",
    "gavillabais": "gavillar",
    "gavillaban": "gavillar",
    "gavillabas": "gavillar",
    "gavillad": "gavillar",
    "gavillada": "gavillar",
    "gavilladas": "gavillar",
    "gavillado": "gavillar",
    "gavilladora": "gavillador",
    "gavilladoras": "gavillador",
    "gavilladores": "gavillador",
    "gavillados": "gavillar",
    "gavillamos": "gavillar",
    "gavillan": "gavillar",
    "gavillando": "gavillar",
    "gavillara": "gavillar",
    "gavillarais": "gavillar",
    "gavillaran": "gavillar",
    "gavillaras": "gavillar",
    "gavillare": "gavillar",
    "gavillareis": "gavillar",
    "gavillaremos": "gavillar",
    "gavillaren": "gavillar",
    "gavillares": "gavillar",
    "gavillaron": "gavillar",
    "gavillará": "gavillar",
    "gavillarán": "gavillar",
    "gavillarás": "gavillar",
    "gavillaré": "gavillar",
    "gavillaréis": "gavillar",
    "gavillaría": "gavillar",
    "gavillaríais": "gavillar",
    "gavillaríamos": "gavillar",
    "gavillarían": "gavillar",
    "gavillarías": "gavillar",
    "gavillas": "gavillar",
    "gavillase": "gavillar",
    "gavillaseis": "gavillar",
    "gavillasen": "gavillar",
    "gavillases": "gavillar",
    "gavillaste": "gavillar",
    "gavillasteis": "gavillar",
    "gaville": "gavillar",
    "gavillemos": "gavillar",
    "gavillen": "gavillar",
    "gavillera": "gavillero",
    "gavilleras": "gavillero",
    "gavilleros": "gavillero",
    "gavilles": "gavillar",
    "gavillo": "gavillar",
    "gavillábamos": "gavillar",
    "gavilláis": "gavillar",
    "gavilláramos": "gavillar",
    "gavilláremos": "gavillar",
    "gavillásemos": "gavillar",
    "gavillé": "gavillar",
    "gavilléis": "gavillar",
    "gavilló": "gavillar",
    "gavimétrica": "gavimétrico",
    "gavimétricas": "gavimétrico",
    "gavimétricos": "gavimétrico",
    "gavinas": "gavina",
    "gavinotes": "gavinote",
    "gaviones": "gavión",
    "gaviotas": "gaviota",
    "gavotas": "gavota",
    "gaya": "gayo",
    "gayaba": "gayar",
    "gayabais": "gayar",
    "gayaban": "gayar",
    "gayabas": "gayar",
    "gayad": "gayar",
    "gayada": "gayar",
    "gayadas": "gayar",
    "gayado": "gayar",
    "gayados": "gayar",
    "gayaduras": "gayadura",
    "gayamos": "gayar",
    "gayan": "gayar",
    "gayando": "gayar",
    "gayara": "gayar",
    "gayarais": "gayar",
    "gayaran": "gayar",
    "gayaras": "gayar",
    "gayare": "gayar",
    "gayareis": "gayar",
    "gayaremos": "gayar",
    "gayaren": "gayar",
    "gayares": "gayar",
    "gayaron": "gayar",
    "gayará": "gayar",
    "gayarán": "gayar",
    "gayarás": "gayar",
    "gayaré": "gayar",
    "gayaréis": "gayar",
    "gayaría": "gayar",
    "gayaríais": "gayar",
    "gayaríamos": "gayar",
    "gayarían": "gayar",
    "gayarías": "gayar",
    "gayas": "gayo",
    "gayase": "gayar",
    "gayaseis": "gayar",
    "gayasen": "gayar",
    "gayases": "gayar",
    "gayaste": "gayar",
    "gayasteis": "gayar",
    "gaye": "gayar",
    "gayemos": "gayar",
    "gayen": "gayar",
    "gayes": "gayar",
    "gaylusitas": "gaylusita",
    "gayo": "gayar",
    "gayolas": "gayola",
    "gayos": "gayo",
    "gays": "gay",
    "gayubas": "gayuba",
    "gayábamos": "gayar",
    "gayáis": "gayar",
    "gayáramos": "gayar",
    "gayáremos": "gayar",
    "gayásemos": "gayar",
    "gayé": "gayar",
    "gayéis": "gayar",
    "gayó": "gayar",
    "gazafatones": "gazafatón",
    "gazapas": "gazapa",
    "gazapatones": "gazapatón",
    "gazapeos": "gazapeo",
    "gazaperas": "gazapera",
    "gazapinas": "gazapina",
    "gazapones": "gazapón",
    "gazapos": "gazapo",
    "gazas": "gaza",
    "gazmia": "gazmiar",
    "gazmiaba": "gazmiar",
    "gazmiabais": "gazmiar",
    "gazmiaban": "gazmiar",
    "gazmiabas": "gazmiar",
    "gazmiad": "gazmiar",
    "gazmiada": "gazmiar",
    "gazmiadas": "gazmiar",
    "gazmiado": "gazmiar",
    "gazmiados": "gazmiar",
    "gazmiamos": "gazmiar",
    "gazmian": "gazmiar",
    "gazmiando": "gazmiar",
    "gazmiara": "gazmiar",
    "gazmiarais": "gazmiar",
    "gazmiaran": "gazmiar",
    "gazmiaras": "gazmiar",
    "gazmiare": "gazmiar",
    "gazmiareis": "gazmiar",
    "gazmiaremos": "gazmiar",
    "gazmiaren": "gazmiar",
    "gazmiares": "gazmiar",
    "gazmiaron": "gazmiar",
    "gazmiará": "gazmiar",
    "gazmiarán": "gazmiar",
    "gazmiarás": "gazmiar",
    "gazmiaré": "gazmiar",
    "gazmiaréis": "gazmiar",
    "gazmiaría": "gazmiar",
    "gazmiaríais": "gazmiar",
    "gazmiaríamos": "gazmiar",
    "gazmiarían": "gazmiar",
    "gazmiarías": "gazmiar",
    "gazmias": "gazmiar",
    "gazmiase": "gazmiar",
    "gazmiaseis": "gazmiar",
    "gazmiasen": "gazmiar",
    "gazmiases": "gazmiar",
    "gazmiaste": "gazmiar",
    "gazmiasteis": "gazmiar",
    "gazmie": "gazmiar",
    "gazmiemos": "gazmiar",
    "gazmien": "gazmiar",
    "gazmies": "gazmiar",
    "gazmio": "gazmiar",
    "gazmiábamos": "gazmiar",
    "gazmiáis": "gazmiar",
    "gazmiáramos": "gazmiar",
    "gazmiáremos": "gazmiar",
    "gazmiásemos": "gazmiar",
    "gazmié": "gazmiar",
    "gazmiéis": "gazmiar",
    "gazmió": "gazmiar",
    "gazmoles": "gazmol",
    "gazmoña": "gazmoño",
    "gazmoñas": "gazmoño",
    "gazmoñera": "gazmoñero",
    "gazmoñeras": "gazmoñero",
    "gazmoñeros": "gazmoñero",
    "gazmoñerías": "gazmoñería",
    "gazmoños": "gazmoño",
    "gaznatadas": "gaznatada",
    "gaznatazos": "gaznatazo",
    "gaznates": "gaznate",
    "gaznatones": "gaznatón",
    "gaznawíes": "gaznawí",
    "gaznápira": "gaznápiro",
    "gaznápiras": "gaznápiro",
    "gaznápiros": "gaznápiro",
    "gazpachos": "gazpacho",
    "gazuzas": "gazuza",
    "gaélica": "gaélico",
    "gaélicas": "gaélico",
    "gaélicos": "gaélico",
    "gaña": "gañir",
    "gañamos": "gañir",
    "gañan": "gañir",
    "gañanes": "gañán",
    "gañanías": "gañanía",
    "gañas": "gañir",
    "gañe": "gañir",
    "gañen": "gañir",
    "gañendo": "gañir",
    "gañera": "gañir",
    "gañerais": "gañir",
    "gañeran": "gañir",
    "gañeras": "gañir",
    "gañere": "gañir",
    "gañereis": "gañir",
    "gañeren": "gañir",
    "gañeres": "gañir",
    "gañeron": "gañir",
    "gañes": "gañir",
    "gañese": "gañir",
    "gañeseis": "gañir",
    "gañesen": "gañir",
    "gañeses": "gañir",
    "gañid": "gañir",
    "gañida": "gañir",
    "gañidas": "gañir",
    "gañido": "gañir",
    "gañidos": "gañir",
    "gañiles": "gañil",
    "gañimos": "gañir",
    "gañiremos": "gañir",
    "gañirá": "gañir",
    "gañirán": "gañir",
    "gañirás": "gañir",
    "gañiré": "gañir",
    "gañiréis": "gañir",
    "gañiría": "gañir",
    "gañiríais": "gañir",
    "gañiríamos": "gañir",
    "gañirían": "gañir",
    "gañirías": "gañir",
    "gañiste": "gañir",
    "gañisteis": "gañir",
    "gaño": "gañir",
    "gañotes": "gañote",
    "gañáis": "gañir",
    "gañéramos": "gañir",
    "gañéremos": "gañir",
    "gañésemos": "gañir",
    "gañí": "gañir",
    "gañía": "gañir",
    "gañíais": "gañir",
    "gañíamos": "gañir",
    "gañían": "gañir",
    "gañías": "gañir",
    "gañís": "gañir",
    "gañó": "gañir",
    "geas": "gea",
    "gecos": "geco",
    "gecónidos": "gecónido",
    "gedanitas": "gedanita",
    "gehenas": "gehena",
    "geiserses": "geiser",
    "geishas": "geisha",
    "geladas": "gelada",
    "gelatinas": "gelatina",
    "gelatinice": "gelatinizar",
    "gelatinicemos": "gelatinizar",
    "gelatinicen": "gelatinizar",
    "gelatinices": "gelatinizar",
    "gelatinicé": "gelatinizar",
    "gelatinicéis": "gelatinizar",
    "gelatiniza": "gelatinizar",
    "gelatinizaba": "gelatinizar",
    "gelatinizabais": "gelatinizar",
    "gelatinizaban": "gelatinizar",
    "gelatinizabas": "gelatinizar",
    "gelatinizaciones": "gelatinización",
    "gelatinizad": "gelatinizar",
    "gelatinizada": "gelatinizar",
    "gelatinizadas": "gelatinizar",
    "gelatinizado": "gelatinizar",
    "gelatinizados": "gelatinizar",
    "gelatinizamos": "gelatinizar",
    "gelatinizan": "gelatinizar",
    "gelatinizando": "gelatinizar",
    "gelatinizara": "gelatinizar",
    "gelatinizarais": "gelatinizar",
    "gelatinizaran": "gelatinizar",
    "gelatinizaras": "gelatinizar",
    "gelatinizare": "gelatinizar",
    "gelatinizareis": "gelatinizar",
    "gelatinizaremos": "gelatinizar",
    "gelatinizaren": "gelatinizar",
    "gelatinizares": "gelatinizar",
    "gelatinizaron": "gelatinizar",
    "gelatinizará": "gelatinizar",
    "gelatinizarán": "gelatinizar",
    "gelatinizarás": "gelatinizar",
    "gelatinizaré": "gelatinizar",
    "gelatinizaréis": "gelatinizar",
    "gelatinizaría": "gelatinizar",
    "gelatinizaríais": "gelatinizar",
    "gelatinizaríamos": "gelatinizar",
    "gelatinizarían": "gelatinizar",
    "gelatinizarías": "gelatinizar",
    "gelatinizas": "gelatinizar",
    "gelatinizase": "gelatinizar",
    "gelatinizaseis": "gelatinizar",
    "gelatinizasen": "gelatinizar",
    "gelatinizases": "gelatinizar",
    "gelatinizaste": "gelatinizar",
    "gelatinizasteis": "gelatinizar",
    "gelatinizo": "gelatinizar",
    "gelatinizábamos": "gelatinizar",
    "gelatinizáis": "gelatinizar",
    "gelatinizáramos": "gelatinizar",
    "gelatinizáremos": "gelatinizar",
    "gelatinizásemos": "gelatinizar",
    "gelatinizó": "gelatinizar",
    "gelatinosa": "gelatinoso",
    "gelatinosas": "gelatinoso",
    "gelatinosos": "gelatinoso",
    "gelenitas": "gelenita",
    "geles": "gel",
    "gelifica": "gelificar",
    "gelificaba": "gelificar",
    "gelificabais": "gelificar",
    "gelificaban": "gelificar",
    "gelificabas": "gelificar",
    "gelificaciones": "gelificación",
    "gelificad": "gelificar",
    "gelificada": "gelificar",
    "gelificadas": "gelificar",
    "gelificado": "gelificar",
    "gelificados": "gelificar",
    "gelificamos": "gelificar",
    "gelifican": "gelificar",
    "gelificando": "gelificar",
    "gelificara": "gelificar",
    "gelificarais": "gelificar",
    "gelificaran": "gelificar",
    "gelificaras": "gelificar",
    "gelificare": "gelificar",
    "gelificareis": "gelificar",
    "gelificaremos": "gelificar",
    "gelificaren": "gelificar",
    "gelificares": "gelificar",
    "gelificaron": "gelificar",
    "gelificará": "gelificar",
    "gelificarán": "gelificar",
    "gelificarás": "gelificar",
    "gelificaré": "gelificar",
    "gelificaréis": "gelificar",
    "gelificaría": "gelificar",
    "gelificaríais": "gelificar",
    "gelificaríamos": "gelificar",
    "gelificarían": "gelificar",
    "gelificarías": "gelificar",
    "gelificas": "gelificar",
    "gelificase": "gelificar",
    "gelificaseis": "gelificar",
    "gelificasen": "gelificar",
    "gelificases": "gelificar",
    "gelificaste": "gelificar",
    "gelificasteis": "gelificar",
    "gelifico": "gelificar",
    "gelificábamos": "gelificar",
    "gelificáis": "gelificar",
    "gelificáramos": "gelificar",
    "gelificáremos": "gelificar",
    "gelificásemos": "gelificar",
    "gelificó": "gelificar",
    "gelifique": "gelificar",
    "gelifiquemos": "gelificar",
    "gelifiquen": "gelificar",
    "gelifiques": "gelificar",
    "gelifiqué": "gelificar",
    "gelifiquéis": "gelificar",
    "gelifluxiones": "gelifluxión",
    "gelifracciones": "gelifracción",
    "geliturbaciones": "geliturbación",
    "gelivaciones": "gelivación",
    "gelosas": "gelosa",
    "gelseminas": "gelsemina",
    "gelsolinas": "gelsolina",
    "gemaciones": "gemación",
    "gemas": "gema",
    "gemebunda": "gemebundo",
    "gemebundas": "gemebundo",
    "gemebundos": "gemebundo",
    "gemela": "gemelo",
    "gemelas": "gemelo",
    "gemelos": "gemelo",
    "gemid": "gemir",
    "gemida": "gemir",
    "gemidas": "gemir",
    "gemido": "gemir",
    "gemidos": "gemir",
    "gemimos": "gemir",
    "gemina": "geminar",
    "geminaba": "geminar",
    "geminabais": "geminar",
    "geminaban": "geminar",
    "geminabas": "geminar",
    "geminaciones": "geminación",
    "geminad": "geminar",
    "geminada": "geminar",
    "geminadas": "geminar",
    "geminado": "geminar",
    "geminados": "geminar",
    "geminamos": "geminar",
    "geminan": "geminar",
    "geminando": "geminar",
    "geminara": "geminar",
    "geminarais": "geminar",
    "geminaran": "geminar",
    "geminaras": "geminar",
    "geminare": "geminar",
    "geminareis": "geminar",
    "geminaremos": "geminar",
    "geminaren": "geminar",
    "geminares": "geminar",
    "geminaron": "geminar",
    "geminará": "geminar",
    "geminarán": "geminar",
    "geminarás": "geminar",
    "geminaré": "geminar",
    "geminaréis": "geminar",
    "geminaría": "geminar",
    "geminaríais": "geminar",
    "geminaríamos": "geminar",
    "geminarían": "geminar",
    "geminarías": "geminar",
    "geminas": "geminar",
    "geminase": "geminar",
    "geminaseis": "geminar",
    "geminasen": "geminar",
    "geminases": "geminar",
    "geminaste": "geminar",
    "geminasteis": "geminar",
    "gemine": "geminar",
    "geminemos": "geminar",
    "geminen": "geminar",
    "gemines": "geminar",
    "gemino": "geminar",
    "geminábamos": "geminar",
    "gemináis": "geminar",
    "gemináramos": "geminar",
    "gemináremos": "geminar",
    "geminásemos": "geminar",
    "geminé": "geminar",
    "geminéis": "geminar",
    "geminó": "geminar",
    "gemiquea": "gemiquear",
    "gemiqueaba": "gemiquear",
    "gemiqueabais": "gemiquear",
    "gemiqueaban": "gemiquear",
    "gemiqueabas": "gemiquear",
    "gemiquead": "gemiquear",
    "gemiqueada": "gemiquear",
    "gemiqueadas": "gemiquear",
    "gemiqueado": "gemiquear",
    "gemiqueados": "gemiquear",
    "gemiqueamos": "gemiquear",
    "gemiquean": "gemiquear",
    "gemiqueando": "gemiquear",
    "gemiqueara": "gemiquear",
    "gemiquearais": "gemiquear",
    "gemiquearan": "gemiquear",
    "gemiquearas": "gemiquear",
    "gemiqueare": "gemiquear",
    "gemiqueareis": "gemiquear",
    "gemiquearemos": "gemiquear",
    "gemiquearen": "gemiquear",
    "gemiqueares": "gemiquear",
    "gemiquearon": "gemiquear",
    "gemiqueará": "gemiquear",
    "gemiquearán": "gemiquear",
    "gemiquearás": "gemiquear",
    "gemiquearé": "gemiquear",
    "gemiquearéis": "gemiquear",
    "gemiquearía": "gemiquear",
    "gemiquearíais": "gemiquear",
    "gemiquearíamos": "gemiquear",
    "gemiquearían": "gemiquear",
    "gemiquearías": "gemiquear",
    "gemiqueas": "gemiquear",
    "gemiquease": "gemiquear",
    "gemiqueaseis": "gemiquear",
    "gemiqueasen": "gemiquear",
    "gemiqueases": "gemiquear",
    "gemiqueaste": "gemiquear",
    "gemiqueasteis": "gemiquear",
    "gemiquee": "gemiquear",
    "gemiqueemos": "gemiquear",
    "gemiqueen": "gemiquear",
    "gemiquees": "gemiquear",
    "gemiqueo": "gemiquear",
    "gemiqueábamos": "gemiquear",
    "gemiqueáis": "gemiquear",
    "gemiqueáramos": "gemiquear",
    "gemiqueáremos": "gemiquear",
    "gemiqueásemos": "gemiquear",
    "gemiqueé": "gemiquear",
    "gemiqueéis": "gemiquear",
    "gemiqueó": "gemiquear",
    "gemiremos": "gemir",
    "gemirá": "gemir",
    "gemirán": "gemir",
    "gemirás": "gemir",
    "gemiré": "gemir",
    "gemiréis": "gemir",
    "gemiría": "gemir",
    "gemiríais": "gemir",
    "gemiríamos": "gemir",
    "gemirían": "gemir",
    "gemirías": "gemir",
    "gemiste": "gemir",
    "gemisteis": "gemir",
    "gemologías": "gemología",
    "gemológica": "gemológico",
    "gemológicas": "gemológico",
    "gemológicos": "gemológico",
    "gemoterapias": "gemoterapia",
    "gemí": "gemir",
    "gemía": "gemir",
    "gemíais": "gemir",
    "gemíamos": "gemir",
    "gemían": "gemir",
    "gemías": "gemir",
    "gemípara": "gemíparo",
    "gemíparas": "gemíparo",
    "gemíparos": "gemíparo",
    "gemís": "gemir",
    "gemóloga": "gemólogo",
    "gemólogas": "gemólogo",
    "gemólogos": "gemólogo",
    "gencianas": "genciana",
    "gencianáceas": "gencianácea",
    "gendarmerías": "gendarmería",
    "gendarmes": "gendarme",
    "genealogistas": "genealogista",
    "genealogías": "genealogía",
    "genealógica": "genealógico",
    "genealógicas": "genealógico",
    "genealógicos": "genealógico",
    "genera": "generar",
    "generaba": "generar",
    "generabais": "generar",
    "generaban": "generar",
    "generabas": "generar",
    "generabilidades": "generabilidad",
    "generables": "generable",
    "generacionales": "generacional",
    "generaciones": "generación",
    "generad": "generar",
    "generada": "generar",
    "generadas": "generar",
    "generado": "generar",
    "generadora": "generador",
    "generadoras": "generador",
    "generadores": "generador",
    "generados": "generar",
    "generala": "general",
    "generalas": "general",
    "generalatos": "generalato",
    "generales": "general",
    "generalice": "generalizar",
    "generalicemos": "generalizar",
    "generalicen": "generalizar",
    "generalices": "generalizar",
    "generalicé": "generalizar",
    "generalicéis": "generalizar",
    "generalidades": "generalidad",
    "generalistas": "generalista",
    "generaliza": "generalizar",
    "generalizaba": "generalizar",
    "generalizabais": "generalizar",
    "generalizaban": "generalizar",
    "generalizabas": "generalizar",
    "generalizables": "generalizable",
    "generalizaciones": "generalización",
    "generalizad": "generalizar",
    "generalizada": "generalizar",
    "generalizadas": "generalizar",
    "generalizado": "generalizar",
    "generalizadores": "generalizador",
    "generalizados": "generalizar",
    "generalizamos": "generalizar",
    "generalizan": "generalizar",
    "generalizando": "generalizar",
    "generalizara": "generalizar",
    "generalizarais": "generalizar",
    "generalizaran": "generalizar",
    "generalizaras": "generalizar",
    "generalizare": "generalizar",
    "generalizareis": "generalizar",
    "generalizaremos": "generalizar",
    "generalizaren": "generalizar",
    "generalizares": "generalizar",
    "generalizaron": "generalizar",
    "generalizará": "generalizar",
    "generalizarán": "generalizar",
    "generalizarás": "generalizar",
    "generalizaré": "generalizar",
    "generalizaréis": "generalizar",
    "generalizaría": "generalizar",
    "generalizaríais": "generalizar",
    "generalizaríamos": "generalizar",
    "generalizarían": "generalizar",
    "generalizarías": "generalizar",
    "generalizas": "generalizar",
    "generalizase": "generalizar",
    "generalizaseis": "generalizar",
    "generalizasen": "generalizar",
    "generalizases": "generalizar",
    "generalizaste": "generalizar",
    "generalizasteis": "generalizar",
    "generalizo": "generalizar",
    "generalizábamos": "generalizar",
    "generalizáis": "generalizar",
    "generalizáramos": "generalizar",
    "generalizáremos": "generalizar",
    "generalizásemos": "generalizar",
    "generalizó": "generalizar",
    "generalísimos": "generalísimo",
    "generamos": "generar",
    "generan": "generar",
    "generando": "generar",
    "generara": "generar",
    "generarais": "generar",
    "generaran": "generar",
    "generaras": "generar",
    "generare": "generar",
    "generareis": "generar",
    "generaremos": "generar",
    "generaren": "generar",
    "generares": "generar",
    "generaron": "generar",
    "generará": "generar",
    "generarán": "generar",
    "generarás": "generar",
    "generaré": "generar",
    "generaréis": "generar",
    "generaría": "generar",
    "generaríais": "generar",
    "generaríamos": "generar",
    "generarían": "generar",
    "generarías": "generar",
    "generas": "generar",
    "generase": "generar",
    "generaseis": "generar",
    "generasen": "generar",
    "generases": "generar",
    "generaste": "generar",
    "generasteis": "generar",
    "generativa": "generativo",
    "generativas": "generativo",
    "generativismos": "generativismo",
    "generativos": "generativo",
    "generatrices": "generatriz",
    "genere": "generar",
    "generemos": "generar",
    "generen": "generar",
    "generes": "generar",
    "genero": "generar",
    "generosa": "generoso",
    "generosas": "generoso",
    "generosidades": "generosidad",
    "generosos": "generoso",
    "generábamos": "generar",
    "generáis": "generar",
    "generáramos": "generar",
    "generáremos": "generar",
    "generásemos": "generar",
    "generé": "generar",
    "generéis": "generar",
    "generó": "generar",
    "genes": "gen",
    "genesiaca": "genesíaco",
    "genesiacas": "genesíaco",
    "genesiacos": "genesíaco",
    "genesiologías": "genesiología",
    "genesíaca": "genesíaco",
    "genesíacas": "genesíaco",
    "genesíacos": "genesíaco",
    "genetistas": "genetista",
    "genetliacas": "genetliaca",
    "genetlíacas": "genetlíaca",
    "geneática": "geneático",
    "geneáticas": "geneático",
    "geneáticos": "geneático",
    "geniales": "genial",
    "genialidades": "genialidad",
    "geniana": "geniano",
    "genianas": "geniano",
    "genianos": "geniano",
    "geniculares": "genicular",
    "geniecillos": "geniecillo",
    "genioglosa": "geniogloso",
    "genioglosas": "geniogloso",
    "genioglosos": "geniogloso",
    "geniohioglosa": "geniohiogloso",
    "geniohioglosas": "geniohiogloso",
    "geniohioglosos": "geniohiogloso",
    "geniohioidea": "geniohioideo",
    "geniohioideas": "geniohioideo",
    "geniohioideos": "geniohioideo",
    "genioplastias": "genioplastia",
    "genios": "genio",
    "geniqueiloplastias": "geniqueiloplastia",
    "geniquiloplastias": "geniquiloplastia",
    "genitales": "genital",
    "genitiva": "genitivo",
    "genitivas": "genitivo",
    "genitivos": "genitivo",
    "genitocrurales": "genitocrural",
    "genitofemorales": "genitofemoral",
    "genitoinguinales": "genitoinguinal",
    "genitora": "genitor",
    "genitoras": "genitor",
    "genitores": "genitor",
    "genitourinaria": "genitourinario",
    "genitourinarias": "genitourinario",
    "genitourinarios": "genitourinario",
    "genoblastos": "genoblasto",
    "genocidas": "genocida",
    "genocidios": "genocidio",
    "genogramas": "genograma",
    "genoles": "genol",
    "genomas": "genoma",
    "genomios": "genomio",
    "genotecas": "genoteca",
    "genotipos": "genotipo",
    "genotípica": "genotípico",
    "genotípicas": "genotípico",
    "genotípicos": "genotípico",
    "genovesa": "genovés",
    "genovesas": "genovés",
    "genoveses": "genovés",
    "gentes": "gente",
    "gentiles": "gentil",
    "gentilezas": "gentileza",
    "gentilhombres": "gentilhombre",
    "gentilicia": "gentilicio",
    "gentilicias": "gentilicio",
    "gentilicios": "gentilicio",
    "gentilidades": "gentilidad",
    "gentilismos": "gentilismo",
    "gentilizaciones": "gentilización",
    "gentisatos": "gentisato",
    "gentlemanes": "gentleman",
    "gentlemans": "gentleman",
    "gentlemen": "gentleman",
    "gentuzas": "gentuza",
    "gentílica": "gentílico",
    "gentílicas": "gentílico",
    "gentílicos": "gentílico",
    "gentíos": "gentío",
    "genuflexiones": "genuflexión",
    "genuina": "genuino",
    "genuinas": "genuino",
    "genuinos": "genuino",
    "genérica": "genérico",
    "genéricas": "genérico",
    "genéricos": "genérico",
    "genésica": "genésico",
    "genésicas": "genésico",
    "genésicos": "genésico",
    "genética": "genético",
    "genéticas": "genético",
    "genéticos": "genético",
    "genízara": "genízaro",
    "genízaras": "genízaro",
    "genízaros": "genízaro",
    "geoanticlinos": "geoanticlino",
    "geoarqueologías": "geoarqueología",
    "geobiologías": "geobiología",
    "geobiotecnologías": "geobiotecnología",
    "geobotánicas": "geobotánica",
    "geocarpias": "geocarpia",
    "geocentrismos": "geocentrismo",
    "geocentros": "geocentro",
    "geocronologías": "geocronología",
    "geocronometrajes": "geocronometraje",
    "geocrática": "geocrático",
    "geocráticas": "geocrático",
    "geocráticos": "geocrático",
    "geocárpica": "geocárpico",
    "geocárpicas": "geocárpico",
    "geocárpicos": "geocárpico",
    "geocéntrica": "geocéntrico",
    "geocéntricas": "geocéntrico",
    "geocéntricos": "geocéntrico",
    "geodas": "geoda",
    "geodesias": "geodesia",
    "geodesistas": "geodesista",
    "geodestas": "geodesta",
    "geodinámicas": "geodinámica",
    "geodésica": "geodésico",
    "geodésicas": "geodésico",
    "geodésicos": "geodésico",
    "geodética": "geodético",
    "geodéticas": "geodético",
    "geodéticos": "geodético",
    "geoeconómica": "geoeconómico",
    "geoeconómicas": "geoeconómico",
    "geoeconómicos": "geoeconómico",
    "geoeléctrica": "geoeléctrico",
    "geoeléctricas": "geoeléctrico",
    "geoeléctricos": "geoeléctrico",
    "geoestacionaria": "geoestacionario",
    "geoestacionarias": "geoestacionario",
    "geoestacionarios": "geoestacionario",
    "geoestaciones": "geoestación",
    "geoestrategias": "geoestrategia",
    "geofagias": "geofagia",
    "geofracturas": "geofractura",
    "geofísica": "geofísico",
    "geofísicas": "geofísico",
    "geofísicos": "geofísico",
    "geogenias": "geogenia",
    "geognosias": "geognosia",
    "geognostas": "geognosta",
    "geognóstica": "geognóstico",
    "geognósticas": "geognóstico",
    "geognósticos": "geognóstico",
    "geogonías": "geogonía",
    "geografías": "geografía",
    "geográfica": "geográfico",
    "geográficas": "geográfico",
    "geográficos": "geográfico",
    "geogénica": "geogénico",
    "geogénicas": "geogénico",
    "geogénicos": "geogénico",
    "geohidrologías": "geohidrología",
    "geohidrológica": "geohidrológico",
    "geohidrológicas": "geohidrológico",
    "geohidrológicos": "geohidrológico",
    "geoides": "geoide",
    "geolingüísticas": "geolingüística",
    "geologías": "geología",
    "geológica": "geológico",
    "geológicas": "geológico",
    "geológicos": "geológico",
    "geomagnetismos": "geomagnetismo",
    "geomagnética": "geomagnético",
    "geomagnéticas": "geomagnético",
    "geomagnéticos": "geomagnético",
    "geomancias": "geomancia",
    "geomancías": "geomancía",
    "geomecánicas": "geomecánica",
    "geometrías": "geometría",
    "geomorfologías": "geomorfología",
    "geomorfológica": "geomorfológico",
    "geomorfológicas": "geomorfológico",
    "geomorfológicos": "geomorfológico",
    "geomorfías": "geomorfía",
    "geomorfóloga": "geomorfólogo",
    "geomorfólogas": "geomorfólogo",
    "geomorfólogos": "geomorfólogo",
    "geomántica": "geomántico",
    "geománticas": "geomántico",
    "geománticos": "geomántico",
    "geomáticas": "geomática",
    "geométrica": "geométrico",
    "geométricas": "geométrico",
    "geométricos": "geométrico",
    "geomórfica": "geomórfico",
    "geomórficas": "geomórfico",
    "geomórficos": "geomórfico",
    "geonomías": "geonomía",
    "geopolítica": "geopolítico",
    "geopolíticas": "geopolítico",
    "geopolíticos": "geopolítico",
    "geoponías": "geoponía",
    "geopónicas": "geopónica",
    "geoquímica": "geoquímico",
    "geoquímicas": "geoquímico",
    "geoquímicos": "geoquímico",
    "georamas": "georama",
    "georgiana": "georgiano",
    "georgianas": "georgiano",
    "georgianos": "georgiano",
    "geosferas": "geosfera",
    "geosinclinales": "geosinclinal",
    "geosinónimos": "geosinónimo",
    "geostrófica": "geostrófico",
    "geostróficas": "geostrófico",
    "geostróficos": "geostrófico",
    "geostática": "geostático",
    "geostáticas": "geostático",
    "geostáticos": "geostático",
    "geotecnias": "geotecnia",
    "geotectónica": "geotectónico",
    "geotectónicas": "geotectónico",
    "geotectónicos": "geotectónico",
    "geotermias": "geotermia",
    "geotermometrías": "geotermometría",
    "geotextiles": "geotextil",
    "geotropismos": "geotropismo",
    "geotrópica": "geotrópico",
    "geotrópicas": "geotrópico",
    "geotrópicos": "geotrópico",
    "geotécnica": "geotécnico",
    "geotécnicas": "geotécnico",
    "geotécnicos": "geotécnico",
    "geotérmica": "geotérmico",
    "geotérmicas": "geotérmico",
    "geotérmicos": "geotérmico",
    "geraniales": "geranial",
    "geranioles": "geraniol",
    "geranios": "geranio",
    "geraniáceas": "geraniácea",
    "gerenciales": "gerencial",
    "gerencias": "gerencia",
    "gerentes": "gerente",
    "geriatras": "geriatra",
    "geriatrías": "geriatría",
    "gerifaltes": "gerifalte",
    "geriátrica": "geriátrico",
    "geriátricas": "geriátrico",
    "geriátricos": "geriátrico",
    "germana": "germano",
    "germanas": "germano",
    "germanice": "germanizar",
    "germanicemos": "germanizar",
    "germanicen": "germanizar",
    "germanices": "germanizar",
    "germanicé": "germanizar",
    "germanicéis": "germanizar",
    "germanios": "germanio",
    "germanismos": "germanismo",
    "germanistas": "germanista",
    "germaniza": "germanizar",
    "germanizaba": "germanizar",
    "germanizabais": "germanizar",
    "germanizaban": "germanizar",
    "germanizabas": "germanizar",
    "germanizaciones": "germanización",
    "germanizad": "germanizar",
    "germanizada": "germanizar",
    "germanizadas": "germanizar",
    "germanizado": "germanizar",
    "germanizados": "germanizar",
    "germanizamos": "germanizar",
    "germanizan": "germanizar",
    "germanizando": "germanizar",
    "germanizara": "germanizar",
    "germanizarais": "germanizar",
    "germanizaran": "germanizar",
    "germanizaras": "germanizar",
    "germanizare": "germanizar",
    "germanizareis": "germanizar",
    "germanizaremos": "germanizar",
    "germanizaren": "germanizar",
    "germanizares": "germanizar",
    "germanizaron": "germanizar",
    "germanizará": "germanizar",
    "germanizarán": "germanizar",
    "germanizarás": "germanizar",
    "germanizaré": "germanizar",
    "germanizaréis": "germanizar",
    "germanizaría": "germanizar",
    "germanizaríais": "germanizar",
    "germanizaríamos": "germanizar",
    "germanizarían": "germanizar",
    "germanizarías": "germanizar",
    "germanizas": "germanizar",
    "germanizase": "germanizar",
    "germanizaseis": "germanizar",
    "germanizasen": "germanizar",
    "germanizases": "germanizar",
    "germanizaste": "germanizar",
    "germanizasteis": "germanizar",
    "germanizo": "germanizar",
    "germanizábamos": "germanizar",
    "germanizáis": "germanizar",
    "germanizáramos": "germanizar",
    "germanizáremos": "germanizar",
    "germanizásemos": "germanizar",
    "germanizó": "germanizar",
    "germanos": "germano",
    "germanías": "germanía",
    "germanófila": "germanófilo",
    "germanófilas": "germanófilo",
    "germanófilos": "germanófilo",
    "germanófona": "germanófono",
    "germanófonas": "germanófono",
    "germanófonos": "germanófono",
    "germectomías": "germectomía",
    "germicidas": "germicida",
    "germina": "germinar",
    "germinaba": "germinar",
    "germinabais": "germinar",
    "germinaban": "germinar",
    "germinabas": "germinar",
    "germinaciones": "germinación",
    "germinad": "germinar",
    "germinada": "germinar",
    "germinadas": "germinar",
    "germinado": "germinar",
    "germinadora": "germinador",
    "germinadoras": "germinador",
    "germinadores": "germinador",
    "germinados": "germinar",
    "germinales": "germinal",
    "germinamos": "germinar",
    "germinan": "germinar",
    "germinando": "germinar",
    "germinara": "germinar",
    "germinarais": "germinar",
    "germinaran": "germinar",
    "germinaras": "germinar",
    "germinare": "germinar",
    "germinareis": "germinar",
    "germinaremos": "germinar",
    "germinaren": "germinar",
    "germinares": "germinar",
    "germinaron": "germinar",
    "germinará": "germinar",
    "germinarán": "germinar",
    "germinarás": "germinar",
    "germinaré": "germinar",
    "germinaréis": "germinar",
    "germinaría": "germinar",
    "germinaríais": "germinar",
    "germinaríamos": "germinar",
    "germinarían": "germinar",
    "germinarías": "germinar",
    "germinas": "germinar",
    "germinase": "germinar",
    "germinaseis": "germinar",
    "germinasen": "germinar",
    "germinases": "germinar",
    "germinaste": "germinar",
    "germinasteis": "germinar",
    "germinativa": "germinativo",
    "germinativas": "germinativo",
    "germinativos": "germinativo",
    "germine": "germinar",
    "germinemos": "germinar",
    "germinen": "germinar",
    "germines": "germinar",
    "germino": "germinar",
    "germinábamos": "germinar",
    "germináis": "germinar",
    "germináramos": "germinar",
    "germináremos": "germinar",
    "germinásemos": "germinar",
    "germiné": "germinar",
    "germinéis": "germinar",
    "germinó": "germinar",
    "germánica": "germánico",
    "germánicas": "germánico",
    "germánicos": "germánico",
    "gerodoncias": "gerodoncia",
    "gerodontologías": "gerodontología",
    "gerontocracias": "gerontocracia",
    "gerontologías": "gerontología",
    "gerontológica": "gerontológico",
    "gerontológicas": "gerontológico",
    "gerontológicos": "gerontológico",
    "gerontóloga": "gerontólogo",
    "gerontólogas": "gerontólogo",
    "gerontólogos": "gerontólogo",
    "gerundenses": "gerundense",
    "gerundiadas": "gerundiada",
    "gerundiana": "gerundiano",
    "gerundianas": "gerundiano",
    "gerundianos": "gerundiano",
    "gerundios": "gerundio",
    "gerundivos": "gerundivo",
    "gerusias": "gerusia",
    "ges": "ge",
    "gesta": "gestar",
    "gestaba": "gestar",
    "gestabais": "gestar",
    "gestaban": "gestar",
    "gestabas": "gestar",
    "gestacionales": "gestacional",
    "gestaciones": "gestación",
    "gestad": "gestar",
    "gestada": "gestar",
    "gestadas": "gestar",
    "gestado": "gestar",
    "gestados": "gestar",
    "gestaltismos": "gestaltismo",
    "gestamos": "gestar",
    "gestan": "gestar",
    "gestando": "gestar",
    "gestantes": "gestante",
    "gestara": "gestar",
    "gestarais": "gestar",
    "gestaran": "gestar",
    "gestaras": "gestar",
    "gestare": "gestar",
    "gestareis": "gestar",
    "gestaremos": "gestar",
    "gestaren": "gestar",
    "gestares": "gestar",
    "gestaron": "gestar",
    "gestará": "gestar",
    "gestarán": "gestar",
    "gestarás": "gestar",
    "gestaré": "gestar",
    "gestaréis": "gestar",
    "gestaría": "gestar",
    "gestaríais": "gestar",
    "gestaríamos": "gestar",
    "gestarían": "gestar",
    "gestarías": "gestar",
    "gestas": "gestar",
    "gestase": "gestar",
    "gestaseis": "gestar",
    "gestasen": "gestar",
    "gestases": "gestar",
    "gestaste": "gestar",
    "gestasteis": "gestar",
    "gestatoria": "gestatorio",
    "gestatorias": "gestatorio",
    "gestatorios": "gestatorio",
    "geste": "gestar",
    "gestea": "gestear",
    "gesteaba": "gestear",
    "gesteabais": "gestear",
    "gesteaban": "gestear",
    "gesteabas": "gestear",
    "gestead": "gestear",
    "gesteada": "gestear",
    "gesteadas": "gestear",
    "gesteado": "gestear",
    "gesteados": "gestear",
    "gesteamos": "gestear",
    "gestean": "gestear",
    "gesteando": "gestear",
    "gesteara": "gestear",
    "gestearais": "gestear",
    "gestearan": "gestear",
    "gestearas": "gestear",
    "gesteare": "gestear",
    "gesteareis": "gestear",
    "gestearemos": "gestear",
    "gestearen": "gestear",
    "gesteares": "gestear",
    "gestearon": "gestear",
    "gesteará": "gestear",
    "gestearán": "gestear",
    "gestearás": "gestear",
    "gestearé": "gestear",
    "gestearéis": "gestear",
    "gestearía": "gestear",
    "gestearíais": "gestear",
    "gestearíamos": "gestear",
    "gestearían": "gestear",
    "gestearías": "gestear",
    "gesteas": "gestear",
    "gestease": "gestear",
    "gesteaseis": "gestear",
    "gesteasen": "gestear",
    "gesteases": "gestear",
    "gesteaste": "gestear",
    "gesteasteis": "gestear",
    "gestee": "gestear",
    "gesteemos": "gestear",
    "gesteen": "gestear",
    "gestees": "gestear",
    "gestemos": "gestar",
    "gesten": "gestar",
    "gesteo": "gestear",
    "gestera": "gestero",
    "gesteras": "gestero",
    "gesteros": "gestero",
    "gestes": "gestar",
    "gesteábamos": "gestear",
    "gesteáis": "gestear",
    "gesteáramos": "gestear",
    "gesteáremos": "gestear",
    "gesteásemos": "gestear",
    "gesteé": "gestear",
    "gesteéis": "gestear",
    "gesteó": "gestear",
    "gesticula": "gesticular",
    "gesticulaba": "gesticular",
    "gesticulabais": "gesticular",
    "gesticulaban": "gesticular",
    "gesticulabas": "gesticular",
    "gesticulaciones": "gesticulación",
    "gesticulad": "gesticular",
    "gesticulada": "gesticular",
    "gesticuladas": "gesticular",
    "gesticulado": "gesticular",
    "gesticulados": "gesticular",
    "gesticulamos": "gesticular",
    "gesticulan": "gesticular",
    "gesticulando": "gesticular",
    "gesticulara": "gesticular",
    "gesticularais": "gesticular",
    "gesticularan": "gesticular",
    "gesticularas": "gesticular",
    "gesticulare": "gesticular",
    "gesticulareis": "gesticular",
    "gesticularemos": "gesticular",
    "gesticularen": "gesticular",
    "gesticulares": "gesticular",
    "gesticularon": "gesticular",
    "gesticulará": "gesticular",
    "gesticularán": "gesticular",
    "gesticularás": "gesticular",
    "gesticularé": "gesticular",
    "gesticularéis": "gesticular",
    "gesticularía": "gesticular",
    "gesticularíais": "gesticular",
    "gesticularíamos": "gesticular",
    "gesticularían": "gesticular",
    "gesticularías": "gesticular",
    "gesticulas": "gesticular",
    "gesticulase": "gesticular",
    "gesticulaseis": "gesticular",
    "gesticulasen": "gesticular",
    "gesticulases": "gesticular",
    "gesticulaste": "gesticular",
    "gesticulasteis": "gesticular",
    "gesticule": "gesticular",
    "gesticulemos": "gesticular",
    "gesticulen": "gesticular",
    "gesticules": "gesticular",
    "gesticulo": "gesticular",
    "gesticulábamos": "gesticular",
    "gesticuláis": "gesticular",
    "gesticuláramos": "gesticular",
    "gesticuláremos": "gesticular",
    "gesticulásemos": "gesticular",
    "gesticulé": "gesticular",
    "gesticuléis": "gesticular",
    "gesticuló": "gesticular",
    "gestiona": "gestionar",
    "gestionaba": "gestionar",
    "gestionabais": "gestionar",
    "gestionaban": "gestionar",
    "gestionabas": "gestionar",
    "gestionables": "gestionable",
    "gestionad": "gestionar",
    "gestionada": "gestionar",
    "gestionadas": "gestionar",
    "gestionado": "gestionar",
    "gestionados": "gestionar",
    "gestionamos": "gestionar",
    "gestionan": "gestionar",
    "gestionando": "gestionar",
    "gestionara": "gestionar",
    "gestionarais": "gestionar",
    "gestionaran": "gestionar",
    "gestionaras": "gestionar",
    "gestionare": "gestionar",
    "gestionareis": "gestionar",
    "gestionaremos": "gestionar",
    "gestionaren": "gestionar",
    "gestionares": "gestionar",
    "gestionaron": "gestionar",
    "gestionará": "gestionar",
    "gestionarán": "gestionar",
    "gestionarás": "gestionar",
    "gestionaré": "gestionar",
    "gestionaréis": "gestionar",
    "gestionaría": "gestionar",
    "gestionaríais": "gestionar",
    "gestionaríamos": "gestionar",
    "gestionarían": "gestionar",
    "gestionarías": "gestionar",
    "gestionas": "gestionar",
    "gestionase": "gestionar",
    "gestionaseis": "gestionar",
    "gestionasen": "gestionar",
    "gestionases": "gestionar",
    "gestionaste": "gestionar",
    "gestionasteis": "gestionar",
    "gestione": "gestionar",
    "gestionemos": "gestionar",
    "gestionen": "gestionar",
    "gestiones": "gestionar",
    "gestiono": "gestionar",
    "gestionábamos": "gestionar",
    "gestionáis": "gestionar",
    "gestionáramos": "gestionar",
    "gestionáremos": "gestionar",
    "gestionásemos": "gestionar",
    "gestioné": "gestionar",
    "gestionéis": "gestionar",
    "gestionó": "gestionar",
    "gesto": "gestar",
    "gestora": "gestor",
    "gestoras": "gestor",
    "gestores": "gestor",
    "gestorías": "gestoría",
    "gestos": "gesto",
    "gestualismos": "gestualismo",
    "gestuda": "gestudo",
    "gestudas": "gestudo",
    "gestudos": "gestudo",
    "gestábamos": "gestar",
    "gestáis": "gestar",
    "gestáramos": "gestar",
    "gestáremos": "gestar",
    "gestásemos": "gestar",
    "gesté": "gestar",
    "gestéis": "gestar",
    "gestó": "gestar",
    "getas": "geta",
    "getos": "geto",
    "geyseritas": "geyserita",
    "geófaga": "geófago",
    "geófagas": "geófago",
    "geófagos": "geófago",
    "geófitos": "geófito",
    "geófonos": "geófono",
    "geógrafa": "geógrafo",
    "geógrafas": "geógrafo",
    "geógrafos": "geógrafo",
    "geóloga": "geólogo",
    "geólogas": "geólogo",
    "geólogos": "geólogo",
    "geómetras": "geómetra",
    "geórgica": "geórgico",
    "geórgicas": "geórgico",
    "geórgicos": "geórgico",
    "ghaneana": "ghaneano",
    "ghaneanas": "ghaneano",
    "ghaneanos": "ghaneano",
    "ghanesa": "ghanés",
    "ghanesas": "ghanés",
    "ghaneses": "ghanés",
    "ghettos": "ghetto",
    "giba": "gibar",
    "gibaba": "gibar",
    "gibabais": "gibar",
    "gibaban": "gibar",
    "gibabas": "gibar",
    "gibad": "gibar",
    "gibada": "gibar",
    "gibadas": "gibar",
    "gibado": "gibar",
    "gibados": "gibar",
    "gibamos": "gibar",
    "giban": "gibar",
    "gibando": "gibar",
    "gibara": "gibar",
    "gibarais": "gibar",
    "gibaran": "gibar",
    "gibaras": "gibar",
    "gibare": "gibar",
    "gibareis": "gibar",
    "gibaremos": "gibar",
    "gibaren": "gibar",
    "gibares": "gibar",
    "gibaron": "gibar",
    "gibará": "gibar",
    "gibarán": "gibar",
    "gibarás": "gibar",
    "gibaré": "gibar",
    "gibaréis": "gibar",
    "gibaría": "gibar",
    "gibaríais": "gibar",
    "gibaríamos": "gibar",
    "gibarían": "gibar",
    "gibarías": "gibar",
    "gibas": "gibar",
    "gibase": "gibar",
    "gibaseis": "gibar",
    "gibasen": "gibar",
    "gibases": "gibar",
    "gibaste": "gibar",
    "gibasteis": "gibar",
    "gibe": "gibar",
    "gibelina": "gibelino",
    "gibelinas": "gibelino",
    "gibelinos": "gibelino",
    "gibemos": "gibar",
    "giben": "gibar",
    "giberélica": "giberélico",
    "giberélicas": "giberélico",
    "giberélicos": "giberélico",
    "gibes": "gibar",
    "gibo": "gibar",
    "gibones": "gibón",
    "gibosa": "giboso",
    "gibosas": "giboso",
    "gibosidades": "gibosidad",
    "gibosos": "giboso",
    "gibraltareña": "gibraltareño",
    "gibraltareñas": "gibraltareño",
    "gibraltareños": "gibraltareño",
    "gibábamos": "gibar",
    "gibáis": "gibar",
    "gibáramos": "gibar",
    "gibáremos": "gibar",
    "gibásemos": "gibar",
    "gibé": "gibar",
    "gibéis": "gibar",
    "gibó": "gibar",
    "gibónes": "gibón",
    "gigabits": "gigabit",
    "gigabytes": "gigabyte",
    "gigahertzios": "gigahertzio",
    "giganta": "gigante",
    "gigantas": "gigante",
    "giganteces": "gigantez",
    "gigantes": "gigante",
    "gigantesca": "gigantesco",
    "gigantescas": "gigantesco",
    "gigantescos": "gigantesco",
    "gigantillas": "gigantilla",
    "gigantillos": "gigantillo",
    "gigantismos": "gigantismo",
    "gigantocitos": "gigantocito",
    "gigantomaquias": "gigantomaquia",
    "gigantona": "gigantón",
    "gigantonas": "gigantón",
    "gigantones": "gigantón",
    "gigaoctetos": "gigaocteto",
    "gigas": "giga",
    "gigolós": "gigoló",
    "gigolóses": "gigoló",
    "gigotes": "gigote",
    "gigots": "gigot",
    "gijonenses": "gijonense",
    "gijonesa": "gijonés",
    "gijonesas": "gijonés",
    "gijoneses": "gijonés",
    "gilbertes": "gilbert",
    "gilberts": "gilbert",
    "gildas": "gilda",
    "giles": "gil",
    "gilsonitas": "gilsonita",
    "gilva": "gilvo",
    "gilvas": "gilvo",
    "gilvos": "gilvo",
    "gilíes": "gilí",
    "gilís": "gilí",
    "gima": "gemir",
    "gimamos": "gemir",
    "giman": "gemir",
    "gimas": "gemir",
    "gime": "gemir",
    "gimen": "gemir",
    "gimes": "gemir",
    "gimiendo": "gemir",
    "gimiera": "gemir",
    "gimierais": "gemir",
    "gimieran": "gemir",
    "gimieras": "gemir",
    "gimiere": "gemir",
    "gimiereis": "gemir",
    "gimieren": "gemir",
    "gimieres": "gemir",
    "gimieron": "gemir",
    "gimiese": "gemir",
    "gimieseis": "gemir",
    "gimiesen": "gemir",
    "gimieses": "gemir",
    "gimiéramos": "gemir",
    "gimiéremos": "gemir",
    "gimiésemos": "gemir",
    "gimió": "gemir",
    "gimnasias": "gimnasia",
    "gimnasios": "gimnasio",
    "gimnastas": "gimnasta",
    "gimnobelídeos": "gimnobelídeo",
    "gimnosofistas": "gimnosofista",
    "gimnospermas": "gimnosperma",
    "gimnotos": "gimnoto",
    "gimnástica": "gimnástico",
    "gimnásticas": "gimnástico",
    "gimnásticos": "gimnástico",
    "gimo": "gemir",
    "gimotea": "gimotear",
    "gimoteaba": "gimotear",
    "gimoteabais": "gimotear",
    "gimoteaban": "gimotear",
    "gimoteabas": "gimotear",
    "gimotead": "gimotear",
    "gimoteada": "gimotear",
    "gimoteadas": "gimotear",
    "gimoteado": "gimotear",
    "gimoteadora": "gimoteador",
    "gimoteadoras": "gimoteador",
    "gimoteadores": "gimoteador",
    "gimoteados": "gimotear",
    "gimoteamos": "gimotear",
    "gimotean": "gimotear",
    "gimoteando": "gimotear",
    "gimoteara": "gimotear",
    "gimotearais": "gimotear",
    "gimotearan": "gimotear",
    "gimotearas": "gimotear",
    "gimoteare": "gimotear",
    "gimoteareis": "gimotear",
    "gimotearemos": "gimotear",
    "gimotearen": "gimotear",
    "gimoteares": "gimotear",
    "gimotearon": "gimotear",
    "gimoteará": "gimotear",
    "gimotearán": "gimotear",
    "gimotearás": "gimotear",
    "gimotearé": "gimotear",
    "gimotearéis": "gimotear",
    "gimotearía": "gimotear",
    "gimotearíais": "gimotear",
    "gimotearíamos": "gimotear",
    "gimotearían": "gimotear",
    "gimotearías": "gimotear",
    "gimoteas": "gimotear",
    "gimotease": "gimotear",
    "gimoteaseis": "gimotear",
    "gimoteasen": "gimotear",
    "gimoteases": "gimotear",
    "gimoteaste": "gimotear",
    "gimoteasteis": "gimotear",
    "gimotee": "gimotear",
    "gimoteemos": "gimotear",
    "gimoteen": "gimotear",
    "gimotees": "gimotear",
    "gimoteo": "gimotear",
    "gimoteos": "gimoteo",
    "gimoteábamos": "gimotear",
    "gimoteáis": "gimotear",
    "gimoteáramos": "gimotear",
    "gimoteáremos": "gimotear",
    "gimoteásemos": "gimotear",
    "gimoteé": "gimotear",
    "gimoteéis": "gimotear",
    "gimoteó": "gimotear",
    "gimáis": "gemir",
    "ginandra": "ginandro",
    "ginandras": "ginandro",
    "ginandros": "ginandro",
    "gincanas": "gincana",
    "ginebradas": "ginebrada",
    "ginebras": "ginebra",
    "ginebresa": "ginebrés",
    "ginebresas": "ginebrés",
    "ginebreses": "ginebrés",
    "ginebrina": "ginebrino",
    "ginebrinas": "ginebrino",
    "ginebrinos": "ginebrino",
    "gineceos": "gineceo",
    "ginecocracias": "ginecocracia",
    "ginecologías": "ginecología",
    "ginecológica": "ginecológico",
    "ginecológicas": "ginecológico",
    "ginecológicos": "ginecológico",
    "ginecomastias": "ginecomastia",
    "ginecomórfica": "ginecomórfico",
    "ginecomórficas": "ginecomórfico",
    "ginecomórficos": "ginecomórfico",
    "ginecóloga": "ginecólogo",
    "ginecólogas": "ginecólogo",
    "ginecólogos": "ginecólogo",
    "ginestas": "ginesta",
    "ginfizces": "ginfizz",
    "ginger-ales": "ginger-ale",
    "gingivales": "gingival",
    "gingivalgias": "gingivalgia",
    "gingivectomías": "gingivectomía",
    "gingivoaxiales": "gingivoaxial",
    "gingivobucoaxiales": "gingivobucoaxial",
    "gingivolabiales": "gingivolabial",
    "gingivolinguoaxiales": "gingivolinguoaxial",
    "gingivoplastias": "gingivoplastia",
    "gingivorragias": "gingivorragia",
    "gingivotomías": "gingivotomía",
    "ginkgoales": "ginkgoal",
    "ginkgos": "ginkgo",
    "ginocardatos": "ginocardato",
    "giorgis": "giorgi",
    "gipsitas": "gipsita",
    "gipsófila": "gipsófilo",
    "gipsófilas": "gipsófilo",
    "gipsófilos": "gipsófilo",
    "gira": "girar",
    "giraba": "girar",
    "girabais": "girar",
    "giraban": "girar",
    "girabas": "girar",
    "girables": "girable",
    "giraciones": "giración",
    "girad": "girar",
    "girada": "girar",
    "giradas": "girar",
    "girado": "girar",
    "giradora": "girador",
    "giradoras": "girador",
    "giradores": "girador",
    "girados": "girar",
    "giraldas": "giralda",
    "giraldetes": "giraldete",
    "giramos": "girar",
    "giran": "girar",
    "girando": "girar",
    "girara": "girar",
    "girarais": "girar",
    "giraran": "girar",
    "giraras": "girar",
    "girare": "girar",
    "girareis": "girar",
    "giraremos": "girar",
    "giraren": "girar",
    "girares": "girar",
    "giraron": "girar",
    "girará": "girar",
    "girarán": "girar",
    "girarás": "girar",
    "giraré": "girar",
    "giraréis": "girar",
    "giraría": "girar",
    "giraríais": "girar",
    "giraríamos": "girar",
    "girarían": "girar",
    "girarías": "girar",
    "giras": "girar",
    "girase": "girar",
    "giraseis": "girar",
    "girasen": "girar",
    "girases": "girar",
    "girasoles": "girasol",
    "giraste": "girar",
    "girasteis": "girar",
    "giratoria": "giratorio",
    "giratorias": "giratorio",
    "giratorios": "giratorio",
    "gire": "girar",
    "girellas": "girella",
    "giremos": "girar",
    "giren": "girar",
    "gires": "girar",
    "girifaltes": "girifalte",
    "giro": "girar",
    "girocompases": "girocompás",
    "girofrecuencias": "girofrecuencia",
    "giroides": "giroide",
    "girolas": "girola",
    "giromagnética": "giromagnético",
    "giromagnéticas": "giromagnético",
    "giromagnéticos": "giromagnético",
    "girondina": "girondino",
    "girondinas": "girondino",
    "girondinos": "girondino",
    "girones": "girón",
    "gironesa": "gironés",
    "gironesas": "gironés",
    "gironeses": "gironés",
    "giros": "giro",
    "giroscopios": "giroscopio",
    "giroscópica": "giroscópico",
    "giroscópicas": "giroscópico",
    "giroscópicos": "giroscópico",
    "girostática": "girostático",
    "girostáticas": "girostático",
    "girostáticos": "girostático",
    "girotrones": "girotrón",
    "girábamos": "girar",
    "giráis": "girar",
    "girándulas": "girándula",
    "giráramos": "girar",
    "giráremos": "girar",
    "girásemos": "girar",
    "giré": "girar",
    "giréis": "girar",
    "giró": "girar",
    "girómetros": "girómetro",
    "giróscopos": "giróscopo",
    "giróstatos": "giróstato",
    "giróvaga": "giróvago",
    "giróvagas": "giróvago",
    "giróvagos": "giróvago",
    "gises": "gis",
    "gistes": "giste",
    "gitana": "gitano",
    "gitanadas": "gitanada",
    "gitanas": "gitano",
    "gitanea": "gitanear",
    "gitaneaba": "gitanear",
    "gitaneabais": "gitanear",
    "gitaneaban": "gitanear",
    "gitaneabas": "gitanear",
    "gitanead": "gitanear",
    "gitaneada": "gitanear",
    "gitaneadas": "gitanear",
    "gitaneado": "gitanear",
    "gitaneados": "gitanear",
    "gitaneamos": "gitanear",
    "gitanean": "gitanear",
    "gitaneando": "gitanear",
    "gitaneara": "gitanear",
    "gitanearais": "gitanear",
    "gitanearan": "gitanear",
    "gitanearas": "gitanear",
    "gitaneare": "gitanear",
    "gitaneareis": "gitanear",
    "gitanearemos": "gitanear",
    "gitanearen": "gitanear",
    "gitaneares": "gitanear",
    "gitanearon": "gitanear",
    "gitaneará": "gitanear",
    "gitanearán": "gitanear",
    "gitanearás": "gitanear",
    "gitanearé": "gitanear",
    "gitanearéis": "gitanear",
    "gitanearía": "gitanear",
    "gitanearíais": "gitanear",
    "gitanearíamos": "gitanear",
    "gitanearían": "gitanear",
    "gitanearías": "gitanear",
    "gitaneas": "gitanear",
    "gitanease": "gitanear",
    "gitaneaseis": "gitanear",
    "gitaneasen": "gitanear",
    "gitaneases": "gitanear",
    "gitaneaste": "gitanear",
    "gitaneasteis": "gitanear",
    "gitanee": "gitanear",
    "gitaneemos": "gitanear",
    "gitaneen": "gitanear",
    "gitanees": "gitanear",
    "gitaneo": "gitanear",
    "gitanerías": "gitanería",
    "gitaneábamos": "gitanear",
    "gitaneáis": "gitanear",
    "gitaneáramos": "gitanear",
    "gitaneáremos": "gitanear",
    "gitaneásemos": "gitanear",
    "gitaneé": "gitanear",
    "gitaneéis": "gitanear",
    "gitaneó": "gitanear",
    "gitanismos": "gitanismo",
    "gitanos": "gitano",
    "gitoxinas": "gitoxina",
    "glabelas": "glabela",
    "glabra": "glabro",
    "glabras": "glabro",
    "glabros": "glabro",
    "glaciaciones": "glaciación",
    "glaciales": "glacial",
    "glaciares": "glaciar",
    "glaciarismos": "glaciarismo",
    "glacioeustatismos": "glacioeustatismo",
    "glaciologías": "glaciología",
    "glaciárica": "glaciárico",
    "glaciáricas": "glaciárico",
    "glaciáricos": "glaciárico",
    "glacióloga": "glaciólogo",
    "glaciólogas": "glaciólogo",
    "glaciólogos": "glaciólogo",
    "gladiadores": "gladiador",
    "gladiatores": "gladiator",
    "gladiatoria": "gladiatorio",
    "gladiatorias": "gladiatorio",
    "gladiatorios": "gladiatorio",
    "gladiolos": "gladiolo",
    "gladíolos": "gladíolo",
    "glamourosa": "glamouroso",
    "glamourosas": "glamouroso",
    "glamourosos": "glamouroso",
    "glandes": "glande",
    "glandulares": "glandular",
    "glasea": "glasear",
    "glaseaba": "glasear",
    "glaseabais": "glasear",
    "glaseaban": "glasear",
    "glaseabas": "glasear",
    "glasead": "glasear",
    "glaseada": "glasear",
    "glaseadas": "glasear",
    "glaseado": "glasear",
    "glaseados": "glasear",
    "glaseamos": "glasear",
    "glasean": "glasear",
    "glaseando": "glasear",
    "glaseara": "glasear",
    "glasearais": "glasear",
    "glasearan": "glasear",
    "glasearas": "glasear",
    "glaseare": "glasear",
    "glaseareis": "glasear",
    "glasearemos": "glasear",
    "glasearen": "glasear",
    "glaseares": "glasear",
    "glasearon": "glasear",
    "glaseará": "glasear",
    "glasearán": "glasear",
    "glasearás": "glasear",
    "glasearé": "glasear",
    "glasearéis": "glasear",
    "glasearía": "glasear",
    "glasearíais": "glasear",
    "glasearíamos": "glasear",
    "glasearían": "glasear",
    "glasearías": "glasear",
    "glaseas": "glasear",
    "glasease": "glasear",
    "glaseaseis": "glasear",
    "glaseasen": "glasear",
    "glaseases": "glasear",
    "glaseaste": "glasear",
    "glaseasteis": "glasear",
    "glasee": "glasear",
    "glaseemos": "glasear",
    "glaseen": "glasear",
    "glasees": "glasear",
    "glaseo": "glasear",
    "glaseritas": "glaserita",
    "glaseábamos": "glasear",
    "glaseáis": "glasear",
    "glaseáramos": "glasear",
    "glaseáremos": "glasear",
    "glaseásemos": "glasear",
    "glaseé": "glasear",
    "glaseéis": "glasear",
    "glaseó": "glasear",
    "glastos": "glasto",
    "glasés": "glasé",
    "glauberitas": "glauberita",
    "glauca": "glauco",
    "glaucas": "glauco",
    "glaucios": "glaucio",
    "glaucodotas": "glaucodota",
    "glaucofanas": "glaucofana",
    "glaucomas": "glaucoma",
    "glaucomatosa": "glaucomatoso",
    "glaucomatosas": "glaucomatoso",
    "glaucomatosos": "glaucomatoso",
    "glauconitas": "glauconita",
    "glaucos": "glauco",
    "glebas": "gleba",
    "gleica": "gleico",
    "gleicas": "gleico",
    "gleicos": "gleico",
    "glenohumerales": "glenohumeral",
    "glenoidea": "glenoideo",
    "glenoideas": "glenoideo",
    "glenoideos": "glenoideo",
    "glenoides": "glenoide",
    "gleras": "glera",
    "gleys": "gley",
    "gliales": "glial",
    "glicemias": "glicemia",
    "gliceraldehídos": "gliceraldehído",
    "glicerinas": "glicerina",
    "glicerofosfatos": "glicerofosfato",
    "glicerofosfórica": "glicerofosfórico",
    "glicerofosfóricas": "glicerofosfórico",
    "glicerofosfóricos": "glicerofosfórico",
    "gliceroles": "glicerol",
    "glicidoles": "glicidol",
    "glicinaciones": "glicinación",
    "glicinas": "glicina",
    "glicinatos": "glicinato",
    "glicirrinas": "glicirrina",
    "glicocolas": "glicocola",
    "glicocálixes": "glicocálix",
    "glicoforinas": "glicoforina",
    "glicolatos": "glicolato",
    "glicoles": "glicol",
    "glicolonitrilos": "glicolonitrilo",
    "glicolípidos": "glicolípido",
    "gliconios": "gliconio",
    "glicoproteínas": "glicoproteína",
    "glicérica": "glicérico",
    "glicéricas": "glicérico",
    "glicéricos": "glicérico",
    "glicéridos": "glicérido",
    "glicógenos": "glicógeno",
    "glicólica": "glicólico",
    "glicólicas": "glicólico",
    "glicólicos": "glicólico",
    "glicósidos": "glicósido",
    "glifos": "glifo",
    "gliodinas": "gliodina",
    "gliofilamentos": "gliofilamento",
    "gliomas": "glioma",
    "glioxisomas": "glioxisoma",
    "gliptales": "gliptal",
    "gliptografías": "gliptografía",
    "gliptotecas": "gliptoteca",
    "glissandos": "glissando",
    "globales": "global",
    "globalice": "globalizar",
    "globalicemos": "globalizar",
    "globalicen": "globalizar",
    "globalices": "globalizar",
    "globalicé": "globalizar",
    "globalicéis": "globalizar",
    "globalidades": "globalidad",
    "globalismos": "globalismo",
    "globaliza": "globalizar",
    "globalizaba": "globalizar",
    "globalizabais": "globalizar",
    "globalizaban": "globalizar",
    "globalizabas": "globalizar",
    "globalizaciones": "globalización",
    "globalizad": "globalizar",
    "globalizada": "globalizar",
    "globalizadas": "globalizar",
    "globalizado": "globalizar",
    "globalizadora": "globalizador",
    "globalizadoras": "globalizador",
    "globalizadores": "globalizador",
    "globalizados": "globalizar",
    "globalizamos": "globalizar",
    "globalizan": "globalizar",
    "globalizando": "globalizar",
    "globalizantes": "globalizante",
    "globalizara": "globalizar",
    "globalizarais": "globalizar",
    "globalizaran": "globalizar",
    "globalizaras": "globalizar",
    "globalizare": "globalizar",
    "globalizareis": "globalizar",
    "globalizaremos": "globalizar",
    "globalizaren": "globalizar",
    "globalizares": "globalizar",
    "globalizaron": "globalizar",
    "globalizará": "globalizar",
    "globalizarán": "globalizar",
    "globalizarás": "globalizar",
    "globalizaré": "globalizar",
    "globalizaréis": "globalizar",
    "globalizaría": "globalizar",
    "globalizaríais": "globalizar",
    "globalizaríamos": "globalizar",
    "globalizarían": "globalizar",
    "globalizarías": "globalizar",
    "globalizas": "globalizar",
    "globalizase": "globalizar",
    "globalizaseis": "globalizar",
    "globalizasen": "globalizar",
    "globalizases": "globalizar",
    "globalizaste": "globalizar",
    "globalizasteis": "globalizar",
    "globalizo": "globalizar",
    "globalizábamos": "globalizar",
    "globalizáis": "globalizar",
    "globalizáramos": "globalizar",
    "globalizáremos": "globalizar",
    "globalizásemos": "globalizar",
    "globalizó": "globalizar",
    "globicéfalos": "globicéfalo",
    "globinas": "globina",
    "globitos": "globito",
    "globoides": "globoide",
    "globos": "globo",
    "globosa": "globoso",
    "globosas": "globoso",
    "globosidades": "globosidad",
    "globosos": "globoso",
    "globotriaocilceramidas": "globotriaocilceramida",
    "globulares": "globular",
    "globulariáceas": "globulariácea",
    "globulinas": "globulina",
    "globulinemias": "globulinemia",
    "globulosa": "globuloso",
    "globulosas": "globuloso",
    "globulosos": "globuloso",
    "globulímetros": "globulímetro",
    "glomerulares": "glomerular",
    "glomiformes": "glomiforme",
    "glomos": "glomo",
    "glomérulos": "glomérulo",
    "gloriaba": "gloriar",
    "gloriabais": "gloriar",
    "gloriaban": "gloriar",
    "gloriabas": "gloriar",
    "gloriad": "gloriar",
    "gloriada": "gloriar",
    "gloriadas": "gloriar",
    "gloriado": "gloriar",
    "gloriados": "gloriar",
    "gloriamos": "gloriar",
    "gloriando": "gloriar",
    "gloriara": "gloriar",
    "gloriarais": "gloriar",
    "gloriaran": "gloriar",
    "gloriaras": "gloriar",
    "gloriare": "gloriar",
    "gloriareis": "gloriar",
    "gloriaremos": "gloriar",
    "gloriaren": "gloriar",
    "gloriares": "gloriar",
    "gloriaron": "gloriar",
    "gloriará": "gloriar",
    "gloriarán": "gloriar",
    "gloriarás": "gloriar",
    "gloriaré": "gloriar",
    "gloriaréis": "gloriar",
    "gloriaría": "gloriar",
    "gloriaríais": "gloriar",
    "gloriaríamos": "gloriar",
    "gloriarían": "gloriar",
    "gloriarías": "gloriar",
    "glorias": "gloria",
    "gloriase": "gloriar",
    "gloriaseis": "gloriar",
    "gloriasen": "gloriar",
    "gloriases": "gloriar",
    "gloriaste": "gloriar",
    "gloriasteis": "gloriar",
    "gloriemos": "gloriar",
    "glorietas": "glorieta",
    "glorifica": "glorificar",
    "glorificaba": "glorificar",
    "glorificabais": "glorificar",
    "glorificaban": "glorificar",
    "glorificabas": "glorificar",
    "glorificables": "glorificable",
    "glorificaciones": "glorificación",
    "glorificad": "glorificar",
    "glorificada": "glorificar",
    "glorificadas": "glorificar",
    "glorificado": "glorificar",
    "glorificadora": "glorificador",
    "glorificadoras": "glorificador",
    "glorificadores": "glorificador",
    "glorificados": "glorificar",
    "glorificamos": "glorificar",
    "glorifican": "glorificar",
    "glorificando": "glorificar",
    "glorificara": "glorificar",
    "glorificarais": "glorificar",
    "glorificaran": "glorificar",
    "glorificaras": "glorificar",
    "glorificare": "glorificar",
    "glorificareis": "glorificar",
    "glorificaremos": "glorificar",
    "glorificaren": "glorificar",
    "glorificares": "glorificar",
    "glorificaron": "glorificar",
    "glorificará": "glorificar",
    "glorificarán": "glorificar",
    "glorificarás": "glorificar",
    "glorificaré": "glorificar",
    "glorificaréis": "glorificar",
    "glorificaría": "glorificar",
    "glorificaríais": "glorificar",
    "glorificaríamos": "glorificar",
    "glorificarían": "glorificar",
    "glorificarías": "glorificar",
    "glorificas": "glorificar",
    "glorificase": "glorificar",
    "glorificaseis": "glorificar",
    "glorificasen": "glorificar",
    "glorificases": "glorificar",
    "glorificaste": "glorificar",
    "glorificasteis": "glorificar",
    "glorifico": "glorificar",
    "glorificábamos": "glorificar",
    "glorificáis": "glorificar",
    "glorificáramos": "glorificar",
    "glorificáremos": "glorificar",
    "glorificásemos": "glorificar",
    "glorificó": "glorificar",
    "glorifique": "glorificar",
    "glorifiquemos": "glorificar",
    "glorifiquen": "glorificar",
    "glorifiques": "glorificar",
    "glorifiqué": "glorificar",
    "glorifiquéis": "glorificar",
    "gloriosa": "glorioso",
    "gloriosas": "glorioso",
    "gloriosos": "glorioso",
    "gloriábamos": "gloriar",
    "gloriáis": "gloriar",
    "gloriáramos": "gloriar",
    "gloriáremos": "gloriar",
    "gloriásemos": "gloriar",
    "glorié": "gloriar",
    "gloriéis": "gloriar",
    "glorió": "gloriar",
    "gloría": "gloriar",
    "glorían": "gloriar",
    "glorías": "gloriar",
    "gloríe": "gloriar",
    "gloríen": "gloriar",
    "gloríes": "gloriar",
    "glorío": "gloriar",
    "glosa": "glosar",
    "glosaba": "glosar",
    "glosabais": "glosar",
    "glosaban": "glosar",
    "glosabas": "glosar",
    "glosad": "glosar",
    "glosada": "glosar",
    "glosadas": "glosar",
    "glosado": "glosar",
    "glosadora": "glosador",
    "glosadoras": "glosador",
    "glosadores": "glosador",
    "glosados": "glosar",
    "glosalgias": "glosalgia",
    "glosamos": "glosar",
    "glosan": "glosar",
    "glosando": "glosar",
    "glosara": "glosar",
    "glosarais": "glosar",
    "glosaran": "glosar",
    "glosaras": "glosar",
    "glosare": "glosar",
    "glosareis": "glosar",
    "glosaremos": "glosar",
    "glosaren": "glosar",
    "glosares": "glosar",
    "glosarios": "glosario",
    "glosaron": "glosar",
    "glosará": "glosar",
    "glosarán": "glosar",
    "glosarás": "glosar",
    "glosaré": "glosar",
    "glosaréis": "glosar",
    "glosaría": "glosar",
    "glosaríais": "glosar",
    "glosaríamos": "glosar",
    "glosarían": "glosar",
    "glosarías": "glosar",
    "glosas": "glosar",
    "glosase": "glosar",
    "glosaseis": "glosar",
    "glosasen": "glosar",
    "glosases": "glosar",
    "glosaste": "glosar",
    "glosasteis": "glosar",
    "glose": "glosar",
    "glosectomías": "glosectomía",
    "glosemas": "glosema",
    "glosemos": "glosar",
    "glosemáticas": "glosemática",
    "glosen": "glosar",
    "gloses": "glose",
    "glosillas": "glosilla",
    "gloso": "glosar",
    "glosobórica": "glosobórico",
    "glosobóricas": "glosobórico",
    "glosobóricos": "glosobórico",
    "glosoceles": "glosocele",
    "glosocinestesias": "glosocinestesia",
    "glosocinestésica": "glosocinestésico",
    "glosocinestésicas": "glosocinestésico",
    "glosocinestésicos": "glosocinestésico",
    "glosodinamómetros": "glosodinamómetro",
    "glosodinias": "glosodinia",
    "glosodonta": "glosodonto",
    "glosodontas": "glosodonto",
    "glosodontos": "glosodonto",
    "glosoepiglótica": "glosoepiglótico",
    "glosoepiglóticas": "glosoepiglótico",
    "glosoepiglóticos": "glosoepiglótico",
    "glosofaríngea": "glosofaríngeo",
    "glosofaríngeas": "glosofaríngeo",
    "glosofaríngeos": "glosofaríngeo",
    "glosofitias": "glosofitia",
    "glosolalias": "glosolalia",
    "glosopalatina": "glosopalatino",
    "glosopalatinas": "glosopalatino",
    "glosopalatinos": "glosopalatino",
    "glosopatías": "glosopatía",
    "glosopedas": "glosopeda",
    "glosoplastias": "glosoplastia",
    "glosoplejías": "glosoplejía",
    "glosotomías": "glosotomía",
    "glosábamos": "glosar",
    "glosácuica": "glosácuico",
    "glosácuicas": "glosácuico",
    "glosácuicos": "glosácuico",
    "glosáis": "glosar",
    "glosántraxes": "glosántrax",
    "glosáramos": "glosar",
    "glosáremos": "glosar",
    "glosásemos": "glosar",
    "glosé": "glosar",
    "gloséis": "glosar",
    "glosó": "glosar",
    "glosógrafos": "glosógrafo",
    "glotales": "glotal",
    "glotalizaciones": "glotalización",
    "glotocronologías": "glotocronología",
    "glotologías": "glotología",
    "glotona": "glotón",
    "glotonas": "glotón",
    "glotonea": "glotonear",
    "glotoneaba": "glotonear",
    "glotoneabais": "glotonear",
    "glotoneaban": "glotonear",
    "glotoneabas": "glotonear",
    "glotonead": "glotonear",
    "glotoneada": "glotonear",
    "glotoneadas": "glotonear",
    "glotoneado": "glotonear",
    "glotoneados": "glotonear",
    "glotoneamos": "glotonear",
    "glotonean": "glotonear",
    "glotoneando": "glotonear",
    "glotoneara": "glotonear",
    "glotonearais": "glotonear",
    "glotonearan": "glotonear",
    "glotonearas": "glotonear",
    "glotoneare": "glotonear",
    "glotoneareis": "glotonear",
    "glotonearemos": "glotonear",
    "glotonearen": "glotonear",
    "glotoneares": "glotonear",
    "glotonearon": "glotonear",
    "glotoneará": "glotonear",
    "glotonearán": "glotonear",
    "glotonearás": "glotonear",
    "glotonearé": "glotonear",
    "glotonearéis": "glotonear",
    "glotonearía": "glotonear",
    "glotonearíais": "glotonear",
    "glotonearíamos": "glotonear",
    "glotonearían": "glotonear",
    "glotonearías": "glotonear",
    "glotoneas": "glotonear",
    "glotonease": "glotonear",
    "glotoneaseis": "glotonear",
    "glotoneasen": "glotonear",
    "glotoneases": "glotonear",
    "glotoneaste": "glotonear",
    "glotoneasteis": "glotonear",
    "glotonee": "glotonear",
    "glotoneemos": "glotonear",
    "glotoneen": "glotonear",
    "glotonees": "glotonear",
    "glotoneo": "glotonear",
    "glotonerías": "glotonería",
    "glotones": "glotón",
    "glotoneábamos": "glotonear",
    "glotoneáis": "glotonear",
    "glotoneáramos": "glotonear",
    "glotoneáremos": "glotonear",
    "glotoneásemos": "glotonear",
    "glotoneé": "glotonear",
    "glotoneéis": "glotonear",
    "glotoneó": "glotonear",
    "glotóloga": "glotólogo",
    "glotólogas": "glotólogo",
    "glotólogos": "glotólogo",
    "gloxíneas": "gloxínea",
    "glucagones": "glucagón",
    "glucemias": "glucemia",
    "glucocorticoides": "glucocorticoide",
    "glucocálices": "glucocáliz",
    "glucoforinas": "glucoforina",
    "glucogénica": "glucogénico",
    "glucogénicas": "glucogénico",
    "glucogénicos": "glucogénico",
    "glucoheptinatos": "glucoheptinato",
    "glucolípidos": "glucolípido",
    "gluconatos": "gluconato",
    "glucoproteínas": "glucoproteína",
    "glucorregulaciones": "glucorregulación",
    "glucosada": "glucosado",
    "glucosadas": "glucosado",
    "glucosados": "glucosado",
    "glucosaminas": "glucosamina",
    "glucosaminoglucanos": "glucosaminoglucano",
    "glucosas": "glucosa",
    "glucosialorreas": "glucosialorrea",
    "glucosomas": "glucosoma",
    "glucosurias": "glucosuria",
    "glucosúrica": "glucosúrico",
    "glucosúricas": "glucosúrico",
    "glucosúricos": "glucosúrico",
    "glucuronolactonas": "glucuronolactona",
    "glucémica": "glucémico",
    "glucémicas": "glucémico",
    "glucémicos": "glucémico",
    "glucógenos": "glucógeno",
    "glucómetros": "glucómetro",
    "glucónica": "glucónico",
    "glucónicas": "glucónico",
    "glucónicos": "glucónico",
    "glucósidos": "glucósido",
    "gluglutea": "gluglutear",
    "glugluteaba": "gluglutear",
    "glugluteabais": "gluglutear",
    "glugluteaban": "gluglutear",
    "glugluteabas": "gluglutear",
    "gluglutead": "gluglutear",
    "glugluteada": "gluglutear",
    "glugluteadas": "gluglutear",
    "glugluteado": "gluglutear",
    "glugluteados": "gluglutear",
    "glugluteamos": "gluglutear",
    "gluglutean": "gluglutear",
    "glugluteando": "gluglutear",
    "glugluteara": "gluglutear",
    "gluglutearais": "gluglutear",
    "gluglutearan": "gluglutear",
    "gluglutearas": "gluglutear",
    "glugluteare": "gluglutear",
    "glugluteareis": "gluglutear",
    "gluglutearemos": "gluglutear",
    "gluglutearen": "gluglutear",
    "glugluteares": "gluglutear",
    "gluglutearon": "gluglutear",
    "glugluteará": "gluglutear",
    "gluglutearán": "gluglutear",
    "gluglutearás": "gluglutear",
    "gluglutearé": "gluglutear",
    "gluglutearéis": "gluglutear",
    "gluglutearía": "gluglutear",
    "gluglutearíais": "gluglutear",
    "gluglutearíamos": "gluglutear",
    "gluglutearían": "gluglutear",
    "gluglutearías": "gluglutear",
    "glugluteas": "gluglutear",
    "gluglutease": "gluglutear",
    "glugluteaseis": "gluglutear",
    "glugluteasen": "gluglutear",
    "glugluteases": "gluglutear",
    "glugluteaste": "gluglutear",
    "glugluteasteis": "gluglutear",
    "gluglutee": "gluglutear",
    "glugluteemos": "gluglutear",
    "glugluteen": "gluglutear",
    "gluglutees": "gluglutear",
    "glugluteo": "gluglutear",
    "glugluteábamos": "gluglutear",
    "glugluteáis": "gluglutear",
    "glugluteáramos": "gluglutear",
    "glugluteáremos": "gluglutear",
    "glugluteásemos": "gluglutear",
    "glugluteé": "gluglutear",
    "glugluteéis": "gluglutear",
    "glugluteó": "gluglutear",
    "glumas": "gluma",
    "glumelas": "glumela",
    "glumifloras": "glumiflora",
    "glumélulas": "glumélula",
    "glutaciones": "glutación",
    "glutamatos": "glutamato",
    "glutaminas": "glutamina",
    "glutaraldehídos": "glutaraldehído",
    "glutaronitrilos": "glutaronitrilo",
    "glutinosa": "glutinoso",
    "glutinosas": "glutinoso",
    "glutinosidades": "glutinosidad",
    "glutinosos": "glutinoso",
    "glutámica": "glutámico",
    "glutámicas": "glutámico",
    "glutámicos": "glutámico",
    "glutárica": "glutárico",
    "glutáricas": "glutárico",
    "glutáricos": "glutárico",
    "glándulas": "glándula",
    "glías": "glía",
    "glípticas": "glíptica",
    "glíridos": "glírido",
    "glóbulos": "glóbulo",
    "glósica": "glósico",
    "glósicas": "glósico",
    "glósicos": "glósico",
    "glótica": "glótico",
    "glóticas": "glótico",
    "glóticos": "glótico",
    "glúcidos": "glúcido",
    "glútea": "glúteo",
    "glúteas": "glúteo",
    "glútenes": "gluten",
    "glúteos": "glúteo",
    "gnatalgias": "gnatalgia",
    "gnationes": "gnatión",
    "gnatodinamias": "gnatodinamia",
    "gnatodinamómetros": "gnatodinamómetro",
    "gnatografías": "gnatografía",
    "gnatologías": "gnatología",
    "gnatoplastias": "gnatoplastia",
    "gnatostáticas": "gnatostática",
    "gnetales": "gnetal",
    "gnetáceas": "gnetácea",
    "gnomos": "gnomo",
    "gnomónica": "gnomónico",
    "gnomónicas": "gnomónico",
    "gnomónicos": "gnomónico",
    "gnoseologías": "gnoseología",
    "gnoseológica": "gnoseológico",
    "gnoseológicas": "gnoseológico",
    "gnoseológicos": "gnoseológico",
    "gnosticismos": "gnosticismo",
    "gnus": "gnu",
    "gnática": "gnático",
    "gnáticas": "gnático",
    "gnáticos": "gnático",
    "gnéisica": "gnéisico",
    "gnéisicas": "gnéisico",
    "gnéisicos": "gnéisico",
    "gnómica": "gnómico",
    "gnómicas": "gnómico",
    "gnómicos": "gnómico",
    "gnómones": "gnomon",
    "gnóstica": "gnóstico",
    "gnósticas": "gnóstico",
    "gnósticos": "gnóstico",
    "gobelinos": "gobelino",
    "gobernaba": "gobernar",
    "gobernabais": "gobernar",
    "gobernaban": "gobernar",
    "gobernabas": "gobernar",
    "gobernabilidades": "gobernabilidad",
    "gobernables": "gobernable",
    "gobernaciones": "gobernación",
    "gobernad": "gobernar",
    "gobernada": "gobernar",
    "gobernadas": "gobernar",
    "gobernado": "gobernar",
    "gobernadora": "gobernador",
    "gobernadoras": "gobernador",
    "gobernadores": "gobernador",
    "gobernados": "gobernar",
    "gobernalles": "gobernalle",
    "gobernamos": "gobernar",
    "gobernando": "gobernar",
    "gobernantas": "gobernanta",
    "gobernantes": "gobernante",
    "gobernanzas": "gobernanza",
    "gobernara": "gobernar",
    "gobernarais": "gobernar",
    "gobernaran": "gobernar",
    "gobernaras": "gobernar",
    "gobernare": "gobernar",
    "gobernareis": "gobernar",
    "gobernaremos": "gobernar",
    "gobernaren": "gobernar",
    "gobernares": "gobernar",
    "gobernaron": "gobernar",
    "gobernará": "gobernar",
    "gobernarán": "gobernar",
    "gobernarás": "gobernar",
    "gobernaré": "gobernar",
    "gobernaréis": "gobernar",
    "gobernaría": "gobernar",
    "gobernaríais": "gobernar",
    "gobernaríamos": "gobernar",
    "gobernarían": "gobernar",
    "gobernarías": "gobernar",
    "gobernase": "gobernar",
    "gobernaseis": "gobernar",
    "gobernasen": "gobernar",
    "gobernases": "gobernar",
    "gobernaste": "gobernar",
    "gobernasteis": "gobernar",
    "gobernemos": "gobernar",
    "gobernosa": "gobernoso",
    "gobernosas": "gobernoso",
    "gobernosos": "gobernoso",
    "gobernábamos": "gobernar",
    "gobernáis": "gobernar",
    "gobernáramos": "gobernar",
    "gobernáremos": "gobernar",
    "gobernásemos": "gobernar",
    "goberné": "gobernar",
    "gobernéis": "gobernar",
    "gobernó": "gobernar",
    "gobierna": "gobernar",
    "gobiernan": "gobernar",
    "gobiernas": "gobierna",
    "gobierne": "gobernar",
    "gobiernen": "gobernar",
    "gobiernes": "gobernar",
    "gobierno": "gobernar",
    "gobiernos": "gobierno",
    "gobios": "gobio",
    "goce": "gozar",
    "gocemos": "gozar",
    "gocen": "gozar",
    "goces": "gozar",
    "gocetes": "gocete",
    "gocha": "gocho",
    "gochas": "gocho",
    "gochos": "gocho",
    "gocé": "gozar",
    "gocéis": "gozar",
    "goda": "godo",
    "godas": "godo",
    "godos": "godo",
    "goethitas": "goethita",
    "goetitas": "goetita",
    "gofa": "gofo",
    "gofas": "gofo",
    "gofios": "gofio",
    "gofos": "gofo",
    "gofra": "gofrar",
    "gofraba": "gofrar",
    "gofrabais": "gofrar",
    "gofraban": "gofrar",
    "gofrabas": "gofrar",
    "gofrad": "gofrar",
    "gofrada": "gofrar",
    "gofradas": "gofrar",
    "gofrado": "gofrar",
    "gofradora": "gofrador",
    "gofradoras": "gofrador",
    "gofradores": "gofrador",
    "gofrados": "gofrar",
    "goframos": "gofrar",
    "gofran": "gofrar",
    "gofrando": "gofrar",
    "gofrara": "gofrar",
    "gofrarais": "gofrar",
    "gofraran": "gofrar",
    "gofraras": "gofrar",
    "gofrare": "gofrar",
    "gofrareis": "gofrar",
    "gofraremos": "gofrar",
    "gofraren": "gofrar",
    "gofrares": "gofrar",
    "gofraron": "gofrar",
    "gofrará": "gofrar",
    "gofrarán": "gofrar",
    "gofrarás": "gofrar",
    "gofraré": "gofrar",
    "gofraréis": "gofrar",
    "gofraría": "gofrar",
    "gofraríais": "gofrar",
    "gofraríamos": "gofrar",
    "gofrarían": "gofrar",
    "gofrarías": "gofrar",
    "gofras": "gofrar",
    "gofrase": "gofrar",
    "gofraseis": "gofrar",
    "gofrasen": "gofrar",
    "gofrases": "gofrar",
    "gofraste": "gofrar",
    "gofrasteis": "gofrar",
    "gofre": "gofrar",
    "gofremos": "gofrar",
    "gofren": "gofrar",
    "gofres": "gofrar",
    "gofro": "gofrar",
    "gofrábamos": "gofrar",
    "gofráis": "gofrar",
    "gofráramos": "gofrar",
    "gofráremos": "gofrar",
    "gofrásemos": "gofrar",
    "gofré": "gofrar",
    "gofréis": "gofrar",
    "gofró": "gofrar",
    "gogo-girls": "gogo-girl",
    "golas": "gola",
    "golea": "golear",
    "goleaba": "golear",
    "goleabais": "golear",
    "goleaban": "golear",
    "goleabas": "golear",
    "golead": "golear",
    "goleada": "golear",
    "goleadas": "golear",
    "goleado": "golear",
    "goleadora": "goleador",
    "goleadoras": "goleador",
    "goleadores": "goleador",
    "goleados": "golear",
    "goleamos": "golear",
    "golean": "golear",
    "goleando": "golear",
    "goleara": "golear",
    "golearais": "golear",
    "golearan": "golear",
    "golearas": "golear",
    "goleare": "golear",
    "goleareis": "golear",
    "golearemos": "golear",
    "golearen": "golear",
    "goleares": "golear",
    "golearon": "golear",
    "goleará": "golear",
    "golearán": "golear",
    "golearás": "golear",
    "golearé": "golear",
    "golearéis": "golear",
    "golearía": "golear",
    "golearíais": "golear",
    "golearíamos": "golear",
    "golearían": "golear",
    "golearías": "golear",
    "goleas": "golear",
    "golease": "golear",
    "goleaseis": "golear",
    "goleasen": "golear",
    "goleases": "golear",
    "goleaste": "golear",
    "goleasteis": "golear",
    "golee": "golear",
    "goleemos": "golear",
    "goleen": "golear",
    "golees": "golear",
    "goleo": "golear",
    "goles": "gol",
    "goletas": "goleta",
    "goleábamos": "golear",
    "goleáis": "golear",
    "goleáramos": "golear",
    "goleáremos": "golear",
    "goleásemos": "golear",
    "goleé": "golear",
    "goleéis": "golear",
    "goleó": "golear",
    "golfa": "golfo",
    "golfanes": "golfán",
    "golfantes": "golfante",
    "golfas": "golfo",
    "golfea": "golfear",
    "golfeaba": "golfear",
    "golfeabais": "golfear",
    "golfeaban": "golfear",
    "golfeabas": "golfear",
    "golfead": "golfear",
    "golfeada": "golfear",
    "golfeadas": "golfear",
    "golfeado": "golfear",
    "golfeados": "golfear",
    "golfeamos": "golfear",
    "golfean": "golfear",
    "golfeando": "golfear",
    "golfeara": "golfear",
    "golfearais": "golfear",
    "golfearan": "golfear",
    "golfearas": "golfear",
    "golfeare": "golfear",
    "golfeareis": "golfear",
    "golfearemos": "golfear",
    "golfearen": "golfear",
    "golfeares": "golfear",
    "golfearon": "golfear",
    "golfeará": "golfear",
    "golfearán": "golfear",
    "golfearás": "golfear",
    "golfearé": "golfear",
    "golfearéis": "golfear",
    "golfearía": "golfear",
    "golfearíais": "golfear",
    "golfearíamos": "golfear",
    "golfearían": "golfear",
    "golfearías": "golfear",
    "golfeas": "golfear",
    "golfease": "golfear",
    "golfeaseis": "golfear",
    "golfeasen": "golfear",
    "golfeases": "golfear",
    "golfeaste": "golfear",
    "golfeasteis": "golfear",
    "golfee": "golfear",
    "golfeemos": "golfear",
    "golfeen": "golfear",
    "golfees": "golfear",
    "golfeo": "golfear",
    "golferías": "golfería",
    "golfeábamos": "golfear",
    "golfeáis": "golfear",
    "golfeáramos": "golfear",
    "golfeáremos": "golfear",
    "golfeásemos": "golfear",
    "golfeé": "golfear",
    "golfeéis": "golfear",
    "golfeó": "golfear",
    "golfistas": "golfista",
    "golfos": "golfo",
    "golfs": "golf",
    "golfística": "golfístico",
    "golfísticas": "golfístico",
    "golfísticos": "golfístico",
    "goliarda": "goliardo",
    "goliardas": "goliardo",
    "goliardesca": "goliardesco",
    "goliardescas": "goliardesco",
    "goliardescos": "goliardesco",
    "goliardos": "goliardo",
    "golillas": "golilla",
    "golillera": "golillero",
    "golilleras": "golillero",
    "golilleros": "golillero",
    "gollerías": "gollería",
    "golletazos": "golletazo",
    "golletes": "gollete",
    "golliznos": "gollizno",
    "gollizos": "gollizo",
    "gollorías": "golloría",
    "golondrinas": "golondrina",
    "golondrinos": "golondrino",
    "golondros": "golondro",
    "golosa": "goloso",
    "golosas": "goloso",
    "golosea": "golosear",
    "goloseaba": "golosear",
    "goloseabais": "golosear",
    "goloseaban": "golosear",
    "goloseabas": "golosear",
    "golosead": "golosear",
    "goloseada": "golosear",
    "goloseadas": "golosear",
    "goloseado": "golosear",
    "goloseados": "golosear",
    "goloseamos": "golosear",
    "golosean": "golosear",
    "goloseando": "golosear",
    "goloseara": "golosear",
    "golosearais": "golosear",
    "golosearan": "golosear",
    "golosearas": "golosear",
    "goloseare": "golosear",
    "goloseareis": "golosear",
    "golosearemos": "golosear",
    "golosearen": "golosear",
    "goloseares": "golosear",
    "golosearon": "golosear",
    "goloseará": "golosear",
    "golosearán": "golosear",
    "golosearás": "golosear",
    "golosearé": "golosear",
    "golosearéis": "golosear",
    "golosearía": "golosear",
    "golosearíais": "golosear",
    "golosearíamos": "golosear",
    "golosearían": "golosear",
    "golosearías": "golosear",
    "goloseas": "golosear",
    "golosease": "golosear",
    "goloseaseis": "golosear",
    "goloseasen": "golosear",
    "goloseases": "golosear",
    "goloseaste": "golosear",
    "goloseasteis": "golosear",
    "golosee": "golosear",
    "goloseemos": "golosear",
    "goloseen": "golosear",
    "golosees": "golosear",
    "goloseo": "golosear",
    "goloseábamos": "golosear",
    "goloseáis": "golosear",
    "goloseáramos": "golosear",
    "goloseáremos": "golosear",
    "goloseásemos": "golosear",
    "goloseé": "golosear",
    "goloseéis": "golosear",
    "goloseó": "golosear",
    "golosina": "golosinar",
    "golosinaba": "golosinar",
    "golosinabais": "golosinar",
    "golosinaban": "golosinar",
    "golosinabas": "golosinar",
    "golosinad": "golosinar",
    "golosinada": "golosinar",
    "golosinadas": "golosinar",
    "golosinado": "golosinar",
    "golosinados": "golosinar",
    "golosinamos": "golosinar",
    "golosinan": "golosinar",
    "golosinando": "golosinar",
    "golosinara": "golosinar",
    "golosinarais": "golosinar",
    "golosinaran": "golosinar",
    "golosinaras": "golosinar",
    "golosinare": "golosinar",
    "golosinareis": "golosinar",
    "golosinaremos": "golosinar",
    "golosinaren": "golosinar",
    "golosinares": "golosinar",
    "golosinaron": "golosinar",
    "golosinará": "golosinar",
    "golosinarán": "golosinar",
    "golosinarás": "golosinar",
    "golosinaré": "golosinar",
    "golosinaréis": "golosinar",
    "golosinaría": "golosinar",
    "golosinaríais": "golosinar",
    "golosinaríamos": "golosinar",
    "golosinarían": "golosinar",
    "golosinarías": "golosinar",
    "golosinas": "golosinar",
    "golosinase": "golosinar",
    "golosinaseis": "golosinar",
    "golosinasen": "golosinar",
    "golosinases": "golosinar",
    "golosinaste": "golosinar",
    "golosinasteis": "golosinar",
    "golosine": "golosinar",
    "golosinea": "golosinear",
    "golosineaba": "golosinear",
    "golosineabais": "golosinear",
    "golosineaban": "golosinear",
    "golosineabas": "golosinear",
    "golosinead": "golosinear",
    "golosineada": "golosinear",
    "golosineadas": "golosinear",
    "golosineado": "golosinear",
    "golosineados": "golosinear",
    "golosineamos": "golosinear",
    "golosinean": "golosinear",
    "golosineando": "golosinear",
    "golosineara": "golosinear",
    "golosinearais": "golosinear",
    "golosinearan": "golosinear",
    "golosinearas": "golosinear",
    "golosineare": "golosinear",
    "golosineareis": "golosinear",
    "golosinearemos": "golosinear",
    "golosinearen": "golosinear",
    "golosineares": "golosinear",
    "golosinearon": "golosinear",
    "golosineará": "golosinear",
    "golosinearán": "golosinear",
    "golosinearás": "golosinear",
    "golosinearé": "golosinear",
    "golosinearéis": "golosinear",
    "golosinearía": "golosinear",
    "golosinearíais": "golosinear",
    "golosinearíamos": "golosinear",
    "golosinearían": "golosinear",
    "golosinearías": "golosinear",
    "golosineas": "golosinear",
    "golosinease": "golosinear",
    "golosineaseis": "golosinear",
    "golosineasen": "golosinear",
    "golosineases": "golosinear",
    "golosineaste": "golosinear",
    "golosineasteis": "golosinear",
    "golosinee": "golosinear",
    "golosineemos": "golosinear",
    "golosineen": "golosinear",
    "golosinees": "golosinear",
    "golosinemos": "golosinar",
    "golosinen": "golosinar",
    "golosineo": "golosinear",
    "golosines": "golosinar",
    "golosineábamos": "golosinear",
    "golosineáis": "golosinear",
    "golosineáramos": "golosinear",
    "golosineáremos": "golosinear",
    "golosineásemos": "golosinear",
    "golosineé": "golosinear",
    "golosineéis": "golosinear",
    "golosineó": "golosinear",
    "golosino": "golosinar",
    "golosinábamos": "golosinar",
    "golosináis": "golosinar",
    "golosináramos": "golosinar",
    "golosináremos": "golosinar",
    "golosinásemos": "golosinar",
    "golosiné": "golosinar",
    "golosinéis": "golosinar",
    "golosinó": "golosinar",
    "golosos": "goloso",
    "golpazos": "golpazo",
    "golpea": "golpear",
    "golpeaba": "golpear",
    "golpeabais": "golpear",
    "golpeaban": "golpear",
    "golpeabas": "golpear",
    "golpead": "golpear",
    "golpeada": "golpear",
    "golpeadas": "golpear",
    "golpeaderos": "golpeadero",
    "golpeado": "golpear",
    "golpeadora": "golpeador",
    "golpeadoras": "golpeador",
    "golpeadores": "golpeador",
    "golpeados": "golpear",
    "golpeaduras": "golpeadura",
    "golpeamos": "golpear",
    "golpean": "golpear",
    "golpeando": "golpear",
    "golpeara": "golpear",
    "golpearais": "golpear",
    "golpearan": "golpear",
    "golpearas": "golpear",
    "golpeare": "golpear",
    "golpeareis": "golpear",
    "golpearemos": "golpear",
    "golpearen": "golpear",
    "golpeares": "golpear",
    "golpearon": "golpear",
    "golpeará": "golpear",
    "golpearán": "golpear",
    "golpearás": "golpear",
    "golpearé": "golpear",
    "golpearéis": "golpear",
    "golpearía": "golpear",
    "golpearíais": "golpear",
    "golpearíamos": "golpear",
    "golpearían": "golpear",
    "golpearías": "golpear",
    "golpeas": "golpear",
    "golpease": "golpear",
    "golpeaseis": "golpear",
    "golpeasen": "golpear",
    "golpeases": "golpear",
    "golpeaste": "golpear",
    "golpeasteis": "golpear",
    "golpecitos": "golpecito",
    "golpee": "golpear",
    "golpeemos": "golpear",
    "golpeen": "golpear",
    "golpees": "golpear",
    "golpeo": "golpear",
    "golpeos": "golpeo",
    "golpes": "golpe",
    "golpetea": "golpetear",
    "golpeteaba": "golpetear",
    "golpeteabais": "golpetear",
    "golpeteaban": "golpetear",
    "golpeteabas": "golpetear",
    "golpetead": "golpetear",
    "golpeteada": "golpetear",
    "golpeteadas": "golpetear",
    "golpeteado": "golpetear",
    "golpeteados": "golpetear",
    "golpeteamos": "golpetear",
    "golpetean": "golpetear",
    "golpeteando": "golpetear",
    "golpeteara": "golpetear",
    "golpetearais": "golpetear",
    "golpetearan": "golpetear",
    "golpetearas": "golpetear",
    "golpeteare": "golpetear",
    "golpeteareis": "golpetear",
    "golpetearemos": "golpetear",
    "golpetearen": "golpetear",
    "golpeteares": "golpetear",
    "golpetearon": "golpetear",
    "golpeteará": "golpetear",
    "golpetearán": "golpetear",
    "golpetearás": "golpetear",
    "golpetearé": "golpetear",
    "golpetearéis": "golpetear",
    "golpetearía": "golpetear",
    "golpetearíais": "golpetear",
    "golpetearíamos": "golpetear",
    "golpetearían": "golpetear",
    "golpetearías": "golpetear",
    "golpeteas": "golpetear",
    "golpetease": "golpetear",
    "golpeteaseis": "golpetear",
    "golpeteasen": "golpetear",
    "golpeteases": "golpetear",
    "golpeteaste": "golpetear",
    "golpeteasteis": "golpetear",
    "golpetee": "golpetear",
    "golpeteemos": "golpetear",
    "golpeteen": "golpetear",
    "golpetees": "golpetear",
    "golpeteo": "golpetear",
    "golpeteos": "golpeteo",
    "golpetes": "golpete",
    "golpeteábamos": "golpetear",
    "golpeteáis": "golpetear",
    "golpeteáramos": "golpetear",
    "golpeteáremos": "golpetear",
    "golpeteásemos": "golpetear",
    "golpeteé": "golpetear",
    "golpeteéis": "golpetear",
    "golpeteó": "golpetear",
    "golpeábamos": "golpear",
    "golpeáis": "golpear",
    "golpeáramos": "golpear",
    "golpeáremos": "golpear",
    "golpeásemos": "golpear",
    "golpeé": "golpear",
    "golpeéis": "golpear",
    "golpeó": "golpear",
    "golpismos": "golpismo",
    "golpistas": "golpista",
    "gomaespumas": "gomaespuma",
    "gomagutas": "gomaguta",
    "gomas": "goma",
    "gomera": "gomero",
    "gomeras": "gomero",
    "gomeros": "gomero",
    "gomias": "gomia",
    "gominas": "gomina",
    "gomistas": "gomista",
    "gomorresinas": "gomorresina",
    "gomosa": "gomoso",
    "gomosas": "gomoso",
    "gomoserías": "gomosería",
    "gomosidades": "gomosidad",
    "gomosos": "gomoso",
    "gonadotropinas": "gonadotropina",
    "gonces": "gonce",
    "gondolera": "gondolero",
    "gondoleras": "gondolero",
    "gondoleros": "gondolero",
    "gonelas": "gonela",
    "gonfaloneros": "gonfalonero",
    "gonfalones": "gonfalón",
    "gonfalonieres": "gonfalonier",
    "gonfalonieros": "gonfaloniero",
    "gongorina": "gongorino",
    "gongorinas": "gongorino",
    "gongorinos": "gongorino",
    "gongorismos": "gongorismo",
    "gongoristas": "gongorista",
    "gongorizaciones": "gongorización",
    "gongos": "gongo",
    "gongs": "gong",
    "goniales": "gonial",
    "goniometrías": "goniometría",
    "goniométrica": "goniométrico",
    "goniométricas": "goniométrico",
    "goniométricos": "goniométrico",
    "goniones": "gonión",
    "gonioscopias": "gonioscopia",
    "gonioscopios": "gonioscopio",
    "goniosinequias": "goniosinequia",
    "goniotomías": "goniotomía",
    "goniómetros": "goniómetro",
    "gonocitos": "gonocito",
    "gonococias": "gonococia",
    "gonococos": "gonococo",
    "gonoconductos": "gonoconducto",
    "gonocócica": "gonocócico",
    "gonocócicas": "gonocócico",
    "gonocócicos": "gonocócico",
    "gonorreas": "gonorrea",
    "gonócitos": "gonócito",
    "gora": "ágora",
    "gorales": "goral",
    "gorbiones": "gorbión",
    "gorda": "gordo",
    "gordanas": "gordana",
    "gordas": "gordo",
    "gordiana": "gordiano",
    "gordianas": "gordiano",
    "gordianos": "gordiano",
    "gordinflona": "gordinflón",
    "gordinflonas": "gordinflón",
    "gordinflones": "gordinflón",
    "gordolobos": "gordolobo",
    "gordos": "gordo",
    "gorduras": "gordura",
    "gorgoja": "gorgojar",
    "gorgojaba": "gorgojar",
    "gorgojabais": "gorgojar",
    "gorgojaban": "gorgojar",
    "gorgojabas": "gorgojar",
    "gorgojad": "gorgojar",
    "gorgojada": "gorgojar",
    "gorgojadas": "gorgojar",
    "gorgojado": "gorgojar",
    "gorgojados": "gorgojar",
    "gorgojamos": "gorgojar",
    "gorgojan": "gorgojar",
    "gorgojando": "gorgojar",
    "gorgojara": "gorgojar",
    "gorgojarais": "gorgojar",
    "gorgojaran": "gorgojar",
    "gorgojaras": "gorgojar",
    "gorgojare": "gorgojar",
    "gorgojareis": "gorgojar",
    "gorgojaremos": "gorgojar",
    "gorgojaren": "gorgojar",
    "gorgojares": "gorgojar",
    "gorgojaron": "gorgojar",
    "gorgojará": "gorgojar",
    "gorgojarán": "gorgojar",
    "gorgojarás": "gorgojar",
    "gorgojaré": "gorgojar",
    "gorgojaréis": "gorgojar",
    "gorgojaría": "gorgojar",
    "gorgojaríais": "gorgojar",
    "gorgojaríamos": "gorgojar",
    "gorgojarían": "gorgojar",
    "gorgojarías": "gorgojar",
    "gorgojas": "gorgojar",
    "gorgojase": "gorgojar",
    "gorgojaseis": "gorgojar",
    "gorgojasen": "gorgojar",
    "gorgojases": "gorgojar",
    "gorgojaste": "gorgojar",
    "gorgojasteis": "gorgojar",
    "gorgoje": "gorgojar",
    "gorgojea": "gorgojear",
    "gorgojeaba": "gorgojear",
    "gorgojeabais": "gorgojear",
    "gorgojeaban": "gorgojear",
    "gorgojeabas": "gorgojear",
    "gorgojead": "gorgojear",
    "gorgojeada": "gorgojear",
    "gorgojeadas": "gorgojear",
    "gorgojeado": "gorgojear",
    "gorgojeados": "gorgojear",
    "gorgojeamos": "gorgojear",
    "gorgojean": "gorgojear",
    "gorgojeando": "gorgojear",
    "gorgojeara": "gorgojear",
    "gorgojearais": "gorgojear",
    "gorgojearan": "gorgojear",
    "gorgojearas": "gorgojear",
    "gorgojeare": "gorgojear",
    "gorgojeareis": "gorgojear",
    "gorgojearemos": "gorgojear",
    "gorgojearen": "gorgojear",
    "gorgojeares": "gorgojear",
    "gorgojearon": "gorgojear",
    "gorgojeará": "gorgojear",
    "gorgojearán": "gorgojear",
    "gorgojearás": "gorgojear",
    "gorgojearé": "gorgojear",
    "gorgojearéis": "gorgojear",
    "gorgojearía": "gorgojear",
    "gorgojearíais": "gorgojear",
    "gorgojearíamos": "gorgojear",
    "gorgojearían": "gorgojear",
    "gorgojearías": "gorgojear",
    "gorgojeas": "gorgojear",
    "gorgojease": "gorgojear",
    "gorgojeaseis": "gorgojear",
    "gorgojeasen": "gorgojear",
    "gorgojeases": "gorgojear",
    "gorgojeaste": "gorgojear",
    "gorgojeasteis": "gorgojear",
    "gorgojee": "gorgojear",
    "gorgojeemos": "gorgojear",
    "gorgojeen": "gorgojear",
    "gorgojees": "gorgojear",
    "gorgojemos": "gorgojar",
    "gorgojen": "gorgojar",
    "gorgojeo": "gorgojear",
    "gorgojes": "gorgojar",
    "gorgojeábamos": "gorgojear",
    "gorgojeáis": "gorgojear",
    "gorgojeáramos": "gorgojear",
    "gorgojeáremos": "gorgojear",
    "gorgojeásemos": "gorgojear",
    "gorgojeé": "gorgojear",
    "gorgojeéis": "gorgojear",
    "gorgojeó": "gorgojear",
    "gorgojo": "gorgojar",
    "gorgojos": "gorgojo",
    "gorgojábamos": "gorgojar",
    "gorgojáis": "gorgojar",
    "gorgojáramos": "gorgojar",
    "gorgojáremos": "gorgojar",
    "gorgojásemos": "gorgojar",
    "gorgojé": "gorgojar",
    "gorgojéis": "gorgojar",
    "gorgojó": "gorgojar",
    "gorgonias": "gorgonia",
    "gorgoranes": "gorgorán",
    "gorgores": "gorgor",
    "gorgoritas": "gorgorita",
    "gorgoritea": "gorgoritear",
    "gorgoriteaba": "gorgoritear",
    "gorgoriteabais": "gorgoritear",
    "gorgoriteaban": "gorgoritear",
    "gorgoriteabas": "gorgoritear",
    "gorgoritead": "gorgoritear",
    "gorgoriteada": "gorgoritear",
    "gorgoriteadas": "gorgoritear",
    "gorgoriteado": "gorgoritear",
    "gorgoriteados": "gorgoritear",
    "gorgoriteamos": "gorgoritear",
    "gorgoritean": "gorgoritear",
    "gorgoriteando": "gorgoritear",
    "gorgoriteara": "gorgoritear",
    "gorgoritearais": "gorgoritear",
    "gorgoritearan": "gorgoritear",
    "gorgoritearas": "gorgoritear",
    "gorgoriteare": "gorgoritear",
    "gorgoriteareis": "gorgoritear",
    "gorgoritearemos": "gorgoritear",
    "gorgoritearen": "gorgoritear",
    "gorgoriteares": "gorgoritear",
    "gorgoritearon": "gorgoritear",
    "gorgoriteará": "gorgoritear",
    "gorgoritearán": "gorgoritear",
    "gorgoritearás": "gorgoritear",
    "gorgoritearé": "gorgoritear",
    "gorgoritearéis": "gorgoritear",
    "gorgoritearía": "gorgoritear",
    "gorgoritearíais": "gorgoritear",
    "gorgoritearíamos": "gorgoritear",
    "gorgoritearían": "gorgoritear",
    "gorgoritearías": "gorgoritear",
    "gorgoriteas": "gorgoritear",
    "gorgoritease": "gorgoritear",
    "gorgoriteaseis": "gorgoritear",
    "gorgoriteasen": "gorgoritear",
    "gorgoriteases": "gorgoritear",
    "gorgoriteaste": "gorgoritear",
    "gorgoriteasteis": "gorgoritear",
    "gorgoritee": "gorgoritear",
    "gorgoriteemos": "gorgoritear",
    "gorgoriteen": "gorgoritear",
    "gorgoritees": "gorgoritear",
    "gorgoriteo": "gorgoritear",
    "gorgoriteábamos": "gorgoritear",
    "gorgoriteáis": "gorgoritear",
    "gorgoriteáramos": "gorgoritear",
    "gorgoriteáremos": "gorgoritear",
    "gorgoriteásemos": "gorgoritear",
    "gorgoriteé": "gorgoritear",
    "gorgoriteéis": "gorgoritear",
    "gorgoriteó": "gorgoritear",
    "gorgoritos": "gorgorito",
    "gorgorotadas": "gorgorotada",
    "gorgotea": "gorgotear",
    "gorgoteaba": "gorgotear",
    "gorgoteabais": "gorgotear",
    "gorgoteaban": "gorgotear",
    "gorgoteabas": "gorgotear",
    "gorgotead": "gorgotear",
    "gorgoteada": "gorgotear",
    "gorgoteadas": "gorgotear",
    "gorgoteado": "gorgotear",
    "gorgoteados": "gorgotear",
    "gorgoteamos": "gorgotear",
    "gorgotean": "gorgotear",
    "gorgoteando": "gorgotear",
    "gorgoteara": "gorgotear",
    "gorgotearais": "gorgotear",
    "gorgotearan": "gorgotear",
    "gorgotearas": "gorgotear",
    "gorgoteare": "gorgotear",
    "gorgoteareis": "gorgotear",
    "gorgotearemos": "gorgotear",
    "gorgotearen": "gorgotear",
    "gorgoteares": "gorgotear",
    "gorgotearon": "gorgotear",
    "gorgoteará": "gorgotear",
    "gorgotearán": "gorgotear",
    "gorgotearás": "gorgotear",
    "gorgotearé": "gorgotear",
    "gorgotearéis": "gorgotear",
    "gorgotearía": "gorgotear",
    "gorgotearíais": "gorgotear",
    "gorgotearíamos": "gorgotear",
    "gorgotearían": "gorgotear",
    "gorgotearías": "gorgotear",
    "gorgoteas": "gorgotear",
    "gorgotease": "gorgotear",
    "gorgoteaseis": "gorgotear",
    "gorgoteasen": "gorgotear",
    "gorgoteases": "gorgotear",
    "gorgoteaste": "gorgotear",
    "gorgoteasteis": "gorgotear",
    "gorgotee": "gorgotear",
    "gorgoteemos": "gorgotear",
    "gorgoteen": "gorgotear",
    "gorgotees": "gorgotear",
    "gorgoteo": "gorgotear",
    "gorgoteos": "gorgoteo",
    "gorgoteros": "gorgotero",
    "gorgoteábamos": "gorgotear",
    "gorgoteáis": "gorgotear",
    "gorgoteáramos": "gorgotear",
    "gorgoteáremos": "gorgotear",
    "gorgoteásemos": "gorgotear",
    "gorgoteé": "gorgotear",
    "gorgoteéis": "gorgotear",
    "gorgoteó": "gorgotear",
    "gorguces": "gorguz",
    "gorgueras": "gorguera",
    "gorigoris": "gorigori",
    "gorilas": "gorila",
    "gorjales": "gorjal",
    "gorjas": "gorja",
    "gorjea": "gorjear",
    "gorjeaba": "gorjear",
    "gorjeabais": "gorjear",
    "gorjeaban": "gorjear",
    "gorjeabas": "gorjear",
    "gorjead": "gorjear",
    "gorjeada": "gorjear",
    "gorjeadas": "gorjear",
    "gorjeado": "gorjear",
    "gorjeadora": "gorjeador",
    "gorjeadoras": "gorjeador",
    "gorjeadores": "gorjeador",
    "gorjeados": "gorjear",
    "gorjeamos": "gorjear",
    "gorjean": "gorjear",
    "gorjeando": "gorjear",
    "gorjeara": "gorjear",
    "gorjearais": "gorjear",
    "gorjearan": "gorjear",
    "gorjearas": "gorjear",
    "gorjeare": "gorjear",
    "gorjeareis": "gorjear",
    "gorjearemos": "gorjear",
    "gorjearen": "gorjear",
    "gorjeares": "gorjear",
    "gorjearon": "gorjear",
    "gorjeará": "gorjear",
    "gorjearán": "gorjear",
    "gorjearás": "gorjear",
    "gorjearé": "gorjear",
    "gorjearéis": "gorjear",
    "gorjearía": "gorjear",
    "gorjearíais": "gorjear",
    "gorjearíamos": "gorjear",
    "gorjearían": "gorjear",
    "gorjearías": "gorjear",
    "gorjeas": "gorjear",
    "gorjease": "gorjear",
    "gorjeaseis": "gorjear",
    "gorjeasen": "gorjear",
    "gorjeases": "gorjear",
    "gorjeaste": "gorjear",
    "gorjeasteis": "gorjear",
    "gorjee": "gorjear",
    "gorjeemos": "gorjear",
    "gorjeen": "gorjear",
    "gorjees": "gorjear",
    "gorjeo": "gorjear",
    "gorjeos": "gorjeo",
    "gorjeábamos": "gorjear",
    "gorjeáis": "gorjear",
    "gorjeáramos": "gorjear",
    "gorjeáremos": "gorjear",
    "gorjeásemos": "gorjear",
    "gorjeé": "gorjear",
    "gorjeéis": "gorjear",
    "gorjeó": "gorjear",
    "gorras": "gorra",
    "gorrea": "gorrear",
    "gorreaba": "gorrear",
    "gorreabais": "gorrear",
    "gorreaban": "gorrear",
    "gorreabas": "gorrear",
    "gorread": "gorrear",
    "gorreada": "gorrear",
    "gorreadas": "gorrear",
    "gorreado": "gorrear",
    "gorreados": "gorrear",
    "gorreamos": "gorrear",
    "gorrean": "gorrear",
    "gorreando": "gorrear",
    "gorreara": "gorrear",
    "gorrearais": "gorrear",
    "gorrearan": "gorrear",
    "gorrearas": "gorrear",
    "gorreare": "gorrear",
    "gorreareis": "gorrear",
    "gorrearemos": "gorrear",
    "gorrearen": "gorrear",
    "gorreares": "gorrear",
    "gorrearon": "gorrear",
    "gorreará": "gorrear",
    "gorrearán": "gorrear",
    "gorrearás": "gorrear",
    "gorrearé": "gorrear",
    "gorrearéis": "gorrear",
    "gorrearía": "gorrear",
    "gorrearíais": "gorrear",
    "gorrearíamos": "gorrear",
    "gorrearían": "gorrear",
    "gorrearías": "gorrear",
    "gorreas": "gorrear",
    "gorrease": "gorrear",
    "gorreaseis": "gorrear",
    "gorreasen": "gorrear",
    "gorreases": "gorrear",
    "gorreaste": "gorrear",
    "gorreasteis": "gorrear",
    "gorree": "gorrear",
    "gorreemos": "gorrear",
    "gorreen": "gorrear",
    "gorrees": "gorrear",
    "gorreo": "gorrear",
    "gorrera": "gorrero",
    "gorreras": "gorrero",
    "gorreros": "gorrero",
    "gorretadas": "gorretada",
    "gorreábamos": "gorrear",
    "gorreáis": "gorrear",
    "gorreáramos": "gorrear",
    "gorreáremos": "gorrear",
    "gorreásemos": "gorrear",
    "gorreé": "gorrear",
    "gorreéis": "gorrear",
    "gorreó": "gorrear",
    "gorrina": "gorrino",
    "gorrinadas": "gorrinada",
    "gorrinas": "gorrino",
    "gorrineras": "gorrinera",
    "gorrinerías": "gorrinería",
    "gorrinos": "gorrino",
    "gorriona": "gorrión",
    "gorrionas": "gorriona",
    "gorrioneras": "gorrionera",
    "gorriones": "gorrión",
    "gorristas": "gorrista",
    "gorrona": "gorrón",
    "gorronas": "gorrón",
    "gorronea": "gorronear",
    "gorroneaba": "gorronear",
    "gorroneabais": "gorronear",
    "gorroneaban": "gorronear",
    "gorroneabas": "gorronear",
    "gorronead": "gorronear",
    "gorroneada": "gorronear",
    "gorroneadas": "gorronear",
    "gorroneado": "gorronear",
    "gorroneados": "gorronear",
    "gorroneamos": "gorronear",
    "gorronean": "gorronear",
    "gorroneando": "gorronear",
    "gorroneara": "gorronear",
    "gorronearais": "gorronear",
    "gorronearan": "gorronear",
    "gorronearas": "gorronear",
    "gorroneare": "gorronear",
    "gorroneareis": "gorronear",
    "gorronearemos": "gorronear",
    "gorronearen": "gorronear",
    "gorroneares": "gorronear",
    "gorronearon": "gorronear",
    "gorroneará": "gorronear",
    "gorronearán": "gorronear",
    "gorronearás": "gorronear",
    "gorronearé": "gorronear",
    "gorronearéis": "gorronear",
    "gorronearía": "gorronear",
    "gorronearíais": "gorronear",
    "gorronearíamos": "gorronear",
    "gorronearían": "gorronear",
    "gorronearías": "gorronear",
    "gorroneas": "gorronear",
    "gorronease": "gorronear",
    "gorroneaseis": "gorronear",
    "gorroneasen": "gorronear",
    "gorroneases": "gorronear",
    "gorroneaste": "gorronear",
    "gorroneasteis": "gorronear",
    "gorronee": "gorronear",
    "gorroneemos": "gorronear",
    "gorroneen": "gorronear",
    "gorronees": "gorronear",
    "gorroneo": "gorronear",
    "gorronerías": "gorronería",
    "gorrones": "gorrón",
    "gorroneábamos": "gorronear",
    "gorroneáis": "gorronear",
    "gorroneáramos": "gorronear",
    "gorroneáremos": "gorronear",
    "gorroneásemos": "gorronear",
    "gorroneé": "gorronear",
    "gorroneéis": "gorronear",
    "gorroneó": "gorronear",
    "gorros": "gorro",
    "gorullos": "gorullo",
    "goshenitas": "goshenita",
    "gosipina": "gosipino",
    "gosipinas": "gosipino",
    "gosipinos": "gosipino",
    "gosipoles": "gosipol",
    "gotas": "gota",
    "gotea": "gotear",
    "goteaba": "gotear",
    "goteabais": "gotear",
    "goteaban": "gotear",
    "goteabas": "gotear",
    "gotead": "gotear",
    "goteada": "gotear",
    "goteadas": "gotear",
    "goteado": "gotear",
    "goteados": "gotear",
    "goteamos": "gotear",
    "gotean": "gotear",
    "goteando": "gotear",
    "goteantes": "goteante",
    "goteara": "gotear",
    "gotearais": "gotear",
    "gotearan": "gotear",
    "gotearas": "gotear",
    "goteare": "gotear",
    "goteareis": "gotear",
    "gotearemos": "gotear",
    "gotearen": "gotear",
    "goteares": "gotear",
    "gotearon": "gotear",
    "goteará": "gotear",
    "gotearán": "gotear",
    "gotearás": "gotear",
    "gotearé": "gotear",
    "gotearéis": "gotear",
    "gotearía": "gotear",
    "gotearíais": "gotear",
    "gotearíamos": "gotear",
    "gotearían": "gotear",
    "gotearías": "gotear",
    "goteas": "gotear",
    "gotease": "gotear",
    "goteaseis": "gotear",
    "goteasen": "gotear",
    "goteases": "gotear",
    "goteaste": "gotear",
    "goteasteis": "gotear",
    "gotee": "gotear",
    "goteemos": "gotear",
    "goteen": "gotear",
    "gotees": "gotear",
    "goteo": "gotear",
    "goteos": "goteo",
    "goteras": "gotera",
    "goterones": "goterón",
    "goteros": "gotero",
    "goteábamos": "gotear",
    "goteáis": "gotear",
    "goteáramos": "gotear",
    "goteáremos": "gotear",
    "goteásemos": "gotear",
    "goteé": "gotear",
    "goteéis": "gotear",
    "goteó": "gotear",
    "gotosa": "gotoso",
    "gotosas": "gotoso",
    "gotosos": "gotoso",
    "gouaches": "gouache",
    "gourdes": "gourde",
    "gourmes": "gourmé",
    "gourmetes": "gourmet",
    "gourmets": "gourmet",
    "goyesca": "goyesco",
    "goyescas": "goyesco",
    "goyescos": "goyesco",
    "goza": "gozar",
    "gozaba": "gozar",
    "gozabais": "gozar",
    "gozaban": "gozar",
    "gozabas": "gozar",
    "gozad": "gozar",
    "gozada": "gozar",
    "gozadas": "gozar",
    "gozado": "gozar",
    "gozados": "gozar",
    "gozamos": "gozar",
    "gozan": "gozar",
    "gozando": "gozar",
    "gozara": "gozar",
    "gozarais": "gozar",
    "gozaran": "gozar",
    "gozaras": "gozar",
    "gozare": "gozar",
    "gozareis": "gozar",
    "gozaremos": "gozar",
    "gozaren": "gozar",
    "gozares": "gozar",
    "gozaron": "gozar",
    "gozará": "gozar",
    "gozarán": "gozar",
    "gozarás": "gozar",
    "gozaré": "gozar",
    "gozaréis": "gozar",
    "gozaría": "gozar",
    "gozaríais": "gozar",
    "gozaríamos": "gozar",
    "gozarían": "gozar",
    "gozarías": "gozar",
    "gozas": "gozar",
    "gozase": "gozar",
    "gozaseis": "gozar",
    "gozasen": "gozar",
    "gozases": "gozar",
    "gozaste": "gozar",
    "gozasteis": "gozar",
    "goznes": "gozne",
    "gozo": "gozar",
    "gozos": "gozo",
    "gozosa": "gozoso",
    "gozosas": "gozoso",
    "gozosos": "gozoso",
    "gozques": "gozque",
    "gozábamos": "gozar",
    "gozáis": "gozar",
    "gozáramos": "gozar",
    "gozáremos": "gozar",
    "gozásemos": "gozar",
    "gozó": "gozar",
    "gr": "gramo",
    "graales": "graal",
    "graba": "grabar",
    "grababa": "grabar",
    "grababais": "grabar",
    "grababan": "grabar",
    "grababas": "grabar",
    "grabaciones": "grabación",
    "grabad": "grabar",
    "grabada": "grabar",
    "grabadas": "grabar",
    "grabado": "grabar",
    "grabadora": "grabador",
    "grabadoras": "grabadora",
    "grabadores": "grabador",
    "grabados": "grabar",
    "grabaduras": "grabadura",
    "grabamos": "grabar",
    "graban": "grabar",
    "grabando": "grabar",
    "grabara": "grabar",
    "grabarais": "grabar",
    "grabaran": "grabar",
    "grabaras": "grabar",
    "grabare": "grabar",
    "grabareis": "grabar",
    "grabaremos": "grabar",
    "grabaren": "grabar",
    "grabares": "grabar",
    "grabaron": "grabar",
    "grabará": "grabar",
    "grabarán": "grabar",
    "grabarás": "grabar",
    "grabaré": "grabar",
    "grabaréis": "grabar",
    "grabaría": "grabar",
    "grabaríais": "grabar",
    "grabaríamos": "grabar",
    "grabarían": "grabar",
    "grabarías": "grabar",
    "grabas": "grabar",
    "grabase": "grabar",
    "grabaseis": "grabar",
    "grabasen": "grabar",
    "grabases": "grabar",
    "grabaste": "grabar",
    "grabasteis": "grabar",
    "grabazones": "grabazón",
    "grabe": "grabar",
    "grabemos": "grabar",
    "graben": "grabar",
    "grabes": "grabar",
    "grabo": "grabar",
    "grabábamos": "grabar",
    "grabáis": "grabar",
    "grabáramos": "grabar",
    "grabáremos": "grabar",
    "grabásemos": "grabar",
    "grabé": "grabar",
    "grabéis": "grabar",
    "grabó": "grabar",
    "graceja": "gracejar",
    "gracejaba": "gracejar",
    "gracejabais": "gracejar",
    "gracejaban": "gracejar",
    "gracejabas": "gracejar",
    "gracejad": "gracejar",
    "gracejada": "gracejar",
    "gracejadas": "gracejar",
    "gracejado": "gracejar",
    "gracejados": "gracejar",
    "gracejamos": "gracejar",
    "gracejan": "gracejar",
    "gracejando": "gracejar",
    "gracejara": "gracejar",
    "gracejarais": "gracejar",
    "gracejaran": "gracejar",
    "gracejaras": "gracejar",
    "gracejare": "gracejar",
    "gracejareis": "gracejar",
    "gracejaremos": "gracejar",
    "gracejaren": "gracejar",
    "gracejares": "gracejar",
    "gracejaron": "gracejar",
    "gracejará": "gracejar",
    "gracejarán": "gracejar",
    "gracejarás": "gracejar",
    "gracejaré": "gracejar",
    "gracejaréis": "gracejar",
    "gracejaría": "gracejar",
    "gracejaríais": "gracejar",
    "gracejaríamos": "gracejar",
    "gracejarían": "gracejar",
    "gracejarías": "gracejar",
    "gracejas": "gracejar",
    "gracejase": "gracejar",
    "gracejaseis": "gracejar",
    "gracejasen": "gracejar",
    "gracejases": "gracejar",
    "gracejaste": "gracejar",
    "gracejasteis": "gracejar",
    "graceje": "gracejar",
    "gracejemos": "gracejar",
    "gracejen": "gracejar",
    "gracejes": "gracejar",
    "gracejo": "gracejar",
    "gracejos": "gracejo",
    "gracejábamos": "gracejar",
    "gracejáis": "gracejar",
    "gracejáramos": "gracejar",
    "gracejáremos": "gracejar",
    "gracejásemos": "gracejar",
    "gracejé": "gracejar",
    "gracejéis": "gracejar",
    "gracejó": "gracejar",
    "graciables": "graciable",
    "gracias": "gracia",
    "gracilidades": "gracilidad",
    "graciosa": "gracioso",
    "graciosas": "gracioso",
    "graciosidades": "graciosidad",
    "graciosos": "gracioso",
    "gracíolas": "gracíola",
    "grada": "gradar",
    "gradaba": "gradar",
    "gradabais": "gradar",
    "gradaban": "gradar",
    "gradabas": "gradar",
    "gradaciones": "gradación",
    "gradad": "gradar",
    "gradada": "gradar",
    "gradadas": "gradar",
    "gradado": "gradar",
    "gradados": "gradar",
    "gradamos": "gradar",
    "gradan": "gradar",
    "gradando": "gradar",
    "gradara": "gradar",
    "gradarais": "gradar",
    "gradaran": "gradar",
    "gradaras": "gradar",
    "gradare": "gradar",
    "gradareis": "gradar",
    "gradaremos": "gradar",
    "gradaren": "gradar",
    "gradares": "gradar",
    "gradaron": "gradar",
    "gradará": "gradar",
    "gradarán": "gradar",
    "gradarás": "gradar",
    "gradaré": "gradar",
    "gradaréis": "gradar",
    "gradaría": "gradar",
    "gradaríais": "gradar",
    "gradaríamos": "gradar",
    "gradarían": "gradar",
    "gradarías": "gradar",
    "gradas": "gradar",
    "gradase": "gradar",
    "gradaseis": "gradar",
    "gradasen": "gradar",
    "gradases": "gradar",
    "gradaste": "gradar",
    "gradasteis": "gradar",
    "grade": "gradar",
    "grademos": "gradar",
    "graden": "gradar",
    "gradeos": "gradeo",
    "graderías": "gradería",
    "graderíos": "graderío",
    "grades": "gradar",
    "gradientes": "gradiente",
    "gradillas": "gradilla",
    "grado": "gradar",
    "grados": "grado",
    "graduaba": "graduar",
    "graduabais": "graduar",
    "graduaban": "graduar",
    "graduabas": "graduar",
    "graduables": "graduable",
    "graduaciones": "graduación",
    "graduad": "graduar",
    "graduada": "graduar",
    "graduadas": "graduar",
    "graduado": "graduar",
    "graduadora": "graduador",
    "graduadoras": "graduador",
    "graduadores": "graduador",
    "graduados": "graduar",
    "graduales": "gradual",
    "gradualidades": "gradualidad",
    "graduamos": "graduar",
    "graduanda": "graduando",
    "graduandas": "graduando",
    "graduando": "graduar",
    "graduandos": "graduando",
    "graduara": "graduar",
    "graduarais": "graduar",
    "graduaran": "graduar",
    "graduaras": "graduar",
    "graduare": "graduar",
    "graduareis": "graduar",
    "graduaremos": "graduar",
    "graduaren": "graduar",
    "graduares": "graduar",
    "graduaron": "graduar",
    "graduará": "graduar",
    "graduarán": "graduar",
    "graduarás": "graduar",
    "graduaré": "graduar",
    "graduaréis": "graduar",
    "graduaría": "graduar",
    "graduaríais": "graduar",
    "graduaríamos": "graduar",
    "graduarían": "graduar",
    "graduarías": "graduar",
    "graduase": "graduar",
    "graduaseis": "graduar",
    "graduasen": "graduar",
    "graduases": "graduar",
    "graduaste": "graduar",
    "graduasteis": "graduar",
    "graduemos": "graduar",
    "graduábamos": "graduar",
    "graduáis": "graduar",
    "graduáramos": "graduar",
    "graduáremos": "graduar",
    "graduásemos": "graduar",
    "gradué": "graduar",
    "graduéis": "graduar",
    "graduó": "graduar",
    "gradábamos": "gradar",
    "gradáis": "gradar",
    "gradáramos": "gradar",
    "gradáremos": "gradar",
    "gradásemos": "gradar",
    "gradé": "gradar",
    "gradéis": "gradar",
    "gradó": "gradar",
    "gradúa": "graduar",
    "gradúan": "graduar",
    "gradúas": "graduar",
    "gradúe": "graduar",
    "gradúen": "graduar",
    "gradúes": "graduar",
    "gradúo": "graduar",
    "grafemas": "grafema",
    "graffitis": "graffiti",
    "graficadores": "graficador",
    "grafilas": "grafila",
    "grafioles": "grafiol",
    "grafios": "grafio",
    "grafismos": "grafismo",
    "grafistas": "grafista",
    "grafitizaciones": "grafitización",
    "grafitos": "grafito",
    "grafologías": "grafología",
    "grafológica": "grafológico",
    "grafológicas": "grafológico",
    "grafológicos": "grafológico",
    "grafomanías": "grafomanía",
    "grafometrías": "grafometría",
    "grafométrica": "grafométrico",
    "grafométricas": "grafométrico",
    "grafométricos": "grafométrico",
    "grafos": "grafo",
    "grafostática": "grafostático",
    "grafostáticas": "grafostático",
    "grafostáticos": "grafostático",
    "grafías": "grafía",
    "grafísticas": "grafística",
    "grafítica": "grafítico",
    "grafíticas": "grafítico",
    "grafíticos": "grafítico",
    "grafóloga": "grafólogo",
    "grafólogas": "grafólogo",
    "grafólogos": "grafólogo",
    "grafómana": "grafómano",
    "grafómanas": "grafómano",
    "grafómanos": "grafómano",
    "grafómetros": "grafómetro",
    "grageas": "gragea",
    "grahamitas": "grahamita",
    "grajas": "graja",
    "grajea": "grajear",
    "grajeaba": "grajear",
    "grajeabais": "grajear",
    "grajeaban": "grajear",
    "grajeabas": "grajear",
    "grajead": "grajear",
    "grajeada": "grajear",
    "grajeadas": "grajear",
    "grajeado": "grajear",
    "grajeados": "grajear",
    "grajeamos": "grajear",
    "grajean": "grajear",
    "grajeando": "grajear",
    "grajeara": "grajear",
    "grajearais": "grajear",
    "grajearan": "grajear",
    "grajearas": "grajear",
    "grajeare": "grajear",
    "grajeareis": "grajear",
    "grajearemos": "grajear",
    "grajearen": "grajear",
    "grajeares": "grajear",
    "grajearon": "grajear",
    "grajeará": "grajear",
    "grajearán": "grajear",
    "grajearás": "grajear",
    "grajearé": "grajear",
    "grajearéis": "grajear",
    "grajearía": "grajear",
    "grajearíais": "grajear",
    "grajearíamos": "grajear",
    "grajearían": "grajear",
    "grajearías": "grajear",
    "grajeas": "grajear",
    "grajease": "grajear",
    "grajeaseis": "grajear",
    "grajeasen": "grajear",
    "grajeases": "grajear",
    "grajeaste": "grajear",
    "grajeasteis": "grajear",
    "grajee": "grajear",
    "grajeemos": "grajear",
    "grajeen": "grajear",
    "grajees": "grajear",
    "grajeo": "grajear",
    "grajeábamos": "grajear",
    "grajeáis": "grajear",
    "grajeáramos": "grajear",
    "grajeáremos": "grajear",
    "grajeásemos": "grajear",
    "grajeé": "grajear",
    "grajeéis": "grajear",
    "grajeó": "grajear",
    "grajillas": "grajilla",
    "grajos": "grajo",
    "grallera": "grallero",
    "gralleras": "grallero",
    "gralleros": "grallero",
    "gramajes": "gramaje",
    "gramales": "gramal",
    "gramallas": "gramalla",
    "gramas": "grama",
    "gramaticales": "gramatical",
    "gramaticalidades": "gramaticalidad",
    "gramaticalizaciones": "gramaticalización",
    "gramatiquerías": "gramatiquería",
    "gramicidinas": "gramicidina",
    "gramiles": "gramil",
    "gramillas": "gramilla",
    "graminicidas": "graminicida",
    "graminácea": "gramináceo",
    "gramináceas": "gramináceo",
    "gramináceos": "gramináceo",
    "gramo-fuerzas": "gramo-fuerza",
    "gramofuerzas": "gramofuerza",
    "gramolas": "gramola",
    "gramomoleculares": "gramomolecular",
    "gramos": "gramo",
    "gramosa": "gramoso",
    "gramosas": "gramoso",
    "gramosos": "gramoso",
    "gramática": "gramático",
    "gramáticas": "gramático",
    "gramáticos": "gramático",
    "gramínea": "gramíneo",
    "gramíneas": "gramíneo",
    "gramíneos": "gramíneo",
    "gramófonos": "gramófono",
    "grana": "granar",
    "granaba": "granar",
    "granabais": "granar",
    "granaban": "granar",
    "granabas": "granar",
    "granad": "granar",
    "granada": "granar",
    "granadales": "granadal",
    "granadas": "granar",
    "granaderas": "granadera",
    "granaderos": "granadero",
    "granadillas": "granadilla",
    "granadillos": "granadillo",
    "granadina": "granadino",
    "granadinas": "granadino",
    "granadinos": "granadino",
    "granado": "granar",
    "granados": "granar",
    "granajes": "granaje",
    "granallas": "granalla",
    "granamos": "granar",
    "granan": "granar",
    "granando": "granar",
    "granara": "granar",
    "granarais": "granar",
    "granaran": "granar",
    "granaras": "granar",
    "granare": "granar",
    "granareis": "granar",
    "granaremos": "granar",
    "granaren": "granar",
    "granares": "granar",
    "granaron": "granar",
    "granará": "granar",
    "granarán": "granar",
    "granarás": "granar",
    "granaré": "granar",
    "granaréis": "granar",
    "granaría": "granar",
    "granaríais": "granar",
    "granaríamos": "granar",
    "granarían": "granar",
    "granarías": "granar",
    "granas": "granar",
    "granase": "granar",
    "granaseis": "granar",
    "granasen": "granar",
    "granases": "granar",
    "granaste": "granar",
    "granasteis": "granar",
    "granates": "granate",
    "granatitas": "granatita",
    "granazones": "granazón",
    "granceros": "grancero",
    "grancillas": "grancilla",
    "grancés": "grancé",
    "grandas": "granda",
    "grandes": "grande",
    "grandeva": "grandevo",
    "grandevas": "grandevo",
    "grandevos": "grandevo",
    "grandezas": "grandeza",
    "grandillona": "grandillón",
    "grandillonas": "grandillón",
    "grandillones": "grandillón",
    "grandilocuencias": "grandilocuencia",
    "grandilocuentes": "grandilocuente",
    "grandiosa": "grandioso",
    "grandiosas": "grandioso",
    "grandiosidades": "grandiosidad",
    "grandiosos": "grandioso",
    "grandores": "grandor",
    "grandullona": "grandullón",
    "grandullonas": "grandullón",
    "grandullones": "grandullón",
    "grandísona": "grandísono",
    "grandísonas": "grandísono",
    "grandísonos": "grandísono",
    "grane": "granar",
    "granea": "granear",
    "graneaba": "granear",
    "graneabais": "granear",
    "graneaban": "granear",
    "graneabas": "granear",
    "granead": "granear",
    "graneada": "granear",
    "graneadas": "granear",
    "graneado": "granear",
    "graneadores": "graneador",
    "graneados": "granear",
    "graneamos": "granear",
    "granean": "granear",
    "graneando": "granear",
    "graneara": "granear",
    "granearais": "granear",
    "granearan": "granear",
    "granearas": "granear",
    "graneare": "granear",
    "graneareis": "granear",
    "granearemos": "granear",
    "granearen": "granear",
    "graneares": "granear",
    "granearon": "granear",
    "graneará": "granear",
    "granearán": "granear",
    "granearás": "granear",
    "granearé": "granear",
    "granearéis": "granear",
    "granearía": "granear",
    "granearíais": "granear",
    "granearíamos": "granear",
    "granearían": "granear",
    "granearías": "granear",
    "graneas": "granear",
    "granease": "granear",
    "graneaseis": "granear",
    "graneasen": "granear",
    "graneases": "granear",
    "graneaste": "granear",
    "graneasteis": "granear",
    "granee": "granear",
    "graneemos": "granear",
    "graneen": "granear",
    "granees": "granear",
    "graneleros": "granelero",
    "granemos": "granar",
    "granen": "granar",
    "graneo": "granear",
    "graneos": "graneo",
    "graneros": "granero",
    "granes": "granar",
    "graneábamos": "granear",
    "graneáis": "granear",
    "graneáramos": "granear",
    "graneáremos": "granear",
    "graneásemos": "granear",
    "graneé": "granear",
    "graneéis": "granear",
    "graneó": "granear",
    "granguardias": "granguardia",
    "granice": "granizar",
    "granicemos": "granizar",
    "granicen": "granizar",
    "granices": "granizar",
    "granicé": "granizar",
    "granicéis": "granizar",
    "granillas": "granilla",
    "granillos": "granillo",
    "granillosa": "granilloso",
    "granillosas": "granilloso",
    "granillosos": "granilloso",
    "granitizaciones": "granitización",
    "granitos": "granito",
    "graniza": "granizar",
    "granizaba": "granizar",
    "granizabais": "granizar",
    "granizaban": "granizar",
    "granizabas": "granizar",
    "granizad": "granizar",
    "granizada": "granizar",
    "granizadas": "granizar",
    "granizado": "granizar",
    "granizados": "granizar",
    "granizamos": "granizar",
    "granizan": "granizar",
    "granizando": "granizar",
    "granizara": "granizar",
    "granizarais": "granizar",
    "granizaran": "granizar",
    "granizaras": "granizar",
    "granizare": "granizar",
    "granizareis": "granizar",
    "granizaremos": "granizar",
    "granizaren": "granizar",
    "granizares": "granizar",
    "granizaron": "granizar",
    "granizará": "granizar",
    "granizarán": "granizar",
    "granizarás": "granizar",
    "granizaré": "granizar",
    "granizaréis": "granizar",
    "granizaría": "granizar",
    "granizaríais": "granizar",
    "granizaríamos": "granizar",
    "granizarían": "granizar",
    "granizarías": "granizar",
    "granizas": "granizar",
    "granizase": "granizar",
    "granizaseis": "granizar",
    "granizasen": "granizar",
    "granizases": "granizar",
    "granizaste": "granizar",
    "granizasteis": "granizar",
    "granizo": "granizar",
    "granizos": "granizo",
    "granizábamos": "granizar",
    "granizáis": "granizar",
    "granizáramos": "granizar",
    "granizáremos": "granizar",
    "granizásemos": "granizar",
    "granizó": "granizar",
    "granjas": "granja",
    "granjea": "granjear",
    "granjeaba": "granjear",
    "granjeabais": "granjear",
    "granjeaban": "granjear",
    "granjeabas": "granjear",
    "granjeables": "granjeable",
    "granjead": "granjear",
    "granjeada": "granjear",
    "granjeadas": "granjear",
    "granjeado": "granjear",
    "granjeados": "granjear",
    "granjeamos": "granjear",
    "granjean": "granjear",
    "granjeando": "granjear",
    "granjeara": "granjear",
    "granjearais": "granjear",
    "granjearan": "granjear",
    "granjearas": "granjear",
    "granjeare": "granjear",
    "granjeareis": "granjear",
    "granjearemos": "granjear",
    "granjearen": "granjear",
    "granjeares": "granjear",
    "granjearon": "granjear",
    "granjeará": "granjear",
    "granjearán": "granjear",
    "granjearás": "granjear",
    "granjearé": "granjear",
    "granjearéis": "granjear",
    "granjearía": "granjear",
    "granjearíais": "granjear",
    "granjearíamos": "granjear",
    "granjearían": "granjear",
    "granjearías": "granjear",
    "granjeas": "granjear",
    "granjease": "granjear",
    "granjeaseis": "granjear",
    "granjeasen": "granjear",
    "granjeases": "granjear",
    "granjeaste": "granjear",
    "granjeasteis": "granjear",
    "granjee": "granjear",
    "granjeemos": "granjear",
    "granjeen": "granjear",
    "granjees": "granjear",
    "granjeo": "granjear",
    "granjeos": "granjeo",
    "granjera": "granjero",
    "granjeras": "granjero",
    "granjeros": "granjero",
    "granjerías": "granjería",
    "granjeábamos": "granjear",
    "granjeáis": "granjear",
    "granjeáramos": "granjear",
    "granjeáremos": "granjear",
    "granjeásemos": "granjear",
    "granjeé": "granjear",
    "granjeéis": "granjear",
    "granjeó": "granjear",
    "grano": "granar",
    "granodioritas": "granodiorita",
    "granos": "grano",
    "granujadas": "granujada",
    "granujas": "granuja",
    "granujerías": "granujería",
    "granujienta": "granujiento",
    "granujientas": "granujiento",
    "granujientos": "granujiento",
    "granujos": "granujo",
    "granula": "granular",
    "granulaba": "granular",
    "granulabais": "granular",
    "granulaban": "granular",
    "granulabas": "granular",
    "granulaciones": "granulación",
    "granulad": "granular",
    "granulada": "granular",
    "granuladas": "granular",
    "granulado": "granular",
    "granuladora": "granulador",
    "granuladoras": "granuladora",
    "granuladores": "granulador",
    "granulados": "granular",
    "granulamos": "granular",
    "granulan": "granular",
    "granulando": "granular",
    "granulara": "granular",
    "granularais": "granular",
    "granularan": "granular",
    "granularas": "granular",
    "granulare": "granular",
    "granulareis": "granular",
    "granularemos": "granular",
    "granularen": "granular",
    "granulares": "granular",
    "granularidades": "granularidad",
    "granularon": "granular",
    "granulará": "granular",
    "granularán": "granular",
    "granularás": "granular",
    "granularé": "granular",
    "granularéis": "granular",
    "granularía": "granular",
    "granularíais": "granular",
    "granularíamos": "granular",
    "granularían": "granular",
    "granularías": "granular",
    "granulas": "granular",
    "granulase": "granular",
    "granulaseis": "granular",
    "granulasen": "granular",
    "granulases": "granular",
    "granulaste": "granular",
    "granulasteis": "granular",
    "granule": "granular",
    "granulemos": "granular",
    "granulen": "granular",
    "granules": "granular",
    "granulitas": "granulita",
    "granulo": "granular",
    "granulocitopenias": "granulocitopenia",
    "granulocitos": "granulocito",
    "granulomas": "granuloma",
    "granulomatosa": "granulomatoso",
    "granulomatosas": "granulomatoso",
    "granulomatosos": "granulomatoso",
    "granulometrías": "granulometría",
    "granulométrica": "granulométrico",
    "granulométricas": "granulométrico",
    "granulométricos": "granulométrico",
    "granulopenias": "granulopenia",
    "granulopoyetinas": "granulopoyetina",
    "granulos": "granulo",
    "granulosa": "granuloso",
    "granulosas": "granuloso",
    "granulosidades": "granulosidad",
    "granulosos": "granuloso",
    "granulábamos": "granular",
    "granuláis": "granular",
    "granuláramos": "granular",
    "granuláremos": "granular",
    "granulásemos": "granular",
    "granulé": "granular",
    "granuléis": "granular",
    "granuló": "granular",
    "granulómeros": "granulómero",
    "granzas": "granza",
    "granzones": "granzón",
    "granzosa": "granzoso",
    "granzosas": "granzoso",
    "granzosos": "granzoso",
    "granábamos": "granar",
    "granáis": "granar",
    "granáramos": "granar",
    "granáremos": "granar",
    "granásemos": "granar",
    "grané": "granar",
    "granéis": "granar",
    "granífuga": "granífugo",
    "granífugas": "granífugo",
    "granífugos": "granífugo",
    "granítica": "granítico",
    "graníticas": "granítico",
    "graníticos": "granítico",
    "granívora": "granívoro",
    "granívoras": "granívoro",
    "granívoros": "granívoro",
    "granó": "granar",
    "graos": "grao",
    "grapa": "grapar",
    "grapaba": "grapar",
    "grapabais": "grapar",
    "grapaban": "grapar",
    "grapabas": "grapar",
    "grapad": "grapar",
    "grapada": "grapar",
    "grapadas": "grapar",
    "grapado": "grapar",
    "grapadora": "grapador",
    "grapadoras": "grapadora",
    "grapadores": "grapador",
    "grapados": "grapar",
    "grapaduras": "grapadura",
    "grapamos": "grapar",
    "grapan": "grapar",
    "grapando": "grapar",
    "grapara": "grapar",
    "graparais": "grapar",
    "graparan": "grapar",
    "graparas": "grapar",
    "grapare": "grapar",
    "grapareis": "grapar",
    "graparemos": "grapar",
    "graparen": "grapar",
    "grapares": "grapar",
    "graparon": "grapar",
    "grapará": "grapar",
    "graparán": "grapar",
    "graparás": "grapar",
    "graparé": "grapar",
    "graparéis": "grapar",
    "graparía": "grapar",
    "graparíais": "grapar",
    "graparíamos": "grapar",
    "graparían": "grapar",
    "graparías": "grapar",
    "grapas": "grapar",
    "grapase": "grapar",
    "grapaseis": "grapar",
    "grapasen": "grapar",
    "grapases": "grapar",
    "grapaste": "grapar",
    "grapasteis": "grapar",
    "grape": "grapar",
    "grapemos": "grapar",
    "grapen": "grapar",
    "grapes": "grapar",
    "grapo": "grapar",
    "graptolites": "graptolite",
    "grapábamos": "grapar",
    "grapáis": "grapar",
    "grapáramos": "grapar",
    "grapáremos": "grapar",
    "grapásemos": "grapar",
    "grapé": "grapar",
    "grapéis": "grapar",
    "grapó": "grapar",
    "grasa": "graso",
    "grasas": "graso",
    "graseras": "grasera",
    "graseros": "grasero",
    "graserías": "grasería",
    "grasienta": "grasiento",
    "grasientas": "grasiento",
    "grasientos": "grasiento",
    "grasillas": "grasilla",
    "grasos": "graso",
    "grasosa": "grasoso",
    "grasosas": "grasoso",
    "grasosos": "grasoso",
    "graspos": "graspo",
    "grasuras": "grasura",
    "grata": "grato",
    "grataba": "gratar",
    "gratabais": "gratar",
    "grataban": "gratar",
    "gratabas": "gratar",
    "gratad": "gratar",
    "gratada": "gratar",
    "gratadas": "gratar",
    "gratado": "gratar",
    "gratados": "gratar",
    "gratamos": "gratar",
    "gratan": "gratar",
    "gratando": "gratar",
    "gratara": "gratar",
    "gratarais": "gratar",
    "grataran": "gratar",
    "grataras": "gratar",
    "gratare": "gratar",
    "gratareis": "gratar",
    "grataremos": "gratar",
    "grataren": "gratar",
    "gratares": "gratar",
    "grataron": "gratar",
    "gratará": "gratar",
    "gratarán": "gratar",
    "gratarás": "gratar",
    "grataré": "gratar",
    "grataréis": "gratar",
    "grataría": "gratar",
    "grataríais": "gratar",
    "grataríamos": "gratar",
    "gratarían": "gratar",
    "gratarías": "gratar",
    "gratas": "grato",
    "gratase": "gratar",
    "grataseis": "gratar",
    "gratasen": "gratar",
    "gratases": "gratar",
    "grataste": "gratar",
    "gratasteis": "gratar",
    "grate": "gratar",
    "gratemos": "gratar",
    "graten": "gratar",
    "gratenes": "gratén",
    "grates": "gratar",
    "gratifica": "gratificar",
    "gratificaba": "gratificar",
    "gratificabais": "gratificar",
    "gratificaban": "gratificar",
    "gratificabas": "gratificar",
    "gratificaciones": "gratificación",
    "gratificad": "gratificar",
    "gratificada": "gratificar",
    "gratificadas": "gratificar",
    "gratificado": "gratificar",
    "gratificadora": "gratificador",
    "gratificadoras": "gratificador",
    "gratificadores": "gratificador",
    "gratificados": "gratificar",
    "gratificamos": "gratificar",
    "gratifican": "gratificar",
    "gratificando": "gratificar",
    "gratificantes": "gratificante",
    "gratificara": "gratificar",
    "gratificarais": "gratificar",
    "gratificaran": "gratificar",
    "gratificaras": "gratificar",
    "gratificare": "gratificar",
    "gratificareis": "gratificar",
    "gratificaremos": "gratificar",
    "gratificaren": "gratificar",
    "gratificares": "gratificar",
    "gratificaron": "gratificar",
    "gratificará": "gratificar",
    "gratificarán": "gratificar",
    "gratificarás": "gratificar",
    "gratificaré": "gratificar",
    "gratificaréis": "gratificar",
    "gratificaría": "gratificar",
    "gratificaríais": "gratificar",
    "gratificaríamos": "gratificar",
    "gratificarían": "gratificar",
    "gratificarías": "gratificar",
    "gratificas": "gratificar",
    "gratificase": "gratificar",
    "gratificaseis": "gratificar",
    "gratificasen": "gratificar",
    "gratificases": "gratificar",
    "gratificaste": "gratificar",
    "gratificasteis": "gratificar",
    "gratifico": "gratificar",
    "gratificábamos": "gratificar",
    "gratificáis": "gratificar",
    "gratificáramos": "gratificar",
    "gratificáremos": "gratificar",
    "gratificásemos": "gratificar",
    "gratificó": "gratificar",
    "gratifique": "gratificar",
    "gratifiquemos": "gratificar",
    "gratifiquen": "gratificar",
    "gratifiques": "gratificar",
    "gratifiqué": "gratificar",
    "gratifiquéis": "gratificar",
    "gratiles": "gratil",
    "gratina": "gratinar",
    "gratinaba": "gratinar",
    "gratinabais": "gratinar",
    "gratinaban": "gratinar",
    "gratinabas": "gratinar",
    "gratinad": "gratinar",
    "gratinada": "gratinar",
    "gratinadas": "gratinar",
    "gratinado": "gratinar",
    "gratinados": "gratinar",
    "gratinamos": "gratinar",
    "gratinan": "gratinar",
    "gratinando": "gratinar",
    "gratinara": "gratinar",
    "gratinarais": "gratinar",
    "gratinaran": "gratinar",
    "gratinaras": "gratinar",
    "gratinare": "gratinar",
    "gratinareis": "gratinar",
    "gratinaremos": "gratinar",
    "gratinaren": "gratinar",
    "gratinares": "gratinar",
    "gratinaron": "gratinar",
    "gratinará": "gratinar",
    "gratinarán": "gratinar",
    "gratinarás": "gratinar",
    "gratinaré": "gratinar",
    "gratinaréis": "gratinar",
    "gratinaría": "gratinar",
    "gratinaríais": "gratinar",
    "gratinaríamos": "gratinar",
    "gratinarían": "gratinar",
    "gratinarías": "gratinar",
    "gratinas": "gratinar",
    "gratinase": "gratinar",
    "gratinaseis": "gratinar",
    "gratinasen": "gratinar",
    "gratinases": "gratinar",
    "gratinaste": "gratinar",
    "gratinasteis": "gratinar",
    "gratine": "gratinar",
    "gratinemos": "gratinar",
    "gratinen": "gratinar",
    "gratines": "gratinar",
    "gratino": "gratinar",
    "gratinábamos": "gratinar",
    "gratináis": "gratinar",
    "gratináramos": "gratinar",
    "gratináremos": "gratinar",
    "gratinásemos": "gratinar",
    "gratiné": "gratinar",
    "gratinéis": "gratinar",
    "gratinó": "gratinar",
    "gratitudes": "gratitud",
    "grato": "gratar",
    "gratos": "grato",
    "gratuidades": "gratuidad",
    "gratuita": "gratuito",
    "gratuitas": "gratuito",
    "gratuitos": "gratuito",
    "gratábamos": "gratar",
    "gratáis": "gratar",
    "gratáramos": "gratar",
    "gratáremos": "gratar",
    "gratásemos": "gratar",
    "graté": "gratar",
    "gratéis": "gratar",
    "grató": "gratar",
    "grauvacas": "grauvaca",
    "grava": "gravar",
    "gravaba": "gravar",
    "gravabais": "gravar",
    "gravaban": "gravar",
    "gravabas": "gravar",
    "gravables": "gravable",
    "gravaciones": "gravación",
    "gravad": "gravar",
    "gravada": "gravar",
    "gravadas": "gravar",
    "gravado": "gravar",
    "gravados": "gravar",
    "gravamos": "gravar",
    "gravan": "gravar",
    "gravando": "gravar",
    "gravara": "gravar",
    "gravarais": "gravar",
    "gravaran": "gravar",
    "gravaras": "gravar",
    "gravare": "gravar",
    "gravareis": "gravar",
    "gravaremos": "gravar",
    "gravaren": "gravar",
    "gravares": "gravar",
    "gravaron": "gravar",
    "gravará": "gravar",
    "gravarán": "gravar",
    "gravarás": "gravar",
    "gravaré": "gravar",
    "gravaréis": "gravar",
    "gravaría": "gravar",
    "gravaríais": "gravar",
    "gravaríamos": "gravar",
    "gravarían": "gravar",
    "gravarías": "gravar",
    "gravas": "gravar",
    "gravase": "gravar",
    "gravaseis": "gravar",
    "gravasen": "gravar",
    "gravases": "gravar",
    "gravaste": "gravar",
    "gravasteis": "gravar",
    "gravativa": "gravativo",
    "gravativas": "gravativo",
    "gravativos": "gravativo",
    "grave": "gravar",
    "gravedades": "gravedad",
    "gravemos": "gravar",
    "graven": "gravar",
    "graveras": "gravera",
    "graves": "grave",
    "gravicentros": "gravicentro",
    "gravideces": "gravidez",
    "gravillas": "gravilla",
    "gravimetrías": "gravimetría",
    "gravimétrica": "gravimétrico",
    "gravimétricas": "gravimétrico",
    "gravimétricos": "gravimétrico",
    "gravita": "gravitar",
    "gravitaba": "gravitar",
    "gravitabais": "gravitar",
    "gravitaban": "gravitar",
    "gravitabas": "gravitar",
    "gravitacionales": "gravitacional",
    "gravitaciones": "gravitación",
    "gravitad": "gravitar",
    "gravitada": "gravitar",
    "gravitadas": "gravitar",
    "gravitado": "gravitar",
    "gravitados": "gravitar",
    "gravitamos": "gravitar",
    "gravitan": "gravitar",
    "gravitando": "gravitar",
    "gravitara": "gravitar",
    "gravitarais": "gravitar",
    "gravitaran": "gravitar",
    "gravitaras": "gravitar",
    "gravitare": "gravitar",
    "gravitareis": "gravitar",
    "gravitaremos": "gravitar",
    "gravitaren": "gravitar",
    "gravitares": "gravitar",
    "gravitaron": "gravitar",
    "gravitará": "gravitar",
    "gravitarán": "gravitar",
    "gravitarás": "gravitar",
    "gravitaré": "gravitar",
    "gravitaréis": "gravitar",
    "gravitaría": "gravitar",
    "gravitaríais": "gravitar",
    "gravitaríamos": "gravitar",
    "gravitarían": "gravitar",
    "gravitarías": "gravitar",
    "gravitas": "gravitar",
    "gravitase": "gravitar",
    "gravitaseis": "gravitar",
    "gravitasen": "gravitar",
    "gravitases": "gravitar",
    "gravitaste": "gravitar",
    "gravitasteis": "gravitar",
    "gravitativa": "gravitativo",
    "gravitativas": "gravitativo",
    "gravitativos": "gravitativo",
    "gravitatoria": "gravitatorio",
    "gravitatorias": "gravitatorio",
    "gravitatorios": "gravitatorio",
    "gravite": "gravitar",
    "gravitemos": "gravitar",
    "graviten": "gravitar",
    "gravites": "gravitar",
    "gravito": "gravitar",
    "gravitones": "gravitón",
    "gravitábamos": "gravitar",
    "gravitáis": "gravitar",
    "gravitáramos": "gravitar",
    "gravitáremos": "gravitar",
    "gravitásemos": "gravitar",
    "gravité": "gravitar",
    "gravitéis": "gravitar",
    "gravitó": "gravitar",
    "gravo": "gravar",
    "gravosa": "gravoso",
    "gravosas": "gravoso",
    "gravosos": "gravoso",
    "gravábamos": "gravar",
    "graváis": "gravar",
    "gravámenes": "gravamen",
    "graváramos": "gravar",
    "graváremos": "gravar",
    "gravásemos": "gravar",
    "gravé": "gravar",
    "gravéis": "gravar",
    "gravídica": "gravídico",
    "gravídicas": "gravídico",
    "gravídicos": "gravídico",
    "gravímetros": "gravímetro",
    "gravó": "gravar",
    "grays": "gray",
    "grazna": "graznar",
    "graznaba": "graznar",
    "graznabais": "graznar",
    "graznaban": "graznar",
    "graznabas": "graznar",
    "graznad": "graznar",
    "graznada": "graznar",
    "graznadas": "graznar",
    "graznado": "graznar",
    "graznadora": "graznador",
    "graznadoras": "graznador",
    "graznadores": "graznador",
    "graznados": "graznar",
    "graznamos": "graznar",
    "graznan": "graznar",
    "graznando": "graznar",
    "graznara": "graznar",
    "graznarais": "graznar",
    "graznaran": "graznar",
    "graznaras": "graznar",
    "graznare": "graznar",
    "graznareis": "graznar",
    "graznaremos": "graznar",
    "graznaren": "graznar",
    "graznares": "graznar",
    "graznaron": "graznar",
    "graznará": "graznar",
    "graznarán": "graznar",
    "graznarás": "graznar",
    "graznaré": "graznar",
    "graznaréis": "graznar",
    "graznaría": "graznar",
    "graznaríais": "graznar",
    "graznaríamos": "graznar",
    "graznarían": "graznar",
    "graznarías": "graznar",
    "graznas": "graznar",
    "graznase": "graznar",
    "graznaseis": "graznar",
    "graznasen": "graznar",
    "graznases": "graznar",
    "graznaste": "graznar",
    "graznasteis": "graznar",
    "grazne": "graznar",
    "graznemos": "graznar",
    "graznen": "graznar",
    "graznes": "graznar",
    "graznidos": "graznido",
    "grazno": "graznar",
    "graznábamos": "graznar",
    "graznáis": "graznar",
    "graznáramos": "graznar",
    "graznáremos": "graznar",
    "graznásemos": "graznar",
    "grazné": "graznar",
    "graznéis": "graznar",
    "graznó": "graznar",
    "grañones": "grañón",
    "grañuelas": "grañuela",
    "grebas": "greba",
    "greca": "greco",
    "grecas": "greco",
    "grecice": "grecizar",
    "grecicemos": "grecizar",
    "grecicen": "grecizar",
    "grecices": "grecizar",
    "grecicé": "grecizar",
    "grecicéis": "grecizar",
    "grecismos": "grecismo",
    "greciza": "grecizar",
    "grecizaba": "grecizar",
    "grecizabais": "grecizar",
    "grecizaban": "grecizar",
    "grecizabas": "grecizar",
    "grecizad": "grecizar",
    "grecizada": "grecizar",
    "grecizadas": "grecizar",
    "grecizado": "grecizar",
    "grecizados": "grecizar",
    "grecizamos": "grecizar",
    "grecizan": "grecizar",
    "grecizando": "grecizar",
    "grecizara": "grecizar",
    "grecizarais": "grecizar",
    "grecizaran": "grecizar",
    "grecizaras": "grecizar",
    "grecizare": "grecizar",
    "grecizareis": "grecizar",
    "grecizaremos": "grecizar",
    "grecizaren": "grecizar",
    "grecizares": "grecizar",
    "grecizaron": "grecizar",
    "grecizará": "grecizar",
    "grecizarán": "grecizar",
    "grecizarás": "grecizar",
    "grecizaré": "grecizar",
    "grecizaréis": "grecizar",
    "grecizaría": "grecizar",
    "grecizaríais": "grecizar",
    "grecizaríamos": "grecizar",
    "grecizarían": "grecizar",
    "grecizarías": "grecizar",
    "grecizas": "grecizar",
    "grecizase": "grecizar",
    "grecizaseis": "grecizar",
    "grecizasen": "grecizar",
    "grecizases": "grecizar",
    "grecizaste": "grecizar",
    "grecizasteis": "grecizar",
    "grecizo": "grecizar",
    "grecizábamos": "grecizar",
    "grecizáis": "grecizar",
    "grecizáramos": "grecizar",
    "grecizáremos": "grecizar",
    "grecizásemos": "grecizar",
    "grecizó": "grecizar",
    "grecolatina": "grecolatino",
    "grecolatinas": "grecolatino",
    "grecolatinos": "grecolatino",
    "grecorromana": "grecorromano",
    "grecorromanas": "grecorromano",
    "grecorromanos": "grecorromano",
    "grecos": "greco",
    "gredales": "gredal",
    "gredas": "greda",
    "gredosa": "gredoso",
    "gredosas": "gredoso",
    "gredosos": "gredoso",
    "greenockitas": "greenockita",
    "greenoquitas": "greenoquita",
    "gregales": "gregal",
    "gregaria": "gregario",
    "gregarias": "gregario",
    "gregarinas": "gregarina",
    "gregarios": "gregario",
    "gregarismos": "gregarismo",
    "gregoriana": "gregoriano",
    "gregorianas": "gregoriano",
    "gregorianos": "gregoriano",
    "greguerías": "greguería",
    "grelos": "grelo",
    "gremiales": "gremial",
    "gremialismos": "gremialismo",
    "gremialistas": "gremialista",
    "gremios": "gremio",
    "grenchuda": "grenchudo",
    "grenchudas": "grenchudo",
    "grenchudos": "grenchudo",
    "grescas": "gresca",
    "grevillos": "grevillo",
    "greys": "grey",
    "greñas": "greña",
    "greñuda": "greñudo",
    "greñudas": "greñudo",
    "greñudos": "greñudo",
    "griales": "grial",
    "griega": "griego",
    "griegas": "griego",
    "griegos": "griego",
    "grieta": "grietar",
    "grietaba": "grietar",
    "grietabais": "grietar",
    "grietaban": "grietar",
    "grietabas": "grietar",
    "grietad": "grietar",
    "grietada": "grietar",
    "grietadas": "grietar",
    "grietado": "grietar",
    "grietados": "grietar",
    "grietamos": "grietar",
    "grietan": "grietar",
    "grietando": "grietar",
    "grietara": "grietar",
    "grietarais": "grietar",
    "grietaran": "grietar",
    "grietaras": "grietar",
    "grietare": "grietar",
    "grietareis": "grietar",
    "grietaremos": "grietar",
    "grietaren": "grietar",
    "grietares": "grietar",
    "grietaron": "grietar",
    "grietará": "grietar",
    "grietarán": "grietar",
    "grietarás": "grietar",
    "grietaré": "grietar",
    "grietaréis": "grietar",
    "grietaría": "grietar",
    "grietaríais": "grietar",
    "grietaríamos": "grietar",
    "grietarían": "grietar",
    "grietarías": "grietar",
    "grietas": "grietar",
    "grietase": "grietar",
    "grietaseis": "grietar",
    "grietasen": "grietar",
    "grietases": "grietar",
    "grietaste": "grietar",
    "grietasteis": "grietar",
    "griete": "grietar",
    "grietemos": "grietar",
    "grieten": "grietar",
    "grietes": "grietar",
    "grieto": "grietar",
    "grietosa": "grietoso",
    "grietosas": "grietoso",
    "grietosos": "grietoso",
    "grietábamos": "grietar",
    "grietáis": "grietar",
    "grietáramos": "grietar",
    "grietáremos": "grietar",
    "grietásemos": "grietar",
    "grieté": "grietar",
    "grietéis": "grietar",
    "grietó": "grietar",
    "grifa": "grifo",
    "grifaltos": "grifalto",
    "grifas": "grifo",
    "griferías": "grifería",
    "grifones": "grifón",
    "grifos": "grifo",
    "grilla": "grillar",
    "grillaba": "grillar",
    "grillabais": "grillar",
    "grillaban": "grillar",
    "grillabas": "grillar",
    "grillad": "grillar",
    "grillada": "grillar",
    "grilladas": "grillar",
    "grillado": "grillar",
    "grillados": "grillar",
    "grillamos": "grillar",
    "grillan": "grillar",
    "grillando": "grillar",
    "grillara": "grillar",
    "grillarais": "grillar",
    "grillaran": "grillar",
    "grillaras": "grillar",
    "grillare": "grillar",
    "grillareis": "grillar",
    "grillaremos": "grillar",
    "grillaren": "grillar",
    "grillares": "grillar",
    "grillaron": "grillar",
    "grillará": "grillar",
    "grillarán": "grillar",
    "grillarás": "grillar",
    "grillaré": "grillar",
    "grillaréis": "grillar",
    "grillaría": "grillar",
    "grillaríais": "grillar",
    "grillaríamos": "grillar",
    "grillarían": "grillar",
    "grillarías": "grillar",
    "grillas": "grillar",
    "grillase": "grillar",
    "grillaseis": "grillar",
    "grillasen": "grillar",
    "grillases": "grillar",
    "grillaste": "grillar",
    "grillasteis": "grillar",
    "grille": "grillar",
    "grillemos": "grillar",
    "grillen": "grillar",
    "grilleras": "grillera",
    "grilles": "grillar",
    "grilletas": "grilleta",
    "grilletes": "grillete",
    "grillo": "grillar",
    "grillos": "grillo",
    "grills": "grill",
    "grillábamos": "grillar",
    "grilláis": "grillar",
    "grilláramos": "grillar",
    "grilláremos": "grillar",
    "grillásemos": "grillar",
    "grillé": "grillar",
    "grilléis": "grillar",
    "grilló": "grillar",
    "grimas": "grima",
    "grimosa": "grimoso",
    "grimosas": "grimoso",
    "grimosos": "grimoso",
    "grindelias": "grindelia",
    "gringa": "gringo",
    "gringas": "gringo",
    "gringos": "gringo",
    "gringueles": "gringuele",
    "gripados": "gripado",
    "gripajes": "gripaje",
    "gripales": "gripal",
    "gripes": "gripe",
    "griposa": "griposo",
    "griposas": "griposo",
    "griposos": "griposo",
    "grisallas": "grisalla",
    "grises": "gris",
    "grisetas": "griseta",
    "grisgrises": "grisgrís",
    "grisosa": "grisoso",
    "grisosas": "grisoso",
    "grisosos": "grisoso",
    "grisutosa": "grisutoso",
    "grisutosas": "grisutoso",
    "grisutosos": "grisutoso",
    "grisácea": "grisáceo",
    "grisáceas": "grisáceo",
    "grisáceos": "grisáceo",
    "grisúes": "grisú",
    "grisús": "grisú",
    "grita": "gritar",
    "gritaba": "gritar",
    "gritabais": "gritar",
    "gritaban": "gritar",
    "gritabas": "gritar",
    "gritad": "gritar",
    "gritada": "gritar",
    "gritadas": "gritar",
    "gritado": "gritar",
    "gritadora": "gritador",
    "gritadoras": "gritador",
    "gritadores": "gritador",
    "gritados": "gritar",
    "gritamos": "gritar",
    "gritan": "gritar",
    "gritando": "gritar",
    "gritara": "gritar",
    "gritarais": "gritar",
    "gritaran": "gritar",
    "gritaras": "gritar",
    "gritare": "gritar",
    "gritareis": "gritar",
    "gritaremos": "gritar",
    "gritaren": "gritar",
    "gritares": "gritar",
    "gritaron": "gritar",
    "gritará": "gritar",
    "gritarán": "gritar",
    "gritarás": "gritar",
    "gritaré": "gritar",
    "gritaréis": "gritar",
    "gritaría": "gritar",
    "gritaríais": "gritar",
    "gritaríamos": "gritar",
    "gritarían": "gritar",
    "gritarías": "gritar",
    "gritas": "gritar",
    "gritase": "gritar",
    "gritaseis": "gritar",
    "gritasen": "gritar",
    "gritases": "gritar",
    "gritaste": "gritar",
    "gritasteis": "gritar",
    "grite": "gritar",
    "gritemos": "gritar",
    "griten": "gritar",
    "griterías": "gritería",
    "griteríos": "griterío",
    "grites": "gritar",
    "grito": "gritar",
    "gritona": "gritón",
    "gritonas": "gritón",
    "gritones": "gritón",
    "gritos": "grito",
    "gritábamos": "gritar",
    "gritáis": "gritar",
    "gritáramos": "gritar",
    "gritáremos": "gritar",
    "gritásemos": "gritar",
    "grité": "gritar",
    "gritéis": "gritar",
    "gritó": "gritar",
    "grivias": "grivia",
    "griñoleras": "griñolera",
    "griñones": "griñón",
    "groenlandesa": "groenlandés",
    "groenlandesas": "groenlandés",
    "groenlandeses": "groenlandés",
    "groenlandésa": "groenlandés",
    "groenlandésas": "groenlandés",
    "groenlandéses": "groenlandés",
    "groeras": "groera",
    "groggys": "groggy",
    "grogs": "grog",
    "groguis": "grogui",
    "gromos": "gromo",
    "gros": "gro",
    "grosa": "groso",
    "grosarénica": "grosarénico",
    "grosarénicas": "grosarénico",
    "grosarénicos": "grosarénico",
    "grosas": "groso",
    "grosellas": "grosella",
    "groselleros": "grosellero",
    "grosera": "grosero",
    "groseras": "grosero",
    "groseros": "grosero",
    "groserías": "grosería",
    "grosores": "grosor",
    "grosos": "groso",
    "grosularias": "grosularia",
    "grosuras": "grosura",
    "grotesca": "grotesco",
    "grotescas": "grotesco",
    "grotescos": "grotesco",
    "gruesa": "grueso",
    "gruesas": "grueso",
    "gruesos": "grueso",
    "gruid": "gruir",
    "gruida": "gruir",
    "gruidas": "gruir",
    "gruido": "gruir",
    "gruidos": "gruir",
    "gruimos": "gruir",
    "gruiremos": "gruir",
    "gruirá": "gruir",
    "gruirán": "gruir",
    "gruirás": "gruir",
    "gruiré": "gruir",
    "gruiréis": "gruir",
    "gruiría": "gruir",
    "gruiríais": "gruir",
    "gruiríamos": "gruir",
    "gruirían": "gruir",
    "gruirías": "gruir",
    "gruiste": "gruir",
    "gruisteis": "gruir",
    "gruja": "grujir",
    "grujamos": "grujir",
    "grujan": "grujir",
    "grujas": "grujir",
    "gruje": "grujir",
    "grujen": "grujir",
    "grujes": "grujir",
    "grujid": "grujir",
    "grujida": "grujir",
    "grujidas": "grujir",
    "grujido": "grujir",
    "grujidores": "grujidor",
    "grujidos": "grujir",
    "grujiendo": "grujir",
    "grujiera": "grujir",
    "grujierais": "grujir",
    "grujieran": "grujir",
    "grujieras": "grujir",
    "grujiere": "grujir",
    "grujiereis": "grujir",
    "grujieren": "grujir",
    "grujieres": "grujir",
    "grujieron": "grujir",
    "grujiese": "grujir",
    "grujieseis": "grujir",
    "grujiesen": "grujir",
    "grujieses": "grujir",
    "grujimos": "grujir",
    "grujiremos": "grujir",
    "grujirá": "grujir",
    "grujirán": "grujir",
    "grujirás": "grujir",
    "grujiré": "grujir",
    "grujiréis": "grujir",
    "grujiría": "grujir",
    "grujiríais": "grujir",
    "grujiríamos": "grujir",
    "grujirían": "grujir",
    "grujirías": "grujir",
    "grujiste": "grujir",
    "grujisteis": "grujir",
    "grujiéramos": "grujir",
    "grujiéremos": "grujir",
    "grujiésemos": "grujir",
    "grujió": "grujir",
    "grujo": "grujir",
    "grujáis": "grujir",
    "grují": "grujir",
    "grujía": "grujir",
    "grujíais": "grujir",
    "grujíamos": "grujir",
    "grujían": "grujir",
    "grujías": "grujir",
    "grujís": "grujir",
    "grulladas": "grullada",
    "grullas": "grulla",
    "grullera": "grullero",
    "grulleras": "grullero",
    "grulleros": "grullero",
    "grumetes": "grumete",
    "grumos": "grumo",
    "grumosa": "grumoso",
    "grumosas": "grumoso",
    "grumosos": "grumoso",
    "grupadas": "grupada",
    "grupales": "grupal",
    "grupas": "grupa",
    "gruperas": "grupera",
    "grupos": "grupo",
    "grupúsculos": "grupúsculo",
    "grutas": "gruta",
    "grutesca": "grutesco",
    "grutescas": "grutesco",
    "grutescos": "grutesco",
    "gruya": "gruir",
    "gruyamos": "gruir",
    "gruyan": "gruir",
    "gruyas": "gruir",
    "gruye": "gruir",
    "gruyen": "gruir",
    "gruyendo": "gruir",
    "gruyera": "gruir",
    "gruyerais": "gruir",
    "gruyeran": "gruir",
    "gruyeras": "gruir",
    "gruyere": "gruir",
    "gruyereis": "gruir",
    "gruyeren": "gruir",
    "gruyeres": "gruyere",
    "gruyeron": "gruir",
    "gruyes": "gruir",
    "gruyese": "gruir",
    "gruyeseis": "gruir",
    "gruyesen": "gruir",
    "gruyeses": "gruir",
    "gruyo": "gruir",
    "gruyáis": "gruir",
    "gruyéramos": "gruir",
    "gruyéremos": "gruir",
    "gruyésemos": "gruir",
    "gruyó": "gruir",
    "gruí": "gruir",
    "gruía": "gruir",
    "gruíais": "gruir",
    "gruíamos": "gruir",
    "gruían": "gruir",
    "gruías": "gruir",
    "gruís": "gruir",
    "gruña": "gruñir",
    "gruñamos": "gruñir",
    "gruñan": "gruñir",
    "gruñas": "gruñir",
    "gruñe": "gruñir",
    "gruñen": "gruñir",
    "gruñendo": "gruñir",
    "gruñera": "gruñir",
    "gruñerais": "gruñir",
    "gruñeran": "gruñir",
    "gruñeras": "gruñir",
    "gruñere": "gruñir",
    "gruñereis": "gruñir",
    "gruñeren": "gruñir",
    "gruñeres": "gruñir",
    "gruñeron": "gruñir",
    "gruñes": "gruñir",
    "gruñese": "gruñir",
    "gruñeseis": "gruñir",
    "gruñesen": "gruñir",
    "gruñeses": "gruñir",
    "gruñid": "gruñir",
    "gruñida": "gruñir",
    "gruñidas": "gruñir",
    "gruñido": "gruñir",
    "gruñidora": "gruñidor",
    "gruñidoras": "gruñidor",
    "gruñidores": "gruñidor",
    "gruñidos": "gruñir",
    "gruñimientos": "gruñimiento",
    "gruñimos": "gruñir",
    "gruñiremos": "gruñir",
    "gruñirá": "gruñir",
    "gruñirán": "gruñir",
    "gruñirás": "gruñir",
    "gruñiré": "gruñir",
    "gruñiréis": "gruñir",
    "gruñiría": "gruñir",
    "gruñiríais": "gruñir",
    "gruñiríamos": "gruñir",
    "gruñirían": "gruñir",
    "gruñirías": "gruñir",
    "gruñiste": "gruñir",
    "gruñisteis": "gruñir",
    "gruño": "gruñir",
    "gruñona": "gruñón",
    "gruñonas": "gruñón",
    "gruñones": "gruñón",
    "gruñáis": "gruñir",
    "gruñéramos": "gruñir",
    "gruñéremos": "gruñir",
    "gruñésemos": "gruñir",
    "gruñí": "gruñir",
    "gruñía": "gruñir",
    "gruñíais": "gruñir",
    "gruñíamos": "gruñir",
    "gruñían": "gruñir",
    "gruñías": "gruñir",
    "gruñís": "gruñir",
    "gruñó": "gruñir",
    "gráciles": "grácil",
    "gráfica": "gráfico",
    "gráficas": "gráfico",
    "gráficos": "gráfico",
    "gráfilas": "gráfila",
    "gránulos": "gránulo",
    "grátiles": "grátil",
    "grávida": "grávido",
    "grávidas": "grávido",
    "grávidos": "grávido",
    "grévoles": "grévol",
    "grímpolas": "grímpola",
    "grúas": "grúa",
    "guabanes": "guabán",
    "guabicos": "guabico",
    "guabirás": "guabirá",
    "guacalotes": "guacalote",
    "guacamayas": "guacamaya",
    "guacamayos": "guacamayo",
    "guacamoles": "guacamole",
    "guacamotes": "guacamote",
    "guacas": "guaca",
    "guacha": "guacho",
    "guachamazos": "guachamazo",
    "guachapazos": "guachapazo",
    "guachapea": "guachapear",
    "guachapeaba": "guachapear",
    "guachapeabais": "guachapear",
    "guachapeaban": "guachapear",
    "guachapeabas": "guachapear",
    "guachapead": "guachapear",
    "guachapeada": "guachapear",
    "guachapeadas": "guachapear",
    "guachapeado": "guachapear",
    "guachapeados": "guachapear",
    "guachapeamos": "guachapear",
    "guachapean": "guachapear",
    "guachapeando": "guachapear",
    "guachapeara": "guachapear",
    "guachapearais": "guachapear",
    "guachapearan": "guachapear",
    "guachapearas": "guachapear",
    "guachapeare": "guachapear",
    "guachapeareis": "guachapear",
    "guachapearemos": "guachapear",
    "guachapearen": "guachapear",
    "guachapeares": "guachapear",
    "guachapearon": "guachapear",
    "guachapeará": "guachapear",
    "guachapearán": "guachapear",
    "guachapearás": "guachapear",
    "guachapearé": "guachapear",
    "guachapearéis": "guachapear",
    "guachapearía": "guachapear",
    "guachapearíais": "guachapear",
    "guachapearíamos": "guachapear",
    "guachapearían": "guachapear",
    "guachapearías": "guachapear",
    "guachapeas": "guachapear",
    "guachapease": "guachapear",
    "guachapeaseis": "guachapear",
    "guachapeasen": "guachapear",
    "guachapeases": "guachapear",
    "guachapeaste": "guachapear",
    "guachapeasteis": "guachapear",
    "guachapee": "guachapear",
    "guachapeemos": "guachapear",
    "guachapeen": "guachapear",
    "guachapees": "guachapear",
    "guachapeo": "guachapear",
    "guachapeábamos": "guachapear",
    "guachapeáis": "guachapear",
    "guachapeáramos": "guachapear",
    "guachapeáremos": "guachapear",
    "guachapeásemos": "guachapear",
    "guachapeé": "guachapear",
    "guachapeéis": "guachapear",
    "guachapeó": "guachapear",
    "guacharradas": "guacharrada",
    "guachas": "guacho",
    "guaches": "guache",
    "guachinanga": "guachinango",
    "guachinangas": "guachinango",
    "guachinangos": "guachinango",
    "guachos": "guacho",
    "guacias": "guacia",
    "guacimillas": "guacimilla",
    "guacos": "guaco",
    "guadalajarenses": "guadalajarense",
    "guadalajareña": "guadalajareño",
    "guadalajareñas": "guadalajareño",
    "guadalajareños": "guadalajareño",
    "guadales": "guadal",
    "guadameciles": "guadamecil",
    "guadamecíes": "guadamecí",
    "guadamecís": "guadamecí",
    "guadaperos": "guadapero",
    "guadarnesa": "guadarnés",
    "guadarnesas": "guadarnés",
    "guadarneses": "guadarnés",
    "guadaña": "guadañar",
    "guadañaba": "guadañar",
    "guadañabais": "guadañar",
    "guadañaban": "guadañar",
    "guadañabas": "guadañar",
    "guadañad": "guadañar",
    "guadañada": "guadañar",
    "guadañadas": "guadañar",
    "guadañado": "guadañar",
    "guadañadora": "guadañador",
    "guadañadoras": "guadañador",
    "guadañadores": "guadañador",
    "guadañados": "guadañar",
    "guadañamos": "guadañar",
    "guadañan": "guadañar",
    "guadañando": "guadañar",
    "guadañara": "guadañar",
    "guadañarais": "guadañar",
    "guadañaran": "guadañar",
    "guadañaras": "guadañar",
    "guadañare": "guadañar",
    "guadañareis": "guadañar",
    "guadañaremos": "guadañar",
    "guadañaren": "guadañar",
    "guadañares": "guadañar",
    "guadañaron": "guadañar",
    "guadañará": "guadañar",
    "guadañarán": "guadañar",
    "guadañarás": "guadañar",
    "guadañaré": "guadañar",
    "guadañaréis": "guadañar",
    "guadañaría": "guadañar",
    "guadañaríais": "guadañar",
    "guadañaríamos": "guadañar",
    "guadañarían": "guadañar",
    "guadañarías": "guadañar",
    "guadañas": "guadañar",
    "guadañase": "guadañar",
    "guadañaseis": "guadañar",
    "guadañasen": "guadañar",
    "guadañases": "guadañar",
    "guadañaste": "guadañar",
    "guadañasteis": "guadañar",
    "guadañe": "guadañar",
    "guadañemos": "guadañar",
    "guadañen": "guadañar",
    "guadañera": "guadañero",
    "guadañeras": "guadañero",
    "guadañeros": "guadañero",
    "guadañes": "guadañar",
    "guadañiles": "guadañil",
    "guadaño": "guadañar",
    "guadañábamos": "guadañar",
    "guadañáis": "guadañar",
    "guadañáramos": "guadañar",
    "guadañáremos": "guadañar",
    "guadañásemos": "guadañar",
    "guadañé": "guadañar",
    "guadañéis": "guadañar",
    "guadañó": "guadañar",
    "guadianesa": "guadianés",
    "guadianesas": "guadianés",
    "guadianeses": "guadianés",
    "guadijeña": "guadijeño",
    "guadijeñas": "guadijeño",
    "guadijeños": "guadijeño",
    "guaduas": "guadua",
    "guaguas": "guagua",
    "guaicána": "guaicán",
    "guaicánas": "guaicán",
    "guaicánes": "guaicán",
    "guaimíses": "guaimí",
    "guairas": "guaira",
    "guairos": "guairo",
    "guajacas": "guajaca",
    "guajares": "guajar",
    "guajas": "guaja",
    "guajira": "guajiro",
    "guajiras": "guajiro",
    "guajiros": "guajiro",
    "guajolotes": "guajolote",
    "gualandayes": "gualanday",
    "gualandays": "gualanday",
    "gualatinas": "gualatina",
    "gualda": "gualdo",
    "gualdada": "gualdado",
    "gualdadas": "gualdado",
    "gualdados": "gualdado",
    "gualdas": "gualdo",
    "gualderas": "gualdera",
    "gualdos": "gualdo",
    "gualdrapas": "gualdrapa",
    "gualdrapazos": "gualdrapazo",
    "gualdrapea": "gualdrapear",
    "gualdrapeaba": "gualdrapear",
    "gualdrapeabais": "gualdrapear",
    "gualdrapeaban": "gualdrapear",
    "gualdrapeabas": "gualdrapear",
    "gualdrapead": "gualdrapear",
    "gualdrapeada": "gualdrapear",
    "gualdrapeadas": "gualdrapear",
    "gualdrapeado": "gualdrapear",
    "gualdrapeados": "gualdrapear",
    "gualdrapeamos": "gualdrapear",
    "gualdrapean": "gualdrapear",
    "gualdrapeando": "gualdrapear",
    "gualdrapeara": "gualdrapear",
    "gualdrapearais": "gualdrapear",
    "gualdrapearan": "gualdrapear",
    "gualdrapearas": "gualdrapear",
    "gualdrapeare": "gualdrapear",
    "gualdrapeareis": "gualdrapear",
    "gualdrapearemos": "gualdrapear",
    "gualdrapearen": "gualdrapear",
    "gualdrapeares": "gualdrapear",
    "gualdrapearon": "gualdrapear",
    "gualdrapeará": "gualdrapear",
    "gualdrapearán": "gualdrapear",
    "gualdrapearás": "gualdrapear",
    "gualdrapearé": "gualdrapear",
    "gualdrapearéis": "gualdrapear",
    "gualdrapearía": "gualdrapear",
    "gualdrapearíais": "gualdrapear",
    "gualdrapearíamos": "gualdrapear",
    "gualdrapearían": "gualdrapear",
    "gualdrapearías": "gualdrapear",
    "gualdrapeas": "gualdrapear",
    "gualdrapease": "gualdrapear",
    "gualdrapeaseis": "gualdrapear",
    "gualdrapeasen": "gualdrapear",
    "gualdrapeases": "gualdrapear",
    "gualdrapeaste": "gualdrapear",
    "gualdrapeasteis": "gualdrapear",
    "gualdrapee": "gualdrapear",
    "gualdrapeemos": "gualdrapear",
    "gualdrapeen": "gualdrapear",
    "gualdrapees": "gualdrapear",
    "gualdrapeo": "gualdrapear",
    "gualdrapeos": "gualdrapeo",
    "gualdraperos": "gualdrapero",
    "gualdrapeábamos": "gualdrapear",
    "gualdrapeáis": "gualdrapear",
    "gualdrapeáramos": "gualdrapear",
    "gualdrapeáremos": "gualdrapear",
    "gualdrapeásemos": "gualdrapear",
    "gualdrapeé": "gualdrapear",
    "gualdrapeéis": "gualdrapear",
    "gualdrapeó": "gualdrapear",
    "gualillas": "gualilla",
    "guamos": "guamo",
    "guanaca": "guanaco",
    "guanacahíbes": "guanacahíbe",
    "guanacas": "guanaco",
    "guanacastes": "guanacaste",
    "guanacos": "guanaco",
    "guanahatabeyes": "guanahatabeye",
    "guanajos": "guanajo",
    "guanches": "guanche",
    "guanera": "guanero",
    "guaneras": "guanero",
    "guaneros": "guanero",
    "guanidinas": "guanidina",
    "guaninas": "guanina",
    "guanines": "guanín",
    "guanos": "guano",
    "guanosinas": "guanosina",
    "guantadas": "guantada",
    "guantanamera": "guantanamero",
    "guantanameras": "guantanamero",
    "guantanameros": "guantanamero",
    "guantazos": "guantazo",
    "guantea": "guantear",
    "guanteaba": "guantear",
    "guanteabais": "guantear",
    "guanteaban": "guantear",
    "guanteabas": "guantear",
    "guantead": "guantear",
    "guanteada": "guantear",
    "guanteadas": "guantear",
    "guanteado": "guantear",
    "guanteados": "guantear",
    "guanteamos": "guantear",
    "guantean": "guantear",
    "guanteando": "guantear",
    "guanteara": "guantear",
    "guantearais": "guantear",
    "guantearan": "guantear",
    "guantearas": "guantear",
    "guanteare": "guantear",
    "guanteareis": "guantear",
    "guantearemos": "guantear",
    "guantearen": "guantear",
    "guanteares": "guantear",
    "guantearon": "guantear",
    "guanteará": "guantear",
    "guantearán": "guantear",
    "guantearás": "guantear",
    "guantearé": "guantear",
    "guantearéis": "guantear",
    "guantearía": "guantear",
    "guantearíais": "guantear",
    "guantearíamos": "guantear",
    "guantearían": "guantear",
    "guantearías": "guantear",
    "guanteas": "guantear",
    "guantease": "guantear",
    "guanteaseis": "guantear",
    "guanteasen": "guantear",
    "guanteases": "guantear",
    "guanteaste": "guantear",
    "guanteasteis": "guantear",
    "guantee": "guantear",
    "guanteemos": "guantear",
    "guanteen": "guantear",
    "guantees": "guantear",
    "guanteletes": "guantelete",
    "guanteo": "guantear",
    "guanteras": "guantera",
    "guanteros": "guantero",
    "guanterías": "guantería",
    "guantes": "guante",
    "guanteábamos": "guantear",
    "guanteáis": "guantear",
    "guanteáramos": "guantear",
    "guanteáremos": "guantear",
    "guanteásemos": "guantear",
    "guanteé": "guantear",
    "guanteéis": "guantear",
    "guanteó": "guantear",
    "guanábanas": "guanábana",
    "guanábanos": "guanábano",
    "guanílica": "guanílico",
    "guanílicas": "guanílico",
    "guanílicos": "guanílico",
    "guapa": "guapo",
    "guapas": "guapo",
    "guapea": "guapear",
    "guapeaba": "guapear",
    "guapeabais": "guapear",
    "guapeaban": "guapear",
    "guapeabas": "guapear",
    "guapead": "guapear",
    "guapeada": "guapear",
    "guapeadas": "guapear",
    "guapeado": "guapear",
    "guapeados": "guapear",
    "guapeamos": "guapear",
    "guapean": "guapear",
    "guapeando": "guapear",
    "guapeara": "guapear",
    "guapearais": "guapear",
    "guapearan": "guapear",
    "guapearas": "guapear",
    "guapeare": "guapear",
    "guapeareis": "guapear",
    "guapearemos": "guapear",
    "guapearen": "guapear",
    "guapeares": "guapear",
    "guapearon": "guapear",
    "guapeará": "guapear",
    "guapearán": "guapear",
    "guapearás": "guapear",
    "guapearé": "guapear",
    "guapearéis": "guapear",
    "guapearía": "guapear",
    "guapearíais": "guapear",
    "guapearíamos": "guapear",
    "guapearían": "guapear",
    "guapearías": "guapear",
    "guapeas": "guapear",
    "guapease": "guapear",
    "guapeaseis": "guapear",
    "guapeasen": "guapear",
    "guapeases": "guapear",
    "guapeaste": "guapear",
    "guapeasteis": "guapear",
    "guapee": "guapear",
    "guapeemos": "guapear",
    "guapeen": "guapear",
    "guapees": "guapear",
    "guapeo": "guapear",
    "guaperías": "guapería",
    "guapezas": "guapeza",
    "guapeábamos": "guapear",
    "guapeáis": "guapear",
    "guapeáramos": "guapear",
    "guapeáremos": "guapear",
    "guapeásemos": "guapear",
    "guapeé": "guapear",
    "guapeéis": "guapear",
    "guapeó": "guapear",
    "guapos": "guapo",
    "guapota": "guapote",
    "guapotas": "guapote",
    "guapotes": "guapote",
    "guapuras": "guapura",
    "guaracaros": "guaracaro",
    "guaraches": "guarache",
    "guaranga": "guarango",
    "guarangas": "guarango",
    "guarangos": "guarango",
    "guaraníes": "guaraní",
    "guaranís": "guaraní",
    "guaraníses": "guaraní",
    "guarapa": "guarapo",
    "guarapas": "guarapo",
    "guarapos": "guarapo",
    "guarda": "guardar",
    "guardaba": "guardar",
    "guardabais": "guardar",
    "guardaban": "guardar",
    "guardabarreras": "guardabarrera",
    "guardabas": "guardar",
    "guardabosques": "guardabosque",
    "guardabrazos": "guardabrazo",
    "guardabrisas": "guardabrisa",
    "guardacantones": "guardacantón",
    "guardad": "guardar",
    "guardada": "guardar",
    "guardadas": "guardar",
    "guardado": "guardar",
    "guardadora": "guardador",
    "guardadoras": "guardador",
    "guardadores": "guardador",
    "guardados": "guardar",
    "guardadurías": "guardaduría",
    "guardafuegos": "guardafuego",
    "guardahúmos": "guardahúmo",
    "guardainfantes": "guardainfante",
    "guardalados": "guardalado",
    "guardalmacenes": "guardalmacén",
    "guardalobos": "guardalobo",
    "guardamalletas": "guardamalleta",
    "guardamangeles": "guardamangel",
    "guardamangieres": "guardamangier",
    "guardamanos": "guardamano",
    "guardamarinas": "guardamarina",
    "guardametas": "guardameta",
    "guardamigos": "guardamigo",
    "guardamontes": "guardamonte",
    "guardamos": "guardar",
    "guardamujeres": "guardamujer",
    "guardan": "guardar",
    "guardando": "guardar",
    "guardapapos": "guardapapo",
    "guardapelos": "guardapelo",
    "guardapescas": "guardapesca",
    "guardapolvos": "guardapolvo",
    "guardapuertas": "guardapuerta",
    "guardara": "guardar",
    "guardarais": "guardar",
    "guardaran": "guardar",
    "guardaras": "guardar",
    "guardare": "guardar",
    "guardareis": "guardar",
    "guardaremos": "guardar",
    "guardaren": "guardar",
    "guardares": "guardar",
    "guardaron": "guardar",
    "guardarropas": "guardarropa",
    "guardarropías": "guardarropía",
    "guardará": "guardar",
    "guardarán": "guardar",
    "guardarás": "guardar",
    "guardaré": "guardar",
    "guardaréis": "guardar",
    "guardaría": "guardar",
    "guardaríais": "guardar",
    "guardaríamos": "guardar",
    "guardarían": "guardar",
    "guardarías": "guardar",
    "guardas": "guardar",
    "guardase": "guardar",
    "guardaseis": "guardar",
    "guardasen": "guardar",
    "guardases": "guardar",
    "guardasillas": "guardasilla",
    "guardasoles": "guardasol",
    "guardaste": "guardar",
    "guardasteis": "guardar",
    "guardatimones": "guardatimón",
    "guardavelas": "guardavela",
    "guardavías": "guardavía",
    "guarde": "guardar",
    "guardemos": "guardar",
    "guarden": "guardar",
    "guarderías": "guardería",
    "guardes": "guardar",
    "guardesas": "guardesa",
    "guardiana": "guardián",
    "guardianas": "guardián",
    "guardianes": "guardián",
    "guardias": "guardia",
    "guardillas": "guardilla",
    "guardillones": "guardillón",
    "guardines": "guardín",
    "guardo": "guardar",
    "guardona": "guardón",
    "guardonas": "guardón",
    "guardones": "guardón",
    "guardosa": "guardoso",
    "guardosas": "guardoso",
    "guardosos": "guardoso",
    "guardábamos": "guardar",
    "guardáis": "guardar",
    "guardáramos": "guardar",
    "guardáremos": "guardar",
    "guardásemos": "guardar",
    "guardé": "guardar",
    "guardéis": "guardar",
    "guardó": "guardar",
    "guarece": "guarecer",
    "guareced": "guarecer",
    "guarecemos": "guarecer",
    "guarecen": "guarecer",
    "guareceremos": "guarecer",
    "guarecerá": "guarecer",
    "guarecerán": "guarecer",
    "guarecerás": "guarecer",
    "guareceré": "guarecer",
    "guareceréis": "guarecer",
    "guarecería": "guarecer",
    "guareceríais": "guarecer",
    "guareceríamos": "guarecer",
    "guarecerían": "guarecer",
    "guarecerías": "guarecer",
    "guareces": "guarecer",
    "guarecida": "guarecer",
    "guarecidas": "guarecer",
    "guarecido": "guarecer",
    "guarecidos": "guarecer",
    "guareciendo": "guarecer",
    "guareciera": "guarecer",
    "guarecierais": "guarecer",
    "guarecieran": "guarecer",
    "guarecieras": "guarecer",
    "guareciere": "guarecer",
    "guareciereis": "guarecer",
    "guarecieren": "guarecer",
    "guarecieres": "guarecer",
    "guarecieron": "guarecer",
    "guareciese": "guarecer",
    "guarecieseis": "guarecer",
    "guareciesen": "guarecer",
    "guarecieses": "guarecer",
    "guarecimos": "guarecer",
    "guareciste": "guarecer",
    "guarecisteis": "guarecer",
    "guareciéramos": "guarecer",
    "guareciéremos": "guarecer",
    "guareciésemos": "guarecer",
    "guareció": "guarecer",
    "guarecéis": "guarecer",
    "guarecí": "guarecer",
    "guarecía": "guarecer",
    "guarecíais": "guarecer",
    "guarecíamos": "guarecer",
    "guarecían": "guarecer",
    "guarecías": "guarecer",
    "guarezca": "guarecer",
    "guarezcamos": "guarecer",
    "guarezcan": "guarecer",
    "guarezcas": "guarecer",
    "guarezco": "guarecer",
    "guarezcáis": "guarecer",
    "guarias": "guaria",
    "guaridas": "guarida",
    "guarimanes": "guarimán",
    "guarines": "guarín",
    "guarismos": "guarismo",
    "guarna": "guarnir",
    "guarnamos": "guarnir",
    "guarnan": "guarnir",
    "guarnas": "guarnir",
    "guarne": "guarnir",
    "guarnece": "guarnecer",
    "guarneced": "guarnecer",
    "guarnecedora": "guarnecedor",
    "guarnecedoras": "guarnecedor",
    "guarnecedores": "guarnecedor",
    "guarnecemos": "guarnecer",
    "guarnecen": "guarnecer",
    "guarneceremos": "guarnecer",
    "guarnecerá": "guarnecer",
    "guarnecerán": "guarnecer",
    "guarnecerás": "guarnecer",
    "guarneceré": "guarnecer",
    "guarneceréis": "guarnecer",
    "guarnecería": "guarnecer",
    "guarneceríais": "guarnecer",
    "guarneceríamos": "guarnecer",
    "guarnecerían": "guarnecer",
    "guarnecerías": "guarnecer",
    "guarneces": "guarnecer",
    "guarnecida": "guarnecer",
    "guarnecidas": "guarnecer",
    "guarnecido": "guarnecer",
    "guarnecidos": "guarnecido",
    "guarneciendo": "guarnecer",
    "guarneciera": "guarnecer",
    "guarnecierais": "guarnecer",
    "guarnecieran": "guarnecer",
    "guarnecieras": "guarnecer",
    "guarneciere": "guarnecer",
    "guarneciereis": "guarnecer",
    "guarnecieren": "guarnecer",
    "guarnecieres": "guarnecer",
    "guarnecieron": "guarnecer",
    "guarneciese": "guarnecer",
    "guarnecieseis": "guarnecer",
    "guarneciesen": "guarnecer",
    "guarnecieses": "guarnecer",
    "guarnecimos": "guarnecer",
    "guarneciste": "guarnecer",
    "guarnecisteis": "guarnecer",
    "guarneciéramos": "guarnecer",
    "guarneciéremos": "guarnecer",
    "guarneciésemos": "guarnecer",
    "guarneció": "guarnecer",
    "guarnecéis": "guarnecer",
    "guarnecí": "guarnecer",
    "guarnecía": "guarnecer",
    "guarnecíais": "guarnecer",
    "guarnecíamos": "guarnecer",
    "guarnecían": "guarnecer",
    "guarnecías": "guarnecer",
    "guarnen": "guarnir",
    "guarnes": "guarnir",
    "guarneses": "guarnés",
    "guarnezca": "guarnecer",
    "guarnezcamos": "guarnecer",
    "guarnezcan": "guarnecer",
    "guarnezcas": "guarnecer",
    "guarnezco": "guarnecer",
    "guarnezcáis": "guarnecer",
    "guarniciona": "guarnicionar",
    "guarnicionaba": "guarnicionar",
    "guarnicionabais": "guarnicionar",
    "guarnicionaban": "guarnicionar",
    "guarnicionabas": "guarnicionar",
    "guarnicionad": "guarnicionar",
    "guarnicionada": "guarnicionar",
    "guarnicionadas": "guarnicionar",
    "guarnicionado": "guarnicionar",
    "guarnicionados": "guarnicionar",
    "guarnicionamos": "guarnicionar",
    "guarnicionan": "guarnicionar",
    "guarnicionando": "guarnicionar",
    "guarnicionara": "guarnicionar",
    "guarnicionarais": "guarnicionar",
    "guarnicionaran": "guarnicionar",
    "guarnicionaras": "guarnicionar",
    "guarnicionare": "guarnicionar",
    "guarnicionareis": "guarnicionar",
    "guarnicionaremos": "guarnicionar",
    "guarnicionaren": "guarnicionar",
    "guarnicionares": "guarnicionar",
    "guarnicionaron": "guarnicionar",
    "guarnicionará": "guarnicionar",
    "guarnicionarán": "guarnicionar",
    "guarnicionarás": "guarnicionar",
    "guarnicionaré": "guarnicionar",
    "guarnicionaréis": "guarnicionar",
    "guarnicionaría": "guarnicionar",
    "guarnicionaríais": "guarnicionar",
    "guarnicionaríamos": "guarnicionar",
    "guarnicionarían": "guarnicionar",
    "guarnicionarías": "guarnicionar",
    "guarnicionas": "guarnicionar",
    "guarnicionase": "guarnicionar",
    "guarnicionaseis": "guarnicionar",
    "guarnicionasen": "guarnicionar",
    "guarnicionases": "guarnicionar",
    "guarnicionaste": "guarnicionar",
    "guarnicionasteis": "guarnicionar",
    "guarnicione": "guarnicionar",
    "guarnicionemos": "guarnicionar",
    "guarnicionen": "guarnicionar",
    "guarnicionera": "guarnicionero",
    "guarnicioneras": "guarnicionero",
    "guarnicioneros": "guarnicionero",
    "guarnicionerías": "guarnicionería",
    "guarniciones": "guarnicionar",
    "guarniciono": "guarnicionar",
    "guarnicionábamos": "guarnicionar",
    "guarnicionáis": "guarnicionar",
    "guarnicionáramos": "guarnicionar",
    "guarnicionáremos": "guarnicionar",
    "guarnicionásemos": "guarnicionar",
    "guarnicioné": "guarnicionar",
    "guarnicionéis": "guarnicionar",
    "guarnicionó": "guarnicionar",
    "guarnid": "guarnir",
    "guarnida": "guarnir",
    "guarnidas": "guarnir",
    "guarnido": "guarnir",
    "guarnidos": "guarnir",
    "guarnieles": "guarniel",
    "guarniendo": "guarnir",
    "guarniera": "guarnir",
    "guarnierais": "guarnir",
    "guarnieran": "guarnir",
    "guarnieras": "guarnir",
    "guarniere": "guarnir",
    "guarniereis": "guarnir",
    "guarnieren": "guarnir",
    "guarnieres": "guarnir",
    "guarnieron": "guarnir",
    "guarniese": "guarnir",
    "guarnieseis": "guarnir",
    "guarniesen": "guarnir",
    "guarnieses": "guarnir",
    "guarnigones": "guarnigón",
    "guarnimientos": "guarnimiento",
    "guarnimos": "guarnir",
    "guarniremos": "guarnir",
    "guarnirá": "guarnir",
    "guarnirán": "guarnir",
    "guarnirás": "guarnir",
    "guarniré": "guarnir",
    "guarniréis": "guarnir",
    "guarniría": "guarnir",
    "guarniríais": "guarnir",
    "guarniríamos": "guarnir",
    "guarnirían": "guarnir",
    "guarnirías": "guarnir",
    "guarniste": "guarnir",
    "guarnisteis": "guarnir",
    "guarniéramos": "guarnir",
    "guarniéremos": "guarnir",
    "guarniésemos": "guarnir",
    "guarnió": "guarnir",
    "guarno": "guarnir",
    "guarnáis": "guarnir",
    "guarní": "guarnir",
    "guarnía": "guarnir",
    "guarníais": "guarnir",
    "guarníamos": "guarnir",
    "guarnían": "guarnir",
    "guarnías": "guarnir",
    "guarnís": "guarnir",
    "guaros": "guaro",
    "guarra": "guarro",
    "guarradas": "guarrada",
    "guarras": "guarro",
    "guarrera": "guarrero",
    "guarreras": "guarrero",
    "guarreros": "guarrero",
    "guarrerías": "guarrería",
    "guarros": "guarro",
    "guarumos": "guarumo",
    "guas": "gua",
    "guasangas": "guasanga",
    "guasas": "guasa",
    "guascas": "guasca",
    "guascazos": "guascazo",
    "guasea": "guasear",
    "guaseaba": "guasear",
    "guaseabais": "guasear",
    "guaseaban": "guasear",
    "guaseabas": "guasear",
    "guasead": "guasear",
    "guaseada": "guasear",
    "guaseadas": "guasear",
    "guaseado": "guasear",
    "guaseados": "guasear",
    "guaseamos": "guasear",
    "guasean": "guasear",
    "guaseando": "guasear",
    "guaseara": "guasear",
    "guasearais": "guasear",
    "guasearan": "guasear",
    "guasearas": "guasear",
    "guaseare": "guasear",
    "guaseareis": "guasear",
    "guasearemos": "guasear",
    "guasearen": "guasear",
    "guaseares": "guasear",
    "guasearon": "guasear",
    "guaseará": "guasear",
    "guasearán": "guasear",
    "guasearás": "guasear",
    "guasearé": "guasear",
    "guasearéis": "guasear",
    "guasearía": "guasear",
    "guasearíais": "guasear",
    "guasearíamos": "guasear",
    "guasearían": "guasear",
    "guasearías": "guasear",
    "guaseas": "guasear",
    "guasease": "guasear",
    "guaseaseis": "guasear",
    "guaseasen": "guasear",
    "guaseases": "guasear",
    "guaseaste": "guasear",
    "guaseasteis": "guasear",
    "guasee": "guasear",
    "guaseemos": "guasear",
    "guaseen": "guasear",
    "guasees": "guasear",
    "guaseo": "guasear",
    "guaseábamos": "guasear",
    "guaseáis": "guasear",
    "guaseáramos": "guasear",
    "guaseáremos": "guasear",
    "guaseásemos": "guasear",
    "guaseé": "guasear",
    "guaseéis": "guasear",
    "guaseó": "guasear",
    "guasona": "guasón",
    "guasonas": "guasón",
    "guasones": "guasón",
    "guasos": "guaso",
    "guatas": "guata",
    "guatea": "guatear",
    "guateaba": "guatear",
    "guateabais": "guatear",
    "guateaban": "guatear",
    "guateabas": "guatear",
    "guatead": "guatear",
    "guateada": "guatear",
    "guateadas": "guatear",
    "guateado": "guatear",
    "guateados": "guatear",
    "guateamos": "guatear",
    "guatean": "guatear",
    "guateando": "guatear",
    "guateara": "guatear",
    "guatearais": "guatear",
    "guatearan": "guatear",
    "guatearas": "guatear",
    "guateare": "guatear",
    "guateareis": "guatear",
    "guatearemos": "guatear",
    "guatearen": "guatear",
    "guateares": "guatear",
    "guatearon": "guatear",
    "guateará": "guatear",
    "guatearán": "guatear",
    "guatearás": "guatear",
    "guatearé": "guatear",
    "guatearéis": "guatear",
    "guatearía": "guatear",
    "guatearíais": "guatear",
    "guatearíamos": "guatear",
    "guatearían": "guatear",
    "guatearías": "guatear",
    "guateas": "guatear",
    "guatease": "guatear",
    "guateaseis": "guatear",
    "guateasen": "guatear",
    "guateases": "guatear",
    "guateaste": "guatear",
    "guateasteis": "guatear",
    "guatee": "guatear",
    "guateemos": "guatear",
    "guateen": "guatear",
    "guatees": "guatear",
    "guatemalteca": "guatemalteco",
    "guatemaltecas": "guatemalteco",
    "guatemaltecos": "guatemalteco",
    "guatemaltequismos": "guatemaltequismo",
    "guateo": "guatear",
    "guateques": "guateque",
    "guateábamos": "guatear",
    "guateáis": "guatear",
    "guateáramos": "guatear",
    "guateáremos": "guatear",
    "guateásemos": "guatear",
    "guateé": "guatear",
    "guateéis": "guatear",
    "guateó": "guatear",
    "guayabas": "guayaba",
    "guayaberas": "guayabera",
    "guayabos": "guayabo",
    "guayacanes": "guayacán",
    "guayacas": "guayaca",
    "guayacoles": "guayacol",
    "guayacos": "guayaco",
    "guayanesa": "guayanés",
    "guayanesas": "guayanés",
    "guayaneses": "guayanés",
    "guayaquiles": "guayaquil",
    "guayaquileña": "guayaquileño",
    "guayaquileñas": "guayaquileño",
    "guayaquileños": "guayaquileño",
    "guayas": "guaya",
    "guazubirás": "guazubirá",
    "gubanes": "gubán",
    "gubernamentales": "gubernamental",
    "gubernativa": "gubernativo",
    "gubernativas": "gubernativo",
    "gubernativos": "gubernativo",
    "gubias": "gubia",
    "guedejada": "guedejado",
    "guedejadas": "guedejado",
    "guedejados": "guedejado",
    "guedejas": "guedeja",
    "guedejona": "guedejón",
    "guedejonas": "guedejón",
    "guedejones": "guedejón",
    "guedejosa": "guedejoso",
    "guedejosas": "guedejoso",
    "guedejosos": "guedejoso",
    "guedejuda": "guedejudo",
    "guedejudas": "guedejudo",
    "guedejudos": "guedejudo",
    "gueldos": "gueldo",
    "guepardos": "guepardo",
    "guerras": "guerra",
    "guerrea": "guerrear",
    "guerreaba": "guerrear",
    "guerreabais": "guerrear",
    "guerreaban": "guerrear",
    "guerreabas": "guerrear",
    "guerread": "guerrear",
    "guerreada": "guerrear",
    "guerreadas": "guerrear",
    "guerreado": "guerrear",
    "guerreadora": "guerreador",
    "guerreadoras": "guerreador",
    "guerreadores": "guerreador",
    "guerreados": "guerrear",
    "guerreamos": "guerrear",
    "guerrean": "guerrear",
    "guerreando": "guerrear",
    "guerreara": "guerrear",
    "guerrearais": "guerrear",
    "guerrearan": "guerrear",
    "guerrearas": "guerrear",
    "guerreare": "guerrear",
    "guerreareis": "guerrear",
    "guerrearemos": "guerrear",
    "guerrearen": "guerrear",
    "guerreares": "guerrear",
    "guerrearon": "guerrear",
    "guerreará": "guerrear",
    "guerrearán": "guerrear",
    "guerrearás": "guerrear",
    "guerrearé": "guerrear",
    "guerrearéis": "guerrear",
    "guerrearía": "guerrear",
    "guerrearíais": "guerrear",
    "guerrearíamos": "guerrear",
    "guerrearían": "guerrear",
    "guerrearías": "guerrear",
    "guerreas": "guerrear",
    "guerrease": "guerrear",
    "guerreaseis": "guerrear",
    "guerreasen": "guerrear",
    "guerreases": "guerrear",
    "guerreaste": "guerrear",
    "guerreasteis": "guerrear",
    "guerree": "guerrear",
    "guerreemos": "guerrear",
    "guerreen": "guerrear",
    "guerrees": "guerrear",
    "guerreo": "guerrear",
    "guerrera": "guerrero",
    "guerreras": "guerrero",
    "guerreros": "guerrero",
    "guerreábamos": "guerrear",
    "guerreáis": "guerrear",
    "guerreáramos": "guerrear",
    "guerreáremos": "guerrear",
    "guerreásemos": "guerrear",
    "guerreé": "guerrear",
    "guerreéis": "guerrear",
    "guerreó": "guerrear",
    "guerrillas": "guerrilla",
    "guerrillea": "guerrillear",
    "guerrilleaba": "guerrillear",
    "guerrilleabais": "guerrillear",
    "guerrilleaban": "guerrillear",
    "guerrilleabas": "guerrillear",
    "guerrillead": "guerrillear",
    "guerrilleada": "guerrillear",
    "guerrilleadas": "guerrillear",
    "guerrilleado": "guerrillear",
    "guerrilleados": "guerrillear",
    "guerrilleamos": "guerrillear",
    "guerrillean": "guerrillear",
    "guerrilleando": "guerrillear",
    "guerrilleara": "guerrillear",
    "guerrillearais": "guerrillear",
    "guerrillearan": "guerrillear",
    "guerrillearas": "guerrillear",
    "guerrilleare": "guerrillear",
    "guerrilleareis": "guerrillear",
    "guerrillearemos": "guerrillear",
    "guerrillearen": "guerrillear",
    "guerrilleares": "guerrillear",
    "guerrillearon": "guerrillear",
    "guerrilleará": "guerrillear",
    "guerrillearán": "guerrillear",
    "guerrillearás": "guerrillear",
    "guerrillearé": "guerrillear",
    "guerrillearéis": "guerrillear",
    "guerrillearía": "guerrillear",
    "guerrillearíais": "guerrillear",
    "guerrillearíamos": "guerrillear",
    "guerrillearían": "guerrillear",
    "guerrillearías": "guerrillear",
    "guerrilleas": "guerrillear",
    "guerrillease": "guerrillear",
    "guerrilleaseis": "guerrillear",
    "guerrilleasen": "guerrillear",
    "guerrilleases": "guerrillear",
    "guerrilleaste": "guerrillear",
    "guerrilleasteis": "guerrillear",
    "guerrillee": "guerrillear",
    "guerrilleemos": "guerrillear",
    "guerrilleen": "guerrillear",
    "guerrillees": "guerrillear",
    "guerrilleo": "guerrillear",
    "guerrillera": "guerrillero",
    "guerrilleras": "guerrillero",
    "guerrilleros": "guerrillero",
    "guerrilleábamos": "guerrillear",
    "guerrilleáis": "guerrillear",
    "guerrilleáramos": "guerrillear",
    "guerrilleáremos": "guerrillear",
    "guerrilleásemos": "guerrillear",
    "guerrilleé": "guerrillear",
    "guerrilleéis": "guerrillear",
    "guerrilleó": "guerrillear",
    "guetos": "gueto",
    "guiaba": "guiar",
    "guiabais": "guiar",
    "guiaban": "guiar",
    "guiabas": "guiar",
    "guiad": "guiar",
    "guiada": "guiar",
    "guiadas": "guiar",
    "guiaderas": "guiadera",
    "guiado": "guiar",
    "guiadora": "guiador",
    "guiadoras": "guiador",
    "guiadores": "guiador",
    "guiados": "guiar",
    "guiajes": "guiaje",
    "guiamos": "guiar",
    "guiando": "guiar",
    "guiara": "guiar",
    "guiarais": "guiar",
    "guiaran": "guiar",
    "guiaras": "guiar",
    "guiare": "guiar",
    "guiareis": "guiar",
    "guiaremos": "guiar",
    "guiaren": "guiar",
    "guiares": "guiar",
    "guiaron": "guiar",
    "guiará": "guiar",
    "guiarán": "guiar",
    "guiarás": "guiar",
    "guiaré": "guiar",
    "guiaréis": "guiar",
    "guiaría": "guiar",
    "guiaríais": "guiar",
    "guiaríamos": "guiar",
    "guiarían": "guiar",
    "guiarías": "guiar",
    "guiase": "guiar",
    "guiaseis": "guiar",
    "guiasen": "guiar",
    "guiases": "guiar",
    "guiaste": "guiar",
    "guiasteis": "guiar",
    "guiemos": "guiar",
    "guijarrales": "guijarral",
    "guijarrazos": "guijarrazo",
    "guijarreña": "guijarreño",
    "guijarreñas": "guijarreño",
    "guijarreños": "guijarreño",
    "guijarros": "guijarro",
    "guijarrosa": "guijarroso",
    "guijarrosas": "guijarroso",
    "guijarrosos": "guijarroso",
    "guijas": "guija",
    "guijeña": "guijeño",
    "guijeñas": "guijeño",
    "guijeños": "guijeño",
    "guijos": "guijo",
    "guijosa": "guijoso",
    "guijosas": "guijoso",
    "guijosos": "guijoso",
    "guildas": "guilda",
    "guilders": "guilder",
    "guilla": "guillar",
    "guillaba": "guillar",
    "guillabais": "guillar",
    "guillaban": "guillar",
    "guillabas": "guillar",
    "guillad": "guillar",
    "guillada": "guillar",
    "guilladas": "guillar",
    "guillado": "guillar",
    "guillados": "guillar",
    "guilladuras": "guilladura",
    "guillames": "guillame",
    "guillamos": "guillar",
    "guillan": "guillar",
    "guillando": "guillar",
    "guillara": "guillar",
    "guillarais": "guillar",
    "guillaran": "guillar",
    "guillaras": "guillar",
    "guillare": "guillar",
    "guillareis": "guillar",
    "guillaremos": "guillar",
    "guillaren": "guillar",
    "guillares": "guillar",
    "guillaron": "guillar",
    "guillará": "guillar",
    "guillarán": "guillar",
    "guillarás": "guillar",
    "guillaré": "guillar",
    "guillaréis": "guillar",
    "guillaría": "guillar",
    "guillaríais": "guillar",
    "guillaríamos": "guillar",
    "guillarían": "guillar",
    "guillarías": "guillar",
    "guillas": "guillar",
    "guillase": "guillar",
    "guillaseis": "guillar",
    "guillasen": "guillar",
    "guillases": "guillar",
    "guillaste": "guillar",
    "guillasteis": "guillar",
    "guille": "guillar",
    "guillemos": "guillar",
    "guillen": "guillar",
    "guilles": "guillar",
    "guillo": "guillar",
    "guillotes": "guillote",
    "guillotina": "guillotinar",
    "guillotinaba": "guillotinar",
    "guillotinabais": "guillotinar",
    "guillotinaban": "guillotinar",
    "guillotinabas": "guillotinar",
    "guillotinad": "guillotinar",
    "guillotinada": "guillotinar",
    "guillotinadas": "guillotinar",
    "guillotinado": "guillotinar",
    "guillotinados": "guillotinar",
    "guillotinamos": "guillotinar",
    "guillotinan": "guillotinar",
    "guillotinando": "guillotinar",
    "guillotinara": "guillotinar",
    "guillotinarais": "guillotinar",
    "guillotinaran": "guillotinar",
    "guillotinaras": "guillotinar",
    "guillotinare": "guillotinar",
    "guillotinareis": "guillotinar",
    "guillotinaremos": "guillotinar",
    "guillotinaren": "guillotinar",
    "guillotinares": "guillotinar",
    "guillotinaron": "guillotinar",
    "guillotinará": "guillotinar",
    "guillotinarán": "guillotinar",
    "guillotinarás": "guillotinar",
    "guillotinaré": "guillotinar",
    "guillotinaréis": "guillotinar",
    "guillotinaría": "guillotinar",
    "guillotinaríais": "guillotinar",
    "guillotinaríamos": "guillotinar",
    "guillotinarían": "guillotinar",
    "guillotinarías": "guillotinar",
    "guillotinas": "guillotinar",
    "guillotinase": "guillotinar",
    "guillotinaseis": "guillotinar",
    "guillotinasen": "guillotinar",
    "guillotinases": "guillotinar",
    "guillotinaste": "guillotinar",
    "guillotinasteis": "guillotinar",
    "guillotine": "guillotinar",
    "guillotinemos": "guillotinar",
    "guillotinen": "guillotinar",
    "guillotines": "guillotinar",
    "guillotino": "guillotinar",
    "guillotinábamos": "guillotinar",
    "guillotináis": "guillotinar",
    "guillotináramos": "guillotinar",
    "guillotináremos": "guillotinar",
    "guillotinásemos": "guillotinar",
    "guillotiné": "guillotinar",
    "guillotinéis": "guillotinar",
    "guillotinó": "guillotinar",
    "guillábamos": "guillar",
    "guilláis": "guillar",
    "guilláramos": "guillar",
    "guilláremos": "guillar",
    "guillásemos": "guillar",
    "guillé": "guillar",
    "guilléis": "guillar",
    "guilló": "guillar",
    "guimbaletes": "guimbalete",
    "guimbardas": "guimbarda",
    "guincha": "guinchar",
    "guinchaba": "guinchar",
    "guinchabais": "guinchar",
    "guinchaban": "guinchar",
    "guinchabas": "guinchar",
    "guinchad": "guinchar",
    "guinchada": "guinchar",
    "guinchadas": "guinchar",
    "guinchado": "guinchar",
    "guinchados": "guinchar",
    "guinchamos": "guinchar",
    "guinchan": "guinchar",
    "guinchando": "guinchar",
    "guinchara": "guinchar",
    "guincharais": "guinchar",
    "guincharan": "guinchar",
    "guincharas": "guinchar",
    "guinchare": "guinchar",
    "guinchareis": "guinchar",
    "guincharemos": "guinchar",
    "guincharen": "guinchar",
    "guinchares": "guinchar",
    "guincharon": "guinchar",
    "guinchará": "guinchar",
    "guincharán": "guinchar",
    "guincharás": "guinchar",
    "guincharé": "guinchar",
    "guincharéis": "guinchar",
    "guincharía": "guinchar",
    "guincharíais": "guinchar",
    "guincharíamos": "guinchar",
    "guincharían": "guinchar",
    "guincharías": "guinchar",
    "guinchas": "guinchar",
    "guinchase": "guinchar",
    "guinchaseis": "guinchar",
    "guinchasen": "guinchar",
    "guinchases": "guinchar",
    "guinchaste": "guinchar",
    "guinchasteis": "guinchar",
    "guinche": "guinchar",
    "guinchemos": "guinchar",
    "guinchen": "guinchar",
    "guinches": "guinchar",
    "guincho": "guinchar",
    "guinchones": "guinchón",
    "guinchos": "guincho",
    "guinchábamos": "guinchar",
    "guincháis": "guinchar",
    "guincháramos": "guinchar",
    "guincháremos": "guinchar",
    "guinchásemos": "guinchar",
    "guinché": "guinchar",
    "guinchéis": "guinchar",
    "guinchó": "guinchar",
    "guinda": "guindar",
    "guindaba": "guindar",
    "guindabais": "guindar",
    "guindaban": "guindar",
    "guindabas": "guindar",
    "guindad": "guindar",
    "guindada": "guindar",
    "guindadas": "guindar",
    "guindado": "guindar",
    "guindados": "guindar",
    "guindaleras": "guindalera",
    "guindales": "guindal",
    "guindaletas": "guindaleta",
    "guindalezas": "guindaleza",
    "guindamainas": "guindamaina",
    "guindamos": "guindar",
    "guindan": "guindar",
    "guindando": "guindar",
    "guindara": "guindar",
    "guindarais": "guindar",
    "guindaran": "guindar",
    "guindaras": "guindar",
    "guindare": "guindar",
    "guindareis": "guindar",
    "guindaremos": "guindar",
    "guindaren": "guindar",
    "guindares": "guindar",
    "guindaron": "guindar",
    "guindará": "guindar",
    "guindarán": "guindar",
    "guindarás": "guindar",
    "guindaré": "guindar",
    "guindaréis": "guindar",
    "guindaría": "guindar",
    "guindaríais": "guindar",
    "guindaríamos": "guindar",
    "guindarían": "guindar",
    "guindarías": "guindar",
    "guindas": "guindar",
    "guindase": "guindar",
    "guindaseis": "guindar",
    "guindasen": "guindar",
    "guindases": "guindar",
    "guindaste": "guindar",
    "guindasteis": "guindar",
    "guindastes": "guindaste",
    "guinde": "guindar",
    "guindemos": "guindar",
    "guinden": "guindar",
    "guindes": "guindar",
    "guindillas": "guindilla",
    "guindo": "guindar",
    "guindolas": "guindola",
    "guindos": "guindo",
    "guindábamos": "guindar",
    "guindáis": "guindar",
    "guindáramos": "guindar",
    "guindáremos": "guindar",
    "guindásemos": "guindar",
    "guindé": "guindar",
    "guindéis": "guindar",
    "guindó": "guindar",
    "guinea": "guineo",
    "guineana": "guineano",
    "guineanas": "guineano",
    "guineanos": "guineano",
    "guineas": "guineo",
    "guineos": "guineo",
    "guingas": "guinga",
    "guinjas": "guinja",
    "guinjoleros": "guinjolero",
    "guinjos": "guinjo",
    "guionajes": "guionaje",
    "guiones": "guión",
    "guionistas": "guionista",
    "guionizaciones": "guionización",
    "guionizaciónes": "guionización",
    "guipa": "guipar",
    "guipaba": "guipar",
    "guipabais": "guipar",
    "guipaban": "guipar",
    "guipabas": "guipar",
    "guipad": "guipar",
    "guipada": "guipar",
    "guipadas": "guipar",
    "guipado": "guipar",
    "guipados": "guipar",
    "guipamos": "guipar",
    "guipan": "guipar",
    "guipando": "guipar",
    "guipara": "guipar",
    "guiparais": "guipar",
    "guiparan": "guipar",
    "guiparas": "guipar",
    "guipare": "guipar",
    "guipareis": "guipar",
    "guiparemos": "guipar",
    "guiparen": "guipar",
    "guipares": "guipar",
    "guiparon": "guipar",
    "guipará": "guipar",
    "guiparán": "guipar",
    "guiparás": "guipar",
    "guiparé": "guipar",
    "guiparéis": "guipar",
    "guiparía": "guipar",
    "guiparíais": "guipar",
    "guiparíamos": "guipar",
    "guiparían": "guipar",
    "guiparías": "guipar",
    "guipas": "guipar",
    "guipase": "guipar",
    "guipaseis": "guipar",
    "guipasen": "guipar",
    "guipases": "guipar",
    "guipaste": "guipar",
    "guipasteis": "guipar",
    "guipe": "guipar",
    "guipemos": "guipar",
    "guipen": "guipar",
    "guipes": "guipar",
    "guipo": "guipar",
    "guipuzcoana": "guipuzcoano",
    "guipuzcoanas": "guipuzcoano",
    "guipuzcoanos": "guipuzcoano",
    "guipábamos": "guipar",
    "guipáis": "guipar",
    "guipáramos": "guipar",
    "guipáremos": "guipar",
    "guipásemos": "guipar",
    "guipé": "guipar",
    "guipéis": "guipar",
    "guipó": "guipar",
    "guirigays": "guirigay",
    "guirigáis": "guirigay",
    "guirindolas": "guirindola",
    "guirlaches": "guirlache",
    "guirnaldas": "guirnalda",
    "guiropas": "guiropa",
    "guisa": "guisar",
    "guisaba": "guisar",
    "guisabais": "guisar",
    "guisaban": "guisar",
    "guisabas": "guisar",
    "guisad": "guisar",
    "guisada": "guisar",
    "guisadas": "guisar",
    "guisado": "guisar",
    "guisadora": "guisador",
    "guisadoras": "guisador",
    "guisadores": "guisador",
    "guisados": "guisar",
    "guisamos": "guisar",
    "guisan": "guisar",
    "guisandera": "guisandero",
    "guisanderas": "guisandero",
    "guisanderos": "guisandero",
    "guisando": "guisar",
    "guisantales": "guisantal",
    "guisantes": "guisante",
    "guisara": "guisar",
    "guisarais": "guisar",
    "guisaran": "guisar",
    "guisaras": "guisar",
    "guisare": "guisar",
    "guisareis": "guisar",
    "guisaremos": "guisar",
    "guisaren": "guisar",
    "guisares": "guisar",
    "guisaron": "guisar",
    "guisará": "guisar",
    "guisarán": "guisar",
    "guisarás": "guisar",
    "guisaré": "guisar",
    "guisaréis": "guisar",
    "guisaría": "guisar",
    "guisaríais": "guisar",
    "guisaríamos": "guisar",
    "guisarían": "guisar",
    "guisarías": "guisar",
    "guisas": "guisar",
    "guisase": "guisar",
    "guisaseis": "guisar",
    "guisasen": "guisar",
    "guisases": "guisar",
    "guisaste": "guisar",
    "guisasteis": "guisar",
    "guise": "guisar",
    "guisemos": "guisar",
    "guisen": "guisar",
    "guises": "guisar",
    "guiso": "guisar",
    "guisopos": "guisopo",
    "guisos": "guiso",
    "guisotes": "guisote",
    "guisábamos": "guisar",
    "guisáis": "guisar",
    "guisáramos": "guisar",
    "guisáremos": "guisar",
    "guisásemos": "guisar",
    "guisé": "guisar",
    "guiséis": "guisar",
    "guisó": "guisar",
    "guita": "guitar",
    "guitaba": "guitar",
    "guitabais": "guitar",
    "guitaban": "guitar",
    "guitabas": "guitar",
    "guitad": "guitar",
    "guitada": "guitar",
    "guitadas": "guitar",
    "guitado": "guitar",
    "guitados": "guitar",
    "guitamos": "guitar",
    "guitan": "guitar",
    "guitando": "guitar",
    "guitara": "guitar",
    "guitarais": "guitar",
    "guitaran": "guitar",
    "guitaras": "guitar",
    "guitare": "guitar",
    "guitareis": "guitar",
    "guitaremos": "guitar",
    "guitaren": "guitar",
    "guitares": "guitar",
    "guitaron": "guitar",
    "guitarras": "guitarra",
    "guitarreos": "guitarreo",
    "guitarrera": "guitarrero",
    "guitarreras": "guitarrero",
    "guitarreros": "guitarrero",
    "guitarrerías": "guitarrería",
    "guitarrillos": "guitarrillo",
    "guitarristas": "guitarrista",
    "guitarrones": "guitarrón",
    "guitarros": "guitarro",
    "guitará": "guitar",
    "guitarán": "guitar",
    "guitarás": "guitar",
    "guitaré": "guitar",
    "guitaréis": "guitar",
    "guitaría": "guitar",
    "guitaríais": "guitar",
    "guitaríamos": "guitar",
    "guitarían": "guitar",
    "guitarías": "guitar",
    "guitas": "guitar",
    "guitase": "guitar",
    "guitaseis": "guitar",
    "guitasen": "guitar",
    "guitases": "guitar",
    "guitaste": "guitar",
    "guitasteis": "guitar",
    "guite": "guitar",
    "guitemos": "guitar",
    "guiten": "guitar",
    "guitera": "guitero",
    "guiteras": "guitero",
    "guiteros": "guitero",
    "guites": "guitar",
    "guito": "guitar",
    "guitona": "guitón",
    "guitonas": "guitón",
    "guitonea": "guitonear",
    "guitoneaba": "guitonear",
    "guitoneabais": "guitonear",
    "guitoneaban": "guitonear",
    "guitoneabas": "guitonear",
    "guitonead": "guitonear",
    "guitoneada": "guitonear",
    "guitoneadas": "guitonear",
    "guitoneado": "guitonear",
    "guitoneados": "guitonear",
    "guitoneamos": "guitonear",
    "guitonean": "guitonear",
    "guitoneando": "guitonear",
    "guitoneara": "guitonear",
    "guitonearais": "guitonear",
    "guitonearan": "guitonear",
    "guitonearas": "guitonear",
    "guitoneare": "guitonear",
    "guitoneareis": "guitonear",
    "guitonearemos": "guitonear",
    "guitonearen": "guitonear",
    "guitoneares": "guitonear",
    "guitonearon": "guitonear",
    "guitoneará": "guitonear",
    "guitonearán": "guitonear",
    "guitonearás": "guitonear",
    "guitonearé": "guitonear",
    "guitonearéis": "guitonear",
    "guitonearía": "guitonear",
    "guitonearíais": "guitonear",
    "guitonearíamos": "guitonear",
    "guitonearían": "guitonear",
    "guitonearías": "guitonear",
    "guitoneas": "guitonear",
    "guitonease": "guitonear",
    "guitoneaseis": "guitonear",
    "guitoneasen": "guitonear",
    "guitoneases": "guitonear",
    "guitoneaste": "guitonear",
    "guitoneasteis": "guitonear",
    "guitonee": "guitonear",
    "guitoneemos": "guitonear",
    "guitoneen": "guitonear",
    "guitonees": "guitonear",
    "guitoneo": "guitonear",
    "guitones": "guitón",
    "guitoneábamos": "guitonear",
    "guitoneáis": "guitonear",
    "guitoneáramos": "guitonear",
    "guitoneáremos": "guitonear",
    "guitoneásemos": "guitonear",
    "guitoneé": "guitonear",
    "guitoneéis": "guitonear",
    "guitoneó": "guitonear",
    "guitábamos": "guitar",
    "guitáis": "guitar",
    "guitáramos": "guitar",
    "guitáremos": "guitar",
    "guitásemos": "guitar",
    "guité": "guitar",
    "guitéis": "guitar",
    "guitó": "guitar",
    "guizques": "guizque",
    "guiábamos": "guiar",
    "guiáis": "guiar",
    "guiáramos": "guiar",
    "guiáremos": "guiar",
    "guiásemos": "guiar",
    "guié": "guiar",
    "guiéis": "guiar",
    "guiña": "guiñar",
    "guiñaba": "guiñar",
    "guiñabais": "guiñar",
    "guiñaban": "guiñar",
    "guiñabas": "guiñar",
    "guiñad": "guiñar",
    "guiñada": "guiñar",
    "guiñadas": "guiñar",
    "guiñado": "guiñar",
    "guiñadora": "guiñador",
    "guiñadoras": "guiñador",
    "guiñadores": "guiñador",
    "guiñados": "guiñar",
    "guiñaduras": "guiñadura",
    "guiñamos": "guiñar",
    "guiñan": "guiñar",
    "guiñando": "guiñar",
    "guiñapienta": "guiñapiento",
    "guiñapientas": "guiñapiento",
    "guiñapientos": "guiñapiento",
    "guiñapos": "guiñapo",
    "guiñaposa": "guiñaposo",
    "guiñaposas": "guiñaposo",
    "guiñaposos": "guiñaposo",
    "guiñara": "guiñar",
    "guiñarais": "guiñar",
    "guiñaran": "guiñar",
    "guiñaras": "guiñar",
    "guiñare": "guiñar",
    "guiñareis": "guiñar",
    "guiñaremos": "guiñar",
    "guiñaren": "guiñar",
    "guiñares": "guiñar",
    "guiñaron": "guiñar",
    "guiñará": "guiñar",
    "guiñarán": "guiñar",
    "guiñarás": "guiñar",
    "guiñaré": "guiñar",
    "guiñaréis": "guiñar",
    "guiñaría": "guiñar",
    "guiñaríais": "guiñar",
    "guiñaríamos": "guiñar",
    "guiñarían": "guiñar",
    "guiñarías": "guiñar",
    "guiñas": "guiñar",
    "guiñase": "guiñar",
    "guiñaseis": "guiñar",
    "guiñasen": "guiñar",
    "guiñases": "guiñar",
    "guiñaste": "guiñar",
    "guiñasteis": "guiñar",
    "guiñe": "guiñar",
    "guiñemos": "guiñar",
    "guiñen": "guiñar",
    "guiñes": "guiñar",
    "guiño": "guiñar",
    "guiñoles": "guiñol",
    "guiños": "guiño",
    "guiñotes": "guiñote",
    "guiñábamos": "guiñar",
    "guiñáis": "guiñar",
    "guiñáramos": "guiñar",
    "guiñáremos": "guiñar",
    "guiñásemos": "guiñar",
    "guiñé": "guiñar",
    "guiñéis": "guiñar",
    "guiñó": "guiñar",
    "guió": "guiar",
    "gujas": "guja",
    "gulas": "gula",
    "gulasches": "gulasch",
    "gulosa": "guloso",
    "gulosas": "guloso",
    "gulosidades": "gulosidad",
    "gulosos": "guloso",
    "gulusmea": "gulusmear",
    "gulusmeaba": "gulusmear",
    "gulusmeabais": "gulusmear",
    "gulusmeaban": "gulusmear",
    "gulusmeabas": "gulusmear",
    "gulusmead": "gulusmear",
    "gulusmeada": "gulusmear",
    "gulusmeadas": "gulusmear",
    "gulusmeado": "gulusmear",
    "gulusmeados": "gulusmear",
    "gulusmeamos": "gulusmear",
    "gulusmean": "gulusmear",
    "gulusmeando": "gulusmear",
    "gulusmeara": "gulusmear",
    "gulusmearais": "gulusmear",
    "gulusmearan": "gulusmear",
    "gulusmearas": "gulusmear",
    "gulusmeare": "gulusmear",
    "gulusmeareis": "gulusmear",
    "gulusmearemos": "gulusmear",
    "gulusmearen": "gulusmear",
    "gulusmeares": "gulusmear",
    "gulusmearon": "gulusmear",
    "gulusmeará": "gulusmear",
    "gulusmearán": "gulusmear",
    "gulusmearás": "gulusmear",
    "gulusmearé": "gulusmear",
    "gulusmearéis": "gulusmear",
    "gulusmearía": "gulusmear",
    "gulusmearíais": "gulusmear",
    "gulusmearíamos": "gulusmear",
    "gulusmearían": "gulusmear",
    "gulusmearías": "gulusmear",
    "gulusmeas": "gulusmear",
    "gulusmease": "gulusmear",
    "gulusmeaseis": "gulusmear",
    "gulusmeasen": "gulusmear",
    "gulusmeases": "gulusmear",
    "gulusmeaste": "gulusmear",
    "gulusmeasteis": "gulusmear",
    "gulusmee": "gulusmear",
    "gulusmeemos": "gulusmear",
    "gulusmeen": "gulusmear",
    "gulusmees": "gulusmear",
    "gulusmeo": "gulusmear",
    "gulusmera": "gulusmero",
    "gulusmeras": "gulusmero",
    "gulusmeros": "gulusmero",
    "gulusmeábamos": "gulusmear",
    "gulusmeáis": "gulusmear",
    "gulusmeáramos": "gulusmear",
    "gulusmeáremos": "gulusmear",
    "gulusmeásemos": "gulusmear",
    "gulusmeé": "gulusmear",
    "gulusmeéis": "gulusmear",
    "gulusmeó": "gulusmear",
    "gumías": "gumía",
    "gunitas": "gunita",
    "gunneráceas": "gunnerácea",
    "guppies": "guppy",
    "gurbia": "gurbio",
    "gurbias": "gurbio",
    "gurbionada": "gurbionado",
    "gurbionadas": "gurbionado",
    "gurbionados": "gurbionado",
    "gurbiones": "gurbión",
    "gurbios": "gurbio",
    "gurda": "gurdo",
    "gurdas": "gurdo",
    "gurdos": "gurdo",
    "guripas": "guripa",
    "gurisas": "gurisa",
    "gurkas": "gurka",
    "gurriatos": "gurriato",
    "gurruferos": "gurrufero",
    "gurrumina": "gurrumino",
    "gurruminas": "gurrumino",
    "gurruminos": "gurrumino",
    "gurulladas": "gurullada",
    "gurullos": "gurullo",
    "gurupas": "gurupa",
    "gurus": "guru",
    "gurís": "gurí",
    "gurús": "gurú",
    "gusanea": "gusanear",
    "gusaneaba": "gusanear",
    "gusaneabais": "gusanear",
    "gusaneaban": "gusanear",
    "gusaneabas": "gusanear",
    "gusanead": "gusanear",
    "gusaneada": "gusanear",
    "gusaneadas": "gusanear",
    "gusaneado": "gusanear",
    "gusaneados": "gusanear",
    "gusaneamos": "gusanear",
    "gusanean": "gusanear",
    "gusaneando": "gusanear",
    "gusaneara": "gusanear",
    "gusanearais": "gusanear",
    "gusanearan": "gusanear",
    "gusanearas": "gusanear",
    "gusaneare": "gusanear",
    "gusaneareis": "gusanear",
    "gusanearemos": "gusanear",
    "gusanearen": "gusanear",
    "gusaneares": "gusanear",
    "gusanearon": "gusanear",
    "gusaneará": "gusanear",
    "gusanearán": "gusanear",
    "gusanearás": "gusanear",
    "gusanearé": "gusanear",
    "gusanearéis": "gusanear",
    "gusanearía": "gusanear",
    "gusanearíais": "gusanear",
    "gusanearíamos": "gusanear",
    "gusanearían": "gusanear",
    "gusanearías": "gusanear",
    "gusaneas": "gusanear",
    "gusanease": "gusanear",
    "gusaneaseis": "gusanear",
    "gusaneasen": "gusanear",
    "gusaneases": "gusanear",
    "gusaneaste": "gusanear",
    "gusaneasteis": "gusanear",
    "gusanee": "gusanear",
    "gusaneemos": "gusanear",
    "gusaneen": "gusanear",
    "gusanees": "gusanear",
    "gusaneo": "gusanear",
    "gusaneras": "gusanera",
    "gusanerías": "gusanería",
    "gusaneábamos": "gusanear",
    "gusaneáis": "gusanear",
    "gusaneáramos": "gusanear",
    "gusaneáremos": "gusanear",
    "gusaneásemos": "gusanear",
    "gusaneé": "gusanear",
    "gusaneéis": "gusanear",
    "gusaneó": "gusanear",
    "gusanienta": "gusaniento",
    "gusanientas": "gusaniento",
    "gusanientos": "gusaniento",
    "gusanillos": "gusanillo",
    "gusanos": "gusano",
    "gusanosa": "gusanoso",
    "gusanosas": "gusanoso",
    "gusanosos": "gusanoso",
    "gusarapa": "gusarapo",
    "gusarapas": "gusarapo",
    "gusarapos": "gusarapo",
    "gusta": "gustar",
    "gustaba": "gustar",
    "gustabais": "gustar",
    "gustaban": "gustar",
    "gustabas": "gustar",
    "gustaciones": "gustación",
    "gustad": "gustar",
    "gustada": "gustar",
    "gustadas": "gustar",
    "gustado": "gustar",
    "gustados": "gustar",
    "gustamos": "gustar",
    "gustan": "gustar",
    "gustando": "gustar",
    "gustara": "gustar",
    "gustarais": "gustar",
    "gustaran": "gustar",
    "gustaras": "gustar",
    "gustare": "gustar",
    "gustareis": "gustar",
    "gustaremos": "gustar",
    "gustaren": "gustar",
    "gustares": "gustar",
    "gustaron": "gustar",
    "gustará": "gustar",
    "gustarán": "gustar",
    "gustarás": "gustar",
    "gustaré": "gustar",
    "gustaréis": "gustar",
    "gustaría": "gustar",
    "gustaríais": "gustar",
    "gustaríamos": "gustar",
    "gustarían": "gustar",
    "gustarías": "gustar",
    "gustas": "gustar",
    "gustase": "gustar",
    "gustaseis": "gustar",
    "gustasen": "gustar",
    "gustases": "gustar",
    "gustaste": "gustar",
    "gustasteis": "gustar",
    "gustativa": "gustativo",
    "gustativas": "gustativo",
    "gustativos": "gustativo",
    "gustazos": "gustazo",
    "guste": "gustar",
    "gustemos": "gustar",
    "gusten": "gustar",
    "gustes": "gustar",
    "gustillos": "gustillo",
    "gusto": "gustar",
    "gustos": "gusto",
    "gustosa": "gustoso",
    "gustosas": "gustoso",
    "gustosos": "gustoso",
    "gustábamos": "gustar",
    "gustáis": "gustar",
    "gustáramos": "gustar",
    "gustáremos": "gustar",
    "gustásemos": "gustar",
    "gusté": "gustar",
    "gustéis": "gustar",
    "gustó": "gustar",
    "gutaciones": "gutación",
    "gutagambas": "gutagamba",
    "gutaperchas": "gutapercha",
    "guties": "guti",
    "gutis": "guti",
    "gutiámbares": "gutiámbar",
    "guturales": "gutural",
    "gutíferas": "gutífera",
    "guyanesa": "guyanés",
    "guyanesas": "guyanés",
    "guyaneses": "guyanés",
    "guyotes": "guyot",
    "guzlas": "guzla",
    "guzmanes": "guzmán",
    "guáchara": "guácharo",
    "guácharas": "guácharo",
    "guácharos": "guácharo",
    "guácimas": "guácima",
    "guájaras": "guájara",
    "guía": "guiar",
    "guían": "guiar",
    "guías": "guiar",
    "guíe": "guiar",
    "guíen": "guiar",
    "guíes": "guiar",
    "guíndolas": "guíndola",
    "guínjoles": "guínjol",
    "guío": "guiar",
    "gymkhanas": "gymkhana",
    "gádida": "gádido",
    "gádidas": "gádido",
    "gádidos": "gádido",
    "gálagos": "gálago",
    "gálatas": "gálata",
    "gálbanos": "gálbano",
    "gálbulos": "gálbulo",
    "gáleas": "gálea",
    "gálibos": "gálibo",
    "gálica": "gálico",
    "gálicas": "gálico",
    "gálicos": "gálico",
    "gándaras": "gándara",
    "gángsteres": "gángster",
    "gángsterses": "gángster",
    "gánguiles": "gánguil",
    "gárgaras": "gárgara",
    "gárgolas": "gárgola",
    "gárrula": "gárrulo",
    "gárrulas": "gárrulo",
    "gárrulos": "gárrulo",
    "gásteres": "gáster",
    "gástrica": "gástrico",
    "gástricas": "gástrico",
    "gástricos": "gástrico",
    "gástrulas": "gástrula",
    "géiseres": "géiser",
    "géiserses": "géiser",
    "gélica": "gélico",
    "gélicas": "gélico",
    "gélicos": "gélico",
    "gélida": "gélido",
    "gélidas": "gélido",
    "gélidos": "gélido",
    "géneros": "género",
    "génica": "génico",
    "génicas": "génico",
    "génicos": "génico",
    "génovas": "génova",
    "gépida": "gépido",
    "gépidas": "gépido",
    "gépidos": "gépido",
    "gérmenes": "germen",
    "gétulos": "gétulo",
    "gímnica": "gímnico",
    "gímnicas": "gímnico",
    "gímnicos": "gímnico",
    "gínglimos": "gínglimo",
    "gípsica": "gípsico",
    "gípsicas": "gípsico",
    "gípsicos": "gípsico",
    "góbidos": "góbido",
    "gónadas": "gónada",
    "góndolas": "góndola",
    "gótica": "gótico",
    "góticas": "gótico",
    "góticos": "gótico",
    "gúmenas": "gúmena",
    "güelfa": "güelfo",
    "güelfas": "güelfo",
    "güelfos": "güelfo",
    "güera": "güero",
    "güeras": "güero",
    "güeros": "güero",
    "güiros": "güiro",
    "güisquis": "güisqui",
    "güitos": "güito",
    "günces": "günz",
    "h": "hora",
    "ha": "haber",
    "habada": "habado",
    "habadas": "habado",
    "habados": "habado",
    "habana": "habano",
    "habanas": "habano",
    "habanera": "habanero",
    "habaneras": "habanero",
    "habaneros": "habanero",
    "habanos": "habano",
    "habares": "habar",
    "habas": "haba",
    "habenulares": "habenular",
    "haberes": "haber",
    "haberíos": "haberío",
    "habices": "habiz",
    "habichuelas": "habichuela",
    "habida": "haber",
    "habidas": "haber",
    "habido": "haber",
    "habidos": "haber",
    "habiendo": "haber",
    "habientes": "habiente",
    "habilidades": "habilidad",
    "habilidosa": "habilidoso",
    "habilidosas": "habilidoso",
    "habilidosos": "habilidoso",
    "habilita": "habilitar",
    "habilitaba": "habilitar",
    "habilitabais": "habilitar",
    "habilitaban": "habilitar",
    "habilitabas": "habilitar",
    "habilitaciones": "habilitación",
    "habilitad": "habilitar",
    "habilitada": "habilitar",
    "habilitadas": "habilitar",
    "habilitado": "habilitar",
    "habilitadora": "habilitador",
    "habilitadoras": "habilitador",
    "habilitadores": "habilitador",
    "habilitados": "habilitar",
    "habilitamos": "habilitar",
    "habilitan": "habilitar",
    "habilitando": "habilitar",
    "habilitara": "habilitar",
    "habilitarais": "habilitar",
    "habilitaran": "habilitar",
    "habilitaras": "habilitar",
    "habilitare": "habilitar",
    "habilitareis": "habilitar",
    "habilitaremos": "habilitar",
    "habilitaren": "habilitar",
    "habilitares": "habilitar",
    "habilitaron": "habilitar",
    "habilitará": "habilitar",
    "habilitarán": "habilitar",
    "habilitarás": "habilitar",
    "habilitaré": "habilitar",
    "habilitaréis": "habilitar",
    "habilitaría": "habilitar",
    "habilitaríais": "habilitar",
    "habilitaríamos": "habilitar",
    "habilitarían": "habilitar",
    "habilitarías": "habilitar",
    "habilitas": "habilitar",
    "habilitase": "habilitar",
    "habilitaseis": "habilitar",
    "habilitasen": "habilitar",
    "habilitases": "habilitar",
    "habilitaste": "habilitar",
    "habilitasteis": "habilitar",
    "habilite": "habilitar",
    "habilitemos": "habilitar",
    "habiliten": "habilitar",
    "habilites": "habilitar",
    "habilito": "habilitar",
    "habilitábamos": "habilitar",
    "habilitáis": "habilitar",
    "habilitáramos": "habilitar",
    "habilitáremos": "habilitar",
    "habilitásemos": "habilitar",
    "habilité": "habilitar",
    "habilitéis": "habilitar",
    "habilitó": "habilitar",
    "habita": "habitar",
    "habitaba": "habitar",
    "habitabais": "habitar",
    "habitaban": "habitar",
    "habitabas": "habitar",
    "habitabilidades": "habitabilidad",
    "habitables": "habitable",
    "habitaciones": "habitación",
    "habitad": "habitar",
    "habitada": "habitar",
    "habitadas": "habitar",
    "habitado": "habitar",
    "habitados": "habitar",
    "habitamos": "habitar",
    "habitan": "habitar",
    "habitando": "habitar",
    "habitantes": "habitante",
    "habitara": "habitar",
    "habitarais": "habitar",
    "habitaran": "habitar",
    "habitaras": "habitar",
    "habitare": "habitar",
    "habitareis": "habitar",
    "habitaremos": "habitar",
    "habitaren": "habitar",
    "habitares": "habitar",
    "habitaron": "habitar",
    "habitará": "habitar",
    "habitarán": "habitar",
    "habitarás": "habitar",
    "habitaré": "habitar",
    "habitaréis": "habitar",
    "habitaría": "habitar",
    "habitaríais": "habitar",
    "habitaríamos": "habitar",
    "habitarían": "habitar",
    "habitarías": "habitar",
    "habitas": "habitar",
    "habitase": "habitar",
    "habitaseis": "habitar",
    "habitasen": "habitar",
    "habitases": "habitar",
    "habitaste": "habitar",
    "habitasteis": "habitar",
    "habitats": "habitat",
    "habite": "habitar",
    "habitemos": "habitar",
    "habiten": "habitar",
    "habites": "habitar",
    "habito": "habitar",
    "habituaba": "habituar",
    "habituabais": "habituar",
    "habituaban": "habituar",
    "habituabas": "habituar",
    "habituaciones": "habituación",
    "habituad": "habituar",
    "habituada": "habituar",
    "habituadas": "habituar",
    "habituado": "habituar",
    "habituados": "habituar",
    "habituales": "habitual",
    "habitualidades": "habitualidad",
    "habituamos": "habituar",
    "habituando": "habituar",
    "habituara": "habituar",
    "habituarais": "habituar",
    "habituaran": "habituar",
    "habituaras": "habituar",
    "habituare": "habituar",
    "habituareis": "habituar",
    "habituaremos": "habituar",
    "habituaren": "habituar",
    "habituares": "habituar",
    "habituaron": "habituar",
    "habituará": "habituar",
    "habituarán": "habituar",
    "habituarás": "habituar",
    "habituaré": "habituar",
    "habituaréis": "habituar",
    "habituaría": "habituar",
    "habituaríais": "habituar",
    "habituaríamos": "habituar",
    "habituarían": "habituar",
    "habituarías": "habituar",
    "habituase": "habituar",
    "habituaseis": "habituar",
    "habituasen": "habituar",
    "habituases": "habituar",
    "habituaste": "habituar",
    "habituasteis": "habituar",
    "habitudes": "habitud",
    "habituemos": "habituar",
    "habituábamos": "habituar",
    "habituáis": "habituar",
    "habituáramos": "habituar",
    "habituáremos": "habituar",
    "habituásemos": "habituar",
    "habitué": "habituar",
    "habituéis": "habituar",
    "habituó": "habituar",
    "habitábamos": "habitar",
    "habitáculos": "habitáculo",
    "habitáis": "habitar",
    "habitáramos": "habitar",
    "habitáremos": "habitar",
    "habitásemos": "habitar",
    "habité": "habitar",
    "habitéis": "habitar",
    "habitó": "habitar",
    "habitúa": "habituar",
    "habitúan": "habituar",
    "habitúas": "habituar",
    "habitúe": "habituar",
    "habitúen": "habituar",
    "habitúes": "habituar",
    "habitúo": "habituar",
    "habla": "hablar",
    "hablaba": "hablar",
    "hablabais": "hablar",
    "hablaban": "hablar",
    "hablabas": "hablar",
    "hablad": "hablar",
    "hablada": "hablar",
    "habladas": "hablar",
    "hablado": "hablar",
    "habladora": "hablador",
    "habladoras": "hablador",
    "habladores": "hablador",
    "hablados": "hablar",
    "habladurías": "habladuría",
    "hablamos": "hablar",
    "hablan": "hablar",
    "hablando": "hablar",
    "hablantes": "hablante",
    "hablara": "hablar",
    "hablarais": "hablar",
    "hablaran": "hablar",
    "hablaras": "hablar",
    "hablare": "hablar",
    "hablareis": "hablar",
    "hablaremos": "hablar",
    "hablaren": "hablar",
    "hablares": "hablar",
    "hablaron": "hablar",
    "hablará": "hablar",
    "hablarán": "hablar",
    "hablarás": "hablar",
    "hablaré": "hablar",
    "hablaréis": "hablar",
    "hablaría": "hablar",
    "hablaríais": "hablar",
    "hablaríamos": "hablar",
    "hablarían": "hablar",
    "hablarías": "hablar",
    "hablas": "hablar",
    "hablase": "hablar",
    "hablaseis": "hablar",
    "hablasen": "hablar",
    "hablases": "hablar",
    "hablaste": "hablar",
    "hablasteis": "hablar",
    "hable": "hablar",
    "hablemos": "hablar",
    "hablen": "hablar",
    "hables": "hablar",
    "hablillas": "hablilla",
    "hablistas": "hablista",
    "hablo": "hablar",
    "hablábamos": "hablar",
    "habláis": "hablar",
    "habláramos": "hablar",
    "habláremos": "hablar",
    "hablásemos": "hablar",
    "hablé": "hablar",
    "habléis": "hablar",
    "habló": "hablar",
    "habones": "habón",
    "habremos": "haber",
    "habrá": "haber",
    "habrán": "haber",
    "habrás": "haber",
    "habré": "haber",
    "habréis": "haber",
    "habría": "haber",
    "habríais": "haber",
    "habríamos": "haber",
    "habrían": "haber",
    "habrías": "haber",
    "habuses": "habús",
    "habéis": "haber",
    "habénulas": "habénula",
    "había": "haber",
    "habíais": "haber",
    "habíamos": "haber",
    "habían": "haber",
    "habías": "haber",
    "hacamaris": "hacamari",
    "hacanes": "hacán",
    "hacas": "haca",
    "hace": "hacer",
    "haced": "hacer",
    "hacedera": "hacedero",
    "hacederas": "hacedero",
    "hacederos": "hacedero",
    "hacedora": "hacedor",
    "hacedoras": "hacedor",
    "hacedores": "hacedor",
    "hacemos": "hacer",
    "hacen": "hacer",
    "hacendaba": "hacendar",
    "hacendabais": "hacendar",
    "hacendaban": "hacendar",
    "hacendabas": "hacendar",
    "hacendad": "hacendar",
    "hacendada": "hacendar",
    "hacendadas": "hacendar",
    "hacendado": "hacendar",
    "hacendados": "hacendar",
    "hacendamos": "hacendar",
    "hacendando": "hacendar",
    "hacendara": "hacendar",
    "hacendarais": "hacendar",
    "hacendaran": "hacendar",
    "hacendaras": "hacendar",
    "hacendare": "hacendar",
    "hacendareis": "hacendar",
    "hacendaremos": "hacendar",
    "hacendaren": "hacendar",
    "hacendares": "hacendar",
    "hacendaria": "hacendario",
    "hacendarias": "hacendario",
    "hacendarios": "hacendario",
    "hacendaron": "hacendar",
    "hacendará": "hacendar",
    "hacendarán": "hacendar",
    "hacendarás": "hacendar",
    "hacendaré": "hacendar",
    "hacendaréis": "hacendar",
    "hacendaría": "hacendar",
    "hacendaríais": "hacendar",
    "hacendaríamos": "hacendar",
    "hacendarían": "hacendar",
    "hacendarías": "hacendar",
    "hacendase": "hacendar",
    "hacendaseis": "hacendar",
    "hacendasen": "hacendar",
    "hacendases": "hacendar",
    "hacendaste": "hacendar",
    "hacendasteis": "hacendar",
    "hacendemos": "hacendar",
    "hacendera": "hacendero",
    "hacenderas": "hacendero",
    "hacenderos": "hacendero",
    "hacendistas": "hacendista",
    "hacendosa": "hacendoso",
    "hacendosas": "hacendoso",
    "hacendosos": "hacendoso",
    "hacendábamos": "hacendar",
    "hacendáis": "hacendar",
    "hacendáramos": "hacendar",
    "hacendáremos": "hacendar",
    "hacendásemos": "hacendar",
    "hacendé": "hacendar",
    "hacendéis": "hacendar",
    "hacendística": "hacendístico",
    "hacendísticas": "hacendístico",
    "hacendísticos": "hacendístico",
    "hacendó": "hacendar",
    "haces": "haz",
    "hacha": "hachar",
    "hachaba": "hachar",
    "hachabais": "hachar",
    "hachaban": "hachar",
    "hachabas": "hachar",
    "hachad": "hachar",
    "hachada": "hachar",
    "hachadas": "hachar",
    "hachado": "hachar",
    "hachados": "hachar",
    "hachamos": "hachar",
    "hachan": "hachar",
    "hachando": "hachar",
    "hachara": "hachar",
    "hacharais": "hachar",
    "hacharan": "hachar",
    "hacharas": "hachar",
    "hachare": "hachar",
    "hachareis": "hachar",
    "hacharemos": "hachar",
    "hacharen": "hachar",
    "hachares": "hachar",
    "hacharon": "hachar",
    "hachará": "hachar",
    "hacharán": "hachar",
    "hacharás": "hachar",
    "hacharé": "hachar",
    "hacharéis": "hachar",
    "hacharía": "hachar",
    "hacharíais": "hachar",
    "hacharíamos": "hachar",
    "hacharían": "hachar",
    "hacharías": "hachar",
    "hachas": "hachar",
    "hachase": "hachar",
    "hachaseis": "hachar",
    "hachasen": "hachar",
    "hachases": "hachar",
    "hachaste": "hachar",
    "hachasteis": "hachar",
    "hachazos": "hachazo",
    "hache": "hachar",
    "hachea": "hachear",
    "hacheaba": "hachear",
    "hacheabais": "hachear",
    "hacheaban": "hachear",
    "hacheabas": "hachear",
    "hachead": "hachear",
    "hacheada": "hachear",
    "hacheadas": "hachear",
    "hacheado": "hachear",
    "hacheados": "hachear",
    "hacheamos": "hachear",
    "hachean": "hachear",
    "hacheando": "hachear",
    "hacheara": "hachear",
    "hachearais": "hachear",
    "hachearan": "hachear",
    "hachearas": "hachear",
    "hacheare": "hachear",
    "hacheareis": "hachear",
    "hachearemos": "hachear",
    "hachearen": "hachear",
    "hacheares": "hachear",
    "hachearon": "hachear",
    "hacheará": "hachear",
    "hachearán": "hachear",
    "hachearás": "hachear",
    "hachearé": "hachear",
    "hachearéis": "hachear",
    "hachearía": "hachear",
    "hachearíais": "hachear",
    "hachearíamos": "hachear",
    "hachearían": "hachear",
    "hachearías": "hachear",
    "hacheas": "hachear",
    "hachease": "hachear",
    "hacheaseis": "hachear",
    "hacheasen": "hachear",
    "hacheases": "hachear",
    "hacheaste": "hachear",
    "hacheasteis": "hachear",
    "hachee": "hachear",
    "hacheemos": "hachear",
    "hacheen": "hachear",
    "hachees": "hachear",
    "hachemitas": "hachemita",
    "hachemos": "hachar",
    "hachen": "hachar",
    "hacheo": "hachear",
    "hachera": "hachero",
    "hacheras": "hachero",
    "hacheros": "hachero",
    "haches": "hache",
    "hacheábamos": "hachear",
    "hacheáis": "hachear",
    "hacheáramos": "hachear",
    "hacheáremos": "hachear",
    "hacheásemos": "hachear",
    "hacheé": "hachear",
    "hacheéis": "hachear",
    "hacheó": "hachear",
    "hachises": "hachís",
    "hacho": "hachar",
    "hachones": "hachón",
    "hachos": "hacho",
    "hachábamos": "hachar",
    "hacháis": "hachar",
    "hacháramos": "hachar",
    "hacháremos": "hachar",
    "hachásemos": "hachar",
    "haché": "hachar",
    "hachéis": "hachar",
    "hachó": "hachar",
    "hacienda": "hacendar",
    "haciendan": "hacendar",
    "haciendas": "hacienda",
    "haciende": "hacendar",
    "hacienden": "hacendar",
    "haciendes": "hacendar",
    "haciendo": "hacer",
    "hacina": "hacinar",
    "hacinaba": "hacinar",
    "hacinabais": "hacinar",
    "hacinaban": "hacinar",
    "hacinabas": "hacinar",
    "hacinaciones": "hacinación",
    "hacinad": "hacinar",
    "hacinada": "hacinar",
    "hacinadas": "hacinar",
    "hacinado": "hacinar",
    "hacinados": "hacinar",
    "hacinamientos": "hacinamiento",
    "hacinamos": "hacinar",
    "hacinan": "hacinar",
    "hacinando": "hacinar",
    "hacinara": "hacinar",
    "hacinarais": "hacinar",
    "hacinaran": "hacinar",
    "hacinaras": "hacinar",
    "hacinare": "hacinar",
    "hacinareis": "hacinar",
    "hacinaremos": "hacinar",
    "hacinaren": "hacinar",
    "hacinares": "hacinar",
    "hacinaron": "hacinar",
    "hacinará": "hacinar",
    "hacinarán": "hacinar",
    "hacinarás": "hacinar",
    "hacinaré": "hacinar",
    "hacinaréis": "hacinar",
    "hacinaría": "hacinar",
    "hacinaríais": "hacinar",
    "hacinaríamos": "hacinar",
    "hacinarían": "hacinar",
    "hacinarías": "hacinar",
    "hacinas": "hacinar",
    "hacinase": "hacinar",
    "hacinaseis": "hacinar",
    "hacinasen": "hacinar",
    "hacinases": "hacinar",
    "hacinaste": "hacinar",
    "hacinasteis": "hacinar",
    "hacine": "hacinar",
    "hacinemos": "hacinar",
    "hacinen": "hacinar",
    "hacines": "hacinar",
    "hacino": "hacinar",
    "hacinábamos": "hacinar",
    "hacináis": "hacinar",
    "hacináramos": "hacinar",
    "hacináremos": "hacinar",
    "hacinásemos": "hacinar",
    "haciné": "hacinar",
    "hacinéis": "hacinar",
    "hacinó": "hacinar",
    "hacéis": "hacer",
    "hacía": "hacer",
    "hacíais": "hacer",
    "hacíamos": "hacer",
    "hacían": "hacer",
    "hacías": "hacer",
    "hadada": "hadado",
    "hadadas": "hadado",
    "hadados": "hadado",
    "hadas": "hada",
    "hados": "hado",
    "hadrones": "hadrón",
    "hafices": "hafiz",
    "hafnios": "hafnio",
    "haga": "hacer",
    "hagamos": "hacer",
    "hagan": "hacer",
    "hagas": "hacer",
    "hagiografías": "hagiografía",
    "hagiográfica": "hagiográfico",
    "hagiográficas": "hagiográfico",
    "hagiográficos": "hagiográfico",
    "hagiologías": "hagiología",
    "hagiógrafa": "hagiógrafo",
    "hagiógrafas": "hagiógrafo",
    "hagiógrafos": "hagiógrafo",
    "hagióloga": "hagiólogo",
    "hagiólogas": "hagiólogo",
    "hagiólogos": "hagiólogo",
    "hago": "hacer",
    "hagáis": "hacer",
    "hahnios": "hahnio",
    "haikais": "haikai",
    "haikus": "haiku",
    "haikúses": "haikú",
    "haitiana": "haitiano",
    "haitianas": "haitiano",
    "haitianos": "haitiano",
    "hala": "halar",
    "halaba": "halar",
    "halabais": "halar",
    "halaban": "halar",
    "halabas": "halar",
    "halad": "halar",
    "halada": "halar",
    "haladas": "halar",
    "halado": "halar",
    "halados": "halar",
    "halaga": "halagar",
    "halagaba": "halagar",
    "halagabais": "halagar",
    "halagaban": "halagar",
    "halagabas": "halagar",
    "halagad": "halagar",
    "halagada": "halagar",
    "halagadas": "halagar",
    "halagado": "halagar",
    "halagadora": "halagador",
    "halagadoras": "halagador",
    "halagadores": "halagador",
    "halagados": "halagar",
    "halagamos": "halagar",
    "halagan": "halagar",
    "halagando": "halagar",
    "halagara": "halagar",
    "halagarais": "halagar",
    "halagaran": "halagar",
    "halagaras": "halagar",
    "halagare": "halagar",
    "halagareis": "halagar",
    "halagaremos": "halagar",
    "halagaren": "halagar",
    "halagares": "halagar",
    "halagaron": "halagar",
    "halagará": "halagar",
    "halagarán": "halagar",
    "halagarás": "halagar",
    "halagaré": "halagar",
    "halagaréis": "halagar",
    "halagaría": "halagar",
    "halagaríais": "halagar",
    "halagaríamos": "halagar",
    "halagarían": "halagar",
    "halagarías": "halagar",
    "halagas": "halagar",
    "halagase": "halagar",
    "halagaseis": "halagar",
    "halagasen": "halagar",
    "halagases": "halagar",
    "halagaste": "halagar",
    "halagasteis": "halagar",
    "halago": "halagar",
    "halagos": "halago",
    "halague": "halagar",
    "halaguemos": "halagar",
    "halaguen": "halagar",
    "halagues": "halagar",
    "halagué": "halagar",
    "halaguéis": "halagar",
    "halagábamos": "halagar",
    "halagáis": "halagar",
    "halagáramos": "halagar",
    "halagáremos": "halagar",
    "halagásemos": "halagar",
    "halagó": "halagar",
    "halagüeña": "halagüeño",
    "halagüeñas": "halagüeño",
    "halagüeños": "halagüeño",
    "halamos": "halar",
    "halan": "halar",
    "halando": "halar",
    "halara": "halar",
    "halarais": "halar",
    "halaran": "halar",
    "halaras": "halar",
    "halare": "halar",
    "halareis": "halar",
    "halaremos": "halar",
    "halaren": "halar",
    "halares": "halar",
    "halaron": "halar",
    "halará": "halar",
    "halarán": "halar",
    "halarás": "halar",
    "halaré": "halar",
    "halaréis": "halar",
    "halaría": "halar",
    "halaríais": "halar",
    "halaríamos": "halar",
    "halarían": "halar",
    "halarías": "halar",
    "halas": "halar",
    "halase": "halar",
    "halaseis": "halar",
    "halasen": "halar",
    "halases": "halar",
    "halaste": "halar",
    "halasteis": "halar",
    "halazonas": "halazona",
    "halconera": "halconero",
    "halconeras": "halconero",
    "halconeros": "halconero",
    "halconerías": "halconería",
    "halcones": "halcón",
    "haldadas": "haldada",
    "haldas": "halda",
    "hale": "halar",
    "halemos": "halar",
    "halen": "halar",
    "hales": "halar",
    "halibutes": "halibut",
    "halibuts": "halibut",
    "halitas": "halita",
    "halla": "hallar",
    "hallaba": "hallar",
    "hallabais": "hallar",
    "hallaban": "hallar",
    "hallabas": "hallar",
    "hallad": "hallar",
    "hallada": "hallar",
    "halladas": "hallar",
    "hallado": "hallar",
    "hallados": "hallar",
    "hallamos": "hallar",
    "hallan": "hallar",
    "hallando": "hallar",
    "hallara": "hallar",
    "hallarais": "hallar",
    "hallaran": "hallar",
    "hallaras": "hallar",
    "hallare": "hallar",
    "hallareis": "hallar",
    "hallaremos": "hallar",
    "hallaren": "hallar",
    "hallares": "hallar",
    "hallaron": "hallar",
    "hallará": "hallar",
    "hallarán": "hallar",
    "hallarás": "hallar",
    "hallaré": "hallar",
    "hallaréis": "hallar",
    "hallaría": "hallar",
    "hallaríais": "hallar",
    "hallaríamos": "hallar",
    "hallarían": "hallar",
    "hallarías": "hallar",
    "hallas": "hallar",
    "hallase": "hallar",
    "hallaseis": "hallar",
    "hallasen": "hallar",
    "hallases": "hallar",
    "hallaste": "hallar",
    "hallasteis": "hallar",
    "hallazgos": "hallazgo",
    "halle": "hallar",
    "hallemos": "hallar",
    "hallen": "hallar",
    "halles": "hallar",
    "hallo": "hallar",
    "halloysitas": "halloysita",
    "halls": "hall",
    "hallullas": "hallulla",
    "hallullos": "hallullo",
    "hallábamos": "hallar",
    "halláis": "hallar",
    "halláramos": "hallar",
    "halláremos": "hallar",
    "hallásemos": "hallar",
    "hallé": "hallar",
    "halléis": "hallar",
    "halló": "hallar",
    "halo": "halar",
    "halobacterias": "halobacteria",
    "halogena": "halogenar",
    "halogenaba": "halogenar",
    "halogenabais": "halogenar",
    "halogenaban": "halogenar",
    "halogenabas": "halogenar",
    "halogenaciones": "halogenación",
    "halogenad": "halogenar",
    "halogenada": "halogenar",
    "halogenadas": "halogenar",
    "halogenado": "halogenar",
    "halogenados": "halogenar",
    "halogenamos": "halogenar",
    "halogenan": "halogenar",
    "halogenando": "halogenar",
    "halogenara": "halogenar",
    "halogenarais": "halogenar",
    "halogenaran": "halogenar",
    "halogenaras": "halogenar",
    "halogenare": "halogenar",
    "halogenareis": "halogenar",
    "halogenaremos": "halogenar",
    "halogenaren": "halogenar",
    "halogenares": "halogenar",
    "halogenaron": "halogenar",
    "halogenará": "halogenar",
    "halogenarán": "halogenar",
    "halogenarás": "halogenar",
    "halogenaré": "halogenar",
    "halogenaréis": "halogenar",
    "halogenaría": "halogenar",
    "halogenaríais": "halogenar",
    "halogenaríamos": "halogenar",
    "halogenarían": "halogenar",
    "halogenarías": "halogenar",
    "halogenas": "halogenar",
    "halogenase": "halogenar",
    "halogenaseis": "halogenar",
    "halogenasen": "halogenar",
    "halogenases": "halogenar",
    "halogenaste": "halogenar",
    "halogenasteis": "halogenar",
    "halogene": "halogenar",
    "halogenemos": "halogenar",
    "halogenen": "halogenar",
    "halogenes": "halogenar",
    "halogeno": "halogenar",
    "halogenábamos": "halogenar",
    "halogenáis": "halogenar",
    "halogenáramos": "halogenar",
    "halogenáremos": "halogenar",
    "halogenásemos": "halogenar",
    "halogené": "halogenar",
    "halogenéis": "halogenar",
    "halogenó": "halogenar",
    "halografías": "halografía",
    "halográfica": "halográfico",
    "halográficas": "halográfico",
    "halográficos": "halográfico",
    "haloidea": "haloideo",
    "haloideas": "haloideo",
    "haloideos": "haloideo",
    "halones": "halón",
    "halorragidáceas": "halorragidácea",
    "halos": "halo",
    "halotecnias": "halotecnia",
    "halotriquitas": "halotriquita",
    "halotécnica": "halotécnico",
    "halotécnicas": "halotécnico",
    "halotécnicos": "halotécnico",
    "haloysitas": "haloysita",
    "halozas": "haloza",
    "halteras": "haltera",
    "halterofilias": "halterofilia",
    "halterofilistas": "halterofilista",
    "halterófila": "halterófilo",
    "halterófilas": "halterófilo",
    "halterófilos": "halterófilo",
    "haluros": "haluro",
    "halábamos": "halar",
    "haláis": "halar",
    "haláramos": "halar",
    "haláremos": "halar",
    "halásemos": "halar",
    "halé": "halar",
    "haléis": "halar",
    "haló": "halar",
    "halófila": "halófilo",
    "halófilas": "halófilo",
    "halófilos": "halófilo",
    "halófitos": "halófito",
    "halógena": "halógeno",
    "halógenas": "halógeno",
    "halógenos": "halógeno",
    "hamacas": "hamaca",
    "hamadríadas": "hamadríada",
    "hamadríades": "hamadríade",
    "hamadrías": "hamadría",
    "hamamelidáceas": "hamamelidácea",
    "hamaquea": "hamaquear",
    "hamaqueaba": "hamaquear",
    "hamaqueabais": "hamaquear",
    "hamaqueaban": "hamaquear",
    "hamaqueabas": "hamaquear",
    "hamaquead": "hamaquear",
    "hamaqueada": "hamaquear",
    "hamaqueadas": "hamaquear",
    "hamaqueado": "hamaquear",
    "hamaqueados": "hamaquear",
    "hamaqueamos": "hamaquear",
    "hamaquean": "hamaquear",
    "hamaqueando": "hamaquear",
    "hamaqueara": "hamaquear",
    "hamaquearais": "hamaquear",
    "hamaquearan": "hamaquear",
    "hamaquearas": "hamaquear",
    "hamaqueare": "hamaquear",
    "hamaqueareis": "hamaquear",
    "hamaquearemos": "hamaquear",
    "hamaquearen": "hamaquear",
    "hamaqueares": "hamaquear",
    "hamaquearon": "hamaquear",
    "hamaqueará": "hamaquear",
    "hamaquearán": "hamaquear",
    "hamaquearás": "hamaquear",
    "hamaquearé": "hamaquear",
    "hamaquearéis": "hamaquear",
    "hamaquearía": "hamaquear",
    "hamaquearíais": "hamaquear",
    "hamaquearíamos": "hamaquear",
    "hamaquearían": "hamaquear",
    "hamaquearías": "hamaquear",
    "hamaqueas": "hamaquear",
    "hamaquease": "hamaquear",
    "hamaqueaseis": "hamaquear",
    "hamaqueasen": "hamaquear",
    "hamaqueases": "hamaquear",
    "hamaqueaste": "hamaquear",
    "hamaqueasteis": "hamaquear",
    "hamaquee": "hamaquear",
    "hamaqueemos": "hamaquear",
    "hamaqueen": "hamaquear",
    "hamaquees": "hamaquear",
    "hamaqueo": "hamaquear",
    "hamaquera": "hamaquero",
    "hamaqueras": "hamaquero",
    "hamaqueros": "hamaquero",
    "hamaqueábamos": "hamaquear",
    "hamaqueáis": "hamaquear",
    "hamaqueáramos": "hamaquear",
    "hamaqueáremos": "hamaquear",
    "hamaqueásemos": "hamaquear",
    "hamaqueé": "hamaquear",
    "hamaqueéis": "hamaquear",
    "hamaqueó": "hamaquear",
    "hambergitas": "hambergita",
    "hambrea": "hambrear",
    "hambreaba": "hambrear",
    "hambreabais": "hambrear",
    "hambreaban": "hambrear",
    "hambreabas": "hambrear",
    "hambread": "hambrear",
    "hambreada": "hambrear",
    "hambreadas": "hambrear",
    "hambreado": "hambrear",
    "hambreados": "hambrear",
    "hambreamos": "hambrear",
    "hambrean": "hambrear",
    "hambreando": "hambrear",
    "hambreara": "hambrear",
    "hambrearais": "hambrear",
    "hambrearan": "hambrear",
    "hambrearas": "hambrear",
    "hambreare": "hambrear",
    "hambreareis": "hambrear",
    "hambrearemos": "hambrear",
    "hambrearen": "hambrear",
    "hambreares": "hambrear",
    "hambrearon": "hambrear",
    "hambreará": "hambrear",
    "hambrearán": "hambrear",
    "hambrearás": "hambrear",
    "hambrearé": "hambrear",
    "hambrearéis": "hambrear",
    "hambrearía": "hambrear",
    "hambrearíais": "hambrear",
    "hambrearíamos": "hambrear",
    "hambrearían": "hambrear",
    "hambrearías": "hambrear",
    "hambreas": "hambrear",
    "hambrease": "hambrear",
    "hambreaseis": "hambrear",
    "hambreasen": "hambrear",
    "hambreases": "hambrear",
    "hambreaste": "hambrear",
    "hambreasteis": "hambrear",
    "hambree": "hambrear",
    "hambreemos": "hambrear",
    "hambreen": "hambrear",
    "hambrees": "hambrear",
    "hambreo": "hambrear",
    "hambres": "hambre",
    "hambreábamos": "hambrear",
    "hambreáis": "hambrear",
    "hambreáramos": "hambrear",
    "hambreáremos": "hambrear",
    "hambreásemos": "hambrear",
    "hambreé": "hambrear",
    "hambreéis": "hambrear",
    "hambreó": "hambrear",
    "hambrienta": "hambriento",
    "hambrientas": "hambriento",
    "hambrientos": "hambriento",
    "hambrona": "hambrón",
    "hambronas": "hambrón",
    "hambrones": "hambrón",
    "hambrunas": "hambruna",
    "hamburguesa": "hamburgués",
    "hamburguesas": "hamburguesa",
    "hamburgueserías": "hamburguesería",
    "hamburgueses": "hamburgués",
    "hampas": "hampa",
    "hampesca": "hampesco",
    "hampescas": "hampesco",
    "hampescos": "hampesco",
    "hampona": "hampón",
    "hamponas": "hampón",
    "hampones": "hampón",
    "hamsters": "hámster",
    "hamulares": "hamular",
    "han": "haber",
    "handicapa": "handicapar",
    "handicapaba": "handicapar",
    "handicapabais": "handicapar",
    "handicapaban": "handicapar",
    "handicapabas": "handicapar",
    "handicapad": "handicapar",
    "handicapada": "handicapar",
    "handicapadas": "handicapar",
    "handicapado": "handicapar",
    "handicapados": "handicapar",
    "handicapamos": "handicapar",
    "handicapan": "handicapar",
    "handicapando": "handicapar",
    "handicapara": "handicapar",
    "handicaparais": "handicapar",
    "handicaparan": "handicapar",
    "handicaparas": "handicapar",
    "handicapare": "handicapar",
    "handicapareis": "handicapar",
    "handicaparemos": "handicapar",
    "handicaparen": "handicapar",
    "handicapares": "handicapar",
    "handicaparon": "handicapar",
    "handicapará": "handicapar",
    "handicaparán": "handicapar",
    "handicaparás": "handicapar",
    "handicaparé": "handicapar",
    "handicaparéis": "handicapar",
    "handicaparía": "handicapar",
    "handicaparíais": "handicapar",
    "handicaparíamos": "handicapar",
    "handicaparían": "handicapar",
    "handicaparías": "handicapar",
    "handicapas": "handicapar",
    "handicapase": "handicapar",
    "handicapaseis": "handicapar",
    "handicapasen": "handicapar",
    "handicapases": "handicapar",
    "handicapaste": "handicapar",
    "handicapasteis": "handicapar",
    "handicape": "handicapar",
    "handicapemos": "handicapar",
    "handicapen": "handicapar",
    "handicapes": "handicapar",
    "handicapo": "handicapar",
    "handicaps": "handicap",
    "handicapábamos": "handicapar",
    "handicapáis": "handicapar",
    "handicapáramos": "handicapar",
    "handicapáremos": "handicapar",
    "handicapásemos": "handicapar",
    "handicapé": "handicapar",
    "handicapéis": "handicapar",
    "handicapó": "handicapar",
    "hanegas": "hanega",
    "hangares": "hangar",
    "hannoveriana": "hannoveriano",
    "hannoverianas": "hannoveriano",
    "hannoverianos": "hannoveriano",
    "hanseática": "hanseático",
    "hanseáticas": "hanseático",
    "hanseáticos": "hanseático",
    "haplobiontes": "haplobionte",
    "haplodiplobiontes": "haplodiplobionte",
    "haplodonta": "haplodonto",
    "haplodontas": "haplodonto",
    "haplodontos": "haplodonto",
    "haploides": "haploide",
    "haploidias": "haploidia",
    "haploidizaciones": "haploidización",
    "haploidías": "haploidía",
    "haplolalias": "haplolalia",
    "haplologías": "haplología",
    "haplontes": "haplonte",
    "haplopoliploides": "haplopoliploide",
    "haploscopios": "haploscopio",
    "haplotipos": "haplotipo",
    "haptenos": "hapteno",
    "haptoglobinas": "haptoglobina",
    "haptógena": "haptógeno",
    "haptógenas": "haptógeno",
    "haptógenos": "haptógeno",
    "hapálidos": "hapálido",
    "haragana": "haragán",
    "haraganas": "haragán",
    "haraganea": "haraganear",
    "haraganeaba": "haraganear",
    "haraganeabais": "haraganear",
    "haraganeaban": "haraganear",
    "haraganeabas": "haraganear",
    "haraganead": "haraganear",
    "haraganeada": "haraganear",
    "haraganeadas": "haraganear",
    "haraganeado": "haraganear",
    "haraganeados": "haraganear",
    "haraganeamos": "haraganear",
    "haraganean": "haraganear",
    "haraganeando": "haraganear",
    "haraganeara": "haraganear",
    "haraganearais": "haraganear",
    "haraganearan": "haraganear",
    "haraganearas": "haraganear",
    "haraganeare": "haraganear",
    "haraganeareis": "haraganear",
    "haraganearemos": "haraganear",
    "haraganearen": "haraganear",
    "haraganeares": "haraganear",
    "haraganearon": "haraganear",
    "haraganeará": "haraganear",
    "haraganearán": "haraganear",
    "haraganearás": "haraganear",
    "haraganearé": "haraganear",
    "haraganearéis": "haraganear",
    "haraganearía": "haraganear",
    "haraganearíais": "haraganear",
    "haraganearíamos": "haraganear",
    "haraganearían": "haraganear",
    "haraganearías": "haraganear",
    "haraganeas": "haraganear",
    "haraganease": "haraganear",
    "haraganeaseis": "haraganear",
    "haraganeasen": "haraganear",
    "haraganeases": "haraganear",
    "haraganeaste": "haraganear",
    "haraganeasteis": "haraganear",
    "haraganee": "haraganear",
    "haraganeemos": "haraganear",
    "haraganeen": "haraganear",
    "haraganees": "haraganear",
    "haraganeo": "haraganear",
    "haraganes": "haragán",
    "haraganeábamos": "haraganear",
    "haraganeáis": "haraganear",
    "haraganeáramos": "haraganear",
    "haraganeáremos": "haraganear",
    "haraganeásemos": "haraganear",
    "haraganeé": "haraganear",
    "haraganeéis": "haraganear",
    "haraganeó": "haraganear",
    "harais": "harai",
    "harapienta": "harapiento",
    "harapientas": "harapiento",
    "harapientos": "harapiento",
    "harapos": "harapo",
    "haraposa": "haraposo",
    "haraposas": "haraposo",
    "haraposos": "haraposo",
    "haraquiris": "haraquiri",
    "harcas": "harca",
    "hardwares": "hardware",
    "haremos": "hacer",
    "harenes": "harén",
    "harijas": "harija",
    "harinados": "harinado",
    "harinas": "harina",
    "harinera": "harinero",
    "harineras": "harinero",
    "harineros": "harinero",
    "harinosa": "harinoso",
    "harinosas": "harinoso",
    "harinosos": "harinoso",
    "harmonice": "harmonizar",
    "harmonicemos": "harmonizar",
    "harmonicen": "harmonizar",
    "harmonices": "harmonizar",
    "harmonicé": "harmonizar",
    "harmonicéis": "harmonizar",
    "harmoniosa": "harmonioso",
    "harmoniosas": "harmonioso",
    "harmoniosos": "harmonioso",
    "harmoniza": "harmonizar",
    "harmonizaba": "harmonizar",
    "harmonizabais": "harmonizar",
    "harmonizaban": "harmonizar",
    "harmonizabas": "harmonizar",
    "harmonizad": "harmonizar",
    "harmonizada": "harmonizar",
    "harmonizadas": "harmonizar",
    "harmonizado": "harmonizar",
    "harmonizados": "harmonizar",
    "harmonizamos": "harmonizar",
    "harmonizan": "harmonizar",
    "harmonizando": "harmonizar",
    "harmonizara": "harmonizar",
    "harmonizarais": "harmonizar",
    "harmonizaran": "harmonizar",
    "harmonizaras": "harmonizar",
    "harmonizare": "harmonizar",
    "harmonizareis": "harmonizar",
    "harmonizaremos": "harmonizar",
    "harmonizaren": "harmonizar",
    "harmonizares": "harmonizar",
    "harmonizaron": "harmonizar",
    "harmonizará": "harmonizar",
    "harmonizarán": "harmonizar",
    "harmonizarás": "harmonizar",
    "harmonizaré": "harmonizar",
    "harmonizaréis": "harmonizar",
    "harmonizaría": "harmonizar",
    "harmonizaríais": "harmonizar",
    "harmonizaríamos": "harmonizar",
    "harmonizarían": "harmonizar",
    "harmonizarías": "harmonizar",
    "harmonizas": "harmonizar",
    "harmonizase": "harmonizar",
    "harmonizaseis": "harmonizar",
    "harmonizasen": "harmonizar",
    "harmonizases": "harmonizar",
    "harmonizaste": "harmonizar",
    "harmonizasteis": "harmonizar",
    "harmonizo": "harmonizar",
    "harmonizábamos": "harmonizar",
    "harmonizáis": "harmonizar",
    "harmonizáramos": "harmonizar",
    "harmonizáremos": "harmonizar",
    "harmonizásemos": "harmonizar",
    "harmonizó": "harmonizar",
    "harmonías": "harmonía",
    "harmotomas": "harmotoma",
    "harnales": "harnal",
    "harnerera": "harnerero",
    "harnereras": "harnerero",
    "harnereros": "harnerero",
    "harneros": "harnero",
    "harneruelos": "harneruelo",
    "harona": "harón",
    "haronas": "harón",
    "haronea": "haronear",
    "haroneaba": "haronear",
    "haroneabais": "haronear",
    "haroneaban": "haronear",
    "haroneabas": "haronear",
    "haronead": "haronear",
    "haroneada": "haronear",
    "haroneadas": "haronear",
    "haroneado": "haronear",
    "haroneados": "haronear",
    "haroneamos": "haronear",
    "haronean": "haronear",
    "haroneando": "haronear",
    "haroneara": "haronear",
    "haronearais": "haronear",
    "haronearan": "haronear",
    "haronearas": "haronear",
    "haroneare": "haronear",
    "haroneareis": "haronear",
    "haronearemos": "haronear",
    "haronearen": "haronear",
    "haroneares": "haronear",
    "haronearon": "haronear",
    "haroneará": "haronear",
    "haronearán": "haronear",
    "haronearás": "haronear",
    "haronearé": "haronear",
    "haronearéis": "haronear",
    "haronearía": "haronear",
    "haronearíais": "haronear",
    "haronearíamos": "haronear",
    "haronearían": "haronear",
    "haronearías": "haronear",
    "haroneas": "haronear",
    "haronease": "haronear",
    "haroneaseis": "haronear",
    "haroneasen": "haronear",
    "haroneases": "haronear",
    "haroneaste": "haronear",
    "haroneasteis": "haronear",
    "haronee": "haronear",
    "haroneemos": "haronear",
    "haroneen": "haronear",
    "haronees": "haronear",
    "haroneo": "haronear",
    "harones": "harón",
    "haroneábamos": "haronear",
    "haroneáis": "haronear",
    "haroneáramos": "haronear",
    "haroneáremos": "haronear",
    "haroneásemos": "haronear",
    "haroneé": "haronear",
    "haroneéis": "haronear",
    "haroneó": "haronear",
    "haronías": "haronía",
    "harpilleras": "harpillera",
    "harpías": "harpía",
    "harqueña": "harqueño",
    "harqueñas": "harqueño",
    "harqueños": "harqueño",
    "harrados": "harrado",
    "harta": "harto",
    "hartaba": "hartar",
    "hartabais": "hartar",
    "hartaban": "hartar",
    "hartabas": "hartar",
    "hartad": "hartar",
    "hartada": "hartar",
    "hartadas": "hartar",
    "hartado": "hartar",
    "hartados": "hartar",
    "hartamos": "hartar",
    "hartan": "hartar",
    "hartando": "hartar",
    "hartara": "hartar",
    "hartarais": "hartar",
    "hartaran": "hartar",
    "hartaras": "hartar",
    "hartare": "hartar",
    "hartareis": "hartar",
    "hartaremos": "hartar",
    "hartaren": "hartar",
    "hartares": "hartar",
    "hartaron": "hartar",
    "hartará": "hartar",
    "hartarán": "hartar",
    "hartarás": "hartar",
    "hartaré": "hartar",
    "hartaréis": "hartar",
    "hartaría": "hartar",
    "hartaríais": "hartar",
    "hartaríamos": "hartar",
    "hartarían": "hartar",
    "hartarías": "hartar",
    "hartas": "harto",
    "hartase": "hartar",
    "hartaseis": "hartar",
    "hartasen": "hartar",
    "hartases": "hartar",
    "hartaste": "hartar",
    "hartasteis": "hartar",
    "hartazgos": "hartazgo",
    "hartazones": "hartazón",
    "harte": "hartar",
    "hartemos": "hartar",
    "harten": "hartar",
    "harteras": "hartera",
    "hartes": "hartar",
    "harto": "hartar",
    "hartones": "hartón",
    "hartos": "harto",
    "harturas": "hartura",
    "hartábamos": "hartar",
    "hartáis": "hartar",
    "hartáramos": "hartar",
    "hartáremos": "hartar",
    "hartásemos": "hartar",
    "harté": "hartar",
    "hartéis": "hartar",
    "hartó": "hartar",
    "hará": "hacer",
    "harán": "hacer",
    "harás": "hacer",
    "haré": "hacer",
    "haréis": "hacer",
    "haría": "hacer",
    "haríais": "hacer",
    "haríamos": "hacer",
    "harían": "hacer",
    "harías": "hacer",
    "has": "haber",
    "hasaníes": "hasaní",
    "hasanís": "hasaní",
    "hashimíes": "hashimí",
    "hashimís": "hashimí",
    "hastiaba": "hastiar",
    "hastiabais": "hastiar",
    "hastiaban": "hastiar",
    "hastiabas": "hastiar",
    "hastiad": "hastiar",
    "hastiada": "hastiar",
    "hastiadas": "hastiar",
    "hastiado": "hastiar",
    "hastiados": "hastiar",
    "hastiales": "hastial",
    "hastiamos": "hastiar",
    "hastiando": "hastiar",
    "hastiara": "hastiar",
    "hastiarais": "hastiar",
    "hastiaran": "hastiar",
    "hastiaras": "hastiar",
    "hastiare": "hastiar",
    "hastiareis": "hastiar",
    "hastiaremos": "hastiar",
    "hastiaren": "hastiar",
    "hastiares": "hastiar",
    "hastiaron": "hastiar",
    "hastiará": "hastiar",
    "hastiarán": "hastiar",
    "hastiarás": "hastiar",
    "hastiaré": "hastiar",
    "hastiaréis": "hastiar",
    "hastiaría": "hastiar",
    "hastiaríais": "hastiar",
    "hastiaríamos": "hastiar",
    "hastiarían": "hastiar",
    "hastiarías": "hastiar",
    "hastiase": "hastiar",
    "hastiaseis": "hastiar",
    "hastiasen": "hastiar",
    "hastiases": "hastiar",
    "hastiaste": "hastiar",
    "hastiasteis": "hastiar",
    "hastiemos": "hastiar",
    "hastiábamos": "hastiar",
    "hastiáis": "hastiar",
    "hastiáramos": "hastiar",
    "hastiáremos": "hastiar",
    "hastiásemos": "hastiar",
    "hastié": "hastiar",
    "hastiéis": "hastiar",
    "hastió": "hastiar",
    "hastía": "hastiar",
    "hastían": "hastiar",
    "hastías": "hastiar",
    "hastíe": "hastiar",
    "hastíen": "hastiar",
    "hastíes": "hastiar",
    "hastío": "hastiar",
    "hastíos": "hastío",
    "hatacas": "hataca",
    "hatajos": "hatajo",
    "hatea": "hatear",
    "hateaba": "hatear",
    "hateabais": "hatear",
    "hateaban": "hatear",
    "hateabas": "hatear",
    "hatead": "hatear",
    "hateada": "hatear",
    "hateadas": "hatear",
    "hateado": "hatear",
    "hateados": "hatear",
    "hateamos": "hatear",
    "hatean": "hatear",
    "hateando": "hatear",
    "hateara": "hatear",
    "hatearais": "hatear",
    "hatearan": "hatear",
    "hatearas": "hatear",
    "hateare": "hatear",
    "hateareis": "hatear",
    "hatearemos": "hatear",
    "hatearen": "hatear",
    "hateares": "hatear",
    "hatearon": "hatear",
    "hateará": "hatear",
    "hatearán": "hatear",
    "hatearás": "hatear",
    "hatearé": "hatear",
    "hatearéis": "hatear",
    "hatearía": "hatear",
    "hatearíais": "hatear",
    "hatearíamos": "hatear",
    "hatearían": "hatear",
    "hatearías": "hatear",
    "hateas": "hatear",
    "hatease": "hatear",
    "hateaseis": "hatear",
    "hateasen": "hatear",
    "hateases": "hatear",
    "hateaste": "hatear",
    "hateasteis": "hatear",
    "hatee": "hatear",
    "hateemos": "hatear",
    "hateen": "hatear",
    "hatees": "hatear",
    "hateo": "hatear",
    "hatera": "hatero",
    "hateras": "hatero",
    "hateros": "hatero",
    "haterías": "hatería",
    "hateábamos": "hatear",
    "hateáis": "hatear",
    "hateáramos": "hatear",
    "hateáremos": "hatear",
    "hateásemos": "hatear",
    "hateé": "hatear",
    "hateéis": "hatear",
    "hateó": "hatear",
    "hatijos": "hatijo",
    "hatillos": "hatillo",
    "hatos": "hato",
    "haueritas": "hauerita",
    "hausas": "hausa",
    "hausmannitas": "hausmannita",
    "haustorios": "haustorio",
    "hautes": "haute",
    "hauynas": "hauyna",
    "haversenos": "haverseno",
    "hawaiana": "hawaiano",
    "hawaianas": "hawaiano",
    "hawaianos": "hawaiano",
    "hay": "haber",
    "haya": "haber",
    "hayales": "hayal",
    "hayamos": "haber",
    "hayan": "haber",
    "hayas": "haya",
    "hayedos": "hayedo",
    "hayones": "hayón",
    "hayos": "hayo",
    "hayucos": "hayuco",
    "hayáis": "haber",
    "haz": "hacer",
    "hazalejas": "hazaleja",
    "hazas": "haza",
    "hazañas": "hazaña",
    "hazañerías": "hazañería",
    "hazañosa": "hazañoso",
    "hazañosas": "hazañoso",
    "hazañosos": "hazañoso",
    "hazmerreíres": "hazmerreír",
    "he": "haber",
    "hebdomadaria": "hebdomadario",
    "hebdomadarias": "hebdomadario",
    "hebdomadarios": "hebdomadario",
    "hebdómadas": "hebdómada",
    "hebenes": "hebén",
    "hebijones": "hebijón",
    "hebillada": "hebillado",
    "hebilladas": "hebillado",
    "hebillados": "hebillado",
    "hebillajes": "hebillaje",
    "hebillas": "hebilla",
    "hebillera": "hebillero",
    "hebilleras": "hebillero",
    "hebilleros": "hebillero",
    "hebraica": "hebraico",
    "hebraicas": "hebraico",
    "hebraice": "hebraizar",
    "hebraicemos": "hebraizar",
    "hebraicen": "hebraizar",
    "hebraices": "hebraizar",
    "hebraicos": "hebraico",
    "hebraicé": "hebraizar",
    "hebraicéis": "hebraizar",
    "hebraiza": "hebraizar",
    "hebraizaba": "hebraizar",
    "hebraizabais": "hebraizar",
    "hebraizaban": "hebraizar",
    "hebraizabas": "hebraizar",
    "hebraizad": "hebraizar",
    "hebraizada": "hebraizar",
    "hebraizadas": "hebraizar",
    "hebraizado": "hebraizar",
    "hebraizados": "hebraizar",
    "hebraizamos": "hebraizar",
    "hebraizan": "hebraizar",
    "hebraizando": "hebraizar",
    "hebraizantes": "hebraizante",
    "hebraizara": "hebraizar",
    "hebraizarais": "hebraizar",
    "hebraizaran": "hebraizar",
    "hebraizaras": "hebraizar",
    "hebraizare": "hebraizar",
    "hebraizareis": "hebraizar",
    "hebraizaremos": "hebraizar",
    "hebraizaren": "hebraizar",
    "hebraizares": "hebraizar",
    "hebraizaron": "hebraizar",
    "hebraizará": "hebraizar",
    "hebraizarán": "hebraizar",
    "hebraizarás": "hebraizar",
    "hebraizaré": "hebraizar",
    "hebraizaréis": "hebraizar",
    "hebraizaría": "hebraizar",
    "hebraizaríais": "hebraizar",
    "hebraizaríamos": "hebraizar",
    "hebraizarían": "hebraizar",
    "hebraizarías": "hebraizar",
    "hebraizas": "hebraizar",
    "hebraizase": "hebraizar",
    "hebraizaseis": "hebraizar",
    "hebraizasen": "hebraizar",
    "hebraizases": "hebraizar",
    "hebraizaste": "hebraizar",
    "hebraizasteis": "hebraizar",
    "hebraizo": "hebraizar",
    "hebraizábamos": "hebraizar",
    "hebraizáis": "hebraizar",
    "hebraizáramos": "hebraizar",
    "hebraizáremos": "hebraizar",
    "hebraizásemos": "hebraizar",
    "hebraizó": "hebraizar",
    "hebras": "hebra",
    "hebraísmos": "hebraísmo",
    "hebraístas": "hebraísta",
    "hebrea": "hebreo",
    "hebreas": "hebreo",
    "hebreos": "hebreo",
    "hebrosa": "hebroso",
    "hebrosas": "hebroso",
    "hebrosos": "hebroso",
    "hecatombes": "hecatombe",
    "heces": "hez",
    "hecha": "hacer",
    "hechas": "hacer",
    "hechice": "hechizar",
    "hechicemos": "hechizar",
    "hechicen": "hechizar",
    "hechicera": "hechicero",
    "hechiceras": "hechicero",
    "hechiceros": "hechicero",
    "hechicerías": "hechicería",
    "hechices": "hechizar",
    "hechicé": "hechizar",
    "hechicéis": "hechizar",
    "hechiza": "hechizo",
    "hechizaba": "hechizar",
    "hechizabais": "hechizar",
    "hechizaban": "hechizar",
    "hechizabas": "hechizar",
    "hechizad": "hechizar",
    "hechizada": "hechizar",
    "hechizadas": "hechizar",
    "hechizado": "hechizar",
    "hechizadora": "hechizador",
    "hechizadoras": "hechizador",
    "hechizadores": "hechizador",
    "hechizados": "hechizar",
    "hechizamos": "hechizar",
    "hechizan": "hechizar",
    "hechizando": "hechizar",
    "hechizara": "hechizar",
    "hechizarais": "hechizar",
    "hechizaran": "hechizar",
    "hechizaras": "hechizar",
    "hechizare": "hechizar",
    "hechizareis": "hechizar",
    "hechizaremos": "hechizar",
    "hechizaren": "hechizar",
    "hechizares": "hechizar",
    "hechizaron": "hechizar",
    "hechizará": "hechizar",
    "hechizarán": "hechizar",
    "hechizarás": "hechizar",
    "hechizaré": "hechizar",
    "hechizaréis": "hechizar",
    "hechizaría": "hechizar",
    "hechizaríais": "hechizar",
    "hechizaríamos": "hechizar",
    "hechizarían": "hechizar",
    "hechizarías": "hechizar",
    "hechizas": "hechizo",
    "hechizase": "hechizar",
    "hechizaseis": "hechizar",
    "hechizasen": "hechizar",
    "hechizases": "hechizar",
    "hechizaste": "hechizar",
    "hechizasteis": "hechizar",
    "hechizo": "hechizar",
    "hechizos": "hechizo",
    "hechizábamos": "hechizar",
    "hechizáis": "hechizar",
    "hechizáramos": "hechizar",
    "hechizáremos": "hechizar",
    "hechizásemos": "hechizar",
    "hechizó": "hechizar",
    "hecho": "hacer",
    "hechos": "hecho",
    "hechuras": "hechura",
    "hect": "hectárea",
    "hectiqueces": "hectiquez",
    "hectografías": "hectografía",
    "hectogramas": "hectograma",
    "hectogramos": "hectogramo",
    "hectográfica": "hectográfico",
    "hectográficas": "hectográfico",
    "hectográficos": "hectográfico",
    "hectolitros": "hectolitro",
    "hectopascales": "hectopascal",
    "hectopascals": "hectopascal",
    "hectoritas": "hectorita",
    "hectáreas": "hectárea",
    "hectógrafos": "hectógrafo",
    "hectógramos": "hectógramo",
    "hectólitros": "hectólitro",
    "hectómetros": "hectómetro",
    "hedamos": "heder",
    "heded": "heder",
    "hedemos": "heder",
    "hedentinas": "hedentina",
    "hederemos": "heder",
    "hederá": "heder",
    "hederán": "heder",
    "hederás": "heder",
    "hederé": "heder",
    "hederéis": "heder",
    "hedería": "heder",
    "hederíais": "heder",
    "hederíamos": "heder",
    "hederían": "heder",
    "hederías": "heder",
    "hedida": "heder",
    "hedidas": "heder",
    "hedido": "heder",
    "hedidos": "heder",
    "hediendo": "heder",
    "hedienta": "hediento",
    "hedientas": "hediento",
    "hedientos": "hediento",
    "hediera": "heder",
    "hedierais": "heder",
    "hedieran": "heder",
    "hedieras": "heder",
    "hediere": "heder",
    "hediereis": "heder",
    "hedieren": "heder",
    "hedieres": "heder",
    "hedieron": "heder",
    "hediese": "heder",
    "hedieseis": "heder",
    "hediesen": "heder",
    "hedieses": "heder",
    "hedimos": "heder",
    "hedionda": "hediondo",
    "hediondas": "hediondo",
    "hediondeces": "hediondez",
    "hediondos": "hediondo",
    "hediste": "heder",
    "hedisteis": "heder",
    "hediéramos": "heder",
    "hediéremos": "heder",
    "hediésemos": "heder",
    "hedió": "heder",
    "hedonismos": "hedonismo",
    "hedonistas": "hedonista",
    "hedonística": "hedonístico",
    "hedonísticas": "hedonístico",
    "hedonísticos": "hedonístico",
    "hedores": "hedor",
    "hedáis": "heder",
    "hedéis": "heder",
    "hedí": "heder",
    "hedía": "heder",
    "hedíais": "heder",
    "hedíamos": "heder",
    "hedían": "heder",
    "hedías": "heder",
    "hedónica": "hedónico",
    "hedónicas": "hedónico",
    "hedónicos": "hedónico",
    "hegeliana": "hegeliano",
    "hegelianas": "hegeliano",
    "hegelianismos": "hegelianismo",
    "hegelianos": "hegeliano",
    "hegemonías": "hegemonía",
    "hegemónica": "hegemónico",
    "hegemónicas": "hegemónico",
    "hegemónicos": "hegemónico",
    "helaba": "helar",
    "helabais": "helar",
    "helaban": "helar",
    "helabas": "helar",
    "helad": "helar",
    "helada": "helar",
    "heladas": "helar",
    "heladera": "heladero",
    "heladeras": "heladero",
    "heladeros": "heladero",
    "heladerías": "heladería",
    "heladiza": "heladizo",
    "heladizas": "heladizo",
    "heladizos": "heladizo",
    "helado": "helar",
    "heladora": "helador",
    "heladoras": "heladora",
    "heladores": "helador",
    "helados": "helar",
    "helamos": "helar",
    "helando": "helar",
    "helara": "helar",
    "helarais": "helar",
    "helaran": "helar",
    "helaras": "helar",
    "helare": "helar",
    "helareis": "helar",
    "helaremos": "helar",
    "helaren": "helar",
    "helares": "helar",
    "helaron": "helar",
    "helará": "helar",
    "helarán": "helar",
    "helarás": "helar",
    "helaré": "helar",
    "helaréis": "helar",
    "helaría": "helar",
    "helaríais": "helar",
    "helaríamos": "helar",
    "helarían": "helar",
    "helarías": "helar",
    "helase": "helar",
    "helaseis": "helar",
    "helasen": "helar",
    "helases": "helar",
    "helaste": "helar",
    "helasteis": "helar",
    "helechos": "helecho",
    "helemos": "helar",
    "helena": "heleno",
    "helenas": "heleno",
    "helenice": "helenizar",
    "helenicemos": "helenizar",
    "helenicen": "helenizar",
    "helenices": "helenizar",
    "helenicé": "helenizar",
    "helenicéis": "helenizar",
    "helenios": "helenio",
    "helenismos": "helenismo",
    "helenistas": "helenista",
    "heleniza": "helenizar",
    "helenizaba": "helenizar",
    "helenizabais": "helenizar",
    "helenizaban": "helenizar",
    "helenizabas": "helenizar",
    "helenizaciones": "helenización",
    "helenizad": "helenizar",
    "helenizada": "helenizar",
    "helenizadas": "helenizar",
    "helenizado": "helenizar",
    "helenizados": "helenizar",
    "helenizamos": "helenizar",
    "helenizan": "helenizar",
    "helenizando": "helenizar",
    "helenizara": "helenizar",
    "helenizarais": "helenizar",
    "helenizaran": "helenizar",
    "helenizaras": "helenizar",
    "helenizare": "helenizar",
    "helenizareis": "helenizar",
    "helenizaremos": "helenizar",
    "helenizaren": "helenizar",
    "helenizares": "helenizar",
    "helenizaron": "helenizar",
    "helenizará": "helenizar",
    "helenizarán": "helenizar",
    "helenizarás": "helenizar",
    "helenizaré": "helenizar",
    "helenizaréis": "helenizar",
    "helenizaría": "helenizar",
    "helenizaríais": "helenizar",
    "helenizaríamos": "helenizar",
    "helenizarían": "helenizar",
    "helenizarías": "helenizar",
    "helenizas": "helenizar",
    "helenizase": "helenizar",
    "helenizaseis": "helenizar",
    "helenizasen": "helenizar",
    "helenizases": "helenizar",
    "helenizaste": "helenizar",
    "helenizasteis": "helenizar",
    "helenizo": "helenizar",
    "helenizábamos": "helenizar",
    "helenizáis": "helenizar",
    "helenizáramos": "helenizar",
    "helenizáremos": "helenizar",
    "helenizásemos": "helenizar",
    "helenizó": "helenizar",
    "helenos": "heleno",
    "helenística": "helenístico",
    "helenísticas": "helenístico",
    "helenísticos": "helenístico",
    "heleros": "helero",
    "helespóntica": "helespóntico",
    "helespónticas": "helespóntico",
    "helespónticos": "helespóntico",
    "helgada": "helgado",
    "helgadas": "helgado",
    "helgados": "helgado",
    "helgaduras": "helgadura",
    "heli-esquies": "heli-esquí",
    "helibaldes": "helibalde",
    "helicina": "helicino",
    "helicinas": "helicino",
    "helicinos": "helicino",
    "helicoidales": "helicoidal",
    "helicoidalidades": "helicoidalidad",
    "helicoides": "helicoide",
    "helicones": "helicón",
    "helicotremas": "helicotrema",
    "helicómetros": "helicómetro",
    "helicópteros": "helicóptero",
    "heliesquises": "heliesquí",
    "heliocéntrica": "heliocéntrico",
    "heliocéntricas": "heliocéntrico",
    "heliocéntricos": "heliocéntrico",
    "heliodoras": "heliodora",
    "heliodoros": "heliodoro",
    "heliofotómetros": "heliofotómetro",
    "heliofísicas": "heliofísica",
    "heliograbados": "heliograbado",
    "heliografías": "heliografía",
    "heliogramas": "heliograma",
    "heliográfica": "heliográfico",
    "heliográficas": "heliográfico",
    "heliográficos": "heliográfico",
    "heliogábalos": "heliogábalo",
    "heliometeorologías": "heliometeorología",
    "heliometrías": "heliometría",
    "heliomotores": "heliomotor",
    "heliones": "helión",
    "heliopausas": "heliopausa",
    "helioplastias": "helioplastia",
    "helios": "helio",
    "helioscopios": "helioscopio",
    "heliotecnias": "heliotecnia",
    "heliotecnologías": "heliotecnología",
    "helioterapias": "helioterapia",
    "heliotropinas": "heliotropina",
    "heliotropismos": "heliotropismo",
    "heliotropos": "heliotropo",
    "heliotrópica": "heliotrópico",
    "heliotrópicas": "heliotrópico",
    "heliotrópicos": "heliotrópico",
    "heliozoarios": "heliozoario",
    "heliozoos": "heliozoo",
    "helipuertos": "helipuerto",
    "heliófila": "heliófilo",
    "heliófilas": "heliófilo",
    "heliófilos": "heliófilo",
    "heliógrafos": "heliógrafo",
    "heliómetros": "heliómetro",
    "helióstatos": "helióstato",
    "helmintologías": "helmintología",
    "helmintológica": "helmintológico",
    "helmintológicas": "helmintológico",
    "helmintológicos": "helmintológico",
    "helmintos": "helminto",
    "helmintóloga": "helmintólogo",
    "helmintólogas": "helmintólogo",
    "helmintólogos": "helmintólogo",
    "helmíntica": "helmíntico",
    "helmínticas": "helmíntico",
    "helmínticos": "helmíntico",
    "helobiales": "helobial",
    "helvecia": "helvecio",
    "helvecias": "helvecio",
    "helvecios": "helvecio",
    "helvitas": "helvita",
    "helvética": "helvético",
    "helvéticas": "helvético",
    "helvéticos": "helvético",
    "helábamos": "helar",
    "heláis": "helar",
    "heláramos": "helar",
    "heláremos": "helar",
    "helásemos": "helar",
    "helé": "helar",
    "heléis": "helar",
    "helénica": "helénico",
    "helénicas": "helénico",
    "helénicos": "helénico",
    "helícidos": "helícido",
    "heló": "helar",
    "helófitos": "helófito",
    "hemacitos": "hemacito",
    "hemaglutinaciones": "hemaglutinación",
    "hemaglutininas": "hemaglutinina",
    "hemales": "hemal",
    "hemangiomas": "hemangioma",
    "hematerma": "hematermo",
    "hematermas": "hematermo",
    "hematermos": "hematermo",
    "hematimetrías": "hematimetría",
    "hematinas": "hematina",
    "hematitas": "hematita",
    "hematoblastos": "hematoblasto",
    "hematocitoblastos": "hematocitoblasto",
    "hematocitos": "hematocito",
    "hematocórneas": "hematocórnea",
    "hematologías": "hematología",
    "hematológica": "hematológico",
    "hematológicas": "hematológico",
    "hematológicos": "hematológico",
    "hematomas": "hematoma",
    "hematomielias": "hematomielia",
    "hematopatías": "hematopatía",
    "hematoporfirinas": "hematoporfirina",
    "hematopoyética": "hematopoyético",
    "hematopoyéticas": "hematopoyético",
    "hematopoyéticos": "hematopoyético",
    "hematotímpanos": "hematotímpano",
    "hematoxilinas": "hematoxilina",
    "hematozoarios": "hematozoario",
    "hematozoos": "hematozoo",
    "hematurias": "hematuria",
    "hematíes": "hematíe",
    "hematímetros": "hematímetro",
    "hematócritos": "hematócrito",
    "hematófaga": "hematófago",
    "hematófagas": "hematófago",
    "hematófagos": "hematófago",
    "hematógena": "hematógeno",
    "hematógenas": "hematógeno",
    "hematógenos": "hematógeno",
    "hematóloga": "hematólogo",
    "hematólogas": "hematólogo",
    "hematólogos": "hematólogo",
    "hematúrica": "hematúrico",
    "hematúricas": "hematúrico",
    "hematúricos": "hematúrico",
    "hembrajes": "hembraje",
    "hembras": "hembra",
    "hembrillas": "hembrilla",
    "hemeralopías": "hemeralopía",
    "hemeralópica": "hemeralópico",
    "hemeralópicas": "hemeralópico",
    "hemeralópicos": "hemeralópico",
    "hemerotecas": "hemeroteca",
    "hemerálopes": "hemerálope",
    "hemes": "hem",
    "hemiacetales": "hemiacetal",
    "hemianestesias": "hemianestesia",
    "hemianopsias": "hemianopsia",
    "hemianopías": "hemianopía",
    "hemicelulosas": "hemicelulosa",
    "hemiciclos": "hemiciclo",
    "hemicigótica": "hemicigótico",
    "hemicigóticas": "hemicigótico",
    "hemicigóticos": "hemicigótico",
    "hemicriptófitos": "hemicriptófito",
    "hemicromatídica": "hemicromatídico",
    "hemicromatídicas": "hemicromatídico",
    "hemicromatídicos": "hemicromatídico",
    "hemicráneas": "hemicránea",
    "hemidesmosomas": "hemidesmosoma",
    "hemiedros": "hemiedro",
    "hemiedrías": "hemiedría",
    "hemifaciales": "hemifacial",
    "hemihaploides": "hemihaploide",
    "hemilaringectomías": "hemilaringectomía",
    "hemimaxilares": "hemimaxilar",
    "hemimembranas": "hemimembrana",
    "hemimetábola": "hemimetábolo",
    "hemimetábolas": "hemimetábolo",
    "hemimetábolos": "hemimetábolo",
    "hemimorfitas": "hemimorfita",
    "heminas": "hemina",
    "hemiones": "hemión",
    "hemiparásitos": "hemiparásito",
    "hemiplejias": "hemiplejia",
    "hemiplejías": "hemiplejía",
    "hemipléjica": "hemipléjico",
    "hemipléjicas": "hemipléjico",
    "hemipléjicos": "hemipléjico",
    "hemisecciones": "hemisección",
    "hemiseptums": "hemiseptum",
    "hemisferios": "hemisferio",
    "hemisférica": "hemisférico",
    "hemisféricas": "hemisférico",
    "hemisféricos": "hemisférico",
    "hemistiquios": "hemistiquio",
    "hemitómbolos": "hemitómbolo",
    "hemiédrica": "hemiédrico",
    "hemiédricas": "hemiédrico",
    "hemiédricos": "hemiédrico",
    "hemoaglutinaciones": "hemoaglutinación",
    "hemocaterética": "hemocaterético",
    "hemocateréticas": "hemocaterético",
    "hemocateréticos": "hemocaterético",
    "hemoceles": "hemocele",
    "hemocianinas": "hemocianina",
    "hemocitoblastos": "hemocitoblasto",
    "hemocitologías": "hemocitología",
    "hemocitos": "hemocito",
    "hemocitómetros": "hemocitómetro",
    "hemoconcentraciones": "hemoconcentración",
    "hemocromos": "hemocromo",
    "hemocromógenos": "hemocromógeno",
    "hemoculturas": "hemocultura",
    "hemoderivada": "hemoderivado",
    "hemoderivadas": "hemoderivado",
    "hemoderivados": "hemoderivado",
    "hemodiluciones": "hemodilución",
    "hemodonaciones": "hemodonación",
    "hemofilias": "hemofilia",
    "hemofílica": "hemofílico",
    "hemofílicas": "hemofílico",
    "hemofílicos": "hemofílico",
    "hemoglobinas": "hemoglobina",
    "hemoglobinemias": "hemoglobinemia",
    "hemoglobinopatías": "hemoglobinopatía",
    "hemoglobinurias": "hemoglobinuria",
    "hemogramas": "hemograma",
    "hemolinfas": "hemolinfa",
    "hemolinfoides": "hemolinfoide",
    "hemolisinas": "hemolisina",
    "hemolítica": "hemolítico",
    "hemolíticas": "hemolítico",
    "hemolíticos": "hemolítico",
    "hemopatías": "hemopatía",
    "hemopoyetinas": "hemopoyetina",
    "hemopoyética": "hemopoyético",
    "hemopoyéticas": "hemopoyético",
    "hemopoyéticos": "hemopoyético",
    "hemoptísica": "hemoptísico",
    "hemoptísicas": "hemoptísico",
    "hemoptísicos": "hemoptísico",
    "hemorragias": "hemorragia",
    "hemorroidales": "hemorroidal",
    "hemorroides": "hemorroide",
    "hemorroisas": "hemorroisa",
    "hemorrágica": "hemorrágico",
    "hemorrágicas": "hemorrágico",
    "hemorrágicos": "hemorrágico",
    "hemos": "hemo",
    "hemosiderinas": "hemosiderina",
    "hemosiderinurias": "hemosiderinuria",
    "hemostasias": "hemostasia",
    "hemostática": "hemostático",
    "hemostáticas": "hemostático",
    "hemostáticos": "hemostático",
    "hemoterapias": "hemoterapia",
    "hemotímpanos": "hemotímpano",
    "hemática": "hemático",
    "hemáticas": "hemático",
    "hemáticos": "hemático",
    "hemípteros": "hemíptero",
    "henales": "henal",
    "henares": "henar",
    "henchid": "henchir",
    "henchida": "henchir",
    "henchidas": "henchir",
    "henchido": "henchir",
    "henchidos": "henchir",
    "henchimos": "henchir",
    "henchiremos": "henchir",
    "henchirá": "henchir",
    "henchirán": "henchir",
    "henchirás": "henchir",
    "henchiré": "henchir",
    "henchiréis": "henchir",
    "henchiría": "henchir",
    "henchiríais": "henchir",
    "henchiríamos": "henchir",
    "henchirían": "henchir",
    "henchirías": "henchir",
    "henchiste": "henchir",
    "henchisteis": "henchir",
    "henchí": "henchir",
    "henchía": "henchir",
    "henchíais": "henchir",
    "henchíamos": "henchir",
    "henchían": "henchir",
    "henchías": "henchir",
    "henchís": "henchir",
    "hendamos": "hendir",
    "hended": "hender",
    "hendemos": "hender",
    "henderemos": "hender",
    "henderá": "hender",
    "henderán": "hender",
    "henderás": "hender",
    "henderé": "hender",
    "henderéis": "hender",
    "hendería": "hender",
    "henderíais": "hender",
    "henderíamos": "hender",
    "henderían": "hender",
    "henderías": "hender",
    "hendibles": "hendible",
    "hendid": "hendir",
    "hendida": "hendir",
    "hendidas": "hendir",
    "hendido": "hendir",
    "hendidos": "hendir",
    "hendiduras": "hendidura",
    "hendiendo": "hendir",
    "hendiera": "hendir",
    "hendierais": "hendir",
    "hendieran": "hendir",
    "hendieras": "hendir",
    "hendiere": "hendir",
    "hendiereis": "hendir",
    "hendieren": "hendir",
    "hendieres": "hendir",
    "hendieron": "hendir",
    "hendiese": "hendir",
    "hendieseis": "hendir",
    "hendiesen": "hendir",
    "hendieses": "hendir",
    "hendimientos": "hendimiento",
    "hendimos": "hendir",
    "hendiremos": "hendir",
    "hendirá": "hendir",
    "hendirán": "hendir",
    "hendirás": "hendir",
    "hendiré": "hendir",
    "hendiréis": "hendir",
    "hendiría": "hendir",
    "hendiríais": "hendir",
    "hendiríamos": "hendir",
    "hendirían": "hendir",
    "hendirías": "hendir",
    "hendiste": "hendir",
    "hendisteis": "hendir",
    "hendiéramos": "hendir",
    "hendiéremos": "hendir",
    "hendiésemos": "hendir",
    "hendió": "hendir",
    "hendáis": "hendir",
    "hendéis": "hender",
    "hendí": "hendir",
    "hendía": "hendir",
    "hendíais": "hendir",
    "hendíamos": "hendir",
    "hendían": "hendir",
    "hendías": "hendir",
    "hendís": "hendir",
    "heneicosanoatos": "heneicosanoato",
    "henequenes": "henequén",
    "henides": "henide",
    "henifica": "henificar",
    "henificaba": "henificar",
    "henificabais": "henificar",
    "henificaban": "henificar",
    "henificabas": "henificar",
    "henificad": "henificar",
    "henificada": "henificar",
    "henificadas": "henificar",
    "henificado": "henificar",
    "henificadoras": "henificadora",
    "henificados": "henificar",
    "henificamos": "henificar",
    "henifican": "henificar",
    "henificando": "henificar",
    "henificara": "henificar",
    "henificarais": "henificar",
    "henificaran": "henificar",
    "henificaras": "henificar",
    "henificare": "henificar",
    "henificareis": "henificar",
    "henificaremos": "henificar",
    "henificaren": "henificar",
    "henificares": "henificar",
    "henificaron": "henificar",
    "henificará": "henificar",
    "henificarán": "henificar",
    "henificarás": "henificar",
    "henificaré": "henificar",
    "henificaréis": "henificar",
    "henificaría": "henificar",
    "henificaríais": "henificar",
    "henificaríamos": "henificar",
    "henificarían": "henificar",
    "henificarías": "henificar",
    "henificas": "henificar",
    "henificase": "henificar",
    "henificaseis": "henificar",
    "henificasen": "henificar",
    "henificases": "henificar",
    "henificaste": "henificar",
    "henificasteis": "henificar",
    "henifico": "henificar",
    "henificábamos": "henificar",
    "henificáis": "henificar",
    "henificáramos": "henificar",
    "henificáremos": "henificar",
    "henificásemos": "henificar",
    "henificó": "henificar",
    "henifique": "henificar",
    "henifiquemos": "henificar",
    "henifiquen": "henificar",
    "henifiques": "henificar",
    "henifiqué": "henificar",
    "henifiquéis": "henificar",
    "heniles": "henil",
    "hennas": "henna",
    "henos": "heno",
    "henries": "henry",
    "henrios": "henrio",
    "henriómetros": "henriómetro",
    "henry": "henrio",
    "henrys": "henrio",
    "heparinas": "heparina",
    "hepatizaciones": "hepatización",
    "hepatoceles": "hepatocele",
    "hepatocitos": "hepatocito",
    "hepatocólica": "hepatocólico",
    "hepatocólicas": "hepatocólico",
    "hepatocólicos": "hepatocólico",
    "hepatoduodenales": "hepatoduodenal",
    "hepatoesplenomegalias": "hepatoesplenomegalia",
    "hepatografías": "hepatografía",
    "hepatogástrica": "hepatogástrico",
    "hepatogástricas": "hepatogástrico",
    "hepatogástricos": "hepatogástrico",
    "hepatologías": "hepatología",
    "hepatomegalias": "hepatomegalia",
    "hepatopancreática": "hepatopancreático",
    "hepatopancreáticas": "hepatopancreático",
    "hepatopancreáticos": "hepatopancreático",
    "hepatopatías": "hepatopatía",
    "hepatorrenales": "hepatorrenal",
    "heptabarbitales": "heptabarbital",
    "heptacloros": "heptacloro",
    "heptacordios": "heptacordio",
    "heptacordos": "heptacordo",
    "heptadecanoatos": "heptadecanoato",
    "heptadecanoles": "heptadecanol",
    "heptadecilglioxalidinas": "heptadecilglioxalidina",
    "heptaedros": "heptaedro",
    "heptafluobutírica": "heptafluobutírico",
    "heptafluobutíricas": "heptafluobutírico",
    "heptafluobutíricos": "heptafluobutírico",
    "heptagonales": "heptagonal",
    "heptanoles": "heptanol",
    "heptanos": "heptano",
    "heptarquías": "heptarquía",
    "heptasilábica": "heptasilábico",
    "heptasilábicas": "heptasilábico",
    "heptasilábicos": "heptasilábico",
    "heptasílaba": "heptasílabo",
    "heptasílabas": "heptasílabo",
    "heptasílabos": "heptasílabo",
    "heptatletas": "heptatleta",
    "heptatlones": "heptatlón",
    "heptatlonianas": "heptatloniana",
    "heptavalentes": "heptavalente",
    "heptenos": "hepteno",
    "heptodos": "heptodo",
    "heptágona": "heptágono",
    "heptágonas": "heptágono",
    "heptágonos": "heptágono",
    "heptámetra": "heptámetro",
    "heptámetras": "heptámetro",
    "heptámetros": "heptámetro",
    "hepática": "hepático",
    "hepáticas": "hepático",
    "hepáticos": "hepático",
    "heraldistas": "heraldista",
    "heraldos": "heraldo",
    "herbaba": "herbar",
    "herbabais": "herbar",
    "herbaban": "herbar",
    "herbabas": "herbar",
    "herbad": "herbar",
    "herbada": "herbar",
    "herbadas": "herbar",
    "herbado": "herbar",
    "herbados": "herbar",
    "herbaja": "herbajar",
    "herbajaba": "herbajar",
    "herbajabais": "herbajar",
    "herbajaban": "herbajar",
    "herbajabas": "herbajar",
    "herbajad": "herbajar",
    "herbajada": "herbajar",
    "herbajadas": "herbajar",
    "herbajado": "herbajar",
    "herbajados": "herbajar",
    "herbajamos": "herbajar",
    "herbajan": "herbajar",
    "herbajando": "herbajar",
    "herbajara": "herbajar",
    "herbajarais": "herbajar",
    "herbajaran": "herbajar",
    "herbajaras": "herbajar",
    "herbajare": "herbajar",
    "herbajareis": "herbajar",
    "herbajaremos": "herbajar",
    "herbajaren": "herbajar",
    "herbajares": "herbajar",
    "herbajaron": "herbajar",
    "herbajará": "herbajar",
    "herbajarán": "herbajar",
    "herbajarás": "herbajar",
    "herbajaré": "herbajar",
    "herbajaréis": "herbajar",
    "herbajaría": "herbajar",
    "herbajaríais": "herbajar",
    "herbajaríamos": "herbajar",
    "herbajarían": "herbajar",
    "herbajarías": "herbajar",
    "herbajas": "herbajar",
    "herbajase": "herbajar",
    "herbajaseis": "herbajar",
    "herbajasen": "herbajar",
    "herbajases": "herbajar",
    "herbajaste": "herbajar",
    "herbajasteis": "herbajar",
    "herbaje": "herbajar",
    "herbajemos": "herbajar",
    "herbajen": "herbajar",
    "herbajera": "herbajero",
    "herbajeras": "herbajero",
    "herbajeros": "herbajero",
    "herbajes": "herbaje",
    "herbajo": "herbajar",
    "herbajábamos": "herbajar",
    "herbajáis": "herbajar",
    "herbajáramos": "herbajar",
    "herbajáremos": "herbajar",
    "herbajásemos": "herbajar",
    "herbajé": "herbajar",
    "herbajéis": "herbajar",
    "herbajó": "herbajar",
    "herbamos": "herbar",
    "herbando": "herbar",
    "herbara": "herbar",
    "herbarais": "herbar",
    "herbaran": "herbar",
    "herbaras": "herbar",
    "herbare": "herbar",
    "herbareis": "herbar",
    "herbaremos": "herbar",
    "herbaren": "herbar",
    "herbares": "herbar",
    "herbaria": "herbario",
    "herbarias": "herbario",
    "herbarios": "herbario",
    "herbaron": "herbar",
    "herbará": "herbar",
    "herbarán": "herbar",
    "herbarás": "herbar",
    "herbaré": "herbar",
    "herbaréis": "herbar",
    "herbaría": "herbar",
    "herbaríais": "herbar",
    "herbaríamos": "herbar",
    "herbarían": "herbar",
    "herbarías": "herbar",
    "herbase": "herbar",
    "herbaseis": "herbar",
    "herbasen": "herbar",
    "herbases": "herbar",
    "herbaste": "herbar",
    "herbasteis": "herbar",
    "herbazales": "herbazal",
    "herbece": "herbecer",
    "herbeced": "herbecer",
    "herbecemos": "herbecer",
    "herbecen": "herbecer",
    "herbeceremos": "herbecer",
    "herbecerá": "herbecer",
    "herbecerán": "herbecer",
    "herbecerás": "herbecer",
    "herbeceré": "herbecer",
    "herbeceréis": "herbecer",
    "herbecería": "herbecer",
    "herbeceríais": "herbecer",
    "herbeceríamos": "herbecer",
    "herbecerían": "herbecer",
    "herbecerías": "herbecer",
    "herbeces": "herbecer",
    "herbecida": "herbecer",
    "herbecidas": "herbecer",
    "herbecido": "herbecer",
    "herbecidos": "herbecer",
    "herbeciendo": "herbecer",
    "herbeciera": "herbecer",
    "herbecierais": "herbecer",
    "herbecieran": "herbecer",
    "herbecieras": "herbecer",
    "herbeciere": "herbecer",
    "herbeciereis": "herbecer",
    "herbecieren": "herbecer",
    "herbecieres": "herbecer",
    "herbecieron": "herbecer",
    "herbeciese": "herbecer",
    "herbecieseis": "herbecer",
    "herbeciesen": "herbecer",
    "herbecieses": "herbecer",
    "herbecimos": "herbecer",
    "herbeciste": "herbecer",
    "herbecisteis": "herbecer",
    "herbeciéramos": "herbecer",
    "herbeciéremos": "herbecer",
    "herbeciésemos": "herbecer",
    "herbeció": "herbecer",
    "herbecéis": "herbecer",
    "herbecí": "herbecer",
    "herbecía": "herbecer",
    "herbecíais": "herbecer",
    "herbecíamos": "herbecer",
    "herbecían": "herbecer",
    "herbecías": "herbecer",
    "herbemos": "herbar",
    "herberos": "herbero",
    "herbezca": "herbecer",
    "herbezcamos": "herbecer",
    "herbezcan": "herbecer",
    "herbezcas": "herbecer",
    "herbezco": "herbecer",
    "herbezcáis": "herbecer",
    "herbicidas": "herbicida",
    "herbolaria": "herbolario",
    "herbolarias": "herbolario",
    "herbolarios": "herbolario",
    "herborice": "herborizar",
    "herboricemos": "herborizar",
    "herboricen": "herborizar",
    "herborices": "herborizar",
    "herboricé": "herborizar",
    "herboricéis": "herborizar",
    "herboristerías": "herboristería",
    "herboriza": "herborizar",
    "herborizaba": "herborizar",
    "herborizabais": "herborizar",
    "herborizaban": "herborizar",
    "herborizabas": "herborizar",
    "herborizaciones": "herborización",
    "herborizad": "herborizar",
    "herborizada": "herborizar",
    "herborizadas": "herborizar",
    "herborizado": "herborizar",
    "herborizadora": "herborizador",
    "herborizadoras": "herborizador",
    "herborizadores": "herborizador",
    "herborizados": "herborizar",
    "herborizamos": "herborizar",
    "herborizan": "herborizar",
    "herborizando": "herborizar",
    "herborizara": "herborizar",
    "herborizarais": "herborizar",
    "herborizaran": "herborizar",
    "herborizaras": "herborizar",
    "herborizare": "herborizar",
    "herborizareis": "herborizar",
    "herborizaremos": "herborizar",
    "herborizaren": "herborizar",
    "herborizares": "herborizar",
    "herborizaron": "herborizar",
    "herborizará": "herborizar",
    "herborizarán": "herborizar",
    "herborizarás": "herborizar",
    "herborizaré": "herborizar",
    "herborizaréis": "herborizar",
    "herborizaría": "herborizar",
    "herborizaríais": "herborizar",
    "herborizaríamos": "herborizar",
    "herborizarían": "herborizar",
    "herborizarías": "herborizar",
    "herborizas": "herborizar",
    "herborizase": "herborizar",
    "herborizaseis": "herborizar",
    "herborizasen": "herborizar",
    "herborizases": "herborizar",
    "herborizaste": "herborizar",
    "herborizasteis": "herborizar",
    "herborizo": "herborizar",
    "herborizábamos": "herborizar",
    "herborizáis": "herborizar",
    "herborizáramos": "herborizar",
    "herborizáremos": "herborizar",
    "herborizásemos": "herborizar",
    "herborizó": "herborizar",
    "herbosa": "herboso",
    "herbosas": "herboso",
    "herbosos": "herboso",
    "herbábamos": "herbar",
    "herbácea": "herbáceo",
    "herbáceas": "herbáceo",
    "herbáceos": "herbáceo",
    "herbáis": "herbar",
    "herbáramos": "herbar",
    "herbáremos": "herbar",
    "herbásemos": "herbar",
    "herbé": "herbar",
    "herbéis": "herbar",
    "herbívora": "herbívoro",
    "herbívoras": "herbívoro",
    "herbívoros": "herbívoro",
    "herbó": "herbar",
    "herciana": "herciano",
    "hercianas": "herciano",
    "hercianos": "herciano",
    "hercinianos": "herciniano",
    "hercinitas": "hercinita",
    "hercios": "hercio",
    "hercúlea": "hercúleo",
    "hercúleas": "hercúleo",
    "hercúleos": "hercúleo",
    "hereda": "heredar",
    "heredaba": "heredar",
    "heredabais": "heredar",
    "heredaban": "heredar",
    "heredabas": "heredar",
    "heredables": "heredable",
    "heredad": "heredar",
    "heredada": "heredar",
    "heredadas": "heredar",
    "heredades": "heredad",
    "heredado": "heredar",
    "heredados": "heredar",
    "heredamientos": "heredamiento",
    "heredamos": "heredar",
    "heredan": "heredar",
    "heredando": "heredar",
    "heredara": "heredar",
    "heredarais": "heredar",
    "heredaran": "heredar",
    "heredaras": "heredar",
    "heredare": "heredar",
    "heredareis": "heredar",
    "heredaremos": "heredar",
    "heredaren": "heredar",
    "heredares": "heredar",
    "heredaron": "heredar",
    "heredará": "heredar",
    "heredarán": "heredar",
    "heredarás": "heredar",
    "heredaré": "heredar",
    "heredaréis": "heredar",
    "heredaría": "heredar",
    "heredaríais": "heredar",
    "heredaríamos": "heredar",
    "heredarían": "heredar",
    "heredarías": "heredar",
    "heredas": "heredar",
    "heredase": "heredar",
    "heredaseis": "heredar",
    "heredasen": "heredar",
    "heredases": "heredar",
    "heredaste": "heredar",
    "heredasteis": "heredar",
    "herede": "heredar",
    "heredemos": "heredar",
    "hereden": "heredar",
    "heredera": "heredero",
    "herederas": "heredero",
    "herederos": "heredero",
    "heredes": "heredar",
    "hereditaria": "hereditario",
    "hereditarias": "hereditario",
    "hereditarios": "hereditario",
    "heredo": "heredar",
    "heredábamos": "heredar",
    "heredáis": "heredar",
    "heredáramos": "heredar",
    "heredáremos": "heredar",
    "heredásemos": "heredar",
    "heredé": "heredar",
    "heredéis": "heredar",
    "heredípetas": "heredípeta",
    "heredó": "heredar",
    "herejes": "hereje",
    "herejías": "herejía",
    "herencias": "herencia",
    "heresiarcas": "heresiarca",
    "herid": "herir",
    "herida": "herir",
    "heridas": "herir",
    "herido": "herir",
    "heridos": "herir",
    "heriles": "heril",
    "herimos": "herir",
    "heriremos": "herir",
    "herirá": "herir",
    "herirán": "herir",
    "herirás": "herir",
    "heriré": "herir",
    "heriréis": "herir",
    "heriría": "herir",
    "heriríais": "herir",
    "heriríamos": "herir",
    "herirían": "herir",
    "herirías": "herir",
    "heriste": "herir",
    "heristeis": "herir",
    "hermafroditas": "hermafrodita",
    "hermafroditismos": "hermafroditismo",
    "hermana": "hermano",
    "hermanaba": "hermanar",
    "hermanabais": "hermanar",
    "hermanaban": "hermanar",
    "hermanabas": "hermanar",
    "hermanables": "hermanable",
    "hermanad": "hermanar",
    "hermanada": "hermanar",
    "hermanadas": "hermanar",
    "hermanado": "hermanar",
    "hermanados": "hermanar",
    "hermanamientos": "hermanamiento",
    "hermanamos": "hermanar",
    "hermanan": "hermanar",
    "hermanando": "hermanar",
    "hermanara": "hermanar",
    "hermanarais": "hermanar",
    "hermanaran": "hermanar",
    "hermanaras": "hermanar",
    "hermanare": "hermanar",
    "hermanareis": "hermanar",
    "hermanaremos": "hermanar",
    "hermanaren": "hermanar",
    "hermanares": "hermanar",
    "hermanaron": "hermanar",
    "hermanará": "hermanar",
    "hermanarán": "hermanar",
    "hermanarás": "hermanar",
    "hermanaré": "hermanar",
    "hermanaréis": "hermanar",
    "hermanaría": "hermanar",
    "hermanaríais": "hermanar",
    "hermanaríamos": "hermanar",
    "hermanarían": "hermanar",
    "hermanarías": "hermanar",
    "hermanas": "hermano",
    "hermanase": "hermanar",
    "hermanaseis": "hermanar",
    "hermanasen": "hermanar",
    "hermanases": "hermanar",
    "hermanaste": "hermanar",
    "hermanasteis": "hermanar",
    "hermanastra": "hermanastro",
    "hermanastras": "hermanastro",
    "hermanastros": "hermanastro",
    "hermandades": "hermandad",
    "hermane": "hermanar",
    "hermanemos": "hermanar",
    "hermanen": "hermanar",
    "hermanes": "hermanar",
    "hermano": "hermanar",
    "hermanos": "hermano",
    "hermanábamos": "hermanar",
    "hermanáis": "hermanar",
    "hermanáramos": "hermanar",
    "hermanáremos": "hermanar",
    "hermanásemos": "hermanar",
    "hermané": "hermanar",
    "hermanéis": "hermanar",
    "hermanó": "hermanar",
    "hermas": "herma",
    "hermeneutas": "hermeneuta",
    "hermenéutica": "hermenéutico",
    "hermenéuticas": "hermenéutico",
    "hermenéuticos": "hermenéutico",
    "hermetice": "hermetizar",
    "hermeticemos": "hermetizar",
    "hermeticen": "hermetizar",
    "hermetices": "hermetizar",
    "hermeticidades": "hermeticidad",
    "hermeticé": "hermetizar",
    "hermeticéis": "hermetizar",
    "hermetismos": "hermetismo",
    "hermetiza": "hermetizar",
    "hermetizaba": "hermetizar",
    "hermetizabais": "hermetizar",
    "hermetizaban": "hermetizar",
    "hermetizabas": "hermetizar",
    "hermetizad": "hermetizar",
    "hermetizada": "hermetizar",
    "hermetizadas": "hermetizar",
    "hermetizado": "hermetizar",
    "hermetizados": "hermetizar",
    "hermetizamos": "hermetizar",
    "hermetizan": "hermetizar",
    "hermetizando": "hermetizar",
    "hermetizara": "hermetizar",
    "hermetizarais": "hermetizar",
    "hermetizaran": "hermetizar",
    "hermetizaras": "hermetizar",
    "hermetizare": "hermetizar",
    "hermetizareis": "hermetizar",
    "hermetizaremos": "hermetizar",
    "hermetizaren": "hermetizar",
    "hermetizares": "hermetizar",
    "hermetizaron": "hermetizar",
    "hermetizará": "hermetizar",
    "hermetizarán": "hermetizar",
    "hermetizarás": "hermetizar",
    "hermetizaré": "hermetizar",
    "hermetizaréis": "hermetizar",
    "hermetizaría": "hermetizar",
    "hermetizaríais": "hermetizar",
    "hermetizaríamos": "hermetizar",
    "hermetizarían": "hermetizar",
    "hermetizarías": "hermetizar",
    "hermetizas": "hermetizar",
    "hermetizase": "hermetizar",
    "hermetizaseis": "hermetizar",
    "hermetizasen": "hermetizar",
    "hermetizases": "hermetizar",
    "hermetizaste": "hermetizar",
    "hermetizasteis": "hermetizar",
    "hermetizo": "hermetizar",
    "hermetizábamos": "hermetizar",
    "hermetizáis": "hermetizar",
    "hermetizáramos": "hermetizar",
    "hermetizáremos": "hermetizar",
    "hermetizásemos": "hermetizar",
    "hermetizó": "hermetizar",
    "hermosa": "hermoso",
    "hermosas": "hermoso",
    "hermosea": "hermosear",
    "hermoseaba": "hermosear",
    "hermoseabais": "hermosear",
    "hermoseaban": "hermosear",
    "hermoseabas": "hermosear",
    "hermosead": "hermosear",
    "hermoseada": "hermosear",
    "hermoseadas": "hermosear",
    "hermoseado": "hermosear",
    "hermoseadora": "hermoseador",
    "hermoseadoras": "hermoseador",
    "hermoseadores": "hermoseador",
    "hermoseados": "hermosear",
    "hermoseamientos": "hermoseamiento",
    "hermoseamos": "hermosear",
    "hermosean": "hermosear",
    "hermoseando": "hermosear",
    "hermoseara": "hermosear",
    "hermosearais": "hermosear",
    "hermosearan": "hermosear",
    "hermosearas": "hermosear",
    "hermoseare": "hermosear",
    "hermoseareis": "hermosear",
    "hermosearemos": "hermosear",
    "hermosearen": "hermosear",
    "hermoseares": "hermosear",
    "hermosearon": "hermosear",
    "hermoseará": "hermosear",
    "hermosearán": "hermosear",
    "hermosearás": "hermosear",
    "hermosearé": "hermosear",
    "hermosearéis": "hermosear",
    "hermosearía": "hermosear",
    "hermosearíais": "hermosear",
    "hermosearíamos": "hermosear",
    "hermosearían": "hermosear",
    "hermosearías": "hermosear",
    "hermoseas": "hermosear",
    "hermosease": "hermosear",
    "hermoseaseis": "hermosear",
    "hermoseasen": "hermosear",
    "hermoseases": "hermosear",
    "hermoseaste": "hermosear",
    "hermoseasteis": "hermosear",
    "hermosee": "hermosear",
    "hermoseemos": "hermosear",
    "hermoseen": "hermosear",
    "hermosees": "hermosear",
    "hermoseo": "hermosear",
    "hermoseábamos": "hermosear",
    "hermoseáis": "hermosear",
    "hermoseáramos": "hermosear",
    "hermoseáremos": "hermosear",
    "hermoseásemos": "hermosear",
    "hermoseé": "hermosear",
    "hermoseéis": "hermosear",
    "hermoseó": "hermosear",
    "hermosos": "hermoso",
    "hermosuras": "hermosura",
    "hermética": "hermético",
    "herméticas": "hermético",
    "herméticos": "hermético",
    "hermítica": "hermítico",
    "hermíticas": "hermítico",
    "hermíticos": "hermítico",
    "hernia": "herniar",
    "herniaba": "herniar",
    "herniabais": "herniar",
    "herniaban": "herniar",
    "herniabas": "herniar",
    "herniad": "herniar",
    "herniada": "herniar",
    "herniadas": "herniar",
    "herniado": "herniar",
    "herniados": "herniar",
    "herniamos": "herniar",
    "hernian": "herniar",
    "herniando": "herniar",
    "herniara": "herniar",
    "herniarais": "herniar",
    "herniaran": "herniar",
    "herniaras": "herniar",
    "herniare": "herniar",
    "herniareis": "herniar",
    "herniaremos": "herniar",
    "herniaren": "herniar",
    "herniares": "herniar",
    "herniaron": "herniar",
    "herniará": "herniar",
    "herniarán": "herniar",
    "herniarás": "herniar",
    "herniaré": "herniar",
    "herniaréis": "herniar",
    "herniaría": "herniar",
    "herniaríais": "herniar",
    "herniaríamos": "herniar",
    "herniarían": "herniar",
    "herniarías": "herniar",
    "hernias": "herniar",
    "herniase": "herniar",
    "herniaseis": "herniar",
    "herniasen": "herniar",
    "herniases": "herniar",
    "herniaste": "herniar",
    "herniasteis": "herniar",
    "hernie": "herniar",
    "herniemos": "herniar",
    "hernien": "herniar",
    "hernies": "herniar",
    "hernio": "herniar",
    "herniosa": "hernioso",
    "herniosas": "hernioso",
    "herniosos": "hernioso",
    "hernistas": "hernista",
    "herniábamos": "herniar",
    "herniáis": "herniar",
    "herniáramos": "herniar",
    "herniáremos": "herniar",
    "herniásemos": "herniar",
    "hernié": "herniar",
    "herniéis": "herniar",
    "hernió": "herniar",
    "herodiana": "herodiano",
    "herodianas": "herodiano",
    "herodianos": "herodiano",
    "heroica": "heroico",
    "heroicas": "heroico",
    "heroicidades": "heroicidad",
    "heroicos": "heroico",
    "heroinómana": "heroinómano",
    "heroinómanas": "heroinómano",
    "heroinómanos": "heroinómano",
    "heroínas": "heroína",
    "heroísmos": "heroísmo",
    "heroïnómana": "heroïnómano",
    "heroïnómanas": "heroïnómano",
    "heroïnómanos": "heroïnómano",
    "herpanginas": "herpangina",
    "herpetiformes": "herpetiforme",
    "herpetismos": "herpetismo",
    "herpetologías": "herpetología",
    "herpetológica": "herpetológico",
    "herpetológicas": "herpetológico",
    "herpetológicos": "herpetológico",
    "herpetóloga": "herpetólogo",
    "herpetólogas": "herpetólogo",
    "herpetólogos": "herpetólogo",
    "herpiles": "herpil",
    "herpética": "herpético",
    "herpéticas": "herpético",
    "herpéticos": "herpético",
    "herraba": "herrar",
    "herrabais": "herrar",
    "herraban": "herrar",
    "herrabas": "herrar",
    "herrad": "herrar",
    "herrada": "herrar",
    "herradas": "herrar",
    "herraderos": "herradero",
    "herrado": "herrar",
    "herradora": "herrador",
    "herradoras": "herrador",
    "herradores": "herrador",
    "herrados": "herrar",
    "herraduras": "herradura",
    "herrajes": "herraje",
    "herramentales": "herramental",
    "herramientas": "herramienta",
    "herramos": "herrar",
    "herrando": "herrar",
    "herrara": "herrar",
    "herrarais": "herrar",
    "herraran": "herrar",
    "herraras": "herrar",
    "herrare": "herrar",
    "herrareis": "herrar",
    "herraremos": "herrar",
    "herraren": "herrar",
    "herrares": "herrar",
    "herraron": "herrar",
    "herrará": "herrar",
    "herrarán": "herrar",
    "herrarás": "herrar",
    "herraré": "herrar",
    "herraréis": "herrar",
    "herraría": "herrar",
    "herraríais": "herrar",
    "herraríamos": "herrar",
    "herrarían": "herrar",
    "herrarías": "herrar",
    "herrase": "herrar",
    "herraseis": "herrar",
    "herrasen": "herrar",
    "herrases": "herrar",
    "herraste": "herrar",
    "herrasteis": "herrar",
    "herremos": "herrar",
    "herrenales": "herrenal",
    "herrenes": "herrén",
    "herrera": "herrero",
    "herreras": "herrero",
    "herreriana": "herreriano",
    "herrerianas": "herreriano",
    "herrerianos": "herreriano",
    "herrerillos": "herrerillo",
    "herreros": "herrero",
    "herreruelos": "herreruelo",
    "herrerías": "herrería",
    "herretea": "herretear",
    "herreteaba": "herretear",
    "herreteabais": "herretear",
    "herreteaban": "herretear",
    "herreteabas": "herretear",
    "herretead": "herretear",
    "herreteada": "herretear",
    "herreteadas": "herretear",
    "herreteado": "herretear",
    "herreteados": "herretear",
    "herreteamos": "herretear",
    "herretean": "herretear",
    "herreteando": "herretear",
    "herreteara": "herretear",
    "herretearais": "herretear",
    "herretearan": "herretear",
    "herretearas": "herretear",
    "herreteare": "herretear",
    "herreteareis": "herretear",
    "herretearemos": "herretear",
    "herretearen": "herretear",
    "herreteares": "herretear",
    "herretearon": "herretear",
    "herreteará": "herretear",
    "herretearán": "herretear",
    "herretearás": "herretear",
    "herretearé": "herretear",
    "herretearéis": "herretear",
    "herretearía": "herretear",
    "herretearíais": "herretear",
    "herretearíamos": "herretear",
    "herretearían": "herretear",
    "herretearías": "herretear",
    "herreteas": "herretear",
    "herretease": "herretear",
    "herreteaseis": "herretear",
    "herreteasen": "herretear",
    "herreteases": "herretear",
    "herreteaste": "herretear",
    "herreteasteis": "herretear",
    "herretee": "herretear",
    "herreteemos": "herretear",
    "herreteen": "herretear",
    "herretees": "herretear",
    "herreteo": "herretear",
    "herretes": "herrete",
    "herreteábamos": "herretear",
    "herreteáis": "herretear",
    "herreteáramos": "herretear",
    "herreteáremos": "herretear",
    "herreteásemos": "herretear",
    "herreteé": "herretear",
    "herreteéis": "herretear",
    "herreteó": "herretear",
    "herrezuelos": "herrezuelo",
    "herreñales": "herreñal",
    "herriales": "herrial",
    "herrines": "herrín",
    "herronadas": "herronada",
    "herrones": "herrón",
    "herrumbra": "herrumbrar",
    "herrumbraba": "herrumbrar",
    "herrumbrabais": "herrumbrar",
    "herrumbraban": "herrumbrar",
    "herrumbrabas": "herrumbrar",
    "herrumbrad": "herrumbrar",
    "herrumbrada": "herrumbrar",
    "herrumbradas": "herrumbrar",
    "herrumbrado": "herrumbrar",
    "herrumbrados": "herrumbrar",
    "herrumbramos": "herrumbrar",
    "herrumbran": "herrumbrar",
    "herrumbrando": "herrumbrar",
    "herrumbrara": "herrumbrar",
    "herrumbrarais": "herrumbrar",
    "herrumbraran": "herrumbrar",
    "herrumbraras": "herrumbrar",
    "herrumbrare": "herrumbrar",
    "herrumbrareis": "herrumbrar",
    "herrumbraremos": "herrumbrar",
    "herrumbraren": "herrumbrar",
    "herrumbrares": "herrumbrar",
    "herrumbraron": "herrumbrar",
    "herrumbrará": "herrumbrar",
    "herrumbrarán": "herrumbrar",
    "herrumbrarás": "herrumbrar",
    "herrumbraré": "herrumbrar",
    "herrumbraréis": "herrumbrar",
    "herrumbraría": "herrumbrar",
    "herrumbraríais": "herrumbrar",
    "herrumbraríamos": "herrumbrar",
    "herrumbrarían": "herrumbrar",
    "herrumbrarías": "herrumbrar",
    "herrumbras": "herrumbrar",
    "herrumbrase": "herrumbrar",
    "herrumbraseis": "herrumbrar",
    "herrumbrasen": "herrumbrar",
    "herrumbrases": "herrumbrar",
    "herrumbraste": "herrumbrar",
    "herrumbrasteis": "herrumbrar",
    "herrumbre": "herrumbrar",
    "herrumbremos": "herrumbrar",
    "herrumbren": "herrumbrar",
    "herrumbres": "herrumbre",
    "herrumbro": "herrumbrar",
    "herrumbrosa": "herrumbroso",
    "herrumbrosas": "herrumbroso",
    "herrumbrosos": "herrumbroso",
    "herrumbrábamos": "herrumbrar",
    "herrumbráis": "herrumbrar",
    "herrumbráramos": "herrumbrar",
    "herrumbráremos": "herrumbrar",
    "herrumbrásemos": "herrumbrar",
    "herrumbré": "herrumbrar",
    "herrumbréis": "herrumbrar",
    "herrumbró": "herrumbrar",
    "herrábamos": "herrar",
    "herráis": "herrar",
    "herráramos": "herrar",
    "herráremos": "herrar",
    "herrásemos": "herrar",
    "herré": "herrar",
    "herréis": "herrar",
    "herró": "herrar",
    "hertces": "hercio",
    "hertz": "hercio",
    "hertziana": "hertziano",
    "hertzianas": "hertziano",
    "hertzianos": "hertziano",
    "hertzio": "hercio",
    "hertzios": "hercio",
    "hertzs": "hercio",
    "hervaduras": "hervadura",
    "hervencias": "hervencia",
    "hervid": "hervir",
    "hervida": "hervir",
    "hervidas": "hervir",
    "hervideros": "hervidero",
    "hervido": "hervir",
    "hervidora": "hervidor",
    "hervidoras": "hervidor",
    "hervidores": "hervidor",
    "hervidos": "hervir",
    "hervimos": "hervir",
    "herviremos": "hervir",
    "hervirá": "hervir",
    "hervirán": "hervir",
    "hervirás": "hervir",
    "herviré": "hervir",
    "herviréis": "hervir",
    "herviría": "hervir",
    "herviríais": "hervir",
    "herviríamos": "hervir",
    "hervirían": "hervir",
    "hervirías": "hervir",
    "herviste": "hervir",
    "hervisteis": "hervir",
    "hervores": "hervor",
    "hervorosa": "hervoroso",
    "hervorosas": "hervoroso",
    "hervorosos": "hervoroso",
    "herví": "hervir",
    "hervía": "hervir",
    "hervíais": "hervir",
    "hervíamos": "hervir",
    "hervían": "hervir",
    "hervías": "hervir",
    "hervís": "hervir",
    "herzios": "herzio",
    "heráclidas": "heráclida",
    "heráldica": "heráldico",
    "heráldicas": "heráldico",
    "heráldicos": "heráldico",
    "herética": "herético",
    "heréticas": "herético",
    "heréticos": "herético",
    "herí": "herir",
    "hería": "herir",
    "heríais": "herir",
    "heríamos": "herir",
    "herían": "herir",
    "herías": "herir",
    "herís": "herir",
    "hesonitas": "hesonita",
    "hesperidinas": "hesperidina",
    "hesperidios": "hesperidio",
    "hessiana": "hessiano",
    "hessianas": "hessiano",
    "hessianos": "hessiano",
    "hetairas": "hetaira",
    "heteras": "hetera",
    "heteroaglutinaciones": "heteroaglutinación",
    "heteroanticuerpos": "heteroanticuerpo",
    "heterocerca": "heterocerco",
    "heterocercas": "heterocerco",
    "heterocercos": "heterocerco",
    "heterociclos": "heterociclo",
    "heterocigotos": "heterocigoto",
    "heterocitotrópica": "heterocitotrópico",
    "heterocitotrópicas": "heterocitotrópico",
    "heterocitotrópicos": "heterocitotrópico",
    "heterocliticidades": "heterocliticidad",
    "heteroclítica": "heteroclítico",
    "heteroclíticas": "heteroclítico",
    "heteroclíticos": "heteroclítico",
    "heterocromatinas": "heterocromatina",
    "heterocromatinosomas": "heterocromatinosoma",
    "heterocromatismos": "heterocromatismo",
    "heterocromosomas": "heterocromosoma",
    "heterocromática": "heterocromático",
    "heterocromáticas": "heterocromático",
    "heterocromáticos": "heterocromático",
    "heterocáriones": "heterocarion",
    "heterocíclica": "heterocíclico",
    "heterocíclicas": "heterocíclico",
    "heterocíclicos": "heterocíclico",
    "heterodifusiones": "heterodifusión",
    "heterodina": "heterodino",
    "heterodinas": "heterodino",
    "heterodinos": "heterodino",
    "heterodinámica": "heterodinámico",
    "heterodinámicas": "heterodinámico",
    "heterodinámicos": "heterodinámico",
    "heterodonta": "heterodonto",
    "heterodontas": "heterodonto",
    "heterodontos": "heterodonto",
    "heterodoxa": "heterodoxo",
    "heterodoxas": "heterodoxo",
    "heterodoxias": "heterodoxia",
    "heterodoxos": "heterodoxo",
    "heterodúplexes": "heterodúplex",
    "heteroepitaxias": "heteroepitaxia",
    "heteroestructuras": "heteroestructura",
    "heterofagosomas": "heterofagosoma",
    "heterofibras": "heterofibra",
    "heterofila": "heterofilo",
    "heterofilas": "heterofilo",
    "heterofilias": "heterofilia",
    "heterofilos": "heterofilo",
    "heterogametos": "heterogameto",
    "heterogamias": "heterogamia",
    "heterogeneidades": "heterogeneidad",
    "heterogenética": "heterogenético",
    "heterogenéticas": "heterogenético",
    "heterogenéticos": "heterogenético",
    "heterogonías": "heterogonía",
    "heterogénea": "heterogéneo",
    "heterogéneas": "heterogéneo",
    "heterogéneos": "heterogéneo",
    "heterogónica": "heterogónico",
    "heterogónicas": "heterogónico",
    "heterogónicos": "heterogónico",
    "heteroinmunes": "heteroinmune",
    "heterolecita": "heterolecito",
    "heterolecitas": "heterolecito",
    "heterolecitos": "heterolecito",
    "heterolisosomas": "heterolisosoma",
    "heteromancias": "heteromancia",
    "heteromancías": "heteromancía",
    "heterometábola": "heterometábolo",
    "heterometábolas": "heterometábolo",
    "heterometábolos": "heterometábolo",
    "heteromolibdatos": "heteromolibdato",
    "heterométrica": "heterométrico",
    "heterométricas": "heterométrico",
    "heterométricos": "heterométrico",
    "heteromórfica": "heteromórfico",
    "heteromórficas": "heteromórfico",
    "heteromórficos": "heteromórfico",
    "heteronimias": "heteronimia",
    "heteronomías": "heteronomía",
    "heteroplastias": "heteroplastia",
    "heteroplástica": "heteroplástico",
    "heteroplásticas": "heteroplástico",
    "heteroplásticos": "heteroplástico",
    "heteropolares": "heteropolar",
    "heteropolímeros": "heteropolímero",
    "heteroscedasticidades": "heteroscedasticidad",
    "heteroscia": "heteroscio",
    "heteroscias": "heteroscio",
    "heteroscios": "heteroscio",
    "heterosexuales": "heterosexual",
    "heterosexualidades": "heterosexualidad",
    "heterosferas": "heterosfera",
    "heterosilábica": "heterosilábico",
    "heterosilábicas": "heterosilábico",
    "heterosilábicos": "heterosilábico",
    "heterospórica": "heterospórico",
    "heterospóricas": "heterospórico",
    "heterospóricos": "heterospórico",
    "heterostática": "heterostático",
    "heterostáticas": "heterostático",
    "heterostáticos": "heterostático",
    "heterotalias": "heterotalia",
    "heterotalismos": "heterotalismo",
    "heterotropías": "heterotropía",
    "heterotrópica": "heterotrópico",
    "heterotrópicas": "heterotrópico",
    "heterotrópicos": "heterotrópico",
    "heterotópica": "heterotópico",
    "heterotópicas": "heterotópico",
    "heterotópicos": "heterotópico",
    "heterouniones": "heterounión",
    "heterozigota": "heterozigoto",
    "heterozigotas": "heterozigoto",
    "heterozigotos": "heterozigoto",
    "heteróclita": "heteróclito",
    "heteróclitas": "heteróclito",
    "heteróclitos": "heteróclito",
    "heterófila": "heterófilo",
    "heterófilas": "heterófilo",
    "heterófilos": "heterófilo",
    "heterógama": "heterógamo",
    "heterógamas": "heterógamo",
    "heterógamos": "heterógamo",
    "heterógrada": "heterógrado",
    "heterógradas": "heterógrado",
    "heterógrados": "heterógrado",
    "heteróloga": "heterólogo",
    "heterólogas": "heterólogo",
    "heterólogos": "heterólogo",
    "heterónima": "heterónimo",
    "heterónimas": "heterónimo",
    "heterónimos": "heterónimo",
    "heterónoma": "heterónomo",
    "heterónomas": "heterónomo",
    "heterónomos": "heterónomo",
    "heterópica": "heterópico",
    "heterópicas": "heterópico",
    "heterópicos": "heterópico",
    "heterópsida": "heterópsido",
    "heterópsidas": "heterópsido",
    "heterópsidos": "heterópsido",
    "heterópteros": "heteróptero",
    "heterótrofa": "heterótrofo",
    "heterótrofas": "heterótrofo",
    "heterótrofos": "heterótrofo",
    "hetiqueces": "hetiquez",
    "heulanditas": "heulandita",
    "heurística": "heurístico",
    "heurísticas": "heurístico",
    "heurísticos": "heurístico",
    "hevlética": "hevlético",
    "hevléticas": "hevlético",
    "hevléticos": "hevlético",
    "hexabromoetanos": "hexabromoetano",
    "hexaclorobencenos": "hexaclorobenceno",
    "hexaclorobutadienos": "hexaclorobutadieno",
    "hexaclorociclohexanos": "hexaclorociclohexano",
    "hexaclorociclopentadienos": "hexaclorociclopentadieno",
    "hexaclorofenos": "hexaclorofeno",
    "hexacloropropilenos": "hexacloropropileno",
    "hexacloruros": "hexacloruro",
    "hexacordos": "hexacordo",
    "hexadecanos": "hexadecano",
    "hexadecenos": "hexadeceno",
    "hexadeciltriclorosilanos": "hexadeciltriclorosilano",
    "hexadecimales": "hexadecimal",
    "hexaedritos": "hexaedrito",
    "hexaedros": "hexaedro",
    "hexafluoridos": "hexafluorido",
    "hexagonales": "hexagonal",
    "hexagonitas": "hexagonita",
    "hexagramas": "hexagrama",
    "hexahidrobenzoica": "hexahidrobenzoico",
    "hexahidrobenzoicas": "hexahidrobenzoico",
    "hexahidrobenzoicos": "hexahidrobenzoico",
    "hexahidroftálica": "hexahidroftálico",
    "hexahidroftálicas": "hexahidroftálico",
    "hexahidroftálicos": "hexahidroftálico",
    "hexaldehídos": "hexaldehído",
    "hexametilendiaminas": "hexametilendiamina",
    "hexametilenotetraminas": "hexametilenotetramina",
    "hexametilentetraminas": "hexametilentetramina",
    "hexametilfosfórica": "hexametilfosfórico",
    "hexametilfosfóricas": "hexametilfosfórico",
    "hexametilfosfóricos": "hexametilfosfórico",
    "hexanitratos": "hexanitrato",
    "hexanoles": "hexanol",
    "hexanos": "hexano",
    "hexanotrioles": "hexanotriol",
    "hexaoctaedros": "hexaoctaedro",
    "hexaquisoctaedros": "hexaquisoctaedro",
    "hexaquistetraedros": "hexaquistetraedro",
    "hexasílaba": "hexasílabo",
    "hexasílabas": "hexasílabo",
    "hexasílabos": "hexasílabo",
    "hexatetraedros": "hexatetraedro",
    "hexavalentes": "hexavalente",
    "hexaédrica": "hexaédrico",
    "hexaédricas": "hexaédrico",
    "hexaédricos": "hexaédrico",
    "hexenoles": "hexenol",
    "hexenos": "hexeno",
    "hexilfenoles": "hexilfenol",
    "hexilresorcinas": "hexilresorcina",
    "hexobarbitales": "hexobarbital",
    "hexodos": "hexodo",
    "hexoquinasas": "hexoquinasa",
    "hexosas": "hexosa",
    "hexágonos": "hexágono",
    "hexámetra": "hexámetro",
    "hexámetras": "hexámetro",
    "hexámetros": "hexámetro",
    "hexángula": "hexángulo",
    "hexángulas": "hexángulo",
    "hexángulos": "hexángulo",
    "hexápodos": "hexápodo",
    "hexástila": "hexástilo",
    "hexástilas": "hexástilo",
    "hexástilos": "hexástilo",
    "heñid": "heñir",
    "heñida": "heñir",
    "heñidas": "heñir",
    "heñido": "heñir",
    "heñidos": "heñir",
    "heñimos": "heñir",
    "heñiremos": "heñir",
    "heñirá": "heñir",
    "heñirán": "heñir",
    "heñirás": "heñir",
    "heñiré": "heñir",
    "heñiréis": "heñir",
    "heñiría": "heñir",
    "heñiríais": "heñir",
    "heñiríamos": "heñir",
    "heñirían": "heñir",
    "heñirías": "heñir",
    "heñiste": "heñir",
    "heñisteis": "heñir",
    "heñí": "heñir",
    "heñía": "heñir",
    "heñíais": "heñir",
    "heñíamos": "heñir",
    "heñían": "heñir",
    "heñías": "heñir",
    "heñís": "heñir",
    "hg": "hectogramo",
    "hialina": "hialino",
    "hialinas": "hialino",
    "hialinizaciones": "hialinización",
    "hialinos": "hialino",
    "hialitas": "hialita",
    "hialografías": "hialografía",
    "hialográfica": "hialográfico",
    "hialográficas": "hialográfico",
    "hialográficos": "hialográfico",
    "hialoidea": "hialoideo",
    "hialoideas": "hialoideo",
    "hialoideos": "hialoideo",
    "hialoplasmas": "hialoplasma",
    "hialuronidasas": "hialuronidasa",
    "hialurónica": "hialurónico",
    "hialurónicas": "hialurónico",
    "hialurónicos": "hialurónico",
    "hialógrafos": "hialógrafo",
    "hiatos": "hiato",
    "hiberna": "hibernar",
    "hibernaba": "hibernar",
    "hibernabais": "hibernar",
    "hibernaban": "hibernar",
    "hibernabas": "hibernar",
    "hibernaciones": "hibernación",
    "hibernad": "hibernar",
    "hibernada": "hibernar",
    "hibernadas": "hibernar",
    "hibernado": "hibernar",
    "hibernados": "hibernar",
    "hibernales": "hibernal",
    "hibernamos": "hibernar",
    "hibernan": "hibernar",
    "hibernando": "hibernar",
    "hibernara": "hibernar",
    "hibernarais": "hibernar",
    "hibernaran": "hibernar",
    "hibernaras": "hibernar",
    "hibernare": "hibernar",
    "hibernareis": "hibernar",
    "hibernaremos": "hibernar",
    "hibernaren": "hibernar",
    "hibernares": "hibernar",
    "hibernaron": "hibernar",
    "hibernará": "hibernar",
    "hibernarán": "hibernar",
    "hibernarás": "hibernar",
    "hibernaré": "hibernar",
    "hibernaréis": "hibernar",
    "hibernaría": "hibernar",
    "hibernaríais": "hibernar",
    "hibernaríamos": "hibernar",
    "hibernarían": "hibernar",
    "hibernarías": "hibernar",
    "hibernas": "hibernar",
    "hibernase": "hibernar",
    "hibernaseis": "hibernar",
    "hibernasen": "hibernar",
    "hibernases": "hibernar",
    "hibernaste": "hibernar",
    "hibernasteis": "hibernar",
    "hiberne": "hibernar",
    "hibernemos": "hibernar",
    "hibernen": "hibernar",
    "hibernes": "hibernar",
    "hiberniza": "hibernizo",
    "hibernizas": "hibernizo",
    "hibernizos": "hibernizo",
    "hiberno": "hibernar",
    "hibernábamos": "hibernar",
    "hibernáculos": "hibernáculo",
    "hibernáis": "hibernar",
    "hibernáramos": "hibernar",
    "hibernáremos": "hibernar",
    "hibernásemos": "hibernar",
    "hiberné": "hibernar",
    "hibernéis": "hibernar",
    "hibernó": "hibernar",
    "hibiscos": "hibisco",
    "hibrida": "hibridar",
    "hibridaba": "hibridar",
    "hibridabais": "hibridar",
    "hibridaban": "hibridar",
    "hibridabas": "hibridar",
    "hibridaciones": "hibridación",
    "hibridad": "hibridar",
    "hibridada": "hibridar",
    "hibridadas": "hibridar",
    "hibridado": "hibridar",
    "hibridados": "hibridar",
    "hibridamos": "hibridar",
    "hibridan": "hibridar",
    "hibridando": "hibridar",
    "hibridara": "hibridar",
    "hibridarais": "hibridar",
    "hibridaran": "hibridar",
    "hibridaras": "hibridar",
    "hibridare": "hibridar",
    "hibridareis": "hibridar",
    "hibridaremos": "hibridar",
    "hibridaren": "hibridar",
    "hibridares": "hibridar",
    "hibridaron": "hibridar",
    "hibridará": "hibridar",
    "hibridarán": "hibridar",
    "hibridarás": "hibridar",
    "hibridaré": "hibridar",
    "hibridaréis": "hibridar",
    "hibridaría": "hibridar",
    "hibridaríais": "hibridar",
    "hibridaríamos": "hibridar",
    "hibridarían": "hibridar",
    "hibridarías": "hibridar",
    "hibridas": "hibridar",
    "hibridase": "hibridar",
    "hibridaseis": "hibridar",
    "hibridasen": "hibridar",
    "hibridases": "hibridar",
    "hibridaste": "hibridar",
    "hibridasteis": "hibridar",
    "hibride": "hibridar",
    "hibridemos": "hibridar",
    "hibriden": "hibridar",
    "hibrides": "hibridar",
    "hibridice": "hibridizar",
    "hibridicemos": "hibridizar",
    "hibridicen": "hibridizar",
    "hibridices": "hibridizar",
    "hibridicé": "hibridizar",
    "hibridicéis": "hibridizar",
    "hibridismos": "hibridismo",
    "hibridiza": "hibridizar",
    "hibridizaba": "hibridizar",
    "hibridizabais": "hibridizar",
    "hibridizaban": "hibridizar",
    "hibridizabas": "hibridizar",
    "hibridizaciones": "hibridización",
    "hibridizad": "hibridizar",
    "hibridizada": "hibridizar",
    "hibridizadas": "hibridizar",
    "hibridizado": "hibridizar",
    "hibridizados": "hibridizar",
    "hibridizamos": "hibridizar",
    "hibridizan": "hibridizar",
    "hibridizando": "hibridizar",
    "hibridizara": "hibridizar",
    "hibridizarais": "hibridizar",
    "hibridizaran": "hibridizar",
    "hibridizaras": "hibridizar",
    "hibridizare": "hibridizar",
    "hibridizareis": "hibridizar",
    "hibridizaremos": "hibridizar",
    "hibridizaren": "hibridizar",
    "hibridizares": "hibridizar",
    "hibridizaron": "hibridizar",
    "hibridizará": "hibridizar",
    "hibridizarán": "hibridizar",
    "hibridizarás": "hibridizar",
    "hibridizaré": "hibridizar",
    "hibridizaréis": "hibridizar",
    "hibridizaría": "hibridizar",
    "hibridizaríais": "hibridizar",
    "hibridizaríamos": "hibridizar",
    "hibridizarían": "hibridizar",
    "hibridizarías": "hibridizar",
    "hibridizas": "hibridizar",
    "hibridizase": "hibridizar",
    "hibridizaseis": "hibridizar",
    "hibridizasen": "hibridizar",
    "hibridizases": "hibridizar",
    "hibridizaste": "hibridizar",
    "hibridizasteis": "hibridizar",
    "hibridizo": "hibridizar",
    "hibridizábamos": "hibridizar",
    "hibridizáis": "hibridizar",
    "hibridizáramos": "hibridizar",
    "hibridizáremos": "hibridizar",
    "hibridizásemos": "hibridizar",
    "hibridizó": "hibridizar",
    "hibrido": "hibridar",
    "hibridomas": "hibridoma",
    "hibridábamos": "hibridar",
    "hibridáis": "hibridar",
    "hibridáramos": "hibridar",
    "hibridáremos": "hibridar",
    "hibridásemos": "hibridar",
    "hibridé": "hibridar",
    "hibridéis": "hibridar",
    "hibridó": "hibridar",
    "hicacos": "hicaco",
    "hice": "hacer",
    "hiciera": "hacer",
    "hicierais": "hacer",
    "hicieran": "hacer",
    "hicieras": "hacer",
    "hiciere": "hacer",
    "hiciereis": "hacer",
    "hicieren": "hacer",
    "hicieres": "hacer",
    "hicieron": "hacer",
    "hiciese": "hacer",
    "hicieseis": "hacer",
    "hiciesen": "hacer",
    "hicieses": "hacer",
    "hicimos": "hacer",
    "hiciste": "hacer",
    "hicisteis": "hacer",
    "hiciéramos": "hacer",
    "hiciéremos": "hacer",
    "hiciésemos": "hacer",
    "hicsos": "hicso",
    "hidalga": "hidalgo",
    "hidalgas": "hidalgo",
    "hidalgos": "hidalgo",
    "hidalgueces": "hidalguez",
    "hidalguías": "hidalguía",
    "hidantoínas": "hidantoína",
    "hidatídica": "hidatídico",
    "hidatídicas": "hidatídico",
    "hidatídicos": "hidatídico",
    "hiddenitas": "hiddenita",
    "hidnocárpica": "hidnocárpico",
    "hidnocárpicas": "hidnocárpico",
    "hidnocárpicos": "hidnocárpico",
    "hidracinas": "hidracina",
    "hidrantes": "hidrante",
    "hidrargilitas": "hidrargilita",
    "hidrargirios": "hidrargirio",
    "hidrargirismos": "hidrargirismo",
    "hidrargiros": "hidrargiro",
    "hidras": "hidra",
    "hidrata": "hidratar",
    "hidrataba": "hidratar",
    "hidratabais": "hidratar",
    "hidrataban": "hidratar",
    "hidratabas": "hidratar",
    "hidrataciones": "hidratación",
    "hidratad": "hidratar",
    "hidratada": "hidratar",
    "hidratadas": "hidratar",
    "hidratado": "hidratar",
    "hidratadora": "hidratador",
    "hidratadoras": "hidratador",
    "hidratadores": "hidratador",
    "hidratados": "hidratar",
    "hidratamos": "hidratar",
    "hidratan": "hidratar",
    "hidratando": "hidratar",
    "hidratantes": "hidratante",
    "hidratara": "hidratar",
    "hidratarais": "hidratar",
    "hidrataran": "hidratar",
    "hidrataras": "hidratar",
    "hidratare": "hidratar",
    "hidratareis": "hidratar",
    "hidrataremos": "hidratar",
    "hidrataren": "hidratar",
    "hidratares": "hidratar",
    "hidrataron": "hidratar",
    "hidratará": "hidratar",
    "hidratarán": "hidratar",
    "hidratarás": "hidratar",
    "hidrataré": "hidratar",
    "hidrataréis": "hidratar",
    "hidrataría": "hidratar",
    "hidrataríais": "hidratar",
    "hidrataríamos": "hidratar",
    "hidratarían": "hidratar",
    "hidratarías": "hidratar",
    "hidratas": "hidratar",
    "hidratase": "hidratar",
    "hidrataseis": "hidratar",
    "hidratasen": "hidratar",
    "hidratases": "hidratar",
    "hidrataste": "hidratar",
    "hidratasteis": "hidratar",
    "hidrate": "hidratar",
    "hidratemos": "hidratar",
    "hidraten": "hidratar",
    "hidrates": "hidratar",
    "hidrato": "hidratar",
    "hidratos": "hidrato",
    "hidratábamos": "hidratar",
    "hidratáis": "hidratar",
    "hidratáramos": "hidratar",
    "hidratáremos": "hidratar",
    "hidratásemos": "hidratar",
    "hidraté": "hidratar",
    "hidratéis": "hidratar",
    "hidrató": "hidratar",
    "hidrazidas": "hidrazida",
    "hidrazobencenos": "hidrazobenceno",
    "hidrazocompuestos": "hidrazocompuesto",
    "hidrazonas": "hidrazona",
    "hidrazos": "hidrazo",
    "hidrias": "hidria",
    "hidroacústica": "hidroacústico",
    "hidroacústicas": "hidroacústico",
    "hidroacústicos": "hidroacústico",
    "hidroalas": "hidroala",
    "hidroaviones": "hidroavión",
    "hidrobiologías": "hidrobiología",
    "hidrobromofluorocarburos": "hidrobromofluorocarburo",
    "hidrocarbonilaciones": "hidrocarbonilación",
    "hidrocarburos": "hidrocarburo",
    "hidrocaritáceas": "hidrocaritácea",
    "hidrocefalias": "hidrocefalia",
    "hidrocefálica": "hidrocefálico",
    "hidrocefálicas": "hidrocefálico",
    "hidrocefálicos": "hidrocefálico",
    "hidroceles": "hidrocele",
    "hidrocelulosas": "hidrocelulosa",
    "hidrocianuraciones": "hidrocianuración",
    "hidrociclones": "hidrociclón",
    "hidrocincitas": "hidrocincita",
    "hidrocinámica": "hidrocinámico",
    "hidrocinámicas": "hidrocinámico",
    "hidrocinámicos": "hidrocinámico",
    "hidroclasificadores": "hidroclasificador",
    "hidrocloraciones": "hidrocloración",
    "hidrocloratos": "hidroclorato",
    "hidroclorofluorocarburos": "hidroclorofluorocarburo",
    "hidroclorotiazidas": "hidroclorotiazida",
    "hidrocloruros": "hidrocloruro",
    "hidrocoloides": "hidrocoloide",
    "hidrocortisonas": "hidrocortisona",
    "hidrocuciones": "hidrocución",
    "hidrocultivos": "hidrocultivo",
    "hidrocéfala": "hidrocéfalo",
    "hidrocéfalas": "hidrocéfalo",
    "hidrocéfalos": "hidrocéfalo",
    "hidrodesalquilaciones": "hidrodesalquilación",
    "hidrodinámica": "hidrodinámico",
    "hidrodinámicas": "hidrodinámico",
    "hidrodinámicos": "hidrodinámico",
    "hidroelectricidades": "hidroelectricidad",
    "hidroeléctrica": "hidroeléctrico",
    "hidroeléctricas": "hidroeléctrico",
    "hidroeléctricos": "hidroeléctrico",
    "hidroextracciones": "hidroextracción",
    "hidroextractores": "hidroextractor",
    "hidrofanas": "hidrofana",
    "hidrofilacios": "hidrofilacio",
    "hidrofisuraciones": "hidrofisuración",
    "hidrofobias": "hidrofobia",
    "hidroformaciones": "hidroformación",
    "hidroformilaciones": "hidroformilación",
    "hidroftalmías": "hidroftalmía",
    "hidrofugacias": "hidrofugacia",
    "hidrofugaciones": "hidrofugación",
    "hidrofuramidas": "hidrofuramida",
    "hidrofóbica": "hidrofóbico",
    "hidrofóbicas": "hidrofóbico",
    "hidrofóbicos": "hidrofóbico",
    "hidrogelas": "hidrogela",
    "hidrogena": "hidrogenar",
    "hidrogenaba": "hidrogenar",
    "hidrogenabais": "hidrogenar",
    "hidrogenaban": "hidrogenar",
    "hidrogenabas": "hidrogenar",
    "hidrogenaciones": "hidrogenación",
    "hidrogenad": "hidrogenar",
    "hidrogenada": "hidrogenar",
    "hidrogenadas": "hidrogenar",
    "hidrogenado": "hidrogenar",
    "hidrogenados": "hidrogenar",
    "hidrogenamos": "hidrogenar",
    "hidrogenan": "hidrogenar",
    "hidrogenando": "hidrogenar",
    "hidrogenara": "hidrogenar",
    "hidrogenarais": "hidrogenar",
    "hidrogenaran": "hidrogenar",
    "hidrogenaras": "hidrogenar",
    "hidrogenare": "hidrogenar",
    "hidrogenareis": "hidrogenar",
    "hidrogenaremos": "hidrogenar",
    "hidrogenaren": "hidrogenar",
    "hidrogenares": "hidrogenar",
    "hidrogenaron": "hidrogenar",
    "hidrogenará": "hidrogenar",
    "hidrogenarán": "hidrogenar",
    "hidrogenarás": "hidrogenar",
    "hidrogenaré": "hidrogenar",
    "hidrogenaréis": "hidrogenar",
    "hidrogenaría": "hidrogenar",
    "hidrogenaríais": "hidrogenar",
    "hidrogenaríamos": "hidrogenar",
    "hidrogenarían": "hidrogenar",
    "hidrogenarías": "hidrogenar",
    "hidrogenas": "hidrogenar",
    "hidrogenasas": "hidrogenasa",
    "hidrogenase": "hidrogenar",
    "hidrogenaseis": "hidrogenar",
    "hidrogenasen": "hidrogenar",
    "hidrogenases": "hidrogenar",
    "hidrogenaste": "hidrogenar",
    "hidrogenasteis": "hidrogenar",
    "hidrogene": "hidrogenar",
    "hidrogenemos": "hidrogenar",
    "hidrogenen": "hidrogenar",
    "hidrogenes": "hidrogenar",
    "hidrogeno": "hidrogenar",
    "hidrogenocarbonatos": "hidrogenocarbonato",
    "hidrogenoides": "hidrogenoide",
    "hidrogenosomas": "hidrogenosoma",
    "hidrogenábamos": "hidrogenar",
    "hidrogenáis": "hidrogenar",
    "hidrogenáramos": "hidrogenar",
    "hidrogenáremos": "hidrogenar",
    "hidrogenásemos": "hidrogenar",
    "hidrogené": "hidrogenar",
    "hidrogenéis": "hidrogenar",
    "hidrogenó": "hidrogenar",
    "hidrogeologías": "hidrogeología",
    "hidrogeológica": "hidrogeológico",
    "hidrogeológicas": "hidrogeológico",
    "hidrogeológicos": "hidrogeológico",
    "hidrogeomorfologías": "hidrogeomorfología",
    "hidrogeoquímica": "hidrogeoquímico",
    "hidrogeoquímicas": "hidrogeoquímico",
    "hidrogeoquímicos": "hidrogeoquímico",
    "hidrogeólica": "hidrogeólico",
    "hidrogeólicas": "hidrogeólico",
    "hidrogeólicos": "hidrogeólico",
    "hidrogeóloga": "hidrogeólogo",
    "hidrogeólogas": "hidrogeólogo",
    "hidrogeólogos": "hidrogeólogo",
    "hidrognosias": "hidrognosia",
    "hidrogogías": "hidrogogía",
    "hidrografías": "hidrografía",
    "hidrogramas": "hidrograma",
    "hidrogrosularias": "hidrogrosularia",
    "hidrográfica": "hidrográfico",
    "hidrográficas": "hidrográfico",
    "hidrográficos": "hidrográfico",
    "hidrohalogenada": "hidrohalogenado",
    "hidrohalogenadas": "hidrohalogenado",
    "hidrohalogenados": "hidrohalogenado",
    "hidrojardineras": "hidrojardinera",
    "hidrolasas": "hidrolasa",
    "hidrolice": "hidrolizar",
    "hidrolicemos": "hidrolizar",
    "hidrolicen": "hidrolizar",
    "hidrolices": "hidrolizar",
    "hidrolicé": "hidrolizar",
    "hidrolicéis": "hidrolizar",
    "hidroliza": "hidrolizar",
    "hidrolizaba": "hidrolizar",
    "hidrolizabais": "hidrolizar",
    "hidrolizaban": "hidrolizar",
    "hidrolizabas": "hidrolizar",
    "hidrolizaciones": "hidrolización",
    "hidrolizad": "hidrolizar",
    "hidrolizada": "hidrolizar",
    "hidrolizadas": "hidrolizar",
    "hidrolizado": "hidrolizar",
    "hidrolizados": "hidrolizar",
    "hidrolizamos": "hidrolizar",
    "hidrolizan": "hidrolizar",
    "hidrolizando": "hidrolizar",
    "hidrolizara": "hidrolizar",
    "hidrolizarais": "hidrolizar",
    "hidrolizaran": "hidrolizar",
    "hidrolizaras": "hidrolizar",
    "hidrolizare": "hidrolizar",
    "hidrolizareis": "hidrolizar",
    "hidrolizaremos": "hidrolizar",
    "hidrolizaren": "hidrolizar",
    "hidrolizares": "hidrolizar",
    "hidrolizaron": "hidrolizar",
    "hidrolizará": "hidrolizar",
    "hidrolizarán": "hidrolizar",
    "hidrolizarás": "hidrolizar",
    "hidrolizaré": "hidrolizar",
    "hidrolizaréis": "hidrolizar",
    "hidrolizaría": "hidrolizar",
    "hidrolizaríais": "hidrolizar",
    "hidrolizaríamos": "hidrolizar",
    "hidrolizarían": "hidrolizar",
    "hidrolizarías": "hidrolizar",
    "hidrolizas": "hidrolizar",
    "hidrolizase": "hidrolizar",
    "hidrolizaseis": "hidrolizar",
    "hidrolizasen": "hidrolizar",
    "hidrolizases": "hidrolizar",
    "hidrolizaste": "hidrolizar",
    "hidrolizasteis": "hidrolizar",
    "hidrolizo": "hidrolizar",
    "hidrolizábamos": "hidrolizar",
    "hidrolizáis": "hidrolizar",
    "hidrolizáramos": "hidrolizar",
    "hidrolizáremos": "hidrolizar",
    "hidrolizásemos": "hidrolizar",
    "hidrolizó": "hidrolizar",
    "hidrologías": "hidrología",
    "hidrolítica": "hidrolítico",
    "hidrolíticas": "hidrolítico",
    "hidrolíticos": "hidrolítico",
    "hidrológica": "hidrológico",
    "hidrológicas": "hidrológico",
    "hidrológicos": "hidrológico",
    "hidromancias": "hidromancia",
    "hidromancías": "hidromancía",
    "hidromas": "hidroma",
    "hidromecánica": "hidromecánico",
    "hidromecánicas": "hidromecánico",
    "hidromecánicos": "hidromecánico",
    "hidromedusas": "hidromedusa",
    "hidromeles": "hidromel",
    "hidromerizaciones": "hidromerización",
    "hidrometalurgias": "hidrometalurgia",
    "hidrometeora": "hidrometeoro",
    "hidrometeoras": "hidrometeoro",
    "hidrometeorologías": "hidrometeorología",
    "hidrometeorológica": "hidrometeorológico",
    "hidrometeorológicas": "hidrometeorológico",
    "hidrometeorológicos": "hidrometeorológico",
    "hidrometeoros": "hidrometeoro",
    "hidrometrías": "hidrometría",
    "hidromicas": "hidromica",
    "hidromieles": "hidromiel",
    "hidromodelismos": "hidromodelismo",
    "hidromorfa": "hidromorfo",
    "hidromorfas": "hidromorfo",
    "hidromorfos": "hidromorfo",
    "hidromorfías": "hidromorfía",
    "hidromántica": "hidromántico",
    "hidrománticas": "hidromántico",
    "hidrománticos": "hidromántico",
    "hidromécanica": "hidromécanico",
    "hidromécanicas": "hidromécanico",
    "hidromécanicos": "hidromécanico",
    "hidrométrica": "hidrométrico",
    "hidrométricas": "hidrométrico",
    "hidrométricos": "hidrométrico",
    "hidromórfica": "hidromórfico",
    "hidromórficas": "hidromórfico",
    "hidromórficos": "hidromórfico",
    "hidronefrótica": "hidronefrótico",
    "hidronefróticas": "hidronefrótico",
    "hidronefróticos": "hidronefrótico",
    "hidroneumática": "hidroneumático",
    "hidroneumáticas": "hidroneumático",
    "hidroneumáticos": "hidroneumático",
    "hidronimias": "hidronimia",
    "hidronios": "hidronio",
    "hidropatías": "hidropatía",
    "hidroperóxidos": "hidroperóxido",
    "hidropesías": "hidropesía",
    "hidroplaneos": "hidroplaneo",
    "hidroplanos": "hidroplano",
    "hidroponías": "hidroponía",
    "hidropotables": "hidropotable",
    "hidroptera": "hidroptero",
    "hidropteras": "hidroptero",
    "hidropteros": "hidroptero",
    "hidropática": "hidropático",
    "hidropáticas": "hidropático",
    "hidropáticos": "hidropático",
    "hidropónica": "hidropónico",
    "hidropónicas": "hidropónico",
    "hidropónicos": "hidropónico",
    "hidroquinonas": "hidroquinona",
    "hidroquímicas": "hidroquímica",
    "hidrosa": "hidroso",
    "hidrosas": "hidroso",
    "hidroscopias": "hidroscopia",
    "hidroseparaciones": "hidroseparación",
    "hidroseparadores": "hidroseparador",
    "hidrosferas": "hidrosfera",
    "hidrosiembras": "hidrosiembra",
    "hidrosoles": "hidrosol",
    "hidrosolubles": "hidrosoluble",
    "hidrosos": "hidroso",
    "hidrospeeds": "hidrospeed",
    "hidrostática": "hidrostático",
    "hidrostáticas": "hidrostático",
    "hidrostáticos": "hidrostático",
    "hidrosulfitos": "hidrosulfito",
    "hidrosulfuros": "hidrosulfuro",
    "hidrotaquímetros": "hidrotaquímetro",
    "hidrotecnias": "hidrotecnia",
    "hidroterapias": "hidroterapia",
    "hidrotermales": "hidrotermal",
    "hidroterápica": "hidroterápico",
    "hidroterápicas": "hidroterápico",
    "hidroterápicos": "hidroterápico",
    "hidrotimetrías": "hidrotimetría",
    "hidrotimétrica": "hidrotimétrico",
    "hidrotimétricas": "hidrotimétrico",
    "hidrotimétricos": "hidrotimétrico",
    "hidrotropismos": "hidrotropismo",
    "hidrotropos": "hidrotropo",
    "hidrotropías": "hidrotropía",
    "hidrotérmica": "hidrotérmico",
    "hidrotérmicas": "hidrotérmico",
    "hidrotérmicos": "hidrotérmico",
    "hidroxiacetonas": "hidroxiacetona",
    "hidroxiacética": "hidroxiacético",
    "hidroxiacéticas": "hidroxiacético",
    "hidroxiacéticos": "hidroxiacético",
    "hidroxiadipaldehídos": "hidroxiadipaldehído",
    "hidroxiapatitas": "hidroxiapatita",
    "hidroxibenzaldehídos": "hidroxibenzaldehído",
    "hidroxibenzoatos": "hidroxibenzoato",
    "hidroxibutírica": "hidroxibutírico",
    "hidroxibutíricas": "hidroxibutírico",
    "hidroxibutíricos": "hidroxibutírico",
    "hidroxicloroquinas": "hidroxicloroquina",
    "hidroxidibenzofuranos": "hidroxidibenzofurano",
    "hidroxidifenilaminas": "hidroxidifenilamina",
    "hidroxiestearatos": "hidroxiestearato",
    "hidroxietilcelulosas": "hidroxietilcelulosa",
    "hidroxietilendiaminas": "hidroxietilendiamina",
    "hidroxietilhidracinas": "hidroxietilhidracina",
    "hidroxifenilglicinas": "hidroxifenilglicina",
    "hidroxifenilmercúrica": "hidroxifenilmercúrico",
    "hidroxifenilmercúricas": "hidroxifenilmercúrico",
    "hidroxifenilmercúricos": "hidroxifenilmercúrico",
    "hidroxiisobutiratos": "hidroxiisobutirato",
    "hidroxilaminas": "hidroxilamina",
    "hidroxilos": "hidroxilo",
    "hidroximercuriclorofenoles": "hidroximercuriclorofenol",
    "hidroximercuricresoles": "hidroximercuricresol",
    "hidroximercurinitrofenoles": "hidroximercurinitrofenol",
    "hidroximetilbutanonas": "hidroximetilbutanona",
    "hidroxinaftoica": "hidroxinaftoico",
    "hidroxinaftoicas": "hidroxinaftoico",
    "hidroxinaftoicos": "hidroxinaftoico",
    "hidroxinaftoquinonas": "hidroxinaftoquinona",
    "hidroxiprolinas": "hidroxiprolina",
    "hidroxipropilglicerinas": "hidroxipropilglicerina",
    "hidroxiquinoleínas": "hidroxiquinoleína",
    "hidroxiácidos": "hidroxiácido",
    "hidrozoos": "hidrozoo",
    "hidruros": "hidruro",
    "hidrácidos": "hidrácido",
    "hidráulica": "hidráulico",
    "hidráulicas": "hidráulico",
    "hidráulicos": "hidráulico",
    "hidrófanas": "hidrófana",
    "hidrófila": "hidrófilo",
    "hidrófilas": "hidrófilo",
    "hidrófilos": "hidrófilo",
    "hidrófitos": "hidrófito",
    "hidrófoba": "hidrófobo",
    "hidrófobas": "hidrófobo",
    "hidrófobos": "hidrófobo",
    "hidrófonos": "hidrófono",
    "hidrófuga": "hidrófugo",
    "hidrófugas": "hidrófugo",
    "hidrófugos": "hidrófugo",
    "hidrógenos": "hidrógeno",
    "hidrógrafa": "hidrógrafo",
    "hidrógrafas": "hidrógrafo",
    "hidrógrafos": "hidrógrafo",
    "hidróloga": "hidrólogo",
    "hidrólogas": "hidrólogo",
    "hidrólogos": "hidrólogo",
    "hidrómetras": "hidrómetra",
    "hidrómetros": "hidrómetro",
    "hidrónimos": "hidrónimo",
    "hidrópatas": "hidrópata",
    "hidrópica": "hidrópico",
    "hidrópicas": "hidrópico",
    "hidrópicos": "hidrópico",
    "hidróxidos": "hidróxido",
    "hidátides": "hidátide",
    "hieda": "heder",
    "hiedan": "heder",
    "hiedas": "heder",
    "hiede": "heder",
    "hieden": "heder",
    "hiedes": "heder",
    "hiedo": "heder",
    "hiedras": "hiedra",
    "hiela": "helar",
    "hielan": "helar",
    "hielas": "helar",
    "hiele": "helar",
    "hielen": "helar",
    "hieles": "hiel",
    "hielo": "helar",
    "hielos": "hielo",
    "hiemales": "hiemal",
    "hiemoscos": "hiemosco",
    "hienas": "hiena",
    "hienda": "hendir",
    "hiendan": "hendir",
    "hiendas": "hendir",
    "hiende": "hendir",
    "hienden": "hendir",
    "hiendes": "hendir",
    "hiendo": "hendir",
    "hiera": "herir",
    "hieran": "herir",
    "hieras": "herir",
    "hieratismos": "hieratismo",
    "hierba": "herbar",
    "hierbabuenas": "hierbabuena",
    "hierban": "herbar",
    "hierbas": "hierba",
    "hierbe": "herbar",
    "hierben": "herbar",
    "hierbes": "herbar",
    "hierbo": "herbar",
    "hiere": "herir",
    "hieren": "herir",
    "hieres": "herir",
    "hiero": "herir",
    "hierofantes": "hierofante",
    "hierografías": "hierografía",
    "hieroscopias": "hieroscopia",
    "hierosolimitana": "hierosolimitano",
    "hierosolimitanas": "hierosolimitano",
    "hierosolimitanos": "hierosolimitano",
    "hierra": "herrar",
    "hierran": "herrar",
    "hierras": "hierra",
    "hierre": "herrar",
    "hierren": "herrar",
    "hierres": "herrar",
    "hierro": "herrar",
    "hierros": "hierro",
    "hierva": "hervir",
    "hiervan": "hervir",
    "hiervas": "hervir",
    "hierve": "hervir",
    "hierven": "hervir",
    "hierves": "hervir",
    "hiervo": "hervir",
    "hierática": "hierático",
    "hieráticas": "hierático",
    "hieráticos": "hierático",
    "hieródula": "hieródulo",
    "hieródulas": "hieródulo",
    "hieródulos": "hieródulo",
    "hifas": "hifa",
    "higadillas": "higadilla",
    "higadillos": "higadillo",
    "higas": "higa",
    "higienees": "higiene",
    "higienes": "higiene",
    "higienice": "higienizar",
    "higienicemos": "higienizar",
    "higienicen": "higienizar",
    "higienices": "higienizar",
    "higienicé": "higienizar",
    "higienicéis": "higienizar",
    "higienistas": "higienista",
    "higieniza": "higienizar",
    "higienizaba": "higienizar",
    "higienizabais": "higienizar",
    "higienizaban": "higienizar",
    "higienizabas": "higienizar",
    "higienizaciones": "higienización",
    "higienizad": "higienizar",
    "higienizada": "higienizar",
    "higienizadas": "higienizar",
    "higienizado": "higienizar",
    "higienizados": "higienizar",
    "higienizamos": "higienizar",
    "higienizan": "higienizar",
    "higienizando": "higienizar",
    "higienizara": "higienizar",
    "higienizarais": "higienizar",
    "higienizaran": "higienizar",
    "higienizaras": "higienizar",
    "higienizare": "higienizar",
    "higienizareis": "higienizar",
    "higienizaremos": "higienizar",
    "higienizaren": "higienizar",
    "higienizares": "higienizar",
    "higienizaron": "higienizar",
    "higienizará": "higienizar",
    "higienizarán": "higienizar",
    "higienizarás": "higienizar",
    "higienizaré": "higienizar",
    "higienizaréis": "higienizar",
    "higienizaría": "higienizar",
    "higienizaríais": "higienizar",
    "higienizaríamos": "higienizar",
    "higienizarían": "higienizar",
    "higienizarías": "higienizar",
    "higienizas": "higienizar",
    "higienizase": "higienizar",
    "higienizaseis": "higienizar",
    "higienizasen": "higienizar",
    "higienizases": "higienizar",
    "higienizaste": "higienizar",
    "higienizasteis": "higienizar",
    "higienizo": "higienizar",
    "higienizábamos": "higienizar",
    "higienizáis": "higienizar",
    "higienizáramos": "higienizar",
    "higienizáremos": "higienizar",
    "higienizásemos": "higienizar",
    "higienizó": "higienizar",
    "higiénica": "higiénico",
    "higiénicas": "higiénico",
    "higiénicos": "higiénico",
    "higos": "higo",
    "higromas": "higroma",
    "higrometrías": "higrometría",
    "higrométrica": "higrométrico",
    "higrométricas": "higrométrico",
    "higrométricos": "higrométrico",
    "higroscopias": "higroscopia",
    "higroscopios": "higroscopio",
    "higroscópica": "higroscópico",
    "higroscópicas": "higroscópico",
    "higroscópicos": "higroscópico",
    "higrófila": "higrófilo",
    "higrófilas": "higrófilo",
    "higrófilos": "higrófilo",
    "higrófitos": "higrófito",
    "higrógrafos": "higrógrafo",
    "higrómetros": "higrómetro",
    "higueras": "higuera",
    "higuerones": "higuerón",
    "higueruelas": "higueruela",
    "hija": "hijo",
    "hijas": "hijo",
    "hijastra": "hijastro",
    "hijastras": "hijastro",
    "hijastros": "hijastro",
    "hijatos": "hijato",
    "hijos": "hijo",
    "hijosdalgo": "hijodalgo",
    "hijuelas": "hijuela",
    "hijuelos": "hijuelo",
    "hila": "hilar",
    "hilaba": "hilar",
    "hilabais": "hilar",
    "hilaban": "hilar",
    "hilabas": "hilar",
    "hilachas": "hilacha",
    "hilachos": "hilacho",
    "hilachosa": "hilachoso",
    "hilachosas": "hilachoso",
    "hilachosos": "hilachoso",
    "hilad": "hilar",
    "hilada": "hilar",
    "hiladas": "hilar",
    "hiladillos": "hiladillo",
    "hiladiza": "hiladizo",
    "hiladizas": "hiladizo",
    "hiladizos": "hiladizo",
    "hilado": "hilar",
    "hiladoras": "hiladora",
    "hilados": "hilar",
    "hilamos": "hilar",
    "hilan": "hilar",
    "hilandera": "hilandero",
    "hilanderas": "hilandero",
    "hilanderos": "hilandero",
    "hilanderías": "hilandería",
    "hilando": "hilar",
    "hilanzas": "hilanza",
    "hilara": "hilar",
    "hilarachas": "hilaracha",
    "hilarais": "hilar",
    "hilaran": "hilar",
    "hilarantes": "hilarante",
    "hilaras": "hilar",
    "hilare": "hilar",
    "hilareis": "hilar",
    "hilaremos": "hilar",
    "hilaren": "hilar",
    "hilares": "hilar",
    "hilaridades": "hilaridad",
    "hilaron": "hilar",
    "hilará": "hilar",
    "hilarán": "hilar",
    "hilarás": "hilar",
    "hilaré": "hilar",
    "hilaréis": "hilar",
    "hilaría": "hilar",
    "hilaríais": "hilar",
    "hilaríamos": "hilar",
    "hilarían": "hilar",
    "hilarías": "hilar",
    "hilas": "hilar",
    "hilase": "hilar",
    "hilaseis": "hilar",
    "hilasen": "hilar",
    "hilases": "hilar",
    "hilaste": "hilar",
    "hilasteis": "hilar",
    "hilaturas": "hilatura",
    "hilazas": "hilaza",
    "hile": "hilar",
    "hilemorfismos": "hilemorfismo",
    "hilemos": "hilar",
    "hilen": "hilar",
    "hileradoras": "hileradora",
    "hileras": "hilera",
    "hileros": "hilero",
    "hiles": "hilar",
    "hilillos": "hilillo",
    "hilios": "hilio",
    "hilo": "hilar",
    "hilomorfismos": "hilomorfismo",
    "hilos": "hilo",
    "hilozoísmos": "hilozoísmo",
    "hilvana": "hilvanar",
    "hilvanaba": "hilvanar",
    "hilvanabais": "hilvanar",
    "hilvanaban": "hilvanar",
    "hilvanabas": "hilvanar",
    "hilvanad": "hilvanar",
    "hilvanada": "hilvanar",
    "hilvanadas": "hilvanar",
    "hilvanado": "hilvanar",
    "hilvanados": "hilvanar",
    "hilvanamos": "hilvanar",
    "hilvanan": "hilvanar",
    "hilvanando": "hilvanar",
    "hilvanara": "hilvanar",
    "hilvanarais": "hilvanar",
    "hilvanaran": "hilvanar",
    "hilvanaras": "hilvanar",
    "hilvanare": "hilvanar",
    "hilvanareis": "hilvanar",
    "hilvanaremos": "hilvanar",
    "hilvanaren": "hilvanar",
    "hilvanares": "hilvanar",
    "hilvanaron": "hilvanar",
    "hilvanará": "hilvanar",
    "hilvanarán": "hilvanar",
    "hilvanarás": "hilvanar",
    "hilvanaré": "hilvanar",
    "hilvanaréis": "hilvanar",
    "hilvanaría": "hilvanar",
    "hilvanaríais": "hilvanar",
    "hilvanaríamos": "hilvanar",
    "hilvanarían": "hilvanar",
    "hilvanarías": "hilvanar",
    "hilvanas": "hilvanar",
    "hilvanase": "hilvanar",
    "hilvanaseis": "hilvanar",
    "hilvanasen": "hilvanar",
    "hilvanases": "hilvanar",
    "hilvanaste": "hilvanar",
    "hilvanasteis": "hilvanar",
    "hilvane": "hilvanar",
    "hilvanemos": "hilvanar",
    "hilvanen": "hilvanar",
    "hilvanes": "hilvanar",
    "hilvano": "hilvanar",
    "hilvanábamos": "hilvanar",
    "hilvanáis": "hilvanar",
    "hilvanáramos": "hilvanar",
    "hilvanáremos": "hilvanar",
    "hilvanásemos": "hilvanar",
    "hilvané": "hilvanar",
    "hilvanéis": "hilvanar",
    "hilvanó": "hilvanar",
    "hilvánes": "hilván",
    "hilábamos": "hilar",
    "hiláis": "hilar",
    "hiláramos": "hilar",
    "hiláremos": "hilar",
    "hilásemos": "hilar",
    "hilé": "hilar",
    "hiléis": "hilar",
    "hiló": "hilar",
    "himatomelánica": "himatomelánico",
    "himatomelánicas": "himatomelánico",
    "himatomelánicos": "himatomelánico",
    "himeneales": "himeneal",
    "himeneos": "himeneo",
    "himenópteros": "himenóptero",
    "himnarios": "himnario",
    "himnodias": "himnodia",
    "himnos": "himno",
    "himpla": "himplar",
    "himplaba": "himplar",
    "himplabais": "himplar",
    "himplaban": "himplar",
    "himplabas": "himplar",
    "himplad": "himplar",
    "himplada": "himplar",
    "himpladas": "himplar",
    "himplado": "himplar",
    "himplados": "himplar",
    "himplamos": "himplar",
    "himplan": "himplar",
    "himplando": "himplar",
    "himplara": "himplar",
    "himplarais": "himplar",
    "himplaran": "himplar",
    "himplaras": "himplar",
    "himplare": "himplar",
    "himplareis": "himplar",
    "himplaremos": "himplar",
    "himplaren": "himplar",
    "himplares": "himplar",
    "himplaron": "himplar",
    "himplará": "himplar",
    "himplarán": "himplar",
    "himplarás": "himplar",
    "himplaré": "himplar",
    "himplaréis": "himplar",
    "himplaría": "himplar",
    "himplaríais": "himplar",
    "himplaríamos": "himplar",
    "himplarían": "himplar",
    "himplarías": "himplar",
    "himplas": "himplar",
    "himplase": "himplar",
    "himplaseis": "himplar",
    "himplasen": "himplar",
    "himplases": "himplar",
    "himplaste": "himplar",
    "himplasteis": "himplar",
    "himple": "himplar",
    "himplemos": "himplar",
    "himplen": "himplar",
    "himples": "himplar",
    "himplo": "himplar",
    "himplábamos": "himplar",
    "himpláis": "himplar",
    "himpláramos": "himplar",
    "himpláremos": "himplar",
    "himplásemos": "himplar",
    "himplé": "himplar",
    "himpléis": "himplar",
    "himpló": "himplar",
    "hinayanas": "hinayana",
    "hinca": "hincar",
    "hincaba": "hincar",
    "hincabais": "hincar",
    "hincaban": "hincar",
    "hincabas": "hincar",
    "hincad": "hincar",
    "hincada": "hincar",
    "hincadas": "hincar",
    "hincado": "hincar",
    "hincados": "hincar",
    "hincaduras": "hincadura",
    "hincamos": "hincar",
    "hincan": "hincar",
    "hincando": "hincar",
    "hincara": "hincar",
    "hincarais": "hincar",
    "hincaran": "hincar",
    "hincaras": "hincar",
    "hincare": "hincar",
    "hincareis": "hincar",
    "hincaremos": "hincar",
    "hincaren": "hincar",
    "hincares": "hincar",
    "hincaron": "hincar",
    "hincará": "hincar",
    "hincarán": "hincar",
    "hincarás": "hincar",
    "hincaré": "hincar",
    "hincaréis": "hincar",
    "hincaría": "hincar",
    "hincaríais": "hincar",
    "hincaríamos": "hincar",
    "hincarían": "hincar",
    "hincarías": "hincar",
    "hincas": "hincar",
    "hincase": "hincar",
    "hincaseis": "hincar",
    "hincasen": "hincar",
    "hincases": "hincar",
    "hincaste": "hincar",
    "hincasteis": "hincar",
    "hincha": "hinchar",
    "hinchaba": "hinchar",
    "hinchabais": "hinchar",
    "hinchaban": "hinchar",
    "hinchabas": "hinchar",
    "hinchables": "hinchable",
    "hinchad": "hinchar",
    "hinchada": "hinchar",
    "hinchadas": "hinchar",
    "hinchado": "hinchar",
    "hinchados": "hinchar",
    "hinchamientos": "hinchamiento",
    "hinchamos": "hinchar",
    "hinchan": "hinchar",
    "hinchando": "hinchar",
    "hinchara": "hinchar",
    "hincharais": "hinchar",
    "hincharan": "hinchar",
    "hincharas": "hinchar",
    "hinchare": "hinchar",
    "hinchareis": "hinchar",
    "hincharemos": "hinchar",
    "hincharen": "hinchar",
    "hinchares": "hinchar",
    "hincharon": "hinchar",
    "hinchará": "hinchar",
    "hincharán": "hinchar",
    "hincharás": "hinchar",
    "hincharé": "hinchar",
    "hincharéis": "hinchar",
    "hincharía": "hinchar",
    "hincharíais": "hinchar",
    "hincharíamos": "hinchar",
    "hincharían": "hinchar",
    "hincharías": "hinchar",
    "hinchas": "hinchar",
    "hinchase": "hinchar",
    "hinchaseis": "hinchar",
    "hinchasen": "hinchar",
    "hinchases": "hinchar",
    "hinchaste": "hinchar",
    "hinchasteis": "hinchar",
    "hinchazones": "hinchazón",
    "hinche": "hinchar",
    "hinchemos": "hinchar",
    "hinchen": "hinchar",
    "hinches": "hinchar",
    "hinchiendo": "henchir",
    "hinchiera": "henchir",
    "hinchierais": "henchir",
    "hinchieran": "henchir",
    "hinchieras": "henchir",
    "hinchiere": "henchir",
    "hinchiereis": "henchir",
    "hinchieren": "henchir",
    "hinchieres": "henchir",
    "hinchieron": "henchir",
    "hinchiese": "henchir",
    "hinchieseis": "henchir",
    "hinchiesen": "henchir",
    "hinchieses": "henchir",
    "hinchiéramos": "henchir",
    "hinchiéremos": "henchir",
    "hinchiésemos": "henchir",
    "hinchió": "henchir",
    "hincho": "hinchar",
    "hinchábamos": "hinchar",
    "hincháis": "hinchar",
    "hincháramos": "hinchar",
    "hincháremos": "hinchar",
    "hinchásemos": "hinchar",
    "hinché": "hinchar",
    "hinchéis": "hinchar",
    "hinchó": "hinchar",
    "hinco": "hincar",
    "hincones": "hincón",
    "hincos": "hinco",
    "hincábamos": "hincar",
    "hincáis": "hincar",
    "hincáramos": "hincar",
    "hincáremos": "hincar",
    "hincásemos": "hincar",
    "hincó": "hincar",
    "hinduismos": "hinduismo",
    "hindúes": "hindú",
    "hindús": "hindú",
    "hindúses": "hindú",
    "hiniestas": "hiniesta",
    "hinojales": "hinojal",
    "hinojos": "hinojo",
    "hinque": "hincar",
    "hinquemos": "hincar",
    "hinquen": "hincar",
    "hinques": "hincar",
    "hinqué": "hincar",
    "hinquéis": "hincar",
    "hinteros": "hintero",
    "hioepiglótica": "hioepiglótico",
    "hioepiglóticas": "hioepiglótico",
    "hioepiglóticos": "hioepiglótico",
    "hioglosa": "hiogloso",
    "hioglosas": "hiogloso",
    "hioglosos": "hiogloso",
    "hioidea": "hioideo",
    "hioideas": "hioideo",
    "hioideos": "hioideo",
    "hipa": "hipar",
    "hipaba": "hipar",
    "hipabais": "hipar",
    "hipaban": "hipar",
    "hipabas": "hipar",
    "hipad": "hipar",
    "hipada": "hipar",
    "hipadas": "hipar",
    "hipado": "hipar",
    "hipados": "hipar",
    "hipamos": "hipar",
    "hipan": "hipar",
    "hipando": "hipar",
    "hipara": "hipar",
    "hiparais": "hipar",
    "hiparan": "hipar",
    "hiparas": "hipar",
    "hipare": "hipar",
    "hipareis": "hipar",
    "hiparemos": "hipar",
    "hiparen": "hipar",
    "hipares": "hipar",
    "hiparon": "hipar",
    "hipará": "hipar",
    "hiparán": "hipar",
    "hiparás": "hipar",
    "hiparé": "hipar",
    "hiparéis": "hipar",
    "hiparía": "hipar",
    "hiparíais": "hipar",
    "hiparíamos": "hipar",
    "hiparían": "hipar",
    "hiparías": "hipar",
    "hipas": "hipar",
    "hipase": "hipar",
    "hipaseis": "hipar",
    "hipasen": "hipar",
    "hipases": "hipar",
    "hipaste": "hipar",
    "hipasteis": "hipar",
    "hipe": "hipar",
    "hipemias": "hipemia",
    "hipemos": "hipar",
    "hipen": "hipar",
    "hiperactiva": "hiperactivo",
    "hiperactivas": "hiperactivo",
    "hiperactividades": "hiperactividad",
    "hiperactivos": "hiperactivo",
    "hiperacusias": "hiperacusia",
    "hiperaguda": "hiperagudo",
    "hiperagudas": "hiperagudo",
    "hiperagudos": "hiperagudo",
    "hiperalérgica": "hiperalérgico",
    "hiperalérgicas": "hiperalérgico",
    "hiperalérgicos": "hiperalérgico",
    "hiperbatones": "hipérbaton",
    "hiperboloides": "hiperboloide",
    "hiperbulias": "hiperbulia",
    "hiperburguesías": "hiperburguesía",
    "hiperbática": "hiperbático",
    "hiperbáticas": "hiperbático",
    "hiperbáticos": "hiperbático",
    "hiperbólica": "hiperbólico",
    "hiperbólicas": "hiperbólico",
    "hiperbólicos": "hiperbólico",
    "hiperbórea": "hiperbóreo",
    "hiperbóreas": "hiperbóreo",
    "hiperbóreos": "hiperbóreo",
    "hipercalcemias": "hipercalcemia",
    "hipercanales": "hipercanal",
    "hipercardioides": "hipercardioide",
    "hipercargas": "hipercarga",
    "hipercinética": "hipercinético",
    "hipercinéticas": "hipercinético",
    "hipercinéticos": "hipercinético",
    "hiperclorhidrias": "hiperclorhidria",
    "hiperclorhídrica": "hiperclorhídrico",
    "hiperclorhídricas": "hiperclorhídrico",
    "hiperclorhídricos": "hiperclorhídrico",
    "hipercoagulabilidades": "hipercoagulabilidad",
    "hipercolesterolemias": "hipercolesterolemia",
    "hiperconductividades": "hiperconductividad",
    "hipercorrecciones": "hipercorrección",
    "hipercriticismos": "hipercriticismo",
    "hipercroma": "hipercromo",
    "hipercromas": "hipercromo",
    "hipercromasias": "hipercromasia",
    "hipercromos": "hipercromo",
    "hipercromías": "hipercromía",
    "hipercrítica": "hipercrítico",
    "hipercríticas": "hipercrítico",
    "hipercríticos": "hipercrítico",
    "hiperdulías": "hiperdulía",
    "hiperemias": "hiperemia",
    "hiperemotiva": "hiperemotivo",
    "hiperemotivas": "hiperemotivo",
    "hiperemotivos": "hiperemotivo",
    "hiperespacios": "hiperespacio",
    "hiperespecializaciones": "hiperespecialización",
    "hiperesplenismos": "hiperesplenismo",
    "hiperestesias": "hiperestesia",
    "hiperestésica": "hiperestésico",
    "hiperestésicas": "hiperestésico",
    "hiperestésicos": "hiperestésico",
    "hipereutectoides": "hipereutectoide",
    "hipereutéctica": "hipereutéctico",
    "hipereutécticas": "hipereutéctico",
    "hipereutécticos": "hipereutéctico",
    "hiperextensiones": "hiperextensión",
    "hiperfina": "hiperfino",
    "hiperfinas": "hiperfino",
    "hiperfinos": "hiperfino",
    "hiperfocales": "hiperfocal",
    "hiperforias": "hiperforia",
    "hiperfrecuencias": "hiperfrecuencia",
    "hiperfunciones": "hiperfunción",
    "hipergammaglobulinemias": "hipergammaglobulinemia",
    "hipergeometrías": "hipergeometría",
    "hipergeométrica": "hipergeométrico",
    "hipergeométricas": "hipergeométrico",
    "hipergeométricos": "hipergeométrico",
    "hiperglobulias": "hiperglobulia",
    "hiperglobulinemias": "hiperglobulinemia",
    "hiperglucemiantes": "hiperglucemiante",
    "hiperglucemias": "hiperglucemia",
    "hipergoles": "hipergol",
    "hiperinflaciones": "hiperinflación",
    "hiperinmunes": "hiperinmune",
    "hiperinmunidades": "hiperinmunidad",
    "hiperinmunizaciones": "hiperinmunización",
    "hiperlepidemias": "hiperlepidemia",
    "hiperlipemias": "hiperlipemia",
    "hipermadura": "hipermaduro",
    "hipermaduras": "hipermaduro",
    "hipermaduros": "hipermaduro",
    "hipermedias": "hipermedia",
    "hipermenorreas": "hipermenorrea",
    "hipermercados": "hipermercado",
    "hipermetropías": "hipermetropía",
    "hipermetrías": "hipermetría",
    "hipermetrópica": "hipermetrópico",
    "hipermetrópicas": "hipermetrópico",
    "hipermetrópicos": "hipermetrópico",
    "hipermutaciones": "hipermutación",
    "hipermétropes": "hipermétrope",
    "hiperones": "hiperón",
    "hiperonimias": "hiperonimia",
    "hiperopías": "hiperopía",
    "hiperosmias": "hiperosmia",
    "hiperoxias": "hiperoxia",
    "hiperparásitos": "hiperparásito",
    "hiperpepsias": "hiperpepsia",
    "hiperplanos": "hiperplano",
    "hiperplasias": "hiperplasia",
    "hiperplásica": "hiperplásico",
    "hiperplásicas": "hiperplásico",
    "hiperplásicos": "hiperplásico",
    "hiperplástica": "hiperplástico",
    "hiperplásticas": "hiperplástico",
    "hiperplásticos": "hiperplástico",
    "hiperpolarizaciones": "hiperpolarización",
    "hiperpotasemias": "hiperpotasemia",
    "hiperproteinemias": "hiperproteinemia",
    "hiperprotrombinemias": "hiperprotrombinemia",
    "hiperrealismos": "hiperrealismo",
    "hipersegregaciones": "hipersegregación",
    "hipersensibilidades": "hipersensibilidad",
    "hipersensibilizaciones": "hipersensibilización",
    "hipersensibles": "hipersensible",
    "hipersonidos": "hipersonido",
    "hipersorciones": "hipersorción",
    "hiperstenas": "hiperstena",
    "hipersustentaciones": "hipersustentación",
    "hipersustentadores": "hipersustentador",
    "hipersónica": "hipersónico",
    "hipersónicas": "hipersónico",
    "hipersónicos": "hipersónico",
    "hipertelorismos": "hipertelorismo",
    "hipertensa": "hipertenso",
    "hipertensas": "hipertenso",
    "hipertensiones": "hipertensión",
    "hipertensiva": "hipertensivo",
    "hipertensivas": "hipertensivo",
    "hipertensivos": "hipertensivo",
    "hipertensos": "hipertenso",
    "hipertermias": "hipertermia",
    "hipertextos": "hipertexto",
    "hipertiroidismos": "hipertiroidismo",
    "hipertonos": "hipertono",
    "hipertrigliceridemias": "hipertrigliceridemia",
    "hipertrofia": "hipertrofiar",
    "hipertrofiaba": "hipertrofiar",
    "hipertrofiabais": "hipertrofiar",
    "hipertrofiaban": "hipertrofiar",
    "hipertrofiabas": "hipertrofiar",
    "hipertrofiad": "hipertrofiar",
    "hipertrofiada": "hipertrofiar",
    "hipertrofiadas": "hipertrofiar",
    "hipertrofiado": "hipertrofiar",
    "hipertrofiados": "hipertrofiar",
    "hipertrofiamos": "hipertrofiar",
    "hipertrofian": "hipertrofiar",
    "hipertrofiando": "hipertrofiar",
    "hipertrofiara": "hipertrofiar",
    "hipertrofiarais": "hipertrofiar",
    "hipertrofiaran": "hipertrofiar",
    "hipertrofiaras": "hipertrofiar",
    "hipertrofiare": "hipertrofiar",
    "hipertrofiareis": "hipertrofiar",
    "hipertrofiaremos": "hipertrofiar",
    "hipertrofiaren": "hipertrofiar",
    "hipertrofiares": "hipertrofiar",
    "hipertrofiaron": "hipertrofiar",
    "hipertrofiará": "hipertrofiar",
    "hipertrofiarán": "hipertrofiar",
    "hipertrofiarás": "hipertrofiar",
    "hipertrofiaré": "hipertrofiar",
    "hipertrofiaréis": "hipertrofiar",
    "hipertrofiaría": "hipertrofiar",
    "hipertrofiaríais": "hipertrofiar",
    "hipertrofiaríamos": "hipertrofiar",
    "hipertrofiarían": "hipertrofiar",
    "hipertrofiarías": "hipertrofiar",
    "hipertrofias": "hipertrofiar",
    "hipertrofiase": "hipertrofiar",
    "hipertrofiaseis": "hipertrofiar",
    "hipertrofiasen": "hipertrofiar",
    "hipertrofiases": "hipertrofiar",
    "hipertrofiaste": "hipertrofiar",
    "hipertrofiasteis": "hipertrofiar",
    "hipertrofie": "hipertrofiar",
    "hipertrofiemos": "hipertrofiar",
    "hipertrofien": "hipertrofiar",
    "hipertrofies": "hipertrofiar",
    "hipertrofio": "hipertrofiar",
    "hipertrofiábamos": "hipertrofiar",
    "hipertrofiáis": "hipertrofiar",
    "hipertrofiáramos": "hipertrofiar",
    "hipertrofiáremos": "hipertrofiar",
    "hipertrofiásemos": "hipertrofiar",
    "hipertrofié": "hipertrofiar",
    "hipertrofiéis": "hipertrofiar",
    "hipertrofió": "hipertrofiar",
    "hipertropías": "hipertropía",
    "hipertrófica": "hipertrófico",
    "hipertróficas": "hipertrófico",
    "hipertróficos": "hipertrófico",
    "hipertérmica": "hipertérmico",
    "hipertérmicas": "hipertérmico",
    "hipertérmicos": "hipertérmico",
    "hiperuricemias": "hiperuricemia",
    "hipervariables": "hipervariable",
    "hiperviscosidades": "hiperviscosidad",
    "hipervolemias": "hipervolemia",
    "hiperémica": "hiperémico",
    "hiperémicas": "hiperémico",
    "hiperémicos": "hiperémico",
    "hiperónimos": "hiperónimo",
    "hipes": "hipar",
    "hipidos": "hipido",
    "hipismos": "hipismo",
    "hipnoblastos": "hipnoblasto",
    "hipnodoncias": "hipnodoncia",
    "hipnoterapias": "hipnoterapia",
    "hipnotice": "hipnotizar",
    "hipnoticemos": "hipnotizar",
    "hipnoticen": "hipnotizar",
    "hipnotices": "hipnotizar",
    "hipnoticé": "hipnotizar",
    "hipnoticéis": "hipnotizar",
    "hipnotismos": "hipnotismo",
    "hipnotiza": "hipnotizar",
    "hipnotizaba": "hipnotizar",
    "hipnotizabais": "hipnotizar",
    "hipnotizaban": "hipnotizar",
    "hipnotizabas": "hipnotizar",
    "hipnotizables": "hipnotizable",
    "hipnotizaciones": "hipnotización",
    "hipnotizad": "hipnotizar",
    "hipnotizada": "hipnotizar",
    "hipnotizadas": "hipnotizar",
    "hipnotizado": "hipnotizar",
    "hipnotizadora": "hipnotizador",
    "hipnotizadoras": "hipnotizador",
    "hipnotizadores": "hipnotizador",
    "hipnotizados": "hipnotizar",
    "hipnotizamos": "hipnotizar",
    "hipnotizan": "hipnotizar",
    "hipnotizando": "hipnotizar",
    "hipnotizantes": "hipnotizante",
    "hipnotizara": "hipnotizar",
    "hipnotizarais": "hipnotizar",
    "hipnotizaran": "hipnotizar",
    "hipnotizaras": "hipnotizar",
    "hipnotizare": "hipnotizar",
    "hipnotizareis": "hipnotizar",
    "hipnotizaremos": "hipnotizar",
    "hipnotizaren": "hipnotizar",
    "hipnotizares": "hipnotizar",
    "hipnotizaron": "hipnotizar",
    "hipnotizará": "hipnotizar",
    "hipnotizarán": "hipnotizar",
    "hipnotizarás": "hipnotizar",
    "hipnotizaré": "hipnotizar",
    "hipnotizaréis": "hipnotizar",
    "hipnotizaría": "hipnotizar",
    "hipnotizaríais": "hipnotizar",
    "hipnotizaríamos": "hipnotizar",
    "hipnotizarían": "hipnotizar",
    "hipnotizarías": "hipnotizar",
    "hipnotizas": "hipnotizar",
    "hipnotizase": "hipnotizar",
    "hipnotizaseis": "hipnotizar",
    "hipnotizasen": "hipnotizar",
    "hipnotizases": "hipnotizar",
    "hipnotizaste": "hipnotizar",
    "hipnotizasteis": "hipnotizar",
    "hipnotizo": "hipnotizar",
    "hipnotizábamos": "hipnotizar",
    "hipnotizáis": "hipnotizar",
    "hipnotizáramos": "hipnotizar",
    "hipnotizáremos": "hipnotizar",
    "hipnotizásemos": "hipnotizar",
    "hipnotizó": "hipnotizar",
    "hipnótica": "hipnótico",
    "hipnóticas": "hipnótico",
    "hipnóticos": "hipnótico",
    "hipo": "hipar",
    "hipoacusias": "hipoacusia",
    "hipoadrenias": "hipoadrenia",
    "hipobromosa": "hipobromoso",
    "hipobromosas": "hipobromoso",
    "hipobromosos": "hipobromoso",
    "hipocampos": "hipocampo",
    "hipocaustos": "hipocausto",
    "hipocentauros": "hipocentauro",
    "hipocentros": "hipocentro",
    "hipocicloides": "hipocicloide",
    "hipoclorhidrias": "hipoclorhidria",
    "hipoclorhídrica": "hipoclorhídrico",
    "hipoclorhídricas": "hipoclorhídrico",
    "hipoclorhídricos": "hipoclorhídrico",
    "hipocloritos": "hipoclorito",
    "hipoclorosa": "hipocloroso",
    "hipoclorosas": "hipocloroso",
    "hipoclorosos": "hipocloroso",
    "hipocoagulabilidades": "hipocoagulabilidad",
    "hipocomplementemias": "hipocomplementemia",
    "hipocondriaca": "hipocondríaco",
    "hipocondriacas": "hipocondríaco",
    "hipocondriacos": "hipocondríaco",
    "hipocondrios": "hipocondrio",
    "hipocondrodistrofias": "hipocondrodistrofia",
    "hipocondríaca": "hipocondríaco",
    "hipocondríacas": "hipocondríaco",
    "hipocondríacos": "hipocondríaco",
    "hipocondrías": "hipocondría",
    "hipocorística": "hipocorístico",
    "hipocorísticas": "hipocorístico",
    "hipocorísticos": "hipocorístico",
    "hipocrases": "hipocrás",
    "hipocresías": "hipocresía",
    "hipocristalina": "hipocristalino",
    "hipocristalinas": "hipocristalino",
    "hipocristalinos": "hipocristalino",
    "hipocromasias": "hipocromasia",
    "hipocromaticidades": "hipocromaticidad",
    "hipocromias": "hipocromia",
    "hipocrática": "hipocrático",
    "hipocráticas": "hipocrático",
    "hipocráticos": "hipocrático",
    "hipocrénides": "hipocrénide",
    "hipocrómica": "hipocrómico",
    "hipocrómicas": "hipocrómico",
    "hipocrómicos": "hipocrómico",
    "hipocótilos": "hipocótilo",
    "hipodiales": "hipodial",
    "hipodoncias": "hipodoncia",
    "hipodérmica": "hipodérmico",
    "hipodérmicas": "hipodérmico",
    "hipodérmicos": "hipodérmico",
    "hipodérmises": "hipodermis",
    "hipoesplenismos": "hipoesplenismo",
    "hipoeutectoides": "hipoeutectoide",
    "hipoeutéctica": "hipoeutéctico",
    "hipoeutécticas": "hipoeutéctico",
    "hipoeutécticos": "hipoeutéctico",
    "hipofaringes": "hipofaringe",
    "hipofaringoscopias": "hipofaringoscopia",
    "hipofaringoscopios": "hipofaringoscopio",
    "hipofaríngea": "hipofaríngeo",
    "hipofaríngeas": "hipofaríngeo",
    "hipofaríngeos": "hipofaríngeo",
    "hipofibrinemias": "hipofibrinemia",
    "hipofibrinogenemias": "hipofibrinogenemia",
    "hipofisaria": "hipofisario",
    "hipofisarias": "hipofisario",
    "hipofisarios": "hipofisario",
    "hipoforias": "hipoforia",
    "hipofosfitos": "hipofosfito",
    "hipofosforosa": "hipofosforoso",
    "hipofosforosas": "hipofosforoso",
    "hipofosforosos": "hipofosforoso",
    "hipofunciones": "hipofunción",
    "hipogalactias": "hipogalactia",
    "hipogammaglobulinemias": "hipogammaglobulinemia",
    "hipogastrios": "hipogastrio",
    "hipogeos": "hipogeo",
    "hipoglosa": "hipogloso",
    "hipoglosas": "hipogloso",
    "hipoglosos": "hipogloso",
    "hipoglucemiantes": "hipoglucemiante",
    "hipoglucemias": "hipoglucemia",
    "hipoglucémica": "hipoglucémico",
    "hipoglucémicas": "hipoglucémico",
    "hipoglucémicos": "hipoglucémico",
    "hipogrifos": "hipogrifo",
    "hipogástrica": "hipogástrico",
    "hipogástricas": "hipogástrico",
    "hipogástricos": "hipogástrico",
    "hipogénica": "hipogénico",
    "hipogénicas": "hipogénico",
    "hipogénicos": "hipogénico",
    "hipohaploides": "hipohaploide",
    "hipohaploidias": "hipohaploidia",
    "hipoides": "hipoide",
    "hipolimniones": "hipolimnion",
    "hipologías": "hipología",
    "hipológica": "hipológico",
    "hipológicas": "hipológico",
    "hipológicos": "hipológico",
    "hipomanías": "hipomanía",
    "hiponatremias": "hiponatremia",
    "hiponimias": "hiponimia",
    "hiponitrosa": "hiponitroso",
    "hiponitrosas": "hiponitroso",
    "hiponitrosos": "hiponitroso",
    "hipoparatiroidismos": "hipoparatiroidismo",
    "hipopiones": "hipopión",
    "hipopituitarismos": "hipopituitarismo",
    "hipoplasias": "hipoplasia",
    "hipoplásica": "hipoplásico",
    "hipoplásicas": "hipoplásico",
    "hipoplásicos": "hipoplásico",
    "hipopotasemias": "hipopotasemia",
    "hipoproteinemias": "hipoproteinemia",
    "hipoproteínias": "hipoproteínia",
    "hipoprotrombinemias": "hipoprotrombinemia",
    "hipopótamos": "hipopótamo",
    "hipos": "hipo",
    "hiposa": "hiposo",
    "hiposas": "hiposo",
    "hiposmias": "hiposmia",
    "hiposos": "hiposo",
    "hipospádica": "hipospádico",
    "hipospádicas": "hipospádico",
    "hipospádicos": "hipospádico",
    "hipostasias": "hipostasia",
    "hipostática": "hipostático",
    "hipostáticas": "hipostático",
    "hipostáticos": "hipostático",
    "hiposulfúrica": "hiposulfúrico",
    "hiposulfúricas": "hiposulfúrico",
    "hiposulfúricos": "hiposulfúrico",
    "hipotalámica": "hipotalámico",
    "hipotalámicas": "hipotalámico",
    "hipotalámicos": "hipotalámico",
    "hipoteca": "hipotecar",
    "hipotecaba": "hipotecar",
    "hipotecabais": "hipotecar",
    "hipotecaban": "hipotecar",
    "hipotecabas": "hipotecar",
    "hipotecables": "hipotecable",
    "hipotecad": "hipotecar",
    "hipotecada": "hipotecar",
    "hipotecadas": "hipotecar",
    "hipotecado": "hipotecar",
    "hipotecados": "hipotecar",
    "hipotecamos": "hipotecar",
    "hipotecan": "hipotecar",
    "hipotecando": "hipotecar",
    "hipotecantes": "hipotecante",
    "hipotecara": "hipotecar",
    "hipotecarais": "hipotecar",
    "hipotecaran": "hipotecar",
    "hipotecaras": "hipotecar",
    "hipotecare": "hipotecar",
    "hipotecareis": "hipotecar",
    "hipotecaremos": "hipotecar",
    "hipotecaren": "hipotecar",
    "hipotecares": "hipotecar",
    "hipotecaria": "hipotecario",
    "hipotecarias": "hipotecario",
    "hipotecarios": "hipotecario",
    "hipotecaron": "hipotecar",
    "hipotecará": "hipotecar",
    "hipotecarán": "hipotecar",
    "hipotecarás": "hipotecar",
    "hipotecaré": "hipotecar",
    "hipotecaréis": "hipotecar",
    "hipotecaría": "hipotecar",
    "hipotecaríais": "hipotecar",
    "hipotecaríamos": "hipotecar",
    "hipotecarían": "hipotecar",
    "hipotecarías": "hipotecar",
    "hipotecas": "hipotecar",
    "hipotecase": "hipotecar",
    "hipotecaseis": "hipotecar",
    "hipotecasen": "hipotecar",
    "hipotecases": "hipotecar",
    "hipotecaste": "hipotecar",
    "hipotecasteis": "hipotecar",
    "hipotecnias": "hipotecnia",
    "hipoteco": "hipotecar",
    "hipotecábamos": "hipotecar",
    "hipotecáis": "hipotecar",
    "hipotecáramos": "hipotecar",
    "hipotecáremos": "hipotecar",
    "hipotecásemos": "hipotecar",
    "hipotecó": "hipotecar",
    "hipotenares": "hipotenar",
    "hipotensa": "hipotenso",
    "hipotensas": "hipotenso",
    "hipotensiones": "hipotensión",
    "hipotensos": "hipotenso",
    "hipotenusas": "hipotenusa",
    "hipoteque": "hipotecar",
    "hipotequemos": "hipotecar",
    "hipotequen": "hipotecar",
    "hipoteques": "hipotecar",
    "hipotequé": "hipotecar",
    "hipotequéis": "hipotecar",
    "hipotermias": "hipotermia",
    "hipotiroidea": "hipotiroideo",
    "hipotiroideas": "hipotiroideo",
    "hipotiroideos": "hipotiroideo",
    "hipotiroidismos": "hipotiroidismo",
    "hipotonías": "hipotonía",
    "hipotrocoides": "hipotrocoide",
    "hipotropías": "hipotropía",
    "hipotáctica": "hipotáctico",
    "hipotácticas": "hipotáctico",
    "hipotácticos": "hipotáctico",
    "hipotálamos": "hipotálamo",
    "hipotérmica": "hipotérmico",
    "hipotérmicas": "hipotérmico",
    "hipotérmicos": "hipotérmico",
    "hipotética": "hipotético",
    "hipotéticas": "hipotético",
    "hipotéticos": "hipotético",
    "hipotímpanos": "hipotímpano",
    "hipotónica": "hipotónico",
    "hipotónicas": "hipotónico",
    "hipotónicos": "hipotónico",
    "hipovalvas": "hipovalva",
    "hipovolemias": "hipovolemia",
    "hipoxantinas": "hipoxantina",
    "hipoxias": "hipoxia",
    "hippies": "hippy",
    "hipsobatimetrías": "hipsobatimetría",
    "hipsobatimétrica": "hipsobatimétrico",
    "hipsobatimétricas": "hipsobatimétrico",
    "hipsobatimétricos": "hipsobatimétrico",
    "hipsodonta": "hipsodonto",
    "hipsodontas": "hipsodonto",
    "hipsodontos": "hipsodonto",
    "hipsografías": "hipsografía",
    "hipsogramas": "hipsograma",
    "hipsográfica": "hipsográfico",
    "hipsográficas": "hipsográfico",
    "hipsográficos": "hipsográfico",
    "hipsometrías": "hipsometría",
    "hipsométrica": "hipsométrico",
    "hipsométricas": "hipsométrico",
    "hipsométricos": "hipsométrico",
    "hipsómetros": "hipsómetro",
    "hipábamos": "hipar",
    "hipáis": "hipar",
    "hipálages": "hipálage",
    "hipáramos": "hipar",
    "hipáremos": "hipar",
    "hipásemos": "hipar",
    "hipé": "hipar",
    "hipéis": "hipar",
    "hipérbatones": "hipérbaton",
    "hipérbatons": "hipérbaton",
    "hipérbatos": "hipérbato",
    "hipérbolas": "hipérbola",
    "hipérboles": "hipérbole",
    "hipéricos": "hipérico",
    "hipó": "hipar",
    "hipócritas": "hipócrita",
    "hipódromos": "hipódromo",
    "hipólogos": "hipólogo",
    "hipónimos": "hipónimo",
    "hipóstila": "hipóstilo",
    "hipóstilas": "hipóstilo",
    "hipóstilos": "hipóstilo",
    "hipúrica": "hipúrico",
    "hipúricas": "hipúrico",
    "hipúricos": "hipúrico",
    "hiramos": "herir",
    "hircocervos": "hircocervo",
    "hircos": "hirco",
    "hiriendo": "herir",
    "hirientes": "hiriente",
    "hiriera": "herir",
    "hirierais": "herir",
    "hirieran": "herir",
    "hirieras": "herir",
    "hiriere": "herir",
    "hiriereis": "herir",
    "hirieren": "herir",
    "hirieres": "herir",
    "hirieron": "herir",
    "hiriese": "herir",
    "hirieseis": "herir",
    "hiriesen": "herir",
    "hirieses": "herir",
    "hiriéramos": "herir",
    "hiriéremos": "herir",
    "hiriésemos": "herir",
    "hirió": "herir",
    "hirsuta": "hirsuto",
    "hirsutas": "hirsuto",
    "hirsutismos": "hirsutismo",
    "hirsutos": "hirsuto",
    "hirvamos": "hervir",
    "hirviendo": "hervir",
    "hirvientes": "hirviente",
    "hirviera": "hervir",
    "hirvierais": "hervir",
    "hirvieran": "hervir",
    "hirvieras": "hervir",
    "hirviere": "hervir",
    "hirviereis": "hervir",
    "hirvieren": "hervir",
    "hirvieres": "hervir",
    "hirvieron": "hervir",
    "hirviese": "hervir",
    "hirvieseis": "hervir",
    "hirviesen": "hervir",
    "hirvieses": "hervir",
    "hirviéramos": "hervir",
    "hirviéremos": "hervir",
    "hirviésemos": "hervir",
    "hirvió": "hervir",
    "hirváis": "hervir",
    "hiráis": "herir",
    "hiscales": "hiscal",
    "hiscas": "hisca",
    "hisopa": "hisopar",
    "hisopaba": "hisopar",
    "hisopabais": "hisopar",
    "hisopaban": "hisopar",
    "hisopabas": "hisopar",
    "hisopad": "hisopar",
    "hisopada": "hisopar",
    "hisopadas": "hisopar",
    "hisopado": "hisopar",
    "hisopados": "hisopar",
    "hisopamos": "hisopar",
    "hisopan": "hisopar",
    "hisopando": "hisopar",
    "hisopara": "hisopar",
    "hisoparais": "hisopar",
    "hisoparan": "hisopar",
    "hisoparas": "hisopar",
    "hisopare": "hisopar",
    "hisopareis": "hisopar",
    "hisoparemos": "hisopar",
    "hisoparen": "hisopar",
    "hisopares": "hisopar",
    "hisoparon": "hisopar",
    "hisopará": "hisopar",
    "hisoparán": "hisopar",
    "hisoparás": "hisopar",
    "hisoparé": "hisopar",
    "hisoparéis": "hisopar",
    "hisoparía": "hisopar",
    "hisoparíais": "hisopar",
    "hisoparíamos": "hisopar",
    "hisoparían": "hisopar",
    "hisoparías": "hisopar",
    "hisopas": "hisopar",
    "hisopase": "hisopar",
    "hisopaseis": "hisopar",
    "hisopasen": "hisopar",
    "hisopases": "hisopar",
    "hisopaste": "hisopar",
    "hisopasteis": "hisopar",
    "hisopazos": "hisopazo",
    "hisope": "hisopar",
    "hisopea": "hisopear",
    "hisopeaba": "hisopear",
    "hisopeabais": "hisopear",
    "hisopeaban": "hisopear",
    "hisopeabas": "hisopear",
    "hisopead": "hisopear",
    "hisopeada": "hisopear",
    "hisopeadas": "hisopear",
    "hisopeado": "hisopear",
    "hisopeados": "hisopear",
    "hisopeamos": "hisopear",
    "hisopean": "hisopear",
    "hisopeando": "hisopear",
    "hisopeara": "hisopear",
    "hisopearais": "hisopear",
    "hisopearan": "hisopear",
    "hisopearas": "hisopear",
    "hisopeare": "hisopear",
    "hisopeareis": "hisopear",
    "hisopearemos": "hisopear",
    "hisopearen": "hisopear",
    "hisopeares": "hisopear",
    "hisopearon": "hisopear",
    "hisopeará": "hisopear",
    "hisopearán": "hisopear",
    "hisopearás": "hisopear",
    "hisopearé": "hisopear",
    "hisopearéis": "hisopear",
    "hisopearía": "hisopear",
    "hisopearíais": "hisopear",
    "hisopearíamos": "hisopear",
    "hisopearían": "hisopear",
    "hisopearías": "hisopear",
    "hisopeas": "hisopear",
    "hisopease": "hisopear",
    "hisopeaseis": "hisopear",
    "hisopeasen": "hisopear",
    "hisopeases": "hisopear",
    "hisopeaste": "hisopear",
    "hisopeasteis": "hisopear",
    "hisopee": "hisopear",
    "hisopeemos": "hisopear",
    "hisopeen": "hisopear",
    "hisopees": "hisopear",
    "hisopemos": "hisopar",
    "hisopen": "hisopar",
    "hisopeo": "hisopear",
    "hisopes": "hisopar",
    "hisopeábamos": "hisopear",
    "hisopeáis": "hisopear",
    "hisopeáramos": "hisopear",
    "hisopeáremos": "hisopear",
    "hisopeásemos": "hisopear",
    "hisopeé": "hisopear",
    "hisopeéis": "hisopear",
    "hisopeó": "hisopear",
    "hisopillos": "hisopillo",
    "hisopo": "hisopar",
    "hisopos": "hisopo",
    "hisopábamos": "hisopar",
    "hisopáis": "hisopar",
    "hisopáramos": "hisopar",
    "hisopáremos": "hisopar",
    "hisopásemos": "hisopar",
    "hisopé": "hisopar",
    "hisopéis": "hisopar",
    "hisopó": "hisopar",
    "hispalenses": "hispalense",
    "hispana": "hispano",
    "hispanas": "hispano",
    "hispanice": "hispanizar",
    "hispanicemos": "hispanizar",
    "hispanicen": "hispanizar",
    "hispanices": "hispanizar",
    "hispanicé": "hispanizar",
    "hispanicéis": "hispanizar",
    "hispanidades": "hispanidad",
    "hispanismos": "hispanismo",
    "hispanistas": "hispanista",
    "hispaniza": "hispanizar",
    "hispanizaba": "hispanizar",
    "hispanizabais": "hispanizar",
    "hispanizaban": "hispanizar",
    "hispanizabas": "hispanizar",
    "hispanizad": "hispanizar",
    "hispanizada": "hispanizar",
    "hispanizadas": "hispanizar",
    "hispanizado": "hispanizar",
    "hispanizados": "hispanizar",
    "hispanizamos": "hispanizar",
    "hispanizan": "hispanizar",
    "hispanizando": "hispanizar",
    "hispanizara": "hispanizar",
    "hispanizarais": "hispanizar",
    "hispanizaran": "hispanizar",
    "hispanizaras": "hispanizar",
    "hispanizare": "hispanizar",
    "hispanizareis": "hispanizar",
    "hispanizaremos": "hispanizar",
    "hispanizaren": "hispanizar",
    "hispanizares": "hispanizar",
    "hispanizaron": "hispanizar",
    "hispanizará": "hispanizar",
    "hispanizarán": "hispanizar",
    "hispanizarás": "hispanizar",
    "hispanizaré": "hispanizar",
    "hispanizaréis": "hispanizar",
    "hispanizaría": "hispanizar",
    "hispanizaríais": "hispanizar",
    "hispanizaríamos": "hispanizar",
    "hispanizarían": "hispanizar",
    "hispanizarías": "hispanizar",
    "hispanizas": "hispanizar",
    "hispanizase": "hispanizar",
    "hispanizaseis": "hispanizar",
    "hispanizasen": "hispanizar",
    "hispanizases": "hispanizar",
    "hispanizaste": "hispanizar",
    "hispanizasteis": "hispanizar",
    "hispanizo": "hispanizar",
    "hispanizábamos": "hispanizar",
    "hispanizáis": "hispanizar",
    "hispanizáramos": "hispanizar",
    "hispanizáremos": "hispanizar",
    "hispanizásemos": "hispanizar",
    "hispanizó": "hispanizar",
    "hispano-polaca": "hispano-polaco",
    "hispano-polacas": "hispano-polaco",
    "hispano-polacos": "hispano-polaco",
    "hispanoamericana": "hispanoamericano",
    "hispanoamericanas": "hispanoamericano",
    "hispanoamericanos": "hispanoamericano",
    "hispanoargentina": "hispanoargentino",
    "hispanoargentinas": "hispanoargentino",
    "hispanoargentinos": "hispanoargentino",
    "hispanocolombiana": "hispanocolombiano",
    "hispanocolombianas": "hispanocolombiano",
    "hispanocolombianos": "hispanocolombiano",
    "hispanocoloniales": "hispanocolonial",
    "hispanohablantes": "hispanohablante",
    "hispanomusulmana": "hispanomusulmán",
    "hispanomusulmanas": "hispanomusulmán",
    "hispanomusulmanes": "hispanomusulmán",
    "hispanorromana": "hispanorromano",
    "hispanorromanas": "hispanorromano",
    "hispanorromanos": "hispanorromano",
    "hispanos": "hispano",
    "hispanoárabes": "hispanoárabe",
    "hispanófila": "hispanófilo",
    "hispanófilas": "hispanófilo",
    "hispanófilos": "hispanófilo",
    "hispánica": "hispánico",
    "hispánicas": "hispánico",
    "hispánicos": "hispánico",
    "histaminas": "histamina",
    "histaminasas": "histaminasa",
    "histamínica": "histamínico",
    "histamínicas": "histamínico",
    "histamínicos": "histamínico",
    "histerectomías": "histerectomía",
    "histerestomías": "histerestomía",
    "histeresígrafos": "histeresígrafo",
    "histeresímetros": "histeresímetro",
    "histerias": "histeria",
    "histerica": "histérico",
    "histericas": "histérico",
    "histericos": "histérico",
    "histerismos": "histerismo",
    "histerografías": "histerografía",
    "histerologías": "histerología",
    "histerética": "histerético",
    "histeréticas": "histerético",
    "histeréticos": "histerético",
    "histidinas": "histidina",
    "histioblastos": "histioblasto",
    "histiocitos": "histiocito",
    "histocompatibilidades": "histocompatibilidad",
    "histocompatibles": "histocompatible",
    "histogramas": "histograma",
    "histoincompatibilidades": "histoincompatibilidad",
    "histologías": "histología",
    "histológica": "histológico",
    "histológicas": "histológico",
    "histológicos": "histológico",
    "histonas": "histona",
    "histopatologías": "histopatología",
    "histoquímicas": "histoquímica",
    "historiaba": "historiar",
    "historiabais": "historiar",
    "historiaban": "historiar",
    "historiabas": "historiar",
    "historiad": "historiar",
    "historiada": "historiar",
    "historiadas": "historiar",
    "historiado": "historiar",
    "historiadora": "historiador",
    "historiadoras": "historiador",
    "historiadores": "historiador",
    "historiados": "historiar",
    "historiales": "historial",
    "historiamos": "historiar",
    "historiando": "historiar",
    "historiara": "historiar",
    "historiarais": "historiar",
    "historiaran": "historiar",
    "historiaras": "historiar",
    "historiare": "historiar",
    "historiareis": "historiar",
    "historiaremos": "historiar",
    "historiaren": "historiar",
    "historiares": "historiar",
    "historiaron": "historiar",
    "historiará": "historiar",
    "historiarán": "historiar",
    "historiarás": "historiar",
    "historiaré": "historiar",
    "historiaréis": "historiar",
    "historiaría": "historiar",
    "historiaríais": "historiar",
    "historiaríamos": "historiar",
    "historiarían": "historiar",
    "historiarías": "historiar",
    "historias": "historia",
    "historiase": "historiar",
    "historiaseis": "historiar",
    "historiasen": "historiar",
    "historiases": "historiar",
    "historiaste": "historiar",
    "historiasteis": "historiar",
    "historicidades": "historicidad",
    "historicismos": "historicismo",
    "historicistas": "historicista",
    "historicopenales": "historicopenal",
    "historiemos": "historiar",
    "historietas": "historieta",
    "historiografías": "historiografía",
    "historiográfica": "historiográfico",
    "historiográficas": "historiográfico",
    "historiográficos": "historiográfico",
    "historiologías": "historiología",
    "historismos": "historismo",
    "historiábamos": "historiar",
    "historiáis": "historiar",
    "historiáramos": "historiar",
    "historiáremos": "historiar",
    "historiásemos": "historiar",
    "historié": "historiar",
    "historiéis": "historiar",
    "historió": "historiar",
    "historiógrafa": "historiógrafo",
    "historiógrafas": "historiógrafo",
    "historiógrafos": "historiógrafo",
    "historía": "historiar",
    "historían": "historiar",
    "historías": "historiar",
    "historíe": "historiar",
    "historíen": "historiar",
    "historíes": "historiar",
    "historío": "historiar",
    "histotopos": "histotopo",
    "histriones": "histrión",
    "histrionisas": "histrionisa",
    "histrionismos": "histrionismo",
    "histriónica": "histriónico",
    "histriónicas": "histriónico",
    "histriónicos": "histriónico",
    "histrícidos": "histrícido",
    "histérica": "histérico",
    "histéricas": "histérico",
    "histéricos": "histérico",
    "históloga": "histólogo",
    "histólogas": "histólogo",
    "histólogos": "histólogo",
    "histórica": "histórico",
    "históricas": "histórico",
    "históricos": "histórico",
    "hita": "hito",
    "hitaba": "hitar",
    "hitabais": "hitar",
    "hitaban": "hitar",
    "hitabas": "hitar",
    "hitaciones": "hitación",
    "hitad": "hitar",
    "hitada": "hitar",
    "hitadas": "hitar",
    "hitado": "hitar",
    "hitados": "hitar",
    "hitamos": "hitar",
    "hitan": "hitar",
    "hitando": "hitar",
    "hitara": "hitar",
    "hitarais": "hitar",
    "hitaran": "hitar",
    "hitaras": "hitar",
    "hitare": "hitar",
    "hitareis": "hitar",
    "hitaremos": "hitar",
    "hitaren": "hitar",
    "hitares": "hitar",
    "hitaron": "hitar",
    "hitará": "hitar",
    "hitarán": "hitar",
    "hitarás": "hitar",
    "hitaré": "hitar",
    "hitaréis": "hitar",
    "hitaría": "hitar",
    "hitaríais": "hitar",
    "hitaríamos": "hitar",
    "hitarían": "hitar",
    "hitarías": "hitar",
    "hitas": "hito",
    "hitase": "hitar",
    "hitaseis": "hitar",
    "hitasen": "hitar",
    "hitases": "hitar",
    "hitaste": "hitar",
    "hitasteis": "hitar",
    "hite": "hitar",
    "hitemos": "hitar",
    "hiten": "hitar",
    "hites": "hitar",
    "hititas": "hitita",
    "hitleriana": "hitleriano",
    "hitlerianas": "hitleriano",
    "hitlerianos": "hitleriano",
    "hito": "hitar",
    "hitones": "hitón",
    "hitos": "hito",
    "hits": "hit",
    "hitábamos": "hitar",
    "hitáis": "hitar",
    "hitáramos": "hitar",
    "hitáremos": "hitar",
    "hitásemos": "hitar",
    "hité": "hitar",
    "hitéis": "hitar",
    "hitó": "hitar",
    "hizo": "hacer",
    "hiña": "heñir",
    "hiñamos": "heñir",
    "hiñan": "heñir",
    "hiñas": "heñir",
    "hiñe": "heñir",
    "hiñen": "heñir",
    "hiñendo": "heñir",
    "hiñera": "heñir",
    "hiñerais": "heñir",
    "hiñeran": "heñir",
    "hiñeras": "heñir",
    "hiñere": "heñir",
    "hiñereis": "heñir",
    "hiñeren": "heñir",
    "hiñeres": "heñir",
    "hiñeron": "heñir",
    "hiñes": "heñir",
    "hiñese": "heñir",
    "hiñeseis": "heñir",
    "hiñesen": "heñir",
    "hiñeses": "heñir",
    "hiño": "heñir",
    "hiñáis": "heñir",
    "hiñéramos": "heñir",
    "hiñéremos": "heñir",
    "hiñésemos": "heñir",
    "hiñó": "heñir",
    "hl": "hectolitro",
    "hm": "hectómetro",
    "hoacines": "hoacín",
    "hoazines": "hoazín",
    "hobbies": "hobby",
    "hobbys": "hobby",
    "hoce": "hozar",
    "hocemos": "hozar",
    "hocen": "hozar",
    "hoces": "hozar",
    "hocica": "hocicar",
    "hocicaba": "hocicar",
    "hocicabais": "hocicar",
    "hocicaban": "hocicar",
    "hocicabas": "hocicar",
    "hocicad": "hocicar",
    "hocicada": "hocicar",
    "hocicadas": "hocicar",
    "hocicado": "hocicar",
    "hocicados": "hocicar",
    "hocicamos": "hocicar",
    "hocican": "hocicar",
    "hocicando": "hocicar",
    "hocicara": "hocicar",
    "hocicarais": "hocicar",
    "hocicaran": "hocicar",
    "hocicaras": "hocicar",
    "hocicare": "hocicar",
    "hocicareis": "hocicar",
    "hocicaremos": "hocicar",
    "hocicaren": "hocicar",
    "hocicares": "hocicar",
    "hocicaron": "hocicar",
    "hocicará": "hocicar",
    "hocicarán": "hocicar",
    "hocicarás": "hocicar",
    "hocicaré": "hocicar",
    "hocicaréis": "hocicar",
    "hocicaría": "hocicar",
    "hocicaríais": "hocicar",
    "hocicaríamos": "hocicar",
    "hocicarían": "hocicar",
    "hocicarías": "hocicar",
    "hocicas": "hocicar",
    "hocicase": "hocicar",
    "hocicaseis": "hocicar",
    "hocicasen": "hocicar",
    "hocicases": "hocicar",
    "hocicaste": "hocicar",
    "hocicasteis": "hocicar",
    "hocico": "hocicar",
    "hocicos": "hocico",
    "hocicuda": "hocicudo",
    "hocicudas": "hocicudo",
    "hocicudos": "hocicudo",
    "hocicábamos": "hocicar",
    "hocicáis": "hocicar",
    "hocicáramos": "hocicar",
    "hocicáremos": "hocicar",
    "hocicásemos": "hocicar",
    "hocicó": "hocicar",
    "hocinos": "hocino",
    "hocique": "hocicar",
    "hociquemos": "hocicar",
    "hociquen": "hocicar",
    "hociques": "hocicar",
    "hociqué": "hocicar",
    "hociquéis": "hocicar",
    "hockeístas": "hockeísta",
    "hocé": "hozar",
    "hocéis": "hozar",
    "hodierna": "hodierno",
    "hodiernas": "hodierno",
    "hodiernos": "hodierno",
    "hodrológica": "hodrológico",
    "hodrológicas": "hodrológico",
    "hodrológicos": "hodrológico",
    "hodógrafas": "hodógrafa",
    "hogares": "hogar",
    "hogareña": "hogareño",
    "hogareñas": "hogareño",
    "hogareños": "hogareño",
    "hogazas": "hogaza",
    "hogbacks": "hogback",
    "hogueras": "hoguera",
    "hojalatas": "hojalata",
    "hojalatera": "hojalatero",
    "hojalateras": "hojalatero",
    "hojalateros": "hojalatero",
    "hojalaterías": "hojalatería",
    "hojaldes": "hojalde",
    "hojaldra": "hojaldrar",
    "hojaldraba": "hojaldrar",
    "hojaldrabais": "hojaldrar",
    "hojaldraban": "hojaldrar",
    "hojaldrabas": "hojaldrar",
    "hojaldrad": "hojaldrar",
    "hojaldrada": "hojaldrar",
    "hojaldradas": "hojaldrar",
    "hojaldrado": "hojaldrar",
    "hojaldrados": "hojaldrar",
    "hojaldramos": "hojaldrar",
    "hojaldran": "hojaldrar",
    "hojaldrando": "hojaldrar",
    "hojaldrara": "hojaldrar",
    "hojaldrarais": "hojaldrar",
    "hojaldraran": "hojaldrar",
    "hojaldraras": "hojaldrar",
    "hojaldrare": "hojaldrar",
    "hojaldrareis": "hojaldrar",
    "hojaldraremos": "hojaldrar",
    "hojaldraren": "hojaldrar",
    "hojaldrares": "hojaldrar",
    "hojaldraron": "hojaldrar",
    "hojaldrará": "hojaldrar",
    "hojaldrarán": "hojaldrar",
    "hojaldrarás": "hojaldrar",
    "hojaldraré": "hojaldrar",
    "hojaldraréis": "hojaldrar",
    "hojaldraría": "hojaldrar",
    "hojaldraríais": "hojaldrar",
    "hojaldraríamos": "hojaldrar",
    "hojaldrarían": "hojaldrar",
    "hojaldrarías": "hojaldrar",
    "hojaldras": "hojaldrar",
    "hojaldrase": "hojaldrar",
    "hojaldraseis": "hojaldrar",
    "hojaldrasen": "hojaldrar",
    "hojaldrases": "hojaldrar",
    "hojaldraste": "hojaldrar",
    "hojaldrasteis": "hojaldrar",
    "hojaldre": "hojaldrar",
    "hojaldremos": "hojaldrar",
    "hojaldren": "hojaldrar",
    "hojaldrera": "hojaldrero",
    "hojaldreras": "hojaldrero",
    "hojaldreros": "hojaldrero",
    "hojaldres": "hojaldre",
    "hojaldristas": "hojaldrista",
    "hojaldro": "hojaldrar",
    "hojaldrábamos": "hojaldrar",
    "hojaldráis": "hojaldrar",
    "hojaldráramos": "hojaldrar",
    "hojaldráremos": "hojaldrar",
    "hojaldrásemos": "hojaldrar",
    "hojaldré": "hojaldrar",
    "hojaldréis": "hojaldrar",
    "hojaldró": "hojaldrar",
    "hojarascas": "hojarasca",
    "hojas": "hoja",
    "hojea": "hojear",
    "hojeaba": "hojear",
    "hojeabais": "hojear",
    "hojeaban": "hojear",
    "hojeabas": "hojear",
    "hojead": "hojear",
    "hojeada": "hojear",
    "hojeadas": "hojear",
    "hojeado": "hojear",
    "hojeados": "hojear",
    "hojeamos": "hojear",
    "hojean": "hojear",
    "hojeando": "hojear",
    "hojeara": "hojear",
    "hojearais": "hojear",
    "hojearan": "hojear",
    "hojearas": "hojear",
    "hojeare": "hojear",
    "hojeareis": "hojear",
    "hojearemos": "hojear",
    "hojearen": "hojear",
    "hojeares": "hojear",
    "hojearon": "hojear",
    "hojeará": "hojear",
    "hojearán": "hojear",
    "hojearás": "hojear",
    "hojearé": "hojear",
    "hojearéis": "hojear",
    "hojearía": "hojear",
    "hojearíais": "hojear",
    "hojearíamos": "hojear",
    "hojearían": "hojear",
    "hojearías": "hojear",
    "hojeas": "hojear",
    "hojease": "hojear",
    "hojeaseis": "hojear",
    "hojeasen": "hojear",
    "hojeases": "hojear",
    "hojeaste": "hojear",
    "hojeasteis": "hojear",
    "hojee": "hojear",
    "hojeemos": "hojear",
    "hojeen": "hojear",
    "hojees": "hojear",
    "hojeo": "hojear",
    "hojeábamos": "hojear",
    "hojeáis": "hojear",
    "hojeáramos": "hojear",
    "hojeáremos": "hojear",
    "hojeásemos": "hojear",
    "hojeé": "hojear",
    "hojeéis": "hojear",
    "hojeó": "hojear",
    "hojosa": "hojoso",
    "hojosas": "hojoso",
    "hojosos": "hojoso",
    "hojuda": "hojudo",
    "hojudas": "hojudo",
    "hojudos": "hojudo",
    "hojuelas": "hojuela",
    "hokas": "hoka",
    "holandesa": "holandés",
    "holandesas": "holandés",
    "holandeses": "holandés",
    "holandetas": "holandeta",
    "holandillas": "holandilla",
    "holanes": "holán",
    "holdings": "holding",
    "holgaba": "holgar",
    "holgabais": "holgar",
    "holgaban": "holgar",
    "holgabas": "holgar",
    "holgad": "holgar",
    "holgada": "holgar",
    "holgadas": "holgar",
    "holgado": "holgar",
    "holgados": "holgar",
    "holgamos": "holgar",
    "holgando": "holgar",
    "holganzas": "holganza",
    "holgara": "holgar",
    "holgarais": "holgar",
    "holgaran": "holgar",
    "holgaras": "holgar",
    "holgare": "holgar",
    "holgareis": "holgar",
    "holgaremos": "holgar",
    "holgaren": "holgar",
    "holgares": "holgar",
    "holgaron": "holgar",
    "holgará": "holgar",
    "holgarán": "holgar",
    "holgarás": "holgar",
    "holgaré": "holgar",
    "holgaréis": "holgar",
    "holgaría": "holgar",
    "holgaríais": "holgar",
    "holgaríamos": "holgar",
    "holgarían": "holgar",
    "holgarías": "holgar",
    "holgase": "holgar",
    "holgaseis": "holgar",
    "holgasen": "holgar",
    "holgases": "holgar",
    "holgaste": "holgar",
    "holgasteis": "holgar",
    "holgazana": "holgazán",
    "holgazanas": "holgazán",
    "holgazanea": "holgazanear",
    "holgazaneaba": "holgazanear",
    "holgazaneabais": "holgazanear",
    "holgazaneaban": "holgazanear",
    "holgazaneabas": "holgazanear",
    "holgazanead": "holgazanear",
    "holgazaneada": "holgazanear",
    "holgazaneadas": "holgazanear",
    "holgazaneado": "holgazanear",
    "holgazaneados": "holgazanear",
    "holgazaneamos": "holgazanear",
    "holgazanean": "holgazanear",
    "holgazaneando": "holgazanear",
    "holgazaneara": "holgazanear",
    "holgazanearais": "holgazanear",
    "holgazanearan": "holgazanear",
    "holgazanearas": "holgazanear",
    "holgazaneare": "holgazanear",
    "holgazaneareis": "holgazanear",
    "holgazanearemos": "holgazanear",
    "holgazanearen": "holgazanear",
    "holgazaneares": "holgazanear",
    "holgazanearon": "holgazanear",
    "holgazaneará": "holgazanear",
    "holgazanearán": "holgazanear",
    "holgazanearás": "holgazanear",
    "holgazanearé": "holgazanear",
    "holgazanearéis": "holgazanear",
    "holgazanearía": "holgazanear",
    "holgazanearíais": "holgazanear",
    "holgazanearíamos": "holgazanear",
    "holgazanearían": "holgazanear",
    "holgazanearías": "holgazanear",
    "holgazaneas": "holgazanear",
    "holgazanease": "holgazanear",
    "holgazaneaseis": "holgazanear",
    "holgazaneasen": "holgazanear",
    "holgazaneases": "holgazanear",
    "holgazaneaste": "holgazanear",
    "holgazaneasteis": "holgazanear",
    "holgazanee": "holgazanear",
    "holgazaneemos": "holgazanear",
    "holgazaneen": "holgazanear",
    "holgazanees": "holgazanear",
    "holgazaneo": "holgazanear",
    "holgazanerías": "holgazanería",
    "holgazanes": "holgazán",
    "holgazaneábamos": "holgazanear",
    "holgazaneáis": "holgazanear",
    "holgazaneáramos": "holgazanear",
    "holgazaneáremos": "holgazanear",
    "holgazaneásemos": "holgazanear",
    "holgazaneé": "holgazanear",
    "holgazaneéis": "holgazanear",
    "holgazaneó": "holgazanear",
    "holgazanos": "holgazán",
    "holgona": "holgón",
    "holgonas": "holgón",
    "holgones": "holgón",
    "holgorios": "holgorio",
    "holguemos": "holgar",
    "holguras": "holgura",
    "holgué": "holgar",
    "holguéis": "holgar",
    "holgábamos": "holgar",
    "holgáis": "holgar",
    "holgáramos": "holgar",
    "holgáremos": "holgar",
    "holgásemos": "holgar",
    "holgó": "holgar",
    "holismos": "holismo",
    "hollaba": "hollar",
    "hollabais": "hollar",
    "hollaban": "hollar",
    "hollabas": "hollar",
    "hollad": "hollar",
    "hollada": "hollar",
    "holladas": "hollar",
    "holladera": "holladero",
    "holladeras": "holladero",
    "holladeros": "holladero",
    "hollado": "hollar",
    "hollados": "hollar",
    "holladuras": "holladura",
    "hollamos": "hollar",
    "hollando": "hollar",
    "hollara": "hollar",
    "hollarais": "hollar",
    "hollaran": "hollar",
    "hollaras": "hollar",
    "hollare": "hollar",
    "hollareis": "hollar",
    "hollaremos": "hollar",
    "hollaren": "hollar",
    "hollares": "hollar",
    "hollaron": "hollar",
    "hollará": "hollar",
    "hollarán": "hollar",
    "hollarás": "hollar",
    "hollaré": "hollar",
    "hollaréis": "hollar",
    "hollaría": "hollar",
    "hollaríais": "hollar",
    "hollaríamos": "hollar",
    "hollarían": "hollar",
    "hollarías": "hollar",
    "hollase": "hollar",
    "hollaseis": "hollar",
    "hollasen": "hollar",
    "hollases": "hollar",
    "hollaste": "hollar",
    "hollasteis": "hollar",
    "hollejos": "hollejo",
    "hollejuda": "hollejudo",
    "hollejudas": "hollejudo",
    "hollejudos": "hollejudo",
    "hollemos": "hollar",
    "hollines": "hollín",
    "hollinienta": "holliniento",
    "hollinientas": "holliniento",
    "hollinientos": "holliniento",
    "hollinosa": "hollinoso",
    "hollinosas": "hollinoso",
    "hollinosos": "hollinoso",
    "hollábamos": "hollar",
    "holláis": "hollar",
    "holláramos": "hollar",
    "holláremos": "hollar",
    "hollásemos": "hollar",
    "hollé": "hollar",
    "holléis": "hollar",
    "holló": "hollar",
    "holmios": "holmio",
    "holobiontes": "holobionte",
    "holoblástica": "holoblástico",
    "holoblásticas": "holoblástico",
    "holoblásticos": "holoblástico",
    "holocaustos": "holocausto",
    "holocenos": "holoceno",
    "holocrinos": "holocrino",
    "holocristalina": "holocristalino",
    "holocristalinas": "holocristalino",
    "holocristalinos": "holocristalino",
    "holodonta": "holodonto",
    "holodontas": "holodonto",
    "holodontos": "holodonto",
    "holoedros": "holoedro",
    "holoedrías": "holoedría",
    "hologamias": "hologamia",
    "holografías": "holografía",
    "hologramas": "holograma",
    "holográfica": "holográfico",
    "holográficas": "holográfico",
    "holográficos": "holográfico",
    "holometabolias": "holometabolia",
    "holometábola": "holometábolo",
    "holometábolas": "holometábolo",
    "holometábolos": "holometábolo",
    "holomorfa": "holomorfo",
    "holomorfas": "holomorfo",
    "holomorfos": "holomorfo",
    "holoparasitismos": "holoparasitismo",
    "holoparásitos": "holoparásito",
    "holotipos": "holotipo",
    "holoturias": "holoturia",
    "holoturioideos": "holoturioideo",
    "holoédrica": "holoédrico",
    "holoédricas": "holoédrico",
    "holoédricos": "holoédrico",
    "holártica": "holártico",
    "holárticas": "holártico",
    "holárticos": "holártico",
    "holística": "holístico",
    "holísticas": "holístico",
    "holísticos": "holístico",
    "hológama": "hológamo",
    "hológamas": "hológamo",
    "hológamos": "hológamo",
    "hológrafos": "hológrafo",
    "holómetros": "holómetro",
    "homatropinas": "homatropina",
    "hombrachos": "hombracho",
    "hombradas": "hombrada",
    "hombradías": "hombradía",
    "hombrea": "hombrear",
    "hombreaba": "hombrear",
    "hombreabais": "hombrear",
    "hombreaban": "hombrear",
    "hombreabas": "hombrear",
    "hombread": "hombrear",
    "hombreada": "hombrear",
    "hombreadas": "hombrear",
    "hombreado": "hombrear",
    "hombreados": "hombrear",
    "hombreamos": "hombrear",
    "hombrean": "hombrear",
    "hombreando": "hombrear",
    "hombreara": "hombrear",
    "hombrearais": "hombrear",
    "hombrearan": "hombrear",
    "hombrearas": "hombrear",
    "hombreare": "hombrear",
    "hombreareis": "hombrear",
    "hombrearemos": "hombrear",
    "hombrearen": "hombrear",
    "hombreares": "hombrear",
    "hombrearon": "hombrear",
    "hombreará": "hombrear",
    "hombrearán": "hombrear",
    "hombrearás": "hombrear",
    "hombrearé": "hombrear",
    "hombrearéis": "hombrear",
    "hombrearía": "hombrear",
    "hombrearíais": "hombrear",
    "hombrearíamos": "hombrear",
    "hombrearían": "hombrear",
    "hombrearías": "hombrear",
    "hombreas": "hombrear",
    "hombrease": "hombrear",
    "hombreaseis": "hombrear",
    "hombreasen": "hombrear",
    "hombreases": "hombrear",
    "hombreaste": "hombrear",
    "hombreasteis": "hombrear",
    "hombrecillos": "hombrecillo",
    "hombree": "hombrear",
    "hombreemos": "hombrear",
    "hombreen": "hombrear",
    "hombrees": "hombrear",
    "hombreo": "hombrear",
    "hombreras": "hombrera",
    "hombres": "hombre",
    "hombreábamos": "hombrear",
    "hombreáis": "hombrear",
    "hombreáramos": "hombrear",
    "hombreáremos": "hombrear",
    "hombreásemos": "hombrear",
    "hombreé": "hombrear",
    "hombreéis": "hombrear",
    "hombreó": "hombrear",
    "hombrillos": "hombrillo",
    "hombros": "hombro",
    "hombruna": "hombruno",
    "hombrunas": "hombruno",
    "hombrunos": "hombruno",
    "hombrías": "hombría",
    "homenajea": "homenajear",
    "homenajeaba": "homenajear",
    "homenajeabais": "homenajear",
    "homenajeaban": "homenajear",
    "homenajeabas": "homenajear",
    "homenajead": "homenajear",
    "homenajeada": "homenajear",
    "homenajeadas": "homenajear",
    "homenajeado": "homenajear",
    "homenajeados": "homenajear",
    "homenajeamos": "homenajear",
    "homenajean": "homenajear",
    "homenajeando": "homenajear",
    "homenajeara": "homenajear",
    "homenajearais": "homenajear",
    "homenajearan": "homenajear",
    "homenajearas": "homenajear",
    "homenajeare": "homenajear",
    "homenajeareis": "homenajear",
    "homenajearemos": "homenajear",
    "homenajearen": "homenajear",
    "homenajeares": "homenajear",
    "homenajearon": "homenajear",
    "homenajeará": "homenajear",
    "homenajearán": "homenajear",
    "homenajearás": "homenajear",
    "homenajearé": "homenajear",
    "homenajearéis": "homenajear",
    "homenajearía": "homenajear",
    "homenajearíais": "homenajear",
    "homenajearíamos": "homenajear",
    "homenajearían": "homenajear",
    "homenajearías": "homenajear",
    "homenajeas": "homenajear",
    "homenajease": "homenajear",
    "homenajeaseis": "homenajear",
    "homenajeasen": "homenajear",
    "homenajeases": "homenajear",
    "homenajeaste": "homenajear",
    "homenajeasteis": "homenajear",
    "homenajee": "homenajear",
    "homenajeemos": "homenajear",
    "homenajeen": "homenajear",
    "homenajees": "homenajear",
    "homenajeo": "homenajear",
    "homenajes": "homenaje",
    "homenajeábamos": "homenajear",
    "homenajeáis": "homenajear",
    "homenajeáramos": "homenajear",
    "homenajeáremos": "homenajear",
    "homenajeásemos": "homenajear",
    "homenajeé": "homenajear",
    "homenajeéis": "homenajear",
    "homenajeó": "homenajear",
    "homentrópica": "homentrópico",
    "homentrópicas": "homentrópico",
    "homentrópicos": "homentrópico",
    "homeomiasmática": "homeomiasmático",
    "homeomiasmáticas": "homeomiasmático",
    "homeomiasmáticos": "homeomiasmático",
    "homeomorfismos": "homeomorfismo",
    "homeopaticidades": "homeopaticidad",
    "homeopatías": "homeopatía",
    "homeopolares": "homeopolar",
    "homeopsórica": "homeopsórico",
    "homeopsóricas": "homeopsórico",
    "homeopsóricos": "homeopsórico",
    "homeopática": "homeopático",
    "homeopáticas": "homeopático",
    "homeopáticos": "homeopático",
    "homeostasias": "homeostasia",
    "homeostática": "homeostático",
    "homeostáticas": "homeostático",
    "homeostáticos": "homeostático",
    "homeoterma": "homeotermo",
    "homeotermas": "homeotermo",
    "homeotermos": "homeotermo",
    "homeópatas": "homeópata",
    "homeótica": "homeótico",
    "homeóticas": "homeótico",
    "homeóticos": "homeótico",
    "homicidas": "homicida",
    "homicidios": "homicidio",
    "homicídica": "homicídico",
    "homicídicas": "homicídico",
    "homicídicos": "homicídico",
    "homiliarios": "homiliario",
    "homilías": "homilía",
    "hominales": "hominal",
    "hominizaciones": "hominización",
    "homocentros": "homocentro",
    "homocercas": "homocerca",
    "homocigotos": "homocigoto",
    "homocinética": "homocinético",
    "homocinéticas": "homocinético",
    "homocinéticos": "homocinético",
    "homocisteinas": "homocisteina",
    "homocitotrópica": "homocitotrópico",
    "homocitotrópicas": "homocitotrópico",
    "homocitotrópicos": "homocitotrópico",
    "homoclimas": "homoclima",
    "homocromías": "homocromía",
    "homocrómica": "homocrómico",
    "homocrómicas": "homocrómico",
    "homocrómicos": "homocrómico",
    "homocéntrica": "homocéntrico",
    "homocéntricas": "homocéntrico",
    "homocéntricos": "homocéntrico",
    "homocíclica": "homocíclico",
    "homocíclicas": "homocíclico",
    "homocíclicos": "homocíclico",
    "homodonta": "homodonto",
    "homodontas": "homodonto",
    "homodontos": "homodonto",
    "homoestructuras": "homoestructura",
    "homofibras": "homofibra",
    "homofobias": "homofobia",
    "homofocales": "homofocal",
    "homofonías": "homofonía",
    "homogamética": "homogamético",
    "homogaméticas": "homogamético",
    "homogaméticos": "homogamético",
    "homogeneice": "homogeneizar",
    "homogeneicemos": "homogeneizar",
    "homogeneicen": "homogeneizar",
    "homogeneices": "homogeneizar",
    "homogeneicé": "homogeneizar",
    "homogeneicéis": "homogeneizar",
    "homogeneidades": "homogeneidad",
    "homogeneiza": "homogeneizar",
    "homogeneizaba": "homogeneizar",
    "homogeneizabais": "homogeneizar",
    "homogeneizaban": "homogeneizar",
    "homogeneizabas": "homogeneizar",
    "homogeneizaciones": "homogeneización",
    "homogeneizad": "homogeneizar",
    "homogeneizada": "homogeneizar",
    "homogeneizadas": "homogeneizar",
    "homogeneizado": "homogeneizar",
    "homogeneizadora": "homogeneizador",
    "homogeneizadoras": "homogeneizador",
    "homogeneizadores": "homogeneizador",
    "homogeneizados": "homogeneizar",
    "homogeneizamos": "homogeneizar",
    "homogeneizan": "homogeneizar",
    "homogeneizando": "homogeneizar",
    "homogeneizara": "homogeneizar",
    "homogeneizarais": "homogeneizar",
    "homogeneizaran": "homogeneizar",
    "homogeneizaras": "homogeneizar",
    "homogeneizare": "homogeneizar",
    "homogeneizareis": "homogeneizar",
    "homogeneizaremos": "homogeneizar",
    "homogeneizaren": "homogeneizar",
    "homogeneizares": "homogeneizar",
    "homogeneizaron": "homogeneizar",
    "homogeneizará": "homogeneizar",
    "homogeneizarán": "homogeneizar",
    "homogeneizarás": "homogeneizar",
    "homogeneizaré": "homogeneizar",
    "homogeneizaréis": "homogeneizar",
    "homogeneizaría": "homogeneizar",
    "homogeneizaríais": "homogeneizar",
    "homogeneizaríamos": "homogeneizar",
    "homogeneizarían": "homogeneizar",
    "homogeneizarías": "homogeneizar",
    "homogeneizas": "homogeneizar",
    "homogeneizase": "homogeneizar",
    "homogeneizaseis": "homogeneizar",
    "homogeneizasen": "homogeneizar",
    "homogeneizases": "homogeneizar",
    "homogeneizaste": "homogeneizar",
    "homogeneizasteis": "homogeneizar",
    "homogeneizo": "homogeneizar",
    "homogeneizábamos": "homogeneizar",
    "homogeneizáis": "homogeneizar",
    "homogeneizáramos": "homogeneizar",
    "homogeneizáremos": "homogeneizar",
    "homogeneizásemos": "homogeneizar",
    "homogeneizó": "homogeneizar",
    "homografías": "homografía",
    "homogénea": "homogéneo",
    "homogéneas": "homogéneo",
    "homogéneos": "homogéneo",
    "homoinjertos": "homoinjerto",
    "homologa": "homologar",
    "homologaba": "homologar",
    "homologabais": "homologar",
    "homologaban": "homologar",
    "homologabas": "homologar",
    "homologables": "homologable",
    "homologaciones": "homologación",
    "homologad": "homologar",
    "homologada": "homologar",
    "homologadas": "homologar",
    "homologado": "homologar",
    "homologados": "homologar",
    "homologamos": "homologar",
    "homologan": "homologar",
    "homologando": "homologar",
    "homologara": "homologar",
    "homologarais": "homologar",
    "homologaran": "homologar",
    "homologaras": "homologar",
    "homologare": "homologar",
    "homologareis": "homologar",
    "homologaremos": "homologar",
    "homologaren": "homologar",
    "homologares": "homologar",
    "homologaron": "homologar",
    "homologará": "homologar",
    "homologarán": "homologar",
    "homologarás": "homologar",
    "homologaré": "homologar",
    "homologaréis": "homologar",
    "homologaría": "homologar",
    "homologaríais": "homologar",
    "homologaríamos": "homologar",
    "homologarían": "homologar",
    "homologarías": "homologar",
    "homologas": "homologar",
    "homologase": "homologar",
    "homologaseis": "homologar",
    "homologasen": "homologar",
    "homologases": "homologar",
    "homologaste": "homologar",
    "homologasteis": "homologar",
    "homologo": "homologar",
    "homologue": "homologar",
    "homologuemos": "homologar",
    "homologuen": "homologar",
    "homologues": "homologar",
    "homologué": "homologar",
    "homologuéis": "homologar",
    "homologábamos": "homologar",
    "homologáis": "homologar",
    "homologáramos": "homologar",
    "homologáremos": "homologar",
    "homologásemos": "homologar",
    "homologías": "homología",
    "homologó": "homologar",
    "homomorfismos": "homomorfismo",
    "homomórfica": "homomórfico",
    "homomórficas": "homomórfico",
    "homomórficos": "homomórfico",
    "homonimias": "homonimia",
    "homonucleares": "homonuclear",
    "homopausas": "homopausa",
    "homoploidias": "homoploidia",
    "homopolares": "homopolar",
    "homopolímera": "homopolímero",
    "homopolímeras": "homopolímero",
    "homopolímeros": "homopolímero",
    "homorgánica": "homorgánico",
    "homorgánicas": "homorgánico",
    "homorgánicos": "homorgánico",
    "homoscedasticidades": "homoscedasticidad",
    "homosexuales": "homosexual",
    "homosexualidades": "homosexualidad",
    "homosferas": "homosfera",
    "homotecias": "homotecia",
    "homotopías": "homotopía",
    "homotoxicologías": "homotoxicología",
    "homotética": "homotético",
    "homotéticas": "homotético",
    "homotéticos": "homotético",
    "homotópica": "homotópico",
    "homotópicas": "homotópico",
    "homotópicos": "homotópico",
    "homozigota": "homozigoto",
    "homozigotas": "homozigoto",
    "homozigotos": "homozigoto",
    "homérica": "homérico",
    "homéricas": "homérico",
    "homéricos": "homérico",
    "homínidos": "homínido",
    "homófila": "homófilo",
    "homófilas": "homófilo",
    "homófilos": "homófilo",
    "homófona": "homófono",
    "homófonas": "homófono",
    "homófonos": "homófono",
    "homógrada": "homógrado",
    "homógradas": "homógrado",
    "homógrados": "homógrado",
    "homógrafa": "homógrafo",
    "homógrafas": "homógrafo",
    "homógrafos": "homógrafo",
    "homóloga": "homólogo",
    "homólogas": "homólogo",
    "homólogos": "homólogo",
    "homónima": "homónimo",
    "homónimas": "homónimo",
    "homónimos": "homónimo",
    "homópteros": "homóptero",
    "honcejos": "honcejo",
    "honda": "hondo",
    "hondables": "hondable",
    "hondadas": "hondada",
    "hondas": "hondo",
    "hondazos": "hondazo",
    "hondea": "hondear",
    "hondeaba": "hondear",
    "hondeabais": "hondear",
    "hondeaban": "hondear",
    "hondeabas": "hondear",
    "hondead": "hondear",
    "hondeada": "hondear",
    "hondeadas": "hondear",
    "hondeado": "hondear",
    "hondeados": "hondear",
    "hondeamos": "hondear",
    "hondean": "hondear",
    "hondeando": "hondear",
    "hondeara": "hondear",
    "hondearais": "hondear",
    "hondearan": "hondear",
    "hondearas": "hondear",
    "hondeare": "hondear",
    "hondeareis": "hondear",
    "hondearemos": "hondear",
    "hondearen": "hondear",
    "hondeares": "hondear",
    "hondearon": "hondear",
    "hondeará": "hondear",
    "hondearán": "hondear",
    "hondearás": "hondear",
    "hondearé": "hondear",
    "hondearéis": "hondear",
    "hondearía": "hondear",
    "hondearíais": "hondear",
    "hondearíamos": "hondear",
    "hondearían": "hondear",
    "hondearías": "hondear",
    "hondeas": "hondear",
    "hondease": "hondear",
    "hondeaseis": "hondear",
    "hondeasen": "hondear",
    "hondeases": "hondear",
    "hondeaste": "hondear",
    "hondeasteis": "hondear",
    "hondee": "hondear",
    "hondeemos": "hondear",
    "hondeen": "hondear",
    "hondees": "hondear",
    "hondeo": "hondear",
    "hondera": "hondero",
    "honderas": "hondero",
    "honderos": "hondero",
    "hondeábamos": "hondear",
    "hondeáis": "hondear",
    "hondeáramos": "hondear",
    "hondeáremos": "hondear",
    "hondeásemos": "hondear",
    "hondeé": "hondear",
    "hondeéis": "hondear",
    "hondeó": "hondear",
    "hondijos": "hondijo",
    "hondonadas": "hondonada",
    "hondones": "hondón",
    "hondos": "hondo",
    "honduras": "hondura",
    "hondureña": "hondureño",
    "hondureñas": "hondureño",
    "hondureñismos": "hondureñismo",
    "hondureños": "hondureño",
    "honesta": "honesto",
    "honestaba": "honestar",
    "honestabais": "honestar",
    "honestaban": "honestar",
    "honestabas": "honestar",
    "honestad": "honestar",
    "honestada": "honestar",
    "honestadas": "honestar",
    "honestado": "honestar",
    "honestados": "honestar",
    "honestamos": "honestar",
    "honestan": "honestar",
    "honestando": "honestar",
    "honestara": "honestar",
    "honestarais": "honestar",
    "honestaran": "honestar",
    "honestaras": "honestar",
    "honestare": "honestar",
    "honestareis": "honestar",
    "honestaremos": "honestar",
    "honestaren": "honestar",
    "honestares": "honestar",
    "honestaron": "honestar",
    "honestará": "honestar",
    "honestarán": "honestar",
    "honestarás": "honestar",
    "honestaré": "honestar",
    "honestaréis": "honestar",
    "honestaría": "honestar",
    "honestaríais": "honestar",
    "honestaríamos": "honestar",
    "honestarían": "honestar",
    "honestarías": "honestar",
    "honestas": "honesto",
    "honestase": "honestar",
    "honestaseis": "honestar",
    "honestasen": "honestar",
    "honestases": "honestar",
    "honestaste": "honestar",
    "honestasteis": "honestar",
    "honeste": "honestar",
    "honestemos": "honestar",
    "honesten": "honestar",
    "honestes": "honestar",
    "honestidades": "honestidad",
    "honesto": "honestar",
    "honestos": "honesto",
    "honestábamos": "honestar",
    "honestáis": "honestar",
    "honestáramos": "honestar",
    "honestáremos": "honestar",
    "honestásemos": "honestar",
    "honesté": "honestar",
    "honestéis": "honestar",
    "honestó": "honestar",
    "honga": "hongo",
    "hongas": "hongo",
    "hongos": "hongo",
    "honora": "honorar",
    "honoraba": "honorar",
    "honorabais": "honorar",
    "honoraban": "honorar",
    "honorabas": "honorar",
    "honorabilidades": "honorabilidad",
    "honorables": "honorable",
    "honorad": "honorar",
    "honorada": "honorar",
    "honoradas": "honorar",
    "honorado": "honorar",
    "honorados": "honorar",
    "honoramos": "honorar",
    "honoran": "honorar",
    "honorando": "honorar",
    "honorara": "honorar",
    "honorarais": "honorar",
    "honoraran": "honorar",
    "honoraras": "honorar",
    "honorare": "honorar",
    "honorareis": "honorar",
    "honoraremos": "honorar",
    "honoraren": "honorar",
    "honorares": "honorar",
    "honoraria": "honorario",
    "honorarias": "honorario",
    "honorarios": "honorario",
    "honoraron": "honorar",
    "honorará": "honorar",
    "honorarán": "honorar",
    "honorarás": "honorar",
    "honoraré": "honorar",
    "honoraréis": "honorar",
    "honoraría": "honorar",
    "honoraríais": "honorar",
    "honoraríamos": "honorar",
    "honorarían": "honorar",
    "honorarías": "honorar",
    "honoras": "honorar",
    "honorase": "honorar",
    "honoraseis": "honorar",
    "honorasen": "honorar",
    "honorases": "honorar",
    "honoraste": "honorar",
    "honorasteis": "honorar",
    "honore": "honorar",
    "honoremos": "honorar",
    "honoren": "honorar",
    "honores": "honorar",
    "honoro": "honorar",
    "honorábamos": "honorar",
    "honoráis": "honorar",
    "honoráramos": "honorar",
    "honoráremos": "honorar",
    "honorásemos": "honorar",
    "honoré": "honorar",
    "honoréis": "honorar",
    "honorífica": "honorífico",
    "honoríficas": "honorífico",
    "honoríficos": "honorífico",
    "honoró": "honorar",
    "honra": "honrar",
    "honraba": "honrar",
    "honrabais": "honrar",
    "honraban": "honrar",
    "honrabas": "honrar",
    "honrad": "honrar",
    "honrada": "honrar",
    "honradas": "honrar",
    "honradeces": "honradez",
    "honrado": "honrar",
    "honrados": "honrar",
    "honramientos": "honramiento",
    "honramos": "honrar",
    "honran": "honrar",
    "honrando": "honrar",
    "honrara": "honrar",
    "honrarais": "honrar",
    "honraran": "honrar",
    "honraras": "honrar",
    "honrare": "honrar",
    "honrareis": "honrar",
    "honraremos": "honrar",
    "honraren": "honrar",
    "honrares": "honrar",
    "honraron": "honrar",
    "honrará": "honrar",
    "honrarán": "honrar",
    "honrarás": "honrar",
    "honraré": "honrar",
    "honraréis": "honrar",
    "honraría": "honrar",
    "honraríais": "honrar",
    "honraríamos": "honrar",
    "honrarían": "honrar",
    "honrarías": "honrar",
    "honras": "honrar",
    "honrase": "honrar",
    "honraseis": "honrar",
    "honrasen": "honrar",
    "honrases": "honrar",
    "honraste": "honrar",
    "honrasteis": "honrar",
    "honre": "honrar",
    "honremos": "honrar",
    "honren": "honrar",
    "honres": "honrar",
    "honrillas": "honrilla",
    "honro": "honrar",
    "honrosa": "honroso",
    "honrosas": "honroso",
    "honrosos": "honroso",
    "honrábamos": "honrar",
    "honráis": "honrar",
    "honráramos": "honrar",
    "honráremos": "honrar",
    "honrásemos": "honrar",
    "honré": "honrar",
    "honréis": "honrar",
    "honró": "honrar",
    "hontanales": "hontanal",
    "hontanares": "hontanar",
    "hopalandas": "hopalanda",
    "hopas": "hopa",
    "hopea": "hopear",
    "hopeaba": "hopear",
    "hopeabais": "hopear",
    "hopeaban": "hopear",
    "hopeabas": "hopear",
    "hopead": "hopear",
    "hopeada": "hopear",
    "hopeadas": "hopear",
    "hopeado": "hopear",
    "hopeados": "hopear",
    "hopeamos": "hopear",
    "hopean": "hopear",
    "hopeando": "hopear",
    "hopeara": "hopear",
    "hopearais": "hopear",
    "hopearan": "hopear",
    "hopearas": "hopear",
    "hopeare": "hopear",
    "hopeareis": "hopear",
    "hopearemos": "hopear",
    "hopearen": "hopear",
    "hopeares": "hopear",
    "hopearon": "hopear",
    "hopeará": "hopear",
    "hopearán": "hopear",
    "hopearás": "hopear",
    "hopearé": "hopear",
    "hopearéis": "hopear",
    "hopearía": "hopear",
    "hopearíais": "hopear",
    "hopearíamos": "hopear",
    "hopearían": "hopear",
    "hopearías": "hopear",
    "hopeas": "hopear",
    "hopease": "hopear",
    "hopeaseis": "hopear",
    "hopeasen": "hopear",
    "hopeases": "hopear",
    "hopeaste": "hopear",
    "hopeasteis": "hopear",
    "hopee": "hopear",
    "hopeemos": "hopear",
    "hopeen": "hopear",
    "hopees": "hopear",
    "hopeo": "hopear",
    "hopeos": "hopeo",
    "hopeábamos": "hopear",
    "hopeáis": "hopear",
    "hopeáramos": "hopear",
    "hopeáremos": "hopear",
    "hopeásemos": "hopear",
    "hopeé": "hopear",
    "hopeéis": "hopear",
    "hopeó": "hopear",
    "hopis": "hopi",
    "hoplitas": "hoplita",
    "hoplotecas": "hoploteca",
    "hopos": "hopo",
    "horaciana": "horaciano",
    "horacianas": "horaciano",
    "horacianos": "horaciano",
    "horada": "horadar",
    "horadaba": "horadar",
    "horadabais": "horadar",
    "horadaban": "horadar",
    "horadabas": "horadar",
    "horadables": "horadable",
    "horadaciones": "horadación",
    "horadad": "horadar",
    "horadada": "horadar",
    "horadadas": "horadar",
    "horadado": "horadar",
    "horadadora": "horadador",
    "horadadoras": "horadador",
    "horadadores": "horadador",
    "horadados": "horadar",
    "horadamos": "horadar",
    "horadan": "horadar",
    "horadando": "horadar",
    "horadara": "horadar",
    "horadarais": "horadar",
    "horadaran": "horadar",
    "horadaras": "horadar",
    "horadare": "horadar",
    "horadareis": "horadar",
    "horadaremos": "horadar",
    "horadaren": "horadar",
    "horadares": "horadar",
    "horadaron": "horadar",
    "horadará": "horadar",
    "horadarán": "horadar",
    "horadarás": "horadar",
    "horadaré": "horadar",
    "horadaréis": "horadar",
    "horadaría": "horadar",
    "horadaríais": "horadar",
    "horadaríamos": "horadar",
    "horadarían": "horadar",
    "horadarías": "horadar",
    "horadas": "horadar",
    "horadase": "horadar",
    "horadaseis": "horadar",
    "horadasen": "horadar",
    "horadases": "horadar",
    "horadaste": "horadar",
    "horadasteis": "horadar",
    "horade": "horadar",
    "horademos": "horadar",
    "horaden": "horadar",
    "horades": "horadar",
    "horado": "horadar",
    "horados": "horado",
    "horadábamos": "horadar",
    "horadáis": "horadar",
    "horadáramos": "horadar",
    "horadáremos": "horadar",
    "horadásemos": "horadar",
    "horadé": "horadar",
    "horadéis": "horadar",
    "horadó": "horadar",
    "horambres": "horambre",
    "horaria": "horario",
    "horarias": "horario",
    "horarios": "horario",
    "horas": "hora",
    "horca": "horcar",
    "horcaba": "horcar",
    "horcabais": "horcar",
    "horcaban": "horcar",
    "horcabas": "horcar",
    "horcad": "horcar",
    "horcada": "horcar",
    "horcadas": "horcar",
    "horcado": "horcar",
    "horcados": "horcar",
    "horcaduras": "horcadura",
    "horcajaduras": "horcajadura",
    "horcajos": "horcajo",
    "horcamos": "horcar",
    "horcan": "horcar",
    "horcando": "horcar",
    "horcara": "horcar",
    "horcarais": "horcar",
    "horcaran": "horcar",
    "horcaras": "horcar",
    "horcare": "horcar",
    "horcareis": "horcar",
    "horcaremos": "horcar",
    "horcaren": "horcar",
    "horcares": "horcar",
    "horcaron": "horcar",
    "horcará": "horcar",
    "horcarán": "horcar",
    "horcarás": "horcar",
    "horcaré": "horcar",
    "horcaréis": "horcar",
    "horcaría": "horcar",
    "horcaríais": "horcar",
    "horcaríamos": "horcar",
    "horcarían": "horcar",
    "horcarías": "horcar",
    "horcas": "horcar",
    "horcase": "horcar",
    "horcaseis": "horcar",
    "horcasen": "horcar",
    "horcases": "horcar",
    "horcaste": "horcar",
    "horcasteis": "horcar",
    "horcates": "horcate",
    "horchatas": "horchata",
    "horchatera": "horchatero",
    "horchateras": "horchatero",
    "horchateros": "horchatero",
    "horchaterías": "horchatería",
    "horco": "horcar",
    "horconadas": "horconada",
    "horconaduras": "horconadura",
    "horcones": "horcón",
    "horcos": "horco",
    "horcábamos": "horcar",
    "horcáis": "horcar",
    "horcáramos": "horcar",
    "horcáremos": "horcar",
    "horcásemos": "horcar",
    "horcó": "horcar",
    "hordas": "horda",
    "hordeninas": "hordenina",
    "hordiates": "hordiate",
    "horizonaciones": "horizonación",
    "horizontales": "horizontal",
    "horizontalidades": "horizontalidad",
    "horizontes": "horizonte",
    "hormas": "horma",
    "hormazas": "hormaza",
    "hormazos": "hormazo",
    "hormigantes": "hormigante",
    "hormigas": "hormiga",
    "hormigona": "hormigonar",
    "hormigonaba": "hormigonar",
    "hormigonabais": "hormigonar",
    "hormigonaban": "hormigonar",
    "hormigonabas": "hormigonar",
    "hormigonad": "hormigonar",
    "hormigonada": "hormigonar",
    "hormigonadas": "hormigonar",
    "hormigonado": "hormigonar",
    "hormigonados": "hormigonar",
    "hormigonamos": "hormigonar",
    "hormigonan": "hormigonar",
    "hormigonando": "hormigonar",
    "hormigonara": "hormigonar",
    "hormigonarais": "hormigonar",
    "hormigonaran": "hormigonar",
    "hormigonaras": "hormigonar",
    "hormigonare": "hormigonar",
    "hormigonareis": "hormigonar",
    "hormigonaremos": "hormigonar",
    "hormigonaren": "hormigonar",
    "hormigonares": "hormigonar",
    "hormigonaron": "hormigonar",
    "hormigonará": "hormigonar",
    "hormigonarán": "hormigonar",
    "hormigonarás": "hormigonar",
    "hormigonaré": "hormigonar",
    "hormigonaréis": "hormigonar",
    "hormigonaría": "hormigonar",
    "hormigonaríais": "hormigonar",
    "hormigonaríamos": "hormigonar",
    "hormigonarían": "hormigonar",
    "hormigonarías": "hormigonar",
    "hormigonas": "hormigonar",
    "hormigonase": "hormigonar",
    "hormigonaseis": "hormigonar",
    "hormigonasen": "hormigonar",
    "hormigonases": "hormigonar",
    "hormigonaste": "hormigonar",
    "hormigonasteis": "hormigonar",
    "hormigone": "hormigonar",
    "hormigonemos": "hormigonar",
    "hormigonen": "hormigonar",
    "hormigoneras": "hormigonera",
    "hormigones": "hormigonar",
    "hormigono": "hormigonar",
    "hormigonábamos": "hormigonar",
    "hormigonáis": "hormigonar",
    "hormigonáramos": "hormigonar",
    "hormigonáremos": "hormigonar",
    "hormigonásemos": "hormigonar",
    "hormigoné": "hormigonar",
    "hormigonéis": "hormigonar",
    "hormigonó": "hormigonar",
    "hormigos": "hormigo",
    "hormigosa": "hormigoso",
    "hormigosas": "hormigoso",
    "hormigosos": "hormigoso",
    "hormiguea": "hormiguear",
    "hormigueaba": "hormiguear",
    "hormigueabais": "hormiguear",
    "hormigueaban": "hormiguear",
    "hormigueabas": "hormiguear",
    "hormiguead": "hormiguear",
    "hormigueada": "hormiguear",
    "hormigueadas": "hormiguear",
    "hormigueado": "hormiguear",
    "hormigueados": "hormiguear",
    "hormigueamientos": "hormigueamiento",
    "hormigueamos": "hormiguear",
    "hormiguean": "hormiguear",
    "hormigueando": "hormiguear",
    "hormigueara": "hormiguear",
    "hormiguearais": "hormiguear",
    "hormiguearan": "hormiguear",
    "hormiguearas": "hormiguear",
    "hormigueare": "hormiguear",
    "hormigueareis": "hormiguear",
    "hormiguearemos": "hormiguear",
    "hormiguearen": "hormiguear",
    "hormigueares": "hormiguear",
    "hormiguearon": "hormiguear",
    "hormigueará": "hormiguear",
    "hormiguearán": "hormiguear",
    "hormiguearás": "hormiguear",
    "hormiguearé": "hormiguear",
    "hormiguearéis": "hormiguear",
    "hormiguearía": "hormiguear",
    "hormiguearíais": "hormiguear",
    "hormiguearíamos": "hormiguear",
    "hormiguearían": "hormiguear",
    "hormiguearías": "hormiguear",
    "hormigueas": "hormiguear",
    "hormiguease": "hormiguear",
    "hormigueaseis": "hormiguear",
    "hormigueasen": "hormiguear",
    "hormigueases": "hormiguear",
    "hormigueaste": "hormiguear",
    "hormigueasteis": "hormiguear",
    "hormiguee": "hormiguear",
    "hormigueemos": "hormiguear",
    "hormigueen": "hormiguear",
    "hormiguees": "hormiguear",
    "hormigueo": "hormiguear",
    "hormigueos": "hormigueo",
    "hormigueros": "hormiguero",
    "hormiguesca": "hormiguesco",
    "hormiguescas": "hormiguesco",
    "hormiguescos": "hormiguesco",
    "hormigueábamos": "hormiguear",
    "hormigueáis": "hormiguear",
    "hormigueáramos": "hormiguear",
    "hormigueáremos": "hormiguear",
    "hormigueásemos": "hormiguear",
    "hormigueé": "hormiguear",
    "hormigueéis": "hormiguear",
    "hormigueó": "hormiguear",
    "hormiguillas": "hormiguilla",
    "hormiguillos": "hormiguillo",
    "hormillas": "hormilla",
    "hormonales": "hormonal",
    "hormonas": "hormona",
    "hormoproteínas": "hormoproteína",
    "hornabeques": "hornabeque",
    "hornacera": "hornacero",
    "hornaceras": "hornacero",
    "hornaceros": "hornacero",
    "hornachos": "hornacho",
    "hornachuelas": "hornachuela",
    "hornacinas": "hornacina",
    "hornadas": "hornada",
    "hornaguea": "hornaguear",
    "hornagueaba": "hornaguear",
    "hornagueabais": "hornaguear",
    "hornagueaban": "hornaguear",
    "hornagueabas": "hornaguear",
    "hornaguead": "hornaguear",
    "hornagueada": "hornaguear",
    "hornagueadas": "hornaguear",
    "hornagueado": "hornaguear",
    "hornagueados": "hornaguear",
    "hornagueamos": "hornaguear",
    "hornaguean": "hornaguear",
    "hornagueando": "hornaguear",
    "hornagueara": "hornaguear",
    "hornaguearais": "hornaguear",
    "hornaguearan": "hornaguear",
    "hornaguearas": "hornaguear",
    "hornagueare": "hornaguear",
    "hornagueareis": "hornaguear",
    "hornaguearemos": "hornaguear",
    "hornaguearen": "hornaguear",
    "hornagueares": "hornaguear",
    "hornaguearon": "hornaguear",
    "hornagueará": "hornaguear",
    "hornaguearán": "hornaguear",
    "hornaguearás": "hornaguear",
    "hornaguearé": "hornaguear",
    "hornaguearéis": "hornaguear",
    "hornaguearía": "hornaguear",
    "hornaguearíais": "hornaguear",
    "hornaguearíamos": "hornaguear",
    "hornaguearían": "hornaguear",
    "hornaguearías": "hornaguear",
    "hornagueas": "hornaguear",
    "hornaguease": "hornaguear",
    "hornagueaseis": "hornaguear",
    "hornagueasen": "hornaguear",
    "hornagueases": "hornaguear",
    "hornagueaste": "hornaguear",
    "hornagueasteis": "hornaguear",
    "hornaguee": "hornaguear",
    "hornagueemos": "hornaguear",
    "hornagueen": "hornaguear",
    "hornaguees": "hornaguear",
    "hornagueo": "hornaguear",
    "hornagueos": "hornagueo",
    "hornaguera": "hornaguero",
    "hornagueras": "hornaguero",
    "hornagueros": "hornaguero",
    "hornagueábamos": "hornaguear",
    "hornagueáis": "hornaguear",
    "hornagueáramos": "hornaguear",
    "hornagueáremos": "hornaguear",
    "hornagueásemos": "hornaguear",
    "hornagueé": "hornaguear",
    "hornagueéis": "hornaguear",
    "hornagueó": "hornaguear",
    "hornazas": "hornaza",
    "hornazos": "hornazo",
    "hornblendas": "hornblenda",
    "hornea": "hornear",
    "horneaba": "hornear",
    "horneabais": "hornear",
    "horneaban": "hornear",
    "horneabas": "hornear",
    "hornead": "hornear",
    "horneada": "hornear",
    "horneadas": "hornear",
    "horneado": "hornear",
    "horneados": "hornear",
    "horneamos": "hornear",
    "hornean": "hornear",
    "horneando": "hornear",
    "horneara": "hornear",
    "hornearais": "hornear",
    "hornearan": "hornear",
    "hornearas": "hornear",
    "horneare": "hornear",
    "horneareis": "hornear",
    "hornearemos": "hornear",
    "hornearen": "hornear",
    "horneares": "hornear",
    "hornearon": "hornear",
    "horneará": "hornear",
    "hornearán": "hornear",
    "hornearás": "hornear",
    "hornearé": "hornear",
    "hornearéis": "hornear",
    "hornearía": "hornear",
    "hornearíais": "hornear",
    "hornearíamos": "hornear",
    "hornearían": "hornear",
    "hornearías": "hornear",
    "horneas": "hornear",
    "hornease": "hornear",
    "horneaseis": "hornear",
    "horneasen": "hornear",
    "horneases": "hornear",
    "horneaste": "hornear",
    "horneasteis": "hornear",
    "hornecina": "hornecino",
    "hornecinas": "hornecino",
    "hornecinos": "hornecino",
    "hornee": "hornear",
    "horneemos": "hornear",
    "horneen": "hornear",
    "hornees": "hornear",
    "horneo": "hornear",
    "hornera": "hornero",
    "horneras": "hornero",
    "horneros": "hornero",
    "hornerías": "hornería",
    "horneábamos": "hornear",
    "horneáis": "hornear",
    "horneáramos": "hornear",
    "horneáremos": "hornear",
    "horneásemos": "hornear",
    "horneé": "hornear",
    "horneéis": "hornear",
    "horneó": "hornear",
    "hornijas": "hornija",
    "hornijera": "hornijero",
    "hornijeras": "hornijero",
    "hornijeros": "hornijero",
    "hornillas": "hornilla",
    "hornillos": "hornillo",
    "hornos": "horno",
    "horque": "horcar",
    "horquemos": "horcar",
    "horquen": "horcar",
    "horques": "horcar",
    "horquetas": "horqueta",
    "horquillada": "horquillado",
    "horquilladas": "horquillado",
    "horquillados": "horquillado",
    "horquillas": "horquilla",
    "horqué": "horcar",
    "horquéis": "horcar",
    "horra": "horro",
    "horraba": "horrar",
    "horrabais": "horrar",
    "horraban": "horrar",
    "horrabas": "horrar",
    "horrad": "horrar",
    "horrada": "horrar",
    "horradas": "horrar",
    "horrado": "horrar",
    "horrados": "horrar",
    "horramos": "horrar",
    "horran": "horrar",
    "horrando": "horrar",
    "horrara": "horrar",
    "horrarais": "horrar",
    "horraran": "horrar",
    "horraras": "horrar",
    "horrare": "horrar",
    "horrareis": "horrar",
    "horraremos": "horrar",
    "horraren": "horrar",
    "horrares": "horrar",
    "horraron": "horrar",
    "horrará": "horrar",
    "horrarán": "horrar",
    "horrarás": "horrar",
    "horraré": "horrar",
    "horraréis": "horrar",
    "horraría": "horrar",
    "horraríais": "horrar",
    "horraríamos": "horrar",
    "horrarían": "horrar",
    "horrarías": "horrar",
    "horras": "horro",
    "horrase": "horrar",
    "horraseis": "horrar",
    "horrasen": "horrar",
    "horrases": "horrar",
    "horraste": "horrar",
    "horrasteis": "horrar",
    "horre": "horrar",
    "horremos": "horrar",
    "horren": "horrar",
    "horrenda": "horrendo",
    "horrendas": "horrendo",
    "horrendos": "horrendo",
    "horrera": "horrero",
    "horreras": "horrero",
    "horreros": "horrero",
    "horres": "horrar",
    "horribilidades": "horribilidad",
    "horribles": "horrible",
    "horripila": "horripilar",
    "horripilaba": "horripilar",
    "horripilabais": "horripilar",
    "horripilaban": "horripilar",
    "horripilabas": "horripilar",
    "horripilaciones": "horripilación",
    "horripilad": "horripilar",
    "horripilada": "horripilar",
    "horripiladas": "horripilar",
    "horripilado": "horripilar",
    "horripilados": "horripilar",
    "horripilamos": "horripilar",
    "horripilan": "horripilar",
    "horripilando": "horripilar",
    "horripilantes": "horripilante",
    "horripilara": "horripilar",
    "horripilarais": "horripilar",
    "horripilaran": "horripilar",
    "horripilaras": "horripilar",
    "horripilare": "horripilar",
    "horripilareis": "horripilar",
    "horripilaremos": "horripilar",
    "horripilaren": "horripilar",
    "horripilares": "horripilar",
    "horripilaron": "horripilar",
    "horripilará": "horripilar",
    "horripilarán": "horripilar",
    "horripilarás": "horripilar",
    "horripilaré": "horripilar",
    "horripilaréis": "horripilar",
    "horripilaría": "horripilar",
    "horripilaríais": "horripilar",
    "horripilaríamos": "horripilar",
    "horripilarían": "horripilar",
    "horripilarías": "horripilar",
    "horripilas": "horripilar",
    "horripilase": "horripilar",
    "horripilaseis": "horripilar",
    "horripilasen": "horripilar",
    "horripilases": "horripilar",
    "horripilaste": "horripilar",
    "horripilasteis": "horripilar",
    "horripile": "horripilar",
    "horripilemos": "horripilar",
    "horripilen": "horripilar",
    "horripiles": "horripilar",
    "horripilo": "horripilar",
    "horripilábamos": "horripilar",
    "horripiláis": "horripilar",
    "horripiláramos": "horripilar",
    "horripiláremos": "horripilar",
    "horripilásemos": "horripilar",
    "horripilé": "horripilar",
    "horripiléis": "horripilar",
    "horripiló": "horripilar",
    "horro": "horrar",
    "horrores": "horror",
    "horrorice": "horrorizar",
    "horroricemos": "horrorizar",
    "horroricen": "horrorizar",
    "horrorices": "horrorizar",
    "horroricé": "horrorizar",
    "horroricéis": "horrorizar",
    "horroriza": "horrorizar",
    "horrorizaba": "horrorizar",
    "horrorizabais": "horrorizar",
    "horrorizaban": "horrorizar",
    "horrorizabas": "horrorizar",
    "horrorizad": "horrorizar",
    "horrorizada": "horrorizar",
    "horrorizadas": "horrorizar",
    "horrorizado": "horrorizar",
    "horrorizados": "horrorizar",
    "horrorizamos": "horrorizar",
    "horrorizan": "horrorizar",
    "horrorizando": "horrorizar",
    "horrorizara": "horrorizar",
    "horrorizarais": "horrorizar",
    "horrorizaran": "horrorizar",
    "horrorizaras": "horrorizar",
    "horrorizare": "horrorizar",
    "horrorizareis": "horrorizar",
    "horrorizaremos": "horrorizar",
    "horrorizaren": "horrorizar",
    "horrorizares": "horrorizar",
    "horrorizaron": "horrorizar",
    "horrorizará": "horrorizar",
    "horrorizarán": "horrorizar",
    "horrorizarás": "horrorizar",
    "horrorizaré": "horrorizar",
    "horrorizaréis": "horrorizar",
    "horrorizaría": "horrorizar",
    "horrorizaríais": "horrorizar",
    "horrorizaríamos": "horrorizar",
    "horrorizarían": "horrorizar",
    "horrorizarías": "horrorizar",
    "horrorizas": "horrorizar",
    "horrorizase": "horrorizar",
    "horrorizaseis": "horrorizar",
    "horrorizasen": "horrorizar",
    "horrorizases": "horrorizar",
    "horrorizaste": "horrorizar",
    "horrorizasteis": "horrorizar",
    "horrorizo": "horrorizar",
    "horrorizábamos": "horrorizar",
    "horrorizáis": "horrorizar",
    "horrorizáramos": "horrorizar",
    "horrorizáremos": "horrorizar",
    "horrorizásemos": "horrorizar",
    "horrorizó": "horrorizar",
    "horrorosa": "horroroso",
    "horrorosas": "horroroso",
    "horrorosos": "horroroso",
    "horros": "horro",
    "horruras": "horrura",
    "horrábamos": "horrar",
    "horráis": "horrar",
    "horráramos": "horrar",
    "horráremos": "horrar",
    "horrásemos": "horrar",
    "horré": "horrar",
    "horréis": "horrar",
    "horrísona": "horrísono",
    "horrísonas": "horrísono",
    "horrísonos": "horrísono",
    "horró": "horrar",
    "horsebacks": "horseback",
    "hortalizas": "hortaliza",
    "hortelana": "hortelano",
    "hortelanas": "hortelano",
    "hortelanos": "hortelano",
    "hortenses": "hortense",
    "hortensias": "hortensia",
    "horteradas": "horterada",
    "horteras": "hortera",
    "horticultora": "horticultor",
    "horticultoras": "horticultor",
    "horticultores": "horticultor",
    "horticulturas": "horticultura",
    "hortofruticulturas": "hortofruticultura",
    "hortofrutícolas": "hortofrutícola",
    "hortonolitas": "hortonolita",
    "hortícolas": "hortícola",
    "horópteros": "horóptero",
    "horóscopos": "horóscopo",
    "hosannas": "hosanna",
    "hosca": "hosco",
    "hoscas": "hosco",
    "hoscos": "hosco",
    "hospeda": "hospedar",
    "hospedaba": "hospedar",
    "hospedabais": "hospedar",
    "hospedaban": "hospedar",
    "hospedabas": "hospedar",
    "hospedad": "hospedar",
    "hospedada": "hospedar",
    "hospedadas": "hospedar",
    "hospedado": "hospedar",
    "hospedados": "hospedar",
    "hospedajes": "hospedaje",
    "hospedamos": "hospedar",
    "hospedan": "hospedar",
    "hospedando": "hospedar",
    "hospedara": "hospedar",
    "hospedarais": "hospedar",
    "hospedaran": "hospedar",
    "hospedaras": "hospedar",
    "hospedare": "hospedar",
    "hospedareis": "hospedar",
    "hospedaremos": "hospedar",
    "hospedaren": "hospedar",
    "hospedares": "hospedar",
    "hospedaron": "hospedar",
    "hospedará": "hospedar",
    "hospedarán": "hospedar",
    "hospedarás": "hospedar",
    "hospedaré": "hospedar",
    "hospedaréis": "hospedar",
    "hospedaría": "hospedar",
    "hospedaríais": "hospedar",
    "hospedaríamos": "hospedar",
    "hospedarían": "hospedar",
    "hospedarías": "hospedar",
    "hospedas": "hospedar",
    "hospedase": "hospedar",
    "hospedaseis": "hospedar",
    "hospedasen": "hospedar",
    "hospedases": "hospedar",
    "hospedaste": "hospedar",
    "hospedasteis": "hospedar",
    "hospede": "hospedar",
    "hospedemos": "hospedar",
    "hospeden": "hospedar",
    "hospedera": "hospedero",
    "hospederas": "hospedero",
    "hospederos": "hospedero",
    "hospederías": "hospedería",
    "hospedes": "hospedar",
    "hospedo": "hospedar",
    "hospedábamos": "hospedar",
    "hospedáis": "hospedar",
    "hospedáramos": "hospedar",
    "hospedáremos": "hospedar",
    "hospedásemos": "hospedar",
    "hospedé": "hospedar",
    "hospedéis": "hospedar",
    "hospedó": "hospedar",
    "hospiciana": "hospiciano",
    "hospicianas": "hospiciano",
    "hospicianos": "hospiciano",
    "hospicios": "hospicio",
    "hospitalaria": "hospitalario",
    "hospitalarias": "hospitalario",
    "hospitalarios": "hospitalario",
    "hospitales": "hospital",
    "hospitalice": "hospitalizar",
    "hospitalicemos": "hospitalizar",
    "hospitalicen": "hospitalizar",
    "hospitalices": "hospitalizar",
    "hospitalicia": "hospitalicio",
    "hospitalicias": "hospitalicio",
    "hospitalicios": "hospitalicio",
    "hospitalicé": "hospitalizar",
    "hospitalicéis": "hospitalizar",
    "hospitalidades": "hospitalidad",
    "hospitaliza": "hospitalizar",
    "hospitalizaba": "hospitalizar",
    "hospitalizabais": "hospitalizar",
    "hospitalizaban": "hospitalizar",
    "hospitalizabas": "hospitalizar",
    "hospitalizaciones": "hospitalización",
    "hospitalizad": "hospitalizar",
    "hospitalizada": "hospitalizar",
    "hospitalizadas": "hospitalizar",
    "hospitalizado": "hospitalizar",
    "hospitalizados": "hospitalizar",
    "hospitalizamos": "hospitalizar",
    "hospitalizan": "hospitalizar",
    "hospitalizando": "hospitalizar",
    "hospitalizara": "hospitalizar",
    "hospitalizarais": "hospitalizar",
    "hospitalizaran": "hospitalizar",
    "hospitalizaras": "hospitalizar",
    "hospitalizare": "hospitalizar",
    "hospitalizareis": "hospitalizar",
    "hospitalizaremos": "hospitalizar",
    "hospitalizaren": "hospitalizar",
    "hospitalizares": "hospitalizar",
    "hospitalizaron": "hospitalizar",
    "hospitalizará": "hospitalizar",
    "hospitalizarán": "hospitalizar",
    "hospitalizarás": "hospitalizar",
    "hospitalizaré": "hospitalizar",
    "hospitalizaréis": "hospitalizar",
    "hospitalizaría": "hospitalizar",
    "hospitalizaríais": "hospitalizar",
    "hospitalizaríamos": "hospitalizar",
    "hospitalizarían": "hospitalizar",
    "hospitalizarías": "hospitalizar",
    "hospitalizas": "hospitalizar",
    "hospitalizase": "hospitalizar",
    "hospitalizaseis": "hospitalizar",
    "hospitalizasen": "hospitalizar",
    "hospitalizases": "hospitalizar",
    "hospitalizaste": "hospitalizar",
    "hospitalizasteis": "hospitalizar",
    "hospitalizo": "hospitalizar",
    "hospitalizábamos": "hospitalizar",
    "hospitalizáis": "hospitalizar",
    "hospitalizáramos": "hospitalizar",
    "hospitalizáremos": "hospitalizar",
    "hospitalizásemos": "hospitalizar",
    "hospitalizó": "hospitalizar",
    "hospodares": "hospodar",
    "hosquedades": "hosquedad",
    "hostales": "hostal",
    "hostelera": "hostelero",
    "hosteleras": "hostelero",
    "hosteleros": "hostelero",
    "hostelerías": "hostelería",
    "hosterías": "hostería",
    "hostiarios": "hostiario",
    "hostias": "hostia",
    "hostiera": "hostiero",
    "hostieras": "hostiero",
    "hostieros": "hostiero",
    "hostiga": "hostigar",
    "hostigaba": "hostigar",
    "hostigabais": "hostigar",
    "hostigaban": "hostigar",
    "hostigabas": "hostigar",
    "hostigad": "hostigar",
    "hostigada": "hostigar",
    "hostigadas": "hostigar",
    "hostigado": "hostigar",
    "hostigadora": "hostigador",
    "hostigadoras": "hostigador",
    "hostigadores": "hostigador",
    "hostigados": "hostigar",
    "hostigamientos": "hostigamiento",
    "hostigamos": "hostigar",
    "hostigan": "hostigar",
    "hostigando": "hostigar",
    "hostigara": "hostigar",
    "hostigarais": "hostigar",
    "hostigaran": "hostigar",
    "hostigaras": "hostigar",
    "hostigare": "hostigar",
    "hostigareis": "hostigar",
    "hostigaremos": "hostigar",
    "hostigaren": "hostigar",
    "hostigares": "hostigar",
    "hostigaron": "hostigar",
    "hostigará": "hostigar",
    "hostigarán": "hostigar",
    "hostigarás": "hostigar",
    "hostigaré": "hostigar",
    "hostigaréis": "hostigar",
    "hostigaría": "hostigar",
    "hostigaríais": "hostigar",
    "hostigaríamos": "hostigar",
    "hostigarían": "hostigar",
    "hostigarías": "hostigar",
    "hostigas": "hostigar",
    "hostigase": "hostigar",
    "hostigaseis": "hostigar",
    "hostigasen": "hostigar",
    "hostigases": "hostigar",
    "hostigaste": "hostigar",
    "hostigasteis": "hostigar",
    "hostigo": "hostigar",
    "hostigos": "hostigo",
    "hostigue": "hostigar",
    "hostiguemos": "hostigar",
    "hostiguen": "hostigar",
    "hostigues": "hostigar",
    "hostigué": "hostigar",
    "hostiguéis": "hostigar",
    "hostigábamos": "hostigar",
    "hostigáis": "hostigar",
    "hostigáramos": "hostigar",
    "hostigáremos": "hostigar",
    "hostigásemos": "hostigar",
    "hostigó": "hostigar",
    "hostiles": "hostil",
    "hostilice": "hostilizar",
    "hostilicemos": "hostilizar",
    "hostilicen": "hostilizar",
    "hostilices": "hostilizar",
    "hostilicé": "hostilizar",
    "hostilicéis": "hostilizar",
    "hostilidades": "hostilidad",
    "hostiliza": "hostilizar",
    "hostilizaba": "hostilizar",
    "hostilizabais": "hostilizar",
    "hostilizaban": "hostilizar",
    "hostilizabas": "hostilizar",
    "hostilizad": "hostilizar",
    "hostilizada": "hostilizar",
    "hostilizadas": "hostilizar",
    "hostilizado": "hostilizar",
    "hostilizados": "hostilizar",
    "hostilizamos": "hostilizar",
    "hostilizan": "hostilizar",
    "hostilizando": "hostilizar",
    "hostilizara": "hostilizar",
    "hostilizarais": "hostilizar",
    "hostilizaran": "hostilizar",
    "hostilizaras": "hostilizar",
    "hostilizare": "hostilizar",
    "hostilizareis": "hostilizar",
    "hostilizaremos": "hostilizar",
    "hostilizaren": "hostilizar",
    "hostilizares": "hostilizar",
    "hostilizaron": "hostilizar",
    "hostilizará": "hostilizar",
    "hostilizarán": "hostilizar",
    "hostilizarás": "hostilizar",
    "hostilizaré": "hostilizar",
    "hostilizaréis": "hostilizar",
    "hostilizaría": "hostilizar",
    "hostilizaríais": "hostilizar",
    "hostilizaríamos": "hostilizar",
    "hostilizarían": "hostilizar",
    "hostilizarías": "hostilizar",
    "hostilizas": "hostilizar",
    "hostilizase": "hostilizar",
    "hostilizaseis": "hostilizar",
    "hostilizasen": "hostilizar",
    "hostilizases": "hostilizar",
    "hostilizaste": "hostilizar",
    "hostilizasteis": "hostilizar",
    "hostilizo": "hostilizar",
    "hostilizábamos": "hostilizar",
    "hostilizáis": "hostilizar",
    "hostilizáramos": "hostilizar",
    "hostilizáremos": "hostilizar",
    "hostilizásemos": "hostilizar",
    "hostilizó": "hostilizar",
    "hosts": "host",
    "hot-dogs": "hot-dog",
    "hot-jazces": "hot-jazz",
    "hotelera": "hotelero",
    "hoteleras": "hotelero",
    "hoteleros": "hotelero",
    "hotelerías": "hotelería",
    "hoteles": "hotel",
    "hoteles-residencia": "hotel-residencia",
    "hotentota": "hotentote",
    "hotentotas": "hotentote",
    "hotentotes": "hotentote",
    "hovera": "hovero",
    "hoveras": "hovero",
    "hovercrafts": "hovercraft",
    "hoveros": "hovero",
    "howlitas": "howlita",
    "hoyadas": "hoyada",
    "hoyancas": "hoyanca",
    "hoyas": "hoya",
    "hoyos": "hoyo",
    "hoyosa": "hoyoso",
    "hoyosas": "hoyoso",
    "hoyosos": "hoyoso",
    "hoyuelas": "hoyuela",
    "hoyuelos": "hoyuelo",
    "hoza": "hozar",
    "hozaba": "hozar",
    "hozabais": "hozar",
    "hozaban": "hozar",
    "hozabas": "hozar",
    "hozad": "hozar",
    "hozada": "hozar",
    "hozadas": "hozar",
    "hozaderos": "hozadero",
    "hozado": "hozar",
    "hozadora": "hozador",
    "hozadoras": "hozador",
    "hozadores": "hozador",
    "hozados": "hozar",
    "hozaduras": "hozadura",
    "hozamos": "hozar",
    "hozan": "hozar",
    "hozando": "hozar",
    "hozara": "hozar",
    "hozarais": "hozar",
    "hozaran": "hozar",
    "hozaras": "hozar",
    "hozare": "hozar",
    "hozareis": "hozar",
    "hozaremos": "hozar",
    "hozaren": "hozar",
    "hozares": "hozar",
    "hozaron": "hozar",
    "hozará": "hozar",
    "hozarán": "hozar",
    "hozarás": "hozar",
    "hozaré": "hozar",
    "hozaréis": "hozar",
    "hozaría": "hozar",
    "hozaríais": "hozar",
    "hozaríamos": "hozar",
    "hozarían": "hozar",
    "hozarías": "hozar",
    "hozas": "hozar",
    "hozase": "hozar",
    "hozaseis": "hozar",
    "hozasen": "hozar",
    "hozases": "hozar",
    "hozaste": "hozar",
    "hozasteis": "hozar",
    "hozo": "hozar",
    "hozábamos": "hozar",
    "hozáis": "hozar",
    "hozáramos": "hozar",
    "hozáremos": "hozar",
    "hozásemos": "hozar",
    "hozó": "hozar",
    "hryvna": "hryvnia",
    "hryvnas": "hryvnia",
    "hryvnias": "hryvnia",
    "huacas": "huaca",
    "huaxtecos": "huaxteco",
    "hubaras": "hubara",
    "hube": "haber",
    "hubiera": "haber",
    "hubierais": "haber",
    "hubieran": "haber",
    "hubieras": "haber",
    "hubiere": "haber",
    "hubiereis": "haber",
    "hubieren": "haber",
    "hubieres": "haber",
    "hubieron": "haber",
    "hubiese": "haber",
    "hubieseis": "haber",
    "hubiesen": "haber",
    "hubieses": "haber",
    "hubimos": "haber",
    "hubiste": "haber",
    "hubisteis": "haber",
    "hubiéramos": "haber",
    "hubiéremos": "haber",
    "hubiésemos": "haber",
    "hubo": "haber",
    "huchas": "hucha",
    "huchea": "huchear",
    "hucheaba": "huchear",
    "hucheabais": "huchear",
    "hucheaban": "huchear",
    "hucheabas": "huchear",
    "huchead": "huchear",
    "hucheada": "huchear",
    "hucheadas": "huchear",
    "hucheado": "huchear",
    "hucheados": "huchear",
    "hucheamos": "huchear",
    "huchean": "huchear",
    "hucheando": "huchear",
    "hucheara": "huchear",
    "huchearais": "huchear",
    "huchearan": "huchear",
    "huchearas": "huchear",
    "hucheare": "huchear",
    "hucheareis": "huchear",
    "huchearemos": "huchear",
    "huchearen": "huchear",
    "hucheares": "huchear",
    "huchearon": "huchear",
    "hucheará": "huchear",
    "huchearán": "huchear",
    "huchearás": "huchear",
    "huchearé": "huchear",
    "huchearéis": "huchear",
    "huchearía": "huchear",
    "huchearíais": "huchear",
    "huchearíamos": "huchear",
    "huchearían": "huchear",
    "huchearías": "huchear",
    "hucheas": "huchear",
    "huchease": "huchear",
    "hucheaseis": "huchear",
    "hucheasen": "huchear",
    "hucheases": "huchear",
    "hucheaste": "huchear",
    "hucheasteis": "huchear",
    "huchee": "huchear",
    "hucheemos": "huchear",
    "hucheen": "huchear",
    "huchees": "huchear",
    "hucheo": "huchear",
    "hucheábamos": "huchear",
    "hucheáis": "huchear",
    "hucheáramos": "huchear",
    "hucheáremos": "huchear",
    "hucheásemos": "huchear",
    "hucheé": "huchear",
    "hucheéis": "huchear",
    "hucheó": "huchear",
    "hudsonienses": "hudsoniense",
    "huebras": "huebra",
    "huebrera": "huebrero",
    "huebreras": "huebrero",
    "huebreros": "huebrero",
    "hueca": "hueco",
    "huecas": "hueco",
    "huecograbados": "huecograbado",
    "huecos": "hueco",
    "huela": "oler",
    "huelan": "oler",
    "huelas": "oler",
    "huele": "oler",
    "huelen": "oler",
    "hueles": "oler",
    "huelga": "holgar",
    "huelgan": "holgar",
    "huelgas": "huelga",
    "huelgo": "holgar",
    "huelgos": "huelgo",
    "huelgue": "holgar",
    "huelguen": "holgar",
    "huelgues": "holgar",
    "huelguistas": "huelguista",
    "huelguística": "huelguístico",
    "huelguísticas": "huelguístico",
    "huelguísticos": "huelguístico",
    "huella": "hollar",
    "huellan": "hollar",
    "huellas": "huella",
    "huelle": "hollar",
    "huellen": "hollar",
    "huelles": "hollar",
    "huello": "hollar",
    "huellos": "huello",
    "huelo": "oler",
    "huelveña": "huelveño",
    "huelveñas": "huelveño",
    "huelveños": "huelveño",
    "huera": "huero",
    "hueras": "huero",
    "hueros": "huero",
    "huertana": "huertano",
    "huertanas": "huertano",
    "huertanos": "huertano",
    "huertas": "huerta",
    "huertos": "huerto",
    "huesas": "huesa",
    "huesecillos": "huesecillo",
    "huesos": "hueso",
    "huesosa": "huesoso",
    "huesosas": "huesoso",
    "huesosos": "huesoso",
    "huestes": "hueste",
    "huesuda": "huesudo",
    "huesudas": "huesudo",
    "huesudos": "huesudo",
    "hueva": "huevar",
    "huevaba": "huevar",
    "huevabais": "huevar",
    "huevaban": "huevar",
    "huevabas": "huevar",
    "huevad": "huevar",
    "huevada": "huevar",
    "huevadas": "huevar",
    "huevado": "huevar",
    "huevados": "huevar",
    "huevamos": "huevar",
    "huevan": "huevar",
    "huevando": "huevar",
    "huevara": "huevar",
    "huevarais": "huevar",
    "huevaran": "huevar",
    "huevaras": "huevar",
    "huevare": "huevar",
    "huevareis": "huevar",
    "huevaremos": "huevar",
    "huevaren": "huevar",
    "huevares": "huevar",
    "huevaron": "huevar",
    "huevará": "huevar",
    "huevarán": "huevar",
    "huevarás": "huevar",
    "huevaré": "huevar",
    "huevaréis": "huevar",
    "huevaría": "huevar",
    "huevaríais": "huevar",
    "huevaríamos": "huevar",
    "huevarían": "huevar",
    "huevarías": "huevar",
    "huevas": "huevar",
    "huevase": "huevar",
    "huevaseis": "huevar",
    "huevasen": "huevar",
    "huevases": "huevar",
    "huevaste": "huevar",
    "huevasteis": "huevar",
    "hueve": "huevar",
    "huevemos": "huevar",
    "hueven": "huevar",
    "hueveras": "huevera",
    "hueveros": "huevero",
    "hueverías": "huevería",
    "hueves": "huevar",
    "huevo": "huevar",
    "huevona": "huevón",
    "huevonas": "huevón",
    "huevones": "huevón",
    "huevos": "huevo",
    "huevábamos": "huevar",
    "hueváis": "huevar",
    "hueváramos": "huevar",
    "hueváremos": "huevar",
    "huevásemos": "huevar",
    "huevé": "huevar",
    "huevéis": "huevar",
    "huevó": "huevar",
    "hugonota": "hugonote",
    "hugonotas": "hugonote",
    "hugonotes": "hugonote",
    "huid": "huir",
    "huida": "huir",
    "huidas": "huir",
    "huidera": "huidero",
    "huideras": "huidero",
    "huideros": "huidero",
    "huidiza": "huidizo",
    "huidizas": "huidizo",
    "huidizos": "huidizo",
    "huido": "huir",
    "huidora": "huidor",
    "huidoras": "huidor",
    "huidores": "huidor",
    "huidos": "huir",
    "huimos": "huir",
    "huinganes": "huingán",
    "huiremos": "huir",
    "huirá": "huir",
    "huirán": "huir",
    "huirás": "huir",
    "huiré": "huir",
    "huiréis": "huir",
    "huiría": "huir",
    "huiríais": "huir",
    "huiríamos": "huir",
    "huirían": "huir",
    "huirías": "huir",
    "huiste": "huir",
    "huisteis": "huir",
    "hukes": "huk",
    "huks": "huk",
    "hulera": "hulero",
    "huleras": "hulero",
    "huleros": "hulero",
    "hules": "hule",
    "hullas": "hulla",
    "hullera": "hullero",
    "hulleras": "hullero",
    "hulleros": "hullero",
    "humadas": "humada",
    "humana": "humano",
    "humanaba": "humanar",
    "humanabais": "humanar",
    "humanaban": "humanar",
    "humanabas": "humanar",
    "humanad": "humanar",
    "humanada": "humanar",
    "humanadas": "humanar",
    "humanado": "humanar",
    "humanados": "humanar",
    "humanamos": "humanar",
    "humanan": "humanar",
    "humanando": "humanar",
    "humanara": "humanar",
    "humanarais": "humanar",
    "humanaran": "humanar",
    "humanaras": "humanar",
    "humanare": "humanar",
    "humanareis": "humanar",
    "humanaremos": "humanar",
    "humanaren": "humanar",
    "humanares": "humanar",
    "humanaron": "humanar",
    "humanará": "humanar",
    "humanarán": "humanar",
    "humanarás": "humanar",
    "humanaré": "humanar",
    "humanaréis": "humanar",
    "humanaría": "humanar",
    "humanaríais": "humanar",
    "humanaríamos": "humanar",
    "humanarían": "humanar",
    "humanarías": "humanar",
    "humanas": "humano",
    "humanase": "humanar",
    "humanaseis": "humanar",
    "humanasen": "humanar",
    "humanases": "humanar",
    "humanaste": "humanar",
    "humanasteis": "humanar",
    "humane": "humanar",
    "humanemos": "humanar",
    "humanen": "humanar",
    "humanes": "humanar",
    "humanice": "humanizar",
    "humanicemos": "humanizar",
    "humanicen": "humanizar",
    "humanices": "humanizar",
    "humanicé": "humanizar",
    "humanicéis": "humanizar",
    "humanidades": "humanidad",
    "humanismos": "humanismo",
    "humanistas": "humanista",
    "humanitaria": "humanitario",
    "humanitarias": "humanitario",
    "humanitarios": "humanitario",
    "humanitarismos": "humanitarismo",
    "humaniza": "humanizar",
    "humanizaba": "humanizar",
    "humanizabais": "humanizar",
    "humanizaban": "humanizar",
    "humanizabas": "humanizar",
    "humanizaciones": "humanización",
    "humanizad": "humanizar",
    "humanizada": "humanizar",
    "humanizadas": "humanizar",
    "humanizado": "humanizar",
    "humanizadora": "humanizador",
    "humanizadoras": "humanizador",
    "humanizadores": "humanizador",
    "humanizados": "humanizar",
    "humanizamos": "humanizar",
    "humanizan": "humanizar",
    "humanizando": "humanizar",
    "humanizara": "humanizar",
    "humanizarais": "humanizar",
    "humanizaran": "humanizar",
    "humanizaras": "humanizar",
    "humanizare": "humanizar",
    "humanizareis": "humanizar",
    "humanizaremos": "humanizar",
    "humanizaren": "humanizar",
    "humanizares": "humanizar",
    "humanizaron": "humanizar",
    "humanizará": "humanizar",
    "humanizarán": "humanizar",
    "humanizarás": "humanizar",
    "humanizaré": "humanizar",
    "humanizaréis": "humanizar",
    "humanizaría": "humanizar",
    "humanizaríais": "humanizar",
    "humanizaríamos": "humanizar",
    "humanizarían": "humanizar",
    "humanizarías": "humanizar",
    "humanizas": "humanizar",
    "humanizase": "humanizar",
    "humanizaseis": "humanizar",
    "humanizasen": "humanizar",
    "humanizases": "humanizar",
    "humanizaste": "humanizar",
    "humanizasteis": "humanizar",
    "humanizo": "humanizar",
    "humanizábamos": "humanizar",
    "humanizáis": "humanizar",
    "humanizáramos": "humanizar",
    "humanizáremos": "humanizar",
    "humanizásemos": "humanizar",
    "humanizó": "humanizar",
    "humano": "humanar",
    "humanoides": "humanoide",
    "humanos": "humano",
    "humanábamos": "humanar",
    "humanáis": "humanar",
    "humanáramos": "humanar",
    "humanáremos": "humanar",
    "humanásemos": "humanar",
    "humané": "humanar",
    "humanéis": "humanar",
    "humanística": "humanístico",
    "humanísticas": "humanístico",
    "humanísticos": "humanístico",
    "humanó": "humanar",
    "humaradas": "humarada",
    "humarazos": "humarazo",
    "humaredas": "humareda",
    "humazos": "humazo",
    "humea": "humear",
    "humeaba": "humear",
    "humeabais": "humear",
    "humeaban": "humear",
    "humeabas": "humear",
    "humead": "humear",
    "humeada": "humear",
    "humeadas": "humear",
    "humeado": "humear",
    "humeados": "humear",
    "humeamos": "humear",
    "humean": "humear",
    "humeando": "humear",
    "humeantes": "humeante",
    "humeara": "humear",
    "humearais": "humear",
    "humearan": "humear",
    "humearas": "humear",
    "humeare": "humear",
    "humeareis": "humear",
    "humearemos": "humear",
    "humearen": "humear",
    "humeares": "humear",
    "humearon": "humear",
    "humeará": "humear",
    "humearán": "humear",
    "humearás": "humear",
    "humearé": "humear",
    "humearéis": "humear",
    "humearía": "humear",
    "humearíais": "humear",
    "humearíamos": "humear",
    "humearían": "humear",
    "humearías": "humear",
    "humeas": "humear",
    "humease": "humear",
    "humeaseis": "humear",
    "humeasen": "humear",
    "humeases": "humear",
    "humeaste": "humear",
    "humeasteis": "humear",
    "humecta": "humectar",
    "humectaba": "humectar",
    "humectabais": "humectar",
    "humectaban": "humectar",
    "humectabas": "humectar",
    "humectabilidades": "humectabilidad",
    "humectaciones": "humectación",
    "humectad": "humectar",
    "humectada": "humectar",
    "humectadas": "humectar",
    "humectado": "humectar",
    "humectadora": "humectador",
    "humectadoras": "humectador",
    "humectadores": "humectador",
    "humectados": "humectar",
    "humectamos": "humectar",
    "humectan": "humectar",
    "humectando": "humectar",
    "humectantes": "humectante",
    "humectara": "humectar",
    "humectarais": "humectar",
    "humectaran": "humectar",
    "humectaras": "humectar",
    "humectare": "humectar",
    "humectareis": "humectar",
    "humectaremos": "humectar",
    "humectaren": "humectar",
    "humectares": "humectar",
    "humectaron": "humectar",
    "humectará": "humectar",
    "humectarán": "humectar",
    "humectarás": "humectar",
    "humectaré": "humectar",
    "humectaréis": "humectar",
    "humectaría": "humectar",
    "humectaríais": "humectar",
    "humectaríamos": "humectar",
    "humectarían": "humectar",
    "humectarías": "humectar",
    "humectas": "humectar",
    "humectase": "humectar",
    "humectaseis": "humectar",
    "humectasen": "humectar",
    "humectases": "humectar",
    "humectaste": "humectar",
    "humectasteis": "humectar",
    "humectativa": "humectativo",
    "humectativas": "humectativo",
    "humectativos": "humectativo",
    "humecte": "humectar",
    "humectemos": "humectar",
    "humecten": "humectar",
    "humectes": "humectar",
    "humecto": "humectar",
    "humectábamos": "humectar",
    "humectáis": "humectar",
    "humectáramos": "humectar",
    "humectáremos": "humectar",
    "humectásemos": "humectar",
    "humecté": "humectar",
    "humectéis": "humectar",
    "humectó": "humectar",
    "humedades": "humedad",
    "humedales": "humedal",
    "humedece": "humedecer",
    "humedeced": "humedecer",
    "humedecedores": "humedecedor",
    "humedecemos": "humedecer",
    "humedecen": "humedecer",
    "humedeceremos": "humedecer",
    "humedecerá": "humedecer",
    "humedecerán": "humedecer",
    "humedecerás": "humedecer",
    "humedeceré": "humedecer",
    "humedeceréis": "humedecer",
    "humedecería": "humedecer",
    "humedeceríais": "humedecer",
    "humedeceríamos": "humedecer",
    "humedecerían": "humedecer",
    "humedecerías": "humedecer",
    "humedeces": "humedecer",
    "humedecida": "humedecer",
    "humedecidas": "humedecer",
    "humedecido": "humedecer",
    "humedecidos": "humedecer",
    "humedeciendo": "humedecer",
    "humedeciera": "humedecer",
    "humedecierais": "humedecer",
    "humedecieran": "humedecer",
    "humedecieras": "humedecer",
    "humedeciere": "humedecer",
    "humedeciereis": "humedecer",
    "humedecieren": "humedecer",
    "humedecieres": "humedecer",
    "humedecieron": "humedecer",
    "humedeciese": "humedecer",
    "humedecieseis": "humedecer",
    "humedeciesen": "humedecer",
    "humedecieses": "humedecer",
    "humedecimientos": "humedecimiento",
    "humedecimos": "humedecer",
    "humedeciste": "humedecer",
    "humedecisteis": "humedecer",
    "humedeciéramos": "humedecer",
    "humedeciéremos": "humedecer",
    "humedeciésemos": "humedecer",
    "humedeció": "humedecer",
    "humedecéis": "humedecer",
    "humedecí": "humedecer",
    "humedecía": "humedecer",
    "humedecíais": "humedecer",
    "humedecíamos": "humedecer",
    "humedecían": "humedecer",
    "humedecías": "humedecer",
    "humedezca": "humedecer",
    "humedezcamos": "humedecer",
    "humedezcan": "humedecer",
    "humedezcas": "humedecer",
    "humedezco": "humedecer",
    "humedezcáis": "humedecer",
    "humee": "humear",
    "humeemos": "humear",
    "humeen": "humear",
    "humees": "humear",
    "humeo": "humear",
    "humerales": "humeral",
    "humerocubitales": "humerocubital",
    "humerorradiales": "humerorradial",
    "humeábamos": "humear",
    "humeáis": "humear",
    "humeáramos": "humear",
    "humeáremos": "humear",
    "humeásemos": "humear",
    "humeé": "humear",
    "humeéis": "humear",
    "humeó": "humear",
    "humidificaciones": "humidificación",
    "humidificadores": "humidificador",
    "humificaciones": "humificación",
    "humilda": "humilde",
    "humildades": "humildad",
    "humildes": "humilde",
    "humilla": "humillar",
    "humillaba": "humillar",
    "humillabais": "humillar",
    "humillaban": "humillar",
    "humillabas": "humillar",
    "humillaciones": "humillación",
    "humillad": "humillar",
    "humillada": "humillar",
    "humilladas": "humillar",
    "humilladeros": "humilladero",
    "humillado": "humillar",
    "humilladora": "humillador",
    "humilladoras": "humillador",
    "humilladores": "humillador",
    "humillados": "humillar",
    "humillamos": "humillar",
    "humillan": "humillar",
    "humillando": "humillar",
    "humillantes": "humillante",
    "humillara": "humillar",
    "humillarais": "humillar",
    "humillaran": "humillar",
    "humillaras": "humillar",
    "humillare": "humillar",
    "humillareis": "humillar",
    "humillaremos": "humillar",
    "humillaren": "humillar",
    "humillares": "humillar",
    "humillaron": "humillar",
    "humillará": "humillar",
    "humillarán": "humillar",
    "humillarás": "humillar",
    "humillaré": "humillar",
    "humillaréis": "humillar",
    "humillaría": "humillar",
    "humillaríais": "humillar",
    "humillaríamos": "humillar",
    "humillarían": "humillar",
    "humillarías": "humillar",
    "humillas": "humillar",
    "humillase": "humillar",
    "humillaseis": "humillar",
    "humillasen": "humillar",
    "humillases": "humillar",
    "humillaste": "humillar",
    "humillasteis": "humillar",
    "humille": "humillar",
    "humillemos": "humillar",
    "humillen": "humillar",
    "humilles": "humillar",
    "humillo": "humillar",
    "humillos": "humillo",
    "humillábamos": "humillar",
    "humilláis": "humillar",
    "humilláramos": "humillar",
    "humilláremos": "humillar",
    "humillásemos": "humillar",
    "humillé": "humillar",
    "humilléis": "humillar",
    "humilló": "humillar",
    "humitas": "humita",
    "humorada": "humorado",
    "humoradas": "humorado",
    "humorados": "humorado",
    "humorales": "humoral",
    "humores": "humor",
    "humorismos": "humorismo",
    "humoristas": "humorista",
    "humorosa": "humoroso",
    "humorosas": "humoroso",
    "humorosidades": "humorosidad",
    "humorosos": "humoroso",
    "humorística": "humorístico",
    "humorísticas": "humorístico",
    "humorísticos": "humorístico",
    "humos": "humo",
    "humosa": "humoso",
    "humosas": "humoso",
    "humosidades": "humosidad",
    "humosos": "humoso",
    "humícolas": "humícola",
    "humífera": "humífero",
    "humíferas": "humífero",
    "humíferos": "humífero",
    "hunda": "hundir",
    "hundamos": "hundir",
    "hundan": "hundir",
    "hundas": "hundir",
    "hunde": "hundir",
    "hunden": "hundir",
    "hundes": "hundir",
    "hundibles": "hundible",
    "hundid": "hundir",
    "hundida": "hundir",
    "hundidas": "hundir",
    "hundido": "hundir",
    "hundidos": "hundir",
    "hundiendo": "hundir",
    "hundiera": "hundir",
    "hundierais": "hundir",
    "hundieran": "hundir",
    "hundieras": "hundir",
    "hundiere": "hundir",
    "hundiereis": "hundir",
    "hundieren": "hundir",
    "hundieres": "hundir",
    "hundieron": "hundir",
    "hundiese": "hundir",
    "hundieseis": "hundir",
    "hundiesen": "hundir",
    "hundieses": "hundir",
    "hundimientos": "hundimiento",
    "hundimos": "hundir",
    "hundiremos": "hundir",
    "hundirá": "hundir",
    "hundirán": "hundir",
    "hundirás": "hundir",
    "hundiré": "hundir",
    "hundiréis": "hundir",
    "hundiría": "hundir",
    "hundiríais": "hundir",
    "hundiríamos": "hundir",
    "hundirían": "hundir",
    "hundirías": "hundir",
    "hundiste": "hundir",
    "hundisteis": "hundir",
    "hundiéramos": "hundir",
    "hundiéremos": "hundir",
    "hundiésemos": "hundir",
    "hundió": "hundir",
    "hundo": "hundir",
    "hundáis": "hundir",
    "hundí": "hundir",
    "hundía": "hundir",
    "hundíais": "hundir",
    "hundíamos": "hundir",
    "hundían": "hundir",
    "hundías": "hundir",
    "hundís": "hundir",
    "hunos": "huno",
    "hupes": "hupe",
    "huracanada": "huracanado",
    "huracanadas": "huracanado",
    "huracanados": "huracanado",
    "huracanes": "huracán",
    "huras": "hura",
    "huraña": "huraño",
    "hurañas": "huraño",
    "huraños": "huraño",
    "hurañías": "hurañía",
    "hureaulitas": "hureaulita",
    "hurga": "hurgar",
    "hurgaba": "hurgar",
    "hurgabais": "hurgar",
    "hurgaban": "hurgar",
    "hurgabas": "hurgar",
    "hurgad": "hurgar",
    "hurgada": "hurgar",
    "hurgadas": "hurgar",
    "hurgado": "hurgar",
    "hurgadora": "hurgador",
    "hurgadoras": "hurgador",
    "hurgadores": "hurgador",
    "hurgados": "hurgar",
    "hurgamientos": "hurgamiento",
    "hurgamos": "hurgar",
    "hurgan": "hurgar",
    "hurgando": "hurgar",
    "hurgantes": "hurgante",
    "hurgara": "hurgar",
    "hurgarais": "hurgar",
    "hurgaran": "hurgar",
    "hurgaras": "hurgar",
    "hurgare": "hurgar",
    "hurgareis": "hurgar",
    "hurgaremos": "hurgar",
    "hurgaren": "hurgar",
    "hurgares": "hurgar",
    "hurgaron": "hurgar",
    "hurgará": "hurgar",
    "hurgarán": "hurgar",
    "hurgarás": "hurgar",
    "hurgaré": "hurgar",
    "hurgaréis": "hurgar",
    "hurgaría": "hurgar",
    "hurgaríais": "hurgar",
    "hurgaríamos": "hurgar",
    "hurgarían": "hurgar",
    "hurgarías": "hurgar",
    "hurgas": "hurgar",
    "hurgase": "hurgar",
    "hurgaseis": "hurgar",
    "hurgasen": "hurgar",
    "hurgases": "hurgar",
    "hurgaste": "hurgar",
    "hurgasteis": "hurgar",
    "hurgo": "hurgar",
    "hurgona": "hurgón",
    "hurgonadas": "hurgonada",
    "hurgonas": "hurgón",
    "hurgonazos": "hurgonazo",
    "hurgonea": "hurgonear",
    "hurgoneaba": "hurgonear",
    "hurgoneabais": "hurgonear",
    "hurgoneaban": "hurgonear",
    "hurgoneabas": "hurgonear",
    "hurgonead": "hurgonear",
    "hurgoneada": "hurgonear",
    "hurgoneadas": "hurgonear",
    "hurgoneado": "hurgonear",
    "hurgoneados": "hurgonear",
    "hurgoneamos": "hurgonear",
    "hurgonean": "hurgonear",
    "hurgoneando": "hurgonear",
    "hurgoneara": "hurgonear",
    "hurgonearais": "hurgonear",
    "hurgonearan": "hurgonear",
    "hurgonearas": "hurgonear",
    "hurgoneare": "hurgonear",
    "hurgoneareis": "hurgonear",
    "hurgonearemos": "hurgonear",
    "hurgonearen": "hurgonear",
    "hurgoneares": "hurgonear",
    "hurgonearon": "hurgonear",
    "hurgoneará": "hurgonear",
    "hurgonearán": "hurgonear",
    "hurgonearás": "hurgonear",
    "hurgonearé": "hurgonear",
    "hurgonearéis": "hurgonear",
    "hurgonearía": "hurgonear",
    "hurgonearíais": "hurgonear",
    "hurgonearíamos": "hurgonear",
    "hurgonearían": "hurgonear",
    "hurgonearías": "hurgonear",
    "hurgoneas": "hurgonear",
    "hurgonease": "hurgonear",
    "hurgoneaseis": "hurgonear",
    "hurgoneasen": "hurgonear",
    "hurgoneases": "hurgonear",
    "hurgoneaste": "hurgonear",
    "hurgoneasteis": "hurgonear",
    "hurgonee": "hurgonear",
    "hurgoneemos": "hurgonear",
    "hurgoneen": "hurgonear",
    "hurgonees": "hurgonear",
    "hurgoneo": "hurgonear",
    "hurgonera": "hurgonero",
    "hurgoneras": "hurgonero",
    "hurgoneros": "hurgonero",
    "hurgones": "hurgón",
    "hurgoneábamos": "hurgonear",
    "hurgoneáis": "hurgonear",
    "hurgoneáramos": "hurgonear",
    "hurgoneáremos": "hurgonear",
    "hurgoneásemos": "hurgonear",
    "hurgoneé": "hurgonear",
    "hurgoneéis": "hurgonear",
    "hurgoneó": "hurgonear",
    "hurgue": "hurgar",
    "hurguemos": "hurgar",
    "hurguen": "hurgar",
    "hurgues": "hurgar",
    "hurguetea": "hurguetear",
    "hurgueteaba": "hurguetear",
    "hurgueteabais": "hurguetear",
    "hurgueteaban": "hurguetear",
    "hurgueteabas": "hurguetear",
    "hurguetead": "hurguetear",
    "hurgueteada": "hurguetear",
    "hurgueteadas": "hurguetear",
    "hurgueteado": "hurguetear",
    "hurgueteados": "hurguetear",
    "hurgueteamos": "hurguetear",
    "hurguetean": "hurguetear",
    "hurgueteando": "hurguetear",
    "hurgueteara": "hurguetear",
    "hurguetearais": "hurguetear",
    "hurguetearan": "hurguetear",
    "hurguetearas": "hurguetear",
    "hurgueteare": "hurguetear",
    "hurgueteareis": "hurguetear",
    "hurguetearemos": "hurguetear",
    "hurguetearen": "hurguetear",
    "hurgueteares": "hurguetear",
    "hurguetearon": "hurguetear",
    "hurgueteará": "hurguetear",
    "hurguetearán": "hurguetear",
    "hurguetearás": "hurguetear",
    "hurguetearé": "hurguetear",
    "hurguetearéis": "hurguetear",
    "hurguetearía": "hurguetear",
    "hurguetearíais": "hurguetear",
    "hurguetearíamos": "hurguetear",
    "hurguetearían": "hurguetear",
    "hurguetearías": "hurguetear",
    "hurgueteas": "hurguetear",
    "hurguetease": "hurguetear",
    "hurgueteaseis": "hurguetear",
    "hurgueteasen": "hurguetear",
    "hurgueteases": "hurguetear",
    "hurgueteaste": "hurguetear",
    "hurgueteasteis": "hurguetear",
    "hurguetee": "hurguetear",
    "hurgueteemos": "hurguetear",
    "hurgueteen": "hurguetear",
    "hurguetees": "hurguetear",
    "hurgueteo": "hurguetear",
    "hurgueteábamos": "hurguetear",
    "hurgueteáis": "hurguetear",
    "hurgueteáramos": "hurguetear",
    "hurgueteáremos": "hurguetear",
    "hurgueteásemos": "hurguetear",
    "hurgueteé": "hurguetear",
    "hurgueteéis": "hurguetear",
    "hurgueteó": "hurguetear",
    "hurgué": "hurgar",
    "hurguéis": "hurgar",
    "hurgábamos": "hurgar",
    "hurgáis": "hurgar",
    "hurgáramos": "hurgar",
    "hurgáremos": "hurgar",
    "hurgásemos": "hurgar",
    "hurgó": "hurgar",
    "huronea": "huronear",
    "huroneaba": "huronear",
    "huroneabais": "huronear",
    "huroneaban": "huronear",
    "huroneabas": "huronear",
    "huronead": "huronear",
    "huroneada": "huronear",
    "huroneadas": "huronear",
    "huroneado": "huronear",
    "huroneados": "huronear",
    "huroneamos": "huronear",
    "huronean": "huronear",
    "huroneando": "huronear",
    "huroneara": "huronear",
    "huronearais": "huronear",
    "huronearan": "huronear",
    "huronearas": "huronear",
    "huroneare": "huronear",
    "huroneareis": "huronear",
    "huronearemos": "huronear",
    "huronearen": "huronear",
    "huroneares": "huronear",
    "huronearon": "huronear",
    "huroneará": "huronear",
    "huronearán": "huronear",
    "huronearás": "huronear",
    "huronearé": "huronear",
    "huronearéis": "huronear",
    "huronearía": "huronear",
    "huronearíais": "huronear",
    "huronearíamos": "huronear",
    "huronearían": "huronear",
    "huronearías": "huronear",
    "huroneas": "huronear",
    "huronease": "huronear",
    "huroneaseis": "huronear",
    "huroneasen": "huronear",
    "huroneases": "huronear",
    "huroneaste": "huronear",
    "huroneasteis": "huronear",
    "huronee": "huronear",
    "huroneemos": "huronear",
    "huroneen": "huronear",
    "huronees": "huronear",
    "huroneo": "huronear",
    "huroneos": "huroneo",
    "huronera": "huronero",
    "huroneras": "huronero",
    "huroneros": "huronero",
    "hurones": "hurón",
    "huroneábamos": "huronear",
    "huroneáis": "huronear",
    "huroneáramos": "huronear",
    "huroneáremos": "huronear",
    "huroneásemos": "huronear",
    "huroneé": "huronear",
    "huroneéis": "huronear",
    "huroneó": "huronear",
    "hurta": "hurtar",
    "hurtaba": "hurtar",
    "hurtabais": "hurtar",
    "hurtaban": "hurtar",
    "hurtabas": "hurtar",
    "hurtad": "hurtar",
    "hurtada": "hurtar",
    "hurtadas": "hurtar",
    "hurtado": "hurtar",
    "hurtadora": "hurtador",
    "hurtadoras": "hurtador",
    "hurtadores": "hurtador",
    "hurtados": "hurtar",
    "hurtamos": "hurtar",
    "hurtan": "hurtar",
    "hurtando": "hurtar",
    "hurtara": "hurtar",
    "hurtarais": "hurtar",
    "hurtaran": "hurtar",
    "hurtaras": "hurtar",
    "hurtare": "hurtar",
    "hurtareis": "hurtar",
    "hurtaremos": "hurtar",
    "hurtaren": "hurtar",
    "hurtares": "hurtar",
    "hurtaron": "hurtar",
    "hurtará": "hurtar",
    "hurtarán": "hurtar",
    "hurtarás": "hurtar",
    "hurtaré": "hurtar",
    "hurtaréis": "hurtar",
    "hurtaría": "hurtar",
    "hurtaríais": "hurtar",
    "hurtaríamos": "hurtar",
    "hurtarían": "hurtar",
    "hurtarías": "hurtar",
    "hurtas": "hurtar",
    "hurtase": "hurtar",
    "hurtaseis": "hurtar",
    "hurtasen": "hurtar",
    "hurtases": "hurtar",
    "hurtaste": "hurtar",
    "hurtasteis": "hurtar",
    "hurte": "hurtar",
    "hurtemos": "hurtar",
    "hurten": "hurtar",
    "hurtes": "hurtar",
    "hurto": "hurtar",
    "hurtos": "hurto",
    "hurtábamos": "hurtar",
    "hurtáis": "hurtar",
    "hurtáramos": "hurtar",
    "hurtáremos": "hurtar",
    "hurtásemos": "hurtar",
    "hurté": "hurtar",
    "hurtéis": "hurtar",
    "hurtó": "hurtar",
    "huríes": "hurí",
    "hurís": "hurí",
    "husadas": "husada",
    "husaradas": "husarada",
    "huseros": "husero",
    "husillera": "husillero",
    "husilleras": "husillero",
    "husilleros": "husillero",
    "husillos": "husillo",
    "husitas": "husita",
    "husmea": "husmear",
    "husmeaba": "husmear",
    "husmeabais": "husmear",
    "husmeaban": "husmear",
    "husmeabas": "husmear",
    "husmead": "husmear",
    "husmeada": "husmear",
    "husmeadas": "husmear",
    "husmeado": "husmear",
    "husmeadora": "husmeador",
    "husmeadoras": "husmeador",
    "husmeadores": "husmeador",
    "husmeados": "husmear",
    "husmeamos": "husmear",
    "husmean": "husmear",
    "husmeando": "husmear",
    "husmeara": "husmear",
    "husmearais": "husmear",
    "husmearan": "husmear",
    "husmearas": "husmear",
    "husmeare": "husmear",
    "husmeareis": "husmear",
    "husmearemos": "husmear",
    "husmearen": "husmear",
    "husmeares": "husmear",
    "husmearon": "husmear",
    "husmeará": "husmear",
    "husmearán": "husmear",
    "husmearás": "husmear",
    "husmearé": "husmear",
    "husmearéis": "husmear",
    "husmearía": "husmear",
    "husmearíais": "husmear",
    "husmearíamos": "husmear",
    "husmearían": "husmear",
    "husmearías": "husmear",
    "husmeas": "husmear",
    "husmease": "husmear",
    "husmeaseis": "husmear",
    "husmeasen": "husmear",
    "husmeases": "husmear",
    "husmeaste": "husmear",
    "husmeasteis": "husmear",
    "husmee": "husmear",
    "husmeemos": "husmear",
    "husmeen": "husmear",
    "husmees": "husmear",
    "husmeo": "husmear",
    "husmeos": "husmeo",
    "husmeábamos": "husmear",
    "husmeáis": "husmear",
    "husmeáramos": "husmear",
    "husmeáremos": "husmear",
    "husmeásemos": "husmear",
    "husmeé": "husmear",
    "husmeéis": "husmear",
    "husmeó": "husmear",
    "husmos": "husmo",
    "husos": "huso",
    "hutas": "huta",
    "hutus": "hutu",
    "huya": "huir",
    "huyamos": "huir",
    "huyan": "huir",
    "huyas": "huir",
    "huye": "huir",
    "huyen": "huir",
    "huyendo": "huir",
    "huyera": "huir",
    "huyerais": "huir",
    "huyeran": "huir",
    "huyeras": "huir",
    "huyere": "huir",
    "huyereis": "huir",
    "huyeren": "huir",
    "huyeres": "huir",
    "huyeron": "huir",
    "huyes": "huir",
    "huyese": "huir",
    "huyeseis": "huir",
    "huyesen": "huir",
    "huyeses": "huir",
    "huyo": "huir",
    "huyáis": "huir",
    "huyéramos": "huir",
    "huyéremos": "huir",
    "huyésemos": "huir",
    "huyó": "huir",
    "huélfagos": "huélfago",
    "huérfana": "huérfano",
    "huérfanas": "huérfano",
    "huérfanos": "huérfano",
    "huéspedes": "huésped",
    "huí": "huir",
    "huía": "huir",
    "huíais": "huir",
    "huíamos": "huir",
    "huían": "huir",
    "huías": "huir",
    "huís": "huir",
    "hábiles": "hábil",
    "hábitats": "hábitat",
    "hábitos": "hábito",
    "hálitos": "hálito",
    "hámagos": "hámago",
    "hámsterses": "hámster",
    "háplica": "háplico",
    "háplicas": "háplico",
    "háplicos": "háplico",
    "héctica": "héctico",
    "hécticas": "héctico",
    "hécticos": "héctico",
    "hégiras": "hégira",
    "hélices": "hélice",
    "hélixes": "hélix",
    "hémica": "hémico",
    "hémicas": "hémico",
    "hémicos": "hémico",
    "hépares": "hépar",
    "héroes": "héroe",
    "hérula": "hérulo",
    "hérulas": "hérulo",
    "hérulos": "hérulo",
    "hética": "hético",
    "héticas": "hético",
    "héticos": "hético",
    "híbrida": "híbrido",
    "híbridas": "híbrido",
    "híbridos": "híbrido",
    "hídrica": "hídrico",
    "hídricas": "hídrico",
    "hídricos": "hídrico",
    "hígados": "hígado",
    "hímenes": "himen",
    "hímnica": "hímnico",
    "hímnicas": "hímnico",
    "hímnicos": "hímnico",
    "hípica": "hípico",
    "hípicas": "hípico",
    "hípicos": "hípico",
    "híspida": "híspido",
    "híspidas": "híspido",
    "híspidos": "híspido",
    "hística": "hístico",
    "hísticas": "hístico",
    "hísticos": "hístico",
    "hóldings": "hólding",
    "hórreos": "hórreo",
    "húmeda": "húmedo",
    "húmedas": "húmedo",
    "húmedos": "húmedo",
    "húmeros": "húmero",
    "húmica": "húmico",
    "húmicas": "húmico",
    "húmicos": "húmico",
    "húngara": "húngaro",
    "húngaras": "húngaro",
    "húngaros": "húngaro",
    "húsares": "húsar",
    "iba": "ir",
    "ibais": "ir",
    "iban": "ir",
    "ibas": "ir",
    "ibera": "íbero",
    "iberas": "íbero",
    "iberia": "iberio",
    "iberias": "iberio",
    "iberios": "iberio",
    "iberismos": "iberismo",
    "iberoamericana": "iberoamericano",
    "iberoamericanas": "iberoamericano",
    "iberoamericanos": "iberoamericano",
    "iberos": "íbero",
    "ibicenca": "ibicenco",
    "ibicencas": "ibicenco",
    "ibicencos": "ibicenco",
    "ibuprofenes": "ibuprofén",
    "ibuprofenos": "ibuprofeno",
    "ibérica": "ibérico",
    "ibéricas": "ibérico",
    "ibéricos": "ibérico",
    "ibídem": "ibidem",
    "ice": "izar",
    "icebergs": "iceberg",
    "icefields": "icefield",
    "icemos": "izar",
    "icen": "izar",
    "ices": "izar",
    "icneumones": "icneumón",
    "icneumónidos": "icneumónido",
    "icnografías": "icnografía",
    "icnográfica": "icnográfico",
    "icnográficas": "icnográfico",
    "icnográficos": "icnográfico",
    "iconicidades": "iconicidad",
    "iconoclasias": "iconoclasia",
    "iconoclastas": "iconoclasta",
    "iconoclastias": "iconoclastia",
    "iconografías": "iconografía",
    "iconográfica": "iconográfico",
    "iconográficas": "iconográfico",
    "iconográficos": "iconográfico",
    "iconolatrías": "iconolatría",
    "iconologías": "iconología",
    "iconológica": "iconológico",
    "iconológicas": "iconológico",
    "iconológicos": "iconológico",
    "iconos": "icono",
    "iconoscopios": "iconoscopio",
    "iconostasios": "iconostasio",
    "iconólatras": "iconólatra",
    "iconómaca": "iconómaco",
    "iconómacas": "iconómaco",
    "iconómacos": "iconómaco",
    "iconómetros": "iconómetro",
    "icores": "icor",
    "icorosa": "icoroso",
    "icorosas": "icoroso",
    "icorosos": "icoroso",
    "icosaedros": "icosaedro",
    "icosaédrica": "icosaédrico",
    "icosaédricas": "icosaédrico",
    "icosaédricos": "icosaédrico",
    "icositetraedros": "icositetraedro",
    "icoságonos": "icoságono",
    "ictericiada": "ictericiado",
    "ictericiadas": "ictericiado",
    "ictericiados": "ictericiado",
    "ictericias": "ictericia",
    "ictiofagias": "ictiofagia",
    "ictiofaunas": "ictiofauna",
    "ictiografías": "ictiografía",
    "ictiologías": "ictiología",
    "ictiológica": "ictiológico",
    "ictiológicas": "ictiológico",
    "ictiológicos": "ictiológico",
    "ictiosaurios": "ictiosaurio",
    "ictiosauros": "ictiosauro",
    "ictiosismos": "ictiosismo",
    "ictiotoxicidades": "ictiotoxicidad",
    "ictiotoxinas": "ictiotoxina",
    "ictiófaga": "ictiófago",
    "ictiófagas": "ictiófago",
    "ictiófagos": "ictiófago",
    "ictióloga": "ictiólogo",
    "ictiólogas": "ictiólogo",
    "ictiólogos": "ictiólogo",
    "ictérica": "ictérico",
    "ictéricas": "ictérico",
    "ictéricos": "ictérico",
    "ictínea": "ictíneo",
    "ictíneas": "ictíneo",
    "ictíneos": "ictíneo",
    "icástica": "icástico",
    "icásticas": "icástico",
    "icásticos": "icástico",
    "icé": "izar",
    "icéis": "izar",
    "icónica": "icónico",
    "icónicas": "icónico",
    "icónicos": "icónico",
    "id": "ir",
    "ida": "ir",
    "idas": "ir",
    "idea": "ideo",
    "ideaba": "idear",
    "ideabais": "idear",
    "ideaban": "idear",
    "ideabas": "idear",
    "ideaciones": "ideación",
    "idead": "idear",
    "ideada": "idear",
    "ideadas": "idear",
    "ideado": "idear",
    "ideados": "idear",
    "ideales": "ideal",
    "idealice": "idealizar",
    "idealicemos": "idealizar",
    "idealicen": "idealizar",
    "idealices": "idealizar",
    "idealicé": "idealizar",
    "idealicéis": "idealizar",
    "idealidades": "idealidad",
    "idealismos": "idealismo",
    "idealistas": "idealista",
    "idealiza": "idealizar",
    "idealizaba": "idealizar",
    "idealizabais": "idealizar",
    "idealizaban": "idealizar",
    "idealizabas": "idealizar",
    "idealizaciones": "idealización",
    "idealizad": "idealizar",
    "idealizada": "idealizar",
    "idealizadas": "idealizar",
    "idealizado": "idealizar",
    "idealizadora": "idealizador",
    "idealizadoras": "idealizador",
    "idealizadores": "idealizador",
    "idealizados": "idealizar",
    "idealizamos": "idealizar",
    "idealizan": "idealizar",
    "idealizando": "idealizar",
    "idealizara": "idealizar",
    "idealizarais": "idealizar",
    "idealizaran": "idealizar",
    "idealizaras": "idealizar",
    "idealizare": "idealizar",
    "idealizareis": "idealizar",
    "idealizaremos": "idealizar",
    "idealizaren": "idealizar",
    "idealizares": "idealizar",
    "idealizaron": "idealizar",
    "idealizará": "idealizar",
    "idealizarán": "idealizar",
    "idealizarás": "idealizar",
    "idealizaré": "idealizar",
    "idealizaréis": "idealizar",
    "idealizaría": "idealizar",
    "idealizaríais": "idealizar",
    "idealizaríamos": "idealizar",
    "idealizarían": "idealizar",
    "idealizarías": "idealizar",
    "idealizas": "idealizar",
    "idealizase": "idealizar",
    "idealizaseis": "idealizar",
    "idealizasen": "idealizar",
    "idealizases": "idealizar",
    "idealizaste": "idealizar",
    "idealizasteis": "idealizar",
    "idealizo": "idealizar",
    "idealizábamos": "idealizar",
    "idealizáis": "idealizar",
    "idealizáramos": "idealizar",
    "idealizáremos": "idealizar",
    "idealizásemos": "idealizar",
    "idealizó": "idealizar",
    "ideamos": "idear",
    "idean": "idear",
    "ideando": "idear",
    "ideara": "idear",
    "idearais": "idear",
    "idearan": "idear",
    "idearas": "idear",
    "ideare": "idear",
    "ideareis": "idear",
    "idearemos": "idear",
    "idearen": "idear",
    "ideares": "idear",
    "idearios": "ideario",
    "idearon": "idear",
    "ideará": "idear",
    "idearán": "idear",
    "idearás": "idear",
    "idearé": "idear",
    "idearéis": "idear",
    "idearía": "idear",
    "idearíais": "idear",
    "idearíamos": "idear",
    "idearían": "idear",
    "idearías": "idear",
    "ideas": "ideo",
    "idease": "idear",
    "ideaseis": "idear",
    "ideasen": "idear",
    "ideases": "idear",
    "ideaste": "idear",
    "ideasteis": "idear",
    "idee": "idear",
    "ideemos": "idear",
    "ideen": "idear",
    "idees": "idear",
    "idem": "ídem",
    "idemfactores": "idemfactor",
    "idempotencias": "idempotencia",
    "idempotentes": "idempotente",
    "identidades": "identidad",
    "identifica": "identificar",
    "identificaba": "identificar",
    "identificabais": "identificar",
    "identificaban": "identificar",
    "identificabas": "identificar",
    "identificabilidades": "identificabilidad",
    "identificables": "identificable",
    "identificaciones": "identificación",
    "identificad": "identificar",
    "identificada": "identificar",
    "identificadas": "identificar",
    "identificado": "identificar",
    "identificadora": "identificador",
    "identificadoras": "identificador",
    "identificadores": "identificador",
    "identificados": "identificar",
    "identificamos": "identificar",
    "identifican": "identificar",
    "identificando": "identificar",
    "identificara": "identificar",
    "identificarais": "identificar",
    "identificaran": "identificar",
    "identificaras": "identificar",
    "identificare": "identificar",
    "identificareis": "identificar",
    "identificaremos": "identificar",
    "identificaren": "identificar",
    "identificares": "identificar",
    "identificaron": "identificar",
    "identificará": "identificar",
    "identificarán": "identificar",
    "identificarás": "identificar",
    "identificaré": "identificar",
    "identificaréis": "identificar",
    "identificaría": "identificar",
    "identificaríais": "identificar",
    "identificaríamos": "identificar",
    "identificarían": "identificar",
    "identificarías": "identificar",
    "identificas": "identificar",
    "identificase": "identificar",
    "identificaseis": "identificar",
    "identificasen": "identificar",
    "identificases": "identificar",
    "identificaste": "identificar",
    "identificasteis": "identificar",
    "identificativa": "identificativo",
    "identificativas": "identificativo",
    "identificativos": "identificativo",
    "identifico": "identificar",
    "identificábamos": "identificar",
    "identificáis": "identificar",
    "identificáramos": "identificar",
    "identificáremos": "identificar",
    "identificásemos": "identificar",
    "identificó": "identificar",
    "identifique": "identificar",
    "identifiquemos": "identificar",
    "identifiquen": "identificar",
    "identifiques": "identificar",
    "identifiqué": "identificar",
    "identifiquéis": "identificar",
    "ideo": "idear",
    "ideografías": "ideografía",
    "ideogramas": "ideograma",
    "ideográfica": "ideográfico",
    "ideográficas": "ideográfico",
    "ideográficos": "ideográfico",
    "ideologice": "ideologizar",
    "ideologicemos": "ideologizar",
    "ideologicen": "ideologizar",
    "ideologices": "ideologizar",
    "ideologicé": "ideologizar",
    "ideologicéis": "ideologizar",
    "ideologiza": "ideologizar",
    "ideologizaba": "ideologizar",
    "ideologizabais": "ideologizar",
    "ideologizaban": "ideologizar",
    "ideologizabas": "ideologizar",
    "ideologizad": "ideologizar",
    "ideologizada": "ideologizar",
    "ideologizadas": "ideologizar",
    "ideologizado": "ideologizar",
    "ideologizados": "ideologizar",
    "ideologizamos": "ideologizar",
    "ideologizan": "ideologizar",
    "ideologizando": "ideologizar",
    "ideologizara": "ideologizar",
    "ideologizarais": "ideologizar",
    "ideologizaran": "ideologizar",
    "ideologizaras": "ideologizar",
    "ideologizare": "ideologizar",
    "ideologizareis": "ideologizar",
    "ideologizaremos": "ideologizar",
    "ideologizaren": "ideologizar",
    "ideologizares": "ideologizar",
    "ideologizaron": "ideologizar",
    "ideologizará": "ideologizar",
    "ideologizarán": "ideologizar",
    "ideologizarás": "ideologizar",
    "ideologizaré": "ideologizar",
    "ideologizaréis": "ideologizar",
    "ideologizaría": "ideologizar",
    "ideologizaríais": "ideologizar",
    "ideologizaríamos": "ideologizar",
    "ideologizarían": "ideologizar",
    "ideologizarías": "ideologizar",
    "ideologizas": "ideologizar",
    "ideologizase": "ideologizar",
    "ideologizaseis": "ideologizar",
    "ideologizasen": "ideologizar",
    "ideologizases": "ideologizar",
    "ideologizaste": "ideologizar",
    "ideologizasteis": "ideologizar",
    "ideologizo": "ideologizar",
    "ideologizábamos": "ideologizar",
    "ideologizáis": "ideologizar",
    "ideologizáramos": "ideologizar",
    "ideologizáremos": "ideologizar",
    "ideologizásemos": "ideologizar",
    "ideologizó": "ideologizar",
    "ideologías": "ideología",
    "ideológica": "ideológico",
    "ideológicas": "ideológico",
    "ideológicos": "ideológico",
    "ideos": "ideo",
    "ideábamos": "idear",
    "ideáis": "idear",
    "ideáramos": "idear",
    "ideáremos": "idear",
    "ideásemos": "idear",
    "ideática": "ideático",
    "ideáticas": "ideático",
    "ideáticos": "ideático",
    "ideé": "idear",
    "ideéis": "idear",
    "ideó": "idear",
    "ideóloga": "ideólogo",
    "ideólogas": "ideólogo",
    "ideólogos": "ideólogo",
    "idilios": "idilio",
    "idioblastos": "idioblasto",
    "idiocias": "idiocia",
    "idiocromática": "idiocromático",
    "idiocromáticas": "idiocromático",
    "idiocromáticos": "idiocromático",
    "idioeléctrica": "idioeléctrico",
    "idioeléctricas": "idioeléctrico",
    "idioeléctricos": "idioeléctrico",
    "idiogramas": "idiograma",
    "idiográfica": "idiográfico",
    "idiográficas": "idiográfico",
    "idiográficos": "idiográfico",
    "idiolectos": "idiolecto",
    "idiomas": "idioma",
    "idiomatismos": "idiomatismo",
    "idiomutaciones": "idiomutación",
    "idiomática": "idiomático",
    "idiomáticas": "idiomático",
    "idiomáticos": "idiomático",
    "idiomórfica": "idiomórfico",
    "idiomórficas": "idiomórfico",
    "idiomórficos": "idiomórfico",
    "idiopatías": "idiopatía",
    "idioplasmas": "idioplasma",
    "idiopática": "idiopático",
    "idiopáticas": "idiopático",
    "idiopáticos": "idiopático",
    "idiosincrasias": "idiosincrasia",
    "idiosincrásica": "idiosincrásico",
    "idiosincrásicas": "idiosincrásico",
    "idiosincrásicos": "idiosincrásico",
    "idiosincrática": "idiosincrático",
    "idiosincráticas": "idiosincrático",
    "idiosincráticos": "idiosincrático",
    "idiosomas": "idiosoma",
    "idiotas": "idiota",
    "idioteces": "idiotez",
    "idiotice": "idiotizar",
    "idioticemos": "idiotizar",
    "idioticen": "idiotizar",
    "idiotices": "idiotizar",
    "idioticé": "idiotizar",
    "idioticéis": "idiotizar",
    "idiotipos": "idiotipo",
    "idiotismos": "idiotismo",
    "idiotiza": "idiotizar",
    "idiotizaba": "idiotizar",
    "idiotizabais": "idiotizar",
    "idiotizaban": "idiotizar",
    "idiotizabas": "idiotizar",
    "idiotizad": "idiotizar",
    "idiotizada": "idiotizar",
    "idiotizadas": "idiotizar",
    "idiotizado": "idiotizar",
    "idiotizados": "idiotizar",
    "idiotizamos": "idiotizar",
    "idiotizan": "idiotizar",
    "idiotizando": "idiotizar",
    "idiotizara": "idiotizar",
    "idiotizarais": "idiotizar",
    "idiotizaran": "idiotizar",
    "idiotizaras": "idiotizar",
    "idiotizare": "idiotizar",
    "idiotizareis": "idiotizar",
    "idiotizaremos": "idiotizar",
    "idiotizaren": "idiotizar",
    "idiotizares": "idiotizar",
    "idiotizaron": "idiotizar",
    "idiotizará": "idiotizar",
    "idiotizarán": "idiotizar",
    "idiotizarás": "idiotizar",
    "idiotizaré": "idiotizar",
    "idiotizaréis": "idiotizar",
    "idiotizaría": "idiotizar",
    "idiotizaríais": "idiotizar",
    "idiotizaríamos": "idiotizar",
    "idiotizarían": "idiotizar",
    "idiotizarías": "idiotizar",
    "idiotizas": "idiotizar",
    "idiotizase": "idiotizar",
    "idiotizaseis": "idiotizar",
    "idiotizasen": "idiotizar",
    "idiotizases": "idiotizar",
    "idiotizaste": "idiotizar",
    "idiotizasteis": "idiotizar",
    "idiotizo": "idiotizar",
    "idiotizábamos": "idiotizar",
    "idiotizáis": "idiotizar",
    "idiotizáramos": "idiotizar",
    "idiotizáremos": "idiotizar",
    "idiotizásemos": "idiotizar",
    "idiotizó": "idiotizar",
    "idiotípica": "idiotípico",
    "idiotípicas": "idiotípico",
    "idiotípicos": "idiotípico",
    "ido": "ir",
    "idocrasas": "idocrasa",
    "idolatra": "idolatrar",
    "idolatraba": "idolatrar",
    "idolatrabais": "idolatrar",
    "idolatraban": "idolatrar",
    "idolatrabas": "idolatrar",
    "idolatrad": "idolatrar",
    "idolatrada": "idolatrar",
    "idolatradas": "idolatrar",
    "idolatrado": "idolatrar",
    "idolatrados": "idolatrar",
    "idolatramos": "idolatrar",
    "idolatran": "idolatrar",
    "idolatrando": "idolatrar",
    "idolatrara": "idolatrar",
    "idolatrarais": "idolatrar",
    "idolatraran": "idolatrar",
    "idolatraras": "idolatrar",
    "idolatrare": "idolatrar",
    "idolatrareis": "idolatrar",
    "idolatraremos": "idolatrar",
    "idolatraren": "idolatrar",
    "idolatrares": "idolatrar",
    "idolatraron": "idolatrar",
    "idolatrará": "idolatrar",
    "idolatrarán": "idolatrar",
    "idolatrarás": "idolatrar",
    "idolatraré": "idolatrar",
    "idolatraréis": "idolatrar",
    "idolatraría": "idolatrar",
    "idolatraríais": "idolatrar",
    "idolatraríamos": "idolatrar",
    "idolatrarían": "idolatrar",
    "idolatrarías": "idolatrar",
    "idolatras": "idolatrar",
    "idolatrase": "idolatrar",
    "idolatraseis": "idolatrar",
    "idolatrasen": "idolatrar",
    "idolatrases": "idolatrar",
    "idolatraste": "idolatrar",
    "idolatrasteis": "idolatrar",
    "idolatre": "idolatrar",
    "idolatremos": "idolatrar",
    "idolatren": "idolatrar",
    "idolatres": "idolatrar",
    "idolatro": "idolatrar",
    "idolatrábamos": "idolatrar",
    "idolatráis": "idolatrar",
    "idolatráramos": "idolatrar",
    "idolatráremos": "idolatrar",
    "idolatrásemos": "idolatrar",
    "idolatré": "idolatrar",
    "idolatréis": "idolatrar",
    "idolatrías": "idolatría",
    "idolatró": "idolatrar",
    "idolologías": "idolología",
    "idolopeyas": "idolopeya",
    "idolátrica": "idolátrico",
    "idolátricas": "idolátrico",
    "idolátricos": "idolátrico",
    "idoneidades": "idoneidad",
    "idos": "ir",
    "idrisíes": "idrisí",
    "idrisís": "idrisí",
    "idumea": "idumeo",
    "idumeas": "idumeo",
    "idumeos": "idumeo",
    "idéntica": "idéntico",
    "idénticas": "idéntico",
    "idénticos": "idéntico",
    "idílica": "idílico",
    "idílicas": "idílico",
    "idílicos": "idílico",
    "idólatras": "idólatra",
    "idónea": "idóneo",
    "idóneas": "idóneo",
    "idóneos": "idóneo",
    "ies": "i",
    "iglesiarios": "iglesiario",
    "iglesias": "iglesia",
    "iglúes": "iglú",
    "iglús": "iglú",
    "ignaciana": "ignaciano",
    "ignacianas": "ignaciano",
    "ignacianos": "ignaciano",
    "ignara": "ignaro",
    "ignaras": "ignaro",
    "ignaros": "ignaro",
    "ignava": "ignavo",
    "ignavas": "ignavo",
    "ignavias": "ignavia",
    "ignavos": "ignavo",
    "igniciones": "ignición",
    "ignifuga": "ignifugar",
    "ignifugaba": "ignifugar",
    "ignifugabais": "ignifugar",
    "ignifugaban": "ignifugar",
    "ignifugabas": "ignifugar",
    "ignifugaciones": "ignifugación",
    "ignifugad": "ignifugar",
    "ignifugada": "ignifugar",
    "ignifugadas": "ignifugar",
    "ignifugado": "ignifugar",
    "ignifugados": "ignifugar",
    "ignifugamos": "ignifugar",
    "ignifugan": "ignifugar",
    "ignifugando": "ignifugar",
    "ignifugara": "ignifugar",
    "ignifugarais": "ignifugar",
    "ignifugaran": "ignifugar",
    "ignifugaras": "ignifugar",
    "ignifugare": "ignifugar",
    "ignifugareis": "ignifugar",
    "ignifugaremos": "ignifugar",
    "ignifugaren": "ignifugar",
    "ignifugares": "ignifugar",
    "ignifugaron": "ignifugar",
    "ignifugará": "ignifugar",
    "ignifugarán": "ignifugar",
    "ignifugarás": "ignifugar",
    "ignifugaré": "ignifugar",
    "ignifugaréis": "ignifugar",
    "ignifugaría": "ignifugar",
    "ignifugaríais": "ignifugar",
    "ignifugaríamos": "ignifugar",
    "ignifugarían": "ignifugar",
    "ignifugarías": "ignifugar",
    "ignifugas": "ignifugar",
    "ignifugase": "ignifugar",
    "ignifugaseis": "ignifugar",
    "ignifugasen": "ignifugar",
    "ignifugases": "ignifugar",
    "ignifugaste": "ignifugar",
    "ignifugasteis": "ignifugar",
    "ignifugo": "ignifugar",
    "ignifugue": "ignifugar",
    "ignifuguemos": "ignifugar",
    "ignifuguen": "ignifugar",
    "ignifugues": "ignifugar",
    "ignifugué": "ignifugar",
    "ignifuguéis": "ignifugar",
    "ignifugábamos": "ignifugar",
    "ignifugáis": "ignifugar",
    "ignifugáramos": "ignifugar",
    "ignifugáremos": "ignifugar",
    "ignifugásemos": "ignifugar",
    "ignifugó": "ignifugar",
    "ignimbritas": "ignimbrita",
    "ignimbrítica": "ignimbrítico",
    "ignimbríticas": "ignimbrítico",
    "ignimbríticos": "ignimbrítico",
    "ignipotentes": "ignipotente",
    "ignita": "ignito",
    "ignitas": "ignito",
    "ignitos": "ignito",
    "ignitrones": "ignitrón",
    "ignografías": "ignografía",
    "ignominias": "ignominia",
    "ignominiosa": "ignominioso",
    "ignominiosas": "ignominioso",
    "ignominiosos": "ignominioso",
    "ignora": "ignorar",
    "ignoraba": "ignorar",
    "ignorabais": "ignorar",
    "ignoraban": "ignorar",
    "ignorabas": "ignorar",
    "ignorad": "ignorar",
    "ignorada": "ignorar",
    "ignoradas": "ignorar",
    "ignorado": "ignorar",
    "ignorados": "ignorar",
    "ignoramos": "ignorar",
    "ignoran": "ignorar",
    "ignorancias": "ignorancia",
    "ignorando": "ignorar",
    "ignorantes": "ignorante",
    "ignorara": "ignorar",
    "ignorarais": "ignorar",
    "ignoraran": "ignorar",
    "ignoraras": "ignorar",
    "ignorare": "ignorar",
    "ignorareis": "ignorar",
    "ignoraremos": "ignorar",
    "ignoraren": "ignorar",
    "ignorares": "ignorar",
    "ignoraron": "ignorar",
    "ignorará": "ignorar",
    "ignorarán": "ignorar",
    "ignorarás": "ignorar",
    "ignoraré": "ignorar",
    "ignoraréis": "ignorar",
    "ignoraría": "ignorar",
    "ignoraríais": "ignorar",
    "ignoraríamos": "ignorar",
    "ignorarían": "ignorar",
    "ignorarías": "ignorar",
    "ignoras": "ignorar",
    "ignorase": "ignorar",
    "ignoraseis": "ignorar",
    "ignorasen": "ignorar",
    "ignorases": "ignorar",
    "ignoraste": "ignorar",
    "ignorasteis": "ignorar",
    "ignore": "ignorar",
    "ignoremos": "ignorar",
    "ignoren": "ignorar",
    "ignores": "ignorar",
    "ignoro": "ignorar",
    "ignorábamos": "ignorar",
    "ignoráis": "ignorar",
    "ignoráramos": "ignorar",
    "ignoráremos": "ignorar",
    "ignorásemos": "ignorar",
    "ignoré": "ignorar",
    "ignoréis": "ignorar",
    "ignoró": "ignorar",
    "ignota": "ignoto",
    "ignotas": "ignoto",
    "ignotos": "ignoto",
    "ignífera": "ignífero",
    "igníferas": "ignífero",
    "igníferos": "ignífero",
    "ignífuga": "ignífugo",
    "ignífugas": "ignífugo",
    "ignífugos": "ignífugo",
    "ignívoma": "ignívomo",
    "ignívomas": "ignívomo",
    "ignívomos": "ignívomo",
    "igorrotes": "igorrote",
    "iguala": "igualar",
    "igualaba": "igualar",
    "igualabais": "igualar",
    "igualaban": "igualar",
    "igualabas": "igualar",
    "igualaciones": "igualación",
    "igualaciónes": "igualación",
    "igualad": "igualar",
    "igualada": "igualar",
    "igualadas": "igualar",
    "igualado": "igualar",
    "igualadora": "igualador",
    "igualadoras": "igualador",
    "igualadores": "igualador",
    "igualados": "igualar",
    "igualamientos": "igualamiento",
    "igualamos": "igualar",
    "igualan": "igualar",
    "igualando": "igualar",
    "igualara": "igualar",
    "igualarais": "igualar",
    "igualaran": "igualar",
    "igualaras": "igualar",
    "igualare": "igualar",
    "igualareis": "igualar",
    "igualaremos": "igualar",
    "igualaren": "igualar",
    "igualares": "igualar",
    "igualaron": "igualar",
    "igualará": "igualar",
    "igualarán": "igualar",
    "igualarás": "igualar",
    "igualaré": "igualar",
    "igualaréis": "igualar",
    "igualaría": "igualar",
    "igualaríais": "igualar",
    "igualaríamos": "igualar",
    "igualarían": "igualar",
    "igualarías": "igualar",
    "igualas": "igualar",
    "igualase": "igualar",
    "igualaseis": "igualar",
    "igualasen": "igualar",
    "igualases": "igualar",
    "igualaste": "igualar",
    "igualasteis": "igualar",
    "igualatoria": "igualatorio",
    "igualatorias": "igualatorio",
    "igualatorios": "igualatorio",
    "igualdades": "igualdad",
    "iguale": "igualar",
    "igualemos": "igualar",
    "igualen": "igualar",
    "iguales": "igualar",
    "igualitaria": "igualitario",
    "igualitarias": "igualitario",
    "igualitarios": "igualitario",
    "igualitarismos": "igualitarismo",
    "igualo": "igualar",
    "igualábamos": "igualar",
    "igualáis": "igualar",
    "igualáramos": "igualar",
    "igualáremos": "igualar",
    "igualásemos": "igualar",
    "igualé": "igualar",
    "igualéis": "igualar",
    "igualó": "igualar",
    "iguanas": "iguana",
    "iguanodontes": "iguanodonte",
    "igüedos": "igüedo",
    "ijadas": "ijada",
    "ijadea": "ijadear",
    "ijadeaba": "ijadear",
    "ijadeabais": "ijadear",
    "ijadeaban": "ijadear",
    "ijadeabas": "ijadear",
    "ijadead": "ijadear",
    "ijadeada": "ijadear",
    "ijadeadas": "ijadear",
    "ijadeado": "ijadear",
    "ijadeados": "ijadear",
    "ijadeamos": "ijadear",
    "ijadean": "ijadear",
    "ijadeando": "ijadear",
    "ijadeara": "ijadear",
    "ijadearais": "ijadear",
    "ijadearan": "ijadear",
    "ijadearas": "ijadear",
    "ijadeare": "ijadear",
    "ijadeareis": "ijadear",
    "ijadearemos": "ijadear",
    "ijadearen": "ijadear",
    "ijadeares": "ijadear",
    "ijadearon": "ijadear",
    "ijadeará": "ijadear",
    "ijadearán": "ijadear",
    "ijadearás": "ijadear",
    "ijadearé": "ijadear",
    "ijadearéis": "ijadear",
    "ijadearía": "ijadear",
    "ijadearíais": "ijadear",
    "ijadearíamos": "ijadear",
    "ijadearían": "ijadear",
    "ijadearías": "ijadear",
    "ijadeas": "ijadear",
    "ijadease": "ijadear",
    "ijadeaseis": "ijadear",
    "ijadeasen": "ijadear",
    "ijadeases": "ijadear",
    "ijadeaste": "ijadear",
    "ijadeasteis": "ijadear",
    "ijadee": "ijadear",
    "ijadeemos": "ijadear",
    "ijadeen": "ijadear",
    "ijadees": "ijadear",
    "ijadeo": "ijadear",
    "ijadeábamos": "ijadear",
    "ijadeáis": "ijadear",
    "ijadeáramos": "ijadear",
    "ijadeáremos": "ijadear",
    "ijadeásemos": "ijadear",
    "ijadeé": "ijadear",
    "ijadeéis": "ijadear",
    "ijadeó": "ijadear",
    "ijares": "ijar",
    "ijolitas": "ijolita",
    "ikebanas": "ikebana",
    "ilaciones": "ilación",
    "ilapsos": "ilapso",
    "ilativa": "ilativo",
    "ilativas": "ilativo",
    "ilativos": "ilativo",
    "ilegales": "ilegal",
    "ilegalice": "ilegalizar",
    "ilegalicemos": "ilegalizar",
    "ilegalicen": "ilegalizar",
    "ilegalices": "ilegalizar",
    "ilegalicé": "ilegalizar",
    "ilegalicéis": "ilegalizar",
    "ilegalidades": "ilegalidad",
    "ilegaliza": "ilegalizar",
    "ilegalizaba": "ilegalizar",
    "ilegalizabais": "ilegalizar",
    "ilegalizaban": "ilegalizar",
    "ilegalizabas": "ilegalizar",
    "ilegalizaciones": "ilegalización",
    "ilegalizad": "ilegalizar",
    "ilegalizada": "ilegalizar",
    "ilegalizadas": "ilegalizar",
    "ilegalizado": "ilegalizar",
    "ilegalizados": "ilegalizar",
    "ilegalizamos": "ilegalizar",
    "ilegalizan": "ilegalizar",
    "ilegalizando": "ilegalizar",
    "ilegalizara": "ilegalizar",
    "ilegalizarais": "ilegalizar",
    "ilegalizaran": "ilegalizar",
    "ilegalizaras": "ilegalizar",
    "ilegalizare": "ilegalizar",
    "ilegalizareis": "ilegalizar",
    "ilegalizaremos": "ilegalizar",
    "ilegalizaren": "ilegalizar",
    "ilegalizares": "ilegalizar",
    "ilegalizaron": "ilegalizar",
    "ilegalizará": "ilegalizar",
    "ilegalizarán": "ilegalizar",
    "ilegalizarás": "ilegalizar",
    "ilegalizaré": "ilegalizar",
    "ilegalizaréis": "ilegalizar",
    "ilegalizaría": "ilegalizar",
    "ilegalizaríais": "ilegalizar",
    "ilegalizaríamos": "ilegalizar",
    "ilegalizarían": "ilegalizar",
    "ilegalizarías": "ilegalizar",
    "ilegalizas": "ilegalizar",
    "ilegalizase": "ilegalizar",
    "ilegalizaseis": "ilegalizar",
    "ilegalizasen": "ilegalizar",
    "ilegalizases": "ilegalizar",
    "ilegalizaste": "ilegalizar",
    "ilegalizasteis": "ilegalizar",
    "ilegalizo": "ilegalizar",
    "ilegalizábamos": "ilegalizar",
    "ilegalizáis": "ilegalizar",
    "ilegalizáramos": "ilegalizar",
    "ilegalizáremos": "ilegalizar",
    "ilegalizásemos": "ilegalizar",
    "ilegalizó": "ilegalizar",
    "ilegibles": "ilegible",
    "ilegitima": "ilegitimar",
    "ilegitimaba": "ilegitimar",
    "ilegitimabais": "ilegitimar",
    "ilegitimaban": "ilegitimar",
    "ilegitimabas": "ilegitimar",
    "ilegitimad": "ilegitimar",
    "ilegitimada": "ilegitimar",
    "ilegitimadas": "ilegitimar",
    "ilegitimado": "ilegitimar",
    "ilegitimados": "ilegitimar",
    "ilegitimamos": "ilegitimar",
    "ilegitiman": "ilegitimar",
    "ilegitimando": "ilegitimar",
    "ilegitimara": "ilegitimar",
    "ilegitimarais": "ilegitimar",
    "ilegitimaran": "ilegitimar",
    "ilegitimaras": "ilegitimar",
    "ilegitimare": "ilegitimar",
    "ilegitimareis": "ilegitimar",
    "ilegitimaremos": "ilegitimar",
    "ilegitimaren": "ilegitimar",
    "ilegitimares": "ilegitimar",
    "ilegitimaron": "ilegitimar",
    "ilegitimará": "ilegitimar",
    "ilegitimarán": "ilegitimar",
    "ilegitimarás": "ilegitimar",
    "ilegitimaré": "ilegitimar",
    "ilegitimaréis": "ilegitimar",
    "ilegitimaría": "ilegitimar",
    "ilegitimaríais": "ilegitimar",
    "ilegitimaríamos": "ilegitimar",
    "ilegitimarían": "ilegitimar",
    "ilegitimarías": "ilegitimar",
    "ilegitimas": "ilegitimar",
    "ilegitimase": "ilegitimar",
    "ilegitimaseis": "ilegitimar",
    "ilegitimasen": "ilegitimar",
    "ilegitimases": "ilegitimar",
    "ilegitimaste": "ilegitimar",
    "ilegitimasteis": "ilegitimar",
    "ilegitime": "ilegitimar",
    "ilegitimemos": "ilegitimar",
    "ilegitimen": "ilegitimar",
    "ilegitimes": "ilegitimar",
    "ilegitimidades": "ilegitimidad",
    "ilegitimo": "ilegitimar",
    "ilegitimábamos": "ilegitimar",
    "ilegitimáis": "ilegitimar",
    "ilegitimáramos": "ilegitimar",
    "ilegitimáremos": "ilegitimar",
    "ilegitimásemos": "ilegitimar",
    "ilegitimé": "ilegitimar",
    "ilegitiméis": "ilegitimar",
    "ilegitimó": "ilegitimar",
    "ilegítima": "ilegítimo",
    "ilegítimas": "ilegítimo",
    "ilegítimos": "ilegítimo",
    "ileocecales": "ileocecal",
    "ileocólica": "ileocólico",
    "ileocólicas": "ileocólico",
    "ileocólicos": "ileocólico",
    "ileones": "íleon",
    "ilerdenses": "ilerdense",
    "ilergetes": "ilergete",
    "ilesa": "ileso",
    "ilesas": "ileso",
    "ilesos": "ileso",
    "iletrada": "iletrado",
    "iletradas": "iletrado",
    "iletrados": "iletrado",
    "iliaca": "ilíaco",
    "iliacas": "ilíaco",
    "iliacos": "ilíaco",
    "iliberitana": "iliberitano",
    "iliberitanas": "iliberitano",
    "iliberitanos": "iliberitano",
    "ilicitana": "ilicitano",
    "ilicitanas": "ilicitano",
    "ilicitanos": "ilicitano",
    "ilicitudes": "ilicitud",
    "ilimitables": "ilimitable",
    "ilimitada": "ilimitado",
    "ilimitadas": "ilimitado",
    "ilimitados": "ilimitado",
    "iliococcígea": "iliococcígeo",
    "iliococcígeas": "iliococcígeo",
    "iliococcígeos": "iliococcígeo",
    "iliocostales": "iliocostal",
    "iliofemorales": "iliofemoral",
    "iliohipogástrica": "iliohipogástrico",
    "iliohipogástricas": "iliohipogástrico",
    "iliohipogástricos": "iliohipogástrico",
    "ilioinguinales": "ilioinguinal",
    "iliolumbares": "iliolumbar",
    "iliones": "ilion",
    "iliopectinia": "iliopectinio",
    "iliopectinias": "iliopectinio",
    "iliopectinios": "iliopectinio",
    "iliopectínea": "iliopectíneo",
    "iliopectíneas": "iliopectíneo",
    "iliopectíneos": "iliopectíneo",
    "iliopúbica": "iliopúbico",
    "iliopúbicas": "iliopúbico",
    "iliopúbicos": "iliopúbico",
    "iliotibiales": "iliotibial",
    "iliquideces": "iliquidez",
    "iliria": "ilirio",
    "ilirias": "ilirio",
    "ilirios": "ilirio",
    "ilirismos": "ilirismo",
    "iliterata": "iliterato",
    "iliteratas": "iliterato",
    "iliteratos": "iliterato",
    "iliturgitana": "iliturgitano",
    "iliturgitanas": "iliturgitano",
    "iliturgitanos": "iliturgitano",
    "illitas": "illita",
    "ilmenitas": "ilmenita",
    "ilocutiva": "ilocutivo",
    "ilocutivas": "ilocutivo",
    "ilocutivos": "ilocutivo",
    "ilotas": "ilota",
    "ilotismos": "ilotismo",
    "ilumina": "iluminar",
    "iluminaba": "iluminar",
    "iluminabais": "iluminar",
    "iluminaban": "iluminar",
    "iluminabas": "iluminar",
    "iluminaciones": "iluminación",
    "iluminad": "iluminar",
    "iluminada": "iluminar",
    "iluminadas": "iluminar",
    "iluminado": "iluminar",
    "iluminadora": "iluminador",
    "iluminadoras": "iluminador",
    "iluminadores": "iluminador",
    "iluminados": "iluminar",
    "iluminamos": "iluminar",
    "iluminan": "iluminar",
    "iluminancias": "iluminancia",
    "iluminando": "iluminar",
    "iluminantes": "iluminante",
    "iluminara": "iluminar",
    "iluminarais": "iluminar",
    "iluminaran": "iluminar",
    "iluminaras": "iluminar",
    "iluminare": "iluminar",
    "iluminareis": "iluminar",
    "iluminaremos": "iluminar",
    "iluminaren": "iluminar",
    "iluminares": "iluminar",
    "iluminarias": "iluminaria",
    "iluminaron": "iluminar",
    "iluminará": "iluminar",
    "iluminarán": "iluminar",
    "iluminarás": "iluminar",
    "iluminaré": "iluminar",
    "iluminaréis": "iluminar",
    "iluminaría": "iluminar",
    "iluminaríais": "iluminar",
    "iluminaríamos": "iluminar",
    "iluminarían": "iluminar",
    "iluminarías": "iluminar",
    "iluminas": "iluminar",
    "iluminase": "iluminar",
    "iluminaseis": "iluminar",
    "iluminasen": "iluminar",
    "iluminases": "iluminar",
    "iluminaste": "iluminar",
    "iluminasteis": "iluminar",
    "iluminativa": "iluminativo",
    "iluminativas": "iluminativo",
    "iluminativos": "iluminativo",
    "ilumine": "iluminar",
    "iluminemos": "iluminar",
    "iluminen": "iluminar",
    "ilumines": "iluminar",
    "iluminismos": "iluminismo",
    "iluministas": "iluminista",
    "ilumino": "iluminar",
    "iluminábamos": "iluminar",
    "ilumináis": "iluminar",
    "ilumináramos": "iluminar",
    "ilumináremos": "iluminar",
    "iluminásemos": "iluminar",
    "iluminé": "iluminar",
    "iluminéis": "iluminar",
    "iluminó": "iluminar",
    "ilusa": "iluso",
    "ilusas": "iluso",
    "ilusiona": "ilusionar",
    "ilusionaba": "ilusionar",
    "ilusionabais": "ilusionar",
    "ilusionaban": "ilusionar",
    "ilusionabas": "ilusionar",
    "ilusionables": "ilusionable",
    "ilusionad": "ilusionar",
    "ilusionada": "ilusionar",
    "ilusionadas": "ilusionar",
    "ilusionado": "ilusionar",
    "ilusionados": "ilusionar",
    "ilusionamos": "ilusionar",
    "ilusionan": "ilusionar",
    "ilusionando": "ilusionar",
    "ilusionara": "ilusionar",
    "ilusionarais": "ilusionar",
    "ilusionaran": "ilusionar",
    "ilusionaras": "ilusionar",
    "ilusionare": "ilusionar",
    "ilusionareis": "ilusionar",
    "ilusionaremos": "ilusionar",
    "ilusionaren": "ilusionar",
    "ilusionares": "ilusionar",
    "ilusionaron": "ilusionar",
    "ilusionará": "ilusionar",
    "ilusionarán": "ilusionar",
    "ilusionarás": "ilusionar",
    "ilusionaré": "ilusionar",
    "ilusionaréis": "ilusionar",
    "ilusionaría": "ilusionar",
    "ilusionaríais": "ilusionar",
    "ilusionaríamos": "ilusionar",
    "ilusionarían": "ilusionar",
    "ilusionarías": "ilusionar",
    "ilusionas": "ilusionar",
    "ilusionase": "ilusionar",
    "ilusionaseis": "ilusionar",
    "ilusionasen": "ilusionar",
    "ilusionases": "ilusionar",
    "ilusionaste": "ilusionar",
    "ilusionasteis": "ilusionar",
    "ilusione": "ilusionar",
    "ilusionemos": "ilusionar",
    "ilusionen": "ilusionar",
    "ilusiones": "ilusionar",
    "ilusionismos": "ilusionismo",
    "ilusionistas": "ilusionista",
    "ilusiono": "ilusionar",
    "ilusionábamos": "ilusionar",
    "ilusionáis": "ilusionar",
    "ilusionáramos": "ilusionar",
    "ilusionáremos": "ilusionar",
    "ilusionásemos": "ilusionar",
    "ilusioné": "ilusionar",
    "ilusionéis": "ilusionar",
    "ilusionó": "ilusionar",
    "ilusiva": "ilusivo",
    "ilusivas": "ilusivo",
    "ilusivos": "ilusivo",
    "ilusoria": "ilusorio",
    "ilusorias": "ilusorio",
    "ilusorios": "ilusorio",
    "ilusos": "iluso",
    "ilustra": "ilustrar",
    "ilustraba": "ilustrar",
    "ilustrabais": "ilustrar",
    "ilustraban": "ilustrar",
    "ilustrabas": "ilustrar",
    "ilustraciones": "ilustración",
    "ilustrad": "ilustrar",
    "ilustrada": "ilustrar",
    "ilustradas": "ilustrar",
    "ilustrado": "ilustrar",
    "ilustradora": "ilustrador",
    "ilustradoras": "ilustrador",
    "ilustradores": "ilustrador",
    "ilustrados": "ilustrar",
    "ilustramos": "ilustrar",
    "ilustran": "ilustrar",
    "ilustrando": "ilustrar",
    "ilustrara": "ilustrar",
    "ilustrarais": "ilustrar",
    "ilustraran": "ilustrar",
    "ilustraras": "ilustrar",
    "ilustrare": "ilustrar",
    "ilustrareis": "ilustrar",
    "ilustraremos": "ilustrar",
    "ilustraren": "ilustrar",
    "ilustrares": "ilustrar",
    "ilustraron": "ilustrar",
    "ilustrará": "ilustrar",
    "ilustrarán": "ilustrar",
    "ilustrarás": "ilustrar",
    "ilustraré": "ilustrar",
    "ilustraréis": "ilustrar",
    "ilustraría": "ilustrar",
    "ilustraríais": "ilustrar",
    "ilustraríamos": "ilustrar",
    "ilustrarían": "ilustrar",
    "ilustrarías": "ilustrar",
    "ilustras": "ilustrar",
    "ilustrase": "ilustrar",
    "ilustraseis": "ilustrar",
    "ilustrasen": "ilustrar",
    "ilustrases": "ilustrar",
    "ilustraste": "ilustrar",
    "ilustrasteis": "ilustrar",
    "ilustrativa": "ilustrativo",
    "ilustrativas": "ilustrativo",
    "ilustrativos": "ilustrativo",
    "ilustre": "ilustrar",
    "ilustremos": "ilustrar",
    "ilustren": "ilustrar",
    "ilustres": "ilustre",
    "ilustro": "ilustrar",
    "ilustrábamos": "ilustrar",
    "ilustráis": "ilustrar",
    "ilustráramos": "ilustrar",
    "ilustráremos": "ilustrar",
    "ilustrásemos": "ilustrar",
    "ilustré": "ilustrar",
    "ilustréis": "ilustrar",
    "ilustró": "ilustrar",
    "iluviaciones": "iluviación",
    "iluviales": "iluvial",
    "ilíaca": "ilíaco",
    "ilíacas": "ilíaco",
    "ilíacos": "ilíaco",
    "ilícita": "ilícito",
    "ilícitas": "ilícito",
    "ilícitos": "ilícito",
    "ilíquidas": "ilíquida",
    "ilógica": "ilógico",
    "ilógicas": "ilógico",
    "ilógicos": "ilógico",
    "imadas": "imada",
    "imagerías": "imagería",
    "imagina": "imaginar",
    "imaginaba": "imaginar",
    "imaginabais": "imaginar",
    "imaginaban": "imaginar",
    "imaginabas": "imaginar",
    "imaginables": "imaginable",
    "imaginaciones": "imaginación",
    "imaginad": "imaginar",
    "imaginada": "imaginar",
    "imaginadas": "imaginar",
    "imaginado": "imaginar",
    "imaginados": "imaginar",
    "imaginales": "imaginal",
    "imaginamos": "imaginar",
    "imaginan": "imaginar",
    "imaginando": "imaginar",
    "imaginara": "imaginar",
    "imaginarais": "imaginar",
    "imaginaran": "imaginar",
    "imaginaras": "imaginar",
    "imaginare": "imaginar",
    "imaginareis": "imaginar",
    "imaginaremos": "imaginar",
    "imaginaren": "imaginar",
    "imaginares": "imaginar",
    "imaginaria": "imaginario",
    "imaginarias": "imaginario",
    "imaginarios": "imaginario",
    "imaginaron": "imaginar",
    "imaginará": "imaginar",
    "imaginarán": "imaginar",
    "imaginarás": "imaginar",
    "imaginaré": "imaginar",
    "imaginaréis": "imaginar",
    "imaginaría": "imaginar",
    "imaginaríais": "imaginar",
    "imaginaríamos": "imaginar",
    "imaginarían": "imaginar",
    "imaginarías": "imaginar",
    "imaginas": "imaginar",
    "imaginase": "imaginar",
    "imaginaseis": "imaginar",
    "imaginasen": "imaginar",
    "imaginases": "imaginar",
    "imaginaste": "imaginar",
    "imaginasteis": "imaginar",
    "imaginativa": "imaginativo",
    "imaginativas": "imaginativo",
    "imaginativos": "imaginativo",
    "imagine": "imaginar",
    "imaginemos": "imaginar",
    "imaginen": "imaginar",
    "imaginera": "imaginero",
    "imagineras": "imaginero",
    "imagineros": "imaginero",
    "imaginerías": "imaginería",
    "imagines": "imaginar",
    "imaginismos": "imaginismo",
    "imagino": "imaginar",
    "imaginábamos": "imaginar",
    "imagináis": "imaginar",
    "imagináramos": "imaginar",
    "imagináremos": "imaginar",
    "imaginásemos": "imaginar",
    "imaginé": "imaginar",
    "imaginéis": "imaginar",
    "imaginó": "imaginar",
    "imagos": "imago",
    "imames": "imam",
    "imana": "imanar",
    "imanaba": "imanar",
    "imanabais": "imanar",
    "imanaban": "imanar",
    "imanabas": "imanar",
    "imanaciones": "imanación",
    "imanad": "imanar",
    "imanada": "imanar",
    "imanadas": "imanar",
    "imanado": "imanar",
    "imanados": "imanar",
    "imanamos": "imanar",
    "imanan": "imanar",
    "imanando": "imanar",
    "imanara": "imanar",
    "imanarais": "imanar",
    "imanaran": "imanar",
    "imanaras": "imanar",
    "imanare": "imanar",
    "imanareis": "imanar",
    "imanaremos": "imanar",
    "imanaren": "imanar",
    "imanares": "imanar",
    "imanaron": "imanar",
    "imanará": "imanar",
    "imanarán": "imanar",
    "imanarás": "imanar",
    "imanaré": "imanar",
    "imanaréis": "imanar",
    "imanaría": "imanar",
    "imanaríais": "imanar",
    "imanaríamos": "imanar",
    "imanarían": "imanar",
    "imanarías": "imanar",
    "imanas": "imanar",
    "imanase": "imanar",
    "imanaseis": "imanar",
    "imanasen": "imanar",
    "imanases": "imanar",
    "imanaste": "imanar",
    "imanasteis": "imanar",
    "imane": "imanar",
    "imanemos": "imanar",
    "imanen": "imanar",
    "imanes": "imanar",
    "imano": "imanar",
    "imanta": "imantar",
    "imantaba": "imantar",
    "imantabais": "imantar",
    "imantaban": "imantar",
    "imantabas": "imantar",
    "imantaciones": "imantación",
    "imantad": "imantar",
    "imantada": "imantar",
    "imantadas": "imantar",
    "imantado": "imantar",
    "imantados": "imantar",
    "imantamos": "imantar",
    "imantan": "imantar",
    "imantando": "imantar",
    "imantara": "imantar",
    "imantarais": "imantar",
    "imantaran": "imantar",
    "imantaras": "imantar",
    "imantare": "imantar",
    "imantareis": "imantar",
    "imantaremos": "imantar",
    "imantaren": "imantar",
    "imantares": "imantar",
    "imantaron": "imantar",
    "imantará": "imantar",
    "imantarán": "imantar",
    "imantarás": "imantar",
    "imantaré": "imantar",
    "imantaréis": "imantar",
    "imantaría": "imantar",
    "imantaríais": "imantar",
    "imantaríamos": "imantar",
    "imantarían": "imantar",
    "imantarías": "imantar",
    "imantas": "imantar",
    "imantase": "imantar",
    "imantaseis": "imantar",
    "imantasen": "imantar",
    "imantases": "imantar",
    "imantaste": "imantar",
    "imantasteis": "imantar",
    "imante": "imantar",
    "imantemos": "imantar",
    "imanten": "imantar",
    "imantes": "imantar",
    "imanto": "imantar",
    "imantábamos": "imantar",
    "imantáis": "imantar",
    "imantáramos": "imantar",
    "imantáremos": "imantar",
    "imantásemos": "imantar",
    "imanté": "imantar",
    "imantéis": "imantar",
    "imantó": "imantar",
    "imanábamos": "imanar",
    "imanáis": "imanar",
    "imanáramos": "imanar",
    "imanáremos": "imanar",
    "imanásemos": "imanar",
    "imané": "imanar",
    "imanéis": "imanar",
    "imanó": "imanar",
    "imbatibilidades": "imbatibilidad",
    "imbatibles": "imbatible",
    "imbatida": "imbatido",
    "imbatidas": "imbatido",
    "imbatidos": "imbatido",
    "imbatilidades": "imbatilidad",
    "imbecilidades": "imbecilidad",
    "imbeles": "imbele",
    "imberbes": "imberbe",
    "imbibiciones": "imbibición",
    "imbornales": "imbornal",
    "imborrables": "imborrable",
    "imbrica": "imbricar",
    "imbricaba": "imbricar",
    "imbricabais": "imbricar",
    "imbricaban": "imbricar",
    "imbricabas": "imbricar",
    "imbricaciones": "imbricación",
    "imbricad": "imbricar",
    "imbricada": "imbricar",
    "imbricadas": "imbricar",
    "imbricado": "imbricar",
    "imbricados": "imbricar",
    "imbricamos": "imbricar",
    "imbrican": "imbricar",
    "imbricando": "imbricar",
    "imbricara": "imbricar",
    "imbricarais": "imbricar",
    "imbricaran": "imbricar",
    "imbricaras": "imbricar",
    "imbricare": "imbricar",
    "imbricareis": "imbricar",
    "imbricaremos": "imbricar",
    "imbricaren": "imbricar",
    "imbricares": "imbricar",
    "imbricaron": "imbricar",
    "imbricará": "imbricar",
    "imbricarán": "imbricar",
    "imbricarás": "imbricar",
    "imbricaré": "imbricar",
    "imbricaréis": "imbricar",
    "imbricaría": "imbricar",
    "imbricaríais": "imbricar",
    "imbricaríamos": "imbricar",
    "imbricarían": "imbricar",
    "imbricarías": "imbricar",
    "imbricas": "imbricar",
    "imbricase": "imbricar",
    "imbricaseis": "imbricar",
    "imbricasen": "imbricar",
    "imbricases": "imbricar",
    "imbricaste": "imbricar",
    "imbricasteis": "imbricar",
    "imbrico": "imbricar",
    "imbricábamos": "imbricar",
    "imbricáis": "imbricar",
    "imbricáramos": "imbricar",
    "imbricáremos": "imbricar",
    "imbricásemos": "imbricar",
    "imbricó": "imbricar",
    "imbrique": "imbricar",
    "imbriquemos": "imbricar",
    "imbriquen": "imbricar",
    "imbriques": "imbricar",
    "imbriqué": "imbricar",
    "imbriquéis": "imbricar",
    "imbuid": "imbuir",
    "imbuida": "imbuir",
    "imbuidas": "imbuir",
    "imbuido": "imbuir",
    "imbuidos": "imbuir",
    "imbuimos": "imbuir",
    "imbuiremos": "imbuir",
    "imbuirá": "imbuir",
    "imbuirán": "imbuir",
    "imbuirás": "imbuir",
    "imbuiré": "imbuir",
    "imbuiréis": "imbuir",
    "imbuiría": "imbuir",
    "imbuiríais": "imbuir",
    "imbuiríamos": "imbuir",
    "imbuirían": "imbuir",
    "imbuirías": "imbuir",
    "imbuiste": "imbuir",
    "imbuisteis": "imbuir",
    "imbunches": "imbunche",
    "imbuya": "imbuir",
    "imbuyamos": "imbuir",
    "imbuyan": "imbuir",
    "imbuyas": "imbuir",
    "imbuye": "imbuir",
    "imbuyen": "imbuir",
    "imbuyendo": "imbuir",
    "imbuyera": "imbuir",
    "imbuyerais": "imbuir",
    "imbuyeran": "imbuir",
    "imbuyeras": "imbuir",
    "imbuyere": "imbuir",
    "imbuyereis": "imbuir",
    "imbuyeren": "imbuir",
    "imbuyeres": "imbuir",
    "imbuyeron": "imbuir",
    "imbuyes": "imbuir",
    "imbuyese": "imbuir",
    "imbuyeseis": "imbuir",
    "imbuyesen": "imbuir",
    "imbuyeses": "imbuir",
    "imbuyo": "imbuir",
    "imbuyáis": "imbuir",
    "imbuyéramos": "imbuir",
    "imbuyéremos": "imbuir",
    "imbuyésemos": "imbuir",
    "imbuyó": "imbuir",
    "imbuí": "imbuir",
    "imbuía": "imbuir",
    "imbuíais": "imbuir",
    "imbuíamos": "imbuir",
    "imbuían": "imbuir",
    "imbuías": "imbuir",
    "imbuís": "imbuir",
    "imbéciles": "imbécil",
    "imelas": "imela",
    "imidas": "imida",
    "imidazoles": "imidazol",
    "imidógenos": "imidógeno",
    "iminas": "imina",
    "iminobispropilaminas": "iminobispropilamina",
    "imita": "imitar",
    "imitaba": "imitar",
    "imitabais": "imitar",
    "imitaban": "imitar",
    "imitabas": "imitar",
    "imitables": "imitable",
    "imitaciones": "imitación",
    "imitad": "imitar",
    "imitada": "imitar",
    "imitadas": "imitar",
    "imitado": "imitar",
    "imitadora": "imitador",
    "imitadoras": "imitador",
    "imitadores": "imitador",
    "imitados": "imitar",
    "imitamos": "imitar",
    "imitan": "imitar",
    "imitando": "imitar",
    "imitara": "imitar",
    "imitarais": "imitar",
    "imitaran": "imitar",
    "imitaras": "imitar",
    "imitare": "imitar",
    "imitareis": "imitar",
    "imitaremos": "imitar",
    "imitaren": "imitar",
    "imitares": "imitar",
    "imitaron": "imitar",
    "imitará": "imitar",
    "imitarán": "imitar",
    "imitarás": "imitar",
    "imitaré": "imitar",
    "imitaréis": "imitar",
    "imitaría": "imitar",
    "imitaríais": "imitar",
    "imitaríamos": "imitar",
    "imitarían": "imitar",
    "imitarías": "imitar",
    "imitas": "imitar",
    "imitase": "imitar",
    "imitaseis": "imitar",
    "imitasen": "imitar",
    "imitases": "imitar",
    "imitaste": "imitar",
    "imitasteis": "imitar",
    "imitativa": "imitativo",
    "imitativas": "imitativo",
    "imitativos": "imitativo",
    "imitatoria": "imitatorio",
    "imitatorias": "imitatorio",
    "imitatorios": "imitatorio",
    "imite": "imitar",
    "imitemos": "imitar",
    "imiten": "imitar",
    "imites": "imitar",
    "imito": "imitar",
    "imitábamos": "imitar",
    "imitáis": "imitar",
    "imitáramos": "imitar",
    "imitáremos": "imitar",
    "imitásemos": "imitar",
    "imité": "imitar",
    "imitéis": "imitar",
    "imitó": "imitar",
    "immunodifusiones": "immunodifusión",
    "imoscapos": "imoscapo",
    "impaciencias": "impaciencia",
    "impacienta": "impacientar",
    "impacientaba": "impacientar",
    "impacientabais": "impacientar",
    "impacientaban": "impacientar",
    "impacientabas": "impacientar",
    "impacientad": "impacientar",
    "impacientada": "impacientar",
    "impacientadas": "impacientar",
    "impacientado": "impacientar",
    "impacientados": "impacientar",
    "impacientamos": "impacientar",
    "impacientan": "impacientar",
    "impacientando": "impacientar",
    "impacientara": "impacientar",
    "impacientarais": "impacientar",
    "impacientaran": "impacientar",
    "impacientaras": "impacientar",
    "impacientare": "impacientar",
    "impacientareis": "impacientar",
    "impacientaremos": "impacientar",
    "impacientaren": "impacientar",
    "impacientares": "impacientar",
    "impacientaron": "impacientar",
    "impacientará": "impacientar",
    "impacientarán": "impacientar",
    "impacientarás": "impacientar",
    "impacientaré": "impacientar",
    "impacientaréis": "impacientar",
    "impacientaría": "impacientar",
    "impacientaríais": "impacientar",
    "impacientaríamos": "impacientar",
    "impacientarían": "impacientar",
    "impacientarías": "impacientar",
    "impacientas": "impacientar",
    "impacientase": "impacientar",
    "impacientaseis": "impacientar",
    "impacientasen": "impacientar",
    "impacientases": "impacientar",
    "impacientaste": "impacientar",
    "impacientasteis": "impacientar",
    "impaciente": "impacientar",
    "impacientemos": "impacientar",
    "impacienten": "impacientar",
    "impacientes": "impaciente",
    "impaciento": "impacientar",
    "impacientábamos": "impacientar",
    "impacientáis": "impacientar",
    "impacientáramos": "impacientar",
    "impacientáremos": "impacientar",
    "impacientásemos": "impacientar",
    "impacienté": "impacientar",
    "impacientéis": "impacientar",
    "impacientó": "impacientar",
    "impacta": "impactar",
    "impactaba": "impactar",
    "impactabais": "impactar",
    "impactaban": "impactar",
    "impactabas": "impactar",
    "impactad": "impactar",
    "impactada": "impactar",
    "impactadas": "impactar",
    "impactado": "impactar",
    "impactados": "impactar",
    "impactamos": "impactar",
    "impactan": "impactar",
    "impactando": "impactar",
    "impactantes": "impactante",
    "impactara": "impactar",
    "impactarais": "impactar",
    "impactaran": "impactar",
    "impactaras": "impactar",
    "impactare": "impactar",
    "impactareis": "impactar",
    "impactaremos": "impactar",
    "impactaren": "impactar",
    "impactares": "impactar",
    "impactaron": "impactar",
    "impactará": "impactar",
    "impactarán": "impactar",
    "impactarás": "impactar",
    "impactaré": "impactar",
    "impactaréis": "impactar",
    "impactaría": "impactar",
    "impactaríais": "impactar",
    "impactaríamos": "impactar",
    "impactarían": "impactar",
    "impactarías": "impactar",
    "impactas": "impactar",
    "impactase": "impactar",
    "impactaseis": "impactar",
    "impactasen": "impactar",
    "impactases": "impactar",
    "impactaste": "impactar",
    "impactasteis": "impactar",
    "impacte": "impactar",
    "impactemos": "impactar",
    "impacten": "impactar",
    "impactes": "impactar",
    "impactitas": "impactita",
    "impacto": "impactar",
    "impactos": "impacto",
    "impactábamos": "impactar",
    "impactáis": "impactar",
    "impactáramos": "impactar",
    "impactáremos": "impactar",
    "impactásemos": "impactar",
    "impacté": "impactar",
    "impactéis": "impactar",
    "impactó": "impactar",
    "impaga": "impago",
    "impagables": "impagable",
    "impagada": "impagado",
    "impagadas": "impagado",
    "impagados": "impagado",
    "impagas": "impago",
    "impagos": "impago",
    "impalas": "impala",
    "impalpables": "impalpable",
    "imparables": "imparable",
    "imparciales": "imparcial",
    "imparcialidades": "imparcialidad",
    "impares": "impar",
    "imparipinnada": "imparipinnado",
    "imparipinnadas": "imparipinnado",
    "imparipinnados": "imparipinnado",
    "imparisílaba": "imparisílabo",
    "imparisílabas": "imparisílabo",
    "imparisílabos": "imparisílabo",
    "imparta": "impartir",
    "impartamos": "impartir",
    "impartan": "impartir",
    "impartas": "impartir",
    "imparte": "impartir",
    "imparten": "impartir",
    "impartes": "impartir",
    "impartibles": "impartible",
    "imparticiones": "impartición",
    "impartid": "impartir",
    "impartida": "impartir",
    "impartidas": "impartir",
    "impartido": "impartir",
    "impartidos": "impartir",
    "impartiendo": "impartir",
    "impartiera": "impartir",
    "impartierais": "impartir",
    "impartieran": "impartir",
    "impartieras": "impartir",
    "impartiere": "impartir",
    "impartiereis": "impartir",
    "impartieren": "impartir",
    "impartieres": "impartir",
    "impartieron": "impartir",
    "impartiese": "impartir",
    "impartieseis": "impartir",
    "impartiesen": "impartir",
    "impartieses": "impartir",
    "impartimos": "impartir",
    "impartiremos": "impartir",
    "impartirá": "impartir",
    "impartirán": "impartir",
    "impartirás": "impartir",
    "impartiré": "impartir",
    "impartiréis": "impartir",
    "impartiría": "impartir",
    "impartiríais": "impartir",
    "impartiríamos": "impartir",
    "impartirían": "impartir",
    "impartirías": "impartir",
    "impartiste": "impartir",
    "impartisteis": "impartir",
    "impartiéramos": "impartir",
    "impartiéremos": "impartir",
    "impartiésemos": "impartir",
    "impartió": "impartir",
    "imparto": "impartir",
    "impartáis": "impartir",
    "impartí": "impartir",
    "impartía": "impartir",
    "impartíais": "impartir",
    "impartíamos": "impartir",
    "impartían": "impartir",
    "impartías": "impartir",
    "impartís": "impartir",
    "impasibilidades": "impasibilidad",
    "impasibles": "impasible",
    "impasses": "impasse",
    "impavideces": "impavidez",
    "impeachmentes": "impeachment",
    "impeachments": "impeachment",
    "impecables": "impecable",
    "impedancias": "impedancia",
    "impedanciometrías": "impedanciometría",
    "impedid": "impedir",
    "impedida": "impedir",
    "impedidas": "impedir",
    "impedido": "impedir",
    "impedidos": "impedir",
    "impedimentas": "impedimenta",
    "impedimentos": "impedimento",
    "impedimos": "impedir",
    "impediremos": "impedir",
    "impedirá": "impedir",
    "impedirán": "impedir",
    "impedirás": "impedir",
    "impediré": "impedir",
    "impediréis": "impedir",
    "impediría": "impedir",
    "impediríais": "impedir",
    "impediríamos": "impedir",
    "impedirían": "impedir",
    "impedirías": "impedir",
    "impediste": "impedir",
    "impedisteis": "impedir",
    "impeditiva": "impeditivo",
    "impeditivas": "impeditivo",
    "impeditivos": "impeditivo",
    "impedores": "impedor",
    "impedí": "impedir",
    "impedía": "impedir",
    "impedíais": "impedir",
    "impedíamos": "impedir",
    "impedían": "impedir",
    "impedías": "impedir",
    "impedís": "impedir",
    "impela": "impeler",
    "impelamos": "impeler",
    "impelan": "impeler",
    "impelas": "impeler",
    "impele": "impeler",
    "impeled": "impeler",
    "impelemos": "impeler",
    "impelen": "impeler",
    "impelentes": "impelente",
    "impeleremos": "impeler",
    "impelerá": "impeler",
    "impelerán": "impeler",
    "impelerás": "impeler",
    "impeleré": "impeler",
    "impeleréis": "impeler",
    "impelería": "impeler",
    "impeleríais": "impeler",
    "impeleríamos": "impeler",
    "impelerían": "impeler",
    "impelerías": "impeler",
    "impeles": "impeler",
    "impelida": "impeler",
    "impelidas": "impeler",
    "impelido": "impeler",
    "impelidos": "impeler",
    "impeliendo": "impeler",
    "impeliera": "impeler",
    "impelierais": "impeler",
    "impelieran": "impeler",
    "impelieras": "impeler",
    "impeliere": "impeler",
    "impeliereis": "impeler",
    "impelieren": "impeler",
    "impelieres": "impeler",
    "impelieron": "impeler",
    "impeliese": "impeler",
    "impelieseis": "impeler",
    "impeliesen": "impeler",
    "impelieses": "impeler",
    "impelimos": "impeler",
    "impeliste": "impeler",
    "impelisteis": "impeler",
    "impeliéramos": "impeler",
    "impeliéremos": "impeler",
    "impeliésemos": "impeler",
    "impelió": "impeler",
    "impelo": "impeler",
    "impeláis": "impeler",
    "impeléis": "impeler",
    "impelí": "impeler",
    "impelía": "impeler",
    "impelíais": "impeler",
    "impelíamos": "impeler",
    "impelían": "impeler",
    "impelías": "impeler",
    "impenda": "impender",
    "impendamos": "impender",
    "impendan": "impender",
    "impendas": "impender",
    "impende": "impender",
    "impended": "impender",
    "impendemos": "impender",
    "impenden": "impender",
    "impenderemos": "impender",
    "impenderá": "impender",
    "impenderán": "impender",
    "impenderás": "impender",
    "impenderé": "impender",
    "impenderéis": "impender",
    "impendería": "impender",
    "impenderíais": "impender",
    "impenderíamos": "impender",
    "impenderían": "impender",
    "impenderías": "impender",
    "impendes": "impender",
    "impendida": "impender",
    "impendidas": "impender",
    "impendido": "impender",
    "impendidos": "impender",
    "impendiendo": "impender",
    "impendiera": "impender",
    "impendierais": "impender",
    "impendieran": "impender",
    "impendieras": "impender",
    "impendiere": "impender",
    "impendiereis": "impender",
    "impendieren": "impender",
    "impendieres": "impender",
    "impendieron": "impender",
    "impendiese": "impender",
    "impendieseis": "impender",
    "impendiesen": "impender",
    "impendieses": "impender",
    "impendimos": "impender",
    "impendiste": "impender",
    "impendisteis": "impender",
    "impendiéramos": "impender",
    "impendiéremos": "impender",
    "impendiésemos": "impender",
    "impendió": "impender",
    "impendo": "impender",
    "impendáis": "impender",
    "impendéis": "impender",
    "impendí": "impender",
    "impendía": "impender",
    "impendíais": "impender",
    "impendíamos": "impender",
    "impendían": "impender",
    "impendías": "impender",
    "impenetrabilidades": "impenetrabilidad",
    "impenetrables": "impenetrable",
    "impenitencias": "impenitencia",
    "impenitentes": "impenitente",
    "impensables": "impensable",
    "impensada": "impensado",
    "impensadas": "impensado",
    "impensados": "impensado",
    "impensas": "impensa",
    "impepinables": "impepinable",
    "impera": "imperar",
    "imperaba": "imperar",
    "imperabais": "imperar",
    "imperaban": "imperar",
    "imperabas": "imperar",
    "imperad": "imperar",
    "imperada": "imperar",
    "imperadas": "imperar",
    "imperado": "imperar",
    "imperadora": "imperador",
    "imperadoras": "imperador",
    "imperadores": "imperador",
    "imperados": "imperar",
    "imperamos": "imperar",
    "imperan": "imperar",
    "imperando": "imperar",
    "imperantes": "imperante",
    "imperara": "imperar",
    "imperarais": "imperar",
    "imperaran": "imperar",
    "imperaras": "imperar",
    "imperare": "imperar",
    "imperareis": "imperar",
    "imperaremos": "imperar",
    "imperaren": "imperar",
    "imperares": "imperar",
    "imperaron": "imperar",
    "imperará": "imperar",
    "imperarán": "imperar",
    "imperarás": "imperar",
    "imperaré": "imperar",
    "imperaréis": "imperar",
    "imperaría": "imperar",
    "imperaríais": "imperar",
    "imperaríamos": "imperar",
    "imperarían": "imperar",
    "imperarías": "imperar",
    "imperas": "imperar",
    "imperase": "imperar",
    "imperaseis": "imperar",
    "imperasen": "imperar",
    "imperases": "imperar",
    "imperaste": "imperar",
    "imperasteis": "imperar",
    "imperativa": "imperativo",
    "imperativas": "imperativo",
    "imperativos": "imperativo",
    "imperatoria": "imperatorio",
    "imperatorias": "imperatorio",
    "imperatorios": "imperatorio",
    "imperceptibles": "imperceptible",
    "imperdibles": "imperdible",
    "imperdonables": "imperdonable",
    "impere": "imperar",
    "imperecedera": "imperecedero",
    "imperecederas": "imperecedero",
    "imperecederos": "imperecedero",
    "imperemos": "imperar",
    "imperen": "imperar",
    "imperes": "imperar",
    "imperfecciones": "imperfección",
    "imperfecta": "imperfecto",
    "imperfectas": "imperfecto",
    "imperfectiva": "imperfectivo",
    "imperfectivas": "imperfectivo",
    "imperfectivos": "imperfectivo",
    "imperfectos": "imperfecto",
    "imperforaciones": "imperforación",
    "imperiales": "imperial",
    "imperialismos": "imperialismo",
    "imperialistas": "imperialista",
    "impericias": "impericia",
    "imperios": "imperio",
    "imperiosa": "imperioso",
    "imperiosas": "imperioso",
    "imperiosos": "imperioso",
    "imperita": "imperito",
    "imperitas": "imperito",
    "imperitos": "imperito",
    "impermeabilice": "impermeabilizar",
    "impermeabilicemos": "impermeabilizar",
    "impermeabilicen": "impermeabilizar",
    "impermeabilices": "impermeabilizar",
    "impermeabilicé": "impermeabilizar",
    "impermeabilicéis": "impermeabilizar",
    "impermeabilidades": "impermeabilidad",
    "impermeabiliza": "impermeabilizar",
    "impermeabilizaba": "impermeabilizar",
    "impermeabilizabais": "impermeabilizar",
    "impermeabilizaban": "impermeabilizar",
    "impermeabilizabas": "impermeabilizar",
    "impermeabilizaciones": "impermeabilización",
    "impermeabilizad": "impermeabilizar",
    "impermeabilizada": "impermeabilizar",
    "impermeabilizadas": "impermeabilizar",
    "impermeabilizado": "impermeabilizar",
    "impermeabilizados": "impermeabilizar",
    "impermeabilizamos": "impermeabilizar",
    "impermeabilizan": "impermeabilizar",
    "impermeabilizando": "impermeabilizar",
    "impermeabilizara": "impermeabilizar",
    "impermeabilizarais": "impermeabilizar",
    "impermeabilizaran": "impermeabilizar",
    "impermeabilizaras": "impermeabilizar",
    "impermeabilizare": "impermeabilizar",
    "impermeabilizareis": "impermeabilizar",
    "impermeabilizaremos": "impermeabilizar",
    "impermeabilizaren": "impermeabilizar",
    "impermeabilizares": "impermeabilizar",
    "impermeabilizaron": "impermeabilizar",
    "impermeabilizará": "impermeabilizar",
    "impermeabilizarán": "impermeabilizar",
    "impermeabilizarás": "impermeabilizar",
    "impermeabilizaré": "impermeabilizar",
    "impermeabilizaréis": "impermeabilizar",
    "impermeabilizaría": "impermeabilizar",
    "impermeabilizaríais": "impermeabilizar",
    "impermeabilizaríamos": "impermeabilizar",
    "impermeabilizarían": "impermeabilizar",
    "impermeabilizarías": "impermeabilizar",
    "impermeabilizas": "impermeabilizar",
    "impermeabilizase": "impermeabilizar",
    "impermeabilizaseis": "impermeabilizar",
    "impermeabilizasen": "impermeabilizar",
    "impermeabilizases": "impermeabilizar",
    "impermeabilizaste": "impermeabilizar",
    "impermeabilizasteis": "impermeabilizar",
    "impermeabilizo": "impermeabilizar",
    "impermeabilizábamos": "impermeabilizar",
    "impermeabilizáis": "impermeabilizar",
    "impermeabilizáramos": "impermeabilizar",
    "impermeabilizáremos": "impermeabilizar",
    "impermeabilizásemos": "impermeabilizar",
    "impermeabilizó": "impermeabilizar",
    "impermeables": "impermeable",
    "impermutabilidades": "impermutabilidad",
    "impermutables": "impermutable",
    "impero": "imperar",
    "impersonales": "impersonal",
    "impersonalice": "impersonalizar",
    "impersonalicemos": "impersonalizar",
    "impersonalicen": "impersonalizar",
    "impersonalices": "impersonalizar",
    "impersonalicé": "impersonalizar",
    "impersonalicéis": "impersonalizar",
    "impersonaliza": "impersonalizar",
    "impersonalizaba": "impersonalizar",
    "impersonalizabais": "impersonalizar",
    "impersonalizaban": "impersonalizar",
    "impersonalizabas": "impersonalizar",
    "impersonalizad": "impersonalizar",
    "impersonalizada": "impersonalizar",
    "impersonalizadas": "impersonalizar",
    "impersonalizado": "impersonalizar",
    "impersonalizados": "impersonalizar",
    "impersonalizamos": "impersonalizar",
    "impersonalizan": "impersonalizar",
    "impersonalizando": "impersonalizar",
    "impersonalizara": "impersonalizar",
    "impersonalizarais": "impersonalizar",
    "impersonalizaran": "impersonalizar",
    "impersonalizaras": "impersonalizar",
    "impersonalizare": "impersonalizar",
    "impersonalizareis": "impersonalizar",
    "impersonalizaremos": "impersonalizar",
    "impersonalizaren": "impersonalizar",
    "impersonalizares": "impersonalizar",
    "impersonalizaron": "impersonalizar",
    "impersonalizará": "impersonalizar",
    "impersonalizarán": "impersonalizar",
    "impersonalizarás": "impersonalizar",
    "impersonalizaré": "impersonalizar",
    "impersonalizaréis": "impersonalizar",
    "impersonalizaría": "impersonalizar",
    "impersonalizaríais": "impersonalizar",
    "impersonalizaríamos": "impersonalizar",
    "impersonalizarían": "impersonalizar",
    "impersonalizarías": "impersonalizar",
    "impersonalizas": "impersonalizar",
    "impersonalizase": "impersonalizar",
    "impersonalizaseis": "impersonalizar",
    "impersonalizasen": "impersonalizar",
    "impersonalizases": "impersonalizar",
    "impersonalizaste": "impersonalizar",
    "impersonalizasteis": "impersonalizar",
    "impersonalizo": "impersonalizar",
    "impersonalizábamos": "impersonalizar",
    "impersonalizáis": "impersonalizar",
    "impersonalizáramos": "impersonalizar",
    "impersonalizáremos": "impersonalizar",
    "impersonalizásemos": "impersonalizar",
    "impersonalizó": "impersonalizar",
    "impertinencias": "impertinencia",
    "impertinentes": "impertinente",
    "imperturbabilidades": "imperturbabilidad",
    "imperturbables": "imperturbable",
    "impertérrita": "impertérrito",
    "impertérritas": "impertérrito",
    "impertérritos": "impertérrito",
    "imperábamos": "imperar",
    "imperáis": "imperar",
    "imperáramos": "imperar",
    "imperáremos": "imperar",
    "imperásemos": "imperar",
    "imperé": "imperar",
    "imperéis": "imperar",
    "imperó": "imperar",
    "impetra": "impetrar",
    "impetraba": "impetrar",
    "impetrabais": "impetrar",
    "impetraban": "impetrar",
    "impetrabas": "impetrar",
    "impetraciones": "impetración",
    "impetrad": "impetrar",
    "impetrada": "impetrar",
    "impetradas": "impetrar",
    "impetrado": "impetrar",
    "impetradora": "impetrador",
    "impetradoras": "impetrador",
    "impetradores": "impetrador",
    "impetrados": "impetrar",
    "impetramos": "impetrar",
    "impetran": "impetrar",
    "impetrando": "impetrar",
    "impetrara": "impetrar",
    "impetrarais": "impetrar",
    "impetraran": "impetrar",
    "impetraras": "impetrar",
    "impetrare": "impetrar",
    "impetrareis": "impetrar",
    "impetraremos": "impetrar",
    "impetraren": "impetrar",
    "impetrares": "impetrar",
    "impetraron": "impetrar",
    "impetrará": "impetrar",
    "impetrarán": "impetrar",
    "impetrarás": "impetrar",
    "impetraré": "impetrar",
    "impetraréis": "impetrar",
    "impetraría": "impetrar",
    "impetraríais": "impetrar",
    "impetraríamos": "impetrar",
    "impetrarían": "impetrar",
    "impetrarías": "impetrar",
    "impetras": "impetrar",
    "impetrase": "impetrar",
    "impetraseis": "impetrar",
    "impetrasen": "impetrar",
    "impetrases": "impetrar",
    "impetraste": "impetrar",
    "impetrasteis": "impetrar",
    "impetratoria": "impetratorio",
    "impetratorias": "impetratorio",
    "impetratorios": "impetratorio",
    "impetre": "impetrar",
    "impetremos": "impetrar",
    "impetren": "impetrar",
    "impetres": "impetrar",
    "impetro": "impetrar",
    "impetrábamos": "impetrar",
    "impetráis": "impetrar",
    "impetráramos": "impetrar",
    "impetráremos": "impetrar",
    "impetrásemos": "impetrar",
    "impetré": "impetrar",
    "impetréis": "impetrar",
    "impetró": "impetrar",
    "impetuosa": "impetuoso",
    "impetuosas": "impetuoso",
    "impetuosidades": "impetuosidad",
    "impetuosos": "impetuoso",
    "impida": "impedir",
    "impidamos": "impedir",
    "impidan": "impedir",
    "impidas": "impedir",
    "impide": "impedir",
    "impiden": "impedir",
    "impides": "impedir",
    "impidiendo": "impedir",
    "impidiera": "impedir",
    "impidierais": "impedir",
    "impidieran": "impedir",
    "impidieras": "impedir",
    "impidiere": "impedir",
    "impidiereis": "impedir",
    "impidieren": "impedir",
    "impidieres": "impedir",
    "impidieron": "impedir",
    "impidiese": "impedir",
    "impidieseis": "impedir",
    "impidiesen": "impedir",
    "impidieses": "impedir",
    "impidiéramos": "impedir",
    "impidiéremos": "impedir",
    "impidiésemos": "impedir",
    "impidió": "impedir",
    "impido": "impedir",
    "impidáis": "impedir",
    "impiedades": "impiedad",
    "implacables": "implacable",
    "implanta": "implantar",
    "implantaba": "implantar",
    "implantabais": "implantar",
    "implantaban": "implantar",
    "implantabas": "implantar",
    "implantaciones": "implantación",
    "implantad": "implantar",
    "implantada": "implantar",
    "implantadas": "implantar",
    "implantado": "implantar",
    "implantadora": "implantador",
    "implantadoras": "implantador",
    "implantadores": "implantador",
    "implantados": "implantar",
    "implantamos": "implantar",
    "implantan": "implantar",
    "implantando": "implantar",
    "implantara": "implantar",
    "implantarais": "implantar",
    "implantaran": "implantar",
    "implantaras": "implantar",
    "implantare": "implantar",
    "implantareis": "implantar",
    "implantaremos": "implantar",
    "implantaren": "implantar",
    "implantares": "implantar",
    "implantaron": "implantar",
    "implantará": "implantar",
    "implantarán": "implantar",
    "implantarás": "implantar",
    "implantaré": "implantar",
    "implantaréis": "implantar",
    "implantaría": "implantar",
    "implantaríais": "implantar",
    "implantaríamos": "implantar",
    "implantarían": "implantar",
    "implantarías": "implantar",
    "implantas": "implantar",
    "implantase": "implantar",
    "implantaseis": "implantar",
    "implantasen": "implantar",
    "implantases": "implantar",
    "implantaste": "implantar",
    "implantasteis": "implantar",
    "implante": "implantar",
    "implantemos": "implantar",
    "implanten": "implantar",
    "implantes": "implante",
    "implanto": "implantar",
    "implantologías": "implantología",
    "implantábamos": "implantar",
    "implantáis": "implantar",
    "implantáramos": "implantar",
    "implantáremos": "implantar",
    "implantásemos": "implantar",
    "implanté": "implantar",
    "implantéis": "implantar",
    "implantó": "implantar",
    "implas": "impla",
    "implementa": "implementar",
    "implementaba": "implementar",
    "implementabais": "implementar",
    "implementaban": "implementar",
    "implementabas": "implementar",
    "implementaciones": "implementación",
    "implementad": "implementar",
    "implementada": "implementar",
    "implementadas": "implementar",
    "implementado": "implementar",
    "implementadora": "implementador",
    "implementadoras": "implementador",
    "implementadores": "implementador",
    "implementados": "implementar",
    "implementamos": "implementar",
    "implementan": "implementar",
    "implementando": "implementar",
    "implementara": "implementar",
    "implementarais": "implementar",
    "implementaran": "implementar",
    "implementaras": "implementar",
    "implementare": "implementar",
    "implementareis": "implementar",
    "implementaremos": "implementar",
    "implementaren": "implementar",
    "implementares": "implementar",
    "implementaron": "implementar",
    "implementará": "implementar",
    "implementarán": "implementar",
    "implementarás": "implementar",
    "implementaré": "implementar",
    "implementaréis": "implementar",
    "implementaría": "implementar",
    "implementaríais": "implementar",
    "implementaríamos": "implementar",
    "implementarían": "implementar",
    "implementarías": "implementar",
    "implementas": "implementar",
    "implementase": "implementar",
    "implementaseis": "implementar",
    "implementasen": "implementar",
    "implementases": "implementar",
    "implementaste": "implementar",
    "implementasteis": "implementar",
    "implemente": "implementar",
    "implementemos": "implementar",
    "implementen": "implementar",
    "implementes": "implementar",
    "implemento": "implementar",
    "implementos": "implemento",
    "implementábamos": "implementar",
    "implementáis": "implementar",
    "implementáramos": "implementar",
    "implementáremos": "implementar",
    "implementásemos": "implementar",
    "implementé": "implementar",
    "implementéis": "implementar",
    "implementó": "implementar",
    "implica": "implicar",
    "implicaba": "implicar",
    "implicabais": "implicar",
    "implicaban": "implicar",
    "implicabas": "implicar",
    "implicaciones": "implicación",
    "implicad": "implicar",
    "implicada": "implicar",
    "implicadas": "implicar",
    "implicado": "implicar",
    "implicados": "implicar",
    "implicaduras": "implicadura",
    "implicamos": "implicar",
    "implican": "implicar",
    "implicancias": "implicancia",
    "implicando": "implicar",
    "implicara": "implicar",
    "implicarais": "implicar",
    "implicaran": "implicar",
    "implicaras": "implicar",
    "implicare": "implicar",
    "implicareis": "implicar",
    "implicaremos": "implicar",
    "implicaren": "implicar",
    "implicares": "implicar",
    "implicaron": "implicar",
    "implicará": "implicar",
    "implicarán": "implicar",
    "implicarás": "implicar",
    "implicaré": "implicar",
    "implicaréis": "implicar",
    "implicaría": "implicar",
    "implicaríais": "implicar",
    "implicaríamos": "implicar",
    "implicarían": "implicar",
    "implicarías": "implicar",
    "implicas": "implicar",
    "implicase": "implicar",
    "implicaseis": "implicar",
    "implicasen": "implicar",
    "implicases": "implicar",
    "implicaste": "implicar",
    "implicasteis": "implicar",
    "implicativa": "implicativo",
    "implicativas": "implicativo",
    "implicativos": "implicativo",
    "implicatoria": "implicatorio",
    "implicatorias": "implicatorio",
    "implicatorios": "implicatorio",
    "implico": "implicar",
    "implicábamos": "implicar",
    "implicáis": "implicar",
    "implicáramos": "implicar",
    "implicáremos": "implicar",
    "implicásemos": "implicar",
    "implicó": "implicar",
    "implique": "implicar",
    "impliquemos": "implicar",
    "impliquen": "implicar",
    "impliques": "implicar",
    "impliqué": "implicar",
    "impliquéis": "implicar",
    "implora": "implorar",
    "imploraba": "implorar",
    "implorabais": "implorar",
    "imploraban": "implorar",
    "implorabas": "implorar",
    "imploraciones": "imploración",
    "implorad": "implorar",
    "implorada": "implorar",
    "imploradas": "implorar",
    "implorado": "implorar",
    "imploradora": "implorador",
    "imploradoras": "implorador",
    "imploradores": "implorador",
    "implorados": "implorar",
    "imploramos": "implorar",
    "imploran": "implorar",
    "implorando": "implorar",
    "implorara": "implorar",
    "implorarais": "implorar",
    "imploraran": "implorar",
    "imploraras": "implorar",
    "implorare": "implorar",
    "implorareis": "implorar",
    "imploraremos": "implorar",
    "imploraren": "implorar",
    "implorares": "implorar",
    "imploraron": "implorar",
    "implorará": "implorar",
    "implorarán": "implorar",
    "implorarás": "implorar",
    "imploraré": "implorar",
    "imploraréis": "implorar",
    "imploraría": "implorar",
    "imploraríais": "implorar",
    "imploraríamos": "implorar",
    "implorarían": "implorar",
    "implorarías": "implorar",
    "imploras": "implorar",
    "implorase": "implorar",
    "imploraseis": "implorar",
    "implorasen": "implorar",
    "implorases": "implorar",
    "imploraste": "implorar",
    "implorasteis": "implorar",
    "implore": "implorar",
    "imploremos": "implorar",
    "imploren": "implorar",
    "implores": "implorar",
    "imploro": "implorar",
    "implorábamos": "implorar",
    "imploráis": "implorar",
    "imploráramos": "implorar",
    "imploráremos": "implorar",
    "implorásemos": "implorar",
    "imploré": "implorar",
    "imploréis": "implorar",
    "imploró": "implorar",
    "implosiones": "implosión",
    "implosiva": "implosivo",
    "implosivas": "implosivo",
    "implosivos": "implosivo",
    "implumes": "implume",
    "impluvios": "impluvio",
    "implícita": "implícito",
    "implícitas": "implícito",
    "implícitos": "implícito",
    "impolarizables": "impolarizable",
    "impoluta": "impoluto",
    "impolutas": "impoluto",
    "impolutos": "impoluto",
    "impolítica": "impolítico",
    "impolíticas": "impolítico",
    "impolíticos": "impolítico",
    "imponderabilidades": "imponderabilidad",
    "imponderables": "imponderable",
    "impondremos": "imponer",
    "impondrá": "imponer",
    "impondrán": "imponer",
    "impondrás": "imponer",
    "impondré": "imponer",
    "impondréis": "imponer",
    "impondría": "imponer",
    "impondríais": "imponer",
    "impondríamos": "imponer",
    "impondrían": "imponer",
    "impondrías": "imponer",
    "impone": "imponer",
    "imponed": "imponer",
    "imponedora": "imponedor",
    "imponedoras": "imponedor",
    "imponedores": "imponedor",
    "imponemos": "imponer",
    "imponen": "imponer",
    "imponencias": "imponencia",
    "imponentes": "imponente",
    "impones": "imponer",
    "imponga": "imponer",
    "impongamos": "imponer",
    "impongan": "imponer",
    "impongas": "imponer",
    "impongo": "imponer",
    "impongáis": "imponer",
    "imponibilidades": "imponibilidad",
    "imponibles": "imponible",
    "imponiendo": "imponer",
    "imponéis": "imponer",
    "imponía": "imponer",
    "imponíais": "imponer",
    "imponíamos": "imponer",
    "imponían": "imponer",
    "imponías": "imponer",
    "impopulares": "impopular",
    "impopularidades": "impopularidad",
    "importa": "importar",
    "importaba": "importar",
    "importabais": "importar",
    "importaban": "importar",
    "importabas": "importar",
    "importables": "importable",
    "importaciones": "importación",
    "importad": "importar",
    "importada": "importar",
    "importadas": "importar",
    "importado": "importar",
    "importadora": "importador",
    "importadoras": "importador",
    "importadores": "importador",
    "importados": "importar",
    "importamos": "importar",
    "importan": "importar",
    "importancias": "importancia",
    "importando": "importar",
    "importantes": "importante",
    "importara": "importar",
    "importarais": "importar",
    "importaran": "importar",
    "importaras": "importar",
    "importare": "importar",
    "importareis": "importar",
    "importaremos": "importar",
    "importaren": "importar",
    "importares": "importar",
    "importaron": "importar",
    "importará": "importar",
    "importarán": "importar",
    "importarás": "importar",
    "importaré": "importar",
    "importaréis": "importar",
    "importaría": "importar",
    "importaríais": "importar",
    "importaríamos": "importar",
    "importarían": "importar",
    "importarías": "importar",
    "importas": "importar",
    "importase": "importar",
    "importaseis": "importar",
    "importasen": "importar",
    "importases": "importar",
    "importaste": "importar",
    "importasteis": "importar",
    "importe": "importar",
    "importemos": "importar",
    "importen": "importar",
    "importes": "importe",
    "importo": "importar",
    "importuna": "importuno",
    "importunaba": "importunar",
    "importunabais": "importunar",
    "importunaban": "importunar",
    "importunabas": "importunar",
    "importunaciones": "importunación",
    "importunad": "importunar",
    "importunada": "importunar",
    "importunadas": "importunar",
    "importunado": "importunar",
    "importunados": "importunar",
    "importunamos": "importunar",
    "importunan": "importunar",
    "importunando": "importunar",
    "importunara": "importunar",
    "importunarais": "importunar",
    "importunaran": "importunar",
    "importunaras": "importunar",
    "importunare": "importunar",
    "importunareis": "importunar",
    "importunaremos": "importunar",
    "importunaren": "importunar",
    "importunares": "importunar",
    "importunaron": "importunar",
    "importunará": "importunar",
    "importunarán": "importunar",
    "importunarás": "importunar",
    "importunaré": "importunar",
    "importunaréis": "importunar",
    "importunaría": "importunar",
    "importunaríais": "importunar",
    "importunaríamos": "importunar",
    "importunarían": "importunar",
    "importunarías": "importunar",
    "importunas": "importuno",
    "importunase": "importunar",
    "importunaseis": "importunar",
    "importunasen": "importunar",
    "importunases": "importunar",
    "importunaste": "importunar",
    "importunasteis": "importunar",
    "importune": "importunar",
    "importunemos": "importunar",
    "importunen": "importunar",
    "importunes": "importunar",
    "importunidades": "importunidad",
    "importuno": "importunar",
    "importunos": "importuno",
    "importunábamos": "importunar",
    "importunáis": "importunar",
    "importunáramos": "importunar",
    "importunáremos": "importunar",
    "importunásemos": "importunar",
    "importuné": "importunar",
    "importunéis": "importunar",
    "importunó": "importunar",
    "importábamos": "importar",
    "importáis": "importar",
    "importáramos": "importar",
    "importáremos": "importar",
    "importásemos": "importar",
    "importé": "importar",
    "importéis": "importar",
    "importó": "importar",
    "imposibilidades": "imposibilidad",
    "imposibilita": "imposibilitar",
    "imposibilitaba": "imposibilitar",
    "imposibilitabais": "imposibilitar",
    "imposibilitaban": "imposibilitar",
    "imposibilitabas": "imposibilitar",
    "imposibilitad": "imposibilitar",
    "imposibilitada": "imposibilitar",
    "imposibilitadas": "imposibilitar",
    "imposibilitado": "imposibilitar",
    "imposibilitados": "imposibilitar",
    "imposibilitamos": "imposibilitar",
    "imposibilitan": "imposibilitar",
    "imposibilitando": "imposibilitar",
    "imposibilitara": "imposibilitar",
    "imposibilitarais": "imposibilitar",
    "imposibilitaran": "imposibilitar",
    "imposibilitaras": "imposibilitar",
    "imposibilitare": "imposibilitar",
    "imposibilitareis": "imposibilitar",
    "imposibilitaremos": "imposibilitar",
    "imposibilitaren": "imposibilitar",
    "imposibilitares": "imposibilitar",
    "imposibilitaron": "imposibilitar",
    "imposibilitará": "imposibilitar",
    "imposibilitarán": "imposibilitar",
    "imposibilitarás": "imposibilitar",
    "imposibilitaré": "imposibilitar",
    "imposibilitaréis": "imposibilitar",
    "imposibilitaría": "imposibilitar",
    "imposibilitaríais": "imposibilitar",
    "imposibilitaríamos": "imposibilitar",
    "imposibilitarían": "imposibilitar",
    "imposibilitarías": "imposibilitar",
    "imposibilitas": "imposibilitar",
    "imposibilitase": "imposibilitar",
    "imposibilitaseis": "imposibilitar",
    "imposibilitasen": "imposibilitar",
    "imposibilitases": "imposibilitar",
    "imposibilitaste": "imposibilitar",
    "imposibilitasteis": "imposibilitar",
    "imposibilite": "imposibilitar",
    "imposibilitemos": "imposibilitar",
    "imposibiliten": "imposibilitar",
    "imposibilites": "imposibilitar",
    "imposibilito": "imposibilitar",
    "imposibilitábamos": "imposibilitar",
    "imposibilitáis": "imposibilitar",
    "imposibilitáramos": "imposibilitar",
    "imposibilitáremos": "imposibilitar",
    "imposibilitásemos": "imposibilitar",
    "imposibilité": "imposibilitar",
    "imposibilitéis": "imposibilitar",
    "imposibilitó": "imposibilitar",
    "imposibles": "imposible",
    "imposiciones": "imposición",
    "impositiva": "impositivo",
    "impositivas": "impositivo",
    "impositivos": "impositivo",
    "impositora": "impositor",
    "impositoras": "impositor",
    "impositores": "impositor",
    "imposta": "impostar",
    "impostaba": "impostar",
    "impostabais": "impostar",
    "impostaban": "impostar",
    "impostabas": "impostar",
    "impostaciones": "impostación",
    "impostad": "impostar",
    "impostada": "impostar",
    "impostadas": "impostar",
    "impostado": "impostar",
    "impostados": "impostar",
    "impostamos": "impostar",
    "impostan": "impostar",
    "impostando": "impostar",
    "impostara": "impostar",
    "impostarais": "impostar",
    "impostaran": "impostar",
    "impostaras": "impostar",
    "impostare": "impostar",
    "impostareis": "impostar",
    "impostaremos": "impostar",
    "impostaren": "impostar",
    "impostares": "impostar",
    "impostaron": "impostar",
    "impostará": "impostar",
    "impostarán": "impostar",
    "impostarás": "impostar",
    "impostaré": "impostar",
    "impostaréis": "impostar",
    "impostaría": "impostar",
    "impostaríais": "impostar",
    "impostaríamos": "impostar",
    "impostarían": "impostar",
    "impostarías": "impostar",
    "impostas": "impostar",
    "impostase": "impostar",
    "impostaseis": "impostar",
    "impostasen": "impostar",
    "impostases": "impostar",
    "impostaste": "impostar",
    "impostasteis": "impostar",
    "imposte": "impostar",
    "impostemos": "impostar",
    "imposten": "impostar",
    "impostes": "impostar",
    "imposto": "impostar",
    "impostora": "impostor",
    "impostoras": "impostor",
    "impostores": "impostor",
    "imposturas": "impostura",
    "impostábamos": "impostar",
    "impostáis": "impostar",
    "impostáramos": "impostar",
    "impostáremos": "impostar",
    "impostásemos": "impostar",
    "imposté": "impostar",
    "impostéis": "impostar",
    "impostó": "impostar",
    "impotencias": "impotencia",
    "impotentes": "impotente",
    "impracticabilidades": "impracticabilidad",
    "impracticables": "impracticable",
    "impreca": "imprecar",
    "imprecaba": "imprecar",
    "imprecabais": "imprecar",
    "imprecaban": "imprecar",
    "imprecabas": "imprecar",
    "imprecaciones": "imprecación",
    "imprecad": "imprecar",
    "imprecada": "imprecar",
    "imprecadas": "imprecar",
    "imprecado": "imprecar",
    "imprecados": "imprecar",
    "imprecamos": "imprecar",
    "imprecan": "imprecar",
    "imprecando": "imprecar",
    "imprecara": "imprecar",
    "imprecarais": "imprecar",
    "imprecaran": "imprecar",
    "imprecaras": "imprecar",
    "imprecare": "imprecar",
    "imprecareis": "imprecar",
    "imprecaremos": "imprecar",
    "imprecaren": "imprecar",
    "imprecares": "imprecar",
    "imprecaron": "imprecar",
    "imprecará": "imprecar",
    "imprecarán": "imprecar",
    "imprecarás": "imprecar",
    "imprecaré": "imprecar",
    "imprecaréis": "imprecar",
    "imprecaría": "imprecar",
    "imprecaríais": "imprecar",
    "imprecaríamos": "imprecar",
    "imprecarían": "imprecar",
    "imprecarías": "imprecar",
    "imprecas": "imprecar",
    "imprecase": "imprecar",
    "imprecaseis": "imprecar",
    "imprecasen": "imprecar",
    "imprecases": "imprecar",
    "imprecaste": "imprecar",
    "imprecasteis": "imprecar",
    "imprecatoria": "imprecatorio",
    "imprecatorias": "imprecatorio",
    "imprecatorios": "imprecatorio",
    "imprecisa": "impreciso",
    "imprecisas": "impreciso",
    "imprecisiones": "imprecisión",
    "imprecisos": "impreciso",
    "impreco": "imprecar",
    "imprecábamos": "imprecar",
    "imprecáis": "imprecar",
    "imprecáramos": "imprecar",
    "imprecáremos": "imprecar",
    "imprecásemos": "imprecar",
    "imprecó": "imprecar",
    "impredecibilidades": "impredecibilidad",
    "impredecibles": "impredecible",
    "impregna": "impregnar",
    "impregnaba": "impregnar",
    "impregnabais": "impregnar",
    "impregnaban": "impregnar",
    "impregnabas": "impregnar",
    "impregnabilidades": "impregnabilidad",
    "impregnables": "impregnable",
    "impregnaciones": "impregnación",
    "impregnad": "impregnar",
    "impregnada": "impregnar",
    "impregnadas": "impregnar",
    "impregnado": "impregnar",
    "impregnados": "impregnar",
    "impregnamos": "impregnar",
    "impregnan": "impregnar",
    "impregnando": "impregnar",
    "impregnara": "impregnar",
    "impregnarais": "impregnar",
    "impregnaran": "impregnar",
    "impregnaras": "impregnar",
    "impregnare": "impregnar",
    "impregnareis": "impregnar",
    "impregnaremos": "impregnar",
    "impregnaren": "impregnar",
    "impregnares": "impregnar",
    "impregnaron": "impregnar",
    "impregnará": "impregnar",
    "impregnarán": "impregnar",
    "impregnarás": "impregnar",
    "impregnaré": "impregnar",
    "impregnaréis": "impregnar",
    "impregnaría": "impregnar",
    "impregnaríais": "impregnar",
    "impregnaríamos": "impregnar",
    "impregnarían": "impregnar",
    "impregnarías": "impregnar",
    "impregnas": "impregnar",
    "impregnase": "impregnar",
    "impregnaseis": "impregnar",
    "impregnasen": "impregnar",
    "impregnases": "impregnar",
    "impregnaste": "impregnar",
    "impregnasteis": "impregnar",
    "impregne": "impregnar",
    "impregnemos": "impregnar",
    "impregnen": "impregnar",
    "impregnes": "impregnar",
    "impregno": "impregnar",
    "impregnábamos": "impregnar",
    "impregnáis": "impregnar",
    "impregnáramos": "impregnar",
    "impregnáremos": "impregnar",
    "impregnásemos": "impregnar",
    "impregné": "impregnar",
    "impregnéis": "impregnar",
    "impregnó": "impregnar",
    "impremeditaciones": "impremeditación",
    "impremeditada": "impremeditado",
    "impremeditadas": "impremeditado",
    "impremeditados": "impremeditado",
    "imprentas": "imprenta",
    "impreque": "imprecar",
    "imprequemos": "imprecar",
    "imprequen": "imprecar",
    "impreques": "imprecar",
    "imprequé": "imprecar",
    "imprequéis": "imprecar",
    "impresa": "imprimir",
    "impresas": "imprimir",
    "imprescindibles": "imprescindible",
    "imprescriptibilidades": "imprescriptibilidad",
    "imprescriptibles": "imprescriptible",
    "impresentables": "impresentable",
    "impresiona": "impresionar",
    "impresionaba": "impresionar",
    "impresionabais": "impresionar",
    "impresionaban": "impresionar",
    "impresionabas": "impresionar",
    "impresionabilidades": "impresionabilidad",
    "impresionables": "impresionable",
    "impresionad": "impresionar",
    "impresionada": "impresionar",
    "impresionadas": "impresionar",
    "impresionado": "impresionar",
    "impresionados": "impresionar",
    "impresionamos": "impresionar",
    "impresionan": "impresionar",
    "impresionando": "impresionar",
    "impresionantes": "impresionante",
    "impresionara": "impresionar",
    "impresionarais": "impresionar",
    "impresionaran": "impresionar",
    "impresionaras": "impresionar",
    "impresionare": "impresionar",
    "impresionareis": "impresionar",
    "impresionaremos": "impresionar",
    "impresionaren": "impresionar",
    "impresionares": "impresionar",
    "impresionaron": "impresionar",
    "impresionará": "impresionar",
    "impresionarán": "impresionar",
    "impresionarás": "impresionar",
    "impresionaré": "impresionar",
    "impresionaréis": "impresionar",
    "impresionaría": "impresionar",
    "impresionaríais": "impresionar",
    "impresionaríamos": "impresionar",
    "impresionarían": "impresionar",
    "impresionarías": "impresionar",
    "impresionas": "impresionar",
    "impresionase": "impresionar",
    "impresionaseis": "impresionar",
    "impresionasen": "impresionar",
    "impresionases": "impresionar",
    "impresionaste": "impresionar",
    "impresionasteis": "impresionar",
    "impresione": "impresionar",
    "impresionemos": "impresionar",
    "impresionen": "impresionar",
    "impresiones": "impresionar",
    "impresionismos": "impresionismo",
    "impresionistas": "impresionista",
    "impresiono": "impresionar",
    "impresionábamos": "impresionar",
    "impresionáis": "impresionar",
    "impresionáramos": "impresionar",
    "impresionáremos": "impresionar",
    "impresionásemos": "impresionar",
    "impresioné": "impresionar",
    "impresionéis": "impresionar",
    "impresionó": "impresionar",
    "impreso": "imprimir",
    "impresora": "impresor",
    "impresoras": "impresora",
    "impresores": "impresor",
    "impresos": "imprimir",
    "imprevisibles": "imprevisible",
    "imprevisiones": "imprevisión",
    "imprevisora": "imprevisor",
    "imprevisoras": "imprevisor",
    "imprevisores": "imprevisor",
    "imprevista": "imprevisto",
    "imprevistas": "imprevisto",
    "imprevistos": "imprevisto",
    "imprima": "imprimir",
    "imprimaba": "imprimar",
    "imprimabais": "imprimar",
    "imprimaban": "imprimar",
    "imprimabas": "imprimar",
    "imprimaciones": "imprimación",
    "imprimad": "imprimar",
    "imprimada": "imprimar",
    "imprimadas": "imprimar",
    "imprimaderas": "imprimadera",
    "imprimado": "imprimar",
    "imprimadora": "imprimador",
    "imprimadoras": "imprimador",
    "imprimadores": "imprimador",
    "imprimados": "imprimar",
    "imprimamos": "imprimir",
    "impriman": "imprimir",
    "imprimando": "imprimar",
    "imprimara": "imprimar",
    "imprimarais": "imprimar",
    "imprimaran": "imprimar",
    "imprimaras": "imprimar",
    "imprimare": "imprimar",
    "imprimareis": "imprimar",
    "imprimaremos": "imprimar",
    "imprimaren": "imprimar",
    "imprimares": "imprimar",
    "imprimaron": "imprimar",
    "imprimará": "imprimar",
    "imprimarán": "imprimar",
    "imprimarás": "imprimar",
    "imprimaré": "imprimar",
    "imprimaréis": "imprimar",
    "imprimaría": "imprimar",
    "imprimaríais": "imprimar",
    "imprimaríamos": "imprimar",
    "imprimarían": "imprimar",
    "imprimarías": "imprimar",
    "imprimas": "imprimir",
    "imprimase": "imprimar",
    "imprimaseis": "imprimar",
    "imprimasen": "imprimar",
    "imprimases": "imprimar",
    "imprimaste": "imprimar",
    "imprimasteis": "imprimar",
    "imprime": "imprimir",
    "imprimemos": "imprimar",
    "imprimen": "imprimir",
    "imprimes": "imprimir",
    "imprimibles": "imprimible",
    "imprimid": "imprimir",
    "imprimiendo": "imprimir",
    "imprimiera": "imprimir",
    "imprimierais": "imprimir",
    "imprimieran": "imprimir",
    "imprimieras": "imprimir",
    "imprimiere": "imprimir",
    "imprimiereis": "imprimir",
    "imprimieren": "imprimir",
    "imprimieres": "imprimir",
    "imprimieron": "imprimir",
    "imprimiese": "imprimir",
    "imprimieseis": "imprimir",
    "imprimiesen": "imprimir",
    "imprimieses": "imprimir",
    "imprimimos": "imprimir",
    "imprimiremos": "imprimir",
    "imprimirá": "imprimir",
    "imprimirán": "imprimir",
    "imprimirás": "imprimir",
    "imprimiré": "imprimir",
    "imprimiréis": "imprimir",
    "imprimiría": "imprimir",
    "imprimiríais": "imprimir",
    "imprimiríamos": "imprimir",
    "imprimirían": "imprimir",
    "imprimirías": "imprimir",
    "imprimiste": "imprimir",
    "imprimisteis": "imprimir",
    "imprimiéramos": "imprimir",
    "imprimiéremos": "imprimir",
    "imprimiésemos": "imprimir",
    "imprimió": "imprimir",
    "imprimo": "imprimir",
    "imprimábamos": "imprimar",
    "imprimáis": "imprimir",
    "imprimáramos": "imprimar",
    "imprimáremos": "imprimar",
    "imprimásemos": "imprimar",
    "imprimátures": "imprimátur",
    "imprimé": "imprimar",
    "impriméis": "imprimar",
    "imprimí": "imprimir",
    "imprimía": "imprimir",
    "imprimíais": "imprimir",
    "imprimíamos": "imprimir",
    "imprimían": "imprimir",
    "imprimías": "imprimir",
    "imprimís": "imprimir",
    "imprimó": "imprimar",
    "improbaba": "improbar",
    "improbabais": "improbar",
    "improbaban": "improbar",
    "improbabas": "improbar",
    "improbabilidades": "improbabilidad",
    "improbables": "improbable",
    "improbad": "improbar",
    "improbada": "improbar",
    "improbadas": "improbar",
    "improbado": "improbar",
    "improbados": "improbar",
    "improbamos": "improbar",
    "improbando": "improbar",
    "improbara": "improbar",
    "improbarais": "improbar",
    "improbaran": "improbar",
    "improbaras": "improbar",
    "improbare": "improbar",
    "improbareis": "improbar",
    "improbaremos": "improbar",
    "improbaren": "improbar",
    "improbares": "improbar",
    "improbaron": "improbar",
    "improbará": "improbar",
    "improbarán": "improbar",
    "improbarás": "improbar",
    "improbaré": "improbar",
    "improbaréis": "improbar",
    "improbaría": "improbar",
    "improbaríais": "improbar",
    "improbaríamos": "improbar",
    "improbarían": "improbar",
    "improbarías": "improbar",
    "improbase": "improbar",
    "improbaseis": "improbar",
    "improbasen": "improbar",
    "improbases": "improbar",
    "improbaste": "improbar",
    "improbasteis": "improbar",
    "improbemos": "improbar",
    "improbidades": "improbidad",
    "improbábamos": "improbar",
    "improbáis": "improbar",
    "improbáramos": "improbar",
    "improbáremos": "improbar",
    "improbásemos": "improbar",
    "improbé": "improbar",
    "improbéis": "improbar",
    "improbó": "improbar",
    "improcedencias": "improcedencia",
    "improcedentes": "improcedente",
    "improductiva": "improductivo",
    "improductivas": "improductivo",
    "improductividades": "improductividad",
    "improductivos": "improductivo",
    "impromptus": "impromptu",
    "improntas": "impronta",
    "impronunciables": "impronunciable",
    "improperios": "improperio",
    "impropia": "impropio",
    "impropias": "impropio",
    "impropiedades": "impropiedad",
    "impropios": "impropio",
    "improporcionada": "improporcionado",
    "improporcionadas": "improporcionado",
    "improporcionados": "improporcionado",
    "improporciones": "improporción",
    "improrrogables": "improrrogable",
    "improvisa": "improviso",
    "improvisaba": "improvisar",
    "improvisabais": "improvisar",
    "improvisaban": "improvisar",
    "improvisabas": "improvisar",
    "improvisaciones": "improvisación",
    "improvisad": "improvisar",
    "improvisada": "improvisar",
    "improvisadas": "improvisar",
    "improvisado": "improvisar",
    "improvisadora": "improvisador",
    "improvisadoras": "improvisador",
    "improvisadores": "improvisador",
    "improvisados": "improvisar",
    "improvisamos": "improvisar",
    "improvisan": "improvisar",
    "improvisando": "improvisar",
    "improvisara": "improvisar",
    "improvisarais": "improvisar",
    "improvisaran": "improvisar",
    "improvisaras": "improvisar",
    "improvisare": "improvisar",
    "improvisareis": "improvisar",
    "improvisaremos": "improvisar",
    "improvisaren": "improvisar",
    "improvisares": "improvisar",
    "improvisaron": "improvisar",
    "improvisará": "improvisar",
    "improvisarán": "improvisar",
    "improvisarás": "improvisar",
    "improvisaré": "improvisar",
    "improvisaréis": "improvisar",
    "improvisaría": "improvisar",
    "improvisaríais": "improvisar",
    "improvisaríamos": "improvisar",
    "improvisarían": "improvisar",
    "improvisarías": "improvisar",
    "improvisas": "improviso",
    "improvisase": "improvisar",
    "improvisaseis": "improvisar",
    "improvisasen": "improvisar",
    "improvisases": "improvisar",
    "improvisaste": "improvisar",
    "improvisasteis": "improvisar",
    "improvisatoria": "improvisatorio",
    "improvisatorias": "improvisatorio",
    "improvisatorios": "improvisatorio",
    "improvise": "improvisar",
    "improvisemos": "improvisar",
    "improvisen": "improvisar",
    "improvises": "improvisar",
    "improviso": "improvisar",
    "improvisos": "improviso",
    "improvista": "improvisto",
    "improvistas": "improvisto",
    "improvistos": "improvisto",
    "improvisábamos": "improvisar",
    "improvisáis": "improvisar",
    "improvisáramos": "improvisar",
    "improvisáremos": "improvisar",
    "improvisásemos": "improvisar",
    "improvisé": "improvisar",
    "improviséis": "improvisar",
    "improvisó": "improvisar",
    "imprudencias": "imprudencia",
    "imprudentes": "imprudente",
    "imprueba": "improbar",
    "imprueban": "improbar",
    "impruebas": "improbar",
    "impruebe": "improbar",
    "imprueben": "improbar",
    "impruebes": "improbar",
    "impruebo": "improbar",
    "impsonitas": "impsonita",
    "impublicables": "impublicable",
    "impudencias": "impudencia",
    "impudentes": "impudente",
    "impudicias": "impudicia",
    "impudores": "impudor",
    "impuesta": "imponer",
    "impuestas": "imponer",
    "impuesto": "imponer",
    "impuestos": "impuesto",
    "impugna": "impugnar",
    "impugnaba": "impugnar",
    "impugnabais": "impugnar",
    "impugnaban": "impugnar",
    "impugnabas": "impugnar",
    "impugnabilidades": "impugnabilidad",
    "impugnables": "impugnable",
    "impugnaciones": "impugnación",
    "impugnad": "impugnar",
    "impugnada": "impugnar",
    "impugnadas": "impugnar",
    "impugnado": "impugnar",
    "impugnadora": "impugnador",
    "impugnadoras": "impugnador",
    "impugnadores": "impugnador",
    "impugnados": "impugnar",
    "impugnamos": "impugnar",
    "impugnan": "impugnar",
    "impugnando": "impugnar",
    "impugnara": "impugnar",
    "impugnarais": "impugnar",
    "impugnaran": "impugnar",
    "impugnaras": "impugnar",
    "impugnare": "impugnar",
    "impugnareis": "impugnar",
    "impugnaremos": "impugnar",
    "impugnaren": "impugnar",
    "impugnares": "impugnar",
    "impugnaron": "impugnar",
    "impugnará": "impugnar",
    "impugnarán": "impugnar",
    "impugnarás": "impugnar",
    "impugnaré": "impugnar",
    "impugnaréis": "impugnar",
    "impugnaría": "impugnar",
    "impugnaríais": "impugnar",
    "impugnaríamos": "impugnar",
    "impugnarían": "impugnar",
    "impugnarías": "impugnar",
    "impugnas": "impugnar",
    "impugnase": "impugnar",
    "impugnaseis": "impugnar",
    "impugnasen": "impugnar",
    "impugnases": "impugnar",
    "impugnaste": "impugnar",
    "impugnasteis": "impugnar",
    "impugnativa": "impugnativo",
    "impugnativas": "impugnativo",
    "impugnativos": "impugnativo",
    "impugnatoria": "impugnatorio",
    "impugnatorias": "impugnatorio",
    "impugnatorios": "impugnatorio",
    "impugne": "impugnar",
    "impugnemos": "impugnar",
    "impugnen": "impugnar",
    "impugnes": "impugnar",
    "impugno": "impugnar",
    "impugnábamos": "impugnar",
    "impugnáis": "impugnar",
    "impugnáramos": "impugnar",
    "impugnáremos": "impugnar",
    "impugnásemos": "impugnar",
    "impugné": "impugnar",
    "impugnéis": "impugnar",
    "impugnó": "impugnar",
    "impulsa": "impulsar",
    "impulsaba": "impulsar",
    "impulsabais": "impulsar",
    "impulsaban": "impulsar",
    "impulsabas": "impulsar",
    "impulsad": "impulsar",
    "impulsada": "impulsar",
    "impulsadas": "impulsar",
    "impulsado": "impulsar",
    "impulsados": "impulsar",
    "impulsamos": "impulsar",
    "impulsan": "impulsar",
    "impulsando": "impulsar",
    "impulsara": "impulsar",
    "impulsarais": "impulsar",
    "impulsaran": "impulsar",
    "impulsaras": "impulsar",
    "impulsare": "impulsar",
    "impulsareis": "impulsar",
    "impulsaremos": "impulsar",
    "impulsaren": "impulsar",
    "impulsares": "impulsar",
    "impulsaron": "impulsar",
    "impulsará": "impulsar",
    "impulsarán": "impulsar",
    "impulsarás": "impulsar",
    "impulsaré": "impulsar",
    "impulsaréis": "impulsar",
    "impulsaría": "impulsar",
    "impulsaríais": "impulsar",
    "impulsaríamos": "impulsar",
    "impulsarían": "impulsar",
    "impulsarías": "impulsar",
    "impulsas": "impulsar",
    "impulsase": "impulsar",
    "impulsaseis": "impulsar",
    "impulsasen": "impulsar",
    "impulsases": "impulsar",
    "impulsaste": "impulsar",
    "impulsasteis": "impulsar",
    "impulse": "impulsar",
    "impulsemos": "impulsar",
    "impulsen": "impulsar",
    "impulses": "impulsar",
    "impulsiones": "impulsión",
    "impulsiva": "impulsivo",
    "impulsivas": "impulsivo",
    "impulsividades": "impulsividad",
    "impulsivos": "impulsivo",
    "impulso": "impulsar",
    "impulsora": "impulsor",
    "impulsoras": "impulsor",
    "impulsores": "impulsor",
    "impulsos": "impulso",
    "impulsábamos": "impulsar",
    "impulsáis": "impulsar",
    "impulsáramos": "impulsar",
    "impulsáremos": "impulsar",
    "impulsásemos": "impulsar",
    "impulsé": "impulsar",
    "impulséis": "impulsar",
    "impulsó": "impulsar",
    "impunes": "impune",
    "impunibilidades": "impunibilidad",
    "impunidades": "impunidad",
    "impuntuales": "impuntual",
    "impuntualidades": "impuntualidad",
    "impura": "impuro",
    "impuras": "impuro",
    "impurezas": "impureza",
    "impurifica": "impurificar",
    "impurificaba": "impurificar",
    "impurificabais": "impurificar",
    "impurificaban": "impurificar",
    "impurificabas": "impurificar",
    "impurificaciones": "impurificación",
    "impurificad": "impurificar",
    "impurificada": "impurificar",
    "impurificadas": "impurificar",
    "impurificado": "impurificar",
    "impurificados": "impurificar",
    "impurificamos": "impurificar",
    "impurifican": "impurificar",
    "impurificando": "impurificar",
    "impurificara": "impurificar",
    "impurificarais": "impurificar",
    "impurificaran": "impurificar",
    "impurificaras": "impurificar",
    "impurificare": "impurificar",
    "impurificareis": "impurificar",
    "impurificaremos": "impurificar",
    "impurificaren": "impurificar",
    "impurificares": "impurificar",
    "impurificaron": "impurificar",
    "impurificará": "impurificar",
    "impurificarán": "impurificar",
    "impurificarás": "impurificar",
    "impurificaré": "impurificar",
    "impurificaréis": "impurificar",
    "impurificaría": "impurificar",
    "impurificaríais": "impurificar",
    "impurificaríamos": "impurificar",
    "impurificarían": "impurificar",
    "impurificarías": "impurificar",
    "impurificas": "impurificar",
    "impurificase": "impurificar",
    "impurificaseis": "impurificar",
    "impurificasen": "impurificar",
    "impurificases": "impurificar",
    "impurificaste": "impurificar",
    "impurificasteis": "impurificar",
    "impurifico": "impurificar",
    "impurificábamos": "impurificar",
    "impurificáis": "impurificar",
    "impurificáramos": "impurificar",
    "impurificáremos": "impurificar",
    "impurificásemos": "impurificar",
    "impurificó": "impurificar",
    "impurifique": "impurificar",
    "impurifiquemos": "impurificar",
    "impurifiquen": "impurificar",
    "impurifiques": "impurificar",
    "impurifiqué": "impurificar",
    "impurifiquéis": "impurificar",
    "impuros": "impuro",
    "impuse": "imponer",
    "impusiera": "imponer",
    "impusierais": "imponer",
    "impusieran": "imponer",
    "impusieras": "imponer",
    "impusiere": "imponer",
    "impusiereis": "imponer",
    "impusieren": "imponer",
    "impusieres": "imponer",
    "impusieron": "imponer",
    "impusiese": "imponer",
    "impusieseis": "imponer",
    "impusiesen": "imponer",
    "impusieses": "imponer",
    "impusimos": "imponer",
    "impusiste": "imponer",
    "impusisteis": "imponer",
    "impusiéramos": "imponer",
    "impusiéremos": "imponer",
    "impusiésemos": "imponer",
    "impuso": "imponer",
    "imputa": "imputar",
    "imputaba": "imputar",
    "imputabais": "imputar",
    "imputaban": "imputar",
    "imputabas": "imputar",
    "imputabilidades": "imputabilidad",
    "imputables": "imputable",
    "imputaciones": "imputación",
    "imputaciónes": "imputación",
    "imputad": "imputar",
    "imputada": "imputar",
    "imputadas": "imputar",
    "imputado": "imputar",
    "imputadora": "imputador",
    "imputadoras": "imputador",
    "imputadores": "imputador",
    "imputados": "imputar",
    "imputamos": "imputar",
    "imputan": "imputar",
    "imputando": "imputar",
    "imputara": "imputar",
    "imputarais": "imputar",
    "imputaran": "imputar",
    "imputaras": "imputar",
    "imputare": "imputar",
    "imputareis": "imputar",
    "imputaremos": "imputar",
    "imputaren": "imputar",
    "imputares": "imputar",
    "imputaron": "imputar",
    "imputará": "imputar",
    "imputarán": "imputar",
    "imputarás": "imputar",
    "imputaré": "imputar",
    "imputaréis": "imputar",
    "imputaría": "imputar",
    "imputaríais": "imputar",
    "imputaríamos": "imputar",
    "imputarían": "imputar",
    "imputarías": "imputar",
    "imputas": "imputar",
    "imputase": "imputar",
    "imputaseis": "imputar",
    "imputasen": "imputar",
    "imputases": "imputar",
    "imputaste": "imputar",
    "imputasteis": "imputar",
    "impute": "imputar",
    "imputemos": "imputar",
    "imputen": "imputar",
    "imputes": "imputar",
    "imputo": "imputar",
    "imputrescibles": "imputrescible",
    "imputábamos": "imputar",
    "imputáis": "imputar",
    "imputáramos": "imputar",
    "imputáremos": "imputar",
    "imputásemos": "imputar",
    "imputé": "imputar",
    "imputéis": "imputar",
    "imputó": "imputar",
    "impávida": "impávido",
    "impávidas": "impávido",
    "impávidos": "impávido",
    "impétigos": "impétigo",
    "impía": "impío",
    "impías": "impío",
    "impíos": "impío",
    "impón": "imponer",
    "impúberes": "impúber",
    "impúdica": "impúdico",
    "impúdicas": "impúdico",
    "impúdicos": "impúdico",
    "imágenes": "imagen",
    "inabarcables": "inabarcable",
    "inabordables": "inabordable",
    "inacabables": "inacabable",
    "inacabada": "inacabado",
    "inacabadas": "inacabado",
    "inacabados": "inacabado",
    "inaccesibilidades": "inaccesibilidad",
    "inaccesibles": "inaccesible",
    "inacciones": "inacción",
    "inacentuaciones": "inacentuación",
    "inacentuada": "inacentuado",
    "inacentuadas": "inacentuado",
    "inacentuados": "inacentuado",
    "inaceptables": "inaceptable",
    "inactiva": "inactivo",
    "inactivaba": "inactivar",
    "inactivabais": "inactivar",
    "inactivaban": "inactivar",
    "inactivabas": "inactivar",
    "inactivaciones": "inactivación",
    "inactivad": "inactivar",
    "inactivada": "inactivar",
    "inactivadas": "inactivar",
    "inactivado": "inactivar",
    "inactivados": "inactivar",
    "inactivamos": "inactivar",
    "inactivan": "inactivar",
    "inactivando": "inactivar",
    "inactivara": "inactivar",
    "inactivarais": "inactivar",
    "inactivaran": "inactivar",
    "inactivaras": "inactivar",
    "inactivare": "inactivar",
    "inactivareis": "inactivar",
    "inactivaremos": "inactivar",
    "inactivaren": "inactivar",
    "inactivares": "inactivar",
    "inactivaron": "inactivar",
    "inactivará": "inactivar",
    "inactivarán": "inactivar",
    "inactivarás": "inactivar",
    "inactivaré": "inactivar",
    "inactivaréis": "inactivar",
    "inactivaría": "inactivar",
    "inactivaríais": "inactivar",
    "inactivaríamos": "inactivar",
    "inactivarían": "inactivar",
    "inactivarías": "inactivar",
    "inactivas": "inactivo",
    "inactivase": "inactivar",
    "inactivaseis": "inactivar",
    "inactivasen": "inactivar",
    "inactivases": "inactivar",
    "inactivaste": "inactivar",
    "inactivasteis": "inactivar",
    "inactive": "inactivar",
    "inactivemos": "inactivar",
    "inactiven": "inactivar",
    "inactives": "inactivar",
    "inactividades": "inactividad",
    "inactivo": "inactivar",
    "inactivos": "inactivo",
    "inactivábamos": "inactivar",
    "inactiváis": "inactivar",
    "inactiváramos": "inactivar",
    "inactiváremos": "inactivar",
    "inactivásemos": "inactivar",
    "inactivé": "inactivar",
    "inactivéis": "inactivar",
    "inactivó": "inactivar",
    "inadaptabilidades": "inadaptabilidad",
    "inadaptables": "inadaptable",
    "inadaptaciones": "inadaptación",
    "inadaptada": "inadaptado",
    "inadaptadas": "inadaptado",
    "inadaptados": "inadaptado",
    "inadecuaciones": "inadecuación",
    "inadecuada": "inadecuado",
    "inadecuadas": "inadecuado",
    "inadecuados": "inadecuado",
    "inadmisibilidades": "inadmisibilidad",
    "inadmisibles": "inadmisible",
    "inadmisiones": "inadmisión",
    "inadvertencias": "inadvertencia",
    "inadvertida": "inadvertido",
    "inadvertidas": "inadvertido",
    "inadvertidos": "inadvertido",
    "inagotabilidades": "inagotabilidad",
    "inagotables": "inagotable",
    "inaguantables": "inaguantable",
    "inajustables": "inajustable",
    "inalcanzabilidades": "inalcanzabilidad",
    "inalcanzables": "inalcanzable",
    "inalienabilidades": "inalienabilidad",
    "inalienables": "inalienable",
    "inalterabilidades": "inalterabilidad",
    "inalterables": "inalterable",
    "inalterada": "inalterado",
    "inalteradas": "inalterado",
    "inalterados": "inalterado",
    "inalámbrica": "inalámbrico",
    "inalámbricas": "inalámbrico",
    "inalámbricos": "inalámbrico",
    "inambigua": "inambiguo",
    "inambiguas": "inambiguo",
    "inambiguos": "inambiguo",
    "inamovibles": "inamovible",
    "inamovilidades": "inamovilidad",
    "inanes": "inane",
    "inaniciones": "inanición",
    "inanidades": "inanidad",
    "inanimada": "inanimado",
    "inanimadas": "inanimado",
    "inanimados": "inanimado",
    "inapagables": "inapagable",
    "inapeables": "inapeable",
    "inapelables": "inapelable",
    "inapetencias": "inapetencia",
    "inapetentes": "inapetente",
    "inaplazables": "inaplazable",
    "inaplicabilidades": "inaplicabilidad",
    "inaplicables": "inaplicable",
    "inapreciables": "inapreciable",
    "inaprensibles": "inaprensible",
    "inaprensiva": "inaprensivo",
    "inaprensivas": "inaprensivo",
    "inaprensivos": "inaprensivo",
    "inapropiada": "inapropiado",
    "inapropiadas": "inapropiado",
    "inapropiados": "inapropiado",
    "inaprovechada": "inaprovechado",
    "inaprovechadas": "inaprovechado",
    "inaprovechados": "inaprovechado",
    "inarmónica": "inarmónico",
    "inarmónicas": "inarmónico",
    "inarmónicos": "inarmónico",
    "inarrugables": "inarrugable",
    "inarticulables": "inarticulable",
    "inarticulada": "inarticulado",
    "inarticuladas": "inarticulado",
    "inarticulados": "inarticulado",
    "inasegurables": "inasegurable",
    "inasequibles": "inasequible",
    "inasimilables": "inasimilable",
    "inasistencias": "inasistencia",
    "inasistentes": "inasistente",
    "inastillables": "inastillable",
    "inasumibles": "inasumible",
    "inatacables": "inatacable",
    "inatenciones": "inatención",
    "inaudibles": "inaudible",
    "inaudita": "inaudito",
    "inauditas": "inaudito",
    "inauditos": "inaudito",
    "inaugura": "inaugurar",
    "inauguraba": "inaugurar",
    "inaugurabais": "inaugurar",
    "inauguraban": "inaugurar",
    "inaugurabas": "inaugurar",
    "inauguraciones": "inauguración",
    "inaugurad": "inaugurar",
    "inaugurada": "inaugurar",
    "inauguradas": "inaugurar",
    "inaugurado": "inaugurar",
    "inauguradora": "inaugurador",
    "inauguradoras": "inaugurador",
    "inauguradores": "inaugurador",
    "inaugurados": "inaugurar",
    "inaugurales": "inaugural",
    "inauguramos": "inaugurar",
    "inauguran": "inaugurar",
    "inaugurando": "inaugurar",
    "inaugurara": "inaugurar",
    "inaugurarais": "inaugurar",
    "inauguraran": "inaugurar",
    "inauguraras": "inaugurar",
    "inaugurare": "inaugurar",
    "inaugurareis": "inaugurar",
    "inauguraremos": "inaugurar",
    "inauguraren": "inaugurar",
    "inaugurares": "inaugurar",
    "inauguraron": "inaugurar",
    "inaugurará": "inaugurar",
    "inaugurarán": "inaugurar",
    "inaugurarás": "inaugurar",
    "inauguraré": "inaugurar",
    "inauguraréis": "inaugurar",
    "inauguraría": "inaugurar",
    "inauguraríais": "inaugurar",
    "inauguraríamos": "inaugurar",
    "inaugurarían": "inaugurar",
    "inaugurarías": "inaugurar",
    "inauguras": "inaugurar",
    "inaugurase": "inaugurar",
    "inauguraseis": "inaugurar",
    "inaugurasen": "inaugurar",
    "inaugurases": "inaugurar",
    "inauguraste": "inaugurar",
    "inaugurasteis": "inaugurar",
    "inaugure": "inaugurar",
    "inauguremos": "inaugurar",
    "inauguren": "inaugurar",
    "inaugures": "inaugurar",
    "inauguro": "inaugurar",
    "inaugurábamos": "inaugurar",
    "inauguráis": "inaugurar",
    "inauguráramos": "inaugurar",
    "inauguráremos": "inaugurar",
    "inaugurásemos": "inaugurar",
    "inauguré": "inaugurar",
    "inauguréis": "inaugurar",
    "inauguró": "inaugurar",
    "inaveriguables": "inaveriguable",
    "incaica": "incaico",
    "incaicas": "incaico",
    "incaicos": "incaico",
    "incalculables": "incalculable",
    "incalificables": "incalificable",
    "incandescencias": "incandescencia",
    "incandescendentes": "incandescendente",
    "incandescentes": "incandescente",
    "incansables": "incansable",
    "incapaces": "incapaz",
    "incapacidades": "incapacidad",
    "incapacita": "incapacitar",
    "incapacitaba": "incapacitar",
    "incapacitabais": "incapacitar",
    "incapacitaban": "incapacitar",
    "incapacitabas": "incapacitar",
    "incapacitaciones": "incapacitación",
    "incapacitad": "incapacitar",
    "incapacitada": "incapacitar",
    "incapacitadas": "incapacitar",
    "incapacitado": "incapacitar",
    "incapacitados": "incapacitar",
    "incapacitamos": "incapacitar",
    "incapacitan": "incapacitar",
    "incapacitando": "incapacitar",
    "incapacitara": "incapacitar",
    "incapacitarais": "incapacitar",
    "incapacitaran": "incapacitar",
    "incapacitaras": "incapacitar",
    "incapacitare": "incapacitar",
    "incapacitareis": "incapacitar",
    "incapacitaremos": "incapacitar",
    "incapacitaren": "incapacitar",
    "incapacitares": "incapacitar",
    "incapacitaron": "incapacitar",
    "incapacitará": "incapacitar",
    "incapacitarán": "incapacitar",
    "incapacitarás": "incapacitar",
    "incapacitaré": "incapacitar",
    "incapacitaréis": "incapacitar",
    "incapacitaría": "incapacitar",
    "incapacitaríais": "incapacitar",
    "incapacitaríamos": "incapacitar",
    "incapacitarían": "incapacitar",
    "incapacitarías": "incapacitar",
    "incapacitas": "incapacitar",
    "incapacitase": "incapacitar",
    "incapacitaseis": "incapacitar",
    "incapacitasen": "incapacitar",
    "incapacitases": "incapacitar",
    "incapacitaste": "incapacitar",
    "incapacitasteis": "incapacitar",
    "incapacite": "incapacitar",
    "incapacitemos": "incapacitar",
    "incapaciten": "incapacitar",
    "incapacites": "incapacitar",
    "incapacito": "incapacitar",
    "incapacitábamos": "incapacitar",
    "incapacitáis": "incapacitar",
    "incapacitáramos": "incapacitar",
    "incapacitáremos": "incapacitar",
    "incapacitásemos": "incapacitar",
    "incapacité": "incapacitar",
    "incapacitéis": "incapacitar",
    "incapacitó": "incapacitar",
    "incapazes": "incapaz",
    "incardina": "incardinar",
    "incardinaba": "incardinar",
    "incardinabais": "incardinar",
    "incardinaban": "incardinar",
    "incardinabas": "incardinar",
    "incardinaciones": "incardinación",
    "incardinad": "incardinar",
    "incardinada": "incardinar",
    "incardinadas": "incardinar",
    "incardinado": "incardinar",
    "incardinados": "incardinar",
    "incardinamos": "incardinar",
    "incardinan": "incardinar",
    "incardinando": "incardinar",
    "incardinara": "incardinar",
    "incardinarais": "incardinar",
    "incardinaran": "incardinar",
    "incardinaras": "incardinar",
    "incardinare": "incardinar",
    "incardinareis": "incardinar",
    "incardinaremos": "incardinar",
    "incardinaren": "incardinar",
    "incardinares": "incardinar",
    "incardinaron": "incardinar",
    "incardinará": "incardinar",
    "incardinarán": "incardinar",
    "incardinarás": "incardinar",
    "incardinaré": "incardinar",
    "incardinaréis": "incardinar",
    "incardinaría": "incardinar",
    "incardinaríais": "incardinar",
    "incardinaríamos": "incardinar",
    "incardinarían": "incardinar",
    "incardinarías": "incardinar",
    "incardinas": "incardinar",
    "incardinase": "incardinar",
    "incardinaseis": "incardinar",
    "incardinasen": "incardinar",
    "incardinases": "incardinar",
    "incardinaste": "incardinar",
    "incardinasteis": "incardinar",
    "incardine": "incardinar",
    "incardinemos": "incardinar",
    "incardinen": "incardinar",
    "incardines": "incardinar",
    "incardino": "incardinar",
    "incardinábamos": "incardinar",
    "incardináis": "incardinar",
    "incardináramos": "incardinar",
    "incardináremos": "incardinar",
    "incardinásemos": "incardinar",
    "incardiné": "incardinar",
    "incardinéis": "incardinar",
    "incardinó": "incardinar",
    "incas": "inca",
    "incasables": "incasable",
    "incauta": "incauto",
    "incautaba": "incautar",
    "incautabais": "incautar",
    "incautaban": "incautar",
    "incautabas": "incautar",
    "incautaciones": "incautación",
    "incautad": "incautar",
    "incautada": "incautar",
    "incautadas": "incautar",
    "incautado": "incautar",
    "incautados": "incautar",
    "incautamos": "incautar",
    "incautan": "incautar",
    "incautando": "incautar",
    "incautara": "incautar",
    "incautarais": "incautar",
    "incautaran": "incautar",
    "incautaras": "incautar",
    "incautare": "incautar",
    "incautareis": "incautar",
    "incautaremos": "incautar",
    "incautaren": "incautar",
    "incautares": "incautar",
    "incautaron": "incautar",
    "incautará": "incautar",
    "incautarán": "incautar",
    "incautarás": "incautar",
    "incautaré": "incautar",
    "incautaréis": "incautar",
    "incautaría": "incautar",
    "incautaríais": "incautar",
    "incautaríamos": "incautar",
    "incautarían": "incautar",
    "incautarías": "incautar",
    "incautas": "incauto",
    "incautase": "incautar",
    "incautaseis": "incautar",
    "incautasen": "incautar",
    "incautases": "incautar",
    "incautaste": "incautar",
    "incautasteis": "incautar",
    "incaute": "incautar",
    "incautemos": "incautar",
    "incauten": "incautar",
    "incautes": "incautar",
    "incauto": "incautar",
    "incautos": "incauto",
    "incautábamos": "incautar",
    "incautáis": "incautar",
    "incautáramos": "incautar",
    "incautáremos": "incautar",
    "incautásemos": "incautar",
    "incauté": "incautar",
    "incautéis": "incautar",
    "incautó": "incautar",
    "incendajas": "incendaja",
    "incendia": "incendiar",
    "incendiaba": "incendiar",
    "incendiabais": "incendiar",
    "incendiaban": "incendiar",
    "incendiabas": "incendiar",
    "incendiad": "incendiar",
    "incendiada": "incendiar",
    "incendiadas": "incendiar",
    "incendiado": "incendiar",
    "incendiados": "incendiar",
    "incendiamos": "incendiar",
    "incendian": "incendiar",
    "incendiando": "incendiar",
    "incendiara": "incendiar",
    "incendiarais": "incendiar",
    "incendiaran": "incendiar",
    "incendiaras": "incendiar",
    "incendiare": "incendiar",
    "incendiareis": "incendiar",
    "incendiaremos": "incendiar",
    "incendiaren": "incendiar",
    "incendiares": "incendiar",
    "incendiaria": "incendiario",
    "incendiarias": "incendiario",
    "incendiarios": "incendiario",
    "incendiarismos": "incendiarismo",
    "incendiaron": "incendiar",
    "incendiará": "incendiar",
    "incendiarán": "incendiar",
    "incendiarás": "incendiar",
    "incendiaré": "incendiar",
    "incendiaréis": "incendiar",
    "incendiaría": "incendiar",
    "incendiaríais": "incendiar",
    "incendiaríamos": "incendiar",
    "incendiarían": "incendiar",
    "incendiarías": "incendiar",
    "incendias": "incendiar",
    "incendiase": "incendiar",
    "incendiaseis": "incendiar",
    "incendiasen": "incendiar",
    "incendiases": "incendiar",
    "incendiaste": "incendiar",
    "incendiasteis": "incendiar",
    "incendie": "incendiar",
    "incendiemos": "incendiar",
    "incendien": "incendiar",
    "incendies": "incendiar",
    "incendio": "incendiar",
    "incendios": "incendio",
    "incendiábamos": "incendiar",
    "incendiáis": "incendiar",
    "incendiáramos": "incendiar",
    "incendiáremos": "incendiar",
    "incendiásemos": "incendiar",
    "incendié": "incendiar",
    "incendiéis": "incendiar",
    "incendió": "incendiar",
    "incensaba": "incensar",
    "incensabais": "incensar",
    "incensaban": "incensar",
    "incensabas": "incensar",
    "incensad": "incensar",
    "incensada": "incensar",
    "incensadas": "incensar",
    "incensado": "incensar",
    "incensados": "incensar",
    "incensamos": "incensar",
    "incensando": "incensar",
    "incensara": "incensar",
    "incensarais": "incensar",
    "incensaran": "incensar",
    "incensaras": "incensar",
    "incensare": "incensar",
    "incensareis": "incensar",
    "incensaremos": "incensar",
    "incensaren": "incensar",
    "incensares": "incensar",
    "incensarios": "incensario",
    "incensaron": "incensar",
    "incensará": "incensar",
    "incensarán": "incensar",
    "incensarás": "incensar",
    "incensaré": "incensar",
    "incensaréis": "incensar",
    "incensaría": "incensar",
    "incensaríais": "incensar",
    "incensaríamos": "incensar",
    "incensarían": "incensar",
    "incensarías": "incensar",
    "incensase": "incensar",
    "incensaseis": "incensar",
    "incensasen": "incensar",
    "incensases": "incensar",
    "incensaste": "incensar",
    "incensasteis": "incensar",
    "incensemos": "incensar",
    "incensurables": "incensurable",
    "incensábamos": "incensar",
    "incensáis": "incensar",
    "incensáramos": "incensar",
    "incensáremos": "incensar",
    "incensásemos": "incensar",
    "incensé": "incensar",
    "incenséis": "incensar",
    "incensó": "incensar",
    "incentiva": "incentivar",
    "incentivaba": "incentivar",
    "incentivabais": "incentivar",
    "incentivaban": "incentivar",
    "incentivabas": "incentivar",
    "incentivaciones": "incentivación",
    "incentivad": "incentivar",
    "incentivada": "incentivar",
    "incentivadas": "incentivar",
    "incentivado": "incentivar",
    "incentivados": "incentivar",
    "incentivamos": "incentivar",
    "incentivan": "incentivar",
    "incentivando": "incentivar",
    "incentivara": "incentivar",
    "incentivarais": "incentivar",
    "incentivaran": "incentivar",
    "incentivaras": "incentivar",
    "incentivare": "incentivar",
    "incentivareis": "incentivar",
    "incentivaremos": "incentivar",
    "incentivaren": "incentivar",
    "incentivares": "incentivar",
    "incentivaron": "incentivar",
    "incentivará": "incentivar",
    "incentivarán": "incentivar",
    "incentivarás": "incentivar",
    "incentivaré": "incentivar",
    "incentivaréis": "incentivar",
    "incentivaría": "incentivar",
    "incentivaríais": "incentivar",
    "incentivaríamos": "incentivar",
    "incentivarían": "incentivar",
    "incentivarías": "incentivar",
    "incentivas": "incentivar",
    "incentivase": "incentivar",
    "incentivaseis": "incentivar",
    "incentivasen": "incentivar",
    "incentivases": "incentivar",
    "incentivaste": "incentivar",
    "incentivasteis": "incentivar",
    "incentive": "incentivar",
    "incentivemos": "incentivar",
    "incentiven": "incentivar",
    "incentives": "incentivar",
    "incentivo": "incentivar",
    "incentivos": "incentivo",
    "incentivábamos": "incentivar",
    "incentiváis": "incentivar",
    "incentiváramos": "incentivar",
    "incentiváremos": "incentivar",
    "incentivásemos": "incentivar",
    "incentivé": "incentivar",
    "incentivéis": "incentivar",
    "incentivó": "incentivar",
    "incentros": "incentro",
    "incertezas": "incerteza",
    "incertidumbres": "incertidumbre",
    "incesables": "incesable",
    "incesantes": "incesante",
    "incestos": "incesto",
    "incestuosa": "incestuoso",
    "incestuosas": "incestuoso",
    "incestuosos": "incestuoso",
    "inches": "inch",
    "incicloforias": "incicloforia",
    "inciclotropías": "inciclotropía",
    "incida": "incidir",
    "incidamos": "incidir",
    "incidan": "incidir",
    "incidas": "incidir",
    "incide": "incidir",
    "inciden": "incidir",
    "incidencias": "incidencia",
    "incidentales": "incidental",
    "incidentes": "incidente",
    "incides": "incidir",
    "incidid": "incidir",
    "incidida": "incidir",
    "incididas": "incidir",
    "incidido": "incidir",
    "incididos": "incidir",
    "incidiendo": "incidir",
    "incidiera": "incidir",
    "incidierais": "incidir",
    "incidieran": "incidir",
    "incidieras": "incidir",
    "incidiere": "incidir",
    "incidiereis": "incidir",
    "incidieren": "incidir",
    "incidieres": "incidir",
    "incidieron": "incidir",
    "incidiese": "incidir",
    "incidieseis": "incidir",
    "incidiesen": "incidir",
    "incidieses": "incidir",
    "incidimos": "incidir",
    "incidiremos": "incidir",
    "incidirá": "incidir",
    "incidirán": "incidir",
    "incidirás": "incidir",
    "incidiré": "incidir",
    "incidiréis": "incidir",
    "incidiría": "incidir",
    "incidiríais": "incidir",
    "incidiríamos": "incidir",
    "incidirían": "incidir",
    "incidirías": "incidir",
    "incidiste": "incidir",
    "incidisteis": "incidir",
    "incidiéramos": "incidir",
    "incidiéremos": "incidir",
    "incidiésemos": "incidir",
    "incidió": "incidir",
    "incido": "incidir",
    "incidáis": "incidir",
    "incidí": "incidir",
    "incidía": "incidir",
    "incidíais": "incidir",
    "incidíamos": "incidir",
    "incidían": "incidir",
    "incidías": "incidir",
    "incidís": "incidir",
    "inciensa": "incensar",
    "inciensan": "incensar",
    "inciensas": "incensar",
    "inciense": "incensar",
    "inciensen": "incensar",
    "incienses": "incensar",
    "incienso": "incensar",
    "inciensos": "incienso",
    "incierta": "incierto",
    "inciertas": "incierto",
    "inciertos": "incierto",
    "incinera": "incinerar",
    "incineraba": "incinerar",
    "incinerabais": "incinerar",
    "incineraban": "incinerar",
    "incinerabas": "incinerar",
    "incinerables": "incinerable",
    "incineraciones": "incineración",
    "incinerad": "incinerar",
    "incinerada": "incinerar",
    "incineradas": "incinerar",
    "incinerado": "incinerar",
    "incineradora": "incinerador",
    "incineradoras": "incineradora",
    "incineradores": "incinerador",
    "incinerados": "incinerar",
    "incineramos": "incinerar",
    "incineran": "incinerar",
    "incinerando": "incinerar",
    "incinerara": "incinerar",
    "incinerarais": "incinerar",
    "incineraran": "incinerar",
    "incineraras": "incinerar",
    "incinerare": "incinerar",
    "incinerareis": "incinerar",
    "incineraremos": "incinerar",
    "incineraren": "incinerar",
    "incinerares": "incinerar",
    "incineraron": "incinerar",
    "incinerará": "incinerar",
    "incinerarán": "incinerar",
    "incinerarás": "incinerar",
    "incineraré": "incinerar",
    "incineraréis": "incinerar",
    "incineraría": "incinerar",
    "incineraríais": "incinerar",
    "incineraríamos": "incinerar",
    "incinerarían": "incinerar",
    "incinerarías": "incinerar",
    "incineras": "incinerar",
    "incinerase": "incinerar",
    "incineraseis": "incinerar",
    "incinerasen": "incinerar",
    "incinerases": "incinerar",
    "incineraste": "incinerar",
    "incinerasteis": "incinerar",
    "incinere": "incinerar",
    "incineremos": "incinerar",
    "incineren": "incinerar",
    "incineres": "incinerar",
    "incinero": "incinerar",
    "incinerábamos": "incinerar",
    "incineráis": "incinerar",
    "incineráramos": "incinerar",
    "incineráremos": "incinerar",
    "incinerásemos": "incinerar",
    "incineré": "incinerar",
    "incineréis": "incinerar",
    "incineró": "incinerar",
    "incipientes": "incipiente",
    "incircuncisa": "incircunciso",
    "incircuncisas": "incircunciso",
    "incircuncisos": "incircunciso",
    "incisa": "inciso",
    "incisas": "inciso",
    "incisiones": "incisión",
    "incisiva": "incisivo",
    "incisivas": "incisivo",
    "incisivos": "incisivo",
    "incisoria": "incisorio",
    "incisorias": "incisorio",
    "incisorios": "incisorio",
    "incisos": "inciso",
    "incisuras": "incisura",
    "incita": "incitar",
    "incitaba": "incitar",
    "incitabais": "incitar",
    "incitaban": "incitar",
    "incitabas": "incitar",
    "incitaciones": "incitación",
    "incitad": "incitar",
    "incitada": "incitar",
    "incitadas": "incitar",
    "incitado": "incitar",
    "incitadora": "incitador",
    "incitadoras": "incitador",
    "incitadores": "incitador",
    "incitados": "incitar",
    "incitamentos": "incitamento",
    "incitamos": "incitar",
    "incitan": "incitar",
    "incitando": "incitar",
    "incitantes": "incitante",
    "incitara": "incitar",
    "incitarais": "incitar",
    "incitaran": "incitar",
    "incitaras": "incitar",
    "incitare": "incitar",
    "incitareis": "incitar",
    "incitaremos": "incitar",
    "incitaren": "incitar",
    "incitares": "incitar",
    "incitaron": "incitar",
    "incitará": "incitar",
    "incitarán": "incitar",
    "incitarás": "incitar",
    "incitaré": "incitar",
    "incitaréis": "incitar",
    "incitaría": "incitar",
    "incitaríais": "incitar",
    "incitaríamos": "incitar",
    "incitarían": "incitar",
    "incitarías": "incitar",
    "incitas": "incitar",
    "incitase": "incitar",
    "incitaseis": "incitar",
    "incitasen": "incitar",
    "incitases": "incitar",
    "incitaste": "incitar",
    "incitasteis": "incitar",
    "incitativa": "incitativo",
    "incitativas": "incitativo",
    "incitativos": "incitativo",
    "incite": "incitar",
    "incitemos": "incitar",
    "inciten": "incitar",
    "incites": "incitar",
    "incito": "incitar",
    "incitábamos": "incitar",
    "incitáis": "incitar",
    "incitáramos": "incitar",
    "incitáremos": "incitar",
    "incitásemos": "incitar",
    "incité": "incitar",
    "incitéis": "incitar",
    "incitó": "incitar",
    "inciviles": "incivil",
    "incivilidades": "incivilidad",
    "incivilizada": "incivilizado",
    "incivilizadas": "incivilizado",
    "incivilizados": "incivilizado",
    "inclasificables": "inclasificable",
    "inclaustraciones": "inclaustración",
    "inclemencias": "inclemencia",
    "inclementes": "inclemente",
    "inclina": "inclinar",
    "inclinaba": "inclinar",
    "inclinabais": "inclinar",
    "inclinaban": "inclinar",
    "inclinabas": "inclinar",
    "inclinables": "inclinable",
    "inclinaciones": "inclinación",
    "inclinad": "inclinar",
    "inclinada": "inclinar",
    "inclinadas": "inclinar",
    "inclinado": "inclinar",
    "inclinadora": "inclinador",
    "inclinadoras": "inclinador",
    "inclinadores": "inclinador",
    "inclinados": "inclinar",
    "inclinamos": "inclinar",
    "inclinan": "inclinar",
    "inclinando": "inclinar",
    "inclinara": "inclinar",
    "inclinarais": "inclinar",
    "inclinaran": "inclinar",
    "inclinaras": "inclinar",
    "inclinare": "inclinar",
    "inclinareis": "inclinar",
    "inclinaremos": "inclinar",
    "inclinaren": "inclinar",
    "inclinares": "inclinar",
    "inclinaron": "inclinar",
    "inclinará": "inclinar",
    "inclinarán": "inclinar",
    "inclinarás": "inclinar",
    "inclinaré": "inclinar",
    "inclinaréis": "inclinar",
    "inclinaría": "inclinar",
    "inclinaríais": "inclinar",
    "inclinaríamos": "inclinar",
    "inclinarían": "inclinar",
    "inclinarías": "inclinar",
    "inclinas": "inclinar",
    "inclinase": "inclinar",
    "inclinaseis": "inclinar",
    "inclinasen": "inclinar",
    "inclinases": "inclinar",
    "inclinaste": "inclinar",
    "inclinasteis": "inclinar",
    "inclinativa": "inclinativo",
    "inclinativas": "inclinativo",
    "inclinativos": "inclinativo",
    "incline": "inclinar",
    "inclinemos": "inclinar",
    "inclinen": "inclinar",
    "inclines": "inclinar",
    "inclino": "inclinar",
    "inclinábamos": "inclinar",
    "inclináis": "inclinar",
    "inclináramos": "inclinar",
    "inclináremos": "inclinar",
    "inclinásemos": "inclinar",
    "incliné": "inclinar",
    "inclinéis": "inclinar",
    "inclinó": "inclinar",
    "inclinómetros": "inclinómetro",
    "incluid": "incluir",
    "incluida": "incluir",
    "incluidas": "incluir",
    "incluido": "incluir",
    "incluidos": "incluir",
    "incluimos": "incluir",
    "incluiremos": "incluir",
    "incluirá": "incluir",
    "incluirán": "incluir",
    "incluirás": "incluir",
    "incluiré": "incluir",
    "incluiréis": "incluir",
    "incluiría": "incluir",
    "incluiríais": "incluir",
    "incluiríamos": "incluir",
    "incluirían": "incluir",
    "incluirías": "incluir",
    "incluiste": "incluir",
    "incluisteis": "incluir",
    "inclusa": "incluso",
    "inclusas": "incluso",
    "inclusera": "inclusero",
    "incluseras": "inclusero",
    "incluseros": "inclusero",
    "inclusiones": "inclusión",
    "inclusiva": "inclusivo",
    "inclusivas": "inclusivo",
    "inclusivos": "inclusivo",
    "inclusos": "incluso",
    "incluya": "incluir",
    "incluyamos": "incluir",
    "incluyan": "incluir",
    "incluyas": "incluir",
    "incluye": "incluir",
    "incluyen": "incluir",
    "incluyendo": "incluir",
    "incluyera": "incluir",
    "incluyerais": "incluir",
    "incluyeran": "incluir",
    "incluyeras": "incluir",
    "incluyere": "incluir",
    "incluyereis": "incluir",
    "incluyeren": "incluir",
    "incluyeres": "incluir",
    "incluyeron": "incluir",
    "incluyes": "incluir",
    "incluyese": "incluir",
    "incluyeseis": "incluir",
    "incluyesen": "incluir",
    "incluyeses": "incluir",
    "incluyo": "incluir",
    "incluyáis": "incluir",
    "incluyéramos": "incluir",
    "incluyéremos": "incluir",
    "incluyésemos": "incluir",
    "incluyó": "incluir",
    "incluí": "incluir",
    "incluía": "incluir",
    "incluíais": "incluir",
    "incluíamos": "incluir",
    "incluían": "incluir",
    "incluías": "incluir",
    "incluís": "incluir",
    "incoa": "incoar",
    "incoaba": "incoar",
    "incoabais": "incoar",
    "incoaban": "incoar",
    "incoabas": "incoar",
    "incoaciones": "incoación",
    "incoad": "incoar",
    "incoada": "incoar",
    "incoadas": "incoar",
    "incoado": "incoar",
    "incoados": "incoar",
    "incoamos": "incoar",
    "incoan": "incoar",
    "incoando": "incoar",
    "incoara": "incoar",
    "incoarais": "incoar",
    "incoaran": "incoar",
    "incoaras": "incoar",
    "incoare": "incoar",
    "incoareis": "incoar",
    "incoaremos": "incoar",
    "incoaren": "incoar",
    "incoares": "incoar",
    "incoaron": "incoar",
    "incoará": "incoar",
    "incoarán": "incoar",
    "incoarás": "incoar",
    "incoaré": "incoar",
    "incoaréis": "incoar",
    "incoaría": "incoar",
    "incoaríais": "incoar",
    "incoaríamos": "incoar",
    "incoarían": "incoar",
    "incoarías": "incoar",
    "incoas": "incoar",
    "incoase": "incoar",
    "incoaseis": "incoar",
    "incoasen": "incoar",
    "incoases": "incoar",
    "incoaste": "incoar",
    "incoasteis": "incoar",
    "incoativa": "incoativo",
    "incoativas": "incoativo",
    "incoativos": "incoativo",
    "incobrables": "incobrable",
    "incodicionada": "incondicionado",
    "incodicionadas": "incondicionado",
    "incodicionado": "incondicionado",
    "incodicionados": "incondicionado",
    "incoe": "incoar",
    "incoemos": "incoar",
    "incoen": "incoar",
    "incoercibles": "incoercible",
    "incoes": "incoar",
    "incognoscibles": "incognoscible",
    "incoherencias": "incoherencia",
    "incoherentes": "incoherente",
    "incolocables": "incolocable",
    "incolora": "incoloro",
    "incoloras": "incoloro",
    "incoloros": "incoloro",
    "incolumidades": "incolumidad",
    "incombinables": "incombinable",
    "incombustibilidades": "incombustibilidad",
    "incombustibles": "incombustible",
    "incomestibles": "incomestible",
    "incomibles": "incomible",
    "incominique": "incomunicar",
    "incominiquemos": "incomunicar",
    "incominiquen": "incomunicar",
    "incominiques": "incomunicar",
    "incominiqué": "incomunicar",
    "incominiquéis": "incomunicar",
    "incomoda": "incomodar",
    "incomodaba": "incomodar",
    "incomodabais": "incomodar",
    "incomodaban": "incomodar",
    "incomodabas": "incomodar",
    "incomodad": "incomodar",
    "incomodada": "incomodar",
    "incomodadas": "incomodar",
    "incomodado": "incomodar",
    "incomodadora": "incomodador",
    "incomodadoras": "incomodador",
    "incomodadores": "incomodador",
    "incomodados": "incomodar",
    "incomodamos": "incomodar",
    "incomodan": "incomodar",
    "incomodando": "incomodar",
    "incomodara": "incomodar",
    "incomodarais": "incomodar",
    "incomodaran": "incomodar",
    "incomodaras": "incomodar",
    "incomodare": "incomodar",
    "incomodareis": "incomodar",
    "incomodaremos": "incomodar",
    "incomodaren": "incomodar",
    "incomodares": "incomodar",
    "incomodaron": "incomodar",
    "incomodará": "incomodar",
    "incomodarán": "incomodar",
    "incomodarás": "incomodar",
    "incomodaré": "incomodar",
    "incomodaréis": "incomodar",
    "incomodaría": "incomodar",
    "incomodaríais": "incomodar",
    "incomodaríamos": "incomodar",
    "incomodarían": "incomodar",
    "incomodarías": "incomodar",
    "incomodas": "incomodar",
    "incomodase": "incomodar",
    "incomodaseis": "incomodar",
    "incomodasen": "incomodar",
    "incomodases": "incomodar",
    "incomodaste": "incomodar",
    "incomodasteis": "incomodar",
    "incomode": "incomodar",
    "incomodemos": "incomodar",
    "incomoden": "incomodar",
    "incomodes": "incomodar",
    "incomodidades": "incomodidad",
    "incomodo": "incomodar",
    "incomodos": "incomodo",
    "incomodábamos": "incomodar",
    "incomodáis": "incomodar",
    "incomodáramos": "incomodar",
    "incomodáremos": "incomodar",
    "incomodásemos": "incomodar",
    "incomodé": "incomodar",
    "incomodéis": "incomodar",
    "incomodó": "incomodar",
    "incomparables": "incomparable",
    "incomparecencias": "incomparecencia",
    "incompartibles": "incompartible",
    "incompasibles": "incompasible",
    "incompasiva": "incompasivo",
    "incompasivas": "incompasivo",
    "incompasivos": "incompasivo",
    "incompatibilidades": "incompatibilidad",
    "incompatibles": "incompatible",
    "incompetencias": "incompetencia",
    "incompetentes": "incompetente",
    "incompleta": "incompleto",
    "incompletas": "incompleto",
    "incompletos": "incompleto",
    "incomprendida": "incomprendido",
    "incomprendidas": "incomprendido",
    "incomprendidos": "incomprendido",
    "incomprensibilidades": "incomprensibilidad",
    "incomprensibles": "incomprensible",
    "incomprensiones": "incomprensión",
    "incompresibilidades": "incompresibilidad",
    "incompresibles": "incompresible",
    "incomprimibles": "incomprimible",
    "incomprobables": "incomprobable",
    "incomunica": "incomunicar",
    "incomunicaba": "incomunicar",
    "incomunicabais": "incomunicar",
    "incomunicaban": "incomunicar",
    "incomunicabas": "incomunicar",
    "incomunicabilidades": "incomunicabilidad",
    "incomunicables": "incomunicable",
    "incomunicaciones": "incomunicación",
    "incomunicad": "incomunicar",
    "incomunicada": "incomunicar",
    "incomunicadas": "incomunicar",
    "incomunicado": "incomunicar",
    "incomunicados": "incomunicar",
    "incomunicamos": "incomunicar",
    "incomunican": "incomunicar",
    "incomunicando": "incomunicar",
    "incomunicara": "incomunicar",
    "incomunicarais": "incomunicar",
    "incomunicaran": "incomunicar",
    "incomunicaras": "incomunicar",
    "incomunicare": "incomunicar",
    "incomunicareis": "incomunicar",
    "incomunicaremos": "incomunicar",
    "incomunicaren": "incomunicar",
    "incomunicares": "incomunicar",
    "incomunicaron": "incomunicar",
    "incomunicará": "incomunicar",
    "incomunicarán": "incomunicar",
    "incomunicarás": "incomunicar",
    "incomunicaré": "incomunicar",
    "incomunicaréis": "incomunicar",
    "incomunicaría": "incomunicar",
    "incomunicaríais": "incomunicar",
    "incomunicaríamos": "incomunicar",
    "incomunicarían": "incomunicar",
    "incomunicarías": "incomunicar",
    "incomunicas": "incomunicar",
    "incomunicase": "incomunicar",
    "incomunicaseis": "incomunicar",
    "incomunicasen": "incomunicar",
    "incomunicases": "incomunicar",
    "incomunicaste": "incomunicar",
    "incomunicasteis": "incomunicar",
    "incomunico": "incomunicar",
    "incomunicábamos": "incomunicar",
    "incomunicáis": "incomunicar",
    "incomunicáramos": "incomunicar",
    "incomunicáremos": "incomunicar",
    "incomunicásemos": "incomunicar",
    "incomunicó": "incomunicar",
    "inconcebibles": "inconcebible",
    "inconciliables": "inconciliable",
    "inconclusa": "inconcluso",
    "inconclusas": "inconcluso",
    "inconclusos": "inconcluso",
    "inconcomitantes": "inconcomitante",
    "inconcreciones": "inconcreción",
    "inconcreta": "inconcreto",
    "inconcretas": "inconcreto",
    "inconcretos": "inconcreto",
    "inconcusa": "inconcuso",
    "inconcusas": "inconcuso",
    "inconcusos": "inconcuso",
    "incondensables": "incondensable",
    "incondicionales": "incondicional",
    "inconducentes": "inconducente",
    "inconecta": "inconecto",
    "inconectas": "inconecto",
    "inconectos": "inconecto",
    "inconexa": "inconexo",
    "inconexas": "inconexo",
    "inconexiones": "inconexión",
    "inconexos": "inconexo",
    "inconfesa": "inconfeso",
    "inconfesables": "inconfesable",
    "inconfesas": "inconfeso",
    "inconfesos": "inconfeso",
    "inconformes": "inconforme",
    "inconformidades": "inconformidad",
    "inconformismos": "inconformismo",
    "inconformistas": "inconformista",
    "inconfundibles": "inconfundible",
    "incongelables": "incongelable",
    "incongrua": "incongruo",
    "incongruas": "incongruo",
    "incongruencias": "incongruencia",
    "incongruentes": "incongruente",
    "incongruos": "incongruo",
    "inconjugables": "inconjugable",
    "inconmensurabilidades": "inconmensurabilidad",
    "inconmensurables": "inconmensurable",
    "inconmovibles": "inconmovible",
    "inconmutabilidades": "inconmutabilidad",
    "inconmutables": "inconmutable",
    "inconquistables": "inconquistable",
    "inconsciencias": "inconsciencia",
    "inconscientes": "inconsciente",
    "inconsecuencias": "inconsecuencia",
    "inconsecuentes": "inconsecuente",
    "inconsideraciones": "inconsideración",
    "inconsiderada": "inconsiderado",
    "inconsideradas": "inconsiderado",
    "inconsiderados": "inconsiderado",
    "inconsistencias": "inconsistencia",
    "inconsistentes": "inconsistente",
    "inconsolables": "inconsolable",
    "inconspicua": "inconspicuo",
    "inconspicuas": "inconspicuo",
    "inconspicuos": "inconspicuo",
    "inconstancias": "inconstancia",
    "inconstantes": "inconstante",
    "inconstitucionales": "inconstitucional",
    "inconstitucionalidades": "inconstitucionalidad",
    "inconstruibles": "inconstruible",
    "inconsútiles": "inconsútil",
    "incontables": "incontable",
    "incontaminada": "incontaminado",
    "incontaminadas": "incontaminado",
    "incontaminados": "incontaminado",
    "incontenibles": "incontenible",
    "incontenida": "incontenido",
    "incontenidas": "incontenido",
    "incontenidos": "incontenido",
    "incontestabilidades": "incontestabilidad",
    "incontestables": "incontestable",
    "incontinencias": "incontinencia",
    "incontinentes": "incontinente",
    "incontinua": "incontinuo",
    "incontinuas": "incontinuo",
    "incontinuos": "incontinuo",
    "incontrastables": "incontrastable",
    "incontrolables": "incontrolable",
    "incontrolada": "incontrolado",
    "incontroladas": "incontrolado",
    "incontrolados": "incontrolado",
    "incontrovertibles": "incontrovertible",
    "inconvenibles": "inconvenible",
    "inconveniencias": "inconveniencia",
    "inconvenientes": "inconveniente",
    "inconvertibilidades": "inconvertibilidad",
    "inconvertibles": "inconvertible",
    "incoo": "incoar",
    "incoordinaciones": "incoordinación",
    "incordia": "incordiar",
    "incordiaba": "incordiar",
    "incordiabais": "incordiar",
    "incordiaban": "incordiar",
    "incordiabas": "incordiar",
    "incordiad": "incordiar",
    "incordiada": "incordiar",
    "incordiadas": "incordiar",
    "incordiado": "incordiar",
    "incordiados": "incordiar",
    "incordiamos": "incordiar",
    "incordian": "incordiar",
    "incordiando": "incordiar",
    "incordiara": "incordiar",
    "incordiarais": "incordiar",
    "incordiaran": "incordiar",
    "incordiaras": "incordiar",
    "incordiare": "incordiar",
    "incordiareis": "incordiar",
    "incordiaremos": "incordiar",
    "incordiaren": "incordiar",
    "incordiares": "incordiar",
    "incordiaron": "incordiar",
    "incordiará": "incordiar",
    "incordiarán": "incordiar",
    "incordiarás": "incordiar",
    "incordiaré": "incordiar",
    "incordiaréis": "incordiar",
    "incordiaría": "incordiar",
    "incordiaríais": "incordiar",
    "incordiaríamos": "incordiar",
    "incordiarían": "incordiar",
    "incordiarías": "incordiar",
    "incordias": "incordiar",
    "incordiase": "incordiar",
    "incordiaseis": "incordiar",
    "incordiasen": "incordiar",
    "incordiases": "incordiar",
    "incordiaste": "incordiar",
    "incordiasteis": "incordiar",
    "incordie": "incordiar",
    "incordiemos": "incordiar",
    "incordien": "incordiar",
    "incordies": "incordiar",
    "incordio": "incordiar",
    "incordios": "incordio",
    "incordiábamos": "incordiar",
    "incordiáis": "incordiar",
    "incordiáramos": "incordiar",
    "incordiáremos": "incordiar",
    "incordiásemos": "incordiar",
    "incordié": "incordiar",
    "incordiéis": "incordiar",
    "incordió": "incordiar",
    "incorpora": "incorporar",
    "incorporaba": "incorporar",
    "incorporabais": "incorporar",
    "incorporaban": "incorporar",
    "incorporabas": "incorporar",
    "incorporables": "incorporable",
    "incorporaciones": "incorporación",
    "incorporad": "incorporar",
    "incorporada": "incorporar",
    "incorporadas": "incorporar",
    "incorporado": "incorporar",
    "incorporados": "incorporar",
    "incorporales": "incorporal",
    "incorporamos": "incorporar",
    "incorporan": "incorporar",
    "incorporando": "incorporar",
    "incorporara": "incorporar",
    "incorporarais": "incorporar",
    "incorporaran": "incorporar",
    "incorporaras": "incorporar",
    "incorporare": "incorporar",
    "incorporareis": "incorporar",
    "incorporaremos": "incorporar",
    "incorporaren": "incorporar",
    "incorporares": "incorporar",
    "incorporaron": "incorporar",
    "incorporará": "incorporar",
    "incorporarán": "incorporar",
    "incorporarás": "incorporar",
    "incorporaré": "incorporar",
    "incorporaréis": "incorporar",
    "incorporaría": "incorporar",
    "incorporaríais": "incorporar",
    "incorporaríamos": "incorporar",
    "incorporarían": "incorporar",
    "incorporarías": "incorporar",
    "incorporas": "incorporar",
    "incorporase": "incorporar",
    "incorporaseis": "incorporar",
    "incorporasen": "incorporar",
    "incorporases": "incorporar",
    "incorporaste": "incorporar",
    "incorporasteis": "incorporar",
    "incorpore": "incorporar",
    "incorporeidades": "incorporeidad",
    "incorporemos": "incorporar",
    "incorporen": "incorporar",
    "incorpores": "incorporar",
    "incorporo": "incorporar",
    "incorporábamos": "incorporar",
    "incorporáis": "incorporar",
    "incorporáramos": "incorporar",
    "incorporáremos": "incorporar",
    "incorporásemos": "incorporar",
    "incorporé": "incorporar",
    "incorporéis": "incorporar",
    "incorporó": "incorporar",
    "incorpórea": "incorpóreo",
    "incorpóreas": "incorpóreo",
    "incorpóreos": "incorpóreo",
    "incorrecciones": "incorrección",
    "incorrecta": "incorrecto",
    "incorrectas": "incorrecto",
    "incorrectos": "incorrecto",
    "incorregibilidades": "incorregibilidad",
    "incorregibles": "incorregible",
    "incorrupciones": "incorrupción",
    "incorrupta": "incorrupto",
    "incorruptas": "incorrupto",
    "incorruptibilidades": "incorruptibilidad",
    "incorruptibles": "incorruptible",
    "incorruptos": "incorrupto",
    "incoábamos": "incoar",
    "incoáis": "incoar",
    "incoáramos": "incoar",
    "incoáremos": "incoar",
    "incoásemos": "incoar",
    "incoé": "incoar",
    "incoéis": "incoar",
    "incoó": "incoar",
    "incrasa": "incrasar",
    "incrasaba": "incrasar",
    "incrasabais": "incrasar",
    "incrasaban": "incrasar",
    "incrasabas": "incrasar",
    "incrasad": "incrasar",
    "incrasada": "incrasar",
    "incrasadas": "incrasar",
    "incrasado": "incrasar",
    "incrasados": "incrasar",
    "incrasamos": "incrasar",
    "incrasan": "incrasar",
    "incrasando": "incrasar",
    "incrasara": "incrasar",
    "incrasarais": "incrasar",
    "incrasaran": "incrasar",
    "incrasaras": "incrasar",
    "incrasare": "incrasar",
    "incrasareis": "incrasar",
    "incrasaremos": "incrasar",
    "incrasaren": "incrasar",
    "incrasares": "incrasar",
    "incrasaron": "incrasar",
    "incrasará": "incrasar",
    "incrasarán": "incrasar",
    "incrasarás": "incrasar",
    "incrasaré": "incrasar",
    "incrasaréis": "incrasar",
    "incrasaría": "incrasar",
    "incrasaríais": "incrasar",
    "incrasaríamos": "incrasar",
    "incrasarían": "incrasar",
    "incrasarías": "incrasar",
    "incrasas": "incrasar",
    "incrasase": "incrasar",
    "incrasaseis": "incrasar",
    "incrasasen": "incrasar",
    "incrasases": "incrasar",
    "incrasaste": "incrasar",
    "incrasasteis": "incrasar",
    "incrase": "incrasar",
    "incrasemos": "incrasar",
    "incrasen": "incrasar",
    "incrases": "incrasar",
    "incraso": "incrasar",
    "incrasábamos": "incrasar",
    "incrasáis": "incrasar",
    "incrasáramos": "incrasar",
    "incrasáremos": "incrasar",
    "incrasásemos": "incrasar",
    "incrasé": "incrasar",
    "incraséis": "incrasar",
    "incrasó": "incrasar",
    "increada": "increado",
    "increadas": "increado",
    "increados": "increado",
    "incredibilidades": "incredibilidad",
    "incredulidades": "incredulidad",
    "incrementa": "incrementar",
    "incrementaba": "incrementar",
    "incrementabais": "incrementar",
    "incrementaban": "incrementar",
    "incrementabas": "incrementar",
    "incrementad": "incrementar",
    "incrementada": "incrementar",
    "incrementadas": "incrementar",
    "incrementado": "incrementar",
    "incrementados": "incrementar",
    "incrementales": "incremental",
    "incrementamos": "incrementar",
    "incrementan": "incrementar",
    "incrementando": "incrementar",
    "incrementara": "incrementar",
    "incrementarais": "incrementar",
    "incrementaran": "incrementar",
    "incrementaras": "incrementar",
    "incrementare": "incrementar",
    "incrementareis": "incrementar",
    "incrementaremos": "incrementar",
    "incrementaren": "incrementar",
    "incrementares": "incrementar",
    "incrementaron": "incrementar",
    "incrementará": "incrementar",
    "incrementarán": "incrementar",
    "incrementarás": "incrementar",
    "incrementaré": "incrementar",
    "incrementaréis": "incrementar",
    "incrementaría": "incrementar",
    "incrementaríais": "incrementar",
    "incrementaríamos": "incrementar",
    "incrementarían": "incrementar",
    "incrementarías": "incrementar",
    "incrementas": "incrementar",
    "incrementase": "incrementar",
    "incrementaseis": "incrementar",
    "incrementasen": "incrementar",
    "incrementases": "incrementar",
    "incrementaste": "incrementar",
    "incrementasteis": "incrementar",
    "incremente": "incrementar",
    "incrementemos": "incrementar",
    "incrementen": "incrementar",
    "incrementes": "incrementar",
    "incremento": "incrementar",
    "incrementos": "incremento",
    "incrementábamos": "incrementar",
    "incrementáis": "incrementar",
    "incrementáramos": "incrementar",
    "incrementáremos": "incrementar",
    "incrementásemos": "incrementar",
    "incrementé": "incrementar",
    "incrementéis": "incrementar",
    "incrementó": "incrementar",
    "increpa": "increpar",
    "increpaba": "increpar",
    "increpabais": "increpar",
    "increpaban": "increpar",
    "increpabas": "increpar",
    "increpaciones": "increpación",
    "increpad": "increpar",
    "increpada": "increpar",
    "increpadas": "increpar",
    "increpado": "increpar",
    "increpadora": "increpador",
    "increpadoras": "increpador",
    "increpadores": "increpador",
    "increpados": "increpar",
    "increpamos": "increpar",
    "increpan": "increpar",
    "increpando": "increpar",
    "increpara": "increpar",
    "increparais": "increpar",
    "increparan": "increpar",
    "increparas": "increpar",
    "increpare": "increpar",
    "increpareis": "increpar",
    "increparemos": "increpar",
    "increparen": "increpar",
    "increpares": "increpar",
    "increparon": "increpar",
    "increpará": "increpar",
    "increparán": "increpar",
    "increparás": "increpar",
    "increparé": "increpar",
    "increparéis": "increpar",
    "increparía": "increpar",
    "increparíais": "increpar",
    "increparíamos": "increpar",
    "increparían": "increpar",
    "increparías": "increpar",
    "increpas": "increpar",
    "increpase": "increpar",
    "increpaseis": "increpar",
    "increpasen": "increpar",
    "increpases": "increpar",
    "increpaste": "increpar",
    "increpasteis": "increpar",
    "increpe": "increpar",
    "increpemos": "increpar",
    "increpen": "increpar",
    "increpes": "increpar",
    "increpo": "increpar",
    "increpábamos": "increpar",
    "increpáis": "increpar",
    "increpáramos": "increpar",
    "increpáremos": "increpar",
    "increpásemos": "increpar",
    "increpé": "increpar",
    "increpéis": "increpar",
    "increpó": "increpar",
    "increíbles": "increíble",
    "incrimina": "incriminar",
    "incriminaba": "incriminar",
    "incriminabais": "incriminar",
    "incriminaban": "incriminar",
    "incriminabas": "incriminar",
    "incriminaciones": "incriminación",
    "incriminad": "incriminar",
    "incriminada": "incriminar",
    "incriminadas": "incriminar",
    "incriminado": "incriminar",
    "incriminadora": "incriminador",
    "incriminadoras": "incriminador",
    "incriminadores": "incriminador",
    "incriminados": "incriminar",
    "incriminamos": "incriminar",
    "incriminan": "incriminar",
    "incriminando": "incriminar",
    "incriminara": "incriminar",
    "incriminarais": "incriminar",
    "incriminaran": "incriminar",
    "incriminaras": "incriminar",
    "incriminare": "incriminar",
    "incriminareis": "incriminar",
    "incriminaremos": "incriminar",
    "incriminaren": "incriminar",
    "incriminares": "incriminar",
    "incriminaron": "incriminar",
    "incriminará": "incriminar",
    "incriminarán": "incriminar",
    "incriminarás": "incriminar",
    "incriminaré": "incriminar",
    "incriminaréis": "incriminar",
    "incriminaría": "incriminar",
    "incriminaríais": "incriminar",
    "incriminaríamos": "incriminar",
    "incriminarían": "incriminar",
    "incriminarías": "incriminar",
    "incriminas": "incriminar",
    "incriminase": "incriminar",
    "incriminaseis": "incriminar",
    "incriminasen": "incriminar",
    "incriminases": "incriminar",
    "incriminaste": "incriminar",
    "incriminasteis": "incriminar",
    "incriminatoria": "incriminatorio",
    "incriminatorias": "incriminatorio",
    "incriminatorios": "incriminatorio",
    "incrimine": "incriminar",
    "incriminemos": "incriminar",
    "incriminen": "incriminar",
    "incrimines": "incriminar",
    "incrimino": "incriminar",
    "incriminábamos": "incriminar",
    "incrimináis": "incriminar",
    "incrimináramos": "incriminar",
    "incrimináremos": "incriminar",
    "incriminásemos": "incriminar",
    "incriminé": "incriminar",
    "incriminéis": "incriminar",
    "incriminó": "incriminar",
    "incristalizables": "incristalizable",
    "incruenta": "incruento",
    "incruentas": "incruento",
    "incruentos": "incruento",
    "incrusta": "incrustar",
    "incrustaba": "incrustar",
    "incrustabais": "incrustar",
    "incrustaban": "incrustar",
    "incrustabas": "incrustar",
    "incrustaciones": "incrustación",
    "incrustad": "incrustar",
    "incrustada": "incrustar",
    "incrustadas": "incrustar",
    "incrustado": "incrustar",
    "incrustados": "incrustar",
    "incrustamos": "incrustar",
    "incrustan": "incrustar",
    "incrustando": "incrustar",
    "incrustantes": "incrustante",
    "incrustara": "incrustar",
    "incrustarais": "incrustar",
    "incrustaran": "incrustar",
    "incrustaras": "incrustar",
    "incrustare": "incrustar",
    "incrustareis": "incrustar",
    "incrustaremos": "incrustar",
    "incrustaren": "incrustar",
    "incrustares": "incrustar",
    "incrustaron": "incrustar",
    "incrustará": "incrustar",
    "incrustarán": "incrustar",
    "incrustarás": "incrustar",
    "incrustaré": "incrustar",
    "incrustaréis": "incrustar",
    "incrustaría": "incrustar",
    "incrustaríais": "incrustar",
    "incrustaríamos": "incrustar",
    "incrustarían": "incrustar",
    "incrustarías": "incrustar",
    "incrustas": "incrustar",
    "incrustase": "incrustar",
    "incrustaseis": "incrustar",
    "incrustasen": "incrustar",
    "incrustases": "incrustar",
    "incrustaste": "incrustar",
    "incrustasteis": "incrustar",
    "incruste": "incrustar",
    "incrustemos": "incrustar",
    "incrusten": "incrustar",
    "incrustes": "incrustar",
    "incrusto": "incrustar",
    "incrustábamos": "incrustar",
    "incrustáis": "incrustar",
    "incrustáramos": "incrustar",
    "incrustáremos": "incrustar",
    "incrustásemos": "incrustar",
    "incrusté": "incrustar",
    "incrustéis": "incrustar",
    "incrustó": "incrustar",
    "incrédula": "incrédulo",
    "incrédulas": "incrédulo",
    "incrédulos": "incrédulo",
    "incuba": "incubar",
    "incubaba": "incubar",
    "incubabais": "incubar",
    "incubaban": "incubar",
    "incubabas": "incubar",
    "incubaciones": "incubación",
    "incubad": "incubar",
    "incubada": "incubar",
    "incubadas": "incubar",
    "incubado": "incubar",
    "incubadoras": "incubadora",
    "incubados": "incubar",
    "incubamos": "incubar",
    "incuban": "incubar",
    "incubando": "incubar",
    "incubara": "incubar",
    "incubarais": "incubar",
    "incubaran": "incubar",
    "incubaras": "incubar",
    "incubare": "incubar",
    "incubareis": "incubar",
    "incubaremos": "incubar",
    "incubaren": "incubar",
    "incubares": "incubar",
    "incubaron": "incubar",
    "incubará": "incubar",
    "incubarán": "incubar",
    "incubarás": "incubar",
    "incubaré": "incubar",
    "incubaréis": "incubar",
    "incubaría": "incubar",
    "incubaríais": "incubar",
    "incubaríamos": "incubar",
    "incubarían": "incubar",
    "incubarías": "incubar",
    "incubas": "incubar",
    "incubase": "incubar",
    "incubaseis": "incubar",
    "incubasen": "incubar",
    "incubases": "incubar",
    "incubaste": "incubar",
    "incubasteis": "incubar",
    "incube": "incubar",
    "incubemos": "incubar",
    "incuben": "incubar",
    "incubes": "incubar",
    "incubo": "incubar",
    "incubábamos": "incubar",
    "incubáis": "incubar",
    "incubáramos": "incubar",
    "incubáremos": "incubar",
    "incubásemos": "incubar",
    "incubé": "incubar",
    "incubéis": "incubar",
    "incubó": "incubar",
    "incudiales": "incudial",
    "incudoestapedia": "incudoestapedio",
    "incudoestapedias": "incudoestapedio",
    "incudoestapedios": "incudoestapedio",
    "incudomaleares": "incudomalear",
    "incudomaleolares": "incudomaleolar",
    "incuestionables": "incuestionable",
    "inculca": "inculcar",
    "inculcaba": "inculcar",
    "inculcabais": "inculcar",
    "inculcaban": "inculcar",
    "inculcabas": "inculcar",
    "inculcaciones": "inculcación",
    "inculcad": "inculcar",
    "inculcada": "inculcar",
    "inculcadas": "inculcar",
    "inculcado": "inculcar",
    "inculcadora": "inculcador",
    "inculcadoras": "inculcador",
    "inculcadores": "inculcador",
    "inculcados": "inculcar",
    "inculcamos": "inculcar",
    "inculcan": "inculcar",
    "inculcando": "inculcar",
    "inculcara": "inculcar",
    "inculcarais": "inculcar",
    "inculcaran": "inculcar",
    "inculcaras": "inculcar",
    "inculcare": "inculcar",
    "inculcareis": "inculcar",
    "inculcaremos": "inculcar",
    "inculcaren": "inculcar",
    "inculcares": "inculcar",
    "inculcaron": "inculcar",
    "inculcará": "inculcar",
    "inculcarán": "inculcar",
    "inculcarás": "inculcar",
    "inculcaré": "inculcar",
    "inculcaréis": "inculcar",
    "inculcaría": "inculcar",
    "inculcaríais": "inculcar",
    "inculcaríamos": "inculcar",
    "inculcarían": "inculcar",
    "inculcarías": "inculcar",
    "inculcas": "inculcar",
    "inculcase": "inculcar",
    "inculcaseis": "inculcar",
    "inculcasen": "inculcar",
    "inculcases": "inculcar",
    "inculcaste": "inculcar",
    "inculcasteis": "inculcar",
    "inculco": "inculcar",
    "inculcábamos": "inculcar",
    "inculcáis": "inculcar",
    "inculcáramos": "inculcar",
    "inculcáremos": "inculcar",
    "inculcásemos": "inculcar",
    "inculcó": "inculcar",
    "inculpa": "inculpar",
    "inculpaba": "inculpar",
    "inculpabais": "inculpar",
    "inculpaban": "inculpar",
    "inculpabas": "inculpar",
    "inculpabilidades": "inculpabilidad",
    "inculpables": "inculpable",
    "inculpaciones": "inculpación",
    "inculpad": "inculpar",
    "inculpada": "inculpar",
    "inculpadas": "inculpar",
    "inculpado": "inculpar",
    "inculpados": "inculpar",
    "inculpamos": "inculpar",
    "inculpan": "inculpar",
    "inculpando": "inculpar",
    "inculpara": "inculpar",
    "inculparais": "inculpar",
    "inculparan": "inculpar",
    "inculparas": "inculpar",
    "inculpare": "inculpar",
    "inculpareis": "inculpar",
    "inculparemos": "inculpar",
    "inculparen": "inculpar",
    "inculpares": "inculpar",
    "inculparon": "inculpar",
    "inculpará": "inculpar",
    "inculparán": "inculpar",
    "inculparás": "inculpar",
    "inculparé": "inculpar",
    "inculparéis": "inculpar",
    "inculparía": "inculpar",
    "inculparíais": "inculpar",
    "inculparíamos": "inculpar",
    "inculparían": "inculpar",
    "inculparías": "inculpar",
    "inculpas": "inculpar",
    "inculpase": "inculpar",
    "inculpaseis": "inculpar",
    "inculpasen": "inculpar",
    "inculpases": "inculpar",
    "inculpaste": "inculpar",
    "inculpasteis": "inculpar",
    "inculpatoria": "inculpatorio",
    "inculpatorias": "inculpatorio",
    "inculpatorios": "inculpatorio",
    "inculpe": "inculpar",
    "inculpemos": "inculpar",
    "inculpen": "inculpar",
    "inculpes": "inculpar",
    "inculpo": "inculpar",
    "inculpábamos": "inculpar",
    "inculpáis": "inculpar",
    "inculpáramos": "inculpar",
    "inculpáremos": "inculpar",
    "inculpásemos": "inculpar",
    "inculpé": "inculpar",
    "inculpéis": "inculpar",
    "inculpó": "inculpar",
    "inculque": "inculcar",
    "inculquemos": "inculcar",
    "inculquen": "inculcar",
    "inculques": "inculcar",
    "inculqué": "inculcar",
    "inculquéis": "inculcar",
    "inculta": "inculto",
    "incultas": "inculto",
    "incultivables": "incultivable",
    "incultos": "inculto",
    "inculturas": "incultura",
    "incumba": "incumbir",
    "incumbamos": "incumbir",
    "incumban": "incumbir",
    "incumbas": "incumbir",
    "incumbe": "incumbir",
    "incumben": "incumbir",
    "incumbencias": "incumbencia",
    "incumbentes": "incumbente",
    "incumbes": "incumbir",
    "incumbid": "incumbir",
    "incumbida": "incumbir",
    "incumbidas": "incumbir",
    "incumbido": "incumbir",
    "incumbidos": "incumbir",
    "incumbiendo": "incumbir",
    "incumbiera": "incumbir",
    "incumbierais": "incumbir",
    "incumbieran": "incumbir",
    "incumbieras": "incumbir",
    "incumbiere": "incumbir",
    "incumbiereis": "incumbir",
    "incumbieren": "incumbir",
    "incumbieres": "incumbir",
    "incumbieron": "incumbir",
    "incumbiese": "incumbir",
    "incumbieseis": "incumbir",
    "incumbiesen": "incumbir",
    "incumbieses": "incumbir",
    "incumbimos": "incumbir",
    "incumbiremos": "incumbir",
    "incumbirá": "incumbir",
    "incumbirán": "incumbir",
    "incumbirás": "incumbir",
    "incumbiré": "incumbir",
    "incumbiréis": "incumbir",
    "incumbiría": "incumbir",
    "incumbiríais": "incumbir",
    "incumbiríamos": "incumbir",
    "incumbirían": "incumbir",
    "incumbirías": "incumbir",
    "incumbiste": "incumbir",
    "incumbisteis": "incumbir",
    "incumbiéramos": "incumbir",
    "incumbiéremos": "incumbir",
    "incumbiésemos": "incumbir",
    "incumbió": "incumbir",
    "incumbo": "incumbir",
    "incumbáis": "incumbir",
    "incumbí": "incumbir",
    "incumbía": "incumbir",
    "incumbíais": "incumbir",
    "incumbíamos": "incumbir",
    "incumbían": "incumbir",
    "incumbías": "incumbir",
    "incumbís": "incumbir",
    "incumpla": "incumplir",
    "incumplamos": "incumplir",
    "incumplan": "incumplir",
    "incumplas": "incumplir",
    "incumple": "incumplir",
    "incumplen": "incumplir",
    "incumples": "incumplir",
    "incumplid": "incumplir",
    "incumplida": "incumplir",
    "incumplidas": "incumplir",
    "incumplido": "incumplir",
    "incumplidos": "incumplir",
    "incumpliendo": "incumplir",
    "incumpliera": "incumplir",
    "incumplierais": "incumplir",
    "incumplieran": "incumplir",
    "incumplieras": "incumplir",
    "incumpliere": "incumplir",
    "incumpliereis": "incumplir",
    "incumplieren": "incumplir",
    "incumplieres": "incumplir",
    "incumplieron": "incumplir",
    "incumpliese": "incumplir",
    "incumplieseis": "incumplir",
    "incumpliesen": "incumplir",
    "incumplieses": "incumplir",
    "incumplimientos": "incumplimiento",
    "incumplimos": "incumplir",
    "incumpliremos": "incumplir",
    "incumplirá": "incumplir",
    "incumplirán": "incumplir",
    "incumplirás": "incumplir",
    "incumpliré": "incumplir",
    "incumpliréis": "incumplir",
    "incumpliría": "incumplir",
    "incumpliríais": "incumplir",
    "incumpliríamos": "incumplir",
    "incumplirían": "incumplir",
    "incumplirías": "incumplir",
    "incumpliste": "incumplir",
    "incumplisteis": "incumplir",
    "incumpliéramos": "incumplir",
    "incumpliéremos": "incumplir",
    "incumpliésemos": "incumplir",
    "incumplió": "incumplir",
    "incumplo": "incumplir",
    "incumpláis": "incumplir",
    "incumplí": "incumplir",
    "incumplía": "incumplir",
    "incumplíais": "incumplir",
    "incumplíamos": "incumplir",
    "incumplían": "incumplir",
    "incumplías": "incumplir",
    "incumplís": "incumplir",
    "incunables": "incunable",
    "incurabilidades": "incurabilidad",
    "incurables": "incurable",
    "incurias": "incuria",
    "incuriosa": "incurioso",
    "incuriosas": "incurioso",
    "incuriosos": "incurioso",
    "incurra": "incurrir",
    "incurramos": "incurrir",
    "incurran": "incurrir",
    "incurras": "incurrir",
    "incurre": "incurrir",
    "incurren": "incurrir",
    "incurres": "incurrir",
    "incurrid": "incurrir",
    "incurrida": "incurrir",
    "incurridas": "incurrir",
    "incurrido": "incurrir",
    "incurridos": "incurrir",
    "incurriendo": "incurrir",
    "incurriera": "incurrir",
    "incurrierais": "incurrir",
    "incurrieran": "incurrir",
    "incurrieras": "incurrir",
    "incurriere": "incurrir",
    "incurriereis": "incurrir",
    "incurrieren": "incurrir",
    "incurrieres": "incurrir",
    "incurrieron": "incurrir",
    "incurriese": "incurrir",
    "incurrieseis": "incurrir",
    "incurriesen": "incurrir",
    "incurrieses": "incurrir",
    "incurrimos": "incurrir",
    "incurriremos": "incurrir",
    "incurrirá": "incurrir",
    "incurrirán": "incurrir",
    "incurrirás": "incurrir",
    "incurriré": "incurrir",
    "incurriréis": "incurrir",
    "incurriría": "incurrir",
    "incurriríais": "incurrir",
    "incurriríamos": "incurrir",
    "incurrirían": "incurrir",
    "incurrirías": "incurrir",
    "incurriste": "incurrir",
    "incurristeis": "incurrir",
    "incurriéramos": "incurrir",
    "incurriéremos": "incurrir",
    "incurriésemos": "incurrir",
    "incurrió": "incurrir",
    "incurro": "incurrir",
    "incurráis": "incurrir",
    "incurrí": "incurrir",
    "incurría": "incurrir",
    "incurríais": "incurrir",
    "incurríamos": "incurrir",
    "incurrían": "incurrir",
    "incurrías": "incurrir",
    "incurrís": "incurrir",
    "incursa": "incurso",
    "incursas": "incurso",
    "incursiona": "incursionar",
    "incursionaba": "incursionar",
    "incursionabais": "incursionar",
    "incursionaban": "incursionar",
    "incursionabas": "incursionar",
    "incursionad": "incursionar",
    "incursionada": "incursionar",
    "incursionadas": "incursionar",
    "incursionado": "incursionar",
    "incursionados": "incursionar",
    "incursionamos": "incursionar",
    "incursionan": "incursionar",
    "incursionando": "incursionar",
    "incursionara": "incursionar",
    "incursionarais": "incursionar",
    "incursionaran": "incursionar",
    "incursionaras": "incursionar",
    "incursionare": "incursionar",
    "incursionareis": "incursionar",
    "incursionaremos": "incursionar",
    "incursionaren": "incursionar",
    "incursionares": "incursionar",
    "incursionaron": "incursionar",
    "incursionará": "incursionar",
    "incursionarán": "incursionar",
    "incursionarás": "incursionar",
    "incursionaré": "incursionar",
    "incursionaréis": "incursionar",
    "incursionaría": "incursionar",
    "incursionaríais": "incursionar",
    "incursionaríamos": "incursionar",
    "incursionarían": "incursionar",
    "incursionarías": "incursionar",
    "incursionas": "incursionar",
    "incursionase": "incursionar",
    "incursionaseis": "incursionar",
    "incursionasen": "incursionar",
    "incursionases": "incursionar",
    "incursionaste": "incursionar",
    "incursionasteis": "incursionar",
    "incursione": "incursionar",
    "incursionemos": "incursionar",
    "incursionen": "incursionar",
    "incursiones": "incursionar",
    "incursiono": "incursionar",
    "incursionábamos": "incursionar",
    "incursionáis": "incursionar",
    "incursionáramos": "incursionar",
    "incursionáremos": "incursionar",
    "incursionásemos": "incursionar",
    "incursioné": "incursionar",
    "incursionéis": "incursionar",
    "incursionó": "incursionar",
    "incursos": "incurso",
    "incusa": "incuso",
    "incusas": "incuso",
    "incusos": "incuso",
    "incásica": "incásico",
    "incásicas": "incásico",
    "incásicos": "incásico",
    "incógnita": "incógnito",
    "incógnitas": "incógnito",
    "incógnitos": "incógnito",
    "incólumes": "incólume",
    "incómoda": "incómodo",
    "incómodas": "incómodo",
    "incómodos": "incómodo",
    "indaga": "indagar",
    "indagaba": "indagar",
    "indagabais": "indagar",
    "indagaban": "indagar",
    "indagabas": "indagar",
    "indagaciones": "indagación",
    "indagad": "indagar",
    "indagada": "indagar",
    "indagadas": "indagar",
    "indagado": "indagar",
    "indagadora": "indagador",
    "indagadoras": "indagador",
    "indagadores": "indagador",
    "indagados": "indagar",
    "indagamos": "indagar",
    "indagan": "indagar",
    "indagando": "indagar",
    "indagara": "indagar",
    "indagarais": "indagar",
    "indagaran": "indagar",
    "indagaras": "indagar",
    "indagare": "indagar",
    "indagareis": "indagar",
    "indagaremos": "indagar",
    "indagaren": "indagar",
    "indagares": "indagar",
    "indagaron": "indagar",
    "indagará": "indagar",
    "indagarán": "indagar",
    "indagarás": "indagar",
    "indagaré": "indagar",
    "indagaréis": "indagar",
    "indagaría": "indagar",
    "indagaríais": "indagar",
    "indagaríamos": "indagar",
    "indagarían": "indagar",
    "indagarías": "indagar",
    "indagas": "indagar",
    "indagase": "indagar",
    "indagaseis": "indagar",
    "indagasen": "indagar",
    "indagases": "indagar",
    "indagaste": "indagar",
    "indagasteis": "indagar",
    "indagatoria": "indagatorio",
    "indagatorias": "indagatorio",
    "indagatorios": "indagatorio",
    "indago": "indagar",
    "indague": "indagar",
    "indaguemos": "indagar",
    "indaguen": "indagar",
    "indagues": "indagar",
    "indagué": "indagar",
    "indaguéis": "indagar",
    "indagábamos": "indagar",
    "indagáis": "indagar",
    "indagáramos": "indagar",
    "indagáremos": "indagar",
    "indagásemos": "indagar",
    "indagó": "indagar",
    "indaminas": "indamina",
    "indanos": "indano",
    "indantrenos": "indantreno",
    "indantronas": "indantrona",
    "indebida": "indebido",
    "indebidas": "indebido",
    "indebidos": "indebido",
    "indecencias": "indecencia",
    "indecentes": "indecente",
    "indecibles": "indecible",
    "indecidibilidades": "indecidibilidad",
    "indecidibles": "indecidible",
    "indecisa": "indeciso",
    "indecisas": "indeciso",
    "indecisiones": "indecisión",
    "indecisoria": "indecisorio",
    "indecisorias": "indecisorio",
    "indecisorios": "indecisorio",
    "indecisos": "indeciso",
    "indeclarables": "indeclarable",
    "indeclinables": "indeclinable",
    "indecoros": "indecoro",
    "indecorosa": "indecoroso",
    "indecorosas": "indecoroso",
    "indecorosos": "indecoroso",
    "indefectibilidades": "indefectibilidad",
    "indefectibles": "indefectible",
    "indefendibles": "indefendible",
    "indefensa": "indefenso",
    "indefensas": "indefenso",
    "indefensibles": "indefensible",
    "indefensiones": "indefensión",
    "indefensos": "indefenso",
    "indefinibles": "indefinible",
    "indefiniciones": "indefinición",
    "indefinida": "indefinido",
    "indefinidas": "indefinido",
    "indefinidos": "indefinido",
    "indeformables": "indeformable",
    "indehiscentes": "indehiscente",
    "indelebles": "indeleble",
    "indelegables": "indelegable",
    "indelicada": "indelicado",
    "indelicadas": "indelicado",
    "indelicadezas": "indelicadeza",
    "indelicados": "indelicado",
    "indemnes": "indemne",
    "indemnice": "indemnizar",
    "indemnicemos": "indemnizar",
    "indemnicen": "indemnizar",
    "indemnices": "indemnizar",
    "indemnicé": "indemnizar",
    "indemnicéis": "indemnizar",
    "indemnidades": "indemnidad",
    "indemniza": "indemnizar",
    "indemnizaba": "indemnizar",
    "indemnizabais": "indemnizar",
    "indemnizaban": "indemnizar",
    "indemnizabas": "indemnizar",
    "indemnizables": "indemnizable",
    "indemnizaciones": "indemnización",
    "indemnizad": "indemnizar",
    "indemnizada": "indemnizar",
    "indemnizadas": "indemnizar",
    "indemnizado": "indemnizar",
    "indemnizados": "indemnizar",
    "indemnizamos": "indemnizar",
    "indemnizan": "indemnizar",
    "indemnizando": "indemnizar",
    "indemnizara": "indemnizar",
    "indemnizarais": "indemnizar",
    "indemnizaran": "indemnizar",
    "indemnizaras": "indemnizar",
    "indemnizare": "indemnizar",
    "indemnizareis": "indemnizar",
    "indemnizaremos": "indemnizar",
    "indemnizaren": "indemnizar",
    "indemnizares": "indemnizar",
    "indemnizaron": "indemnizar",
    "indemnizará": "indemnizar",
    "indemnizarán": "indemnizar",
    "indemnizarás": "indemnizar",
    "indemnizaré": "indemnizar",
    "indemnizaréis": "indemnizar",
    "indemnizaría": "indemnizar",
    "indemnizaríais": "indemnizar",
    "indemnizaríamos": "indemnizar",
    "indemnizarían": "indemnizar",
    "indemnizarías": "indemnizar",
    "indemnizas": "indemnizar",
    "indemnizase": "indemnizar",
    "indemnizaseis": "indemnizar",
    "indemnizasen": "indemnizar",
    "indemnizases": "indemnizar",
    "indemnizaste": "indemnizar",
    "indemnizasteis": "indemnizar",
    "indemnizatoria": "indemnizatorio",
    "indemnizatorias": "indemnizatorio",
    "indemnizatorios": "indemnizatorio",
    "indemnizo": "indemnizar",
    "indemnizábamos": "indemnizar",
    "indemnizáis": "indemnizar",
    "indemnizáramos": "indemnizar",
    "indemnizáremos": "indemnizar",
    "indemnizásemos": "indemnizar",
    "indemnizó": "indemnizar",
    "indemorables": "indemorable",
    "indemostrabilidades": "indemostrabilidad",
    "indemostrables": "indemostrable",
    "indenos": "indeno",
    "indentaciones": "indentación",
    "independencias": "independencia",
    "independentismos": "independentismo",
    "independentistas": "independentista",
    "independice": "independizar",
    "independicemos": "independizar",
    "independicen": "independizar",
    "independices": "independizar",
    "independicé": "independizar",
    "independicéis": "independizar",
    "independientes": "independiente",
    "independiza": "independizar",
    "independizaba": "independizar",
    "independizabais": "independizar",
    "independizaban": "independizar",
    "independizabas": "independizar",
    "independizad": "independizar",
    "independizada": "independizar",
    "independizadas": "independizar",
    "independizado": "independizar",
    "independizados": "independizar",
    "independizamos": "independizar",
    "independizan": "independizar",
    "independizando": "independizar",
    "independizara": "independizar",
    "independizarais": "independizar",
    "independizaran": "independizar",
    "independizaras": "independizar",
    "independizare": "independizar",
    "independizareis": "independizar",
    "independizaremos": "independizar",
    "independizaren": "independizar",
    "independizares": "independizar",
    "independizaron": "independizar",
    "independizará": "independizar",
    "independizarán": "independizar",
    "independizarás": "independizar",
    "independizaré": "independizar",
    "independizaréis": "independizar",
    "independizaría": "independizar",
    "independizaríais": "independizar",
    "independizaríamos": "independizar",
    "independizarían": "independizar",
    "independizarías": "independizar",
    "independizas": "independizar",
    "independizase": "independizar",
    "independizaseis": "independizar",
    "independizasen": "independizar",
    "independizases": "independizar",
    "independizaste": "independizar",
    "independizasteis": "independizar",
    "independizo": "independizar",
    "independizábamos": "independizar",
    "independizáis": "independizar",
    "independizáramos": "independizar",
    "independizáremos": "independizar",
    "independizásemos": "independizar",
    "independizó": "independizar",
    "indescifrables": "indescifrable",
    "indescomponibles": "indescomponible",
    "indescriptibles": "indescriptible",
    "indeseables": "indeseable",
    "indeseada": "indeseado",
    "indeseadas": "indeseado",
    "indeseados": "indeseado",
    "indesmallables": "indesmallable",
    "indestructibilidades": "indestructibilidad",
    "indestructibles": "indestructible",
    "indeterminables": "indeterminable",
    "indeterminaciones": "indeterminación",
    "indeterminada": "indeterminado",
    "indeterminadas": "indeterminado",
    "indeterminados": "indeterminado",
    "indeterminismos": "indeterminismo",
    "indeterministas": "indeterminista",
    "indevociones": "indevoción",
    "indevota": "indevoto",
    "indevotas": "indevoto",
    "indevotos": "indevoto",
    "indexa": "indexar",
    "indexaba": "indexar",
    "indexabais": "indexar",
    "indexaban": "indexar",
    "indexabas": "indexar",
    "indexaciones": "indexación",
    "indexaciónes": "indexación",
    "indexad": "indexar",
    "indexada": "indexar",
    "indexadas": "indexar",
    "indexado": "indexar",
    "indexados": "indexar",
    "indexamos": "indexar",
    "indexan": "indexar",
    "indexando": "indexar",
    "indexara": "indexar",
    "indexarais": "indexar",
    "indexaran": "indexar",
    "indexaras": "indexar",
    "indexare": "indexar",
    "indexareis": "indexar",
    "indexaremos": "indexar",
    "indexaren": "indexar",
    "indexares": "indexar",
    "indexaron": "indexar",
    "indexará": "indexar",
    "indexarán": "indexar",
    "indexarás": "indexar",
    "indexaré": "indexar",
    "indexaréis": "indexar",
    "indexaría": "indexar",
    "indexaríais": "indexar",
    "indexaríamos": "indexar",
    "indexarían": "indexar",
    "indexarías": "indexar",
    "indexas": "indexar",
    "indexase": "indexar",
    "indexaseis": "indexar",
    "indexasen": "indexar",
    "indexases": "indexar",
    "indexaste": "indexar",
    "indexasteis": "indexar",
    "indexe": "indexar",
    "indexemos": "indexar",
    "indexen": "indexar",
    "indexes": "indexar",
    "indexo": "indexar",
    "indexábamos": "indexar",
    "indexáis": "indexar",
    "indexáramos": "indexar",
    "indexáremos": "indexar",
    "indexásemos": "indexar",
    "indexé": "indexar",
    "indexéis": "indexar",
    "indexó": "indexar",
    "india": "indio",
    "indiadas": "indiada",
    "indiana": "indiano",
    "indianas": "indiano",
    "indianismos": "indianismo",
    "indianistas": "indianista",
    "indianos": "indiano",
    "indias": "indio",
    "indica": "indicar",
    "indicaba": "indicar",
    "indicabais": "indicar",
    "indicaban": "indicar",
    "indicabas": "indicar",
    "indicaciones": "indicación",
    "indicad": "indicar",
    "indicada": "indicar",
    "indicadas": "indicar",
    "indicado": "indicar",
    "indicadora": "indicador",
    "indicadoras": "indicador",
    "indicadores": "indicador",
    "indicados": "indicar",
    "indicamos": "indicar",
    "indican": "indicar",
    "indicando": "indicar",
    "indicara": "indicar",
    "indicarais": "indicar",
    "indicaran": "indicar",
    "indicaras": "indicar",
    "indicare": "indicar",
    "indicareis": "indicar",
    "indicaremos": "indicar",
    "indicaren": "indicar",
    "indicares": "indicar",
    "indicaron": "indicar",
    "indicará": "indicar",
    "indicarán": "indicar",
    "indicarás": "indicar",
    "indicaré": "indicar",
    "indicaréis": "indicar",
    "indicaría": "indicar",
    "indicaríais": "indicar",
    "indicaríamos": "indicar",
    "indicarían": "indicar",
    "indicarías": "indicar",
    "indicas": "indicar",
    "indicase": "indicar",
    "indicaseis": "indicar",
    "indicasen": "indicar",
    "indicases": "indicar",
    "indicaste": "indicar",
    "indicasteis": "indicar",
    "indicativa": "indicativo",
    "indicativas": "indicativo",
    "indicativos": "indicativo",
    "indicatrices": "indicatriz",
    "indicciones": "indicción",
    "indicia": "indiciar",
    "indiciaba": "indiciar",
    "indiciabais": "indiciar",
    "indiciaban": "indiciar",
    "indiciabas": "indiciar",
    "indiciaciones": "indiciación",
    "indiciad": "indiciar",
    "indiciada": "indiciar",
    "indiciadas": "indiciar",
    "indiciado": "indiciar",
    "indiciadora": "indiciador",
    "indiciadoras": "indiciador",
    "indiciadores": "indiciador",
    "indiciados": "indiciar",
    "indiciamos": "indiciar",
    "indician": "indiciar",
    "indiciando": "indiciar",
    "indiciara": "indiciar",
    "indiciarais": "indiciar",
    "indiciaran": "indiciar",
    "indiciaras": "indiciar",
    "indiciare": "indiciar",
    "indiciareis": "indiciar",
    "indiciaremos": "indiciar",
    "indiciaren": "indiciar",
    "indiciares": "indiciar",
    "indiciaria": "indiciario",
    "indiciarias": "indiciario",
    "indiciarios": "indiciario",
    "indiciaron": "indiciar",
    "indiciará": "indiciar",
    "indiciarán": "indiciar",
    "indiciarás": "indiciar",
    "indiciaré": "indiciar",
    "indiciaréis": "indiciar",
    "indiciaría": "indiciar",
    "indiciaríais": "indiciar",
    "indiciaríamos": "indiciar",
    "indiciarían": "indiciar",
    "indiciarías": "indiciar",
    "indicias": "indiciar",
    "indiciase": "indiciar",
    "indiciaseis": "indiciar",
    "indiciasen": "indiciar",
    "indiciases": "indiciar",
    "indiciaste": "indiciar",
    "indiciasteis": "indiciar",
    "indicie": "indiciar",
    "indiciemos": "indiciar",
    "indicien": "indiciar",
    "indicies": "indiciar",
    "indicio": "indiciar",
    "indicios": "indicio",
    "indiciosa": "indicioso",
    "indiciosas": "indicioso",
    "indiciosos": "indicioso",
    "indiciábamos": "indiciar",
    "indiciáis": "indiciar",
    "indiciáramos": "indiciar",
    "indiciáremos": "indiciar",
    "indiciásemos": "indiciar",
    "indicié": "indiciar",
    "indiciéis": "indiciar",
    "indició": "indiciar",
    "indico": "indicar",
    "indicolitas": "indicolita",
    "indicábamos": "indicar",
    "indicáis": "indicar",
    "indicáramos": "indicar",
    "indicáremos": "indicar",
    "indicásemos": "indicar",
    "indicó": "indicar",
    "indiferenciables": "indiferenciable",
    "indiferenciada": "indiferenciado",
    "indiferenciadas": "indiferenciado",
    "indiferenciados": "indiferenciado",
    "indiferencias": "indiferencia",
    "indiferentes": "indiferente",
    "indiferentismos": "indiferentismo",
    "indigencias": "indigencia",
    "indigenismos": "indigenismo",
    "indigenistas": "indigenista",
    "indigentes": "indigente",
    "indigesta": "indigesto",
    "indigestaba": "indigestar",
    "indigestabais": "indigestar",
    "indigestaban": "indigestar",
    "indigestabas": "indigestar",
    "indigestad": "indigestar",
    "indigestada": "indigestar",
    "indigestadas": "indigestar",
    "indigestado": "indigestar",
    "indigestados": "indigestar",
    "indigestamos": "indigestar",
    "indigestan": "indigestar",
    "indigestando": "indigestar",
    "indigestara": "indigestar",
    "indigestarais": "indigestar",
    "indigestaran": "indigestar",
    "indigestaras": "indigestar",
    "indigestare": "indigestar",
    "indigestareis": "indigestar",
    "indigestaremos": "indigestar",
    "indigestaren": "indigestar",
    "indigestares": "indigestar",
    "indigestaron": "indigestar",
    "indigestará": "indigestar",
    "indigestarán": "indigestar",
    "indigestarás": "indigestar",
    "indigestaré": "indigestar",
    "indigestaréis": "indigestar",
    "indigestaría": "indigestar",
    "indigestaríais": "indigestar",
    "indigestaríamos": "indigestar",
    "indigestarían": "indigestar",
    "indigestarías": "indigestar",
    "indigestas": "indigesto",
    "indigestase": "indigestar",
    "indigestaseis": "indigestar",
    "indigestasen": "indigestar",
    "indigestases": "indigestar",
    "indigestaste": "indigestar",
    "indigestasteis": "indigestar",
    "indigeste": "indigestar",
    "indigestemos": "indigestar",
    "indigesten": "indigestar",
    "indigestes": "indigestar",
    "indigestibles": "indigestible",
    "indigestiones": "indigestión",
    "indigesto": "indigestar",
    "indigestos": "indigesto",
    "indigestábamos": "indigestar",
    "indigestáis": "indigestar",
    "indigestáramos": "indigestar",
    "indigestáremos": "indigestar",
    "indigestásemos": "indigestar",
    "indigesté": "indigestar",
    "indigestéis": "indigestar",
    "indigestó": "indigestar",
    "indigetes": "indigete",
    "indigna": "indigno",
    "indignaba": "indignar",
    "indignabais": "indignar",
    "indignaban": "indignar",
    "indignabas": "indignar",
    "indignaciones": "indignación",
    "indignad": "indignar",
    "indignada": "indignar",
    "indignadas": "indignar",
    "indignado": "indignar",
    "indignados": "indignar",
    "indignamos": "indignar",
    "indignan": "indignar",
    "indignando": "indignar",
    "indignantes": "indignante",
    "indignara": "indignar",
    "indignarais": "indignar",
    "indignaran": "indignar",
    "indignaras": "indignar",
    "indignare": "indignar",
    "indignareis": "indignar",
    "indignaremos": "indignar",
    "indignaren": "indignar",
    "indignares": "indignar",
    "indignaron": "indignar",
    "indignará": "indignar",
    "indignarán": "indignar",
    "indignarás": "indignar",
    "indignaré": "indignar",
    "indignaréis": "indignar",
    "indignaría": "indignar",
    "indignaríais": "indignar",
    "indignaríamos": "indignar",
    "indignarían": "indignar",
    "indignarías": "indignar",
    "indignas": "indigno",
    "indignase": "indignar",
    "indignaseis": "indignar",
    "indignasen": "indignar",
    "indignases": "indignar",
    "indignaste": "indignar",
    "indignasteis": "indignar",
    "indigne": "indignar",
    "indignemos": "indignar",
    "indignen": "indignar",
    "indignes": "indignar",
    "indignidades": "indignidad",
    "indigno": "indignar",
    "indignos": "indigno",
    "indignábamos": "indignar",
    "indignáis": "indignar",
    "indignáramos": "indignar",
    "indignáremos": "indignar",
    "indignásemos": "indignar",
    "indigné": "indignar",
    "indignéis": "indignar",
    "indignó": "indignar",
    "indigolitas": "indigolita",
    "indios": "indio",
    "indique": "indicar",
    "indiquemos": "indicar",
    "indiquen": "indicar",
    "indiques": "indicar",
    "indiqué": "indicar",
    "indiquéis": "indicar",
    "indirecta": "indirecto",
    "indirectas": "indirecto",
    "indirectos": "indirecto",
    "indiscernibles": "indiscernible",
    "indisciplina": "indisciplinar",
    "indisciplinaba": "indisciplinar",
    "indisciplinabais": "indisciplinar",
    "indisciplinaban": "indisciplinar",
    "indisciplinabas": "indisciplinar",
    "indisciplinables": "indisciplinable",
    "indisciplinad": "indisciplinar",
    "indisciplinada": "indisciplinar",
    "indisciplinadas": "indisciplinar",
    "indisciplinado": "indisciplinar",
    "indisciplinados": "indisciplinar",
    "indisciplinamos": "indisciplinar",
    "indisciplinan": "indisciplinar",
    "indisciplinando": "indisciplinar",
    "indisciplinara": "indisciplinar",
    "indisciplinarais": "indisciplinar",
    "indisciplinaran": "indisciplinar",
    "indisciplinaras": "indisciplinar",
    "indisciplinare": "indisciplinar",
    "indisciplinareis": "indisciplinar",
    "indisciplinaremos": "indisciplinar",
    "indisciplinaren": "indisciplinar",
    "indisciplinares": "indisciplinar",
    "indisciplinaron": "indisciplinar",
    "indisciplinará": "indisciplinar",
    "indisciplinarán": "indisciplinar",
    "indisciplinarás": "indisciplinar",
    "indisciplinaré": "indisciplinar",
    "indisciplinaréis": "indisciplinar",
    "indisciplinaría": "indisciplinar",
    "indisciplinaríais": "indisciplinar",
    "indisciplinaríamos": "indisciplinar",
    "indisciplinarían": "indisciplinar",
    "indisciplinarías": "indisciplinar",
    "indisciplinas": "indisciplinar",
    "indisciplinase": "indisciplinar",
    "indisciplinaseis": "indisciplinar",
    "indisciplinasen": "indisciplinar",
    "indisciplinases": "indisciplinar",
    "indisciplinaste": "indisciplinar",
    "indisciplinasteis": "indisciplinar",
    "indiscipline": "indisciplinar",
    "indisciplinemos": "indisciplinar",
    "indisciplinen": "indisciplinar",
    "indisciplines": "indisciplinar",
    "indisciplino": "indisciplinar",
    "indisciplinábamos": "indisciplinar",
    "indisciplináis": "indisciplinar",
    "indisciplináramos": "indisciplinar",
    "indisciplináremos": "indisciplinar",
    "indisciplinásemos": "indisciplinar",
    "indiscipliné": "indisciplinar",
    "indisciplinéis": "indisciplinar",
    "indisciplinó": "indisciplinar",
    "indiscreciones": "indiscreción",
    "indiscreta": "indiscreto",
    "indiscretas": "indiscreto",
    "indiscretos": "indiscreto",
    "indiscriminada": "indiscriminado",
    "indiscriminadas": "indiscriminado",
    "indiscriminados": "indiscriminado",
    "indisculpables": "indisculpable",
    "indiscutibles": "indiscutible",
    "indiscutida": "indiscutido",
    "indiscutidas": "indiscutido",
    "indiscutidos": "indiscutido",
    "indisolubilidades": "indisolubilidad",
    "indisolubles": "indisoluble",
    "indispensabilidades": "indispensabilidad",
    "indispensables": "indispensable",
    "indispondremos": "indisponer",
    "indispondrá": "indisponer",
    "indispondrán": "indisponer",
    "indispondrás": "indisponer",
    "indispondré": "indisponer",
    "indispondréis": "indisponer",
    "indispondría": "indisponer",
    "indispondríais": "indisponer",
    "indispondríamos": "indisponer",
    "indispondrían": "indisponer",
    "indispondrías": "indisponer",
    "indispone": "indisponer",
    "indisponed": "indisponer",
    "indisponemos": "indisponer",
    "indisponen": "indisponer",
    "indispones": "indisponer",
    "indisponga": "indisponer",
    "indispongamos": "indisponer",
    "indispongan": "indisponer",
    "indispongas": "indisponer",
    "indispongo": "indisponer",
    "indispongáis": "indisponer",
    "indisponibilidades": "indisponibilidad",
    "indisponiendo": "indisponer",
    "indisponéis": "indisponer",
    "indisponía": "indisponer",
    "indisponíais": "indisponer",
    "indisponíamos": "indisponer",
    "indisponían": "indisponer",
    "indisponías": "indisponer",
    "indisposiciones": "indisposición",
    "indispuesta": "indisponer",
    "indispuestas": "indisponer",
    "indispuesto": "indisponer",
    "indispuestos": "indisponer",
    "indispuse": "indisponer",
    "indispusiera": "indisponer",
    "indispusierais": "indisponer",
    "indispusieran": "indisponer",
    "indispusieras": "indisponer",
    "indispusiere": "indisponer",
    "indispusiereis": "indisponer",
    "indispusieren": "indisponer",
    "indispusieres": "indisponer",
    "indispusieron": "indisponer",
    "indispusiese": "indisponer",
    "indispusieseis": "indisponer",
    "indispusiesen": "indisponer",
    "indispusieses": "indisponer",
    "indispusimos": "indisponer",
    "indispusiste": "indisponer",
    "indispusisteis": "indisponer",
    "indispusiéramos": "indisponer",
    "indispusiéremos": "indisponer",
    "indispusiésemos": "indisponer",
    "indispuso": "indisponer",
    "indisputabilidades": "indisputabilidad",
    "indispón": "indisponer",
    "indistinguibles": "indistinguible",
    "indistinta": "indistinto",
    "indistintas": "indistinto",
    "indistintos": "indistinto",
    "individua": "individuo",
    "individuaba": "individuar",
    "individuabais": "individuar",
    "individuaban": "individuar",
    "individuabas": "individuar",
    "individuaciones": "individuación",
    "individuad": "individuar",
    "individuada": "individuar",
    "individuadas": "individuar",
    "individuado": "individuar",
    "individuados": "individuar",
    "individuales": "individual",
    "individualice": "individualizar",
    "individualicemos": "individualizar",
    "individualicen": "individualizar",
    "individualices": "individualizar",
    "individualicé": "individualizar",
    "individualicéis": "individualizar",
    "individualidades": "individualidad",
    "individualismos": "individualismo",
    "individualistas": "individualista",
    "individualiza": "individualizar",
    "individualizaba": "individualizar",
    "individualizabais": "individualizar",
    "individualizaban": "individualizar",
    "individualizabas": "individualizar",
    "individualizables": "individualizable",
    "individualizaciones": "individualización",
    "individualizad": "individualizar",
    "individualizada": "individualizar",
    "individualizadas": "individualizar",
    "individualizado": "individualizar",
    "individualizados": "individualizar",
    "individualizamos": "individualizar",
    "individualizan": "individualizar",
    "individualizando": "individualizar",
    "individualizara": "individualizar",
    "individualizarais": "individualizar",
    "individualizaran": "individualizar",
    "individualizaras": "individualizar",
    "individualizare": "individualizar",
    "individualizareis": "individualizar",
    "individualizaremos": "individualizar",
    "individualizaren": "individualizar",
    "individualizares": "individualizar",
    "individualizaron": "individualizar",
    "individualizará": "individualizar",
    "individualizarán": "individualizar",
    "individualizarás": "individualizar",
    "individualizaré": "individualizar",
    "individualizaréis": "individualizar",
    "individualizaría": "individualizar",
    "individualizaríais": "individualizar",
    "individualizaríamos": "individualizar",
    "individualizarían": "individualizar",
    "individualizarías": "individualizar",
    "individualizas": "individualizar",
    "individualizase": "individualizar",
    "individualizaseis": "individualizar",
    "individualizasen": "individualizar",
    "individualizases": "individualizar",
    "individualizaste": "individualizar",
    "individualizasteis": "individualizar",
    "individualizo": "individualizar",
    "individualizábamos": "individualizar",
    "individualizáis": "individualizar",
    "individualizáramos": "individualizar",
    "individualizáremos": "individualizar",
    "individualizásemos": "individualizar",
    "individualizó": "individualizar",
    "individuamos": "individuar",
    "individuando": "individuar",
    "individuara": "individuar",
    "individuarais": "individuar",
    "individuaran": "individuar",
    "individuaras": "individuar",
    "individuare": "individuar",
    "individuareis": "individuar",
    "individuaremos": "individuar",
    "individuaren": "individuar",
    "individuares": "individuar",
    "individuaron": "individuar",
    "individuará": "individuar",
    "individuarán": "individuar",
    "individuarás": "individuar",
    "individuaré": "individuar",
    "individuaréis": "individuar",
    "individuaría": "individuar",
    "individuaríais": "individuar",
    "individuaríamos": "individuar",
    "individuarían": "individuar",
    "individuarías": "individuar",
    "individuas": "individuo",
    "individuase": "individuar",
    "individuaseis": "individuar",
    "individuasen": "individuar",
    "individuases": "individuar",
    "individuaste": "individuar",
    "individuasteis": "individuar",
    "individuemos": "individuar",
    "individuos": "individuo",
    "individuábamos": "individuar",
    "individuáis": "individuar",
    "individuáramos": "individuar",
    "individuáremos": "individuar",
    "individuásemos": "individuar",
    "individué": "individuar",
    "individuéis": "individuar",
    "individuó": "individuar",
    "individúa": "individuar",
    "individúan": "individuar",
    "individúas": "individuar",
    "individúe": "individuar",
    "individúen": "individuar",
    "individúes": "individuar",
    "individúo": "individuar",
    "indivisa": "indiviso",
    "indivisas": "indiviso",
    "indivisibilidades": "indivisibilidad",
    "indivisibles": "indivisible",
    "indivisiones": "indivisión",
    "indivisos": "indiviso",
    "indizaciones": "indización",
    "indizada": "indizado",
    "indizadas": "indizado",
    "indizados": "indizado",
    "indiófila": "indiófilo",
    "indiófilas": "indiófilo",
    "indiófilos": "indiófilo",
    "indoafganas": "indoafgana",
    "indochina": "indochino",
    "indochinas": "indochino",
    "indochinos": "indochino",
    "indocilidades": "indocilidad",
    "indocta": "indocto",
    "indoctas": "indocto",
    "indoctos": "indocto",
    "indocumentaciones": "indocumentación",
    "indocumentada": "indocumentado",
    "indocumentadas": "indocumentado",
    "indocumentados": "indocumentado",
    "indoeuropea": "indoeuropeo",
    "indoeuropeas": "indoeuropeo",
    "indoeuropeos": "indoeuropeo",
    "indofenoles": "indofenol",
    "indogermánica": "indogermánico",
    "indogermánicas": "indogermánico",
    "indogermánicos": "indogermánico",
    "indoirania": "indoiranio",
    "indoiranias": "indoiranio",
    "indoiranios": "indoiranio",
    "indolbutírica": "indolbutírico",
    "indolbutíricas": "indolbutírico",
    "indolbutíricos": "indolbutírico",
    "indolencias": "indolencia",
    "indolentes": "indolente",
    "indoles": "indol",
    "indolora": "indoloro",
    "indoloras": "indoloro",
    "indoloros": "indoloro",
    "indomabilidades": "indomabilidad",
    "indomables": "indomable",
    "indomada": "indomado",
    "indomadas": "indomado",
    "indomados": "indomado",
    "indomesticables": "indomesticable",
    "indometacinas": "indometacina",
    "indonesia": "indonesio",
    "indonesias": "indonesio",
    "indonesios": "indonesio",
    "indonésica": "indonésico",
    "indonésicas": "indonésico",
    "indonésicos": "indonésico",
    "indoors": "indoor",
    "indorromana": "indorromano",
    "indorromanas": "indorromano",
    "indorromanos": "indorromano",
    "indostana": "indostano",
    "indostanas": "indostano",
    "indostanesa": "indostanés",
    "indostanesas": "indostanés",
    "indostaneses": "indostanés",
    "indostanos": "indostano",
    "indostaníes": "indostaní",
    "indostanís": "indostaní",
    "indostánica": "indostánico",
    "indostánicas": "indostánico",
    "indostánicos": "indostánico",
    "indotaciones": "indotación",
    "indotada": "indotado",
    "indotadas": "indotado",
    "indotados": "indotado",
    "indubitables": "indubitable",
    "indubitada": "indubitado",
    "indubitadas": "indubitado",
    "indubitados": "indubitado",
    "inducciones": "inducción",
    "induce": "inducir",
    "inducen": "inducir",
    "induces": "inducir",
    "inducid": "inducir",
    "inducida": "inducir",
    "inducidas": "inducir",
    "inducido": "inducir",
    "inducidos": "inducir",
    "induciendo": "inducir",
    "inducimos": "inducir",
    "induciremos": "inducir",
    "inducirá": "inducir",
    "inducirán": "inducir",
    "inducirás": "inducir",
    "induciré": "inducir",
    "induciréis": "inducir",
    "induciría": "inducir",
    "induciríais": "inducir",
    "induciríamos": "inducir",
    "inducirían": "inducir",
    "inducirías": "inducir",
    "inductancias": "inductancia",
    "inductiva": "inductivo",
    "inductivas": "inductivo",
    "inductividades": "inductividad",
    "inductivos": "inductivo",
    "inductora": "inductor",
    "inductoras": "inductor",
    "inductores": "inductor",
    "inductómetros": "inductómetro",
    "inducía": "inducir",
    "inducíais": "inducir",
    "inducíamos": "inducir",
    "inducían": "inducir",
    "inducías": "inducir",
    "inducís": "inducir",
    "indudables": "indudable",
    "induje": "inducir",
    "indujera": "inducir",
    "indujerais": "inducir",
    "indujeran": "inducir",
    "indujeras": "inducir",
    "indujere": "inducir",
    "indujereis": "inducir",
    "indujeren": "inducir",
    "indujeres": "inducir",
    "indujeron": "inducir",
    "indujese": "inducir",
    "indujeseis": "inducir",
    "indujesen": "inducir",
    "indujeses": "inducir",
    "indujimos": "inducir",
    "indujiste": "inducir",
    "indujisteis": "inducir",
    "indujo": "inducir",
    "indujéramos": "inducir",
    "indujéremos": "inducir",
    "indujésemos": "inducir",
    "indulgencia": "indulgenciar",
    "indulgenciaba": "indulgenciar",
    "indulgenciabais": "indulgenciar",
    "indulgenciaban": "indulgenciar",
    "indulgenciabas": "indulgenciar",
    "indulgenciad": "indulgenciar",
    "indulgenciada": "indulgenciar",
    "indulgenciadas": "indulgenciar",
    "indulgenciado": "indulgenciar",
    "indulgenciados": "indulgenciar",
    "indulgenciamos": "indulgenciar",
    "indulgencian": "indulgenciar",
    "indulgenciando": "indulgenciar",
    "indulgenciara": "indulgenciar",
    "indulgenciarais": "indulgenciar",
    "indulgenciaran": "indulgenciar",
    "indulgenciaras": "indulgenciar",
    "indulgenciare": "indulgenciar",
    "indulgenciareis": "indulgenciar",
    "indulgenciaremos": "indulgenciar",
    "indulgenciaren": "indulgenciar",
    "indulgenciares": "indulgenciar",
    "indulgenciaron": "indulgenciar",
    "indulgenciará": "indulgenciar",
    "indulgenciarán": "indulgenciar",
    "indulgenciarás": "indulgenciar",
    "indulgenciaré": "indulgenciar",
    "indulgenciaréis": "indulgenciar",
    "indulgenciaría": "indulgenciar",
    "indulgenciaríais": "indulgenciar",
    "indulgenciaríamos": "indulgenciar",
    "indulgenciarían": "indulgenciar",
    "indulgenciarías": "indulgenciar",
    "indulgencias": "indulgenciar",
    "indulgenciase": "indulgenciar",
    "indulgenciaseis": "indulgenciar",
    "indulgenciasen": "indulgenciar",
    "indulgenciases": "indulgenciar",
    "indulgenciaste": "indulgenciar",
    "indulgenciasteis": "indulgenciar",
    "indulgencie": "indulgenciar",
    "indulgenciemos": "indulgenciar",
    "indulgencien": "indulgenciar",
    "indulgencies": "indulgenciar",
    "indulgencio": "indulgenciar",
    "indulgenciábamos": "indulgenciar",
    "indulgenciáis": "indulgenciar",
    "indulgenciáramos": "indulgenciar",
    "indulgenciáremos": "indulgenciar",
    "indulgenciásemos": "indulgenciar",
    "indulgencié": "indulgenciar",
    "indulgenciéis": "indulgenciar",
    "indulgenció": "indulgenciar",
    "indulgentes": "indulgente",
    "indulinas": "indulina",
    "indulta": "indultar",
    "indultaba": "indultar",
    "indultabais": "indultar",
    "indultaban": "indultar",
    "indultabas": "indultar",
    "indultad": "indultar",
    "indultada": "indultar",
    "indultadas": "indultar",
    "indultado": "indultar",
    "indultados": "indultar",
    "indultamos": "indultar",
    "indultan": "indultar",
    "indultando": "indultar",
    "indultara": "indultar",
    "indultarais": "indultar",
    "indultaran": "indultar",
    "indultaras": "indultar",
    "indultare": "indultar",
    "indultareis": "indultar",
    "indultaremos": "indultar",
    "indultaren": "indultar",
    "indultares": "indultar",
    "indultaron": "indultar",
    "indultará": "indultar",
    "indultarán": "indultar",
    "indultarás": "indultar",
    "indultaré": "indultar",
    "indultaréis": "indultar",
    "indultaría": "indultar",
    "indultaríais": "indultar",
    "indultaríamos": "indultar",
    "indultarían": "indultar",
    "indultarías": "indultar",
    "indultas": "indultar",
    "indultase": "indultar",
    "indultaseis": "indultar",
    "indultasen": "indultar",
    "indultases": "indultar",
    "indultaste": "indultar",
    "indultasteis": "indultar",
    "indulte": "indultar",
    "indultemos": "indultar",
    "indulten": "indultar",
    "indultes": "indultar",
    "indulto": "indultar",
    "indultos": "indulto",
    "indultábamos": "indultar",
    "indultáis": "indultar",
    "indultáramos": "indultar",
    "indultáremos": "indultar",
    "indultásemos": "indultar",
    "indulté": "indultar",
    "indultéis": "indultar",
    "indultó": "indultar",
    "indumentaria": "indumentario",
    "indumentarias": "indumentario",
    "indumentarios": "indumentario",
    "indumentos": "indumento",
    "indura": "indurar",
    "induraba": "indurar",
    "indurabais": "indurar",
    "induraban": "indurar",
    "indurabas": "indurar",
    "induraciones": "induración",
    "indurad": "indurar",
    "indurada": "indurar",
    "induradas": "indurar",
    "indurado": "indurar",
    "indurados": "indurar",
    "induramos": "indurar",
    "induran": "indurar",
    "indurando": "indurar",
    "indurara": "indurar",
    "indurarais": "indurar",
    "induraran": "indurar",
    "induraras": "indurar",
    "indurare": "indurar",
    "indurareis": "indurar",
    "induraremos": "indurar",
    "induraren": "indurar",
    "indurares": "indurar",
    "induraron": "indurar",
    "indurará": "indurar",
    "indurarán": "indurar",
    "indurarás": "indurar",
    "induraré": "indurar",
    "induraréis": "indurar",
    "induraría": "indurar",
    "induraríais": "indurar",
    "induraríamos": "indurar",
    "indurarían": "indurar",
    "indurarías": "indurar",
    "induras": "indurar",
    "indurase": "indurar",
    "induraseis": "indurar",
    "indurasen": "indurar",
    "indurases": "indurar",
    "induraste": "indurar",
    "indurasteis": "indurar",
    "indure": "indurar",
    "induremos": "indurar",
    "induren": "indurar",
    "indures": "indurar",
    "induro": "indurar",
    "indurábamos": "indurar",
    "induráis": "indurar",
    "induráramos": "indurar",
    "induráremos": "indurar",
    "indurásemos": "indurar",
    "induré": "indurar",
    "induréis": "indurar",
    "induró": "indurar",
    "industria": "industriar",
    "industriaba": "industriar",
    "industriabais": "industriar",
    "industriaban": "industriar",
    "industriabas": "industriar",
    "industriad": "industriar",
    "industriada": "industriar",
    "industriadas": "industriar",
    "industriado": "industriar",
    "industriados": "industriar",
    "industriales": "industrial",
    "industrialice": "industrializar",
    "industrialicemos": "industrializar",
    "industrialicen": "industrializar",
    "industrialices": "industrializar",
    "industrialicé": "industrializar",
    "industrialicéis": "industrializar",
    "industrialismos": "industrialismo",
    "industrialistas": "industrialista",
    "industrializa": "industrializar",
    "industrializaba": "industrializar",
    "industrializabais": "industrializar",
    "industrializaban": "industrializar",
    "industrializabas": "industrializar",
    "industrializaciones": "industrialización",
    "industrializad": "industrializar",
    "industrializada": "industrializar",
    "industrializadas": "industrializar",
    "industrializado": "industrializar",
    "industrializados": "industrializar",
    "industrializamos": "industrializar",
    "industrializan": "industrializar",
    "industrializando": "industrializar",
    "industrializara": "industrializar",
    "industrializarais": "industrializar",
    "industrializaran": "industrializar",
    "industrializaras": "industrializar",
    "industrializare": "industrializar",
    "industrializareis": "industrializar",
    "industrializaremos": "industrializar",
    "industrializaren": "industrializar",
    "industrializares": "industrializar",
    "industrializaron": "industrializar",
    "industrializará": "industrializar",
    "industrializarán": "industrializar",
    "industrializarás": "industrializar",
    "industrializaré": "industrializar",
    "industrializaréis": "industrializar",
    "industrializaría": "industrializar",
    "industrializaríais": "industrializar",
    "industrializaríamos": "industrializar",
    "industrializarían": "industrializar",
    "industrializarías": "industrializar",
    "industrializas": "industrializar",
    "industrializase": "industrializar",
    "industrializaseis": "industrializar",
    "industrializasen": "industrializar",
    "industrializases": "industrializar",
    "industrializaste": "industrializar",
    "industrializasteis": "industrializar",
    "industrializo": "industrializar",
    "industrializábamos": "industrializar",
    "industrializáis": "industrializar",
    "industrializáramos": "industrializar",
    "industrializáremos": "industrializar",
    "industrializásemos": "industrializar",
    "industrializó": "industrializar",
    "industriamos": "industriar",
    "industrian": "industriar",
    "industriando": "industriar",
    "industriara": "industriar",
    "industriarais": "industriar",
    "industriaran": "industriar",
    "industriaras": "industriar",
    "industriare": "industriar",
    "industriareis": "industriar",
    "industriaremos": "industriar",
    "industriaren": "industriar",
    "industriares": "industriar",
    "industriaron": "industriar",
    "industriará": "industriar",
    "industriarán": "industriar",
    "industriarás": "industriar",
    "industriaré": "industriar",
    "industriaréis": "industriar",
    "industriaría": "industriar",
    "industriaríais": "industriar",
    "industriaríamos": "industriar",
    "industriarían": "industriar",
    "industriarías": "industriar",
    "industrias": "industriar",
    "industriase": "industriar",
    "industriaseis": "industriar",
    "industriasen": "industriar",
    "industriases": "industriar",
    "industriaste": "industriar",
    "industriasteis": "industriar",
    "industrie": "industriar",
    "industriemos": "industriar",
    "industrien": "industriar",
    "industries": "industriar",
    "industrio": "industriar",
    "industriosa": "industrioso",
    "industriosas": "industrioso",
    "industriosos": "industrioso",
    "industriábamos": "industriar",
    "industriáis": "industriar",
    "industriáramos": "industriar",
    "industriáremos": "industriar",
    "industriásemos": "industriar",
    "industrié": "industriar",
    "industriéis": "industriar",
    "industrió": "industriar",
    "induzca": "inducir",
    "induzcamos": "inducir",
    "induzcan": "inducir",
    "induzcas": "inducir",
    "induzco": "inducir",
    "induzcáis": "inducir",
    "indígenas": "indígena",
    "indígenos": "indígeno",
    "indóciles": "indócil",
    "indólica": "indólico",
    "indólicas": "indólico",
    "indólicos": "indólico",
    "indómita": "indómito",
    "indómitas": "indómito",
    "indómitos": "indómito",
    "inecuaciones": "inecuación",
    "ineducaciones": "ineducación",
    "ineducada": "ineducado",
    "ineducadas": "ineducado",
    "ineducados": "ineducado",
    "inefabilidades": "inefabilidad",
    "inefables": "inefable",
    "inefectiva": "inefectivo",
    "inefectivas": "inefectivo",
    "inefectivos": "inefectivo",
    "ineficaces": "ineficaz",
    "ineficacias": "ineficacia",
    "ineficazes": "ineficaz",
    "ineficiencias": "ineficiencia",
    "ineficientes": "ineficiente",
    "inelasticidades": "inelasticidad",
    "inelegancias": "inelegancia",
    "inelegantes": "inelegante",
    "inelegibilidades": "inelegibilidad",
    "inelegibles": "inelegible",
    "ineluctables": "ineluctable",
    "ineludibles": "ineludible",
    "inelástica": "inelástico",
    "inelásticas": "inelástico",
    "inelásticos": "inelástico",
    "inembargabilidades": "inembargabilidad",
    "inembargables": "inembargable",
    "inenarrables": "inenarrable",
    "inepcias": "inepcia",
    "inepta": "inepto",
    "ineptas": "inepto",
    "ineptitudes": "ineptitud",
    "ineptos": "inepto",
    "inequívoca": "inequívoco",
    "inequívocas": "inequívoco",
    "inequívocos": "inequívoco",
    "inerciales": "inercial",
    "inercias": "inercia",
    "inermes": "inerme",
    "inerrantes": "inerrante",
    "inertes": "inerte",
    "inertizaciones": "inertización",
    "inerva": "inervar",
    "inervaba": "inervar",
    "inervabais": "inervar",
    "inervaban": "inervar",
    "inervabas": "inervar",
    "inervaciones": "inervación",
    "inervad": "inervar",
    "inervada": "inervar",
    "inervadas": "inervar",
    "inervado": "inervar",
    "inervadora": "inervador",
    "inervadoras": "inervador",
    "inervadores": "inervador",
    "inervados": "inervar",
    "inervamos": "inervar",
    "inervan": "inervar",
    "inervando": "inervar",
    "inervara": "inervar",
    "inervarais": "inervar",
    "inervaran": "inervar",
    "inervaras": "inervar",
    "inervare": "inervar",
    "inervareis": "inervar",
    "inervaremos": "inervar",
    "inervaren": "inervar",
    "inervares": "inervar",
    "inervaron": "inervar",
    "inervará": "inervar",
    "inervarán": "inervar",
    "inervarás": "inervar",
    "inervaré": "inervar",
    "inervaréis": "inervar",
    "inervaría": "inervar",
    "inervaríais": "inervar",
    "inervaríamos": "inervar",
    "inervarían": "inervar",
    "inervarías": "inervar",
    "inervas": "inervar",
    "inervase": "inervar",
    "inervaseis": "inervar",
    "inervasen": "inervar",
    "inervases": "inervar",
    "inervaste": "inervar",
    "inervasteis": "inervar",
    "inerve": "inervar",
    "inervemos": "inervar",
    "inerven": "inervar",
    "inerves": "inervar",
    "inervo": "inervar",
    "inervábamos": "inervar",
    "inerváis": "inervar",
    "inerváramos": "inervar",
    "inerváremos": "inervar",
    "inervásemos": "inervar",
    "inervé": "inervar",
    "inervéis": "inervar",
    "inervó": "inervar",
    "inescrupulosa": "inescrupuloso",
    "inescrupulosas": "inescrupuloso",
    "inescrupulosidades": "inescrupulosidad",
    "inescrupulosos": "inescrupuloso",
    "inescrutabilidades": "inescrutabilidad",
    "inescrutables": "inescrutable",
    "inescudriñables": "inescudriñable",
    "inesiva": "inesivo",
    "inesivas": "inesivo",
    "inesivos": "inesivo",
    "inesperada": "inesperado",
    "inesperadas": "inesperado",
    "inesperados": "inesperado",
    "inestabilidades": "inestabilidad",
    "inestables": "inestable",
    "inestimabilidades": "inestimabilidad",
    "inestimables": "inestimable",
    "inestimada": "inestimado",
    "inestimadas": "inestimado",
    "inestimados": "inestimado",
    "inevaluables": "inevaluable",
    "inevitabilidades": "inevitabilidad",
    "inevitables": "inevitable",
    "inexacta": "inexacto",
    "inexactas": "inexacto",
    "inexactitudes": "inexactitud",
    "inexactos": "inexacto",
    "inexcusables": "inexcusable",
    "inexhausta": "inexhausto",
    "inexhaustas": "inexhausto",
    "inexhaustos": "inexhausto",
    "inexistencias": "inexistencia",
    "inexistentes": "inexistente",
    "inexorabilidades": "inexorabilidad",
    "inexorables": "inexorable",
    "inexperiencias": "inexperiencia",
    "inexperta": "inexperto",
    "inexpertas": "inexperto",
    "inexpertos": "inexperto",
    "inexplicables": "inexplicable",
    "inexplicada": "inexplicado",
    "inexplicadas": "inexplicado",
    "inexplicados": "inexplicado",
    "inexplorada": "inexplorado",
    "inexploradas": "inexplorado",
    "inexplorados": "inexplorado",
    "inexplotada": "inexplotado",
    "inexplotadas": "inexplotado",
    "inexplotados": "inexplotado",
    "inexpresables": "inexpresable",
    "inexpresiva": "inexpresivo",
    "inexpresivas": "inexpresivo",
    "inexpresivos": "inexpresivo",
    "inexpugnables": "inexpugnable",
    "inextensa": "inextenso",
    "inextensas": "inextenso",
    "inextensibles": "inextensible",
    "inextensos": "inextenso",
    "inextinguibles": "inextinguible",
    "inextricables": "inextricable",
    "infacunda": "infacundo",
    "infacundas": "infacundo",
    "infacundos": "infacundo",
    "infalibilidades": "infalibilidad",
    "infalibles": "infalible",
    "infalsificabilidades": "infalsificabilidad",
    "infalsificables": "infalsificable",
    "infama": "infamar",
    "infamaba": "infamar",
    "infamabais": "infamar",
    "infamaban": "infamar",
    "infamabas": "infamar",
    "infamaciones": "infamación",
    "infamad": "infamar",
    "infamada": "infamar",
    "infamadas": "infamar",
    "infamado": "infamar",
    "infamadora": "infamador",
    "infamadoras": "infamador",
    "infamadores": "infamador",
    "infamados": "infamar",
    "infamamos": "infamar",
    "infaman": "infamar",
    "infamando": "infamar",
    "infamantes": "infamante",
    "infamara": "infamar",
    "infamarais": "infamar",
    "infamaran": "infamar",
    "infamaras": "infamar",
    "infamare": "infamar",
    "infamareis": "infamar",
    "infamaremos": "infamar",
    "infamaren": "infamar",
    "infamares": "infamar",
    "infamaron": "infamar",
    "infamará": "infamar",
    "infamarán": "infamar",
    "infamarás": "infamar",
    "infamaré": "infamar",
    "infamaréis": "infamar",
    "infamaría": "infamar",
    "infamaríais": "infamar",
    "infamaríamos": "infamar",
    "infamarían": "infamar",
    "infamarías": "infamar",
    "infamas": "infamar",
    "infamase": "infamar",
    "infamaseis": "infamar",
    "infamasen": "infamar",
    "infamases": "infamar",
    "infamaste": "infamar",
    "infamasteis": "infamar",
    "infamativa": "infamativo",
    "infamativas": "infamativo",
    "infamativos": "infamativo",
    "infamatoria": "infamatorio",
    "infamatorias": "infamatorio",
    "infamatorios": "infamatorio",
    "infame": "infamar",
    "infamemos": "infamar",
    "infamen": "infamar",
    "infames": "infame",
    "infamias": "infamia",
    "infamo": "infamar",
    "infamábamos": "infamar",
    "infamáis": "infamar",
    "infamáramos": "infamar",
    "infamáremos": "infamar",
    "infamásemos": "infamar",
    "infamé": "infamar",
    "infaméis": "infamar",
    "infamó": "infamar",
    "infancias": "infancia",
    "infanda": "infando",
    "infandas": "infando",
    "infandos": "infando",
    "infanta": "infante",
    "infantados": "infantado",
    "infantas": "infante",
    "infantazgos": "infantazgo",
    "infanterías": "infantería",
    "infantes": "infante",
    "infanticidas": "infanticida",
    "infanticidios": "infanticidio",
    "infantiles": "infantil",
    "infantilismos": "infantilismo",
    "infantiloides": "infantiloide",
    "infantinas": "infantina",
    "infanzona": "infanzón",
    "infanzonada": "infanzonado",
    "infanzonadas": "infanzonado",
    "infanzonados": "infanzonado",
    "infanzonas": "infanzón",
    "infanzonazgos": "infanzonazgo",
    "infanzones": "infanzón",
    "infanzonías": "infanzonía",
    "infarta": "infartar",
    "infartaba": "infartar",
    "infartabais": "infartar",
    "infartaban": "infartar",
    "infartabas": "infartar",
    "infartad": "infartar",
    "infartada": "infartar",
    "infartadas": "infartar",
    "infartado": "infartar",
    "infartados": "infartar",
    "infartamos": "infartar",
    "infartan": "infartar",
    "infartando": "infartar",
    "infartara": "infartar",
    "infartarais": "infartar",
    "infartaran": "infartar",
    "infartaras": "infartar",
    "infartare": "infartar",
    "infartareis": "infartar",
    "infartaremos": "infartar",
    "infartaren": "infartar",
    "infartares": "infartar",
    "infartaron": "infartar",
    "infartará": "infartar",
    "infartarán": "infartar",
    "infartarás": "infartar",
    "infartaré": "infartar",
    "infartaréis": "infartar",
    "infartaría": "infartar",
    "infartaríais": "infartar",
    "infartaríamos": "infartar",
    "infartarían": "infartar",
    "infartarías": "infartar",
    "infartas": "infartar",
    "infartase": "infartar",
    "infartaseis": "infartar",
    "infartasen": "infartar",
    "infartases": "infartar",
    "infartaste": "infartar",
    "infartasteis": "infartar",
    "infarte": "infartar",
    "infartemos": "infartar",
    "infarten": "infartar",
    "infartes": "infartar",
    "infarto": "infartar",
    "infartos": "infarto",
    "infartábamos": "infartar",
    "infartáis": "infartar",
    "infartáramos": "infartar",
    "infartáremos": "infartar",
    "infartásemos": "infartar",
    "infarté": "infartar",
    "infartéis": "infartar",
    "infartó": "infartar",
    "infatigables": "infatigable",
    "infatuaba": "infatuar",
    "infatuabais": "infatuar",
    "infatuaban": "infatuar",
    "infatuabas": "infatuar",
    "infatuaciones": "infatuación",
    "infatuad": "infatuar",
    "infatuada": "infatuar",
    "infatuadas": "infatuar",
    "infatuado": "infatuar",
    "infatuados": "infatuar",
    "infatuamos": "infatuar",
    "infatuando": "infatuar",
    "infatuara": "infatuar",
    "infatuarais": "infatuar",
    "infatuaran": "infatuar",
    "infatuaras": "infatuar",
    "infatuare": "infatuar",
    "infatuareis": "infatuar",
    "infatuaremos": "infatuar",
    "infatuaren": "infatuar",
    "infatuares": "infatuar",
    "infatuaron": "infatuar",
    "infatuará": "infatuar",
    "infatuarán": "infatuar",
    "infatuarás": "infatuar",
    "infatuaré": "infatuar",
    "infatuaréis": "infatuar",
    "infatuaría": "infatuar",
    "infatuaríais": "infatuar",
    "infatuaríamos": "infatuar",
    "infatuarían": "infatuar",
    "infatuarías": "infatuar",
    "infatuase": "infatuar",
    "infatuaseis": "infatuar",
    "infatuasen": "infatuar",
    "infatuases": "infatuar",
    "infatuaste": "infatuar",
    "infatuasteis": "infatuar",
    "infatuemos": "infatuar",
    "infatuábamos": "infatuar",
    "infatuáis": "infatuar",
    "infatuáramos": "infatuar",
    "infatuáremos": "infatuar",
    "infatuásemos": "infatuar",
    "infatué": "infatuar",
    "infatuéis": "infatuar",
    "infatuó": "infatuar",
    "infatúa": "infatuar",
    "infatúan": "infatuar",
    "infatúas": "infatuar",
    "infatúe": "infatuar",
    "infatúen": "infatuar",
    "infatúes": "infatuar",
    "infatúo": "infatuar",
    "infaunas": "infauna",
    "infausta": "infausto",
    "infaustas": "infausto",
    "infaustos": "infausto",
    "infebriles": "infebril",
    "infecciones": "infección",
    "infecciosa": "infeccioso",
    "infecciosas": "infeccioso",
    "infecciosos": "infeccioso",
    "infecta": "infecto",
    "infectaba": "infectar",
    "infectabais": "infectar",
    "infectaban": "infectar",
    "infectabas": "infectar",
    "infectad": "infectar",
    "infectada": "infectar",
    "infectadas": "infectar",
    "infectado": "infectar",
    "infectados": "infectar",
    "infectamos": "infectar",
    "infectan": "infectar",
    "infectando": "infectar",
    "infectara": "infectar",
    "infectarais": "infectar",
    "infectaran": "infectar",
    "infectaras": "infectar",
    "infectare": "infectar",
    "infectareis": "infectar",
    "infectaremos": "infectar",
    "infectaren": "infectar",
    "infectares": "infectar",
    "infectaron": "infectar",
    "infectará": "infectar",
    "infectarán": "infectar",
    "infectarás": "infectar",
    "infectaré": "infectar",
    "infectaréis": "infectar",
    "infectaría": "infectar",
    "infectaríais": "infectar",
    "infectaríamos": "infectar",
    "infectarían": "infectar",
    "infectarías": "infectar",
    "infectas": "infecto",
    "infectase": "infectar",
    "infectaseis": "infectar",
    "infectasen": "infectar",
    "infectases": "infectar",
    "infectaste": "infectar",
    "infectasteis": "infectar",
    "infecte": "infectar",
    "infectemos": "infectar",
    "infecten": "infectar",
    "infectes": "infectar",
    "infectiva": "infectivo",
    "infectivas": "infectivo",
    "infectivos": "infectivo",
    "infecto": "infectar",
    "infectos": "infecto",
    "infectábamos": "infectar",
    "infectáis": "infectar",
    "infectáramos": "infectar",
    "infectáremos": "infectar",
    "infectásemos": "infectar",
    "infecté": "infectar",
    "infectéis": "infectar",
    "infectó": "infectar",
    "infecunda": "infecundo",
    "infecundas": "infecundo",
    "infecundidades": "infecundidad",
    "infecundos": "infecundo",
    "infelice": "infeliz",
    "infelices": "infeliz",
    "infelicidades": "infelicidad",
    "inferenciales": "inferencial",
    "inferencias": "inferencia",
    "inferid": "inferir",
    "inferida": "inferir",
    "inferidas": "inferir",
    "inferido": "inferir",
    "inferidos": "inferir",
    "inferimos": "inferir",
    "inferiores": "inferior",
    "inferioridades": "inferioridad",
    "inferiremos": "inferir",
    "inferirá": "inferir",
    "inferirán": "inferir",
    "inferirás": "inferir",
    "inferiré": "inferir",
    "inferiréis": "inferir",
    "inferiría": "inferir",
    "inferiríais": "inferir",
    "inferiríamos": "inferir",
    "inferirían": "inferir",
    "inferirías": "inferir",
    "inferiste": "inferir",
    "inferisteis": "inferir",
    "inferna": "inferno",
    "infernales": "infernal",
    "infernas": "inferno",
    "infernillos": "infernillo",
    "infernos": "inferno",
    "infernáculos": "infernáculo",
    "infertilidades": "infertilidad",
    "infertilidads": "infertilidad",
    "inferí": "inferir",
    "infería": "inferir",
    "inferíais": "inferir",
    "inferíamos": "inferir",
    "inferían": "inferir",
    "inferías": "inferir",
    "inferís": "inferir",
    "infesta": "infesto",
    "infestaba": "infestar",
    "infestabais": "infestar",
    "infestaban": "infestar",
    "infestabas": "infestar",
    "infestaciones": "infestación",
    "infestad": "infestar",
    "infestada": "infestar",
    "infestadas": "infestar",
    "infestado": "infestar",
    "infestados": "infestar",
    "infestamos": "infestar",
    "infestan": "infestar",
    "infestando": "infestar",
    "infestara": "infestar",
    "infestarais": "infestar",
    "infestaran": "infestar",
    "infestaras": "infestar",
    "infestare": "infestar",
    "infestareis": "infestar",
    "infestaremos": "infestar",
    "infestaren": "infestar",
    "infestares": "infestar",
    "infestaron": "infestar",
    "infestará": "infestar",
    "infestarán": "infestar",
    "infestarás": "infestar",
    "infestaré": "infestar",
    "infestaréis": "infestar",
    "infestaría": "infestar",
    "infestaríais": "infestar",
    "infestaríamos": "infestar",
    "infestarían": "infestar",
    "infestarías": "infestar",
    "infestas": "infesto",
    "infestase": "infestar",
    "infestaseis": "infestar",
    "infestasen": "infestar",
    "infestases": "infestar",
    "infestaste": "infestar",
    "infestasteis": "infestar",
    "infeste": "infestar",
    "infestemos": "infestar",
    "infesten": "infestar",
    "infestes": "infestar",
    "infesto": "infestar",
    "infestos": "infesto",
    "infestábamos": "infestar",
    "infestáis": "infestar",
    "infestáramos": "infestar",
    "infestáremos": "infestar",
    "infestásemos": "infestar",
    "infesté": "infestar",
    "infestéis": "infestar",
    "infestó": "infestar",
    "infibula": "infibular",
    "infibulaba": "infibular",
    "infibulabais": "infibular",
    "infibulaban": "infibular",
    "infibulabas": "infibular",
    "infibulaciones": "infibulación",
    "infibulad": "infibular",
    "infibulada": "infibular",
    "infibuladas": "infibular",
    "infibulado": "infibular",
    "infibulados": "infibular",
    "infibulamos": "infibular",
    "infibulan": "infibular",
    "infibulando": "infibular",
    "infibulara": "infibular",
    "infibularais": "infibular",
    "infibularan": "infibular",
    "infibularas": "infibular",
    "infibulare": "infibular",
    "infibulareis": "infibular",
    "infibularemos": "infibular",
    "infibularen": "infibular",
    "infibulares": "infibular",
    "infibularon": "infibular",
    "infibulará": "infibular",
    "infibularán": "infibular",
    "infibularás": "infibular",
    "infibularé": "infibular",
    "infibularéis": "infibular",
    "infibularía": "infibular",
    "infibularíais": "infibular",
    "infibularíamos": "infibular",
    "infibularían": "infibular",
    "infibularías": "infibular",
    "infibulas": "infibular",
    "infibulase": "infibular",
    "infibulaseis": "infibular",
    "infibulasen": "infibular",
    "infibulases": "infibular",
    "infibulaste": "infibular",
    "infibulasteis": "infibular",
    "infibule": "infibular",
    "infibulemos": "infibular",
    "infibulen": "infibular",
    "infibules": "infibular",
    "infibulo": "infibular",
    "infibulábamos": "infibular",
    "infibuláis": "infibular",
    "infibuláramos": "infibular",
    "infibuláremos": "infibular",
    "infibulásemos": "infibular",
    "infibulé": "infibular",
    "infibuléis": "infibular",
    "infibuló": "infibular",
    "inficiona": "inficionar",
    "inficionaba": "inficionar",
    "inficionabais": "inficionar",
    "inficionaban": "inficionar",
    "inficionabas": "inficionar",
    "inficionad": "inficionar",
    "inficionada": "inficionar",
    "inficionadas": "inficionar",
    "inficionado": "inficionar",
    "inficionados": "inficionar",
    "inficionamos": "inficionar",
    "inficionan": "inficionar",
    "inficionando": "inficionar",
    "inficionara": "inficionar",
    "inficionarais": "inficionar",
    "inficionaran": "inficionar",
    "inficionaras": "inficionar",
    "inficionare": "inficionar",
    "inficionareis": "inficionar",
    "inficionaremos": "inficionar",
    "inficionaren": "inficionar",
    "inficionares": "inficionar",
    "inficionaron": "inficionar",
    "inficionará": "inficionar",
    "inficionarán": "inficionar",
    "inficionarás": "inficionar",
    "inficionaré": "inficionar",
    "inficionaréis": "inficionar",
    "inficionaría": "inficionar",
    "inficionaríais": "inficionar",
    "inficionaríamos": "inficionar",
    "inficionarían": "inficionar",
    "inficionarías": "inficionar",
    "inficionas": "inficionar",
    "inficionase": "inficionar",
    "inficionaseis": "inficionar",
    "inficionasen": "inficionar",
    "inficionases": "inficionar",
    "inficionaste": "inficionar",
    "inficionasteis": "inficionar",
    "inficione": "inficionar",
    "inficionemos": "inficionar",
    "inficionen": "inficionar",
    "inficiones": "inficionar",
    "inficiono": "inficionar",
    "inficionábamos": "inficionar",
    "inficionáis": "inficionar",
    "inficionáramos": "inficionar",
    "inficionáremos": "inficionar",
    "inficionásemos": "inficionar",
    "inficioné": "inficionar",
    "inficionéis": "inficionar",
    "inficionó": "inficionar",
    "infidelidades": "infidelidad",
    "infidencias": "infidencia",
    "infidentes": "infidente",
    "infieles": "infiel",
    "infiera": "inferir",
    "infieran": "inferir",
    "infieras": "inferir",
    "infiere": "inferir",
    "infieren": "inferir",
    "infieres": "inferir",
    "infiernillos": "infiernillo",
    "infiernos": "infierno",
    "infiero": "inferir",
    "infigurables": "infigurable",
    "infijos": "infijo",
    "infiltra": "infiltrar",
    "infiltraba": "infiltrar",
    "infiltrabais": "infiltrar",
    "infiltraban": "infiltrar",
    "infiltrabas": "infiltrar",
    "infiltraciones": "infiltración",
    "infiltrad": "infiltrar",
    "infiltrada": "infiltrar",
    "infiltradas": "infiltrar",
    "infiltrado": "infiltrar",
    "infiltrados": "infiltrar",
    "infiltramos": "infiltrar",
    "infiltran": "infiltrar",
    "infiltrando": "infiltrar",
    "infiltrara": "infiltrar",
    "infiltrarais": "infiltrar",
    "infiltraran": "infiltrar",
    "infiltraras": "infiltrar",
    "infiltrare": "infiltrar",
    "infiltrareis": "infiltrar",
    "infiltraremos": "infiltrar",
    "infiltraren": "infiltrar",
    "infiltrares": "infiltrar",
    "infiltraron": "infiltrar",
    "infiltrará": "infiltrar",
    "infiltrarán": "infiltrar",
    "infiltrarás": "infiltrar",
    "infiltraré": "infiltrar",
    "infiltraréis": "infiltrar",
    "infiltraría": "infiltrar",
    "infiltraríais": "infiltrar",
    "infiltraríamos": "infiltrar",
    "infiltrarían": "infiltrar",
    "infiltrarías": "infiltrar",
    "infiltras": "infiltrar",
    "infiltrase": "infiltrar",
    "infiltraseis": "infiltrar",
    "infiltrasen": "infiltrar",
    "infiltrases": "infiltrar",
    "infiltraste": "infiltrar",
    "infiltrasteis": "infiltrar",
    "infiltre": "infiltrar",
    "infiltremos": "infiltrar",
    "infiltren": "infiltrar",
    "infiltres": "infiltrar",
    "infiltro": "infiltrar",
    "infiltrábamos": "infiltrar",
    "infiltráis": "infiltrar",
    "infiltráramos": "infiltrar",
    "infiltráremos": "infiltrar",
    "infiltrásemos": "infiltrar",
    "infiltré": "infiltrar",
    "infiltréis": "infiltrar",
    "infiltró": "infiltrar",
    "infiltrómetros": "infiltrómetro",
    "infinidades": "infinidad",
    "infinita": "infinito",
    "infinitas": "infinito",
    "infinitesimales": "infinitesimal",
    "infinitivos": "infinitivo",
    "infinitos": "infinito",
    "infinitudes": "infinitud",
    "infinitésima": "infinitésimo",
    "infinitésimas": "infinitésimo",
    "infinitésimos": "infinitésimo",
    "infiramos": "inferir",
    "infiriendo": "inferir",
    "infiriera": "inferir",
    "infirierais": "inferir",
    "infirieran": "inferir",
    "infirieras": "inferir",
    "infiriere": "inferir",
    "infiriereis": "inferir",
    "infirieren": "inferir",
    "infirieres": "inferir",
    "infirieron": "inferir",
    "infiriese": "inferir",
    "infirieseis": "inferir",
    "infiriesen": "inferir",
    "infirieses": "inferir",
    "infiriéramos": "inferir",
    "infiriéremos": "inferir",
    "infiriésemos": "inferir",
    "infirió": "inferir",
    "infirma": "infirmar",
    "infirmaba": "infirmar",
    "infirmabais": "infirmar",
    "infirmaban": "infirmar",
    "infirmabas": "infirmar",
    "infirmad": "infirmar",
    "infirmada": "infirmar",
    "infirmadas": "infirmar",
    "infirmado": "infirmar",
    "infirmados": "infirmar",
    "infirmamos": "infirmar",
    "infirman": "infirmar",
    "infirmando": "infirmar",
    "infirmara": "infirmar",
    "infirmarais": "infirmar",
    "infirmaran": "infirmar",
    "infirmaras": "infirmar",
    "infirmare": "infirmar",
    "infirmareis": "infirmar",
    "infirmaremos": "infirmar",
    "infirmaren": "infirmar",
    "infirmares": "infirmar",
    "infirmaron": "infirmar",
    "infirmará": "infirmar",
    "infirmarán": "infirmar",
    "infirmarás": "infirmar",
    "infirmaré": "infirmar",
    "infirmaréis": "infirmar",
    "infirmaría": "infirmar",
    "infirmaríais": "infirmar",
    "infirmaríamos": "infirmar",
    "infirmarían": "infirmar",
    "infirmarías": "infirmar",
    "infirmas": "infirmar",
    "infirmase": "infirmar",
    "infirmaseis": "infirmar",
    "infirmasen": "infirmar",
    "infirmases": "infirmar",
    "infirmaste": "infirmar",
    "infirmasteis": "infirmar",
    "infirme": "infirmar",
    "infirmemos": "infirmar",
    "infirmen": "infirmar",
    "infirmes": "infirmar",
    "infirmo": "infirmar",
    "infirmábamos": "infirmar",
    "infirmáis": "infirmar",
    "infirmáramos": "infirmar",
    "infirmáremos": "infirmar",
    "infirmásemos": "infirmar",
    "infirmé": "infirmar",
    "infirméis": "infirmar",
    "infirmó": "infirmar",
    "infiráis": "inferir",
    "infla": "inflar",
    "inflaba": "inflar",
    "inflabais": "inflar",
    "inflaban": "inflar",
    "inflabas": "inflar",
    "inflables": "inflable",
    "inflacionaria": "inflacionario",
    "inflacionarias": "inflacionario",
    "inflacionarios": "inflacionario",
    "inflaciones": "inflación",
    "inflacionismos": "inflacionismo",
    "inflacionistas": "inflacionista",
    "inflad": "inflar",
    "inflada": "inflar",
    "infladas": "inflar",
    "inflado": "inflar",
    "inflados": "inflar",
    "inflama": "inflamar",
    "inflamaba": "inflamar",
    "inflamabais": "inflamar",
    "inflamaban": "inflamar",
    "inflamabas": "inflamar",
    "inflamabilidades": "inflamabilidad",
    "inflamables": "inflamable",
    "inflamaciones": "inflamación",
    "inflamad": "inflamar",
    "inflamada": "inflamar",
    "inflamadas": "inflamar",
    "inflamado": "inflamar",
    "inflamadora": "inflamador",
    "inflamadoras": "inflamador",
    "inflamadores": "inflamador",
    "inflamados": "inflamar",
    "inflamamos": "inflamar",
    "inflaman": "inflamar",
    "inflamando": "inflamar",
    "inflamara": "inflamar",
    "inflamarais": "inflamar",
    "inflamaran": "inflamar",
    "inflamaras": "inflamar",
    "inflamare": "inflamar",
    "inflamareis": "inflamar",
    "inflamaremos": "inflamar",
    "inflamaren": "inflamar",
    "inflamares": "inflamar",
    "inflamaron": "inflamar",
    "inflamará": "inflamar",
    "inflamarán": "inflamar",
    "inflamarás": "inflamar",
    "inflamaré": "inflamar",
    "inflamaréis": "inflamar",
    "inflamaría": "inflamar",
    "inflamaríais": "inflamar",
    "inflamaríamos": "inflamar",
    "inflamarían": "inflamar",
    "inflamarías": "inflamar",
    "inflamas": "inflamar",
    "inflamase": "inflamar",
    "inflamaseis": "inflamar",
    "inflamasen": "inflamar",
    "inflamases": "inflamar",
    "inflamaste": "inflamar",
    "inflamasteis": "inflamar",
    "inflamatoria": "inflamatorio",
    "inflamatorias": "inflamatorio",
    "inflamatorios": "inflamatorio",
    "inflame": "inflamar",
    "inflamemos": "inflamar",
    "inflamen": "inflamar",
    "inflames": "inflamar",
    "inflamientos": "inflamiento",
    "inflamo": "inflamar",
    "inflamos": "inflar",
    "inflamábamos": "inflamar",
    "inflamáis": "inflamar",
    "inflamáramos": "inflamar",
    "inflamáremos": "inflamar",
    "inflamásemos": "inflamar",
    "inflamé": "inflamar",
    "inflaméis": "inflamar",
    "inflamó": "inflamar",
    "inflan": "inflar",
    "inflando": "inflar",
    "inflara": "inflar",
    "inflarais": "inflar",
    "inflaran": "inflar",
    "inflaras": "inflar",
    "inflare": "inflar",
    "inflareis": "inflar",
    "inflaremos": "inflar",
    "inflaren": "inflar",
    "inflares": "inflar",
    "inflaron": "inflar",
    "inflará": "inflar",
    "inflarán": "inflar",
    "inflarás": "inflar",
    "inflaré": "inflar",
    "inflaréis": "inflar",
    "inflaría": "inflar",
    "inflaríais": "inflar",
    "inflaríamos": "inflar",
    "inflarían": "inflar",
    "inflarías": "inflar",
    "inflas": "inflar",
    "inflase": "inflar",
    "inflaseis": "inflar",
    "inflasen": "inflar",
    "inflases": "inflar",
    "inflaste": "inflar",
    "inflasteis": "inflar",
    "inflativa": "inflativo",
    "inflativas": "inflativo",
    "inflativos": "inflativo",
    "infle": "inflar",
    "inflemos": "inflar",
    "inflen": "inflar",
    "infles": "inflar",
    "inflexibilidades": "inflexibilidad",
    "inflexibles": "inflexible",
    "inflexionales": "inflexional",
    "inflexiones": "inflexión",
    "inflige": "infligir",
    "infligen": "infligir",
    "infliges": "infligir",
    "infligid": "infligir",
    "infligida": "infligir",
    "infligidas": "infligir",
    "infligido": "infligir",
    "infligidos": "infligir",
    "infligiendo": "infligir",
    "infligiera": "infligir",
    "infligierais": "infligir",
    "infligieran": "infligir",
    "infligieras": "infligir",
    "infligiere": "infligir",
    "infligiereis": "infligir",
    "infligieren": "infligir",
    "infligieres": "infligir",
    "infligieron": "infligir",
    "infligiese": "infligir",
    "infligieseis": "infligir",
    "infligiesen": "infligir",
    "infligieses": "infligir",
    "infligimos": "infligir",
    "infligiremos": "infligir",
    "infligirá": "infligir",
    "infligirán": "infligir",
    "infligirás": "infligir",
    "infligiré": "infligir",
    "infligiréis": "infligir",
    "infligiría": "infligir",
    "infligiríais": "infligir",
    "infligiríamos": "infligir",
    "infligirían": "infligir",
    "infligirías": "infligir",
    "infligiste": "infligir",
    "infligisteis": "infligir",
    "infligiéramos": "infligir",
    "infligiéremos": "infligir",
    "infligiésemos": "infligir",
    "infligió": "infligir",
    "infligí": "infligir",
    "infligía": "infligir",
    "infligíais": "infligir",
    "infligíamos": "infligir",
    "infligían": "infligir",
    "infligías": "infligir",
    "infligís": "infligir",
    "inflija": "infligir",
    "inflijamos": "infligir",
    "inflijan": "infligir",
    "inflijas": "infligir",
    "inflijo": "infligir",
    "inflijáis": "infligir",
    "inflo": "inflar",
    "inflorescencias": "inflorescencia",
    "influencia": "influenciar",
    "influenciaba": "influenciar",
    "influenciabais": "influenciar",
    "influenciaban": "influenciar",
    "influenciabas": "influenciar",
    "influenciad": "influenciar",
    "influenciada": "influenciar",
    "influenciadas": "influenciar",
    "influenciado": "influenciar",
    "influenciados": "influenciar",
    "influenciamos": "influenciar",
    "influencian": "influenciar",
    "influenciando": "influenciar",
    "influenciara": "influenciar",
    "influenciarais": "influenciar",
    "influenciaran": "influenciar",
    "influenciaras": "influenciar",
    "influenciare": "influenciar",
    "influenciareis": "influenciar",
    "influenciaremos": "influenciar",
    "influenciaren": "influenciar",
    "influenciares": "influenciar",
    "influenciaron": "influenciar",
    "influenciará": "influenciar",
    "influenciarán": "influenciar",
    "influenciarás": "influenciar",
    "influenciaré": "influenciar",
    "influenciaréis": "influenciar",
    "influenciaría": "influenciar",
    "influenciaríais": "influenciar",
    "influenciaríamos": "influenciar",
    "influenciarían": "influenciar",
    "influenciarías": "influenciar",
    "influencias": "influenciar",
    "influenciase": "influenciar",
    "influenciaseis": "influenciar",
    "influenciasen": "influenciar",
    "influenciases": "influenciar",
    "influenciaste": "influenciar",
    "influenciasteis": "influenciar",
    "influencie": "influenciar",
    "influenciemos": "influenciar",
    "influencien": "influenciar",
    "influencies": "influenciar",
    "influencio": "influenciar",
    "influenciábamos": "influenciar",
    "influenciáis": "influenciar",
    "influenciáramos": "influenciar",
    "influenciáremos": "influenciar",
    "influenciásemos": "influenciar",
    "influencié": "influenciar",
    "influenciéis": "influenciar",
    "influenció": "influenciar",
    "influentes": "influente",
    "influenzas": "influenza",
    "influid": "influir",
    "influida": "influir",
    "influidas": "influir",
    "influido": "influir",
    "influidos": "influir",
    "influimos": "influir",
    "influiremos": "influir",
    "influirá": "influir",
    "influirán": "influir",
    "influirás": "influir",
    "influiré": "influir",
    "influiréis": "influir",
    "influiría": "influir",
    "influiríais": "influir",
    "influiríamos": "influir",
    "influirían": "influir",
    "influirías": "influir",
    "influiste": "influir",
    "influisteis": "influir",
    "influjos": "influjo",
    "influya": "influir",
    "influyamos": "influir",
    "influyan": "influir",
    "influyas": "influir",
    "influye": "influir",
    "influyen": "influir",
    "influyendo": "influir",
    "influyentes": "influyente",
    "influyera": "influir",
    "influyerais": "influir",
    "influyeran": "influir",
    "influyeras": "influir",
    "influyere": "influir",
    "influyereis": "influir",
    "influyeren": "influir",
    "influyeres": "influir",
    "influyeron": "influir",
    "influyes": "influir",
    "influyese": "influir",
    "influyeseis": "influir",
    "influyesen": "influir",
    "influyeses": "influir",
    "influyo": "influir",
    "influyáis": "influir",
    "influyéramos": "influir",
    "influyéremos": "influir",
    "influyésemos": "influir",
    "influyó": "influir",
    "influí": "influir",
    "influía": "influir",
    "influíais": "influir",
    "influíamos": "influir",
    "influían": "influir",
    "influías": "influir",
    "influís": "influir",
    "inflábamos": "inflar",
    "infláis": "inflar",
    "infláramos": "inflar",
    "infláremos": "inflar",
    "inflásemos": "inflar",
    "inflé": "inflar",
    "infléis": "inflar",
    "infló": "inflar",
    "infografías": "infografía",
    "infográfica": "infográfico",
    "infográficas": "infográfico",
    "infográficos": "infográfico",
    "infolios": "infolio",
    "inforciados": "inforciado",
    "informa": "informar",
    "informaba": "informar",
    "informabais": "informar",
    "informaban": "informar",
    "informabas": "informar",
    "informacionales": "informacional",
    "informaciones": "información",
    "informad": "informar",
    "informada": "informar",
    "informadas": "informar",
    "informado": "informar",
    "informadora": "informador",
    "informadoras": "informador",
    "informadores": "informador",
    "informados": "informar",
    "informales": "informal",
    "informalidades": "informalidad",
    "informalismos": "informalismo",
    "informalistas": "informalista",
    "informamos": "informar",
    "informan": "informar",
    "informando": "informar",
    "informantes": "informante",
    "informara": "informar",
    "informarais": "informar",
    "informaran": "informar",
    "informaras": "informar",
    "informare": "informar",
    "informareis": "informar",
    "informaremos": "informar",
    "informaren": "informar",
    "informares": "informar",
    "informaron": "informar",
    "informará": "informar",
    "informarán": "informar",
    "informarás": "informar",
    "informaré": "informar",
    "informaréis": "informar",
    "informaría": "informar",
    "informaríais": "informar",
    "informaríamos": "informar",
    "informarían": "informar",
    "informarías": "informar",
    "informas": "informar",
    "informase": "informar",
    "informaseis": "informar",
    "informasen": "informar",
    "informases": "informar",
    "informaste": "informar",
    "informasteis": "informar",
    "informatice": "informatizar",
    "informaticemos": "informatizar",
    "informaticen": "informatizar",
    "informatices": "informatizar",
    "informaticé": "informatizar",
    "informaticéis": "informatizar",
    "informativa": "informativo",
    "informativas": "informativo",
    "informativos": "informativo",
    "informatiza": "informatizar",
    "informatizaba": "informatizar",
    "informatizabais": "informatizar",
    "informatizaban": "informatizar",
    "informatizabas": "informatizar",
    "informatizaciones": "informatización",
    "informatizad": "informatizar",
    "informatizada": "informatizar",
    "informatizadas": "informatizar",
    "informatizado": "informatizar",
    "informatizados": "informatizar",
    "informatizamos": "informatizar",
    "informatizan": "informatizar",
    "informatizando": "informatizar",
    "informatizara": "informatizar",
    "informatizarais": "informatizar",
    "informatizaran": "informatizar",
    "informatizaras": "informatizar",
    "informatizare": "informatizar",
    "informatizareis": "informatizar",
    "informatizaremos": "informatizar",
    "informatizaren": "informatizar",
    "informatizares": "informatizar",
    "informatizaron": "informatizar",
    "informatizará": "informatizar",
    "informatizarán": "informatizar",
    "informatizarás": "informatizar",
    "informatizaré": "informatizar",
    "informatizaréis": "informatizar",
    "informatizaría": "informatizar",
    "informatizaríais": "informatizar",
    "informatizaríamos": "informatizar",
    "informatizarían": "informatizar",
    "informatizarías": "informatizar",
    "informatizas": "informatizar",
    "informatizase": "informatizar",
    "informatizaseis": "informatizar",
    "informatizasen": "informatizar",
    "informatizases": "informatizar",
    "informatizaste": "informatizar",
    "informatizasteis": "informatizar",
    "informatizo": "informatizar",
    "informatizábamos": "informatizar",
    "informatizáis": "informatizar",
    "informatizáramos": "informatizar",
    "informatizáremos": "informatizar",
    "informatizásemos": "informatizar",
    "informatizó": "informatizar",
    "informe": "informar",
    "informemos": "informar",
    "informen": "informar",
    "informes": "informe",
    "informidades": "informidad",
    "informo": "informar",
    "informologías": "informología",
    "informábamos": "informar",
    "informáis": "informar",
    "informáramos": "informar",
    "informáremos": "informar",
    "informásemos": "informar",
    "informática": "informático",
    "informáticas": "informático",
    "informáticos": "informático",
    "informé": "informar",
    "informéis": "informar",
    "informó": "informar",
    "infortunada": "infortunado",
    "infortunadas": "infortunado",
    "infortunados": "infortunado",
    "infortunios": "infortunio",
    "infosuras": "infosura",
    "infraacústica": "infraacústico",
    "infraacústicas": "infraacústico",
    "infraacústicos": "infraacústico",
    "infracciones": "infracción",
    "infraclaviculares": "infraclavicular",
    "infracostos": "infracosto",
    "infracta": "infracto",
    "infractas": "infracto",
    "infractora": "infractor",
    "infractoras": "infractor",
    "infractores": "infractor",
    "infractos": "infracto",
    "infradina": "infradino",
    "infradinas": "infradino",
    "infradinos": "infradino",
    "infradotaciones": "infradotación",
    "infraespinosa": "infraespinoso",
    "infraespinosas": "infraespinoso",
    "infraespinosos": "infraespinoso",
    "infraestructurales": "infraestructural",
    "infraestructuras": "infraestructura",
    "infraglenoidea": "infraglenoideo",
    "infraglenoideas": "infraglenoideo",
    "infraglenoideos": "infraglenoideo",
    "infraglótica": "infraglótico",
    "infraglóticas": "infraglótico",
    "infraglóticos": "infraglótico",
    "infragnacias": "infragnacia",
    "infrahioidea": "infrahioideo",
    "infrahioideas": "infrahioideo",
    "infrahioideos": "infrahioideo",
    "infrahumana": "infrahumano",
    "infrahumanas": "infrahumano",
    "infrahumanos": "infrahumano",
    "inframodulaciones": "inframodulación",
    "infrangibles": "infrangible",
    "infranqueables": "infranqueable",
    "infraoccipitales": "infraoccipital",
    "infraoclusiones": "infraoclusión",
    "infraoctava": "infraoctavo",
    "infraoctavas": "infraoctavo",
    "infraoctavos": "infraoctavo",
    "infraorbitaria": "infraorbitario",
    "infraorbitarias": "infraorbitario",
    "infraorbitarios": "infraorbitario",
    "infrapilórica": "infrapilórico",
    "infrapilóricas": "infrapilórico",
    "infrapilóricos": "infrapilórico",
    "infrapiriformes": "infrapiriforme",
    "infrapoblaciones": "infrapoblación",
    "infraproteínas": "infraproteína",
    "infrarregionales": "infrarregional",
    "infrarroja": "infrarrojo",
    "infrarrojas": "infrarrojo",
    "infrarrojos": "infrarrojo",
    "infrarrotuliana": "infrarrotuliano",
    "infrarrotulianas": "infrarrotuliano",
    "infrarrotulianos": "infrarrotuliano",
    "infrascripta": "infrascripto",
    "infrascriptas": "infrascripto",
    "infrascriptos": "infrascripto",
    "infrascrita": "infrascrito",
    "infrascritas": "infrascrito",
    "infrascritos": "infrascrito",
    "infraseguros": "infraseguro",
    "infrasonidos": "infrasonido",
    "infrasonora": "infrasonoro",
    "infrasonoras": "infrasonoro",
    "infrasonoros": "infrasonoro",
    "infrasónica": "infrasónico",
    "infrasónicas": "infrasónico",
    "infrasónicos": "infrasónico",
    "infratemporales": "infratemporal",
    "infratrocleares": "infratroclear",
    "infrautilice": "infrautilizar",
    "infrautilicemos": "infrautilizar",
    "infrautilicen": "infrautilizar",
    "infrautilices": "infrautilizar",
    "infrautilicé": "infrautilizar",
    "infrautilicéis": "infrautilizar",
    "infrautiliza": "infrautilizar",
    "infrautilizaba": "infrautilizar",
    "infrautilizabais": "infrautilizar",
    "infrautilizaban": "infrautilizar",
    "infrautilizabas": "infrautilizar",
    "infrautilizad": "infrautilizar",
    "infrautilizada": "infrautilizar",
    "infrautilizadas": "infrautilizar",
    "infrautilizado": "infrautilizar",
    "infrautilizados": "infrautilizar",
    "infrautilizamos": "infrautilizar",
    "infrautilizan": "infrautilizar",
    "infrautilizando": "infrautilizar",
    "infrautilizara": "infrautilizar",
    "infrautilizarais": "infrautilizar",
    "infrautilizaran": "infrautilizar",
    "infrautilizaras": "infrautilizar",
    "infrautilizare": "infrautilizar",
    "infrautilizareis": "infrautilizar",
    "infrautilizaremos": "infrautilizar",
    "infrautilizaren": "infrautilizar",
    "infrautilizares": "infrautilizar",
    "infrautilizaron": "infrautilizar",
    "infrautilizará": "infrautilizar",
    "infrautilizarán": "infrautilizar",
    "infrautilizarás": "infrautilizar",
    "infrautilizaré": "infrautilizar",
    "infrautilizaréis": "infrautilizar",
    "infrautilizaría": "infrautilizar",
    "infrautilizaríais": "infrautilizar",
    "infrautilizaríamos": "infrautilizar",
    "infrautilizarían": "infrautilizar",
    "infrautilizarías": "infrautilizar",
    "infrautilizas": "infrautilizar",
    "infrautilizase": "infrautilizar",
    "infrautilizaseis": "infrautilizar",
    "infrautilizasen": "infrautilizar",
    "infrautilizases": "infrautilizar",
    "infrautilizaste": "infrautilizar",
    "infrautilizasteis": "infrautilizar",
    "infrautilizo": "infrautilizar",
    "infrautilizábamos": "infrautilizar",
    "infrautilizáis": "infrautilizar",
    "infrautilizáramos": "infrautilizar",
    "infrautilizáremos": "infrautilizar",
    "infrautilizásemos": "infrautilizar",
    "infrautilizó": "infrautilizar",
    "infravalora": "infravalorar",
    "infravaloraba": "infravalorar",
    "infravalorabais": "infravalorar",
    "infravaloraban": "infravalorar",
    "infravalorabas": "infravalorar",
    "infravaloraciones": "infravaloración",
    "infravalorad": "infravalorar",
    "infravalorada": "infravalorar",
    "infravaloradas": "infravalorar",
    "infravalorado": "infravalorar",
    "infravalorados": "infravalorar",
    "infravaloramos": "infravalorar",
    "infravaloran": "infravalorar",
    "infravalorando": "infravalorar",
    "infravalorara": "infravalorar",
    "infravalorarais": "infravalorar",
    "infravaloraran": "infravalorar",
    "infravaloraras": "infravalorar",
    "infravalorare": "infravalorar",
    "infravalorareis": "infravalorar",
    "infravaloraremos": "infravalorar",
    "infravaloraren": "infravalorar",
    "infravalorares": "infravalorar",
    "infravaloraron": "infravalorar",
    "infravalorará": "infravalorar",
    "infravalorarán": "infravalorar",
    "infravalorarás": "infravalorar",
    "infravaloraré": "infravalorar",
    "infravaloraréis": "infravalorar",
    "infravaloraría": "infravalorar",
    "infravaloraríais": "infravalorar",
    "infravaloraríamos": "infravalorar",
    "infravalorarían": "infravalorar",
    "infravalorarías": "infravalorar",
    "infravaloras": "infravalorar",
    "infravalorase": "infravalorar",
    "infravaloraseis": "infravalorar",
    "infravalorasen": "infravalorar",
    "infravalorases": "infravalorar",
    "infravaloraste": "infravalorar",
    "infravalorasteis": "infravalorar",
    "infravalore": "infravalorar",
    "infravaloremos": "infravalorar",
    "infravaloren": "infravalorar",
    "infravalores": "infravalorar",
    "infravaloro": "infravalorar",
    "infravalorábamos": "infravalorar",
    "infravaloráis": "infravalorar",
    "infravaloráramos": "infravalorar",
    "infravaloráremos": "infravalorar",
    "infravalorásemos": "infravalorar",
    "infravaloré": "infravalorar",
    "infravaloréis": "infravalorar",
    "infravaloró": "infravalorar",
    "infrecuencias": "infrecuencia",
    "infrecuentes": "infrecuente",
    "infringe": "infringir",
    "infringen": "infringir",
    "infringes": "infringir",
    "infringid": "infringir",
    "infringida": "infringir",
    "infringidas": "infringir",
    "infringido": "infringir",
    "infringidos": "infringir",
    "infringiendo": "infringir",
    "infringiera": "infringir",
    "infringierais": "infringir",
    "infringieran": "infringir",
    "infringieras": "infringir",
    "infringiere": "infringir",
    "infringiereis": "infringir",
    "infringieren": "infringir",
    "infringieres": "infringir",
    "infringieron": "infringir",
    "infringiese": "infringir",
    "infringieseis": "infringir",
    "infringiesen": "infringir",
    "infringieses": "infringir",
    "infringimos": "infringir",
    "infringiremos": "infringir",
    "infringirá": "infringir",
    "infringirán": "infringir",
    "infringirás": "infringir",
    "infringiré": "infringir",
    "infringiréis": "infringir",
    "infringiría": "infringir",
    "infringiríais": "infringir",
    "infringiríamos": "infringir",
    "infringirían": "infringir",
    "infringirías": "infringir",
    "infringiste": "infringir",
    "infringisteis": "infringir",
    "infringiéramos": "infringir",
    "infringiéremos": "infringir",
    "infringiésemos": "infringir",
    "infringió": "infringir",
    "infringí": "infringir",
    "infringía": "infringir",
    "infringíais": "infringir",
    "infringíamos": "infringir",
    "infringían": "infringir",
    "infringías": "infringir",
    "infringís": "infringir",
    "infrinja": "infringir",
    "infrinjamos": "infringir",
    "infrinjan": "infringir",
    "infrinjas": "infringir",
    "infrinjo": "infringir",
    "infrinjáis": "infringir",
    "infructuosa": "infructuoso",
    "infructuosas": "infructuoso",
    "infructuosidades": "infructuosidad",
    "infructuosos": "infructuoso",
    "infructífera": "infructífero",
    "infructíferas": "infructífero",
    "infructíferos": "infructífero",
    "infrugífera": "infrugífero",
    "infrugíferas": "infrugífero",
    "infrugíferos": "infrugífero",
    "infrutescencias": "infrutescencia",
    "infumables": "infumable",
    "infunda": "infundir",
    "infundada": "infundado",
    "infundadas": "infundado",
    "infundados": "infundado",
    "infundamos": "infundir",
    "infundan": "infundir",
    "infundas": "infundir",
    "infunde": "infundir",
    "infunden": "infundir",
    "infundes": "infundir",
    "infundibulares": "infundibular",
    "infundibuliformes": "infundibuliforme",
    "infundid": "infundir",
    "infundida": "infundir",
    "infundidas": "infundir",
    "infundido": "infundir",
    "infundidos": "infundir",
    "infundiendo": "infundir",
    "infundiera": "infundir",
    "infundierais": "infundir",
    "infundieran": "infundir",
    "infundieras": "infundir",
    "infundiere": "infundir",
    "infundiereis": "infundir",
    "infundieren": "infundir",
    "infundieres": "infundir",
    "infundieron": "infundir",
    "infundiese": "infundir",
    "infundieseis": "infundir",
    "infundiesen": "infundir",
    "infundieses": "infundir",
    "infundimos": "infundir",
    "infundios": "infundio",
    "infundiosa": "infundioso",
    "infundiosas": "infundioso",
    "infundiosos": "infundioso",
    "infundiremos": "infundir",
    "infundirá": "infundir",
    "infundirán": "infundir",
    "infundirás": "infundir",
    "infundiré": "infundir",
    "infundiréis": "infundir",
    "infundiría": "infundir",
    "infundiríais": "infundir",
    "infundiríamos": "infundir",
    "infundirían": "infundir",
    "infundirías": "infundir",
    "infundiste": "infundir",
    "infundisteis": "infundir",
    "infundiéramos": "infundir",
    "infundiéremos": "infundir",
    "infundiésemos": "infundir",
    "infundió": "infundir",
    "infundo": "infundir",
    "infundáis": "infundir",
    "infundí": "infundir",
    "infundía": "infundir",
    "infundíais": "infundir",
    "infundíamos": "infundir",
    "infundían": "infundir",
    "infundías": "infundir",
    "infundíbulos": "infundíbulo",
    "infundís": "infundir",
    "infurciones": "infurción",
    "infurcioniega": "infurcioniego",
    "infurcioniegas": "infurcioniego",
    "infurcioniegos": "infurcioniego",
    "infusa": "infuso",
    "infusas": "infuso",
    "infusibilidades": "infusibilidad",
    "infusibles": "infusible",
    "infusiones": "infusión",
    "infusorios": "infusorio",
    "infusos": "infuso",
    "ingaes": "ingá",
    "ingas": "inga",
    "ingenia": "ingeniar",
    "ingeniaba": "ingeniar",
    "ingeniabais": "ingeniar",
    "ingeniaban": "ingeniar",
    "ingeniabas": "ingeniar",
    "ingeniad": "ingeniar",
    "ingeniada": "ingeniar",
    "ingeniadas": "ingeniar",
    "ingeniado": "ingeniar",
    "ingeniados": "ingeniar",
    "ingeniamos": "ingeniar",
    "ingenian": "ingeniar",
    "ingeniando": "ingeniar",
    "ingeniara": "ingeniar",
    "ingeniarais": "ingeniar",
    "ingeniaran": "ingeniar",
    "ingeniaras": "ingeniar",
    "ingeniare": "ingeniar",
    "ingeniareis": "ingeniar",
    "ingeniaremos": "ingeniar",
    "ingeniaren": "ingeniar",
    "ingeniares": "ingeniar",
    "ingeniaron": "ingeniar",
    "ingeniará": "ingeniar",
    "ingeniarán": "ingeniar",
    "ingeniarás": "ingeniar",
    "ingeniaré": "ingeniar",
    "ingeniaréis": "ingeniar",
    "ingeniaría": "ingeniar",
    "ingeniaríais": "ingeniar",
    "ingeniaríamos": "ingeniar",
    "ingeniarían": "ingeniar",
    "ingeniarías": "ingeniar",
    "ingenias": "ingeniar",
    "ingeniase": "ingeniar",
    "ingeniaseis": "ingeniar",
    "ingeniasen": "ingeniar",
    "ingeniases": "ingeniar",
    "ingeniaste": "ingeniar",
    "ingeniasteis": "ingeniar",
    "ingenie": "ingeniar",
    "ingeniemos": "ingeniar",
    "ingenien": "ingeniar",
    "ingeniera": "ingeniero",
    "ingenieras": "ingeniero",
    "ingenierizaciones": "ingenierización",
    "ingenieros": "ingeniero",
    "ingenierías": "ingeniería",
    "ingenies": "ingeniar",
    "ingenio": "ingeniar",
    "ingenios": "ingenio",
    "ingeniosa": "ingenioso",
    "ingeniosas": "ingenioso",
    "ingeniosidades": "ingeniosidad",
    "ingeniosos": "ingenioso",
    "ingeniábamos": "ingeniar",
    "ingeniáis": "ingeniar",
    "ingeniáramos": "ingeniar",
    "ingeniáremos": "ingeniar",
    "ingeniásemos": "ingeniar",
    "ingenié": "ingeniar",
    "ingeniéis": "ingeniar",
    "ingenió": "ingeniar",
    "ingentes": "ingente",
    "ingenua": "ingenuo",
    "ingenuas": "ingenuo",
    "ingenuidades": "ingenuidad",
    "ingenuos": "ingenuo",
    "ingerencias": "ingerencia",
    "ingerid": "ingerir",
    "ingerida": "ingerir",
    "ingeridas": "ingerir",
    "ingerido": "ingerir",
    "ingeridos": "ingerir",
    "ingerimos": "ingerir",
    "ingeriremos": "ingerir",
    "ingerirá": "ingerir",
    "ingerirán": "ingerir",
    "ingerirás": "ingerir",
    "ingeriré": "ingerir",
    "ingeriréis": "ingerir",
    "ingeriría": "ingerir",
    "ingeriríais": "ingerir",
    "ingeriríamos": "ingerir",
    "ingerirían": "ingerir",
    "ingerirías": "ingerir",
    "ingeriste": "ingerir",
    "ingeristeis": "ingerir",
    "ingerí": "ingerir",
    "ingería": "ingerir",
    "ingeríais": "ingerir",
    "ingeríamos": "ingerir",
    "ingerían": "ingerir",
    "ingerías": "ingerir",
    "ingerís": "ingerir",
    "ingestas": "ingesta",
    "ingestiones": "ingestión",
    "ingestiónes": "ingestión",
    "ingiera": "ingerir",
    "ingieran": "ingerir",
    "ingieras": "ingerir",
    "ingiere": "ingerir",
    "ingieren": "ingerir",
    "ingieres": "ingerir",
    "ingiero": "ingerir",
    "ingiramos": "ingerir",
    "ingiriendo": "ingerir",
    "ingiriera": "ingerir",
    "ingirierais": "ingerir",
    "ingirieran": "ingerir",
    "ingirieras": "ingerir",
    "ingiriere": "ingerir",
    "ingiriereis": "ingerir",
    "ingirieren": "ingerir",
    "ingirieres": "ingerir",
    "ingirieron": "ingerir",
    "ingiriese": "ingerir",
    "ingirieseis": "ingerir",
    "ingiriesen": "ingerir",
    "ingirieses": "ingerir",
    "ingiriéramos": "ingerir",
    "ingiriéremos": "ingerir",
    "ingiriésemos": "ingerir",
    "ingirió": "ingerir",
    "ingiráis": "ingerir",
    "ingles": "ingle",
    "inglesa": "inglés",
    "inglesas": "inglés",
    "ingleses": "inglés",
    "inglesismos": "inglesismo",
    "ingletes": "inglete",
    "ingobernables": "ingobernable",
    "ingrata": "ingrato",
    "ingratas": "ingrato",
    "ingratitudes": "ingratitud",
    "ingratos": "ingrato",
    "ingravideces": "ingravidez",
    "ingredientes": "ingrediente",
    "ingresa": "ingresar",
    "ingresaba": "ingresar",
    "ingresabais": "ingresar",
    "ingresaban": "ingresar",
    "ingresabas": "ingresar",
    "ingresad": "ingresar",
    "ingresada": "ingresar",
    "ingresadas": "ingresar",
    "ingresado": "ingresar",
    "ingresados": "ingresar",
    "ingresamos": "ingresar",
    "ingresan": "ingresar",
    "ingresando": "ingresar",
    "ingresara": "ingresar",
    "ingresarais": "ingresar",
    "ingresaran": "ingresar",
    "ingresaras": "ingresar",
    "ingresare": "ingresar",
    "ingresareis": "ingresar",
    "ingresaremos": "ingresar",
    "ingresaren": "ingresar",
    "ingresares": "ingresar",
    "ingresaron": "ingresar",
    "ingresará": "ingresar",
    "ingresarán": "ingresar",
    "ingresarás": "ingresar",
    "ingresaré": "ingresar",
    "ingresaréis": "ingresar",
    "ingresaría": "ingresar",
    "ingresaríais": "ingresar",
    "ingresaríamos": "ingresar",
    "ingresarían": "ingresar",
    "ingresarías": "ingresar",
    "ingresas": "ingresar",
    "ingresase": "ingresar",
    "ingresaseis": "ingresar",
    "ingresasen": "ingresar",
    "ingresases": "ingresar",
    "ingresaste": "ingresar",
    "ingresasteis": "ingresar",
    "ingrese": "ingresar",
    "ingresemos": "ingresar",
    "ingresen": "ingresar",
    "ingreses": "ingresar",
    "ingresiones": "ingresión",
    "ingresiva": "ingresivo",
    "ingresivas": "ingresivo",
    "ingresivos": "ingresivo",
    "ingreso": "ingresar",
    "ingresos": "ingreso",
    "ingresábamos": "ingresar",
    "ingresáis": "ingresar",
    "ingresáramos": "ingresar",
    "ingresáremos": "ingresar",
    "ingresásemos": "ingresar",
    "ingresé": "ingresar",
    "ingreséis": "ingresar",
    "ingresó": "ingresar",
    "ingrávida": "ingrávido",
    "ingrávidas": "ingrávido",
    "ingrávidos": "ingrávido",
    "inguinales": "inguinal",
    "inguinaria": "inguinario",
    "inguinarias": "inguinario",
    "inguinarios": "inguinario",
    "ingurgita": "ingurgitar",
    "ingurgitaba": "ingurgitar",
    "ingurgitabais": "ingurgitar",
    "ingurgitaban": "ingurgitar",
    "ingurgitabas": "ingurgitar",
    "ingurgitaciones": "ingurgitación",
    "ingurgitad": "ingurgitar",
    "ingurgitada": "ingurgitar",
    "ingurgitadas": "ingurgitar",
    "ingurgitado": "ingurgitar",
    "ingurgitados": "ingurgitar",
    "ingurgitamos": "ingurgitar",
    "ingurgitan": "ingurgitar",
    "ingurgitando": "ingurgitar",
    "ingurgitara": "ingurgitar",
    "ingurgitarais": "ingurgitar",
    "ingurgitaran": "ingurgitar",
    "ingurgitaras": "ingurgitar",
    "ingurgitare": "ingurgitar",
    "ingurgitareis": "ingurgitar",
    "ingurgitaremos": "ingurgitar",
    "ingurgitaren": "ingurgitar",
    "ingurgitares": "ingurgitar",
    "ingurgitaron": "ingurgitar",
    "ingurgitará": "ingurgitar",
    "ingurgitarán": "ingurgitar",
    "ingurgitarás": "ingurgitar",
    "ingurgitaré": "ingurgitar",
    "ingurgitaréis": "ingurgitar",
    "ingurgitaría": "ingurgitar",
    "ingurgitaríais": "ingurgitar",
    "ingurgitaríamos": "ingurgitar",
    "ingurgitarían": "ingurgitar",
    "ingurgitarías": "ingurgitar",
    "ingurgitas": "ingurgitar",
    "ingurgitase": "ingurgitar",
    "ingurgitaseis": "ingurgitar",
    "ingurgitasen": "ingurgitar",
    "ingurgitases": "ingurgitar",
    "ingurgitaste": "ingurgitar",
    "ingurgitasteis": "ingurgitar",
    "ingurgite": "ingurgitar",
    "ingurgitemos": "ingurgitar",
    "ingurgiten": "ingurgitar",
    "ingurgites": "ingurgitar",
    "ingurgito": "ingurgitar",
    "ingurgitábamos": "ingurgitar",
    "ingurgitáis": "ingurgitar",
    "ingurgitáramos": "ingurgitar",
    "ingurgitáremos": "ingurgitar",
    "ingurgitásemos": "ingurgitar",
    "ingurgité": "ingurgitar",
    "ingurgitéis": "ingurgitar",
    "ingurgitó": "ingurgitar",
    "ingushes": "ingush",
    "ingás": "ingá",
    "ingénita": "ingénito",
    "ingénitas": "ingénito",
    "ingénitos": "ingénito",
    "inhabilidades": "inhabilidad",
    "inhabilita": "inhabilitar",
    "inhabilitaba": "inhabilitar",
    "inhabilitabais": "inhabilitar",
    "inhabilitaban": "inhabilitar",
    "inhabilitabas": "inhabilitar",
    "inhabilitaciones": "inhabilitación",
    "inhabilitad": "inhabilitar",
    "inhabilitada": "inhabilitar",
    "inhabilitadas": "inhabilitar",
    "inhabilitado": "inhabilitar",
    "inhabilitados": "inhabilitar",
    "inhabilitamos": "inhabilitar",
    "inhabilitan": "inhabilitar",
    "inhabilitando": "inhabilitar",
    "inhabilitara": "inhabilitar",
    "inhabilitarais": "inhabilitar",
    "inhabilitaran": "inhabilitar",
    "inhabilitaras": "inhabilitar",
    "inhabilitare": "inhabilitar",
    "inhabilitareis": "inhabilitar",
    "inhabilitaremos": "inhabilitar",
    "inhabilitaren": "inhabilitar",
    "inhabilitares": "inhabilitar",
    "inhabilitaron": "inhabilitar",
    "inhabilitará": "inhabilitar",
    "inhabilitarán": "inhabilitar",
    "inhabilitarás": "inhabilitar",
    "inhabilitaré": "inhabilitar",
    "inhabilitaréis": "inhabilitar",
    "inhabilitaría": "inhabilitar",
    "inhabilitaríais": "inhabilitar",
    "inhabilitaríamos": "inhabilitar",
    "inhabilitarían": "inhabilitar",
    "inhabilitarías": "inhabilitar",
    "inhabilitas": "inhabilitar",
    "inhabilitase": "inhabilitar",
    "inhabilitaseis": "inhabilitar",
    "inhabilitasen": "inhabilitar",
    "inhabilitases": "inhabilitar",
    "inhabilitaste": "inhabilitar",
    "inhabilitasteis": "inhabilitar",
    "inhabilite": "inhabilitar",
    "inhabilitemos": "inhabilitar",
    "inhabiliten": "inhabilitar",
    "inhabilites": "inhabilitar",
    "inhabilito": "inhabilitar",
    "inhabilitábamos": "inhabilitar",
    "inhabilitáis": "inhabilitar",
    "inhabilitáramos": "inhabilitar",
    "inhabilitáremos": "inhabilitar",
    "inhabilitásemos": "inhabilitar",
    "inhabilité": "inhabilitar",
    "inhabilitéis": "inhabilitar",
    "inhabilitó": "inhabilitar",
    "inhabitables": "inhabitable",
    "inhabitada": "inhabitado",
    "inhabitadas": "inhabitado",
    "inhabitados": "inhabitado",
    "inhacedera": "inhacedero",
    "inhacederas": "inhacedero",
    "inhacederos": "inhacedero",
    "inhala": "inhalar",
    "inhalaba": "inhalar",
    "inhalabais": "inhalar",
    "inhalaban": "inhalar",
    "inhalabas": "inhalar",
    "inhalaciones": "inhalación",
    "inhalad": "inhalar",
    "inhalada": "inhalar",
    "inhaladas": "inhalar",
    "inhalado": "inhalar",
    "inhaladora": "inhalador",
    "inhaladoras": "inhalador",
    "inhaladores": "inhalador",
    "inhalados": "inhalar",
    "inhalamos": "inhalar",
    "inhalan": "inhalar",
    "inhalando": "inhalar",
    "inhalantes": "inhalante",
    "inhalara": "inhalar",
    "inhalarais": "inhalar",
    "inhalaran": "inhalar",
    "inhalaras": "inhalar",
    "inhalare": "inhalar",
    "inhalareis": "inhalar",
    "inhalaremos": "inhalar",
    "inhalaren": "inhalar",
    "inhalares": "inhalar",
    "inhalaron": "inhalar",
    "inhalará": "inhalar",
    "inhalarán": "inhalar",
    "inhalarás": "inhalar",
    "inhalaré": "inhalar",
    "inhalaréis": "inhalar",
    "inhalaría": "inhalar",
    "inhalaríais": "inhalar",
    "inhalaríamos": "inhalar",
    "inhalarían": "inhalar",
    "inhalarías": "inhalar",
    "inhalas": "inhalar",
    "inhalase": "inhalar",
    "inhalaseis": "inhalar",
    "inhalasen": "inhalar",
    "inhalases": "inhalar",
    "inhalaste": "inhalar",
    "inhalasteis": "inhalar",
    "inhale": "inhalar",
    "inhalemos": "inhalar",
    "inhalen": "inhalar",
    "inhales": "inhalar",
    "inhalo": "inhalar",
    "inhalábamos": "inhalar",
    "inhaláis": "inhalar",
    "inhaláramos": "inhalar",
    "inhaláremos": "inhalar",
    "inhalásemos": "inhalar",
    "inhalé": "inhalar",
    "inhaléis": "inhalar",
    "inhaló": "inhalar",
    "inherencias": "inherencia",
    "inherentes": "inherente",
    "inhiba": "inhibir",
    "inhibamos": "inhibir",
    "inhiban": "inhibir",
    "inhibas": "inhibir",
    "inhibe": "inhibir",
    "inhiben": "inhibir",
    "inhibes": "inhibir",
    "inhibiciones": "inhibición",
    "inhibid": "inhibir",
    "inhibida": "inhibir",
    "inhibidas": "inhibir",
    "inhibido": "inhibir",
    "inhibidora": "inhibidor",
    "inhibidoras": "inhibidor",
    "inhibidores": "inhibidor",
    "inhibidos": "inhibir",
    "inhibiendo": "inhibir",
    "inhibiera": "inhibir",
    "inhibierais": "inhibir",
    "inhibieran": "inhibir",
    "inhibieras": "inhibir",
    "inhibiere": "inhibir",
    "inhibiereis": "inhibir",
    "inhibieren": "inhibir",
    "inhibieres": "inhibir",
    "inhibieron": "inhibir",
    "inhibiese": "inhibir",
    "inhibieseis": "inhibir",
    "inhibiesen": "inhibir",
    "inhibieses": "inhibir",
    "inhibimos": "inhibir",
    "inhibiremos": "inhibir",
    "inhibirá": "inhibir",
    "inhibirán": "inhibir",
    "inhibirás": "inhibir",
    "inhibiré": "inhibir",
    "inhibiréis": "inhibir",
    "inhibiría": "inhibir",
    "inhibiríais": "inhibir",
    "inhibiríamos": "inhibir",
    "inhibirían": "inhibir",
    "inhibirías": "inhibir",
    "inhibiste": "inhibir",
    "inhibisteis": "inhibir",
    "inhibitoria": "inhibitorio",
    "inhibitorias": "inhibitorio",
    "inhibitorios": "inhibitorio",
    "inhibiéramos": "inhibir",
    "inhibiéremos": "inhibir",
    "inhibiésemos": "inhibir",
    "inhibió": "inhibir",
    "inhibo": "inhibir",
    "inhibáis": "inhibir",
    "inhibí": "inhibir",
    "inhibía": "inhibir",
    "inhibíais": "inhibir",
    "inhibíamos": "inhibir",
    "inhibían": "inhibir",
    "inhibías": "inhibir",
    "inhibís": "inhibir",
    "inhomogeneidades": "inhomogeneidad",
    "inhonesta": "inhonesto",
    "inhonestas": "inhonesto",
    "inhonestidades": "inhonestidad",
    "inhonestos": "inhonesto",
    "inhospitalaria": "inhospitalario",
    "inhospitalarias": "inhospitalario",
    "inhospitalarios": "inhospitalario",
    "inhospitalidades": "inhospitalidad",
    "inhuma": "inhumar",
    "inhumaba": "inhumar",
    "inhumabais": "inhumar",
    "inhumaban": "inhumar",
    "inhumabas": "inhumar",
    "inhumaciones": "inhumación",
    "inhumad": "inhumar",
    "inhumada": "inhumar",
    "inhumadas": "inhumar",
    "inhumado": "inhumar",
    "inhumados": "inhumar",
    "inhumamos": "inhumar",
    "inhuman": "inhumar",
    "inhumana": "inhumano",
    "inhumanas": "inhumano",
    "inhumando": "inhumar",
    "inhumanidades": "inhumanidad",
    "inhumanos": "inhumano",
    "inhumara": "inhumar",
    "inhumarais": "inhumar",
    "inhumaran": "inhumar",
    "inhumaras": "inhumar",
    "inhumare": "inhumar",
    "inhumareis": "inhumar",
    "inhumaremos": "inhumar",
    "inhumaren": "inhumar",
    "inhumares": "inhumar",
    "inhumaron": "inhumar",
    "inhumará": "inhumar",
    "inhumarán": "inhumar",
    "inhumarás": "inhumar",
    "inhumaré": "inhumar",
    "inhumaréis": "inhumar",
    "inhumaría": "inhumar",
    "inhumaríais": "inhumar",
    "inhumaríamos": "inhumar",
    "inhumarían": "inhumar",
    "inhumarías": "inhumar",
    "inhumas": "inhumar",
    "inhumase": "inhumar",
    "inhumaseis": "inhumar",
    "inhumasen": "inhumar",
    "inhumases": "inhumar",
    "inhumaste": "inhumar",
    "inhumasteis": "inhumar",
    "inhume": "inhumar",
    "inhumemos": "inhumar",
    "inhumen": "inhumar",
    "inhumes": "inhumar",
    "inhumo": "inhumar",
    "inhumábamos": "inhumar",
    "inhumáis": "inhumar",
    "inhumáramos": "inhumar",
    "inhumáremos": "inhumar",
    "inhumásemos": "inhumar",
    "inhumé": "inhumar",
    "inhuméis": "inhumar",
    "inhumó": "inhumar",
    "inhábiles": "inhábil",
    "inhóspita": "inhóspito",
    "inhóspitas": "inhóspito",
    "inhóspitos": "inhóspito",
    "inias": "inia",
    "inicia": "iniciar",
    "iniciaba": "iniciar",
    "iniciabais": "iniciar",
    "iniciaban": "iniciar",
    "iniciabas": "iniciar",
    "iniciaciones": "iniciación",
    "iniciad": "iniciar",
    "iniciada": "iniciar",
    "iniciadas": "iniciar",
    "iniciado": "iniciar",
    "iniciadora": "iniciador",
    "iniciadoras": "iniciador",
    "iniciadores": "iniciador",
    "iniciados": "iniciar",
    "iniciales": "inicial",
    "inicialice": "inicializar",
    "inicialicemos": "inicializar",
    "inicialicen": "inicializar",
    "inicialices": "inicializar",
    "inicialicé": "inicializar",
    "inicialicéis": "inicializar",
    "inicializa": "inicializar",
    "inicializaba": "inicializar",
    "inicializabais": "inicializar",
    "inicializaban": "inicializar",
    "inicializabas": "inicializar",
    "inicializaciones": "inicialización",
    "inicializad": "inicializar",
    "inicializada": "inicializar",
    "inicializadas": "inicializar",
    "inicializado": "inicializar",
    "inicializadora": "inicializador",
    "inicializadoras": "inicializador",
    "inicializadores": "inicializador",
    "inicializados": "inicializar",
    "inicializamos": "inicializar",
    "inicializan": "inicializar",
    "inicializando": "inicializar",
    "inicializara": "inicializar",
    "inicializarais": "inicializar",
    "inicializaran": "inicializar",
    "inicializaras": "inicializar",
    "inicializare": "inicializar",
    "inicializareis": "inicializar",
    "inicializaremos": "inicializar",
    "inicializaren": "inicializar",
    "inicializares": "inicializar",
    "inicializaron": "inicializar",
    "inicializará": "inicializar",
    "inicializarán": "inicializar",
    "inicializarás": "inicializar",
    "inicializaré": "inicializar",
    "inicializaréis": "inicializar",
    "inicializaría": "inicializar",
    "inicializaríais": "inicializar",
    "inicializaríamos": "inicializar",
    "inicializarían": "inicializar",
    "inicializarías": "inicializar",
    "inicializas": "inicializar",
    "inicializase": "inicializar",
    "inicializaseis": "inicializar",
    "inicializasen": "inicializar",
    "inicializases": "inicializar",
    "inicializaste": "inicializar",
    "inicializasteis": "inicializar",
    "inicializo": "inicializar",
    "inicializábamos": "inicializar",
    "inicializáis": "inicializar",
    "inicializáramos": "inicializar",
    "inicializáremos": "inicializar",
    "inicializásemos": "inicializar",
    "inicializó": "inicializar",
    "iniciamos": "iniciar",
    "inician": "iniciar",
    "iniciando": "iniciar",
    "iniciara": "iniciar",
    "iniciarais": "iniciar",
    "iniciaran": "iniciar",
    "iniciaras": "iniciar",
    "iniciare": "iniciar",
    "iniciareis": "iniciar",
    "iniciaremos": "iniciar",
    "iniciaren": "iniciar",
    "iniciares": "iniciar",
    "iniciaron": "iniciar",
    "iniciará": "iniciar",
    "iniciarán": "iniciar",
    "iniciarás": "iniciar",
    "iniciaré": "iniciar",
    "iniciaréis": "iniciar",
    "iniciaría": "iniciar",
    "iniciaríais": "iniciar",
    "iniciaríamos": "iniciar",
    "iniciarían": "iniciar",
    "iniciarías": "iniciar",
    "inicias": "iniciar",
    "iniciase": "iniciar",
    "iniciaseis": "iniciar",
    "iniciasen": "iniciar",
    "iniciases": "iniciar",
    "iniciaste": "iniciar",
    "iniciasteis": "iniciar",
    "iniciativa": "iniciativo",
    "iniciativas": "iniciativo",
    "iniciativos": "iniciativo",
    "inicie": "iniciar",
    "iniciemos": "iniciar",
    "inicien": "iniciar",
    "inicies": "iniciar",
    "inicio": "iniciar",
    "inicios": "inicio",
    "iniciábamos": "iniciar",
    "iniciáis": "iniciar",
    "iniciáramos": "iniciar",
    "iniciáremos": "iniciar",
    "iniciásemos": "iniciar",
    "iniciática": "iniciático",
    "iniciáticas": "iniciático",
    "iniciáticos": "iniciático",
    "inicié": "iniciar",
    "iniciéis": "iniciar",
    "inició": "iniciar",
    "inicua": "inicuo",
    "inicuas": "inicuo",
    "inicuos": "inicuo",
    "inidentificables": "inidentificable",
    "inigualables": "inigualable",
    "inigualada": "inigualado",
    "inigualadas": "inigualado",
    "inigualados": "inigualado",
    "inimaginables": "inimaginable",
    "inimitables": "inimitable",
    "inimputabilidades": "inimputabilidad",
    "inimputables": "inimputable",
    "ininflamabilidades": "ininflamabilidad",
    "ininflamables": "ininflamable",
    "ininteligencias": "ininteligencia",
    "ininteligentes": "ininteligente",
    "ininteligibilidades": "ininteligibilidad",
    "ininteligibles": "ininteligible",
    "ininterrumpibilidades": "ininterrumpibilidad",
    "ininterrumpida": "ininterrumpido",
    "ininterrumpidas": "ininterrumpido",
    "ininterrumpidos": "ininterrumpido",
    "iniones": "inión",
    "iniquidades": "iniquidad",
    "injerencias": "injerencia",
    "injerid": "injerir",
    "injerida": "injerir",
    "injeridas": "injerir",
    "injerido": "injerir",
    "injeridos": "injerir",
    "injeriduras": "injeridura",
    "injerimos": "injerir",
    "injeriremos": "injerir",
    "injerirá": "injerir",
    "injerirán": "injerir",
    "injerirás": "injerir",
    "injeriré": "injerir",
    "injeriréis": "injerir",
    "injeriría": "injerir",
    "injeriríais": "injerir",
    "injeriríamos": "injerir",
    "injerirían": "injerir",
    "injerirías": "injerir",
    "injeriste": "injerir",
    "injeristeis": "injerir",
    "injerta": "injerto",
    "injertaba": "injertar",
    "injertabais": "injertar",
    "injertaban": "injertar",
    "injertabas": "injertar",
    "injertad": "injertar",
    "injertada": "injertar",
    "injertadas": "injertar",
    "injertado": "injertar",
    "injertadora": "injertador",
    "injertadoras": "injertador",
    "injertadores": "injertador",
    "injertados": "injertar",
    "injertamos": "injertar",
    "injertan": "injertar",
    "injertando": "injertar",
    "injertara": "injertar",
    "injertarais": "injertar",
    "injertaran": "injertar",
    "injertaras": "injertar",
    "injertare": "injertar",
    "injertareis": "injertar",
    "injertaremos": "injertar",
    "injertaren": "injertar",
    "injertares": "injertar",
    "injertaron": "injertar",
    "injertará": "injertar",
    "injertarán": "injertar",
    "injertarás": "injertar",
    "injertaré": "injertar",
    "injertaréis": "injertar",
    "injertaría": "injertar",
    "injertaríais": "injertar",
    "injertaríamos": "injertar",
    "injertarían": "injertar",
    "injertarías": "injertar",
    "injertas": "injerto",
    "injertase": "injertar",
    "injertaseis": "injertar",
    "injertasen": "injertar",
    "injertases": "injertar",
    "injertaste": "injertar",
    "injertasteis": "injertar",
    "injerte": "injertar",
    "injertemos": "injertar",
    "injerten": "injertar",
    "injerteras": "injertera",
    "injertes": "injertar",
    "injerto": "injertar",
    "injertos": "injerto",
    "injertábamos": "injertar",
    "injertáis": "injertar",
    "injertáramos": "injertar",
    "injertáremos": "injertar",
    "injertásemos": "injertar",
    "injerté": "injertar",
    "injertéis": "injertar",
    "injertó": "injertar",
    "injerí": "injerir",
    "injería": "injerir",
    "injeríais": "injerir",
    "injeríamos": "injerir",
    "injerían": "injerir",
    "injerías": "injerir",
    "injerís": "injerir",
    "injiera": "injerir",
    "injieran": "injerir",
    "injieras": "injerir",
    "injiere": "injerir",
    "injieren": "injerir",
    "injieres": "injerir",
    "injiero": "injerir",
    "injiramos": "injerir",
    "injiriendo": "injerir",
    "injiriera": "injerir",
    "injirierais": "injerir",
    "injirieran": "injerir",
    "injirieras": "injerir",
    "injiriere": "injerir",
    "injiriereis": "injerir",
    "injirieren": "injerir",
    "injirieres": "injerir",
    "injirieron": "injerir",
    "injiriese": "injerir",
    "injirieseis": "injerir",
    "injiriesen": "injerir",
    "injirieses": "injerir",
    "injiriéramos": "injerir",
    "injiriéremos": "injerir",
    "injiriésemos": "injerir",
    "injirió": "injerir",
    "injiráis": "injerir",
    "injugables": "injugable",
    "injundias": "injundia",
    "injuria": "injuriar",
    "injuriaba": "injuriar",
    "injuriabais": "injuriar",
    "injuriaban": "injuriar",
    "injuriabas": "injuriar",
    "injuriad": "injuriar",
    "injuriada": "injuriar",
    "injuriadas": "injuriar",
    "injuriado": "injuriar",
    "injuriadora": "injuriador",
    "injuriadoras": "injuriador",
    "injuriadores": "injuriador",
    "injuriados": "injuriar",
    "injuriamos": "injuriar",
    "injurian": "injuriar",
    "injuriando": "injuriar",
    "injuriara": "injuriar",
    "injuriarais": "injuriar",
    "injuriaran": "injuriar",
    "injuriaras": "injuriar",
    "injuriare": "injuriar",
    "injuriareis": "injuriar",
    "injuriaremos": "injuriar",
    "injuriaren": "injuriar",
    "injuriares": "injuriar",
    "injuriaron": "injuriar",
    "injuriará": "injuriar",
    "injuriarán": "injuriar",
    "injuriarás": "injuriar",
    "injuriaré": "injuriar",
    "injuriaréis": "injuriar",
    "injuriaría": "injuriar",
    "injuriaríais": "injuriar",
    "injuriaríamos": "injuriar",
    "injuriarían": "injuriar",
    "injuriarías": "injuriar",
    "injurias": "injuriar",
    "injuriase": "injuriar",
    "injuriaseis": "injuriar",
    "injuriasen": "injuriar",
    "injuriases": "injuriar",
    "injuriaste": "injuriar",
    "injuriasteis": "injuriar",
    "injurie": "injuriar",
    "injuriemos": "injuriar",
    "injurien": "injuriar",
    "injuries": "injuriar",
    "injurio": "injuriar",
    "injuriosa": "injurioso",
    "injuriosas": "injurioso",
    "injuriosos": "injurioso",
    "injuriábamos": "injuriar",
    "injuriáis": "injuriar",
    "injuriáramos": "injuriar",
    "injuriáremos": "injuriar",
    "injuriásemos": "injuriar",
    "injurié": "injuriar",
    "injuriéis": "injuriar",
    "injurió": "injuriar",
    "injusta": "injusto",
    "injustas": "injusto",
    "injusticias": "injusticia",
    "injustificables": "injustificable",
    "injustificada": "injustificado",
    "injustificadas": "injustificado",
    "injustificados": "injustificado",
    "injustos": "injusto",
    "inllevables": "inllevable",
    "inmaculada": "inmaculado",
    "inmaculadas": "inmaculado",
    "inmaculados": "inmaculado",
    "inmadura": "inmaduro",
    "inmaduras": "inmaduro",
    "inmadureces": "inmadurez",
    "inmaduros": "inmaduro",
    "inmanejabilidades": "inmanejabilidad",
    "inmanejables": "inmanejable",
    "inmanencias": "inmanencia",
    "inmanentes": "inmanente",
    "inmarcesibles": "inmarcesible",
    "inmarchitables": "inmarchitable",
    "inmateriales": "inmaterial",
    "inmaterialidades": "inmaterialidad",
    "inmatizables": "inmatizable",
    "inmediaciones": "inmediación",
    "inmediaciónes": "inmediación",
    "inmediata": "inmediato",
    "inmediatas": "inmediato",
    "inmediateces": "inmediatez",
    "inmediatos": "inmediato",
    "inmedicables": "inmedicable",
    "inmejorables": "inmejorable",
    "inmemorables": "inmemorable",
    "inmemoriales": "inmemorial",
    "inmensa": "inmenso",
    "inmensas": "inmenso",
    "inmensidades": "inmensidad",
    "inmensos": "inmenso",
    "inmensurables": "inmensurable",
    "inmerecida": "inmerecido",
    "inmerecidas": "inmerecido",
    "inmerecidos": "inmerecido",
    "inmerge": "inmergir",
    "inmergen": "inmergir",
    "inmerges": "inmergir",
    "inmergid": "inmergir",
    "inmergida": "inmergir",
    "inmergidas": "inmergir",
    "inmergido": "inmergir",
    "inmergidos": "inmergir",
    "inmergiendo": "inmergir",
    "inmergiera": "inmergir",
    "inmergierais": "inmergir",
    "inmergieran": "inmergir",
    "inmergieras": "inmergir",
    "inmergiere": "inmergir",
    "inmergiereis": "inmergir",
    "inmergieren": "inmergir",
    "inmergieres": "inmergir",
    "inmergieron": "inmergir",
    "inmergiese": "inmergir",
    "inmergieseis": "inmergir",
    "inmergiesen": "inmergir",
    "inmergieses": "inmergir",
    "inmergimos": "inmergir",
    "inmergiremos": "inmergir",
    "inmergirá": "inmergir",
    "inmergirán": "inmergir",
    "inmergirás": "inmergir",
    "inmergiré": "inmergir",
    "inmergiréis": "inmergir",
    "inmergiría": "inmergir",
    "inmergiríais": "inmergir",
    "inmergiríamos": "inmergir",
    "inmergirían": "inmergir",
    "inmergirías": "inmergir",
    "inmergiste": "inmergir",
    "inmergisteis": "inmergir",
    "inmergiéramos": "inmergir",
    "inmergiéremos": "inmergir",
    "inmergiésemos": "inmergir",
    "inmergió": "inmergir",
    "inmergí": "inmergir",
    "inmergía": "inmergir",
    "inmergíais": "inmergir",
    "inmergíamos": "inmergir",
    "inmergían": "inmergir",
    "inmergías": "inmergir",
    "inmergís": "inmergir",
    "inmerja": "inmergir",
    "inmerjamos": "inmergir",
    "inmerjan": "inmergir",
    "inmerjas": "inmergir",
    "inmerjo": "inmergir",
    "inmerjáis": "inmergir",
    "inmersa": "inmerso",
    "inmersas": "inmerso",
    "inmersiones": "inmersión",
    "inmersos": "inmerso",
    "inmigra": "inmigrar",
    "inmigraba": "inmigrar",
    "inmigrabais": "inmigrar",
    "inmigraban": "inmigrar",
    "inmigrabas": "inmigrar",
    "inmigraciones": "inmigración",
    "inmigrad": "inmigrar",
    "inmigrada": "inmigrar",
    "inmigradas": "inmigrar",
    "inmigrado": "inmigrar",
    "inmigrados": "inmigrar",
    "inmigramos": "inmigrar",
    "inmigran": "inmigrar",
    "inmigrando": "inmigrar",
    "inmigrantes": "inmigrante",
    "inmigrara": "inmigrar",
    "inmigrarais": "inmigrar",
    "inmigraran": "inmigrar",
    "inmigraras": "inmigrar",
    "inmigrare": "inmigrar",
    "inmigrareis": "inmigrar",
    "inmigraremos": "inmigrar",
    "inmigraren": "inmigrar",
    "inmigrares": "inmigrar",
    "inmigraron": "inmigrar",
    "inmigrará": "inmigrar",
    "inmigrarán": "inmigrar",
    "inmigrarás": "inmigrar",
    "inmigraré": "inmigrar",
    "inmigraréis": "inmigrar",
    "inmigraría": "inmigrar",
    "inmigraríais": "inmigrar",
    "inmigraríamos": "inmigrar",
    "inmigrarían": "inmigrar",
    "inmigrarías": "inmigrar",
    "inmigras": "inmigrar",
    "inmigrase": "inmigrar",
    "inmigraseis": "inmigrar",
    "inmigrasen": "inmigrar",
    "inmigrases": "inmigrar",
    "inmigraste": "inmigrar",
    "inmigrasteis": "inmigrar",
    "inmigratoria": "inmigratorio",
    "inmigratorias": "inmigratorio",
    "inmigratorios": "inmigratorio",
    "inmigre": "inmigrar",
    "inmigremos": "inmigrar",
    "inmigren": "inmigrar",
    "inmigres": "inmigrar",
    "inmigro": "inmigrar",
    "inmigrábamos": "inmigrar",
    "inmigráis": "inmigrar",
    "inmigráramos": "inmigrar",
    "inmigráremos": "inmigrar",
    "inmigrásemos": "inmigrar",
    "inmigré": "inmigrar",
    "inmigréis": "inmigrar",
    "inmigró": "inmigrar",
    "inminencias": "inminencia",
    "inminentes": "inminente",
    "inmiscibles": "inmiscible",
    "inmiscuid": "inmiscuir",
    "inmiscuida": "inmiscuir",
    "inmiscuidas": "inmiscuir",
    "inmiscuido": "inmiscuir",
    "inmiscuidos": "inmiscuir",
    "inmiscuimos": "inmiscuir",
    "inmiscuiremos": "inmiscuir",
    "inmiscuirá": "inmiscuir",
    "inmiscuirán": "inmiscuir",
    "inmiscuirás": "inmiscuir",
    "inmiscuiré": "inmiscuir",
    "inmiscuiréis": "inmiscuir",
    "inmiscuiría": "inmiscuir",
    "inmiscuiríais": "inmiscuir",
    "inmiscuiríamos": "inmiscuir",
    "inmiscuirían": "inmiscuir",
    "inmiscuirías": "inmiscuir",
    "inmiscuiste": "inmiscuir",
    "inmiscuisteis": "inmiscuir",
    "inmiscuya": "inmiscuir",
    "inmiscuyamos": "inmiscuir",
    "inmiscuyan": "inmiscuir",
    "inmiscuyas": "inmiscuir",
    "inmiscuye": "inmiscuir",
    "inmiscuyen": "inmiscuir",
    "inmiscuyendo": "inmiscuir",
    "inmiscuyera": "inmiscuir",
    "inmiscuyerais": "inmiscuir",
    "inmiscuyeran": "inmiscuir",
    "inmiscuyeras": "inmiscuir",
    "inmiscuyere": "inmiscuir",
    "inmiscuyereis": "inmiscuir",
    "inmiscuyeren": "inmiscuir",
    "inmiscuyeres": "inmiscuir",
    "inmiscuyeron": "inmiscuir",
    "inmiscuyes": "inmiscuir",
    "inmiscuyese": "inmiscuir",
    "inmiscuyeseis": "inmiscuir",
    "inmiscuyesen": "inmiscuir",
    "inmiscuyeses": "inmiscuir",
    "inmiscuyo": "inmiscuir",
    "inmiscuyáis": "inmiscuir",
    "inmiscuyéramos": "inmiscuir",
    "inmiscuyéremos": "inmiscuir",
    "inmiscuyésemos": "inmiscuir",
    "inmiscuyó": "inmiscuir",
    "inmiscuí": "inmiscuir",
    "inmiscuía": "inmiscuir",
    "inmiscuíais": "inmiscuir",
    "inmiscuíamos": "inmiscuir",
    "inmiscuían": "inmiscuir",
    "inmiscuías": "inmiscuir",
    "inmiscuís": "inmiscuir",
    "inmisericordes": "inmisericorde",
    "inmisiones": "inmisión",
    "inmitancias": "inmitancia",
    "inmobiliaria": "inmobiliario",
    "inmobiliarias": "inmobiliario",
    "inmobiliarios": "inmobiliario",
    "inmobles": "inmoble",
    "inmoderaciones": "inmoderación",
    "inmoderada": "inmoderado",
    "inmoderadas": "inmoderado",
    "inmoderados": "inmoderado",
    "inmodesta": "inmodesto",
    "inmodestas": "inmodesto",
    "inmodestias": "inmodestia",
    "inmodestos": "inmodesto",
    "inmodificables": "inmodificable",
    "inmola": "inmolar",
    "inmolaba": "inmolar",
    "inmolabais": "inmolar",
    "inmolaban": "inmolar",
    "inmolabas": "inmolar",
    "inmolaciones": "inmolación",
    "inmolad": "inmolar",
    "inmolada": "inmolar",
    "inmoladas": "inmolar",
    "inmolado": "inmolar",
    "inmoladora": "inmolador",
    "inmoladoras": "inmolador",
    "inmoladores": "inmolador",
    "inmolados": "inmolar",
    "inmolamos": "inmolar",
    "inmolan": "inmolar",
    "inmolando": "inmolar",
    "inmolara": "inmolar",
    "inmolarais": "inmolar",
    "inmolaran": "inmolar",
    "inmolaras": "inmolar",
    "inmolare": "inmolar",
    "inmolareis": "inmolar",
    "inmolaremos": "inmolar",
    "inmolaren": "inmolar",
    "inmolares": "inmolar",
    "inmolaron": "inmolar",
    "inmolará": "inmolar",
    "inmolarán": "inmolar",
    "inmolarás": "inmolar",
    "inmolaré": "inmolar",
    "inmolaréis": "inmolar",
    "inmolaría": "inmolar",
    "inmolaríais": "inmolar",
    "inmolaríamos": "inmolar",
    "inmolarían": "inmolar",
    "inmolarías": "inmolar",
    "inmolas": "inmolar",
    "inmolase": "inmolar",
    "inmolaseis": "inmolar",
    "inmolasen": "inmolar",
    "inmolases": "inmolar",
    "inmolaste": "inmolar",
    "inmolasteis": "inmolar",
    "inmole": "inmolar",
    "inmolemos": "inmolar",
    "inmolen": "inmolar",
    "inmoles": "inmolar",
    "inmolo": "inmolar",
    "inmolábamos": "inmolar",
    "inmoláis": "inmolar",
    "inmoláramos": "inmolar",
    "inmoláremos": "inmolar",
    "inmolásemos": "inmolar",
    "inmolé": "inmolar",
    "inmoléis": "inmolar",
    "inmoló": "inmolar",
    "inmorales": "inmoral",
    "inmoralidades": "inmoralidad",
    "inmortales": "inmortal",
    "inmortalice": "inmortalizar",
    "inmortalicemos": "inmortalizar",
    "inmortalicen": "inmortalizar",
    "inmortalices": "inmortalizar",
    "inmortalicé": "inmortalizar",
    "inmortalicéis": "inmortalizar",
    "inmortalidades": "inmortalidad",
    "inmortaliza": "inmortalizar",
    "inmortalizaba": "inmortalizar",
    "inmortalizabais": "inmortalizar",
    "inmortalizaban": "inmortalizar",
    "inmortalizabas": "inmortalizar",
    "inmortalizad": "inmortalizar",
    "inmortalizada": "inmortalizar",
    "inmortalizadas": "inmortalizar",
    "inmortalizado": "inmortalizar",
    "inmortalizados": "inmortalizar",
    "inmortalizamos": "inmortalizar",
    "inmortalizan": "inmortalizar",
    "inmortalizando": "inmortalizar",
    "inmortalizara": "inmortalizar",
    "inmortalizarais": "inmortalizar",
    "inmortalizaran": "inmortalizar",
    "inmortalizaras": "inmortalizar",
    "inmortalizare": "inmortalizar",
    "inmortalizareis": "inmortalizar",
    "inmortalizaremos": "inmortalizar",
    "inmortalizaren": "inmortalizar",
    "inmortalizares": "inmortalizar",
    "inmortalizaron": "inmortalizar",
    "inmortalizará": "inmortalizar",
    "inmortalizarán": "inmortalizar",
    "inmortalizarás": "inmortalizar",
    "inmortalizaré": "inmortalizar",
    "inmortalizaréis": "inmortalizar",
    "inmortalizaría": "inmortalizar",
    "inmortalizaríais": "inmortalizar",
    "inmortalizaríamos": "inmortalizar",
    "inmortalizarían": "inmortalizar",
    "inmortalizarías": "inmortalizar",
    "inmortalizas": "inmortalizar",
    "inmortalizase": "inmortalizar",
    "inmortalizaseis": "inmortalizar",
    "inmortalizasen": "inmortalizar",
    "inmortalizases": "inmortalizar",
    "inmortalizaste": "inmortalizar",
    "inmortalizasteis": "inmortalizar",
    "inmortalizo": "inmortalizar",
    "inmortalizábamos": "inmortalizar",
    "inmortalizáis": "inmortalizar",
    "inmortalizáramos": "inmortalizar",
    "inmortalizáremos": "inmortalizar",
    "inmortalizásemos": "inmortalizar",
    "inmortalizó": "inmortalizar",
    "inmotivada": "inmotivado",
    "inmotivadas": "inmotivado",
    "inmotivados": "inmotivado",
    "inmovibles": "inmovible",
    "inmovilice": "inmovilizar",
    "inmovilicemos": "inmovilizar",
    "inmovilicen": "inmovilizar",
    "inmovilices": "inmovilizar",
    "inmovilicé": "inmovilizar",
    "inmovilicéis": "inmovilizar",
    "inmovilidades": "inmovilidad",
    "inmovilismos": "inmovilismo",
    "inmovilistas": "inmovilista",
    "inmoviliza": "inmovilizar",
    "inmovilizaba": "inmovilizar",
    "inmovilizabais": "inmovilizar",
    "inmovilizaban": "inmovilizar",
    "inmovilizabas": "inmovilizar",
    "inmovilizaciones": "inmovilización",
    "inmovilizad": "inmovilizar",
    "inmovilizada": "inmovilizar",
    "inmovilizadas": "inmovilizar",
    "inmovilizado": "inmovilizar",
    "inmovilizadora": "inmovilizador",
    "inmovilizadoras": "inmovilizador",
    "inmovilizadores": "inmovilizador",
    "inmovilizados": "inmovilizar",
    "inmovilizamos": "inmovilizar",
    "inmovilizan": "inmovilizar",
    "inmovilizando": "inmovilizar",
    "inmovilizara": "inmovilizar",
    "inmovilizarais": "inmovilizar",
    "inmovilizaran": "inmovilizar",
    "inmovilizaras": "inmovilizar",
    "inmovilizare": "inmovilizar",
    "inmovilizareis": "inmovilizar",
    "inmovilizaremos": "inmovilizar",
    "inmovilizaren": "inmovilizar",
    "inmovilizares": "inmovilizar",
    "inmovilizaron": "inmovilizar",
    "inmovilizará": "inmovilizar",
    "inmovilizarán": "inmovilizar",
    "inmovilizarás": "inmovilizar",
    "inmovilizaré": "inmovilizar",
    "inmovilizaréis": "inmovilizar",
    "inmovilizaría": "inmovilizar",
    "inmovilizaríais": "inmovilizar",
    "inmovilizaríamos": "inmovilizar",
    "inmovilizarían": "inmovilizar",
    "inmovilizarías": "inmovilizar",
    "inmovilizas": "inmovilizar",
    "inmovilizase": "inmovilizar",
    "inmovilizaseis": "inmovilizar",
    "inmovilizasen": "inmovilizar",
    "inmovilizases": "inmovilizar",
    "inmovilizaste": "inmovilizar",
    "inmovilizasteis": "inmovilizar",
    "inmovilizo": "inmovilizar",
    "inmovilizábamos": "inmovilizar",
    "inmovilizáis": "inmovilizar",
    "inmovilizáramos": "inmovilizar",
    "inmovilizáremos": "inmovilizar",
    "inmovilizásemos": "inmovilizar",
    "inmovilizó": "inmovilizar",
    "inmudables": "inmudable",
    "inmuebles": "inmueble",
    "inmunda": "inmundo",
    "inmundas": "inmundo",
    "inmundicias": "inmundicia",
    "inmundos": "inmundo",
    "inmunes": "inmune",
    "inmunice": "inmunizar",
    "inmunicemos": "inmunizar",
    "inmunicen": "inmunizar",
    "inmunices": "inmunizar",
    "inmunicé": "inmunizar",
    "inmunicéis": "inmunizar",
    "inmunidades": "inmunidad",
    "inmunitaria": "inmunitario",
    "inmunitarias": "inmunitario",
    "inmunitarios": "inmunitario",
    "inmuniza": "inmunizar",
    "inmunizaba": "inmunizar",
    "inmunizabais": "inmunizar",
    "inmunizaban": "inmunizar",
    "inmunizabas": "inmunizar",
    "inmunizaciones": "inmunización",
    "inmunizad": "inmunizar",
    "inmunizada": "inmunizar",
    "inmunizadas": "inmunizar",
    "inmunizado": "inmunizar",
    "inmunizadora": "inmunizador",
    "inmunizadoras": "inmunizador",
    "inmunizadores": "inmunizador",
    "inmunizados": "inmunizar",
    "inmunizamos": "inmunizar",
    "inmunizan": "inmunizar",
    "inmunizando": "inmunizar",
    "inmunizantes": "inmunizante",
    "inmunizara": "inmunizar",
    "inmunizarais": "inmunizar",
    "inmunizaran": "inmunizar",
    "inmunizaras": "inmunizar",
    "inmunizare": "inmunizar",
    "inmunizareis": "inmunizar",
    "inmunizaremos": "inmunizar",
    "inmunizaren": "inmunizar",
    "inmunizares": "inmunizar",
    "inmunizaron": "inmunizar",
    "inmunizará": "inmunizar",
    "inmunizarán": "inmunizar",
    "inmunizarás": "inmunizar",
    "inmunizaré": "inmunizar",
    "inmunizaréis": "inmunizar",
    "inmunizaría": "inmunizar",
    "inmunizaríais": "inmunizar",
    "inmunizaríamos": "inmunizar",
    "inmunizarían": "inmunizar",
    "inmunizarías": "inmunizar",
    "inmunizas": "inmunizar",
    "inmunizase": "inmunizar",
    "inmunizaseis": "inmunizar",
    "inmunizasen": "inmunizar",
    "inmunizases": "inmunizar",
    "inmunizaste": "inmunizar",
    "inmunizasteis": "inmunizar",
    "inmunizo": "inmunizar",
    "inmunizábamos": "inmunizar",
    "inmunizáis": "inmunizar",
    "inmunizáramos": "inmunizar",
    "inmunizáremos": "inmunizar",
    "inmunizásemos": "inmunizar",
    "inmunizó": "inmunizar",
    "inmunoabsorciones": "inmunoabsorción",
    "inmunocitos": "inmunocito",
    "inmunocompetencias": "inmunocompetencia",
    "inmunocompetentes": "inmunocompetente",
    "inmunocomplejos": "inmunocomplejo",
    "inmunoconglutinaciones": "inmunoconglutinación",
    "inmunoconglutininas": "inmunoconglutinina",
    "inmunodeficiencias": "inmunodeficiencia",
    "inmunodifusiones": "inmunodifusión",
    "inmunodominantes": "inmunodominante",
    "inmunoensayos": "inmunoensayo",
    "inmunoenzimática": "inmunoenzimático",
    "inmunoenzimáticas": "inmunoenzimático",
    "inmunoenzimáticos": "inmunoenzimático",
    "inmunofijaciones": "inmunofijación",
    "inmunofluorescencias": "inmunofluorescencia",
    "inmunogenicidades": "inmunogenicidad",
    "inmunoglobulinas": "inmunoglobulina",
    "inmunoglobulínica": "inmunoglobulínico",
    "inmunoglobulínicas": "inmunoglobulínico",
    "inmunoglobulínicos": "inmunoglobulínico",
    "inmunogénica": "inmunogénico",
    "inmunogénicas": "inmunogénico",
    "inmunogénicos": "inmunogénico",
    "inmunohematologías": "inmunohematología",
    "inmunoincompetencias": "inmunoincompetencia",
    "inmunologías": "inmunología",
    "inmunológica": "inmunológico",
    "inmunológicas": "inmunológico",
    "inmunológicos": "inmunológico",
    "inmunopatologías": "inmunopatología",
    "inmunopotenciaciones": "inmunopotenciación",
    "inmunoprecipitaciones": "inmunoprecipitación",
    "inmunoprofilaxias": "inmunoprofilaxia",
    "inmunoproteínas": "inmunoproteína",
    "inmunoquímicas": "inmunoquímica",
    "inmunorradiométrica": "inmunorradiométrico",
    "inmunorradiométricas": "inmunorradiométrico",
    "inmunorradiométricos": "inmunorradiométrico",
    "inmunorregulaciones": "inmunorregulación",
    "inmunorreguladora": "inmunorregulador",
    "inmunorreguladoras": "inmunorregulador",
    "inmunorreguladores": "inmunorregulador",
    "inmunosupresiones": "inmunosupresión",
    "inmunosupresora": "inmunosupresor",
    "inmunosupresoras": "inmunosupresor",
    "inmunosupresores": "inmunosupresor",
    "inmunoterapias": "inmunoterapia",
    "inmunotolerancias": "inmunotolerancia",
    "inmunotoxinas": "inmunotoxina",
    "inmunotransferencias": "inmunotransferencia",
    "inmunotransfusiones": "inmunotransfusión",
    "inmunógena": "inmunógeno",
    "inmunógenas": "inmunógeno",
    "inmunógenos": "inmunógeno",
    "inmunóloga": "inmunólogo",
    "inmunólogas": "inmunólogo",
    "inmunólogos": "inmunólogo",
    "inmuta": "inmutar",
    "inmutaba": "inmutar",
    "inmutabais": "inmutar",
    "inmutaban": "inmutar",
    "inmutabas": "inmutar",
    "inmutabilidades": "inmutabilidad",
    "inmutables": "inmutable",
    "inmutaciones": "inmutación",
    "inmutad": "inmutar",
    "inmutada": "inmutar",
    "inmutadas": "inmutar",
    "inmutado": "inmutar",
    "inmutados": "inmutar",
    "inmutamos": "inmutar",
    "inmutan": "inmutar",
    "inmutando": "inmutar",
    "inmutara": "inmutar",
    "inmutarais": "inmutar",
    "inmutaran": "inmutar",
    "inmutaras": "inmutar",
    "inmutare": "inmutar",
    "inmutareis": "inmutar",
    "inmutaremos": "inmutar",
    "inmutaren": "inmutar",
    "inmutares": "inmutar",
    "inmutaron": "inmutar",
    "inmutará": "inmutar",
    "inmutarán": "inmutar",
    "inmutarás": "inmutar",
    "inmutaré": "inmutar",
    "inmutaréis": "inmutar",
    "inmutaría": "inmutar",
    "inmutaríais": "inmutar",
    "inmutaríamos": "inmutar",
    "inmutarían": "inmutar",
    "inmutarías": "inmutar",
    "inmutas": "inmutar",
    "inmutase": "inmutar",
    "inmutaseis": "inmutar",
    "inmutasen": "inmutar",
    "inmutases": "inmutar",
    "inmutaste": "inmutar",
    "inmutasteis": "inmutar",
    "inmutativa": "inmutativo",
    "inmutativas": "inmutativo",
    "inmutativos": "inmutativo",
    "inmute": "inmutar",
    "inmutemos": "inmutar",
    "inmuten": "inmutar",
    "inmutes": "inmutar",
    "inmuto": "inmutar",
    "inmutábamos": "inmutar",
    "inmutáis": "inmutar",
    "inmutáramos": "inmutar",
    "inmutáremos": "inmutar",
    "inmutásemos": "inmutar",
    "inmuté": "inmutar",
    "inmutéis": "inmutar",
    "inmutó": "inmutar",
    "inmóviles": "inmóvil",
    "innata": "innato",
    "innatas": "innato",
    "innatismos": "innatismo",
    "innatos": "innato",
    "innaturales": "innatural",
    "innavegables": "innavegable",
    "innecesaria": "innecesario",
    "innecesarias": "innecesario",
    "innecesariedades": "innecesariedad",
    "innecesarios": "innecesario",
    "innegables": "innegable",
    "innobles": "innoble",
    "innocua": "innocuo",
    "innocuas": "innocuo",
    "innocuos": "innocuo",
    "innombrables": "innombrable",
    "innominables": "innominable",
    "innominada": "innominado",
    "innominadas": "innominado",
    "innominados": "innominado",
    "innova": "innovar",
    "innovaba": "innovar",
    "innovabais": "innovar",
    "innovaban": "innovar",
    "innovabas": "innovar",
    "innovaciones": "innovación",
    "innovad": "innovar",
    "innovada": "innovar",
    "innovadas": "innovar",
    "innovado": "innovar",
    "innovadora": "innovador",
    "innovadoras": "innovador",
    "innovadores": "innovador",
    "innovados": "innovar",
    "innovamos": "innovar",
    "innovan": "innovar",
    "innovando": "innovar",
    "innovara": "innovar",
    "innovarais": "innovar",
    "innovaran": "innovar",
    "innovaras": "innovar",
    "innovare": "innovar",
    "innovareis": "innovar",
    "innovaremos": "innovar",
    "innovaren": "innovar",
    "innovares": "innovar",
    "innovaron": "innovar",
    "innovará": "innovar",
    "innovarán": "innovar",
    "innovarás": "innovar",
    "innovaré": "innovar",
    "innovaréis": "innovar",
    "innovaría": "innovar",
    "innovaríais": "innovar",
    "innovaríamos": "innovar",
    "innovarían": "innovar",
    "innovarías": "innovar",
    "innovas": "innovar",
    "innovase": "innovar",
    "innovaseis": "innovar",
    "innovasen": "innovar",
    "innovases": "innovar",
    "innovaste": "innovar",
    "innovasteis": "innovar",
    "innove": "innovar",
    "innovemos": "innovar",
    "innoven": "innovar",
    "innoves": "innovar",
    "innovo": "innovar",
    "innovábamos": "innovar",
    "innováis": "innovar",
    "innováramos": "innovar",
    "innováremos": "innovar",
    "innovásemos": "innovar",
    "innové": "innovar",
    "innovéis": "innovar",
    "innovó": "innovar",
    "innumerabilidades": "innumerabilidad",
    "innumerables": "innumerable",
    "innúmera": "innúmero",
    "innúmeras": "innúmero",
    "innúmeros": "innúmero",
    "inobediencias": "inobediencia",
    "inobedientes": "inobediente",
    "inobservables": "inobservable",
    "inobservancias": "inobservancia",
    "inobservantes": "inobservante",
    "inocencias": "inocencia",
    "inocentadas": "inocentada",
    "inocentes": "inocente",
    "inocentona": "inocentón",
    "inocentonas": "inocentón",
    "inocentones": "inocentón",
    "inoclusiones": "inoclusión",
    "inocua": "inocuo",
    "inocuas": "inocuo",
    "inocuidades": "inocuidad",
    "inocula": "inocular",
    "inoculaba": "inocular",
    "inoculabais": "inocular",
    "inoculaban": "inocular",
    "inoculabas": "inocular",
    "inoculables": "inoculable",
    "inoculaciones": "inoculación",
    "inoculad": "inocular",
    "inoculada": "inocular",
    "inoculadas": "inocular",
    "inoculado": "inocular",
    "inoculadora": "inoculador",
    "inoculadoras": "inoculador",
    "inoculadores": "inoculador",
    "inoculados": "inocular",
    "inoculamos": "inocular",
    "inoculan": "inocular",
    "inoculando": "inocular",
    "inoculara": "inocular",
    "inocularais": "inocular",
    "inocularan": "inocular",
    "inocularas": "inocular",
    "inoculare": "inocular",
    "inoculareis": "inocular",
    "inocularemos": "inocular",
    "inocularen": "inocular",
    "inoculares": "inocular",
    "inocularon": "inocular",
    "inoculará": "inocular",
    "inocularán": "inocular",
    "inocularás": "inocular",
    "inocularé": "inocular",
    "inocularéis": "inocular",
    "inocularía": "inocular",
    "inocularíais": "inocular",
    "inocularíamos": "inocular",
    "inocularían": "inocular",
    "inocularías": "inocular",
    "inoculas": "inocular",
    "inoculase": "inocular",
    "inoculaseis": "inocular",
    "inoculasen": "inocular",
    "inoculases": "inocular",
    "inoculaste": "inocular",
    "inoculasteis": "inocular",
    "inocule": "inocular",
    "inoculemos": "inocular",
    "inoculen": "inocular",
    "inocules": "inocular",
    "inoculo": "inocular",
    "inocultables": "inocultable",
    "inoculábamos": "inocular",
    "inoculáis": "inocular",
    "inoculáramos": "inocular",
    "inoculáremos": "inocular",
    "inoculásemos": "inocular",
    "inoculé": "inocular",
    "inoculéis": "inocular",
    "inoculó": "inocular",
    "inocuos": "inocuo",
    "inodora": "inodoro",
    "inodoras": "inodoro",
    "inodoros": "inodoro",
    "inofensiva": "inofensivo",
    "inofensivas": "inofensivo",
    "inofensivos": "inofensivo",
    "inoficiosa": "inoficioso",
    "inoficiosas": "inoficioso",
    "inoficiosidades": "inoficiosidad",
    "inoficiosos": "inoficioso",
    "inolvidables": "inolvidable",
    "inoperables": "inoperable",
    "inoperancias": "inoperancia",
    "inoperantes": "inoperante",
    "inoperativa": "inoperativo",
    "inoperativas": "inoperativo",
    "inoperativos": "inoperativo",
    "inopes": "inope",
    "inopias": "inopia",
    "inopinables": "inopinable",
    "inopinada": "inopinado",
    "inopinadas": "inopinado",
    "inopinados": "inopinado",
    "inoportuna": "inoportuno",
    "inoportunas": "inoportuno",
    "inoportunidades": "inoportunidad",
    "inoportunos": "inoportuno",
    "inordenada": "inordenado",
    "inordenadas": "inordenado",
    "inordenados": "inordenado",
    "inorgánica": "inorgánico",
    "inorgánicas": "inorgánico",
    "inorgánicos": "inorgánico",
    "inosilicatos": "inosilicato",
    "inosinas": "inosina",
    "inositoles": "inositol",
    "inotrópica": "inotrópico",
    "inotrópicas": "inotrópico",
    "inotrópicos": "inotrópico",
    "inoxidables": "inoxidable",
    "inputs": "input",
    "inquebrantables": "inquebrantable",
    "inquiera": "inquirir",
    "inquieran": "inquirir",
    "inquieras": "inquirir",
    "inquiere": "inquirir",
    "inquieren": "inquirir",
    "inquieres": "inquirir",
    "inquiero": "inquirir",
    "inquieta": "inquieto",
    "inquietaba": "inquietar",
    "inquietabais": "inquietar",
    "inquietaban": "inquietar",
    "inquietabas": "inquietar",
    "inquietad": "inquietar",
    "inquietada": "inquietar",
    "inquietadas": "inquietar",
    "inquietado": "inquietar",
    "inquietadora": "inquietador",
    "inquietadoras": "inquietador",
    "inquietadores": "inquietador",
    "inquietados": "inquietar",
    "inquietamos": "inquietar",
    "inquietan": "inquietar",
    "inquietando": "inquietar",
    "inquietantes": "inquietante",
    "inquietara": "inquietar",
    "inquietarais": "inquietar",
    "inquietaran": "inquietar",
    "inquietaras": "inquietar",
    "inquietare": "inquietar",
    "inquietareis": "inquietar",
    "inquietaremos": "inquietar",
    "inquietaren": "inquietar",
    "inquietares": "inquietar",
    "inquietaron": "inquietar",
    "inquietará": "inquietar",
    "inquietarán": "inquietar",
    "inquietarás": "inquietar",
    "inquietaré": "inquietar",
    "inquietaréis": "inquietar",
    "inquietaría": "inquietar",
    "inquietaríais": "inquietar",
    "inquietaríamos": "inquietar",
    "inquietarían": "inquietar",
    "inquietarías": "inquietar",
    "inquietas": "inquieto",
    "inquietase": "inquietar",
    "inquietaseis": "inquietar",
    "inquietasen": "inquietar",
    "inquietases": "inquietar",
    "inquietaste": "inquietar",
    "inquietasteis": "inquietar",
    "inquiete": "inquietar",
    "inquietemos": "inquietar",
    "inquieten": "inquietar",
    "inquietes": "inquietar",
    "inquieto": "inquietar",
    "inquietos": "inquieto",
    "inquietudes": "inquietud",
    "inquietábamos": "inquietar",
    "inquietáis": "inquietar",
    "inquietáramos": "inquietar",
    "inquietáremos": "inquietar",
    "inquietásemos": "inquietar",
    "inquieté": "inquietar",
    "inquietéis": "inquietar",
    "inquietó": "inquietar",
    "inquilina": "inquilino",
    "inquilinas": "inquilino",
    "inquilinatos": "inquilinato",
    "inquilinos": "inquilino",
    "inquinas": "inquina",
    "inquiramos": "inquirir",
    "inquirid": "inquirir",
    "inquirida": "inquirir",
    "inquiridas": "inquirir",
    "inquirido": "inquirir",
    "inquiridora": "inquiridor",
    "inquiridoras": "inquiridor",
    "inquiridores": "inquiridor",
    "inquiridos": "inquirir",
    "inquiriendo": "inquirir",
    "inquiriera": "inquirir",
    "inquirierais": "inquirir",
    "inquirieran": "inquirir",
    "inquirieras": "inquirir",
    "inquiriere": "inquirir",
    "inquiriereis": "inquirir",
    "inquirieren": "inquirir",
    "inquirieres": "inquirir",
    "inquirieron": "inquirir",
    "inquiriese": "inquirir",
    "inquirieseis": "inquirir",
    "inquiriesen": "inquirir",
    "inquirieses": "inquirir",
    "inquirimos": "inquirir",
    "inquiriremos": "inquirir",
    "inquirirá": "inquirir",
    "inquirirán": "inquirir",
    "inquirirás": "inquirir",
    "inquiriré": "inquirir",
    "inquiriréis": "inquirir",
    "inquiriría": "inquirir",
    "inquiriríais": "inquirir",
    "inquiriríamos": "inquirir",
    "inquirirían": "inquirir",
    "inquirirías": "inquirir",
    "inquiriste": "inquirir",
    "inquiristeis": "inquirir",
    "inquiriéramos": "inquirir",
    "inquiriéremos": "inquirir",
    "inquiriésemos": "inquirir",
    "inquirió": "inquirir",
    "inquiráis": "inquirir",
    "inquirí": "inquirir",
    "inquiría": "inquirir",
    "inquiríais": "inquirir",
    "inquiríamos": "inquirir",
    "inquirían": "inquirir",
    "inquirías": "inquirir",
    "inquirís": "inquirir",
    "inquisiciones": "inquisición",
    "inquisidora": "inquisidor",
    "inquisidoras": "inquisidor",
    "inquisidores": "inquisidor",
    "inquisiones": "inquisión",
    "inquisitiva": "inquisitivo",
    "inquisitivas": "inquisitivo",
    "inquisitivos": "inquisitivo",
    "inquisitoria": "inquisitorio",
    "inquisitoriales": "inquisitorial",
    "inquisitorias": "inquisitorio",
    "inquisitorios": "inquisitorio",
    "insaciabilidades": "insaciabilidad",
    "insaciables": "insaciable",
    "insacula": "insacular",
    "insaculaba": "insacular",
    "insaculabais": "insacular",
    "insaculaban": "insacular",
    "insaculabas": "insacular",
    "insaculaciones": "insaculación",
    "insaculad": "insacular",
    "insaculada": "insacular",
    "insaculadas": "insacular",
    "insaculado": "insacular",
    "insaculados": "insacular",
    "insaculamos": "insacular",
    "insaculan": "insacular",
    "insaculando": "insacular",
    "insaculara": "insacular",
    "insacularais": "insacular",
    "insacularan": "insacular",
    "insacularas": "insacular",
    "insaculare": "insacular",
    "insaculareis": "insacular",
    "insacularemos": "insacular",
    "insacularen": "insacular",
    "insaculares": "insacular",
    "insacularon": "insacular",
    "insaculará": "insacular",
    "insacularán": "insacular",
    "insacularás": "insacular",
    "insacularé": "insacular",
    "insacularéis": "insacular",
    "insacularía": "insacular",
    "insacularíais": "insacular",
    "insacularíamos": "insacular",
    "insacularían": "insacular",
    "insacularías": "insacular",
    "insaculas": "insacular",
    "insaculase": "insacular",
    "insaculaseis": "insacular",
    "insaculasen": "insacular",
    "insaculases": "insacular",
    "insaculaste": "insacular",
    "insaculasteis": "insacular",
    "insacule": "insacular",
    "insaculemos": "insacular",
    "insaculen": "insacular",
    "insacules": "insacular",
    "insaculo": "insacular",
    "insaculábamos": "insacular",
    "insaculáis": "insacular",
    "insaculáramos": "insacular",
    "insaculáremos": "insacular",
    "insaculásemos": "insacular",
    "insaculé": "insacular",
    "insaculéis": "insacular",
    "insaculó": "insacular",
    "insaliva": "insalivar",
    "insalivaba": "insalivar",
    "insalivabais": "insalivar",
    "insalivaban": "insalivar",
    "insalivabas": "insalivar",
    "insalivaciones": "insalivación",
    "insalivad": "insalivar",
    "insalivada": "insalivar",
    "insalivadas": "insalivar",
    "insalivado": "insalivar",
    "insalivados": "insalivar",
    "insalivamos": "insalivar",
    "insalivan": "insalivar",
    "insalivando": "insalivar",
    "insalivara": "insalivar",
    "insalivarais": "insalivar",
    "insalivaran": "insalivar",
    "insalivaras": "insalivar",
    "insalivare": "insalivar",
    "insalivareis": "insalivar",
    "insalivaremos": "insalivar",
    "insalivaren": "insalivar",
    "insalivares": "insalivar",
    "insalivaron": "insalivar",
    "insalivará": "insalivar",
    "insalivarán": "insalivar",
    "insalivarás": "insalivar",
    "insalivaré": "insalivar",
    "insalivaréis": "insalivar",
    "insalivaría": "insalivar",
    "insalivaríais": "insalivar",
    "insalivaríamos": "insalivar",
    "insalivarían": "insalivar",
    "insalivarías": "insalivar",
    "insalivas": "insalivar",
    "insalivase": "insalivar",
    "insalivaseis": "insalivar",
    "insalivasen": "insalivar",
    "insalivases": "insalivar",
    "insalivaste": "insalivar",
    "insalivasteis": "insalivar",
    "insalive": "insalivar",
    "insalivemos": "insalivar",
    "insaliven": "insalivar",
    "insalives": "insalivar",
    "insalivo": "insalivar",
    "insalivábamos": "insalivar",
    "insaliváis": "insalivar",
    "insaliváramos": "insalivar",
    "insaliváremos": "insalivar",
    "insalivásemos": "insalivar",
    "insalivé": "insalivar",
    "insalivéis": "insalivar",
    "insalivó": "insalivar",
    "insalubres": "insalubre",
    "insalubridades": "insalubridad",
    "insalvables": "insalvable",
    "insana": "insano",
    "insanables": "insanable",
    "insanas": "insano",
    "insanias": "insania",
    "insanos": "insano",
    "insanías": "insanía",
    "insatisfacciones": "insatisfacción",
    "insatisfactoria": "insatisfactorio",
    "insatisfactorias": "insatisfactorio",
    "insatisfactorios": "insatisfactorio",
    "insatisfecha": "insatisfecho",
    "insatisfechas": "insatisfecho",
    "insatisfechos": "insatisfecho",
    "insaturaciones": "insaturación",
    "insaturada": "insaturado",
    "insaturadas": "insaturado",
    "insaturados": "insaturado",
    "inscriba": "inscribir",
    "inscribamos": "inscribir",
    "inscriban": "inscribir",
    "inscribas": "inscribir",
    "inscribe": "inscribir",
    "inscriben": "inscribir",
    "inscribes": "inscribir",
    "inscribibles": "inscribible",
    "inscribid": "inscribir",
    "inscribiendo": "inscribir",
    "inscribiera": "inscribir",
    "inscribierais": "inscribir",
    "inscribieran": "inscribir",
    "inscribieras": "inscribir",
    "inscribiere": "inscribir",
    "inscribiereis": "inscribir",
    "inscribieren": "inscribir",
    "inscribieres": "inscribir",
    "inscribieron": "inscribir",
    "inscribiese": "inscribir",
    "inscribieseis": "inscribir",
    "inscribiesen": "inscribir",
    "inscribieses": "inscribir",
    "inscribimos": "inscribir",
    "inscribiremos": "inscribir",
    "inscribirá": "inscribir",
    "inscribirán": "inscribir",
    "inscribirás": "inscribir",
    "inscribiré": "inscribir",
    "inscribiréis": "inscribir",
    "inscribiría": "inscribir",
    "inscribiríais": "inscribir",
    "inscribiríamos": "inscribir",
    "inscribirían": "inscribir",
    "inscribirías": "inscribir",
    "inscribiste": "inscribir",
    "inscribisteis": "inscribir",
    "inscribiéramos": "inscribir",
    "inscribiéremos": "inscribir",
    "inscribiésemos": "inscribir",
    "inscribió": "inscribir",
    "inscribo": "inscribir",
    "inscribáis": "inscribir",
    "inscribí": "inscribir",
    "inscribía": "inscribir",
    "inscribíais": "inscribir",
    "inscribíamos": "inscribir",
    "inscribían": "inscribir",
    "inscribías": "inscribir",
    "inscribís": "inscribir",
    "inscripciones": "inscripción",
    "inscrita": "inscribir",
    "inscritas": "inscribir",
    "inscrito": "inscribir",
    "inscritos": "inscribir",
    "insculturas": "inscultura",
    "insecables": "insecable",
    "insecticidas": "insecticida",
    "insectiles": "insectil",
    "insectos": "insecto",
    "insectívora": "insectívoro",
    "insectívoras": "insectívoro",
    "insectívoros": "insectívoro",
    "insegura": "inseguro",
    "inseguras": "inseguro",
    "inseguridades": "inseguridad",
    "inseguros": "inseguro",
    "inselbergs": "inselberg",
    "insemina": "inseminar",
    "inseminaba": "inseminar",
    "inseminabais": "inseminar",
    "inseminaban": "inseminar",
    "inseminabas": "inseminar",
    "inseminaciones": "inseminación",
    "inseminaciónes": "inseminación",
    "inseminad": "inseminar",
    "inseminada": "inseminar",
    "inseminadas": "inseminar",
    "inseminado": "inseminar",
    "inseminados": "inseminar",
    "inseminamos": "inseminar",
    "inseminan": "inseminar",
    "inseminando": "inseminar",
    "inseminara": "inseminar",
    "inseminarais": "inseminar",
    "inseminaran": "inseminar",
    "inseminaras": "inseminar",
    "inseminare": "inseminar",
    "inseminareis": "inseminar",
    "inseminaremos": "inseminar",
    "inseminaren": "inseminar",
    "inseminares": "inseminar",
    "inseminaron": "inseminar",
    "inseminará": "inseminar",
    "inseminarán": "inseminar",
    "inseminarás": "inseminar",
    "inseminaré": "inseminar",
    "inseminaréis": "inseminar",
    "inseminaría": "inseminar",
    "inseminaríais": "inseminar",
    "inseminaríamos": "inseminar",
    "inseminarían": "inseminar",
    "inseminarías": "inseminar",
    "inseminas": "inseminar",
    "inseminase": "inseminar",
    "inseminaseis": "inseminar",
    "inseminasen": "inseminar",
    "inseminases": "inseminar",
    "inseminaste": "inseminar",
    "inseminasteis": "inseminar",
    "insemine": "inseminar",
    "inseminemos": "inseminar",
    "inseminen": "inseminar",
    "insemines": "inseminar",
    "insemino": "inseminar",
    "inseminábamos": "inseminar",
    "insemináis": "inseminar",
    "insemináramos": "inseminar",
    "insemináremos": "inseminar",
    "inseminásemos": "inseminar",
    "inseminé": "inseminar",
    "inseminéis": "inseminar",
    "inseminó": "inseminar",
    "insenescencias": "insenescencia",
    "insenescentes": "insenescente",
    "insensata": "insensato",
    "insensatas": "insensato",
    "insensateces": "insensatez",
    "insensatos": "insensato",
    "insensibilice": "insensibilizar",
    "insensibilicemos": "insensibilizar",
    "insensibilicen": "insensibilizar",
    "insensibilices": "insensibilizar",
    "insensibilicé": "insensibilizar",
    "insensibilicéis": "insensibilizar",
    "insensibilidades": "insensibilidad",
    "insensibiliza": "insensibilizar",
    "insensibilizaba": "insensibilizar",
    "insensibilizabais": "insensibilizar",
    "insensibilizaban": "insensibilizar",
    "insensibilizabas": "insensibilizar",
    "insensibilizaciones": "insensibilización",
    "insensibilizad": "insensibilizar",
    "insensibilizada": "insensibilizar",
    "insensibilizadas": "insensibilizar",
    "insensibilizado": "insensibilizar",
    "insensibilizados": "insensibilizar",
    "insensibilizamos": "insensibilizar",
    "insensibilizan": "insensibilizar",
    "insensibilizando": "insensibilizar",
    "insensibilizara": "insensibilizar",
    "insensibilizarais": "insensibilizar",
    "insensibilizaran": "insensibilizar",
    "insensibilizaras": "insensibilizar",
    "insensibilizare": "insensibilizar",
    "insensibilizareis": "insensibilizar",
    "insensibilizaremos": "insensibilizar",
    "insensibilizaren": "insensibilizar",
    "insensibilizares": "insensibilizar",
    "insensibilizaron": "insensibilizar",
    "insensibilizará": "insensibilizar",
    "insensibilizarán": "insensibilizar",
    "insensibilizarás": "insensibilizar",
    "insensibilizaré": "insensibilizar",
    "insensibilizaréis": "insensibilizar",
    "insensibilizaría": "insensibilizar",
    "insensibilizaríais": "insensibilizar",
    "insensibilizaríamos": "insensibilizar",
    "insensibilizarían": "insensibilizar",
    "insensibilizarías": "insensibilizar",
    "insensibilizas": "insensibilizar",
    "insensibilizase": "insensibilizar",
    "insensibilizaseis": "insensibilizar",
    "insensibilizasen": "insensibilizar",
    "insensibilizases": "insensibilizar",
    "insensibilizaste": "insensibilizar",
    "insensibilizasteis": "insensibilizar",
    "insensibilizo": "insensibilizar",
    "insensibilizábamos": "insensibilizar",
    "insensibilizáis": "insensibilizar",
    "insensibilizáramos": "insensibilizar",
    "insensibilizáremos": "insensibilizar",
    "insensibilizásemos": "insensibilizar",
    "insensibilizó": "insensibilizar",
    "insensibles": "insensible",
    "inseparabilidades": "inseparabilidad",
    "inseparables": "inseparable",
    "insepulta": "insepulto",
    "insepultas": "insepulto",
    "insepultos": "insepulto",
    "inserciones": "inserción",
    "inserid": "inserir",
    "inserida": "inserir",
    "inseridas": "inserir",
    "inserido": "inserir",
    "inseridos": "inserir",
    "inserimos": "inserir",
    "inseriremos": "inserir",
    "inserirá": "inserir",
    "inserirán": "inserir",
    "inserirás": "inserir",
    "inseriré": "inserir",
    "inseriréis": "inserir",
    "inseriría": "inserir",
    "inseriríais": "inserir",
    "inseriríamos": "inserir",
    "inserirían": "inserir",
    "inserirías": "inserir",
    "inseriste": "inserir",
    "inseristeis": "inserir",
    "inserta": "inserto",
    "insertaba": "insertar",
    "insertabais": "insertar",
    "insertaban": "insertar",
    "insertabas": "insertar",
    "insertad": "insertar",
    "insertada": "insertar",
    "insertadas": "insertar",
    "insertado": "insertar",
    "insertados": "insertar",
    "insertamos": "insertar",
    "insertan": "insertar",
    "insertando": "insertar",
    "insertara": "insertar",
    "insertarais": "insertar",
    "insertaran": "insertar",
    "insertaras": "insertar",
    "insertare": "insertar",
    "insertareis": "insertar",
    "insertaremos": "insertar",
    "insertaren": "insertar",
    "insertares": "insertar",
    "insertaron": "insertar",
    "insertará": "insertar",
    "insertarán": "insertar",
    "insertarás": "insertar",
    "insertaré": "insertar",
    "insertaréis": "insertar",
    "insertaría": "insertar",
    "insertaríais": "insertar",
    "insertaríamos": "insertar",
    "insertarían": "insertar",
    "insertarías": "insertar",
    "insertas": "inserto",
    "insertase": "insertar",
    "insertaseis": "insertar",
    "insertasen": "insertar",
    "insertases": "insertar",
    "insertaste": "insertar",
    "insertasteis": "insertar",
    "inserte": "insertar",
    "insertemos": "insertar",
    "inserten": "insertar",
    "insertes": "insertar",
    "inserto": "insertar",
    "insertos": "inserto",
    "insertábamos": "insertar",
    "insertáis": "insertar",
    "insertáramos": "insertar",
    "insertáremos": "insertar",
    "insertásemos": "insertar",
    "inserté": "insertar",
    "insertéis": "insertar",
    "insertó": "insertar",
    "inservibles": "inservible",
    "inserí": "inserir",
    "insería": "inserir",
    "inseríais": "inserir",
    "inseríamos": "inserir",
    "inserían": "inserir",
    "inserías": "inserir",
    "inserís": "inserir",
    "insidia": "insidiar",
    "insidiaba": "insidiar",
    "insidiabais": "insidiar",
    "insidiaban": "insidiar",
    "insidiabas": "insidiar",
    "insidiad": "insidiar",
    "insidiada": "insidiar",
    "insidiadas": "insidiar",
    "insidiado": "insidiar",
    "insidiadora": "insidiador",
    "insidiadoras": "insidiador",
    "insidiadores": "insidiador",
    "insidiados": "insidiar",
    "insidiamos": "insidiar",
    "insidian": "insidiar",
    "insidiando": "insidiar",
    "insidiara": "insidiar",
    "insidiarais": "insidiar",
    "insidiaran": "insidiar",
    "insidiaras": "insidiar",
    "insidiare": "insidiar",
    "insidiareis": "insidiar",
    "insidiaremos": "insidiar",
    "insidiaren": "insidiar",
    "insidiares": "insidiar",
    "insidiaron": "insidiar",
    "insidiará": "insidiar",
    "insidiarán": "insidiar",
    "insidiarás": "insidiar",
    "insidiaré": "insidiar",
    "insidiaréis": "insidiar",
    "insidiaría": "insidiar",
    "insidiaríais": "insidiar",
    "insidiaríamos": "insidiar",
    "insidiarían": "insidiar",
    "insidiarías": "insidiar",
    "insidias": "insidiar",
    "insidiase": "insidiar",
    "insidiaseis": "insidiar",
    "insidiasen": "insidiar",
    "insidiases": "insidiar",
    "insidiaste": "insidiar",
    "insidiasteis": "insidiar",
    "insidie": "insidiar",
    "insidiemos": "insidiar",
    "insidien": "insidiar",
    "insidies": "insidiar",
    "insidio": "insidiar",
    "insidiosa": "insidioso",
    "insidiosas": "insidioso",
    "insidiosos": "insidioso",
    "insidiábamos": "insidiar",
    "insidiáis": "insidiar",
    "insidiáramos": "insidiar",
    "insidiáremos": "insidiar",
    "insidiásemos": "insidiar",
    "insidié": "insidiar",
    "insidiéis": "insidiar",
    "insidió": "insidiar",
    "insiera": "inserir",
    "insieran": "inserir",
    "insieras": "inserir",
    "insiere": "inserir",
    "insieren": "inserir",
    "insieres": "inserir",
    "insiero": "inserir",
    "insights": "insight",
    "insignes": "insigne",
    "insignias": "insignia",
    "insignificancias": "insignificancia",
    "insignificantes": "insignificante",
    "insincera": "insincero",
    "insinceras": "insincero",
    "insinceridades": "insinceridad",
    "insinceros": "insincero",
    "insinuaba": "insinuar",
    "insinuabais": "insinuar",
    "insinuaban": "insinuar",
    "insinuabas": "insinuar",
    "insinuaciones": "insinuación",
    "insinuaciónes": "insinuación",
    "insinuad": "insinuar",
    "insinuada": "insinuar",
    "insinuadas": "insinuar",
    "insinuado": "insinuar",
    "insinuadora": "insinuador",
    "insinuadoras": "insinuador",
    "insinuadores": "insinuador",
    "insinuados": "insinuar",
    "insinuamos": "insinuar",
    "insinuando": "insinuar",
    "insinuantes": "insinuante",
    "insinuara": "insinuar",
    "insinuarais": "insinuar",
    "insinuaran": "insinuar",
    "insinuaras": "insinuar",
    "insinuare": "insinuar",
    "insinuareis": "insinuar",
    "insinuaremos": "insinuar",
    "insinuaren": "insinuar",
    "insinuares": "insinuar",
    "insinuaron": "insinuar",
    "insinuará": "insinuar",
    "insinuarán": "insinuar",
    "insinuarás": "insinuar",
    "insinuaré": "insinuar",
    "insinuaréis": "insinuar",
    "insinuaría": "insinuar",
    "insinuaríais": "insinuar",
    "insinuaríamos": "insinuar",
    "insinuarían": "insinuar",
    "insinuarías": "insinuar",
    "insinuase": "insinuar",
    "insinuaseis": "insinuar",
    "insinuasen": "insinuar",
    "insinuases": "insinuar",
    "insinuaste": "insinuar",
    "insinuasteis": "insinuar",
    "insinuativa": "insinuativo",
    "insinuativas": "insinuativo",
    "insinuativos": "insinuativo",
    "insinuemos": "insinuar",
    "insinuábamos": "insinuar",
    "insinuáis": "insinuar",
    "insinuáramos": "insinuar",
    "insinuáremos": "insinuar",
    "insinuásemos": "insinuar",
    "insinué": "insinuar",
    "insinuéis": "insinuar",
    "insinuó": "insinuar",
    "insinúa": "insinuar",
    "insinúan": "insinuar",
    "insinúas": "insinuar",
    "insinúe": "insinuar",
    "insinúen": "insinuar",
    "insinúes": "insinuar",
    "insinúo": "insinuar",
    "insipideces": "insipidez",
    "insipiencias": "insipiencia",
    "insipientes": "insipiente",
    "insiramos": "inserir",
    "insiriendo": "inserir",
    "insiriera": "inserir",
    "insirierais": "inserir",
    "insirieran": "inserir",
    "insirieras": "inserir",
    "insiriere": "inserir",
    "insiriereis": "inserir",
    "insirieren": "inserir",
    "insirieres": "inserir",
    "insirieron": "inserir",
    "insiriese": "inserir",
    "insirieseis": "inserir",
    "insiriesen": "inserir",
    "insirieses": "inserir",
    "insiriéramos": "inserir",
    "insiriéremos": "inserir",
    "insiriésemos": "inserir",
    "insirió": "inserir",
    "insiráis": "inserir",
    "insista": "insistir",
    "insistamos": "insistir",
    "insistan": "insistir",
    "insistas": "insistir",
    "insiste": "insistir",
    "insisten": "insistir",
    "insistencias": "insistencia",
    "insistentes": "insistente",
    "insistes": "insistir",
    "insistid": "insistir",
    "insistida": "insistir",
    "insistidas": "insistir",
    "insistido": "insistir",
    "insistidos": "insistir",
    "insistiendo": "insistir",
    "insistiera": "insistir",
    "insistierais": "insistir",
    "insistieran": "insistir",
    "insistieras": "insistir",
    "insistiere": "insistir",
    "insistiereis": "insistir",
    "insistieren": "insistir",
    "insistieres": "insistir",
    "insistieron": "insistir",
    "insistiese": "insistir",
    "insistieseis": "insistir",
    "insistiesen": "insistir",
    "insistieses": "insistir",
    "insistimos": "insistir",
    "insistiremos": "insistir",
    "insistirá": "insistir",
    "insistirán": "insistir",
    "insistirás": "insistir",
    "insistiré": "insistir",
    "insistiréis": "insistir",
    "insistiría": "insistir",
    "insistiríais": "insistir",
    "insistiríamos": "insistir",
    "insistirían": "insistir",
    "insistirías": "insistir",
    "insististe": "insistir",
    "insististeis": "insistir",
    "insistiéramos": "insistir",
    "insistiéremos": "insistir",
    "insistiésemos": "insistir",
    "insistió": "insistir",
    "insisto": "insistir",
    "insistáis": "insistir",
    "insistí": "insistir",
    "insistía": "insistir",
    "insistíais": "insistir",
    "insistíamos": "insistir",
    "insistían": "insistir",
    "insistías": "insistir",
    "insistís": "insistir",
    "insobornables": "insobornable",
    "insociabilidades": "insociabilidad",
    "insociables": "insociable",
    "insociales": "insocial",
    "insola": "insolar",
    "insolaba": "insolar",
    "insolabais": "insolar",
    "insolaban": "insolar",
    "insolabas": "insolar",
    "insolaciones": "insolación",
    "insolad": "insolar",
    "insolada": "insolar",
    "insoladas": "insolar",
    "insolado": "insolar",
    "insolados": "insolar",
    "insolamos": "insolar",
    "insolan": "insolar",
    "insolando": "insolar",
    "insolara": "insolar",
    "insolarais": "insolar",
    "insolaran": "insolar",
    "insolaras": "insolar",
    "insolare": "insolar",
    "insolareis": "insolar",
    "insolaremos": "insolar",
    "insolaren": "insolar",
    "insolares": "insolar",
    "insolaron": "insolar",
    "insolará": "insolar",
    "insolarán": "insolar",
    "insolarás": "insolar",
    "insolaré": "insolar",
    "insolaréis": "insolar",
    "insolaría": "insolar",
    "insolaríais": "insolar",
    "insolaríamos": "insolar",
    "insolarían": "insolar",
    "insolarías": "insolar",
    "insolas": "insolar",
    "insolase": "insolar",
    "insolaseis": "insolar",
    "insolasen": "insolar",
    "insolases": "insolar",
    "insolaste": "insolar",
    "insolasteis": "insolar",
    "insole": "insolar",
    "insolemos": "insolar",
    "insolen": "insolar",
    "insolencias": "insolencia",
    "insolenta": "insolentar",
    "insolentaba": "insolentar",
    "insolentabais": "insolentar",
    "insolentaban": "insolentar",
    "insolentabas": "insolentar",
    "insolentad": "insolentar",
    "insolentada": "insolentar",
    "insolentadas": "insolentar",
    "insolentado": "insolentar",
    "insolentados": "insolentar",
    "insolentamos": "insolentar",
    "insolentan": "insolentar",
    "insolentando": "insolentar",
    "insolentara": "insolentar",
    "insolentarais": "insolentar",
    "insolentaran": "insolentar",
    "insolentaras": "insolentar",
    "insolentare": "insolentar",
    "insolentareis": "insolentar",
    "insolentaremos": "insolentar",
    "insolentaren": "insolentar",
    "insolentares": "insolentar",
    "insolentaron": "insolentar",
    "insolentará": "insolentar",
    "insolentarán": "insolentar",
    "insolentarás": "insolentar",
    "insolentaré": "insolentar",
    "insolentaréis": "insolentar",
    "insolentaría": "insolentar",
    "insolentaríais": "insolentar",
    "insolentaríamos": "insolentar",
    "insolentarían": "insolentar",
    "insolentarías": "insolentar",
    "insolentas": "insolentar",
    "insolentase": "insolentar",
    "insolentaseis": "insolentar",
    "insolentasen": "insolentar",
    "insolentases": "insolentar",
    "insolentaste": "insolentar",
    "insolentasteis": "insolentar",
    "insolente": "insolentar",
    "insolentemos": "insolentar",
    "insolenten": "insolentar",
    "insolentes": "insolente",
    "insolento": "insolentar",
    "insolentábamos": "insolentar",
    "insolentáis": "insolentar",
    "insolentáramos": "insolentar",
    "insolentáremos": "insolentar",
    "insolentásemos": "insolentar",
    "insolenté": "insolentar",
    "insolentéis": "insolentar",
    "insolentó": "insolentar",
    "insoles": "insolar",
    "insolidaria": "insolidario",
    "insolidarias": "insolidario",
    "insolidaridades": "insolidaridad",
    "insolidarios": "insolidario",
    "insolo": "insolar",
    "insolubilidades": "insolubilidad",
    "insolubles": "insoluble",
    "insoluta": "insoluto",
    "insolutas": "insoluto",
    "insolutos": "insoluto",
    "insolutumdaciones": "insolutumdación",
    "insolvencias": "insolvencia",
    "insolventes": "insolvente",
    "insolábamos": "insolar",
    "insoláis": "insolar",
    "insoláramos": "insolar",
    "insoláremos": "insolar",
    "insolásemos": "insolar",
    "insolé": "insolar",
    "insoléis": "insolar",
    "insoló": "insolar",
    "insomnes": "insomne",
    "insomnios": "insomnio",
    "insondables": "insondable",
    "insonora": "insonoro",
    "insonoras": "insonoro",
    "insonorice": "insonorizar",
    "insonoricemos": "insonorizar",
    "insonoricen": "insonorizar",
    "insonorices": "insonorizar",
    "insonoricé": "insonorizar",
    "insonoricéis": "insonorizar",
    "insonoridades": "insonoridad",
    "insonoriza": "insonorizar",
    "insonorizaba": "insonorizar",
    "insonorizabais": "insonorizar",
    "insonorizaban": "insonorizar",
    "insonorizabas": "insonorizar",
    "insonorizaciones": "insonorización",
    "insonorizad": "insonorizar",
    "insonorizada": "insonorizar",
    "insonorizadas": "insonorizar",
    "insonorizado": "insonorizar",
    "insonorizadora": "insonorizador",
    "insonorizadoras": "insonorizador",
    "insonorizadores": "insonorizador",
    "insonorizados": "insonorizar",
    "insonorizamos": "insonorizar",
    "insonorizan": "insonorizar",
    "insonorizando": "insonorizar",
    "insonorizara": "insonorizar",
    "insonorizarais": "insonorizar",
    "insonorizaran": "insonorizar",
    "insonorizaras": "insonorizar",
    "insonorizare": "insonorizar",
    "insonorizareis": "insonorizar",
    "insonorizaremos": "insonorizar",
    "insonorizaren": "insonorizar",
    "insonorizares": "insonorizar",
    "insonorizaron": "insonorizar",
    "insonorizará": "insonorizar",
    "insonorizarán": "insonorizar",
    "insonorizarás": "insonorizar",
    "insonorizaré": "insonorizar",
    "insonorizaréis": "insonorizar",
    "insonorizaría": "insonorizar",
    "insonorizaríais": "insonorizar",
    "insonorizaríamos": "insonorizar",
    "insonorizarían": "insonorizar",
    "insonorizarías": "insonorizar",
    "insonorizas": "insonorizar",
    "insonorizase": "insonorizar",
    "insonorizaseis": "insonorizar",
    "insonorizasen": "insonorizar",
    "insonorizases": "insonorizar",
    "insonorizaste": "insonorizar",
    "insonorizasteis": "insonorizar",
    "insonorizo": "insonorizar",
    "insonorizábamos": "insonorizar",
    "insonorizáis": "insonorizar",
    "insonorizáramos": "insonorizar",
    "insonorizáremos": "insonorizar",
    "insonorizásemos": "insonorizar",
    "insonorizó": "insonorizar",
    "insonoros": "insonoro",
    "insoportables": "insoportable",
    "insoslayables": "insoslayable",
    "insospechables": "insospechable",
    "insospechada": "insospechado",
    "insospechadas": "insospechado",
    "insospechados": "insospechado",
    "insostenibilidades": "insostenibilidad",
    "insostenibles": "insostenible",
    "inspecciona": "inspeccionar",
    "inspeccionaba": "inspeccionar",
    "inspeccionabais": "inspeccionar",
    "inspeccionaban": "inspeccionar",
    "inspeccionabas": "inspeccionar",
    "inspeccionad": "inspeccionar",
    "inspeccionada": "inspeccionar",
    "inspeccionadas": "inspeccionar",
    "inspeccionado": "inspeccionar",
    "inspeccionados": "inspeccionar",
    "inspeccionamos": "inspeccionar",
    "inspeccionan": "inspeccionar",
    "inspeccionando": "inspeccionar",
    "inspeccionara": "inspeccionar",
    "inspeccionarais": "inspeccionar",
    "inspeccionaran": "inspeccionar",
    "inspeccionaras": "inspeccionar",
    "inspeccionare": "inspeccionar",
    "inspeccionareis": "inspeccionar",
    "inspeccionaremos": "inspeccionar",
    "inspeccionaren": "inspeccionar",
    "inspeccionares": "inspeccionar",
    "inspeccionaron": "inspeccionar",
    "inspeccionará": "inspeccionar",
    "inspeccionarán": "inspeccionar",
    "inspeccionarás": "inspeccionar",
    "inspeccionaré": "inspeccionar",
    "inspeccionaréis": "inspeccionar",
    "inspeccionaría": "inspeccionar",
    "inspeccionaríais": "inspeccionar",
    "inspeccionaríamos": "inspeccionar",
    "inspeccionarían": "inspeccionar",
    "inspeccionarías": "inspeccionar",
    "inspeccionas": "inspeccionar",
    "inspeccionase": "inspeccionar",
    "inspeccionaseis": "inspeccionar",
    "inspeccionasen": "inspeccionar",
    "inspeccionases": "inspeccionar",
    "inspeccionaste": "inspeccionar",
    "inspeccionasteis": "inspeccionar",
    "inspeccione": "inspeccionar",
    "inspeccionemos": "inspeccionar",
    "inspeccionen": "inspeccionar",
    "inspecciones": "inspeccionar",
    "inspecciono": "inspeccionar",
    "inspeccionábamos": "inspeccionar",
    "inspeccionáis": "inspeccionar",
    "inspeccionáramos": "inspeccionar",
    "inspeccionáremos": "inspeccionar",
    "inspeccionásemos": "inspeccionar",
    "inspeccioné": "inspeccionar",
    "inspeccionéis": "inspeccionar",
    "inspeccionó": "inspeccionar",
    "inspectora": "inspector",
    "inspectoras": "inspector",
    "inspectores": "inspector",
    "inspectorías": "inspectoría",
    "inspira": "inspirar",
    "inspiraba": "inspirar",
    "inspirabais": "inspirar",
    "inspiraban": "inspirar",
    "inspirabas": "inspirar",
    "inspiraciones": "inspiración",
    "inspirad": "inspirar",
    "inspirada": "inspirar",
    "inspiradas": "inspirar",
    "inspirado": "inspirar",
    "inspiradora": "inspirador",
    "inspiradoras": "inspirador",
    "inspiradores": "inspirador",
    "inspirados": "inspirar",
    "inspiramos": "inspirar",
    "inspiran": "inspirar",
    "inspirando": "inspirar",
    "inspirara": "inspirar",
    "inspirarais": "inspirar",
    "inspiraran": "inspirar",
    "inspiraras": "inspirar",
    "inspirare": "inspirar",
    "inspirareis": "inspirar",
    "inspiraremos": "inspirar",
    "inspiraren": "inspirar",
    "inspirares": "inspirar",
    "inspiraron": "inspirar",
    "inspirará": "inspirar",
    "inspirarán": "inspirar",
    "inspirarás": "inspirar",
    "inspiraré": "inspirar",
    "inspiraréis": "inspirar",
    "inspiraría": "inspirar",
    "inspiraríais": "inspirar",
    "inspiraríamos": "inspirar",
    "inspirarían": "inspirar",
    "inspirarías": "inspirar",
    "inspiras": "inspirar",
    "inspirase": "inspirar",
    "inspiraseis": "inspirar",
    "inspirasen": "inspirar",
    "inspirases": "inspirar",
    "inspiraste": "inspirar",
    "inspirasteis": "inspirar",
    "inspirativa": "inspirativo",
    "inspirativas": "inspirativo",
    "inspirativos": "inspirativo",
    "inspire": "inspirar",
    "inspiremos": "inspirar",
    "inspiren": "inspirar",
    "inspires": "inspirar",
    "inspiro": "inspirar",
    "inspirábamos": "inspirar",
    "inspiráis": "inspirar",
    "inspiráramos": "inspirar",
    "inspiráremos": "inspirar",
    "inspirásemos": "inspirar",
    "inspiré": "inspirar",
    "inspiréis": "inspirar",
    "inspiró": "inspirar",
    "insta": "instar",
    "instaba": "instar",
    "instabais": "instar",
    "instaban": "instar",
    "instabas": "instar",
    "instad": "instar",
    "instada": "instar",
    "instadas": "instar",
    "instado": "instar",
    "instados": "instar",
    "instala": "instalar",
    "instalaba": "instalar",
    "instalabais": "instalar",
    "instalaban": "instalar",
    "instalabas": "instalar",
    "instalables": "instalable",
    "instalaciones": "instalación",
    "instalad": "instalar",
    "instalada": "instalar",
    "instaladas": "instalar",
    "instalado": "instalar",
    "instaladora": "instalador",
    "instaladoras": "instalador",
    "instaladores": "instalador",
    "instalados": "instalar",
    "instalamos": "instalar",
    "instalan": "instalar",
    "instalando": "instalar",
    "instalara": "instalar",
    "instalarais": "instalar",
    "instalaran": "instalar",
    "instalaras": "instalar",
    "instalare": "instalar",
    "instalareis": "instalar",
    "instalaremos": "instalar",
    "instalaren": "instalar",
    "instalares": "instalar",
    "instalaron": "instalar",
    "instalará": "instalar",
    "instalarán": "instalar",
    "instalarás": "instalar",
    "instalaré": "instalar",
    "instalaréis": "instalar",
    "instalaría": "instalar",
    "instalaríais": "instalar",
    "instalaríamos": "instalar",
    "instalarían": "instalar",
    "instalarías": "instalar",
    "instalas": "instalar",
    "instalase": "instalar",
    "instalaseis": "instalar",
    "instalasen": "instalar",
    "instalases": "instalar",
    "instalaste": "instalar",
    "instalasteis": "instalar",
    "instale": "instalar",
    "instalemos": "instalar",
    "instalen": "instalar",
    "instales": "instalar",
    "instalo": "instalar",
    "instalábamos": "instalar",
    "instaláis": "instalar",
    "instaláramos": "instalar",
    "instaláremos": "instalar",
    "instalásemos": "instalar",
    "instalé": "instalar",
    "instaléis": "instalar",
    "instaló": "instalar",
    "instamos": "instar",
    "instan": "instar",
    "instancias": "instancia",
    "instando": "instar",
    "instantaneidades": "instantaneidad",
    "instantes": "instante",
    "instantánea": "instantáneo",
    "instantáneas": "instantáneo",
    "instantáneos": "instantáneo",
    "instara": "instar",
    "instarais": "instar",
    "instaran": "instar",
    "instaras": "instar",
    "instare": "instar",
    "instareis": "instar",
    "instaremos": "instar",
    "instaren": "instar",
    "instares": "instar",
    "instaron": "instar",
    "instará": "instar",
    "instarán": "instar",
    "instarás": "instar",
    "instaré": "instar",
    "instaréis": "instar",
    "instaría": "instar",
    "instaríais": "instar",
    "instaríamos": "instar",
    "instarían": "instar",
    "instarías": "instar",
    "instas": "instar",
    "instase": "instar",
    "instaseis": "instar",
    "instasen": "instar",
    "instases": "instar",
    "instaste": "instar",
    "instasteis": "instar",
    "instaura": "instaurar",
    "instauraba": "instaurar",
    "instaurabais": "instaurar",
    "instauraban": "instaurar",
    "instaurabas": "instaurar",
    "instauraciones": "instauración",
    "instaurad": "instaurar",
    "instaurada": "instaurar",
    "instauradas": "instaurar",
    "instaurado": "instaurar",
    "instauradora": "instaurador",
    "instauradoras": "instaurador",
    "instauradores": "instaurador",
    "instaurados": "instaurar",
    "instauramos": "instaurar",
    "instauran": "instaurar",
    "instaurando": "instaurar",
    "instaurara": "instaurar",
    "instaurarais": "instaurar",
    "instauraran": "instaurar",
    "instauraras": "instaurar",
    "instaurare": "instaurar",
    "instaurareis": "instaurar",
    "instauraremos": "instaurar",
    "instauraren": "instaurar",
    "instaurares": "instaurar",
    "instauraron": "instaurar",
    "instaurará": "instaurar",
    "instaurarán": "instaurar",
    "instaurarás": "instaurar",
    "instauraré": "instaurar",
    "instauraréis": "instaurar",
    "instauraría": "instaurar",
    "instauraríais": "instaurar",
    "instauraríamos": "instaurar",
    "instaurarían": "instaurar",
    "instaurarías": "instaurar",
    "instauras": "instaurar",
    "instaurase": "instaurar",
    "instauraseis": "instaurar",
    "instaurasen": "instaurar",
    "instaurases": "instaurar",
    "instauraste": "instaurar",
    "instaurasteis": "instaurar",
    "instaurativos": "instaurativo",
    "instaure": "instaurar",
    "instauremos": "instaurar",
    "instauren": "instaurar",
    "instaures": "instaurar",
    "instauro": "instaurar",
    "instaurábamos": "instaurar",
    "instauráis": "instaurar",
    "instauráramos": "instaurar",
    "instauráremos": "instaurar",
    "instaurásemos": "instaurar",
    "instauré": "instaurar",
    "instauréis": "instaurar",
    "instauró": "instaurar",
    "inste": "instar",
    "instemos": "instar",
    "insten": "instar",
    "instes": "instar",
    "instiga": "instigar",
    "instigaba": "instigar",
    "instigabais": "instigar",
    "instigaban": "instigar",
    "instigabas": "instigar",
    "instigaciones": "instigación",
    "instigad": "instigar",
    "instigada": "instigar",
    "instigadas": "instigar",
    "instigado": "instigar",
    "instigadora": "instigador",
    "instigadoras": "instigador",
    "instigadores": "instigador",
    "instigados": "instigar",
    "instigamos": "instigar",
    "instigan": "instigar",
    "instigando": "instigar",
    "instigara": "instigar",
    "instigarais": "instigar",
    "instigaran": "instigar",
    "instigaras": "instigar",
    "instigare": "instigar",
    "instigareis": "instigar",
    "instigaremos": "instigar",
    "instigaren": "instigar",
    "instigares": "instigar",
    "instigaron": "instigar",
    "instigará": "instigar",
    "instigarán": "instigar",
    "instigarás": "instigar",
    "instigaré": "instigar",
    "instigaréis": "instigar",
    "instigaría": "instigar",
    "instigaríais": "instigar",
    "instigaríamos": "instigar",
    "instigarían": "instigar",
    "instigarías": "instigar",
    "instigas": "instigar",
    "instigase": "instigar",
    "instigaseis": "instigar",
    "instigasen": "instigar",
    "instigases": "instigar",
    "instigaste": "instigar",
    "instigasteis": "instigar",
    "instigo": "instigar",
    "instigue": "instigar",
    "instiguemos": "instigar",
    "instiguen": "instigar",
    "instigues": "instigar",
    "instigué": "instigar",
    "instiguéis": "instigar",
    "instigábamos": "instigar",
    "instigáis": "instigar",
    "instigáramos": "instigar",
    "instigáremos": "instigar",
    "instigásemos": "instigar",
    "instigó": "instigar",
    "instila": "instilar",
    "instilaba": "instilar",
    "instilabais": "instilar",
    "instilaban": "instilar",
    "instilabas": "instilar",
    "instilaciones": "instilación",
    "instilad": "instilar",
    "instilada": "instilar",
    "instiladas": "instilar",
    "instilado": "instilar",
    "instilados": "instilar",
    "instilamos": "instilar",
    "instilan": "instilar",
    "instilando": "instilar",
    "instilara": "instilar",
    "instilarais": "instilar",
    "instilaran": "instilar",
    "instilaras": "instilar",
    "instilare": "instilar",
    "instilareis": "instilar",
    "instilaremos": "instilar",
    "instilaren": "instilar",
    "instilares": "instilar",
    "instilaron": "instilar",
    "instilará": "instilar",
    "instilarán": "instilar",
    "instilarás": "instilar",
    "instilaré": "instilar",
    "instilaréis": "instilar",
    "instilaría": "instilar",
    "instilaríais": "instilar",
    "instilaríamos": "instilar",
    "instilarían": "instilar",
    "instilarías": "instilar",
    "instilas": "instilar",
    "instilase": "instilar",
    "instilaseis": "instilar",
    "instilasen": "instilar",
    "instilases": "instilar",
    "instilaste": "instilar",
    "instilasteis": "instilar",
    "instile": "instilar",
    "instilemos": "instilar",
    "instilen": "instilar",
    "instiles": "instilar",
    "instilo": "instilar",
    "instilábamos": "instilar",
    "instiláis": "instilar",
    "instiláramos": "instilar",
    "instiláremos": "instilar",
    "instilásemos": "instilar",
    "instilé": "instilar",
    "instiléis": "instilar",
    "instiló": "instilar",
    "instintiva": "instintivo",
    "instintivas": "instintivo",
    "instintivos": "instintivo",
    "instintos": "instinto",
    "institucionales": "institucional",
    "institucionalice": "institucionalizar",
    "institucionalicemos": "institucionalizar",
    "institucionalicen": "institucionalizar",
    "institucionalices": "institucionalizar",
    "institucionalicé": "institucionalizar",
    "institucionalicéis": "institucionalizar",
    "institucionalismos": "institucionalismo",
    "institucionalistas": "institucionalista",
    "institucionaliza": "institucionalizar",
    "institucionalizaba": "institucionalizar",
    "institucionalizabais": "institucionalizar",
    "institucionalizaban": "institucionalizar",
    "institucionalizabas": "institucionalizar",
    "institucionalizaciones": "institucionalización",
    "institucionalizad": "institucionalizar",
    "institucionalizada": "institucionalizar",
    "institucionalizadas": "institucionalizar",
    "institucionalizado": "institucionalizar",
    "institucionalizados": "institucionalizar",
    "institucionalizamos": "institucionalizar",
    "institucionalizan": "institucionalizar",
    "institucionalizando": "institucionalizar",
    "institucionalizara": "institucionalizar",
    "institucionalizarais": "institucionalizar",
    "institucionalizaran": "institucionalizar",
    "institucionalizaras": "institucionalizar",
    "institucionalizare": "institucionalizar",
    "institucionalizareis": "institucionalizar",
    "institucionalizaremos": "institucionalizar",
    "institucionalizaren": "institucionalizar",
    "institucionalizares": "institucionalizar",
    "institucionalizaron": "institucionalizar",
    "institucionalizará": "institucionalizar",
    "institucionalizarán": "institucionalizar",
    "institucionalizarás": "institucionalizar",
    "institucionalizaré": "institucionalizar",
    "institucionalizaréis": "institucionalizar",
    "institucionalizaría": "institucionalizar",
    "institucionalizaríais": "institucionalizar",
    "institucionalizaríamos": "institucionalizar",
    "institucionalizarían": "institucionalizar",
    "institucionalizarías": "institucionalizar",
    "institucionalizas": "institucionalizar",
    "institucionalizase": "institucionalizar",
    "institucionalizaseis": "institucionalizar",
    "institucionalizasen": "institucionalizar",
    "institucionalizases": "institucionalizar",
    "institucionalizaste": "institucionalizar",
    "institucionalizasteis": "institucionalizar",
    "institucionalizo": "institucionalizar",
    "institucionalizábamos": "institucionalizar",
    "institucionalizáis": "institucionalizar",
    "institucionalizáramos": "institucionalizar",
    "institucionalizáremos": "institucionalizar",
    "institucionalizásemos": "institucionalizar",
    "institucionalizó": "institucionalizar",
    "instituciones": "institución",
    "institucionistas": "institucionista",
    "instituid": "instituir",
    "instituida": "instituir",
    "instituidas": "instituir",
    "instituido": "instituir",
    "instituidora": "instituidor",
    "instituidoras": "instituidor",
    "instituidores": "instituidor",
    "instituidos": "instituir",
    "instituimos": "instituir",
    "instituiremos": "instituir",
    "instituirá": "instituir",
    "instituirán": "instituir",
    "instituirás": "instituir",
    "instituiré": "instituir",
    "instituiréis": "instituir",
    "instituiría": "instituir",
    "instituiríais": "instituir",
    "instituiríamos": "instituir",
    "instituirían": "instituir",
    "instituirías": "instituir",
    "instituiste": "instituir",
    "instituisteis": "instituir",
    "institutora": "institutor",
    "institutoras": "institutor",
    "institutores": "institutor",
    "institutos": "instituto",
    "institutrices": "institutriz",
    "instituya": "instituir",
    "instituyamos": "instituir",
    "instituyan": "instituir",
    "instituyas": "instituir",
    "instituye": "instituir",
    "instituyen": "instituir",
    "instituyendo": "instituir",
    "instituyera": "instituir",
    "instituyerais": "instituir",
    "instituyeran": "instituir",
    "instituyeras": "instituir",
    "instituyere": "instituir",
    "instituyereis": "instituir",
    "instituyeren": "instituir",
    "instituyeres": "instituir",
    "instituyeron": "instituir",
    "instituyes": "instituir",
    "instituyese": "instituir",
    "instituyeseis": "instituir",
    "instituyesen": "instituir",
    "instituyeses": "instituir",
    "instituyo": "instituir",
    "instituyáis": "instituir",
    "instituyéramos": "instituir",
    "instituyéremos": "instituir",
    "instituyésemos": "instituir",
    "instituyó": "instituir",
    "instituí": "instituir",
    "instituía": "instituir",
    "instituíais": "instituir",
    "instituíamos": "instituir",
    "instituían": "instituir",
    "instituías": "instituir",
    "instituís": "instituir",
    "insto": "instar",
    "instruccionales": "instruccional",
    "instrucciones": "instrucción",
    "instructiva": "instructivo",
    "instructivas": "instructivo",
    "instructivos": "instructivo",
    "instructora": "instructor",
    "instructoras": "instructor",
    "instructores": "instructor",
    "instructoria": "instructorio",
    "instructorias": "instructorio",
    "instructorios": "instructorio",
    "instruid": "instruir",
    "instruida": "instruir",
    "instruidas": "instruir",
    "instruido": "instruir",
    "instruidos": "instruir",
    "instruimos": "instruir",
    "instruiremos": "instruir",
    "instruirá": "instruir",
    "instruirán": "instruir",
    "instruirás": "instruir",
    "instruiré": "instruir",
    "instruiréis": "instruir",
    "instruiría": "instruir",
    "instruiríais": "instruir",
    "instruiríamos": "instruir",
    "instruirían": "instruir",
    "instruirías": "instruir",
    "instruiste": "instruir",
    "instruisteis": "instruir",
    "instrumenta": "instrumentar",
    "instrumentaba": "instrumentar",
    "instrumentabais": "instrumentar",
    "instrumentaban": "instrumentar",
    "instrumentabas": "instrumentar",
    "instrumentaciones": "instrumentación",
    "instrumentad": "instrumentar",
    "instrumentada": "instrumentar",
    "instrumentadas": "instrumentar",
    "instrumentado": "instrumentar",
    "instrumentados": "instrumentar",
    "instrumentales": "instrumental",
    "instrumentalice": "instrumentalizar",
    "instrumentalicemos": "instrumentalizar",
    "instrumentalicen": "instrumentalizar",
    "instrumentalices": "instrumentalizar",
    "instrumentalicé": "instrumentalizar",
    "instrumentalicéis": "instrumentalizar",
    "instrumentalismos": "instrumentalismo",
    "instrumentalistas": "instrumentalista",
    "instrumentaliza": "instrumentalizar",
    "instrumentalizaba": "instrumentalizar",
    "instrumentalizabais": "instrumentalizar",
    "instrumentalizaban": "instrumentalizar",
    "instrumentalizabas": "instrumentalizar",
    "instrumentalizaciones": "instrumentalización",
    "instrumentalizad": "instrumentalizar",
    "instrumentalizada": "instrumentalizar",
    "instrumentalizadas": "instrumentalizar",
    "instrumentalizado": "instrumentalizar",
    "instrumentalizados": "instrumentalizar",
    "instrumentalizamos": "instrumentalizar",
    "instrumentalizan": "instrumentalizar",
    "instrumentalizando": "instrumentalizar",
    "instrumentalizara": "instrumentalizar",
    "instrumentalizarais": "instrumentalizar",
    "instrumentalizaran": "instrumentalizar",
    "instrumentalizaras": "instrumentalizar",
    "instrumentalizare": "instrumentalizar",
    "instrumentalizareis": "instrumentalizar",
    "instrumentalizaremos": "instrumentalizar",
    "instrumentalizaren": "instrumentalizar",
    "instrumentalizares": "instrumentalizar",
    "instrumentalizaron": "instrumentalizar",
    "instrumentalizará": "instrumentalizar",
    "instrumentalizarán": "instrumentalizar",
    "instrumentalizarás": "instrumentalizar",
    "instrumentalizaré": "instrumentalizar",
    "instrumentalizaréis": "instrumentalizar",
    "instrumentalizaría": "instrumentalizar",
    "instrumentalizaríais": "instrumentalizar",
    "instrumentalizaríamos": "instrumentalizar",
    "instrumentalizarían": "instrumentalizar",
    "instrumentalizarías": "instrumentalizar",
    "instrumentalizas": "instrumentalizar",
    "instrumentalizase": "instrumentalizar",
    "instrumentalizaseis": "instrumentalizar",
    "instrumentalizasen": "instrumentalizar",
    "instrumentalizases": "instrumentalizar",
    "instrumentalizaste": "instrumentalizar",
    "instrumentalizasteis": "instrumentalizar",
    "instrumentalizo": "instrumentalizar",
    "instrumentalizábamos": "instrumentalizar",
    "instrumentalizáis": "instrumentalizar",
    "instrumentalizáramos": "instrumentalizar",
    "instrumentalizáremos": "instrumentalizar",
    "instrumentalizásemos": "instrumentalizar",
    "instrumentalizó": "instrumentalizar",
    "instrumentamos": "instrumentar",
    "instrumentan": "instrumentar",
    "instrumentando": "instrumentar",
    "instrumentara": "instrumentar",
    "instrumentarais": "instrumentar",
    "instrumentaran": "instrumentar",
    "instrumentaras": "instrumentar",
    "instrumentare": "instrumentar",
    "instrumentareis": "instrumentar",
    "instrumentaremos": "instrumentar",
    "instrumentaren": "instrumentar",
    "instrumentares": "instrumentar",
    "instrumentaron": "instrumentar",
    "instrumentará": "instrumentar",
    "instrumentarán": "instrumentar",
    "instrumentarás": "instrumentar",
    "instrumentaré": "instrumentar",
    "instrumentaréis": "instrumentar",
    "instrumentaría": "instrumentar",
    "instrumentaríais": "instrumentar",
    "instrumentaríamos": "instrumentar",
    "instrumentarían": "instrumentar",
    "instrumentarías": "instrumentar",
    "instrumentas": "instrumentar",
    "instrumentase": "instrumentar",
    "instrumentaseis": "instrumentar",
    "instrumentasen": "instrumentar",
    "instrumentases": "instrumentar",
    "instrumentaste": "instrumentar",
    "instrumentasteis": "instrumentar",
    "instrumente": "instrumentar",
    "instrumentemos": "instrumentar",
    "instrumenten": "instrumentar",
    "instrumentes": "instrumentar",
    "instrumentistas": "instrumentista",
    "instrumento": "instrumentar",
    "instrumentos": "instrumento",
    "instrumentábamos": "instrumentar",
    "instrumentáis": "instrumentar",
    "instrumentáramos": "instrumentar",
    "instrumentáremos": "instrumentar",
    "instrumentásemos": "instrumentar",
    "instrumenté": "instrumentar",
    "instrumentéis": "instrumentar",
    "instrumentó": "instrumentar",
    "instruya": "instruir",
    "instruyamos": "instruir",
    "instruyan": "instruir",
    "instruyas": "instruir",
    "instruye": "instruir",
    "instruyen": "instruir",
    "instruyendo": "instruir",
    "instruyera": "instruir",
    "instruyerais": "instruir",
    "instruyeran": "instruir",
    "instruyeras": "instruir",
    "instruyere": "instruir",
    "instruyereis": "instruir",
    "instruyeren": "instruir",
    "instruyeres": "instruir",
    "instruyeron": "instruir",
    "instruyes": "instruir",
    "instruyese": "instruir",
    "instruyeseis": "instruir",
    "instruyesen": "instruir",
    "instruyeses": "instruir",
    "instruyo": "instruir",
    "instruyáis": "instruir",
    "instruyéramos": "instruir",
    "instruyéremos": "instruir",
    "instruyésemos": "instruir",
    "instruyó": "instruir",
    "instruí": "instruir",
    "instruía": "instruir",
    "instruíais": "instruir",
    "instruíamos": "instruir",
    "instruían": "instruir",
    "instruías": "instruir",
    "instruís": "instruir",
    "instábamos": "instar",
    "instáis": "instar",
    "instáramos": "instar",
    "instáremos": "instar",
    "instásemos": "instar",
    "insté": "instar",
    "instéis": "instar",
    "instó": "instar",
    "insubordina": "insubordinar",
    "insubordinaba": "insubordinar",
    "insubordinabais": "insubordinar",
    "insubordinaban": "insubordinar",
    "insubordinabas": "insubordinar",
    "insubordinaciones": "insubordinación",
    "insubordinad": "insubordinar",
    "insubordinada": "insubordinar",
    "insubordinadas": "insubordinar",
    "insubordinado": "insubordinar",
    "insubordinados": "insubordinar",
    "insubordinamos": "insubordinar",
    "insubordinan": "insubordinar",
    "insubordinando": "insubordinar",
    "insubordinara": "insubordinar",
    "insubordinarais": "insubordinar",
    "insubordinaran": "insubordinar",
    "insubordinaras": "insubordinar",
    "insubordinare": "insubordinar",
    "insubordinareis": "insubordinar",
    "insubordinaremos": "insubordinar",
    "insubordinaren": "insubordinar",
    "insubordinares": "insubordinar",
    "insubordinaron": "insubordinar",
    "insubordinará": "insubordinar",
    "insubordinarán": "insubordinar",
    "insubordinarás": "insubordinar",
    "insubordinaré": "insubordinar",
    "insubordinaréis": "insubordinar",
    "insubordinaría": "insubordinar",
    "insubordinaríais": "insubordinar",
    "insubordinaríamos": "insubordinar",
    "insubordinarían": "insubordinar",
    "insubordinarías": "insubordinar",
    "insubordinas": "insubordinar",
    "insubordinase": "insubordinar",
    "insubordinaseis": "insubordinar",
    "insubordinasen": "insubordinar",
    "insubordinases": "insubordinar",
    "insubordinaste": "insubordinar",
    "insubordinasteis": "insubordinar",
    "insubordine": "insubordinar",
    "insubordinemos": "insubordinar",
    "insubordinen": "insubordinar",
    "insubordines": "insubordinar",
    "insubordino": "insubordinar",
    "insubordinábamos": "insubordinar",
    "insubordináis": "insubordinar",
    "insubordináramos": "insubordinar",
    "insubordináremos": "insubordinar",
    "insubordinásemos": "insubordinar",
    "insubordiné": "insubordinar",
    "insubordinéis": "insubordinar",
    "insubordinó": "insubordinar",
    "insubsanables": "insubsanable",
    "insubsistencias": "insubsistencia",
    "insubsistentes": "insubsistente",
    "insubstanciales": "insubstancial",
    "insubstituibles": "insubstituible",
    "insuficiencias": "insuficiencia",
    "insuficientes": "insuficiente",
    "insufla": "insuflar",
    "insuflaba": "insuflar",
    "insuflabais": "insuflar",
    "insuflaban": "insuflar",
    "insuflabas": "insuflar",
    "insuflaciones": "insuflación",
    "insuflad": "insuflar",
    "insuflada": "insuflar",
    "insufladas": "insuflar",
    "insuflado": "insuflar",
    "insufladores": "insuflador",
    "insuflados": "insuflar",
    "insuflamos": "insuflar",
    "insuflan": "insuflar",
    "insuflando": "insuflar",
    "insuflara": "insuflar",
    "insuflarais": "insuflar",
    "insuflaran": "insuflar",
    "insuflaras": "insuflar",
    "insuflare": "insuflar",
    "insuflareis": "insuflar",
    "insuflaremos": "insuflar",
    "insuflaren": "insuflar",
    "insuflares": "insuflar",
    "insuflaron": "insuflar",
    "insuflará": "insuflar",
    "insuflarán": "insuflar",
    "insuflarás": "insuflar",
    "insuflaré": "insuflar",
    "insuflaréis": "insuflar",
    "insuflaría": "insuflar",
    "insuflaríais": "insuflar",
    "insuflaríamos": "insuflar",
    "insuflarían": "insuflar",
    "insuflarías": "insuflar",
    "insuflas": "insuflar",
    "insuflase": "insuflar",
    "insuflaseis": "insuflar",
    "insuflasen": "insuflar",
    "insuflases": "insuflar",
    "insuflaste": "insuflar",
    "insuflasteis": "insuflar",
    "insufle": "insuflar",
    "insuflemos": "insuflar",
    "insuflen": "insuflar",
    "insufles": "insuflar",
    "insuflo": "insuflar",
    "insuflábamos": "insuflar",
    "insufláis": "insuflar",
    "insufláramos": "insuflar",
    "insufláremos": "insuflar",
    "insuflásemos": "insuflar",
    "insuflé": "insuflar",
    "insufléis": "insuflar",
    "insufló": "insuflar",
    "insufribles": "insufrible",
    "insulana": "insulano",
    "insulanas": "insulano",
    "insulanos": "insulano",
    "insulares": "insular",
    "insularidades": "insularidad",
    "insulinas": "insulina",
    "insulinemias": "insulinemia",
    "insulinizaciones": "insulinización",
    "insulinorresistencias": "insulinorresistencia",
    "insulsa": "insulso",
    "insulsas": "insulso",
    "insulseces": "insulsez",
    "insulsos": "insulso",
    "insulta": "insultar",
    "insultaba": "insultar",
    "insultabais": "insultar",
    "insultaban": "insultar",
    "insultabas": "insultar",
    "insultad": "insultar",
    "insultada": "insultar",
    "insultadas": "insultar",
    "insultado": "insultar",
    "insultadora": "insultador",
    "insultadoras": "insultador",
    "insultadores": "insultador",
    "insultados": "insultar",
    "insultamos": "insultar",
    "insultan": "insultar",
    "insultando": "insultar",
    "insultantes": "insultante",
    "insultara": "insultar",
    "insultarais": "insultar",
    "insultaran": "insultar",
    "insultaras": "insultar",
    "insultare": "insultar",
    "insultareis": "insultar",
    "insultaremos": "insultar",
    "insultaren": "insultar",
    "insultares": "insultar",
    "insultaron": "insultar",
    "insultará": "insultar",
    "insultarán": "insultar",
    "insultarás": "insultar",
    "insultaré": "insultar",
    "insultaréis": "insultar",
    "insultaría": "insultar",
    "insultaríais": "insultar",
    "insultaríamos": "insultar",
    "insultarían": "insultar",
    "insultarías": "insultar",
    "insultas": "insultar",
    "insultase": "insultar",
    "insultaseis": "insultar",
    "insultasen": "insultar",
    "insultases": "insultar",
    "insultaste": "insultar",
    "insultasteis": "insultar",
    "insulte": "insultar",
    "insultemos": "insultar",
    "insulten": "insultar",
    "insultes": "insultar",
    "insulto": "insultar",
    "insultos": "insulto",
    "insultábamos": "insultar",
    "insultáis": "insultar",
    "insultáramos": "insultar",
    "insultáremos": "insultar",
    "insultásemos": "insultar",
    "insulté": "insultar",
    "insultéis": "insultar",
    "insultó": "insultar",
    "insumables": "insumable",
    "insumergibles": "insumergible",
    "insumes": "insume",
    "insumisa": "insumiso",
    "insumisas": "insumiso",
    "insumisiones": "insumisión",
    "insumisos": "insumiso",
    "insumos": "insumo",
    "insuperables": "insuperable",
    "insurgentes": "insurgente",
    "insurrecciona": "insurreccionar",
    "insurreccionaba": "insurreccionar",
    "insurreccionabais": "insurreccionar",
    "insurreccionaban": "insurreccionar",
    "insurreccionabas": "insurreccionar",
    "insurreccionad": "insurreccionar",
    "insurreccionada": "insurreccionar",
    "insurreccionadas": "insurreccionar",
    "insurreccionado": "insurreccionar",
    "insurreccionados": "insurreccionar",
    "insurreccionales": "insurreccional",
    "insurreccionamos": "insurreccionar",
    "insurreccionan": "insurreccionar",
    "insurreccionando": "insurreccionar",
    "insurreccionara": "insurreccionar",
    "insurreccionarais": "insurreccionar",
    "insurreccionaran": "insurreccionar",
    "insurreccionaras": "insurreccionar",
    "insurreccionare": "insurreccionar",
    "insurreccionareis": "insurreccionar",
    "insurreccionaremos": "insurreccionar",
    "insurreccionaren": "insurreccionar",
    "insurreccionares": "insurreccionar",
    "insurreccionaron": "insurreccionar",
    "insurreccionará": "insurreccionar",
    "insurreccionarán": "insurreccionar",
    "insurreccionarás": "insurreccionar",
    "insurreccionaré": "insurreccionar",
    "insurreccionaréis": "insurreccionar",
    "insurreccionaría": "insurreccionar",
    "insurreccionaríais": "insurreccionar",
    "insurreccionaríamos": "insurreccionar",
    "insurreccionarían": "insurreccionar",
    "insurreccionarías": "insurreccionar",
    "insurreccionas": "insurreccionar",
    "insurreccionase": "insurreccionar",
    "insurreccionaseis": "insurreccionar",
    "insurreccionasen": "insurreccionar",
    "insurreccionases": "insurreccionar",
    "insurreccionaste": "insurreccionar",
    "insurreccionasteis": "insurreccionar",
    "insurreccione": "insurreccionar",
    "insurreccionemos": "insurreccionar",
    "insurreccionen": "insurreccionar",
    "insurrecciones": "insurreccionar",
    "insurrecciono": "insurreccionar",
    "insurreccionábamos": "insurreccionar",
    "insurreccionáis": "insurreccionar",
    "insurreccionáramos": "insurreccionar",
    "insurreccionáremos": "insurreccionar",
    "insurreccionásemos": "insurreccionar",
    "insurreccioné": "insurreccionar",
    "insurreccionéis": "insurreccionar",
    "insurreccionó": "insurreccionar",
    "insurrecta": "insurrecto",
    "insurrectas": "insurrecto",
    "insurrectos": "insurrecto",
    "insusceptibilidades": "insusceptibilidad",
    "insustanciales": "insustancial",
    "insustancialidades": "insustancialidad",
    "insustituibles": "insustituible",
    "insípida": "insípido",
    "insípidas": "insípido",
    "insípidos": "insípido",
    "insólita": "insólito",
    "insólitas": "insólito",
    "insólitos": "insólito",
    "intachables": "intachable",
    "intacta": "intacto",
    "intactas": "intacto",
    "intactos": "intacto",
    "intangibilidades": "intangibilidad",
    "intangibles": "intangible",
    "integmentos": "integmento",
    "integra": "integrar",
    "integraba": "integrar",
    "integrabais": "integrar",
    "integraban": "integrar",
    "integrabas": "integrar",
    "integrables": "integrable",
    "integraciones": "integración",
    "integracionistas": "integracionista",
    "integrad": "integrar",
    "integrada": "integrar",
    "integradas": "integrar",
    "integrado": "integrar",
    "integradora": "integrador",
    "integradoras": "integrador",
    "integradores": "integrador",
    "integrados": "integrar",
    "integrales": "integral",
    "integramos": "integrar",
    "integran": "integrar",
    "integrando": "integrar",
    "integrandos": "integrando",
    "integrantes": "integrante",
    "integrara": "integrar",
    "integrarais": "integrar",
    "integraran": "integrar",
    "integraras": "integrar",
    "integrare": "integrar",
    "integrareis": "integrar",
    "integraremos": "integrar",
    "integraren": "integrar",
    "integrares": "integrar",
    "integraron": "integrar",
    "integrará": "integrar",
    "integrarán": "integrar",
    "integrarás": "integrar",
    "integraré": "integrar",
    "integraréis": "integrar",
    "integraría": "integrar",
    "integraríais": "integrar",
    "integraríamos": "integrar",
    "integrarían": "integrar",
    "integrarías": "integrar",
    "integras": "integrar",
    "integrase": "integrar",
    "integraseis": "integrar",
    "integrasen": "integrar",
    "integrases": "integrar",
    "integraste": "integrar",
    "integrasteis": "integrar",
    "integrativa": "integrativo",
    "integrativas": "integrativo",
    "integrativos": "integrativo",
    "integre": "integrar",
    "integremos": "integrar",
    "integren": "integrar",
    "integres": "integrar",
    "integridades": "integridad",
    "integrinas": "integrina",
    "integrismos": "integrismo",
    "integristas": "integrista",
    "integro": "integrar",
    "integrábamos": "integrar",
    "integráis": "integrar",
    "integráramos": "integrar",
    "integráremos": "integrar",
    "integrásemos": "integrar",
    "integré": "integrar",
    "integréis": "integrar",
    "integró": "integrar",
    "integumentos": "integumento",
    "integérrima": "integérrimo",
    "integérrimas": "integérrimo",
    "integérrimos": "integérrimo",
    "intelecciones": "intelección",
    "intelectiva": "intelectivo",
    "intelectivas": "intelectivo",
    "intelectivos": "intelectivo",
    "intelectos": "intelecto",
    "intelectuales": "intelectual",
    "intelectualidades": "intelectualidad",
    "intelectualismos": "intelectualismo",
    "intelectualizaciones": "intelectualización",
    "inteligencias": "inteligencia",
    "inteligentes": "inteligente",
    "inteligibilidades": "inteligibilidad",
    "inteligibles": "inteligible",
    "intemperancias": "intemperancia",
    "intemperantes": "intemperante",
    "intemperies": "intemperie",
    "intempestas": "intempesta",
    "intempestiva": "intempestivo",
    "intempestivas": "intempestivo",
    "intempestivos": "intempestivo",
    "intemporales": "intemporal",
    "intemporalidades": "intemporalidad",
    "intencionada": "intencionado",
    "intencionadas": "intencionado",
    "intencionados": "intencionado",
    "intencionales": "intencional",
    "intencionalidades": "intencionalidad",
    "intenciones": "intención",
    "intendencias": "intendencia",
    "intendentes": "intendente",
    "intensa": "intenso",
    "intensaba": "intensar",
    "intensabais": "intensar",
    "intensaban": "intensar",
    "intensabas": "intensar",
    "intensad": "intensar",
    "intensada": "intensar",
    "intensadas": "intensar",
    "intensado": "intensar",
    "intensados": "intensar",
    "intensamos": "intensar",
    "intensan": "intensar",
    "intensando": "intensar",
    "intensara": "intensar",
    "intensarais": "intensar",
    "intensaran": "intensar",
    "intensaras": "intensar",
    "intensare": "intensar",
    "intensareis": "intensar",
    "intensaremos": "intensar",
    "intensaren": "intensar",
    "intensares": "intensar",
    "intensaron": "intensar",
    "intensará": "intensar",
    "intensarán": "intensar",
    "intensarás": "intensar",
    "intensaré": "intensar",
    "intensaréis": "intensar",
    "intensaría": "intensar",
    "intensaríais": "intensar",
    "intensaríamos": "intensar",
    "intensarían": "intensar",
    "intensarías": "intensar",
    "intensas": "intenso",
    "intensase": "intensar",
    "intensaseis": "intensar",
    "intensasen": "intensar",
    "intensases": "intensar",
    "intensaste": "intensar",
    "intensasteis": "intensar",
    "intense": "intensar",
    "intensemos": "intensar",
    "intensen": "intensar",
    "intenses": "intensar",
    "intensidades": "intensidad",
    "intensifica": "intensificar",
    "intensificaba": "intensificar",
    "intensificabais": "intensificar",
    "intensificaban": "intensificar",
    "intensificabas": "intensificar",
    "intensificaciones": "intensificación",
    "intensificad": "intensificar",
    "intensificada": "intensificar",
    "intensificadas": "intensificar",
    "intensificado": "intensificar",
    "intensificadora": "intensificador",
    "intensificadoras": "intensificador",
    "intensificadores": "intensificador",
    "intensificados": "intensificar",
    "intensificamos": "intensificar",
    "intensifican": "intensificar",
    "intensificando": "intensificar",
    "intensificara": "intensificar",
    "intensificarais": "intensificar",
    "intensificaran": "intensificar",
    "intensificaras": "intensificar",
    "intensificare": "intensificar",
    "intensificareis": "intensificar",
    "intensificaremos": "intensificar",
    "intensificaren": "intensificar",
    "intensificares": "intensificar",
    "intensificaron": "intensificar",
    "intensificará": "intensificar",
    "intensificarán": "intensificar",
    "intensificarás": "intensificar",
    "intensificaré": "intensificar",
    "intensificaréis": "intensificar",
    "intensificaría": "intensificar",
    "intensificaríais": "intensificar",
    "intensificaríamos": "intensificar",
    "intensificarían": "intensificar",
    "intensificarías": "intensificar",
    "intensificas": "intensificar",
    "intensificase": "intensificar",
    "intensificaseis": "intensificar",
    "intensificasen": "intensificar",
    "intensificases": "intensificar",
    "intensificaste": "intensificar",
    "intensificasteis": "intensificar",
    "intensifico": "intensificar",
    "intensificábamos": "intensificar",
    "intensificáis": "intensificar",
    "intensificáramos": "intensificar",
    "intensificáremos": "intensificar",
    "intensificásemos": "intensificar",
    "intensificó": "intensificar",
    "intensifique": "intensificar",
    "intensifiquemos": "intensificar",
    "intensifiquen": "intensificar",
    "intensifiques": "intensificar",
    "intensifiqué": "intensificar",
    "intensifiquéis": "intensificar",
    "intensiva": "intensivo",
    "intensivas": "intensivo",
    "intensivos": "intensivo",
    "intenso": "intensar",
    "intensos": "intenso",
    "intensábamos": "intensar",
    "intensáis": "intensar",
    "intensáramos": "intensar",
    "intensáremos": "intensar",
    "intensásemos": "intensar",
    "intensé": "intensar",
    "intenséis": "intensar",
    "intensó": "intensar",
    "intenta": "intentar",
    "intentaba": "intentar",
    "intentabais": "intentar",
    "intentaban": "intentar",
    "intentabas": "intentar",
    "intentad": "intentar",
    "intentada": "intentar",
    "intentadas": "intentar",
    "intentado": "intentar",
    "intentados": "intentar",
    "intentamos": "intentar",
    "intentan": "intentar",
    "intentando": "intentar",
    "intentara": "intentar",
    "intentarais": "intentar",
    "intentaran": "intentar",
    "intentaras": "intentar",
    "intentare": "intentar",
    "intentareis": "intentar",
    "intentaremos": "intentar",
    "intentaren": "intentar",
    "intentares": "intentar",
    "intentaron": "intentar",
    "intentará": "intentar",
    "intentarán": "intentar",
    "intentarás": "intentar",
    "intentaré": "intentar",
    "intentaréis": "intentar",
    "intentaría": "intentar",
    "intentaríais": "intentar",
    "intentaríamos": "intentar",
    "intentarían": "intentar",
    "intentarías": "intentar",
    "intentas": "intentar",
    "intentase": "intentar",
    "intentaseis": "intentar",
    "intentasen": "intentar",
    "intentases": "intentar",
    "intentaste": "intentar",
    "intentasteis": "intentar",
    "intente": "intentar",
    "intentemos": "intentar",
    "intenten": "intentar",
    "intentes": "intentar",
    "intento": "intentar",
    "intentonas": "intentona",
    "intentos": "intento",
    "intentábamos": "intentar",
    "intentáis": "intentar",
    "intentáramos": "intentar",
    "intentáremos": "intentar",
    "intentásemos": "intentar",
    "intenté": "intentar",
    "intentéis": "intentar",
    "intentó": "intentar",
    "interacciones": "interacción",
    "interaccionismos": "interaccionismo",
    "interactiva": "interactivo",
    "interactivas": "interactivo",
    "interactividades": "interactividad",
    "interactivos": "interactivo",
    "interactuaba": "interactuar",
    "interactuabais": "interactuar",
    "interactuaban": "interactuar",
    "interactuabas": "interactuar",
    "interactuad": "interactuar",
    "interactuada": "interactuar",
    "interactuadas": "interactuar",
    "interactuado": "interactuar",
    "interactuados": "interactuar",
    "interactuamos": "interactuar",
    "interactuando": "interactuar",
    "interactuara": "interactuar",
    "interactuarais": "interactuar",
    "interactuaran": "interactuar",
    "interactuaras": "interactuar",
    "interactuare": "interactuar",
    "interactuareis": "interactuar",
    "interactuaremos": "interactuar",
    "interactuaren": "interactuar",
    "interactuares": "interactuar",
    "interactuaron": "interactuar",
    "interactuará": "interactuar",
    "interactuarán": "interactuar",
    "interactuarás": "interactuar",
    "interactuaré": "interactuar",
    "interactuaréis": "interactuar",
    "interactuaría": "interactuar",
    "interactuaríais": "interactuar",
    "interactuaríamos": "interactuar",
    "interactuarían": "interactuar",
    "interactuarías": "interactuar",
    "interactuase": "interactuar",
    "interactuaseis": "interactuar",
    "interactuasen": "interactuar",
    "interactuases": "interactuar",
    "interactuaste": "interactuar",
    "interactuasteis": "interactuar",
    "interactuemos": "interactuar",
    "interactuábamos": "interactuar",
    "interactuáis": "interactuar",
    "interactuáramos": "interactuar",
    "interactuáremos": "interactuar",
    "interactuásemos": "interactuar",
    "interactué": "interactuar",
    "interactuéis": "interactuar",
    "interactuó": "interactuar",
    "interactúa": "interactuar",
    "interactúan": "interactuar",
    "interactúas": "interactuar",
    "interactúe": "interactuar",
    "interactúen": "interactuar",
    "interactúes": "interactuar",
    "interactúo": "interactuar",
    "interalveolares": "interalveolar",
    "interambulacrales": "interambulacral",
    "interambulacros": "interambulacro",
    "interamericana": "interamericano",
    "interamericanas": "interamericano",
    "interamericanos": "interamericano",
    "interandina": "interandino",
    "interandinas": "interandino",
    "interandinos": "interandino",
    "interanuales": "interanual",
    "interaritenoidea": "interaritenoideo",
    "interaritenoideas": "interaritenoideo",
    "interaritenoideos": "interaritenoideo",
    "interarticulares": "interarticular",
    "interatriales": "interatrial",
    "interauriculares": "interauricular",
    "interbancaria": "interbancario",
    "interbancarias": "interbancario",
    "interbancarios": "interbancario",
    "interbibliotecaria": "interbibliotecario",
    "interbibliotecarias": "interbibliotecario",
    "interbibliotecarios": "interbibliotecario",
    "interbloqueos": "interbloqueo",
    "intercadencias": "intercadencia",
    "intercadentes": "intercadente",
    "intercala": "intercalar",
    "intercalaba": "intercalar",
    "intercalabais": "intercalar",
    "intercalaban": "intercalar",
    "intercalabas": "intercalar",
    "intercalaciones": "intercalación",
    "intercalad": "intercalar",
    "intercalada": "intercalar",
    "intercaladas": "intercalar",
    "intercalado": "intercalar",
    "intercaladora": "intercalador",
    "intercaladoras": "intercalador",
    "intercaladores": "intercalador",
    "intercalados": "intercalar",
    "intercaladuras": "intercaladura",
    "intercalamos": "intercalar",
    "intercalan": "intercalar",
    "intercalando": "intercalar",
    "intercalara": "intercalar",
    "intercalarais": "intercalar",
    "intercalaran": "intercalar",
    "intercalaras": "intercalar",
    "intercalare": "intercalar",
    "intercalareis": "intercalar",
    "intercalaremos": "intercalar",
    "intercalaren": "intercalar",
    "intercalares": "intercalar",
    "intercalaron": "intercalar",
    "intercalará": "intercalar",
    "intercalarán": "intercalar",
    "intercalarás": "intercalar",
    "intercalaré": "intercalar",
    "intercalaréis": "intercalar",
    "intercalaría": "intercalar",
    "intercalaríais": "intercalar",
    "intercalaríamos": "intercalar",
    "intercalarían": "intercalar",
    "intercalarías": "intercalar",
    "intercalas": "intercalar",
    "intercalase": "intercalar",
    "intercalaseis": "intercalar",
    "intercalasen": "intercalar",
    "intercalases": "intercalar",
    "intercalaste": "intercalar",
    "intercalasteis": "intercalar",
    "intercale": "intercalar",
    "intercalemos": "intercalar",
    "intercalen": "intercalar",
    "intercales": "intercalar",
    "intercalo": "intercalar",
    "intercalábamos": "intercalar",
    "intercaláis": "intercalar",
    "intercaláramos": "intercalar",
    "intercaláremos": "intercalar",
    "intercalásemos": "intercalar",
    "intercalé": "intercalar",
    "intercaléis": "intercalar",
    "intercaló": "intercalar",
    "intercambia": "intercambiar",
    "intercambiaba": "intercambiar",
    "intercambiabais": "intercambiar",
    "intercambiaban": "intercambiar",
    "intercambiabas": "intercambiar",
    "intercambiabilidades": "intercambiabilidad",
    "intercambiables": "intercambiable",
    "intercambiad": "intercambiar",
    "intercambiada": "intercambiar",
    "intercambiadas": "intercambiar",
    "intercambiado": "intercambiar",
    "intercambiadora": "intercambiador",
    "intercambiadoras": "intercambiador",
    "intercambiadores": "intercambiador",
    "intercambiados": "intercambiar",
    "intercambiamos": "intercambiar",
    "intercambian": "intercambiar",
    "intercambiando": "intercambiar",
    "intercambiara": "intercambiar",
    "intercambiarais": "intercambiar",
    "intercambiaran": "intercambiar",
    "intercambiaras": "intercambiar",
    "intercambiare": "intercambiar",
    "intercambiareis": "intercambiar",
    "intercambiaremos": "intercambiar",
    "intercambiaren": "intercambiar",
    "intercambiares": "intercambiar",
    "intercambiaron": "intercambiar",
    "intercambiará": "intercambiar",
    "intercambiarán": "intercambiar",
    "intercambiarás": "intercambiar",
    "intercambiaré": "intercambiar",
    "intercambiaréis": "intercambiar",
    "intercambiaría": "intercambiar",
    "intercambiaríais": "intercambiar",
    "intercambiaríamos": "intercambiar",
    "intercambiarían": "intercambiar",
    "intercambiarías": "intercambiar",
    "intercambias": "intercambiar",
    "intercambiase": "intercambiar",
    "intercambiaseis": "intercambiar",
    "intercambiasen": "intercambiar",
    "intercambiases": "intercambiar",
    "intercambiaste": "intercambiar",
    "intercambiasteis": "intercambiar",
    "intercambie": "intercambiar",
    "intercambiemos": "intercambiar",
    "intercambien": "intercambiar",
    "intercambies": "intercambiar",
    "intercambio": "intercambiar",
    "intercambios": "intercambio",
    "intercambiábamos": "intercambiar",
    "intercambiáis": "intercambiar",
    "intercambiáramos": "intercambiar",
    "intercambiáremos": "intercambiar",
    "intercambiásemos": "intercambiar",
    "intercambié": "intercambiar",
    "intercambiéis": "intercambiar",
    "intercambió": "intercambiar",
    "intercanales": "intercanal",
    "intercanina": "intercanino",
    "intercaninas": "intercanino",
    "intercaninos": "intercanino",
    "intercantonales": "intercantonal",
    "intercapitales": "intercapital",
    "intercarpiana": "intercarpiano",
    "intercarpianas": "intercarpiano",
    "intercarpianos": "intercarpiano",
    "intercartilaginosa": "intercartilaginoso",
    "intercartilaginosas": "intercartilaginoso",
    "intercartilaginosos": "intercartilaginoso",
    "intercavernosa": "intercavernoso",
    "intercavernosas": "intercavernoso",
    "intercavernosos": "intercavernoso",
    "interceda": "interceder",
    "intercedamos": "interceder",
    "intercedan": "interceder",
    "intercedas": "interceder",
    "intercede": "interceder",
    "interceded": "interceder",
    "intercedemos": "interceder",
    "interceden": "interceder",
    "intercederemos": "interceder",
    "intercederá": "interceder",
    "intercederán": "interceder",
    "intercederás": "interceder",
    "intercederé": "interceder",
    "intercederéis": "interceder",
    "intercedería": "interceder",
    "intercederíais": "interceder",
    "intercederíamos": "interceder",
    "intercederían": "interceder",
    "intercederías": "interceder",
    "intercedes": "interceder",
    "intercedida": "interceder",
    "intercedidas": "interceder",
    "intercedido": "interceder",
    "intercedidos": "interceder",
    "intercediendo": "interceder",
    "intercediera": "interceder",
    "intercedierais": "interceder",
    "intercedieran": "interceder",
    "intercedieras": "interceder",
    "intercediere": "interceder",
    "intercediereis": "interceder",
    "intercedieren": "interceder",
    "intercedieres": "interceder",
    "intercedieron": "interceder",
    "intercediese": "interceder",
    "intercedieseis": "interceder",
    "intercediesen": "interceder",
    "intercedieses": "interceder",
    "intercedimos": "interceder",
    "intercediste": "interceder",
    "intercedisteis": "interceder",
    "intercediéramos": "interceder",
    "intercediéremos": "interceder",
    "intercediésemos": "interceder",
    "intercedió": "interceder",
    "intercedo": "interceder",
    "intercedáis": "interceder",
    "intercedéis": "interceder",
    "intercedí": "interceder",
    "intercedía": "interceder",
    "intercedíais": "interceder",
    "intercedíamos": "interceder",
    "intercedían": "interceder",
    "intercedías": "interceder",
    "intercelulares": "intercelular",
    "intercepciones": "intercepción",
    "intercepta": "interceptar",
    "interceptaba": "interceptar",
    "interceptabais": "interceptar",
    "interceptaban": "interceptar",
    "interceptabas": "interceptar",
    "interceptaciones": "interceptación",
    "interceptad": "interceptar",
    "interceptada": "interceptar",
    "interceptadas": "interceptar",
    "interceptado": "interceptar",
    "interceptadora": "interceptador",
    "interceptadoras": "interceptador",
    "interceptadores": "interceptador",
    "interceptados": "interceptar",
    "interceptamos": "interceptar",
    "interceptan": "interceptar",
    "interceptando": "interceptar",
    "interceptara": "interceptar",
    "interceptarais": "interceptar",
    "interceptaran": "interceptar",
    "interceptaras": "interceptar",
    "interceptare": "interceptar",
    "interceptareis": "interceptar",
    "interceptaremos": "interceptar",
    "interceptaren": "interceptar",
    "interceptares": "interceptar",
    "interceptaron": "interceptar",
    "interceptará": "interceptar",
    "interceptarán": "interceptar",
    "interceptarás": "interceptar",
    "interceptaré": "interceptar",
    "interceptaréis": "interceptar",
    "interceptaría": "interceptar",
    "interceptaríais": "interceptar",
    "interceptaríamos": "interceptar",
    "interceptarían": "interceptar",
    "interceptarías": "interceptar",
    "interceptas": "interceptar",
    "interceptase": "interceptar",
    "interceptaseis": "interceptar",
    "interceptasen": "interceptar",
    "interceptases": "interceptar",
    "interceptaste": "interceptar",
    "interceptasteis": "interceptar",
    "intercepte": "interceptar",
    "interceptemos": "interceptar",
    "intercepten": "interceptar",
    "interceptes": "interceptar",
    "interceptiva": "interceptivo",
    "interceptivas": "interceptivo",
    "interceptivos": "interceptivo",
    "intercepto": "interceptar",
    "interceptora": "interceptor",
    "interceptoras": "interceptor",
    "interceptores": "interceptor",
    "interceptábamos": "interceptar",
    "interceptáis": "interceptar",
    "interceptáramos": "interceptar",
    "interceptáremos": "interceptar",
    "interceptásemos": "interceptar",
    "intercepté": "interceptar",
    "interceptéis": "interceptar",
    "interceptó": "interceptar",
    "intercesiones": "intercesión",
    "intercesora": "intercesor",
    "intercesoras": "intercesor",
    "intercesores": "intercesor",
    "interciclos": "interciclo",
    "intercitys": "intercity",
    "interclasismos": "interclasismo",
    "interclaviculares": "interclavicular",
    "intercolumnares": "intercolumnar",
    "intercolumnios": "intercolumnio",
    "intercolunios": "intercolunio",
    "intercombinaciones": "intercombinación",
    "intercomparaciones": "intercomparación",
    "intercompañías": "intercompañía",
    "intercomunicaciones": "intercomunicación",
    "intercomunicadores": "intercomunicador",
    "intercondilares": "intercondilar",
    "intercondrales": "intercondral",
    "intercondílea": "intercondíleo",
    "intercondíleas": "intercondíleo",
    "intercondíleos": "intercondíleo",
    "interconecta": "interconectar",
    "interconectaba": "interconectar",
    "interconectabais": "interconectar",
    "interconectaban": "interconectar",
    "interconectabas": "interconectar",
    "interconectabilidades": "interconectabilidad",
    "interconectad": "interconectar",
    "interconectada": "interconectar",
    "interconectadas": "interconectar",
    "interconectado": "interconectar",
    "interconectadora": "interconectador",
    "interconectadoras": "interconectador",
    "interconectadores": "interconectador",
    "interconectados": "interconectar",
    "interconectamos": "interconectar",
    "interconectan": "interconectar",
    "interconectando": "interconectar",
    "interconectara": "interconectar",
    "interconectarais": "interconectar",
    "interconectaran": "interconectar",
    "interconectaras": "interconectar",
    "interconectare": "interconectar",
    "interconectareis": "interconectar",
    "interconectaremos": "interconectar",
    "interconectaren": "interconectar",
    "interconectares": "interconectar",
    "interconectaron": "interconectar",
    "interconectará": "interconectar",
    "interconectarán": "interconectar",
    "interconectarás": "interconectar",
    "interconectaré": "interconectar",
    "interconectaréis": "interconectar",
    "interconectaría": "interconectar",
    "interconectaríais": "interconectar",
    "interconectaríamos": "interconectar",
    "interconectarían": "interconectar",
    "interconectarías": "interconectar",
    "interconectas": "interconectar",
    "interconectase": "interconectar",
    "interconectaseis": "interconectar",
    "interconectasen": "interconectar",
    "interconectases": "interconectar",
    "interconectaste": "interconectar",
    "interconectasteis": "interconectar",
    "interconecte": "interconectar",
    "interconectemos": "interconectar",
    "interconecten": "interconectar",
    "interconectes": "interconectar",
    "interconectividades": "interconectividad",
    "interconecto": "interconectar",
    "interconectábamos": "interconectar",
    "interconectáis": "interconectar",
    "interconectáramos": "interconectar",
    "interconectáremos": "interconectar",
    "interconectásemos": "interconectar",
    "interconecté": "interconectar",
    "interconectéis": "interconectar",
    "interconectó": "interconectar",
    "interconexiones": "interconexión",
    "intercontinentales": "intercontinental",
    "intercorporativa": "intercorporativo",
    "intercorporativas": "intercorporativo",
    "intercorporativos": "intercorporativo",
    "intercorrelaciones": "intercorrelación",
    "intercostales": "intercostal",
    "intercostobraquiales": "intercostobraquial",
    "intercromosómica": "intercromosómico",
    "intercromosómicas": "intercromosómico",
    "intercromosómicos": "intercromosómico",
    "intercrurales": "intercrural",
    "intercuantilas": "intercuantila",
    "intercuartílica": "intercuartílico",
    "intercuartílicas": "intercuartílico",
    "intercuartílicos": "intercuartílico",
    "intercuneales": "intercuneal",
    "intercurrentes": "intercurrente",
    "intercuspidaciones": "intercuspidación",
    "interdendrítica": "interdendrítico",
    "interdendríticas": "interdendrítico",
    "interdendríticos": "interdendrítico",
    "interdentales": "interdental",
    "interdentaria": "interdentario",
    "interdentarias": "interdentario",
    "interdentarios": "interdentario",
    "interdepartamentales": "interdepartamental",
    "interdependencias": "interdependencia",
    "interdependientes": "interdependiente",
    "interdicciones": "interdicción",
    "interdictos": "interdicto",
    "interdigitada": "interdigitado",
    "interdigitadas": "interdigitado",
    "interdigitados": "interdigitado",
    "interdigitales": "interdigital",
    "interdigitantes": "interdigitante",
    "interdisciplinares": "interdisciplinar",
    "interdisciplinaria": "interdisciplinario",
    "interdisciplinarias": "interdisciplinario",
    "interdisciplinarios": "interdisciplinario",
    "interempresariales": "interempresarial",
    "interesa": "interesar",
    "interesaba": "interesar",
    "interesabais": "interesar",
    "interesaban": "interesar",
    "interesabas": "interesar",
    "interesad": "interesar",
    "interesada": "interesar",
    "interesadas": "interesar",
    "interesado": "interesar",
    "interesados": "interesar",
    "interesamos": "interesar",
    "interesan": "interesar",
    "interesando": "interesar",
    "interesantes": "interesante",
    "interesara": "interesar",
    "interesarais": "interesar",
    "interesaran": "interesar",
    "interesaras": "interesar",
    "interesare": "interesar",
    "interesareis": "interesar",
    "interesaremos": "interesar",
    "interesaren": "interesar",
    "interesares": "interesar",
    "interesaron": "interesar",
    "interesará": "interesar",
    "interesarán": "interesar",
    "interesarás": "interesar",
    "interesaré": "interesar",
    "interesaréis": "interesar",
    "interesaría": "interesar",
    "interesaríais": "interesar",
    "interesaríamos": "interesar",
    "interesarían": "interesar",
    "interesarías": "interesar",
    "interesas": "interesar",
    "interesase": "interesar",
    "interesaseis": "interesar",
    "interesasen": "interesar",
    "interesases": "interesar",
    "interesaste": "interesar",
    "interesasteis": "interesar",
    "interese": "interesar",
    "interesemos": "interesar",
    "interesen": "interesar",
    "intereses": "interesar",
    "intereso": "interesar",
    "interespacios": "interespacio",
    "interespecífica": "interespecífico",
    "interespecíficas": "interespecífico",
    "interespecíficos": "interespecífico",
    "interespinales": "interespinal",
    "interespinosa": "interespinoso",
    "interespinosas": "interespinoso",
    "interespinosos": "interespinoso",
    "interestatales": "interestatal",
    "interestelares": "interestelar",
    "interesterificaciones": "interesterificación",
    "interestratificaciones": "interestratificación",
    "interestratificada": "interestratificado",
    "interestratificadas": "interestratificado",
    "interestratificados": "interestratificado",
    "interesábamos": "interesar",
    "interesáis": "interesar",
    "interesáramos": "interesar",
    "interesáremos": "interesar",
    "interesásemos": "interesar",
    "interesé": "interesar",
    "intereséis": "interesar",
    "interesó": "interesar",
    "interetapas": "interetapa",
    "interfaces": "interfaz",
    "interfaciales": "interfacial",
    "interfalángica": "interfalángico",
    "interfalángicas": "interfalángico",
    "interfalángicos": "interfalángico",
    "interfasciales": "interfascial",
    "interfasciculares": "interfascicular",
    "interfases": "interfase",
    "interfecta": "interfecto",
    "interfectas": "interfecto",
    "interfectos": "interfecto",
    "interfederales": "interfederal",
    "interferenciales": "interferencial",
    "interferencias": "interferencia",
    "interferentes": "interferente",
    "interferid": "interferir",
    "interferida": "interferir",
    "interferidas": "interferir",
    "interferido": "interferir",
    "interferidos": "interferir",
    "interferimos": "interferir",
    "interferiremos": "interferir",
    "interferirá": "interferir",
    "interferirán": "interferir",
    "interferirás": "interferir",
    "interferiré": "interferir",
    "interferiréis": "interferir",
    "interferiría": "interferir",
    "interferiríais": "interferir",
    "interferiríamos": "interferir",
    "interferirían": "interferir",
    "interferirías": "interferir",
    "interferiste": "interferir",
    "interferisteis": "interferir",
    "interferogramas": "interferograma",
    "interferometrías": "interferometría",
    "interferométrica": "interferométrico",
    "interferométricas": "interferométrico",
    "interferométricos": "interferométrico",
    "interferones": "interferón",
    "interferí": "interferir",
    "interfería": "interferir",
    "interferíais": "interferir",
    "interferíamos": "interferir",
    "interferían": "interferir",
    "interferías": "interferir",
    "interferís": "interferir",
    "interferómetros": "interferómetro",
    "interfibrilares": "interfibrilar",
    "interficies": "interficie",
    "interfiera": "interferir",
    "interfieran": "interferir",
    "interfieras": "interferir",
    "interfiere": "interferir",
    "interfieren": "interferir",
    "interfieres": "interferir",
    "interfiero": "interferir",
    "interfijos": "interfijo",
    "interfiramos": "interferir",
    "interfiriendo": "interferir",
    "interfiriera": "interferir",
    "interfirierais": "interferir",
    "interfirieran": "interferir",
    "interfirieras": "interferir",
    "interfiriere": "interferir",
    "interfiriereis": "interferir",
    "interfirieren": "interferir",
    "interfirieres": "interferir",
    "interfirieron": "interferir",
    "interfiriese": "interferir",
    "interfirieseis": "interferir",
    "interfiriesen": "interferir",
    "interfirieses": "interferir",
    "interfiriéramos": "interferir",
    "interfiriéremos": "interferir",
    "interfiriésemos": "interferir",
    "interfirió": "interferir",
    "interfiráis": "interferir",
    "interfluvios": "interfluvio",
    "interfolia": "interfoliar",
    "interfoliaba": "interfoliar",
    "interfoliabais": "interfoliar",
    "interfoliaban": "interfoliar",
    "interfoliabas": "interfoliar",
    "interfoliad": "interfoliar",
    "interfoliada": "interfoliar",
    "interfoliadas": "interfoliar",
    "interfoliado": "interfoliar",
    "interfoliados": "interfoliar",
    "interfoliamos": "interfoliar",
    "interfolian": "interfoliar",
    "interfoliando": "interfoliar",
    "interfoliara": "interfoliar",
    "interfoliarais": "interfoliar",
    "interfoliaran": "interfoliar",
    "interfoliaras": "interfoliar",
    "interfoliare": "interfoliar",
    "interfoliareis": "interfoliar",
    "interfoliaremos": "interfoliar",
    "interfoliaren": "interfoliar",
    "interfoliares": "interfoliar",
    "interfoliaron": "interfoliar",
    "interfoliará": "interfoliar",
    "interfoliarán": "interfoliar",
    "interfoliarás": "interfoliar",
    "interfoliaré": "interfoliar",
    "interfoliaréis": "interfoliar",
    "interfoliaría": "interfoliar",
    "interfoliaríais": "interfoliar",
    "interfoliaríamos": "interfoliar",
    "interfoliarían": "interfoliar",
    "interfoliarías": "interfoliar",
    "interfolias": "interfoliar",
    "interfoliase": "interfoliar",
    "interfoliaseis": "interfoliar",
    "interfoliasen": "interfoliar",
    "interfoliases": "interfoliar",
    "interfoliaste": "interfoliar",
    "interfoliasteis": "interfoliar",
    "interfolie": "interfoliar",
    "interfoliemos": "interfoliar",
    "interfolien": "interfoliar",
    "interfolies": "interfoliar",
    "interfolio": "interfoliar",
    "interfoliábamos": "interfoliar",
    "interfoliáis": "interfoliar",
    "interfoliáramos": "interfoliar",
    "interfoliáremos": "interfoliar",
    "interfoliásemos": "interfoliar",
    "interfolié": "interfoliar",
    "interfoliéis": "interfoliar",
    "interfolió": "interfoliar",
    "interfonos": "interfono",
    "interfonías": "interfonía",
    "interfoveolares": "interfoveolar",
    "interfranjas": "interfranja",
    "interfuncionamientos": "interfuncionamiento",
    "interfuniculares": "interfunicular",
    "interfásica": "interfásico",
    "interfásicas": "interfásico",
    "interfásicos": "interfásico",
    "intergaláctica": "intergaláctico",
    "intergalácticas": "intergaláctico",
    "intergalácticos": "intergaláctico",
    "interganglionares": "interganglionar",
    "interglaciares": "interglaciar",
    "intergrados": "intergrado",
    "intergranulares": "intergranular",
    "intergubernamentales": "intergubernamental",
    "intergénica": "intergénico",
    "intergénicas": "intergénico",
    "intergénicos": "intergénico",
    "interhalógenos": "interhalógeno",
    "interhemisférica": "interhemisférico",
    "interhemisféricas": "interhemisférico",
    "interhemisféricos": "interhemisférico",
    "interina": "interino",
    "interinas": "interino",
    "interinatos": "interinato",
    "interincisiva": "interincisivo",
    "interincisivas": "interincisivo",
    "interincisivos": "interincisivo",
    "interindustriales": "interindustrial",
    "interines": "ínterin",
    "interinidades": "interinidad",
    "interinos": "interino",
    "interinstitucionales": "interinstitucional",
    "interinsulares": "interinsular",
    "interiores": "interior",
    "interioridades": "interioridad",
    "interiorismos": "interiorismo",
    "interioristas": "interiorista",
    "interiorizaciones": "interiorización",
    "interistas": "interista",
    "interjecciones": "interjección",
    "interjectiva": "interjectivo",
    "interjectivas": "interjectivo",
    "interjectivos": "interjectivo",
    "interlenguas": "interlengua",
    "interleucinas": "interleucina",
    "interleuquinas": "interleuquina",
    "interlinea": "interlinear",
    "interlineaba": "interlinear",
    "interlineabais": "interlinear",
    "interlineaban": "interlinear",
    "interlineabas": "interlinear",
    "interlineaciones": "interlineación",
    "interlinead": "interlinear",
    "interlineada": "interlinear",
    "interlineadas": "interlinear",
    "interlineado": "interlinear",
    "interlineados": "interlinear",
    "interlineales": "interlineal",
    "interlineamos": "interlinear",
    "interlinean": "interlinear",
    "interlineando": "interlinear",
    "interlineara": "interlinear",
    "interlinearais": "interlinear",
    "interlinearan": "interlinear",
    "interlinearas": "interlinear",
    "interlineare": "interlinear",
    "interlineareis": "interlinear",
    "interlinearemos": "interlinear",
    "interlinearen": "interlinear",
    "interlineares": "interlinear",
    "interlinearon": "interlinear",
    "interlineará": "interlinear",
    "interlinearán": "interlinear",
    "interlinearás": "interlinear",
    "interlinearé": "interlinear",
    "interlinearéis": "interlinear",
    "interlinearía": "interlinear",
    "interlinearíais": "interlinear",
    "interlinearíamos": "interlinear",
    "interlinearían": "interlinear",
    "interlinearías": "interlinear",
    "interlineas": "interlinear",
    "interlinease": "interlinear",
    "interlineaseis": "interlinear",
    "interlineasen": "interlinear",
    "interlineases": "interlinear",
    "interlineaste": "interlinear",
    "interlineasteis": "interlinear",
    "interlinee": "interlinear",
    "interlineemos": "interlinear",
    "interlineen": "interlinear",
    "interlinees": "interlinear",
    "interlineo": "interlinear",
    "interlineábamos": "interlinear",
    "interlineáis": "interlinear",
    "interlineáramos": "interlinear",
    "interlineáremos": "interlinear",
    "interlineásemos": "interlinear",
    "interlineé": "interlinear",
    "interlineéis": "interlinear",
    "interlineó": "interlinear",
    "interlinguas": "interlingua",
    "interlobulares": "interlobular",
    "interlocks": "interlock",
    "interlocuciones": "interlocución",
    "interlocutora": "interlocutor",
    "interlocutoras": "interlocutor",
    "interlocutores": "interlocutor",
    "interlocutoria": "interlocutorio",
    "interlocutorias": "interlocutorio",
    "interlocutorios": "interlocutorio",
    "interludios": "interludio",
    "interlunios": "interlunio",
    "interlíneas": "interlínea",
    "intermareales": "intermareal",
    "intermaxilares": "intermaxilar",
    "intermedia": "intermedio",
    "intermediaba": "intermediar",
    "intermediabais": "intermediar",
    "intermediaban": "intermediar",
    "intermediabas": "intermediar",
    "intermediaciones": "intermediación",
    "intermediad": "intermediar",
    "intermediada": "intermediar",
    "intermediadas": "intermediar",
    "intermediado": "intermediar",
    "intermediados": "intermediar",
    "intermediamos": "intermediar",
    "intermedian": "intermediar",
    "intermediando": "intermediar",
    "intermediara": "intermediar",
    "intermediarais": "intermediar",
    "intermediaran": "intermediar",
    "intermediaras": "intermediar",
    "intermediare": "intermediar",
    "intermediareis": "intermediar",
    "intermediaremos": "intermediar",
    "intermediaren": "intermediar",
    "intermediares": "intermediar",
    "intermediaria": "intermediario",
    "intermediarias": "intermediario",
    "intermediarios": "intermediario",
    "intermediaron": "intermediar",
    "intermediará": "intermediar",
    "intermediarán": "intermediar",
    "intermediarás": "intermediar",
    "intermediaré": "intermediar",
    "intermediaréis": "intermediar",
    "intermediaría": "intermediar",
    "intermediaríais": "intermediar",
    "intermediaríamos": "intermediar",
    "intermediarían": "intermediar",
    "intermediarías": "intermediar",
    "intermedias": "intermedio",
    "intermediase": "intermediar",
    "intermediaseis": "intermediar",
    "intermediasen": "intermediar",
    "intermediases": "intermediar",
    "intermediaste": "intermediar",
    "intermediasteis": "intermediar",
    "intermedie": "intermediar",
    "intermediemos": "intermediar",
    "intermedien": "intermediar",
    "intermedies": "intermediar",
    "intermedinas": "intermedina",
    "intermedio": "intermediar",
    "intermedios": "intermedio",
    "intermediábamos": "intermediar",
    "intermediáis": "intermediar",
    "intermediáramos": "intermediar",
    "intermediáremos": "intermediar",
    "intermediásemos": "intermediar",
    "intermedié": "intermediar",
    "intermediéis": "intermediar",
    "intermedió": "intermediar",
    "intermembranosa": "intermembranoso",
    "intermembranosas": "intermembranoso",
    "intermembranosos": "intermembranoso",
    "intermesentérica": "intermesentérico",
    "intermesentéricas": "intermesentérico",
    "intermesentéricos": "intermesentérico",
    "intermetacarpiana": "intermetacarpiano",
    "intermetacarpianas": "intermetacarpiano",
    "intermetacarpianos": "intermetacarpiano",
    "intermetatarsiana": "intermetatarsiano",
    "intermetatarsianas": "intermetatarsiano",
    "intermetatarsianos": "intermetatarsiano",
    "intermetálica": "intermetálico",
    "intermetálicas": "intermetálico",
    "intermetálicos": "intermetálico",
    "interminables": "interminable",
    "interministeriales": "interministerial",
    "intermisa": "intermiso",
    "intermisas": "intermiso",
    "intermisiones": "intermisión",
    "intermisos": "intermiso",
    "intermitencias": "intermitencia",
    "intermitentes": "intermitente",
    "intermodulaciones": "intermodulación",
    "intermoleculares": "intermolecular",
    "intermunicipales": "intermunicipal",
    "intermusculares": "intermuscular",
    "intermódulos": "intermódulo",
    "interna": "interno",
    "internaba": "internar",
    "internabais": "internar",
    "internaban": "internar",
    "internabas": "internar",
    "internacionales": "internacional",
    "internacionalice": "internacionalizar",
    "internacionalicemos": "internacionalizar",
    "internacionalicen": "internacionalizar",
    "internacionalices": "internacionalizar",
    "internacionalicé": "internacionalizar",
    "internacionalicéis": "internacionalizar",
    "internacionalidades": "internacionalidad",
    "internacionalismos": "internacionalismo",
    "internacionalistas": "internacionalista",
    "internacionaliza": "internacionalizar",
    "internacionalizaba": "internacionalizar",
    "internacionalizabais": "internacionalizar",
    "internacionalizaban": "internacionalizar",
    "internacionalizabas": "internacionalizar",
    "internacionalizaciones": "internacionalización",
    "internacionalizad": "internacionalizar",
    "internacionalizada": "internacionalizar",
    "internacionalizadas": "internacionalizar",
    "internacionalizado": "internacionalizar",
    "internacionalizados": "internacionalizar",
    "internacionalizamos": "internacionalizar",
    "internacionalizan": "internacionalizar",
    "internacionalizando": "internacionalizar",
    "internacionalizara": "internacionalizar",
    "internacionalizarais": "internacionalizar",
    "internacionalizaran": "internacionalizar",
    "internacionalizaras": "internacionalizar",
    "internacionalizare": "internacionalizar",
    "internacionalizareis": "internacionalizar",
    "internacionalizaremos": "internacionalizar",
    "internacionalizaren": "internacionalizar",
    "internacionalizares": "internacionalizar",
    "internacionalizaron": "internacionalizar",
    "internacionalizará": "internacionalizar",
    "internacionalizarán": "internacionalizar",
    "internacionalizarás": "internacionalizar",
    "internacionalizaré": "internacionalizar",
    "internacionalizaréis": "internacionalizar",
    "internacionalizaría": "internacionalizar",
    "internacionalizaríais": "internacionalizar",
    "internacionalizaríamos": "internacionalizar",
    "internacionalizarían": "internacionalizar",
    "internacionalizarías": "internacionalizar",
    "internacionalizas": "internacionalizar",
    "internacionalizase": "internacionalizar",
    "internacionalizaseis": "internacionalizar",
    "internacionalizasen": "internacionalizar",
    "internacionalizases": "internacionalizar",
    "internacionalizaste": "internacionalizar",
    "internacionalizasteis": "internacionalizar",
    "internacionalizo": "internacionalizar",
    "internacionalizábamos": "internacionalizar",
    "internacionalizáis": "internacionalizar",
    "internacionalizáramos": "internacionalizar",
    "internacionalizáremos": "internacionalizar",
    "internacionalizásemos": "internacionalizar",
    "internacionalizó": "internacionalizar",
    "internaciones": "internación",
    "internad": "internar",
    "internada": "internar",
    "internadas": "internar",
    "internado": "internar",
    "internados": "internar",
    "internalismos": "internalismo",
    "internalizaciones": "internalización",
    "internamientos": "internamiento",
    "internamos": "internar",
    "internan": "internar",
    "internando": "internar",
    "internara": "internar",
    "internarais": "internar",
    "internaran": "internar",
    "internaras": "internar",
    "internare": "internar",
    "internareis": "internar",
    "internaremos": "internar",
    "internaren": "internar",
    "internares": "internar",
    "internaron": "internar",
    "internará": "internar",
    "internarán": "internar",
    "internarás": "internar",
    "internaré": "internar",
    "internaréis": "internar",
    "internaría": "internar",
    "internaríais": "internar",
    "internaríamos": "internar",
    "internarían": "internar",
    "internarías": "internar",
    "internas": "interno",
    "internasales": "internasal",
    "internase": "internar",
    "internaseis": "internar",
    "internasen": "internar",
    "internases": "internar",
    "internaste": "internar",
    "internasteis": "internar",
    "internautas": "internauta",
    "interne": "internar",
    "internemos": "internar",
    "internen": "internar",
    "internes": "internar",
    "internistas": "internista",
    "interno": "internar",
    "internodios": "internodio",
    "internos": "interno",
    "internuncios": "internuncio",
    "internábamos": "internar",
    "internáis": "internar",
    "internáramos": "internar",
    "internáremos": "internar",
    "internásemos": "internar",
    "interné": "internar",
    "internéis": "internar",
    "internó": "internar",
    "interoceptiva": "interoceptivo",
    "interoceptivas": "interoceptivo",
    "interoceptivos": "interoceptivo",
    "interoceánica": "interoceánico",
    "interoceánicas": "interoceánico",
    "interoceánicos": "interoceánico",
    "interoclusales": "interoclusal",
    "interoculares": "interocular",
    "interoperabilidades": "interoperabilidad",
    "interorganisma": "interorganismo",
    "interorganismas": "interorganismo",
    "interorganismos": "interorganismo",
    "interorgánica": "interorgánico",
    "interorgánicas": "interorgánico",
    "interorgánicos": "interorgánico",
    "interpagina": "interpaginar",
    "interpaginaba": "interpaginar",
    "interpaginabais": "interpaginar",
    "interpaginaban": "interpaginar",
    "interpaginabas": "interpaginar",
    "interpaginad": "interpaginar",
    "interpaginada": "interpaginar",
    "interpaginadas": "interpaginar",
    "interpaginado": "interpaginar",
    "interpaginados": "interpaginar",
    "interpaginamos": "interpaginar",
    "interpaginan": "interpaginar",
    "interpaginando": "interpaginar",
    "interpaginara": "interpaginar",
    "interpaginarais": "interpaginar",
    "interpaginaran": "interpaginar",
    "interpaginaras": "interpaginar",
    "interpaginare": "interpaginar",
    "interpaginareis": "interpaginar",
    "interpaginaremos": "interpaginar",
    "interpaginaren": "interpaginar",
    "interpaginares": "interpaginar",
    "interpaginaron": "interpaginar",
    "interpaginará": "interpaginar",
    "interpaginarán": "interpaginar",
    "interpaginarás": "interpaginar",
    "interpaginaré": "interpaginar",
    "interpaginaréis": "interpaginar",
    "interpaginaría": "interpaginar",
    "interpaginaríais": "interpaginar",
    "interpaginaríamos": "interpaginar",
    "interpaginarían": "interpaginar",
    "interpaginarías": "interpaginar",
    "interpaginas": "interpaginar",
    "interpaginase": "interpaginar",
    "interpaginaseis": "interpaginar",
    "interpaginasen": "interpaginar",
    "interpaginases": "interpaginar",
    "interpaginaste": "interpaginar",
    "interpaginasteis": "interpaginar",
    "interpagine": "interpaginar",
    "interpaginemos": "interpaginar",
    "interpaginen": "interpaginar",
    "interpagines": "interpaginar",
    "interpagino": "interpaginar",
    "interpaginábamos": "interpaginar",
    "interpagináis": "interpaginar",
    "interpagináramos": "interpaginar",
    "interpagináremos": "interpaginar",
    "interpaginásemos": "interpaginar",
    "interpaginé": "interpaginar",
    "interpaginéis": "interpaginar",
    "interpaginó": "interpaginar",
    "interparadigmática": "interparadigmático",
    "interparadigmáticas": "interparadigmático",
    "interparadigmáticos": "interparadigmático",
    "interparietales": "interparietal",
    "interparlamentaria": "interparlamentario",
    "interparlamentarias": "interparlamentario",
    "interparlamentarios": "interparlamentario",
    "interpedunculares": "interpeduncular",
    "interpela": "interpelar",
    "interpelaba": "interpelar",
    "interpelabais": "interpelar",
    "interpelaban": "interpelar",
    "interpelabas": "interpelar",
    "interpelaciones": "interpelación",
    "interpelad": "interpelar",
    "interpelada": "interpelar",
    "interpeladas": "interpelar",
    "interpelado": "interpelar",
    "interpeladora": "interpelador",
    "interpeladoras": "interpelador",
    "interpeladores": "interpelador",
    "interpelados": "interpelar",
    "interpelamos": "interpelar",
    "interpelan": "interpelar",
    "interpelando": "interpelar",
    "interpelara": "interpelar",
    "interpelarais": "interpelar",
    "interpelaran": "interpelar",
    "interpelaras": "interpelar",
    "interpelare": "interpelar",
    "interpelareis": "interpelar",
    "interpelaremos": "interpelar",
    "interpelaren": "interpelar",
    "interpelares": "interpelar",
    "interpelaron": "interpelar",
    "interpelará": "interpelar",
    "interpelarán": "interpelar",
    "interpelarás": "interpelar",
    "interpelaré": "interpelar",
    "interpelaréis": "interpelar",
    "interpelaría": "interpelar",
    "interpelaríais": "interpelar",
    "interpelaríamos": "interpelar",
    "interpelarían": "interpelar",
    "interpelarías": "interpelar",
    "interpelas": "interpelar",
    "interpelase": "interpelar",
    "interpelaseis": "interpelar",
    "interpelasen": "interpelar",
    "interpelases": "interpelar",
    "interpelaste": "interpelar",
    "interpelasteis": "interpelar",
    "interpele": "interpelar",
    "interpelemos": "interpelar",
    "interpelen": "interpelar",
    "interpeles": "interpelar",
    "interpelo": "interpelar",
    "interpelábamos": "interpelar",
    "interpeláis": "interpelar",
    "interpeláramos": "interpelar",
    "interpeláremos": "interpelar",
    "interpelásemos": "interpelar",
    "interpelé": "interpelar",
    "interpeléis": "interpelar",
    "interpeló": "interpelar",
    "interpenetraciones": "interpenetración",
    "interpenetraciónes": "interpenetración",
    "interpersonales": "interpersonal",
    "interplanetaria": "interplanetario",
    "interplanetarias": "interplanetario",
    "interplanetarios": "interplanetario",
    "interpola": "interpolar",
    "interpolaba": "interpolar",
    "interpolabais": "interpolar",
    "interpolaban": "interpolar",
    "interpolabas": "interpolar",
    "interpolaciones": "interpolación",
    "interpolad": "interpolar",
    "interpolada": "interpolar",
    "interpoladas": "interpolar",
    "interpolado": "interpolar",
    "interpoladora": "interpolador",
    "interpoladoras": "interpolador",
    "interpoladores": "interpolador",
    "interpolados": "interpolar",
    "interpolamos": "interpolar",
    "interpolan": "interpolar",
    "interpolando": "interpolar",
    "interpolara": "interpolar",
    "interpolarais": "interpolar",
    "interpolaran": "interpolar",
    "interpolaras": "interpolar",
    "interpolare": "interpolar",
    "interpolareis": "interpolar",
    "interpolaremos": "interpolar",
    "interpolaren": "interpolar",
    "interpolares": "interpolar",
    "interpolaron": "interpolar",
    "interpolará": "interpolar",
    "interpolarán": "interpolar",
    "interpolarás": "interpolar",
    "interpolaré": "interpolar",
    "interpolaréis": "interpolar",
    "interpolaría": "interpolar",
    "interpolaríais": "interpolar",
    "interpolaríamos": "interpolar",
    "interpolarían": "interpolar",
    "interpolarías": "interpolar",
    "interpolas": "interpolar",
    "interpolase": "interpolar",
    "interpolaseis": "interpolar",
    "interpolasen": "interpolar",
    "interpolases": "interpolar",
    "interpolaste": "interpolar",
    "interpolasteis": "interpolar",
    "interpole": "interpolar",
    "interpolemos": "interpolar",
    "interpolen": "interpolar",
    "interpoles": "interpolar",
    "interpolo": "interpolar",
    "interpolábamos": "interpolar",
    "interpoláis": "interpolar",
    "interpoláramos": "interpolar",
    "interpoláremos": "interpolar",
    "interpolásemos": "interpolar",
    "interpolé": "interpolar",
    "interpoléis": "interpolar",
    "interpoló": "interpolar",
    "interpondremos": "interponer",
    "interpondrá": "interponer",
    "interpondrán": "interponer",
    "interpondrás": "interponer",
    "interpondré": "interponer",
    "interpondréis": "interponer",
    "interpondría": "interponer",
    "interpondríais": "interponer",
    "interpondríamos": "interponer",
    "interpondrían": "interponer",
    "interpondrías": "interponer",
    "interpone": "interponer",
    "interponed": "interponer",
    "interponemos": "interponer",
    "interponen": "interponer",
    "interpones": "interponer",
    "interponga": "interponer",
    "interpongamos": "interponer",
    "interpongan": "interponer",
    "interpongas": "interponer",
    "interpongo": "interponer",
    "interpongáis": "interponer",
    "interponiendo": "interponer",
    "interponéis": "interponer",
    "interponía": "interponer",
    "interponíais": "interponer",
    "interponíamos": "interponer",
    "interponían": "interponer",
    "interponías": "interponer",
    "interposiciones": "interposición",
    "interpreta": "interpretar",
    "interpretaba": "interpretar",
    "interpretabais": "interpretar",
    "interpretaban": "interpretar",
    "interpretabas": "interpretar",
    "interpretabilidades": "interpretabilidad",
    "interpretables": "interpretable",
    "interpretaciones": "interpretación",
    "interpretad": "interpretar",
    "interpretada": "interpretar",
    "interpretadas": "interpretar",
    "interpretado": "interpretar",
    "interpretadora": "interpretador",
    "interpretadoras": "interpretador",
    "interpretadores": "interpretador",
    "interpretados": "interpretar",
    "interpretamos": "interpretar",
    "interpretan": "interpretar",
    "interpretando": "interpretar",
    "interpretara": "interpretar",
    "interpretarais": "interpretar",
    "interpretaran": "interpretar",
    "interpretaras": "interpretar",
    "interpretare": "interpretar",
    "interpretareis": "interpretar",
    "interpretaremos": "interpretar",
    "interpretaren": "interpretar",
    "interpretares": "interpretar",
    "interpretaron": "interpretar",
    "interpretará": "interpretar",
    "interpretarán": "interpretar",
    "interpretarás": "interpretar",
    "interpretaré": "interpretar",
    "interpretaréis": "interpretar",
    "interpretaría": "interpretar",
    "interpretaríais": "interpretar",
    "interpretaríamos": "interpretar",
    "interpretarían": "interpretar",
    "interpretarías": "interpretar",
    "interpretas": "interpretar",
    "interpretase": "interpretar",
    "interpretaseis": "interpretar",
    "interpretasen": "interpretar",
    "interpretases": "interpretar",
    "interpretaste": "interpretar",
    "interpretasteis": "interpretar",
    "interpretativa": "interpretativo",
    "interpretativas": "interpretativo",
    "interpretativos": "interpretativo",
    "interprete": "interpretar",
    "interpretemos": "interpretar",
    "interpreten": "interpretar",
    "interpretes": "interpretar",
    "interpreto": "interpretar",
    "interpretábamos": "interpretar",
    "interpretáis": "interpretar",
    "interpretáramos": "interpretar",
    "interpretáremos": "interpretar",
    "interpretásemos": "interpretar",
    "interpreté": "interpretar",
    "interpretéis": "interpretar",
    "interpretó": "interpretar",
    "interprocesos": "interproceso",
    "interprofesionales": "interprofesional",
    "interprovinciales": "interprovincial",
    "interproximales": "interproximal",
    "interpuesta": "interponer",
    "interpuestas": "interponer",
    "interpuesto": "interponer",
    "interpuestos": "interponer",
    "interpupilares": "interpupilar",
    "interpuse": "interponer",
    "interpusiera": "interponer",
    "interpusierais": "interponer",
    "interpusieran": "interponer",
    "interpusieras": "interponer",
    "interpusiere": "interponer",
    "interpusiereis": "interponer",
    "interpusieren": "interponer",
    "interpusieres": "interponer",
    "interpusieron": "interponer",
    "interpusiese": "interponer",
    "interpusieseis": "interponer",
    "interpusiesen": "interponer",
    "interpusieses": "interponer",
    "interpusimos": "interponer",
    "interpusiste": "interponer",
    "interpusisteis": "interponer",
    "interpusiéramos": "interponer",
    "interpusiéremos": "interponer",
    "interpusiésemos": "interponer",
    "interpuso": "interponer",
    "interpón": "interponer",
    "interpúbica": "interpúbico",
    "interpúbicas": "interpúbico",
    "interpúbicos": "interpúbico",
    "interradiculares": "interradicular",
    "interregionales": "interregional",
    "interregnos": "interregno",
    "interrelaciona": "interrelacionar",
    "interrelacionaba": "interrelacionar",
    "interrelacionabais": "interrelacionar",
    "interrelacionaban": "interrelacionar",
    "interrelacionabas": "interrelacionar",
    "interrelacionad": "interrelacionar",
    "interrelacionada": "interrelacionar",
    "interrelacionadas": "interrelacionar",
    "interrelacionado": "interrelacionar",
    "interrelacionados": "interrelacionar",
    "interrelacionamos": "interrelacionar",
    "interrelacionan": "interrelacionar",
    "interrelacionando": "interrelacionar",
    "interrelacionara": "interrelacionar",
    "interrelacionarais": "interrelacionar",
    "interrelacionaran": "interrelacionar",
    "interrelacionaras": "interrelacionar",
    "interrelacionare": "interrelacionar",
    "interrelacionareis": "interrelacionar",
    "interrelacionaremos": "interrelacionar",
    "interrelacionaren": "interrelacionar",
    "interrelacionares": "interrelacionar",
    "interrelacionaron": "interrelacionar",
    "interrelacionará": "interrelacionar",
    "interrelacionarán": "interrelacionar",
    "interrelacionarás": "interrelacionar",
    "interrelacionaré": "interrelacionar",
    "interrelacionaréis": "interrelacionar",
    "interrelacionaría": "interrelacionar",
    "interrelacionaríais": "interrelacionar",
    "interrelacionaríamos": "interrelacionar",
    "interrelacionarían": "interrelacionar",
    "interrelacionarías": "interrelacionar",
    "interrelacionas": "interrelacionar",
    "interrelacionase": "interrelacionar",
    "interrelacionaseis": "interrelacionar",
    "interrelacionasen": "interrelacionar",
    "interrelacionases": "interrelacionar",
    "interrelacionaste": "interrelacionar",
    "interrelacionasteis": "interrelacionar",
    "interrelacione": "interrelacionar",
    "interrelacionemos": "interrelacionar",
    "interrelacionen": "interrelacionar",
    "interrelaciones": "interrelacionar",
    "interrelaciono": "interrelacionar",
    "interrelacionábamos": "interrelacionar",
    "interrelacionáis": "interrelacionar",
    "interrelacionáramos": "interrelacionar",
    "interrelacionáremos": "interrelacionar",
    "interrelacionásemos": "interrelacionar",
    "interrelacioné": "interrelacionar",
    "interrelacionéis": "interrelacionar",
    "interrelacionó": "interrelacionar",
    "interroga": "interrogar",
    "interrogaba": "interrogar",
    "interrogabais": "interrogar",
    "interrogaban": "interrogar",
    "interrogabas": "interrogar",
    "interrogaciones": "interrogación",
    "interrogad": "interrogar",
    "interrogada": "interrogar",
    "interrogadas": "interrogar",
    "interrogado": "interrogar",
    "interrogadora": "interrogador",
    "interrogadoras": "interrogador",
    "interrogadores": "interrogador",
    "interrogados": "interrogar",
    "interrogamos": "interrogar",
    "interrogan": "interrogar",
    "interrogando": "interrogar",
    "interrogantes": "interrogante",
    "interrogara": "interrogar",
    "interrogarais": "interrogar",
    "interrogaran": "interrogar",
    "interrogaras": "interrogar",
    "interrogare": "interrogar",
    "interrogareis": "interrogar",
    "interrogaremos": "interrogar",
    "interrogaren": "interrogar",
    "interrogares": "interrogar",
    "interrogaron": "interrogar",
    "interrogará": "interrogar",
    "interrogarán": "interrogar",
    "interrogarás": "interrogar",
    "interrogaré": "interrogar",
    "interrogaréis": "interrogar",
    "interrogaría": "interrogar",
    "interrogaríais": "interrogar",
    "interrogaríamos": "interrogar",
    "interrogarían": "interrogar",
    "interrogarías": "interrogar",
    "interrogas": "interrogar",
    "interrogase": "interrogar",
    "interrogaseis": "interrogar",
    "interrogasen": "interrogar",
    "interrogases": "interrogar",
    "interrogaste": "interrogar",
    "interrogasteis": "interrogar",
    "interrogativa": "interrogativo",
    "interrogativas": "interrogativo",
    "interrogativos": "interrogativo",
    "interrogatorios": "interrogatorio",
    "interrogo": "interrogar",
    "interrogue": "interrogar",
    "interroguemos": "interrogar",
    "interroguen": "interrogar",
    "interrogues": "interrogar",
    "interrogué": "interrogar",
    "interroguéis": "interrogar",
    "interrogábamos": "interrogar",
    "interrogáis": "interrogar",
    "interrogáramos": "interrogar",
    "interrogáremos": "interrogar",
    "interrogásemos": "interrogar",
    "interrogó": "interrogar",
    "interrumpa": "interrumpir",
    "interrumpamos": "interrumpir",
    "interrumpan": "interrumpir",
    "interrumpas": "interrumpir",
    "interrumpe": "interrumpir",
    "interrumpen": "interrumpir",
    "interrumpes": "interrumpir",
    "interrumpibilidades": "interrumpibilidad",
    "interrumpibles": "interrumpible",
    "interrumpid": "interrumpir",
    "interrumpida": "interrumpir",
    "interrumpidas": "interrumpir",
    "interrumpido": "interrumpir",
    "interrumpidos": "interrumpir",
    "interrumpiendo": "interrumpir",
    "interrumpiera": "interrumpir",
    "interrumpierais": "interrumpir",
    "interrumpieran": "interrumpir",
    "interrumpieras": "interrumpir",
    "interrumpiere": "interrumpir",
    "interrumpiereis": "interrumpir",
    "interrumpieren": "interrumpir",
    "interrumpieres": "interrumpir",
    "interrumpieron": "interrumpir",
    "interrumpiese": "interrumpir",
    "interrumpieseis": "interrumpir",
    "interrumpiesen": "interrumpir",
    "interrumpieses": "interrumpir",
    "interrumpimos": "interrumpir",
    "interrumpiremos": "interrumpir",
    "interrumpirá": "interrumpir",
    "interrumpirán": "interrumpir",
    "interrumpirás": "interrumpir",
    "interrumpiré": "interrumpir",
    "interrumpiréis": "interrumpir",
    "interrumpiría": "interrumpir",
    "interrumpiríais": "interrumpir",
    "interrumpiríamos": "interrumpir",
    "interrumpirían": "interrumpir",
    "interrumpirías": "interrumpir",
    "interrumpiste": "interrumpir",
    "interrumpisteis": "interrumpir",
    "interrumpiéramos": "interrumpir",
    "interrumpiéremos": "interrumpir",
    "interrumpiésemos": "interrumpir",
    "interrumpió": "interrumpir",
    "interrumpo": "interrumpir",
    "interrumpáis": "interrumpir",
    "interrumpí": "interrumpir",
    "interrumpía": "interrumpir",
    "interrumpíais": "interrumpir",
    "interrumpíamos": "interrumpir",
    "interrumpían": "interrumpir",
    "interrumpías": "interrumpir",
    "interrumpís": "interrumpir",
    "interrupciones": "interrupción",
    "interruptora": "interruptor",
    "interruptoras": "interruptor",
    "interruptores": "interruptor",
    "interseca": "intersecar",
    "intersecaba": "intersecar",
    "intersecabais": "intersecar",
    "intersecaban": "intersecar",
    "intersecabas": "intersecar",
    "intersecad": "intersecar",
    "intersecada": "intersecar",
    "intersecadas": "intersecar",
    "intersecado": "intersecar",
    "intersecados": "intersecar",
    "intersecamos": "intersecar",
    "intersecan": "intersecar",
    "intersecando": "intersecar",
    "intersecantes": "intersecante",
    "intersecara": "intersecar",
    "intersecarais": "intersecar",
    "intersecaran": "intersecar",
    "intersecaras": "intersecar",
    "intersecare": "intersecar",
    "intersecareis": "intersecar",
    "intersecaremos": "intersecar",
    "intersecaren": "intersecar",
    "intersecares": "intersecar",
    "intersecaron": "intersecar",
    "intersecará": "intersecar",
    "intersecarán": "intersecar",
    "intersecarás": "intersecar",
    "intersecaré": "intersecar",
    "intersecaréis": "intersecar",
    "intersecaría": "intersecar",
    "intersecaríais": "intersecar",
    "intersecaríamos": "intersecar",
    "intersecarían": "intersecar",
    "intersecarías": "intersecar",
    "intersecas": "intersecar",
    "intersecase": "intersecar",
    "intersecaseis": "intersecar",
    "intersecasen": "intersecar",
    "intersecases": "intersecar",
    "intersecaste": "intersecar",
    "intersecasteis": "intersecar",
    "intersecciones": "intersección",
    "interseco": "intersecar",
    "intersecábamos": "intersecar",
    "intersecáis": "intersecar",
    "intersecáramos": "intersecar",
    "intersecáremos": "intersecar",
    "intersecásemos": "intersecar",
    "intersecó": "intersecar",
    "interseque": "intersecar",
    "intersequemos": "intersecar",
    "intersequen": "intersecar",
    "interseques": "intersecar",
    "intersequé": "intersecar",
    "intersequéis": "intersecar",
    "intersexuales": "intersexual",
    "intersexualidades": "intersexualidad",
    "intersigmoidea": "intersigmoideo",
    "intersigmoideas": "intersigmoideo",
    "intersigmoideos": "intersigmoideo",
    "intersolidaridades": "intersolidaridad",
    "intersticiales": "intersticial",
    "intersticios": "intersticio",
    "intersubjetiva": "intersubjetivo",
    "intersubjetivas": "intersubjetivo",
    "intersubjetividades": "intersubjetividad",
    "intersubjetivos": "intersubjetivo",
    "intertalámica": "intertalámico",
    "intertalámicas": "intertalámico",
    "intertalámicos": "intertalámico",
    "intertareas": "intertarea",
    "interterritoriales": "interterritorial",
    "intertipos": "intertipo",
    "intertransversa": "intertransverso",
    "intertransversas": "intertransverso",
    "intertransversos": "intertransverso",
    "intertrigos": "intertrigo",
    "intertrocantérea": "intertrocantéreo",
    "intertrocantéreas": "intertrocantéreo",
    "intertrocantéreos": "intertrocantéreo",
    "intertropicales": "intertropical",
    "intertuberculares": "intertubercular",
    "interuniversitaria": "interuniversitario",
    "interuniversitarias": "interuniversitario",
    "interuniversitarios": "interuniversitario",
    "interurbana": "interurbano",
    "interurbanas": "interurbano",
    "interurbanos": "interurbano",
    "interuretérica": "interuretérico",
    "interuretéricas": "interuretérico",
    "interuretéricos": "interuretérico",
    "intervaginales": "intervaginal",
    "intervalos": "intervalo",
    "interven": "intervenir",
    "intervenciones": "intervención",
    "intervencionismos": "intervencionismo",
    "intervencionistas": "intervencionista",
    "intervendremos": "intervenir",
    "intervendrá": "intervenir",
    "intervendrán": "intervenir",
    "intervendrás": "intervenir",
    "intervendré": "intervenir",
    "intervendréis": "intervenir",
    "intervendría": "intervenir",
    "intervendríais": "intervenir",
    "intervendríamos": "intervenir",
    "intervendrían": "intervenir",
    "intervendrías": "intervenir",
    "intervenga": "intervenir",
    "intervengamos": "intervenir",
    "intervengan": "intervenir",
    "intervengas": "intervenir",
    "intervengo": "intervenir",
    "intervengáis": "intervenir",
    "intervenid": "intervenir",
    "intervenida": "intervenir",
    "intervenidas": "intervenir",
    "intervenido": "intervenir",
    "intervenidos": "intervenir",
    "intervenimos": "intervenir",
    "intervenosa": "intervenoso",
    "intervenosas": "intervenoso",
    "intervenosos": "intervenoso",
    "interventora": "interventor",
    "interventoras": "interventor",
    "interventores": "interventor",
    "interventriculares": "interventricular",
    "intervenía": "intervenir",
    "interveníais": "intervenir",
    "interveníamos": "intervenir",
    "intervenían": "intervenir",
    "intervenías": "intervenir",
    "intervenís": "intervenir",
    "intervertebrales": "intervertebral",
    "interviene": "intervenir",
    "intervienen": "intervenir",
    "intervienes": "intervenir",
    "intervine": "intervenir",
    "interviniendo": "intervenir",
    "intervinientes": "interviniente",
    "interviniera": "intervenir",
    "intervinierais": "intervenir",
    "intervinieran": "intervenir",
    "intervinieras": "intervenir",
    "interviniere": "intervenir",
    "interviniereis": "intervenir",
    "intervinieren": "intervenir",
    "intervinieres": "intervenir",
    "intervinieron": "intervenir",
    "interviniese": "intervenir",
    "intervinieseis": "intervenir",
    "interviniesen": "intervenir",
    "intervinieses": "intervenir",
    "intervinimos": "intervenir",
    "interviniste": "intervenir",
    "intervinisteis": "intervenir",
    "interviniéramos": "intervenir",
    "interviniéremos": "intervenir",
    "interviniésemos": "intervenir",
    "intervino": "intervenir",
    "intervisiones": "intervisión",
    "intervius": "interviu",
    "interviuva": "interviuvar",
    "interviuvaba": "interviuvar",
    "interviuvabais": "interviuvar",
    "interviuvaban": "interviuvar",
    "interviuvabas": "interviuvar",
    "interviuvad": "interviuvar",
    "interviuvada": "interviuvar",
    "interviuvadas": "interviuvar",
    "interviuvado": "interviuvar",
    "interviuvados": "interviuvar",
    "interviuvamos": "interviuvar",
    "interviuvan": "interviuvar",
    "interviuvando": "interviuvar",
    "interviuvara": "interviuvar",
    "interviuvarais": "interviuvar",
    "interviuvaran": "interviuvar",
    "interviuvaras": "interviuvar",
    "interviuvare": "interviuvar",
    "interviuvareis": "interviuvar",
    "interviuvaremos": "interviuvar",
    "interviuvaren": "interviuvar",
    "interviuvares": "interviuvar",
    "interviuvaron": "interviuvar",
    "interviuvará": "interviuvar",
    "interviuvarán": "interviuvar",
    "interviuvarás": "interviuvar",
    "interviuvaré": "interviuvar",
    "interviuvaréis": "interviuvar",
    "interviuvaría": "interviuvar",
    "interviuvaríais": "interviuvar",
    "interviuvaríamos": "interviuvar",
    "interviuvarían": "interviuvar",
    "interviuvarías": "interviuvar",
    "interviuvas": "interviuvar",
    "interviuvase": "interviuvar",
    "interviuvaseis": "interviuvar",
    "interviuvasen": "interviuvar",
    "interviuvases": "interviuvar",
    "interviuvaste": "interviuvar",
    "interviuvasteis": "interviuvar",
    "interviuve": "interviuvar",
    "interviuvemos": "interviuvar",
    "interviuven": "interviuvar",
    "interviuves": "interviuvar",
    "interviuvo": "interviuvar",
    "interviuvábamos": "interviuvar",
    "interviuváis": "interviuvar",
    "interviuváramos": "interviuvar",
    "interviuváremos": "interviuvar",
    "interviuvásemos": "interviuvar",
    "interviuvé": "interviuvar",
    "interviuvéis": "interviuvar",
    "interviuvó": "interviuvar",
    "interviúses": "interviú",
    "intervocálica": "intervocálico",
    "intervocálicas": "intervocálico",
    "intervocálicos": "intervocálico",
    "interválica": "interválico",
    "interválicas": "interválico",
    "interválicos": "interválico",
    "interyacentes": "interyacente",
    "interósea": "interóseo",
    "interóseas": "interóseo",
    "interóseos": "interóseo",
    "intesta": "intestar",
    "intestaba": "intestar",
    "intestabais": "intestar",
    "intestaban": "intestar",
    "intestabas": "intestar",
    "intestad": "intestar",
    "intestada": "intestar",
    "intestadas": "intestar",
    "intestado": "intestar",
    "intestados": "intestar",
    "intestamos": "intestar",
    "intestan": "intestar",
    "intestando": "intestar",
    "intestara": "intestar",
    "intestarais": "intestar",
    "intestaran": "intestar",
    "intestaras": "intestar",
    "intestare": "intestar",
    "intestareis": "intestar",
    "intestaremos": "intestar",
    "intestaren": "intestar",
    "intestares": "intestar",
    "intestaron": "intestar",
    "intestará": "intestar",
    "intestarán": "intestar",
    "intestarás": "intestar",
    "intestaré": "intestar",
    "intestaréis": "intestar",
    "intestaría": "intestar",
    "intestaríais": "intestar",
    "intestaríamos": "intestar",
    "intestarían": "intestar",
    "intestarías": "intestar",
    "intestas": "intestar",
    "intestase": "intestar",
    "intestaseis": "intestar",
    "intestasen": "intestar",
    "intestases": "intestar",
    "intestaste": "intestar",
    "intestasteis": "intestar",
    "inteste": "intestar",
    "intestemos": "intestar",
    "intesten": "intestar",
    "intestes": "intestar",
    "intestina": "intestino",
    "intestinales": "intestinal",
    "intestinas": "intestino",
    "intestinos": "intestino",
    "intesto": "intestar",
    "intestábamos": "intestar",
    "intestáis": "intestar",
    "intestáramos": "intestar",
    "intestáremos": "intestar",
    "intestásemos": "intestar",
    "intesté": "intestar",
    "intestéis": "intestar",
    "intestó": "intestar",
    "intifadas": "intifada",
    "intima": "intimar",
    "intimaba": "intimar",
    "intimabais": "intimar",
    "intimaban": "intimar",
    "intimabas": "intimar",
    "intimaciones": "intimación",
    "intimad": "intimar",
    "intimada": "intimar",
    "intimadas": "intimar",
    "intimado": "intimar",
    "intimados": "intimar",
    "intimamos": "intimar",
    "intiman": "intimar",
    "intimando": "intimar",
    "intimara": "intimar",
    "intimarais": "intimar",
    "intimaran": "intimar",
    "intimaras": "intimar",
    "intimare": "intimar",
    "intimareis": "intimar",
    "intimaremos": "intimar",
    "intimaren": "intimar",
    "intimares": "intimar",
    "intimaron": "intimar",
    "intimará": "intimar",
    "intimarán": "intimar",
    "intimarás": "intimar",
    "intimaré": "intimar",
    "intimaréis": "intimar",
    "intimaría": "intimar",
    "intimaríais": "intimar",
    "intimaríamos": "intimar",
    "intimarían": "intimar",
    "intimarías": "intimar",
    "intimas": "intimar",
    "intimase": "intimar",
    "intimaseis": "intimar",
    "intimasen": "intimar",
    "intimases": "intimar",
    "intimaste": "intimar",
    "intimasteis": "intimar",
    "intimatoria": "intimatorio",
    "intimatorias": "intimatorio",
    "intimatorios": "intimatorio",
    "intime": "intimar",
    "intimemos": "intimar",
    "intimen": "intimar",
    "intimes": "intimar",
    "intimida": "intimidar",
    "intimidaba": "intimidar",
    "intimidabais": "intimidar",
    "intimidaban": "intimidar",
    "intimidabas": "intimidar",
    "intimidaciones": "intimidación",
    "intimidad": "intimidar",
    "intimidada": "intimidar",
    "intimidadas": "intimidar",
    "intimidades": "intimidad",
    "intimidado": "intimidar",
    "intimidados": "intimidar",
    "intimidamos": "intimidar",
    "intimidan": "intimidar",
    "intimidando": "intimidar",
    "intimidara": "intimidar",
    "intimidarais": "intimidar",
    "intimidaran": "intimidar",
    "intimidaras": "intimidar",
    "intimidare": "intimidar",
    "intimidareis": "intimidar",
    "intimidaremos": "intimidar",
    "intimidaren": "intimidar",
    "intimidares": "intimidar",
    "intimidaron": "intimidar",
    "intimidará": "intimidar",
    "intimidarán": "intimidar",
    "intimidarás": "intimidar",
    "intimidaré": "intimidar",
    "intimidaréis": "intimidar",
    "intimidaría": "intimidar",
    "intimidaríais": "intimidar",
    "intimidaríamos": "intimidar",
    "intimidarían": "intimidar",
    "intimidarías": "intimidar",
    "intimidas": "intimidar",
    "intimidase": "intimidar",
    "intimidaseis": "intimidar",
    "intimidasen": "intimidar",
    "intimidases": "intimidar",
    "intimidaste": "intimidar",
    "intimidasteis": "intimidar",
    "intimidatoria": "intimidatorio",
    "intimidatorias": "intimidatorio",
    "intimidatorios": "intimidatorio",
    "intimide": "intimidar",
    "intimidemos": "intimidar",
    "intimiden": "intimidar",
    "intimides": "intimidar",
    "intimido": "intimidar",
    "intimidábamos": "intimidar",
    "intimidáis": "intimidar",
    "intimidáramos": "intimidar",
    "intimidáremos": "intimidar",
    "intimidásemos": "intimidar",
    "intimidé": "intimidar",
    "intimidéis": "intimidar",
    "intimidó": "intimidar",
    "intimistas": "intimista",
    "intimo": "intimar",
    "intimábamos": "intimar",
    "intimáis": "intimar",
    "intimáramos": "intimar",
    "intimáremos": "intimar",
    "intimásemos": "intimar",
    "intimé": "intimar",
    "intiméis": "intimar",
    "intimó": "intimar",
    "intis": "inti",
    "intitula": "intitular",
    "intitulaba": "intitular",
    "intitulabais": "intitular",
    "intitulaban": "intitular",
    "intitulabas": "intitular",
    "intitulad": "intitular",
    "intitulada": "intitular",
    "intituladas": "intitular",
    "intitulado": "intitular",
    "intitulados": "intitular",
    "intitulamos": "intitular",
    "intitulan": "intitular",
    "intitulando": "intitular",
    "intitulara": "intitular",
    "intitularais": "intitular",
    "intitularan": "intitular",
    "intitularas": "intitular",
    "intitulare": "intitular",
    "intitulareis": "intitular",
    "intitularemos": "intitular",
    "intitularen": "intitular",
    "intitulares": "intitular",
    "intitularon": "intitular",
    "intitulará": "intitular",
    "intitularán": "intitular",
    "intitularás": "intitular",
    "intitularé": "intitular",
    "intitularéis": "intitular",
    "intitularía": "intitular",
    "intitularíais": "intitular",
    "intitularíamos": "intitular",
    "intitularían": "intitular",
    "intitularías": "intitular",
    "intitulas": "intitular",
    "intitulase": "intitular",
    "intitulaseis": "intitular",
    "intitulasen": "intitular",
    "intitulases": "intitular",
    "intitulaste": "intitular",
    "intitulasteis": "intitular",
    "intitule": "intitular",
    "intitulemos": "intitular",
    "intitulen": "intitular",
    "intitules": "intitular",
    "intitulo": "intitular",
    "intitulábamos": "intitular",
    "intituláis": "intitular",
    "intituláramos": "intitular",
    "intituláremos": "intitular",
    "intitulásemos": "intitular",
    "intitulé": "intitular",
    "intituléis": "intitular",
    "intituló": "intitular",
    "intocables": "intocable",
    "intolerabilidades": "intolerabilidad",
    "intolerables": "intolerable",
    "intolerancias": "intolerancia",
    "intolerantes": "intolerante",
    "intonsa": "intonso",
    "intonsas": "intonso",
    "intonsos": "intonso",
    "intorsiones": "intorsión",
    "intoxica": "intoxicar",
    "intoxicaba": "intoxicar",
    "intoxicabais": "intoxicar",
    "intoxicaban": "intoxicar",
    "intoxicabas": "intoxicar",
    "intoxicaciones": "intoxicación",
    "intoxicad": "intoxicar",
    "intoxicada": "intoxicar",
    "intoxicadas": "intoxicar",
    "intoxicado": "intoxicar",
    "intoxicadora": "intoxicador",
    "intoxicadoras": "intoxicador",
    "intoxicadores": "intoxicador",
    "intoxicados": "intoxicar",
    "intoxicamos": "intoxicar",
    "intoxican": "intoxicar",
    "intoxicando": "intoxicar",
    "intoxicara": "intoxicar",
    "intoxicarais": "intoxicar",
    "intoxicaran": "intoxicar",
    "intoxicaras": "intoxicar",
    "intoxicare": "intoxicar",
    "intoxicareis": "intoxicar",
    "intoxicaremos": "intoxicar",
    "intoxicaren": "intoxicar",
    "intoxicares": "intoxicar",
    "intoxicaron": "intoxicar",
    "intoxicará": "intoxicar",
    "intoxicarán": "intoxicar",
    "intoxicarás": "intoxicar",
    "intoxicaré": "intoxicar",
    "intoxicaréis": "intoxicar",
    "intoxicaría": "intoxicar",
    "intoxicaríais": "intoxicar",
    "intoxicaríamos": "intoxicar",
    "intoxicarían": "intoxicar",
    "intoxicarías": "intoxicar",
    "intoxicas": "intoxicar",
    "intoxicase": "intoxicar",
    "intoxicaseis": "intoxicar",
    "intoxicasen": "intoxicar",
    "intoxicases": "intoxicar",
    "intoxicaste": "intoxicar",
    "intoxicasteis": "intoxicar",
    "intoxico": "intoxicar",
    "intoxicábamos": "intoxicar",
    "intoxicáis": "intoxicar",
    "intoxicáramos": "intoxicar",
    "intoxicáremos": "intoxicar",
    "intoxicásemos": "intoxicar",
    "intoxicó": "intoxicar",
    "intoxique": "intoxicar",
    "intoxiquemos": "intoxicar",
    "intoxiquen": "intoxicar",
    "intoxiques": "intoxicar",
    "intoxiqué": "intoxicar",
    "intoxiquéis": "intoxicar",
    "intraalveolares": "intraalveolar",
    "intraarticulares": "intraarticular",
    "intraauriculares": "intraauricular",
    "intracastigos": "intracastigo",
    "intracelulares": "intracelular",
    "intracomunitaria": "intracomunitario",
    "intracomunitarias": "intracomunitario",
    "intracomunitarios": "intracomunitario",
    "intracoronales": "intracoronal",
    "intracoronaria": "intracoronario",
    "intracoronarias": "intracoronario",
    "intracoronarios": "intracoronario",
    "intracristalina": "intracristalino",
    "intracristalinas": "intracristalino",
    "intracristalinos": "intracristalino",
    "intradermorreacciones": "intradermorreacción",
    "intradoses": "intradós",
    "intraducibilidades": "intraducibilidad",
    "intraducibles": "intraducible",
    "intradérmica": "intradérmico",
    "intradérmicas": "intradérmico",
    "intradérmicos": "intradérmico",
    "intradóses": "intradós",
    "intraempresariales": "intraempresarial",
    "intraepiteliales": "intraepitelial",
    "intraespecífica": "intraespecífico",
    "intraespecíficas": "intraespecífico",
    "intraespecíficos": "intraespecífico",
    "intraestatales": "intraestatal",
    "intrafirmas": "intrafirma",
    "intragénica": "intragénico",
    "intragénicas": "intragénico",
    "intragénicos": "intragénico",
    "intraindustriales": "intraindustrial",
    "intraligamentosa": "intraligamentoso",
    "intraligamentosas": "intraligamentoso",
    "intraligamentosos": "intraligamentoso",
    "intramembranosa": "intramembranoso",
    "intramembranosas": "intramembranoso",
    "intramembranosos": "intramembranoso",
    "intramoleculares": "intramolecular",
    "intramusculares": "intramuscular",
    "intranquila": "intranquilo",
    "intranquilas": "intranquilo",
    "intranquilice": "intranquilizar",
    "intranquilicemos": "intranquilizar",
    "intranquilicen": "intranquilizar",
    "intranquilices": "intranquilizar",
    "intranquilicé": "intranquilizar",
    "intranquilicéis": "intranquilizar",
    "intranquilidades": "intranquilidad",
    "intranquiliza": "intranquilizar",
    "intranquilizaba": "intranquilizar",
    "intranquilizabais": "intranquilizar",
    "intranquilizaban": "intranquilizar",
    "intranquilizabas": "intranquilizar",
    "intranquilizad": "intranquilizar",
    "intranquilizada": "intranquilizar",
    "intranquilizadas": "intranquilizar",
    "intranquilizado": "intranquilizar",
    "intranquilizadora": "intranquilizador",
    "intranquilizadoras": "intranquilizador",
    "intranquilizadores": "intranquilizador",
    "intranquilizados": "intranquilizar",
    "intranquilizamos": "intranquilizar",
    "intranquilizan": "intranquilizar",
    "intranquilizando": "intranquilizar",
    "intranquilizara": "intranquilizar",
    "intranquilizarais": "intranquilizar",
    "intranquilizaran": "intranquilizar",
    "intranquilizaras": "intranquilizar",
    "intranquilizare": "intranquilizar",
    "intranquilizareis": "intranquilizar",
    "intranquilizaremos": "intranquilizar",
    "intranquilizaren": "intranquilizar",
    "intranquilizares": "intranquilizar",
    "intranquilizaron": "intranquilizar",
    "intranquilizará": "intranquilizar",
    "intranquilizarán": "intranquilizar",
    "intranquilizarás": "intranquilizar",
    "intranquilizaré": "intranquilizar",
    "intranquilizaréis": "intranquilizar",
    "intranquilizaría": "intranquilizar",
    "intranquilizaríais": "intranquilizar",
    "intranquilizaríamos": "intranquilizar",
    "intranquilizarían": "intranquilizar",
    "intranquilizarías": "intranquilizar",
    "intranquilizas": "intranquilizar",
    "intranquilizase": "intranquilizar",
    "intranquilizaseis": "intranquilizar",
    "intranquilizasen": "intranquilizar",
    "intranquilizases": "intranquilizar",
    "intranquilizaste": "intranquilizar",
    "intranquilizasteis": "intranquilizar",
    "intranquilizo": "intranquilizar",
    "intranquilizábamos": "intranquilizar",
    "intranquilizáis": "intranquilizar",
    "intranquilizáramos": "intranquilizar",
    "intranquilizáremos": "intranquilizar",
    "intranquilizásemos": "intranquilizar",
    "intranquilizó": "intranquilizar",
    "intranquilos": "intranquilo",
    "intransferibles": "intransferible",
    "intransigencias": "intransigencia",
    "intransigentes": "intransigente",
    "intransitables": "intransitable",
    "intransitiva": "intransitivo",
    "intransitivas": "intransitivo",
    "intransitivos": "intransitivo",
    "intransmisibles": "intransmisible",
    "intransmutabilidades": "intransmutabilidad",
    "intransmutables": "intransmutable",
    "intraoccipitales": "intraoccipital",
    "intraoculares": "intraocular",
    "intraorales": "intraoral",
    "intraparadigmática": "intraparadigmático",
    "intraparadigmáticas": "intraparadigmático",
    "intraparadigmáticos": "intraparadigmático",
    "intraparietales": "intraparietal",
    "intraperióstica": "intraperióstico",
    "intraperiósticas": "intraperióstico",
    "intraperiósticos": "intraperióstico",
    "intrapulpares": "intrapulpar",
    "intrarraquídea": "intrarraquídeo",
    "intrarraquídeas": "intrarraquídeo",
    "intrarraquídeos": "intrarraquídeo",
    "intrarregionales": "intrarregional",
    "intrascendencias": "intrascendencia",
    "intrascendentales": "intrascendental",
    "intrascendentes": "intrascendente",
    "intraseptales": "intraseptal",
    "intratables": "intratable",
    "intrauterina": "intrauterino",
    "intrauterinas": "intrauterino",
    "intrauterinos": "intrauterino",
    "intravasculares": "intravascular",
    "intravenosa": "intravenoso",
    "intravenosas": "intravenoso",
    "intravenosos": "intravenoso",
    "intrayugulares": "intrayugular",
    "intrazonales": "intrazonal",
    "intraósea": "intraóseo",
    "intraóseas": "intraóseo",
    "intraóseos": "intraóseo",
    "intrepideces": "intrepidez",
    "intriga": "intrigar",
    "intrigaba": "intrigar",
    "intrigabais": "intrigar",
    "intrigaban": "intrigar",
    "intrigabas": "intrigar",
    "intrigad": "intrigar",
    "intrigada": "intrigar",
    "intrigadas": "intrigar",
    "intrigado": "intrigar",
    "intrigados": "intrigar",
    "intrigamos": "intrigar",
    "intrigan": "intrigar",
    "intrigando": "intrigar",
    "intrigantes": "intrigante",
    "intrigara": "intrigar",
    "intrigarais": "intrigar",
    "intrigaran": "intrigar",
    "intrigaras": "intrigar",
    "intrigare": "intrigar",
    "intrigareis": "intrigar",
    "intrigaremos": "intrigar",
    "intrigaren": "intrigar",
    "intrigares": "intrigar",
    "intrigaron": "intrigar",
    "intrigará": "intrigar",
    "intrigarán": "intrigar",
    "intrigarás": "intrigar",
    "intrigaré": "intrigar",
    "intrigaréis": "intrigar",
    "intrigaría": "intrigar",
    "intrigaríais": "intrigar",
    "intrigaríamos": "intrigar",
    "intrigarían": "intrigar",
    "intrigarías": "intrigar",
    "intrigas": "intrigar",
    "intrigase": "intrigar",
    "intrigaseis": "intrigar",
    "intrigasen": "intrigar",
    "intrigases": "intrigar",
    "intrigaste": "intrigar",
    "intrigasteis": "intrigar",
    "intrigo": "intrigar",
    "intrigue": "intrigar",
    "intriguemos": "intrigar",
    "intriguen": "intrigar",
    "intrigues": "intrigar",
    "intrigué": "intrigar",
    "intriguéis": "intrigar",
    "intrigábamos": "intrigar",
    "intrigáis": "intrigar",
    "intrigáramos": "intrigar",
    "intrigáremos": "intrigar",
    "intrigásemos": "intrigar",
    "intrigó": "intrigar",
    "intrinca": "intrincar",
    "intrincaba": "intrincar",
    "intrincabais": "intrincar",
    "intrincaban": "intrincar",
    "intrincabas": "intrincar",
    "intrincables": "intrincable",
    "intrincaciones": "intrincación",
    "intrincad": "intrincar",
    "intrincada": "intrincar",
    "intrincadas": "intrincar",
    "intrincado": "intrincar",
    "intrincados": "intrincar",
    "intrincamientos": "intrincamiento",
    "intrincamos": "intrincar",
    "intrincan": "intrincar",
    "intrincando": "intrincar",
    "intrincara": "intrincar",
    "intrincarais": "intrincar",
    "intrincaran": "intrincar",
    "intrincaras": "intrincar",
    "intrincare": "intrincar",
    "intrincareis": "intrincar",
    "intrincaremos": "intrincar",
    "intrincaren": "intrincar",
    "intrincares": "intrincar",
    "intrincaron": "intrincar",
    "intrincará": "intrincar",
    "intrincarán": "intrincar",
    "intrincarás": "intrincar",
    "intrincaré": "intrincar",
    "intrincaréis": "intrincar",
    "intrincaría": "intrincar",
    "intrincaríais": "intrincar",
    "intrincaríamos": "intrincar",
    "intrincarían": "intrincar",
    "intrincarías": "intrincar",
    "intrincas": "intrincar",
    "intrincase": "intrincar",
    "intrincaseis": "intrincar",
    "intrincasen": "intrincar",
    "intrincases": "intrincar",
    "intrincaste": "intrincar",
    "intrincasteis": "intrincar",
    "intrinco": "intrincar",
    "intrincábamos": "intrincar",
    "intrincáis": "intrincar",
    "intrincáramos": "intrincar",
    "intrincáremos": "intrincar",
    "intrincásemos": "intrincar",
    "intrincó": "intrincar",
    "intrinque": "intrincar",
    "intrinquemos": "intrincar",
    "intrinquen": "intrincar",
    "intrinques": "intrincar",
    "intrinqué": "intrincar",
    "intrinquéis": "intrincar",
    "introducciones": "introducción",
    "introduce": "introducir",
    "introducen": "introducir",
    "introduces": "introducir",
    "introducibles": "introducible",
    "introducid": "introducir",
    "introducida": "introducir",
    "introducidas": "introducir",
    "introducido": "introducir",
    "introducidos": "introducir",
    "introduciendo": "introducir",
    "introducimos": "introducir",
    "introduciremos": "introducir",
    "introducirá": "introducir",
    "introducirán": "introducir",
    "introducirás": "introducir",
    "introduciré": "introducir",
    "introduciréis": "introducir",
    "introduciría": "introducir",
    "introduciríais": "introducir",
    "introduciríamos": "introducir",
    "introducirían": "introducir",
    "introducirías": "introducir",
    "introductora": "introductor",
    "introductoras": "introductor",
    "introductores": "introductor",
    "introductoria": "introductorio",
    "introductorias": "introductorio",
    "introductorios": "introductorio",
    "introducía": "introducir",
    "introducíais": "introducir",
    "introducíamos": "introducir",
    "introducían": "introducir",
    "introducías": "introducir",
    "introducís": "introducir",
    "introduje": "introducir",
    "introdujera": "introducir",
    "introdujerais": "introducir",
    "introdujeran": "introducir",
    "introdujeras": "introducir",
    "introdujere": "introducir",
    "introdujereis": "introducir",
    "introdujeren": "introducir",
    "introdujeres": "introducir",
    "introdujeron": "introducir",
    "introdujese": "introducir",
    "introdujeseis": "introducir",
    "introdujesen": "introducir",
    "introdujeses": "introducir",
    "introdujimos": "introducir",
    "introdujiste": "introducir",
    "introdujisteis": "introducir",
    "introdujo": "introducir",
    "introdujéramos": "introducir",
    "introdujéremos": "introducir",
    "introdujésemos": "introducir",
    "introduzca": "introducir",
    "introduzcamos": "introducir",
    "introduzcan": "introducir",
    "introduzcas": "introducir",
    "introduzco": "introducir",
    "introduzcáis": "introducir",
    "introitos": "introito",
    "intromisiones": "intromisión",
    "intromisiva": "intromisivo",
    "intromisivas": "intromisivo",
    "intromisivos": "intromisivo",
    "intrones": "intrón",
    "introspecciones": "introspección",
    "introspectiva": "introspectivo",
    "introspectivas": "introspectivo",
    "introspectivos": "introspectivo",
    "introversa": "introverso",
    "introversas": "introverso",
    "introversiones": "introversión",
    "introversos": "introverso",
    "introvertida": "introvertido",
    "introvertidas": "introvertido",
    "introvertidos": "introvertido",
    "introyecciones": "introyección",
    "intrusa": "intruso",
    "intrusas": "intruso",
    "intrusiones": "intrusión",
    "intrusismos": "intrusismo",
    "intrusiva": "intrusivo",
    "intrusivas": "intrusivo",
    "intrusivos": "intrusivo",
    "intrusos": "intruso",
    "intrépida": "intrépido",
    "intrépidas": "intrépido",
    "intrépidos": "intrépido",
    "intrínseca": "intrínseco",
    "intrínsecas": "intrínseco",
    "intrínsecos": "intrínseco",
    "intuba": "intubar",
    "intubaba": "intubar",
    "intubabais": "intubar",
    "intubaban": "intubar",
    "intubabas": "intubar",
    "intubaciones": "intubación",
    "intubad": "intubar",
    "intubada": "intubar",
    "intubadas": "intubar",
    "intubado": "intubar",
    "intubadores": "intubador",
    "intubados": "intubar",
    "intubamos": "intubar",
    "intuban": "intubar",
    "intubando": "intubar",
    "intubara": "intubar",
    "intubarais": "intubar",
    "intubaran": "intubar",
    "intubaras": "intubar",
    "intubare": "intubar",
    "intubareis": "intubar",
    "intubaremos": "intubar",
    "intubaren": "intubar",
    "intubares": "intubar",
    "intubaron": "intubar",
    "intubará": "intubar",
    "intubarán": "intubar",
    "intubarás": "intubar",
    "intubaré": "intubar",
    "intubaréis": "intubar",
    "intubaría": "intubar",
    "intubaríais": "intubar",
    "intubaríamos": "intubar",
    "intubarían": "intubar",
    "intubarías": "intubar",
    "intubas": "intubar",
    "intubase": "intubar",
    "intubaseis": "intubar",
    "intubasen": "intubar",
    "intubases": "intubar",
    "intubaste": "intubar",
    "intubasteis": "intubar",
    "intube": "intubar",
    "intubemos": "intubar",
    "intuben": "intubar",
    "intubes": "intubar",
    "intubo": "intubar",
    "intubábamos": "intubar",
    "intubáis": "intubar",
    "intubáramos": "intubar",
    "intubáremos": "intubar",
    "intubásemos": "intubar",
    "intubé": "intubar",
    "intubéis": "intubar",
    "intubó": "intubar",
    "intuiciones": "intuición",
    "intuicionismos": "intuicionismo",
    "intuicionistas": "intuicionista",
    "intuid": "intuir",
    "intuida": "intuir",
    "intuidas": "intuir",
    "intuido": "intuir",
    "intuidos": "intuir",
    "intuimos": "intuir",
    "intuiremos": "intuir",
    "intuirá": "intuir",
    "intuirán": "intuir",
    "intuirás": "intuir",
    "intuiré": "intuir",
    "intuiréis": "intuir",
    "intuiría": "intuir",
    "intuiríais": "intuir",
    "intuiríamos": "intuir",
    "intuirían": "intuir",
    "intuirías": "intuir",
    "intuiste": "intuir",
    "intuisteis": "intuir",
    "intuitiva": "intuitivo",
    "intuitivas": "intuitivo",
    "intuitivos": "intuitivo",
    "intuitos": "intuito",
    "intumescencias": "intumescencia",
    "intumescentes": "intumescente",
    "intususcepciones": "intususcepción",
    "intuya": "intuir",
    "intuyamos": "intuir",
    "intuyan": "intuir",
    "intuyas": "intuir",
    "intuye": "intuir",
    "intuyen": "intuir",
    "intuyendo": "intuir",
    "intuyera": "intuir",
    "intuyerais": "intuir",
    "intuyeran": "intuir",
    "intuyeras": "intuir",
    "intuyere": "intuir",
    "intuyereis": "intuir",
    "intuyeren": "intuir",
    "intuyeres": "intuir",
    "intuyeron": "intuir",
    "intuyes": "intuir",
    "intuyese": "intuir",
    "intuyeseis": "intuir",
    "intuyesen": "intuir",
    "intuyeses": "intuir",
    "intuyo": "intuir",
    "intuyáis": "intuir",
    "intuyéramos": "intuir",
    "intuyéremos": "intuir",
    "intuyésemos": "intuir",
    "intuyó": "intuir",
    "intuí": "intuir",
    "intuía": "intuir",
    "intuíais": "intuir",
    "intuíamos": "intuir",
    "intuían": "intuir",
    "intuías": "intuir",
    "intuís": "intuir",
    "intégrafos": "intégrafo",
    "intérpretes": "intérprete",
    "inulinas": "inulina",
    "inunda": "inundar",
    "inundaba": "inundar",
    "inundabais": "inundar",
    "inundaban": "inundar",
    "inundabas": "inundar",
    "inundables": "inundable",
    "inundaciones": "inundación",
    "inundad": "inundar",
    "inundada": "inundar",
    "inundadas": "inundar",
    "inundado": "inundar",
    "inundados": "inundar",
    "inundamos": "inundar",
    "inundan": "inundar",
    "inundando": "inundar",
    "inundara": "inundar",
    "inundarais": "inundar",
    "inundaran": "inundar",
    "inundaras": "inundar",
    "inundare": "inundar",
    "inundareis": "inundar",
    "inundaremos": "inundar",
    "inundaren": "inundar",
    "inundares": "inundar",
    "inundaron": "inundar",
    "inundará": "inundar",
    "inundarán": "inundar",
    "inundarás": "inundar",
    "inundaré": "inundar",
    "inundaréis": "inundar",
    "inundaría": "inundar",
    "inundaríais": "inundar",
    "inundaríamos": "inundar",
    "inundarían": "inundar",
    "inundarías": "inundar",
    "inundas": "inundar",
    "inundase": "inundar",
    "inundaseis": "inundar",
    "inundasen": "inundar",
    "inundases": "inundar",
    "inundaste": "inundar",
    "inundasteis": "inundar",
    "inunde": "inundar",
    "inundemos": "inundar",
    "inunden": "inundar",
    "inundes": "inundar",
    "inundo": "inundar",
    "inundábamos": "inundar",
    "inundáis": "inundar",
    "inundáramos": "inundar",
    "inundáremos": "inundar",
    "inundásemos": "inundar",
    "inundé": "inundar",
    "inundéis": "inundar",
    "inundó": "inundar",
    "inusitada": "inusitado",
    "inusitadas": "inusitado",
    "inusitados": "inusitado",
    "inusuales": "inusual",
    "inutilice": "inutilizar",
    "inutilicemos": "inutilizar",
    "inutilicen": "inutilizar",
    "inutilices": "inutilizar",
    "inutilicé": "inutilizar",
    "inutilicéis": "inutilizar",
    "inutilidades": "inutilidad",
    "inutiliza": "inutilizar",
    "inutilizaba": "inutilizar",
    "inutilizabais": "inutilizar",
    "inutilizaban": "inutilizar",
    "inutilizabas": "inutilizar",
    "inutilizables": "inutilizable",
    "inutilizaciones": "inutilización",
    "inutilizad": "inutilizar",
    "inutilizada": "inutilizar",
    "inutilizadas": "inutilizar",
    "inutilizado": "inutilizar",
    "inutilizados": "inutilizar",
    "inutilizamos": "inutilizar",
    "inutilizan": "inutilizar",
    "inutilizando": "inutilizar",
    "inutilizara": "inutilizar",
    "inutilizarais": "inutilizar",
    "inutilizaran": "inutilizar",
    "inutilizaras": "inutilizar",
    "inutilizare": "inutilizar",
    "inutilizareis": "inutilizar",
    "inutilizaremos": "inutilizar",
    "inutilizaren": "inutilizar",
    "inutilizares": "inutilizar",
    "inutilizaron": "inutilizar",
    "inutilizará": "inutilizar",
    "inutilizarán": "inutilizar",
    "inutilizarás": "inutilizar",
    "inutilizaré": "inutilizar",
    "inutilizaréis": "inutilizar",
    "inutilizaría": "inutilizar",
    "inutilizaríais": "inutilizar",
    "inutilizaríamos": "inutilizar",
    "inutilizarían": "inutilizar",
    "inutilizarías": "inutilizar",
    "inutilizas": "inutilizar",
    "inutilizase": "inutilizar",
    "inutilizaseis": "inutilizar",
    "inutilizasen": "inutilizar",
    "inutilizases": "inutilizar",
    "inutilizaste": "inutilizar",
    "inutilizasteis": "inutilizar",
    "inutilizo": "inutilizar",
    "inutilizábamos": "inutilizar",
    "inutilizáis": "inutilizar",
    "inutilizáramos": "inutilizar",
    "inutilizáremos": "inutilizar",
    "inutilizásemos": "inutilizar",
    "inutilizó": "inutilizar",
    "invada": "invadir",
    "invadamos": "invadir",
    "invadan": "invadir",
    "invadas": "invadir",
    "invade": "invadir",
    "invaden": "invadir",
    "invades": "invadir",
    "invadid": "invadir",
    "invadida": "invadir",
    "invadidas": "invadir",
    "invadido": "invadir",
    "invadidos": "invadir",
    "invadiendo": "invadir",
    "invadiera": "invadir",
    "invadierais": "invadir",
    "invadieran": "invadir",
    "invadieras": "invadir",
    "invadiere": "invadir",
    "invadiereis": "invadir",
    "invadieren": "invadir",
    "invadieres": "invadir",
    "invadieron": "invadir",
    "invadiese": "invadir",
    "invadieseis": "invadir",
    "invadiesen": "invadir",
    "invadieses": "invadir",
    "invadimos": "invadir",
    "invadiremos": "invadir",
    "invadirá": "invadir",
    "invadirán": "invadir",
    "invadirás": "invadir",
    "invadiré": "invadir",
    "invadiréis": "invadir",
    "invadiría": "invadir",
    "invadiríais": "invadir",
    "invadiríamos": "invadir",
    "invadirían": "invadir",
    "invadirías": "invadir",
    "invadiste": "invadir",
    "invadisteis": "invadir",
    "invadiéramos": "invadir",
    "invadiéremos": "invadir",
    "invadiésemos": "invadir",
    "invadió": "invadir",
    "invado": "invadir",
    "invadáis": "invadir",
    "invadí": "invadir",
    "invadía": "invadir",
    "invadíais": "invadir",
    "invadíamos": "invadir",
    "invadían": "invadir",
    "invadías": "invadir",
    "invadís": "invadir",
    "invagina": "invaginar",
    "invaginaba": "invaginar",
    "invaginabais": "invaginar",
    "invaginaban": "invaginar",
    "invaginabas": "invaginar",
    "invaginaciones": "invaginación",
    "invaginad": "invaginar",
    "invaginada": "invaginar",
    "invaginadas": "invaginar",
    "invaginado": "invaginar",
    "invaginados": "invaginar",
    "invaginamos": "invaginar",
    "invaginan": "invaginar",
    "invaginando": "invaginar",
    "invaginara": "invaginar",
    "invaginarais": "invaginar",
    "invaginaran": "invaginar",
    "invaginaras": "invaginar",
    "invaginare": "invaginar",
    "invaginareis": "invaginar",
    "invaginaremos": "invaginar",
    "invaginaren": "invaginar",
    "invaginares": "invaginar",
    "invaginaron": "invaginar",
    "invaginará": "invaginar",
    "invaginarán": "invaginar",
    "invaginarás": "invaginar",
    "invaginaré": "invaginar",
    "invaginaréis": "invaginar",
    "invaginaría": "invaginar",
    "invaginaríais": "invaginar",
    "invaginaríamos": "invaginar",
    "invaginarían": "invaginar",
    "invaginarías": "invaginar",
    "invaginas": "invaginar",
    "invaginase": "invaginar",
    "invaginaseis": "invaginar",
    "invaginasen": "invaginar",
    "invaginases": "invaginar",
    "invaginaste": "invaginar",
    "invaginasteis": "invaginar",
    "invagine": "invaginar",
    "invaginemos": "invaginar",
    "invaginen": "invaginar",
    "invagines": "invaginar",
    "invagino": "invaginar",
    "invaginábamos": "invaginar",
    "invagináis": "invaginar",
    "invagináramos": "invaginar",
    "invagináremos": "invaginar",
    "invaginásemos": "invaginar",
    "invaginé": "invaginar",
    "invaginéis": "invaginar",
    "invaginó": "invaginar",
    "invalida": "invalidar",
    "invalidaba": "invalidar",
    "invalidabais": "invalidar",
    "invalidaban": "invalidar",
    "invalidabas": "invalidar",
    "invalidaciones": "invalidación",
    "invalidaciónes": "invalidación",
    "invalidad": "invalidar",
    "invalidada": "invalidar",
    "invalidadas": "invalidar",
    "invalidado": "invalidar",
    "invalidados": "invalidar",
    "invalidamos": "invalidar",
    "invalidan": "invalidar",
    "invalidando": "invalidar",
    "invalidantes": "invalidante",
    "invalidara": "invalidar",
    "invalidarais": "invalidar",
    "invalidaran": "invalidar",
    "invalidaras": "invalidar",
    "invalidare": "invalidar",
    "invalidareis": "invalidar",
    "invalidaremos": "invalidar",
    "invalidaren": "invalidar",
    "invalidares": "invalidar",
    "invalidaron": "invalidar",
    "invalidará": "invalidar",
    "invalidarán": "invalidar",
    "invalidarás": "invalidar",
    "invalidaré": "invalidar",
    "invalidaréis": "invalidar",
    "invalidaría": "invalidar",
    "invalidaríais": "invalidar",
    "invalidaríamos": "invalidar",
    "invalidarían": "invalidar",
    "invalidarías": "invalidar",
    "invalidas": "invalidar",
    "invalidase": "invalidar",
    "invalidaseis": "invalidar",
    "invalidasen": "invalidar",
    "invalidases": "invalidar",
    "invalidaste": "invalidar",
    "invalidasteis": "invalidar",
    "invalide": "invalidar",
    "invalideces": "invalidez",
    "invalidemos": "invalidar",
    "invaliden": "invalidar",
    "invalides": "invalidar",
    "invalido": "invalidar",
    "invalidábamos": "invalidar",
    "invalidáis": "invalidar",
    "invalidáramos": "invalidar",
    "invalidáremos": "invalidar",
    "invalidásemos": "invalidar",
    "invalidé": "invalidar",
    "invalidéis": "invalidar",
    "invalidó": "invalidar",
    "invares": "invar",
    "invariabilidades": "invariabilidad",
    "invariables": "invariable",
    "invariaciones": "invariación",
    "invariada": "invariado",
    "invariadas": "invariado",
    "invariados": "invariado",
    "invariancias": "invariancia",
    "invariantes": "invariante",
    "invasiones": "invasión",
    "invasora": "invasor",
    "invasoras": "invasor",
    "invasores": "invasor",
    "invectivas": "invectiva",
    "invencibilidades": "invencibilidad",
    "invencibles": "invencible",
    "invenciones": "invención",
    "invendibles": "invendible",
    "inventa": "inventar",
    "inventaba": "inventar",
    "inventabais": "inventar",
    "inventaban": "inventar",
    "inventabas": "inventar",
    "inventad": "inventar",
    "inventada": "inventar",
    "inventadas": "inventar",
    "inventado": "inventar",
    "inventados": "inventar",
    "inventamos": "inventar",
    "inventan": "inventar",
    "inventando": "inventar",
    "inventara": "inventar",
    "inventarais": "inventar",
    "inventaran": "inventar",
    "inventaras": "inventar",
    "inventare": "inventar",
    "inventareis": "inventar",
    "inventaremos": "inventar",
    "inventaren": "inventar",
    "inventares": "inventar",
    "inventariaba": "inventariar",
    "inventariabais": "inventariar",
    "inventariaban": "inventariar",
    "inventariabas": "inventariar",
    "inventariaciones": "inventariación",
    "inventariad": "inventariar",
    "inventariada": "inventariar",
    "inventariadas": "inventariar",
    "inventariado": "inventariar",
    "inventariados": "inventariar",
    "inventariamos": "inventariar",
    "inventariando": "inventariar",
    "inventariara": "inventariar",
    "inventariarais": "inventariar",
    "inventariaran": "inventariar",
    "inventariaras": "inventariar",
    "inventariare": "inventariar",
    "inventariareis": "inventariar",
    "inventariaremos": "inventariar",
    "inventariaren": "inventariar",
    "inventariares": "inventariar",
    "inventariaron": "inventariar",
    "inventariará": "inventariar",
    "inventariarán": "inventariar",
    "inventariarás": "inventariar",
    "inventariaré": "inventariar",
    "inventariaréis": "inventariar",
    "inventariaría": "inventariar",
    "inventariaríais": "inventariar",
    "inventariaríamos": "inventariar",
    "inventariarían": "inventariar",
    "inventariarías": "inventariar",
    "inventariase": "inventariar",
    "inventariaseis": "inventariar",
    "inventariasen": "inventariar",
    "inventariases": "inventariar",
    "inventariaste": "inventariar",
    "inventariasteis": "inventariar",
    "inventariemos": "inventariar",
    "inventarios": "inventario",
    "inventariábamos": "inventariar",
    "inventariáis": "inventariar",
    "inventariáramos": "inventariar",
    "inventariáremos": "inventariar",
    "inventariásemos": "inventariar",
    "inventarié": "inventariar",
    "inventariéis": "inventariar",
    "inventarió": "inventariar",
    "inventaron": "inventar",
    "inventará": "inventar",
    "inventarán": "inventar",
    "inventarás": "inventar",
    "inventaré": "inventar",
    "inventaréis": "inventar",
    "inventaría": "inventariar",
    "inventaríais": "inventar",
    "inventaríamos": "inventar",
    "inventarían": "inventariar",
    "inventarías": "inventariar",
    "inventaríe": "inventariar",
    "inventaríen": "inventariar",
    "inventaríes": "inventariar",
    "inventarío": "inventariar",
    "inventas": "inventar",
    "inventase": "inventar",
    "inventaseis": "inventar",
    "inventasen": "inventar",
    "inventases": "inventar",
    "inventaste": "inventar",
    "inventasteis": "inventar",
    "invente": "inventar",
    "inventemos": "inventar",
    "inventen": "inventar",
    "inventes": "inventar",
    "inventiva": "inventivo",
    "inventivas": "inventivo",
    "inventivos": "inventivo",
    "invento": "inventar",
    "inventora": "inventor",
    "inventoras": "inventor",
    "inventores": "inventor",
    "inventos": "invento",
    "inventábamos": "inventar",
    "inventáis": "inventar",
    "inventáramos": "inventar",
    "inventáremos": "inventar",
    "inventásemos": "inventar",
    "inventé": "inventar",
    "inventéis": "inventar",
    "inventó": "inventar",
    "inverecunda": "inverecundo",
    "inverecundas": "inverecundo",
    "inverecundias": "inverecundia",
    "inverecundos": "inverecundo",
    "inverna": "invernar",
    "invernaba": "invernar",
    "invernabais": "invernar",
    "invernaban": "invernar",
    "invernabas": "invernar",
    "invernad": "invernar",
    "invernada": "invernar",
    "invernadas": "invernar",
    "invernaderos": "invernadero",
    "invernado": "invernar",
    "invernados": "invernar",
    "invernales": "invernal",
    "invernamos": "invernar",
    "invernan": "invernar",
    "invernando": "invernar",
    "invernara": "invernar",
    "invernarais": "invernar",
    "invernaran": "invernar",
    "invernaras": "invernar",
    "invernare": "invernar",
    "invernareis": "invernar",
    "invernaremos": "invernar",
    "invernaren": "invernar",
    "invernares": "invernar",
    "invernaron": "invernar",
    "invernará": "invernar",
    "invernarán": "invernar",
    "invernarás": "invernar",
    "invernaré": "invernar",
    "invernaréis": "invernar",
    "invernaría": "invernar",
    "invernaríais": "invernar",
    "invernaríamos": "invernar",
    "invernarían": "invernar",
    "invernarías": "invernar",
    "invernas": "invernar",
    "invernase": "invernar",
    "invernaseis": "invernar",
    "invernasen": "invernar",
    "invernases": "invernar",
    "invernaste": "invernar",
    "invernasteis": "invernar",
    "inverne": "invernar",
    "invernemos": "invernar",
    "invernen": "invernar",
    "invernes": "invernar",
    "inverniza": "invernizo",
    "invernizas": "invernizo",
    "invernizos": "invernizo",
    "inverno": "invernar",
    "invernábamos": "invernar",
    "invernáculos": "invernáculo",
    "invernáis": "invernar",
    "invernáramos": "invernar",
    "invernáremos": "invernar",
    "invernásemos": "invernar",
    "inverné": "invernar",
    "invernéis": "invernar",
    "invernó": "invernar",
    "inverosimilitudes": "inverosimilitud",
    "inverosímiles": "inverosímil",
    "inversa": "inverso",
    "inversas": "inverso",
    "inversiones": "inversión",
    "inversionistas": "inversionista",
    "inversora": "inversor",
    "inversoras": "inversor",
    "inversores": "inversor",
    "inversos": "inverso",
    "invertasas": "invertasa",
    "invertebrada": "invertebrado",
    "invertebradas": "invertebrado",
    "invertebrados": "invertebrado",
    "invertibles": "invertible",
    "invertid": "invertir",
    "invertida": "invertir",
    "invertidas": "invertir",
    "invertido": "invertir",
    "invertidos": "invertir",
    "invertimos": "invertir",
    "invertiremos": "invertir",
    "invertirá": "invertir",
    "invertirán": "invertir",
    "invertirás": "invertir",
    "invertiré": "invertir",
    "invertiréis": "invertir",
    "invertiría": "invertir",
    "invertiríais": "invertir",
    "invertiríamos": "invertir",
    "invertirían": "invertir",
    "invertirías": "invertir",
    "invertiste": "invertir",
    "invertisteis": "invertir",
    "invertí": "invertir",
    "invertía": "invertir",
    "invertíais": "invertir",
    "invertíamos": "invertir",
    "invertían": "invertir",
    "invertías": "invertir",
    "invertís": "invertir",
    "investid": "investir",
    "investida": "investir",
    "investidas": "investir",
    "investido": "investir",
    "investidos": "investir",
    "investiduras": "investidura",
    "investiga": "investigar",
    "investigaba": "investigar",
    "investigabais": "investigar",
    "investigaban": "investigar",
    "investigabas": "investigar",
    "investigables": "investigable",
    "investigaciones": "investigación",
    "investigad": "investigar",
    "investigada": "investigar",
    "investigadas": "investigar",
    "investigado": "investigar",
    "investigadora": "investigador",
    "investigadoras": "investigador",
    "investigadores": "investigador",
    "investigados": "investigar",
    "investigamos": "investigar",
    "investigan": "investigar",
    "investigando": "investigar",
    "investigara": "investigar",
    "investigarais": "investigar",
    "investigaran": "investigar",
    "investigaras": "investigar",
    "investigare": "investigar",
    "investigareis": "investigar",
    "investigaremos": "investigar",
    "investigaren": "investigar",
    "investigares": "investigar",
    "investigaron": "investigar",
    "investigará": "investigar",
    "investigarán": "investigar",
    "investigarás": "investigar",
    "investigaré": "investigar",
    "investigaréis": "investigar",
    "investigaría": "investigar",
    "investigaríais": "investigar",
    "investigaríamos": "investigar",
    "investigarían": "investigar",
    "investigarías": "investigar",
    "investigas": "investigar",
    "investigase": "investigar",
    "investigaseis": "investigar",
    "investigasen": "investigar",
    "investigases": "investigar",
    "investigaste": "investigar",
    "investigasteis": "investigar",
    "investigo": "investigar",
    "investigue": "investigar",
    "investiguemos": "investigar",
    "investiguen": "investigar",
    "investigues": "investigar",
    "investigué": "investigar",
    "investiguéis": "investigar",
    "investigábamos": "investigar",
    "investigáis": "investigar",
    "investigáramos": "investigar",
    "investigáremos": "investigar",
    "investigásemos": "investigar",
    "investigó": "investigar",
    "investimos": "investir",
    "investiremos": "investir",
    "investirá": "investir",
    "investirán": "investir",
    "investirás": "investir",
    "investiré": "investir",
    "investiréis": "investir",
    "investiría": "investir",
    "investiríais": "investir",
    "investiríamos": "investir",
    "investirían": "investir",
    "investirías": "investir",
    "investiste": "investir",
    "investisteis": "investir",
    "investí": "investir",
    "investía": "investir",
    "investíais": "investir",
    "investíamos": "investir",
    "investían": "investir",
    "investías": "investir",
    "investís": "investir",
    "inveterada": "inveterado",
    "inveteradas": "inveterado",
    "inveterados": "inveterado",
    "inviabilidades": "inviabilidad",
    "inviables": "inviable",
    "invicta": "invicto",
    "invictas": "invicto",
    "invictos": "invicto",
    "invidencias": "invidencia",
    "invidentes": "invidente",
    "inviernos": "invierno",
    "invierta": "invertir",
    "inviertan": "invertir",
    "inviertas": "invertir",
    "invierte": "invertir",
    "invierten": "invertir",
    "inviertes": "invertir",
    "invierto": "invertir",
    "inviolabilidades": "inviolabilidad",
    "inviolables": "inviolable",
    "inviolada": "inviolado",
    "invioladas": "inviolado",
    "inviolados": "inviolado",
    "invirtamos": "invertir",
    "invirtiendo": "invertir",
    "invirtiera": "invertir",
    "invirtierais": "invertir",
    "invirtieran": "invertir",
    "invirtieras": "invertir",
    "invirtiere": "invertir",
    "invirtiereis": "invertir",
    "invirtieren": "invertir",
    "invirtieres": "invertir",
    "invirtieron": "invertir",
    "invirtiese": "invertir",
    "invirtieseis": "invertir",
    "invirtiesen": "invertir",
    "invirtieses": "invertir",
    "invirtiéramos": "invertir",
    "invirtiéremos": "invertir",
    "invirtiésemos": "invertir",
    "invirtió": "invertir",
    "invirtáis": "invertir",
    "invisibilidades": "invisibilidad",
    "invisibles": "invisible",
    "invista": "investir",
    "invistamos": "investir",
    "invistan": "investir",
    "invistas": "investir",
    "inviste": "investir",
    "invisten": "investir",
    "invistes": "investir",
    "invistiendo": "investir",
    "invistiera": "investir",
    "invistierais": "investir",
    "invistieran": "investir",
    "invistieras": "investir",
    "invistiere": "investir",
    "invistiereis": "investir",
    "invistieren": "investir",
    "invistieres": "investir",
    "invistieron": "investir",
    "invistiese": "investir",
    "invistieseis": "investir",
    "invistiesen": "investir",
    "invistieses": "investir",
    "invistiéramos": "investir",
    "invistiéremos": "investir",
    "invistiésemos": "investir",
    "invistió": "investir",
    "invisto": "investir",
    "invistáis": "investir",
    "invita": "invitar",
    "invitaba": "invitar",
    "invitabais": "invitar",
    "invitaban": "invitar",
    "invitabas": "invitar",
    "invitaciones": "invitación",
    "invitad": "invitar",
    "invitada": "invitar",
    "invitadas": "invitar",
    "invitado": "invitar",
    "invitadora": "invitador",
    "invitadoras": "invitador",
    "invitadores": "invitador",
    "invitados": "invitar",
    "invitamos": "invitar",
    "invitan": "invitar",
    "invitando": "invitar",
    "invitara": "invitar",
    "invitarais": "invitar",
    "invitaran": "invitar",
    "invitaras": "invitar",
    "invitare": "invitar",
    "invitareis": "invitar",
    "invitaremos": "invitar",
    "invitaren": "invitar",
    "invitares": "invitar",
    "invitaron": "invitar",
    "invitará": "invitar",
    "invitarán": "invitar",
    "invitarás": "invitar",
    "invitaré": "invitar",
    "invitaréis": "invitar",
    "invitaría": "invitar",
    "invitaríais": "invitar",
    "invitaríamos": "invitar",
    "invitarían": "invitar",
    "invitarías": "invitar",
    "invitas": "invitar",
    "invitase": "invitar",
    "invitaseis": "invitar",
    "invitasen": "invitar",
    "invitases": "invitar",
    "invitaste": "invitar",
    "invitasteis": "invitar",
    "invitatorios": "invitatorio",
    "invite": "invitar",
    "invitemos": "invitar",
    "inviten": "invitar",
    "invites": "invitar",
    "invito": "invitar",
    "invitábamos": "invitar",
    "invitáis": "invitar",
    "invitáramos": "invitar",
    "invitáremos": "invitar",
    "invitásemos": "invitar",
    "invité": "invitar",
    "invitéis": "invitar",
    "invitó": "invitar",
    "invoca": "invocar",
    "invocaba": "invocar",
    "invocabais": "invocar",
    "invocaban": "invocar",
    "invocabas": "invocar",
    "invocaciones": "invocación",
    "invocad": "invocar",
    "invocada": "invocar",
    "invocadas": "invocar",
    "invocado": "invocar",
    "invocadora": "invocador",
    "invocadoras": "invocador",
    "invocadores": "invocador",
    "invocados": "invocar",
    "invocamos": "invocar",
    "invocan": "invocar",
    "invocando": "invocar",
    "invocara": "invocar",
    "invocarais": "invocar",
    "invocaran": "invocar",
    "invocaras": "invocar",
    "invocare": "invocar",
    "invocareis": "invocar",
    "invocaremos": "invocar",
    "invocaren": "invocar",
    "invocares": "invocar",
    "invocaron": "invocar",
    "invocará": "invocar",
    "invocarán": "invocar",
    "invocarás": "invocar",
    "invocaré": "invocar",
    "invocaréis": "invocar",
    "invocaría": "invocar",
    "invocaríais": "invocar",
    "invocaríamos": "invocar",
    "invocarían": "invocar",
    "invocarías": "invocar",
    "invocas": "invocar",
    "invocase": "invocar",
    "invocaseis": "invocar",
    "invocasen": "invocar",
    "invocases": "invocar",
    "invocaste": "invocar",
    "invocasteis": "invocar",
    "invocatoria": "invocatorio",
    "invocatorias": "invocatorio",
    "invocatorios": "invocatorio",
    "invoco": "invocar",
    "invocábamos": "invocar",
    "invocáis": "invocar",
    "invocáramos": "invocar",
    "invocáremos": "invocar",
    "invocásemos": "invocar",
    "invocó": "invocar",
    "involuciones": "involución",
    "involucra": "involucrar",
    "involucraba": "involucrar",
    "involucrabais": "involucrar",
    "involucraban": "involucrar",
    "involucrabas": "involucrar",
    "involucraciones": "involucración",
    "involucrad": "involucrar",
    "involucrada": "involucrar",
    "involucradas": "involucrar",
    "involucrado": "involucrar",
    "involucrados": "involucrar",
    "involucramos": "involucrar",
    "involucran": "involucrar",
    "involucrando": "involucrar",
    "involucrara": "involucrar",
    "involucrarais": "involucrar",
    "involucraran": "involucrar",
    "involucraras": "involucrar",
    "involucrare": "involucrar",
    "involucrareis": "involucrar",
    "involucraremos": "involucrar",
    "involucraren": "involucrar",
    "involucrares": "involucrar",
    "involucraron": "involucrar",
    "involucrará": "involucrar",
    "involucrarán": "involucrar",
    "involucrarás": "involucrar",
    "involucraré": "involucrar",
    "involucraréis": "involucrar",
    "involucraría": "involucrar",
    "involucraríais": "involucrar",
    "involucraríamos": "involucrar",
    "involucrarían": "involucrar",
    "involucrarías": "involucrar",
    "involucras": "involucrar",
    "involucrase": "involucrar",
    "involucraseis": "involucrar",
    "involucrasen": "involucrar",
    "involucrases": "involucrar",
    "involucraste": "involucrar",
    "involucrasteis": "involucrar",
    "involucre": "involucrar",
    "involucremos": "involucrar",
    "involucren": "involucrar",
    "involucres": "involucrar",
    "involucro": "involucrar",
    "involucros": "involucro",
    "involucrábamos": "involucrar",
    "involucráis": "involucrar",
    "involucráramos": "involucrar",
    "involucráremos": "involucrar",
    "involucrásemos": "involucrar",
    "involucré": "involucrar",
    "involucréis": "involucrar",
    "involucró": "involucrar",
    "involuntaria": "involuntario",
    "involuntarias": "involuntario",
    "involuntariedades": "involuntariedad",
    "involuntarios": "involuntario",
    "involutas": "involuta",
    "involutiva": "involutivo",
    "involutivas": "involutivo",
    "involutivos": "involutivo",
    "invoque": "invocar",
    "invoquemos": "invocar",
    "invoquen": "invocar",
    "invoques": "invocar",
    "invoqué": "invocar",
    "invoquéis": "invocar",
    "invulnerabilidades": "invulnerabilidad",
    "invulnerables": "invulnerable",
    "inválida": "inválido",
    "inválidas": "inválido",
    "inválidos": "inválido",
    "inyecciones": "inyección",
    "inyecta": "inyectar",
    "inyectaba": "inyectar",
    "inyectabais": "inyectar",
    "inyectaban": "inyectar",
    "inyectabas": "inyectar",
    "inyectables": "inyectable",
    "inyectad": "inyectar",
    "inyectada": "inyectar",
    "inyectadas": "inyectar",
    "inyectado": "inyectar",
    "inyectados": "inyectar",
    "inyectamos": "inyectar",
    "inyectan": "inyectar",
    "inyectando": "inyectar",
    "inyectara": "inyectar",
    "inyectarais": "inyectar",
    "inyectaran": "inyectar",
    "inyectaras": "inyectar",
    "inyectare": "inyectar",
    "inyectareis": "inyectar",
    "inyectaremos": "inyectar",
    "inyectaren": "inyectar",
    "inyectares": "inyectar",
    "inyectaron": "inyectar",
    "inyectará": "inyectar",
    "inyectarán": "inyectar",
    "inyectarás": "inyectar",
    "inyectaré": "inyectar",
    "inyectaréis": "inyectar",
    "inyectaría": "inyectar",
    "inyectaríais": "inyectar",
    "inyectaríamos": "inyectar",
    "inyectarían": "inyectar",
    "inyectarías": "inyectar",
    "inyectas": "inyectar",
    "inyectase": "inyectar",
    "inyectaseis": "inyectar",
    "inyectasen": "inyectar",
    "inyectases": "inyectar",
    "inyectaste": "inyectar",
    "inyectasteis": "inyectar",
    "inyecte": "inyectar",
    "inyectemos": "inyectar",
    "inyecten": "inyectar",
    "inyectes": "inyectar",
    "inyectiva": "inyectivo",
    "inyectivas": "inyectivo",
    "inyectivos": "inyectivo",
    "inyecto": "inyectar",
    "inyectores": "inyector",
    "inyectábamos": "inyectar",
    "inyectáis": "inyectar",
    "inyectáramos": "inyectar",
    "inyectáremos": "inyectar",
    "inyectásemos": "inyectar",
    "inyecté": "inyectar",
    "inyectéis": "inyectar",
    "inyectó": "inyectar",
    "inánimes": "inánime",
    "inédita": "inédito",
    "inéditas": "inédito",
    "inéditos": "inédito",
    "inútiles": "inútil",
    "iodaciones": "iodación",
    "iodada": "iodado",
    "iodadas": "iodado",
    "iodados": "iodado",
    "iodargiritas": "iodargirita",
    "iodatos": "iodato",
    "iodofilias": "iodofilia",
    "iodoformos": "iodoformo",
    "iodometrías": "iodometría",
    "iodométrica": "iodométrico",
    "iodométricas": "iodométrico",
    "iodométricos": "iodométrico",
    "iodos": "iodo",
    "ioduros": "ioduro",
    "iolitas": "iolita",
    "iones": "ion",
    "iongramos": "iongramo",
    "ionice": "ionizar",
    "ionicemos": "ionizar",
    "ionicen": "ionizar",
    "ionices": "ionizar",
    "ionicé": "ionizar",
    "ionicéis": "ionizar",
    "ionios": "ionio",
    "ioniza": "ionizar",
    "ionizaba": "ionizar",
    "ionizabais": "ionizar",
    "ionizaban": "ionizar",
    "ionizabas": "ionizar",
    "ionizaciones": "ionización",
    "ionizad": "ionizar",
    "ionizada": "ionizar",
    "ionizadas": "ionizar",
    "ionizado": "ionizar",
    "ionizados": "ionizar",
    "ionizamos": "ionizar",
    "ionizan": "ionizar",
    "ionizando": "ionizar",
    "ionizantes": "ionizante",
    "ionizara": "ionizar",
    "ionizarais": "ionizar",
    "ionizaran": "ionizar",
    "ionizaras": "ionizar",
    "ionizare": "ionizar",
    "ionizareis": "ionizar",
    "ionizaremos": "ionizar",
    "ionizaren": "ionizar",
    "ionizares": "ionizar",
    "ionizaron": "ionizar",
    "ionizará": "ionizar",
    "ionizarán": "ionizar",
    "ionizarás": "ionizar",
    "ionizaré": "ionizar",
    "ionizaréis": "ionizar",
    "ionizaría": "ionizar",
    "ionizaríais": "ionizar",
    "ionizaríamos": "ionizar",
    "ionizarían": "ionizar",
    "ionizarías": "ionizar",
    "ionizas": "ionizar",
    "ionizase": "ionizar",
    "ionizaseis": "ionizar",
    "ionizasen": "ionizar",
    "ionizases": "ionizar",
    "ionizaste": "ionizar",
    "ionizasteis": "ionizar",
    "ionizo": "ionizar",
    "ionizábamos": "ionizar",
    "ionizáis": "ionizar",
    "ionizáramos": "ionizar",
    "ionizáremos": "ionizar",
    "ionizásemos": "ionizar",
    "ionizó": "ionizar",
    "ionogramas": "ionograma",
    "ionometrías": "ionometría",
    "iononas": "ionona",
    "ionosferas": "ionosfera",
    "ionosférica": "ionosférico",
    "ionosféricas": "ionosférico",
    "ionosféricos": "ionosférico",
    "ionosondas": "ionosonda",
    "ionóforos": "ionóforo",
    "ionógena": "ionógeno",
    "ionógenas": "ionógeno",
    "ionógenos": "ionógeno",
    "ionómeros": "ionómero",
    "ionómetros": "ionómetro",
    "iotas": "iota",
    "iotizaciones": "iotización",
    "ipecacuanas": "ipecacuana",
    "iperitas": "iperita",
    "ipiles": "ipil",
    "ipsómetros": "ipsómetro",
    "iracunda": "iracundo",
    "iracundas": "iracundo",
    "iracundias": "iracundia",
    "iracundos": "iracundo",
    "irania": "iranio",
    "iranias": "iranio",
    "iranios": "iranio",
    "iraníes": "iraní",
    "iranís": "iraní",
    "iraníses": "iraní",
    "iraquíes": "iraquí",
    "iraquís": "iraquí",
    "iraquíses": "iraquí",
    "iras": "ira",
    "irascibilidades": "irascibilidad",
    "irascibles": "irascible",
    "iremos": "ir",
    "irga": "erguir",
    "irgamos": "erguir",
    "irgan": "erguir",
    "irgas": "erguir",
    "irgo": "erguir",
    "irgue": "erguir",
    "irguen": "erguir",
    "irgues": "erguir",
    "irguiendo": "erguir",
    "irguiera": "erguir",
    "irguierais": "erguir",
    "irguieran": "erguir",
    "irguieras": "erguir",
    "irguiere": "erguir",
    "irguiereis": "erguir",
    "irguieren": "erguir",
    "irguieres": "erguir",
    "irguieron": "erguir",
    "irguiese": "erguir",
    "irguieseis": "erguir",
    "irguiesen": "erguir",
    "irguieses": "erguir",
    "irguiéramos": "erguir",
    "irguiéremos": "erguir",
    "irguiésemos": "erguir",
    "irguió": "erguir",
    "irgáis": "erguir",
    "iridectomías": "iridectomía",
    "iridemias": "iridemia",
    "irideremias": "irideremia",
    "iridescencias": "iridescencia",
    "iridios": "iridio",
    "iridiscencias": "iridiscencia",
    "iridiscentes": "iridiscente",
    "iridizaciones": "iridización",
    "iridoceles": "iridocele",
    "iridociclectomías": "iridociclectomía",
    "iridocinesias": "iridocinesia",
    "iridocolobomas": "iridocoloboma",
    "iridocorneales": "iridocorneal",
    "iridocorneosclerectomías": "iridocorneosclerectomía",
    "iridodiagnósticos": "iridodiagnóstico",
    "iridoesclerocorneales": "iridoesclerocorneal",
    "iridomalacias": "iridomalacia",
    "iridoplejías": "iridoplejía",
    "iridopupilares": "iridopupilar",
    "iridosmios": "iridosmio",
    "iridotomías": "iridotomía",
    "iridáceas": "iridácea",
    "irisa": "irisar",
    "irisaba": "irisar",
    "irisabais": "irisar",
    "irisaban": "irisar",
    "irisabas": "irisar",
    "irisaciones": "irisación",
    "irisad": "irisar",
    "irisada": "irisar",
    "irisadas": "irisar",
    "irisado": "irisar",
    "irisados": "irisar",
    "irisamos": "irisar",
    "irisan": "irisar",
    "irisando": "irisar",
    "irisara": "irisar",
    "irisarais": "irisar",
    "irisaran": "irisar",
    "irisaras": "irisar",
    "irisare": "irisar",
    "irisareis": "irisar",
    "irisaremos": "irisar",
    "irisaren": "irisar",
    "irisares": "irisar",
    "irisaron": "irisar",
    "irisará": "irisar",
    "irisarán": "irisar",
    "irisarás": "irisar",
    "irisaré": "irisar",
    "irisaréis": "irisar",
    "irisaría": "irisar",
    "irisaríais": "irisar",
    "irisaríamos": "irisar",
    "irisarían": "irisar",
    "irisarías": "irisar",
    "irisas": "irisar",
    "irisase": "irisar",
    "irisaseis": "irisar",
    "irisasen": "irisar",
    "irisases": "irisar",
    "irisaste": "irisar",
    "irisasteis": "irisar",
    "irise": "irisar",
    "irisemos": "irisar",
    "irisen": "irisar",
    "irises": "irisar",
    "iriso": "irisar",
    "irisábamos": "irisar",
    "irisáis": "irisar",
    "irisáramos": "irisar",
    "irisáremos": "irisar",
    "irisásemos": "irisar",
    "irisé": "irisar",
    "iriséis": "irisar",
    "irisó": "irisar",
    "iritoectomías": "iritoectomía",
    "irlandesa": "irlandés",
    "irlandesas": "irlandés",
    "irlandeses": "irlandés",
    "irmandiña": "irmandiño",
    "irmandiñas": "irmandiño",
    "irmandiños": "irmandiño",
    "irokos": "iroko",
    "ironice": "ironizar",
    "ironicemos": "ironizar",
    "ironicen": "ironizar",
    "ironices": "ironizar",
    "ironicé": "ironizar",
    "ironicéis": "ironizar",
    "ironistas": "ironista",
    "ironiza": "ironizar",
    "ironizaba": "ironizar",
    "ironizabais": "ironizar",
    "ironizaban": "ironizar",
    "ironizabas": "ironizar",
    "ironizad": "ironizar",
    "ironizada": "ironizar",
    "ironizadas": "ironizar",
    "ironizado": "ironizar",
    "ironizados": "ironizar",
    "ironizamos": "ironizar",
    "ironizan": "ironizar",
    "ironizando": "ironizar",
    "ironizara": "ironizar",
    "ironizarais": "ironizar",
    "ironizaran": "ironizar",
    "ironizaras": "ironizar",
    "ironizare": "ironizar",
    "ironizareis": "ironizar",
    "ironizaremos": "ironizar",
    "ironizaren": "ironizar",
    "ironizares": "ironizar",
    "ironizaron": "ironizar",
    "ironizará": "ironizar",
    "ironizarán": "ironizar",
    "ironizarás": "ironizar",
    "ironizaré": "ironizar",
    "ironizaréis": "ironizar",
    "ironizaría": "ironizar",
    "ironizaríais": "ironizar",
    "ironizaríamos": "ironizar",
    "ironizarían": "ironizar",
    "ironizarías": "ironizar",
    "ironizas": "ironizar",
    "ironizase": "ironizar",
    "ironizaseis": "ironizar",
    "ironizasen": "ironizar",
    "ironizases": "ironizar",
    "ironizaste": "ironizar",
    "ironizasteis": "ironizar",
    "ironizo": "ironizar",
    "ironizábamos": "ironizar",
    "ironizáis": "ironizar",
    "ironizáramos": "ironizar",
    "ironizáremos": "ironizar",
    "ironizásemos": "ironizar",
    "ironizó": "ironizar",
    "ironías": "ironía",
    "iroquesa": "iroqués",
    "iroquesas": "iroqués",
    "iroqueses": "iroqués",
    "irracionales": "irracional",
    "irracionalidades": "irracionalidad",
    "irracionalismos": "irracionalismo",
    "irracionalistas": "irracionalista",
    "irradia": "irradiar",
    "irradiaba": "irradiar",
    "irradiabais": "irradiar",
    "irradiaban": "irradiar",
    "irradiabas": "irradiar",
    "irradiaciones": "irradiación",
    "irradiad": "irradiar",
    "irradiada": "irradiar",
    "irradiadas": "irradiar",
    "irradiado": "irradiar",
    "irradiados": "irradiar",
    "irradiamos": "irradiar",
    "irradian": "irradiar",
    "irradiancias": "irradiancia",
    "irradiando": "irradiar",
    "irradiara": "irradiar",
    "irradiarais": "irradiar",
    "irradiaran": "irradiar",
    "irradiaras": "irradiar",
    "irradiare": "irradiar",
    "irradiareis": "irradiar",
    "irradiaremos": "irradiar",
    "irradiaren": "irradiar",
    "irradiares": "irradiar",
    "irradiaron": "irradiar",
    "irradiará": "irradiar",
    "irradiarán": "irradiar",
    "irradiarás": "irradiar",
    "irradiaré": "irradiar",
    "irradiaréis": "irradiar",
    "irradiaría": "irradiar",
    "irradiaríais": "irradiar",
    "irradiaríamos": "irradiar",
    "irradiarían": "irradiar",
    "irradiarías": "irradiar",
    "irradias": "irradiar",
    "irradiase": "irradiar",
    "irradiaseis": "irradiar",
    "irradiasen": "irradiar",
    "irradiases": "irradiar",
    "irradiaste": "irradiar",
    "irradiasteis": "irradiar",
    "irradie": "irradiar",
    "irradiemos": "irradiar",
    "irradien": "irradiar",
    "irradies": "irradiar",
    "irradio": "irradiar",
    "irradiábamos": "irradiar",
    "irradiáis": "irradiar",
    "irradiáramos": "irradiar",
    "irradiáremos": "irradiar",
    "irradiásemos": "irradiar",
    "irradié": "irradiar",
    "irradiéis": "irradiar",
    "irradió": "irradiar",
    "irrazonables": "irrazonable",
    "irreales": "irreal",
    "irrealidades": "irrealidad",
    "irrealizables": "irrealizable",
    "irrebatibilidades": "irrebatibilidad",
    "irrebatibles": "irrebatible",
    "irreconciliables": "irreconciliable",
    "irreconocibles": "irreconocible",
    "irrecuperables": "irrecuperable",
    "irrecusables": "irrecusable",
    "irredenta": "irredento",
    "irredentas": "irredento",
    "irredentismos": "irredentismo",
    "irredentistas": "irredentista",
    "irredentos": "irredento",
    "irredimibles": "irredimible",
    "irreducibles": "irreducible",
    "irreductibilidades": "irreductibilidad",
    "irreductibles": "irreductible",
    "irreemplazables": "irreemplazable",
    "irreflexiones": "irreflexión",
    "irreflexiva": "irreflexivo",
    "irreflexivas": "irreflexivo",
    "irreflexivos": "irreflexivo",
    "irreformables": "irreformable",
    "irrefrenables": "irrefrenable",
    "irrefutables": "irrefutable",
    "irreglamentaria": "irreglamentario",
    "irreglamentarias": "irreglamentario",
    "irreglamentarios": "irreglamentario",
    "irregulares": "irregular",
    "irregularidades": "irregularidad",
    "irrelevancias": "irrelevancia",
    "irrelevantes": "irrelevante",
    "irreligiones": "irreligión",
    "irreligiosa": "irreligioso",
    "irreligiosas": "irreligioso",
    "irreligiosidades": "irreligiosidad",
    "irreligiosos": "irreligioso",
    "irremediables": "irremediable",
    "irremisibles": "irremisible",
    "irremplazables": "irremplazable",
    "irrenunciables": "irrenunciable",
    "irreparabilidades": "irreparabilidad",
    "irreparables": "irreparable",
    "irrepetibles": "irrepetible",
    "irreprensibles": "irreprensible",
    "irrepresentables": "irrepresentable",
    "irreprimibles": "irreprimible",
    "irreprochabilidades": "irreprochabilidad",
    "irreprochables": "irreprochable",
    "irrescatables": "irrescatable",
    "irrescindibles": "irrescindible",
    "irresistibilidades": "irresistibilidad",
    "irresistibles": "irresistible",
    "irresolubles": "irresoluble",
    "irresoluciones": "irresolución",
    "irresoluta": "irresoluto",
    "irresolutas": "irresoluto",
    "irresolutos": "irresoluto",
    "irrespectuosa": "irrespectuoso",
    "irrespectuosas": "irrespectuoso",
    "irrespectuosos": "irrespectuoso",
    "irrespetuosa": "irrespetuoso",
    "irrespetuosas": "irrespetuoso",
    "irrespetuosos": "irrespetuoso",
    "irrespirables": "irrespirable",
    "irresponsabilidades": "irresponsabilidad",
    "irresponsables": "irresponsable",
    "irrestricta": "irrestricto",
    "irrestrictas": "irrestricto",
    "irrestrictos": "irrestricto",
    "irresuelta": "irresuelto",
    "irresueltas": "irresuelto",
    "irresueltos": "irresuelto",
    "irretroactividades": "irretroactividad",
    "irreverencias": "irreverencia",
    "irreverentes": "irreverente",
    "irreversibilidades": "irreversibilidad",
    "irreversibles": "irreversible",
    "irrevocabilidades": "irrevocabilidad",
    "irrevocables": "irrevocable",
    "irriga": "irrigar",
    "irrigaba": "irrigar",
    "irrigabais": "irrigar",
    "irrigaban": "irrigar",
    "irrigabas": "irrigar",
    "irrigables": "irrigable",
    "irrigaciones": "irrigación",
    "irrigad": "irrigar",
    "irrigada": "irrigar",
    "irrigadas": "irrigar",
    "irrigado": "irrigar",
    "irrigadora": "irrigador",
    "irrigadoras": "irrigador",
    "irrigadores": "irrigador",
    "irrigados": "irrigar",
    "irrigamos": "irrigar",
    "irrigan": "irrigar",
    "irrigando": "irrigar",
    "irrigara": "irrigar",
    "irrigarais": "irrigar",
    "irrigaran": "irrigar",
    "irrigaras": "irrigar",
    "irrigare": "irrigar",
    "irrigareis": "irrigar",
    "irrigaremos": "irrigar",
    "irrigaren": "irrigar",
    "irrigares": "irrigar",
    "irrigaron": "irrigar",
    "irrigará": "irrigar",
    "irrigarán": "irrigar",
    "irrigarás": "irrigar",
    "irrigaré": "irrigar",
    "irrigaréis": "irrigar",
    "irrigaría": "irrigar",
    "irrigaríais": "irrigar",
    "irrigaríamos": "irrigar",
    "irrigarían": "irrigar",
    "irrigarías": "irrigar",
    "irrigas": "irrigar",
    "irrigase": "irrigar",
    "irrigaseis": "irrigar",
    "irrigasen": "irrigar",
    "irrigases": "irrigar",
    "irrigaste": "irrigar",
    "irrigasteis": "irrigar",
    "irrigo": "irrigar",
    "irrigue": "irrigar",
    "irriguemos": "irrigar",
    "irriguen": "irrigar",
    "irrigues": "irrigar",
    "irrigué": "irrigar",
    "irriguéis": "irrigar",
    "irrigábamos": "irrigar",
    "irrigáis": "irrigar",
    "irrigáramos": "irrigar",
    "irrigáremos": "irrigar",
    "irrigásemos": "irrigar",
    "irrigó": "irrigar",
    "irrisibles": "irrisible",
    "irrisiones": "irrisión",
    "irrisoria": "irrisorio",
    "irrisorias": "irrisorio",
    "irrisorios": "irrisorio",
    "irrita": "irritar",
    "irritaba": "irritar",
    "irritabais": "irritar",
    "irritaban": "irritar",
    "irritabas": "irritar",
    "irritabilidades": "irritabilidad",
    "irritables": "irritable",
    "irritaciones": "irritación",
    "irritad": "irritar",
    "irritada": "irritar",
    "irritadas": "irritar",
    "irritado": "irritar",
    "irritadora": "irritador",
    "irritadoras": "irritador",
    "irritadores": "irritador",
    "irritados": "irritar",
    "irritamientos": "irritamiento",
    "irritamos": "irritar",
    "irritan": "irritar",
    "irritando": "irritar",
    "irritantes": "irritante",
    "irritara": "irritar",
    "irritarais": "irritar",
    "irritaran": "irritar",
    "irritaras": "irritar",
    "irritare": "irritar",
    "irritareis": "irritar",
    "irritaremos": "irritar",
    "irritaren": "irritar",
    "irritares": "irritar",
    "irritaron": "irritar",
    "irritará": "irritar",
    "irritarán": "irritar",
    "irritarás": "irritar",
    "irritaré": "irritar",
    "irritaréis": "irritar",
    "irritaría": "irritar",
    "irritaríais": "irritar",
    "irritaríamos": "irritar",
    "irritarían": "irritar",
    "irritarías": "irritar",
    "irritas": "irritar",
    "irritase": "irritar",
    "irritaseis": "irritar",
    "irritasen": "irritar",
    "irritases": "irritar",
    "irritaste": "irritar",
    "irritasteis": "irritar",
    "irritativa": "irritativo",
    "irritativas": "irritativo",
    "irritativos": "irritativo",
    "irrite": "irritar",
    "irritemos": "irritar",
    "irriten": "irritar",
    "irrites": "irritar",
    "irrito": "irritar",
    "irritábamos": "irritar",
    "irritáis": "irritar",
    "irritáramos": "irritar",
    "irritáremos": "irritar",
    "irritásemos": "irritar",
    "irrité": "irritar",
    "irritéis": "irritar",
    "irritó": "irritar",
    "irroga": "irrogar",
    "irrogaba": "irrogar",
    "irrogabais": "irrogar",
    "irrogaban": "irrogar",
    "irrogabas": "irrogar",
    "irrogaciones": "irrogación",
    "irrogad": "irrogar",
    "irrogada": "irrogar",
    "irrogadas": "irrogar",
    "irrogado": "irrogar",
    "irrogados": "irrogar",
    "irrogamos": "irrogar",
    "irrogan": "irrogar",
    "irrogando": "irrogar",
    "irrogara": "irrogar",
    "irrogarais": "irrogar",
    "irrogaran": "irrogar",
    "irrogaras": "irrogar",
    "irrogare": "irrogar",
    "irrogareis": "irrogar",
    "irrogaremos": "irrogar",
    "irrogaren": "irrogar",
    "irrogares": "irrogar",
    "irrogaron": "irrogar",
    "irrogará": "irrogar",
    "irrogarán": "irrogar",
    "irrogarás": "irrogar",
    "irrogaré": "irrogar",
    "irrogaréis": "irrogar",
    "irrogaría": "irrogar",
    "irrogaríais": "irrogar",
    "irrogaríamos": "irrogar",
    "irrogarían": "irrogar",
    "irrogarías": "irrogar",
    "irrogas": "irrogar",
    "irrogase": "irrogar",
    "irrogaseis": "irrogar",
    "irrogasen": "irrogar",
    "irrogases": "irrogar",
    "irrogaste": "irrogar",
    "irrogasteis": "irrogar",
    "irrogo": "irrogar",
    "irrogue": "irrogar",
    "irroguemos": "irrogar",
    "irroguen": "irrogar",
    "irrogues": "irrogar",
    "irrogué": "irrogar",
    "irroguéis": "irrogar",
    "irrogábamos": "irrogar",
    "irrogáis": "irrogar",
    "irrogáramos": "irrogar",
    "irrogáremos": "irrogar",
    "irrogásemos": "irrogar",
    "irrogó": "irrogar",
    "irrompibles": "irrompible",
    "irrotacionales": "irrotacional",
    "irrumpa": "irrumpir",
    "irrumpamos": "irrumpir",
    "irrumpan": "irrumpir",
    "irrumpas": "irrumpir",
    "irrumpe": "irrumpir",
    "irrumpen": "irrumpir",
    "irrumpes": "irrumpir",
    "irrumpid": "irrumpir",
    "irrumpida": "irrumpir",
    "irrumpidas": "irrumpir",
    "irrumpido": "irrumpir",
    "irrumpidos": "irrumpir",
    "irrumpiendo": "irrumpir",
    "irrumpiera": "irrumpir",
    "irrumpierais": "irrumpir",
    "irrumpieran": "irrumpir",
    "irrumpieras": "irrumpir",
    "irrumpiere": "irrumpir",
    "irrumpiereis": "irrumpir",
    "irrumpieren": "irrumpir",
    "irrumpieres": "irrumpir",
    "irrumpieron": "irrumpir",
    "irrumpiese": "irrumpir",
    "irrumpieseis": "irrumpir",
    "irrumpiesen": "irrumpir",
    "irrumpieses": "irrumpir",
    "irrumpimos": "irrumpir",
    "irrumpiremos": "irrumpir",
    "irrumpirá": "irrumpir",
    "irrumpirán": "irrumpir",
    "irrumpirás": "irrumpir",
    "irrumpiré": "irrumpir",
    "irrumpiréis": "irrumpir",
    "irrumpiría": "irrumpir",
    "irrumpiríais": "irrumpir",
    "irrumpiríamos": "irrumpir",
    "irrumpirían": "irrumpir",
    "irrumpirías": "irrumpir",
    "irrumpiste": "irrumpir",
    "irrumpisteis": "irrumpir",
    "irrumpiéramos": "irrumpir",
    "irrumpiéremos": "irrumpir",
    "irrumpiésemos": "irrumpir",
    "irrumpió": "irrumpir",
    "irrumpo": "irrumpir",
    "irrumpáis": "irrumpir",
    "irrumpí": "irrumpir",
    "irrumpía": "irrumpir",
    "irrumpíais": "irrumpir",
    "irrumpíamos": "irrumpir",
    "irrumpían": "irrumpir",
    "irrumpías": "irrumpir",
    "irrumpís": "irrumpir",
    "irrupciones": "irrupción",
    "irrupciónes": "irrupción",
    "irunesa": "irunés",
    "irunesas": "irunés",
    "iruneses": "irunés",
    "irá": "ir",
    "irán": "ir",
    "irás": "ir",
    "iré": "ir",
    "iréis": "ir",
    "iría": "ir",
    "iríais": "ir",
    "iríamos": "ir",
    "irían": "ir",
    "irías": "ir",
    "irídea": "irídeo",
    "irídeas": "irídeo",
    "irídeos": "irídeo",
    "irídica": "irídico",
    "irídicas": "irídico",
    "irídicos": "irídico",
    "irónica": "irónico",
    "irónicas": "irónico",
    "irónicos": "irónico",
    "is": "i",
    "isabelina": "isabelino",
    "isabelinas": "isabelino",
    "isabelinos": "isabelino",
    "isagoges": "isagoge",
    "isagógica": "isagógico",
    "isagógicas": "isagógico",
    "isagógicos": "isagógico",
    "isalaobaras": "isalaobara",
    "isalobaras": "isalobara",
    "isalohipsas": "isalohipsa",
    "isalotermas": "isaloterma",
    "isalóbaras": "isalóbara",
    "isanemonas": "isanemona",
    "isanémonas": "isanémona",
    "isanómalas": "isanómala",
    "isatinas": "isatina",
    "isbas": "isba",
    "isetionatos": "isetionato",
    "isetiónica": "isetiónico",
    "isetiónicas": "isetiónico",
    "isetiónicos": "isetiónico",
    "isiaca": "isíaco",
    "isiacas": "isíaco",
    "isiacos": "isíaco",
    "isidra": "isidro",
    "isidras": "isidro",
    "isidros": "isidro",
    "islamice": "islamizar",
    "islamicemos": "islamizar",
    "islamicen": "islamizar",
    "islamices": "islamizar",
    "islamicé": "islamizar",
    "islamicéis": "islamizar",
    "islamismos": "islamismo",
    "islamistas": "islamista",
    "islamitas": "islamita",
    "islamiza": "islamizar",
    "islamizaba": "islamizar",
    "islamizabais": "islamizar",
    "islamizaban": "islamizar",
    "islamizabas": "islamizar",
    "islamizaciones": "islamización",
    "islamizad": "islamizar",
    "islamizada": "islamizar",
    "islamizadas": "islamizar",
    "islamizado": "islamizar",
    "islamizados": "islamizar",
    "islamizamos": "islamizar",
    "islamizan": "islamizar",
    "islamizando": "islamizar",
    "islamizara": "islamizar",
    "islamizarais": "islamizar",
    "islamizaran": "islamizar",
    "islamizaras": "islamizar",
    "islamizare": "islamizar",
    "islamizareis": "islamizar",
    "islamizaremos": "islamizar",
    "islamizaren": "islamizar",
    "islamizares": "islamizar",
    "islamizaron": "islamizar",
    "islamizará": "islamizar",
    "islamizarán": "islamizar",
    "islamizarás": "islamizar",
    "islamizaré": "islamizar",
    "islamizaréis": "islamizar",
    "islamizaría": "islamizar",
    "islamizaríais": "islamizar",
    "islamizaríamos": "islamizar",
    "islamizarían": "islamizar",
    "islamizarías": "islamizar",
    "islamizas": "islamizar",
    "islamizase": "islamizar",
    "islamizaseis": "islamizar",
    "islamizasen": "islamizar",
    "islamizases": "islamizar",
    "islamizaste": "islamizar",
    "islamizasteis": "islamizar",
    "islamizo": "islamizar",
    "islamizábamos": "islamizar",
    "islamizáis": "islamizar",
    "islamizáramos": "islamizar",
    "islamizáremos": "islamizar",
    "islamizásemos": "islamizar",
    "islamizó": "islamizar",
    "islandesa": "islandés",
    "islandesas": "islandés",
    "islandeses": "islandés",
    "islanditoxinas": "islanditoxina",
    "islandésa": "islandés",
    "islandésas": "islandés",
    "islandéses": "islandés",
    "islarios": "islario",
    "islas": "isla",
    "isleos": "isleo",
    "isletas": "isleta",
    "isleña": "isleño",
    "isleñas": "isleño",
    "isleños": "isleño",
    "islotes": "islote",
    "islámica": "islámico",
    "islámicas": "islámico",
    "islámicos": "islámico",
    "islándica": "islándico",
    "islándicas": "islándico",
    "islándicos": "islándico",
    "ismaelitas": "ismaelita",
    "isoaglutininas": "isoaglutinina",
    "isoalotípica": "isoalotípico",
    "isoalotípicas": "isoalotípico",
    "isoalotípicos": "isoalotípico",
    "isoaloxacinas": "isoaloxacina",
    "isoamilaminas": "isoamilamina",
    "isoamilenos": "isoamileno",
    "isoanticuerpos": "isoanticuerpo",
    "isoantígenos": "isoantígeno",
    "isoautopoliploidias": "isoautopoliploidia",
    "isobara": "isóbaro",
    "isobaras": "isóbaro",
    "isobaros": "isóbaro",
    "isobasas": "isobasa",
    "isobatas": "isobata",
    "isobutanos": "isobutano",
    "isobutenos": "isobuteno",
    "isobutilaminas": "isobutilamina",
    "isobutilundecilenamidas": "isobutilundecilenamida",
    "isobutiratos": "isobutirato",
    "isobutironitrilos": "isobutironitrilo",
    "isobutírica": "isobutírico",
    "isobutíricas": "isobutírico",
    "isobutíricos": "isobutírico",
    "isobárica": "isobárico",
    "isobáricas": "isobárico",
    "isobáricos": "isobárico",
    "isobática": "isobático",
    "isobáticas": "isobático",
    "isobáticos": "isobático",
    "isocianúrica": "isocianúrico",
    "isocianúricas": "isocianúrico",
    "isocianúricos": "isocianúrico",
    "isociánica": "isociánico",
    "isociánicas": "isociánico",
    "isociánicos": "isociánico",
    "isoclina": "isoclino",
    "isoclinales": "isoclinal",
    "isoclinas": "isoclino",
    "isoclinos": "isoclino",
    "isoclínica": "isoclínico",
    "isoclínicas": "isoclínico",
    "isoclínicos": "isoclínico",
    "isocoras": "isocora",
    "isocostes": "isocoste",
    "isocostos": "isocosto",
    "isocrimas": "isocrima",
    "isocromosomas": "isocromosoma",
    "isocromática": "isocromático",
    "isocromáticas": "isocromático",
    "isocromáticos": "isocromático",
    "isocrona": "isocrono",
    "isocronas": "isocrono",
    "isocronismos": "isocronismo",
    "isocronos": "isocrono",
    "isocrónica": "isocrónico",
    "isocrónicas": "isocrónico",
    "isocrónicos": "isocrónico",
    "isocuanta": "isocuanto",
    "isocuantas": "isocuanto",
    "isocuantos": "isocuanto",
    "isocíclica": "isocíclico",
    "isocíclicas": "isocíclico",
    "isocíclicos": "isocíclico",
    "isocórica": "isocórico",
    "isocóricas": "isocórico",
    "isocóricos": "isocórico",
    "isodapanas": "isodapana",
    "isodecaldehídos": "isodecaldehído",
    "isodecanoica": "isodecanoico",
    "isodecanoicas": "isodecanoico",
    "isodecanoicos": "isodecanoico",
    "isodecanoles": "isodecanol",
    "isodinámica": "isodinámico",
    "isodinámicas": "isodinámico",
    "isodinámicos": "isodinámico",
    "isodonta": "isodonto",
    "isodontas": "isodonto",
    "isodontes": "isodonte",
    "isodontias": "isodontia",
    "isodontos": "isodonto",
    "isodurenos": "isodureno",
    "isodáctila": "isodáctilo",
    "isodáctilas": "isodáctilo",
    "isodáctilos": "isodáctilo",
    "isoelectroenfoques": "isoelectroenfoque",
    "isoelectrónica": "isoelectrónico",
    "isoelectrónicas": "isoelectrónico",
    "isoelectrónicos": "isoelectrónico",
    "isoeléctrica": "isoeléctrico",
    "isoeléctricas": "isoeléctrico",
    "isoeléctricos": "isoeléctrico",
    "isoentálpica": "isoentálpico",
    "isoentálpicas": "isoentálpico",
    "isoentálpicos": "isoentálpico",
    "isoenzimas": "isoenzima",
    "isoetales": "isoetal",
    "isoetáceas": "isoetácea",
    "isoeugenoles": "isoeugenol",
    "isofases": "isofase",
    "isofenas": "isofena",
    "isofonías": "isofonía",
    "isoforonas": "isoforona",
    "isofotas": "isofota",
    "isoftalatos": "isoftalato",
    "isoftálica": "isoftálico",
    "isoftálicas": "isoftálico",
    "isoftálicos": "isoftálico",
    "isofónica": "isofónico",
    "isofónicas": "isofónico",
    "isofónicos": "isofónico",
    "isogamas": "isogama",
    "isogametos": "isogameto",
    "isogamias": "isogamia",
    "isogamética": "isogamético",
    "isogaméticas": "isogamético",
    "isogaméticos": "isogamético",
    "isogeneica": "isogeneico",
    "isogeneicas": "isogeneico",
    "isogeneicos": "isogeneico",
    "isogenias": "isogenia",
    "isogeotermas": "isogeoterma",
    "isogiras": "isogira",
    "isoglosas": "isoglosa",
    "isogonales": "isogonal",
    "isogramas": "isograma",
    "isogénica": "isogénico",
    "isogénicas": "isogénico",
    "isogénicos": "isogénico",
    "isogónica": "isogónico",
    "isogónicas": "isogónico",
    "isogónicos": "isogónico",
    "isohalinas": "isohalina",
    "isohelias": "isohelia",
    "isohemaglutininas": "isohemaglutinina",
    "isohemolisinas": "isohemolisina",
    "isoheptanos": "isoheptano",
    "isohexanos": "isohexano",
    "isohietas": "isohieta",
    "isohipsas": "isohipsa",
    "isohípsas": "isohípsa",
    "isohúmica": "isohúmico",
    "isohúmicas": "isohúmico",
    "isohúmicos": "isohúmico",
    "isoinjertos": "isoinjerto",
    "isoinmunizaciones": "isoinmunización",
    "isolecita": "isolecito",
    "isolecitas": "isolecito",
    "isolecitos": "isolecito",
    "isoleucinas": "isoleucina",
    "isolíneas": "isolínea",
    "isomerasas": "isomerasa",
    "isomerizaciones": "isomerización",
    "isomerías": "isomería",
    "isometrías": "isometría",
    "isomorfa": "isomorfo",
    "isomorfas": "isomorfo",
    "isomorfismos": "isomorfismo",
    "isomorfos": "isomorfo",
    "isométrica": "isométrico",
    "isométricas": "isométrico",
    "isométricos": "isométrico",
    "isomórfica": "isomórfico",
    "isomórficas": "isomórfico",
    "isomórficos": "isomórfico",
    "isonefas": "isonefa",
    "isonicotínica": "isonicotínico",
    "isonicotínicas": "isonicotínico",
    "isonicotínicos": "isonicotínico",
    "isooctanos": "isooctano",
    "isooctenos": "isoocteno",
    "isoosmótica": "isoosmótico",
    "isoosmóticas": "isoosmótico",
    "isoosmóticos": "isoosmótico",
    "isopacas": "isopaca",
    "isopatías": "isopatía",
    "isopentaldehídos": "isopentaldehído",
    "isopentanoica": "isopentanoico",
    "isopentanoicas": "isopentanoico",
    "isopentanoicos": "isopentanoico",
    "isopentanos": "isopentano",
    "isoperimetrías": "isoperimetría",
    "isoperimétrica": "isoperimétrico",
    "isoperimétricas": "isoperimétrico",
    "isoperimétricos": "isoperimétrico",
    "isoperímetra": "isoperímetro",
    "isoperímetras": "isoperímetro",
    "isoperímetros": "isoperímetro",
    "isopletas": "isopleta",
    "isopoliploides": "isopoliploide",
    "isopreferencias": "isopreferencia",
    "isoprenos": "isopreno",
    "isopropanolaminas": "isopropanolamina",
    "isopropenilacetilenos": "isopropenilacetileno",
    "isopropilaminas": "isopropilamina",
    "isopropilaminodifenilaminas": "isopropilaminodifenilamina",
    "isopropilaminoetanoles": "isopropilaminoetanol",
    "isopropilatos": "isopropilato",
    "isopropilbencenos": "isopropilbenceno",
    "isopropilfenoles": "isopropilfenol",
    "isopulegoles": "isopulegol",
    "isoquimenas": "isoquimena",
    "isoquinoleínas": "isoquinoleína",
    "isoquímenas": "isoquímena",
    "isosafroles": "isosafrol",
    "isosilabismos": "isosilabismo",
    "isosilábica": "isosilábico",
    "isosilábicas": "isosilábico",
    "isosilábicos": "isosilábico",
    "isosistas": "isosista",
    "isosmótica": "isosmótico",
    "isosmóticas": "isosmótico",
    "isosmóticos": "isosmótico",
    "isospines": "isospín",
    "isospórica": "isospórico",
    "isospóricas": "isospórico",
    "isospóricos": "isospórico",
    "isostasias": "isostasia",
    "isosterismos": "isosterismo",
    "isosterías": "isostería",
    "isostática": "isostático",
    "isostáticas": "isostático",
    "isostáticos": "isostático",
    "isostérica": "isostérico",
    "isostéricas": "isostérico",
    "isostéricos": "isostérico",
    "isosísmica": "isosísmico",
    "isosísmicas": "isosísmico",
    "isosísmicos": "isosísmico",
    "isosónica": "isosónico",
    "isosónicas": "isosónico",
    "isosónicos": "isosónico",
    "isotacas": "isotaca",
    "isoterapias": "isoterapia",
    "isoterma": "isotermo",
    "isotermas": "isotermo",
    "isotermos": "isotermo",
    "isoterápica": "isoterápico",
    "isoterápicas": "isoterápico",
    "isoterápicos": "isoterápico",
    "isotiocianatos": "isotiocianato",
    "isotipias": "isotipia",
    "isotipismos": "isotipismo",
    "isotipos": "isotipo",
    "isotopos": "isotopo",
    "isotransplantaciones": "isotransplantación",
    "isotropías": "isotropía",
    "isotrópica": "isotrópico",
    "isotrópicas": "isotrópico",
    "isotrópicos": "isotrópico",
    "isotáctica": "isotáctico",
    "isotácticas": "isotáctico",
    "isotácticos": "isotáctico",
    "isotérmica": "isotérmico",
    "isotérmicas": "isotérmico",
    "isotérmicos": "isotérmico",
    "isotípica": "isotípico",
    "isotípicas": "isotípico",
    "isotípicos": "isotípico",
    "isotónica": "isotónico",
    "isotónicas": "isotónico",
    "isotónicos": "isotónico",
    "isotópica": "isotópico",
    "isotópicas": "isotópico",
    "isotópicos": "isotópico",
    "isovaleratos": "isovalerato",
    "isovaleriánica": "isovaleriánico",
    "isovaleriánicas": "isovaleriánico",
    "isovaleriánicos": "isovaleriánico",
    "isovelas": "isovela",
    "isoyetas": "isoyeta",
    "isozimas": "isozima",
    "isoédrica": "isoédrico",
    "isoédricas": "isoédrico",
    "isoédricos": "isoédrico",
    "isquemias": "isquemia",
    "isquiocavernosa": "isquiocavernoso",
    "isquiocavernosas": "isquiocavernoso",
    "isquiocavernosos": "isquiocavernoso",
    "isquiofemorales": "isquiofemoral",
    "isquiones": "isquion",
    "isquiorrectales": "isquiorrectal",
    "isquiotibiales": "isquiotibial",
    "isquiática": "isquiático",
    "isquiáticas": "isquiático",
    "isquiáticos": "isquiático",
    "isquémica": "isquémico",
    "isquémicas": "isquémico",
    "isquémicos": "isquémico",
    "israelitas": "israelita",
    "israelíes": "israelí",
    "israelís": "israelí",
    "israelíses": "israelí",
    "israelítica": "israelítico",
    "israelíticas": "israelítico",
    "israelíticos": "israelítico",
    "istmeña": "istmeño",
    "istmeñas": "istmeño",
    "istmeños": "istmeño",
    "istmos": "istmo",
    "isíaca": "isíaco",
    "isíacas": "isíaco",
    "isíacos": "isíaco",
    "isóbara": "isóbaro",
    "isóbaras": "isóbaro",
    "isóbaros": "isóbaro",
    "isóbatas": "isóbata",
    "isócrona": "isócrono",
    "isócronas": "isócrono",
    "isócronos": "isócrono",
    "isófena": "isófeno",
    "isófenas": "isófeno",
    "isófenos": "isófeno",
    "isófila": "isófilo",
    "isófilas": "isófilo",
    "isófilos": "isófilo",
    "isófona": "isófono",
    "isófonas": "isófono",
    "isófonos": "isófono",
    "isógona": "isógono",
    "isógonas": "isógono",
    "isógonos": "isógono",
    "isómera": "isómero",
    "isómeras": "isómero",
    "isómeros": "isómero",
    "isópodos": "isópodo",
    "isópteros": "isóptero",
    "isóstera": "isóstero",
    "isósteras": "isóstero",
    "isósteros": "isóstero",
    "isótera": "isótero",
    "isóteras": "isótero",
    "isóteros": "isótero",
    "isótona": "isótono",
    "isótonas": "isótono",
    "isótonos": "isótono",
    "isótopos": "isótopo",
    "isótropa": "isótropo",
    "isótropas": "isótropo",
    "isótropos": "isótropo",
    "itaconatos": "itaconato",
    "itacónica": "itacónico",
    "itacónicas": "itacónico",
    "itacónicos": "itacónico",
    "italiana": "italiano",
    "italianas": "italiano",
    "italianice": "italianizar",
    "italianicemos": "italianizar",
    "italianicen": "italianizar",
    "italianices": "italianizar",
    "italianicé": "italianizar",
    "italianicéis": "italianizar",
    "italianismos": "italianismo",
    "italianiza": "italianizar",
    "italianizaba": "italianizar",
    "italianizabais": "italianizar",
    "italianizaban": "italianizar",
    "italianizabas": "italianizar",
    "italianizad": "italianizar",
    "italianizada": "italianizar",
    "italianizadas": "italianizar",
    "italianizado": "italianizar",
    "italianizados": "italianizar",
    "italianizamos": "italianizar",
    "italianizan": "italianizar",
    "italianizando": "italianizar",
    "italianizantes": "italianizante",
    "italianizara": "italianizar",
    "italianizarais": "italianizar",
    "italianizaran": "italianizar",
    "italianizaras": "italianizar",
    "italianizare": "italianizar",
    "italianizareis": "italianizar",
    "italianizaremos": "italianizar",
    "italianizaren": "italianizar",
    "italianizares": "italianizar",
    "italianizaron": "italianizar",
    "italianizará": "italianizar",
    "italianizarán": "italianizar",
    "italianizarás": "italianizar",
    "italianizaré": "italianizar",
    "italianizaréis": "italianizar",
    "italianizaría": "italianizar",
    "italianizaríais": "italianizar",
    "italianizaríamos": "italianizar",
    "italianizarían": "italianizar",
    "italianizarías": "italianizar",
    "italianizas": "italianizar",
    "italianizase": "italianizar",
    "italianizaseis": "italianizar",
    "italianizasen": "italianizar",
    "italianizases": "italianizar",
    "italianizaste": "italianizar",
    "italianizasteis": "italianizar",
    "italianizo": "italianizar",
    "italianizábamos": "italianizar",
    "italianizáis": "italianizar",
    "italianizáramos": "italianizar",
    "italianizáremos": "italianizar",
    "italianizásemos": "italianizar",
    "italianizó": "italianizar",
    "italianos": "italiano",
    "italoamericana": "italoamericano",
    "italoamericanas": "italoamericano",
    "italoamericanos": "italoamericano",
    "itera": "iterar",
    "iteraba": "iterar",
    "iterabais": "iterar",
    "iteraban": "iterar",
    "iterabas": "iterar",
    "iterables": "iterable",
    "iteraciones": "iteración",
    "iterad": "iterar",
    "iterada": "iterar",
    "iteradas": "iterar",
    "iterado": "iterar",
    "iterados": "iterar",
    "iteramos": "iterar",
    "iteran": "iterar",
    "iterando": "iterar",
    "iterara": "iterar",
    "iterarais": "iterar",
    "iteraran": "iterar",
    "iteraras": "iterar",
    "iterare": "iterar",
    "iterareis": "iterar",
    "iteraremos": "iterar",
    "iteraren": "iterar",
    "iterares": "iterar",
    "iteraron": "iterar",
    "iterará": "iterar",
    "iterarán": "iterar",
    "iterarás": "iterar",
    "iteraré": "iterar",
    "iteraréis": "iterar",
    "iteraría": "iterar",
    "iteraríais": "iterar",
    "iteraríamos": "iterar",
    "iterarían": "iterar",
    "iterarías": "iterar",
    "iteras": "iterar",
    "iterase": "iterar",
    "iteraseis": "iterar",
    "iterasen": "iterar",
    "iterases": "iterar",
    "iteraste": "iterar",
    "iterasteis": "iterar",
    "iterativa": "iterativo",
    "iterativas": "iterativo",
    "iterativos": "iterativo",
    "iterbios": "iterbio",
    "itere": "iterar",
    "iteremos": "iterar",
    "iteren": "iterar",
    "iteres": "iterar",
    "itero": "iterar",
    "iterábamos": "iterar",
    "iteráis": "iterar",
    "iteráramos": "iterar",
    "iteráremos": "iterar",
    "iterásemos": "iterar",
    "iteré": "iterar",
    "iteréis": "iterar",
    "iteró": "iterar",
    "itinerantes": "itinerante",
    "itineraria": "itinerario",
    "itinerarias": "itinerario",
    "itinerarios": "itinerario",
    "itrios": "itrio",
    "itzás": "itzá",
    "itálica": "itálico",
    "itálicas": "itálico",
    "itálicos": "itálico",
    "ivas": "iva",
    "ivermectinas": "ivermectina",
    "iwanes": "iwan",
    "ixolitas": "ixolita",
    "iza": "izar",
    "izaba": "izar",
    "izabais": "izar",
    "izaban": "izar",
    "izabas": "izar",
    "izad": "izar",
    "izada": "izar",
    "izadas": "izar",
    "izado": "izar",
    "izadora": "izador",
    "izadoras": "izador",
    "izadores": "izador",
    "izados": "izar",
    "izamos": "izar",
    "izan": "izar",
    "izando": "izar",
    "izara": "izar",
    "izarais": "izar",
    "izaran": "izar",
    "izaras": "izar",
    "izare": "izar",
    "izareis": "izar",
    "izaremos": "izar",
    "izaren": "izar",
    "izares": "izar",
    "izaron": "izar",
    "izará": "izar",
    "izarán": "izar",
    "izarás": "izar",
    "izaré": "izar",
    "izaréis": "izar",
    "izaría": "izar",
    "izaríais": "izar",
    "izaríamos": "izar",
    "izarían": "izar",
    "izarías": "izar",
    "izas": "izar",
    "izase": "izar",
    "izaseis": "izar",
    "izasen": "izar",
    "izases": "izar",
    "izaste": "izar",
    "izasteis": "izar",
    "izo": "izar",
    "izquierda": "izquierdo",
    "izquierdas": "izquierdo",
    "izquierdea": "izquierdear",
    "izquierdeaba": "izquierdear",
    "izquierdeabais": "izquierdear",
    "izquierdeaban": "izquierdear",
    "izquierdeabas": "izquierdear",
    "izquierdead": "izquierdear",
    "izquierdeada": "izquierdear",
    "izquierdeadas": "izquierdear",
    "izquierdeado": "izquierdear",
    "izquierdeados": "izquierdear",
    "izquierdeamos": "izquierdear",
    "izquierdean": "izquierdear",
    "izquierdeando": "izquierdear",
    "izquierdeara": "izquierdear",
    "izquierdearais": "izquierdear",
    "izquierdearan": "izquierdear",
    "izquierdearas": "izquierdear",
    "izquierdeare": "izquierdear",
    "izquierdeareis": "izquierdear",
    "izquierdearemos": "izquierdear",
    "izquierdearen": "izquierdear",
    "izquierdeares": "izquierdear",
    "izquierdearon": "izquierdear",
    "izquierdeará": "izquierdear",
    "izquierdearán": "izquierdear",
    "izquierdearás": "izquierdear",
    "izquierdearé": "izquierdear",
    "izquierdearéis": "izquierdear",
    "izquierdearía": "izquierdear",
    "izquierdearíais": "izquierdear",
    "izquierdearíamos": "izquierdear",
    "izquierdearían": "izquierdear",
    "izquierdearías": "izquierdear",
    "izquierdeas": "izquierdear",
    "izquierdease": "izquierdear",
    "izquierdeaseis": "izquierdear",
    "izquierdeasen": "izquierdear",
    "izquierdeases": "izquierdear",
    "izquierdeaste": "izquierdear",
    "izquierdeasteis": "izquierdear",
    "izquierdee": "izquierdear",
    "izquierdeemos": "izquierdear",
    "izquierdeen": "izquierdear",
    "izquierdees": "izquierdear",
    "izquierdeo": "izquierdear",
    "izquierdeábamos": "izquierdear",
    "izquierdeáis": "izquierdear",
    "izquierdeáramos": "izquierdear",
    "izquierdeáremos": "izquierdear",
    "izquierdeásemos": "izquierdear",
    "izquierdeé": "izquierdear",
    "izquierdeéis": "izquierdear",
    "izquierdeó": "izquierdear",
    "izquierdismos": "izquierdismo",
    "izquierdistas": "izquierdista",
    "izquierdos": "izquierdo",
    "izábamos": "izar",
    "izáis": "izar",
    "izáramos": "izar",
    "izáremos": "izar",
    "izásemos": "izar",
    "izó": "izar",
    "iñiguistas": "iñiguista",
    "iódica": "iódico",
    "iódicas": "iódico",
    "iódicos": "iódico",
    "iónica": "iónico",
    "iónicas": "iónico",
    "iónicos": "iónico",
    "jabada": "jabado",
    "jabadas": "jabado",
    "jabados": "jabado",
    "jabalcona": "jabalconar",
    "jabalconaba": "jabalconar",
    "jabalconabais": "jabalconar",
    "jabalconaban": "jabalconar",
    "jabalconabas": "jabalconar",
    "jabalconad": "jabalconar",
    "jabalconada": "jabalconar",
    "jabalconadas": "jabalconar",
    "jabalconado": "jabalconar",
    "jabalconados": "jabalconar",
    "jabalconamos": "jabalconar",
    "jabalconan": "jabalconar",
    "jabalconando": "jabalconar",
    "jabalconara": "jabalconar",
    "jabalconarais": "jabalconar",
    "jabalconaran": "jabalconar",
    "jabalconaras": "jabalconar",
    "jabalconare": "jabalconar",
    "jabalconareis": "jabalconar",
    "jabalconaremos": "jabalconar",
    "jabalconaren": "jabalconar",
    "jabalconares": "jabalconar",
    "jabalconaron": "jabalconar",
    "jabalconará": "jabalconar",
    "jabalconarán": "jabalconar",
    "jabalconarás": "jabalconar",
    "jabalconaré": "jabalconar",
    "jabalconaréis": "jabalconar",
    "jabalconaría": "jabalconar",
    "jabalconaríais": "jabalconar",
    "jabalconaríamos": "jabalconar",
    "jabalconarían": "jabalconar",
    "jabalconarías": "jabalconar",
    "jabalconas": "jabalconar",
    "jabalconase": "jabalconar",
    "jabalconaseis": "jabalconar",
    "jabalconasen": "jabalconar",
    "jabalconases": "jabalconar",
    "jabalconaste": "jabalconar",
    "jabalconasteis": "jabalconar",
    "jabalcone": "jabalconar",
    "jabalconemos": "jabalconar",
    "jabalconen": "jabalconar",
    "jabalcones": "jabalconar",
    "jabalcono": "jabalconar",
    "jabalconábamos": "jabalconar",
    "jabalconáis": "jabalconar",
    "jabalconáramos": "jabalconar",
    "jabalconáremos": "jabalconar",
    "jabalconásemos": "jabalconar",
    "jabalconé": "jabalconar",
    "jabalconéis": "jabalconar",
    "jabalconó": "jabalconar",
    "jabalinas": "jabalina",
    "jabalones": "jabalón",
    "jabalíes": "jabalí",
    "jabalís": "jabalí",
    "jabalíses": "jabalí",
    "jabardas": "jabarda",
    "jabardea": "jabardear",
    "jabardeaba": "jabardear",
    "jabardeabais": "jabardear",
    "jabardeaban": "jabardear",
    "jabardeabas": "jabardear",
    "jabardead": "jabardear",
    "jabardeada": "jabardear",
    "jabardeadas": "jabardear",
    "jabardeado": "jabardear",
    "jabardeados": "jabardear",
    "jabardeamos": "jabardear",
    "jabardean": "jabardear",
    "jabardeando": "jabardear",
    "jabardeara": "jabardear",
    "jabardearais": "jabardear",
    "jabardearan": "jabardear",
    "jabardearas": "jabardear",
    "jabardeare": "jabardear",
    "jabardeareis": "jabardear",
    "jabardearemos": "jabardear",
    "jabardearen": "jabardear",
    "jabardeares": "jabardear",
    "jabardearon": "jabardear",
    "jabardeará": "jabardear",
    "jabardearán": "jabardear",
    "jabardearás": "jabardear",
    "jabardearé": "jabardear",
    "jabardearéis": "jabardear",
    "jabardearía": "jabardear",
    "jabardearíais": "jabardear",
    "jabardearíamos": "jabardear",
    "jabardearían": "jabardear",
    "jabardearías": "jabardear",
    "jabardeas": "jabardear",
    "jabardease": "jabardear",
    "jabardeaseis": "jabardear",
    "jabardeasen": "jabardear",
    "jabardeases": "jabardear",
    "jabardeaste": "jabardear",
    "jabardeasteis": "jabardear",
    "jabardee": "jabardear",
    "jabardeemos": "jabardear",
    "jabardeen": "jabardear",
    "jabardees": "jabardear",
    "jabardeo": "jabardear",
    "jabardeábamos": "jabardear",
    "jabardeáis": "jabardear",
    "jabardeáramos": "jabardear",
    "jabardeáremos": "jabardear",
    "jabardeásemos": "jabardear",
    "jabardeé": "jabardear",
    "jabardeéis": "jabardear",
    "jabardeó": "jabardear",
    "jabardillos": "jabardillo",
    "jabardos": "jabardo",
    "jabata": "jabato",
    "jabatas": "jabato",
    "jabatos": "jabato",
    "jabebas": "jabeba",
    "jabecas": "jabeca",
    "jabegas": "jabega",
    "jabegotes": "jabegote",
    "jabeguera": "jabeguero",
    "jabegueras": "jabeguero",
    "jabegueros": "jabeguero",
    "jabeques": "jabeque",
    "jaberas": "jabera",
    "jabilla": "jabillo",
    "jabillas": "jabillo",
    "jabillos": "jabillo",
    "jables": "jable",
    "jabona": "jabonar",
    "jabonaba": "jabonar",
    "jabonabais": "jabonar",
    "jabonaban": "jabonar",
    "jabonabas": "jabonar",
    "jabonad": "jabonar",
    "jabonada": "jabonar",
    "jabonadas": "jabonar",
    "jabonado": "jabonar",
    "jabonadora": "jabonador",
    "jabonadoras": "jabonador",
    "jabonadores": "jabonador",
    "jabonados": "jabonar",
    "jabonaduras": "jabonadura",
    "jabonamos": "jabonar",
    "jabonan": "jabonar",
    "jabonando": "jabonar",
    "jabonara": "jabonar",
    "jabonarais": "jabonar",
    "jabonaran": "jabonar",
    "jabonaras": "jabonar",
    "jabonare": "jabonar",
    "jabonareis": "jabonar",
    "jabonaremos": "jabonar",
    "jabonaren": "jabonar",
    "jabonares": "jabonar",
    "jabonaron": "jabonar",
    "jabonará": "jabonar",
    "jabonarán": "jabonar",
    "jabonarás": "jabonar",
    "jabonaré": "jabonar",
    "jabonaréis": "jabonar",
    "jabonaría": "jabonar",
    "jabonaríais": "jabonar",
    "jabonaríamos": "jabonar",
    "jabonarían": "jabonar",
    "jabonarías": "jabonar",
    "jabonas": "jabonar",
    "jabonase": "jabonar",
    "jabonaseis": "jabonar",
    "jabonasen": "jabonar",
    "jabonases": "jabonar",
    "jabonaste": "jabonar",
    "jabonasteis": "jabonar",
    "jaboncillos": "jaboncillo",
    "jabone": "jabonar",
    "jabonemos": "jabonar",
    "jabonen": "jabonar",
    "jabonera": "jabonero",
    "jaboneras": "jabonero",
    "jaboneros": "jabonero",
    "jabonerías": "jabonería",
    "jabones": "jabonar",
    "jabono": "jabonar",
    "jabonosa": "jabonoso",
    "jabonosas": "jabonoso",
    "jabonosos": "jabonoso",
    "jabonábamos": "jabonar",
    "jabonáis": "jabonar",
    "jabonáramos": "jabonar",
    "jabonáremos": "jabonar",
    "jabonásemos": "jabonar",
    "jaboné": "jabonar",
    "jabonéis": "jabonar",
    "jabonó": "jabonar",
    "jaborandis": "jaborandi",
    "jabíes": "jabí",
    "jabís": "jabí",
    "jacaranda": "jacarando",
    "jacarandaes": "jacarandá",
    "jacarandas": "jacarando",
    "jacarandos": "jacarando",
    "jacarandosa": "jacarandoso",
    "jacarandosas": "jacarandoso",
    "jacarandosos": "jacarandoso",
    "jacarandás": "jacarandá",
    "jacarea": "jacarear",
    "jacareaba": "jacarear",
    "jacareabais": "jacarear",
    "jacareaban": "jacarear",
    "jacareabas": "jacarear",
    "jacaread": "jacarear",
    "jacareada": "jacarear",
    "jacareadas": "jacarear",
    "jacareado": "jacarear",
    "jacareados": "jacarear",
    "jacareamos": "jacarear",
    "jacarean": "jacarear",
    "jacareando": "jacarear",
    "jacareara": "jacarear",
    "jacarearais": "jacarear",
    "jacarearan": "jacarear",
    "jacarearas": "jacarear",
    "jacareare": "jacarear",
    "jacareareis": "jacarear",
    "jacarearemos": "jacarear",
    "jacarearen": "jacarear",
    "jacareares": "jacarear",
    "jacarearon": "jacarear",
    "jacareará": "jacarear",
    "jacarearán": "jacarear",
    "jacarearás": "jacarear",
    "jacarearé": "jacarear",
    "jacarearéis": "jacarear",
    "jacarearía": "jacarear",
    "jacarearíais": "jacarear",
    "jacarearíamos": "jacarear",
    "jacarearían": "jacarear",
    "jacarearías": "jacarear",
    "jacareas": "jacarear",
    "jacarease": "jacarear",
    "jacareaseis": "jacarear",
    "jacareasen": "jacarear",
    "jacareases": "jacarear",
    "jacareaste": "jacarear",
    "jacareasteis": "jacarear",
    "jacaree": "jacarear",
    "jacareemos": "jacarear",
    "jacareen": "jacarear",
    "jacarees": "jacarear",
    "jacareo": "jacarear",
    "jacarera": "jacarero",
    "jacareras": "jacarero",
    "jacareros": "jacarero",
    "jacareábamos": "jacarear",
    "jacareáis": "jacarear",
    "jacareáramos": "jacarear",
    "jacareáremos": "jacarear",
    "jacareásemos": "jacarear",
    "jacareé": "jacarear",
    "jacareéis": "jacarear",
    "jacareó": "jacarear",
    "jacaristas": "jacarista",
    "jacas": "jaca",
    "jacerinas": "jacerina",
    "jacetana": "jacetano",
    "jacetanas": "jacetano",
    "jacetanos": "jacetano",
    "jacillas": "jacilla",
    "jacintina": "jacintino",
    "jacintinas": "jacintino",
    "jacintinos": "jacintino",
    "jacintos": "jacinto",
    "jacks": "jack",
    "jacobea": "jacobeo",
    "jacobeas": "jacobeo",
    "jacobeos": "jacobeo",
    "jacobiana": "jacobiano",
    "jacobianas": "jacobiano",
    "jacobianos": "jacobiano",
    "jacobina": "jacobino",
    "jacobinas": "jacobino",
    "jacobinismos": "jacobinismo",
    "jacobinos": "jacobino",
    "jacobitas": "jacobita",
    "jacos": "jaco",
    "jacquardes": "jacquard",
    "jacqueries": "jacquerie",
    "jacta": "jactar",
    "jactaba": "jactar",
    "jactabais": "jactar",
    "jactaban": "jactar",
    "jactabas": "jactar",
    "jactad": "jactar",
    "jactada": "jactar",
    "jactadas": "jactar",
    "jactado": "jactar",
    "jactados": "jactar",
    "jactamos": "jactar",
    "jactan": "jactar",
    "jactancias": "jactancia",
    "jactanciosa": "jactancioso",
    "jactanciosas": "jactancioso",
    "jactanciosos": "jactancioso",
    "jactando": "jactar",
    "jactara": "jactar",
    "jactarais": "jactar",
    "jactaran": "jactar",
    "jactaras": "jactar",
    "jactare": "jactar",
    "jactareis": "jactar",
    "jactaremos": "jactar",
    "jactaren": "jactar",
    "jactares": "jactar",
    "jactaron": "jactar",
    "jactará": "jactar",
    "jactarán": "jactar",
    "jactarás": "jactar",
    "jactaré": "jactar",
    "jactaréis": "jactar",
    "jactaría": "jactar",
    "jactaríais": "jactar",
    "jactaríamos": "jactar",
    "jactarían": "jactar",
    "jactarías": "jactar",
    "jactas": "jactar",
    "jactase": "jactar",
    "jactaseis": "jactar",
    "jactasen": "jactar",
    "jactases": "jactar",
    "jactaste": "jactar",
    "jactasteis": "jactar",
    "jacte": "jactar",
    "jactemos": "jactar",
    "jacten": "jactar",
    "jactes": "jactar",
    "jacto": "jactar",
    "jactábamos": "jactar",
    "jactáis": "jactar",
    "jactáramos": "jactar",
    "jactáremos": "jactar",
    "jactásemos": "jactar",
    "jacté": "jactar",
    "jactéis": "jactar",
    "jactó": "jactar",
    "jaculatoria": "jaculatorio",
    "jaculatorias": "jaculatorio",
    "jaculatorios": "jaculatorio",
    "jadea": "jadear",
    "jadeaba": "jadear",
    "jadeabais": "jadear",
    "jadeaban": "jadear",
    "jadeabas": "jadear",
    "jadead": "jadear",
    "jadeada": "jadear",
    "jadeadas": "jadear",
    "jadeado": "jadear",
    "jadeados": "jadear",
    "jadeamos": "jadear",
    "jadean": "jadear",
    "jadeando": "jadear",
    "jadeantes": "jadeante",
    "jadeara": "jadear",
    "jadearais": "jadear",
    "jadearan": "jadear",
    "jadearas": "jadear",
    "jadeare": "jadear",
    "jadeareis": "jadear",
    "jadearemos": "jadear",
    "jadearen": "jadear",
    "jadeares": "jadear",
    "jadearon": "jadear",
    "jadeará": "jadear",
    "jadearán": "jadear",
    "jadearás": "jadear",
    "jadearé": "jadear",
    "jadearéis": "jadear",
    "jadearía": "jadear",
    "jadearíais": "jadear",
    "jadearíamos": "jadear",
    "jadearían": "jadear",
    "jadearías": "jadear",
    "jadeas": "jadear",
    "jadease": "jadear",
    "jadeaseis": "jadear",
    "jadeasen": "jadear",
    "jadeases": "jadear",
    "jadeaste": "jadear",
    "jadeasteis": "jadear",
    "jadee": "jadear",
    "jadeemos": "jadear",
    "jadeen": "jadear",
    "jadees": "jadear",
    "jadeitas": "jadeita",
    "jadeo": "jadear",
    "jadeos": "jadeo",
    "jades": "jade",
    "jadeábamos": "jadear",
    "jadeáis": "jadear",
    "jadeáramos": "jadear",
    "jadeáremos": "jadear",
    "jadeásemos": "jadear",
    "jadeé": "jadear",
    "jadeéis": "jadear",
    "jadeítas": "jadeíta",
    "jadeó": "jadear",
    "jadraques": "jadraque",
    "jaece": "jaezar",
    "jaecemos": "jaezar",
    "jaecen": "jaezar",
    "jaecera": "jaecero",
    "jaeceras": "jaecero",
    "jaeceros": "jaecero",
    "jaeces": "jaezar",
    "jaecé": "jaezar",
    "jaecéis": "jaezar",
    "jaenera": "jaenero",
    "jaeneras": "jaenero",
    "jaeneros": "jaenero",
    "jaenesa": "jaenés",
    "jaenesas": "jaenés",
    "jaeneses": "jaenés",
    "jaeza": "jaezar",
    "jaezaba": "jaezar",
    "jaezabais": "jaezar",
    "jaezaban": "jaezar",
    "jaezabas": "jaezar",
    "jaezad": "jaezar",
    "jaezada": "jaezar",
    "jaezadas": "jaezar",
    "jaezado": "jaezar",
    "jaezados": "jaezar",
    "jaezamos": "jaezar",
    "jaezan": "jaezar",
    "jaezando": "jaezar",
    "jaezara": "jaezar",
    "jaezarais": "jaezar",
    "jaezaran": "jaezar",
    "jaezaras": "jaezar",
    "jaezare": "jaezar",
    "jaezareis": "jaezar",
    "jaezaremos": "jaezar",
    "jaezaren": "jaezar",
    "jaezares": "jaezar",
    "jaezaron": "jaezar",
    "jaezará": "jaezar",
    "jaezarán": "jaezar",
    "jaezarás": "jaezar",
    "jaezaré": "jaezar",
    "jaezaréis": "jaezar",
    "jaezaría": "jaezar",
    "jaezaríais": "jaezar",
    "jaezaríamos": "jaezar",
    "jaezarían": "jaezar",
    "jaezarías": "jaezar",
    "jaezas": "jaezar",
    "jaezase": "jaezar",
    "jaezaseis": "jaezar",
    "jaezasen": "jaezar",
    "jaezases": "jaezar",
    "jaezaste": "jaezar",
    "jaezasteis": "jaezar",
    "jaezo": "jaezar",
    "jaezábamos": "jaezar",
    "jaezáis": "jaezar",
    "jaezáramos": "jaezar",
    "jaezáremos": "jaezar",
    "jaezásemos": "jaezar",
    "jaezó": "jaezar",
    "jafética": "jafético",
    "jaféticas": "jafético",
    "jaféticos": "jafético",
    "jaguares": "jaguar",
    "jaguas": "jagua",
    "jagüeyes": "jagüey",
    "jagüeys": "jagüey",
    "jaharra": "jaharrar",
    "jaharraba": "jaharrar",
    "jaharrabais": "jaharrar",
    "jaharraban": "jaharrar",
    "jaharrabas": "jaharrar",
    "jaharrad": "jaharrar",
    "jaharrada": "jaharrar",
    "jaharradas": "jaharrar",
    "jaharrado": "jaharrar",
    "jaharrados": "jaharrar",
    "jaharramos": "jaharrar",
    "jaharran": "jaharrar",
    "jaharrando": "jaharrar",
    "jaharrara": "jaharrar",
    "jaharrarais": "jaharrar",
    "jaharraran": "jaharrar",
    "jaharraras": "jaharrar",
    "jaharrare": "jaharrar",
    "jaharrareis": "jaharrar",
    "jaharraremos": "jaharrar",
    "jaharraren": "jaharrar",
    "jaharrares": "jaharrar",
    "jaharraron": "jaharrar",
    "jaharrará": "jaharrar",
    "jaharrarán": "jaharrar",
    "jaharrarás": "jaharrar",
    "jaharraré": "jaharrar",
    "jaharraréis": "jaharrar",
    "jaharraría": "jaharrar",
    "jaharraríais": "jaharrar",
    "jaharraríamos": "jaharrar",
    "jaharrarían": "jaharrar",
    "jaharrarías": "jaharrar",
    "jaharras": "jaharrar",
    "jaharrase": "jaharrar",
    "jaharraseis": "jaharrar",
    "jaharrasen": "jaharrar",
    "jaharrases": "jaharrar",
    "jaharraste": "jaharrar",
    "jaharrasteis": "jaharrar",
    "jaharre": "jaharrar",
    "jaharremos": "jaharrar",
    "jaharren": "jaharrar",
    "jaharres": "jaharrar",
    "jaharro": "jaharrar",
    "jaharrábamos": "jaharrar",
    "jaharráis": "jaharrar",
    "jaharráramos": "jaharrar",
    "jaharráremos": "jaharrar",
    "jaharrásemos": "jaharrar",
    "jaharré": "jaharrar",
    "jaharréis": "jaharrar",
    "jaharró": "jaharrar",
    "jainismos": "jainismo",
    "jala": "jalar",
    "jalaba": "jalar",
    "jalabais": "jalar",
    "jalaban": "jalar",
    "jalabas": "jalar",
    "jalad": "jalar",
    "jalada": "jalar",
    "jaladas": "jalar",
    "jalado": "jalar",
    "jalados": "jalar",
    "jalamos": "jalar",
    "jalan": "jalar",
    "jalando": "jalar",
    "jalapas": "jalapa",
    "jalara": "jalar",
    "jalarais": "jalar",
    "jalaran": "jalar",
    "jalaras": "jalar",
    "jalare": "jalar",
    "jalareis": "jalar",
    "jalaremos": "jalar",
    "jalaren": "jalar",
    "jalares": "jalar",
    "jalaron": "jalar",
    "jalará": "jalar",
    "jalarán": "jalar",
    "jalarás": "jalar",
    "jalaré": "jalar",
    "jalaréis": "jalar",
    "jalaría": "jalar",
    "jalaríais": "jalar",
    "jalaríamos": "jalar",
    "jalarían": "jalar",
    "jalarías": "jalar",
    "jalas": "jalar",
    "jalase": "jalar",
    "jalaseis": "jalar",
    "jalasen": "jalar",
    "jalases": "jalar",
    "jalaste": "jalar",
    "jalasteis": "jalar",
    "jalbega": "jalbegar",
    "jalbegaba": "jalbegar",
    "jalbegabais": "jalbegar",
    "jalbegaban": "jalbegar",
    "jalbegabas": "jalbegar",
    "jalbegad": "jalbegar",
    "jalbegada": "jalbegar",
    "jalbegadas": "jalbegar",
    "jalbegado": "jalbegar",
    "jalbegados": "jalbegar",
    "jalbegamos": "jalbegar",
    "jalbegan": "jalbegar",
    "jalbegando": "jalbegar",
    "jalbegara": "jalbegar",
    "jalbegarais": "jalbegar",
    "jalbegaran": "jalbegar",
    "jalbegaras": "jalbegar",
    "jalbegare": "jalbegar",
    "jalbegareis": "jalbegar",
    "jalbegaremos": "jalbegar",
    "jalbegaren": "jalbegar",
    "jalbegares": "jalbegar",
    "jalbegaron": "jalbegar",
    "jalbegará": "jalbegar",
    "jalbegarán": "jalbegar",
    "jalbegarás": "jalbegar",
    "jalbegaré": "jalbegar",
    "jalbegaréis": "jalbegar",
    "jalbegaría": "jalbegar",
    "jalbegaríais": "jalbegar",
    "jalbegaríamos": "jalbegar",
    "jalbegarían": "jalbegar",
    "jalbegarías": "jalbegar",
    "jalbegas": "jalbegar",
    "jalbegase": "jalbegar",
    "jalbegaseis": "jalbegar",
    "jalbegasen": "jalbegar",
    "jalbegases": "jalbegar",
    "jalbegaste": "jalbegar",
    "jalbegasteis": "jalbegar",
    "jalbego": "jalbegar",
    "jalbegue": "jalbegar",
    "jalbeguemos": "jalbegar",
    "jalbeguen": "jalbegar",
    "jalbegues": "jalbegue",
    "jalbegué": "jalbegar",
    "jalbeguéis": "jalbegar",
    "jalbegábamos": "jalbegar",
    "jalbegáis": "jalbegar",
    "jalbegáramos": "jalbegar",
    "jalbegáremos": "jalbegar",
    "jalbegásemos": "jalbegar",
    "jalbegó": "jalbegar",
    "jaldas": "jalda",
    "jale": "jalar",
    "jalea": "jalear",
    "jaleaba": "jalear",
    "jaleabais": "jalear",
    "jaleaban": "jalear",
    "jaleabas": "jalear",
    "jalead": "jalear",
    "jaleada": "jalear",
    "jaleadas": "jalear",
    "jaleado": "jalear",
    "jaleadora": "jaleador",
    "jaleadoras": "jaleador",
    "jaleadores": "jaleador",
    "jaleados": "jalear",
    "jaleamos": "jalear",
    "jalean": "jalear",
    "jaleando": "jalear",
    "jaleara": "jalear",
    "jalearais": "jalear",
    "jalearan": "jalear",
    "jalearas": "jalear",
    "jaleare": "jalear",
    "jaleareis": "jalear",
    "jalearemos": "jalear",
    "jalearen": "jalear",
    "jaleares": "jalear",
    "jalearon": "jalear",
    "jaleará": "jalear",
    "jalearán": "jalear",
    "jalearás": "jalear",
    "jalearé": "jalear",
    "jalearéis": "jalear",
    "jalearía": "jalear",
    "jalearíais": "jalear",
    "jalearíamos": "jalear",
    "jalearían": "jalear",
    "jalearías": "jalear",
    "jaleas": "jalear",
    "jalease": "jalear",
    "jaleaseis": "jalear",
    "jaleasen": "jalear",
    "jaleases": "jalear",
    "jaleaste": "jalear",
    "jaleasteis": "jalear",
    "jalecos": "jaleco",
    "jalee": "jalear",
    "jaleemos": "jalear",
    "jaleen": "jalear",
    "jalees": "jalear",
    "jalemos": "jalar",
    "jalen": "jalar",
    "jaleo": "jalear",
    "jaleos": "jaleo",
    "jales": "jalar",
    "jaleábamos": "jalear",
    "jaleáis": "jalear",
    "jaleáramos": "jalear",
    "jaleáremos": "jalear",
    "jaleásemos": "jalear",
    "jaleé": "jalear",
    "jaleéis": "jalear",
    "jaleó": "jalear",
    "jalifas": "jalifa",
    "jalifatos": "jalifato",
    "jalifiana": "jalifiano",
    "jalifianas": "jalifiano",
    "jalifianos": "jalifiano",
    "jalo": "jalar",
    "jalona": "jalonar",
    "jalonaba": "jalonar",
    "jalonabais": "jalonar",
    "jalonaban": "jalonar",
    "jalonabas": "jalonar",
    "jalonad": "jalonar",
    "jalonada": "jalonar",
    "jalonadas": "jalonar",
    "jalonado": "jalonar",
    "jalonados": "jalonar",
    "jalonamientos": "jalonamiento",
    "jalonamos": "jalonar",
    "jalonan": "jalonar",
    "jalonando": "jalonar",
    "jalonara": "jalonar",
    "jalonarais": "jalonar",
    "jalonaran": "jalonar",
    "jalonaras": "jalonar",
    "jalonare": "jalonar",
    "jalonareis": "jalonar",
    "jalonaremos": "jalonar",
    "jalonaren": "jalonar",
    "jalonares": "jalonar",
    "jalonaron": "jalonar",
    "jalonará": "jalonar",
    "jalonarán": "jalonar",
    "jalonarás": "jalonar",
    "jalonaré": "jalonar",
    "jalonaréis": "jalonar",
    "jalonaría": "jalonar",
    "jalonaríais": "jalonar",
    "jalonaríamos": "jalonar",
    "jalonarían": "jalonar",
    "jalonarías": "jalonar",
    "jalonas": "jalonar",
    "jalonase": "jalonar",
    "jalonaseis": "jalonar",
    "jalonasen": "jalonar",
    "jalonases": "jalonar",
    "jalonaste": "jalonar",
    "jalonasteis": "jalonar",
    "jalone": "jalonar",
    "jalonemos": "jalonar",
    "jalonen": "jalonar",
    "jalones": "jalonar",
    "jalono": "jalonar",
    "jalonábamos": "jalonar",
    "jalonáis": "jalonar",
    "jalonáramos": "jalonar",
    "jalonáremos": "jalonar",
    "jalonásemos": "jalonar",
    "jaloné": "jalonar",
    "jalonéis": "jalonar",
    "jalonó": "jalonar",
    "jaloques": "jaloque",
    "jalábamos": "jalar",
    "jaláis": "jalar",
    "jaláramos": "jalar",
    "jaláremos": "jalar",
    "jalásemos": "jalar",
    "jalé": "jalar",
    "jaléis": "jalar",
    "jaló": "jalar",
    "jama": "jamar",
    "jamaba": "jamar",
    "jamabais": "jamar",
    "jamaban": "jamar",
    "jamabas": "jamar",
    "jamad": "jamar",
    "jamada": "jamar",
    "jamadas": "jamar",
    "jamado": "jamar",
    "jamados": "jamar",
    "jamaicana": "jamaicano",
    "jamaicanas": "jamaicano",
    "jamaicanos": "jamaicano",
    "jamamos": "jamar",
    "jaman": "jamar",
    "jamando": "jamar",
    "jamara": "jamar",
    "jamarais": "jamar",
    "jamaran": "jamar",
    "jamaras": "jamar",
    "jamare": "jamar",
    "jamareis": "jamar",
    "jamaremos": "jamar",
    "jamaren": "jamar",
    "jamares": "jamar",
    "jamaron": "jamar",
    "jamará": "jamar",
    "jamarán": "jamar",
    "jamarás": "jamar",
    "jamaré": "jamar",
    "jamaréis": "jamar",
    "jamaría": "jamar",
    "jamaríais": "jamar",
    "jamaríamos": "jamar",
    "jamarían": "jamar",
    "jamarías": "jamar",
    "jamas": "jamar",
    "jamase": "jamar",
    "jamaseis": "jamar",
    "jamasen": "jamar",
    "jamases": "jamar",
    "jamaste": "jamar",
    "jamasteis": "jamar",
    "jambajes": "jambaje",
    "jambas": "jamba",
    "jame": "jamar",
    "jamelgos": "jamelgo",
    "jamemos": "jamar",
    "jamen": "jamar",
    "jamerdanas": "jamerdana",
    "james": "jamar",
    "jamesonitas": "jamesonita",
    "jametes": "jamete",
    "jamo": "jamar",
    "jamonas": "jamona",
    "jamones": "jamón",
    "jamábamos": "jamar",
    "jamáis": "jamar",
    "jamáramos": "jamar",
    "jamáremos": "jamar",
    "jamásemos": "jamar",
    "jamé": "jamar",
    "jaméis": "jamar",
    "jamó": "jamar",
    "jamónes": "jamón",
    "janatos": "janato",
    "janes": "jan",
    "janguas": "jangua",
    "jans": "jan",
    "jansenismos": "jansenismo",
    "jansenistas": "jansenista",
    "japones": "japón",
    "japonesa": "japonés",
    "japonesas": "japonés",
    "japoneses": "japonés",
    "jaquea": "jaquear",
    "jaqueaba": "jaquear",
    "jaqueabais": "jaquear",
    "jaqueaban": "jaquear",
    "jaqueabas": "jaquear",
    "jaquead": "jaquear",
    "jaqueada": "jaquear",
    "jaqueadas": "jaquear",
    "jaqueado": "jaquear",
    "jaqueados": "jaquear",
    "jaqueamos": "jaquear",
    "jaquean": "jaquear",
    "jaqueando": "jaquear",
    "jaqueara": "jaquear",
    "jaquearais": "jaquear",
    "jaquearan": "jaquear",
    "jaquearas": "jaquear",
    "jaqueare": "jaquear",
    "jaqueareis": "jaquear",
    "jaquearemos": "jaquear",
    "jaquearen": "jaquear",
    "jaqueares": "jaquear",
    "jaquearon": "jaquear",
    "jaqueará": "jaquear",
    "jaquearán": "jaquear",
    "jaquearás": "jaquear",
    "jaquearé": "jaquear",
    "jaquearéis": "jaquear",
    "jaquearía": "jaquear",
    "jaquearíais": "jaquear",
    "jaquearíamos": "jaquear",
    "jaquearían": "jaquear",
    "jaquearías": "jaquear",
    "jaqueas": "jaquear",
    "jaquease": "jaquear",
    "jaqueaseis": "jaquear",
    "jaqueasen": "jaquear",
    "jaqueases": "jaquear",
    "jaqueaste": "jaquear",
    "jaqueasteis": "jaquear",
    "jaquecas": "jaqueca",
    "jaquee": "jaquear",
    "jaqueemos": "jaquear",
    "jaqueen": "jaquear",
    "jaquees": "jaquear",
    "jaquelada": "jaquelado",
    "jaqueladas": "jaquelado",
    "jaquelados": "jaquelado",
    "jaqueo": "jaquear",
    "jaques": "jaque",
    "jaqueses": "jaqués",
    "jaquetones": "jaquetón",
    "jaqueábamos": "jaquear",
    "jaqueáis": "jaquear",
    "jaqueáramos": "jaquear",
    "jaqueáremos": "jaquear",
    "jaqueásemos": "jaquear",
    "jaqueé": "jaquear",
    "jaqueéis": "jaquear",
    "jaqueó": "jaquear",
    "jaquimazos": "jaquimazo",
    "jaquimera": "jaquimero",
    "jaquimeras": "jaquimero",
    "jaquimeros": "jaquimero",
    "jara": "jaro",
    "jarabea": "jarabear",
    "jarabeaba": "jarabear",
    "jarabeabais": "jarabear",
    "jarabeaban": "jarabear",
    "jarabeabas": "jarabear",
    "jarabead": "jarabear",
    "jarabeada": "jarabear",
    "jarabeadas": "jarabear",
    "jarabeado": "jarabear",
    "jarabeados": "jarabear",
    "jarabeamos": "jarabear",
    "jarabean": "jarabear",
    "jarabeando": "jarabear",
    "jarabeara": "jarabear",
    "jarabearais": "jarabear",
    "jarabearan": "jarabear",
    "jarabearas": "jarabear",
    "jarabeare": "jarabear",
    "jarabeareis": "jarabear",
    "jarabearemos": "jarabear",
    "jarabearen": "jarabear",
    "jarabeares": "jarabear",
    "jarabearon": "jarabear",
    "jarabeará": "jarabear",
    "jarabearán": "jarabear",
    "jarabearás": "jarabear",
    "jarabearé": "jarabear",
    "jarabearéis": "jarabear",
    "jarabearía": "jarabear",
    "jarabearíais": "jarabear",
    "jarabearíamos": "jarabear",
    "jarabearían": "jarabear",
    "jarabearías": "jarabear",
    "jarabeas": "jarabear",
    "jarabease": "jarabear",
    "jarabeaseis": "jarabear",
    "jarabeasen": "jarabear",
    "jarabeases": "jarabear",
    "jarabeaste": "jarabear",
    "jarabeasteis": "jarabear",
    "jarabee": "jarabear",
    "jarabeemos": "jarabear",
    "jarabeen": "jarabear",
    "jarabees": "jarabear",
    "jarabeo": "jarabear",
    "jarabes": "jarabe",
    "jarabeábamos": "jarabear",
    "jarabeáis": "jarabear",
    "jarabeáramos": "jarabear",
    "jarabeáremos": "jarabear",
    "jarabeásemos": "jarabear",
    "jarabeé": "jarabear",
    "jarabeéis": "jarabear",
    "jarabeó": "jarabear",
    "jarales": "jaral",
    "jaramagales": "jaramagal",
    "jaramagos": "jaramago",
    "jaramugos": "jaramugo",
    "jaranas": "jarana",
    "jaranea": "jaranear",
    "jaraneaba": "jaranear",
    "jaraneabais": "jaranear",
    "jaraneaban": "jaranear",
    "jaraneabas": "jaranear",
    "jaranead": "jaranear",
    "jaraneada": "jaranear",
    "jaraneadas": "jaranear",
    "jaraneado": "jaranear",
    "jaraneados": "jaranear",
    "jaraneamos": "jaranear",
    "jaranean": "jaranear",
    "jaraneando": "jaranear",
    "jaraneara": "jaranear",
    "jaranearais": "jaranear",
    "jaranearan": "jaranear",
    "jaranearas": "jaranear",
    "jaraneare": "jaranear",
    "jaraneareis": "jaranear",
    "jaranearemos": "jaranear",
    "jaranearen": "jaranear",
    "jaraneares": "jaranear",
    "jaranearon": "jaranear",
    "jaraneará": "jaranear",
    "jaranearán": "jaranear",
    "jaranearás": "jaranear",
    "jaranearé": "jaranear",
    "jaranearéis": "jaranear",
    "jaranearía": "jaranear",
    "jaranearíais": "jaranear",
    "jaranearíamos": "jaranear",
    "jaranearían": "jaranear",
    "jaranearías": "jaranear",
    "jaraneas": "jaranear",
    "jaranease": "jaranear",
    "jaraneaseis": "jaranear",
    "jaraneasen": "jaranear",
    "jaraneases": "jaranear",
    "jaraneaste": "jaranear",
    "jaraneasteis": "jaranear",
    "jaranee": "jaranear",
    "jaraneemos": "jaranear",
    "jaraneen": "jaranear",
    "jaranees": "jaranear",
    "jaraneo": "jaranear",
    "jaranera": "jaranero",
    "jaraneras": "jaranero",
    "jaraneros": "jaranero",
    "jaraneábamos": "jaranear",
    "jaraneáis": "jaranear",
    "jaraneáramos": "jaranear",
    "jaraneáremos": "jaranear",
    "jaraneásemos": "jaranear",
    "jaraneé": "jaranear",
    "jaraneéis": "jaranear",
    "jaraneó": "jaranear",
    "jararacas": "jararaca",
    "jaras": "jaro",
    "jaraíces": "jaraíz",
    "jarchas": "jarcha",
    "jarcia": "jarciar",
    "jarciaba": "jarciar",
    "jarciabais": "jarciar",
    "jarciaban": "jarciar",
    "jarciabas": "jarciar",
    "jarciad": "jarciar",
    "jarciada": "jarciar",
    "jarciadas": "jarciar",
    "jarciado": "jarciar",
    "jarciados": "jarciar",
    "jarciamos": "jarciar",
    "jarcian": "jarciar",
    "jarciando": "jarciar",
    "jarciara": "jarciar",
    "jarciarais": "jarciar",
    "jarciaran": "jarciar",
    "jarciaras": "jarciar",
    "jarciare": "jarciar",
    "jarciareis": "jarciar",
    "jarciaremos": "jarciar",
    "jarciaren": "jarciar",
    "jarciares": "jarciar",
    "jarciaron": "jarciar",
    "jarciará": "jarciar",
    "jarciarán": "jarciar",
    "jarciarás": "jarciar",
    "jarciaré": "jarciar",
    "jarciaréis": "jarciar",
    "jarciaría": "jarciar",
    "jarciaríais": "jarciar",
    "jarciaríamos": "jarciar",
    "jarciarían": "jarciar",
    "jarciarías": "jarciar",
    "jarcias": "jarciar",
    "jarciase": "jarciar",
    "jarciaseis": "jarciar",
    "jarciasen": "jarciar",
    "jarciases": "jarciar",
    "jarciaste": "jarciar",
    "jarciasteis": "jarciar",
    "jarcie": "jarciar",
    "jarciemos": "jarciar",
    "jarcien": "jarciar",
    "jarcies": "jarciar",
    "jarcio": "jarciar",
    "jarciábamos": "jarciar",
    "jarciáis": "jarciar",
    "jarciáramos": "jarciar",
    "jarciáremos": "jarciar",
    "jarciásemos": "jarciar",
    "jarcié": "jarciar",
    "jarciéis": "jarciar",
    "jarció": "jarciar",
    "jardinera": "jardinero",
    "jardineras": "jardinero",
    "jardineros": "jardinero",
    "jardinerías": "jardinería",
    "jardines": "jardín",
    "jaretas": "jareta",
    "jaretones": "jaretón",
    "jargonafasias": "jargonafasia",
    "jargones": "jargón",
    "jaropa": "jaropar",
    "jaropaba": "jaropar",
    "jaropabais": "jaropar",
    "jaropaban": "jaropar",
    "jaropabas": "jaropar",
    "jaropad": "jaropar",
    "jaropada": "jaropar",
    "jaropadas": "jaropar",
    "jaropado": "jaropar",
    "jaropados": "jaropar",
    "jaropamos": "jaropar",
    "jaropan": "jaropar",
    "jaropando": "jaropar",
    "jaropara": "jaropar",
    "jaroparais": "jaropar",
    "jaroparan": "jaropar",
    "jaroparas": "jaropar",
    "jaropare": "jaropar",
    "jaropareis": "jaropar",
    "jaroparemos": "jaropar",
    "jaroparen": "jaropar",
    "jaropares": "jaropar",
    "jaroparon": "jaropar",
    "jaropará": "jaropar",
    "jaroparán": "jaropar",
    "jaroparás": "jaropar",
    "jaroparé": "jaropar",
    "jaroparéis": "jaropar",
    "jaroparía": "jaropar",
    "jaroparíais": "jaropar",
    "jaroparíamos": "jaropar",
    "jaroparían": "jaropar",
    "jaroparías": "jaropar",
    "jaropas": "jaropar",
    "jaropase": "jaropar",
    "jaropaseis": "jaropar",
    "jaropasen": "jaropar",
    "jaropases": "jaropar",
    "jaropaste": "jaropar",
    "jaropasteis": "jaropar",
    "jarope": "jaropar",
    "jaropea": "jaropear",
    "jaropeaba": "jaropear",
    "jaropeabais": "jaropear",
    "jaropeaban": "jaropear",
    "jaropeabas": "jaropear",
    "jaropead": "jaropear",
    "jaropeada": "jaropear",
    "jaropeadas": "jaropear",
    "jaropeado": "jaropear",
    "jaropeados": "jaropear",
    "jaropeamos": "jaropear",
    "jaropean": "jaropear",
    "jaropeando": "jaropear",
    "jaropeara": "jaropear",
    "jaropearais": "jaropear",
    "jaropearan": "jaropear",
    "jaropearas": "jaropear",
    "jaropeare": "jaropear",
    "jaropeareis": "jaropear",
    "jaropearemos": "jaropear",
    "jaropearen": "jaropear",
    "jaropeares": "jaropear",
    "jaropearon": "jaropear",
    "jaropeará": "jaropear",
    "jaropearán": "jaropear",
    "jaropearás": "jaropear",
    "jaropearé": "jaropear",
    "jaropearéis": "jaropear",
    "jaropearía": "jaropear",
    "jaropearíais": "jaropear",
    "jaropearíamos": "jaropear",
    "jaropearían": "jaropear",
    "jaropearías": "jaropear",
    "jaropeas": "jaropear",
    "jaropease": "jaropear",
    "jaropeaseis": "jaropear",
    "jaropeasen": "jaropear",
    "jaropeases": "jaropear",
    "jaropeaste": "jaropear",
    "jaropeasteis": "jaropear",
    "jaropee": "jaropear",
    "jaropeemos": "jaropear",
    "jaropeen": "jaropear",
    "jaropees": "jaropear",
    "jaropemos": "jaropar",
    "jaropen": "jaropar",
    "jaropeo": "jaropear",
    "jaropeos": "jaropeo",
    "jaropes": "jarope",
    "jaropeábamos": "jaropear",
    "jaropeáis": "jaropear",
    "jaropeáramos": "jaropear",
    "jaropeáremos": "jaropear",
    "jaropeásemos": "jaropear",
    "jaropeé": "jaropear",
    "jaropeéis": "jaropear",
    "jaropeó": "jaropear",
    "jaropo": "jaropar",
    "jaropábamos": "jaropar",
    "jaropáis": "jaropar",
    "jaropáramos": "jaropar",
    "jaropáremos": "jaropar",
    "jaropásemos": "jaropar",
    "jaropé": "jaropar",
    "jaropéis": "jaropar",
    "jaropó": "jaropar",
    "jaros": "jaro",
    "jarositas": "jarosita",
    "jarras": "jarra",
    "jarrazos": "jarrazo",
    "jarrea": "jarrear",
    "jarreaba": "jarrear",
    "jarreabais": "jarrear",
    "jarreaban": "jarrear",
    "jarreabas": "jarrear",
    "jarread": "jarrear",
    "jarreada": "jarrear",
    "jarreadas": "jarrear",
    "jarreado": "jarrear",
    "jarreados": "jarrear",
    "jarreamos": "jarrear",
    "jarrean": "jarrear",
    "jarreando": "jarrear",
    "jarreara": "jarrear",
    "jarrearais": "jarrear",
    "jarrearan": "jarrear",
    "jarrearas": "jarrear",
    "jarreare": "jarrear",
    "jarreareis": "jarrear",
    "jarrearemos": "jarrear",
    "jarrearen": "jarrear",
    "jarreares": "jarrear",
    "jarrearon": "jarrear",
    "jarreará": "jarrear",
    "jarrearán": "jarrear",
    "jarrearás": "jarrear",
    "jarrearé": "jarrear",
    "jarrearéis": "jarrear",
    "jarrearía": "jarrear",
    "jarrearíais": "jarrear",
    "jarrearíamos": "jarrear",
    "jarrearían": "jarrear",
    "jarrearías": "jarrear",
    "jarreas": "jarrear",
    "jarrease": "jarrear",
    "jarreaseis": "jarrear",
    "jarreasen": "jarrear",
    "jarreases": "jarrear",
    "jarreaste": "jarrear",
    "jarreasteis": "jarrear",
    "jarree": "jarrear",
    "jarreemos": "jarrear",
    "jarreen": "jarrear",
    "jarrees": "jarrear",
    "jarreo": "jarrear",
    "jarrera": "jarrero",
    "jarreras": "jarrero",
    "jarreros": "jarrero",
    "jarretes": "jarrete",
    "jarreábamos": "jarrear",
    "jarreáis": "jarrear",
    "jarreáramos": "jarrear",
    "jarreáremos": "jarrear",
    "jarreásemos": "jarrear",
    "jarreé": "jarrear",
    "jarreéis": "jarrear",
    "jarreó": "jarrear",
    "jarrones": "jarrón",
    "jarros": "jarro",
    "jarrónes": "jarrón",
    "jaspea": "jaspear",
    "jaspeaba": "jaspear",
    "jaspeabais": "jaspear",
    "jaspeaban": "jaspear",
    "jaspeabas": "jaspear",
    "jaspead": "jaspear",
    "jaspeada": "jaspear",
    "jaspeadas": "jaspear",
    "jaspeado": "jaspear",
    "jaspeados": "jaspear",
    "jaspeamos": "jaspear",
    "jaspean": "jaspear",
    "jaspeando": "jaspear",
    "jaspeara": "jaspear",
    "jaspearais": "jaspear",
    "jaspearan": "jaspear",
    "jaspearas": "jaspear",
    "jaspeare": "jaspear",
    "jaspeareis": "jaspear",
    "jaspearemos": "jaspear",
    "jaspearen": "jaspear",
    "jaspeares": "jaspear",
    "jaspearon": "jaspear",
    "jaspeará": "jaspear",
    "jaspearán": "jaspear",
    "jaspearás": "jaspear",
    "jaspearé": "jaspear",
    "jaspearéis": "jaspear",
    "jaspearía": "jaspear",
    "jaspearíais": "jaspear",
    "jaspearíamos": "jaspear",
    "jaspearían": "jaspear",
    "jaspearías": "jaspear",
    "jaspeas": "jaspear",
    "jaspease": "jaspear",
    "jaspeaseis": "jaspear",
    "jaspeasen": "jaspear",
    "jaspeases": "jaspear",
    "jaspeaste": "jaspear",
    "jaspeasteis": "jaspear",
    "jaspee": "jaspear",
    "jaspeemos": "jaspear",
    "jaspeen": "jaspear",
    "jaspees": "jaspear",
    "jaspeo": "jaspear",
    "jaspes": "jaspe",
    "jaspeábamos": "jaspear",
    "jaspeáis": "jaspear",
    "jaspeáramos": "jaspear",
    "jaspeáremos": "jaspear",
    "jaspeásemos": "jaspear",
    "jaspeé": "jaspear",
    "jaspeéis": "jaspear",
    "jaspeó": "jaspear",
    "jaspilitas": "jaspilita",
    "jaspones": "jaspón",
    "jata": "jato",
    "jatas": "jato",
    "jativesa": "jativés",
    "jativesas": "jativés",
    "jativeses": "jativés",
    "jatos": "jato",
    "jauda": "jaudo",
    "jaudas": "jaudo",
    "jaudos": "jaudo",
    "jaujas": "jauja",
    "jaulas": "jaula",
    "jaulones": "jaulón",
    "jaurías": "jauría",
    "java": "javo",
    "javanesa": "javanés",
    "javanesas": "javanés",
    "javaneses": "javanés",
    "javas": "javo",
    "javeras": "javera",
    "javos": "javo",
    "jayana": "jayán",
    "jayanas": "jayán",
    "jayanes": "jayán",
    "jazces": "jazz",
    "jazmines": "jazmín",
    "jazmínes": "jazmín",
    "jazzistas": "jazzista",
    "jazzística": "jazzístico",
    "jazzísticas": "jazzístico",
    "jazzísticos": "jazzístico",
    "jedives": "jedive",
    "jeepes": "jeep",
    "jeeps": "jeep",
    "jefa": "jefe",
    "jefas": "jefe",
    "jefaturas": "jefatura",
    "jefes": "jefe",
    "jeitos": "jeito",
    "jemeres": "jemer",
    "jemes": "jeme",
    "jenabes": "jenabe",
    "jengibres": "jengibre",
    "jenízara": "jenízaro",
    "jenízaras": "jenízaro",
    "jenízaros": "jenízaro",
    "jeques": "jeque",
    "jerapellinas": "jerapellina",
    "jerarcas": "jerarca",
    "jerarquice": "jerarquizar",
    "jerarquicemos": "jerarquizar",
    "jerarquicen": "jerarquizar",
    "jerarquices": "jerarquizar",
    "jerarquicé": "jerarquizar",
    "jerarquicéis": "jerarquizar",
    "jerarquiza": "jerarquizar",
    "jerarquizaba": "jerarquizar",
    "jerarquizabais": "jerarquizar",
    "jerarquizaban": "jerarquizar",
    "jerarquizabas": "jerarquizar",
    "jerarquizaciones": "jerarquización",
    "jerarquizad": "jerarquizar",
    "jerarquizada": "jerarquizar",
    "jerarquizadas": "jerarquizar",
    "jerarquizado": "jerarquizar",
    "jerarquizados": "jerarquizar",
    "jerarquizamos": "jerarquizar",
    "jerarquizan": "jerarquizar",
    "jerarquizando": "jerarquizar",
    "jerarquizantes": "jerarquizante",
    "jerarquizara": "jerarquizar",
    "jerarquizarais": "jerarquizar",
    "jerarquizaran": "jerarquizar",
    "jerarquizaras": "jerarquizar",
    "jerarquizare": "jerarquizar",
    "jerarquizareis": "jerarquizar",
    "jerarquizaremos": "jerarquizar",
    "jerarquizaren": "jerarquizar",
    "jerarquizares": "jerarquizar",
    "jerarquizaron": "jerarquizar",
    "jerarquizará": "jerarquizar",
    "jerarquizarán": "jerarquizar",
    "jerarquizarás": "jerarquizar",
    "jerarquizaré": "jerarquizar",
    "jerarquizaréis": "jerarquizar",
    "jerarquizaría": "jerarquizar",
    "jerarquizaríais": "jerarquizar",
    "jerarquizaríamos": "jerarquizar",
    "jerarquizarían": "jerarquizar",
    "jerarquizarías": "jerarquizar",
    "jerarquizas": "jerarquizar",
    "jerarquizase": "jerarquizar",
    "jerarquizaseis": "jerarquizar",
    "jerarquizasen": "jerarquizar",
    "jerarquizases": "jerarquizar",
    "jerarquizaste": "jerarquizar",
    "jerarquizasteis": "jerarquizar",
    "jerarquizo": "jerarquizar",
    "jerarquizábamos": "jerarquizar",
    "jerarquizáis": "jerarquizar",
    "jerarquizáramos": "jerarquizar",
    "jerarquizáremos": "jerarquizar",
    "jerarquizásemos": "jerarquizar",
    "jerarquizó": "jerarquizar",
    "jerarquías": "jerarquía",
    "jerbos": "jerbo",
    "jereces": "jerez",
    "jeremiadas": "jeremiada",
    "jeremiquea": "jeremiquear",
    "jeremiqueaba": "jeremiquear",
    "jeremiqueabais": "jeremiquear",
    "jeremiqueaban": "jeremiquear",
    "jeremiqueabas": "jeremiquear",
    "jeremiquead": "jeremiquear",
    "jeremiqueada": "jeremiquear",
    "jeremiqueadas": "jeremiquear",
    "jeremiqueado": "jeremiquear",
    "jeremiqueados": "jeremiquear",
    "jeremiqueamos": "jeremiquear",
    "jeremiquean": "jeremiquear",
    "jeremiqueando": "jeremiquear",
    "jeremiqueara": "jeremiquear",
    "jeremiquearais": "jeremiquear",
    "jeremiquearan": "jeremiquear",
    "jeremiquearas": "jeremiquear",
    "jeremiqueare": "jeremiquear",
    "jeremiqueareis": "jeremiquear",
    "jeremiquearemos": "jeremiquear",
    "jeremiquearen": "jeremiquear",
    "jeremiqueares": "jeremiquear",
    "jeremiquearon": "jeremiquear",
    "jeremiqueará": "jeremiquear",
    "jeremiquearán": "jeremiquear",
    "jeremiquearás": "jeremiquear",
    "jeremiquearé": "jeremiquear",
    "jeremiquearéis": "jeremiquear",
    "jeremiquearía": "jeremiquear",
    "jeremiquearíais": "jeremiquear",
    "jeremiquearíamos": "jeremiquear",
    "jeremiquearían": "jeremiquear",
    "jeremiquearías": "jeremiquear",
    "jeremiqueas": "jeremiquear",
    "jeremiquease": "jeremiquear",
    "jeremiqueaseis": "jeremiquear",
    "jeremiqueasen": "jeremiquear",
    "jeremiqueases": "jeremiquear",
    "jeremiqueaste": "jeremiquear",
    "jeremiqueasteis": "jeremiquear",
    "jeremiquee": "jeremiquear",
    "jeremiqueemos": "jeremiquear",
    "jeremiqueen": "jeremiquear",
    "jeremiquees": "jeremiquear",
    "jeremiqueo": "jeremiquear",
    "jeremiqueábamos": "jeremiquear",
    "jeremiqueáis": "jeremiquear",
    "jeremiqueáramos": "jeremiquear",
    "jeremiqueáremos": "jeremiquear",
    "jeremiqueásemos": "jeremiquear",
    "jeremiqueé": "jeremiquear",
    "jeremiqueéis": "jeremiquear",
    "jeremiqueó": "jeremiquear",
    "jerezana": "jerezano",
    "jerezanas": "jerezano",
    "jerezanos": "jerezano",
    "jergas": "jerga",
    "jergones": "jergón",
    "jerguillas": "jerguilla",
    "jeribeques": "jeribeque",
    "jerifaltes": "jerifalte",
    "jerifes": "jerife",
    "jerifiana": "jerifiano",
    "jerifianas": "jerifiano",
    "jerifianos": "jerifiano",
    "jerigonzas": "jerigonza",
    "jeringa": "jeringar",
    "jeringaba": "jeringar",
    "jeringabais": "jeringar",
    "jeringaban": "jeringar",
    "jeringabas": "jeringar",
    "jeringad": "jeringar",
    "jeringada": "jeringar",
    "jeringadas": "jeringar",
    "jeringado": "jeringar",
    "jeringadora": "jeringador",
    "jeringadoras": "jeringador",
    "jeringadores": "jeringador",
    "jeringados": "jeringar",
    "jeringamos": "jeringar",
    "jeringan": "jeringar",
    "jeringando": "jeringar",
    "jeringara": "jeringar",
    "jeringarais": "jeringar",
    "jeringaran": "jeringar",
    "jeringaras": "jeringar",
    "jeringare": "jeringar",
    "jeringareis": "jeringar",
    "jeringaremos": "jeringar",
    "jeringaren": "jeringar",
    "jeringares": "jeringar",
    "jeringaron": "jeringar",
    "jeringará": "jeringar",
    "jeringarán": "jeringar",
    "jeringarás": "jeringar",
    "jeringaré": "jeringar",
    "jeringaréis": "jeringar",
    "jeringaría": "jeringar",
    "jeringaríais": "jeringar",
    "jeringaríamos": "jeringar",
    "jeringarían": "jeringar",
    "jeringarías": "jeringar",
    "jeringas": "jeringar",
    "jeringase": "jeringar",
    "jeringaseis": "jeringar",
    "jeringasen": "jeringar",
    "jeringases": "jeringar",
    "jeringaste": "jeringar",
    "jeringasteis": "jeringar",
    "jeringazos": "jeringazo",
    "jeringo": "jeringar",
    "jeringonzas": "jeringonza",
    "jeringue": "jeringar",
    "jeringuemos": "jeringar",
    "jeringuen": "jeringar",
    "jeringues": "jeringar",
    "jeringuillas": "jeringuilla",
    "jeringué": "jeringar",
    "jeringuéis": "jeringar",
    "jeringábamos": "jeringar",
    "jeringáis": "jeringar",
    "jeringáramos": "jeringar",
    "jeringáremos": "jeringar",
    "jeringásemos": "jeringar",
    "jeringó": "jeringar",
    "jeroglífica": "jeroglífico",
    "jeroglíficas": "jeroglífico",
    "jeroglíficos": "jeroglífico",
    "jeronimiana": "jeronimiano",
    "jeronimianas": "jeronimiano",
    "jeronimianos": "jeronimiano",
    "jerosolimitana": "jerosolimitano",
    "jerosolimitanas": "jerosolimitano",
    "jerosolimitanos": "jerosolimitano",
    "jerpas": "jerpa",
    "jerricotes": "jerricote",
    "jerseys": "jersey",
    "jerséis": "jersey",
    "jerugas": "jeruga",
    "jervillas": "jervilla",
    "jerárquica": "jerárquico",
    "jerárquicas": "jerárquico",
    "jerárquicos": "jerárquico",
    "jerónima": "jerónimo",
    "jerónimas": "jerónimo",
    "jerónimos": "jerónimo",
    "jesuitas": "jesuita",
    "jesusea": "jesusear",
    "jesuseaba": "jesusear",
    "jesuseabais": "jesusear",
    "jesuseaban": "jesusear",
    "jesuseabas": "jesusear",
    "jesusead": "jesusear",
    "jesuseada": "jesusear",
    "jesuseadas": "jesusear",
    "jesuseado": "jesusear",
    "jesuseados": "jesusear",
    "jesuseamos": "jesusear",
    "jesusean": "jesusear",
    "jesuseando": "jesusear",
    "jesuseara": "jesusear",
    "jesusearais": "jesusear",
    "jesusearan": "jesusear",
    "jesusearas": "jesusear",
    "jesuseare": "jesusear",
    "jesuseareis": "jesusear",
    "jesusearemos": "jesusear",
    "jesusearen": "jesusear",
    "jesuseares": "jesusear",
    "jesusearon": "jesusear",
    "jesuseará": "jesusear",
    "jesusearán": "jesusear",
    "jesusearás": "jesusear",
    "jesusearé": "jesusear",
    "jesusearéis": "jesusear",
    "jesusearía": "jesusear",
    "jesusearíais": "jesusear",
    "jesusearíamos": "jesusear",
    "jesusearían": "jesusear",
    "jesusearías": "jesusear",
    "jesuseas": "jesusear",
    "jesusease": "jesusear",
    "jesuseaseis": "jesusear",
    "jesuseasen": "jesusear",
    "jesuseases": "jesusear",
    "jesuseaste": "jesusear",
    "jesuseasteis": "jesusear",
    "jesusee": "jesusear",
    "jesuseemos": "jesusear",
    "jesuseen": "jesusear",
    "jesusees": "jesusear",
    "jesuseo": "jesusear",
    "jesuses": "jesús",
    "jesuseábamos": "jesusear",
    "jesuseáis": "jesusear",
    "jesuseáramos": "jesusear",
    "jesuseáremos": "jesusear",
    "jesuseásemos": "jesusear",
    "jesuseé": "jesusear",
    "jesuseéis": "jesusear",
    "jesuseó": "jesusear",
    "jetas": "jeta",
    "jetes": "jet",
    "jets": "jet",
    "jetuda": "jetudo",
    "jetudas": "jetudo",
    "jetudos": "jetudo",
    "jibias": "jibia",
    "jibiones": "jibión",
    "jiddishes": "jiddish",
    "jiennenses": "jiennense",
    "jifas": "jifa",
    "jifera": "jifero",
    "jiferas": "jifero",
    "jiferos": "jifero",
    "jijonas": "jijona",
    "jilguera": "jilguero",
    "jilgueras": "jilguero",
    "jilgueros": "jilguero",
    "jimelgas": "jimelga",
    "jimia": "jimio",
    "jimias": "jimio",
    "jimios": "jimio",
    "jindamas": "jindama",
    "jinestadas": "jinestada",
    "jinetas": "jineta",
    "jinetea": "jinetear",
    "jineteaba": "jinetear",
    "jineteabais": "jinetear",
    "jineteaban": "jinetear",
    "jineteabas": "jinetear",
    "jinetead": "jinetear",
    "jineteada": "jinetear",
    "jineteadas": "jinetear",
    "jineteado": "jinetear",
    "jineteados": "jinetear",
    "jineteamos": "jinetear",
    "jinetean": "jinetear",
    "jineteando": "jinetear",
    "jineteara": "jinetear",
    "jinetearais": "jinetear",
    "jinetearan": "jinetear",
    "jinetearas": "jinetear",
    "jineteare": "jinetear",
    "jineteareis": "jinetear",
    "jinetearemos": "jinetear",
    "jinetearen": "jinetear",
    "jineteares": "jinetear",
    "jinetearon": "jinetear",
    "jineteará": "jinetear",
    "jinetearán": "jinetear",
    "jinetearás": "jinetear",
    "jinetearé": "jinetear",
    "jinetearéis": "jinetear",
    "jinetearía": "jinetear",
    "jinetearíais": "jinetear",
    "jinetearíamos": "jinetear",
    "jinetearían": "jinetear",
    "jinetearías": "jinetear",
    "jineteas": "jinetear",
    "jinetease": "jinetear",
    "jineteaseis": "jinetear",
    "jineteasen": "jinetear",
    "jineteases": "jinetear",
    "jineteaste": "jinetear",
    "jineteasteis": "jinetear",
    "jinetee": "jinetear",
    "jineteemos": "jinetear",
    "jineteen": "jinetear",
    "jinetees": "jinetear",
    "jineteo": "jinetear",
    "jinetes": "jinete",
    "jineteábamos": "jinetear",
    "jineteáis": "jinetear",
    "jineteáramos": "jinetear",
    "jineteáremos": "jinetear",
    "jineteásemos": "jinetear",
    "jineteé": "jinetear",
    "jineteéis": "jinetear",
    "jineteó": "jinetear",
    "jingoísmos": "jingoísmo",
    "jingoístas": "jingoísta",
    "jipijapas": "jipijapa",
    "jirafas": "jirafa",
    "jirapliegas": "jirapliega",
    "jiras": "jira",
    "jireles": "jirel",
    "jironada": "jironado",
    "jironadas": "jironado",
    "jironados": "jironado",
    "jirones": "jirón",
    "jis": "ji",
    "jiu-jitsus": "jiu-jitsu",
    "jiujitsus": "jiujitsu",
    "jobes": "job",
    "jobs": "job",
    "jockeis": "jockey",
    "jockeys": "jockey",
    "jocosa": "jocoso",
    "jocosas": "jocoso",
    "jocoseria": "jocoserio",
    "jocoserias": "jocoserio",
    "jocoserios": "jocoserio",
    "jocosidades": "jocosidad",
    "jocosos": "jocoso",
    "jocunda": "jocundo",
    "jocundas": "jocundo",
    "jocundidades": "jocundidad",
    "jocundos": "jocundo",
    "joda": "joder",
    "jodamos": "joder",
    "jodan": "joder",
    "jodas": "joder",
    "jode": "joder",
    "joded": "joder",
    "jodemos": "joder",
    "joden": "joder",
    "joderemos": "joder",
    "joderá": "joder",
    "joderán": "joder",
    "joderás": "joder",
    "joderé": "joder",
    "joderéis": "joder",
    "jodería": "joder",
    "joderíais": "joder",
    "joderíamos": "joder",
    "joderían": "joder",
    "joderías": "joder",
    "jodes": "joder",
    "jodida": "joder",
    "jodidas": "joder",
    "jodido": "joder",
    "jodidos": "joder",
    "jodiendo": "joder",
    "jodiera": "joder",
    "jodierais": "joder",
    "jodieran": "joder",
    "jodieras": "joder",
    "jodiere": "joder",
    "jodiereis": "joder",
    "jodieren": "joder",
    "jodieres": "joder",
    "jodieron": "joder",
    "jodiese": "joder",
    "jodieseis": "joder",
    "jodiesen": "joder",
    "jodieses": "joder",
    "jodimos": "joder",
    "jodiste": "joder",
    "jodisteis": "joder",
    "jodiéramos": "joder",
    "jodiéremos": "joder",
    "jodiésemos": "joder",
    "jodió": "joder",
    "jodo": "joder",
    "jodos": "jodo",
    "jodáis": "joder",
    "jodéis": "joder",
    "jodí": "joder",
    "jodía": "joder",
    "jodíais": "joder",
    "jodíamos": "joder",
    "jodían": "joder",
    "jodías": "joder",
    "jofainas": "jofaina",
    "jolgorios": "jolgorio",
    "jollines": "jollín",
    "jolopes": "jolop",
    "jolops": "jolop",
    "jonia": "jonio",
    "jonias": "jonio",
    "jonios": "jonio",
    "jonjaba": "jonjabar",
    "jonjababa": "jonjabar",
    "jonjababais": "jonjabar",
    "jonjababan": "jonjabar",
    "jonjababas": "jonjabar",
    "jonjabad": "jonjabar",
    "jonjabada": "jonjabar",
    "jonjabadas": "jonjabar",
    "jonjabado": "jonjabar",
    "jonjabados": "jonjabar",
    "jonjabamos": "jonjabar",
    "jonjaban": "jonjabar",
    "jonjabando": "jonjabar",
    "jonjabara": "jonjabar",
    "jonjabarais": "jonjabar",
    "jonjabaran": "jonjabar",
    "jonjabaras": "jonjabar",
    "jonjabare": "jonjabar",
    "jonjabareis": "jonjabar",
    "jonjabaremos": "jonjabar",
    "jonjabaren": "jonjabar",
    "jonjabares": "jonjabar",
    "jonjabaron": "jonjabar",
    "jonjabará": "jonjabar",
    "jonjabarán": "jonjabar",
    "jonjabarás": "jonjabar",
    "jonjabaré": "jonjabar",
    "jonjabaréis": "jonjabar",
    "jonjabaría": "jonjabar",
    "jonjabaríais": "jonjabar",
    "jonjabaríamos": "jonjabar",
    "jonjabarían": "jonjabar",
    "jonjabarías": "jonjabar",
    "jonjabas": "jonjabar",
    "jonjabase": "jonjabar",
    "jonjabaseis": "jonjabar",
    "jonjabasen": "jonjabar",
    "jonjabases": "jonjabar",
    "jonjabaste": "jonjabar",
    "jonjabasteis": "jonjabar",
    "jonjabe": "jonjabar",
    "jonjabemos": "jonjabar",
    "jonjaben": "jonjabar",
    "jonjabes": "jonjabar",
    "jonjabo": "jonjabar",
    "jonjabábamos": "jonjabar",
    "jonjabáis": "jonjabar",
    "jonjabáramos": "jonjabar",
    "jonjabáremos": "jonjabar",
    "jonjabásemos": "jonjabar",
    "jonjabé": "jonjabar",
    "jonjabéis": "jonjabar",
    "jonjabó": "jonjabar",
    "joogings": "jooging",
    "jopeos": "jopeo",
    "jopos": "jopo",
    "jordana": "jordano",
    "jordanas": "jordano",
    "jordanos": "jordano",
    "jorfes": "jorfe",
    "jornadas": "jornada",
    "jornalera": "jornalero",
    "jornaleras": "jornalero",
    "jornaleros": "jornalero",
    "jornales": "jornal",
    "joroba": "jorobar",
    "jorobaba": "jorobar",
    "jorobabais": "jorobar",
    "jorobaban": "jorobar",
    "jorobabas": "jorobar",
    "jorobad": "jorobar",
    "jorobada": "jorobar",
    "jorobadas": "jorobar",
    "jorobado": "jorobar",
    "jorobados": "jorobar",
    "jorobaduras": "jorobadura",
    "jorobamos": "jorobar",
    "joroban": "jorobar",
    "jorobando": "jorobar",
    "jorobara": "jorobar",
    "jorobarais": "jorobar",
    "jorobaran": "jorobar",
    "jorobaras": "jorobar",
    "jorobare": "jorobar",
    "jorobareis": "jorobar",
    "jorobaremos": "jorobar",
    "jorobaren": "jorobar",
    "jorobares": "jorobar",
    "jorobaron": "jorobar",
    "jorobará": "jorobar",
    "jorobarán": "jorobar",
    "jorobarás": "jorobar",
    "jorobaré": "jorobar",
    "jorobaréis": "jorobar",
    "jorobaría": "jorobar",
    "jorobaríais": "jorobar",
    "jorobaríamos": "jorobar",
    "jorobarían": "jorobar",
    "jorobarías": "jorobar",
    "jorobas": "jorobar",
    "jorobase": "jorobar",
    "jorobaseis": "jorobar",
    "jorobasen": "jorobar",
    "jorobases": "jorobar",
    "jorobaste": "jorobar",
    "jorobasteis": "jorobar",
    "jorobe": "jorobar",
    "jorobemos": "jorobar",
    "joroben": "jorobar",
    "jorobes": "jorobar",
    "jorobetas": "jorobeta",
    "jorobo": "jorobar",
    "jorobábamos": "jorobar",
    "jorobáis": "jorobar",
    "jorobáramos": "jorobar",
    "jorobáremos": "jorobar",
    "jorobásemos": "jorobar",
    "jorobé": "jorobar",
    "jorobéis": "jorobar",
    "jorobó": "jorobar",
    "jorra": "jorrar",
    "jorraba": "jorrar",
    "jorrabais": "jorrar",
    "jorraban": "jorrar",
    "jorrabas": "jorrar",
    "jorrad": "jorrar",
    "jorrada": "jorrar",
    "jorradas": "jorrar",
    "jorrado": "jorrar",
    "jorrados": "jorrar",
    "jorramos": "jorrar",
    "jorran": "jorrar",
    "jorrando": "jorrar",
    "jorrara": "jorrar",
    "jorrarais": "jorrar",
    "jorraran": "jorrar",
    "jorraras": "jorrar",
    "jorrare": "jorrar",
    "jorrareis": "jorrar",
    "jorraremos": "jorrar",
    "jorraren": "jorrar",
    "jorrares": "jorrar",
    "jorraron": "jorrar",
    "jorrará": "jorrar",
    "jorrarán": "jorrar",
    "jorrarás": "jorrar",
    "jorraré": "jorrar",
    "jorraréis": "jorrar",
    "jorraría": "jorrar",
    "jorraríais": "jorrar",
    "jorraríamos": "jorrar",
    "jorrarían": "jorrar",
    "jorrarías": "jorrar",
    "jorras": "jorrar",
    "jorrase": "jorrar",
    "jorraseis": "jorrar",
    "jorrasen": "jorrar",
    "jorrases": "jorrar",
    "jorraste": "jorrar",
    "jorrasteis": "jorrar",
    "jorre": "jorrar",
    "jorremos": "jorrar",
    "jorren": "jorrar",
    "jorres": "jorrar",
    "jorro": "jorrar",
    "jorrábamos": "jorrar",
    "jorráis": "jorrar",
    "jorráramos": "jorrar",
    "jorráremos": "jorrar",
    "jorrásemos": "jorrar",
    "jorré": "jorrar",
    "jorréis": "jorrar",
    "jorró": "jorrar",
    "josas": "josa",
    "josefina": "josefino",
    "josefinas": "josefino",
    "josefinos": "josefino",
    "josefismos": "josefismo",
    "jotas": "jota",
    "joules": "joule",
    "jovenados": "jovenado",
    "jovencita": "jovencito",
    "jovencitas": "jovencito",
    "jovencitos": "jovencito",
    "joviales": "jovial",
    "jovialidades": "jovialidad",
    "joyas": "joya",
    "joyeles": "joyel",
    "joyera": "joyero",
    "joyeras": "joyero",
    "joyeros": "joyero",
    "joyerías": "joyería",
    "joystickes": "joystick",
    "joysticks": "joystick",
    "joyuyos": "joyuyo",
    "jrivnia": "hryvnia",
    "jrivnias": "hryvnia",
    "juaneteros": "juanetero",
    "juanetes": "juanete",
    "juardas": "juarda",
    "juardosa": "juardoso",
    "juardosas": "juardoso",
    "juardosos": "juardoso",
    "jubas": "juba",
    "jubetera": "jubetero",
    "jubeteras": "jubetero",
    "jubeteros": "jubetero",
    "jubeterías": "jubetería",
    "jubetes": "jubete",
    "jubila": "jubilar",
    "jubilaba": "jubilar",
    "jubilabais": "jubilar",
    "jubilaban": "jubilar",
    "jubilabas": "jubilar",
    "jubilaciones": "jubilación",
    "jubilad": "jubilar",
    "jubilada": "jubilar",
    "jubiladas": "jubilar",
    "jubilado": "jubilar",
    "jubilados": "jubilar",
    "jubilamos": "jubilar",
    "jubilan": "jubilar",
    "jubilando": "jubilar",
    "jubilara": "jubilar",
    "jubilarais": "jubilar",
    "jubilaran": "jubilar",
    "jubilaras": "jubilar",
    "jubilare": "jubilar",
    "jubilareis": "jubilar",
    "jubilaremos": "jubilar",
    "jubilaren": "jubilar",
    "jubilares": "jubilar",
    "jubilaron": "jubilar",
    "jubilará": "jubilar",
    "jubilarán": "jubilar",
    "jubilarás": "jubilar",
    "jubilaré": "jubilar",
    "jubilaréis": "jubilar",
    "jubilaría": "jubilar",
    "jubilaríais": "jubilar",
    "jubilaríamos": "jubilar",
    "jubilarían": "jubilar",
    "jubilarías": "jubilar",
    "jubilas": "jubilar",
    "jubilase": "jubilar",
    "jubilaseis": "jubilar",
    "jubilasen": "jubilar",
    "jubilases": "jubilar",
    "jubilaste": "jubilar",
    "jubilasteis": "jubilar",
    "jubile": "jubilar",
    "jubilemos": "jubilar",
    "jubilen": "jubilar",
    "jubileos": "jubileo",
    "jubiles": "jubilar",
    "jubilo": "jubilar",
    "jubilosa": "jubiloso",
    "jubilosas": "jubiloso",
    "jubilosos": "jubiloso",
    "jubilábamos": "jubilar",
    "jubiláis": "jubilar",
    "jubiláramos": "jubilar",
    "jubiláremos": "jubilar",
    "jubilásemos": "jubilar",
    "jubilé": "jubilar",
    "jubiléis": "jubilar",
    "jubiló": "jubilar",
    "jubonera": "jubonero",
    "juboneras": "jubonero",
    "juboneros": "jubonero",
    "jubones": "jubón",
    "jubónes": "jubón",
    "judaica": "judaico",
    "judaicas": "judaico",
    "judaicemos": "judaizar",
    "judaicos": "judaico",
    "judaicé": "judaizar",
    "judaicéis": "judaizar",
    "judaismos": "judaismo",
    "judaizaba": "judaizar",
    "judaizabais": "judaizar",
    "judaizaban": "judaizar",
    "judaizabas": "judaizar",
    "judaizaciones": "judaización",
    "judaizad": "judaizar",
    "judaizada": "judaizar",
    "judaizadas": "judaizar",
    "judaizado": "judaizar",
    "judaizados": "judaizar",
    "judaizamos": "judaizar",
    "judaizando": "judaizar",
    "judaizantes": "judaizante",
    "judaizara": "judaizar",
    "judaizarais": "judaizar",
    "judaizaran": "judaizar",
    "judaizaras": "judaizar",
    "judaizare": "judaizar",
    "judaizareis": "judaizar",
    "judaizaremos": "judaizar",
    "judaizaren": "judaizar",
    "judaizares": "judaizar",
    "judaizaron": "judaizar",
    "judaizará": "judaizar",
    "judaizarán": "judaizar",
    "judaizarás": "judaizar",
    "judaizaré": "judaizar",
    "judaizaréis": "judaizar",
    "judaizaría": "judaizar",
    "judaizaríais": "judaizar",
    "judaizaríamos": "judaizar",
    "judaizarían": "judaizar",
    "judaizarías": "judaizar",
    "judaizase": "judaizar",
    "judaizaseis": "judaizar",
    "judaizasen": "judaizar",
    "judaizases": "judaizar",
    "judaizaste": "judaizar",
    "judaizasteis": "judaizar",
    "judaizábamos": "judaizar",
    "judaizáis": "judaizar",
    "judaizáramos": "judaizar",
    "judaizáremos": "judaizar",
    "judaizásemos": "judaizar",
    "judaizó": "judaizar",
    "judaíce": "judaizar",
    "judaícen": "judaizar",
    "judaíces": "judaizar",
    "judaísmos": "judaísmo",
    "judaíza": "judaizar",
    "judaízan": "judaizar",
    "judaízas": "judaizar",
    "judaízo": "judaizar",
    "judeoespañola": "judeoespañol",
    "judeoespañolas": "judeoespañol",
    "judeoespañoles": "judeoespañol",
    "juderías": "judería",
    "judiadas": "judiada",
    "judicaturas": "judicatura",
    "judiciales": "judicial",
    "judicialidades": "judicialidad",
    "judogis": "judogi",
    "judokas": "judoka",
    "judos": "judo",
    "judía": "judío",
    "judías": "judío",
    "judíos": "judío",
    "jueces": "juez",
    "jueces-árbitro": "juez-árbitro",
    "jueces-árbitro-composición": "juez-árbitro-composición",
    "jueces-árbitro-ejecución": "juez-árbitro-ejecución",
    "juega": "jugar",
    "juegan": "jugar",
    "juegas": "jugar",
    "juego": "jugar",
    "juegos": "juego",
    "juegue": "jugar",
    "jueguen": "jugar",
    "juegues": "jugar",
    "juergas": "juerga",
    "juerguea": "juerguear",
    "juergueaba": "juerguear",
    "juergueabais": "juerguear",
    "juergueaban": "juerguear",
    "juergueabas": "juerguear",
    "juerguead": "juerguear",
    "juergueada": "juerguear",
    "juergueadas": "juerguear",
    "juergueado": "juerguear",
    "juergueados": "juerguear",
    "juergueamos": "juerguear",
    "juerguean": "juerguear",
    "juergueando": "juerguear",
    "juergueara": "juerguear",
    "juerguearais": "juerguear",
    "juerguearan": "juerguear",
    "juerguearas": "juerguear",
    "juergueare": "juerguear",
    "juergueareis": "juerguear",
    "juerguearemos": "juerguear",
    "juerguearen": "juerguear",
    "juergueares": "juerguear",
    "juerguearon": "juerguear",
    "juergueará": "juerguear",
    "juerguearán": "juerguear",
    "juerguearás": "juerguear",
    "juerguearé": "juerguear",
    "juerguearéis": "juerguear",
    "juerguearía": "juerguear",
    "juerguearíais": "juerguear",
    "juerguearíamos": "juerguear",
    "juerguearían": "juerguear",
    "juerguearías": "juerguear",
    "juergueas": "juerguear",
    "juerguease": "juerguear",
    "juergueaseis": "juerguear",
    "juergueasen": "juerguear",
    "juergueases": "juerguear",
    "juergueaste": "juerguear",
    "juergueasteis": "juerguear",
    "juerguee": "juerguear",
    "juergueemos": "juerguear",
    "juergueen": "juerguear",
    "juerguees": "juerguear",
    "juergueo": "juerguear",
    "juergueábamos": "juerguear",
    "juergueáis": "juerguear",
    "juergueáramos": "juerguear",
    "juergueáremos": "juerguear",
    "juergueásemos": "juerguear",
    "juergueé": "juerguear",
    "juergueéis": "juerguear",
    "juergueó": "juerguear",
    "juerguistas": "juerguista",
    "jueza": "juez",
    "juezas": "juez",
    "jugaba": "jugar",
    "jugabais": "jugar",
    "jugaban": "jugar",
    "jugabas": "jugar",
    "jugad": "jugar",
    "jugada": "jugar",
    "jugadas": "jugar",
    "jugado": "jugar",
    "jugadora": "jugador",
    "jugadoras": "jugador",
    "jugadores": "jugador",
    "jugados": "jugar",
    "jugamos": "jugar",
    "jugando": "jugar",
    "jugara": "jugar",
    "jugarais": "jugar",
    "jugaran": "jugar",
    "jugaras": "jugar",
    "jugare": "jugar",
    "jugareis": "jugar",
    "jugaremos": "jugar",
    "jugaren": "jugar",
    "jugares": "jugar",
    "jugaron": "jugar",
    "jugarretas": "jugarreta",
    "jugará": "jugar",
    "jugarán": "jugar",
    "jugarás": "jugar",
    "jugaré": "jugar",
    "jugaréis": "jugar",
    "jugaría": "jugar",
    "jugaríais": "jugar",
    "jugaríamos": "jugar",
    "jugarían": "jugar",
    "jugarías": "jugar",
    "jugase": "jugar",
    "jugaseis": "jugar",
    "jugasen": "jugar",
    "jugases": "jugar",
    "jugaste": "jugar",
    "jugasteis": "jugar",
    "juglandáceas": "juglandácea",
    "juglares": "juglar",
    "juglaresca": "juglaresco",
    "juglarescas": "juglaresco",
    "juglarescos": "juglaresco",
    "juglarías": "juglaría",
    "juglerías": "juglería",
    "jugos": "jugo",
    "jugosa": "jugoso",
    "jugosas": "jugoso",
    "jugosidades": "jugosidad",
    "jugosos": "jugoso",
    "juguemos": "jugar",
    "juguetea": "juguetear",
    "jugueteaba": "juguetear",
    "jugueteabais": "juguetear",
    "jugueteaban": "juguetear",
    "jugueteabas": "juguetear",
    "juguetead": "juguetear",
    "jugueteada": "juguetear",
    "jugueteadas": "juguetear",
    "jugueteado": "juguetear",
    "jugueteados": "juguetear",
    "jugueteamos": "juguetear",
    "juguetean": "juguetear",
    "jugueteando": "juguetear",
    "jugueteara": "juguetear",
    "juguetearais": "juguetear",
    "juguetearan": "juguetear",
    "juguetearas": "juguetear",
    "jugueteare": "juguetear",
    "jugueteareis": "juguetear",
    "juguetearemos": "juguetear",
    "juguetearen": "juguetear",
    "jugueteares": "juguetear",
    "juguetearon": "juguetear",
    "jugueteará": "juguetear",
    "juguetearán": "juguetear",
    "juguetearás": "juguetear",
    "juguetearé": "juguetear",
    "juguetearéis": "juguetear",
    "juguetearía": "juguetear",
    "juguetearíais": "juguetear",
    "juguetearíamos": "juguetear",
    "juguetearían": "juguetear",
    "juguetearías": "juguetear",
    "jugueteas": "juguetear",
    "juguetease": "juguetear",
    "jugueteaseis": "juguetear",
    "jugueteasen": "juguetear",
    "jugueteases": "juguetear",
    "jugueteaste": "juguetear",
    "jugueteasteis": "juguetear",
    "juguetee": "juguetear",
    "jugueteemos": "juguetear",
    "jugueteen": "juguetear",
    "juguetees": "juguetear",
    "jugueteo": "juguetear",
    "jugueteos": "jugueteo",
    "jugueterías": "juguetería",
    "juguetes": "juguete",
    "jugueteábamos": "juguetear",
    "jugueteáis": "juguetear",
    "jugueteáramos": "juguetear",
    "jugueteáremos": "juguetear",
    "jugueteásemos": "juguetear",
    "jugueteé": "juguetear",
    "jugueteéis": "juguetear",
    "jugueteó": "juguetear",
    "juguetona": "juguetón",
    "juguetonas": "juguetón",
    "juguetones": "juguetón",
    "jugué": "jugar",
    "juguéis": "jugar",
    "jugábamos": "jugar",
    "jugáis": "jugar",
    "jugáramos": "jugar",
    "jugáremos": "jugar",
    "jugásemos": "jugar",
    "jugó": "jugar",
    "juicios": "juicio",
    "juiciosa": "juicioso",
    "juiciosas": "juicioso",
    "juiciosos": "juicioso",
    "jul.": "julio",
    "julepea": "julepear",
    "julepeaba": "julepear",
    "julepeabais": "julepear",
    "julepeaban": "julepear",
    "julepeabas": "julepear",
    "julepead": "julepear",
    "julepeada": "julepear",
    "julepeadas": "julepear",
    "julepeado": "julepear",
    "julepeados": "julepear",
    "julepeamos": "julepear",
    "julepean": "julepear",
    "julepeando": "julepear",
    "julepeara": "julepear",
    "julepearais": "julepear",
    "julepearan": "julepear",
    "julepearas": "julepear",
    "julepeare": "julepear",
    "julepeareis": "julepear",
    "julepearemos": "julepear",
    "julepearen": "julepear",
    "julepeares": "julepear",
    "julepearon": "julepear",
    "julepeará": "julepear",
    "julepearán": "julepear",
    "julepearás": "julepear",
    "julepearé": "julepear",
    "julepearéis": "julepear",
    "julepearía": "julepear",
    "julepearíais": "julepear",
    "julepearíamos": "julepear",
    "julepearían": "julepear",
    "julepearías": "julepear",
    "julepeas": "julepear",
    "julepease": "julepear",
    "julepeaseis": "julepear",
    "julepeasen": "julepear",
    "julepeases": "julepear",
    "julepeaste": "julepear",
    "julepeasteis": "julepear",
    "julepee": "julepear",
    "julepeemos": "julepear",
    "julepeen": "julepear",
    "julepees": "julepear",
    "julepeo": "julepear",
    "julepes": "julepe",
    "julepeábamos": "julepear",
    "julepeáis": "julepear",
    "julepeáramos": "julepear",
    "julepeáremos": "julepear",
    "julepeásemos": "julepear",
    "julepeé": "julepear",
    "julepeéis": "julepear",
    "julepeó": "julepear",
    "juliana": "juliano",
    "julianas": "juliano",
    "julianos": "juliano",
    "julias": "julia",
    "julios": "julio",
    "julos": "julo",
    "juma": "jumar",
    "jumaba": "jumar",
    "jumabais": "jumar",
    "jumaban": "jumar",
    "jumabas": "jumar",
    "jumad": "jumar",
    "jumada": "jumar",
    "jumadas": "jumar",
    "jumado": "jumar",
    "jumados": "jumar",
    "jumamos": "jumar",
    "juman": "jumar",
    "jumando": "jumar",
    "jumara": "jumar",
    "jumarais": "jumar",
    "jumaran": "jumar",
    "jumaras": "jumar",
    "jumare": "jumar",
    "jumareis": "jumar",
    "jumaremos": "jumar",
    "jumaren": "jumar",
    "jumares": "jumar",
    "jumaron": "jumar",
    "jumará": "jumar",
    "jumarán": "jumar",
    "jumarás": "jumar",
    "jumaré": "jumar",
    "jumaréis": "jumar",
    "jumaría": "jumar",
    "jumaríais": "jumar",
    "jumaríamos": "jumar",
    "jumarían": "jumar",
    "jumarías": "jumar",
    "jumas": "jumar",
    "jumase": "jumar",
    "jumaseis": "jumar",
    "jumasen": "jumar",
    "jumases": "jumar",
    "jumaste": "jumar",
    "jumasteis": "jumar",
    "jume": "jumar",
    "jumemos": "jumar",
    "jumen": "jumar",
    "jumenta": "jumento",
    "jumentales": "jumental",
    "jumentas": "jumento",
    "jumentos": "jumento",
    "jumes": "jumar",
    "jumo": "jumar",
    "jumábamos": "jumar",
    "jumáis": "jumar",
    "jumáramos": "jumar",
    "jumáremos": "jumar",
    "jumásemos": "jumar",
    "jumé": "jumar",
    "juméis": "jumar",
    "jumó": "jumar",
    "jun.": "junio",
    "juncadas": "juncada",
    "juncales": "juncal",
    "juncares": "juncar",
    "junciales": "juncial",
    "juncianas": "junciana",
    "juncias": "juncia",
    "juncina": "juncino",
    "juncinas": "juncino",
    "juncinos": "juncino",
    "juncos": "junco",
    "juncosa": "juncoso",
    "juncosas": "juncoso",
    "juncosos": "juncoso",
    "juncáceas": "juncácea",
    "junglas": "jungla",
    "juniora": "júnior",
    "junioras": "júnior",
    "juniores": "júnior",
    "juniors": "junior",
    "junios": "junio",
    "junquerales": "junqueral",
    "junqueras": "junquera",
    "junquillos": "junquillo",
    "junta": "junto",
    "juntaba": "juntar",
    "juntabais": "juntar",
    "juntaban": "juntar",
    "juntabas": "juntar",
    "juntad": "juntar",
    "juntada": "juntar",
    "juntadas": "juntar",
    "juntado": "juntar",
    "juntados": "juntar",
    "juntamos": "juntar",
    "juntan": "juntar",
    "juntando": "juntar",
    "juntara": "juntar",
    "juntarais": "juntar",
    "juntaran": "juntar",
    "juntaras": "juntar",
    "juntare": "juntar",
    "juntareis": "juntar",
    "juntaremos": "juntar",
    "juntaren": "juntar",
    "juntares": "juntar",
    "juntaron": "juntar",
    "juntará": "juntar",
    "juntarán": "juntar",
    "juntarás": "juntar",
    "juntaré": "juntar",
    "juntaréis": "juntar",
    "juntaría": "juntar",
    "juntaríais": "juntar",
    "juntaríamos": "juntar",
    "juntarían": "juntar",
    "juntarías": "juntar",
    "juntas": "junto",
    "juntase": "juntar",
    "juntaseis": "juntar",
    "juntasen": "juntar",
    "juntases": "juntar",
    "juntaste": "juntar",
    "juntasteis": "juntar",
    "junte": "juntar",
    "juntemos": "juntar",
    "junten": "juntar",
    "junteras": "juntera",
    "juntes": "juntar",
    "junto": "juntar",
    "juntos": "junto",
    "junturas": "juntura",
    "juntábamos": "juntar",
    "juntáis": "juntar",
    "juntáramos": "juntar",
    "juntáremos": "juntar",
    "juntásemos": "juntar",
    "junté": "juntar",
    "juntéis": "juntar",
    "juntó": "juntar",
    "juníperos": "junípero",
    "jupiterina": "jupiterino",
    "jupiterinas": "jupiterino",
    "jupiterinos": "jupiterino",
    "jura": "jurar",
    "juraba": "jurar",
    "jurabais": "jurar",
    "juraban": "jurar",
    "jurabas": "jurar",
    "jurad": "jurar",
    "jurada": "jurar",
    "juradas": "jurar",
    "jurado": "jurar",
    "juradora": "jurador",
    "juradoras": "jurador",
    "juradores": "jurador",
    "jurados": "jurar",
    "jurados-composición": "jurado-composición",
    "jurados-ejecución": "jurado-ejecución",
    "juramenta": "juramentar",
    "juramentaba": "juramentar",
    "juramentabais": "juramentar",
    "juramentaban": "juramentar",
    "juramentabas": "juramentar",
    "juramentad": "juramentar",
    "juramentada": "juramentar",
    "juramentadas": "juramentar",
    "juramentado": "juramentar",
    "juramentados": "juramentar",
    "juramentamos": "juramentar",
    "juramentan": "juramentar",
    "juramentando": "juramentar",
    "juramentara": "juramentar",
    "juramentarais": "juramentar",
    "juramentaran": "juramentar",
    "juramentaras": "juramentar",
    "juramentare": "juramentar",
    "juramentareis": "juramentar",
    "juramentaremos": "juramentar",
    "juramentaren": "juramentar",
    "juramentares": "juramentar",
    "juramentaron": "juramentar",
    "juramentará": "juramentar",
    "juramentarán": "juramentar",
    "juramentarás": "juramentar",
    "juramentaré": "juramentar",
    "juramentaréis": "juramentar",
    "juramentaría": "juramentar",
    "juramentaríais": "juramentar",
    "juramentaríamos": "juramentar",
    "juramentarían": "juramentar",
    "juramentarías": "juramentar",
    "juramentas": "juramentar",
    "juramentase": "juramentar",
    "juramentaseis": "juramentar",
    "juramentasen": "juramentar",
    "juramentases": "juramentar",
    "juramentaste": "juramentar",
    "juramentasteis": "juramentar",
    "juramente": "juramentar",
    "juramentemos": "juramentar",
    "juramenten": "juramentar",
    "juramentes": "juramentar",
    "juramento": "juramentar",
    "juramentos": "juramento",
    "juramentábamos": "juramentar",
    "juramentáis": "juramentar",
    "juramentáramos": "juramentar",
    "juramentáremos": "juramentar",
    "juramentásemos": "juramentar",
    "juramenté": "juramentar",
    "juramentéis": "juramentar",
    "juramentó": "juramentar",
    "juramos": "jurar",
    "juran": "jurar",
    "jurando": "jurar",
    "jurara": "jurar",
    "jurarais": "jurar",
    "juraran": "jurar",
    "juraras": "jurar",
    "jurare": "jurar",
    "jurareis": "jurar",
    "juraremos": "jurar",
    "juraren": "jurar",
    "jurares": "jurar",
    "juraron": "jurar",
    "jurará": "jurar",
    "jurarán": "jurar",
    "jurarás": "jurar",
    "juraré": "jurar",
    "juraréis": "jurar",
    "juraría": "jurar",
    "juraríais": "jurar",
    "juraríamos": "jurar",
    "jurarían": "jurar",
    "jurarías": "jurar",
    "juras": "jurar",
    "jurase": "jurar",
    "juraseis": "jurar",
    "jurasen": "jurar",
    "jurases": "jurar",
    "juraste": "jurar",
    "jurasteis": "jurar",
    "juratoria": "juratorio",
    "juratorias": "juratorio",
    "juratorios": "juratorio",
    "jure": "jurar",
    "jureles": "jurel",
    "juremos": "jurar",
    "juren": "jurar",
    "jures": "jurar",
    "juridicidades": "juridicidad",
    "juridicopenales": "juridicopenal",
    "jurisconsulta": "jurisconsulto",
    "jurisconsultas": "jurisconsulto",
    "jurisconsultos": "jurisconsulto",
    "jurisdiccionales": "jurisdiccional",
    "jurisdicciones": "jurisdicción",
    "jurispericias": "jurispericia",
    "jurisperita": "jurisperito",
    "jurisperitas": "jurisperito",
    "jurisperitos": "jurisperito",
    "jurisprudenciales": "jurisprudencial",
    "jurisprudencias": "jurisprudencia",
    "juristas": "jurista",
    "juro": "jurar",
    "jurábamos": "jurar",
    "juráis": "jurar",
    "juráramos": "jurar",
    "juráremos": "jurar",
    "jurásemos": "jurar",
    "jurásica": "jurásico",
    "jurásicas": "jurásico",
    "jurásicos": "jurásico",
    "juré": "jurar",
    "juréis": "jurar",
    "jurídica": "jurídico",
    "jurídicas": "jurídico",
    "jurídico-políticoa": "jurídico-político",
    "jurídico-políticoas": "jurídico-político",
    "jurídico-políticoo": "jurídico-político",
    "jurídico-políticoos": "jurídico-político",
    "jurídicos": "jurídico",
    "juró": "jurar",
    "justa": "justo",
    "justaba": "justar",
    "justabais": "justar",
    "justaban": "justar",
    "justabas": "justar",
    "justad": "justar",
    "justada": "justar",
    "justadas": "justar",
    "justado": "justar",
    "justadores": "justador",
    "justados": "justar",
    "justamos": "justar",
    "justan": "justar",
    "justando": "justar",
    "justara": "justar",
    "justarais": "justar",
    "justaran": "justar",
    "justaras": "justar",
    "justare": "justar",
    "justareis": "justar",
    "justaremos": "justar",
    "justaren": "justar",
    "justares": "justar",
    "justaron": "justar",
    "justará": "justar",
    "justarán": "justar",
    "justarás": "justar",
    "justaré": "justar",
    "justaréis": "justar",
    "justaría": "justar",
    "justaríais": "justar",
    "justaríamos": "justar",
    "justarían": "justar",
    "justarías": "justar",
    "justas": "justo",
    "justase": "justar",
    "justaseis": "justar",
    "justasen": "justar",
    "justases": "justar",
    "justaste": "justar",
    "justasteis": "justar",
    "juste": "justar",
    "justemos": "justar",
    "justen": "justar",
    "justes": "justar",
    "justiciables": "justiciable",
    "justicialismos": "justicialismo",
    "justicialistas": "justicialista",
    "justicias": "justicia",
    "justiciera": "justiciero",
    "justicieras": "justiciero",
    "justicieros": "justiciero",
    "justifica": "justificar",
    "justificaba": "justificar",
    "justificabais": "justificar",
    "justificaban": "justificar",
    "justificabas": "justificar",
    "justificables": "justificable",
    "justificaciones": "justificación",
    "justificacionismos": "justificacionismo",
    "justificad": "justificar",
    "justificada": "justificar",
    "justificadas": "justificar",
    "justificado": "justificar",
    "justificadora": "justificador",
    "justificadoras": "justificador",
    "justificadores": "justificador",
    "justificados": "justificar",
    "justificamos": "justificar",
    "justifican": "justificar",
    "justificando": "justificar",
    "justificantes": "justificante",
    "justificara": "justificar",
    "justificarais": "justificar",
    "justificaran": "justificar",
    "justificaras": "justificar",
    "justificare": "justificar",
    "justificareis": "justificar",
    "justificaremos": "justificar",
    "justificaren": "justificar",
    "justificares": "justificar",
    "justificaron": "justificar",
    "justificará": "justificar",
    "justificarán": "justificar",
    "justificarás": "justificar",
    "justificaré": "justificar",
    "justificaréis": "justificar",
    "justificaría": "justificar",
    "justificaríais": "justificar",
    "justificaríamos": "justificar",
    "justificarían": "justificar",
    "justificarías": "justificar",
    "justificas": "justificar",
    "justificase": "justificar",
    "justificaseis": "justificar",
    "justificasen": "justificar",
    "justificases": "justificar",
    "justificaste": "justificar",
    "justificasteis": "justificar",
    "justificativa": "justificativo",
    "justificativas": "justificativo",
    "justificativos": "justificativo",
    "justificatoria": "justificatorio",
    "justificatorias": "justificatorio",
    "justificatorios": "justificatorio",
    "justifico": "justificar",
    "justificábamos": "justificar",
    "justificáis": "justificar",
    "justificáramos": "justificar",
    "justificáremos": "justificar",
    "justificásemos": "justificar",
    "justificó": "justificar",
    "justifique": "justificar",
    "justifiquemos": "justificar",
    "justifiquen": "justificar",
    "justifiques": "justificar",
    "justifiqué": "justificar",
    "justifiquéis": "justificar",
    "justillos": "justillo",
    "justinianea": "justinianeo",
    "justinianeas": "justinianeo",
    "justinianeos": "justinianeo",
    "justiprecia": "justipreciar",
    "justipreciaba": "justipreciar",
    "justipreciabais": "justipreciar",
    "justipreciaban": "justipreciar",
    "justipreciabas": "justipreciar",
    "justipreciaciones": "justipreciación",
    "justipreciad": "justipreciar",
    "justipreciada": "justipreciar",
    "justipreciadas": "justipreciar",
    "justipreciado": "justipreciar",
    "justipreciados": "justipreciar",
    "justipreciamos": "justipreciar",
    "justiprecian": "justipreciar",
    "justipreciando": "justipreciar",
    "justipreciara": "justipreciar",
    "justipreciarais": "justipreciar",
    "justipreciaran": "justipreciar",
    "justipreciaras": "justipreciar",
    "justipreciare": "justipreciar",
    "justipreciareis": "justipreciar",
    "justipreciaremos": "justipreciar",
    "justipreciaren": "justipreciar",
    "justipreciares": "justipreciar",
    "justipreciaron": "justipreciar",
    "justipreciará": "justipreciar",
    "justipreciarán": "justipreciar",
    "justipreciarás": "justipreciar",
    "justipreciaré": "justipreciar",
    "justipreciaréis": "justipreciar",
    "justipreciaría": "justipreciar",
    "justipreciaríais": "justipreciar",
    "justipreciaríamos": "justipreciar",
    "justipreciarían": "justipreciar",
    "justipreciarías": "justipreciar",
    "justiprecias": "justipreciar",
    "justipreciase": "justipreciar",
    "justipreciaseis": "justipreciar",
    "justipreciasen": "justipreciar",
    "justipreciases": "justipreciar",
    "justipreciaste": "justipreciar",
    "justipreciasteis": "justipreciar",
    "justiprecie": "justipreciar",
    "justipreciemos": "justipreciar",
    "justiprecien": "justipreciar",
    "justiprecies": "justipreciar",
    "justiprecio": "justipreciar",
    "justiprecios": "justiprecio",
    "justipreciábamos": "justipreciar",
    "justipreciáis": "justipreciar",
    "justipreciáramos": "justipreciar",
    "justipreciáremos": "justipreciar",
    "justipreciásemos": "justipreciar",
    "justiprecié": "justipreciar",
    "justipreciéis": "justipreciar",
    "justipreció": "justipreciar",
    "justo": "justar",
    "justos": "justo",
    "justábamos": "justar",
    "justáis": "justar",
    "justáramos": "justar",
    "justáremos": "justar",
    "justásemos": "justar",
    "justé": "justar",
    "justéis": "justar",
    "justó": "justar",
    "juvenca": "juvenco",
    "juvencas": "juvenco",
    "juvencos": "juvenco",
    "juveniles": "juvenil",
    "juventudes": "juventud",
    "juzga": "juzgar",
    "juzgaba": "juzgar",
    "juzgabais": "juzgar",
    "juzgaban": "juzgar",
    "juzgabas": "juzgar",
    "juzgad": "juzgar",
    "juzgada": "juzgar",
    "juzgadas": "juzgar",
    "juzgado": "juzgar",
    "juzgadora": "juzgador",
    "juzgadoras": "juzgador",
    "juzgadores": "juzgador",
    "juzgados": "juzgar",
    "juzgamos": "juzgar",
    "juzgan": "juzgar",
    "juzgando": "juzgar",
    "juzgantes": "juzgante",
    "juzgara": "juzgar",
    "juzgarais": "juzgar",
    "juzgaran": "juzgar",
    "juzgaras": "juzgar",
    "juzgare": "juzgar",
    "juzgareis": "juzgar",
    "juzgaremos": "juzgar",
    "juzgaren": "juzgar",
    "juzgares": "juzgar",
    "juzgaron": "juzgar",
    "juzgará": "juzgar",
    "juzgarán": "juzgar",
    "juzgarás": "juzgar",
    "juzgaré": "juzgar",
    "juzgaréis": "juzgar",
    "juzgaría": "juzgar",
    "juzgaríais": "juzgar",
    "juzgaríamos": "juzgar",
    "juzgarían": "juzgar",
    "juzgarías": "juzgar",
    "juzgas": "juzgar",
    "juzgase": "juzgar",
    "juzgaseis": "juzgar",
    "juzgasen": "juzgar",
    "juzgases": "juzgar",
    "juzgaste": "juzgar",
    "juzgasteis": "juzgar",
    "juzgo": "juzgar",
    "juzgona": "juzgón",
    "juzgonas": "juzgón",
    "juzgones": "juzgón",
    "juzgue": "juzgar",
    "juzguemos": "juzgar",
    "juzguen": "juzgar",
    "juzgues": "juzgar",
    "juzgué": "juzgar",
    "juzguéis": "juzgar",
    "juzgábamos": "juzgar",
    "juzgáis": "juzgar",
    "juzgáramos": "juzgar",
    "juzgáremos": "juzgar",
    "juzgásemos": "juzgar",
    "juzgó": "juzgar",
    "jábecas": "jábeca",
    "jábegas": "jábega",
    "jácara": "jácaro",
    "jácaras": "jácaro",
    "jácaros": "jácaro",
    "jácenas": "jácena",
    "jáculos": "jáculo",
    "jámilas": "jámila",
    "jándala": "jándalo",
    "jándalas": "jándalo",
    "jándalos": "jándalo",
    "jáquimas": "jáquima",
    "jázaros": "jázaro",
    "jíbara": "jíbaro",
    "jíbaras": "jíbaro",
    "jíbaros": "jíbaro",
    "jícaras": "jícara",
    "jónica": "jónico",
    "jónicas": "jónico",
    "jónicos": "jónico",
    "jóqueres": "jóquer",
    "jóvenes": "joven",
    "júbilos": "júbilo",
    "júcaros": "júcaro",
    "júniors": "júnior",
    "kHz": "kilohercio",
    "kJ": "kilojulio",
    "kabukis": "kabuki",
    "kafkiana": "kafkiano",
    "kafkianas": "kafkiano",
    "kafkianos": "kafkiano",
    "kainitas": "kainita",
    "kaiots": "kaiot",
    "kakemonos": "kakemono",
    "kakis": "kaki",
    "kamacitas": "kamacita",
    "kames": "kame",
    "kamikazes": "kamikaze",
    "kampucheana": "kampucheano",
    "kampucheanas": "kampucheano",
    "kampucheanos": "kampucheano",
    "kanaras": "kanara",
    "kanes": "kan",
    "kantiana": "kantiano",
    "kantianas": "kantiano",
    "kantianos": "kantiano",
    "kantismos": "kantismo",
    "kaones": "kaón",
    "kapocs": "kapoc",
    "kappas": "kappa",
    "kaputt": "caput",
    "karakules": "karakul",
    "karatecas": "karateca",
    "karatekas": "karateka",
    "karates": "karate",
    "karmas": "karma",
    "karstes": "karst",
    "karstificaciones": "karstificación",
    "karstologías": "karstología",
    "karts": "kart",
    "kasolitas": "kasolita",
    "katiuskas": "katiuska",
    "kayacs": "kayac",
    "kayak-polos": "kayak-polo",
    "kayakes": "kayak",
    "kayakistas": "kayakista",
    "kayaks": "kayak",
    "kazakhstana": "kazakhstano",
    "kazakhstanas": "kazakhstano",
    "kazakhstanos": "kazakhstano",
    "kcal": "kilocaloría",
    "kefirs": "kéfir",
    "kelvins": "kelvin",
    "kenafs": "kenaf",
    "keniana": "keniano",
    "kenianas": "keniano",
    "kenianos": "keniano",
    "keniatas": "keniata",
    "kenotrones": "kenotrón",
    "kentismos": "kentismo",
    "kermesses": "kermesse",
    "kernitas": "kernita",
    "kerosenos": "keroseno",
    "ketches": "ketch",
    "ketchups": "ketchup",
    "keynesiana": "keynesiano",
    "keynesianas": "keynesiano",
    "keynesianos": "keynesiano",
    "kg": "kilogramo",
    "khmeres": "khmer",
    "khmers": "khmer",
    "kianizaciones": "kianización",
    "kianizaciónes": "kianización",
    "kibbutces": "kibbutz",
    "kibbutcs": "kibbutz",
    "kibbutzes": "kibbutz",
    "kieselgures": "kieselgur",
    "kieseritas": "kieserita",
    "kiloamperios": "kiloamperio",
    "kilobases": "kilobase",
    "kilobaudios": "kilobaudio",
    "kilobites": "kilobit",
    "kilobits": "kilobit",
    "kilobytes": "kilobyte",
    "kilocalorías": "kilocaloría",
    "kilociclos": "kilociclo",
    "kilogramos": "kilogramo",
    "kilográmetros": "kilográmetro",
    "kilohercios": "kilohercio",
    "kilohertces": "kilohertz",
    "kilohertz": "kilohercio",
    "kilohertzios": "kilohertzio",
    "kilojoule": "kilojulio",
    "kilojoules": "kilojulio",
    "kilojulios": "kilojulio",
    "kilolitros": "kilolitro",
    "kilometra": "kilometrar",
    "kilometraba": "kilometrar",
    "kilometrabais": "kilometrar",
    "kilometraban": "kilometrar",
    "kilometrabas": "kilometrar",
    "kilometrad": "kilometrar",
    "kilometrada": "kilometrar",
    "kilometradas": "kilometrar",
    "kilometrado": "kilometrar",
    "kilometrados": "kilometrar",
    "kilometrajes": "kilometraje",
    "kilometramos": "kilometrar",
    "kilometran": "kilometrar",
    "kilometrando": "kilometrar",
    "kilometrara": "kilometrar",
    "kilometrarais": "kilometrar",
    "kilometraran": "kilometrar",
    "kilometraras": "kilometrar",
    "kilometrare": "kilometrar",
    "kilometrareis": "kilometrar",
    "kilometraremos": "kilometrar",
    "kilometraren": "kilometrar",
    "kilometrares": "kilometrar",
    "kilometraron": "kilometrar",
    "kilometrará": "kilometrar",
    "kilometrarán": "kilometrar",
    "kilometrarás": "kilometrar",
    "kilometraré": "kilometrar",
    "kilometraréis": "kilometrar",
    "kilometraría": "kilometrar",
    "kilometraríais": "kilometrar",
    "kilometraríamos": "kilometrar",
    "kilometrarían": "kilometrar",
    "kilometrarías": "kilometrar",
    "kilometras": "kilometrar",
    "kilometrase": "kilometrar",
    "kilometraseis": "kilometrar",
    "kilometrasen": "kilometrar",
    "kilometrases": "kilometrar",
    "kilometraste": "kilometrar",
    "kilometrasteis": "kilometrar",
    "kilometre": "kilometrar",
    "kilometremos": "kilometrar",
    "kilometren": "kilometrar",
    "kilometres": "kilometrar",
    "kilometro": "kilometrar",
    "kilometrábamos": "kilometrar",
    "kilometráis": "kilometrar",
    "kilometráramos": "kilometrar",
    "kilometráremos": "kilometrar",
    "kilometrásemos": "kilometrar",
    "kilometré": "kilometrar",
    "kilometréis": "kilometrar",
    "kilometró": "kilometrar",
    "kilométrica": "kilométrico",
    "kilométricas": "kilométrico",
    "kilométricos": "kilométrico",
    "kilonéwtones": "kilonewton",
    "kilooctetos": "kiloocteto",
    "kiloohmios": "kiloohmio",
    "kilopascales": "kilopascal",
    "kilopondes": "kilopond",
    "kilopondios": "kilopondio",
    "kiloponds": "kilopond",
    "kilopondímetros": "kilopondímetro",
    "kilos": "kilo",
    "kilotones": "kilotón",
    "kilovares": "kilovar",
    "kilovars": "kilovar",
    "kilovatios": "kilovatio",
    "kilovatioshora": "kilovatiohora",
    "kilovoltios": "kilovoltio",
    "kilowates": "kilowat",
    "kilowats": "kilowat",
    "kiltes": "kilt",
    "kilts": "kilt",
    "kilómetros": "kilómetro",
    "kimberlitas": "kimberlita",
    "kimonos": "kimono",
    "kinas": "kina",
    "kinescopios": "kinescopio",
    "kinetoscopios": "kinetoscopio",
    "kinésicas": "kinésica",
    "kioscos": "kiosco",
    "kiowas": "kiowa",
    "kipes": "kip",
    "kips": "kip",
    "kirieleisones": "kirieleisón",
    "kiries": "kirie",
    "kirschs": "kirsch",
    "kits": "kit",
    "kitschs": "kitsch",
    "kiwis": "kiwi",
    "kl": "kilolitro",
    "klistrones": "klistrón",
    "km": "kilómetro",
    "knock-outs": "knock-out",
    "knockouts": "knockout",
    "know-hows": "know-how",
    "koalas": "koala",
    "kobes": "kob",
    "kobs": "kob",
    "kohles": "kohl",
    "koinés": "koiné",
    "koljoses": "koljos",
    "koljoz": "koljos",
    "koljós": "koljos",
    "kores": "kore",
    "kornerupinas": "kornerupina",
    "krakes": "krak",
    "kraks": "krak",
    "krausismos": "krausismo",
    "krausistas": "krausista",
    "kriptones": "kriptón",
    "kronas": "krona",
    "kroneckeriana": "kroneckeriano",
    "kroneckerianas": "kroneckeriano",
    "kroneckerianos": "kroneckeriano",
    "kronprinces": "kronprinz",
    "kudúes": "kudú",
    "kudús": "kudú",
    "kulaks": "kulak",
    "kunas": "kuna",
    "kuncitas": "kuncita",
    "kundis": "kundi",
    "kunzitas": "kunzita",
    "kupones": "kupon",
    "kurchatovios": "kurchatovio",
    "kurda": "kurdo",
    "kurdas": "kurdo",
    "kurdos": "kurdo",
    "kuroi": "kuros",
    "kuwaitíes": "kuwaití",
    "kuwaitís": "kuwaití",
    "kuwaitíses": "kuwaití",
    "kwachas": "kwacha",
    "kwanzas": "kwanza",
    "kyanitas": "kyanita",
    "kyates": "kyat",
    "kyats": "kyat",
    "káiseres": "káiser",
    "káisers": "káiser",
    "kárates": "kárate",
    "kárstica": "kárstico",
    "kársticas": "kárstico",
    "kársticos": "kárstico",
    "kéfires": "kéfir",
    "kéfirs": "kéfir",
    "kílixes": "kílix",
    "kóljoses": "koljos",
    "l": "litro",
    "la": "lo",
    "laberintectomías": "laberintectomía",
    "laberintoplejías": "laberintoplejía",
    "laberintos": "laberinto",
    "laberíntica": "laberíntico",
    "laberínticas": "laberíntico",
    "laberínticos": "laberíntico",
    "labetaloles": "labetalol",
    "labiadas": "labiada",
    "labiales": "labial",
    "labialice": "labializar",
    "labialicemos": "labializar",
    "labialicen": "labializar",
    "labialices": "labializar",
    "labialicé": "labializar",
    "labialicéis": "labializar",
    "labializa": "labializar",
    "labializaba": "labializar",
    "labializabais": "labializar",
    "labializaban": "labializar",
    "labializabas": "labializar",
    "labializaciones": "labialización",
    "labializad": "labializar",
    "labializada": "labializar",
    "labializadas": "labializar",
    "labializado": "labializar",
    "labializados": "labializar",
    "labializamos": "labializar",
    "labializan": "labializar",
    "labializando": "labializar",
    "labializara": "labializar",
    "labializarais": "labializar",
    "labializaran": "labializar",
    "labializaras": "labializar",
    "labializare": "labializar",
    "labializareis": "labializar",
    "labializaremos": "labializar",
    "labializaren": "labializar",
    "labializares": "labializar",
    "labializaron": "labializar",
    "labializará": "labializar",
    "labializarán": "labializar",
    "labializarás": "labializar",
    "labializaré": "labializar",
    "labializaréis": "labializar",
    "labializaría": "labializar",
    "labializaríais": "labializar",
    "labializaríamos": "labializar",
    "labializarían": "labializar",
    "labializarías": "labializar",
    "labializas": "labializar",
    "labializase": "labializar",
    "labializaseis": "labializar",
    "labializasen": "labializar",
    "labializases": "labializar",
    "labializaste": "labializar",
    "labializasteis": "labializar",
    "labializo": "labializar",
    "labializábamos": "labializar",
    "labializáis": "labializar",
    "labializáramos": "labializar",
    "labializáremos": "labializar",
    "labializásemos": "labializar",
    "labializó": "labializar",
    "labias": "labia",
    "labihendida": "labihendido",
    "labihendidas": "labihendido",
    "labihendidos": "labihendido",
    "labilidades": "labilidad",
    "labiodentales": "labiodental",
    "labiolecturas": "labiolectura",
    "labiolinguales": "labiolingual",
    "labios": "labio",
    "labiovelares": "labiovelar",
    "labiovelarizaciones": "labiovelarización",
    "labioversiones": "labioversión",
    "labiumes": "labium",
    "labiérnagos": "labiérnago",
    "labora": "laborar",
    "laboraba": "laborar",
    "laborabais": "laborar",
    "laboraban": "laborar",
    "laborabas": "laborar",
    "laborables": "laborable",
    "laborad": "laborar",
    "laborada": "laborar",
    "laboradas": "laborar",
    "laborado": "laborar",
    "laborados": "laborar",
    "laborales": "laboral",
    "laboralistas": "laboralista",
    "laboramos": "laborar",
    "laboran": "laborar",
    "laborando": "laborar",
    "laborantes": "laborante",
    "laborara": "laborar",
    "laborarais": "laborar",
    "laboraran": "laborar",
    "laboraras": "laborar",
    "laborare": "laborar",
    "laborareis": "laborar",
    "laboraremos": "laborar",
    "laboraren": "laborar",
    "laborares": "laborar",
    "laboraron": "laborar",
    "laborará": "laborar",
    "laborarán": "laborar",
    "laborarás": "laborar",
    "laboraré": "laborar",
    "laboraréis": "laborar",
    "laboraría": "laborar",
    "laboraríais": "laborar",
    "laboraríamos": "laborar",
    "laborarían": "laborar",
    "laborarías": "laborar",
    "laboras": "laborar",
    "laborase": "laborar",
    "laboraseis": "laborar",
    "laborasen": "laborar",
    "laborases": "laborar",
    "laboraste": "laborar",
    "laborasteis": "laborar",
    "laboratorios": "laboratorio",
    "labore": "laborar",
    "laborea": "laborear",
    "laboreaba": "laborear",
    "laboreabais": "laborear",
    "laboreaban": "laborear",
    "laboreabas": "laborear",
    "laboread": "laborear",
    "laboreada": "laborear",
    "laboreadas": "laborear",
    "laboreado": "laborear",
    "laboreados": "laborear",
    "laboreamos": "laborear",
    "laborean": "laborear",
    "laboreando": "laborear",
    "laboreara": "laborear",
    "laborearais": "laborear",
    "laborearan": "laborear",
    "laborearas": "laborear",
    "laboreare": "laborear",
    "laboreareis": "laborear",
    "laborearemos": "laborear",
    "laborearen": "laborear",
    "laboreares": "laborear",
    "laborearon": "laborear",
    "laboreará": "laborear",
    "laborearán": "laborear",
    "laborearás": "laborear",
    "laborearé": "laborear",
    "laborearéis": "laborear",
    "laborearía": "laborear",
    "laborearíais": "laborear",
    "laborearíamos": "laborear",
    "laborearían": "laborear",
    "laborearías": "laborear",
    "laboreas": "laborear",
    "laborease": "laborear",
    "laboreaseis": "laborear",
    "laboreasen": "laborear",
    "laboreases": "laborear",
    "laboreaste": "laborear",
    "laboreasteis": "laborear",
    "laboree": "laborear",
    "laboreemos": "laborear",
    "laboreen": "laborear",
    "laborees": "laborear",
    "laboremos": "laborar",
    "laboren": "laborar",
    "laboreo": "laborear",
    "laboreos": "laboreo",
    "labores": "laborar",
    "laboreábamos": "laborear",
    "laboreáis": "laborear",
    "laboreáramos": "laborear",
    "laboreáremos": "laborear",
    "laboreásemos": "laborear",
    "laboreé": "laborear",
    "laboreéis": "laborear",
    "laboreó": "laborear",
    "laboriosa": "laborioso",
    "laboriosas": "laborioso",
    "laboriosidades": "laboriosidad",
    "laboriosos": "laborioso",
    "laborismos": "laborismo",
    "laboristas": "laborista",
    "laboro": "laborar",
    "labortanos": "labortano",
    "laborábamos": "laborar",
    "laboráis": "laborar",
    "laboráramos": "laborar",
    "laboráremos": "laborar",
    "laborásemos": "laborar",
    "laboré": "laborar",
    "laboréis": "laborar",
    "laboríos": "laborío",
    "laboró": "laborar",
    "labra": "labrar",
    "labraba": "labrar",
    "labrabais": "labrar",
    "labraban": "labrar",
    "labrabas": "labrar",
    "labrad": "labrar",
    "labrada": "labrar",
    "labradas": "labrar",
    "labradera": "labradero",
    "labraderas": "labradero",
    "labraderos": "labradero",
    "labrado": "labrar",
    "labradora": "labrador",
    "labradoras": "labrador",
    "labradores": "labrador",
    "labradoresca": "labradoresco",
    "labradorescas": "labradoresco",
    "labradorescencias": "labradorescencia",
    "labradorescos": "labradoresco",
    "labradoriles": "labradoril",
    "labradoritas": "labradorita",
    "labrados": "labrar",
    "labradía": "labradío",
    "labradías": "labradío",
    "labradíos": "labradío",
    "labramos": "labrar",
    "labran": "labrar",
    "labrando": "labrar",
    "labrantines": "labrantín",
    "labrantía": "labrantío",
    "labrantías": "labrantío",
    "labrantíos": "labrantío",
    "labranzas": "labranza",
    "labrara": "labrar",
    "labrarais": "labrar",
    "labraran": "labrar",
    "labraras": "labrar",
    "labrare": "labrar",
    "labrareis": "labrar",
    "labraremos": "labrar",
    "labraren": "labrar",
    "labrares": "labrar",
    "labraron": "labrar",
    "labrará": "labrar",
    "labrarán": "labrar",
    "labrarás": "labrar",
    "labraré": "labrar",
    "labraréis": "labrar",
    "labraría": "labrar",
    "labraríais": "labrar",
    "labraríamos": "labrar",
    "labrarían": "labrar",
    "labrarías": "labrar",
    "labras": "labrar",
    "labrase": "labrar",
    "labraseis": "labrar",
    "labrasen": "labrar",
    "labrases": "labrar",
    "labraste": "labrar",
    "labrasteis": "labrar",
    "labre": "labrar",
    "labremos": "labrar",
    "labren": "labrar",
    "labres": "labrar",
    "labriega": "labriego",
    "labriegas": "labriego",
    "labriegos": "labriego",
    "labro": "labrar",
    "labrocitos": "labrocito",
    "labros": "labro",
    "labruscas": "labrusca",
    "labrábamos": "labrar",
    "labráis": "labrar",
    "labráramos": "labrar",
    "labráremos": "labrar",
    "labrásemos": "labrar",
    "labré": "labrar",
    "labréis": "labrar",
    "labró": "labrar",
    "laburnos": "laburno",
    "lacada": "lacado",
    "lacadas": "lacado",
    "lacados": "lacado",
    "lacandona": "lacandón",
    "lacandonas": "lacandón",
    "lacandones": "lacandón",
    "lacas": "laca",
    "lacaya": "lacayo",
    "lacayas": "lacayo",
    "lacayos": "lacayo",
    "lacayuna": "lacayuno",
    "lacayunas": "lacayuno",
    "lacayunos": "lacayuno",
    "lace": "lazar",
    "lacea": "lacear",
    "laceaba": "lacear",
    "laceabais": "lacear",
    "laceaban": "lacear",
    "laceabas": "lacear",
    "lacead": "lacear",
    "laceada": "lacear",
    "laceadas": "lacear",
    "laceado": "lacear",
    "laceados": "lacear",
    "laceamos": "lacear",
    "lacean": "lacear",
    "laceando": "lacear",
    "laceara": "lacear",
    "lacearais": "lacear",
    "lacearan": "lacear",
    "lacearas": "lacear",
    "laceare": "lacear",
    "laceareis": "lacear",
    "lacearemos": "lacear",
    "lacearen": "lacear",
    "laceares": "lacear",
    "lacearon": "lacear",
    "laceará": "lacear",
    "lacearán": "lacear",
    "lacearás": "lacear",
    "lacearé": "lacear",
    "lacearéis": "lacear",
    "lacearía": "lacear",
    "lacearíais": "lacear",
    "lacearíamos": "lacear",
    "lacearían": "lacear",
    "lacearías": "lacear",
    "laceas": "lacear",
    "lacease": "lacear",
    "laceaseis": "lacear",
    "laceasen": "lacear",
    "laceases": "lacear",
    "laceaste": "lacear",
    "laceasteis": "lacear",
    "lacedemona": "lacedemón",
    "lacedemonas": "lacedemón",
    "lacedemones": "lacedemón",
    "lacedemonia": "lacedemonio",
    "lacedemonias": "lacedemonio",
    "lacedemonios": "lacedemonio",
    "lacee": "lacear",
    "laceemos": "lacear",
    "laceen": "lacear",
    "lacees": "lacear",
    "lacemos": "lazar",
    "lacen": "lazar",
    "lacenas": "lacena",
    "laceo": "lacear",
    "lacera": "lacerar",
    "laceraba": "lacerar",
    "lacerabais": "lacerar",
    "laceraban": "lacerar",
    "lacerabas": "lacerar",
    "laceraciones": "laceración",
    "lacerad": "lacerar",
    "lacerada": "lacerar",
    "laceradas": "lacerar",
    "lacerado": "lacerar",
    "lacerados": "lacerar",
    "laceramos": "lacerar",
    "laceran": "lacerar",
    "lacerando": "lacerar",
    "lacerara": "lacerar",
    "lacerarais": "lacerar",
    "laceraran": "lacerar",
    "laceraras": "lacerar",
    "lacerare": "lacerar",
    "lacerareis": "lacerar",
    "laceraremos": "lacerar",
    "laceraren": "lacerar",
    "lacerares": "lacerar",
    "laceraron": "lacerar",
    "lacerará": "lacerar",
    "lacerarán": "lacerar",
    "lacerarás": "lacerar",
    "laceraré": "lacerar",
    "laceraréis": "lacerar",
    "laceraría": "lacerar",
    "laceraríais": "lacerar",
    "laceraríamos": "lacerar",
    "lacerarían": "lacerar",
    "lacerarías": "lacerar",
    "laceras": "lacerar",
    "lacerase": "lacerar",
    "laceraseis": "lacerar",
    "lacerasen": "lacerar",
    "lacerases": "lacerar",
    "laceraste": "lacerar",
    "lacerasteis": "lacerar",
    "lacere": "lacerar",
    "laceremos": "lacerar",
    "laceren": "lacerar",
    "laceres": "lacerar",
    "lacerias": "laceria",
    "laceriosa": "lacerioso",
    "laceriosas": "lacerioso",
    "laceriosos": "lacerioso",
    "lacero": "lacerar",
    "laceros": "lacero",
    "lacertosa": "lacertoso",
    "lacertosas": "lacertoso",
    "lacertosos": "lacertoso",
    "lacerábamos": "lacerar",
    "laceráis": "lacerar",
    "laceráramos": "lacerar",
    "laceráremos": "lacerar",
    "lacerásemos": "lacerar",
    "laceré": "lacerar",
    "laceréis": "lacerar",
    "lacerías": "lacería",
    "laceró": "lacerar",
    "laces": "lazar",
    "lacetana": "lacetano",
    "lacetanas": "lacetano",
    "lacetanos": "lacetano",
    "laceábamos": "lacear",
    "laceáis": "lacear",
    "laceáramos": "lacear",
    "laceáremos": "lacear",
    "laceásemos": "lacear",
    "laceé": "lacear",
    "laceéis": "lacear",
    "laceó": "lacear",
    "lachas": "lacha",
    "lacia": "lacio",
    "lacias": "lacio",
    "laciniada": "laciniado",
    "laciniadas": "laciniado",
    "laciniados": "laciniado",
    "lacinias": "lacinia",
    "lacios": "lacio",
    "lacmoides": "lacmoide",
    "lacolitos": "lacolito",
    "lacones": "lacón",
    "laconia": "laconio",
    "laconias": "laconio",
    "laconios": "laconio",
    "laconismos": "laconismo",
    "lacra": "lacrar",
    "lacraba": "lacrar",
    "lacrabais": "lacrar",
    "lacraban": "lacrar",
    "lacrabas": "lacrar",
    "lacrad": "lacrar",
    "lacrada": "lacrar",
    "lacradas": "lacrar",
    "lacrado": "lacrar",
    "lacrados": "lacrar",
    "lacramos": "lacrar",
    "lacran": "lacrar",
    "lacrando": "lacrar",
    "lacrara": "lacrar",
    "lacrarais": "lacrar",
    "lacraran": "lacrar",
    "lacraras": "lacrar",
    "lacrare": "lacrar",
    "lacrareis": "lacrar",
    "lacraremos": "lacrar",
    "lacraren": "lacrar",
    "lacrares": "lacrar",
    "lacraron": "lacrar",
    "lacrará": "lacrar",
    "lacrarán": "lacrar",
    "lacrarás": "lacrar",
    "lacraré": "lacrar",
    "lacraréis": "lacrar",
    "lacraría": "lacrar",
    "lacraríais": "lacrar",
    "lacraríamos": "lacrar",
    "lacrarían": "lacrar",
    "lacrarías": "lacrar",
    "lacras": "lacrar",
    "lacrase": "lacrar",
    "lacraseis": "lacrar",
    "lacrasen": "lacrar",
    "lacrases": "lacrar",
    "lacraste": "lacrar",
    "lacrasteis": "lacrar",
    "lacre": "lacrar",
    "lacremos": "lacrar",
    "lacren": "lacrar",
    "lacres": "lacre",
    "lacrimaciones": "lacrimación",
    "lacrimales": "lacrimal",
    "lacrimomaxilares": "lacrimomaxilar",
    "lacrimonasales": "lacrimonasal",
    "lacrimosa": "lacrimoso",
    "lacrimosas": "lacrimoso",
    "lacrimosos": "lacrimoso",
    "lacrimotomías": "lacrimotomía",
    "lacrimógena": "lacrimógeno",
    "lacrimógenas": "lacrimógeno",
    "lacrimógenos": "lacrimógeno",
    "lacro": "lacrar",
    "lacrábamos": "lacrar",
    "lacráis": "lacrar",
    "lacráramos": "lacrar",
    "lacráremos": "lacrar",
    "lacrásemos": "lacrar",
    "lacré": "lacrar",
    "lacréis": "lacrar",
    "lacró": "lacrar",
    "lacta": "lactar",
    "lactaba": "lactar",
    "lactabais": "lactar",
    "lactaban": "lactar",
    "lactabas": "lactar",
    "lactaciones": "lactación",
    "lactad": "lactar",
    "lactada": "lactar",
    "lactadas": "lactar",
    "lactado": "lactar",
    "lactados": "lactar",
    "lactalbúminas": "lactalbúmina",
    "lactamas": "lactama",
    "lactamos": "lactar",
    "lactan": "lactar",
    "lactancias": "lactancia",
    "lactando": "lactar",
    "lactantes": "lactante",
    "lactara": "lactar",
    "lactarais": "lactar",
    "lactaran": "lactar",
    "lactaras": "lactar",
    "lactare": "lactar",
    "lactareis": "lactar",
    "lactaremos": "lactar",
    "lactaren": "lactar",
    "lactares": "lactar",
    "lactaria": "lactario",
    "lactarias": "lactario",
    "lactarios": "lactario",
    "lactaron": "lactar",
    "lactará": "lactar",
    "lactarán": "lactar",
    "lactarás": "lactar",
    "lactaré": "lactar",
    "lactaréis": "lactar",
    "lactaría": "lactar",
    "lactaríais": "lactar",
    "lactaríamos": "lactar",
    "lactarían": "lactar",
    "lactarías": "lactar",
    "lactas": "lactar",
    "lactasas": "lactasa",
    "lactase": "lactar",
    "lactaseis": "lactar",
    "lactasen": "lactar",
    "lactases": "lactar",
    "lactaste": "lactar",
    "lactasteis": "lactar",
    "lactatos": "lactato",
    "lacte": "lactar",
    "lacteada": "lacteado",
    "lacteadas": "lacteado",
    "lacteados": "lacteado",
    "lactemos": "lactar",
    "lacten": "lactar",
    "lactes": "lactar",
    "lactescencias": "lactescencia",
    "lactescentes": "lactescente",
    "lacticinios": "lacticinio",
    "lactinas": "lactina",
    "lactinosa": "lactinoso",
    "lactinosas": "lactinoso",
    "lactinosos": "lactinoso",
    "lacto": "lactar",
    "lactoalbúminas": "lactoalbúmina",
    "lactobacilos": "lactobacilo",
    "lactobutirómetros": "lactobutirómetro",
    "lactoductos": "lactoducto",
    "lactofeninas": "lactofenina",
    "lactoferrinas": "lactoferrina",
    "lactoflavinas": "lactoflavina",
    "lactoglobulinas": "lactoglobulina",
    "lactoles": "lactol",
    "lactonas": "lactona",
    "lactonitrilos": "lactonitrilo",
    "lactonizaciones": "lactonización",
    "lactoproteínas": "lactoproteína",
    "lactosas": "lactosa",
    "lactucarios": "lactucario",
    "lactábamos": "lactar",
    "lactáis": "lactar",
    "lactáramos": "lactar",
    "lactáremos": "lactar",
    "lactásemos": "lactar",
    "lacté": "lactar",
    "lactéis": "lactar",
    "lactífera": "lactífero",
    "lactíferas": "lactífero",
    "lactíferos": "lactífero",
    "lactínea": "lactíneo",
    "lactíneas": "lactíneo",
    "lactíneos": "lactíneo",
    "lactó": "lactar",
    "lactómetros": "lactómetro",
    "lactúmenes": "lactumen",
    "lacunares": "lacunar",
    "lacunarios": "lacunario",
    "lacustres": "lacustre",
    "lacé": "lazar",
    "lacéis": "lazar",
    "lacónica": "lacónico",
    "lacónicas": "lacónico",
    "lacónicos": "lacónico",
    "ladas": "lada",
    "ladea": "ladear",
    "ladeaba": "ladear",
    "ladeabais": "ladear",
    "ladeaban": "ladear",
    "ladeabas": "ladear",
    "ladead": "ladear",
    "ladeada": "ladear",
    "ladeadas": "ladear",
    "ladeado": "ladear",
    "ladeados": "ladear",
    "ladeamos": "ladear",
    "ladean": "ladear",
    "ladeando": "ladear",
    "ladeara": "ladear",
    "ladearais": "ladear",
    "ladearan": "ladear",
    "ladearas": "ladear",
    "ladeare": "ladear",
    "ladeareis": "ladear",
    "ladearemos": "ladear",
    "ladearen": "ladear",
    "ladeares": "ladear",
    "ladearmientos": "ladearmiento",
    "ladearon": "ladear",
    "ladeará": "ladear",
    "ladearán": "ladear",
    "ladearás": "ladear",
    "ladearé": "ladear",
    "ladearéis": "ladear",
    "ladearía": "ladear",
    "ladearíais": "ladear",
    "ladearíamos": "ladear",
    "ladearían": "ladear",
    "ladearías": "ladear",
    "ladeas": "ladear",
    "ladease": "ladear",
    "ladeaseis": "ladear",
    "ladeasen": "ladear",
    "ladeases": "ladear",
    "ladeaste": "ladear",
    "ladeasteis": "ladear",
    "ladee": "ladear",
    "ladeemos": "ladear",
    "ladeen": "ladear",
    "ladees": "ladear",
    "ladeo": "ladear",
    "ladeos": "ladeo",
    "laderas": "ladera",
    "laderías": "ladería",
    "ladeábamos": "ladear",
    "ladeáis": "ladear",
    "ladeáramos": "ladear",
    "ladeáremos": "ladear",
    "ladeásemos": "ladear",
    "ladeé": "ladear",
    "ladeéis": "ladear",
    "ladeó": "ladear",
    "ladies": "lady",
    "ladillas": "ladilla",
    "ladillos": "ladillo",
    "ladina": "ladino",
    "ladinas": "ladino",
    "ladinos": "ladino",
    "lados": "lado",
    "ladra": "ladrar",
    "ladraba": "ladrar",
    "ladrabais": "ladrar",
    "ladraban": "ladrar",
    "ladrabas": "ladrar",
    "ladrad": "ladrar",
    "ladrada": "ladrar",
    "ladradas": "ladrar",
    "ladrado": "ladrar",
    "ladrados": "ladrar",
    "ladramos": "ladrar",
    "ladran": "ladrar",
    "ladrando": "ladrar",
    "ladrara": "ladrar",
    "ladrarais": "ladrar",
    "ladraran": "ladrar",
    "ladraras": "ladrar",
    "ladrare": "ladrar",
    "ladrareis": "ladrar",
    "ladraremos": "ladrar",
    "ladraren": "ladrar",
    "ladrares": "ladrar",
    "ladraron": "ladrar",
    "ladrará": "ladrar",
    "ladrarán": "ladrar",
    "ladrarás": "ladrar",
    "ladraré": "ladrar",
    "ladraréis": "ladrar",
    "ladraría": "ladrar",
    "ladraríais": "ladrar",
    "ladraríamos": "ladrar",
    "ladrarían": "ladrar",
    "ladrarías": "ladrar",
    "ladras": "ladrar",
    "ladrase": "ladrar",
    "ladraseis": "ladrar",
    "ladrasen": "ladrar",
    "ladrases": "ladrar",
    "ladraste": "ladrar",
    "ladrasteis": "ladrar",
    "ladre": "ladrar",
    "ladremos": "ladrar",
    "ladren": "ladrar",
    "ladres": "ladrar",
    "ladridos": "ladrido",
    "ladrilla": "ladrillar",
    "ladrillaba": "ladrillar",
    "ladrillabais": "ladrillar",
    "ladrillaban": "ladrillar",
    "ladrillabas": "ladrillar",
    "ladrillad": "ladrillar",
    "ladrillada": "ladrillar",
    "ladrilladas": "ladrillar",
    "ladrillado": "ladrillar",
    "ladrillados": "ladrillar",
    "ladrillamos": "ladrillar",
    "ladrillan": "ladrillar",
    "ladrillando": "ladrillar",
    "ladrillara": "ladrillar",
    "ladrillarais": "ladrillar",
    "ladrillaran": "ladrillar",
    "ladrillaras": "ladrillar",
    "ladrillare": "ladrillar",
    "ladrillareis": "ladrillar",
    "ladrillaremos": "ladrillar",
    "ladrillaren": "ladrillar",
    "ladrillares": "ladrillar",
    "ladrillaron": "ladrillar",
    "ladrillará": "ladrillar",
    "ladrillarán": "ladrillar",
    "ladrillarás": "ladrillar",
    "ladrillaré": "ladrillar",
    "ladrillaréis": "ladrillar",
    "ladrillaría": "ladrillar",
    "ladrillaríais": "ladrillar",
    "ladrillaríamos": "ladrillar",
    "ladrillarían": "ladrillar",
    "ladrillarías": "ladrillar",
    "ladrillas": "ladrillar",
    "ladrillase": "ladrillar",
    "ladrillaseis": "ladrillar",
    "ladrillasen": "ladrillar",
    "ladrillases": "ladrillar",
    "ladrillaste": "ladrillar",
    "ladrillasteis": "ladrillar",
    "ladrillazos": "ladrillazo",
    "ladrille": "ladrillar",
    "ladrillemos": "ladrillar",
    "ladrillen": "ladrillar",
    "ladrillera": "ladrillero",
    "ladrilleras": "ladrillero",
    "ladrilleros": "ladrillero",
    "ladrilles": "ladrillar",
    "ladrillo": "ladrillar",
    "ladrillos": "ladrillo",
    "ladrillosa": "ladrilloso",
    "ladrillosas": "ladrilloso",
    "ladrillosos": "ladrilloso",
    "ladrillábamos": "ladrillar",
    "ladrilláis": "ladrillar",
    "ladrilláramos": "ladrillar",
    "ladrilláremos": "ladrillar",
    "ladrillásemos": "ladrillar",
    "ladrillé": "ladrillar",
    "ladrilléis": "ladrillar",
    "ladrilló": "ladrillar",
    "ladro": "ladrar",
    "ladrona": "ladrón",
    "ladronas": "ladrón",
    "ladronea": "ladronear",
    "ladroneaba": "ladronear",
    "ladroneabais": "ladronear",
    "ladroneaban": "ladronear",
    "ladroneabas": "ladronear",
    "ladronead": "ladronear",
    "ladroneada": "ladronear",
    "ladroneadas": "ladronear",
    "ladroneado": "ladronear",
    "ladroneados": "ladronear",
    "ladroneamos": "ladronear",
    "ladronean": "ladronear",
    "ladroneando": "ladronear",
    "ladroneara": "ladronear",
    "ladronearais": "ladronear",
    "ladronearan": "ladronear",
    "ladronearas": "ladronear",
    "ladroneare": "ladronear",
    "ladroneareis": "ladronear",
    "ladronearemos": "ladronear",
    "ladronearen": "ladronear",
    "ladroneares": "ladronear",
    "ladronearon": "ladronear",
    "ladroneará": "ladronear",
    "ladronearán": "ladronear",
    "ladronearás": "ladronear",
    "ladronearé": "ladronear",
    "ladronearéis": "ladronear",
    "ladronearía": "ladronear",
    "ladronearíais": "ladronear",
    "ladronearíamos": "ladronear",
    "ladronearían": "ladronear",
    "ladronearías": "ladronear",
    "ladroneas": "ladronear",
    "ladronease": "ladronear",
    "ladroneaseis": "ladronear",
    "ladroneasen": "ladronear",
    "ladroneases": "ladronear",
    "ladroneaste": "ladronear",
    "ladroneasteis": "ladronear",
    "ladronee": "ladronear",
    "ladroneemos": "ladronear",
    "ladroneen": "ladronear",
    "ladronees": "ladronear",
    "ladroneo": "ladronear",
    "ladroneras": "ladronera",
    "ladronerías": "ladronería",
    "ladrones": "ladrón",
    "ladronesca": "ladronesco",
    "ladronescas": "ladronesco",
    "ladronescos": "ladronesco",
    "ladroneábamos": "ladronear",
    "ladroneáis": "ladronear",
    "ladroneáramos": "ladronear",
    "ladroneáremos": "ladronear",
    "ladroneásemos": "ladronear",
    "ladroneé": "ladronear",
    "ladroneéis": "ladronear",
    "ladroneó": "ladronear",
    "ladronicios": "ladronicio",
    "ladronzuela": "ladronzuelo",
    "ladronzuelas": "ladronzuelo",
    "ladronzuelos": "ladronzuelo",
    "ladrábamos": "ladrar",
    "ladráis": "ladrar",
    "ladráramos": "ladrar",
    "ladráremos": "ladrar",
    "ladrásemos": "ladrar",
    "ladré": "ladrar",
    "ladréis": "ladrar",
    "ladró": "ladrar",
    "lagarera": "lagarero",
    "lagareras": "lagarero",
    "lagareros": "lagarero",
    "lagares": "lagar",
    "lagaretas": "lagareta",
    "lagarta": "lagarto",
    "lagartas": "lagarto",
    "lagartea": "lagartear",
    "lagarteaba": "lagartear",
    "lagarteabais": "lagartear",
    "lagarteaban": "lagartear",
    "lagarteabas": "lagartear",
    "lagartead": "lagartear",
    "lagarteada": "lagartear",
    "lagarteadas": "lagartear",
    "lagarteado": "lagartear",
    "lagarteados": "lagartear",
    "lagarteamos": "lagartear",
    "lagartean": "lagartear",
    "lagarteando": "lagartear",
    "lagarteara": "lagartear",
    "lagartearais": "lagartear",
    "lagartearan": "lagartear",
    "lagartearas": "lagartear",
    "lagarteare": "lagartear",
    "lagarteareis": "lagartear",
    "lagartearemos": "lagartear",
    "lagartearen": "lagartear",
    "lagarteares": "lagartear",
    "lagartearon": "lagartear",
    "lagarteará": "lagartear",
    "lagartearán": "lagartear",
    "lagartearás": "lagartear",
    "lagartearé": "lagartear",
    "lagartearéis": "lagartear",
    "lagartearía": "lagartear",
    "lagartearíais": "lagartear",
    "lagartearíamos": "lagartear",
    "lagartearían": "lagartear",
    "lagartearías": "lagartear",
    "lagarteas": "lagartear",
    "lagartease": "lagartear",
    "lagarteaseis": "lagartear",
    "lagarteasen": "lagartear",
    "lagarteases": "lagartear",
    "lagarteaste": "lagartear",
    "lagarteasteis": "lagartear",
    "lagartee": "lagartear",
    "lagarteemos": "lagartear",
    "lagarteen": "lagartear",
    "lagartees": "lagartear",
    "lagarteo": "lagartear",
    "lagartera": "lagartero",
    "lagarterana": "lagarterano",
    "lagarteranas": "lagarterano",
    "lagarteranos": "lagarterano",
    "lagarteras": "lagartero",
    "lagarteros": "lagartero",
    "lagarteábamos": "lagartear",
    "lagarteáis": "lagartear",
    "lagarteáramos": "lagartear",
    "lagarteáremos": "lagartear",
    "lagarteásemos": "lagartear",
    "lagarteé": "lagartear",
    "lagarteéis": "lagartear",
    "lagarteó": "lagartear",
    "lagartijas": "lagartija",
    "lagartijera": "lagartijero",
    "lagartijeras": "lagartijero",
    "lagartijeros": "lagartijero",
    "lagartona": "lagartón",
    "lagartonas": "lagartón",
    "lagartones": "lagartón",
    "lagartos": "lagarto",
    "lagañas": "lagaña",
    "lagañosa": "lagañoso",
    "lagañosas": "lagañoso",
    "lagañosos": "lagañoso",
    "lagoftalmías": "lagoftalmía",
    "lagoftálmica": "lagoftálmico",
    "lagoftálmicas": "lagoftálmico",
    "lagoftálmicos": "lagoftálmico",
    "lagos": "lago",
    "lagotea": "lagotear",
    "lagoteaba": "lagotear",
    "lagoteabais": "lagotear",
    "lagoteaban": "lagotear",
    "lagoteabas": "lagotear",
    "lagotead": "lagotear",
    "lagoteada": "lagotear",
    "lagoteadas": "lagotear",
    "lagoteado": "lagotear",
    "lagoteados": "lagotear",
    "lagoteamos": "lagotear",
    "lagotean": "lagotear",
    "lagoteando": "lagotear",
    "lagoteara": "lagotear",
    "lagotearais": "lagotear",
    "lagotearan": "lagotear",
    "lagotearas": "lagotear",
    "lagoteare": "lagotear",
    "lagoteareis": "lagotear",
    "lagotearemos": "lagotear",
    "lagotearen": "lagotear",
    "lagoteares": "lagotear",
    "lagotearon": "lagotear",
    "lagoteará": "lagotear",
    "lagotearán": "lagotear",
    "lagotearás": "lagotear",
    "lagotearé": "lagotear",
    "lagotearéis": "lagotear",
    "lagotearía": "lagotear",
    "lagotearíais": "lagotear",
    "lagotearíamos": "lagotear",
    "lagotearían": "lagotear",
    "lagotearías": "lagotear",
    "lagoteas": "lagotear",
    "lagotease": "lagotear",
    "lagoteaseis": "lagotear",
    "lagoteasen": "lagotear",
    "lagoteases": "lagotear",
    "lagoteaste": "lagotear",
    "lagoteasteis": "lagotear",
    "lagotee": "lagotear",
    "lagoteemos": "lagotear",
    "lagoteen": "lagotear",
    "lagotees": "lagotear",
    "lagoteo": "lagotear",
    "lagotera": "lagotero",
    "lagoteras": "lagotero",
    "lagoteros": "lagotero",
    "lagoterías": "lagotería",
    "lagoteábamos": "lagotear",
    "lagoteáis": "lagotear",
    "lagoteáramos": "lagotear",
    "lagoteáremos": "lagotear",
    "lagoteásemos": "lagotear",
    "lagoteé": "lagotear",
    "lagoteéis": "lagotear",
    "lagoteó": "lagotear",
    "lagrima": "lagrimar",
    "lagrimaba": "lagrimar",
    "lagrimabais": "lagrimar",
    "lagrimaban": "lagrimar",
    "lagrimabas": "lagrimar",
    "lagrimad": "lagrimar",
    "lagrimada": "lagrimar",
    "lagrimadas": "lagrimar",
    "lagrimado": "lagrimar",
    "lagrimados": "lagrimar",
    "lagrimales": "lagrimal",
    "lagrimamos": "lagrimar",
    "lagriman": "lagrimar",
    "lagrimando": "lagrimar",
    "lagrimara": "lagrimar",
    "lagrimarais": "lagrimar",
    "lagrimaran": "lagrimar",
    "lagrimaras": "lagrimar",
    "lagrimare": "lagrimar",
    "lagrimareis": "lagrimar",
    "lagrimaremos": "lagrimar",
    "lagrimaren": "lagrimar",
    "lagrimares": "lagrimar",
    "lagrimaron": "lagrimar",
    "lagrimará": "lagrimar",
    "lagrimarán": "lagrimar",
    "lagrimarás": "lagrimar",
    "lagrimaré": "lagrimar",
    "lagrimaréis": "lagrimar",
    "lagrimaría": "lagrimar",
    "lagrimaríais": "lagrimar",
    "lagrimaríamos": "lagrimar",
    "lagrimarían": "lagrimar",
    "lagrimarías": "lagrimar",
    "lagrimas": "lagrimar",
    "lagrimase": "lagrimar",
    "lagrimaseis": "lagrimar",
    "lagrimasen": "lagrimar",
    "lagrimases": "lagrimar",
    "lagrimaste": "lagrimar",
    "lagrimasteis": "lagrimar",
    "lagrime": "lagrimar",
    "lagrimea": "lagrimear",
    "lagrimeaba": "lagrimear",
    "lagrimeabais": "lagrimear",
    "lagrimeaban": "lagrimear",
    "lagrimeabas": "lagrimear",
    "lagrimead": "lagrimear",
    "lagrimeada": "lagrimear",
    "lagrimeadas": "lagrimear",
    "lagrimeado": "lagrimear",
    "lagrimeados": "lagrimear",
    "lagrimeamos": "lagrimear",
    "lagrimean": "lagrimear",
    "lagrimeando": "lagrimear",
    "lagrimeara": "lagrimear",
    "lagrimearais": "lagrimear",
    "lagrimearan": "lagrimear",
    "lagrimearas": "lagrimear",
    "lagrimeare": "lagrimear",
    "lagrimeareis": "lagrimear",
    "lagrimearemos": "lagrimear",
    "lagrimearen": "lagrimear",
    "lagrimeares": "lagrimear",
    "lagrimearon": "lagrimear",
    "lagrimeará": "lagrimear",
    "lagrimearán": "lagrimear",
    "lagrimearás": "lagrimear",
    "lagrimearé": "lagrimear",
    "lagrimearéis": "lagrimear",
    "lagrimearía": "lagrimear",
    "lagrimearíais": "lagrimear",
    "lagrimearíamos": "lagrimear",
    "lagrimearían": "lagrimear",
    "lagrimearías": "lagrimear",
    "lagrimeas": "lagrimear",
    "lagrimease": "lagrimear",
    "lagrimeaseis": "lagrimear",
    "lagrimeasen": "lagrimear",
    "lagrimeases": "lagrimear",
    "lagrimeaste": "lagrimear",
    "lagrimeasteis": "lagrimear",
    "lagrimee": "lagrimear",
    "lagrimeemos": "lagrimear",
    "lagrimeen": "lagrimear",
    "lagrimees": "lagrimear",
    "lagrimemos": "lagrimar",
    "lagrimen": "lagrimar",
    "lagrimeo": "lagrimear",
    "lagrimeos": "lagrimeo",
    "lagrimes": "lagrimar",
    "lagrimeábamos": "lagrimear",
    "lagrimeáis": "lagrimear",
    "lagrimeáramos": "lagrimear",
    "lagrimeáremos": "lagrimear",
    "lagrimeásemos": "lagrimear",
    "lagrimeé": "lagrimear",
    "lagrimeéis": "lagrimear",
    "lagrimeó": "lagrimear",
    "lagrimitas": "lagrimita",
    "lagrimo": "lagrimar",
    "lagrimosa": "lagrimoso",
    "lagrimosas": "lagrimoso",
    "lagrimosos": "lagrimoso",
    "lagrimábamos": "lagrimar",
    "lagrimáis": "lagrimar",
    "lagrimáramos": "lagrimar",
    "lagrimáremos": "lagrimar",
    "lagrimásemos": "lagrimar",
    "lagrimé": "lagrimar",
    "lagriméis": "lagrimar",
    "lagrimó": "lagrimar",
    "lagunajos": "lagunajo",
    "lagunares": "lagunar",
    "lagunas": "laguna",
    "lagunera": "lagunero",
    "laguneras": "lagunero",
    "laguneros": "lagunero",
    "lagunosa": "lagunoso",
    "lagunosas": "lagunoso",
    "lagunosos": "lagunoso",
    "lagópodos": "lagópodo",
    "laica": "laico",
    "laicados": "laicado",
    "laicales": "laical",
    "laicas": "laico",
    "laicice": "laicizar",
    "laicicemos": "laicizar",
    "laicicen": "laicizar",
    "laicices": "laicizar",
    "laicicé": "laicizar",
    "laicicéis": "laicizar",
    "laicismos": "laicismo",
    "laicistas": "laicista",
    "laiciza": "laicizar",
    "laicizaba": "laicizar",
    "laicizabais": "laicizar",
    "laicizaban": "laicizar",
    "laicizabas": "laicizar",
    "laicizaciones": "laicización",
    "laicizad": "laicizar",
    "laicizada": "laicizar",
    "laicizadas": "laicizar",
    "laicizado": "laicizar",
    "laicizados": "laicizar",
    "laicizamos": "laicizar",
    "laicizan": "laicizar",
    "laicizando": "laicizar",
    "laicizara": "laicizar",
    "laicizarais": "laicizar",
    "laicizaran": "laicizar",
    "laicizaras": "laicizar",
    "laicizare": "laicizar",
    "laicizareis": "laicizar",
    "laicizaremos": "laicizar",
    "laicizaren": "laicizar",
    "laicizares": "laicizar",
    "laicizaron": "laicizar",
    "laicizará": "laicizar",
    "laicizarán": "laicizar",
    "laicizarás": "laicizar",
    "laicizaré": "laicizar",
    "laicizaréis": "laicizar",
    "laicizaría": "laicizar",
    "laicizaríais": "laicizar",
    "laicizaríamos": "laicizar",
    "laicizarían": "laicizar",
    "laicizarías": "laicizar",
    "laicizas": "laicizar",
    "laicizase": "laicizar",
    "laicizaseis": "laicizar",
    "laicizasen": "laicizar",
    "laicizases": "laicizar",
    "laicizaste": "laicizar",
    "laicizasteis": "laicizar",
    "laicizo": "laicizar",
    "laicizábamos": "laicizar",
    "laicizáis": "laicizar",
    "laicizáramos": "laicizar",
    "laicizáremos": "laicizar",
    "laicizásemos": "laicizar",
    "laicizó": "laicizar",
    "laicos": "laico",
    "lairenes": "lairén",
    "lais": "lai",
    "lajas": "laja",
    "lama": "lamer",
    "lamamos": "lamer",
    "laman": "lamer",
    "lamarckistas": "lamarckista",
    "lamarquismos": "lamarquismo",
    "lamarquistas": "lamarquista",
    "lamas": "lamer",
    "lamaísmos": "lamaísmo",
    "lamaístas": "lamaísta",
    "lamba": "lamber",
    "lambamos": "lamber",
    "lamban": "lamber",
    "lambas": "lamber",
    "lambdas": "lambda",
    "lambdoidea": "lambdoideo",
    "lambdoideas": "lambdoideo",
    "lambdoideos": "lambdoideo",
    "lambe": "lamber",
    "lambed": "lamber",
    "lambeles": "lambel",
    "lambelibranquia": "lambelibranquio",
    "lambelibranquias": "lambelibranquio",
    "lambelibranquios": "lambelibranquio",
    "lambemos": "lamber",
    "lamben": "lamber",
    "lambeos": "lambeo",
    "lamberemos": "lamber",
    "lamberá": "lamber",
    "lamberán": "lamber",
    "lamberás": "lamber",
    "lamberé": "lamber",
    "lamberéis": "lamber",
    "lambería": "lamber",
    "lamberíais": "lamber",
    "lamberíamos": "lamber",
    "lamberían": "lamber",
    "lamberías": "lamber",
    "lambes": "lamber",
    "lambetas": "lambeta",
    "lambetazos": "lambetazo",
    "lambida": "lamber",
    "lambidas": "lamber",
    "lambido": "lamber",
    "lambidos": "lamber",
    "lambiendo": "lamber",
    "lambiera": "lamber",
    "lambierais": "lamber",
    "lambieran": "lamber",
    "lambieras": "lamber",
    "lambiere": "lamber",
    "lambiereis": "lamber",
    "lambieren": "lamber",
    "lambieres": "lamber",
    "lambieron": "lamber",
    "lambiese": "lamber",
    "lambieseis": "lamber",
    "lambiesen": "lamber",
    "lambieses": "lamber",
    "lambimos": "lamber",
    "lambiste": "lamber",
    "lambisteis": "lamber",
    "lambiéramos": "lamber",
    "lambiéremos": "lamber",
    "lambiésemos": "lamber",
    "lambió": "lamber",
    "lambo": "lamber",
    "lambrequines": "lambrequín",
    "lambrijas": "lambrija",
    "lambrucia": "lambrucio",
    "lambrucias": "lambrucio",
    "lambrucios": "lambrucio",
    "lambáis": "lamber",
    "lambéis": "lamber",
    "lambí": "lamber",
    "lambía": "lamber",
    "lambíais": "lamber",
    "lambíamos": "lamber",
    "lambían": "lamber",
    "lambías": "lamber",
    "lame": "lamer",
    "lamed": "lamer",
    "lamedales": "lamedal",
    "lamedora": "lamedor",
    "lamedoras": "lamedor",
    "lamedores": "lamedor",
    "lameduras": "lamedura",
    "lamelares": "lamelar",
    "lamelibranquia": "lamelibranquio",
    "lamelibranquias": "lamelibranquio",
    "lamelibranquios": "lamelibranquio",
    "lamelipodios": "lamelipodio",
    "lamemos": "lamer",
    "lamen": "lamer",
    "lamenta": "lamentar",
    "lamentaba": "lamentar",
    "lamentabais": "lamentar",
    "lamentaban": "lamentar",
    "lamentabas": "lamentar",
    "lamentables": "lamentable",
    "lamentaciones": "lamentación",
    "lamentad": "lamentar",
    "lamentada": "lamentar",
    "lamentadas": "lamentar",
    "lamentado": "lamentar",
    "lamentados": "lamentar",
    "lamentamos": "lamentar",
    "lamentan": "lamentar",
    "lamentando": "lamentar",
    "lamentara": "lamentar",
    "lamentarais": "lamentar",
    "lamentaran": "lamentar",
    "lamentaras": "lamentar",
    "lamentare": "lamentar",
    "lamentareis": "lamentar",
    "lamentaremos": "lamentar",
    "lamentaren": "lamentar",
    "lamentares": "lamentar",
    "lamentaron": "lamentar",
    "lamentará": "lamentar",
    "lamentarán": "lamentar",
    "lamentarás": "lamentar",
    "lamentaré": "lamentar",
    "lamentaréis": "lamentar",
    "lamentaría": "lamentar",
    "lamentaríais": "lamentar",
    "lamentaríamos": "lamentar",
    "lamentarían": "lamentar",
    "lamentarías": "lamentar",
    "lamentas": "lamentar",
    "lamentase": "lamentar",
    "lamentaseis": "lamentar",
    "lamentasen": "lamentar",
    "lamentases": "lamentar",
    "lamentaste": "lamentar",
    "lamentasteis": "lamentar",
    "lamente": "lamentar",
    "lamentemos": "lamentar",
    "lamenten": "lamentar",
    "lamentes": "lamentar",
    "lamento": "lamentar",
    "lamentos": "lamento",
    "lamentosa": "lamentoso",
    "lamentosas": "lamentoso",
    "lamentosos": "lamentoso",
    "lamentábamos": "lamentar",
    "lamentáis": "lamentar",
    "lamentáramos": "lamentar",
    "lamentáremos": "lamentar",
    "lamentásemos": "lamentar",
    "lamenté": "lamentar",
    "lamentéis": "lamentar",
    "lamentó": "lamentar",
    "lameremos": "lamer",
    "lamerona": "lamerón",
    "lameronas": "lamerón",
    "lamerones": "lamerón",
    "lamerá": "lamer",
    "lamerán": "lamer",
    "lamerás": "lamer",
    "lameré": "lamer",
    "lameréis": "lamer",
    "lamería": "lamer",
    "lameríais": "lamer",
    "lameríamos": "lamer",
    "lamerían": "lamer",
    "lamerías": "lamer",
    "lames": "lamer",
    "lametones": "lametón",
    "lamias": "lamia",
    "lamida": "lamer",
    "lamidas": "lamer",
    "lamido": "lamer",
    "lamidos": "lamer",
    "lamiendo": "lamer",
    "lamiera": "lamer",
    "lamierais": "lamer",
    "lamieran": "lamer",
    "lamieras": "lamer",
    "lamiere": "lamer",
    "lamiereis": "lamer",
    "lamieren": "lamer",
    "lamieres": "lamer",
    "lamieron": "lamer",
    "lamiese": "lamer",
    "lamieseis": "lamer",
    "lamiesen": "lamer",
    "lamieses": "lamer",
    "lamimos": "lamer",
    "lamina": "laminar",
    "laminaba": "laminar",
    "laminabais": "laminar",
    "laminaban": "laminar",
    "laminabas": "laminar",
    "laminaciones": "laminación",
    "laminacióoes": "laminación",
    "laminad": "laminar",
    "laminada": "laminar",
    "laminadas": "laminar",
    "laminado": "laminar",
    "laminadora": "laminador",
    "laminadoras": "laminador",
    "laminadores": "laminador",
    "laminados": "laminar",
    "laminamos": "laminar",
    "laminan": "laminar",
    "laminando": "laminar",
    "laminara": "laminar",
    "laminarais": "laminar",
    "laminaran": "laminar",
    "laminaras": "laminar",
    "laminare": "laminar",
    "laminareis": "laminar",
    "laminaremos": "laminar",
    "laminaren": "laminar",
    "laminares": "laminar",
    "laminariales": "laminarial",
    "laminaron": "laminar",
    "laminará": "laminar",
    "laminarán": "laminar",
    "laminarás": "laminar",
    "laminaré": "laminar",
    "laminaréis": "laminar",
    "laminaría": "laminar",
    "laminaríais": "laminar",
    "laminaríamos": "laminar",
    "laminarían": "laminar",
    "laminarías": "laminar",
    "laminas": "laminar",
    "laminase": "laminar",
    "laminaseis": "laminar",
    "laminasen": "laminar",
    "laminases": "laminar",
    "laminaste": "laminar",
    "laminasteis": "laminar",
    "lamine": "laminar",
    "laminectomías": "laminectomía",
    "laminemos": "laminar",
    "laminen": "laminar",
    "laminera": "laminero",
    "lamineras": "laminero",
    "lamineros": "laminero",
    "lamines": "laminar",
    "laminillas": "laminilla",
    "lamininas": "laminina",
    "lamino": "laminar",
    "laminosa": "laminoso",
    "laminosas": "laminoso",
    "laminosos": "laminoso",
    "laminábamos": "laminar",
    "lamináis": "laminar",
    "lamináramos": "laminar",
    "lamináremos": "laminar",
    "laminásemos": "laminar",
    "laminé": "laminar",
    "laminéis": "laminar",
    "laminó": "laminar",
    "lamisca": "lamiscar",
    "lamiscaba": "lamiscar",
    "lamiscabais": "lamiscar",
    "lamiscaban": "lamiscar",
    "lamiscabas": "lamiscar",
    "lamiscad": "lamiscar",
    "lamiscada": "lamiscar",
    "lamiscadas": "lamiscar",
    "lamiscado": "lamiscar",
    "lamiscados": "lamiscar",
    "lamiscamos": "lamiscar",
    "lamiscan": "lamiscar",
    "lamiscando": "lamiscar",
    "lamiscara": "lamiscar",
    "lamiscarais": "lamiscar",
    "lamiscaran": "lamiscar",
    "lamiscaras": "lamiscar",
    "lamiscare": "lamiscar",
    "lamiscareis": "lamiscar",
    "lamiscaremos": "lamiscar",
    "lamiscaren": "lamiscar",
    "lamiscares": "lamiscar",
    "lamiscaron": "lamiscar",
    "lamiscará": "lamiscar",
    "lamiscarán": "lamiscar",
    "lamiscarás": "lamiscar",
    "lamiscaré": "lamiscar",
    "lamiscaréis": "lamiscar",
    "lamiscaría": "lamiscar",
    "lamiscaríais": "lamiscar",
    "lamiscaríamos": "lamiscar",
    "lamiscarían": "lamiscar",
    "lamiscarías": "lamiscar",
    "lamiscas": "lamiscar",
    "lamiscase": "lamiscar",
    "lamiscaseis": "lamiscar",
    "lamiscasen": "lamiscar",
    "lamiscases": "lamiscar",
    "lamiscaste": "lamiscar",
    "lamiscasteis": "lamiscar",
    "lamisco": "lamiscar",
    "lamiscábamos": "lamiscar",
    "lamiscáis": "lamiscar",
    "lamiscáramos": "lamiscar",
    "lamiscáremos": "lamiscar",
    "lamiscásemos": "lamiscar",
    "lamiscó": "lamiscar",
    "lamisque": "lamiscar",
    "lamisquemos": "lamiscar",
    "lamisquen": "lamiscar",
    "lamisques": "lamiscar",
    "lamisqué": "lamiscar",
    "lamisquéis": "lamiscar",
    "lamiste": "lamer",
    "lamisteis": "lamer",
    "lamiéramos": "lamer",
    "lamiéremos": "lamer",
    "lamiésemos": "lamer",
    "lamió": "lamer",
    "lamo": "lamer",
    "lamosa": "lamoso",
    "lamosas": "lamoso",
    "lamosos": "lamoso",
    "lamotea": "lamotear",
    "lamoteaba": "lamotear",
    "lamoteabais": "lamotear",
    "lamoteaban": "lamotear",
    "lamoteabas": "lamotear",
    "lamotead": "lamotear",
    "lamoteada": "lamotear",
    "lamoteadas": "lamotear",
    "lamoteado": "lamotear",
    "lamoteados": "lamotear",
    "lamoteamos": "lamotear",
    "lamotean": "lamotear",
    "lamoteando": "lamotear",
    "lamoteara": "lamotear",
    "lamotearais": "lamotear",
    "lamotearan": "lamotear",
    "lamotearas": "lamotear",
    "lamoteare": "lamotear",
    "lamoteareis": "lamotear",
    "lamotearemos": "lamotear",
    "lamotearen": "lamotear",
    "lamoteares": "lamotear",
    "lamotearon": "lamotear",
    "lamoteará": "lamotear",
    "lamotearán": "lamotear",
    "lamotearás": "lamotear",
    "lamotearé": "lamotear",
    "lamotearéis": "lamotear",
    "lamotearía": "lamotear",
    "lamotearíais": "lamotear",
    "lamotearíamos": "lamotear",
    "lamotearían": "lamotear",
    "lamotearías": "lamotear",
    "lamoteas": "lamotear",
    "lamotease": "lamotear",
    "lamoteaseis": "lamotear",
    "lamoteasen": "lamotear",
    "lamoteases": "lamotear",
    "lamoteaste": "lamotear",
    "lamoteasteis": "lamotear",
    "lamotee": "lamotear",
    "lamoteemos": "lamotear",
    "lamoteen": "lamotear",
    "lamotees": "lamotear",
    "lamoteo": "lamotear",
    "lamoteábamos": "lamotear",
    "lamoteáis": "lamotear",
    "lamoteáramos": "lamotear",
    "lamoteáremos": "lamotear",
    "lamoteásemos": "lamotear",
    "lamoteé": "lamotear",
    "lamoteéis": "lamotear",
    "lamoteó": "lamotear",
    "lampacea": "lampacear",
    "lampaceaba": "lampacear",
    "lampaceabais": "lampacear",
    "lampaceaban": "lampacear",
    "lampaceabas": "lampacear",
    "lampacead": "lampacear",
    "lampaceada": "lampacear",
    "lampaceadas": "lampacear",
    "lampaceado": "lampacear",
    "lampaceados": "lampacear",
    "lampaceamos": "lampacear",
    "lampacean": "lampacear",
    "lampaceando": "lampacear",
    "lampaceara": "lampacear",
    "lampacearais": "lampacear",
    "lampacearan": "lampacear",
    "lampacearas": "lampacear",
    "lampaceare": "lampacear",
    "lampaceareis": "lampacear",
    "lampacearemos": "lampacear",
    "lampacearen": "lampacear",
    "lampaceares": "lampacear",
    "lampacearon": "lampacear",
    "lampaceará": "lampacear",
    "lampacearán": "lampacear",
    "lampacearás": "lampacear",
    "lampacearé": "lampacear",
    "lampacearéis": "lampacear",
    "lampacearía": "lampacear",
    "lampacearíais": "lampacear",
    "lampacearíamos": "lampacear",
    "lampacearían": "lampacear",
    "lampacearías": "lampacear",
    "lampaceas": "lampacear",
    "lampacease": "lampacear",
    "lampaceaseis": "lampacear",
    "lampaceasen": "lampacear",
    "lampaceases": "lampacear",
    "lampaceaste": "lampacear",
    "lampaceasteis": "lampacear",
    "lampacee": "lampacear",
    "lampaceemos": "lampacear",
    "lampaceen": "lampacear",
    "lampacees": "lampacear",
    "lampaceo": "lampacear",
    "lampaceábamos": "lampacear",
    "lampaceáis": "lampacear",
    "lampaceáramos": "lampacear",
    "lampaceáremos": "lampacear",
    "lampaceásemos": "lampacear",
    "lampaceé": "lampacear",
    "lampaceéis": "lampacear",
    "lampaceó": "lampacear",
    "lampadarios": "lampadario",
    "lamparera": "lamparero",
    "lampareras": "lamparero",
    "lampareros": "lamparero",
    "lamparerías": "lamparería",
    "lamparillas": "lamparilla",
    "lamparines": "lamparín",
    "lamparistas": "lamparista",
    "lamparones": "lamparón",
    "lamparámetros": "lamparámetro",
    "lampazos": "lampazo",
    "lampeones": "lampeón",
    "lampiones": "lampión",
    "lampistas": "lampista",
    "lampiña": "lampiño",
    "lampiñas": "lampiño",
    "lampiños": "lampiño",
    "lampos": "lampo",
    "lamprea": "lamprear",
    "lampreaba": "lamprear",
    "lampreabais": "lamprear",
    "lampreaban": "lamprear",
    "lampreabas": "lamprear",
    "lampread": "lamprear",
    "lampreada": "lamprear",
    "lampreadas": "lamprear",
    "lampreado": "lamprear",
    "lampreados": "lamprear",
    "lampreamos": "lamprear",
    "lamprean": "lamprear",
    "lampreando": "lamprear",
    "lampreara": "lamprear",
    "lamprearais": "lamprear",
    "lamprearan": "lamprear",
    "lamprearas": "lamprear",
    "lampreare": "lamprear",
    "lampreareis": "lamprear",
    "lamprearemos": "lamprear",
    "lamprearen": "lamprear",
    "lampreares": "lamprear",
    "lamprearon": "lamprear",
    "lampreará": "lamprear",
    "lamprearán": "lamprear",
    "lamprearás": "lamprear",
    "lamprearé": "lamprear",
    "lamprearéis": "lamprear",
    "lamprearía": "lamprear",
    "lamprearíais": "lamprear",
    "lamprearíamos": "lamprear",
    "lamprearían": "lamprear",
    "lamprearías": "lamprear",
    "lampreas": "lamprear",
    "lamprease": "lamprear",
    "lampreaseis": "lamprear",
    "lampreasen": "lamprear",
    "lampreases": "lamprear",
    "lampreaste": "lamprear",
    "lampreasteis": "lamprear",
    "lampreazos": "lampreazo",
    "lampree": "lamprear",
    "lampreemos": "lamprear",
    "lampreen": "lamprear",
    "lamprees": "lamprear",
    "lampreo": "lamprear",
    "lampreábamos": "lamprear",
    "lampreáis": "lamprear",
    "lampreáramos": "lamprear",
    "lampreáremos": "lamprear",
    "lampreásemos": "lamprear",
    "lampreé": "lamprear",
    "lampreéis": "lamprear",
    "lampreó": "lamprear",
    "lamprófidos": "lamprófido",
    "lampugas": "lampuga",
    "lampugueras": "lampuguera",
    "lamáis": "lamer",
    "laméis": "lamer",
    "lamés": "lamé",
    "lamí": "lamer",
    "lamía": "lamer",
    "lamíais": "lamer",
    "lamíamos": "lamer",
    "lamían": "lamer",
    "lamías": "lamer",
    "lanada": "lanado",
    "lanadas": "lanado",
    "lanados": "lanado",
    "lanares": "lanar",
    "lanas": "lana",
    "lance": "lanzar",
    "lancea": "lancear",
    "lanceaba": "lancear",
    "lanceabais": "lancear",
    "lanceaban": "lancear",
    "lanceabas": "lancear",
    "lancead": "lancear",
    "lanceada": "lancear",
    "lanceadas": "lancear",
    "lanceado": "lancear",
    "lanceados": "lancear",
    "lanceamos": "lancear",
    "lancean": "lancear",
    "lanceando": "lancear",
    "lanceara": "lancear",
    "lancearais": "lancear",
    "lancearan": "lancear",
    "lancearas": "lancear",
    "lanceare": "lancear",
    "lanceareis": "lancear",
    "lancearemos": "lancear",
    "lancearen": "lancear",
    "lanceares": "lancear",
    "lancearon": "lancear",
    "lanceará": "lancear",
    "lancearán": "lancear",
    "lancearás": "lancear",
    "lancearé": "lancear",
    "lancearéis": "lancear",
    "lancearía": "lancear",
    "lancearíais": "lancear",
    "lancearíamos": "lancear",
    "lancearían": "lancear",
    "lancearías": "lancear",
    "lanceas": "lancear",
    "lancease": "lancear",
    "lanceaseis": "lancear",
    "lanceasen": "lancear",
    "lanceases": "lancear",
    "lanceaste": "lancear",
    "lanceasteis": "lancear",
    "lancee": "lancear",
    "lanceemos": "lancear",
    "lanceen": "lancear",
    "lancees": "lancear",
    "lancemos": "lanzar",
    "lancen": "lanzar",
    "lanceo": "lancear",
    "lanceolada": "lanceolado",
    "lanceoladas": "lanceolado",
    "lanceolados": "lanceolado",
    "lancera": "lancero",
    "lanceras": "lancero",
    "lanceros": "lancero",
    "lancerías": "lancería",
    "lances": "lanzar",
    "lancetadas": "lancetada",
    "lancetas": "lanceta",
    "lancetazos": "lancetazo",
    "lanceteros": "lancetero",
    "lanceábamos": "lancear",
    "lanceáis": "lancear",
    "lanceáramos": "lancear",
    "lanceáremos": "lancear",
    "lanceásemos": "lancear",
    "lanceé": "lancear",
    "lanceéis": "lancear",
    "lanceó": "lancear",
    "lanchadas": "lanchada",
    "lanchajes": "lanchaje",
    "lanchares": "lanchar",
    "lanchas": "lancha",
    "lanchazos": "lanchazo",
    "lanchera": "lanchero",
    "lancheras": "lanchero",
    "lancheros": "lanchero",
    "lancina": "lancinar",
    "lancinaba": "lancinar",
    "lancinabais": "lancinar",
    "lancinaban": "lancinar",
    "lancinabas": "lancinar",
    "lancinad": "lancinar",
    "lancinada": "lancinar",
    "lancinadas": "lancinar",
    "lancinado": "lancinar",
    "lancinados": "lancinar",
    "lancinamos": "lancinar",
    "lancinan": "lancinar",
    "lancinando": "lancinar",
    "lancinantes": "lancinante",
    "lancinara": "lancinar",
    "lancinarais": "lancinar",
    "lancinaran": "lancinar",
    "lancinaras": "lancinar",
    "lancinare": "lancinar",
    "lancinareis": "lancinar",
    "lancinaremos": "lancinar",
    "lancinaren": "lancinar",
    "lancinares": "lancinar",
    "lancinaron": "lancinar",
    "lancinará": "lancinar",
    "lancinarán": "lancinar",
    "lancinarás": "lancinar",
    "lancinaré": "lancinar",
    "lancinaréis": "lancinar",
    "lancinaría": "lancinar",
    "lancinaríais": "lancinar",
    "lancinaríamos": "lancinar",
    "lancinarían": "lancinar",
    "lancinarías": "lancinar",
    "lancinas": "lancinar",
    "lancinase": "lancinar",
    "lancinaseis": "lancinar",
    "lancinasen": "lancinar",
    "lancinases": "lancinar",
    "lancinaste": "lancinar",
    "lancinasteis": "lancinar",
    "lancine": "lancinar",
    "lancinemos": "lancinar",
    "lancinen": "lancinar",
    "lancines": "lancinar",
    "lancino": "lancinar",
    "lancinábamos": "lancinar",
    "lancináis": "lancinar",
    "lancináramos": "lancinar",
    "lancináremos": "lancinar",
    "lancinásemos": "lancinar",
    "lanciné": "lancinar",
    "lancinéis": "lancinar",
    "lancinó": "lancinar",
    "lancurdias": "lancurdia",
    "lancé": "lanzar",
    "lancéis": "lanzar",
    "landas": "landa",
    "landgraves": "landgrave",
    "landgraviatos": "landgraviato",
    "landrecillas": "landrecilla",
    "landrera": "landrero",
    "landreras": "landrero",
    "landreros": "landrero",
    "landres": "landre",
    "landrillas": "landrilla",
    "lands": "land",
    "landós": "landó",
    "lanera": "lanero",
    "laneras": "lanero",
    "laneros": "lanero",
    "lanerías": "lanería",
    "langaruta": "langaruto",
    "langarutas": "langaruto",
    "langarutos": "langaruto",
    "langbeinitas": "langbeinita",
    "langostas": "langosta",
    "langostera": "langostero",
    "langosteras": "langostero",
    "langosteros": "langostero",
    "langostinera": "langostiner",
    "langostineras": "langostiner",
    "langostineres": "langostiner",
    "langostines": "langostín",
    "langostinos": "langostino",
    "languedociana": "languedociano",
    "languedocianas": "languedociano",
    "languedocianos": "languedociano",
    "languidece": "languidecer",
    "languideced": "languidecer",
    "languidecemos": "languidecer",
    "languidecen": "languidecer",
    "languideceremos": "languidecer",
    "languidecerá": "languidecer",
    "languidecerán": "languidecer",
    "languidecerás": "languidecer",
    "languideceré": "languidecer",
    "languideceréis": "languidecer",
    "languidecería": "languidecer",
    "languideceríais": "languidecer",
    "languideceríamos": "languidecer",
    "languidecerían": "languidecer",
    "languidecerías": "languidecer",
    "languideces": "languidez",
    "languidecida": "languidecer",
    "languidecidas": "languidecer",
    "languidecido": "languidecer",
    "languidecidos": "languidecer",
    "languideciendo": "languidecer",
    "languideciera": "languidecer",
    "languidecierais": "languidecer",
    "languidecieran": "languidecer",
    "languidecieras": "languidecer",
    "languideciere": "languidecer",
    "languideciereis": "languidecer",
    "languidecieren": "languidecer",
    "languidecieres": "languidecer",
    "languidecieron": "languidecer",
    "languideciese": "languidecer",
    "languidecieseis": "languidecer",
    "languideciesen": "languidecer",
    "languidecieses": "languidecer",
    "languidecimos": "languidecer",
    "languideciste": "languidecer",
    "languidecisteis": "languidecer",
    "languideciéramos": "languidecer",
    "languideciéremos": "languidecer",
    "languideciésemos": "languidecer",
    "languideció": "languidecer",
    "languidecéis": "languidecer",
    "languidecí": "languidecer",
    "languidecía": "languidecer",
    "languidecíais": "languidecer",
    "languidecíamos": "languidecer",
    "languidecían": "languidecer",
    "languidecías": "languidecer",
    "languidezca": "languidecer",
    "languidezcamos": "languidecer",
    "languidezcan": "languidecer",
    "languidezcas": "languidecer",
    "languidezco": "languidecer",
    "languidezcáis": "languidecer",
    "langures": "langur",
    "lanificaciones": "lanificación",
    "lanificios": "lanificio",
    "lanillas": "lanilla",
    "lanolinas": "lanolina",
    "lanosa": "lanoso",
    "lanosas": "lanoso",
    "lanosidades": "lanosidad",
    "lanosos": "lanoso",
    "lansquenetes": "lansquenete",
    "lantanos": "lantano",
    "lantioninas": "lantionina",
    "lantánidos": "lantánido",
    "lanuda": "lanudo",
    "lanudas": "lanudo",
    "lanudos": "lanudo",
    "lanuginosa": "lanuginoso",
    "lanuginosas": "lanuginoso",
    "lanuginosos": "lanuginoso",
    "lanugos": "lanugo",
    "lanza": "lanzar",
    "lanzaba": "lanzar",
    "lanzabais": "lanzar",
    "lanzaban": "lanzar",
    "lanzabas": "lanzar",
    "lanzacementos": "lanzacemento",
    "lanzad": "lanzar",
    "lanzada": "lanzar",
    "lanzadas": "lanzar",
    "lanzaderas": "lanzadera",
    "lanzado": "lanzar",
    "lanzadora": "lanzador",
    "lanzadoras": "lanzador",
    "lanzadores": "lanzador",
    "lanzados": "lanzar",
    "lanzafuegos": "lanzafuego",
    "lanzahielos": "lanzahielo",
    "lanzamientos": "lanzamiento",
    "lanzamos": "lanzar",
    "lanzan": "lanzar",
    "lanzando": "lanzar",
    "lanzara": "lanzar",
    "lanzarais": "lanzar",
    "lanzaran": "lanzar",
    "lanzaras": "lanzar",
    "lanzare": "lanzar",
    "lanzareis": "lanzar",
    "lanzaremos": "lanzar",
    "lanzaren": "lanzar",
    "lanzares": "lanzar",
    "lanzaron": "lanzar",
    "lanzará": "lanzar",
    "lanzarán": "lanzar",
    "lanzarás": "lanzar",
    "lanzaré": "lanzar",
    "lanzaréis": "lanzar",
    "lanzaría": "lanzar",
    "lanzaríais": "lanzar",
    "lanzaríamos": "lanzar",
    "lanzarían": "lanzar",
    "lanzarías": "lanzar",
    "lanzas": "lanzar",
    "lanzase": "lanzar",
    "lanzaseis": "lanzar",
    "lanzasen": "lanzar",
    "lanzases": "lanzar",
    "lanzaste": "lanzar",
    "lanzasteis": "lanzar",
    "lanzazos": "lanzazo",
    "lanzo": "lanzar",
    "lanzones": "lanzón",
    "lanzábamos": "lanzar",
    "lanzáis": "lanzar",
    "lanzáramos": "lanzar",
    "lanzáremos": "lanzar",
    "lanzásemos": "lanzar",
    "lanzó": "lanzar",
    "lanía": "lanío",
    "lanías": "lanío",
    "lanífera": "lanífero",
    "laníferas": "lanífero",
    "laníferos": "lanífero",
    "laníos": "lanío",
    "laosiana": "laosiano",
    "laosianas": "laosiano",
    "laosianos": "laosiano",
    "lapachares": "lapachar",
    "lapachas": "lapacha",
    "lapachos": "lapacho",
    "laparoscopias": "laparoscopia",
    "laparotomías": "laparotomía",
    "lapas": "lapa",
    "lapeadoras": "lapeadora",
    "lapiaces": "lapiaz",
    "lapiceros": "lapicero",
    "lapida": "lapidar",
    "lapidaba": "lapidar",
    "lapidabais": "lapidar",
    "lapidaban": "lapidar",
    "lapidabas": "lapidar",
    "lapidaciones": "lapidación",
    "lapidad": "lapidar",
    "lapidada": "lapidar",
    "lapidadas": "lapidar",
    "lapidado": "lapidar",
    "lapidados": "lapidar",
    "lapidamos": "lapidar",
    "lapidan": "lapidar",
    "lapidando": "lapidar",
    "lapidara": "lapidar",
    "lapidarais": "lapidar",
    "lapidaran": "lapidar",
    "lapidaras": "lapidar",
    "lapidare": "lapidar",
    "lapidareis": "lapidar",
    "lapidaremos": "lapidar",
    "lapidaren": "lapidar",
    "lapidares": "lapidar",
    "lapidaria": "lapidario",
    "lapidarias": "lapidario",
    "lapidarios": "lapidario",
    "lapidaron": "lapidar",
    "lapidará": "lapidar",
    "lapidarán": "lapidar",
    "lapidarás": "lapidar",
    "lapidaré": "lapidar",
    "lapidaréis": "lapidar",
    "lapidaría": "lapidar",
    "lapidaríais": "lapidar",
    "lapidaríamos": "lapidar",
    "lapidarían": "lapidar",
    "lapidarías": "lapidar",
    "lapidas": "lapidar",
    "lapidase": "lapidar",
    "lapidaseis": "lapidar",
    "lapidasen": "lapidar",
    "lapidases": "lapidar",
    "lapidaste": "lapidar",
    "lapidasteis": "lapidar",
    "lapide": "lapidar",
    "lapidemos": "lapidar",
    "lapiden": "lapidar",
    "lapides": "lapidar",
    "lapidifica": "lapidificar",
    "lapidificaba": "lapidificar",
    "lapidificabais": "lapidificar",
    "lapidificaban": "lapidificar",
    "lapidificabas": "lapidificar",
    "lapidificaciones": "lapidificación",
    "lapidificad": "lapidificar",
    "lapidificada": "lapidificar",
    "lapidificadas": "lapidificar",
    "lapidificado": "lapidificar",
    "lapidificados": "lapidificar",
    "lapidificamos": "lapidificar",
    "lapidifican": "lapidificar",
    "lapidificando": "lapidificar",
    "lapidificara": "lapidificar",
    "lapidificarais": "lapidificar",
    "lapidificaran": "lapidificar",
    "lapidificaras": "lapidificar",
    "lapidificare": "lapidificar",
    "lapidificareis": "lapidificar",
    "lapidificaremos": "lapidificar",
    "lapidificaren": "lapidificar",
    "lapidificares": "lapidificar",
    "lapidificaron": "lapidificar",
    "lapidificará": "lapidificar",
    "lapidificarán": "lapidificar",
    "lapidificarás": "lapidificar",
    "lapidificaré": "lapidificar",
    "lapidificaréis": "lapidificar",
    "lapidificaría": "lapidificar",
    "lapidificaríais": "lapidificar",
    "lapidificaríamos": "lapidificar",
    "lapidificarían": "lapidificar",
    "lapidificarías": "lapidificar",
    "lapidificas": "lapidificar",
    "lapidificase": "lapidificar",
    "lapidificaseis": "lapidificar",
    "lapidificasen": "lapidificar",
    "lapidificases": "lapidificar",
    "lapidificaste": "lapidificar",
    "lapidificasteis": "lapidificar",
    "lapidifico": "lapidificar",
    "lapidificábamos": "lapidificar",
    "lapidificáis": "lapidificar",
    "lapidificáramos": "lapidificar",
    "lapidificáremos": "lapidificar",
    "lapidificásemos": "lapidificar",
    "lapidificó": "lapidificar",
    "lapidifique": "lapidificar",
    "lapidifiquemos": "lapidificar",
    "lapidifiquen": "lapidificar",
    "lapidifiques": "lapidificar",
    "lapidifiqué": "lapidificar",
    "lapidifiquéis": "lapidificar",
    "lapido": "lapidar",
    "lapidosa": "lapidoso",
    "lapidosas": "lapidoso",
    "lapidosos": "lapidoso",
    "lapidábamos": "lapidar",
    "lapidáis": "lapidar",
    "lapidáramos": "lapidar",
    "lapidáremos": "lapidar",
    "lapidásemos": "lapidar",
    "lapidé": "lapidar",
    "lapidéis": "lapidar",
    "lapidífica": "lapidífico",
    "lapidíficas": "lapidífico",
    "lapidíficos": "lapidífico",
    "lapidó": "lapidar",
    "lapislázulis": "lapislázuli",
    "lapitas": "lapita",
    "lapizares": "lapizar",
    "laplaciana": "laplaciano",
    "laplacianas": "laplaciano",
    "laplacianos": "laplaciano",
    "lapona": "lapón",
    "laponas": "lapón",
    "lapones": "lapón",
    "lapos": "lapo",
    "lapsos": "lapso",
    "lapídea": "lapídeo",
    "lapídeas": "lapídeo",
    "lapídeos": "lapídeo",
    "lapóna": "lapón",
    "lapónas": "lapón",
    "lapónes": "lapón",
    "laquea": "laquear",
    "laqueaba": "laquear",
    "laqueabais": "laquear",
    "laqueaban": "laquear",
    "laqueabas": "laquear",
    "laquead": "laquear",
    "laqueada": "laquear",
    "laqueadas": "laquear",
    "laqueado": "laquear",
    "laqueados": "laquear",
    "laqueamos": "laquear",
    "laquean": "laquear",
    "laqueando": "laquear",
    "laqueara": "laquear",
    "laquearais": "laquear",
    "laquearan": "laquear",
    "laquearas": "laquear",
    "laqueare": "laquear",
    "laqueareis": "laquear",
    "laquearemos": "laquear",
    "laquearen": "laquear",
    "laqueares": "laquear",
    "laquearon": "laquear",
    "laqueará": "laquear",
    "laquearán": "laquear",
    "laquearás": "laquear",
    "laquearé": "laquear",
    "laquearéis": "laquear",
    "laquearía": "laquear",
    "laquearíais": "laquear",
    "laquearíamos": "laquear",
    "laquearían": "laquear",
    "laquearías": "laquear",
    "laqueas": "laquear",
    "laquease": "laquear",
    "laqueaseis": "laquear",
    "laqueasen": "laquear",
    "laqueases": "laquear",
    "laqueaste": "laquear",
    "laqueasteis": "laquear",
    "laquee": "laquear",
    "laqueemos": "laquear",
    "laqueen": "laquear",
    "laquees": "laquear",
    "laqueo": "laquear",
    "laqueábamos": "laquear",
    "laqueáis": "laquear",
    "laqueáramos": "laquear",
    "laqueáremos": "laquear",
    "laqueásemos": "laquear",
    "laqueé": "laquear",
    "laqueéis": "laquear",
    "laqueó": "laquear",
    "lararios": "larario",
    "larda": "lardar",
    "lardaba": "lardar",
    "lardabais": "lardar",
    "lardaban": "lardar",
    "lardabas": "lardar",
    "lardad": "lardar",
    "lardada": "lardar",
    "lardadas": "lardar",
    "lardado": "lardar",
    "lardados": "lardar",
    "lardamos": "lardar",
    "lardan": "lardar",
    "lardando": "lardar",
    "lardara": "lardar",
    "lardarais": "lardar",
    "lardaran": "lardar",
    "lardaras": "lardar",
    "lardare": "lardar",
    "lardareis": "lardar",
    "lardaremos": "lardar",
    "lardaren": "lardar",
    "lardares": "lardar",
    "lardaron": "lardar",
    "lardará": "lardar",
    "lardarán": "lardar",
    "lardarás": "lardar",
    "lardaré": "lardar",
    "lardaréis": "lardar",
    "lardaría": "lardar",
    "lardaríais": "lardar",
    "lardaríamos": "lardar",
    "lardarían": "lardar",
    "lardarías": "lardar",
    "lardas": "lardar",
    "lardase": "lardar",
    "lardaseis": "lardar",
    "lardasen": "lardar",
    "lardases": "lardar",
    "lardaste": "lardar",
    "lardasteis": "lardar",
    "larde": "lardar",
    "lardea": "lardear",
    "lardeaba": "lardear",
    "lardeabais": "lardear",
    "lardeaban": "lardear",
    "lardeabas": "lardear",
    "lardead": "lardear",
    "lardeada": "lardear",
    "lardeadas": "lardear",
    "lardeado": "lardear",
    "lardeados": "lardear",
    "lardeamos": "lardear",
    "lardean": "lardear",
    "lardeando": "lardear",
    "lardeara": "lardear",
    "lardearais": "lardear",
    "lardearan": "lardear",
    "lardearas": "lardear",
    "lardeare": "lardear",
    "lardeareis": "lardear",
    "lardearemos": "lardear",
    "lardearen": "lardear",
    "lardeares": "lardear",
    "lardearon": "lardear",
    "lardeará": "lardear",
    "lardearán": "lardear",
    "lardearás": "lardear",
    "lardearé": "lardear",
    "lardearéis": "lardear",
    "lardearía": "lardear",
    "lardearíais": "lardear",
    "lardearíamos": "lardear",
    "lardearían": "lardear",
    "lardearías": "lardear",
    "lardeas": "lardear",
    "lardease": "lardear",
    "lardeaseis": "lardear",
    "lardeasen": "lardear",
    "lardeases": "lardear",
    "lardeaste": "lardear",
    "lardeasteis": "lardear",
    "lardee": "lardear",
    "lardeemos": "lardear",
    "lardeen": "lardear",
    "lardees": "lardear",
    "lardemos": "lardar",
    "larden": "lardar",
    "lardeo": "lardear",
    "lardera": "lardero",
    "larderas": "lardero",
    "larderos": "lardero",
    "lardes": "lardar",
    "lardeábamos": "lardear",
    "lardeáis": "lardear",
    "lardeáramos": "lardear",
    "lardeáremos": "lardear",
    "lardeásemos": "lardear",
    "lardeé": "lardear",
    "lardeéis": "lardear",
    "lardeó": "lardear",
    "lardizabaláceas": "lardizabalácea",
    "lardo": "lardar",
    "lardones": "lardón",
    "lardos": "lardo",
    "lardosa": "lardoso",
    "lardosas": "lardoso",
    "lardosos": "lardoso",
    "lardábamos": "lardar",
    "lardáis": "lardar",
    "lardáramos": "lardar",
    "lardáremos": "lardar",
    "lardásemos": "lardar",
    "lardé": "lardar",
    "lardéis": "lardar",
    "lardó": "lardar",
    "lares": "lar",
    "larga": "largo",
    "largaba": "largar",
    "largabais": "largar",
    "largaban": "largar",
    "largabas": "largar",
    "largad": "largar",
    "largada": "largar",
    "largadas": "largar",
    "largado": "largar",
    "largados": "largar",
    "largamos": "largar",
    "largan": "largar",
    "largando": "largar",
    "largara": "largar",
    "largarais": "largar",
    "largaran": "largar",
    "largaras": "largar",
    "largare": "largar",
    "largareis": "largar",
    "largaremos": "largar",
    "largaren": "largar",
    "largares": "largar",
    "largaron": "largar",
    "largará": "largar",
    "largarán": "largar",
    "largarás": "largar",
    "largaré": "largar",
    "largaréis": "largar",
    "largaría": "largar",
    "largaríais": "largar",
    "largaríamos": "largar",
    "largarían": "largar",
    "largarías": "largar",
    "largas": "largo",
    "largase": "largar",
    "largaseis": "largar",
    "largasen": "largar",
    "largases": "largar",
    "largaste": "largar",
    "largasteis": "largar",
    "largo": "largar",
    "largometrajes": "largometraje",
    "largomiras": "largomira",
    "largores": "largor",
    "largos": "largo",
    "largue": "largar",
    "largueada": "largueado",
    "largueadas": "largueado",
    "largueados": "largueado",
    "larguemos": "largar",
    "larguen": "largar",
    "largueros": "larguero",
    "largues": "largar",
    "larguezas": "largueza",
    "larguirucha": "larguirucho",
    "larguiruchas": "larguirucho",
    "larguiruchos": "larguirucho",
    "larguras": "largura",
    "largué": "largar",
    "larguéis": "largar",
    "largábamos": "largar",
    "largáis": "largar",
    "largáramos": "largar",
    "largáremos": "largar",
    "largásemos": "largar",
    "largó": "largar",
    "laricia": "laricio",
    "laricias": "laricio",
    "laricios": "laricio",
    "laringalgias": "laringalgia",
    "laringectomías": "laringectomía",
    "laringes": "laringe",
    "laringismos": "laringismo",
    "laringoceles": "laringocele",
    "laringoespasmos": "laringoespasmo",
    "laringoestroboscopios": "laringoestroboscopio",
    "laringofaringectomías": "laringofaringectomía",
    "laringofaringes": "laringofaringe",
    "laringofisuras": "laringofisura",
    "laringografías": "laringografía",
    "laringologías": "laringología",
    "laringomalacias": "laringomalacia",
    "laringopatías": "laringopatía",
    "laringopioceles": "laringopiocele",
    "laringoplastias": "laringoplastia",
    "laringoplejías": "laringoplejía",
    "laringorragias": "laringorragia",
    "laringoscopias": "laringoscopia",
    "laringoscopios": "laringoscopio",
    "laringospasmos": "laringospasmo",
    "laringostomías": "laringostomía",
    "laringotomías": "laringotomía",
    "laringófonos": "laringófono",
    "laringóloga": "laringólogo",
    "laringólogas": "laringólogo",
    "laringólogos": "laringólogo",
    "larvada": "larvado",
    "larvadas": "larvado",
    "larvados": "larvado",
    "larvales": "larval",
    "larvaria": "larvario",
    "larvarias": "larvario",
    "larvarios": "larvario",
    "larvas": "larva",
    "larvicidas": "larvicida",
    "larvikitas": "larvikita",
    "laríngea": "laríngeo",
    "laríngeas": "laríngeo",
    "laríngeos": "laríngeo",
    "las": "los",
    "lasa": "laso",
    "lasas": "laso",
    "lasañas": "lasaña",
    "lasca": "lascar",
    "lascaba": "lascar",
    "lascabais": "lascar",
    "lascaban": "lascar",
    "lascabas": "lascar",
    "lascad": "lascar",
    "lascada": "lascar",
    "lascadas": "lascar",
    "lascado": "lascar",
    "lascados": "lascar",
    "lascamos": "lascar",
    "lascan": "lascar",
    "lascando": "lascar",
    "lascara": "lascar",
    "lascarais": "lascar",
    "lascaran": "lascar",
    "lascaras": "lascar",
    "lascare": "lascar",
    "lascareis": "lascar",
    "lascaremos": "lascar",
    "lascaren": "lascar",
    "lascares": "lascar",
    "lascaron": "lascar",
    "lascará": "lascar",
    "lascarán": "lascar",
    "lascarás": "lascar",
    "lascaré": "lascar",
    "lascaréis": "lascar",
    "lascaría": "lascar",
    "lascaríais": "lascar",
    "lascaríamos": "lascar",
    "lascarían": "lascar",
    "lascarías": "lascar",
    "lascas": "lascar",
    "lascase": "lascar",
    "lascaseis": "lascar",
    "lascasen": "lascar",
    "lascases": "lascar",
    "lascaste": "lascar",
    "lascasteis": "lascar",
    "lasciva": "lascivo",
    "lascivas": "lascivo",
    "lascivias": "lascivia",
    "lascivos": "lascivo",
    "lasco": "lascar",
    "lascábamos": "lascar",
    "lascáis": "lascar",
    "lascáramos": "lascar",
    "lascáremos": "lascar",
    "lascásemos": "lascar",
    "lascó": "lascar",
    "lasitudes": "lasitud",
    "lasos": "laso",
    "lasque": "lascar",
    "lasquemos": "lascar",
    "lasquen": "lascar",
    "lasques": "lascar",
    "lasqué": "lascar",
    "lasquéis": "lascar",
    "lasta": "lastar",
    "lastaba": "lastar",
    "lastabais": "lastar",
    "lastaban": "lastar",
    "lastabas": "lastar",
    "lastad": "lastar",
    "lastada": "lastar",
    "lastadas": "lastar",
    "lastado": "lastar",
    "lastados": "lastar",
    "lastamos": "lastar",
    "lastan": "lastar",
    "lastando": "lastar",
    "lastara": "lastar",
    "lastarais": "lastar",
    "lastaran": "lastar",
    "lastaras": "lastar",
    "lastare": "lastar",
    "lastareis": "lastar",
    "lastaremos": "lastar",
    "lastaren": "lastar",
    "lastares": "lastar",
    "lastaron": "lastar",
    "lastará": "lastar",
    "lastarán": "lastar",
    "lastarás": "lastar",
    "lastaré": "lastar",
    "lastaréis": "lastar",
    "lastaría": "lastar",
    "lastaríais": "lastar",
    "lastaríamos": "lastar",
    "lastarían": "lastar",
    "lastarías": "lastar",
    "lastas": "lastar",
    "lastase": "lastar",
    "lastaseis": "lastar",
    "lastasen": "lastar",
    "lastases": "lastar",
    "lastaste": "lastar",
    "lastasteis": "lastar",
    "laste": "lastar",
    "lastemos": "lastar",
    "lasten": "lastar",
    "lastes": "lastar",
    "lastima": "lastimar",
    "lastimaba": "lastimar",
    "lastimabais": "lastimar",
    "lastimaban": "lastimar",
    "lastimabas": "lastimar",
    "lastimad": "lastimar",
    "lastimada": "lastimar",
    "lastimadas": "lastimar",
    "lastimado": "lastimar",
    "lastimadora": "lastimador",
    "lastimadoras": "lastimador",
    "lastimadores": "lastimador",
    "lastimados": "lastimar",
    "lastimaduras": "lastimadura",
    "lastimamos": "lastimar",
    "lastiman": "lastimar",
    "lastimando": "lastimar",
    "lastimara": "lastimar",
    "lastimarais": "lastimar",
    "lastimaran": "lastimar",
    "lastimaras": "lastimar",
    "lastimare": "lastimar",
    "lastimareis": "lastimar",
    "lastimaremos": "lastimar",
    "lastimaren": "lastimar",
    "lastimares": "lastimar",
    "lastimaron": "lastimar",
    "lastimará": "lastimar",
    "lastimarán": "lastimar",
    "lastimarás": "lastimar",
    "lastimaré": "lastimar",
    "lastimaréis": "lastimar",
    "lastimaría": "lastimar",
    "lastimaríais": "lastimar",
    "lastimaríamos": "lastimar",
    "lastimarían": "lastimar",
    "lastimarías": "lastimar",
    "lastimas": "lastimar",
    "lastimase": "lastimar",
    "lastimaseis": "lastimar",
    "lastimasen": "lastimar",
    "lastimases": "lastimar",
    "lastimaste": "lastimar",
    "lastimasteis": "lastimar",
    "lastime": "lastimar",
    "lastimemos": "lastimar",
    "lastimen": "lastimar",
    "lastimera": "lastimero",
    "lastimeras": "lastimero",
    "lastimeros": "lastimero",
    "lastimes": "lastimar",
    "lastimo": "lastimar",
    "lastimosa": "lastimoso",
    "lastimosas": "lastimoso",
    "lastimosos": "lastimoso",
    "lastimábamos": "lastimar",
    "lastimáis": "lastimar",
    "lastimáramos": "lastimar",
    "lastimáremos": "lastimar",
    "lastimásemos": "lastimar",
    "lastimé": "lastimar",
    "lastiméis": "lastimar",
    "lastimó": "lastimar",
    "lasto": "lastar",
    "lastones": "lastón",
    "lastos": "lasto",
    "lastra": "lastrar",
    "lastraba": "lastrar",
    "lastrabais": "lastrar",
    "lastraban": "lastrar",
    "lastrabas": "lastrar",
    "lastrad": "lastrar",
    "lastrada": "lastrar",
    "lastradas": "lastrar",
    "lastrado": "lastrar",
    "lastrados": "lastrar",
    "lastramos": "lastrar",
    "lastran": "lastrar",
    "lastrando": "lastrar",
    "lastrara": "lastrar",
    "lastrarais": "lastrar",
    "lastraran": "lastrar",
    "lastraras": "lastrar",
    "lastrare": "lastrar",
    "lastrareis": "lastrar",
    "lastraremos": "lastrar",
    "lastraren": "lastrar",
    "lastrares": "lastrar",
    "lastraron": "lastrar",
    "lastrará": "lastrar",
    "lastrarán": "lastrar",
    "lastrarás": "lastrar",
    "lastraré": "lastrar",
    "lastraréis": "lastrar",
    "lastraría": "lastrar",
    "lastraríais": "lastrar",
    "lastraríamos": "lastrar",
    "lastrarían": "lastrar",
    "lastrarías": "lastrar",
    "lastras": "lastrar",
    "lastrase": "lastrar",
    "lastraseis": "lastrar",
    "lastrasen": "lastrar",
    "lastrases": "lastrar",
    "lastraste": "lastrar",
    "lastrasteis": "lastrar",
    "lastre": "lastrar",
    "lastremos": "lastrar",
    "lastren": "lastrar",
    "lastres": "lastre",
    "lastro": "lastrar",
    "lastrábamos": "lastrar",
    "lastráis": "lastrar",
    "lastráramos": "lastrar",
    "lastráremos": "lastrar",
    "lastrásemos": "lastrar",
    "lastré": "lastrar",
    "lastréis": "lastrar",
    "lastró": "lastrar",
    "lastábamos": "lastar",
    "lastáis": "lastar",
    "lastáramos": "lastar",
    "lastáremos": "lastar",
    "lastásemos": "lastar",
    "lasté": "lastar",
    "lastéis": "lastar",
    "lastó": "lastar",
    "lata": "lato",
    "latamos": "latir",
    "latan": "latir",
    "latas": "lato",
    "latastros": "latastro",
    "late": "latir",
    "latebras": "latebra",
    "latebrosa": "latebroso",
    "latebrosas": "latebroso",
    "latebrosos": "latebroso",
    "laten": "latir",
    "latencias": "latencia",
    "latentes": "latente",
    "laterales": "lateral",
    "lateralidades": "lateralidad",
    "lateralizaciones": "lateralización",
    "lateralizada": "lateralizado",
    "lateralizadas": "lateralizado",
    "lateralizados": "lateralizado",
    "lateranenses": "lateranense",
    "lateritas": "laterita",
    "lateritizaciones": "lateritización",
    "laterizaciones": "laterización",
    "laterofaríngea": "laterofaríngeo",
    "laterofaríngeas": "laterofaríngeo",
    "laterofaríngeos": "laterofaríngeo",
    "laterognacias": "laterognacia",
    "laterotrusiones": "laterotrusión",
    "laterías": "latería",
    "laterítica": "laterítico",
    "lateríticas": "laterítico",
    "lateríticos": "laterítico",
    "lates": "latir",
    "laticífera": "laticífero",
    "laticíferas": "laticífero",
    "laticíferos": "laticífero",
    "latid": "latir",
    "latida": "latir",
    "latidas": "latir",
    "latido": "latir",
    "latidos": "latir",
    "latiendo": "latir",
    "latiera": "latir",
    "latierais": "latir",
    "latieran": "latir",
    "latieras": "latir",
    "latiere": "latir",
    "latiereis": "latir",
    "latieren": "latir",
    "latieres": "latir",
    "latieron": "latir",
    "latiese": "latir",
    "latieseis": "latir",
    "latiesen": "latir",
    "latieses": "latir",
    "latifundios": "latifundio",
    "latifundismos": "latifundismo",
    "latifundistas": "latifundista",
    "latigazos": "latigazo",
    "latiguea": "latiguear",
    "latigueaba": "latiguear",
    "latigueabais": "latiguear",
    "latigueaban": "latiguear",
    "latigueabas": "latiguear",
    "latiguead": "latiguear",
    "latigueada": "latiguear",
    "latigueadas": "latiguear",
    "latigueado": "latiguear",
    "latigueados": "latiguear",
    "latigueamos": "latiguear",
    "latiguean": "latiguear",
    "latigueando": "latiguear",
    "latigueara": "latiguear",
    "latiguearais": "latiguear",
    "latiguearan": "latiguear",
    "latiguearas": "latiguear",
    "latigueare": "latiguear",
    "latigueareis": "latiguear",
    "latiguearemos": "latiguear",
    "latiguearen": "latiguear",
    "latigueares": "latiguear",
    "latiguearon": "latiguear",
    "latigueará": "latiguear",
    "latiguearán": "latiguear",
    "latiguearás": "latiguear",
    "latiguearé": "latiguear",
    "latiguearéis": "latiguear",
    "latiguearía": "latiguear",
    "latiguearíais": "latiguear",
    "latiguearíamos": "latiguear",
    "latiguearían": "latiguear",
    "latiguearías": "latiguear",
    "latigueas": "latiguear",
    "latiguease": "latiguear",
    "latigueaseis": "latiguear",
    "latigueasen": "latiguear",
    "latigueases": "latiguear",
    "latigueaste": "latiguear",
    "latigueasteis": "latiguear",
    "latiguee": "latiguear",
    "latigueemos": "latiguear",
    "latigueen": "latiguear",
    "latiguees": "latiguear",
    "latigueo": "latiguear",
    "latigueos": "latigueo",
    "latiguera": "latiguero",
    "latigueras": "latiguero",
    "latigueros": "latiguero",
    "latigueábamos": "latiguear",
    "latigueáis": "latiguear",
    "latigueáramos": "latiguear",
    "latigueáremos": "latiguear",
    "latigueásemos": "latiguear",
    "latigueé": "latiguear",
    "latigueéis": "latiguear",
    "latigueó": "latiguear",
    "latiguillos": "latiguillo",
    "latimos": "latir",
    "latina": "latino",
    "latinaba": "latinar",
    "latinabais": "latinar",
    "latinaban": "latinar",
    "latinabas": "latinar",
    "latinad": "latinar",
    "latinada": "latinar",
    "latinadas": "latinar",
    "latinado": "latinar",
    "latinados": "latinar",
    "latinajos": "latinajo",
    "latinamos": "latinar",
    "latinan": "latinar",
    "latinando": "latinar",
    "latinara": "latinar",
    "latinarais": "latinar",
    "latinaran": "latinar",
    "latinaras": "latinar",
    "latinare": "latinar",
    "latinareis": "latinar",
    "latinaremos": "latinar",
    "latinaren": "latinar",
    "latinares": "latinar",
    "latinaron": "latinar",
    "latinará": "latinar",
    "latinarán": "latinar",
    "latinarás": "latinar",
    "latinaré": "latinar",
    "latinaréis": "latinar",
    "latinaría": "latinar",
    "latinaríais": "latinar",
    "latinaríamos": "latinar",
    "latinarían": "latinar",
    "latinarías": "latinar",
    "latinas": "latino",
    "latinase": "latinar",
    "latinaseis": "latinar",
    "latinasen": "latinar",
    "latinases": "latinar",
    "latinaste": "latinar",
    "latinasteis": "latinar",
    "latine": "latinar",
    "latinea": "latinear",
    "latineaba": "latinear",
    "latineabais": "latinear",
    "latineaban": "latinear",
    "latineabas": "latinear",
    "latinead": "latinear",
    "latineada": "latinear",
    "latineadas": "latinear",
    "latineado": "latinear",
    "latineados": "latinear",
    "latineamos": "latinear",
    "latinean": "latinear",
    "latineando": "latinear",
    "latineara": "latinear",
    "latinearais": "latinear",
    "latinearan": "latinear",
    "latinearas": "latinear",
    "latineare": "latinear",
    "latineareis": "latinear",
    "latinearemos": "latinear",
    "latinearen": "latinear",
    "latineares": "latinear",
    "latinearon": "latinear",
    "latineará": "latinear",
    "latinearán": "latinear",
    "latinearás": "latinear",
    "latinearé": "latinear",
    "latinearéis": "latinear",
    "latinearía": "latinear",
    "latinearíais": "latinear",
    "latinearíamos": "latinear",
    "latinearían": "latinear",
    "latinearías": "latinear",
    "latineas": "latinear",
    "latinease": "latinear",
    "latineaseis": "latinear",
    "latineasen": "latinear",
    "latineases": "latinear",
    "latineaste": "latinear",
    "latineasteis": "latinear",
    "latinee": "latinear",
    "latineemos": "latinear",
    "latineen": "latinear",
    "latinees": "latinear",
    "latinemos": "latinar",
    "latinen": "latinar",
    "latineo": "latinear",
    "latines": "latinar",
    "latineábamos": "latinear",
    "latineáis": "latinear",
    "latineáramos": "latinear",
    "latineáremos": "latinear",
    "latineásemos": "latinear",
    "latineé": "latinear",
    "latineéis": "latinear",
    "latineó": "latinear",
    "latinice": "latinizar",
    "latinicemos": "latinizar",
    "latinicen": "latinizar",
    "latinices": "latinizar",
    "latinicé": "latinizar",
    "latinicéis": "latinizar",
    "latinidades": "latinidad",
    "latiniparlas": "latiniparla",
    "latinismos": "latinismo",
    "latinistas": "latinista",
    "latiniza": "latinizar",
    "latinizaba": "latinizar",
    "latinizabais": "latinizar",
    "latinizaban": "latinizar",
    "latinizabas": "latinizar",
    "latinizaciones": "latinización",
    "latinizad": "latinizar",
    "latinizada": "latinizar",
    "latinizadas": "latinizar",
    "latinizado": "latinizar",
    "latinizados": "latinizar",
    "latinizamos": "latinizar",
    "latinizan": "latinizar",
    "latinizando": "latinizar",
    "latinizara": "latinizar",
    "latinizarais": "latinizar",
    "latinizaran": "latinizar",
    "latinizaras": "latinizar",
    "latinizare": "latinizar",
    "latinizareis": "latinizar",
    "latinizaremos": "latinizar",
    "latinizaren": "latinizar",
    "latinizares": "latinizar",
    "latinizaron": "latinizar",
    "latinizará": "latinizar",
    "latinizarán": "latinizar",
    "latinizarás": "latinizar",
    "latinizaré": "latinizar",
    "latinizaréis": "latinizar",
    "latinizaría": "latinizar",
    "latinizaríais": "latinizar",
    "latinizaríamos": "latinizar",
    "latinizarían": "latinizar",
    "latinizarías": "latinizar",
    "latinizas": "latinizar",
    "latinizase": "latinizar",
    "latinizaseis": "latinizar",
    "latinizasen": "latinizar",
    "latinizases": "latinizar",
    "latinizaste": "latinizar",
    "latinizasteis": "latinizar",
    "latinizo": "latinizar",
    "latinizábamos": "latinizar",
    "latinizáis": "latinizar",
    "latinizáramos": "latinizar",
    "latinizáremos": "latinizar",
    "latinizásemos": "latinizar",
    "latinizó": "latinizar",
    "latino": "latinar",
    "latinoamericana": "latinoamericano",
    "latinoamericanas": "latinoamericano",
    "latinoamericanos": "latinoamericano",
    "latinos": "latino",
    "latinábamos": "latinar",
    "latináis": "latinar",
    "latináramos": "latinar",
    "latináremos": "latinar",
    "latinásemos": "latinar",
    "latiné": "latinar",
    "latinéis": "latinar",
    "latinó": "latinar",
    "latiremos": "latir",
    "latirismos": "latirismo",
    "latirá": "latir",
    "latirán": "latir",
    "latirás": "latir",
    "latiré": "latir",
    "latiréis": "latir",
    "latiría": "latir",
    "latiríais": "latir",
    "latiríamos": "latir",
    "latirían": "latir",
    "latirías": "latir",
    "latiste": "latir",
    "latisteis": "latir",
    "latitas": "latita",
    "latitudes": "latitud",
    "latitudinales": "latitudinal",
    "latitudinaria": "latitudinario",
    "latitudinarias": "latitudinario",
    "latitudinarios": "latitudinario",
    "latitudinarismos": "latitudinarismo",
    "latiéramos": "latir",
    "latiéremos": "latir",
    "latiésemos": "latir",
    "latió": "latir",
    "lato": "latir",
    "latonera": "latonero",
    "latoneras": "latonero",
    "latoneros": "latonero",
    "latonerías": "latonería",
    "latones": "latón",
    "latonia": "latonio",
    "latonias": "latonio",
    "latonios": "latonio",
    "latos": "lato",
    "latosa": "latoso",
    "latosas": "latoso",
    "latosos": "latoso",
    "latrocinios": "latrocinio",
    "latréutica": "latréutico",
    "latréuticas": "latréutico",
    "latréuticos": "latréutico",
    "latrías": "latría",
    "lats": "lat",
    "latvia": "latvio",
    "latvias": "latvio",
    "latvios": "latvio",
    "latáis": "latir",
    "latí": "latir",
    "latía": "latir",
    "latíais": "latir",
    "latíamos": "latir",
    "latían": "latir",
    "latías": "latir",
    "latís": "latir",
    "lauchas": "laucha",
    "lauda": "laudar",
    "laudaba": "laudar",
    "laudabais": "laudar",
    "laudaban": "laudar",
    "laudabas": "laudar",
    "laudables": "laudable",
    "laudad": "laudar",
    "laudada": "laudar",
    "laudadas": "laudar",
    "laudado": "laudar",
    "laudados": "laudar",
    "laudamos": "laudar",
    "laudan": "laudar",
    "laudando": "laudar",
    "laudaninas": "laudanina",
    "laudanosinas": "laudanosina",
    "laudara": "laudar",
    "laudarais": "laudar",
    "laudaran": "laudar",
    "laudaras": "laudar",
    "laudare": "laudar",
    "laudareis": "laudar",
    "laudaremos": "laudar",
    "laudaren": "laudar",
    "laudares": "laudar",
    "laudaron": "laudar",
    "laudará": "laudar",
    "laudarán": "laudar",
    "laudarás": "laudar",
    "laudaré": "laudar",
    "laudaréis": "laudar",
    "laudaría": "laudar",
    "laudaríais": "laudar",
    "laudaríamos": "laudar",
    "laudarían": "laudar",
    "laudarías": "laudar",
    "laudas": "laudar",
    "laudase": "laudar",
    "laudaseis": "laudar",
    "laudasen": "laudar",
    "laudases": "laudar",
    "laudaste": "laudar",
    "laudasteis": "laudar",
    "laudatoria": "laudatorio",
    "laudatorias": "laudatorio",
    "laudatorios": "laudatorio",
    "laude": "laudar",
    "laudemios": "laudemio",
    "laudemos": "laudar",
    "lauden": "laudar",
    "laudes": "laude",
    "laudo": "laudar",
    "laudos": "laudo",
    "laudábamos": "laudar",
    "laudáis": "laudar",
    "laudáramos": "laudar",
    "laudáremos": "laudar",
    "laudásemos": "laudar",
    "laudé": "laudar",
    "laudéis": "laudar",
    "laudó": "laudar",
    "lauegramas": "lauegrama",
    "laumontitas": "laumontita",
    "launas": "launa",
    "lauras": "laura",
    "lauratos": "laurato",
    "laurea": "laurear",
    "laureaba": "laurear",
    "laureabais": "laurear",
    "laureaban": "laurear",
    "laureabas": "laurear",
    "lauread": "laurear",
    "laureada": "laurear",
    "laureadas": "laurear",
    "laureado": "laurear",
    "laureados": "laurear",
    "laureamos": "laurear",
    "laurean": "laurear",
    "laureando": "laurear",
    "laureara": "laurear",
    "laurearais": "laurear",
    "laurearan": "laurear",
    "laurearas": "laurear",
    "laureare": "laurear",
    "laureareis": "laurear",
    "laurearemos": "laurear",
    "laurearen": "laurear",
    "laureares": "laurear",
    "laurearon": "laurear",
    "laureará": "laurear",
    "laurearán": "laurear",
    "laurearás": "laurear",
    "laurearé": "laurear",
    "laurearéis": "laurear",
    "laurearía": "laurear",
    "laurearíais": "laurear",
    "laurearíamos": "laurear",
    "laurearían": "laurear",
    "laurearías": "laurear",
    "laureas": "laurear",
    "laurease": "laurear",
    "laureaseis": "laurear",
    "laureasen": "laurear",
    "laureases": "laurear",
    "laureaste": "laurear",
    "laureasteis": "laurear",
    "lauredales": "lauredal",
    "lauree": "laurear",
    "laureemos": "laurear",
    "laureen": "laurear",
    "laurees": "laurear",
    "laureles": "laurel",
    "laurencios": "laurencio",
    "laurentes": "laurente",
    "laureo": "laurear",
    "laureolas": "laureola",
    "laureábamos": "laurear",
    "laureáis": "laurear",
    "laureáramos": "laurear",
    "laureáremos": "laurear",
    "laureásemos": "laurear",
    "laureé": "laurear",
    "laureéis": "laurear",
    "laureó": "laurear",
    "laurina": "laurino",
    "laurinas": "laurino",
    "laurinos": "laurino",
    "laurocerasos": "lauroceraso",
    "lauroleatos": "lauroleato",
    "lauros": "lauro",
    "lauráceas": "laurácea",
    "lauréolas": "lauréola",
    "laurífera": "laurífero",
    "lauríferas": "laurífero",
    "lauríferos": "laurífero",
    "lauríneas": "laurínea",
    "lauta": "lauto",
    "lautas": "lauto",
    "lautos": "lauto",
    "lava": "lavar",
    "lavaba": "lavar",
    "lavabais": "lavar",
    "lavaban": "lavar",
    "lavabas": "lavar",
    "lavabilidades": "lavabilidad",
    "lavables": "lavable",
    "lavabos": "lavabo",
    "lavaciones": "lavación",
    "lavad": "lavar",
    "lavada": "lavar",
    "lavadas": "lavar",
    "lavaderos": "lavadero",
    "lavado": "lavar",
    "lavadora": "lavador",
    "lavadoras": "lavadora",
    "lavadores": "lavador",
    "lavados": "lavar",
    "lavaduras": "lavadura",
    "lavajes": "lavaje",
    "lavajos": "lavajo",
    "lavalunetas": "lavaluneta",
    "lavamientos": "lavamiento",
    "lavamos": "lavar",
    "lavan": "lavar",
    "lavancos": "lavanco",
    "lavandas": "lavanda",
    "lavanderas": "lavandera",
    "lavanderos": "lavandero",
    "lavanderías": "lavandería",
    "lavando": "lavar",
    "lavara": "lavar",
    "lavarais": "lavar",
    "lavaran": "lavar",
    "lavaras": "lavar",
    "lavare": "lavar",
    "lavareis": "lavar",
    "lavaremos": "lavar",
    "lavaren": "lavar",
    "lavares": "lavar",
    "lavaron": "lavar",
    "lavará": "lavar",
    "lavarán": "lavar",
    "lavarás": "lavar",
    "lavaré": "lavar",
    "lavaréis": "lavar",
    "lavaría": "lavar",
    "lavaríais": "lavar",
    "lavaríamos": "lavar",
    "lavarían": "lavar",
    "lavarías": "lavar",
    "lavas": "lavar",
    "lavase": "lavar",
    "lavaseis": "lavar",
    "lavasen": "lavar",
    "lavases": "lavar",
    "lavaste": "lavar",
    "lavasteis": "lavar",
    "lavativas": "lavativa",
    "lavatorios": "lavatorio",
    "lave": "lavar",
    "lavemos": "lavar",
    "laven": "lavar",
    "laves": "lave",
    "lavo": "lavar",
    "lavotea": "lavotear",
    "lavoteaba": "lavotear",
    "lavoteabais": "lavotear",
    "lavoteaban": "lavotear",
    "lavoteabas": "lavotear",
    "lavotead": "lavotear",
    "lavoteada": "lavotear",
    "lavoteadas": "lavotear",
    "lavoteado": "lavotear",
    "lavoteados": "lavotear",
    "lavoteamos": "lavotear",
    "lavotean": "lavotear",
    "lavoteando": "lavotear",
    "lavoteara": "lavotear",
    "lavotearais": "lavotear",
    "lavotearan": "lavotear",
    "lavotearas": "lavotear",
    "lavoteare": "lavotear",
    "lavoteareis": "lavotear",
    "lavotearemos": "lavotear",
    "lavotearen": "lavotear",
    "lavoteares": "lavotear",
    "lavotearon": "lavotear",
    "lavoteará": "lavotear",
    "lavotearán": "lavotear",
    "lavotearás": "lavotear",
    "lavotearé": "lavotear",
    "lavotearéis": "lavotear",
    "lavotearía": "lavotear",
    "lavotearíais": "lavotear",
    "lavotearíamos": "lavotear",
    "lavotearían": "lavotear",
    "lavotearías": "lavotear",
    "lavoteas": "lavotear",
    "lavotease": "lavotear",
    "lavoteaseis": "lavotear",
    "lavoteasen": "lavotear",
    "lavoteases": "lavotear",
    "lavoteaste": "lavotear",
    "lavoteasteis": "lavotear",
    "lavotee": "lavotear",
    "lavoteemos": "lavotear",
    "lavoteen": "lavotear",
    "lavotees": "lavotear",
    "lavoteo": "lavotear",
    "lavoteos": "lavoteo",
    "lavoteábamos": "lavotear",
    "lavoteáis": "lavotear",
    "lavoteáramos": "lavotear",
    "lavoteáremos": "lavotear",
    "lavoteásemos": "lavotear",
    "lavoteé": "lavotear",
    "lavoteéis": "lavotear",
    "lavoteó": "lavotear",
    "lavábamos": "lavar",
    "laváis": "lavar",
    "lavándulas": "lavándula",
    "laváramos": "lavar",
    "laváremos": "lavar",
    "lavásemos": "lavar",
    "lavé": "lavar",
    "lavéis": "lavar",
    "lavó": "lavar",
    "lawrencios": "lawrencio",
    "lawsonitas": "lawsonita",
    "laxa": "laxo",
    "laxaba": "laxar",
    "laxabais": "laxar",
    "laxaban": "laxar",
    "laxabas": "laxar",
    "laxaciones": "laxación",
    "laxad": "laxar",
    "laxada": "laxar",
    "laxadas": "laxar",
    "laxado": "laxar",
    "laxados": "laxar",
    "laxamos": "laxar",
    "laxan": "laxar",
    "laxando": "laxar",
    "laxantes": "laxante",
    "laxara": "laxar",
    "laxarais": "laxar",
    "laxaran": "laxar",
    "laxaras": "laxar",
    "laxare": "laxar",
    "laxareis": "laxar",
    "laxaremos": "laxar",
    "laxaren": "laxar",
    "laxares": "laxar",
    "laxaron": "laxar",
    "laxará": "laxar",
    "laxarán": "laxar",
    "laxarás": "laxar",
    "laxaré": "laxar",
    "laxaréis": "laxar",
    "laxaría": "laxar",
    "laxaríais": "laxar",
    "laxaríamos": "laxar",
    "laxarían": "laxar",
    "laxarías": "laxar",
    "laxas": "laxo",
    "laxase": "laxar",
    "laxaseis": "laxar",
    "laxasen": "laxar",
    "laxases": "laxar",
    "laxaste": "laxar",
    "laxasteis": "laxar",
    "laxativa": "laxativo",
    "laxativas": "laxativo",
    "laxativos": "laxativo",
    "laxe": "laxar",
    "laxemos": "laxar",
    "laxen": "laxar",
    "laxes": "laxar",
    "laxismos": "laxismo",
    "laxistas": "laxista",
    "laxitudes": "laxitud",
    "laxo": "laxar",
    "laxos": "laxo",
    "laxábamos": "laxar",
    "laxáis": "laxar",
    "laxáramos": "laxar",
    "laxáremos": "laxar",
    "laxásemos": "laxar",
    "laxé": "laxar",
    "laxéis": "laxar",
    "laxó": "laxar",
    "laya": "layar",
    "layaba": "layar",
    "layabais": "layar",
    "layaban": "layar",
    "layabas": "layar",
    "layad": "layar",
    "layada": "layar",
    "layadas": "layar",
    "layado": "layar",
    "layadora": "layador",
    "layadoras": "layador",
    "layadores": "layador",
    "layados": "layar",
    "layamos": "layar",
    "layan": "layar",
    "layando": "layar",
    "layara": "layar",
    "layarais": "layar",
    "layaran": "layar",
    "layaras": "layar",
    "layare": "layar",
    "layareis": "layar",
    "layaremos": "layar",
    "layaren": "layar",
    "layares": "layar",
    "layaron": "layar",
    "layará": "layar",
    "layarán": "layar",
    "layarás": "layar",
    "layaré": "layar",
    "layaréis": "layar",
    "layaría": "layar",
    "layaríais": "layar",
    "layaríamos": "layar",
    "layarían": "layar",
    "layarías": "layar",
    "layas": "layar",
    "layase": "layar",
    "layaseis": "layar",
    "layasen": "layar",
    "layases": "layar",
    "layaste": "layar",
    "layasteis": "layar",
    "laye": "layar",
    "layemos": "layar",
    "layen": "layar",
    "layes": "layar",
    "layetana": "layetano",
    "layetanas": "layetano",
    "layetanos": "layetano",
    "layo": "layar",
    "lays": "lay",
    "layábamos": "layar",
    "layáis": "layar",
    "layáramos": "layar",
    "layáremos": "layar",
    "layásemos": "layar",
    "layé": "layar",
    "layéis": "layar",
    "layó": "layar",
    "laza": "lazar",
    "lazaba": "lazar",
    "lazabais": "lazar",
    "lazaban": "lazar",
    "lazabas": "lazar",
    "lazad": "lazar",
    "lazada": "lazar",
    "lazadas": "lazar",
    "lazado": "lazar",
    "lazados": "lazar",
    "lazamos": "lazar",
    "lazan": "lazar",
    "lazando": "lazar",
    "lazara": "lazar",
    "lazarais": "lazar",
    "lazaran": "lazar",
    "lazaras": "lazar",
    "lazare": "lazar",
    "lazareis": "lazar",
    "lazaremos": "lazar",
    "lazaren": "lazar",
    "lazares": "lazar",
    "lazaretos": "lazareto",
    "lazarillos": "lazarillo",
    "lazarina": "lazarino",
    "lazarinas": "lazarino",
    "lazarinos": "lazarino",
    "lazaristas": "lazarista",
    "lazaron": "lazar",
    "lazarosa": "lazaroso",
    "lazarosas": "lazaroso",
    "lazarosos": "lazaroso",
    "lazará": "lazar",
    "lazarán": "lazar",
    "lazarás": "lazar",
    "lazaré": "lazar",
    "lazaréis": "lazar",
    "lazaría": "lazar",
    "lazaríais": "lazar",
    "lazaríamos": "lazar",
    "lazarían": "lazar",
    "lazarías": "lazar",
    "lazas": "lazar",
    "lazase": "lazar",
    "lazaseis": "lazar",
    "lazasen": "lazar",
    "lazases": "lazar",
    "lazaste": "lazar",
    "lazasteis": "lazar",
    "lazo": "lazar",
    "lazos": "lazo",
    "lazulitas": "lazulita",
    "lazuritas": "lazurita",
    "lazábamos": "lazar",
    "lazáis": "lazar",
    "lazáramos": "lazar",
    "lazáremos": "lazar",
    "lazásemos": "lazar",
    "lazó": "lazar",
    "laísmos": "laísmo",
    "laístas": "laísta",
    "laña": "lañar",
    "lañaba": "lañar",
    "lañabais": "lañar",
    "lañaban": "lañar",
    "lañabas": "lañar",
    "lañad": "lañar",
    "lañada": "lañar",
    "lañadas": "lañar",
    "lañado": "lañar",
    "lañadores": "lañador",
    "lañados": "lañar",
    "lañamos": "lañar",
    "lañan": "lañar",
    "lañando": "lañar",
    "lañara": "lañar",
    "lañarais": "lañar",
    "lañaran": "lañar",
    "lañaras": "lañar",
    "lañare": "lañar",
    "lañareis": "lañar",
    "lañaremos": "lañar",
    "lañaren": "lañar",
    "lañares": "lañar",
    "lañaron": "lañar",
    "lañará": "lañar",
    "lañarán": "lañar",
    "lañarás": "lañar",
    "lañaré": "lañar",
    "lañaréis": "lañar",
    "lañaría": "lañar",
    "lañaríais": "lañar",
    "lañaríamos": "lañar",
    "lañarían": "lañar",
    "lañarías": "lañar",
    "lañas": "lañar",
    "lañase": "lañar",
    "lañaseis": "lañar",
    "lañasen": "lañar",
    "lañases": "lañar",
    "lañaste": "lañar",
    "lañasteis": "lañar",
    "lañe": "lañar",
    "lañemos": "lañar",
    "lañen": "lañar",
    "lañes": "lañar",
    "laño": "lañar",
    "lañábamos": "lañar",
    "lañáis": "lañar",
    "lañáramos": "lañar",
    "lañáremos": "lañar",
    "lañásemos": "lañar",
    "lañé": "lañar",
    "lañéis": "lañar",
    "lañó": "lañar",
    "laúdes": "laúd",
    "lea": "leer",
    "leadhillitas": "leadhillita",
    "leales": "leal",
    "lealistas": "lealista",
    "lealtades": "lealtad",
    "leamos": "leer",
    "lean": "leer",
    "leas": "leer",
    "leasings": "leasing",
    "lebeches": "lebeche",
    "lebradas": "lebrada",
    "lebrastones": "lebrastón",
    "lebratos": "lebrato",
    "lebrela": "lebrel",
    "lebrelas": "lebrel",
    "lebreles": "lebrel",
    "lebrera": "lebrero",
    "lebreras": "lebrero",
    "lebreros": "lebrero",
    "lebrillos": "lebrillo",
    "lebroncillos": "lebroncillo",
    "lebrones": "lebrón",
    "lebruna": "lebruno",
    "lebrunas": "lebruno",
    "lebrunos": "lebruno",
    "leccionarios": "leccionario",
    "lecciones": "lección",
    "leccionistas": "leccionista",
    "lechadas": "lechada",
    "lechales": "lechal",
    "lechares": "lechar",
    "lechas": "lecha",
    "lechatelieritas": "lechatelierita",
    "lechazas": "lechaza",
    "lechazos": "lechazo",
    "lechecillas": "lechecilla",
    "lechera": "lechero",
    "lecheras": "lechero",
    "lecheros": "lechero",
    "lecherías": "lechería",
    "leches": "leche",
    "lechetreznas": "lechetrezna",
    "lechigadas": "lechigada",
    "lechina": "lechín",
    "lechinas": "lechín",
    "lechines": "lechín",
    "lechinos": "lechino",
    "lechona": "lechón",
    "lechonas": "lechón",
    "lechones": "lechón",
    "lechos": "lecho",
    "lechosa": "lechoso",
    "lechosas": "lechoso",
    "lechosidades": "lechosidad",
    "lechosos": "lechoso",
    "lechugada": "lechugado",
    "lechugadas": "lechugado",
    "lechugados": "lechugado",
    "lechugas": "lechuga",
    "lechuguillas": "lechuguilla",
    "lechuguina": "lechuguino",
    "lechuguinas": "lechuguino",
    "lechuguinos": "lechuguino",
    "lechuza": "lechuzo",
    "lechuzas": "lechuzo",
    "lechuzos": "lechuzo",
    "lecitinas": "lecitina",
    "lectinas": "lectina",
    "lectisternios": "lectisternio",
    "lectiva": "lectivo",
    "lectivas": "lectivo",
    "lectivos": "lectivo",
    "lectora": "lector",
    "lectorados": "lectorado",
    "lectoras": "lector",
    "lectores": "lector",
    "lectorías": "lectoría",
    "lecturas": "lectura",
    "leda": "ledo",
    "ledas": "ledo",
    "ledeburitas": "ledeburita",
    "ledes": "led",
    "ledos": "ledo",
    "leds": "led",
    "lee": "leer",
    "leed": "leer",
    "leemos": "leer",
    "leen": "leer",
    "leeremos": "leer",
    "leerá": "leer",
    "leerán": "leer",
    "leerás": "leer",
    "leeré": "leer",
    "leeréis": "leer",
    "leería": "leer",
    "leeríais": "leer",
    "leeríamos": "leer",
    "leerían": "leer",
    "leerías": "leer",
    "lees": "leer",
    "lega": "lego",
    "legaba": "legar",
    "legabais": "legar",
    "legaban": "legar",
    "legabas": "legar",
    "legaciones": "legación",
    "legacías": "legacía",
    "legad": "legar",
    "legada": "legar",
    "legadas": "legar",
    "legado": "legar",
    "legadores": "legador",
    "legados": "legar",
    "legaduras": "legadura",
    "legajos": "legajo",
    "legales": "legal",
    "legalice": "legalizar",
    "legalicemos": "legalizar",
    "legalicen": "legalizar",
    "legalices": "legalizar",
    "legalicé": "legalizar",
    "legalicéis": "legalizar",
    "legalidades": "legalidad",
    "legalismos": "legalismo",
    "legalistas": "legalista",
    "legaliza": "legalizar",
    "legalizaba": "legalizar",
    "legalizabais": "legalizar",
    "legalizaban": "legalizar",
    "legalizabas": "legalizar",
    "legalizaciones": "legalización",
    "legalizad": "legalizar",
    "legalizada": "legalizar",
    "legalizadas": "legalizar",
    "legalizado": "legalizar",
    "legalizados": "legalizar",
    "legalizamos": "legalizar",
    "legalizan": "legalizar",
    "legalizando": "legalizar",
    "legalizara": "legalizar",
    "legalizarais": "legalizar",
    "legalizaran": "legalizar",
    "legalizaras": "legalizar",
    "legalizare": "legalizar",
    "legalizareis": "legalizar",
    "legalizaremos": "legalizar",
    "legalizaren": "legalizar",
    "legalizares": "legalizar",
    "legalizaron": "legalizar",
    "legalizará": "legalizar",
    "legalizarán": "legalizar",
    "legalizarás": "legalizar",
    "legalizaré": "legalizar",
    "legalizaréis": "legalizar",
    "legalizaría": "legalizar",
    "legalizaríais": "legalizar",
    "legalizaríamos": "legalizar",
    "legalizarían": "legalizar",
    "legalizarías": "legalizar",
    "legalizas": "legalizar",
    "legalizase": "legalizar",
    "legalizaseis": "legalizar",
    "legalizasen": "legalizar",
    "legalizases": "legalizar",
    "legalizaste": "legalizar",
    "legalizasteis": "legalizar",
    "legalizo": "legalizar",
    "legalizábamos": "legalizar",
    "legalizáis": "legalizar",
    "legalizáramos": "legalizar",
    "legalizáremos": "legalizar",
    "legalizásemos": "legalizar",
    "legalizó": "legalizar",
    "legamos": "legar",
    "legamosa": "legamoso",
    "legamosas": "legamoso",
    "legamosos": "legamoso",
    "legan": "legar",
    "leganales": "leganal",
    "legando": "legar",
    "leganosa": "legañoso",
    "leganosas": "legañoso",
    "leganosos": "legañoso",
    "legara": "legar",
    "legarais": "legar",
    "legaran": "legar",
    "legaras": "legar",
    "legare": "legar",
    "legareis": "legar",
    "legaremos": "legar",
    "legaren": "legar",
    "legares": "legar",
    "legaron": "legar",
    "legará": "legar",
    "legarán": "legar",
    "legarás": "legar",
    "legaré": "legar",
    "legaréis": "legar",
    "legaría": "legar",
    "legaríais": "legar",
    "legaríamos": "legar",
    "legarían": "legar",
    "legarías": "legar",
    "legas": "lego",
    "legase": "legar",
    "legaseis": "legar",
    "legasen": "legar",
    "legases": "legar",
    "legaste": "legar",
    "legasteis": "legar",
    "legataria": "legatario",
    "legatarias": "legatario",
    "legatarios": "legatario",
    "legañas": "legaña",
    "legañosa": "legañoso",
    "legañosas": "legañoso",
    "legañosos": "legañoso",
    "legendaria": "legendario",
    "legendarias": "legendario",
    "legendarios": "legendario",
    "legendas": "legenda",
    "legibilidades": "legibilidad",
    "legibles": "legible",
    "legionaria": "legionario",
    "legionarias": "legionario",
    "legionarios": "legionario",
    "legionelas": "legionela",
    "legionellas": "legionella",
    "legionenses": "legionense",
    "legiones": "legión",
    "legisla": "legislar",
    "legislaba": "legislar",
    "legislabais": "legislar",
    "legislaban": "legislar",
    "legislabas": "legislar",
    "legislables": "legislable",
    "legislaciones": "legislación",
    "legislad": "legislar",
    "legislada": "legislar",
    "legisladas": "legislar",
    "legislado": "legislar",
    "legisladora": "legislador",
    "legisladoras": "legislador",
    "legisladores": "legislador",
    "legislados": "legislar",
    "legislamos": "legislar",
    "legislan": "legislar",
    "legislando": "legislar",
    "legislara": "legislar",
    "legislarais": "legislar",
    "legislaran": "legislar",
    "legislaras": "legislar",
    "legislare": "legislar",
    "legislareis": "legislar",
    "legislaremos": "legislar",
    "legislaren": "legislar",
    "legislares": "legislar",
    "legislaron": "legislar",
    "legislará": "legislar",
    "legislarán": "legislar",
    "legislarás": "legislar",
    "legislaré": "legislar",
    "legislaréis": "legislar",
    "legislaría": "legislar",
    "legislaríais": "legislar",
    "legislaríamos": "legislar",
    "legislarían": "legislar",
    "legislarías": "legislar",
    "legislas": "legislar",
    "legislase": "legislar",
    "legislaseis": "legislar",
    "legislasen": "legislar",
    "legislases": "legislar",
    "legislaste": "legislar",
    "legislasteis": "legislar",
    "legislativa": "legislativo",
    "legislativas": "legislativo",
    "legislativos": "legislativo",
    "legislaturas": "legislatura",
    "legisle": "legislar",
    "legislemos": "legislar",
    "legislen": "legislar",
    "legisles": "legislar",
    "legislo": "legislar",
    "legislábamos": "legislar",
    "legisláis": "legislar",
    "legisláramos": "legislar",
    "legisláremos": "legislar",
    "legislásemos": "legislar",
    "legislé": "legislar",
    "legisléis": "legislar",
    "legisló": "legislar",
    "legisperitos": "legisperito",
    "legispéritos": "legispérito",
    "legistas": "legista",
    "legitima": "legitimar",
    "legitimaba": "legitimar",
    "legitimabais": "legitimar",
    "legitimaban": "legitimar",
    "legitimabas": "legitimar",
    "legitimacidades": "legitimacidad",
    "legitimaciones": "legitimación",
    "legitimad": "legitimar",
    "legitimada": "legitimar",
    "legitimadas": "legitimar",
    "legitimado": "legitimar",
    "legitimadora": "legitimador",
    "legitimadoras": "legitimador",
    "legitimadores": "legitimador",
    "legitimados": "legitimar",
    "legitimamos": "legitimar",
    "legitiman": "legitimar",
    "legitimando": "legitimar",
    "legitimara": "legitimar",
    "legitimarais": "legitimar",
    "legitimaran": "legitimar",
    "legitimaras": "legitimar",
    "legitimare": "legitimar",
    "legitimareis": "legitimar",
    "legitimaremos": "legitimar",
    "legitimaren": "legitimar",
    "legitimares": "legitimar",
    "legitimaria": "legitimario",
    "legitimarias": "legitimario",
    "legitimarios": "legitimario",
    "legitimaron": "legitimar",
    "legitimará": "legitimar",
    "legitimarán": "legitimar",
    "legitimarás": "legitimar",
    "legitimaré": "legitimar",
    "legitimaréis": "legitimar",
    "legitimaría": "legitimar",
    "legitimaríais": "legitimar",
    "legitimaríamos": "legitimar",
    "legitimarían": "legitimar",
    "legitimarías": "legitimar",
    "legitimas": "legitimar",
    "legitimase": "legitimar",
    "legitimaseis": "legitimar",
    "legitimasen": "legitimar",
    "legitimases": "legitimar",
    "legitimaste": "legitimar",
    "legitimasteis": "legitimar",
    "legitime": "legitimar",
    "legitimemos": "legitimar",
    "legitimen": "legitimar",
    "legitimes": "legitimar",
    "legitimidades": "legitimidad",
    "legitimistas": "legitimista",
    "legitimizaciones": "legitimización",
    "legitimo": "legitimar",
    "legitimábamos": "legitimar",
    "legitimáis": "legitimar",
    "legitimáramos": "legitimar",
    "legitimáremos": "legitimar",
    "legitimásemos": "legitimar",
    "legitimé": "legitimar",
    "legitiméis": "legitimar",
    "legitimó": "legitimar",
    "lego": "legar",
    "legones": "legón",
    "legos": "lego",
    "legra": "legrar",
    "legraba": "legrar",
    "legrabais": "legrar",
    "legraban": "legrar",
    "legrabas": "legrar",
    "legraciones": "legración",
    "legrad": "legrar",
    "legrada": "legrar",
    "legradas": "legrar",
    "legrado": "legrar",
    "legrados": "legrar",
    "legraduras": "legradura",
    "legramos": "legrar",
    "legran": "legrar",
    "legrando": "legrar",
    "legrara": "legrar",
    "legrarais": "legrar",
    "legraran": "legrar",
    "legraras": "legrar",
    "legrare": "legrar",
    "legrareis": "legrar",
    "legraremos": "legrar",
    "legraren": "legrar",
    "legrares": "legrar",
    "legraron": "legrar",
    "legrará": "legrar",
    "legrarán": "legrar",
    "legrarás": "legrar",
    "legraré": "legrar",
    "legraréis": "legrar",
    "legraría": "legrar",
    "legraríais": "legrar",
    "legraríamos": "legrar",
    "legrarían": "legrar",
    "legrarías": "legrar",
    "legras": "legrar",
    "legrase": "legrar",
    "legraseis": "legrar",
    "legrasen": "legrar",
    "legrases": "legrar",
    "legraste": "legrar",
    "legrasteis": "legrar",
    "legre": "legrar",
    "legremos": "legrar",
    "legren": "legrar",
    "legres": "legrar",
    "legro": "legrar",
    "legrábamos": "legrar",
    "legráis": "legrar",
    "legráramos": "legrar",
    "legráremos": "legrar",
    "legrásemos": "legrar",
    "legré": "legrar",
    "legréis": "legrar",
    "legró": "legrar",
    "leguaria": "leguario",
    "leguarias": "leguario",
    "leguarios": "leguario",
    "leguas": "legua",
    "legue": "legar",
    "leguemos": "legar",
    "leguen": "legar",
    "legues": "legar",
    "leguleya": "leguleyo",
    "leguleyas": "leguleyo",
    "leguleyos": "leguleyo",
    "legumbres": "legumbre",
    "leguminosa": "leguminoso",
    "leguminosas": "leguminoso",
    "leguminosos": "leguminoso",
    "legué": "legar",
    "leguéis": "legar",
    "legábamos": "legar",
    "legáis": "legar",
    "legáramos": "legar",
    "legáremos": "legar",
    "legásemos": "legar",
    "legítima": "legítimo",
    "legítimas": "legítimo",
    "legítimos": "legítimo",
    "legó": "legar",
    "legúminas": "legúmina",
    "lehendakaris": "lehendakari",
    "lei": "leu",
    "leilas": "leila",
    "leimas": "leima",
    "lejana": "lejano",
    "lejanas": "lejano",
    "lejanos": "lejano",
    "lejanías": "lejanía",
    "lejías": "lejía",
    "lejíos": "lejío",
    "lekes": "lek",
    "leks": "lek",
    "lekë": "lek",
    "lela": "lelo",
    "lelas": "lelo",
    "lelilíes": "lelilí",
    "lelos": "lelo",
    "lemanitas": "lemanita",
    "lemas": "lema",
    "lematizaciones": "lematización",
    "lematizadores": "lematizador",
    "lembarios": "lembario",
    "lemminges": "lemming",
    "lemmings": "lemming",
    "lemnia": "lemnio",
    "lemnias": "lemnio",
    "lemnios": "lemnio",
    "lemniscatas": "lemniscata",
    "lemniscos": "lemnisco",
    "lemnáceas": "lemnácea",
    "lemnícolas": "lemnícola",
    "lemosina": "lemosín",
    "lemosinas": "lemosín",
    "lemosines": "lemosín",
    "lempiras": "lempira",
    "lemúrida": "lemúrido",
    "lemúridas": "lemúrido",
    "lemúridos": "lemúrido",
    "lenares": "lenar",
    "lencera": "lencero",
    "lenceras": "lencero",
    "lenceros": "lencero",
    "lencerías": "lencería",
    "lendeles": "lendel",
    "lendreras": "lendrera",
    "lendreros": "lendrero",
    "lendrosa": "lendroso",
    "lendrosas": "lendroso",
    "lendrosos": "lendroso",
    "lenes": "lene",
    "lenguaces": "lenguaz",
    "lenguadinas": "lenguadina",
    "lenguados": "lenguado",
    "lenguajes": "lenguaje",
    "lenguaraces": "lenguaraz",
    "lenguaradas": "lenguarada",
    "lenguas": "lengua",
    "lengüetadas": "lengüetada",
    "lengüetas": "lengüeta",
    "lengüetazos": "lengüetazo",
    "lengüeterías": "lengüetería",
    "lengüetillas": "lengüetilla",
    "lengüicorta": "lengüicorto",
    "lengüicortas": "lengüicorto",
    "lengüicortos": "lengüicorto",
    "lengüilarga": "lengüilargo",
    "lengüilargas": "lengüilargo",
    "lengüilargos": "lengüilargo",
    "lenidades": "lenidad",
    "lenifica": "lenificar",
    "lenificaba": "lenificar",
    "lenificabais": "lenificar",
    "lenificaban": "lenificar",
    "lenificabas": "lenificar",
    "lenificaciones": "lenificación",
    "lenificad": "lenificar",
    "lenificada": "lenificar",
    "lenificadas": "lenificar",
    "lenificado": "lenificar",
    "lenificados": "lenificar",
    "lenificamos": "lenificar",
    "lenifican": "lenificar",
    "lenificando": "lenificar",
    "lenificara": "lenificar",
    "lenificarais": "lenificar",
    "lenificaran": "lenificar",
    "lenificaras": "lenificar",
    "lenificare": "lenificar",
    "lenificareis": "lenificar",
    "lenificaremos": "lenificar",
    "lenificaren": "lenificar",
    "lenificares": "lenificar",
    "lenificaron": "lenificar",
    "lenificará": "lenificar",
    "lenificarán": "lenificar",
    "lenificarás": "lenificar",
    "lenificaré": "lenificar",
    "lenificaréis": "lenificar",
    "lenificaría": "lenificar",
    "lenificaríais": "lenificar",
    "lenificaríamos": "lenificar",
    "lenificarían": "lenificar",
    "lenificarías": "lenificar",
    "lenificas": "lenificar",
    "lenificase": "lenificar",
    "lenificaseis": "lenificar",
    "lenificasen": "lenificar",
    "lenificases": "lenificar",
    "lenificaste": "lenificar",
    "lenificasteis": "lenificar",
    "lenificativa": "lenificativo",
    "lenificativas": "lenificativo",
    "lenificativos": "lenificativo",
    "lenifico": "lenificar",
    "lenificábamos": "lenificar",
    "lenificáis": "lenificar",
    "lenificáramos": "lenificar",
    "lenificáremos": "lenificar",
    "lenificásemos": "lenificar",
    "lenificó": "lenificar",
    "lenifique": "lenificar",
    "lenifiquemos": "lenificar",
    "lenifiquen": "lenificar",
    "lenifiques": "lenificar",
    "lenifiqué": "lenificar",
    "lenifiquéis": "lenificar",
    "leninismos": "leninismo",
    "leninistas": "leninista",
    "lenitiva": "lenitivo",
    "lenitivas": "lenitivo",
    "lenitivos": "lenitivo",
    "lenocinios": "lenocinio",
    "lenta": "lento",
    "lentas": "lento",
    "lentejares": "lentejar",
    "lentejas": "lenteja",
    "lentejuelas": "lentejuela",
    "lentes": "lente",
    "lentibulariáceas": "lentibulariácea",
    "lenticelas": "lenticela",
    "lenticulares": "lenticular",
    "lentigos": "lentigo",
    "lentillas": "lentilla",
    "lentiscales": "lentiscal",
    "lentiscos": "lentisco",
    "lentitudes": "lentitud",
    "lentores": "lentor",
    "lentos": "lento",
    "lentículas": "lentícula",
    "lenzuelos": "lenzuelo",
    "leo": "leer",
    "leona": "león",
    "leonada": "leonado",
    "leonadas": "leonado",
    "leonados": "leonado",
    "leonas": "león",
    "leoncillos": "leoncillo",
    "leoneras": "leonera",
    "leonerías": "leonería",
    "leones": "león",
    "leonesa": "leonés",
    "leonesas": "leonés",
    "leoneses": "leonés",
    "leonesismos": "leonesismo",
    "leonina": "leonino",
    "leoninas": "leonino",
    "leoninos": "leonino",
    "leonitas": "leonita",
    "leontopodios": "leontopodio",
    "leopardos": "leopardo",
    "leopoldinas": "leopoldina",
    "leotardos": "leotardo",
    "lepidinas": "lepidina",
    "lepidocrocitas": "lepidocrocita",
    "lepidocroítas": "lepidocroíta",
    "lepidolitas": "lepidolita",
    "lepidomelanas": "lepidomelana",
    "lepidópteros": "lepidóptero",
    "leporina": "leporino",
    "leporinas": "leporino",
    "leporinos": "leporino",
    "lepras": "lepra",
    "leprominas": "lepromina",
    "leprosa": "leproso",
    "leprosarios": "leprosario",
    "leprosas": "leproso",
    "leproserías": "leprosería",
    "leprosos": "leproso",
    "leptocitos": "leptocito",
    "leptocloritas": "leptoclorita",
    "leptocúrtica": "leptocúrtico",
    "leptocúrticas": "leptocúrtico",
    "leptocúrticos": "leptocúrtico",
    "leptodonta": "leptodonto",
    "leptodontas": "leptodonto",
    "leptodontos": "leptodonto",
    "leptoestafilina": "leptoestafilino",
    "leptoestafilinas": "leptoestafilino",
    "leptoestafilinos": "leptoestafilino",
    "leptomeninges": "leptomeninge",
    "leptomitáceas": "leptomitácea",
    "leptonemas": "leptonema",
    "leptones": "leptón",
    "leptoprosopias": "leptoprosopia",
    "leptoprosópica": "leptoprosópico",
    "leptoprosópicas": "leptoprosópico",
    "leptoprosópicos": "leptoprosópico",
    "leptorrina": "leptorrino",
    "leptorrinas": "leptorrino",
    "leptorrinos": "leptorrino",
    "leptotenas": "leptotena",
    "leptotenos": "leptoteno",
    "leptómetros": "leptómetro",
    "lerchas": "lercha",
    "lerda": "lerdo",
    "lerdas": "lerdo",
    "lerdones": "lerdón",
    "lerdos": "lerdo",
    "lerenses": "lerense",
    "leridana": "leridano",
    "leridanas": "leridano",
    "leridanos": "leridano",
    "lerrouxismos": "lerrouxismo",
    "lerrouxistas": "lerrouxista",
    "lesa": "leso",
    "lesas": "leso",
    "lesbia": "lesbio",
    "lesbiana": "lesbiano",
    "lesbianas": "lesbiano",
    "lesbianismos": "lesbianismo",
    "lesbianos": "lesbiano",
    "lesbias": "lesbio",
    "lesbios": "lesbio",
    "lesiona": "lesionar",
    "lesionaba": "lesionar",
    "lesionabais": "lesionar",
    "lesionaban": "lesionar",
    "lesionabas": "lesionar",
    "lesionad": "lesionar",
    "lesionada": "lesionar",
    "lesionadas": "lesionar",
    "lesionado": "lesionar",
    "lesionados": "lesionar",
    "lesionamos": "lesionar",
    "lesionan": "lesionar",
    "lesionando": "lesionar",
    "lesionara": "lesionar",
    "lesionarais": "lesionar",
    "lesionaran": "lesionar",
    "lesionaras": "lesionar",
    "lesionare": "lesionar",
    "lesionareis": "lesionar",
    "lesionaremos": "lesionar",
    "lesionaren": "lesionar",
    "lesionares": "lesionar",
    "lesionaron": "lesionar",
    "lesionará": "lesionar",
    "lesionarán": "lesionar",
    "lesionarás": "lesionar",
    "lesionaré": "lesionar",
    "lesionaréis": "lesionar",
    "lesionaría": "lesionar",
    "lesionaríais": "lesionar",
    "lesionaríamos": "lesionar",
    "lesionarían": "lesionar",
    "lesionarías": "lesionar",
    "lesionas": "lesionar",
    "lesionase": "lesionar",
    "lesionaseis": "lesionar",
    "lesionasen": "lesionar",
    "lesionases": "lesionar",
    "lesionaste": "lesionar",
    "lesionasteis": "lesionar",
    "lesione": "lesionar",
    "lesionemos": "lesionar",
    "lesionen": "lesionar",
    "lesiones": "lesionar",
    "lesiono": "lesionar",
    "lesionábamos": "lesionar",
    "lesionáis": "lesionar",
    "lesionáramos": "lesionar",
    "lesionáremos": "lesionar",
    "lesionásemos": "lesionar",
    "lesioné": "lesionar",
    "lesionéis": "lesionar",
    "lesionó": "lesionar",
    "lesiva": "lesivo",
    "lesivas": "lesivo",
    "lesividades": "lesividad",
    "lesivos": "lesivo",
    "lesnas": "lesna",
    "lesos": "leso",
    "lestes": "leste",
    "lestodelfos": "lestodelfo",
    "lestrigona": "lestrigón",
    "lestrigonas": "lestrigón",
    "lestrigones": "lestrigón",
    "letales": "letal",
    "letames": "letame",
    "letanías": "letanía",
    "letargias": "letargia",
    "letargos": "letargo",
    "letargosa": "letargoso",
    "letargosas": "letargoso",
    "letargosos": "letargoso",
    "letea": "leteo",
    "leteas": "leteo",
    "leteos": "leteo",
    "letifica": "letificar",
    "letificaba": "letificar",
    "letificabais": "letificar",
    "letificaban": "letificar",
    "letificabas": "letificar",
    "letificad": "letificar",
    "letificada": "letificar",
    "letificadas": "letificar",
    "letificado": "letificar",
    "letificados": "letificar",
    "letificamos": "letificar",
    "letifican": "letificar",
    "letificando": "letificar",
    "letificara": "letificar",
    "letificarais": "letificar",
    "letificaran": "letificar",
    "letificaras": "letificar",
    "letificare": "letificar",
    "letificareis": "letificar",
    "letificaremos": "letificar",
    "letificaren": "letificar",
    "letificares": "letificar",
    "letificaron": "letificar",
    "letificará": "letificar",
    "letificarán": "letificar",
    "letificarás": "letificar",
    "letificaré": "letificar",
    "letificaréis": "letificar",
    "letificaría": "letificar",
    "letificaríais": "letificar",
    "letificaríamos": "letificar",
    "letificarían": "letificar",
    "letificarías": "letificar",
    "letificas": "letificar",
    "letificase": "letificar",
    "letificaseis": "letificar",
    "letificasen": "letificar",
    "letificases": "letificar",
    "letificaste": "letificar",
    "letificasteis": "letificar",
    "letifico": "letificar",
    "letificábamos": "letificar",
    "letificáis": "letificar",
    "letificáramos": "letificar",
    "letificáremos": "letificar",
    "letificásemos": "letificar",
    "letificó": "letificar",
    "letifique": "letificar",
    "letifiquemos": "letificar",
    "letifiquen": "letificar",
    "letifiques": "letificar",
    "letifiqué": "letificar",
    "letifiquéis": "letificar",
    "letona": "letón",
    "letonas": "letón",
    "letones": "letón",
    "letonia": "letonio",
    "letonias": "letonio",
    "letonios": "letonio",
    "letrada": "letrado",
    "letradas": "letrado",
    "letrados": "letrado",
    "letras": "letra",
    "letreros": "letrero",
    "letrillas": "letrilla",
    "letrinas": "letrina",
    "letristas": "letrista",
    "letárgica": "letárgico",
    "letárgicas": "letárgico",
    "letárgicos": "letárgico",
    "letífica": "letífico",
    "letíficas": "letífico",
    "letíficos": "letífico",
    "letóna": "letón",
    "letónas": "letón",
    "letónes": "letón",
    "leucemias": "leucemia",
    "leucinas": "leucina",
    "leucitas": "leucita",
    "leucoaglutininas": "leucoaglutinina",
    "leucoblastos": "leucoblasto",
    "leucoblástica": "leucoblástico",
    "leucoblásticas": "leucoblástico",
    "leucoblásticos": "leucoblástico",
    "leucocitaria": "leucocitario",
    "leucocitarias": "leucocitario",
    "leucocitarios": "leucocitario",
    "leucocitemias": "leucocitemia",
    "leucocitoides": "leucocitoide",
    "leucocitopenias": "leucocitopenia",
    "leucocitos": "leucocito",
    "leucocorias": "leucocoria",
    "leucocítica": "leucocítico",
    "leucocíticas": "leucocítico",
    "leucocíticos": "leucocítico",
    "leucodermias": "leucodermia",
    "leucodistrofias": "leucodistrofia",
    "leucodérmica": "leucodérmico",
    "leucodérmicas": "leucodérmico",
    "leucodérmicos": "leucodérmico",
    "leucoeritroblástica": "leucoeritroblástico",
    "leucoeritroblásticas": "leucoeritroblástico",
    "leucoeritroblásticos": "leucoeritroblástico",
    "leucomas": "leucoma",
    "leucopenias": "leucopenia",
    "leucoplaquias": "leucoplaquia",
    "leucoplasias": "leucoplasia",
    "leucoplastos": "leucoplasto",
    "leucorreas": "leucorrea",
    "leucosialinas": "leucosialina",
    "leucotrienos": "leucotrieno",
    "leucozafiros": "leucozafiro",
    "leucémica": "leucémico",
    "leucémicas": "leucémico",
    "leucémicos": "leucémico",
    "leucócrata": "leucócrato",
    "leucócratas": "leucócrato",
    "leucócratos": "leucócrato",
    "leuda": "leudo",
    "leudaba": "leudar",
    "leudabais": "leudar",
    "leudaban": "leudar",
    "leudabas": "leudar",
    "leudad": "leudar",
    "leudada": "leudar",
    "leudadas": "leudar",
    "leudado": "leudar",
    "leudados": "leudar",
    "leudamos": "leudar",
    "leudan": "leudar",
    "leudando": "leudar",
    "leudara": "leudar",
    "leudarais": "leudar",
    "leudaran": "leudar",
    "leudaras": "leudar",
    "leudare": "leudar",
    "leudareis": "leudar",
    "leudaremos": "leudar",
    "leudaren": "leudar",
    "leudares": "leudar",
    "leudaron": "leudar",
    "leudará": "leudar",
    "leudarán": "leudar",
    "leudarás": "leudar",
    "leudaré": "leudar",
    "leudaréis": "leudar",
    "leudaría": "leudar",
    "leudaríais": "leudar",
    "leudaríamos": "leudar",
    "leudarían": "leudar",
    "leudarías": "leudar",
    "leudas": "leudo",
    "leudase": "leudar",
    "leudaseis": "leudar",
    "leudasen": "leudar",
    "leudases": "leudar",
    "leudaste": "leudar",
    "leudasteis": "leudar",
    "leude": "leudar",
    "leudemos": "leudar",
    "leuden": "leudar",
    "leudes": "leude",
    "leudo": "leudar",
    "leudos": "leudo",
    "leudábamos": "leudar",
    "leudáis": "leudar",
    "leudáramos": "leudar",
    "leudáremos": "leudar",
    "leudásemos": "leudar",
    "leudé": "leudar",
    "leudéis": "leudar",
    "leudó": "leudar",
    "leus": "leu",
    "lev": "leva",
    "leva": "levar",
    "levaba": "levar",
    "levabais": "levar",
    "levaban": "levar",
    "levabas": "levar",
    "levad": "levar",
    "levada": "levar",
    "levadas": "levar",
    "levadera": "levadero",
    "levaderas": "levadero",
    "levaderos": "levadero",
    "levadiza": "levadizo",
    "levadizas": "levadizo",
    "levadizos": "levadizo",
    "levado": "levar",
    "levadora": "levador",
    "levadoras": "levador",
    "levadores": "levador",
    "levados": "levar",
    "levaduras": "levadura",
    "levamos": "levar",
    "levan": "levar",
    "levando": "levar",
    "levanta": "levantar",
    "levantaba": "levantar",
    "levantabais": "levantar",
    "levantaban": "levantar",
    "levantabas": "levantar",
    "levantad": "levantar",
    "levantada": "levantar",
    "levantadas": "levantar",
    "levantado": "levantar",
    "levantadora": "levantador",
    "levantadoras": "levantador",
    "levantadores": "levantador",
    "levantados": "levantar",
    "levantamientos": "levantamiento",
    "levantamos": "levantar",
    "levantan": "levantar",
    "levantando": "levantar",
    "levantara": "levantar",
    "levantarais": "levantar",
    "levantaran": "levantar",
    "levantaras": "levantar",
    "levantare": "levantar",
    "levantareis": "levantar",
    "levantaremos": "levantar",
    "levantaren": "levantar",
    "levantares": "levantar",
    "levantaron": "levantar",
    "levantará": "levantar",
    "levantarán": "levantar",
    "levantarás": "levantar",
    "levantaré": "levantar",
    "levantaréis": "levantar",
    "levantaría": "levantar",
    "levantaríais": "levantar",
    "levantaríamos": "levantar",
    "levantarían": "levantar",
    "levantarías": "levantar",
    "levantas": "levantar",
    "levantase": "levantar",
    "levantaseis": "levantar",
    "levantasen": "levantar",
    "levantases": "levantar",
    "levantaste": "levantar",
    "levantasteis": "levantar",
    "levante": "levantar",
    "levantemos": "levantar",
    "levanten": "levantar",
    "levantes": "levante",
    "levantina": "levantino",
    "levantinas": "levantino",
    "levantinos": "levantino",
    "levantisca": "levantisco",
    "levantiscas": "levantisco",
    "levantiscos": "levantisco",
    "levanto": "levantar",
    "levantábamos": "levantar",
    "levantáis": "levantar",
    "levantáramos": "levantar",
    "levantáremos": "levantar",
    "levantásemos": "levantar",
    "levanté": "levantar",
    "levantéis": "levantar",
    "levantó": "levantar",
    "levara": "levar",
    "levarais": "levar",
    "levaran": "levar",
    "levaras": "levar",
    "levare": "levar",
    "levareis": "levar",
    "levaremos": "levar",
    "levaren": "levar",
    "levares": "levar",
    "levaron": "levar",
    "levará": "levar",
    "levarán": "levar",
    "levarás": "levar",
    "levaré": "levar",
    "levaréis": "levar",
    "levaría": "levar",
    "levaríais": "levar",
    "levaríamos": "levar",
    "levarían": "levar",
    "levarías": "levar",
    "levas": "levar",
    "levase": "levar",
    "levaseis": "levar",
    "levasen": "levar",
    "levases": "levar",
    "levaste": "levar",
    "levasteis": "levar",
    "leve": "levar",
    "levedades": "levedad",
    "levemos": "levar",
    "leven": "levar",
    "leventes": "levente",
    "leves": "leve",
    "leviatanes": "leviatán",
    "leviga": "levigar",
    "levigaba": "levigar",
    "levigabais": "levigar",
    "levigaban": "levigar",
    "levigabas": "levigar",
    "levigaciones": "levigación",
    "levigad": "levigar",
    "levigada": "levigar",
    "levigadas": "levigar",
    "levigado": "levigar",
    "levigados": "levigar",
    "levigamos": "levigar",
    "levigan": "levigar",
    "levigando": "levigar",
    "levigara": "levigar",
    "levigarais": "levigar",
    "levigaran": "levigar",
    "levigaras": "levigar",
    "levigare": "levigar",
    "levigareis": "levigar",
    "levigaremos": "levigar",
    "levigaren": "levigar",
    "levigares": "levigar",
    "levigaron": "levigar",
    "levigará": "levigar",
    "levigarán": "levigar",
    "levigarás": "levigar",
    "levigaré": "levigar",
    "levigaréis": "levigar",
    "levigaría": "levigar",
    "levigaríais": "levigar",
    "levigaríamos": "levigar",
    "levigarían": "levigar",
    "levigarías": "levigar",
    "levigas": "levigar",
    "levigase": "levigar",
    "levigaseis": "levigar",
    "levigasen": "levigar",
    "levigases": "levigar",
    "levigaste": "levigar",
    "levigasteis": "levigar",
    "levigo": "levigar",
    "levigue": "levigar",
    "leviguemos": "levigar",
    "leviguen": "levigar",
    "levigues": "levigar",
    "levigué": "levigar",
    "leviguéis": "levigar",
    "levigábamos": "levigar",
    "levigáis": "levigar",
    "levigáramos": "levigar",
    "levigáremos": "levigar",
    "levigásemos": "levigar",
    "levigó": "levigar",
    "leviratos": "levirato",
    "levirrostra": "levirrostro",
    "levirrostras": "levirrostro",
    "levirrostros": "levirrostro",
    "levitaciones": "levitación",
    "levitas": "levita",
    "levitones": "levitón",
    "levo": "levar",
    "levorrotatoria": "levorrotatorio",
    "levorrotatorias": "levorrotatorio",
    "levorrotatorios": "levorrotatorio",
    "levs": "leva",
    "levulinatos": "levulinato",
    "levulosas": "levulosa",
    "levábamos": "levar",
    "leváis": "levar",
    "leváramos": "levar",
    "leváremos": "levar",
    "levásemos": "levar",
    "levé": "levar",
    "levéis": "levar",
    "levítica": "levítico",
    "levíticas": "levítico",
    "levíticos": "levítico",
    "levó": "levar",
    "levógira": "levógiro",
    "levógiras": "levógiro",
    "levógiros": "levógiro",
    "levúlica": "levúlico",
    "levúlicas": "levúlico",
    "levúlicos": "levúlico",
    "lewisitas": "lewisita",
    "lexemas": "lexema",
    "lexemática": "lexemático",
    "lexemáticas": "lexemático",
    "lexemáticos": "lexemático",
    "lexiarcas": "lexiarca",
    "lexicales": "lexical",
    "lexicalice": "lexicalizar",
    "lexicalicemos": "lexicalizar",
    "lexicalicen": "lexicalizar",
    "lexicalices": "lexicalizar",
    "lexicalicé": "lexicalizar",
    "lexicalicéis": "lexicalizar",
    "lexicaliza": "lexicalizar",
    "lexicalizaba": "lexicalizar",
    "lexicalizabais": "lexicalizar",
    "lexicalizaban": "lexicalizar",
    "lexicalizabas": "lexicalizar",
    "lexicalizaciones": "lexicalización",
    "lexicalizad": "lexicalizar",
    "lexicalizada": "lexicalizar",
    "lexicalizadas": "lexicalizar",
    "lexicalizado": "lexicalizar",
    "lexicalizados": "lexicalizar",
    "lexicalizamos": "lexicalizar",
    "lexicalizan": "lexicalizar",
    "lexicalizando": "lexicalizar",
    "lexicalizara": "lexicalizar",
    "lexicalizarais": "lexicalizar",
    "lexicalizaran": "lexicalizar",
    "lexicalizaras": "lexicalizar",
    "lexicalizare": "lexicalizar",
    "lexicalizareis": "lexicalizar",
    "lexicalizaremos": "lexicalizar",
    "lexicalizaren": "lexicalizar",
    "lexicalizares": "lexicalizar",
    "lexicalizaron": "lexicalizar",
    "lexicalizará": "lexicalizar",
    "lexicalizarán": "lexicalizar",
    "lexicalizarás": "lexicalizar",
    "lexicalizaré": "lexicalizar",
    "lexicalizaréis": "lexicalizar",
    "lexicalizaría": "lexicalizar",
    "lexicalizaríais": "lexicalizar",
    "lexicalizaríamos": "lexicalizar",
    "lexicalizarían": "lexicalizar",
    "lexicalizarías": "lexicalizar",
    "lexicalizas": "lexicalizar",
    "lexicalizase": "lexicalizar",
    "lexicalizaseis": "lexicalizar",
    "lexicalizasen": "lexicalizar",
    "lexicalizases": "lexicalizar",
    "lexicalizaste": "lexicalizar",
    "lexicalizasteis": "lexicalizar",
    "lexicalizo": "lexicalizar",
    "lexicalizábamos": "lexicalizar",
    "lexicalizáis": "lexicalizar",
    "lexicalizáramos": "lexicalizar",
    "lexicalizáremos": "lexicalizar",
    "lexicalizásemos": "lexicalizar",
    "lexicalizó": "lexicalizar",
    "lexicografías": "lexicografía",
    "lexicográfica": "lexicográfico",
    "lexicográficas": "lexicográfico",
    "lexicográficos": "lexicográfico",
    "lexicologías": "lexicología",
    "lexicológica": "lexicológico",
    "lexicológicas": "lexicológico",
    "lexicológicos": "lexicológico",
    "lexicometrías": "lexicometría",
    "lexicones": "lexicón",
    "lexicógrafa": "lexicógrafo",
    "lexicógrafas": "lexicógrafo",
    "lexicógrafos": "lexicógrafo",
    "lexicóloga": "lexicólogo",
    "lexicólogas": "lexicólogo",
    "lexicólogos": "lexicólogo",
    "leyendas": "leyenda",
    "leyendo": "leer",
    "leyera": "leer",
    "leyerais": "leer",
    "leyeran": "leer",
    "leyeras": "leer",
    "leyere": "leer",
    "leyereis": "leer",
    "leyeren": "leer",
    "leyeres": "leer",
    "leyeron": "leer",
    "leyes": "ley",
    "leyese": "leer",
    "leyeseis": "leer",
    "leyesen": "leer",
    "leyeses": "leer",
    "leyéramos": "leer",
    "leyéremos": "leer",
    "leyésemos": "leer",
    "leyó": "leer",
    "leznas": "lezna",
    "leáis": "leer",
    "leéis": "leer",
    "leí": "leer",
    "leía": "leer",
    "leíais": "leer",
    "leíamos": "leer",
    "leían": "leer",
    "leías": "leer",
    "leíbles": "leíble",
    "leída": "leer",
    "leídas": "leída",
    "leído": "leer",
    "leídos": "leer",
    "leímos": "leer",
    "leísmos": "leísmo",
    "leístas": "leísta",
    "leíste": "leer",
    "leísteis": "leer",
    "leñadora": "leñador",
    "leñadoras": "leñador",
    "leñadores": "leñador",
    "leñames": "leñame",
    "leñas": "leña",
    "leñatera": "leñatero",
    "leñateras": "leñatero",
    "leñateros": "leñatero",
    "leñazos": "leñazo",
    "leñera": "leñero",
    "leñeras": "leñero",
    "leñeros": "leñero",
    "leños": "leño",
    "leñosa": "leñoso",
    "leñosas": "leñoso",
    "leñosos": "leñoso",
    "liaba": "liar",
    "liabais": "liar",
    "liaban": "liar",
    "liabas": "liar",
    "liad": "liar",
    "liada": "liar",
    "liadas": "liar",
    "liado": "liar",
    "liados": "liar",
    "liaisons": "liaison",
    "liamos": "liar",
    "lianas": "liana",
    "liando": "liar",
    "liara": "liar",
    "liarais": "liar",
    "liaran": "liar",
    "liaras": "liar",
    "liare": "liar",
    "liareis": "liar",
    "liaremos": "liar",
    "liaren": "liar",
    "liares": "liar",
    "liaron": "liar",
    "liará": "liar",
    "liarán": "liar",
    "liarás": "liar",
    "liaré": "liar",
    "liaréis": "liar",
    "liaría": "liar",
    "liaríais": "liar",
    "liaríamos": "liar",
    "liarían": "liar",
    "liarías": "liar",
    "liasas": "liasa",
    "liase": "liar",
    "liaseis": "liar",
    "liasen": "liar",
    "liases": "liar",
    "liaste": "liar",
    "liasteis": "liar",
    "liazas": "liaza",
    "liba": "libar",
    "libaba": "libar",
    "libabais": "libar",
    "libaban": "libar",
    "libabas": "libar",
    "libaciones": "libación",
    "libad": "libar",
    "libada": "libar",
    "libadas": "libar",
    "libado": "libar",
    "libados": "libar",
    "libamientos": "libamiento",
    "libamos": "libar",
    "liban": "libar",
    "libando": "libar",
    "libanesa": "libanés",
    "libanesas": "libanés",
    "libaneses": "libanés",
    "libanésa": "libanés",
    "libanésas": "libanés",
    "libanéses": "libanés",
    "libara": "libar",
    "libarais": "libar",
    "libaran": "libar",
    "libaras": "libar",
    "libare": "libar",
    "libareis": "libar",
    "libaremos": "libar",
    "libaren": "libar",
    "libares": "libar",
    "libaron": "libar",
    "libará": "libar",
    "libarán": "libar",
    "libarás": "libar",
    "libaré": "libar",
    "libaréis": "libar",
    "libaría": "libar",
    "libaríais": "libar",
    "libaríamos": "libar",
    "libarían": "libar",
    "libarías": "libar",
    "libas": "libar",
    "libase": "libar",
    "libaseis": "libar",
    "libasen": "libar",
    "libases": "libar",
    "libaste": "libar",
    "libasteis": "libar",
    "libatorios": "libatorio",
    "libe": "libar",
    "libela": "libelar",
    "libelaba": "libelar",
    "libelabais": "libelar",
    "libelaban": "libelar",
    "libelabas": "libelar",
    "libelad": "libelar",
    "libelada": "libelar",
    "libeladas": "libelar",
    "libelado": "libelar",
    "libelados": "libelar",
    "libelamos": "libelar",
    "libelan": "libelar",
    "libelando": "libelar",
    "libelara": "libelar",
    "libelarais": "libelar",
    "libelaran": "libelar",
    "libelaras": "libelar",
    "libelare": "libelar",
    "libelareis": "libelar",
    "libelaremos": "libelar",
    "libelaren": "libelar",
    "libelares": "libelar",
    "libelaron": "libelar",
    "libelará": "libelar",
    "libelarán": "libelar",
    "libelarás": "libelar",
    "libelaré": "libelar",
    "libelaréis": "libelar",
    "libelaría": "libelar",
    "libelaríais": "libelar",
    "libelaríamos": "libelar",
    "libelarían": "libelar",
    "libelarías": "libelar",
    "libelas": "libelar",
    "libelase": "libelar",
    "libelaseis": "libelar",
    "libelasen": "libelar",
    "libelases": "libelar",
    "libelaste": "libelar",
    "libelasteis": "libelar",
    "libele": "libelar",
    "libelemos": "libelar",
    "libelen": "libelar",
    "libeles": "libelar",
    "libelistas": "libelista",
    "libelo": "libelar",
    "libelos": "libelo",
    "libelábamos": "libelar",
    "libeláis": "libelar",
    "libeláramos": "libelar",
    "libeláremos": "libelar",
    "libelásemos": "libelar",
    "libelé": "libelar",
    "libeléis": "libelar",
    "libeló": "libelar",
    "libemos": "libar",
    "liben": "libar",
    "libera": "liberar",
    "liberaba": "liberar",
    "liberabais": "liberar",
    "liberaban": "liberar",
    "liberabas": "liberar",
    "liberaciones": "liberación",
    "liberad": "liberar",
    "liberada": "liberar",
    "liberadas": "liberar",
    "liberado": "liberar",
    "liberadora": "liberador",
    "liberadoras": "liberador",
    "liberadores": "liberador",
    "liberados": "liberar",
    "liberaldemócratas": "liberaldemócrata",
    "liberales": "liberal",
    "liberalice": "liberalizar",
    "liberalicemos": "liberalizar",
    "liberalicen": "liberalizar",
    "liberalices": "liberalizar",
    "liberalicé": "liberalizar",
    "liberalicéis": "liberalizar",
    "liberalidades": "liberalidad",
    "liberalismos": "liberalismo",
    "liberaliza": "liberalizar",
    "liberalizaba": "liberalizar",
    "liberalizabais": "liberalizar",
    "liberalizaban": "liberalizar",
    "liberalizabas": "liberalizar",
    "liberalizaciones": "liberalización",
    "liberalizaciónes": "liberalización",
    "liberalizad": "liberalizar",
    "liberalizada": "liberalizar",
    "liberalizadas": "liberalizar",
    "liberalizado": "liberalizar",
    "liberalizadora": "liberalizador",
    "liberalizadoras": "liberalizador",
    "liberalizadores": "liberalizador",
    "liberalizados": "liberalizar",
    "liberalizamos": "liberalizar",
    "liberalizan": "liberalizar",
    "liberalizando": "liberalizar",
    "liberalizara": "liberalizar",
    "liberalizarais": "liberalizar",
    "liberalizaran": "liberalizar",
    "liberalizaras": "liberalizar",
    "liberalizare": "liberalizar",
    "liberalizareis": "liberalizar",
    "liberalizaremos": "liberalizar",
    "liberalizaren": "liberalizar",
    "liberalizares": "liberalizar",
    "liberalizaron": "liberalizar",
    "liberalizará": "liberalizar",
    "liberalizarán": "liberalizar",
    "liberalizarás": "liberalizar",
    "liberalizaré": "liberalizar",
    "liberalizaréis": "liberalizar",
    "liberalizaría": "liberalizar",
    "liberalizaríais": "liberalizar",
    "liberalizaríamos": "liberalizar",
    "liberalizarían": "liberalizar",
    "liberalizarías": "liberalizar",
    "liberalizas": "liberalizar",
    "liberalizase": "liberalizar",
    "liberalizaseis": "liberalizar",
    "liberalizasen": "liberalizar",
    "liberalizases": "liberalizar",
    "liberalizaste": "liberalizar",
    "liberalizasteis": "liberalizar",
    "liberalizo": "liberalizar",
    "liberalizábamos": "liberalizar",
    "liberalizáis": "liberalizar",
    "liberalizáramos": "liberalizar",
    "liberalizáremos": "liberalizar",
    "liberalizásemos": "liberalizar",
    "liberalizó": "liberalizar",
    "liberamos": "liberar",
    "liberan": "liberar",
    "liberando": "liberar",
    "liberara": "liberar",
    "liberarais": "liberar",
    "liberaran": "liberar",
    "liberaras": "liberar",
    "liberare": "liberar",
    "liberareis": "liberar",
    "liberaremos": "liberar",
    "liberaren": "liberar",
    "liberares": "liberar",
    "liberaron": "liberar",
    "liberará": "liberar",
    "liberarán": "liberar",
    "liberarás": "liberar",
    "liberaré": "liberar",
    "liberaréis": "liberar",
    "liberaría": "liberar",
    "liberaríais": "liberar",
    "liberaríamos": "liberar",
    "liberarían": "liberar",
    "liberarías": "liberar",
    "liberas": "liberar",
    "liberase": "liberar",
    "liberaseis": "liberar",
    "liberasen": "liberar",
    "liberases": "liberar",
    "liberaste": "liberar",
    "liberasteis": "liberar",
    "liberatoria": "liberatorio",
    "liberatorias": "liberatorio",
    "liberatorios": "liberatorio",
    "libere": "liberar",
    "liberemos": "liberar",
    "liberen": "liberar",
    "liberes": "liberar",
    "liberiana": "liberiano",
    "liberianas": "liberiano",
    "liberianos": "liberiano",
    "libero": "liberar",
    "liberta": "liberto",
    "libertaba": "libertar",
    "libertabais": "libertar",
    "libertaban": "libertar",
    "libertabas": "libertar",
    "libertad": "libertar",
    "libertada": "libertar",
    "libertadas": "libertar",
    "libertades": "libertad",
    "libertado": "libertar",
    "libertadora": "libertador",
    "libertadoras": "libertador",
    "libertadores": "libertador",
    "libertados": "libertar",
    "libertamos": "libertar",
    "libertan": "libertar",
    "libertando": "libertar",
    "libertara": "libertar",
    "libertarais": "libertar",
    "libertaran": "libertar",
    "libertaras": "libertar",
    "libertare": "libertar",
    "libertareis": "libertar",
    "libertaremos": "libertar",
    "libertaren": "libertar",
    "libertares": "libertar",
    "libertaria": "libertario",
    "libertarias": "libertario",
    "libertarios": "libertario",
    "libertarismos": "libertarismo",
    "libertaron": "libertar",
    "libertará": "libertar",
    "libertarán": "libertar",
    "libertarás": "libertar",
    "libertaré": "libertar",
    "libertaréis": "libertar",
    "libertaría": "libertar",
    "libertaríais": "libertar",
    "libertaríamos": "libertar",
    "libertarían": "libertar",
    "libertarías": "libertar",
    "libertas": "liberto",
    "libertase": "libertar",
    "libertaseis": "libertar",
    "libertasen": "libertar",
    "libertases": "libertar",
    "libertaste": "libertar",
    "libertasteis": "libertar",
    "liberte": "libertar",
    "libertemos": "libertar",
    "liberten": "libertar",
    "libertes": "libertar",
    "liberticidas": "liberticida",
    "libertina": "libertino",
    "libertinajes": "libertinaje",
    "libertinas": "libertino",
    "libertinos": "libertino",
    "liberto": "libertar",
    "libertos": "liberto",
    "libertábamos": "libertar",
    "libertáis": "libertar",
    "libertáramos": "libertar",
    "libertáremos": "libertar",
    "libertásemos": "libertar",
    "liberté": "libertar",
    "libertéis": "libertar",
    "libertó": "libertar",
    "liberábamos": "liberar",
    "liberáis": "liberar",
    "liberáramos": "liberar",
    "liberáremos": "liberar",
    "liberásemos": "liberar",
    "liberé": "liberar",
    "liberéis": "liberar",
    "liberó": "liberar",
    "libes": "libar",
    "libia": "libio",
    "libias": "libio",
    "libidinosa": "libidinoso",
    "libidinosas": "libidinoso",
    "libidinosidades": "libidinosidad",
    "libidinosos": "libidinoso",
    "libidos": "libido",
    "libios": "libio",
    "libo": "libar",
    "libores": "libor",
    "libra": "librar",
    "libraba": "librar",
    "librabais": "librar",
    "libraban": "librar",
    "librabas": "librar",
    "libraciones": "libración",
    "librad": "librar",
    "librada": "librar",
    "libradas": "librar",
    "librado": "librar",
    "libradora": "librador",
    "libradoras": "librador",
    "libradores": "librador",
    "librados": "librar",
    "libramientos": "libramiento",
    "libramos": "librar",
    "libran": "librar",
    "librancistas": "librancista",
    "librando": "librar",
    "librantes": "librante",
    "libranzas": "libranza",
    "librara": "librar",
    "librarais": "librar",
    "libraran": "librar",
    "libraras": "librar",
    "librare": "librar",
    "librareis": "librar",
    "libraremos": "librar",
    "libraren": "librar",
    "librares": "librar",
    "libraron": "librar",
    "librará": "librar",
    "librarán": "librar",
    "librarás": "librar",
    "libraré": "librar",
    "libraréis": "librar",
    "libraría": "librar",
    "libraríais": "librar",
    "libraríamos": "librar",
    "librarían": "librar",
    "librarías": "librar",
    "libras": "librar",
    "librase": "librar",
    "libraseis": "librar",
    "librasen": "librar",
    "librases": "librar",
    "libraste": "librar",
    "librasteis": "librar",
    "libratorios": "libratorio",
    "librazos": "librazo",
    "libre": "librar",
    "libreas": "librea",
    "librecambios": "librecambio",
    "librecambismos": "librecambismo",
    "librecambistas": "librecambista",
    "libremos": "librar",
    "libren": "librar",
    "librepensadora": "librepensador",
    "librepensadoras": "librepensador",
    "librepensadores": "librepensador",
    "librepensamientos": "librepensamiento",
    "librera": "librero",
    "libreras": "librero",
    "libreriles": "libreril",
    "libreros": "librero",
    "librerías": "librería",
    "libres": "libre",
    "libresca": "libresco",
    "librescas": "libresco",
    "librescos": "libresco",
    "libretas": "libreta",
    "libretistas": "libretista",
    "libretos": "libreto",
    "librillos": "librillo",
    "libro": "librar",
    "libros": "libro",
    "librábamos": "librar",
    "libráis": "librar",
    "libráramos": "librar",
    "libráremos": "librar",
    "librásemos": "librar",
    "libré": "librar",
    "libréis": "librar",
    "libró": "librar",
    "libábamos": "libar",
    "libáis": "libar",
    "libámenes": "libamen",
    "libáramos": "libar",
    "libáremos": "libar",
    "libásemos": "libar",
    "libé": "libar",
    "libéis": "libar",
    "libélulas": "libélula",
    "libérrima": "libérrimo",
    "libérrimas": "libérrimo",
    "libérrimos": "libérrimo",
    "libídines": "libídine",
    "libó": "libar",
    "licantropías": "licantropía",
    "licaones": "licaón",
    "licencia": "licenciar",
    "licenciaba": "licenciar",
    "licenciabais": "licenciar",
    "licenciaban": "licenciar",
    "licenciabas": "licenciar",
    "licenciad": "licenciar",
    "licenciada": "licenciar",
    "licenciadas": "licenciar",
    "licenciado": "licenciar",
    "licenciados": "licenciar",
    "licenciamientos": "licenciamiento",
    "licenciamos": "licenciar",
    "licencian": "licenciar",
    "licenciando": "licenciar",
    "licenciara": "licenciar",
    "licenciarais": "licenciar",
    "licenciaran": "licenciar",
    "licenciaras": "licenciar",
    "licenciare": "licenciar",
    "licenciareis": "licenciar",
    "licenciaremos": "licenciar",
    "licenciaren": "licenciar",
    "licenciares": "licenciar",
    "licenciaron": "licenciar",
    "licenciará": "licenciar",
    "licenciarán": "licenciar",
    "licenciarás": "licenciar",
    "licenciaré": "licenciar",
    "licenciaréis": "licenciar",
    "licenciaría": "licenciar",
    "licenciaríais": "licenciar",
    "licenciaríamos": "licenciar",
    "licenciarían": "licenciar",
    "licenciarías": "licenciar",
    "licencias": "licenciar",
    "licenciase": "licenciar",
    "licenciaseis": "licenciar",
    "licenciasen": "licenciar",
    "licenciases": "licenciar",
    "licenciaste": "licenciar",
    "licenciasteis": "licenciar",
    "licenciataria": "licenciatario",
    "licenciatarias": "licenciatario",
    "licenciatarios": "licenciatario",
    "licenciaturas": "licenciatura",
    "licencie": "licenciar",
    "licenciemos": "licenciar",
    "licencien": "licenciar",
    "licencies": "licenciar",
    "licencio": "licenciar",
    "licenciosa": "licencioso",
    "licenciosas": "licencioso",
    "licenciosidades": "licenciosidad",
    "licenciosos": "licencioso",
    "licenciábamos": "licenciar",
    "licenciáis": "licenciar",
    "licenciáramos": "licenciar",
    "licenciáremos": "licenciar",
    "licenciásemos": "licenciar",
    "licencié": "licenciar",
    "licenciéis": "licenciar",
    "licenció": "licenciar",
    "liceos": "liceo",
    "liceístas": "liceísta",
    "liceística": "liceístico",
    "liceísticas": "liceístico",
    "liceísticos": "liceístico",
    "lichis": "lichi",
    "licia": "licio",
    "licias": "licio",
    "licios": "licio",
    "licita": "licitar",
    "licitaba": "licitar",
    "licitabais": "licitar",
    "licitaban": "licitar",
    "licitabas": "licitar",
    "licitaciones": "licitación",
    "licitad": "licitar",
    "licitada": "licitar",
    "licitadas": "licitar",
    "licitado": "licitar",
    "licitadora": "licitador",
    "licitadoras": "licitador",
    "licitadores": "licitador",
    "licitados": "licitar",
    "licitamos": "licitar",
    "licitan": "licitar",
    "licitando": "licitar",
    "licitantes": "licitante",
    "licitara": "licitar",
    "licitarais": "licitar",
    "licitaran": "licitar",
    "licitaras": "licitar",
    "licitare": "licitar",
    "licitareis": "licitar",
    "licitaremos": "licitar",
    "licitaren": "licitar",
    "licitares": "licitar",
    "licitaron": "licitar",
    "licitará": "licitar",
    "licitarán": "licitar",
    "licitarás": "licitar",
    "licitaré": "licitar",
    "licitaréis": "licitar",
    "licitaría": "licitar",
    "licitaríais": "licitar",
    "licitaríamos": "licitar",
    "licitarían": "licitar",
    "licitarías": "licitar",
    "licitas": "licitar",
    "licitase": "licitar",
    "licitaseis": "licitar",
    "licitasen": "licitar",
    "licitases": "licitar",
    "licitaste": "licitar",
    "licitasteis": "licitar",
    "licite": "licitar",
    "licitemos": "licitar",
    "liciten": "licitar",
    "licites": "licitar",
    "licito": "licitar",
    "licitudes": "licitud",
    "licitábamos": "licitar",
    "licitáis": "licitar",
    "licitáramos": "licitar",
    "licitáremos": "licitar",
    "licitásemos": "licitar",
    "licité": "licitar",
    "licitéis": "licitar",
    "licitó": "licitar",
    "licnobia": "licnobio",
    "licnobias": "licnobio",
    "licnobios": "licnobio",
    "licopodiales": "licopodial",
    "licopodios": "licopodio",
    "licoreras": "licorera",
    "licorerías": "licorería",
    "licores": "licor",
    "licoristas": "licorista",
    "licorosa": "licoroso",
    "licorosas": "licoroso",
    "licorosos": "licoroso",
    "licras": "licra",
    "lictidos": "lictido",
    "lictores": "lictor",
    "licuaba": "licuar",
    "licuabais": "licuar",
    "licuaban": "licuar",
    "licuabas": "licuar",
    "licuables": "licuable",
    "licuaciones": "licuación",
    "licuad": "licuar",
    "licuada": "licuar",
    "licuadas": "licuar",
    "licuado": "licuar",
    "licuadoras": "licuadora",
    "licuados": "licuar",
    "licuamos": "licuar",
    "licuando": "licuar",
    "licuara": "licuar",
    "licuarais": "licuar",
    "licuaran": "licuar",
    "licuaras": "licuar",
    "licuare": "licuar",
    "licuareis": "licuar",
    "licuaremos": "licuar",
    "licuaren": "licuar",
    "licuares": "licuar",
    "licuaron": "licuar",
    "licuará": "licuar",
    "licuarán": "licuar",
    "licuarás": "licuar",
    "licuaré": "licuar",
    "licuaréis": "licuar",
    "licuaría": "licuar",
    "licuaríais": "licuar",
    "licuaríamos": "licuar",
    "licuarían": "licuar",
    "licuarías": "licuar",
    "licuase": "licuar",
    "licuaseis": "licuar",
    "licuasen": "licuar",
    "licuases": "licuar",
    "licuaste": "licuar",
    "licuasteis": "licuar",
    "licuefacciones": "licuefacción",
    "licueface": "licuefacer",
    "licuefaced": "licuefacer",
    "licuefacemos": "licuefacer",
    "licuefacen": "licuefacer",
    "licuefaces": "licuefacer",
    "licuefaciendo": "licuefacer",
    "licuefactiva": "licuefactivo",
    "licuefactivas": "licuefactivo",
    "licuefactivos": "licuefactivo",
    "licuefactores": "licuefactor",
    "licuefacéis": "licuefacer",
    "licuefacía": "licuefacer",
    "licuefacíais": "licuefacer",
    "licuefacíamos": "licuefacer",
    "licuefacían": "licuefacer",
    "licuefacías": "licuefacer",
    "licuefaga": "licuefacer",
    "licuefagamos": "licuefacer",
    "licuefagan": "licuefacer",
    "licuefagas": "licuefacer",
    "licuefago": "licuefacer",
    "licuefagáis": "licuefacer",
    "licuefaremos": "licuefacer",
    "licuefará": "licuefacer",
    "licuefarán": "licuefacer",
    "licuefarás": "licuefacer",
    "licuefaré": "licuefacer",
    "licuefaréis": "licuefacer",
    "licuefaría": "licuefacer",
    "licuefaríais": "licuefacer",
    "licuefaríamos": "licuefacer",
    "licuefarían": "licuefacer",
    "licuefarías": "licuefacer",
    "licuefaz": "licuefacer",
    "licuefecha": "licuefacer",
    "licuefechas": "licuefacer",
    "licuefecho": "licuefacer",
    "licuefechos": "licuefacer",
    "licuefice": "licuefacer",
    "licueficiera": "licuefacer",
    "licueficierais": "licuefacer",
    "licueficieran": "licuefacer",
    "licueficieras": "licuefacer",
    "licueficiere": "licuefacer",
    "licueficiereis": "licuefacer",
    "licueficieren": "licuefacer",
    "licueficieres": "licuefacer",
    "licueficieron": "licuefacer",
    "licueficiese": "licuefacer",
    "licueficieseis": "licuefacer",
    "licueficiesen": "licuefacer",
    "licueficieses": "licuefacer",
    "licueficimos": "licuefacer",
    "licueficiste": "licuefacer",
    "licueficisteis": "licuefacer",
    "licueficiéramos": "licuefacer",
    "licueficiéremos": "licuefacer",
    "licueficiésemos": "licuefacer",
    "licuefizo": "licuefacer",
    "licuemos": "licuar",
    "licuescencias": "licuescencia",
    "licuescentes": "licuescente",
    "licuorreas": "licuorrea",
    "licurga": "licurgo",
    "licurgas": "licurgo",
    "licurgos": "licurgo",
    "licuábamos": "licuar",
    "licuáis": "licuar",
    "licuáramos": "licuar",
    "licuáremos": "licuar",
    "licuásemos": "licuar",
    "licué": "licuar",
    "licuéis": "licuar",
    "licuó": "licuar",
    "licántropa": "licántropo",
    "licántropas": "licántropo",
    "licántropos": "licántropo",
    "licúa": "licuar",
    "licúan": "licuar",
    "licúas": "licuar",
    "licúe": "licuar",
    "licúen": "licuar",
    "licúes": "licuar",
    "licúo": "licuar",
    "lidares": "lidar",
    "liddicoatitas": "liddicoatita",
    "lidera": "liderar",
    "lideraba": "liderar",
    "liderabais": "liderar",
    "lideraban": "liderar",
    "liderabas": "liderar",
    "liderad": "liderar",
    "liderada": "liderar",
    "lideradas": "liderar",
    "liderado": "liderar",
    "liderados": "liderar",
    "lideramos": "liderar",
    "lideran": "liderar",
    "liderando": "liderar",
    "liderara": "liderar",
    "liderarais": "liderar",
    "lideraran": "liderar",
    "lideraras": "liderar",
    "liderare": "liderar",
    "liderareis": "liderar",
    "lideraremos": "liderar",
    "lideraren": "liderar",
    "liderares": "liderar",
    "lideraron": "liderar",
    "liderará": "liderar",
    "liderarán": "liderar",
    "liderarás": "liderar",
    "lideraré": "liderar",
    "lideraréis": "liderar",
    "lideraría": "liderar",
    "lideraríais": "liderar",
    "lideraríamos": "liderar",
    "liderarían": "liderar",
    "liderarías": "liderar",
    "lideras": "liderar",
    "liderase": "liderar",
    "lideraseis": "liderar",
    "liderasen": "liderar",
    "liderases": "liderar",
    "lideraste": "liderar",
    "liderasteis": "liderar",
    "lideratos": "liderato",
    "liderazgos": "liderazgo",
    "lidere": "liderar",
    "lideremos": "liderar",
    "lideren": "liderar",
    "lideres": "liderar",
    "lidero": "liderar",
    "liderábamos": "liderar",
    "lideráis": "liderar",
    "lideráramos": "liderar",
    "lideráremos": "liderar",
    "liderásemos": "liderar",
    "lideré": "liderar",
    "lideréis": "liderar",
    "lideró": "liderar",
    "lides": "lid",
    "lidia": "lidio",
    "lidiaba": "lidiar",
    "lidiabais": "lidiar",
    "lidiaban": "lidiar",
    "lidiabas": "lidiar",
    "lidiables": "lidiable",
    "lidiad": "lidiar",
    "lidiada": "lidiar",
    "lidiadas": "lidiar",
    "lidiadera": "lidiadero",
    "lidiaderas": "lidiadero",
    "lidiaderos": "lidiadero",
    "lidiado": "lidiar",
    "lidiadora": "lidiador",
    "lidiadoras": "lidiador",
    "lidiadores": "lidiador",
    "lidiados": "lidiar",
    "lidiamos": "lidiar",
    "lidian": "lidiar",
    "lidiando": "lidiar",
    "lidiara": "lidiar",
    "lidiarais": "lidiar",
    "lidiaran": "lidiar",
    "lidiaras": "lidiar",
    "lidiare": "lidiar",
    "lidiareis": "lidiar",
    "lidiaremos": "lidiar",
    "lidiaren": "lidiar",
    "lidiares": "lidiar",
    "lidiaron": "lidiar",
    "lidiará": "lidiar",
    "lidiarán": "lidiar",
    "lidiarás": "lidiar",
    "lidiaré": "lidiar",
    "lidiaréis": "lidiar",
    "lidiaría": "lidiar",
    "lidiaríais": "lidiar",
    "lidiaríamos": "lidiar",
    "lidiarían": "lidiar",
    "lidiarías": "lidiar",
    "lidias": "lidio",
    "lidiase": "lidiar",
    "lidiaseis": "lidiar",
    "lidiasen": "lidiar",
    "lidiases": "lidiar",
    "lidiaste": "lidiar",
    "lidiasteis": "lidiar",
    "lidie": "lidiar",
    "lidiemos": "lidiar",
    "lidien": "lidiar",
    "lidies": "lidiar",
    "lidio": "lidiar",
    "lidios": "lidio",
    "liditas": "lidita",
    "lidiábamos": "lidiar",
    "lidiáis": "lidiar",
    "lidiáramos": "lidiar",
    "lidiáremos": "lidiar",
    "lidiásemos": "lidiar",
    "lidié": "lidiar",
    "lidiéis": "lidiar",
    "lidió": "lidiar",
    "lidos": "lido",
    "liebrastones": "liebrastón",
    "liebratones": "liebratón",
    "liebres": "liebre",
    "lieder": "lied",
    "liedes": "lied",
    "lieds": "lied",
    "liega": "liego",
    "liegas": "liego",
    "liegos": "liego",
    "liemos": "liar",
    "lienales": "lienal",
    "liendres": "liendre",
    "lienteras": "lientera",
    "lienterías": "lientería",
    "lientérica": "lientérico",
    "lientéricas": "lientérico",
    "lientéricos": "lientérico",
    "lienzas": "lienza",
    "lienzos": "lienzo",
    "lifta": "liftar",
    "liftaba": "liftar",
    "liftabais": "liftar",
    "liftaban": "liftar",
    "liftabas": "liftar",
    "liftad": "liftar",
    "liftada": "liftar",
    "liftadas": "liftar",
    "liftado": "liftar",
    "liftados": "liftar",
    "liftamos": "liftar",
    "liftan": "liftar",
    "liftando": "liftar",
    "liftara": "liftar",
    "liftarais": "liftar",
    "liftaran": "liftar",
    "liftaras": "liftar",
    "liftare": "liftar",
    "liftareis": "liftar",
    "liftaremos": "liftar",
    "liftaren": "liftar",
    "liftares": "liftar",
    "liftaron": "liftar",
    "liftará": "liftar",
    "liftarán": "liftar",
    "liftarás": "liftar",
    "liftaré": "liftar",
    "liftaréis": "liftar",
    "liftaría": "liftar",
    "liftaríais": "liftar",
    "liftaríamos": "liftar",
    "liftarían": "liftar",
    "liftarías": "liftar",
    "liftas": "liftar",
    "liftase": "liftar",
    "liftaseis": "liftar",
    "liftasen": "liftar",
    "liftases": "liftar",
    "liftaste": "liftar",
    "liftasteis": "liftar",
    "lifte": "liftar",
    "liftemos": "liftar",
    "liften": "liftar",
    "liftes": "liftar",
    "liftings": "lifting",
    "lifto": "liftar",
    "liftábamos": "liftar",
    "liftáis": "liftar",
    "liftáramos": "liftar",
    "liftáremos": "liftar",
    "liftásemos": "liftar",
    "lifté": "liftar",
    "liftéis": "liftar",
    "liftó": "liftar",
    "liga": "ligar",
    "ligaba": "ligar",
    "ligabais": "ligar",
    "ligaban": "ligar",
    "ligabas": "ligar",
    "ligad": "ligar",
    "ligada": "ligar",
    "ligadas": "ligar",
    "ligado": "ligar",
    "ligados": "ligar",
    "ligaduras": "ligadura",
    "ligamazas": "ligamaza",
    "ligamentos": "ligamento",
    "ligamentosa": "ligamentoso",
    "ligamentosas": "ligamentoso",
    "ligamentosos": "ligamentoso",
    "ligamientos": "ligamiento",
    "ligamos": "ligar",
    "ligan": "ligar",
    "ligando": "ligar",
    "ligandos": "ligando",
    "ligara": "ligar",
    "ligarais": "ligar",
    "ligaran": "ligar",
    "ligaras": "ligar",
    "ligare": "ligar",
    "ligareis": "ligar",
    "ligaremos": "ligar",
    "ligaren": "ligar",
    "ligares": "ligar",
    "ligaron": "ligar",
    "ligará": "ligar",
    "ligarán": "ligar",
    "ligarás": "ligar",
    "ligaré": "ligar",
    "ligaréis": "ligar",
    "ligaría": "ligar",
    "ligaríais": "ligar",
    "ligaríamos": "ligar",
    "ligarían": "ligar",
    "ligarías": "ligar",
    "ligas": "ligar",
    "ligasas": "ligasa",
    "ligase": "ligar",
    "ligaseis": "ligar",
    "ligasen": "ligar",
    "ligases": "ligar",
    "ligaste": "ligar",
    "ligasteis": "ligar",
    "ligazones": "ligazón",
    "ligera": "ligero",
    "ligeras": "ligero",
    "ligerezas": "ligereza",
    "ligeros": "ligero",
    "lights": "light",
    "lignaria": "lignario",
    "lignarias": "lignario",
    "lignarios": "lignario",
    "lignifica": "lignificar",
    "lignificaba": "lignificar",
    "lignificabais": "lignificar",
    "lignificaban": "lignificar",
    "lignificabas": "lignificar",
    "lignificaciones": "lignificación",
    "lignificad": "lignificar",
    "lignificada": "lignificar",
    "lignificadas": "lignificar",
    "lignificado": "lignificar",
    "lignificados": "lignificar",
    "lignificamos": "lignificar",
    "lignifican": "lignificar",
    "lignificando": "lignificar",
    "lignificara": "lignificar",
    "lignificarais": "lignificar",
    "lignificaran": "lignificar",
    "lignificaras": "lignificar",
    "lignificare": "lignificar",
    "lignificareis": "lignificar",
    "lignificaremos": "lignificar",
    "lignificaren": "lignificar",
    "lignificares": "lignificar",
    "lignificaron": "lignificar",
    "lignificará": "lignificar",
    "lignificarán": "lignificar",
    "lignificarás": "lignificar",
    "lignificaré": "lignificar",
    "lignificaréis": "lignificar",
    "lignificaría": "lignificar",
    "lignificaríais": "lignificar",
    "lignificaríamos": "lignificar",
    "lignificarían": "lignificar",
    "lignificarías": "lignificar",
    "lignificas": "lignificar",
    "lignificase": "lignificar",
    "lignificaseis": "lignificar",
    "lignificasen": "lignificar",
    "lignificases": "lignificar",
    "lignificaste": "lignificar",
    "lignificasteis": "lignificar",
    "lignifico": "lignificar",
    "lignificábamos": "lignificar",
    "lignificáis": "lignificar",
    "lignificáramos": "lignificar",
    "lignificáremos": "lignificar",
    "lignificásemos": "lignificar",
    "lignificó": "lignificar",
    "lignifique": "lignificar",
    "lignifiquemos": "lignificar",
    "lignifiquen": "lignificar",
    "lignifiques": "lignificar",
    "lignifiqué": "lignificar",
    "lignifiquéis": "lignificar",
    "ligninas": "lignina",
    "lignitos": "lignito",
    "lignocelulosas": "lignocelulosa",
    "lignoceratos": "lignocerato",
    "lignocerínica": "lignocerínico",
    "lignocerínicas": "lignocerínico",
    "lignocerínicos": "lignocerínico",
    "lignosulfatos": "lignosulfato",
    "lignícola": "lignícolo",
    "lignícolas": "lignícolo",
    "lignícolos": "lignícolo",
    "lignívora": "lignívoro",
    "lignívoras": "lignívoro",
    "lignívoros": "lignívoro",
    "ligo": "ligar",
    "ligona": "ligón",
    "ligonas": "ligón",
    "ligones": "ligón",
    "ligroínas": "ligroína",
    "ligue": "ligar",
    "liguemos": "ligar",
    "liguen": "ligar",
    "liguera": "liguero",
    "ligueras": "liguero",
    "ligueros": "liguero",
    "ligues": "ligue",
    "liguillas": "liguilla",
    "ligures": "ligur",
    "ligustrales": "ligustral",
    "ligué": "ligar",
    "liguéis": "ligar",
    "ligábamos": "ligar",
    "ligáis": "ligar",
    "ligámenes": "ligamen",
    "ligáramos": "ligar",
    "ligáremos": "ligar",
    "ligásemos": "ligar",
    "ligó": "ligar",
    "lija": "lijar",
    "lijaba": "lijar",
    "lijabais": "lijar",
    "lijaban": "lijar",
    "lijabas": "lijar",
    "lijad": "lijar",
    "lijada": "lijar",
    "lijadas": "lijar",
    "lijado": "lijar",
    "lijadoras": "lijadora",
    "lijados": "lijar",
    "lijamos": "lijar",
    "lijan": "lijar",
    "lijando": "lijar",
    "lijara": "lijar",
    "lijarais": "lijar",
    "lijaran": "lijar",
    "lijaras": "lijar",
    "lijare": "lijar",
    "lijareis": "lijar",
    "lijaremos": "lijar",
    "lijaren": "lijar",
    "lijares": "lijar",
    "lijaron": "lijar",
    "lijará": "lijar",
    "lijarán": "lijar",
    "lijarás": "lijar",
    "lijaré": "lijar",
    "lijaréis": "lijar",
    "lijaría": "lijar",
    "lijaríais": "lijar",
    "lijaríamos": "lijar",
    "lijarían": "lijar",
    "lijarías": "lijar",
    "lijas": "lijar",
    "lijase": "lijar",
    "lijaseis": "lijar",
    "lijasen": "lijar",
    "lijases": "lijar",
    "lijaste": "lijar",
    "lijasteis": "lijar",
    "lije": "lijar",
    "lijemos": "lijar",
    "lijen": "lijar",
    "lijes": "lijar",
    "lijo": "lijar",
    "lijábamos": "lijar",
    "lijáis": "lijar",
    "lijáramos": "lijar",
    "lijáremos": "lijar",
    "lijásemos": "lijar",
    "lijé": "lijar",
    "lijéis": "lijar",
    "lijó": "lijar",
    "lilailas": "lilaila",
    "lilangenis": "lilangeni",
    "lilaos": "lilao",
    "lilas": "lila",
    "lilifloras": "liliflora",
    "liliputienses": "liliputiense",
    "liliáceas": "liliácea",
    "lima": "limar",
    "limaba": "limar",
    "limabais": "limar",
    "limaban": "limar",
    "limabas": "limar",
    "limad": "limar",
    "limada": "limar",
    "limadas": "limar",
    "limado": "limar",
    "limadora": "limador",
    "limadoras": "limadora",
    "limadores": "limador",
    "limados": "limar",
    "limaduras": "limadura",
    "limallas": "limalla",
    "limamos": "limar",
    "liman": "limar",
    "limandas": "limanda",
    "limando": "limar",
    "limara": "limar",
    "limarais": "limar",
    "limaran": "limar",
    "limaras": "limar",
    "limare": "limar",
    "limareis": "limar",
    "limaremos": "limar",
    "limaren": "limar",
    "limares": "limar",
    "limaron": "limar",
    "limará": "limar",
    "limarán": "limar",
    "limarás": "limar",
    "limaré": "limar",
    "limaréis": "limar",
    "limaría": "limar",
    "limaríais": "limar",
    "limaríamos": "limar",
    "limarían": "limar",
    "limarías": "limar",
    "limas": "limar",
    "limase": "limar",
    "limaseis": "limar",
    "limasen": "limar",
    "limases": "limar",
    "limaste": "limar",
    "limasteis": "limar",
    "limatones": "limatón",
    "limazas": "limaza",
    "limazos": "limazo",
    "limbos": "limbo",
    "lime": "limar",
    "limemos": "limar",
    "limen": "limar",
    "limera": "limero",
    "limeras": "limero",
    "limeros": "limero",
    "limes": "limar",
    "limetas": "limeta",
    "limeña": "limeño",
    "limeñas": "limeño",
    "limeños": "limeño",
    "limfoblastos": "limfoblasto",
    "limfocitos": "limfocito",
    "liminales": "liminal",
    "liminares": "liminar",
    "limita": "limitar",
    "limitaba": "limitar",
    "limitabais": "limitar",
    "limitaban": "limitar",
    "limitabas": "limitar",
    "limitables": "limitable",
    "limitaciones": "limitación",
    "limitad": "limitar",
    "limitada": "limitar",
    "limitadas": "limitar",
    "limitado": "limitar",
    "limitadora": "limitador",
    "limitadoras": "limitador",
    "limitadores": "limitador",
    "limitados": "limitar",
    "limitamos": "limitar",
    "limitan": "limitar",
    "limitando": "limitar",
    "limitantes": "limitante",
    "limitara": "limitar",
    "limitarais": "limitar",
    "limitaran": "limitar",
    "limitaras": "limitar",
    "limitare": "limitar",
    "limitareis": "limitar",
    "limitaremos": "limitar",
    "limitaren": "limitar",
    "limitares": "limitar",
    "limitaron": "limitar",
    "limitará": "limitar",
    "limitarán": "limitar",
    "limitarás": "limitar",
    "limitaré": "limitar",
    "limitaréis": "limitar",
    "limitaría": "limitar",
    "limitaríais": "limitar",
    "limitaríamos": "limitar",
    "limitarían": "limitar",
    "limitarías": "limitar",
    "limitas": "limitar",
    "limitase": "limitar",
    "limitaseis": "limitar",
    "limitasen": "limitar",
    "limitases": "limitar",
    "limitaste": "limitar",
    "limitasteis": "limitar",
    "limitativa": "limitativo",
    "limitativas": "limitativo",
    "limitativos": "limitativo",
    "limite": "limitar",
    "limitemos": "limitar",
    "limiten": "limitar",
    "limites": "limitar",
    "limito": "limitar",
    "limitábamos": "limitar",
    "limitáis": "limitar",
    "limitánea": "limitáneo",
    "limitáneas": "limitáneo",
    "limitáneos": "limitáneo",
    "limitáramos": "limitar",
    "limitáremos": "limitar",
    "limitásemos": "limitar",
    "limité": "limitar",
    "limitéis": "limitar",
    "limitó": "limitar",
    "limnologías": "limnología",
    "limnológica": "limnológico",
    "limnológicas": "limnológico",
    "limnológicos": "limnológico",
    "limnícolas": "limnícola",
    "limnígrafos": "limnígrafo",
    "limnímetros": "limnímetro",
    "limo": "limar",
    "limolitas": "limolita",
    "limonada": "limonado",
    "limonadas": "limonado",
    "limonados": "limonado",
    "limonares": "limonar",
    "limonenos": "limoneno",
    "limonera": "limonero",
    "limoneras": "limonero",
    "limoneros": "limonero",
    "limones": "limón",
    "limonios": "limonio",
    "limonitas": "limonita",
    "limos": "limo",
    "limosa": "limoso",
    "limosas": "limoso",
    "limosnas": "limosna",
    "limosnea": "limosnear",
    "limosneaba": "limosnear",
    "limosneabais": "limosnear",
    "limosneaban": "limosnear",
    "limosneabas": "limosnear",
    "limosnead": "limosnear",
    "limosneada": "limosnear",
    "limosneadas": "limosnear",
    "limosneado": "limosnear",
    "limosneados": "limosnear",
    "limosneamos": "limosnear",
    "limosnean": "limosnear",
    "limosneando": "limosnear",
    "limosneara": "limosnear",
    "limosnearais": "limosnear",
    "limosnearan": "limosnear",
    "limosnearas": "limosnear",
    "limosneare": "limosnear",
    "limosneareis": "limosnear",
    "limosnearemos": "limosnear",
    "limosnearen": "limosnear",
    "limosneares": "limosnear",
    "limosnearon": "limosnear",
    "limosneará": "limosnear",
    "limosnearán": "limosnear",
    "limosnearás": "limosnear",
    "limosnearé": "limosnear",
    "limosnearéis": "limosnear",
    "limosnearía": "limosnear",
    "limosnearíais": "limosnear",
    "limosnearíamos": "limosnear",
    "limosnearían": "limosnear",
    "limosnearías": "limosnear",
    "limosneas": "limosnear",
    "limosnease": "limosnear",
    "limosneaseis": "limosnear",
    "limosneasen": "limosnear",
    "limosneases": "limosnear",
    "limosneaste": "limosnear",
    "limosneasteis": "limosnear",
    "limosnee": "limosnear",
    "limosneemos": "limosnear",
    "limosneen": "limosnear",
    "limosnees": "limosnear",
    "limosneo": "limosnear",
    "limosnera": "limosnero",
    "limosneras": "limosnero",
    "limosneros": "limosnero",
    "limosneábamos": "limosnear",
    "limosneáis": "limosnear",
    "limosneáramos": "limosnear",
    "limosneáremos": "limosnear",
    "limosneásemos": "limosnear",
    "limosneé": "limosnear",
    "limosneéis": "limosnear",
    "limosneó": "limosnear",
    "limosos": "limoso",
    "limpia": "limpio",
    "limpiaba": "limpiar",
    "limpiabais": "limpiar",
    "limpiaban": "limpiar",
    "limpiabas": "limpiar",
    "limpiables": "limpiable",
    "limpiad": "limpiar",
    "limpiada": "limpiar",
    "limpiadas": "limpiar",
    "limpiaderas": "limpiadera",
    "limpiado": "limpiar",
    "limpiadora": "limpiador",
    "limpiadoras": "limpiador",
    "limpiadores": "limpiador",
    "limpiados": "limpiar",
    "limpiaduras": "limpiadura",
    "limpiamos": "limpiar",
    "limpian": "limpiar",
    "limpiando": "limpiar",
    "limpiara": "limpiar",
    "limpiarais": "limpiar",
    "limpiaran": "limpiar",
    "limpiaras": "limpiar",
    "limpiare": "limpiar",
    "limpiareis": "limpiar",
    "limpiaremos": "limpiar",
    "limpiaren": "limpiar",
    "limpiares": "limpiar",
    "limpiaron": "limpiar",
    "limpiará": "limpiar",
    "limpiarán": "limpiar",
    "limpiarás": "limpiar",
    "limpiaré": "limpiar",
    "limpiaréis": "limpiar",
    "limpiaría": "limpiar",
    "limpiaríais": "limpiar",
    "limpiaríamos": "limpiar",
    "limpiarían": "limpiar",
    "limpiarías": "limpiar",
    "limpias": "limpio",
    "limpiase": "limpiar",
    "limpiaseis": "limpiar",
    "limpiasen": "limpiar",
    "limpiases": "limpiar",
    "limpiaste": "limpiar",
    "limpiasteis": "limpiar",
    "limpideces": "limpidez",
    "limpie": "limpiar",
    "limpiemos": "limpiar",
    "limpien": "limpiar",
    "limpies": "limpiar",
    "limpiezas": "limpieza",
    "limpio": "limpiar",
    "limpiones": "limpión",
    "limpios": "limpio",
    "limpiábamos": "limpiar",
    "limpiáis": "limpiar",
    "limpiáramos": "limpiar",
    "limpiáremos": "limpiar",
    "limpiásemos": "limpiar",
    "limpié": "limpiar",
    "limpiéis": "limpiar",
    "limpió": "limpiar",
    "limusinas": "limusina",
    "limábamos": "limar",
    "limáis": "limar",
    "limáramos": "limar",
    "limáremos": "limar",
    "limásemos": "limar",
    "limé": "limar",
    "liméis": "limar",
    "limícolas": "limícola",
    "limítrofes": "limítrofe",
    "limívora": "limívoro",
    "limívoras": "limívoro",
    "limívoros": "limívoro",
    "limó": "limar",
    "linajes": "linaje",
    "linajistas": "linajista",
    "linajuda": "linajudo",
    "linajudas": "linajudo",
    "linajudos": "linajudo",
    "linaloles": "linalol",
    "linares": "linar",
    "linazas": "linaza",
    "lincea": "lincear",
    "linceaba": "lincear",
    "linceabais": "lincear",
    "linceaban": "lincear",
    "linceabas": "lincear",
    "lincead": "lincear",
    "linceada": "lincear",
    "linceadas": "lincear",
    "linceado": "lincear",
    "linceados": "lincear",
    "linceamos": "lincear",
    "lincean": "lincear",
    "linceando": "lincear",
    "linceara": "lincear",
    "lincearais": "lincear",
    "lincearan": "lincear",
    "lincearas": "lincear",
    "linceare": "lincear",
    "linceareis": "lincear",
    "lincearemos": "lincear",
    "lincearen": "lincear",
    "linceares": "lincear",
    "lincearon": "lincear",
    "linceará": "lincear",
    "lincearán": "lincear",
    "lincearás": "lincear",
    "lincearé": "lincear",
    "lincearéis": "lincear",
    "lincearía": "lincear",
    "lincearíais": "lincear",
    "lincearíamos": "lincear",
    "lincearían": "lincear",
    "lincearías": "lincear",
    "linceas": "lincear",
    "lincease": "lincear",
    "linceaseis": "lincear",
    "linceasen": "lincear",
    "linceases": "lincear",
    "linceaste": "lincear",
    "linceasteis": "lincear",
    "lincee": "lincear",
    "linceemos": "lincear",
    "linceen": "lincear",
    "lincees": "lincear",
    "linceo": "lincear",
    "linces": "lince",
    "linceábamos": "lincear",
    "linceáis": "lincear",
    "linceáramos": "lincear",
    "linceáremos": "lincear",
    "linceásemos": "lincear",
    "linceé": "lincear",
    "linceéis": "lincear",
    "linceó": "lincear",
    "lincha": "linchar",
    "linchaba": "linchar",
    "linchabais": "linchar",
    "linchaban": "linchar",
    "linchabas": "linchar",
    "linchad": "linchar",
    "linchada": "linchar",
    "linchadas": "linchar",
    "linchado": "linchar",
    "linchadora": "linchador",
    "linchadoras": "linchador",
    "linchadores": "linchador",
    "linchados": "linchar",
    "linchamientos": "linchamiento",
    "linchamos": "linchar",
    "linchan": "linchar",
    "linchando": "linchar",
    "linchara": "linchar",
    "lincharais": "linchar",
    "lincharan": "linchar",
    "lincharas": "linchar",
    "linchare": "linchar",
    "linchareis": "linchar",
    "lincharemos": "linchar",
    "lincharen": "linchar",
    "linchares": "linchar",
    "lincharon": "linchar",
    "linchará": "linchar",
    "lincharán": "linchar",
    "lincharás": "linchar",
    "lincharé": "linchar",
    "lincharéis": "linchar",
    "lincharía": "linchar",
    "lincharíais": "linchar",
    "lincharíamos": "linchar",
    "lincharían": "linchar",
    "lincharías": "linchar",
    "linchas": "linchar",
    "linchase": "linchar",
    "linchaseis": "linchar",
    "linchasen": "linchar",
    "linchases": "linchar",
    "linchaste": "linchar",
    "linchasteis": "linchar",
    "linche": "linchar",
    "linchemos": "linchar",
    "linchen": "linchar",
    "linches": "linchar",
    "lincho": "linchar",
    "linchábamos": "linchar",
    "lincháis": "linchar",
    "lincháramos": "linchar",
    "lincháremos": "linchar",
    "linchásemos": "linchar",
    "linché": "linchar",
    "linchéis": "linchar",
    "linchó": "linchar",
    "linda": "lindo",
    "lindaba": "lindar",
    "lindabais": "lindar",
    "lindaban": "lindar",
    "lindabas": "lindar",
    "lindad": "lindar",
    "lindada": "lindar",
    "lindadas": "lindar",
    "lindado": "lindar",
    "lindados": "lindar",
    "lindamos": "lindar",
    "lindan": "lindar",
    "lindando": "lindar",
    "lindanes": "lindane",
    "lindantes": "lindante",
    "lindara": "lindar",
    "lindarais": "lindar",
    "lindaran": "lindar",
    "lindaras": "lindar",
    "lindare": "lindar",
    "lindareis": "lindar",
    "lindaremos": "lindar",
    "lindaren": "lindar",
    "lindares": "lindar",
    "lindaron": "lindar",
    "lindará": "lindar",
    "lindarán": "lindar",
    "lindarás": "lindar",
    "lindaré": "lindar",
    "lindaréis": "lindar",
    "lindaría": "lindar",
    "lindaríais": "lindar",
    "lindaríamos": "lindar",
    "lindarían": "lindar",
    "lindarías": "lindar",
    "lindas": "lindo",
    "lindase": "lindar",
    "lindaseis": "lindar",
    "lindasen": "lindar",
    "lindases": "lindar",
    "lindaste": "lindar",
    "lindasteis": "lindar",
    "lindazos": "lindazo",
    "linde": "lindar",
    "lindeles": "lindel",
    "lindemos": "lindar",
    "linden": "lindar",
    "lindera": "lindero",
    "linderas": "lindero",
    "linderos": "lindero",
    "lindes": "linde",
    "lindezas": "lindeza",
    "lindo": "lindar",
    "lindones": "lindón",
    "lindos": "lindo",
    "linduras": "lindura",
    "lindábamos": "lindar",
    "lindáis": "lindar",
    "lindáramos": "lindar",
    "lindáremos": "lindar",
    "lindásemos": "lindar",
    "lindé": "lindar",
    "lindéis": "lindar",
    "lindó": "lindar",
    "linea": "linear",
    "lineaba": "linear",
    "lineabais": "linear",
    "lineaban": "linear",
    "lineabas": "linear",
    "lineaciones": "lineación",
    "linead": "linear",
    "lineada": "linear",
    "lineadas": "linear",
    "lineado": "linear",
    "lineados": "linear",
    "lineajes": "lineaje",
    "lineales": "lineal",
    "linealidades": "linealidad",
    "linealizaciones": "linealización",
    "lineamentos": "lineamento",
    "lineamientos": "lineamiento",
    "lineamos": "linear",
    "linean": "linear",
    "lineando": "linear",
    "lineara": "linear",
    "linearais": "linear",
    "linearan": "linear",
    "linearas": "linear",
    "lineare": "linear",
    "lineareis": "linear",
    "linearemos": "linear",
    "linearen": "linear",
    "lineares": "linear",
    "linearidades": "linearidad",
    "linearizaciones": "linearización",
    "linearon": "linear",
    "lineará": "linear",
    "linearán": "linear",
    "linearás": "linear",
    "linearé": "linear",
    "linearéis": "linear",
    "linearía": "linear",
    "linearíais": "linear",
    "linearíamos": "linear",
    "linearían": "linear",
    "linearías": "linear",
    "lineas": "linear",
    "linease": "linear",
    "lineaseis": "linear",
    "lineasen": "linear",
    "lineases": "linear",
    "lineaste": "linear",
    "lineasteis": "linear",
    "linee": "linear",
    "lineemos": "linear",
    "lineen": "linear",
    "linees": "linear",
    "linenses": "linense",
    "lineo": "linear",
    "linera": "linero",
    "lineras": "linero",
    "lineros": "linero",
    "lineábamos": "linear",
    "lineáis": "linear",
    "lineáramos": "linear",
    "lineáremos": "linear",
    "lineásemos": "linear",
    "lineé": "linear",
    "lineéis": "linear",
    "lineítas": "lineíta",
    "lineó": "linear",
    "linfadenopatias": "linfadenopatia",
    "linfadenopatías": "linfadenopatía",
    "linfangiectasias": "linfangiectasia",
    "linfas": "linfa",
    "linfatismos": "linfatismo",
    "linfoblastos": "linfoblasto",
    "linfoblástica": "linfoblástico",
    "linfoblásticas": "linfoblástico",
    "linfoblásticos": "linfoblástico",
    "linfocinas": "linfocina",
    "linfocitaria": "linfocitario",
    "linfocitarias": "linfocitario",
    "linfocitarios": "linfocitario",
    "linfocitemias": "linfocitemia",
    "linfocitopenias": "linfocitopenia",
    "linfocitos": "linfocito",
    "linfocitotrópica": "linfocitotrópico",
    "linfocitotrópicas": "linfocitotrópico",
    "linfocitotrópicos": "linfocitotrópico",
    "linfocítica": "linfocítico",
    "linfocíticas": "linfocítico",
    "linfocíticos": "linfocítico",
    "linfografías": "linfografía",
    "linfogranulomas": "linfogranuloma",
    "linfoides": "linfoide",
    "linfomas": "linfoma",
    "linfomatosa": "linfomatoso",
    "linfomatosas": "linfomatoso",
    "linfomatosos": "linfomatoso",
    "linfopenias": "linfopenia",
    "linfoquinas": "linfoquina",
    "linfosarcomas": "linfosarcoma",
    "linfotoxinas": "linfotoxina",
    "linfática": "linfático",
    "linfáticas": "linfático",
    "linfáticos": "linfático",
    "lingoteras": "lingotera",
    "lingotes": "lingote",
    "linguada": "linguado",
    "linguadas": "linguado",
    "linguados": "linguado",
    "linguales": "lingual",
    "linguetes": "linguete",
    "linguoclusales": "linguoclusal",
    "linguoclusiones": "linguoclusión",
    "linguodentales": "linguodental",
    "linguofaciales": "linguofacial",
    "linguogingivales": "linguogingival",
    "linguogramas": "linguograma",
    "linguoversiones": "linguoversión",
    "lingüistas": "lingüista",
    "lingüística": "lingüístico",
    "lingüísticas": "lingüístico",
    "lingüísticos": "lingüístico",
    "linieres": "linier",
    "liniers": "linier",
    "linimentos": "linimento",
    "linografías": "linografía",
    "linográfica": "linográfico",
    "linográficas": "linográfico",
    "linográficos": "linográfico",
    "linoleatos": "linoleato",
    "linoleica": "linoleico",
    "linoleicas": "linoleico",
    "linoleicos": "linoleico",
    "linoleínas": "linoleína",
    "linolénica": "linolénico",
    "linolénicas": "linolénico",
    "linolénicos": "linolénico",
    "linones": "linón",
    "linos": "lino",
    "linotipias": "linotipia",
    "linotipistas": "linotipista",
    "linotipos": "linotipo",
    "linoxinas": "linoxina",
    "linteles": "lintel",
    "linternas": "linterna",
    "linternera": "linternero",
    "linterneras": "linternero",
    "linterneros": "linternero",
    "linternillas": "linternilla",
    "linternones": "linternón",
    "linuezos": "linuezo",
    "lináceas": "linácea",
    "linóleos": "linóleo",
    "linóleums": "linóleum",
    "liofilice": "liofilizar",
    "liofilicemos": "liofilizar",
    "liofilicen": "liofilizar",
    "liofilices": "liofilizar",
    "liofilicé": "liofilizar",
    "liofilicéis": "liofilizar",
    "liofiliza": "liofilizar",
    "liofilizaba": "liofilizar",
    "liofilizabais": "liofilizar",
    "liofilizaban": "liofilizar",
    "liofilizabas": "liofilizar",
    "liofilizaciones": "liofilización",
    "liofilizad": "liofilizar",
    "liofilizada": "liofilizar",
    "liofilizadas": "liofilizar",
    "liofilizado": "liofilizar",
    "liofilizadora": "liofilizador",
    "liofilizadoras": "liofilizador",
    "liofilizadores": "liofilizador",
    "liofilizados": "liofilizar",
    "liofilizamos": "liofilizar",
    "liofilizan": "liofilizar",
    "liofilizando": "liofilizar",
    "liofilizara": "liofilizar",
    "liofilizarais": "liofilizar",
    "liofilizaran": "liofilizar",
    "liofilizaras": "liofilizar",
    "liofilizare": "liofilizar",
    "liofilizareis": "liofilizar",
    "liofilizaremos": "liofilizar",
    "liofilizaren": "liofilizar",
    "liofilizares": "liofilizar",
    "liofilizaron": "liofilizar",
    "liofilizará": "liofilizar",
    "liofilizarán": "liofilizar",
    "liofilizarás": "liofilizar",
    "liofilizaré": "liofilizar",
    "liofilizaréis": "liofilizar",
    "liofilizaría": "liofilizar",
    "liofilizaríais": "liofilizar",
    "liofilizaríamos": "liofilizar",
    "liofilizarían": "liofilizar",
    "liofilizarías": "liofilizar",
    "liofilizas": "liofilizar",
    "liofilizase": "liofilizar",
    "liofilizaseis": "liofilizar",
    "liofilizasen": "liofilizar",
    "liofilizases": "liofilizar",
    "liofilizaste": "liofilizar",
    "liofilizasteis": "liofilizar",
    "liofilizo": "liofilizar",
    "liofilizábamos": "liofilizar",
    "liofilizáis": "liofilizar",
    "liofilizáramos": "liofilizar",
    "liofilizáremos": "liofilizar",
    "liofilizásemos": "liofilizar",
    "liofilizó": "liofilizar",
    "liofílica": "liofílico",
    "liofílicas": "liofílico",
    "liofílicos": "liofílico",
    "lionesa": "lionés",
    "lionesas": "lionesa",
    "lioneses": "lionés",
    "liosa": "lioso",
    "liosas": "lioso",
    "liosos": "lioso",
    "liparitas": "liparita",
    "liparoidea": "liparoideo",
    "liparoideas": "liparoideo",
    "liparoideos": "liparoideo",
    "lipasas": "lipasa",
    "lipemaníaca": "lipemaníaco",
    "lipemaníacas": "lipemaníaco",
    "lipemaníacos": "lipemaníaco",
    "lipemanías": "lipemanía",
    "lipemias": "lipemia",
    "lipendis": "lipendi",
    "lipidemias": "lipidemia",
    "lipidoplastos": "lipidoplasto",
    "lipocitos": "lipocito",
    "lipocromos": "lipocromo",
    "lipodistrofias": "lipodistrofia",
    "lipodistrófica": "lipodistrófico",
    "lipodistróficas": "lipodistrófico",
    "lipodistróficos": "lipodistrófico",
    "lipofílica": "lipofílico",
    "lipofílicas": "lipofílico",
    "lipofílicos": "lipofílico",
    "lipoidea": "lipoideo",
    "lipoideas": "lipoideo",
    "lipoideos": "lipoideo",
    "lipomas": "lipoma",
    "lipopolisacáridos": "lipopolisacárido",
    "lipoproteinas": "lipoproteina",
    "lipoproteínas": "lipoproteína",
    "liposomas": "liposoma",
    "liposucciones": "liposucción",
    "lipotimias": "lipotimia",
    "lipotrópica": "lipotrópico",
    "lipotrópicas": "lipotrópico",
    "lipotrópicos": "lipotrópico",
    "lipoxidasas": "lipoxidasa",
    "lipoxigenasas": "lipoxigenasa",
    "lipídica": "lipídico",
    "lipídicas": "lipídico",
    "lipídicos": "lipídico",
    "liquenes": "liquen",
    "liqueninas": "liquenina",
    "liquida": "liquidar",
    "liquidaba": "liquidar",
    "liquidabais": "liquidar",
    "liquidaban": "liquidar",
    "liquidabas": "liquidar",
    "liquidables": "liquidable",
    "liquidaciones": "liquidación",
    "liquidad": "liquidar",
    "liquidada": "liquidar",
    "liquidadas": "liquidar",
    "liquidado": "liquidar",
    "liquidadora": "liquidador",
    "liquidadoras": "liquidador",
    "liquidadores": "liquidador",
    "liquidados": "liquidar",
    "liquidamos": "liquidar",
    "liquidan": "liquidar",
    "liquidando": "liquidar",
    "liquidara": "liquidar",
    "liquidarais": "liquidar",
    "liquidaran": "liquidar",
    "liquidaras": "liquidar",
    "liquidare": "liquidar",
    "liquidareis": "liquidar",
    "liquidaremos": "liquidar",
    "liquidaren": "liquidar",
    "liquidares": "liquidar",
    "liquidaron": "liquidar",
    "liquidará": "liquidar",
    "liquidarán": "liquidar",
    "liquidarás": "liquidar",
    "liquidaré": "liquidar",
    "liquidaréis": "liquidar",
    "liquidaría": "liquidar",
    "liquidaríais": "liquidar",
    "liquidaríamos": "liquidar",
    "liquidarían": "liquidar",
    "liquidarías": "liquidar",
    "liquidas": "liquidar",
    "liquidase": "liquidar",
    "liquidaseis": "liquidar",
    "liquidasen": "liquidar",
    "liquidases": "liquidar",
    "liquidaste": "liquidar",
    "liquidasteis": "liquidar",
    "liquidatoria": "liquidatorio",
    "liquidatorias": "liquidatorio",
    "liquidatorios": "liquidatorio",
    "liquide": "liquidar",
    "liquideces": "liquidez",
    "liquidemos": "liquidar",
    "liquiden": "liquidar",
    "liquides": "liquidar",
    "liquido": "liquidar",
    "liquidábamos": "liquidar",
    "liquidáis": "liquidar",
    "liquidámbares": "liquidámbar",
    "liquidáramos": "liquidar",
    "liquidáremos": "liquidar",
    "liquidásemos": "liquidar",
    "liquidé": "liquidar",
    "liquidéis": "liquidar",
    "liquidó": "liquidar",
    "liras": "lira",
    "liricidades": "liricidad",
    "lirios": "lirio",
    "lirismos": "lirismo",
    "lirones": "lirón",
    "lirófora": "liróforo",
    "liróforas": "liróforo",
    "liróforos": "liróforo",
    "lisa": "liso",
    "lisas": "liso",
    "lisboetas": "lisboeta",
    "lisbonenses": "lisbonense",
    "lisbonesa": "lisbonés",
    "lisbonesas": "lisbonés",
    "lisboneses": "lisbonés",
    "lises": "lis",
    "lisia": "lisiar",
    "lisiaba": "lisiar",
    "lisiabais": "lisiar",
    "lisiaban": "lisiar",
    "lisiabas": "lisiar",
    "lisiad": "lisiar",
    "lisiada": "lisiar",
    "lisiadas": "lisiar",
    "lisiado": "lisiar",
    "lisiados": "lisiar",
    "lisiaduras": "lisiadura",
    "lisiamos": "lisiar",
    "lisian": "lisiar",
    "lisiando": "lisiar",
    "lisiara": "lisiar",
    "lisiarais": "lisiar",
    "lisiaran": "lisiar",
    "lisiaras": "lisiar",
    "lisiare": "lisiar",
    "lisiareis": "lisiar",
    "lisiaremos": "lisiar",
    "lisiaren": "lisiar",
    "lisiares": "lisiar",
    "lisiaron": "lisiar",
    "lisiará": "lisiar",
    "lisiarán": "lisiar",
    "lisiarás": "lisiar",
    "lisiaré": "lisiar",
    "lisiaréis": "lisiar",
    "lisiaría": "lisiar",
    "lisiaríais": "lisiar",
    "lisiaríamos": "lisiar",
    "lisiarían": "lisiar",
    "lisiarías": "lisiar",
    "lisias": "lisiar",
    "lisiase": "lisiar",
    "lisiaseis": "lisiar",
    "lisiasen": "lisiar",
    "lisiases": "lisiar",
    "lisiaste": "lisiar",
    "lisiasteis": "lisiar",
    "lisie": "lisiar",
    "lisiemos": "lisiar",
    "lisien": "lisiar",
    "lisies": "lisiar",
    "lisimaquias": "lisimaquia",
    "lisinas": "lisina",
    "lisio": "lisiar",
    "lisizimas": "lisizima",
    "lisiábamos": "lisiar",
    "lisiáis": "lisiar",
    "lisiáramos": "lisiar",
    "lisiáremos": "lisiar",
    "lisiásemos": "lisiar",
    "lisié": "lisiar",
    "lisiéis": "lisiar",
    "lisió": "lisiar",
    "lisocefalinas": "lisocefalina",
    "lisogenisaciones": "lisogenisación",
    "lisogénica": "lisogénico",
    "lisogénicas": "lisogénico",
    "lisogénicos": "lisogénico",
    "lisoles": "lisol",
    "lisonjas": "lisonja",
    "lisonjea": "lisonjear",
    "lisonjeaba": "lisonjear",
    "lisonjeabais": "lisonjear",
    "lisonjeaban": "lisonjear",
    "lisonjeabas": "lisonjear",
    "lisonjead": "lisonjear",
    "lisonjeada": "lisonjear",
    "lisonjeadas": "lisonjear",
    "lisonjeado": "lisonjear",
    "lisonjeadora": "lisonjeador",
    "lisonjeadoras": "lisonjeador",
    "lisonjeadores": "lisonjeador",
    "lisonjeados": "lisonjear",
    "lisonjeamos": "lisonjear",
    "lisonjean": "lisonjear",
    "lisonjeando": "lisonjear",
    "lisonjeara": "lisonjear",
    "lisonjearais": "lisonjear",
    "lisonjearan": "lisonjear",
    "lisonjearas": "lisonjear",
    "lisonjeare": "lisonjear",
    "lisonjeareis": "lisonjear",
    "lisonjearemos": "lisonjear",
    "lisonjearen": "lisonjear",
    "lisonjeares": "lisonjear",
    "lisonjearon": "lisonjear",
    "lisonjeará": "lisonjear",
    "lisonjearán": "lisonjear",
    "lisonjearás": "lisonjear",
    "lisonjearé": "lisonjear",
    "lisonjearéis": "lisonjear",
    "lisonjearía": "lisonjear",
    "lisonjearíais": "lisonjear",
    "lisonjearíamos": "lisonjear",
    "lisonjearían": "lisonjear",
    "lisonjearías": "lisonjear",
    "lisonjeas": "lisonjear",
    "lisonjease": "lisonjear",
    "lisonjeaseis": "lisonjear",
    "lisonjeasen": "lisonjear",
    "lisonjeases": "lisonjear",
    "lisonjeaste": "lisonjear",
    "lisonjeasteis": "lisonjear",
    "lisonjee": "lisonjear",
    "lisonjeemos": "lisonjear",
    "lisonjeen": "lisonjear",
    "lisonjees": "lisonjear",
    "lisonjeo": "lisonjear",
    "lisonjera": "lisonjero",
    "lisonjeras": "lisonjero",
    "lisonjeros": "lisonjero",
    "lisonjeábamos": "lisonjear",
    "lisonjeáis": "lisonjear",
    "lisonjeáramos": "lisonjear",
    "lisonjeáremos": "lisonjear",
    "lisonjeásemos": "lisonjear",
    "lisonjeé": "lisonjear",
    "lisonjeéis": "lisonjear",
    "lisonjeó": "lisonjear",
    "lisos": "liso",
    "lisosomas": "lisosoma",
    "lisosómica": "lisosómico",
    "lisosómicas": "lisosómico",
    "lisosómicos": "lisosómico",
    "lisozimas": "lisozima",
    "lisps": "lisp",
    "lista": "listo",
    "listaba": "listar",
    "listabais": "listar",
    "listaban": "listar",
    "listabas": "listar",
    "listad": "listar",
    "listada": "listar",
    "listadas": "listar",
    "listado": "listar",
    "listados": "listar",
    "listamos": "listar",
    "listan": "listar",
    "listando": "listar",
    "listara": "listar",
    "listarais": "listar",
    "listaran": "listar",
    "listaras": "listar",
    "listare": "listar",
    "listareis": "listar",
    "listaremos": "listar",
    "listaren": "listar",
    "listares": "listar",
    "listaron": "listar",
    "listará": "listar",
    "listarán": "listar",
    "listarás": "listar",
    "listaré": "listar",
    "listaréis": "listar",
    "listaría": "listar",
    "listaríais": "listar",
    "listaríamos": "listar",
    "listarían": "listar",
    "listarías": "listar",
    "listas": "listo",
    "listase": "listar",
    "listaseis": "listar",
    "listasen": "listar",
    "listases": "listar",
    "listaste": "listar",
    "listasteis": "listar",
    "liste": "listar",
    "listeada": "listeado",
    "listeadas": "listeado",
    "listeados": "listeado",
    "listeles": "listel",
    "listemos": "listar",
    "listen": "listar",
    "listera": "listero",
    "listeras": "listero",
    "listeros": "listero",
    "listes": "listar",
    "listezas": "listeza",
    "listines": "listín",
    "listo": "listar",
    "listona": "listonar",
    "listonaba": "listonar",
    "listonabais": "listonar",
    "listonaban": "listonar",
    "listonabas": "listonar",
    "listonad": "listonar",
    "listonada": "listonar",
    "listonadas": "listonar",
    "listonado": "listonar",
    "listonados": "listonar",
    "listonamos": "listonar",
    "listonan": "listonar",
    "listonando": "listonar",
    "listonara": "listonar",
    "listonarais": "listonar",
    "listonaran": "listonar",
    "listonaras": "listonar",
    "listonare": "listonar",
    "listonareis": "listonar",
    "listonaremos": "listonar",
    "listonaren": "listonar",
    "listonares": "listonar",
    "listonaron": "listonar",
    "listonará": "listonar",
    "listonarán": "listonar",
    "listonarás": "listonar",
    "listonaré": "listonar",
    "listonaréis": "listonar",
    "listonaría": "listonar",
    "listonaríais": "listonar",
    "listonaríamos": "listonar",
    "listonarían": "listonar",
    "listonarías": "listonar",
    "listonas": "listonar",
    "listonase": "listonar",
    "listonaseis": "listonar",
    "listonasen": "listonar",
    "listonases": "listonar",
    "listonaste": "listonar",
    "listonasteis": "listonar",
    "listone": "listonar",
    "listonemos": "listonar",
    "listonen": "listonar",
    "listonera": "listonero",
    "listoneras": "listonero",
    "listoneros": "listonero",
    "listonerías": "listonería",
    "listones": "listonar",
    "listono": "listonar",
    "listonábamos": "listonar",
    "listonáis": "listonar",
    "listonáramos": "listonar",
    "listonáremos": "listonar",
    "listonásemos": "listonar",
    "listoné": "listonar",
    "listonéis": "listonar",
    "listonó": "listonar",
    "listos": "listo",
    "listábamos": "listar",
    "listáis": "listar",
    "listáramos": "listar",
    "listáremos": "listar",
    "listásemos": "listar",
    "listé": "listar",
    "listéis": "listar",
    "listó": "listar",
    "lisuras": "lisura",
    "lisérgica": "lisérgico",
    "lisérgicas": "lisérgico",
    "lisérgicos": "lisérgico",
    "lisímetros": "lisímetro",
    "lita": "litar",
    "litaba": "litar",
    "litabais": "litar",
    "litaban": "litar",
    "litabas": "litar",
    "litad": "litar",
    "litada": "litar",
    "litadas": "litar",
    "litado": "litar",
    "litados": "litar",
    "litamos": "litar",
    "litan": "litar",
    "litando": "litar",
    "litantrácidos": "litantrácido",
    "litara": "litar",
    "litarais": "litar",
    "litaran": "litar",
    "litaras": "litar",
    "litare": "litar",
    "litareis": "litar",
    "litaremos": "litar",
    "litaren": "litar",
    "litares": "litar",
    "litarges": "litarge",
    "litargirios": "litargirio",
    "litaron": "litar",
    "litará": "litar",
    "litarán": "litar",
    "litarás": "litar",
    "litaré": "litar",
    "litaréis": "litar",
    "litaría": "litar",
    "litaríais": "litar",
    "litaríamos": "litar",
    "litarían": "litar",
    "litarías": "litar",
    "litas": "litar",
    "litase": "litar",
    "litaseis": "litar",
    "litasen": "litar",
    "litases": "litar",
    "litaste": "litar",
    "litasteis": "litar",
    "lite": "litar",
    "litemos": "litar",
    "liten": "litar",
    "literales": "literal",
    "literalidades": "literalidad",
    "literaria": "literario",
    "literarias": "literario",
    "literarios": "literario",
    "literas": "litera",
    "literata": "literato",
    "literatas": "literato",
    "literatos": "literato",
    "literaturas": "literatura",
    "lites": "litar",
    "liticonsorciales": "liticonsorcial",
    "litificaciones": "litificación",
    "litiga": "litigar",
    "litigaba": "litigar",
    "litigabais": "litigar",
    "litigaban": "litigar",
    "litigabas": "litigar",
    "litigaciones": "litigación",
    "litigad": "litigar",
    "litigada": "litigar",
    "litigadas": "litigar",
    "litigado": "litigar",
    "litigadora": "litigador",
    "litigadoras": "litigador",
    "litigadores": "litigador",
    "litigados": "litigar",
    "litigamos": "litigar",
    "litigan": "litigar",
    "litigando": "litigar",
    "litigantes": "litigante",
    "litigara": "litigar",
    "litigarais": "litigar",
    "litigaran": "litigar",
    "litigaras": "litigar",
    "litigare": "litigar",
    "litigareis": "litigar",
    "litigaremos": "litigar",
    "litigaren": "litigar",
    "litigares": "litigar",
    "litigaron": "litigar",
    "litigará": "litigar",
    "litigarán": "litigar",
    "litigarás": "litigar",
    "litigaré": "litigar",
    "litigaréis": "litigar",
    "litigaría": "litigar",
    "litigaríais": "litigar",
    "litigaríamos": "litigar",
    "litigarían": "litigar",
    "litigarías": "litigar",
    "litigas": "litigar",
    "litigase": "litigar",
    "litigaseis": "litigar",
    "litigasen": "litigar",
    "litigases": "litigar",
    "litigaste": "litigar",
    "litigasteis": "litigar",
    "litigios": "litigio",
    "litigiosa": "litigioso",
    "litigiosas": "litigioso",
    "litigiosos": "litigioso",
    "litigo": "litigar",
    "litigue": "litigar",
    "litiguemos": "litigar",
    "litiguen": "litigar",
    "litigues": "litigar",
    "litigué": "litigar",
    "litiguéis": "litigar",
    "litigábamos": "litigar",
    "litigáis": "litigar",
    "litigáramos": "litigar",
    "litigáremos": "litigar",
    "litigásemos": "litigar",
    "litigó": "litigar",
    "litinas": "litina",
    "litios": "litio",
    "litisconsorciales": "litisconsorcial",
    "litisconsorcios": "litisconsorcio",
    "litisconsortes": "litisconsorte",
    "litiscontestaciones": "litiscontestación",
    "litispendencias": "litispendencia",
    "litiásica": "litiásico",
    "litiásicas": "litiásico",
    "litiásicos": "litiásico",
    "lito": "litar",
    "litocalcarenitas": "litocalcarenita",
    "litocálamos": "litocálamo",
    "litocólica": "litocólico",
    "litocólicas": "litocólico",
    "litocólicos": "litocólico",
    "litoestratigrafías": "litoestratigrafía",
    "litofanías": "litofanía",
    "litofotografiaba": "litofotografiar",
    "litofotografiabais": "litofotografiar",
    "litofotografiaban": "litofotografiar",
    "litofotografiabas": "litofotografiar",
    "litofotografiad": "litofotografiar",
    "litofotografiada": "litofotografiar",
    "litofotografiadas": "litofotografiar",
    "litofotografiado": "litofotografiar",
    "litofotografiados": "litofotografiar",
    "litofotografiamos": "litofotografiar",
    "litofotografiando": "litofotografiar",
    "litofotografiara": "litofotografiar",
    "litofotografiarais": "litofotografiar",
    "litofotografiaran": "litofotografiar",
    "litofotografiaras": "litofotografiar",
    "litofotografiare": "litofotografiar",
    "litofotografiareis": "litofotografiar",
    "litofotografiaremos": "litofotografiar",
    "litofotografiaren": "litofotografiar",
    "litofotografiares": "litofotografiar",
    "litofotografiaron": "litofotografiar",
    "litofotografiará": "litofotografiar",
    "litofotografiarán": "litofotografiar",
    "litofotografiarás": "litofotografiar",
    "litofotografiaré": "litofotografiar",
    "litofotografiaréis": "litofotografiar",
    "litofotografiaría": "litofotografiar",
    "litofotografiaríais": "litofotografiar",
    "litofotografiaríamos": "litofotografiar",
    "litofotografiarían": "litofotografiar",
    "litofotografiarías": "litofotografiar",
    "litofotografiase": "litofotografiar",
    "litofotografiaseis": "litofotografiar",
    "litofotografiasen": "litofotografiar",
    "litofotografiases": "litofotografiar",
    "litofotografiaste": "litofotografiar",
    "litofotografiasteis": "litofotografiar",
    "litofotografiemos": "litofotografiar",
    "litofotografiábamos": "litofotografiar",
    "litofotografiáis": "litofotografiar",
    "litofotografiáramos": "litofotografiar",
    "litofotografiáremos": "litofotografiar",
    "litofotografiásemos": "litofotografiar",
    "litofotografié": "litofotografiar",
    "litofotografiéis": "litofotografiar",
    "litofotografió": "litofotografiar",
    "litofotografía": "litofotografiar",
    "litofotografían": "litofotografiar",
    "litofotografías": "litofotografiar",
    "litofotografíe": "litofotografiar",
    "litofotografíen": "litofotografiar",
    "litofotografíes": "litofotografiar",
    "litofotografío": "litofotografiar",
    "litogenesias": "litogenesia",
    "litograbadores": "litograbador",
    "litografiaba": "litografiar",
    "litografiabais": "litografiar",
    "litografiaban": "litografiar",
    "litografiabas": "litografiar",
    "litografiad": "litografiar",
    "litografiada": "litografiar",
    "litografiadas": "litografiar",
    "litografiado": "litografiar",
    "litografiados": "litografiar",
    "litografiamos": "litografiar",
    "litografiando": "litografiar",
    "litografiara": "litografiar",
    "litografiarais": "litografiar",
    "litografiaran": "litografiar",
    "litografiaras": "litografiar",
    "litografiare": "litografiar",
    "litografiareis": "litografiar",
    "litografiaremos": "litografiar",
    "litografiaren": "litografiar",
    "litografiares": "litografiar",
    "litografiaron": "litografiar",
    "litografiará": "litografiar",
    "litografiarán": "litografiar",
    "litografiarás": "litografiar",
    "litografiaré": "litografiar",
    "litografiaréis": "litografiar",
    "litografiaría": "litografiar",
    "litografiaríais": "litografiar",
    "litografiaríamos": "litografiar",
    "litografiarían": "litografiar",
    "litografiarías": "litografiar",
    "litografiase": "litografiar",
    "litografiaseis": "litografiar",
    "litografiasen": "litografiar",
    "litografiases": "litografiar",
    "litografiaste": "litografiar",
    "litografiasteis": "litografiar",
    "litografiemos": "litografiar",
    "litografiábamos": "litografiar",
    "litografiáis": "litografiar",
    "litografiáramos": "litografiar",
    "litografiáremos": "litografiar",
    "litografiásemos": "litografiar",
    "litografié": "litografiar",
    "litografiéis": "litografiar",
    "litografió": "litografiar",
    "litografía": "litografiar",
    "litografían": "litografiar",
    "litografías": "litografiar",
    "litografíe": "litografiar",
    "litografíen": "litografiar",
    "litografíes": "litografiar",
    "litografío": "litografiar",
    "litográfica": "litográfico",
    "litográficas": "litográfico",
    "litográficos": "litográfico",
    "litologías": "litología",
    "litológica": "litológico",
    "litológicas": "litológico",
    "litológicos": "litológico",
    "litopediones": "litopedion",
    "litopones": "litopón",
    "litopédiones": "litopedion",
    "litorales": "litoral",
    "litosecuencias": "litosecuencia",
    "litosferas": "litosfera",
    "litosférica": "litosférico",
    "litosféricas": "litosférico",
    "litosféricos": "litosférico",
    "litosideritos": "litosiderito",
    "litosoles": "litosol",
    "litosuelos": "litosuelo",
    "litotomías": "litotomía",
    "litotricias": "litotricia",
    "litris": "litri",
    "litronas": "litrona",
    "litros": "litro",
    "litráceas": "litrácea",
    "lituana": "lituano",
    "lituanas": "lituano",
    "lituanos": "lituano",
    "lituos": "lituo",
    "liturgias": "liturgia",
    "liturgistas": "liturgista",
    "litábamos": "litar",
    "litáis": "litar",
    "litáramos": "litar",
    "litáremos": "litar",
    "litásemos": "litar",
    "lité": "litar",
    "litéis": "litar",
    "litó": "litar",
    "litófaga": "litófago",
    "litófagas": "litófago",
    "litófagos": "litófago",
    "litófila": "litófilo",
    "litófilas": "litófilo",
    "litófilos": "litófilo",
    "litógena": "litógeno",
    "litógenas": "litógeno",
    "litógenos": "litógeno",
    "litógrafa": "litógrafo",
    "litógrafas": "litógrafo",
    "litógrafos": "litógrafo",
    "litóloga": "litólogo",
    "litólogas": "litólogo",
    "litólogos": "litólogo",
    "litúrgica": "litúrgico",
    "litúrgicas": "litúrgico",
    "litúrgicos": "litúrgico",
    "liuda": "liudar",
    "liudaba": "liudar",
    "liudabais": "liudar",
    "liudaban": "liudar",
    "liudabas": "liudar",
    "liudad": "liudar",
    "liudada": "liudar",
    "liudadas": "liudar",
    "liudado": "liudar",
    "liudados": "liudar",
    "liudamos": "liudar",
    "liudan": "liudar",
    "liudando": "liudar",
    "liudara": "liudar",
    "liudarais": "liudar",
    "liudaran": "liudar",
    "liudaras": "liudar",
    "liudare": "liudar",
    "liudareis": "liudar",
    "liudaremos": "liudar",
    "liudaren": "liudar",
    "liudares": "liudar",
    "liudaron": "liudar",
    "liudará": "liudar",
    "liudarán": "liudar",
    "liudarás": "liudar",
    "liudaré": "liudar",
    "liudaréis": "liudar",
    "liudaría": "liudar",
    "liudaríais": "liudar",
    "liudaríamos": "liudar",
    "liudarían": "liudar",
    "liudarías": "liudar",
    "liudas": "liudar",
    "liudase": "liudar",
    "liudaseis": "liudar",
    "liudasen": "liudar",
    "liudases": "liudar",
    "liudaste": "liudar",
    "liudasteis": "liudar",
    "liude": "liudar",
    "liudemos": "liudar",
    "liuden": "liudar",
    "liudes": "liudar",
    "liudo": "liudar",
    "liudábamos": "liudar",
    "liudáis": "liudar",
    "liudáramos": "liudar",
    "liudáremos": "liudar",
    "liudásemos": "liudar",
    "liudé": "liudar",
    "liudéis": "liudar",
    "liudó": "liudar",
    "livetinas": "livetina",
    "liviana": "liviano",
    "livianas": "liviano",
    "liviandades": "liviandad",
    "livianos": "liviano",
    "lividece": "lividecer",
    "livideced": "lividecer",
    "lividecemos": "lividecer",
    "lividecen": "lividecer",
    "livideceremos": "lividecer",
    "lividecerá": "lividecer",
    "lividecerán": "lividecer",
    "lividecerás": "lividecer",
    "livideceré": "lividecer",
    "livideceréis": "lividecer",
    "lividecería": "lividecer",
    "livideceríais": "lividecer",
    "livideceríamos": "lividecer",
    "lividecerían": "lividecer",
    "lividecerías": "lividecer",
    "livideces": "lividez",
    "lividecida": "lividecer",
    "lividecidas": "lividecer",
    "lividecido": "lividecer",
    "lividecidos": "lividecer",
    "livideciendo": "lividecer",
    "livideciera": "lividecer",
    "lividecierais": "lividecer",
    "lividecieran": "lividecer",
    "lividecieras": "lividecer",
    "livideciere": "lividecer",
    "livideciereis": "lividecer",
    "lividecieren": "lividecer",
    "lividecieres": "lividecer",
    "lividecieron": "lividecer",
    "livideciese": "lividecer",
    "lividecieseis": "lividecer",
    "livideciesen": "lividecer",
    "lividecieses": "lividecer",
    "lividecimos": "lividecer",
    "livideciste": "lividecer",
    "lividecisteis": "lividecer",
    "livideciéramos": "lividecer",
    "livideciéremos": "lividecer",
    "livideciésemos": "lividecer",
    "livideció": "lividecer",
    "lividecéis": "lividecer",
    "lividecí": "lividecer",
    "lividecía": "lividecer",
    "lividecíais": "lividecer",
    "lividecíamos": "lividecer",
    "lividecían": "lividecer",
    "lividecías": "lividecer",
    "lividezca": "lividecer",
    "lividezcamos": "lividecer",
    "lividezcan": "lividecer",
    "lividezcas": "lividecer",
    "lividezco": "lividecer",
    "lividezcáis": "lividecer",
    "livings": "living",
    "livonia": "livonio",
    "livonias": "livonio",
    "livonios": "livonio",
    "livores": "livor",
    "lixivaciones": "lixivación",
    "lixivia": "lixiviar",
    "lixiviaba": "lixiviar",
    "lixiviabais": "lixiviar",
    "lixiviaban": "lixiviar",
    "lixiviabas": "lixiviar",
    "lixiviaciones": "lixiviación",
    "lixiviad": "lixiviar",
    "lixiviada": "lixiviar",
    "lixiviadas": "lixiviar",
    "lixiviado": "lixiviar",
    "lixiviados": "lixiviar",
    "lixiviamos": "lixiviar",
    "lixivian": "lixiviar",
    "lixiviando": "lixiviar",
    "lixiviara": "lixiviar",
    "lixiviarais": "lixiviar",
    "lixiviaran": "lixiviar",
    "lixiviaras": "lixiviar",
    "lixiviare": "lixiviar",
    "lixiviareis": "lixiviar",
    "lixiviaremos": "lixiviar",
    "lixiviaren": "lixiviar",
    "lixiviares": "lixiviar",
    "lixiviaron": "lixiviar",
    "lixiviará": "lixiviar",
    "lixiviarán": "lixiviar",
    "lixiviarás": "lixiviar",
    "lixiviaré": "lixiviar",
    "lixiviaréis": "lixiviar",
    "lixiviaría": "lixiviar",
    "lixiviaríais": "lixiviar",
    "lixiviaríamos": "lixiviar",
    "lixiviarían": "lixiviar",
    "lixiviarías": "lixiviar",
    "lixivias": "lixiviar",
    "lixiviase": "lixiviar",
    "lixiviaseis": "lixiviar",
    "lixiviasen": "lixiviar",
    "lixiviases": "lixiviar",
    "lixiviaste": "lixiviar",
    "lixiviasteis": "lixiviar",
    "lixivie": "lixiviar",
    "lixiviemos": "lixiviar",
    "lixivien": "lixiviar",
    "lixivies": "lixiviar",
    "lixivio": "lixiviar",
    "lixiviábamos": "lixiviar",
    "lixiviáis": "lixiviar",
    "lixiviáramos": "lixiviar",
    "lixiviáremos": "lixiviar",
    "lixiviásemos": "lixiviar",
    "lixivié": "lixiviar",
    "lixiviéis": "lixiviar",
    "lixivió": "lixiviar",
    "lizas": "liza",
    "lizos": "lizo",
    "liábamos": "liar",
    "liáis": "liar",
    "liáramos": "liar",
    "liáremos": "liar",
    "liásemos": "liar",
    "liásica": "liásico",
    "liásicas": "liásico",
    "liásicos": "liásico",
    "lié": "liar",
    "liéis": "liar",
    "liños": "liño",
    "liñuelos": "liñuelo",
    "lió": "liar",
    "liófila": "liófilo",
    "liófilas": "liófilo",
    "liófilos": "liófilo",
    "liófoba": "liófobo",
    "liófobas": "liófobo",
    "liófobos": "liófobo",
    "llaga": "llagar",
    "llagaba": "llagar",
    "llagabais": "llagar",
    "llagaban": "llagar",
    "llagabas": "llagar",
    "llagad": "llagar",
    "llagada": "llagar",
    "llagadas": "llagar",
    "llagado": "llagar",
    "llagados": "llagar",
    "llagamos": "llagar",
    "llagan": "llagar",
    "llagando": "llagar",
    "llagara": "llagar",
    "llagarais": "llagar",
    "llagaran": "llagar",
    "llagaras": "llagar",
    "llagare": "llagar",
    "llagareis": "llagar",
    "llagaremos": "llagar",
    "llagaren": "llagar",
    "llagares": "llagar",
    "llagaron": "llagar",
    "llagará": "llagar",
    "llagarán": "llagar",
    "llagarás": "llagar",
    "llagaré": "llagar",
    "llagaréis": "llagar",
    "llagaría": "llagar",
    "llagaríais": "llagar",
    "llagaríamos": "llagar",
    "llagarían": "llagar",
    "llagarías": "llagar",
    "llagas": "llagar",
    "llagase": "llagar",
    "llagaseis": "llagar",
    "llagasen": "llagar",
    "llagases": "llagar",
    "llagaste": "llagar",
    "llagasteis": "llagar",
    "llago": "llagar",
    "llague": "llagar",
    "llaguemos": "llagar",
    "llaguen": "llagar",
    "llagueros": "llaguero",
    "llagues": "llagar",
    "llagué": "llagar",
    "llaguéis": "llagar",
    "llagábamos": "llagar",
    "llagáis": "llagar",
    "llagáramos": "llagar",
    "llagáremos": "llagar",
    "llagásemos": "llagar",
    "llagó": "llagar",
    "llama": "llamar",
    "llamaba": "llamar",
    "llamabais": "llamar",
    "llamaban": "llamar",
    "llamabas": "llamar",
    "llamables": "llamable",
    "llamad": "llamar",
    "llamada": "llamar",
    "llamadas": "llamar",
    "llamado": "llamar",
    "llamadora": "llamador",
    "llamadoras": "llamador",
    "llamadores": "llamador",
    "llamados": "llamar",
    "llamamientos": "llamamiento",
    "llamamos": "llamar",
    "llaman": "llamar",
    "llamando": "llamar",
    "llamara": "llamar",
    "llamaradas": "llamarada",
    "llamarais": "llamar",
    "llamaran": "llamar",
    "llamaras": "llamar",
    "llamare": "llamar",
    "llamareis": "llamar",
    "llamaremos": "llamar",
    "llamaren": "llamar",
    "llamares": "llamar",
    "llamaron": "llamar",
    "llamará": "llamar",
    "llamarán": "llamar",
    "llamarás": "llamar",
    "llamaré": "llamar",
    "llamaréis": "llamar",
    "llamaría": "llamar",
    "llamaríais": "llamar",
    "llamaríamos": "llamar",
    "llamarían": "llamar",
    "llamarías": "llamar",
    "llamas": "llamar",
    "llamase": "llamar",
    "llamaseis": "llamar",
    "llamasen": "llamar",
    "llamases": "llamar",
    "llamaste": "llamar",
    "llamasteis": "llamar",
    "llamativa": "llamativo",
    "llamativas": "llamativo",
    "llamativos": "llamativo",
    "llamazares": "llamazar",
    "llambrias": "llambria",
    "llame": "llamar",
    "llamea": "llamear",
    "llameaba": "llamear",
    "llameabais": "llamear",
    "llameaban": "llamear",
    "llameabas": "llamear",
    "llamead": "llamear",
    "llameada": "llamear",
    "llameadas": "llamear",
    "llameado": "llamear",
    "llameados": "llamear",
    "llameamos": "llamear",
    "llamean": "llamear",
    "llameando": "llamear",
    "llameara": "llamear",
    "llamearais": "llamear",
    "llamearan": "llamear",
    "llamearas": "llamear",
    "llameare": "llamear",
    "llameareis": "llamear",
    "llamearemos": "llamear",
    "llamearen": "llamear",
    "llameares": "llamear",
    "llamearon": "llamear",
    "llameará": "llamear",
    "llamearán": "llamear",
    "llamearás": "llamear",
    "llamearé": "llamear",
    "llamearéis": "llamear",
    "llamearía": "llamear",
    "llamearíais": "llamear",
    "llamearíamos": "llamear",
    "llamearían": "llamear",
    "llamearías": "llamear",
    "llameas": "llamear",
    "llamease": "llamear",
    "llameaseis": "llamear",
    "llameasen": "llamear",
    "llameases": "llamear",
    "llameaste": "llamear",
    "llameasteis": "llamear",
    "llamee": "llamear",
    "llameemos": "llamear",
    "llameen": "llamear",
    "llamees": "llamear",
    "llamemos": "llamar",
    "llamen": "llamar",
    "llameo": "llamear",
    "llames": "llamar",
    "llameábamos": "llamear",
    "llameáis": "llamear",
    "llameáramos": "llamear",
    "llameáremos": "llamear",
    "llameásemos": "llamear",
    "llameé": "llamear",
    "llameéis": "llamear",
    "llameó": "llamear",
    "llamo": "llamar",
    "llampugas": "llampuga",
    "llamábamos": "llamar",
    "llamáis": "llamar",
    "llamáramos": "llamar",
    "llamáremos": "llamar",
    "llamásemos": "llamar",
    "llamé": "llamar",
    "llaméis": "llamar",
    "llamó": "llamar",
    "llana": "llano",
    "llanadas": "llanada",
    "llanas": "llano",
    "llandes": "llande",
    "llanera": "llanero",
    "llaneras": "llanero",
    "llaneros": "llanero",
    "llanezas": "llaneza",
    "llanos": "llano",
    "llantas": "llanta",
    "llantenes": "llantén",
    "llanteras": "llantera",
    "llanterías": "llantería",
    "llanteríos": "llanterío",
    "llantinas": "llantina",
    "llantos": "llanto",
    "llanuras": "llanura",
    "llapas": "llapa",
    "llares": "llar",
    "llavera": "llavero",
    "llaveras": "llavero",
    "llaveros": "llavero",
    "llaves": "llave",
    "llavines": "llavín",
    "lleca": "lleco",
    "llecas": "lleco",
    "llecos": "lleco",
    "llega": "llegar",
    "llegaba": "llegar",
    "llegabais": "llegar",
    "llegaban": "llegar",
    "llegabas": "llegar",
    "llegad": "llegar",
    "llegada": "llegar",
    "llegadas": "llegar",
    "llegado": "llegar",
    "llegados": "llegar",
    "llegamos": "llegar",
    "llegan": "llegar",
    "llegando": "llegar",
    "llegara": "llegar",
    "llegarais": "llegar",
    "llegaran": "llegar",
    "llegaras": "llegar",
    "llegare": "llegar",
    "llegareis": "llegar",
    "llegaremos": "llegar",
    "llegaren": "llegar",
    "llegares": "llegar",
    "llegaron": "llegar",
    "llegará": "llegar",
    "llegarán": "llegar",
    "llegarás": "llegar",
    "llegaré": "llegar",
    "llegaréis": "llegar",
    "llegaría": "llegar",
    "llegaríais": "llegar",
    "llegaríamos": "llegar",
    "llegarían": "llegar",
    "llegarías": "llegar",
    "llegas": "llegar",
    "llegase": "llegar",
    "llegaseis": "llegar",
    "llegasen": "llegar",
    "llegases": "llegar",
    "llegaste": "llegar",
    "llegasteis": "llegar",
    "llego": "llegar",
    "llegue": "llegar",
    "lleguemos": "llegar",
    "lleguen": "llegar",
    "llegues": "llegar",
    "llegué": "llegar",
    "lleguéis": "llegar",
    "llegábamos": "llegar",
    "llegáis": "llegar",
    "llegáramos": "llegar",
    "llegáremos": "llegar",
    "llegásemos": "llegar",
    "llegó": "llegar",
    "llena": "lleno",
    "llenaba": "llenar",
    "llenabais": "llenar",
    "llenaban": "llenar",
    "llenabas": "llenar",
    "llenad": "llenar",
    "llenada": "llenar",
    "llenadas": "llenar",
    "llenado": "llenar",
    "llenadoras": "llenadora",
    "llenados": "llenar",
    "llenamos": "llenar",
    "llenan": "llenar",
    "llenando": "llenar",
    "llenara": "llenar",
    "llenarais": "llenar",
    "llenaran": "llenar",
    "llenaras": "llenar",
    "llenare": "llenar",
    "llenareis": "llenar",
    "llenaremos": "llenar",
    "llenaren": "llenar",
    "llenares": "llenar",
    "llenaron": "llenar",
    "llenará": "llenar",
    "llenarán": "llenar",
    "llenarás": "llenar",
    "llenaré": "llenar",
    "llenaréis": "llenar",
    "llenaría": "llenar",
    "llenaríais": "llenar",
    "llenaríamos": "llenar",
    "llenarían": "llenar",
    "llenarías": "llenar",
    "llenas": "lleno",
    "llenase": "llenar",
    "llenaseis": "llenar",
    "llenasen": "llenar",
    "llenases": "llenar",
    "llenaste": "llenar",
    "llenasteis": "llenar",
    "llene": "llenar",
    "llenemos": "llenar",
    "llenen": "llenar",
    "llenera": "llenero",
    "lleneras": "llenero",
    "lleneros": "llenero",
    "llenes": "llenar",
    "lleno": "llenar",
    "llenos": "lleno",
    "llenábamos": "llenar",
    "llenáis": "llenar",
    "llenáramos": "llenar",
    "llenáremos": "llenar",
    "llenásemos": "llenar",
    "llené": "llenar",
    "llenéis": "llenar",
    "llenó": "llenar",
    "lleuda": "lleudar",
    "lleudaba": "lleudar",
    "lleudabais": "lleudar",
    "lleudaban": "lleudar",
    "lleudabas": "lleudar",
    "lleudad": "lleudar",
    "lleudada": "lleudar",
    "lleudadas": "lleudar",
    "lleudado": "lleudar",
    "lleudados": "lleudar",
    "lleudamos": "lleudar",
    "lleudan": "lleudar",
    "lleudando": "lleudar",
    "lleudara": "lleudar",
    "lleudarais": "lleudar",
    "lleudaran": "lleudar",
    "lleudaras": "lleudar",
    "lleudare": "lleudar",
    "lleudareis": "lleudar",
    "lleudaremos": "lleudar",
    "lleudaren": "lleudar",
    "lleudares": "lleudar",
    "lleudaron": "lleudar",
    "lleudará": "lleudar",
    "lleudarán": "lleudar",
    "lleudarás": "lleudar",
    "lleudaré": "lleudar",
    "lleudaréis": "lleudar",
    "lleudaría": "lleudar",
    "lleudaríais": "lleudar",
    "lleudaríamos": "lleudar",
    "lleudarían": "lleudar",
    "lleudarías": "lleudar",
    "lleudas": "lleudar",
    "lleudase": "lleudar",
    "lleudaseis": "lleudar",
    "lleudasen": "lleudar",
    "lleudases": "lleudar",
    "lleudaste": "lleudar",
    "lleudasteis": "lleudar",
    "lleude": "lleudar",
    "lleudemos": "lleudar",
    "lleuden": "lleudar",
    "lleudes": "lleudar",
    "lleudo": "lleudar",
    "lleudábamos": "lleudar",
    "lleudáis": "lleudar",
    "lleudáramos": "lleudar",
    "lleudáremos": "lleudar",
    "lleudásemos": "lleudar",
    "lleudé": "lleudar",
    "lleudéis": "lleudar",
    "lleudó": "lleudar",
    "lleva": "llevar",
    "llevaba": "llevar",
    "llevabais": "llevar",
    "llevaban": "llevar",
    "llevabas": "llevar",
    "llevad": "llevar",
    "llevada": "llevar",
    "llevadas": "llevar",
    "llevadera": "llevadero",
    "llevaderas": "llevadero",
    "llevaderos": "llevadero",
    "llevado": "llevar",
    "llevadora": "llevador",
    "llevadoras": "llevador",
    "llevadores": "llevador",
    "llevados": "llevar",
    "llevamos": "llevar",
    "llevan": "llevar",
    "llevando": "llevar",
    "llevara": "llevar",
    "llevarais": "llevar",
    "llevaran": "llevar",
    "llevaras": "llevar",
    "llevare": "llevar",
    "llevareis": "llevar",
    "llevaremos": "llevar",
    "llevaren": "llevar",
    "llevares": "llevar",
    "llevaron": "llevar",
    "llevará": "llevar",
    "llevarán": "llevar",
    "llevarás": "llevar",
    "llevaré": "llevar",
    "llevaréis": "llevar",
    "llevaría": "llevar",
    "llevaríais": "llevar",
    "llevaríamos": "llevar",
    "llevarían": "llevar",
    "llevarías": "llevar",
    "llevas": "llevar",
    "llevase": "llevar",
    "llevaseis": "llevar",
    "llevasen": "llevar",
    "llevases": "llevar",
    "llevaste": "llevar",
    "llevasteis": "llevar",
    "lleve": "llevar",
    "llevemos": "llevar",
    "lleven": "llevar",
    "lleves": "llevar",
    "llevo": "llevar",
    "llevábamos": "llevar",
    "lleváis": "llevar",
    "lleváramos": "llevar",
    "lleváremos": "llevar",
    "llevásemos": "llevar",
    "llevé": "llevar",
    "llevéis": "llevar",
    "llevó": "llevar",
    "llocuras": "llocura",
    "llora": "llorar",
    "lloraba": "llorar",
    "llorabais": "llorar",
    "lloraban": "llorar",
    "llorabas": "llorar",
    "llorad": "llorar",
    "llorada": "llorar",
    "lloradas": "llorar",
    "llorado": "llorar",
    "lloradora": "llorador",
    "lloradoras": "llorador",
    "lloradores": "llorador",
    "llorados": "llorar",
    "lloramos": "llorar",
    "lloran": "llorar",
    "llorando": "llorar",
    "llorara": "llorar",
    "llorarais": "llorar",
    "lloraran": "llorar",
    "lloraras": "llorar",
    "llorare": "llorar",
    "llorareis": "llorar",
    "lloraremos": "llorar",
    "lloraren": "llorar",
    "llorares": "llorar",
    "lloraron": "llorar",
    "llorará": "llorar",
    "llorarán": "llorar",
    "llorarás": "llorar",
    "lloraré": "llorar",
    "lloraréis": "llorar",
    "lloraría": "llorar",
    "lloraríais": "llorar",
    "lloraríamos": "llorar",
    "llorarían": "llorar",
    "llorarías": "llorar",
    "lloras": "llorar",
    "llorase": "llorar",
    "lloraseis": "llorar",
    "llorasen": "llorar",
    "llorases": "llorar",
    "lloraste": "llorar",
    "llorasteis": "llorar",
    "llore": "llorar",
    "lloremos": "llorar",
    "lloren": "llorar",
    "lloreras": "llorera",
    "llores": "llorar",
    "lloricas": "llorica",
    "lloriquea": "lloriquear",
    "lloriqueaba": "lloriquear",
    "lloriqueabais": "lloriquear",
    "lloriqueaban": "lloriquear",
    "lloriqueabas": "lloriquear",
    "lloriquead": "lloriquear",
    "lloriqueada": "lloriquear",
    "lloriqueadas": "lloriquear",
    "lloriqueado": "lloriquear",
    "lloriqueados": "lloriquear",
    "lloriqueamos": "lloriquear",
    "lloriquean": "lloriquear",
    "lloriqueando": "lloriquear",
    "lloriqueara": "lloriquear",
    "lloriquearais": "lloriquear",
    "lloriquearan": "lloriquear",
    "lloriquearas": "lloriquear",
    "lloriqueare": "lloriquear",
    "lloriqueareis": "lloriquear",
    "lloriquearemos": "lloriquear",
    "lloriquearen": "lloriquear",
    "lloriqueares": "lloriquear",
    "lloriquearon": "lloriquear",
    "lloriqueará": "lloriquear",
    "lloriquearán": "lloriquear",
    "lloriquearás": "lloriquear",
    "lloriquearé": "lloriquear",
    "lloriquearéis": "lloriquear",
    "lloriquearía": "lloriquear",
    "lloriquearíais": "lloriquear",
    "lloriquearíamos": "lloriquear",
    "lloriquearían": "lloriquear",
    "lloriquearías": "lloriquear",
    "lloriqueas": "lloriquear",
    "lloriquease": "lloriquear",
    "lloriqueaseis": "lloriquear",
    "lloriqueasen": "lloriquear",
    "lloriqueases": "lloriquear",
    "lloriqueaste": "lloriquear",
    "lloriqueasteis": "lloriquear",
    "lloriquee": "lloriquear",
    "lloriqueemos": "lloriquear",
    "lloriqueen": "lloriquear",
    "lloriquees": "lloriquear",
    "lloriqueo": "lloriquear",
    "lloriqueos": "lloriqueo",
    "lloriqueábamos": "lloriquear",
    "lloriqueáis": "lloriquear",
    "lloriqueáramos": "lloriquear",
    "lloriqueáremos": "lloriquear",
    "lloriqueásemos": "lloriquear",
    "lloriqueé": "lloriquear",
    "lloriqueéis": "lloriquear",
    "lloriqueó": "lloriquear",
    "lloro": "llorar",
    "llorona": "llorón",
    "lloronas": "llorón",
    "llorones": "llorón",
    "lloros": "lloro",
    "llorosa": "lloroso",
    "llorosas": "lloroso",
    "llorosos": "lloroso",
    "llorábamos": "llorar",
    "lloráis": "llorar",
    "lloráramos": "llorar",
    "lloráremos": "llorar",
    "llorásemos": "llorar",
    "lloré": "llorar",
    "lloréis": "llorar",
    "lloró": "llorar",
    "llovediza": "llovedizo",
    "llovedizas": "llovedizo",
    "llovedizos": "llovedizo",
    "lloverá": "llover",
    "lloverán": "llover",
    "llovería": "llover",
    "lloverían": "llover",
    "llovido": "llover",
    "lloviendo": "llover",
    "lloviera": "llover",
    "llovieran": "llover",
    "llovieron": "llover",
    "lloviese": "llover",
    "lloviesen": "llover",
    "llovizna": "lloviznar",
    "lloviznaba": "lloviznar",
    "lloviznabais": "lloviznar",
    "lloviznaban": "lloviznar",
    "lloviznabas": "lloviznar",
    "lloviznad": "lloviznar",
    "lloviznada": "lloviznar",
    "lloviznadas": "lloviznar",
    "lloviznado": "lloviznar",
    "lloviznados": "lloviznar",
    "lloviznamos": "lloviznar",
    "lloviznan": "lloviznar",
    "lloviznando": "lloviznar",
    "lloviznara": "lloviznar",
    "lloviznarais": "lloviznar",
    "lloviznaran": "lloviznar",
    "lloviznaras": "lloviznar",
    "lloviznare": "lloviznar",
    "lloviznareis": "lloviznar",
    "lloviznaremos": "lloviznar",
    "lloviznaren": "lloviznar",
    "lloviznares": "lloviznar",
    "lloviznaron": "lloviznar",
    "lloviznará": "lloviznar",
    "lloviznarán": "lloviznar",
    "lloviznarás": "lloviznar",
    "lloviznaré": "lloviznar",
    "lloviznaréis": "lloviznar",
    "lloviznaría": "lloviznar",
    "lloviznaríais": "lloviznar",
    "lloviznaríamos": "lloviznar",
    "lloviznarían": "lloviznar",
    "lloviznarías": "lloviznar",
    "lloviznas": "lloviznar",
    "lloviznase": "lloviznar",
    "lloviznaseis": "lloviznar",
    "lloviznasen": "lloviznar",
    "lloviznases": "lloviznar",
    "lloviznaste": "lloviznar",
    "lloviznasteis": "lloviznar",
    "llovizne": "lloviznar",
    "lloviznemos": "lloviznar",
    "lloviznen": "lloviznar",
    "lloviznes": "lloviznar",
    "llovizno": "lloviznar",
    "lloviznábamos": "lloviznar",
    "lloviznáis": "lloviznar",
    "lloviznáramos": "lloviznar",
    "lloviznáremos": "lloviznar",
    "lloviznásemos": "lloviznar",
    "llovizné": "lloviznar",
    "lloviznéis": "lloviznar",
    "lloviznó": "lloviznar",
    "llovió": "llover",
    "llovía": "llover",
    "llovían": "llover",
    "lluecas": "llueca",
    "llueva": "llover",
    "lluevan": "llover",
    "llueve": "llover",
    "llueven": "llover",
    "lluvias": "lluvia",
    "lluviosa": "lluvioso",
    "lluviosas": "lluvioso",
    "lluviosos": "lluvioso",
    "llábanas": "llábana",
    "lo": "el",
    "loa": "loar",
    "loaba": "loar",
    "loabais": "loar",
    "loaban": "loar",
    "loabas": "loar",
    "loables": "loable",
    "load": "loar",
    "loada": "loar",
    "loadas": "loar",
    "loado": "loar",
    "loadora": "loador",
    "loadoras": "loador",
    "loadores": "loador",
    "loados": "loar",
    "loamos": "loar",
    "loan": "loar",
    "loandas": "loanda",
    "loando": "loar",
    "loara": "loar",
    "loarais": "loar",
    "loaran": "loar",
    "loaras": "loar",
    "loare": "loar",
    "loareis": "loar",
    "loaremos": "loar",
    "loaren": "loar",
    "loares": "loar",
    "loaron": "loar",
    "loará": "loar",
    "loarán": "loar",
    "loarás": "loar",
    "loaré": "loar",
    "loaréis": "loar",
    "loaría": "loar",
    "loaríais": "loar",
    "loaríamos": "loar",
    "loarían": "loar",
    "loarías": "loar",
    "loas": "loar",
    "loase": "loar",
    "loaseis": "loar",
    "loasen": "loar",
    "loases": "loar",
    "loaste": "loar",
    "loasteis": "loar",
    "loba": "lobo",
    "lobada": "lobado",
    "lobadas": "lobado",
    "lobados": "lobado",
    "lobagantes": "lobagante",
    "lobanillos": "lobanillo",
    "lobas": "lobo",
    "lobatos": "lobato",
    "lobbies": "lobby",
    "lobelias": "lobelia",
    "lobelinas": "lobelina",
    "lobeliáceas": "lobeliácea",
    "lobera": "lobero",
    "loberas": "lobero",
    "loberos": "lobero",
    "loberías": "lobería",
    "lobezna": "lobezno",
    "lobeznas": "lobezno",
    "lobeznos": "lobezno",
    "lobos": "lobo",
    "lobosa": "loboso",
    "lobosas": "loboso",
    "lobosos": "loboso",
    "lobreguece": "lobreguecer",
    "lobregueced": "lobreguecer",
    "lobreguecemos": "lobreguecer",
    "lobreguecen": "lobreguecer",
    "lobregueceremos": "lobreguecer",
    "lobreguecerá": "lobreguecer",
    "lobreguecerán": "lobreguecer",
    "lobreguecerás": "lobreguecer",
    "lobregueceré": "lobreguecer",
    "lobregueceréis": "lobreguecer",
    "lobreguecería": "lobreguecer",
    "lobregueceríais": "lobreguecer",
    "lobregueceríamos": "lobreguecer",
    "lobreguecerían": "lobreguecer",
    "lobreguecerías": "lobreguecer",
    "lobregueces": "lobreguez",
    "lobreguecida": "lobreguecer",
    "lobreguecidas": "lobreguecer",
    "lobreguecido": "lobreguecer",
    "lobreguecidos": "lobreguecer",
    "lobregueciendo": "lobreguecer",
    "lobregueciera": "lobreguecer",
    "lobreguecierais": "lobreguecer",
    "lobreguecieran": "lobreguecer",
    "lobreguecieras": "lobreguecer",
    "lobregueciere": "lobreguecer",
    "lobregueciereis": "lobreguecer",
    "lobreguecieren": "lobreguecer",
    "lobreguecieres": "lobreguecer",
    "lobreguecieron": "lobreguecer",
    "lobregueciese": "lobreguecer",
    "lobreguecieseis": "lobreguecer",
    "lobregueciesen": "lobreguecer",
    "lobreguecieses": "lobreguecer",
    "lobreguecimos": "lobreguecer",
    "lobregueciste": "lobreguecer",
    "lobreguecisteis": "lobreguecer",
    "lobregueciéramos": "lobreguecer",
    "lobregueciéremos": "lobreguecer",
    "lobregueciésemos": "lobreguecer",
    "lobregueció": "lobreguecer",
    "lobreguecéis": "lobreguecer",
    "lobreguecí": "lobreguecer",
    "lobreguecía": "lobreguecer",
    "lobreguecíais": "lobreguecer",
    "lobreguecíamos": "lobreguecer",
    "lobreguecían": "lobreguecer",
    "lobreguecías": "lobreguecer",
    "lobreguezca": "lobreguecer",
    "lobreguezcamos": "lobreguecer",
    "lobreguezcan": "lobreguecer",
    "lobreguezcas": "lobreguecer",
    "lobreguezco": "lobreguecer",
    "lobreguezcáis": "lobreguecer",
    "lobreguras": "lobregura",
    "lobs": "lob",
    "lobulada": "lobulado",
    "lobuladas": "lobulado",
    "lobulados": "lobulado",
    "lobulares": "lobular",
    "lobulillos": "lobulillo",
    "lobuna": "lobuno",
    "lobunas": "lobuno",
    "lobunos": "lobuno",
    "loca": "loco",
    "locaciones": "locación",
    "locadora": "locador",
    "locadoras": "locador",
    "locadores": "locador",
    "locales": "local",
    "localice": "localizar",
    "localicemos": "localizar",
    "localicen": "localizar",
    "localices": "localizar",
    "localicé": "localizar",
    "localicéis": "localizar",
    "localidades": "localidad",
    "localismos": "localismo",
    "localistas": "localista",
    "localiza": "localizar",
    "localizaba": "localizar",
    "localizabais": "localizar",
    "localizaban": "localizar",
    "localizabas": "localizar",
    "localizables": "localizable",
    "localizaciones": "localización",
    "localizad": "localizar",
    "localizada": "localizar",
    "localizadas": "localizar",
    "localizado": "localizar",
    "localizadora": "localizador",
    "localizadoras": "localizador",
    "localizadores": "localizador",
    "localizados": "localizar",
    "localizamos": "localizar",
    "localizan": "localizar",
    "localizando": "localizar",
    "localizara": "localizar",
    "localizarais": "localizar",
    "localizaran": "localizar",
    "localizaras": "localizar",
    "localizare": "localizar",
    "localizareis": "localizar",
    "localizaremos": "localizar",
    "localizaren": "localizar",
    "localizares": "localizar",
    "localizaron": "localizar",
    "localizará": "localizar",
    "localizarán": "localizar",
    "localizarás": "localizar",
    "localizaré": "localizar",
    "localizaréis": "localizar",
    "localizaría": "localizar",
    "localizaríais": "localizar",
    "localizaríamos": "localizar",
    "localizarían": "localizar",
    "localizarías": "localizar",
    "localizas": "localizar",
    "localizase": "localizar",
    "localizaseis": "localizar",
    "localizasen": "localizar",
    "localizases": "localizar",
    "localizaste": "localizar",
    "localizasteis": "localizar",
    "localizo": "localizar",
    "localizábamos": "localizar",
    "localizáis": "localizar",
    "localizáramos": "localizar",
    "localizáremos": "localizar",
    "localizásemos": "localizar",
    "localizó": "localizar",
    "locas": "loco",
    "locataria": "locatario",
    "locatarias": "locatario",
    "locatarios": "locatario",
    "locativa": "locativo",
    "locativas": "locativo",
    "locativos": "locativo",
    "locera": "locero",
    "loceras": "locero",
    "loceros": "locero",
    "locerías": "locería",
    "lochas": "locha",
    "loches": "loche",
    "lochs": "loch",
    "lociones": "loción",
    "lock-outs": "lock-out",
    "lockouts": "lockout",
    "locomociones": "locomoción",
    "locomotiva": "locomotivo",
    "locomotivas": "locomotivo",
    "locomotivos": "locomotivo",
    "locomotora": "locomotor",
    "locomotoras": "locomotora",
    "locomotores": "locomotor",
    "locomotrices": "locomotriz",
    "locomovibles": "locomovible",
    "locomóviles": "locomóvil",
    "locos": "loco",
    "locotractora": "locotractor",
    "locotractoras": "locotractor",
    "locotractores": "locotractor",
    "locros": "locro",
    "locuaces": "locuaz",
    "locuacidades": "locuacidad",
    "locuazes": "locuaz",
    "locuciones": "locución",
    "locuela": "locuelo",
    "locuelas": "locuelo",
    "locuelos": "locuelo",
    "loculares": "locular",
    "locuras": "locura",
    "locutiva": "locutivo",
    "locutivas": "locutivo",
    "locutivos": "locutivo",
    "locutora": "locutor",
    "locutoras": "locutor",
    "locutores": "locutor",
    "locutorios": "locutorio",
    "lodazales": "lodazal",
    "lodos": "lodo",
    "lodosa": "lodoso",
    "lodosas": "lodoso",
    "lodosos": "lodoso",
    "loe": "loar",
    "loemos": "loar",
    "loen": "loar",
    "loes": "loar",
    "lofodonta": "lofodonto",
    "lofodontas": "lofodonto",
    "lofodontos": "lofodonto",
    "logaritmos": "logaritmo",
    "logarítmica": "logarítmico",
    "logarítmicas": "logarítmico",
    "logarítmicos": "logarítmico",
    "logias": "logia",
    "logicals": "logical",
    "logicismos": "logicismo",
    "logicistas": "logicista",
    "logicosistemática": "logicosistemático",
    "logicosistemáticas": "logicosistemático",
    "logicosistemáticos": "logicosistemático",
    "logogrifos": "logogrifo",
    "logogrífica": "logogrífico",
    "logogríficas": "logogrífico",
    "logogríficos": "logogrífico",
    "logomaquias": "logomaquia",
    "logopedas": "logopeda",
    "logopedias": "logopedia",
    "logoplejías": "logoplejía",
    "logorreas": "logorrea",
    "logotipos": "logotipo",
    "logra": "lograr",
    "lograba": "lograr",
    "lograbais": "lograr",
    "lograban": "lograr",
    "lograbas": "lograr",
    "lograd": "lograr",
    "lograda": "lograr",
    "logradas": "lograr",
    "logrado": "lograr",
    "logrados": "lograr",
    "logramos": "lograr",
    "logran": "lograr",
    "logrando": "lograr",
    "lograra": "lograr",
    "lograrais": "lograr",
    "lograran": "lograr",
    "lograras": "lograr",
    "lograre": "lograr",
    "lograreis": "lograr",
    "lograremos": "lograr",
    "lograren": "lograr",
    "lograres": "lograr",
    "lograron": "lograr",
    "logrará": "lograr",
    "lograrán": "lograr",
    "lograrás": "lograr",
    "lograré": "lograr",
    "lograréis": "lograr",
    "lograría": "lograr",
    "lograríais": "lograr",
    "lograríamos": "lograr",
    "lograrían": "lograr",
    "lograrías": "lograr",
    "logras": "lograr",
    "lograse": "lograr",
    "lograseis": "lograr",
    "lograsen": "lograr",
    "lograses": "lograr",
    "lograste": "lograr",
    "lograsteis": "lograr",
    "logre": "lograr",
    "logrea": "logrear",
    "logreaba": "logrear",
    "logreabais": "logrear",
    "logreaban": "logrear",
    "logreabas": "logrear",
    "logread": "logrear",
    "logreada": "logrear",
    "logreadas": "logrear",
    "logreado": "logrear",
    "logreados": "logrear",
    "logreamos": "logrear",
    "logrean": "logrear",
    "logreando": "logrear",
    "logreara": "logrear",
    "logrearais": "logrear",
    "logrearan": "logrear",
    "logrearas": "logrear",
    "logreare": "logrear",
    "logreareis": "logrear",
    "logrearemos": "logrear",
    "logrearen": "logrear",
    "logreares": "logrear",
    "logrearon": "logrear",
    "logreará": "logrear",
    "logrearán": "logrear",
    "logrearás": "logrear",
    "logrearé": "logrear",
    "logrearéis": "logrear",
    "logrearía": "logrear",
    "logrearíais": "logrear",
    "logrearíamos": "logrear",
    "logrearían": "logrear",
    "logrearías": "logrear",
    "logreas": "logrear",
    "logrease": "logrear",
    "logreaseis": "logrear",
    "logreasen": "logrear",
    "logreases": "logrear",
    "logreaste": "logrear",
    "logreasteis": "logrear",
    "logree": "logrear",
    "logreemos": "logrear",
    "logreen": "logrear",
    "logrees": "logrear",
    "logremos": "lograr",
    "logren": "lograr",
    "logreo": "logrear",
    "logrera": "logrero",
    "logreras": "logrero",
    "logreros": "logrero",
    "logrerías": "logrería",
    "logres": "lograr",
    "logreábamos": "logrear",
    "logreáis": "logrear",
    "logreáramos": "logrear",
    "logreáremos": "logrear",
    "logreásemos": "logrear",
    "logreé": "logrear",
    "logreéis": "logrear",
    "logreó": "logrear",
    "logro": "lograr",
    "logros": "logro",
    "logroñesa": "logroñés",
    "logroñesas": "logroñés",
    "logroñeses": "logroñés",
    "logroñésa": "logroñés",
    "logroñésas": "logroñés",
    "logroñéses": "logroñés",
    "lográbamos": "lograr",
    "lográis": "lograr",
    "lográramos": "lograr",
    "lográremos": "lograr",
    "lográsemos": "lograr",
    "logré": "lograr",
    "logréis": "lograr",
    "logró": "lograr",
    "logística": "logístico",
    "logísticas": "logístico",
    "logísticos": "logístico",
    "logógrafos": "logógrafo",
    "loicas": "loica",
    "lokayatas": "lokayata",
    "lollingitas": "lollingita",
    "lomas": "loma",
    "lombarda": "lombardo",
    "lombardadas": "lombardada",
    "lombardas": "lombardo",
    "lombarderos": "lombardero",
    "lombarderías": "lombardería",
    "lombardos": "lombardo",
    "lombrices": "lombriz",
    "lombrigueras": "lombriguera",
    "lomea": "lomear",
    "lomeaba": "lomear",
    "lomeabais": "lomear",
    "lomeaban": "lomear",
    "lomeabas": "lomear",
    "lomead": "lomear",
    "lomeada": "lomear",
    "lomeadas": "lomear",
    "lomeado": "lomear",
    "lomeados": "lomear",
    "lomeamos": "lomear",
    "lomean": "lomear",
    "lomeando": "lomear",
    "lomeara": "lomear",
    "lomearais": "lomear",
    "lomearan": "lomear",
    "lomearas": "lomear",
    "lomeare": "lomear",
    "lomeareis": "lomear",
    "lomearemos": "lomear",
    "lomearen": "lomear",
    "lomeares": "lomear",
    "lomearon": "lomear",
    "lomeará": "lomear",
    "lomearán": "lomear",
    "lomearás": "lomear",
    "lomearé": "lomear",
    "lomearéis": "lomear",
    "lomearía": "lomear",
    "lomearíais": "lomear",
    "lomearíamos": "lomear",
    "lomearían": "lomear",
    "lomearías": "lomear",
    "lomeas": "lomear",
    "lomease": "lomear",
    "lomeaseis": "lomear",
    "lomeasen": "lomear",
    "lomeases": "lomear",
    "lomeaste": "lomear",
    "lomeasteis": "lomear",
    "lomee": "lomear",
    "lomeemos": "lomear",
    "lomeen": "lomear",
    "lomees": "lomear",
    "lomentos": "lomento",
    "lomeo": "lomear",
    "lomeras": "lomera",
    "lometas": "lometa",
    "lomeábamos": "lomear",
    "lomeáis": "lomear",
    "lomeáramos": "lomear",
    "lomeáremos": "lomear",
    "lomeásemos": "lomear",
    "lomeé": "lomear",
    "lomeéis": "lomear",
    "lomeó": "lomear",
    "lomillos": "lomillo",
    "lomos": "lomo",
    "lomuda": "lomudo",
    "lomudas": "lomudo",
    "lomudos": "lomudo",
    "lonas": "lona",
    "lonchas": "loncha",
    "londinenses": "londinense",
    "long-plays": "long-play",
    "longanimidades": "longanimidad",
    "longanizas": "longaniza",
    "longeva": "longevo",
    "longevas": "longevo",
    "longevidades": "longevidad",
    "longevos": "longevo",
    "longirrostra": "longirrostro",
    "longirrostras": "longirrostro",
    "longirrostros": "longirrostro",
    "longitudes": "longitud",
    "longitudinales": "longitudinal",
    "longobarda": "longobardo",
    "longobardas": "longobardo",
    "longobardos": "longobardo",
    "longplays": "longplay",
    "longueras": "longuera",
    "longuerías": "longuería",
    "longánima": "longánimo",
    "longánimas": "longánimo",
    "longánimos": "longánimo",
    "lonjas": "lonja",
    "lonjetas": "lonjeta",
    "lonjistas": "lonjista",
    "lonogramas": "lonograma",
    "lontananzas": "lontananza",
    "loo": "loar",
    "looks": "look",
    "loopings": "looping",
    "loores": "loor",
    "lopigias": "lopigia",
    "lopistas": "lopista",
    "loquea": "loquear",
    "loqueaba": "loquear",
    "loqueabais": "loquear",
    "loqueaban": "loquear",
    "loqueabas": "loquear",
    "loquead": "loquear",
    "loqueada": "loquear",
    "loqueadas": "loquear",
    "loqueado": "loquear",
    "loqueados": "loquear",
    "loqueamos": "loquear",
    "loquean": "loquear",
    "loqueando": "loquear",
    "loqueara": "loquear",
    "loquearais": "loquear",
    "loquearan": "loquear",
    "loquearas": "loquear",
    "loqueare": "loquear",
    "loqueareis": "loquear",
    "loquearemos": "loquear",
    "loquearen": "loquear",
    "loqueares": "loquear",
    "loquearon": "loquear",
    "loqueará": "loquear",
    "loquearán": "loquear",
    "loquearás": "loquear",
    "loquearé": "loquear",
    "loquearéis": "loquear",
    "loquearía": "loquear",
    "loquearíais": "loquear",
    "loquearíamos": "loquear",
    "loquearían": "loquear",
    "loquearías": "loquear",
    "loqueas": "loquear",
    "loquease": "loquear",
    "loqueaseis": "loquear",
    "loqueasen": "loquear",
    "loqueases": "loquear",
    "loqueaste": "loquear",
    "loqueasteis": "loquear",
    "loquee": "loquear",
    "loqueemos": "loquear",
    "loqueen": "loquear",
    "loquees": "loquear",
    "loqueo": "loquear",
    "loquera": "loquero",
    "loqueras": "loquero",
    "loqueros": "loquero",
    "loquesca": "loquesco",
    "loquescas": "loquesco",
    "loquescos": "loquesco",
    "loqueábamos": "loquear",
    "loqueáis": "loquear",
    "loqueáramos": "loquear",
    "loqueáremos": "loquear",
    "loqueásemos": "loquear",
    "loqueé": "loquear",
    "loqueéis": "loquear",
    "loqueó": "loquear",
    "loquiales": "loquial",
    "lorantáceas": "lorantácea",
    "lorchas": "lorcha",
    "lordótica": "lordótico",
    "lordóticas": "lordótico",
    "lordóticos": "lordótico",
    "lores": "lord",
    "lorigada": "lorigado",
    "lorigadas": "lorigado",
    "lorigados": "lorigado",
    "lorigas": "loriga",
    "lorigones": "lorigón",
    "loriguera": "loriguero",
    "lorigueras": "loriguero",
    "lorigueros": "loriguero",
    "loris": "lori",
    "loritos": "lorito",
    "loros": "loro",
    "lorquiana": "lorquiano",
    "lorquianas": "lorquiano",
    "lorquianos": "lorquiano",
    "lorzas": "lorza",
    "lorísidos": "lorísido",
    "los": "lo",
    "losa": "losar",
    "losaba": "losar",
    "losabais": "losar",
    "losaban": "losar",
    "losabas": "losar",
    "losad": "losar",
    "losada": "losar",
    "losadas": "losar",
    "losado": "losar",
    "losados": "losar",
    "losamos": "losar",
    "losan": "losar",
    "losando": "losar",
    "losanges": "losange",
    "losanjada": "losanjado",
    "losanjadas": "losanjado",
    "losanjados": "losanjado",
    "losara": "losar",
    "losarais": "losar",
    "losaran": "losar",
    "losaras": "losar",
    "losare": "losar",
    "losareis": "losar",
    "losaremos": "losar",
    "losaren": "losar",
    "losares": "losar",
    "losaron": "losar",
    "losará": "losar",
    "losarán": "losar",
    "losarás": "losar",
    "losaré": "losar",
    "losaréis": "losar",
    "losaría": "losar",
    "losaríais": "losar",
    "losaríamos": "losar",
    "losarían": "losar",
    "losarías": "losar",
    "losas": "losar",
    "losase": "losar",
    "losaseis": "losar",
    "losasen": "losar",
    "losases": "losar",
    "losaste": "losar",
    "losasteis": "losar",
    "lose": "losar",
    "losemos": "losar",
    "losen": "losar",
    "loses": "losar",
    "losetas": "loseta",
    "loso": "losar",
    "losábamos": "losar",
    "losáis": "losar",
    "losáramos": "losar",
    "losáremos": "losar",
    "losásemos": "losar",
    "losé": "losar",
    "loséis": "losar",
    "losó": "losar",
    "lotas": "lota",
    "loteos": "loteo",
    "lotera": "lotero",
    "loteras": "lotero",
    "loteros": "lotero",
    "loterías": "lotería",
    "lotes": "lote",
    "lotificaciones": "lotificación",
    "lotiformes": "lotiforme",
    "lotis": "loti",
    "lotos": "loto",
    "loxodromias": "loxodromia",
    "loxodrómica": "loxodrómico",
    "loxodrómicas": "loxodrómico",
    "loxodrómicos": "loxodrómico",
    "lozana": "lozano",
    "lozanas": "lozano",
    "lozanos": "lozano",
    "lozanías": "lozanía",
    "lozas": "loza",
    "loábamos": "loar",
    "loáis": "loar",
    "loáramos": "loar",
    "loáremos": "loar",
    "loásemos": "loar",
    "loé": "loar",
    "loéis": "loar",
    "loísmos": "loísmo",
    "loístas": "loísta",
    "loó": "loar",
    "lpes": "lp",
    "lps": "lp",
    "lubas": "luba",
    "lubinas": "lubina",
    "lubrica": "lubricar",
    "lubricaba": "lubricar",
    "lubricabais": "lubricar",
    "lubricaban": "lubricar",
    "lubricabas": "lubricar",
    "lubricaciones": "lubricación",
    "lubricad": "lubricar",
    "lubricada": "lubricar",
    "lubricadas": "lubricar",
    "lubricado": "lubricar",
    "lubricadora": "lubricador",
    "lubricadoras": "lubricador",
    "lubricadores": "lubricador",
    "lubricados": "lubricar",
    "lubricamos": "lubricar",
    "lubrican": "lubricar",
    "lubricando": "lubricar",
    "lubricantes": "lubricante",
    "lubricara": "lubricar",
    "lubricarais": "lubricar",
    "lubricaran": "lubricar",
    "lubricaras": "lubricar",
    "lubricare": "lubricar",
    "lubricareis": "lubricar",
    "lubricaremos": "lubricar",
    "lubricaren": "lubricar",
    "lubricares": "lubricar",
    "lubricaron": "lubricar",
    "lubricará": "lubricar",
    "lubricarán": "lubricar",
    "lubricarás": "lubricar",
    "lubricaré": "lubricar",
    "lubricaréis": "lubricar",
    "lubricaría": "lubricar",
    "lubricaríais": "lubricar",
    "lubricaríamos": "lubricar",
    "lubricarían": "lubricar",
    "lubricarías": "lubricar",
    "lubricas": "lubricar",
    "lubricase": "lubricar",
    "lubricaseis": "lubricar",
    "lubricasen": "lubricar",
    "lubricases": "lubricar",
    "lubricaste": "lubricar",
    "lubricasteis": "lubricar",
    "lubricativa": "lubricativo",
    "lubricativas": "lubricativo",
    "lubricativos": "lubricativo",
    "lubricidades": "lubricidad",
    "lubrico": "lubricar",
    "lubricábamos": "lubricar",
    "lubricáis": "lubricar",
    "lubricáramos": "lubricar",
    "lubricáremos": "lubricar",
    "lubricásemos": "lubricar",
    "lubricó": "lubricar",
    "lubrifica": "lubrificar",
    "lubrificaba": "lubrificar",
    "lubrificabais": "lubrificar",
    "lubrificaban": "lubrificar",
    "lubrificabas": "lubrificar",
    "lubrificables": "lubrificable",
    "lubrificaciones": "lubrificación",
    "lubrificad": "lubrificar",
    "lubrificada": "lubrificar",
    "lubrificadas": "lubrificar",
    "lubrificado": "lubrificar",
    "lubrificados": "lubrificar",
    "lubrificamos": "lubrificar",
    "lubrifican": "lubrificar",
    "lubrificando": "lubrificar",
    "lubrificantes": "lubrificante",
    "lubrificara": "lubrificar",
    "lubrificarais": "lubrificar",
    "lubrificaran": "lubrificar",
    "lubrificaras": "lubrificar",
    "lubrificare": "lubrificar",
    "lubrificareis": "lubrificar",
    "lubrificaremos": "lubrificar",
    "lubrificaren": "lubrificar",
    "lubrificares": "lubrificar",
    "lubrificaron": "lubrificar",
    "lubrificará": "lubrificar",
    "lubrificarán": "lubrificar",
    "lubrificarás": "lubrificar",
    "lubrificaré": "lubrificar",
    "lubrificaréis": "lubrificar",
    "lubrificaría": "lubrificar",
    "lubrificaríais": "lubrificar",
    "lubrificaríamos": "lubrificar",
    "lubrificarían": "lubrificar",
    "lubrificarías": "lubrificar",
    "lubrificas": "lubrificar",
    "lubrificase": "lubrificar",
    "lubrificaseis": "lubrificar",
    "lubrificasen": "lubrificar",
    "lubrificases": "lubrificar",
    "lubrificaste": "lubrificar",
    "lubrificasteis": "lubrificar",
    "lubrifico": "lubrificar",
    "lubrificábamos": "lubrificar",
    "lubrificáis": "lubrificar",
    "lubrificáramos": "lubrificar",
    "lubrificáremos": "lubrificar",
    "lubrificásemos": "lubrificar",
    "lubrificó": "lubrificar",
    "lubrifique": "lubrificar",
    "lubrifiquemos": "lubrificar",
    "lubrifiquen": "lubrificar",
    "lubrifiques": "lubrificar",
    "lubrifiqué": "lubrificar",
    "lubrifiquéis": "lubrificar",
    "lubrique": "lubricar",
    "lubriquemos": "lubricar",
    "lubriquen": "lubricar",
    "lubriques": "lubricar",
    "lubriqué": "lubricar",
    "lubriquéis": "lubricar",
    "luce": "lucir",
    "lucen": "lucir",
    "lucenses": "lucense",
    "lucernarios": "lucernario",
    "lucernas": "lucerna",
    "luceros": "lucero",
    "luces": "luz",
    "lucha": "luchar",
    "luchaba": "luchar",
    "luchabais": "luchar",
    "luchaban": "luchar",
    "luchabas": "luchar",
    "luchad": "luchar",
    "luchada": "luchar",
    "luchadas": "luchar",
    "luchado": "luchar",
    "luchadora": "luchador",
    "luchadoras": "luchador",
    "luchadores": "luchador",
    "luchados": "luchar",
    "luchamos": "luchar",
    "luchan": "luchar",
    "luchando": "luchar",
    "luchara": "luchar",
    "lucharais": "luchar",
    "lucharan": "luchar",
    "lucharas": "luchar",
    "luchare": "luchar",
    "luchareis": "luchar",
    "lucharemos": "luchar",
    "lucharen": "luchar",
    "luchares": "luchar",
    "lucharniega": "lucharniego",
    "lucharniegas": "lucharniego",
    "lucharniegos": "lucharniego",
    "lucharon": "luchar",
    "luchará": "luchar",
    "lucharán": "luchar",
    "lucharás": "luchar",
    "lucharé": "luchar",
    "lucharéis": "luchar",
    "lucharía": "luchar",
    "lucharíais": "luchar",
    "lucharíamos": "luchar",
    "lucharían": "luchar",
    "lucharías": "luchar",
    "luchas": "luchar",
    "luchase": "luchar",
    "luchaseis": "luchar",
    "luchasen": "luchar",
    "luchases": "luchar",
    "luchaste": "luchar",
    "luchasteis": "luchar",
    "luche": "luchar",
    "luchemos": "luchar",
    "luchen": "luchar",
    "luches": "luchar",
    "lucho": "luchar",
    "luchábamos": "luchar",
    "lucháis": "luchar",
    "lucháramos": "luchar",
    "lucháremos": "luchar",
    "luchásemos": "luchar",
    "luché": "luchar",
    "luchéis": "luchar",
    "luchó": "luchar",
    "lucia": "lucio",
    "lucias": "lucio",
    "lucid": "lucir",
    "lucida": "lucir",
    "lucidas": "lucir",
    "lucideces": "lucidez",
    "lucidificaciones": "lucidificación",
    "lucido": "lucir",
    "lucidora": "lucidor",
    "lucidoras": "lucidor",
    "lucidores": "lucidor",
    "lucidos": "lucir",
    "lucidoscopios": "lucidoscopio",
    "luciduras": "lucidura",
    "luciendo": "lucir",
    "lucientes": "luciente",
    "luciera": "lucir",
    "lucierais": "lucir",
    "lucieran": "lucir",
    "lucieras": "lucir",
    "luciere": "lucir",
    "luciereis": "lucir",
    "lucieren": "lucir",
    "lucieres": "lucir",
    "lucieron": "lucir",
    "luciese": "lucir",
    "lucieseis": "lucir",
    "luciesen": "lucir",
    "lucieses": "lucir",
    "luciferes": "lucifer",
    "luciferina": "luciferino",
    "luciferinas": "luciferino",
    "luciferinos": "luciferino",
    "lucimientos": "lucimiento",
    "lucimos": "lucir",
    "luciones": "lución",
    "lucios": "lucio",
    "luciremos": "lucir",
    "lucirá": "lucir",
    "lucirán": "lucir",
    "lucirás": "lucir",
    "luciré": "lucir",
    "luciréis": "lucir",
    "luciría": "lucir",
    "luciríais": "lucir",
    "luciríamos": "lucir",
    "lucirían": "lucir",
    "lucirías": "lucir",
    "luciste": "lucir",
    "lucisteis": "lucir",
    "luciéramos": "lucir",
    "luciéremos": "lucir",
    "luciérnagas": "luciérnaga",
    "luciésemos": "lucir",
    "lució": "lucir",
    "lucra": "lucrar",
    "lucraba": "lucrar",
    "lucrabais": "lucrar",
    "lucraban": "lucrar",
    "lucrabas": "lucrar",
    "lucrad": "lucrar",
    "lucrada": "lucrar",
    "lucradas": "lucrar",
    "lucrado": "lucrar",
    "lucrados": "lucrar",
    "lucramos": "lucrar",
    "lucran": "lucrar",
    "lucrando": "lucrar",
    "lucrara": "lucrar",
    "lucrarais": "lucrar",
    "lucraran": "lucrar",
    "lucraras": "lucrar",
    "lucrare": "lucrar",
    "lucrareis": "lucrar",
    "lucraremos": "lucrar",
    "lucraren": "lucrar",
    "lucrares": "lucrar",
    "lucraron": "lucrar",
    "lucrará": "lucrar",
    "lucrarán": "lucrar",
    "lucrarás": "lucrar",
    "lucraré": "lucrar",
    "lucraréis": "lucrar",
    "lucraría": "lucrar",
    "lucraríais": "lucrar",
    "lucraríamos": "lucrar",
    "lucrarían": "lucrar",
    "lucrarías": "lucrar",
    "lucras": "lucrar",
    "lucrase": "lucrar",
    "lucraseis": "lucrar",
    "lucrasen": "lucrar",
    "lucrases": "lucrar",
    "lucraste": "lucrar",
    "lucrasteis": "lucrar",
    "lucrativa": "lucrativo",
    "lucrativas": "lucrativo",
    "lucrativos": "lucrativo",
    "lucre": "lucrar",
    "lucremos": "lucrar",
    "lucren": "lucrar",
    "lucres": "lucrar",
    "lucro": "lucrar",
    "lucros": "lucro",
    "lucrosa": "lucroso",
    "lucrosas": "lucroso",
    "lucrosos": "lucroso",
    "lucrábamos": "lucrar",
    "lucráis": "lucrar",
    "lucráramos": "lucrar",
    "lucráremos": "lucrar",
    "lucrásemos": "lucrar",
    "lucré": "lucrar",
    "lucréis": "lucrar",
    "lucró": "lucrar",
    "luctuosa": "luctuoso",
    "luctuosas": "luctuoso",
    "luctuosos": "luctuoso",
    "lucubra": "lucubrar",
    "lucubraba": "lucubrar",
    "lucubrabais": "lucubrar",
    "lucubraban": "lucubrar",
    "lucubrabas": "lucubrar",
    "lucubraciones": "lucubración",
    "lucubrad": "lucubrar",
    "lucubrada": "lucubrar",
    "lucubradas": "lucubrar",
    "lucubrado": "lucubrar",
    "lucubrados": "lucubrar",
    "lucubramos": "lucubrar",
    "lucubran": "lucubrar",
    "lucubrando": "lucubrar",
    "lucubrara": "lucubrar",
    "lucubrarais": "lucubrar",
    "lucubraran": "lucubrar",
    "lucubraras": "lucubrar",
    "lucubrare": "lucubrar",
    "lucubrareis": "lucubrar",
    "lucubraremos": "lucubrar",
    "lucubraren": "lucubrar",
    "lucubrares": "lucubrar",
    "lucubraron": "lucubrar",
    "lucubrará": "lucubrar",
    "lucubrarán": "lucubrar",
    "lucubrarás": "lucubrar",
    "lucubraré": "lucubrar",
    "lucubraréis": "lucubrar",
    "lucubraría": "lucubrar",
    "lucubraríais": "lucubrar",
    "lucubraríamos": "lucubrar",
    "lucubrarían": "lucubrar",
    "lucubrarías": "lucubrar",
    "lucubras": "lucubrar",
    "lucubrase": "lucubrar",
    "lucubraseis": "lucubrar",
    "lucubrasen": "lucubrar",
    "lucubrases": "lucubrar",
    "lucubraste": "lucubrar",
    "lucubrasteis": "lucubrar",
    "lucubre": "lucubrar",
    "lucubremos": "lucubrar",
    "lucubren": "lucubrar",
    "lucubres": "lucubrar",
    "lucubro": "lucubrar",
    "lucubrábamos": "lucubrar",
    "lucubráis": "lucubrar",
    "lucubráramos": "lucubrar",
    "lucubráremos": "lucubrar",
    "lucubrásemos": "lucubrar",
    "lucubré": "lucubrar",
    "lucubréis": "lucubrar",
    "lucubró": "lucubrar",
    "lucí": "lucir",
    "lucía": "lucir",
    "lucíais": "lucir",
    "lucíamos": "lucir",
    "lucían": "lucir",
    "lucías": "lucir",
    "lucífera": "lucífero",
    "lucíferas": "lucífero",
    "lucíferos": "lucífero",
    "lucífuga": "lucífugo",
    "lucífugas": "lucífugo",
    "lucífugos": "lucífugo",
    "lucís": "lucir",
    "luda": "ludir",
    "ludamos": "ludir",
    "ludan": "ludir",
    "ludas": "ludir",
    "lude": "ludir",
    "luden": "ludir",
    "ludes": "ludir",
    "ludibrios": "ludibrio",
    "ludid": "ludir",
    "ludida": "ludir",
    "ludidas": "ludir",
    "ludido": "ludir",
    "ludidos": "ludir",
    "ludiendo": "ludir",
    "ludiera": "ludir",
    "ludierais": "ludir",
    "ludieran": "ludir",
    "ludieras": "ludir",
    "ludiere": "ludir",
    "ludiereis": "ludir",
    "ludieren": "ludir",
    "ludieres": "ludir",
    "ludieron": "ludir",
    "ludiese": "ludir",
    "ludieseis": "ludir",
    "ludiesen": "ludir",
    "ludieses": "ludir",
    "ludimientos": "ludimiento",
    "ludimos": "ludir",
    "ludiones": "ludión",
    "ludiremos": "ludir",
    "ludirá": "ludir",
    "ludirán": "ludir",
    "ludirás": "ludir",
    "ludiré": "ludir",
    "ludiréis": "ludir",
    "ludiría": "ludir",
    "ludiríais": "ludir",
    "ludiríamos": "ludir",
    "ludirían": "ludir",
    "ludirías": "ludir",
    "ludismos": "ludismo",
    "ludiste": "ludir",
    "ludisteis": "ludir",
    "ludiéramos": "ludir",
    "ludiéremos": "ludir",
    "ludiésemos": "ludir",
    "ludió": "ludir",
    "ludo": "ludir",
    "ludopatías": "ludopatía",
    "ludos": "ludo",
    "ludotecas": "ludoteca",
    "ludáis": "ludir",
    "ludí": "ludir",
    "ludía": "ludir",
    "ludíais": "ludir",
    "ludíamos": "ludir",
    "ludían": "ludir",
    "ludías": "ludir",
    "ludís": "ludir",
    "luenga": "luengo",
    "luengas": "luengo",
    "luengos": "luengo",
    "lugares": "lugar",
    "lugareña": "lugareño",
    "lugareñas": "lugareño",
    "lugareños": "lugareño",
    "lugartenencias": "lugartenencia",
    "lugartenientes": "lugarteniente",
    "luges": "luge",
    "lugres": "lugre",
    "luiciones": "luición",
    "luid": "luir",
    "luida": "luir",
    "luidas": "luir",
    "luido": "luir",
    "luidos": "luir",
    "luimos": "luir",
    "luiremos": "luir",
    "luirá": "luir",
    "luirán": "luir",
    "luirás": "luir",
    "luiré": "luir",
    "luiréis": "luir",
    "luiría": "luir",
    "luiríais": "luir",
    "luiríamos": "luir",
    "luirían": "luir",
    "luirías": "luir",
    "luisas": "luisa",
    "luises": "luis",
    "luiste": "luir",
    "luisteis": "luir",
    "lujos": "lujo",
    "lujosa": "lujoso",
    "lujosas": "lujoso",
    "lujosos": "lujoso",
    "lujuria": "lujuriar",
    "lujuriaba": "lujuriar",
    "lujuriabais": "lujuriar",
    "lujuriaban": "lujuriar",
    "lujuriabas": "lujuriar",
    "lujuriad": "lujuriar",
    "lujuriada": "lujuriar",
    "lujuriadas": "lujuriar",
    "lujuriado": "lujuriar",
    "lujuriados": "lujuriar",
    "lujuriamos": "lujuriar",
    "lujurian": "lujuriar",
    "lujuriando": "lujuriar",
    "lujuriantes": "lujuriante",
    "lujuriara": "lujuriar",
    "lujuriarais": "lujuriar",
    "lujuriaran": "lujuriar",
    "lujuriaras": "lujuriar",
    "lujuriare": "lujuriar",
    "lujuriareis": "lujuriar",
    "lujuriaremos": "lujuriar",
    "lujuriaren": "lujuriar",
    "lujuriares": "lujuriar",
    "lujuriaron": "lujuriar",
    "lujuriará": "lujuriar",
    "lujuriarán": "lujuriar",
    "lujuriarás": "lujuriar",
    "lujuriaré": "lujuriar",
    "lujuriaréis": "lujuriar",
    "lujuriaría": "lujuriar",
    "lujuriaríais": "lujuriar",
    "lujuriaríamos": "lujuriar",
    "lujuriarían": "lujuriar",
    "lujuriarías": "lujuriar",
    "lujurias": "lujuriar",
    "lujuriase": "lujuriar",
    "lujuriaseis": "lujuriar",
    "lujuriasen": "lujuriar",
    "lujuriases": "lujuriar",
    "lujuriaste": "lujuriar",
    "lujuriasteis": "lujuriar",
    "lujurie": "lujuriar",
    "lujuriemos": "lujuriar",
    "lujurien": "lujuriar",
    "lujuries": "lujuriar",
    "lujurio": "lujuriar",
    "lujuriosa": "lujurioso",
    "lujuriosas": "lujurioso",
    "lujuriosos": "lujurioso",
    "lujuriábamos": "lujuriar",
    "lujuriáis": "lujuriar",
    "lujuriáramos": "lujuriar",
    "lujuriáremos": "lujuriar",
    "lujuriásemos": "lujuriar",
    "lujurié": "lujuriar",
    "lujuriéis": "lujuriar",
    "lujurió": "lujuriar",
    "luliana": "luliano",
    "lulianas": "luliano",
    "lulianos": "luliano",
    "lulismos": "lulismo",
    "lulistas": "lulista",
    "lulus": "lulú",
    "lulúes": "lulú",
    "lulús": "lulú",
    "lumaquelas": "lumaquela",
    "lumbagos": "lumbago",
    "lumbalgias": "lumbalgia",
    "lumbares": "lumbar",
    "lumbocostales": "lumbocostal",
    "lumbodorsales": "lumbodorsal",
    "lumbosacra": "lumbosacro",
    "lumbosacras": "lumbosacro",
    "lumbosacros": "lumbosacro",
    "lumbradas": "lumbrada",
    "lumbraradas": "lumbrarada",
    "lumbreras": "lumbrera",
    "lumbres": "lumbre",
    "lumbricales": "lumbrical",
    "luminales": "luminal",
    "luminancias": "luminancia",
    "luminares": "luminar",
    "luminarias": "luminaria",
    "luminiscencias": "luminiscencia",
    "luminiscentes": "luminiscente",
    "luminosa": "luminoso",
    "luminosas": "luminoso",
    "luminosidades": "luminosidad",
    "luminosos": "luminoso",
    "luminotecnias": "luminotecnia",
    "luminotécnica": "luminotécnico",
    "luminotécnicas": "luminotécnico",
    "luminotécnicos": "luminotécnico",
    "luminífera": "luminífero",
    "luminíferas": "luminífero",
    "luminíferos": "luminífero",
    "luminóforos": "luminóforo",
    "lumpenproletariados": "lumpenproletariado",
    "lumínica": "lumínico",
    "lumínicas": "lumínico",
    "lumínicos": "lumínico",
    "lunaciones": "lunación",
    "lunada": "lunado",
    "lunadas": "lunado",
    "lunados": "lunado",
    "lunanca": "lunanco",
    "lunancas": "lunanco",
    "lunancos": "lunanco",
    "lunares": "lunar",
    "lunaria": "lunario",
    "lunarias": "lunario",
    "lunarios": "lunario",
    "lunas": "luna",
    "lunches": "lunch",
    "lunecillas": "lunecilla",
    "luneles": "lunel",
    "lunetas": "luneta",
    "lunetos": "luneto",
    "lunfarda": "lunfardo",
    "lunfardas": "lunfardo",
    "lunfardismos": "lunfardismo",
    "lunfardos": "lunfardo",
    "lunática": "lunático",
    "lunáticas": "lunático",
    "lunáticos": "lunático",
    "lupanares": "lupanar",
    "lupanaria": "lupanario",
    "lupanarias": "lupanario",
    "lupanarios": "lupanario",
    "lupaninas": "lupanina",
    "lupas": "lupa",
    "lupias": "lupia",
    "lupicias": "lupicia",
    "lupina": "lupino",
    "lupinas": "lupino",
    "lupinos": "lupino",
    "lupulinas": "lupulina",
    "luquetes": "luquete",
    "lusa": "luso",
    "lusas": "luso",
    "lusitana": "lusitano",
    "lusitanas": "lusitano",
    "lusitanismos": "lusitanismo",
    "lusitanos": "lusitano",
    "lusitánica": "lusitánico",
    "lusitánicas": "lusitánico",
    "lusitánicos": "lusitánico",
    "lusos": "luso",
    "lustra": "lustrar",
    "lustraba": "lustrar",
    "lustrabais": "lustrar",
    "lustraban": "lustrar",
    "lustrabas": "lustrar",
    "lustraciones": "lustración",
    "lustrad": "lustrar",
    "lustrada": "lustrar",
    "lustradas": "lustrar",
    "lustrado": "lustrar",
    "lustradora": "lustrador",
    "lustradoras": "lustrador",
    "lustradores": "lustrador",
    "lustrados": "lustrar",
    "lustrales": "lustral",
    "lustramos": "lustrar",
    "lustran": "lustrar",
    "lustrando": "lustrar",
    "lustrara": "lustrar",
    "lustrarais": "lustrar",
    "lustraran": "lustrar",
    "lustraras": "lustrar",
    "lustrare": "lustrar",
    "lustrareis": "lustrar",
    "lustraremos": "lustrar",
    "lustraren": "lustrar",
    "lustrares": "lustrar",
    "lustraron": "lustrar",
    "lustrará": "lustrar",
    "lustrarán": "lustrar",
    "lustrarás": "lustrar",
    "lustraré": "lustrar",
    "lustraréis": "lustrar",
    "lustraría": "lustrar",
    "lustraríais": "lustrar",
    "lustraríamos": "lustrar",
    "lustrarían": "lustrar",
    "lustrarías": "lustrar",
    "lustras": "lustrar",
    "lustrase": "lustrar",
    "lustraseis": "lustrar",
    "lustrasen": "lustrar",
    "lustrases": "lustrar",
    "lustraste": "lustrar",
    "lustrasteis": "lustrar",
    "lustre": "lustrar",
    "lustremos": "lustrar",
    "lustren": "lustrar",
    "lustres": "lustre",
    "lustrina": "lustrín",
    "lustrinas": "lustrina",
    "lustrines": "lustrín",
    "lustro": "lustrar",
    "lustros": "lustro",
    "lustrosa": "lustroso",
    "lustrosas": "lustroso",
    "lustrosos": "lustroso",
    "lustrábamos": "lustrar",
    "lustráis": "lustrar",
    "lustráramos": "lustrar",
    "lustráremos": "lustrar",
    "lustrásemos": "lustrar",
    "lustré": "lustrar",
    "lustréis": "lustrar",
    "lustró": "lustrar",
    "lutecios": "lutecio",
    "luteoskirinas": "luteoskirina",
    "luterana": "luterano",
    "luteranas": "luterano",
    "luteranismos": "luteranismo",
    "luteranos": "luterano",
    "luteínas": "luteína",
    "luthieres": "luthier",
    "lutidinas": "lutidina",
    "lutos": "luto",
    "luvisoles": "luvisol",
    "luxa": "luxar",
    "luxaba": "luxar",
    "luxabais": "luxar",
    "luxaban": "luxar",
    "luxabas": "luxar",
    "luxaciones": "luxación",
    "luxaciónes": "luxación",
    "luxad": "luxar",
    "luxada": "luxar",
    "luxadas": "luxar",
    "luxado": "luxar",
    "luxados": "luxar",
    "luxamos": "luxar",
    "luxan": "luxar",
    "luxando": "luxar",
    "luxara": "luxar",
    "luxarais": "luxar",
    "luxaran": "luxar",
    "luxaras": "luxar",
    "luxare": "luxar",
    "luxareis": "luxar",
    "luxaremos": "luxar",
    "luxaren": "luxar",
    "luxares": "luxar",
    "luxaron": "luxar",
    "luxará": "luxar",
    "luxarán": "luxar",
    "luxarás": "luxar",
    "luxaré": "luxar",
    "luxaréis": "luxar",
    "luxaría": "luxar",
    "luxaríais": "luxar",
    "luxaríamos": "luxar",
    "luxarían": "luxar",
    "luxarías": "luxar",
    "luxas": "luxar",
    "luxase": "luxar",
    "luxaseis": "luxar",
    "luxasen": "luxar",
    "luxases": "luxar",
    "luxaste": "luxar",
    "luxasteis": "luxar",
    "luxe": "luxar",
    "luxemburguesa": "luxemburgués",
    "luxemburguesas": "luxemburgués",
    "luxemburgueses": "luxemburgués",
    "luxemos": "luxar",
    "luxen": "luxar",
    "luxes": "luxar",
    "luxo": "luxar",
    "luxs": "lux",
    "luxábamos": "luxar",
    "luxáis": "luxar",
    "luxáramos": "luxar",
    "luxáremos": "luxar",
    "luxásemos": "luxar",
    "luxé": "luxar",
    "luxéis": "luxar",
    "luxímetros": "luxímetro",
    "luxó": "luxar",
    "luxómetros": "luxómetro",
    "luya": "luir",
    "luyamos": "luir",
    "luyan": "luir",
    "luyas": "luir",
    "luye": "luir",
    "luyen": "luir",
    "luyendo": "luir",
    "luyera": "luir",
    "luyerais": "luir",
    "luyeran": "luir",
    "luyeras": "luir",
    "luyere": "luir",
    "luyereis": "luir",
    "luyeren": "luir",
    "luyeres": "luir",
    "luyeron": "luir",
    "luyes": "luir",
    "luyese": "luir",
    "luyeseis": "luir",
    "luyesen": "luir",
    "luyeses": "luir",
    "luyo": "luir",
    "luyáis": "luir",
    "luyéramos": "luir",
    "luyéremos": "luir",
    "luyésemos": "luir",
    "luyó": "luir",
    "luzca": "lucir",
    "luzcamos": "lucir",
    "luzcan": "lucir",
    "luzcas": "lucir",
    "luzco": "lucir",
    "luzcáis": "lucir",
    "luzonitas": "luzonita",
    "luí": "luir",
    "luía": "luir",
    "luíais": "luir",
    "luíamos": "luir",
    "luían": "luir",
    "luías": "luir",
    "luís": "luir",
    "lx": "lux",
    "lycra": "licra",
    "lycras": "licra",
    "lábaros": "lábaro",
    "lábiles": "lábil",
    "lábiumes": "lábium",
    "lábridos": "lábrido",
    "láctea": "lácteo",
    "lácteas": "lácteo",
    "lácteos": "lácteo",
    "láctica": "láctico",
    "lácticas": "láctico",
    "lácticos": "láctico",
    "ládanos": "ládano",
    "lágrimas": "lágrima",
    "láminas": "lámina",
    "lámparas": "lámpara",
    "lánguida": "lánguido",
    "lánguidas": "lánguido",
    "lánguidos": "lánguido",
    "lápades": "lápade",
    "lápices": "lápiz",
    "lápidas": "lápida",
    "lárices": "lárice",
    "láseres": "láser",
    "lásers": "láser",
    "láserses": "láser",
    "lástimas": "lástima",
    "látexes": "látex",
    "látigos": "látigo",
    "láudanos": "láudano",
    "láurea": "láureo",
    "láureas": "láureo",
    "láureos": "láureo",
    "láurica": "láurico",
    "láuricas": "láurico",
    "láuricos": "láurico",
    "länder": "land",
    "légamos": "légamo",
    "léganos": "légano",
    "lémures": "lémur",
    "léptica": "léptico",
    "lépticas": "léptico",
    "lépticos": "léptico",
    "lésbica": "lésbico",
    "lésbicas": "lésbico",
    "lésbicos": "lésbico",
    "léxica": "léxico",
    "léxicas": "léxico",
    "léxicos": "léxico",
    "lía": "liar",
    "lían": "liar",
    "lías": "liar",
    "líberes": "líber",
    "líberos": "líbero",
    "líbica": "líbico",
    "líbicas": "líbico",
    "líbicos": "líbico",
    "líbidos": "líbido",
    "lícita": "lícito",
    "lícitas": "lícito",
    "lícitos": "lícito",
    "lídares": "lídar",
    "líderes": "líder",
    "líe": "liar",
    "líen": "liar",
    "líes": "liar",
    "lígulas": "lígula",
    "límenes": "limen",
    "límites": "límite",
    "límnica": "límnico",
    "límnicas": "límnico",
    "límnicos": "límnico",
    "límpida": "límpido",
    "límpidas": "límpido",
    "límpidos": "límpido",
    "línea": "líneo",
    "líneas": "líneo",
    "líneos": "líneo",
    "línteres": "línter",
    "lío": "liar",
    "líos": "lío",
    "lípidos": "lípido",
    "líquenes": "liquen",
    "líquida": "líquido",
    "líquidas": "líquido",
    "líquidos": "líquido",
    "lírica": "lírico",
    "líricas": "lírico",
    "líricos": "lírico",
    "lítica": "lítico",
    "líticas": "lítico",
    "líticos": "lítico",
    "lívida": "lívido",
    "lívidas": "lívido",
    "lívidos": "lívido",
    "lóbrega": "lóbrego",
    "lóbregas": "lóbrego",
    "lóbregos": "lóbrego",
    "lóbulos": "lóbulo",
    "lóculos": "lóculo",
    "lógica": "lógico",
    "lógicas": "lógico",
    "lógico-jurídica": "lógico-jurídico",
    "lógico-jurídicas": "lógico-jurídico",
    "lógico-jurídicos": "lógico-jurídico",
    "lógicos": "lógico",
    "lóndigas": "lóndiga",
    "lótuses": "lotus",
    "lúas": "lúa",
    "lúbrica": "lúbrico",
    "lúbricas": "lúbrico",
    "lúbricos": "lúbrico",
    "lúcida": "lúcido",
    "lúcidas": "lúcido",
    "lúcidos": "lúcido",
    "lúdica": "lúdico",
    "lúdicas": "lúdico",
    "lúdicos": "lúdico",
    "lúdrica": "lúdrico",
    "lúdricas": "lúdrico",
    "lúdricos": "lúdrico",
    "lúes": "lúe",
    "lúganos": "lúgano",
    "lúgubres": "lúgubre",
    "lúmenes": "lumen",
    "lúmpenes": "lumpen",
    "lúnulas": "lúnula",
    "lúpulos": "lúpulo",
    "lúpuses": "lupus",
    "lústrica": "lústrico",
    "lústricas": "lústrico",
    "lústricos": "lústrico",
    "lútea": "lúteo",
    "lúteas": "lúteo",
    "lúteos": "lúteo",
    "lúvica": "lúvico",
    "lúvicas": "lúvico",
    "lúvicos": "lúvico",
    "m": "metro",
    "maares": "maar",
    "maars": "maar",
    "mabolos": "mabolo",
    "mabrelaciones": "mabrelación",
    "maca": "macar",
    "macaba": "macar",
    "macabais": "macar",
    "macaban": "macar",
    "macabas": "macar",
    "macabra": "macabro",
    "macabras": "macabro",
    "macabros": "macabro",
    "macaca": "macaco",
    "macacas": "macaco",
    "macacos": "macaco",
    "macad": "macar",
    "macada": "macar",
    "macadames": "macadam",
    "macadamice": "macadamizar",
    "macadamicemos": "macadamizar",
    "macadamicen": "macadamizar",
    "macadamices": "macadamizar",
    "macadamicé": "macadamizar",
    "macadamicéis": "macadamizar",
    "macadamiza": "macadamizar",
    "macadamizaba": "macadamizar",
    "macadamizabais": "macadamizar",
    "macadamizaban": "macadamizar",
    "macadamizabas": "macadamizar",
    "macadamizad": "macadamizar",
    "macadamizada": "macadamizar",
    "macadamizadas": "macadamizar",
    "macadamizado": "macadamizar",
    "macadamizados": "macadamizar",
    "macadamizamos": "macadamizar",
    "macadamizan": "macadamizar",
    "macadamizando": "macadamizar",
    "macadamizara": "macadamizar",
    "macadamizarais": "macadamizar",
    "macadamizaran": "macadamizar",
    "macadamizaras": "macadamizar",
    "macadamizare": "macadamizar",
    "macadamizareis": "macadamizar",
    "macadamizaremos": "macadamizar",
    "macadamizaren": "macadamizar",
    "macadamizares": "macadamizar",
    "macadamizaron": "macadamizar",
    "macadamizará": "macadamizar",
    "macadamizarán": "macadamizar",
    "macadamizarás": "macadamizar",
    "macadamizaré": "macadamizar",
    "macadamizaréis": "macadamizar",
    "macadamizaría": "macadamizar",
    "macadamizaríais": "macadamizar",
    "macadamizaríamos": "macadamizar",
    "macadamizarían": "macadamizar",
    "macadamizarías": "macadamizar",
    "macadamizas": "macadamizar",
    "macadamizase": "macadamizar",
    "macadamizaseis": "macadamizar",
    "macadamizasen": "macadamizar",
    "macadamizases": "macadamizar",
    "macadamizaste": "macadamizar",
    "macadamizasteis": "macadamizar",
    "macadamizo": "macadamizar",
    "macadamizábamos": "macadamizar",
    "macadamizáis": "macadamizar",
    "macadamizáramos": "macadamizar",
    "macadamizáremos": "macadamizar",
    "macadamizásemos": "macadamizar",
    "macadamizó": "macadamizar",
    "macadanes": "macadán",
    "macadas": "macar",
    "macado": "macar",
    "macados": "macar",
    "macadánes": "macadán",
    "macaguas": "macagua",
    "macamos": "macar",
    "macan": "macar",
    "macanas": "macana",
    "macanazos": "macanazo",
    "macando": "macar",
    "macanea": "macanear",
    "macaneaba": "macanear",
    "macaneabais": "macanear",
    "macaneaban": "macanear",
    "macaneabas": "macanear",
    "macanead": "macanear",
    "macaneada": "macanear",
    "macaneadas": "macanear",
    "macaneado": "macanear",
    "macaneados": "macanear",
    "macaneamos": "macanear",
    "macanean": "macanear",
    "macaneando": "macanear",
    "macaneara": "macanear",
    "macanearais": "macanear",
    "macanearan": "macanear",
    "macanearas": "macanear",
    "macaneare": "macanear",
    "macaneareis": "macanear",
    "macanearemos": "macanear",
    "macanearen": "macanear",
    "macaneares": "macanear",
    "macanearon": "macanear",
    "macaneará": "macanear",
    "macanearán": "macanear",
    "macanearás": "macanear",
    "macanearé": "macanear",
    "macanearéis": "macanear",
    "macanearía": "macanear",
    "macanearíais": "macanear",
    "macanearíamos": "macanear",
    "macanearían": "macanear",
    "macanearías": "macanear",
    "macaneas": "macanear",
    "macanease": "macanear",
    "macaneaseis": "macanear",
    "macaneasen": "macanear",
    "macaneases": "macanear",
    "macaneaste": "macanear",
    "macaneasteis": "macanear",
    "macanee": "macanear",
    "macaneemos": "macanear",
    "macaneen": "macanear",
    "macanees": "macanear",
    "macaneo": "macanear",
    "macaneábamos": "macanear",
    "macaneáis": "macanear",
    "macaneáramos": "macanear",
    "macaneáremos": "macanear",
    "macaneásemos": "macanear",
    "macaneé": "macanear",
    "macaneéis": "macanear",
    "macaneó": "macanear",
    "macanuda": "macanudo",
    "macanudas": "macanudo",
    "macanudos": "macanudo",
    "macaones": "macaón",
    "macara": "macar",
    "macarais": "macar",
    "macaran": "macar",
    "macaras": "macar",
    "macare": "macar",
    "macareis": "macar",
    "macaremos": "macar",
    "macaren": "macar",
    "macareos": "macareo",
    "macares": "macar",
    "macaron": "macar",
    "macarraques": "macarraque",
    "macarras": "macarra",
    "macarroneas": "macarronea",
    "macarrones": "macarrón",
    "macarrónica": "macarrónico",
    "macarrónicas": "macarrónico",
    "macarrónicos": "macarrónico",
    "macartismos": "macartismo",
    "macartistas": "macartista",
    "macará": "macar",
    "macarán": "macar",
    "macarás": "macar",
    "macaré": "macar",
    "macaréis": "macar",
    "macaría": "macar",
    "macaríais": "macar",
    "macaríamos": "macar",
    "macarían": "macar",
    "macarías": "macar",
    "macas": "macar",
    "macasares": "macasar",
    "macase": "macar",
    "macaseis": "macar",
    "macasen": "macar",
    "macases": "macar",
    "macaste": "macar",
    "macasteis": "macar",
    "mace": "mazar",
    "macea": "macear",
    "maceaba": "macear",
    "maceabais": "macear",
    "maceaban": "macear",
    "maceabas": "macear",
    "macead": "macear",
    "maceada": "macear",
    "maceadas": "macear",
    "maceado": "macear",
    "maceados": "macear",
    "maceamos": "macear",
    "macean": "macear",
    "maceando": "macear",
    "maceara": "macear",
    "macearais": "macear",
    "macearan": "macear",
    "macearas": "macear",
    "maceare": "macear",
    "maceareis": "macear",
    "macearemos": "macear",
    "macearen": "macear",
    "maceares": "macear",
    "macearon": "macear",
    "maceará": "macear",
    "macearán": "macear",
    "macearás": "macear",
    "macearé": "macear",
    "macearéis": "macear",
    "macearía": "macear",
    "macearíais": "macear",
    "macearíamos": "macear",
    "macearían": "macear",
    "macearías": "macear",
    "maceas": "macear",
    "macease": "macear",
    "maceaseis": "macear",
    "maceasen": "macear",
    "maceases": "macear",
    "maceaste": "macear",
    "maceasteis": "macear",
    "macedona": "macedón",
    "macedonas": "macedón",
    "macedones": "macedón",
    "macedonia": "macedonio",
    "macedonias": "macedonio",
    "macedonios": "macedonio",
    "macedónica": "macedónico",
    "macedónicas": "macedónico",
    "macedónicos": "macedónico",
    "macee": "macear",
    "maceemos": "macear",
    "maceen": "macear",
    "macees": "macear",
    "macemos": "mazar",
    "macen": "mazar",
    "maceo": "macear",
    "macera": "macero",
    "maceraba": "macerar",
    "macerabais": "macerar",
    "maceraban": "macerar",
    "macerabas": "macerar",
    "maceraciones": "maceración",
    "macerad": "macerar",
    "macerada": "macerar",
    "maceradas": "macerar",
    "macerado": "macerar",
    "maceradores": "macerador",
    "macerados": "macerar",
    "maceramientos": "maceramiento",
    "maceramos": "macerar",
    "maceran": "macerar",
    "macerando": "macerar",
    "macerara": "macerar",
    "macerarais": "macerar",
    "maceraran": "macerar",
    "maceraras": "macerar",
    "macerare": "macerar",
    "macerareis": "macerar",
    "maceraremos": "macerar",
    "maceraren": "macerar",
    "macerares": "macerar",
    "maceraron": "macerar",
    "macerará": "macerar",
    "macerarán": "macerar",
    "macerarás": "macerar",
    "maceraré": "macerar",
    "maceraréis": "macerar",
    "maceraría": "macerar",
    "maceraríais": "macerar",
    "maceraríamos": "macerar",
    "macerarían": "macerar",
    "macerarías": "macerar",
    "maceras": "macero",
    "macerase": "macerar",
    "maceraseis": "macerar",
    "macerasen": "macerar",
    "macerases": "macerar",
    "maceraste": "macerar",
    "macerasteis": "macerar",
    "macere": "macerar",
    "maceremos": "macerar",
    "maceren": "macerar",
    "maceres": "macerar",
    "macerinas": "macerina",
    "macero": "macerar",
    "maceros": "macero",
    "macerábamos": "macerar",
    "maceráis": "macerar",
    "maceráramos": "macerar",
    "maceráremos": "macerar",
    "macerásemos": "macerar",
    "maceré": "macerar",
    "maceréis": "macerar",
    "maceró": "macerar",
    "maces": "mazar",
    "macetas": "maceta",
    "maceteros": "macetero",
    "maceábamos": "macear",
    "maceáis": "macear",
    "maceáramos": "macear",
    "maceáremos": "macear",
    "maceásemos": "macear",
    "maceé": "macear",
    "maceéis": "macear",
    "maceó": "macear",
    "macha": "machar",
    "machaba": "machar",
    "machabais": "machar",
    "machaban": "machar",
    "machabas": "machar",
    "machaca": "machacar",
    "machacaba": "machacar",
    "machacabais": "machacar",
    "machacaban": "machacar",
    "machacabas": "machacar",
    "machacad": "machacar",
    "machacada": "machacar",
    "machacadas": "machacar",
    "machacaderas": "machacadera",
    "machacado": "machacar",
    "machacadora": "machacador",
    "machacadoras": "machacadora",
    "machacadores": "machacador",
    "machacados": "machacar",
    "machacaduras": "machacadura",
    "machacamos": "machacar",
    "machacan": "machacar",
    "machacando": "machacar",
    "machacantes": "machacante",
    "machacara": "machacar",
    "machacarais": "machacar",
    "machacaran": "machacar",
    "machacaras": "machacar",
    "machacare": "machacar",
    "machacareis": "machacar",
    "machacaremos": "machacar",
    "machacaren": "machacar",
    "machacares": "machacar",
    "machacaron": "machacar",
    "machacará": "machacar",
    "machacarán": "machacar",
    "machacarás": "machacar",
    "machacaré": "machacar",
    "machacaréis": "machacar",
    "machacaría": "machacar",
    "machacaríais": "machacar",
    "machacaríamos": "machacar",
    "machacarían": "machacar",
    "machacarías": "machacar",
    "machacas": "machacar",
    "machacase": "machacar",
    "machacaseis": "machacar",
    "machacasen": "machacar",
    "machacases": "machacar",
    "machacaste": "machacar",
    "machacasteis": "machacar",
    "machaco": "machacar",
    "machacona": "machacón",
    "machaconas": "machacón",
    "machaconerías": "machaconería",
    "machacones": "machacón",
    "machacábamos": "machacar",
    "machacáis": "machacar",
    "machacáramos": "machacar",
    "machacáremos": "machacar",
    "machacásemos": "machacar",
    "machacó": "machacar",
    "machad": "machar",
    "machada": "machar",
    "machadas": "machar",
    "machado": "machar",
    "machados": "machar",
    "machamos": "machar",
    "machan": "machar",
    "machando": "machar",
    "machaque": "machacar",
    "machaquemos": "machacar",
    "machaquen": "machacar",
    "machaqueos": "machaqueo",
    "machaquerías": "machaquería",
    "machaques": "machacar",
    "machaqué": "machacar",
    "machaquéis": "machacar",
    "machara": "machar",
    "macharais": "machar",
    "macharan": "machar",
    "macharas": "machar",
    "machare": "machar",
    "machareis": "machar",
    "macharemos": "machar",
    "macharen": "machar",
    "machares": "machar",
    "macharon": "machar",
    "machará": "machar",
    "macharán": "machar",
    "macharás": "machar",
    "macharé": "machar",
    "macharéis": "machar",
    "macharía": "machar",
    "macharíais": "machar",
    "macharíamos": "machar",
    "macharían": "machar",
    "macharías": "machar",
    "machas": "machar",
    "machase": "machar",
    "machaseis": "machar",
    "machasen": "machar",
    "machases": "machar",
    "machaste": "machar",
    "machasteis": "machar",
    "mache": "machar",
    "machea": "machear",
    "macheaba": "machear",
    "macheabais": "machear",
    "macheaban": "machear",
    "macheabas": "machear",
    "machead": "machear",
    "macheada": "machear",
    "macheadas": "machear",
    "macheado": "machear",
    "macheados": "machear",
    "macheamos": "machear",
    "machean": "machear",
    "macheando": "machear",
    "macheara": "machear",
    "machearais": "machear",
    "machearan": "machear",
    "machearas": "machear",
    "macheare": "machear",
    "macheareis": "machear",
    "machearemos": "machear",
    "machearen": "machear",
    "macheares": "machear",
    "machearon": "machear",
    "macheará": "machear",
    "machearán": "machear",
    "machearás": "machear",
    "machearé": "machear",
    "machearéis": "machear",
    "machearía": "machear",
    "machearíais": "machear",
    "machearíamos": "machear",
    "machearían": "machear",
    "machearías": "machear",
    "macheas": "machear",
    "machease": "machear",
    "macheaseis": "machear",
    "macheasen": "machear",
    "macheases": "machear",
    "macheaste": "machear",
    "macheasteis": "machear",
    "machee": "machear",
    "macheemos": "machear",
    "macheen": "machear",
    "machees": "machear",
    "machemos": "machar",
    "machen": "machar",
    "macheo": "machear",
    "maches": "machar",
    "machetas": "macheta",
    "machetazos": "machetazo",
    "machetea": "machetear",
    "macheteaba": "machetear",
    "macheteabais": "machetear",
    "macheteaban": "machetear",
    "macheteabas": "machetear",
    "machetead": "machetear",
    "macheteada": "machetear",
    "macheteadas": "machetear",
    "macheteado": "machetear",
    "macheteados": "machetear",
    "macheteamos": "machetear",
    "machetean": "machetear",
    "macheteando": "machetear",
    "macheteara": "machetear",
    "machetearais": "machetear",
    "machetearan": "machetear",
    "machetearas": "machetear",
    "macheteare": "machetear",
    "macheteareis": "machetear",
    "machetearemos": "machetear",
    "machetearen": "machetear",
    "macheteares": "machetear",
    "machetearon": "machetear",
    "macheteará": "machetear",
    "machetearán": "machetear",
    "machetearás": "machetear",
    "machetearé": "machetear",
    "machetearéis": "machetear",
    "machetearía": "machetear",
    "machetearíais": "machetear",
    "machetearíamos": "machetear",
    "machetearían": "machetear",
    "machetearías": "machetear",
    "macheteas": "machetear",
    "machetease": "machetear",
    "macheteaseis": "machetear",
    "macheteasen": "machetear",
    "macheteases": "machetear",
    "macheteaste": "machetear",
    "macheteasteis": "machetear",
    "machetee": "machetear",
    "macheteemos": "machetear",
    "macheteen": "machetear",
    "machetees": "machetear",
    "macheteo": "machetear",
    "macheteos": "macheteo",
    "machetera": "machetero",
    "macheteras": "machetero",
    "macheteros": "machetero",
    "machetes": "machete",
    "macheteábamos": "machetear",
    "macheteáis": "machetear",
    "macheteáramos": "machetear",
    "macheteáremos": "machetear",
    "macheteásemos": "machetear",
    "macheteé": "machetear",
    "macheteéis": "machetear",
    "macheteó": "machetear",
    "macheábamos": "machear",
    "macheáis": "machear",
    "macheáramos": "machear",
    "macheáremos": "machear",
    "macheásemos": "machear",
    "macheé": "machear",
    "macheéis": "machear",
    "macheó": "machear",
    "machihembra": "machihembrar",
    "machihembraba": "machihembrar",
    "machihembrabais": "machihembrar",
    "machihembraban": "machihembrar",
    "machihembrabas": "machihembrar",
    "machihembrad": "machihembrar",
    "machihembrada": "machihembrar",
    "machihembradas": "machihembrar",
    "machihembrado": "machihembrar",
    "machihembradora": "machihembrador",
    "machihembradoras": "machihembradora",
    "machihembradores": "machihembrador",
    "machihembrados": "machihembrar",
    "machihembramos": "machihembrar",
    "machihembran": "machihembrar",
    "machihembrando": "machihembrar",
    "machihembrara": "machihembrar",
    "machihembrarais": "machihembrar",
    "machihembraran": "machihembrar",
    "machihembraras": "machihembrar",
    "machihembrare": "machihembrar",
    "machihembrareis": "machihembrar",
    "machihembraremos": "machihembrar",
    "machihembraren": "machihembrar",
    "machihembrares": "machihembrar",
    "machihembraron": "machihembrar",
    "machihembrará": "machihembrar",
    "machihembrarán": "machihembrar",
    "machihembrarás": "machihembrar",
    "machihembraré": "machihembrar",
    "machihembraréis": "machihembrar",
    "machihembraría": "machihembrar",
    "machihembraríais": "machihembrar",
    "machihembraríamos": "machihembrar",
    "machihembrarían": "machihembrar",
    "machihembrarías": "machihembrar",
    "machihembras": "machihembrar",
    "machihembrase": "machihembrar",
    "machihembraseis": "machihembrar",
    "machihembrasen": "machihembrar",
    "machihembrases": "machihembrar",
    "machihembraste": "machihembrar",
    "machihembrasteis": "machihembrar",
    "machihembre": "machihembrar",
    "machihembremos": "machihembrar",
    "machihembren": "machihembrar",
    "machihembres": "machihembrar",
    "machihembro": "machihembrar",
    "machihembrábamos": "machihembrar",
    "machihembráis": "machihembrar",
    "machihembráramos": "machihembrar",
    "machihembráremos": "machihembrar",
    "machihembrásemos": "machihembrar",
    "machihembré": "machihembrar",
    "machihembréis": "machihembrar",
    "machihembró": "machihembrar",
    "machinadas": "machinada",
    "machinas": "machina",
    "machismos": "machismo",
    "machistas": "machista",
    "macho": "machar",
    "machones": "machón",
    "machorra": "machorro",
    "machorras": "machorro",
    "machorros": "machorro",
    "machos": "macho",
    "machotas": "machota",
    "machotes": "machote",
    "machuca": "machucar",
    "machucaba": "machucar",
    "machucabais": "machucar",
    "machucaban": "machucar",
    "machucabas": "machucar",
    "machucad": "machucar",
    "machucada": "machucar",
    "machucadas": "machucar",
    "machucado": "machucar",
    "machucadora": "machucador",
    "machucadoras": "machucador",
    "machucadores": "machucador",
    "machucados": "machucar",
    "machucaduras": "machucadura",
    "machucamientos": "machucamiento",
    "machucamos": "machucar",
    "machucan": "machucar",
    "machucando": "machucar",
    "machucara": "machucar",
    "machucarais": "machucar",
    "machucaran": "machucar",
    "machucaras": "machucar",
    "machucare": "machucar",
    "machucareis": "machucar",
    "machucaremos": "machucar",
    "machucaren": "machucar",
    "machucares": "machucar",
    "machucaron": "machucar",
    "machucará": "machucar",
    "machucarán": "machucar",
    "machucarás": "machucar",
    "machucaré": "machucar",
    "machucaréis": "machucar",
    "machucaría": "machucar",
    "machucaríais": "machucar",
    "machucaríamos": "machucar",
    "machucarían": "machucar",
    "machucarías": "machucar",
    "machucas": "machucar",
    "machucase": "machucar",
    "machucaseis": "machucar",
    "machucasen": "machucar",
    "machucases": "machucar",
    "machucaste": "machucar",
    "machucasteis": "machucar",
    "machucha": "machucho",
    "machuchas": "machucho",
    "machuchos": "machucho",
    "machuco": "machucar",
    "machucones": "machucón",
    "machucábamos": "machucar",
    "machucáis": "machucar",
    "machucáramos": "machucar",
    "machucáremos": "machucar",
    "machucásemos": "machucar",
    "machucó": "machucar",
    "machuelos": "machuelo",
    "machuque": "machucar",
    "machuquemos": "machucar",
    "machuquen": "machucar",
    "machuques": "machucar",
    "machuqué": "machucar",
    "machuquéis": "machucar",
    "machábamos": "machar",
    "macháis": "machar",
    "macháramos": "machar",
    "macháremos": "machar",
    "machásemos": "machar",
    "maché": "machar",
    "machéis": "machar",
    "machó": "machar",
    "macice": "macizar",
    "macicemos": "macizar",
    "macicen": "macizar",
    "macices": "macizar",
    "macicé": "macizar",
    "macicéis": "macizar",
    "macilenta": "macilento",
    "macilentas": "macilento",
    "macilentos": "macilento",
    "macillos": "macillo",
    "maciza": "macizo",
    "macizaba": "macizar",
    "macizabais": "macizar",
    "macizaban": "macizar",
    "macizabas": "macizar",
    "macizad": "macizar",
    "macizada": "macizar",
    "macizadas": "macizar",
    "macizado": "macizar",
    "macizados": "macizar",
    "macizamos": "macizar",
    "macizan": "macizar",
    "macizando": "macizar",
    "macizara": "macizar",
    "macizarais": "macizar",
    "macizaran": "macizar",
    "macizaras": "macizar",
    "macizare": "macizar",
    "macizareis": "macizar",
    "macizaremos": "macizar",
    "macizaren": "macizar",
    "macizares": "macizar",
    "macizaron": "macizar",
    "macizará": "macizar",
    "macizarán": "macizar",
    "macizarás": "macizar",
    "macizaré": "macizar",
    "macizaréis": "macizar",
    "macizaría": "macizar",
    "macizaríais": "macizar",
    "macizaríamos": "macizar",
    "macizarían": "macizar",
    "macizarías": "macizar",
    "macizas": "macizo",
    "macizase": "macizar",
    "macizaseis": "macizar",
    "macizasen": "macizar",
    "macizases": "macizar",
    "macizaste": "macizar",
    "macizasteis": "macizar",
    "macizo": "macizar",
    "macizos": "macizo",
    "macizábamos": "macizar",
    "macizáis": "macizar",
    "macizáramos": "macizar",
    "macizáremos": "macizar",
    "macizásemos": "macizar",
    "macizó": "macizar",
    "maclados": "maclado",
    "maclas": "macla",
    "maco": "macar",
    "macolla": "macollar",
    "macollaba": "macollar",
    "macollabais": "macollar",
    "macollaban": "macollar",
    "macollabas": "macollar",
    "macollad": "macollar",
    "macollada": "macollar",
    "macolladas": "macollar",
    "macollado": "macollar",
    "macollados": "macollar",
    "macollamos": "macollar",
    "macollan": "macollar",
    "macollando": "macollar",
    "macollara": "macollar",
    "macollarais": "macollar",
    "macollaran": "macollar",
    "macollaras": "macollar",
    "macollare": "macollar",
    "macollareis": "macollar",
    "macollaremos": "macollar",
    "macollaren": "macollar",
    "macollares": "macollar",
    "macollaron": "macollar",
    "macollará": "macollar",
    "macollarán": "macollar",
    "macollarás": "macollar",
    "macollaré": "macollar",
    "macollaréis": "macollar",
    "macollaría": "macollar",
    "macollaríais": "macollar",
    "macollaríamos": "macollar",
    "macollarían": "macollar",
    "macollarías": "macollar",
    "macollas": "macollar",
    "macollase": "macollar",
    "macollaseis": "macollar",
    "macollasen": "macollar",
    "macollases": "macollar",
    "macollaste": "macollar",
    "macollasteis": "macollar",
    "macolle": "macollar",
    "macollemos": "macollar",
    "macollen": "macollar",
    "macolles": "macollar",
    "macollo": "macollar",
    "macollábamos": "macollar",
    "macolláis": "macollar",
    "macolláramos": "macollar",
    "macolláremos": "macollar",
    "macollásemos": "macollar",
    "macollé": "macollar",
    "macolléis": "macollar",
    "macolló": "macollar",
    "maconas": "macona",
    "macones": "macón",
    "macramés": "macramé",
    "macroanalítica": "macroanalítico",
    "macroanalíticas": "macroanalítico",
    "macroanalíticos": "macroanalítico",
    "macroangiopática": "macroangiopático",
    "macroangiopáticas": "macroangiopático",
    "macroangiopáticos": "macroangiopático",
    "macrobiocarpias": "macrobiocarpia",
    "macrobiótica": "macrobiótico",
    "macrobióticas": "macrobiótico",
    "macrobióticos": "macrobiótico",
    "macrocarpos": "macrocarpo",
    "macrocefalias": "macrocefalia",
    "macrocentros": "macrocentro",
    "macrocitos": "macrocito",
    "macroclimas": "macroclima",
    "macrocomportamientos": "macrocomportamiento",
    "macroconexiones": "macroconexión",
    "macrocranias": "macrocrania",
    "macrocristalina": "macrocristalino",
    "macrocristalinas": "macrocristalino",
    "macrocristalinos": "macrocristalino",
    "macrocéfala": "macrocéfalo",
    "macrocéfalas": "macrocéfalo",
    "macrocéfalos": "macrocéfalo",
    "macrocíclica": "macrocíclico",
    "macrocíclicas": "macrocíclico",
    "macrocíclicos": "macrocíclico",
    "macrocítica": "macrocítico",
    "macrocíticas": "macrocítico",
    "macrocíticos": "macrocítico",
    "macrocódigos": "macrocódigo",
    "macrodoncias": "macrodoncia",
    "macrodonta": "macrodonto",
    "macrodontas": "macrodonto",
    "macrodontismos": "macrodontismo",
    "macrodontos": "macrodonto",
    "macroeconomías": "macroeconomía",
    "macroeconómica": "macroeconómico",
    "macroeconómicas": "macroeconómico",
    "macroeconómicos": "macroeconómico",
    "macroelementos": "macroelemento",
    "macroenconómica": "macroenconómico",
    "macroenconómicas": "macroenconómico",
    "macroenconómicos": "macroenconómico",
    "macroensambladora": "macroensamblador",
    "macroensambladoras": "macroensamblador",
    "macroensambladores": "macroensamblador",
    "macroestados": "macroestado",
    "macroestructuras": "macroestructura",
    "macrofagias": "macrofagia",
    "macrofotografías": "macrofotografía",
    "macrogametos": "macrogameto",
    "macrogeneradores": "macrogenerador",
    "macrogenias": "macrogenia",
    "macroglobulinemias": "macroglobulinemia",
    "macroglosias": "macroglosia",
    "macroglubinemias": "macroglubinemia",
    "macroglósica": "macroglósico",
    "macroglósicas": "macroglósico",
    "macroglósicos": "macroglósico",
    "macrognacias": "macrognacia",
    "macrognática": "macrognático",
    "macrognáticas": "macrognático",
    "macrognáticos": "macrognático",
    "macrografías": "macrografía",
    "macrográfica": "macrográfico",
    "macrográficas": "macrográfico",
    "macrográficos": "macrográfico",
    "macroinstrucciones": "macroinstrucción",
    "macrojuicios": "macrojuicio",
    "macromagnitudes": "macromagnitud",
    "macromarketings": "macromarketing",
    "macromelias": "macromelia",
    "macrometeorologías": "macrometeorología",
    "macromicetes": "macromicete",
    "macromoleculares": "macromolecular",
    "macromoléculas": "macromolécula",
    "macromundos": "macromundo",
    "macromárketings": "macromárketing",
    "macronucleares": "macronuclear",
    "macronúcleos": "macronúcleo",
    "macroordenadores": "macroordenador",
    "macroprocesadores": "macroprocesador",
    "macroprogramaciones": "macroprogramación",
    "macroprosopias": "macroprosopia",
    "macroproyectos": "macroproyecto",
    "macropódidos": "macropódido",
    "macroqueilias": "macroqueilia",
    "macroquilias": "macroquilia",
    "macroquímicas": "macroquímica",
    "macros": "macro",
    "macroscopías": "macroscopía",
    "macroscópica": "macroscópico",
    "macroscópicas": "macroscópico",
    "macroscópicos": "macroscópico",
    "macrosistemas": "macrosistema",
    "macrosociologías": "macrosociología",
    "macrosomas": "macrosoma",
    "macrosomías": "macrosomía",
    "macrosporas": "macrospora",
    "macrostomías": "macrostomía",
    "macrosísmica": "macrosísmico",
    "macrosísmicas": "macrosísmico",
    "macrosísmicos": "macrosísmico",
    "macrotias": "macrotia",
    "macroáreas": "macroárea",
    "macrura": "macruro",
    "macruras": "macruro",
    "macruros": "macruro",
    "macrófaga": "macrófago",
    "macrófagas": "macrófago",
    "macrófagos": "macrófago",
    "macrópoda": "macrópodo",
    "macrópodas": "macrópodo",
    "macrópodos": "macrópodo",
    "macrósporas": "macróspora",
    "macsuras": "macsura",
    "macuaches": "macuache",
    "macubas": "macuba",
    "macucas": "macuca",
    "macula": "macular",
    "maculaba": "macular",
    "maculabais": "macular",
    "maculaban": "macular",
    "maculabas": "macular",
    "maculad": "macular",
    "maculada": "macular",
    "maculadas": "macular",
    "maculado": "macular",
    "maculados": "macular",
    "maculaduras": "maculadura",
    "maculamos": "macular",
    "maculan": "macular",
    "maculando": "macular",
    "maculara": "macular",
    "macularais": "macular",
    "macularan": "macular",
    "macularas": "macular",
    "maculare": "macular",
    "maculareis": "macular",
    "macularemos": "macular",
    "macularen": "macular",
    "maculares": "macular",
    "macularon": "macular",
    "maculará": "macular",
    "macularán": "macular",
    "macularás": "macular",
    "macularé": "macular",
    "macularéis": "macular",
    "macularía": "macular",
    "macularíais": "macular",
    "macularíamos": "macular",
    "macularían": "macular",
    "macularías": "macular",
    "maculas": "macular",
    "maculase": "macular",
    "maculaseis": "macular",
    "maculasen": "macular",
    "maculases": "macular",
    "maculaste": "macular",
    "maculasteis": "macular",
    "maculaturas": "maculatura",
    "macule": "macular",
    "maculemos": "macular",
    "maculen": "macular",
    "macules": "macular",
    "maculo": "macular",
    "maculábamos": "macular",
    "maculáis": "macular",
    "maculáramos": "macular",
    "maculáremos": "macular",
    "maculásemos": "macular",
    "maculé": "macular",
    "maculéis": "macular",
    "maculó": "macular",
    "macuquera": "macuquero",
    "macuqueras": "macuquero",
    "macuqueros": "macuquero",
    "macutos": "macuto",
    "macábamos": "macar",
    "macáis": "macar",
    "macáramos": "macar",
    "macáremos": "macar",
    "macásemos": "macar",
    "macé": "mazar",
    "macéis": "mazar",
    "macó": "macar",
    "madagascaríses": "madagascarí",
    "madamas": "madama",
    "madames": "madame",
    "madammes": "madamme",
    "madams": "madam",
    "madapolanes": "madapolán",
    "madeirenses": "madeirense",
    "madejas": "madeja",
    "madera": "maderar",
    "maderaba": "maderar",
    "maderabais": "maderar",
    "maderaban": "maderar",
    "maderabas": "maderar",
    "maderables": "maderable",
    "maderad": "maderar",
    "maderada": "maderar",
    "maderadas": "maderar",
    "maderado": "maderar",
    "maderados": "maderar",
    "maderajes": "maderaje",
    "maderamos": "maderar",
    "maderan": "maderar",
    "maderando": "maderar",
    "maderara": "maderar",
    "maderarais": "maderar",
    "maderaran": "maderar",
    "maderaras": "maderar",
    "maderare": "maderar",
    "maderareis": "maderar",
    "maderaremos": "maderar",
    "maderaren": "maderar",
    "maderares": "maderar",
    "maderaron": "maderar",
    "maderará": "maderar",
    "maderarán": "maderar",
    "maderarás": "maderar",
    "maderaré": "maderar",
    "maderaréis": "maderar",
    "maderaría": "maderar",
    "maderaríais": "maderar",
    "maderaríamos": "maderar",
    "maderarían": "maderar",
    "maderarías": "maderar",
    "maderas": "maderar",
    "maderase": "maderar",
    "maderaseis": "maderar",
    "maderasen": "maderar",
    "maderases": "maderar",
    "maderaste": "maderar",
    "maderasteis": "maderar",
    "madere": "maderar",
    "maderemos": "maderar",
    "maderen": "maderar",
    "maderenses": "maderense",
    "maderera": "maderero",
    "madereras": "maderero",
    "madereros": "maderero",
    "madererías": "maderería",
    "maderes": "maderar",
    "madero": "maderar",
    "maderos": "madero",
    "maderábamos": "maderar",
    "maderáis": "maderar",
    "maderámenes": "maderamen",
    "maderáramos": "maderar",
    "maderáremos": "maderar",
    "maderásemos": "maderar",
    "maderé": "maderar",
    "maderéis": "maderar",
    "maderó": "maderar",
    "madescentes": "madescente",
    "madianitas": "madianita",
    "madonas": "madona",
    "madores": "mador",
    "madorosa": "madoroso",
    "madorosas": "madoroso",
    "madorosos": "madoroso",
    "madrastras": "madrastra",
    "madrazas": "madraza",
    "madrecillas": "madrecilla",
    "madreclavos": "madreclavo",
    "madreperlas": "madreperla",
    "madreporaria": "madreporario",
    "madreporarias": "madreporario",
    "madreporarios": "madreporario",
    "madrepórica": "madrepórico",
    "madrepóricas": "madrepórico",
    "madrepóricos": "madrepórico",
    "madrera": "madrero",
    "madreras": "madrero",
    "madreros": "madrero",
    "madres": "madre",
    "madreselvas": "madreselva",
    "madridismos": "madridismo",
    "madridistas": "madridista",
    "madrigada": "madrigado",
    "madrigadas": "madrigado",
    "madrigados": "madrigado",
    "madrigales": "madrigal",
    "madrigalesca": "madrigalesco",
    "madrigalescas": "madrigalesco",
    "madrigalescos": "madrigalesco",
    "madrigalistas": "madrigalista",
    "madrigueras": "madriguera",
    "madrileña": "madrileño",
    "madrileñas": "madrileño",
    "madrileños": "madrileño",
    "madrinas": "madrina",
    "madrinazgos": "madrinazgo",
    "madroñales": "madroñal",
    "madroñeras": "madroñera",
    "madroños": "madroño",
    "madruga": "madrugar",
    "madrugaba": "madrugar",
    "madrugabais": "madrugar",
    "madrugaban": "madrugar",
    "madrugabas": "madrugar",
    "madrugad": "madrugar",
    "madrugada": "madrugar",
    "madrugadas": "madrugar",
    "madrugado": "madrugar",
    "madrugadora": "madrugador",
    "madrugadoras": "madrugador",
    "madrugadores": "madrugador",
    "madrugados": "madrugar",
    "madrugamos": "madrugar",
    "madrugan": "madrugar",
    "madrugando": "madrugar",
    "madrugara": "madrugar",
    "madrugarais": "madrugar",
    "madrugaran": "madrugar",
    "madrugaras": "madrugar",
    "madrugare": "madrugar",
    "madrugareis": "madrugar",
    "madrugaremos": "madrugar",
    "madrugaren": "madrugar",
    "madrugares": "madrugar",
    "madrugaron": "madrugar",
    "madrugará": "madrugar",
    "madrugarán": "madrugar",
    "madrugarás": "madrugar",
    "madrugaré": "madrugar",
    "madrugaréis": "madrugar",
    "madrugaría": "madrugar",
    "madrugaríais": "madrugar",
    "madrugaríamos": "madrugar",
    "madrugarían": "madrugar",
    "madrugarías": "madrugar",
    "madrugas": "madrugar",
    "madrugase": "madrugar",
    "madrugaseis": "madrugar",
    "madrugasen": "madrugar",
    "madrugases": "madrugar",
    "madrugaste": "madrugar",
    "madrugasteis": "madrugar",
    "madrugo": "madrugar",
    "madrugona": "madrugón",
    "madrugonas": "madrugón",
    "madrugones": "madrugón",
    "madrugue": "madrugar",
    "madruguemos": "madrugar",
    "madruguen": "madrugar",
    "madrugues": "madrugar",
    "madrugué": "madrugar",
    "madruguéis": "madrugar",
    "madrugábamos": "madrugar",
    "madrugáis": "madrugar",
    "madrugáramos": "madrugar",
    "madrugáremos": "madrugar",
    "madrugásemos": "madrugar",
    "madrugó": "madrugar",
    "madréporas": "madrépora",
    "madura": "maduro",
    "maduraba": "madurar",
    "madurabais": "madurar",
    "maduraban": "madurar",
    "madurabas": "madurar",
    "maduraciones": "maduración",
    "maduraciónes": "maduración",
    "madurad": "madurar",
    "madurada": "madurar",
    "maduradas": "madurar",
    "maduraderos": "maduradero",
    "madurado": "madurar",
    "maduradora": "madurador",
    "maduradoras": "madurador",
    "maduradores": "madurador",
    "madurados": "madurar",
    "maduramos": "madurar",
    "maduran": "madurar",
    "madurando": "madurar",
    "madurara": "madurar",
    "madurarais": "madurar",
    "maduraran": "madurar",
    "maduraras": "madurar",
    "madurare": "madurar",
    "madurareis": "madurar",
    "maduraremos": "madurar",
    "maduraren": "madurar",
    "madurares": "madurar",
    "maduraron": "madurar",
    "madurará": "madurar",
    "madurarán": "madurar",
    "madurarás": "madurar",
    "maduraré": "madurar",
    "maduraréis": "madurar",
    "maduraría": "madurar",
    "maduraríais": "madurar",
    "maduraríamos": "madurar",
    "madurarían": "madurar",
    "madurarías": "madurar",
    "maduras": "maduro",
    "madurase": "madurar",
    "maduraseis": "madurar",
    "madurasen": "madurar",
    "madurases": "madurar",
    "maduraste": "madurar",
    "madurasteis": "madurar",
    "madurativa": "madurativo",
    "madurativas": "madurativo",
    "madurativos": "madurativo",
    "madure": "madurar",
    "madureces": "madurez",
    "maduremos": "madurar",
    "maduren": "madurar",
    "madures": "madurar",
    "maduro": "madurar",
    "maduros": "maduro",
    "madurábamos": "madurar",
    "maduráis": "madurar",
    "maduráramos": "madurar",
    "maduráremos": "madurar",
    "madurásemos": "madurar",
    "maduré": "madurar",
    "maduréis": "madurar",
    "maduró": "madurar",
    "maelstromes": "maelstrom",
    "maesillas": "maesilla",
    "maestra": "maestro",
    "maestrales": "maestral",
    "maestrantes": "maestrante",
    "maestranzas": "maestranza",
    "maestras": "maestro",
    "maestrazgos": "maestrazgo",
    "maestrea": "maestrear",
    "maestreaba": "maestrear",
    "maestreabais": "maestrear",
    "maestreaban": "maestrear",
    "maestreabas": "maestrear",
    "maestread": "maestrear",
    "maestreada": "maestrear",
    "maestreadas": "maestrear",
    "maestreado": "maestrear",
    "maestreados": "maestrear",
    "maestreamos": "maestrear",
    "maestrean": "maestrear",
    "maestreando": "maestrear",
    "maestreara": "maestrear",
    "maestrearais": "maestrear",
    "maestrearan": "maestrear",
    "maestrearas": "maestrear",
    "maestreare": "maestrear",
    "maestreareis": "maestrear",
    "maestrearemos": "maestrear",
    "maestrearen": "maestrear",
    "maestreares": "maestrear",
    "maestrearon": "maestrear",
    "maestreará": "maestrear",
    "maestrearán": "maestrear",
    "maestrearás": "maestrear",
    "maestrearé": "maestrear",
    "maestrearéis": "maestrear",
    "maestrearía": "maestrear",
    "maestrearíais": "maestrear",
    "maestrearíamos": "maestrear",
    "maestrearían": "maestrear",
    "maestrearías": "maestrear",
    "maestreas": "maestrear",
    "maestrease": "maestrear",
    "maestreaseis": "maestrear",
    "maestreasen": "maestrear",
    "maestreases": "maestrear",
    "maestreaste": "maestrear",
    "maestreasteis": "maestrear",
    "maestree": "maestrear",
    "maestreemos": "maestrear",
    "maestreen": "maestrear",
    "maestrees": "maestrear",
    "maestreo": "maestrear",
    "maestres": "maestre",
    "maestresalas": "maestresala",
    "maestrescolías": "maestrescolía",
    "maestrescuelas": "maestrescuela",
    "maestreábamos": "maestrear",
    "maestreáis": "maestrear",
    "maestreáramos": "maestrear",
    "maestreáremos": "maestrear",
    "maestreásemos": "maestrear",
    "maestreé": "maestrear",
    "maestreéis": "maestrear",
    "maestreó": "maestrear",
    "maestriles": "maestril",
    "maestrillos": "maestrillo",
    "maestros": "maestro",
    "maestrías": "maestría",
    "mafias": "mafia",
    "mafiosa": "mafioso",
    "mafiosas": "mafioso",
    "mafiosos": "mafioso",
    "maga": "mago",
    "magacines": "magacín",
    "magallánica": "magallánico",
    "magallánicas": "magallánico",
    "magallánicos": "magallánico",
    "magancea": "magancear",
    "maganceaba": "magancear",
    "maganceabais": "magancear",
    "maganceaban": "magancear",
    "maganceabas": "magancear",
    "magancead": "magancear",
    "maganceada": "magancear",
    "maganceadas": "magancear",
    "maganceado": "magancear",
    "maganceados": "magancear",
    "maganceamos": "magancear",
    "magancean": "magancear",
    "maganceando": "magancear",
    "maganceara": "magancear",
    "magancearais": "magancear",
    "magancearan": "magancear",
    "magancearas": "magancear",
    "maganceare": "magancear",
    "maganceareis": "magancear",
    "magancearemos": "magancear",
    "magancearen": "magancear",
    "maganceares": "magancear",
    "magancearon": "magancear",
    "maganceará": "magancear",
    "magancearán": "magancear",
    "magancearás": "magancear",
    "magancearé": "magancear",
    "magancearéis": "magancear",
    "magancearía": "magancear",
    "magancearíais": "magancear",
    "magancearíamos": "magancear",
    "magancearían": "magancear",
    "magancearías": "magancear",
    "maganceas": "magancear",
    "magancease": "magancear",
    "maganceaseis": "magancear",
    "maganceasen": "magancear",
    "maganceases": "magancear",
    "maganceaste": "magancear",
    "maganceasteis": "magancear",
    "magancee": "magancear",
    "maganceemos": "magancear",
    "maganceen": "magancear",
    "magancees": "magancear",
    "maganceo": "magancear",
    "maganceábamos": "magancear",
    "maganceáis": "magancear",
    "maganceáramos": "magancear",
    "maganceáremos": "magancear",
    "maganceásemos": "magancear",
    "maganceé": "magancear",
    "maganceéis": "magancear",
    "maganceó": "magancear",
    "maganeles": "maganel",
    "magantos": "maganto",
    "magas": "mago",
    "magazines": "magazine",
    "magañas": "magaña",
    "magdalenas": "magdalena",
    "magdalenienses": "magdaleniense",
    "magdaleones": "magdaleón",
    "magentas": "magenta",
    "magiares": "magiar",
    "magias": "magia",
    "magines": "magín",
    "magisteriales": "magisterial",
    "magisterios": "magisterio",
    "magistrada": "magistrado",
    "magistradas": "magistrado",
    "magistrados": "magistrado",
    "magistrales": "magistral",
    "magistraturas": "magistratura",
    "magmas": "magma",
    "magmatismos": "magmatismo",
    "magmática": "magmático",
    "magmáticas": "magmático",
    "magmáticos": "magmático",
    "magna": "magno",
    "magnalios": "magnalio",
    "magnanimidades": "magnanimidad",
    "magnas": "magno",
    "magnates": "magnate",
    "magnesias": "magnesia",
    "magnesios": "magnesio",
    "magnesitas": "magnesita",
    "magnetas": "magneta",
    "magnetice": "magnetizar",
    "magneticemos": "magnetizar",
    "magneticen": "magnetizar",
    "magnetices": "magnetizar",
    "magneticé": "magnetizar",
    "magneticéis": "magnetizar",
    "magnetismos": "magnetismo",
    "magnetitas": "magnetita",
    "magnetiza": "magnetizar",
    "magnetizaba": "magnetizar",
    "magnetizabais": "magnetizar",
    "magnetizaban": "magnetizar",
    "magnetizabas": "magnetizar",
    "magnetizabilidades": "magnetizabilidad",
    "magnetizables": "magnetizable",
    "magnetizaciones": "magnetización",
    "magnetizad": "magnetizar",
    "magnetizada": "magnetizar",
    "magnetizadas": "magnetizar",
    "magnetizado": "magnetizar",
    "magnetizadora": "magnetizador",
    "magnetizadoras": "magnetizador",
    "magnetizadores": "magnetizador",
    "magnetizados": "magnetizar",
    "magnetizamos": "magnetizar",
    "magnetizan": "magnetizar",
    "magnetizando": "magnetizar",
    "magnetizantes": "magnetizante",
    "magnetizara": "magnetizar",
    "magnetizarais": "magnetizar",
    "magnetizaran": "magnetizar",
    "magnetizaras": "magnetizar",
    "magnetizare": "magnetizar",
    "magnetizareis": "magnetizar",
    "magnetizaremos": "magnetizar",
    "magnetizaren": "magnetizar",
    "magnetizares": "magnetizar",
    "magnetizaron": "magnetizar",
    "magnetizará": "magnetizar",
    "magnetizarán": "magnetizar",
    "magnetizarás": "magnetizar",
    "magnetizaré": "magnetizar",
    "magnetizaréis": "magnetizar",
    "magnetizaría": "magnetizar",
    "magnetizaríais": "magnetizar",
    "magnetizaríamos": "magnetizar",
    "magnetizarían": "magnetizar",
    "magnetizarías": "magnetizar",
    "magnetizas": "magnetizar",
    "magnetizase": "magnetizar",
    "magnetizaseis": "magnetizar",
    "magnetizasen": "magnetizar",
    "magnetizases": "magnetizar",
    "magnetizaste": "magnetizar",
    "magnetizasteis": "magnetizar",
    "magnetizo": "magnetizar",
    "magnetizábamos": "magnetizar",
    "magnetizáis": "magnetizar",
    "magnetizáramos": "magnetizar",
    "magnetizáremos": "magnetizar",
    "magnetizásemos": "magnetizar",
    "magnetizó": "magnetizar",
    "magnetodinamómetros": "magnetodinamómetro",
    "magnetodinámica": "magnetodinámico",
    "magnetodinámicas": "magnetodinámico",
    "magnetodinámicos": "magnetodinámico",
    "magnetoeléctrica": "magnetoeléctrico",
    "magnetoeléctricas": "magnetoeléctrico",
    "magnetoeléctricos": "magnetoeléctrico",
    "magnetoestricciones": "magnetoestricción",
    "magnetoestrictiva": "magnetoestrictivo",
    "magnetoestrictivas": "magnetoestrictivo",
    "magnetoestrictivos": "magnetoestrictivo",
    "magnetofones": "magnetofón",
    "magnetofónica": "magnetofónico",
    "magnetofónicas": "magnetofónico",
    "magnetofónicos": "magnetofónico",
    "magnetográfica": "magnetográfico",
    "magnetográficas": "magnetográfico",
    "magnetográficos": "magnetográfico",
    "magnetohidrodinámica": "magnetohidrodinámico",
    "magnetohidrodinámicas": "magnetohidrodinámico",
    "magnetohidrodinámicos": "magnetohidrodinámico",
    "magnetoinductiva": "magnetoinductivo",
    "magnetoinductivas": "magnetoinductivo",
    "magnetoinductivos": "magnetoinductivo",
    "magnetomotrices": "magnetomotriz",
    "magnetones": "magnetón",
    "magnetopausas": "magnetopausa",
    "magnetoresistencias": "magnetoresistencia",
    "magnetorresistencias": "magnetorresistencia",
    "magnetos": "magneto",
    "magnetoscopias": "magnetoscopia",
    "magnetoscopios": "magnetoscopio",
    "magnetosferas": "magnetosfera",
    "magnetostricciones": "magnetostricción",
    "magnetostrictiva": "magnetostrictivo",
    "magnetostrictivas": "magnetostrictivo",
    "magnetostrictivos": "magnetostrictivo",
    "magnetostáticas": "magnetostática",
    "magnetoóptica": "magnetoóptico",
    "magnetoópticas": "magnetoóptico",
    "magnetoópticos": "magnetoóptico",
    "magnetrones": "magnetrón",
    "magnetímetros": "magnetímetro",
    "magnetófonos": "magnetófono",
    "magnetómetros": "magnetómetro",
    "magnicidas": "magnicida",
    "magnicidios": "magnicidio",
    "magnifica": "magnificar",
    "magnificaba": "magnificar",
    "magnificabais": "magnificar",
    "magnificaban": "magnificar",
    "magnificabas": "magnificar",
    "magnificad": "magnificar",
    "magnificada": "magnificar",
    "magnificadas": "magnificar",
    "magnificado": "magnificar",
    "magnificados": "magnificar",
    "magnificamos": "magnificar",
    "magnifican": "magnificar",
    "magnificando": "magnificar",
    "magnificara": "magnificar",
    "magnificarais": "magnificar",
    "magnificaran": "magnificar",
    "magnificaras": "magnificar",
    "magnificare": "magnificar",
    "magnificareis": "magnificar",
    "magnificaremos": "magnificar",
    "magnificaren": "magnificar",
    "magnificares": "magnificar",
    "magnificaron": "magnificar",
    "magnificará": "magnificar",
    "magnificarán": "magnificar",
    "magnificarás": "magnificar",
    "magnificaré": "magnificar",
    "magnificaréis": "magnificar",
    "magnificaría": "magnificar",
    "magnificaríais": "magnificar",
    "magnificaríamos": "magnificar",
    "magnificarían": "magnificar",
    "magnificarías": "magnificar",
    "magnificas": "magnificar",
    "magnificase": "magnificar",
    "magnificaseis": "magnificar",
    "magnificasen": "magnificar",
    "magnificases": "magnificar",
    "magnificaste": "magnificar",
    "magnificasteis": "magnificar",
    "magnificencias": "magnificencia",
    "magnificiencias": "magnificiencia",
    "magnifico": "magnificar",
    "magnificábamos": "magnificar",
    "magnificáis": "magnificar",
    "magnificáramos": "magnificar",
    "magnificáremos": "magnificar",
    "magnificásemos": "magnificar",
    "magnificó": "magnificar",
    "magnifique": "magnificar",
    "magnifiquemos": "magnificar",
    "magnifiquen": "magnificar",
    "magnifiques": "magnificar",
    "magnifiqué": "magnificar",
    "magnifiquéis": "magnificar",
    "magnitudes": "magnitud",
    "magnolias": "magnolia",
    "magnoliácea": "magnoliáceo",
    "magnoliáceas": "magnoliáceo",
    "magnoliáceos": "magnoliáceo",
    "magnos": "magno",
    "magnánima": "magnánimo",
    "magnánimas": "magnánimo",
    "magnánimos": "magnánimo",
    "magnésica": "magnésico",
    "magnésicas": "magnésico",
    "magnésicos": "magnésico",
    "magnética": "magnético",
    "magnéticas": "magnético",
    "magnéticos": "magnético",
    "magnífica": "magnífico",
    "magníficas": "magnífico",
    "magníficates": "magníficat",
    "magníficos": "magnífico",
    "magos": "mago",
    "magostos": "magosto",
    "magra": "magro",
    "magras": "magro",
    "magrea": "magrear",
    "magreaba": "magrear",
    "magreabais": "magrear",
    "magreaban": "magrear",
    "magreabas": "magrear",
    "magread": "magrear",
    "magreada": "magrear",
    "magreadas": "magrear",
    "magreado": "magrear",
    "magreados": "magrear",
    "magreamos": "magrear",
    "magrean": "magrear",
    "magreando": "magrear",
    "magreara": "magrear",
    "magrearais": "magrear",
    "magrearan": "magrear",
    "magrearas": "magrear",
    "magreare": "magrear",
    "magreareis": "magrear",
    "magrearemos": "magrear",
    "magrearen": "magrear",
    "magreares": "magrear",
    "magrearon": "magrear",
    "magreará": "magrear",
    "magrearán": "magrear",
    "magrearás": "magrear",
    "magrearé": "magrear",
    "magrearéis": "magrear",
    "magrearía": "magrear",
    "magrearíais": "magrear",
    "magrearíamos": "magrear",
    "magrearían": "magrear",
    "magrearías": "magrear",
    "magreas": "magrear",
    "magrease": "magrear",
    "magreaseis": "magrear",
    "magreasen": "magrear",
    "magreases": "magrear",
    "magreaste": "magrear",
    "magreasteis": "magrear",
    "magrebíes": "magrebí",
    "magrebís": "magrebí",
    "magrebíses": "magrebí",
    "magreces": "magrez",
    "magree": "magrear",
    "magreemos": "magrear",
    "magreen": "magrear",
    "magrees": "magrear",
    "magreo": "magrear",
    "magreos": "magreo",
    "magreábamos": "magrear",
    "magreáis": "magrear",
    "magreáramos": "magrear",
    "magreáremos": "magrear",
    "magreásemos": "magrear",
    "magreé": "magrear",
    "magreéis": "magrear",
    "magreó": "magrear",
    "magros": "magro",
    "magruras": "magrura",
    "magueyes": "maguey",
    "magueys": "maguey",
    "magujos": "magujo",
    "magulla": "magullar",
    "magullaba": "magullar",
    "magullabais": "magullar",
    "magullaban": "magullar",
    "magullabas": "magullar",
    "magullad": "magullar",
    "magullada": "magullar",
    "magulladas": "magullar",
    "magullado": "magullar",
    "magullados": "magullar",
    "magulladuras": "magulladura",
    "magullamientos": "magullamiento",
    "magullamos": "magullar",
    "magullan": "magullar",
    "magullando": "magullar",
    "magullara": "magullar",
    "magullarais": "magullar",
    "magullaran": "magullar",
    "magullaras": "magullar",
    "magullare": "magullar",
    "magullareis": "magullar",
    "magullaremos": "magullar",
    "magullaren": "magullar",
    "magullares": "magullar",
    "magullaron": "magullar",
    "magullará": "magullar",
    "magullarán": "magullar",
    "magullarás": "magullar",
    "magullaré": "magullar",
    "magullaréis": "magullar",
    "magullaría": "magullar",
    "magullaríais": "magullar",
    "magullaríamos": "magullar",
    "magullarían": "magullar",
    "magullarías": "magullar",
    "magullas": "magullar",
    "magullase": "magullar",
    "magullaseis": "magullar",
    "magullasen": "magullar",
    "magullases": "magullar",
    "magullaste": "magullar",
    "magullasteis": "magullar",
    "magulle": "magullar",
    "magullemos": "magullar",
    "magullen": "magullar",
    "magulles": "magullar",
    "magullo": "magullar",
    "magullábamos": "magullar",
    "magulláis": "magullar",
    "magulláramos": "magullar",
    "magulláremos": "magullar",
    "magullásemos": "magullar",
    "magullé": "magullar",
    "magulléis": "magullar",
    "magulló": "magullar",
    "magüeta": "magüeto",
    "magüetas": "magüeto",
    "magüetos": "magüeto",
    "maharajás": "maharajá",
    "mahatmas": "mahatma",
    "mahayanas": "mahayana",
    "mahdies": "mahdi",
    "mahdis": "mahdi",
    "mahdíes": "mahdí",
    "mahdís": "mahdí",
    "mahometana": "mahometano",
    "mahometanas": "mahometano",
    "mahometanos": "mahometano",
    "mahometice": "mahometizar",
    "mahometicemos": "mahometizar",
    "mahometicen": "mahometizar",
    "mahometices": "mahometizar",
    "mahometicé": "mahometizar",
    "mahometicéis": "mahometizar",
    "mahometismos": "mahometismo",
    "mahometistas": "mahometista",
    "mahometiza": "mahometizar",
    "mahometizaba": "mahometizar",
    "mahometizabais": "mahometizar",
    "mahometizaban": "mahometizar",
    "mahometizabas": "mahometizar",
    "mahometizad": "mahometizar",
    "mahometizada": "mahometizar",
    "mahometizadas": "mahometizar",
    "mahometizado": "mahometizar",
    "mahometizados": "mahometizar",
    "mahometizamos": "mahometizar",
    "mahometizan": "mahometizar",
    "mahometizando": "mahometizar",
    "mahometizara": "mahometizar",
    "mahometizarais": "mahometizar",
    "mahometizaran": "mahometizar",
    "mahometizaras": "mahometizar",
    "mahometizare": "mahometizar",
    "mahometizareis": "mahometizar",
    "mahometizaremos": "mahometizar",
    "mahometizaren": "mahometizar",
    "mahometizares": "mahometizar",
    "mahometizaron": "mahometizar",
    "mahometizará": "mahometizar",
    "mahometizarán": "mahometizar",
    "mahometizarás": "mahometizar",
    "mahometizaré": "mahometizar",
    "mahometizaréis": "mahometizar",
    "mahometizaría": "mahometizar",
    "mahometizaríais": "mahometizar",
    "mahometizaríamos": "mahometizar",
    "mahometizarían": "mahometizar",
    "mahometizarías": "mahometizar",
    "mahometizas": "mahometizar",
    "mahometizase": "mahometizar",
    "mahometizaseis": "mahometizar",
    "mahometizasen": "mahometizar",
    "mahometizases": "mahometizar",
    "mahometizaste": "mahometizar",
    "mahometizasteis": "mahometizar",
    "mahometizo": "mahometizar",
    "mahometizábamos": "mahometizar",
    "mahometizáis": "mahometizar",
    "mahometizáramos": "mahometizar",
    "mahometizáremos": "mahometizar",
    "mahometizásemos": "mahometizar",
    "mahometizó": "mahometizar",
    "mahonesa": "mahonés",
    "mahonesas": "mahonesa",
    "mahoneses": "mahonés",
    "maicera": "maicero",
    "maiceras": "maicero",
    "maiceros": "maicero",
    "maicerías": "maicería",
    "mailings": "mailing",
    "maillots": "maillot",
    "maimones": "maimón",
    "maimonismos": "maimonismo",
    "maimonistas": "maimonista",
    "maineles": "mainel",
    "mainframes": "mainframe",
    "maitres": "maitre",
    "maizales": "maizal",
    "maja": "majo",
    "majaba": "majar",
    "majabais": "majar",
    "majaban": "majar",
    "majabas": "majar",
    "majad": "majar",
    "majada": "majar",
    "majadales": "majadal",
    "majadas": "majar",
    "majadea": "majadear",
    "majadeaba": "majadear",
    "majadeabais": "majadear",
    "majadeaban": "majadear",
    "majadeabas": "majadear",
    "majadead": "majadear",
    "majadeada": "majadear",
    "majadeadas": "majadear",
    "majadeado": "majadear",
    "majadeados": "majadear",
    "majadeamos": "majadear",
    "majadean": "majadear",
    "majadeando": "majadear",
    "majadeara": "majadear",
    "majadearais": "majadear",
    "majadearan": "majadear",
    "majadearas": "majadear",
    "majadeare": "majadear",
    "majadeareis": "majadear",
    "majadearemos": "majadear",
    "majadearen": "majadear",
    "majadeares": "majadear",
    "majadearon": "majadear",
    "majadeará": "majadear",
    "majadearán": "majadear",
    "majadearás": "majadear",
    "majadearé": "majadear",
    "majadearéis": "majadear",
    "majadearía": "majadear",
    "majadearíais": "majadear",
    "majadearíamos": "majadear",
    "majadearían": "majadear",
    "majadearías": "majadear",
    "majadeas": "majadear",
    "majadease": "majadear",
    "majadeaseis": "majadear",
    "majadeasen": "majadear",
    "majadeases": "majadear",
    "majadeaste": "majadear",
    "majadeasteis": "majadear",
    "majadee": "majadear",
    "majadeemos": "majadear",
    "majadeen": "majadear",
    "majadees": "majadear",
    "majadeo": "majadear",
    "majadera": "majadero",
    "majaderas": "majadero",
    "majadericos": "majaderico",
    "majaderillos": "majaderillo",
    "majaderos": "majadero",
    "majaderías": "majadería",
    "majadeábamos": "majadear",
    "majadeáis": "majadear",
    "majadeáramos": "majadear",
    "majadeáremos": "majadear",
    "majadeásemos": "majadear",
    "majadeé": "majadear",
    "majadeéis": "majadear",
    "majadeó": "majadear",
    "majado": "majar",
    "majadora": "majador",
    "majadoras": "majador",
    "majadores": "majador",
    "majados": "majar",
    "majaduras": "majadura",
    "majaguas": "majagua",
    "majales": "majal",
    "majamientos": "majamiento",
    "majamos": "majar",
    "majan": "majar",
    "majando": "majar",
    "majanos": "majano",
    "majara": "majar",
    "majarais": "majar",
    "majaran": "majar",
    "majaras": "majar",
    "majare": "majar",
    "majareis": "majar",
    "majaremos": "majar",
    "majaren": "majar",
    "majares": "majar",
    "majaretas": "majareta",
    "majaron": "majar",
    "majarros": "majarro",
    "majará": "majar",
    "majarán": "majar",
    "majarás": "majar",
    "majaré": "majar",
    "majaréis": "majar",
    "majaría": "majar",
    "majaríais": "majar",
    "majaríamos": "majar",
    "majarían": "majar",
    "majarías": "majar",
    "majas": "majo",
    "majase": "majar",
    "majaseis": "majar",
    "majasen": "majar",
    "majases": "majar",
    "majaste": "majar",
    "majasteis": "majar",
    "maje": "majar",
    "majemos": "majar",
    "majen": "majar",
    "majerías": "majería",
    "majes": "majar",
    "majestades": "majestad",
    "majestuosa": "majestuoso",
    "majestuosas": "majestuoso",
    "majestuosidades": "majestuosidad",
    "majestuosos": "majestuoso",
    "majezas": "majeza",
    "majo": "majar",
    "majorcas": "majorca",
    "majorettes": "majorette",
    "majos": "majo",
    "majuelas": "majuela",
    "majuelos": "majuelo",
    "majzenes": "majzén",
    "majábamos": "majar",
    "majáis": "majar",
    "majáramos": "majar",
    "majáremos": "majar",
    "majásemos": "majar",
    "majé": "majar",
    "majéis": "majar",
    "majó": "majar",
    "makemonos": "makemono",
    "mala": "malo",
    "malabares": "malabar",
    "malabarismos": "malabarismo",
    "malabaristas": "malabarista",
    "malacates": "malacate",
    "malacias": "malacia",
    "malacitana": "malacitano",
    "malacitanas": "malacitano",
    "malacitanos": "malacitano",
    "malacologías": "malacología",
    "malacológica": "malacológico",
    "malacológicas": "malacológico",
    "malacológicos": "malacológico",
    "malaconsejada": "malaconsejado",
    "malaconsejadas": "malaconsejado",
    "malaconsejados": "malaconsejado",
    "malacostumbra": "malacostumbrar",
    "malacostumbraba": "malacostumbrar",
    "malacostumbrabais": "malacostumbrar",
    "malacostumbraban": "malacostumbrar",
    "malacostumbrabas": "malacostumbrar",
    "malacostumbrad": "malacostumbrar",
    "malacostumbrada": "malacostumbrar",
    "malacostumbradas": "malacostumbrar",
    "malacostumbrado": "malacostumbrar",
    "malacostumbrados": "malacostumbrar",
    "malacostumbramos": "malacostumbrar",
    "malacostumbran": "malacostumbrar",
    "malacostumbrando": "malacostumbrar",
    "malacostumbrara": "malacostumbrar",
    "malacostumbrarais": "malacostumbrar",
    "malacostumbraran": "malacostumbrar",
    "malacostumbraras": "malacostumbrar",
    "malacostumbrare": "malacostumbrar",
    "malacostumbrareis": "malacostumbrar",
    "malacostumbraremos": "malacostumbrar",
    "malacostumbraren": "malacostumbrar",
    "malacostumbrares": "malacostumbrar",
    "malacostumbraron": "malacostumbrar",
    "malacostumbrará": "malacostumbrar",
    "malacostumbrarán": "malacostumbrar",
    "malacostumbrarás": "malacostumbrar",
    "malacostumbraré": "malacostumbrar",
    "malacostumbraréis": "malacostumbrar",
    "malacostumbraría": "malacostumbrar",
    "malacostumbraríais": "malacostumbrar",
    "malacostumbraríamos": "malacostumbrar",
    "malacostumbrarían": "malacostumbrar",
    "malacostumbrarías": "malacostumbrar",
    "malacostumbras": "malacostumbrar",
    "malacostumbrase": "malacostumbrar",
    "malacostumbraseis": "malacostumbrar",
    "malacostumbrasen": "malacostumbrar",
    "malacostumbrases": "malacostumbrar",
    "malacostumbraste": "malacostumbrar",
    "malacostumbrasteis": "malacostumbrar",
    "malacostumbre": "malacostumbrar",
    "malacostumbremos": "malacostumbrar",
    "malacostumbren": "malacostumbrar",
    "malacostumbres": "malacostumbrar",
    "malacostumbro": "malacostumbrar",
    "malacostumbrábamos": "malacostumbrar",
    "malacostumbráis": "malacostumbrar",
    "malacostumbráramos": "malacostumbrar",
    "malacostumbráremos": "malacostumbrar",
    "malacostumbrásemos": "malacostumbrar",
    "malacostumbré": "malacostumbrar",
    "malacostumbréis": "malacostumbrar",
    "malacostumbró": "malacostumbrar",
    "malacrianzas": "malacrianza",
    "malacuendas": "malacuenda",
    "malacóloga": "malacólogo",
    "malacólogas": "malacólogo",
    "malacólogos": "malacólogo",
    "malacótica": "malacótico",
    "malacóticas": "malacótico",
    "malacóticos": "malacótico",
    "malaganas": "malagana",
    "malagradecida": "malagradecido",
    "malagradecidas": "malagradecido",
    "malagradecidos": "malagradecido",
    "malagueña": "malagueño",
    "malagueñas": "malagueño",
    "malagueños": "malagueño",
    "malaguistas": "malaguista",
    "malandantes": "malandante",
    "malandanzas": "malandanza",
    "malandares": "malandar",
    "malandrina": "malandrín",
    "malandrinas": "malandrín",
    "malandrines": "malandrín",
    "malapatas": "malapata",
    "malaquitas": "malaquita",
    "malares": "malar",
    "malarias": "malaria",
    "malas": "malo",
    "malasangres": "malasangre",
    "malasia": "malasio",
    "malasias": "malasio",
    "malasios": "malasio",
    "malasombras": "malasombra",
    "malata": "malato",
    "malatas": "malato",
    "malaterías": "malatería",
    "malationes": "malatión",
    "malatos": "malato",
    "malavenida": "malavenido",
    "malavenidas": "malavenido",
    "malavenidos": "malavenido",
    "malaventurada": "malaventurado",
    "malaventuradas": "malaventurado",
    "malaventurados": "malaventurado",
    "malaventuranzas": "malaventuranza",
    "malaventuras": "malaventura",
    "malawiana": "malawiano",
    "malawianas": "malawiano",
    "malawianos": "malawiano",
    "malaxa": "malaxar",
    "malaxaba": "malaxar",
    "malaxabais": "malaxar",
    "malaxaban": "malaxar",
    "malaxabas": "malaxar",
    "malaxad": "malaxar",
    "malaxada": "malaxar",
    "malaxadas": "malaxar",
    "malaxado": "malaxar",
    "malaxadoras": "malaxadora",
    "malaxados": "malaxar",
    "malaxamos": "malaxar",
    "malaxan": "malaxar",
    "malaxando": "malaxar",
    "malaxara": "malaxar",
    "malaxarais": "malaxar",
    "malaxaran": "malaxar",
    "malaxaras": "malaxar",
    "malaxare": "malaxar",
    "malaxareis": "malaxar",
    "malaxaremos": "malaxar",
    "malaxaren": "malaxar",
    "malaxares": "malaxar",
    "malaxaron": "malaxar",
    "malaxará": "malaxar",
    "malaxarán": "malaxar",
    "malaxarás": "malaxar",
    "malaxaré": "malaxar",
    "malaxaréis": "malaxar",
    "malaxaría": "malaxar",
    "malaxaríais": "malaxar",
    "malaxaríamos": "malaxar",
    "malaxarían": "malaxar",
    "malaxarías": "malaxar",
    "malaxas": "malaxar",
    "malaxase": "malaxar",
    "malaxaseis": "malaxar",
    "malaxasen": "malaxar",
    "malaxases": "malaxar",
    "malaxaste": "malaxar",
    "malaxasteis": "malaxar",
    "malaxe": "malaxar",
    "malaxemos": "malaxar",
    "malaxen": "malaxar",
    "malaxes": "malaxar",
    "malaxo": "malaxar",
    "malaxábamos": "malaxar",
    "malaxáis": "malaxar",
    "malaxáramos": "malaxar",
    "malaxáremos": "malaxar",
    "malaxásemos": "malaxar",
    "malaxé": "malaxar",
    "malaxéis": "malaxar",
    "malaxó": "malaxar",
    "malaya": "malayo",
    "malayas": "malayo",
    "malayopolinesia": "malayopolinesio",
    "malayopolinesias": "malayopolinesio",
    "malayopolinesios": "malayopolinesio",
    "malayos": "malayo",
    "malbarata": "malbaratar",
    "malbarataba": "malbaratar",
    "malbaratabais": "malbaratar",
    "malbarataban": "malbaratar",
    "malbaratabas": "malbaratar",
    "malbaratad": "malbaratar",
    "malbaratada": "malbaratar",
    "malbaratadas": "malbaratar",
    "malbaratado": "malbaratar",
    "malbaratadora": "malbaratador",
    "malbaratadoras": "malbaratador",
    "malbaratadores": "malbaratador",
    "malbaratados": "malbaratar",
    "malbaratamos": "malbaratar",
    "malbaratan": "malbaratar",
    "malbaratando": "malbaratar",
    "malbaratara": "malbaratar",
    "malbaratarais": "malbaratar",
    "malbarataran": "malbaratar",
    "malbarataras": "malbaratar",
    "malbaratare": "malbaratar",
    "malbaratareis": "malbaratar",
    "malbarataremos": "malbaratar",
    "malbarataren": "malbaratar",
    "malbaratares": "malbaratar",
    "malbarataron": "malbaratar",
    "malbaratará": "malbaratar",
    "malbaratarán": "malbaratar",
    "malbaratarás": "malbaratar",
    "malbarataré": "malbaratar",
    "malbarataréis": "malbaratar",
    "malbarataría": "malbaratar",
    "malbarataríais": "malbaratar",
    "malbarataríamos": "malbaratar",
    "malbaratarían": "malbaratar",
    "malbaratarías": "malbaratar",
    "malbaratas": "malbaratar",
    "malbaratase": "malbaratar",
    "malbarataseis": "malbaratar",
    "malbaratasen": "malbaratar",
    "malbaratases": "malbaratar",
    "malbarataste": "malbaratar",
    "malbaratasteis": "malbaratar",
    "malbarate": "malbaratar",
    "malbaratemos": "malbaratar",
    "malbaraten": "malbaratar",
    "malbarates": "malbaratar",
    "malbarato": "malbaratar",
    "malbaratos": "malbarato",
    "malbaratábamos": "malbaratar",
    "malbaratáis": "malbaratar",
    "malbaratáramos": "malbaratar",
    "malbaratáremos": "malbaratar",
    "malbaratásemos": "malbaratar",
    "malbaraté": "malbaratar",
    "malbaratéis": "malbaratar",
    "malbarató": "malbaratar",
    "malcarada": "malcarado",
    "malcaradas": "malcarado",
    "malcarados": "malcarado",
    "malcasa": "malcasar",
    "malcasaba": "malcasar",
    "malcasabais": "malcasar",
    "malcasaban": "malcasar",
    "malcasabas": "malcasar",
    "malcasad": "malcasar",
    "malcasada": "malcasar",
    "malcasadas": "malcasar",
    "malcasado": "malcasar",
    "malcasados": "malcasar",
    "malcasamos": "malcasar",
    "malcasan": "malcasar",
    "malcasando": "malcasar",
    "malcasara": "malcasar",
    "malcasarais": "malcasar",
    "malcasaran": "malcasar",
    "malcasaras": "malcasar",
    "malcasare": "malcasar",
    "malcasareis": "malcasar",
    "malcasaremos": "malcasar",
    "malcasaren": "malcasar",
    "malcasares": "malcasar",
    "malcasaron": "malcasar",
    "malcasará": "malcasar",
    "malcasarán": "malcasar",
    "malcasarás": "malcasar",
    "malcasaré": "malcasar",
    "malcasaréis": "malcasar",
    "malcasaría": "malcasar",
    "malcasaríais": "malcasar",
    "malcasaríamos": "malcasar",
    "malcasarían": "malcasar",
    "malcasarías": "malcasar",
    "malcasas": "malcasar",
    "malcasase": "malcasar",
    "malcasaseis": "malcasar",
    "malcasasen": "malcasar",
    "malcasases": "malcasar",
    "malcasaste": "malcasar",
    "malcasasteis": "malcasar",
    "malcase": "malcasar",
    "malcasemos": "malcasar",
    "malcasen": "malcasar",
    "malcases": "malcasar",
    "malcaso": "malcasar",
    "malcasos": "malcaso",
    "malcasábamos": "malcasar",
    "malcasáis": "malcasar",
    "malcasáramos": "malcasar",
    "malcasáremos": "malcasar",
    "malcasásemos": "malcasar",
    "malcasé": "malcasar",
    "malcaséis": "malcasar",
    "malcasó": "malcasar",
    "malcocinados": "malcocinado",
    "malcoma": "malcomer",
    "malcomamos": "malcomer",
    "malcoman": "malcomer",
    "malcomas": "malcomer",
    "malcome": "malcomer",
    "malcomed": "malcomer",
    "malcomemos": "malcomer",
    "malcomen": "malcomer",
    "malcomeremos": "malcomer",
    "malcomerá": "malcomer",
    "malcomerán": "malcomer",
    "malcomerás": "malcomer",
    "malcomeré": "malcomer",
    "malcomeréis": "malcomer",
    "malcomería": "malcomer",
    "malcomeríais": "malcomer",
    "malcomeríamos": "malcomer",
    "malcomerían": "malcomer",
    "malcomerías": "malcomer",
    "malcomes": "malcomer",
    "malcomida": "malcomer",
    "malcomidas": "malcomer",
    "malcomido": "malcomer",
    "malcomidos": "malcomer",
    "malcomiendo": "malcomer",
    "malcomiera": "malcomer",
    "malcomierais": "malcomer",
    "malcomieran": "malcomer",
    "malcomieras": "malcomer",
    "malcomiere": "malcomer",
    "malcomiereis": "malcomer",
    "malcomieren": "malcomer",
    "malcomieres": "malcomer",
    "malcomieron": "malcomer",
    "malcomiese": "malcomer",
    "malcomieseis": "malcomer",
    "malcomiesen": "malcomer",
    "malcomieses": "malcomer",
    "malcomimos": "malcomer",
    "malcomiste": "malcomer",
    "malcomisteis": "malcomer",
    "malcomiéramos": "malcomer",
    "malcomiéremos": "malcomer",
    "malcomiésemos": "malcomer",
    "malcomió": "malcomer",
    "malcomo": "malcomer",
    "malcomáis": "malcomer",
    "malcoméis": "malcomer",
    "malcomí": "malcomer",
    "malcomía": "malcomer",
    "malcomíais": "malcomer",
    "malcomíamos": "malcomer",
    "malcomían": "malcomer",
    "malcomías": "malcomer",
    "malconsiderada": "malconsiderado",
    "malconsideradas": "malconsiderado",
    "malconsiderados": "malconsiderado",
    "malcontenta": "malcontento",
    "malcontentadiza": "malcontentadizo",
    "malcontentadizas": "malcontentadizo",
    "malcontentadizos": "malcontentadizo",
    "malcontentas": "malcontento",
    "malcontentos": "malcontento",
    "malcortes": "malcorte",
    "malcriaba": "malcriar",
    "malcriabais": "malcriar",
    "malcriaban": "malcriar",
    "malcriabas": "malcriar",
    "malcriad": "malcriar",
    "malcriada": "malcriar",
    "malcriadas": "malcriar",
    "malcriado": "malcriar",
    "malcriados": "malcriar",
    "malcriamos": "malcriar",
    "malcriando": "malcriar",
    "malcriara": "malcriar",
    "malcriarais": "malcriar",
    "malcriaran": "malcriar",
    "malcriaras": "malcriar",
    "malcriare": "malcriar",
    "malcriareis": "malcriar",
    "malcriaremos": "malcriar",
    "malcriaren": "malcriar",
    "malcriares": "malcriar",
    "malcriaron": "malcriar",
    "malcriará": "malcriar",
    "malcriarán": "malcriar",
    "malcriarás": "malcriar",
    "malcriaré": "malcriar",
    "malcriaréis": "malcriar",
    "malcriaría": "malcriar",
    "malcriaríais": "malcriar",
    "malcriaríamos": "malcriar",
    "malcriarían": "malcriar",
    "malcriarías": "malcriar",
    "malcriase": "malcriar",
    "malcriaseis": "malcriar",
    "malcriasen": "malcriar",
    "malcriases": "malcriar",
    "malcriaste": "malcriar",
    "malcriasteis": "malcriar",
    "malcriemos": "malcriar",
    "malcriábamos": "malcriar",
    "malcriáis": "malcriar",
    "malcriáramos": "malcriar",
    "malcriáremos": "malcriar",
    "malcriásemos": "malcriar",
    "malcrié": "malcriar",
    "malcriéis": "malcriar",
    "malcrió": "malcriar",
    "malcría": "malcriar",
    "malcrían": "malcriar",
    "malcrías": "malcriar",
    "malcríe": "malcriar",
    "malcríen": "malcriar",
    "malcríes": "malcriar",
    "malcrío": "malcriar",
    "maldades": "maldad",
    "maldecid": "maldecir",
    "maldecida": "maldecido",
    "maldecidas": "maldecido",
    "maldecidora": "maldecidor",
    "maldecidoras": "maldecidor",
    "maldecidores": "maldecidor",
    "maldecidos": "maldecido",
    "maldecimos": "maldecir",
    "maldeciremos": "maldecir",
    "maldecirá": "maldecir",
    "maldecirán": "maldecir",
    "maldecirás": "maldecir",
    "maldeciré": "maldecir",
    "maldeciréis": "maldecir",
    "maldeciría": "maldecir",
    "maldeciríais": "maldecir",
    "maldeciríamos": "maldecir",
    "maldecirían": "maldecir",
    "maldecirías": "maldecir",
    "maldecía": "maldecir",
    "maldecíais": "maldecir",
    "maldecíamos": "maldecir",
    "maldecían": "maldecir",
    "maldecías": "maldecir",
    "maldecís": "maldecir",
    "maldice": "maldecir",
    "maldicen": "maldecir",
    "maldices": "maldecir",
    "maldicha": "maldecir",
    "maldichas": "maldecir",
    "maldicho": "maldecir",
    "maldichos": "maldecir",
    "maldiciendo": "maldecir",
    "maldicientes": "maldiciente",
    "maldiciones": "maldición",
    "maldiciónes": "maldición",
    "maldiga": "maldecir",
    "maldigamos": "maldecir",
    "maldigan": "maldecir",
    "maldigas": "maldecir",
    "maldigo": "maldecir",
    "maldigáis": "maldecir",
    "maldije": "maldecir",
    "maldijera": "maldecir",
    "maldijerais": "maldecir",
    "maldijeran": "maldecir",
    "maldijeras": "maldecir",
    "maldijere": "maldecir",
    "maldijereis": "maldecir",
    "maldijeren": "maldecir",
    "maldijeres": "maldecir",
    "maldijeron": "maldecir",
    "maldijese": "maldecir",
    "maldijeseis": "maldecir",
    "maldijesen": "maldecir",
    "maldijeses": "maldecir",
    "maldijimos": "maldecir",
    "maldijiste": "maldecir",
    "maldijisteis": "maldecir",
    "maldijo": "maldecir",
    "maldijéramos": "maldecir",
    "maldijéremos": "maldecir",
    "maldijésemos": "maldecir",
    "maldispuesta": "maldispuesto",
    "maldispuestas": "maldispuesto",
    "maldispuestos": "maldispuesto",
    "maldita": "maldito",
    "malditas": "maldito",
    "malditos": "maldito",
    "malea": "malear",
    "maleaba": "malear",
    "maleabais": "malear",
    "maleaban": "malear",
    "maleabas": "malear",
    "maleabilice": "maleabilizar",
    "maleabilicemos": "maleabilizar",
    "maleabilicen": "maleabilizar",
    "maleabilices": "maleabilizar",
    "maleabilicé": "maleabilizar",
    "maleabilicéis": "maleabilizar",
    "maleabilidades": "maleabilidad",
    "maleabiliza": "maleabilizar",
    "maleabilizaba": "maleabilizar",
    "maleabilizabais": "maleabilizar",
    "maleabilizaban": "maleabilizar",
    "maleabilizabas": "maleabilizar",
    "maleabilizaciones": "maleabilización",
    "maleabilizad": "maleabilizar",
    "maleabilizada": "maleabilizar",
    "maleabilizadas": "maleabilizar",
    "maleabilizado": "maleabilizar",
    "maleabilizados": "maleabilizar",
    "maleabilizamos": "maleabilizar",
    "maleabilizan": "maleabilizar",
    "maleabilizando": "maleabilizar",
    "maleabilizara": "maleabilizar",
    "maleabilizarais": "maleabilizar",
    "maleabilizaran": "maleabilizar",
    "maleabilizaras": "maleabilizar",
    "maleabilizare": "maleabilizar",
    "maleabilizareis": "maleabilizar",
    "maleabilizaremos": "maleabilizar",
    "maleabilizaren": "maleabilizar",
    "maleabilizares": "maleabilizar",
    "maleabilizaron": "maleabilizar",
    "maleabilizará": "maleabilizar",
    "maleabilizarán": "maleabilizar",
    "maleabilizarás": "maleabilizar",
    "maleabilizaré": "maleabilizar",
    "maleabilizaréis": "maleabilizar",
    "maleabilizaría": "maleabilizar",
    "maleabilizaríais": "maleabilizar",
    "maleabilizaríamos": "maleabilizar",
    "maleabilizarían": "maleabilizar",
    "maleabilizarías": "maleabilizar",
    "maleabilizas": "maleabilizar",
    "maleabilizase": "maleabilizar",
    "maleabilizaseis": "maleabilizar",
    "maleabilizasen": "maleabilizar",
    "maleabilizases": "maleabilizar",
    "maleabilizaste": "maleabilizar",
    "maleabilizasteis": "maleabilizar",
    "maleabilizo": "maleabilizar",
    "maleabilizábamos": "maleabilizar",
    "maleabilizáis": "maleabilizar",
    "maleabilizáramos": "maleabilizar",
    "maleabilizáremos": "maleabilizar",
    "maleabilizásemos": "maleabilizar",
    "maleabilizó": "maleabilizar",
    "maleables": "maleable",
    "malead": "malear",
    "maleada": "malear",
    "maleadas": "malear",
    "maleado": "malear",
    "maleadora": "maleador",
    "maleadoras": "maleador",
    "maleadores": "maleador",
    "maleados": "malear",
    "maleales": "maleal",
    "maleamos": "malear",
    "malean": "malear",
    "maleando": "malear",
    "maleantes": "maleante",
    "maleara": "malear",
    "malearais": "malear",
    "malearan": "malear",
    "malearas": "malear",
    "maleare": "malear",
    "maleareis": "malear",
    "malearemos": "malear",
    "malearen": "malear",
    "maleares": "malear",
    "malearon": "malear",
    "maleará": "malear",
    "malearán": "malear",
    "malearás": "malear",
    "malearé": "malear",
    "malearéis": "malear",
    "malearía": "malear",
    "malearíais": "malear",
    "malearíamos": "malear",
    "malearían": "malear",
    "malearías": "malear",
    "maleas": "malear",
    "malease": "malear",
    "maleaseis": "malear",
    "maleasen": "malear",
    "maleases": "malear",
    "maleaste": "malear",
    "maleasteis": "malear",
    "maleatos": "maleato",
    "malecones": "malecón",
    "maledicencias": "maledicencia",
    "maleducada": "maleducado",
    "maleducadas": "maleducado",
    "maleducados": "maleducado",
    "malee": "malear",
    "maleemos": "malear",
    "maleen": "malear",
    "malees": "malear",
    "maleficencias": "maleficencia",
    "maleficia": "maleficiar",
    "maleficiaba": "maleficiar",
    "maleficiabais": "maleficiar",
    "maleficiaban": "maleficiar",
    "maleficiabas": "maleficiar",
    "maleficiad": "maleficiar",
    "maleficiada": "maleficiar",
    "maleficiadas": "maleficiar",
    "maleficiado": "maleficiar",
    "maleficiados": "maleficiar",
    "maleficiamos": "maleficiar",
    "malefician": "maleficiar",
    "maleficiando": "maleficiar",
    "maleficiara": "maleficiar",
    "maleficiarais": "maleficiar",
    "maleficiaran": "maleficiar",
    "maleficiaras": "maleficiar",
    "maleficiare": "maleficiar",
    "maleficiareis": "maleficiar",
    "maleficiaremos": "maleficiar",
    "maleficiaren": "maleficiar",
    "maleficiares": "maleficiar",
    "maleficiaron": "maleficiar",
    "maleficiará": "maleficiar",
    "maleficiarán": "maleficiar",
    "maleficiarás": "maleficiar",
    "maleficiaré": "maleficiar",
    "maleficiaréis": "maleficiar",
    "maleficiaría": "maleficiar",
    "maleficiaríais": "maleficiar",
    "maleficiaríamos": "maleficiar",
    "maleficiarían": "maleficiar",
    "maleficiarías": "maleficiar",
    "maleficias": "maleficiar",
    "maleficiase": "maleficiar",
    "maleficiaseis": "maleficiar",
    "maleficiasen": "maleficiar",
    "maleficiases": "maleficiar",
    "maleficiaste": "maleficiar",
    "maleficiasteis": "maleficiar",
    "maleficie": "maleficiar",
    "maleficiemos": "maleficiar",
    "maleficien": "maleficiar",
    "maleficies": "maleficiar",
    "maleficio": "maleficiar",
    "maleficios": "maleficio",
    "maleficiábamos": "maleficiar",
    "maleficiáis": "maleficiar",
    "maleficiáramos": "maleficiar",
    "maleficiáremos": "maleficiar",
    "maleficiásemos": "maleficiar",
    "maleficié": "maleficiar",
    "maleficiéis": "maleficiar",
    "malefició": "maleficiar",
    "maleica": "maleico",
    "maleicas": "maleico",
    "maleicos": "maleico",
    "malentendamos": "malentender",
    "malentended": "malentender",
    "malentendemos": "malentender",
    "malentenderemos": "malentender",
    "malentenderá": "malentender",
    "malentenderán": "malentender",
    "malentenderás": "malentender",
    "malentenderé": "malentender",
    "malentenderéis": "malentender",
    "malentendería": "malentender",
    "malentenderíais": "malentender",
    "malentenderíamos": "malentender",
    "malentenderían": "malentender",
    "malentenderías": "malentender",
    "malentendida": "malentender",
    "malentendidas": "malentender",
    "malentendido": "malentender",
    "malentendidos": "malentendido",
    "malentendiendo": "malentender",
    "malentendiera": "malentender",
    "malentendierais": "malentender",
    "malentendieran": "malentender",
    "malentendieras": "malentender",
    "malentendiere": "malentender",
    "malentendiereis": "malentender",
    "malentendieren": "malentender",
    "malentendieres": "malentender",
    "malentendieron": "malentender",
    "malentendiese": "malentender",
    "malentendieseis": "malentender",
    "malentendiesen": "malentender",
    "malentendieses": "malentender",
    "malentendimos": "malentender",
    "malentendiste": "malentender",
    "malentendisteis": "malentender",
    "malentendiéramos": "malentender",
    "malentendiéremos": "malentender",
    "malentendiésemos": "malentender",
    "malentendió": "malentender",
    "malentendáis": "malentender",
    "malentendéis": "malentender",
    "malentendí": "malentender",
    "malentendía": "malentender",
    "malentendíais": "malentender",
    "malentendíamos": "malentender",
    "malentendían": "malentender",
    "malentendías": "malentender",
    "malentienda": "malentender",
    "malentiendan": "malentender",
    "malentiendas": "malentender",
    "malentiende": "malentender",
    "malentienden": "malentender",
    "malentiendes": "malentender",
    "malentiendo": "malentender",
    "maleo": "malear",
    "maleolares": "maleolar",
    "maleolos": "maleolo",
    "malerupciones": "malerupción",
    "males": "mal",
    "malestares": "malestar",
    "maletas": "maleta",
    "maletera": "maletero",
    "maleteras": "maletero",
    "maleteros": "maletero",
    "maletillas": "maletilla",
    "maletines": "maletín",
    "malevolencias": "malevolencia",
    "malezas": "maleza",
    "maleábamos": "malear",
    "maleáis": "malear",
    "maleáramos": "malear",
    "maleáremos": "malear",
    "maleásemos": "malear",
    "maleé": "malear",
    "maleéis": "malear",
    "maleó": "malear",
    "malformaciones": "malformación",
    "malgaches": "malgache",
    "malgasta": "malgastar",
    "malgastaba": "malgastar",
    "malgastabais": "malgastar",
    "malgastaban": "malgastar",
    "malgastabas": "malgastar",
    "malgastad": "malgastar",
    "malgastada": "malgastar",
    "malgastadas": "malgastar",
    "malgastado": "malgastar",
    "malgastadora": "malgastador",
    "malgastadoras": "malgastador",
    "malgastadores": "malgastador",
    "malgastados": "malgastar",
    "malgastamos": "malgastar",
    "malgastan": "malgastar",
    "malgastando": "malgastar",
    "malgastara": "malgastar",
    "malgastarais": "malgastar",
    "malgastaran": "malgastar",
    "malgastaras": "malgastar",
    "malgastare": "malgastar",
    "malgastareis": "malgastar",
    "malgastaremos": "malgastar",
    "malgastaren": "malgastar",
    "malgastares": "malgastar",
    "malgastaron": "malgastar",
    "malgastará": "malgastar",
    "malgastarán": "malgastar",
    "malgastarás": "malgastar",
    "malgastaré": "malgastar",
    "malgastaréis": "malgastar",
    "malgastaría": "malgastar",
    "malgastaríais": "malgastar",
    "malgastaríamos": "malgastar",
    "malgastarían": "malgastar",
    "malgastarías": "malgastar",
    "malgastas": "malgastar",
    "malgastase": "malgastar",
    "malgastaseis": "malgastar",
    "malgastasen": "malgastar",
    "malgastases": "malgastar",
    "malgastaste": "malgastar",
    "malgastasteis": "malgastar",
    "malgaste": "malgastar",
    "malgastemos": "malgastar",
    "malgasten": "malgastar",
    "malgastes": "malgastar",
    "malgasto": "malgastar",
    "malgastábamos": "malgastar",
    "malgastáis": "malgastar",
    "malgastáramos": "malgastar",
    "malgastáremos": "malgastar",
    "malgastásemos": "malgastar",
    "malgasté": "malgastar",
    "malgastéis": "malgastar",
    "malgastó": "malgastar",
    "malhablada": "malhablado",
    "malhabladas": "malhablado",
    "malhablados": "malhablado",
    "malhadada": "malhadado",
    "malhadadas": "malhadado",
    "malhadados": "malhadado",
    "malhechora": "malhechor",
    "malhechoras": "malhechor",
    "malhechores": "malhechor",
    "malhechos": "malhecho",
    "malherid": "malherir",
    "malherida": "malherir",
    "malheridas": "malherir",
    "malherido": "malherir",
    "malheridos": "malherir",
    "malherimos": "malherir",
    "malheriremos": "malherir",
    "malherirá": "malherir",
    "malherirán": "malherir",
    "malherirás": "malherir",
    "malheriré": "malherir",
    "malheriréis": "malherir",
    "malheriría": "malherir",
    "malheriríais": "malherir",
    "malheriríamos": "malherir",
    "malherirían": "malherir",
    "malherirías": "malherir",
    "malheriste": "malherir",
    "malheristeis": "malherir",
    "malherí": "malherir",
    "malhería": "malherir",
    "malheríais": "malherir",
    "malheríamos": "malherir",
    "malherían": "malherir",
    "malherías": "malherir",
    "malherís": "malherir",
    "malhiera": "malherir",
    "malhieran": "malherir",
    "malhieras": "malherir",
    "malhiere": "malherir",
    "malhieren": "malherir",
    "malhieres": "malherir",
    "malhiero": "malherir",
    "malhiramos": "malherir",
    "malhiriendo": "malherir",
    "malhiriera": "malherir",
    "malhirierais": "malherir",
    "malhirieran": "malherir",
    "malhirieras": "malherir",
    "malhiriere": "malherir",
    "malhiriereis": "malherir",
    "malhirieren": "malherir",
    "malhirieres": "malherir",
    "malhirieron": "malherir",
    "malhiriese": "malherir",
    "malhirieseis": "malherir",
    "malhiriesen": "malherir",
    "malhirieses": "malherir",
    "malhiriéramos": "malherir",
    "malhiriéremos": "malherir",
    "malhiriésemos": "malherir",
    "malhirió": "malherir",
    "malhiráis": "malherir",
    "malhumora": "malhumorar",
    "malhumoraba": "malhumorar",
    "malhumorabais": "malhumorar",
    "malhumoraban": "malhumorar",
    "malhumorabas": "malhumorar",
    "malhumorad": "malhumorar",
    "malhumorada": "malhumorar",
    "malhumoradas": "malhumorar",
    "malhumorado": "malhumorar",
    "malhumorados": "malhumorar",
    "malhumoramos": "malhumorar",
    "malhumoran": "malhumorar",
    "malhumorando": "malhumorar",
    "malhumorara": "malhumorar",
    "malhumorarais": "malhumorar",
    "malhumoraran": "malhumorar",
    "malhumoraras": "malhumorar",
    "malhumorare": "malhumorar",
    "malhumorareis": "malhumorar",
    "malhumoraremos": "malhumorar",
    "malhumoraren": "malhumorar",
    "malhumorares": "malhumorar",
    "malhumoraron": "malhumorar",
    "malhumorará": "malhumorar",
    "malhumorarán": "malhumorar",
    "malhumorarás": "malhumorar",
    "malhumoraré": "malhumorar",
    "malhumoraréis": "malhumorar",
    "malhumoraría": "malhumorar",
    "malhumoraríais": "malhumorar",
    "malhumoraríamos": "malhumorar",
    "malhumorarían": "malhumorar",
    "malhumorarías": "malhumorar",
    "malhumoras": "malhumorar",
    "malhumorase": "malhumorar",
    "malhumoraseis": "malhumorar",
    "malhumorasen": "malhumorar",
    "malhumorases": "malhumorar",
    "malhumoraste": "malhumorar",
    "malhumorasteis": "malhumorar",
    "malhumore": "malhumorar",
    "malhumoremos": "malhumorar",
    "malhumoren": "malhumorar",
    "malhumores": "malhumorar",
    "malhumoro": "malhumorar",
    "malhumorábamos": "malhumorar",
    "malhumoráis": "malhumorar",
    "malhumoráramos": "malhumorar",
    "malhumoráremos": "malhumorar",
    "malhumorásemos": "malhumorar",
    "malhumoré": "malhumorar",
    "malhumoréis": "malhumorar",
    "malhumoró": "malhumorar",
    "malicia": "maliciar",
    "maliciaba": "maliciar",
    "maliciabais": "maliciar",
    "maliciaban": "maliciar",
    "maliciabas": "maliciar",
    "maliciables": "maliciable",
    "maliciad": "maliciar",
    "maliciada": "maliciar",
    "maliciadas": "maliciar",
    "maliciado": "maliciar",
    "maliciadora": "maliciador",
    "maliciadoras": "maliciador",
    "maliciadores": "maliciador",
    "maliciados": "maliciar",
    "maliciamos": "maliciar",
    "malician": "maliciar",
    "maliciando": "maliciar",
    "maliciara": "maliciar",
    "maliciarais": "maliciar",
    "maliciaran": "maliciar",
    "maliciaras": "maliciar",
    "maliciare": "maliciar",
    "maliciareis": "maliciar",
    "maliciaremos": "maliciar",
    "maliciaren": "maliciar",
    "maliciares": "maliciar",
    "maliciaron": "maliciar",
    "maliciará": "maliciar",
    "maliciarán": "maliciar",
    "maliciarás": "maliciar",
    "maliciaré": "maliciar",
    "maliciaréis": "maliciar",
    "maliciaría": "maliciar",
    "maliciaríais": "maliciar",
    "maliciaríamos": "maliciar",
    "maliciarían": "maliciar",
    "maliciarías": "maliciar",
    "malicias": "maliciar",
    "maliciase": "maliciar",
    "maliciaseis": "maliciar",
    "maliciasen": "maliciar",
    "maliciases": "maliciar",
    "maliciaste": "maliciar",
    "maliciasteis": "maliciar",
    "malicie": "maliciar",
    "maliciemos": "maliciar",
    "malicien": "maliciar",
    "malicies": "maliciar",
    "malicio": "maliciar",
    "maliciosa": "malicioso",
    "maliciosas": "malicioso",
    "maliciosos": "malicioso",
    "maliciábamos": "maliciar",
    "maliciáis": "maliciar",
    "maliciáramos": "maliciar",
    "maliciáremos": "maliciar",
    "maliciásemos": "maliciar",
    "malicié": "maliciar",
    "maliciéis": "maliciar",
    "malició": "maliciar",
    "maligna": "maligno",
    "malignas": "maligno",
    "malignidades": "malignidad",
    "malignos": "maligno",
    "malikíes": "malikí",
    "malikís": "malikí",
    "malillas": "malilla",
    "malintencionada": "malintencionado",
    "malintencionadas": "malintencionado",
    "malintencionados": "malintencionado",
    "malinterpreta": "malinterpretar",
    "malinterpretaba": "malinterpretar",
    "malinterpretabais": "malinterpretar",
    "malinterpretaban": "malinterpretar",
    "malinterpretabas": "malinterpretar",
    "malinterpretad": "malinterpretar",
    "malinterpretada": "malinterpretar",
    "malinterpretadas": "malinterpretar",
    "malinterpretado": "malinterpretar",
    "malinterpretados": "malinterpretar",
    "malinterpretamos": "malinterpretar",
    "malinterpretan": "malinterpretar",
    "malinterpretando": "malinterpretar",
    "malinterpretara": "malinterpretar",
    "malinterpretarais": "malinterpretar",
    "malinterpretaran": "malinterpretar",
    "malinterpretaras": "malinterpretar",
    "malinterpretare": "malinterpretar",
    "malinterpretareis": "malinterpretar",
    "malinterpretaremos": "malinterpretar",
    "malinterpretaren": "malinterpretar",
    "malinterpretares": "malinterpretar",
    "malinterpretaron": "malinterpretar",
    "malinterpretará": "malinterpretar",
    "malinterpretarán": "malinterpretar",
    "malinterpretarás": "malinterpretar",
    "malinterpretaré": "malinterpretar",
    "malinterpretaréis": "malinterpretar",
    "malinterpretaría": "malinterpretar",
    "malinterpretaríais": "malinterpretar",
    "malinterpretaríamos": "malinterpretar",
    "malinterpretarían": "malinterpretar",
    "malinterpretarías": "malinterpretar",
    "malinterpretas": "malinterpretar",
    "malinterpretase": "malinterpretar",
    "malinterpretaseis": "malinterpretar",
    "malinterpretasen": "malinterpretar",
    "malinterpretases": "malinterpretar",
    "malinterpretaste": "malinterpretar",
    "malinterpretasteis": "malinterpretar",
    "malinterprete": "malinterpretar",
    "malinterpretemos": "malinterpretar",
    "malinterpreten": "malinterpretar",
    "malinterpretes": "malinterpretar",
    "malinterpreto": "malinterpretar",
    "malinterpretábamos": "malinterpretar",
    "malinterpretáis": "malinterpretar",
    "malinterpretáramos": "malinterpretar",
    "malinterpretáremos": "malinterpretar",
    "malinterpretásemos": "malinterpretar",
    "malinterpreté": "malinterpretar",
    "malinterpretéis": "malinterpretar",
    "malinterpretó": "malinterpretar",
    "malla": "mallar",
    "mallaba": "mallar",
    "mallabais": "mallar",
    "mallaban": "mallar",
    "mallabas": "mallar",
    "mallad": "mallar",
    "mallada": "mallar",
    "malladas": "mallar",
    "mallado": "mallar",
    "mallados": "mallar",
    "mallamos": "mallar",
    "mallan": "mallar",
    "mallando": "mallar",
    "mallara": "mallar",
    "mallarais": "mallar",
    "mallaran": "mallar",
    "mallaras": "mallar",
    "mallare": "mallar",
    "mallareis": "mallar",
    "mallaremos": "mallar",
    "mallaren": "mallar",
    "mallares": "mallar",
    "mallaron": "mallar",
    "mallará": "mallar",
    "mallarán": "mallar",
    "mallarás": "mallar",
    "mallaré": "mallar",
    "mallaréis": "mallar",
    "mallaría": "mallar",
    "mallaríais": "mallar",
    "mallaríamos": "mallar",
    "mallarían": "mallar",
    "mallarías": "mallar",
    "mallas": "mallar",
    "mallase": "mallar",
    "mallaseis": "mallar",
    "mallasen": "mallar",
    "mallases": "mallar",
    "mallaste": "mallar",
    "mallasteis": "mallar",
    "malle": "mallar",
    "mallemos": "mallar",
    "mallen": "mallar",
    "mallera": "mallero",
    "malleras": "mallero",
    "malleros": "mallero",
    "malles": "mallar",
    "malletas": "malleta",
    "malletes": "mallete",
    "malletos": "malleto",
    "mallo": "mallar",
    "mallorquina": "mallorquín",
    "mallorquinas": "mallorquín",
    "mallorquines": "mallorquín",
    "mallorquinistas": "mallorquinista",
    "mallos": "mallo",
    "mallots": "mallot",
    "mallábamos": "mallar",
    "malláis": "mallar",
    "malláramos": "mallar",
    "malláremos": "mallar",
    "mallásemos": "mallar",
    "mallé": "mallar",
    "malléis": "mallar",
    "malló": "mallar",
    "malmandada": "malmandado",
    "malmandadas": "malmandado",
    "malmandados": "malmandado",
    "malmaridadas": "malmaridada",
    "malmes": "malm",
    "malmirada": "malmirado",
    "malmiradas": "malmirado",
    "malmirados": "malmirado",
    "malnacida": "malnacido",
    "malnacidas": "malnacido",
    "malnacidos": "malnacido",
    "malnutriciones": "malnutrición",
    "malnutrida": "malnutrido",
    "malnutridas": "malnutrido",
    "malnutridos": "malnutrido",
    "maloclusiones": "maloclusión",
    "malogra": "malograr",
    "malograba": "malograr",
    "malograbais": "malograr",
    "malograban": "malograr",
    "malograbas": "malograr",
    "malograd": "malograr",
    "malograda": "malograr",
    "malogradas": "malograr",
    "malogrado": "malograr",
    "malogrados": "malograr",
    "malogramientos": "malogramiento",
    "malogramos": "malograr",
    "malogran": "malograr",
    "malogrando": "malograr",
    "malograra": "malograr",
    "malograrais": "malograr",
    "malograran": "malograr",
    "malograras": "malograr",
    "malograre": "malograr",
    "malograreis": "malograr",
    "malograremos": "malograr",
    "malograren": "malograr",
    "malograres": "malograr",
    "malograron": "malograr",
    "malogrará": "malograr",
    "malograrán": "malograr",
    "malograrás": "malograr",
    "malograré": "malograr",
    "malograréis": "malograr",
    "malograría": "malograr",
    "malograríais": "malograr",
    "malograríamos": "malograr",
    "malograrían": "malograr",
    "malograrías": "malograr",
    "malogras": "malograr",
    "malograse": "malograr",
    "malograseis": "malograr",
    "malograsen": "malograr",
    "malograses": "malograr",
    "malograste": "malograr",
    "malograsteis": "malograr",
    "malogre": "malograr",
    "malogremos": "malograr",
    "malogren": "malograr",
    "malogres": "malograr",
    "malogro": "malograr",
    "malogros": "malogro",
    "malográbamos": "malograr",
    "malográis": "malograr",
    "malográramos": "malograr",
    "malográremos": "malograr",
    "malográsemos": "malograr",
    "malogré": "malograr",
    "malogréis": "malograr",
    "malogró": "malograr",
    "malolientes": "maloliente",
    "malonatos": "malonato",
    "maloquea": "maloquear",
    "maloqueaba": "maloquear",
    "maloqueabais": "maloquear",
    "maloqueaban": "maloquear",
    "maloqueabas": "maloquear",
    "maloquead": "maloquear",
    "maloqueada": "maloquear",
    "maloqueadas": "maloquear",
    "maloqueado": "maloquear",
    "maloqueados": "maloquear",
    "maloqueamos": "maloquear",
    "maloquean": "maloquear",
    "maloqueando": "maloquear",
    "maloqueara": "maloquear",
    "maloquearais": "maloquear",
    "maloquearan": "maloquear",
    "maloquearas": "maloquear",
    "maloqueare": "maloquear",
    "maloqueareis": "maloquear",
    "maloquearemos": "maloquear",
    "maloquearen": "maloquear",
    "maloqueares": "maloquear",
    "maloquearon": "maloquear",
    "maloqueará": "maloquear",
    "maloquearán": "maloquear",
    "maloquearás": "maloquear",
    "maloquearé": "maloquear",
    "maloquearéis": "maloquear",
    "maloquearía": "maloquear",
    "maloquearíais": "maloquear",
    "maloquearíamos": "maloquear",
    "maloquearían": "maloquear",
    "maloquearías": "maloquear",
    "maloqueas": "maloquear",
    "maloquease": "maloquear",
    "maloqueaseis": "maloquear",
    "maloqueasen": "maloquear",
    "maloqueases": "maloquear",
    "maloqueaste": "maloquear",
    "maloqueasteis": "maloquear",
    "maloquee": "maloquear",
    "maloqueemos": "maloquear",
    "maloqueen": "maloquear",
    "maloquees": "maloquear",
    "maloqueo": "maloquear",
    "maloqueábamos": "maloquear",
    "maloqueáis": "maloquear",
    "maloqueáramos": "maloquear",
    "maloqueáremos": "maloquear",
    "maloqueásemos": "maloquear",
    "maloqueé": "maloquear",
    "maloqueéis": "maloquear",
    "maloqueó": "maloquear",
    "malos": "malo",
    "malpara": "malparir",
    "malparaba": "malparar",
    "malparabais": "malparar",
    "malparaban": "malparar",
    "malparabas": "malparar",
    "malparad": "malparar",
    "malparada": "malparar",
    "malparadas": "malparar",
    "malparado": "malparar",
    "malparados": "malparar",
    "malparamos": "malparir",
    "malparan": "malparir",
    "malparando": "malparar",
    "malparara": "malparar",
    "malpararais": "malparar",
    "malpararan": "malparar",
    "malpararas": "malparar",
    "malparare": "malparar",
    "malparareis": "malparar",
    "malpararemos": "malparar",
    "malpararen": "malparar",
    "malparares": "malparar",
    "malpararon": "malparar",
    "malparará": "malparar",
    "malpararán": "malparar",
    "malpararás": "malparar",
    "malpararé": "malparar",
    "malpararéis": "malparar",
    "malpararía": "malparar",
    "malpararíais": "malparar",
    "malpararíamos": "malparar",
    "malpararían": "malparar",
    "malpararías": "malparar",
    "malparas": "malparir",
    "malparase": "malparar",
    "malparaseis": "malparar",
    "malparasen": "malparar",
    "malparases": "malparar",
    "malparaste": "malparar",
    "malparasteis": "malparar",
    "malpare": "malparir",
    "malparemos": "malparar",
    "malparen": "malparir",
    "malpares": "malparir",
    "malparid": "malparir",
    "malparida": "malparir",
    "malparidas": "malparir",
    "malparido": "malparir",
    "malparidos": "malparir",
    "malpariendo": "malparir",
    "malpariera": "malparir",
    "malparierais": "malparir",
    "malparieran": "malparir",
    "malparieras": "malparir",
    "malpariere": "malparir",
    "malpariereis": "malparir",
    "malparieren": "malparir",
    "malparieres": "malparir",
    "malparieron": "malparir",
    "malpariese": "malparir",
    "malparieseis": "malparir",
    "malpariesen": "malparir",
    "malparieses": "malparir",
    "malparimos": "malparir",
    "malpariremos": "malparir",
    "malparirá": "malparir",
    "malparirán": "malparir",
    "malparirás": "malparir",
    "malpariré": "malparir",
    "malpariréis": "malparir",
    "malpariría": "malparir",
    "malpariríais": "malparir",
    "malpariríamos": "malparir",
    "malparirían": "malparir",
    "malparirías": "malparir",
    "malpariste": "malparir",
    "malparisteis": "malparir",
    "malpariéramos": "malparir",
    "malpariéremos": "malparir",
    "malpariésemos": "malparir",
    "malparió": "malparir",
    "malparo": "malparir",
    "malpartos": "malparto",
    "malparábamos": "malparar",
    "malparáis": "malparir",
    "malparáramos": "malparar",
    "malparáremos": "malparar",
    "malparásemos": "malparar",
    "malparé": "malparar",
    "malparéis": "malparar",
    "malparí": "malparir",
    "malparía": "malparir",
    "malparíais": "malparir",
    "malparíamos": "malparir",
    "malparían": "malparir",
    "malparías": "malparir",
    "malparís": "malparir",
    "malparó": "malparar",
    "malpensada": "malpensado",
    "malpensadas": "malpensado",
    "malpensados": "malpensado",
    "malposiciones": "malposición",
    "malqueramos": "malquerer",
    "malquered": "malquerer",
    "malqueremos": "malquerer",
    "malquerencias": "malquerencia",
    "malquerida": "malquerer",
    "malqueridas": "malquerida",
    "malquerido": "malquerer",
    "malqueridos": "malquerer",
    "malqueriendo": "malquerer",
    "malquerremos": "malquerer",
    "malquerrá": "malquerer",
    "malquerrán": "malquerer",
    "malquerrás": "malquerer",
    "malquerré": "malquerer",
    "malquerréis": "malquerer",
    "malquerría": "malquerer",
    "malquerríais": "malquerer",
    "malquerríamos": "malquerer",
    "malquerrían": "malquerer",
    "malquerrías": "malquerer",
    "malqueráis": "malquerer",
    "malqueréis": "malquerer",
    "malquería": "malquerer",
    "malqueríais": "malquerer",
    "malqueríamos": "malquerer",
    "malquerían": "malquerer",
    "malquerías": "malquerer",
    "malquiera": "malquerer",
    "malquieran": "malquerer",
    "malquieras": "malquerer",
    "malquiere": "malquerer",
    "malquieren": "malquerer",
    "malquieres": "malquerer",
    "malquiero": "malquerer",
    "malquise": "malquerer",
    "malquisiera": "malquerer",
    "malquisierais": "malquerer",
    "malquisieran": "malquerer",
    "malquisieras": "malquerer",
    "malquisiere": "malquerer",
    "malquisiereis": "malquerer",
    "malquisieren": "malquerer",
    "malquisieres": "malquerer",
    "malquisieron": "malquerer",
    "malquisiese": "malquerer",
    "malquisieseis": "malquerer",
    "malquisiesen": "malquerer",
    "malquisieses": "malquerer",
    "malquisimos": "malquerer",
    "malquisiste": "malquerer",
    "malquisisteis": "malquerer",
    "malquisiéramos": "malquerer",
    "malquisiéremos": "malquerer",
    "malquisiésemos": "malquerer",
    "malquiso": "malquerer",
    "malquista": "malquisto",
    "malquistaba": "malquistar",
    "malquistabais": "malquistar",
    "malquistaban": "malquistar",
    "malquistabas": "malquistar",
    "malquistad": "malquistar",
    "malquistada": "malquistar",
    "malquistadas": "malquistar",
    "malquistado": "malquistar",
    "malquistados": "malquistar",
    "malquistamientos": "malquistamiento",
    "malquistamos": "malquistar",
    "malquistan": "malquistar",
    "malquistando": "malquistar",
    "malquistara": "malquistar",
    "malquistarais": "malquistar",
    "malquistaran": "malquistar",
    "malquistaras": "malquistar",
    "malquistare": "malquistar",
    "malquistareis": "malquistar",
    "malquistaremos": "malquistar",
    "malquistaren": "malquistar",
    "malquistares": "malquistar",
    "malquistaron": "malquistar",
    "malquistará": "malquistar",
    "malquistarán": "malquistar",
    "malquistarás": "malquistar",
    "malquistaré": "malquistar",
    "malquistaréis": "malquistar",
    "malquistaría": "malquistar",
    "malquistaríais": "malquistar",
    "malquistaríamos": "malquistar",
    "malquistarían": "malquistar",
    "malquistarías": "malquistar",
    "malquistas": "malquisto",
    "malquistase": "malquistar",
    "malquistaseis": "malquistar",
    "malquistasen": "malquistar",
    "malquistases": "malquistar",
    "malquistaste": "malquistar",
    "malquistasteis": "malquistar",
    "malquiste": "malquistar",
    "malquistemos": "malquistar",
    "malquisten": "malquistar",
    "malquistes": "malquistar",
    "malquisto": "malquistar",
    "malquistos": "malquisto",
    "malquistábamos": "malquistar",
    "malquistáis": "malquistar",
    "malquistáramos": "malquistar",
    "malquistáremos": "malquistar",
    "malquistásemos": "malquistar",
    "malquisté": "malquistar",
    "malquistéis": "malquistar",
    "malquistó": "malquistar",
    "malsana": "malsano",
    "malsanas": "malsano",
    "malsanos": "malsano",
    "malsonantes": "malsonante",
    "malsufrida": "malsufrido",
    "malsufridas": "malsufrido",
    "malsufridos": "malsufrido",
    "maltas": "malta",
    "maltasas": "maltasa",
    "maltea": "maltear",
    "malteaba": "maltear",
    "malteabais": "maltear",
    "malteaban": "maltear",
    "malteabas": "maltear",
    "maltead": "maltear",
    "malteada": "maltear",
    "malteadas": "maltear",
    "malteado": "maltear",
    "malteados": "maltear",
    "malteamos": "maltear",
    "maltean": "maltear",
    "malteando": "maltear",
    "malteara": "maltear",
    "maltearais": "maltear",
    "maltearan": "maltear",
    "maltearas": "maltear",
    "malteare": "maltear",
    "malteareis": "maltear",
    "maltearemos": "maltear",
    "maltearen": "maltear",
    "malteares": "maltear",
    "maltearon": "maltear",
    "malteará": "maltear",
    "maltearán": "maltear",
    "maltearás": "maltear",
    "maltearé": "maltear",
    "maltearéis": "maltear",
    "maltearía": "maltear",
    "maltearíais": "maltear",
    "maltearíamos": "maltear",
    "maltearían": "maltear",
    "maltearías": "maltear",
    "malteas": "maltear",
    "maltease": "maltear",
    "malteaseis": "maltear",
    "malteasen": "maltear",
    "malteases": "maltear",
    "malteaste": "maltear",
    "malteasteis": "maltear",
    "maltee": "maltear",
    "malteemos": "maltear",
    "malteen": "maltear",
    "maltees": "maltear",
    "malteo": "maltear",
    "malterías": "maltería",
    "maltesa": "maltés",
    "maltesas": "maltés",
    "malteses": "maltés",
    "malteábamos": "maltear",
    "malteáis": "maltear",
    "malteáramos": "maltear",
    "malteáremos": "maltear",
    "malteásemos": "maltear",
    "malteé": "maltear",
    "malteéis": "maltear",
    "malteó": "maltear",
    "malthusianismos": "malthusianismo",
    "maltosas": "maltosa",
    "maltrae": "maltraer",
    "maltraed": "maltraer",
    "maltraemos": "maltraer",
    "maltraen": "maltraer",
    "maltraeremos": "maltraer",
    "maltraerá": "maltraer",
    "maltraerán": "maltraer",
    "maltraerás": "maltraer",
    "maltraeré": "maltraer",
    "maltraeréis": "maltraer",
    "maltraería": "maltraer",
    "maltraeríais": "maltraer",
    "maltraeríamos": "maltraer",
    "maltraerían": "maltraer",
    "maltraerías": "maltraer",
    "maltraes": "maltraer",
    "maltraiga": "maltraer",
    "maltraigamos": "maltraer",
    "maltraigan": "maltraer",
    "maltraigas": "maltraer",
    "maltraigo": "maltraer",
    "maltraigáis": "maltraer",
    "maltraje": "maltraer",
    "maltrajera": "maltraer",
    "maltrajerais": "maltraer",
    "maltrajeran": "maltraer",
    "maltrajeras": "maltraer",
    "maltrajere": "maltraer",
    "maltrajereis": "maltraer",
    "maltrajeren": "maltraer",
    "maltrajeres": "maltraer",
    "maltrajeron": "maltraer",
    "maltrajese": "maltraer",
    "maltrajeseis": "maltraer",
    "maltrajesen": "maltraer",
    "maltrajeses": "maltraer",
    "maltrajimos": "maltraer",
    "maltrajiste": "maltraer",
    "maltrajisteis": "maltraer",
    "maltrajo": "maltraer",
    "maltrajéramos": "maltraer",
    "maltrajéremos": "maltraer",
    "maltrajésemos": "maltraer",
    "maltrata": "maltratar",
    "maltrataba": "maltratar",
    "maltratabais": "maltratar",
    "maltrataban": "maltratar",
    "maltratabas": "maltratar",
    "maltratad": "maltratar",
    "maltratada": "maltratar",
    "maltratadas": "maltratar",
    "maltratado": "maltratar",
    "maltratadora": "maltratador",
    "maltratadoras": "maltratador",
    "maltratadores": "maltratador",
    "maltratados": "maltratar",
    "maltratamos": "maltratar",
    "maltratan": "maltratar",
    "maltratando": "maltratar",
    "maltratara": "maltratar",
    "maltratarais": "maltratar",
    "maltrataran": "maltratar",
    "maltrataras": "maltratar",
    "maltratare": "maltratar",
    "maltratareis": "maltratar",
    "maltrataremos": "maltratar",
    "maltrataren": "maltratar",
    "maltratares": "maltratar",
    "maltrataron": "maltratar",
    "maltratará": "maltratar",
    "maltratarán": "maltratar",
    "maltratarás": "maltratar",
    "maltrataré": "maltratar",
    "maltrataréis": "maltratar",
    "maltrataría": "maltratar",
    "maltrataríais": "maltratar",
    "maltrataríamos": "maltratar",
    "maltratarían": "maltratar",
    "maltratarías": "maltratar",
    "maltratas": "maltratar",
    "maltratase": "maltratar",
    "maltrataseis": "maltratar",
    "maltratasen": "maltratar",
    "maltratases": "maltratar",
    "maltrataste": "maltratar",
    "maltratasteis": "maltratar",
    "maltrate": "maltratar",
    "maltratemos": "maltratar",
    "maltraten": "maltratar",
    "maltrates": "maltratar",
    "maltrato": "maltratar",
    "maltratos": "maltrato",
    "maltratábamos": "maltratar",
    "maltratáis": "maltratar",
    "maltratáramos": "maltratar",
    "maltratáremos": "maltratar",
    "maltratásemos": "maltratar",
    "maltraté": "maltratar",
    "maltratéis": "maltratar",
    "maltrató": "maltratar",
    "maltrayendo": "maltraer",
    "maltraéis": "maltraer",
    "maltraía": "maltraer",
    "maltraíais": "maltraer",
    "maltraíamos": "maltraer",
    "maltraían": "maltraer",
    "maltraías": "maltraer",
    "maltraída": "maltraer",
    "maltraídas": "maltraer",
    "maltraído": "maltraer",
    "maltraídos": "maltraer",
    "maltrecha": "maltrecho",
    "maltrechas": "maltrecho",
    "maltrechos": "maltrecho",
    "maltusiana": "maltusiano",
    "maltusianas": "maltusiano",
    "maltusianismos": "maltusianismo",
    "maltusianos": "maltusiano",
    "maluca": "maluco",
    "malucas": "maluco",
    "malucha": "malucho",
    "maluchas": "malucho",
    "maluchos": "malucho",
    "malucos": "maluco",
    "malvada": "malvado",
    "malvadas": "malvado",
    "malvados": "malvado",
    "malvales": "malval",
    "malvares": "malvar",
    "malvas": "malva",
    "malvasías": "malvasía",
    "malvaviscos": "malvavisco",
    "malvenda": "malvender",
    "malvendamos": "malvender",
    "malvendan": "malvender",
    "malvendas": "malvender",
    "malvende": "malvender",
    "malvended": "malvender",
    "malvendemos": "malvender",
    "malvenden": "malvender",
    "malvenderemos": "malvender",
    "malvenderá": "malvender",
    "malvenderán": "malvender",
    "malvenderás": "malvender",
    "malvenderé": "malvender",
    "malvenderéis": "malvender",
    "malvendería": "malvender",
    "malvenderíais": "malvender",
    "malvenderíamos": "malvender",
    "malvenderían": "malvender",
    "malvenderías": "malvender",
    "malvendes": "malvender",
    "malvendida": "malvender",
    "malvendidas": "malvender",
    "malvendido": "malvender",
    "malvendidos": "malvender",
    "malvendiendo": "malvender",
    "malvendiera": "malvender",
    "malvendierais": "malvender",
    "malvendieran": "malvender",
    "malvendieras": "malvender",
    "malvendiere": "malvender",
    "malvendiereis": "malvender",
    "malvendieren": "malvender",
    "malvendieres": "malvender",
    "malvendieron": "malvender",
    "malvendiese": "malvender",
    "malvendieseis": "malvender",
    "malvendiesen": "malvender",
    "malvendieses": "malvender",
    "malvendimos": "malvender",
    "malvendiste": "malvender",
    "malvendisteis": "malvender",
    "malvendiéramos": "malvender",
    "malvendiéremos": "malvender",
    "malvendiésemos": "malvender",
    "malvendió": "malvender",
    "malvendo": "malvender",
    "malvendáis": "malvender",
    "malvendéis": "malvender",
    "malvendí": "malvender",
    "malvendía": "malvender",
    "malvendíais": "malvender",
    "malvendíamos": "malvender",
    "malvendían": "malvender",
    "malvendías": "malvender",
    "malversa": "malversar",
    "malversaba": "malversar",
    "malversabais": "malversar",
    "malversaban": "malversar",
    "malversabas": "malversar",
    "malversaciones": "malversación",
    "malversad": "malversar",
    "malversada": "malversar",
    "malversadas": "malversar",
    "malversado": "malversar",
    "malversadora": "malversador",
    "malversadoras": "malversador",
    "malversadores": "malversador",
    "malversados": "malversar",
    "malversamos": "malversar",
    "malversan": "malversar",
    "malversando": "malversar",
    "malversara": "malversar",
    "malversarais": "malversar",
    "malversaran": "malversar",
    "malversaras": "malversar",
    "malversare": "malversar",
    "malversareis": "malversar",
    "malversaremos": "malversar",
    "malversaren": "malversar",
    "malversares": "malversar",
    "malversaron": "malversar",
    "malversará": "malversar",
    "malversarán": "malversar",
    "malversarás": "malversar",
    "malversaré": "malversar",
    "malversaréis": "malversar",
    "malversaría": "malversar",
    "malversaríais": "malversar",
    "malversaríamos": "malversar",
    "malversarían": "malversar",
    "malversarías": "malversar",
    "malversas": "malversar",
    "malversase": "malversar",
    "malversaseis": "malversar",
    "malversasen": "malversar",
    "malversases": "malversar",
    "malversaste": "malversar",
    "malversasteis": "malversar",
    "malverse": "malversar",
    "malversemos": "malversar",
    "malversen": "malversar",
    "malverses": "malversar",
    "malverso": "malversar",
    "malversábamos": "malversar",
    "malversáis": "malversar",
    "malversáramos": "malversar",
    "malversáremos": "malversar",
    "malversásemos": "malversar",
    "malversé": "malversar",
    "malverséis": "malversar",
    "malversó": "malversar",
    "malvinenses": "malvinense",
    "malvinera": "malvinero",
    "malvineras": "malvinero",
    "malvineros": "malvinero",
    "malviva": "malvivir",
    "malvivamos": "malvivir",
    "malvivan": "malvivir",
    "malvivas": "malvivir",
    "malvive": "malvivir",
    "malviven": "malvivir",
    "malvives": "malvivir",
    "malvivid": "malvivir",
    "malvivida": "malvivir",
    "malvividas": "malvivir",
    "malvivido": "malvivir",
    "malvividos": "malvivir",
    "malviviendo": "malvivir",
    "malviviera": "malvivir",
    "malvivierais": "malvivir",
    "malvivieran": "malvivir",
    "malvivieras": "malvivir",
    "malviviere": "malvivir",
    "malviviereis": "malvivir",
    "malvivieren": "malvivir",
    "malvivieres": "malvivir",
    "malvivieron": "malvivir",
    "malviviese": "malvivir",
    "malvivieseis": "malvivir",
    "malviviesen": "malvivir",
    "malvivieses": "malvivir",
    "malvivimos": "malvivir",
    "malviviremos": "malvivir",
    "malvivirá": "malvivir",
    "malvivirán": "malvivir",
    "malvivirás": "malvivir",
    "malviviré": "malvivir",
    "malviviréis": "malvivir",
    "malviviría": "malvivir",
    "malviviríais": "malvivir",
    "malviviríamos": "malvivir",
    "malvivirían": "malvivir",
    "malvivirías": "malvivir",
    "malviviste": "malvivir",
    "malvivisteis": "malvivir",
    "malviviéramos": "malvivir",
    "malviviéremos": "malvivir",
    "malviviésemos": "malvivir",
    "malvivió": "malvivir",
    "malvivo": "malvivir",
    "malviváis": "malvivir",
    "malviví": "malvivir",
    "malvivía": "malvivir",
    "malvivíais": "malvivir",
    "malvivíamos": "malvivir",
    "malvivían": "malvivir",
    "malvivías": "malvivir",
    "malvivís": "malvivir",
    "malvácea": "malváceo",
    "malváceas": "malváceo",
    "malváceos": "malváceo",
    "maléfica": "maléfico",
    "maléficas": "maléfico",
    "maléficos": "maléfico",
    "maléolos": "maléolo",
    "malévola": "malévolo",
    "malévolas": "malévolo",
    "malévolos": "malévolo",
    "malónica": "malónico",
    "malónicas": "malónico",
    "malónicos": "malónico",
    "mama": "mamar",
    "mamaba": "mamar",
    "mamabais": "mamar",
    "mamaban": "mamar",
    "mamabas": "mamar",
    "mamaconas": "mamacona",
    "mamad": "mamar",
    "mamada": "mamar",
    "mamadas": "mamar",
    "mamaderas": "mamadera",
    "mamado": "mamar",
    "mamados": "mamar",
    "mamamos": "mamar",
    "maman": "mamar",
    "mamando": "mamar",
    "mamandurrias": "mamandurria",
    "mamara": "mamar",
    "mamarais": "mamar",
    "mamaran": "mamar",
    "mamaras": "mamar",
    "mamare": "mamar",
    "mamareis": "mamar",
    "mamaremos": "mamar",
    "mamaren": "mamar",
    "mamares": "mamar",
    "mamaria": "mamario",
    "mamarias": "mamario",
    "mamarios": "mamario",
    "mamaron": "mamar",
    "mamarrachadas": "mamarrachada",
    "mamarrachos": "mamarracho",
    "mamará": "mamar",
    "mamarán": "mamar",
    "mamarás": "mamar",
    "mamaré": "mamar",
    "mamaréis": "mamar",
    "mamaría": "mamar",
    "mamaríais": "mamar",
    "mamaríamos": "mamar",
    "mamarían": "mamar",
    "mamarías": "mamar",
    "mamas": "mamar",
    "mamase": "mamar",
    "mamaseis": "mamar",
    "mamasen": "mamar",
    "mamases": "mamar",
    "mamaste": "mamar",
    "mamasteis": "mamar",
    "mambises": "mambís",
    "mamblas": "mambla",
    "mambos": "mambo",
    "mambrúes": "mambrú",
    "mame": "mamar",
    "mamellada": "mamellado",
    "mamelladas": "mamellado",
    "mamellados": "mamellado",
    "mamellas": "mamella",
    "mamelonada": "mamelonado",
    "mamelonadas": "mamelonado",
    "mamelonados": "mamelonado",
    "mamelones": "mamelón",
    "mameluca": "mameluco",
    "mamelucas": "mameluco",
    "mamelucos": "mameluco",
    "mamemos": "mamar",
    "mamen": "mamar",
    "mames": "mamar",
    "mameyes": "mamey",
    "mamilares": "mamilar",
    "mamilas": "mamila",
    "mamilotalámica": "mamilotalámico",
    "mamilotalámicas": "mamilotalámico",
    "mamilotalámicos": "mamilotalámico",
    "mamilotegmentales": "mamilotegmental",
    "mamo": "mamar",
    "mamografías": "mamografía",
    "mamolas": "mamola",
    "mamona": "mamón",
    "mamonas": "mamón",
    "mamones": "mamón",
    "mamotretos": "mamotreto",
    "mamparas": "mampara",
    "mamparos": "mamparo",
    "mamperlanes": "mamperlán",
    "mampirlanes": "mampirlán",
    "mamporros": "mamporro",
    "mampostea": "mampostear",
    "mamposteaba": "mampostear",
    "mamposteabais": "mampostear",
    "mamposteaban": "mampostear",
    "mamposteabas": "mampostear",
    "mampostead": "mampostear",
    "mamposteada": "mampostear",
    "mamposteadas": "mampostear",
    "mamposteado": "mampostear",
    "mamposteados": "mampostear",
    "mamposteamos": "mampostear",
    "mampostean": "mampostear",
    "mamposteando": "mampostear",
    "mamposteara": "mampostear",
    "mampostearais": "mampostear",
    "mampostearan": "mampostear",
    "mampostearas": "mampostear",
    "mamposteare": "mampostear",
    "mamposteareis": "mampostear",
    "mampostearemos": "mampostear",
    "mampostearen": "mampostear",
    "mamposteares": "mampostear",
    "mampostearon": "mampostear",
    "mamposteará": "mampostear",
    "mampostearán": "mampostear",
    "mampostearás": "mampostear",
    "mampostearé": "mampostear",
    "mampostearéis": "mampostear",
    "mampostearía": "mampostear",
    "mampostearíais": "mampostear",
    "mampostearíamos": "mampostear",
    "mampostearían": "mampostear",
    "mampostearías": "mampostear",
    "mamposteas": "mampostear",
    "mampostease": "mampostear",
    "mamposteaseis": "mampostear",
    "mamposteasen": "mampostear",
    "mamposteases": "mampostear",
    "mamposteaste": "mampostear",
    "mamposteasteis": "mampostear",
    "mampostee": "mampostear",
    "mamposteemos": "mampostear",
    "mamposteen": "mampostear",
    "mampostees": "mampostear",
    "mamposteo": "mampostear",
    "mampostera": "mampostero",
    "mamposteras": "mampostero",
    "mamposteros": "mampostero",
    "mamposterías": "mampostería",
    "mamposteábamos": "mampostear",
    "mamposteáis": "mampostear",
    "mamposteáramos": "mampostear",
    "mamposteáremos": "mampostear",
    "mamposteásemos": "mampostear",
    "mamposteé": "mampostear",
    "mamposteéis": "mampostear",
    "mamposteó": "mampostear",
    "mampresa": "mampresar",
    "mampresaba": "mampresar",
    "mampresabais": "mampresar",
    "mampresaban": "mampresar",
    "mampresabas": "mampresar",
    "mampresad": "mampresar",
    "mampresada": "mampresar",
    "mampresadas": "mampresar",
    "mampresado": "mampresar",
    "mampresados": "mampresar",
    "mampresamos": "mampresar",
    "mampresan": "mampresar",
    "mampresando": "mampresar",
    "mampresara": "mampresar",
    "mampresarais": "mampresar",
    "mampresaran": "mampresar",
    "mampresaras": "mampresar",
    "mampresare": "mampresar",
    "mampresareis": "mampresar",
    "mampresaremos": "mampresar",
    "mampresaren": "mampresar",
    "mampresares": "mampresar",
    "mampresaron": "mampresar",
    "mampresará": "mampresar",
    "mampresarán": "mampresar",
    "mampresarás": "mampresar",
    "mampresaré": "mampresar",
    "mampresaréis": "mampresar",
    "mampresaría": "mampresar",
    "mampresaríais": "mampresar",
    "mampresaríamos": "mampresar",
    "mampresarían": "mampresar",
    "mampresarías": "mampresar",
    "mampresas": "mampresar",
    "mampresase": "mampresar",
    "mampresaseis": "mampresar",
    "mampresasen": "mampresar",
    "mampresases": "mampresar",
    "mampresaste": "mampresar",
    "mampresasteis": "mampresar",
    "mamprese": "mampresar",
    "mampresemos": "mampresar",
    "mampresen": "mampresar",
    "mampreses": "mampresar",
    "mampreso": "mampresar",
    "mampresábamos": "mampresar",
    "mampresáis": "mampresar",
    "mampresáramos": "mampresar",
    "mampresáremos": "mampresar",
    "mampresásemos": "mampresar",
    "mampresé": "mampresar",
    "mampreséis": "mampresar",
    "mampresó": "mampresar",
    "mampuesta": "mampuesto",
    "mampuestas": "mampuesto",
    "mampuestos": "mampuesto",
    "mamulla": "mamullar",
    "mamullaba": "mamullar",
    "mamullabais": "mamullar",
    "mamullaban": "mamullar",
    "mamullabas": "mamullar",
    "mamullad": "mamullar",
    "mamullada": "mamullar",
    "mamulladas": "mamullar",
    "mamullado": "mamullar",
    "mamullados": "mamullar",
    "mamullamos": "mamullar",
    "mamullan": "mamullar",
    "mamullando": "mamullar",
    "mamullara": "mamullar",
    "mamullarais": "mamullar",
    "mamullaran": "mamullar",
    "mamullaras": "mamullar",
    "mamullare": "mamullar",
    "mamullareis": "mamullar",
    "mamullaremos": "mamullar",
    "mamullaren": "mamullar",
    "mamullares": "mamullar",
    "mamullaron": "mamullar",
    "mamullará": "mamullar",
    "mamullarán": "mamullar",
    "mamullarás": "mamullar",
    "mamullaré": "mamullar",
    "mamullaréis": "mamullar",
    "mamullaría": "mamullar",
    "mamullaríais": "mamullar",
    "mamullaríamos": "mamullar",
    "mamullarían": "mamullar",
    "mamullarías": "mamullar",
    "mamullas": "mamullar",
    "mamullase": "mamullar",
    "mamullaseis": "mamullar",
    "mamullasen": "mamullar",
    "mamullases": "mamullar",
    "mamullaste": "mamullar",
    "mamullasteis": "mamullar",
    "mamulle": "mamullar",
    "mamullemos": "mamullar",
    "mamullen": "mamullar",
    "mamulles": "mamullar",
    "mamullo": "mamullar",
    "mamullábamos": "mamullar",
    "mamulláis": "mamullar",
    "mamulláramos": "mamullar",
    "mamulláremos": "mamullar",
    "mamullásemos": "mamullar",
    "mamullé": "mamullar",
    "mamulléis": "mamullar",
    "mamulló": "mamullar",
    "mamuts": "mamut",
    "mamábamos": "mamar",
    "mamáis": "mamar",
    "mamáramos": "mamar",
    "mamáremos": "mamar",
    "mamás": "mamá",
    "mamásemos": "mamar",
    "mamé": "mamar",
    "maméis": "mamar",
    "mamías": "mamía",
    "mamífera": "mamífero",
    "mamíferas": "mamífero",
    "mamíferos": "mamífero",
    "mamó": "mamar",
    "mana": "manir",
    "manaba": "manar",
    "manabais": "manar",
    "manaban": "manar",
    "manabas": "manar",
    "manad": "manar",
    "manada": "manar",
    "manadas": "manar",
    "manaderos": "manadero",
    "manado": "manar",
    "manados": "manar",
    "managements": "management",
    "managers": "manager",
    "manamos": "manir",
    "manan": "manir",
    "manando": "manar",
    "mananos": "manano",
    "manantiales": "manantial",
    "manantía": "manantío",
    "manantías": "manantío",
    "manantíos": "manantío",
    "manara": "manar",
    "manarais": "manar",
    "manaran": "manar",
    "manaras": "manar",
    "manare": "manar",
    "manareis": "manar",
    "manaremos": "manar",
    "manaren": "manar",
    "manares": "manar",
    "manaron": "manar",
    "manará": "manar",
    "manarán": "manar",
    "manarás": "manar",
    "manaré": "manar",
    "manaréis": "manar",
    "manaría": "manar",
    "manaríais": "manar",
    "manaríamos": "manar",
    "manarían": "manar",
    "manarías": "manar",
    "manas": "manir",
    "manase": "manar",
    "manaseis": "manar",
    "manasen": "manar",
    "manases": "manar",
    "manaste": "manar",
    "manasteis": "manar",
    "manatos": "manato",
    "manats": "manat",
    "manatíes": "manatí",
    "manca": "manco",
    "mancaba": "mancar",
    "mancabais": "mancar",
    "mancaban": "mancar",
    "mancabas": "mancar",
    "mancad": "mancar",
    "mancada": "mancar",
    "mancadas": "mancar",
    "mancado": "mancar",
    "mancados": "mancar",
    "mancamos": "mancar",
    "mancan": "mancar",
    "mancando": "mancar",
    "mancara": "mancar",
    "mancarais": "mancar",
    "mancaran": "mancar",
    "mancaras": "mancar",
    "mancare": "mancar",
    "mancareis": "mancar",
    "mancaremos": "mancar",
    "mancaren": "mancar",
    "mancares": "mancar",
    "mancaron": "mancar",
    "mancarrona": "mancarrón",
    "mancarronas": "mancarrón",
    "mancarrones": "mancarrón",
    "mancará": "mancar",
    "mancarán": "mancar",
    "mancarás": "mancar",
    "mancaré": "mancar",
    "mancaréis": "mancar",
    "mancaría": "mancar",
    "mancaríais": "mancar",
    "mancaríamos": "mancar",
    "mancarían": "mancar",
    "mancarías": "mancar",
    "mancas": "manco",
    "mancase": "mancar",
    "mancaseis": "mancar",
    "mancasen": "mancar",
    "mancases": "mancar",
    "mancaste": "mancar",
    "mancasteis": "mancar",
    "manceba": "mancebo",
    "mancebas": "mancebo",
    "mancebos": "mancebo",
    "mancebías": "mancebía",
    "manceras": "mancera",
    "mancerinas": "mancerina",
    "mancha": "manchar",
    "manchaba": "manchar",
    "manchabais": "manchar",
    "manchaban": "manchar",
    "manchabas": "manchar",
    "manchad": "manchar",
    "manchada": "manchar",
    "manchadas": "manchar",
    "manchadiza": "manchadizo",
    "manchadizas": "manchadizo",
    "manchadizos": "manchadizo",
    "manchado": "manchar",
    "manchados": "manchar",
    "manchamos": "manchar",
    "manchan": "manchar",
    "manchando": "manchar",
    "manchara": "manchar",
    "mancharais": "manchar",
    "mancharan": "manchar",
    "mancharas": "manchar",
    "manchare": "manchar",
    "manchareis": "manchar",
    "mancharemos": "manchar",
    "mancharen": "manchar",
    "manchares": "manchar",
    "mancharon": "manchar",
    "manchará": "manchar",
    "mancharán": "manchar",
    "mancharás": "manchar",
    "mancharé": "manchar",
    "mancharéis": "manchar",
    "mancharía": "manchar",
    "mancharíais": "manchar",
    "mancharíamos": "manchar",
    "mancharían": "manchar",
    "mancharías": "manchar",
    "manchas": "manchar",
    "manchase": "manchar",
    "manchaseis": "manchar",
    "manchasen": "manchar",
    "manchases": "manchar",
    "manchaste": "manchar",
    "manchasteis": "manchar",
    "manche": "manchar",
    "manchega": "manchego",
    "manchegas": "manchego",
    "manchegos": "manchego",
    "manchemos": "manchar",
    "manchen": "manchar",
    "manches": "manchar",
    "manchetas": "mancheta",
    "mancho": "manchar",
    "manchones": "manchón",
    "manchosa": "manchoso",
    "manchosas": "manchoso",
    "manchosos": "manchoso",
    "manchábamos": "manchar",
    "mancháis": "manchar",
    "mancháramos": "manchar",
    "mancháremos": "manchar",
    "manchásemos": "manchar",
    "manché": "manchar",
    "manchéis": "manchar",
    "manchó": "manchar",
    "manchúes": "manchú",
    "manchús": "manchú",
    "manchúses": "manchú",
    "mancilla": "mancillar",
    "mancillaba": "mancillar",
    "mancillabais": "mancillar",
    "mancillaban": "mancillar",
    "mancillabas": "mancillar",
    "mancillad": "mancillar",
    "mancillada": "mancillar",
    "mancilladas": "mancillar",
    "mancillado": "mancillar",
    "mancillados": "mancillar",
    "mancillamos": "mancillar",
    "mancillan": "mancillar",
    "mancillando": "mancillar",
    "mancillara": "mancillar",
    "mancillarais": "mancillar",
    "mancillaran": "mancillar",
    "mancillaras": "mancillar",
    "mancillare": "mancillar",
    "mancillareis": "mancillar",
    "mancillaremos": "mancillar",
    "mancillaren": "mancillar",
    "mancillares": "mancillar",
    "mancillaron": "mancillar",
    "mancillará": "mancillar",
    "mancillarán": "mancillar",
    "mancillarás": "mancillar",
    "mancillaré": "mancillar",
    "mancillaréis": "mancillar",
    "mancillaría": "mancillar",
    "mancillaríais": "mancillar",
    "mancillaríamos": "mancillar",
    "mancillarían": "mancillar",
    "mancillarías": "mancillar",
    "mancillas": "mancillar",
    "mancillase": "mancillar",
    "mancillaseis": "mancillar",
    "mancillasen": "mancillar",
    "mancillases": "mancillar",
    "mancillaste": "mancillar",
    "mancillasteis": "mancillar",
    "mancille": "mancillar",
    "mancillemos": "mancillar",
    "mancillen": "mancillar",
    "mancilles": "mancillar",
    "mancillo": "mancillar",
    "mancillábamos": "mancillar",
    "mancilláis": "mancillar",
    "mancilláramos": "mancillar",
    "mancilláremos": "mancillar",
    "mancillásemos": "mancillar",
    "mancillé": "mancillar",
    "mancilléis": "mancillar",
    "mancilló": "mancillar",
    "mancipa": "mancipar",
    "mancipaba": "mancipar",
    "mancipabais": "mancipar",
    "mancipaban": "mancipar",
    "mancipabas": "mancipar",
    "mancipaciones": "mancipación",
    "mancipad": "mancipar",
    "mancipada": "mancipar",
    "mancipadas": "mancipar",
    "mancipado": "mancipar",
    "mancipados": "mancipar",
    "mancipamos": "mancipar",
    "mancipan": "mancipar",
    "mancipando": "mancipar",
    "mancipara": "mancipar",
    "manciparais": "mancipar",
    "manciparan": "mancipar",
    "manciparas": "mancipar",
    "mancipare": "mancipar",
    "mancipareis": "mancipar",
    "manciparemos": "mancipar",
    "manciparen": "mancipar",
    "mancipares": "mancipar",
    "manciparon": "mancipar",
    "mancipará": "mancipar",
    "manciparán": "mancipar",
    "manciparás": "mancipar",
    "manciparé": "mancipar",
    "manciparéis": "mancipar",
    "manciparía": "mancipar",
    "manciparíais": "mancipar",
    "manciparíamos": "mancipar",
    "manciparían": "mancipar",
    "manciparías": "mancipar",
    "mancipas": "mancipar",
    "mancipase": "mancipar",
    "mancipaseis": "mancipar",
    "mancipasen": "mancipar",
    "mancipases": "mancipar",
    "mancipaste": "mancipar",
    "mancipasteis": "mancipar",
    "mancipe": "mancipar",
    "mancipemos": "mancipar",
    "mancipen": "mancipar",
    "mancipes": "mancipar",
    "mancipo": "mancipar",
    "mancipábamos": "mancipar",
    "mancipáis": "mancipar",
    "mancipáramos": "mancipar",
    "mancipáremos": "mancipar",
    "mancipásemos": "mancipar",
    "mancipé": "mancipar",
    "mancipéis": "mancipar",
    "mancipó": "mancipar",
    "manco": "mancar",
    "mancomuna": "mancomunar",
    "mancomunaba": "mancomunar",
    "mancomunabais": "mancomunar",
    "mancomunaban": "mancomunar",
    "mancomunabas": "mancomunar",
    "mancomunad": "mancomunar",
    "mancomunada": "mancomunar",
    "mancomunadas": "mancomunar",
    "mancomunado": "mancomunar",
    "mancomunados": "mancomunar",
    "mancomunamos": "mancomunar",
    "mancomunan": "mancomunar",
    "mancomunando": "mancomunar",
    "mancomunara": "mancomunar",
    "mancomunarais": "mancomunar",
    "mancomunaran": "mancomunar",
    "mancomunaras": "mancomunar",
    "mancomunare": "mancomunar",
    "mancomunareis": "mancomunar",
    "mancomunaremos": "mancomunar",
    "mancomunaren": "mancomunar",
    "mancomunares": "mancomunar",
    "mancomunaron": "mancomunar",
    "mancomunará": "mancomunar",
    "mancomunarán": "mancomunar",
    "mancomunarás": "mancomunar",
    "mancomunaré": "mancomunar",
    "mancomunaréis": "mancomunar",
    "mancomunaría": "mancomunar",
    "mancomunaríais": "mancomunar",
    "mancomunaríamos": "mancomunar",
    "mancomunarían": "mancomunar",
    "mancomunarías": "mancomunar",
    "mancomunas": "mancomunar",
    "mancomunase": "mancomunar",
    "mancomunaseis": "mancomunar",
    "mancomunasen": "mancomunar",
    "mancomunases": "mancomunar",
    "mancomunaste": "mancomunar",
    "mancomunasteis": "mancomunar",
    "mancomune": "mancomunar",
    "mancomunemos": "mancomunar",
    "mancomunen": "mancomunar",
    "mancomunes": "mancomunar",
    "mancomunidades": "mancomunidad",
    "mancomuno": "mancomunar",
    "mancomunábamos": "mancomunar",
    "mancomunáis": "mancomunar",
    "mancomunáramos": "mancomunar",
    "mancomunáremos": "mancomunar",
    "mancomunásemos": "mancomunar",
    "mancomuné": "mancomunar",
    "mancomunéis": "mancomunar",
    "mancomunó": "mancomunar",
    "mancornaba": "mancornar",
    "mancornabais": "mancornar",
    "mancornaban": "mancornar",
    "mancornabas": "mancornar",
    "mancornad": "mancornar",
    "mancornada": "mancornar",
    "mancornadas": "mancornar",
    "mancornado": "mancornar",
    "mancornados": "mancornar",
    "mancornamos": "mancornar",
    "mancornando": "mancornar",
    "mancornara": "mancornar",
    "mancornarais": "mancornar",
    "mancornaran": "mancornar",
    "mancornaras": "mancornar",
    "mancornare": "mancornar",
    "mancornareis": "mancornar",
    "mancornaremos": "mancornar",
    "mancornaren": "mancornar",
    "mancornares": "mancornar",
    "mancornaron": "mancornar",
    "mancornará": "mancornar",
    "mancornarán": "mancornar",
    "mancornarás": "mancornar",
    "mancornaré": "mancornar",
    "mancornaréis": "mancornar",
    "mancornaría": "mancornar",
    "mancornaríais": "mancornar",
    "mancornaríamos": "mancornar",
    "mancornarían": "mancornar",
    "mancornarías": "mancornar",
    "mancornase": "mancornar",
    "mancornaseis": "mancornar",
    "mancornasen": "mancornar",
    "mancornases": "mancornar",
    "mancornaste": "mancornar",
    "mancornasteis": "mancornar",
    "mancornemos": "mancornar",
    "mancornábamos": "mancornar",
    "mancornáis": "mancornar",
    "mancornáramos": "mancornar",
    "mancornáremos": "mancornar",
    "mancornásemos": "mancornar",
    "mancorné": "mancornar",
    "mancornéis": "mancornar",
    "mancornó": "mancornar",
    "mancos": "manco",
    "mancuerdas": "mancuerda",
    "mancuerna": "mancornar",
    "mancuernan": "mancornar",
    "mancuernas": "mancuerna",
    "mancuerne": "mancornar",
    "mancuernen": "mancornar",
    "mancuernes": "mancornar",
    "mancuerno": "mancornar",
    "mancábamos": "mancar",
    "mancáis": "mancar",
    "mancáramos": "mancar",
    "mancáremos": "mancar",
    "mancásemos": "mancar",
    "mancó": "mancar",
    "manda": "mandar",
    "mandaba": "mandar",
    "mandabais": "mandar",
    "mandaban": "mandar",
    "mandabas": "mandar",
    "mandad": "mandar",
    "mandada": "mandar",
    "mandadas": "mandar",
    "mandadera": "mandadero",
    "mandaderas": "mandadero",
    "mandaderos": "mandadero",
    "mandado": "mandar",
    "mandados": "mandar",
    "mandamases": "mandamás",
    "mandamientos": "mandamiento",
    "mandamos": "mandar",
    "mandan": "mandar",
    "mandando": "mandar",
    "mandangas": "mandanga",
    "mandantes": "mandante",
    "mandara": "mandar",
    "mandarais": "mandar",
    "mandaran": "mandar",
    "mandaras": "mandar",
    "mandare": "mandar",
    "mandareis": "mandar",
    "mandaremos": "mandar",
    "mandaren": "mandar",
    "mandares": "mandar",
    "mandarina": "mandarín",
    "mandarinas": "mandarina",
    "mandarinatos": "mandarinato",
    "mandarineros": "mandarinero",
    "mandarines": "mandarín",
    "mandarinismos": "mandarinismo",
    "mandarinos": "mandarino",
    "mandaron": "mandar",
    "mandarrias": "mandarria",
    "mandará": "mandar",
    "mandarán": "mandar",
    "mandarás": "mandar",
    "mandaré": "mandar",
    "mandaréis": "mandar",
    "mandaría": "mandar",
    "mandaríais": "mandar",
    "mandaríamos": "mandar",
    "mandarían": "mandar",
    "mandarías": "mandar",
    "mandas": "mandar",
    "mandase": "mandar",
    "mandaseis": "mandar",
    "mandasen": "mandar",
    "mandases": "mandar",
    "mandaste": "mandar",
    "mandasteis": "mandar",
    "mandataria": "mandatario",
    "mandatarias": "mandatario",
    "mandatarios": "mandatario",
    "mandatos": "mandato",
    "mande": "mandar",
    "mandemos": "mandar",
    "manden": "mandar",
    "mandes": "mandar",
    "mandeísmos": "mandeísmo",
    "mandibulares": "mandibular",
    "mandibulofaciales": "mandibulofacial",
    "mandila": "mandilar",
    "mandilaba": "mandilar",
    "mandilabais": "mandilar",
    "mandilaban": "mandilar",
    "mandilabas": "mandilar",
    "mandilad": "mandilar",
    "mandilada": "mandilar",
    "mandiladas": "mandilar",
    "mandilado": "mandilar",
    "mandilados": "mandilar",
    "mandilamos": "mandilar",
    "mandilan": "mandilar",
    "mandilando": "mandilar",
    "mandilara": "mandilar",
    "mandilarais": "mandilar",
    "mandilaran": "mandilar",
    "mandilaras": "mandilar",
    "mandilare": "mandilar",
    "mandilareis": "mandilar",
    "mandilaremos": "mandilar",
    "mandilaren": "mandilar",
    "mandilares": "mandilar",
    "mandilaron": "mandilar",
    "mandilará": "mandilar",
    "mandilarán": "mandilar",
    "mandilarás": "mandilar",
    "mandilaré": "mandilar",
    "mandilaréis": "mandilar",
    "mandilaría": "mandilar",
    "mandilaríais": "mandilar",
    "mandilaríamos": "mandilar",
    "mandilarían": "mandilar",
    "mandilarías": "mandilar",
    "mandilas": "mandilar",
    "mandilase": "mandilar",
    "mandilaseis": "mandilar",
    "mandilasen": "mandilar",
    "mandilases": "mandilar",
    "mandilaste": "mandilar",
    "mandilasteis": "mandilar",
    "mandile": "mandilar",
    "mandilemos": "mandilar",
    "mandilen": "mandilar",
    "mandiles": "mandilar",
    "mandiletes": "mandilete",
    "mandilo": "mandilar",
    "mandilones": "mandilón",
    "mandilábamos": "mandilar",
    "mandiláis": "mandilar",
    "mandiláramos": "mandilar",
    "mandiláremos": "mandilar",
    "mandilásemos": "mandilar",
    "mandilé": "mandilar",
    "mandiléis": "mandilar",
    "mandiló": "mandilar",
    "mandinga": "mandingo",
    "mandingas": "mandingo",
    "mandingos": "mandingo",
    "mandiocas": "mandioca",
    "mando": "mandar",
    "mandobles": "mandoble",
    "mandolinas": "mandolina",
    "mandona": "mandón",
    "mandonas": "mandón",
    "mandones": "mandón",
    "mandorlas": "mandorla",
    "mandos": "mando",
    "mandraches": "mandrache",
    "mandrachos": "mandracho",
    "mandrias": "mandria",
    "mandrila": "mandrilar",
    "mandrilaba": "mandrilar",
    "mandrilabais": "mandrilar",
    "mandrilaban": "mandrilar",
    "mandrilabas": "mandrilar",
    "mandrilad": "mandrilar",
    "mandrilada": "mandrilar",
    "mandriladas": "mandrilar",
    "mandrilado": "mandrilar",
    "mandriladoras": "mandriladora",
    "mandrilados": "mandrilar",
    "mandrilamos": "mandrilar",
    "mandrilan": "mandrilar",
    "mandrilando": "mandrilar",
    "mandrilara": "mandrilar",
    "mandrilarais": "mandrilar",
    "mandrilaran": "mandrilar",
    "mandrilaras": "mandrilar",
    "mandrilare": "mandrilar",
    "mandrilareis": "mandrilar",
    "mandrilaremos": "mandrilar",
    "mandrilaren": "mandrilar",
    "mandrilares": "mandrilar",
    "mandrilaron": "mandrilar",
    "mandrilará": "mandrilar",
    "mandrilarán": "mandrilar",
    "mandrilarás": "mandrilar",
    "mandrilaré": "mandrilar",
    "mandrilaréis": "mandrilar",
    "mandrilaría": "mandrilar",
    "mandrilaríais": "mandrilar",
    "mandrilaríamos": "mandrilar",
    "mandrilarían": "mandrilar",
    "mandrilarías": "mandrilar",
    "mandrilas": "mandrilar",
    "mandrilase": "mandrilar",
    "mandrilaseis": "mandrilar",
    "mandrilasen": "mandrilar",
    "mandrilases": "mandrilar",
    "mandrilaste": "mandrilar",
    "mandrilasteis": "mandrilar",
    "mandrile": "mandrilar",
    "mandrilemos": "mandrilar",
    "mandrilen": "mandrilar",
    "mandriles": "mandrilar",
    "mandrilo": "mandrilar",
    "mandrilábamos": "mandrilar",
    "mandriláis": "mandrilar",
    "mandriláramos": "mandrilar",
    "mandriláremos": "mandrilar",
    "mandrilásemos": "mandrilar",
    "mandrilé": "mandrilar",
    "mandriléis": "mandrilar",
    "mandriló": "mandrilar",
    "mandrinadoras": "mandrinadora",
    "mandrágoras": "mandrágora",
    "manduca": "manducar",
    "manducaba": "manducar",
    "manducabais": "manducar",
    "manducaban": "manducar",
    "manducabas": "manducar",
    "manducaciones": "manducación",
    "manducad": "manducar",
    "manducada": "manducar",
    "manducadas": "manducar",
    "manducado": "manducar",
    "manducados": "manducar",
    "manducamos": "manducar",
    "manducan": "manducar",
    "manducando": "manducar",
    "manducara": "manducar",
    "manducarais": "manducar",
    "manducaran": "manducar",
    "manducaras": "manducar",
    "manducare": "manducar",
    "manducareis": "manducar",
    "manducaremos": "manducar",
    "manducaren": "manducar",
    "manducares": "manducar",
    "manducaron": "manducar",
    "manducará": "manducar",
    "manducarán": "manducar",
    "manducarás": "manducar",
    "manducaré": "manducar",
    "manducaréis": "manducar",
    "manducaría": "manducar",
    "manducaríais": "manducar",
    "manducaríamos": "manducar",
    "manducarían": "manducar",
    "manducarías": "manducar",
    "manducas": "manducar",
    "manducase": "manducar",
    "manducaseis": "manducar",
    "manducasen": "manducar",
    "manducases": "manducar",
    "manducaste": "manducar",
    "manducasteis": "manducar",
    "manducatorias": "manducatoria",
    "manduco": "manducar",
    "manducábamos": "manducar",
    "manducáis": "manducar",
    "manducáramos": "manducar",
    "manducáremos": "manducar",
    "manducásemos": "manducar",
    "manducó": "manducar",
    "manduque": "manducar",
    "manduquemos": "manducar",
    "manduquen": "manducar",
    "manduques": "manducar",
    "manduqué": "manducar",
    "manduquéis": "manducar",
    "mandábamos": "mandar",
    "mandáis": "mandar",
    "mandáramos": "mandar",
    "mandáremos": "mandar",
    "mandásemos": "mandar",
    "mandé": "mandar",
    "mandéis": "mandar",
    "mandélica": "mandélico",
    "mandélicas": "mandélico",
    "mandélicos": "mandélico",
    "mandíbulas": "mandíbula",
    "mandó": "mandar",
    "mane": "manir",
    "manea": "manear",
    "maneaba": "manear",
    "maneabais": "manear",
    "maneaban": "manear",
    "maneabas": "manear",
    "manead": "manear",
    "maneada": "manear",
    "maneadas": "manear",
    "maneado": "manear",
    "maneados": "manear",
    "maneamos": "manear",
    "manean": "manear",
    "maneando": "manear",
    "maneara": "manear",
    "manearais": "manear",
    "manearan": "manear",
    "manearas": "manear",
    "maneare": "manear",
    "maneareis": "manear",
    "manearemos": "manear",
    "manearen": "manear",
    "maneares": "manear",
    "manearon": "manear",
    "maneará": "manear",
    "manearán": "manear",
    "manearás": "manear",
    "manearé": "manear",
    "manearéis": "manear",
    "manearía": "manear",
    "manearíais": "manear",
    "manearíamos": "manear",
    "manearían": "manear",
    "manearías": "manear",
    "maneas": "manear",
    "manease": "manear",
    "maneaseis": "manear",
    "maneasen": "manear",
    "maneases": "manear",
    "maneaste": "manear",
    "maneasteis": "manear",
    "manecillas": "manecilla",
    "manee": "manear",
    "maneemos": "manear",
    "maneen": "manear",
    "manees": "manear",
    "maneja": "manejar",
    "manejaba": "manejar",
    "manejabais": "manejar",
    "manejaban": "manejar",
    "manejabas": "manejar",
    "manejabilidades": "manejabilidad",
    "manejables": "manejable",
    "manejad": "manejar",
    "manejada": "manejar",
    "manejadas": "manejar",
    "manejado": "manejar",
    "manejados": "manejar",
    "manejamos": "manejar",
    "manejan": "manejar",
    "manejando": "manejar",
    "manejara": "manejar",
    "manejarais": "manejar",
    "manejaran": "manejar",
    "manejaras": "manejar",
    "manejare": "manejar",
    "manejareis": "manejar",
    "manejaremos": "manejar",
    "manejaren": "manejar",
    "manejares": "manejar",
    "manejaron": "manejar",
    "manejará": "manejar",
    "manejarán": "manejar",
    "manejarás": "manejar",
    "manejaré": "manejar",
    "manejaréis": "manejar",
    "manejaría": "manejar",
    "manejaríais": "manejar",
    "manejaríamos": "manejar",
    "manejarían": "manejar",
    "manejarías": "manejar",
    "manejas": "manejar",
    "manejase": "manejar",
    "manejaseis": "manejar",
    "manejasen": "manejar",
    "manejases": "manejar",
    "manejaste": "manejar",
    "manejasteis": "manejar",
    "maneje": "manejar",
    "manejemos": "manejar",
    "manejen": "manejar",
    "manejes": "manejar",
    "manejo": "manejar",
    "manejos": "manejo",
    "manejábamos": "manejar",
    "manejáis": "manejar",
    "manejáramos": "manejar",
    "manejáremos": "manejar",
    "manejásemos": "manejar",
    "manejé": "manejar",
    "manejéis": "manejar",
    "manejó": "manejar",
    "manemos": "manar",
    "manen": "manir",
    "maneo": "manear",
    "maneras": "manera",
    "manes": "manir",
    "manetas": "maneta",
    "manezuelas": "manezuela",
    "maneábamos": "manear",
    "maneáis": "manear",
    "maneáramos": "manear",
    "maneáremos": "manear",
    "maneásemos": "manear",
    "maneé": "manear",
    "maneéis": "manear",
    "maneó": "manear",
    "manga": "mangar",
    "mangaba": "mangar",
    "mangabais": "mangar",
    "mangaban": "mangar",
    "mangabas": "mangar",
    "mangad": "mangar",
    "mangada": "mangar",
    "mangadas": "mangar",
    "mangado": "mangar",
    "mangados": "mangar",
    "mangajarros": "mangajarro",
    "mangamos": "mangar",
    "mangan": "mangar",
    "manganas": "mangana",
    "manganatos": "manganato",
    "mangancias": "mangancia",
    "mangando": "mangar",
    "manganea": "manganear",
    "manganeaba": "manganear",
    "manganeabais": "manganear",
    "manganeaban": "manganear",
    "manganeabas": "manganear",
    "manganead": "manganear",
    "manganeada": "manganear",
    "manganeadas": "manganear",
    "manganeado": "manganear",
    "manganeados": "manganear",
    "manganeamos": "manganear",
    "manganean": "manganear",
    "manganeando": "manganear",
    "manganeara": "manganear",
    "manganearais": "manganear",
    "manganearan": "manganear",
    "manganearas": "manganear",
    "manganeare": "manganear",
    "manganeareis": "manganear",
    "manganearemos": "manganear",
    "manganearen": "manganear",
    "manganeares": "manganear",
    "manganearon": "manganear",
    "manganeará": "manganear",
    "manganearán": "manganear",
    "manganearás": "manganear",
    "manganearé": "manganear",
    "manganearéis": "manganear",
    "manganearía": "manganear",
    "manganearíais": "manganear",
    "manganearíamos": "manganear",
    "manganearían": "manganear",
    "manganearías": "manganear",
    "manganeas": "manganear",
    "manganease": "manganear",
    "manganeaseis": "manganear",
    "manganeasen": "manganear",
    "manganeases": "manganear",
    "manganeaste": "manganear",
    "manganeasteis": "manganear",
    "manganee": "manganear",
    "manganeemos": "manganear",
    "manganeen": "manganear",
    "manganees": "manganear",
    "manganeo": "manganear",
    "manganeos": "manganeo",
    "manganesos": "manganeso",
    "manganeábamos": "manganear",
    "manganeáis": "manganear",
    "manganeáramos": "manganear",
    "manganeáremos": "manganear",
    "manganeásemos": "manganear",
    "manganeé": "manganear",
    "manganeéis": "manganear",
    "manganeó": "manganear",
    "manganinas": "manganina",
    "manganismos": "manganismo",
    "manganitas": "manganita",
    "manganosa": "manganoso",
    "manganosas": "manganoso",
    "manganosos": "manganoso",
    "mangantes": "mangante",
    "manganífera": "manganífero",
    "manganíferas": "manganífero",
    "manganíferos": "manganífero",
    "mangara": "mangar",
    "mangarais": "mangar",
    "mangaran": "mangar",
    "mangaras": "mangar",
    "mangare": "mangar",
    "mangareis": "mangar",
    "mangaremos": "mangar",
    "mangaren": "mangar",
    "mangares": "mangar",
    "mangaron": "mangar",
    "mangará": "mangar",
    "mangarán": "mangar",
    "mangarás": "mangar",
    "mangaré": "mangar",
    "mangaréis": "mangar",
    "mangaría": "mangar",
    "mangaríais": "mangar",
    "mangaríamos": "mangar",
    "mangarían": "mangar",
    "mangarías": "mangar",
    "mangas": "mangar",
    "mangase": "mangar",
    "mangaseis": "mangar",
    "mangasen": "mangar",
    "mangases": "mangar",
    "mangaste": "mangar",
    "mangasteis": "mangar",
    "manglares": "manglar",
    "mangles": "mangle",
    "mango": "mangar",
    "mangona": "mangón",
    "mangonas": "mangón",
    "mangonea": "mangonear",
    "mangoneaba": "mangonear",
    "mangoneabais": "mangonear",
    "mangoneaban": "mangonear",
    "mangoneabas": "mangonear",
    "mangonead": "mangonear",
    "mangoneada": "mangonear",
    "mangoneadas": "mangonear",
    "mangoneado": "mangonear",
    "mangoneadora": "mangoneador",
    "mangoneadoras": "mangoneador",
    "mangoneadores": "mangoneador",
    "mangoneados": "mangonear",
    "mangoneamos": "mangonear",
    "mangonean": "mangonear",
    "mangoneando": "mangonear",
    "mangoneara": "mangonear",
    "mangonearais": "mangonear",
    "mangonearan": "mangonear",
    "mangonearas": "mangonear",
    "mangoneare": "mangonear",
    "mangoneareis": "mangonear",
    "mangonearemos": "mangonear",
    "mangonearen": "mangonear",
    "mangoneares": "mangonear",
    "mangonearon": "mangonear",
    "mangoneará": "mangonear",
    "mangonearán": "mangonear",
    "mangonearás": "mangonear",
    "mangonearé": "mangonear",
    "mangonearéis": "mangonear",
    "mangonearía": "mangonear",
    "mangonearíais": "mangonear",
    "mangonearíamos": "mangonear",
    "mangonearían": "mangonear",
    "mangonearías": "mangonear",
    "mangoneas": "mangonear",
    "mangonease": "mangonear",
    "mangoneaseis": "mangonear",
    "mangoneasen": "mangonear",
    "mangoneases": "mangonear",
    "mangoneaste": "mangonear",
    "mangoneasteis": "mangonear",
    "mangonee": "mangonear",
    "mangoneemos": "mangonear",
    "mangoneen": "mangonear",
    "mangonees": "mangonear",
    "mangoneo": "mangonear",
    "mangoneos": "mangoneo",
    "mangonera": "mangonero",
    "mangoneras": "mangonero",
    "mangoneros": "mangonero",
    "mangones": "mangón",
    "mangoneábamos": "mangonear",
    "mangoneáis": "mangonear",
    "mangoneáramos": "mangonear",
    "mangoneáremos": "mangonear",
    "mangoneásemos": "mangonear",
    "mangoneé": "mangonear",
    "mangoneéis": "mangonear",
    "mangoneó": "mangonear",
    "mangorrillos": "mangorrillo",
    "mangos": "mango",
    "mangostanes": "mangostán",
    "mangostas": "mangosta",
    "mangotes": "mangote",
    "manguales": "mangual",
    "manguardias": "manguardia",
    "mangue": "mangar",
    "manguea": "manguear",
    "mangueaba": "manguear",
    "mangueabais": "manguear",
    "mangueaban": "manguear",
    "mangueabas": "manguear",
    "manguead": "manguear",
    "mangueada": "manguear",
    "mangueadas": "manguear",
    "mangueado": "manguear",
    "mangueados": "manguear",
    "mangueamos": "manguear",
    "manguean": "manguear",
    "mangueando": "manguear",
    "mangueara": "manguear",
    "manguearais": "manguear",
    "manguearan": "manguear",
    "manguearas": "manguear",
    "mangueare": "manguear",
    "mangueareis": "manguear",
    "manguearemos": "manguear",
    "manguearen": "manguear",
    "mangueares": "manguear",
    "manguearon": "manguear",
    "mangueará": "manguear",
    "manguearán": "manguear",
    "manguearás": "manguear",
    "manguearé": "manguear",
    "manguearéis": "manguear",
    "manguearía": "manguear",
    "manguearíais": "manguear",
    "manguearíamos": "manguear",
    "manguearían": "manguear",
    "manguearías": "manguear",
    "mangueas": "manguear",
    "manguease": "manguear",
    "mangueaseis": "manguear",
    "mangueasen": "manguear",
    "mangueases": "manguear",
    "mangueaste": "manguear",
    "mangueasteis": "manguear",
    "manguee": "manguear",
    "mangueemos": "manguear",
    "mangueen": "manguear",
    "manguees": "manguear",
    "manguemos": "mangar",
    "manguen": "mangar",
    "mangueo": "manguear",
    "mangueras": "manguera",
    "mangueros": "manguero",
    "manguerotes": "manguerote",
    "mangues": "mangar",
    "manguetas": "mangueta",
    "mangueábamos": "manguear",
    "mangueáis": "manguear",
    "mangueáramos": "manguear",
    "mangueáremos": "manguear",
    "mangueásemos": "manguear",
    "mangueé": "manguear",
    "mangueéis": "manguear",
    "mangueó": "manguear",
    "manguitos": "manguito",
    "mangué": "mangar",
    "manguéis": "mangar",
    "mangábamos": "mangar",
    "mangáis": "mangar",
    "mangánica": "mangánico",
    "mangánicas": "mangánico",
    "mangánicos": "mangánico",
    "mangáramos": "mangar",
    "mangáremos": "mangar",
    "mangásemos": "mangar",
    "mangó": "mangar",
    "maniaca": "maníaco",
    "maniacas": "maníaco",
    "maniacos": "maníaco",
    "manialba": "manialbo",
    "manialbas": "manialbo",
    "manialbos": "manialbo",
    "maniata": "maniatar",
    "maniataba": "maniatar",
    "maniatabais": "maniatar",
    "maniataban": "maniatar",
    "maniatabas": "maniatar",
    "maniatad": "maniatar",
    "maniatada": "maniatar",
    "maniatadas": "maniatar",
    "maniatado": "maniatar",
    "maniatados": "maniatar",
    "maniatamos": "maniatar",
    "maniatan": "maniatar",
    "maniatando": "maniatar",
    "maniatara": "maniatar",
    "maniatarais": "maniatar",
    "maniataran": "maniatar",
    "maniataras": "maniatar",
    "maniatare": "maniatar",
    "maniatareis": "maniatar",
    "maniataremos": "maniatar",
    "maniataren": "maniatar",
    "maniatares": "maniatar",
    "maniataron": "maniatar",
    "maniatará": "maniatar",
    "maniatarán": "maniatar",
    "maniatarás": "maniatar",
    "maniataré": "maniatar",
    "maniataréis": "maniatar",
    "maniataría": "maniatar",
    "maniataríais": "maniatar",
    "maniataríamos": "maniatar",
    "maniatarían": "maniatar",
    "maniatarías": "maniatar",
    "maniatas": "maniatar",
    "maniatase": "maniatar",
    "maniataseis": "maniatar",
    "maniatasen": "maniatar",
    "maniatases": "maniatar",
    "maniataste": "maniatar",
    "maniatasteis": "maniatar",
    "maniate": "maniatar",
    "maniatemos": "maniatar",
    "maniaten": "maniatar",
    "maniates": "maniatar",
    "maniato": "maniatar",
    "maniatábamos": "maniatar",
    "maniatáis": "maniatar",
    "maniatáramos": "maniatar",
    "maniatáremos": "maniatar",
    "maniatásemos": "maniatar",
    "maniaté": "maniatar",
    "maniatéis": "maniatar",
    "maniató": "maniatar",
    "maniblanca": "maniblanco",
    "maniblancas": "maniblanco",
    "maniblancos": "maniblanco",
    "manicomios": "manicomio",
    "manicordios": "manicordio",
    "manicorta": "manicorto",
    "manicortas": "manicorto",
    "manicortos": "manicorto",
    "manicuras": "manicura",
    "manicuristas": "manicurista",
    "manicuros": "manicuro",
    "manid": "manir",
    "manida": "manir",
    "manidas": "manir",
    "manido": "manir",
    "manidos": "manir",
    "maniendo": "manir",
    "maniera": "manir",
    "manierais": "manir",
    "manieran": "manir",
    "manieras": "manir",
    "maniere": "manir",
    "maniereis": "manir",
    "manieren": "manir",
    "manieres": "manir",
    "manierismos": "manierismo",
    "manieristas": "manierista",
    "manieron": "manir",
    "maniese": "manir",
    "manieseis": "manir",
    "maniesen": "manir",
    "manieses": "manir",
    "manifacturas": "manifactura",
    "manifesta": "manifesto",
    "manifestaba": "manifestar",
    "manifestabais": "manifestar",
    "manifestaban": "manifestar",
    "manifestabas": "manifestar",
    "manifestaciones": "manifestación",
    "manifestad": "manifestar",
    "manifestada": "manifestar",
    "manifestadas": "manifestar",
    "manifestado": "manifestar",
    "manifestadora": "manifestador",
    "manifestadoras": "manifestador",
    "manifestadores": "manifestador",
    "manifestados": "manifestar",
    "manifestamos": "manifestar",
    "manifestando": "manifestar",
    "manifestantes": "manifestante",
    "manifestara": "manifestar",
    "manifestarais": "manifestar",
    "manifestaran": "manifestar",
    "manifestaras": "manifestar",
    "manifestare": "manifestar",
    "manifestareis": "manifestar",
    "manifestaremos": "manifestar",
    "manifestaren": "manifestar",
    "manifestares": "manifestar",
    "manifestaron": "manifestar",
    "manifestará": "manifestar",
    "manifestarán": "manifestar",
    "manifestarás": "manifestar",
    "manifestaré": "manifestar",
    "manifestaréis": "manifestar",
    "manifestaría": "manifestar",
    "manifestaríais": "manifestar",
    "manifestaríamos": "manifestar",
    "manifestarían": "manifestar",
    "manifestarías": "manifestar",
    "manifestas": "manifesto",
    "manifestase": "manifestar",
    "manifestaseis": "manifestar",
    "manifestasen": "manifestar",
    "manifestases": "manifestar",
    "manifestaste": "manifestar",
    "manifestasteis": "manifestar",
    "manifestativa": "manifestativo",
    "manifestativas": "manifestativo",
    "manifestativos": "manifestativo",
    "manifestemos": "manifestar",
    "manifestos": "manifesto",
    "manifestábamos": "manifestar",
    "manifestáis": "manifestar",
    "manifestáramos": "manifestar",
    "manifestáremos": "manifestar",
    "manifestásemos": "manifestar",
    "manifesté": "manifestar",
    "manifestéis": "manifestar",
    "manifestó": "manifestar",
    "manifiesta": "manifiesto",
    "manifiestan": "manifestar",
    "manifiestas": "manifiesto",
    "manifieste": "manifestar",
    "manifiesten": "manifestar",
    "manifiestes": "manifestar",
    "manifiesto": "manifestar",
    "manifiestos": "manifiesto",
    "manigeros": "manigero",
    "maniguas": "manigua",
    "maniguetas": "manigueta",
    "manijas": "manija",
    "manilarga": "manilargo",
    "manilargas": "manilargo",
    "manilargos": "manilargo",
    "manillares": "manillar",
    "manillas": "manilla",
    "maniluvios": "maniluvio",
    "manimos": "manir",
    "maniobra": "maniobrar",
    "maniobraba": "maniobrar",
    "maniobrabais": "maniobrar",
    "maniobraban": "maniobrar",
    "maniobrabas": "maniobrar",
    "maniobrabilidades": "maniobrabilidad",
    "maniobrables": "maniobrable",
    "maniobrad": "maniobrar",
    "maniobrada": "maniobrar",
    "maniobradas": "maniobrar",
    "maniobrado": "maniobrar",
    "maniobrados": "maniobrar",
    "maniobramos": "maniobrar",
    "maniobran": "maniobrar",
    "maniobrando": "maniobrar",
    "maniobrara": "maniobrar",
    "maniobrarais": "maniobrar",
    "maniobraran": "maniobrar",
    "maniobraras": "maniobrar",
    "maniobrare": "maniobrar",
    "maniobrareis": "maniobrar",
    "maniobraremos": "maniobrar",
    "maniobraren": "maniobrar",
    "maniobrares": "maniobrar",
    "maniobraron": "maniobrar",
    "maniobrará": "maniobrar",
    "maniobrarán": "maniobrar",
    "maniobrarás": "maniobrar",
    "maniobraré": "maniobrar",
    "maniobraréis": "maniobrar",
    "maniobraría": "maniobrar",
    "maniobraríais": "maniobrar",
    "maniobraríamos": "maniobrar",
    "maniobrarían": "maniobrar",
    "maniobrarías": "maniobrar",
    "maniobras": "maniobrar",
    "maniobrase": "maniobrar",
    "maniobraseis": "maniobrar",
    "maniobrasen": "maniobrar",
    "maniobrases": "maniobrar",
    "maniobraste": "maniobrar",
    "maniobrasteis": "maniobrar",
    "maniobre": "maniobrar",
    "maniobremos": "maniobrar",
    "maniobren": "maniobrar",
    "maniobrera": "maniobrero",
    "maniobreras": "maniobrero",
    "maniobreros": "maniobrero",
    "maniobres": "maniobrar",
    "maniobristas": "maniobrista",
    "maniobro": "maniobrar",
    "maniobrábamos": "maniobrar",
    "maniobráis": "maniobrar",
    "maniobráramos": "maniobrar",
    "maniobráremos": "maniobrar",
    "maniobrásemos": "maniobrar",
    "maniobré": "maniobrar",
    "maniobréis": "maniobrar",
    "maniobró": "maniobrar",
    "maniotas": "maniota",
    "manipula": "manipular",
    "manipulaba": "manipular",
    "manipulabais": "manipular",
    "manipulaban": "manipular",
    "manipulabas": "manipular",
    "manipulables": "manipulable",
    "manipulaciones": "manipulación",
    "manipulad": "manipular",
    "manipulada": "manipular",
    "manipuladas": "manipular",
    "manipulado": "manipular",
    "manipuladora": "manipulador",
    "manipuladoras": "manipulador",
    "manipuladores": "manipulador",
    "manipulados": "manipular",
    "manipulamos": "manipular",
    "manipulan": "manipular",
    "manipulando": "manipular",
    "manipulara": "manipular",
    "manipularais": "manipular",
    "manipularan": "manipular",
    "manipularas": "manipular",
    "manipulare": "manipular",
    "manipulareis": "manipular",
    "manipularemos": "manipular",
    "manipularen": "manipular",
    "manipulares": "manipular",
    "manipularon": "manipular",
    "manipulará": "manipular",
    "manipularán": "manipular",
    "manipularás": "manipular",
    "manipularé": "manipular",
    "manipularéis": "manipular",
    "manipularía": "manipular",
    "manipularíais": "manipular",
    "manipularíamos": "manipular",
    "manipularían": "manipular",
    "manipularías": "manipular",
    "manipulas": "manipular",
    "manipulase": "manipular",
    "manipulaseis": "manipular",
    "manipulasen": "manipular",
    "manipulases": "manipular",
    "manipulaste": "manipular",
    "manipulasteis": "manipular",
    "manipule": "manipular",
    "manipulemos": "manipular",
    "manipulen": "manipular",
    "manipuleos": "manipuleo",
    "manipules": "manipular",
    "manipulo": "manipular",
    "manipulábamos": "manipular",
    "manipuláis": "manipular",
    "manipuláramos": "manipular",
    "manipuláremos": "manipular",
    "manipulásemos": "manipular",
    "manipulé": "manipular",
    "manipuléis": "manipular",
    "manipuló": "manipular",
    "maniquea": "maniqueo",
    "maniqueas": "maniqueo",
    "maniqueos": "maniqueo",
    "maniquetes": "maniquete",
    "maniqueísmos": "maniqueísmo",
    "maniqueístas": "maniqueísta",
    "maniquíes": "maniquí",
    "maniquís": "maniquí",
    "maniquíses": "maniquí",
    "maniremos": "manir",
    "manirrota": "manirroto",
    "manirrotas": "manirroto",
    "manirrotos": "manirroto",
    "manirá": "manir",
    "manirán": "manir",
    "manirás": "manir",
    "maniré": "manir",
    "maniréis": "manir",
    "maniría": "manir",
    "maniríais": "manir",
    "maniríamos": "manir",
    "manirían": "manir",
    "manirías": "manir",
    "manisera": "manisero",
    "maniseras": "manisero",
    "maniseros": "manisero",
    "manises": "maní",
    "manismos": "manismo",
    "maniste": "manir",
    "manisteis": "manir",
    "manitas": "manita",
    "manitoles": "manitol",
    "manitos": "manito",
    "manivelas": "manivela",
    "maniática": "ralentización",
    "maniáticas": "ralentización",
    "maniáticos": "ralentización",
    "maniéramos": "manir",
    "maniéremos": "manir",
    "maniésemos": "manir",
    "manió": "manir",
    "manjares": "manjar",
    "manjorradas": "manjorrada",
    "manlievas": "manlieva",
    "mano": "manir",
    "manobreros": "manobrero",
    "manojeras": "manojera",
    "manojos": "manojo",
    "manola": "manolo",
    "manolas": "manolo",
    "manoletinas": "manoletina",
    "manolos": "manolo",
    "manometrías": "manometría",
    "manométrica": "manométrico",
    "manométricas": "manométrico",
    "manométricos": "manométrico",
    "manoplas": "manopla",
    "manorreductores": "manorreductor",
    "manos": "mano",
    "manosas": "manosa",
    "manosea": "manosear",
    "manoseaba": "manosear",
    "manoseabais": "manosear",
    "manoseaban": "manosear",
    "manoseabas": "manosear",
    "manosead": "manosear",
    "manoseada": "manosear",
    "manoseadas": "manosear",
    "manoseado": "manosear",
    "manoseadora": "manoseador",
    "manoseadoras": "manoseador",
    "manoseadores": "manoseador",
    "manoseados": "manosear",
    "manoseamos": "manosear",
    "manosean": "manosear",
    "manoseando": "manosear",
    "manoseara": "manosear",
    "manosearais": "manosear",
    "manosearan": "manosear",
    "manosearas": "manosear",
    "manoseare": "manosear",
    "manoseareis": "manosear",
    "manosearemos": "manosear",
    "manosearen": "manosear",
    "manoseares": "manosear",
    "manosearon": "manosear",
    "manoseará": "manosear",
    "manosearán": "manosear",
    "manosearás": "manosear",
    "manosearé": "manosear",
    "manosearéis": "manosear",
    "manosearía": "manosear",
    "manosearíais": "manosear",
    "manosearíamos": "manosear",
    "manosearían": "manosear",
    "manosearías": "manosear",
    "manoseas": "manosear",
    "manosease": "manosear",
    "manoseaseis": "manosear",
    "manoseasen": "manosear",
    "manoseases": "manosear",
    "manoseaste": "manosear",
    "manoseasteis": "manosear",
    "manosee": "manosear",
    "manoseemos": "manosear",
    "manoseen": "manosear",
    "manosees": "manosear",
    "manoseo": "manosear",
    "manoseos": "manoseo",
    "manoseábamos": "manosear",
    "manoseáis": "manosear",
    "manoseáramos": "manosear",
    "manoseáremos": "manosear",
    "manoseásemos": "manosear",
    "manoseé": "manosear",
    "manoseéis": "manosear",
    "manoseó": "manosear",
    "manostatos": "manostato",
    "manotadas": "manotada",
    "manotazos": "manotazo",
    "manotea": "manotear",
    "manoteaba": "manotear",
    "manoteabais": "manotear",
    "manoteaban": "manotear",
    "manoteabas": "manotear",
    "manotead": "manotear",
    "manoteada": "manotear",
    "manoteadas": "manotear",
    "manoteado": "manotear",
    "manoteados": "manotear",
    "manoteamos": "manotear",
    "manotean": "manotear",
    "manoteando": "manotear",
    "manoteara": "manotear",
    "manotearais": "manotear",
    "manotearan": "manotear",
    "manotearas": "manotear",
    "manoteare": "manotear",
    "manoteareis": "manotear",
    "manotearemos": "manotear",
    "manotearen": "manotear",
    "manoteares": "manotear",
    "manotearon": "manotear",
    "manoteará": "manotear",
    "manotearán": "manotear",
    "manotearás": "manotear",
    "manotearé": "manotear",
    "manotearéis": "manotear",
    "manotearía": "manotear",
    "manotearíais": "manotear",
    "manotearíamos": "manotear",
    "manotearían": "manotear",
    "manotearías": "manotear",
    "manoteas": "manotear",
    "manotease": "manotear",
    "manoteaseis": "manotear",
    "manoteasen": "manotear",
    "manoteases": "manotear",
    "manoteaste": "manotear",
    "manoteasteis": "manotear",
    "manotee": "manotear",
    "manoteemos": "manotear",
    "manoteen": "manotear",
    "manotees": "manotear",
    "manoteo": "manotear",
    "manoteos": "manoteo",
    "manoteábamos": "manotear",
    "manoteáis": "manotear",
    "manoteáramos": "manotear",
    "manoteáremos": "manotear",
    "manoteásemos": "manotear",
    "manoteé": "manotear",
    "manoteéis": "manotear",
    "manoteó": "manotear",
    "manotones": "manotón",
    "manovacuómetros": "manovacuómetro",
    "manque": "mancar",
    "manquea": "manquear",
    "manqueaba": "manquear",
    "manqueabais": "manquear",
    "manqueaban": "manquear",
    "manqueabas": "manquear",
    "manquead": "manquear",
    "manqueada": "manquear",
    "manqueadas": "manquear",
    "manqueado": "manquear",
    "manqueados": "manquear",
    "manqueamos": "manquear",
    "manquean": "manquear",
    "manqueando": "manquear",
    "manqueara": "manquear",
    "manquearais": "manquear",
    "manquearan": "manquear",
    "manquearas": "manquear",
    "manqueare": "manquear",
    "manqueareis": "manquear",
    "manquearemos": "manquear",
    "manquearen": "manquear",
    "manqueares": "manquear",
    "manquearon": "manquear",
    "manqueará": "manquear",
    "manquearán": "manquear",
    "manquearás": "manquear",
    "manquearé": "manquear",
    "manquearéis": "manquear",
    "manquearía": "manquear",
    "manquearíais": "manquear",
    "manquearíamos": "manquear",
    "manquearían": "manquear",
    "manquearías": "manquear",
    "manqueas": "manquear",
    "manquease": "manquear",
    "manqueaseis": "manquear",
    "manqueasen": "manquear",
    "manqueases": "manquear",
    "manqueaste": "manquear",
    "manqueasteis": "manquear",
    "manquedades": "manquedad",
    "manquee": "manquear",
    "manqueemos": "manquear",
    "manqueen": "manquear",
    "manquees": "manquear",
    "manquemos": "mancar",
    "manquen": "mancar",
    "manqueo": "manquear",
    "manqueras": "manquera",
    "manques": "mancar",
    "manqueábamos": "manquear",
    "manqueáis": "manquear",
    "manqueáramos": "manquear",
    "manqueáremos": "manquear",
    "manqueásemos": "manquear",
    "manqueé": "manquear",
    "manqueéis": "manquear",
    "manqueó": "manquear",
    "manqué": "mancar",
    "manquéis": "mancar",
    "manresana": "manresano",
    "manresanas": "manresano",
    "manresanos": "manresano",
    "mansa": "manso",
    "mansas": "manso",
    "mansedumbres": "mansedumbre",
    "mansiones": "mansión",
    "mansos": "manso",
    "mansurrona": "mansurrón",
    "mansurronas": "mansurrón",
    "mansurrones": "mansurrón",
    "mantas": "manta",
    "mantea": "mantear",
    "manteaba": "mantear",
    "manteabais": "mantear",
    "manteaban": "mantear",
    "manteabas": "mantear",
    "mantead": "mantear",
    "manteada": "mantear",
    "manteadas": "mantear",
    "manteado": "mantear",
    "manteadora": "manteador",
    "manteadoras": "manteador",
    "manteadores": "manteador",
    "manteados": "mantear",
    "manteamientos": "manteamiento",
    "manteamos": "mantear",
    "mantean": "mantear",
    "manteando": "mantear",
    "manteara": "mantear",
    "mantearais": "mantear",
    "mantearan": "mantear",
    "mantearas": "mantear",
    "manteare": "mantear",
    "manteareis": "mantear",
    "mantearemos": "mantear",
    "mantearen": "mantear",
    "manteares": "mantear",
    "mantearon": "mantear",
    "manteará": "mantear",
    "mantearán": "mantear",
    "mantearás": "mantear",
    "mantearé": "mantear",
    "mantearéis": "mantear",
    "mantearía": "mantear",
    "mantearíais": "mantear",
    "mantearíamos": "mantear",
    "mantearían": "mantear",
    "mantearías": "mantear",
    "manteas": "mantear",
    "mantease": "mantear",
    "manteaseis": "mantear",
    "manteasen": "mantear",
    "manteases": "mantear",
    "manteaste": "mantear",
    "manteasteis": "mantear",
    "mantecadas": "mantecada",
    "mantecados": "mantecado",
    "mantecas": "manteca",
    "mantecosa": "mantecoso",
    "mantecosas": "mantecoso",
    "mantecosidades": "mantecosidad",
    "mantecosos": "mantecoso",
    "mantee": "mantear",
    "manteemos": "mantear",
    "manteen": "mantear",
    "mantees": "mantear",
    "mantelerías": "mantelería",
    "manteles": "mantel",
    "manteletas": "manteleta",
    "manteletes": "mantelete",
    "mantellinas": "mantellina",
    "mantendremos": "mantener",
    "mantendrá": "mantener",
    "mantendrán": "mantener",
    "mantendrás": "mantener",
    "mantendré": "mantener",
    "mantendréis": "mantener",
    "mantendría": "mantener",
    "mantendríais": "mantener",
    "mantendríamos": "mantener",
    "mantendrían": "mantener",
    "mantendrías": "mantener",
    "mantened": "mantener",
    "mantenedora": "mantenedor",
    "mantenedoras": "mantenedor",
    "mantenedores": "mantenedor",
    "mantenemos": "mantener",
    "mantenga": "mantener",
    "mantengamos": "mantener",
    "mantengan": "mantener",
    "mantengas": "mantener",
    "mantengo": "mantener",
    "mantengáis": "mantener",
    "mantenibilidades": "mantenibilidad",
    "mantenida": "mantener",
    "mantenidas": "mantener",
    "mantenido": "mantener",
    "mantenidos": "mantener",
    "manteniendo": "mantener",
    "mantenimientos": "mantenimiento",
    "mantenéis": "mantener",
    "mantenía": "mantener",
    "manteníais": "mantener",
    "manteníamos": "mantener",
    "mantenían": "mantener",
    "mantenías": "mantener",
    "manteo": "mantear",
    "manteos": "manteo",
    "mantequera": "mantequero",
    "mantequeras": "mantequero",
    "mantequeros": "mantequero",
    "mantequerías": "mantequería",
    "mantequillas": "mantequilla",
    "mantera": "mantero",
    "manteras": "mantero",
    "manteros": "mantero",
    "mantesa": "mantés",
    "mantesas": "mantés",
    "manteses": "mantés",
    "manteábamos": "mantear",
    "manteáis": "mantear",
    "manteáramos": "mantear",
    "manteáremos": "mantear",
    "manteásemos": "mantear",
    "manteé": "mantear",
    "manteéis": "mantear",
    "manteístas": "manteísta",
    "manteó": "mantear",
    "mantiene": "mantener",
    "mantienen": "mantener",
    "mantienes": "mantener",
    "mantillas": "mantilla",
    "mantillos": "mantillo",
    "mantisas": "mantisa",
    "mantones": "mantón",
    "mantos": "manto",
    "mantuana": "mantuano",
    "mantuanas": "mantuano",
    "mantuanos": "mantuano",
    "mantuda": "mantudo",
    "mantudas": "mantudo",
    "mantudos": "mantudo",
    "mantuve": "mantener",
    "mantuviera": "mantener",
    "mantuvierais": "mantener",
    "mantuvieran": "mantener",
    "mantuvieras": "mantener",
    "mantuviere": "mantener",
    "mantuviereis": "mantener",
    "mantuvieren": "mantener",
    "mantuvieres": "mantener",
    "mantuvieron": "mantener",
    "mantuviese": "mantener",
    "mantuvieseis": "mantener",
    "mantuviesen": "mantener",
    "mantuvieses": "mantener",
    "mantuvimos": "mantener",
    "mantuviste": "mantener",
    "mantuvisteis": "mantener",
    "mantuviéramos": "mantener",
    "mantuviéremos": "mantener",
    "mantuviésemos": "mantener",
    "mantuvo": "mantener",
    "mantén": "mantener",
    "manual-automática": "manual-automático",
    "manual-automáticas": "manual-automático",
    "manual-automáticos": "manual-automático",
    "manuales": "manual",
    "manualidades": "manualidad",
    "manuares": "manuar",
    "manubrioesternales": "manubrioesternal",
    "manubrios": "manubrio",
    "manuelina": "manuelino",
    "manuelinas": "manuelino",
    "manuelinos": "manuelino",
    "manuellas": "manuella",
    "manufactura": "manufacturar",
    "manufacturaba": "manufacturar",
    "manufacturabais": "manufacturar",
    "manufacturaban": "manufacturar",
    "manufacturabas": "manufacturar",
    "manufacturaciones": "manufacturación",
    "manufacturad": "manufacturar",
    "manufacturada": "manufacturar",
    "manufacturadas": "manufacturar",
    "manufacturado": "manufacturar",
    "manufacturados": "manufacturar",
    "manufacturamos": "manufacturar",
    "manufacturan": "manufacturar",
    "manufacturando": "manufacturar",
    "manufacturara": "manufacturar",
    "manufacturarais": "manufacturar",
    "manufacturaran": "manufacturar",
    "manufacturaras": "manufacturar",
    "manufacturare": "manufacturar",
    "manufacturareis": "manufacturar",
    "manufacturaremos": "manufacturar",
    "manufacturaren": "manufacturar",
    "manufacturares": "manufacturar",
    "manufacturaron": "manufacturar",
    "manufacturará": "manufacturar",
    "manufacturarán": "manufacturar",
    "manufacturarás": "manufacturar",
    "manufacturaré": "manufacturar",
    "manufacturaréis": "manufacturar",
    "manufacturaría": "manufacturar",
    "manufacturaríais": "manufacturar",
    "manufacturaríamos": "manufacturar",
    "manufacturarían": "manufacturar",
    "manufacturarías": "manufacturar",
    "manufacturas": "manufacturar",
    "manufacturase": "manufacturar",
    "manufacturaseis": "manufacturar",
    "manufacturasen": "manufacturar",
    "manufacturases": "manufacturar",
    "manufacturaste": "manufacturar",
    "manufacturasteis": "manufacturar",
    "manufacture": "manufacturar",
    "manufacturemos": "manufacturar",
    "manufacturen": "manufacturar",
    "manufacturera": "manufacturero",
    "manufactureras": "manufacturero",
    "manufactureros": "manufacturero",
    "manufactures": "manufacturar",
    "manufacturo": "manufacturar",
    "manufacturábamos": "manufacturar",
    "manufacturáis": "manufacturar",
    "manufacturáramos": "manufacturar",
    "manufacturáremos": "manufacturar",
    "manufacturásemos": "manufacturar",
    "manufacturé": "manufacturar",
    "manufacturéis": "manufacturar",
    "manufacturó": "manufacturar",
    "manumisa": "manumiso",
    "manumisas": "manumiso",
    "manumisiones": "manumisión",
    "manumisores": "manumisor",
    "manumisos": "manumiso",
    "manumita": "manumitir",
    "manumitamos": "manumitir",
    "manumitan": "manumitir",
    "manumitas": "manumitir",
    "manumite": "manumitir",
    "manumiten": "manumitir",
    "manumites": "manumitir",
    "manumitid": "manumitir",
    "manumitida": "manumitir",
    "manumitidas": "manumitir",
    "manumitido": "manumitir",
    "manumitidos": "manumitir",
    "manumitiendo": "manumitir",
    "manumitiera": "manumitir",
    "manumitierais": "manumitir",
    "manumitieran": "manumitir",
    "manumitieras": "manumitir",
    "manumitiere": "manumitir",
    "manumitiereis": "manumitir",
    "manumitieren": "manumitir",
    "manumitieres": "manumitir",
    "manumitieron": "manumitir",
    "manumitiese": "manumitir",
    "manumitieseis": "manumitir",
    "manumitiesen": "manumitir",
    "manumitieses": "manumitir",
    "manumitimos": "manumitir",
    "manumitiremos": "manumitir",
    "manumitirá": "manumitir",
    "manumitirán": "manumitir",
    "manumitirás": "manumitir",
    "manumitiré": "manumitir",
    "manumitiréis": "manumitir",
    "manumitiría": "manumitir",
    "manumitiríais": "manumitir",
    "manumitiríamos": "manumitir",
    "manumitirían": "manumitir",
    "manumitirías": "manumitir",
    "manumitiste": "manumitir",
    "manumitisteis": "manumitir",
    "manumitiéramos": "manumitir",
    "manumitiéremos": "manumitir",
    "manumitiésemos": "manumitir",
    "manumitió": "manumitir",
    "manumito": "manumitir",
    "manumitáis": "manumitir",
    "manumití": "manumitir",
    "manumitía": "manumitir",
    "manumitíais": "manumitir",
    "manumitíamos": "manumitir",
    "manumitían": "manumitir",
    "manumitías": "manumitir",
    "manumitís": "manumitir",
    "manuscriba": "manuscribir",
    "manuscribamos": "manuscribir",
    "manuscriban": "manuscribir",
    "manuscribas": "manuscribir",
    "manuscribe": "manuscribir",
    "manuscriben": "manuscribir",
    "manuscribes": "manuscribir",
    "manuscribid": "manuscribir",
    "manuscribiendo": "manuscribir",
    "manuscribiera": "manuscribir",
    "manuscribierais": "manuscribir",
    "manuscribieran": "manuscribir",
    "manuscribieras": "manuscribir",
    "manuscribiere": "manuscribir",
    "manuscribiereis": "manuscribir",
    "manuscribieren": "manuscribir",
    "manuscribieres": "manuscribir",
    "manuscribieron": "manuscribir",
    "manuscribiese": "manuscribir",
    "manuscribieseis": "manuscribir",
    "manuscribiesen": "manuscribir",
    "manuscribieses": "manuscribir",
    "manuscribimos": "manuscribir",
    "manuscribiremos": "manuscribir",
    "manuscribirá": "manuscribir",
    "manuscribirán": "manuscribir",
    "manuscribirás": "manuscribir",
    "manuscribiré": "manuscribir",
    "manuscribiréis": "manuscribir",
    "manuscribiría": "manuscribir",
    "manuscribiríais": "manuscribir",
    "manuscribiríamos": "manuscribir",
    "manuscribirían": "manuscribir",
    "manuscribirías": "manuscribir",
    "manuscribiste": "manuscribir",
    "manuscribisteis": "manuscribir",
    "manuscribiéramos": "manuscribir",
    "manuscribiéremos": "manuscribir",
    "manuscribiésemos": "manuscribir",
    "manuscribió": "manuscribir",
    "manuscribo": "manuscribir",
    "manuscribáis": "manuscribir",
    "manuscribí": "manuscribir",
    "manuscribía": "manuscribir",
    "manuscribíais": "manuscribir",
    "manuscribíamos": "manuscribir",
    "manuscribían": "manuscribir",
    "manuscribías": "manuscribir",
    "manuscribís": "manuscribir",
    "manuscrita": "manuscribir",
    "manuscritas": "manuscribir",
    "manuscrito": "manuscribir",
    "manuscritos": "manuscrito",
    "manutenciones": "manutención",
    "manutendremos": "manutener",
    "manutendrá": "manutener",
    "manutendrán": "manutener",
    "manutendrás": "manutener",
    "manutendré": "manutener",
    "manutendréis": "manutener",
    "manutendría": "manutener",
    "manutendríais": "manutener",
    "manutendríamos": "manutener",
    "manutendrían": "manutener",
    "manutendrías": "manutener",
    "manutened": "manutener",
    "manutenemos": "manutener",
    "manutenga": "manutener",
    "manutengamos": "manutener",
    "manutengan": "manutener",
    "manutengas": "manutener",
    "manutengo": "manutener",
    "manutengáis": "manutener",
    "manutenida": "manutener",
    "manutenidas": "manutener",
    "manutenido": "manutener",
    "manutenidos": "manutener",
    "manuteniendo": "manutener",
    "manutenéis": "manutener",
    "manutenía": "manutener",
    "manuteníais": "manutener",
    "manuteníamos": "manutener",
    "manutenían": "manutener",
    "manutenías": "manutener",
    "manutiene": "manutener",
    "manutienen": "manutener",
    "manutienes": "manutener",
    "manutuve": "manutener",
    "manutuviera": "manutener",
    "manutuvierais": "manutener",
    "manutuvieran": "manutener",
    "manutuvieras": "manutener",
    "manutuviere": "manutener",
    "manutuviereis": "manutener",
    "manutuvieren": "manutener",
    "manutuvieres": "manutener",
    "manutuvieron": "manutener",
    "manutuviese": "manutener",
    "manutuvieseis": "manutener",
    "manutuviesen": "manutener",
    "manutuvieses": "manutener",
    "manutuvimos": "manutener",
    "manutuviste": "manutener",
    "manutuvisteis": "manutener",
    "manutuviéramos": "manutener",
    "manutuviéremos": "manutener",
    "manutuviésemos": "manutener",
    "manutuvo": "manutener",
    "manutén": "manutener",
    "manzanales": "manzanal",
    "manzanares": "manzanar",
    "manzanas": "manzana",
    "manzaneras": "manzanera",
    "manzaniles": "manzanil",
    "manzanillas": "manzanilla",
    "manzanos": "manzano",
    "manábamos": "manar",
    "manáis": "manir",
    "manáramos": "manar",
    "manáremos": "manar",
    "manásemos": "manar",
    "mané": "manar",
    "manéis": "manar",
    "maní": "manir",
    "manía": "manir",
    "maníaca": "maníaco",
    "maníacas": "maníaco",
    "maníacos": "maníaco",
    "maníais": "manir",
    "maníamos": "manir",
    "manían": "manir",
    "manías": "manir",
    "manípulos": "manípulo",
    "manís": "manir",
    "manó": "manar",
    "manómetros": "manómetro",
    "manóstatos": "manóstato",
    "maories": "maorí",
    "maoris": "maori",
    "maoríes": "maorí",
    "maorís": "maorí",
    "maoríses": "maorí",
    "maoísmos": "maoísmo",
    "maoístas": "maoísta",
    "mapaches": "mapache",
    "mapamundies": "mapamundi",
    "mapamundis": "mapamundi",
    "mapas": "mapa",
    "mapeos": "mapeo",
    "mapuches": "mapuche",
    "maque": "macar",
    "maquea": "maquear",
    "maqueaba": "maquear",
    "maqueabais": "maquear",
    "maqueaban": "maquear",
    "maqueabas": "maquear",
    "maquead": "maquear",
    "maqueada": "maquear",
    "maqueadas": "maquear",
    "maqueado": "maquear",
    "maqueadora": "maqueador",
    "maqueadoras": "maqueador",
    "maqueadores": "maqueador",
    "maqueados": "maquear",
    "maqueamos": "maquear",
    "maquean": "maquear",
    "maqueando": "maquear",
    "maqueara": "maquear",
    "maquearais": "maquear",
    "maquearan": "maquear",
    "maquearas": "maquear",
    "maqueare": "maquear",
    "maqueareis": "maquear",
    "maquearemos": "maquear",
    "maquearen": "maquear",
    "maqueares": "maquear",
    "maquearon": "maquear",
    "maqueará": "maquear",
    "maquearán": "maquear",
    "maquearás": "maquear",
    "maquearé": "maquear",
    "maquearéis": "maquear",
    "maquearía": "maquear",
    "maquearíais": "maquear",
    "maquearíamos": "maquear",
    "maquearían": "maquear",
    "maquearías": "maquear",
    "maqueas": "maquear",
    "maquease": "maquear",
    "maqueaseis": "maquear",
    "maqueasen": "maquear",
    "maqueases": "maquear",
    "maqueaste": "maquear",
    "maqueasteis": "maquear",
    "maquee": "maquear",
    "maqueemos": "maquear",
    "maqueen": "maquear",
    "maquees": "maquear",
    "maquemos": "macar",
    "maquen": "macar",
    "maqueo": "maquear",
    "maques": "maque",
    "maquetaciones": "maquetación",
    "maquetas": "maqueta",
    "maquetistas": "maquetista",
    "maqueábamos": "maquear",
    "maqueáis": "maquear",
    "maqueáramos": "maquear",
    "maqueáremos": "maquear",
    "maqueásemos": "maquear",
    "maqueé": "maquear",
    "maqueéis": "maquear",
    "maqueó": "maquear",
    "maquias": "maquia",
    "maquiavelismos": "maquiavelismo",
    "maquiavelistas": "maquiavelista",
    "maquiavélica": "maquiavélico",
    "maquiavélicas": "maquiavélico",
    "maquiavélicos": "maquiavélico",
    "maquila": "maquilar",
    "maquilaba": "maquilar",
    "maquilabais": "maquilar",
    "maquilaban": "maquilar",
    "maquilabas": "maquilar",
    "maquilad": "maquilar",
    "maquilada": "maquilar",
    "maquiladas": "maquilar",
    "maquilado": "maquilar",
    "maquiladora": "maquilador",
    "maquiladoras": "maquilador",
    "maquiladores": "maquilador",
    "maquilados": "maquilar",
    "maquilamos": "maquilar",
    "maquilan": "maquilar",
    "maquilando": "maquilar",
    "maquilara": "maquilar",
    "maquilarais": "maquilar",
    "maquilaran": "maquilar",
    "maquilaras": "maquilar",
    "maquilare": "maquilar",
    "maquilareis": "maquilar",
    "maquilaremos": "maquilar",
    "maquilaren": "maquilar",
    "maquilares": "maquilar",
    "maquilaron": "maquilar",
    "maquilará": "maquilar",
    "maquilarán": "maquilar",
    "maquilarás": "maquilar",
    "maquilaré": "maquilar",
    "maquilaréis": "maquilar",
    "maquilaría": "maquilar",
    "maquilaríais": "maquilar",
    "maquilaríamos": "maquilar",
    "maquilarían": "maquilar",
    "maquilarías": "maquilar",
    "maquilas": "maquilar",
    "maquilase": "maquilar",
    "maquilaseis": "maquilar",
    "maquilasen": "maquilar",
    "maquilases": "maquilar",
    "maquilaste": "maquilar",
    "maquilasteis": "maquilar",
    "maquile": "maquilar",
    "maquilemos": "maquilar",
    "maquilen": "maquilar",
    "maquilera": "maquilero",
    "maquileras": "maquilero",
    "maquileros": "maquilero",
    "maquiles": "maquilar",
    "maquilla": "maquillar",
    "maquillaba": "maquillar",
    "maquillabais": "maquillar",
    "maquillaban": "maquillar",
    "maquillabas": "maquillar",
    "maquillad": "maquillar",
    "maquillada": "maquillar",
    "maquilladas": "maquillar",
    "maquillado": "maquillar",
    "maquilladora": "maquillador",
    "maquilladoras": "maquillador",
    "maquilladores": "maquillador",
    "maquillados": "maquillar",
    "maquillajes": "maquillaje",
    "maquillamos": "maquillar",
    "maquillan": "maquillar",
    "maquillando": "maquillar",
    "maquillara": "maquillar",
    "maquillarais": "maquillar",
    "maquillaran": "maquillar",
    "maquillaras": "maquillar",
    "maquillare": "maquillar",
    "maquillareis": "maquillar",
    "maquillaremos": "maquillar",
    "maquillaren": "maquillar",
    "maquillares": "maquillar",
    "maquillaron": "maquillar",
    "maquillará": "maquillar",
    "maquillarán": "maquillar",
    "maquillarás": "maquillar",
    "maquillaré": "maquillar",
    "maquillaréis": "maquillar",
    "maquillaría": "maquillar",
    "maquillaríais": "maquillar",
    "maquillaríamos": "maquillar",
    "maquillarían": "maquillar",
    "maquillarías": "maquillar",
    "maquillas": "maquillar",
    "maquillase": "maquillar",
    "maquillaseis": "maquillar",
    "maquillasen": "maquillar",
    "maquillases": "maquillar",
    "maquillaste": "maquillar",
    "maquillasteis": "maquillar",
    "maquille": "maquillar",
    "maquillemos": "maquillar",
    "maquillen": "maquillar",
    "maquilles": "maquillar",
    "maquillo": "maquillar",
    "maquillábamos": "maquillar",
    "maquilláis": "maquillar",
    "maquilláramos": "maquillar",
    "maquilláremos": "maquillar",
    "maquillásemos": "maquillar",
    "maquillé": "maquillar",
    "maquilléis": "maquillar",
    "maquilló": "maquillar",
    "maquilo": "maquilar",
    "maquilábamos": "maquilar",
    "maquiláis": "maquilar",
    "maquiláramos": "maquilar",
    "maquiláremos": "maquilar",
    "maquilásemos": "maquilar",
    "maquilé": "maquilar",
    "maquiléis": "maquilar",
    "maquiló": "maquilar",
    "maquina": "maquinar",
    "maquinaba": "maquinar",
    "maquinabais": "maquinar",
    "maquinaban": "maquinar",
    "maquinabas": "maquinar",
    "maquinabilidades": "maquinabilidad",
    "maquinaciones": "maquinación",
    "maquinad": "maquinar",
    "maquinada": "maquinar",
    "maquinadas": "maquinar",
    "maquinado": "maquinar",
    "maquinadora": "maquinador",
    "maquinadoras": "maquinador",
    "maquinadores": "maquinador",
    "maquinados": "maquinar",
    "maquinales": "maquinal",
    "maquinamos": "maquinar",
    "maquinan": "maquinar",
    "maquinando": "maquinar",
    "maquinara": "maquinar",
    "maquinarais": "maquinar",
    "maquinaran": "maquinar",
    "maquinaras": "maquinar",
    "maquinare": "maquinar",
    "maquinareis": "maquinar",
    "maquinaremos": "maquinar",
    "maquinaren": "maquinar",
    "maquinares": "maquinar",
    "maquinarias": "maquinaria",
    "maquinaron": "maquinar",
    "maquinará": "maquinar",
    "maquinarán": "maquinar",
    "maquinarás": "maquinar",
    "maquinaré": "maquinar",
    "maquinaréis": "maquinar",
    "maquinaría": "maquinar",
    "maquinaríais": "maquinar",
    "maquinaríamos": "maquinar",
    "maquinarían": "maquinar",
    "maquinarías": "maquinar",
    "maquinas": "maquinar",
    "maquinase": "maquinar",
    "maquinaseis": "maquinar",
    "maquinasen": "maquinar",
    "maquinases": "maquinar",
    "maquinaste": "maquinar",
    "maquinasteis": "maquinar",
    "maquine": "maquinar",
    "maquinemos": "maquinar",
    "maquinen": "maquinar",
    "maquines": "maquinar",
    "maquinillas": "maquinilla",
    "maquinismos": "maquinismo",
    "maquinistas": "maquinista",
    "maquinizaciones": "maquinización",
    "maquino": "maquinar",
    "maquinábamos": "maquinar",
    "maquináis": "maquinar",
    "maquináramos": "maquinar",
    "maquináremos": "maquinar",
    "maquinásemos": "maquinar",
    "maquiné": "maquinar",
    "maquinéis": "maquinar",
    "maquinó": "maquinar",
    "maquis": "maqui",
    "maqué": "macar",
    "maquéis": "macar",
    "mar.": "marzo",
    "marabuntas": "marabunta",
    "marabutos": "marabuto",
    "marabúes": "marabú",
    "marabúses": "marabú",
    "maracas": "maraca",
    "maraeses": "mará",
    "maragata": "maragato",
    "maragatas": "maragato",
    "maragatos": "maragato",
    "marantácea": "marantáceo",
    "marantáceas": "marantáceo",
    "marantáceos": "marantáceo",
    "maraquera": "maraquero",
    "maraqueras": "maraquero",
    "maraqueros": "maraquero",
    "marasmos": "marasmo",
    "marathas": "maratha",
    "maratiales": "maratial",
    "maratones": "maratón",
    "maratoniana": "maratoniano",
    "maratonianas": "maratoniano",
    "maratonianos": "maratoniano",
    "maravedises": "maravedí",
    "maravedíes": "maravedí",
    "maravedís": "maravedí",
    "maravedíses": "maravedí",
    "maravilla": "maravillar",
    "maravillaba": "maravillar",
    "maravillabais": "maravillar",
    "maravillaban": "maravillar",
    "maravillabas": "maravillar",
    "maravillad": "maravillar",
    "maravillada": "maravillar",
    "maravilladas": "maravillar",
    "maravillado": "maravillar",
    "maravillados": "maravillar",
    "maravillamos": "maravillar",
    "maravillan": "maravillar",
    "maravillando": "maravillar",
    "maravillara": "maravillar",
    "maravillarais": "maravillar",
    "maravillaran": "maravillar",
    "maravillaras": "maravillar",
    "maravillare": "maravillar",
    "maravillareis": "maravillar",
    "maravillaremos": "maravillar",
    "maravillaren": "maravillar",
    "maravillares": "maravillar",
    "maravillaron": "maravillar",
    "maravillará": "maravillar",
    "maravillarán": "maravillar",
    "maravillarás": "maravillar",
    "maravillaré": "maravillar",
    "maravillaréis": "maravillar",
    "maravillaría": "maravillar",
    "maravillaríais": "maravillar",
    "maravillaríamos": "maravillar",
    "maravillarían": "maravillar",
    "maravillarías": "maravillar",
    "maravillas": "maravillar",
    "maravillase": "maravillar",
    "maravillaseis": "maravillar",
    "maravillasen": "maravillar",
    "maravillases": "maravillar",
    "maravillaste": "maravillar",
    "maravillasteis": "maravillar",
    "maraville": "maravillar",
    "maravillemos": "maravillar",
    "maravillen": "maravillar",
    "maravilles": "maravillar",
    "maravillo": "maravillar",
    "maravillosa": "maravilloso",
    "maravillosas": "maravilloso",
    "maravillosos": "maravilloso",
    "maravillábamos": "maravillar",
    "maravilláis": "maravillar",
    "maravilláramos": "maravillar",
    "maravilláremos": "maravillar",
    "maravillásemos": "maravillar",
    "maravillé": "maravillar",
    "maravilléis": "maravillar",
    "maravilló": "maravillar",
    "marañas": "maraña",
    "marañera": "marañero",
    "marañeras": "marañero",
    "marañeros": "marañero",
    "marbellíes": "marbellí",
    "marbellís": "marbellí",
    "marbetes": "marbete",
    "marca": "marcar",
    "marcaba": "marcar",
    "marcabais": "marcar",
    "marcaban": "marcar",
    "marcabas": "marcar",
    "marcables": "marcable",
    "marcaciones": "marcación",
    "marcad": "marcar",
    "marcada": "marcar",
    "marcadas": "marcar",
    "marcado": "marcar",
    "marcadora": "marcador",
    "marcadoras": "marcador",
    "marcadores": "marcador",
    "marcados": "marcar",
    "marcajes": "marcaje",
    "marcamos": "marcar",
    "marcan": "marcar",
    "marcando": "marcar",
    "marcara": "marcar",
    "marcarais": "marcar",
    "marcaran": "marcar",
    "marcaras": "marcar",
    "marcare": "marcar",
    "marcareis": "marcar",
    "marcaremos": "marcar",
    "marcaren": "marcar",
    "marcares": "marcar",
    "marcaron": "marcar",
    "marcará": "marcar",
    "marcarán": "marcar",
    "marcarás": "marcar",
    "marcaré": "marcar",
    "marcaréis": "marcar",
    "marcaría": "marcar",
    "marcaríais": "marcar",
    "marcaríamos": "marcar",
    "marcarían": "marcar",
    "marcarías": "marcar",
    "marcas": "marcar",
    "marcase": "marcar",
    "marcaseis": "marcar",
    "marcasen": "marcar",
    "marcases": "marcar",
    "marcasitas": "marcasita",
    "marcaste": "marcar",
    "marcasteis": "marcar",
    "marceos": "marceo",
    "marcera": "marcero",
    "marceras": "marcero",
    "marceros": "marcero",
    "marcescentes": "marcescente",
    "marceña": "marceño",
    "marceñas": "marceño",
    "marceños": "marceño",
    "marcha": "marchar",
    "marchaba": "marchar",
    "marchabais": "marchar",
    "marchaban": "marchar",
    "marchabas": "marchar",
    "marchad": "marchar",
    "marchada": "marchar",
    "marchadas": "marchar",
    "marchado": "marchar",
    "marchadora": "marchador",
    "marchadoras": "marchador",
    "marchadores": "marchador",
    "marchados": "marchar",
    "marchama": "marchamar",
    "marchamaba": "marchamar",
    "marchamabais": "marchamar",
    "marchamaban": "marchamar",
    "marchamabas": "marchamar",
    "marchamad": "marchamar",
    "marchamada": "marchamar",
    "marchamadas": "marchamar",
    "marchamado": "marchamar",
    "marchamados": "marchamar",
    "marchamamos": "marchamar",
    "marchaman": "marchamar",
    "marchamando": "marchamar",
    "marchamara": "marchamar",
    "marchamarais": "marchamar",
    "marchamaran": "marchamar",
    "marchamaras": "marchamar",
    "marchamare": "marchamar",
    "marchamareis": "marchamar",
    "marchamaremos": "marchamar",
    "marchamaren": "marchamar",
    "marchamares": "marchamar",
    "marchamaron": "marchamar",
    "marchamará": "marchamar",
    "marchamarán": "marchamar",
    "marchamarás": "marchamar",
    "marchamaré": "marchamar",
    "marchamaréis": "marchamar",
    "marchamaría": "marchamar",
    "marchamaríais": "marchamar",
    "marchamaríamos": "marchamar",
    "marchamarían": "marchamar",
    "marchamarías": "marchamar",
    "marchamas": "marchamar",
    "marchamase": "marchamar",
    "marchamaseis": "marchamar",
    "marchamasen": "marchamar",
    "marchamases": "marchamar",
    "marchamaste": "marchamar",
    "marchamasteis": "marchamar",
    "marchame": "marchamar",
    "marchamemos": "marchamar",
    "marchamen": "marchamar",
    "marchamera": "marchamero",
    "marchameras": "marchamero",
    "marchameros": "marchamero",
    "marchames": "marchamar",
    "marchamo": "marchamar",
    "marchamos": "marchar",
    "marchamábamos": "marchamar",
    "marchamáis": "marchamar",
    "marchamáramos": "marchamar",
    "marchamáremos": "marchamar",
    "marchamásemos": "marchamar",
    "marchamé": "marchamar",
    "marchaméis": "marchamar",
    "marchamó": "marchamar",
    "marchan": "marchar",
    "marchando": "marchar",
    "marchantes": "marchante",
    "marchapiés": "marchapié",
    "marchara": "marchar",
    "marcharais": "marchar",
    "marcharan": "marchar",
    "marcharas": "marchar",
    "marchare": "marchar",
    "marchareis": "marchar",
    "marcharemos": "marchar",
    "marcharen": "marchar",
    "marchares": "marchar",
    "marcharon": "marchar",
    "marchará": "marchar",
    "marcharán": "marchar",
    "marcharás": "marchar",
    "marcharé": "marchar",
    "marcharéis": "marchar",
    "marcharía": "marchar",
    "marcharíais": "marchar",
    "marcharíamos": "marchar",
    "marcharían": "marchar",
    "marcharías": "marchar",
    "marchas": "marchar",
    "marchase": "marchar",
    "marchaseis": "marchar",
    "marchasen": "marchar",
    "marchases": "marchar",
    "marchaste": "marchar",
    "marchasteis": "marchar",
    "marche": "marchar",
    "marchemos": "marchar",
    "marchen": "marchar",
    "marches": "marchar",
    "marchita": "marchito",
    "marchitaba": "marchitar",
    "marchitabais": "marchitar",
    "marchitaban": "marchitar",
    "marchitabas": "marchitar",
    "marchitables": "marchitable",
    "marchitad": "marchitar",
    "marchitada": "marchitar",
    "marchitadas": "marchitar",
    "marchitado": "marchitar",
    "marchitados": "marchitar",
    "marchitamientos": "marchitamiento",
    "marchitamos": "marchitar",
    "marchitan": "marchitar",
    "marchitando": "marchitar",
    "marchitara": "marchitar",
    "marchitarais": "marchitar",
    "marchitaran": "marchitar",
    "marchitaras": "marchitar",
    "marchitare": "marchitar",
    "marchitareis": "marchitar",
    "marchitaremos": "marchitar",
    "marchitaren": "marchitar",
    "marchitares": "marchitar",
    "marchitaron": "marchitar",
    "marchitará": "marchitar",
    "marchitarán": "marchitar",
    "marchitarás": "marchitar",
    "marchitaré": "marchitar",
    "marchitaréis": "marchitar",
    "marchitaría": "marchitar",
    "marchitaríais": "marchitar",
    "marchitaríamos": "marchitar",
    "marchitarían": "marchitar",
    "marchitarías": "marchitar",
    "marchitas": "marchito",
    "marchitase": "marchitar",
    "marchitaseis": "marchitar",
    "marchitasen": "marchitar",
    "marchitases": "marchitar",
    "marchitaste": "marchitar",
    "marchitasteis": "marchitar",
    "marchite": "marchitar",
    "marchiteces": "marchitez",
    "marchitemos": "marchitar",
    "marchiten": "marchitar",
    "marchites": "marchitar",
    "marchito": "marchitar",
    "marchitos": "marchito",
    "marchitábamos": "marchitar",
    "marchitáis": "marchitar",
    "marchitáramos": "marchitar",
    "marchitáremos": "marchitar",
    "marchitásemos": "marchitar",
    "marchité": "marchitar",
    "marchitéis": "marchitar",
    "marchitó": "marchitar",
    "marcho": "marchar",
    "marchábamos": "marchar",
    "marcháis": "marchar",
    "marcháramos": "marchar",
    "marcháremos": "marchar",
    "marchásemos": "marchar",
    "marché": "marchar",
    "marchéis": "marchar",
    "marchó": "marchar",
    "marciales": "marcial",
    "marcialidades": "marcialidad",
    "marciana": "marciano",
    "marcianas": "marciano",
    "marcianos": "marciano",
    "marco": "marcar",
    "marcoladora": "marcolador",
    "marcoladoras": "marcolador",
    "marcoladores": "marcolador",
    "marcos": "marco",
    "marcábamos": "marcar",
    "marcáis": "marcar",
    "marcáramos": "marcar",
    "marcáremos": "marcar",
    "marcásemos": "marcar",
    "marcó": "marcar",
    "marea": "marear",
    "mareaba": "marear",
    "mareabais": "marear",
    "mareaban": "marear",
    "mareabas": "marear",
    "maread": "marear",
    "mareada": "marear",
    "mareadas": "marear",
    "mareado": "marear",
    "mareados": "marear",
    "mareajes": "mareaje",
    "mareales": "mareal",
    "mareamos": "marear",
    "marean": "marear",
    "mareando": "marear",
    "mareantes": "mareante",
    "mareara": "marear",
    "marearais": "marear",
    "marearan": "marear",
    "marearas": "marear",
    "mareare": "marear",
    "mareareis": "marear",
    "marearemos": "marear",
    "marearen": "marear",
    "mareares": "marear",
    "marearon": "marear",
    "mareará": "marear",
    "marearán": "marear",
    "marearás": "marear",
    "marearé": "marear",
    "marearéis": "marear",
    "marearía": "marear",
    "marearíais": "marear",
    "marearíamos": "marear",
    "marearían": "marear",
    "marearías": "marear",
    "mareas": "marear",
    "marease": "marear",
    "mareaseis": "marear",
    "mareasen": "marear",
    "mareases": "marear",
    "mareaste": "marear",
    "mareasteis": "marear",
    "maree": "marear",
    "mareemos": "marear",
    "mareen": "marear",
    "marees": "marear",
    "marejadas": "marejada",
    "marejadillas": "marejadilla",
    "maremagnos": "maremagno",
    "maremotores": "maremotor",
    "maremotos": "maremoto",
    "maremotrices": "maremotriz",
    "marengos": "marengo",
    "marenqueras": "marenquera",
    "mareo": "marear",
    "mareogramas": "mareograma",
    "mareológica": "mareológico",
    "mareológicas": "mareológico",
    "mareológicos": "mareológico",
    "mareomotores": "mareomotor",
    "mareomotrices": "mareomotriz",
    "mareomotriza": "mareomotor",
    "mareomotrizas": "mareomotor",
    "mareos": "mareo",
    "marera": "marero",
    "mareras": "marero",
    "mareros": "marero",
    "mares": "mar",
    "maretas": "mareta",
    "maretazos": "maretazo",
    "mareábamos": "marear",
    "mareáis": "marear",
    "mareáramos": "marear",
    "mareáremos": "marear",
    "mareásemos": "marear",
    "mareé": "marear",
    "mareéis": "marear",
    "mareó": "marear",
    "mareógrafos": "mareógrafo",
    "mareómetros": "mareómetro",
    "marfiles": "marfil",
    "marfileña": "marfileño",
    "marfileñas": "marfileño",
    "marfileños": "marfileño",
    "marfilinas": "marfilina",
    "marga": "margar",
    "margaba": "margar",
    "margabais": "margar",
    "margaban": "margar",
    "margabas": "margar",
    "margad": "margar",
    "margada": "margar",
    "margadas": "margar",
    "margado": "margar",
    "margados": "margar",
    "margales": "margal",
    "margamos": "margar",
    "margan": "margar",
    "margando": "margar",
    "margara": "margar",
    "margarais": "margar",
    "margaran": "margar",
    "margaras": "margar",
    "margare": "margar",
    "margareis": "margar",
    "margaremos": "margar",
    "margaren": "margar",
    "margares": "margar",
    "margarinas": "margarina",
    "margaritas": "margarita",
    "margaron": "margar",
    "margará": "margar",
    "margarán": "margar",
    "margarás": "margar",
    "margaré": "margar",
    "margaréis": "margar",
    "margaría": "margar",
    "margaríais": "margar",
    "margaríamos": "margar",
    "margarían": "margar",
    "margarías": "margar",
    "margas": "margar",
    "margase": "margar",
    "margaseis": "margar",
    "margasen": "margar",
    "margases": "margar",
    "margaste": "margar",
    "margasteis": "margar",
    "margina": "marginar",
    "marginaba": "marginar",
    "marginabais": "marginar",
    "marginaban": "marginar",
    "marginabas": "marginar",
    "marginaciones": "marginación",
    "marginad": "marginar",
    "marginada": "marginar",
    "marginadas": "marginar",
    "marginado": "marginar",
    "marginadores": "marginador",
    "marginados": "marginar",
    "marginales": "marginal",
    "marginalidades": "marginalidad",
    "marginalismos": "marginalismo",
    "marginalistas": "marginalista",
    "marginamos": "marginar",
    "marginan": "marginar",
    "marginando": "marginar",
    "marginara": "marginar",
    "marginarais": "marginar",
    "marginaran": "marginar",
    "marginaras": "marginar",
    "marginare": "marginar",
    "marginareis": "marginar",
    "marginaremos": "marginar",
    "marginaren": "marginar",
    "marginares": "marginar",
    "marginaron": "marginar",
    "marginará": "marginar",
    "marginarán": "marginar",
    "marginarás": "marginar",
    "marginaré": "marginar",
    "marginaréis": "marginar",
    "marginaría": "marginar",
    "marginaríais": "marginar",
    "marginaríamos": "marginar",
    "marginarían": "marginar",
    "marginarías": "marginar",
    "marginas": "marginar",
    "marginase": "marginar",
    "marginaseis": "marginar",
    "marginasen": "marginar",
    "marginases": "marginar",
    "marginaste": "marginar",
    "marginasteis": "marginar",
    "margine": "marginar",
    "marginemos": "marginar",
    "marginen": "marginar",
    "margines": "marginar",
    "margino": "marginar",
    "marginábamos": "marginar",
    "margináis": "marginar",
    "margináramos": "marginar",
    "margináremos": "marginar",
    "marginásemos": "marginar",
    "marginé": "marginar",
    "marginéis": "marginar",
    "marginó": "marginar",
    "margo": "margar",
    "margosa": "margoso",
    "margosas": "margoso",
    "margosos": "margoso",
    "margraves": "margrave",
    "margraviatos": "margraviato",
    "margue": "margar",
    "marguemos": "margar",
    "marguen": "margar",
    "margueras": "marguera",
    "margues": "margar",
    "margué": "margar",
    "marguéis": "margar",
    "margábamos": "margar",
    "margáis": "margar",
    "margáramos": "margar",
    "margáremos": "margar",
    "margásemos": "margar",
    "margó": "margar",
    "mariachis": "mariachi",
    "mariales": "marial",
    "mariana": "mariano",
    "marianas": "mariano",
    "marianos": "mariano",
    "maricas": "marica",
    "mariconadas": "mariconada",
    "mariconerías": "mariconería",
    "maricones": "maricón",
    "marida": "maridar",
    "maridaba": "maridar",
    "maridabais": "maridar",
    "maridaban": "maridar",
    "maridabas": "maridar",
    "maridad": "maridar",
    "maridada": "maridar",
    "maridadas": "maridar",
    "maridado": "maridar",
    "maridados": "maridar",
    "maridajes": "maridaje",
    "maridamos": "maridar",
    "maridan": "maridar",
    "maridando": "maridar",
    "maridara": "maridar",
    "maridarais": "maridar",
    "maridaran": "maridar",
    "maridaras": "maridar",
    "maridare": "maridar",
    "maridareis": "maridar",
    "maridaremos": "maridar",
    "maridaren": "maridar",
    "maridares": "maridar",
    "maridaron": "maridar",
    "maridará": "maridar",
    "maridarán": "maridar",
    "maridarás": "maridar",
    "maridaré": "maridar",
    "maridaréis": "maridar",
    "maridaría": "maridar",
    "maridaríais": "maridar",
    "maridaríamos": "maridar",
    "maridarían": "maridar",
    "maridarías": "maridar",
    "maridas": "maridar",
    "maridase": "maridar",
    "maridaseis": "maridar",
    "maridasen": "maridar",
    "maridases": "maridar",
    "maridaste": "maridar",
    "maridasteis": "maridar",
    "maridazos": "maridazo",
    "maride": "maridar",
    "maridemos": "maridar",
    "mariden": "maridar",
    "marides": "maridar",
    "marido": "maridar",
    "maridos": "marido",
    "maridábamos": "maridar",
    "maridáis": "maridar",
    "maridáramos": "maridar",
    "maridáremos": "maridar",
    "maridásemos": "maridar",
    "maridé": "maridar",
    "maridéis": "maridar",
    "maridó": "maridar",
    "mariguanas": "mariguana",
    "marihuanas": "marihuana",
    "marijuanas": "marijuana",
    "marimachos": "marimacho",
    "marimandona": "marimandón",
    "marimandonas": "marimandón",
    "marimandones": "marimandón",
    "marimantas": "marimanta",
    "marimaricas": "marimarica",
    "marimbas": "marimba",
    "marimbera": "marimbero",
    "marimberas": "marimbero",
    "marimberos": "marimbero",
    "marimorenas": "marimorena",
    "marina": "marino",
    "marinaba": "marinar",
    "marinabais": "marinar",
    "marinaban": "marinar",
    "marinabas": "marinar",
    "marinad": "marinar",
    "marinada": "marinar",
    "marinadas": "marinar",
    "marinado": "marinar",
    "marinados": "marinar",
    "marinamos": "marinar",
    "marinan": "marinar",
    "marinando": "marinar",
    "marinara": "marinar",
    "marinarais": "marinar",
    "marinaran": "marinar",
    "marinaras": "marinar",
    "marinare": "marinar",
    "marinareis": "marinar",
    "marinaremos": "marinar",
    "marinaren": "marinar",
    "marinares": "marinar",
    "marinaron": "marinar",
    "marinará": "marinar",
    "marinarán": "marinar",
    "marinarás": "marinar",
    "marinaré": "marinar",
    "marinaréis": "marinar",
    "marinaría": "marinar",
    "marinaríais": "marinar",
    "marinaríamos": "marinar",
    "marinarían": "marinar",
    "marinarías": "marinar",
    "marinas": "marino",
    "marinase": "marinar",
    "marinaseis": "marinar",
    "marinasen": "marinar",
    "marinases": "marinar",
    "marinaste": "marinar",
    "marinasteis": "marinar",
    "marine": "marinar",
    "marinea": "marinear",
    "marineaba": "marinear",
    "marineabais": "marinear",
    "marineaban": "marinear",
    "marineabas": "marinear",
    "marinead": "marinear",
    "marineada": "marinear",
    "marineadas": "marinear",
    "marineado": "marinear",
    "marineados": "marinear",
    "marineamos": "marinear",
    "marinean": "marinear",
    "marineando": "marinear",
    "marineara": "marinear",
    "marinearais": "marinear",
    "marinearan": "marinear",
    "marinearas": "marinear",
    "marineare": "marinear",
    "marineareis": "marinear",
    "marinearemos": "marinear",
    "marinearen": "marinear",
    "marineares": "marinear",
    "marinearon": "marinear",
    "marineará": "marinear",
    "marinearán": "marinear",
    "marinearás": "marinear",
    "marinearé": "marinear",
    "marinearéis": "marinear",
    "marinearía": "marinear",
    "marinearíais": "marinear",
    "marinearíamos": "marinear",
    "marinearían": "marinear",
    "marinearías": "marinear",
    "marineas": "marinear",
    "marinease": "marinear",
    "marineaseis": "marinear",
    "marineasen": "marinear",
    "marineases": "marinear",
    "marineaste": "marinear",
    "marineasteis": "marinear",
    "marinee": "marinear",
    "marineemos": "marinear",
    "marineen": "marinear",
    "marinees": "marinear",
    "marinemos": "marinar",
    "marinen": "marinar",
    "marineo": "marinear",
    "marinera": "marinero",
    "marinerada": "marinerado",
    "marineradas": "marinerado",
    "marinerados": "marinerado",
    "marineras": "marinero",
    "marinerazos": "marinerazo",
    "marineros": "marinero",
    "marinerías": "marinería",
    "marines": "marine",
    "marineábamos": "marinear",
    "marineáis": "marinear",
    "marineáramos": "marinear",
    "marineáremos": "marinear",
    "marineásemos": "marinear",
    "marineé": "marinear",
    "marineéis": "marinear",
    "marineó": "marinear",
    "marinistas": "marinista",
    "marino": "marinar",
    "marinos": "marino",
    "marinábamos": "marinar",
    "marináis": "marinar",
    "marináramos": "marinar",
    "marináremos": "marinar",
    "marinásemos": "marinar",
    "mariné": "marinar",
    "marinéis": "marinar",
    "mariníes": "mariní",
    "marinó": "marinar",
    "mariolatrías": "mariolatría",
    "mariologías": "mariología",
    "marionetas": "marioneta",
    "mariposas": "mariposa",
    "mariposea": "mariposear",
    "mariposeaba": "mariposear",
    "mariposeabais": "mariposear",
    "mariposeaban": "mariposear",
    "mariposeabas": "mariposear",
    "mariposead": "mariposear",
    "mariposeada": "mariposear",
    "mariposeadas": "mariposear",
    "mariposeado": "mariposear",
    "mariposeados": "mariposear",
    "mariposeamos": "mariposear",
    "mariposean": "mariposear",
    "mariposeando": "mariposear",
    "mariposeara": "mariposear",
    "mariposearais": "mariposear",
    "mariposearan": "mariposear",
    "mariposearas": "mariposear",
    "mariposeare": "mariposear",
    "mariposeareis": "mariposear",
    "mariposearemos": "mariposear",
    "mariposearen": "mariposear",
    "mariposeares": "mariposear",
    "mariposearon": "mariposear",
    "mariposeará": "mariposear",
    "mariposearán": "mariposear",
    "mariposearás": "mariposear",
    "mariposearé": "mariposear",
    "mariposearéis": "mariposear",
    "mariposearía": "mariposear",
    "mariposearíais": "mariposear",
    "mariposearíamos": "mariposear",
    "mariposearían": "mariposear",
    "mariposearías": "mariposear",
    "mariposeas": "mariposear",
    "mariposease": "mariposear",
    "mariposeaseis": "mariposear",
    "mariposeasen": "mariposear",
    "mariposeases": "mariposear",
    "mariposeaste": "mariposear",
    "mariposeasteis": "mariposear",
    "mariposee": "mariposear",
    "mariposeemos": "mariposear",
    "mariposeen": "mariposear",
    "mariposees": "mariposear",
    "mariposeo": "mariposear",
    "mariposeos": "mariposeo",
    "mariposeábamos": "mariposear",
    "mariposeáis": "mariposear",
    "mariposeáramos": "mariposear",
    "mariposeáremos": "mariposear",
    "mariposeásemos": "mariposear",
    "mariposeé": "mariposear",
    "mariposeéis": "mariposear",
    "mariposeó": "mariposear",
    "mariposones": "mariposón",
    "mariquitas": "mariquita",
    "marisabidillas": "marisabidilla",
    "marisca": "mariscar",
    "mariscaba": "mariscar",
    "mariscabais": "mariscar",
    "mariscaban": "mariscar",
    "mariscabas": "mariscar",
    "mariscad": "mariscar",
    "mariscada": "mariscar",
    "mariscadas": "mariscar",
    "mariscado": "mariscar",
    "mariscadora": "mariscador",
    "mariscadoras": "mariscador",
    "mariscadores": "mariscador",
    "mariscados": "mariscar",
    "mariscalas": "mariscala",
    "mariscalatos": "mariscalato",
    "mariscales": "mariscal",
    "mariscalías": "mariscalía",
    "mariscamos": "mariscar",
    "mariscan": "mariscar",
    "mariscando": "mariscar",
    "mariscara": "mariscar",
    "mariscarais": "mariscar",
    "mariscaran": "mariscar",
    "mariscaras": "mariscar",
    "mariscare": "mariscar",
    "mariscareis": "mariscar",
    "mariscaremos": "mariscar",
    "mariscaren": "mariscar",
    "mariscares": "mariscar",
    "mariscaron": "mariscar",
    "mariscará": "mariscar",
    "mariscarán": "mariscar",
    "mariscarás": "mariscar",
    "mariscaré": "mariscar",
    "mariscaréis": "mariscar",
    "mariscaría": "mariscar",
    "mariscaríais": "mariscar",
    "mariscaríamos": "mariscar",
    "mariscarían": "mariscar",
    "mariscarías": "mariscar",
    "mariscas": "mariscar",
    "mariscase": "mariscar",
    "mariscaseis": "mariscar",
    "mariscasen": "mariscar",
    "mariscases": "mariscar",
    "mariscaste": "mariscar",
    "mariscasteis": "mariscar",
    "marisco": "mariscar",
    "mariscos": "marisco",
    "mariscábamos": "mariscar",
    "mariscáis": "mariscar",
    "mariscáramos": "mariscar",
    "mariscáremos": "mariscar",
    "mariscásemos": "mariscar",
    "mariscó": "mariscar",
    "marismas": "marisma",
    "marismeña": "marismeño",
    "marismeñas": "marismeño",
    "marismeños": "marismeño",
    "marisque": "mariscar",
    "marisquemos": "mariscar",
    "marisquen": "mariscar",
    "marisquera": "marisquero",
    "marisqueras": "marisquero",
    "marisqueros": "marisquero",
    "marisquerías": "marisquería",
    "marisques": "mariscar",
    "marisqué": "mariscar",
    "marisquéis": "mariscar",
    "maristas": "marista",
    "maritales": "marital",
    "marióloga": "mariólogo",
    "mariólogas": "mariólogo",
    "mariólogos": "mariólogo",
    "marjales": "marjal",
    "marjoletos": "marjoleto",
    "marjores": "marjor",
    "marmellas": "marmella",
    "marmitas": "marmita",
    "marmitona": "marmitón",
    "marmitonas": "marmitón",
    "marmitones": "marmitón",
    "marmolejos": "marmolejo",
    "marmolerías": "marmolería",
    "marmoleña": "marmoleño",
    "marmoleñas": "marmoleño",
    "marmoleños": "marmoleño",
    "marmolillos": "marmolillo",
    "marmolistas": "marmolista",
    "marmoraciones": "marmoración",
    "marmorizaciones": "marmorización",
    "marmorosa": "marmoroso",
    "marmorosas": "marmoroso",
    "marmorosos": "marmoroso",
    "marmosas": "marmosa",
    "marmosetes": "marmosete",
    "marmotas": "marmota",
    "marmólea": "marmóleo",
    "marmóleas": "marmóleo",
    "marmóleos": "marmóleo",
    "marmórea": "marmóreo",
    "marmóreas": "marmóreo",
    "marmóreos": "marmóreo",
    "marojales": "marojal",
    "marojos": "marojo",
    "marolas": "marola",
    "maromas": "maroma",
    "maronitas": "maronita",
    "marque": "marcar",
    "marquemos": "marcar",
    "marquen": "marcar",
    "marques": "marcar",
    "marquesa": "marqués",
    "marquesados": "marquesado",
    "marquesas": "marqués",
    "marqueses": "marqués",
    "marquesinas": "marquesina",
    "marquetas": "marqueta",
    "marqueterías": "marquetería",
    "marquistas": "marquista",
    "marqué": "marcar",
    "marquéis": "marcar",
    "marra": "marrar",
    "marraba": "marrar",
    "marrabais": "marrar",
    "marraban": "marrar",
    "marrabas": "marrar",
    "marrad": "marrar",
    "marrada": "marrar",
    "marradas": "marrar",
    "marrado": "marrar",
    "marrados": "marrar",
    "marraja": "marrajo",
    "marrajas": "marrajo",
    "marrajos": "marrajo",
    "marramaos": "marramao",
    "marramos": "marrar",
    "marramáus": "marramáu",
    "marran": "marrar",
    "marrana": "marrano",
    "marranadas": "marranada",
    "marranas": "marrano",
    "marrando": "marrar",
    "marranerías": "marranería",
    "marranos": "marrano",
    "marrara": "marrar",
    "marrarais": "marrar",
    "marraran": "marrar",
    "marraras": "marrar",
    "marrare": "marrar",
    "marrareis": "marrar",
    "marraremos": "marrar",
    "marraren": "marrar",
    "marrares": "marrar",
    "marraron": "marrar",
    "marrará": "marrar",
    "marrarán": "marrar",
    "marrarás": "marrar",
    "marraré": "marrar",
    "marraréis": "marrar",
    "marraría": "marrar",
    "marraríais": "marrar",
    "marraríamos": "marrar",
    "marrarían": "marrar",
    "marrarías": "marrar",
    "marras": "marrar",
    "marrase": "marrar",
    "marraseis": "marrar",
    "marrasen": "marrar",
    "marrases": "marrar",
    "marrasquinos": "marrasquino",
    "marraste": "marrar",
    "marrasteis": "marrar",
    "marrazos": "marrazo",
    "marre": "marrar",
    "marremos": "marrar",
    "marren": "marrar",
    "marres": "marrar",
    "marrillos": "marrillo",
    "marro": "marrar",
    "marrones": "marrón",
    "marronácea": "marronáceo",
    "marronáceas": "marronáceo",
    "marronáceos": "marronáceo",
    "marroquina": "marroquín",
    "marroquinas": "marroquín",
    "marroquinera": "marroquinero",
    "marroquineras": "marroquinero",
    "marroquineros": "marroquinero",
    "marroquinerías": "marroquinería",
    "marroquines": "marroquín",
    "marroquíes": "marroquí",
    "marroquín": "marroquí",
    "marroquís": "marroquí",
    "marroquíses": "marroquí",
    "marros": "marro",
    "marrubiales": "marrubial",
    "marrubios": "marrubio",
    "marrueca": "marrueco",
    "marruecas": "marrueco",
    "marrueco": "marroquí",
    "marruecos": "marrueco",
    "marrullera": "marrullero",
    "marrulleras": "marrullero",
    "marrulleros": "marrullero",
    "marrullerías": "marrullería",
    "marrábamos": "marrar",
    "marráis": "marrar",
    "marráramos": "marrar",
    "marráremos": "marrar",
    "marrásemos": "marrar",
    "marré": "marrar",
    "marréis": "marrar",
    "marró": "marrar",
    "marsellesa": "marsellés",
    "marsellesas": "marsellés",
    "marselleses": "marsellés",
    "marsileáceas": "marsileácea",
    "marsopas": "marsopa",
    "marsupiales": "marsupial",
    "marsupios": "marsupio",
    "martagona": "martagón",
    "martagonas": "martagón",
    "martagones": "martagón",
    "martas": "marta",
    "martellinas": "martellina",
    "martemples": "martemple",
    "martensitas": "martensita",
    "martensítica": "martensítico",
    "martensíticas": "martensítico",
    "martensíticos": "martensítico",
    "martilla": "martillar",
    "martillaba": "martillar",
    "martillabais": "martillar",
    "martillaban": "martillar",
    "martillabas": "martillar",
    "martillad": "martillar",
    "martillada": "martillar",
    "martilladas": "martillar",
    "martillado": "martillar",
    "martilladora": "martillador",
    "martilladoras": "martillador",
    "martilladores": "martillador",
    "martillados": "martillar",
    "martillamos": "martillar",
    "martillan": "martillar",
    "martillando": "martillar",
    "martillara": "martillar",
    "martillarais": "martillar",
    "martillaran": "martillar",
    "martillaras": "martillar",
    "martillare": "martillar",
    "martillareis": "martillar",
    "martillaremos": "martillar",
    "martillaren": "martillar",
    "martillares": "martillar",
    "martillaron": "martillar",
    "martillará": "martillar",
    "martillarán": "martillar",
    "martillarás": "martillar",
    "martillaré": "martillar",
    "martillaréis": "martillar",
    "martillaría": "martillar",
    "martillaríais": "martillar",
    "martillaríamos": "martillar",
    "martillarían": "martillar",
    "martillarías": "martillar",
    "martillas": "martillar",
    "martillase": "martillar",
    "martillaseis": "martillar",
    "martillasen": "martillar",
    "martillases": "martillar",
    "martillaste": "martillar",
    "martillasteis": "martillar",
    "martille": "martillar",
    "martillea": "martillear",
    "martilleaba": "martillear",
    "martilleabais": "martillear",
    "martilleaban": "martillear",
    "martilleabas": "martillear",
    "martillead": "martillear",
    "martilleada": "martillear",
    "martilleadas": "martillear",
    "martilleado": "martillear",
    "martilleados": "martillear",
    "martilleamos": "martillear",
    "martillean": "martillear",
    "martilleando": "martillear",
    "martilleara": "martillear",
    "martillearais": "martillear",
    "martillearan": "martillear",
    "martillearas": "martillear",
    "martilleare": "martillear",
    "martilleareis": "martillear",
    "martillearemos": "martillear",
    "martillearen": "martillear",
    "martilleares": "martillear",
    "martillearon": "martillear",
    "martilleará": "martillear",
    "martillearán": "martillear",
    "martillearás": "martillear",
    "martillearé": "martillear",
    "martillearéis": "martillear",
    "martillearía": "martillear",
    "martillearíais": "martillear",
    "martillearíamos": "martillear",
    "martillearían": "martillear",
    "martillearías": "martillear",
    "martilleas": "martillear",
    "martillease": "martillear",
    "martilleaseis": "martillear",
    "martilleasen": "martillear",
    "martilleases": "martillear",
    "martilleaste": "martillear",
    "martilleasteis": "martillear",
    "martillee": "martillear",
    "martilleemos": "martillear",
    "martilleen": "martillear",
    "martillees": "martillear",
    "martillemos": "martillar",
    "martillen": "martillar",
    "martilleo": "martillear",
    "martilleos": "martilleo",
    "martilles": "martillar",
    "martilletes": "martillete",
    "martilleábamos": "martillear",
    "martilleáis": "martillear",
    "martilleáramos": "martillear",
    "martilleáremos": "martillear",
    "martilleásemos": "martillear",
    "martilleé": "martillear",
    "martilleéis": "martillear",
    "martilleó": "martillear",
    "martillo": "martillar",
    "martillos": "martillo",
    "martillábamos": "martillar",
    "martilláis": "martillar",
    "martilláramos": "martillar",
    "martilláremos": "martillar",
    "martillásemos": "martillar",
    "martillé": "martillar",
    "martilléis": "martillar",
    "martilló": "martillar",
    "martinetes": "martinete",
    "martingalas": "martingala",
    "martinicos": "martinico",
    "martiriales": "martirial",
    "martirice": "martirizar",
    "martiricemos": "martirizar",
    "martiricen": "martirizar",
    "martirices": "martirizar",
    "martiricé": "martirizar",
    "martiricéis": "martirizar",
    "martirios": "martirio",
    "martiriza": "martirizar",
    "martirizaba": "martirizar",
    "martirizabais": "martirizar",
    "martirizaban": "martirizar",
    "martirizabas": "martirizar",
    "martirizad": "martirizar",
    "martirizada": "martirizar",
    "martirizadas": "martirizar",
    "martirizado": "martirizar",
    "martirizados": "martirizar",
    "martirizamos": "martirizar",
    "martirizan": "martirizar",
    "martirizando": "martirizar",
    "martirizara": "martirizar",
    "martirizarais": "martirizar",
    "martirizaran": "martirizar",
    "martirizaras": "martirizar",
    "martirizare": "martirizar",
    "martirizareis": "martirizar",
    "martirizaremos": "martirizar",
    "martirizaren": "martirizar",
    "martirizares": "martirizar",
    "martirizaron": "martirizar",
    "martirizará": "martirizar",
    "martirizarán": "martirizar",
    "martirizarás": "martirizar",
    "martirizaré": "martirizar",
    "martirizaréis": "martirizar",
    "martirizaría": "martirizar",
    "martirizaríais": "martirizar",
    "martirizaríamos": "martirizar",
    "martirizarían": "martirizar",
    "martirizarías": "martirizar",
    "martirizas": "martirizar",
    "martirizase": "martirizar",
    "martirizaseis": "martirizar",
    "martirizasen": "martirizar",
    "martirizases": "martirizar",
    "martirizaste": "martirizar",
    "martirizasteis": "martirizar",
    "martirizo": "martirizar",
    "martirizábamos": "martirizar",
    "martirizáis": "martirizar",
    "martirizáramos": "martirizar",
    "martirizáremos": "martirizar",
    "martirizásemos": "martirizar",
    "martirizó": "martirizar",
    "martirologios": "martirologio",
    "maruecos": "marueco",
    "marullos": "marullo",
    "marxismoleninismos": "marxismoleninismo",
    "marxismos": "marxismo",
    "marxistaleninistas": "marxistaleninista",
    "marxistas": "marxista",
    "marzales": "marzal",
    "marzos": "marzo",
    "marás": "mará",
    "marías": "maría",
    "marítima": "marítimo",
    "marítimas": "marítimo",
    "marítimos": "marítimo",
    "masacra": "masacrar",
    "masacraba": "masacrar",
    "masacrabais": "masacrar",
    "masacraban": "masacrar",
    "masacrabas": "masacrar",
    "masacrad": "masacrar",
    "masacrada": "masacrar",
    "masacradas": "masacrar",
    "masacrado": "masacrar",
    "masacrados": "masacrar",
    "masacramos": "masacrar",
    "masacran": "masacrar",
    "masacrando": "masacrar",
    "masacrara": "masacrar",
    "masacrarais": "masacrar",
    "masacraran": "masacrar",
    "masacraras": "masacrar",
    "masacrare": "masacrar",
    "masacrareis": "masacrar",
    "masacraremos": "masacrar",
    "masacraren": "masacrar",
    "masacrares": "masacrar",
    "masacraron": "masacrar",
    "masacrará": "masacrar",
    "masacrarán": "masacrar",
    "masacrarás": "masacrar",
    "masacraré": "masacrar",
    "masacraréis": "masacrar",
    "masacraría": "masacrar",
    "masacraríais": "masacrar",
    "masacraríamos": "masacrar",
    "masacrarían": "masacrar",
    "masacrarías": "masacrar",
    "masacras": "masacrar",
    "masacrase": "masacrar",
    "masacraseis": "masacrar",
    "masacrasen": "masacrar",
    "masacrases": "masacrar",
    "masacraste": "masacrar",
    "masacrasteis": "masacrar",
    "masacre": "masacrar",
    "masacremos": "masacrar",
    "masacren": "masacrar",
    "masacres": "masacre",
    "masacro": "masacrar",
    "masacrábamos": "masacrar",
    "masacráis": "masacrar",
    "masacráramos": "masacrar",
    "masacráremos": "masacrar",
    "masacrásemos": "masacrar",
    "masacré": "masacrar",
    "masacréis": "masacrar",
    "masacró": "masacrar",
    "masadas": "masada",
    "masaderos": "masadero",
    "masajes": "masaje",
    "masajistas": "masajista",
    "masas": "masa",
    "masca": "mascar",
    "mascaba": "mascar",
    "mascabada": "mascabado",
    "mascabadas": "mascabado",
    "mascabados": "mascabado",
    "mascabais": "mascar",
    "mascaban": "mascar",
    "mascabas": "mascar",
    "mascad": "mascar",
    "mascada": "mascar",
    "mascadas": "mascar",
    "mascado": "mascar",
    "mascadora": "mascador",
    "mascadoras": "mascador",
    "mascadores": "mascador",
    "mascados": "mascar",
    "mascaduras": "mascadura",
    "mascamos": "mascar",
    "mascan": "mascar",
    "mascando": "mascar",
    "mascara": "mascar",
    "mascaradas": "mascarada",
    "mascarais": "mascar",
    "mascaran": "mascar",
    "mascaras": "mascar",
    "mascare": "mascar",
    "mascareis": "mascar",
    "mascaremos": "mascar",
    "mascaren": "mascar",
    "mascarera": "mascarero",
    "mascareras": "mascarero",
    "mascareros": "mascarero",
    "mascares": "mascar",
    "mascarillas": "mascarilla",
    "mascaron": "mascar",
    "mascarones": "mascarón",
    "mascará": "mascar",
    "mascarán": "mascar",
    "mascarás": "mascar",
    "mascaré": "mascar",
    "mascaréis": "mascar",
    "mascaría": "mascar",
    "mascaríais": "mascar",
    "mascaríamos": "mascar",
    "mascarían": "mascar",
    "mascarías": "mascar",
    "mascas": "mascar",
    "mascase": "mascar",
    "mascaseis": "mascar",
    "mascasen": "mascar",
    "mascases": "mascar",
    "mascaste": "mascar",
    "mascasteis": "mascar",
    "masco": "mascar",
    "mascotas": "mascota",
    "mascotines": "mascotín",
    "mascuja": "mascujar",
    "mascujaba": "mascujar",
    "mascujabais": "mascujar",
    "mascujaban": "mascujar",
    "mascujabas": "mascujar",
    "mascujad": "mascujar",
    "mascujada": "mascujar",
    "mascujadas": "mascujar",
    "mascujado": "mascujar",
    "mascujados": "mascujar",
    "mascujamos": "mascujar",
    "mascujan": "mascujar",
    "mascujando": "mascujar",
    "mascujara": "mascujar",
    "mascujarais": "mascujar",
    "mascujaran": "mascujar",
    "mascujaras": "mascujar",
    "mascujare": "mascujar",
    "mascujareis": "mascujar",
    "mascujaremos": "mascujar",
    "mascujaren": "mascujar",
    "mascujares": "mascujar",
    "mascujaron": "mascujar",
    "mascujará": "mascujar",
    "mascujarán": "mascujar",
    "mascujarás": "mascujar",
    "mascujaré": "mascujar",
    "mascujaréis": "mascujar",
    "mascujaría": "mascujar",
    "mascujaríais": "mascujar",
    "mascujaríamos": "mascujar",
    "mascujarían": "mascujar",
    "mascujarías": "mascujar",
    "mascujas": "mascujar",
    "mascujase": "mascujar",
    "mascujaseis": "mascujar",
    "mascujasen": "mascujar",
    "mascujases": "mascujar",
    "mascujaste": "mascujar",
    "mascujasteis": "mascujar",
    "mascuje": "mascujar",
    "mascujemos": "mascujar",
    "mascujen": "mascujar",
    "mascujes": "mascujar",
    "mascujo": "mascujar",
    "mascujábamos": "mascujar",
    "mascujáis": "mascujar",
    "mascujáramos": "mascujar",
    "mascujáremos": "mascujar",
    "mascujásemos": "mascujar",
    "mascujé": "mascujar",
    "mascujéis": "mascujar",
    "mascujó": "mascujar",
    "masculillos": "masculillo",
    "masculina": "masculino",
    "masculinas": "masculino",
    "masculinidades": "masculinidad",
    "masculinos": "masculino",
    "masculla": "mascullar",
    "mascullaba": "mascullar",
    "mascullabais": "mascullar",
    "mascullaban": "mascullar",
    "mascullabas": "mascullar",
    "mascullad": "mascullar",
    "mascullada": "mascullar",
    "masculladas": "mascullar",
    "mascullado": "mascullar",
    "mascullados": "mascullar",
    "mascullamos": "mascullar",
    "mascullan": "mascullar",
    "mascullando": "mascullar",
    "mascullara": "mascullar",
    "mascullarais": "mascullar",
    "mascullaran": "mascullar",
    "mascullaras": "mascullar",
    "mascullare": "mascullar",
    "mascullareis": "mascullar",
    "mascullaremos": "mascullar",
    "mascullaren": "mascullar",
    "mascullares": "mascullar",
    "mascullaron": "mascullar",
    "mascullará": "mascullar",
    "mascullarán": "mascullar",
    "mascullarás": "mascullar",
    "mascullaré": "mascullar",
    "mascullaréis": "mascullar",
    "mascullaría": "mascullar",
    "mascullaríais": "mascullar",
    "mascullaríamos": "mascullar",
    "mascullarían": "mascullar",
    "mascullarías": "mascullar",
    "mascullas": "mascullar",
    "mascullase": "mascullar",
    "mascullaseis": "mascullar",
    "mascullasen": "mascullar",
    "mascullases": "mascullar",
    "mascullaste": "mascullar",
    "mascullasteis": "mascullar",
    "masculle": "mascullar",
    "mascullemos": "mascullar",
    "mascullen": "mascullar",
    "masculles": "mascullar",
    "mascullo": "mascullar",
    "mascullábamos": "mascullar",
    "masculláis": "mascullar",
    "masculláramos": "mascullar",
    "masculláremos": "mascullar",
    "mascullásemos": "mascullar",
    "mascullé": "mascullar",
    "masculléis": "mascullar",
    "masculló": "mascullar",
    "mascábamos": "mascar",
    "mascáis": "mascar",
    "mascáramos": "mascar",
    "mascáremos": "mascar",
    "mascásemos": "mascar",
    "mascó": "mascar",
    "maseras": "masera",
    "mases": "mas",
    "masetera": "masetero",
    "maseteras": "masetero",
    "maseteros": "masetero",
    "masetérica": "masetérico",
    "masetéricas": "masetérico",
    "masetéricos": "masetérico",
    "masicotes": "masicote",
    "masifica": "masificar",
    "masificaba": "masificar",
    "masificabais": "masificar",
    "masificaban": "masificar",
    "masificabas": "masificar",
    "masificaciones": "masificación",
    "masificad": "masificar",
    "masificada": "masificar",
    "masificadas": "masificar",
    "masificado": "masificar",
    "masificados": "masificar",
    "masificamos": "masificar",
    "masifican": "masificar",
    "masificando": "masificar",
    "masificara": "masificar",
    "masificarais": "masificar",
    "masificaran": "masificar",
    "masificaras": "masificar",
    "masificare": "masificar",
    "masificareis": "masificar",
    "masificaremos": "masificar",
    "masificaren": "masificar",
    "masificares": "masificar",
    "masificaron": "masificar",
    "masificará": "masificar",
    "masificarán": "masificar",
    "masificarás": "masificar",
    "masificaré": "masificar",
    "masificaréis": "masificar",
    "masificaría": "masificar",
    "masificaríais": "masificar",
    "masificaríamos": "masificar",
    "masificarían": "masificar",
    "masificarías": "masificar",
    "masificas": "masificar",
    "masificase": "masificar",
    "masificaseis": "masificar",
    "masificasen": "masificar",
    "masificases": "masificar",
    "masificaste": "masificar",
    "masificasteis": "masificar",
    "masifico": "masificar",
    "masificábamos": "masificar",
    "masificáis": "masificar",
    "masificáramos": "masificar",
    "masificáremos": "masificar",
    "masificásemos": "masificar",
    "masificó": "masificar",
    "masifique": "masificar",
    "masifiquemos": "masificar",
    "masifiquen": "masificar",
    "masifiques": "masificar",
    "masifiqué": "masificar",
    "masifiquéis": "masificar",
    "masillas": "masilla",
    "masitas": "masita",
    "masiva": "masivo",
    "masivas": "masivo",
    "masivos": "masivo",
    "maslos": "maslo",
    "masona": "masón",
    "masonas": "masón",
    "masonerías": "masonería",
    "masones": "masón",
    "masoquismos": "masoquismo",
    "masoquistas": "masoquista",
    "masoras": "masora",
    "masoretas": "masoreta",
    "masorética": "masorético",
    "masoréticas": "masorético",
    "masoréticos": "masorético",
    "masque": "mascar",
    "masquemos": "mascar",
    "masquen": "mascar",
    "masques": "mascar",
    "masqué": "mascar",
    "masquéis": "mascar",
    "mastabas": "mastaba",
    "mastectomías": "mastectomía",
    "mastelerillos": "mastelerillo",
    "masteleros": "mastelero",
    "masteles": "mastel",
    "masters": "master",
    "mastica": "masticar",
    "masticaba": "masticar",
    "masticabais": "masticar",
    "masticaban": "masticar",
    "masticabas": "masticar",
    "masticaciones": "masticación",
    "masticad": "masticar",
    "masticada": "masticar",
    "masticadas": "masticar",
    "masticado": "masticar",
    "masticadora": "masticador",
    "masticadoras": "masticador",
    "masticadores": "masticador",
    "masticados": "masticar",
    "masticamos": "masticar",
    "mastican": "masticar",
    "masticando": "masticar",
    "masticara": "masticar",
    "masticarais": "masticar",
    "masticaran": "masticar",
    "masticaras": "masticar",
    "masticare": "masticar",
    "masticareis": "masticar",
    "masticaremos": "masticar",
    "masticaren": "masticar",
    "masticares": "masticar",
    "masticaron": "masticar",
    "masticará": "masticar",
    "masticarán": "masticar",
    "masticarás": "masticar",
    "masticaré": "masticar",
    "masticaréis": "masticar",
    "masticaría": "masticar",
    "masticaríais": "masticar",
    "masticaríamos": "masticar",
    "masticarían": "masticar",
    "masticarías": "masticar",
    "masticas": "masticar",
    "masticase": "masticar",
    "masticaseis": "masticar",
    "masticasen": "masticar",
    "masticases": "masticar",
    "masticaste": "masticar",
    "masticasteis": "masticar",
    "masticatoria": "masticatorio",
    "masticatorias": "masticatorio",
    "masticatorios": "masticatorio",
    "masticina": "masticino",
    "masticinas": "masticino",
    "masticinos": "masticino",
    "mastico": "masticar",
    "masticábamos": "masticar",
    "masticáis": "masticar",
    "masticáramos": "masticar",
    "masticáremos": "masticar",
    "masticásemos": "masticar",
    "masticó": "masticar",
    "mastigadores": "mastigador",
    "mastina": "mastín",
    "mastinas": "mastín",
    "mastines": "mastín",
    "mastique": "masticar",
    "mastiqueaciones": "mastiqueación",
    "mastiquemos": "masticar",
    "mastiquen": "masticar",
    "mastiques": "mastique",
    "mastiqué": "masticar",
    "mastiquéis": "masticar",
    "mastocitos": "mastocito",
    "mastodontes": "mastodonte",
    "mastodóntica": "mastodóntico",
    "mastodónticas": "mastodóntico",
    "mastodónticos": "mastodóntico",
    "mastografías": "mastografía",
    "mastoidalgias": "mastoidalgia",
    "mastoidea": "mastoideo",
    "mastoideas": "mastoideo",
    "mastoidectomías": "mastoidectomía",
    "mastoideos": "mastoideo",
    "mastoidotimpanectomías": "mastoidotimpanectomía",
    "mastoidotomías": "mastoidotomía",
    "mastooccipitales": "mastooccipital",
    "mastoparietales": "mastoparietal",
    "mastopexias": "mastopexia",
    "mastoplastias": "mastoplastia",
    "mastuerzos": "mastuerzo",
    "masturba": "masturbar",
    "masturbaba": "masturbar",
    "masturbabais": "masturbar",
    "masturbaban": "masturbar",
    "masturbabas": "masturbar",
    "masturbaciones": "masturbación",
    "masturbad": "masturbar",
    "masturbada": "masturbar",
    "masturbadas": "masturbar",
    "masturbado": "masturbar",
    "masturbados": "masturbar",
    "masturbamos": "masturbar",
    "masturban": "masturbar",
    "masturbando": "masturbar",
    "masturbara": "masturbar",
    "masturbarais": "masturbar",
    "masturbaran": "masturbar",
    "masturbaras": "masturbar",
    "masturbare": "masturbar",
    "masturbareis": "masturbar",
    "masturbaremos": "masturbar",
    "masturbaren": "masturbar",
    "masturbares": "masturbar",
    "masturbaron": "masturbar",
    "masturbará": "masturbar",
    "masturbarán": "masturbar",
    "masturbarás": "masturbar",
    "masturbaré": "masturbar",
    "masturbaréis": "masturbar",
    "masturbaría": "masturbar",
    "masturbaríais": "masturbar",
    "masturbaríamos": "masturbar",
    "masturbarían": "masturbar",
    "masturbarías": "masturbar",
    "masturbas": "masturbar",
    "masturbase": "masturbar",
    "masturbaseis": "masturbar",
    "masturbasen": "masturbar",
    "masturbases": "masturbar",
    "masturbaste": "masturbar",
    "masturbasteis": "masturbar",
    "masturbe": "masturbar",
    "masturbemos": "masturbar",
    "masturben": "masturbar",
    "masturbes": "masturbar",
    "masturbo": "masturbar",
    "masturbábamos": "masturbar",
    "masturbáis": "masturbar",
    "masturbáramos": "masturbar",
    "masturbáremos": "masturbar",
    "masturbásemos": "masturbar",
    "masturbé": "masturbar",
    "masturbéis": "masturbar",
    "masturbó": "masturbar",
    "masvaviscos": "masvavisco",
    "masías": "masía",
    "masónica": "masónico",
    "masónicas": "masónico",
    "masónicos": "masónico",
    "mata": "matar",
    "mataba": "matar",
    "matabais": "matar",
    "mataban": "matar",
    "matabas": "matar",
    "matabueyes": "matabuey",
    "matacanes": "matacán",
    "matachines": "matachín",
    "matad": "matar",
    "matada": "matar",
    "matadas": "matar",
    "mataderos": "matadero",
    "matado": "matar",
    "matadora": "matador",
    "matadoras": "matador",
    "matadores": "matador",
    "matados": "matar",
    "mataduras": "matadura",
    "matalahugas": "matalahuga",
    "matalahuvas": "matalahuva",
    "matalahúgas": "matalahúga",
    "matalahúvas": "matalahúva",
    "matalona": "matalón",
    "matalonas": "matalón",
    "matalones": "matalón",
    "matalotajes": "matalotaje",
    "matalotes": "matalote",
    "matamatas": "matamata",
    "matamos": "matar",
    "matan": "matar",
    "matanceros": "matancero",
    "matando": "matar",
    "matanzas": "matanza",
    "matapalos": "matapalo",
    "matapardas": "mataparda",
    "matapolvos": "matapolvo",
    "matara": "matar",
    "matarais": "matar",
    "mataran": "matar",
    "mataras": "matar",
    "matare": "matar",
    "matareis": "matar",
    "mataremos": "matar",
    "mataren": "matar",
    "matares": "matar",
    "matarifes": "matarife",
    "mataron": "matar",
    "mataronenses": "mataronense",
    "mataronesa": "mataronés",
    "mataronesas": "mataronés",
    "mataroneses": "mataronés",
    "mataronésa": "mataronés",
    "mataronésas": "mataronés",
    "mataronéses": "mataronés",
    "matará": "matar",
    "matarán": "matar",
    "matarás": "matar",
    "mataré": "matar",
    "mataréis": "matar",
    "mataría": "matar",
    "mataríais": "matar",
    "mataríamos": "matar",
    "matarían": "matar",
    "matarías": "matar",
    "matas": "matar",
    "matase": "matar",
    "mataseis": "matar",
    "matasen": "matar",
    "matases": "matar",
    "matasietes": "matasiete",
    "mataste": "matar",
    "matasteis": "matar",
    "matches": "match",
    "mate": "matar",
    "matea": "matear",
    "mateaba": "matear",
    "mateabais": "matear",
    "mateaban": "matear",
    "mateabas": "matear",
    "matead": "matear",
    "mateada": "matear",
    "mateadas": "matear",
    "mateado": "matear",
    "mateadora": "mateador",
    "mateadoras": "mateador",
    "mateadores": "mateador",
    "mateados": "matear",
    "mateamos": "matear",
    "matean": "matear",
    "mateando": "matear",
    "mateara": "matear",
    "matearais": "matear",
    "matearan": "matear",
    "matearas": "matear",
    "mateare": "matear",
    "mateareis": "matear",
    "matearemos": "matear",
    "matearen": "matear",
    "mateares": "matear",
    "matearon": "matear",
    "mateará": "matear",
    "matearán": "matear",
    "matearás": "matear",
    "matearé": "matear",
    "matearéis": "matear",
    "matearía": "matear",
    "matearíais": "matear",
    "matearíamos": "matear",
    "matearían": "matear",
    "matearías": "matear",
    "mateas": "matear",
    "matease": "matear",
    "mateaseis": "matear",
    "mateasen": "matear",
    "mateases": "matear",
    "mateaste": "matear",
    "mateasteis": "matear",
    "matee": "matear",
    "mateemos": "matear",
    "mateen": "matear",
    "matees": "matear",
    "matematismos": "matematismo",
    "matematizaciones": "matematización",
    "matemos": "matar",
    "matemática": "matemático",
    "matemáticas": "matemático",
    "matemáticos": "matemático",
    "maten": "matar",
    "mateo": "matear",
    "materiales": "material",
    "materialice": "materializar",
    "materialicemos": "materializar",
    "materialicen": "materializar",
    "materialices": "materializar",
    "materialicé": "materializar",
    "materialicéis": "materializar",
    "materialidades": "materialidad",
    "materialismos": "materialismo",
    "materialistas": "materialista",
    "materializa": "materializar",
    "materializaba": "materializar",
    "materializabais": "materializar",
    "materializaban": "materializar",
    "materializabas": "materializar",
    "materializaciones": "materialización",
    "materializad": "materializar",
    "materializada": "materializar",
    "materializadas": "materializar",
    "materializado": "materializar",
    "materializados": "materializar",
    "materializamos": "materializar",
    "materializan": "materializar",
    "materializando": "materializar",
    "materializara": "materializar",
    "materializarais": "materializar",
    "materializaran": "materializar",
    "materializaras": "materializar",
    "materializare": "materializar",
    "materializareis": "materializar",
    "materializaremos": "materializar",
    "materializaren": "materializar",
    "materializares": "materializar",
    "materializaron": "materializar",
    "materializará": "materializar",
    "materializarán": "materializar",
    "materializarás": "materializar",
    "materializaré": "materializar",
    "materializaréis": "materializar",
    "materializaría": "materializar",
    "materializaríais": "materializar",
    "materializaríamos": "materializar",
    "materializarían": "materializar",
    "materializarías": "materializar",
    "materializas": "materializar",
    "materializase": "materializar",
    "materializaseis": "materializar",
    "materializasen": "materializar",
    "materializases": "materializar",
    "materializaste": "materializar",
    "materializasteis": "materializar",
    "materializo": "materializar",
    "materializábamos": "materializar",
    "materializáis": "materializar",
    "materializáramos": "materializar",
    "materializáremos": "materializar",
    "materializásemos": "materializar",
    "materializó": "materializar",
    "materials": "material",
    "materias": "materia",
    "materna": "materno",
    "maternales": "maternal",
    "maternas": "materno",
    "maternidades": "maternidad",
    "maternos": "materno",
    "mates": "mate",
    "mateábamos": "matear",
    "mateáis": "matear",
    "mateáramos": "matear",
    "mateáremos": "matear",
    "mateásemos": "matear",
    "mateé": "matear",
    "mateéis": "matear",
    "mateó": "matear",
    "matice": "matizar",
    "maticemos": "matizar",
    "maticen": "matizar",
    "matices": "matizar",
    "maticos": "matico",
    "maticé": "matizar",
    "maticéis": "matizar",
    "matideces": "matidez",
    "matinales": "matinal",
    "matinés": "matiné",
    "matiza": "matizar",
    "matizaba": "matizar",
    "matizabais": "matizar",
    "matizaban": "matizar",
    "matizabas": "matizar",
    "matizables": "matizable",
    "matizaciones": "matización",
    "matizad": "matizar",
    "matizada": "matizar",
    "matizadas": "matizar",
    "matizado": "matizar",
    "matizados": "matizar",
    "matizamos": "matizar",
    "matizan": "matizar",
    "matizando": "matizar",
    "matizara": "matizar",
    "matizarais": "matizar",
    "matizaran": "matizar",
    "matizaras": "matizar",
    "matizare": "matizar",
    "matizareis": "matizar",
    "matizaremos": "matizar",
    "matizaren": "matizar",
    "matizares": "matizar",
    "matizaron": "matizar",
    "matizará": "matizar",
    "matizarán": "matizar",
    "matizarás": "matizar",
    "matizaré": "matizar",
    "matizaréis": "matizar",
    "matizaría": "matizar",
    "matizaríais": "matizar",
    "matizaríamos": "matizar",
    "matizarían": "matizar",
    "matizarías": "matizar",
    "matizas": "matizar",
    "matizase": "matizar",
    "matizaseis": "matizar",
    "matizasen": "matizar",
    "matizases": "matizar",
    "matizaste": "matizar",
    "matizasteis": "matizar",
    "matizo": "matizar",
    "matizábamos": "matizar",
    "matizáis": "matizar",
    "matizáramos": "matizar",
    "matizáremos": "matizar",
    "matizásemos": "matizar",
    "matizó": "matizar",
    "mato": "matar",
    "matojos": "matojo",
    "matona": "matón",
    "matonas": "matón",
    "matonea": "matonear",
    "matoneaba": "matonear",
    "matoneabais": "matonear",
    "matoneaban": "matonear",
    "matoneabas": "matonear",
    "matonead": "matonear",
    "matoneada": "matonear",
    "matoneadas": "matonear",
    "matoneado": "matonear",
    "matoneados": "matonear",
    "matoneamos": "matonear",
    "matonean": "matonear",
    "matoneando": "matonear",
    "matoneara": "matonear",
    "matonearais": "matonear",
    "matonearan": "matonear",
    "matonearas": "matonear",
    "matoneare": "matonear",
    "matoneareis": "matonear",
    "matonearemos": "matonear",
    "matonearen": "matonear",
    "matoneares": "matonear",
    "matonearon": "matonear",
    "matoneará": "matonear",
    "matonearán": "matonear",
    "matonearás": "matonear",
    "matonearé": "matonear",
    "matonearéis": "matonear",
    "matonearía": "matonear",
    "matonearíais": "matonear",
    "matonearíamos": "matonear",
    "matonearían": "matonear",
    "matonearías": "matonear",
    "matoneas": "matonear",
    "matonease": "matonear",
    "matoneaseis": "matonear",
    "matoneasen": "matonear",
    "matoneases": "matonear",
    "matoneaste": "matonear",
    "matoneasteis": "matonear",
    "matonee": "matonear",
    "matoneemos": "matonear",
    "matoneen": "matonear",
    "matonees": "matonear",
    "matoneo": "matonear",
    "matones": "matón",
    "matoneábamos": "matonear",
    "matoneáis": "matonear",
    "matoneáramos": "matonear",
    "matoneáremos": "matonear",
    "matoneásemos": "matonear",
    "matoneé": "matonear",
    "matoneéis": "matonear",
    "matoneó": "matonear",
    "matonismos": "matonismo",
    "matorrales": "matorral",
    "matosa": "matoso",
    "matosas": "matoso",
    "matosos": "matoso",
    "matracaladas": "matracalada",
    "matracas": "matraca",
    "matraces": "matraz",
    "matraquea": "matraquear",
    "matraqueaba": "matraquear",
    "matraqueabais": "matraquear",
    "matraqueaban": "matraquear",
    "matraqueabas": "matraquear",
    "matraquead": "matraquear",
    "matraqueada": "matraquear",
    "matraqueadas": "matraquear",
    "matraqueado": "matraquear",
    "matraqueados": "matraquear",
    "matraqueamos": "matraquear",
    "matraquean": "matraquear",
    "matraqueando": "matraquear",
    "matraqueara": "matraquear",
    "matraquearais": "matraquear",
    "matraquearan": "matraquear",
    "matraquearas": "matraquear",
    "matraqueare": "matraquear",
    "matraqueareis": "matraquear",
    "matraquearemos": "matraquear",
    "matraquearen": "matraquear",
    "matraqueares": "matraquear",
    "matraquearon": "matraquear",
    "matraqueará": "matraquear",
    "matraquearán": "matraquear",
    "matraquearás": "matraquear",
    "matraquearé": "matraquear",
    "matraquearéis": "matraquear",
    "matraquearía": "matraquear",
    "matraquearíais": "matraquear",
    "matraquearíamos": "matraquear",
    "matraquearían": "matraquear",
    "matraquearías": "matraquear",
    "matraqueas": "matraquear",
    "matraquease": "matraquear",
    "matraqueaseis": "matraquear",
    "matraqueasen": "matraquear",
    "matraqueases": "matraquear",
    "matraqueaste": "matraquear",
    "matraqueasteis": "matraquear",
    "matraquee": "matraquear",
    "matraqueemos": "matraquear",
    "matraqueen": "matraquear",
    "matraquees": "matraquear",
    "matraqueo": "matraquear",
    "matraqueos": "matraqueo",
    "matraqueábamos": "matraquear",
    "matraqueáis": "matraquear",
    "matraqueáramos": "matraquear",
    "matraqueáremos": "matraquear",
    "matraqueásemos": "matraquear",
    "matraqueé": "matraquear",
    "matraqueéis": "matraquear",
    "matraqueó": "matraquear",
    "matraquistas": "matraquista",
    "matrera": "matrero",
    "matreras": "matrero",
    "matreros": "matrero",
    "matrerías": "matrería",
    "matriarcados": "matriarcado",
    "matriarcales": "matriarcal",
    "matriarcas": "matriarca",
    "matricarias": "matricaria",
    "matrice": "matrizar",
    "matricemos": "matrizar",
    "matricen": "matrizar",
    "matricera": "matricero",
    "matriceras": "matricero",
    "matriceros": "matricero",
    "matrices": "matrizar",
    "matriciales": "matricial",
    "matricidas": "matricida",
    "matricidios": "matricidio",
    "matriclanes": "matriclán",
    "matricula": "matricular",
    "matriculaba": "matricular",
    "matriculabais": "matricular",
    "matriculaban": "matricular",
    "matriculabas": "matricular",
    "matriculaciones": "matriculación",
    "matriculad": "matricular",
    "matriculada": "matricular",
    "matriculadas": "matricular",
    "matriculado": "matricular",
    "matriculados": "matricular",
    "matriculamos": "matricular",
    "matriculan": "matricular",
    "matriculando": "matricular",
    "matriculara": "matricular",
    "matricularais": "matricular",
    "matricularan": "matricular",
    "matricularas": "matricular",
    "matriculare": "matricular",
    "matriculareis": "matricular",
    "matricularemos": "matricular",
    "matricularen": "matricular",
    "matriculares": "matricular",
    "matricularon": "matricular",
    "matriculará": "matricular",
    "matricularán": "matricular",
    "matricularás": "matricular",
    "matricularé": "matricular",
    "matricularéis": "matricular",
    "matricularía": "matricular",
    "matricularíais": "matricular",
    "matricularíamos": "matricular",
    "matricularían": "matricular",
    "matricularías": "matricular",
    "matriculas": "matricular",
    "matriculase": "matricular",
    "matriculaseis": "matricular",
    "matriculasen": "matricular",
    "matriculases": "matricular",
    "matriculaste": "matricular",
    "matriculasteis": "matricular",
    "matricule": "matricular",
    "matriculemos": "matricular",
    "matriculen": "matricular",
    "matricules": "matricular",
    "matriculo": "matricular",
    "matriculábamos": "matricular",
    "matriculáis": "matricular",
    "matriculáramos": "matricular",
    "matriculáremos": "matricular",
    "matriculásemos": "matricular",
    "matriculé": "matricular",
    "matriculéis": "matricular",
    "matriculó": "matricular",
    "matricé": "matrizar",
    "matricéis": "matrizar",
    "matrifocales": "matrifocal",
    "matrilaterales": "matrilateral",
    "matrilinajes": "matrilinaje",
    "matrilineales": "matrilineal",
    "matrilinealidades": "matrilinealidad",
    "matrilocales": "matrilocal",
    "matrilocalidades": "matrilocalidad",
    "matrimonesca": "matrimonesco",
    "matrimonescas": "matrimonesco",
    "matrimonescos": "matrimonesco",
    "matrimonia": "matrimoniar",
    "matrimoniaba": "matrimoniar",
    "matrimoniabais": "matrimoniar",
    "matrimoniaban": "matrimoniar",
    "matrimoniabas": "matrimoniar",
    "matrimoniad": "matrimoniar",
    "matrimoniada": "matrimoniar",
    "matrimoniadas": "matrimoniar",
    "matrimoniado": "matrimoniar",
    "matrimoniados": "matrimoniar",
    "matrimoniales": "matrimonial",
    "matrimonialistas": "matrimonialista",
    "matrimoniamos": "matrimoniar",
    "matrimonian": "matrimoniar",
    "matrimoniando": "matrimoniar",
    "matrimoniara": "matrimoniar",
    "matrimoniarais": "matrimoniar",
    "matrimoniaran": "matrimoniar",
    "matrimoniaras": "matrimoniar",
    "matrimoniare": "matrimoniar",
    "matrimoniareis": "matrimoniar",
    "matrimoniaremos": "matrimoniar",
    "matrimoniaren": "matrimoniar",
    "matrimoniares": "matrimoniar",
    "matrimoniaron": "matrimoniar",
    "matrimoniará": "matrimoniar",
    "matrimoniarán": "matrimoniar",
    "matrimoniarás": "matrimoniar",
    "matrimoniaré": "matrimoniar",
    "matrimoniaréis": "matrimoniar",
    "matrimoniaría": "matrimoniar",
    "matrimoniaríais": "matrimoniar",
    "matrimoniaríamos": "matrimoniar",
    "matrimoniarían": "matrimoniar",
    "matrimoniarías": "matrimoniar",
    "matrimonias": "matrimoniar",
    "matrimoniase": "matrimoniar",
    "matrimoniaseis": "matrimoniar",
    "matrimoniasen": "matrimoniar",
    "matrimoniases": "matrimoniar",
    "matrimoniaste": "matrimoniar",
    "matrimoniasteis": "matrimoniar",
    "matrimonie": "matrimoniar",
    "matrimoniemos": "matrimoniar",
    "matrimonien": "matrimoniar",
    "matrimonies": "matrimoniar",
    "matrimonio": "matrimoniar",
    "matrimonios": "matrimonio",
    "matrimoniábamos": "matrimoniar",
    "matrimoniáis": "matrimoniar",
    "matrimoniáramos": "matrimoniar",
    "matrimoniáremos": "matrimoniar",
    "matrimoniásemos": "matrimoniar",
    "matrimonié": "matrimoniar",
    "matrimoniéis": "matrimoniar",
    "matrimonió": "matrimoniar",
    "matritenses": "matritense",
    "matriza": "matrizar",
    "matrizaba": "matrizar",
    "matrizabais": "matrizar",
    "matrizaban": "matrizar",
    "matrizabas": "matrizar",
    "matrizad": "matrizar",
    "matrizada": "matrizar",
    "matrizadas": "matrizar",
    "matrizado": "matrizar",
    "matrizados": "matrizar",
    "matrizamos": "matrizar",
    "matrizan": "matrizar",
    "matrizando": "matrizar",
    "matrizara": "matrizar",
    "matrizarais": "matrizar",
    "matrizaran": "matrizar",
    "matrizaras": "matrizar",
    "matrizare": "matrizar",
    "matrizareis": "matrizar",
    "matrizaremos": "matrizar",
    "matrizaren": "matrizar",
    "matrizares": "matrizar",
    "matrizaron": "matrizar",
    "matrizará": "matrizar",
    "matrizarán": "matrizar",
    "matrizarás": "matrizar",
    "matrizaré": "matrizar",
    "matrizaréis": "matrizar",
    "matrizaría": "matrizar",
    "matrizaríais": "matrizar",
    "matrizaríamos": "matrizar",
    "matrizarían": "matrizar",
    "matrizarías": "matrizar",
    "matrizas": "matrizar",
    "matrizase": "matrizar",
    "matrizaseis": "matrizar",
    "matrizasen": "matrizar",
    "matrizases": "matrizar",
    "matrizaste": "matrizar",
    "matrizasteis": "matrizar",
    "matrizo": "matrizar",
    "matrizábamos": "matrizar",
    "matrizáis": "matrizar",
    "matrizáramos": "matrizar",
    "matrizáremos": "matrizar",
    "matrizásemos": "matrizar",
    "matrizó": "matrizar",
    "matronales": "matronal",
    "matronas": "matrona",
    "matrículas": "matrícula",
    "matules": "matul",
    "maturrangas": "maturranga",
    "matusalenes": "matusalén",
    "matutea": "matutear",
    "matuteaba": "matutear",
    "matuteabais": "matutear",
    "matuteaban": "matutear",
    "matuteabas": "matutear",
    "matutead": "matutear",
    "matuteada": "matutear",
    "matuteadas": "matutear",
    "matuteado": "matutear",
    "matuteados": "matutear",
    "matuteamos": "matutear",
    "matutean": "matutear",
    "matuteando": "matutear",
    "matuteara": "matutear",
    "matutearais": "matutear",
    "matutearan": "matutear",
    "matutearas": "matutear",
    "matuteare": "matutear",
    "matuteareis": "matutear",
    "matutearemos": "matutear",
    "matutearen": "matutear",
    "matuteares": "matutear",
    "matutearon": "matutear",
    "matuteará": "matutear",
    "matutearán": "matutear",
    "matutearás": "matutear",
    "matutearé": "matutear",
    "matutearéis": "matutear",
    "matutearía": "matutear",
    "matutearíais": "matutear",
    "matutearíamos": "matutear",
    "matutearían": "matutear",
    "matutearías": "matutear",
    "matuteas": "matutear",
    "matutease": "matutear",
    "matuteaseis": "matutear",
    "matuteasen": "matutear",
    "matuteases": "matutear",
    "matuteaste": "matutear",
    "matuteasteis": "matutear",
    "matutee": "matutear",
    "matuteemos": "matutear",
    "matuteen": "matutear",
    "matutees": "matutear",
    "matuteo": "matutear",
    "matutera": "matutero",
    "matuteras": "matutero",
    "matuteros": "matutero",
    "matutes": "matute",
    "matuteábamos": "matutear",
    "matuteáis": "matutear",
    "matuteáramos": "matutear",
    "matuteáremos": "matutear",
    "matuteásemos": "matutear",
    "matuteé": "matutear",
    "matuteéis": "matutear",
    "matuteó": "matutear",
    "matutina": "matutino",
    "matutinas": "matutino",
    "matutinos": "matutino",
    "matábamos": "matar",
    "matáis": "matar",
    "matáramos": "matar",
    "matáremos": "matar",
    "matásemos": "matar",
    "maté": "matar",
    "matéis": "matar",
    "mató": "matar",
    "maulas": "maula",
    "maulera": "maulero",
    "mauleras": "maulero",
    "mauleros": "maulero",
    "maulerías": "maulería",
    "maullaba": "maullar",
    "maullabais": "maullar",
    "maullaban": "maullar",
    "maullabas": "maullar",
    "maullad": "maullar",
    "maullada": "maullar",
    "maulladas": "maullar",
    "maullado": "maullar",
    "maulladora": "maullador",
    "maulladoras": "maullador",
    "maulladores": "maullador",
    "maullados": "maullar",
    "maullamos": "maullar",
    "maullando": "maullar",
    "maullara": "maullar",
    "maullarais": "maullar",
    "maullaran": "maullar",
    "maullaras": "maullar",
    "maullare": "maullar",
    "maullareis": "maullar",
    "maullaremos": "maullar",
    "maullaren": "maullar",
    "maullares": "maullar",
    "maullaron": "maullar",
    "maullará": "maullar",
    "maullarán": "maullar",
    "maullarás": "maullar",
    "maullaré": "maullar",
    "maullaréis": "maullar",
    "maullaría": "maullar",
    "maullaríais": "maullar",
    "maullaríamos": "maullar",
    "maullarían": "maullar",
    "maullarías": "maullar",
    "maullase": "maullar",
    "maullaseis": "maullar",
    "maullasen": "maullar",
    "maullases": "maullar",
    "maullaste": "maullar",
    "maullasteis": "maullar",
    "maullemos": "maullar",
    "maullidos": "maullido",
    "maullábamos": "maullar",
    "maulláis": "maullar",
    "maulláramos": "maullar",
    "maulláremos": "maullar",
    "maullásemos": "maullar",
    "maullé": "maullar",
    "maulléis": "maullar",
    "maulló": "maullar",
    "maulona": "maulón",
    "maulonas": "maulón",
    "maulones": "maulón",
    "maures": "maure",
    "mauritana": "mauritano",
    "mauritanas": "mauritano",
    "mauritanos": "mauritano",
    "mauseolos": "mauseolo",
    "mausoleos": "mausoleo",
    "mauveínas": "mauveína",
    "maxilares": "maxilar",
    "maxilas": "maxila",
    "maxilectomías": "maxilectomía",
    "maxilodentales": "maxilodental",
    "maxilofaciales": "maxilofacial",
    "maxilolabiales": "maxilolabial",
    "maxilomandibulares": "maxilomandibular",
    "maxilonasales": "maxilonasal",
    "maxilopalatina": "maxilopalatino",
    "maxilopalatinas": "maxilopalatino",
    "maxilopalatinos": "maxilopalatino",
    "maximalismos": "maximalismo",
    "maximalistas": "maximalista",
    "maximice": "maximizar",
    "maximicemos": "maximizar",
    "maximicen": "maximizar",
    "maximices": "maximizar",
    "maximicé": "maximizar",
    "maximicéis": "maximizar",
    "maximiza": "maximizar",
    "maximizaba": "maximizar",
    "maximizabais": "maximizar",
    "maximizaban": "maximizar",
    "maximizabas": "maximizar",
    "maximizaciones": "maximización",
    "maximizad": "maximizar",
    "maximizada": "maximizar",
    "maximizadas": "maximizar",
    "maximizado": "maximizar",
    "maximizados": "maximizar",
    "maximizamos": "maximizar",
    "maximizan": "maximizar",
    "maximizando": "maximizar",
    "maximizara": "maximizar",
    "maximizarais": "maximizar",
    "maximizaran": "maximizar",
    "maximizaras": "maximizar",
    "maximizare": "maximizar",
    "maximizareis": "maximizar",
    "maximizaremos": "maximizar",
    "maximizaren": "maximizar",
    "maximizares": "maximizar",
    "maximizaron": "maximizar",
    "maximizará": "maximizar",
    "maximizarán": "maximizar",
    "maximizarás": "maximizar",
    "maximizaré": "maximizar",
    "maximizaréis": "maximizar",
    "maximizaría": "maximizar",
    "maximizaríais": "maximizar",
    "maximizaríamos": "maximizar",
    "maximizarían": "maximizar",
    "maximizarías": "maximizar",
    "maximizas": "maximizar",
    "maximizase": "maximizar",
    "maximizaseis": "maximizar",
    "maximizasen": "maximizar",
    "maximizases": "maximizar",
    "maximizaste": "maximizar",
    "maximizasteis": "maximizar",
    "maximizo": "maximizar",
    "maximizábamos": "maximizar",
    "maximizáis": "maximizar",
    "maximizáramos": "maximizar",
    "maximizáremos": "maximizar",
    "maximizásemos": "maximizar",
    "maximizó": "maximizar",
    "maxwells": "maxwell",
    "may.": "mayo",
    "maya": "mayar",
    "maya-quichés": "maya-quiché",
    "mayaba": "mayar",
    "mayabais": "mayar",
    "mayaban": "mayar",
    "mayabas": "mayar",
    "mayad": "mayar",
    "mayada": "mayar",
    "mayadas": "mayar",
    "mayado": "mayar",
    "mayadora": "mayador",
    "mayadoras": "mayador",
    "mayadores": "mayador",
    "mayados": "mayar",
    "mayales": "mayal",
    "mayamos": "mayar",
    "mayan": "mayar",
    "mayando": "mayar",
    "mayaquichés": "mayaquiché",
    "mayara": "mayar",
    "mayarais": "mayar",
    "mayaran": "mayar",
    "mayaras": "mayar",
    "mayare": "mayar",
    "mayareis": "mayar",
    "mayaremos": "mayar",
    "mayaren": "mayar",
    "mayares": "mayar",
    "mayaron": "mayar",
    "mayará": "mayar",
    "mayarán": "mayar",
    "mayarás": "mayar",
    "mayaré": "mayar",
    "mayaréis": "mayar",
    "mayaría": "mayar",
    "mayaríais": "mayar",
    "mayaríamos": "mayar",
    "mayarían": "mayar",
    "mayarías": "mayar",
    "mayas": "mayar",
    "mayase": "mayar",
    "mayaseis": "mayar",
    "mayasen": "mayar",
    "mayases": "mayar",
    "mayaste": "mayar",
    "mayasteis": "mayar",
    "maye": "mayar",
    "mayemos": "mayar",
    "mayen": "mayar",
    "mayes": "mayar",
    "mayestática": "mayestático",
    "mayestáticas": "mayestático",
    "mayestáticos": "mayestático",
    "mayidos": "mayido",
    "mayo": "mayar",
    "mayonesas": "mayonesa",
    "mayorales": "mayoral",
    "mayorantes": "mayorante",
    "mayorazgas": "mayorazga",
    "mayorazgos": "mayorazgo",
    "mayorcita": "mayorcito",
    "mayorcitas": "mayorcito",
    "mayorcitos": "mayorcito",
    "mayordoma": "mayordomo",
    "mayordomas": "mayordomo",
    "mayordomos": "mayordomo",
    "mayordomías": "mayordomía",
    "mayores": "mayor",
    "mayoridades": "mayoridad",
    "mayoristas": "mayorista",
    "mayoritaria": "mayoritario",
    "mayoritarias": "mayoritario",
    "mayoritarios": "mayoritario",
    "mayorías": "mayoría",
    "mayos": "mayo",
    "mayábamos": "mayar",
    "mayáis": "mayar",
    "mayáramos": "mayar",
    "mayáremos": "mayar",
    "mayásemos": "mayar",
    "mayé": "mayar",
    "mayéis": "mayar",
    "mayéutica": "mayéutico",
    "mayéuticas": "mayéutico",
    "mayéuticos": "mayéutico",
    "mayó": "mayar",
    "mayólicas": "mayólica",
    "mayúscula": "mayúsculo",
    "mayúsculas": "mayúsculo",
    "mayúsculos": "mayúsculo",
    "maza": "mazar",
    "mazaba": "mazar",
    "mazabais": "mazar",
    "mazaban": "mazar",
    "mazabas": "mazar",
    "mazacotes": "mazacote",
    "mazad": "mazar",
    "mazada": "mazar",
    "mazadas": "mazar",
    "mazado": "mazar",
    "mazados": "mazar",
    "mazamorras": "mazamorra",
    "mazamos": "mazar",
    "mazan": "mazar",
    "mazando": "mazar",
    "mazanetas": "mazaneta",
    "mazapanes": "mazapán",
    "mazara": "mazar",
    "mazarais": "mazar",
    "mazaran": "mazar",
    "mazaras": "mazar",
    "mazare": "mazar",
    "mazareis": "mazar",
    "mazaremos": "mazar",
    "mazaren": "mazar",
    "mazares": "mazar",
    "mazaron": "mazar",
    "mazarotas": "mazarota",
    "mazará": "mazar",
    "mazarán": "mazar",
    "mazarás": "mazar",
    "mazaré": "mazar",
    "mazaréis": "mazar",
    "mazaría": "mazar",
    "mazaríais": "mazar",
    "mazaríamos": "mazar",
    "mazarían": "mazar",
    "mazarías": "mazar",
    "mazaríes": "mazarí",
    "mazarís": "mazarí",
    "mazas": "mazar",
    "mazase": "mazar",
    "mazaseis": "mazar",
    "mazasen": "mazar",
    "mazases": "mazar",
    "mazaste": "mazar",
    "mazasteis": "mazar",
    "mazazos": "mazazo",
    "mazdeísmos": "mazdeísmo",
    "mazdeístas": "mazdeísta",
    "mazmorras": "mazmorra",
    "mazna": "maznar",
    "maznaba": "maznar",
    "maznabais": "maznar",
    "maznaban": "maznar",
    "maznabas": "maznar",
    "maznad": "maznar",
    "maznada": "maznar",
    "maznadas": "maznar",
    "maznado": "maznar",
    "maznados": "maznar",
    "maznamos": "maznar",
    "maznan": "maznar",
    "maznando": "maznar",
    "maznara": "maznar",
    "maznarais": "maznar",
    "maznaran": "maznar",
    "maznaras": "maznar",
    "maznare": "maznar",
    "maznareis": "maznar",
    "maznaremos": "maznar",
    "maznaren": "maznar",
    "maznares": "maznar",
    "maznaron": "maznar",
    "maznará": "maznar",
    "maznarán": "maznar",
    "maznarás": "maznar",
    "maznaré": "maznar",
    "maznaréis": "maznar",
    "maznaría": "maznar",
    "maznaríais": "maznar",
    "maznaríamos": "maznar",
    "maznarían": "maznar",
    "maznarías": "maznar",
    "maznas": "maznar",
    "maznase": "maznar",
    "maznaseis": "maznar",
    "maznasen": "maznar",
    "maznases": "maznar",
    "maznaste": "maznar",
    "maznasteis": "maznar",
    "mazne": "maznar",
    "maznemos": "maznar",
    "maznen": "maznar",
    "maznes": "maznar",
    "mazno": "maznar",
    "maznábamos": "maznar",
    "maznáis": "maznar",
    "maznáramos": "maznar",
    "maznáremos": "maznar",
    "maznásemos": "maznar",
    "mazné": "maznar",
    "maznéis": "maznar",
    "maznó": "maznar",
    "mazo": "mazar",
    "mazonada": "mazonado",
    "mazonadas": "mazonado",
    "mazonados": "mazonado",
    "mazonea": "mazonear",
    "mazoneaba": "mazonear",
    "mazoneabais": "mazonear",
    "mazoneaban": "mazonear",
    "mazoneabas": "mazonear",
    "mazonead": "mazonear",
    "mazoneada": "mazonear",
    "mazoneadas": "mazonear",
    "mazoneado": "mazonear",
    "mazoneados": "mazonear",
    "mazoneamos": "mazonear",
    "mazonean": "mazonear",
    "mazoneando": "mazonear",
    "mazoneara": "mazonear",
    "mazonearais": "mazonear",
    "mazonearan": "mazonear",
    "mazonearas": "mazonear",
    "mazoneare": "mazonear",
    "mazoneareis": "mazonear",
    "mazonearemos": "mazonear",
    "mazonearen": "mazonear",
    "mazoneares": "mazonear",
    "mazonearon": "mazonear",
    "mazoneará": "mazonear",
    "mazonearán": "mazonear",
    "mazonearás": "mazonear",
    "mazonearé": "mazonear",
    "mazonearéis": "mazonear",
    "mazonearía": "mazonear",
    "mazonearíais": "mazonear",
    "mazonearíamos": "mazonear",
    "mazonearían": "mazonear",
    "mazonearías": "mazonear",
    "mazoneas": "mazonear",
    "mazonease": "mazonear",
    "mazoneaseis": "mazonear",
    "mazoneasen": "mazonear",
    "mazoneases": "mazonear",
    "mazoneaste": "mazonear",
    "mazoneasteis": "mazonear",
    "mazonee": "mazonear",
    "mazoneemos": "mazonear",
    "mazoneen": "mazonear",
    "mazonees": "mazonear",
    "mazoneo": "mazonear",
    "mazonerías": "mazonería",
    "mazoneábamos": "mazonear",
    "mazoneáis": "mazonear",
    "mazoneáramos": "mazonear",
    "mazoneáremos": "mazonear",
    "mazoneásemos": "mazonear",
    "mazoneé": "mazonear",
    "mazoneéis": "mazonear",
    "mazoneó": "mazonear",
    "mazorcas": "mazorca",
    "mazorrales": "mazorral",
    "mazos": "mazo",
    "mazuelos": "mazuelo",
    "mazurcas": "mazurca",
    "mazutes": "mazut",
    "mazuts": "mazut",
    "mazábamos": "mazar",
    "mazáis": "mazar",
    "mazáramos": "mazar",
    "mazáremos": "mazar",
    "mazásemos": "mazar",
    "mazó": "mazar",
    "maíces": "maíz",
    "maña": "maño",
    "mañanas": "mañana",
    "mañanea": "mañanear",
    "mañaneaba": "mañanear",
    "mañaneabais": "mañanear",
    "mañaneaban": "mañanear",
    "mañaneabas": "mañanear",
    "mañanead": "mañanear",
    "mañaneada": "mañanear",
    "mañaneadas": "mañanear",
    "mañaneado": "mañanear",
    "mañaneados": "mañanear",
    "mañaneamos": "mañanear",
    "mañanean": "mañanear",
    "mañaneando": "mañanear",
    "mañaneara": "mañanear",
    "mañanearais": "mañanear",
    "mañanearan": "mañanear",
    "mañanearas": "mañanear",
    "mañaneare": "mañanear",
    "mañaneareis": "mañanear",
    "mañanearemos": "mañanear",
    "mañanearen": "mañanear",
    "mañaneares": "mañanear",
    "mañanearon": "mañanear",
    "mañaneará": "mañanear",
    "mañanearán": "mañanear",
    "mañanearás": "mañanear",
    "mañanearé": "mañanear",
    "mañanearéis": "mañanear",
    "mañanearía": "mañanear",
    "mañanearíais": "mañanear",
    "mañanearíamos": "mañanear",
    "mañanearían": "mañanear",
    "mañanearías": "mañanear",
    "mañaneas": "mañanear",
    "mañanease": "mañanear",
    "mañaneaseis": "mañanear",
    "mañaneasen": "mañanear",
    "mañaneases": "mañanear",
    "mañaneaste": "mañanear",
    "mañaneasteis": "mañanear",
    "mañanee": "mañanear",
    "mañaneemos": "mañanear",
    "mañaneen": "mañanear",
    "mañanees": "mañanear",
    "mañaneo": "mañanear",
    "mañanera": "mañanero",
    "mañaneras": "mañanero",
    "mañaneros": "mañanero",
    "mañaneábamos": "mañanear",
    "mañaneáis": "mañanear",
    "mañaneáramos": "mañanear",
    "mañaneáremos": "mañanear",
    "mañaneásemos": "mañanear",
    "mañaneé": "mañanear",
    "mañaneéis": "mañanear",
    "mañaneó": "mañanear",
    "mañas": "maño",
    "mañea": "mañear",
    "mañeaba": "mañear",
    "mañeabais": "mañear",
    "mañeaban": "mañear",
    "mañeabas": "mañear",
    "mañead": "mañear",
    "mañeada": "mañear",
    "mañeadas": "mañear",
    "mañeado": "mañear",
    "mañeados": "mañear",
    "mañeamos": "mañear",
    "mañean": "mañear",
    "mañeando": "mañear",
    "mañeara": "mañear",
    "mañearais": "mañear",
    "mañearan": "mañear",
    "mañearas": "mañear",
    "mañeare": "mañear",
    "mañeareis": "mañear",
    "mañearemos": "mañear",
    "mañearen": "mañear",
    "mañeares": "mañear",
    "mañearon": "mañear",
    "mañeará": "mañear",
    "mañearán": "mañear",
    "mañearás": "mañear",
    "mañearé": "mañear",
    "mañearéis": "mañear",
    "mañearía": "mañear",
    "mañearíais": "mañear",
    "mañearíamos": "mañear",
    "mañearían": "mañear",
    "mañearías": "mañear",
    "mañeas": "mañear",
    "mañease": "mañear",
    "mañeaseis": "mañear",
    "mañeasen": "mañear",
    "mañeases": "mañear",
    "mañeaste": "mañear",
    "mañeasteis": "mañear",
    "mañee": "mañear",
    "mañeemos": "mañear",
    "mañeen": "mañear",
    "mañees": "mañear",
    "mañeo": "mañear",
    "mañera": "mañero",
    "mañeras": "mañero",
    "mañeros": "mañero",
    "mañerías": "mañería",
    "mañeábamos": "mañear",
    "mañeáis": "mañear",
    "mañeáramos": "mañear",
    "mañeáremos": "mañear",
    "mañeásemos": "mañear",
    "mañeé": "mañear",
    "mañeéis": "mañear",
    "mañeó": "mañear",
    "maños": "maño",
    "mañosa": "mañoso",
    "mañosas": "mañoso",
    "mañosos": "mañoso",
    "mañuelas": "mañuela",
    "maúlla": "maullar",
    "maúllan": "maullar",
    "maúllas": "maullar",
    "maúlle": "maullar",
    "maúllen": "maullar",
    "maúlles": "maullar",
    "maúllo": "maullar",
    "maúllos": "maúllo",
    "mea": "mear",
    "meaba": "mear",
    "meabais": "mear",
    "meaban": "mear",
    "meabas": "mear",
    "mead": "mear",
    "meada": "mear",
    "meadas": "mear",
    "meaderos": "meadero",
    "meado": "mear",
    "meados": "mear",
    "meajas": "meaja",
    "meajuelas": "meajuela",
    "meamos": "mear",
    "mean": "mear",
    "meando": "mear",
    "meandros": "meandro",
    "meara": "mear",
    "mearais": "mear",
    "mearan": "mear",
    "mearas": "mear",
    "meare": "mear",
    "meareis": "mear",
    "mearemos": "mear",
    "mearen": "mear",
    "meares": "mear",
    "mearon": "mear",
    "meará": "mear",
    "mearán": "mear",
    "mearás": "mear",
    "mearé": "mear",
    "mearéis": "mear",
    "mearía": "mear",
    "mearíais": "mear",
    "mearíamos": "mear",
    "mearían": "mear",
    "mearías": "mear",
    "meas": "mear",
    "mease": "mear",
    "measeis": "mear",
    "measen": "mear",
    "meases": "mear",
    "measte": "mear",
    "measteis": "mear",
    "meatos": "meato",
    "mecana": "mecano",
    "mecanas": "mecano",
    "mecanice": "mecanizar",
    "mecanicemos": "mecanizar",
    "mecanicen": "mecanizar",
    "mecanices": "mecanizar",
    "mecanicismos": "mecanicismo",
    "mecanicistas": "mecanicista",
    "mecanicé": "mecanizar",
    "mecanicéis": "mecanizar",
    "mecanismos": "mecanismo",
    "mecaniza": "mecanizar",
    "mecanizaba": "mecanizar",
    "mecanizabais": "mecanizar",
    "mecanizaban": "mecanizar",
    "mecanizabas": "mecanizar",
    "mecanizables": "mecanizable",
    "mecanizaciones": "mecanización",
    "mecanizad": "mecanizar",
    "mecanizada": "mecanizar",
    "mecanizadas": "mecanizar",
    "mecanizado": "mecanizar",
    "mecanizados": "mecanizar",
    "mecanizamos": "mecanizar",
    "mecanizan": "mecanizar",
    "mecanizando": "mecanizar",
    "mecanizara": "mecanizar",
    "mecanizarais": "mecanizar",
    "mecanizaran": "mecanizar",
    "mecanizaras": "mecanizar",
    "mecanizare": "mecanizar",
    "mecanizareis": "mecanizar",
    "mecanizaremos": "mecanizar",
    "mecanizaren": "mecanizar",
    "mecanizares": "mecanizar",
    "mecanizaron": "mecanizar",
    "mecanizará": "mecanizar",
    "mecanizarán": "mecanizar",
    "mecanizarás": "mecanizar",
    "mecanizaré": "mecanizar",
    "mecanizaréis": "mecanizar",
    "mecanizaría": "mecanizar",
    "mecanizaríais": "mecanizar",
    "mecanizaríamos": "mecanizar",
    "mecanizarían": "mecanizar",
    "mecanizarías": "mecanizar",
    "mecanizas": "mecanizar",
    "mecanizase": "mecanizar",
    "mecanizaseis": "mecanizar",
    "mecanizasen": "mecanizar",
    "mecanizases": "mecanizar",
    "mecanizaste": "mecanizar",
    "mecanizasteis": "mecanizar",
    "mecanizo": "mecanizar",
    "mecanizábamos": "mecanizar",
    "mecanizáis": "mecanizar",
    "mecanizáramos": "mecanizar",
    "mecanizáremos": "mecanizar",
    "mecanizásemos": "mecanizar",
    "mecanizó": "mecanizar",
    "mecanodinámica": "mecanodinámico",
    "mecanodinámicas": "mecanodinámico",
    "mecanodinámicos": "mecanodinámico",
    "mecanografiaba": "mecanografiar",
    "mecanografiabais": "mecanografiar",
    "mecanografiaban": "mecanografiar",
    "mecanografiabas": "mecanografiar",
    "mecanografiad": "mecanografiar",
    "mecanografiada": "mecanografiar",
    "mecanografiadas": "mecanografiar",
    "mecanografiado": "mecanografiar",
    "mecanografiados": "mecanografiar",
    "mecanografiamos": "mecanografiar",
    "mecanografiando": "mecanografiar",
    "mecanografiara": "mecanografiar",
    "mecanografiarais": "mecanografiar",
    "mecanografiaran": "mecanografiar",
    "mecanografiaras": "mecanografiar",
    "mecanografiare": "mecanografiar",
    "mecanografiareis": "mecanografiar",
    "mecanografiaremos": "mecanografiar",
    "mecanografiaren": "mecanografiar",
    "mecanografiares": "mecanografiar",
    "mecanografiaron": "mecanografiar",
    "mecanografiará": "mecanografiar",
    "mecanografiarán": "mecanografiar",
    "mecanografiarás": "mecanografiar",
    "mecanografiaré": "mecanografiar",
    "mecanografiaréis": "mecanografiar",
    "mecanografiaría": "mecanografiar",
    "mecanografiaríais": "mecanografiar",
    "mecanografiaríamos": "mecanografiar",
    "mecanografiarían": "mecanografiar",
    "mecanografiarías": "mecanografiar",
    "mecanografiase": "mecanografiar",
    "mecanografiaseis": "mecanografiar",
    "mecanografiasen": "mecanografiar",
    "mecanografiases": "mecanografiar",
    "mecanografiaste": "mecanografiar",
    "mecanografiasteis": "mecanografiar",
    "mecanografiemos": "mecanografiar",
    "mecanografiábamos": "mecanografiar",
    "mecanografiáis": "mecanografiar",
    "mecanografiáramos": "mecanografiar",
    "mecanografiáremos": "mecanografiar",
    "mecanografiásemos": "mecanografiar",
    "mecanografié": "mecanografiar",
    "mecanografiéis": "mecanografiar",
    "mecanografió": "mecanografiar",
    "mecanografía": "mecanografiar",
    "mecanografían": "mecanografiar",
    "mecanografías": "mecanografiar",
    "mecanografíe": "mecanografiar",
    "mecanografíen": "mecanografiar",
    "mecanografíes": "mecanografiar",
    "mecanografío": "mecanografiar",
    "mecanográfica": "mecanográfico",
    "mecanográficas": "mecanográfico",
    "mecanográficos": "mecanográfico",
    "mecanorregulables": "mecanorregulable",
    "mecanos": "mecano",
    "mecanoterapias": "mecanoterapia",
    "mecanotipias": "mecanotipia",
    "mecanística": "mecanístico",
    "mecanísticas": "mecanístico",
    "mecanísticos": "mecanístico",
    "mecanógrafa": "mecanógrafo",
    "mecanógrafas": "mecanógrafo",
    "mecanógrafos": "mecanógrafo",
    "mecas": "meca",
    "mecates": "mecate",
    "mece": "mecer",
    "meced": "mecer",
    "mecedora": "mecedor",
    "mecedoras": "mecedora",
    "mecedores": "mecedor",
    "meceduras": "mecedura",
    "mecemos": "mecer",
    "mecen": "mecer",
    "mecenazgos": "mecenazgo",
    "meceremos": "mecer",
    "mecerá": "mecer",
    "mecerán": "mecer",
    "mecerás": "mecer",
    "meceré": "mecer",
    "meceréis": "mecer",
    "mecería": "mecer",
    "meceríais": "mecer",
    "meceríamos": "mecer",
    "mecerían": "mecer",
    "mecerías": "mecer",
    "meces": "mecer",
    "mecha": "mechar",
    "mechaba": "mechar",
    "mechabais": "mechar",
    "mechaban": "mechar",
    "mechabas": "mechar",
    "mechad": "mechar",
    "mechada": "mechar",
    "mechadas": "mechar",
    "mechado": "mechar",
    "mechados": "mechar",
    "mechamos": "mechar",
    "mechan": "mechar",
    "mechando": "mechar",
    "mechara": "mechar",
    "mecharais": "mechar",
    "mecharan": "mechar",
    "mecharas": "mechar",
    "mechare": "mechar",
    "mechareis": "mechar",
    "mecharemos": "mechar",
    "mecharen": "mechar",
    "mechares": "mechar",
    "mecharon": "mechar",
    "mechará": "mechar",
    "mecharán": "mechar",
    "mecharás": "mechar",
    "mecharé": "mechar",
    "mecharéis": "mechar",
    "mecharía": "mechar",
    "mecharíais": "mechar",
    "mecharíamos": "mechar",
    "mecharían": "mechar",
    "mecharías": "mechar",
    "mechas": "mechar",
    "mechase": "mechar",
    "mechaseis": "mechar",
    "mechasen": "mechar",
    "mechases": "mechar",
    "mechaste": "mechar",
    "mechasteis": "mechar",
    "mechazos": "mechazo",
    "meche": "mechar",
    "mechemos": "mechar",
    "mechen": "mechar",
    "mecheras": "mechera",
    "mecheros": "mechero",
    "meches": "mechar",
    "mechinales": "mechinal",
    "mecho": "mechar",
    "mechones": "mechón",
    "mechosa": "mechoso",
    "mechosas": "mechoso",
    "mechosos": "mechoso",
    "mechábamos": "mechar",
    "mecháis": "mechar",
    "mecháramos": "mechar",
    "mecháremos": "mechar",
    "mechásemos": "mechar",
    "meché": "mechar",
    "mechéis": "mechar",
    "mechó": "mechar",
    "mecida": "mecer",
    "mecidas": "mecer",
    "mecido": "mecer",
    "mecidos": "mecer",
    "meciendo": "mecer",
    "meciera": "mecer",
    "mecierais": "mecer",
    "mecieran": "mecer",
    "mecieras": "mecer",
    "meciere": "mecer",
    "meciereis": "mecer",
    "mecieren": "mecer",
    "mecieres": "mecer",
    "mecieron": "mecer",
    "meciese": "mecer",
    "mecieseis": "mecer",
    "meciesen": "mecer",
    "mecieses": "mecer",
    "mecimos": "mecer",
    "meciste": "mecer",
    "mecisteis": "mecer",
    "meciéramos": "mecer",
    "meciéremos": "mecer",
    "meciésemos": "mecer",
    "meció": "mecer",
    "meconios": "meconio",
    "mecánica": "mecánico",
    "mecánicas": "mecánico",
    "mecánicos": "mecánico",
    "mecéis": "mecer",
    "mecí": "mecer",
    "mecía": "mecer",
    "mecíais": "mecer",
    "mecíamos": "mecer",
    "mecían": "mecer",
    "mecías": "mecer",
    "mecónica": "mecónico",
    "mecónicas": "mecónico",
    "mecónicos": "mecónico",
    "meda": "medo",
    "medallas": "medalla",
    "medalleros": "medallero",
    "medallistas": "medallista",
    "medallones": "medallón",
    "medanosa": "medanoso",
    "medanosas": "medanoso",
    "medanosos": "medanoso",
    "medas": "medo",
    "medaños": "medaño",
    "media": "medio",
    "mediaba": "mediar",
    "mediabais": "mediar",
    "mediaban": "mediar",
    "mediabas": "mediar",
    "mediacañas": "mediacaña",
    "mediaciones": "mediación",
    "mediad": "mediar",
    "mediada": "mediar",
    "mediadas": "mediar",
    "mediado": "mediar",
    "mediadora": "mediador",
    "mediadoras": "mediador",
    "mediadores": "mediador",
    "mediados": "mediar",
    "mediales": "medial",
    "medialunas": "medialuna",
    "mediamos": "mediar",
    "median": "mediar",
    "mediana": "mediano",
    "medianas": "mediano",
    "mediando": "mediar",
    "medianeja": "medianejo",
    "medianejas": "medianejo",
    "medianejos": "medianejo",
    "medianera": "medianero",
    "medianeras": "medianero",
    "medianeros": "medianero",
    "medianerías": "medianería",
    "medianiles": "medianil",
    "medianoches": "medianoche",
    "medianos": "mediano",
    "mediantes": "mediante",
    "medianías": "medianía",
    "mediapuntas": "mediapunta",
    "mediara": "mediar",
    "mediarais": "mediar",
    "mediaran": "mediar",
    "mediaras": "mediar",
    "mediare": "mediar",
    "mediareis": "mediar",
    "mediaremos": "mediar",
    "mediaren": "mediar",
    "mediares": "mediar",
    "mediaron": "mediar",
    "mediará": "mediar",
    "mediarán": "mediar",
    "mediarás": "mediar",
    "mediaré": "mediar",
    "mediaréis": "mediar",
    "mediaría": "mediar",
    "mediaríais": "mediar",
    "mediaríamos": "mediar",
    "mediarían": "mediar",
    "mediarías": "mediar",
    "medias": "medio",
    "mediase": "mediar",
    "mediaseis": "mediar",
    "mediasen": "mediar",
    "mediases": "mediar",
    "mediaste": "mediar",
    "mediasteis": "mediar",
    "mediastinos": "mediastino",
    "mediastínica": "mediastínico",
    "mediastínicas": "mediastínico",
    "mediastínicos": "mediastínico",
    "mediata": "mediato",
    "mediatas": "mediato",
    "mediatecas": "mediateca",
    "mediatice": "mediatizar",
    "mediaticemos": "mediatizar",
    "mediaticen": "mediatizar",
    "mediatices": "mediatizar",
    "mediaticé": "mediatizar",
    "mediaticéis": "mediatizar",
    "mediatiza": "mediatizar",
    "mediatizaba": "mediatizar",
    "mediatizabais": "mediatizar",
    "mediatizaban": "mediatizar",
    "mediatizabas": "mediatizar",
    "mediatizaciones": "mediatización",
    "mediatizad": "mediatizar",
    "mediatizada": "mediatizar",
    "mediatizadas": "mediatizar",
    "mediatizado": "mediatizar",
    "mediatizados": "mediatizar",
    "mediatizamos": "mediatizar",
    "mediatizan": "mediatizar",
    "mediatizando": "mediatizar",
    "mediatizara": "mediatizar",
    "mediatizarais": "mediatizar",
    "mediatizaran": "mediatizar",
    "mediatizaras": "mediatizar",
    "mediatizare": "mediatizar",
    "mediatizareis": "mediatizar",
    "mediatizaremos": "mediatizar",
    "mediatizaren": "mediatizar",
    "mediatizares": "mediatizar",
    "mediatizaron": "mediatizar",
    "mediatizará": "mediatizar",
    "mediatizarán": "mediatizar",
    "mediatizarás": "mediatizar",
    "mediatizaré": "mediatizar",
    "mediatizaréis": "mediatizar",
    "mediatizaría": "mediatizar",
    "mediatizaríais": "mediatizar",
    "mediatizaríamos": "mediatizar",
    "mediatizarían": "mediatizar",
    "mediatizarías": "mediatizar",
    "mediatizas": "mediatizar",
    "mediatizase": "mediatizar",
    "mediatizaseis": "mediatizar",
    "mediatizasen": "mediatizar",
    "mediatizases": "mediatizar",
    "mediatizaste": "mediatizar",
    "mediatizasteis": "mediatizar",
    "mediatizo": "mediatizar",
    "mediatizábamos": "mediatizar",
    "mediatizáis": "mediatizar",
    "mediatizáramos": "mediatizar",
    "mediatizáremos": "mediatizar",
    "mediatizásemos": "mediatizar",
    "mediatizó": "mediatizar",
    "mediatos": "mediato",
    "mediatrices": "mediatriz",
    "medibilidades": "medibilidad",
    "medibles": "medible",
    "medica": "medicar",
    "medicaba": "medicar",
    "medicabais": "medicar",
    "medicaban": "medicar",
    "medicabas": "medicar",
    "medicables": "medicable",
    "medicaciones": "medicación",
    "medicad": "medicar",
    "medicada": "medicar",
    "medicadas": "medicar",
    "medicado": "medicar",
    "medicados": "medicar",
    "medicalizaciones": "medicalización",
    "medicamentazos": "medicamentazo",
    "medicamentos": "medicamento",
    "medicamentosa": "medicamentoso",
    "medicamentosas": "medicamentoso",
    "medicamentosos": "medicamentoso",
    "medicamos": "medicar",
    "medican": "medicar",
    "medicando": "medicar",
    "medicara": "medicar",
    "medicarais": "medicar",
    "medicaran": "medicar",
    "medicaras": "medicar",
    "medicare": "medicar",
    "medicareis": "medicar",
    "medicaremos": "medicar",
    "medicaren": "medicar",
    "medicares": "medicar",
    "medicaron": "medicar",
    "medicará": "medicar",
    "medicarán": "medicar",
    "medicarás": "medicar",
    "medicaré": "medicar",
    "medicaréis": "medicar",
    "medicaría": "medicar",
    "medicaríais": "medicar",
    "medicaríamos": "medicar",
    "medicarían": "medicar",
    "medicarías": "medicar",
    "medicas": "medicar",
    "medicase": "medicar",
    "medicaseis": "medicar",
    "medicasen": "medicar",
    "medicases": "medicar",
    "medicaste": "medicar",
    "medicasteis": "medicar",
    "medicastros": "medicastro",
    "medicina": "medicinar",
    "medicinaba": "medicinar",
    "medicinabais": "medicinar",
    "medicinaban": "medicinar",
    "medicinabas": "medicinar",
    "medicinad": "medicinar",
    "medicinada": "medicinar",
    "medicinadas": "medicinar",
    "medicinado": "medicinar",
    "medicinados": "medicinar",
    "medicinales": "medicinal",
    "medicinamos": "medicinar",
    "medicinan": "medicinar",
    "medicinando": "medicinar",
    "medicinara": "medicinar",
    "medicinarais": "medicinar",
    "medicinaran": "medicinar",
    "medicinaras": "medicinar",
    "medicinare": "medicinar",
    "medicinareis": "medicinar",
    "medicinaremos": "medicinar",
    "medicinaren": "medicinar",
    "medicinares": "medicinar",
    "medicinaron": "medicinar",
    "medicinará": "medicinar",
    "medicinarán": "medicinar",
    "medicinarás": "medicinar",
    "medicinaré": "medicinar",
    "medicinaréis": "medicinar",
    "medicinaría": "medicinar",
    "medicinaríais": "medicinar",
    "medicinaríamos": "medicinar",
    "medicinarían": "medicinar",
    "medicinarías": "medicinar",
    "medicinas": "medicinar",
    "medicinase": "medicinar",
    "medicinaseis": "medicinar",
    "medicinasen": "medicinar",
    "medicinases": "medicinar",
    "medicinaste": "medicinar",
    "medicinasteis": "medicinar",
    "medicine": "medicinar",
    "medicinemos": "medicinar",
    "medicinen": "medicinar",
    "medicines": "medicinar",
    "medicino": "medicinar",
    "medicinábamos": "medicinar",
    "medicináis": "medicinar",
    "medicináramos": "medicinar",
    "medicináremos": "medicinar",
    "medicinásemos": "medicinar",
    "mediciné": "medicinar",
    "medicinéis": "medicinar",
    "medicinó": "medicinar",
    "mediciones": "medición",
    "medico": "medicar",
    "medicoforenses": "medicoforense",
    "medicuchos": "medicucho",
    "medicábamos": "medicar",
    "medicáis": "medicar",
    "medicáramos": "medicar",
    "medicáremos": "medicar",
    "medicásemos": "medicar",
    "medicó": "medicar",
    "medid": "medir",
    "medida": "medir",
    "medidas": "medir",
    "medido": "medir",
    "medidora": "medidor",
    "medidoras": "medidor",
    "medidores": "medidor",
    "medidos": "medir",
    "medie": "mediar",
    "mediemos": "mediar",
    "medien": "mediar",
    "mediera": "mediero",
    "medieras": "mediero",
    "medieros": "mediero",
    "medierías": "mediería",
    "medies": "mediar",
    "medievales": "medieval",
    "medievalidades": "medievalidad",
    "medievalismos": "medievalismo",
    "medievalistas": "medievalista",
    "medievos": "medievo",
    "medimos": "medir",
    "medio": "mediar",
    "medioambientales": "medioambiental",
    "medioambientalistas": "medioambientalista",
    "mediocampistas": "mediocampista",
    "mediocampos": "mediocampo",
    "mediocarpiana": "mediocarpiano",
    "mediocarpianas": "mediocarpiano",
    "mediocarpianos": "mediocarpiano",
    "mediocentros": "mediocentro",
    "medioclaviculares": "medioclavicular",
    "mediocres": "mediocre",
    "mediocridades": "mediocridad",
    "mediodías": "mediodía",
    "medioevales": "medioeval",
    "medioevos": "medioevo",
    "mediofondistas": "mediofondista",
    "mediomundos": "mediomundo",
    "mediopaños": "mediopaño",
    "mediopensionistas": "mediopensionista",
    "medios": "medio",
    "mediotarsiana": "mediotarsiano",
    "mediotarsianas": "mediotarsiano",
    "mediotarsianos": "mediotarsiano",
    "medique": "medicar",
    "mediquemos": "medicar",
    "mediquen": "medicar",
    "mediques": "medicar",
    "mediquillos": "mediquillo",
    "mediqué": "medicar",
    "mediquéis": "medicar",
    "mediremos": "medir",
    "medirá": "medir",
    "medirán": "medir",
    "medirás": "medir",
    "mediré": "medir",
    "mediréis": "medir",
    "mediría": "medir",
    "mediríais": "medir",
    "mediríamos": "medir",
    "medirían": "medir",
    "medirías": "medir",
    "mediste": "medir",
    "medisteis": "medir",
    "medita": "meditar",
    "meditaba": "meditar",
    "meditabais": "meditar",
    "meditaban": "meditar",
    "meditabas": "meditar",
    "meditabunda": "meditabundo",
    "meditabundas": "meditabundo",
    "meditabundos": "meditabundo",
    "meditaciones": "meditación",
    "meditaciónes": "meditación",
    "meditad": "meditar",
    "meditada": "meditar",
    "meditadas": "meditar",
    "meditado": "meditar",
    "meditadora": "meditador",
    "meditadoras": "meditador",
    "meditadores": "meditador",
    "meditados": "meditar",
    "meditamos": "meditar",
    "meditan": "meditar",
    "meditando": "meditar",
    "meditara": "meditar",
    "meditarais": "meditar",
    "meditaran": "meditar",
    "meditaras": "meditar",
    "meditare": "meditar",
    "meditareis": "meditar",
    "meditaremos": "meditar",
    "meditaren": "meditar",
    "meditares": "meditar",
    "meditaron": "meditar",
    "meditará": "meditar",
    "meditarán": "meditar",
    "meditarás": "meditar",
    "meditaré": "meditar",
    "meditaréis": "meditar",
    "meditaría": "meditar",
    "meditaríais": "meditar",
    "meditaríamos": "meditar",
    "meditarían": "meditar",
    "meditarías": "meditar",
    "meditas": "meditar",
    "meditase": "meditar",
    "meditaseis": "meditar",
    "meditasen": "meditar",
    "meditases": "meditar",
    "meditaste": "meditar",
    "meditasteis": "meditar",
    "meditativa": "meditativo",
    "meditativas": "meditativo",
    "meditativos": "meditativo",
    "medite": "meditar",
    "meditemos": "meditar",
    "mediten": "meditar",
    "mediterránea": "mediterráneo",
    "mediterráneas": "mediterráneo",
    "mediterráneos": "mediterráneo",
    "medites": "meditar",
    "medito": "meditar",
    "meditábamos": "meditar",
    "meditáis": "meditar",
    "meditáramos": "meditar",
    "meditáremos": "meditar",
    "meditásemos": "meditar",
    "medité": "meditar",
    "meditéis": "meditar",
    "meditó": "meditar",
    "medium": "médium",
    "mediums": "médium",
    "mediábamos": "mediar",
    "mediáis": "mediar",
    "mediáramos": "mediar",
    "mediáremos": "mediar",
    "mediásemos": "mediar",
    "mediática": "mediático",
    "mediáticas": "mediático",
    "mediáticos": "mediático",
    "medié": "mediar",
    "mediéis": "mediar",
    "medió": "mediar",
    "medos": "medo",
    "medra": "medrar",
    "medraba": "medrar",
    "medrabais": "medrar",
    "medraban": "medrar",
    "medrabas": "medrar",
    "medrad": "medrar",
    "medrada": "medrar",
    "medradas": "medrar",
    "medrado": "medrar",
    "medrados": "medrar",
    "medramos": "medrar",
    "medran": "medrar",
    "medranas": "medrana",
    "medrando": "medrar",
    "medrara": "medrar",
    "medrarais": "medrar",
    "medraran": "medrar",
    "medraras": "medrar",
    "medrare": "medrar",
    "medrareis": "medrar",
    "medraremos": "medrar",
    "medraren": "medrar",
    "medrares": "medrar",
    "medraron": "medrar",
    "medrará": "medrar",
    "medrarán": "medrar",
    "medrarás": "medrar",
    "medraré": "medrar",
    "medraréis": "medrar",
    "medraría": "medrar",
    "medraríais": "medrar",
    "medraríamos": "medrar",
    "medrarían": "medrar",
    "medrarías": "medrar",
    "medras": "medrar",
    "medrase": "medrar",
    "medraseis": "medrar",
    "medrasen": "medrar",
    "medrases": "medrar",
    "medraste": "medrar",
    "medrasteis": "medrar",
    "medre": "medrar",
    "medremos": "medrar",
    "medren": "medrar",
    "medres": "medrar",
    "medro": "medrar",
    "medros": "medro",
    "medrosa": "medroso",
    "medrosas": "medroso",
    "medrosos": "medroso",
    "medrábamos": "medrar",
    "medráis": "medrar",
    "medráramos": "medrar",
    "medráremos": "medrar",
    "medrásemos": "medrar",
    "medré": "medrar",
    "medréis": "medrar",
    "medró": "medrar",
    "medulada": "medulado",
    "meduladas": "medulado",
    "medulados": "medulado",
    "medulares": "medular",
    "medulas": "medula",
    "medusas": "medusa",
    "medí": "medir",
    "medía": "medir",
    "medíais": "medir",
    "medíamos": "medir",
    "medían": "medir",
    "medías": "medir",
    "medís": "medir",
    "mee": "mear",
    "meemos": "mear",
    "meen": "mear",
    "mees": "mear",
    "mefistofélica": "mefistofélico",
    "mefistofélicas": "mefistofélico",
    "mefistofélicos": "mefistofélico",
    "mefítica": "mefítico",
    "mefíticas": "mefítico",
    "mefíticos": "mefítico",
    "megabits": "megabit",
    "megabytes": "megabyte",
    "megacarioblastos": "megacarioblasto",
    "megacariocitos": "megacariocito",
    "megacefalias": "megacefalia",
    "megaciclos": "megaciclo",
    "megadonta": "megadonto",
    "megadontas": "megadonto",
    "megadontismos": "megadontismo",
    "megadontos": "megadonto",
    "megafonías": "megafonía",
    "megagramos": "megagramo",
    "megahercios": "megahercio",
    "megahertz": "megahercio",
    "megahertzios": "megahertzio",
    "megajulios": "megajulio",
    "megalitos": "megalito",
    "megaloblastos": "megaloblasto",
    "megaloblástica": "megaloblástico",
    "megaloblásticas": "megaloblástico",
    "megaloblásticos": "megaloblástico",
    "megalocariocitos": "megalocariocito",
    "megalocitos": "megalocito",
    "megalocórneas": "megalocórnea",
    "megalodoncias": "megalodoncia",
    "megaloglosias": "megaloglosia",
    "megalomaníaca": "megalomaníaco",
    "megalomaníacas": "megalomaníaco",
    "megalomaníacos": "megalomaníaco",
    "megalomanías": "megalomanía",
    "megalítica": "megalítico",
    "megalíticas": "megalítico",
    "megalíticos": "megalítico",
    "megalómana": "megalómano",
    "megalómanas": "megalómano",
    "megalómanos": "megalómano",
    "meganéwtones": "meganewton",
    "megaoctetos": "megaocteto",
    "megaohmes": "megaohm",
    "megapascales": "megapascal",
    "megapascals": "megapascal",
    "megarones": "megarón",
    "megas": "mego",
    "megascopios": "megascopio",
    "megasporocitos": "megasporocito",
    "megaterios": "megaterio",
    "megatones": "megatón",
    "megatérmica": "megatérmico",
    "megatérmicas": "megatérmico",
    "megatérmicos": "megatérmico",
    "megavatios": "megavatio",
    "megawatios": "megawatio",
    "meggioristas": "meggiorista",
    "megos": "mego",
    "megáfonos": "megáfono",
    "megóhmetros": "megóhmetro",
    "mehalas": "mehala",
    "meharis": "mehari",
    "meiga": "meigo",
    "meigas": "meigo",
    "meigos": "meigo",
    "meiocitos": "meiocito",
    "meiosomas": "meiosoma",
    "meiótica": "meiótico",
    "meióticas": "meiótico",
    "meióticos": "meiótico",
    "mejanas": "mejana",
    "mejicana": "mejicano",
    "mejicanas": "mejicano",
    "mejicanismos": "mejicanismo",
    "mejicanos": "mejicano",
    "mejillas": "mejilla",
    "mejillonera": "mejillonero",
    "mejilloneras": "mejillonero",
    "mejilloneros": "mejillonero",
    "mejillones": "mejillón",
    "mejora": "mejorar",
    "mejoraba": "mejorar",
    "mejorabais": "mejorar",
    "mejoraban": "mejorar",
    "mejorabas": "mejorar",
    "mejorables": "mejorable",
    "mejorad": "mejorar",
    "mejorada": "mejorar",
    "mejoradas": "mejorar",
    "mejorado": "mejorar",
    "mejoradora": "mejorador",
    "mejoradoras": "mejorador",
    "mejoradores": "mejorador",
    "mejorados": "mejorar",
    "mejoramientos": "mejoramiento",
    "mejoramos": "mejorar",
    "mejoran": "mejorar",
    "mejoranas": "mejorana",
    "mejorando": "mejorar",
    "mejorara": "mejorar",
    "mejorarais": "mejorar",
    "mejoraran": "mejorar",
    "mejoraras": "mejorar",
    "mejorare": "mejorar",
    "mejorareis": "mejorar",
    "mejoraremos": "mejorar",
    "mejoraren": "mejorar",
    "mejorares": "mejorar",
    "mejoraron": "mejorar",
    "mejorará": "mejorar",
    "mejorarán": "mejorar",
    "mejorarás": "mejorar",
    "mejoraré": "mejorar",
    "mejoraréis": "mejorar",
    "mejoraría": "mejorar",
    "mejoraríais": "mejorar",
    "mejoraríamos": "mejorar",
    "mejorarían": "mejorar",
    "mejorarías": "mejorar",
    "mejoras": "mejorar",
    "mejorase": "mejorar",
    "mejoraseis": "mejorar",
    "mejorasen": "mejorar",
    "mejorases": "mejorar",
    "mejoraste": "mejorar",
    "mejorasteis": "mejorar",
    "mejore": "mejorar",
    "mejoremos": "mejorar",
    "mejoren": "mejorar",
    "mejores": "mejorar",
    "mejoro": "mejorar",
    "mejorábamos": "mejorar",
    "mejoráis": "mejorar",
    "mejoráramos": "mejorar",
    "mejoráremos": "mejorar",
    "mejorásemos": "mejorar",
    "mejoré": "mejorar",
    "mejoréis": "mejorar",
    "mejorías": "mejoría",
    "mejoró": "mejorar",
    "mejunjes": "mejunje",
    "melaba": "melar",
    "melabais": "melar",
    "melaban": "melar",
    "melabas": "melar",
    "melad": "melar",
    "melada": "melar",
    "meladas": "melar",
    "melado": "melar",
    "melados": "melar",
    "meladuras": "meladura",
    "melaminas": "melamina",
    "melamos": "melar",
    "melampos": "melampo",
    "melancolice": "melancolizar",
    "melancolicemos": "melancolizar",
    "melancolicen": "melancolizar",
    "melancolices": "melancolizar",
    "melancolicé": "melancolizar",
    "melancolicéis": "melancolizar",
    "melancoliza": "melancolizar",
    "melancolizaba": "melancolizar",
    "melancolizabais": "melancolizar",
    "melancolizaban": "melancolizar",
    "melancolizabas": "melancolizar",
    "melancolizad": "melancolizar",
    "melancolizada": "melancolizar",
    "melancolizadas": "melancolizar",
    "melancolizado": "melancolizar",
    "melancolizados": "melancolizar",
    "melancolizamos": "melancolizar",
    "melancolizan": "melancolizar",
    "melancolizando": "melancolizar",
    "melancolizara": "melancolizar",
    "melancolizarais": "melancolizar",
    "melancolizaran": "melancolizar",
    "melancolizaras": "melancolizar",
    "melancolizare": "melancolizar",
    "melancolizareis": "melancolizar",
    "melancolizaremos": "melancolizar",
    "melancolizaren": "melancolizar",
    "melancolizares": "melancolizar",
    "melancolizaron": "melancolizar",
    "melancolizará": "melancolizar",
    "melancolizarán": "melancolizar",
    "melancolizarás": "melancolizar",
    "melancolizaré": "melancolizar",
    "melancolizaréis": "melancolizar",
    "melancolizaría": "melancolizar",
    "melancolizaríais": "melancolizar",
    "melancolizaríamos": "melancolizar",
    "melancolizarían": "melancolizar",
    "melancolizarías": "melancolizar",
    "melancolizas": "melancolizar",
    "melancolizase": "melancolizar",
    "melancolizaseis": "melancolizar",
    "melancolizasen": "melancolizar",
    "melancolizases": "melancolizar",
    "melancolizaste": "melancolizar",
    "melancolizasteis": "melancolizar",
    "melancolizo": "melancolizar",
    "melancolizábamos": "melancolizar",
    "melancolizáis": "melancolizar",
    "melancolizáramos": "melancolizar",
    "melancolizáremos": "melancolizar",
    "melancolizásemos": "melancolizar",
    "melancolizó": "melancolizar",
    "melancolías": "melancolía",
    "melancólica": "melancólico",
    "melancólicas": "melancólico",
    "melancólicos": "melancólico",
    "melando": "melar",
    "melanesios": "melanesio",
    "melaninas": "melanina",
    "melanitas": "melanita",
    "melanizaciones": "melanización",
    "melanocitos": "melanocito",
    "melanodermas": "melanoderma",
    "melanodermias": "melanodermia",
    "melanoglosias": "melanoglosia",
    "melanoindias": "melanoindia",
    "melanomas": "melanoma",
    "melanosomas": "melanosoma",
    "melanteritas": "melanterita",
    "melanurias": "melanuria",
    "melanésicos": "melanésico",
    "melapias": "melapia",
    "melara": "melar",
    "melarais": "melar",
    "melaran": "melar",
    "melaras": "melar",
    "melare": "melar",
    "melareis": "melar",
    "melaremos": "melar",
    "melaren": "melar",
    "melares": "melar",
    "melaron": "melar",
    "melará": "melar",
    "melarán": "melar",
    "melarás": "melar",
    "melaré": "melar",
    "melaréis": "melar",
    "melaría": "melar",
    "melaríais": "melar",
    "melaríamos": "melar",
    "melarían": "melar",
    "melarías": "melar",
    "melase": "melar",
    "melaseis": "melar",
    "melasen": "melar",
    "melases": "melar",
    "melaste": "melar",
    "melasteis": "melar",
    "melastomatácea": "melastomatáceo",
    "melastomatáceas": "melastomatáceo",
    "melastomatáceos": "melastomatáceo",
    "melazas": "melaza",
    "melcochas": "melcocha",
    "melcochera": "melcochero",
    "melcocheras": "melcochero",
    "melcocheros": "melcochero",
    "meldómetros": "meldómetro",
    "melemos": "melar",
    "melena": "meleno",
    "melenas": "meleno",
    "meleneras": "melenera",
    "melenos": "meleno",
    "melenuda": "melenudo",
    "melenudas": "melenudo",
    "melenudos": "melenudo",
    "melera": "melero",
    "meleras": "melero",
    "meleros": "melero",
    "melga": "melgo",
    "melgares": "melgar",
    "melgas": "melgo",
    "melgos": "melgo",
    "melifica": "melificar",
    "melificaba": "melificar",
    "melificabais": "melificar",
    "melificaban": "melificar",
    "melificabas": "melificar",
    "melificaciones": "melificación",
    "melificad": "melificar",
    "melificada": "melificar",
    "melificadas": "melificar",
    "melificado": "melificar",
    "melificados": "melificar",
    "melificamos": "melificar",
    "melifican": "melificar",
    "melificando": "melificar",
    "melificara": "melificar",
    "melificarais": "melificar",
    "melificaran": "melificar",
    "melificaras": "melificar",
    "melificare": "melificar",
    "melificareis": "melificar",
    "melificaremos": "melificar",
    "melificaren": "melificar",
    "melificares": "melificar",
    "melificaron": "melificar",
    "melificará": "melificar",
    "melificarán": "melificar",
    "melificarás": "melificar",
    "melificaré": "melificar",
    "melificaréis": "melificar",
    "melificaría": "melificar",
    "melificaríais": "melificar",
    "melificaríamos": "melificar",
    "melificarían": "melificar",
    "melificarías": "melificar",
    "melificas": "melificar",
    "melificase": "melificar",
    "melificaseis": "melificar",
    "melificasen": "melificar",
    "melificases": "melificar",
    "melificaste": "melificar",
    "melificasteis": "melificar",
    "melifico": "melificar",
    "melificábamos": "melificar",
    "melificáis": "melificar",
    "melificáramos": "melificar",
    "melificáremos": "melificar",
    "melificásemos": "melificar",
    "melificó": "melificar",
    "melifique": "melificar",
    "melifiquemos": "melificar",
    "melifiquen": "melificar",
    "melifiques": "melificar",
    "melifiqué": "melificar",
    "melifiquéis": "melificar",
    "meliflua": "melifluo",
    "melifluas": "melifluo",
    "melifluencias": "melifluencia",
    "melifluidades": "melifluidad",
    "melifluos": "melifluo",
    "melilitas": "melilita",
    "melillenses": "melillense",
    "melilotos": "meliloto",
    "melindrera": "melindrero",
    "melindreras": "melindrero",
    "melindreros": "melindrero",
    "melindrerías": "melindrería",
    "melindres": "melindre",
    "melindros": "melindro",
    "melindrosa": "melindroso",
    "melindrosas": "melindroso",
    "melindrosos": "melindroso",
    "melinitas": "melinita",
    "meliorativa": "meliorativo",
    "meliorativas": "meliorativo",
    "meliorativos": "meliorativo",
    "melisas": "melisa",
    "melismas": "melisma",
    "melitos": "melito",
    "melitosas": "melitosa",
    "meliácea": "meliáceo",
    "meliáceas": "meliáceo",
    "meliáceos": "meliáceo",
    "melkitas": "melkita",
    "mella": "mellar",
    "mellaba": "mellar",
    "mellabais": "mellar",
    "mellaban": "mellar",
    "mellabas": "mellar",
    "mellad": "mellar",
    "mellada": "mellar",
    "melladas": "mellar",
    "mellado": "mellar",
    "mellados": "mellar",
    "mellamos": "mellar",
    "mellan": "mellar",
    "mellando": "mellar",
    "mellara": "mellar",
    "mellarais": "mellar",
    "mellaran": "mellar",
    "mellaras": "mellar",
    "mellare": "mellar",
    "mellareis": "mellar",
    "mellaremos": "mellar",
    "mellaren": "mellar",
    "mellares": "mellar",
    "mellaron": "mellar",
    "mellará": "mellar",
    "mellarán": "mellar",
    "mellarás": "mellar",
    "mellaré": "mellar",
    "mellaréis": "mellar",
    "mellaría": "mellar",
    "mellaríais": "mellar",
    "mellaríamos": "mellar",
    "mellarían": "mellar",
    "mellarías": "mellar",
    "mellas": "mellar",
    "mellase": "mellar",
    "mellaseis": "mellar",
    "mellasen": "mellar",
    "mellases": "mellar",
    "mellaste": "mellar",
    "mellasteis": "mellar",
    "melle": "mellar",
    "mellemos": "mellar",
    "mellen": "mellar",
    "melles": "mellar",
    "melliza": "mellizo",
    "mellizas": "mellizo",
    "mellizos": "mellizo",
    "mello": "mellar",
    "mellones": "mellón",
    "mellábamos": "mellar",
    "melláis": "mellar",
    "melláramos": "mellar",
    "melláremos": "mellar",
    "mellásemos": "mellar",
    "mellé": "mellar",
    "melléis": "mellar",
    "melló": "mellar",
    "melocotonares": "melocotonar",
    "melocotoneros": "melocotonero",
    "melocotones": "melocotón",
    "melodiosa": "melodioso",
    "melodiosas": "melodioso",
    "melodiosos": "melodioso",
    "melodistas": "melodista",
    "melodramas": "melodrama",
    "melodramática": "melodramático",
    "melodramáticas": "melodramático",
    "melodramáticos": "melodramático",
    "melodreñas": "melodreña",
    "melodías": "melodía",
    "melografías": "melografía",
    "melojares": "melojar",
    "melojas": "meloja",
    "melojos": "melojo",
    "melomanías": "melomanía",
    "melona": "melón",
    "melonares": "melonar",
    "melonas": "melón",
    "meloncillos": "meloncillo",
    "melonera": "melonero",
    "meloneras": "melonero",
    "meloneros": "melonero",
    "melones": "melón",
    "melopeas": "melopea",
    "melopeyas": "melopeya",
    "melosa": "meloso",
    "melosas": "meloso",
    "melosidades": "melosidad",
    "melosos": "meloso",
    "melotes": "melote",
    "meltones": "meltón",
    "melusinas": "melusina",
    "melvas": "melva",
    "melábamos": "melar",
    "meláfidos": "meláfido",
    "meláis": "melar",
    "meláramos": "melar",
    "meláremos": "melar",
    "melásemos": "melar",
    "melé": "melar",
    "meléis": "melar",
    "melés": "melé",
    "melífera": "melífero",
    "melíferas": "melífero",
    "melíferos": "melífero",
    "melísica": "melísico",
    "melísicas": "melísico",
    "melísicos": "melísico",
    "meló": "melar",
    "melódica": "melódico",
    "melódicas": "melódico",
    "melódicos": "melódico",
    "melómana": "melómano",
    "melómanas": "melómano",
    "melómanos": "melómano",
    "mema": "memo",
    "memadas": "memada",
    "memas": "memo",
    "membrada": "membrado",
    "membradas": "membrado",
    "membrados": "membrado",
    "membranas": "membrana",
    "membranoproliferativa": "membranoproliferativo",
    "membranoproliferativas": "membranoproliferativo",
    "membranoproliferativos": "membranoproliferativo",
    "membranosa": "membranoso",
    "membranosas": "membranoso",
    "membranosos": "membranoso",
    "membretes": "membrete",
    "membrilleros": "membrillero",
    "membrillos": "membrillo",
    "membruda": "membrudo",
    "membrudas": "membrudo",
    "membrudos": "membrudo",
    "memeces": "memez",
    "mementos": "memento",
    "memnónidas": "memnónida",
    "memora": "memorar",
    "memoraba": "memorar",
    "memorabais": "memorar",
    "memoraban": "memorar",
    "memorabas": "memorar",
    "memorabilidades": "memorabilidad",
    "memorables": "memorable",
    "memorad": "memorar",
    "memorada": "memorar",
    "memoradas": "memorar",
    "memorado": "memorar",
    "memorados": "memorar",
    "memoramos": "memorar",
    "memoran": "memorar",
    "memoranda": "memorando",
    "memorandas": "memorando",
    "memorando": "memorar",
    "memorandos": "memorando",
    "memorara": "memorar",
    "memorarais": "memorar",
    "memoraran": "memorar",
    "memoraras": "memorar",
    "memorare": "memorar",
    "memorareis": "memorar",
    "memoraremos": "memorar",
    "memoraren": "memorar",
    "memorares": "memorar",
    "memoraron": "memorar",
    "memorará": "memorar",
    "memorarán": "memorar",
    "memorarás": "memorar",
    "memoraré": "memorar",
    "memoraréis": "memorar",
    "memoraría": "memorar",
    "memoraríais": "memorar",
    "memoraríamos": "memorar",
    "memorarían": "memorar",
    "memorarías": "memorar",
    "memoras": "memorar",
    "memorase": "memorar",
    "memoraseis": "memorar",
    "memorasen": "memorar",
    "memorases": "memorar",
    "memoraste": "memorar",
    "memorasteis": "memorar",
    "memorativa": "memorativo",
    "memorativas": "memorativo",
    "memorativos": "memorativo",
    "memore": "memorar",
    "memoremos": "memorar",
    "memoren": "memorar",
    "memores": "memorar",
    "memoriales": "memorial",
    "memorialesca": "memorialesco",
    "memorialescas": "memorialesco",
    "memorialescos": "memorialesco",
    "memorialistas": "memorialista",
    "memorias": "memoria",
    "memorice": "memorizar",
    "memoricemos": "memorizar",
    "memoricen": "memorizar",
    "memorices": "memorizar",
    "memoricé": "memorizar",
    "memoricéis": "memorizar",
    "memoriona": "memorión",
    "memorionas": "memorión",
    "memoriones": "memorión",
    "memoriosa": "memorioso",
    "memoriosas": "memorioso",
    "memoriosos": "memorioso",
    "memorismos": "memorismo",
    "memoristas": "memorista",
    "memoriza": "memorizar",
    "memorizaba": "memorizar",
    "memorizabais": "memorizar",
    "memorizaban": "memorizar",
    "memorizabas": "memorizar",
    "memorizaciones": "memorización",
    "memorizad": "memorizar",
    "memorizada": "memorizar",
    "memorizadas": "memorizar",
    "memorizado": "memorizar",
    "memorizados": "memorizar",
    "memorizamos": "memorizar",
    "memorizan": "memorizar",
    "memorizando": "memorizar",
    "memorizara": "memorizar",
    "memorizarais": "memorizar",
    "memorizaran": "memorizar",
    "memorizaras": "memorizar",
    "memorizare": "memorizar",
    "memorizareis": "memorizar",
    "memorizaremos": "memorizar",
    "memorizaren": "memorizar",
    "memorizares": "memorizar",
    "memorizaron": "memorizar",
    "memorizará": "memorizar",
    "memorizarán": "memorizar",
    "memorizarás": "memorizar",
    "memorizaré": "memorizar",
    "memorizaréis": "memorizar",
    "memorizaría": "memorizar",
    "memorizaríais": "memorizar",
    "memorizaríamos": "memorizar",
    "memorizarían": "memorizar",
    "memorizarías": "memorizar",
    "memorizas": "memorizar",
    "memorizase": "memorizar",
    "memorizaseis": "memorizar",
    "memorizasen": "memorizar",
    "memorizases": "memorizar",
    "memorizaste": "memorizar",
    "memorizasteis": "memorizar",
    "memorizo": "memorizar",
    "memorizábamos": "memorizar",
    "memorizáis": "memorizar",
    "memorizáramos": "memorizar",
    "memorizáremos": "memorizar",
    "memorizásemos": "memorizar",
    "memorizó": "memorizar",
    "memoro": "memorar",
    "memorábamos": "memorar",
    "memoráis": "memorar",
    "memorándums": "memorándum",
    "memoráramos": "memorar",
    "memoráremos": "memorar",
    "memorásemos": "memorar",
    "memoré": "memorar",
    "memoréis": "memorar",
    "memorística": "memorístico",
    "memorísticas": "memorístico",
    "memorísticos": "memorístico",
    "memoró": "memorar",
    "memos": "memo",
    "menajes": "menaje",
    "menarquías": "menarquía",
    "menas": "mena",
    "mencheviques": "menchevique",
    "menciona": "mencionar",
    "mencionaba": "mencionar",
    "mencionabais": "mencionar",
    "mencionaban": "mencionar",
    "mencionabas": "mencionar",
    "mencionad": "mencionar",
    "mencionada": "mencionar",
    "mencionadas": "mencionar",
    "mencionado": "mencionar",
    "mencionados": "mencionar",
    "mencionamos": "mencionar",
    "mencionan": "mencionar",
    "mencionando": "mencionar",
    "mencionara": "mencionar",
    "mencionarais": "mencionar",
    "mencionaran": "mencionar",
    "mencionaras": "mencionar",
    "mencionare": "mencionar",
    "mencionareis": "mencionar",
    "mencionaremos": "mencionar",
    "mencionaren": "mencionar",
    "mencionares": "mencionar",
    "mencionaron": "mencionar",
    "mencionará": "mencionar",
    "mencionarán": "mencionar",
    "mencionarás": "mencionar",
    "mencionaré": "mencionar",
    "mencionaréis": "mencionar",
    "mencionaría": "mencionar",
    "mencionaríais": "mencionar",
    "mencionaríamos": "mencionar",
    "mencionarían": "mencionar",
    "mencionarías": "mencionar",
    "mencionas": "mencionar",
    "mencionase": "mencionar",
    "mencionaseis": "mencionar",
    "mencionasen": "mencionar",
    "mencionases": "mencionar",
    "mencionaste": "mencionar",
    "mencionasteis": "mencionar",
    "mencione": "mencionar",
    "mencionemos": "mencionar",
    "mencionen": "mencionar",
    "menciones": "mencionar",
    "menciono": "mencionar",
    "mencionábamos": "mencionar",
    "mencionáis": "mencionar",
    "mencionáramos": "mencionar",
    "mencionáremos": "mencionar",
    "mencionásemos": "mencionar",
    "mencioné": "mencionar",
    "mencionéis": "mencionar",
    "mencionó": "mencionar",
    "mendaces": "mendaz",
    "mendacidades": "mendacidad",
    "mendazes": "mendaz",
    "mendelevios": "mendelevio",
    "mendeliana": "mendeliano",
    "mendelianas": "mendeliano",
    "mendelianos": "mendeliano",
    "mendelismos": "mendelismo",
    "mendicaciones": "mendicación",
    "mendicantes": "mendicante",
    "mendicidades": "mendicidad",
    "mendiga": "mendigo",
    "mendigaba": "mendigar",
    "mendigabais": "mendigar",
    "mendigaban": "mendigar",
    "mendigabas": "mendigar",
    "mendigad": "mendigar",
    "mendigada": "mendigar",
    "mendigadas": "mendigar",
    "mendigado": "mendigar",
    "mendigados": "mendigar",
    "mendigamos": "mendigar",
    "mendigan": "mendigar",
    "mendigando": "mendigar",
    "mendigara": "mendigar",
    "mendigarais": "mendigar",
    "mendigaran": "mendigar",
    "mendigaras": "mendigar",
    "mendigare": "mendigar",
    "mendigareis": "mendigar",
    "mendigaremos": "mendigar",
    "mendigaren": "mendigar",
    "mendigares": "mendigar",
    "mendigaron": "mendigar",
    "mendigará": "mendigar",
    "mendigarán": "mendigar",
    "mendigarás": "mendigar",
    "mendigaré": "mendigar",
    "mendigaréis": "mendigar",
    "mendigaría": "mendigar",
    "mendigaríais": "mendigar",
    "mendigaríamos": "mendigar",
    "mendigarían": "mendigar",
    "mendigarías": "mendigar",
    "mendigas": "mendigo",
    "mendigase": "mendigar",
    "mendigaseis": "mendigar",
    "mendigasen": "mendigar",
    "mendigases": "mendigar",
    "mendigaste": "mendigar",
    "mendigasteis": "mendigar",
    "mendigo": "mendigar",
    "mendigos": "mendigo",
    "mendigue": "mendigar",
    "mendiguemos": "mendigar",
    "mendiguen": "mendigar",
    "mendigues": "mendigar",
    "mendigué": "mendigar",
    "mendiguéis": "mendigar",
    "mendigábamos": "mendigar",
    "mendigáis": "mendigar",
    "mendigáramos": "mendigar",
    "mendigáremos": "mendigar",
    "mendigásemos": "mendigar",
    "mendigó": "mendigar",
    "mendosa": "mendoso",
    "mendosas": "mendoso",
    "mendosos": "mendoso",
    "mendrugos": "mendrugo",
    "menea": "menear",
    "meneaba": "menear",
    "meneabais": "menear",
    "meneaban": "menear",
    "meneabas": "menear",
    "menead": "menear",
    "meneada": "menear",
    "meneadas": "menear",
    "meneado": "menear",
    "meneados": "menear",
    "meneamos": "menear",
    "menean": "menear",
    "meneando": "menear",
    "meneara": "menear",
    "menearais": "menear",
    "menearan": "menear",
    "menearas": "menear",
    "meneare": "menear",
    "meneareis": "menear",
    "menearemos": "menear",
    "menearen": "menear",
    "meneares": "menear",
    "menearon": "menear",
    "meneará": "menear",
    "menearán": "menear",
    "menearás": "menear",
    "menearé": "menear",
    "menearéis": "menear",
    "menearía": "menear",
    "menearíais": "menear",
    "menearíamos": "menear",
    "menearían": "menear",
    "menearías": "menear",
    "meneas": "menear",
    "menease": "menear",
    "meneaseis": "menear",
    "meneasen": "menear",
    "meneases": "menear",
    "meneaste": "menear",
    "meneasteis": "menear",
    "menee": "menear",
    "meneemos": "menear",
    "meneen": "menear",
    "menees": "menear",
    "meneo": "menear",
    "meneos": "meneo",
    "menesteres": "menester",
    "menesterosa": "menesteroso",
    "menesterosas": "menesteroso",
    "menesterosos": "menesteroso",
    "menestrala": "menestral",
    "menestralas": "menestral",
    "menestralerías": "menestralería",
    "menestrales": "menestral",
    "menestralías": "menestralía",
    "menestras": "menestra",
    "menestretes": "menestrete",
    "meneábamos": "menear",
    "meneáis": "menear",
    "meneáramos": "menear",
    "meneáremos": "menear",
    "meneásemos": "menear",
    "meneé": "menear",
    "meneéis": "menear",
    "meneó": "menear",
    "menfitas": "menfita",
    "menfítica": "menfítico",
    "menfíticas": "menfítico",
    "menfíticos": "menfítico",
    "mengana": "mengano",
    "menganas": "mengano",
    "menganos": "mengano",
    "mengua": "menguar",
    "menguaba": "menguar",
    "menguabais": "menguar",
    "menguaban": "menguar",
    "menguabas": "menguar",
    "menguad": "menguar",
    "menguada": "menguar",
    "menguadas": "menguar",
    "menguado": "menguar",
    "menguados": "menguar",
    "menguamos": "menguar",
    "menguan": "menguar",
    "menguando": "menguar",
    "menguantes": "menguante",
    "menguara": "menguar",
    "menguarais": "menguar",
    "menguaran": "menguar",
    "menguaras": "menguar",
    "menguare": "menguar",
    "menguareis": "menguar",
    "menguaremos": "menguar",
    "menguaren": "menguar",
    "menguares": "menguar",
    "menguaron": "menguar",
    "menguará": "menguar",
    "menguarán": "menguar",
    "menguarás": "menguar",
    "menguaré": "menguar",
    "menguaréis": "menguar",
    "menguaría": "menguar",
    "menguaríais": "menguar",
    "menguaríamos": "menguar",
    "menguarían": "menguar",
    "menguarías": "menguar",
    "menguas": "menguar",
    "menguase": "menguar",
    "menguaseis": "menguar",
    "menguasen": "menguar",
    "menguases": "menguar",
    "menguaste": "menguar",
    "menguasteis": "menguar",
    "mengues": "mengue",
    "menguo": "menguar",
    "menguábamos": "menguar",
    "menguáis": "menguar",
    "menguáramos": "menguar",
    "menguáremos": "menguar",
    "menguásemos": "menguar",
    "menguó": "menguar",
    "mengüe": "menguar",
    "mengüemos": "menguar",
    "mengüen": "menguar",
    "mengües": "menguar",
    "mengüé": "menguar",
    "mengüéis": "menguar",
    "menhires": "menhir",
    "menina": "menino",
    "meninas": "menino",
    "meninges": "meninge",
    "meningococos": "meningococo",
    "meningítica": "meningítico",
    "meningíticas": "meningítico",
    "meningíticos": "meningítico",
    "meninos": "menino",
    "meniques": "menique",
    "meniscofemorales": "meniscofemoral",
    "meniscos": "menisco",
    "menispermácea": "menispermáceo",
    "menispermáceas": "menispermáceo",
    "menispermáceos": "menispermáceo",
    "menjunjes": "menjunje",
    "menjurjes": "menjurje",
    "menjuíes": "menjuí",
    "menologios": "menologio",
    "menonias": "menonia",
    "menopausias": "menopausia",
    "menopáusica": "menopáusico",
    "menopáusicas": "menopáusico",
    "menopáusicos": "menopáusico",
    "menores": "menor",
    "menoristas": "menorista",
    "menorquina": "menorquín",
    "menorquinas": "menorquín",
    "menorquines": "menorquín",
    "menorragias": "menorragia",
    "menorrágica": "menorrágico",
    "menorrágicas": "menorrágico",
    "menorrágicos": "menorrágico",
    "menorías": "menoría",
    "menoscaba": "menoscabar",
    "menoscababa": "menoscabar",
    "menoscababais": "menoscabar",
    "menoscababan": "menoscabar",
    "menoscababas": "menoscabar",
    "menoscabad": "menoscabar",
    "menoscabada": "menoscabar",
    "menoscabadas": "menoscabar",
    "menoscabado": "menoscabar",
    "menoscabadora": "menoscabador",
    "menoscabadoras": "menoscabador",
    "menoscabadores": "menoscabador",
    "menoscabados": "menoscabar",
    "menoscabamos": "menoscabar",
    "menoscaban": "menoscabar",
    "menoscabando": "menoscabar",
    "menoscabara": "menoscabar",
    "menoscabarais": "menoscabar",
    "menoscabaran": "menoscabar",
    "menoscabaras": "menoscabar",
    "menoscabare": "menoscabar",
    "menoscabareis": "menoscabar",
    "menoscabaremos": "menoscabar",
    "menoscabaren": "menoscabar",
    "menoscabares": "menoscabar",
    "menoscabaron": "menoscabar",
    "menoscabará": "menoscabar",
    "menoscabarán": "menoscabar",
    "menoscabarás": "menoscabar",
    "menoscabaré": "menoscabar",
    "menoscabaréis": "menoscabar",
    "menoscabaría": "menoscabar",
    "menoscabaríais": "menoscabar",
    "menoscabaríamos": "menoscabar",
    "menoscabarían": "menoscabar",
    "menoscabarías": "menoscabar",
    "menoscabas": "menoscabar",
    "menoscabase": "menoscabar",
    "menoscabaseis": "menoscabar",
    "menoscabasen": "menoscabar",
    "menoscabases": "menoscabar",
    "menoscabaste": "menoscabar",
    "menoscabasteis": "menoscabar",
    "menoscabe": "menoscabar",
    "menoscabemos": "menoscabar",
    "menoscaben": "menoscabar",
    "menoscabes": "menoscabar",
    "menoscabo": "menoscabar",
    "menoscabos": "menoscabo",
    "menoscabábamos": "menoscabar",
    "menoscabáis": "menoscabar",
    "menoscabáramos": "menoscabar",
    "menoscabáremos": "menoscabar",
    "menoscabásemos": "menoscabar",
    "menoscabé": "menoscabar",
    "menoscabéis": "menoscabar",
    "menoscabó": "menoscabar",
    "menosprecia": "menospreciar",
    "menospreciaba": "menospreciar",
    "menospreciabais": "menospreciar",
    "menospreciaban": "menospreciar",
    "menospreciabas": "menospreciar",
    "menospreciables": "menospreciable",
    "menospreciad": "menospreciar",
    "menospreciada": "menospreciar",
    "menospreciadas": "menospreciar",
    "menospreciado": "menospreciar",
    "menospreciadora": "menospreciador",
    "menospreciadoras": "menospreciador",
    "menospreciadores": "menospreciador",
    "menospreciados": "menospreciar",
    "menospreciamos": "menospreciar",
    "menosprecian": "menospreciar",
    "menospreciando": "menospreciar",
    "menospreciara": "menospreciar",
    "menospreciarais": "menospreciar",
    "menospreciaran": "menospreciar",
    "menospreciaras": "menospreciar",
    "menospreciare": "menospreciar",
    "menospreciareis": "menospreciar",
    "menospreciaremos": "menospreciar",
    "menospreciaren": "menospreciar",
    "menospreciares": "menospreciar",
    "menospreciaron": "menospreciar",
    "menospreciará": "menospreciar",
    "menospreciarán": "menospreciar",
    "menospreciarás": "menospreciar",
    "menospreciaré": "menospreciar",
    "menospreciaréis": "menospreciar",
    "menospreciaría": "menospreciar",
    "menospreciaríais": "menospreciar",
    "menospreciaríamos": "menospreciar",
    "menospreciarían": "menospreciar",
    "menospreciarías": "menospreciar",
    "menosprecias": "menospreciar",
    "menospreciase": "menospreciar",
    "menospreciaseis": "menospreciar",
    "menospreciasen": "menospreciar",
    "menospreciases": "menospreciar",
    "menospreciaste": "menospreciar",
    "menospreciasteis": "menospreciar",
    "menospreciativa": "menospreciativo",
    "menospreciativas": "menospreciativo",
    "menospreciativos": "menospreciativo",
    "menosprecie": "menospreciar",
    "menospreciemos": "menospreciar",
    "menosprecien": "menospreciar",
    "menosprecies": "menospreciar",
    "menosprecio": "menospreciar",
    "menosprecios": "menosprecio",
    "menospreciábamos": "menospreciar",
    "menospreciáis": "menospreciar",
    "menospreciáramos": "menospreciar",
    "menospreciáremos": "menospreciar",
    "menospreciásemos": "menospreciar",
    "menosprecié": "menospreciar",
    "menospreciéis": "menospreciar",
    "menospreció": "menospreciar",
    "menostasias": "menostasia",
    "mensajera": "mensajero",
    "mensajeras": "mensajero",
    "mensajeros": "mensajero",
    "mensajerías": "mensajería",
    "mensajes": "mensaje",
    "menstrua": "menstruo",
    "menstruaba": "menstruar",
    "menstruabais": "menstruar",
    "menstruaban": "menstruar",
    "menstruabas": "menstruar",
    "menstruaciones": "menstruación",
    "menstruad": "menstruar",
    "menstruada": "menstruar",
    "menstruadas": "menstruar",
    "menstruado": "menstruar",
    "menstruados": "menstruar",
    "menstruales": "menstrual",
    "menstruamos": "menstruar",
    "menstruando": "menstruar",
    "menstruantes": "menstruante",
    "menstruara": "menstruar",
    "menstruarais": "menstruar",
    "menstruaran": "menstruar",
    "menstruaras": "menstruar",
    "menstruare": "menstruar",
    "menstruareis": "menstruar",
    "menstruaremos": "menstruar",
    "menstruaren": "menstruar",
    "menstruares": "menstruar",
    "menstruaron": "menstruar",
    "menstruará": "menstruar",
    "menstruarán": "menstruar",
    "menstruarás": "menstruar",
    "menstruaré": "menstruar",
    "menstruaréis": "menstruar",
    "menstruaría": "menstruar",
    "menstruaríais": "menstruar",
    "menstruaríamos": "menstruar",
    "menstruarían": "menstruar",
    "menstruarías": "menstruar",
    "menstruas": "menstruo",
    "menstruase": "menstruar",
    "menstruaseis": "menstruar",
    "menstruasen": "menstruar",
    "menstruases": "menstruar",
    "menstruaste": "menstruar",
    "menstruasteis": "menstruar",
    "menstruemos": "menstruar",
    "menstruos": "menstruo",
    "menstruosa": "menstruoso",
    "menstruosas": "menstruoso",
    "menstruosos": "menstruoso",
    "menstruábamos": "menstruar",
    "menstruáis": "menstruar",
    "menstruáramos": "menstruar",
    "menstruáremos": "menstruar",
    "menstruásemos": "menstruar",
    "menstrué": "menstruar",
    "menstruéis": "menstruar",
    "menstruó": "menstruar",
    "menstrúa": "menstruar",
    "menstrúan": "menstruar",
    "menstrúas": "menstruar",
    "menstrúe": "menstruar",
    "menstrúen": "menstruar",
    "menstrúes": "menstruar",
    "menstrúo": "menstruar",
    "mensuales": "mensual",
    "mensualidades": "mensualidad",
    "mensuarios": "mensuario",
    "mensulares": "mensular",
    "mensura": "mensurar",
    "mensuraba": "mensurar",
    "mensurabais": "mensurar",
    "mensuraban": "mensurar",
    "mensurabas": "mensurar",
    "mensurabilidades": "mensurabilidad",
    "mensurables": "mensurable",
    "mensurad": "mensurar",
    "mensurada": "mensurar",
    "mensuradas": "mensurar",
    "mensurado": "mensurar",
    "mensuradora": "mensurador",
    "mensuradoras": "mensurador",
    "mensuradores": "mensurador",
    "mensurados": "mensurar",
    "mensuramos": "mensurar",
    "mensuran": "mensurar",
    "mensurando": "mensurar",
    "mensurara": "mensurar",
    "mensurarais": "mensurar",
    "mensuraran": "mensurar",
    "mensuraras": "mensurar",
    "mensurare": "mensurar",
    "mensurareis": "mensurar",
    "mensuraremos": "mensurar",
    "mensuraren": "mensurar",
    "mensurares": "mensurar",
    "mensuraron": "mensurar",
    "mensurará": "mensurar",
    "mensurarán": "mensurar",
    "mensurarás": "mensurar",
    "mensuraré": "mensurar",
    "mensuraréis": "mensurar",
    "mensuraría": "mensurar",
    "mensuraríais": "mensurar",
    "mensuraríamos": "mensurar",
    "mensurarían": "mensurar",
    "mensurarías": "mensurar",
    "mensuras": "mensurar",
    "mensurase": "mensurar",
    "mensuraseis": "mensurar",
    "mensurasen": "mensurar",
    "mensurases": "mensurar",
    "mensuraste": "mensurar",
    "mensurasteis": "mensurar",
    "mensure": "mensurar",
    "mensuremos": "mensurar",
    "mensuren": "mensurar",
    "mensures": "mensurar",
    "mensuro": "mensurar",
    "mensurábamos": "mensurar",
    "mensuráis": "mensurar",
    "mensuráramos": "mensurar",
    "mensuráremos": "mensurar",
    "mensurásemos": "mensurar",
    "mensuré": "mensurar",
    "mensuréis": "mensurar",
    "mensuró": "mensurar",
    "mensáfonos": "mensáfono",
    "mentaba": "mentar",
    "mentabais": "mentar",
    "mentaban": "mentar",
    "mentabas": "mentar",
    "mentad": "mentar",
    "mentada": "mentar",
    "mentadas": "mentar",
    "mentado": "mentar",
    "mentados": "mentar",
    "mentales": "mental",
    "mentalice": "mentalizar",
    "mentalicemos": "mentalizar",
    "mentalicen": "mentalizar",
    "mentalices": "mentalizar",
    "mentalicé": "mentalizar",
    "mentalicéis": "mentalizar",
    "mentalidades": "mentalidad",
    "mentaliza": "mentalizar",
    "mentalizaba": "mentalizar",
    "mentalizabais": "mentalizar",
    "mentalizaban": "mentalizar",
    "mentalizabas": "mentalizar",
    "mentalizad": "mentalizar",
    "mentalizada": "mentalizar",
    "mentalizadas": "mentalizar",
    "mentalizado": "mentalizar",
    "mentalizados": "mentalizar",
    "mentalizamos": "mentalizar",
    "mentalizan": "mentalizar",
    "mentalizando": "mentalizar",
    "mentalizara": "mentalizar",
    "mentalizarais": "mentalizar",
    "mentalizaran": "mentalizar",
    "mentalizaras": "mentalizar",
    "mentalizare": "mentalizar",
    "mentalizareis": "mentalizar",
    "mentalizaremos": "mentalizar",
    "mentalizaren": "mentalizar",
    "mentalizares": "mentalizar",
    "mentalizaron": "mentalizar",
    "mentalizará": "mentalizar",
    "mentalizarán": "mentalizar",
    "mentalizarás": "mentalizar",
    "mentalizaré": "mentalizar",
    "mentalizaréis": "mentalizar",
    "mentalizaría": "mentalizar",
    "mentalizaríais": "mentalizar",
    "mentalizaríamos": "mentalizar",
    "mentalizarían": "mentalizar",
    "mentalizarías": "mentalizar",
    "mentalizas": "mentalizar",
    "mentalizase": "mentalizar",
    "mentalizaseis": "mentalizar",
    "mentalizasen": "mentalizar",
    "mentalizases": "mentalizar",
    "mentalizaste": "mentalizar",
    "mentalizasteis": "mentalizar",
    "mentalizo": "mentalizar",
    "mentalizábamos": "mentalizar",
    "mentalizáis": "mentalizar",
    "mentalizáramos": "mentalizar",
    "mentalizáremos": "mentalizar",
    "mentalizásemos": "mentalizar",
    "mentalizó": "mentalizar",
    "mentamos": "mentar",
    "mentando": "mentar",
    "mentanodiaminas": "mentanodiamina",
    "mentara": "mentar",
    "mentarais": "mentar",
    "mentaran": "mentar",
    "mentaras": "mentar",
    "mentare": "mentar",
    "mentareis": "mentar",
    "mentaremos": "mentar",
    "mentaren": "mentar",
    "mentares": "mentar",
    "mentaron": "mentar",
    "mentará": "mentar",
    "mentarán": "mentar",
    "mentarás": "mentar",
    "mentaré": "mentar",
    "mentaréis": "mentar",
    "mentaría": "mentar",
    "mentaríais": "mentar",
    "mentaríamos": "mentar",
    "mentarían": "mentar",
    "mentarías": "mentar",
    "mentas": "menta",
    "mentase": "mentar",
    "mentaseis": "mentar",
    "mentasen": "mentar",
    "mentases": "mentar",
    "mentaste": "mentar",
    "mentasteis": "mentar",
    "mentecata": "mentecato",
    "mentecatadas": "mentecatada",
    "mentecatas": "mentecato",
    "mentecateces": "mentecatez",
    "mentecaterías": "mentecatería",
    "mentecatos": "mentecato",
    "mentemos": "mentar",
    "mentes": "mente",
    "mentid": "mentir",
    "mentida": "mentir",
    "mentidas": "mentir",
    "mentideros": "mentidero",
    "mentido": "mentir",
    "mentidos": "mentir",
    "mentimos": "mentir",
    "mentiras": "mentira",
    "mentiremos": "mentir",
    "mentirosa": "mentiroso",
    "mentirosas": "mentiroso",
    "mentirosos": "mentiroso",
    "mentirá": "mentir",
    "mentirán": "mentir",
    "mentirás": "mentir",
    "mentiré": "mentir",
    "mentiréis": "mentir",
    "mentiría": "mentir",
    "mentiríais": "mentir",
    "mentiríamos": "mentir",
    "mentirían": "mentir",
    "mentirías": "mentir",
    "mentises": "mentís",
    "mentiste": "mentir",
    "mentisteis": "mentir",
    "mentolabiales": "mentolabial",
    "mentolada": "mentolado",
    "mentoladas": "mentolado",
    "mentolados": "mentolado",
    "mentoles": "mentol",
    "mentonas": "mentona",
    "mentoneras": "mentonera",
    "mentones": "mentón",
    "mentoniana": "mentoniano",
    "mentonianas": "mentoniano",
    "mentonianos": "mentoniano",
    "mentora": "mentor",
    "mentoras": "mentor",
    "mentores": "mentor",
    "mentábamos": "mentar",
    "mentáis": "mentar",
    "mentáramos": "mentar",
    "mentáremos": "mentar",
    "mentásemos": "mentar",
    "menté": "mentar",
    "mentéis": "mentar",
    "mentí": "mentir",
    "mentía": "mentir",
    "mentíais": "mentir",
    "mentíamos": "mentir",
    "mentían": "mentir",
    "mentías": "mentir",
    "mentís": "mentir",
    "mentó": "mentar",
    "menuda": "menudo",
    "menudas": "menudo",
    "menudea": "menudear",
    "menudeaba": "menudear",
    "menudeabais": "menudear",
    "menudeaban": "menudear",
    "menudeabas": "menudear",
    "menudead": "menudear",
    "menudeada": "menudear",
    "menudeadas": "menudear",
    "menudeado": "menudear",
    "menudeados": "menudear",
    "menudeamos": "menudear",
    "menudean": "menudear",
    "menudeando": "menudear",
    "menudeara": "menudear",
    "menudearais": "menudear",
    "menudearan": "menudear",
    "menudearas": "menudear",
    "menudeare": "menudear",
    "menudeareis": "menudear",
    "menudearemos": "menudear",
    "menudearen": "menudear",
    "menudeares": "menudear",
    "menudearon": "menudear",
    "menudeará": "menudear",
    "menudearán": "menudear",
    "menudearás": "menudear",
    "menudearé": "menudear",
    "menudearéis": "menudear",
    "menudearía": "menudear",
    "menudearíais": "menudear",
    "menudearíamos": "menudear",
    "menudearían": "menudear",
    "menudearías": "menudear",
    "menudeas": "menudear",
    "menudease": "menudear",
    "menudeaseis": "menudear",
    "menudeasen": "menudear",
    "menudeases": "menudear",
    "menudeaste": "menudear",
    "menudeasteis": "menudear",
    "menudee": "menudear",
    "menudeemos": "menudear",
    "menudeen": "menudear",
    "menudees": "menudear",
    "menudencias": "menudencia",
    "menudeo": "menudear",
    "menudeos": "menudeo",
    "menudeábamos": "menudear",
    "menudeáis": "menudear",
    "menudeáramos": "menudear",
    "menudeáremos": "menudear",
    "menudeásemos": "menudear",
    "menudeé": "menudear",
    "menudeéis": "menudear",
    "menudeó": "menudear",
    "menudillos": "menudillo",
    "menudos": "menudo",
    "meníngea": "meníngeo",
    "meníngeas": "meníngeo",
    "meníngeos": "meníngeo",
    "menús": "menú",
    "meo": "mear",
    "meollares": "meollar",
    "meollos": "meollo",
    "meolluda": "meolludo",
    "meolludas": "meolludo",
    "meolludos": "meolludo",
    "meona": "meón",
    "meonas": "meón",
    "meones": "meón",
    "meos": "meo",
    "mequetrefes": "mequetrefe",
    "mera": "mero",
    "meraba": "merar",
    "merabais": "merar",
    "meraban": "merar",
    "merabas": "merar",
    "merad": "merar",
    "merada": "merar",
    "meradas": "merar",
    "merado": "merar",
    "merados": "merar",
    "meramos": "merar",
    "meran": "merar",
    "merando": "merar",
    "merantis": "meranti",
    "merara": "merar",
    "merarais": "merar",
    "meraran": "merar",
    "meraras": "merar",
    "merare": "merar",
    "merareis": "merar",
    "meraremos": "merar",
    "meraren": "merar",
    "merares": "merar",
    "meraron": "merar",
    "merará": "merar",
    "merarán": "merar",
    "merarás": "merar",
    "meraré": "merar",
    "meraréis": "merar",
    "meraría": "merar",
    "meraríais": "merar",
    "meraríamos": "merar",
    "merarían": "merar",
    "merarías": "merar",
    "meras": "mero",
    "merase": "merar",
    "meraseis": "merar",
    "merasen": "merar",
    "merases": "merar",
    "meraste": "merar",
    "merasteis": "merar",
    "merca": "mercar",
    "mercaba": "mercar",
    "mercabais": "mercar",
    "mercaban": "mercar",
    "mercabas": "mercar",
    "mercachifles": "mercachifle",
    "mercad": "mercar",
    "mercada": "mercar",
    "mercadas": "mercar",
    "mercadea": "mercadear",
    "mercadeaba": "mercadear",
    "mercadeabais": "mercadear",
    "mercadeaban": "mercadear",
    "mercadeabas": "mercadear",
    "mercadead": "mercadear",
    "mercadeada": "mercadear",
    "mercadeadas": "mercadear",
    "mercadeado": "mercadear",
    "mercadeados": "mercadear",
    "mercadeamos": "mercadear",
    "mercadean": "mercadear",
    "mercadeando": "mercadear",
    "mercadeara": "mercadear",
    "mercadearais": "mercadear",
    "mercadearan": "mercadear",
    "mercadearas": "mercadear",
    "mercadeare": "mercadear",
    "mercadeareis": "mercadear",
    "mercadearemos": "mercadear",
    "mercadearen": "mercadear",
    "mercadeares": "mercadear",
    "mercadearon": "mercadear",
    "mercadeará": "mercadear",
    "mercadearán": "mercadear",
    "mercadearás": "mercadear",
    "mercadearé": "mercadear",
    "mercadearéis": "mercadear",
    "mercadearía": "mercadear",
    "mercadearíais": "mercadear",
    "mercadearíamos": "mercadear",
    "mercadearían": "mercadear",
    "mercadearías": "mercadear",
    "mercadeas": "mercadear",
    "mercadease": "mercadear",
    "mercadeaseis": "mercadear",
    "mercadeasen": "mercadear",
    "mercadeases": "mercadear",
    "mercadeaste": "mercadear",
    "mercadeasteis": "mercadear",
    "mercadee": "mercadear",
    "mercadeemos": "mercadear",
    "mercadeen": "mercadear",
    "mercadees": "mercadear",
    "mercadeo": "mercadear",
    "mercadeos": "mercadeo",
    "mercaderes": "mercader",
    "mercaderías": "mercadería",
    "mercadeábamos": "mercadear",
    "mercadeáis": "mercadear",
    "mercadeáramos": "mercadear",
    "mercadeáremos": "mercadear",
    "mercadeásemos": "mercadear",
    "mercadeé": "mercadear",
    "mercadeéis": "mercadear",
    "mercadeó": "mercadear",
    "mercadillos": "mercadillo",
    "mercado": "mercar",
    "mercadologías": "mercadología",
    "mercados": "mercar",
    "mercadotecnias": "mercadotecnia",
    "mercadotécnica": "mercadotécnico",
    "mercadotécnicas": "mercadotécnico",
    "mercadotécnicos": "mercadotécnico",
    "mercallitas": "mercallita",
    "mercamos": "mercar",
    "mercan": "mercar",
    "mercancías": "mercancía",
    "mercando": "mercar",
    "mercantes": "mercante",
    "mercantiles": "mercantil",
    "mercantilice": "mercantilizar",
    "mercantilicemos": "mercantilizar",
    "mercantilicen": "mercantilizar",
    "mercantilices": "mercantilizar",
    "mercantilicé": "mercantilizar",
    "mercantilicéis": "mercantilizar",
    "mercantilismos": "mercantilismo",
    "mercantilistas": "mercantilista",
    "mercantiliza": "mercantilizar",
    "mercantilizaba": "mercantilizar",
    "mercantilizabais": "mercantilizar",
    "mercantilizaban": "mercantilizar",
    "mercantilizabas": "mercantilizar",
    "mercantilizaciones": "mercantilización",
    "mercantilizad": "mercantilizar",
    "mercantilizada": "mercantilizar",
    "mercantilizadas": "mercantilizar",
    "mercantilizado": "mercantilizar",
    "mercantilizados": "mercantilizar",
    "mercantilizamos": "mercantilizar",
    "mercantilizan": "mercantilizar",
    "mercantilizando": "mercantilizar",
    "mercantilizara": "mercantilizar",
    "mercantilizarais": "mercantilizar",
    "mercantilizaran": "mercantilizar",
    "mercantilizaras": "mercantilizar",
    "mercantilizare": "mercantilizar",
    "mercantilizareis": "mercantilizar",
    "mercantilizaremos": "mercantilizar",
    "mercantilizaren": "mercantilizar",
    "mercantilizares": "mercantilizar",
    "mercantilizaron": "mercantilizar",
    "mercantilizará": "mercantilizar",
    "mercantilizarán": "mercantilizar",
    "mercantilizarás": "mercantilizar",
    "mercantilizaré": "mercantilizar",
    "mercantilizaréis": "mercantilizar",
    "mercantilizaría": "mercantilizar",
    "mercantilizaríais": "mercantilizar",
    "mercantilizaríamos": "mercantilizar",
    "mercantilizarían": "mercantilizar",
    "mercantilizarías": "mercantilizar",
    "mercantilizas": "mercantilizar",
    "mercantilizase": "mercantilizar",
    "mercantilizaseis": "mercantilizar",
    "mercantilizasen": "mercantilizar",
    "mercantilizases": "mercantilizar",
    "mercantilizaste": "mercantilizar",
    "mercantilizasteis": "mercantilizar",
    "mercantilizo": "mercantilizar",
    "mercantilizábamos": "mercantilizar",
    "mercantilizáis": "mercantilizar",
    "mercantilizáramos": "mercantilizar",
    "mercantilizáremos": "mercantilizar",
    "mercantilizásemos": "mercantilizar",
    "mercantilizó": "mercantilizar",
    "mercantiva": "mercantivo",
    "mercantivas": "mercantivo",
    "mercantivos": "mercantivo",
    "mercaptanos": "mercaptano",
    "mercaptobenzotiazoles": "mercaptobenzotiazol",
    "mercaptoetanoles": "mercaptoetanol",
    "mercaptotiazolinas": "mercaptotiazolina",
    "mercara": "mercar",
    "mercarais": "mercar",
    "mercaran": "mercar",
    "mercaras": "mercar",
    "mercare": "mercar",
    "mercareis": "mercar",
    "mercaremos": "mercar",
    "mercaren": "mercar",
    "mercares": "mercar",
    "mercaron": "mercar",
    "mercará": "mercar",
    "mercarán": "mercar",
    "mercarás": "mercar",
    "mercaré": "mercar",
    "mercaréis": "mercar",
    "mercaría": "mercar",
    "mercaríais": "mercar",
    "mercaríamos": "mercar",
    "mercarían": "mercar",
    "mercarías": "mercar",
    "mercas": "mercar",
    "mercase": "mercar",
    "mercaseis": "mercar",
    "mercasen": "mercar",
    "mercases": "mercar",
    "mercaste": "mercar",
    "mercasteis": "mercar",
    "mercedaria": "mercedario",
    "mercedarias": "mercedario",
    "mercedarios": "mercedario",
    "mercedes": "merced",
    "mercenaria": "mercenario",
    "mercenarias": "mercenario",
    "mercenarios": "mercenario",
    "mercera": "mercero",
    "merceras": "mercero",
    "mercerice": "mercerizar",
    "mercericemos": "mercerizar",
    "mercericen": "mercerizar",
    "mercerices": "mercerizar",
    "mercericé": "mercerizar",
    "mercericéis": "mercerizar",
    "merceriza": "mercerizar",
    "mercerizaba": "mercerizar",
    "mercerizabais": "mercerizar",
    "mercerizaban": "mercerizar",
    "mercerizabas": "mercerizar",
    "mercerizaciones": "mercerización",
    "mercerizad": "mercerizar",
    "mercerizada": "mercerizar",
    "mercerizadas": "mercerizar",
    "mercerizado": "mercerizar",
    "mercerizados": "mercerizar",
    "mercerizamos": "mercerizar",
    "mercerizan": "mercerizar",
    "mercerizando": "mercerizar",
    "mercerizara": "mercerizar",
    "mercerizarais": "mercerizar",
    "mercerizaran": "mercerizar",
    "mercerizaras": "mercerizar",
    "mercerizare": "mercerizar",
    "mercerizareis": "mercerizar",
    "mercerizaremos": "mercerizar",
    "mercerizaren": "mercerizar",
    "mercerizares": "mercerizar",
    "mercerizaron": "mercerizar",
    "mercerizará": "mercerizar",
    "mercerizarán": "mercerizar",
    "mercerizarás": "mercerizar",
    "mercerizaré": "mercerizar",
    "mercerizaréis": "mercerizar",
    "mercerizaría": "mercerizar",
    "mercerizaríais": "mercerizar",
    "mercerizaríamos": "mercerizar",
    "mercerizarían": "mercerizar",
    "mercerizarías": "mercerizar",
    "mercerizas": "mercerizar",
    "mercerizase": "mercerizar",
    "mercerizaseis": "mercerizar",
    "mercerizasen": "mercerizar",
    "mercerizases": "mercerizar",
    "mercerizaste": "mercerizar",
    "mercerizasteis": "mercerizar",
    "mercerizo": "mercerizar",
    "mercerizábamos": "mercerizar",
    "mercerizáis": "mercerizar",
    "mercerizáramos": "mercerizar",
    "mercerizáremos": "mercerizar",
    "mercerizásemos": "mercerizar",
    "mercerizó": "mercerizar",
    "merceros": "mercero",
    "mercerías": "mercería",
    "merchandises": "merchandise",
    "merchandisings": "merchandising",
    "merchantes": "merchante",
    "merco": "mercar",
    "mercuriales": "mercurial",
    "mercurialismos": "mercurialismo",
    "mercurios": "mercurio",
    "mercuriosa": "mercurioso",
    "mercuriosas": "mercurioso",
    "mercuriosos": "mercurioso",
    "mercurocromos": "mercurocromo",
    "mercábamos": "mercar",
    "mercáis": "mercar",
    "mercáramos": "mercar",
    "mercáremos": "mercar",
    "mercásemos": "mercar",
    "mercó": "mercar",
    "mercúrica": "mercúrico",
    "mercúricas": "mercúrico",
    "mercúricos": "mercúrico",
    "merdellona": "merdellón",
    "merdellonas": "merdellón",
    "merdellones": "merdellón",
    "merdosa": "merdoso",
    "merdosas": "merdoso",
    "merdosos": "merdoso",
    "mere": "merar",
    "merece": "merecer",
    "mereced": "merecer",
    "merecedora": "merecedor",
    "merecedoras": "merecedor",
    "merecedores": "merecedor",
    "merecemos": "merecer",
    "merecen": "merecer",
    "mereceremos": "merecer",
    "merecerá": "merecer",
    "merecerán": "merecer",
    "merecerás": "merecer",
    "mereceré": "merecer",
    "mereceréis": "merecer",
    "merecería": "merecer",
    "mereceríais": "merecer",
    "mereceríamos": "merecer",
    "merecerían": "merecer",
    "merecerías": "merecer",
    "mereces": "merecer",
    "merecida": "merecer",
    "merecidas": "merecer",
    "merecido": "merecer",
    "merecidos": "merecido",
    "mereciendo": "merecer",
    "mereciera": "merecer",
    "merecierais": "merecer",
    "merecieran": "merecer",
    "merecieras": "merecer",
    "mereciere": "merecer",
    "mereciereis": "merecer",
    "merecieren": "merecer",
    "merecieres": "merecer",
    "merecieron": "merecer",
    "mereciese": "merecer",
    "merecieseis": "merecer",
    "mereciesen": "merecer",
    "merecieses": "merecer",
    "merecimientos": "merecimiento",
    "merecimos": "merecer",
    "mereciste": "merecer",
    "merecisteis": "merecer",
    "mereciéramos": "merecer",
    "mereciéremos": "merecer",
    "mereciésemos": "merecer",
    "mereció": "merecer",
    "merecéis": "merecer",
    "merecí": "merecer",
    "merecía": "merecer",
    "merecíais": "merecer",
    "merecíamos": "merecer",
    "merecían": "merecer",
    "merecías": "merecer",
    "meremos": "merar",
    "meren": "merar",
    "merendaba": "merendar",
    "merendabais": "merendar",
    "merendaban": "merendar",
    "merendabas": "merendar",
    "merendad": "merendar",
    "merendada": "merendar",
    "merendadas": "merendar",
    "merendado": "merendar",
    "merendados": "merendar",
    "merendamos": "merendar",
    "merendando": "merendar",
    "merendara": "merendar",
    "merendarais": "merendar",
    "merendaran": "merendar",
    "merendaras": "merendar",
    "merendare": "merendar",
    "merendareis": "merendar",
    "merendaremos": "merendar",
    "merendaren": "merendar",
    "merendares": "merendar",
    "merendaron": "merendar",
    "merendará": "merendar",
    "merendarán": "merendar",
    "merendarás": "merendar",
    "merendaré": "merendar",
    "merendaréis": "merendar",
    "merendaría": "merendar",
    "merendaríais": "merendar",
    "merendaríamos": "merendar",
    "merendarían": "merendar",
    "merendarías": "merendar",
    "merendase": "merendar",
    "merendaseis": "merendar",
    "merendasen": "merendar",
    "merendases": "merendar",
    "merendaste": "merendar",
    "merendasteis": "merendar",
    "merendemos": "merendar",
    "merendera": "merendero",
    "merenderas": "merendero",
    "merenderos": "merendero",
    "merendolas": "merendola",
    "merendonas": "merendona",
    "merendábamos": "merendar",
    "merendáis": "merendar",
    "merendáramos": "merendar",
    "merendáremos": "merendar",
    "merendásemos": "merendar",
    "merendé": "merendar",
    "merendéis": "merendar",
    "merendó": "merendar",
    "merengues": "merengue",
    "meres": "merar",
    "meretrices": "meretriz",
    "meretricia": "meretricio",
    "meretricias": "meretricio",
    "meretricios": "meretricio",
    "merezca": "merecer",
    "merezcamos": "merecer",
    "merezcan": "merecer",
    "merezcas": "merecer",
    "merezco": "merecer",
    "merezcáis": "merecer",
    "mergones": "mergón",
    "mergos": "mergo",
    "mericarpos": "mericarpo",
    "merideña": "merideño",
    "merideñas": "merideño",
    "merideños": "merideño",
    "meridiana": "meridiano",
    "meridianas": "meridiano",
    "meridianos": "meridiano",
    "meridionales": "meridional",
    "merienda": "merendar",
    "meriendan": "merendar",
    "meriendas": "merienda",
    "meriende": "merendar",
    "merienden": "merendar",
    "meriendes": "merendar",
    "meriendo": "merendar",
    "merillos": "merillo",
    "merina": "merino",
    "merinas": "merino",
    "merindades": "merindad",
    "merinos": "merino",
    "meristemas": "meristema",
    "meristemos": "meristemo",
    "meristemática": "meristemático",
    "meristemáticas": "meristemático",
    "meristemáticos": "meristemático",
    "meritoria": "meritorio",
    "meritorias": "meritorio",
    "meritorios": "meritorio",
    "meriñaques": "meriñaque",
    "merlanes": "merlán",
    "merletas": "merleta",
    "merlines": "merlín",
    "merlones": "merlón",
    "merlos": "merlo",
    "merluzas": "merluza",
    "merma": "mermar",
    "mermaba": "mermar",
    "mermabais": "mermar",
    "mermaban": "mermar",
    "mermabas": "mermar",
    "mermad": "mermar",
    "mermada": "mermar",
    "mermadas": "mermar",
    "mermado": "mermar",
    "mermadora": "mermador",
    "mermadoras": "mermador",
    "mermadores": "mermador",
    "mermados": "mermar",
    "mermamos": "mermar",
    "merman": "mermar",
    "mermando": "mermar",
    "mermara": "mermar",
    "mermarais": "mermar",
    "mermaran": "mermar",
    "mermaras": "mermar",
    "mermare": "mermar",
    "mermareis": "mermar",
    "mermaremos": "mermar",
    "mermaren": "mermar",
    "mermares": "mermar",
    "mermaron": "mermar",
    "mermará": "mermar",
    "mermarán": "mermar",
    "mermarás": "mermar",
    "mermaré": "mermar",
    "mermaréis": "mermar",
    "mermaría": "mermar",
    "mermaríais": "mermar",
    "mermaríamos": "mermar",
    "mermarían": "mermar",
    "mermarías": "mermar",
    "mermas": "mermar",
    "mermase": "mermar",
    "mermaseis": "mermar",
    "mermasen": "mermar",
    "mermases": "mermar",
    "mermaste": "mermar",
    "mermasteis": "mermar",
    "merme": "mermar",
    "mermeladas": "mermelada",
    "mermemos": "mermar",
    "mermen": "mermar",
    "mermes": "mermar",
    "mermo": "mermar",
    "mermábamos": "mermar",
    "mermáis": "mermar",
    "mermáramos": "mermar",
    "mermáremos": "mermar",
    "mermásemos": "mermar",
    "mermé": "mermar",
    "merméis": "mermar",
    "mermó": "mermar",
    "mero": "merar",
    "meroblástica": "meroblástico",
    "meroblásticas": "meroblástico",
    "meroblásticos": "meroblástico",
    "merodea": "merodear",
    "merodeaba": "merodear",
    "merodeabais": "merodear",
    "merodeaban": "merodear",
    "merodeabas": "merodear",
    "merodead": "merodear",
    "merodeada": "merodear",
    "merodeadas": "merodear",
    "merodeado": "merodear",
    "merodeadora": "merodeador",
    "merodeadoras": "merodeador",
    "merodeadores": "merodeador",
    "merodeados": "merodear",
    "merodeamos": "merodear",
    "merodean": "merodear",
    "merodeando": "merodear",
    "merodeara": "merodear",
    "merodearais": "merodear",
    "merodearan": "merodear",
    "merodearas": "merodear",
    "merodeare": "merodear",
    "merodeareis": "merodear",
    "merodearemos": "merodear",
    "merodearen": "merodear",
    "merodeares": "merodear",
    "merodearon": "merodear",
    "merodeará": "merodear",
    "merodearán": "merodear",
    "merodearás": "merodear",
    "merodearé": "merodear",
    "merodearéis": "merodear",
    "merodearía": "merodear",
    "merodearíais": "merodear",
    "merodearíamos": "merodear",
    "merodearían": "merodear",
    "merodearías": "merodear",
    "merodeas": "merodear",
    "merodease": "merodear",
    "merodeaseis": "merodear",
    "merodeasen": "merodear",
    "merodeases": "merodear",
    "merodeaste": "merodear",
    "merodeasteis": "merodear",
    "merodee": "merodear",
    "merodeemos": "merodear",
    "merodeen": "merodear",
    "merodees": "merodear",
    "merodeo": "merodear",
    "merodeos": "merodeo",
    "merodeábamos": "merodear",
    "merodeáis": "merodear",
    "merodeáramos": "merodear",
    "merodeáremos": "merodear",
    "merodeásemos": "merodear",
    "merodeé": "merodear",
    "merodeéis": "merodear",
    "merodeó": "merodear",
    "merodistas": "merodista",
    "meromorfa": "meromorfo",
    "meromorfas": "meromorfo",
    "meromorfos": "meromorfo",
    "meros": "mero",
    "merotomías": "merotomía",
    "merovingia": "merovingio",
    "merovingias": "merovingio",
    "merovingios": "merovingio",
    "meroédrica": "meroédrico",
    "meroédricas": "meroédrico",
    "meroédricos": "meroédrico",
    "merque": "mercar",
    "merquemos": "mercar",
    "merquen": "mercar",
    "merques": "mercar",
    "merqué": "mercar",
    "merquéis": "mercar",
    "merábamos": "merar",
    "meráis": "merar",
    "meráramos": "merar",
    "meráremos": "merar",
    "merásemos": "merar",
    "meré": "merar",
    "meréis": "merar",
    "merénquimas": "merénquima",
    "meró": "merar",
    "mesa": "mesar",
    "mesaba": "mesar",
    "mesabais": "mesar",
    "mesaban": "mesar",
    "mesabas": "mesar",
    "mesad": "mesar",
    "mesada": "mesar",
    "mesadas": "mesar",
    "mesado": "mesar",
    "mesados": "mesar",
    "mesaduras": "mesadura",
    "mesamos": "mesar",
    "mesan": "mesar",
    "mesanas": "mesana",
    "mesando": "mesar",
    "mesara": "mesar",
    "mesarais": "mesar",
    "mesaran": "mesar",
    "mesaras": "mesar",
    "mesare": "mesar",
    "mesareis": "mesar",
    "mesaremos": "mesar",
    "mesaren": "mesar",
    "mesares": "mesar",
    "mesaron": "mesar",
    "mesará": "mesar",
    "mesarán": "mesar",
    "mesarás": "mesar",
    "mesaré": "mesar",
    "mesaréis": "mesar",
    "mesaría": "mesar",
    "mesaríais": "mesar",
    "mesaríamos": "mesar",
    "mesarían": "mesar",
    "mesarías": "mesar",
    "mesas": "mesar",
    "mesase": "mesar",
    "mesaseis": "mesar",
    "mesasen": "mesar",
    "mesases": "mesar",
    "mesaste": "mesar",
    "mesasteis": "mesar",
    "mescalera": "mescalero",
    "mescaleras": "mescalero",
    "mescaleros": "mescalero",
    "mescalinas": "mescalina",
    "mescolanzas": "mescolanza",
    "mese": "mesar",
    "meseguera": "meseguero",
    "mesegueras": "meseguero",
    "mesegueros": "meseguero",
    "meseguerías": "meseguería",
    "mesemos": "mesar",
    "mesen": "mesar",
    "mesencéfalos": "mesencéfalo",
    "mesenquimales": "mesenquimal",
    "mesenquimatosa": "mesenquimatoso",
    "mesenquimatosas": "mesenquimatoso",
    "mesenquimatosos": "mesenquimatoso",
    "mesenquimática": "mesenquimático",
    "mesenquimáticas": "mesenquimático",
    "mesenquimáticos": "mesenquimático",
    "mesenterios": "mesenterio",
    "mesentérica": "mesentérico",
    "mesentéricas": "mesentérico",
    "mesentéricos": "mesentérico",
    "mesera": "mesero",
    "meseraica": "meseraico",
    "meseraicas": "meseraico",
    "meseraicos": "meseraico",
    "meseras": "mesero",
    "meseros": "mesero",
    "meses": "mesar",
    "mesetas": "meseta",
    "mesiados": "mesiado",
    "mesiales": "mesial",
    "mesializaciones": "mesialización",
    "mesianismos": "mesianismo",
    "mesiazgos": "mesiazgo",
    "mesidores": "mesidor",
    "mesillas": "mesilla",
    "mesillos": "mesillo",
    "mesinesa": "mesinés",
    "mesinesas": "mesinés",
    "mesineses": "mesinés",
    "mesiobucales": "mesiobucal",
    "mesioclusiones": "mesioclusión",
    "mesioclusodentales": "mesioclusodental",
    "mesiodistales": "mesiodistal",
    "mesiogingivales": "mesiogingival",
    "mesiolabiales": "mesiolabial",
    "mesiolinguales": "mesiolingual",
    "mesiooclusodistales": "mesiooclusodistal",
    "mesitas": "mesita",
    "mesitilenos": "mesitileno",
    "mesiánica": "mesiánico",
    "mesiánicas": "mesiánico",
    "mesiánicos": "mesiánico",
    "mesma": "mesmo",
    "mesmas": "mesmo",
    "mesmerismos": "mesmerismo",
    "mesmos": "mesmo",
    "mesnadas": "mesnada",
    "mesnaderos": "mesnadero",
    "mesnaderías": "mesnadería",
    "meso": "mesar",
    "mesoamericana": "mesoamericano",
    "mesoamericanas": "mesoamericano",
    "mesoamericanos": "mesoamericano",
    "mesoapéndices": "mesoapéndice",
    "mesocarpios": "mesocarpio",
    "mesocarpos": "mesocarpo",
    "mesocefalias": "mesocefalia",
    "mesoclimas": "mesoclima",
    "mesocracias": "mesocracia",
    "mesocrática": "mesocrático",
    "mesocráticas": "mesocrático",
    "mesocráticos": "mesocrático",
    "mesocéfala": "mesocéfalo",
    "mesocéfalas": "mesocéfalo",
    "mesocéfalos": "mesocéfalo",
    "mesocólica": "mesocólico",
    "mesocólicas": "mesocólico",
    "mesocólicos": "mesocólico",
    "mesocólones": "mesocolon",
    "mesocúrtica": "mesocúrtico",
    "mesocúrticas": "mesocúrtico",
    "mesocúrticos": "mesocúrtico",
    "mesodermos": "mesodermo",
    "mesodonta": "mesodonto",
    "mesodontas": "mesodonto",
    "mesodontismos": "mesodontismo",
    "mesodontos": "mesodonto",
    "mesodérmica": "mesodérmico",
    "mesodérmicas": "mesodérmico",
    "mesodérmicos": "mesodérmico",
    "mesoescalas": "mesoescala",
    "mesoestafilina": "mesoestafilino",
    "mesoestafilinas": "mesoestafilino",
    "mesoestafilinos": "mesoestafilino",
    "mesoesternones": "mesoesternón",
    "mesofaringes": "mesofaringe",
    "mesofilos": "mesofilo",
    "mesognata": "mesognato",
    "mesognatas": "mesognato",
    "mesognatos": "mesognato",
    "mesognática": "mesognático",
    "mesognáticas": "mesognático",
    "mesognáticos": "mesognático",
    "mesolitas": "mesolita",
    "mesolítica": "mesolítico",
    "mesolíticas": "mesolítico",
    "mesolíticos": "mesolítico",
    "mesomerías": "mesomería",
    "mesometeorologías": "mesometeorología",
    "mesonasales": "mesonasal",
    "mesonera": "mesonero",
    "mesoneras": "mesonero",
    "mesoneros": "mesonero",
    "mesones": "mesón",
    "mesoniles": "mesonil",
    "mesonistas": "mesonista",
    "mesonéfrica": "mesonéfrico",
    "mesonéfricas": "mesonéfrico",
    "mesonéfricos": "mesonéfrico",
    "mesoovarios": "mesoovario",
    "mesopausas": "mesopausa",
    "mesopotámica": "mesopotámico",
    "mesopotámicas": "mesopotámico",
    "mesopotámicos": "mesopotámico",
    "mesoprosópica": "mesoprosópico",
    "mesoprosópicas": "mesoprosópico",
    "mesoprosópicos": "mesoprosópico",
    "mesorrectos": "mesorrecto",
    "mesorrina": "mesorrino",
    "mesorrinas": "mesorrino",
    "mesorrinias": "mesorrinia",
    "mesorrinos": "mesorrino",
    "mesorrínica": "mesorrínico",
    "mesorrínicas": "mesorrínico",
    "mesorrínicos": "mesorrínico",
    "mesosalpinxes": "mesosalpinx",
    "mesosferas": "mesosfera",
    "mesosomas": "mesosoma",
    "mesotendones": "mesotendón",
    "mesotérmica": "mesotérmico",
    "mesotérmicas": "mesotérmico",
    "mesotérmicos": "mesotérmico",
    "mesouránica": "mesouránico",
    "mesouránicas": "mesouránico",
    "mesouránicos": "mesouránico",
    "mesozoica": "mesozoico",
    "mesozoicas": "mesozoico",
    "mesozoicos": "mesozoico",
    "mesta": "mesto",
    "mestas": "mesto",
    "mestice": "mestizar",
    "mesticemos": "mestizar",
    "mesticen": "mestizar",
    "mestices": "mestizar",
    "mesticé": "mestizar",
    "mesticéis": "mestizar",
    "mestiza": "mestizo",
    "mestizaba": "mestizar",
    "mestizabais": "mestizar",
    "mestizaban": "mestizar",
    "mestizabas": "mestizar",
    "mestizad": "mestizar",
    "mestizada": "mestizar",
    "mestizadas": "mestizar",
    "mestizado": "mestizar",
    "mestizados": "mestizar",
    "mestizajes": "mestizaje",
    "mestizamos": "mestizar",
    "mestizan": "mestizar",
    "mestizando": "mestizar",
    "mestizara": "mestizar",
    "mestizarais": "mestizar",
    "mestizaran": "mestizar",
    "mestizaras": "mestizar",
    "mestizare": "mestizar",
    "mestizareis": "mestizar",
    "mestizaremos": "mestizar",
    "mestizaren": "mestizar",
    "mestizares": "mestizar",
    "mestizaron": "mestizar",
    "mestizará": "mestizar",
    "mestizarán": "mestizar",
    "mestizarás": "mestizar",
    "mestizaré": "mestizar",
    "mestizaréis": "mestizar",
    "mestizaría": "mestizar",
    "mestizaríais": "mestizar",
    "mestizaríamos": "mestizar",
    "mestizarían": "mestizar",
    "mestizarías": "mestizar",
    "mestizas": "mestizo",
    "mestizase": "mestizar",
    "mestizaseis": "mestizar",
    "mestizasen": "mestizar",
    "mestizases": "mestizar",
    "mestizaste": "mestizar",
    "mestizasteis": "mestizar",
    "mestizo": "mestizar",
    "mestizos": "mestizo",
    "mestizábamos": "mestizar",
    "mestizáis": "mestizar",
    "mestizáramos": "mestizar",
    "mestizáremos": "mestizar",
    "mestizásemos": "mestizar",
    "mestizó": "mestizar",
    "mestos": "mesto",
    "mesura": "mesurar",
    "mesuraba": "mesurar",
    "mesurabais": "mesurar",
    "mesuraban": "mesurar",
    "mesurabas": "mesurar",
    "mesurables": "mesurable",
    "mesurad": "mesurar",
    "mesurada": "mesurar",
    "mesuradas": "mesurar",
    "mesurado": "mesurar",
    "mesurados": "mesurar",
    "mesuramos": "mesurar",
    "mesuran": "mesurar",
    "mesurando": "mesurar",
    "mesurara": "mesurar",
    "mesurarais": "mesurar",
    "mesuraran": "mesurar",
    "mesuraras": "mesurar",
    "mesurare": "mesurar",
    "mesurareis": "mesurar",
    "mesuraremos": "mesurar",
    "mesuraren": "mesurar",
    "mesurares": "mesurar",
    "mesuraron": "mesurar",
    "mesurará": "mesurar",
    "mesurarán": "mesurar",
    "mesurarás": "mesurar",
    "mesuraré": "mesurar",
    "mesuraréis": "mesurar",
    "mesuraría": "mesurar",
    "mesuraríais": "mesurar",
    "mesuraríamos": "mesurar",
    "mesurarían": "mesurar",
    "mesurarías": "mesurar",
    "mesuras": "mesurar",
    "mesurase": "mesurar",
    "mesuraseis": "mesurar",
    "mesurasen": "mesurar",
    "mesurases": "mesurar",
    "mesuraste": "mesurar",
    "mesurasteis": "mesurar",
    "mesure": "mesurar",
    "mesuremos": "mesurar",
    "mesuren": "mesurar",
    "mesures": "mesurar",
    "mesuro": "mesurar",
    "mesurábamos": "mesurar",
    "mesuráis": "mesurar",
    "mesuráramos": "mesurar",
    "mesuráremos": "mesurar",
    "mesurásemos": "mesurar",
    "mesuré": "mesurar",
    "mesuréis": "mesurar",
    "mesuró": "mesurar",
    "mesábamos": "mesar",
    "mesáis": "mesar",
    "mesáramos": "mesar",
    "mesáremos": "mesar",
    "mesásemos": "mesar",
    "mesé": "mesar",
    "meséis": "mesar",
    "mesénquimas": "mesénquima",
    "mesó": "mesar",
    "mesócrona": "mesócrono",
    "mesócronas": "mesócrono",
    "mesócronos": "mesócrono",
    "mesófila": "mesófilo",
    "mesófilas": "mesófilo",
    "mesófilos": "mesófilo",
    "mesófitos": "mesófito",
    "meta": "meter",
    "metabisulfitos": "metabisulfito",
    "metabolismos": "metabolismo",
    "metabolitas": "metabolita",
    "metabolitos": "metabolito",
    "metaboratos": "metaborato",
    "metabólica": "metabólico",
    "metabólicas": "metabólico",
    "metabólicos": "metabólico",
    "metacarpiana": "metacarpiano",
    "metacarpianas": "metacarpiano",
    "metacarpianos": "metacarpiano",
    "metacarpofalángica": "metacarpofalángico",
    "metacarpofalángicas": "metacarpofalángico",
    "metacarpofalángicos": "metacarpofalángico",
    "metacarpos": "metacarpo",
    "metacartografías": "metacartografía",
    "metacentros": "metacentro",
    "metaciencias": "metaciencia",
    "metacresoles": "metacresol",
    "metacrilatocrómica": "metacrilatocrómico",
    "metacrilatocrómicas": "metacrilatocrómico",
    "metacrilatocrómicos": "metacrilatocrómico",
    "metacrilatos": "metacrilato",
    "metacroleínas": "metacroleína",
    "metacromasias": "metacromasia",
    "metacromática": "metacromático",
    "metacromáticas": "metacromático",
    "metacromáticos": "metacromático",
    "metacronales": "metacronal",
    "metacrílica": "metacrílico",
    "metacrílicas": "metacrílico",
    "metacrílicos": "metacrílico",
    "metacrónica": "metacrónico",
    "metacrónicas": "metacrónico",
    "metacrónicos": "metacrónico",
    "metacéntrica": "metacéntrico",
    "metacéntricas": "metacéntrico",
    "metacéntricos": "metacéntrico",
    "metadinamos": "metadinamo",
    "metadonas": "metadona",
    "metaestables": "metaestable",
    "metafases": "metafase",
    "metafilosofías": "metafilosofía",
    "metafonías": "metafonía",
    "metaforice": "metaforizar",
    "metaforicemos": "metaforizar",
    "metaforicen": "metaforizar",
    "metaforices": "metaforizar",
    "metaforicé": "metaforizar",
    "metaforicéis": "metaforizar",
    "metaforiza": "metaforizar",
    "metaforizaba": "metaforizar",
    "metaforizabais": "metaforizar",
    "metaforizaban": "metaforizar",
    "metaforizabas": "metaforizar",
    "metaforizad": "metaforizar",
    "metaforizada": "metaforizar",
    "metaforizadas": "metaforizar",
    "metaforizado": "metaforizar",
    "metaforizados": "metaforizar",
    "metaforizamos": "metaforizar",
    "metaforizan": "metaforizar",
    "metaforizando": "metaforizar",
    "metaforizara": "metaforizar",
    "metaforizarais": "metaforizar",
    "metaforizaran": "metaforizar",
    "metaforizaras": "metaforizar",
    "metaforizare": "metaforizar",
    "metaforizareis": "metaforizar",
    "metaforizaremos": "metaforizar",
    "metaforizaren": "metaforizar",
    "metaforizares": "metaforizar",
    "metaforizaron": "metaforizar",
    "metaforizará": "metaforizar",
    "metaforizarán": "metaforizar",
    "metaforizarás": "metaforizar",
    "metaforizaré": "metaforizar",
    "metaforizaréis": "metaforizar",
    "metaforizaría": "metaforizar",
    "metaforizaríais": "metaforizar",
    "metaforizaríamos": "metaforizar",
    "metaforizarían": "metaforizar",
    "metaforizarías": "metaforizar",
    "metaforizas": "metaforizar",
    "metaforizase": "metaforizar",
    "metaforizaseis": "metaforizar",
    "metaforizasen": "metaforizar",
    "metaforizases": "metaforizar",
    "metaforizaste": "metaforizar",
    "metaforizasteis": "metaforizar",
    "metaforizo": "metaforizar",
    "metaforizábamos": "metaforizar",
    "metaforizáis": "metaforizar",
    "metaforizáramos": "metaforizar",
    "metaforizáremos": "metaforizar",
    "metaforizásemos": "metaforizar",
    "metaforizó": "metaforizar",
    "metafosfatos": "metafosfato",
    "metafásica": "metafásico",
    "metafásicas": "metafásico",
    "metafásicos": "metafásico",
    "metafísica": "metafísico",
    "metafísicas": "metafísico",
    "metafísicos": "metafísico",
    "metafórica": "metafórico",
    "metafóricas": "metafórico",
    "metafóricos": "metafórico",
    "metagoges": "metagoge",
    "metagásteres": "metagáster",
    "metahemoglobinas": "metahemoglobina",
    "metahemoglobinemias": "metahemoglobinemia",
    "metaherpética": "metaherpético",
    "metaherpéticas": "metaherpético",
    "metaherpéticos": "metaherpético",
    "metalaria": "metalario",
    "metalarias": "metalario",
    "metalarios": "metalario",
    "metaldehídos": "metaldehído",
    "metalenguajes": "metalenguaje",
    "metaleros": "metalero",
    "metales": "metal",
    "metalice": "metalizar",
    "metalicemos": "metalizar",
    "metalicen": "metalizar",
    "metalices": "metalizar",
    "metalicé": "metalizar",
    "metalicéis": "metalizar",
    "metalingüística": "metalingüístico",
    "metalingüísticas": "metalingüístico",
    "metalingüísticos": "metalingüístico",
    "metalismos": "metalismo",
    "metalistas": "metalista",
    "metalisterías": "metalistería",
    "metaliza": "metalizar",
    "metalizaba": "metalizar",
    "metalizabais": "metalizar",
    "metalizaban": "metalizar",
    "metalizabas": "metalizar",
    "metalizaciones": "metalización",
    "metalizad": "metalizar",
    "metalizada": "metalizar",
    "metalizadas": "metalizar",
    "metalizado": "metalizar",
    "metalizados": "metalizar",
    "metalizamos": "metalizar",
    "metalizan": "metalizar",
    "metalizando": "metalizar",
    "metalizara": "metalizar",
    "metalizarais": "metalizar",
    "metalizaran": "metalizar",
    "metalizaras": "metalizar",
    "metalizare": "metalizar",
    "metalizareis": "metalizar",
    "metalizaremos": "metalizar",
    "metalizaren": "metalizar",
    "metalizares": "metalizar",
    "metalizaron": "metalizar",
    "metalizará": "metalizar",
    "metalizarán": "metalizar",
    "metalizarás": "metalizar",
    "metalizaré": "metalizar",
    "metalizaréis": "metalizar",
    "metalizaría": "metalizar",
    "metalizaríais": "metalizar",
    "metalizaríamos": "metalizar",
    "metalizarían": "metalizar",
    "metalizarías": "metalizar",
    "metalizas": "metalizar",
    "metalizase": "metalizar",
    "metalizaseis": "metalizar",
    "metalizasen": "metalizar",
    "metalizases": "metalizar",
    "metalizaste": "metalizar",
    "metalizasteis": "metalizar",
    "metalizo": "metalizar",
    "metalizábamos": "metalizar",
    "metalizáis": "metalizar",
    "metalizáramos": "metalizar",
    "metalizáremos": "metalizar",
    "metalizásemos": "metalizar",
    "metalizó": "metalizar",
    "metallas": "metalla",
    "metalmecánicas": "metalmecánica",
    "metalocromías": "metalocromía",
    "metalografías": "metalografía",
    "metalográfica": "metalográfico",
    "metalográficas": "metalográfico",
    "metalográficos": "metalográfico",
    "metaloidea": "metaloideo",
    "metaloideas": "metaloideo",
    "metaloideos": "metaloideo",
    "metaloides": "metaloide",
    "metaloproteínas": "metaloproteína",
    "metaloterapias": "metaloterapia",
    "metalotermias": "metalotermia",
    "metalurgias": "metalurgia",
    "metalurgistas": "metalurgista",
    "metalífera": "metalífero",
    "metalíferas": "metalífero",
    "metalíferos": "metalífero",
    "metalógicas": "metalógica",
    "metalúrgica": "metalúrgico",
    "metalúrgicas": "metalúrgico",
    "metalúrgicos": "metalúrgico",
    "metamagnetismos": "metamagnetismo",
    "metamerismos": "metamerismo",
    "metamerías": "metamería",
    "metametales": "metametal",
    "metamielocitos": "metamielocito",
    "metamorfismos": "metamorfismo",
    "metamorfopsias": "metamorfopsia",
    "metamorfosea": "metamorfosear",
    "metamorfoseaba": "metamorfosear",
    "metamorfoseabais": "metamorfosear",
    "metamorfoseaban": "metamorfosear",
    "metamorfoseabas": "metamorfosear",
    "metamorfosead": "metamorfosear",
    "metamorfoseada": "metamorfosear",
    "metamorfoseadas": "metamorfosear",
    "metamorfoseado": "metamorfosear",
    "metamorfoseados": "metamorfosear",
    "metamorfoseamos": "metamorfosear",
    "metamorfosean": "metamorfosear",
    "metamorfoseando": "metamorfosear",
    "metamorfoseara": "metamorfosear",
    "metamorfosearais": "metamorfosear",
    "metamorfosearan": "metamorfosear",
    "metamorfosearas": "metamorfosear",
    "metamorfoseare": "metamorfosear",
    "metamorfoseareis": "metamorfosear",
    "metamorfosearemos": "metamorfosear",
    "metamorfosearen": "metamorfosear",
    "metamorfoseares": "metamorfosear",
    "metamorfosearon": "metamorfosear",
    "metamorfoseará": "metamorfosear",
    "metamorfosearán": "metamorfosear",
    "metamorfosearás": "metamorfosear",
    "metamorfosearé": "metamorfosear",
    "metamorfosearéis": "metamorfosear",
    "metamorfosearía": "metamorfosear",
    "metamorfosearíais": "metamorfosear",
    "metamorfosearíamos": "metamorfosear",
    "metamorfosearían": "metamorfosear",
    "metamorfosearías": "metamorfosear",
    "metamorfoseas": "metamorfosear",
    "metamorfosease": "metamorfosear",
    "metamorfoseaseis": "metamorfosear",
    "metamorfoseasen": "metamorfosear",
    "metamorfoseases": "metamorfosear",
    "metamorfoseaste": "metamorfosear",
    "metamorfoseasteis": "metamorfosear",
    "metamorfosee": "metamorfosear",
    "metamorfoseemos": "metamorfosear",
    "metamorfoseen": "metamorfosear",
    "metamorfosees": "metamorfosear",
    "metamorfoseo": "metamorfosear",
    "metamorfoseábamos": "metamorfosear",
    "metamorfoseáis": "metamorfosear",
    "metamorfoseáramos": "metamorfosear",
    "metamorfoseáremos": "metamorfosear",
    "metamorfoseásemos": "metamorfosear",
    "metamorfoseé": "metamorfosear",
    "metamorfoseéis": "metamorfosear",
    "metamorfoseó": "metamorfosear",
    "metamos": "meter",
    "metamérica": "metamérico",
    "metaméricas": "metamérico",
    "metaméricos": "metamérico",
    "metamórfica": "metamórfico",
    "metamórficas": "metamórfico",
    "metamórficos": "metamórfico",
    "metan": "meter",
    "metanales": "metanal",
    "metanamidas": "metanamida",
    "metanera": "metanero",
    "metaneras": "metanero",
    "metaneros": "metanero",
    "metanilatos": "metanilato",
    "metanizaciones": "metanización",
    "metanodiaminas": "metanodiamina",
    "metanoles": "metanol",
    "metanos": "metano",
    "metanílica": "metanílico",
    "metanílicas": "metanílico",
    "metanílicos": "metanílico",
    "metaplasias": "metaplasia",
    "metaplasmos": "metaplasmo",
    "metapsicologías": "metapsicología",
    "metapsíquica": "metapsíquico",
    "metapsíquicas": "metapsíquico",
    "metapsíquicos": "metapsíquico",
    "metarreglas": "metarregla",
    "metas": "meter",
    "metasilicatos": "metasilicato",
    "metasomatismos": "metasomatismo",
    "metastables": "metastable",
    "metastásica": "metastásico",
    "metastásicas": "metastásico",
    "metastásicos": "metastásico",
    "metatarsiana": "metatarsiano",
    "metatarsianas": "metatarsiano",
    "metatarsianos": "metatarsiano",
    "metatarsofalángica": "metatarsofalángico",
    "metatarsofalángicas": "metatarsofalángico",
    "metatarsofalángicos": "metatarsofalángico",
    "metatarsos": "metatarso",
    "metateorías": "metateoría",
    "metates": "metate",
    "metatonías": "metatonía",
    "metatálamos": "metatálamo",
    "metatécnica": "metatécnico",
    "metatécnicas": "metatécnico",
    "metatécnicos": "metatécnico",
    "metavanadatos": "metavanadato",
    "metazoarios": "metazoario",
    "metazoos": "metazoo",
    "mete": "meter",
    "meteca": "meteco",
    "metecas": "meteco",
    "metecos": "meteco",
    "meted": "meter",
    "metedora": "metedor",
    "metedoras": "metedor",
    "metedores": "metedor",
    "meteduras": "metedura",
    "metedurías": "meteduría",
    "metemos": "meter",
    "meten": "meter",
    "meteorice": "meteorizar",
    "meteoricemos": "meteorizar",
    "meteoricen": "meteorizar",
    "meteorices": "meteorizar",
    "meteoricé": "meteorizar",
    "meteoricéis": "meteorizar",
    "meteorismos": "meteorismo",
    "meteoritos": "meteorito",
    "meteoriza": "meteorizar",
    "meteorizaba": "meteorizar",
    "meteorizabais": "meteorizar",
    "meteorizaban": "meteorizar",
    "meteorizabas": "meteorizar",
    "meteorizaciones": "meteorización",
    "meteorizad": "meteorizar",
    "meteorizada": "meteorizar",
    "meteorizadas": "meteorizar",
    "meteorizado": "meteorizar",
    "meteorizados": "meteorizar",
    "meteorizamos": "meteorizar",
    "meteorizan": "meteorizar",
    "meteorizando": "meteorizar",
    "meteorizara": "meteorizar",
    "meteorizarais": "meteorizar",
    "meteorizaran": "meteorizar",
    "meteorizaras": "meteorizar",
    "meteorizare": "meteorizar",
    "meteorizareis": "meteorizar",
    "meteorizaremos": "meteorizar",
    "meteorizaren": "meteorizar",
    "meteorizares": "meteorizar",
    "meteorizaron": "meteorizar",
    "meteorizará": "meteorizar",
    "meteorizarán": "meteorizar",
    "meteorizarás": "meteorizar",
    "meteorizaré": "meteorizar",
    "meteorizaréis": "meteorizar",
    "meteorizaría": "meteorizar",
    "meteorizaríais": "meteorizar",
    "meteorizaríamos": "meteorizar",
    "meteorizarían": "meteorizar",
    "meteorizarías": "meteorizar",
    "meteorizas": "meteorizar",
    "meteorizase": "meteorizar",
    "meteorizaseis": "meteorizar",
    "meteorizasen": "meteorizar",
    "meteorizases": "meteorizar",
    "meteorizaste": "meteorizar",
    "meteorizasteis": "meteorizar",
    "meteorizo": "meteorizar",
    "meteorizábamos": "meteorizar",
    "meteorizáis": "meteorizar",
    "meteorizáramos": "meteorizar",
    "meteorizáremos": "meteorizar",
    "meteorizásemos": "meteorizar",
    "meteorizó": "meteorizar",
    "meteoroides": "meteoroide",
    "meteorologistas": "meteorologista",
    "meteorologías": "meteorología",
    "meteorológica": "meteorológico",
    "meteorológicas": "meteorológico",
    "meteorológicos": "meteorológico",
    "meteoros": "meteoro",
    "meteorítica": "meteorítico",
    "meteoríticas": "meteorítico",
    "meteoríticos": "meteorítico",
    "meteorógrafos": "meteorógrafo",
    "meteoróloga": "meteorólogo",
    "meteorólogas": "meteorólogo",
    "meteorólogos": "meteorólogo",
    "meteremos": "meter",
    "metereológica": "metereológico",
    "metereológicas": "metereológico",
    "metereológicos": "metereológico",
    "meterá": "meter",
    "meterán": "meter",
    "meterás": "meter",
    "meteré": "meter",
    "meteréis": "meter",
    "metería": "meter",
    "meteríais": "meter",
    "meteríamos": "meter",
    "meterían": "meter",
    "meterías": "meter",
    "metes": "meter",
    "meteórica": "meteórico",
    "meteóricas": "meteórico",
    "meteóricos": "meteórico",
    "meticales": "metical",
    "meticona": "meticón",
    "meticonas": "meticón",
    "meticones": "meticón",
    "meticulosa": "meticuloso",
    "meticulosas": "meticuloso",
    "meticulosidades": "meticulosidad",
    "meticulosos": "meticuloso",
    "metida": "meter",
    "metidas": "metida",
    "metido": "meter",
    "metidos": "metido",
    "metiendo": "meter",
    "metiera": "meter",
    "metierais": "meter",
    "metieran": "meter",
    "metieras": "meter",
    "metiere": "meter",
    "metiereis": "meter",
    "metieren": "meter",
    "metieres": "meter",
    "metieron": "meter",
    "metiese": "meter",
    "metieseis": "meter",
    "metiesen": "meter",
    "metieses": "meter",
    "metijona": "metijón",
    "metijonas": "metijón",
    "metijones": "metijón",
    "metilacetanilidas": "metilacetanilida",
    "metilacetilenos": "metilacetileno",
    "metilacetofenonas": "metilacetofenona",
    "metilacetonas": "metilacetona",
    "metilaciones": "metilación",
    "metilales": "metilal",
    "metilaminas": "metilamina",
    "metilaminofenoles": "metilaminofenol",
    "metilanilinas": "metilanilina",
    "metilantracenos": "metilantraceno",
    "metilantraquinonas": "metilantraquinona",
    "metilatos": "metilato",
    "metilbencilaminas": "metilbencilamina",
    "metilbencildietanolaminas": "metilbencildietanolamina",
    "metilbencildimetilaminas": "metilbencildimetilamina",
    "metilbutanoles": "metilbutanol",
    "metilbutenoles": "metilbutenol",
    "metilbutenos": "metilbuteno",
    "metilbutilcetonas": "metilbutilcetona",
    "metilbutinoles": "metilbutinol",
    "metilcelulosas": "metilcelulosa",
    "metilciclohexanoles": "metilciclohexanol",
    "metilciclohexanonas": "metilciclohexanona",
    "metilciclohexanos": "metilciclohexano",
    "metilciclohexilaminas": "metilciclohexilamina",
    "metilciclopentanos": "metilciclopentano",
    "metilclotiazidas": "metilclotiazida",
    "metilcumarinas": "metilcumarina",
    "metildietanolaminas": "metildietanolamina",
    "metildioxolanos": "metildioxolano",
    "metilenbisacrilamidas": "metilenbisacrilamida",
    "metilendianilinas": "metilendianilina",
    "metiles": "metil",
    "metilestirenos": "metilestireno",
    "metiletilpiridinas": "metiletilpiridina",
    "metilfloroglucinas": "metilfloroglucina",
    "metilformanilidas": "metilformanilida",
    "metilfosfórica": "metilfosfórico",
    "metilfosfóricas": "metilfosfórico",
    "metilfosfóricos": "metilfosfórico",
    "metilfuranos": "metilfurano",
    "metilheptanos": "metilheptano",
    "metilheptenonas": "metilheptenona",
    "metilhexanoaminas": "metilhexanoamina",
    "metilhexanos": "metilhexano",
    "metilhidracinas": "metilhidracina",
    "metilhidroxibutanonas": "metilhidroxibutanona",
    "metiliononas": "metilionona",
    "metilisoeugenoles": "metilisoeugenol",
    "metilnaftalenos": "metilnaftaleno",
    "metilnonilacetaldehídos": "metilnonilacetaldehído",
    "metilortoanisidinas": "metilortoanisidina",
    "metilos": "metilo",
    "metilparationas": "metilparationa",
    "metilpentadienos": "metilpentadieno",
    "metilpentaldehídos": "metilpentaldehído",
    "metilpentanodioles": "metilpentanodiol",
    "metilpentanos": "metilpentano",
    "metilpentenos": "metilpenteno",
    "metilpentinoles": "metilpentinol",
    "metilpiperacinas": "metilpiperacina",
    "metilpirroles": "metilpirrol",
    "metilpirrolidonas": "metilpirrolidona",
    "metilsulfatos": "metilsulfato",
    "metiltaurinas": "metiltaurina",
    "metiltetrahidrofuranos": "metiltetrahidrofurano",
    "metiltriclorosilanos": "metiltriclorosilano",
    "metilvinildiclorosilanos": "metilvinildiclorosilano",
    "metimientos": "metimiento",
    "metimos": "meter",
    "metioninas": "metionina",
    "metiste": "meter",
    "metisteis": "meter",
    "metiéramos": "meter",
    "metiéremos": "meter",
    "metiésemos": "meter",
    "metió": "meter",
    "meto": "meter",
    "metodice": "metodizar",
    "metodicemos": "metodizar",
    "metodicen": "metodizar",
    "metodices": "metodizar",
    "metodicé": "metodizar",
    "metodicéis": "metodizar",
    "metodismos": "metodismo",
    "metodistas": "metodista",
    "metodiza": "metodizar",
    "metodizaba": "metodizar",
    "metodizabais": "metodizar",
    "metodizaban": "metodizar",
    "metodizabas": "metodizar",
    "metodizad": "metodizar",
    "metodizada": "metodizar",
    "metodizadas": "metodizar",
    "metodizado": "metodizar",
    "metodizados": "metodizar",
    "metodizamos": "metodizar",
    "metodizan": "metodizar",
    "metodizando": "metodizar",
    "metodizara": "metodizar",
    "metodizarais": "metodizar",
    "metodizaran": "metodizar",
    "metodizaras": "metodizar",
    "metodizare": "metodizar",
    "metodizareis": "metodizar",
    "metodizaremos": "metodizar",
    "metodizaren": "metodizar",
    "metodizares": "metodizar",
    "metodizaron": "metodizar",
    "metodizará": "metodizar",
    "metodizarán": "metodizar",
    "metodizarás": "metodizar",
    "metodizaré": "metodizar",
    "metodizaréis": "metodizar",
    "metodizaría": "metodizar",
    "metodizaríais": "metodizar",
    "metodizaríamos": "metodizar",
    "metodizarían": "metodizar",
    "metodizarías": "metodizar",
    "metodizas": "metodizar",
    "metodizase": "metodizar",
    "metodizaseis": "metodizar",
    "metodizasen": "metodizar",
    "metodizases": "metodizar",
    "metodizaste": "metodizar",
    "metodizasteis": "metodizar",
    "metodizo": "metodizar",
    "metodizábamos": "metodizar",
    "metodizáis": "metodizar",
    "metodizáramos": "metodizar",
    "metodizáremos": "metodizar",
    "metodizásemos": "metodizar",
    "metodizó": "metodizar",
    "metodologías": "metodología",
    "metodológica": "metodológico",
    "metodológicas": "metodológico",
    "metodológicos": "metodológico",
    "metomentodos": "metomentodo",
    "metonimias": "metonimia",
    "metonímica": "metonímico",
    "metonímicas": "metonímico",
    "metonímicos": "metonímico",
    "metopas": "metopa",
    "metoposcopias": "metoposcopia",
    "metoprololes": "metoprolol",
    "metotrexatos": "metotrexato",
    "metoxiacetofenonas": "metoxiacetofenona",
    "metoxiacética": "metoxiacético",
    "metoxiacéticas": "metoxiacético",
    "metoxiacéticos": "metoxiacético",
    "metoxibutanoles": "metoxibutanol",
    "metoxicloros": "metoxicloro",
    "metoximetilpentanoles": "metoximetilpentanol",
    "metoximetilpentanonas": "metoximetilpentanona",
    "metoxipropilaminas": "metoxipropilamina",
    "metrajes": "metraje",
    "metrallas": "metralla",
    "metrallazos": "metrallazo",
    "metralletas": "metralleta",
    "metrifica": "metrificar",
    "metrificaba": "metrificar",
    "metrificabais": "metrificar",
    "metrificaban": "metrificar",
    "metrificabas": "metrificar",
    "metrificaciones": "metrificación",
    "metrificad": "metrificar",
    "metrificada": "metrificar",
    "metrificadas": "metrificar",
    "metrificado": "metrificar",
    "metrificadora": "metrificador",
    "metrificadoras": "metrificador",
    "metrificadores": "metrificador",
    "metrificados": "metrificar",
    "metrificamos": "metrificar",
    "metrifican": "metrificar",
    "metrificando": "metrificar",
    "metrificara": "metrificar",
    "metrificarais": "metrificar",
    "metrificaran": "metrificar",
    "metrificaras": "metrificar",
    "metrificare": "metrificar",
    "metrificareis": "metrificar",
    "metrificaremos": "metrificar",
    "metrificaren": "metrificar",
    "metrificares": "metrificar",
    "metrificaron": "metrificar",
    "metrificará": "metrificar",
    "metrificarán": "metrificar",
    "metrificarás": "metrificar",
    "metrificaré": "metrificar",
    "metrificaréis": "metrificar",
    "metrificaría": "metrificar",
    "metrificaríais": "metrificar",
    "metrificaríamos": "metrificar",
    "metrificarían": "metrificar",
    "metrificarías": "metrificar",
    "metrificas": "metrificar",
    "metrificase": "metrificar",
    "metrificaseis": "metrificar",
    "metrificasen": "metrificar",
    "metrificases": "metrificar",
    "metrificaste": "metrificar",
    "metrificasteis": "metrificar",
    "metrifico": "metrificar",
    "metrificábamos": "metrificar",
    "metrificáis": "metrificar",
    "metrificáramos": "metrificar",
    "metrificáremos": "metrificar",
    "metrificásemos": "metrificar",
    "metrificó": "metrificar",
    "metrifique": "metrificar",
    "metrifiquemos": "metrificar",
    "metrifiquen": "metrificar",
    "metrifiques": "metrificar",
    "metrifiqué": "metrificar",
    "metrifiquéis": "metrificar",
    "metriocránea": "metriocráneo",
    "metriocráneas": "metriocráneo",
    "metriocráneos": "metriocráneo",
    "metriométopes": "metriométope",
    "metristas": "metrista",
    "metrizables": "metrizable",
    "metrologías": "metrología",
    "metropolitana": "metropolitano",
    "metropolitanas": "metropolitano",
    "metropolitanos": "metropolitano",
    "metrorragias": "metrorragia",
    "metrorrágica": "metrorrágico",
    "metrorrágicas": "metrorrágico",
    "metrorrágicos": "metrorrágico",
    "metros": "metro",
    "metrónomos": "metrónomo",
    "metáforas": "metáfora",
    "metáis": "meter",
    "metálica": "metálico",
    "metálicas": "metálico",
    "metálicos": "metálico",
    "metámera": "metámero",
    "metámeras": "metámero",
    "metámeros": "metámero",
    "metánica": "metánico",
    "metánicas": "metánico",
    "metánicos": "metánico",
    "metéis": "meter",
    "metéoros": "metéoro",
    "metí": "meter",
    "metía": "meter",
    "metíais": "meter",
    "metíamos": "meter",
    "metían": "meter",
    "metías": "meter",
    "metílica": "metílico",
    "metílicas": "metílico",
    "metílicos": "metílico",
    "metódica": "metódico",
    "metódicas": "metódico",
    "metódicos": "metódico",
    "metópica": "metópico",
    "metópicas": "metópico",
    "metópicos": "metópico",
    "meublés": "meublé",
    "mexicana": "mexicano",
    "mexicanas": "mexicano",
    "mexicanos": "mexicano",
    "mexicas": "mexica",
    "meyótica": "meyótico",
    "meyóticas": "meyótico",
    "meyóticos": "meyótico",
    "meza": "mecer",
    "mezamos": "mecer",
    "mezan": "mecer",
    "mezas": "mecer",
    "mezcales": "mezcal",
    "mezcla": "mezclar",
    "mezclaba": "mezclar",
    "mezclabais": "mezclar",
    "mezclaban": "mezclar",
    "mezclabas": "mezclar",
    "mezclables": "mezclable",
    "mezclad": "mezclar",
    "mezclada": "mezclar",
    "mezcladas": "mezclar",
    "mezclado": "mezclar",
    "mezcladora": "mezclador",
    "mezcladoras": "mezcladora",
    "mezcladores": "mezclador",
    "mezclados": "mezclar",
    "mezcladuras": "mezcladura",
    "mezclamientos": "mezclamiento",
    "mezclamos": "mezclar",
    "mezclan": "mezclar",
    "mezclando": "mezclar",
    "mezclara": "mezclar",
    "mezclarais": "mezclar",
    "mezclaran": "mezclar",
    "mezclaras": "mezclar",
    "mezclare": "mezclar",
    "mezclareis": "mezclar",
    "mezclaremos": "mezclar",
    "mezclaren": "mezclar",
    "mezclares": "mezclar",
    "mezclaron": "mezclar",
    "mezclará": "mezclar",
    "mezclarán": "mezclar",
    "mezclarás": "mezclar",
    "mezclaré": "mezclar",
    "mezclaréis": "mezclar",
    "mezclaría": "mezclar",
    "mezclaríais": "mezclar",
    "mezclaríamos": "mezclar",
    "mezclarían": "mezclar",
    "mezclarías": "mezclar",
    "mezclas": "mezclar",
    "mezclase": "mezclar",
    "mezclaseis": "mezclar",
    "mezclasen": "mezclar",
    "mezclases": "mezclar",
    "mezclaste": "mezclar",
    "mezclasteis": "mezclar",
    "mezcle": "mezclar",
    "mezclemos": "mezclar",
    "mezclen": "mezclar",
    "mezcles": "mezclar",
    "mezclillas": "mezclilla",
    "mezclo": "mezclar",
    "mezclábamos": "mezclar",
    "mezcláis": "mezclar",
    "mezcláramos": "mezclar",
    "mezcláremos": "mezclar",
    "mezclásemos": "mezclar",
    "mezclé": "mezclar",
    "mezcléis": "mezclar",
    "mezcló": "mezclar",
    "mezcolanzas": "mezcolanza",
    "mezo": "mecer",
    "mezquina": "mezquino",
    "mezquinas": "mezquino",
    "mezquindades": "mezquindad",
    "mezquinos": "mezquino",
    "mezquitas": "mezquita",
    "mezquites": "mezquite",
    "mezzo": "mezzosoprano",
    "mezzos": "mezzosoprano",
    "mezzosopranos": "mezzosoprano",
    "mezáis": "mecer",
    "meábamos": "mear",
    "meáis": "mear",
    "meáramos": "mear",
    "meáremos": "mear",
    "meásemos": "mear",
    "meé": "mear",
    "meéis": "mear",
    "meñiques": "meñique",
    "meó": "mear",
    "mg": "miligramo",
    "mhos": "mho",
    "miaba": "miar",
    "miabais": "miar",
    "miaban": "miar",
    "miabas": "miar",
    "miad": "miar",
    "miada": "miar",
    "miadas": "miar",
    "miado": "miar",
    "miados": "miar",
    "miajas": "miaja",
    "mialgias": "mialgia",
    "miamos": "miar",
    "miando": "miar",
    "miara": "miar",
    "miarais": "miar",
    "miaran": "miar",
    "miaras": "miar",
    "miare": "miar",
    "miareis": "miar",
    "miaremos": "miar",
    "miaren": "miar",
    "miares": "miar",
    "miargiritas": "miargirita",
    "miaron": "miar",
    "miará": "miar",
    "miarán": "miar",
    "miarás": "miar",
    "miaré": "miar",
    "miaréis": "miar",
    "miaría": "miar",
    "miaríais": "miar",
    "miaríamos": "miar",
    "miarían": "miar",
    "miarías": "miar",
    "miase": "miar",
    "miaseis": "miar",
    "miasen": "miar",
    "miases": "miar",
    "miasmas": "miasma",
    "miasmática": "miasmático",
    "miasmáticas": "miasmático",
    "miasmáticos": "miasmático",
    "miaste": "miar",
    "miasteis": "miar",
    "miastenias": "miastenia",
    "miaña": "miañar",
    "miañaba": "miañar",
    "miañabais": "miañar",
    "miañaban": "miañar",
    "miañabas": "miañar",
    "miañad": "miañar",
    "miañada": "miañar",
    "miañadas": "miañar",
    "miañado": "miañar",
    "miañados": "miañar",
    "miañamos": "miañar",
    "miañan": "miañar",
    "miañando": "miañar",
    "miañara": "miañar",
    "miañarais": "miañar",
    "miañaran": "miañar",
    "miañaras": "miañar",
    "miañare": "miañar",
    "miañareis": "miañar",
    "miañaremos": "miañar",
    "miañaren": "miañar",
    "miañares": "miañar",
    "miañaron": "miañar",
    "miañará": "miañar",
    "miañarán": "miañar",
    "miañarás": "miañar",
    "miañaré": "miañar",
    "miañaréis": "miañar",
    "miañaría": "miañar",
    "miañaríais": "miañar",
    "miañaríamos": "miañar",
    "miañarían": "miañar",
    "miañarías": "miañar",
    "miañas": "miañar",
    "miañase": "miañar",
    "miañaseis": "miañar",
    "miañasen": "miañar",
    "miañases": "miañar",
    "miañaste": "miañar",
    "miañasteis": "miañar",
    "miañe": "miañar",
    "miañemos": "miañar",
    "miañen": "miañar",
    "miañes": "miañar",
    "miaño": "miañar",
    "miañábamos": "miañar",
    "miañáis": "miañar",
    "miañáramos": "miañar",
    "miañáremos": "miañar",
    "miañásemos": "miañar",
    "miañé": "miañar",
    "miañéis": "miañar",
    "miañó": "miañar",
    "micacitas": "micacita",
    "micados": "micado",
    "micas": "mica",
    "micasquistos": "micasquisto",
    "micciones": "micción",
    "micelares": "micelar",
    "micelas": "micela",
    "micelios": "micelio",
    "miceres": "micer",
    "micetologías": "micetología",
    "micha": "micho",
    "michas": "micho",
    "michina": "michino",
    "michinas": "michino",
    "michinos": "michino",
    "michos": "micho",
    "miciculturas": "micicultura",
    "micologías": "micología",
    "micoplasmas": "micoplasma",
    "micoplasmologías": "micoplasmología",
    "micorrizas": "micorriza",
    "micos": "mico",
    "micostática": "micostático",
    "micostáticas": "micostático",
    "micostáticos": "micostático",
    "micotoxinas": "micotoxina",
    "micras": "micra",
    "microadoquines": "microadoquín",
    "microaglomerados": "microaglomerado",
    "microalbuminurias": "microalbuminuria",
    "microamperios": "microamperio",
    "microanalítica": "microanalítico",
    "microanalíticas": "microanalítico",
    "microanalíticos": "microanalítico",
    "microaspersiones": "microaspersión",
    "microaspersora": "microaspersor",
    "microaspersoras": "microaspersor",
    "microaspersores": "microaspersor",
    "microbalanzas": "microbalanza",
    "microbalones": "microbalón",
    "microbiana": "microbiano",
    "microbianas": "microbiano",
    "microbianos": "microbiano",
    "microbiologías": "microbiología",
    "microbiológica": "microbiológico",
    "microbiológicas": "microbiológico",
    "microbiológicos": "microbiológico",
    "microbios": "microbio",
    "microbióloga": "microbiólogo",
    "microbiólogas": "microbiólogo",
    "microbiólogos": "microbiólogo",
    "microburiles": "microburil",
    "microbuses": "microbús",
    "microcalculadores": "microcalculador",
    "microcalorímetros": "microcalorímetro",
    "microcanales": "microcanal",
    "microcarpa": "microcarpo",
    "microcarpas": "microcarpo",
    "microcarpos": "microcarpo",
    "microcasetes": "microcasete",
    "microcefalias": "microcefalia",
    "microcelulares": "microcelular",
    "microchipes": "microchip",
    "microciclos": "microciclo",
    "microcintas": "microcinta",
    "microcircuitos": "microcircuito",
    "microcirugías": "microcirugía",
    "microcitos": "microcito",
    "microclimas": "microclima",
    "microclimatologías": "microclimatología",
    "microclinas": "microclina",
    "microcolumnas": "microcolumna",
    "microcomputadora": "microcomputador",
    "microcomputadoras": "microcomputador",
    "microcomputadores": "microcomputador",
    "microconexiones": "microconexión",
    "microcontroladores": "microcontrolador",
    "microcontroles": "microcontrol",
    "microcopias": "microcopia",
    "microcranias": "microcrania",
    "microcristalina": "microcristalino",
    "microcristalinas": "microcristalino",
    "microcristalinos": "microcristalino",
    "microcromosomas": "microcromosoma",
    "microcuerpos": "microcuerpo",
    "microcéfala": "microcéfalo",
    "microcéfalas": "microcéfalo",
    "microcéfalos": "microcéfalo",
    "microcélulas": "microcélula",
    "microcítica": "microcítico",
    "microcíticas": "microcítico",
    "microcíticos": "microcítico",
    "microcódigos": "microcódigo",
    "microcórneas": "microcórnea",
    "microcósmica": "microcósmico",
    "microcósmicas": "microcósmico",
    "microcósmicos": "microcósmico",
    "microdescargas": "microdescarga",
    "microdifusora": "microdifusor",
    "microdifusoras": "microdifusor",
    "microdifusores": "microdifusor",
    "microdiscos": "microdisco",
    "microdisquetes": "microdisquete",
    "microdoncias": "microdoncia",
    "microdonta": "microdonto",
    "microdontas": "microdonto",
    "microdontismos": "microdontismo",
    "microdontos": "microdonto",
    "microduridades": "microduridad",
    "microdurómetros": "microdurómetro",
    "microeconomías": "microeconomía",
    "microeconómica": "microeconómico",
    "microeconómicas": "microeconómico",
    "microeconómicos": "microeconómico",
    "microediciones": "microedición",
    "microeditores": "microeditor",
    "microelectrónica": "microelectrónico",
    "microelectrónicas": "microelectrónico",
    "microelectrónicos": "microelectrónico",
    "microelementos": "microelemento",
    "microempaquetados": "microempaquetado",
    "microemulsiones": "microemulsión",
    "microencapsulaciones": "microencapsulación",
    "microensambladores": "microensamblador",
    "microenseñanzas": "microenseñanza",
    "microesquejes": "microesqueje",
    "microestaciones": "microestación",
    "microestada": "microestado",
    "microestadas": "microestado",
    "microestados": "microestado",
    "microestrabismos": "microestrabismo",
    "microestructuras": "microestructura",
    "microfagias": "microfagia",
    "microfaquias": "microfaquia",
    "microfaradios": "microfaradio",
    "microfarads": "microfarad",
    "microfaunas": "microfauna",
    "microfibras": "microfibra",
    "microfibrillas": "microfibrilla",
    "microfichas": "microficha",
    "microfila": "microfilo",
    "microfilamentos": "microfilamento",
    "microfilas": "microfilo",
    "microfilma": "microfilmar",
    "microfilmaba": "microfilmar",
    "microfilmabais": "microfilmar",
    "microfilmaban": "microfilmar",
    "microfilmabas": "microfilmar",
    "microfilmaciones": "microfilmación",
    "microfilmad": "microfilmar",
    "microfilmada": "microfilmar",
    "microfilmadas": "microfilmar",
    "microfilmado": "microfilmar",
    "microfilmadoras": "microfilmadora",
    "microfilmados": "microfilmar",
    "microfilmamos": "microfilmar",
    "microfilman": "microfilmar",
    "microfilmando": "microfilmar",
    "microfilmara": "microfilmar",
    "microfilmarais": "microfilmar",
    "microfilmaran": "microfilmar",
    "microfilmaras": "microfilmar",
    "microfilmare": "microfilmar",
    "microfilmareis": "microfilmar",
    "microfilmaremos": "microfilmar",
    "microfilmaren": "microfilmar",
    "microfilmares": "microfilmar",
    "microfilmaron": "microfilmar",
    "microfilmará": "microfilmar",
    "microfilmarán": "microfilmar",
    "microfilmarás": "microfilmar",
    "microfilmaré": "microfilmar",
    "microfilmaréis": "microfilmar",
    "microfilmaría": "microfilmar",
    "microfilmaríais": "microfilmar",
    "microfilmaríamos": "microfilmar",
    "microfilmarían": "microfilmar",
    "microfilmarías": "microfilmar",
    "microfilmas": "microfilmar",
    "microfilmase": "microfilmar",
    "microfilmaseis": "microfilmar",
    "microfilmasen": "microfilmar",
    "microfilmases": "microfilmar",
    "microfilmaste": "microfilmar",
    "microfilmasteis": "microfilmar",
    "microfilme": "microfilmar",
    "microfilmemos": "microfilmar",
    "microfilmen": "microfilmar",
    "microfilmes": "microfilme",
    "microfilmo": "microfilmar",
    "microfilms": "microfilm",
    "microfilmábamos": "microfilmar",
    "microfilmáis": "microfilmar",
    "microfilmáramos": "microfilmar",
    "microfilmáremos": "microfilmar",
    "microfilmásemos": "microfilmar",
    "microfilmé": "microfilmar",
    "microfilméis": "microfilmar",
    "microfilmó": "microfilmar",
    "microfilos": "microfilo",
    "microfiltraciones": "microfiltración",
    "microfisuras": "microfisura",
    "microfloras": "microflora",
    "microfonismos": "microfonismo",
    "microformas": "microforma",
    "microfotografías": "microfotografía",
    "microfotómetros": "microfotómetro",
    "microftalma": "microftalmo",
    "microftalmas": "microftalmo",
    "microftalmos": "microftalmo",
    "microftalmías": "microftalmía",
    "microfusiones": "microfusión",
    "microfónica": "microfónico",
    "microfónicas": "microfónico",
    "microfónicos": "microfónico",
    "microfósiles": "microfósil",
    "microgenias": "microgenia",
    "microgliacitos": "microgliacito",
    "microgliales": "microglial",
    "microglias": "microglia",
    "microglobulinas": "microglobulina",
    "microglosias": "microglosia",
    "microglósica": "microglósico",
    "microglósicas": "microglósico",
    "microglósicos": "microglósico",
    "micrognatias": "micrognatia",
    "micrografías": "micrografía",
    "micrográfica": "micrográfico",
    "micrográficas": "micrográfico",
    "micrográficos": "micrográfico",
    "microheterocromática": "microheterocromático",
    "microheterocromáticas": "microheterocromático",
    "microheterocromáticos": "microheterocromático",
    "microimpresoras": "microimpresora",
    "microimpresos": "microimpreso",
    "microimágenes": "microimagen",
    "microinformáticas": "microinformática",
    "microinstrucciones": "microinstrucción",
    "microinyecciones": "microinyección",
    "microlaringoscopias": "microlaringoscopia",
    "microlenguajes": "microlenguaje",
    "microlentes": "microlente",
    "microlitas": "microlita",
    "microlitografías": "microlitografía",
    "microlitos": "microlito",
    "micrologiciales": "micrologicial",
    "micrologías": "micrología",
    "microlíneas": "microlínea",
    "microlítica": "microlítico",
    "microlíticas": "microlítico",
    "microlíticos": "microlítico",
    "micrológica": "micrológico",
    "micrológicas": "micrológico",
    "micrológicos": "micrológico",
    "micromandíbulas": "micromandíbula",
    "micromanómetros": "micromanómetro",
    "micromarketings": "micromarketing",
    "micromecanizaciones": "micromecanización",
    "micromecánicas": "micromecánica",
    "micromelias": "micromelia",
    "micromemorias": "micromemoria",
    "micromerismos": "micromerismo",
    "micrometeoroides": "micrometeoroide",
    "micrometeorologías": "micrometeorología",
    "micrometrías": "micrometría",
    "micromilímetros": "micromilímetro",
    "microminiaturas": "microminiatura",
    "microminiaturizaciones": "microminiaturización",
    "micromorfologías": "micromorfología",
    "micromorfóloga": "micromorfólogo",
    "micromorfólogas": "micromorfólogo",
    "micromorfólogos": "micromorfólogo",
    "micromotores": "micromotor",
    "micromuestreos": "micromuestreo",
    "micromundos": "micromundo",
    "micromárketings": "micromárketing",
    "micromárquetings": "micromárqueting",
    "micrométodos": "micrométodo",
    "micrométrica": "micrométrico",
    "micrométricas": "micrométrico",
    "micrométricos": "micrométrico",
    "micromódulos": "micromódulo",
    "micrones": "micrón",
    "micronesia": "micronesio",
    "micronesias": "micronesio",
    "micronesios": "micronesio",
    "micronizaciones": "micronización",
    "micronutrientes": "micronutriente",
    "micronúcleos": "micronúcleo",
    "microoperaciones": "microoperación",
    "microordenadores": "microordenador",
    "microorganismos": "microorganismo",
    "micropaleontologías": "micropaleontología",
    "micropartículas": "micropartícula",
    "microparásitos": "microparásito",
    "microperforaciones": "microperforación",
    "micropilares": "micropilar",
    "microplacas": "microplaca",
    "microplaquetas": "microplaqueta",
    "microporosa": "microporoso",
    "microporosas": "microporoso",
    "microporosidades": "microporosidad",
    "microporosos": "microporoso",
    "microposicionamientos": "microposicionamiento",
    "microprocesadores": "microprocesador",
    "microprogramables": "microprogramable",
    "microprogramaciones": "microprogramación",
    "microprogramadores": "microprogramador",
    "microprogramas": "microprograma",
    "micropropagaciones": "micropropagación",
    "microproyectos": "microproyecto",
    "micropúas": "micropúa",
    "microquímicas": "microquímica",
    "microrreproducciones": "microrreproducción",
    "microrruptores": "microrruptor",
    "microrrutinas": "microrrutina",
    "micros": "micro",
    "microscopias": "microscopia",
    "microscopios": "microscopio",
    "microscópica": "microscópico",
    "microscópicas": "microscópico",
    "microscópicos": "microscópico",
    "microsecciones": "microsección",
    "microsegundos": "microsegundo",
    "microsismos": "microsismo",
    "microsistemas": "microsistema",
    "microsociologías": "microsociología",
    "microsomas": "microsoma",
    "microsondas": "microsonda",
    "microstructuras": "microstructura",
    "microsubrutinas": "microsubrutina",
    "microsurcos": "microsurco",
    "microsísmica": "microsísmico",
    "microsísmicas": "microsísmico",
    "microsísmicos": "microsísmico",
    "microsómica": "microsómico",
    "microsómicas": "microsómico",
    "microsómicos": "microsómico",
    "microtamizados": "microtamizado",
    "microteléfonos": "microteléfono",
    "microterminales": "microterminal",
    "microterremotos": "microterremoto",
    "microtexturas": "microtextura",
    "microtias": "microtia",
    "microtomías": "microtomía",
    "microtrabeculares": "microtrabecular",
    "microtractora": "microtractor",
    "microtractoras": "microtractor",
    "microtractores": "microtractor",
    "microtransparentes": "microtransparente",
    "microtratamientos": "microtratamiento",
    "microtérmica": "microtérmico",
    "microtérmicas": "microtérmico",
    "microtérmicos": "microtérmico",
    "microtúbulos": "microtúbulo",
    "microuniones": "microunión",
    "microvatios": "microvatio",
    "microvigas": "microviga",
    "microvoltios": "microvoltio",
    "microválvulas": "microválvula",
    "micrófaga": "micrófago",
    "micrófagas": "micrófago",
    "micrófagos": "micrófago",
    "micrófitos": "micrófito",
    "micrófonos": "micrófono",
    "micrómetros": "micrómetro",
    "micrónes": "micrón",
    "micrópilos": "micrópilo",
    "micrósporas": "micróspora",
    "micrótomos": "micrótomo",
    "micurés": "micuré",
    "micácea": "micáceo",
    "micáceas": "micáceo",
    "micáceos": "micáceo",
    "micénica": "micénico",
    "micénicas": "micénico",
    "micénicos": "micénico",
    "micóloga": "micólogo",
    "micólogas": "micólogo",
    "micólogos": "micólogo",
    "micótica": "micótico",
    "micóticas": "micótico",
    "micóticos": "micótico",
    "mida": "medir",
    "midamos": "medir",
    "midan": "medir",
    "midas": "medir",
    "mide": "medir",
    "miden": "medir",
    "mides": "medir",
    "midiendo": "medir",
    "midiera": "medir",
    "midierais": "medir",
    "midieran": "medir",
    "midieras": "medir",
    "midiere": "medir",
    "midiereis": "medir",
    "midieren": "medir",
    "midieres": "medir",
    "midieron": "medir",
    "midiese": "medir",
    "midieseis": "medir",
    "midiesen": "medir",
    "midieses": "medir",
    "midiéramos": "medir",
    "midiéremos": "medir",
    "midiésemos": "medir",
    "midió": "medir",
    "mido": "medir",
    "midriática": "midriático",
    "midriáticas": "midriático",
    "midriáticos": "midriático",
    "midáis": "medir",
    "miedos": "miedo",
    "miedosa": "miedoso",
    "miedosas": "miedoso",
    "miedosos": "miedoso",
    "miela": "melar",
    "mieladas": "mielada",
    "mielan": "melar",
    "mielas": "melar",
    "miele": "melar",
    "mielen": "melar",
    "mieles": "miel",
    "mielga": "mielgo",
    "mielgas": "mielgo",
    "mielgos": "mielgo",
    "mielinas": "mielina",
    "mielo": "melar",
    "mieloblastos": "mieloblasto",
    "mieloblástica": "mieloblástico",
    "mieloblásticas": "mieloblástico",
    "mieloblásticos": "mieloblástico",
    "mielocitos": "mielocito",
    "mielografías": "mielografía",
    "mieloides": "mieloide",
    "mielomas": "mieloma",
    "mielopatías": "mielopatía",
    "mieloperoxidasas": "mieloperoxidasa",
    "mieloplaxas": "mieloplaxa",
    "mielínica": "mielínico",
    "mielínicas": "mielínico",
    "mielínicos": "mielínico",
    "mielítica": "mielítico",
    "mielíticas": "mielítico",
    "mielíticos": "mielítico",
    "miembra": "miembro",
    "miembras": "miembro",
    "miembros": "miembro",
    "miemos": "miar",
    "mienta": "mentir",
    "mientan": "mentir",
    "mientas": "mentir",
    "miente": "mentir",
    "mienten": "mentir",
    "mientes": "miente",
    "miento": "mentir",
    "mientérica": "mientérico",
    "mientéricas": "mientérico",
    "mientéricos": "mientérico",
    "mieras": "miera",
    "mierdas": "mierda",
    "mies": "my",
    "mieses": "mies",
    "miga": "migar",
    "migaba": "migar",
    "migabais": "migar",
    "migaban": "migar",
    "migabas": "migar",
    "migad": "migar",
    "migada": "migar",
    "migadas": "migar",
    "migado": "migar",
    "migados": "migar",
    "migajas": "migaja",
    "migajones": "migajón",
    "migajosa": "migajoso",
    "migajosas": "migajoso",
    "migajosos": "migajoso",
    "migamos": "migar",
    "migan": "migar",
    "migando": "migar",
    "migara": "migar",
    "migarais": "migar",
    "migaran": "migar",
    "migaras": "migar",
    "migare": "migar",
    "migareis": "migar",
    "migaremos": "migar",
    "migaren": "migar",
    "migares": "migar",
    "migaron": "migar",
    "migará": "migar",
    "migarán": "migar",
    "migarás": "migar",
    "migaré": "migar",
    "migaréis": "migar",
    "migaría": "migar",
    "migaríais": "migar",
    "migaríamos": "migar",
    "migarían": "migar",
    "migarías": "migar",
    "migas": "migar",
    "migase": "migar",
    "migaseis": "migar",
    "migasen": "migar",
    "migases": "migar",
    "migaste": "migar",
    "migasteis": "migar",
    "migmatitas": "migmatita",
    "migo": "migar",
    "migra": "migrar",
    "migraba": "migrar",
    "migrabais": "migrar",
    "migraban": "migrar",
    "migrabas": "migrar",
    "migraciones": "migración",
    "migrad": "migrar",
    "migrada": "migrar",
    "migradas": "migrar",
    "migrado": "migrar",
    "migrados": "migrar",
    "migramos": "migrar",
    "migran": "migrar",
    "migrando": "migrar",
    "migrantes": "migrante",
    "migrara": "migrar",
    "migrarais": "migrar",
    "migraran": "migrar",
    "migraras": "migrar",
    "migrare": "migrar",
    "migrareis": "migrar",
    "migraremos": "migrar",
    "migraren": "migrar",
    "migrares": "migrar",
    "migraron": "migrar",
    "migrará": "migrar",
    "migrarán": "migrar",
    "migrarás": "migrar",
    "migraré": "migrar",
    "migraréis": "migrar",
    "migraría": "migrar",
    "migraríais": "migrar",
    "migraríamos": "migrar",
    "migrarían": "migrar",
    "migrarías": "migrar",
    "migras": "migrar",
    "migrase": "migrar",
    "migraseis": "migrar",
    "migrasen": "migrar",
    "migrases": "migrar",
    "migraste": "migrar",
    "migrasteis": "migrar",
    "migratoria": "migratorio",
    "migratorias": "migratorio",
    "migratorios": "migratorio",
    "migrañas": "migraña",
    "migre": "migrar",
    "migremos": "migrar",
    "migren": "migrar",
    "migres": "migrar",
    "migro": "migrar",
    "migrábamos": "migrar",
    "migráis": "migrar",
    "migráramos": "migrar",
    "migráremos": "migrar",
    "migrásemos": "migrar",
    "migré": "migrar",
    "migréis": "migrar",
    "migró": "migrar",
    "migue": "migar",
    "migueles": "miguel",
    "migueletes": "miguelete",
    "miguemos": "migar",
    "miguen": "migar",
    "miguera": "miguero",
    "migueras": "miguero",
    "migueros": "miguero",
    "migues": "migar",
    "migué": "migar",
    "miguéis": "migar",
    "migábamos": "migar",
    "migáis": "migar",
    "migáramos": "migar",
    "migáremos": "migar",
    "migásemos": "migar",
    "migó": "migar",
    "mihrabs": "mihrab",
    "mijos": "mijo",
    "mikados": "mikado",
    "miladies": "milady",
    "milagrera": "milagrero",
    "milagreras": "milagrero",
    "milagreros": "milagrero",
    "milagrerías": "milagrería",
    "milagros": "milagro",
    "milagrosa": "milagroso",
    "milagrosas": "milagroso",
    "milagrosos": "milagroso",
    "milanes": "milán",
    "milanesa": "milanés",
    "milanesas": "milanés",
    "milaneses": "milanés",
    "milanos": "milano",
    "mildeus": "mildeu",
    "mildius": "mildiu",
    "milena": "mileno",
    "milenaria": "milenario",
    "milenarias": "milenario",
    "milenarios": "milenario",
    "milenarismos": "milenarismo",
    "milenaristas": "milenarista",
    "milenas": "mileno",
    "milenios": "milenio",
    "milenos": "mileno",
    "milenramas": "milenrama",
    "miles": "mil",
    "milesia": "milesio",
    "milesias": "milesio",
    "milesios": "milesio",
    "milgranares": "milgranar",
    "miliamperios": "miliamperio",
    "miliares": "miliar",
    "miliaria": "miliario",
    "miliarias": "miliario",
    "miliarios": "miliario",
    "milibares": "milibar",
    "miliciana": "miliciano",
    "milicianas": "miliciano",
    "milicianos": "miliciano",
    "milicias": "milicia",
    "miligramos": "miligramo",
    "mililitros": "mililitro",
    "milimetrada": "milimetrado",
    "milimetradas": "milimetrado",
    "milimetrados": "milimetrado",
    "milipulgadas": "milipulgada",
    "milis": "mili",
    "milisegundos": "milisegundo",
    "milita": "militar",
    "militaba": "militar",
    "militabais": "militar",
    "militaban": "militar",
    "militabas": "militar",
    "militad": "militar",
    "militada": "militar",
    "militadas": "militar",
    "militado": "militar",
    "militados": "militar",
    "militamos": "militar",
    "militan": "militar",
    "militancias": "militancia",
    "militando": "militar",
    "militantes": "militante",
    "militara": "militar",
    "militaradas": "militarada",
    "militarais": "militar",
    "militaran": "militar",
    "militaras": "militara",
    "militare": "militar",
    "militareis": "militar",
    "militaremos": "militar",
    "militaren": "militar",
    "militares": "militar",
    "militarice": "militarizar",
    "militaricemos": "militarizar",
    "militaricen": "militarizar",
    "militarices": "militarizar",
    "militaricé": "militarizar",
    "militaricéis": "militarizar",
    "militarismos": "militarismo",
    "militaristas": "militarista",
    "militariza": "militarizar",
    "militarizaba": "militarizar",
    "militarizabais": "militarizar",
    "militarizaban": "militarizar",
    "militarizabas": "militarizar",
    "militarizaciones": "militarización",
    "militarizad": "militarizar",
    "militarizada": "militarizar",
    "militarizadas": "militarizar",
    "militarizado": "militarizar",
    "militarizados": "militarizar",
    "militarizamos": "militarizar",
    "militarizan": "militarizar",
    "militarizando": "militarizar",
    "militarizara": "militarizar",
    "militarizarais": "militarizar",
    "militarizaran": "militarizar",
    "militarizaras": "militarizar",
    "militarizare": "militarizar",
    "militarizareis": "militarizar",
    "militarizaremos": "militarizar",
    "militarizaren": "militarizar",
    "militarizares": "militarizar",
    "militarizaron": "militarizar",
    "militarizará": "militarizar",
    "militarizarán": "militarizar",
    "militarizarás": "militarizar",
    "militarizaré": "militarizar",
    "militarizaréis": "militarizar",
    "militarizaría": "militarizar",
    "militarizaríais": "militarizar",
    "militarizaríamos": "militarizar",
    "militarizarían": "militarizar",
    "militarizarías": "militarizar",
    "militarizas": "militarizar",
    "militarizase": "militarizar",
    "militarizaseis": "militarizar",
    "militarizasen": "militarizar",
    "militarizases": "militarizar",
    "militarizaste": "militarizar",
    "militarizasteis": "militarizar",
    "militarizo": "militarizar",
    "militarizábamos": "militarizar",
    "militarizáis": "militarizar",
    "militarizáramos": "militarizar",
    "militarizáremos": "militarizar",
    "militarizásemos": "militarizar",
    "militarizó": "militarizar",
    "militaron": "militar",
    "militará": "militar",
    "militarán": "militar",
    "militarás": "militar",
    "militaré": "militar",
    "militaréis": "militar",
    "militaría": "militar",
    "militaríais": "militar",
    "militaríamos": "militar",
    "militarían": "militar",
    "militarías": "militar",
    "militas": "militar",
    "militase": "militar",
    "militaseis": "militar",
    "militasen": "militar",
    "militases": "militar",
    "militaste": "militar",
    "militasteis": "militar",
    "milite": "militar",
    "militemos": "militar",
    "militen": "militar",
    "milites": "militar",
    "milito": "militar",
    "militábamos": "militar",
    "militáis": "militar",
    "militáramos": "militar",
    "militáremos": "militar",
    "militásemos": "militar",
    "milité": "militar",
    "militéis": "militar",
    "militó": "militar",
    "milivoltios": "milivoltio",
    "miliáreas": "miliárea",
    "millaradas": "millarada",
    "millardos": "millardo",
    "millares": "millar",
    "millas": "milla",
    "milleritas": "millerita",
    "millonadas": "millonada",
    "millonaria": "millonario",
    "millonarias": "millonario",
    "millonarios": "millonario",
    "milloncitos": "milloncito",
    "millones": "millón",
    "millonésima": "millonésimo",
    "millonésimas": "millonésimo",
    "millonésimo": "1000000",
    "millonésimos": "millonésimo",
    "millos": "millo",
    "milmillonésima": "milmillonésimo",
    "milmillonésimas": "milmillonésimo",
    "milmillonésimo": "1000000000",
    "milmillonésimos": "milmillonésimo",
    "milochas": "milocha",
    "milohioidea": "milohioideo",
    "milohioideas": "milohioideo",
    "milohioideos": "milohioideo",
    "milongas": "milonga",
    "milonguera": "milonguero",
    "milongueras": "milonguero",
    "milongueros": "milonguero",
    "milonitas": "milonita",
    "milores": "milord",
    "milpas": "milpa",
    "milpea": "milpear",
    "milpeaba": "milpear",
    "milpeabais": "milpear",
    "milpeaban": "milpear",
    "milpeabas": "milpear",
    "milpead": "milpear",
    "milpeada": "milpear",
    "milpeadas": "milpear",
    "milpeado": "milpear",
    "milpeados": "milpear",
    "milpeamos": "milpear",
    "milpean": "milpear",
    "milpeando": "milpear",
    "milpeara": "milpear",
    "milpearais": "milpear",
    "milpearan": "milpear",
    "milpearas": "milpear",
    "milpeare": "milpear",
    "milpeareis": "milpear",
    "milpearemos": "milpear",
    "milpearen": "milpear",
    "milpeares": "milpear",
    "milpearon": "milpear",
    "milpeará": "milpear",
    "milpearán": "milpear",
    "milpearás": "milpear",
    "milpearé": "milpear",
    "milpearéis": "milpear",
    "milpearía": "milpear",
    "milpearíais": "milpear",
    "milpearíamos": "milpear",
    "milpearían": "milpear",
    "milpearías": "milpear",
    "milpeas": "milpear",
    "milpease": "milpear",
    "milpeaseis": "milpear",
    "milpeasen": "milpear",
    "milpeases": "milpear",
    "milpeaste": "milpear",
    "milpeasteis": "milpear",
    "milpee": "milpear",
    "milpeemos": "milpear",
    "milpeen": "milpear",
    "milpees": "milpear",
    "milpeo": "milpear",
    "milpeábamos": "milpear",
    "milpeáis": "milpear",
    "milpeáramos": "milpear",
    "milpeáremos": "milpear",
    "milpeásemos": "milpear",
    "milpeé": "milpear",
    "milpeéis": "milpear",
    "milpeó": "milpear",
    "milésima": "1000",
    "milésimas": "milésima",
    "milésimo": "1000",
    "milésimos": "milésimo",
    "milígramos": "milígramo",
    "milímetros": "milímetro",
    "mima": "mimar",
    "mimaba": "mimar",
    "mimabais": "mimar",
    "mimaban": "mimar",
    "mimabas": "mimar",
    "mimad": "mimar",
    "mimada": "mimar",
    "mimadas": "mimar",
    "mimado": "mimar",
    "mimadora": "mimador",
    "mimadoras": "mimador",
    "mimadores": "mimador",
    "mimados": "mimar",
    "mimamos": "mimar",
    "miman": "mimar",
    "mimando": "mimar",
    "mimara": "mimar",
    "mimarais": "mimar",
    "mimaran": "mimar",
    "mimaras": "mimar",
    "mimare": "mimar",
    "mimareis": "mimar",
    "mimaremos": "mimar",
    "mimaren": "mimar",
    "mimares": "mimar",
    "mimaron": "mimar",
    "mimará": "mimar",
    "mimarán": "mimar",
    "mimarás": "mimar",
    "mimaré": "mimar",
    "mimaréis": "mimar",
    "mimaría": "mimar",
    "mimaríais": "mimar",
    "mimaríamos": "mimar",
    "mimarían": "mimar",
    "mimarías": "mimar",
    "mimas": "mimar",
    "mimase": "mimar",
    "mimaseis": "mimar",
    "mimasen": "mimar",
    "mimases": "mimar",
    "mimaste": "mimar",
    "mimasteis": "mimar",
    "mimbrales": "mimbral",
    "mimbrea": "mimbrear",
    "mimbreaba": "mimbrear",
    "mimbreabais": "mimbrear",
    "mimbreaban": "mimbrear",
    "mimbreabas": "mimbrear",
    "mimbread": "mimbrear",
    "mimbreada": "mimbrear",
    "mimbreadas": "mimbrear",
    "mimbreado": "mimbrear",
    "mimbreados": "mimbrear",
    "mimbreamos": "mimbrear",
    "mimbrean": "mimbrear",
    "mimbreando": "mimbrear",
    "mimbreara": "mimbrear",
    "mimbrearais": "mimbrear",
    "mimbrearan": "mimbrear",
    "mimbrearas": "mimbrear",
    "mimbreare": "mimbrear",
    "mimbreareis": "mimbrear",
    "mimbrearemos": "mimbrear",
    "mimbrearen": "mimbrear",
    "mimbreares": "mimbrear",
    "mimbrearon": "mimbrear",
    "mimbreará": "mimbrear",
    "mimbrearán": "mimbrear",
    "mimbrearás": "mimbrear",
    "mimbrearé": "mimbrear",
    "mimbrearéis": "mimbrear",
    "mimbrearía": "mimbrear",
    "mimbrearíais": "mimbrear",
    "mimbrearíamos": "mimbrear",
    "mimbrearían": "mimbrear",
    "mimbrearías": "mimbrear",
    "mimbreas": "mimbrear",
    "mimbrease": "mimbrear",
    "mimbreaseis": "mimbrear",
    "mimbreasen": "mimbrear",
    "mimbreases": "mimbrear",
    "mimbreaste": "mimbrear",
    "mimbreasteis": "mimbrear",
    "mimbree": "mimbrear",
    "mimbreemos": "mimbrear",
    "mimbreen": "mimbrear",
    "mimbrees": "mimbrear",
    "mimbreo": "mimbrear",
    "mimbrerales": "mimbreral",
    "mimbreras": "mimbrera",
    "mimbres": "mimbre",
    "mimbreábamos": "mimbrear",
    "mimbreáis": "mimbrear",
    "mimbreáramos": "mimbrear",
    "mimbreáremos": "mimbrear",
    "mimbreásemos": "mimbrear",
    "mimbreé": "mimbrear",
    "mimbreéis": "mimbrear",
    "mimbreña": "mimbreño",
    "mimbreñas": "mimbreño",
    "mimbreños": "mimbreño",
    "mimbreó": "mimbrear",
    "mimbrones": "mimbrón",
    "mimbrosa": "mimbroso",
    "mimbrosas": "mimbroso",
    "mimbrosos": "mimbroso",
    "mime": "mimar",
    "mimemos": "mimar",
    "mimen": "mimar",
    "mimeografías": "mimeografía",
    "mimes": "mimar",
    "mimetesitas": "mimetesita",
    "mimetice": "mimetizar",
    "mimeticemos": "mimetizar",
    "mimeticen": "mimetizar",
    "mimetices": "mimetizar",
    "mimeticé": "mimetizar",
    "mimeticéis": "mimetizar",
    "mimetismos": "mimetismo",
    "mimetitas": "mimetita",
    "mimetiza": "mimetizar",
    "mimetizaba": "mimetizar",
    "mimetizabais": "mimetizar",
    "mimetizaban": "mimetizar",
    "mimetizabas": "mimetizar",
    "mimetizad": "mimetizar",
    "mimetizada": "mimetizar",
    "mimetizadas": "mimetizar",
    "mimetizado": "mimetizar",
    "mimetizados": "mimetizar",
    "mimetizamos": "mimetizar",
    "mimetizan": "mimetizar",
    "mimetizando": "mimetizar",
    "mimetizara": "mimetizar",
    "mimetizarais": "mimetizar",
    "mimetizaran": "mimetizar",
    "mimetizaras": "mimetizar",
    "mimetizare": "mimetizar",
    "mimetizareis": "mimetizar",
    "mimetizaremos": "mimetizar",
    "mimetizaren": "mimetizar",
    "mimetizares": "mimetizar",
    "mimetizaron": "mimetizar",
    "mimetizará": "mimetizar",
    "mimetizarán": "mimetizar",
    "mimetizarás": "mimetizar",
    "mimetizaré": "mimetizar",
    "mimetizaréis": "mimetizar",
    "mimetizaría": "mimetizar",
    "mimetizaríais": "mimetizar",
    "mimetizaríamos": "mimetizar",
    "mimetizarían": "mimetizar",
    "mimetizarías": "mimetizar",
    "mimetizas": "mimetizar",
    "mimetizase": "mimetizar",
    "mimetizaseis": "mimetizar",
    "mimetizasen": "mimetizar",
    "mimetizases": "mimetizar",
    "mimetizaste": "mimetizar",
    "mimetizasteis": "mimetizar",
    "mimetizo": "mimetizar",
    "mimetizábamos": "mimetizar",
    "mimetizáis": "mimetizar",
    "mimetizáramos": "mimetizar",
    "mimetizáremos": "mimetizar",
    "mimetizásemos": "mimetizar",
    "mimetizó": "mimetizar",
    "mimeógrafos": "mimeógrafo",
    "mimo": "mimar",
    "mimodramas": "mimodrama",
    "mimos": "mimo",
    "mimosa": "mimoso",
    "mimosas": "mimoso",
    "mimosos": "mimoso",
    "mimosácea": "mimosáceo",
    "mimosáceas": "mimosáceo",
    "mimosáceos": "mimosáceo",
    "mimábamos": "mimar",
    "mimáis": "mimar",
    "mimáramos": "mimar",
    "mimáremos": "mimar",
    "mimásemos": "mimar",
    "mimé": "mimar",
    "miméis": "mimar",
    "mimética": "mimético",
    "miméticas": "mimético",
    "miméticos": "mimético",
    "mimó": "mimar",
    "mimógrafa": "mimógrafo",
    "mimógrafas": "mimógrafo",
    "mimógrafos": "mimógrafo",
    "min": "minuto",
    "mina": "minar",
    "minaba": "minar",
    "minabais": "minar",
    "minaban": "minar",
    "minabas": "minar",
    "minad": "minar",
    "minada": "minar",
    "minadas": "minar",
    "minado": "minar",
    "minadora": "minador",
    "minadoras": "minador",
    "minadores": "minador",
    "minados": "minar",
    "minales": "minal",
    "minamos": "minar",
    "minan": "minar",
    "minando": "minar",
    "minara": "minar",
    "minarais": "minar",
    "minaran": "minar",
    "minaras": "minar",
    "minare": "minar",
    "minareis": "minar",
    "minaremos": "minar",
    "minaren": "minar",
    "minares": "minar",
    "minaretes": "minarete",
    "minaron": "minar",
    "minará": "minar",
    "minarán": "minar",
    "minarás": "minar",
    "minaré": "minar",
    "minaréis": "minar",
    "minaría": "minar",
    "minaríais": "minar",
    "minaríamos": "minar",
    "minarían": "minar",
    "minarías": "minar",
    "minas": "minar",
    "minase": "minar",
    "minaseis": "minar",
    "minasen": "minar",
    "minases": "minar",
    "minaste": "minar",
    "minasteis": "minar",
    "mindeles": "mindel",
    "mine": "minar",
    "minemos": "minar",
    "minen": "minar",
    "minera": "minero",
    "minerajes": "mineraje",
    "mineralera": "mineralero",
    "mineraleras": "mineralero",
    "mineraleros": "mineralero",
    "minerales": "mineral",
    "mineralice": "mineralizar",
    "mineralicemos": "mineralizar",
    "mineralicen": "mineralizar",
    "mineralices": "mineralizar",
    "mineralicé": "mineralizar",
    "mineralicéis": "mineralizar",
    "mineraliza": "mineralizar",
    "mineralizaba": "mineralizar",
    "mineralizabais": "mineralizar",
    "mineralizaban": "mineralizar",
    "mineralizabas": "mineralizar",
    "mineralizaciones": "mineralización",
    "mineralizad": "mineralizar",
    "mineralizada": "mineralizar",
    "mineralizadas": "mineralizar",
    "mineralizado": "mineralizar",
    "mineralizadores": "mineralizador",
    "mineralizados": "mineralizar",
    "mineralizamos": "mineralizar",
    "mineralizan": "mineralizar",
    "mineralizando": "mineralizar",
    "mineralizara": "mineralizar",
    "mineralizarais": "mineralizar",
    "mineralizaran": "mineralizar",
    "mineralizaras": "mineralizar",
    "mineralizare": "mineralizar",
    "mineralizareis": "mineralizar",
    "mineralizaremos": "mineralizar",
    "mineralizaren": "mineralizar",
    "mineralizares": "mineralizar",
    "mineralizaron": "mineralizar",
    "mineralizará": "mineralizar",
    "mineralizarán": "mineralizar",
    "mineralizarás": "mineralizar",
    "mineralizaré": "mineralizar",
    "mineralizaréis": "mineralizar",
    "mineralizaría": "mineralizar",
    "mineralizaríais": "mineralizar",
    "mineralizaríamos": "mineralizar",
    "mineralizarían": "mineralizar",
    "mineralizarías": "mineralizar",
    "mineralizas": "mineralizar",
    "mineralizase": "mineralizar",
    "mineralizaseis": "mineralizar",
    "mineralizasen": "mineralizar",
    "mineralizases": "mineralizar",
    "mineralizaste": "mineralizar",
    "mineralizasteis": "mineralizar",
    "mineralizo": "mineralizar",
    "mineralizábamos": "mineralizar",
    "mineralizáis": "mineralizar",
    "mineralizáramos": "mineralizar",
    "mineralizáremos": "mineralizar",
    "mineralizásemos": "mineralizar",
    "mineralizó": "mineralizar",
    "mineralogistas": "mineralogista",
    "mineralogías": "mineralogía",
    "mineraloides": "mineraloide",
    "mineralurgias": "mineralurgia",
    "mineralógica": "mineralógico",
    "mineralógicas": "mineralógico",
    "mineralógicos": "mineralógico",
    "mineras": "minero",
    "mineros": "minero",
    "minervas": "minerva",
    "minervistas": "minervista",
    "minerías": "minería",
    "mines": "minar",
    "minestrones": "minestrone",
    "mingitoria": "mingitorio",
    "mingitorias": "mingitorio",
    "mingitorios": "mingitorio",
    "mingos": "mingo",
    "mini-bares": "mini-bar",
    "minia": "miniar",
    "miniaba": "miniar",
    "miniabais": "miniar",
    "miniaban": "miniar",
    "miniabas": "miniar",
    "miniad": "miniar",
    "miniada": "miniar",
    "miniadas": "miniar",
    "miniado": "miniar",
    "miniados": "miniar",
    "miniamos": "miniar",
    "minian": "miniar",
    "miniando": "miniar",
    "miniara": "miniar",
    "miniarais": "miniar",
    "miniaran": "miniar",
    "miniaras": "miniar",
    "miniare": "miniar",
    "miniareis": "miniar",
    "miniaremos": "miniar",
    "miniaren": "miniar",
    "miniares": "miniar",
    "miniaron": "miniar",
    "miniará": "miniar",
    "miniarán": "miniar",
    "miniarás": "miniar",
    "miniaré": "miniar",
    "miniaréis": "miniar",
    "miniaría": "miniar",
    "miniaríais": "miniar",
    "miniaríamos": "miniar",
    "miniarían": "miniar",
    "miniarías": "miniar",
    "minias": "miniar",
    "miniase": "miniar",
    "miniaseis": "miniar",
    "miniasen": "miniar",
    "miniases": "miniar",
    "miniaste": "miniar",
    "miniasteis": "miniar",
    "miniaturas": "miniatura",
    "miniaturistas": "miniaturista",
    "miniaturizaciones": "miniaturización",
    "minibares": "minibar",
    "miniboomes": "miniboom",
    "minibooms": "miniboom",
    "minicartuchos": "minicartucho",
    "minicasetes": "minicasete",
    "minicelebraciones": "minicelebración",
    "minicomputadores": "minicomputador",
    "minidevaluaciones": "minidevaluación",
    "minidiscos": "minidisco",
    "minidisquetes": "minidisquete",
    "minidistribuidora": "minidistribuidor",
    "minidistribuidoras": "minidistribuidor",
    "minidistribuidores": "minidistribuidor",
    "minie": "miniar",
    "miniemos": "miniar",
    "minien": "miniar",
    "minies": "miniar",
    "minifaldas": "minifalda",
    "miniferias": "miniferia",
    "minifundios": "minifundio",
    "minifundismos": "minifundismo",
    "minifundistas": "minifundista",
    "minimalice": "minimalizar",
    "minimalicemos": "minimalizar",
    "minimalicen": "minimalizar",
    "minimalices": "minimalizar",
    "minimalicé": "minimalizar",
    "minimalicéis": "minimalizar",
    "minimalismos": "minimalismo",
    "minimalistas": "minimalista",
    "minimaliza": "minimalizar",
    "minimalizaba": "minimalizar",
    "minimalizabais": "minimalizar",
    "minimalizaban": "minimalizar",
    "minimalizabas": "minimalizar",
    "minimalizad": "minimalizar",
    "minimalizada": "minimalizar",
    "minimalizadas": "minimalizar",
    "minimalizado": "minimalizar",
    "minimalizados": "minimalizar",
    "minimalizamos": "minimalizar",
    "minimalizan": "minimalizar",
    "minimalizando": "minimalizar",
    "minimalizara": "minimalizar",
    "minimalizarais": "minimalizar",
    "minimalizaran": "minimalizar",
    "minimalizaras": "minimalizar",
    "minimalizare": "minimalizar",
    "minimalizareis": "minimalizar",
    "minimalizaremos": "minimalizar",
    "minimalizaren": "minimalizar",
    "minimalizares": "minimalizar",
    "minimalizaron": "minimalizar",
    "minimalizará": "minimalizar",
    "minimalizarán": "minimalizar",
    "minimalizarás": "minimalizar",
    "minimalizaré": "minimalizar",
    "minimalizaréis": "minimalizar",
    "minimalizaría": "minimalizar",
    "minimalizaríais": "minimalizar",
    "minimalizaríamos": "minimalizar",
    "minimalizarían": "minimalizar",
    "minimalizarías": "minimalizar",
    "minimalizas": "minimalizar",
    "minimalizase": "minimalizar",
    "minimalizaseis": "minimalizar",
    "minimalizasen": "minimalizar",
    "minimalizases": "minimalizar",
    "minimalizaste": "minimalizar",
    "minimalizasteis": "minimalizar",
    "minimalizo": "minimalizar",
    "minimalizábamos": "minimalizar",
    "minimalizáis": "minimalizar",
    "minimalizáramos": "minimalizar",
    "minimalizáremos": "minimalizar",
    "minimalizásemos": "minimalizar",
    "minimalizó": "minimalizar",
    "minimice": "minimizar",
    "minimicemos": "minimizar",
    "minimicen": "minimizar",
    "minimices": "minimizar",
    "minimicé": "minimizar",
    "minimicéis": "minimizar",
    "minimiosinas": "minimiosina",
    "minimistas": "minimista",
    "minimiza": "minimizar",
    "minimizaba": "minimizar",
    "minimizabais": "minimizar",
    "minimizaban": "minimizar",
    "minimizabas": "minimizar",
    "minimizables": "minimizable",
    "minimizaciones": "minimización",
    "minimizad": "minimizar",
    "minimizada": "minimizar",
    "minimizadas": "minimizar",
    "minimizado": "minimizar",
    "minimizados": "minimizar",
    "minimizamos": "minimizar",
    "minimizan": "minimizar",
    "minimizando": "minimizar",
    "minimizara": "minimizar",
    "minimizarais": "minimizar",
    "minimizaran": "minimizar",
    "minimizaras": "minimizar",
    "minimizare": "minimizar",
    "minimizareis": "minimizar",
    "minimizaremos": "minimizar",
    "minimizaren": "minimizar",
    "minimizares": "minimizar",
    "minimizaron": "minimizar",
    "minimizará": "minimizar",
    "minimizarán": "minimizar",
    "minimizarás": "minimizar",
    "minimizaré": "minimizar",
    "minimizaréis": "minimizar",
    "minimizaría": "minimizar",
    "minimizaríais": "minimizar",
    "minimizaríamos": "minimizar",
    "minimizarían": "minimizar",
    "minimizarías": "minimizar",
    "minimizas": "minimizar",
    "minimizase": "minimizar",
    "minimizaseis": "minimizar",
    "minimizasen": "minimizar",
    "minimizases": "minimizar",
    "minimizaste": "minimizar",
    "minimizasteis": "minimizar",
    "minimizo": "minimizar",
    "minimizábamos": "minimizar",
    "minimizáis": "minimizar",
    "minimizáramos": "minimizar",
    "minimizáremos": "minimizar",
    "minimizásemos": "minimizar",
    "minimizó": "minimizar",
    "minina": "minino",
    "mininas": "minino",
    "mininos": "minino",
    "minio": "miniar",
    "miniordenadores": "miniordenador",
    "minios": "minio",
    "minirotondas": "minirotonda",
    "ministeriales": "ministerial",
    "ministerialismos": "ministerialismo",
    "ministerios": "ministerio",
    "ministra": "ministro",
    "ministrables": "ministrable",
    "ministrantes": "ministrante",
    "ministras": "ministro",
    "ministriles": "ministril",
    "ministros": "ministro",
    "minitractora": "minitractor",
    "minitractoras": "minitractor",
    "minitractores": "minitractor",
    "miniversiones": "miniversión",
    "miniábamos": "miniar",
    "miniáis": "miniar",
    "miniáramos": "miniar",
    "miniáremos": "miniar",
    "miniásemos": "miniar",
    "minié": "miniar",
    "miniéis": "miniar",
    "minió": "miniar",
    "mino": "minar",
    "minoica": "minoico",
    "minoicas": "minoico",
    "minoicos": "minoico",
    "minora": "minorar",
    "minoraba": "minorar",
    "minorabais": "minorar",
    "minoraban": "minorar",
    "minorabas": "minorar",
    "minoraciones": "minoración",
    "minorad": "minorar",
    "minorada": "minorar",
    "minoradas": "minorar",
    "minorado": "minorar",
    "minorados": "minorar",
    "minoramos": "minorar",
    "minoran": "minorar",
    "minorando": "minorar",
    "minorara": "minorar",
    "minorarais": "minorar",
    "minoraran": "minorar",
    "minoraras": "minorar",
    "minorare": "minorar",
    "minorareis": "minorar",
    "minoraremos": "minorar",
    "minoraren": "minorar",
    "minorares": "minorar",
    "minoraron": "minorar",
    "minorará": "minorar",
    "minorarán": "minorar",
    "minorarás": "minorar",
    "minoraré": "minorar",
    "minoraréis": "minorar",
    "minoraría": "minorar",
    "minoraríais": "minorar",
    "minoraríamos": "minorar",
    "minorarían": "minorar",
    "minorarías": "minorar",
    "minoras": "minorar",
    "minorase": "minorar",
    "minoraseis": "minorar",
    "minorasen": "minorar",
    "minorases": "minorar",
    "minoraste": "minorar",
    "minorasteis": "minorar",
    "minorativa": "minorativo",
    "minorativas": "minorativo",
    "minorativos": "minorativo",
    "minore": "minorar",
    "minoremos": "minorar",
    "minoren": "minorar",
    "minores": "minorar",
    "minoridades": "minoridad",
    "minoristas": "minorista",
    "minoritaria": "minoritario",
    "minoritarias": "minoritario",
    "minoritarios": "minoritario",
    "minoritas": "minorita",
    "minoro": "minorar",
    "minorábamos": "minorar",
    "minoráis": "minorar",
    "minoráramos": "minorar",
    "minoráremos": "minorar",
    "minorásemos": "minorar",
    "minoré": "minorar",
    "minoréis": "minorar",
    "minorías": "minoría",
    "minoró": "minorar",
    "minotauros": "minotauro",
    "mintamos": "mentir",
    "mintiendo": "mentir",
    "mintiera": "mentir",
    "mintierais": "mentir",
    "mintieran": "mentir",
    "mintieras": "mentir",
    "mintiere": "mentir",
    "mintiereis": "mentir",
    "mintieren": "mentir",
    "mintieres": "mentir",
    "mintieron": "mentir",
    "mintiese": "mentir",
    "mintieseis": "mentir",
    "mintiesen": "mentir",
    "mintieses": "mentir",
    "mintiéramos": "mentir",
    "mintiéremos": "mentir",
    "mintiésemos": "mentir",
    "mintió": "mentir",
    "mintáis": "mentir",
    "minucias": "minucia",
    "minuciosa": "minucioso",
    "minuciosas": "minucioso",
    "minuciosidades": "minuciosidad",
    "minuciosos": "minucioso",
    "minuendos": "minuendo",
    "minuetos": "minueto",
    "minusvalora": "minusvalorar",
    "minusvaloraba": "minusvalorar",
    "minusvalorabais": "minusvalorar",
    "minusvaloraban": "minusvalorar",
    "minusvalorabas": "minusvalorar",
    "minusvalorad": "minusvalorar",
    "minusvalorada": "minusvalorar",
    "minusvaloradas": "minusvalorar",
    "minusvalorado": "minusvalorar",
    "minusvalorados": "minusvalorar",
    "minusvaloramos": "minusvalorar",
    "minusvaloran": "minusvalorar",
    "minusvalorando": "minusvalorar",
    "minusvalorara": "minusvalorar",
    "minusvalorarais": "minusvalorar",
    "minusvaloraran": "minusvalorar",
    "minusvaloraras": "minusvalorar",
    "minusvalorare": "minusvalorar",
    "minusvalorareis": "minusvalorar",
    "minusvaloraremos": "minusvalorar",
    "minusvaloraren": "minusvalorar",
    "minusvalorares": "minusvalorar",
    "minusvaloraron": "minusvalorar",
    "minusvalorará": "minusvalorar",
    "minusvalorarán": "minusvalorar",
    "minusvalorarás": "minusvalorar",
    "minusvaloraré": "minusvalorar",
    "minusvaloraréis": "minusvalorar",
    "minusvaloraría": "minusvalorar",
    "minusvaloraríais": "minusvalorar",
    "minusvaloraríamos": "minusvalorar",
    "minusvalorarían": "minusvalorar",
    "minusvalorarías": "minusvalorar",
    "minusvaloras": "minusvalorar",
    "minusvalorase": "minusvalorar",
    "minusvaloraseis": "minusvalorar",
    "minusvalorasen": "minusvalorar",
    "minusvalorases": "minusvalorar",
    "minusvaloraste": "minusvalorar",
    "minusvalorasteis": "minusvalorar",
    "minusvalore": "minusvalorar",
    "minusvaloremos": "minusvalorar",
    "minusvaloren": "minusvalorar",
    "minusvalores": "minusvalorar",
    "minusvaloro": "minusvalorar",
    "minusvalorábamos": "minusvalorar",
    "minusvaloráis": "minusvalorar",
    "minusvaloráramos": "minusvalorar",
    "minusvaloráremos": "minusvalorar",
    "minusvalorásemos": "minusvalorar",
    "minusvaloré": "minusvalorar",
    "minusvaloréis": "minusvalorar",
    "minusvaloró": "minusvalorar",
    "minusvalías": "minusvalía",
    "minusválida": "minusválido",
    "minusválidas": "minusválido",
    "minusválidos": "minusválido",
    "minuta": "minuto",
    "minutaba": "minutar",
    "minutabais": "minutar",
    "minutaban": "minutar",
    "minutabas": "minutar",
    "minutad": "minutar",
    "minutada": "minutar",
    "minutadas": "minutar",
    "minutado": "minutar",
    "minutados": "minutar",
    "minutajes": "minutaje",
    "minutamos": "minutar",
    "minutan": "minutar",
    "minutando": "minutar",
    "minutara": "minutar",
    "minutarais": "minutar",
    "minutaran": "minutar",
    "minutaras": "minutar",
    "minutare": "minutar",
    "minutareis": "minutar",
    "minutaremos": "minutar",
    "minutaren": "minutar",
    "minutares": "minutar",
    "minutarios": "minutario",
    "minutaron": "minutar",
    "minutará": "minutar",
    "minutarán": "minutar",
    "minutarás": "minutar",
    "minutaré": "minutar",
    "minutaréis": "minutar",
    "minutaría": "minutar",
    "minutaríais": "minutar",
    "minutaríamos": "minutar",
    "minutarían": "minutar",
    "minutarías": "minutar",
    "minutas": "minuto",
    "minutase": "minutar",
    "minutaseis": "minutar",
    "minutasen": "minutar",
    "minutases": "minutar",
    "minutaste": "minutar",
    "minutasteis": "minutar",
    "minute": "minutar",
    "minutemos": "minutar",
    "minuten": "minutar",
    "minuteras": "minutera",
    "minuteros": "minutero",
    "minutes": "minutar",
    "minutisas": "minutisa",
    "minuto": "minutar",
    "minutos": "minuto",
    "minutábamos": "minutar",
    "minutáis": "minutar",
    "minutáramos": "minutar",
    "minutáremos": "minutar",
    "minutásemos": "minutar",
    "minuté": "minutar",
    "minutéis": "minutar",
    "minutó": "minutar",
    "minués": "minué",
    "minábamos": "minar",
    "mináis": "minar",
    "mináramos": "minar",
    "mináremos": "minar",
    "minásemos": "minar",
    "miné": "minar",
    "minéis": "minar",
    "minó": "minar",
    "minúscula": "minúsculo",
    "minúsculas": "minúsculo",
    "minúsculos": "minúsculo",
    "mioblastos": "mioblasto",
    "miocardias": "miocardia",
    "miocardiopatías": "miocardiopatía",
    "miocardios": "miocardio",
    "mioceles": "miocele",
    "miocena": "mioceno",
    "miocenas": "mioceno",
    "miocenos": "mioceno",
    "miocénica": "miocénico",
    "miocénicas": "miocénico",
    "miocénicos": "miocénico",
    "miodesopsias": "miodesopsia",
    "miodinias": "miodinia",
    "mioelástica": "mioelástico",
    "mioelásticas": "mioelástico",
    "mioelásticos": "mioelástico",
    "miofibrillas": "miofibrilla",
    "miofilamentos": "miofilamento",
    "miofuncionales": "miofuncional",
    "mioglobinas": "mioglobina",
    "miografías": "miografía",
    "miográfica": "miográfico",
    "miográficas": "miográfico",
    "miográficos": "miográfico",
    "miolemas": "miolema",
    "miologías": "miología",
    "miológica": "miológico",
    "miológicas": "miológico",
    "miológicos": "miológico",
    "miomas": "mioma",
    "miomesinas": "miomesina",
    "mionemas": "mionema",
    "miopatías": "miopatía",
    "miopes": "miope",
    "miopías": "miopía",
    "miosinas": "miosina",
    "miotomías": "miotomía",
    "miotonías": "miotonía",
    "miotónica": "miotónico",
    "miotónicas": "miotónico",
    "miotónicos": "miotónico",
    "miqueletes": "miquelete",
    "mira": "mirar",
    "miraba": "mirar",
    "mirabais": "mirar",
    "miraban": "mirar",
    "mirabas": "mirar",
    "mirabilitas": "mirabilita",
    "mirabolanos": "mirabolano",
    "miracielos": "miracielo",
    "mirad": "mirar",
    "mirada": "mirar",
    "miradas": "mirar",
    "miraderos": "miradero",
    "mirado": "mirar",
    "miradora": "mirador",
    "miradoras": "mirador",
    "miradores": "mirador",
    "mirados": "mirar",
    "miraduras": "miradura",
    "miraguanos": "miraguano",
    "mirajes": "miraje",
    "mirallada": "mirallado",
    "miralladas": "mirallado",
    "mirallados": "mirallado",
    "miramamolines": "miramamolín",
    "miramientos": "miramiento",
    "miramos": "mirar",
    "miran": "mirar",
    "mirandas": "miranda",
    "mirando": "mirar",
    "mirara": "mirar",
    "mirarais": "mirar",
    "miraran": "mirar",
    "miraras": "mirar",
    "mirare": "mirar",
    "mirareis": "mirar",
    "miraremos": "mirar",
    "miraren": "mirar",
    "mirares": "mirar",
    "miraron": "mirar",
    "mirará": "mirar",
    "mirarán": "mirar",
    "mirarás": "mirar",
    "miraré": "mirar",
    "miraréis": "mirar",
    "miraría": "mirar",
    "miraríais": "mirar",
    "miraríamos": "mirar",
    "mirarían": "mirar",
    "mirarías": "mirar",
    "miras": "mirar",
    "mirase": "mirar",
    "miraseis": "mirar",
    "mirasen": "mirar",
    "mirases": "mirar",
    "mirasoles": "mirasol",
    "miraste": "mirar",
    "mirasteis": "mirar",
    "mire": "mirar",
    "miremos": "mirar",
    "miren": "mirar",
    "mires": "mirar",
    "miriada": "miriado",
    "miriadas": "miriado",
    "miriados": "miriado",
    "miriamétrica": "miriamétrico",
    "miriamétricas": "miriamétrico",
    "miriamétricos": "miriamétrico",
    "mirillas": "mirilla",
    "miringoestapediopexias": "miringoestapediopexia",
    "miringoplastias": "miringoplastia",
    "miringotomías": "miringotomía",
    "miristatos": "miristato",
    "miristicácea": "miristicáceo",
    "miristicáceas": "miristicáceo",
    "miristicáceos": "miristicáceo",
    "miristoleatos": "miristoleato",
    "miriámetros": "miriámetro",
    "miriápodos": "miriápodo",
    "miriñaques": "miriñaque",
    "miriópodos": "miriópodo",
    "mirlos": "mirlo",
    "miro": "mirar",
    "mirobrigenses": "mirobrigense",
    "mirona": "mirón",
    "mironas": "mirón",
    "mirones": "mirón",
    "mirrada": "mirrado",
    "mirradas": "mirrado",
    "mirrados": "mirrado",
    "mirras": "mirra",
    "mirrina": "mirrino",
    "mirrinas": "mirrino",
    "mirrinos": "mirrino",
    "mirsinácea": "mirsináceo",
    "mirsináceas": "mirsináceo",
    "mirsináceos": "mirsináceo",
    "mirtiformes": "mirtiforme",
    "mirtilos": "mirtilo",
    "mirtina": "mirtino",
    "mirtinas": "mirtino",
    "mirtinos": "mirtino",
    "mirtos": "mirto",
    "mirtácea": "mirtáceo",
    "mirtáceas": "mirtáceo",
    "mirtáceos": "mirtáceo",
    "mirtídanos": "mirtídano",
    "mirzas": "mirza",
    "mirábamos": "mirar",
    "miráis": "mirar",
    "miráramos": "mirar",
    "miráremos": "mirar",
    "mirásemos": "mirar",
    "miré": "mirar",
    "miréis": "mirar",
    "miríadas": "miríada",
    "mirífica": "mirífico",
    "miríficas": "mirífico",
    "miríficos": "mirífico",
    "mirística": "mirístico",
    "mirísticas": "mirístico",
    "mirísticos": "mirístico",
    "miró": "mirar",
    "mis": "mi",
    "misacantanos": "misacantano",
    "misales": "misal",
    "misantropías": "misantropía",
    "misantrópica": "misantrópico",
    "misantrópicas": "misantrópico",
    "misantrópicos": "misantrópico",
    "misas": "misa",
    "miscelánea": "misceláneo",
    "misceláneas": "misceláneo",
    "misceláneos": "misceláneo",
    "miscibilidades": "miscibilidad",
    "miscibles": "miscible",
    "misera": "mísero",
    "miserables": "miserable",
    "miseraciones": "miseración",
    "miseranda": "miserando",
    "miserandas": "miserando",
    "miserandos": "miserando",
    "miseras": "mísero",
    "miserea": "miserear",
    "misereaba": "miserear",
    "misereabais": "miserear",
    "misereaban": "miserear",
    "misereabas": "miserear",
    "miseread": "miserear",
    "misereada": "miserear",
    "misereadas": "miserear",
    "misereado": "miserear",
    "misereados": "miserear",
    "misereamos": "miserear",
    "miserean": "miserear",
    "misereando": "miserear",
    "misereara": "miserear",
    "miserearais": "miserear",
    "miserearan": "miserear",
    "miserearas": "miserear",
    "misereare": "miserear",
    "misereareis": "miserear",
    "miserearemos": "miserear",
    "miserearen": "miserear",
    "misereares": "miserear",
    "miserearon": "miserear",
    "misereará": "miserear",
    "miserearán": "miserear",
    "miserearás": "miserear",
    "miserearé": "miserear",
    "miserearéis": "miserear",
    "miserearía": "miserear",
    "miserearíais": "miserear",
    "miserearíamos": "miserear",
    "miserearían": "miserear",
    "miserearías": "miserear",
    "misereas": "miserear",
    "miserease": "miserear",
    "misereaseis": "miserear",
    "misereasen": "miserear",
    "misereases": "miserear",
    "misereaste": "miserear",
    "misereasteis": "miserear",
    "miseree": "miserear",
    "misereemos": "miserear",
    "misereen": "miserear",
    "miserees": "miserear",
    "misereo": "miserear",
    "misereres": "miserere",
    "misereábamos": "miserear",
    "misereáis": "miserear",
    "misereáramos": "miserear",
    "misereáremos": "miserear",
    "misereásemos": "miserear",
    "misereé": "miserear",
    "misereéis": "miserear",
    "misereó": "miserear",
    "miserias": "miseria",
    "misericordias": "misericordia",
    "misericordiosa": "misericordioso",
    "misericordiosas": "misericordioso",
    "misericordiosos": "misericordioso",
    "miseros": "mísero",
    "misiles": "misil",
    "misiona": "misionar",
    "misionaba": "misionar",
    "misionabais": "misionar",
    "misionaban": "misionar",
    "misionabas": "misionar",
    "misionad": "misionar",
    "misionada": "misionar",
    "misionadas": "misionar",
    "misionado": "misionar",
    "misionados": "misionar",
    "misionales": "misional",
    "misionamos": "misionar",
    "misionan": "misionar",
    "misionando": "misionar",
    "misionara": "misionar",
    "misionarais": "misionar",
    "misionaran": "misionar",
    "misionaras": "misionar",
    "misionare": "misionar",
    "misionareis": "misionar",
    "misionaremos": "misionar",
    "misionaren": "misionar",
    "misionares": "misionar",
    "misionaria": "misionario",
    "misionarias": "misionario",
    "misionarios": "misionario",
    "misionaron": "misionar",
    "misionará": "misionar",
    "misionarán": "misionar",
    "misionarás": "misionar",
    "misionaré": "misionar",
    "misionaréis": "misionar",
    "misionaría": "misionar",
    "misionaríais": "misionar",
    "misionaríamos": "misionar",
    "misionarían": "misionar",
    "misionarías": "misionar",
    "misionas": "misionar",
    "misionase": "misionar",
    "misionaseis": "misionar",
    "misionasen": "misionar",
    "misionases": "misionar",
    "misionaste": "misionar",
    "misionasteis": "misionar",
    "misione": "misionar",
    "misionemos": "misionar",
    "misionen": "misionar",
    "misionera": "misionero",
    "misioneras": "misionero",
    "misioneros": "misionero",
    "misiones": "misionar",
    "misiono": "misionar",
    "misionábamos": "misionar",
    "misionáis": "misionar",
    "misionáramos": "misionar",
    "misionáremos": "misionar",
    "misionásemos": "misionar",
    "misioné": "misionar",
    "misionéis": "misionar",
    "misionó": "misionar",
    "misiva": "misivo",
    "misivas": "misivo",
    "misivos": "misivo",
    "misma": "mismo",
    "mismas": "mismo",
    "mismidades": "mismidad",
    "mismos": "mismo",
    "misogamias": "misogamia",
    "misoginias": "misoginia",
    "misologías": "misología",
    "misoneísmos": "misoneísmo",
    "misoneístas": "misoneísta",
    "mispiqueles": "mispiquel",
    "misses": "miss",
    "mista": "mistar",
    "mistaba": "mistar",
    "mistabais": "mistar",
    "mistaban": "mistar",
    "mistabas": "mistar",
    "mistad": "mistar",
    "mistada": "mistar",
    "mistadas": "mistar",
    "mistado": "mistar",
    "mistados": "mistar",
    "mistagogos": "mistagogo",
    "mistagógica": "mistagógico",
    "mistagógicas": "mistagógico",
    "mistagógicos": "mistagógico",
    "mistamos": "mistar",
    "mistan": "mistar",
    "mistando": "mistar",
    "mistara": "mistar",
    "mistarais": "mistar",
    "mistaran": "mistar",
    "mistaras": "mistar",
    "mistare": "mistar",
    "mistareis": "mistar",
    "mistaremos": "mistar",
    "mistaren": "mistar",
    "mistares": "mistar",
    "mistaron": "mistar",
    "mistará": "mistar",
    "mistarán": "mistar",
    "mistarás": "mistar",
    "mistaré": "mistar",
    "mistaréis": "mistar",
    "mistaría": "mistar",
    "mistaríais": "mistar",
    "mistaríamos": "mistar",
    "mistarían": "mistar",
    "mistarías": "mistar",
    "mistas": "mistar",
    "mistase": "mistar",
    "mistaseis": "mistar",
    "mistasen": "mistar",
    "mistases": "mistar",
    "mistaste": "mistar",
    "mistasteis": "mistar",
    "miste": "mistar",
    "mistelas": "mistela",
    "mistemos": "mistar",
    "misten": "mistar",
    "misterios": "misterio",
    "misteriosa": "misterioso",
    "misteriosas": "misterioso",
    "misteriosos": "misterioso",
    "misters": "mister",
    "mistes": "mistar",
    "misticismos": "misticismo",
    "misticona": "misticón",
    "misticonas": "misticón",
    "misticones": "misticón",
    "mistifica": "mistificar",
    "mistificaba": "mistificar",
    "mistificabais": "mistificar",
    "mistificaban": "mistificar",
    "mistificabas": "mistificar",
    "mistificaciones": "mistificación",
    "mistificad": "mistificar",
    "mistificada": "mistificar",
    "mistificadas": "mistificar",
    "mistificado": "mistificar",
    "mistificadora": "mistificador",
    "mistificadoras": "mistificador",
    "mistificadores": "mistificador",
    "mistificados": "mistificar",
    "mistificamos": "mistificar",
    "mistifican": "mistificar",
    "mistificando": "mistificar",
    "mistificara": "mistificar",
    "mistificarais": "mistificar",
    "mistificaran": "mistificar",
    "mistificaras": "mistificar",
    "mistificare": "mistificar",
    "mistificareis": "mistificar",
    "mistificaremos": "mistificar",
    "mistificaren": "mistificar",
    "mistificares": "mistificar",
    "mistificaron": "mistificar",
    "mistificará": "mistificar",
    "mistificarán": "mistificar",
    "mistificarás": "mistificar",
    "mistificaré": "mistificar",
    "mistificaréis": "mistificar",
    "mistificaría": "mistificar",
    "mistificaríais": "mistificar",
    "mistificaríamos": "mistificar",
    "mistificarían": "mistificar",
    "mistificarías": "mistificar",
    "mistificas": "mistificar",
    "mistificase": "mistificar",
    "mistificaseis": "mistificar",
    "mistificasen": "mistificar",
    "mistificases": "mistificar",
    "mistificaste": "mistificar",
    "mistificasteis": "mistificar",
    "mistifico": "mistificar",
    "mistificábamos": "mistificar",
    "mistificáis": "mistificar",
    "mistificáramos": "mistificar",
    "mistificáremos": "mistificar",
    "mistificásemos": "mistificar",
    "mistificó": "mistificar",
    "mistifique": "mistificar",
    "mistifiquemos": "mistificar",
    "mistifiquen": "mistificar",
    "mistifiques": "mistificar",
    "mistifiqué": "mistificar",
    "mistifiquéis": "mistificar",
    "misto": "mistar",
    "mistos": "misto",
    "mistrales": "mistral",
    "mistábamos": "mistar",
    "mistáis": "mistar",
    "mistáramos": "mistar",
    "mistáremos": "mistar",
    "mistásemos": "mistar",
    "misté": "mistar",
    "mistéis": "mistar",
    "mistérica": "mistérico",
    "mistéricas": "mistérico",
    "mistéricos": "mistérico",
    "mistó": "mistar",
    "misántropa": "misántropo",
    "misántropas": "misántropo",
    "misántropos": "misántropo",
    "misérrima": "misérrimo",
    "misérrimas": "misérrimo",
    "misérrimos": "misérrimo",
    "misógamos": "misógamo",
    "misógina": "misógino",
    "misóginas": "misógino",
    "misóginos": "misógino",
    "mitadenca": "mitadenco",
    "mitadencas": "mitadenco",
    "mitadencos": "mitadenco",
    "mitades": "mitad",
    "mitas": "mita",
    "mitayos": "mitayo",
    "miticultora": "miticultor",
    "miticultoras": "miticultor",
    "miticultores": "miticultor",
    "miticulturas": "miticultura",
    "mitifica": "mitificar",
    "mitificaba": "mitificar",
    "mitificabais": "mitificar",
    "mitificaban": "mitificar",
    "mitificabas": "mitificar",
    "mitificaciones": "mitificación",
    "mitificad": "mitificar",
    "mitificada": "mitificar",
    "mitificadas": "mitificar",
    "mitificado": "mitificar",
    "mitificados": "mitificar",
    "mitificamos": "mitificar",
    "mitifican": "mitificar",
    "mitificando": "mitificar",
    "mitificara": "mitificar",
    "mitificarais": "mitificar",
    "mitificaran": "mitificar",
    "mitificaras": "mitificar",
    "mitificare": "mitificar",
    "mitificareis": "mitificar",
    "mitificaremos": "mitificar",
    "mitificaren": "mitificar",
    "mitificares": "mitificar",
    "mitificaron": "mitificar",
    "mitificará": "mitificar",
    "mitificarán": "mitificar",
    "mitificarás": "mitificar",
    "mitificaré": "mitificar",
    "mitificaréis": "mitificar",
    "mitificaría": "mitificar",
    "mitificaríais": "mitificar",
    "mitificaríamos": "mitificar",
    "mitificarían": "mitificar",
    "mitificarías": "mitificar",
    "mitificas": "mitificar",
    "mitificase": "mitificar",
    "mitificaseis": "mitificar",
    "mitificasen": "mitificar",
    "mitificases": "mitificar",
    "mitificaste": "mitificar",
    "mitificasteis": "mitificar",
    "mitifico": "mitificar",
    "mitificábamos": "mitificar",
    "mitificáis": "mitificar",
    "mitificáramos": "mitificar",
    "mitificáremos": "mitificar",
    "mitificásemos": "mitificar",
    "mitificó": "mitificar",
    "mitifique": "mitificar",
    "mitifiquemos": "mitificar",
    "mitifiquen": "mitificar",
    "mitifiques": "mitificar",
    "mitifiqué": "mitificar",
    "mitifiquéis": "mitificar",
    "mitiga": "mitigar",
    "mitigaba": "mitigar",
    "mitigabais": "mitigar",
    "mitigaban": "mitigar",
    "mitigabas": "mitigar",
    "mitigaciones": "mitigación",
    "mitigad": "mitigar",
    "mitigada": "mitigar",
    "mitigadas": "mitigar",
    "mitigado": "mitigar",
    "mitigadora": "mitigador",
    "mitigadoras": "mitigador",
    "mitigadores": "mitigador",
    "mitigados": "mitigar",
    "mitigamos": "mitigar",
    "mitigan": "mitigar",
    "mitigando": "mitigar",
    "mitigara": "mitigar",
    "mitigarais": "mitigar",
    "mitigaran": "mitigar",
    "mitigaras": "mitigar",
    "mitigare": "mitigar",
    "mitigareis": "mitigar",
    "mitigaremos": "mitigar",
    "mitigaren": "mitigar",
    "mitigares": "mitigar",
    "mitigaron": "mitigar",
    "mitigará": "mitigar",
    "mitigarán": "mitigar",
    "mitigarás": "mitigar",
    "mitigaré": "mitigar",
    "mitigaréis": "mitigar",
    "mitigaría": "mitigar",
    "mitigaríais": "mitigar",
    "mitigaríamos": "mitigar",
    "mitigarían": "mitigar",
    "mitigarías": "mitigar",
    "mitigas": "mitigar",
    "mitigase": "mitigar",
    "mitigaseis": "mitigar",
    "mitigasen": "mitigar",
    "mitigases": "mitigar",
    "mitigaste": "mitigar",
    "mitigasteis": "mitigar",
    "mitigativa": "mitigativo",
    "mitigativas": "mitigativo",
    "mitigativos": "mitigativo",
    "mitigo": "mitigar",
    "mitigue": "mitigar",
    "mitiguemos": "mitigar",
    "mitiguen": "mitigar",
    "mitigues": "mitigar",
    "mitigué": "mitigar",
    "mitiguéis": "mitigar",
    "mitigábamos": "mitigar",
    "mitigáis": "mitigar",
    "mitigáramos": "mitigar",
    "mitigáremos": "mitigar",
    "mitigásemos": "mitigar",
    "mitigó": "mitigar",
    "mitiliculturas": "mitilicultura",
    "mitilotoxinas": "mitilotoxina",
    "mitjapuntas": "mitjapunta",
    "mitocondriales": "mitocondrial",
    "mitocondrias": "mitocondria",
    "mitologistas": "mitologista",
    "mitologías": "mitología",
    "mitológica": "mitológico",
    "mitológicas": "mitológico",
    "mitológicos": "mitológico",
    "mitomanías": "mitomanía",
    "mitones": "mitón",
    "mitorribosomas": "mitorribosoma",
    "mitos": "mito",
    "mitosomas": "mitosoma",
    "mitotes": "mitote",
    "mitrada": "mitrado",
    "mitradas": "mitrado",
    "mitrados": "mitrado",
    "mitrales": "mitral",
    "mitras": "mitra",
    "mitridatismos": "mitridatismo",
    "mitridatos": "mitridato",
    "mitterandismos": "mitterandismo",
    "mitógenos": "mitógeno",
    "mitógrafa": "mitógrafo",
    "mitógrafas": "mitógrafo",
    "mitógrafos": "mitógrafo",
    "mitóloga": "mitólogo",
    "mitólogas": "mitólogo",
    "mitólogos": "mitólogo",
    "mitómana": "mitómano",
    "mitómanas": "mitómano",
    "mitómanos": "mitómano",
    "mitótica": "mitótico",
    "mitóticas": "mitótico",
    "mitóticos": "mitótico",
    "mixedemas": "mixedema",
    "mixinos": "mixino",
    "mixocromosomas": "mixocromosoma",
    "mixomas": "mixoma",
    "mixta": "mixto",
    "mixtas": "mixto",
    "mixteca": "mixteco",
    "mixtecas": "mixteco",
    "mixtecos": "mixteco",
    "mixtificaciones": "mixtificación",
    "mixtilínea": "mixtilíneo",
    "mixtilíneas": "mixtilíneo",
    "mixtilíneos": "mixtilíneo",
    "mixtiones": "mixtión",
    "mixtos": "mixto",
    "mixtura": "mixturar",
    "mixturaba": "mixturar",
    "mixturabais": "mixturar",
    "mixturaban": "mixturar",
    "mixturabas": "mixturar",
    "mixturad": "mixturar",
    "mixturada": "mixturar",
    "mixturadas": "mixturar",
    "mixturado": "mixturar",
    "mixturados": "mixturar",
    "mixturamos": "mixturar",
    "mixturan": "mixturar",
    "mixturando": "mixturar",
    "mixturara": "mixturar",
    "mixturarais": "mixturar",
    "mixturaran": "mixturar",
    "mixturaras": "mixturar",
    "mixturare": "mixturar",
    "mixturareis": "mixturar",
    "mixturaremos": "mixturar",
    "mixturaren": "mixturar",
    "mixturares": "mixturar",
    "mixturaron": "mixturar",
    "mixturará": "mixturar",
    "mixturarán": "mixturar",
    "mixturarás": "mixturar",
    "mixturaré": "mixturar",
    "mixturaréis": "mixturar",
    "mixturaría": "mixturar",
    "mixturaríais": "mixturar",
    "mixturaríamos": "mixturar",
    "mixturarían": "mixturar",
    "mixturarías": "mixturar",
    "mixturas": "mixturar",
    "mixturase": "mixturar",
    "mixturaseis": "mixturar",
    "mixturasen": "mixturar",
    "mixturases": "mixturar",
    "mixturaste": "mixturar",
    "mixturasteis": "mixturar",
    "mixture": "mixturar",
    "mixturemos": "mixturar",
    "mixturen": "mixturar",
    "mixturera": "mixturero",
    "mixtureras": "mixturero",
    "mixtureros": "mixturero",
    "mixtures": "mixturar",
    "mixturo": "mixturar",
    "mixturábamos": "mixturar",
    "mixturáis": "mixturar",
    "mixturáramos": "mixturar",
    "mixturáremos": "mixturar",
    "mixturásemos": "mixturar",
    "mixturé": "mixturar",
    "mixturéis": "mixturar",
    "mixturó": "mixturar",
    "miza": "mizo",
    "mizas": "mizo",
    "mizos": "mizo",
    "miábamos": "miar",
    "miáis": "miar",
    "miáramos": "miar",
    "miáremos": "miar",
    "miásemos": "miar",
    "mié": "miar",
    "miéis": "miar",
    "miñonas": "miñona",
    "mió": "miar",
    "miógrafos": "miógrafo",
    "miópica": "miópico",
    "miópicas": "miópico",
    "miópicos": "miópico",
    "miótica": "miótico",
    "mióticas": "miótico",
    "mióticos": "miótico",
    "ml": "mililitro",
    "mm": "milímetro",
    "mnemotecnias": "mnemotecnia",
    "mnemotécnica": "mnemotécnico",
    "mnemotécnicas": "mnemotécnico",
    "mnemotécnicos": "mnemotécnico",
    "mnemónica": "mnemónico",
    "mnemónicas": "mnemónico",
    "mnemónicos": "mnemónico",
    "moabitas": "moabita",
    "moarés": "moaré",
    "moaxajas": "moaxaja",
    "mobiliaria": "mobiliario",
    "mobiliarias": "mobiliario",
    "mobiliarios": "mobiliario",
    "moblaba": "moblar",
    "moblabais": "moblar",
    "moblaban": "moblar",
    "moblabas": "moblar",
    "moblad": "moblar",
    "moblada": "moblar",
    "mobladas": "moblar",
    "moblado": "moblar",
    "moblados": "moblar",
    "moblajes": "moblaje",
    "moblamos": "moblar",
    "moblando": "moblar",
    "moblara": "moblar",
    "moblarais": "moblar",
    "moblaran": "moblar",
    "moblaras": "moblar",
    "moblare": "moblar",
    "moblareis": "moblar",
    "moblaremos": "moblar",
    "moblaren": "moblar",
    "moblares": "moblar",
    "moblaron": "moblar",
    "moblará": "moblar",
    "moblarán": "moblar",
    "moblarás": "moblar",
    "moblaré": "moblar",
    "moblaréis": "moblar",
    "moblaría": "moblar",
    "moblaríais": "moblar",
    "moblaríamos": "moblar",
    "moblarían": "moblar",
    "moblarías": "moblar",
    "moblase": "moblar",
    "moblaseis": "moblar",
    "moblasen": "moblar",
    "moblases": "moblar",
    "moblaste": "moblar",
    "moblasteis": "moblar",
    "moblemos": "moblar",
    "moblábamos": "moblar",
    "mobláis": "moblar",
    "mobláramos": "moblar",
    "mobláremos": "moblar",
    "moblásemos": "moblar",
    "moblé": "moblar",
    "mobléis": "moblar",
    "mobló": "moblar",
    "moca": "mocar",
    "mocaba": "mocar",
    "mocabais": "mocar",
    "mocaban": "mocar",
    "mocabas": "mocar",
    "mocad": "mocar",
    "mocada": "mocar",
    "mocadas": "mocar",
    "mocado": "mocar",
    "mocadores": "mocador",
    "mocados": "mocar",
    "mocamos": "mocar",
    "mocan": "mocar",
    "mocando": "mocar",
    "mocara": "mocar",
    "mocarais": "mocar",
    "mocaran": "mocar",
    "mocaras": "mocar",
    "mocare": "mocar",
    "mocareis": "mocar",
    "mocaremos": "mocar",
    "mocaren": "mocar",
    "mocares": "mocar",
    "mocaron": "mocar",
    "mocarros": "mocarro",
    "mocará": "mocar",
    "mocarán": "mocar",
    "mocarás": "mocar",
    "mocaré": "mocar",
    "mocaréis": "mocar",
    "mocaría": "mocar",
    "mocaríais": "mocar",
    "mocaríamos": "mocar",
    "mocarían": "mocar",
    "mocarías": "mocar",
    "mocas": "mocar",
    "mocase": "mocar",
    "mocaseis": "mocar",
    "mocasen": "mocar",
    "mocases": "mocar",
    "mocasines": "mocasín",
    "mocaste": "mocar",
    "mocasteis": "mocar",
    "mocea": "mocear",
    "moceaba": "mocear",
    "moceabais": "mocear",
    "moceaban": "mocear",
    "moceabas": "mocear",
    "mocead": "mocear",
    "moceada": "mocear",
    "moceadas": "mocear",
    "moceado": "mocear",
    "moceados": "mocear",
    "moceamos": "mocear",
    "mocean": "mocear",
    "moceando": "mocear",
    "moceara": "mocear",
    "mocearais": "mocear",
    "mocearan": "mocear",
    "mocearas": "mocear",
    "moceare": "mocear",
    "moceareis": "mocear",
    "mocearemos": "mocear",
    "mocearen": "mocear",
    "moceares": "mocear",
    "mocearon": "mocear",
    "moceará": "mocear",
    "mocearán": "mocear",
    "mocearás": "mocear",
    "mocearé": "mocear",
    "mocearéis": "mocear",
    "mocearía": "mocear",
    "mocearíais": "mocear",
    "mocearíamos": "mocear",
    "mocearían": "mocear",
    "mocearías": "mocear",
    "moceas": "mocear",
    "mocease": "mocear",
    "moceaseis": "mocear",
    "moceasen": "mocear",
    "moceases": "mocear",
    "moceaste": "mocear",
    "moceasteis": "mocear",
    "mocedades": "mocedad",
    "mocee": "mocear",
    "moceemos": "mocear",
    "moceen": "mocear",
    "mocees": "mocear",
    "moceo": "mocear",
    "moceriles": "moceril",
    "moceríos": "mocerío",
    "mocetona": "mocetón",
    "mocetonas": "mocetón",
    "mocetones": "mocetón",
    "moceábamos": "mocear",
    "moceáis": "mocear",
    "moceáramos": "mocear",
    "moceáremos": "mocear",
    "moceásemos": "mocear",
    "moceé": "mocear",
    "moceéis": "mocear",
    "moceó": "mocear",
    "mocha": "mocho",
    "mochaba": "mochar",
    "mochabais": "mochar",
    "mochaban": "mochar",
    "mochabas": "mochar",
    "mochad": "mochar",
    "mochada": "mochar",
    "mochadas": "mochar",
    "mochado": "mochar",
    "mochados": "mochar",
    "mochamos": "mochar",
    "mochan": "mochar",
    "mochando": "mochar",
    "mochara": "mochar",
    "mocharais": "mochar",
    "mocharan": "mochar",
    "mocharas": "mochar",
    "mochare": "mochar",
    "mochareis": "mochar",
    "mocharemos": "mochar",
    "mocharen": "mochar",
    "mochares": "mochar",
    "mocharon": "mochar",
    "mochará": "mochar",
    "mocharán": "mochar",
    "mocharás": "mochar",
    "mocharé": "mochar",
    "mocharéis": "mochar",
    "mocharía": "mochar",
    "mocharíais": "mochar",
    "mocharíamos": "mochar",
    "mocharían": "mochar",
    "mocharías": "mochar",
    "mochas": "mocho",
    "mochase": "mochar",
    "mochaseis": "mochar",
    "mochasen": "mochar",
    "mochases": "mochar",
    "mochaste": "mochar",
    "mochasteis": "mochar",
    "mochazos": "mochazo",
    "moche": "mochar",
    "mochemos": "mochar",
    "mochen": "mochar",
    "moches": "mochar",
    "mochetas": "mocheta",
    "mochicas": "mochica",
    "mochilas": "mochila",
    "mochilera": "mochilero",
    "mochileras": "mochilero",
    "mochileros": "mochilero",
    "mochiles": "mochil",
    "mochines": "mochín",
    "mocho": "mochar",
    "mochos": "mocho",
    "mochuelos": "mochuelo",
    "mochábamos": "mochar",
    "mocháis": "mochar",
    "mocháramos": "mochar",
    "mocháremos": "mochar",
    "mochásemos": "mochar",
    "moché": "mochar",
    "mochéis": "mochar",
    "mochó": "mochar",
    "mociones": "moción",
    "mocita": "mocito",
    "mocitas": "mocito",
    "mocitos": "mocito",
    "moco": "mocar",
    "mocos": "moco",
    "mocosa": "mocoso",
    "mocosas": "mocoso",
    "mocosos": "mocoso",
    "mocábamos": "mocar",
    "mocáis": "mocar",
    "mocárabes": "mocárabe",
    "mocáramos": "mocar",
    "mocáremos": "mocar",
    "mocásemos": "mocar",
    "mocó": "mocar",
    "modacrílica": "modacrílico",
    "modacrílicas": "modacrílico",
    "modacrílicos": "modacrílico",
    "modales": "modal",
    "modalidades": "modalidad",
    "modalizaciones": "modalización",
    "modalizadores": "modalizador",
    "modas": "moda",
    "modela": "modelar",
    "modelaba": "modelar",
    "modelabais": "modelar",
    "modelaban": "modelar",
    "modelabas": "modelar",
    "modelaciones": "modelación",
    "modelad": "modelar",
    "modelada": "modelar",
    "modeladas": "modelar",
    "modelado": "modelar",
    "modeladora": "modelador",
    "modeladoras": "modelador",
    "modeladores": "modelador",
    "modelados": "modelar",
    "modelamos": "modelar",
    "modelan": "modelar",
    "modelando": "modelar",
    "modelara": "modelar",
    "modelarais": "modelar",
    "modelaran": "modelar",
    "modelaras": "modelar",
    "modelare": "modelar",
    "modelareis": "modelar",
    "modelaremos": "modelar",
    "modelaren": "modelar",
    "modelares": "modelar",
    "modelaron": "modelar",
    "modelará": "modelar",
    "modelarán": "modelar",
    "modelarás": "modelar",
    "modelaré": "modelar",
    "modelaréis": "modelar",
    "modelaría": "modelar",
    "modelaríais": "modelar",
    "modelaríamos": "modelar",
    "modelarían": "modelar",
    "modelarías": "modelar",
    "modelas": "modelar",
    "modelase": "modelar",
    "modelaseis": "modelar",
    "modelasen": "modelar",
    "modelases": "modelar",
    "modelaste": "modelar",
    "modelasteis": "modelar",
    "modele": "modelar",
    "modelemos": "modelar",
    "modelen": "modelar",
    "modeles": "modelar",
    "modelice": "modelizar",
    "modelicemos": "modelizar",
    "modelicen": "modelizar",
    "modelices": "modelizar",
    "modelicé": "modelizar",
    "modelicéis": "modelizar",
    "modelistas": "modelista",
    "modeliza": "modelizar",
    "modelizaba": "modelizar",
    "modelizabais": "modelizar",
    "modelizaban": "modelizar",
    "modelizabas": "modelizar",
    "modelizaciones": "modelización",
    "modelizad": "modelizar",
    "modelizada": "modelizar",
    "modelizadas": "modelizar",
    "modelizado": "modelizar",
    "modelizados": "modelizar",
    "modelizamos": "modelizar",
    "modelizan": "modelizar",
    "modelizando": "modelizar",
    "modelizara": "modelizar",
    "modelizarais": "modelizar",
    "modelizaran": "modelizar",
    "modelizaras": "modelizar",
    "modelizare": "modelizar",
    "modelizareis": "modelizar",
    "modelizaremos": "modelizar",
    "modelizaren": "modelizar",
    "modelizares": "modelizar",
    "modelizaron": "modelizar",
    "modelizará": "modelizar",
    "modelizarán": "modelizar",
    "modelizarás": "modelizar",
    "modelizaré": "modelizar",
    "modelizaréis": "modelizar",
    "modelizaría": "modelizar",
    "modelizaríais": "modelizar",
    "modelizaríamos": "modelizar",
    "modelizarían": "modelizar",
    "modelizarías": "modelizar",
    "modelizas": "modelizar",
    "modelizase": "modelizar",
    "modelizaseis": "modelizar",
    "modelizasen": "modelizar",
    "modelizases": "modelizar",
    "modelizaste": "modelizar",
    "modelizasteis": "modelizar",
    "modelizo": "modelizar",
    "modelizábamos": "modelizar",
    "modelizáis": "modelizar",
    "modelizáramos": "modelizar",
    "modelizáremos": "modelizar",
    "modelizásemos": "modelizar",
    "modelizó": "modelizar",
    "modelo": "modelar",
    "modelos": "modelo",
    "modelábamos": "modelar",
    "modeláis": "modelar",
    "modeláramos": "modelar",
    "modeláremos": "modelar",
    "modelásemos": "modelar",
    "modelé": "modelar",
    "modeléis": "modelar",
    "modeló": "modelar",
    "modemes": "modem",
    "modems": "modem",
    "modera": "moderar",
    "moderaba": "moderar",
    "moderabais": "moderar",
    "moderaban": "moderar",
    "moderabas": "moderar",
    "moderaciones": "moderación",
    "moderad": "moderar",
    "moderada": "moderar",
    "moderadas": "moderar",
    "moderado": "moderar",
    "moderadora": "moderador",
    "moderadoras": "moderador",
    "moderadores": "moderador",
    "moderados": "moderar",
    "moderamos": "moderar",
    "moderan": "moderar",
    "moderando": "moderar",
    "moderantismos": "moderantismo",
    "moderara": "moderar",
    "moderarais": "moderar",
    "moderaran": "moderar",
    "moderaras": "moderar",
    "moderare": "moderar",
    "moderareis": "moderar",
    "moderaremos": "moderar",
    "moderaren": "moderar",
    "moderares": "moderar",
    "moderaron": "moderar",
    "moderará": "moderar",
    "moderarán": "moderar",
    "moderarás": "moderar",
    "moderaré": "moderar",
    "moderaréis": "moderar",
    "moderaría": "moderar",
    "moderaríais": "moderar",
    "moderaríamos": "moderar",
    "moderarían": "moderar",
    "moderarías": "moderar",
    "moderas": "moderar",
    "moderase": "moderar",
    "moderaseis": "moderar",
    "moderasen": "moderar",
    "moderases": "moderar",
    "moderaste": "moderar",
    "moderasteis": "moderar",
    "moderativa": "moderativo",
    "moderativas": "moderativo",
    "moderativos": "moderativo",
    "moderatoria": "moderatorio",
    "moderatorias": "moderatorio",
    "moderatorios": "moderatorio",
    "moderatos": "moderato",
    "modere": "moderar",
    "moderemos": "moderar",
    "moderen": "moderar",
    "moderes": "moderar",
    "moderna": "moderno",
    "modernas": "moderno",
    "modernice": "modernizar",
    "modernicemos": "modernizar",
    "modernicen": "modernizar",
    "modernices": "modernizar",
    "modernicé": "modernizar",
    "modernicéis": "modernizar",
    "modernidades": "modernidad",
    "modernismos": "modernismo",
    "modernistas": "modernista",
    "moderniza": "modernizar",
    "modernizaba": "modernizar",
    "modernizabais": "modernizar",
    "modernizaban": "modernizar",
    "modernizabas": "modernizar",
    "modernizaciones": "modernización",
    "modernizad": "modernizar",
    "modernizada": "modernizar",
    "modernizadas": "modernizar",
    "modernizado": "modernizar",
    "modernizadora": "modernizador",
    "modernizadoras": "modernizador",
    "modernizadores": "modernizador",
    "modernizados": "modernizar",
    "modernizamos": "modernizar",
    "modernizan": "modernizar",
    "modernizando": "modernizar",
    "modernizara": "modernizar",
    "modernizarais": "modernizar",
    "modernizaran": "modernizar",
    "modernizaras": "modernizar",
    "modernizare": "modernizar",
    "modernizareis": "modernizar",
    "modernizaremos": "modernizar",
    "modernizaren": "modernizar",
    "modernizares": "modernizar",
    "modernizaron": "modernizar",
    "modernizará": "modernizar",
    "modernizarán": "modernizar",
    "modernizarás": "modernizar",
    "modernizaré": "modernizar",
    "modernizaréis": "modernizar",
    "modernizaría": "modernizar",
    "modernizaríais": "modernizar",
    "modernizaríamos": "modernizar",
    "modernizarían": "modernizar",
    "modernizarías": "modernizar",
    "modernizas": "modernizar",
    "modernizase": "modernizar",
    "modernizaseis": "modernizar",
    "modernizasen": "modernizar",
    "modernizases": "modernizar",
    "modernizaste": "modernizar",
    "modernizasteis": "modernizar",
    "modernizo": "modernizar",
    "modernizábamos": "modernizar",
    "modernizáis": "modernizar",
    "modernizáramos": "modernizar",
    "modernizáremos": "modernizar",
    "modernizásemos": "modernizar",
    "modernizó": "modernizar",
    "modernos": "moderno",
    "modero": "moderar",
    "moderábamos": "moderar",
    "moderáis": "moderar",
    "moderáramos": "moderar",
    "moderáremos": "moderar",
    "moderásemos": "moderar",
    "moderé": "moderar",
    "moderéis": "moderar",
    "moderó": "moderar",
    "modesta": "modesto",
    "modestas": "modesto",
    "modestias": "modestia",
    "modestos": "modesto",
    "modicidades": "modicidad",
    "modifica": "modificar",
    "modificaba": "modificar",
    "modificabais": "modificar",
    "modificaban": "modificar",
    "modificabas": "modificar",
    "modificables": "modificable",
    "modificaciones": "modificación",
    "modificad": "modificar",
    "modificada": "modificar",
    "modificadas": "modificar",
    "modificado": "modificar",
    "modificadora": "modificador",
    "modificadoras": "modificador",
    "modificadores": "modificador",
    "modificados": "modificar",
    "modificamos": "modificar",
    "modifican": "modificar",
    "modificando": "modificar",
    "modificantes": "modificante",
    "modificara": "modificar",
    "modificarais": "modificar",
    "modificaran": "modificar",
    "modificaras": "modificar",
    "modificare": "modificar",
    "modificareis": "modificar",
    "modificaremos": "modificar",
    "modificaren": "modificar",
    "modificares": "modificar",
    "modificaron": "modificar",
    "modificará": "modificar",
    "modificarán": "modificar",
    "modificarás": "modificar",
    "modificaré": "modificar",
    "modificaréis": "modificar",
    "modificaría": "modificar",
    "modificaríais": "modificar",
    "modificaríamos": "modificar",
    "modificarían": "modificar",
    "modificarías": "modificar",
    "modificas": "modificar",
    "modificase": "modificar",
    "modificaseis": "modificar",
    "modificasen": "modificar",
    "modificases": "modificar",
    "modificaste": "modificar",
    "modificasteis": "modificar",
    "modificativa": "modificativo",
    "modificativas": "modificativo",
    "modificativos": "modificativo",
    "modificatoria": "modificatorio",
    "modificatorias": "modificatorio",
    "modificatorios": "modificatorio",
    "modifico": "modificar",
    "modificábamos": "modificar",
    "modificáis": "modificar",
    "modificáramos": "modificar",
    "modificáremos": "modificar",
    "modificásemos": "modificar",
    "modificó": "modificar",
    "modifique": "modificar",
    "modifiquemos": "modificar",
    "modifiquen": "modificar",
    "modifiques": "modificar",
    "modifiqué": "modificar",
    "modifiquéis": "modificar",
    "modillones": "modillón",
    "modiolos": "modiolo",
    "modios": "modio",
    "modismos": "modismo",
    "modistas": "modista",
    "modistillas": "modistilla",
    "modistos": "modisto",
    "modorra": "modorro",
    "modorraba": "modorrar",
    "modorrabais": "modorrar",
    "modorraban": "modorrar",
    "modorrabas": "modorrar",
    "modorrad": "modorrar",
    "modorrada": "modorrar",
    "modorradas": "modorrar",
    "modorrado": "modorrar",
    "modorrados": "modorrar",
    "modorramos": "modorrar",
    "modorran": "modorrar",
    "modorrando": "modorrar",
    "modorrara": "modorrar",
    "modorrarais": "modorrar",
    "modorraran": "modorrar",
    "modorraras": "modorrar",
    "modorrare": "modorrar",
    "modorrareis": "modorrar",
    "modorraremos": "modorrar",
    "modorraren": "modorrar",
    "modorrares": "modorrar",
    "modorraron": "modorrar",
    "modorrará": "modorrar",
    "modorrarán": "modorrar",
    "modorrarás": "modorrar",
    "modorraré": "modorrar",
    "modorraréis": "modorrar",
    "modorraría": "modorrar",
    "modorraríais": "modorrar",
    "modorraríamos": "modorrar",
    "modorrarían": "modorrar",
    "modorrarías": "modorrar",
    "modorras": "modorro",
    "modorrase": "modorrar",
    "modorraseis": "modorrar",
    "modorrasen": "modorrar",
    "modorrases": "modorrar",
    "modorraste": "modorrar",
    "modorrasteis": "modorrar",
    "modorre": "modorrar",
    "modorremos": "modorrar",
    "modorren": "modorrar",
    "modorres": "modorrar",
    "modorrillas": "modorrilla",
    "modorro": "modorrar",
    "modorros": "modorro",
    "modorrábamos": "modorrar",
    "modorráis": "modorrar",
    "modorráramos": "modorrar",
    "modorráremos": "modorrar",
    "modorrásemos": "modorrar",
    "modorré": "modorrar",
    "modorréis": "modorrar",
    "modorró": "modorrar",
    "modos": "modo",
    "modosa": "modoso",
    "modosas": "modoso",
    "modosidades": "modosidad",
    "modosos": "modoso",
    "modregos": "modrego",
    "modula": "modular",
    "modulaba": "modular",
    "modulabais": "modular",
    "modulaban": "modular",
    "modulabas": "modular",
    "modulables": "modulable",
    "modulaciones": "modulación",
    "modulad": "modular",
    "modulada": "modular",
    "moduladas": "modular",
    "modulado": "modular",
    "moduladora": "modulador",
    "moduladoras": "modulador",
    "moduladores": "modulador",
    "modulados": "modular",
    "modulamos": "modular",
    "modulan": "modular",
    "modulando": "modular",
    "modulantes": "modulante",
    "modulara": "modular",
    "modularais": "modular",
    "modularan": "modular",
    "modularas": "modular",
    "modulare": "modular",
    "modulareis": "modular",
    "modularemos": "modular",
    "modularen": "modular",
    "modulares": "modular",
    "modularidades": "modularidad",
    "modularizaciones": "modularización",
    "modularon": "modular",
    "modulará": "modular",
    "modularán": "modular",
    "modularás": "modular",
    "modularé": "modular",
    "modularéis": "modular",
    "modularía": "modular",
    "modularíais": "modular",
    "modularíamos": "modular",
    "modularían": "modular",
    "modularías": "modular",
    "modulas": "modular",
    "modulase": "modular",
    "modulaseis": "modular",
    "modulasen": "modular",
    "modulases": "modular",
    "modulaste": "modular",
    "modulasteis": "modular",
    "module": "modular",
    "modulemos": "modular",
    "modulen": "modular",
    "modules": "modular",
    "modulo": "modular",
    "modulábamos": "modular",
    "moduláis": "modular",
    "moduláramos": "modular",
    "moduláremos": "modular",
    "modulásemos": "modular",
    "modulé": "modular",
    "moduléis": "modular",
    "moduló": "modular",
    "modulómetros": "modulómetro",
    "modélica": "modélico",
    "modélicas": "modélico",
    "modélicos": "modélico",
    "moerakis": "moeraki",
    "moers": "moer",
    "mofa": "mofar",
    "mofaba": "mofar",
    "mofabais": "mofar",
    "mofaban": "mofar",
    "mofabas": "mofar",
    "mofad": "mofar",
    "mofada": "mofar",
    "mofadas": "mofar",
    "mofado": "mofar",
    "mofadora": "mofador",
    "mofadoras": "mofador",
    "mofadores": "mofador",
    "mofados": "mofar",
    "mofaduras": "mofadura",
    "mofamos": "mofar",
    "mofan": "mofar",
    "mofando": "mofar",
    "mofara": "mofar",
    "mofarais": "mofar",
    "mofaran": "mofar",
    "mofaras": "mofar",
    "mofare": "mofar",
    "mofareis": "mofar",
    "mofaremos": "mofar",
    "mofaren": "mofar",
    "mofares": "mofar",
    "mofaron": "mofar",
    "mofará": "mofar",
    "mofarán": "mofar",
    "mofarás": "mofar",
    "mofaré": "mofar",
    "mofaréis": "mofar",
    "mofaría": "mofar",
    "mofaríais": "mofar",
    "mofaríamos": "mofar",
    "mofarían": "mofar",
    "mofarías": "mofar",
    "mofas": "mofar",
    "mofase": "mofar",
    "mofaseis": "mofar",
    "mofasen": "mofar",
    "mofases": "mofar",
    "mofaste": "mofar",
    "mofasteis": "mofar",
    "mofe": "mofar",
    "mofemos": "mofar",
    "mofen": "mofar",
    "mofes": "mofar",
    "mofetas": "mofeta",
    "mofletes": "moflete",
    "mofletuda": "mofletudo",
    "mofletudas": "mofletudo",
    "mofletudos": "mofletudo",
    "mofo": "mofar",
    "mofábamos": "mofar",
    "mofáis": "mofar",
    "mofáramos": "mofar",
    "mofáremos": "mofar",
    "mofásemos": "mofar",
    "mofé": "mofar",
    "moféis": "mofar",
    "mofó": "mofar",
    "mogataces": "mogataz",
    "mogates": "mogate",
    "mogoles": "mogol",
    "mogollona": "mogollón",
    "mogollonas": "mogollón",
    "mogollones": "mogollón",
    "mogona": "mogón",
    "mogonas": "mogón",
    "mogones": "mogón",
    "mogotes": "mogote",
    "moguexes": "moguex",
    "mogólica": "mogólico",
    "mogólicas": "mogólico",
    "mogólicos": "mogólico",
    "mohaires": "mohair",
    "mohairs": "mohair",
    "mohametana": "mohametano",
    "mohametanas": "mohametano",
    "mohametanos": "mohametano",
    "moharraches": "moharrache",
    "moharrachos": "moharracho",
    "moharras": "moharra",
    "mohatra": "mohatrar",
    "mohatraba": "mohatrar",
    "mohatrabais": "mohatrar",
    "mohatraban": "mohatrar",
    "mohatrabas": "mohatrar",
    "mohatrad": "mohatrar",
    "mohatrada": "mohatrar",
    "mohatradas": "mohatrar",
    "mohatrado": "mohatrar",
    "mohatrados": "mohatrar",
    "mohatramos": "mohatrar",
    "mohatran": "mohatrar",
    "mohatrando": "mohatrar",
    "mohatrara": "mohatrar",
    "mohatrarais": "mohatrar",
    "mohatraran": "mohatrar",
    "mohatraras": "mohatrar",
    "mohatrare": "mohatrar",
    "mohatrareis": "mohatrar",
    "mohatraremos": "mohatrar",
    "mohatraren": "mohatrar",
    "mohatrares": "mohatrar",
    "mohatraron": "mohatrar",
    "mohatrará": "mohatrar",
    "mohatrarán": "mohatrar",
    "mohatrarás": "mohatrar",
    "mohatraré": "mohatrar",
    "mohatraréis": "mohatrar",
    "mohatraría": "mohatrar",
    "mohatraríais": "mohatrar",
    "mohatraríamos": "mohatrar",
    "mohatrarían": "mohatrar",
    "mohatrarías": "mohatrar",
    "mohatras": "mohatrar",
    "mohatrase": "mohatrar",
    "mohatraseis": "mohatrar",
    "mohatrasen": "mohatrar",
    "mohatrases": "mohatrar",
    "mohatraste": "mohatrar",
    "mohatrasteis": "mohatrar",
    "mohatre": "mohatrar",
    "mohatremos": "mohatrar",
    "mohatren": "mohatrar",
    "mohatrera": "mohatrero",
    "mohatreras": "mohatrero",
    "mohatreros": "mohatrero",
    "mohatres": "mohatrar",
    "mohatro": "mohatrar",
    "mohatrábamos": "mohatrar",
    "mohatráis": "mohatrar",
    "mohatráramos": "mohatrar",
    "mohatráremos": "mohatrar",
    "mohatrásemos": "mohatrar",
    "mohatré": "mohatrar",
    "mohatréis": "mohatrar",
    "mohatró": "mohatrar",
    "mohece": "mohecer",
    "moheced": "mohecer",
    "mohecemos": "mohecer",
    "mohecen": "mohecer",
    "moheceremos": "mohecer",
    "mohecerá": "mohecer",
    "mohecerán": "mohecer",
    "mohecerás": "mohecer",
    "moheceré": "mohecer",
    "moheceréis": "mohecer",
    "mohecería": "mohecer",
    "moheceríais": "mohecer",
    "moheceríamos": "mohecer",
    "mohecerían": "mohecer",
    "mohecerías": "mohecer",
    "moheces": "mohecer",
    "mohecida": "mohecer",
    "mohecidas": "mohecer",
    "mohecido": "mohecer",
    "mohecidos": "mohecer",
    "moheciendo": "mohecer",
    "moheciera": "mohecer",
    "mohecierais": "mohecer",
    "mohecieran": "mohecer",
    "mohecieras": "mohecer",
    "moheciere": "mohecer",
    "moheciereis": "mohecer",
    "mohecieren": "mohecer",
    "mohecieres": "mohecer",
    "mohecieron": "mohecer",
    "moheciese": "mohecer",
    "mohecieseis": "mohecer",
    "moheciesen": "mohecer",
    "mohecieses": "mohecer",
    "mohecimos": "mohecer",
    "moheciste": "mohecer",
    "mohecisteis": "mohecer",
    "moheciéramos": "mohecer",
    "moheciéremos": "mohecer",
    "moheciésemos": "mohecer",
    "moheció": "mohecer",
    "mohecéis": "mohecer",
    "mohecí": "mohecer",
    "mohecía": "mohecer",
    "mohecíais": "mohecer",
    "mohecíamos": "mohecer",
    "mohecían": "mohecer",
    "mohecías": "mohecer",
    "mohedales": "mohedal",
    "mohedas": "moheda",
    "moheres": "moher",
    "mohezca": "mohecer",
    "mohezcamos": "mohecer",
    "mohezcan": "mohecer",
    "mohezcas": "mohecer",
    "mohezco": "mohecer",
    "mohezcáis": "mohecer",
    "mohicana": "mohicano",
    "mohicanas": "mohicano",
    "mohicanos": "mohicano",
    "mohienta": "mohiento",
    "mohientas": "mohiento",
    "mohientos": "mohiento",
    "mohines": "mohín",
    "mohos": "moho",
    "mohosa": "mohoso",
    "mohosas": "mohoso",
    "mohosos": "mohoso",
    "mohína": "mohíno",
    "mohínas": "mohíno",
    "mohínos": "mohíno",
    "moituraciones": "moituración",
    "moja": "mojar",
    "mojaba": "mojar",
    "mojabais": "mojar",
    "mojaban": "mojar",
    "mojabas": "mojar",
    "mojables": "mojable",
    "mojad": "mojar",
    "mojada": "mojar",
    "mojadas": "mojar",
    "mojado": "mojar",
    "mojadora": "mojador",
    "mojadoras": "mojador",
    "mojadores": "mojador",
    "mojados": "mojar",
    "mojaduras": "mojadura",
    "mojamas": "mojama",
    "mojamos": "mojar",
    "mojan": "mojar",
    "mojando": "mojar",
    "mojara": "mojar",
    "mojarais": "mojar",
    "mojaran": "mojar",
    "mojaras": "mojar",
    "mojardones": "mojardón",
    "mojare": "mojar",
    "mojareis": "mojar",
    "mojaremos": "mojar",
    "mojaren": "mojar",
    "mojares": "mojar",
    "mojaron": "mojar",
    "mojarras": "mojarra",
    "mojará": "mojar",
    "mojarán": "mojar",
    "mojarás": "mojar",
    "mojaré": "mojar",
    "mojaréis": "mojar",
    "mojaría": "mojar",
    "mojaríais": "mojar",
    "mojaríamos": "mojar",
    "mojarían": "mojar",
    "mojarías": "mojar",
    "mojas": "mojar",
    "mojase": "mojar",
    "mojaseis": "mojar",
    "mojasen": "mojar",
    "mojases": "mojar",
    "mojaste": "mojar",
    "mojasteis": "mojar",
    "moje": "mojar",
    "mojeles": "mojel",
    "mojemos": "mojar",
    "mojen": "mojar",
    "mojes": "moje",
    "mojicones": "mojicón",
    "mojigangas": "mojiganga",
    "mojigata": "mojigato",
    "mojigatas": "mojigato",
    "mojigateces": "mojigatez",
    "mojigaterías": "mojigatería",
    "mojigatos": "mojigato",
    "mojinetes": "mojinete",
    "mojo": "mojar",
    "mojona": "mojonar",
    "mojonaba": "mojonar",
    "mojonabais": "mojonar",
    "mojonaban": "mojonar",
    "mojonabas": "mojonar",
    "mojonaciones": "mojonación",
    "mojonad": "mojonar",
    "mojonada": "mojonar",
    "mojonadas": "mojonar",
    "mojonado": "mojonar",
    "mojonados": "mojonar",
    "mojonamos": "mojonar",
    "mojonan": "mojonar",
    "mojonando": "mojonar",
    "mojonara": "mojonar",
    "mojonarais": "mojonar",
    "mojonaran": "mojonar",
    "mojonaras": "mojonar",
    "mojonare": "mojonar",
    "mojonareis": "mojonar",
    "mojonaremos": "mojonar",
    "mojonaren": "mojonar",
    "mojonares": "mojonar",
    "mojonaron": "mojonar",
    "mojonará": "mojonar",
    "mojonarán": "mojonar",
    "mojonarás": "mojonar",
    "mojonaré": "mojonar",
    "mojonaréis": "mojonar",
    "mojonaría": "mojonar",
    "mojonaríais": "mojonar",
    "mojonaríamos": "mojonar",
    "mojonarían": "mojonar",
    "mojonarías": "mojonar",
    "mojonas": "mojonar",
    "mojonase": "mojonar",
    "mojonaseis": "mojonar",
    "mojonasen": "mojonar",
    "mojonases": "mojonar",
    "mojonaste": "mojonar",
    "mojonasteis": "mojonar",
    "mojone": "mojonar",
    "mojonemos": "mojonar",
    "mojonen": "mojonar",
    "mojoneras": "mojonera",
    "mojoneros": "mojonero",
    "mojones": "mojonar",
    "mojono": "mojonar",
    "mojonábamos": "mojonar",
    "mojonáis": "mojonar",
    "mojonáramos": "mojonar",
    "mojonáremos": "mojonar",
    "mojonásemos": "mojonar",
    "mojoné": "mojonar",
    "mojonéis": "mojonar",
    "mojonó": "mojonar",
    "mojos": "mojo",
    "mojábamos": "mojar",
    "mojáis": "mojar",
    "mojáramos": "mojar",
    "mojáremos": "mojar",
    "mojásemos": "mojar",
    "mojé": "mojar",
    "mojéis": "mojar",
    "mojíes": "mojí",
    "mojó": "mojar",
    "moladas": "molada",
    "molales": "molal",
    "molalidades": "molalidad",
    "molamos": "moler",
    "molares": "molar",
    "molaridades": "molaridad",
    "molariformes": "molariforme",
    "molas": "mola",
    "molasas": "molasa",
    "molda": "moldar",
    "moldaba": "moldar",
    "moldabais": "moldar",
    "moldaban": "moldar",
    "moldabas": "moldar",
    "moldad": "moldar",
    "moldada": "moldar",
    "moldadas": "moldar",
    "moldado": "moldar",
    "moldados": "moldar",
    "moldamos": "moldar",
    "moldan": "moldar",
    "moldando": "moldar",
    "moldara": "moldar",
    "moldarais": "moldar",
    "moldaran": "moldar",
    "moldaras": "moldar",
    "moldare": "moldar",
    "moldareis": "moldar",
    "moldaremos": "moldar",
    "moldaren": "moldar",
    "moldares": "moldar",
    "moldaron": "moldar",
    "moldará": "moldar",
    "moldarán": "moldar",
    "moldarás": "moldar",
    "moldaré": "moldar",
    "moldaréis": "moldar",
    "moldaría": "moldar",
    "moldaríais": "moldar",
    "moldaríamos": "moldar",
    "moldarían": "moldar",
    "moldarías": "moldar",
    "moldas": "moldar",
    "moldase": "moldar",
    "moldaseis": "moldar",
    "moldasen": "moldar",
    "moldases": "moldar",
    "moldaste": "moldar",
    "moldasteis": "moldar",
    "moldava": "moldavo",
    "moldavas": "moldavo",
    "moldavitas": "moldavita",
    "moldavos": "moldavo",
    "molde": "moldar",
    "moldea": "moldear",
    "moldeaba": "moldear",
    "moldeabais": "moldear",
    "moldeaban": "moldear",
    "moldeabas": "moldear",
    "moldeabilidades": "moldeabilidad",
    "moldeables": "moldeable",
    "moldead": "moldear",
    "moldeada": "moldear",
    "moldeadas": "moldear",
    "moldeado": "moldear",
    "moldeadora": "moldeador",
    "moldeadoras": "moldeadora",
    "moldeadores": "moldeador",
    "moldeados": "moldear",
    "moldeamientos": "moldeamiento",
    "moldeamos": "moldear",
    "moldean": "moldear",
    "moldeando": "moldear",
    "moldeara": "moldear",
    "moldearais": "moldear",
    "moldearan": "moldear",
    "moldearas": "moldear",
    "moldeare": "moldear",
    "moldeareis": "moldear",
    "moldearemos": "moldear",
    "moldearen": "moldear",
    "moldeares": "moldear",
    "moldearon": "moldear",
    "moldeará": "moldear",
    "moldearán": "moldear",
    "moldearás": "moldear",
    "moldearé": "moldear",
    "moldearéis": "moldear",
    "moldearía": "moldear",
    "moldearíais": "moldear",
    "moldearíamos": "moldear",
    "moldearían": "moldear",
    "moldearías": "moldear",
    "moldeas": "moldear",
    "moldease": "moldear",
    "moldeaseis": "moldear",
    "moldeasen": "moldear",
    "moldeases": "moldear",
    "moldeaste": "moldear",
    "moldeasteis": "moldear",
    "moldee": "moldear",
    "moldeemos": "moldear",
    "moldeen": "moldear",
    "moldees": "moldear",
    "moldemos": "moldar",
    "molden": "moldar",
    "moldeo": "moldear",
    "moldeos": "moldeo",
    "moldes": "molde",
    "moldeábamos": "moldear",
    "moldeáis": "moldear",
    "moldeáramos": "moldear",
    "moldeáremos": "moldear",
    "moldeásemos": "moldear",
    "moldeé": "moldear",
    "moldeéis": "moldear",
    "moldeó": "moldear",
    "moldo": "moldar",
    "moldova": "moldovo",
    "moldovas": "moldovo",
    "moldovos": "moldovo",
    "moldura": "moldurar",
    "molduraba": "moldurar",
    "moldurabais": "moldurar",
    "molduraban": "moldurar",
    "moldurabas": "moldurar",
    "moldurad": "moldurar",
    "moldurada": "moldurar",
    "molduradas": "moldurar",
    "moldurado": "moldurar",
    "molduradoras": "molduradora",
    "moldurados": "moldurar",
    "moldurajes": "molduraje",
    "molduramos": "moldurar",
    "molduran": "moldurar",
    "moldurando": "moldurar",
    "moldurara": "moldurar",
    "moldurarais": "moldurar",
    "molduraran": "moldurar",
    "molduraras": "moldurar",
    "moldurare": "moldurar",
    "moldurareis": "moldurar",
    "molduraremos": "moldurar",
    "molduraren": "moldurar",
    "moldurares": "moldurar",
    "molduraron": "moldurar",
    "moldurará": "moldurar",
    "moldurarán": "moldurar",
    "moldurarás": "moldurar",
    "molduraré": "moldurar",
    "molduraréis": "moldurar",
    "molduraría": "moldurar",
    "molduraríais": "moldurar",
    "molduraríamos": "moldurar",
    "moldurarían": "moldurar",
    "moldurarías": "moldurar",
    "molduras": "moldurar",
    "moldurase": "moldurar",
    "molduraseis": "moldurar",
    "moldurasen": "moldurar",
    "moldurases": "moldurar",
    "molduraste": "moldurar",
    "moldurasteis": "moldurar",
    "moldure": "moldurar",
    "molduremos": "moldurar",
    "molduren": "moldurar",
    "moldurera": "moldurero",
    "moldureras": "moldurero",
    "moldureros": "moldurero",
    "moldures": "moldurar",
    "molduro": "moldurar",
    "moldurábamos": "moldurar",
    "molduráis": "moldurar",
    "molduráramos": "moldurar",
    "molduráremos": "moldurar",
    "moldurásemos": "moldurar",
    "molduré": "moldurar",
    "molduréis": "moldurar",
    "molduró": "moldurar",
    "moldábamos": "moldar",
    "moldáis": "moldar",
    "moldáramos": "moldar",
    "moldáremos": "moldar",
    "moldásemos": "moldar",
    "moldé": "moldar",
    "moldéis": "moldar",
    "moldó": "moldar",
    "moleculares": "molecular",
    "molecularidades": "molecularidad",
    "moled": "moler",
    "moledera": "moledero",
    "molederas": "moledero",
    "molederos": "moledero",
    "moledora": "moledor",
    "moledoras": "moledor",
    "moledores": "moledor",
    "moleduras": "moledura",
    "molemos": "moler",
    "molendera": "molendero",
    "molenderas": "molendero",
    "molenderos": "molendero",
    "moleremos": "moler",
    "moleros": "molero",
    "molerá": "moler",
    "molerán": "moler",
    "molerás": "moler",
    "moleré": "moler",
    "moleréis": "moler",
    "molería": "moler",
    "moleríais": "moler",
    "moleríamos": "moler",
    "molerían": "moler",
    "molerías": "moler",
    "moles": "mole",
    "molesquines": "molesquín",
    "molesta": "molesto",
    "molestaba": "molestar",
    "molestabais": "molestar",
    "molestaban": "molestar",
    "molestabas": "molestar",
    "molestad": "molestar",
    "molestada": "molestar",
    "molestadas": "molestar",
    "molestado": "molestar",
    "molestadora": "molestador",
    "molestadoras": "molestador",
    "molestadores": "molestador",
    "molestados": "molestar",
    "molestamos": "molestar",
    "molestan": "molestar",
    "molestando": "molestar",
    "molestara": "molestar",
    "molestarais": "molestar",
    "molestaran": "molestar",
    "molestaras": "molestar",
    "molestare": "molestar",
    "molestareis": "molestar",
    "molestaremos": "molestar",
    "molestaren": "molestar",
    "molestares": "molestar",
    "molestaron": "molestar",
    "molestará": "molestar",
    "molestarán": "molestar",
    "molestarás": "molestar",
    "molestaré": "molestar",
    "molestaréis": "molestar",
    "molestaría": "molestar",
    "molestaríais": "molestar",
    "molestaríamos": "molestar",
    "molestarían": "molestar",
    "molestarías": "molestar",
    "molestas": "molesto",
    "molestase": "molestar",
    "molestaseis": "molestar",
    "molestasen": "molestar",
    "molestases": "molestar",
    "molestaste": "molestar",
    "molestasteis": "molestar",
    "moleste": "molestar",
    "molestemos": "molestar",
    "molesten": "molestar",
    "molestes": "molestar",
    "molestias": "molestia",
    "molesto": "molestar",
    "molestos": "molesto",
    "molestábamos": "molestar",
    "molestáis": "molestar",
    "molestáramos": "molestar",
    "molestáremos": "molestar",
    "molestásemos": "molestar",
    "molesté": "molestar",
    "molestéis": "molestar",
    "molestó": "molestar",
    "moletas": "moleta",
    "moleteada": "moleteado",
    "moleteadas": "moleteado",
    "moleteados": "moleteado",
    "moleña": "moleño",
    "moleñas": "moleño",
    "moleños": "moleño",
    "molibdatos": "molibdato",
    "molibdenitas": "molibdenita",
    "molibdenos": "molibdeno",
    "molibditas": "molibdita",
    "molicies": "molicie",
    "molida": "moler",
    "molidas": "moler",
    "molido": "moler",
    "molidos": "moler",
    "moliendas": "molienda",
    "moliendo": "moler",
    "moliera": "moler",
    "molierais": "moler",
    "molieran": "moler",
    "molieras": "moler",
    "moliere": "moler",
    "moliereis": "moler",
    "molieren": "moler",
    "molieres": "moler",
    "molieron": "moler",
    "moliese": "moler",
    "molieseis": "moler",
    "moliesen": "moler",
    "molieses": "moler",
    "molifica": "molificar",
    "molificaba": "molificar",
    "molificabais": "molificar",
    "molificaban": "molificar",
    "molificabas": "molificar",
    "molificables": "molificable",
    "molificaciones": "molificación",
    "molificad": "molificar",
    "molificada": "molificar",
    "molificadas": "molificar",
    "molificado": "molificar",
    "molificados": "molificar",
    "molificamos": "molificar",
    "molifican": "molificar",
    "molificando": "molificar",
    "molificara": "molificar",
    "molificarais": "molificar",
    "molificaran": "molificar",
    "molificaras": "molificar",
    "molificare": "molificar",
    "molificareis": "molificar",
    "molificaremos": "molificar",
    "molificaren": "molificar",
    "molificares": "molificar",
    "molificaron": "molificar",
    "molificará": "molificar",
    "molificarán": "molificar",
    "molificarás": "molificar",
    "molificaré": "molificar",
    "molificaréis": "molificar",
    "molificaría": "molificar",
    "molificaríais": "molificar",
    "molificaríamos": "molificar",
    "molificarían": "molificar",
    "molificarías": "molificar",
    "molificas": "molificar",
    "molificase": "molificar",
    "molificaseis": "molificar",
    "molificasen": "molificar",
    "molificases": "molificar",
    "molificaste": "molificar",
    "molificasteis": "molificar",
    "molificativa": "molificativo",
    "molificativas": "molificativo",
    "molificativos": "molificativo",
    "molifico": "molificar",
    "molificábamos": "molificar",
    "molificáis": "molificar",
    "molificáramos": "molificar",
    "molificáremos": "molificar",
    "molificásemos": "molificar",
    "molificó": "molificar",
    "molifique": "molificar",
    "molifiquemos": "molificar",
    "molifiquen": "molificar",
    "molifiques": "molificar",
    "molifiqué": "molificar",
    "molifiquéis": "molificar",
    "molimientos": "molimiento",
    "molimos": "moler",
    "molinadas": "molinada",
    "molinares": "molinar",
    "molinera": "molinero",
    "molineras": "molinero",
    "molineros": "molinero",
    "molinerías": "molinería",
    "molinetes": "molinete",
    "molinillos": "molinillo",
    "molinos": "molino",
    "moliste": "moler",
    "molisteis": "moler",
    "molitiva": "molitivo",
    "molitivas": "molitivo",
    "molitivos": "molitivo",
    "moliéramos": "moler",
    "moliéremos": "moler",
    "moliésemos": "moler",
    "molió": "moler",
    "mollares": "mollar",
    "mollas": "molla",
    "molledos": "molledo",
    "molleja": "mollejo",
    "mollejas": "mollejo",
    "mollejones": "mollejón",
    "mollejos": "mollejo",
    "molleras": "mollera",
    "molleros": "mollero",
    "molles": "molle",
    "molletas": "molleta",
    "molletera": "molletero",
    "molleteras": "molletero",
    "molleteros": "molletero",
    "molletes": "mollete",
    "molletuda": "molletudo",
    "molletudas": "molletudo",
    "molletudos": "molletudo",
    "mollina": "mollino",
    "mollinas": "mollino",
    "mollinos": "mollino",
    "mollisoles": "mollisol",
    "mollisuelos": "mollisuelo",
    "mollizna": "molliznar",
    "molliznaba": "molliznar",
    "molliznabais": "molliznar",
    "molliznaban": "molliznar",
    "molliznabas": "molliznar",
    "molliznad": "molliznar",
    "molliznada": "molliznar",
    "molliznadas": "molliznar",
    "molliznado": "molliznar",
    "molliznados": "molliznar",
    "molliznamos": "molliznar",
    "molliznan": "molliznar",
    "molliznando": "molliznar",
    "molliznara": "molliznar",
    "molliznarais": "molliznar",
    "molliznaran": "molliznar",
    "molliznaras": "molliznar",
    "molliznare": "molliznar",
    "molliznareis": "molliznar",
    "molliznaremos": "molliznar",
    "molliznaren": "molliznar",
    "molliznares": "molliznar",
    "molliznaron": "molliznar",
    "molliznará": "molliznar",
    "molliznarán": "molliznar",
    "molliznarás": "molliznar",
    "molliznaré": "molliznar",
    "molliznaréis": "molliznar",
    "molliznaría": "molliznar",
    "molliznaríais": "molliznar",
    "molliznaríamos": "molliznar",
    "molliznarían": "molliznar",
    "molliznarías": "molliznar",
    "molliznas": "molliznar",
    "molliznase": "molliznar",
    "molliznaseis": "molliznar",
    "molliznasen": "molliznar",
    "molliznases": "molliznar",
    "molliznaste": "molliznar",
    "molliznasteis": "molliznar",
    "mollizne": "molliznar",
    "molliznea": "molliznear",
    "mollizneaba": "molliznear",
    "mollizneabais": "molliznear",
    "mollizneaban": "molliznear",
    "mollizneabas": "molliznear",
    "molliznead": "molliznear",
    "mollizneada": "molliznear",
    "mollizneadas": "molliznear",
    "mollizneado": "molliznear",
    "mollizneados": "molliznear",
    "mollizneamos": "molliznear",
    "molliznean": "molliznear",
    "mollizneando": "molliznear",
    "mollizneara": "molliznear",
    "molliznearais": "molliznear",
    "molliznearan": "molliznear",
    "molliznearas": "molliznear",
    "mollizneare": "molliznear",
    "mollizneareis": "molliznear",
    "molliznearemos": "molliznear",
    "molliznearen": "molliznear",
    "mollizneares": "molliznear",
    "molliznearon": "molliznear",
    "mollizneará": "molliznear",
    "molliznearán": "molliznear",
    "molliznearás": "molliznear",
    "molliznearé": "molliznear",
    "molliznearéis": "molliznear",
    "molliznearía": "molliznear",
    "molliznearíais": "molliznear",
    "molliznearíamos": "molliznear",
    "molliznearían": "molliznear",
    "molliznearías": "molliznear",
    "mollizneas": "molliznear",
    "molliznease": "molliznear",
    "mollizneaseis": "molliznear",
    "mollizneasen": "molliznear",
    "mollizneases": "molliznear",
    "mollizneaste": "molliznear",
    "mollizneasteis": "molliznear",
    "molliznee": "molliznear",
    "mollizneemos": "molliznear",
    "mollizneen": "molliznear",
    "molliznees": "molliznear",
    "molliznemos": "molliznar",
    "molliznen": "molliznar",
    "mollizneo": "molliznear",
    "molliznes": "molliznar",
    "mollizneábamos": "molliznear",
    "mollizneáis": "molliznear",
    "mollizneáramos": "molliznear",
    "mollizneáremos": "molliznear",
    "mollizneásemos": "molliznear",
    "mollizneé": "molliznear",
    "mollizneéis": "molliznear",
    "mollizneó": "molliznear",
    "mollizno": "molliznar",
    "molliznábamos": "molliznar",
    "molliznáis": "molliznar",
    "molliznáramos": "molliznar",
    "molliznáremos": "molliznar",
    "molliznásemos": "molliznar",
    "mollizné": "molliznar",
    "molliznéis": "molliznar",
    "molliznó": "molliznar",
    "molondrones": "molondrón",
    "molondros": "molondro",
    "mols": "mol",
    "moltura": "molturar",
    "molturaba": "molturar",
    "molturabais": "molturar",
    "molturaban": "molturar",
    "molturabas": "molturar",
    "molturabilidades": "molturabilidad",
    "molturaciones": "molturación",
    "molturad": "molturar",
    "molturada": "molturar",
    "molturadas": "molturar",
    "molturado": "molturar",
    "molturadora": "molturador",
    "molturadoras": "molturador",
    "molturadores": "molturador",
    "molturados": "molturar",
    "molturamos": "molturar",
    "molturan": "molturar",
    "molturando": "molturar",
    "molturara": "molturar",
    "molturarais": "molturar",
    "molturaran": "molturar",
    "molturaras": "molturar",
    "molturare": "molturar",
    "molturareis": "molturar",
    "molturaremos": "molturar",
    "molturaren": "molturar",
    "molturares": "molturar",
    "molturaron": "molturar",
    "molturará": "molturar",
    "molturarán": "molturar",
    "molturarás": "molturar",
    "molturaré": "molturar",
    "molturaréis": "molturar",
    "molturaría": "molturar",
    "molturaríais": "molturar",
    "molturaríamos": "molturar",
    "molturarían": "molturar",
    "molturarías": "molturar",
    "molturas": "molturar",
    "molturase": "molturar",
    "molturaseis": "molturar",
    "molturasen": "molturar",
    "molturases": "molturar",
    "molturaste": "molturar",
    "molturasteis": "molturar",
    "molture": "molturar",
    "molturemos": "molturar",
    "molturen": "molturar",
    "moltures": "molturar",
    "molturo": "molturar",
    "molturábamos": "molturar",
    "molturáis": "molturar",
    "molturáramos": "molturar",
    "molturáremos": "molturar",
    "molturásemos": "molturar",
    "molturé": "molturar",
    "molturéis": "molturar",
    "molturó": "molturar",
    "moluscos": "molusco",
    "moláis": "moler",
    "moléculas": "molécula",
    "moléis": "moler",
    "molí": "moler",
    "molía": "moler",
    "molíais": "moler",
    "molíamos": "moler",
    "molían": "moler",
    "molías": "moler",
    "molíbdica": "molíbdico",
    "molíbdicas": "molíbdico",
    "molíbdicos": "molíbdico",
    "momea": "momear",
    "momeaba": "momear",
    "momeabais": "momear",
    "momeaban": "momear",
    "momeabas": "momear",
    "momead": "momear",
    "momeada": "momear",
    "momeadas": "momear",
    "momeado": "momear",
    "momeadora": "momeador",
    "momeadoras": "momeador",
    "momeadores": "momeador",
    "momeados": "momear",
    "momeamos": "momear",
    "momean": "momear",
    "momeando": "momear",
    "momeara": "momear",
    "momearais": "momear",
    "momearan": "momear",
    "momearas": "momear",
    "momeare": "momear",
    "momeareis": "momear",
    "momearemos": "momear",
    "momearen": "momear",
    "momeares": "momear",
    "momearon": "momear",
    "momeará": "momear",
    "momearán": "momear",
    "momearás": "momear",
    "momearé": "momear",
    "momearéis": "momear",
    "momearía": "momear",
    "momearíais": "momear",
    "momearíamos": "momear",
    "momearían": "momear",
    "momearías": "momear",
    "momeas": "momear",
    "momease": "momear",
    "momeaseis": "momear",
    "momeasen": "momear",
    "momeases": "momear",
    "momeaste": "momear",
    "momeasteis": "momear",
    "momee": "momear",
    "momeemos": "momear",
    "momeen": "momear",
    "momees": "momear",
    "momentos": "momento",
    "momentánea": "momentáneo",
    "momentáneas": "momentáneo",
    "momentáneos": "momentáneo",
    "momeo": "momear",
    "momera": "momero",
    "momeras": "momero",
    "momeros": "momero",
    "momerías": "momería",
    "momeábamos": "momear",
    "momeáis": "momear",
    "momeáramos": "momear",
    "momeáremos": "momear",
    "momeásemos": "momear",
    "momeé": "momear",
    "momeéis": "momear",
    "momeó": "momear",
    "momia": "momio",
    "momias": "momio",
    "momifica": "momificar",
    "momificaba": "momificar",
    "momificabais": "momificar",
    "momificaban": "momificar",
    "momificabas": "momificar",
    "momificaciones": "momificación",
    "momificad": "momificar",
    "momificada": "momificar",
    "momificadas": "momificar",
    "momificado": "momificar",
    "momificados": "momificar",
    "momificamos": "momificar",
    "momifican": "momificar",
    "momificando": "momificar",
    "momificara": "momificar",
    "momificarais": "momificar",
    "momificaran": "momificar",
    "momificaras": "momificar",
    "momificare": "momificar",
    "momificareis": "momificar",
    "momificaremos": "momificar",
    "momificaren": "momificar",
    "momificares": "momificar",
    "momificaron": "momificar",
    "momificará": "momificar",
    "momificarán": "momificar",
    "momificarás": "momificar",
    "momificaré": "momificar",
    "momificaréis": "momificar",
    "momificaría": "momificar",
    "momificaríais": "momificar",
    "momificaríamos": "momificar",
    "momificarían": "momificar",
    "momificarías": "momificar",
    "momificas": "momificar",
    "momificase": "momificar",
    "momificaseis": "momificar",
    "momificasen": "momificar",
    "momificases": "momificar",
    "momificaste": "momificar",
    "momificasteis": "momificar",
    "momifico": "momificar",
    "momificábamos": "momificar",
    "momificáis": "momificar",
    "momificáramos": "momificar",
    "momificáremos": "momificar",
    "momificásemos": "momificar",
    "momificó": "momificar",
    "momifique": "momificar",
    "momifiquemos": "momificar",
    "momifiquen": "momificar",
    "momifiques": "momificar",
    "momifiqué": "momificar",
    "momifiquéis": "momificar",
    "momios": "momio",
    "momos": "momo",
    "mona": "moño",
    "monacales": "monacal",
    "monacatos": "monacato",
    "monacitas": "monacita",
    "monacordios": "monacordio",
    "monadas": "monada",
    "monadelfa": "monadelfo",
    "monadelfas": "monadelfo",
    "monadelfos": "monadelfo",
    "monadismos": "monadismo",
    "monadologías": "monadología",
    "monagos": "monago",
    "monaguillos": "monaguillo",
    "monaquismos": "monaquismo",
    "monarcas": "monarca",
    "monaria": "monario",
    "monarias": "monario",
    "monarios": "monario",
    "monarquismos": "monarquismo",
    "monarquías": "monarquía",
    "monas": "moño",
    "monasteriales": "monasterial",
    "monasterios": "monasterio",
    "monda": "mondo",
    "mondaba": "mondar",
    "mondabais": "mondar",
    "mondaban": "mondar",
    "mondabas": "mondar",
    "mondad": "mondar",
    "mondada": "mondar",
    "mondadas": "mondar",
    "mondado": "mondar",
    "mondadora": "mondador",
    "mondadoras": "mondador",
    "mondadores": "mondador",
    "mondados": "mondar",
    "mondaduras": "mondadura",
    "mondamos": "mondar",
    "mondan": "mondar",
    "mondando": "mondar",
    "mondara": "mondar",
    "mondarais": "mondar",
    "mondaran": "mondar",
    "mondaras": "mondar",
    "mondare": "mondar",
    "mondareis": "mondar",
    "mondaremos": "mondar",
    "mondaren": "mondar",
    "mondares": "mondar",
    "mondarias": "mondaria",
    "mondaron": "mondar",
    "mondará": "mondar",
    "mondarán": "mondar",
    "mondarás": "mondar",
    "mondaré": "mondar",
    "mondaréis": "mondar",
    "mondaría": "mondar",
    "mondaríais": "mondar",
    "mondaríamos": "mondar",
    "mondarían": "mondar",
    "mondarías": "mondar",
    "mondas": "mondo",
    "mondase": "mondar",
    "mondaseis": "mondar",
    "mondasen": "mondar",
    "mondases": "mondar",
    "mondaste": "mondar",
    "mondasteis": "mondar",
    "monde": "mondar",
    "mondejos": "mondejo",
    "mondemos": "mondar",
    "monden": "mondar",
    "mondes": "mondar",
    "mondo": "mondar",
    "mondones": "mondón",
    "mondongos": "mondongo",
    "mondonguera": "mondonguero",
    "mondongueras": "mondonguero",
    "mondongueros": "mondonguero",
    "mondonguerías": "mondonguería",
    "mondonguiles": "mondonguil",
    "mondos": "mondo",
    "mondábamos": "mondar",
    "mondáis": "mondar",
    "mondáramos": "mondar",
    "mondáremos": "mondar",
    "mondásemos": "mondar",
    "mondé": "mondar",
    "mondéis": "mondar",
    "mondó": "mondar",
    "monea": "monear",
    "moneaba": "monear",
    "moneabais": "monear",
    "moneaban": "monear",
    "moneabas": "monear",
    "monead": "monear",
    "moneada": "monear",
    "moneadas": "monear",
    "moneado": "monear",
    "moneados": "monear",
    "moneamos": "monear",
    "monean": "monear",
    "moneando": "monear",
    "moneara": "monear",
    "monearais": "monear",
    "monearan": "monear",
    "monearas": "monear",
    "moneare": "monear",
    "moneareis": "monear",
    "monearemos": "monear",
    "monearen": "monear",
    "moneares": "monear",
    "monearon": "monear",
    "moneará": "monear",
    "monearán": "monear",
    "monearás": "monear",
    "monearé": "monear",
    "monearéis": "monear",
    "monearía": "monear",
    "monearíais": "monear",
    "monearíamos": "monear",
    "monearían": "monear",
    "monearías": "monear",
    "moneas": "monear",
    "monease": "monear",
    "moneaseis": "monear",
    "moneasen": "monear",
    "moneases": "monear",
    "moneaste": "monear",
    "moneasteis": "monear",
    "moneda": "monedar",
    "monedaba": "monedar",
    "monedabais": "monedar",
    "monedaban": "monedar",
    "monedabas": "monedar",
    "monedad": "monedar",
    "monedada": "monedar",
    "monedadas": "monedar",
    "monedado": "monedar",
    "monedados": "monedar",
    "monedamos": "monedar",
    "monedan": "monedar",
    "monedando": "monedar",
    "monedara": "monedar",
    "monedarais": "monedar",
    "monedaran": "monedar",
    "monedaras": "monedar",
    "monedare": "monedar",
    "monedareis": "monedar",
    "monedaremos": "monedar",
    "monedaren": "monedar",
    "monedares": "monedar",
    "monedaron": "monedar",
    "monedará": "monedar",
    "monedarán": "monedar",
    "monedarás": "monedar",
    "monedaré": "monedar",
    "monedaréis": "monedar",
    "monedaría": "monedar",
    "monedaríais": "monedar",
    "monedaríamos": "monedar",
    "monedarían": "monedar",
    "monedarías": "monedar",
    "monedas": "monedar",
    "monedase": "monedar",
    "monedaseis": "monedar",
    "monedasen": "monedar",
    "monedases": "monedar",
    "monedaste": "monedar",
    "monedasteis": "monedar",
    "monede": "monedar",
    "monedea": "monedear",
    "monedeaba": "monedear",
    "monedeabais": "monedear",
    "monedeaban": "monedear",
    "monedeabas": "monedear",
    "monedead": "monedear",
    "monedeada": "monedear",
    "monedeadas": "monedear",
    "monedeado": "monedear",
    "monedeados": "monedear",
    "monedeamos": "monedear",
    "monedean": "monedear",
    "monedeando": "monedear",
    "monedeara": "monedear",
    "monedearais": "monedear",
    "monedearan": "monedear",
    "monedearas": "monedear",
    "monedeare": "monedear",
    "monedeareis": "monedear",
    "monedearemos": "monedear",
    "monedearen": "monedear",
    "monedeares": "monedear",
    "monedearon": "monedear",
    "monedeará": "monedear",
    "monedearán": "monedear",
    "monedearás": "monedear",
    "monedearé": "monedear",
    "monedearéis": "monedear",
    "monedearía": "monedear",
    "monedearíais": "monedear",
    "monedearíamos": "monedear",
    "monedearían": "monedear",
    "monedearías": "monedear",
    "monedeas": "monedear",
    "monedease": "monedear",
    "monedeaseis": "monedear",
    "monedeasen": "monedear",
    "monedeases": "monedear",
    "monedeaste": "monedear",
    "monedeasteis": "monedear",
    "monedee": "monedear",
    "monedeemos": "monedear",
    "monedeen": "monedear",
    "monedees": "monedear",
    "monedemos": "monedar",
    "moneden": "monedar",
    "monedeo": "monedear",
    "monederos": "monedero",
    "monederías": "monedería",
    "monedes": "monedar",
    "monedeábamos": "monedear",
    "monedeáis": "monedear",
    "monedeáramos": "monedear",
    "monedeáremos": "monedear",
    "monedeásemos": "monedear",
    "monedeé": "monedear",
    "monedeéis": "monedear",
    "monedeó": "monedear",
    "monedo": "monedar",
    "monedábamos": "monedar",
    "monedáis": "monedar",
    "monedáramos": "monedar",
    "monedáremos": "monedar",
    "monedásemos": "monedar",
    "monedé": "monedar",
    "monedéis": "monedar",
    "monedó": "monedar",
    "monee": "monear",
    "moneemos": "monear",
    "moneen": "monear",
    "monees": "monear",
    "monegasca": "monegasco",
    "monegascas": "monegasco",
    "monegascos": "monegasco",
    "moneles": "monel",
    "monemas": "monema",
    "moneo": "monear",
    "monerías": "monería",
    "monetaria": "monetario",
    "monetarias": "monetario",
    "monetariedades": "monetariedad",
    "monetarios": "monetario",
    "monetarismos": "monetarismo",
    "monetaristas": "monetarista",
    "monetarizaciones": "monetarización",
    "monetice": "monetizar",
    "moneticemos": "monetizar",
    "moneticen": "monetizar",
    "monetices": "monetizar",
    "moneticé": "monetizar",
    "moneticéis": "monetizar",
    "monetiza": "monetizar",
    "monetizaba": "monetizar",
    "monetizabais": "monetizar",
    "monetizaban": "monetizar",
    "monetizabas": "monetizar",
    "monetizaciones": "monetización",
    "monetizad": "monetizar",
    "monetizada": "monetizar",
    "monetizadas": "monetizar",
    "monetizado": "monetizar",
    "monetizados": "monetizar",
    "monetizamos": "monetizar",
    "monetizan": "monetizar",
    "monetizando": "monetizar",
    "monetizara": "monetizar",
    "monetizarais": "monetizar",
    "monetizaran": "monetizar",
    "monetizaras": "monetizar",
    "monetizare": "monetizar",
    "monetizareis": "monetizar",
    "monetizaremos": "monetizar",
    "monetizaren": "monetizar",
    "monetizares": "monetizar",
    "monetizaron": "monetizar",
    "monetizará": "monetizar",
    "monetizarán": "monetizar",
    "monetizarás": "monetizar",
    "monetizaré": "monetizar",
    "monetizaréis": "monetizar",
    "monetizaría": "monetizar",
    "monetizaríais": "monetizar",
    "monetizaríamos": "monetizar",
    "monetizarían": "monetizar",
    "monetizarías": "monetizar",
    "monetizas": "monetizar",
    "monetizase": "monetizar",
    "monetizaseis": "monetizar",
    "monetizasen": "monetizar",
    "monetizases": "monetizar",
    "monetizaste": "monetizar",
    "monetizasteis": "monetizar",
    "monetizo": "monetizar",
    "monetizábamos": "monetizar",
    "monetizáis": "monetizar",
    "monetizáramos": "monetizar",
    "monetizáremos": "monetizar",
    "monetizásemos": "monetizar",
    "monetizó": "monetizar",
    "moneábamos": "monear",
    "moneáis": "monear",
    "moneáramos": "monear",
    "moneáremos": "monear",
    "moneásemos": "monear",
    "moneé": "monear",
    "moneéis": "monear",
    "moneó": "monear",
    "monfíes": "monfí",
    "mongola": "mongol",
    "mongolas": "mongol",
    "mongoles": "mongol",
    "mongolismos": "mongolismo",
    "mongoloides": "mongoloide",
    "mongólica": "mongólico",
    "mongólicas": "mongólico",
    "mongólicos": "mongólico",
    "moniatos": "moniato",
    "moniciones": "monición",
    "monigotes": "monigote",
    "monipodios": "monipodio",
    "monises": "monís",
    "monismos": "monismo",
    "monistas": "monista",
    "monitora": "monitor",
    "monitoras": "monitor",
    "monitores": "monitor",
    "monitoria": "monitorio",
    "monitorias": "monitorio",
    "monitorice": "monitorizar",
    "monitoricemos": "monitorizar",
    "monitoricen": "monitorizar",
    "monitorices": "monitorizar",
    "monitoricé": "monitorizar",
    "monitoricéis": "monitorizar",
    "monitorios": "monitorio",
    "monitoriza": "monitorizar",
    "monitorizaba": "monitorizar",
    "monitorizabais": "monitorizar",
    "monitorizaban": "monitorizar",
    "monitorizabas": "monitorizar",
    "monitorizaciones": "monitorización",
    "monitorizad": "monitorizar",
    "monitorizada": "monitorizar",
    "monitorizadas": "monitorizar",
    "monitorizado": "monitorizar",
    "monitorizados": "monitorizar",
    "monitorizamos": "monitorizar",
    "monitorizan": "monitorizar",
    "monitorizando": "monitorizar",
    "monitorizara": "monitorizar",
    "monitorizarais": "monitorizar",
    "monitorizaran": "monitorizar",
    "monitorizaras": "monitorizar",
    "monitorizare": "monitorizar",
    "monitorizareis": "monitorizar",
    "monitorizaremos": "monitorizar",
    "monitorizaren": "monitorizar",
    "monitorizares": "monitorizar",
    "monitorizaron": "monitorizar",
    "monitorizará": "monitorizar",
    "monitorizarán": "monitorizar",
    "monitorizarás": "monitorizar",
    "monitorizaré": "monitorizar",
    "monitorizaréis": "monitorizar",
    "monitorizaría": "monitorizar",
    "monitorizaríais": "monitorizar",
    "monitorizaríamos": "monitorizar",
    "monitorizarían": "monitorizar",
    "monitorizarías": "monitorizar",
    "monitorizas": "monitorizar",
    "monitorizase": "monitorizar",
    "monitorizaseis": "monitorizar",
    "monitorizasen": "monitorizar",
    "monitorizases": "monitorizar",
    "monitorizaste": "monitorizar",
    "monitorizasteis": "monitorizar",
    "monitorizo": "monitorizar",
    "monitorizábamos": "monitorizar",
    "monitorizáis": "monitorizar",
    "monitorizáramos": "monitorizar",
    "monitorizáremos": "monitorizar",
    "monitorizásemos": "monitorizar",
    "monitorizó": "monitorizar",
    "monja": "monje",
    "monjas": "monje",
    "monjes": "monje",
    "monjiles": "monjil",
    "monjías": "monjía",
    "monjíos": "monjío",
    "monoacetatos": "monoacetato",
    "monoaplicaciones": "monoaplicación",
    "monoatómica": "monoatómico",
    "monoatómicas": "monoatómico",
    "monoatómicos": "monoatómico",
    "monoaurales": "monoaural",
    "monoauriculares": "monoauricular",
    "monobenzoatos": "monobenzoato",
    "monoblastos": "monoblasto",
    "monobloces": "monobloc",
    "monobromonaftalenos": "monobromonaftaleno",
    "monobromuros": "monobromuro",
    "monobásica": "monobásico",
    "monobásicas": "monobásico",
    "monobásicos": "monobásico",
    "monocables": "monocable",
    "monocameralismos": "monocameralismo",
    "monocarburos": "monocarburo",
    "monocarriles": "monocarril",
    "monocascos": "monocasco",
    "monocelulares": "monocelular",
    "monocerontes": "monoceronte",
    "monocerotes": "monocerote",
    "monociclos": "monociclo",
    "monocilíndrica": "monocilíndrico",
    "monocilíndricas": "monocilíndrico",
    "monocilíndricos": "monocilíndrico",
    "monocinas": "monocina",
    "monocitopenias": "monocitopenia",
    "monocitos": "monocito",
    "monoclamídeas": "monoclamídea",
    "monoclinales": "monoclinal",
    "monoclonales": "monoclonal",
    "monocloracética": "monocloracético",
    "monocloracéticas": "monocloracético",
    "monocloracéticos": "monocloracético",
    "monocloruros": "monocloruro",
    "monoclínica": "monoclínico",
    "monoclínicas": "monoclínico",
    "monoclínicos": "monoclínico",
    "monocolores": "monocolor",
    "monoconversiones": "monoconversión",
    "monocordes": "monocorde",
    "monocordios": "monocordio",
    "monocotiledónea": "monocotiledóneo",
    "monocotiledóneas": "monocotiledóneo",
    "monocotiledóneos": "monocotiledóneo",
    "monocracias": "monocracia",
    "monocristales": "monocristal",
    "monocristalina": "monocristalino",
    "monocristalinas": "monocristalino",
    "monocristalinos": "monocristalino",
    "monocroma": "monocromo",
    "monocromadores": "monocromador",
    "monocromas": "monocromo",
    "monocromatismos": "monocromatismo",
    "monocromos": "monocromo",
    "monocromática": "monocromático",
    "monocromáticas": "monocromático",
    "monocromáticos": "monocromático",
    "monoculares": "monocular",
    "monocultivos": "monocultivo",
    "monocuspidada": "monocuspidado",
    "monocuspidadas": "monocuspidado",
    "monocuspidados": "monocuspidado",
    "monocytogenes": "monocytogén",
    "monocárpica": "monocárpico",
    "monocárpicas": "monocárpico",
    "monocárpicos": "monocárpico",
    "monocélulas": "monocélula",
    "monocíclica": "monocíclico",
    "monocíclicas": "monocíclico",
    "monocíclicos": "monocíclico",
    "monocítica": "monocítico",
    "monocíticas": "monocítico",
    "monocíticos": "monocítico",
    "monodelta": "monodelto",
    "monodeltas": "monodelto",
    "monodeltos": "monodelto",
    "monodias": "monodia",
    "monodimensionales": "monodimensional",
    "monodireccionales": "monodireccional",
    "monodiscos": "monodisco",
    "monodispersa": "monodisperso",
    "monodispersas": "monodisperso",
    "monodispersos": "monodisperso",
    "monoecias": "monoecia",
    "monoergismos": "monoergismo",
    "monoespecífica": "monoespecífico",
    "monoespecíficas": "monoespecífico",
    "monoespecíficos": "monoespecífico",
    "monoesquises": "monoesquí",
    "monoestables": "monoestable",
    "monoexportadora": "monoexportador",
    "monoexportadoras": "monoexportador",
    "monoexportadores": "monoexportador",
    "monofila": "monofilo",
    "monofilamentos": "monofilamento",
    "monofilares": "monofilar",
    "monofilas": "monofilo",
    "monofilos": "monofilo",
    "monofiodonta": "monofiodonto",
    "monofiodontas": "monofiodonto",
    "monofiodontos": "monofiodonto",
    "monofisismos": "monofisismo",
    "monofisitas": "monofisita",
    "monofosfatos": "monofosfato",
    "monofásica": "monofásico",
    "monofásicas": "monofásico",
    "monofásicos": "monofásico",
    "monofónica": "monofónico",
    "monofónicas": "monofónico",
    "monofónicos": "monofónico",
    "monogamias": "monogamia",
    "monogenismos": "monogenismo",
    "monogenistas": "monogenista",
    "monoglucosilesingosinas": "monoglucosilesingosina",
    "monografistas": "monografista",
    "monografías": "monografía",
    "monogramas": "monograma",
    "monográfica": "monográfico",
    "monográficas": "monográfico",
    "monográficos": "monográfico",
    "monogámica": "monogámico",
    "monogámicas": "monogámico",
    "monogámicos": "monogámico",
    "monohídrica": "monohídrico",
    "monohídricas": "monohídrico",
    "monohídricos": "monohídrico",
    "monoica": "monoico",
    "monoicas": "monoico",
    "monoicos": "monoico",
    "monoinductores": "monoinductor",
    "monokinis": "monokini",
    "monolauratos": "monolaurato",
    "monoleatos": "monoleato",
    "monolingües": "monolingüe",
    "monolingüismos": "monolingüismo",
    "monolitismos": "monolitismo",
    "monolitos": "monolito",
    "monologa": "monologar",
    "monologaba": "monologar",
    "monologabais": "monologar",
    "monologaban": "monologar",
    "monologabas": "monologar",
    "monologad": "monologar",
    "monologada": "monologar",
    "monologadas": "monologar",
    "monologado": "monologar",
    "monologados": "monologar",
    "monologamos": "monologar",
    "monologan": "monologar",
    "monologando": "monologar",
    "monologara": "monologar",
    "monologarais": "monologar",
    "monologaran": "monologar",
    "monologaras": "monologar",
    "monologare": "monologar",
    "monologareis": "monologar",
    "monologaremos": "monologar",
    "monologaren": "monologar",
    "monologares": "monologar",
    "monologaron": "monologar",
    "monologará": "monologar",
    "monologarán": "monologar",
    "monologarás": "monologar",
    "monologaré": "monologar",
    "monologaréis": "monologar",
    "monologaría": "monologar",
    "monologaríais": "monologar",
    "monologaríamos": "monologar",
    "monologarían": "monologar",
    "monologarías": "monologar",
    "monologas": "monologar",
    "monologase": "monologar",
    "monologaseis": "monologar",
    "monologasen": "monologar",
    "monologases": "monologar",
    "monologaste": "monologar",
    "monologasteis": "monologar",
    "monologo": "monologar",
    "monologue": "monologar",
    "monologuemos": "monologar",
    "monologuen": "monologar",
    "monologues": "monologar",
    "monologué": "monologar",
    "monologuéis": "monologar",
    "monologábamos": "monologar",
    "monologáis": "monologar",
    "monologáramos": "monologar",
    "monologáremos": "monologar",
    "monologásemos": "monologar",
    "monologó": "monologar",
    "monoláminas": "monolámina",
    "monolítica": "monolítico",
    "monolíticas": "monolítico",
    "monolíticos": "monolítico",
    "monomaniaca": "monomaníaco",
    "monomaniacas": "monomaníaco",
    "monomaniacos": "monomaníaco",
    "monomaniática": "monomaniático",
    "monomaniáticas": "monomaniático",
    "monomaniáticos": "monomaniático",
    "monomaníaca": "monomaníaco",
    "monomaníacas": "monomaníaco",
    "monomaníacos": "monomaníaco",
    "monomanías": "monomanía",
    "monomaquias": "monomaquia",
    "monomaxilares": "monomaxilar",
    "monomerías": "monomería",
    "monometalismos": "monometalismo",
    "monometalistas": "monometalista",
    "monomios": "monomio",
    "monomodales": "monomodal",
    "monomoleculares": "monomolecular",
    "monomorfa": "monomorfo",
    "monomorfas": "monomorfo",
    "monomorfismos": "monomorfismo",
    "monomorfos": "monomorfo",
    "mononas": "monona",
    "mononitrorresorcinatos": "mononitrorresorcinato",
    "mononucleada": "mononucleado",
    "mononucleadas": "mononucleado",
    "mononucleados": "mononucleado",
    "mononucleares": "mononuclear",
    "monoocasionales": "monoocasional",
    "monoparentales": "monoparental",
    "monopartidismos": "monopartidismo",
    "monopatinajes": "monopatinaje",
    "monopatines": "monopatín",
    "monoplanares": "monoplanar",
    "monoplanos": "monoplano",
    "monoplazas": "monoplaza",
    "monoploides": "monoploide",
    "monoploidías": "monoploidía",
    "monopolares": "monopolar",
    "monopolice": "monopolizar",
    "monopolicemos": "monopolizar",
    "monopolicen": "monopolizar",
    "monopolices": "monopolizar",
    "monopolicé": "monopolizar",
    "monopolicéis": "monopolizar",
    "monopolios": "monopolio",
    "monopolistas": "monopolista",
    "monopoliza": "monopolizar",
    "monopolizaba": "monopolizar",
    "monopolizabais": "monopolizar",
    "monopolizaban": "monopolizar",
    "monopolizabas": "monopolizar",
    "monopolizaciones": "monopolización",
    "monopolizad": "monopolizar",
    "monopolizada": "monopolizar",
    "monopolizadas": "monopolizar",
    "monopolizado": "monopolizar",
    "monopolizadora": "monopolizador",
    "monopolizadoras": "monopolizador",
    "monopolizadores": "monopolizador",
    "monopolizados": "monopolizar",
    "monopolizamos": "monopolizar",
    "monopolizan": "monopolizar",
    "monopolizando": "monopolizar",
    "monopolizara": "monopolizar",
    "monopolizarais": "monopolizar",
    "monopolizaran": "monopolizar",
    "monopolizaras": "monopolizar",
    "monopolizare": "monopolizar",
    "monopolizareis": "monopolizar",
    "monopolizaremos": "monopolizar",
    "monopolizaren": "monopolizar",
    "monopolizares": "monopolizar",
    "monopolizaron": "monopolizar",
    "monopolizará": "monopolizar",
    "monopolizarán": "monopolizar",
    "monopolizarás": "monopolizar",
    "monopolizaré": "monopolizar",
    "monopolizaréis": "monopolizar",
    "monopolizaría": "monopolizar",
    "monopolizaríais": "monopolizar",
    "monopolizaríamos": "monopolizar",
    "monopolizarían": "monopolizar",
    "monopolizarías": "monopolizar",
    "monopolizas": "monopolizar",
    "monopolizase": "monopolizar",
    "monopolizaseis": "monopolizar",
    "monopolizasen": "monopolizar",
    "monopolizases": "monopolizar",
    "monopolizaste": "monopolizar",
    "monopolizasteis": "monopolizar",
    "monopolizo": "monopolizar",
    "monopolizábamos": "monopolizar",
    "monopolizáis": "monopolizar",
    "monopolizáramos": "monopolizar",
    "monopolizáremos": "monopolizar",
    "monopolizásemos": "monopolizar",
    "monopolizó": "monopolizar",
    "monopolos": "monopolo",
    "monopolística": "monopolístico",
    "monopolísticas": "monopolístico",
    "monopolísticos": "monopolístico",
    "monoprocesadores": "monoprocesador",
    "monoprogramaciones": "monoprogramación",
    "monopsonios": "monopsonio",
    "monopsonistas": "monopsonista",
    "monopterigia": "monopterigio",
    "monopterigias": "monopterigio",
    "monopterigios": "monopterigio",
    "monoptongaciones": "monoptongación",
    "monoptongos": "monoptongo",
    "monopuertas": "monopuerta",
    "monopuestos": "monopuesto",
    "monopétala": "monopétalo",
    "monopétalas": "monopétalo",
    "monopétalos": "monopétalo",
    "monopólica": "monopólico",
    "monopólicas": "monopólico",
    "monopólicos": "monopólico",
    "monoquinas": "monoquina",
    "monoraíles": "monoraíl",
    "monorquidias": "monorquidia",
    "monorradiculares": "monorradicular",
    "monorrefringentes": "monorrefringente",
    "monorrima": "monorrimo",
    "monorrimas": "monorrimo",
    "monorrimos": "monorrimo",
    "monorrítmica": "monorrítmico",
    "monorrítmicas": "monorrítmico",
    "monorrítmicos": "monorrítmico",
    "monos": "moño",
    "monosabios": "monosabio",
    "monosacarosas": "monosacarosa",
    "monosacáridos": "monosacárido",
    "monoscopios": "monoscopio",
    "monosemias": "monosemia",
    "monosialitizaciones": "monosialitización",
    "monosialsoles": "monosialsol",
    "monosilabismos": "monosilabismo",
    "monosilábica": "monosilábico",
    "monosilábicas": "monosilábico",
    "monosilábicos": "monosilábico",
    "monosomas": "monosoma",
    "monosperma": "monospermo",
    "monospermas": "monospermo",
    "monospermias": "monospermia",
    "monospermos": "monospermo",
    "monostearatos": "monostearato",
    "monostrófica": "monostrófico",
    "monostróficas": "monostrófico",
    "monostróficos": "monostrófico",
    "monosulfuros": "monosulfuro",
    "monosémica": "monosémico",
    "monosémicas": "monosémico",
    "monosémicos": "monosémico",
    "monosépala": "monosépalo",
    "monosépalas": "monosépalo",
    "monosépalos": "monosépalo",
    "monosílaba": "monosílabo",
    "monosílabas": "monosílabo",
    "monosílabos": "monosílabo",
    "monotareas": "monotarea",
    "monotarjetas": "monotarjeta",
    "monotelismos": "monotelismo",
    "monotes": "monote",
    "monoteísmos": "monoteísmo",
    "monoteístas": "monoteísta",
    "monotipias": "monotipia",
    "monotipistas": "monotipista",
    "monotipos": "monotipo",
    "monotonicidades": "monotonicidad",
    "monotonocidades": "monotonocidad",
    "monotonías": "monotonía",
    "monotratamientos": "monotratamiento",
    "monotremas": "monotrema",
    "monotrópica": "monotrópico",
    "monotrópicas": "monotrópico",
    "monotrópicos": "monotrópico",
    "monotónica": "monotónico",
    "monotónicas": "monotónico",
    "monotónicos": "monotónico",
    "monousuaria": "monousuario",
    "monousuarias": "monousuario",
    "monousuarios": "monousuario",
    "monovalentes": "monovalente",
    "monovalvulares": "monovalvular",
    "monovolúmenes": "monovolumen",
    "monozigótica": "monozigótico",
    "monozigóticas": "monozigótico",
    "monozigóticos": "monozigótico",
    "monsergas": "monserga",
    "monseñores": "monseñor",
    "monstruos": "monstruo",
    "monstruosa": "monstruoso",
    "monstruosas": "monstruoso",
    "monstruosidades": "monstruosidad",
    "monstruosos": "monstruoso",
    "monta": "montar",
    "montaba": "montar",
    "montabais": "montar",
    "montaban": "montar",
    "montabas": "montar",
    "montad": "montar",
    "montada": "montar",
    "montadas": "montar",
    "montaderos": "montadero",
    "montado": "montar",
    "montadora": "montador",
    "montadoras": "montadora",
    "montadores": "montador",
    "montados": "montar",
    "montaduras": "montadura",
    "montajes": "montaje",
    "montamos": "montar",
    "montan": "montar",
    "montana": "montano",
    "montanas": "montano",
    "montando": "montar",
    "montanera": "montañero",
    "montaneras": "montañero",
    "montaneros": "montañero",
    "montanismos": "montanismo",
    "montanistas": "montanista",
    "montanos": "montano",
    "montantes": "montante",
    "montara": "montar",
    "montaraces": "montaraz",
    "montarais": "montar",
    "montaran": "montar",
    "montaras": "montar",
    "montare": "montar",
    "montareis": "montar",
    "montaremos": "montar",
    "montaren": "montar",
    "montares": "montar",
    "montaron": "montar",
    "montará": "montar",
    "montarán": "montar",
    "montarás": "montar",
    "montaré": "montar",
    "montaréis": "montar",
    "montaría": "montar",
    "montaríais": "montar",
    "montaríamos": "montar",
    "montarían": "montar",
    "montarías": "montar",
    "montas": "montar",
    "montase": "montar",
    "montaseis": "montar",
    "montasen": "montar",
    "montases": "montar",
    "montaste": "montar",
    "montasteis": "montar",
    "montazgos": "montazgo",
    "montañas": "montaña",
    "montañera": "montañero",
    "montañeras": "montañero",
    "montañeros": "montañero",
    "montañesa": "montañés",
    "montañesas": "montañés",
    "montañeses": "montañés",
    "montañismos": "montañismo",
    "montañosa": "montañoso",
    "montañosas": "montañoso",
    "montañosos": "montañoso",
    "monte": "montar",
    "montea": "montear",
    "monteaba": "montear",
    "monteabais": "montear",
    "monteaban": "montear",
    "monteabas": "montear",
    "montead": "montear",
    "monteada": "montear",
    "monteadas": "montear",
    "monteado": "montear",
    "monteadores": "monteador",
    "monteados": "montear",
    "monteamos": "montear",
    "montean": "montear",
    "monteando": "montear",
    "monteara": "montear",
    "montearais": "montear",
    "montearan": "montear",
    "montearas": "montear",
    "monteare": "montear",
    "monteareis": "montear",
    "montearemos": "montear",
    "montearen": "montear",
    "monteares": "montear",
    "montearon": "montear",
    "monteará": "montear",
    "montearán": "montear",
    "montearás": "montear",
    "montearé": "montear",
    "montearéis": "montear",
    "montearía": "montear",
    "montearíais": "montear",
    "montearíamos": "montear",
    "montearían": "montear",
    "montearías": "montear",
    "monteas": "montear",
    "montease": "montear",
    "monteaseis": "montear",
    "monteasen": "montear",
    "monteases": "montear",
    "monteaste": "montear",
    "monteasteis": "montear",
    "montee": "montear",
    "monteemos": "montear",
    "monteen": "montear",
    "montees": "montear",
    "montemos": "montar",
    "monten": "montar",
    "montenegrina": "montenegrino",
    "montenegrinas": "montenegrino",
    "monteo": "montear",
    "montepíos": "montepío",
    "monteras": "montera",
    "monterera": "monterero",
    "montereras": "monterero",
    "montereros": "monterero",
    "montererías": "monterería",
    "monterillas": "monterilla",
    "monteros": "montero",
    "monterías": "montería",
    "montes": "monte",
    "montesa": "montés",
    "montesas": "montés",
    "monteses": "montés",
    "montesina": "montesino",
    "montesinas": "montesino",
    "montesinos": "montesino",
    "montevideana": "montevideano",
    "montevideanas": "montevideano",
    "montevideanos": "montevideano",
    "monteábamos": "montear",
    "monteáis": "montear",
    "monteáramos": "montear",
    "monteáremos": "montear",
    "monteásemos": "montear",
    "monteé": "montear",
    "monteéis": "montear",
    "monteó": "montear",
    "montgomeries": "montgomery",
    "monticellitas": "monticellita",
    "montillas": "montilla",
    "montmorillonitas": "montmorillonita",
    "monto": "montar",
    "montonera": "montonero",
    "montoneras": "montonero",
    "montoneros": "montonero",
    "montones": "montón",
    "montos": "monto",
    "montuna": "montuno",
    "montunas": "montuno",
    "montunos": "montuno",
    "montuosa": "montuoso",
    "montuosas": "montuoso",
    "montuosidades": "montuosidad",
    "montuosos": "montuoso",
    "monturas": "montura",
    "montábamos": "montar",
    "montáis": "montar",
    "montáramos": "montar",
    "montáremos": "montar",
    "montásemos": "montar",
    "monté": "montar",
    "montéis": "montar",
    "montículos": "montículo",
    "montó": "montar",
    "montónes": "montón",
    "monumentales": "monumental",
    "monumentalidades": "monumentalidad",
    "monumentalismos": "monumentalismo",
    "monumentos": "monumento",
    "monzones": "monzón",
    "monzonitas": "monzonita",
    "monzónica": "monzónico",
    "monzónicas": "monzónico",
    "monzónicos": "monzónico",
    "monádica": "monádico",
    "monádicas": "monádico",
    "monádicos": "monádico",
    "monárquica": "monárquico",
    "monárquicas": "monárquico",
    "monárquicos": "monárquico",
    "monástica": "monástico",
    "monásticas": "monástico",
    "monásticos": "monástico",
    "monócula": "monóculo",
    "monóculas": "monóculo",
    "monóculos": "monóculo",
    "monódica": "monódico",
    "monódicas": "monódico",
    "monódicos": "monódico",
    "monógama": "monógamo",
    "monógamas": "monógamo",
    "monógamos": "monógamo",
    "monólogos": "monólogo",
    "monómeros": "monómero",
    "monóptera": "monóptero",
    "monópteras": "monóptero",
    "monópteros": "monóptero",
    "monóstrofes": "monóstrofe",
    "monótona": "monótono",
    "monótonas": "monótono",
    "monótonos": "monótono",
    "monóxidos": "monóxido",
    "monóxilos": "monóxilo",
    "mopas": "mopa",
    "moque": "mocar",
    "moquea": "moquear",
    "moqueaba": "moquear",
    "moqueabais": "moquear",
    "moqueaban": "moquear",
    "moqueabas": "moquear",
    "moquead": "moquear",
    "moqueada": "moquear",
    "moqueadas": "moquear",
    "moqueado": "moquear",
    "moqueados": "moquear",
    "moqueamos": "moquear",
    "moquean": "moquear",
    "moqueando": "moquear",
    "moqueara": "moquear",
    "moquearais": "moquear",
    "moquearan": "moquear",
    "moquearas": "moquear",
    "moqueare": "moquear",
    "moqueareis": "moquear",
    "moquearemos": "moquear",
    "moquearen": "moquear",
    "moqueares": "moquear",
    "moquearon": "moquear",
    "moqueará": "moquear",
    "moquearán": "moquear",
    "moquearás": "moquear",
    "moquearé": "moquear",
    "moquearéis": "moquear",
    "moquearía": "moquear",
    "moquearíais": "moquear",
    "moquearíamos": "moquear",
    "moquearían": "moquear",
    "moquearías": "moquear",
    "moqueas": "moquear",
    "moquease": "moquear",
    "moqueaseis": "moquear",
    "moqueasen": "moquear",
    "moqueases": "moquear",
    "moqueaste": "moquear",
    "moqueasteis": "moquear",
    "moquee": "moquear",
    "moqueemos": "moquear",
    "moqueen": "moquear",
    "moquees": "moquear",
    "moquemos": "mocar",
    "moquen": "mocar",
    "moqueo": "moquear",
    "moqueos": "moqueo",
    "moqueros": "moquero",
    "moques": "mocar",
    "moquetas": "moqueta",
    "moquetea": "moquetear",
    "moqueteaba": "moquetear",
    "moqueteabais": "moquetear",
    "moqueteaban": "moquetear",
    "moqueteabas": "moquetear",
    "moquetead": "moquetear",
    "moqueteada": "moquetear",
    "moqueteadas": "moquetear",
    "moqueteado": "moquetear",
    "moqueteados": "moquetear",
    "moqueteamos": "moquetear",
    "moquetean": "moquetear",
    "moqueteando": "moquetear",
    "moqueteara": "moquetear",
    "moquetearais": "moquetear",
    "moquetearan": "moquetear",
    "moquetearas": "moquetear",
    "moqueteare": "moquetear",
    "moqueteareis": "moquetear",
    "moquetearemos": "moquetear",
    "moquetearen": "moquetear",
    "moqueteares": "moquetear",
    "moquetearon": "moquetear",
    "moqueteará": "moquetear",
    "moquetearán": "moquetear",
    "moquetearás": "moquetear",
    "moquetearé": "moquetear",
    "moquetearéis": "moquetear",
    "moquetearía": "moquetear",
    "moquetearíais": "moquetear",
    "moquetearíamos": "moquetear",
    "moquetearían": "moquetear",
    "moquetearías": "moquetear",
    "moqueteas": "moquetear",
    "moquetease": "moquetear",
    "moqueteaseis": "moquetear",
    "moqueteasen": "moquetear",
    "moqueteases": "moquetear",
    "moqueteaste": "moquetear",
    "moqueteasteis": "moquetear",
    "moquetee": "moquetear",
    "moqueteemos": "moquetear",
    "moqueteen": "moquetear",
    "moquetees": "moquetear",
    "moqueteo": "moquetear",
    "moqueteos": "moqueteo",
    "moquetes": "moquete",
    "moqueteábamos": "moquetear",
    "moqueteáis": "moquetear",
    "moqueteáramos": "moquetear",
    "moqueteáremos": "moquetear",
    "moqueteásemos": "moquetear",
    "moqueteé": "moquetear",
    "moqueteéis": "moquetear",
    "moqueteó": "moquetear",
    "moqueábamos": "moquear",
    "moqueáis": "moquear",
    "moqueáramos": "moquear",
    "moqueáremos": "moquear",
    "moqueásemos": "moquear",
    "moqueé": "moquear",
    "moqueéis": "moquear",
    "moqueó": "moquear",
    "moquillos": "moquillo",
    "moquitas": "moquita",
    "moquiteos": "moquiteo",
    "moqué": "mocar",
    "moquéis": "mocar",
    "mora": "moro",
    "moraba": "morar",
    "morabais": "morar",
    "moraban": "morar",
    "morabas": "morar",
    "morabetinos": "morabetino",
    "morabitos": "morabito",
    "morad": "morar",
    "morada": "morar",
    "moradas": "morar",
    "morado": "morar",
    "moradoes": "morado",
    "moradora": "morador",
    "moradoras": "morador",
    "moradores": "morador",
    "morados": "morar",
    "moraduras": "moradura",
    "moragas": "moraga",
    "moralejas": "moraleja",
    "morales": "moral",
    "moralice": "moralizar",
    "moralicemos": "moralizar",
    "moralicen": "moralizar",
    "moralices": "moralizar",
    "moralicé": "moralizar",
    "moralicéis": "moralizar",
    "moralidades": "moralidad",
    "moralinas": "moralina",
    "moralismos": "moralismo",
    "moralistas": "moralista",
    "moraliza": "moralizar",
    "moralizaba": "moralizar",
    "moralizabais": "moralizar",
    "moralizaban": "moralizar",
    "moralizabas": "moralizar",
    "moralizaciones": "moralización",
    "moralizad": "moralizar",
    "moralizada": "moralizar",
    "moralizadas": "moralizar",
    "moralizado": "moralizar",
    "moralizadora": "moralizador",
    "moralizadoras": "moralizador",
    "moralizadores": "moralizador",
    "moralizados": "moralizar",
    "moralizamos": "moralizar",
    "moralizan": "moralizar",
    "moralizando": "moralizar",
    "moralizara": "moralizar",
    "moralizarais": "moralizar",
    "moralizaran": "moralizar",
    "moralizaras": "moralizar",
    "moralizare": "moralizar",
    "moralizareis": "moralizar",
    "moralizaremos": "moralizar",
    "moralizaren": "moralizar",
    "moralizares": "moralizar",
    "moralizaron": "moralizar",
    "moralizará": "moralizar",
    "moralizarán": "moralizar",
    "moralizarás": "moralizar",
    "moralizaré": "moralizar",
    "moralizaréis": "moralizar",
    "moralizaría": "moralizar",
    "moralizaríais": "moralizar",
    "moralizaríamos": "moralizar",
    "moralizarían": "moralizar",
    "moralizarías": "moralizar",
    "moralizas": "moralizar",
    "moralizase": "moralizar",
    "moralizaseis": "moralizar",
    "moralizasen": "moralizar",
    "moralizases": "moralizar",
    "moralizaste": "moralizar",
    "moralizasteis": "moralizar",
    "moralizo": "moralizar",
    "moralizábamos": "moralizar",
    "moralizáis": "moralizar",
    "moralizáramos": "moralizar",
    "moralizáremos": "moralizar",
    "moralizásemos": "moralizar",
    "moralizó": "moralizar",
    "moramos": "morar",
    "moran": "morar",
    "morando": "morar",
    "moranzas": "moranza",
    "morapios": "morapio",
    "morara": "morar",
    "morarais": "morar",
    "moraran": "morar",
    "moraras": "morar",
    "morare": "morar",
    "morareis": "morar",
    "moraremos": "morar",
    "moraren": "morar",
    "morares": "morar",
    "moraron": "morar",
    "morará": "morar",
    "morarán": "morar",
    "morarás": "morar",
    "moraré": "morar",
    "moraréis": "morar",
    "moraría": "morar",
    "moraríais": "morar",
    "moraríamos": "morar",
    "morarían": "morar",
    "morarías": "morar",
    "moras": "moro",
    "morase": "morar",
    "moraseis": "morar",
    "morasen": "morar",
    "morases": "morar",
    "moraste": "morar",
    "morasteis": "morar",
    "moratiniana": "moratiniano",
    "moratinianas": "moratiniano",
    "moratinianos": "moratiniano",
    "moratones": "moratón",
    "moratoria": "moratorio",
    "moratorias": "moratorio",
    "moratorios": "moratorio",
    "morava": "moravo",
    "moravas": "moravo",
    "moravos": "moravo",
    "morbideces": "morbidez",
    "morbididades": "morbididad",
    "morbilidades": "morbilidad",
    "morbos": "morbo",
    "morbosa": "morboso",
    "morbosas": "morboso",
    "morbosidades": "morbosidad",
    "morbosos": "morboso",
    "morbífica": "morbífico",
    "morbíficas": "morbífico",
    "morbíficos": "morbífico",
    "morcajos": "morcajo",
    "morcales": "morcal",
    "morceguilas": "morceguila",
    "morcellas": "morcella",
    "morcilla": "morcillo",
    "morcillas": "morcillo",
    "morcillera": "morcillero",
    "morcilleras": "morcillero",
    "morcilleros": "morcillero",
    "morcillones": "morcillón",
    "morcillos": "morcillo",
    "morcones": "morcón",
    "mordaces": "mordaz",
    "mordacidades": "mordacidad",
    "mordagas": "mordaga",
    "mordamos": "morder",
    "mordazas": "mordaza",
    "mordazes": "mordaz",
    "morded": "morder",
    "mordedora": "mordedor",
    "mordedoras": "mordedor",
    "mordedores": "mordedor",
    "mordeduras": "mordedura",
    "mordemos": "morder",
    "mordentados": "mordentado",
    "mordentes": "mordente",
    "morderemos": "morder",
    "morderá": "morder",
    "morderán": "morder",
    "morderás": "morder",
    "morderé": "morder",
    "morderéis": "morder",
    "mordería": "morder",
    "morderíais": "morder",
    "morderíamos": "morder",
    "morderían": "morder",
    "morderías": "morder",
    "mordica": "mordicar",
    "mordicaba": "mordicar",
    "mordicabais": "mordicar",
    "mordicaban": "mordicar",
    "mordicabas": "mordicar",
    "mordicaciones": "mordicación",
    "mordicad": "mordicar",
    "mordicada": "mordicar",
    "mordicadas": "mordicar",
    "mordicado": "mordicar",
    "mordicados": "mordicar",
    "mordicamos": "mordicar",
    "mordican": "mordicar",
    "mordicando": "mordicar",
    "mordicantes": "mordicante",
    "mordicara": "mordicar",
    "mordicarais": "mordicar",
    "mordicaran": "mordicar",
    "mordicaras": "mordicar",
    "mordicare": "mordicar",
    "mordicareis": "mordicar",
    "mordicaremos": "mordicar",
    "mordicaren": "mordicar",
    "mordicares": "mordicar",
    "mordicaron": "mordicar",
    "mordicará": "mordicar",
    "mordicarán": "mordicar",
    "mordicarás": "mordicar",
    "mordicaré": "mordicar",
    "mordicaréis": "mordicar",
    "mordicaría": "mordicar",
    "mordicaríais": "mordicar",
    "mordicaríamos": "mordicar",
    "mordicarían": "mordicar",
    "mordicarías": "mordicar",
    "mordicas": "mordicar",
    "mordicase": "mordicar",
    "mordicaseis": "mordicar",
    "mordicasen": "mordicar",
    "mordicases": "mordicar",
    "mordicaste": "mordicar",
    "mordicasteis": "mordicar",
    "mordicativa": "mordicativo",
    "mordicativas": "mordicativo",
    "mordicativos": "mordicativo",
    "mordico": "mordicar",
    "mordicábamos": "mordicar",
    "mordicáis": "mordicar",
    "mordicáramos": "mordicar",
    "mordicáremos": "mordicar",
    "mordicásemos": "mordicar",
    "mordicó": "mordicar",
    "mordida": "morder",
    "mordidas": "morder",
    "mordido": "morder",
    "mordidos": "morder",
    "mordiendo": "morder",
    "mordientes": "mordiente",
    "mordiera": "morder",
    "mordierais": "morder",
    "mordieran": "morder",
    "mordieras": "morder",
    "mordiere": "morder",
    "mordiereis": "morder",
    "mordieren": "morder",
    "mordieres": "morder",
    "mordieron": "morder",
    "mordiese": "morder",
    "mordieseis": "morder",
    "mordiesen": "morder",
    "mordieses": "morder",
    "mordihuíes": "mordihuí",
    "mordimientos": "mordimiento",
    "mordimos": "morder",
    "mordique": "mordicar",
    "mordiquemos": "mordicar",
    "mordiquen": "mordicar",
    "mordiques": "mordicar",
    "mordiqué": "mordicar",
    "mordiquéis": "mordicar",
    "mordisca": "mordiscar",
    "mordiscaba": "mordiscar",
    "mordiscabais": "mordiscar",
    "mordiscaban": "mordiscar",
    "mordiscabas": "mordiscar",
    "mordiscad": "mordiscar",
    "mordiscada": "mordiscar",
    "mordiscadas": "mordiscar",
    "mordiscado": "mordiscar",
    "mordiscados": "mordiscar",
    "mordiscamos": "mordiscar",
    "mordiscan": "mordiscar",
    "mordiscando": "mordiscar",
    "mordiscara": "mordiscar",
    "mordiscarais": "mordiscar",
    "mordiscaran": "mordiscar",
    "mordiscaras": "mordiscar",
    "mordiscare": "mordiscar",
    "mordiscareis": "mordiscar",
    "mordiscaremos": "mordiscar",
    "mordiscaren": "mordiscar",
    "mordiscares": "mordiscar",
    "mordiscaron": "mordiscar",
    "mordiscará": "mordiscar",
    "mordiscarán": "mordiscar",
    "mordiscarás": "mordiscar",
    "mordiscaré": "mordiscar",
    "mordiscaréis": "mordiscar",
    "mordiscaría": "mordiscar",
    "mordiscaríais": "mordiscar",
    "mordiscaríamos": "mordiscar",
    "mordiscarían": "mordiscar",
    "mordiscarías": "mordiscar",
    "mordiscas": "mordiscar",
    "mordiscase": "mordiscar",
    "mordiscaseis": "mordiscar",
    "mordiscasen": "mordiscar",
    "mordiscases": "mordiscar",
    "mordiscaste": "mordiscar",
    "mordiscasteis": "mordiscar",
    "mordisco": "mordiscar",
    "mordiscos": "mordisco",
    "mordiscábamos": "mordiscar",
    "mordiscáis": "mordiscar",
    "mordiscáramos": "mordiscar",
    "mordiscáremos": "mordiscar",
    "mordiscásemos": "mordiscar",
    "mordiscó": "mordiscar",
    "mordisque": "mordiscar",
    "mordisquea": "mordisquear",
    "mordisqueaba": "mordisquear",
    "mordisqueabais": "mordisquear",
    "mordisqueaban": "mordisquear",
    "mordisqueabas": "mordisquear",
    "mordisquead": "mordisquear",
    "mordisqueada": "mordisquear",
    "mordisqueadas": "mordisquear",
    "mordisqueado": "mordisquear",
    "mordisqueados": "mordisquear",
    "mordisqueamos": "mordisquear",
    "mordisquean": "mordisquear",
    "mordisqueando": "mordisquear",
    "mordisqueara": "mordisquear",
    "mordisquearais": "mordisquear",
    "mordisquearan": "mordisquear",
    "mordisquearas": "mordisquear",
    "mordisqueare": "mordisquear",
    "mordisqueareis": "mordisquear",
    "mordisquearemos": "mordisquear",
    "mordisquearen": "mordisquear",
    "mordisqueares": "mordisquear",
    "mordisquearon": "mordisquear",
    "mordisqueará": "mordisquear",
    "mordisquearán": "mordisquear",
    "mordisquearás": "mordisquear",
    "mordisquearé": "mordisquear",
    "mordisquearéis": "mordisquear",
    "mordisquearía": "mordisquear",
    "mordisquearíais": "mordisquear",
    "mordisquearíamos": "mordisquear",
    "mordisquearían": "mordisquear",
    "mordisquearías": "mordisquear",
    "mordisqueas": "mordisquear",
    "mordisquease": "mordisquear",
    "mordisqueaseis": "mordisquear",
    "mordisqueasen": "mordisquear",
    "mordisqueases": "mordisquear",
    "mordisqueaste": "mordisquear",
    "mordisqueasteis": "mordisquear",
    "mordisquee": "mordisquear",
    "mordisqueemos": "mordisquear",
    "mordisqueen": "mordisquear",
    "mordisquees": "mordisquear",
    "mordisquemos": "mordiscar",
    "mordisquen": "mordiscar",
    "mordisqueo": "mordisquear",
    "mordisques": "mordiscar",
    "mordisqueábamos": "mordisquear",
    "mordisqueáis": "mordisquear",
    "mordisqueáramos": "mordisquear",
    "mordisqueáremos": "mordisquear",
    "mordisqueásemos": "mordisquear",
    "mordisqueé": "mordisquear",
    "mordisqueéis": "mordisquear",
    "mordisqueó": "mordisquear",
    "mordisqué": "mordiscar",
    "mordisquéis": "mordiscar",
    "mordiste": "morder",
    "mordisteis": "morder",
    "mordiéramos": "morder",
    "mordiéremos": "morder",
    "mordiésemos": "morder",
    "mordió": "morder",
    "mordáis": "morder",
    "mordéis": "morder",
    "mordí": "morder",
    "mordía": "morder",
    "mordíais": "morder",
    "mordíamos": "morder",
    "mordían": "morder",
    "mordías": "morder",
    "more": "morar",
    "moremos": "morar",
    "moren": "morar",
    "morena": "moreno",
    "morenas": "moreno",
    "moreneces": "morenez",
    "morenos": "moreno",
    "morenuras": "morenura",
    "morerales": "moreral",
    "moreras": "morera",
    "morerías": "morería",
    "mores": "morar",
    "moretones": "moretón",
    "morfemas": "morfema",
    "morfemática": "morfemático",
    "morfemáticas": "morfemático",
    "morfemáticos": "morfemático",
    "morfinas": "morfina",
    "morfinomanías": "morfinomanía",
    "morfinómana": "morfinómano",
    "morfinómanas": "morfinómano",
    "morfinómanos": "morfinómano",
    "morfismos": "morfismo",
    "morfoclimática": "morfoclimático",
    "morfoclimáticas": "morfoclimático",
    "morfoclimáticos": "morfoclimático",
    "morfofonemas": "morfofonema",
    "morfofonemática": "morfofonemático",
    "morfofonemáticas": "morfofonemático",
    "morfofonemáticos": "morfofonemático",
    "morfofonologías": "morfofonología",
    "morfofonológica": "morfofonológico",
    "morfofonológicas": "morfofonológico",
    "morfofonológicos": "morfofonológico",
    "morfogenética": "morfogenético",
    "morfogenéticas": "morfogenético",
    "morfogenéticos": "morfogenético",
    "morfográfica": "morfográfico",
    "morfográficas": "morfográfico",
    "morfográficos": "morfográfico",
    "morfolinas": "morfolina",
    "morfologías": "morfología",
    "morfológica": "morfológico",
    "morfológicas": "morfológico",
    "morfológicos": "morfológico",
    "morfometrías": "morfometría",
    "morfométrica": "morfométrico",
    "morfométricas": "morfométrico",
    "morfométricos": "morfométrico",
    "morfonemas": "morfonema",
    "morfonologías": "morfonología",
    "morfonológica": "morfonológico",
    "morfonológicas": "morfonológico",
    "morfonológicos": "morfonológico",
    "morfoplasmas": "morfoplasma",
    "morfos": "morfo",
    "morfosintáctica": "morfosintáctico",
    "morfosintácticas": "morfosintáctico",
    "morfosintácticos": "morfosintáctico",
    "morfotectónica": "morfotectónico",
    "morfotectónicas": "morfotectónico",
    "morfotectónicos": "morfotectónico",
    "morfémica": "morfémico",
    "morfémicas": "morfémico",
    "morfémicos": "morfémico",
    "morfóloga": "morfólogo",
    "morfólogas": "morfólogo",
    "morfólogos": "morfólogo",
    "morganitas": "morganita",
    "morganática": "morganático",
    "morganáticas": "morganático",
    "morganáticos": "morganático",
    "morgas": "morga",
    "morgones": "morgón",
    "morgues": "morgue",
    "moribunda": "moribundo",
    "moribundas": "moribundo",
    "moribundos": "moribundo",
    "morichales": "morichal",
    "moriches": "moriche",
    "morid": "morir",
    "morideras": "moridera",
    "morigera": "morigerar",
    "morigeraba": "morigerar",
    "morigerabais": "morigerar",
    "morigeraban": "morigerar",
    "morigerabas": "morigerar",
    "morigeraciones": "morigeración",
    "morigerad": "morigerar",
    "morigerada": "morigerar",
    "morigeradas": "morigerar",
    "morigerado": "morigerar",
    "morigerados": "morigerar",
    "morigeramos": "morigerar",
    "morigeran": "morigerar",
    "morigerando": "morigerar",
    "morigerara": "morigerar",
    "morigerarais": "morigerar",
    "morigeraran": "morigerar",
    "morigeraras": "morigerar",
    "morigerare": "morigerar",
    "morigerareis": "morigerar",
    "morigeraremos": "morigerar",
    "morigeraren": "morigerar",
    "morigerares": "morigerar",
    "morigeraron": "morigerar",
    "morigerará": "morigerar",
    "morigerarán": "morigerar",
    "morigerarás": "morigerar",
    "morigeraré": "morigerar",
    "morigeraréis": "morigerar",
    "morigeraría": "morigerar",
    "morigeraríais": "morigerar",
    "morigeraríamos": "morigerar",
    "morigerarían": "morigerar",
    "morigerarías": "morigerar",
    "morigeras": "morigerar",
    "morigerase": "morigerar",
    "morigeraseis": "morigerar",
    "morigerasen": "morigerar",
    "morigerases": "morigerar",
    "morigeraste": "morigerar",
    "morigerasteis": "morigerar",
    "morigere": "morigerar",
    "morigeremos": "morigerar",
    "morigeren": "morigerar",
    "morigeres": "morigerar",
    "morigero": "morigerar",
    "morigerábamos": "morigerar",
    "morigeráis": "morigerar",
    "morigeráramos": "morigerar",
    "morigeráremos": "morigerar",
    "morigerásemos": "morigerar",
    "morigeré": "morigerar",
    "morigeréis": "morigerar",
    "morigeró": "morigerar",
    "morillas": "morilla",
    "morillera": "morillero",
    "morilleras": "morillero",
    "morilleros": "morillero",
    "morillos": "morillo",
    "morimos": "morir",
    "morionda": "moriondo",
    "moriondas": "moriondo",
    "moriondos": "moriondo",
    "moriremos": "morir",
    "morirá": "morir",
    "morirán": "morir",
    "morirás": "morir",
    "moriré": "morir",
    "moriréis": "morir",
    "moriría": "morir",
    "moriríais": "morir",
    "moriríamos": "morir",
    "morirían": "morir",
    "morirías": "morir",
    "morisca": "morisco",
    "moriscas": "morisco",
    "moriscos": "morisco",
    "morismas": "morisma",
    "morisquetas": "morisqueta",
    "moriste": "morir",
    "moristeis": "morir",
    "moritos": "morito",
    "morlaca": "morlaco",
    "morlacas": "morlaco",
    "morlacos": "morlaco",
    "morlones": "morlón",
    "mormona": "mormón",
    "mormonas": "mormón",
    "mormones": "mormón",
    "mormonismos": "mormonismo",
    "mormullos": "mormullo",
    "mormónica": "mormónico",
    "mormónicas": "mormónico",
    "mormónicos": "mormónico",
    "moro": "morar",
    "moroncha": "moroncho",
    "moronchas": "moroncho",
    "moronchos": "moroncho",
    "moronda": "morondo",
    "morondangas": "morondanga",
    "morondas": "morondo",
    "morondos": "morondo",
    "morones": "morón",
    "moronías": "moronía",
    "moros": "moro",
    "morosa": "moroso",
    "morosas": "moroso",
    "morosidades": "morosidad",
    "morosos": "moroso",
    "morradas": "morrada",
    "morraladas": "morralada",
    "morrales": "morral",
    "morrallas": "morralla",
    "morras": "morra",
    "morrenas": "morrena",
    "morreos": "morreo",
    "morrillos": "morrillo",
    "morriones": "morrión",
    "morriñas": "morriña",
    "morriñosa": "morriñoso",
    "morriñosas": "morriñoso",
    "morriñosos": "morriñoso",
    "morrocotuda": "morrocotudo",
    "morrocotudas": "morrocotudo",
    "morrocotudos": "morrocotudo",
    "morrones": "morrón",
    "morronga": "morrongo",
    "morrongas": "morrongo",
    "morrongos": "morrongo",
    "morros": "morro",
    "morroña": "morroño",
    "morroñas": "morroño",
    "morroños": "morroño",
    "morruda": "morrudo",
    "morrudas": "morrudo",
    "morrudos": "morrudo",
    "morrénica": "morrénico",
    "morrénicas": "morrénico",
    "morrénicos": "morrénico",
    "morsanas": "morsana",
    "morsas": "morsa",
    "mortadelas": "mortadela",
    "mortajadora": "mortajador",
    "mortajadoras": "mortajadora",
    "mortajadores": "mortajador",
    "mortajados": "mortajado",
    "mortajas": "mortaja",
    "mortales": "mortal",
    "mortalidades": "mortalidad",
    "mortandades": "mortandad",
    "mortecina": "mortecino",
    "mortecinas": "mortecino",
    "mortecinos": "mortecino",
    "morteradas": "morterada",
    "morteras": "mortera",
    "morterazos": "morterazo",
    "morteretes": "morterete",
    "morteros": "mortero",
    "morteruelos": "morteruelo",
    "mortifica": "mortificar",
    "mortificaba": "mortificar",
    "mortificabais": "mortificar",
    "mortificaban": "mortificar",
    "mortificabas": "mortificar",
    "mortificaciones": "mortificación",
    "mortificad": "mortificar",
    "mortificada": "mortificar",
    "mortificadas": "mortificar",
    "mortificado": "mortificar",
    "mortificadora": "mortificador",
    "mortificadoras": "mortificador",
    "mortificadores": "mortificador",
    "mortificados": "mortificar",
    "mortificamos": "mortificar",
    "mortifican": "mortificar",
    "mortificando": "mortificar",
    "mortificara": "mortificar",
    "mortificarais": "mortificar",
    "mortificaran": "mortificar",
    "mortificaras": "mortificar",
    "mortificare": "mortificar",
    "mortificareis": "mortificar",
    "mortificaremos": "mortificar",
    "mortificaren": "mortificar",
    "mortificares": "mortificar",
    "mortificaron": "mortificar",
    "mortificará": "mortificar",
    "mortificarán": "mortificar",
    "mortificarás": "mortificar",
    "mortificaré": "mortificar",
    "mortificaréis": "mortificar",
    "mortificaría": "mortificar",
    "mortificaríais": "mortificar",
    "mortificaríamos": "mortificar",
    "mortificarían": "mortificar",
    "mortificarías": "mortificar",
    "mortificas": "mortificar",
    "mortificase": "mortificar",
    "mortificaseis": "mortificar",
    "mortificasen": "mortificar",
    "mortificases": "mortificar",
    "mortificaste": "mortificar",
    "mortificasteis": "mortificar",
    "mortifico": "mortificar",
    "mortificábamos": "mortificar",
    "mortificáis": "mortificar",
    "mortificáramos": "mortificar",
    "mortificáremos": "mortificar",
    "mortificásemos": "mortificar",
    "mortificó": "mortificar",
    "mortifique": "mortificar",
    "mortifiquemos": "mortificar",
    "mortifiquen": "mortificar",
    "mortifiques": "mortificar",
    "mortifiqué": "mortificar",
    "mortifiquéis": "mortificar",
    "mortinata": "mortinato",
    "mortinatalidades": "mortinatalidad",
    "mortinatas": "mortinato",
    "mortinatos": "mortinato",
    "mortuoria": "mortuorio",
    "mortuorias": "mortuorio",
    "mortuorios": "mortuorio",
    "mortífera": "mortífero",
    "mortíferas": "mortífero",
    "mortíferos": "mortífero",
    "moruchos": "morucho",
    "moruecos": "morueco",
    "moruna": "moruno",
    "morunas": "moruno",
    "morunos": "moruno",
    "morábamos": "morar",
    "moráis": "morar",
    "moráramos": "morar",
    "moráremos": "morar",
    "morásemos": "morar",
    "moré": "morar",
    "moréis": "morar",
    "morí": "morir",
    "moría": "morir",
    "moríais": "morir",
    "moríamos": "morir",
    "morían": "morir",
    "morías": "morir",
    "morís": "morir",
    "moró": "morar",
    "mosaica": "mosaico",
    "mosaicas": "mosaico",
    "mosaicos": "mosaico",
    "mosaísmos": "mosaísmo",
    "mosaístas": "mosaísta",
    "moscaderos": "moscadero",
    "moscardas": "moscarda",
    "moscardones": "moscardón",
    "moscas": "mosca",
    "moscateles": "moscatel",
    "mosconas": "moscona",
    "mosconea": "mosconear",
    "mosconeaba": "mosconear",
    "mosconeabais": "mosconear",
    "mosconeaban": "mosconear",
    "mosconeabas": "mosconear",
    "mosconead": "mosconear",
    "mosconeada": "mosconear",
    "mosconeadas": "mosconear",
    "mosconeado": "mosconear",
    "mosconeados": "mosconear",
    "mosconeamos": "mosconear",
    "mosconean": "mosconear",
    "mosconeando": "mosconear",
    "mosconeara": "mosconear",
    "mosconearais": "mosconear",
    "mosconearan": "mosconear",
    "mosconearas": "mosconear",
    "mosconeare": "mosconear",
    "mosconeareis": "mosconear",
    "mosconearemos": "mosconear",
    "mosconearen": "mosconear",
    "mosconeares": "mosconear",
    "mosconearon": "mosconear",
    "mosconeará": "mosconear",
    "mosconearán": "mosconear",
    "mosconearás": "mosconear",
    "mosconearé": "mosconear",
    "mosconearéis": "mosconear",
    "mosconearía": "mosconear",
    "mosconearíais": "mosconear",
    "mosconearíamos": "mosconear",
    "mosconearían": "mosconear",
    "mosconearías": "mosconear",
    "mosconeas": "mosconear",
    "mosconease": "mosconear",
    "mosconeaseis": "mosconear",
    "mosconeasen": "mosconear",
    "mosconeases": "mosconear",
    "mosconeaste": "mosconear",
    "mosconeasteis": "mosconear",
    "mosconee": "mosconear",
    "mosconeemos": "mosconear",
    "mosconeen": "mosconear",
    "mosconees": "mosconear",
    "mosconeo": "mosconear",
    "mosconeos": "mosconeo",
    "moscones": "moscón",
    "mosconeábamos": "mosconear",
    "mosconeáis": "mosconear",
    "mosconeáramos": "mosconear",
    "mosconeáremos": "mosconear",
    "mosconeásemos": "mosconear",
    "mosconeé": "mosconear",
    "mosconeéis": "mosconear",
    "mosconeó": "mosconear",
    "moscovitas": "moscovita",
    "mosenes": "mosén",
    "mosquea": "mosquear",
    "mosqueaba": "mosquear",
    "mosqueabais": "mosquear",
    "mosqueaban": "mosquear",
    "mosqueabas": "mosquear",
    "mosquead": "mosquear",
    "mosqueada": "mosquear",
    "mosqueadas": "mosquear",
    "mosqueado": "mosquear",
    "mosqueadores": "mosqueador",
    "mosqueados": "mosquear",
    "mosqueamos": "mosquear",
    "mosquean": "mosquear",
    "mosqueando": "mosquear",
    "mosqueara": "mosquear",
    "mosquearais": "mosquear",
    "mosquearan": "mosquear",
    "mosquearas": "mosquear",
    "mosqueare": "mosquear",
    "mosqueareis": "mosquear",
    "mosquearemos": "mosquear",
    "mosquearen": "mosquear",
    "mosqueares": "mosquear",
    "mosquearon": "mosquear",
    "mosqueará": "mosquear",
    "mosquearán": "mosquear",
    "mosquearás": "mosquear",
    "mosquearé": "mosquear",
    "mosquearéis": "mosquear",
    "mosquearía": "mosquear",
    "mosquearíais": "mosquear",
    "mosquearíamos": "mosquear",
    "mosquearían": "mosquear",
    "mosquearías": "mosquear",
    "mosqueas": "mosquear",
    "mosquease": "mosquear",
    "mosqueaseis": "mosquear",
    "mosqueasen": "mosquear",
    "mosqueases": "mosquear",
    "mosqueaste": "mosquear",
    "mosqueasteis": "mosquear",
    "mosquee": "mosquear",
    "mosqueemos": "mosquear",
    "mosqueen": "mosquear",
    "mosquees": "mosquear",
    "mosqueo": "mosquear",
    "mosqueos": "mosqueo",
    "mosqueros": "mosquero",
    "mosquetazos": "mosquetazo",
    "mosquetera": "mosquetero",
    "mosqueteras": "mosquetero",
    "mosqueteriles": "mosqueteril",
    "mosqueteros": "mosquetero",
    "mosqueterías": "mosquetería",
    "mosquetes": "mosquete",
    "mosquetones": "mosquetón",
    "mosqueábamos": "mosquear",
    "mosqueáis": "mosquear",
    "mosqueáramos": "mosquear",
    "mosqueáremos": "mosquear",
    "mosqueásemos": "mosquear",
    "mosqueé": "mosquear",
    "mosqueéis": "mosquear",
    "mosqueó": "mosquear",
    "mosquitera": "mosquitero",
    "mosquiteras": "mosquitero",
    "mosquiteros": "mosquitero",
    "mosquitos": "mosquito",
    "mostaceras": "mostacera",
    "mostaceros": "mostacero",
    "mostachones": "mostachón",
    "mostachos": "mostacho",
    "mostachosa": "mostachoso",
    "mostachosas": "mostachoso",
    "mostachosos": "mostachoso",
    "mostacillas": "mostacilla",
    "mostaganes": "mostagán",
    "mostazales": "mostazal",
    "mostazas": "mostaza",
    "mostazos": "mostazo",
    "mostea": "mostear",
    "mosteaba": "mostear",
    "mosteabais": "mostear",
    "mosteaban": "mostear",
    "mosteabas": "mostear",
    "mostead": "mostear",
    "mosteada": "mostear",
    "mosteadas": "mostear",
    "mosteado": "mostear",
    "mosteados": "mostear",
    "mosteamos": "mostear",
    "mostean": "mostear",
    "mosteando": "mostear",
    "mosteara": "mostear",
    "mostearais": "mostear",
    "mostearan": "mostear",
    "mostearas": "mostear",
    "mosteare": "mostear",
    "mosteareis": "mostear",
    "mostearemos": "mostear",
    "mostearen": "mostear",
    "mosteares": "mostear",
    "mostearon": "mostear",
    "mosteará": "mostear",
    "mostearán": "mostear",
    "mostearás": "mostear",
    "mostearé": "mostear",
    "mostearéis": "mostear",
    "mostearía": "mostear",
    "mostearíais": "mostear",
    "mostearíamos": "mostear",
    "mostearían": "mostear",
    "mostearías": "mostear",
    "mosteas": "mostear",
    "mostease": "mostear",
    "mosteaseis": "mostear",
    "mosteasen": "mostear",
    "mosteases": "mostear",
    "mosteaste": "mostear",
    "mosteasteis": "mostear",
    "mostee": "mostear",
    "mosteemos": "mostear",
    "mosteen": "mostear",
    "mostees": "mostear",
    "mostelas": "mostela",
    "mosteleras": "mostelera",
    "mosteo": "mostear",
    "mosteábamos": "mostear",
    "mosteáis": "mostear",
    "mosteáramos": "mostear",
    "mosteáremos": "mostear",
    "mosteásemos": "mostear",
    "mosteé": "mostear",
    "mosteéis": "mostear",
    "mosteó": "mostear",
    "mostillos": "mostillo",
    "mostos": "mosto",
    "mostraba": "mostrar",
    "mostrabais": "mostrar",
    "mostraban": "mostrar",
    "mostrabas": "mostrar",
    "mostrad": "mostrar",
    "mostrada": "mostrar",
    "mostradas": "mostrar",
    "mostrado": "mostrar",
    "mostradora": "mostrador",
    "mostradoras": "mostrador",
    "mostradores": "mostrador",
    "mostrados": "mostrar",
    "mostramos": "mostrar",
    "mostrando": "mostrar",
    "mostrara": "mostrar",
    "mostrarais": "mostrar",
    "mostraran": "mostrar",
    "mostraras": "mostrar",
    "mostrare": "mostrar",
    "mostrareis": "mostrar",
    "mostraremos": "mostrar",
    "mostraren": "mostrar",
    "mostrares": "mostrar",
    "mostraron": "mostrar",
    "mostrará": "mostrar",
    "mostrarán": "mostrar",
    "mostrarás": "mostrar",
    "mostraré": "mostrar",
    "mostraréis": "mostrar",
    "mostraría": "mostrar",
    "mostraríais": "mostrar",
    "mostraríamos": "mostrar",
    "mostrarían": "mostrar",
    "mostrarías": "mostrar",
    "mostrase": "mostrar",
    "mostraseis": "mostrar",
    "mostrasen": "mostrar",
    "mostrases": "mostrar",
    "mostraste": "mostrar",
    "mostrasteis": "mostrar",
    "mostremos": "mostrar",
    "mostrenca": "mostrenco",
    "mostrencas": "mostrenco",
    "mostrencos": "mostrenco",
    "mostrábamos": "mostrar",
    "mostráis": "mostrar",
    "mostráramos": "mostrar",
    "mostráremos": "mostrar",
    "mostrásemos": "mostrar",
    "mostré": "mostrar",
    "mostréis": "mostrar",
    "mostró": "mostrar",
    "motardes": "motard",
    "motards": "motard",
    "motas": "mota",
    "motea": "motear",
    "moteaba": "motear",
    "moteabais": "motear",
    "moteaban": "motear",
    "moteabas": "motear",
    "motead": "motear",
    "moteada": "motear",
    "moteadas": "motear",
    "moteado": "motear",
    "moteados": "motear",
    "moteamos": "motear",
    "motean": "motear",
    "moteando": "motear",
    "moteara": "motear",
    "motearais": "motear",
    "motearan": "motear",
    "motearas": "motear",
    "moteare": "motear",
    "moteareis": "motear",
    "motearemos": "motear",
    "motearen": "motear",
    "moteares": "motear",
    "motearon": "motear",
    "moteará": "motear",
    "motearán": "motear",
    "motearás": "motear",
    "motearé": "motear",
    "motearéis": "motear",
    "motearía": "motear",
    "motearíais": "motear",
    "motearíamos": "motear",
    "motearían": "motear",
    "motearías": "motear",
    "moteas": "motear",
    "motease": "motear",
    "moteaseis": "motear",
    "moteasen": "motear",
    "moteases": "motear",
    "moteaste": "motear",
    "moteasteis": "motear",
    "motee": "motear",
    "moteemos": "motear",
    "moteen": "motear",
    "motees": "motear",
    "moteja": "motejar",
    "motejaba": "motejar",
    "motejabais": "motejar",
    "motejaban": "motejar",
    "motejabas": "motejar",
    "motejad": "motejar",
    "motejada": "motejar",
    "motejadas": "motejar",
    "motejado": "motejar",
    "motejadora": "motejador",
    "motejadoras": "motejador",
    "motejadores": "motejador",
    "motejados": "motejar",
    "motejamos": "motejar",
    "motejan": "motejar",
    "motejando": "motejar",
    "motejara": "motejar",
    "motejarais": "motejar",
    "motejaran": "motejar",
    "motejaras": "motejar",
    "motejare": "motejar",
    "motejareis": "motejar",
    "motejaremos": "motejar",
    "motejaren": "motejar",
    "motejares": "motejar",
    "motejaron": "motejar",
    "motejará": "motejar",
    "motejarán": "motejar",
    "motejarás": "motejar",
    "motejaré": "motejar",
    "motejaréis": "motejar",
    "motejaría": "motejar",
    "motejaríais": "motejar",
    "motejaríamos": "motejar",
    "motejarían": "motejar",
    "motejarías": "motejar",
    "motejas": "motejar",
    "motejase": "motejar",
    "motejaseis": "motejar",
    "motejasen": "motejar",
    "motejases": "motejar",
    "motejaste": "motejar",
    "motejasteis": "motejar",
    "moteje": "motejar",
    "motejemos": "motejar",
    "motejen": "motejar",
    "motejes": "motejar",
    "motejo": "motejar",
    "motejos": "motejo",
    "motejábamos": "motejar",
    "motejáis": "motejar",
    "motejáramos": "motejar",
    "motejáremos": "motejar",
    "motejásemos": "motejar",
    "motejé": "motejar",
    "motejéis": "motejar",
    "motejó": "motejar",
    "moteles": "motel",
    "moteo": "motear",
    "motera": "motero",
    "moteras": "motero",
    "moteros": "motero",
    "motes": "mote",
    "motetes": "motete",
    "moteábamos": "motear",
    "moteáis": "motear",
    "moteáramos": "motear",
    "moteáremos": "motear",
    "moteásemos": "motear",
    "moteé": "motear",
    "moteéis": "motear",
    "moteó": "motear",
    "motila": "motilar",
    "motilaba": "motilar",
    "motilabais": "motilar",
    "motilaban": "motilar",
    "motilabas": "motilar",
    "motilad": "motilar",
    "motilada": "motilar",
    "motiladas": "motilar",
    "motilado": "motilar",
    "motilados": "motilar",
    "motilamos": "motilar",
    "motilan": "motilar",
    "motilando": "motilar",
    "motilara": "motilar",
    "motilarais": "motilar",
    "motilaran": "motilar",
    "motilaras": "motilar",
    "motilare": "motilar",
    "motilareis": "motilar",
    "motilaremos": "motilar",
    "motilaren": "motilar",
    "motilares": "motilar",
    "motilaron": "motilar",
    "motilará": "motilar",
    "motilarán": "motilar",
    "motilarás": "motilar",
    "motilaré": "motilar",
    "motilaréis": "motilar",
    "motilaría": "motilar",
    "motilaríais": "motilar",
    "motilaríamos": "motilar",
    "motilarían": "motilar",
    "motilarías": "motilar",
    "motilas": "motilar",
    "motilase": "motilar",
    "motilaseis": "motilar",
    "motilasen": "motilar",
    "motilases": "motilar",
    "motilaste": "motilar",
    "motilasteis": "motilar",
    "motile": "motilar",
    "motilemos": "motilar",
    "motilen": "motilar",
    "motiles": "motilar",
    "motilidades": "motilidad",
    "motilo": "motilar",
    "motilona": "motilón",
    "motilonas": "motilón",
    "motilones": "motilón",
    "motilábamos": "motilar",
    "motiláis": "motilar",
    "motiláramos": "motilar",
    "motiláremos": "motilar",
    "motilásemos": "motilar",
    "motilé": "motilar",
    "motiléis": "motilar",
    "motiló": "motilar",
    "motines": "motín",
    "motiva": "motivo",
    "motivaba": "motivar",
    "motivabais": "motivar",
    "motivaban": "motivar",
    "motivabas": "motivar",
    "motivacionales": "motivacional",
    "motivaciones": "motivación",
    "motivad": "motivar",
    "motivada": "motivar",
    "motivadas": "motivar",
    "motivado": "motivar",
    "motivadora": "motivador",
    "motivadoras": "motivador",
    "motivadores": "motivador",
    "motivados": "motivar",
    "motivamos": "motivar",
    "motivan": "motivar",
    "motivando": "motivar",
    "motivara": "motivar",
    "motivarais": "motivar",
    "motivaran": "motivar",
    "motivaras": "motivar",
    "motivare": "motivar",
    "motivareis": "motivar",
    "motivaremos": "motivar",
    "motivaren": "motivar",
    "motivares": "motivar",
    "motivaron": "motivar",
    "motivará": "motivar",
    "motivarán": "motivar",
    "motivarás": "motivar",
    "motivaré": "motivar",
    "motivaréis": "motivar",
    "motivaría": "motivar",
    "motivaríais": "motivar",
    "motivaríamos": "motivar",
    "motivarían": "motivar",
    "motivarías": "motivar",
    "motivas": "motivo",
    "motivase": "motivar",
    "motivaseis": "motivar",
    "motivasen": "motivar",
    "motivases": "motivar",
    "motivaste": "motivar",
    "motivasteis": "motivar",
    "motive": "motivar",
    "motivemos": "motivar",
    "motiven": "motivar",
    "motives": "motivar",
    "motivo": "motivar",
    "motivos": "motivo",
    "motivábamos": "motivar",
    "motiváis": "motivar",
    "motiváramos": "motivar",
    "motiváremos": "motivar",
    "motivásemos": "motivar",
    "motivé": "motivar",
    "motivéis": "motivar",
    "motivó": "motivar",
    "motoaspiradoras": "motoaspiradora",
    "motoazadas": "motoazada",
    "motobombas": "motobomba",
    "motocarretillas": "motocarretilla",
    "motocarros": "motocarro",
    "motocicletas": "motocicleta",
    "motociclismos": "motociclismo",
    "motociclistas": "motociclista",
    "motociclos": "motociclo",
    "motocompresora": "motocompresor",
    "motocompresoras": "motocompresor",
    "motocompresores": "motocompresor",
    "motocultivadores": "motocultivador",
    "motocultivos": "motocultivo",
    "motocultores": "motocultor",
    "motonaves": "motonave",
    "motonerías": "motonería",
    "motones": "motón",
    "motoneuronas": "motoneurona",
    "motonieves": "motonieve",
    "motoniveladoras": "motoniveladora",
    "motonáutica": "motonáutico",
    "motonáuticas": "motonáutico",
    "motonáuticos": "motonáutico",
    "motora": "motor",
    "motoras": "motora",
    "motores": "motor",
    "motorice": "motorizar",
    "motoricemos": "motorizar",
    "motoricen": "motorizar",
    "motorices": "motorizar",
    "motoricé": "motorizar",
    "motoricéis": "motorizar",
    "motorismos": "motorismo",
    "motoristas": "motorista",
    "motoriza": "motorizar",
    "motorizaba": "motorizar",
    "motorizabais": "motorizar",
    "motorizaban": "motorizar",
    "motorizabas": "motorizar",
    "motorizaciones": "motorización",
    "motorizad": "motorizar",
    "motorizada": "motorizar",
    "motorizadas": "motorizar",
    "motorizado": "motorizar",
    "motorizados": "motorizar",
    "motorizamos": "motorizar",
    "motorizan": "motorizar",
    "motorizando": "motorizar",
    "motorizara": "motorizar",
    "motorizarais": "motorizar",
    "motorizaran": "motorizar",
    "motorizaras": "motorizar",
    "motorizare": "motorizar",
    "motorizareis": "motorizar",
    "motorizaremos": "motorizar",
    "motorizaren": "motorizar",
    "motorizares": "motorizar",
    "motorizaron": "motorizar",
    "motorizará": "motorizar",
    "motorizarán": "motorizar",
    "motorizarás": "motorizar",
    "motorizaré": "motorizar",
    "motorizaréis": "motorizar",
    "motorizaría": "motorizar",
    "motorizaríais": "motorizar",
    "motorizaríamos": "motorizar",
    "motorizarían": "motorizar",
    "motorizarías": "motorizar",
    "motorizas": "motorizar",
    "motorizase": "motorizar",
    "motorizaseis": "motorizar",
    "motorizasen": "motorizar",
    "motorizases": "motorizar",
    "motorizaste": "motorizar",
    "motorizasteis": "motorizar",
    "motorizo": "motorizar",
    "motorizábamos": "motorizar",
    "motorizáis": "motorizar",
    "motorizáramos": "motorizar",
    "motorizáremos": "motorizar",
    "motorizásemos": "motorizar",
    "motorizó": "motorizar",
    "motorrampas": "motorrampa",
    "motorreductores": "motorreductor",
    "motos": "moto",
    "motosegadoras": "motosegadora",
    "motosierras": "motosierra",
    "mototraíllas": "mototraílla",
    "motoveleros": "motovelero",
    "motrices": "motriz",
    "mousses": "mousse",
    "movamos": "mover",
    "moved": "mover",
    "movediza": "movedizo",
    "movedizas": "movedizo",
    "movedizos": "movedizo",
    "movedora": "movedor",
    "movedoras": "movedor",
    "movedores": "movedor",
    "movemos": "mover",
    "moveremos": "mover",
    "moverá": "mover",
    "moverán": "mover",
    "moverás": "mover",
    "moveré": "mover",
    "moveréis": "mover",
    "movería": "mover",
    "moveríais": "mover",
    "moveríamos": "mover",
    "moverían": "mover",
    "moverías": "mover",
    "movibles": "movible",
    "moviciones": "movición",
    "movida": "mover",
    "movidas": "movida",
    "movido": "mover",
    "movidos": "mover",
    "moviendo": "mover",
    "movientes": "moviente",
    "moviera": "mover",
    "movierais": "mover",
    "movieran": "mover",
    "movieras": "mover",
    "moviere": "mover",
    "moviereis": "mover",
    "movieren": "mover",
    "movieres": "mover",
    "movieron": "mover",
    "moviese": "mover",
    "movieseis": "mover",
    "moviesen": "mover",
    "movieses": "mover",
    "movilice": "movilizar",
    "movilicemos": "movilizar",
    "movilicen": "movilizar",
    "movilices": "movilizar",
    "movilicé": "movilizar",
    "movilicéis": "movilizar",
    "movilidades": "movilidad",
    "moviliza": "movilizar",
    "movilizaba": "movilizar",
    "movilizabais": "movilizar",
    "movilizaban": "movilizar",
    "movilizabas": "movilizar",
    "movilizables": "movilizable",
    "movilizaciones": "movilización",
    "movilizad": "movilizar",
    "movilizada": "movilizar",
    "movilizadas": "movilizar",
    "movilizado": "movilizar",
    "movilizados": "movilizar",
    "movilizamos": "movilizar",
    "movilizan": "movilizar",
    "movilizando": "movilizar",
    "movilizara": "movilizar",
    "movilizarais": "movilizar",
    "movilizaran": "movilizar",
    "movilizaras": "movilizar",
    "movilizare": "movilizar",
    "movilizareis": "movilizar",
    "movilizaremos": "movilizar",
    "movilizaren": "movilizar",
    "movilizares": "movilizar",
    "movilizaron": "movilizar",
    "movilizará": "movilizar",
    "movilizarán": "movilizar",
    "movilizarás": "movilizar",
    "movilizaré": "movilizar",
    "movilizaréis": "movilizar",
    "movilizaría": "movilizar",
    "movilizaríais": "movilizar",
    "movilizaríamos": "movilizar",
    "movilizarían": "movilizar",
    "movilizarías": "movilizar",
    "movilizas": "movilizar",
    "movilizase": "movilizar",
    "movilizaseis": "movilizar",
    "movilizasen": "movilizar",
    "movilizases": "movilizar",
    "movilizaste": "movilizar",
    "movilizasteis": "movilizar",
    "movilizo": "movilizar",
    "movilizábamos": "movilizar",
    "movilizáis": "movilizar",
    "movilizáramos": "movilizar",
    "movilizáremos": "movilizar",
    "movilizásemos": "movilizar",
    "movilizó": "movilizar",
    "movimientos": "movimiento",
    "movimos": "mover",
    "moviolas": "moviola",
    "moviste": "mover",
    "movisteis": "mover",
    "moviéramos": "mover",
    "moviéremos": "mover",
    "moviésemos": "mover",
    "movió": "mover",
    "mováis": "mover",
    "movéis": "mover",
    "moví": "mover",
    "movía": "mover",
    "movíais": "mover",
    "movíamos": "mover",
    "movían": "mover",
    "movías": "mover",
    "moxas": "moxa",
    "moyanas": "moyana",
    "moyuelos": "moyuelo",
    "moza": "mozo",
    "mozalbeta": "mozalbete",
    "mozalbetas": "mozalbete",
    "mozalbetes": "mozalbete",
    "mozalbilla": "mozalbillo",
    "mozalbillas": "mozalbillo",
    "mozalbillos": "mozalbillo",
    "mozallona": "mozallón",
    "mozallonas": "mozallón",
    "mozallones": "mozallón",
    "mozambiqueña": "mozambiqueño",
    "mozambiqueñas": "mozambiqueño",
    "mozambiqueños": "mozambiqueño",
    "mozancona": "mozancón",
    "mozanconas": "mozancón",
    "mozancones": "mozancón",
    "mozartiana": "mozartiano",
    "mozartianas": "mozartiano",
    "mozartianos": "mozartiano",
    "mozas": "mozo",
    "mozos": "mozo",
    "mozuela": "mozuelo",
    "mozuelas": "mozuelo",
    "mozuelos": "mozuelo",
    "mozzarellas": "mozzarella",
    "mozárabes": "mozárabe",
    "moña": "moño",
    "moñas": "moño",
    "moñistas": "moñista",
    "moñona": "moñón",
    "moñonas": "moñón",
    "moñones": "moñón",
    "moños": "moño",
    "moñuda": "moñudo",
    "moñudas": "moñudo",
    "moñudos": "moñudo",
    "ms": "milisegundo",
    "mtorolitas": "mtorolita",
    "muarés": "muaré",
    "mucama": "mucamo",
    "mucamas": "mucamo",
    "mucamos": "mucamo",
    "muces": "muz",
    "mucetas": "muceta",
    "mucha": "mucho",
    "muchacha": "muchacho",
    "muchachadas": "muchachada",
    "muchachas": "muchacho",
    "muchachea": "muchachear",
    "muchacheaba": "muchachear",
    "muchacheabais": "muchachear",
    "muchacheaban": "muchachear",
    "muchacheabas": "muchachear",
    "muchachead": "muchachear",
    "muchacheada": "muchachear",
    "muchacheadas": "muchachear",
    "muchacheado": "muchachear",
    "muchacheados": "muchachear",
    "muchacheamos": "muchachear",
    "muchachean": "muchachear",
    "muchacheando": "muchachear",
    "muchacheara": "muchachear",
    "muchachearais": "muchachear",
    "muchachearan": "muchachear",
    "muchachearas": "muchachear",
    "muchacheare": "muchachear",
    "muchacheareis": "muchachear",
    "muchachearemos": "muchachear",
    "muchachearen": "muchachear",
    "muchacheares": "muchachear",
    "muchachearon": "muchachear",
    "muchacheará": "muchachear",
    "muchachearán": "muchachear",
    "muchachearás": "muchachear",
    "muchachearé": "muchachear",
    "muchachearéis": "muchachear",
    "muchachearía": "muchachear",
    "muchachearíais": "muchachear",
    "muchachearíamos": "muchachear",
    "muchachearían": "muchachear",
    "muchachearías": "muchachear",
    "muchacheas": "muchachear",
    "muchachease": "muchachear",
    "muchacheaseis": "muchachear",
    "muchacheasen": "muchachear",
    "muchacheases": "muchachear",
    "muchacheaste": "muchachear",
    "muchacheasteis": "muchachear",
    "muchacheces": "muchachez",
    "muchachee": "muchachear",
    "muchacheemos": "muchachear",
    "muchacheen": "muchachear",
    "muchachees": "muchachear",
    "muchacheo": "muchachear",
    "muchacherías": "muchachería",
    "muchacheábamos": "muchachear",
    "muchacheáis": "muchachear",
    "muchacheáramos": "muchachear",
    "muchacheáremos": "muchachear",
    "muchacheásemos": "muchachear",
    "muchacheé": "muchachear",
    "muchacheéis": "muchachear",
    "muchacheó": "muchachear",
    "muchachiles": "muchachil",
    "muchachos": "muchacho",
    "muchas": "mucho",
    "muchedumbres": "muchedumbre",
    "muchos": "mucho",
    "mucilaginosa": "mucilaginoso",
    "mucilaginosas": "mucilaginoso",
    "mucilaginosos": "mucilaginoso",
    "mucilagos": "mucilago",
    "mucinas": "mucina",
    "mucinosa": "mucinoso",
    "mucinosas": "mucinoso",
    "mucinosos": "mucinoso",
    "mucoceles": "mucocele",
    "mucogingivales": "mucogingival",
    "mucolabiales": "mucolabial",
    "mucoondulatoria": "mucoondulatorio",
    "mucoondulatorias": "mucoondulatorio",
    "mucoondulatorios": "mucoondulatorio",
    "mucopolisacáridos": "mucopolisacárido",
    "mucoproteínas": "mucoproteína",
    "mucosa": "mucoso",
    "mucosas": "mucoso",
    "mucosidades": "mucosidad",
    "mucosos": "mucoso",
    "mucronata": "mucronato",
    "mucronatas": "mucronato",
    "mucronatos": "mucronato",
    "mucílagos": "mucílago",
    "muda": "mudo",
    "mudaba": "mudar",
    "mudabais": "mudar",
    "mudaban": "mudar",
    "mudabas": "mudar",
    "mudables": "mudable",
    "mudad": "mudar",
    "mudada": "mudar",
    "mudadas": "mudar",
    "mudadiza": "mudadizo",
    "mudadizas": "mudadizo",
    "mudadizos": "mudadizo",
    "mudado": "mudar",
    "mudados": "mudar",
    "mudamos": "mudar",
    "mudan": "mudar",
    "mudando": "mudar",
    "mudanzas": "mudanza",
    "mudara": "mudar",
    "mudarais": "mudar",
    "mudaran": "mudar",
    "mudaras": "mudar",
    "mudare": "mudar",
    "mudareis": "mudar",
    "mudaremos": "mudar",
    "mudaren": "mudar",
    "mudares": "mudar",
    "mudaron": "mudar",
    "mudará": "mudar",
    "mudarán": "mudar",
    "mudarás": "mudar",
    "mudaré": "mudar",
    "mudaréis": "mudar",
    "mudaría": "mudar",
    "mudaríais": "mudar",
    "mudaríamos": "mudar",
    "mudarían": "mudar",
    "mudarías": "mudar",
    "mudaríes": "mudarí",
    "mudas": "mudo",
    "mudase": "mudar",
    "mudaseis": "mudar",
    "mudasen": "mudar",
    "mudases": "mudar",
    "mudaste": "mudar",
    "mudasteis": "mudar",
    "mude": "mudar",
    "mudeces": "mudez",
    "mudemos": "mudar",
    "muden": "mudar",
    "mudes": "mudar",
    "mudo": "mudar",
    "mudos": "mudo",
    "mudábamos": "mudar",
    "mudáis": "mudar",
    "mudáramos": "mudar",
    "mudáremos": "mudar",
    "mudásemos": "mudar",
    "mudé": "mudar",
    "mudéis": "mudar",
    "mudéjares": "mudéjar",
    "mudó": "mudar",
    "muebla": "moblar",
    "mueblajes": "mueblaje",
    "mueblan": "moblar",
    "mueblas": "moblar",
    "mueble": "moblar",
    "mueblen": "moblar",
    "mueblerías": "mueblería",
    "muebles": "mueble",
    "mueblistas": "mueblista",
    "mueblo": "moblar",
    "muecas": "mueca",
    "muecines": "muecín",
    "muela": "moler",
    "muelan": "moler",
    "muelas": "muela",
    "muele": "moler",
    "muelen": "moler",
    "mueles": "moler",
    "muellajes": "muellaje",
    "muelles": "muelle",
    "muelo": "moler",
    "muelos": "muelo",
    "muera": "morir",
    "mueran": "morir",
    "mueras": "muera",
    "muerda": "morder",
    "muerdan": "morder",
    "muerdas": "morder",
    "muerde": "morder",
    "muerden": "morder",
    "muerdes": "morder",
    "muerdo": "morder",
    "muere": "morir",
    "mueren": "morir",
    "mueres": "morir",
    "muermos": "muermo",
    "muermosa": "muermoso",
    "muermosas": "muermoso",
    "muermosos": "muermoso",
    "muero": "morir",
    "muerta": "morir",
    "muertas": "muerta",
    "muertes": "muerte",
    "muerto": "morir",
    "muertos": "muerto",
    "muescas": "muesca",
    "muesos": "mueso",
    "muestra": "mostrar",
    "muestrales": "muestral",
    "muestran": "mostrar",
    "muestrarios": "muestrario",
    "muestras": "muestra",
    "muestre": "mostrar",
    "muestreadores": "muestreador",
    "muestren": "mostrar",
    "muestreos": "muestreo",
    "muestres": "mostrar",
    "muestro": "mostrar",
    "mueva": "mover",
    "muevan": "mover",
    "muevas": "mover",
    "mueve": "mover",
    "mueven": "mover",
    "mueves": "mover",
    "muevo": "mover",
    "muflas": "mufla",
    "muflones": "muflón",
    "muflónes": "muflón",
    "muftíes": "muftí",
    "mugadas": "mugada",
    "mugas": "muga",
    "muge": "mugir",
    "mugen": "mugir",
    "muges": "mugir",
    "mugid": "mugir",
    "mugida": "mugir",
    "mugidas": "mugir",
    "mugido": "mugir",
    "mugidora": "mugidor",
    "mugidoras": "mugidor",
    "mugidores": "mugidor",
    "mugidos": "mugir",
    "mugiendo": "mugir",
    "mugiera": "mugir",
    "mugierais": "mugir",
    "mugieran": "mugir",
    "mugieras": "mugir",
    "mugiere": "mugir",
    "mugiereis": "mugir",
    "mugieren": "mugir",
    "mugieres": "mugir",
    "mugieron": "mugir",
    "mugiese": "mugir",
    "mugieseis": "mugir",
    "mugiesen": "mugir",
    "mugieses": "mugir",
    "mugimos": "mugir",
    "mugiremos": "mugir",
    "mugirá": "mugir",
    "mugirán": "mugir",
    "mugirás": "mugir",
    "mugiré": "mugir",
    "mugiréis": "mugir",
    "mugiría": "mugir",
    "mugiríais": "mugir",
    "mugiríamos": "mugir",
    "mugirían": "mugir",
    "mugirías": "mugir",
    "mugiste": "mugir",
    "mugisteis": "mugir",
    "mugiéramos": "mugir",
    "mugiéremos": "mugir",
    "mugiésemos": "mugir",
    "mugió": "mugir",
    "mugores": "mugor",
    "mugres": "mugre",
    "mugrienta": "mugriento",
    "mugrientas": "mugriento",
    "mugrientos": "mugriento",
    "mugrones": "mugrón",
    "mugrosa": "mugroso",
    "mugrosas": "mugroso",
    "mugrosos": "mugroso",
    "muguetes": "muguete",
    "mugí": "mugir",
    "mugía": "mugir",
    "mugíais": "mugir",
    "mugíamos": "mugir",
    "mugían": "mugir",
    "mugías": "mugir",
    "mugís": "mugir",
    "muharras": "muharra",
    "muja": "mugir",
    "mujamos": "mugir",
    "mujan": "mugir",
    "mujas": "mugir",
    "mujercillas": "mujercilla",
    "mujeres": "mujer",
    "mujeriega": "mujeriego",
    "mujeriegas": "mujeriego",
    "mujeriegos": "mujeriego",
    "mujeriles": "mujeril",
    "mujeronas": "mujerona",
    "mujerzuelas": "mujerzuela",
    "mujeríos": "mujerío",
    "mujiks": "mujik",
    "mujo": "mugir",
    "mujáis": "mugir",
    "mula": "mulo",
    "muladares": "muladar",
    "muladas": "mulada",
    "muladíes": "muladí",
    "muladís": "muladí",
    "mulares": "mular",
    "mulas": "mulo",
    "mulata": "mulato",
    "mulatas": "mulato",
    "mulatera": "mulatero",
    "mulateras": "mulatero",
    "mulateros": "mulatero",
    "mulatos": "mulato",
    "mulera": "mulero",
    "muleras": "mulero",
    "muleros": "mulero",
    "muletadas": "muletada",
    "muletas": "muleta",
    "muletillas": "muletilla",
    "muletillera": "muletillero",
    "muletilleras": "muletillero",
    "muletilleros": "muletillero",
    "muletones": "muletón",
    "muletos": "muleto",
    "mulilingües": "mulilingüe",
    "mulillas": "mulilla",
    "mulla": "mullir",
    "mullamos": "mullir",
    "mullan": "mullir",
    "mullas": "mullir",
    "mulle": "mullir",
    "mullen": "mullir",
    "mullendo": "mullir",
    "mullera": "mullir",
    "mullerais": "mullir",
    "mulleran": "mullir",
    "mulleras": "mullir",
    "mullere": "mullir",
    "mullereis": "mullir",
    "mulleren": "mullir",
    "mulleres": "mullir",
    "mulleron": "mullir",
    "mulles": "mullir",
    "mullese": "mullir",
    "mulleseis": "mullir",
    "mullesen": "mullir",
    "mulleses": "mullir",
    "mullid": "mullir",
    "mullida": "mullir",
    "mullidas": "mullir",
    "mullido": "mullir",
    "mullidos": "mullir",
    "mullimos": "mullir",
    "mulliremos": "mullir",
    "mullirá": "mullir",
    "mullirán": "mullir",
    "mullirás": "mullir",
    "mulliré": "mullir",
    "mulliréis": "mullir",
    "mulliría": "mullir",
    "mulliríais": "mullir",
    "mulliríamos": "mullir",
    "mullirían": "mullir",
    "mullirías": "mullir",
    "mulliste": "mullir",
    "mullisteis": "mullir",
    "mullitas": "mullita",
    "mullo": "mullir",
    "mullos": "mullo",
    "mulláis": "mullir",
    "mulléramos": "mullir",
    "mulléremos": "mullir",
    "mullésemos": "mullir",
    "mullí": "mullir",
    "mullía": "mullir",
    "mullíais": "mullir",
    "mullíamos": "mullir",
    "mullían": "mullir",
    "mullías": "mullir",
    "mullís": "mullir",
    "mulló": "mullir",
    "mulos": "mulo",
    "mulsa": "mulso",
    "mulsas": "mulso",
    "mulsos": "mulso",
    "multa": "multar",
    "multaba": "multar",
    "multabais": "multar",
    "multaban": "multar",
    "multabas": "multar",
    "multad": "multar",
    "multada": "multar",
    "multadas": "multar",
    "multado": "multar",
    "multados": "multar",
    "multamos": "multar",
    "multan": "multar",
    "multando": "multar",
    "multara": "multar",
    "multarais": "multar",
    "multaran": "multar",
    "multaras": "multar",
    "multare": "multar",
    "multareis": "multar",
    "multaremos": "multar",
    "multaren": "multar",
    "multares": "multar",
    "multaron": "multar",
    "multará": "multar",
    "multarán": "multar",
    "multarás": "multar",
    "multaré": "multar",
    "multaréis": "multar",
    "multaría": "multar",
    "multaríais": "multar",
    "multaríamos": "multar",
    "multarían": "multar",
    "multarías": "multar",
    "multas": "multar",
    "multase": "multar",
    "multaseis": "multar",
    "multasen": "multar",
    "multases": "multar",
    "multaste": "multar",
    "multasteis": "multar",
    "multe": "multar",
    "multemos": "multar",
    "multen": "multar",
    "multes": "multar",
    "multialmacenes": "multialmacén",
    "multiangulares": "multiangular",
    "multianuales": "multianual",
    "multiaplicaciones": "multiaplicación",
    "multibuses": "multibús",
    "multicaderas": "multicadera",
    "multicanales": "multicanal",
    "multicapas": "multicapa",
    "multicascos": "multicasco",
    "multicaules": "multicaule",
    "multicelulares": "multicelular",
    "multicentros": "multicentro",
    "multicolinealidades": "multicolinealidad",
    "multicolores": "multicolor",
    "multicompuesta": "multicompuesto",
    "multicompuestas": "multicompuesto",
    "multicompuestos": "multicompuesto",
    "multicomputadores": "multicomputador",
    "multicopiados": "multicopiado",
    "multicopistas": "multicopista",
    "multiculturales": "multicultural",
    "multiculturalidades": "multiculturalidad",
    "multiculturalismos": "multiculturalismo",
    "multicéntrica": "multicéntrico",
    "multicéntricas": "multicéntrico",
    "multicéntricos": "multicéntrico",
    "multicúspides": "multicúspide",
    "multidestina": "multidestino",
    "multidestinas": "multidestino",
    "multidestinos": "multidestino",
    "multidimensiones": "multidimensión",
    "multidireccionales": "multidireccional",
    "multidisciplinares": "multidisciplinar",
    "multidisciplinaria": "multidisciplinario",
    "multidisciplinarias": "multidisciplinario",
    "multidisciplinarios": "multidisciplinario",
    "multielectródica": "multielectródico",
    "multielectródicas": "multielectródico",
    "multielectródicos": "multielectródico",
    "multiempresas": "multiempresa",
    "multiespectrales": "multiespectral",
    "multiespirales": "multiespiral",
    "multietapas": "multietapa",
    "multifilamentos": "multifilamento",
    "multifilares": "multifilar",
    "multifocales": "multifocal",
    "multiformes": "multiforme",
    "multifrecuencias": "multifrecuencia",
    "multifuncionales": "multifuncional",
    "multifuncionalidades": "multifuncionalidad",
    "multifásica": "multifásico",
    "multifásicas": "multifásico",
    "multifásicos": "multifásico",
    "multigraduales": "multigradual",
    "multiidiomas": "multiidioma",
    "multiinstrumentistas": "multiinstrumentista",
    "multilaterales": "multilateral",
    "multilateralismos": "multilateralismo",
    "multilineales": "multilineal",
    "multilingües": "multilingüe",
    "multilingüismos": "multilingüismo",
    "multilobulada": "multilobulado",
    "multilobuladas": "multilobulado",
    "multilobulados": "multilobulado",
    "multiloculares": "multilocular",
    "multilátera": "multilátero",
    "multiláteras": "multilátero",
    "multiláteros": "multilátero",
    "multimedias": "multimedia",
    "multimillonaria": "multimillonario",
    "multimillonarias": "multimillonario",
    "multimillonarios": "multimillonario",
    "multimodales": "multimodal",
    "multimotora": "multimotor",
    "multimotoras": "multimotor",
    "multimotores": "multimotor",
    "multinacionales": "multinacional",
    "multinacionalidades": "multinacionalidad",
    "multinomiales": "multinomial",
    "multinomios": "multinomio",
    "multipartes": "multiparte",
    "multipartitas": "multipartita",
    "multipenniformes": "multipenniforme",
    "multiperforaciones": "multiperforación",
    "multipistas": "multipista",
    "multiplanares": "multiplanar",
    "multiplexa": "multiplexar",
    "multiplexaba": "multiplexar",
    "multiplexabais": "multiplexar",
    "multiplexaban": "multiplexar",
    "multiplexabas": "multiplexar",
    "multiplexaciones": "multiplexación",
    "multiplexad": "multiplexar",
    "multiplexada": "multiplexar",
    "multiplexadas": "multiplexar",
    "multiplexado": "multiplexar",
    "multiplexados": "multiplexar",
    "multiplexamos": "multiplexar",
    "multiplexan": "multiplexar",
    "multiplexando": "multiplexar",
    "multiplexara": "multiplexar",
    "multiplexarais": "multiplexar",
    "multiplexaran": "multiplexar",
    "multiplexaras": "multiplexar",
    "multiplexare": "multiplexar",
    "multiplexareis": "multiplexar",
    "multiplexaremos": "multiplexar",
    "multiplexaren": "multiplexar",
    "multiplexares": "multiplexar",
    "multiplexaron": "multiplexar",
    "multiplexará": "multiplexar",
    "multiplexarán": "multiplexar",
    "multiplexarás": "multiplexar",
    "multiplexaré": "multiplexar",
    "multiplexaréis": "multiplexar",
    "multiplexaría": "multiplexar",
    "multiplexaríais": "multiplexar",
    "multiplexaríamos": "multiplexar",
    "multiplexarían": "multiplexar",
    "multiplexarías": "multiplexar",
    "multiplexas": "multiplexar",
    "multiplexase": "multiplexar",
    "multiplexaseis": "multiplexar",
    "multiplexasen": "multiplexar",
    "multiplexases": "multiplexar",
    "multiplexaste": "multiplexar",
    "multiplexasteis": "multiplexar",
    "multiplexe": "multiplexar",
    "multiplexemos": "multiplexar",
    "multiplexen": "multiplexar",
    "multiplexes": "multiplexar",
    "multiplexiones": "multiplexión",
    "multiplexo": "multiplexar",
    "multiplexores": "multiplexor",
    "multiplexábamos": "multiplexar",
    "multiplexáis": "multiplexar",
    "multiplexáramos": "multiplexar",
    "multiplexáremos": "multiplexar",
    "multiplexásemos": "multiplexar",
    "multiplexé": "multiplexar",
    "multiplexéis": "multiplexar",
    "multiplexó": "multiplexar",
    "multiplica": "multiplicar",
    "multiplicaba": "multiplicar",
    "multiplicabais": "multiplicar",
    "multiplicaban": "multiplicar",
    "multiplicabas": "multiplicar",
    "multiplicaciones": "multiplicación",
    "multiplicad": "multiplicar",
    "multiplicada": "multiplicar",
    "multiplicadas": "multiplicar",
    "multiplicado": "multiplicar",
    "multiplicadora": "multiplicador",
    "multiplicadoras": "multiplicador",
    "multiplicadores": "multiplicador",
    "multiplicados": "multiplicar",
    "multiplicamos": "multiplicar",
    "multiplican": "multiplicar",
    "multiplicando": "multiplicar",
    "multiplicandos": "multiplicando",
    "multiplicara": "multiplicar",
    "multiplicarais": "multiplicar",
    "multiplicaran": "multiplicar",
    "multiplicaras": "multiplicar",
    "multiplicare": "multiplicar",
    "multiplicareis": "multiplicar",
    "multiplicaremos": "multiplicar",
    "multiplicaren": "multiplicar",
    "multiplicares": "multiplicar",
    "multiplicaron": "multiplicar",
    "multiplicará": "multiplicar",
    "multiplicarán": "multiplicar",
    "multiplicarás": "multiplicar",
    "multiplicaré": "multiplicar",
    "multiplicaréis": "multiplicar",
    "multiplicaría": "multiplicar",
    "multiplicaríais": "multiplicar",
    "multiplicaríamos": "multiplicar",
    "multiplicarían": "multiplicar",
    "multiplicarías": "multiplicar",
    "multiplicas": "multiplicar",
    "multiplicase": "multiplicar",
    "multiplicaseis": "multiplicar",
    "multiplicasen": "multiplicar",
    "multiplicases": "multiplicar",
    "multiplicaste": "multiplicar",
    "multiplicasteis": "multiplicar",
    "multiplicativa": "multiplicativo",
    "multiplicativas": "multiplicativo",
    "multiplicativos": "multiplicativo",
    "multiplicidades": "multiplicidad",
    "multiplico": "multiplicar",
    "multiplicábamos": "multiplicar",
    "multiplicáis": "multiplicar",
    "multiplicáramos": "multiplicar",
    "multiplicáremos": "multiplicar",
    "multiplicásemos": "multiplicar",
    "multiplicó": "multiplicar",
    "multiplique": "multiplicar",
    "multipliquemos": "multiplicar",
    "multipliquen": "multiplicar",
    "multipliques": "multiplicar",
    "multipliqué": "multiplicar",
    "multipliquéis": "multiplicar",
    "multipolares": "multipolar",
    "multipolos": "multipolo",
    "multipowers": "multipower",
    "multiprocesadores": "multiprocesador",
    "multiprocesamientos": "multiprocesamiento",
    "multiprocesos": "multiproceso",
    "multiprogramaciones": "multiprogramación",
    "multipuertas": "multipuerta",
    "multipuestos": "multipuesto",
    "multipuntos": "multipunto",
    "multirraciales": "multirracial",
    "multirradiales": "multirradial",
    "multirradiculares": "multirradicular",
    "multisectoriales": "multisectorial",
    "multisimétrica": "multisimétrico",
    "multisimétricas": "multisimétrico",
    "multisimétricos": "multisimétrico",
    "multitareas": "multitarea",
    "multitramas": "multitrama",
    "multitratamientos": "multitratamiento",
    "multituberculados": "multituberculado",
    "multitubulares": "multitubular",
    "multitudes": "multitud",
    "multitudinaria": "multitudinario",
    "multitudinarias": "multitudinario",
    "multitudinarios": "multitudinario",
    "multiunitaria": "multiunitario",
    "multiunitarias": "multiunitario",
    "multiunitarios": "multiunitario",
    "multiusos": "multiuso",
    "multiusuaria": "multiusuario",
    "multiusuarias": "multiusuario",
    "multiusuarios": "multiusuario",
    "multivalentes": "multivalente",
    "multivariables": "multivariable",
    "multivariada": "multivariado",
    "multivariadas": "multivariado",
    "multivariados": "multivariado",
    "multivariantes": "multivariante",
    "multivesiculares": "multivesicular",
    "multivibradores": "multivibrador",
    "multivitaminas": "multivitamina",
    "multivoltajes": "multivoltaje",
    "multo": "multar",
    "multábamos": "multar",
    "multáis": "multar",
    "multáramos": "multar",
    "multáremos": "multar",
    "multásemos": "multar",
    "multé": "multar",
    "multéis": "multar",
    "multífida": "multífido",
    "multífidas": "multífido",
    "multífidos": "multífido",
    "multíparas": "multípara",
    "multívoca": "multívoco",
    "multívocas": "multívoco",
    "multívocos": "multívoco",
    "multó": "multar",
    "muléolos": "muléolo",
    "mundana": "mundano",
    "mundanales": "mundanal",
    "mundanalidades": "mundanalidad",
    "mundanas": "mundano",
    "mundanerías": "mundanería",
    "mundanos": "mundano",
    "mundiales": "mundial",
    "mundialistas": "mundialista",
    "mundializaciones": "mundialización",
    "mundializada": "mundializado",
    "mundializadas": "mundializado",
    "mundializados": "mundializado",
    "mundificaciones": "mundificación",
    "mundificativa": "mundificativo",
    "mundificativas": "mundificativo",
    "mundificativos": "mundificativo",
    "mundillos": "mundillo",
    "mundologías": "mundología",
    "mundos": "mundo",
    "mundovisiones": "mundovisión",
    "municiona": "municionar",
    "municionaba": "municionar",
    "municionabais": "municionar",
    "municionaban": "municionar",
    "municionabas": "municionar",
    "municionad": "municionar",
    "municionada": "municionar",
    "municionadas": "municionar",
    "municionado": "municionar",
    "municionados": "municionar",
    "municionamientos": "municionamiento",
    "municionamos": "municionar",
    "municionan": "municionar",
    "municionando": "municionar",
    "municionara": "municionar",
    "municionarais": "municionar",
    "municionaran": "municionar",
    "municionaras": "municionar",
    "municionare": "municionar",
    "municionareis": "municionar",
    "municionaremos": "municionar",
    "municionaren": "municionar",
    "municionares": "municionar",
    "municionaron": "municionar",
    "municionará": "municionar",
    "municionarán": "municionar",
    "municionarás": "municionar",
    "municionaré": "municionar",
    "municionaréis": "municionar",
    "municionaría": "municionar",
    "municionaríais": "municionar",
    "municionaríamos": "municionar",
    "municionarían": "municionar",
    "municionarías": "municionar",
    "municionas": "municionar",
    "municionase": "municionar",
    "municionaseis": "municionar",
    "municionasen": "municionar",
    "municionases": "municionar",
    "municionaste": "municionar",
    "municionasteis": "municionar",
    "municione": "municionar",
    "municionemos": "municionar",
    "municionen": "municionar",
    "municionera": "municionero",
    "municioneras": "municionero",
    "municioneros": "municionero",
    "municiones": "municionar",
    "municiono": "municionar",
    "municionábamos": "municionar",
    "municionáis": "municionar",
    "municionáramos": "municionar",
    "municionáremos": "municionar",
    "municionásemos": "municionar",
    "municioné": "municionar",
    "municionéis": "municionar",
    "municionó": "municionar",
    "municipales": "municipal",
    "municipalice": "municipalizar",
    "municipalicemos": "municipalizar",
    "municipalicen": "municipalizar",
    "municipalices": "municipalizar",
    "municipalicé": "municipalizar",
    "municipalicéis": "municipalizar",
    "municipalidades": "municipalidad",
    "municipalistas": "municipalista",
    "municipaliza": "municipalizar",
    "municipalizaba": "municipalizar",
    "municipalizabais": "municipalizar",
    "municipalizaban": "municipalizar",
    "municipalizabas": "municipalizar",
    "municipalizad": "municipalizar",
    "municipalizada": "municipalizar",
    "municipalizadas": "municipalizar",
    "municipalizado": "municipalizar",
    "municipalizados": "municipalizar",
    "municipalizamos": "municipalizar",
    "municipalizan": "municipalizar",
    "municipalizando": "municipalizar",
    "municipalizara": "municipalizar",
    "municipalizarais": "municipalizar",
    "municipalizaran": "municipalizar",
    "municipalizaras": "municipalizar",
    "municipalizare": "municipalizar",
    "municipalizareis": "municipalizar",
    "municipalizaremos": "municipalizar",
    "municipalizaren": "municipalizar",
    "municipalizares": "municipalizar",
    "municipalizaron": "municipalizar",
    "municipalizará": "municipalizar",
    "municipalizarán": "municipalizar",
    "municipalizarás": "municipalizar",
    "municipalizaré": "municipalizar",
    "municipalizaréis": "municipalizar",
    "municipalizaría": "municipalizar",
    "municipalizaríais": "municipalizar",
    "municipalizaríamos": "municipalizar",
    "municipalizarían": "municipalizar",
    "municipalizarías": "municipalizar",
    "municipalizas": "municipalizar",
    "municipalizase": "municipalizar",
    "municipalizaseis": "municipalizar",
    "municipalizasen": "municipalizar",
    "municipalizases": "municipalizar",
    "municipalizaste": "municipalizar",
    "municipalizasteis": "municipalizar",
    "municipalizo": "municipalizar",
    "municipalizábamos": "municipalizar",
    "municipalizáis": "municipalizar",
    "municipalizáramos": "municipalizar",
    "municipalizáremos": "municipalizar",
    "municipalizásemos": "municipalizar",
    "municipalizó": "municipalizar",
    "municipios": "municipio",
    "munificencias": "munificencia",
    "munificentes": "munificente",
    "muniquesa": "muniqués",
    "muniquesas": "muniqués",
    "muniqueses": "muniqués",
    "munitorias": "munitoria",
    "munífica": "munífico",
    "muníficas": "munífico",
    "muníficos": "munífico",
    "muones": "muón",
    "mura": "murar",
    "muraba": "murar",
    "murabais": "murar",
    "muraban": "murar",
    "murabas": "murar",
    "murad": "murar",
    "murada": "murar",
    "muradales": "muradal",
    "muradas": "murar",
    "murado": "murar",
    "murados": "murar",
    "murales": "mural",
    "murallas": "muralla",
    "murallones": "murallón",
    "muramos": "murar",
    "muran": "murar",
    "murando": "murar",
    "murara": "murar",
    "murarais": "murar",
    "muraran": "murar",
    "muraras": "murar",
    "murare": "murar",
    "murareis": "murar",
    "muraremos": "murar",
    "muraren": "murar",
    "murares": "murar",
    "muraron": "murar",
    "murará": "murar",
    "murarán": "murar",
    "murarás": "murar",
    "muraré": "murar",
    "muraréis": "murar",
    "muraría": "murar",
    "muraríais": "murar",
    "muraríamos": "murar",
    "murarían": "murar",
    "murarías": "murar",
    "muras": "murar",
    "murase": "murar",
    "muraseis": "murar",
    "murasen": "murar",
    "murases": "murar",
    "muraste": "murar",
    "murasteis": "murar",
    "murciana": "murciano",
    "murcianas": "murciano",
    "murcianos": "murciano",
    "murcielaguinas": "murcielaguina",
    "murciélagos": "murciélago",
    "mure": "murar",
    "muremos": "murar",
    "muren": "murar",
    "mures": "murar",
    "murgas": "murga",
    "murgones": "murgón",
    "murguistas": "murguista",
    "muriendo": "morir",
    "muriera": "morir",
    "murierais": "morir",
    "murieran": "morir",
    "murieras": "morir",
    "muriere": "morir",
    "muriereis": "morir",
    "murieren": "morir",
    "murieres": "morir",
    "murieron": "morir",
    "muriese": "morir",
    "murieseis": "morir",
    "muriesen": "morir",
    "murieses": "morir",
    "muriática": "muriático",
    "muriáticas": "muriático",
    "muriáticos": "muriático",
    "muriéramos": "morir",
    "muriéremos": "morir",
    "muriésemos": "morir",
    "murió": "morir",
    "murmuea": "murmuear",
    "murmueaba": "murmuear",
    "murmueabais": "murmuear",
    "murmueaban": "murmuear",
    "murmueabas": "murmuear",
    "murmuead": "murmuear",
    "murmueada": "murmuear",
    "murmueadas": "murmuear",
    "murmueado": "murmuear",
    "murmueados": "murmuear",
    "murmueamos": "murmuear",
    "murmuean": "murmuear",
    "murmueando": "murmuear",
    "murmueara": "murmuear",
    "murmuearais": "murmuear",
    "murmuearan": "murmuear",
    "murmuearas": "murmuear",
    "murmueare": "murmuear",
    "murmueareis": "murmuear",
    "murmuearemos": "murmuear",
    "murmuearen": "murmuear",
    "murmueares": "murmuear",
    "murmuearon": "murmuear",
    "murmueará": "murmuear",
    "murmuearán": "murmuear",
    "murmuearás": "murmuear",
    "murmuearé": "murmuear",
    "murmuearéis": "murmuear",
    "murmuearía": "murmuear",
    "murmuearíais": "murmuear",
    "murmuearíamos": "murmuear",
    "murmuearían": "murmuear",
    "murmuearías": "murmuear",
    "murmueas": "murmuear",
    "murmuease": "murmuear",
    "murmueaseis": "murmuear",
    "murmueasen": "murmuear",
    "murmueases": "murmuear",
    "murmueaste": "murmuear",
    "murmueasteis": "murmuear",
    "murmuee": "murmuear",
    "murmueemos": "murmuear",
    "murmueen": "murmuear",
    "murmuees": "murmuear",
    "murmueo": "murmuear",
    "murmueábamos": "murmuear",
    "murmueáis": "murmuear",
    "murmueáramos": "murmuear",
    "murmueáremos": "murmuear",
    "murmueásemos": "murmuear",
    "murmueé": "murmuear",
    "murmueéis": "murmuear",
    "murmueó": "murmuear",
    "murmulla": "murmullar",
    "murmullaba": "murmullar",
    "murmullabais": "murmullar",
    "murmullaban": "murmullar",
    "murmullabas": "murmullar",
    "murmullad": "murmullar",
    "murmullada": "murmullar",
    "murmulladas": "murmullar",
    "murmullado": "murmullar",
    "murmullados": "murmullar",
    "murmullamos": "murmullar",
    "murmullan": "murmullar",
    "murmullando": "murmullar",
    "murmullara": "murmullar",
    "murmullarais": "murmullar",
    "murmullaran": "murmullar",
    "murmullaras": "murmullar",
    "murmullare": "murmullar",
    "murmullareis": "murmullar",
    "murmullaremos": "murmullar",
    "murmullaren": "murmullar",
    "murmullares": "murmullar",
    "murmullaron": "murmullar",
    "murmullará": "murmullar",
    "murmullarán": "murmullar",
    "murmullarás": "murmullar",
    "murmullaré": "murmullar",
    "murmullaréis": "murmullar",
    "murmullaría": "murmullar",
    "murmullaríais": "murmullar",
    "murmullaríamos": "murmullar",
    "murmullarían": "murmullar",
    "murmullarías": "murmullar",
    "murmullas": "murmullar",
    "murmullase": "murmullar",
    "murmullaseis": "murmullar",
    "murmullasen": "murmullar",
    "murmullases": "murmullar",
    "murmullaste": "murmullar",
    "murmullasteis": "murmullar",
    "murmulle": "murmullar",
    "murmullemos": "murmullar",
    "murmullen": "murmullar",
    "murmulles": "murmullar",
    "murmullo": "murmullar",
    "murmullos": "murmullo",
    "murmullábamos": "murmullar",
    "murmulláis": "murmullar",
    "murmulláramos": "murmullar",
    "murmulláremos": "murmullar",
    "murmullásemos": "murmullar",
    "murmullé": "murmullar",
    "murmulléis": "murmullar",
    "murmulló": "murmullar",
    "murmura": "murmurar",
    "murmuraba": "murmurar",
    "murmurabais": "murmurar",
    "murmuraban": "murmurar",
    "murmurabas": "murmurar",
    "murmuraciones": "murmuración",
    "murmurad": "murmurar",
    "murmurada": "murmurar",
    "murmuradas": "murmurar",
    "murmurado": "murmurar",
    "murmuradora": "murmurador",
    "murmuradoras": "murmurador",
    "murmuradores": "murmurador",
    "murmurados": "murmurar",
    "murmuramos": "murmurar",
    "murmuran": "murmurar",
    "murmurando": "murmurar",
    "murmurara": "murmurar",
    "murmurarais": "murmurar",
    "murmuraran": "murmurar",
    "murmuraras": "murmurar",
    "murmurare": "murmurar",
    "murmurareis": "murmurar",
    "murmuraremos": "murmurar",
    "murmuraren": "murmurar",
    "murmurares": "murmurar",
    "murmuraron": "murmurar",
    "murmurará": "murmurar",
    "murmurarán": "murmurar",
    "murmurarás": "murmurar",
    "murmuraré": "murmurar",
    "murmuraréis": "murmurar",
    "murmuraría": "murmurar",
    "murmuraríais": "murmurar",
    "murmuraríamos": "murmurar",
    "murmurarían": "murmurar",
    "murmurarías": "murmurar",
    "murmuras": "murmurar",
    "murmurase": "murmurar",
    "murmuraseis": "murmurar",
    "murmurasen": "murmurar",
    "murmurases": "murmurar",
    "murmuraste": "murmurar",
    "murmurasteis": "murmurar",
    "murmure": "murmurar",
    "murmuremos": "murmurar",
    "murmuren": "murmurar",
    "murmures": "murmurar",
    "murmuro": "murmurar",
    "murmurábamos": "murmurar",
    "murmuráis": "murmurar",
    "murmuráramos": "murmurar",
    "murmuráremos": "murmurar",
    "murmurásemos": "murmurar",
    "murmuré": "murmurar",
    "murmuréis": "murmurar",
    "murmuró": "murmurar",
    "muro": "murar",
    "muros": "muro",
    "murria": "murrio",
    "murrias": "murrio",
    "murrios": "murrio",
    "murriosa": "murrioso",
    "murriosas": "murrioso",
    "murriosos": "murrioso",
    "murtas": "murta",
    "murtillas": "murtilla",
    "murtones": "murtón",
    "murtos": "murto",
    "muruecos": "murueco",
    "murábamos": "murar",
    "muráis": "murar",
    "muráramos": "murar",
    "muráremos": "murar",
    "murásemos": "murar",
    "muré": "murar",
    "muréis": "murar",
    "muró": "murar",
    "mus": "mu",
    "musacas": "musaca",
    "musarañas": "musaraña",
    "musas": "musa",
    "musca": "musco",
    "muscarinas": "muscarina",
    "muscas": "musco",
    "muscinales": "muscinal",
    "muscos": "musco",
    "muscosa": "muscoso",
    "muscosas": "muscoso",
    "muscosos": "muscoso",
    "musculaciones": "musculación",
    "musculares": "muscular",
    "musculaturas": "musculatura",
    "musculocutánea": "musculocutáneo",
    "musculocutáneas": "musculocutáneo",
    "musculocutáneos": "musculocutáneo",
    "musculofrénica": "musculofrénico",
    "musculofrénicas": "musculofrénico",
    "musculofrénicos": "musculofrénico",
    "musculosa": "musculoso",
    "musculosas": "musculoso",
    "musculosos": "musculoso",
    "musculotubárica": "musculotubárico",
    "musculotubáricas": "musculotubárico",
    "musculotubáricos": "musculotubárico",
    "muselina": "muselino",
    "muselinas": "muselino",
    "muselinos": "muselino",
    "museografías": "museografía",
    "museologías": "museología",
    "museológica": "museológico",
    "museológicas": "museológico",
    "museológicos": "museológico",
    "museos": "museo",
    "muserolas": "muserola",
    "musga": "musgo",
    "musgas": "musgo",
    "musgaños": "musgaño",
    "musgos": "musgo",
    "musgosa": "musgoso",
    "musgosas": "musgoso",
    "musgosos": "musgoso",
    "music-halls": "music-hall",
    "musica": "musicar",
    "musicaba": "musicar",
    "musicabais": "musicar",
    "musicaban": "musicar",
    "musicabas": "musicar",
    "musicables": "musicable",
    "musicad": "musicar",
    "musicada": "musicar",
    "musicadas": "musicar",
    "musicado": "musicar",
    "musicados": "musicar",
    "musicales": "musical",
    "musicalidades": "musicalidad",
    "musicamos": "musicar",
    "musican": "musicar",
    "musicando": "musicar",
    "musicara": "musicar",
    "musicarais": "musicar",
    "musicaran": "musicar",
    "musicaras": "musicar",
    "musicare": "musicar",
    "musicareis": "musicar",
    "musicaremos": "musicar",
    "musicaren": "musicar",
    "musicares": "musicar",
    "musicaron": "musicar",
    "musicará": "musicar",
    "musicarán": "musicar",
    "musicarás": "musicar",
    "musicaré": "musicar",
    "musicaréis": "musicar",
    "musicaría": "musicar",
    "musicaríais": "musicar",
    "musicaríamos": "musicar",
    "musicarían": "musicar",
    "musicarías": "musicar",
    "musicas": "musicar",
    "musicase": "musicar",
    "musicaseis": "musicar",
    "musicasen": "musicar",
    "musicases": "musicar",
    "musicaste": "musicar",
    "musicasteis": "musicar",
    "musicastros": "musicastro",
    "musichalls": "musichall",
    "musico": "musicar",
    "musicologías": "musicología",
    "musicomanías": "musicomanía",
    "musicábamos": "musicar",
    "musicáis": "musicar",
    "musicáramos": "musicar",
    "musicáremos": "musicar",
    "musicásemos": "musicar",
    "musicó": "musicar",
    "musicógrafa": "musicógrafo",
    "musicógrafas": "musicógrafo",
    "musicógrafos": "musicógrafo",
    "musicóloga": "musicólogo",
    "musicólogas": "musicólogo",
    "musicólogos": "musicólogo",
    "musicómana": "musicómano",
    "musicómanas": "musicómano",
    "musicómanos": "musicómano",
    "musique": "musicar",
    "musiquemos": "musicar",
    "musiquen": "musicar",
    "musiqueros": "musiquero",
    "musiques": "musicar",
    "musiqué": "musicar",
    "musiquéis": "musicar",
    "musita": "musitar",
    "musitaba": "musitar",
    "musitabais": "musitar",
    "musitaban": "musitar",
    "musitabas": "musitar",
    "musitaciones": "musitación",
    "musitad": "musitar",
    "musitada": "musitar",
    "musitadas": "musitar",
    "musitado": "musitar",
    "musitados": "musitar",
    "musitamos": "musitar",
    "musitan": "musitar",
    "musitando": "musitar",
    "musitara": "musitar",
    "musitarais": "musitar",
    "musitaran": "musitar",
    "musitaras": "musitar",
    "musitare": "musitar",
    "musitareis": "musitar",
    "musitaremos": "musitar",
    "musitaren": "musitar",
    "musitares": "musitar",
    "musitaron": "musitar",
    "musitará": "musitar",
    "musitarán": "musitar",
    "musitarás": "musitar",
    "musitaré": "musitar",
    "musitaréis": "musitar",
    "musitaría": "musitar",
    "musitaríais": "musitar",
    "musitaríamos": "musitar",
    "musitarían": "musitar",
    "musitarías": "musitar",
    "musitas": "musitar",
    "musitase": "musitar",
    "musitaseis": "musitar",
    "musitasen": "musitar",
    "musitases": "musitar",
    "musitaste": "musitar",
    "musitasteis": "musitar",
    "musite": "musitar",
    "musitemos": "musitar",
    "musiten": "musitar",
    "musites": "musitar",
    "musito": "musitar",
    "musitábamos": "musitar",
    "musitáis": "musitar",
    "musitáramos": "musitar",
    "musitáremos": "musitar",
    "musitásemos": "musitar",
    "musité": "musitar",
    "musitéis": "musitar",
    "musitó": "musitar",
    "musivaria": "musivario",
    "musivarias": "musivario",
    "musivarios": "musivario",
    "musleras": "muslera",
    "muslimes": "muslime",
    "muslos": "muslo",
    "muslímica": "muslímico",
    "muslímicas": "muslímico",
    "muslímicos": "muslímico",
    "musmones": "musmón",
    "musolas": "musola",
    "mustacos": "mustaco",
    "musterienses": "musteriense",
    "mustia": "mustio",
    "mustiaba": "mustiar",
    "mustiabais": "mustiar",
    "mustiaban": "mustiar",
    "mustiabas": "mustiar",
    "mustiad": "mustiar",
    "mustiada": "mustiar",
    "mustiadas": "mustiar",
    "mustiado": "mustiar",
    "mustiados": "mustiar",
    "mustiamos": "mustiar",
    "mustian": "mustiar",
    "mustiando": "mustiar",
    "mustiara": "mustiar",
    "mustiarais": "mustiar",
    "mustiaran": "mustiar",
    "mustiaras": "mustiar",
    "mustiare": "mustiar",
    "mustiareis": "mustiar",
    "mustiaremos": "mustiar",
    "mustiaren": "mustiar",
    "mustiares": "mustiar",
    "mustiaron": "mustiar",
    "mustiará": "mustiar",
    "mustiarán": "mustiar",
    "mustiarás": "mustiar",
    "mustiaré": "mustiar",
    "mustiaréis": "mustiar",
    "mustiaría": "mustiar",
    "mustiaríais": "mustiar",
    "mustiaríamos": "mustiar",
    "mustiarían": "mustiar",
    "mustiarías": "mustiar",
    "mustias": "mustio",
    "mustiase": "mustiar",
    "mustiaseis": "mustiar",
    "mustiasen": "mustiar",
    "mustiases": "mustiar",
    "mustiaste": "mustiar",
    "mustiasteis": "mustiar",
    "mustie": "mustiar",
    "mustiemos": "mustiar",
    "mustien": "mustiar",
    "musties": "mustiar",
    "mustio": "mustiar",
    "mustios": "mustio",
    "mustiábamos": "mustiar",
    "mustiáis": "mustiar",
    "mustiáramos": "mustiar",
    "mustiáremos": "mustiar",
    "mustiásemos": "mustiar",
    "mustié": "mustiar",
    "mustiéis": "mustiar",
    "mustió": "mustiar",
    "mustélidos": "mustélido",
    "musulmana": "musulmán",
    "musulmanas": "musulmán",
    "musulmanes": "musulmán",
    "musáceas": "musácea",
    "muta": "mutar",
    "mutaba": "mutar",
    "mutabais": "mutar",
    "mutaban": "mutar",
    "mutabas": "mutar",
    "mutabilidades": "mutabilidad",
    "mutables": "mutable",
    "mutacionales": "mutacional",
    "mutaciones": "mutación",
    "mutacionismos": "mutacionismo",
    "mutacionistas": "mutacionista",
    "mutad": "mutar",
    "mutada": "mutar",
    "mutadas": "mutar",
    "mutado": "mutar",
    "mutadores": "mutador",
    "mutados": "mutar",
    "mutagénica": "mutagénico",
    "mutagénicas": "mutagénico",
    "mutagénicos": "mutagénico",
    "mutamos": "mutar",
    "mutan": "mutar",
    "mutando": "mutar",
    "mutantes": "mutante",
    "mutara": "mutar",
    "mutarais": "mutar",
    "mutaran": "mutar",
    "mutaras": "mutar",
    "mutare": "mutar",
    "mutareis": "mutar",
    "mutaremos": "mutar",
    "mutaren": "mutar",
    "mutares": "mutar",
    "mutaron": "mutar",
    "mutará": "mutar",
    "mutarán": "mutar",
    "mutarás": "mutar",
    "mutaré": "mutar",
    "mutaréis": "mutar",
    "mutaría": "mutar",
    "mutaríais": "mutar",
    "mutaríamos": "mutar",
    "mutarían": "mutar",
    "mutarías": "mutar",
    "mutas": "mutar",
    "mutase": "mutar",
    "mutaseis": "mutar",
    "mutasen": "mutar",
    "mutases": "mutar",
    "mutaste": "mutar",
    "mutasteis": "mutar",
    "mute": "mutar",
    "mutemos": "mutar",
    "muten": "mutar",
    "mutes": "mutar",
    "mutila": "mutilar",
    "mutilaba": "mutilar",
    "mutilabais": "mutilar",
    "mutilaban": "mutilar",
    "mutilabas": "mutilar",
    "mutilaciones": "mutilación",
    "mutilad": "mutilar",
    "mutilada": "mutilar",
    "mutiladas": "mutilar",
    "mutilado": "mutilar",
    "mutiladora": "mutilador",
    "mutiladoras": "mutilador",
    "mutiladores": "mutilador",
    "mutilados": "mutilar",
    "mutilamos": "mutilar",
    "mutilan": "mutilar",
    "mutilando": "mutilar",
    "mutilara": "mutilar",
    "mutilarais": "mutilar",
    "mutilaran": "mutilar",
    "mutilaras": "mutilar",
    "mutilare": "mutilar",
    "mutilareis": "mutilar",
    "mutilaremos": "mutilar",
    "mutilaren": "mutilar",
    "mutilares": "mutilar",
    "mutilaron": "mutilar",
    "mutilará": "mutilar",
    "mutilarán": "mutilar",
    "mutilarás": "mutilar",
    "mutilaré": "mutilar",
    "mutilaréis": "mutilar",
    "mutilaría": "mutilar",
    "mutilaríais": "mutilar",
    "mutilaríamos": "mutilar",
    "mutilarían": "mutilar",
    "mutilarías": "mutilar",
    "mutilas": "mutilar",
    "mutilase": "mutilar",
    "mutilaseis": "mutilar",
    "mutilasen": "mutilar",
    "mutilases": "mutilar",
    "mutilaste": "mutilar",
    "mutilasteis": "mutilar",
    "mutile": "mutilar",
    "mutilemos": "mutilar",
    "mutilen": "mutilar",
    "mutiles": "mutilar",
    "mutilo": "mutilar",
    "mutilábamos": "mutilar",
    "mutiláis": "mutilar",
    "mutiláramos": "mutilar",
    "mutiláremos": "mutilar",
    "mutilásemos": "mutilar",
    "mutilé": "mutilar",
    "mutiléis": "mutilar",
    "mutiló": "mutilar",
    "mutismos": "mutismo",
    "muto": "mutar",
    "mutones": "mutón",
    "mutua": "mutuo",
    "mutuales": "mutual",
    "mutualidades": "mutualidad",
    "mutualismos": "mutualismo",
    "mutualistas": "mutualista",
    "mutuaria": "mutuario",
    "mutuarias": "mutuario",
    "mutuarios": "mutuario",
    "mutuas": "mutuo",
    "mutuos": "mutuo",
    "mutábamos": "mutar",
    "mutágenos": "mutágeno",
    "mutáis": "mutar",
    "mutáramos": "mutar",
    "mutáremos": "mutar",
    "mutásemos": "mutar",
    "muté": "mutar",
    "mutéis": "mutar",
    "mutó": "mutar",
    "muérdagos": "muérdago",
    "muñecas": "muñeca",
    "muñecos": "muñeco",
    "muñeiras": "muñeira",
    "muñequeras": "muñequera",
    "muñequillas": "muñequilla",
    "muñoneras": "muñonera",
    "muñones": "muñón",
    "muónes": "muón",
    "myanmaresa": "myanmarés",
    "myanmaresas": "myanmarés",
    "myanmareses": "myanmarés",
    "máculas": "mácula",
    "máfica": "máfico",
    "máficas": "máfico",
    "máficos": "máfico",
    "mágica": "mágico",
    "mágicas": "mágico",
    "mágicos": "mágico",
    "málagas": "málaga",
    "málica": "málico",
    "málicas": "málico",
    "málicos": "málico",
    "mánagers": "mánager",
    "mánticas": "mántica",
    "máquinas": "máquina",
    "márcolas": "márcola",
    "márfagas": "márfaga",
    "márfegas": "márfega",
    "márgenes": "margen",
    "márketings": "márketing",
    "mármoles": "mármol",
    "márquetings": "márqueting",
    "mártires": "mártir",
    "máscaras": "máscara",
    "máseres": "máser",
    "másers": "máser",
    "másica": "másico",
    "másicas": "másico",
    "másicos": "másico",
    "másters": "máster",
    "mástices": "mástic",
    "mástiles": "mástil",
    "mástiques": "mástique",
    "máuseres": "máuser",
    "máxima": "máximo",
    "máximas": "máximo",
    "máximos": "máximo",
    "médanos": "médano",
    "médica": "médico",
    "médicas": "médico",
    "médicos": "médico",
    "médiums": "médium",
    "médulas": "médula",
    "méganos": "mégano",
    "mélica": "mélico",
    "mélicas": "mélico",
    "mélicos": "mélico",
    "mélées": "mélée",
    "ménades": "ménade",
    "ménsulas": "ménsula",
    "mérgulos": "mérgulo",
    "méritos": "mérito",
    "métodos": "método",
    "métopas": "métopa",
    "métrica": "métrico",
    "métricas": "métrico",
    "métricos": "métrico",
    "mía": "mío",
    "mían": "miar",
    "mías": "mío",
    "míe": "miar",
    "míen": "miar",
    "míes": "miar",
    "mílites": "mílite",
    "mímica": "mímico",
    "mímicas": "mímico",
    "mímicos": "mímico",
    "mínima": "mínimo",
    "mínimas": "mínimo",
    "mínimos": "mínimo",
    "mío": "miar",
    "míos": "mío",
    "mísera": "mísero",
    "míseras": "mísero",
    "míseros": "mísero",
    "mísiles": "mísil",
    "mísquitos": "mísquito",
    "mística": "místico",
    "místicas": "místico",
    "místicos": "místico",
    "mítica": "mítico",
    "míticas": "mítico",
    "míticos": "mítico",
    "mítines": "mitin",
    "mízcalos": "mízcalo",
    "módems": "módem",
    "módica": "módico",
    "módicas": "módico",
    "módicos": "módico",
    "módulos": "módulo",
    "móllica": "móllico",
    "móllicas": "móllico",
    "móllicos": "móllico",
    "mónadas": "mónada",
    "mónitas": "mónita",
    "mórbida": "mórbido",
    "mórbidas": "mórbido",
    "mórbidos": "mórbido",
    "mórulas": "mórula",
    "móviles": "móvil",
    "múcaras": "múcara",
    "múcica": "múcico",
    "múcicas": "múcico",
    "múcicos": "múcico",
    "múcores": "múcor",
    "múgiles": "múgil",
    "múldemes": "múldem",
    "múleos": "múleo",
    "múltipla": "múltiplo",
    "múltiplas": "múltiplo",
    "múltiples": "múltiple",
    "múltiplexos": "múltiplex",
    "múltiplos": "múltiplo",
    "múrices": "múrice",
    "múrrina": "múrrino",
    "múrrinas": "múrrino",
    "múrrinos": "múrrino",
    "músculos": "músculo",
    "música": "músico",
    "músicas": "músico",
    "músicos": "músico",
    "mútulas": "mútula",
    "mútulos": "mútulo",
    "nababs": "nabab",
    "nabares": "nabar",
    "nabas": "naba",
    "nabatea": "nabateo",
    "nabateas": "nabateo",
    "nabateos": "nabateo",
    "naberías": "nabería",
    "nabicoles": "nabicol",
    "nabinas": "nabina",
    "nabises": "nabis",
    "nabizas": "nabiza",
    "nablas": "nabla",
    "naborías": "naboría",
    "naboríes": "naborí",
    "naborís": "naborí",
    "nabos": "nabo",
    "nabíes": "nabí",
    "nacarada": "nacarado",
    "nacaradas": "nacarado",
    "nacarados": "nacarado",
    "nacarina": "nacarino",
    "nacarinas": "nacarino",
    "nacarinos": "nacarino",
    "nacarones": "nacarón",
    "nace": "nacer",
    "naced": "nacer",
    "nacedera": "nacedero",
    "nacederas": "nacedero",
    "nacederos": "nacedero",
    "nacelas": "nacela",
    "nacemos": "nacer",
    "nacen": "nacer",
    "nacencias": "nacencia",
    "naceremos": "nacer",
    "nacerá": "nacer",
    "nacerán": "nacer",
    "nacerás": "nacer",
    "naceré": "nacer",
    "naceréis": "nacer",
    "nacería": "nacer",
    "naceríais": "nacer",
    "naceríamos": "nacer",
    "nacerían": "nacer",
    "nacerías": "nacer",
    "naces": "nacer",
    "nacida": "nacido",
    "nacidas": "nacido",
    "nacido": "nacer",
    "nacidos": "nacido",
    "naciendo": "nacer",
    "nacientes": "naciente",
    "naciera": "nacer",
    "nacierais": "nacer",
    "nacieran": "nacer",
    "nacieras": "nacer",
    "naciere": "nacer",
    "naciereis": "nacer",
    "nacieren": "nacer",
    "nacieres": "nacer",
    "nacieron": "nacer",
    "naciese": "nacer",
    "nacieseis": "nacer",
    "naciesen": "nacer",
    "nacieses": "nacer",
    "nacimientos": "nacimiento",
    "nacimos": "nacer",
    "nacionalcatolicismos": "nacionalcatolicismo",
    "nacionalcatólica": "nacionalcatólico",
    "nacionalcatólicas": "nacionalcatólico",
    "nacionalcatólicos": "nacionalcatólico",
    "nacionales": "nacional",
    "nacionalice": "nacionalizar",
    "nacionalicemos": "nacionalizar",
    "nacionalicen": "nacionalizar",
    "nacionalices": "nacionalizar",
    "nacionalicé": "nacionalizar",
    "nacionalicéis": "nacionalizar",
    "nacionalidades": "nacionalidad",
    "nacionalismos": "nacionalismo",
    "nacionalistas": "nacionalista",
    "nacionaliza": "nacionalizar",
    "nacionalizaba": "nacionalizar",
    "nacionalizabais": "nacionalizar",
    "nacionalizaban": "nacionalizar",
    "nacionalizabas": "nacionalizar",
    "nacionalizaciones": "nacionalización",
    "nacionalizad": "nacionalizar",
    "nacionalizada": "nacionalizar",
    "nacionalizadas": "nacionalizar",
    "nacionalizado": "nacionalizar",
    "nacionalizados": "nacionalizar",
    "nacionalizamos": "nacionalizar",
    "nacionalizan": "nacionalizar",
    "nacionalizando": "nacionalizar",
    "nacionalizara": "nacionalizar",
    "nacionalizarais": "nacionalizar",
    "nacionalizaran": "nacionalizar",
    "nacionalizaras": "nacionalizar",
    "nacionalizare": "nacionalizar",
    "nacionalizareis": "nacionalizar",
    "nacionalizaremos": "nacionalizar",
    "nacionalizaren": "nacionalizar",
    "nacionalizares": "nacionalizar",
    "nacionalizaron": "nacionalizar",
    "nacionalizará": "nacionalizar",
    "nacionalizarán": "nacionalizar",
    "nacionalizarás": "nacionalizar",
    "nacionalizaré": "nacionalizar",
    "nacionalizaréis": "nacionalizar",
    "nacionalizaría": "nacionalizar",
    "nacionalizaríais": "nacionalizar",
    "nacionalizaríamos": "nacionalizar",
    "nacionalizarían": "nacionalizar",
    "nacionalizarías": "nacionalizar",
    "nacionalizas": "nacionalizar",
    "nacionalizase": "nacionalizar",
    "nacionalizaseis": "nacionalizar",
    "nacionalizasen": "nacionalizar",
    "nacionalizases": "nacionalizar",
    "nacionalizaste": "nacionalizar",
    "nacionalizasteis": "nacionalizar",
    "nacionalizo": "nacionalizar",
    "nacionalizábamos": "nacionalizar",
    "nacionalizáis": "nacionalizar",
    "nacionalizáramos": "nacionalizar",
    "nacionalizáremos": "nacionalizar",
    "nacionalizásemos": "nacionalizar",
    "nacionalizó": "nacionalizar",
    "nacionalsindicalismos": "nacionalsindicalismo",
    "nacionalsindicalistas": "nacionalsindicalista",
    "nacionalsocialismos": "nacionalsocialismo",
    "nacionalsocialistas": "nacionalsocialista",
    "naciones": "nación",
    "naciste": "nacer",
    "nacisteis": "nacer",
    "naciéramos": "nacer",
    "naciéremos": "nacer",
    "naciésemos": "nacer",
    "nació": "nacer",
    "nacras": "nacra",
    "nacritas": "nacrita",
    "nacéis": "nacer",
    "nací": "nacer",
    "nacía": "nacer",
    "nacíais": "nacer",
    "nacíamos": "nacer",
    "nacían": "nacer",
    "nacías": "nacer",
    "nada": "nadar",
    "nadaba": "nadar",
    "nadabais": "nadar",
    "nadaban": "nadar",
    "nadabas": "nadar",
    "nadad": "nadar",
    "nadada": "nadar",
    "nadadas": "nadar",
    "nadaderas": "nadadera",
    "nadaderos": "nadadero",
    "nadado": "nadar",
    "nadadora": "nadador",
    "nadadoras": "nadador",
    "nadadores": "nadador",
    "nadados": "nadar",
    "nadamos": "nadar",
    "nadan": "nadar",
    "nadando": "nadar",
    "nadara": "nadar",
    "nadarais": "nadar",
    "nadaran": "nadar",
    "nadaras": "nadar",
    "nadare": "nadar",
    "nadareis": "nadar",
    "nadaremos": "nadar",
    "nadaren": "nadar",
    "nadares": "nadar",
    "nadaron": "nadar",
    "nadará": "nadar",
    "nadarán": "nadar",
    "nadarás": "nadar",
    "nadaré": "nadar",
    "nadaréis": "nadar",
    "nadaría": "nadar",
    "nadaríais": "nadar",
    "nadaríamos": "nadar",
    "nadarían": "nadar",
    "nadarías": "nadar",
    "nadas": "nadar",
    "nadase": "nadar",
    "nadaseis": "nadar",
    "nadasen": "nadar",
    "nadases": "nadar",
    "nadaste": "nadar",
    "nadasteis": "nadar",
    "nade": "nadar",
    "nademos": "nadar",
    "naden": "nadar",
    "naderías": "nadería",
    "nades": "nadar",
    "nadires": "nadir",
    "nado": "nadar",
    "nadololes": "nadolol",
    "nadábamos": "nadar",
    "nadáis": "nadar",
    "nadáramos": "nadar",
    "nadáremos": "nadar",
    "nadásemos": "nadar",
    "nadé": "nadar",
    "nadéis": "nadar",
    "nadó": "nadar",
    "naftacenos": "naftaceno",
    "naftalendisulfónica": "naftalendisulfónico",
    "naftalendisulfónicas": "naftalendisulfónico",
    "naftalendisulfónicos": "naftalendisulfónico",
    "naftalenos": "naftaleno",
    "naftalenosulfonatos": "naftalenosulfonato",
    "naftalensulfónica": "naftalensulfónico",
    "naftalensulfónicas": "naftalensulfónico",
    "naftalensulfónicos": "naftalensulfónico",
    "naftalinas": "naftalina",
    "naftas": "nafta",
    "naftenatos": "naftenato",
    "naftilaminas": "naftilamina",
    "naftilendiaminas": "naftilendiamina",
    "naftiltioureas": "naftiltiourea",
    "naftionatos": "naftionato",
    "naftiónica": "naftiónico",
    "naftiónicas": "naftiónico",
    "naftiónicos": "naftiónico",
    "naftoles": "naftol",
    "naftoquinonas": "naftoquinona",
    "nafténica": "nafténico",
    "nafténicas": "nafténico",
    "nafténicos": "nafténico",
    "naftílica": "naftílico",
    "naftílicas": "naftílico",
    "naftílicos": "naftílico",
    "naganas": "nagana",
    "naguatlata": "naguatlato",
    "naguatlatas": "naguatlato",
    "naguatlatos": "naguatlato",
    "naguatles": "naguatle",
    "nahores": "nahor",
    "nahualismos": "nahualismo",
    "nahuas": "nahua",
    "nahuatles": "nahuatle",
    "naide": "nadie",
    "naifes": "naife",
    "naifs": "naif",
    "naipes": "naipe",
    "naipesca": "naipesco",
    "naipescas": "naipesco",
    "naipescos": "naipesco",
    "nairas": "naira",
    "naires": "naire",
    "naja": "najar",
    "najaba": "najar",
    "najabais": "najar",
    "najaban": "najar",
    "najabas": "najar",
    "najad": "najar",
    "najada": "najar",
    "najadas": "najar",
    "najado": "najar",
    "najados": "najar",
    "najamos": "najar",
    "najan": "najar",
    "najando": "najar",
    "najara": "najar",
    "najarais": "najar",
    "najaran": "najar",
    "najaras": "najar",
    "najare": "najar",
    "najareis": "najar",
    "najaremos": "najar",
    "najaren": "najar",
    "najares": "najar",
    "najaron": "najar",
    "najará": "najar",
    "najarán": "najar",
    "najarás": "najar",
    "najaré": "najar",
    "najaréis": "najar",
    "najaría": "najar",
    "najaríais": "najar",
    "najaríamos": "najar",
    "najarían": "najar",
    "najarías": "najar",
    "najas": "najar",
    "najase": "najar",
    "najaseis": "najar",
    "najasen": "najar",
    "najases": "najar",
    "najaste": "najar",
    "najasteis": "najar",
    "naje": "najar",
    "najemos": "najar",
    "najen": "najar",
    "najes": "najar",
    "najo": "najar",
    "najábamos": "najar",
    "najáis": "najar",
    "najáramos": "najar",
    "najáremos": "najar",
    "najásemos": "najar",
    "najé": "najar",
    "najéis": "najar",
    "najó": "najar",
    "nalgadas": "nalgada",
    "nalgares": "nalgar",
    "nalgas": "nalga",
    "nalgatorios": "nalgatorio",
    "nalguda": "nalgudo",
    "nalgudas": "nalgudo",
    "nalgudos": "nalgudo",
    "namibia": "namibio",
    "namibias": "namibio",
    "namibios": "namibio",
    "nanas": "nana",
    "nanocompuestos": "nanocompuesto",
    "nanoestructuras": "nanoestructura",
    "nanoftalmías": "nanoftalmía",
    "nanopartículas": "nanopartícula",
    "nanoprogramaciones": "nanoprogramación",
    "nanoprogramas": "nanoprograma",
    "nanosegundos": "nanosegundo",
    "nansas": "nansa",
    "nansúes": "nansú",
    "nansús": "nansú",
    "nansúses": "nansú",
    "nanómetros": "nanómetro",
    "naonata": "naonato",
    "naonatas": "naonato",
    "naonatos": "naonato",
    "naos": "nao",
    "napas": "napa",
    "napea": "napeo",
    "napeas": "napeo",
    "napeos": "napeo",
    "napias": "napia",
    "napiformes": "napiforme",
    "napoleones": "napoleón",
    "napoleónica": "napoleónico",
    "napoleónicas": "napoleónico",
    "napoleónicos": "napoleónico",
    "napolitana": "napolitano",
    "napolitanas": "napolitano",
    "napolitanos": "napolitano",
    "naranjada": "naranjado",
    "naranjadas": "naranjado",
    "naranjados": "naranjado",
    "naranjales": "naranjal",
    "naranjas": "naranja",
    "naranjera": "naranjero",
    "naranjeras": "naranjero",
    "naranjeros": "naranjero",
    "naranjillas": "naranjilla",
    "naranjos": "naranjo",
    "narbonenses": "narbonense",
    "narbonesa": "narbonés",
    "narbonesas": "narbonés",
    "narboneses": "narbonés",
    "narceínas": "narceína",
    "narcisismos": "narcisismo",
    "narcisistas": "narcisista",
    "narcisitas": "narcisita",
    "narcisos": "narciso",
    "narcodineros": "narcodinero",
    "narcotice": "narcotizar",
    "narcoticemos": "narcotizar",
    "narcoticen": "narcotizar",
    "narcotices": "narcotizar",
    "narcoticé": "narcotizar",
    "narcoticéis": "narcotizar",
    "narcotinas": "narcotina",
    "narcotismos": "narcotismo",
    "narcotiza": "narcotizar",
    "narcotizaba": "narcotizar",
    "narcotizabais": "narcotizar",
    "narcotizaban": "narcotizar",
    "narcotizabas": "narcotizar",
    "narcotizaciones": "narcotización",
    "narcotizad": "narcotizar",
    "narcotizada": "narcotizar",
    "narcotizadas": "narcotizar",
    "narcotizado": "narcotizar",
    "narcotizadora": "narcotizador",
    "narcotizadoras": "narcotizador",
    "narcotizadores": "narcotizador",
    "narcotizados": "narcotizar",
    "narcotizamos": "narcotizar",
    "narcotizan": "narcotizar",
    "narcotizando": "narcotizar",
    "narcotizara": "narcotizar",
    "narcotizarais": "narcotizar",
    "narcotizaran": "narcotizar",
    "narcotizaras": "narcotizar",
    "narcotizare": "narcotizar",
    "narcotizareis": "narcotizar",
    "narcotizaremos": "narcotizar",
    "narcotizaren": "narcotizar",
    "narcotizares": "narcotizar",
    "narcotizaron": "narcotizar",
    "narcotizará": "narcotizar",
    "narcotizarán": "narcotizar",
    "narcotizarás": "narcotizar",
    "narcotizaré": "narcotizar",
    "narcotizaréis": "narcotizar",
    "narcotizaría": "narcotizar",
    "narcotizaríais": "narcotizar",
    "narcotizaríamos": "narcotizar",
    "narcotizarían": "narcotizar",
    "narcotizarías": "narcotizar",
    "narcotizas": "narcotizar",
    "narcotizase": "narcotizar",
    "narcotizaseis": "narcotizar",
    "narcotizasen": "narcotizar",
    "narcotizases": "narcotizar",
    "narcotizaste": "narcotizar",
    "narcotizasteis": "narcotizar",
    "narcotizo": "narcotizar",
    "narcotizábamos": "narcotizar",
    "narcotizáis": "narcotizar",
    "narcotizáramos": "narcotizar",
    "narcotizáremos": "narcotizar",
    "narcotizásemos": "narcotizar",
    "narcotizó": "narcotizar",
    "narcotraficantes": "narcotraficante",
    "narcotráficos": "narcotráfico",
    "narcótica": "narcótico",
    "narcóticas": "narcótico",
    "narcóticos": "narcótico",
    "nardina": "nardino",
    "nardinas": "nardino",
    "nardinos": "nardino",
    "nardos": "nardo",
    "narguiles": "narguile",
    "narices": "nariz",
    "narigona": "narigón",
    "narigonas": "narigón",
    "narigones": "narigón",
    "nariguda": "narigudo",
    "narigudas": "narigudo",
    "narigudos": "narigudo",
    "narigueras": "nariguera",
    "narinas": "narina",
    "narizona": "narizón",
    "narizonas": "narizón",
    "narizones": "narizón",
    "narizotas": "narizota",
    "narra": "narrar",
    "narraba": "narrar",
    "narrabais": "narrar",
    "narraban": "narrar",
    "narrabas": "narrar",
    "narrables": "narrable",
    "narraciones": "narración",
    "narrad": "narrar",
    "narrada": "narrar",
    "narradas": "narrar",
    "narrado": "narrar",
    "narradora": "narrador",
    "narradoras": "narrador",
    "narradores": "narrador",
    "narrados": "narrar",
    "narramos": "narrar",
    "narran": "narrar",
    "narrando": "narrar",
    "narrara": "narrar",
    "narrarais": "narrar",
    "narraran": "narrar",
    "narraras": "narrar",
    "narrare": "narrar",
    "narrareis": "narrar",
    "narraremos": "narrar",
    "narraren": "narrar",
    "narrares": "narrar",
    "narraron": "narrar",
    "narrará": "narrar",
    "narrarán": "narrar",
    "narrarás": "narrar",
    "narraré": "narrar",
    "narraréis": "narrar",
    "narraría": "narrar",
    "narraríais": "narrar",
    "narraríamos": "narrar",
    "narrarían": "narrar",
    "narrarías": "narrar",
    "narras": "narrar",
    "narrase": "narrar",
    "narraseis": "narrar",
    "narrasen": "narrar",
    "narrases": "narrar",
    "narraste": "narrar",
    "narrasteis": "narrar",
    "narrativa": "narrativo",
    "narrativas": "narrativo",
    "narrativos": "narrativo",
    "narratoria": "narratorio",
    "narratorias": "narratorio",
    "narratorios": "narratorio",
    "narre": "narrar",
    "narremos": "narrar",
    "narren": "narrar",
    "narres": "narrar",
    "narrias": "narria",
    "narro": "narrar",
    "narrábamos": "narrar",
    "narráis": "narrar",
    "narráramos": "narrar",
    "narráremos": "narrar",
    "narrásemos": "narrar",
    "narré": "narrar",
    "narréis": "narrar",
    "narró": "narrar",
    "narvales": "narval",
    "nasales": "nasal",
    "nasalice": "nasalizar",
    "nasalicemos": "nasalizar",
    "nasalicen": "nasalizar",
    "nasalices": "nasalizar",
    "nasalicé": "nasalizar",
    "nasalicéis": "nasalizar",
    "nasalidades": "nasalidad",
    "nasaliza": "nasalizar",
    "nasalizaba": "nasalizar",
    "nasalizabais": "nasalizar",
    "nasalizaban": "nasalizar",
    "nasalizabas": "nasalizar",
    "nasalizaciones": "nasalización",
    "nasalizad": "nasalizar",
    "nasalizada": "nasalizar",
    "nasalizadas": "nasalizar",
    "nasalizado": "nasalizar",
    "nasalizados": "nasalizar",
    "nasalizamos": "nasalizar",
    "nasalizan": "nasalizar",
    "nasalizando": "nasalizar",
    "nasalizara": "nasalizar",
    "nasalizarais": "nasalizar",
    "nasalizaran": "nasalizar",
    "nasalizaras": "nasalizar",
    "nasalizare": "nasalizar",
    "nasalizareis": "nasalizar",
    "nasalizaremos": "nasalizar",
    "nasalizaren": "nasalizar",
    "nasalizares": "nasalizar",
    "nasalizaron": "nasalizar",
    "nasalizará": "nasalizar",
    "nasalizarán": "nasalizar",
    "nasalizarás": "nasalizar",
    "nasalizaré": "nasalizar",
    "nasalizaréis": "nasalizar",
    "nasalizaría": "nasalizar",
    "nasalizaríais": "nasalizar",
    "nasalizaríamos": "nasalizar",
    "nasalizarían": "nasalizar",
    "nasalizarías": "nasalizar",
    "nasalizas": "nasalizar",
    "nasalizase": "nasalizar",
    "nasalizaseis": "nasalizar",
    "nasalizasen": "nasalizar",
    "nasalizases": "nasalizar",
    "nasalizaste": "nasalizar",
    "nasalizasteis": "nasalizar",
    "nasalizo": "nasalizar",
    "nasalizábamos": "nasalizar",
    "nasalizáis": "nasalizar",
    "nasalizáramos": "nasalizar",
    "nasalizáremos": "nasalizar",
    "nasalizásemos": "nasalizar",
    "nasalizó": "nasalizar",
    "nasamones": "nasamón",
    "nasardos": "nasardo",
    "nasas": "nasa",
    "nashis": "nashi",
    "nasiones": "nasión",
    "nasobasales": "nasobasal",
    "nasociliares": "nasociliar",
    "nasofaringes": "nasofaringe",
    "nasofaríngea": "nasofaríngeo",
    "nasofaríngeas": "nasofaríngeo",
    "nasofaríngeos": "nasofaríngeo",
    "nasofrontales": "nasofrontal",
    "nasolabiales": "nasolabial",
    "nasolagrimales": "nasolagrimal",
    "nasomaxilares": "nasomaxilar",
    "nasopalatina": "nasopalatino",
    "nasopalatinas": "nasopalatino",
    "nasopalatinos": "nasopalatino",
    "nasos": "naso",
    "nasosinusales": "nasosinusal",
    "nastias": "nastia",
    "nata": "nato",
    "nataciones": "natación",
    "natales": "natal",
    "natalicia": "natalicio",
    "natalicias": "natalicio",
    "natalicios": "natalicio",
    "natalidades": "natalidad",
    "natas": "nato",
    "natatoria": "natatorio",
    "natatorias": "natatorio",
    "natatorios": "natatorio",
    "naterones": "naterón",
    "nativa": "nativo",
    "nativas": "nativo",
    "natividades": "natividad",
    "nativos": "nativo",
    "natolocales": "natolocal",
    "natos": "nato",
    "natritas": "natrita",
    "natrolitas": "natrolita",
    "naturales": "natural",
    "naturalezas": "naturaleza",
    "naturalice": "naturalizar",
    "naturalicemos": "naturalizar",
    "naturalicen": "naturalizar",
    "naturalices": "naturalizar",
    "naturalicé": "naturalizar",
    "naturalicéis": "naturalizar",
    "naturalidades": "naturalidad",
    "naturalismos": "naturalismo",
    "naturalistas": "naturalista",
    "naturaliza": "naturalizar",
    "naturalizaba": "naturalizar",
    "naturalizabais": "naturalizar",
    "naturalizaban": "naturalizar",
    "naturalizabas": "naturalizar",
    "naturalizaciones": "naturalización",
    "naturalizad": "naturalizar",
    "naturalizada": "naturalizar",
    "naturalizadas": "naturalizar",
    "naturalizado": "naturalizar",
    "naturalizados": "naturalizar",
    "naturalizamos": "naturalizar",
    "naturalizan": "naturalizar",
    "naturalizando": "naturalizar",
    "naturalizara": "naturalizar",
    "naturalizarais": "naturalizar",
    "naturalizaran": "naturalizar",
    "naturalizaras": "naturalizar",
    "naturalizare": "naturalizar",
    "naturalizareis": "naturalizar",
    "naturalizaremos": "naturalizar",
    "naturalizaren": "naturalizar",
    "naturalizares": "naturalizar",
    "naturalizaron": "naturalizar",
    "naturalizará": "naturalizar",
    "naturalizarán": "naturalizar",
    "naturalizarás": "naturalizar",
    "naturalizaré": "naturalizar",
    "naturalizaréis": "naturalizar",
    "naturalizaría": "naturalizar",
    "naturalizaríais": "naturalizar",
    "naturalizaríamos": "naturalizar",
    "naturalizarían": "naturalizar",
    "naturalizarías": "naturalizar",
    "naturalizas": "naturalizar",
    "naturalizase": "naturalizar",
    "naturalizaseis": "naturalizar",
    "naturalizasen": "naturalizar",
    "naturalizases": "naturalizar",
    "naturalizaste": "naturalizar",
    "naturalizasteis": "naturalizar",
    "naturalizo": "naturalizar",
    "naturalizábamos": "naturalizar",
    "naturalizáis": "naturalizar",
    "naturalizáramos": "naturalizar",
    "naturalizáremos": "naturalizar",
    "naturalizásemos": "naturalizar",
    "naturalizó": "naturalizar",
    "naturas": "natura",
    "naturismos": "naturismo",
    "naturistas": "naturista",
    "naturopatías": "naturopatía",
    "natátiles": "natátil",
    "naufraga": "naufragar",
    "naufragaba": "naufragar",
    "naufragabais": "naufragar",
    "naufragaban": "naufragar",
    "naufragabas": "naufragar",
    "naufragad": "naufragar",
    "naufragada": "naufragar",
    "naufragadas": "naufragar",
    "naufragado": "naufragar",
    "naufragados": "naufragar",
    "naufragamos": "naufragar",
    "naufragan": "naufragar",
    "naufragando": "naufragar",
    "naufragara": "naufragar",
    "naufragarais": "naufragar",
    "naufragaran": "naufragar",
    "naufragaras": "naufragar",
    "naufragare": "naufragar",
    "naufragareis": "naufragar",
    "naufragaremos": "naufragar",
    "naufragaren": "naufragar",
    "naufragares": "naufragar",
    "naufragaron": "naufragar",
    "naufragará": "naufragar",
    "naufragarán": "naufragar",
    "naufragarás": "naufragar",
    "naufragaré": "naufragar",
    "naufragaréis": "naufragar",
    "naufragaría": "naufragar",
    "naufragaríais": "naufragar",
    "naufragaríamos": "naufragar",
    "naufragarían": "naufragar",
    "naufragarías": "naufragar",
    "naufragas": "naufragar",
    "naufragase": "naufragar",
    "naufragaseis": "naufragar",
    "naufragasen": "naufragar",
    "naufragases": "naufragar",
    "naufragaste": "naufragar",
    "naufragasteis": "naufragar",
    "naufragios": "naufragio",
    "naufrago": "naufragar",
    "naufrague": "naufragar",
    "naufraguemos": "naufragar",
    "naufraguen": "naufragar",
    "naufragues": "naufragar",
    "naufragué": "naufragar",
    "naufraguéis": "naufragar",
    "naufragábamos": "naufragar",
    "naufragáis": "naufragar",
    "naufragáramos": "naufragar",
    "naufragáremos": "naufragar",
    "naufragásemos": "naufragar",
    "naufragó": "naufragar",
    "naumaquias": "naumaquia",
    "nausea": "nausear",
    "nauseaba": "nausear",
    "nauseabais": "nausear",
    "nauseaban": "nausear",
    "nauseabas": "nausear",
    "nauseabunda": "nauseabundo",
    "nauseabundas": "nauseabundo",
    "nauseabundos": "nauseabundo",
    "nausead": "nausear",
    "nauseada": "nausear",
    "nauseadas": "nausear",
    "nauseado": "nausear",
    "nauseados": "nausear",
    "nauseamos": "nausear",
    "nausean": "nausear",
    "nauseando": "nausear",
    "nauseara": "nausear",
    "nausearais": "nausear",
    "nausearan": "nausear",
    "nausearas": "nausear",
    "nauseare": "nausear",
    "nauseareis": "nausear",
    "nausearemos": "nausear",
    "nausearen": "nausear",
    "nauseares": "nausear",
    "nausearon": "nausear",
    "nauseará": "nausear",
    "nausearán": "nausear",
    "nausearás": "nausear",
    "nausearé": "nausear",
    "nausearéis": "nausear",
    "nausearía": "nausear",
    "nausearíais": "nausear",
    "nausearíamos": "nausear",
    "nausearían": "nausear",
    "nausearías": "nausear",
    "nauseas": "nausear",
    "nausease": "nausear",
    "nauseaseis": "nausear",
    "nauseasen": "nausear",
    "nauseases": "nausear",
    "nauseaste": "nausear",
    "nauseasteis": "nausear",
    "nausee": "nausear",
    "nauseemos": "nausear",
    "nauseen": "nausear",
    "nausees": "nausear",
    "nauseo": "nausear",
    "nauseábamos": "nausear",
    "nauseáis": "nausear",
    "nauseáramos": "nausear",
    "nauseáremos": "nausear",
    "nauseásemos": "nausear",
    "nauseé": "nausear",
    "nauseéis": "nausear",
    "nauseó": "nausear",
    "nautas": "nauta",
    "nautilos": "nautilo",
    "nautófonos": "nautófono",
    "navajadas": "navajada",
    "navajas": "navaja",
    "navajazos": "navajazo",
    "navajera": "navajero",
    "navajeras": "navajero",
    "navajeros": "navajero",
    "navajos": "navajo",
    "navales": "naval",
    "navarcas": "navarca",
    "navarra": "navarro",
    "navarras": "navarro",
    "navarros": "navarro",
    "navas": "nava",
    "navecillas": "navecilla",
    "navega": "navegar",
    "navegaba": "navegar",
    "navegabais": "navegar",
    "navegaban": "navegar",
    "navegabas": "navegar",
    "navegabilidades": "navegabilidad",
    "navegables": "navegable",
    "navegaciones": "navegación",
    "navegad": "navegar",
    "navegada": "navegar",
    "navegadas": "navegar",
    "navegado": "navegar",
    "navegadora": "navegador",
    "navegadoras": "navegador",
    "navegadores": "navegador",
    "navegados": "navegar",
    "navegamos": "navegar",
    "navegan": "navegar",
    "navegando": "navegar",
    "navegantes": "navegante",
    "navegara": "navegar",
    "navegarais": "navegar",
    "navegaran": "navegar",
    "navegaras": "navegar",
    "navegare": "navegar",
    "navegareis": "navegar",
    "navegaremos": "navegar",
    "navegaren": "navegar",
    "navegares": "navegar",
    "navegaron": "navegar",
    "navegará": "navegar",
    "navegarán": "navegar",
    "navegarás": "navegar",
    "navegaré": "navegar",
    "navegaréis": "navegar",
    "navegaría": "navegar",
    "navegaríais": "navegar",
    "navegaríamos": "navegar",
    "navegarían": "navegar",
    "navegarías": "navegar",
    "navegas": "navegar",
    "navegase": "navegar",
    "navegaseis": "navegar",
    "navegasen": "navegar",
    "navegases": "navegar",
    "navegaste": "navegar",
    "navegasteis": "navegar",
    "navego": "navegar",
    "navegue": "navegar",
    "naveguemos": "navegar",
    "naveguen": "navegar",
    "navegues": "navegar",
    "navegué": "navegar",
    "naveguéis": "navegar",
    "navegábamos": "navegar",
    "navegáis": "navegar",
    "navegáramos": "navegar",
    "navegáremos": "navegar",
    "navegásemos": "navegar",
    "navegó": "navegar",
    "naves": "nave",
    "navetas": "naveta",
    "naviculares": "navicular",
    "navidades": "navidad",
    "navideña": "navideño",
    "navideñas": "navideño",
    "navideños": "navideño",
    "naviera": "naviero",
    "navieras": "naviero",
    "navieros": "naviero",
    "navículas": "navícula",
    "navíos": "navío",
    "nazarena": "nazareno",
    "nazarenas": "nazareno",
    "nazarenos": "nazareno",
    "nazaritas": "nazarita",
    "nazaríes": "nazarí",
    "nazarís": "nazarí",
    "nazca": "nacer",
    "nazcamos": "nacer",
    "nazcan": "nacer",
    "nazcas": "nacer",
    "nazco": "nacer",
    "nazcáis": "nacer",
    "nazificaciones": "nazificación",
    "nazis": "nazi",
    "nazismos": "nazismo",
    "naífs": "naíf",
    "naïfs": "naïf",
    "neandertales": "neandertal",
    "neandertaloides": "neandertaloide",
    "neanderthales": "neanderthal",
    "nearcas": "nearca",
    "nebeles": "nebel",
    "nebladuras": "nebladura",
    "neblinas": "neblina",
    "neblinosa": "neblinoso",
    "neblinosas": "neblinoso",
    "neblinosos": "neblinoso",
    "neblíes": "neblí",
    "nebrales": "nebral",
    "nebredas": "nebreda",
    "nebrinas": "nebrina",
    "nebros": "nebro",
    "nebulada": "nebulado",
    "nebuladas": "nebulado",
    "nebulados": "nebulado",
    "nebulares": "nebular",
    "nebulizaciones": "nebulización",
    "nebulizadora": "nebulizador",
    "nebulizadoras": "nebulizador",
    "nebulizadores": "nebulizador",
    "nebulosa": "nebuloso",
    "nebulosas": "nebuloso",
    "nebulosidades": "nebulosidad",
    "nebulosos": "nebuloso",
    "necea": "necear",
    "neceaba": "necear",
    "neceabais": "necear",
    "neceaban": "necear",
    "neceabas": "necear",
    "necead": "necear",
    "neceada": "necear",
    "neceadas": "necear",
    "neceado": "necear",
    "neceados": "necear",
    "neceamos": "necear",
    "necean": "necear",
    "neceando": "necear",
    "neceara": "necear",
    "necearais": "necear",
    "necearan": "necear",
    "necearas": "necear",
    "neceare": "necear",
    "neceareis": "necear",
    "necearemos": "necear",
    "necearen": "necear",
    "neceares": "necear",
    "necearon": "necear",
    "neceará": "necear",
    "necearán": "necear",
    "necearás": "necear",
    "necearé": "necear",
    "necearéis": "necear",
    "necearía": "necear",
    "necearíais": "necear",
    "necearíamos": "necear",
    "necearían": "necear",
    "necearías": "necear",
    "neceas": "necear",
    "necease": "necear",
    "neceaseis": "necear",
    "neceasen": "necear",
    "neceases": "necear",
    "neceaste": "necear",
    "neceasteis": "necear",
    "necedades": "necedad",
    "necee": "necear",
    "neceemos": "necear",
    "neceen": "necear",
    "necees": "necear",
    "neceo": "necear",
    "necesaria": "necesario",
    "necesarias": "necesario",
    "necesariedades": "necesariedad",
    "necesarios": "necesario",
    "neceseres": "neceser",
    "necesidades": "necesidad",
    "necesita": "necesitar",
    "necesitaba": "necesitar",
    "necesitabais": "necesitar",
    "necesitaban": "necesitar",
    "necesitabas": "necesitar",
    "necesitad": "necesitar",
    "necesitada": "necesitar",
    "necesitadas": "necesitar",
    "necesitado": "necesitar",
    "necesitados": "necesitar",
    "necesitamos": "necesitar",
    "necesitan": "necesitar",
    "necesitando": "necesitar",
    "necesitara": "necesitar",
    "necesitarais": "necesitar",
    "necesitaran": "necesitar",
    "necesitaras": "necesitar",
    "necesitare": "necesitar",
    "necesitareis": "necesitar",
    "necesitaremos": "necesitar",
    "necesitaren": "necesitar",
    "necesitares": "necesitar",
    "necesitaron": "necesitar",
    "necesitará": "necesitar",
    "necesitarán": "necesitar",
    "necesitarás": "necesitar",
    "necesitaré": "necesitar",
    "necesitaréis": "necesitar",
    "necesitaría": "necesitar",
    "necesitaríais": "necesitar",
    "necesitaríamos": "necesitar",
    "necesitarían": "necesitar",
    "necesitarías": "necesitar",
    "necesitas": "necesitar",
    "necesitase": "necesitar",
    "necesitaseis": "necesitar",
    "necesitasen": "necesitar",
    "necesitases": "necesitar",
    "necesitaste": "necesitar",
    "necesitasteis": "necesitar",
    "necesite": "necesitar",
    "necesitemos": "necesitar",
    "necesiten": "necesitar",
    "necesites": "necesitar",
    "necesito": "necesitar",
    "necesitábamos": "necesitar",
    "necesitáis": "necesitar",
    "necesitáramos": "necesitar",
    "necesitáremos": "necesitar",
    "necesitásemos": "necesitar",
    "necesité": "necesitar",
    "necesitéis": "necesitar",
    "necesitó": "necesitar",
    "neceábamos": "necear",
    "neceáis": "necear",
    "neceáramos": "necear",
    "neceáremos": "necear",
    "neceásemos": "necear",
    "neceé": "necear",
    "neceéis": "necear",
    "neceó": "necear",
    "necia": "necio",
    "necias": "necio",
    "necios": "necio",
    "necrobiótica": "necrobiótico",
    "necrobióticas": "necrobiótico",
    "necrobióticos": "necrobiótico",
    "necrofilias": "necrofilia",
    "necroidentificaciones": "necroidentificación",
    "necrologías": "necrología",
    "necrológica": "necrológico",
    "necrológicas": "necrológico",
    "necrológicos": "necrológico",
    "necromancias": "necromancia",
    "necropsias": "necropsia",
    "necropulpectomías": "necropulpectomía",
    "necropulpotomías": "necropulpotomía",
    "necrorreseñas": "necrorreseña",
    "necroscopias": "necroscopia",
    "necroscópica": "necroscópico",
    "necroscópicas": "necroscópico",
    "necroscópicos": "necroscópico",
    "necrófaga": "necrófago",
    "necrófagas": "necrófago",
    "necrófagos": "necrófago",
    "necrófila": "necrófilo",
    "necrófilas": "necrófilo",
    "necrófilos": "necrófilo",
    "necrófora": "necróforo",
    "necróforas": "necróforo",
    "necróforos": "necróforo",
    "necrótica": "necrótico",
    "necróticas": "necrótico",
    "necróticos": "necrótico",
    "nectarina": "nectarino",
    "nectarinas": "nectarino",
    "nectarinos": "nectarino",
    "nectarios": "nectario",
    "nectárea": "nectáreo",
    "nectáreas": "nectáreo",
    "nectáreos": "nectáreo",
    "neerlandesa": "neerlandés",
    "neerlandesas": "neerlandés",
    "neerlandeses": "neerlandés",
    "nefanda": "nefando",
    "nefandas": "nefando",
    "nefandos": "nefando",
    "nefasta": "nefasto",
    "nefastas": "nefasto",
    "nefastos": "nefasto",
    "nefelinas": "nefelina",
    "nefelitas": "nefelita",
    "nefelometrías": "nefelometría",
    "nefelómetros": "nefelómetro",
    "nefoscopios": "nefoscopio",
    "nefrectomías": "nefrectomía",
    "nefridios": "nefridio",
    "nefritas": "nefrita",
    "nefrologías": "nefrología",
    "nefrológica": "nefrológico",
    "nefrológicas": "nefrológico",
    "nefrológicos": "nefrológico",
    "nefronas": "nefrona",
    "nefrones": "nefrón",
    "nefropatías": "nefropatía",
    "nefrítica": "nefrítico",
    "nefríticas": "nefrítico",
    "nefríticos": "nefrítico",
    "nefróloga": "nefrólogo",
    "nefrólogas": "nefrólogo",
    "nefrólogos": "nefrólogo",
    "nefómetros": "nefómetro",
    "negaba": "negar",
    "negabais": "negar",
    "negaban": "negar",
    "negabas": "negar",
    "negables": "negable",
    "negaciones": "negación",
    "negacionistas": "negacionista",
    "negad": "negar",
    "negada": "negar",
    "negadas": "negar",
    "negado": "negar",
    "negadora": "negador",
    "negadoras": "negador",
    "negadores": "negador",
    "negados": "negar",
    "negamos": "negar",
    "negando": "negar",
    "negara": "negar",
    "negarais": "negar",
    "negaran": "negar",
    "negaras": "negar",
    "negare": "negar",
    "negareis": "negar",
    "negaremos": "negar",
    "negaren": "negar",
    "negares": "negar",
    "negaron": "negar",
    "negará": "negar",
    "negarán": "negar",
    "negarás": "negar",
    "negaré": "negar",
    "negaréis": "negar",
    "negaría": "negar",
    "negaríais": "negar",
    "negaríamos": "negar",
    "negarían": "negar",
    "negarías": "negar",
    "negase": "negar",
    "negaseis": "negar",
    "negasen": "negar",
    "negases": "negar",
    "negaste": "negar",
    "negasteis": "negar",
    "negativa": "negativo",
    "negativas": "negativo",
    "negatividades": "negatividad",
    "negativos": "negativo",
    "negatores": "negator",
    "negatoria": "negatorio",
    "negatorias": "negatorio",
    "negatorios": "negatorio",
    "negatoscopios": "negatoscopio",
    "negatrones": "negatrón",
    "neglige": "negligir",
    "negligen": "negligir",
    "negligencias": "negligencia",
    "negligentes": "negligente",
    "negliges": "negligir",
    "negligibles": "negligible",
    "negligid": "negligir",
    "negligida": "negligir",
    "negligidas": "negligir",
    "negligido": "negligir",
    "negligidos": "negligir",
    "negligiendo": "negligir",
    "negligiera": "negligir",
    "negligierais": "negligir",
    "negligieran": "negligir",
    "negligieras": "negligir",
    "negligiere": "negligir",
    "negligiereis": "negligir",
    "negligieren": "negligir",
    "negligieres": "negligir",
    "negligieron": "negligir",
    "negligiese": "negligir",
    "negligieseis": "negligir",
    "negligiesen": "negligir",
    "negligieses": "negligir",
    "negligimos": "negligir",
    "negligiremos": "negligir",
    "negligirá": "negligir",
    "negligirán": "negligir",
    "negligirás": "negligir",
    "negligiré": "negligir",
    "negligiréis": "negligir",
    "negligiría": "negligir",
    "negligiríais": "negligir",
    "negligiríamos": "negligir",
    "negligirían": "negligir",
    "negligirías": "negligir",
    "negligiste": "negligir",
    "negligisteis": "negligir",
    "negligiéramos": "negligir",
    "negligiéremos": "negligir",
    "negligiésemos": "negligir",
    "negligió": "negligir",
    "negligés": "negligé",
    "negligí": "negligir",
    "negligía": "negligir",
    "negligíais": "negligir",
    "negligíamos": "negligir",
    "negligían": "negligir",
    "negligías": "negligir",
    "negligís": "negligir",
    "neglija": "negligir",
    "neglijamos": "negligir",
    "neglijan": "negligir",
    "neglijas": "negligir",
    "neglijo": "negligir",
    "neglijáis": "negligir",
    "negocia": "negociar",
    "negociaba": "negociar",
    "negociabais": "negociar",
    "negociaban": "negociar",
    "negociabas": "negociar",
    "negociables": "negociable",
    "negociaciones": "negociación",
    "negociad": "negociar",
    "negociada": "negociar",
    "negociadas": "negociar",
    "negociado": "negociar",
    "negociadora": "negociador",
    "negociadoras": "negociador",
    "negociadores": "negociador",
    "negociados": "negociar",
    "negociales": "negocial",
    "negociamos": "negociar",
    "negocian": "negociar",
    "negociando": "negociar",
    "negociantes": "negociante",
    "negociara": "negociar",
    "negociarais": "negociar",
    "negociaran": "negociar",
    "negociaras": "negociar",
    "negociare": "negociar",
    "negociareis": "negociar",
    "negociaremos": "negociar",
    "negociaren": "negociar",
    "negociares": "negociar",
    "negociaron": "negociar",
    "negociará": "negociar",
    "negociarán": "negociar",
    "negociarás": "negociar",
    "negociaré": "negociar",
    "negociaréis": "negociar",
    "negociaría": "negociar",
    "negociaríais": "negociar",
    "negociaríamos": "negociar",
    "negociarían": "negociar",
    "negociarías": "negociar",
    "negocias": "negociar",
    "negociase": "negociar",
    "negociaseis": "negociar",
    "negociasen": "negociar",
    "negociases": "negociar",
    "negociaste": "negociar",
    "negociasteis": "negociar",
    "negocie": "negociar",
    "negociemos": "negociar",
    "negocien": "negociar",
    "negocies": "negociar",
    "negocio": "negociar",
    "negocios": "negocio",
    "negociábamos": "negociar",
    "negociáis": "negociar",
    "negociáramos": "negociar",
    "negociáremos": "negociar",
    "negociásemos": "negociar",
    "negocié": "negociar",
    "negociéis": "negociar",
    "negoció": "negociar",
    "negra": "negro",
    "negrales": "negral",
    "negras": "negro",
    "negrea": "negrear",
    "negreaba": "negrear",
    "negreabais": "negrear",
    "negreaban": "negrear",
    "negreabas": "negrear",
    "negread": "negrear",
    "negreada": "negrear",
    "negreadas": "negrear",
    "negreado": "negrear",
    "negreados": "negrear",
    "negreamos": "negrear",
    "negrean": "negrear",
    "negreando": "negrear",
    "negreara": "negrear",
    "negrearais": "negrear",
    "negrearan": "negrear",
    "negrearas": "negrear",
    "negreare": "negrear",
    "negreareis": "negrear",
    "negrearemos": "negrear",
    "negrearen": "negrear",
    "negreares": "negrear",
    "negrearon": "negrear",
    "negreará": "negrear",
    "negrearán": "negrear",
    "negrearás": "negrear",
    "negrearé": "negrear",
    "negrearéis": "negrear",
    "negrearía": "negrear",
    "negrearíais": "negrear",
    "negrearíamos": "negrear",
    "negrearían": "negrear",
    "negrearías": "negrear",
    "negreas": "negrear",
    "negrease": "negrear",
    "negreaseis": "negrear",
    "negreasen": "negrear",
    "negreases": "negrear",
    "negreaste": "negrear",
    "negreasteis": "negrear",
    "negrece": "negrecer",
    "negreced": "negrecer",
    "negrecemos": "negrecer",
    "negrecen": "negrecer",
    "negreceremos": "negrecer",
    "negrecerá": "negrecer",
    "negrecerán": "negrecer",
    "negrecerás": "negrecer",
    "negreceré": "negrecer",
    "negreceréis": "negrecer",
    "negrecería": "negrecer",
    "negreceríais": "negrecer",
    "negreceríamos": "negrecer",
    "negrecerían": "negrecer",
    "negrecerías": "negrecer",
    "negreces": "negrecer",
    "negrecida": "negrecer",
    "negrecidas": "negrecer",
    "negrecido": "negrecer",
    "negrecidos": "negrecer",
    "negreciendo": "negrecer",
    "negreciera": "negrecer",
    "negrecierais": "negrecer",
    "negrecieran": "negrecer",
    "negrecieras": "negrecer",
    "negreciere": "negrecer",
    "negreciereis": "negrecer",
    "negrecieren": "negrecer",
    "negrecieres": "negrecer",
    "negrecieron": "negrecer",
    "negreciese": "negrecer",
    "negrecieseis": "negrecer",
    "negreciesen": "negrecer",
    "negrecieses": "negrecer",
    "negrecimos": "negrecer",
    "negreciste": "negrecer",
    "negrecisteis": "negrecer",
    "negreciéramos": "negrecer",
    "negreciéremos": "negrecer",
    "negreciésemos": "negrecer",
    "negreció": "negrecer",
    "negrecéis": "negrecer",
    "negrecí": "negrecer",
    "negrecía": "negrecer",
    "negrecíais": "negrecer",
    "negrecíamos": "negrecer",
    "negrecían": "negrecer",
    "negrecías": "negrecer",
    "negree": "negrear",
    "negreemos": "negrear",
    "negreen": "negrear",
    "negrees": "negrear",
    "negreguea": "negreguear",
    "negregueaba": "negreguear",
    "negregueabais": "negreguear",
    "negregueaban": "negreguear",
    "negregueabas": "negreguear",
    "negreguead": "negreguear",
    "negregueada": "negreguear",
    "negregueadas": "negreguear",
    "negregueado": "negreguear",
    "negregueados": "negreguear",
    "negregueamos": "negreguear",
    "negreguean": "negreguear",
    "negregueando": "negreguear",
    "negregueara": "negreguear",
    "negreguearais": "negreguear",
    "negreguearan": "negreguear",
    "negreguearas": "negreguear",
    "negregueare": "negreguear",
    "negregueareis": "negreguear",
    "negreguearemos": "negreguear",
    "negreguearen": "negreguear",
    "negregueares": "negreguear",
    "negreguearon": "negreguear",
    "negregueará": "negreguear",
    "negreguearán": "negreguear",
    "negreguearás": "negreguear",
    "negreguearé": "negreguear",
    "negreguearéis": "negreguear",
    "negreguearía": "negreguear",
    "negreguearíais": "negreguear",
    "negreguearíamos": "negreguear",
    "negreguearían": "negreguear",
    "negreguearías": "negreguear",
    "negregueas": "negreguear",
    "negreguease": "negreguear",
    "negregueaseis": "negreguear",
    "negregueasen": "negreguear",
    "negregueases": "negreguear",
    "negregueaste": "negreguear",
    "negregueasteis": "negreguear",
    "negreguee": "negreguear",
    "negregueemos": "negreguear",
    "negregueen": "negreguear",
    "negreguees": "negreguear",
    "negregueo": "negreguear",
    "negregueábamos": "negreguear",
    "negregueáis": "negreguear",
    "negregueáramos": "negreguear",
    "negregueáremos": "negreguear",
    "negregueásemos": "negreguear",
    "negregueé": "negreguear",
    "negregueéis": "negreguear",
    "negregueó": "negreguear",
    "negreo": "negrear",
    "negrera": "negrero",
    "negreras": "negrero",
    "negreros": "negrero",
    "negrerías": "negrería",
    "negrezca": "negrecer",
    "negrezcamos": "negrecer",
    "negrezcan": "negrecer",
    "negrezcas": "negrecer",
    "negrezco": "negrecer",
    "negrezcáis": "negrecer",
    "negreábamos": "negrear",
    "negreáis": "negrear",
    "negreáramos": "negrear",
    "negreáremos": "negrear",
    "negreásemos": "negrear",
    "negreé": "negrear",
    "negreéis": "negrear",
    "negreó": "negrear",
    "negrillas": "negrilla",
    "negrita": "negrito",
    "negritas": "negrito",
    "negritos": "negrito",
    "negritudes": "negritud",
    "negroides": "negroide",
    "negrones": "negrón",
    "negrores": "negror",
    "negros": "negro",
    "negruras": "negrura",
    "negruzca": "negruzco",
    "negruzcas": "negruzco",
    "negruzcos": "negruzco",
    "negrófila": "negrófilo",
    "negrófilas": "negrófilo",
    "negrófilos": "negrófilo",
    "neguemos": "negar",
    "neguijones": "neguijón",
    "neguillas": "neguilla",
    "negundos": "negundo",
    "negué": "negar",
    "neguéis": "negar",
    "negábamos": "negar",
    "negáis": "negar",
    "negáramos": "negar",
    "negáremos": "negar",
    "negásemos": "negar",
    "negó": "negar",
    "nematelmintos": "nematelminto",
    "nematicidas": "nematicida",
    "nematocidas": "nematocida",
    "nematodos": "nematodo",
    "nematomorfos": "nematomorfo",
    "nematosomas": "nematosoma",
    "nemertinos": "nemertino",
    "nemorales": "nemoral",
    "nemorosa": "nemoroso",
    "nemorosas": "nemoroso",
    "nemorosos": "nemoroso",
    "nemotécnica": "nemotécnico",
    "nemotécnicas": "nemotécnico",
    "nemotécnicos": "nemotécnico",
    "nemátodos": "nemátodo",
    "nena": "nene",
    "nenas": "nene",
    "nenes": "nene",
    "nenias": "nenia",
    "nenúfares": "nenúfar",
    "neoantígenos": "neoantígeno",
    "neoarsfenaminas": "neoarsfenamina",
    "neocapitalismos": "neocapitalismo",
    "neocapitalistas": "neocapitalista",
    "neocatolicismos": "neocatolicismo",
    "neocatólica": "neocatólico",
    "neocatólicas": "neocatólico",
    "neocatólicos": "neocatólico",
    "neocelandeses": "neocelandés",
    "neocerebelos": "neocerebelo",
    "neocincófenos": "neocincófeno",
    "neoclasicismos": "neoclasicismo",
    "neoclásica": "neoclásico",
    "neoclásicas": "neoclásico",
    "neoclásicos": "neoclásico",
    "neocolonialismos": "neocolonialismo",
    "neocolonialistas": "neocolonialista",
    "neocolonias": "neocolonia",
    "neoconductismos": "neoconductismo",
    "neocriticismos": "neocriticismo",
    "neocórtexes": "neocórtex",
    "neodarwinismos": "neodarwinismo",
    "neodarwinistas": "neodarwinista",
    "neodimios": "neodimio",
    "neoescolásticas": "neoescolástica",
    "neoevolucionismos": "neoevolucionismo",
    "neofascismos": "neofascismo",
    "neofascistas": "neofascista",
    "neoformaciones": "neoformación",
    "neogramáticas": "neogramática",
    "neogramáticos": "neogramático",
    "neogranadina": "neogranadino",
    "neogranadinas": "neogranadino",
    "neogranadinos": "neogranadino",
    "neogriega": "neogriego",
    "neogriegas": "neogriego",
    "neogriegos": "neogriego",
    "neogótica": "neogótico",
    "neogóticas": "neogótico",
    "neogóticos": "neogótico",
    "neohexanos": "neohexano",
    "neohilomios": "neohilomio",
    "neoimpresionismos": "neoimpresionismo",
    "neoimpresionistas": "neoimpresionista",
    "neokantismos": "neokantismo",
    "neokantistas": "neokantista",
    "neokeynesiana": "neokeynesiano",
    "neokeynesianas": "neokeynesiano",
    "neokeynesianismos": "neokeynesianismo",
    "neokeynesianos": "neokeynesiano",
    "neolatina": "neolatino",
    "neolatinas": "neolatino",
    "neolatinos": "neolatino",
    "neoliberales": "neoliberal",
    "neoliberalismos": "neoliberalismo",
    "neolingüist": "neolingüista",
    "neolingüistas": "neolingüista",
    "neolingüists": "neolingüista",
    "neolingüísticas": "neolingüística",
    "neolocales": "neolocal",
    "neolocalidades": "neolocalidad",
    "neologismos": "neologismo",
    "neologías": "neología",
    "neolítica": "neolítico",
    "neolíticas": "neolítico",
    "neolíticos": "neolítico",
    "neológica": "neológico",
    "neológicas": "neológico",
    "neológicos": "neológico",
    "neomalthusianismos": "neomalthusianismo",
    "neomaltusianismos": "neomaltusianismo",
    "neomenias": "neomenia",
    "neomicinas": "neomicina",
    "neonata": "neonato",
    "neonatales": "neonatal",
    "neonatas": "neonato",
    "neonatos": "neonato",
    "neonazis": "neonazi",
    "neonazismos": "neonazismo",
    "neones": "neón",
    "neopalios": "neopalio",
    "neoplasias": "neoplasia",
    "neoplasmas": "neoplasma",
    "neoplatonicismos": "neoplatonicismo",
    "neoplatonismos": "neoplatonismo",
    "neoplatónica": "neoplatónico",
    "neoplatónicas": "neoplatónico",
    "neoplatónicos": "neoplatónico",
    "neoplásica": "neoplásico",
    "neoplásicas": "neoplásico",
    "neoplásicos": "neoplásico",
    "neopositivismos": "neopositivismo",
    "neopositivistas": "neopositivista",
    "neoprenos": "neopreno",
    "neoprofesionales": "neoprofesional",
    "neoramas": "neorama",
    "neorrealismos": "neorrealismo",
    "neorrealistas": "neorrealista",
    "neorromanticismos": "neorromanticismo",
    "neorrománicos": "neorrománico",
    "neosilicatos": "neosilicato",
    "neostigminas": "neostigmina",
    "neosurrealismos": "neosurrealismo",
    "neotectónicas": "neotectónica",
    "neotetracos": "neotetraco",
    "neotomismos": "neotomismo",
    "neotropicales": "neotropical",
    "neotécnicas": "neotécnica",
    "neovasculares": "neovascular",
    "neoyorkina": "neoyorkino",
    "neoyorkinas": "neoyorkino",
    "neoyorkinos": "neoyorkino",
    "neoyorquina": "neoyorquino",
    "neoyorquinas": "neoyorquino",
    "neoyorquinos": "neoyorquino",
    "neozelandesa": "neozelandés",
    "neozelandesas": "neozelandés",
    "neozelandeses": "neozelandés",
    "neozelandésa": "neozelandés",
    "neozelandésas": "neozelandés",
    "neozelandéses": "neozelandés",
    "nepalesa": "nepalés",
    "nepalesas": "nepalés",
    "nepaleses": "nepalés",
    "nepalíes": "nepalí",
    "nepalís": "nepalí",
    "nepentáceas": "nepentácea",
    "neperes": "neper",
    "neperiana": "neperiano",
    "neperianas": "neperiano",
    "neperianos": "neperiano",
    "nepers": "neper",
    "nepotes": "nepote",
    "nepotismos": "nepotismo",
    "neptuniana": "neptuniano",
    "neptunianas": "neptuniano",
    "neptunianos": "neptuniano",
    "neptunios": "neptunio",
    "neptunismos": "neptunismo",
    "neptunistas": "neptunista",
    "neptúnea": "neptúneo",
    "neptúneas": "neptúneo",
    "neptúneos": "neptúneo",
    "neptúnica": "neptúnico",
    "neptúnicas": "neptúnico",
    "neptúnicos": "neptúnico",
    "nepómetros": "nepómetro",
    "nequicias": "nequicia",
    "nereida": "nereido",
    "nereidas": "nereido",
    "nereidos": "nereido",
    "nerolies": "neroli",
    "nerolis": "neroli",
    "nerolíes": "nerolí",
    "nerolís": "nerolí",
    "nerones": "nerón",
    "neroniana": "neroniano",
    "neronianas": "neroniano",
    "neronianos": "neroniano",
    "nervada": "nervado",
    "nervadas": "nervado",
    "nervados": "nervado",
    "nervaduras": "nervadura",
    "nervina": "nervino",
    "nervinas": "nervino",
    "nervinos": "nervino",
    "nervios": "nervio",
    "nerviosa": "nervioso",
    "nerviosas": "nervioso",
    "nerviosidades": "nerviosidad",
    "nerviosismos": "nerviosismo",
    "nerviosos": "nervioso",
    "nervosidades": "nervosidad",
    "nervuda": "nervudo",
    "nervudas": "nervudo",
    "nervudos": "nervudo",
    "nervurada": "nervurado",
    "nervuradas": "nervurado",
    "nervurados": "nervurado",
    "nervuras": "nervura",
    "nerítica": "nerítico",
    "neríticas": "nerítico",
    "neríticos": "nerítico",
    "nesciencias": "nesciencia",
    "nescientes": "nesciente",
    "nesga": "nesgar",
    "nesgaba": "nesgar",
    "nesgabais": "nesgar",
    "nesgaban": "nesgar",
    "nesgabas": "nesgar",
    "nesgad": "nesgar",
    "nesgada": "nesgar",
    "nesgadas": "nesgar",
    "nesgado": "nesgar",
    "nesgados": "nesgar",
    "nesgamos": "nesgar",
    "nesgan": "nesgar",
    "nesgando": "nesgar",
    "nesgara": "nesgar",
    "nesgarais": "nesgar",
    "nesgaran": "nesgar",
    "nesgaras": "nesgar",
    "nesgare": "nesgar",
    "nesgareis": "nesgar",
    "nesgaremos": "nesgar",
    "nesgaren": "nesgar",
    "nesgares": "nesgar",
    "nesgaron": "nesgar",
    "nesgará": "nesgar",
    "nesgarán": "nesgar",
    "nesgarás": "nesgar",
    "nesgaré": "nesgar",
    "nesgaréis": "nesgar",
    "nesgaría": "nesgar",
    "nesgaríais": "nesgar",
    "nesgaríamos": "nesgar",
    "nesgarían": "nesgar",
    "nesgarías": "nesgar",
    "nesgas": "nesgar",
    "nesgase": "nesgar",
    "nesgaseis": "nesgar",
    "nesgasen": "nesgar",
    "nesgases": "nesgar",
    "nesgaste": "nesgar",
    "nesgasteis": "nesgar",
    "nesgo": "nesgar",
    "nesgue": "nesgar",
    "nesguemos": "nesgar",
    "nesguen": "nesgar",
    "nesgues": "nesgar",
    "nesgué": "nesgar",
    "nesguéis": "nesgar",
    "nesgábamos": "nesgar",
    "nesgáis": "nesgar",
    "nesgáramos": "nesgar",
    "nesgáremos": "nesgar",
    "nesgásemos": "nesgar",
    "nesgó": "nesgar",
    "nesosilicatos": "nesosilicato",
    "nestoriana": "nestoriano",
    "nestorianas": "nestoriano",
    "nestorianismos": "nestorianismo",
    "nestorianos": "nestoriano",
    "neta": "neto",
    "netas": "neto",
    "netos": "neto",
    "neumas": "neuma",
    "neumatizaciones": "neumatización",
    "neumatolítica": "neumatolítico",
    "neumatolíticas": "neumatolítico",
    "neumatolíticos": "neumatolítico",
    "neumectomías": "neumectomía",
    "neumococos": "neumococo",
    "neumoconiótica": "neumoconiótico",
    "neumoconióticas": "neumoconiótico",
    "neumoconióticos": "neumoconiótico",
    "neumogástrica": "neumogástrico",
    "neumogástricas": "neumogástrico",
    "neumogástricos": "neumogástrico",
    "neumologías": "neumología",
    "neumológica": "neumológico",
    "neumológicas": "neumológico",
    "neumológicos": "neumológico",
    "neumonías": "neumonía",
    "neumática": "neumático",
    "neumáticas": "neumático",
    "neumáticos": "neumático",
    "neumóloga": "neumólogo",
    "neumólogas": "neumólogo",
    "neumólogos": "neumólogo",
    "neumónica": "neumónico",
    "neumónicas": "neumónico",
    "neumónicos": "neumónico",
    "neurales": "neural",
    "neuralgias": "neuralgia",
    "neurastenias": "neurastenia",
    "neurasténica": "neurasténico",
    "neurasténicas": "neurasténico",
    "neurasténicos": "neurasténico",
    "neurectomías": "neurectomía",
    "neurinas": "neurina",
    "neurinomas": "neurinoma",
    "neuritas": "neurita",
    "neuroblastos": "neuroblasto",
    "neurociencias": "neurociencia",
    "neurocirugías": "neurocirugía",
    "neurocirujana": "neurocirujano",
    "neurocirujanas": "neurocirujano",
    "neurocirujanos": "neurocirujano",
    "neurocitos": "neurocito",
    "neurocronáxica": "neurocronáxico",
    "neurocronáxicas": "neurocronáxico",
    "neurocronáxicos": "neurocronáxico",
    "neurocráneos": "neurocráneo",
    "neurodegenerativa": "neurodegenerativo",
    "neurodegenerativas": "neurodegenerativo",
    "neurodegenerativos": "neurodegenerativo",
    "neuroejes": "neuroeje",
    "neuroepitelios": "neuroepitelio",
    "neuroesqueletos": "neuroesqueleto",
    "neurofibrillas": "neurofibrilla",
    "neurofilamentos": "neurofilamento",
    "neurofisiologías": "neurofisiología",
    "neurofísica": "neurofísico",
    "neurofísicas": "neurofísico",
    "neurofísicos": "neurofísico",
    "neurogliales": "neuroglial",
    "neuroglias": "neuroglia",
    "neurohumorales": "neurohumoral",
    "neurologías": "neurología",
    "neurológica": "neurológico",
    "neurológicas": "neurológico",
    "neurológicos": "neurológico",
    "neuromas": "neuroma",
    "neuromediadores": "neuromediador",
    "neuromusculares": "neuromuscular",
    "neuronales": "neuronal",
    "neuronas": "neurona",
    "neuroparalítica": "neuroparalítico",
    "neuroparalíticas": "neuroparalítico",
    "neuroparalíticos": "neuroparalítico",
    "neuropatías": "neuropatía",
    "neuropsicologías": "neuropsicología",
    "neuropsiquiatrías": "neuropsiquiatría",
    "neuropteroides": "neuropteroide",
    "neuropática": "neuropático",
    "neuropáticas": "neuropático",
    "neuropáticos": "neuropático",
    "neuroqueratinas": "neuroqueratina",
    "neuroquirúrgica": "neuroquirúrgico",
    "neuroquirúrgicas": "neuroquirúrgico",
    "neuroquirúrgicos": "neuroquirúrgico",
    "neurosensoriales": "neurosensorial",
    "neurotice": "neurotizar",
    "neuroticemos": "neurotizar",
    "neuroticen": "neurotizar",
    "neurotices": "neurotizar",
    "neuroticé": "neurotizar",
    "neuroticéis": "neurotizar",
    "neurotiza": "neurotizar",
    "neurotizaba": "neurotizar",
    "neurotizabais": "neurotizar",
    "neurotizaban": "neurotizar",
    "neurotizabas": "neurotizar",
    "neurotizad": "neurotizar",
    "neurotizada": "neurotizar",
    "neurotizadas": "neurotizar",
    "neurotizado": "neurotizar",
    "neurotizados": "neurotizar",
    "neurotizamos": "neurotizar",
    "neurotizan": "neurotizar",
    "neurotizando": "neurotizar",
    "neurotizara": "neurotizar",
    "neurotizarais": "neurotizar",
    "neurotizaran": "neurotizar",
    "neurotizaras": "neurotizar",
    "neurotizare": "neurotizar",
    "neurotizareis": "neurotizar",
    "neurotizaremos": "neurotizar",
    "neurotizaren": "neurotizar",
    "neurotizares": "neurotizar",
    "neurotizaron": "neurotizar",
    "neurotizará": "neurotizar",
    "neurotizarán": "neurotizar",
    "neurotizarás": "neurotizar",
    "neurotizaré": "neurotizar",
    "neurotizaréis": "neurotizar",
    "neurotizaría": "neurotizar",
    "neurotizaríais": "neurotizar",
    "neurotizaríamos": "neurotizar",
    "neurotizarían": "neurotizar",
    "neurotizarías": "neurotizar",
    "neurotizas": "neurotizar",
    "neurotizase": "neurotizar",
    "neurotizaseis": "neurotizar",
    "neurotizasen": "neurotizar",
    "neurotizases": "neurotizar",
    "neurotizaste": "neurotizar",
    "neurotizasteis": "neurotizar",
    "neurotizo": "neurotizar",
    "neurotizábamos": "neurotizar",
    "neurotizáis": "neurotizar",
    "neurotizáramos": "neurotizar",
    "neurotizáremos": "neurotizar",
    "neurotizásemos": "neurotizar",
    "neurotizó": "neurotizar",
    "neurotomías": "neurotomía",
    "neurotransmisora": "neurotransmisor",
    "neurotransmisoras": "neurotransmisor",
    "neurotransmisores": "neurotransmisor",
    "neurovasculares": "neurovascular",
    "neurovegetativa": "neurovegetativo",
    "neurovegetativas": "neurovegetativo",
    "neurovegetativos": "neurovegetativo",
    "neurulaciones": "neurulación",
    "neurálgica": "neurálgico",
    "neurálgicas": "neurálgico",
    "neurálgicos": "neurálgico",
    "neurítica": "neurítico",
    "neuríticas": "neurítico",
    "neuríticos": "neurítico",
    "neurógena": "neurógeno",
    "neurógenas": "neurógeno",
    "neurógenos": "neurógeno",
    "neuróloga": "neurólogo",
    "neurólogas": "neurólogo",
    "neurólogos": "neurólogo",
    "neurópatas": "neurópata",
    "neurópteros": "neuróptero",
    "neurótica": "neurótico",
    "neuróticas": "neurótico",
    "neuróticos": "neurótico",
    "neurótomos": "neurótomo",
    "neustones": "neuston",
    "neutra": "neutro",
    "neutrales": "neutral",
    "neutralice": "neutralizar",
    "neutralicemos": "neutralizar",
    "neutralicen": "neutralizar",
    "neutralices": "neutralizar",
    "neutralicé": "neutralizar",
    "neutralicéis": "neutralizar",
    "neutralidades": "neutralidad",
    "neutralismos": "neutralismo",
    "neutralistas": "neutralista",
    "neutraliza": "neutralizar",
    "neutralizaba": "neutralizar",
    "neutralizabais": "neutralizar",
    "neutralizaban": "neutralizar",
    "neutralizabas": "neutralizar",
    "neutralizables": "neutralizable",
    "neutralizaciones": "neutralización",
    "neutralizad": "neutralizar",
    "neutralizada": "neutralizar",
    "neutralizadas": "neutralizar",
    "neutralizado": "neutralizar",
    "neutralizadora": "neutralizador",
    "neutralizadoras": "neutralizador",
    "neutralizadores": "neutralizador",
    "neutralizados": "neutralizar",
    "neutralizamos": "neutralizar",
    "neutralizan": "neutralizar",
    "neutralizando": "neutralizar",
    "neutralizara": "neutralizar",
    "neutralizarais": "neutralizar",
    "neutralizaran": "neutralizar",
    "neutralizaras": "neutralizar",
    "neutralizare": "neutralizar",
    "neutralizareis": "neutralizar",
    "neutralizaremos": "neutralizar",
    "neutralizaren": "neutralizar",
    "neutralizares": "neutralizar",
    "neutralizaron": "neutralizar",
    "neutralizará": "neutralizar",
    "neutralizarán": "neutralizar",
    "neutralizarás": "neutralizar",
    "neutralizaré": "neutralizar",
    "neutralizaréis": "neutralizar",
    "neutralizaría": "neutralizar",
    "neutralizaríais": "neutralizar",
    "neutralizaríamos": "neutralizar",
    "neutralizarían": "neutralizar",
    "neutralizarías": "neutralizar",
    "neutralizas": "neutralizar",
    "neutralizase": "neutralizar",
    "neutralizaseis": "neutralizar",
    "neutralizasen": "neutralizar",
    "neutralizases": "neutralizar",
    "neutralizaste": "neutralizar",
    "neutralizasteis": "neutralizar",
    "neutralizo": "neutralizar",
    "neutralizábamos": "neutralizar",
    "neutralizáis": "neutralizar",
    "neutralizáramos": "neutralizar",
    "neutralizáremos": "neutralizar",
    "neutralizásemos": "neutralizar",
    "neutralizó": "neutralizar",
    "neutras": "neutro",
    "neutrinos": "neutrino",
    "neutroclusiones": "neutroclusión",
    "neutrofilias": "neutrofilia",
    "neutrones": "neutrón",
    "neutrooclusiones": "neutrooclusión",
    "neutropenias": "neutropenia",
    "neutros": "neutro",
    "neutrófila": "neutrófilo",
    "neutrófilas": "neutrófilo",
    "neutrófilos": "neutrófilo",
    "nevaba": "nevar",
    "nevadas": "nevada",
    "nevadillas": "nevadilla",
    "nevado": "nevar",
    "nevados": "nevado",
    "nevando": "nevar",
    "nevara": "nevar",
    "nevará": "nevar",
    "nevaría": "nevar",
    "nevascas": "nevasca",
    "nevase": "nevar",
    "nevazos": "nevazo",
    "neveras": "nevera",
    "neveros": "nevero",
    "nevisca": "neviscar",
    "neviscaba": "neviscar",
    "neviscado": "neviscar",
    "neviscando": "neviscar",
    "neviscara": "neviscar",
    "neviscará": "neviscar",
    "neviscaría": "neviscar",
    "neviscas": "nevisca",
    "neviscase": "neviscar",
    "neviscó": "neviscar",
    "nevisque": "neviscar",
    "nevizas": "neviza",
    "nevosa": "nevoso",
    "nevosas": "nevoso",
    "nevosos": "nevoso",
    "nevó": "nevar",
    "newtoniana": "newtoniano",
    "newtonianas": "newtoniano",
    "newtonianos": "newtoniano",
    "newtons": "newton",
    "nexos": "nexo",
    "neófita": "neófito",
    "neófitas": "neófito",
    "neófitos": "neófito",
    "neógena": "neógeno",
    "neógenas": "neógeno",
    "neógenos": "neógeno",
    "neóloga": "neólogo",
    "neólogas": "neólogo",
    "neólogos": "neólogo",
    "ngultrums": "ngultrum",
    "niacinas": "niacina",
    "niacitinas": "niacitina",
    "nialas": "niala",
    "nibbles": "nibble",
    "nicaragüenses": "nicaragüense",
    "nicaragüeña": "nicaragüeño",
    "nicaragüeñas": "nicaragüeño",
    "nicaragüeñismos": "nicaragüeñismo",
    "nicaragüeños": "nicaragüeño",
    "nicaraos": "nicarao",
    "nicena": "niceno",
    "nicenas": "niceno",
    "nicenos": "niceno",
    "nichos": "nicho",
    "nicles": "nicle",
    "nicolitas": "nicolita",
    "nicotinamidas": "nicotinamida",
    "nicotinas": "nicotina",
    "nicotinismos": "nicotinismo",
    "nicotismos": "nicotismo",
    "nicotínica": "nicotínico",
    "nicotínicas": "nicotínico",
    "nicotínicos": "nicotínico",
    "nicromos": "nicromo",
    "nictagináceas": "nictaginácea",
    "nictalopías": "nictalopía",
    "nictalófila": "nictalófilo",
    "nictalófilas": "nictalófilo",
    "nictalófilos": "nictalófilo",
    "nictalópica": "nictalópico",
    "nictalópicas": "nictalópico",
    "nictalópicos": "nictalópico",
    "nictálopes": "nictálope",
    "nicótica": "nicótico",
    "nicóticas": "nicótico",
    "nicóticos": "nicótico",
    "nidaciones": "nidación",
    "nidadas": "nidada",
    "nidales": "nidal",
    "nidifica": "nidificar",
    "nidificaba": "nidificar",
    "nidificabais": "nidificar",
    "nidificaban": "nidificar",
    "nidificabas": "nidificar",
    "nidificaciones": "nidificación",
    "nidificad": "nidificar",
    "nidificada": "nidificar",
    "nidificadas": "nidificar",
    "nidificado": "nidificar",
    "nidificados": "nidificar",
    "nidificamos": "nidificar",
    "nidifican": "nidificar",
    "nidificando": "nidificar",
    "nidificara": "nidificar",
    "nidificarais": "nidificar",
    "nidificaran": "nidificar",
    "nidificaras": "nidificar",
    "nidificare": "nidificar",
    "nidificareis": "nidificar",
    "nidificaremos": "nidificar",
    "nidificaren": "nidificar",
    "nidificares": "nidificar",
    "nidificaron": "nidificar",
    "nidificará": "nidificar",
    "nidificarán": "nidificar",
    "nidificarás": "nidificar",
    "nidificaré": "nidificar",
    "nidificaréis": "nidificar",
    "nidificaría": "nidificar",
    "nidificaríais": "nidificar",
    "nidificaríamos": "nidificar",
    "nidificarían": "nidificar",
    "nidificarías": "nidificar",
    "nidificas": "nidificar",
    "nidificase": "nidificar",
    "nidificaseis": "nidificar",
    "nidificasen": "nidificar",
    "nidificases": "nidificar",
    "nidificaste": "nidificar",
    "nidificasteis": "nidificar",
    "nidifico": "nidificar",
    "nidificábamos": "nidificar",
    "nidificáis": "nidificar",
    "nidificáramos": "nidificar",
    "nidificáremos": "nidificar",
    "nidificásemos": "nidificar",
    "nidificó": "nidificar",
    "nidifique": "nidificar",
    "nidifiquemos": "nidificar",
    "nidifiquen": "nidificar",
    "nidifiques": "nidificar",
    "nidifiqué": "nidificar",
    "nidifiquéis": "nidificar",
    "nidos": "nido",
    "nidícolas": "nidícola",
    "nidófila": "nidófilo",
    "nidófilas": "nidófilo",
    "nidófilos": "nidófilo",
    "nieblas": "niebla",
    "niega": "negar",
    "niegan": "negar",
    "niegas": "negar",
    "niego": "negar",
    "niegue": "negar",
    "nieguen": "negar",
    "niegues": "negar",
    "niela": "nielar",
    "nielaba": "nielar",
    "nielabais": "nielar",
    "nielaban": "nielar",
    "nielabas": "nielar",
    "nielad": "nielar",
    "nielada": "nielar",
    "nieladas": "nielar",
    "nielado": "nielar",
    "nielados": "nielar",
    "nielamos": "nielar",
    "nielan": "nielar",
    "nielando": "nielar",
    "nielara": "nielar",
    "nielarais": "nielar",
    "nielaran": "nielar",
    "nielaras": "nielar",
    "nielare": "nielar",
    "nielareis": "nielar",
    "nielaremos": "nielar",
    "nielaren": "nielar",
    "nielares": "nielar",
    "nielaron": "nielar",
    "nielará": "nielar",
    "nielarán": "nielar",
    "nielarás": "nielar",
    "nielaré": "nielar",
    "nielaréis": "nielar",
    "nielaría": "nielar",
    "nielaríais": "nielar",
    "nielaríamos": "nielar",
    "nielarían": "nielar",
    "nielarías": "nielar",
    "nielas": "nielar",
    "nielase": "nielar",
    "nielaseis": "nielar",
    "nielasen": "nielar",
    "nielases": "nielar",
    "nielaste": "nielar",
    "nielasteis": "nielar",
    "niele": "nielar",
    "nielemos": "nielar",
    "nielen": "nielar",
    "nieles": "nielar",
    "nielo": "nielar",
    "nielábamos": "nielar",
    "nieláis": "nielar",
    "nieláramos": "nielar",
    "nieláremos": "nielar",
    "nielásemos": "nielar",
    "nielé": "nielar",
    "nieléis": "nielar",
    "nieló": "nielar",
    "nies": "ny",
    "nieta": "nieto",
    "nietas": "nieto",
    "nietastra": "nietastro",
    "nietastras": "nietastro",
    "nietastros": "nietastro",
    "nietecita": "nietecito",
    "nietecitas": "nietecito",
    "nietecitos": "nietecito",
    "nietos": "nieto",
    "nieva": "nevar",
    "nieve": "nevar",
    "nieves": "nieve",
    "nifes": "nife",
    "nigeriana": "nigeriano",
    "nigerianas": "nigeriano",
    "nigerianos": "nigeriano",
    "nigolas": "nigola",
    "nigromancias": "nigromancia",
    "nigromantes": "nigromante",
    "nigromántica": "nigromántico",
    "nigrománticas": "nigromántico",
    "nigrománticos": "nigromántico",
    "nigrosinas": "nigrosina",
    "niguas": "nigua",
    "nihilismos": "nihilismo",
    "nihilistas": "nihilista",
    "nilgós": "nilgó",
    "nilones": "nilón",
    "nilótica": "nilótico",
    "nilóticas": "nilótico",
    "nilóticos": "nilótico",
    "nimba": "nimbar",
    "nimbaba": "nimbar",
    "nimbabais": "nimbar",
    "nimbaban": "nimbar",
    "nimbabas": "nimbar",
    "nimbad": "nimbar",
    "nimbada": "nimbar",
    "nimbadas": "nimbar",
    "nimbado": "nimbar",
    "nimbados": "nimbar",
    "nimbamos": "nimbar",
    "nimban": "nimbar",
    "nimbando": "nimbar",
    "nimbara": "nimbar",
    "nimbarais": "nimbar",
    "nimbaran": "nimbar",
    "nimbaras": "nimbar",
    "nimbare": "nimbar",
    "nimbareis": "nimbar",
    "nimbaremos": "nimbar",
    "nimbaren": "nimbar",
    "nimbares": "nimbar",
    "nimbaron": "nimbar",
    "nimbará": "nimbar",
    "nimbarán": "nimbar",
    "nimbarás": "nimbar",
    "nimbaré": "nimbar",
    "nimbaréis": "nimbar",
    "nimbaría": "nimbar",
    "nimbaríais": "nimbar",
    "nimbaríamos": "nimbar",
    "nimbarían": "nimbar",
    "nimbarías": "nimbar",
    "nimbas": "nimbar",
    "nimbase": "nimbar",
    "nimbaseis": "nimbar",
    "nimbasen": "nimbar",
    "nimbases": "nimbar",
    "nimbaste": "nimbar",
    "nimbasteis": "nimbar",
    "nimbe": "nimbar",
    "nimbemos": "nimbar",
    "nimben": "nimbar",
    "nimbes": "nimbar",
    "nimbo": "nimbar",
    "nimboestratos": "nimboestrato",
    "nimbos": "nimbo",
    "nimbostratos": "nimbostrato",
    "nimbábamos": "nimbar",
    "nimbáis": "nimbar",
    "nimbáramos": "nimbar",
    "nimbáremos": "nimbar",
    "nimbásemos": "nimbar",
    "nimbé": "nimbar",
    "nimbéis": "nimbar",
    "nimbó": "nimbar",
    "nimia": "nimio",
    "nimias": "nimio",
    "nimiedades": "nimiedad",
    "nimios": "nimio",
    "ninfas": "ninfa",
    "ninfeas": "ninfea",
    "ninfeos": "ninfeo",
    "ninfeáceas": "ninfeácea",
    "ninfomaníaca": "ninfomaníaco",
    "ninfomaníacas": "ninfomaníaco",
    "ninfomaníacos": "ninfomaníaco",
    "ninfomanías": "ninfomanía",
    "ninfos": "ninfo",
    "ninfómana": "ninfómano",
    "ninfómanas": "ninfómano",
    "ninfómanos": "ninfómano",
    "ninguna": "ninguno",
    "ningunas": "ninguno",
    "ningunos": "ninguno",
    "ningún": "ninguno",
    "ninivitas": "ninivita",
    "niobatos": "niobato",
    "niobios": "niobio",
    "nipona": "nipón",
    "niponas": "nipón",
    "nipones": "nipón",
    "niquela": "niquelar",
    "niquelaba": "niquelar",
    "niquelabais": "niquelar",
    "niquelaban": "niquelar",
    "niquelabas": "niquelar",
    "niquelad": "niquelar",
    "niquelada": "niquelar",
    "niqueladas": "niquelar",
    "niquelado": "niquelar",
    "niqueladora": "niquelador",
    "niqueladoras": "niquelador",
    "niqueladores": "niquelador",
    "niquelados": "niquelar",
    "niqueladuras": "niqueladura",
    "niquelamos": "niquelar",
    "niquelan": "niquelar",
    "niquelando": "niquelar",
    "niquelara": "niquelar",
    "niquelarais": "niquelar",
    "niquelaran": "niquelar",
    "niquelaras": "niquelar",
    "niquelare": "niquelar",
    "niquelareis": "niquelar",
    "niquelaremos": "niquelar",
    "niquelaren": "niquelar",
    "niquelares": "niquelar",
    "niquelaron": "niquelar",
    "niquelará": "niquelar",
    "niquelarán": "niquelar",
    "niquelarás": "niquelar",
    "niquelaré": "niquelar",
    "niquelaréis": "niquelar",
    "niquelaría": "niquelar",
    "niquelaríais": "niquelar",
    "niquelaríamos": "niquelar",
    "niquelarían": "niquelar",
    "niquelarías": "niquelar",
    "niquelas": "niquelar",
    "niquelase": "niquelar",
    "niquelaseis": "niquelar",
    "niquelasen": "niquelar",
    "niquelases": "niquelar",
    "niquelaste": "niquelar",
    "niquelasteis": "niquelar",
    "niquele": "niquelar",
    "niquelemos": "niquelar",
    "niquelen": "niquelar",
    "niqueles": "niquelar",
    "niquelinas": "niquelina",
    "niquelo": "niquelar",
    "niquelábamos": "niquelar",
    "niqueláis": "niquelar",
    "niqueláramos": "niquelar",
    "niqueláremos": "niquelar",
    "niquelásemos": "niquelar",
    "niquelé": "niquelar",
    "niqueléis": "niquelar",
    "niquelífera": "niquelífero",
    "niquelíferas": "niquelífero",
    "niquelíferos": "niquelífero",
    "niqueló": "niquelar",
    "niquetamidas": "niquetamida",
    "nirvanas": "nirvana",
    "nisanes": "nisán",
    "nisinas": "nisina",
    "nistagmos": "nistagmo",
    "nistagmógrafos": "nistagmógrafo",
    "nistágmica": "nistágmico",
    "nistágmicas": "nistágmico",
    "nistágmicos": "nistágmico",
    "nitideces": "nitidez",
    "nitosoles": "nitosol",
    "nitraciones": "nitración",
    "nitradoras": "nitradora",
    "nitrata": "nitratar",
    "nitrataba": "nitratar",
    "nitratabais": "nitratar",
    "nitrataban": "nitratar",
    "nitratabas": "nitratar",
    "nitratad": "nitratar",
    "nitratada": "nitratar",
    "nitratadas": "nitratar",
    "nitratado": "nitratar",
    "nitratados": "nitratar",
    "nitratamos": "nitratar",
    "nitratan": "nitratar",
    "nitratando": "nitratar",
    "nitratara": "nitratar",
    "nitratarais": "nitratar",
    "nitrataran": "nitratar",
    "nitrataras": "nitratar",
    "nitratare": "nitratar",
    "nitratareis": "nitratar",
    "nitrataremos": "nitratar",
    "nitrataren": "nitratar",
    "nitratares": "nitratar",
    "nitrataron": "nitratar",
    "nitratará": "nitratar",
    "nitratarán": "nitratar",
    "nitratarás": "nitratar",
    "nitrataré": "nitratar",
    "nitrataréis": "nitratar",
    "nitrataría": "nitratar",
    "nitrataríais": "nitratar",
    "nitrataríamos": "nitratar",
    "nitratarían": "nitratar",
    "nitratarías": "nitratar",
    "nitratas": "nitratar",
    "nitratase": "nitratar",
    "nitrataseis": "nitratar",
    "nitratasen": "nitratar",
    "nitratases": "nitratar",
    "nitrataste": "nitratar",
    "nitratasteis": "nitratar",
    "nitrate": "nitratar",
    "nitratemos": "nitratar",
    "nitraten": "nitratar",
    "nitrates": "nitratar",
    "nitratinas": "nitratina",
    "nitrato": "nitratar",
    "nitratos": "nitrato",
    "nitratábamos": "nitratar",
    "nitratáis": "nitratar",
    "nitratáramos": "nitratar",
    "nitratáremos": "nitratar",
    "nitratásemos": "nitratar",
    "nitraté": "nitratar",
    "nitratéis": "nitratar",
    "nitrató": "nitratar",
    "nitrerías": "nitrería",
    "nitrifica": "nitrificar",
    "nitrificaba": "nitrificar",
    "nitrificabais": "nitrificar",
    "nitrificaban": "nitrificar",
    "nitrificabas": "nitrificar",
    "nitrificaciones": "nitrificación",
    "nitrificad": "nitrificar",
    "nitrificada": "nitrificar",
    "nitrificadas": "nitrificar",
    "nitrificado": "nitrificar",
    "nitrificadora": "nitrificador",
    "nitrificadoras": "nitrificador",
    "nitrificadores": "nitrificador",
    "nitrificados": "nitrificar",
    "nitrificamos": "nitrificar",
    "nitrifican": "nitrificar",
    "nitrificando": "nitrificar",
    "nitrificantes": "nitrificante",
    "nitrificara": "nitrificar",
    "nitrificarais": "nitrificar",
    "nitrificaran": "nitrificar",
    "nitrificaras": "nitrificar",
    "nitrificare": "nitrificar",
    "nitrificareis": "nitrificar",
    "nitrificaremos": "nitrificar",
    "nitrificaren": "nitrificar",
    "nitrificares": "nitrificar",
    "nitrificaron": "nitrificar",
    "nitrificará": "nitrificar",
    "nitrificarán": "nitrificar",
    "nitrificarás": "nitrificar",
    "nitrificaré": "nitrificar",
    "nitrificaréis": "nitrificar",
    "nitrificaría": "nitrificar",
    "nitrificaríais": "nitrificar",
    "nitrificaríamos": "nitrificar",
    "nitrificarían": "nitrificar",
    "nitrificarías": "nitrificar",
    "nitrificas": "nitrificar",
    "nitrificase": "nitrificar",
    "nitrificaseis": "nitrificar",
    "nitrificasen": "nitrificar",
    "nitrificases": "nitrificar",
    "nitrificaste": "nitrificar",
    "nitrificasteis": "nitrificar",
    "nitrifico": "nitrificar",
    "nitrificábamos": "nitrificar",
    "nitrificáis": "nitrificar",
    "nitrificáramos": "nitrificar",
    "nitrificáremos": "nitrificar",
    "nitrificásemos": "nitrificar",
    "nitrificó": "nitrificar",
    "nitrifique": "nitrificar",
    "nitrifiquemos": "nitrificar",
    "nitrifiquen": "nitrificar",
    "nitrifiques": "nitrificar",
    "nitrifiqué": "nitrificar",
    "nitrifiquéis": "nitrificar",
    "nitrilos": "nitrilo",
    "nitritos": "nitrito",
    "nitroacetanilidas": "nitroacetanilida",
    "nitroalmidones": "nitroalmidón",
    "nitroaminofenoles": "nitroaminofenol",
    "nitroanilinas": "nitroanilina",
    "nitroanisoles": "nitroanisol",
    "nitrobencenoazorresorcinas": "nitrobencenoazorresorcina",
    "nitrobencenos": "nitrobenceno",
    "nitrobenzaldehídos": "nitrobenzaldehído",
    "nitrobenzoica": "nitrobenzoico",
    "nitrobenzoicas": "nitrobenzoico",
    "nitrobenzoicos": "nitrobenzoico",
    "nitrobifenilos": "nitrobifenilo",
    "nitrocalcitas": "nitrocalcita",
    "nitrocelulosas": "nitrocelulosa",
    "nitrocompuestos": "nitrocompuesto",
    "nitroderivados": "nitroderivado",
    "nitrodifenilaminas": "nitrodifenilamina",
    "nitroestirenos": "nitroestireno",
    "nitroetanos": "nitroetano",
    "nitrofenetoles": "nitrofenetol",
    "nitrofenidas": "nitrofenida",
    "nitrofenilacética": "nitrofenilacético",
    "nitrofenilacéticas": "nitrofenilacético",
    "nitrofenilacéticos": "nitrofenilacético",
    "nitrofenoles": "nitrofenol",
    "nitroferricianuros": "nitroferricianuro",
    "nitrofurantoínas": "nitrofurantoína",
    "nitrofurazonas": "nitrofurazona",
    "nitrogenada": "nitrogenado",
    "nitrogenadas": "nitrogenado",
    "nitrogenados": "nitrogenado",
    "nitrogenasas": "nitrogenasa",
    "nitroglicerinas": "nitroglicerina",
    "nitroguanidinas": "nitroguanidina",
    "nitrogénica": "nitrogénico",
    "nitrogénicas": "nitrogénico",
    "nitrogénicos": "nitrogénico",
    "nitromersoles": "nitromersol",
    "nitrometanos": "nitrometano",
    "nitronaftalenos": "nitronaftaleno",
    "nitrones": "nitrón",
    "nitroparacresoles": "nitroparacresol",
    "nitroparafinas": "nitroparafina",
    "nitropropanos": "nitropropano",
    "nitroprusiatos": "nitroprusiato",
    "nitros": "nitro",
    "nitrosa": "nitroso",
    "nitrosalicílica": "nitrosalicílico",
    "nitrosalicílicas": "nitrosalicílico",
    "nitrosalicílicos": "nitrosalicílico",
    "nitrosas": "nitroso",
    "nitrosilos": "nitrosilo",
    "nitrositas": "nitrosita",
    "nitrosoderivados": "nitrosoderivado",
    "nitrosodifenilaminas": "nitrosodifenilamina",
    "nitrosodimetilanilinas": "nitrosodimetilanilina",
    "nitrosofenoles": "nitrosofenol",
    "nitrosoguanidinas": "nitrosoguanidina",
    "nitrosonaftoles": "nitrosonaftol",
    "nitrosos": "nitroso",
    "nitrosulfatiazoles": "nitrosulfatiazol",
    "nitrotoluenos": "nitrotolueno",
    "nitrotoluidinas": "nitrotoluidina",
    "nitrotrifluometilbenzonitrilos": "nitrotrifluometilbenzonitrilo",
    "nitroureas": "nitrourea",
    "nitroxilenos": "nitroxileno",
    "nitrura": "nitrurar",
    "nitruraba": "nitrurar",
    "nitrurabais": "nitrurar",
    "nitruraban": "nitrurar",
    "nitrurabas": "nitrurar",
    "nitruraciones": "nitruración",
    "nitrurad": "nitrurar",
    "nitrurada": "nitrurar",
    "nitruradas": "nitrurar",
    "nitrurado": "nitrurar",
    "nitrurados": "nitrurar",
    "nitruramos": "nitrurar",
    "nitruran": "nitrurar",
    "nitrurando": "nitrurar",
    "nitrurara": "nitrurar",
    "nitrurarais": "nitrurar",
    "nitruraran": "nitrurar",
    "nitruraras": "nitrurar",
    "nitrurare": "nitrurar",
    "nitrurareis": "nitrurar",
    "nitruraremos": "nitrurar",
    "nitruraren": "nitrurar",
    "nitrurares": "nitrurar",
    "nitruraron": "nitrurar",
    "nitrurará": "nitrurar",
    "nitrurarán": "nitrurar",
    "nitrurarás": "nitrurar",
    "nitruraré": "nitrurar",
    "nitruraréis": "nitrurar",
    "nitruraría": "nitrurar",
    "nitruraríais": "nitrurar",
    "nitruraríamos": "nitrurar",
    "nitrurarían": "nitrurar",
    "nitrurarías": "nitrurar",
    "nitruras": "nitrurar",
    "nitrurase": "nitrurar",
    "nitruraseis": "nitrurar",
    "nitrurasen": "nitrurar",
    "nitrurases": "nitrurar",
    "nitruraste": "nitrurar",
    "nitrurasteis": "nitrurar",
    "nitrure": "nitrurar",
    "nitruremos": "nitrurar",
    "nitruren": "nitrurar",
    "nitrures": "nitrurar",
    "nitruro": "nitrurar",
    "nitruros": "nitruro",
    "nitrurábamos": "nitrurar",
    "nitruráis": "nitrurar",
    "nitruráramos": "nitrurar",
    "nitruráremos": "nitrurar",
    "nitrurásemos": "nitrurar",
    "nitruré": "nitrurar",
    "nitruréis": "nitrurar",
    "nitruró": "nitrurar",
    "nitrófila": "nitrófilo",
    "nitrófilas": "nitrófilo",
    "nitrófilos": "nitrófilo",
    "nitrógenos": "nitrógeno",
    "nitrómetros": "nitrómetro",
    "nits": "nit",
    "nitómetros": "nitómetro",
    "nivaciones": "nivación",
    "nivalenoles": "nivalenol",
    "nivales": "nival",
    "nivela": "nivelar",
    "nivelaba": "nivelar",
    "nivelabais": "nivelar",
    "nivelaban": "nivelar",
    "nivelabas": "nivelar",
    "nivelaciones": "nivelación",
    "nivelad": "nivelar",
    "nivelada": "nivelar",
    "niveladas": "nivelar",
    "nivelado": "nivelar",
    "niveladora": "nivelador",
    "niveladoras": "niveladora",
    "niveladores": "nivelador",
    "nivelados": "nivelar",
    "nivelamos": "nivelar",
    "nivelan": "nivelar",
    "nivelando": "nivelar",
    "nivelara": "nivelar",
    "nivelarais": "nivelar",
    "nivelaran": "nivelar",
    "nivelaras": "nivelar",
    "nivelare": "nivelar",
    "nivelareis": "nivelar",
    "nivelaremos": "nivelar",
    "nivelaren": "nivelar",
    "nivelares": "nivelar",
    "nivelaron": "nivelar",
    "nivelará": "nivelar",
    "nivelarán": "nivelar",
    "nivelarás": "nivelar",
    "nivelaré": "nivelar",
    "nivelaréis": "nivelar",
    "nivelaría": "nivelar",
    "nivelaríais": "nivelar",
    "nivelaríamos": "nivelar",
    "nivelarían": "nivelar",
    "nivelarías": "nivelar",
    "nivelas": "nivelar",
    "nivelase": "nivelar",
    "nivelaseis": "nivelar",
    "nivelasen": "nivelar",
    "nivelases": "nivelar",
    "nivelaste": "nivelar",
    "nivelasteis": "nivelar",
    "nivele": "nivelar",
    "nivelemos": "nivelar",
    "nivelen": "nivelar",
    "niveles": "nivelar",
    "nivelo": "nivelar",
    "nivelábamos": "nivelar",
    "niveláis": "nivelar",
    "niveláramos": "nivelar",
    "niveláremos": "nivelar",
    "nivelásemos": "nivelar",
    "nivelé": "nivelar",
    "niveléis": "nivelar",
    "niveló": "nivelar",
    "nivenitas": "nivenita",
    "nivosa": "nivoso",
    "nivosas": "nivoso",
    "nivosos": "nivoso",
    "nivómetros": "nivómetro",
    "nizaríes": "nizarí",
    "nizarís": "nizarí",
    "niña": "niño",
    "niñas": "niño",
    "niñata": "niñato",
    "niñatas": "niñato",
    "niñatos": "niñato",
    "niñea": "niñear",
    "niñeaba": "niñear",
    "niñeabais": "niñear",
    "niñeaban": "niñear",
    "niñeabas": "niñear",
    "niñead": "niñear",
    "niñeada": "niñear",
    "niñeadas": "niñear",
    "niñeado": "niñear",
    "niñeados": "niñear",
    "niñeamos": "niñear",
    "niñean": "niñear",
    "niñeando": "niñear",
    "niñeara": "niñear",
    "niñearais": "niñear",
    "niñearan": "niñear",
    "niñearas": "niñear",
    "niñeare": "niñear",
    "niñeareis": "niñear",
    "niñearemos": "niñear",
    "niñearen": "niñear",
    "niñeares": "niñear",
    "niñearon": "niñear",
    "niñeará": "niñear",
    "niñearán": "niñear",
    "niñearás": "niñear",
    "niñearé": "niñear",
    "niñearéis": "niñear",
    "niñearía": "niñear",
    "niñearíais": "niñear",
    "niñearíamos": "niñear",
    "niñearían": "niñear",
    "niñearías": "niñear",
    "niñeas": "niñear",
    "niñease": "niñear",
    "niñeaseis": "niñear",
    "niñeasen": "niñear",
    "niñeases": "niñear",
    "niñeaste": "niñear",
    "niñeasteis": "niñear",
    "niñeces": "niñez",
    "niñee": "niñear",
    "niñeemos": "niñear",
    "niñeen": "niñear",
    "niñees": "niñear",
    "niñeo": "niñear",
    "niñera": "niñero",
    "niñeras": "niñero",
    "niñeros": "niñero",
    "niñerías": "niñería",
    "niñeábamos": "niñear",
    "niñeáis": "niñear",
    "niñeáramos": "niñear",
    "niñeáremos": "niñear",
    "niñeásemos": "niñear",
    "niñeé": "niñear",
    "niñeéis": "niñear",
    "niñeó": "niñear",
    "niños": "niño",
    "nm": "nanómetro",
    "nobelios": "nobelio",
    "nobiliaria": "nobiliario",
    "nobiliarias": "nobiliario",
    "nobiliarios": "nobiliario",
    "nobles": "noble",
    "noblezas": "nobleza",
    "noblota": "noblote",
    "noblotas": "noblote",
    "noblotes": "noblote",
    "nocedales": "nocedal",
    "nocedas": "noceda",
    "nochebuenas": "nochebuena",
    "nochebuenos": "nochebueno",
    "noches": "noche",
    "nocheviejas": "nochevieja",
    "nocionales": "nocional",
    "nociones": "noción",
    "nociva": "nocivo",
    "nocivas": "nocivo",
    "nocividades": "nocividad",
    "nocivos": "nocivo",
    "noctambulismos": "noctambulismo",
    "noctilucas": "noctiluca",
    "nocturna": "nocturno",
    "nocturnas": "nocturno",
    "nocturnidades": "nocturnidad",
    "nocturnos": "nocturno",
    "noctámbula": "noctámbulo",
    "noctámbulas": "noctámbulo",
    "noctámbulos": "noctámbulo",
    "noctívaga": "noctívago",
    "noctívagas": "noctívago",
    "noctívagos": "noctívago",
    "nodales": "nodal",
    "nodos": "nodo",
    "nodrizas": "nodriza",
    "nodulares": "nodular",
    "nodulizaciones": "nodulización",
    "noemas": "noema",
    "noemática": "noemático",
    "noemáticas": "noemático",
    "noemáticos": "noemático",
    "noes": "no",
    "nogadas": "nogada",
    "nogales": "nogal",
    "nogalinas": "nogalina",
    "noguerada": "noguerado",
    "nogueradas": "noguerado",
    "noguerados": "noguerado",
    "noguerales": "nogueral",
    "noliciones": "nolición",
    "noluntades": "noluntad",
    "nomadismos": "nomadismo",
    "nombra": "nombrar",
    "nombraba": "nombrar",
    "nombrabais": "nombrar",
    "nombraban": "nombrar",
    "nombrabas": "nombrar",
    "nombrad": "nombrar",
    "nombrada": "nombrar",
    "nombradas": "nombrar",
    "nombrado": "nombrar",
    "nombrados": "nombrar",
    "nombradías": "nombradía",
    "nombramientos": "nombramiento",
    "nombramos": "nombrar",
    "nombran": "nombrar",
    "nombrando": "nombrar",
    "nombrara": "nombrar",
    "nombrarais": "nombrar",
    "nombraran": "nombrar",
    "nombraras": "nombrar",
    "nombrare": "nombrar",
    "nombrareis": "nombrar",
    "nombraremos": "nombrar",
    "nombraren": "nombrar",
    "nombrares": "nombrar",
    "nombraron": "nombrar",
    "nombrará": "nombrar",
    "nombrarán": "nombrar",
    "nombrarás": "nombrar",
    "nombraré": "nombrar",
    "nombraréis": "nombrar",
    "nombraría": "nombrar",
    "nombraríais": "nombrar",
    "nombraríamos": "nombrar",
    "nombrarían": "nombrar",
    "nombrarías": "nombrar",
    "nombras": "nombrar",
    "nombrase": "nombrar",
    "nombraseis": "nombrar",
    "nombrasen": "nombrar",
    "nombrases": "nombrar",
    "nombraste": "nombrar",
    "nombrasteis": "nombrar",
    "nombre": "nombrar",
    "nombremos": "nombrar",
    "nombren": "nombrar",
    "nombres": "nombre",
    "nombro": "nombrar",
    "nombrábamos": "nombrar",
    "nombráis": "nombrar",
    "nombráramos": "nombrar",
    "nombráremos": "nombrar",
    "nombrásemos": "nombrar",
    "nombré": "nombrar",
    "nombréis": "nombrar",
    "nombró": "nombrar",
    "nomencladora": "nomenclador",
    "nomencladoras": "nomenclador",
    "nomencladores": "nomenclador",
    "nomenclatora": "nomenclator",
    "nomenclatoras": "nomenclator",
    "nomenclatores": "nomenclator",
    "nomenclaturas": "nomenclatura",
    "nomenclátores": "nomenclátor",
    "nomina": "nominar",
    "nominaba": "nominar",
    "nominabais": "nominar",
    "nominaban": "nominar",
    "nominabas": "nominar",
    "nominaciones": "nominación",
    "nominad": "nominar",
    "nominada": "nominar",
    "nominadas": "nominar",
    "nominado": "nominar",
    "nominadora": "nominador",
    "nominadoras": "nominador",
    "nominadores": "nominador",
    "nominados": "nominar",
    "nominales": "nominal",
    "nominalice": "nominalizar",
    "nominalicemos": "nominalizar",
    "nominalicen": "nominalizar",
    "nominalices": "nominalizar",
    "nominalicé": "nominalizar",
    "nominalicéis": "nominalizar",
    "nominalismos": "nominalismo",
    "nominalistas": "nominalista",
    "nominaliza": "nominalizar",
    "nominalizaba": "nominalizar",
    "nominalizabais": "nominalizar",
    "nominalizaban": "nominalizar",
    "nominalizabas": "nominalizar",
    "nominalizaciones": "nominalización",
    "nominalizad": "nominalizar",
    "nominalizada": "nominalizar",
    "nominalizadas": "nominalizar",
    "nominalizado": "nominalizar",
    "nominalizadora": "nominalizador",
    "nominalizadoras": "nominalizador",
    "nominalizadores": "nominalizador",
    "nominalizados": "nominalizar",
    "nominalizamos": "nominalizar",
    "nominalizan": "nominalizar",
    "nominalizando": "nominalizar",
    "nominalizara": "nominalizar",
    "nominalizarais": "nominalizar",
    "nominalizaran": "nominalizar",
    "nominalizaras": "nominalizar",
    "nominalizare": "nominalizar",
    "nominalizareis": "nominalizar",
    "nominalizaremos": "nominalizar",
    "nominalizaren": "nominalizar",
    "nominalizares": "nominalizar",
    "nominalizaron": "nominalizar",
    "nominalizará": "nominalizar",
    "nominalizarán": "nominalizar",
    "nominalizarás": "nominalizar",
    "nominalizaré": "nominalizar",
    "nominalizaréis": "nominalizar",
    "nominalizaría": "nominalizar",
    "nominalizaríais": "nominalizar",
    "nominalizaríamos": "nominalizar",
    "nominalizarían": "nominalizar",
    "nominalizarías": "nominalizar",
    "nominalizas": "nominalizar",
    "nominalizase": "nominalizar",
    "nominalizaseis": "nominalizar",
    "nominalizasen": "nominalizar",
    "nominalizases": "nominalizar",
    "nominalizaste": "nominalizar",
    "nominalizasteis": "nominalizar",
    "nominalizo": "nominalizar",
    "nominalizábamos": "nominalizar",
    "nominalizáis": "nominalizar",
    "nominalizáramos": "nominalizar",
    "nominalizáremos": "nominalizar",
    "nominalizásemos": "nominalizar",
    "nominalizó": "nominalizar",
    "nominamos": "nominar",
    "nominan": "nominar",
    "nominando": "nominar",
    "nominara": "nominar",
    "nominarais": "nominar",
    "nominaran": "nominar",
    "nominaras": "nominar",
    "nominare": "nominar",
    "nominareis": "nominar",
    "nominaremos": "nominar",
    "nominaren": "nominar",
    "nominares": "nominar",
    "nominaron": "nominar",
    "nominará": "nominar",
    "nominarán": "nominar",
    "nominarás": "nominar",
    "nominaré": "nominar",
    "nominaréis": "nominar",
    "nominaría": "nominar",
    "nominaríais": "nominar",
    "nominaríamos": "nominar",
    "nominarían": "nominar",
    "nominarías": "nominar",
    "nominas": "nominar",
    "nominase": "nominar",
    "nominaseis": "nominar",
    "nominasen": "nominar",
    "nominases": "nominar",
    "nominaste": "nominar",
    "nominasteis": "nominar",
    "nominativa": "nominativo",
    "nominativas": "nominativo",
    "nominativos": "nominativo",
    "nomine": "nominar",
    "nominemos": "nominar",
    "nominen": "nominar",
    "nomines": "nominar",
    "nominillas": "nominilla",
    "nomino": "nominar",
    "nominábamos": "nominar",
    "nomináis": "nominar",
    "nomináramos": "nominar",
    "nomináremos": "nominar",
    "nominásemos": "nominar",
    "nominé": "nominar",
    "nominéis": "nominar",
    "nominó": "nominar",
    "nomografías": "nomografía",
    "nomogramas": "nomograma",
    "nomográfica": "nomográfico",
    "nomográficas": "nomográfico",
    "nomográficos": "nomográfico",
    "nomologías": "nomología",
    "nomos": "nomo",
    "nomotética": "nomotético",
    "nomotéticas": "nomotético",
    "nomotéticos": "nomotético",
    "nomógrafos": "nomógrafo",
    "nomóloga": "nomólogo",
    "nomólogas": "nomólogo",
    "nomólogos": "nomólogo",
    "nomónica": "nomónico",
    "nomónicas": "nomónico",
    "nomónicos": "nomónico",
    "nona": "ñoño",
    "nonadas": "nonada",
    "nonadecanoatos": "nonadecanoato",
    "nonagenaria": "nonagenario",
    "nonagenarias": "nonagenario",
    "nonagenarios": "nonagenario",
    "nonagesimoctava": "98",
    "nonagesimoctavas": "98",
    "nonagesimoctavo": "98",
    "nonagesimoctavos": "98",
    "nonagesimocuarta": "94",
    "nonagesimocuartas": "94",
    "nonagesimocuarto": "94",
    "nonagesimocuartos": "94",
    "nonagesimonona": "99",
    "nonagesimononas": "99",
    "nonagesimonono": "99",
    "nonagesimononos": "99",
    "nonagesimonovena": "99",
    "nonagesimonovenas": "99",
    "nonagesimonoveno": "99",
    "nonagesimonovenos": "99",
    "nonagesimoprimera": "91",
    "nonagesimoprimeras": "91",
    "nonagesimoprimero": "91",
    "nonagesimoprimeros": "91",
    "nonagesimoquinta": "95",
    "nonagesimoquintas": "95",
    "nonagesimoquinto": "95",
    "nonagesimoquintos": "95",
    "nonagesimosegunda": "92",
    "nonagesimosegundas": "92",
    "nonagesimosegundo": "92",
    "nonagesimosegundos": "92",
    "nonagesimosexta": "96",
    "nonagesimosextas": "96",
    "nonagesimosexto": "96",
    "nonagesimosextos": "96",
    "nonagesimoséptima": "97",
    "nonagesimoséptimas": "97",
    "nonagesimoséptimo": "97",
    "nonagesimoséptimos": "97",
    "nonagesimotercera": "93",
    "nonagesimoterceras": "93",
    "nonagesimotercero": "93",
    "nonagesimoterceros": "93",
    "nonagésima": "nonagésimo",
    "nonagésimas": "nonagésimo",
    "nonagésimo": "90",
    "nonagésimos": "nonagésimo",
    "nonaldecanoica": "nonaldecanoico",
    "nonaldecanoicas": "nonaldecanoico",
    "nonaldecanoicos": "nonaldecanoico",
    "nonanoatos": "nonanoato",
    "nonas": "ñoño",
    "nonata": "nonato",
    "nonatas": "nonato",
    "nonatos": "nonato",
    "nones": "non",
    "nonigentésima": "nonigentésimo",
    "nonigentésimas": "nonigentésimo",
    "nonigentésimo": "900",
    "nonigentésimos": "nonigentésimo",
    "nonilaminas": "nonilamina",
    "nonilbencenos": "nonilbenceno",
    "noningentésima": "900",
    "noningentésimas": "900",
    "noningentésimo": "900",
    "noningentésimos": "900",
    "nonios": "nonio",
    "nonos": "ñoño",
    "nontronitas": "nontronita",
    "nonágona": "nonágono",
    "nonágonas": "nonágono",
    "nonágonos": "nonágono",
    "noologías": "noología",
    "nopaledas": "nopaleda",
    "nopaleras": "nopalera",
    "nopales": "nopal",
    "noquea": "noquear",
    "noqueaba": "noquear",
    "noqueabais": "noquear",
    "noqueaban": "noquear",
    "noqueabas": "noquear",
    "noquead": "noquear",
    "noqueada": "noquear",
    "noqueadas": "noquear",
    "noqueado": "noquear",
    "noqueados": "noquear",
    "noqueamos": "noquear",
    "noquean": "noquear",
    "noqueando": "noquear",
    "noqueara": "noquear",
    "noquearais": "noquear",
    "noquearan": "noquear",
    "noquearas": "noquear",
    "noqueare": "noquear",
    "noqueareis": "noquear",
    "noquearemos": "noquear",
    "noquearen": "noquear",
    "noqueares": "noquear",
    "noquearon": "noquear",
    "noqueará": "noquear",
    "noquearán": "noquear",
    "noquearás": "noquear",
    "noquearé": "noquear",
    "noquearéis": "noquear",
    "noquearía": "noquear",
    "noquearíais": "noquear",
    "noquearíamos": "noquear",
    "noquearían": "noquear",
    "noquearías": "noquear",
    "noqueas": "noquear",
    "noquease": "noquear",
    "noqueaseis": "noquear",
    "noqueasen": "noquear",
    "noqueases": "noquear",
    "noqueaste": "noquear",
    "noqueasteis": "noquear",
    "noquee": "noquear",
    "noqueemos": "noquear",
    "noqueen": "noquear",
    "noquees": "noquear",
    "noqueo": "noquear",
    "noqueros": "noquero",
    "noques": "noque",
    "noqueábamos": "noquear",
    "noqueáis": "noquear",
    "noqueáramos": "noquear",
    "noqueáremos": "noquear",
    "noqueásemos": "noquear",
    "noqueé": "noquear",
    "noqueéis": "noquear",
    "noqueó": "noquear",
    "norabuena": "enhorabuena",
    "norabuenas": "norabuena",
    "noradrenalinas": "noradrenalina",
    "noramala": "enhoramala",
    "norayes": "noray",
    "norcoreana": "norcoreano",
    "norcoreanas": "norcoreano",
    "norcoreanos": "norcoreano",
    "nordestes": "nordeste",
    "nordoccidentales": "nordoccidental",
    "nordorientales": "nordoriental",
    "norestes": "noreste",
    "norias": "noria",
    "noriegas": "noriega",
    "norirlandesa": "norirlandés",
    "norirlandesas": "norirlandés",
    "norirlandeses": "norirlandés",
    "norirlendésa": "norirlandés",
    "norirlendésas": "norirlandés",
    "norirlendéses": "norirlandés",
    "noritas": "norita",
    "norleucinas": "norleucina",
    "normales": "normal",
    "normalice": "normalizar",
    "normalicemos": "normalizar",
    "normalicen": "normalizar",
    "normalices": "normalizar",
    "normalicé": "normalizar",
    "normalicéis": "normalizar",
    "normalidades": "normalidad",
    "normalistas": "normalista",
    "normaliza": "normalizar",
    "normalizaba": "normalizar",
    "normalizabais": "normalizar",
    "normalizaban": "normalizar",
    "normalizabas": "normalizar",
    "normalizaciones": "normalización",
    "normalizaciónes": "normalización",
    "normalizad": "normalizar",
    "normalizada": "normalizar",
    "normalizadas": "normalizar",
    "normalizado": "normalizar",
    "normalizadora": "normalizador",
    "normalizadoras": "normalizador",
    "normalizadores": "normalizador",
    "normalizados": "normalizar",
    "normalizamos": "normalizar",
    "normalizan": "normalizar",
    "normalizando": "normalizar",
    "normalizara": "normalizar",
    "normalizarais": "normalizar",
    "normalizaran": "normalizar",
    "normalizaras": "normalizar",
    "normalizare": "normalizar",
    "normalizareis": "normalizar",
    "normalizaremos": "normalizar",
    "normalizaren": "normalizar",
    "normalizares": "normalizar",
    "normalizaron": "normalizar",
    "normalizará": "normalizar",
    "normalizarán": "normalizar",
    "normalizarás": "normalizar",
    "normalizaré": "normalizar",
    "normalizaréis": "normalizar",
    "normalizaría": "normalizar",
    "normalizaríais": "normalizar",
    "normalizaríamos": "normalizar",
    "normalizarían": "normalizar",
    "normalizarías": "normalizar",
    "normalizas": "normalizar",
    "normalizase": "normalizar",
    "normalizaseis": "normalizar",
    "normalizasen": "normalizar",
    "normalizases": "normalizar",
    "normalizaste": "normalizar",
    "normalizasteis": "normalizar",
    "normalizo": "normalizar",
    "normalizábamos": "normalizar",
    "normalizáis": "normalizar",
    "normalizáramos": "normalizar",
    "normalizáremos": "normalizar",
    "normalizásemos": "normalizar",
    "normalizó": "normalizar",
    "normanda": "normando",
    "normandas": "normando",
    "normandos": "normando",
    "normas": "norma",
    "normativa": "normativo",
    "normativas": "normativo",
    "normatividades": "normatividad",
    "normativos": "normativo",
    "normoblastos": "normoblasto",
    "normocitos": "normocito",
    "normocromías": "normocromía",
    "normongólidas": "normongólida",
    "nornordestes": "nornordeste",
    "nornorestes": "nornoreste",
    "nornoroestes": "nornoroeste",
    "nornoruestes": "nornorueste",
    "noroccidentales": "noroccidental",
    "noroestes": "noroeste",
    "nororientales": "nororiental",
    "nortadas": "nortada",
    "nortea": "nortear",
    "norteaba": "nortear",
    "norteabais": "nortear",
    "norteaban": "nortear",
    "norteabas": "nortear",
    "nortead": "nortear",
    "norteada": "nortear",
    "norteadas": "nortear",
    "norteado": "nortear",
    "norteados": "nortear",
    "norteafricana": "norteafricano",
    "norteafricanas": "norteafricano",
    "norteafricanos": "norteafricano",
    "norteamericana": "norteamericano",
    "norteamericanas": "norteamericano",
    "norteamericanos": "norteamericano",
    "norteamos": "nortear",
    "nortean": "nortear",
    "norteando": "nortear",
    "norteara": "nortear",
    "nortearais": "nortear",
    "nortearan": "nortear",
    "nortearas": "nortear",
    "norteare": "nortear",
    "norteareis": "nortear",
    "nortearemos": "nortear",
    "nortearen": "nortear",
    "norteares": "nortear",
    "nortearon": "nortear",
    "norteará": "nortear",
    "nortearán": "nortear",
    "nortearás": "nortear",
    "nortearé": "nortear",
    "nortearéis": "nortear",
    "nortearía": "nortear",
    "nortearíais": "nortear",
    "nortearíamos": "nortear",
    "nortearían": "nortear",
    "nortearías": "nortear",
    "norteas": "nortear",
    "nortease": "nortear",
    "norteaseis": "nortear",
    "norteasen": "nortear",
    "norteases": "nortear",
    "norteaste": "nortear",
    "norteasteis": "nortear",
    "nortee": "nortear",
    "norteemos": "nortear",
    "norteen": "nortear",
    "nortees": "nortear",
    "norteo": "nortear",
    "nortes": "norte",
    "norteuropea": "norteuropeo",
    "norteuropeas": "norteuropeo",
    "norteuropeos": "norteuropeo",
    "norteábamos": "nortear",
    "norteáis": "nortear",
    "norteáramos": "nortear",
    "norteáremos": "nortear",
    "norteásemos": "nortear",
    "norteé": "nortear",
    "norteéis": "nortear",
    "norteña": "norteño",
    "norteñas": "norteño",
    "norteños": "norteño",
    "norteó": "nortear",
    "noruega": "noruego",
    "noruegas": "noruego",
    "noruegos": "noruego",
    "nos": "no",
    "noseanas": "noseana",
    "noseologías": "noseología",
    "noseológica": "noseológico",
    "noseológicas": "noseológico",
    "noseológicos": "noseológico",
    "nosocomios": "nosocomio",
    "nosodes": "nosode",
    "nosogenias": "nosogenia",
    "nosologías": "nosología",
    "nosológica": "nosológico",
    "nosológicas": "nosológico",
    "nosológicos": "nosológico",
    "nosománticas": "nosomántica",
    "nosotras": "nosotros",
    "nostalgias": "nostalgia",
    "nosticismos": "nosticismo",
    "nostocales": "nostocal",
    "nostrama": "nostramo",
    "nostramas": "nostramo",
    "nostramos": "nostramo",
    "nostálgica": "nostálgico",
    "nostálgicas": "nostálgico",
    "nostálgicos": "nostálgico",
    "nota": "noto",
    "notaba": "notar",
    "notabais": "notar",
    "notaban": "notar",
    "notabas": "notar",
    "notabilidades": "notabilidad",
    "notables": "notable",
    "notaciones": "notación",
    "notad": "notar",
    "notada": "notar",
    "notadas": "notar",
    "notado": "notar",
    "notados": "notar",
    "notamos": "notar",
    "notan": "notar",
    "notando": "notar",
    "notara": "notar",
    "notarais": "notar",
    "notaran": "notar",
    "notaras": "notar",
    "notare": "notar",
    "notareis": "notar",
    "notaremos": "notar",
    "notaren": "notar",
    "notares": "notar",
    "notaria": "notario",
    "notariada": "notariado",
    "notariadas": "notariado",
    "notariados": "notariado",
    "notariales": "notarial",
    "notarias": "notario",
    "notariatos": "notariato",
    "notarios": "notario",
    "notaron": "notar",
    "notará": "notar",
    "notarán": "notar",
    "notarás": "notar",
    "notaré": "notar",
    "notaréis": "notar",
    "notaría": "notar",
    "notaríais": "notar",
    "notaríamos": "notar",
    "notarían": "notar",
    "notarías": "notaría",
    "notas": "noto",
    "notase": "notar",
    "notaseis": "notar",
    "notasen": "notar",
    "notases": "notar",
    "notaste": "notar",
    "notasteis": "notar",
    "note": "notar",
    "notemos": "notar",
    "noten": "notar",
    "notes": "notar",
    "noticia": "noticiar",
    "noticiaba": "noticiar",
    "noticiabais": "noticiar",
    "noticiaban": "noticiar",
    "noticiabas": "noticiar",
    "noticiad": "noticiar",
    "noticiada": "noticiar",
    "noticiadas": "noticiar",
    "noticiado": "noticiar",
    "noticiados": "noticiar",
    "noticiamos": "noticiar",
    "notician": "noticiar",
    "noticiando": "noticiar",
    "noticiara": "noticiar",
    "noticiarais": "noticiar",
    "noticiaran": "noticiar",
    "noticiaras": "noticiar",
    "noticiare": "noticiar",
    "noticiareis": "noticiar",
    "noticiaremos": "noticiar",
    "noticiaren": "noticiar",
    "noticiares": "noticiar",
    "noticiarios": "noticiario",
    "noticiaron": "noticiar",
    "noticiará": "noticiar",
    "noticiarán": "noticiar",
    "noticiarás": "noticiar",
    "noticiaré": "noticiar",
    "noticiaréis": "noticiar",
    "noticiaría": "noticiar",
    "noticiaríais": "noticiar",
    "noticiaríamos": "noticiar",
    "noticiarían": "noticiar",
    "noticiarías": "noticiar",
    "noticias": "noticiar",
    "noticiase": "noticiar",
    "noticiaseis": "noticiar",
    "noticiasen": "noticiar",
    "noticiases": "noticiar",
    "noticiaste": "noticiar",
    "noticiasteis": "noticiar",
    "noticie": "noticiar",
    "noticiemos": "noticiar",
    "noticien": "noticiar",
    "noticiera": "noticiero",
    "noticieras": "noticiero",
    "noticieros": "noticiero",
    "noticies": "noticiar",
    "noticio": "noticiar",
    "noticiones": "notición",
    "noticiosa": "noticioso",
    "noticiosas": "noticioso",
    "noticiosos": "noticioso",
    "noticiábamos": "noticiar",
    "noticiáis": "noticiar",
    "noticiáramos": "noticiar",
    "noticiáremos": "noticiar",
    "noticiásemos": "noticiar",
    "noticié": "noticiar",
    "noticiéis": "noticiar",
    "notició": "noticiar",
    "notifica": "notificar",
    "notificaba": "notificar",
    "notificabais": "notificar",
    "notificaban": "notificar",
    "notificabas": "notificar",
    "notificaciones": "notificación",
    "notificad": "notificar",
    "notificada": "notificar",
    "notificadas": "notificar",
    "notificado": "notificar",
    "notificados": "notificar",
    "notificamos": "notificar",
    "notifican": "notificar",
    "notificando": "notificar",
    "notificara": "notificar",
    "notificarais": "notificar",
    "notificaran": "notificar",
    "notificaras": "notificar",
    "notificare": "notificar",
    "notificareis": "notificar",
    "notificaremos": "notificar",
    "notificaren": "notificar",
    "notificares": "notificar",
    "notificaron": "notificar",
    "notificará": "notificar",
    "notificarán": "notificar",
    "notificarás": "notificar",
    "notificaré": "notificar",
    "notificaréis": "notificar",
    "notificaría": "notificar",
    "notificaríais": "notificar",
    "notificaríamos": "notificar",
    "notificarían": "notificar",
    "notificarías": "notificar",
    "notificas": "notificar",
    "notificase": "notificar",
    "notificaseis": "notificar",
    "notificasen": "notificar",
    "notificases": "notificar",
    "notificaste": "notificar",
    "notificasteis": "notificar",
    "notificativa": "notificativo",
    "notificativas": "notificativo",
    "notificativos": "notificativo",
    "notifico": "notificar",
    "notificábamos": "notificar",
    "notificáis": "notificar",
    "notificáramos": "notificar",
    "notificáremos": "notificar",
    "notificásemos": "notificar",
    "notificó": "notificar",
    "notifique": "notificar",
    "notifiquemos": "notificar",
    "notifiquen": "notificar",
    "notifiques": "notificar",
    "notifiqué": "notificar",
    "notifiquéis": "notificar",
    "noto": "notar",
    "notocordios": "notocordio",
    "notoria": "notorio",
    "notorias": "notorio",
    "notoriedades": "notoriedad",
    "notorios": "notorio",
    "notos": "noto",
    "notábamos": "notar",
    "notáis": "notar",
    "notáramos": "notar",
    "notáremos": "notar",
    "notásemos": "notar",
    "noté": "notar",
    "notéis": "notar",
    "notó": "notar",
    "noucentismes": "noucentisme",
    "nouménica": "nouménico",
    "nouménicas": "nouménico",
    "nouménicos": "nouménico",
    "nov.": "noviembre",
    "nova": "novar",
    "novaba": "novar",
    "novabais": "novar",
    "novaban": "novar",
    "novabas": "novar",
    "novaciana": "novaciano",
    "novacianas": "novaciano",
    "novacianismos": "novacianismo",
    "novacianos": "novaciano",
    "novaciones": "novación",
    "novad": "novar",
    "novada": "novar",
    "novadas": "novar",
    "novado": "novar",
    "novados": "novar",
    "novales": "noval",
    "novamos": "novar",
    "novan": "novar",
    "novando": "novar",
    "novara": "novar",
    "novarais": "novar",
    "novaran": "novar",
    "novaras": "novar",
    "novare": "novar",
    "novareis": "novar",
    "novaremos": "novar",
    "novaren": "novar",
    "novares": "novar",
    "novaron": "novar",
    "novará": "novar",
    "novarán": "novar",
    "novarás": "novar",
    "novaré": "novar",
    "novaréis": "novar",
    "novaría": "novar",
    "novaríais": "novar",
    "novaríamos": "novar",
    "novarían": "novar",
    "novarías": "novar",
    "novas": "novar",
    "novase": "novar",
    "novaseis": "novar",
    "novasen": "novar",
    "novases": "novar",
    "novaste": "novar",
    "novasteis": "novar",
    "novata": "novato",
    "novatadas": "novatada",
    "novatas": "novato",
    "novatos": "novato",
    "nove": "novar",
    "novecientosava": "novecientosavo",
    "novecientosavas": "novecientosavo",
    "novecientosavo": "900",
    "novecientosavos": "novecientosavo",
    "novedades": "novedad",
    "novedosa": "novedoso",
    "novedosas": "novedoso",
    "novedosos": "novedoso",
    "novela": "novelar",
    "novelaba": "novelar",
    "novelabais": "novelar",
    "novelaban": "novelar",
    "novelabas": "novelar",
    "novelables": "novelable",
    "novelad": "novelar",
    "novelada": "novelar",
    "noveladas": "novelar",
    "novelado": "novelar",
    "noveladora": "novelador",
    "noveladoras": "novelador",
    "noveladores": "novelador",
    "novelados": "novelar",
    "novelamos": "novelar",
    "novelan": "novelar",
    "novelando": "novelar",
    "novelara": "novelar",
    "novelarais": "novelar",
    "novelaran": "novelar",
    "novelaras": "novelar",
    "novelare": "novelar",
    "novelareis": "novelar",
    "novelaremos": "novelar",
    "novelaren": "novelar",
    "novelares": "novelar",
    "novelaron": "novelar",
    "novelará": "novelar",
    "novelarán": "novelar",
    "novelarás": "novelar",
    "novelaré": "novelar",
    "novelaréis": "novelar",
    "novelaría": "novelar",
    "novelaríais": "novelar",
    "novelaríamos": "novelar",
    "novelarían": "novelar",
    "novelarías": "novelar",
    "novelas": "novelar",
    "novelase": "novelar",
    "novelaseis": "novelar",
    "novelasen": "novelar",
    "novelases": "novelar",
    "novelaste": "novelar",
    "novelasteis": "novelar",
    "novele": "novelar",
    "novelemos": "novelar",
    "novelen": "novelar",
    "novelera": "novelero",
    "noveleras": "novelero",
    "noveleros": "novelero",
    "novelerías": "novelería",
    "noveles": "novelar",
    "novelesca": "novelesco",
    "novelescas": "novelesco",
    "novelescos": "novelesco",
    "novelice": "novelizar",
    "novelicemos": "novelizar",
    "novelicen": "novelizar",
    "novelices": "novelizar",
    "novelicé": "novelizar",
    "novelicéis": "novelizar",
    "novelistas": "novelista",
    "noveliza": "novelizar",
    "novelizaba": "novelizar",
    "novelizabais": "novelizar",
    "novelizaban": "novelizar",
    "novelizabas": "novelizar",
    "novelizad": "novelizar",
    "novelizada": "novelizar",
    "novelizadas": "novelizar",
    "novelizado": "novelizar",
    "novelizados": "novelizar",
    "novelizamos": "novelizar",
    "novelizan": "novelizar",
    "novelizando": "novelizar",
    "novelizara": "novelizar",
    "novelizarais": "novelizar",
    "novelizaran": "novelizar",
    "novelizaras": "novelizar",
    "novelizare": "novelizar",
    "novelizareis": "novelizar",
    "novelizaremos": "novelizar",
    "novelizaren": "novelizar",
    "novelizares": "novelizar",
    "novelizaron": "novelizar",
    "novelizará": "novelizar",
    "novelizarán": "novelizar",
    "novelizarás": "novelizar",
    "novelizaré": "novelizar",
    "novelizaréis": "novelizar",
    "novelizaría": "novelizar",
    "novelizaríais": "novelizar",
    "novelizaríamos": "novelizar",
    "novelizarían": "novelizar",
    "novelizarías": "novelizar",
    "novelizas": "novelizar",
    "novelizase": "novelizar",
    "novelizaseis": "novelizar",
    "novelizasen": "novelizar",
    "novelizases": "novelizar",
    "novelizaste": "novelizar",
    "novelizasteis": "novelizar",
    "novelizo": "novelizar",
    "novelizábamos": "novelizar",
    "novelizáis": "novelizar",
    "novelizáramos": "novelizar",
    "novelizáremos": "novelizar",
    "novelizásemos": "novelizar",
    "novelizó": "novelizar",
    "novelo": "novelar",
    "novelones": "novelón",
    "novelábamos": "novelar",
    "noveláis": "novelar",
    "noveláramos": "novelar",
    "noveláremos": "novelar",
    "novelásemos": "novelar",
    "novelé": "novelar",
    "noveléis": "novelar",
    "novelística": "novelístico",
    "novelísticas": "novelístico",
    "novelísticos": "novelístico",
    "noveló": "novelar",
    "novemos": "novar",
    "noven": "novar",
    "novena": "9",
    "novenarios": "novenario",
    "novenas": "novena",
    "noveno": "9",
    "novenos": "noveno",
    "noventaicincoava": "noventaicincoavo",
    "noventaicincoavas": "noventaicincoavo",
    "noventaicincoavo": "95",
    "noventaicincoavos": "noventaicincoavo",
    "noventaicuatroava": "noventaicuatroavo",
    "noventaicuatroavas": "noventaicuatroavo",
    "noventaicuatroavo": "94",
    "noventaicuatroavos": "noventaicuatroavo",
    "noventaidosava": "noventaidosavo",
    "noventaidosavas": "noventaidosavo",
    "noventaidosavo": "92",
    "noventaidosavos": "noventaidosavo",
    "noventainueveava": "noventainueveavo",
    "noventainueveavas": "noventainueveavo",
    "noventainueveavo": "99",
    "noventainueveavos": "noventainueveavo",
    "noventaiochava": "noventaiochoavo",
    "noventaiochavas": "noventaiochoavo",
    "noventaiochavo": "noventaiochoavo",
    "noventaiochavos": "noventaiochoavo",
    "noventaiochoava": "noventaiochoavo",
    "noventaiochoavas": "noventaiochoavo",
    "noventaiochoavo": "98",
    "noventaiochoavos": "noventaiochoavo",
    "noventaiseisava": "noventaiseisavo",
    "noventaiseisavas": "noventaiseisavo",
    "noventaiseisavo": "96",
    "noventaiseisavos": "noventaiseisavo",
    "noventaisieteava": "noventaisieteavo",
    "noventaisieteavas": "noventaisieteavo",
    "noventaisieteavo": "97",
    "noventaisieteavos": "noventaisieteavo",
    "noventaitresava": "noventaitresavo",
    "noventaitresavas": "noventaitresavo",
    "noventaitresavo": "93",
    "noventaitresavos": "noventaitresavo",
    "noventaiunava": "noventaiunavo",
    "noventaiunavas": "noventaiunavo",
    "noventaiunavo": "91",
    "noventaiunavos": "noventaiunavo",
    "noventava": "noventavo",
    "noventavas": "noventavo",
    "noventavo": "90",
    "noventavos": "noventavo",
    "noventona": "noventón",
    "noventonas": "noventón",
    "noventones": "noventón",
    "noves": "novar",
    "novia": "novio",
    "novias": "novio",
    "noviazgos": "noviazgo",
    "novicia": "novicio",
    "noviciados": "noviciado",
    "novicias": "novicio",
    "novicios": "novicio",
    "noviembres": "noviembre",
    "novilla": "novillo",
    "novilladas": "novillada",
    "novillas": "novillo",
    "novillera": "novillero",
    "novilleras": "novillero",
    "novilleros": "novillero",
    "novillos": "novillo",
    "novilunios": "novilunio",
    "novios": "novio",
    "novo": "novar",
    "novobiocinas": "novobiocina",
    "novolacas": "novolaca",
    "novábamos": "novar",
    "nováis": "novar",
    "nováramos": "novar",
    "nováremos": "novar",
    "novásemos": "novar",
    "nové": "novar",
    "novéis": "novar",
    "novísimos": "novísimo",
    "novó": "novar",
    "noyoes": "noyó",
    "noética": "noético",
    "noéticas": "noético",
    "noéticos": "noético",
    "noúmenos": "noúmeno",
    "nubada": "nubado",
    "nubadas": "nubado",
    "nubados": "nubado",
    "nubarrada": "nubarrado",
    "nubarradas": "nubarrado",
    "nubarrados": "nubarrado",
    "nubarrones": "nubarrón",
    "nubecillas": "nubecilla",
    "nubecilles": "nubecill",
    "nubes": "nube",
    "nubla": "nublo",
    "nublaba": "nublar",
    "nublabais": "nublar",
    "nublaban": "nublar",
    "nublabas": "nublar",
    "nublad": "nublar",
    "nublada": "nublar",
    "nubladas": "nublar",
    "nublado": "nublar",
    "nublados": "nublar",
    "nublamos": "nublar",
    "nublan": "nublar",
    "nublando": "nublar",
    "nublara": "nublar",
    "nublarais": "nublar",
    "nublaran": "nublar",
    "nublaras": "nublar",
    "nublare": "nublar",
    "nublareis": "nublar",
    "nublaremos": "nublar",
    "nublaren": "nublar",
    "nublares": "nublar",
    "nublaron": "nublar",
    "nublará": "nublar",
    "nublarán": "nublar",
    "nublarás": "nublar",
    "nublaré": "nublar",
    "nublaréis": "nublar",
    "nublaría": "nublar",
    "nublaríais": "nublar",
    "nublaríamos": "nublar",
    "nublarían": "nublar",
    "nublarías": "nublar",
    "nublas": "nublo",
    "nublase": "nublar",
    "nublaseis": "nublar",
    "nublasen": "nublar",
    "nublases": "nublar",
    "nublaste": "nublar",
    "nublasteis": "nublar",
    "nuble": "nublar",
    "nublemos": "nublar",
    "nublen": "nublar",
    "nubles": "nublar",
    "nublo": "nublar",
    "nublos": "nublo",
    "nublábamos": "nublar",
    "nubláis": "nublar",
    "nubláramos": "nublar",
    "nubláremos": "nublar",
    "nublásemos": "nublar",
    "nublé": "nublar",
    "nubléis": "nublar",
    "nubló": "nublar",
    "nubosa": "nuboso",
    "nubosas": "nuboso",
    "nubosidades": "nubosidad",
    "nubosos": "nuboso",
    "nubuces": "nubuc",
    "nubuck": "nubuc",
    "nubucks": "nubuc",
    "nubífera": "nubífero",
    "nubíferas": "nubífero",
    "nubíferos": "nubífero",
    "nucales": "nucal",
    "nucas": "nuca",
    "nucleaciones": "nucleación",
    "nucleada": "nucleado",
    "nucleadas": "nucleado",
    "nucleados": "nucleado",
    "nucleares": "nuclear",
    "nuclearizaciones": "nuclearización",
    "nucleica": "nucleico",
    "nucleicas": "nucleico",
    "nucleicos": "nucleico",
    "nucleidos": "nucleido",
    "nucleinaciones": "nucleinación",
    "nucleoalbúminas": "nucleoalbúmina",
    "nucleocitoplásmica": "nucleocitoplásmico",
    "nucleocitoplásmicas": "nucleocitoplásmico",
    "nucleocitoplásmicos": "nucleocitoplásmico",
    "nucleocápsidas": "nucleocápsida",
    "nucleoides": "nucleoide",
    "nucleolares": "nucleolar",
    "nucleolos": "nucleolo",
    "nucleones": "nucleón",
    "nucleoplasmas": "nucleoplasma",
    "nucleoplasmática": "nucleoplasmático",
    "nucleoplasmáticas": "nucleoplasmático",
    "nucleoplasmáticos": "nucleoplasmático",
    "nucleoproteínas": "nucleoproteína",
    "nucleoquimos": "nucleoquimo",
    "nucleosomas": "nucleosoma",
    "nucleínas": "nucleína",
    "nucleófila": "nucleófilo",
    "nucleófilas": "nucleófilo",
    "nucleófilos": "nucleófilo",
    "nucleónica": "nucleónico",
    "nucleónicas": "nucleónico",
    "nucleónicos": "nucleónico",
    "nucleósidos": "nucleósido",
    "nucleótidas": "nucleótida",
    "nucleótidos": "nucleótido",
    "nucléolos": "nucléolo",
    "nuda": "nudo",
    "nudas": "nudo",
    "nudibranquios": "nudibranquio",
    "nudillos": "nudillo",
    "nudismos": "nudismo",
    "nudistas": "nudista",
    "nudos": "nudo",
    "nudosa": "nudoso",
    "nudosas": "nudoso",
    "nudosidades": "nudosidad",
    "nudosos": "nudoso",
    "nuececillas": "nuececilla",
    "nueces": "nuez",
    "nueras": "nuera",
    "nuestra": "nuestro",
    "nuestras": "nuestro",
    "nuestros": "nuestro",
    "nueva": "nuevo",
    "nuevas": "nuevo",
    "nuevos": "nuevo",
    "nuezas": "nueza",
    "nugatoria": "nugatorio",
    "nugatorias": "nugatorio",
    "nugatorios": "nugatorio",
    "nula": "nulo",
    "nulas": "nulo",
    "nulidades": "nulidad",
    "nulificaciones": "nulificación",
    "nulisómica": "nulisómico",
    "nulisómicas": "nulisómico",
    "nulisómicos": "nulisómico",
    "nulos": "nulo",
    "numantina": "numantino",
    "numantinas": "numantino",
    "numantinos": "numantino",
    "numbates": "numbat",
    "numera": "numerar",
    "numeraba": "numerar",
    "numerabais": "numerar",
    "numeraban": "numerar",
    "numerabas": "numerar",
    "numerabilidades": "numerabilidad",
    "numerables": "numerable",
    "numeraciones": "numeración",
    "numerad": "numerar",
    "numerada": "numerar",
    "numeradas": "numerar",
    "numerado": "numerar",
    "numeradora": "numerador",
    "numeradoras": "numerador",
    "numeradores": "numerador",
    "numerados": "numerar",
    "numerales": "numeral",
    "numeramos": "numerar",
    "numeran": "numerar",
    "numerando": "numerar",
    "numerara": "numerar",
    "numerarais": "numerar",
    "numeraran": "numerar",
    "numeraras": "numerar",
    "numerare": "numerar",
    "numerareis": "numerar",
    "numeraremos": "numerar",
    "numeraren": "numerar",
    "numerares": "numerar",
    "numeraria": "numerario",
    "numerarias": "numerario",
    "numerarios": "numerario",
    "numeraron": "numerar",
    "numerará": "numerar",
    "numerarán": "numerar",
    "numerarás": "numerar",
    "numeraré": "numerar",
    "numeraréis": "numerar",
    "numeraría": "numerar",
    "numeraríais": "numerar",
    "numeraríamos": "numerar",
    "numerarían": "numerar",
    "numerarías": "numerar",
    "numeras": "numerar",
    "numerase": "numerar",
    "numeraseis": "numerar",
    "numerasen": "numerar",
    "numerases": "numerar",
    "numeraste": "numerar",
    "numerasteis": "numerar",
    "numere": "numerar",
    "numeremos": "numerar",
    "numeren": "numerar",
    "numeres": "numerar",
    "numero": "numerar",
    "numerosa": "numeroso",
    "numerosas": "numeroso",
    "numerosos": "numeroso",
    "numerábamos": "numerar",
    "numeráis": "numerar",
    "numeráramos": "numerar",
    "numeráremos": "numerar",
    "numerásemos": "numerar",
    "numeré": "numerar",
    "numeréis": "numerar",
    "numeró": "numerar",
    "numismas": "numisma",
    "numismática": "numismático",
    "numismáticas": "numismático",
    "numismáticos": "numismático",
    "numulares": "numular",
    "numularios": "numulario",
    "numulítica": "numulítico",
    "numulíticas": "numulítico",
    "numulíticos": "numulítico",
    "numérica": "numérico",
    "numéricas": "numérico",
    "numéricos": "numérico",
    "numídica": "numídico",
    "numídicas": "numídico",
    "numídicos": "numídico",
    "nunciaturas": "nunciatura",
    "nuncios": "nuncio",
    "nuncupaciones": "nuncupación",
    "nuncupativa": "nuncupativo",
    "nuncupativas": "nuncupativo",
    "nuncupativos": "nuncupativo",
    "nuncupatoria": "nuncupatorio",
    "nuncupatorias": "nuncupatorio",
    "nuncupatorios": "nuncupatorio",
    "nupciales": "nupcial",
    "nupcialidades": "nupcialidad",
    "nuragas": "nuraga",
    "nurses": "nurse",
    "nutaciones": "nutación",
    "nutra": "nutrir",
    "nutramos": "nutrir",
    "nutran": "nutrir",
    "nutras": "nutrir",
    "nutre": "nutrir",
    "nutren": "nutrir",
    "nutres": "nutrir",
    "nutrias": "nutria",
    "nutricia": "nutricio",
    "nutricias": "nutricio",
    "nutricionales": "nutricional",
    "nutriciones": "nutrición",
    "nutricionistas": "nutricionista",
    "nutricios": "nutricio",
    "nutrid": "nutrir",
    "nutrida": "nutrir",
    "nutridas": "nutrir",
    "nutrido": "nutrir",
    "nutridos": "nutrir",
    "nutriendo": "nutrir",
    "nutrientes": "nutriente",
    "nutriera": "nutrir",
    "nutrierais": "nutrir",
    "nutrieran": "nutrir",
    "nutrieras": "nutrir",
    "nutriere": "nutrir",
    "nutriereis": "nutrir",
    "nutrieren": "nutrir",
    "nutrieres": "nutrir",
    "nutrieron": "nutrir",
    "nutriese": "nutrir",
    "nutrieseis": "nutrir",
    "nutriesen": "nutrir",
    "nutrieses": "nutrir",
    "nutrimentos": "nutrimento",
    "nutrimientos": "nutrimiento",
    "nutrimos": "nutrir",
    "nutriremos": "nutrir",
    "nutrirá": "nutrir",
    "nutrirán": "nutrir",
    "nutrirás": "nutrir",
    "nutriré": "nutrir",
    "nutriréis": "nutrir",
    "nutriría": "nutrir",
    "nutriríais": "nutrir",
    "nutriríamos": "nutrir",
    "nutrirían": "nutrir",
    "nutrirías": "nutrir",
    "nutriste": "nutrir",
    "nutristeis": "nutrir",
    "nutritiva": "nutritivo",
    "nutritivas": "nutritivo",
    "nutritivos": "nutritivo",
    "nutriéramos": "nutrir",
    "nutriéremos": "nutrir",
    "nutriésemos": "nutrir",
    "nutrió": "nutrir",
    "nutrióloga": "nutriólogo",
    "nutriólogas": "nutriólogo",
    "nutriólogos": "nutriólogo",
    "nutro": "nutrir",
    "nutráis": "nutrir",
    "nutrí": "nutrir",
    "nutría": "nutrir",
    "nutríais": "nutrir",
    "nutríamos": "nutrir",
    "nutrían": "nutrir",
    "nutrías": "nutrir",
    "nutrís": "nutrir",
    "nutuales": "nutual",
    "nuégados": "nuégado",
    "nuñismos": "nuñismo",
    "nyanjas": "nyanja",
    "nylones": "nylon",
    "nylons": "nylon",
    "nylónes": "nylón",
    "nys": "ny",
    "nº": "número",
    "nácaras": "nácara",
    "nácares": "nácar",
    "nádires": "nádir",
    "náguatles": "náguatle",
    "náhuatles": "náhuatle",
    "nártexes": "nártex",
    "násicos": "násico",
    "nátrica": "nátrico",
    "nátricas": "nátrico",
    "nátricos": "nátrico",
    "náufraga": "náufrago",
    "náufragas": "náufrago",
    "náufragos": "náufrago",
    "náuseas": "náusea",
    "náutica": "náutico",
    "náuticas": "náutico",
    "náuticos": "náutico",
    "náyades": "náyade",
    "nébedas": "nébeda",
    "nébulums": "nébulum",
    "nécoras": "nécora",
    "néctares": "néctar",
    "néperes": "néper",
    "nérvea": "nérveo",
    "nérveas": "nérveo",
    "nérveos": "nérveo",
    "néurulas": "néurula",
    "níqueles": "níquel",
    "níscalos": "níscalo",
    "nísperos": "níspero",
    "níspolas": "níspola",
    "nítida": "nítido",
    "nítidas": "nítido",
    "nítidos": "nítido",
    "nítrica": "nítrico",
    "nítricas": "nítrico",
    "nítricos": "nítrico",
    "nívea": "níveo",
    "níveas": "níveo",
    "níveos": "níveo",
    "nóctulos": "nóctulo",
    "nódulas": "nódula",
    "nódulos": "nódulo",
    "nómadas": "nómada",
    "nómica": "nómico",
    "nómicas": "nómico",
    "nómicos": "nómico",
    "nóminas": "nómina",
    "nómones": "nomon",
    "nónupla": "nónuplo",
    "nónuplas": "nónuplo",
    "nónuples": "nónuple",
    "nónuplos": "nónuplo",
    "nórdica": "nórdico",
    "nórdicas": "nórdico",
    "nórdicos": "nórdico",
    "nórtica": "nórtico",
    "nórticas": "nórtico",
    "nórticos": "nórtico",
    "nóstica": "nóstico",
    "nósticas": "nóstico",
    "nósticos": "nóstico",
    "núbiles": "núbil",
    "núcleos": "núcleo",
    "núclidos": "núclido",
    "núculas": "núcula",
    "númenes": "numen",
    "números": "número",
    "númidas": "númida",
    "obceca": "obcecar",
    "obcecaba": "obcecar",
    "obcecabais": "obcecar",
    "obcecaban": "obcecar",
    "obcecabas": "obcecar",
    "obcecaciones": "obcecación",
    "obcecad": "obcecar",
    "obcecada": "obcecar",
    "obcecadas": "obcecar",
    "obcecado": "obcecar",
    "obcecados": "obcecar",
    "obcecamos": "obcecar",
    "obcecan": "obcecar",
    "obcecando": "obcecar",
    "obcecara": "obcecar",
    "obcecarais": "obcecar",
    "obcecaran": "obcecar",
    "obcecaras": "obcecar",
    "obcecare": "obcecar",
    "obcecareis": "obcecar",
    "obcecaremos": "obcecar",
    "obcecaren": "obcecar",
    "obcecares": "obcecar",
    "obcecaron": "obcecar",
    "obcecará": "obcecar",
    "obcecarán": "obcecar",
    "obcecarás": "obcecar",
    "obcecaré": "obcecar",
    "obcecaréis": "obcecar",
    "obcecaría": "obcecar",
    "obcecaríais": "obcecar",
    "obcecaríamos": "obcecar",
    "obcecarían": "obcecar",
    "obcecarías": "obcecar",
    "obcecas": "obcecar",
    "obcecase": "obcecar",
    "obcecaseis": "obcecar",
    "obcecasen": "obcecar",
    "obcecases": "obcecar",
    "obcecaste": "obcecar",
    "obcecasteis": "obcecar",
    "obceco": "obcecar",
    "obcecábamos": "obcecar",
    "obcecáis": "obcecar",
    "obcecáramos": "obcecar",
    "obcecáremos": "obcecar",
    "obcecásemos": "obcecar",
    "obcecó": "obcecar",
    "obceque": "obcecar",
    "obcequemos": "obcecar",
    "obcequen": "obcecar",
    "obceques": "obcecar",
    "obcequé": "obcecar",
    "obcequéis": "obcecar",
    "obduraciones": "obduración",
    "obedece": "obedecer",
    "obedeced": "obedecer",
    "obedecemos": "obedecer",
    "obedecen": "obedecer",
    "obedeceremos": "obedecer",
    "obedecerá": "obedecer",
    "obedecerán": "obedecer",
    "obedecerás": "obedecer",
    "obedeceré": "obedecer",
    "obedeceréis": "obedecer",
    "obedecería": "obedecer",
    "obedeceríais": "obedecer",
    "obedeceríamos": "obedecer",
    "obedecerían": "obedecer",
    "obedecerías": "obedecer",
    "obedeces": "obedecer",
    "obedecibles": "obedecible",
    "obedecida": "obedecer",
    "obedecidas": "obedecer",
    "obedecido": "obedecer",
    "obedecidos": "obedecer",
    "obedeciendo": "obedecer",
    "obedeciera": "obedecer",
    "obedecierais": "obedecer",
    "obedecieran": "obedecer",
    "obedecieras": "obedecer",
    "obedeciere": "obedecer",
    "obedeciereis": "obedecer",
    "obedecieren": "obedecer",
    "obedecieres": "obedecer",
    "obedecieron": "obedecer",
    "obedeciese": "obedecer",
    "obedecieseis": "obedecer",
    "obedeciesen": "obedecer",
    "obedecieses": "obedecer",
    "obedecimientos": "obedecimiento",
    "obedecimos": "obedecer",
    "obedeciste": "obedecer",
    "obedecisteis": "obedecer",
    "obedeciéramos": "obedecer",
    "obedeciéremos": "obedecer",
    "obedeciésemos": "obedecer",
    "obedeció": "obedecer",
    "obedecéis": "obedecer",
    "obedecí": "obedecer",
    "obedecía": "obedecer",
    "obedecíais": "obedecer",
    "obedecíamos": "obedecer",
    "obedecían": "obedecer",
    "obedecías": "obedecer",
    "obedezca": "obedecer",
    "obedezcamos": "obedecer",
    "obedezcan": "obedecer",
    "obedezcas": "obedecer",
    "obedezco": "obedecer",
    "obedezcáis": "obedecer",
    "obediencias": "obediencia",
    "obedientes": "obediente",
    "obeliscos": "obelisco",
    "obelos": "obelo",
    "obencaduras": "obencadura",
    "obenques": "obenque",
    "oberturas": "obertura",
    "obesa": "obeso",
    "obesas": "obeso",
    "obesidades": "obesidad",
    "obesos": "obeso",
    "obispados": "obispado",
    "obispalías": "obispalía",
    "obispillos": "obispillo",
    "obispos": "obispo",
    "obituarios": "obituario",
    "objeciones": "objeción",
    "objeta": "objetar",
    "objetaba": "objetar",
    "objetabais": "objetar",
    "objetaban": "objetar",
    "objetabas": "objetar",
    "objetables": "objetable",
    "objetad": "objetar",
    "objetada": "objetar",
    "objetadas": "objetar",
    "objetado": "objetar",
    "objetados": "objetar",
    "objetamos": "objetar",
    "objetan": "objetar",
    "objetando": "objetar",
    "objetantes": "objetante",
    "objetara": "objetar",
    "objetarais": "objetar",
    "objetaran": "objetar",
    "objetaras": "objetar",
    "objetare": "objetar",
    "objetareis": "objetar",
    "objetaremos": "objetar",
    "objetaren": "objetar",
    "objetares": "objetar",
    "objetaron": "objetar",
    "objetará": "objetar",
    "objetarán": "objetar",
    "objetarás": "objetar",
    "objetaré": "objetar",
    "objetaréis": "objetar",
    "objetaría": "objetar",
    "objetaríais": "objetar",
    "objetaríamos": "objetar",
    "objetarían": "objetar",
    "objetarías": "objetar",
    "objetas": "objetar",
    "objetase": "objetar",
    "objetaseis": "objetar",
    "objetasen": "objetar",
    "objetases": "objetar",
    "objetaste": "objetar",
    "objetasteis": "objetar",
    "objete": "objetar",
    "objetemos": "objetar",
    "objeten": "objetar",
    "objetes": "objetar",
    "objetiva": "objetivo",
    "objetivaba": "objetivar",
    "objetivabais": "objetivar",
    "objetivaban": "objetivar",
    "objetivabas": "objetivar",
    "objetivaciones": "objetivación",
    "objetivad": "objetivar",
    "objetivada": "objetivar",
    "objetivadas": "objetivar",
    "objetivado": "objetivar",
    "objetivados": "objetivar",
    "objetivamos": "objetivar",
    "objetivan": "objetivar",
    "objetivando": "objetivar",
    "objetivara": "objetivar",
    "objetivarais": "objetivar",
    "objetivaran": "objetivar",
    "objetivaras": "objetivar",
    "objetivare": "objetivar",
    "objetivareis": "objetivar",
    "objetivaremos": "objetivar",
    "objetivaren": "objetivar",
    "objetivares": "objetivar",
    "objetivaron": "objetivar",
    "objetivará": "objetivar",
    "objetivarán": "objetivar",
    "objetivarás": "objetivar",
    "objetivaré": "objetivar",
    "objetivaréis": "objetivar",
    "objetivaría": "objetivar",
    "objetivaríais": "objetivar",
    "objetivaríamos": "objetivar",
    "objetivarían": "objetivar",
    "objetivarías": "objetivar",
    "objetivas": "objetivo",
    "objetivase": "objetivar",
    "objetivaseis": "objetivar",
    "objetivasen": "objetivar",
    "objetivases": "objetivar",
    "objetivaste": "objetivar",
    "objetivasteis": "objetivar",
    "objetive": "objetivar",
    "objetivemos": "objetivar",
    "objetiven": "objetivar",
    "objetives": "objetivar",
    "objetividades": "objetividad",
    "objetivismos": "objetivismo",
    "objetivo": "objetivar",
    "objetivos": "objetivo",
    "objetivábamos": "objetivar",
    "objetiváis": "objetivar",
    "objetiváramos": "objetivar",
    "objetiváremos": "objetivar",
    "objetivásemos": "objetivar",
    "objetivé": "objetivar",
    "objetivéis": "objetivar",
    "objetivó": "objetivar",
    "objeto": "objetar",
    "objetora": "objetor",
    "objetoras": "objetor",
    "objetores": "objetor",
    "objetos": "objeto",
    "objetábamos": "objetar",
    "objetáis": "objetar",
    "objetáramos": "objetar",
    "objetáremos": "objetar",
    "objetásemos": "objetar",
    "objeté": "objetar",
    "objetéis": "objetar",
    "objetó": "objetar",
    "oblaciones": "oblación",
    "obladas": "oblada",
    "oblata": "oblato",
    "oblatas": "oblato",
    "oblativa": "oblativo",
    "oblativas": "oblativo",
    "oblativos": "oblativo",
    "oblatos": "oblato",
    "obleas": "oblea",
    "obleeras": "obleera",
    "oblicua": "oblicuo",
    "oblicuaba": "oblicuar",
    "oblicuabais": "oblicuar",
    "oblicuaban": "oblicuar",
    "oblicuabas": "oblicuar",
    "oblicuad": "oblicuar",
    "oblicuada": "oblicuar",
    "oblicuadas": "oblicuar",
    "oblicuado": "oblicuar",
    "oblicuados": "oblicuar",
    "oblicuamos": "oblicuar",
    "oblicuando": "oblicuar",
    "oblicuara": "oblicuar",
    "oblicuarais": "oblicuar",
    "oblicuaran": "oblicuar",
    "oblicuaras": "oblicuar",
    "oblicuare": "oblicuar",
    "oblicuareis": "oblicuar",
    "oblicuaremos": "oblicuar",
    "oblicuaren": "oblicuar",
    "oblicuares": "oblicuar",
    "oblicuaron": "oblicuar",
    "oblicuará": "oblicuar",
    "oblicuarán": "oblicuar",
    "oblicuarás": "oblicuar",
    "oblicuaré": "oblicuar",
    "oblicuaréis": "oblicuar",
    "oblicuaría": "oblicuar",
    "oblicuaríais": "oblicuar",
    "oblicuaríamos": "oblicuar",
    "oblicuarían": "oblicuar",
    "oblicuarías": "oblicuar",
    "oblicuas": "oblicuo",
    "oblicuase": "oblicuar",
    "oblicuaseis": "oblicuar",
    "oblicuasen": "oblicuar",
    "oblicuases": "oblicuar",
    "oblicuaste": "oblicuar",
    "oblicuasteis": "oblicuar",
    "oblicuemos": "oblicuar",
    "oblicuidades": "oblicuidad",
    "oblicuos": "oblicuo",
    "oblicuábamos": "oblicuar",
    "oblicuáis": "oblicuar",
    "oblicuángula": "oblicuángulo",
    "oblicuángulas": "oblicuángulo",
    "oblicuángulos": "oblicuángulo",
    "oblicuáramos": "oblicuar",
    "oblicuáremos": "oblicuar",
    "oblicuásemos": "oblicuar",
    "oblicué": "oblicuar",
    "oblicuéis": "oblicuar",
    "oblicuó": "oblicuar",
    "oblicúa": "oblicuar",
    "oblicúan": "oblicuar",
    "oblicúas": "oblicuar",
    "oblicúe": "oblicuar",
    "oblicúen": "oblicuar",
    "oblicúes": "oblicuar",
    "oblicúo": "oblicuar",
    "obliga": "obligar",
    "obligaba": "obligar",
    "obligabais": "obligar",
    "obligaban": "obligar",
    "obligabas": "obligar",
    "obligacionales": "obligacional",
    "obligaciones": "obligación",
    "obligacionistas": "obligacionista",
    "obligad": "obligar",
    "obligada": "obligar",
    "obligadas": "obligar",
    "obligado": "obligar",
    "obligados": "obligar",
    "obligamos": "obligar",
    "obligan": "obligar",
    "obligando": "obligar",
    "obligantes": "obligante",
    "obligara": "obligar",
    "obligarais": "obligar",
    "obligaran": "obligar",
    "obligaras": "obligar",
    "obligare": "obligar",
    "obligareis": "obligar",
    "obligaremos": "obligar",
    "obligaren": "obligar",
    "obligares": "obligar",
    "obligaron": "obligar",
    "obligará": "obligar",
    "obligarán": "obligar",
    "obligarás": "obligar",
    "obligaré": "obligar",
    "obligaréis": "obligar",
    "obligaría": "obligar",
    "obligaríais": "obligar",
    "obligaríamos": "obligar",
    "obligarían": "obligar",
    "obligarías": "obligar",
    "obligas": "obligar",
    "obligase": "obligar",
    "obligaseis": "obligar",
    "obligasen": "obligar",
    "obligases": "obligar",
    "obligaste": "obligar",
    "obligasteis": "obligar",
    "obligativa": "obligativo",
    "obligativas": "obligativo",
    "obligativos": "obligativo",
    "obligatoria": "obligatorio",
    "obligatorias": "obligatorio",
    "obligatoriedades": "obligatoriedad",
    "obligatorios": "obligatorio",
    "obligo": "obligar",
    "obligue": "obligar",
    "obliguemos": "obligar",
    "obliguen": "obligar",
    "obligues": "obligar",
    "obligué": "obligar",
    "obliguéis": "obligar",
    "obligábamos": "obligar",
    "obligáis": "obligar",
    "obligáramos": "obligar",
    "obligáremos": "obligar",
    "obligásemos": "obligar",
    "obligó": "obligar",
    "obliqüidades": "obliqüidad",
    "oblitera": "obliterar",
    "obliteraba": "obliterar",
    "obliterabais": "obliterar",
    "obliteraban": "obliterar",
    "obliterabas": "obliterar",
    "obliteraciones": "obliteración",
    "obliterad": "obliterar",
    "obliterada": "obliterar",
    "obliteradas": "obliterar",
    "obliterado": "obliterar",
    "obliterados": "obliterar",
    "obliteramos": "obliterar",
    "obliteran": "obliterar",
    "obliterando": "obliterar",
    "obliterara": "obliterar",
    "obliterarais": "obliterar",
    "obliteraran": "obliterar",
    "obliteraras": "obliterar",
    "obliterare": "obliterar",
    "obliterareis": "obliterar",
    "obliteraremos": "obliterar",
    "obliteraren": "obliterar",
    "obliterares": "obliterar",
    "obliteraron": "obliterar",
    "obliterará": "obliterar",
    "obliterarán": "obliterar",
    "obliterarás": "obliterar",
    "obliteraré": "obliterar",
    "obliteraréis": "obliterar",
    "obliteraría": "obliterar",
    "obliteraríais": "obliterar",
    "obliteraríamos": "obliterar",
    "obliterarían": "obliterar",
    "obliterarías": "obliterar",
    "obliteras": "obliterar",
    "obliterase": "obliterar",
    "obliteraseis": "obliterar",
    "obliterasen": "obliterar",
    "obliterases": "obliterar",
    "obliteraste": "obliterar",
    "obliterasteis": "obliterar",
    "obliterativa": "obliterativo",
    "obliterativas": "obliterativo",
    "obliterativos": "obliterativo",
    "oblitere": "obliterar",
    "obliteremos": "obliterar",
    "obliteren": "obliterar",
    "obliteres": "obliterar",
    "oblitero": "obliterar",
    "obliterábamos": "obliterar",
    "obliteráis": "obliterar",
    "obliteráramos": "obliterar",
    "obliteráremos": "obliterar",
    "obliterásemos": "obliterar",
    "obliteré": "obliterar",
    "obliteréis": "obliterar",
    "obliteró": "obliterar",
    "oblonga": "oblongo",
    "oblongas": "oblongo",
    "oblongos": "oblongo",
    "obnubila": "obnubilar",
    "obnubilaba": "obnubilar",
    "obnubilabais": "obnubilar",
    "obnubilaban": "obnubilar",
    "obnubilabas": "obnubilar",
    "obnubilaciones": "obnubilación",
    "obnubilad": "obnubilar",
    "obnubilada": "obnubilar",
    "obnubiladas": "obnubilar",
    "obnubilado": "obnubilar",
    "obnubilados": "obnubilar",
    "obnubilamos": "obnubilar",
    "obnubilan": "obnubilar",
    "obnubilando": "obnubilar",
    "obnubilara": "obnubilar",
    "obnubilarais": "obnubilar",
    "obnubilaran": "obnubilar",
    "obnubilaras": "obnubilar",
    "obnubilare": "obnubilar",
    "obnubilareis": "obnubilar",
    "obnubilaremos": "obnubilar",
    "obnubilaren": "obnubilar",
    "obnubilares": "obnubilar",
    "obnubilaron": "obnubilar",
    "obnubilará": "obnubilar",
    "obnubilarán": "obnubilar",
    "obnubilarás": "obnubilar",
    "obnubilaré": "obnubilar",
    "obnubilaréis": "obnubilar",
    "obnubilaría": "obnubilar",
    "obnubilaríais": "obnubilar",
    "obnubilaríamos": "obnubilar",
    "obnubilarían": "obnubilar",
    "obnubilarías": "obnubilar",
    "obnubilas": "obnubilar",
    "obnubilase": "obnubilar",
    "obnubilaseis": "obnubilar",
    "obnubilasen": "obnubilar",
    "obnubilases": "obnubilar",
    "obnubilaste": "obnubilar",
    "obnubilasteis": "obnubilar",
    "obnubile": "obnubilar",
    "obnubilemos": "obnubilar",
    "obnubilen": "obnubilar",
    "obnubiles": "obnubilar",
    "obnubilo": "obnubilar",
    "obnubilábamos": "obnubilar",
    "obnubiláis": "obnubilar",
    "obnubiláramos": "obnubilar",
    "obnubiláremos": "obnubilar",
    "obnubilásemos": "obnubilar",
    "obnubilé": "obnubilar",
    "obnubiléis": "obnubilar",
    "obnubiló": "obnubilar",
    "oboes": "oboe",
    "oboístas": "oboísta",
    "obra": "obrar",
    "obraba": "obrar",
    "obrabais": "obrar",
    "obraban": "obrar",
    "obrabas": "obrar",
    "obrad": "obrar",
    "obrada": "obrar",
    "obradas": "obrar",
    "obrado": "obrar",
    "obradora": "obrador",
    "obradoras": "obrador",
    "obradores": "obrador",
    "obrados": "obrar",
    "obraduras": "obradura",
    "obrajera": "obrajero",
    "obrajeras": "obrajero",
    "obrajeros": "obrajero",
    "obrajes": "obraje",
    "obramos": "obrar",
    "obran": "obrar",
    "obrando": "obrar",
    "obrara": "obrar",
    "obrarais": "obrar",
    "obraran": "obrar",
    "obraras": "obrar",
    "obrare": "obrar",
    "obrareis": "obrar",
    "obraremos": "obrar",
    "obraren": "obrar",
    "obrares": "obrar",
    "obraron": "obrar",
    "obrará": "obrar",
    "obrarán": "obrar",
    "obrarás": "obrar",
    "obraré": "obrar",
    "obraréis": "obrar",
    "obraría": "obrar",
    "obraríais": "obrar",
    "obraríamos": "obrar",
    "obrarían": "obrar",
    "obrarías": "obrar",
    "obras": "obrar",
    "obrase": "obrar",
    "obraseis": "obrar",
    "obrasen": "obrar",
    "obrases": "obrar",
    "obraste": "obrar",
    "obrasteis": "obrar",
    "obre": "obrar",
    "obremos": "obrar",
    "obren": "obrar",
    "obrepciones": "obrepción",
    "obrepticia": "obrepticio",
    "obrepticias": "obrepticio",
    "obrepticios": "obrepticio",
    "obrera": "obrero",
    "obreras": "obrero",
    "obrerismos": "obrerismo",
    "obreristas": "obrerista",
    "obreros": "obrero",
    "obrerías": "obrería",
    "obres": "obrar",
    "obro": "obrar",
    "obrábamos": "obrar",
    "obráis": "obrar",
    "obráramos": "obrar",
    "obráremos": "obrar",
    "obrásemos": "obrar",
    "obré": "obrar",
    "obréis": "obrar",
    "obró": "obrar",
    "obscena": "obsceno",
    "obscenas": "obsceno",
    "obscenidades": "obscenidad",
    "obscenos": "obsceno",
    "obscura": "obscuro",
    "obscurantismos": "obscurantismo",
    "obscurantistas": "obscurantista",
    "obscuras": "obscuro",
    "obscurece": "obscurecer",
    "obscureced": "obscurecer",
    "obscurecemos": "obscurecer",
    "obscurecen": "obscurecer",
    "obscureceremos": "obscurecer",
    "obscurecerá": "obscurecer",
    "obscurecerán": "obscurecer",
    "obscurecerás": "obscurecer",
    "obscureceré": "obscurecer",
    "obscureceréis": "obscurecer",
    "obscurecería": "obscurecer",
    "obscureceríais": "obscurecer",
    "obscureceríamos": "obscurecer",
    "obscurecerían": "obscurecer",
    "obscurecerías": "obscurecer",
    "obscureces": "obscurecer",
    "obscurecida": "obscurecer",
    "obscurecidas": "obscurecer",
    "obscurecido": "obscurecer",
    "obscurecidos": "obscurecer",
    "obscureciendo": "obscurecer",
    "obscureciera": "obscurecer",
    "obscurecierais": "obscurecer",
    "obscurecieran": "obscurecer",
    "obscurecieras": "obscurecer",
    "obscureciere": "obscurecer",
    "obscureciereis": "obscurecer",
    "obscurecieren": "obscurecer",
    "obscurecieres": "obscurecer",
    "obscurecieron": "obscurecer",
    "obscureciese": "obscurecer",
    "obscurecieseis": "obscurecer",
    "obscureciesen": "obscurecer",
    "obscurecieses": "obscurecer",
    "obscurecimientos": "obscurecimiento",
    "obscurecimos": "obscurecer",
    "obscureciste": "obscurecer",
    "obscurecisteis": "obscurecer",
    "obscureciéramos": "obscurecer",
    "obscureciéremos": "obscurecer",
    "obscureciésemos": "obscurecer",
    "obscureció": "obscurecer",
    "obscurecéis": "obscurecer",
    "obscurecí": "obscurecer",
    "obscurecía": "obscurecer",
    "obscurecíais": "obscurecer",
    "obscurecíamos": "obscurecer",
    "obscurecían": "obscurecer",
    "obscurecías": "obscurecer",
    "obscurezca": "obscurecer",
    "obscurezcamos": "obscurecer",
    "obscurezcan": "obscurecer",
    "obscurezcas": "obscurecer",
    "obscurezco": "obscurecer",
    "obscurezcáis": "obscurecer",
    "obscuridades": "obscuridad",
    "obscuros": "obscuro",
    "obsecraciones": "obsecración",
    "obsecuencias": "obsecuencia",
    "obsecuentes": "obsecuente",
    "obseda": "obseder",
    "obsedamos": "obseder",
    "obsedan": "obseder",
    "obsedas": "obseder",
    "obsede": "obseder",
    "obseded": "obseder",
    "obsedemos": "obseder",
    "obseden": "obseder",
    "obsederemos": "obseder",
    "obsederá": "obseder",
    "obsederán": "obseder",
    "obsederás": "obseder",
    "obsederé": "obseder",
    "obsederéis": "obseder",
    "obsedería": "obseder",
    "obsederíais": "obseder",
    "obsederíamos": "obseder",
    "obsederían": "obseder",
    "obsederías": "obseder",
    "obsedes": "obseder",
    "obsedida": "obseder",
    "obsedidas": "obseder",
    "obsedido": "obseder",
    "obsedidos": "obseder",
    "obsediendo": "obseder",
    "obsediera": "obseder",
    "obsedierais": "obseder",
    "obsedieran": "obseder",
    "obsedieras": "obseder",
    "obsediere": "obseder",
    "obsediereis": "obseder",
    "obsedieren": "obseder",
    "obsedieres": "obseder",
    "obsedieron": "obseder",
    "obsediese": "obseder",
    "obsedieseis": "obseder",
    "obsediesen": "obseder",
    "obsedieses": "obseder",
    "obsedimos": "obseder",
    "obsediste": "obseder",
    "obsedisteis": "obseder",
    "obsediéramos": "obseder",
    "obsediéremos": "obseder",
    "obsediésemos": "obseder",
    "obsedió": "obseder",
    "obsedo": "obseder",
    "obsedáis": "obseder",
    "obsedéis": "obseder",
    "obsedí": "obseder",
    "obsedía": "obseder",
    "obsedíais": "obseder",
    "obsedíamos": "obseder",
    "obsedían": "obseder",
    "obsedías": "obseder",
    "obsequia": "obsequiar",
    "obsequiaba": "obsequiar",
    "obsequiabais": "obsequiar",
    "obsequiaban": "obsequiar",
    "obsequiabas": "obsequiar",
    "obsequiad": "obsequiar",
    "obsequiada": "obsequiar",
    "obsequiadas": "obsequiar",
    "obsequiado": "obsequiar",
    "obsequiadora": "obsequiador",
    "obsequiadoras": "obsequiador",
    "obsequiadores": "obsequiador",
    "obsequiados": "obsequiar",
    "obsequiamos": "obsequiar",
    "obsequian": "obsequiar",
    "obsequiando": "obsequiar",
    "obsequiantes": "obsequiante",
    "obsequiara": "obsequiar",
    "obsequiarais": "obsequiar",
    "obsequiaran": "obsequiar",
    "obsequiaras": "obsequiar",
    "obsequiare": "obsequiar",
    "obsequiareis": "obsequiar",
    "obsequiaremos": "obsequiar",
    "obsequiaren": "obsequiar",
    "obsequiares": "obsequiar",
    "obsequiaron": "obsequiar",
    "obsequiará": "obsequiar",
    "obsequiarán": "obsequiar",
    "obsequiarás": "obsequiar",
    "obsequiaré": "obsequiar",
    "obsequiaréis": "obsequiar",
    "obsequiaría": "obsequiar",
    "obsequiaríais": "obsequiar",
    "obsequiaríamos": "obsequiar",
    "obsequiarían": "obsequiar",
    "obsequiarías": "obsequiar",
    "obsequias": "obsequiar",
    "obsequiase": "obsequiar",
    "obsequiaseis": "obsequiar",
    "obsequiasen": "obsequiar",
    "obsequiases": "obsequiar",
    "obsequiaste": "obsequiar",
    "obsequiasteis": "obsequiar",
    "obsequie": "obsequiar",
    "obsequiemos": "obsequiar",
    "obsequien": "obsequiar",
    "obsequies": "obsequiar",
    "obsequio": "obsequiar",
    "obsequios": "obsequio",
    "obsequiosa": "obsequioso",
    "obsequiosas": "obsequioso",
    "obsequiosidades": "obsequiosidad",
    "obsequiosos": "obsequioso",
    "obsequiábamos": "obsequiar",
    "obsequiáis": "obsequiar",
    "obsequiáramos": "obsequiar",
    "obsequiáremos": "obsequiar",
    "obsequiásemos": "obsequiar",
    "obsequié": "obsequiar",
    "obsequiéis": "obsequiar",
    "obsequió": "obsequiar",
    "observa": "observar",
    "observaba": "observar",
    "observabais": "observar",
    "observaban": "observar",
    "observabas": "observar",
    "observabilidades": "observabilidad",
    "observables": "observable",
    "observacionales": "observacional",
    "observaciones": "observación",
    "observad": "observar",
    "observada": "observar",
    "observadas": "observar",
    "observado": "observar",
    "observadora": "observador",
    "observadoras": "observador",
    "observadores": "observador",
    "observados": "observar",
    "observamos": "observar",
    "observan": "observar",
    "observancias": "observancia",
    "observando": "observar",
    "observantes": "observante",
    "observara": "observar",
    "observarais": "observar",
    "observaran": "observar",
    "observaras": "observar",
    "observare": "observar",
    "observareis": "observar",
    "observaremos": "observar",
    "observaren": "observar",
    "observares": "observar",
    "observaron": "observar",
    "observará": "observar",
    "observarán": "observar",
    "observarás": "observar",
    "observaré": "observar",
    "observaréis": "observar",
    "observaría": "observar",
    "observaríais": "observar",
    "observaríamos": "observar",
    "observarían": "observar",
    "observarías": "observar",
    "observas": "observar",
    "observase": "observar",
    "observaseis": "observar",
    "observasen": "observar",
    "observases": "observar",
    "observaste": "observar",
    "observasteis": "observar",
    "observatorios": "observatorio",
    "observe": "observar",
    "observemos": "observar",
    "observen": "observar",
    "observes": "observar",
    "observo": "observar",
    "observábamos": "observar",
    "observáis": "observar",
    "observáramos": "observar",
    "observáremos": "observar",
    "observásemos": "observar",
    "observé": "observar",
    "observéis": "observar",
    "observó": "observar",
    "obsesa": "obseso",
    "obsesas": "obseso",
    "obsesiona": "obsesionar",
    "obsesionaba": "obsesionar",
    "obsesionabais": "obsesionar",
    "obsesionaban": "obsesionar",
    "obsesionabas": "obsesionar",
    "obsesionad": "obsesionar",
    "obsesionada": "obsesionar",
    "obsesionadas": "obsesionar",
    "obsesionado": "obsesionar",
    "obsesionados": "obsesionar",
    "obsesionamos": "obsesionar",
    "obsesionan": "obsesionar",
    "obsesionando": "obsesionar",
    "obsesionantes": "obsesionante",
    "obsesionara": "obsesionar",
    "obsesionarais": "obsesionar",
    "obsesionaran": "obsesionar",
    "obsesionaras": "obsesionar",
    "obsesionare": "obsesionar",
    "obsesionareis": "obsesionar",
    "obsesionaremos": "obsesionar",
    "obsesionaren": "obsesionar",
    "obsesionares": "obsesionar",
    "obsesionaron": "obsesionar",
    "obsesionará": "obsesionar",
    "obsesionarán": "obsesionar",
    "obsesionarás": "obsesionar",
    "obsesionaré": "obsesionar",
    "obsesionaréis": "obsesionar",
    "obsesionaría": "obsesionar",
    "obsesionaríais": "obsesionar",
    "obsesionaríamos": "obsesionar",
    "obsesionarían": "obsesionar",
    "obsesionarías": "obsesionar",
    "obsesionas": "obsesionar",
    "obsesionase": "obsesionar",
    "obsesionaseis": "obsesionar",
    "obsesionasen": "obsesionar",
    "obsesionases": "obsesionar",
    "obsesionaste": "obsesionar",
    "obsesionasteis": "obsesionar",
    "obsesione": "obsesionar",
    "obsesionemos": "obsesionar",
    "obsesionen": "obsesionar",
    "obsesiones": "obsesionar",
    "obsesiono": "obsesionar",
    "obsesionábamos": "obsesionar",
    "obsesionáis": "obsesionar",
    "obsesionáramos": "obsesionar",
    "obsesionáremos": "obsesionar",
    "obsesionásemos": "obsesionar",
    "obsesioné": "obsesionar",
    "obsesionéis": "obsesionar",
    "obsesionó": "obsesionar",
    "obsesiva": "obsesivo",
    "obsesivas": "obsesivo",
    "obsesivos": "obsesivo",
    "obsesos": "obseso",
    "obsidianas": "obsidiana",
    "obsidionales": "obsidional",
    "obsolescencias": "obsolescencia",
    "obsolescentes": "obsolescente",
    "obsoleta": "obsoleto",
    "obsoletas": "obsoleto",
    "obsoletos": "obsoleto",
    "obsta": "obstar",
    "obstaba": "obstar",
    "obstabais": "obstar",
    "obstaban": "obstar",
    "obstabas": "obstar",
    "obstaculice": "obstaculizar",
    "obstaculicemos": "obstaculizar",
    "obstaculicen": "obstaculizar",
    "obstaculices": "obstaculizar",
    "obstaculicé": "obstaculizar",
    "obstaculicéis": "obstaculizar",
    "obstaculiza": "obstaculizar",
    "obstaculizaba": "obstaculizar",
    "obstaculizabais": "obstaculizar",
    "obstaculizaban": "obstaculizar",
    "obstaculizabas": "obstaculizar",
    "obstaculizad": "obstaculizar",
    "obstaculizada": "obstaculizar",
    "obstaculizadas": "obstaculizar",
    "obstaculizado": "obstaculizar",
    "obstaculizadora": "obstaculizador",
    "obstaculizadoras": "obstaculizador",
    "obstaculizadores": "obstaculizador",
    "obstaculizados": "obstaculizar",
    "obstaculizamos": "obstaculizar",
    "obstaculizan": "obstaculizar",
    "obstaculizando": "obstaculizar",
    "obstaculizara": "obstaculizar",
    "obstaculizarais": "obstaculizar",
    "obstaculizaran": "obstaculizar",
    "obstaculizaras": "obstaculizar",
    "obstaculizare": "obstaculizar",
    "obstaculizareis": "obstaculizar",
    "obstaculizaremos": "obstaculizar",
    "obstaculizaren": "obstaculizar",
    "obstaculizares": "obstaculizar",
    "obstaculizaron": "obstaculizar",
    "obstaculizará": "obstaculizar",
    "obstaculizarán": "obstaculizar",
    "obstaculizarás": "obstaculizar",
    "obstaculizaré": "obstaculizar",
    "obstaculizaréis": "obstaculizar",
    "obstaculizaría": "obstaculizar",
    "obstaculizaríais": "obstaculizar",
    "obstaculizaríamos": "obstaculizar",
    "obstaculizarían": "obstaculizar",
    "obstaculizarías": "obstaculizar",
    "obstaculizas": "obstaculizar",
    "obstaculizase": "obstaculizar",
    "obstaculizaseis": "obstaculizar",
    "obstaculizasen": "obstaculizar",
    "obstaculizases": "obstaculizar",
    "obstaculizaste": "obstaculizar",
    "obstaculizasteis": "obstaculizar",
    "obstaculizo": "obstaculizar",
    "obstaculizábamos": "obstaculizar",
    "obstaculizáis": "obstaculizar",
    "obstaculizáramos": "obstaculizar",
    "obstaculizáremos": "obstaculizar",
    "obstaculizásemos": "obstaculizar",
    "obstaculizó": "obstaculizar",
    "obstad": "obstar",
    "obstada": "obstar",
    "obstadas": "obstar",
    "obstado": "obstar",
    "obstados": "obstar",
    "obstamos": "obstar",
    "obstan": "obstar",
    "obstando": "obstar",
    "obstara": "obstar",
    "obstarais": "obstar",
    "obstaran": "obstar",
    "obstaras": "obstar",
    "obstare": "obstar",
    "obstareis": "obstar",
    "obstaremos": "obstar",
    "obstaren": "obstar",
    "obstares": "obstar",
    "obstaron": "obstar",
    "obstará": "obstar",
    "obstarán": "obstar",
    "obstarás": "obstar",
    "obstaré": "obstar",
    "obstaréis": "obstar",
    "obstaría": "obstar",
    "obstaríais": "obstar",
    "obstaríamos": "obstar",
    "obstarían": "obstar",
    "obstarías": "obstar",
    "obstas": "obstar",
    "obstase": "obstar",
    "obstaseis": "obstar",
    "obstasen": "obstar",
    "obstases": "obstar",
    "obstaste": "obstar",
    "obstasteis": "obstar",
    "obste": "obstar",
    "obstemos": "obstar",
    "obsten": "obstar",
    "obstes": "obstar",
    "obstetricias": "obstetricia",
    "obstina": "obstinar",
    "obstinaba": "obstinar",
    "obstinabais": "obstinar",
    "obstinaban": "obstinar",
    "obstinabas": "obstinar",
    "obstinaciones": "obstinación",
    "obstinad": "obstinar",
    "obstinada": "obstinar",
    "obstinadas": "obstinar",
    "obstinado": "obstinar",
    "obstinados": "obstinar",
    "obstinamos": "obstinar",
    "obstinan": "obstinar",
    "obstinando": "obstinar",
    "obstinara": "obstinar",
    "obstinarais": "obstinar",
    "obstinaran": "obstinar",
    "obstinaras": "obstinar",
    "obstinare": "obstinar",
    "obstinareis": "obstinar",
    "obstinaremos": "obstinar",
    "obstinaren": "obstinar",
    "obstinares": "obstinar",
    "obstinaron": "obstinar",
    "obstinará": "obstinar",
    "obstinarán": "obstinar",
    "obstinarás": "obstinar",
    "obstinaré": "obstinar",
    "obstinaréis": "obstinar",
    "obstinaría": "obstinar",
    "obstinaríais": "obstinar",
    "obstinaríamos": "obstinar",
    "obstinarían": "obstinar",
    "obstinarías": "obstinar",
    "obstinas": "obstinar",
    "obstinase": "obstinar",
    "obstinaseis": "obstinar",
    "obstinasen": "obstinar",
    "obstinases": "obstinar",
    "obstinaste": "obstinar",
    "obstinasteis": "obstinar",
    "obstine": "obstinar",
    "obstinemos": "obstinar",
    "obstinen": "obstinar",
    "obstines": "obstinar",
    "obstino": "obstinar",
    "obstinábamos": "obstinar",
    "obstináis": "obstinar",
    "obstináramos": "obstinar",
    "obstináremos": "obstinar",
    "obstinásemos": "obstinar",
    "obstiné": "obstinar",
    "obstinéis": "obstinar",
    "obstinó": "obstinar",
    "obsto": "obstar",
    "obstrucciones": "obstrucción",
    "obstruccionismos": "obstruccionismo",
    "obstruccionistas": "obstruccionista",
    "obstructiva": "obstructivo",
    "obstructivas": "obstructivo",
    "obstructivismos": "obstructivismo",
    "obstructivos": "obstructivo",
    "obstructora": "obstructor",
    "obstructoras": "obstructor",
    "obstructores": "obstructor",
    "obstruid": "obstruir",
    "obstruida": "obstruir",
    "obstruidas": "obstruir",
    "obstruido": "obstruir",
    "obstruidos": "obstruir",
    "obstruimos": "obstruir",
    "obstruiremos": "obstruir",
    "obstruirá": "obstruir",
    "obstruirán": "obstruir",
    "obstruirás": "obstruir",
    "obstruiré": "obstruir",
    "obstruiréis": "obstruir",
    "obstruiría": "obstruir",
    "obstruiríais": "obstruir",
    "obstruiríamos": "obstruir",
    "obstruirían": "obstruir",
    "obstruirías": "obstruir",
    "obstruiste": "obstruir",
    "obstruisteis": "obstruir",
    "obstruya": "obstruir",
    "obstruyamos": "obstruir",
    "obstruyan": "obstruir",
    "obstruyas": "obstruir",
    "obstruye": "obstruir",
    "obstruyen": "obstruir",
    "obstruyendo": "obstruir",
    "obstruyentes": "obstruyente",
    "obstruyera": "obstruir",
    "obstruyerais": "obstruir",
    "obstruyeran": "obstruir",
    "obstruyeras": "obstruir",
    "obstruyere": "obstruir",
    "obstruyereis": "obstruir",
    "obstruyeren": "obstruir",
    "obstruyeres": "obstruir",
    "obstruyeron": "obstruir",
    "obstruyes": "obstruir",
    "obstruyese": "obstruir",
    "obstruyeseis": "obstruir",
    "obstruyesen": "obstruir",
    "obstruyeses": "obstruir",
    "obstruyo": "obstruir",
    "obstruyáis": "obstruir",
    "obstruyéramos": "obstruir",
    "obstruyéremos": "obstruir",
    "obstruyésemos": "obstruir",
    "obstruyó": "obstruir",
    "obstruí": "obstruir",
    "obstruía": "obstruir",
    "obstruíais": "obstruir",
    "obstruíamos": "obstruir",
    "obstruían": "obstruir",
    "obstruías": "obstruir",
    "obstruís": "obstruir",
    "obstábamos": "obstar",
    "obstáculos": "obstáculo",
    "obstáis": "obstar",
    "obstáramos": "obstar",
    "obstáremos": "obstar",
    "obstásemos": "obstar",
    "obsté": "obstar",
    "obstéis": "obstar",
    "obstétrica": "obstétrico",
    "obstétricas": "obstétrico",
    "obstétricos": "obstétrico",
    "obstó": "obstar",
    "obtempera": "obtemperar",
    "obtemperaba": "obtemperar",
    "obtemperabais": "obtemperar",
    "obtemperaban": "obtemperar",
    "obtemperabas": "obtemperar",
    "obtemperad": "obtemperar",
    "obtemperada": "obtemperar",
    "obtemperadas": "obtemperar",
    "obtemperado": "obtemperar",
    "obtemperados": "obtemperar",
    "obtemperamos": "obtemperar",
    "obtemperan": "obtemperar",
    "obtemperando": "obtemperar",
    "obtemperara": "obtemperar",
    "obtemperarais": "obtemperar",
    "obtemperaran": "obtemperar",
    "obtemperaras": "obtemperar",
    "obtemperare": "obtemperar",
    "obtemperareis": "obtemperar",
    "obtemperaremos": "obtemperar",
    "obtemperaren": "obtemperar",
    "obtemperares": "obtemperar",
    "obtemperaron": "obtemperar",
    "obtemperará": "obtemperar",
    "obtemperarán": "obtemperar",
    "obtemperarás": "obtemperar",
    "obtemperaré": "obtemperar",
    "obtemperaréis": "obtemperar",
    "obtemperaría": "obtemperar",
    "obtemperaríais": "obtemperar",
    "obtemperaríamos": "obtemperar",
    "obtemperarían": "obtemperar",
    "obtemperarías": "obtemperar",
    "obtemperas": "obtemperar",
    "obtemperase": "obtemperar",
    "obtemperaseis": "obtemperar",
    "obtemperasen": "obtemperar",
    "obtemperases": "obtemperar",
    "obtemperaste": "obtemperar",
    "obtemperasteis": "obtemperar",
    "obtempere": "obtemperar",
    "obtemperemos": "obtemperar",
    "obtemperen": "obtemperar",
    "obtemperes": "obtemperar",
    "obtempero": "obtemperar",
    "obtemperábamos": "obtemperar",
    "obtemperáis": "obtemperar",
    "obtemperáramos": "obtemperar",
    "obtemperáremos": "obtemperar",
    "obtemperásemos": "obtemperar",
    "obtemperé": "obtemperar",
    "obtemperéis": "obtemperar",
    "obtemperó": "obtemperar",
    "obtenciones": "obtención",
    "obtendremos": "obtener",
    "obtendrá": "obtener",
    "obtendrán": "obtener",
    "obtendrás": "obtener",
    "obtendré": "obtener",
    "obtendréis": "obtener",
    "obtendría": "obtener",
    "obtendríais": "obtener",
    "obtendríamos": "obtener",
    "obtendrían": "obtener",
    "obtendrías": "obtener",
    "obtened": "obtener",
    "obtenedora": "obtenedor",
    "obtenedoras": "obtenedor",
    "obtenedores": "obtenedor",
    "obtenemos": "obtener",
    "obtenga": "obtener",
    "obtengamos": "obtener",
    "obtengan": "obtener",
    "obtengas": "obtener",
    "obtengo": "obtener",
    "obtengáis": "obtener",
    "obtenibles": "obtenible",
    "obtenida": "obtener",
    "obtenidas": "obtener",
    "obtenido": "obtener",
    "obtenidos": "obtener",
    "obteniendo": "obtener",
    "obtenéis": "obtener",
    "obtenía": "obtener",
    "obteníais": "obtener",
    "obteníamos": "obtener",
    "obtenían": "obtener",
    "obtenías": "obtener",
    "obtestaciones": "obtestación",
    "obtiene": "obtener",
    "obtienen": "obtener",
    "obtienes": "obtener",
    "obtura": "obturar",
    "obturaba": "obturar",
    "obturabais": "obturar",
    "obturaban": "obturar",
    "obturabas": "obturar",
    "obturaciones": "obturación",
    "obturad": "obturar",
    "obturada": "obturar",
    "obturadas": "obturar",
    "obturado": "obturar",
    "obturadora": "obturador",
    "obturadoras": "obturador",
    "obturadores": "obturador",
    "obturados": "obturar",
    "obturamos": "obturar",
    "obturan": "obturar",
    "obturando": "obturar",
    "obturara": "obturar",
    "obturarais": "obturar",
    "obturaran": "obturar",
    "obturaras": "obturar",
    "obturare": "obturar",
    "obturareis": "obturar",
    "obturaremos": "obturar",
    "obturaren": "obturar",
    "obturares": "obturar",
    "obturaron": "obturar",
    "obturará": "obturar",
    "obturarán": "obturar",
    "obturarás": "obturar",
    "obturaré": "obturar",
    "obturaréis": "obturar",
    "obturaría": "obturar",
    "obturaríais": "obturar",
    "obturaríamos": "obturar",
    "obturarían": "obturar",
    "obturarías": "obturar",
    "obturas": "obturar",
    "obturase": "obturar",
    "obturaseis": "obturar",
    "obturasen": "obturar",
    "obturases": "obturar",
    "obturaste": "obturar",
    "obturasteis": "obturar",
    "obture": "obturar",
    "obturemos": "obturar",
    "obturen": "obturar",
    "obtures": "obturar",
    "obturo": "obturar",
    "obturábamos": "obturar",
    "obturáis": "obturar",
    "obturáramos": "obturar",
    "obturáremos": "obturar",
    "obturásemos": "obturar",
    "obturé": "obturar",
    "obturéis": "obturar",
    "obturó": "obturar",
    "obtusa": "obtuso",
    "obtusas": "obtuso",
    "obtusos": "obtuso",
    "obtusángulos": "obtusángulo",
    "obtuve": "obtener",
    "obtuviera": "obtener",
    "obtuvierais": "obtener",
    "obtuvieran": "obtener",
    "obtuvieras": "obtener",
    "obtuviere": "obtener",
    "obtuviereis": "obtener",
    "obtuvieren": "obtener",
    "obtuvieres": "obtener",
    "obtuvieron": "obtener",
    "obtuviese": "obtener",
    "obtuvieseis": "obtener",
    "obtuviesen": "obtener",
    "obtuvieses": "obtener",
    "obtuvimos": "obtener",
    "obtuviste": "obtener",
    "obtuvisteis": "obtener",
    "obtuviéramos": "obtener",
    "obtuviéremos": "obtener",
    "obtuviésemos": "obtener",
    "obtuvo": "obtener",
    "obtén": "obtener",
    "obuses": "obús",
    "obvencionales": "obvencional",
    "obvenciones": "obvención",
    "obventicia": "obventicio",
    "obventicias": "obventicio",
    "obventicios": "obventicio",
    "obvia": "obvio",
    "obviaba": "obviar",
    "obviabais": "obviar",
    "obviaban": "obviar",
    "obviabas": "obviar",
    "obviad": "obviar",
    "obviada": "obviar",
    "obviadas": "obviar",
    "obviado": "obviar",
    "obviados": "obviar",
    "obviamos": "obviar",
    "obvian": "obviar",
    "obviando": "obviar",
    "obviara": "obviar",
    "obviarais": "obviar",
    "obviaran": "obviar",
    "obviaras": "obviar",
    "obviare": "obviar",
    "obviareis": "obviar",
    "obviaremos": "obviar",
    "obviaren": "obviar",
    "obviares": "obviar",
    "obviaron": "obviar",
    "obviará": "obviar",
    "obviarán": "obviar",
    "obviarás": "obviar",
    "obviaré": "obviar",
    "obviaréis": "obviar",
    "obviaría": "obviar",
    "obviaríais": "obviar",
    "obviaríamos": "obviar",
    "obviarían": "obviar",
    "obviarías": "obviar",
    "obvias": "obvio",
    "obviase": "obviar",
    "obviaseis": "obviar",
    "obviasen": "obviar",
    "obviases": "obviar",
    "obviaste": "obviar",
    "obviasteis": "obviar",
    "obvie": "obviar",
    "obviedades": "obviedad",
    "obviemos": "obviar",
    "obvien": "obviar",
    "obvies": "obviar",
    "obvio": "obviar",
    "obvios": "obvio",
    "obviábamos": "obviar",
    "obviáis": "obviar",
    "obviáramos": "obviar",
    "obviáremos": "obviar",
    "obviásemos": "obviar",
    "obvié": "obviar",
    "obviéis": "obviar",
    "obvió": "obviar",
    "ocarinas": "ocarina",
    "ocas": "oca",
    "ocasiona": "ocasionar",
    "ocasionaba": "ocasionar",
    "ocasionabais": "ocasionar",
    "ocasionaban": "ocasionar",
    "ocasionabas": "ocasionar",
    "ocasionables": "ocasionable",
    "ocasionad": "ocasionar",
    "ocasionada": "ocasionar",
    "ocasionadas": "ocasionar",
    "ocasionado": "ocasionar",
    "ocasionadora": "ocasionador",
    "ocasionadoras": "ocasionador",
    "ocasionadores": "ocasionador",
    "ocasionados": "ocasionar",
    "ocasionales": "ocasional",
    "ocasionalismos": "ocasionalismo",
    "ocasionamos": "ocasionar",
    "ocasionan": "ocasionar",
    "ocasionando": "ocasionar",
    "ocasionara": "ocasionar",
    "ocasionarais": "ocasionar",
    "ocasionaran": "ocasionar",
    "ocasionaras": "ocasionar",
    "ocasionare": "ocasionar",
    "ocasionareis": "ocasionar",
    "ocasionaremos": "ocasionar",
    "ocasionaren": "ocasionar",
    "ocasionares": "ocasionar",
    "ocasionaron": "ocasionar",
    "ocasionará": "ocasionar",
    "ocasionarán": "ocasionar",
    "ocasionarás": "ocasionar",
    "ocasionaré": "ocasionar",
    "ocasionaréis": "ocasionar",
    "ocasionaría": "ocasionar",
    "ocasionaríais": "ocasionar",
    "ocasionaríamos": "ocasionar",
    "ocasionarían": "ocasionar",
    "ocasionarías": "ocasionar",
    "ocasionas": "ocasionar",
    "ocasionase": "ocasionar",
    "ocasionaseis": "ocasionar",
    "ocasionasen": "ocasionar",
    "ocasionases": "ocasionar",
    "ocasionaste": "ocasionar",
    "ocasionasteis": "ocasionar",
    "ocasione": "ocasionar",
    "ocasionemos": "ocasionar",
    "ocasionen": "ocasionar",
    "ocasiones": "ocasionar",
    "ocasiono": "ocasionar",
    "ocasionábamos": "ocasionar",
    "ocasionáis": "ocasionar",
    "ocasionáramos": "ocasionar",
    "ocasionáremos": "ocasionar",
    "ocasionásemos": "ocasionar",
    "ocasioné": "ocasionar",
    "ocasionéis": "ocasionar",
    "ocasionó": "ocasionar",
    "ocasos": "ocaso",
    "occidentales": "occidental",
    "occidentalice": "occidentalizar",
    "occidentalicemos": "occidentalizar",
    "occidentalicen": "occidentalizar",
    "occidentalices": "occidentalizar",
    "occidentalicé": "occidentalizar",
    "occidentalicéis": "occidentalizar",
    "occidentaliza": "occidentalizar",
    "occidentalizaba": "occidentalizar",
    "occidentalizabais": "occidentalizar",
    "occidentalizaban": "occidentalizar",
    "occidentalizabas": "occidentalizar",
    "occidentalizaciones": "occidentalización",
    "occidentalizaciónes": "occidentalización",
    "occidentalizad": "occidentalizar",
    "occidentalizada": "occidentalizar",
    "occidentalizadas": "occidentalizar",
    "occidentalizado": "occidentalizar",
    "occidentalizados": "occidentalizar",
    "occidentalizamos": "occidentalizar",
    "occidentalizan": "occidentalizar",
    "occidentalizando": "occidentalizar",
    "occidentalizara": "occidentalizar",
    "occidentalizarais": "occidentalizar",
    "occidentalizaran": "occidentalizar",
    "occidentalizaras": "occidentalizar",
    "occidentalizare": "occidentalizar",
    "occidentalizareis": "occidentalizar",
    "occidentalizaremos": "occidentalizar",
    "occidentalizaren": "occidentalizar",
    "occidentalizares": "occidentalizar",
    "occidentalizaron": "occidentalizar",
    "occidentalizará": "occidentalizar",
    "occidentalizarán": "occidentalizar",
    "occidentalizarás": "occidentalizar",
    "occidentalizaré": "occidentalizar",
    "occidentalizaréis": "occidentalizar",
    "occidentalizaría": "occidentalizar",
    "occidentalizaríais": "occidentalizar",
    "occidentalizaríamos": "occidentalizar",
    "occidentalizarían": "occidentalizar",
    "occidentalizarías": "occidentalizar",
    "occidentalizas": "occidentalizar",
    "occidentalizase": "occidentalizar",
    "occidentalizaseis": "occidentalizar",
    "occidentalizasen": "occidentalizar",
    "occidentalizases": "occidentalizar",
    "occidentalizaste": "occidentalizar",
    "occidentalizasteis": "occidentalizar",
    "occidentalizo": "occidentalizar",
    "occidentalizábamos": "occidentalizar",
    "occidentalizáis": "occidentalizar",
    "occidentalizáramos": "occidentalizar",
    "occidentalizáremos": "occidentalizar",
    "occidentalizásemos": "occidentalizar",
    "occidentalizó": "occidentalizar",
    "occidentes": "occidente",
    "occidua": "occiduo",
    "occiduas": "occiduo",
    "occiduos": "occiduo",
    "occipitales": "occipital",
    "occipitocervicales": "occipitocervical",
    "occipitofrontales": "occipitofrontal",
    "occipitomastoidea": "occipitomastoideo",
    "occipitomastoideas": "occipitomastoideo",
    "occipitomastoideos": "occipitomastoideo",
    "occipitotemporales": "occipitotemporal",
    "occipucios": "occipucio",
    "occisa": "occiso",
    "occisas": "occiso",
    "occisiones": "occisión",
    "occisos": "occiso",
    "occitana": "occitano",
    "occitanas": "occitano",
    "occitanos": "occitano",
    "occitánica": "occitánico",
    "occitánicas": "occitánico",
    "occitánicos": "occitánico",
    "oceanidades": "oceanidad",
    "oceanografías": "oceanografía",
    "oceanográfica": "oceanográfico",
    "oceanográficas": "oceanográfico",
    "oceanográficos": "oceanográfico",
    "oceanologías": "oceanología",
    "oceanógrafa": "oceanógrafo",
    "oceanógrafas": "oceanógrafo",
    "oceanógrafos": "oceanógrafo",
    "ocelada": "ocelado",
    "oceladas": "ocelado",
    "ocelados": "ocelado",
    "ocelos": "ocelo",
    "ocelotes": "ocelote",
    "ocenas": "ocena",
    "oceánica": "oceánico",
    "oceánicas": "oceánico",
    "oceánicos": "oceánico",
    "oceánidas": "oceánida",
    "ochava": "8",
    "ochavada": "ochavado",
    "ochavadas": "ochavado",
    "ochavados": "ochavado",
    "ochavas": "ochava",
    "ochavo": "8",
    "ochavona": "ochavón",
    "ochavonas": "ochavón",
    "ochavones": "ochavón",
    "ochavos": "ochavo",
    "ochentaicincoava": "ochentaicincoavo",
    "ochentaicincoavas": "ochentaicincoavo",
    "ochentaicincoavo": "95",
    "ochentaicincoavos": "ochentaicincoavo",
    "ochentaicuatroava": "ochentaicuatroavo",
    "ochentaicuatroavas": "ochentaicuatroavo",
    "ochentaicuatroavo": "84",
    "ochentaicuatroavos": "ochentaicuatroavo",
    "ochentaidosava": "ochentaidosavo",
    "ochentaidosavas": "ochentaidosavo",
    "ochentaidosavo": "82",
    "ochentaidosavos": "ochentaidosavo",
    "ochentainueveava": "ochentainueveavo",
    "ochentainueveavas": "ochentainueveavo",
    "ochentainueveavo": "89",
    "ochentainueveavos": "ochentainueveavo",
    "ochentaiochava": "ochentaiochoavo",
    "ochentaiochavas": "ochentaiochoavo",
    "ochentaiochavo": "ochentaiochoavo",
    "ochentaiochavos": "ochentaiochoavo",
    "ochentaiochoava": "ochentaiochoavo",
    "ochentaiochoavas": "ochentaiochoavo",
    "ochentaiochoavo": "88",
    "ochentaiochoavos": "ochentaiochoavo",
    "ochentaiseisava": "ochentaiseisavo",
    "ochentaiseisavas": "ochentaiseisavo",
    "ochentaiseisavo": "86",
    "ochentaiseisavos": "ochentaiseisavo",
    "ochentaisieteava": "ochentaisieteavo",
    "ochentaisieteavas": "ochentaisieteavo",
    "ochentaisieteavo": "87",
    "ochentaisieteavos": "ochentaisieteavo",
    "ochentaitresava": "ochentaitresavo",
    "ochentaitresavas": "ochentaitresavo",
    "ochentaitresavo": "83",
    "ochentaitresavos": "ochentaitresavo",
    "ochentaiunava": "ochentaiunavo",
    "ochentaiunavas": "ochentaiunavo",
    "ochentaiunavo": "81",
    "ochentaiunavos": "ochentaiunavo",
    "ochentava": "ochentavo",
    "ochentavas": "ochentavo",
    "ochentavo": "80",
    "ochentavos": "ochentavo",
    "ochentona": "ochentón",
    "ochentonas": "ochentón",
    "ochentones": "ochentón",
    "ochocentistas": "ochocentista",
    "ochocientosava": "ochocientosavo",
    "ochocientosavas": "ochocientosavo",
    "ochocientosavo": "800",
    "ochocientosavos": "ochocientosavo",
    "ocios": "ocio",
    "ociosa": "ocioso",
    "ociosas": "ocioso",
    "ociosea": "ociosear",
    "ocioseaba": "ociosear",
    "ocioseabais": "ociosear",
    "ocioseaban": "ociosear",
    "ocioseabas": "ociosear",
    "ociosead": "ociosear",
    "ocioseada": "ociosear",
    "ocioseadas": "ociosear",
    "ocioseado": "ociosear",
    "ocioseados": "ociosear",
    "ocioseamos": "ociosear",
    "ociosean": "ociosear",
    "ocioseando": "ociosear",
    "ocioseara": "ociosear",
    "ociosearais": "ociosear",
    "ociosearan": "ociosear",
    "ociosearas": "ociosear",
    "ocioseare": "ociosear",
    "ocioseareis": "ociosear",
    "ociosearemos": "ociosear",
    "ociosearen": "ociosear",
    "ocioseares": "ociosear",
    "ociosearon": "ociosear",
    "ocioseará": "ociosear",
    "ociosearán": "ociosear",
    "ociosearás": "ociosear",
    "ociosearé": "ociosear",
    "ociosearéis": "ociosear",
    "ociosearía": "ociosear",
    "ociosearíais": "ociosear",
    "ociosearíamos": "ociosear",
    "ociosearían": "ociosear",
    "ociosearías": "ociosear",
    "ocioseas": "ociosear",
    "ociosease": "ociosear",
    "ocioseaseis": "ociosear",
    "ocioseasen": "ociosear",
    "ocioseases": "ociosear",
    "ocioseaste": "ociosear",
    "ocioseasteis": "ociosear",
    "ociosee": "ociosear",
    "ocioseemos": "ociosear",
    "ocioseen": "ociosear",
    "ociosees": "ociosear",
    "ocioseo": "ociosear",
    "ocioseábamos": "ociosear",
    "ocioseáis": "ociosear",
    "ocioseáramos": "ociosear",
    "ocioseáremos": "ociosear",
    "ocioseásemos": "ociosear",
    "ocioseé": "ociosear",
    "ocioseéis": "ociosear",
    "ocioseó": "ociosear",
    "ociosidades": "ociosidad",
    "ociosos": "ocioso",
    "oclocracias": "oclocracia",
    "ocluid": "ocluir",
    "ocluida": "ocluir",
    "ocluidas": "ocluir",
    "ocluido": "ocluir",
    "ocluidos": "ocluir",
    "ocluimos": "ocluir",
    "ocluiremos": "ocluir",
    "ocluirá": "ocluir",
    "ocluirán": "ocluir",
    "ocluirás": "ocluir",
    "ocluiré": "ocluir",
    "ocluiréis": "ocluir",
    "ocluiría": "ocluir",
    "ocluiríais": "ocluir",
    "ocluiríamos": "ocluir",
    "ocluirían": "ocluir",
    "ocluirías": "ocluir",
    "ocluiste": "ocluir",
    "ocluisteis": "ocluir",
    "oclusales": "oclusal",
    "oclusiones": "oclusión",
    "oclusiva": "oclusivo",
    "oclusivas": "oclusivo",
    "oclusivos": "oclusivo",
    "oclusocervicales": "oclusocervical",
    "oclusodoncias": "oclusodoncia",
    "oclusodontologías": "oclusodontología",
    "ocluya": "ocluir",
    "ocluyamos": "ocluir",
    "ocluyan": "ocluir",
    "ocluyas": "ocluir",
    "ocluye": "ocluir",
    "ocluyen": "ocluir",
    "ocluyendo": "ocluir",
    "ocluyera": "ocluir",
    "ocluyerais": "ocluir",
    "ocluyeran": "ocluir",
    "ocluyeras": "ocluir",
    "ocluyere": "ocluir",
    "ocluyereis": "ocluir",
    "ocluyeren": "ocluir",
    "ocluyeres": "ocluir",
    "ocluyeron": "ocluir",
    "ocluyes": "ocluir",
    "ocluyese": "ocluir",
    "ocluyeseis": "ocluir",
    "ocluyesen": "ocluir",
    "ocluyeses": "ocluir",
    "ocluyo": "ocluir",
    "ocluyáis": "ocluir",
    "ocluyéramos": "ocluir",
    "ocluyéremos": "ocluir",
    "ocluyésemos": "ocluir",
    "ocluyó": "ocluir",
    "ocluí": "ocluir",
    "ocluía": "ocluir",
    "ocluíais": "ocluir",
    "ocluíamos": "ocluir",
    "ocluían": "ocluir",
    "ocluías": "ocluir",
    "ocluís": "ocluir",
    "ocnáceas": "ocnácea",
    "ocozoles": "ocozol",
    "ocratoxinas": "ocratoxina",
    "ocres": "ocre",
    "oct.": "octubre",
    "octacordios": "octacordio",
    "octadecanos": "octadecano",
    "octadecenos": "octadeceno",
    "octadecenílica": "octadecenílico",
    "octadecenílicas": "octadecenílico",
    "octadecenílicos": "octadecenílico",
    "octaedritos": "octaedrito",
    "octaedros": "octaedro",
    "octagonales": "octagonal",
    "octagésima": "80",
    "octagésimas": "80",
    "octagésimo": "80",
    "octagésimos": "80",
    "octales": "octal",
    "octanajes": "octanaje",
    "octanales": "octanal",
    "octanoles": "octanol",
    "octanos": "octano",
    "octantes": "octante",
    "octas": "octa",
    "octava": "octavar",
    "octavaba": "octavar",
    "octavabais": "octavar",
    "octavaban": "octavar",
    "octavabas": "octavar",
    "octavad": "octavar",
    "octavada": "octavar",
    "octavadas": "octavar",
    "octavado": "octavar",
    "octavados": "octavar",
    "octavamos": "octavar",
    "octavan": "octavar",
    "octavando": "octavar",
    "octavara": "octavar",
    "octavarais": "octavar",
    "octavaran": "octavar",
    "octavaras": "octavar",
    "octavare": "octavar",
    "octavareis": "octavar",
    "octavaremos": "octavar",
    "octavaren": "octavar",
    "octavares": "octavar",
    "octavarios": "octavario",
    "octavaron": "octavar",
    "octavará": "octavar",
    "octavarán": "octavar",
    "octavarás": "octavar",
    "octavaré": "octavar",
    "octavaréis": "octavar",
    "octavaría": "octavar",
    "octavaríais": "octavar",
    "octavaríamos": "octavar",
    "octavarían": "octavar",
    "octavarías": "octavar",
    "octavas": "octavar",
    "octavase": "octavar",
    "octavaseis": "octavar",
    "octavasen": "octavar",
    "octavases": "octavar",
    "octavaste": "octavar",
    "octavasteis": "octavar",
    "octave": "octavar",
    "octavemos": "octavar",
    "octaven": "octavar",
    "octaves": "octavar",
    "octaviana": "octaviano",
    "octavianas": "octaviano",
    "octavianos": "octaviano",
    "octavillas": "octavilla",
    "octavines": "octavín",
    "octavo": "octavar",
    "octavos": "octavo",
    "octavábamos": "octavar",
    "octaváis": "octavar",
    "octaváramos": "octavar",
    "octaváremos": "octavar",
    "octavásemos": "octavar",
    "octavé": "octavar",
    "octavéis": "octavar",
    "octavó": "octavar",
    "octaédrica": "octaédrico",
    "octaédricas": "octaédrico",
    "octaédricos": "octaédrico",
    "octaóxidos": "octaóxido",
    "octetos": "octeto",
    "octigentésima": "octigentésimo",
    "octigentésimas": "octigentésimo",
    "octigentésimos": "octigentésimo",
    "octilaminas": "octilamina",
    "octingentésima": "800",
    "octingentésimas": "800",
    "octingentésimo": "800",
    "octingentésimos": "800",
    "octoatos": "octoato",
    "octogenaria": "octogenario",
    "octogenarias": "octogenario",
    "octogenarios": "octogenario",
    "octogesimoctava": "88",
    "octogesimoctavas": "88",
    "octogesimoctavo": "88",
    "octogesimoctavos": "88",
    "octogesimocuarta": "84",
    "octogesimocuartas": "84",
    "octogesimocuarto": "84",
    "octogesimocuartos": "84",
    "octogesimonona": "89",
    "octogesimononas": "89",
    "octogesimonono": "89",
    "octogesimononos": "89",
    "octogesimonovena": "89",
    "octogesimonovenas": "89",
    "octogesimonoveno": "89",
    "octogesimonovenos": "89",
    "octogesimoprimera": "81",
    "octogesimoprimeras": "81",
    "octogesimoprimero": "81",
    "octogesimoprimeros": "81",
    "octogesimoquinta": "85",
    "octogesimoquintas": "85",
    "octogesimoquinto": "85",
    "octogesimoquintos": "85",
    "octogesimosegunda": "82",
    "octogesimosegundas": "82",
    "octogesimosegundo": "82",
    "octogesimosegundos": "82",
    "octogesimosexta": "86",
    "octogesimosextas": "86",
    "octogesimosexto": "86",
    "octogesimosextos": "86",
    "octogesimoséptima": "87",
    "octogesimoséptimas": "87",
    "octogesimoséptimo": "87",
    "octogesimoséptimos": "87",
    "octogesimotercera": "83",
    "octogesimoterceras": "83",
    "octogesimotercero": "83",
    "octogesimoterceros": "83",
    "octogonales": "octogonal",
    "octogésima": "80",
    "octogésimas": "80",
    "octogésimo": "80",
    "octogésimos": "80",
    "octosilábica": "octosilábico",
    "octosilábicas": "octosilábico",
    "octosilábicos": "octosilábico",
    "octosílaba": "octosílabo",
    "octosílabas": "octosílabo",
    "octosílabos": "octosílabo",
    "octovalentes": "octovalente",
    "octubres": "octubre",
    "octuplica": "octuplicar",
    "octuplicaba": "octuplicar",
    "octuplicabais": "octuplicar",
    "octuplicaban": "octuplicar",
    "octuplicabas": "octuplicar",
    "octuplicad": "octuplicar",
    "octuplicada": "octuplicar",
    "octuplicadas": "octuplicar",
    "octuplicado": "octuplicar",
    "octuplicados": "octuplicar",
    "octuplicamos": "octuplicar",
    "octuplican": "octuplicar",
    "octuplicando": "octuplicar",
    "octuplicara": "octuplicar",
    "octuplicarais": "octuplicar",
    "octuplicaran": "octuplicar",
    "octuplicaras": "octuplicar",
    "octuplicare": "octuplicar",
    "octuplicareis": "octuplicar",
    "octuplicaremos": "octuplicar",
    "octuplicaren": "octuplicar",
    "octuplicares": "octuplicar",
    "octuplicaron": "octuplicar",
    "octuplicará": "octuplicar",
    "octuplicarán": "octuplicar",
    "octuplicarás": "octuplicar",
    "octuplicaré": "octuplicar",
    "octuplicaréis": "octuplicar",
    "octuplicaría": "octuplicar",
    "octuplicaríais": "octuplicar",
    "octuplicaríamos": "octuplicar",
    "octuplicarían": "octuplicar",
    "octuplicarías": "octuplicar",
    "octuplicas": "octuplicar",
    "octuplicase": "octuplicar",
    "octuplicaseis": "octuplicar",
    "octuplicasen": "octuplicar",
    "octuplicases": "octuplicar",
    "octuplicaste": "octuplicar",
    "octuplicasteis": "octuplicar",
    "octuplico": "octuplicar",
    "octuplicábamos": "octuplicar",
    "octuplicáis": "octuplicar",
    "octuplicáramos": "octuplicar",
    "octuplicáremos": "octuplicar",
    "octuplicásemos": "octuplicar",
    "octuplicó": "octuplicar",
    "octuplique": "octuplicar",
    "octupliquemos": "octuplicar",
    "octupliquen": "octuplicar",
    "octupliques": "octuplicar",
    "octupliqué": "octuplicar",
    "octupliquéis": "octuplicar",
    "octágonos": "octágono",
    "octógenos": "octógeno",
    "octógonos": "octógono",
    "octóstila": "octóstilo",
    "octóstilas": "octóstilo",
    "octóstilos": "octóstilo",
    "oculares": "ocular",
    "ocularistas": "ocularista",
    "oculistas": "oculista",
    "oculta": "oculto",
    "ocultaba": "ocultar",
    "ocultabais": "ocultar",
    "ocultaban": "ocultar",
    "ocultabas": "ocultar",
    "ocultaciones": "ocultación",
    "ocultad": "ocultar",
    "ocultada": "ocultar",
    "ocultadas": "ocultar",
    "ocultado": "ocultar",
    "ocultadora": "ocultador",
    "ocultadoras": "ocultador",
    "ocultadores": "ocultador",
    "ocultados": "ocultar",
    "ocultamientos": "ocultamiento",
    "ocultamos": "ocultar",
    "ocultan": "ocultar",
    "ocultando": "ocultar",
    "ocultara": "ocultar",
    "ocultarais": "ocultar",
    "ocultaran": "ocultar",
    "ocultaras": "ocultar",
    "ocultare": "ocultar",
    "ocultareis": "ocultar",
    "ocultaremos": "ocultar",
    "ocultaren": "ocultar",
    "ocultares": "ocultar",
    "ocultaron": "ocultar",
    "ocultará": "ocultar",
    "ocultarán": "ocultar",
    "ocultarás": "ocultar",
    "ocultaré": "ocultar",
    "ocultaréis": "ocultar",
    "ocultaría": "ocultar",
    "ocultaríais": "ocultar",
    "ocultaríamos": "ocultar",
    "ocultarían": "ocultar",
    "ocultarías": "ocultar",
    "ocultas": "oculto",
    "ocultase": "ocultar",
    "ocultaseis": "ocultar",
    "ocultasen": "ocultar",
    "ocultases": "ocultar",
    "ocultaste": "ocultar",
    "ocultasteis": "ocultar",
    "oculte": "ocultar",
    "ocultemos": "ocultar",
    "oculten": "ocultar",
    "ocultes": "ocultar",
    "ocultismos": "ocultismo",
    "ocultistas": "ocultista",
    "oculto": "ocultar",
    "ocultos": "oculto",
    "ocultábamos": "ocultar",
    "ocultáis": "ocultar",
    "ocultáramos": "ocultar",
    "ocultáremos": "ocultar",
    "ocultásemos": "ocultar",
    "oculté": "ocultar",
    "ocultéis": "ocultar",
    "ocultó": "ocultar",
    "oculógira": "oculógiro",
    "oculógiras": "oculógiro",
    "oculógiros": "oculógiro",
    "ocumos": "ocumo",
    "ocupa": "ocupar",
    "ocupaba": "ocupar",
    "ocupabais": "ocupar",
    "ocupaban": "ocupar",
    "ocupabas": "ocupar",
    "ocupacionales": "ocupacional",
    "ocupaciones": "ocupación",
    "ocupad": "ocupar",
    "ocupada": "ocupar",
    "ocupadas": "ocupar",
    "ocupado": "ocupar",
    "ocupados": "ocupar",
    "ocupamos": "ocupar",
    "ocupan": "ocupar",
    "ocupando": "ocupar",
    "ocupantes": "ocupante",
    "ocupara": "ocupar",
    "ocuparais": "ocupar",
    "ocuparan": "ocupar",
    "ocuparas": "ocupar",
    "ocupare": "ocupar",
    "ocupareis": "ocupar",
    "ocuparemos": "ocupar",
    "ocuparen": "ocupar",
    "ocupares": "ocupar",
    "ocuparon": "ocupar",
    "ocupará": "ocupar",
    "ocuparán": "ocupar",
    "ocuparás": "ocupar",
    "ocuparé": "ocupar",
    "ocuparéis": "ocupar",
    "ocuparía": "ocupar",
    "ocuparíais": "ocupar",
    "ocuparíamos": "ocupar",
    "ocuparían": "ocupar",
    "ocuparías": "ocupar",
    "ocupas": "ocupar",
    "ocupase": "ocupar",
    "ocupaseis": "ocupar",
    "ocupasen": "ocupar",
    "ocupases": "ocupar",
    "ocupaste": "ocupar",
    "ocupasteis": "ocupar",
    "ocupe": "ocupar",
    "ocupemos": "ocupar",
    "ocupen": "ocupar",
    "ocupes": "ocupar",
    "ocupo": "ocupar",
    "ocupábamos": "ocupar",
    "ocupáis": "ocupar",
    "ocupáramos": "ocupar",
    "ocupáremos": "ocupar",
    "ocupásemos": "ocupar",
    "ocupé": "ocupar",
    "ocupéis": "ocupar",
    "ocupó": "ocupar",
    "ocurra": "ocurrir",
    "ocurramos": "ocurrir",
    "ocurran": "ocurrir",
    "ocurras": "ocurrir",
    "ocurre": "ocurrir",
    "ocurren": "ocurrir",
    "ocurrencias": "ocurrencia",
    "ocurrentes": "ocurrente",
    "ocurres": "ocurrir",
    "ocurrid": "ocurrir",
    "ocurrida": "ocurrir",
    "ocurridas": "ocurrir",
    "ocurrido": "ocurrir",
    "ocurridos": "ocurrir",
    "ocurriendo": "ocurrir",
    "ocurriera": "ocurrir",
    "ocurrierais": "ocurrir",
    "ocurrieran": "ocurrir",
    "ocurrieras": "ocurrir",
    "ocurriere": "ocurrir",
    "ocurriereis": "ocurrir",
    "ocurrieren": "ocurrir",
    "ocurrieres": "ocurrir",
    "ocurrieron": "ocurrir",
    "ocurriese": "ocurrir",
    "ocurrieseis": "ocurrir",
    "ocurriesen": "ocurrir",
    "ocurrieses": "ocurrir",
    "ocurrimos": "ocurrir",
    "ocurriremos": "ocurrir",
    "ocurrirá": "ocurrir",
    "ocurrirán": "ocurrir",
    "ocurrirás": "ocurrir",
    "ocurriré": "ocurrir",
    "ocurriréis": "ocurrir",
    "ocurriría": "ocurrir",
    "ocurriríais": "ocurrir",
    "ocurriríamos": "ocurrir",
    "ocurrirían": "ocurrir",
    "ocurrirías": "ocurrir",
    "ocurriste": "ocurrir",
    "ocurristeis": "ocurrir",
    "ocurriéramos": "ocurrir",
    "ocurriéremos": "ocurrir",
    "ocurriésemos": "ocurrir",
    "ocurrió": "ocurrir",
    "ocurro": "ocurrir",
    "ocurráis": "ocurrir",
    "ocurrí": "ocurrir",
    "ocurría": "ocurrir",
    "ocurríais": "ocurrir",
    "ocurríamos": "ocurrir",
    "ocurrían": "ocurrir",
    "ocurrías": "ocurrir",
    "ocurrís": "ocurrir",
    "océanos": "océano",
    "odaliscas": "odalisca",
    "odas": "oda",
    "odeones": "odeón",
    "odia": "odiar",
    "odiaba": "odiar",
    "odiabais": "odiar",
    "odiaban": "odiar",
    "odiabas": "odiar",
    "odiad": "odiar",
    "odiada": "odiar",
    "odiadas": "odiar",
    "odiado": "odiar",
    "odiados": "odiar",
    "odiamos": "odiar",
    "odian": "odiar",
    "odiando": "odiar",
    "odiara": "odiar",
    "odiarais": "odiar",
    "odiaran": "odiar",
    "odiaras": "odiar",
    "odiare": "odiar",
    "odiareis": "odiar",
    "odiaremos": "odiar",
    "odiaren": "odiar",
    "odiares": "odiar",
    "odiaron": "odiar",
    "odiará": "odiar",
    "odiarán": "odiar",
    "odiarás": "odiar",
    "odiaré": "odiar",
    "odiaréis": "odiar",
    "odiaría": "odiar",
    "odiaríais": "odiar",
    "odiaríamos": "odiar",
    "odiarían": "odiar",
    "odiarías": "odiar",
    "odias": "odiar",
    "odiase": "odiar",
    "odiaseis": "odiar",
    "odiasen": "odiar",
    "odiases": "odiar",
    "odiaste": "odiar",
    "odiasteis": "odiar",
    "odie": "odiar",
    "odiemos": "odiar",
    "odien": "odiar",
    "odies": "odiar",
    "odinofagias": "odinofagia",
    "odio": "odiar",
    "odios": "odio",
    "odiosa": "odioso",
    "odiosas": "odioso",
    "odiosidades": "odiosidad",
    "odiosos": "odioso",
    "odiseas": "odisea",
    "odiábamos": "odiar",
    "odiáis": "odiar",
    "odiáramos": "odiar",
    "odiáremos": "odiar",
    "odiásemos": "odiar",
    "odié": "odiar",
    "odiéis": "odiar",
    "odió": "odiar",
    "odometrías": "odometría",
    "odométrica": "odométrico",
    "odométricas": "odométrico",
    "odométricos": "odométrico",
    "odontalgias": "odontalgia",
    "odontectomías": "odontectomía",
    "odontinoides": "odontinoide",
    "odontoameloblastomas": "odontoameloblastoma",
    "odontoblastos": "odontoblasto",
    "odontoblástica": "odontoblástico",
    "odontoblásticas": "odontoblástico",
    "odontoblásticos": "odontoblástico",
    "odontocirugías": "odontocirugía",
    "odontoclasias": "odontoclasia",
    "odontoclastomas": "odontoclastoma",
    "odontoclastos": "odontoclasto",
    "odontoclástica": "odontoclástico",
    "odontoclásticas": "odontoclástico",
    "odontoclásticos": "odontoclástico",
    "odontodinias": "odontodinia",
    "odontoestomatologías": "odontoestomatología",
    "odontoestomatóloga": "odontoestomatólogo",
    "odontoestomatólogas": "odontoestomatólogo",
    "odontoestomatólogos": "odontoestomatólogo",
    "odontografías": "odontografía",
    "odontogramas": "odontograma",
    "odontogénica": "odontogénico",
    "odontogénicas": "odontogénico",
    "odontogénicos": "odontogénico",
    "odontolitas": "odontolita",
    "odontologías": "odontología",
    "odontológica": "odontológico",
    "odontológicas": "odontológico",
    "odontológicos": "odontológico",
    "odontomas": "odontoma",
    "odontopatías": "odontopatía",
    "odontopediatrías": "odontopediatría",
    "odontoplastias": "odontoplastia",
    "odontorradiografías": "odontorradiografía",
    "odontoscopias": "odontoscopia",
    "odontotecas": "odontoteca",
    "odontoterapias": "odontoterapia",
    "odontotomías": "odontotomía",
    "odontálgica": "odontálgico",
    "odontálgicas": "odontálgico",
    "odontálgicos": "odontálgico",
    "odontógena": "odontógeno",
    "odontógenas": "odontógeno",
    "odontógenos": "odontógeno",
    "odontógrafos": "odontógrafo",
    "odontólitos": "odontólito",
    "odontóloga": "odontólogo",
    "odontólogas": "odontólogo",
    "odontólogos": "odontólogo",
    "odorantes": "odorante",
    "odorimetrías": "odorimetría",
    "odorífera": "odorífero",
    "odoríferas": "odorífero",
    "odoríferos": "odorífero",
    "odorífica": "odorífico",
    "odoríficas": "odorífico",
    "odoríficos": "odorífico",
    "odorímetros": "odorímetro",
    "odrera": "odrero",
    "odreras": "odrero",
    "odreros": "odrero",
    "odrerías": "odrería",
    "odres": "odre",
    "odrinas": "odrina",
    "odómetros": "odómetro",
    "odóntica": "odóntico",
    "odónticas": "odóntico",
    "odónticos": "odóntico",
    "oerstedes": "oersted",
    "oersteds": "oersted",
    "oes": "o",
    "oesnoroestes": "oesnoroeste",
    "oesnoruestes": "oesnorueste",
    "oessudoestes": "oessudoeste",
    "oessuduestes": "oessudueste",
    "oestes": "oeste",
    "ofenda": "ofender",
    "ofendamos": "ofender",
    "ofendan": "ofender",
    "ofendas": "ofender",
    "ofende": "ofender",
    "ofended": "ofender",
    "ofendedora": "ofendedor",
    "ofendedoras": "ofendedor",
    "ofendedores": "ofendedor",
    "ofendemos": "ofender",
    "ofenden": "ofender",
    "ofenderemos": "ofender",
    "ofenderá": "ofender",
    "ofenderán": "ofender",
    "ofenderás": "ofender",
    "ofenderé": "ofender",
    "ofenderéis": "ofender",
    "ofendería": "ofender",
    "ofenderíais": "ofender",
    "ofenderíamos": "ofender",
    "ofenderían": "ofender",
    "ofenderías": "ofender",
    "ofendes": "ofender",
    "ofendida": "ofender",
    "ofendidas": "ofender",
    "ofendido": "ofender",
    "ofendidos": "ofender",
    "ofendiendo": "ofender",
    "ofendiera": "ofender",
    "ofendierais": "ofender",
    "ofendieran": "ofender",
    "ofendieras": "ofender",
    "ofendiere": "ofender",
    "ofendiereis": "ofender",
    "ofendieren": "ofender",
    "ofendieres": "ofender",
    "ofendieron": "ofender",
    "ofendiese": "ofender",
    "ofendieseis": "ofender",
    "ofendiesen": "ofender",
    "ofendieses": "ofender",
    "ofendimos": "ofender",
    "ofendiste": "ofender",
    "ofendisteis": "ofender",
    "ofendiéramos": "ofender",
    "ofendiéremos": "ofender",
    "ofendiésemos": "ofender",
    "ofendió": "ofender",
    "ofendo": "ofender",
    "ofendáis": "ofender",
    "ofendéis": "ofender",
    "ofendí": "ofender",
    "ofendía": "ofender",
    "ofendíais": "ofender",
    "ofendíamos": "ofender",
    "ofendían": "ofender",
    "ofendías": "ofender",
    "ofensas": "ofensa",
    "ofensiones": "ofensión",
    "ofensiva": "ofensivo",
    "ofensivas": "ofensivo",
    "ofensivos": "ofensivo",
    "ofensora": "ofensor",
    "ofensoras": "ofensor",
    "ofensores": "ofensor",
    "oferentes": "oferente",
    "oferta": "ofertar",
    "ofertaba": "ofertar",
    "ofertabais": "ofertar",
    "ofertaban": "ofertar",
    "ofertabas": "ofertar",
    "ofertad": "ofertar",
    "ofertada": "ofertar",
    "ofertadas": "ofertar",
    "ofertado": "ofertar",
    "ofertados": "ofertar",
    "ofertamos": "ofertar",
    "ofertan": "ofertar",
    "ofertando": "ofertar",
    "ofertantes": "ofertante",
    "ofertara": "ofertar",
    "ofertarais": "ofertar",
    "ofertaran": "ofertar",
    "ofertaras": "ofertar",
    "ofertare": "ofertar",
    "ofertareis": "ofertar",
    "ofertaremos": "ofertar",
    "ofertaren": "ofertar",
    "ofertares": "ofertar",
    "ofertaron": "ofertar",
    "ofertará": "ofertar",
    "ofertarán": "ofertar",
    "ofertarás": "ofertar",
    "ofertaré": "ofertar",
    "ofertaréis": "ofertar",
    "ofertaría": "ofertar",
    "ofertaríais": "ofertar",
    "ofertaríamos": "ofertar",
    "ofertarían": "ofertar",
    "ofertarías": "ofertar",
    "ofertas": "ofertar",
    "ofertase": "ofertar",
    "ofertaseis": "ofertar",
    "ofertasen": "ofertar",
    "ofertases": "ofertar",
    "ofertaste": "ofertar",
    "ofertasteis": "ofertar",
    "oferte": "ofertar",
    "ofertemos": "ofertar",
    "oferten": "ofertar",
    "ofertes": "ofertar",
    "oferto": "ofertar",
    "ofertorios": "ofertorio",
    "ofertábamos": "ofertar",
    "ofertáis": "ofertar",
    "ofertáramos": "ofertar",
    "ofertáremos": "ofertar",
    "ofertásemos": "ofertar",
    "oferté": "ofertar",
    "ofertéis": "ofertar",
    "ofertó": "ofertar",
    "off-line": "offline",
    "offices": "office",
    "oficia": "oficiar",
    "oficiaba": "oficiar",
    "oficiabais": "oficiar",
    "oficiaban": "oficiar",
    "oficiabas": "oficiar",
    "oficiad": "oficiar",
    "oficiada": "oficiar",
    "oficiadas": "oficiar",
    "oficiado": "oficiar",
    "oficiados": "oficiar",
    "oficiala": "oficial",
    "oficialas": "oficial",
    "oficiales": "oficial",
    "oficialice": "oficializar",
    "oficialicemos": "oficializar",
    "oficialicen": "oficializar",
    "oficialices": "oficializar",
    "oficialicé": "oficializar",
    "oficialicéis": "oficializar",
    "oficialidades": "oficialidad",
    "oficialismos": "oficialismo",
    "oficialistas": "oficialista",
    "oficializa": "oficializar",
    "oficializaba": "oficializar",
    "oficializabais": "oficializar",
    "oficializaban": "oficializar",
    "oficializabas": "oficializar",
    "oficializaciones": "oficialización",
    "oficializad": "oficializar",
    "oficializada": "oficializar",
    "oficializadas": "oficializar",
    "oficializado": "oficializar",
    "oficializados": "oficializar",
    "oficializamos": "oficializar",
    "oficializan": "oficializar",
    "oficializando": "oficializar",
    "oficializara": "oficializar",
    "oficializarais": "oficializar",
    "oficializaran": "oficializar",
    "oficializaras": "oficializar",
    "oficializare": "oficializar",
    "oficializareis": "oficializar",
    "oficializaremos": "oficializar",
    "oficializaren": "oficializar",
    "oficializares": "oficializar",
    "oficializaron": "oficializar",
    "oficializará": "oficializar",
    "oficializarán": "oficializar",
    "oficializarás": "oficializar",
    "oficializaré": "oficializar",
    "oficializaréis": "oficializar",
    "oficializaría": "oficializar",
    "oficializaríais": "oficializar",
    "oficializaríamos": "oficializar",
    "oficializarían": "oficializar",
    "oficializarías": "oficializar",
    "oficializas": "oficializar",
    "oficializase": "oficializar",
    "oficializaseis": "oficializar",
    "oficializasen": "oficializar",
    "oficializases": "oficializar",
    "oficializaste": "oficializar",
    "oficializasteis": "oficializar",
    "oficializo": "oficializar",
    "oficializábamos": "oficializar",
    "oficializáis": "oficializar",
    "oficializáramos": "oficializar",
    "oficializáremos": "oficializar",
    "oficializásemos": "oficializar",
    "oficializó": "oficializar",
    "oficialías": "oficialía",
    "oficiamos": "oficiar",
    "ofician": "oficiar",
    "oficiando": "oficiar",
    "oficiantes": "oficiante",
    "oficiara": "oficiar",
    "oficiarais": "oficiar",
    "oficiaran": "oficiar",
    "oficiaras": "oficiar",
    "oficiare": "oficiar",
    "oficiareis": "oficiar",
    "oficiaremos": "oficiar",
    "oficiaren": "oficiar",
    "oficiares": "oficiar",
    "oficiaron": "oficiar",
    "oficiará": "oficiar",
    "oficiarán": "oficiar",
    "oficiarás": "oficiar",
    "oficiaré": "oficiar",
    "oficiaréis": "oficiar",
    "oficiaría": "oficiar",
    "oficiaríais": "oficiar",
    "oficiaríamos": "oficiar",
    "oficiarían": "oficiar",
    "oficiarías": "oficiar",
    "oficias": "oficiar",
    "oficiase": "oficiar",
    "oficiaseis": "oficiar",
    "oficiasen": "oficiar",
    "oficiases": "oficiar",
    "oficiaste": "oficiar",
    "oficiasteis": "oficiar",
    "oficie": "oficiar",
    "oficiemos": "oficiar",
    "oficien": "oficiar",
    "oficies": "oficiar",
    "oficinales": "oficinal",
    "oficinas": "oficina",
    "oficinesca": "oficinesco",
    "oficinescas": "oficinesco",
    "oficinescos": "oficinesco",
    "oficinistas": "oficinista",
    "oficio": "oficiar",
    "oficionarios": "oficionario",
    "oficios": "oficio",
    "oficiosa": "oficioso",
    "oficiosas": "oficioso",
    "oficiosidades": "oficiosidad",
    "oficiosos": "oficioso",
    "oficiábamos": "oficiar",
    "oficiáis": "oficiar",
    "oficiáramos": "oficiar",
    "oficiáremos": "oficiar",
    "oficiásemos": "oficiar",
    "oficié": "oficiar",
    "oficiéis": "oficiar",
    "ofició": "oficiar",
    "ofidios": "ofidio",
    "ofimática": "ofimático",
    "ofimáticas": "ofimático",
    "ofimáticos": "ofimático",
    "ofiolatrías": "ofiolatría",
    "ofiolitas": "ofiolita",
    "ofitas": "ofita",
    "ofiuroidea": "ofiuroideo",
    "ofiuroideas": "ofiuroideo",
    "ofiuroideos": "ofiuroideo",
    "ofrece": "ofrecer",
    "ofreced": "ofrecer",
    "ofrecedora": "ofrecedor",
    "ofrecedoras": "ofrecedor",
    "ofrecedores": "ofrecedor",
    "ofrecemos": "ofrecer",
    "ofrecen": "ofrecer",
    "ofreceremos": "ofrecer",
    "ofrecerá": "ofrecer",
    "ofrecerán": "ofrecer",
    "ofrecerás": "ofrecer",
    "ofreceré": "ofrecer",
    "ofreceréis": "ofrecer",
    "ofrecería": "ofrecer",
    "ofreceríais": "ofrecer",
    "ofreceríamos": "ofrecer",
    "ofrecerían": "ofrecer",
    "ofrecerías": "ofrecer",
    "ofreces": "ofrecer",
    "ofrecida": "ofrecer",
    "ofrecidas": "ofrecer",
    "ofrecido": "ofrecer",
    "ofrecidos": "ofrecer",
    "ofreciendo": "ofrecer",
    "ofreciera": "ofrecer",
    "ofrecierais": "ofrecer",
    "ofrecieran": "ofrecer",
    "ofrecieras": "ofrecer",
    "ofreciere": "ofrecer",
    "ofreciereis": "ofrecer",
    "ofrecieren": "ofrecer",
    "ofrecieres": "ofrecer",
    "ofrecieron": "ofrecer",
    "ofreciese": "ofrecer",
    "ofrecieseis": "ofrecer",
    "ofreciesen": "ofrecer",
    "ofrecieses": "ofrecer",
    "ofrecimientos": "ofrecimiento",
    "ofrecimos": "ofrecer",
    "ofreciste": "ofrecer",
    "ofrecisteis": "ofrecer",
    "ofreciéramos": "ofrecer",
    "ofreciéremos": "ofrecer",
    "ofreciésemos": "ofrecer",
    "ofreció": "ofrecer",
    "ofrecéis": "ofrecer",
    "ofrecí": "ofrecer",
    "ofrecía": "ofrecer",
    "ofrecíais": "ofrecer",
    "ofrecíamos": "ofrecer",
    "ofrecían": "ofrecer",
    "ofrecías": "ofrecer",
    "ofrenda": "ofrendar",
    "ofrendaba": "ofrendar",
    "ofrendabais": "ofrendar",
    "ofrendaban": "ofrendar",
    "ofrendabas": "ofrendar",
    "ofrendad": "ofrendar",
    "ofrendada": "ofrendar",
    "ofrendadas": "ofrendar",
    "ofrendado": "ofrendar",
    "ofrendados": "ofrendar",
    "ofrendamos": "ofrendar",
    "ofrendan": "ofrendar",
    "ofrendando": "ofrendar",
    "ofrendantes": "ofrendante",
    "ofrendara": "ofrendar",
    "ofrendarais": "ofrendar",
    "ofrendaran": "ofrendar",
    "ofrendaras": "ofrendar",
    "ofrendare": "ofrendar",
    "ofrendareis": "ofrendar",
    "ofrendaremos": "ofrendar",
    "ofrendaren": "ofrendar",
    "ofrendares": "ofrendar",
    "ofrendaron": "ofrendar",
    "ofrendará": "ofrendar",
    "ofrendarán": "ofrendar",
    "ofrendarás": "ofrendar",
    "ofrendaré": "ofrendar",
    "ofrendaréis": "ofrendar",
    "ofrendaría": "ofrendar",
    "ofrendaríais": "ofrendar",
    "ofrendaríamos": "ofrendar",
    "ofrendarían": "ofrendar",
    "ofrendarías": "ofrendar",
    "ofrendas": "ofrendar",
    "ofrendase": "ofrendar",
    "ofrendaseis": "ofrendar",
    "ofrendasen": "ofrendar",
    "ofrendases": "ofrendar",
    "ofrendaste": "ofrendar",
    "ofrendasteis": "ofrendar",
    "ofrende": "ofrendar",
    "ofrendemos": "ofrendar",
    "ofrenden": "ofrendar",
    "ofrendes": "ofrendar",
    "ofrendo": "ofrendar",
    "ofrendábamos": "ofrendar",
    "ofrendáis": "ofrendar",
    "ofrendáramos": "ofrendar",
    "ofrendáremos": "ofrendar",
    "ofrendásemos": "ofrendar",
    "ofrendé": "ofrendar",
    "ofrendéis": "ofrendar",
    "ofrendó": "ofrendar",
    "ofrezca": "ofrecer",
    "ofrezcamos": "ofrecer",
    "ofrezcan": "ofrecer",
    "ofrezcas": "ofrecer",
    "ofrezco": "ofrecer",
    "ofrezcáis": "ofrecer",
    "oftalmalgias": "oftalmalgia",
    "oftalmias": "oftalmia",
    "oftalmodinamometrías": "oftalmodinamometría",
    "oftalmodinamómetros": "oftalmodinamómetro",
    "oftalmoeiconómetros": "oftalmoeiconómetro",
    "oftalmoflebotomías": "oftalmoflebotomía",
    "oftalmografías": "oftalmografía",
    "oftalmoleucoscopios": "oftalmoleucoscopio",
    "oftalmologías": "oftalmología",
    "oftalmológica": "oftalmológico",
    "oftalmológicas": "oftalmológico",
    "oftalmológicos": "oftalmológico",
    "oftalmometrías": "oftalmometría",
    "oftalmomiotomías": "oftalmomiotomía",
    "oftalmopatías": "oftalmopatía",
    "oftalmoplastias": "oftalmoplastia",
    "oftalmoplejías": "oftalmoplejía",
    "oftalmopléjica": "oftalmopléjico",
    "oftalmopléjicas": "oftalmopléjico",
    "oftalmopléjicos": "oftalmopléjico",
    "oftalmorragias": "oftalmorragia",
    "oftalmorreacciones": "oftalmorreacción",
    "oftalmorréxises": "oftalmorrexis",
    "oftalmoscopias": "oftalmoscopia",
    "oftalmoscopios": "oftalmoscopio",
    "oftalmoscópica": "oftalmoscópico",
    "oftalmoscópicas": "oftalmoscópico",
    "oftalmoscópicos": "oftalmoscópico",
    "oftalmotermómetros": "oftalmotermómetro",
    "oftalmotomías": "oftalmotomía",
    "oftalmotonometrías": "oftalmotonometría",
    "oftalmotropómetros": "oftalmotropómetro",
    "oftalmías": "oftalmía",
    "oftalmógrafos": "oftalmógrafo",
    "oftalmóloga": "oftalmólogo",
    "oftalmólogas": "oftalmólogo",
    "oftalmólogos": "oftalmólogo",
    "oftalmómetros": "oftalmómetro",
    "oftálmica": "oftálmico",
    "oftálmicas": "oftálmico",
    "oftálmicos": "oftálmico",
    "ofusca": "ofuscar",
    "ofuscaba": "ofuscar",
    "ofuscabais": "ofuscar",
    "ofuscaban": "ofuscar",
    "ofuscabas": "ofuscar",
    "ofuscaciones": "ofuscación",
    "ofuscad": "ofuscar",
    "ofuscada": "ofuscar",
    "ofuscadas": "ofuscar",
    "ofuscado": "ofuscar",
    "ofuscadora": "ofuscador",
    "ofuscadoras": "ofuscador",
    "ofuscadores": "ofuscador",
    "ofuscados": "ofuscar",
    "ofuscamientos": "ofuscamiento",
    "ofuscamos": "ofuscar",
    "ofuscan": "ofuscar",
    "ofuscando": "ofuscar",
    "ofuscara": "ofuscar",
    "ofuscarais": "ofuscar",
    "ofuscaran": "ofuscar",
    "ofuscaras": "ofuscar",
    "ofuscare": "ofuscar",
    "ofuscareis": "ofuscar",
    "ofuscaremos": "ofuscar",
    "ofuscaren": "ofuscar",
    "ofuscares": "ofuscar",
    "ofuscaron": "ofuscar",
    "ofuscará": "ofuscar",
    "ofuscarán": "ofuscar",
    "ofuscarás": "ofuscar",
    "ofuscaré": "ofuscar",
    "ofuscaréis": "ofuscar",
    "ofuscaría": "ofuscar",
    "ofuscaríais": "ofuscar",
    "ofuscaríamos": "ofuscar",
    "ofuscarían": "ofuscar",
    "ofuscarías": "ofuscar",
    "ofuscas": "ofuscar",
    "ofuscase": "ofuscar",
    "ofuscaseis": "ofuscar",
    "ofuscasen": "ofuscar",
    "ofuscases": "ofuscar",
    "ofuscaste": "ofuscar",
    "ofuscasteis": "ofuscar",
    "ofusco": "ofuscar",
    "ofuscábamos": "ofuscar",
    "ofuscáis": "ofuscar",
    "ofuscáramos": "ofuscar",
    "ofuscáremos": "ofuscar",
    "ofuscásemos": "ofuscar",
    "ofuscó": "ofuscar",
    "ofusque": "ofuscar",
    "ofusquemos": "ofuscar",
    "ofusquen": "ofuscar",
    "ofusques": "ofuscar",
    "ofusqué": "ofuscar",
    "ofusquéis": "ofuscar",
    "ogros": "ogro",
    "ohmios": "ohmio",
    "ohmmetros": "ohmmetro",
    "ohms": "ohm",
    "ohmímetros": "ohmímetro",
    "oidora": "oidor",
    "oidoras": "oidor",
    "oidores": "oidor",
    "oidorías": "oidoría",
    "oiga": "oír",
    "oigamos": "oír",
    "oigan": "oír",
    "oigas": "oír",
    "oigo": "oír",
    "oigáis": "oír",
    "oiremos": "oír",
    "oirá": "oír",
    "oirán": "oír",
    "oirás": "oír",
    "oiré": "oír",
    "oiréis": "oír",
    "oiría": "oír",
    "oiríais": "oír",
    "oiríamos": "oír",
    "oirían": "oír",
    "oirías": "oír",
    "ojala": "ojalar",
    "ojalaba": "ojalar",
    "ojalabais": "ojalar",
    "ojalaban": "ojalar",
    "ojalabas": "ojalar",
    "ojalad": "ojalar",
    "ojalada": "ojalar",
    "ojaladas": "ojalar",
    "ojalado": "ojalar",
    "ojaladora": "ojalador",
    "ojaladoras": "ojalador",
    "ojaladores": "ojalador",
    "ojalados": "ojalar",
    "ojaladuras": "ojaladura",
    "ojalamos": "ojalar",
    "ojalan": "ojalar",
    "ojalando": "ojalar",
    "ojalara": "ojalar",
    "ojalarais": "ojalar",
    "ojalaran": "ojalar",
    "ojalaras": "ojalar",
    "ojalare": "ojalar",
    "ojalareis": "ojalar",
    "ojalaremos": "ojalar",
    "ojalaren": "ojalar",
    "ojalares": "ojalar",
    "ojalaron": "ojalar",
    "ojalará": "ojalar",
    "ojalarán": "ojalar",
    "ojalarás": "ojalar",
    "ojalaré": "ojalar",
    "ojalaréis": "ojalar",
    "ojalaría": "ojalar",
    "ojalaríais": "ojalar",
    "ojalaríamos": "ojalar",
    "ojalarían": "ojalar",
    "ojalarías": "ojalar",
    "ojalas": "ojalar",
    "ojalase": "ojalar",
    "ojalaseis": "ojalar",
    "ojalasen": "ojalar",
    "ojalases": "ojalar",
    "ojalaste": "ojalar",
    "ojalasteis": "ojalar",
    "ojale": "ojalar",
    "ojalemos": "ojalar",
    "ojalen": "ojalar",
    "ojales": "ojalar",
    "ojalo": "ojalar",
    "ojalábamos": "ojalar",
    "ojaláis": "ojalar",
    "ojaláramos": "ojalar",
    "ojaláremos": "ojalar",
    "ojalásemos": "ojalar",
    "ojalé": "ojalar",
    "ojaléis": "ojalar",
    "ojaló": "ojalar",
    "ojancos": "ojanco",
    "ojaranzos": "ojaranzo",
    "ojea": "ojear",
    "ojeaba": "ojear",
    "ojeabais": "ojear",
    "ojeaban": "ojear",
    "ojeabas": "ojear",
    "ojead": "ojear",
    "ojeada": "ojear",
    "ojeadas": "ojear",
    "ojeado": "ojear",
    "ojeadora": "ojeador",
    "ojeadoras": "ojeador",
    "ojeadores": "ojeador",
    "ojeados": "ojear",
    "ojeamos": "ojear",
    "ojean": "ojear",
    "ojeando": "ojear",
    "ojeara": "ojear",
    "ojearais": "ojear",
    "ojearan": "ojear",
    "ojearas": "ojear",
    "ojeare": "ojear",
    "ojeareis": "ojear",
    "ojearemos": "ojear",
    "ojearen": "ojear",
    "ojeares": "ojear",
    "ojearon": "ojear",
    "ojeará": "ojear",
    "ojearán": "ojear",
    "ojearás": "ojear",
    "ojearé": "ojear",
    "ojearéis": "ojear",
    "ojearía": "ojear",
    "ojearíais": "ojear",
    "ojearíamos": "ojear",
    "ojearían": "ojear",
    "ojearías": "ojear",
    "ojeas": "ojear",
    "ojease": "ojear",
    "ojeaseis": "ojear",
    "ojeasen": "ojear",
    "ojeases": "ojear",
    "ojeaste": "ojear",
    "ojeasteis": "ojear",
    "ojee": "ojear",
    "ojeemos": "ojear",
    "ojeen": "ojear",
    "ojees": "ojear",
    "ojenes": "ojén",
    "ojeo": "ojear",
    "ojeos": "ojeo",
    "ojeras": "ojera",
    "ojerizas": "ojeriza",
    "ojerosa": "ojeroso",
    "ojerosas": "ojeroso",
    "ojerosos": "ojeroso",
    "ojeruda": "ojerudo",
    "ojerudas": "ojerudo",
    "ojerudos": "ojerudo",
    "ojetea": "ojetear",
    "ojeteaba": "ojetear",
    "ojeteabais": "ojetear",
    "ojeteaban": "ojetear",
    "ojeteabas": "ojetear",
    "ojetead": "ojetear",
    "ojeteada": "ojetear",
    "ojeteadas": "ojetear",
    "ojeteado": "ojetear",
    "ojeteados": "ojetear",
    "ojeteamos": "ojetear",
    "ojetean": "ojetear",
    "ojeteando": "ojetear",
    "ojeteara": "ojetear",
    "ojetearais": "ojetear",
    "ojetearan": "ojetear",
    "ojetearas": "ojetear",
    "ojeteare": "ojetear",
    "ojeteareis": "ojetear",
    "ojetearemos": "ojetear",
    "ojetearen": "ojetear",
    "ojeteares": "ojetear",
    "ojetearon": "ojetear",
    "ojeteará": "ojetear",
    "ojetearán": "ojetear",
    "ojetearás": "ojetear",
    "ojetearé": "ojetear",
    "ojetearéis": "ojetear",
    "ojetearía": "ojetear",
    "ojetearíais": "ojetear",
    "ojetearíamos": "ojetear",
    "ojetearían": "ojetear",
    "ojetearías": "ojetear",
    "ojeteas": "ojetear",
    "ojetease": "ojetear",
    "ojeteaseis": "ojetear",
    "ojeteasen": "ojetear",
    "ojeteases": "ojetear",
    "ojeteaste": "ojetear",
    "ojeteasteis": "ojetear",
    "ojetee": "ojetear",
    "ojeteemos": "ojetear",
    "ojeteen": "ojetear",
    "ojetees": "ojetear",
    "ojeteo": "ojetear",
    "ojeteras": "ojetera",
    "ojetes": "ojete",
    "ojeteábamos": "ojetear",
    "ojeteáis": "ojetear",
    "ojeteáramos": "ojetear",
    "ojeteáremos": "ojetear",
    "ojeteásemos": "ojetear",
    "ojeteé": "ojetear",
    "ojeteéis": "ojetear",
    "ojeteó": "ojetear",
    "ojeábamos": "ojear",
    "ojeáis": "ojear",
    "ojeáramos": "ojear",
    "ojeáremos": "ojear",
    "ojeásemos": "ojear",
    "ojeé": "ojear",
    "ojeéis": "ojear",
    "ojeó": "ojear",
    "ojialegres": "ojialegre",
    "ojibwas": "ojibwa",
    "ojienjuta": "ojienjuto",
    "ojienjutas": "ojienjuto",
    "ojienjutos": "ojienjuto",
    "ojigarza": "ojigarzo",
    "ojigarzas": "ojigarzo",
    "ojigarzos": "ojigarzo",
    "ojimeles": "ojimel",
    "ojimieles": "ojimiel",
    "ojimorena": "ojimoreno",
    "ojimorenas": "ojimoreno",
    "ojimorenos": "ojimoreno",
    "ojinegra": "ojinegro",
    "ojinegras": "ojinegro",
    "ojinegros": "ojinegro",
    "ojiprieta": "ojiprieto",
    "ojiprietas": "ojiprieto",
    "ojiprietos": "ojiprieto",
    "ojitos": "ojito",
    "ojituerta": "ojituerto",
    "ojituertas": "ojituerto",
    "ojituertos": "ojituerto",
    "ojivales": "ojival",
    "ojivas": "ojiva",
    "ojizaina": "ojizaino",
    "ojizainas": "ojizaino",
    "ojizainos": "ojizaino",
    "ojizarca": "ojizarco",
    "ojizarcas": "ojizarco",
    "ojizarcos": "ojizarco",
    "ojos": "ojo",
    "ojosa": "ojoso",
    "ojosas": "ojoso",
    "ojosos": "ojoso",
    "ojuelos": "ojuelo",
    "okapis": "okapi",
    "okumes": "okume",
    "okupas": "okupa",
    "olambrillas": "olambrilla",
    "olamos": "oler",
    "olas": "ola",
    "olea": "olear",
    "oleaba": "olear",
    "oleabais": "olear",
    "oleaban": "olear",
    "oleabas": "olear",
    "olead": "olear",
    "oleada": "olear",
    "oleadas": "olear",
    "oleado": "olear",
    "oleados": "olear",
    "oleaginosa": "oleaginoso",
    "oleaginosas": "oleaginoso",
    "oleaginosidades": "oleaginosidad",
    "oleaginosos": "oleaginoso",
    "oleajes": "oleaje",
    "oleamidas": "oleamida",
    "oleamos": "olear",
    "olean": "olear",
    "oleando": "olear",
    "oleara": "olear",
    "olearais": "olear",
    "olearan": "olear",
    "olearas": "olear",
    "oleare": "olear",
    "oleareis": "olear",
    "olearemos": "olear",
    "olearen": "olear",
    "oleares": "olear",
    "olearon": "olear",
    "oleará": "olear",
    "olearán": "olear",
    "olearás": "olear",
    "olearé": "olear",
    "olearéis": "olear",
    "olearía": "olear",
    "olearíais": "olear",
    "olearíamos": "olear",
    "olearían": "olear",
    "olearías": "olear",
    "oleas": "olear",
    "olease": "olear",
    "oleaseis": "olear",
    "oleasen": "olear",
    "oleases": "olear",
    "oleaste": "olear",
    "oleasteis": "olear",
    "oleatos": "oleato",
    "olecraniana": "olecraniano",
    "olecranianas": "olecraniano",
    "olecranianos": "olecraniano",
    "oled": "oler",
    "oledera": "oledero",
    "olederas": "oledero",
    "olederos": "oledero",
    "oledora": "oledor",
    "oledoras": "oledor",
    "oledores": "oledor",
    "olee": "olear",
    "oleemos": "olear",
    "oleen": "olear",
    "olees": "olear",
    "olefinas": "olefina",
    "oleica": "oleico",
    "oleicas": "oleico",
    "oleicos": "oleico",
    "oleicultora": "oleicultor",
    "oleicultoras": "oleicultor",
    "oleicultores": "oleicultor",
    "oleiculturas": "oleicultura",
    "olemos": "oler",
    "oleo": "olear",
    "oleoandomicinas": "oleoandomicina",
    "oleodinámicas": "oleodinámica",
    "oleoductos": "oleoducto",
    "oleografías": "oleografía",
    "oleoneumática": "oleoneumático",
    "oleoneumáticas": "oleoneumático",
    "oleoneumáticos": "oleoneumático",
    "oleoplastos": "oleoplasto",
    "oleorresinas": "oleorresina",
    "oleosa": "oleoso",
    "oleosas": "oleoso",
    "oleosidades": "oleosidad",
    "oleosos": "oleoso",
    "oleremos": "oler",
    "olerá": "oler",
    "olerán": "oler",
    "olerás": "oler",
    "oleré": "oler",
    "oleréis": "oler",
    "olería": "oler",
    "oleríais": "oler",
    "oleríamos": "oler",
    "olerían": "oler",
    "olerías": "oler",
    "oles": "ole",
    "oleábamos": "olear",
    "oleáceas": "oleácea",
    "oleáis": "olear",
    "oleáramos": "olear",
    "oleáremos": "olear",
    "oleásemos": "olear",
    "oleé": "olear",
    "oleéis": "olear",
    "oleícolas": "oleícola",
    "oleífera": "oleífero",
    "oleíferas": "oleífero",
    "oleíferos": "oleífero",
    "oleínas": "oleína",
    "oleó": "olear",
    "oleófila": "oleófilo",
    "oleófilas": "oleófilo",
    "oleófilos": "oleófilo",
    "oleómetros": "oleómetro",
    "olfacciones": "olfacción",
    "olfatea": "olfatear",
    "olfateaba": "olfatear",
    "olfateabais": "olfatear",
    "olfateaban": "olfatear",
    "olfateabas": "olfatear",
    "olfatead": "olfatear",
    "olfateada": "olfatear",
    "olfateadas": "olfatear",
    "olfateado": "olfatear",
    "olfateados": "olfatear",
    "olfateamos": "olfatear",
    "olfatean": "olfatear",
    "olfateando": "olfatear",
    "olfateara": "olfatear",
    "olfatearais": "olfatear",
    "olfatearan": "olfatear",
    "olfatearas": "olfatear",
    "olfateare": "olfatear",
    "olfateareis": "olfatear",
    "olfatearemos": "olfatear",
    "olfatearen": "olfatear",
    "olfateares": "olfatear",
    "olfatearon": "olfatear",
    "olfateará": "olfatear",
    "olfatearán": "olfatear",
    "olfatearás": "olfatear",
    "olfatearé": "olfatear",
    "olfatearéis": "olfatear",
    "olfatearía": "olfatear",
    "olfatearíais": "olfatear",
    "olfatearíamos": "olfatear",
    "olfatearían": "olfatear",
    "olfatearías": "olfatear",
    "olfateas": "olfatear",
    "olfatease": "olfatear",
    "olfateaseis": "olfatear",
    "olfateasen": "olfatear",
    "olfateases": "olfatear",
    "olfateaste": "olfatear",
    "olfateasteis": "olfatear",
    "olfatee": "olfatear",
    "olfateemos": "olfatear",
    "olfateen": "olfatear",
    "olfatees": "olfatear",
    "olfateo": "olfatear",
    "olfateos": "olfateo",
    "olfateábamos": "olfatear",
    "olfateáis": "olfatear",
    "olfateáramos": "olfatear",
    "olfateáremos": "olfatear",
    "olfateásemos": "olfatear",
    "olfateé": "olfatear",
    "olfateéis": "olfatear",
    "olfateó": "olfatear",
    "olfativa": "olfativo",
    "olfativas": "olfativo",
    "olfativos": "olfativo",
    "olfatometrías": "olfatometría",
    "olfatoria": "olfatorio",
    "olfatorias": "olfatorio",
    "olfatorios": "olfatorio",
    "olfatos": "olfato",
    "olfatómetros": "olfatómetro",
    "olida": "oler",
    "olidas": "oler",
    "olido": "oler",
    "olidos": "oler",
    "oliendo": "oler",
    "oliera": "oler",
    "olierais": "oler",
    "olieran": "oler",
    "olieras": "oliera",
    "oliere": "oler",
    "oliereis": "oler",
    "olieren": "oler",
    "olieres": "oler",
    "olieron": "oler",
    "oliese": "oler",
    "olieseis": "oler",
    "oliesen": "oler",
    "olieses": "oler",
    "olifantes": "olifante",
    "oligarcas": "oligarca",
    "oligarquías": "oligarquía",
    "oligistos": "oligisto",
    "oligocena": "oligoceno",
    "oligocenas": "oligoceno",
    "oligocenos": "oligoceno",
    "oligoclasas": "oligoclasa",
    "oligoclonales": "oligoclonal",
    "oligodendrocitos": "oligodendrocito",
    "oligoelementos": "oligoelemento",
    "oligofrenias": "oligofrenia",
    "oligogenes": "oligogén",
    "oligomerizaciones": "oligomerización",
    "oligopolios": "oligopolio",
    "oligopolistas": "oligopolista",
    "oligopsonios": "oligopsonio",
    "oligopólica": "oligopólico",
    "oligopólicas": "oligopólico",
    "oligopólicos": "oligopólico",
    "oligoquetos": "oligoqueto",
    "oligosacáridos": "oligosacárido",
    "oligospermias": "oligospermia",
    "oligotrofias": "oligotrofia",
    "oligotrófica": "oligotrófico",
    "oligotróficas": "oligotrófico",
    "oligotróficos": "oligotrófico",
    "oligurias": "oliguria",
    "oligárquica": "oligárquico",
    "oligárquicas": "oligárquico",
    "oligárquicos": "oligárquico",
    "oligómera": "oligómero",
    "oligómeras": "oligómero",
    "oligómeros": "oligómero",
    "olimos": "oler",
    "olimpiadas": "olimpiada",
    "olimpismos": "olimpismo",
    "olimpos": "olimpo",
    "olimpíadas": "olimpíada",
    "olisca": "olisco",
    "oliscaba": "oliscar",
    "oliscabais": "oliscar",
    "oliscaban": "oliscar",
    "oliscabas": "oliscar",
    "oliscad": "oliscar",
    "oliscada": "oliscar",
    "oliscadas": "oliscar",
    "oliscado": "oliscar",
    "oliscados": "oliscar",
    "oliscamos": "oliscar",
    "oliscan": "oliscar",
    "oliscando": "oliscar",
    "oliscara": "oliscar",
    "oliscarais": "oliscar",
    "oliscaran": "oliscar",
    "oliscaras": "oliscar",
    "oliscare": "oliscar",
    "oliscareis": "oliscar",
    "oliscaremos": "oliscar",
    "oliscaren": "oliscar",
    "oliscares": "oliscar",
    "oliscaron": "oliscar",
    "oliscará": "oliscar",
    "oliscarán": "oliscar",
    "oliscarás": "oliscar",
    "oliscaré": "oliscar",
    "oliscaréis": "oliscar",
    "oliscaría": "oliscar",
    "oliscaríais": "oliscar",
    "oliscaríamos": "oliscar",
    "oliscarían": "oliscar",
    "oliscarías": "oliscar",
    "oliscas": "olisco",
    "oliscase": "oliscar",
    "oliscaseis": "oliscar",
    "oliscasen": "oliscar",
    "oliscases": "oliscar",
    "oliscaste": "oliscar",
    "oliscasteis": "oliscar",
    "olisco": "oliscar",
    "oliscos": "olisco",
    "oliscosa": "oliscoso",
    "oliscosas": "oliscoso",
    "oliscosos": "oliscoso",
    "oliscábamos": "oliscar",
    "oliscáis": "oliscar",
    "oliscáramos": "oliscar",
    "oliscáremos": "oliscar",
    "oliscásemos": "oliscar",
    "oliscó": "oliscar",
    "olisque": "oliscar",
    "olisquea": "olisquear",
    "olisqueaba": "olisquear",
    "olisqueabais": "olisquear",
    "olisqueaban": "olisquear",
    "olisqueabas": "olisquear",
    "olisquead": "olisquear",
    "olisqueada": "olisquear",
    "olisqueadas": "olisquear",
    "olisqueado": "olisquear",
    "olisqueados": "olisquear",
    "olisqueamos": "olisquear",
    "olisquean": "olisquear",
    "olisqueando": "olisquear",
    "olisqueara": "olisquear",
    "olisquearais": "olisquear",
    "olisquearan": "olisquear",
    "olisquearas": "olisquear",
    "olisqueare": "olisquear",
    "olisqueareis": "olisquear",
    "olisquearemos": "olisquear",
    "olisquearen": "olisquear",
    "olisqueares": "olisquear",
    "olisquearon": "olisquear",
    "olisqueará": "olisquear",
    "olisquearán": "olisquear",
    "olisquearás": "olisquear",
    "olisquearé": "olisquear",
    "olisquearéis": "olisquear",
    "olisquearía": "olisquear",
    "olisquearíais": "olisquear",
    "olisquearíamos": "olisquear",
    "olisquearían": "olisquear",
    "olisquearías": "olisquear",
    "olisqueas": "olisquear",
    "olisquease": "olisquear",
    "olisqueaseis": "olisquear",
    "olisqueasen": "olisquear",
    "olisqueases": "olisquear",
    "olisqueaste": "olisquear",
    "olisqueasteis": "olisquear",
    "olisquee": "olisquear",
    "olisqueemos": "olisquear",
    "olisqueen": "olisquear",
    "olisquees": "olisquear",
    "olisquemos": "oliscar",
    "olisquen": "oliscar",
    "olisqueo": "olisquear",
    "olisques": "oliscar",
    "olisqueábamos": "olisquear",
    "olisqueáis": "olisquear",
    "olisqueáramos": "olisquear",
    "olisqueáremos": "olisquear",
    "olisqueásemos": "olisquear",
    "olisqueé": "olisquear",
    "olisqueéis": "olisquear",
    "olisqueó": "olisquear",
    "olisqué": "oliscar",
    "olisquéis": "oliscar",
    "oliste": "oler",
    "olisteis": "oler",
    "oliva": "olivar",
    "olivaba": "olivar",
    "olivabais": "olivar",
    "olivaban": "olivar",
    "olivabas": "olivar",
    "olivad": "olivar",
    "olivada": "olivar",
    "olivadas": "olivar",
    "olivado": "olivar",
    "olivados": "olivar",
    "olivamos": "olivar",
    "olivan": "olivar",
    "olivando": "olivar",
    "olivara": "olivar",
    "olivarais": "olivar",
    "olivaran": "olivar",
    "olivaras": "olivar",
    "olivardas": "olivarda",
    "olivare": "olivar",
    "olivareis": "olivar",
    "olivaremos": "olivar",
    "olivaren": "olivar",
    "olivarera": "olivarero",
    "olivareras": "olivarero",
    "olivareros": "olivarero",
    "olivares": "olivar",
    "olivaron": "olivar",
    "olivará": "olivar",
    "olivarán": "olivar",
    "olivarás": "olivar",
    "olivaré": "olivar",
    "olivaréis": "olivar",
    "olivaría": "olivar",
    "olivaríais": "olivar",
    "olivaríamos": "olivar",
    "olivarían": "olivar",
    "olivarías": "olivar",
    "olivas": "olivar",
    "olivase": "olivar",
    "olivaseis": "olivar",
    "olivasen": "olivar",
    "olivases": "olivar",
    "olivaste": "olivar",
    "olivasteis": "olivar",
    "olive": "olivar",
    "olivemos": "olivar",
    "oliven": "olivar",
    "olivenitas": "olivenita",
    "oliveras": "olivera",
    "oliveros": "olivero",
    "olives": "olivar",
    "olivicultora": "olivicultor",
    "olivicultoras": "olivicultor",
    "olivicultores": "olivicultor",
    "oliviculturas": "olivicultura",
    "olivinos": "olivino",
    "olivo": "olivar",
    "olivos": "olivo",
    "olivábamos": "olivar",
    "olivácea": "oliváceo",
    "oliváceas": "oliváceo",
    "oliváceos": "oliváceo",
    "oliváis": "olivar",
    "oliváramos": "olivar",
    "oliváremos": "olivar",
    "olivásemos": "olivar",
    "olivé": "olivar",
    "olivéis": "olivar",
    "olivícolas": "olivícola",
    "olivó": "olivar",
    "oliéramos": "oler",
    "oliéremos": "oler",
    "oliésemos": "oler",
    "olió": "oler",
    "ollados": "ollado",
    "ollaos": "ollao",
    "ollares": "ollar",
    "ollas": "olla",
    "ollera": "ollero",
    "olleras": "ollero",
    "olleros": "ollero",
    "ollerías": "ollería",
    "olmas": "olma",
    "olmecas": "olmeca",
    "olmedas": "olmeda",
    "olmedos": "olmedo",
    "olmos": "olmo",
    "olores": "olor",
    "olorice": "olorizar",
    "oloricemos": "olorizar",
    "oloricen": "olorizar",
    "olorices": "olorizar",
    "oloricé": "olorizar",
    "oloricéis": "olorizar",
    "oloriza": "olorizar",
    "olorizaba": "olorizar",
    "olorizabais": "olorizar",
    "olorizaban": "olorizar",
    "olorizabas": "olorizar",
    "olorizad": "olorizar",
    "olorizada": "olorizar",
    "olorizadas": "olorizar",
    "olorizado": "olorizar",
    "olorizados": "olorizar",
    "olorizamos": "olorizar",
    "olorizan": "olorizar",
    "olorizando": "olorizar",
    "olorizara": "olorizar",
    "olorizarais": "olorizar",
    "olorizaran": "olorizar",
    "olorizaras": "olorizar",
    "olorizare": "olorizar",
    "olorizareis": "olorizar",
    "olorizaremos": "olorizar",
    "olorizaren": "olorizar",
    "olorizares": "olorizar",
    "olorizaron": "olorizar",
    "olorizará": "olorizar",
    "olorizarán": "olorizar",
    "olorizarás": "olorizar",
    "olorizaré": "olorizar",
    "olorizaréis": "olorizar",
    "olorizaría": "olorizar",
    "olorizaríais": "olorizar",
    "olorizaríamos": "olorizar",
    "olorizarían": "olorizar",
    "olorizarías": "olorizar",
    "olorizas": "olorizar",
    "olorizase": "olorizar",
    "olorizaseis": "olorizar",
    "olorizasen": "olorizar",
    "olorizases": "olorizar",
    "olorizaste": "olorizar",
    "olorizasteis": "olorizar",
    "olorizo": "olorizar",
    "olorizábamos": "olorizar",
    "olorizáis": "olorizar",
    "olorizáramos": "olorizar",
    "olorizáremos": "olorizar",
    "olorizásemos": "olorizar",
    "olorizó": "olorizar",
    "olorosa": "oloroso",
    "olorosas": "oloroso",
    "olorosos": "oloroso",
    "olvida": "olvidar",
    "olvidaba": "olvidar",
    "olvidabais": "olvidar",
    "olvidaban": "olvidar",
    "olvidabas": "olvidar",
    "olvidad": "olvidar",
    "olvidada": "olvidar",
    "olvidadas": "olvidar",
    "olvidadiza": "olvidadizo",
    "olvidadizas": "olvidadizo",
    "olvidadizos": "olvidadizo",
    "olvidado": "olvidar",
    "olvidados": "olvidar",
    "olvidamos": "olvidar",
    "olvidan": "olvidar",
    "olvidando": "olvidar",
    "olvidara": "olvidar",
    "olvidarais": "olvidar",
    "olvidaran": "olvidar",
    "olvidaras": "olvidar",
    "olvidare": "olvidar",
    "olvidareis": "olvidar",
    "olvidaremos": "olvidar",
    "olvidaren": "olvidar",
    "olvidares": "olvidar",
    "olvidaron": "olvidar",
    "olvidará": "olvidar",
    "olvidarán": "olvidar",
    "olvidarás": "olvidar",
    "olvidaré": "olvidar",
    "olvidaréis": "olvidar",
    "olvidaría": "olvidar",
    "olvidaríais": "olvidar",
    "olvidaríamos": "olvidar",
    "olvidarían": "olvidar",
    "olvidarías": "olvidar",
    "olvidas": "olvidar",
    "olvidase": "olvidar",
    "olvidaseis": "olvidar",
    "olvidasen": "olvidar",
    "olvidases": "olvidar",
    "olvidaste": "olvidar",
    "olvidasteis": "olvidar",
    "olvide": "olvidar",
    "olvidemos": "olvidar",
    "olviden": "olvidar",
    "olvides": "olvidar",
    "olvido": "olvidar",
    "olvidos": "olvido",
    "olvidábamos": "olvidar",
    "olvidáis": "olvidar",
    "olvidáramos": "olvidar",
    "olvidáremos": "olvidar",
    "olvidásemos": "olvidar",
    "olvidé": "olvidar",
    "olvidéis": "olvidar",
    "olvidó": "olvidar",
    "oláis": "oler",
    "oléis": "oler",
    "olí": "oler",
    "olía": "oler",
    "olíais": "oler",
    "olíamos": "oler",
    "olían": "oler",
    "olías": "oler",
    "olíbanos": "olíbano",
    "olímpica": "olímpico",
    "olímpicas": "olímpico",
    "olímpicos": "olímpico",
    "ológrafa": "ológrafo",
    "ológrafas": "ológrafo",
    "ológrafos": "ológrafo",
    "omaguas": "omagua",
    "omaníses": "omaní",
    "omasos": "omaso",
    "ombligadas": "ombligada",
    "ombligos": "ombligo",
    "ombligueros": "ombliguero",
    "ombrías": "ombría",
    "ombudsmanes": "ombudsman",
    "ombudsmans": "ombudsman",
    "omegas": "omega",
    "omentales": "omental",
    "omentos": "omento",
    "omeyas": "omeya",
    "omina": "ominar",
    "ominaba": "ominar",
    "ominabais": "ominar",
    "ominaban": "ominar",
    "ominabas": "ominar",
    "ominad": "ominar",
    "ominada": "ominar",
    "ominadas": "ominar",
    "ominado": "ominar",
    "ominados": "ominar",
    "ominamos": "ominar",
    "ominan": "ominar",
    "ominando": "ominar",
    "ominara": "ominar",
    "ominarais": "ominar",
    "ominaran": "ominar",
    "ominaras": "ominar",
    "ominare": "ominar",
    "ominareis": "ominar",
    "ominaremos": "ominar",
    "ominaren": "ominar",
    "ominares": "ominar",
    "ominaron": "ominar",
    "ominará": "ominar",
    "ominarán": "ominar",
    "ominarás": "ominar",
    "ominaré": "ominar",
    "ominaréis": "ominar",
    "ominaría": "ominar",
    "ominaríais": "ominar",
    "ominaríamos": "ominar",
    "ominarían": "ominar",
    "ominarías": "ominar",
    "ominas": "ominar",
    "ominase": "ominar",
    "ominaseis": "ominar",
    "ominasen": "ominar",
    "ominases": "ominar",
    "ominaste": "ominar",
    "ominasteis": "ominar",
    "omine": "ominar",
    "ominemos": "ominar",
    "ominen": "ominar",
    "omines": "ominar",
    "omino": "ominar",
    "ominosa": "ominoso",
    "ominosas": "ominoso",
    "ominosos": "ominoso",
    "ominábamos": "ominar",
    "omináis": "ominar",
    "omináramos": "ominar",
    "omináremos": "ominar",
    "ominásemos": "ominar",
    "ominé": "ominar",
    "ominéis": "ominar",
    "ominó": "ominar",
    "omisa": "omiso",
    "omisas": "omiso",
    "omisibles": "omisible",
    "omisiones": "omisión",
    "omisiónes": "omisión",
    "omisos": "omiso",
    "omita": "omitir",
    "omitamos": "omitir",
    "omitan": "omitir",
    "omitas": "omitir",
    "omite": "omitir",
    "omiten": "omitir",
    "omites": "omitir",
    "omitibles": "omitible",
    "omitid": "omitir",
    "omitida": "omitir",
    "omitidas": "omitir",
    "omitido": "omitir",
    "omitidos": "omitir",
    "omitiendo": "omitir",
    "omitiera": "omitir",
    "omitierais": "omitir",
    "omitieran": "omitir",
    "omitieras": "omitir",
    "omitiere": "omitir",
    "omitiereis": "omitir",
    "omitieren": "omitir",
    "omitieres": "omitir",
    "omitieron": "omitir",
    "omitiese": "omitir",
    "omitieseis": "omitir",
    "omitiesen": "omitir",
    "omitieses": "omitir",
    "omitimos": "omitir",
    "omitiremos": "omitir",
    "omitirá": "omitir",
    "omitirán": "omitir",
    "omitirás": "omitir",
    "omitiré": "omitir",
    "omitiréis": "omitir",
    "omitiría": "omitir",
    "omitiríais": "omitir",
    "omitiríamos": "omitir",
    "omitirían": "omitir",
    "omitirías": "omitir",
    "omitiste": "omitir",
    "omitisteis": "omitir",
    "omitiéramos": "omitir",
    "omitiéremos": "omitir",
    "omitiésemos": "omitir",
    "omitió": "omitir",
    "omito": "omitir",
    "omitáis": "omitir",
    "omití": "omitir",
    "omitía": "omitir",
    "omitíais": "omitir",
    "omitíamos": "omitir",
    "omitían": "omitir",
    "omitías": "omitir",
    "omitís": "omitir",
    "omniabarcadora": "omniabarcador",
    "omniabarcadoras": "omniabarcador",
    "omniabarcadores": "omniabarcador",
    "omnibuses": "ómnibus",
    "omnibús": "ómnibus",
    "omnidireccionales": "omnidireccional",
    "omnipotencias": "omnipotencia",
    "omnipotentes": "omnipotente",
    "omnipresencias": "omnipresencia",
    "omnipresentes": "omnipresente",
    "omnisapientes": "omnisapiente",
    "omniscia": "omniscio",
    "omniscias": "omniscio",
    "omnisciencias": "omnisciencia",
    "omniscientes": "omnisciente",
    "omniscios": "omniscio",
    "omnímoda": "omnímodo",
    "omnímodas": "omnímodo",
    "omnímodos": "omnímodo",
    "omnívora": "omnívoro",
    "omnívoras": "omnívoro",
    "omnívoros": "omnívoro",
    "omoclaviculares": "omoclavicular",
    "omohioidea": "omohioideo",
    "omohioideas": "omohioideo",
    "omohioideos": "omohioideo",
    "omoplatos": "omoplato",
    "omóplatos": "omóplato",
    "on-line": "online",
    "onagros": "onagro",
    "onanismos": "onanismo",
    "onanistas": "onanista",
    "onceava": "onceavo",
    "onceavas": "onceavo",
    "onceavo": "11",
    "onceavos": "onceavo",
    "oncena": "onceno",
    "oncenas": "onceno",
    "oncenos": "onceno",
    "oncofetales": "oncofetal",
    "oncogenes": "oncogén",
    "oncologías": "oncología",
    "oncológica": "oncológico",
    "oncológicas": "oncológico",
    "oncológicos": "oncológico",
    "oncógena": "oncógeno",
    "oncógenas": "oncógeno",
    "oncógenos": "oncógeno",
    "oncóloga": "oncólogo",
    "oncólogas": "oncólogo",
    "oncólogos": "oncólogo",
    "oncótica": "oncótico",
    "oncóticas": "oncótico",
    "oncóticos": "oncótico",
    "ondas": "onda",
    "ondea": "ondear",
    "ondeaba": "ondear",
    "ondeabais": "ondear",
    "ondeaban": "ondear",
    "ondeabas": "ondear",
    "ondead": "ondear",
    "ondeada": "ondear",
    "ondeadas": "ondear",
    "ondeado": "ondear",
    "ondeados": "ondear",
    "ondeamos": "ondear",
    "ondean": "ondear",
    "ondeando": "ondear",
    "ondeara": "ondear",
    "ondearais": "ondear",
    "ondearan": "ondear",
    "ondearas": "ondear",
    "ondeare": "ondear",
    "ondeareis": "ondear",
    "ondearemos": "ondear",
    "ondearen": "ondear",
    "ondeares": "ondear",
    "ondearon": "ondear",
    "ondeará": "ondear",
    "ondearán": "ondear",
    "ondearás": "ondear",
    "ondearé": "ondear",
    "ondearéis": "ondear",
    "ondearía": "ondear",
    "ondearíais": "ondear",
    "ondearíamos": "ondear",
    "ondearían": "ondear",
    "ondearías": "ondear",
    "ondeas": "ondear",
    "ondease": "ondear",
    "ondeaseis": "ondear",
    "ondeasen": "ondear",
    "ondeases": "ondear",
    "ondeaste": "ondear",
    "ondeasteis": "ondear",
    "ondee": "ondear",
    "ondeemos": "ondear",
    "ondeen": "ondear",
    "ondees": "ondear",
    "ondeo": "ondear",
    "ondeos": "ondeo",
    "ondeábamos": "ondear",
    "ondeáis": "ondear",
    "ondeáramos": "ondear",
    "ondeáremos": "ondear",
    "ondeásemos": "ondear",
    "ondeé": "ondear",
    "ondeéis": "ondear",
    "ondeó": "ondear",
    "ondinas": "ondina",
    "ondisonantes": "ondisonante",
    "ondula": "ondular",
    "ondulaba": "ondular",
    "ondulabais": "ondular",
    "ondulaban": "ondular",
    "ondulabas": "ondular",
    "ondulaciones": "ondulación",
    "ondulad": "ondular",
    "ondulada": "ondular",
    "onduladas": "ondular",
    "ondulado": "ondular",
    "onduladora": "ondulador",
    "onduladoras": "ondulador",
    "onduladores": "ondulador",
    "ondulados": "ondular",
    "ondulamos": "ondular",
    "ondulan": "ondular",
    "ondulando": "ondular",
    "ondulantes": "ondulante",
    "ondulara": "ondular",
    "ondularais": "ondular",
    "ondularan": "ondular",
    "ondularas": "ondular",
    "ondulare": "ondular",
    "ondulareis": "ondular",
    "ondularemos": "ondular",
    "ondularen": "ondular",
    "ondulares": "ondular",
    "ondularon": "ondular",
    "ondulará": "ondular",
    "ondularán": "ondular",
    "ondularás": "ondular",
    "ondularé": "ondular",
    "ondularéis": "ondular",
    "ondularía": "ondular",
    "ondularíais": "ondular",
    "ondularíamos": "ondular",
    "ondularían": "ondular",
    "ondularías": "ondular",
    "ondulas": "ondular",
    "ondulase": "ondular",
    "ondulaseis": "ondular",
    "ondulasen": "ondular",
    "ondulases": "ondular",
    "ondulaste": "ondular",
    "ondulasteis": "ondular",
    "ondulatoria": "ondulatorio",
    "ondulatorias": "ondulatorio",
    "ondulatorios": "ondulatorio",
    "ondule": "ondular",
    "ondulemos": "ondular",
    "ondulen": "ondular",
    "ondules": "ondular",
    "ondulitas": "ondulita",
    "ondulo": "ondular",
    "ondulábamos": "ondular",
    "onduláis": "ondular",
    "onduláramos": "ondular",
    "onduláremos": "ondular",
    "ondulásemos": "ondular",
    "ondulé": "ondular",
    "onduléis": "ondular",
    "onduló": "ondular",
    "ondámetros": "ondámetro",
    "oneraria": "onerario",
    "onerarias": "onerario",
    "onerarios": "onerario",
    "onerosa": "oneroso",
    "onerosas": "oneroso",
    "onerosos": "oneroso",
    "onfalomesentérica": "onfalomesentérico",
    "onfalomesentéricas": "onfalomesentérico",
    "onfalomesentéricos": "onfalomesentérico",
    "onicofagias": "onicofagia",
    "onicomancias": "onicomancia",
    "onicóforos": "onicóforo",
    "onirismos": "onirismo",
    "oniromancias": "oniromancia",
    "onomancias": "onomancia",
    "onomasiologías": "onomasiología",
    "onomasiológica": "onomasiológico",
    "onomasiológicas": "onomasiológico",
    "onomasiológicos": "onomasiológico",
    "onomatologías": "onomatología",
    "onomatopeyas": "onomatopeya",
    "onomatopéyica": "onomatopéyico",
    "onomatopéyicas": "onomatopéyico",
    "onomatopéyicos": "onomatopéyico",
    "onomástica": "onomástico",
    "onomásticas": "onomástico",
    "onomásticos": "onomástico",
    "onsetes": "onset",
    "onsets": "onset",
    "ontogenias": "ontogenia",
    "ontogénica": "ontogénico",
    "ontogénicas": "ontogénico",
    "ontogénicos": "ontogénico",
    "ontologismos": "ontologismo",
    "ontologías": "ontología",
    "ontológica": "ontológico",
    "ontológicas": "ontológico",
    "ontológicos": "ontológico",
    "ontóloga": "ontólogo",
    "ontólogas": "ontólogo",
    "ontólogos": "ontólogo",
    "onubenses": "onubense",
    "onzas": "onza",
    "onzavos": "onzavo",
    "onírica": "onírico",
    "oníricas": "onírico",
    "oníricos": "onírico",
    "oocitos": "oocito",
    "oogamias": "oogamia",
    "oogonias": "oogonia",
    "oogonios": "oogonio",
    "oolitos": "oolito",
    "oologías": "oología",
    "oomicetales": "oomicetal",
    "oosferas": "oosfera",
    "opaca": "opaco",
    "opacaba": "opacar",
    "opacabais": "opacar",
    "opacaban": "opacar",
    "opacabas": "opacar",
    "opacad": "opacar",
    "opacada": "opacar",
    "opacadas": "opacar",
    "opacado": "opacar",
    "opacados": "opacar",
    "opacamos": "opacar",
    "opacan": "opacar",
    "opacando": "opacar",
    "opacara": "opacar",
    "opacarais": "opacar",
    "opacaran": "opacar",
    "opacaras": "opacar",
    "opacare": "opacar",
    "opacareis": "opacar",
    "opacaremos": "opacar",
    "opacaren": "opacar",
    "opacares": "opacar",
    "opacaron": "opacar",
    "opacará": "opacar",
    "opacarán": "opacar",
    "opacarás": "opacar",
    "opacaré": "opacar",
    "opacaréis": "opacar",
    "opacaría": "opacar",
    "opacaríais": "opacar",
    "opacaríamos": "opacar",
    "opacarían": "opacar",
    "opacarías": "opacar",
    "opacas": "opaco",
    "opacase": "opacar",
    "opacaseis": "opacar",
    "opacasen": "opacar",
    "opacases": "opacar",
    "opacaste": "opacar",
    "opacasteis": "opacar",
    "opacidades": "opacidad",
    "opacificaciones": "opacificación",
    "opacificadora": "opacificador",
    "opacificadoras": "opacificador",
    "opacificadores": "opacificador",
    "opaco": "opacar",
    "opacos": "opaco",
    "opacábamos": "opacar",
    "opacáis": "opacar",
    "opacáramos": "opacar",
    "opacáremos": "opacar",
    "opacásemos": "opacar",
    "opacímetros": "opacímetro",
    "opacó": "opacar",
    "opada": "opado",
    "opadas": "opado",
    "opados": "opado",
    "opales": "opal",
    "opalescencias": "opalescencia",
    "opalescentes": "opalescente",
    "opalice": "opalizar",
    "opalicemos": "opalizar",
    "opalicen": "opalizar",
    "opalices": "opalizar",
    "opalicé": "opalizar",
    "opalicéis": "opalizar",
    "opalina": "opalino",
    "opalinas": "opalino",
    "opalinos": "opalino",
    "opaliza": "opalizar",
    "opalizaba": "opalizar",
    "opalizabais": "opalizar",
    "opalizaban": "opalizar",
    "opalizabas": "opalizar",
    "opalizad": "opalizar",
    "opalizada": "opalizar",
    "opalizadas": "opalizar",
    "opalizado": "opalizar",
    "opalizados": "opalizar",
    "opalizamos": "opalizar",
    "opalizan": "opalizar",
    "opalizando": "opalizar",
    "opalizara": "opalizar",
    "opalizarais": "opalizar",
    "opalizaran": "opalizar",
    "opalizaras": "opalizar",
    "opalizare": "opalizar",
    "opalizareis": "opalizar",
    "opalizaremos": "opalizar",
    "opalizaren": "opalizar",
    "opalizares": "opalizar",
    "opalizaron": "opalizar",
    "opalizará": "opalizar",
    "opalizarán": "opalizar",
    "opalizarás": "opalizar",
    "opalizaré": "opalizar",
    "opalizaréis": "opalizar",
    "opalizaría": "opalizar",
    "opalizaríais": "opalizar",
    "opalizaríamos": "opalizar",
    "opalizarían": "opalizar",
    "opalizarías": "opalizar",
    "opalizas": "opalizar",
    "opalizase": "opalizar",
    "opalizaseis": "opalizar",
    "opalizasen": "opalizar",
    "opalizases": "opalizar",
    "opalizaste": "opalizar",
    "opalizasteis": "opalizar",
    "opalizo": "opalizar",
    "opalizábamos": "opalizar",
    "opalizáis": "opalizar",
    "opalizáramos": "opalizar",
    "opalizáremos": "opalizar",
    "opalizásemos": "opalizar",
    "opalizó": "opalizar",
    "opaque": "opacar",
    "opaquemos": "opacar",
    "opaquen": "opacar",
    "opaques": "opacar",
    "opaqué": "opacar",
    "opaquéis": "opacar",
    "opas": "opa",
    "opcionales": "opcional",
    "opcionalidades": "opcionalidad",
    "opciones": "opción",
    "opera": "operar",
    "operaba": "operar",
    "operabais": "operar",
    "operaban": "operar",
    "operabas": "operar",
    "operabilidades": "operabilidad",
    "operables": "operable",
    "operacionales": "operacional",
    "operacionalismos": "operacionalismo",
    "operaciones": "operación",
    "operad": "operar",
    "operada": "operar",
    "operadas": "operar",
    "operado": "operar",
    "operadora": "operador",
    "operadoras": "operador",
    "operadores": "operador",
    "operados": "operar",
    "operamos": "operar",
    "operan": "operar",
    "operando": "operar",
    "operandos": "operando",
    "operantes": "operante",
    "operara": "operar",
    "operarais": "operar",
    "operaran": "operar",
    "operaras": "operar",
    "operare": "operar",
    "operareis": "operar",
    "operaremos": "operar",
    "operaren": "operar",
    "operares": "operar",
    "operaria": "operario",
    "operarias": "operario",
    "operarios": "operario",
    "operaron": "operar",
    "operará": "operar",
    "operarán": "operar",
    "operarás": "operar",
    "operaré": "operar",
    "operaréis": "operar",
    "operaría": "operar",
    "operaríais": "operar",
    "operaríamos": "operar",
    "operarían": "operar",
    "operarías": "operar",
    "operas": "operar",
    "operase": "operar",
    "operaseis": "operar",
    "operasen": "operar",
    "operases": "operar",
    "operaste": "operar",
    "operasteis": "operar",
    "operativa": "operativo",
    "operativas": "operativo",
    "operatividades": "operatividad",
    "operativos": "operativo",
    "operatoria": "operatorio",
    "operatorias": "operatorio",
    "operatorios": "operatorio",
    "operculares": "opercular",
    "opere": "operar",
    "operemos": "operar",
    "operen": "operar",
    "operes": "operar",
    "operetas": "opereta",
    "operistas": "operista",
    "opero": "operar",
    "operosa": "operoso",
    "operosas": "operoso",
    "operosos": "operoso",
    "operábamos": "operar",
    "operáis": "operar",
    "operáramos": "operar",
    "operáremos": "operar",
    "operásemos": "operar",
    "operé": "operar",
    "operéis": "operar",
    "operística": "operístico",
    "operísticas": "operístico",
    "operísticos": "operístico",
    "operó": "operar",
    "opiada": "opiado",
    "opiadas": "opiado",
    "opiados": "opiado",
    "opiatas": "opiata",
    "opiatos": "opiato",
    "opila": "opilar",
    "opilaba": "opilar",
    "opilabais": "opilar",
    "opilaban": "opilar",
    "opilabas": "opilar",
    "opilaciones": "opilación",
    "opilad": "opilar",
    "opilada": "opilar",
    "opiladas": "opilar",
    "opilado": "opilar",
    "opilados": "opilar",
    "opilamos": "opilar",
    "opilan": "opilar",
    "opilando": "opilar",
    "opilara": "opilar",
    "opilarais": "opilar",
    "opilaran": "opilar",
    "opilaras": "opilar",
    "opilare": "opilar",
    "opilareis": "opilar",
    "opilaremos": "opilar",
    "opilaren": "opilar",
    "opilares": "opilar",
    "opilaron": "opilar",
    "opilará": "opilar",
    "opilarán": "opilar",
    "opilarás": "opilar",
    "opilaré": "opilar",
    "opilaréis": "opilar",
    "opilaría": "opilar",
    "opilaríais": "opilar",
    "opilaríamos": "opilar",
    "opilarían": "opilar",
    "opilarías": "opilar",
    "opilas": "opilar",
    "opilase": "opilar",
    "opilaseis": "opilar",
    "opilasen": "opilar",
    "opilases": "opilar",
    "opilaste": "opilar",
    "opilasteis": "opilar",
    "opilativa": "opilativo",
    "opilativas": "opilativo",
    "opilativos": "opilativo",
    "opile": "opilar",
    "opilemos": "opilar",
    "opilen": "opilar",
    "opiles": "opilar",
    "opiliones": "opilión",
    "opiliónes": "opilión",
    "opilo": "opilar",
    "opilábamos": "opilar",
    "opiláis": "opilar",
    "opiláramos": "opilar",
    "opiláremos": "opilar",
    "opilásemos": "opilar",
    "opilé": "opilar",
    "opiléis": "opilar",
    "opiló": "opilar",
    "opima": "opimo",
    "opimas": "opimo",
    "opimos": "opimo",
    "opina": "opinar",
    "opinaba": "opinar",
    "opinabais": "opinar",
    "opinaban": "opinar",
    "opinabas": "opinar",
    "opinables": "opinable",
    "opinad": "opinar",
    "opinada": "opinar",
    "opinadas": "opinar",
    "opinado": "opinar",
    "opinados": "opinar",
    "opinamos": "opinar",
    "opinan": "opinar",
    "opinando": "opinar",
    "opinara": "opinar",
    "opinarais": "opinar",
    "opinaran": "opinar",
    "opinaras": "opinar",
    "opinare": "opinar",
    "opinareis": "opinar",
    "opinaremos": "opinar",
    "opinaren": "opinar",
    "opinares": "opinar",
    "opinaron": "opinar",
    "opinará": "opinar",
    "opinarán": "opinar",
    "opinarás": "opinar",
    "opinaré": "opinar",
    "opinaréis": "opinar",
    "opinaría": "opinar",
    "opinaríais": "opinar",
    "opinaríamos": "opinar",
    "opinarían": "opinar",
    "opinarías": "opinar",
    "opinas": "opinar",
    "opinase": "opinar",
    "opinaseis": "opinar",
    "opinasen": "opinar",
    "opinases": "opinar",
    "opinaste": "opinar",
    "opinasteis": "opinar",
    "opinativa": "opinativo",
    "opinativas": "opinativo",
    "opinativos": "opinativo",
    "opine": "opinar",
    "opinemos": "opinar",
    "opinen": "opinar",
    "opines": "opinar",
    "opiniones": "opinión",
    "opino": "opinar",
    "opinábamos": "opinar",
    "opináis": "opinar",
    "opináramos": "opinar",
    "opináremos": "opinar",
    "opinásemos": "opinar",
    "opiné": "opinar",
    "opinéis": "opinar",
    "opinó": "opinar",
    "opiomanías": "opiomanía",
    "opios": "opio",
    "opistognacias": "opistognacia",
    "opisómetros": "opisómetro",
    "opiácea": "opiáceo",
    "opiáceas": "opiáceo",
    "opiáceos": "opiáceo",
    "opiómana": "opiómano",
    "opiómanas": "opiómano",
    "opiómanos": "opiómano",
    "oplotecas": "oploteca",
    "opobálsamos": "opobálsamo",
    "opondremos": "oponer",
    "opondrá": "oponer",
    "opondrán": "oponer",
    "opondrás": "oponer",
    "opondré": "oponer",
    "opondréis": "oponer",
    "opondría": "oponer",
    "opondríais": "oponer",
    "opondríamos": "oponer",
    "opondrían": "oponer",
    "opondrías": "oponer",
    "opone": "oponer",
    "oponed": "oponer",
    "oponemos": "oponer",
    "oponen": "oponer",
    "oponentes": "oponente",
    "opones": "oponer",
    "oponga": "oponer",
    "opongamos": "oponer",
    "opongan": "oponer",
    "opongas": "oponer",
    "opongo": "oponer",
    "opongáis": "oponer",
    "oponibles": "oponible",
    "oponiendo": "oponer",
    "oponéis": "oponer",
    "oponía": "oponer",
    "oponíais": "oponer",
    "oponíamos": "oponer",
    "oponían": "oponer",
    "oponías": "oponer",
    "opopónacos": "opopónaco",
    "oportos": "oporto",
    "oportuna": "oportuno",
    "oportunas": "oportuno",
    "oportunidades": "oportunidad",
    "oportunismos": "oportunismo",
    "oportunistas": "oportunista",
    "oportunos": "oportuno",
    "oposiciones": "oposición",
    "oposicionistas": "oposicionista",
    "oposita": "opositar",
    "opositaba": "opositar",
    "opositabais": "opositar",
    "opositaban": "opositar",
    "opositabas": "opositar",
    "opositad": "opositar",
    "opositada": "opositar",
    "opositadas": "opositar",
    "opositado": "opositar",
    "opositados": "opositar",
    "opositamos": "opositar",
    "opositan": "opositar",
    "opositando": "opositar",
    "opositara": "opositar",
    "opositarais": "opositar",
    "opositaran": "opositar",
    "opositaras": "opositar",
    "opositare": "opositar",
    "opositareis": "opositar",
    "opositaremos": "opositar",
    "opositaren": "opositar",
    "opositares": "opositar",
    "opositaron": "opositar",
    "opositará": "opositar",
    "opositarán": "opositar",
    "opositarás": "opositar",
    "opositaré": "opositar",
    "opositaréis": "opositar",
    "opositaría": "opositar",
    "opositaríais": "opositar",
    "opositaríamos": "opositar",
    "opositarían": "opositar",
    "opositarías": "opositar",
    "opositas": "opositar",
    "opositase": "opositar",
    "opositaseis": "opositar",
    "opositasen": "opositar",
    "opositases": "opositar",
    "opositaste": "opositar",
    "opositasteis": "opositar",
    "oposite": "opositar",
    "opositemos": "opositar",
    "opositen": "opositar",
    "oposites": "opositar",
    "oposito": "opositar",
    "opositora": "opositor",
    "opositoras": "opositor",
    "opositores": "opositor",
    "opositábamos": "opositar",
    "opositáis": "opositar",
    "opositáramos": "opositar",
    "opositáremos": "opositar",
    "opositásemos": "opositar",
    "oposité": "opositar",
    "opositéis": "opositar",
    "opositó": "opositar",
    "oposums": "oposum",
    "opoterapias": "opoterapia",
    "opoterápica": "opoterápico",
    "opoterápicas": "opoterápico",
    "opoterápicos": "opoterápico",
    "opresa": "oprimir",
    "opresas": "oprimir",
    "opresiones": "opresión",
    "opresiva": "opresivo",
    "opresivas": "opresivo",
    "opresivos": "opresivo",
    "opreso": "oprimir",
    "opresora": "opresor",
    "opresoras": "opresor",
    "opresores": "opresor",
    "opresos": "oprimir",
    "oprima": "oprimir",
    "oprimamos": "oprimir",
    "opriman": "oprimir",
    "oprimas": "oprimir",
    "oprime": "oprimir",
    "oprimen": "oprimir",
    "oprimes": "oprimir",
    "oprimid": "oprimir",
    "oprimida": "oprimir",
    "oprimidas": "oprimir",
    "oprimido": "oprimir",
    "oprimidos": "oprimir",
    "oprimiendo": "oprimir",
    "oprimiera": "oprimir",
    "oprimierais": "oprimir",
    "oprimieran": "oprimir",
    "oprimieras": "oprimir",
    "oprimiere": "oprimir",
    "oprimiereis": "oprimir",
    "oprimieren": "oprimir",
    "oprimieres": "oprimir",
    "oprimieron": "oprimir",
    "oprimiese": "oprimir",
    "oprimieseis": "oprimir",
    "oprimiesen": "oprimir",
    "oprimieses": "oprimir",
    "oprimimos": "oprimir",
    "oprimiremos": "oprimir",
    "oprimirá": "oprimir",
    "oprimirán": "oprimir",
    "oprimirás": "oprimir",
    "oprimiré": "oprimir",
    "oprimiréis": "oprimir",
    "oprimiría": "oprimir",
    "oprimiríais": "oprimir",
    "oprimiríamos": "oprimir",
    "oprimirían": "oprimir",
    "oprimirías": "oprimir",
    "oprimiste": "oprimir",
    "oprimisteis": "oprimir",
    "oprimiéramos": "oprimir",
    "oprimiéremos": "oprimir",
    "oprimiésemos": "oprimir",
    "oprimió": "oprimir",
    "oprimo": "oprimir",
    "oprimáis": "oprimir",
    "oprimí": "oprimir",
    "oprimía": "oprimir",
    "oprimíais": "oprimir",
    "oprimíamos": "oprimir",
    "oprimían": "oprimir",
    "oprimías": "oprimir",
    "oprimís": "oprimir",
    "oprobia": "oprobiar",
    "oprobiaba": "oprobiar",
    "oprobiabais": "oprobiar",
    "oprobiaban": "oprobiar",
    "oprobiabas": "oprobiar",
    "oprobiad": "oprobiar",
    "oprobiada": "oprobiar",
    "oprobiadas": "oprobiar",
    "oprobiado": "oprobiar",
    "oprobiados": "oprobiar",
    "oprobiamos": "oprobiar",
    "oprobian": "oprobiar",
    "oprobiando": "oprobiar",
    "oprobiara": "oprobiar",
    "oprobiarais": "oprobiar",
    "oprobiaran": "oprobiar",
    "oprobiaras": "oprobiar",
    "oprobiare": "oprobiar",
    "oprobiareis": "oprobiar",
    "oprobiaremos": "oprobiar",
    "oprobiaren": "oprobiar",
    "oprobiares": "oprobiar",
    "oprobiaron": "oprobiar",
    "oprobiará": "oprobiar",
    "oprobiarán": "oprobiar",
    "oprobiarás": "oprobiar",
    "oprobiaré": "oprobiar",
    "oprobiaréis": "oprobiar",
    "oprobiaría": "oprobiar",
    "oprobiaríais": "oprobiar",
    "oprobiaríamos": "oprobiar",
    "oprobiarían": "oprobiar",
    "oprobiarías": "oprobiar",
    "oprobias": "oprobiar",
    "oprobiase": "oprobiar",
    "oprobiaseis": "oprobiar",
    "oprobiasen": "oprobiar",
    "oprobiases": "oprobiar",
    "oprobiaste": "oprobiar",
    "oprobiasteis": "oprobiar",
    "oprobie": "oprobiar",
    "oprobiemos": "oprobiar",
    "oprobien": "oprobiar",
    "oprobies": "oprobiar",
    "oprobio": "oprobiar",
    "oprobios": "oprobio",
    "oprobiosa": "oprobioso",
    "oprobiosas": "oprobioso",
    "oprobiosos": "oprobioso",
    "oprobiábamos": "oprobiar",
    "oprobiáis": "oprobiar",
    "oprobiáramos": "oprobiar",
    "oprobiáremos": "oprobiar",
    "oprobiásemos": "oprobiar",
    "oprobié": "oprobiar",
    "oprobiéis": "oprobiar",
    "oprobió": "oprobiar",
    "opsoclonías": "opsoclonía",
    "opsoninas": "opsonina",
    "opsonizaciones": "opsonización",
    "opta": "optar",
    "optaba": "optar",
    "optabais": "optar",
    "optaban": "optar",
    "optabas": "optar",
    "optaciones": "optación",
    "optad": "optar",
    "optada": "optar",
    "optadas": "optar",
    "optado": "optar",
    "optados": "optar",
    "optamos": "optar",
    "optan": "optar",
    "optando": "optar",
    "optara": "optar",
    "optarais": "optar",
    "optaran": "optar",
    "optaras": "optar",
    "optare": "optar",
    "optareis": "optar",
    "optaremos": "optar",
    "optaren": "optar",
    "optares": "optar",
    "optaron": "optar",
    "optará": "optar",
    "optarán": "optar",
    "optarás": "optar",
    "optaré": "optar",
    "optaréis": "optar",
    "optaría": "optar",
    "optaríais": "optar",
    "optaríamos": "optar",
    "optarían": "optar",
    "optarías": "optar",
    "optas": "optar",
    "optase": "optar",
    "optaseis": "optar",
    "optasen": "optar",
    "optases": "optar",
    "optaste": "optar",
    "optasteis": "optar",
    "optativa": "optativo",
    "optativas": "optativo",
    "optativos": "optativo",
    "opte": "optar",
    "optemos": "optar",
    "opten": "optar",
    "optes": "optar",
    "opticoestriada": "opticoestriado",
    "opticoestriadas": "opticoestriado",
    "opticoestriados": "opticoestriado",
    "opticógrafos": "opticógrafo",
    "optimaciones": "optimación",
    "optimales": "optimal",
    "optimates": "optimate",
    "optimice": "optimizar",
    "optimicemos": "optimizar",
    "optimicen": "optimizar",
    "optimices": "optimizar",
    "optimicé": "optimizar",
    "optimicéis": "optimizar",
    "optimismos": "optimismo",
    "optimistas": "optimista",
    "optimiza": "optimizar",
    "optimizaba": "optimizar",
    "optimizabais": "optimizar",
    "optimizaban": "optimizar",
    "optimizabas": "optimizar",
    "optimizaciones": "optimización",
    "optimizad": "optimizar",
    "optimizada": "optimizar",
    "optimizadas": "optimizar",
    "optimizado": "optimizar",
    "optimizadores": "optimizador",
    "optimizados": "optimizar",
    "optimizamos": "optimizar",
    "optimizan": "optimizar",
    "optimizando": "optimizar",
    "optimizara": "optimizar",
    "optimizarais": "optimizar",
    "optimizaran": "optimizar",
    "optimizaras": "optimizar",
    "optimizare": "optimizar",
    "optimizareis": "optimizar",
    "optimizaremos": "optimizar",
    "optimizaren": "optimizar",
    "optimizares": "optimizar",
    "optimizaron": "optimizar",
    "optimizará": "optimizar",
    "optimizarán": "optimizar",
    "optimizarás": "optimizar",
    "optimizaré": "optimizar",
    "optimizaréis": "optimizar",
    "optimizaría": "optimizar",
    "optimizaríais": "optimizar",
    "optimizaríamos": "optimizar",
    "optimizarían": "optimizar",
    "optimizarías": "optimizar",
    "optimizas": "optimizar",
    "optimizase": "optimizar",
    "optimizaseis": "optimizar",
    "optimizasen": "optimizar",
    "optimizases": "optimizar",
    "optimizaste": "optimizar",
    "optimizasteis": "optimizar",
    "optimizo": "optimizar",
    "optimizábamos": "optimizar",
    "optimizáis": "optimizar",
    "optimizáramos": "optimizar",
    "optimizáremos": "optimizar",
    "optimizásemos": "optimizar",
    "optimizó": "optimizar",
    "opto": "optar",
    "opto-electrónicas": "opto-electrónica",
    "optocinética": "optocinético",
    "optocinéticas": "optocinético",
    "optocinéticos": "optocinético",
    "optoelectrónica": "optoelectrónico",
    "optoelectrónicas": "optoelectrónico",
    "optoelectrónicos": "optoelectrónico",
    "optometristas": "optometrista",
    "optometrías": "optometría",
    "optomiómetros": "optomiómetro",
    "optométrica": "optométrico",
    "optométricas": "optométrico",
    "optométricos": "optométrico",
    "optotipos": "optotipo",
    "optábamos": "optar",
    "optáis": "optar",
    "optáramos": "optar",
    "optáremos": "optar",
    "optásemos": "optar",
    "opté": "optar",
    "optéis": "optar",
    "optó": "optar",
    "optómetros": "optómetro",
    "opuesta": "oponer",
    "opuestas": "oponer",
    "opuesto": "oponer",
    "opuestos": "oponer",
    "opugna": "opugnar",
    "opugnaba": "opugnar",
    "opugnabais": "opugnar",
    "opugnaban": "opugnar",
    "opugnabas": "opugnar",
    "opugnaciones": "opugnación",
    "opugnad": "opugnar",
    "opugnada": "opugnar",
    "opugnadas": "opugnar",
    "opugnado": "opugnar",
    "opugnadora": "opugnador",
    "opugnadoras": "opugnador",
    "opugnadores": "opugnador",
    "opugnados": "opugnar",
    "opugnamos": "opugnar",
    "opugnan": "opugnar",
    "opugnando": "opugnar",
    "opugnara": "opugnar",
    "opugnarais": "opugnar",
    "opugnaran": "opugnar",
    "opugnaras": "opugnar",
    "opugnare": "opugnar",
    "opugnareis": "opugnar",
    "opugnaremos": "opugnar",
    "opugnaren": "opugnar",
    "opugnares": "opugnar",
    "opugnaron": "opugnar",
    "opugnará": "opugnar",
    "opugnarán": "opugnar",
    "opugnarás": "opugnar",
    "opugnaré": "opugnar",
    "opugnaréis": "opugnar",
    "opugnaría": "opugnar",
    "opugnaríais": "opugnar",
    "opugnaríamos": "opugnar",
    "opugnarían": "opugnar",
    "opugnarías": "opugnar",
    "opugnas": "opugnar",
    "opugnase": "opugnar",
    "opugnaseis": "opugnar",
    "opugnasen": "opugnar",
    "opugnases": "opugnar",
    "opugnaste": "opugnar",
    "opugnasteis": "opugnar",
    "opugne": "opugnar",
    "opugnemos": "opugnar",
    "opugnen": "opugnar",
    "opugnes": "opugnar",
    "opugno": "opugnar",
    "opugnábamos": "opugnar",
    "opugnáis": "opugnar",
    "opugnáramos": "opugnar",
    "opugnáremos": "opugnar",
    "opugnásemos": "opugnar",
    "opugné": "opugnar",
    "opugnéis": "opugnar",
    "opugnó": "opugnar",
    "opulencias": "opulencia",
    "opulenta": "opulento",
    "opulentas": "opulento",
    "opulentos": "opulento",
    "opuse": "oponer",
    "opusiera": "oponer",
    "opusierais": "oponer",
    "opusieran": "oponer",
    "opusieras": "oponer",
    "opusiere": "oponer",
    "opusiereis": "oponer",
    "opusieren": "oponer",
    "opusieres": "oponer",
    "opusieron": "oponer",
    "opusiese": "oponer",
    "opusieseis": "oponer",
    "opusiesen": "oponer",
    "opusieses": "oponer",
    "opusimos": "oponer",
    "opusiste": "oponer",
    "opusisteis": "oponer",
    "opusiéramos": "oponer",
    "opusiéremos": "oponer",
    "opusiésemos": "oponer",
    "opuso": "oponer",
    "opérculos": "opérculo",
    "opípara": "opíparo",
    "opíparas": "opíparo",
    "opíparos": "opíparo",
    "opón": "oponer",
    "opósita": "oponer",
    "opósitas": "oponer",
    "opósito": "oponer",
    "opósitos": "oponer",
    "opósums": "opósum",
    "opúsculos": "opúsculo",
    "oquedades": "oquedad",
    "oquedales": "oquedal",
    "ora": "orar",
    "oraba": "orar",
    "orabais": "orar",
    "oraban": "orar",
    "orabas": "orar",
    "oracionales": "oracional",
    "oraciones": "oración",
    "orad": "orar",
    "orada": "orar",
    "oradas": "orar",
    "orado": "orar",
    "oradora": "orador",
    "oradoras": "orador",
    "oradores": "orador",
    "orados": "orar",
    "orales": "oral",
    "oralidades": "oralidad",
    "oramos": "orar",
    "oran": "orar",
    "orando": "orar",
    "orangistas": "orangista",
    "orangutana": "orangután",
    "orangutanas": "orangután",
    "orangutanes": "orangután",
    "orantes": "orante",
    "orara": "orar",
    "orarais": "orar",
    "oraran": "orar",
    "oraras": "orar",
    "orare": "orar",
    "orareis": "orar",
    "oraremos": "orar",
    "oraren": "orar",
    "orares": "orar",
    "oraron": "orar",
    "orará": "orar",
    "orarán": "orar",
    "orarás": "orar",
    "oraré": "orar",
    "oraréis": "orar",
    "oraría": "orar",
    "oraríais": "orar",
    "oraríamos": "orar",
    "orarían": "orar",
    "orarías": "orar",
    "oras": "orar",
    "orase": "orar",
    "oraseis": "orar",
    "orasen": "orar",
    "orases": "orar",
    "oraste": "orar",
    "orasteis": "orar",
    "orates": "orate",
    "oratios": "oratio",
    "oratoria": "oratorio",
    "oratoriana": "oratoriano",
    "oratorianas": "oratoriano",
    "oratorianos": "oratoriano",
    "oratorias": "oratorio",
    "oratorios": "oratorio",
    "orbes": "orbe",
    "orbiculares": "orbicular",
    "orbita": "orbitar",
    "orbitaba": "orbitar",
    "orbitabais": "orbitar",
    "orbitaban": "orbitar",
    "orbitabas": "orbitar",
    "orbitad": "orbitar",
    "orbitada": "orbitar",
    "orbitadas": "orbitar",
    "orbitado": "orbitar",
    "orbitados": "orbitar",
    "orbitales": "orbital",
    "orbitamos": "orbitar",
    "orbitan": "orbitar",
    "orbitando": "orbitar",
    "orbitara": "orbitar",
    "orbitarais": "orbitar",
    "orbitaran": "orbitar",
    "orbitaras": "orbitar",
    "orbitare": "orbitar",
    "orbitareis": "orbitar",
    "orbitaremos": "orbitar",
    "orbitaren": "orbitar",
    "orbitares": "orbitar",
    "orbitaria": "orbitario",
    "orbitarias": "orbitario",
    "orbitarios": "orbitario",
    "orbitaron": "orbitar",
    "orbitará": "orbitar",
    "orbitarán": "orbitar",
    "orbitarás": "orbitar",
    "orbitaré": "orbitar",
    "orbitaréis": "orbitar",
    "orbitaría": "orbitar",
    "orbitaríais": "orbitar",
    "orbitaríamos": "orbitar",
    "orbitarían": "orbitar",
    "orbitarías": "orbitar",
    "orbitas": "orbitar",
    "orbitase": "orbitar",
    "orbitaseis": "orbitar",
    "orbitasen": "orbitar",
    "orbitases": "orbitar",
    "orbitaste": "orbitar",
    "orbitasteis": "orbitar",
    "orbite": "orbitar",
    "orbitemos": "orbitar",
    "orbiten": "orbitar",
    "orbites": "orbitar",
    "orbito": "orbitar",
    "orbitonasales": "orbitonasal",
    "orbitonometrías": "orbitonometría",
    "orbitonómetros": "orbitonómetro",
    "orbitotomías": "orbitotomía",
    "orbitábamos": "orbitar",
    "orbitáis": "orbitar",
    "orbitáramos": "orbitar",
    "orbitáremos": "orbitar",
    "orbitásemos": "orbitar",
    "orbité": "orbitar",
    "orbitéis": "orbitar",
    "orbitó": "orbitar",
    "orcanetas": "orcaneta",
    "orcas": "orca",
    "orce": "orzar",
    "orcemos": "orzar",
    "orcen": "orzar",
    "orces": "orzar",
    "orchestras": "orchestra",
    "orcinas": "orcina",
    "orcé": "orzar",
    "orcéis": "orzar",
    "ordalías": "ordalía",
    "ordena": "ordenar",
    "ordenaba": "ordenar",
    "ordenabais": "ordenar",
    "ordenaban": "ordenar",
    "ordenabas": "ordenar",
    "ordenaciones": "ordenación",
    "ordenad": "ordenar",
    "ordenada": "ordenar",
    "ordenadas": "ordenar",
    "ordenado": "ordenar",
    "ordenadora": "ordenador",
    "ordenadoras": "ordenador",
    "ordenadores": "ordenador",
    "ordenados": "ordenar",
    "ordenamientos": "ordenamiento",
    "ordenamos": "ordenar",
    "ordenan": "ordenar",
    "ordenancistas": "ordenancista",
    "ordenando": "ordenar",
    "ordenandos": "ordenando",
    "ordenantes": "ordenante",
    "ordenanzas": "ordenanza",
    "ordenara": "ordenar",
    "ordenarais": "ordenar",
    "ordenaran": "ordenar",
    "ordenaras": "ordenar",
    "ordenare": "ordenar",
    "ordenareis": "ordenar",
    "ordenaremos": "ordenar",
    "ordenaren": "ordenar",
    "ordenares": "ordenar",
    "ordenaron": "ordenar",
    "ordenará": "ordenar",
    "ordenarán": "ordenar",
    "ordenarás": "ordenar",
    "ordenaré": "ordenar",
    "ordenaréis": "ordenar",
    "ordenaría": "ordenar",
    "ordenaríais": "ordenar",
    "ordenaríamos": "ordenar",
    "ordenarían": "ordenar",
    "ordenarías": "ordenar",
    "ordenas": "ordenar",
    "ordenase": "ordenar",
    "ordenaseis": "ordenar",
    "ordenasen": "ordenar",
    "ordenases": "ordenar",
    "ordenaste": "ordenar",
    "ordenasteis": "ordenar",
    "ordene": "ordenar",
    "ordenemos": "ordenar",
    "ordenen": "ordenar",
    "ordenes": "ordenar",
    "ordeno": "ordenar",
    "ordenábamos": "ordenar",
    "ordenáis": "ordenar",
    "ordenáramos": "ordenar",
    "ordenáremos": "ordenar",
    "ordenásemos": "ordenar",
    "ordené": "ordenar",
    "ordenéis": "ordenar",
    "ordenó": "ordenar",
    "ordeña": "ordeñar",
    "ordeñaba": "ordeñar",
    "ordeñabais": "ordeñar",
    "ordeñaban": "ordeñar",
    "ordeñabas": "ordeñar",
    "ordeñad": "ordeñar",
    "ordeñada": "ordeñar",
    "ordeñadas": "ordeñar",
    "ordeñaderos": "ordeñadero",
    "ordeñado": "ordeñar",
    "ordeñadora": "ordeñador",
    "ordeñadoras": "ordeñadora",
    "ordeñadores": "ordeñador",
    "ordeñados": "ordeñar",
    "ordeñamos": "ordeñar",
    "ordeñan": "ordeñar",
    "ordeñando": "ordeñar",
    "ordeñara": "ordeñar",
    "ordeñarais": "ordeñar",
    "ordeñaran": "ordeñar",
    "ordeñaras": "ordeñar",
    "ordeñare": "ordeñar",
    "ordeñareis": "ordeñar",
    "ordeñaremos": "ordeñar",
    "ordeñaren": "ordeñar",
    "ordeñares": "ordeñar",
    "ordeñaron": "ordeñar",
    "ordeñará": "ordeñar",
    "ordeñarán": "ordeñar",
    "ordeñarás": "ordeñar",
    "ordeñaré": "ordeñar",
    "ordeñaréis": "ordeñar",
    "ordeñaría": "ordeñar",
    "ordeñaríais": "ordeñar",
    "ordeñaríamos": "ordeñar",
    "ordeñarían": "ordeñar",
    "ordeñarías": "ordeñar",
    "ordeñas": "ordeñar",
    "ordeñase": "ordeñar",
    "ordeñaseis": "ordeñar",
    "ordeñasen": "ordeñar",
    "ordeñases": "ordeñar",
    "ordeñaste": "ordeñar",
    "ordeñasteis": "ordeñar",
    "ordeñe": "ordeñar",
    "ordeñemos": "ordeñar",
    "ordeñen": "ordeñar",
    "ordeñes": "ordeñar",
    "ordeño": "ordeñar",
    "ordeños": "ordeño",
    "ordeñábamos": "ordeñar",
    "ordeñáis": "ordeñar",
    "ordeñáramos": "ordeñar",
    "ordeñáremos": "ordeñar",
    "ordeñásemos": "ordeñar",
    "ordeñé": "ordeñar",
    "ordeñéis": "ordeñar",
    "ordeñó": "ordeñar",
    "ordinales": "ordinal",
    "ordinaria": "ordinario",
    "ordinarias": "ordinario",
    "ordinarieces": "ordinariez",
    "ordinarios": "ordinario",
    "ordinogramas": "ordinograma",
    "ordovicienses": "ordoviciense",
    "ore": "orar",
    "orea": "orear",
    "oreaba": "orear",
    "oreabais": "orear",
    "oreaban": "orear",
    "oreabas": "orear",
    "oread": "orear",
    "oreada": "orear",
    "oreadas": "orear",
    "oreado": "orear",
    "oreados": "orear",
    "oreamos": "orear",
    "orean": "orear",
    "oreando": "orear",
    "oreara": "orear",
    "orearais": "orear",
    "orearan": "orear",
    "orearas": "orear",
    "oreare": "orear",
    "oreareis": "orear",
    "orearemos": "orear",
    "orearen": "orear",
    "oreares": "orear",
    "orearon": "orear",
    "oreará": "orear",
    "orearán": "orear",
    "orearás": "orear",
    "orearé": "orear",
    "orearéis": "orear",
    "orearía": "orear",
    "orearíais": "orear",
    "orearíamos": "orear",
    "orearían": "orear",
    "orearías": "orear",
    "oreas": "orear",
    "orease": "orear",
    "oreaseis": "orear",
    "oreasen": "orear",
    "oreases": "orear",
    "oreaste": "orear",
    "oreasteis": "orear",
    "oree": "orear",
    "oreemos": "orear",
    "oreen": "orear",
    "orees": "orear",
    "orejana": "orejano",
    "orejanas": "orejano",
    "orejanos": "orejano",
    "orejas": "oreja",
    "orejea": "orejear",
    "orejeaba": "orejear",
    "orejeabais": "orejear",
    "orejeaban": "orejear",
    "orejeabas": "orejear",
    "orejead": "orejear",
    "orejeada": "orejear",
    "orejeadas": "orejear",
    "orejeado": "orejear",
    "orejeados": "orejear",
    "orejeamos": "orejear",
    "orejean": "orejear",
    "orejeando": "orejear",
    "orejeara": "orejear",
    "orejearais": "orejear",
    "orejearan": "orejear",
    "orejearas": "orejear",
    "orejeare": "orejear",
    "orejeareis": "orejear",
    "orejearemos": "orejear",
    "orejearen": "orejear",
    "orejeares": "orejear",
    "orejearon": "orejear",
    "orejeará": "orejear",
    "orejearán": "orejear",
    "orejearás": "orejear",
    "orejearé": "orejear",
    "orejearéis": "orejear",
    "orejearía": "orejear",
    "orejearíais": "orejear",
    "orejearíamos": "orejear",
    "orejearían": "orejear",
    "orejearías": "orejear",
    "orejeas": "orejear",
    "orejease": "orejear",
    "orejeaseis": "orejear",
    "orejeasen": "orejear",
    "orejeases": "orejear",
    "orejeaste": "orejear",
    "orejeasteis": "orejear",
    "orejee": "orejear",
    "orejeemos": "orejear",
    "orejeen": "orejear",
    "orejees": "orejear",
    "orejeo": "orejear",
    "orejera": "orejero",
    "orejeras": "orejero",
    "orejeros": "orejero",
    "orejetas": "orejeta",
    "orejeábamos": "orejear",
    "orejeáis": "orejear",
    "orejeáramos": "orejear",
    "orejeáremos": "orejear",
    "orejeásemos": "orejear",
    "orejeé": "orejear",
    "orejeéis": "orejear",
    "orejeó": "orejear",
    "orejisana": "orejisano",
    "orejisanas": "orejisano",
    "orejisanos": "orejisano",
    "orejones": "orejón",
    "orejuda": "orejudo",
    "orejudas": "orejudo",
    "orejudos": "orejudo",
    "orejuelas": "orejuela",
    "oremos": "orar",
    "oren": "orar",
    "orensana": "orensano",
    "orensanas": "orensano",
    "orensanos": "orensano",
    "oreo": "orear",
    "oreos": "oreo",
    "oreoselinos": "oreoselino",
    "ores": "orar",
    "oreábamos": "orear",
    "oreáis": "orear",
    "oreáramos": "orear",
    "oreáremos": "orear",
    "oreásemos": "orear",
    "oreé": "orear",
    "oreéis": "orear",
    "oreó": "orear",
    "orfanatos": "orfanato",
    "orfandades": "orfandad",
    "orfebrerías": "orfebrería",
    "orfebres": "orfebre",
    "orfelinatos": "orfelinato",
    "orfeones": "orfeón",
    "orfeonistas": "orfeonista",
    "orfeónica": "orfeónico",
    "orfeónicas": "orfeónico",
    "orfeónicos": "orfeónico",
    "orfismos": "orfismo",
    "organdises": "organdí",
    "organdíes": "organdí",
    "organdís": "organdí",
    "organera": "organero",
    "organeras": "organero",
    "organeros": "organero",
    "organice": "organizar",
    "organicemos": "organizar",
    "organicen": "organizar",
    "organices": "organizar",
    "organicismos": "organicismo",
    "organicistas": "organicista",
    "organicé": "organizar",
    "organicéis": "organizar",
    "organigramas": "organigrama",
    "organillera": "organillero",
    "organilleras": "organillero",
    "organilleros": "organillero",
    "organillos": "organillo",
    "organismos": "organismo",
    "organistas": "organista",
    "organiza": "organizar",
    "organizaba": "organizar",
    "organizabais": "organizar",
    "organizaban": "organizar",
    "organizabas": "organizar",
    "organizacionales": "organizacional",
    "organizaciones": "organización",
    "organizad": "organizar",
    "organizada": "organizar",
    "organizadas": "organizar",
    "organizado": "organizar",
    "organizadora": "organizador",
    "organizadoras": "organizador",
    "organizadores": "organizador",
    "organizados": "organizar",
    "organizamos": "organizar",
    "organizan": "organizar",
    "organizando": "organizar",
    "organizara": "organizar",
    "organizarais": "organizar",
    "organizaran": "organizar",
    "organizaras": "organizar",
    "organizare": "organizar",
    "organizareis": "organizar",
    "organizaremos": "organizar",
    "organizaren": "organizar",
    "organizares": "organizar",
    "organizaron": "organizar",
    "organizará": "organizar",
    "organizarán": "organizar",
    "organizarás": "organizar",
    "organizaré": "organizar",
    "organizaréis": "organizar",
    "organizaría": "organizar",
    "organizaríais": "organizar",
    "organizaríamos": "organizar",
    "organizarían": "organizar",
    "organizarías": "organizar",
    "organizas": "organizar",
    "organizase": "organizar",
    "organizaseis": "organizar",
    "organizasen": "organizar",
    "organizases": "organizar",
    "organizaste": "organizar",
    "organizasteis": "organizar",
    "organizativa": "organizativo",
    "organizativas": "organizativo",
    "organizativos": "organizativo",
    "organizo": "organizar",
    "organizábamos": "organizar",
    "organizáis": "organizar",
    "organizáramos": "organizar",
    "organizáremos": "organizar",
    "organizásemos": "organizar",
    "organizó": "organizar",
    "organoclorada": "organoclorado",
    "organocloradas": "organoclorado",
    "organoclorados": "organoclorado",
    "organogenias": "organogenia",
    "organografías": "organografía",
    "organográfica": "organográfico",
    "organográficas": "organográfico",
    "organográficos": "organográfico",
    "organologías": "organología",
    "organoléptica": "organoléptico",
    "organolépticas": "organoléptico",
    "organolépticos": "organoléptico",
    "organomegalias": "organomegalia",
    "organometálica": "organometálico",
    "organometálicas": "organometálico",
    "organometálicos": "organometálico",
    "organosoles": "organosol",
    "organoterapias": "organoterapia",
    "organotropismos": "organotropismo",
    "organzas": "organza",
    "organógena": "organógeno",
    "organógenas": "organógeno",
    "organógenos": "organógeno",
    "orgasmos": "orgasmo",
    "orgiástica": "orgiástico",
    "orgiásticas": "orgiástico",
    "orgiásticos": "orgiástico",
    "orgullos": "orgullo",
    "orgullosa": "orgulloso",
    "orgullosas": "orgulloso",
    "orgullosos": "orgulloso",
    "orgánica": "orgánico",
    "orgánicas": "orgánico",
    "orgánicos": "orgánico",
    "orgánulos": "orgánulo",
    "orgásmica": "orgásmico",
    "orgásmicas": "orgásmico",
    "orgásmicos": "orgásmico",
    "orgástica": "orgástico",
    "orgásticas": "orgástico",
    "orgásticos": "orgástico",
    "orgías": "orgía",
    "orienta": "orientar",
    "orientaba": "orientar",
    "orientabais": "orientar",
    "orientaban": "orientar",
    "orientabas": "orientar",
    "orientables": "orientable",
    "orientaciones": "orientación",
    "orientad": "orientar",
    "orientada": "orientar",
    "orientadas": "orientar",
    "orientado": "orientar",
    "orientadora": "orientador",
    "orientadoras": "orientador",
    "orientadores": "orientador",
    "orientados": "orientar",
    "orientales": "oriental",
    "orientalice": "orientalizar",
    "orientalicemos": "orientalizar",
    "orientalicen": "orientalizar",
    "orientalices": "orientalizar",
    "orientalicé": "orientalizar",
    "orientalicéis": "orientalizar",
    "orientalismos": "orientalismo",
    "orientalistas": "orientalista",
    "orientaliza": "orientalizar",
    "orientalizaba": "orientalizar",
    "orientalizabais": "orientalizar",
    "orientalizaban": "orientalizar",
    "orientalizabas": "orientalizar",
    "orientalizad": "orientalizar",
    "orientalizada": "orientalizar",
    "orientalizadas": "orientalizar",
    "orientalizado": "orientalizar",
    "orientalizados": "orientalizar",
    "orientalizamos": "orientalizar",
    "orientalizan": "orientalizar",
    "orientalizando": "orientalizar",
    "orientalizara": "orientalizar",
    "orientalizarais": "orientalizar",
    "orientalizaran": "orientalizar",
    "orientalizaras": "orientalizar",
    "orientalizare": "orientalizar",
    "orientalizareis": "orientalizar",
    "orientalizaremos": "orientalizar",
    "orientalizaren": "orientalizar",
    "orientalizares": "orientalizar",
    "orientalizaron": "orientalizar",
    "orientalizará": "orientalizar",
    "orientalizarán": "orientalizar",
    "orientalizarás": "orientalizar",
    "orientalizaré": "orientalizar",
    "orientalizaréis": "orientalizar",
    "orientalizaría": "orientalizar",
    "orientalizaríais": "orientalizar",
    "orientalizaríamos": "orientalizar",
    "orientalizarían": "orientalizar",
    "orientalizarías": "orientalizar",
    "orientalizas": "orientalizar",
    "orientalizase": "orientalizar",
    "orientalizaseis": "orientalizar",
    "orientalizasen": "orientalizar",
    "orientalizases": "orientalizar",
    "orientalizaste": "orientalizar",
    "orientalizasteis": "orientalizar",
    "orientalizo": "orientalizar",
    "orientalizábamos": "orientalizar",
    "orientalizáis": "orientalizar",
    "orientalizáramos": "orientalizar",
    "orientalizáremos": "orientalizar",
    "orientalizásemos": "orientalizar",
    "orientalizó": "orientalizar",
    "orientamos": "orientar",
    "orientan": "orientar",
    "orientando": "orientar",
    "orientara": "orientar",
    "orientarais": "orientar",
    "orientaran": "orientar",
    "orientaras": "orientar",
    "orientare": "orientar",
    "orientareis": "orientar",
    "orientaremos": "orientar",
    "orientaren": "orientar",
    "orientares": "orientar",
    "orientaron": "orientar",
    "orientará": "orientar",
    "orientarán": "orientar",
    "orientarás": "orientar",
    "orientaré": "orientar",
    "orientaréis": "orientar",
    "orientaría": "orientar",
    "orientaríais": "orientar",
    "orientaríamos": "orientar",
    "orientarían": "orientar",
    "orientarías": "orientar",
    "orientas": "orientar",
    "orientase": "orientar",
    "orientaseis": "orientar",
    "orientasen": "orientar",
    "orientases": "orientar",
    "orientaste": "orientar",
    "orientasteis": "orientar",
    "orientativa": "orientativo",
    "orientativas": "orientativo",
    "orientativos": "orientativo",
    "oriente": "orientar",
    "orientemos": "orientar",
    "orienten": "orientar",
    "orientes": "oriente",
    "oriento": "orientar",
    "orientábamos": "orientar",
    "orientáis": "orientar",
    "orientáramos": "orientar",
    "orientáremos": "orientar",
    "orientásemos": "orientar",
    "orienté": "orientar",
    "orientéis": "orientar",
    "orientó": "orientar",
    "orifica": "orificar",
    "orificaba": "orificar",
    "orificabais": "orificar",
    "orificaban": "orificar",
    "orificabas": "orificar",
    "orificaciones": "orificación",
    "orificad": "orificar",
    "orificada": "orificar",
    "orificadas": "orificar",
    "orificado": "orificar",
    "orificadores": "orificador",
    "orificados": "orificar",
    "orificamos": "orificar",
    "orifican": "orificar",
    "orificando": "orificar",
    "orificara": "orificar",
    "orificarais": "orificar",
    "orificaran": "orificar",
    "orificaras": "orificar",
    "orificare": "orificar",
    "orificareis": "orificar",
    "orificaremos": "orificar",
    "orificaren": "orificar",
    "orificares": "orificar",
    "orificaron": "orificar",
    "orificará": "orificar",
    "orificarán": "orificar",
    "orificarás": "orificar",
    "orificaré": "orificar",
    "orificaréis": "orificar",
    "orificaría": "orificar",
    "orificaríais": "orificar",
    "orificaríamos": "orificar",
    "orificarían": "orificar",
    "orificarías": "orificar",
    "orificas": "orificar",
    "orificase": "orificar",
    "orificaseis": "orificar",
    "orificasen": "orificar",
    "orificases": "orificar",
    "orificaste": "orificar",
    "orificasteis": "orificar",
    "orificios": "orificio",
    "orifico": "orificar",
    "orificábamos": "orificar",
    "orificáis": "orificar",
    "orificáramos": "orificar",
    "orificáremos": "orificar",
    "orificásemos": "orificar",
    "orificó": "orificar",
    "orifique": "orificar",
    "orifiquemos": "orificar",
    "orifiquen": "orificar",
    "orifiques": "orificar",
    "orifiqué": "orificar",
    "orifiquéis": "orificar",
    "oriflamas": "oriflama",
    "orifreses": "orifrés",
    "origenismos": "origenismo",
    "origenistas": "origenista",
    "origina": "originar",
    "originaba": "originar",
    "originabais": "originar",
    "originaban": "originar",
    "originabas": "originar",
    "originad": "originar",
    "originada": "originar",
    "originadas": "originar",
    "originado": "originar",
    "originadora": "originador",
    "originadoras": "originador",
    "originadores": "originador",
    "originados": "originar",
    "originales": "original",
    "originalidades": "originalidad",
    "originamos": "originar",
    "originan": "originar",
    "originando": "originar",
    "originara": "originar",
    "originarais": "originar",
    "originaran": "originar",
    "originaras": "originar",
    "originare": "originar",
    "originareis": "originar",
    "originaremos": "originar",
    "originaren": "originar",
    "originares": "originar",
    "originaria": "originario",
    "originarias": "originario",
    "originarios": "originario",
    "originaron": "originar",
    "originará": "originar",
    "originarán": "originar",
    "originarás": "originar",
    "originaré": "originar",
    "originaréis": "originar",
    "originaría": "originar",
    "originaríais": "originar",
    "originaríamos": "originar",
    "originarían": "originar",
    "originarías": "originar",
    "originas": "originar",
    "originase": "originar",
    "originaseis": "originar",
    "originasen": "originar",
    "originases": "originar",
    "originaste": "originar",
    "originasteis": "originar",
    "origine": "originar",
    "originemos": "originar",
    "originen": "originar",
    "origines": "originar",
    "origino": "originar",
    "originábamos": "originar",
    "origináis": "originar",
    "origináramos": "originar",
    "origináremos": "originar",
    "originásemos": "originar",
    "originé": "originar",
    "originéis": "originar",
    "originó": "originar",
    "orilla": "orillar",
    "orillaba": "orillar",
    "orillabais": "orillar",
    "orillaban": "orillar",
    "orillabas": "orillar",
    "orillad": "orillar",
    "orillada": "orillar",
    "orilladas": "orillar",
    "orillado": "orillar",
    "orillados": "orillar",
    "orillamos": "orillar",
    "orillan": "orillar",
    "orillando": "orillar",
    "orillara": "orillar",
    "orillarais": "orillar",
    "orillaran": "orillar",
    "orillaras": "orillar",
    "orillare": "orillar",
    "orillareis": "orillar",
    "orillaremos": "orillar",
    "orillaren": "orillar",
    "orillares": "orillar",
    "orillaron": "orillar",
    "orillará": "orillar",
    "orillarán": "orillar",
    "orillarás": "orillar",
    "orillaré": "orillar",
    "orillaréis": "orillar",
    "orillaría": "orillar",
    "orillaríais": "orillar",
    "orillaríamos": "orillar",
    "orillarían": "orillar",
    "orillarías": "orillar",
    "orillas": "orillar",
    "orillase": "orillar",
    "orillaseis": "orillar",
    "orillasen": "orillar",
    "orillases": "orillar",
    "orillaste": "orillar",
    "orillasteis": "orillar",
    "orille": "orillar",
    "orillemos": "orillar",
    "orillen": "orillar",
    "orilles": "orillar",
    "orillo": "orillar",
    "orillos": "orillo",
    "orillábamos": "orillar",
    "orilláis": "orillar",
    "orilláramos": "orillar",
    "orilláremos": "orillar",
    "orillásemos": "orillar",
    "orillé": "orillar",
    "orilléis": "orillar",
    "orilló": "orillar",
    "orina": "orinar",
    "orinaba": "orinar",
    "orinabais": "orinar",
    "orinaban": "orinar",
    "orinabas": "orinar",
    "orinad": "orinar",
    "orinada": "orinar",
    "orinadas": "orinar",
    "orinado": "orinar",
    "orinados": "orinar",
    "orinales": "orinal",
    "orinamos": "orinar",
    "orinan": "orinar",
    "orinando": "orinar",
    "orinara": "orinar",
    "orinarais": "orinar",
    "orinaran": "orinar",
    "orinaras": "orinar",
    "orinare": "orinar",
    "orinareis": "orinar",
    "orinaremos": "orinar",
    "orinaren": "orinar",
    "orinares": "orinar",
    "orinaron": "orinar",
    "orinará": "orinar",
    "orinarán": "orinar",
    "orinarás": "orinar",
    "orinaré": "orinar",
    "orinaréis": "orinar",
    "orinaría": "orinar",
    "orinaríais": "orinar",
    "orinaríamos": "orinar",
    "orinarían": "orinar",
    "orinarías": "orinar",
    "orinas": "orinar",
    "orinase": "orinar",
    "orinaseis": "orinar",
    "orinasen": "orinar",
    "orinases": "orinar",
    "orinaste": "orinar",
    "orinasteis": "orinar",
    "orine": "orinar",
    "orinemos": "orinar",
    "orinen": "orinar",
    "orines": "orinar",
    "orinienta": "oriniento",
    "orinientas": "oriniento",
    "orinientos": "oriniento",
    "orino": "orinar",
    "orinques": "orinque",
    "orinábamos": "orinar",
    "orináis": "orinar",
    "orináramos": "orinar",
    "orináremos": "orinar",
    "orinásemos": "orinar",
    "oriné": "orinar",
    "orinéis": "orinar",
    "orinó": "orinar",
    "oriunda": "oriundo",
    "oriundas": "oriundo",
    "oriundeces": "oriundez",
    "oriundos": "oriundo",
    "orla": "orlar",
    "orlaba": "orlar",
    "orlabais": "orlar",
    "orlaban": "orlar",
    "orlabas": "orlar",
    "orlad": "orlar",
    "orlada": "orlar",
    "orladas": "orlar",
    "orlado": "orlar",
    "orladora": "orlador",
    "orladoras": "orlador",
    "orladores": "orlador",
    "orlados": "orlar",
    "orladuras": "orladura",
    "orlamos": "orlar",
    "orlan": "orlar",
    "orlando": "orlar",
    "orlara": "orlar",
    "orlarais": "orlar",
    "orlaran": "orlar",
    "orlaras": "orlar",
    "orlare": "orlar",
    "orlareis": "orlar",
    "orlaremos": "orlar",
    "orlaren": "orlar",
    "orlares": "orlar",
    "orlaron": "orlar",
    "orlará": "orlar",
    "orlarán": "orlar",
    "orlarás": "orlar",
    "orlaré": "orlar",
    "orlaréis": "orlar",
    "orlaría": "orlar",
    "orlaríais": "orlar",
    "orlaríamos": "orlar",
    "orlarían": "orlar",
    "orlarías": "orlar",
    "orlas": "orlar",
    "orlase": "orlar",
    "orlaseis": "orlar",
    "orlasen": "orlar",
    "orlases": "orlar",
    "orlaste": "orlar",
    "orlasteis": "orlar",
    "orle": "orlar",
    "orleanismos": "orleanismo",
    "orleanistas": "orleanista",
    "orlemos": "orlar",
    "orlen": "orlar",
    "orles": "orlar",
    "orlo": "orlar",
    "orlos": "orlo",
    "orlábamos": "orlar",
    "orláis": "orlar",
    "orláramos": "orlar",
    "orláremos": "orlar",
    "orlásemos": "orlar",
    "orlé": "orlar",
    "orléis": "orlar",
    "orló": "orlar",
    "ormesíes": "ormesí",
    "ormesís": "ormesí",
    "orna": "ornar",
    "ornaba": "ornar",
    "ornabais": "ornar",
    "ornaban": "ornar",
    "ornabas": "ornar",
    "ornad": "ornar",
    "ornada": "ornar",
    "ornadas": "ornar",
    "ornado": "ornar",
    "ornados": "ornar",
    "ornamenta": "ornamentar",
    "ornamentaba": "ornamentar",
    "ornamentabais": "ornamentar",
    "ornamentaban": "ornamentar",
    "ornamentabas": "ornamentar",
    "ornamentaciones": "ornamentación",
    "ornamentad": "ornamentar",
    "ornamentada": "ornamentar",
    "ornamentadas": "ornamentar",
    "ornamentado": "ornamentar",
    "ornamentados": "ornamentar",
    "ornamentales": "ornamental",
    "ornamentamos": "ornamentar",
    "ornamentan": "ornamentar",
    "ornamentando": "ornamentar",
    "ornamentara": "ornamentar",
    "ornamentarais": "ornamentar",
    "ornamentaran": "ornamentar",
    "ornamentaras": "ornamentar",
    "ornamentare": "ornamentar",
    "ornamentareis": "ornamentar",
    "ornamentaremos": "ornamentar",
    "ornamentaren": "ornamentar",
    "ornamentares": "ornamentar",
    "ornamentaron": "ornamentar",
    "ornamentará": "ornamentar",
    "ornamentarán": "ornamentar",
    "ornamentarás": "ornamentar",
    "ornamentaré": "ornamentar",
    "ornamentaréis": "ornamentar",
    "ornamentaría": "ornamentar",
    "ornamentaríais": "ornamentar",
    "ornamentaríamos": "ornamentar",
    "ornamentarían": "ornamentar",
    "ornamentarías": "ornamentar",
    "ornamentas": "ornamentar",
    "ornamentase": "ornamentar",
    "ornamentaseis": "ornamentar",
    "ornamentasen": "ornamentar",
    "ornamentases": "ornamentar",
    "ornamentaste": "ornamentar",
    "ornamentasteis": "ornamentar",
    "ornamente": "ornamentar",
    "ornamentemos": "ornamentar",
    "ornamenten": "ornamentar",
    "ornamentes": "ornamentar",
    "ornamento": "ornamentar",
    "ornamentos": "ornamento",
    "ornamentábamos": "ornamentar",
    "ornamentáis": "ornamentar",
    "ornamentáramos": "ornamentar",
    "ornamentáremos": "ornamentar",
    "ornamentásemos": "ornamentar",
    "ornamenté": "ornamentar",
    "ornamentéis": "ornamentar",
    "ornamentó": "ornamentar",
    "ornamos": "ornar",
    "ornan": "ornar",
    "ornando": "ornar",
    "ornara": "ornar",
    "ornarais": "ornar",
    "ornaran": "ornar",
    "ornaras": "ornar",
    "ornare": "ornar",
    "ornareis": "ornar",
    "ornaremos": "ornar",
    "ornaren": "ornar",
    "ornares": "ornar",
    "ornaron": "ornar",
    "ornará": "ornar",
    "ornarán": "ornar",
    "ornarás": "ornar",
    "ornaré": "ornar",
    "ornaréis": "ornar",
    "ornaría": "ornar",
    "ornaríais": "ornar",
    "ornaríamos": "ornar",
    "ornarían": "ornar",
    "ornarías": "ornar",
    "ornas": "ornar",
    "ornase": "ornar",
    "ornaseis": "ornar",
    "ornasen": "ornar",
    "ornases": "ornar",
    "ornaste": "ornar",
    "ornasteis": "ornar",
    "ornatos": "ornato",
    "orne": "ornar",
    "ornemos": "ornar",
    "ornen": "ornar",
    "ornes": "ornar",
    "ornitinas": "ornitina",
    "ornitologías": "ornitología",
    "ornitológica": "ornitológico",
    "ornitológicas": "ornitológico",
    "ornitológicos": "ornitológico",
    "ornitomancias": "ornitomancia",
    "ornitorrincos": "ornitorrinco",
    "ornitófila": "ornitófilo",
    "ornitófilas": "ornitófilo",
    "ornitófilos": "ornitófilo",
    "ornitóloga": "ornitólogo",
    "ornitólogas": "ornitólogo",
    "ornitólogos": "ornitólogo",
    "orno": "ornar",
    "ornos": "orno",
    "ornábamos": "ornar",
    "ornáis": "ornar",
    "ornáramos": "ornar",
    "ornáremos": "ornar",
    "ornásemos": "ornar",
    "orné": "ornar",
    "ornéis": "ornar",
    "ornó": "ornar",
    "oro": "orar",
    "orobancas": "orobanca",
    "orobancáceas": "orobancácea",
    "orofaringes": "orofaringe",
    "orofaringoscopias": "orofaringoscopia",
    "orofitos": "orofito",
    "orogenias": "orogenia",
    "orografías": "orografía",
    "orográfica": "orográfico",
    "orográficas": "orográfico",
    "orográficos": "orográfico",
    "orogénica": "orogénico",
    "orogénicas": "orogénico",
    "orogénicos": "orogénico",
    "orolinguales": "orolingual",
    "oromaxilares": "oromaxilar",
    "oronasales": "oronasal",
    "oronda": "orondo",
    "orondas": "orondo",
    "orondos": "orondo",
    "orones": "orón",
    "oronimias": "oronimia",
    "oronjas": "oronja",
    "oronímica": "oronímico",
    "oronímicas": "oronímico",
    "oronímicos": "oronímico",
    "oropelera": "oropelero",
    "oropeleras": "oropelero",
    "oropeleros": "oropelero",
    "oropeles": "oropel",
    "oropimentes": "oropimente",
    "oropéndolas": "oropéndola",
    "oros": "oro",
    "oroyas": "oroya",
    "orozuces": "orozuz",
    "orquesta": "orquestar",
    "orquestaba": "orquestar",
    "orquestabais": "orquestar",
    "orquestaban": "orquestar",
    "orquestabas": "orquestar",
    "orquestaciones": "orquestación",
    "orquestad": "orquestar",
    "orquestada": "orquestar",
    "orquestadas": "orquestar",
    "orquestado": "orquestar",
    "orquestados": "orquestar",
    "orquestales": "orquestal",
    "orquestamos": "orquestar",
    "orquestan": "orquestar",
    "orquestando": "orquestar",
    "orquestara": "orquestar",
    "orquestarais": "orquestar",
    "orquestaran": "orquestar",
    "orquestaras": "orquestar",
    "orquestare": "orquestar",
    "orquestareis": "orquestar",
    "orquestaremos": "orquestar",
    "orquestaren": "orquestar",
    "orquestares": "orquestar",
    "orquestaron": "orquestar",
    "orquestará": "orquestar",
    "orquestarán": "orquestar",
    "orquestarás": "orquestar",
    "orquestaré": "orquestar",
    "orquestaréis": "orquestar",
    "orquestaría": "orquestar",
    "orquestaríais": "orquestar",
    "orquestaríamos": "orquestar",
    "orquestarían": "orquestar",
    "orquestarías": "orquestar",
    "orquestas": "orquestar",
    "orquestase": "orquestar",
    "orquestaseis": "orquestar",
    "orquestasen": "orquestar",
    "orquestases": "orquestar",
    "orquestaste": "orquestar",
    "orquestasteis": "orquestar",
    "orqueste": "orquestar",
    "orquestemos": "orquestar",
    "orquesten": "orquestar",
    "orquestes": "orquestar",
    "orquestinas": "orquestina",
    "orquesto": "orquestar",
    "orquestábamos": "orquestar",
    "orquestáis": "orquestar",
    "orquestáramos": "orquestar",
    "orquestáremos": "orquestar",
    "orquestásemos": "orquestar",
    "orquesté": "orquestar",
    "orquestéis": "orquestar",
    "orquestó": "orquestar",
    "orquidáceas": "orquidácea",
    "orquídeas": "orquídea",
    "orsais": "orsai",
    "ortegadas": "ortegada",
    "ortegas": "ortega",
    "orticones": "orticón",
    "ortigales": "ortigal",
    "ortigas": "ortiga",
    "ortitas": "ortita",
    "ortiva": "ortivo",
    "ortivas": "ortivo",
    "ortivos": "ortivo",
    "ortocentros": "ortocentro",
    "ortoclasas": "ortoclasa",
    "ortocromática": "ortocromático",
    "ortocromáticas": "ortocromático",
    "ortocromáticos": "ortocromático",
    "ortocuarcitas": "ortocuarcita",
    "ortodentinas": "ortodentina",
    "ortodoncias": "ortodoncia",
    "ortodonta": "ortodonto",
    "ortodontas": "ortodonto",
    "ortodontologías": "ortodontología",
    "ortodontos": "ortodonto",
    "ortodoxa": "ortodoxo",
    "ortodoxas": "ortodoxo",
    "ortodoxias": "ortodoxia",
    "ortodoxos": "ortodoxo",
    "ortodromias": "ortodromia",
    "ortodrómica": "ortodrómico",
    "ortodrómicas": "ortodrómico",
    "ortodrómicos": "ortodrómico",
    "ortodóncica": "ortodóncico",
    "ortodóncicas": "ortodóncico",
    "ortodóncicos": "ortodóncico",
    "ortodóntica": "ortodóntico",
    "ortodónticas": "ortodóntico",
    "ortodónticos": "ortodóntico",
    "ortoedros": "ortoedro",
    "ortoepias": "ortoepia",
    "ortofonías": "ortofonía",
    "ortoformiatos": "ortoformiato",
    "ortofosfatos": "ortofosfato",
    "ortofosforosa": "ortofosforoso",
    "ortofosforosas": "ortofosforoso",
    "ortofosforosos": "ortofosforoso",
    "ortofotografías": "ortofotografía",
    "ortofotomapas": "ortofotomapa",
    "ortofónica": "ortofónico",
    "ortofónicas": "ortofónico",
    "ortofónicos": "ortofónico",
    "ortogenética": "ortogenético",
    "ortogenéticas": "ortogenético",
    "ortogenéticos": "ortogenético",
    "ortognata": "ortognato",
    "ortognatas": "ortognato",
    "ortognatodoncias": "ortognatodoncia",
    "ortognatos": "ortognato",
    "ortognática": "ortognático",
    "ortognáticas": "ortognático",
    "ortognáticos": "ortognático",
    "ortogonales": "ortogonal",
    "ortogonalidades": "ortogonalidad",
    "ortogonalizaciones": "ortogonalización",
    "ortografiaba": "ortografiar",
    "ortografiabais": "ortografiar",
    "ortografiaban": "ortografiar",
    "ortografiabas": "ortografiar",
    "ortografiad": "ortografiar",
    "ortografiada": "ortografiar",
    "ortografiadas": "ortografiar",
    "ortografiado": "ortografiar",
    "ortografiados": "ortografiar",
    "ortografiamos": "ortografiar",
    "ortografiando": "ortografiar",
    "ortografiara": "ortografiar",
    "ortografiarais": "ortografiar",
    "ortografiaran": "ortografiar",
    "ortografiaras": "ortografiar",
    "ortografiare": "ortografiar",
    "ortografiareis": "ortografiar",
    "ortografiaremos": "ortografiar",
    "ortografiaren": "ortografiar",
    "ortografiares": "ortografiar",
    "ortografiaron": "ortografiar",
    "ortografiará": "ortografiar",
    "ortografiarán": "ortografiar",
    "ortografiarás": "ortografiar",
    "ortografiaré": "ortografiar",
    "ortografiaréis": "ortografiar",
    "ortografiaría": "ortografiar",
    "ortografiaríais": "ortografiar",
    "ortografiaríamos": "ortografiar",
    "ortografiarían": "ortografiar",
    "ortografiarías": "ortografiar",
    "ortografiase": "ortografiar",
    "ortografiaseis": "ortografiar",
    "ortografiasen": "ortografiar",
    "ortografiases": "ortografiar",
    "ortografiaste": "ortografiar",
    "ortografiasteis": "ortografiar",
    "ortografiemos": "ortografiar",
    "ortografiábamos": "ortografiar",
    "ortografiáis": "ortografiar",
    "ortografiáramos": "ortografiar",
    "ortografiáremos": "ortografiar",
    "ortografiásemos": "ortografiar",
    "ortografié": "ortografiar",
    "ortografiéis": "ortografiar",
    "ortografió": "ortografiar",
    "ortografía": "ortografiar",
    "ortografían": "ortografiar",
    "ortografías": "ortografiar",
    "ortografíe": "ortografiar",
    "ortografíen": "ortografiar",
    "ortografíes": "ortografiar",
    "ortografío": "ortografiar",
    "ortográfica": "ortográfico",
    "ortográficas": "ortográfico",
    "ortográficos": "ortográfico",
    "ortohelios": "ortohelio",
    "ortohidrógenos": "ortohidrógeno",
    "ortologías": "ortología",
    "ortonormales": "ortonormal",
    "ortopantomografías": "ortopantomografía",
    "ortopedagogías": "ortopedagogía",
    "ortopedas": "ortopeda",
    "ortopedias": "ortopedia",
    "ortopedistas": "ortopedista",
    "ortopiroxenos": "ortopiroxeno",
    "ortoptoscopios": "ortoptoscopio",
    "ortopédica": "ortopédico",
    "ortopédicas": "ortopédico",
    "ortopédicos": "ortopédico",
    "ortorrómbica": "ortorrómbico",
    "ortorrómbicas": "ortorrómbico",
    "ortorrómbicos": "ortorrómbico",
    "ortos": "orto",
    "ortosas": "ortosa",
    "ortoscopias": "ortoscopia",
    "ortoscopios": "ortoscopio",
    "ortoscópica": "ortoscópico",
    "ortoscópicas": "ortoscópico",
    "ortoscópicos": "ortoscópico",
    "ortotópica": "ortotópico",
    "ortotópicas": "ortotópico",
    "ortotópicos": "ortotópico",
    "ortógrafa": "ortógrafo",
    "ortógrafas": "ortógrafo",
    "ortógrafos": "ortógrafo",
    "ortópteros": "ortóptero",
    "ortóptica": "ortóptico",
    "ortópticas": "ortóptico",
    "ortópticos": "ortóptico",
    "ortótropa": "ortótropo",
    "ortótropas": "ortótropo",
    "ortótropos": "ortótropo",
    "orugas": "oruga",
    "orujos": "orujo",
    "orza": "orzar",
    "orzaba": "orzar",
    "orzabais": "orzar",
    "orzaban": "orzar",
    "orzabas": "orzar",
    "orzad": "orzar",
    "orzada": "orzar",
    "orzadas": "orzar",
    "orzado": "orzar",
    "orzados": "orzar",
    "orzamos": "orzar",
    "orzan": "orzar",
    "orzando": "orzar",
    "orzara": "orzar",
    "orzarais": "orzar",
    "orzaran": "orzar",
    "orzaras": "orzar",
    "orzare": "orzar",
    "orzareis": "orzar",
    "orzaremos": "orzar",
    "orzaren": "orzar",
    "orzares": "orzar",
    "orzaron": "orzar",
    "orzará": "orzar",
    "orzarán": "orzar",
    "orzarás": "orzar",
    "orzaré": "orzar",
    "orzaréis": "orzar",
    "orzaría": "orzar",
    "orzaríais": "orzar",
    "orzaríamos": "orzar",
    "orzarían": "orzar",
    "orzarías": "orzar",
    "orzas": "orzar",
    "orzase": "orzar",
    "orzaseis": "orzar",
    "orzasen": "orzar",
    "orzases": "orzar",
    "orzaste": "orzar",
    "orzasteis": "orzar",
    "orzo": "orzar",
    "orzoyos": "orzoyo",
    "orzuelos": "orzuelo",
    "orzábamos": "orzar",
    "orzáis": "orzar",
    "orzáramos": "orzar",
    "orzáremos": "orzar",
    "orzásemos": "orzar",
    "orzó": "orzar",
    "orábamos": "orar",
    "oráculos": "oráculo",
    "oráis": "orar",
    "oráramos": "orar",
    "oráremos": "orar",
    "orásemos": "orar",
    "oré": "orar",
    "oréades": "oréade",
    "oréganos": "orégano",
    "oréis": "orar",
    "orífices": "orífice",
    "orígenes": "origen",
    "oró": "orar",
    "orófitos": "orófito",
    "orógenos": "orógeno",
    "orónimos": "orónimo",
    "os": "o",
    "osa": "oso",
    "osaba": "osar",
    "osabais": "osar",
    "osaban": "osar",
    "osabas": "osar",
    "osad": "osar",
    "osada": "osar",
    "osadas": "osar",
    "osado": "osar",
    "osados": "osar",
    "osadías": "osadía",
    "osambres": "osambre",
    "osamentas": "osamenta",
    "osamos": "osar",
    "osan": "osar",
    "osando": "osar",
    "osara": "osar",
    "osarais": "osar",
    "osaran": "osar",
    "osaras": "osar",
    "osare": "osar",
    "osareis": "osar",
    "osaremos": "osar",
    "osaren": "osar",
    "osares": "osar",
    "osarios": "osario",
    "osaron": "osar",
    "osará": "osar",
    "osarán": "osar",
    "osarás": "osar",
    "osaré": "osar",
    "osaréis": "osar",
    "osaría": "osar",
    "osaríais": "osar",
    "osaríamos": "osar",
    "osarían": "osar",
    "osarías": "osar",
    "osas": "oso",
    "osase": "osar",
    "osaseis": "osar",
    "osasen": "osar",
    "osases": "osar",
    "osaste": "osar",
    "osasteis": "osar",
    "osca": "osco",
    "oscars": "oscar",
    "oscas": "osco",
    "oscenses": "oscense",
    "oscila": "oscilar",
    "oscilaba": "oscilar",
    "oscilabais": "oscilar",
    "oscilaban": "oscilar",
    "oscilabas": "oscilar",
    "oscilaciones": "oscilación",
    "oscilad": "oscilar",
    "oscilada": "oscilar",
    "osciladas": "oscilar",
    "oscilado": "oscilar",
    "osciladora": "oscilador",
    "osciladoras": "oscilador",
    "osciladores": "oscilador",
    "oscilados": "oscilar",
    "oscilamos": "oscilar",
    "oscilan": "oscilar",
    "oscilando": "oscilar",
    "oscilantes": "oscilante",
    "oscilara": "oscilar",
    "oscilarais": "oscilar",
    "oscilaran": "oscilar",
    "oscilaras": "oscilar",
    "oscilare": "oscilar",
    "oscilareis": "oscilar",
    "oscilaremos": "oscilar",
    "oscilaren": "oscilar",
    "oscilares": "oscilar",
    "oscilaron": "oscilar",
    "oscilará": "oscilar",
    "oscilarán": "oscilar",
    "oscilarás": "oscilar",
    "oscilaré": "oscilar",
    "oscilaréis": "oscilar",
    "oscilaría": "oscilar",
    "oscilaríais": "oscilar",
    "oscilaríamos": "oscilar",
    "oscilarían": "oscilar",
    "oscilarías": "oscilar",
    "oscilas": "oscilar",
    "oscilase": "oscilar",
    "oscilaseis": "oscilar",
    "oscilasen": "oscilar",
    "oscilases": "oscilar",
    "oscilaste": "oscilar",
    "oscilasteis": "oscilar",
    "oscilatoria": "oscilatorio",
    "oscilatorias": "oscilatorio",
    "oscilatorios": "oscilatorio",
    "oscile": "oscilar",
    "oscilemos": "oscilar",
    "oscilen": "oscilar",
    "osciles": "oscilar",
    "oscilo": "oscilar",
    "oscilogramas": "oscilograma",
    "oscilográfica": "oscilográfico",
    "oscilográficas": "oscilográfico",
    "oscilográficos": "oscilográfico",
    "oscilometrías": "oscilometría",
    "osciloscopios": "osciloscopio",
    "oscilábamos": "oscilar",
    "osciláis": "oscilar",
    "osciláramos": "oscilar",
    "osciláremos": "oscilar",
    "oscilásemos": "oscilar",
    "oscilé": "oscilar",
    "osciléis": "oscilar",
    "osciló": "oscilar",
    "oscilógrafos": "oscilógrafo",
    "oscos": "osco",
    "osculaciones": "osculación",
    "osculadores": "osculador",
    "oscultaciones": "oscultación",
    "oscura": "oscuro",
    "oscurantismos": "oscurantismo",
    "oscurantistas": "oscurantista",
    "oscuras": "oscuro",
    "oscurece": "oscurecer",
    "oscureced": "oscurecer",
    "oscurecemos": "oscurecer",
    "oscurecen": "oscurecer",
    "oscureceremos": "oscurecer",
    "oscurecerá": "oscurecer",
    "oscurecerán": "oscurecer",
    "oscurecerás": "oscurecer",
    "oscureceré": "oscurecer",
    "oscureceréis": "oscurecer",
    "oscurecería": "oscurecer",
    "oscureceríais": "oscurecer",
    "oscureceríamos": "oscurecer",
    "oscurecerían": "oscurecer",
    "oscurecerías": "oscurecer",
    "oscureces": "oscurecer",
    "oscurecida": "oscurecer",
    "oscurecidas": "oscurecer",
    "oscurecido": "oscurecer",
    "oscurecidos": "oscurecer",
    "oscureciendo": "oscurecer",
    "oscureciera": "oscurecer",
    "oscurecierais": "oscurecer",
    "oscurecieran": "oscurecer",
    "oscurecieras": "oscurecer",
    "oscureciere": "oscurecer",
    "oscureciereis": "oscurecer",
    "oscurecieren": "oscurecer",
    "oscurecieres": "oscurecer",
    "oscurecieron": "oscurecer",
    "oscureciese": "oscurecer",
    "oscurecieseis": "oscurecer",
    "oscureciesen": "oscurecer",
    "oscurecieses": "oscurecer",
    "oscurecimentos": "oscurecimento",
    "oscurecimientos": "oscurecimiento",
    "oscurecimos": "oscurecer",
    "oscureciste": "oscurecer",
    "oscurecisteis": "oscurecer",
    "oscureciéramos": "oscurecer",
    "oscureciéremos": "oscurecer",
    "oscureciésemos": "oscurecer",
    "oscureció": "oscurecer",
    "oscurecéis": "oscurecer",
    "oscurecí": "oscurecer",
    "oscurecía": "oscurecer",
    "oscurecíais": "oscurecer",
    "oscurecíamos": "oscurecer",
    "oscurecían": "oscurecer",
    "oscurecías": "oscurecer",
    "oscurezca": "oscurecer",
    "oscurezcamos": "oscurecer",
    "oscurezcan": "oscurecer",
    "oscurezcas": "oscurecer",
    "oscurezco": "oscurecer",
    "oscurezcáis": "oscurecer",
    "oscuridades": "oscuridad",
    "oscuros": "oscuro",
    "ose": "osar",
    "osemos": "osar",
    "osen": "osar",
    "oseras": "osera",
    "oseros": "osero",
    "oses": "osar",
    "osezna": "osezno",
    "oseznas": "osezno",
    "oseznos": "osezno",
    "osiculares": "osicular",
    "osiculectomías": "osiculectomía",
    "osiculoplastias": "osiculoplastia",
    "osifica": "osificar",
    "osificaba": "osificar",
    "osificabais": "osificar",
    "osificaban": "osificar",
    "osificabas": "osificar",
    "osificaciones": "osificación",
    "osificaciónes": "osificación",
    "osificad": "osificar",
    "osificada": "osificar",
    "osificadas": "osificar",
    "osificado": "osificar",
    "osificados": "osificar",
    "osificamos": "osificar",
    "osifican": "osificar",
    "osificando": "osificar",
    "osificara": "osificar",
    "osificarais": "osificar",
    "osificaran": "osificar",
    "osificaras": "osificar",
    "osificare": "osificar",
    "osificareis": "osificar",
    "osificaremos": "osificar",
    "osificaren": "osificar",
    "osificares": "osificar",
    "osificaron": "osificar",
    "osificará": "osificar",
    "osificarán": "osificar",
    "osificarás": "osificar",
    "osificaré": "osificar",
    "osificaréis": "osificar",
    "osificaría": "osificar",
    "osificaríais": "osificar",
    "osificaríamos": "osificar",
    "osificarían": "osificar",
    "osificarías": "osificar",
    "osificas": "osificar",
    "osificase": "osificar",
    "osificaseis": "osificar",
    "osificasen": "osificar",
    "osificases": "osificar",
    "osificaste": "osificar",
    "osificasteis": "osificar",
    "osifico": "osificar",
    "osificábamos": "osificar",
    "osificáis": "osificar",
    "osificáramos": "osificar",
    "osificáremos": "osificar",
    "osificásemos": "osificar",
    "osificó": "osificar",
    "osifique": "osificar",
    "osifiquemos": "osificar",
    "osifiquen": "osificar",
    "osifiques": "osificar",
    "osifiqué": "osificar",
    "osifiquéis": "osificar",
    "osmanlíes": "osmanlí",
    "osmanlís": "osmanlí",
    "osmiatos": "osmiato",
    "osmios": "osmio",
    "osmiridios": "osmiridio",
    "osmolaridades": "osmolaridad",
    "osmometrías": "osmometría",
    "osmorregulaciones": "osmorregulación",
    "osmundales": "osmundal",
    "osmómetros": "osmómetro",
    "osmótica": "osmótico",
    "osmóticas": "osmótico",
    "osmóticos": "osmótico",
    "oso": "osar",
    "osos": "oso",
    "ososa": "ososo",
    "ososas": "ososo",
    "ososos": "ososo",
    "ossobuccos": "ossobucco",
    "ostagas": "ostaga",
    "ostalgias": "ostalgia",
    "ostas": "osta",
    "ostensibles": "ostensible",
    "ostensiva": "ostensivo",
    "ostensivas": "ostensivo",
    "ostensivos": "ostensivo",
    "ostenta": "ostentar",
    "ostentaba": "ostentar",
    "ostentabais": "ostentar",
    "ostentaban": "ostentar",
    "ostentabas": "ostentar",
    "ostentaciones": "ostentación",
    "ostentad": "ostentar",
    "ostentada": "ostentar",
    "ostentadas": "ostentar",
    "ostentado": "ostentar",
    "ostentadora": "ostentador",
    "ostentadoras": "ostentador",
    "ostentadores": "ostentador",
    "ostentados": "ostentar",
    "ostentamos": "ostentar",
    "ostentan": "ostentar",
    "ostentando": "ostentar",
    "ostentara": "ostentar",
    "ostentarais": "ostentar",
    "ostentaran": "ostentar",
    "ostentaras": "ostentar",
    "ostentare": "ostentar",
    "ostentareis": "ostentar",
    "ostentaremos": "ostentar",
    "ostentaren": "ostentar",
    "ostentares": "ostentar",
    "ostentaron": "ostentar",
    "ostentará": "ostentar",
    "ostentarán": "ostentar",
    "ostentarás": "ostentar",
    "ostentaré": "ostentar",
    "ostentaréis": "ostentar",
    "ostentaría": "ostentar",
    "ostentaríais": "ostentar",
    "ostentaríamos": "ostentar",
    "ostentarían": "ostentar",
    "ostentarías": "ostentar",
    "ostentas": "ostentar",
    "ostentase": "ostentar",
    "ostentaseis": "ostentar",
    "ostentasen": "ostentar",
    "ostentases": "ostentar",
    "ostentaste": "ostentar",
    "ostentasteis": "ostentar",
    "ostentativa": "ostentativo",
    "ostentativas": "ostentativo",
    "ostentativos": "ostentativo",
    "ostente": "ostentar",
    "ostentemos": "ostentar",
    "ostenten": "ostentar",
    "ostentes": "ostentar",
    "ostento": "ostentar",
    "ostentosa": "ostentoso",
    "ostentosas": "ostentoso",
    "ostentosos": "ostentoso",
    "ostentábamos": "ostentar",
    "ostentáis": "ostentar",
    "ostentáramos": "ostentar",
    "ostentáremos": "ostentar",
    "ostentásemos": "ostentar",
    "ostenté": "ostentar",
    "ostentéis": "ostentar",
    "ostentó": "ostentar",
    "osteoblastos": "osteoblasto",
    "osteocementos": "osteocemento",
    "osteocitos": "osteocito",
    "osteoclasias": "osteoclasia",
    "osteoclastos": "osteoclasto",
    "osteodentinas": "osteodentina",
    "osteodentinomas": "osteodentinoma",
    "osteoectomías": "osteoectomía",
    "osteogénica": "osteogénico",
    "osteogénicas": "osteogénico",
    "osteogénicos": "osteogénico",
    "osteointegraciones": "osteointegración",
    "osteointegrada": "osteointegrado",
    "osteointegradas": "osteointegrado",
    "osteointegrados": "osteointegrado",
    "osteologías": "osteología",
    "osteomalacias": "osteomalacia",
    "osteomucoides": "osteomucoide",
    "osteopatías": "osteopatía",
    "osteoplastias": "osteoplastia",
    "osteotomías": "osteotomía",
    "osteotrimeres": "osteotrimer",
    "osteíctios": "osteíctio",
    "osteínas": "osteína",
    "osteócitos": "osteócito",
    "osteófonos": "osteófono",
    "osteótomos": "osteótomo",
    "ostiales": "ostial",
    "ostiariados": "ostiariado",
    "ostiarios": "ostiario",
    "ostias": "ostia",
    "ostiolos": "ostiolo",
    "ostiones": "ostión",
    "ostracismos": "ostracismo",
    "ostrales": "ostral",
    "ostras": "ostra",
    "ostrera": "ostrero",
    "ostreras": "ostrero",
    "ostreros": "ostrero",
    "ostricultora": "ostricultor",
    "ostricultoras": "ostricultor",
    "ostricultores": "ostricultor",
    "ostriculturas": "ostricultura",
    "ostrogoda": "ostrogodo",
    "ostrogodas": "ostrogodo",
    "ostrogodos": "ostrogodo",
    "ostrones": "ostrón",
    "ostros": "ostro",
    "ostrícolas": "ostrícola",
    "ostrífera": "ostrífero",
    "ostríferas": "ostrífero",
    "ostríferos": "ostrífero",
    "osuda": "osudo",
    "osudas": "osudo",
    "osudos": "osudo",
    "osuna": "osuno",
    "osunas": "osuno",
    "osunos": "osuno",
    "osábamos": "osar",
    "osáis": "osar",
    "osáramos": "osar",
    "osáremos": "osar",
    "osásemos": "osar",
    "osé": "osar",
    "oséis": "osar",
    "osó": "osar",
    "otacústica": "otacústico",
    "otacústicas": "otacústico",
    "otacústicos": "otacústico",
    "otalgias": "otalgia",
    "otaria": "otario",
    "otarias": "otario",
    "otarios": "otario",
    "otea": "otear",
    "oteaba": "otear",
    "oteabais": "otear",
    "oteaban": "otear",
    "oteabas": "otear",
    "otead": "otear",
    "oteada": "otear",
    "oteadas": "otear",
    "oteado": "otear",
    "oteadora": "oteador",
    "oteadoras": "oteador",
    "oteadores": "oteador",
    "oteados": "otear",
    "oteamos": "otear",
    "otean": "otear",
    "oteando": "otear",
    "oteara": "otear",
    "otearais": "otear",
    "otearan": "otear",
    "otearas": "otear",
    "oteare": "otear",
    "oteareis": "otear",
    "otearemos": "otear",
    "otearen": "otear",
    "oteares": "otear",
    "otearon": "otear",
    "oteará": "otear",
    "otearán": "otear",
    "otearás": "otear",
    "otearé": "otear",
    "otearéis": "otear",
    "otearía": "otear",
    "otearíais": "otear",
    "otearíamos": "otear",
    "otearían": "otear",
    "otearías": "otear",
    "oteas": "otear",
    "otease": "otear",
    "oteaseis": "otear",
    "oteasen": "otear",
    "oteases": "otear",
    "oteaste": "otear",
    "oteasteis": "otear",
    "otee": "otear",
    "oteemos": "otear",
    "oteen": "otear",
    "otees": "otear",
    "oteo": "otear",
    "oteros": "otero",
    "oteábamos": "otear",
    "oteáis": "otear",
    "oteáramos": "otear",
    "oteáremos": "otear",
    "oteásemos": "otear",
    "oteé": "otear",
    "oteéis": "otear",
    "oteó": "otear",
    "otociones": "otoción",
    "otoemisiones": "otoemisión",
    "otogénica": "otogénico",
    "otogénicas": "otogénico",
    "otogénicos": "otogénico",
    "otohematomas": "otohematoma",
    "otolitos": "otolito",
    "otologías": "otología",
    "otolítica": "otolítico",
    "otolíticas": "otolítico",
    "otolíticos": "otolítico",
    "otológica": "otológico",
    "otológicas": "otológico",
    "otológicos": "otológico",
    "otomana": "otomano",
    "otomanas": "otomano",
    "otomandibulares": "otomandibular",
    "otomanes": "otomán",
    "otomanos": "otomano",
    "otomicroscopios": "otomicroscopio",
    "otomíes": "otomí",
    "otomís": "otomí",
    "otopatías": "otopatía",
    "otoplastias": "otoplastia",
    "otorga": "otorgar",
    "otorgaba": "otorgar",
    "otorgabais": "otorgar",
    "otorgaban": "otorgar",
    "otorgabas": "otorgar",
    "otorgaciones": "otorgación",
    "otorgad": "otorgar",
    "otorgada": "otorgar",
    "otorgadas": "otorgar",
    "otorgadera": "otorgadero",
    "otorgaderas": "otorgadero",
    "otorgaderos": "otorgadero",
    "otorgado": "otorgar",
    "otorgadora": "otorgador",
    "otorgadoras": "otorgador",
    "otorgadores": "otorgador",
    "otorgados": "otorgar",
    "otorgamientos": "otorgamiento",
    "otorgamos": "otorgar",
    "otorgan": "otorgar",
    "otorgando": "otorgar",
    "otorgantes": "otorgante",
    "otorgara": "otorgar",
    "otorgarais": "otorgar",
    "otorgaran": "otorgar",
    "otorgaras": "otorgar",
    "otorgare": "otorgar",
    "otorgareis": "otorgar",
    "otorgaremos": "otorgar",
    "otorgaren": "otorgar",
    "otorgares": "otorgar",
    "otorgaron": "otorgar",
    "otorgará": "otorgar",
    "otorgarán": "otorgar",
    "otorgarás": "otorgar",
    "otorgaré": "otorgar",
    "otorgaréis": "otorgar",
    "otorgaría": "otorgar",
    "otorgaríais": "otorgar",
    "otorgaríamos": "otorgar",
    "otorgarían": "otorgar",
    "otorgarías": "otorgar",
    "otorgas": "otorgar",
    "otorgase": "otorgar",
    "otorgaseis": "otorgar",
    "otorgasen": "otorgar",
    "otorgases": "otorgar",
    "otorgaste": "otorgar",
    "otorgasteis": "otorgar",
    "otorgo": "otorgar",
    "otorgue": "otorgar",
    "otorguemos": "otorgar",
    "otorguen": "otorgar",
    "otorgues": "otorgar",
    "otorgué": "otorgar",
    "otorguéis": "otorgar",
    "otorgábamos": "otorgar",
    "otorgáis": "otorgar",
    "otorgáramos": "otorgar",
    "otorgáremos": "otorgar",
    "otorgásemos": "otorgar",
    "otorgó": "otorgar",
    "otorragias": "otorragia",
    "otorraquias": "otorraquia",
    "otorreas": "otorrea",
    "otorreica": "otorreico",
    "otorreicas": "otorreico",
    "otorreicos": "otorreico",
    "otorrinolaringologías": "otorrinolaringología",
    "otorrinolaringóloga": "otorrinolaringólogo",
    "otorrinolaringólogas": "otorrinolaringólogo",
    "otorrinolaringólogos": "otorrinolaringólogo",
    "otorrinonaringologías": "otorrinonaringología",
    "otoscopias": "otoscopia",
    "otoscopios": "otoscopio",
    "ototoxicidades": "ototoxicidad",
    "otoña": "otoñar",
    "otoñaba": "otoñar",
    "otoñabais": "otoñar",
    "otoñaban": "otoñar",
    "otoñabas": "otoñar",
    "otoñad": "otoñar",
    "otoñada": "otoñar",
    "otoñadas": "otoñar",
    "otoñado": "otoñar",
    "otoñados": "otoñar",
    "otoñales": "otoñal",
    "otoñamos": "otoñar",
    "otoñan": "otoñar",
    "otoñando": "otoñar",
    "otoñara": "otoñar",
    "otoñarais": "otoñar",
    "otoñaran": "otoñar",
    "otoñaras": "otoñar",
    "otoñare": "otoñar",
    "otoñareis": "otoñar",
    "otoñaremos": "otoñar",
    "otoñaren": "otoñar",
    "otoñares": "otoñar",
    "otoñaron": "otoñar",
    "otoñará": "otoñar",
    "otoñarán": "otoñar",
    "otoñarás": "otoñar",
    "otoñaré": "otoñar",
    "otoñaréis": "otoñar",
    "otoñaría": "otoñar",
    "otoñaríais": "otoñar",
    "otoñaríamos": "otoñar",
    "otoñarían": "otoñar",
    "otoñarías": "otoñar",
    "otoñas": "otoñar",
    "otoñase": "otoñar",
    "otoñaseis": "otoñar",
    "otoñasen": "otoñar",
    "otoñases": "otoñar",
    "otoñaste": "otoñar",
    "otoñasteis": "otoñar",
    "otoñe": "otoñar",
    "otoñemos": "otoñar",
    "otoñen": "otoñar",
    "otoñes": "otoñar",
    "otoñiza": "otoñizo",
    "otoñizas": "otoñizo",
    "otoñizos": "otoñizo",
    "otoño": "otoñar",
    "otoños": "otoño",
    "otoñábamos": "otoñar",
    "otoñáis": "otoñar",
    "otoñáramos": "otoñar",
    "otoñáremos": "otoñar",
    "otoñásemos": "otoñar",
    "otoñé": "otoñar",
    "otoñéis": "otoñar",
    "otoñó": "otoñar",
    "otra": "otro",
    "otras": "otro",
    "otros": "otro",
    "otrosis": "otrosí",
    "otrosís": "otrosí",
    "otálgica": "otálgico",
    "otálgicas": "otálgico",
    "otálgicos": "otálgico",
    "otítica": "otítico",
    "otíticas": "otítico",
    "otíticos": "otítico",
    "otóloga": "otólogo",
    "otólogas": "otólogo",
    "otólogos": "otólogo",
    "otónica": "otónico",
    "otónicas": "otónico",
    "otónicos": "otónico",
    "ouguiyas": "ouguiya",
    "outputs": "output",
    "ouzos": "ouzo",
    "ova": "ovar",
    "ovaba": "ovar",
    "ovabais": "ovar",
    "ovaban": "ovar",
    "ovabas": "ovar",
    "ovaciona": "ovacionar",
    "ovacionaba": "ovacionar",
    "ovacionabais": "ovacionar",
    "ovacionaban": "ovacionar",
    "ovacionabas": "ovacionar",
    "ovacionad": "ovacionar",
    "ovacionada": "ovacionar",
    "ovacionadas": "ovacionar",
    "ovacionado": "ovacionar",
    "ovacionados": "ovacionar",
    "ovacionamos": "ovacionar",
    "ovacionan": "ovacionar",
    "ovacionando": "ovacionar",
    "ovacionara": "ovacionar",
    "ovacionarais": "ovacionar",
    "ovacionaran": "ovacionar",
    "ovacionaras": "ovacionar",
    "ovacionare": "ovacionar",
    "ovacionareis": "ovacionar",
    "ovacionaremos": "ovacionar",
    "ovacionaren": "ovacionar",
    "ovacionares": "ovacionar",
    "ovacionaron": "ovacionar",
    "ovacionará": "ovacionar",
    "ovacionarán": "ovacionar",
    "ovacionarás": "ovacionar",
    "ovacionaré": "ovacionar",
    "ovacionaréis": "ovacionar",
    "ovacionaría": "ovacionar",
    "ovacionaríais": "ovacionar",
    "ovacionaríamos": "ovacionar",
    "ovacionarían": "ovacionar",
    "ovacionarías": "ovacionar",
    "ovacionas": "ovacionar",
    "ovacionase": "ovacionar",
    "ovacionaseis": "ovacionar",
    "ovacionasen": "ovacionar",
    "ovacionases": "ovacionar",
    "ovacionaste": "ovacionar",
    "ovacionasteis": "ovacionar",
    "ovacione": "ovacionar",
    "ovacionemos": "ovacionar",
    "ovacionen": "ovacionar",
    "ovaciones": "ovacionar",
    "ovaciono": "ovacionar",
    "ovacionábamos": "ovacionar",
    "ovacionáis": "ovacionar",
    "ovacionáramos": "ovacionar",
    "ovacionáremos": "ovacionar",
    "ovacionásemos": "ovacionar",
    "ovacioné": "ovacionar",
    "ovacionéis": "ovacionar",
    "ovacionó": "ovacionar",
    "ovad": "ovar",
    "ovada": "ovar",
    "ovadas": "ovar",
    "ovado": "ovar",
    "ovados": "ovar",
    "ovala": "ovalar",
    "ovalaba": "ovalar",
    "ovalabais": "ovalar",
    "ovalaban": "ovalar",
    "ovalabas": "ovalar",
    "ovalad": "ovalar",
    "ovalada": "ovalar",
    "ovaladas": "ovalar",
    "ovalado": "ovalar",
    "ovalados": "ovalar",
    "ovalamos": "ovalar",
    "ovalan": "ovalar",
    "ovalando": "ovalar",
    "ovalara": "ovalar",
    "ovalarais": "ovalar",
    "ovalaran": "ovalar",
    "ovalaras": "ovalar",
    "ovalare": "ovalar",
    "ovalareis": "ovalar",
    "ovalaremos": "ovalar",
    "ovalaren": "ovalar",
    "ovalares": "ovalar",
    "ovalaron": "ovalar",
    "ovalará": "ovalar",
    "ovalarán": "ovalar",
    "ovalarás": "ovalar",
    "ovalaré": "ovalar",
    "ovalaréis": "ovalar",
    "ovalaría": "ovalar",
    "ovalaríais": "ovalar",
    "ovalaríamos": "ovalar",
    "ovalarían": "ovalar",
    "ovalarías": "ovalar",
    "ovalas": "ovalar",
    "ovalase": "ovalar",
    "ovalaseis": "ovalar",
    "ovalasen": "ovalar",
    "ovalases": "ovalar",
    "ovalaste": "ovalar",
    "ovalasteis": "ovalar",
    "ovale": "ovalar",
    "ovalemos": "ovalar",
    "ovalen": "ovalar",
    "ovales": "ovalar",
    "ovalo": "ovalar",
    "ovalocitos": "ovalocito",
    "ovalábamos": "ovalar",
    "ovaláis": "ovalar",
    "ovaláramos": "ovalar",
    "ovaláremos": "ovalar",
    "ovalásemos": "ovalar",
    "ovalé": "ovalar",
    "ovaléis": "ovalar",
    "ovaló": "ovalar",
    "ovamos": "ovar",
    "ovan": "ovar",
    "ovando": "ovar",
    "ovantes": "ovante",
    "ovara": "ovar",
    "ovarais": "ovar",
    "ovaran": "ovar",
    "ovaras": "ovar",
    "ovare": "ovar",
    "ovareis": "ovar",
    "ovaremos": "ovar",
    "ovaren": "ovar",
    "ovares": "ovar",
    "ovarios": "ovario",
    "ovariotomías": "ovariotomía",
    "ovaron": "ovar",
    "ovará": "ovar",
    "ovarán": "ovar",
    "ovarás": "ovar",
    "ovaré": "ovar",
    "ovaréis": "ovar",
    "ovaría": "ovar",
    "ovaríais": "ovar",
    "ovaríamos": "ovar",
    "ovarían": "ovar",
    "ovarías": "ovar",
    "ovas": "ovar",
    "ovase": "ovar",
    "ovaseis": "ovar",
    "ovasen": "ovar",
    "ovases": "ovar",
    "ovaste": "ovar",
    "ovasteis": "ovar",
    "ove": "ovar",
    "ovejas": "oveja",
    "ovejera": "ovejero",
    "ovejeras": "ovejero",
    "ovejeros": "ovejero",
    "ovejuna": "ovejuno",
    "ovejunas": "ovejuno",
    "ovejunos": "ovejuno",
    "ovemos": "ovar",
    "oven": "ovar",
    "overa": "overo",
    "overas": "overo",
    "overbookings": "overbooking",
    "overflowings": "overflowing",
    "overoles": "overol",
    "overos": "overo",
    "oves": "ovar",
    "ovetenses": "ovetense",
    "oviculturas": "ovicultura",
    "ovicáprida": "ovicáprido",
    "ovicápridas": "ovicáprido",
    "ovicápridos": "ovicáprido",
    "oviductos": "oviducto",
    "ovilla": "ovillar",
    "ovillaba": "ovillar",
    "ovillabais": "ovillar",
    "ovillaban": "ovillar",
    "ovillabas": "ovillar",
    "ovillad": "ovillar",
    "ovillada": "ovillar",
    "ovilladas": "ovillar",
    "ovillado": "ovillar",
    "ovillados": "ovillar",
    "ovillamos": "ovillar",
    "ovillan": "ovillar",
    "ovillando": "ovillar",
    "ovillara": "ovillar",
    "ovillarais": "ovillar",
    "ovillaran": "ovillar",
    "ovillaras": "ovillar",
    "ovillare": "ovillar",
    "ovillareis": "ovillar",
    "ovillaremos": "ovillar",
    "ovillaren": "ovillar",
    "ovillares": "ovillar",
    "ovillaron": "ovillar",
    "ovillará": "ovillar",
    "ovillarán": "ovillar",
    "ovillarás": "ovillar",
    "ovillaré": "ovillar",
    "ovillaréis": "ovillar",
    "ovillaría": "ovillar",
    "ovillaríais": "ovillar",
    "ovillaríamos": "ovillar",
    "ovillarían": "ovillar",
    "ovillarías": "ovillar",
    "ovillas": "ovillar",
    "ovillase": "ovillar",
    "ovillaseis": "ovillar",
    "ovillasen": "ovillar",
    "ovillases": "ovillar",
    "ovillaste": "ovillar",
    "ovillasteis": "ovillar",
    "oville": "ovillar",
    "ovillejos": "ovillejo",
    "ovillemos": "ovillar",
    "ovillen": "ovillar",
    "ovilles": "ovillar",
    "ovillo": "ovillar",
    "ovillos": "ovillo",
    "ovillábamos": "ovillar",
    "ovilláis": "ovillar",
    "ovilláramos": "ovillar",
    "ovilláremos": "ovillar",
    "ovillásemos": "ovillar",
    "ovillé": "ovillar",
    "ovilléis": "ovillar",
    "ovilló": "ovillar",
    "ovina": "ovino",
    "ovinas": "ovino",
    "ovinos": "ovino",
    "oviscaptos": "oviscapto",
    "ovnis": "ovni",
    "ovo": "ovar",
    "ovoalbúminas": "ovoalbúmina",
    "ovocitos": "ovocito",
    "ovoidales": "ovoidal",
    "ovoidea": "ovoideo",
    "ovoideas": "ovoideo",
    "ovoideos": "ovoideo",
    "ovoides": "ovoide",
    "ovopositores": "ovopositor",
    "ovoproductos": "ovoproducto",
    "ovos": "ovo",
    "ovosa": "ovoso",
    "ovosas": "ovoso",
    "ovoscopios": "ovoscopio",
    "ovosos": "ovoso",
    "ovovivípara": "ovovivíparo",
    "ovovivíparas": "ovovivíparo",
    "ovovivíparos": "ovovivíparo",
    "ovulaciones": "ovulación",
    "ovulares": "ovular",
    "ovábamos": "ovar",
    "ováis": "ovar",
    "ováramos": "ovar",
    "ováremos": "ovar",
    "ovárica": "ovárico",
    "ováricas": "ovárico",
    "ováricos": "ovárico",
    "ovásemos": "ovar",
    "ové": "ovar",
    "ovéis": "ovar",
    "ovípara": "ovíparo",
    "ovíparas": "ovíparo",
    "ovíparos": "ovíparo",
    "ovó": "ovar",
    "owenismos": "owenismo",
    "oxalataciones": "oxalatación",
    "oxalatos": "oxalato",
    "oxalidácea": "oxalidáceo",
    "oxalidáceas": "oxalidáceo",
    "oxalidáceos": "oxalidáceo",
    "oxalmes": "oxalme",
    "oxamidas": "oxamida",
    "oxea": "oxear",
    "oxeaba": "oxear",
    "oxeabais": "oxear",
    "oxeaban": "oxear",
    "oxeabas": "oxear",
    "oxead": "oxear",
    "oxeada": "oxear",
    "oxeadas": "oxear",
    "oxeado": "oxear",
    "oxeados": "oxear",
    "oxeamos": "oxear",
    "oxean": "oxear",
    "oxeando": "oxear",
    "oxeara": "oxear",
    "oxearais": "oxear",
    "oxearan": "oxear",
    "oxearas": "oxear",
    "oxeare": "oxear",
    "oxeareis": "oxear",
    "oxearemos": "oxear",
    "oxearen": "oxear",
    "oxeares": "oxear",
    "oxearon": "oxear",
    "oxeará": "oxear",
    "oxearán": "oxear",
    "oxearás": "oxear",
    "oxearé": "oxear",
    "oxearéis": "oxear",
    "oxearía": "oxear",
    "oxearíais": "oxear",
    "oxearíamos": "oxear",
    "oxearían": "oxear",
    "oxearías": "oxear",
    "oxeas": "oxear",
    "oxease": "oxear",
    "oxeaseis": "oxear",
    "oxeasen": "oxear",
    "oxeases": "oxear",
    "oxeaste": "oxear",
    "oxeasteis": "oxear",
    "oxee": "oxear",
    "oxeemos": "oxear",
    "oxeen": "oxear",
    "oxees": "oxear",
    "oxeo": "oxear",
    "oxers": "oxer",
    "oxeábamos": "oxear",
    "oxeáis": "oxear",
    "oxeáramos": "oxear",
    "oxeáremos": "oxear",
    "oxeásemos": "oxear",
    "oxeé": "oxear",
    "oxeéis": "oxear",
    "oxeó": "oxear",
    "oxhidrilos": "oxhidrilo",
    "oxhídrica": "oxhídrico",
    "oxhídricas": "oxhídrico",
    "oxhídricos": "oxhídrico",
    "oxiacetilenos": "oxiacetileno",
    "oxiacetilénica": "oxiacetilénico",
    "oxiacetilénicas": "oxiacetilénico",
    "oxiacetilénicos": "oxiacetilénico",
    "oxialquilaciones": "oxialquilación",
    "oxicarbonilaciones": "oxicarbonilación",
    "oxicelulosas": "oxicelulosa",
    "oxicianuros": "oxicianuro",
    "oxicloraciones": "oxicloración",
    "oxicloruros": "oxicloruro",
    "oxicortes": "oxicorte",
    "oxicrininas": "oxicrinina",
    "oxida": "oxidar",
    "oxidaba": "oxidar",
    "oxidabais": "oxidar",
    "oxidaban": "oxidar",
    "oxidabas": "oxidar",
    "oxidabilidades": "oxidabilidad",
    "oxidables": "oxidable",
    "oxidaciones": "oxidación",
    "oxidad": "oxidar",
    "oxidada": "oxidar",
    "oxidadas": "oxidar",
    "oxidado": "oxidar",
    "oxidados": "oxidar",
    "oxidamos": "oxidar",
    "oxidan": "oxidar",
    "oxidando": "oxidar",
    "oxidantes": "oxidante",
    "oxidara": "oxidar",
    "oxidarais": "oxidar",
    "oxidaran": "oxidar",
    "oxidaras": "oxidar",
    "oxidare": "oxidar",
    "oxidareis": "oxidar",
    "oxidaremos": "oxidar",
    "oxidaren": "oxidar",
    "oxidares": "oxidar",
    "oxidaron": "oxidar",
    "oxidará": "oxidar",
    "oxidarán": "oxidar",
    "oxidarás": "oxidar",
    "oxidaré": "oxidar",
    "oxidaréis": "oxidar",
    "oxidaría": "oxidar",
    "oxidaríais": "oxidar",
    "oxidaríamos": "oxidar",
    "oxidarían": "oxidar",
    "oxidarías": "oxidar",
    "oxidas": "oxidar",
    "oxidasas": "oxidasa",
    "oxidase": "oxidar",
    "oxidaseis": "oxidar",
    "oxidasen": "oxidar",
    "oxidases": "oxidar",
    "oxidaste": "oxidar",
    "oxidasteis": "oxidar",
    "oxidativa": "oxidativo",
    "oxidativas": "oxidativo",
    "oxidativos": "oxidativo",
    "oxide": "oxidar",
    "oxidemos": "oxidar",
    "oxiden": "oxidar",
    "oxides": "oxidar",
    "oxidisoles": "oxidisol",
    "oxido": "oxidar",
    "oxidoreducciones": "oxidoreducción",
    "oxidorreducciones": "oxidorreducción",
    "oxidorreductasas": "oxidorreductasa",
    "oxidrilos": "oxidrilo",
    "oxidábamos": "oxidar",
    "oxidáis": "oxidar",
    "oxidáramos": "oxidar",
    "oxidáremos": "oxidar",
    "oxidásemos": "oxidar",
    "oxidé": "oxidar",
    "oxidéis": "oxidar",
    "oxidó": "oxidar",
    "oxietilaciones": "oxietilación",
    "oxifilia": "oxifilio",
    "oxifilias": "oxifilio",
    "oxifilios": "oxifilio",
    "oxigena": "oxigenar",
    "oxigenaba": "oxigenar",
    "oxigenabais": "oxigenar",
    "oxigenaban": "oxigenar",
    "oxigenabas": "oxigenar",
    "oxigenables": "oxigenable",
    "oxigenaciones": "oxigenación",
    "oxigenad": "oxigenar",
    "oxigenada": "oxigenar",
    "oxigenadas": "oxigenar",
    "oxigenado": "oxigenar",
    "oxigenados": "oxigenar",
    "oxigenamos": "oxigenar",
    "oxigenan": "oxigenar",
    "oxigenando": "oxigenar",
    "oxigenara": "oxigenar",
    "oxigenarais": "oxigenar",
    "oxigenaran": "oxigenar",
    "oxigenaras": "oxigenar",
    "oxigenare": "oxigenar",
    "oxigenareis": "oxigenar",
    "oxigenaremos": "oxigenar",
    "oxigenaren": "oxigenar",
    "oxigenares": "oxigenar",
    "oxigenaron": "oxigenar",
    "oxigenará": "oxigenar",
    "oxigenarán": "oxigenar",
    "oxigenarás": "oxigenar",
    "oxigenaré": "oxigenar",
    "oxigenaréis": "oxigenar",
    "oxigenaría": "oxigenar",
    "oxigenaríais": "oxigenar",
    "oxigenaríamos": "oxigenar",
    "oxigenarían": "oxigenar",
    "oxigenarías": "oxigenar",
    "oxigenas": "oxigenar",
    "oxigenasas": "oxigenasa",
    "oxigenase": "oxigenar",
    "oxigenaseis": "oxigenar",
    "oxigenasen": "oxigenar",
    "oxigenases": "oxigenar",
    "oxigenaste": "oxigenar",
    "oxigenasteis": "oxigenar",
    "oxigene": "oxigenar",
    "oxigenemos": "oxigenar",
    "oxigenen": "oxigenar",
    "oxigenes": "oxigenar",
    "oxigeno": "oxigenar",
    "oxigenoterapias": "oxigenoterapia",
    "oxigenotropismos": "oxigenotropismo",
    "oxigenábamos": "oxigenar",
    "oxigenáis": "oxigenar",
    "oxigenáramos": "oxigenar",
    "oxigenáremos": "oxigenar",
    "oxigenásemos": "oxigenar",
    "oxigené": "oxigenar",
    "oxigenéis": "oxigenar",
    "oxigenó": "oxigenar",
    "oxigonia": "oxigonio",
    "oxigonias": "oxigonio",
    "oxigonios": "oxigonio",
    "oxihemoglobinas": "oxihemoglobina",
    "oxihídrica": "oxihídrico",
    "oxihídricas": "oxihídrico",
    "oxihídricos": "oxihídrico",
    "oximas": "oxima",
    "oximeles": "oximel",
    "oximetandrolonas": "oximetandrolona",
    "oximieles": "oximiel",
    "oxiprolinas": "oxiprolina",
    "oxipropilaciones": "oxipropilación",
    "oxitetraciclinas": "oxitetraciclina",
    "oxitocinas": "oxitocina",
    "oxitricloruros": "oxitricloruro",
    "oxitócica": "oxitócico",
    "oxitócicas": "oxitócico",
    "oxitócicos": "oxitócico",
    "oxiuros": "oxiuro",
    "oxivapogaceificaciones": "oxivapogaceificación",
    "oxiácidos": "oxiácido",
    "oxonios": "oxonio",
    "oxoácidos": "oxoácido",
    "oxprenololes": "oxprenolol",
    "oxtrifilinas": "oxtrifilina",
    "oxácidos": "oxácido",
    "oxálica": "oxálico",
    "oxálicas": "oxálico",
    "oxálicos": "oxálico",
    "oxígenos": "oxígeno",
    "oxímorones": "oxímoron",
    "oxítona": "oxítono",
    "oxítonas": "oxítono",
    "oxítonos": "oxítono",
    "oye": "oír",
    "oyen": "oír",
    "oyendo": "oír",
    "oyentes": "oyente",
    "oyera": "oír",
    "oyerais": "oír",
    "oyeran": "oír",
    "oyeras": "oír",
    "oyere": "oír",
    "oyereis": "oír",
    "oyeren": "oír",
    "oyeres": "oír",
    "oyeron": "oír",
    "oyes": "oír",
    "oyese": "oír",
    "oyeseis": "oír",
    "oyesen": "oír",
    "oyeses": "oír",
    "oyéramos": "oír",
    "oyéremos": "oír",
    "oyésemos": "oír",
    "oyó": "oír",
    "ozona": "ozonar",
    "ozonaba": "ozonar",
    "ozonabais": "ozonar",
    "ozonaban": "ozonar",
    "ozonabas": "ozonar",
    "ozonad": "ozonar",
    "ozonada": "ozonar",
    "ozonadas": "ozonar",
    "ozonado": "ozonar",
    "ozonadora": "ozonador",
    "ozonadoras": "ozonador",
    "ozonadores": "ozonador",
    "ozonados": "ozonar",
    "ozonamos": "ozonar",
    "ozonan": "ozonar",
    "ozonando": "ozonar",
    "ozonara": "ozonar",
    "ozonarais": "ozonar",
    "ozonaran": "ozonar",
    "ozonaras": "ozonar",
    "ozonare": "ozonar",
    "ozonareis": "ozonar",
    "ozonaremos": "ozonar",
    "ozonaren": "ozonar",
    "ozonares": "ozonar",
    "ozonaron": "ozonar",
    "ozonará": "ozonar",
    "ozonarán": "ozonar",
    "ozonarás": "ozonar",
    "ozonaré": "ozonar",
    "ozonaréis": "ozonar",
    "ozonaría": "ozonar",
    "ozonaríais": "ozonar",
    "ozonaríamos": "ozonar",
    "ozonarían": "ozonar",
    "ozonarías": "ozonar",
    "ozonas": "ozonar",
    "ozonase": "ozonar",
    "ozonaseis": "ozonar",
    "ozonasen": "ozonar",
    "ozonases": "ozonar",
    "ozonaste": "ozonar",
    "ozonasteis": "ozonar",
    "ozone": "ozonar",
    "ozonemos": "ozonar",
    "ozonen": "ozonar",
    "ozones": "ozonar",
    "ozonesferas": "ozonesfera",
    "ozonice": "ozonizar",
    "ozonicemos": "ozonizar",
    "ozonicen": "ozonizar",
    "ozonices": "ozonizar",
    "ozonicé": "ozonizar",
    "ozonicéis": "ozonizar",
    "ozonidas": "ozonida",
    "ozonifica": "ozonificar",
    "ozonificaba": "ozonificar",
    "ozonificabais": "ozonificar",
    "ozonificaban": "ozonificar",
    "ozonificabas": "ozonificar",
    "ozonificad": "ozonificar",
    "ozonificada": "ozonificar",
    "ozonificadas": "ozonificar",
    "ozonificado": "ozonificar",
    "ozonificados": "ozonificar",
    "ozonificamos": "ozonificar",
    "ozonifican": "ozonificar",
    "ozonificando": "ozonificar",
    "ozonificara": "ozonificar",
    "ozonificarais": "ozonificar",
    "ozonificaran": "ozonificar",
    "ozonificaras": "ozonificar",
    "ozonificare": "ozonificar",
    "ozonificareis": "ozonificar",
    "ozonificaremos": "ozonificar",
    "ozonificaren": "ozonificar",
    "ozonificares": "ozonificar",
    "ozonificaron": "ozonificar",
    "ozonificará": "ozonificar",
    "ozonificarán": "ozonificar",
    "ozonificarás": "ozonificar",
    "ozonificaré": "ozonificar",
    "ozonificaréis": "ozonificar",
    "ozonificaría": "ozonificar",
    "ozonificaríais": "ozonificar",
    "ozonificaríamos": "ozonificar",
    "ozonificarían": "ozonificar",
    "ozonificarías": "ozonificar",
    "ozonificas": "ozonificar",
    "ozonificase": "ozonificar",
    "ozonificaseis": "ozonificar",
    "ozonificasen": "ozonificar",
    "ozonificases": "ozonificar",
    "ozonificaste": "ozonificar",
    "ozonificasteis": "ozonificar",
    "ozonifico": "ozonificar",
    "ozonificábamos": "ozonificar",
    "ozonificáis": "ozonificar",
    "ozonificáramos": "ozonificar",
    "ozonificáremos": "ozonificar",
    "ozonificásemos": "ozonificar",
    "ozonificó": "ozonificar",
    "ozoniza": "ozonizar",
    "ozonizaba": "ozonizar",
    "ozonizabais": "ozonizar",
    "ozonizaban": "ozonizar",
    "ozonizabas": "ozonizar",
    "ozonizaciones": "ozonización",
    "ozonizad": "ozonizar",
    "ozonizada": "ozonizar",
    "ozonizadas": "ozonizar",
    "ozonizado": "ozonizar",
    "ozonizadora": "ozonizador",
    "ozonizadoras": "ozonizador",
    "ozonizadores": "ozonizador",
    "ozonizados": "ozonizar",
    "ozonizamos": "ozonizar",
    "ozonizan": "ozonizar",
    "ozonizando": "ozonizar",
    "ozonizara": "ozonizar",
    "ozonizarais": "ozonizar",
    "ozonizaran": "ozonizar",
    "ozonizaras": "ozonizar",
    "ozonizare": "ozonizar",
    "ozonizareis": "ozonizar",
    "ozonizaremos": "ozonizar",
    "ozonizaren": "ozonizar",
    "ozonizares": "ozonizar",
    "ozonizaron": "ozonizar",
    "ozonizará": "ozonizar",
    "ozonizarán": "ozonizar",
    "ozonizarás": "ozonizar",
    "ozonizaré": "ozonizar",
    "ozonizaréis": "ozonizar",
    "ozonizaría": "ozonizar",
    "ozonizaríais": "ozonizar",
    "ozonizaríamos": "ozonizar",
    "ozonizarían": "ozonizar",
    "ozonizarías": "ozonizar",
    "ozonizas": "ozonizar",
    "ozonizase": "ozonizar",
    "ozonizaseis": "ozonizar",
    "ozonizasen": "ozonizar",
    "ozonizases": "ozonizar",
    "ozonizaste": "ozonizar",
    "ozonizasteis": "ozonizar",
    "ozonizo": "ozonizar",
    "ozonizábamos": "ozonizar",
    "ozonizáis": "ozonizar",
    "ozonizáramos": "ozonizar",
    "ozonizáremos": "ozonizar",
    "ozonizásemos": "ozonizar",
    "ozonizó": "ozonizar",
    "ozono": "ozonar",
    "ozonofique": "ozonificar",
    "ozonofiquemos": "ozonificar",
    "ozonofiquen": "ozonificar",
    "ozonofiques": "ozonificar",
    "ozonofiqué": "ozonificar",
    "ozonofiquéis": "ozonificar",
    "ozonos": "ozono",
    "ozonoscópica": "ozonoscópico",
    "ozonoscópicas": "ozonoscópico",
    "ozonoscópicos": "ozonoscópico",
    "ozonosferas": "ozonosfera",
    "ozonábamos": "ozonar",
    "ozonáis": "ozonar",
    "ozonáramos": "ozonar",
    "ozonáremos": "ozonar",
    "ozonásemos": "ozonar",
    "ozoné": "ozonar",
    "ozonéis": "ozonar",
    "ozonó": "ozonar",
    "ozonómetros": "ozonómetro",
    "ozoqueritas": "ozoquerita",
    "ozónidos": "ozónido",
    "oí": "oír",
    "oía": "oír",
    "oíais": "oír",
    "oíamos": "oír",
    "oían": "oír",
    "oías": "oír",
    "oíbles": "oíble",
    "oíd": "oír",
    "oída": "oír",
    "oídas": "oír",
    "oídios": "oídio",
    "oído": "oír",
    "oídos": "oír",
    "oímos": "oír",
    "oís": "oír",
    "oíste": "oír",
    "oísteis": "oír",
    "oófora": "oóforo",
    "oóforas": "oóforo",
    "oóforos": "oóforo",
    "oótidas": "oótida",
    "p": "página",
    "p-clorofenoles": "p-clorofenol",
    "pabellones": "pabellón",
    "pabilones": "pabilón",
    "pabilos": "pabilo",
    "pabilosa": "pabiloso",
    "pabilosas": "pabiloso",
    "pabilosos": "pabiloso",
    "pabla": "pablar",
    "pablaba": "pablar",
    "pablabais": "pablar",
    "pablaban": "pablar",
    "pablabas": "pablar",
    "pablad": "pablar",
    "pablada": "pablar",
    "pabladas": "pablar",
    "pablado": "pablar",
    "pablados": "pablar",
    "pablamos": "pablar",
    "pablan": "pablar",
    "pablando": "pablar",
    "pablara": "pablar",
    "pablarais": "pablar",
    "pablaran": "pablar",
    "pablaras": "pablar",
    "pablare": "pablar",
    "pablareis": "pablar",
    "pablaremos": "pablar",
    "pablaren": "pablar",
    "pablares": "pablar",
    "pablaron": "pablar",
    "pablará": "pablar",
    "pablarán": "pablar",
    "pablarás": "pablar",
    "pablaré": "pablar",
    "pablaréis": "pablar",
    "pablaría": "pablar",
    "pablaríais": "pablar",
    "pablaríamos": "pablar",
    "pablarían": "pablar",
    "pablarías": "pablar",
    "pablas": "pablar",
    "pablase": "pablar",
    "pablaseis": "pablar",
    "pablasen": "pablar",
    "pablases": "pablar",
    "pablaste": "pablar",
    "pablasteis": "pablar",
    "pable": "pablar",
    "pablemos": "pablar",
    "pablen": "pablar",
    "pables": "pablar",
    "pablo": "pablar",
    "pablos": "pablo",
    "pablábamos": "pablar",
    "pabláis": "pablar",
    "pabláramos": "pablar",
    "pabláremos": "pablar",
    "pablásemos": "pablar",
    "pablé": "pablar",
    "pabléis": "pablar",
    "pabló": "pablar",
    "pacaes": "pacay",
    "pacanas": "pacana",
    "pacaranas": "pacarana",
    "pacas": "paca",
    "pacata": "pacato",
    "pacatas": "pacato",
    "pacatos": "pacato",
    "pacayas": "pacaya",
    "pacayes": "pacay",
    "pace": "pacer",
    "paced": "pacer",
    "pacedera": "pacedero",
    "pacederas": "pacedero",
    "pacederos": "pacedero",
    "paceduras": "pacedura",
    "pacemos": "pacer",
    "pacen": "pacer",
    "pacenses": "pacense",
    "paceremos": "pacer",
    "pacerá": "pacer",
    "pacerán": "pacer",
    "pacerás": "pacer",
    "paceré": "pacer",
    "paceréis": "pacer",
    "pacería": "pacer",
    "paceríais": "pacer",
    "paceríamos": "pacer",
    "pacerían": "pacer",
    "pacerías": "pacer",
    "paces": "paz",
    "paceña": "paceño",
    "paceñas": "paceño",
    "paceños": "paceño",
    "pacha": "pacho",
    "pachaes": "pachá",
    "pachamancas": "pachamanca",
    "pachas": "pacho",
    "pachona": "pachón",
    "pachonas": "pachón",
    "pachones": "pachón",
    "pachorras": "pachorra",
    "pachorruda": "pachorrudo",
    "pachorrudas": "pachorrudo",
    "pachorrudos": "pachorrudo",
    "pachos": "pacho",
    "pachucha": "pachucho",
    "pachuchas": "pachucho",
    "pachuchos": "pachucho",
    "pachulíes": "pachulí",
    "pachulís": "pachulí",
    "pachás": "pachá",
    "pacida": "pacer",
    "pacidas": "pacer",
    "pacido": "pacer",
    "pacidos": "pacer",
    "paciencias": "paciencia",
    "paciendo": "pacer",
    "pacientes": "paciente",
    "pacienzuda": "pacienzudo",
    "pacienzudas": "pacienzudo",
    "pacienzudos": "pacienzudo",
    "paciera": "pacer",
    "pacierais": "pacer",
    "pacieran": "pacer",
    "pacieras": "pacer",
    "paciere": "pacer",
    "paciereis": "pacer",
    "pacieren": "pacer",
    "pacieres": "pacer",
    "pacieron": "pacer",
    "paciese": "pacer",
    "pacieseis": "pacer",
    "paciesen": "pacer",
    "pacieses": "pacer",
    "pacifica": "pacificar",
    "pacificaba": "pacificar",
    "pacificabais": "pacificar",
    "pacificaban": "pacificar",
    "pacificabas": "pacificar",
    "pacificaciones": "pacificación",
    "pacificaciónes": "pacificación",
    "pacificad": "pacificar",
    "pacificada": "pacificar",
    "pacificadas": "pacificar",
    "pacificado": "pacificar",
    "pacificadora": "pacificador",
    "pacificadoras": "pacificador",
    "pacificadores": "pacificador",
    "pacificados": "pacificar",
    "pacificamos": "pacificar",
    "pacifican": "pacificar",
    "pacificando": "pacificar",
    "pacificara": "pacificar",
    "pacificarais": "pacificar",
    "pacificaran": "pacificar",
    "pacificaras": "pacificar",
    "pacificare": "pacificar",
    "pacificareis": "pacificar",
    "pacificaremos": "pacificar",
    "pacificaren": "pacificar",
    "pacificares": "pacificar",
    "pacificaron": "pacificar",
    "pacificará": "pacificar",
    "pacificarán": "pacificar",
    "pacificarás": "pacificar",
    "pacificaré": "pacificar",
    "pacificaréis": "pacificar",
    "pacificaría": "pacificar",
    "pacificaríais": "pacificar",
    "pacificaríamos": "pacificar",
    "pacificarían": "pacificar",
    "pacificarías": "pacificar",
    "pacificas": "pacificar",
    "pacificase": "pacificar",
    "pacificaseis": "pacificar",
    "pacificasen": "pacificar",
    "pacificases": "pacificar",
    "pacificaste": "pacificar",
    "pacificasteis": "pacificar",
    "pacifico": "pacificar",
    "pacificábamos": "pacificar",
    "pacificáis": "pacificar",
    "pacificáramos": "pacificar",
    "pacificáremos": "pacificar",
    "pacificásemos": "pacificar",
    "pacificó": "pacificar",
    "pacifique": "pacificar",
    "pacifiquemos": "pacificar",
    "pacifiquen": "pacificar",
    "pacifiques": "pacificar",
    "pacifiqué": "pacificar",
    "pacifiquéis": "pacificar",
    "pacifismos": "pacifismo",
    "pacifistas": "pacifista",
    "pacimos": "pacer",
    "paciste": "pacer",
    "pacisteis": "pacer",
    "paciéramos": "pacer",
    "paciéremos": "pacer",
    "paciésemos": "pacer",
    "pació": "pacer",
    "packs": "pack",
    "pacos": "paco",
    "pacotillas": "pacotilla",
    "pacotillera": "pacotillero",
    "pacotilleras": "pacotillero",
    "pacotilleros": "pacotillero",
    "pacta": "pactar",
    "pactaba": "pactar",
    "pactabais": "pactar",
    "pactaban": "pactar",
    "pactabas": "pactar",
    "pactad": "pactar",
    "pactada": "pactar",
    "pactadas": "pactar",
    "pactado": "pactar",
    "pactados": "pactar",
    "pactamos": "pactar",
    "pactan": "pactar",
    "pactando": "pactar",
    "pactantes": "pactante",
    "pactara": "pactar",
    "pactarais": "pactar",
    "pactaran": "pactar",
    "pactaras": "pactar",
    "pactare": "pactar",
    "pactareis": "pactar",
    "pactaremos": "pactar",
    "pactaren": "pactar",
    "pactares": "pactar",
    "pactaron": "pactar",
    "pactará": "pactar",
    "pactarán": "pactar",
    "pactarás": "pactar",
    "pactaré": "pactar",
    "pactaréis": "pactar",
    "pactaría": "pactar",
    "pactaríais": "pactar",
    "pactaríamos": "pactar",
    "pactarían": "pactar",
    "pactarías": "pactar",
    "pactas": "pactar",
    "pactase": "pactar",
    "pactaseis": "pactar",
    "pactasen": "pactar",
    "pactases": "pactar",
    "pactaste": "pactar",
    "pactasteis": "pactar",
    "pacte": "pactar",
    "pactemos": "pactar",
    "pacten": "pactar",
    "pactes": "pactar",
    "pactismos": "pactismo",
    "pactistas": "pactista",
    "pacto": "pactar",
    "pactos": "pacto",
    "pactábamos": "pactar",
    "pactáis": "pactar",
    "pactáramos": "pactar",
    "pactáremos": "pactar",
    "pactásemos": "pactar",
    "pacté": "pactar",
    "pactéis": "pactar",
    "pactó": "pactar",
    "pacéis": "pacer",
    "pací": "pacer",
    "pacía": "pacer",
    "pacíais": "pacer",
    "pacíamos": "pacer",
    "pacían": "pacer",
    "pacías": "pacer",
    "pacífica": "pacífico",
    "pacíficas": "pacífico",
    "pacíficos": "pacífico",
    "padauks": "padauk",
    "paddocks": "paddock",
    "padece": "padecer",
    "padeced": "padecer",
    "padecemos": "padecer",
    "padecen": "padecer",
    "padeceremos": "padecer",
    "padecerá": "padecer",
    "padecerán": "padecer",
    "padecerás": "padecer",
    "padeceré": "padecer",
    "padeceréis": "padecer",
    "padecería": "padecer",
    "padeceríais": "padecer",
    "padeceríamos": "padecer",
    "padecerían": "padecer",
    "padecerías": "padecer",
    "padeces": "padecer",
    "padecida": "padecer",
    "padecidas": "padecer",
    "padecido": "padecer",
    "padecidos": "padecer",
    "padeciendo": "padecer",
    "padeciera": "padecer",
    "padecierais": "padecer",
    "padecieran": "padecer",
    "padecieras": "padecer",
    "padeciere": "padecer",
    "padeciereis": "padecer",
    "padecieren": "padecer",
    "padecieres": "padecer",
    "padecieron": "padecer",
    "padeciese": "padecer",
    "padecieseis": "padecer",
    "padeciesen": "padecer",
    "padecieses": "padecer",
    "padecimientos": "padecimiento",
    "padecimos": "padecer",
    "padeciste": "padecer",
    "padecisteis": "padecer",
    "padeciéramos": "padecer",
    "padeciéremos": "padecer",
    "padeciésemos": "padecer",
    "padeció": "padecer",
    "padecéis": "padecer",
    "padecí": "padecer",
    "padecía": "padecer",
    "padecíais": "padecer",
    "padecíamos": "padecer",
    "padecían": "padecer",
    "padecías": "padecer",
    "padezca": "padecer",
    "padezcamos": "padecer",
    "padezcan": "padecer",
    "padezcas": "padecer",
    "padezco": "padecer",
    "padezcáis": "padecer",
    "padillas": "padilla",
    "padrastros": "padrastro",
    "padrazos": "padrazo",
    "padrea": "padrear",
    "padreaba": "padrear",
    "padreabais": "padrear",
    "padreaban": "padrear",
    "padreabas": "padrear",
    "padread": "padrear",
    "padreada": "padrear",
    "padreadas": "padrear",
    "padreado": "padrear",
    "padreados": "padrear",
    "padreamos": "padrear",
    "padrean": "padrear",
    "padreando": "padrear",
    "padreara": "padrear",
    "padrearais": "padrear",
    "padrearan": "padrear",
    "padrearas": "padrear",
    "padreare": "padrear",
    "padreareis": "padrear",
    "padrearemos": "padrear",
    "padrearen": "padrear",
    "padreares": "padrear",
    "padrearon": "padrear",
    "padreará": "padrear",
    "padrearán": "padrear",
    "padrearás": "padrear",
    "padrearé": "padrear",
    "padrearéis": "padrear",
    "padrearía": "padrear",
    "padrearíais": "padrear",
    "padrearíamos": "padrear",
    "padrearían": "padrear",
    "padrearías": "padrear",
    "padreas": "padrear",
    "padrease": "padrear",
    "padreaseis": "padrear",
    "padreasen": "padrear",
    "padreases": "padrear",
    "padreaste": "padrear",
    "padreasteis": "padrear",
    "padree": "padrear",
    "padreemos": "padrear",
    "padreen": "padrear",
    "padrees": "padrear",
    "padrejones": "padrejón",
    "padrenuestros": "padrenuestro",
    "padreo": "padrear",
    "padres": "padre",
    "padreábamos": "padrear",
    "padreáis": "padrear",
    "padreáramos": "padrear",
    "padreáremos": "padrear",
    "padreásemos": "padrear",
    "padreé": "padrear",
    "padreéis": "padrear",
    "padreó": "padrear",
    "padrinas": "padrina",
    "padrinazgos": "padrinazgo",
    "padrinos": "padrino",
    "padrones": "padrón",
    "paduana": "paduano",
    "paduanas": "paduano",
    "paduanos": "paduano",
    "paellas": "paella",
    "paelleras": "paellera",
    "paflones": "paflón",
    "paga": "pagar",
    "pagaba": "pagar",
    "pagabais": "pagar",
    "pagaban": "pagar",
    "pagabas": "pagar",
    "pagables": "pagable",
    "pagad": "pagar",
    "pagada": "pagar",
    "pagadas": "pagar",
    "pagadera": "pagadero",
    "pagaderas": "pagadero",
    "pagaderos": "pagadero",
    "pagado": "pagar",
    "pagadora": "pagador",
    "pagadoras": "pagador",
    "pagadores": "pagador",
    "pagados": "pagar",
    "pagadurías": "pagaduría",
    "pagamentos": "pagamento",
    "pagamos": "pagar",
    "pagan": "pagar",
    "pagana": "pagano",
    "paganas": "pagano",
    "pagando": "pagar",
    "paganice": "paganizar",
    "paganicemos": "paganizar",
    "paganicen": "paganizar",
    "paganices": "paganizar",
    "paganicé": "paganizar",
    "paganicéis": "paganizar",
    "paganismos": "paganismo",
    "paganiza": "paganizar",
    "paganizaba": "paganizar",
    "paganizabais": "paganizar",
    "paganizaban": "paganizar",
    "paganizabas": "paganizar",
    "paganizad": "paganizar",
    "paganizada": "paganizar",
    "paganizadas": "paganizar",
    "paganizado": "paganizar",
    "paganizados": "paganizar",
    "paganizamos": "paganizar",
    "paganizan": "paganizar",
    "paganizando": "paganizar",
    "paganizara": "paganizar",
    "paganizarais": "paganizar",
    "paganizaran": "paganizar",
    "paganizaras": "paganizar",
    "paganizare": "paganizar",
    "paganizareis": "paganizar",
    "paganizaremos": "paganizar",
    "paganizaren": "paganizar",
    "paganizares": "paganizar",
    "paganizaron": "paganizar",
    "paganizará": "paganizar",
    "paganizarán": "paganizar",
    "paganizarás": "paganizar",
    "paganizaré": "paganizar",
    "paganizaréis": "paganizar",
    "paganizaría": "paganizar",
    "paganizaríais": "paganizar",
    "paganizaríamos": "paganizar",
    "paganizarían": "paganizar",
    "paganizarías": "paganizar",
    "paganizas": "paganizar",
    "paganizase": "paganizar",
    "paganizaseis": "paganizar",
    "paganizasen": "paganizar",
    "paganizases": "paganizar",
    "paganizaste": "paganizar",
    "paganizasteis": "paganizar",
    "paganizo": "paganizar",
    "paganizábamos": "paganizar",
    "paganizáis": "paganizar",
    "paganizáramos": "paganizar",
    "paganizáremos": "paganizar",
    "paganizásemos": "paganizar",
    "paganizó": "paganizar",
    "paganos": "pagano",
    "pagara": "pagar",
    "pagarais": "pagar",
    "pagaran": "pagar",
    "pagaras": "pagar",
    "pagare": "pagar",
    "pagareis": "pagar",
    "pagaremos": "pagar",
    "pagaren": "pagar",
    "pagares": "pagar",
    "pagaron": "pagar",
    "pagará": "pagar",
    "pagarán": "pagar",
    "pagarás": "pagar",
    "pagaré": "pagar",
    "pagaréis": "pagar",
    "pagarés": "pagaré",
    "pagaría": "pagar",
    "pagaríais": "pagar",
    "pagaríamos": "pagar",
    "pagarían": "pagar",
    "pagarías": "pagar",
    "pagas": "pagar",
    "pagase": "pagar",
    "pagaseis": "pagar",
    "pagasen": "pagar",
    "pagases": "pagar",
    "pagaste": "pagar",
    "pagasteis": "pagar",
    "pagazas": "pagaza",
    "pageles": "pagel",
    "pagina": "paginar",
    "paginaba": "paginar",
    "paginabais": "paginar",
    "paginaban": "paginar",
    "paginabas": "paginar",
    "paginables": "paginable",
    "paginaciones": "paginación",
    "paginad": "paginar",
    "paginada": "paginar",
    "paginadas": "paginar",
    "paginado": "paginar",
    "paginados": "paginar",
    "paginamos": "paginar",
    "paginan": "paginar",
    "paginando": "paginar",
    "paginara": "paginar",
    "paginarais": "paginar",
    "paginaran": "paginar",
    "paginaras": "paginar",
    "paginare": "paginar",
    "paginareis": "paginar",
    "paginaremos": "paginar",
    "paginaren": "paginar",
    "paginares": "paginar",
    "paginaron": "paginar",
    "paginará": "paginar",
    "paginarán": "paginar",
    "paginarás": "paginar",
    "paginaré": "paginar",
    "paginaréis": "paginar",
    "paginaría": "paginar",
    "paginaríais": "paginar",
    "paginaríamos": "paginar",
    "paginarían": "paginar",
    "paginarías": "paginar",
    "paginas": "paginar",
    "paginase": "paginar",
    "paginaseis": "paginar",
    "paginasen": "paginar",
    "paginases": "paginar",
    "paginaste": "paginar",
    "paginasteis": "paginar",
    "pagine": "paginar",
    "paginemos": "paginar",
    "paginen": "paginar",
    "pagines": "paginar",
    "pagino": "paginar",
    "paginábamos": "paginar",
    "pagináis": "paginar",
    "pagináramos": "paginar",
    "pagináremos": "paginar",
    "paginásemos": "paginar",
    "paginé": "paginar",
    "paginéis": "paginar",
    "paginó": "paginar",
    "pago": "pagar",
    "pagodas": "pagoda",
    "pagoditas": "pagodita",
    "pagos": "pago",
    "pagotes": "pagote",
    "pague": "pagar",
    "paguemos": "pagar",
    "paguen": "pagar",
    "pagues": "pagar",
    "pagué": "pagar",
    "paguéis": "pagar",
    "pagábamos": "pagar",
    "pagáis": "pagar",
    "pagáramos": "pagar",
    "pagáremos": "pagar",
    "pagásemos": "pagar",
    "pagó": "pagar",
    "paidologías": "paidología",
    "paidológica": "paidológico",
    "paidológicas": "paidológico",
    "paidológicos": "paidológico",
    "paidóloga": "paidólogo",
    "paidólogas": "paidólogo",
    "paidólogos": "paidólogo",
    "pailas": "paila",
    "pailebot": "pailebote",
    "pailebotes": "pailebote",
    "pailebots": "pailebot",
    "paineles": "painel",
    "painitas": "painita",
    "paipais": "paipai",
    "paipáis": "paipay",
    "paira": "pairar",
    "pairaba": "pairar",
    "pairabais": "pairar",
    "pairaban": "pairar",
    "pairabas": "pairar",
    "pairad": "pairar",
    "pairada": "pairar",
    "pairadas": "pairar",
    "pairado": "pairar",
    "pairados": "pairar",
    "pairamos": "pairar",
    "pairan": "pairar",
    "pairando": "pairar",
    "pairara": "pairar",
    "pairarais": "pairar",
    "pairaran": "pairar",
    "pairaras": "pairar",
    "pairare": "pairar",
    "pairareis": "pairar",
    "pairaremos": "pairar",
    "pairaren": "pairar",
    "pairares": "pairar",
    "pairaron": "pairar",
    "pairará": "pairar",
    "pairarán": "pairar",
    "pairarás": "pairar",
    "pairaré": "pairar",
    "pairaréis": "pairar",
    "pairaría": "pairar",
    "pairaríais": "pairar",
    "pairaríamos": "pairar",
    "pairarían": "pairar",
    "pairarías": "pairar",
    "pairas": "pairar",
    "pairase": "pairar",
    "pairaseis": "pairar",
    "pairasen": "pairar",
    "pairases": "pairar",
    "pairaste": "pairar",
    "pairasteis": "pairar",
    "paire": "pairar",
    "pairemos": "pairar",
    "pairen": "pairar",
    "paires": "pairar",
    "pairo": "pairar",
    "pairábamos": "pairar",
    "pairáis": "pairar",
    "pairáramos": "pairar",
    "pairáremos": "pairar",
    "pairásemos": "pairar",
    "pairé": "pairar",
    "pairéis": "pairar",
    "pairó": "pairar",
    "paisajes": "paisaje",
    "paisajismos": "paisajismo",
    "paisajistas": "paisajista",
    "paisajística": "paisajístico",
    "paisajísticas": "paisajístico",
    "paisajísticos": "paisajístico",
    "paisana": "paisano",
    "paisanajes": "paisanaje",
    "paisanas": "paisano",
    "paisanos": "paisano",
    "pajada": "pajado",
    "pajadas": "pajado",
    "pajados": "pajado",
    "pajarea": "pajarear",
    "pajareaba": "pajarear",
    "pajareabais": "pajarear",
    "pajareaban": "pajarear",
    "pajareabas": "pajarear",
    "pajaread": "pajarear",
    "pajareada": "pajarear",
    "pajareadas": "pajarear",
    "pajareado": "pajarear",
    "pajareados": "pajarear",
    "pajareamos": "pajarear",
    "pajarean": "pajarear",
    "pajareando": "pajarear",
    "pajareara": "pajarear",
    "pajarearais": "pajarear",
    "pajarearan": "pajarear",
    "pajarearas": "pajarear",
    "pajareare": "pajarear",
    "pajareareis": "pajarear",
    "pajarearemos": "pajarear",
    "pajarearen": "pajarear",
    "pajareares": "pajarear",
    "pajarearon": "pajarear",
    "pajareará": "pajarear",
    "pajarearán": "pajarear",
    "pajarearás": "pajarear",
    "pajarearé": "pajarear",
    "pajarearéis": "pajarear",
    "pajarearía": "pajarear",
    "pajarearíais": "pajarear",
    "pajarearíamos": "pajarear",
    "pajarearían": "pajarear",
    "pajarearías": "pajarear",
    "pajareas": "pajarear",
    "pajarease": "pajarear",
    "pajareaseis": "pajarear",
    "pajareasen": "pajarear",
    "pajareases": "pajarear",
    "pajareaste": "pajarear",
    "pajareasteis": "pajarear",
    "pajaree": "pajarear",
    "pajareemos": "pajarear",
    "pajareen": "pajarear",
    "pajarees": "pajarear",
    "pajareo": "pajarear",
    "pajarera": "pajarero",
    "pajareras": "pajarero",
    "pajareros": "pajarero",
    "pajarerías": "pajarería",
    "pajares": "pajar",
    "pajaretes": "pajarete",
    "pajareábamos": "pajarear",
    "pajareáis": "pajarear",
    "pajareáramos": "pajarear",
    "pajareáremos": "pajarear",
    "pajareásemos": "pajarear",
    "pajareé": "pajarear",
    "pajareéis": "pajarear",
    "pajareó": "pajarear",
    "pajarillas": "pajarilla",
    "pajarillos": "pajarillo",
    "pajaritas": "pajarita",
    "pajaritos": "pajarito",
    "pajarotadas": "pajarotada",
    "pajarotas": "pajarota",
    "pajarracos": "pajarraco",
    "pajarucos": "pajaruco",
    "pajas": "paja",
    "pajazas": "pajaza",
    "pajazos": "pajazo",
    "pajera": "pajero",
    "pajeras": "pajero",
    "pajeros": "pajero",
    "pajerías": "pajería",
    "pajes": "paje",
    "pajiles": "pajil",
    "pajillas": "pajilla",
    "pajitas": "pajita",
    "pajiza": "pajizo",
    "pajizas": "pajizo",
    "pajizos": "pajizo",
    "pajolera": "pajolero",
    "pajoleras": "pajolero",
    "pajoleros": "pajolero",
    "pajonales": "pajonal",
    "pajones": "pajón",
    "pajosa": "pajoso",
    "pajosas": "pajoso",
    "pajosos": "pajoso",
    "pajotes": "pajote",
    "pajuelas": "pajuela",
    "pajuna": "pajuno",
    "pajunas": "pajuno",
    "pajunos": "pajuno",
    "pakistanies": "pakistaní",
    "pakistaníes": "pakistaní",
    "pakistanís": "pakistaní",
    "palabradas": "palabrada",
    "palabras": "palabra",
    "palabrejas": "palabreja",
    "palabreos": "palabreo",
    "palabrera": "palabrero",
    "palabreras": "palabrero",
    "palabreros": "palabrero",
    "palabrerías": "palabrería",
    "palabristas": "palabrista",
    "palabritas": "palabrita",
    "palabros": "palabro",
    "palabrotas": "palabrota",
    "palacetes": "palacete",
    "palaciales": "palacial",
    "palaciega": "palaciego",
    "palaciegas": "palaciego",
    "palaciegos": "palaciego",
    "palacios": "palacio",
    "palacranas": "palacrana",
    "palacras": "palacra",
    "palada": "palado",
    "paladares": "paladar",
    "paladas": "palado",
    "paladea": "paladear",
    "paladeaba": "paladear",
    "paladeabais": "paladear",
    "paladeaban": "paladear",
    "paladeabas": "paladear",
    "paladead": "paladear",
    "paladeada": "paladear",
    "paladeadas": "paladear",
    "paladeado": "paladear",
    "paladeados": "paladear",
    "paladeamos": "paladear",
    "paladean": "paladear",
    "paladeando": "paladear",
    "paladeara": "paladear",
    "paladearais": "paladear",
    "paladearan": "paladear",
    "paladearas": "paladear",
    "paladeare": "paladear",
    "paladeareis": "paladear",
    "paladearemos": "paladear",
    "paladearen": "paladear",
    "paladeares": "paladear",
    "paladearon": "paladear",
    "paladeará": "paladear",
    "paladearán": "paladear",
    "paladearás": "paladear",
    "paladearé": "paladear",
    "paladearéis": "paladear",
    "paladearía": "paladear",
    "paladearíais": "paladear",
    "paladearíamos": "paladear",
    "paladearían": "paladear",
    "paladearías": "paladear",
    "paladeas": "paladear",
    "paladease": "paladear",
    "paladeaseis": "paladear",
    "paladeasen": "paladear",
    "paladeases": "paladear",
    "paladeaste": "paladear",
    "paladeasteis": "paladear",
    "paladee": "paladear",
    "paladeemos": "paladear",
    "paladeen": "paladear",
    "paladees": "paladear",
    "paladeo": "paladear",
    "paladeos": "paladeo",
    "paladeábamos": "paladear",
    "paladeáis": "paladear",
    "paladeáramos": "paladear",
    "paladeáremos": "paladear",
    "paladeásemos": "paladear",
    "paladeé": "paladear",
    "paladeéis": "paladear",
    "paladeó": "paladear",
    "paladiados": "paladiado",
    "paladiales": "paladial",
    "paladina": "paladino",
    "paladinas": "paladino",
    "paladines": "paladín",
    "paladinos": "paladino",
    "paladiones": "paladión",
    "paladios": "paladio",
    "paladiosa": "paladioso",
    "paladiosas": "paladioso",
    "paladiosos": "paladioso",
    "palados": "palado",
    "palafitos": "palafito",
    "palafrenera": "palafrenero",
    "palafreneras": "palafrenero",
    "palafreneros": "palafrenero",
    "palafrenes": "palafrén",
    "palahierros": "palahierro",
    "palamentas": "palamenta",
    "palancadas": "palancada",
    "palancanas": "palancana",
    "palancas": "palanca",
    "palanganas": "palangana",
    "palanganeros": "palanganero",
    "palangrera": "palangrero",
    "palangreras": "palangrero",
    "palangreros": "palangrero",
    "palangres": "palangre",
    "palanquetas": "palanqueta",
    "palanquillas": "palanquilla",
    "palanquines": "palanquín",
    "palanquitas": "palanquita",
    "palas": "pala",
    "palastros": "palastro",
    "palatales": "palatal",
    "palatalice": "palatalizar",
    "palatalicemos": "palatalizar",
    "palatalicen": "palatalizar",
    "palatalices": "palatalizar",
    "palatalicé": "palatalizar",
    "palatalicéis": "palatalizar",
    "palataliza": "palatalizar",
    "palatalizaba": "palatalizar",
    "palatalizabais": "palatalizar",
    "palatalizaban": "palatalizar",
    "palatalizabas": "palatalizar",
    "palatalizaciones": "palatalización",
    "palatalizad": "palatalizar",
    "palatalizada": "palatalizar",
    "palatalizadas": "palatalizar",
    "palatalizado": "palatalizar",
    "palatalizados": "palatalizar",
    "palatalizamos": "palatalizar",
    "palatalizan": "palatalizar",
    "palatalizando": "palatalizar",
    "palatalizara": "palatalizar",
    "palatalizarais": "palatalizar",
    "palatalizaran": "palatalizar",
    "palatalizaras": "palatalizar",
    "palatalizare": "palatalizar",
    "palatalizareis": "palatalizar",
    "palatalizaremos": "palatalizar",
    "palatalizaren": "palatalizar",
    "palatalizares": "palatalizar",
    "palatalizaron": "palatalizar",
    "palatalizará": "palatalizar",
    "palatalizarán": "palatalizar",
    "palatalizarás": "palatalizar",
    "palatalizaré": "palatalizar",
    "palatalizaréis": "palatalizar",
    "palatalizaría": "palatalizar",
    "palatalizaríais": "palatalizar",
    "palatalizaríamos": "palatalizar",
    "palatalizarían": "palatalizar",
    "palatalizarías": "palatalizar",
    "palatalizas": "palatalizar",
    "palatalizase": "palatalizar",
    "palatalizaseis": "palatalizar",
    "palatalizasen": "palatalizar",
    "palatalizases": "palatalizar",
    "palatalizaste": "palatalizar",
    "palatalizasteis": "palatalizar",
    "palatalizo": "palatalizar",
    "palatalizábamos": "palatalizar",
    "palatalizáis": "palatalizar",
    "palatalizáramos": "palatalizar",
    "palatalizáremos": "palatalizar",
    "palatalizásemos": "palatalizar",
    "palatalizó": "palatalizar",
    "palatina": "palatino",
    "palatinados": "palatinado",
    "palatinas": "palatino",
    "palatinos": "palatino",
    "palatoetmoidales": "palatoetmoidal",
    "palatofaríngea": "palatofaríngeo",
    "palatofaríngeas": "palatofaríngeo",
    "palatofaríngeos": "palatofaríngeo",
    "palatoglosa": "palatogloso",
    "palatoglosas": "palatogloso",
    "palatoglosos": "palatogloso",
    "palatografías": "palatografía",
    "palatogramas": "palatograma",
    "palatográfica": "palatográfico",
    "palatográficas": "palatográfico",
    "palatográficos": "palatográfico",
    "palatomaxilares": "palatomaxilar",
    "palatonasales": "palatonasal",
    "palatooclusales": "palatooclusal",
    "palatoplastias": "palatoplastia",
    "palatorrafias": "palatorrafia",
    "palatovaginales": "palatovaginal",
    "palatógrafos": "palatógrafo",
    "palazones": "palazón",
    "palazos": "palazo",
    "palcos": "palco",
    "palea": "palear",
    "paleaba": "palear",
    "paleabais": "palear",
    "paleaban": "palear",
    "paleabas": "palear",
    "palead": "palear",
    "paleada": "palear",
    "paleadas": "palear",
    "paleado": "palear",
    "paleadora": "paleador",
    "paleadoras": "paleador",
    "paleadores": "paleador",
    "paleados": "palear",
    "paleamos": "palear",
    "palean": "palear",
    "paleando": "palear",
    "paleara": "palear",
    "palearais": "palear",
    "palearan": "palear",
    "palearas": "palear",
    "paleare": "palear",
    "paleareis": "palear",
    "palearemos": "palear",
    "palearen": "palear",
    "paleares": "palear",
    "palearon": "palear",
    "paleará": "palear",
    "palearán": "palear",
    "palearás": "palear",
    "palearé": "palear",
    "palearéis": "palear",
    "palearía": "palear",
    "palearíais": "palear",
    "palearíamos": "palear",
    "palearían": "palear",
    "palearías": "palear",
    "paleas": "palear",
    "palease": "palear",
    "paleaseis": "palear",
    "paleasen": "palear",
    "paleases": "palear",
    "paleaste": "palear",
    "paleasteis": "palear",
    "palee": "palear",
    "paleemos": "palear",
    "paleen": "palear",
    "palees": "palear",
    "palenques": "palenque",
    "palentina": "palentino",
    "palentinas": "palentino",
    "palentinos": "palentino",
    "paleo": "palear",
    "paleoantropologías": "paleoantropología",
    "paleobotánicas": "paleobotánica",
    "paleobotánicos": "paleobotánico",
    "paleoceanografías": "paleoceanografía",
    "paleocena": "paleoceno",
    "paleocenas": "paleoceno",
    "paleocenos": "paleoceno",
    "paleocerebelos": "paleocerebelo",
    "paleoclimas": "paleoclima",
    "paleoclimatologías": "paleoclimatología",
    "paleocrecidas": "paleocrecida",
    "paleocristiana": "paleocristiano",
    "paleocristianas": "paleocristiano",
    "paleocristianos": "paleocristiano",
    "paleocórtexes": "paleocórtex",
    "paleodemografías": "paleodemografía",
    "paleoecologías": "paleoecología",
    "paleoeconomías": "paleoeconomía",
    "paleoetnobotánicas": "paleoetnobotánica",
    "paleoetnologías": "paleoetnología",
    "paleografías": "paleografía",
    "paleográfica": "paleográfico",
    "paleográficas": "paleográfico",
    "paleográficos": "paleográfico",
    "paleolítica": "paleolítico",
    "paleolíticas": "paleolítico",
    "paleolíticos": "paleolítico",
    "paleomagnetismos": "paleomagnetismo",
    "paleontografías": "paleontografía",
    "paleontográfica": "paleontográfico",
    "paleontográficas": "paleontográfico",
    "paleontográficos": "paleontográfico",
    "paleontologías": "paleontología",
    "paleontológica": "paleontológico",
    "paleontológicas": "paleontológico",
    "paleontológicos": "paleontológico",
    "paleontóloga": "paleontólogo",
    "paleontólogas": "paleontólogo",
    "paleontólogos": "paleontólogo",
    "paleopalios": "paleopalio",
    "paleopatologías": "paleopatología",
    "paleopatóloga": "paleopatólogo",
    "paleopatólogas": "paleopatólogo",
    "paleopatólogos": "paleopatólogo",
    "paleos": "paleo",
    "paleosoles": "paleosol",
    "paleotropicales": "paleotropical",
    "paleozoica": "paleozoico",
    "paleozoicas": "paleozoico",
    "paleozoicos": "paleozoico",
    "paleozoologías": "paleozoología",
    "palera": "palero",
    "paleras": "palero",
    "paleros": "palero",
    "palerías": "palería",
    "pales": "pal",
    "palestina": "palestino",
    "palestinas": "palestino",
    "palestinos": "palestino",
    "palestras": "palestra",
    "palestritas": "palestrita",
    "paleta": "paleto",
    "paletadas": "paletada",
    "paletas": "paleto",
    "paletazos": "paletazo",
    "paletea": "paletear",
    "paleteaba": "paletear",
    "paleteabais": "paletear",
    "paleteaban": "paletear",
    "paleteabas": "paletear",
    "paletead": "paletear",
    "paleteada": "paletear",
    "paleteadas": "paletear",
    "paleteado": "paletear",
    "paleteados": "paletear",
    "paleteamos": "paletear",
    "paletean": "paletear",
    "paleteando": "paletear",
    "paleteara": "paletear",
    "paletearais": "paletear",
    "paletearan": "paletear",
    "paletearas": "paletear",
    "paleteare": "paletear",
    "paleteareis": "paletear",
    "paletearemos": "paletear",
    "paletearen": "paletear",
    "paleteares": "paletear",
    "paletearon": "paletear",
    "paleteará": "paletear",
    "paletearán": "paletear",
    "paletearás": "paletear",
    "paletearé": "paletear",
    "paletearéis": "paletear",
    "paletearía": "paletear",
    "paletearíais": "paletear",
    "paletearíamos": "paletear",
    "paletearían": "paletear",
    "paletearías": "paletear",
    "paleteas": "paletear",
    "paletease": "paletear",
    "paleteaseis": "paletear",
    "paleteasen": "paletear",
    "paleteases": "paletear",
    "paleteaste": "paletear",
    "paleteasteis": "paletear",
    "paletee": "paletear",
    "paleteemos": "paletear",
    "paleteen": "paletear",
    "paletees": "paletear",
    "paleteo": "paletear",
    "paleteos": "paleteo",
    "paletera": "paletero",
    "paleteras": "paletero",
    "paleteros": "paletero",
    "paleteábamos": "paletear",
    "paleteáis": "paletear",
    "paleteáramos": "paletear",
    "paleteáremos": "paletear",
    "paleteásemos": "paletear",
    "paleteé": "paletear",
    "paleteéis": "paletear",
    "paleteó": "paletear",
    "paletillas": "paletilla",
    "paletinas": "paletina",
    "paletistas": "paletista",
    "paletizables": "paletizable",
    "paletizaciones": "paletización",
    "paletizada": "paletizado",
    "paletizadas": "paletizado",
    "paletizadoras": "paletizadora",
    "paletizados": "paletizado",
    "paletnologías": "paletnología",
    "paletones": "paletón",
    "paletos": "paleto",
    "palets": "palet",
    "paleábamos": "palear",
    "paleáis": "palear",
    "paleáramos": "palear",
    "paleáremos": "palear",
    "paleártica": "paleártico",
    "paleárticas": "paleártico",
    "paleárticos": "paleártico",
    "paleásemos": "palear",
    "paleé": "palear",
    "paleéis": "palear",
    "paleó": "palear",
    "paleógena": "paleógeno",
    "paleógenas": "paleógeno",
    "paleógenos": "paleógeno",
    "paleógrafa": "paleógrafo",
    "paleógrafas": "paleógrafo",
    "paleógrafos": "paleógrafo",
    "paleóloga": "paleólogo",
    "paleólogas": "paleólogo",
    "paleólogos": "paleólogo",
    "paliaba": "paliar",
    "paliabais": "paliar",
    "paliaban": "paliar",
    "paliabas": "paliar",
    "paliaciones": "paliación",
    "paliad": "paliar",
    "paliada": "paliar",
    "paliadas": "paliar",
    "paliado": "paliar",
    "paliados": "paliar",
    "paliamos": "paliar",
    "paliando": "paliar",
    "paliara": "paliar",
    "paliarais": "paliar",
    "paliaran": "paliar",
    "paliaras": "paliar",
    "paliare": "paliar",
    "paliareis": "paliar",
    "paliaremos": "paliar",
    "paliaren": "paliar",
    "paliares": "paliar",
    "paliaron": "paliar",
    "paliará": "paliar",
    "paliarán": "paliar",
    "paliarás": "paliar",
    "paliaré": "paliar",
    "paliaréis": "paliar",
    "paliaría": "paliar",
    "paliaríais": "paliar",
    "paliaríamos": "paliar",
    "paliarían": "paliar",
    "paliarías": "paliar",
    "palias": "palia",
    "paliase": "paliar",
    "paliaseis": "paliar",
    "paliasen": "paliar",
    "paliases": "paliar",
    "paliaste": "paliar",
    "paliasteis": "paliar",
    "paliativa": "paliativo",
    "paliativas": "paliativo",
    "paliativos": "paliativo",
    "paliatoria": "paliatorio",
    "paliatorias": "paliatorio",
    "paliatorios": "paliatorio",
    "palidece": "palidecer",
    "palideced": "palidecer",
    "palidecemos": "palidecer",
    "palidecen": "palidecer",
    "palideceremos": "palidecer",
    "palidecerá": "palidecer",
    "palidecerán": "palidecer",
    "palidecerás": "palidecer",
    "palideceré": "palidecer",
    "palideceréis": "palidecer",
    "palidecería": "palidecer",
    "palideceríais": "palidecer",
    "palideceríamos": "palidecer",
    "palidecerían": "palidecer",
    "palidecerías": "palidecer",
    "palideces": "palidez",
    "palidecida": "palidecer",
    "palidecidas": "palidecer",
    "palidecido": "palidecer",
    "palidecidos": "palidecer",
    "palideciendo": "palidecer",
    "palideciera": "palidecer",
    "palidecierais": "palidecer",
    "palidecieran": "palidecer",
    "palidecieras": "palidecer",
    "palideciere": "palidecer",
    "palideciereis": "palidecer",
    "palidecieren": "palidecer",
    "palidecieres": "palidecer",
    "palidecieron": "palidecer",
    "palideciese": "palidecer",
    "palidecieseis": "palidecer",
    "palideciesen": "palidecer",
    "palidecieses": "palidecer",
    "palidecimos": "palidecer",
    "palideciste": "palidecer",
    "palidecisteis": "palidecer",
    "palideciéramos": "palidecer",
    "palideciéremos": "palidecer",
    "palideciésemos": "palidecer",
    "palideció": "palidecer",
    "palidecéis": "palidecer",
    "palidecí": "palidecer",
    "palidecía": "palidecer",
    "palidecíais": "palidecer",
    "palidecíamos": "palidecer",
    "palidecían": "palidecer",
    "palidecías": "palidecer",
    "palidezca": "palidecer",
    "palidezcamos": "palidecer",
    "palidezcan": "palidecer",
    "palidezcas": "palidecer",
    "palidezco": "palidecer",
    "palidezcáis": "palidecer",
    "paliducha": "paliducho",
    "paliduchas": "paliducho",
    "paliduchos": "paliducho",
    "paliemos": "paliar",
    "palieres": "palier",
    "palies": "pali",
    "palillera": "palillero",
    "palilleras": "palillero",
    "palilleros": "palillero",
    "palillos": "palillo",
    "palimpsestos": "palimpsesto",
    "palingenesias": "palingenesia",
    "palingenésica": "palingenésico",
    "palingenésicas": "palingenésico",
    "palingenésicos": "palingenésico",
    "palinodias": "palinodia",
    "palinologías": "palinología",
    "palinóloga": "palinólogo",
    "palinólogas": "palinólogo",
    "palinólogos": "palinólogo",
    "palios": "palio",
    "paliquea": "paliquear",
    "paliqueaba": "paliquear",
    "paliqueabais": "paliquear",
    "paliqueaban": "paliquear",
    "paliqueabas": "paliquear",
    "paliquead": "paliquear",
    "paliqueada": "paliquear",
    "paliqueadas": "paliquear",
    "paliqueado": "paliquear",
    "paliqueados": "paliquear",
    "paliqueamos": "paliquear",
    "paliquean": "paliquear",
    "paliqueando": "paliquear",
    "paliqueara": "paliquear",
    "paliquearais": "paliquear",
    "paliquearan": "paliquear",
    "paliquearas": "paliquear",
    "paliqueare": "paliquear",
    "paliqueareis": "paliquear",
    "paliquearemos": "paliquear",
    "paliquearen": "paliquear",
    "paliqueares": "paliquear",
    "paliquearon": "paliquear",
    "paliqueará": "paliquear",
    "paliquearán": "paliquear",
    "paliquearás": "paliquear",
    "paliquearé": "paliquear",
    "paliquearéis": "paliquear",
    "paliquearía": "paliquear",
    "paliquearíais": "paliquear",
    "paliquearíamos": "paliquear",
    "paliquearían": "paliquear",
    "paliquearías": "paliquear",
    "paliqueas": "paliquear",
    "paliquease": "paliquear",
    "paliqueaseis": "paliquear",
    "paliqueasen": "paliquear",
    "paliqueases": "paliquear",
    "paliqueaste": "paliquear",
    "paliqueasteis": "paliquear",
    "paliquee": "paliquear",
    "paliqueemos": "paliquear",
    "paliqueen": "paliquear",
    "paliquees": "paliquear",
    "paliqueo": "paliquear",
    "paliques": "palique",
    "paliqueábamos": "paliquear",
    "paliqueáis": "paliquear",
    "paliqueáramos": "paliquear",
    "paliqueáremos": "paliquear",
    "paliqueásemos": "paliquear",
    "paliqueé": "paliquear",
    "paliqueéis": "paliquear",
    "paliqueó": "paliquear",
    "palis": "pali",
    "palisandros": "palisandro",
    "palistas": "palista",
    "palitoques": "palitoque",
    "palitos": "palito",
    "palitroques": "palitroque",
    "palizadas": "palizada",
    "palizas": "paliza",
    "paliábamos": "paliar",
    "paliáis": "paliar",
    "paliáramos": "paliar",
    "paliáremos": "paliar",
    "paliásemos": "paliar",
    "palié": "paliar",
    "paliéis": "paliar",
    "palió": "paliar",
    "palla": "pallar",
    "pallaba": "pallar",
    "pallabais": "pallar",
    "pallaban": "pallar",
    "pallabas": "pallar",
    "pallad": "pallar",
    "pallada": "pallar",
    "palladas": "pallar",
    "pallado": "pallar",
    "pallados": "pallar",
    "pallamos": "pallar",
    "pallan": "pallar",
    "pallando": "pallar",
    "pallara": "pallar",
    "pallarais": "pallar",
    "pallaran": "pallar",
    "pallaras": "pallar",
    "pallare": "pallar",
    "pallareis": "pallar",
    "pallaremos": "pallar",
    "pallaren": "pallar",
    "pallares": "pallar",
    "pallaron": "pallar",
    "pallará": "pallar",
    "pallarán": "pallar",
    "pallarás": "pallar",
    "pallaré": "pallar",
    "pallaréis": "pallar",
    "pallaría": "pallar",
    "pallaríais": "pallar",
    "pallaríamos": "pallar",
    "pallarían": "pallar",
    "pallarías": "pallar",
    "pallas": "pallar",
    "pallase": "pallar",
    "pallaseis": "pallar",
    "pallasen": "pallar",
    "pallases": "pallar",
    "pallaste": "pallar",
    "pallasteis": "pallar",
    "palle": "pallar",
    "pallemos": "pallar",
    "pallen": "pallar",
    "palles": "pallar",
    "palletes": "pallete",
    "pallidums": "pallidum",
    "pallo": "pallar",
    "pallones": "pallón",
    "pallábamos": "pallar",
    "palláis": "pallar",
    "palláramos": "pallar",
    "palláremos": "pallar",
    "pallásemos": "pallar",
    "pallé": "pallar",
    "palléis": "pallar",
    "palló": "pallar",
    "palma": "palmar",
    "palmaba": "palmar",
    "palmabais": "palmar",
    "palmaban": "palmar",
    "palmabas": "palmar",
    "palmad": "palmar",
    "palmada": "palmar",
    "palmadas": "palmar",
    "palmado": "palmar",
    "palmados": "palmar",
    "palmamos": "palmar",
    "palman": "palmar",
    "palmando": "palmar",
    "palmara": "palmar",
    "palmarais": "palmar",
    "palmaran": "palmar",
    "palmaras": "palmar",
    "palmare": "palmar",
    "palmareis": "palmar",
    "palmaremos": "palmar",
    "palmaren": "palmar",
    "palmares": "palmar",
    "palmareses": "palmarés",
    "palmaria": "palmario",
    "palmarias": "palmario",
    "palmarios": "palmario",
    "palmaron": "palmar",
    "palmará": "palmar",
    "palmarán": "palmar",
    "palmarás": "palmar",
    "palmaré": "palmar",
    "palmaréis": "palmar",
    "palmaría": "palmar",
    "palmaríais": "palmar",
    "palmaríamos": "palmar",
    "palmarían": "palmar",
    "palmarías": "palmar",
    "palmas": "palmar",
    "palmase": "palmar",
    "palmaseis": "palmar",
    "palmasen": "palmar",
    "palmases": "palmar",
    "palmaste": "palmar",
    "palmasteis": "palmar",
    "palmatorias": "palmatoria",
    "palme": "palmar",
    "palmea": "palmear",
    "palmeaba": "palmear",
    "palmeabais": "palmear",
    "palmeaban": "palmear",
    "palmeabas": "palmear",
    "palmead": "palmear",
    "palmeada": "palmear",
    "palmeadas": "palmear",
    "palmeado": "palmear",
    "palmeados": "palmear",
    "palmeamos": "palmear",
    "palmean": "palmear",
    "palmeando": "palmear",
    "palmeara": "palmear",
    "palmearais": "palmear",
    "palmearan": "palmear",
    "palmearas": "palmear",
    "palmeare": "palmear",
    "palmeareis": "palmear",
    "palmearemos": "palmear",
    "palmearen": "palmear",
    "palmeares": "palmear",
    "palmearon": "palmear",
    "palmeará": "palmear",
    "palmearán": "palmear",
    "palmearás": "palmear",
    "palmearé": "palmear",
    "palmearéis": "palmear",
    "palmearía": "palmear",
    "palmearíais": "palmear",
    "palmearíamos": "palmear",
    "palmearían": "palmear",
    "palmearías": "palmear",
    "palmeas": "palmear",
    "palmease": "palmear",
    "palmeaseis": "palmear",
    "palmeasen": "palmear",
    "palmeases": "palmear",
    "palmeaste": "palmear",
    "palmeasteis": "palmear",
    "palmee": "palmear",
    "palmeemos": "palmear",
    "palmeen": "palmear",
    "palmees": "palmear",
    "palmejares": "palmejar",
    "palmemos": "palmar",
    "palmen": "palmar",
    "palmenses": "palmense",
    "palmeo": "palmear",
    "palmeos": "palmeo",
    "palmera": "palmero",
    "palmerales": "palmeral",
    "palmeras": "palmero",
    "palmeros": "palmero",
    "palmers": "palmer",
    "palmes": "palmar",
    "palmesana": "palmesano",
    "palmesanas": "palmesano",
    "palmesanos": "palmesano",
    "palmetas": "palmeta",
    "palmetazos": "palmetazo",
    "palmeábamos": "palmear",
    "palmeáis": "palmear",
    "palmeáramos": "palmear",
    "palmeáremos": "palmear",
    "palmeásemos": "palmear",
    "palmeé": "palmear",
    "palmeéis": "palmear",
    "palmeó": "palmear",
    "palmiches": "palmiche",
    "palmillas": "palmilla",
    "palmiras": "palmira",
    "palmitatos": "palmitato",
    "palmitiesa": "palmitieso",
    "palmitiesas": "palmitieso",
    "palmitiesos": "palmitieso",
    "palmitoleatos": "palmitoleato",
    "palmitoleica": "palmitoleico",
    "palmitoleicas": "palmitoleico",
    "palmitoleicos": "palmitoleico",
    "palmitos": "palmito",
    "palmo": "palmar",
    "palmos": "palmo",
    "palmotea": "palmotear",
    "palmoteaba": "palmotear",
    "palmoteabais": "palmotear",
    "palmoteaban": "palmotear",
    "palmoteabas": "palmotear",
    "palmotead": "palmotear",
    "palmoteada": "palmotear",
    "palmoteadas": "palmotear",
    "palmoteado": "palmotear",
    "palmoteados": "palmotear",
    "palmoteamos": "palmotear",
    "palmotean": "palmotear",
    "palmoteando": "palmotear",
    "palmoteara": "palmotear",
    "palmotearais": "palmotear",
    "palmotearan": "palmotear",
    "palmotearas": "palmotear",
    "palmoteare": "palmotear",
    "palmoteareis": "palmotear",
    "palmotearemos": "palmotear",
    "palmotearen": "palmotear",
    "palmoteares": "palmotear",
    "palmotearon": "palmotear",
    "palmoteará": "palmotear",
    "palmotearán": "palmotear",
    "palmotearás": "palmotear",
    "palmotearé": "palmotear",
    "palmotearéis": "palmotear",
    "palmotearía": "palmotear",
    "palmotearíais": "palmotear",
    "palmotearíamos": "palmotear",
    "palmotearían": "palmotear",
    "palmotearías": "palmotear",
    "palmoteas": "palmotear",
    "palmotease": "palmotear",
    "palmoteaseis": "palmotear",
    "palmoteasen": "palmotear",
    "palmoteases": "palmotear",
    "palmoteaste": "palmotear",
    "palmoteasteis": "palmotear",
    "palmotee": "palmotear",
    "palmoteemos": "palmotear",
    "palmoteen": "palmotear",
    "palmotees": "palmotear",
    "palmoteo": "palmotear",
    "palmoteos": "palmoteo",
    "palmoteábamos": "palmotear",
    "palmoteáis": "palmotear",
    "palmoteáramos": "palmotear",
    "palmoteáremos": "palmotear",
    "palmoteásemos": "palmotear",
    "palmoteé": "palmotear",
    "palmoteéis": "palmotear",
    "palmoteó": "palmotear",
    "palmábamos": "palmar",
    "palmácea": "palmáceo",
    "palmáceas": "palmáceo",
    "palmáceos": "palmáceo",
    "palmáis": "palmar",
    "palmáramos": "palmar",
    "palmáremos": "palmar",
    "palmásemos": "palmar",
    "palmé": "palmar",
    "palméis": "palmar",
    "palmífera": "palmífero",
    "palmíferas": "palmífero",
    "palmíferos": "palmífero",
    "palmípeda": "palmípedo",
    "palmípedas": "palmípedo",
    "palmípedos": "palmípedo",
    "palmítica": "palmítico",
    "palmíticas": "palmítico",
    "palmíticos": "palmítico",
    "palmó": "palmar",
    "palomaduras": "palomadura",
    "palomares": "palomar",
    "palomas": "paloma",
    "palomera": "palomero",
    "palomeras": "palomero",
    "palomeros": "palomero",
    "palomerías": "palomería",
    "palometas": "palometa",
    "palomillas": "palomilla",
    "palomina": "palomino",
    "palominas": "palomino",
    "palominos": "palomino",
    "palomitas": "palomita",
    "palomos": "palomo",
    "palones": "palón",
    "palores": "palor",
    "palos": "palo",
    "palotadas": "palotada",
    "palotea": "palotear",
    "paloteaba": "palotear",
    "paloteabais": "palotear",
    "paloteaban": "palotear",
    "paloteabas": "palotear",
    "palotead": "palotear",
    "paloteada": "palotear",
    "paloteadas": "palotear",
    "paloteado": "palotear",
    "paloteados": "palotear",
    "paloteamos": "palotear",
    "palotean": "palotear",
    "paloteando": "palotear",
    "paloteara": "palotear",
    "palotearais": "palotear",
    "palotearan": "palotear",
    "palotearas": "palotear",
    "paloteare": "palotear",
    "paloteareis": "palotear",
    "palotearemos": "palotear",
    "palotearen": "palotear",
    "paloteares": "palotear",
    "palotearon": "palotear",
    "paloteará": "palotear",
    "palotearán": "palotear",
    "palotearás": "palotear",
    "palotearé": "palotear",
    "palotearéis": "palotear",
    "palotearía": "palotear",
    "palotearíais": "palotear",
    "palotearíamos": "palotear",
    "palotearían": "palotear",
    "palotearías": "palotear",
    "paloteas": "palotear",
    "palotease": "palotear",
    "paloteaseis": "palotear",
    "paloteasen": "palotear",
    "paloteases": "palotear",
    "paloteaste": "palotear",
    "paloteasteis": "palotear",
    "palotee": "palotear",
    "paloteemos": "palotear",
    "paloteen": "palotear",
    "palotees": "palotear",
    "paloteo": "palotear",
    "paloteos": "paloteo",
    "palotes": "palote",
    "paloteábamos": "palotear",
    "paloteáis": "palotear",
    "paloteáramos": "palotear",
    "paloteáremos": "palotear",
    "paloteásemos": "palotear",
    "paloteé": "palotear",
    "paloteéis": "palotear",
    "paloteó": "palotear",
    "palpa": "palpar",
    "palpaba": "palpar",
    "palpabais": "palpar",
    "palpaban": "palpar",
    "palpabas": "palpar",
    "palpables": "palpable",
    "palpaciones": "palpación",
    "palpad": "palpar",
    "palpada": "palpar",
    "palpadas": "palpar",
    "palpado": "palpar",
    "palpadores": "palpador",
    "palpados": "palpar",
    "palpamientos": "palpamiento",
    "palpamos": "palpar",
    "palpan": "palpar",
    "palpando": "palpar",
    "palpara": "palpar",
    "palparais": "palpar",
    "palparan": "palpar",
    "palparas": "palpar",
    "palpare": "palpar",
    "palpareis": "palpar",
    "palparemos": "palpar",
    "palparen": "palpar",
    "palpares": "palpar",
    "palparon": "palpar",
    "palpará": "palpar",
    "palparán": "palpar",
    "palparás": "palpar",
    "palparé": "palpar",
    "palparéis": "palpar",
    "palparía": "palpar",
    "palparíais": "palpar",
    "palparíamos": "palpar",
    "palparían": "palpar",
    "palparías": "palpar",
    "palpas": "palpar",
    "palpase": "palpar",
    "palpaseis": "palpar",
    "palpasen": "palpar",
    "palpases": "palpar",
    "palpaste": "palpar",
    "palpasteis": "palpar",
    "palpe": "palpar",
    "palpebrales": "palpebral",
    "palpebronasales": "palpebronasal",
    "palpemos": "palpar",
    "palpen": "palpar",
    "palpes": "palpar",
    "palpita": "palpitar",
    "palpitaba": "palpitar",
    "palpitabais": "palpitar",
    "palpitaban": "palpitar",
    "palpitabas": "palpitar",
    "palpitaciones": "palpitación",
    "palpitad": "palpitar",
    "palpitada": "palpitar",
    "palpitadas": "palpitar",
    "palpitado": "palpitar",
    "palpitados": "palpitar",
    "palpitamos": "palpitar",
    "palpitan": "palpitar",
    "palpitando": "palpitar",
    "palpitantes": "palpitante",
    "palpitara": "palpitar",
    "palpitarais": "palpitar",
    "palpitaran": "palpitar",
    "palpitaras": "palpitar",
    "palpitare": "palpitar",
    "palpitareis": "palpitar",
    "palpitaremos": "palpitar",
    "palpitaren": "palpitar",
    "palpitares": "palpitar",
    "palpitaron": "palpitar",
    "palpitará": "palpitar",
    "palpitarán": "palpitar",
    "palpitarás": "palpitar",
    "palpitaré": "palpitar",
    "palpitaréis": "palpitar",
    "palpitaría": "palpitar",
    "palpitaríais": "palpitar",
    "palpitaríamos": "palpitar",
    "palpitarían": "palpitar",
    "palpitarías": "palpitar",
    "palpitas": "palpitar",
    "palpitase": "palpitar",
    "palpitaseis": "palpitar",
    "palpitasen": "palpitar",
    "palpitases": "palpitar",
    "palpitaste": "palpitar",
    "palpitasteis": "palpitar",
    "palpite": "palpitar",
    "palpitemos": "palpitar",
    "palpiten": "palpitar",
    "palpites": "palpitar",
    "palpito": "palpitar",
    "palpitábamos": "palpitar",
    "palpitáis": "palpitar",
    "palpitáramos": "palpitar",
    "palpitáremos": "palpitar",
    "palpitásemos": "palpitar",
    "palpité": "palpitar",
    "palpitéis": "palpitar",
    "palpitó": "palpitar",
    "palpo": "palpar",
    "palpos": "palpo",
    "palpábamos": "palpar",
    "palpáis": "palpar",
    "palpáramos": "palpar",
    "palpáremos": "palpar",
    "palpásemos": "palpar",
    "palpé": "palpar",
    "palpéis": "palpar",
    "palpó": "palpar",
    "pals": "pal",
    "paltas": "palta",
    "paltos": "palto",
    "paludamentos": "paludamento",
    "paludismos": "paludismo",
    "paludícolas": "paludícola",
    "palurda": "palurdo",
    "palurdas": "palurdo",
    "palurdos": "palurdo",
    "palustres": "palustre",
    "palygorskitas": "palygorskita",
    "paléstrica": "paléstrico",
    "paléstricas": "paléstrico",
    "paléstricos": "paléstrico",
    "palía": "paliar",
    "palían": "paliar",
    "palías": "paliar",
    "palíe": "paliar",
    "palíen": "paliar",
    "palíes": "paliar",
    "palíndroma": "palíndromo",
    "palíndromas": "palíndromo",
    "palíndromos": "palíndromo",
    "palío": "paliar",
    "palúdica": "palúdico",
    "palúdicas": "palúdico",
    "palúdicos": "palúdico",
    "pameada": "pameado",
    "pameadas": "pameado",
    "pameados": "pameado",
    "pamelas": "pamela",
    "pamemas": "pamema",
    "pampanadas": "pampanada",
    "pampanajes": "pampanaje",
    "pampanillas": "pampanilla",
    "pampanosa": "pampanoso",
    "pampanosas": "pampanoso",
    "pampanosos": "pampanoso",
    "pampas": "pampa",
    "pampea": "pampear",
    "pampeaba": "pampear",
    "pampeabais": "pampear",
    "pampeaban": "pampear",
    "pampeabas": "pampear",
    "pampead": "pampear",
    "pampeada": "pampear",
    "pampeadas": "pampear",
    "pampeado": "pampear",
    "pampeados": "pampear",
    "pampeamos": "pampear",
    "pampean": "pampear",
    "pampeana": "pampeano",
    "pampeanas": "pampeano",
    "pampeando": "pampear",
    "pampeanos": "pampeano",
    "pampeara": "pampear",
    "pampearais": "pampear",
    "pampearan": "pampear",
    "pampearas": "pampear",
    "pampeare": "pampear",
    "pampeareis": "pampear",
    "pampearemos": "pampear",
    "pampearen": "pampear",
    "pampeares": "pampear",
    "pampearon": "pampear",
    "pampeará": "pampear",
    "pampearán": "pampear",
    "pampearás": "pampear",
    "pampearé": "pampear",
    "pampearéis": "pampear",
    "pampearía": "pampear",
    "pampearíais": "pampear",
    "pampearíamos": "pampear",
    "pampearían": "pampear",
    "pampearías": "pampear",
    "pampeas": "pampear",
    "pampease": "pampear",
    "pampeaseis": "pampear",
    "pampeasen": "pampear",
    "pampeases": "pampear",
    "pampeaste": "pampear",
    "pampeasteis": "pampear",
    "pampee": "pampear",
    "pampeemos": "pampear",
    "pampeen": "pampear",
    "pampees": "pampear",
    "pampeo": "pampear",
    "pampera": "pampero",
    "pamperas": "pampero",
    "pamperos": "pampero",
    "pampeábamos": "pampear",
    "pampeáis": "pampear",
    "pampeáramos": "pampear",
    "pampeáremos": "pampear",
    "pampeásemos": "pampear",
    "pampeé": "pampear",
    "pampeéis": "pampear",
    "pampeó": "pampear",
    "pampina": "pampino",
    "pampinas": "pampino",
    "pampiniformes": "pampiniforme",
    "pampinos": "pampino",
    "pampiroladas": "pampirolada",
    "pamplinadas": "pamplinada",
    "pamplinas": "pamplina",
    "pamplinera": "pamplinero",
    "pamplineras": "pamplinero",
    "pamplineros": "pamplinero",
    "pamplinosa": "pamplinoso",
    "pamplinosas": "pamplinoso",
    "pamplinosos": "pamplinoso",
    "pamplonesa": "pamplonés",
    "pamplonesas": "pamplonés",
    "pamploneses": "pamplonés",
    "pamplonicas": "pamplonica",
    "pamporcinos": "pamporcino",
    "pampringadas": "pampringada",
    "pamues": "pamue",
    "pamués": "pamué",
    "panaceas": "panacea",
    "panada": "panado",
    "panadas": "panado",
    "panadera": "panadero",
    "panaderas": "panadero",
    "panaderos": "panadero",
    "panaderías": "panadería",
    "panadizos": "panadizo",
    "panados": "panado",
    "panafricana": "panafricano",
    "panafricanas": "panafricano",
    "panafricanismos": "panafricanismo",
    "panafricanistas": "panafricanista",
    "panafricanos": "panafricano",
    "panaglutinaciones": "panaglutinación",
    "panales": "panal",
    "panamericana": "panamericano",
    "panamericanas": "panamericano",
    "panamericanismos": "panamericanismo",
    "panamericanistas": "panamericanista",
    "panamericanos": "panamericano",
    "panameña": "panameño",
    "panameñas": "panameño",
    "panameñismos": "panameñismo",
    "panameños": "panameño",
    "panarabismos": "panarabismo",
    "panarabistas": "panarabista",
    "panarras": "panarra",
    "panas": "pana",
    "panasiatismos": "panasiatismo",
    "panatelas": "panatela",
    "panatenaica": "panatenaico",
    "panatenaicas": "panatenaico",
    "panatenaicos": "panatenaico",
    "panatenea": "panateneo",
    "panateneas": "panateneo",
    "panateneos": "panateneo",
    "panavisiones": "panavisión",
    "pancadas": "pancada",
    "pancarpias": "pancarpia",
    "pancartas": "pancarta",
    "pancas": "panca",
    "pancatalanismos": "pancatalanismo",
    "pancatalanistas": "pancatalanista",
    "pancellares": "pancellar",
    "panceras": "pancera",
    "pancha": "pancho",
    "panchas": "pancho",
    "panchos": "pancho",
    "pancistas": "pancista",
    "panclastitas": "panclastita",
    "pancraciastas": "pancraciasta",
    "pancracios": "pancracio",
    "pancreatectomías": "pancreatectomía",
    "pancreaticoduodenales": "pancreaticoduodenal",
    "pancreatinas": "pancreatina",
    "pancreática": "pancreático",
    "pancreáticas": "pancreático",
    "pancreáticos": "pancreático",
    "pancromática": "pancromático",
    "pancromáticas": "pancromático",
    "pancromáticos": "pancromático",
    "pancrática": "pancrático",
    "pancráticas": "pancrático",
    "pancráticos": "pancrático",
    "panda": "pando",
    "pandanales": "pandanal",
    "pandas": "pando",
    "pandea": "pandear",
    "pandeaba": "pandear",
    "pandeabais": "pandear",
    "pandeaban": "pandear",
    "pandeabas": "pandear",
    "pandead": "pandear",
    "pandeada": "pandear",
    "pandeadas": "pandear",
    "pandeado": "pandear",
    "pandeados": "pandear",
    "pandeamos": "pandear",
    "pandean": "pandear",
    "pandeando": "pandear",
    "pandeara": "pandear",
    "pandearais": "pandear",
    "pandearan": "pandear",
    "pandearas": "pandear",
    "pandeare": "pandear",
    "pandeareis": "pandear",
    "pandearemos": "pandear",
    "pandearen": "pandear",
    "pandeares": "pandear",
    "pandearon": "pandear",
    "pandeará": "pandear",
    "pandearán": "pandear",
    "pandearás": "pandear",
    "pandearé": "pandear",
    "pandearéis": "pandear",
    "pandearía": "pandear",
    "pandearíais": "pandear",
    "pandearíamos": "pandear",
    "pandearían": "pandear",
    "pandearías": "pandear",
    "pandeas": "pandear",
    "pandease": "pandear",
    "pandeaseis": "pandear",
    "pandeasen": "pandear",
    "pandeases": "pandear",
    "pandeaste": "pandear",
    "pandeasteis": "pandear",
    "pandee": "pandear",
    "pandeemos": "pandear",
    "pandeen": "pandear",
    "pandees": "pandear",
    "pandemias": "pandemia",
    "pandemóniums": "pandemónium",
    "pandeo": "pandear",
    "pandeos": "pandeo",
    "panderadas": "panderada",
    "panderas": "pandera",
    "panderetas": "pandereta",
    "panderetazos": "panderetazo",
    "panderetea": "panderetear",
    "pandereteaba": "panderetear",
    "pandereteabais": "panderetear",
    "pandereteaban": "panderetear",
    "pandereteabas": "panderetear",
    "panderetead": "panderetear",
    "pandereteada": "panderetear",
    "pandereteadas": "panderetear",
    "pandereteado": "panderetear",
    "pandereteados": "panderetear",
    "pandereteamos": "panderetear",
    "panderetean": "panderetear",
    "pandereteando": "panderetear",
    "pandereteara": "panderetear",
    "panderetearais": "panderetear",
    "panderetearan": "panderetear",
    "panderetearas": "panderetear",
    "pandereteare": "panderetear",
    "pandereteareis": "panderetear",
    "panderetearemos": "panderetear",
    "panderetearen": "panderetear",
    "pandereteares": "panderetear",
    "panderetearon": "panderetear",
    "pandereteará": "panderetear",
    "panderetearán": "panderetear",
    "panderetearás": "panderetear",
    "panderetearé": "panderetear",
    "panderetearéis": "panderetear",
    "panderetearía": "panderetear",
    "panderetearíais": "panderetear",
    "panderetearíamos": "panderetear",
    "panderetearían": "panderetear",
    "panderetearías": "panderetear",
    "pandereteas": "panderetear",
    "panderetease": "panderetear",
    "pandereteaseis": "panderetear",
    "pandereteasen": "panderetear",
    "pandereteases": "panderetear",
    "pandereteaste": "panderetear",
    "pandereteasteis": "panderetear",
    "panderetee": "panderetear",
    "pandereteemos": "panderetear",
    "pandereteen": "panderetear",
    "panderetees": "panderetear",
    "pandereteo": "panderetear",
    "pandereteos": "pandereteo",
    "panderetera": "panderetero",
    "pandereteras": "panderetero",
    "pandereteros": "panderetero",
    "panderetes": "panderete",
    "pandereteábamos": "panderetear",
    "pandereteáis": "panderetear",
    "pandereteáramos": "panderetear",
    "pandereteáremos": "panderetear",
    "pandereteásemos": "panderetear",
    "pandereteé": "panderetear",
    "pandereteéis": "panderetear",
    "pandereteó": "panderetear",
    "panderos": "pandero",
    "pandeábamos": "pandear",
    "pandeáis": "pandear",
    "pandeáramos": "pandear",
    "pandeáremos": "pandear",
    "pandeásemos": "pandear",
    "pandeé": "pandear",
    "pandeéis": "pandear",
    "pandeó": "pandear",
    "pandillajes": "pandillaje",
    "pandillas": "pandilla",
    "pandillera": "pandillero",
    "pandilleras": "pandillero",
    "pandilleros": "pandillero",
    "pandillistas": "pandillista",
    "pandits": "pandit",
    "pandoras": "pandora",
    "pandorgas": "pandorga",
    "pandos": "pando",
    "pandémica": "pandémico",
    "pandémicas": "pandémico",
    "pandémicos": "pandémico",
    "panecillos": "panecillo",
    "panegiristas": "panegirista",
    "panegírica": "panegírico",
    "panegíricas": "panegírico",
    "panegíricos": "panegírico",
    "panelas": "panela",
    "paneles": "panel",
    "paneras": "panera",
    "paneros": "panero",
    "panes": "pan",
    "paneslavismos": "paneslavismo",
    "paneslavistas": "paneslavista",
    "panetelas": "panetela",
    "panetera": "panetero",
    "paneteras": "panetero",
    "paneteros": "panetero",
    "paneterías": "panetería",
    "paneuropea": "paneuropeo",
    "paneuropeas": "paneuropeo",
    "paneuropeos": "paneuropeo",
    "paneuropeísmos": "paneuropeísmo",
    "paneuropeístas": "paneuropeísta",
    "panfletaria": "panfletario",
    "panfletarias": "panfletario",
    "panfletarios": "panfletario",
    "panfletistas": "panfletista",
    "panfletos": "panfleto",
    "pangelines": "pangelín",
    "pangermanismos": "pangermanismo",
    "pangermanistas": "pangermanista",
    "pangermánica": "pangermánico",
    "pangermánicas": "pangermánico",
    "pangermánicos": "pangermánico",
    "pangolines": "pangolín",
    "pangues": "pangue",
    "panhelenismos": "panhelenismo",
    "panhelenistas": "panhelenista",
    "panhelénica": "panhelénico",
    "panhelénicas": "panhelénico",
    "panhelénicos": "panhelénico",
    "paniculada": "paniculado",
    "paniculadas": "paniculado",
    "paniculados": "paniculado",
    "paniculares": "panicular",
    "paniega": "paniego",
    "paniegas": "paniego",
    "paniegos": "paniego",
    "panifica": "panificar",
    "panificaba": "panificar",
    "panificabais": "panificar",
    "panificaban": "panificar",
    "panificabas": "panificar",
    "panificables": "panificable",
    "panificaciones": "panificación",
    "panificad": "panificar",
    "panificada": "panificar",
    "panificadas": "panificar",
    "panificado": "panificar",
    "panificadoras": "panificadora",
    "panificados": "panificar",
    "panificamos": "panificar",
    "panifican": "panificar",
    "panificando": "panificar",
    "panificara": "panificar",
    "panificarais": "panificar",
    "panificaran": "panificar",
    "panificaras": "panificar",
    "panificare": "panificar",
    "panificareis": "panificar",
    "panificaremos": "panificar",
    "panificaren": "panificar",
    "panificares": "panificar",
    "panificaron": "panificar",
    "panificará": "panificar",
    "panificarán": "panificar",
    "panificarás": "panificar",
    "panificaré": "panificar",
    "panificaréis": "panificar",
    "panificaría": "panificar",
    "panificaríais": "panificar",
    "panificaríamos": "panificar",
    "panificarían": "panificar",
    "panificarías": "panificar",
    "panificas": "panificar",
    "panificase": "panificar",
    "panificaseis": "panificar",
    "panificasen": "panificar",
    "panificases": "panificar",
    "panificaste": "panificar",
    "panificasteis": "panificar",
    "panifico": "panificar",
    "panificábamos": "panificar",
    "panificáis": "panificar",
    "panificáramos": "panificar",
    "panificáremos": "panificar",
    "panificásemos": "panificar",
    "panificó": "panificar",
    "panifique": "panificar",
    "panifiquemos": "panificar",
    "panifiquen": "panificar",
    "panifiques": "panificar",
    "panifiqué": "panificar",
    "panifiquéis": "panificar",
    "panislamismos": "panislamismo",
    "panislámica": "panislámico",
    "panislámicas": "panislámico",
    "panislámicos": "panislámico",
    "panizos": "panizo",
    "panlogismos": "panlogismo",
    "panmíctica": "panmíctico",
    "panmícticas": "panmíctico",
    "panmícticos": "panmíctico",
    "panocha": "panocho",
    "panochas": "panocho",
    "panochos": "panocho",
    "panojas": "panoja",
    "panoles": "panol",
    "panolis": "panoli",
    "panoplias": "panoplia",
    "panoramas": "panorama",
    "panormitana": "panormitano",
    "panormitanas": "panormitano",
    "panormitanos": "panormitano",
    "panorámica": "panorámico",
    "panorámicas": "panorámico",
    "panorámicos": "panorámico",
    "panspermias": "panspermia",
    "panspérmica": "panspérmico",
    "panspérmicas": "panspérmico",
    "panspérmicos": "panspérmico",
    "pantagruélica": "pantagruélico",
    "pantagruélicas": "pantagruélico",
    "pantagruélicos": "pantagruélico",
    "pantalanes": "pantalán",
    "pantallas": "pantalla",
    "pantalonera": "pantalonero",
    "pantaloneras": "pantalonero",
    "pantaloneros": "pantalonero",
    "pantalones": "pantalón",
    "pantanales": "pantanal",
    "pantanos": "pantano",
    "pantanosa": "pantanoso",
    "pantanosas": "pantanoso",
    "pantanosos": "pantanoso",
    "pantasanas": "pantasana",
    "panteones": "panteón",
    "panteras": "pantera",
    "panteísmos": "panteísmo",
    "panteístas": "panteísta",
    "pantocrátores": "pantocrátor",
    "pantografías": "pantografía",
    "pantográfica": "pantográfico",
    "pantográficas": "pantográfico",
    "pantográficos": "pantográfico",
    "pantomimas": "pantomima",
    "pantomimos": "pantomimo",
    "pantomografías": "pantomografía",
    "pantomogramas": "pantomograma",
    "pantomímica": "pantomímico",
    "pantomímicas": "pantomímico",
    "pantomímicos": "pantomímico",
    "pantoques": "pantoque",
    "pantorras": "pantorra",
    "pantorrillas": "pantorrilla",
    "pantorrilluda": "pantorrilludo",
    "pantorrilludas": "pantorrilludo",
    "pantorrilludos": "pantorrilludo",
    "pantotenatos": "pantotenato",
    "pantoténica": "pantoténico",
    "pantoténicas": "pantoténico",
    "pantoténicos": "pantoténico",
    "pantribales": "pantribal",
    "pantropicales": "pantropical",
    "pantuflas": "pantufla",
    "pantuflazos": "pantuflazo",
    "pantuflos": "pantuflo",
    "pantógrafos": "pantógrafo",
    "pantómetras": "pantómetra",
    "pantómetros": "pantómetro",
    "panuveítises": "panuveítis",
    "panzadas": "panzada",
    "panzas": "panza",
    "panzona": "panzón",
    "panzonas": "panzón",
    "panzones": "panzón",
    "panzuda": "panzudo",
    "panzudas": "panzudo",
    "panzudos": "panzudo",
    "panáticas": "panática",
    "panículas": "panícula",
    "panículos": "panículo",
    "panóptica": "panóptico",
    "panópticas": "panóptico",
    "panópticos": "panóptico",
    "papa": "papar",
    "papaba": "papar",
    "papabais": "papar",
    "papaban": "papar",
    "papabas": "papar",
    "papables": "papable",
    "papaces": "papaz",
    "papad": "papar",
    "papada": "papar",
    "papadas": "papar",
    "papadillas": "papadilla",
    "papado": "papar",
    "papados": "papar",
    "papagayos": "papagayo",
    "papahígos": "papahígo",
    "papales": "papal",
    "papalina": "papalino",
    "papalinas": "papalino",
    "papalinos": "papalino",
    "papamos": "papar",
    "papamóviles": "papamóvil",
    "papan": "papar",
    "papando": "papar",
    "papanduja": "papandujo",
    "papandujas": "papandujo",
    "papandujos": "papandujo",
    "papara": "papar",
    "paparais": "papar",
    "paparan": "papar",
    "paparas": "papar",
    "papare": "papar",
    "papareis": "papar",
    "paparemos": "papar",
    "paparen": "papar",
    "papares": "papar",
    "paparon": "papar",
    "paparotes": "paparote",
    "paparruchas": "paparrucha",
    "papará": "papar",
    "paparán": "papar",
    "paparás": "papar",
    "paparé": "papar",
    "paparéis": "papar",
    "paparía": "papar",
    "paparíais": "papar",
    "paparíamos": "papar",
    "paparían": "papar",
    "paparías": "papar",
    "papas": "papar",
    "papasales": "papasal",
    "papase": "papar",
    "papaseis": "papar",
    "papasen": "papar",
    "papases": "papar",
    "papaste": "papar",
    "papasteis": "papar",
    "papaverinas": "papaverina",
    "papaverácea": "papaveráceo",
    "papaveráceas": "papaveráceo",
    "papaveráceos": "papaveráceo",
    "papayas": "papaya",
    "papayos": "papayo",
    "papaínas": "papaína",
    "pape": "papar",
    "papelea": "papelear",
    "papeleaba": "papelear",
    "papeleabais": "papelear",
    "papeleaban": "papelear",
    "papeleabas": "papelear",
    "papelead": "papelear",
    "papeleada": "papelear",
    "papeleadas": "papelear",
    "papeleado": "papelear",
    "papeleados": "papelear",
    "papeleamos": "papelear",
    "papelean": "papelear",
    "papeleando": "papelear",
    "papeleara": "papelear",
    "papelearais": "papelear",
    "papelearan": "papelear",
    "papelearas": "papelear",
    "papeleare": "papelear",
    "papeleareis": "papelear",
    "papelearemos": "papelear",
    "papelearen": "papelear",
    "papeleares": "papelear",
    "papelearon": "papelear",
    "papeleará": "papelear",
    "papelearán": "papelear",
    "papelearás": "papelear",
    "papelearé": "papelear",
    "papelearéis": "papelear",
    "papelearía": "papelear",
    "papelearíais": "papelear",
    "papelearíamos": "papelear",
    "papelearían": "papelear",
    "papelearías": "papelear",
    "papeleas": "papelear",
    "papelease": "papelear",
    "papeleaseis": "papelear",
    "papeleasen": "papelear",
    "papeleases": "papelear",
    "papeleaste": "papelear",
    "papeleasteis": "papelear",
    "papelee": "papelear",
    "papeleemos": "papelear",
    "papeleen": "papelear",
    "papelees": "papelear",
    "papeleo": "papelear",
    "papeleos": "papeleo",
    "papelera": "papelero",
    "papeleras": "papelero",
    "papeleros": "papelero",
    "papelerías": "papelería",
    "papeles": "papel",
    "papeletas": "papeleta",
    "papeletea": "papeletear",
    "papeleteaba": "papeletear",
    "papeleteabais": "papeletear",
    "papeleteaban": "papeletear",
    "papeleteabas": "papeletear",
    "papeletead": "papeletear",
    "papeleteada": "papeletear",
    "papeleteadas": "papeletear",
    "papeleteado": "papeletear",
    "papeleteados": "papeletear",
    "papeleteamos": "papeletear",
    "papeletean": "papeletear",
    "papeleteando": "papeletear",
    "papeleteara": "papeletear",
    "papeletearais": "papeletear",
    "papeletearan": "papeletear",
    "papeletearas": "papeletear",
    "papeleteare": "papeletear",
    "papeleteareis": "papeletear",
    "papeletearemos": "papeletear",
    "papeletearen": "papeletear",
    "papeleteares": "papeletear",
    "papeletearon": "papeletear",
    "papeleteará": "papeletear",
    "papeletearán": "papeletear",
    "papeletearás": "papeletear",
    "papeletearé": "papeletear",
    "papeletearéis": "papeletear",
    "papeletearía": "papeletear",
    "papeletearíais": "papeletear",
    "papeletearíamos": "papeletear",
    "papeletearían": "papeletear",
    "papeletearías": "papeletear",
    "papeleteas": "papeletear",
    "papeletease": "papeletear",
    "papeleteaseis": "papeletear",
    "papeleteasen": "papeletear",
    "papeleteases": "papeletear",
    "papeleteaste": "papeletear",
    "papeleteasteis": "papeletear",
    "papeletee": "papeletear",
    "papeleteemos": "papeletear",
    "papeleteen": "papeletear",
    "papeletees": "papeletear",
    "papeleteo": "papeletear",
    "papeleteábamos": "papeletear",
    "papeleteáis": "papeletear",
    "papeleteáramos": "papeletear",
    "papeleteáremos": "papeletear",
    "papeleteásemos": "papeletear",
    "papeleteé": "papeletear",
    "papeleteéis": "papeletear",
    "papeleteó": "papeletear",
    "papeletice": "papeletizar",
    "papeleticemos": "papeletizar",
    "papeleticen": "papeletizar",
    "papeletices": "papeletizar",
    "papeleticé": "papeletizar",
    "papeleticéis": "papeletizar",
    "papeletiza": "papeletizar",
    "papeletizaba": "papeletizar",
    "papeletizabais": "papeletizar",
    "papeletizaban": "papeletizar",
    "papeletizabas": "papeletizar",
    "papeletizad": "papeletizar",
    "papeletizada": "papeletizar",
    "papeletizadas": "papeletizar",
    "papeletizado": "papeletizar",
    "papeletizados": "papeletizar",
    "papeletizamos": "papeletizar",
    "papeletizan": "papeletizar",
    "papeletizando": "papeletizar",
    "papeletizara": "papeletizar",
    "papeletizarais": "papeletizar",
    "papeletizaran": "papeletizar",
    "papeletizaras": "papeletizar",
    "papeletizare": "papeletizar",
    "papeletizareis": "papeletizar",
    "papeletizaremos": "papeletizar",
    "papeletizaren": "papeletizar",
    "papeletizares": "papeletizar",
    "papeletizaron": "papeletizar",
    "papeletizará": "papeletizar",
    "papeletizarán": "papeletizar",
    "papeletizarás": "papeletizar",
    "papeletizaré": "papeletizar",
    "papeletizaréis": "papeletizar",
    "papeletizaría": "papeletizar",
    "papeletizaríais": "papeletizar",
    "papeletizaríamos": "papeletizar",
    "papeletizarían": "papeletizar",
    "papeletizarías": "papeletizar",
    "papeletizas": "papeletizar",
    "papeletizase": "papeletizar",
    "papeletizaseis": "papeletizar",
    "papeletizasen": "papeletizar",
    "papeletizases": "papeletizar",
    "papeletizaste": "papeletizar",
    "papeletizasteis": "papeletizar",
    "papeletizo": "papeletizar",
    "papeletizábamos": "papeletizar",
    "papeletizáis": "papeletizar",
    "papeletizáramos": "papeletizar",
    "papeletizáremos": "papeletizar",
    "papeletizásemos": "papeletizar",
    "papeletizó": "papeletizar",
    "papeleábamos": "papelear",
    "papeleáis": "papelear",
    "papeleáramos": "papelear",
    "papeleáremos": "papelear",
    "papeleásemos": "papelear",
    "papeleé": "papelear",
    "papeleéis": "papelear",
    "papeleó": "papelear",
    "papelillos": "papelillo",
    "papelinas": "papelina",
    "papelistas": "papelista",
    "papelona": "papelón",
    "papelonada": "papelonado",
    "papelonadas": "papelonado",
    "papelonados": "papelonado",
    "papelonas": "papelón",
    "papelonea": "papelonear",
    "papeloneaba": "papelonear",
    "papeloneabais": "papelonear",
    "papeloneaban": "papelonear",
    "papeloneabas": "papelonear",
    "papelonead": "papelonear",
    "papeloneada": "papelonear",
    "papeloneadas": "papelonear",
    "papeloneado": "papelonear",
    "papeloneados": "papelonear",
    "papeloneamos": "papelonear",
    "papelonean": "papelonear",
    "papeloneando": "papelonear",
    "papeloneara": "papelonear",
    "papelonearais": "papelonear",
    "papelonearan": "papelonear",
    "papelonearas": "papelonear",
    "papeloneare": "papelonear",
    "papeloneareis": "papelonear",
    "papelonearemos": "papelonear",
    "papelonearen": "papelonear",
    "papeloneares": "papelonear",
    "papelonearon": "papelonear",
    "papeloneará": "papelonear",
    "papelonearán": "papelonear",
    "papelonearás": "papelonear",
    "papelonearé": "papelonear",
    "papelonearéis": "papelonear",
    "papelonearía": "papelonear",
    "papelonearíais": "papelonear",
    "papelonearíamos": "papelonear",
    "papelonearían": "papelonear",
    "papelonearías": "papelonear",
    "papeloneas": "papelonear",
    "papelonease": "papelonear",
    "papeloneaseis": "papelonear",
    "papeloneasen": "papelonear",
    "papeloneases": "papelonear",
    "papeloneaste": "papelonear",
    "papeloneasteis": "papelonear",
    "papelonee": "papelonear",
    "papeloneemos": "papelonear",
    "papeloneen": "papelonear",
    "papelonees": "papelonear",
    "papeloneo": "papelonear",
    "papelones": "papelón",
    "papeloneábamos": "papelonear",
    "papeloneáis": "papelonear",
    "papeloneáramos": "papelonear",
    "papeloneáremos": "papelonear",
    "papeloneásemos": "papelonear",
    "papeloneé": "papelonear",
    "papeloneéis": "papelonear",
    "papeloneó": "papelonear",
    "papelorios": "papelorio",
    "papelotes": "papelote",
    "papeluchos": "papelucho",
    "papemos": "papar",
    "papen": "papar",
    "papes": "papar",
    "papiamentos": "papiamento",
    "papilares": "papilar",
    "papilas": "papila",
    "papiledemas": "papiledema",
    "papilionada": "papilionado",
    "papilionadas": "papilionado",
    "papilionados": "papilionado",
    "papilionáceas": "papilionácea",
    "papilionáceos": "papilionáceo",
    "papiliónidos": "papiliónido",
    "papillas": "papilla",
    "papillotas": "papillota",
    "papillotes": "papillote",
    "papilomas": "papiloma",
    "papiones": "papión",
    "papiroladas": "papirolada",
    "papirologías": "papirología",
    "papirológica": "papirológico",
    "papirológicas": "papirológico",
    "papirológicos": "papirológico",
    "papiros": "papiro",
    "papirotadas": "papirotada",
    "papirotazos": "papirotazo",
    "papirotes": "papirote",
    "papirácea": "papiráceo",
    "papiráceas": "papiráceo",
    "papiráceos": "papiráceo",
    "papiróloga": "papirólogo",
    "papirólogas": "papirólogo",
    "papirólogos": "papirólogo",
    "papis": "papi",
    "papisas": "papisa",
    "papismos": "papismo",
    "papistas": "papista",
    "papo": "papar",
    "papones": "papón",
    "papos": "papo",
    "paprikas": "paprika",
    "papuda": "papudo",
    "papudas": "papudo",
    "papudos": "papudo",
    "papujada": "papujado",
    "papujadas": "papujado",
    "papujados": "papujado",
    "papulosa": "papuloso",
    "papulosas": "papuloso",
    "papulosos": "papuloso",
    "papábamos": "papar",
    "papáis": "papar",
    "papáramos": "papar",
    "papáremos": "papar",
    "papás": "papá",
    "papásemos": "papar",
    "papé": "papar",
    "papéis": "papar",
    "papó": "papar",
    "papúas": "papúa",
    "papúes": "papú",
    "papús": "papú",
    "papúses": "papú",
    "paquea": "paquear",
    "paqueaba": "paquear",
    "paqueabais": "paquear",
    "paqueaban": "paquear",
    "paqueabas": "paquear",
    "paquead": "paquear",
    "paqueada": "paquear",
    "paqueadas": "paquear",
    "paqueado": "paquear",
    "paqueados": "paquear",
    "paqueamos": "paquear",
    "paquean": "paquear",
    "paqueando": "paquear",
    "paqueara": "paquear",
    "paquearais": "paquear",
    "paquearan": "paquear",
    "paquearas": "paquear",
    "paqueare": "paquear",
    "paqueareis": "paquear",
    "paquearemos": "paquear",
    "paquearen": "paquear",
    "paqueares": "paquear",
    "paquearon": "paquear",
    "paqueará": "paquear",
    "paquearán": "paquear",
    "paquearás": "paquear",
    "paquearé": "paquear",
    "paquearéis": "paquear",
    "paquearía": "paquear",
    "paquearíais": "paquear",
    "paquearíamos": "paquear",
    "paquearían": "paquear",
    "paquearías": "paquear",
    "paqueas": "paquear",
    "paquease": "paquear",
    "paqueaseis": "paquear",
    "paqueasen": "paquear",
    "paqueases": "paquear",
    "paqueaste": "paquear",
    "paqueasteis": "paquear",
    "paquebotes": "paquebote",
    "paquebots": "paquebot",
    "paquee": "paquear",
    "paqueemos": "paquear",
    "paqueen": "paquear",
    "paquees": "paquear",
    "paqueo": "paquear",
    "paqueos": "paqueo",
    "paquetera": "paquetero",
    "paqueteras": "paquetero",
    "paqueteros": "paquetero",
    "paqueterías": "paquetería",
    "paquetes": "paquete",
    "paqueábamos": "paquear",
    "paqueáis": "paquear",
    "paqueáramos": "paquear",
    "paqueáremos": "paquear",
    "paqueásemos": "paquear",
    "paqueé": "paquear",
    "paqueéis": "paquear",
    "paqueó": "paquear",
    "paquiderma": "paquidermo",
    "paquidermas": "paquidermo",
    "paquidermias": "paquidermia",
    "paquidermos": "paquidermo",
    "paquimeninges": "paquimeninge",
    "paquioniquias": "paquioniquia",
    "paquistanesa": "paquistanés",
    "paquistanesas": "paquistanés",
    "paquistaneses": "paquistanés",
    "paquistanises": "paquistaní",
    "paquistaníes": "paquistaní",
    "paquistanís": "paquistaní",
    "paquistaníses": "paquistaní",
    "paquitenas": "paquitena",
    "para": "parir",
    "paraapicales": "paraapical",
    "paraavalanchas": "paraavalancha",
    "paraaórtica": "paraaórtico",
    "paraaórticas": "paraaórtico",
    "paraaórticos": "paraaórtico",
    "paraba": "parar",
    "parabais": "parar",
    "paraban": "parar",
    "parabas": "parar",
    "parabienes": "parabién",
    "parabolanos": "parabolano",
    "parabolice": "parabolizar",
    "parabolicemos": "parabolizar",
    "parabolicen": "parabolizar",
    "parabolices": "parabolizar",
    "parabolicidades": "parabolicidad",
    "parabolicé": "parabolizar",
    "parabolicéis": "parabolizar",
    "paraboliza": "parabolizar",
    "parabolizaba": "parabolizar",
    "parabolizabais": "parabolizar",
    "parabolizaban": "parabolizar",
    "parabolizabas": "parabolizar",
    "parabolizad": "parabolizar",
    "parabolizada": "parabolizar",
    "parabolizadas": "parabolizar",
    "parabolizado": "parabolizar",
    "parabolizados": "parabolizar",
    "parabolizamos": "parabolizar",
    "parabolizan": "parabolizar",
    "parabolizando": "parabolizar",
    "parabolizara": "parabolizar",
    "parabolizarais": "parabolizar",
    "parabolizaran": "parabolizar",
    "parabolizaras": "parabolizar",
    "parabolizare": "parabolizar",
    "parabolizareis": "parabolizar",
    "parabolizaremos": "parabolizar",
    "parabolizaren": "parabolizar",
    "parabolizares": "parabolizar",
    "parabolizaron": "parabolizar",
    "parabolizará": "parabolizar",
    "parabolizarán": "parabolizar",
    "parabolizarás": "parabolizar",
    "parabolizaré": "parabolizar",
    "parabolizaréis": "parabolizar",
    "parabolizaría": "parabolizar",
    "parabolizaríais": "parabolizar",
    "parabolizaríamos": "parabolizar",
    "parabolizarían": "parabolizar",
    "parabolizarías": "parabolizar",
    "parabolizas": "parabolizar",
    "parabolizase": "parabolizar",
    "parabolizaseis": "parabolizar",
    "parabolizasen": "parabolizar",
    "parabolizases": "parabolizar",
    "parabolizaste": "parabolizar",
    "parabolizasteis": "parabolizar",
    "parabolizo": "parabolizar",
    "parabolizábamos": "parabolizar",
    "parabolizáis": "parabolizar",
    "parabolizáramos": "parabolizar",
    "parabolizáremos": "parabolizar",
    "parabolizásemos": "parabolizar",
    "parabolizó": "parabolizar",
    "paraboloides": "paraboloide",
    "parabólica": "parabólico",
    "parabólicas": "parabólico",
    "parabólicos": "parabólico",
    "paracaidismos": "paracaidismo",
    "paracaidistas": "paracaidista",
    "paracas": "paraca",
    "paracentrales": "paracentral",
    "paracetamoles": "paracetamol",
    "paraciencias": "paraciencia",
    "paracletos": "paracleto",
    "paracompacta": "paracompacto",
    "paracompactas": "paracompacto",
    "paracompactos": "paracompacto",
    "paraconglomerados": "paraconglomerado",
    "paracortezas": "paracorteza",
    "paracorticales": "paracortical",
    "paracromatinas": "paracromatina",
    "paracromias": "paracromia",
    "paracronismos": "paracronismo",
    "paracusias": "paracusia",
    "paracólica": "paracólico",
    "paracólicas": "paracólico",
    "paracólicos": "paracólico",
    "parad": "parar",
    "parada": "parar",
    "paradas": "parar",
    "paraderas": "paradera",
    "paraderos": "paradero",
    "paradiafonías": "paradiafonía",
    "paradigmas": "paradigma",
    "paradigmática": "paradigmático",
    "paradigmáticas": "paradigmático",
    "paradigmáticos": "paradigmático",
    "paradinas": "paradina",
    "paradisiaca": "paradisiaco",
    "paradisiacas": "paradisiaco",
    "paradisiacos": "paradisiaco",
    "paradisíaca": "paradisíaco",
    "paradisíacas": "paradisíaco",
    "paradisíacos": "paradisíaco",
    "paradiástoles": "paradiástole",
    "parado": "parar",
    "paradojas": "paradoja",
    "paradora": "parador",
    "paradoras": "parador",
    "paradores": "parador",
    "parados": "parar",
    "paraduodenales": "paraduodenal",
    "paradídimos": "paradídimo",
    "paradógica": "paradógico",
    "paradógicas": "paradógico",
    "paradógicos": "paradógico",
    "paradójica": "paradójico",
    "paradójicas": "paradójico",
    "paradójicos": "paradójico",
    "paraestatales": "paraestatal",
    "parafarmacias": "parafarmacia",
    "parafascistas": "parafascista",
    "parafasias": "parafasia",
    "parafernales": "parafernal",
    "parafernalias": "parafernalia",
    "parafina": "parafinar",
    "parafinaba": "parafinar",
    "parafinabais": "parafinar",
    "parafinaban": "parafinar",
    "parafinabas": "parafinar",
    "parafinad": "parafinar",
    "parafinada": "parafinar",
    "parafinadas": "parafinar",
    "parafinado": "parafinar",
    "parafinados": "parafinar",
    "parafinamos": "parafinar",
    "parafinan": "parafinar",
    "parafinando": "parafinar",
    "parafinara": "parafinar",
    "parafinarais": "parafinar",
    "parafinaran": "parafinar",
    "parafinaras": "parafinar",
    "parafinare": "parafinar",
    "parafinareis": "parafinar",
    "parafinaremos": "parafinar",
    "parafinaren": "parafinar",
    "parafinares": "parafinar",
    "parafinaron": "parafinar",
    "parafinará": "parafinar",
    "parafinarán": "parafinar",
    "parafinarás": "parafinar",
    "parafinaré": "parafinar",
    "parafinaréis": "parafinar",
    "parafinaría": "parafinar",
    "parafinaríais": "parafinar",
    "parafinaríamos": "parafinar",
    "parafinarían": "parafinar",
    "parafinarías": "parafinar",
    "parafinas": "parafinar",
    "parafinase": "parafinar",
    "parafinaseis": "parafinar",
    "parafinasen": "parafinar",
    "parafinases": "parafinar",
    "parafinaste": "parafinar",
    "parafinasteis": "parafinar",
    "parafine": "parafinar",
    "parafinemos": "parafinar",
    "parafinen": "parafinar",
    "parafines": "parafinar",
    "parafino": "parafinar",
    "parafinábamos": "parafinar",
    "parafináis": "parafinar",
    "parafináramos": "parafinar",
    "parafináremos": "parafinar",
    "parafinásemos": "parafinar",
    "parafiné": "parafinar",
    "parafinéis": "parafinar",
    "parafinó": "parafinar",
    "parafiscalidades": "parafiscalidad",
    "parafonías": "parafonía",
    "paraformaldehídos": "paraformaldehído",
    "parafrasea": "parafrasear",
    "parafraseaba": "parafrasear",
    "parafraseabais": "parafrasear",
    "parafraseaban": "parafrasear",
    "parafraseabas": "parafrasear",
    "parafrasead": "parafrasear",
    "parafraseada": "parafrasear",
    "parafraseadas": "parafrasear",
    "parafraseado": "parafrasear",
    "parafraseadora": "parafraseador",
    "parafraseadoras": "parafraseador",
    "parafraseadores": "parafraseador",
    "parafraseados": "parafrasear",
    "parafraseamos": "parafrasear",
    "parafrasean": "parafrasear",
    "parafraseando": "parafrasear",
    "parafraseara": "parafrasear",
    "parafrasearais": "parafrasear",
    "parafrasearan": "parafrasear",
    "parafrasearas": "parafrasear",
    "parafraseare": "parafrasear",
    "parafraseareis": "parafrasear",
    "parafrasearemos": "parafrasear",
    "parafrasearen": "parafrasear",
    "parafraseares": "parafrasear",
    "parafrasearon": "parafrasear",
    "parafraseará": "parafrasear",
    "parafrasearán": "parafrasear",
    "parafrasearás": "parafrasear",
    "parafrasearé": "parafrasear",
    "parafrasearéis": "parafrasear",
    "parafrasearía": "parafrasear",
    "parafrasearíais": "parafrasear",
    "parafrasearíamos": "parafrasear",
    "parafrasearían": "parafrasear",
    "parafrasearías": "parafrasear",
    "parafraseas": "parafrasear",
    "parafrasease": "parafrasear",
    "parafraseaseis": "parafrasear",
    "parafraseasen": "parafrasear",
    "parafraseases": "parafrasear",
    "parafraseaste": "parafrasear",
    "parafraseasteis": "parafrasear",
    "parafrasee": "parafrasear",
    "parafraseemos": "parafrasear",
    "parafraseen": "parafrasear",
    "parafrasees": "parafrasear",
    "parafraseo": "parafrasear",
    "parafraseábamos": "parafrasear",
    "parafraseáis": "parafrasear",
    "parafraseáramos": "parafrasear",
    "parafraseáremos": "parafrasear",
    "parafraseásemos": "parafrasear",
    "parafraseé": "parafrasear",
    "parafraseéis": "parafrasear",
    "parafraseó": "parafrasear",
    "parafrastes": "parafraste",
    "parafrástica": "parafrástico",
    "parafrásticas": "parafrástico",
    "parafrásticos": "parafrástico",
    "parafunciones": "parafunción",
    "parafínica": "parafínico",
    "parafínicas": "parafínico",
    "parafínicos": "parafínico",
    "paraglobinas": "paraglobina",
    "paraglobulinas": "paraglobulina",
    "paraglótica": "paraglótico",
    "paraglóticas": "paraglótico",
    "paraglóticos": "paraglótico",
    "paragoges": "paragoge",
    "paragolpes": "parachoques",
    "paragranizos": "paragranizo",
    "paraguaya": "paraguayo",
    "paraguayana": "paraguayano",
    "paraguayanas": "paraguayano",
    "paraguayanos": "paraguayano",
    "paraguayas": "paraguayo",
    "paraguayismos": "paraguayismo",
    "paraguayos": "paraguayo",
    "paragógica": "paragógico",
    "paragógicas": "paragógico",
    "paragógicos": "paragógico",
    "paragüera": "paragüero",
    "paragüeras": "paragüero",
    "paragüeros": "paragüero",
    "paragüerías": "paragüería",
    "parahelios": "parahelio",
    "parahidrógenos": "parahidrógeno",
    "parahusaba": "parahusar",
    "parahusabais": "parahusar",
    "parahusaban": "parahusar",
    "parahusabas": "parahusar",
    "parahusad": "parahusar",
    "parahusada": "parahusar",
    "parahusadas": "parahusar",
    "parahusado": "parahusar",
    "parahusados": "parahusar",
    "parahusamos": "parahusar",
    "parahusando": "parahusar",
    "parahusara": "parahusar",
    "parahusarais": "parahusar",
    "parahusaran": "parahusar",
    "parahusaras": "parahusar",
    "parahusare": "parahusar",
    "parahusareis": "parahusar",
    "parahusaremos": "parahusar",
    "parahusaren": "parahusar",
    "parahusares": "parahusar",
    "parahusaron": "parahusar",
    "parahusará": "parahusar",
    "parahusarán": "parahusar",
    "parahusarás": "parahusar",
    "parahusaré": "parahusar",
    "parahusaréis": "parahusar",
    "parahusaría": "parahusar",
    "parahusaríais": "parahusar",
    "parahusaríamos": "parahusar",
    "parahusarían": "parahusar",
    "parahusarías": "parahusar",
    "parahusase": "parahusar",
    "parahusaseis": "parahusar",
    "parahusasen": "parahusar",
    "parahusases": "parahusar",
    "parahusaste": "parahusar",
    "parahusasteis": "parahusar",
    "parahusemos": "parahusar",
    "parahusábamos": "parahusar",
    "parahusáis": "parahusar",
    "parahusáramos": "parahusar",
    "parahusáremos": "parahusar",
    "parahusásemos": "parahusar",
    "parahusé": "parahusar",
    "parahuséis": "parahusar",
    "parahusó": "parahusar",
    "parahúsa": "parahusar",
    "parahúsan": "parahusar",
    "parahúsas": "parahusar",
    "parahúse": "parahusar",
    "parahúsen": "parahusar",
    "parahúses": "parahusar",
    "parahúso": "parahusar",
    "parahúsos": "parahúso",
    "parajes": "paraje",
    "paralabios": "paralabio",
    "paralajes": "paralaje",
    "paraldehídos": "paraldehído",
    "paralegales": "paralegal",
    "paralela": "paralelo",
    "paralelas": "paralelo",
    "paralelepípedos": "paralelepípedo",
    "paralelismos": "paralelismo",
    "paralelizaciones": "paralelización",
    "paralelizadores": "paralelizador",
    "paralelogramas": "paralelograma",
    "paralelogramos": "paralelogramo",
    "paraleloguías": "paraleloguía",
    "paralelos": "paralelo",
    "paralelotopos": "paralelotopo",
    "paralelógrafos": "paralelógrafo",
    "paralelómetros": "paralelómetro",
    "parales": "paral",
    "paralexias": "paralexia",
    "paralice": "paralizar",
    "paralicemos": "paralizar",
    "paralicen": "paralizar",
    "paralices": "paralizar",
    "paralicé": "paralizar",
    "paralicéis": "paralizar",
    "paralingüística": "paralingüístico",
    "paralingüísticas": "paralingüístico",
    "paralingüísticos": "paralingüístico",
    "paraliza": "paralizar",
    "paralizaba": "paralizar",
    "paralizabais": "paralizar",
    "paralizaban": "paralizar",
    "paralizabas": "paralizar",
    "paralizaciones": "paralización",
    "paralizad": "paralizar",
    "paralizada": "paralizar",
    "paralizadas": "paralizar",
    "paralizado": "paralizar",
    "paralizadora": "paralizador",
    "paralizadoras": "paralizador",
    "paralizadores": "paralizador",
    "paralizados": "paralizar",
    "paralizamos": "paralizar",
    "paralizan": "paralizar",
    "paralizando": "paralizar",
    "paralizantes": "paralizante",
    "paralizara": "paralizar",
    "paralizarais": "paralizar",
    "paralizaran": "paralizar",
    "paralizaras": "paralizar",
    "paralizare": "paralizar",
    "paralizareis": "paralizar",
    "paralizaremos": "paralizar",
    "paralizaren": "paralizar",
    "paralizares": "paralizar",
    "paralizaron": "paralizar",
    "paralizará": "paralizar",
    "paralizarán": "paralizar",
    "paralizarás": "paralizar",
    "paralizaré": "paralizar",
    "paralizaréis": "paralizar",
    "paralizaría": "paralizar",
    "paralizaríais": "paralizar",
    "paralizaríamos": "paralizar",
    "paralizarían": "paralizar",
    "paralizarías": "paralizar",
    "paralizas": "paralizar",
    "paralizase": "paralizar",
    "paralizaseis": "paralizar",
    "paralizasen": "paralizar",
    "paralizases": "paralizar",
    "paralizaste": "paralizar",
    "paralizasteis": "paralizar",
    "paralizo": "paralizar",
    "paralizábamos": "paralizar",
    "paralizáis": "paralizar",
    "paralizáramos": "paralizar",
    "paralizáremos": "paralizar",
    "paralizásemos": "paralizar",
    "paralizó": "paralizar",
    "paralogice": "paralogizar",
    "paralogicemos": "paralogizar",
    "paralogicen": "paralogizar",
    "paralogices": "paralogizar",
    "paralogicé": "paralogizar",
    "paralogicéis": "paralogizar",
    "paralogismos": "paralogismo",
    "paralogiza": "paralogizar",
    "paralogizaba": "paralogizar",
    "paralogizabais": "paralogizar",
    "paralogizaban": "paralogizar",
    "paralogizabas": "paralogizar",
    "paralogizad": "paralogizar",
    "paralogizada": "paralogizar",
    "paralogizadas": "paralogizar",
    "paralogizado": "paralogizar",
    "paralogizados": "paralogizar",
    "paralogizamos": "paralogizar",
    "paralogizan": "paralogizar",
    "paralogizando": "paralogizar",
    "paralogizara": "paralogizar",
    "paralogizarais": "paralogizar",
    "paralogizaran": "paralogizar",
    "paralogizaras": "paralogizar",
    "paralogizare": "paralogizar",
    "paralogizareis": "paralogizar",
    "paralogizaremos": "paralogizar",
    "paralogizaren": "paralogizar",
    "paralogizares": "paralogizar",
    "paralogizaron": "paralogizar",
    "paralogizará": "paralogizar",
    "paralogizarán": "paralogizar",
    "paralogizarás": "paralogizar",
    "paralogizaré": "paralogizar",
    "paralogizaréis": "paralogizar",
    "paralogizaría": "paralogizar",
    "paralogizaríais": "paralogizar",
    "paralogizaríamos": "paralogizar",
    "paralogizarían": "paralogizar",
    "paralogizarías": "paralogizar",
    "paralogizas": "paralogizar",
    "paralogizase": "paralogizar",
    "paralogizaseis": "paralogizar",
    "paralogizasen": "paralogizar",
    "paralogizases": "paralogizar",
    "paralogizaste": "paralogizar",
    "paralogizasteis": "paralogizar",
    "paralogizo": "paralogizar",
    "paralogizábamos": "paralogizar",
    "paralogizáis": "paralogizar",
    "paralogizáramos": "paralogizar",
    "paralogizáremos": "paralogizar",
    "paralogizásemos": "paralogizar",
    "paralogizó": "paralogizar",
    "paraláctica": "paraláctico",
    "paralácticas": "paraláctico",
    "paralácticos": "paraláctico",
    "paralímpica": "paralímpico",
    "paralímpicas": "paralímpico",
    "paralímpicos": "paralímpico",
    "paralítica": "paralítico",
    "paralíticas": "paralítico",
    "paralíticos": "paralítico",
    "paramagnetismos": "paramagnetismo",
    "paramagnética": "paramagnético",
    "paramagnéticas": "paramagnético",
    "paramagnéticos": "paramagnético",
    "paramecios": "paramecio",
    "paramediana": "paramediano",
    "paramedianas": "paramediano",
    "paramedianos": "paramediano",
    "paramenta": "paramentar",
    "paramentaba": "paramentar",
    "paramentabais": "paramentar",
    "paramentaban": "paramentar",
    "paramentabas": "paramentar",
    "paramentad": "paramentar",
    "paramentada": "paramentar",
    "paramentadas": "paramentar",
    "paramentado": "paramentar",
    "paramentados": "paramentar",
    "paramentamos": "paramentar",
    "paramentan": "paramentar",
    "paramentando": "paramentar",
    "paramentara": "paramentar",
    "paramentarais": "paramentar",
    "paramentaran": "paramentar",
    "paramentaras": "paramentar",
    "paramentare": "paramentar",
    "paramentareis": "paramentar",
    "paramentaremos": "paramentar",
    "paramentaren": "paramentar",
    "paramentares": "paramentar",
    "paramentaron": "paramentar",
    "paramentará": "paramentar",
    "paramentarán": "paramentar",
    "paramentarás": "paramentar",
    "paramentaré": "paramentar",
    "paramentaréis": "paramentar",
    "paramentaría": "paramentar",
    "paramentaríais": "paramentar",
    "paramentaríamos": "paramentar",
    "paramentarían": "paramentar",
    "paramentarías": "paramentar",
    "paramentas": "paramentar",
    "paramentase": "paramentar",
    "paramentaseis": "paramentar",
    "paramentasen": "paramentar",
    "paramentases": "paramentar",
    "paramentaste": "paramentar",
    "paramentasteis": "paramentar",
    "paramente": "paramentar",
    "paramentemos": "paramentar",
    "paramenten": "paramentar",
    "paramentes": "paramentar",
    "paramento": "paramentar",
    "paramentos": "paramento",
    "paramentábamos": "paramentar",
    "paramentáis": "paramentar",
    "paramentáramos": "paramentar",
    "paramentáremos": "paramentar",
    "paramentásemos": "paramentar",
    "paramenté": "paramentar",
    "paramentéis": "paramentar",
    "paramentó": "paramentar",
    "parameras": "paramera",
    "paramesonéfrica": "paramesonéfrico",
    "paramesonéfricas": "paramesonéfrico",
    "paramesonéfricos": "paramesonéfrico",
    "parametadionas": "parametadiona",
    "parametrice": "parametrizar",
    "parametricemos": "parametrizar",
    "parametricen": "parametrizar",
    "parametrices": "parametrizar",
    "parametricé": "parametrizar",
    "parametricéis": "parametrizar",
    "parametrios": "parametrio",
    "parametriza": "parametrizar",
    "parametrizaba": "parametrizar",
    "parametrizabais": "parametrizar",
    "parametrizaban": "parametrizar",
    "parametrizabas": "parametrizar",
    "parametrizables": "parametrizable",
    "parametrizaciones": "parametrización",
    "parametrizad": "parametrizar",
    "parametrizada": "parametrizar",
    "parametrizadas": "parametrizar",
    "parametrizado": "parametrizar",
    "parametrizados": "parametrizar",
    "parametrizamos": "parametrizar",
    "parametrizan": "parametrizar",
    "parametrizando": "parametrizar",
    "parametrizara": "parametrizar",
    "parametrizarais": "parametrizar",
    "parametrizaran": "parametrizar",
    "parametrizaras": "parametrizar",
    "parametrizare": "parametrizar",
    "parametrizareis": "parametrizar",
    "parametrizaremos": "parametrizar",
    "parametrizaren": "parametrizar",
    "parametrizares": "parametrizar",
    "parametrizaron": "parametrizar",
    "parametrizará": "parametrizar",
    "parametrizarán": "parametrizar",
    "parametrizarás": "parametrizar",
    "parametrizaré": "parametrizar",
    "parametrizaréis": "parametrizar",
    "parametrizaría": "parametrizar",
    "parametrizaríais": "parametrizar",
    "parametrizaríamos": "parametrizar",
    "parametrizarían": "parametrizar",
    "parametrizarías": "parametrizar",
    "parametrizas": "parametrizar",
    "parametrizase": "parametrizar",
    "parametrizaseis": "parametrizar",
    "parametrizasen": "parametrizar",
    "parametrizases": "parametrizar",
    "parametrizaste": "parametrizar",
    "parametrizasteis": "parametrizar",
    "parametrizo": "parametrizar",
    "parametrizábamos": "parametrizar",
    "parametrizáis": "parametrizar",
    "parametrizáramos": "parametrizar",
    "parametrizáremos": "parametrizar",
    "parametrizásemos": "parametrizar",
    "parametrizó": "parametrizar",
    "paramilitares": "paramilitar",
    "paramnesias": "paramnesia",
    "paramos": "parir",
    "paramédica": "paramédico",
    "paramédicas": "paramédico",
    "paramédicos": "paramédico",
    "paramétrica": "paramétrico",
    "paramétricas": "paramétrico",
    "paramétricos": "paramétrico",
    "paramíctica": "paramíctico",
    "paramícticas": "paramíctico",
    "paramícticos": "paramíctico",
    "paran": "parir",
    "paranasales": "paranasal",
    "parancera": "parancero",
    "paranceras": "parancero",
    "paranceros": "parancero",
    "parando": "parar",
    "parangona": "parangonar",
    "parangonaba": "parangonar",
    "parangonabais": "parangonar",
    "parangonaban": "parangonar",
    "parangonabas": "parangonar",
    "parangonaciones": "parangonación",
    "parangonad": "parangonar",
    "parangonada": "parangonar",
    "parangonadas": "parangonar",
    "parangonado": "parangonar",
    "parangonados": "parangonar",
    "parangonamos": "parangonar",
    "parangonan": "parangonar",
    "parangonando": "parangonar",
    "parangonara": "parangonar",
    "parangonarais": "parangonar",
    "parangonaran": "parangonar",
    "parangonaras": "parangonar",
    "parangonare": "parangonar",
    "parangonareis": "parangonar",
    "parangonaremos": "parangonar",
    "parangonaren": "parangonar",
    "parangonares": "parangonar",
    "parangonaron": "parangonar",
    "parangonará": "parangonar",
    "parangonarán": "parangonar",
    "parangonarás": "parangonar",
    "parangonaré": "parangonar",
    "parangonaréis": "parangonar",
    "parangonaría": "parangonar",
    "parangonaríais": "parangonar",
    "parangonaríamos": "parangonar",
    "parangonarían": "parangonar",
    "parangonarías": "parangonar",
    "parangonas": "parangonar",
    "parangonase": "parangonar",
    "parangonaseis": "parangonar",
    "parangonasen": "parangonar",
    "parangonases": "parangonar",
    "parangonaste": "parangonar",
    "parangonasteis": "parangonar",
    "parangone": "parangonar",
    "parangonemos": "parangonar",
    "parangonen": "parangonar",
    "parangones": "parangonar",
    "parangono": "parangonar",
    "parangonábamos": "parangonar",
    "parangonáis": "parangonar",
    "parangonáramos": "parangonar",
    "parangonáremos": "parangonar",
    "parangonásemos": "parangonar",
    "parangoné": "parangonar",
    "parangonéis": "parangonar",
    "parangonó": "parangonar",
    "paraninfos": "paraninfo",
    "paranoias": "paranoia",
    "paranoica": "paranoico",
    "paranoicas": "paranoico",
    "paranoicos": "paranoico",
    "paranormales": "paranormal",
    "paranucleínas": "paranucleína",
    "paranzas": "paranza",
    "paraovarios": "paraovario",
    "parapentes": "parapente",
    "paraperióstica": "paraperióstico",
    "paraperiósticas": "paraperióstico",
    "paraperiósticos": "paraperióstico",
    "parapeta": "parapetar",
    "parapetaba": "parapetar",
    "parapetabais": "parapetar",
    "parapetaban": "parapetar",
    "parapetabas": "parapetar",
    "parapetad": "parapetar",
    "parapetada": "parapetar",
    "parapetadas": "parapetar",
    "parapetado": "parapetar",
    "parapetados": "parapetar",
    "parapetamos": "parapetar",
    "parapetan": "parapetar",
    "parapetando": "parapetar",
    "parapetara": "parapetar",
    "parapetarais": "parapetar",
    "parapetaran": "parapetar",
    "parapetaras": "parapetar",
    "parapetare": "parapetar",
    "parapetareis": "parapetar",
    "parapetaremos": "parapetar",
    "parapetaren": "parapetar",
    "parapetares": "parapetar",
    "parapetaron": "parapetar",
    "parapetará": "parapetar",
    "parapetarán": "parapetar",
    "parapetarás": "parapetar",
    "parapetaré": "parapetar",
    "parapetaréis": "parapetar",
    "parapetaría": "parapetar",
    "parapetaríais": "parapetar",
    "parapetaríamos": "parapetar",
    "parapetarían": "parapetar",
    "parapetarías": "parapetar",
    "parapetas": "parapetar",
    "parapetase": "parapetar",
    "parapetaseis": "parapetar",
    "parapetasen": "parapetar",
    "parapetases": "parapetar",
    "parapetaste": "parapetar",
    "parapetasteis": "parapetar",
    "parapete": "parapetar",
    "parapetemos": "parapetar",
    "parapeten": "parapetar",
    "parapetes": "parapetar",
    "parapeto": "parapetar",
    "parapetos": "parapeto",
    "parapetábamos": "parapetar",
    "parapetáis": "parapetar",
    "parapetáramos": "parapetar",
    "parapetáremos": "parapetar",
    "parapetásemos": "parapetar",
    "parapeté": "parapetar",
    "parapetéis": "parapetar",
    "parapetó": "parapetar",
    "paraplasmas": "paraplasma",
    "paraplejias": "paraplejia",
    "paraplejías": "paraplejía",
    "parapléjica": "parapléjico",
    "parapléjicas": "parapléjico",
    "parapléjicos": "parapléjico",
    "parapocos": "parapoco",
    "paraproteinemias": "paraproteinemia",
    "paraproteínas": "paraproteína",
    "parapsicologías": "parapsicología",
    "parara": "parar",
    "pararais": "parar",
    "pararan": "parar",
    "pararas": "parar",
    "parare": "parar",
    "parareis": "parar",
    "pararemos": "parar",
    "pararen": "parar",
    "pararendsinas": "pararendsina",
    "parares": "parar",
    "pararon": "parar",
    "parará": "parar",
    "pararán": "parar",
    "pararás": "parar",
    "pararé": "parar",
    "pararéis": "parar",
    "pararía": "parar",
    "pararíais": "parar",
    "pararíamos": "parar",
    "pararían": "parar",
    "pararías": "parar",
    "paras": "parir",
    "parase": "parar",
    "paraseis": "parar",
    "paraselenes": "paraselene",
    "parasemos": "parasemo",
    "parasen": "parar",
    "parases": "parar",
    "parasimpática": "parasimpático",
    "parasimpáticas": "parasimpático",
    "parasimpáticos": "parasimpático",
    "parasintética": "parasintético",
    "parasintéticas": "parasintético",
    "parasintéticos": "parasintético",
    "parasitada": "parasitado",
    "parasitadas": "parasitado",
    "parasitados": "parasitado",
    "parasitaria": "parasitario",
    "parasitarias": "parasitario",
    "parasitarios": "parasitario",
    "parasiticidas": "parasiticida",
    "parasitismos": "parasitismo",
    "parasitologías": "parasitología",
    "parasoles": "parasol",
    "paraste": "parar",
    "parasteis": "parar",
    "parasítica": "parasítico",
    "parasíticas": "parasítico",
    "parasíticos": "parasítico",
    "paratas": "parata",
    "paraterminales": "paraterminal",
    "paratifoideas": "paratifoidea",
    "paratiroidea": "paratiroideo",
    "paratiroideas": "paratiroideo",
    "paratiroideos": "paratiroideo",
    "paratáctica": "paratáctico",
    "paratácticas": "paratáctico",
    "paratácticos": "paratáctico",
    "paratífica": "paratífico",
    "paratíficas": "paratífico",
    "paratíficos": "paratífico",
    "paraumbilicales": "paraumbilical",
    "parauretrales": "parauretral",
    "paraventriculares": "paraventricular",
    "paraviento": "paravientos",
    "paraxiales": "paraxial",
    "paraísos": "paraíso",
    "parca": "parco",
    "parcas": "parco",
    "parcela": "parcelar",
    "parcelaba": "parcelar",
    "parcelabais": "parcelar",
    "parcelaban": "parcelar",
    "parcelabas": "parcelar",
    "parcelables": "parcelable",
    "parcelaciones": "parcelación",
    "parcelad": "parcelar",
    "parcelada": "parcelar",
    "parceladas": "parcelar",
    "parcelado": "parcelar",
    "parcelados": "parcelar",
    "parcelamos": "parcelar",
    "parcelan": "parcelar",
    "parcelando": "parcelar",
    "parcelara": "parcelar",
    "parcelarais": "parcelar",
    "parcelaran": "parcelar",
    "parcelaras": "parcelar",
    "parcelare": "parcelar",
    "parcelareis": "parcelar",
    "parcelaremos": "parcelar",
    "parcelaren": "parcelar",
    "parcelares": "parcelar",
    "parcelaria": "parcelario",
    "parcelarias": "parcelario",
    "parcelarios": "parcelario",
    "parcelaron": "parcelar",
    "parcelará": "parcelar",
    "parcelarán": "parcelar",
    "parcelarás": "parcelar",
    "parcelaré": "parcelar",
    "parcelaréis": "parcelar",
    "parcelaría": "parcelar",
    "parcelaríais": "parcelar",
    "parcelaríamos": "parcelar",
    "parcelarían": "parcelar",
    "parcelarías": "parcelar",
    "parcelas": "parcelar",
    "parcelase": "parcelar",
    "parcelaseis": "parcelar",
    "parcelasen": "parcelar",
    "parcelases": "parcelar",
    "parcelaste": "parcelar",
    "parcelasteis": "parcelar",
    "parcele": "parcelar",
    "parcelemos": "parcelar",
    "parcelen": "parcelar",
    "parceles": "parcelar",
    "parcelo": "parcelar",
    "parcelábamos": "parcelar",
    "parceláis": "parcelar",
    "parceláramos": "parcelar",
    "parceláremos": "parcelar",
    "parcelásemos": "parcelar",
    "parcelé": "parcelar",
    "parceléis": "parcelar",
    "parceló": "parcelar",
    "parchas": "parcha",
    "parchazos": "parchazo",
    "parchea": "parchear",
    "parcheaba": "parchear",
    "parcheabais": "parchear",
    "parcheaban": "parchear",
    "parcheabas": "parchear",
    "parchead": "parchear",
    "parcheada": "parchear",
    "parcheadas": "parchear",
    "parcheado": "parchear",
    "parcheadora": "parcheador",
    "parcheadoras": "parcheador",
    "parcheadores": "parcheador",
    "parcheados": "parchear",
    "parcheamos": "parchear",
    "parchean": "parchear",
    "parcheando": "parchear",
    "parcheara": "parchear",
    "parchearais": "parchear",
    "parchearan": "parchear",
    "parchearas": "parchear",
    "parcheare": "parchear",
    "parcheareis": "parchear",
    "parchearemos": "parchear",
    "parchearen": "parchear",
    "parcheares": "parchear",
    "parchearon": "parchear",
    "parcheará": "parchear",
    "parchearán": "parchear",
    "parchearás": "parchear",
    "parchearé": "parchear",
    "parchearéis": "parchear",
    "parchearía": "parchear",
    "parchearíais": "parchear",
    "parchearíamos": "parchear",
    "parchearían": "parchear",
    "parchearías": "parchear",
    "parcheas": "parchear",
    "parchease": "parchear",
    "parcheaseis": "parchear",
    "parcheasen": "parchear",
    "parcheases": "parchear",
    "parcheaste": "parchear",
    "parcheasteis": "parchear",
    "parchee": "parchear",
    "parcheemos": "parchear",
    "parcheen": "parchear",
    "parchees": "parchear",
    "parcheo": "parchear",
    "parcheos": "parcheo",
    "parches": "parche",
    "parcheábamos": "parchear",
    "parcheáis": "parchear",
    "parcheáramos": "parchear",
    "parcheáremos": "parchear",
    "parcheásemos": "parchear",
    "parcheé": "parchear",
    "parcheéis": "parchear",
    "parcheó": "parchear",
    "parchises": "parchís",
    "parciales": "parcial",
    "parcialidades": "parcialidad",
    "parcializaciones": "parcialización",
    "parcos": "parco",
    "parda": "pardo",
    "pardales": "pardal",
    "pardas": "pardo",
    "pardea": "pardear",
    "pardeaba": "pardear",
    "pardeabais": "pardear",
    "pardeaban": "pardear",
    "pardeabas": "pardear",
    "pardead": "pardear",
    "pardeada": "pardear",
    "pardeadas": "pardear",
    "pardeado": "pardear",
    "pardeados": "pardear",
    "pardeamos": "pardear",
    "pardean": "pardear",
    "pardeando": "pardear",
    "pardeara": "pardear",
    "pardearais": "pardear",
    "pardearan": "pardear",
    "pardearas": "pardear",
    "pardeare": "pardear",
    "pardeareis": "pardear",
    "pardearemos": "pardear",
    "pardearen": "pardear",
    "pardeares": "pardear",
    "pardearon": "pardear",
    "pardeará": "pardear",
    "pardearán": "pardear",
    "pardearás": "pardear",
    "pardearé": "pardear",
    "pardearéis": "pardear",
    "pardearía": "pardear",
    "pardearíais": "pardear",
    "pardearíamos": "pardear",
    "pardearían": "pardear",
    "pardearías": "pardear",
    "pardeas": "pardear",
    "pardease": "pardear",
    "pardeaseis": "pardear",
    "pardeasen": "pardear",
    "pardeases": "pardear",
    "pardeaste": "pardear",
    "pardeasteis": "pardear",
    "pardee": "pardear",
    "pardeemos": "pardear",
    "pardeen": "pardear",
    "pardees": "pardear",
    "pardeo": "pardear",
    "pardeábamos": "pardear",
    "pardeáis": "pardear",
    "pardeáramos": "pardear",
    "pardeáremos": "pardear",
    "pardeásemos": "pardear",
    "pardeé": "pardear",
    "pardeéis": "pardear",
    "pardeó": "pardear",
    "pardilla": "pardillo",
    "pardillas": "pardillo",
    "pardillos": "pardillo",
    "pardisca": "pardisco",
    "pardiscas": "pardisco",
    "pardiscos": "pardisco",
    "pardogrisácea": "pardogrisáceo",
    "pardogrisáceas": "pardogrisáceo",
    "pardogrisáceos": "pardogrisáceo",
    "pardos": "pardo",
    "pardusca": "pardusco",
    "parduscas": "pardusco",
    "parduscos": "pardusco",
    "pare": "parir",
    "parea": "parear",
    "pareaba": "parear",
    "pareabais": "parear",
    "pareaban": "parear",
    "pareabas": "parear",
    "paread": "parear",
    "pareada": "parear",
    "pareadas": "parear",
    "pareado": "parear",
    "pareados": "parear",
    "pareamos": "parear",
    "parean": "parear",
    "pareando": "parear",
    "pareara": "parear",
    "parearais": "parear",
    "parearan": "parear",
    "parearas": "parear",
    "pareare": "parear",
    "pareareis": "parear",
    "parearemos": "parear",
    "parearen": "parear",
    "pareares": "parear",
    "parearon": "parear",
    "pareará": "parear",
    "parearán": "parear",
    "parearás": "parear",
    "parearé": "parear",
    "parearéis": "parear",
    "parearía": "parear",
    "parearíais": "parear",
    "parearíamos": "parear",
    "parearían": "parear",
    "parearías": "parear",
    "pareas": "parear",
    "parease": "parear",
    "pareaseis": "parear",
    "pareasen": "parear",
    "pareases": "parear",
    "pareaste": "parear",
    "pareasteis": "parear",
    "parece": "parecer",
    "pareced": "parecer",
    "parecemos": "parecer",
    "parecen": "parecer",
    "pareceremos": "parecer",
    "pareceres": "parecer",
    "parecerá": "parecer",
    "parecerán": "parecer",
    "parecerás": "parecer",
    "pareceré": "parecer",
    "pareceréis": "parecer",
    "parecería": "parecer",
    "pareceríais": "parecer",
    "pareceríamos": "parecer",
    "parecerían": "parecer",
    "parecerías": "parecer",
    "pareces": "parecer",
    "parecida": "parecer",
    "parecidas": "parecer",
    "parecido": "parecer",
    "parecidos": "parecido",
    "pareciendo": "parecer",
    "pareciera": "parecer",
    "parecierais": "parecer",
    "parecieran": "parecer",
    "parecieras": "parecer",
    "pareciere": "parecer",
    "pareciereis": "parecer",
    "parecieren": "parecer",
    "parecieres": "parecer",
    "parecieron": "parecer",
    "pareciese": "parecer",
    "parecieseis": "parecer",
    "pareciesen": "parecer",
    "parecieses": "parecer",
    "parecimos": "parecer",
    "pareciste": "parecer",
    "parecisteis": "parecer",
    "pareciéramos": "parecer",
    "pareciéremos": "parecer",
    "pareciésemos": "parecer",
    "pareció": "parecer",
    "parecéis": "parecer",
    "parecí": "parecer",
    "parecía": "parecer",
    "parecíais": "parecer",
    "parecíamos": "parecer",
    "parecían": "parecer",
    "parecías": "parecer",
    "paredaña": "paredaño",
    "paredañas": "paredaño",
    "paredaños": "paredaño",
    "paredes": "pared",
    "paredones": "paredón",
    "paree": "parear",
    "pareemos": "parear",
    "pareen": "parear",
    "parees": "parear",
    "pareja": "parejo",
    "parejas": "parejo",
    "parejitas": "parejita",
    "parejos": "parejo",
    "parejuras": "parejura",
    "parelias": "parelia",
    "parelios": "parelio",
    "paremias": "paremia",
    "paremiologías": "paremiología",
    "paremiológica": "paremiológico",
    "paremiológicas": "paremiológico",
    "paremiológicos": "paremiológico",
    "paremióloga": "paremiólogo",
    "paremiólogas": "paremiólogo",
    "paremiólogos": "paremiólogo",
    "paremos": "parar",
    "paren": "parir",
    "parenquimatosa": "parenquimatoso",
    "parenquimatosas": "parenquimatoso",
    "parenquimatosos": "parenquimatoso",
    "parenquimática": "parenquimático",
    "parenquimáticas": "parenquimático",
    "parenquimáticos": "parenquimático",
    "parentales": "parental",
    "parentelas": "parentela",
    "parentescos": "parentesco",
    "parentetizaciones": "parentetización",
    "parentética": "parentético",
    "parentéticas": "parentético",
    "parentéticos": "parentético",
    "parenética": "parenético",
    "parenéticas": "parenético",
    "parenéticos": "parenético",
    "pareo": "parear",
    "pareos": "pareo",
    "pares": "parir",
    "paresas": "paresa",
    "paresias": "paresia",
    "parestesias": "parestesia",
    "parezca": "parecer",
    "parezcamos": "parecer",
    "parezcan": "parecer",
    "parezcas": "parecer",
    "parezco": "parecer",
    "parezcáis": "parecer",
    "pareábamos": "parear",
    "pareáis": "parear",
    "pareáramos": "parear",
    "pareáremos": "parear",
    "pareásemos": "parear",
    "pareé": "parear",
    "pareéis": "parear",
    "pareó": "parear",
    "pargasitas": "pargasita",
    "pargos": "pargo",
    "parhelios": "parhelio",
    "parhileras": "parhilera",
    "parhélica": "parhélico",
    "parhélicas": "parhélico",
    "parhélicos": "parhélico",
    "pariambos": "pariambo",
    "parias": "paria",
    "pariciones": "parición",
    "parid": "parir",
    "parida": "parir",
    "paridades": "paridad",
    "paridas": "parir",
    "parideras": "paridera",
    "parido": "parir",
    "paridoras": "paridora",
    "paridos": "parir",
    "pariendo": "parir",
    "parienta": "pariente",
    "parientas": "pariente",
    "parientes": "pariente",
    "pariera": "parir",
    "parierais": "parir",
    "parieran": "parir",
    "parieras": "parir",
    "pariere": "parir",
    "pariereis": "parir",
    "parieren": "parir",
    "parieres": "parir",
    "parieron": "parir",
    "pariese": "parir",
    "parieseis": "parir",
    "pariesen": "parir",
    "parieses": "parir",
    "parietales": "parietal",
    "parietarias": "parietaria",
    "parietomastoidea": "parietomastoideo",
    "parietomastoideas": "parietomastoideo",
    "parietomastoideos": "parietomastoideo",
    "parietooccipitales": "parietooccipital",
    "parifica": "parificar",
    "parificaba": "parificar",
    "parificabais": "parificar",
    "parificaban": "parificar",
    "parificabas": "parificar",
    "parificaciones": "parificación",
    "parificad": "parificar",
    "parificada": "parificar",
    "parificadas": "parificar",
    "parificado": "parificar",
    "parificados": "parificar",
    "parificamos": "parificar",
    "parifican": "parificar",
    "parificando": "parificar",
    "parificara": "parificar",
    "parificarais": "parificar",
    "parificaran": "parificar",
    "parificaras": "parificar",
    "parificare": "parificar",
    "parificareis": "parificar",
    "parificaremos": "parificar",
    "parificaren": "parificar",
    "parificares": "parificar",
    "parificaron": "parificar",
    "parificará": "parificar",
    "parificarán": "parificar",
    "parificarás": "parificar",
    "parificaré": "parificar",
    "parificaréis": "parificar",
    "parificaría": "parificar",
    "parificaríais": "parificar",
    "parificaríamos": "parificar",
    "parificarían": "parificar",
    "parificarías": "parificar",
    "parificas": "parificar",
    "parificase": "parificar",
    "parificaseis": "parificar",
    "parificasen": "parificar",
    "parificases": "parificar",
    "parificaste": "parificar",
    "parificasteis": "parificar",
    "parifico": "parificar",
    "parificábamos": "parificar",
    "parificáis": "parificar",
    "parificáramos": "parificar",
    "parificáremos": "parificar",
    "parificásemos": "parificar",
    "parificó": "parificar",
    "parifique": "parificar",
    "parifiquemos": "parificar",
    "parifiquen": "parificar",
    "parifiques": "parificar",
    "parifiqué": "parificar",
    "parifiquéis": "parificar",
    "pariguales": "parigual",
    "parihuelas": "parihuela",
    "parimos": "parir",
    "paripés": "paripé",
    "pariremos": "parir",
    "parirá": "parir",
    "parirán": "parir",
    "parirás": "parir",
    "pariré": "parir",
    "pariréis": "parir",
    "pariría": "parir",
    "pariríais": "parir",
    "pariríamos": "parir",
    "parirían": "parir",
    "parirías": "parir",
    "parisienas": "parisiena",
    "parisienses": "parisiense",
    "parisilábica": "parisilábico",
    "parisilábicas": "parisilábico",
    "parisilábicos": "parisilábico",
    "parisina": "parisino",
    "parisinas": "parisino",
    "parisinos": "parisino",
    "parisiénes": "parisién",
    "pariste": "parir",
    "paristeis": "parir",
    "parisílaba": "parisílabo",
    "parisílabas": "parisílabo",
    "parisílabos": "parisílabo",
    "paritaria": "paritario",
    "paritarias": "paritario",
    "paritarios": "paritario",
    "pariéramos": "parir",
    "pariéremos": "parir",
    "pariésemos": "parir",
    "parió": "parir",
    "parkas": "parka",
    "parkerizaciones": "parkerización",
    "parkings": "parking",
    "parla": "parlar",
    "parlaba": "parlar",
    "parlabais": "parlar",
    "parlaban": "parlar",
    "parlabas": "parlar",
    "parlad": "parlar",
    "parlada": "parlar",
    "parladas": "parlar",
    "parlado": "parlar",
    "parlados": "parlar",
    "parlamenta": "parlamentar",
    "parlamentaba": "parlamentar",
    "parlamentabais": "parlamentar",
    "parlamentaban": "parlamentar",
    "parlamentabas": "parlamentar",
    "parlamentad": "parlamentar",
    "parlamentada": "parlamentar",
    "parlamentadas": "parlamentar",
    "parlamentado": "parlamentar",
    "parlamentados": "parlamentar",
    "parlamentamos": "parlamentar",
    "parlamentan": "parlamentar",
    "parlamentando": "parlamentar",
    "parlamentara": "parlamentar",
    "parlamentarais": "parlamentar",
    "parlamentaran": "parlamentar",
    "parlamentaras": "parlamentar",
    "parlamentare": "parlamentar",
    "parlamentareis": "parlamentar",
    "parlamentaremos": "parlamentar",
    "parlamentaren": "parlamentar",
    "parlamentares": "parlamentar",
    "parlamentaria": "parlamentario",
    "parlamentarias": "parlamentario",
    "parlamentarios": "parlamentario",
    "parlamentarismos": "parlamentarismo",
    "parlamentaristas": "parlamentarista",
    "parlamentaron": "parlamentar",
    "parlamentará": "parlamentar",
    "parlamentarán": "parlamentar",
    "parlamentarás": "parlamentar",
    "parlamentaré": "parlamentar",
    "parlamentaréis": "parlamentar",
    "parlamentaría": "parlamentar",
    "parlamentaríais": "parlamentar",
    "parlamentaríamos": "parlamentar",
    "parlamentarían": "parlamentar",
    "parlamentarías": "parlamentar",
    "parlamentas": "parlamentar",
    "parlamentase": "parlamentar",
    "parlamentaseis": "parlamentar",
    "parlamentasen": "parlamentar",
    "parlamentases": "parlamentar",
    "parlamentaste": "parlamentar",
    "parlamentasteis": "parlamentar",
    "parlamente": "parlamentar",
    "parlamentemos": "parlamentar",
    "parlamenten": "parlamentar",
    "parlamentes": "parlamentar",
    "parlamento": "parlamentar",
    "parlamentos": "parlamento",
    "parlamentábamos": "parlamentar",
    "parlamentáis": "parlamentar",
    "parlamentáramos": "parlamentar",
    "parlamentáremos": "parlamentar",
    "parlamentásemos": "parlamentar",
    "parlamenté": "parlamentar",
    "parlamentéis": "parlamentar",
    "parlamentó": "parlamentar",
    "parlamos": "parlar",
    "parlan": "parlar",
    "parlanchina": "parlanchín",
    "parlanchinas": "parlanchín",
    "parlanchines": "parlanchín",
    "parlando": "parlar",
    "parlara": "parlar",
    "parlarais": "parlar",
    "parlaran": "parlar",
    "parlaras": "parlar",
    "parlare": "parlar",
    "parlareis": "parlar",
    "parlaremos": "parlar",
    "parlaren": "parlar",
    "parlares": "parlar",
    "parlaron": "parlar",
    "parlará": "parlar",
    "parlarán": "parlar",
    "parlarás": "parlar",
    "parlaré": "parlar",
    "parlaréis": "parlar",
    "parlaría": "parlar",
    "parlaríais": "parlar",
    "parlaríamos": "parlar",
    "parlarían": "parlar",
    "parlarías": "parlar",
    "parlas": "parlar",
    "parlase": "parlar",
    "parlaseis": "parlar",
    "parlasen": "parlar",
    "parlases": "parlar",
    "parlaste": "parlar",
    "parlasteis": "parlar",
    "parlatorios": "parlatorio",
    "parle": "parlar",
    "parlemos": "parlar",
    "parlen": "parlar",
    "parlera": "parlero",
    "parleras": "parlero",
    "parleros": "parlero",
    "parles": "parlar",
    "parlo": "parlar",
    "parlona": "parlón",
    "parlonas": "parlón",
    "parlones": "parlón",
    "parlotea": "parlotear",
    "parloteaba": "parlotear",
    "parloteabais": "parlotear",
    "parloteaban": "parlotear",
    "parloteabas": "parlotear",
    "parlotead": "parlotear",
    "parloteada": "parlotear",
    "parloteadas": "parlotear",
    "parloteado": "parlotear",
    "parloteados": "parlotear",
    "parloteamos": "parlotear",
    "parlotean": "parlotear",
    "parloteando": "parlotear",
    "parloteara": "parlotear",
    "parlotearais": "parlotear",
    "parlotearan": "parlotear",
    "parlotearas": "parlotear",
    "parloteare": "parlotear",
    "parloteareis": "parlotear",
    "parlotearemos": "parlotear",
    "parlotearen": "parlotear",
    "parloteares": "parlotear",
    "parlotearon": "parlotear",
    "parloteará": "parlotear",
    "parlotearán": "parlotear",
    "parlotearás": "parlotear",
    "parlotearé": "parlotear",
    "parlotearéis": "parlotear",
    "parlotearía": "parlotear",
    "parlotearíais": "parlotear",
    "parlotearíamos": "parlotear",
    "parlotearían": "parlotear",
    "parlotearías": "parlotear",
    "parloteas": "parlotear",
    "parlotease": "parlotear",
    "parloteaseis": "parlotear",
    "parloteasen": "parlotear",
    "parloteases": "parlotear",
    "parloteaste": "parlotear",
    "parloteasteis": "parlotear",
    "parlotee": "parlotear",
    "parloteemos": "parlotear",
    "parloteen": "parlotear",
    "parlotees": "parlotear",
    "parloteo": "parlotear",
    "parloteos": "parloteo",
    "parloteábamos": "parlotear",
    "parloteáis": "parlotear",
    "parloteáramos": "parlotear",
    "parloteáremos": "parlotear",
    "parloteásemos": "parlotear",
    "parloteé": "parlotear",
    "parloteéis": "parlotear",
    "parloteó": "parlotear",
    "parlábamos": "parlar",
    "parláis": "parlar",
    "parláramos": "parlar",
    "parláremos": "parlar",
    "parlásemos": "parlar",
    "parlé": "parlar",
    "parléis": "parlar",
    "parló": "parlar",
    "parmesana": "parmesano",
    "parmesanas": "parmesano",
    "parmesanos": "parmesano",
    "parnasiana": "parnasiano",
    "parnasianas": "parnasiano",
    "parnasianismos": "parnasianismo",
    "parnasianos": "parnasiano",
    "parnasos": "parnaso",
    "parnés": "parné",
    "paro": "parir",
    "parodia": "parodiar",
    "parodiaba": "parodiar",
    "parodiabais": "parodiar",
    "parodiaban": "parodiar",
    "parodiabas": "parodiar",
    "parodiad": "parodiar",
    "parodiada": "parodiar",
    "parodiadas": "parodiar",
    "parodiado": "parodiar",
    "parodiadora": "parodiador",
    "parodiadoras": "parodiador",
    "parodiadores": "parodiador",
    "parodiados": "parodiar",
    "parodiamos": "parodiar",
    "parodian": "parodiar",
    "parodiando": "parodiar",
    "parodiara": "parodiar",
    "parodiarais": "parodiar",
    "parodiaran": "parodiar",
    "parodiaras": "parodiar",
    "parodiare": "parodiar",
    "parodiareis": "parodiar",
    "parodiaremos": "parodiar",
    "parodiaren": "parodiar",
    "parodiares": "parodiar",
    "parodiaron": "parodiar",
    "parodiará": "parodiar",
    "parodiarán": "parodiar",
    "parodiarás": "parodiar",
    "parodiaré": "parodiar",
    "parodiaréis": "parodiar",
    "parodiaría": "parodiar",
    "parodiaríais": "parodiar",
    "parodiaríamos": "parodiar",
    "parodiarían": "parodiar",
    "parodiarías": "parodiar",
    "parodias": "parodiar",
    "parodiase": "parodiar",
    "parodiaseis": "parodiar",
    "parodiasen": "parodiar",
    "parodiases": "parodiar",
    "parodiaste": "parodiar",
    "parodiasteis": "parodiar",
    "parodie": "parodiar",
    "parodiemos": "parodiar",
    "parodien": "parodiar",
    "parodies": "parodiar",
    "parodio": "parodiar",
    "parodistas": "parodista",
    "parodiábamos": "parodiar",
    "parodiáis": "parodiar",
    "parodiáramos": "parodiar",
    "parodiáremos": "parodiar",
    "parodiásemos": "parodiar",
    "parodié": "parodiar",
    "parodiéis": "parodiar",
    "parodió": "parodiar",
    "parones": "parón",
    "paronimias": "paronimia",
    "paronomasias": "paronomasia",
    "paronomástica": "paronomástico",
    "paronomásticas": "paronomástico",
    "paronomásticos": "paronomástico",
    "paronímica": "paronímico",
    "paronímicas": "paronímico",
    "paronímicos": "paronímico",
    "paros": "paro",
    "parosmias": "parosmia",
    "parotídea": "parotídeo",
    "parotídeas": "parotídeo",
    "parotídeoa": "parotídeo",
    "parotídeoas": "parotídeo",
    "parotídeoos": "parotídeo",
    "parotídeos": "parotídeo",
    "paroxismales": "paroxismal",
    "paroxismos": "paroxismo",
    "paroxística": "paroxístico",
    "paroxísticas": "paroxístico",
    "paroxísticos": "paroxístico",
    "paroxítona": "paroxítono",
    "paroxítonas": "paroxítono",
    "paroxítonos": "paroxítono",
    "parpa": "parpar",
    "parpaba": "parpar",
    "parpabais": "parpar",
    "parpaban": "parpar",
    "parpabas": "parpar",
    "parpad": "parpar",
    "parpada": "parpar",
    "parpadas": "parpar",
    "parpadea": "parpadear",
    "parpadeaba": "parpadear",
    "parpadeabais": "parpadear",
    "parpadeaban": "parpadear",
    "parpadeabas": "parpadear",
    "parpadead": "parpadear",
    "parpadeada": "parpadear",
    "parpadeadas": "parpadear",
    "parpadeado": "parpadear",
    "parpadeados": "parpadear",
    "parpadeamos": "parpadear",
    "parpadean": "parpadear",
    "parpadeando": "parpadear",
    "parpadeara": "parpadear",
    "parpadearais": "parpadear",
    "parpadearan": "parpadear",
    "parpadearas": "parpadear",
    "parpadeare": "parpadear",
    "parpadeareis": "parpadear",
    "parpadearemos": "parpadear",
    "parpadearen": "parpadear",
    "parpadeares": "parpadear",
    "parpadearon": "parpadear",
    "parpadeará": "parpadear",
    "parpadearán": "parpadear",
    "parpadearás": "parpadear",
    "parpadearé": "parpadear",
    "parpadearéis": "parpadear",
    "parpadearía": "parpadear",
    "parpadearíais": "parpadear",
    "parpadearíamos": "parpadear",
    "parpadearían": "parpadear",
    "parpadearías": "parpadear",
    "parpadeas": "parpadear",
    "parpadease": "parpadear",
    "parpadeaseis": "parpadear",
    "parpadeasen": "parpadear",
    "parpadeases": "parpadear",
    "parpadeaste": "parpadear",
    "parpadeasteis": "parpadear",
    "parpadee": "parpadear",
    "parpadeemos": "parpadear",
    "parpadeen": "parpadear",
    "parpadees": "parpadear",
    "parpadeo": "parpadear",
    "parpadeos": "parpadeo",
    "parpadeábamos": "parpadear",
    "parpadeáis": "parpadear",
    "parpadeáramos": "parpadear",
    "parpadeáremos": "parpadear",
    "parpadeásemos": "parpadear",
    "parpadeé": "parpadear",
    "parpadeéis": "parpadear",
    "parpadeó": "parpadear",
    "parpado": "parpar",
    "parpados": "parpar",
    "parpamos": "parpar",
    "parpan": "parpar",
    "parpando": "parpar",
    "parpara": "parpar",
    "parparais": "parpar",
    "parparan": "parpar",
    "parparas": "parpar",
    "parpare": "parpar",
    "parpareis": "parpar",
    "parparemos": "parpar",
    "parparen": "parpar",
    "parpares": "parpar",
    "parparon": "parpar",
    "parpará": "parpar",
    "parparán": "parpar",
    "parparás": "parpar",
    "parparé": "parpar",
    "parparéis": "parpar",
    "parparía": "parpar",
    "parparíais": "parpar",
    "parparíamos": "parpar",
    "parparían": "parpar",
    "parparías": "parpar",
    "parpas": "parpar",
    "parpase": "parpar",
    "parpaseis": "parpar",
    "parpasen": "parpar",
    "parpases": "parpar",
    "parpaste": "parpar",
    "parpasteis": "parpar",
    "parpe": "parpar",
    "parpemos": "parpar",
    "parpen": "parpar",
    "parpes": "parpar",
    "parpo": "parpar",
    "parpábamos": "parpar",
    "parpáis": "parpar",
    "parpáramos": "parpar",
    "parpáremos": "parpar",
    "parpásemos": "parpar",
    "parpé": "parpar",
    "parpéis": "parpar",
    "parpó": "parpar",
    "parquea": "parquear",
    "parqueaba": "parquear",
    "parqueabais": "parquear",
    "parqueaban": "parquear",
    "parqueabas": "parquear",
    "parquead": "parquear",
    "parqueada": "parquear",
    "parqueadas": "parquear",
    "parqueado": "parquear",
    "parqueados": "parquear",
    "parqueamos": "parquear",
    "parquean": "parquear",
    "parqueando": "parquear",
    "parqueara": "parquear",
    "parquearais": "parquear",
    "parquearan": "parquear",
    "parquearas": "parquear",
    "parqueare": "parquear",
    "parqueareis": "parquear",
    "parquearemos": "parquear",
    "parquearen": "parquear",
    "parqueares": "parquear",
    "parquearon": "parquear",
    "parqueará": "parquear",
    "parquearán": "parquear",
    "parquearás": "parquear",
    "parquearé": "parquear",
    "parquearéis": "parquear",
    "parquearía": "parquear",
    "parquearíais": "parquear",
    "parquearíamos": "parquear",
    "parquearían": "parquear",
    "parquearías": "parquear",
    "parqueas": "parquear",
    "parquease": "parquear",
    "parqueaseis": "parquear",
    "parqueasen": "parquear",
    "parqueases": "parquear",
    "parqueaste": "parquear",
    "parqueasteis": "parquear",
    "parquedades": "parquedad",
    "parquee": "parquear",
    "parqueemos": "parquear",
    "parqueen": "parquear",
    "parquees": "parquear",
    "parqueo": "parquear",
    "parqueos": "parqueo",
    "parques": "parque",
    "parquets": "parquet",
    "parqueábamos": "parquear",
    "parqueáis": "parquear",
    "parqueáramos": "parquear",
    "parqueáremos": "parquear",
    "parqueásemos": "parquear",
    "parqueé": "parquear",
    "parqueéis": "parquear",
    "parqueó": "parquear",
    "parqués": "parqué",
    "parquímetros": "parquímetro",
    "parrafadas": "parrafada",
    "parrafea": "parrafear",
    "parrafeaba": "parrafear",
    "parrafeabais": "parrafear",
    "parrafeaban": "parrafear",
    "parrafeabas": "parrafear",
    "parrafead": "parrafear",
    "parrafeada": "parrafear",
    "parrafeadas": "parrafear",
    "parrafeado": "parrafear",
    "parrafeados": "parrafear",
    "parrafeamos": "parrafear",
    "parrafean": "parrafear",
    "parrafeando": "parrafear",
    "parrafeara": "parrafear",
    "parrafearais": "parrafear",
    "parrafearan": "parrafear",
    "parrafearas": "parrafear",
    "parrafeare": "parrafear",
    "parrafeareis": "parrafear",
    "parrafearemos": "parrafear",
    "parrafearen": "parrafear",
    "parrafeares": "parrafear",
    "parrafearon": "parrafear",
    "parrafeará": "parrafear",
    "parrafearán": "parrafear",
    "parrafearás": "parrafear",
    "parrafearé": "parrafear",
    "parrafearéis": "parrafear",
    "parrafearía": "parrafear",
    "parrafearíais": "parrafear",
    "parrafearíamos": "parrafear",
    "parrafearían": "parrafear",
    "parrafearías": "parrafear",
    "parrafeas": "parrafear",
    "parrafease": "parrafear",
    "parrafeaseis": "parrafear",
    "parrafeasen": "parrafear",
    "parrafeases": "parrafear",
    "parrafeaste": "parrafear",
    "parrafeasteis": "parrafear",
    "parrafee": "parrafear",
    "parrafeemos": "parrafear",
    "parrafeen": "parrafear",
    "parrafees": "parrafear",
    "parrafeo": "parrafear",
    "parrafeos": "parrafeo",
    "parrafeábamos": "parrafear",
    "parrafeáis": "parrafear",
    "parrafeáramos": "parrafear",
    "parrafeáremos": "parrafear",
    "parrafeásemos": "parrafear",
    "parrafeé": "parrafear",
    "parrafeéis": "parrafear",
    "parrafeó": "parrafear",
    "parrales": "parral",
    "parrandas": "parranda",
    "parrandea": "parrandear",
    "parrandeaba": "parrandear",
    "parrandeabais": "parrandear",
    "parrandeaban": "parrandear",
    "parrandeabas": "parrandear",
    "parrandead": "parrandear",
    "parrandeada": "parrandear",
    "parrandeadas": "parrandear",
    "parrandeado": "parrandear",
    "parrandeados": "parrandear",
    "parrandeamos": "parrandear",
    "parrandean": "parrandear",
    "parrandeando": "parrandear",
    "parrandeara": "parrandear",
    "parrandearais": "parrandear",
    "parrandearan": "parrandear",
    "parrandearas": "parrandear",
    "parrandeare": "parrandear",
    "parrandeareis": "parrandear",
    "parrandearemos": "parrandear",
    "parrandearen": "parrandear",
    "parrandeares": "parrandear",
    "parrandearon": "parrandear",
    "parrandeará": "parrandear",
    "parrandearán": "parrandear",
    "parrandearás": "parrandear",
    "parrandearé": "parrandear",
    "parrandearéis": "parrandear",
    "parrandearía": "parrandear",
    "parrandearíais": "parrandear",
    "parrandearíamos": "parrandear",
    "parrandearían": "parrandear",
    "parrandearías": "parrandear",
    "parrandeas": "parrandear",
    "parrandease": "parrandear",
    "parrandeaseis": "parrandear",
    "parrandeasen": "parrandear",
    "parrandeases": "parrandear",
    "parrandeaste": "parrandear",
    "parrandeasteis": "parrandear",
    "parrandee": "parrandear",
    "parrandeemos": "parrandear",
    "parrandeen": "parrandear",
    "parrandees": "parrandear",
    "parrandeo": "parrandear",
    "parrandeos": "parrandeo",
    "parrandera": "parrandero",
    "parranderas": "parrandero",
    "parranderos": "parrandero",
    "parrandeábamos": "parrandear",
    "parrandeáis": "parrandear",
    "parrandeáramos": "parrandear",
    "parrandeáremos": "parrandear",
    "parrandeásemos": "parrandear",
    "parrandeé": "parrandear",
    "parrandeéis": "parrandear",
    "parrandeó": "parrandear",
    "parras": "parra",
    "parresias": "parresia",
    "parricidas": "parricida",
    "parricidios": "parricidio",
    "parrilladas": "parrillada",
    "parrillas": "parrilla",
    "parrizas": "parriza",
    "parrones": "parrón",
    "parroquiales": "parroquial",
    "parroquialidades": "parroquialidad",
    "parroquiana": "parroquiano",
    "parroquianas": "parroquiano",
    "parroquianos": "parroquiano",
    "parroquias": "parroquia",
    "parros": "parro",
    "parsecs": "parsec",
    "parsies": "parsi",
    "parsimonias": "parsimonia",
    "parsimoniosa": "parsimonioso",
    "parsimoniosas": "parsimonioso",
    "parsimoniosos": "parsimonioso",
    "parsis": "parsi",
    "parsismos": "parsismo",
    "parta": "partir",
    "partamos": "partir",
    "partan": "partir",
    "partas": "partir",
    "parte": "partir",
    "partea": "partear",
    "parteaba": "partear",
    "parteabais": "partear",
    "parteaban": "partear",
    "parteabas": "partear",
    "partead": "partear",
    "parteada": "partear",
    "parteadas": "partear",
    "parteado": "partear",
    "parteados": "partear",
    "parteamos": "partear",
    "partean": "partear",
    "parteando": "partear",
    "parteara": "partear",
    "partearais": "partear",
    "partearan": "partear",
    "partearas": "partear",
    "parteare": "partear",
    "parteareis": "partear",
    "partearemos": "partear",
    "partearen": "partear",
    "parteares": "partear",
    "partearon": "partear",
    "parteará": "partear",
    "partearán": "partear",
    "partearás": "partear",
    "partearé": "partear",
    "partearéis": "partear",
    "partearía": "partear",
    "partearíais": "partear",
    "partearíamos": "partear",
    "partearían": "partear",
    "partearías": "partear",
    "parteas": "partear",
    "partease": "partear",
    "parteaseis": "partear",
    "parteasen": "partear",
    "parteases": "partear",
    "parteaste": "partear",
    "parteasteis": "partear",
    "partee": "partear",
    "parteemos": "partear",
    "parteen": "partear",
    "partees": "partear",
    "parteluces": "parteluz",
    "parten": "partir",
    "partenocarpias": "partenocarpia",
    "partenogamias": "partenogamia",
    "partenogenética": "partenogenético",
    "partenogenéticas": "partenogenético",
    "partenogenéticos": "partenogenético",
    "parteo": "partear",
    "partera": "partero",
    "parteras": "partero",
    "parteros": "partero",
    "parterres": "parterre",
    "parterías": "partería",
    "partes": "partir",
    "partesanas": "partesana",
    "parteábamos": "partear",
    "parteáis": "partear",
    "parteáramos": "partear",
    "parteáremos": "partear",
    "parteásemos": "partear",
    "parteé": "partear",
    "parteéis": "partear",
    "parteó": "partear",
    "partibles": "partible",
    "particiones": "partición",
    "participa": "participar",
    "participaba": "participar",
    "participabais": "participar",
    "participaban": "participar",
    "participabas": "participar",
    "participables": "participable",
    "participaciones": "participación",
    "participad": "participar",
    "participada": "participar",
    "participadas": "participar",
    "participado": "participar",
    "participados": "participar",
    "participamos": "participar",
    "participan": "participar",
    "participando": "participar",
    "participantes": "participante",
    "participara": "participar",
    "participarais": "participar",
    "participaran": "participar",
    "participaras": "participar",
    "participare": "participar",
    "participareis": "participar",
    "participaremos": "participar",
    "participaren": "participar",
    "participares": "participar",
    "participaron": "participar",
    "participará": "participar",
    "participarán": "participar",
    "participarás": "participar",
    "participaré": "participar",
    "participaréis": "participar",
    "participaría": "participar",
    "participaríais": "participar",
    "participaríamos": "participar",
    "participarían": "participar",
    "participarías": "participar",
    "participas": "participar",
    "participase": "participar",
    "participaseis": "participar",
    "participasen": "participar",
    "participases": "participar",
    "participaste": "participar",
    "participasteis": "participar",
    "participativa": "participativo",
    "participativas": "participativo",
    "participativos": "participativo",
    "participe": "participar",
    "participemos": "participar",
    "participen": "participar",
    "participes": "participar",
    "participios": "participio",
    "participo": "participar",
    "participábamos": "participar",
    "participáis": "participar",
    "participáramos": "participar",
    "participáremos": "participar",
    "participásemos": "participar",
    "participé": "participar",
    "participéis": "participar",
    "participó": "participar",
    "particulada": "particulado",
    "particuladas": "particulado",
    "particulados": "particulado",
    "particulares": "particular",
    "particularice": "particularizar",
    "particularicemos": "particularizar",
    "particularicen": "particularizar",
    "particularices": "particularizar",
    "particularicé": "particularizar",
    "particularicéis": "particularizar",
    "particularidades": "particularidad",
    "particularismos": "particularismo",
    "particularistas": "particularista",
    "particulariza": "particularizar",
    "particularizaba": "particularizar",
    "particularizabais": "particularizar",
    "particularizaban": "particularizar",
    "particularizabas": "particularizar",
    "particularizad": "particularizar",
    "particularizada": "particularizar",
    "particularizadas": "particularizar",
    "particularizado": "particularizar",
    "particularizados": "particularizar",
    "particularizamos": "particularizar",
    "particularizan": "particularizar",
    "particularizando": "particularizar",
    "particularizara": "particularizar",
    "particularizarais": "particularizar",
    "particularizaran": "particularizar",
    "particularizaras": "particularizar",
    "particularizare": "particularizar",
    "particularizareis": "particularizar",
    "particularizaremos": "particularizar",
    "particularizaren": "particularizar",
    "particularizares": "particularizar",
    "particularizaron": "particularizar",
    "particularizará": "particularizar",
    "particularizarán": "particularizar",
    "particularizarás": "particularizar",
    "particularizaré": "particularizar",
    "particularizaréis": "particularizar",
    "particularizaría": "particularizar",
    "particularizaríais": "particularizar",
    "particularizaríamos": "particularizar",
    "particularizarían": "particularizar",
    "particularizarías": "particularizar",
    "particularizas": "particularizar",
    "particularizase": "particularizar",
    "particularizaseis": "particularizar",
    "particularizasen": "particularizar",
    "particularizases": "particularizar",
    "particularizaste": "particularizar",
    "particularizasteis": "particularizar",
    "particularizo": "particularizar",
    "particularizábamos": "particularizar",
    "particularizáis": "particularizar",
    "particularizáramos": "particularizar",
    "particularizáremos": "particularizar",
    "particularizásemos": "particularizar",
    "particularizó": "particularizar",
    "partid": "partir",
    "partida": "partir",
    "partidaria": "partidario",
    "partidarias": "partidario",
    "partidarios": "partidario",
    "partidas": "partir",
    "partidazos": "partidazo",
    "partidillos": "partidillo",
    "partidismos": "partidismo",
    "partidistas": "partidista",
    "partido": "partir",
    "partidora": "partidor",
    "partidoras": "partidor",
    "partidores": "partidor",
    "partidos": "partir",
    "partiduras": "partidura",
    "partiendo": "partir",
    "partiera": "partir",
    "partierais": "partir",
    "partieran": "partir",
    "partieras": "partir",
    "partiere": "partir",
    "partiereis": "partir",
    "partieren": "partir",
    "partieres": "partir",
    "partieron": "partir",
    "partiese": "partir",
    "partieseis": "partir",
    "partiesen": "partir",
    "partieses": "partir",
    "partijas": "partija",
    "partimientos": "partimiento",
    "partimos": "partir",
    "partiquina": "partiquino",
    "partiquinas": "partiquino",
    "partiquinos": "partiquino",
    "partiremos": "partir",
    "partirá": "partir",
    "partirán": "partir",
    "partirás": "partir",
    "partiré": "partir",
    "partiréis": "partir",
    "partiría": "partir",
    "partiríais": "partir",
    "partiríamos": "partir",
    "partirían": "partir",
    "partirías": "partir",
    "partisana": "partisano",
    "partisanas": "partisano",
    "partisanos": "partisano",
    "partiste": "partir",
    "partisteis": "partir",
    "partitas": "partita",
    "partitiva": "partitivo",
    "partitivas": "partitivo",
    "partitivos": "partitivo",
    "partituras": "partitura",
    "partiéramos": "partir",
    "partiéremos": "partir",
    "partiésemos": "partir",
    "partió": "partir",
    "parto": "partir",
    "partos": "parto",
    "parturientas": "parturienta",
    "partáis": "partir",
    "partí": "partir",
    "partía": "partir",
    "partíais": "partir",
    "partíamos": "partir",
    "partían": "partir",
    "partías": "partir",
    "partícipes": "partícipe",
    "partículas": "partícula",
    "partís": "partir",
    "parusías": "parusía",
    "parva": "parvo",
    "parvadas": "parvada",
    "parvas": "parvo",
    "parvedades": "parvedad",
    "parveros": "parvero",
    "parvidades": "parvidad",
    "parvificencias": "parvificencia",
    "parvos": "parvo",
    "parvularios": "parvulario",
    "parvulistas": "parvulista",
    "parvífica": "parvífico",
    "parvíficas": "parvífico",
    "parvíficos": "parvífico",
    "parábamos": "parar",
    "parábolas": "parábola",
    "paráclitos": "paráclito",
    "parágrafos": "parágrafo",
    "paráis": "parir",
    "parámetros": "parámetro",
    "paráramos": "parar",
    "paráremos": "parar",
    "parásemos": "parar",
    "parásita": "parásito",
    "parásitas": "parásito",
    "parásitos": "parásito",
    "parástades": "parástade",
    "parátrofa": "parátrofo",
    "parátrofas": "parátrofo",
    "parátrofos": "parátrofo",
    "paré": "parar",
    "paréis": "parar",
    "parénquimas": "parénquima",
    "parética": "parético",
    "paréticas": "parético",
    "paréticos": "parético",
    "parí": "parir",
    "paría": "parir",
    "paríais": "parir",
    "paríamos": "parir",
    "parían": "parir",
    "parías": "parir",
    "parís": "parir",
    "paró": "parar",
    "paródica": "paródico",
    "paródicas": "paródico",
    "paródicos": "paródico",
    "parónima": "parónimo",
    "parónimas": "parónimo",
    "parónimos": "parónimo",
    "paróptica": "paróptico",
    "parópticas": "paróptico",
    "parópticos": "paróptico",
    "parótida": "parótido",
    "parótidas": "parótido",
    "parótidos": "parótido",
    "pasa": "paso",
    "pasaba": "pasar",
    "pasabais": "pasar",
    "pasaban": "pasar",
    "pasabandas": "pasabanda",
    "pasabas": "pasar",
    "pasables": "pasable",
    "pasacaballos": "pasacaballo",
    "pasacalles": "pasacalle",
    "pasacólicas": "pasacólica",
    "pasad": "pasar",
    "pasada": "pasar",
    "pasadas": "pasar",
    "pasadera": "pasadero",
    "pasaderas": "pasadero",
    "pasaderos": "pasadero",
    "pasadizos": "pasadizo",
    "pasado": "pasar",
    "pasadora": "pasador",
    "pasadoras": "pasador",
    "pasadores": "pasador",
    "pasados": "pasar",
    "pasadías": "pasadía",
    "pasajera": "pasajero",
    "pasajeras": "pasajero",
    "pasajeros": "pasajero",
    "pasajes": "pasaje",
    "pasajuegos": "pasajuego",
    "pasamana": "pasamanar",
    "pasamanaba": "pasamanar",
    "pasamanabais": "pasamanar",
    "pasamanaban": "pasamanar",
    "pasamanabas": "pasamanar",
    "pasamanad": "pasamanar",
    "pasamanada": "pasamanar",
    "pasamanadas": "pasamanar",
    "pasamanado": "pasamanar",
    "pasamanados": "pasamanar",
    "pasamanamos": "pasamanar",
    "pasamanan": "pasamanar",
    "pasamanando": "pasamanar",
    "pasamanara": "pasamanar",
    "pasamanarais": "pasamanar",
    "pasamanaran": "pasamanar",
    "pasamanaras": "pasamanar",
    "pasamanare": "pasamanar",
    "pasamanareis": "pasamanar",
    "pasamanaremos": "pasamanar",
    "pasamanaren": "pasamanar",
    "pasamanares": "pasamanar",
    "pasamanaron": "pasamanar",
    "pasamanará": "pasamanar",
    "pasamanarán": "pasamanar",
    "pasamanarás": "pasamanar",
    "pasamanaré": "pasamanar",
    "pasamanaréis": "pasamanar",
    "pasamanaría": "pasamanar",
    "pasamanaríais": "pasamanar",
    "pasamanaríamos": "pasamanar",
    "pasamanarían": "pasamanar",
    "pasamanarías": "pasamanar",
    "pasamanas": "pasamanar",
    "pasamanase": "pasamanar",
    "pasamanaseis": "pasamanar",
    "pasamanasen": "pasamanar",
    "pasamanases": "pasamanar",
    "pasamanaste": "pasamanar",
    "pasamanasteis": "pasamanar",
    "pasamane": "pasamanar",
    "pasamanemos": "pasamanar",
    "pasamanen": "pasamanar",
    "pasamanera": "pasamanero",
    "pasamaneras": "pasamanero",
    "pasamaneros": "pasamanero",
    "pasamanerías": "pasamanería",
    "pasamanes": "pasamanar",
    "pasamano": "pasamanar",
    "pasamanábamos": "pasamanar",
    "pasamanáis": "pasamanar",
    "pasamanáramos": "pasamanar",
    "pasamanáremos": "pasamanar",
    "pasamanásemos": "pasamanar",
    "pasamané": "pasamanar",
    "pasamanéis": "pasamanar",
    "pasamanó": "pasamanar",
    "pasamos": "pasar",
    "pasamuro": "pasamuros",
    "pasan": "pasar",
    "pasando": "pasar",
    "pasantes": "pasante",
    "pasantías": "pasantía",
    "pasapanes": "pasapán",
    "pasaporta": "pasaportar",
    "pasaportaba": "pasaportar",
    "pasaportabais": "pasaportar",
    "pasaportaban": "pasaportar",
    "pasaportabas": "pasaportar",
    "pasaportad": "pasaportar",
    "pasaportada": "pasaportar",
    "pasaportadas": "pasaportar",
    "pasaportado": "pasaportar",
    "pasaportados": "pasaportar",
    "pasaportamos": "pasaportar",
    "pasaportan": "pasaportar",
    "pasaportando": "pasaportar",
    "pasaportara": "pasaportar",
    "pasaportarais": "pasaportar",
    "pasaportaran": "pasaportar",
    "pasaportaras": "pasaportar",
    "pasaportare": "pasaportar",
    "pasaportareis": "pasaportar",
    "pasaportaremos": "pasaportar",
    "pasaportaren": "pasaportar",
    "pasaportares": "pasaportar",
    "pasaportaron": "pasaportar",
    "pasaportará": "pasaportar",
    "pasaportarán": "pasaportar",
    "pasaportarás": "pasaportar",
    "pasaportaré": "pasaportar",
    "pasaportaréis": "pasaportar",
    "pasaportaría": "pasaportar",
    "pasaportaríais": "pasaportar",
    "pasaportaríamos": "pasaportar",
    "pasaportarían": "pasaportar",
    "pasaportarías": "pasaportar",
    "pasaportas": "pasaportar",
    "pasaportase": "pasaportar",
    "pasaportaseis": "pasaportar",
    "pasaportasen": "pasaportar",
    "pasaportases": "pasaportar",
    "pasaportaste": "pasaportar",
    "pasaportasteis": "pasaportar",
    "pasaporte": "pasaportar",
    "pasaportemos": "pasaportar",
    "pasaporten": "pasaportar",
    "pasaportes": "pasaporte",
    "pasaporto": "pasaportar",
    "pasaportábamos": "pasaportar",
    "pasaportáis": "pasaportar",
    "pasaportáramos": "pasaportar",
    "pasaportáremos": "pasaportar",
    "pasaportásemos": "pasaportar",
    "pasaporté": "pasaportar",
    "pasaportéis": "pasaportar",
    "pasaportó": "pasaportar",
    "pasapureses": "pasapurés",
    "pasara": "pasar",
    "pasarais": "pasar",
    "pasaran": "pasar",
    "pasaras": "pasar",
    "pasare": "pasar",
    "pasareis": "pasar",
    "pasarelas": "pasarela",
    "pasaremos": "pasar",
    "pasaren": "pasar",
    "pasares": "pasar",
    "pasaron": "pasar",
    "pasará": "pasar",
    "pasarán": "pasar",
    "pasarás": "pasar",
    "pasaré": "pasar",
    "pasaréis": "pasar",
    "pasaría": "pasar",
    "pasaríais": "pasar",
    "pasaríamos": "pasar",
    "pasarían": "pasar",
    "pasarías": "pasar",
    "pasas": "paso",
    "pasase": "pasar",
    "pasaseis": "pasar",
    "pasasen": "pasar",
    "pasases": "pasar",
    "pasaste": "pasar",
    "pasasteis": "pasar",
    "pasatiempos": "pasatiempo",
    "pasavantes": "pasavante",
    "pasavolantes": "pasavolante",
    "pasavoleos": "pasavoleo",
    "pascales": "pascal",
    "pascals": "pascal",
    "pascuales": "pascual",
    "pascuas": "pascua",
    "pascuillas": "pascuilla",
    "pase": "pasar",
    "pasea": "pasear",
    "paseaba": "pasear",
    "paseabais": "pasear",
    "paseaban": "pasear",
    "paseabas": "pasear",
    "pasead": "pasear",
    "paseada": "pasear",
    "paseadas": "pasear",
    "paseado": "pasear",
    "paseadora": "paseador",
    "paseadoras": "paseador",
    "paseadores": "paseador",
    "paseados": "pasear",
    "paseamos": "pasear",
    "pasean": "pasear",
    "paseando": "pasear",
    "paseantes": "paseante",
    "paseara": "pasear",
    "pasearais": "pasear",
    "pasearan": "pasear",
    "pasearas": "pasear",
    "paseare": "pasear",
    "paseareis": "pasear",
    "pasearemos": "pasear",
    "pasearen": "pasear",
    "paseares": "pasear",
    "pasearon": "pasear",
    "paseará": "pasear",
    "pasearán": "pasear",
    "pasearás": "pasear",
    "pasearé": "pasear",
    "pasearéis": "pasear",
    "pasearía": "pasear",
    "pasearíais": "pasear",
    "pasearíamos": "pasear",
    "pasearían": "pasear",
    "pasearías": "pasear",
    "paseas": "pasear",
    "pasease": "pasear",
    "paseaseis": "pasear",
    "paseasen": "pasear",
    "paseases": "pasear",
    "paseaste": "pasear",
    "paseasteis": "pasear",
    "pasee": "pasear",
    "paseemos": "pasear",
    "paseen": "pasear",
    "pasees": "pasear",
    "pasemos": "pasar",
    "pasen": "pasar",
    "paseo": "pasear",
    "paseos": "paseo",
    "pasera": "pasero",
    "paseras": "pasero",
    "paseriformes": "paseriforme",
    "paseros": "pasero",
    "pases": "pase",
    "paseábamos": "pasear",
    "paseáis": "pasear",
    "paseáramos": "pasear",
    "paseáremos": "pasear",
    "paseásemos": "pasear",
    "paseé": "pasear",
    "paseéis": "pasear",
    "paseíllos": "paseíllo",
    "paseó": "pasear",
    "pashas": "pasha",
    "pashtues": "pashtu",
    "pashtus": "pashtu",
    "pasibilidades": "pasibilidad",
    "pasibles": "pasible",
    "pasicorta": "pasicorto",
    "pasicortas": "pasicorto",
    "pasicortos": "pasicorto",
    "pasifloras": "pasiflora",
    "pasiflorácea": "pasifloráceo",
    "pasifloráceas": "pasifloráceo",
    "pasifloráceos": "pasifloráceo",
    "pasilarga": "pasilargo",
    "pasilargas": "pasilargo",
    "pasilargos": "pasilargo",
    "pasiles": "pasil",
    "pasillos": "pasillo",
    "pasionales": "pasional",
    "pasionarias": "pasionaria",
    "pasionarios": "pasionario",
    "pasioncillas": "pasioncilla",
    "pasioneros": "pasionero",
    "pasiones": "pasión",
    "pasitos": "pasito",
    "pasitrotes": "pasitrote",
    "pasiva": "pasivo",
    "pasivaciones": "pasivación",
    "pasivas": "pasivo",
    "pasividades": "pasividad",
    "pasivos": "pasivo",
    "pasma": "pasmar",
    "pasmaba": "pasmar",
    "pasmabais": "pasmar",
    "pasmaban": "pasmar",
    "pasmabas": "pasmar",
    "pasmad": "pasmar",
    "pasmada": "pasmar",
    "pasmadas": "pasmar",
    "pasmado": "pasmar",
    "pasmados": "pasmar",
    "pasmamos": "pasmar",
    "pasman": "pasmar",
    "pasmando": "pasmar",
    "pasmara": "pasmar",
    "pasmarais": "pasmar",
    "pasmaran": "pasmar",
    "pasmaras": "pasmar",
    "pasmare": "pasmar",
    "pasmareis": "pasmar",
    "pasmaremos": "pasmar",
    "pasmaren": "pasmar",
    "pasmares": "pasmar",
    "pasmaron": "pasmar",
    "pasmarotas": "pasmarota",
    "pasmarotes": "pasmarote",
    "pasmará": "pasmar",
    "pasmarán": "pasmar",
    "pasmarás": "pasmar",
    "pasmaré": "pasmar",
    "pasmaréis": "pasmar",
    "pasmaría": "pasmar",
    "pasmaríais": "pasmar",
    "pasmaríamos": "pasmar",
    "pasmarían": "pasmar",
    "pasmarías": "pasmar",
    "pasmas": "pasmar",
    "pasmase": "pasmar",
    "pasmaseis": "pasmar",
    "pasmasen": "pasmar",
    "pasmases": "pasmar",
    "pasmaste": "pasmar",
    "pasmasteis": "pasmar",
    "pasme": "pasmar",
    "pasmemos": "pasmar",
    "pasmen": "pasmar",
    "pasmes": "pasmar",
    "pasmo": "pasmar",
    "pasmona": "pasmón",
    "pasmonas": "pasmón",
    "pasmones": "pasmón",
    "pasmos": "pasmo",
    "pasmosa": "pasmoso",
    "pasmosas": "pasmoso",
    "pasmosos": "pasmoso",
    "pasmábamos": "pasmar",
    "pasmáis": "pasmar",
    "pasmáramos": "pasmar",
    "pasmáremos": "pasmar",
    "pasmásemos": "pasmar",
    "pasmé": "pasmar",
    "pasméis": "pasmar",
    "pasmó": "pasmar",
    "paso": "pasar",
    "pasodobles": "pasodoble",
    "pasos": "paso",
    "pasotas": "pasota",
    "pasotismos": "pasotismo",
    "paspartúes": "paspartú",
    "paspartús": "paspartú",
    "paspiés": "paspié",
    "pasquinadas": "pasquinada",
    "pasquines": "pasquín",
    "pasquínes": "pasquín",
    "passwords": "password",
    "pasta": "pastar",
    "pastaba": "pastar",
    "pastabais": "pastar",
    "pastaban": "pastar",
    "pastabas": "pastar",
    "pastad": "pastar",
    "pastada": "pastar",
    "pastadas": "pastar",
    "pastaderos": "pastadero",
    "pastado": "pastar",
    "pastados": "pastar",
    "pastafloras": "pastaflora",
    "pastamos": "pastar",
    "pastan": "pastar",
    "pastando": "pastar",
    "pastara": "pastar",
    "pastarais": "pastar",
    "pastaran": "pastar",
    "pastaras": "pastar",
    "pastare": "pastar",
    "pastareis": "pastar",
    "pastaremos": "pastar",
    "pastaren": "pastar",
    "pastares": "pastar",
    "pastaron": "pastar",
    "pastará": "pastar",
    "pastarán": "pastar",
    "pastarás": "pastar",
    "pastaré": "pastar",
    "pastaréis": "pastar",
    "pastaría": "pastar",
    "pastaríais": "pastar",
    "pastaríamos": "pastar",
    "pastarían": "pastar",
    "pastarías": "pastar",
    "pastas": "pastar",
    "pastase": "pastar",
    "pastaseis": "pastar",
    "pastasen": "pastar",
    "pastases": "pastar",
    "pastaste": "pastar",
    "pastasteis": "pastar",
    "paste": "pastar",
    "pastecas": "pasteca",
    "pastelea": "pastelear",
    "pasteleaba": "pastelear",
    "pasteleabais": "pastelear",
    "pasteleaban": "pastelear",
    "pasteleabas": "pastelear",
    "pastelead": "pastelear",
    "pasteleada": "pastelear",
    "pasteleadas": "pastelear",
    "pasteleado": "pastelear",
    "pasteleados": "pastelear",
    "pasteleamos": "pastelear",
    "pastelean": "pastelear",
    "pasteleando": "pastelear",
    "pasteleara": "pastelear",
    "pastelearais": "pastelear",
    "pastelearan": "pastelear",
    "pastelearas": "pastelear",
    "pasteleare": "pastelear",
    "pasteleareis": "pastelear",
    "pastelearemos": "pastelear",
    "pastelearen": "pastelear",
    "pasteleares": "pastelear",
    "pastelearon": "pastelear",
    "pasteleará": "pastelear",
    "pastelearán": "pastelear",
    "pastelearás": "pastelear",
    "pastelearé": "pastelear",
    "pastelearéis": "pastelear",
    "pastelearía": "pastelear",
    "pastelearíais": "pastelear",
    "pastelearíamos": "pastelear",
    "pastelearían": "pastelear",
    "pastelearías": "pastelear",
    "pasteleas": "pastelear",
    "pastelease": "pastelear",
    "pasteleaseis": "pastelear",
    "pasteleasen": "pastelear",
    "pasteleases": "pastelear",
    "pasteleaste": "pastelear",
    "pasteleasteis": "pastelear",
    "pastelee": "pastelear",
    "pasteleemos": "pastelear",
    "pasteleen": "pastelear",
    "pastelees": "pastelear",
    "pasteleo": "pastelear",
    "pasteleos": "pasteleo",
    "pastelera": "pastelero",
    "pasteleras": "pastelero",
    "pasteleros": "pastelero",
    "pastelerías": "pastelería",
    "pasteles": "pastel",
    "pasteleábamos": "pastelear",
    "pasteleáis": "pastelear",
    "pasteleáramos": "pastelear",
    "pasteleáremos": "pastelear",
    "pasteleásemos": "pastelear",
    "pasteleé": "pastelear",
    "pasteleéis": "pastelear",
    "pasteleó": "pastelear",
    "pastelillos": "pastelillo",
    "pastelistas": "pastelista",
    "pastemos": "pastar",
    "pasten": "pastar",
    "pastenca": "pastenco",
    "pastencas": "pastenco",
    "pastencos": "pastenco",
    "pasterice": "pasterizar",
    "pastericemos": "pasterizar",
    "pastericen": "pasterizar",
    "pasterices": "pasterizar",
    "pastericé": "pasterizar",
    "pastericéis": "pasterizar",
    "pasteriza": "pasterizar",
    "pasterizaba": "pasterizar",
    "pasterizabais": "pasterizar",
    "pasterizaban": "pasterizar",
    "pasterizabas": "pasterizar",
    "pasterizaciones": "pasterización",
    "pasterizad": "pasterizar",
    "pasterizada": "pasterizar",
    "pasterizadas": "pasterizar",
    "pasterizado": "pasterizar",
    "pasterizados": "pasterizar",
    "pasterizamos": "pasterizar",
    "pasterizan": "pasterizar",
    "pasterizando": "pasterizar",
    "pasterizara": "pasterizar",
    "pasterizarais": "pasterizar",
    "pasterizaran": "pasterizar",
    "pasterizaras": "pasterizar",
    "pasterizare": "pasterizar",
    "pasterizareis": "pasterizar",
    "pasterizaremos": "pasterizar",
    "pasterizaren": "pasterizar",
    "pasterizares": "pasterizar",
    "pasterizaron": "pasterizar",
    "pasterizará": "pasterizar",
    "pasterizarán": "pasterizar",
    "pasterizarás": "pasterizar",
    "pasterizaré": "pasterizar",
    "pasterizaréis": "pasterizar",
    "pasterizaría": "pasterizar",
    "pasterizaríais": "pasterizar",
    "pasterizaríamos": "pasterizar",
    "pasterizarían": "pasterizar",
    "pasterizarías": "pasterizar",
    "pasterizas": "pasterizar",
    "pasterizase": "pasterizar",
    "pasterizaseis": "pasterizar",
    "pasterizasen": "pasterizar",
    "pasterizases": "pasterizar",
    "pasterizaste": "pasterizar",
    "pasterizasteis": "pasterizar",
    "pasterizo": "pasterizar",
    "pasterizábamos": "pasterizar",
    "pasterizáis": "pasterizar",
    "pasterizáramos": "pasterizar",
    "pasterizáremos": "pasterizar",
    "pasterizásemos": "pasterizar",
    "pasterizó": "pasterizar",
    "pastes": "pastar",
    "pasteurice": "pasteurizar",
    "pasteuricemos": "pasteurizar",
    "pasteuricen": "pasteurizar",
    "pasteurices": "pasteurizar",
    "pasteuricé": "pasteurizar",
    "pasteuricéis": "pasteurizar",
    "pasteuriza": "pasteurizar",
    "pasteurizaba": "pasteurizar",
    "pasteurizabais": "pasteurizar",
    "pasteurizaban": "pasteurizar",
    "pasteurizabas": "pasteurizar",
    "pasteurizaciones": "pasteurización",
    "pasteurizad": "pasteurizar",
    "pasteurizada": "pasteurizar",
    "pasteurizadas": "pasteurizar",
    "pasteurizado": "pasteurizar",
    "pasteurizados": "pasteurizar",
    "pasteurizamos": "pasteurizar",
    "pasteurizan": "pasteurizar",
    "pasteurizando": "pasteurizar",
    "pasteurizara": "pasteurizar",
    "pasteurizarais": "pasteurizar",
    "pasteurizaran": "pasteurizar",
    "pasteurizaras": "pasteurizar",
    "pasteurizare": "pasteurizar",
    "pasteurizareis": "pasteurizar",
    "pasteurizaremos": "pasteurizar",
    "pasteurizaren": "pasteurizar",
    "pasteurizares": "pasteurizar",
    "pasteurizaron": "pasteurizar",
    "pasteurizará": "pasteurizar",
    "pasteurizarán": "pasteurizar",
    "pasteurizarás": "pasteurizar",
    "pasteurizaré": "pasteurizar",
    "pasteurizaréis": "pasteurizar",
    "pasteurizaría": "pasteurizar",
    "pasteurizaríais": "pasteurizar",
    "pasteurizaríamos": "pasteurizar",
    "pasteurizarían": "pasteurizar",
    "pasteurizarías": "pasteurizar",
    "pasteurizas": "pasteurizar",
    "pasteurizase": "pasteurizar",
    "pasteurizaseis": "pasteurizar",
    "pasteurizasen": "pasteurizar",
    "pasteurizases": "pasteurizar",
    "pasteurizaste": "pasteurizar",
    "pasteurizasteis": "pasteurizar",
    "pasteurizo": "pasteurizar",
    "pasteurizábamos": "pasteurizar",
    "pasteurizáis": "pasteurizar",
    "pasteurizáramos": "pasteurizar",
    "pasteurizáremos": "pasteurizar",
    "pasteurizásemos": "pasteurizar",
    "pasteurizó": "pasteurizar",
    "pastiches": "pastiche",
    "pasticulturas": "pasticultura",
    "pastilladoras": "pastilladora",
    "pastillados": "pastillado",
    "pastillas": "pastilla",
    "pastinacas": "pastinaca",
    "pastizales": "pastizal",
    "pasto": "pastar",
    "pastoforios": "pastoforio",
    "pastora": "pastor",
    "pastorales": "pastoral",
    "pastoralismos": "pastoralismo",
    "pastoras": "pastor",
    "pastorea": "pastorear",
    "pastoreaba": "pastorear",
    "pastoreabais": "pastorear",
    "pastoreaban": "pastorear",
    "pastoreabas": "pastorear",
    "pastoread": "pastorear",
    "pastoreada": "pastorear",
    "pastoreadas": "pastorear",
    "pastoreado": "pastorear",
    "pastoreados": "pastorear",
    "pastoreamos": "pastorear",
    "pastorean": "pastorear",
    "pastoreando": "pastorear",
    "pastoreara": "pastorear",
    "pastorearais": "pastorear",
    "pastorearan": "pastorear",
    "pastorearas": "pastorear",
    "pastoreare": "pastorear",
    "pastoreareis": "pastorear",
    "pastorearemos": "pastorear",
    "pastorearen": "pastorear",
    "pastoreares": "pastorear",
    "pastorearon": "pastorear",
    "pastoreará": "pastorear",
    "pastorearán": "pastorear",
    "pastorearás": "pastorear",
    "pastorearé": "pastorear",
    "pastorearéis": "pastorear",
    "pastorearía": "pastorear",
    "pastorearíais": "pastorear",
    "pastorearíamos": "pastorear",
    "pastorearían": "pastorear",
    "pastorearías": "pastorear",
    "pastoreas": "pastorear",
    "pastorease": "pastorear",
    "pastoreaseis": "pastorear",
    "pastoreasen": "pastorear",
    "pastoreases": "pastorear",
    "pastoreaste": "pastorear",
    "pastoreasteis": "pastorear",
    "pastoree": "pastorear",
    "pastoreemos": "pastorear",
    "pastoreen": "pastorear",
    "pastorees": "pastorear",
    "pastorelas": "pastorela",
    "pastoreo": "pastorear",
    "pastoreos": "pastoreo",
    "pastores": "pastor",
    "pastoreábamos": "pastorear",
    "pastoreáis": "pastorear",
    "pastoreáramos": "pastorear",
    "pastoreáremos": "pastorear",
    "pastoreásemos": "pastorear",
    "pastoreé": "pastorear",
    "pastoreéis": "pastorear",
    "pastoreó": "pastorear",
    "pastoricia": "pastoricio",
    "pastoricias": "pastoricio",
    "pastoricios": "pastoricio",
    "pastoriles": "pastoril",
    "pastorías": "pastoría",
    "pastos": "pasto",
    "pastosa": "pastoso",
    "pastosas": "pastoso",
    "pastosidades": "pastosidad",
    "pastosos": "pastoso",
    "pastrana": "pastrano",
    "pastranas": "pastrano",
    "pastranos": "pastrano",
    "pastrijas": "pastrija",
    "pastueña": "pastueño",
    "pastueñas": "pastueño",
    "pastueños": "pastueño",
    "pasturajes": "pasturaje",
    "pasturas": "pastura",
    "pastábamos": "pastar",
    "pastáis": "pastar",
    "pastáramos": "pastar",
    "pastáremos": "pastar",
    "pastásemos": "pastar",
    "pasté": "pastar",
    "pastéis": "pastar",
    "pastó": "pastar",
    "pasábamos": "pasar",
    "pasáis": "pasar",
    "pasáramos": "pasar",
    "pasáremos": "pasar",
    "pasásemos": "pasar",
    "pasé": "pasar",
    "paséis": "pasar",
    "pasó": "pasar",
    "pata": "pato",
    "pataca": "pataco",
    "patacas": "pataco",
    "pataches": "patache",
    "patacones": "patacón",
    "patacos": "pataco",
    "patadas": "patada",
    "patadiones": "patadión",
    "patagios": "patagio",
    "patagona": "patagón",
    "patagonas": "patagón",
    "patagones": "patagón",
    "patagorrillas": "patagorrilla",
    "patagorrillos": "patagorrillo",
    "patagónica": "patagónico",
    "patagónicas": "patagónico",
    "patagónicos": "patagónico",
    "patajes": "pataje",
    "patajúes": "patajú",
    "patajús": "patajú",
    "patalea": "patalear",
    "pataleaba": "patalear",
    "pataleabais": "patalear",
    "pataleaban": "patalear",
    "pataleabas": "patalear",
    "patalead": "patalear",
    "pataleada": "patalear",
    "pataleadas": "patalear",
    "pataleado": "patalear",
    "pataleados": "patalear",
    "pataleamos": "patalear",
    "patalean": "patalear",
    "pataleando": "patalear",
    "pataleara": "patalear",
    "patalearais": "patalear",
    "patalearan": "patalear",
    "patalearas": "patalear",
    "pataleare": "patalear",
    "pataleareis": "patalear",
    "patalearemos": "patalear",
    "patalearen": "patalear",
    "pataleares": "patalear",
    "patalearon": "patalear",
    "pataleará": "patalear",
    "patalearán": "patalear",
    "patalearás": "patalear",
    "patalearé": "patalear",
    "patalearéis": "patalear",
    "patalearía": "patalear",
    "patalearíais": "patalear",
    "patalearíamos": "patalear",
    "patalearían": "patalear",
    "patalearías": "patalear",
    "pataleas": "patalear",
    "patalease": "patalear",
    "pataleaseis": "patalear",
    "pataleasen": "patalear",
    "pataleases": "patalear",
    "pataleaste": "patalear",
    "pataleasteis": "patalear",
    "patalee": "patalear",
    "pataleemos": "patalear",
    "pataleen": "patalear",
    "patalees": "patalear",
    "pataleo": "patalear",
    "pataleos": "pataleo",
    "pataletas": "pataleta",
    "pataletea": "pataletear",
    "pataleteaba": "pataletear",
    "pataleteabais": "pataletear",
    "pataleteaban": "pataletear",
    "pataleteabas": "pataletear",
    "pataletead": "pataletear",
    "pataleteada": "pataletear",
    "pataleteadas": "pataletear",
    "pataleteado": "pataletear",
    "pataleteados": "pataletear",
    "pataleteamos": "pataletear",
    "pataletean": "pataletear",
    "pataleteando": "pataletear",
    "pataleteara": "pataletear",
    "pataletearais": "pataletear",
    "pataletearan": "pataletear",
    "pataletearas": "pataletear",
    "pataleteare": "pataletear",
    "pataleteareis": "pataletear",
    "pataletearemos": "pataletear",
    "pataletearen": "pataletear",
    "pataleteares": "pataletear",
    "pataletearon": "pataletear",
    "pataleteará": "pataletear",
    "pataletearán": "pataletear",
    "pataletearás": "pataletear",
    "pataletearé": "pataletear",
    "pataletearéis": "pataletear",
    "pataletearía": "pataletear",
    "pataletearíais": "pataletear",
    "pataletearíamos": "pataletear",
    "pataletearían": "pataletear",
    "pataletearías": "pataletear",
    "pataleteas": "pataletear",
    "pataletease": "pataletear",
    "pataleteaseis": "pataletear",
    "pataleteasen": "pataletear",
    "pataleteases": "pataletear",
    "pataleteaste": "pataletear",
    "pataleteasteis": "pataletear",
    "pataletee": "pataletear",
    "pataleteemos": "pataletear",
    "pataleteen": "pataletear",
    "pataletees": "pataletear",
    "pataleteo": "pataletear",
    "pataleteábamos": "pataletear",
    "pataleteáis": "pataletear",
    "pataleteáramos": "pataletear",
    "pataleteáremos": "pataletear",
    "pataleteásemos": "pataletear",
    "pataleteé": "pataletear",
    "pataleteéis": "pataletear",
    "pataleteó": "pataletear",
    "pataleábamos": "patalear",
    "pataleáis": "patalear",
    "pataleáramos": "patalear",
    "pataleáremos": "patalear",
    "pataleásemos": "patalear",
    "pataleé": "patalear",
    "pataleéis": "patalear",
    "pataleó": "patalear",
    "patanerías": "patanería",
    "patanes": "patán",
    "pataratas": "patarata",
    "pataratera": "pataratero",
    "patarateras": "pataratero",
    "patarateros": "pataratero",
    "patarráeces": "patarráez",
    "patas": "pato",
    "patatales": "patatal",
    "patatares": "patatar",
    "patatas": "patata",
    "patatera": "patatero",
    "patateras": "patatero",
    "patateros": "patatero",
    "patavina": "patavino",
    "patavinas": "patavino",
    "patavinos": "patavino",
    "patchworks": "patchwork",
    "patea": "patear",
    "pateaba": "patear",
    "pateabais": "patear",
    "pateaban": "patear",
    "pateabas": "patear",
    "patead": "patear",
    "pateada": "patear",
    "pateadas": "patear",
    "pateado": "patear",
    "pateadora": "pateador",
    "pateadoras": "pateador",
    "pateadores": "pateador",
    "pateados": "patear",
    "pateaduras": "pateadura",
    "pateamientos": "pateamiento",
    "pateamos": "patear",
    "patean": "patear",
    "pateando": "patear",
    "pateara": "patear",
    "patearais": "patear",
    "patearan": "patear",
    "patearas": "patear",
    "pateare": "patear",
    "pateareis": "patear",
    "patearemos": "patear",
    "patearen": "patear",
    "pateares": "patear",
    "patearon": "patear",
    "pateará": "patear",
    "patearán": "patear",
    "patearás": "patear",
    "patearé": "patear",
    "patearéis": "patear",
    "patearía": "patear",
    "patearíais": "patear",
    "patearíamos": "patear",
    "patearían": "patear",
    "patearías": "patear",
    "pateas": "patear",
    "patease": "patear",
    "pateaseis": "patear",
    "pateasen": "patear",
    "pateases": "patear",
    "pateaste": "patear",
    "pateasteis": "patear",
    "patee": "patear",
    "pateemos": "patear",
    "pateen": "patear",
    "patees": "patear",
    "patelares": "patelar",
    "patelariales": "patelarial",
    "patelas": "patela",
    "patenas": "patena",
    "patenta": "patentar",
    "patentaba": "patentar",
    "patentabais": "patentar",
    "patentaban": "patentar",
    "patentabas": "patentar",
    "patentabilidades": "patentabilidad",
    "patentables": "patentable",
    "patentad": "patentar",
    "patentada": "patentar",
    "patentadas": "patentar",
    "patentado": "patentar",
    "patentados": "patentar",
    "patentamos": "patentar",
    "patentan": "patentar",
    "patentando": "patentar",
    "patentara": "patentar",
    "patentarais": "patentar",
    "patentaran": "patentar",
    "patentaras": "patentar",
    "patentare": "patentar",
    "patentareis": "patentar",
    "patentaremos": "patentar",
    "patentaren": "patentar",
    "patentares": "patentar",
    "patentaron": "patentar",
    "patentará": "patentar",
    "patentarán": "patentar",
    "patentarás": "patentar",
    "patentaré": "patentar",
    "patentaréis": "patentar",
    "patentaría": "patentar",
    "patentaríais": "patentar",
    "patentaríamos": "patentar",
    "patentarían": "patentar",
    "patentarías": "patentar",
    "patentas": "patentar",
    "patentase": "patentar",
    "patentaseis": "patentar",
    "patentasen": "patentar",
    "patentases": "patentar",
    "patentaste": "patentar",
    "patentasteis": "patentar",
    "patente": "patentar",
    "patentemos": "patentar",
    "patenten": "patentar",
    "patentes": "patente",
    "patentice": "patentizar",
    "patenticemos": "patentizar",
    "patenticen": "patentizar",
    "patentices": "patentizar",
    "patenticé": "patentizar",
    "patenticéis": "patentizar",
    "patentiza": "patentizar",
    "patentizaba": "patentizar",
    "patentizabais": "patentizar",
    "patentizaban": "patentizar",
    "patentizabas": "patentizar",
    "patentizad": "patentizar",
    "patentizada": "patentizar",
    "patentizadas": "patentizar",
    "patentizado": "patentizar",
    "patentizados": "patentizar",
    "patentizamos": "patentizar",
    "patentizan": "patentizar",
    "patentizando": "patentizar",
    "patentizara": "patentizar",
    "patentizarais": "patentizar",
    "patentizaran": "patentizar",
    "patentizaras": "patentizar",
    "patentizare": "patentizar",
    "patentizareis": "patentizar",
    "patentizaremos": "patentizar",
    "patentizaren": "patentizar",
    "patentizares": "patentizar",
    "patentizaron": "patentizar",
    "patentizará": "patentizar",
    "patentizarán": "patentizar",
    "patentizarás": "patentizar",
    "patentizaré": "patentizar",
    "patentizaréis": "patentizar",
    "patentizaría": "patentizar",
    "patentizaríais": "patentizar",
    "patentizaríamos": "patentizar",
    "patentizarían": "patentizar",
    "patentizarías": "patentizar",
    "patentizas": "patentizar",
    "patentizase": "patentizar",
    "patentizaseis": "patentizar",
    "patentizasen": "patentizar",
    "patentizases": "patentizar",
    "patentizaste": "patentizar",
    "patentizasteis": "patentizar",
    "patentizo": "patentizar",
    "patentizábamos": "patentizar",
    "patentizáis": "patentizar",
    "patentizáramos": "patentizar",
    "patentizáremos": "patentizar",
    "patentizásemos": "patentizar",
    "patentizó": "patentizar",
    "patento": "patentar",
    "patentábamos": "patentar",
    "patentáis": "patentar",
    "patentáramos": "patentar",
    "patentáremos": "patentar",
    "patentásemos": "patentar",
    "patenté": "patentar",
    "patentéis": "patentar",
    "patentó": "patentar",
    "pateo": "patear",
    "pateos": "pateo",
    "pateras": "patera",
    "paterna": "paterno",
    "paternales": "paternal",
    "paternalismos": "paternalismo",
    "paternalistas": "paternalista",
    "paternas": "paterno",
    "paternidades": "paternidad",
    "paternos": "paterno",
    "paters": "páter",
    "patetas": "pateta",
    "patetismos": "patetismo",
    "pateábamos": "patear",
    "pateáis": "patear",
    "pateáramos": "patear",
    "pateáremos": "patear",
    "pateásemos": "patear",
    "pateé": "patear",
    "pateéis": "patear",
    "pateó": "patear",
    "patiabierta": "patiabierto",
    "patiabiertas": "patiabierto",
    "patiabiertos": "patiabierto",
    "patialba": "patialbo",
    "patialbas": "patialbo",
    "patialbos": "patialbo",
    "patiblanca": "patiblanco",
    "patiblancas": "patiblanco",
    "patiblancos": "patiblanco",
    "patibularia": "patibulario",
    "patibularias": "patibulario",
    "patibularios": "patibulario",
    "patiburrillos": "patiburrillo",
    "paticoja": "paticojo",
    "paticojas": "paticojo",
    "paticojos": "paticojo",
    "patidifusa": "patidifuso",
    "patidifusas": "patidifuso",
    "patidifusos": "patidifuso",
    "patiestevada": "patiestevado",
    "patiestevadas": "patiestevado",
    "patiestevados": "patiestevado",
    "patihendida": "patihendido",
    "patihendidas": "patihendido",
    "patihendidos": "patihendido",
    "patillas": "patilla",
    "patilluda": "patilludo",
    "patilludas": "patilludo",
    "patilludos": "patilludo",
    "patimuleña": "patimuleño",
    "patimuleñas": "patimuleño",
    "patimuleños": "patimuleño",
    "patina": "patinar",
    "patinaba": "patinar",
    "patinabais": "patinar",
    "patinaban": "patinar",
    "patinabas": "patinar",
    "patinad": "patinar",
    "patinada": "patinar",
    "patinadas": "patinar",
    "patinaderos": "patinadero",
    "patinado": "patinar",
    "patinadora": "patinador",
    "patinadoras": "patinador",
    "patinadores": "patinador",
    "patinados": "patinar",
    "patinajes": "patinaje",
    "patinamos": "patinar",
    "patinan": "patinar",
    "patinando": "patinar",
    "patinara": "patinar",
    "patinarais": "patinar",
    "patinaran": "patinar",
    "patinaras": "patinar",
    "patinare": "patinar",
    "patinareis": "patinar",
    "patinaremos": "patinar",
    "patinaren": "patinar",
    "patinares": "patinar",
    "patinaron": "patinar",
    "patinará": "patinar",
    "patinarán": "patinar",
    "patinarás": "patinar",
    "patinaré": "patinar",
    "patinaréis": "patinar",
    "patinaría": "patinar",
    "patinaríais": "patinar",
    "patinaríamos": "patinar",
    "patinarían": "patinar",
    "patinarías": "patinar",
    "patinas": "patinar",
    "patinase": "patinar",
    "patinaseis": "patinar",
    "patinasen": "patinar",
    "patinases": "patinar",
    "patinaste": "patinar",
    "patinasteis": "patinar",
    "patinazos": "patinazo",
    "patine": "patinar",
    "patinemos": "patinar",
    "patinen": "patinar",
    "patines": "patinar",
    "patinetas": "patineta",
    "patinetes": "patinete",
    "patino": "patinar",
    "patinábamos": "patinar",
    "patináis": "patinar",
    "patináramos": "patinar",
    "patináremos": "patinar",
    "patinásemos": "patinar",
    "patiné": "patinar",
    "patinéis": "patinar",
    "patinó": "patinar",
    "patios": "patio",
    "patiquebraba": "patiquebrar",
    "patiquebrabais": "patiquebrar",
    "patiquebraban": "patiquebrar",
    "patiquebrabas": "patiquebrar",
    "patiquebrad": "patiquebrar",
    "patiquebrada": "patiquebrar",
    "patiquebradas": "patiquebrar",
    "patiquebrado": "patiquebrar",
    "patiquebrados": "patiquebrar",
    "patiquebramos": "patiquebrar",
    "patiquebrando": "patiquebrar",
    "patiquebrara": "patiquebrar",
    "patiquebrarais": "patiquebrar",
    "patiquebraran": "patiquebrar",
    "patiquebraras": "patiquebrar",
    "patiquebrare": "patiquebrar",
    "patiquebrareis": "patiquebrar",
    "patiquebraremos": "patiquebrar",
    "patiquebraren": "patiquebrar",
    "patiquebrares": "patiquebrar",
    "patiquebraron": "patiquebrar",
    "patiquebrará": "patiquebrar",
    "patiquebrarán": "patiquebrar",
    "patiquebrarás": "patiquebrar",
    "patiquebraré": "patiquebrar",
    "patiquebraréis": "patiquebrar",
    "patiquebraría": "patiquebrar",
    "patiquebraríais": "patiquebrar",
    "patiquebraríamos": "patiquebrar",
    "patiquebrarían": "patiquebrar",
    "patiquebrarías": "patiquebrar",
    "patiquebrase": "patiquebrar",
    "patiquebraseis": "patiquebrar",
    "patiquebrasen": "patiquebrar",
    "patiquebrases": "patiquebrar",
    "patiquebraste": "patiquebrar",
    "patiquebrasteis": "patiquebrar",
    "patiquebremos": "patiquebrar",
    "patiquebrábamos": "patiquebrar",
    "patiquebráis": "patiquebrar",
    "patiquebráramos": "patiquebrar",
    "patiquebráremos": "patiquebrar",
    "patiquebrásemos": "patiquebrar",
    "patiquebré": "patiquebrar",
    "patiquebréis": "patiquebrar",
    "patiquebró": "patiquebrar",
    "patiquiebra": "patiquebrar",
    "patiquiebran": "patiquebrar",
    "patiquiebras": "patiquebrar",
    "patiquiebre": "patiquebrar",
    "patiquiebren": "patiquebrar",
    "patiquiebres": "patiquebrar",
    "patiquiebro": "patiquebrar",
    "patitiesa": "patitieso",
    "patitiesas": "patitieso",
    "patitiesos": "patitieso",
    "patituerta": "patituerto",
    "patituertas": "patituerto",
    "patituertos": "patituerto",
    "patizamba": "patizambo",
    "patizambas": "patizambo",
    "patizambos": "patizambo",
    "patochadas": "patochada",
    "patofobias": "patofobia",
    "patogenesias": "patogenesia",
    "patogenias": "patogenia",
    "patognomónica": "patognomónico",
    "patognomónicas": "patognomónico",
    "patognomónicos": "patognomónico",
    "patografías": "patografía",
    "patogénica": "patogénico",
    "patogénicas": "patogénico",
    "patogénicos": "patogénico",
    "patoja": "patojo",
    "patojas": "patojo",
    "patojeras": "patojera",
    "patojos": "patojo",
    "patologías": "patología",
    "patológica": "patológico",
    "patológicas": "patológico",
    "patológicos": "patológico",
    "patona": "patón",
    "patonas": "patón",
    "patones": "patón",
    "patos": "pato",
    "patosa": "patoso",
    "patosas": "patoso",
    "patosos": "patoso",
    "patrañas": "patraña",
    "patrañera": "patrañero",
    "patrañeras": "patrañero",
    "patrañeros": "patrañero",
    "patria": "patrio",
    "patriarcados": "patriarcado",
    "patriarcales": "patriarcal",
    "patriarcas": "patriarca",
    "patrias": "patrio",
    "patricia": "patricio",
    "patriciados": "patriciado",
    "patricias": "patricio",
    "patricios": "patricio",
    "patriclanes": "patriclán",
    "patrilaterales": "patrilateral",
    "patrilinajes": "patrilinaje",
    "patrilineales": "patrilineal",
    "patrilinealidades": "patrilinealidad",
    "patrilocales": "patrilocal",
    "patrilocalidades": "patrilocalidad",
    "patrimoniales": "patrimonial",
    "patrimonialidades": "patrimonialidad",
    "patrimonialistas": "patrimonialista",
    "patrimonios": "patrimonio",
    "patrios": "patrio",
    "patriotas": "patriota",
    "patriotera": "patriotero",
    "patrioteras": "patriotero",
    "patrioterismos": "patrioterismo",
    "patrioteros": "patriotero",
    "patrioterías": "patriotería",
    "patriotismos": "patriotismo",
    "patriótica": "patriótico",
    "patrióticas": "patriótico",
    "patrióticos": "patriótico",
    "patrocina": "patrocinar",
    "patrocinaba": "patrocinar",
    "patrocinabais": "patrocinar",
    "patrocinaban": "patrocinar",
    "patrocinabas": "patrocinar",
    "patrocinad": "patrocinar",
    "patrocinada": "patrocinar",
    "patrocinadas": "patrocinar",
    "patrocinado": "patrocinar",
    "patrocinadora": "patrocinador",
    "patrocinadoras": "patrocinador",
    "patrocinadores": "patrocinador",
    "patrocinados": "patrocinar",
    "patrocinamos": "patrocinar",
    "patrocinan": "patrocinar",
    "patrocinando": "patrocinar",
    "patrocinara": "patrocinar",
    "patrocinarais": "patrocinar",
    "patrocinaran": "patrocinar",
    "patrocinaras": "patrocinar",
    "patrocinare": "patrocinar",
    "patrocinareis": "patrocinar",
    "patrocinaremos": "patrocinar",
    "patrocinaren": "patrocinar",
    "patrocinares": "patrocinar",
    "patrocinaron": "patrocinar",
    "patrocinará": "patrocinar",
    "patrocinarán": "patrocinar",
    "patrocinarás": "patrocinar",
    "patrocinaré": "patrocinar",
    "patrocinaréis": "patrocinar",
    "patrocinaría": "patrocinar",
    "patrocinaríais": "patrocinar",
    "patrocinaríamos": "patrocinar",
    "patrocinarían": "patrocinar",
    "patrocinarías": "patrocinar",
    "patrocinas": "patrocinar",
    "patrocinase": "patrocinar",
    "patrocinaseis": "patrocinar",
    "patrocinasen": "patrocinar",
    "patrocinases": "patrocinar",
    "patrocinaste": "patrocinar",
    "patrocinasteis": "patrocinar",
    "patrocine": "patrocinar",
    "patrocinemos": "patrocinar",
    "patrocinen": "patrocinar",
    "patrocines": "patrocinar",
    "patrocinios": "patrocinio",
    "patrocino": "patrocinar",
    "patrocinábamos": "patrocinar",
    "patrocináis": "patrocinar",
    "patrocináramos": "patrocinar",
    "patrocináremos": "patrocinar",
    "patrocinásemos": "patrocinar",
    "patrociné": "patrocinar",
    "patrocinéis": "patrocinar",
    "patrocinó": "patrocinar",
    "patrologías": "patrología",
    "patrona": "patrón",
    "patronales": "patronal",
    "patronas": "patrón",
    "patronatos": "patronato",
    "patronazgos": "patronazgo",
    "patronea": "patronear",
    "patroneaba": "patronear",
    "patroneabais": "patronear",
    "patroneaban": "patronear",
    "patroneabas": "patronear",
    "patronead": "patronear",
    "patroneada": "patronear",
    "patroneadas": "patronear",
    "patroneado": "patronear",
    "patroneados": "patronear",
    "patroneamos": "patronear",
    "patronean": "patronear",
    "patroneando": "patronear",
    "patroneara": "patronear",
    "patronearais": "patronear",
    "patronearan": "patronear",
    "patronearas": "patronear",
    "patroneare": "patronear",
    "patroneareis": "patronear",
    "patronearemos": "patronear",
    "patronearen": "patronear",
    "patroneares": "patronear",
    "patronearon": "patronear",
    "patroneará": "patronear",
    "patronearán": "patronear",
    "patronearás": "patronear",
    "patronearé": "patronear",
    "patronearéis": "patronear",
    "patronearía": "patronear",
    "patronearíais": "patronear",
    "patronearíamos": "patronear",
    "patronearían": "patronear",
    "patronearías": "patronear",
    "patroneas": "patronear",
    "patronease": "patronear",
    "patroneaseis": "patronear",
    "patroneasen": "patronear",
    "patroneases": "patronear",
    "patroneaste": "patronear",
    "patroneasteis": "patronear",
    "patronee": "patronear",
    "patroneemos": "patronear",
    "patroneen": "patronear",
    "patronees": "patronear",
    "patroneo": "patronear",
    "patronera": "patronero",
    "patroneras": "patronero",
    "patroneros": "patronero",
    "patrones": "patrón",
    "patroneábamos": "patronear",
    "patroneáis": "patronear",
    "patroneáramos": "patronear",
    "patroneáremos": "patronear",
    "patroneásemos": "patronear",
    "patroneé": "patronear",
    "patroneéis": "patronear",
    "patroneó": "patronear",
    "patroniles": "patronil",
    "patronos": "patrono",
    "patronímica": "patronímico",
    "patronímicas": "patronímico",
    "patronímicos": "patronímico",
    "patrulla": "patrullar",
    "patrullaba": "patrullar",
    "patrullabais": "patrullar",
    "patrullaban": "patrullar",
    "patrullabas": "patrullar",
    "patrullad": "patrullar",
    "patrullada": "patrullar",
    "patrulladas": "patrullar",
    "patrullado": "patrullar",
    "patrullados": "patrullar",
    "patrullajes": "patrullaje",
    "patrullamos": "patrullar",
    "patrullan": "patrullar",
    "patrullando": "patrullar",
    "patrullara": "patrullar",
    "patrullarais": "patrullar",
    "patrullaran": "patrullar",
    "patrullaras": "patrullar",
    "patrullare": "patrullar",
    "patrullareis": "patrullar",
    "patrullaremos": "patrullar",
    "patrullaren": "patrullar",
    "patrullares": "patrullar",
    "patrullaron": "patrullar",
    "patrullará": "patrullar",
    "patrullarán": "patrullar",
    "patrullarás": "patrullar",
    "patrullaré": "patrullar",
    "patrullaréis": "patrullar",
    "patrullaría": "patrullar",
    "patrullaríais": "patrullar",
    "patrullaríamos": "patrullar",
    "patrullarían": "patrullar",
    "patrullarías": "patrullar",
    "patrullas": "patrullar",
    "patrullase": "patrullar",
    "patrullaseis": "patrullar",
    "patrullasen": "patrullar",
    "patrullases": "patrullar",
    "patrullaste": "patrullar",
    "patrullasteis": "patrullar",
    "patrulle": "patrullar",
    "patrullemos": "patrullar",
    "patrullen": "patrullar",
    "patrullera": "patrullero",
    "patrulleras": "patrullero",
    "patrulleros": "patrullero",
    "patrulles": "patrullar",
    "patrullo": "patrullar",
    "patrullábamos": "patrullar",
    "patrulláis": "patrullar",
    "patrulláramos": "patrullar",
    "patrulláremos": "patrullar",
    "patrullásemos": "patrullar",
    "patrullé": "patrullar",
    "patrulléis": "patrullar",
    "patrulló": "patrullar",
    "patrística": "patrístico",
    "patrísticas": "patrístico",
    "patrísticos": "patrístico",
    "patucos": "patuco",
    "patuda": "patudo",
    "patudas": "patudo",
    "patudos": "patudo",
    "patuleas": "patulea",
    "patulinas": "patulina",
    "patulla": "patullar",
    "patullaba": "patullar",
    "patullabais": "patullar",
    "patullaban": "patullar",
    "patullabas": "patullar",
    "patullad": "patullar",
    "patullada": "patullar",
    "patulladas": "patullar",
    "patullado": "patullar",
    "patullados": "patullar",
    "patullamos": "patullar",
    "patullan": "patullar",
    "patullando": "patullar",
    "patullara": "patullar",
    "patullarais": "patullar",
    "patullaran": "patullar",
    "patullaras": "patullar",
    "patullare": "patullar",
    "patullareis": "patullar",
    "patullaremos": "patullar",
    "patullaren": "patullar",
    "patullares": "patullar",
    "patullaron": "patullar",
    "patullará": "patullar",
    "patullarán": "patullar",
    "patullarás": "patullar",
    "patullaré": "patullar",
    "patullaréis": "patullar",
    "patullaría": "patullar",
    "patullaríais": "patullar",
    "patullaríamos": "patullar",
    "patullarían": "patullar",
    "patullarías": "patullar",
    "patullas": "patullar",
    "patullase": "patullar",
    "patullaseis": "patullar",
    "patullasen": "patullar",
    "patullases": "patullar",
    "patullaste": "patullar",
    "patullasteis": "patullar",
    "patulle": "patullar",
    "patullemos": "patullar",
    "patullen": "patullar",
    "patulles": "patullar",
    "patullo": "patullar",
    "patullábamos": "patullar",
    "patulláis": "patullar",
    "patulláramos": "patullar",
    "patulláremos": "patullar",
    "patullásemos": "patullar",
    "patullé": "patullar",
    "patulléis": "patullar",
    "patulló": "patullar",
    "patés": "paté",
    "patética": "patético",
    "patéticas": "patético",
    "patéticos": "patético",
    "patíbulos": "patíbulo",
    "patógena": "patógeno",
    "patógenas": "patógeno",
    "patógenos": "patógeno",
    "patóloga": "patólogo",
    "patólogas": "patólogo",
    "patólogos": "patólogo",
    "paula": "paular",
    "paulaba": "paular",
    "paulabais": "paular",
    "paulaban": "paular",
    "paulabas": "paular",
    "paulad": "paular",
    "paulada": "paular",
    "pauladas": "paular",
    "paulado": "paular",
    "paulados": "paular",
    "paulamos": "paular",
    "paulan": "paular",
    "paulando": "paular",
    "paulara": "paular",
    "paularais": "paular",
    "paularan": "paular",
    "paularas": "paular",
    "paulare": "paular",
    "paulareis": "paular",
    "paularemos": "paular",
    "paularen": "paular",
    "paulares": "paular",
    "paularon": "paular",
    "paulará": "paular",
    "paularán": "paular",
    "paularás": "paular",
    "paularé": "paular",
    "paularéis": "paular",
    "paularía": "paular",
    "paularíais": "paular",
    "paularíamos": "paular",
    "paularían": "paular",
    "paularías": "paular",
    "paulas": "paular",
    "paulase": "paular",
    "paulaseis": "paular",
    "paulasen": "paular",
    "paulases": "paular",
    "paulaste": "paular",
    "paulasteis": "paular",
    "paulatina": "paulatino",
    "paulatinas": "paulatino",
    "paulatinos": "paulatino",
    "paule": "paular",
    "paulemos": "paular",
    "paulen": "paular",
    "paules": "paular",
    "paulina": "paulino",
    "paulinas": "paulino",
    "paulinos": "paulino",
    "paulistas": "paulista",
    "paulo": "paular",
    "paulábamos": "paular",
    "pauláis": "paular",
    "pauláramos": "paular",
    "pauláremos": "paular",
    "paulásemos": "paular",
    "paulé": "paular",
    "pauléis": "paular",
    "pauló": "paular",
    "pauperismos": "pauperismo",
    "pauperizaciones": "pauperización",
    "paupérrima": "paupérrimo",
    "paupérrimas": "paupérrimo",
    "paupérrimos": "paupérrimo",
    "paurópodos": "paurópodo",
    "pausa": "pausar",
    "pausaba": "pausar",
    "pausabais": "pausar",
    "pausaban": "pausar",
    "pausabas": "pausar",
    "pausad": "pausar",
    "pausada": "pausar",
    "pausadas": "pausar",
    "pausado": "pausar",
    "pausados": "pausar",
    "pausamos": "pausar",
    "pausan": "pausar",
    "pausando": "pausar",
    "pausara": "pausar",
    "pausarais": "pausar",
    "pausaran": "pausar",
    "pausaras": "pausar",
    "pausare": "pausar",
    "pausareis": "pausar",
    "pausaremos": "pausar",
    "pausaren": "pausar",
    "pausares": "pausar",
    "pausaron": "pausar",
    "pausará": "pausar",
    "pausarán": "pausar",
    "pausarás": "pausar",
    "pausaré": "pausar",
    "pausaréis": "pausar",
    "pausaría": "pausar",
    "pausaríais": "pausar",
    "pausaríamos": "pausar",
    "pausarían": "pausar",
    "pausarías": "pausar",
    "pausas": "pausar",
    "pausase": "pausar",
    "pausaseis": "pausar",
    "pausasen": "pausar",
    "pausases": "pausar",
    "pausaste": "pausar",
    "pausasteis": "pausar",
    "pause": "pausar",
    "pausemos": "pausar",
    "pausen": "pausar",
    "pauses": "pausar",
    "pauso": "pausar",
    "pausábamos": "pausar",
    "pausáis": "pausar",
    "pausáramos": "pausar",
    "pausáremos": "pausar",
    "pausásemos": "pausar",
    "pausé": "pausar",
    "pauséis": "pausar",
    "pausó": "pausar",
    "pauta": "pautar",
    "pautaba": "pautar",
    "pautabais": "pautar",
    "pautaban": "pautar",
    "pautabas": "pautar",
    "pautad": "pautar",
    "pautada": "pautar",
    "pautadas": "pautar",
    "pautado": "pautar",
    "pautados": "pautar",
    "pautamos": "pautar",
    "pautan": "pautar",
    "pautando": "pautar",
    "pautara": "pautar",
    "pautarais": "pautar",
    "pautaran": "pautar",
    "pautaras": "pautar",
    "pautare": "pautar",
    "pautareis": "pautar",
    "pautaremos": "pautar",
    "pautaren": "pautar",
    "pautares": "pautar",
    "pautaron": "pautar",
    "pautará": "pautar",
    "pautarán": "pautar",
    "pautarás": "pautar",
    "pautaré": "pautar",
    "pautaréis": "pautar",
    "pautaría": "pautar",
    "pautaríais": "pautar",
    "pautaríamos": "pautar",
    "pautarían": "pautar",
    "pautarías": "pautar",
    "pautas": "pautar",
    "pautase": "pautar",
    "pautaseis": "pautar",
    "pautasen": "pautar",
    "pautases": "pautar",
    "pautaste": "pautar",
    "pautasteis": "pautar",
    "paute": "pautar",
    "pautemos": "pautar",
    "pauten": "pautar",
    "pautes": "pautar",
    "pauto": "pautar",
    "pautábamos": "pautar",
    "pautáis": "pautar",
    "pautáramos": "pautar",
    "pautáremos": "pautar",
    "pautásemos": "pautar",
    "pauté": "pautar",
    "pautéis": "pautar",
    "pautó": "pautar",
    "pava": "pavo",
    "pavadas": "pavada",
    "pavanas": "pavana",
    "pavas": "pavo",
    "pavera": "pavero",
    "paveras": "pavero",
    "paveros": "pavero",
    "pavesadas": "pavesada",
    "pavesas": "pavesa",
    "paveses": "pavés",
    "pavesinas": "pavesina",
    "pavimenta": "pavimentar",
    "pavimentaba": "pavimentar",
    "pavimentabais": "pavimentar",
    "pavimentaban": "pavimentar",
    "pavimentabas": "pavimentar",
    "pavimentaciones": "pavimentación",
    "pavimentad": "pavimentar",
    "pavimentada": "pavimentar",
    "pavimentadas": "pavimentar",
    "pavimentado": "pavimentar",
    "pavimentadora": "pavimentador",
    "pavimentadoras": "pavimentador",
    "pavimentadores": "pavimentador",
    "pavimentados": "pavimentar",
    "pavimentamos": "pavimentar",
    "pavimentan": "pavimentar",
    "pavimentando": "pavimentar",
    "pavimentara": "pavimentar",
    "pavimentarais": "pavimentar",
    "pavimentaran": "pavimentar",
    "pavimentaras": "pavimentar",
    "pavimentare": "pavimentar",
    "pavimentareis": "pavimentar",
    "pavimentaremos": "pavimentar",
    "pavimentaren": "pavimentar",
    "pavimentares": "pavimentar",
    "pavimentaron": "pavimentar",
    "pavimentará": "pavimentar",
    "pavimentarán": "pavimentar",
    "pavimentarás": "pavimentar",
    "pavimentaré": "pavimentar",
    "pavimentaréis": "pavimentar",
    "pavimentaría": "pavimentar",
    "pavimentaríais": "pavimentar",
    "pavimentaríamos": "pavimentar",
    "pavimentarían": "pavimentar",
    "pavimentarías": "pavimentar",
    "pavimentas": "pavimentar",
    "pavimentase": "pavimentar",
    "pavimentaseis": "pavimentar",
    "pavimentasen": "pavimentar",
    "pavimentases": "pavimentar",
    "pavimentaste": "pavimentar",
    "pavimentasteis": "pavimentar",
    "pavimente": "pavimentar",
    "pavimentemos": "pavimentar",
    "pavimenten": "pavimentar",
    "pavimentes": "pavimentar",
    "pavimento": "pavimentar",
    "pavimentos": "pavimento",
    "pavimentábamos": "pavimentar",
    "pavimentáis": "pavimentar",
    "pavimentáramos": "pavimentar",
    "pavimentáremos": "pavimentar",
    "pavimentásemos": "pavimentar",
    "pavimenté": "pavimentar",
    "pavimentéis": "pavimentar",
    "pavimentó": "pavimentar",
    "pavipollos": "pavipollo",
    "pavisosa": "pavisoso",
    "pavisosas": "pavisoso",
    "pavisosos": "pavisoso",
    "pavitonta": "pavitonto",
    "pavitontas": "pavitonto",
    "pavitontos": "pavitonto",
    "pavona": "pavonar",
    "pavonaba": "pavonar",
    "pavonabais": "pavonar",
    "pavonaban": "pavonar",
    "pavonabas": "pavonar",
    "pavonad": "pavonar",
    "pavonada": "pavonar",
    "pavonadas": "pavonar",
    "pavonado": "pavonar",
    "pavonados": "pavonar",
    "pavonamos": "pavonar",
    "pavonan": "pavonar",
    "pavonando": "pavonar",
    "pavonara": "pavonar",
    "pavonarais": "pavonar",
    "pavonaran": "pavonar",
    "pavonaras": "pavonar",
    "pavonare": "pavonar",
    "pavonareis": "pavonar",
    "pavonaremos": "pavonar",
    "pavonaren": "pavonar",
    "pavonares": "pavonar",
    "pavonaron": "pavonar",
    "pavonará": "pavonar",
    "pavonarán": "pavonar",
    "pavonarás": "pavonar",
    "pavonaré": "pavonar",
    "pavonaréis": "pavonar",
    "pavonaría": "pavonar",
    "pavonaríais": "pavonar",
    "pavonaríamos": "pavonar",
    "pavonarían": "pavonar",
    "pavonarías": "pavonar",
    "pavonas": "pavonar",
    "pavonase": "pavonar",
    "pavonaseis": "pavonar",
    "pavonasen": "pavonar",
    "pavonases": "pavonar",
    "pavonaste": "pavonar",
    "pavonasteis": "pavonar",
    "pavonazos": "pavonazo",
    "pavone": "pavonar",
    "pavonea": "pavonear",
    "pavoneaba": "pavonear",
    "pavoneabais": "pavonear",
    "pavoneaban": "pavonear",
    "pavoneabas": "pavonear",
    "pavonead": "pavonear",
    "pavoneada": "pavonear",
    "pavoneadas": "pavonear",
    "pavoneado": "pavonear",
    "pavoneados": "pavonear",
    "pavoneamos": "pavonear",
    "pavonean": "pavonear",
    "pavoneando": "pavonear",
    "pavoneara": "pavonear",
    "pavonearais": "pavonear",
    "pavonearan": "pavonear",
    "pavonearas": "pavonear",
    "pavoneare": "pavonear",
    "pavoneareis": "pavonear",
    "pavonearemos": "pavonear",
    "pavonearen": "pavonear",
    "pavoneares": "pavonear",
    "pavonearon": "pavonear",
    "pavoneará": "pavonear",
    "pavonearán": "pavonear",
    "pavonearás": "pavonear",
    "pavonearé": "pavonear",
    "pavonearéis": "pavonear",
    "pavonearía": "pavonear",
    "pavonearíais": "pavonear",
    "pavonearíamos": "pavonear",
    "pavonearían": "pavonear",
    "pavonearías": "pavonear",
    "pavoneas": "pavonear",
    "pavonease": "pavonear",
    "pavoneaseis": "pavonear",
    "pavoneasen": "pavonear",
    "pavoneases": "pavonear",
    "pavoneaste": "pavonear",
    "pavoneasteis": "pavonear",
    "pavonee": "pavonear",
    "pavoneemos": "pavonear",
    "pavoneen": "pavonear",
    "pavonees": "pavonear",
    "pavonemos": "pavonar",
    "pavonen": "pavonar",
    "pavoneo": "pavonear",
    "pavoneos": "pavoneo",
    "pavones": "pavonar",
    "pavoneábamos": "pavonear",
    "pavoneáis": "pavonear",
    "pavoneáramos": "pavonear",
    "pavoneáremos": "pavonear",
    "pavoneásemos": "pavonear",
    "pavoneé": "pavonear",
    "pavoneéis": "pavonear",
    "pavoneó": "pavonear",
    "pavono": "pavonar",
    "pavonábamos": "pavonar",
    "pavonáis": "pavonar",
    "pavonáramos": "pavonar",
    "pavonáremos": "pavonar",
    "pavonásemos": "pavonar",
    "pavoné": "pavonar",
    "pavonéis": "pavonar",
    "pavonó": "pavonar",
    "pavordes": "pavorde",
    "pavordías": "pavordía",
    "pavores": "pavor",
    "pavorida": "pavorido",
    "pavoridas": "pavorido",
    "pavoridos": "pavorido",
    "pavorosa": "pavoroso",
    "pavorosas": "pavoroso",
    "pavorosos": "pavoroso",
    "pavos": "pavo",
    "pavuras": "pavura",
    "pavías": "pavía",
    "paya": "payo",
    "payadores": "payador",
    "payas": "payo",
    "payasa": "payaso",
    "payasadas": "payasada",
    "payasas": "payaso",
    "payasos": "payaso",
    "payesa": "payés",
    "payesas": "payés",
    "payeses": "payés",
    "payos": "payo",
    "pazca": "pacer",
    "pazcamos": "pacer",
    "pazcan": "pacer",
    "pazcas": "pacer",
    "pazco": "pacer",
    "pazcáis": "pacer",
    "pazguata": "pazguato",
    "pazguatas": "pazguato",
    "pazguaterías": "pazguatería",
    "pazguatos": "pazguato",
    "pazos": "pazo",
    "pazotes": "pazote",
    "pazpuercas": "pazpuerca",
    "países": "país",
    "paíños": "paíño",
    "pañales": "pañal",
    "pañalones": "pañalón",
    "pañera": "pañero",
    "pañeras": "pañero",
    "pañeros": "pañero",
    "pañerías": "pañería",
    "pañetes": "pañete",
    "pañoladas": "pañolada",
    "pañolera": "pañolero",
    "pañoleras": "pañolero",
    "pañoleros": "pañolero",
    "pañolerías": "pañolería",
    "pañoles": "pañol",
    "pañoletas": "pañoleta",
    "pañolones": "pañolón",
    "paños": "paño",
    "pañosa": "pañoso",
    "pañosas": "pañoso",
    "pañosos": "pañoso",
    "pañuelos": "pañuelo",
    "paúles": "paúl",
    "pea": "peer",
    "peajes": "peaje",
    "peales": "peal",
    "peamos": "peer",
    "pean": "peer",
    "peanas": "peana",
    "peas": "peer",
    "peatonales": "peatonal",
    "peatones": "peatón",
    "pebetera": "pebetero",
    "pebeteras": "pebetero",
    "pebeteros": "pebetero",
    "pebetes": "pebete",
    "pebradas": "pebrada",
    "pebres": "pebre",
    "peca": "pecar",
    "pecaba": "pecar",
    "pecabais": "pecar",
    "pecaban": "pecar",
    "pecabas": "pecar",
    "pecables": "pecable",
    "pecad": "pecar",
    "pecada": "pecar",
    "pecadas": "pecar",
    "pecado": "pecar",
    "pecadora": "pecador",
    "pecadoras": "pecador",
    "pecadores": "pecador",
    "pecadoriza": "pecadorizo",
    "pecadorizas": "pecadorizo",
    "pecadorizos": "pecadorizo",
    "pecados": "pecar",
    "pecaminosa": "pecaminoso",
    "pecaminosas": "pecaminoso",
    "pecaminosos": "pecaminoso",
    "pecamos": "pecar",
    "pecan": "pecar",
    "pecando": "pecar",
    "pecantes": "pecante",
    "pecara": "pecar",
    "pecarais": "pecar",
    "pecaran": "pecar",
    "pecaras": "pecar",
    "pecare": "pecar",
    "pecareis": "pecar",
    "pecaremos": "pecar",
    "pecaren": "pecar",
    "pecares": "pecar",
    "pecaron": "pecar",
    "pecará": "pecar",
    "pecarán": "pecar",
    "pecarás": "pecar",
    "pecaré": "pecar",
    "pecaréis": "pecar",
    "pecaría": "pecar",
    "pecaríais": "pecar",
    "pecaríamos": "pecar",
    "pecarían": "pecar",
    "pecarías": "pecar",
    "pecaríes": "pecarí",
    "pecarís": "pecarí",
    "pecas": "pecar",
    "pecase": "pecar",
    "pecaseis": "pecar",
    "pecasen": "pecar",
    "pecases": "pecar",
    "pecaste": "pecar",
    "pecasteis": "pecar",
    "pecblendas": "pecblenda",
    "pecdecks": "pecdeck",
    "peceras": "pecera",
    "peces": "pez",
    "peceña": "peceño",
    "peceñas": "peceño",
    "peceños": "peceño",
    "pecha": "pechar",
    "pechaba": "pechar",
    "pechabais": "pechar",
    "pechaban": "pechar",
    "pechabas": "pechar",
    "pechad": "pechar",
    "pechada": "pechar",
    "pechadas": "pechar",
    "pechado": "pechar",
    "pechados": "pechar",
    "pechamos": "pechar",
    "pechan": "pechar",
    "pechando": "pechar",
    "pechara": "pechar",
    "pecharais": "pechar",
    "pecharan": "pechar",
    "pecharas": "pechar",
    "pechare": "pechar",
    "pechareis": "pechar",
    "pecharemos": "pechar",
    "pecharen": "pechar",
    "pechares": "pechar",
    "pecharon": "pechar",
    "pechará": "pechar",
    "pecharán": "pechar",
    "pecharás": "pechar",
    "pecharé": "pechar",
    "pecharéis": "pechar",
    "pecharía": "pechar",
    "pecharíais": "pechar",
    "pecharíamos": "pechar",
    "pecharían": "pechar",
    "pecharías": "pechar",
    "pechas": "pechar",
    "pechase": "pechar",
    "pechaseis": "pechar",
    "pechasen": "pechar",
    "pechases": "pechar",
    "pechaste": "pechar",
    "pechasteis": "pechar",
    "pechblendas": "pechblenda",
    "peche": "pechar",
    "pechemos": "pechar",
    "pechen": "pechar",
    "pechera": "pechero",
    "pecheras": "pechero",
    "pecheros": "pechero",
    "pecherías": "pechería",
    "peches": "pechar",
    "pechinas": "pechina",
    "pechirrojos": "pechirrojo",
    "pechisacada": "pechisacado",
    "pechisacadas": "pechisacado",
    "pechisacados": "pechisacado",
    "pecho": "pechar",
    "pechos": "pecho",
    "pechugas": "pechuga",
    "pechugazos": "pechugazo",
    "pechugona": "pechugón",
    "pechugonas": "pechugón",
    "pechugones": "pechugón",
    "pechugueras": "pechuguera",
    "pechábamos": "pechar",
    "pecháis": "pechar",
    "pecháramos": "pechar",
    "pecháremos": "pechar",
    "pechásemos": "pechar",
    "peché": "pechar",
    "pechéis": "pechar",
    "pechó": "pechar",
    "pecienta": "peciento",
    "pecientas": "peciento",
    "pecientos": "peciento",
    "pecilocitos": "pecilocito",
    "peciluenga": "peciluengo",
    "peciluengas": "peciluengo",
    "peciluengos": "peciluengo",
    "pecinales": "pecinal",
    "pecinas": "pecina",
    "pecinosa": "pecinoso",
    "pecinosas": "pecinoso",
    "pecinosos": "pecinoso",
    "peciolada": "peciolado",
    "pecioladas": "peciolado",
    "peciolados": "peciolado",
    "peciolos": "peciolo",
    "pecios": "pecio",
    "pecks": "peck",
    "peco": "pecar",
    "pecoreas": "pecorea",
    "pecosa": "pecoso",
    "pecosas": "pecoso",
    "pecosos": "pecoso",
    "pectinada": "pectinado",
    "pectinadas": "pectinado",
    "pectinados": "pectinado",
    "pectinas": "pectina",
    "pectinasas": "pectinasa",
    "pectiniformes": "pectiniforme",
    "pectizaciones": "pectización",
    "pectografías": "pectografía",
    "pectolitas": "pectolita",
    "pectorales": "pectoral",
    "pectínea": "pectíneo",
    "pectíneas": "pectíneo",
    "pectíneos": "pectíneo",
    "pectínidos": "pectínido",
    "pecuaria": "pecuario",
    "pecuarias": "pecuario",
    "pecuarios": "pecuario",
    "peculados": "peculado",
    "peculiares": "peculiar",
    "peculiaridades": "peculiaridad",
    "peculiariedades": "peculiariedad",
    "peculiarismos": "peculiarismo",
    "peculios": "peculio",
    "pecuniaria": "pecuniario",
    "pecuniarias": "pecuniario",
    "pecuniarios": "pecuniario",
    "pecunias": "pecunia",
    "pecábamos": "pecar",
    "pecáis": "pecar",
    "pecáramos": "pecar",
    "pecáremos": "pecar",
    "pecásemos": "pecar",
    "pecíolos": "pecíolo",
    "pecó": "pecar",
    "peda": "peder",
    "pedagoga": "pedagogo",
    "pedagogas": "pedagogo",
    "pedagogos": "pedagogo",
    "pedagogías": "pedagogía",
    "pedagógica": "pedagógico",
    "pedagógicas": "pedagógico",
    "pedagógicos": "pedagógico",
    "pedajes": "pedaje",
    "pedaladas": "pedalada",
    "pedalea": "pedalear",
    "pedaleaba": "pedalear",
    "pedaleabais": "pedalear",
    "pedaleaban": "pedalear",
    "pedaleabas": "pedalear",
    "pedalead": "pedalear",
    "pedaleada": "pedalear",
    "pedaleadas": "pedalear",
    "pedaleado": "pedalear",
    "pedaleados": "pedalear",
    "pedaleamos": "pedalear",
    "pedalean": "pedalear",
    "pedaleando": "pedalear",
    "pedaleara": "pedalear",
    "pedalearais": "pedalear",
    "pedalearan": "pedalear",
    "pedalearas": "pedalear",
    "pedaleare": "pedalear",
    "pedaleareis": "pedalear",
    "pedalearemos": "pedalear",
    "pedalearen": "pedalear",
    "pedaleares": "pedalear",
    "pedalearon": "pedalear",
    "pedaleará": "pedalear",
    "pedalearán": "pedalear",
    "pedalearás": "pedalear",
    "pedalearé": "pedalear",
    "pedalearéis": "pedalear",
    "pedalearía": "pedalear",
    "pedalearíais": "pedalear",
    "pedalearíamos": "pedalear",
    "pedalearían": "pedalear",
    "pedalearías": "pedalear",
    "pedaleas": "pedalear",
    "pedalease": "pedalear",
    "pedaleaseis": "pedalear",
    "pedaleasen": "pedalear",
    "pedaleases": "pedalear",
    "pedaleaste": "pedalear",
    "pedaleasteis": "pedalear",
    "pedalee": "pedalear",
    "pedaleemos": "pedalear",
    "pedaleen": "pedalear",
    "pedalees": "pedalear",
    "pedaleo": "pedalear",
    "pedaleos": "pedaleo",
    "pedales": "pedal",
    "pedaleábamos": "pedalear",
    "pedaleáis": "pedalear",
    "pedaleáramos": "pedalear",
    "pedaleáremos": "pedalear",
    "pedaleásemos": "pedalear",
    "pedaleé": "pedalear",
    "pedaleéis": "pedalear",
    "pedaleó": "pedalear",
    "pedalferes": "pedalfer",
    "pedaliers": "pedalier",
    "pedalinas": "pedalina",
    "pedaliácea": "pedaliáceo",
    "pedaliáceas": "pedaliáceo",
    "pedaliáceos": "pedaliáceo",
    "pedamos": "peder",
    "pedan": "peder",
    "pedantea": "pedantear",
    "pedanteaba": "pedantear",
    "pedanteabais": "pedantear",
    "pedanteaban": "pedantear",
    "pedanteabas": "pedantear",
    "pedantead": "pedantear",
    "pedanteada": "pedantear",
    "pedanteadas": "pedantear",
    "pedanteado": "pedantear",
    "pedanteados": "pedantear",
    "pedanteamos": "pedantear",
    "pedantean": "pedantear",
    "pedanteando": "pedantear",
    "pedanteara": "pedantear",
    "pedantearais": "pedantear",
    "pedantearan": "pedantear",
    "pedantearas": "pedantear",
    "pedanteare": "pedantear",
    "pedanteareis": "pedantear",
    "pedantearemos": "pedantear",
    "pedantearen": "pedantear",
    "pedanteares": "pedantear",
    "pedantearon": "pedantear",
    "pedanteará": "pedantear",
    "pedantearán": "pedantear",
    "pedantearás": "pedantear",
    "pedantearé": "pedantear",
    "pedantearéis": "pedantear",
    "pedantearía": "pedantear",
    "pedantearíais": "pedantear",
    "pedantearíamos": "pedantear",
    "pedantearían": "pedantear",
    "pedantearías": "pedantear",
    "pedanteas": "pedantear",
    "pedantease": "pedantear",
    "pedanteaseis": "pedantear",
    "pedanteasen": "pedantear",
    "pedanteases": "pedantear",
    "pedanteaste": "pedantear",
    "pedanteasteis": "pedantear",
    "pedantee": "pedantear",
    "pedanteemos": "pedantear",
    "pedanteen": "pedantear",
    "pedantees": "pedantear",
    "pedanteo": "pedantear",
    "pedanterías": "pedantería",
    "pedantes": "pedante",
    "pedantesca": "pedantesco",
    "pedantescas": "pedantesco",
    "pedantescos": "pedantesco",
    "pedanteábamos": "pedantear",
    "pedanteáis": "pedantear",
    "pedanteáramos": "pedantear",
    "pedanteáremos": "pedantear",
    "pedanteásemos": "pedantear",
    "pedanteé": "pedantear",
    "pedanteéis": "pedantear",
    "pedanteó": "pedantear",
    "pedantismos": "pedantismo",
    "pedas": "peder",
    "pedazos": "pedazo",
    "pede": "peder",
    "peded": "peder",
    "pedemos": "peder",
    "peden": "peder",
    "pederastas": "pederasta",
    "pederastias": "pederastia",
    "pederemos": "peder",
    "pedernales": "pedernal",
    "pedernalina": "pedernalino",
    "pedernalinas": "pedernalino",
    "pedernalinos": "pedernalino",
    "pederá": "peder",
    "pederán": "peder",
    "pederás": "peder",
    "pederástica": "pederástico",
    "pederásticas": "pederástico",
    "pederásticos": "pederástico",
    "pederé": "peder",
    "pederéis": "peder",
    "pedería": "peder",
    "pederíais": "peder",
    "pederíamos": "peder",
    "pederían": "peder",
    "pederías": "peder",
    "pedes": "peder",
    "pedestales": "pedestal",
    "pedestres": "pedestre",
    "pedestrismos": "pedestrismo",
    "pedia": "pedio",
    "pediales": "pedial",
    "pedias": "pedio",
    "pediatras": "pediatra",
    "pediatrías": "pediatría",
    "pedicelos": "pedicelo",
    "pediculada": "pediculado",
    "pediculadas": "pediculado",
    "pediculados": "pediculado",
    "pediculares": "pedicular",
    "pedicuras": "pedicura",
    "pedicuros": "pedicuro",
    "pedid": "pedir",
    "pedida": "pedir",
    "pedidas": "pedir",
    "pedido": "pedir",
    "pedidora": "pedidor",
    "pedidoras": "pedidor",
    "pedidores": "pedidor",
    "pedidos": "pedir",
    "pediduras": "pedidura",
    "pediendo": "peder",
    "pediera": "peder",
    "pedierais": "peder",
    "pedieran": "peder",
    "pedieras": "peder",
    "pediere": "peder",
    "pediereis": "peder",
    "pedieren": "peder",
    "pedieres": "peder",
    "pedieron": "peder",
    "pediese": "peder",
    "pedieseis": "peder",
    "pediesen": "peder",
    "pedieses": "peder",
    "pedigona": "pedigón",
    "pedigonas": "pedigón",
    "pedigones": "pedigón",
    "pedigrees": "pedigree",
    "pedigríes": "pedigrí",
    "pedigrís": "pedigrí",
    "pedigríses": "pedigrí",
    "pedigüeña": "pedigüeño",
    "pedigüeñas": "pedigüeño",
    "pedigüeñerías": "pedigüeñería",
    "pedigüeños": "pedigüeño",
    "pedillanuras": "pedillanura",
    "pediluvios": "pediluvio",
    "pedimos": "pedir",
    "pedinomonadales": "pedinomonadal",
    "pediones": "pedión",
    "pedios": "pedio",
    "pedipalpos": "pedipalpo",
    "pediremos": "pedir",
    "pedirá": "pedir",
    "pedirán": "pedir",
    "pedirás": "pedir",
    "pediré": "pedir",
    "pediréis": "pedir",
    "pediría": "pedir",
    "pediríais": "pedir",
    "pediríamos": "pedir",
    "pedirían": "pedir",
    "pedirías": "pedir",
    "pediste": "pedir",
    "pedisteis": "pedir",
    "pediátrica": "pediátrico",
    "pediátricas": "pediátrico",
    "pediátricos": "pediátrico",
    "pediéramos": "peder",
    "pediéremos": "peder",
    "pediésemos": "peder",
    "pedió": "peder",
    "pedo": "peder",
    "pedocales": "pedocal",
    "pedofilias": "pedofilia",
    "pedográfica": "pedográfico",
    "pedográficas": "pedográfico",
    "pedográficos": "pedográfico",
    "pedologías": "pedología",
    "pedorreos": "pedorreo",
    "pedorrera": "pedorrero",
    "pedorreras": "pedorrero",
    "pedorreros": "pedorrero",
    "pedorretas": "pedorreta",
    "pedos": "pedo",
    "pedradas": "pedrada",
    "pedrales": "pedral",
    "pedreas": "pedrea",
    "pedregales": "pedregal",
    "pedregosa": "pedregoso",
    "pedregosas": "pedregoso",
    "pedregosidades": "pedregosidad",
    "pedregosos": "pedregoso",
    "pedrejones": "pedrejón",
    "pedrerales": "pedreral",
    "pedreras": "pedrera",
    "pedreros": "pedrero",
    "pedrerías": "pedrería",
    "pedriscales": "pedriscal",
    "pedriscas": "pedrisca",
    "pedriscos": "pedrisco",
    "pedriza": "pedrizo",
    "pedrizas": "pedrizo",
    "pedrizos": "pedrizo",
    "pedruscos": "pedrusco",
    "pedunculada": "pedunculado",
    "pedunculadas": "pedunculado",
    "pedunculados": "pedunculado",
    "pedunculares": "peduncular",
    "pedáis": "peder",
    "pedánea": "pedáneo",
    "pedáneas": "pedáneo",
    "pedáneos": "pedáneo",
    "pedéis": "peder",
    "pedí": "pedir",
    "pedía": "pedir",
    "pedíais": "pedir",
    "pedíamos": "pedir",
    "pedían": "pedir",
    "pedías": "pedir",
    "pedículos": "pedículo",
    "pedís": "pedir",
    "pedófila": "pedófilo",
    "pedófilas": "pedófilo",
    "pedófilos": "pedófilo",
    "pedóloga": "pedólogo",
    "pedólogas": "pedólogo",
    "pedólogos": "pedólogo",
    "pedúnculos": "pedúnculo",
    "pee": "peer",
    "peed": "peer",
    "peelings": "peeling",
    "peemos": "peer",
    "peen": "peer",
    "peeremos": "peer",
    "peerá": "peer",
    "peerán": "peer",
    "peerás": "peer",
    "peeré": "peer",
    "peeréis": "peer",
    "peería": "peer",
    "peeríais": "peer",
    "peeríamos": "peer",
    "peerían": "peer",
    "peerías": "peer",
    "pees": "peer",
    "pega": "pegar",
    "pegaba": "pegar",
    "pegabais": "pegar",
    "pegaban": "pegar",
    "pegabas": "pegar",
    "pegad": "pegar",
    "pegada": "pegar",
    "pegadas": "pegar",
    "pegadiza": "pegadizo",
    "pegadizas": "pegadizo",
    "pegadizos": "pegadizo",
    "pegado": "pegar",
    "pegadora": "pegador",
    "pegadoras": "pegador",
    "pegadores": "pegador",
    "pegados": "pegar",
    "pegaduras": "pegadura",
    "pegajosa": "pegajoso",
    "pegajosas": "pegajoso",
    "pegajosidades": "pegajosidad",
    "pegajosos": "pegajoso",
    "pegamentos": "pegamento",
    "pegamientos": "pegamiento",
    "pegamoides": "pegamoide",
    "pegamos": "pegar",
    "pegan": "pegar",
    "pegando": "pegar",
    "pegara": "pegar",
    "pegarais": "pegar",
    "pegaran": "pegar",
    "pegaras": "pegar",
    "pegare": "pegar",
    "pegareis": "pegar",
    "pegaremos": "pegar",
    "pegaren": "pegar",
    "pegares": "pegar",
    "pegaron": "pegar",
    "pegará": "pegar",
    "pegarán": "pegar",
    "pegarás": "pegar",
    "pegaré": "pegar",
    "pegaréis": "pegar",
    "pegaría": "pegar",
    "pegaríais": "pegar",
    "pegaríamos": "pegar",
    "pegarían": "pegar",
    "pegarías": "pegar",
    "pegas": "pegar",
    "pegase": "pegar",
    "pegasea": "pegaseo",
    "pegaseas": "pegaseo",
    "pegaseis": "pegar",
    "pegasen": "pegar",
    "pegaseos": "pegaseo",
    "pegases": "pegar",
    "pegasos": "pegaso",
    "pegaste": "pegar",
    "pegasteis": "pegar",
    "pegatas": "pegata",
    "pegatinas": "pegatina",
    "peggioristas": "peggiorista",
    "pegmatitas": "pegmatita",
    "pego": "pegar",
    "pegos": "pego",
    "pegoterías": "pegotería",
    "pegotes": "pegote",
    "pegue": "pegar",
    "peguemos": "pegar",
    "peguen": "pegar",
    "peguera": "peguero",
    "pegueras": "peguero",
    "pegueros": "peguero",
    "pegues": "pegar",
    "pegujalera": "pegujalero",
    "pegujaleras": "pegujalero",
    "pegujaleros": "pegujalero",
    "pegujales": "pegujal",
    "pegujarera": "pegujarero",
    "pegujareras": "pegujarero",
    "pegujareros": "pegujarero",
    "pegujares": "pegujar",
    "pegujones": "pegujón",
    "pegullones": "pegullón",
    "pegunta": "peguntar",
    "peguntaba": "peguntar",
    "peguntabais": "peguntar",
    "peguntaban": "peguntar",
    "peguntabas": "peguntar",
    "peguntad": "peguntar",
    "peguntada": "peguntar",
    "peguntadas": "peguntar",
    "peguntado": "peguntar",
    "peguntados": "peguntar",
    "peguntamos": "peguntar",
    "peguntan": "peguntar",
    "peguntando": "peguntar",
    "peguntara": "peguntar",
    "peguntarais": "peguntar",
    "peguntaran": "peguntar",
    "peguntaras": "peguntar",
    "peguntare": "peguntar",
    "peguntareis": "peguntar",
    "peguntaremos": "peguntar",
    "peguntaren": "peguntar",
    "peguntares": "peguntar",
    "peguntaron": "peguntar",
    "peguntará": "peguntar",
    "peguntarán": "peguntar",
    "peguntarás": "peguntar",
    "peguntaré": "peguntar",
    "peguntaréis": "peguntar",
    "peguntaría": "peguntar",
    "peguntaríais": "peguntar",
    "peguntaríamos": "peguntar",
    "peguntarían": "peguntar",
    "peguntarías": "peguntar",
    "peguntas": "peguntar",
    "peguntase": "peguntar",
    "peguntaseis": "peguntar",
    "peguntasen": "peguntar",
    "peguntases": "peguntar",
    "peguntaste": "peguntar",
    "peguntasteis": "peguntar",
    "pegunte": "peguntar",
    "peguntemos": "peguntar",
    "pegunten": "peguntar",
    "peguntes": "pegunte",
    "pegunto": "peguntar",
    "peguntosa": "peguntoso",
    "peguntosas": "peguntoso",
    "peguntosos": "peguntoso",
    "peguntábamos": "peguntar",
    "peguntáis": "peguntar",
    "peguntáramos": "peguntar",
    "peguntáremos": "peguntar",
    "peguntásemos": "peguntar",
    "pegunté": "peguntar",
    "peguntéis": "peguntar",
    "peguntó": "peguntar",
    "pegué": "pegar",
    "peguéis": "pegar",
    "pegábamos": "pegar",
    "pegáis": "pegar",
    "pegáramos": "pegar",
    "pegáremos": "pegar",
    "pegásemos": "pegar",
    "pegó": "pegar",
    "pehachímetros": "pehachímetro",
    "peina": "peinar",
    "peinaba": "peinar",
    "peinabais": "peinar",
    "peinaban": "peinar",
    "peinabas": "peinar",
    "peinad": "peinar",
    "peinada": "peinar",
    "peinadas": "peinar",
    "peinado": "peinar",
    "peinadora": "peinador",
    "peinadoras": "peinadora",
    "peinadores": "peinador",
    "peinados": "peinar",
    "peinamos": "peinar",
    "peinan": "peinar",
    "peinando": "peinar",
    "peinara": "peinar",
    "peinarais": "peinar",
    "peinaran": "peinar",
    "peinaras": "peinar",
    "peinare": "peinar",
    "peinareis": "peinar",
    "peinaremos": "peinar",
    "peinaren": "peinar",
    "peinares": "peinar",
    "peinaron": "peinar",
    "peinará": "peinar",
    "peinarán": "peinar",
    "peinarás": "peinar",
    "peinaré": "peinar",
    "peinaréis": "peinar",
    "peinaría": "peinar",
    "peinaríais": "peinar",
    "peinaríamos": "peinar",
    "peinarían": "peinar",
    "peinarías": "peinar",
    "peinas": "peinar",
    "peinase": "peinar",
    "peinaseis": "peinar",
    "peinasen": "peinar",
    "peinases": "peinar",
    "peinaste": "peinar",
    "peinasteis": "peinar",
    "peinazos": "peinazo",
    "peine": "peinar",
    "peinecillos": "peinecillo",
    "peinemos": "peinar",
    "peinen": "peinar",
    "peinera": "peinero",
    "peineras": "peinero",
    "peineros": "peinero",
    "peinerías": "peinería",
    "peines": "peine",
    "peinetas": "peineta",
    "peinetera": "peinetero",
    "peineteras": "peinetero",
    "peineteros": "peinetero",
    "peino": "peinar",
    "peinábamos": "peinar",
    "peináis": "peinar",
    "peináramos": "peinar",
    "peináremos": "peinar",
    "peinásemos": "peinar",
    "peiné": "peinar",
    "peinéis": "peinar",
    "peinó": "peinar",
    "pejepalos": "pejepalo",
    "pejerreyes": "pejerrey",
    "pejes": "peje",
    "pejesapos": "pejesapo",
    "pejigueras": "pejiguera",
    "pekinesa": "pekinés",
    "pekinesas": "pekinés",
    "pekineses": "pekinés",
    "pela": "pelar",
    "pelaba": "pelar",
    "pelabais": "pelar",
    "pelaban": "pelar",
    "pelabas": "pelar",
    "pelad": "pelar",
    "pelada": "pelar",
    "peladas": "pelar",
    "peladeras": "peladera",
    "peladeros": "peladero",
    "peladillas": "peladilla",
    "peladillos": "peladillo",
    "pelado": "pelar",
    "peladora": "pelador",
    "peladoras": "peladora",
    "peladores": "pelador",
    "pelados": "pelar",
    "peladuras": "peladura",
    "pelafustana": "pelafustán",
    "pelafustanas": "pelafustán",
    "pelafustanes": "pelafustán",
    "pelagiana": "pelagiano",
    "pelagianas": "pelagiano",
    "pelagianismos": "pelagianismo",
    "pelagianos": "pelagiano",
    "pelagoscopios": "pelagoscopio",
    "pelagras": "pelagra",
    "pelagrosa": "pelagroso",
    "pelagrosas": "pelagroso",
    "pelagrosos": "pelagroso",
    "pelaires": "pelaire",
    "pelairías": "pelairía",
    "pelajes": "pelaje",
    "pelambra": "pelambrar",
    "pelambraba": "pelambrar",
    "pelambrabais": "pelambrar",
    "pelambraban": "pelambrar",
    "pelambrabas": "pelambrar",
    "pelambrad": "pelambrar",
    "pelambrada": "pelambrar",
    "pelambradas": "pelambrar",
    "pelambrado": "pelambrar",
    "pelambrados": "pelambrar",
    "pelambramos": "pelambrar",
    "pelambran": "pelambrar",
    "pelambrando": "pelambrar",
    "pelambrara": "pelambrar",
    "pelambrarais": "pelambrar",
    "pelambraran": "pelambrar",
    "pelambraras": "pelambrar",
    "pelambrare": "pelambrar",
    "pelambrareis": "pelambrar",
    "pelambraremos": "pelambrar",
    "pelambraren": "pelambrar",
    "pelambrares": "pelambrar",
    "pelambraron": "pelambrar",
    "pelambrará": "pelambrar",
    "pelambrarán": "pelambrar",
    "pelambrarás": "pelambrar",
    "pelambraré": "pelambrar",
    "pelambraréis": "pelambrar",
    "pelambraría": "pelambrar",
    "pelambraríais": "pelambrar",
    "pelambraríamos": "pelambrar",
    "pelambrarían": "pelambrar",
    "pelambrarías": "pelambrar",
    "pelambras": "pelambrar",
    "pelambrase": "pelambrar",
    "pelambraseis": "pelambrar",
    "pelambrasen": "pelambrar",
    "pelambrases": "pelambrar",
    "pelambraste": "pelambrar",
    "pelambrasteis": "pelambrar",
    "pelambre": "pelambrar",
    "pelambremos": "pelambrar",
    "pelambren": "pelambrar",
    "pelambrera": "pelambrero",
    "pelambreras": "pelambrero",
    "pelambreros": "pelambrero",
    "pelambres": "pelambre",
    "pelambro": "pelambrar",
    "pelambrábamos": "pelambrar",
    "pelambráis": "pelambrar",
    "pelambráramos": "pelambrar",
    "pelambráremos": "pelambrar",
    "pelambrásemos": "pelambrar",
    "pelambré": "pelambrar",
    "pelambréis": "pelambrar",
    "pelambró": "pelambrar",
    "pelamos": "pelar",
    "pelan": "pelar",
    "pelando": "pelar",
    "pelanduscas": "pelandusca",
    "pelantrines": "pelantrín",
    "pelara": "pelar",
    "pelarais": "pelar",
    "pelaran": "pelar",
    "pelaras": "pelar",
    "pelare": "pelar",
    "pelareis": "pelar",
    "pelarelas": "pelarela",
    "pelaremos": "pelar",
    "pelaren": "pelar",
    "pelares": "pelar",
    "pelargonatos": "pelargonato",
    "pelargonios": "pelargonio",
    "pelargónica": "pelargónico",
    "pelargónicas": "pelargónico",
    "pelargónicos": "pelargónico",
    "pelaron": "pelar",
    "pelará": "pelar",
    "pelarán": "pelar",
    "pelarás": "pelar",
    "pelaré": "pelar",
    "pelaréis": "pelar",
    "pelaría": "pelar",
    "pelaríais": "pelar",
    "pelaríamos": "pelar",
    "pelarían": "pelar",
    "pelarías": "pelar",
    "pelas": "pelar",
    "pelase": "pelar",
    "pelaseis": "pelar",
    "pelasen": "pelar",
    "pelases": "pelar",
    "pelasga": "pelasgo",
    "pelasgas": "pelasgo",
    "pelasgos": "pelasgo",
    "pelaste": "pelar",
    "pelasteis": "pelar",
    "pelazas": "pelaza",
    "pelazgas": "pelazga",
    "peldaños": "peldaño",
    "pele": "pelar",
    "pelea": "pelear",
    "peleaba": "pelear",
    "peleabais": "pelear",
    "peleaban": "pelear",
    "peleabas": "pelear",
    "pelead": "pelear",
    "peleada": "pelear",
    "peleadas": "pelear",
    "peleado": "pelear",
    "peleadora": "peleador",
    "peleadoras": "peleador",
    "peleadores": "peleador",
    "peleados": "pelear",
    "peleamos": "pelear",
    "pelean": "pelear",
    "peleana": "peleano",
    "peleanas": "peleano",
    "peleando": "pelear",
    "peleanos": "peleano",
    "peleara": "pelear",
    "pelearais": "pelear",
    "pelearan": "pelear",
    "pelearas": "pelear",
    "peleare": "pelear",
    "peleareis": "pelear",
    "pelearemos": "pelear",
    "pelearen": "pelear",
    "peleares": "pelear",
    "pelearon": "pelear",
    "peleará": "pelear",
    "pelearán": "pelear",
    "pelearás": "pelear",
    "pelearé": "pelear",
    "pelearéis": "pelear",
    "pelearía": "pelear",
    "pelearíais": "pelear",
    "pelearíamos": "pelear",
    "pelearían": "pelear",
    "pelearías": "pelear",
    "peleas": "pelear",
    "pelease": "pelear",
    "peleaseis": "pelear",
    "peleasen": "pelear",
    "peleases": "pelear",
    "peleaste": "pelear",
    "peleasteis": "pelear",
    "pelecaniformes": "pelecaniforme",
    "pelecha": "pelechar",
    "pelechaba": "pelechar",
    "pelechabais": "pelechar",
    "pelechaban": "pelechar",
    "pelechabas": "pelechar",
    "pelechad": "pelechar",
    "pelechada": "pelechar",
    "pelechadas": "pelechar",
    "pelechado": "pelechar",
    "pelechados": "pelechar",
    "pelechamos": "pelechar",
    "pelechan": "pelechar",
    "pelechando": "pelechar",
    "pelechara": "pelechar",
    "pelecharais": "pelechar",
    "pelecharan": "pelechar",
    "pelecharas": "pelechar",
    "pelechare": "pelechar",
    "pelechareis": "pelechar",
    "pelecharemos": "pelechar",
    "pelecharen": "pelechar",
    "pelechares": "pelechar",
    "pelecharon": "pelechar",
    "pelechará": "pelechar",
    "pelecharán": "pelechar",
    "pelecharás": "pelechar",
    "pelecharé": "pelechar",
    "pelecharéis": "pelechar",
    "pelecharía": "pelechar",
    "pelecharíais": "pelechar",
    "pelecharíamos": "pelechar",
    "pelecharían": "pelechar",
    "pelecharías": "pelechar",
    "pelechas": "pelechar",
    "pelechase": "pelechar",
    "pelechaseis": "pelechar",
    "pelechasen": "pelechar",
    "pelechases": "pelechar",
    "pelechaste": "pelechar",
    "pelechasteis": "pelechar",
    "peleche": "pelechar",
    "pelechemos": "pelechar",
    "pelechen": "pelechar",
    "peleches": "pelechar",
    "pelecho": "pelechar",
    "pelechos": "pelecho",
    "pelechábamos": "pelechar",
    "pelecháis": "pelechar",
    "pelecháramos": "pelechar",
    "pelecháremos": "pelechar",
    "pelechásemos": "pelechar",
    "peleché": "pelechar",
    "pelechéis": "pelechar",
    "pelechó": "pelechar",
    "pelee": "pelear",
    "peleemos": "pelear",
    "peleen": "pelear",
    "pelees": "pelear",
    "peleles": "pelele",
    "pelemos": "pelar",
    "pelen": "pelar",
    "pelendengues": "pelendengue",
    "peleo": "pelear",
    "peleona": "peleón",
    "peleonas": "peleón",
    "peleones": "peleón",
    "peles": "pelar",
    "peletera": "peletero",
    "peleteras": "peletero",
    "peleteros": "peletero",
    "peleterías": "peletería",
    "peletes": "pelete",
    "peletizaciones": "peletización",
    "peleábamos": "pelear",
    "peleáis": "pelear",
    "peleáramos": "pelear",
    "peleáremos": "pelear",
    "peleásemos": "pelear",
    "peleé": "pelear",
    "peleéis": "pelear",
    "peleó": "pelear",
    "pelgares": "pelgar",
    "peliaguda": "peliagudo",
    "peliagudas": "peliagudo",
    "peliagudos": "peliagudo",
    "peliblanca": "peliblanco",
    "peliblancas": "peliblanco",
    "peliblancos": "peliblanco",
    "pelicana": "pelícano",
    "pelicanas": "pelícano",
    "pelicaniformes": "pelicaniforme",
    "pelicanos": "pelícano",
    "pelicorta": "pelicorto",
    "pelicortas": "pelicorto",
    "pelicortos": "pelicorto",
    "peliculares": "pelicular",
    "peliculera": "peliculero",
    "peliculeras": "peliculero",
    "peliculeros": "peliculero",
    "pelidura": "peliduro",
    "peliduras": "peliduro",
    "peliduros": "peliduro",
    "peligna": "peligno",
    "pelignas": "peligno",
    "pelignos": "peligno",
    "peligra": "peligrar",
    "peligraba": "peligrar",
    "peligrabais": "peligrar",
    "peligraban": "peligrar",
    "peligrabas": "peligrar",
    "peligrad": "peligrar",
    "peligrada": "peligrar",
    "peligradas": "peligrar",
    "peligrado": "peligrar",
    "peligrados": "peligrar",
    "peligramos": "peligrar",
    "peligran": "peligrar",
    "peligrando": "peligrar",
    "peligrara": "peligrar",
    "peligrarais": "peligrar",
    "peligraran": "peligrar",
    "peligraras": "peligrar",
    "peligrare": "peligrar",
    "peligrareis": "peligrar",
    "peligraremos": "peligrar",
    "peligraren": "peligrar",
    "peligrares": "peligrar",
    "peligraron": "peligrar",
    "peligrará": "peligrar",
    "peligrarán": "peligrar",
    "peligrarás": "peligrar",
    "peligraré": "peligrar",
    "peligraréis": "peligrar",
    "peligraría": "peligrar",
    "peligraríais": "peligrar",
    "peligraríamos": "peligrar",
    "peligrarían": "peligrar",
    "peligrarías": "peligrar",
    "peligras": "peligrar",
    "peligrase": "peligrar",
    "peligraseis": "peligrar",
    "peligrasen": "peligrar",
    "peligrases": "peligrar",
    "peligraste": "peligrar",
    "peligrasteis": "peligrar",
    "peligre": "peligrar",
    "peligremos": "peligrar",
    "peligren": "peligrar",
    "peligres": "peligrar",
    "peligro": "peligrar",
    "peligros": "peligro",
    "peligrosa": "peligroso",
    "peligrosas": "peligroso",
    "peligrosidades": "peligrosidad",
    "peligrosos": "peligroso",
    "peligrábamos": "peligrar",
    "peligráis": "peligrar",
    "peligráramos": "peligrar",
    "peligráremos": "peligrar",
    "peligrásemos": "peligrar",
    "peligré": "peligrar",
    "peligréis": "peligrar",
    "peligró": "peligrar",
    "pelilarga": "pelilargo",
    "pelilargas": "pelilargo",
    "pelilargos": "pelilargo",
    "pelillos": "pelillo",
    "pelillosa": "pelilloso",
    "pelillosas": "pelilloso",
    "pelillosos": "pelilloso",
    "pelinegra": "pelinegro",
    "pelinegras": "pelinegro",
    "pelinegros": "pelinegro",
    "pelirroja": "pelirrojo",
    "pelirrojas": "pelirrojo",
    "pelirrojos": "pelirrojo",
    "pelirrubia": "pelirrubio",
    "pelirrubias": "pelirrubio",
    "pelirrubios": "pelirrubio",
    "pelitiesa": "pelitieso",
    "pelitiesas": "pelitieso",
    "pelitiesos": "pelitieso",
    "pelitres": "pelitre",
    "pelitriques": "pelitrique",
    "pelladas": "pellada",
    "pellas": "pella",
    "pelleja": "pellejo",
    "pellejas": "pellejo",
    "pellejera": "pellejero",
    "pellejeras": "pellejero",
    "pellejeros": "pellejero",
    "pellejerías": "pellejería",
    "pellejos": "pellejo",
    "pellejuda": "pellejudo",
    "pellejudas": "pellejudo",
    "pellejudos": "pellejudo",
    "pelletas": "pelleta",
    "pellicas": "pellica",
    "pellicos": "pellico",
    "pelliquera": "pelliquero",
    "pelliqueras": "pelliquero",
    "pelliqueros": "pelliquero",
    "pellizas": "pelliza",
    "pellizca": "pellizcar",
    "pellizcaba": "pellizcar",
    "pellizcabais": "pellizcar",
    "pellizcaban": "pellizcar",
    "pellizcabas": "pellizcar",
    "pellizcad": "pellizcar",
    "pellizcada": "pellizcar",
    "pellizcadas": "pellizcar",
    "pellizcado": "pellizcar",
    "pellizcadora": "pellizcador",
    "pellizcadoras": "pellizcador",
    "pellizcadores": "pellizcador",
    "pellizcados": "pellizcar",
    "pellizcamos": "pellizcar",
    "pellizcan": "pellizcar",
    "pellizcando": "pellizcar",
    "pellizcara": "pellizcar",
    "pellizcarais": "pellizcar",
    "pellizcaran": "pellizcar",
    "pellizcaras": "pellizcar",
    "pellizcare": "pellizcar",
    "pellizcareis": "pellizcar",
    "pellizcaremos": "pellizcar",
    "pellizcaren": "pellizcar",
    "pellizcares": "pellizcar",
    "pellizcaron": "pellizcar",
    "pellizcará": "pellizcar",
    "pellizcarán": "pellizcar",
    "pellizcarás": "pellizcar",
    "pellizcaré": "pellizcar",
    "pellizcaréis": "pellizcar",
    "pellizcaría": "pellizcar",
    "pellizcaríais": "pellizcar",
    "pellizcaríamos": "pellizcar",
    "pellizcarían": "pellizcar",
    "pellizcarías": "pellizcar",
    "pellizcas": "pellizcar",
    "pellizcase": "pellizcar",
    "pellizcaseis": "pellizcar",
    "pellizcasen": "pellizcar",
    "pellizcases": "pellizcar",
    "pellizcaste": "pellizcar",
    "pellizcasteis": "pellizcar",
    "pellizco": "pellizcar",
    "pellizcos": "pellizco",
    "pellizcábamos": "pellizcar",
    "pellizcáis": "pellizcar",
    "pellizcáramos": "pellizcar",
    "pellizcáremos": "pellizcar",
    "pellizcásemos": "pellizcar",
    "pellizcó": "pellizcar",
    "pellizque": "pellizcar",
    "pellizquemos": "pellizcar",
    "pellizquen": "pellizcar",
    "pellizques": "pellizcar",
    "pellizqué": "pellizcar",
    "pellizquéis": "pellizcar",
    "pellones": "pellón",
    "pellos": "pello",
    "pellotes": "pellote",
    "pelluzgones": "pelluzgón",
    "pelmas": "pelma",
    "pelmatozoos": "pelmatozoo",
    "pelmaza": "pelmazo",
    "pelmazas": "pelmazo",
    "pelmazos": "pelmazo",
    "pelo": "pelar",
    "pelona": "pelón",
    "pelonas": "pelón",
    "pelonerías": "pelonería",
    "pelones": "pelón",
    "pelonías": "pelonía",
    "peloponenses": "peloponense",
    "peloponesiaca": "peloponesíaco",
    "peloponesiacas": "peloponesíaco",
    "peloponesiacos": "peloponesíaco",
    "peloponesíaca": "peloponesíaco",
    "peloponesíacas": "peloponesíaco",
    "peloponesíacos": "peloponesíaco",
    "pelos": "pelo",
    "pelosa": "peloso",
    "pelosas": "peloso",
    "pelosillas": "pelosilla",
    "pelosoles": "pelosol",
    "pelosos": "peloso",
    "pelotaris": "pelotari",
    "pelotas": "pelota",
    "pelotazos": "pelotazo",
    "pelotea": "pelotear",
    "peloteaba": "pelotear",
    "peloteabais": "pelotear",
    "peloteaban": "pelotear",
    "peloteabas": "pelotear",
    "pelotead": "pelotear",
    "peloteada": "pelotear",
    "peloteadas": "pelotear",
    "peloteado": "pelotear",
    "peloteados": "pelotear",
    "peloteamos": "pelotear",
    "pelotean": "pelotear",
    "peloteando": "pelotear",
    "peloteara": "pelotear",
    "pelotearais": "pelotear",
    "pelotearan": "pelotear",
    "pelotearas": "pelotear",
    "peloteare": "pelotear",
    "peloteareis": "pelotear",
    "pelotearemos": "pelotear",
    "pelotearen": "pelotear",
    "peloteares": "pelotear",
    "pelotearon": "pelotear",
    "peloteará": "pelotear",
    "pelotearán": "pelotear",
    "pelotearás": "pelotear",
    "pelotearé": "pelotear",
    "pelotearéis": "pelotear",
    "pelotearía": "pelotear",
    "pelotearíais": "pelotear",
    "pelotearíamos": "pelotear",
    "pelotearían": "pelotear",
    "pelotearías": "pelotear",
    "peloteas": "pelotear",
    "pelotease": "pelotear",
    "peloteaseis": "pelotear",
    "peloteasen": "pelotear",
    "peloteases": "pelotear",
    "peloteaste": "pelotear",
    "peloteasteis": "pelotear",
    "pelotee": "pelotear",
    "peloteemos": "pelotear",
    "peloteen": "pelotear",
    "pelotees": "pelotear",
    "peloteo": "pelotear",
    "peloteos": "peloteo",
    "pelotera": "pelotero",
    "peloteras": "pelotero",
    "peloteros": "pelotero",
    "peloterías": "pelotería",
    "pelotes": "pelote",
    "peloteábamos": "pelotear",
    "peloteáis": "pelotear",
    "peloteáramos": "pelotear",
    "peloteáremos": "pelotear",
    "peloteásemos": "pelotear",
    "peloteé": "pelotear",
    "peloteéis": "pelotear",
    "peloteó": "pelotear",
    "pelotillas": "pelotilla",
    "pelotillera": "pelotillero",
    "pelotilleras": "pelotillero",
    "pelotilleros": "pelotillero",
    "pelotones": "pelotón",
    "pelouses": "pelouse",
    "peltada": "peltado",
    "peltadas": "peltado",
    "peltados": "peltado",
    "peltas": "pelta",
    "peltastas": "peltasta",
    "peltres": "peltre",
    "pelucas": "peluca",
    "peluches": "peluche",
    "pelucones": "pelucón",
    "peluda": "peludo",
    "peludas": "peludo",
    "peludos": "peludo",
    "peluquera": "peluquero",
    "peluqueras": "peluquero",
    "peluqueros": "peluquero",
    "peluquerías": "peluquería",
    "peluquines": "peluquín",
    "pelusas": "pelusa",
    "pelusillas": "pelusilla",
    "pelviana": "pelviano",
    "pelvianas": "pelviano",
    "pelvianos": "pelviano",
    "pelvímetros": "pelvímetro",
    "pelábamos": "pelar",
    "pelágica": "pelágico",
    "pelágicas": "pelágico",
    "pelágicos": "pelágico",
    "peláis": "pelar",
    "pelámenes": "pelamen",
    "peláramos": "pelar",
    "peláremos": "pelar",
    "pelásemos": "pelar",
    "pelásgica": "pelásgico",
    "pelásgicas": "pelásgico",
    "pelásgicos": "pelásgico",
    "pelé": "pelar",
    "peléis": "pelar",
    "pelícanos": "pelícano",
    "películas": "película",
    "peló": "pelar",
    "pelúcida": "pelúcido",
    "pelúcidas": "pelúcido",
    "pelúcidos": "pelúcido",
    "pena": "penar",
    "penaba": "penar",
    "penabais": "penar",
    "penaban": "penar",
    "penabas": "penar",
    "penables": "penable",
    "penacheras": "penachera",
    "penachos": "penacho",
    "penachuda": "penachudo",
    "penachudas": "penachudo",
    "penachudos": "penachudo",
    "penad": "penar",
    "penada": "penar",
    "penadas": "penar",
    "penadillas": "penadilla",
    "penado": "penar",
    "penados": "penar",
    "penales": "penal",
    "penalice": "penalizar",
    "penalicemos": "penalizar",
    "penalicen": "penalizar",
    "penalices": "penalizar",
    "penalicé": "penalizar",
    "penalicéis": "penalizar",
    "penalidades": "penalidad",
    "penalistas": "penalista",
    "penaliza": "penalizar",
    "penalizaba": "penalizar",
    "penalizabais": "penalizar",
    "penalizaban": "penalizar",
    "penalizabas": "penalizar",
    "penalizaciones": "penalización",
    "penalizad": "penalizar",
    "penalizada": "penalizar",
    "penalizadas": "penalizar",
    "penalizado": "penalizar",
    "penalizados": "penalizar",
    "penalizamos": "penalizar",
    "penalizan": "penalizar",
    "penalizando": "penalizar",
    "penalizara": "penalizar",
    "penalizarais": "penalizar",
    "penalizaran": "penalizar",
    "penalizaras": "penalizar",
    "penalizare": "penalizar",
    "penalizareis": "penalizar",
    "penalizaremos": "penalizar",
    "penalizaren": "penalizar",
    "penalizares": "penalizar",
    "penalizaron": "penalizar",
    "penalizará": "penalizar",
    "penalizarán": "penalizar",
    "penalizarás": "penalizar",
    "penalizaré": "penalizar",
    "penalizaréis": "penalizar",
    "penalizaría": "penalizar",
    "penalizaríais": "penalizar",
    "penalizaríamos": "penalizar",
    "penalizarían": "penalizar",
    "penalizarías": "penalizar",
    "penalizas": "penalizar",
    "penalizase": "penalizar",
    "penalizaseis": "penalizar",
    "penalizasen": "penalizar",
    "penalizases": "penalizar",
    "penalizaste": "penalizar",
    "penalizasteis": "penalizar",
    "penalizo": "penalizar",
    "penalizábamos": "penalizar",
    "penalizáis": "penalizar",
    "penalizáramos": "penalizar",
    "penalizáremos": "penalizar",
    "penalizásemos": "penalizar",
    "penalizó": "penalizar",
    "penaltis": "penalti",
    "penamos": "penar",
    "penan": "penar",
    "penando": "penar",
    "penara": "penar",
    "penarais": "penar",
    "penaran": "penar",
    "penaras": "penar",
    "penare": "penar",
    "penareis": "penar",
    "penaremos": "penar",
    "penaren": "penar",
    "penares": "penar",
    "penaron": "penar",
    "penará": "penar",
    "penarán": "penar",
    "penarás": "penar",
    "penaré": "penar",
    "penaréis": "penar",
    "penaría": "penar",
    "penaríais": "penar",
    "penaríamos": "penar",
    "penarían": "penar",
    "penarías": "penar",
    "penas": "penar",
    "penase": "penar",
    "penaseis": "penar",
    "penasen": "penar",
    "penases": "penar",
    "penaste": "penar",
    "penasteis": "penar",
    "penbutololes": "penbutolol",
    "pencas": "penca",
    "pencazos": "pencazo",
    "pencos": "penco",
    "penda": "pender",
    "pendamos": "pender",
    "pendan": "pender",
    "pendas": "pender",
    "pende": "pender",
    "pended": "pender",
    "pendeja": "pendejo",
    "pendejas": "pendejo",
    "pendejos": "pendejo",
    "pendemos": "pender",
    "penden": "pender",
    "pendencia": "pendenciar",
    "pendenciaba": "pendenciar",
    "pendenciabais": "pendenciar",
    "pendenciaban": "pendenciar",
    "pendenciabas": "pendenciar",
    "pendenciad": "pendenciar",
    "pendenciada": "pendenciar",
    "pendenciadas": "pendenciar",
    "pendenciado": "pendenciar",
    "pendenciados": "pendenciar",
    "pendenciamos": "pendenciar",
    "pendencian": "pendenciar",
    "pendenciando": "pendenciar",
    "pendenciara": "pendenciar",
    "pendenciarais": "pendenciar",
    "pendenciaran": "pendenciar",
    "pendenciaras": "pendenciar",
    "pendenciare": "pendenciar",
    "pendenciareis": "pendenciar",
    "pendenciaremos": "pendenciar",
    "pendenciaren": "pendenciar",
    "pendenciares": "pendenciar",
    "pendenciaron": "pendenciar",
    "pendenciará": "pendenciar",
    "pendenciarán": "pendenciar",
    "pendenciarás": "pendenciar",
    "pendenciaré": "pendenciar",
    "pendenciaréis": "pendenciar",
    "pendenciaría": "pendenciar",
    "pendenciaríais": "pendenciar",
    "pendenciaríamos": "pendenciar",
    "pendenciarían": "pendenciar",
    "pendenciarías": "pendenciar",
    "pendencias": "pendenciar",
    "pendenciase": "pendenciar",
    "pendenciaseis": "pendenciar",
    "pendenciasen": "pendenciar",
    "pendenciases": "pendenciar",
    "pendenciaste": "pendenciar",
    "pendenciasteis": "pendenciar",
    "pendencie": "pendenciar",
    "pendenciemos": "pendenciar",
    "pendencien": "pendenciar",
    "pendenciera": "pendenciero",
    "pendencieras": "pendenciero",
    "pendencieros": "pendenciero",
    "pendencies": "pendenciar",
    "pendencio": "pendenciar",
    "pendenciábamos": "pendenciar",
    "pendenciáis": "pendenciar",
    "pendenciáramos": "pendenciar",
    "pendenciáremos": "pendenciar",
    "pendenciásemos": "pendenciar",
    "pendencié": "pendenciar",
    "pendenciéis": "pendenciar",
    "pendenció": "pendenciar",
    "penderemos": "pender",
    "penderá": "pender",
    "penderán": "pender",
    "penderás": "pender",
    "penderé": "pender",
    "penderéis": "pender",
    "pendería": "pender",
    "penderíais": "pender",
    "penderíamos": "pender",
    "penderían": "pender",
    "penderías": "pender",
    "pendes": "pender",
    "pendida": "pender",
    "pendidas": "pender",
    "pendido": "pender",
    "pendidos": "pender",
    "pendiendo": "pender",
    "pendientes": "pendiente",
    "pendiera": "pender",
    "pendierais": "pender",
    "pendieran": "pender",
    "pendieras": "pender",
    "pendiere": "pender",
    "pendiereis": "pender",
    "pendieren": "pender",
    "pendieres": "pender",
    "pendieron": "pender",
    "pendiese": "pender",
    "pendieseis": "pender",
    "pendiesen": "pender",
    "pendieses": "pender",
    "pendiles": "pendil",
    "pendimos": "pender",
    "pendiste": "pender",
    "pendisteis": "pender",
    "pendiéramos": "pender",
    "pendiéremos": "pender",
    "pendiésemos": "pender",
    "pendió": "pender",
    "pendo": "pender",
    "pendolajes": "pendolaje",
    "pendoles": "pendol",
    "pendolistas": "pendolista",
    "pendolones": "pendolón",
    "pendona": "pendón",
    "pendonas": "pendón",
    "pendonea": "pendonear",
    "pendoneaba": "pendonear",
    "pendoneabais": "pendonear",
    "pendoneaban": "pendonear",
    "pendoneabas": "pendonear",
    "pendonead": "pendonear",
    "pendoneada": "pendonear",
    "pendoneadas": "pendonear",
    "pendoneado": "pendonear",
    "pendoneados": "pendonear",
    "pendoneamos": "pendonear",
    "pendonean": "pendonear",
    "pendoneando": "pendonear",
    "pendoneara": "pendonear",
    "pendonearais": "pendonear",
    "pendonearan": "pendonear",
    "pendonearas": "pendonear",
    "pendoneare": "pendonear",
    "pendoneareis": "pendonear",
    "pendonearemos": "pendonear",
    "pendonearen": "pendonear",
    "pendoneares": "pendonear",
    "pendonearon": "pendonear",
    "pendoneará": "pendonear",
    "pendonearán": "pendonear",
    "pendonearás": "pendonear",
    "pendonearé": "pendonear",
    "pendonearéis": "pendonear",
    "pendonearía": "pendonear",
    "pendonearíais": "pendonear",
    "pendonearíamos": "pendonear",
    "pendonearían": "pendonear",
    "pendonearías": "pendonear",
    "pendoneas": "pendonear",
    "pendonease": "pendonear",
    "pendoneaseis": "pendonear",
    "pendoneasen": "pendonear",
    "pendoneases": "pendonear",
    "pendoneaste": "pendonear",
    "pendoneasteis": "pendonear",
    "pendonee": "pendonear",
    "pendoneemos": "pendonear",
    "pendoneen": "pendonear",
    "pendonees": "pendonear",
    "pendoneo": "pendonear",
    "pendoneos": "pendoneo",
    "pendones": "pendón",
    "pendoneábamos": "pendonear",
    "pendoneáis": "pendonear",
    "pendoneáramos": "pendonear",
    "pendoneáremos": "pendonear",
    "pendoneásemos": "pendonear",
    "pendoneé": "pendonear",
    "pendoneéis": "pendonear",
    "pendoneó": "pendonear",
    "pendonistas": "pendonista",
    "pendonos": "pendón",
    "pendulares": "pendular",
    "penduleos": "penduleo",
    "pendáis": "pender",
    "pendéis": "pender",
    "pendí": "pender",
    "pendía": "pender",
    "pendíais": "pender",
    "pendíamos": "pender",
    "pendían": "pender",
    "pendías": "pender",
    "pene": "penar",
    "peneana": "peneano",
    "peneanas": "peneano",
    "peneanos": "peneano",
    "penemos": "penar",
    "penen": "penar",
    "peneques": "peneque",
    "penes": "pene",
    "penetra": "penetrar",
    "penetraba": "penetrar",
    "penetrabais": "penetrar",
    "penetraban": "penetrar",
    "penetrabas": "penetrar",
    "penetrabilidades": "penetrabilidad",
    "penetrables": "penetrable",
    "penetraciones": "penetración",
    "penetrad": "penetrar",
    "penetrada": "penetrar",
    "penetradas": "penetrar",
    "penetrado": "penetrar",
    "penetradora": "penetrador",
    "penetradoras": "penetrador",
    "penetradores": "penetrador",
    "penetrados": "penetrar",
    "penetramos": "penetrar",
    "penetran": "penetrar",
    "penetrando": "penetrar",
    "penetrantes": "penetrante",
    "penetrara": "penetrar",
    "penetrarais": "penetrar",
    "penetraran": "penetrar",
    "penetraras": "penetrar",
    "penetrare": "penetrar",
    "penetrareis": "penetrar",
    "penetraremos": "penetrar",
    "penetraren": "penetrar",
    "penetrares": "penetrar",
    "penetraron": "penetrar",
    "penetrará": "penetrar",
    "penetrarán": "penetrar",
    "penetrarás": "penetrar",
    "penetraré": "penetrar",
    "penetraréis": "penetrar",
    "penetraría": "penetrar",
    "penetraríais": "penetrar",
    "penetraríamos": "penetrar",
    "penetrarían": "penetrar",
    "penetrarías": "penetrar",
    "penetras": "penetrar",
    "penetrase": "penetrar",
    "penetraseis": "penetrar",
    "penetrasen": "penetrar",
    "penetrases": "penetrar",
    "penetraste": "penetrar",
    "penetrasteis": "penetrar",
    "penetrativa": "penetrativo",
    "penetrativas": "penetrativo",
    "penetrativos": "penetrativo",
    "penetre": "penetrar",
    "penetremos": "penetrar",
    "penetren": "penetrar",
    "penetres": "penetrar",
    "penetro": "penetrar",
    "penetrábamos": "penetrar",
    "penetráis": "penetrar",
    "penetráramos": "penetrar",
    "penetráremos": "penetrar",
    "penetrásemos": "penetrar",
    "penetré": "penetrar",
    "penetréis": "penetrar",
    "penetró": "penetrar",
    "penetrómetros": "penetrómetro",
    "peneuvistas": "peneuvista",
    "penibética": "penibético",
    "penibéticas": "penibético",
    "penibéticos": "penibético",
    "penicilada": "penicilado",
    "peniciladas": "penicilado",
    "penicilados": "penicilado",
    "penicilinas": "penicilina",
    "penicilinasas": "penicilinasa",
    "peniformes": "peniforme",
    "penillanuras": "penillanura",
    "peninsulares": "peninsular",
    "peninsulars": "peninsular",
    "peniques": "penique",
    "penitencia": "penitenciar",
    "penitenciaba": "penitenciar",
    "penitenciabais": "penitenciar",
    "penitenciaban": "penitenciar",
    "penitenciabas": "penitenciar",
    "penitenciad": "penitenciar",
    "penitenciada": "penitenciar",
    "penitenciadas": "penitenciar",
    "penitenciado": "penitenciar",
    "penitenciados": "penitenciar",
    "penitenciales": "penitencial",
    "penitenciamos": "penitenciar",
    "penitencian": "penitenciar",
    "penitenciando": "penitenciar",
    "penitenciara": "penitenciar",
    "penitenciarais": "penitenciar",
    "penitenciaran": "penitenciar",
    "penitenciaras": "penitenciar",
    "penitenciare": "penitenciar",
    "penitenciareis": "penitenciar",
    "penitenciaremos": "penitenciar",
    "penitenciaren": "penitenciar",
    "penitenciares": "penitenciar",
    "penitenciaria": "penitenciario",
    "penitenciarias": "penitenciario",
    "penitenciarios": "penitenciario",
    "penitenciarismos": "penitenciarismo",
    "penitenciaron": "penitenciar",
    "penitenciará": "penitenciar",
    "penitenciarán": "penitenciar",
    "penitenciarás": "penitenciar",
    "penitenciaré": "penitenciar",
    "penitenciaréis": "penitenciar",
    "penitenciaría": "penitenciar",
    "penitenciaríais": "penitenciar",
    "penitenciaríamos": "penitenciar",
    "penitenciarían": "penitenciar",
    "penitenciarías": "penitenciaría",
    "penitencias": "penitenciar",
    "penitenciase": "penitenciar",
    "penitenciaseis": "penitenciar",
    "penitenciasen": "penitenciar",
    "penitenciases": "penitenciar",
    "penitenciaste": "penitenciar",
    "penitenciasteis": "penitenciar",
    "penitencie": "penitenciar",
    "penitenciemos": "penitenciar",
    "penitencien": "penitenciar",
    "penitencies": "penitenciar",
    "penitencio": "penitenciar",
    "penitenciábamos": "penitenciar",
    "penitenciáis": "penitenciar",
    "penitenciáramos": "penitenciar",
    "penitenciáremos": "penitenciar",
    "penitenciásemos": "penitenciar",
    "penitencié": "penitenciar",
    "penitenciéis": "penitenciar",
    "penitenció": "penitenciar",
    "penitentes": "penitente",
    "peno": "penar",
    "penoles": "penol",
    "penologías": "penología",
    "penológica": "penológico",
    "penológicas": "penológico",
    "penológicos": "penológico",
    "penosa": "penoso",
    "penosas": "penoso",
    "penosos": "penoso",
    "pensaba": "pensar",
    "pensabais": "pensar",
    "pensaban": "pensar",
    "pensabas": "pensar",
    "pensad": "pensar",
    "pensada": "pensar",
    "pensadas": "pensar",
    "pensado": "pensar",
    "pensadora": "pensador",
    "pensadoras": "pensador",
    "pensadores": "pensador",
    "pensados": "pensar",
    "pensamientos": "pensamiento",
    "pensamos": "pensar",
    "pensando": "pensar",
    "pensantes": "pensante",
    "pensara": "pensar",
    "pensarais": "pensar",
    "pensaran": "pensar",
    "pensaras": "pensar",
    "pensare": "pensar",
    "pensareis": "pensar",
    "pensaremos": "pensar",
    "pensaren": "pensar",
    "pensares": "pensar",
    "pensaron": "pensar",
    "pensará": "pensar",
    "pensarán": "pensar",
    "pensarás": "pensar",
    "pensaré": "pensar",
    "pensaréis": "pensar",
    "pensaría": "pensar",
    "pensaríais": "pensar",
    "pensaríamos": "pensar",
    "pensarían": "pensar",
    "pensarías": "pensar",
    "pensase": "pensar",
    "pensaseis": "pensar",
    "pensasen": "pensar",
    "pensases": "pensar",
    "pensaste": "pensar",
    "pensasteis": "pensar",
    "pensativa": "pensativo",
    "pensativas": "pensativo",
    "pensativos": "pensativo",
    "penseles": "pensel",
    "pensemos": "pensar",
    "penseques": "penseque",
    "pensiles": "pensil",
    "pensilvana": "pensilvano",
    "pensilvanas": "pensilvano",
    "pensilvanos": "pensilvano",
    "pensiona": "pensionar",
    "pensionaba": "pensionar",
    "pensionabais": "pensionar",
    "pensionaban": "pensionar",
    "pensionabas": "pensionar",
    "pensionad": "pensionar",
    "pensionada": "pensionar",
    "pensionadas": "pensionar",
    "pensionado": "pensionar",
    "pensionados": "pensionar",
    "pensionamos": "pensionar",
    "pensionan": "pensionar",
    "pensionando": "pensionar",
    "pensionara": "pensionar",
    "pensionarais": "pensionar",
    "pensionaran": "pensionar",
    "pensionaras": "pensionar",
    "pensionare": "pensionar",
    "pensionareis": "pensionar",
    "pensionaremos": "pensionar",
    "pensionaren": "pensionar",
    "pensionares": "pensionar",
    "pensionarios": "pensionario",
    "pensionaron": "pensionar",
    "pensionará": "pensionar",
    "pensionarán": "pensionar",
    "pensionarás": "pensionar",
    "pensionaré": "pensionar",
    "pensionaréis": "pensionar",
    "pensionaría": "pensionar",
    "pensionaríais": "pensionar",
    "pensionaríamos": "pensionar",
    "pensionarían": "pensionar",
    "pensionarías": "pensionar",
    "pensionas": "pensionar",
    "pensionase": "pensionar",
    "pensionaseis": "pensionar",
    "pensionasen": "pensionar",
    "pensionases": "pensionar",
    "pensionaste": "pensionar",
    "pensionasteis": "pensionar",
    "pensione": "pensionar",
    "pensionemos": "pensionar",
    "pensionen": "pensionar",
    "pensiones": "pensionar",
    "pensionistas": "pensionista",
    "pensiono": "pensionar",
    "pensionábamos": "pensionar",
    "pensionáis": "pensionar",
    "pensionáramos": "pensionar",
    "pensionáremos": "pensionar",
    "pensionásemos": "pensionar",
    "pensioné": "pensionar",
    "pensionéis": "pensionar",
    "pensionó": "pensionar",
    "pensábamos": "pensar",
    "pensáis": "pensar",
    "pensáramos": "pensar",
    "pensáremos": "pensar",
    "pensásemos": "pensar",
    "pensé": "pensar",
    "penséis": "pensar",
    "pensó": "pensar",
    "pentaboranos": "pentaborano",
    "pentaboratos": "pentaborato",
    "pentabromuros": "pentabromuro",
    "pentacampeona": "pentacampeón",
    "pentacampeonas": "pentacampeón",
    "pentacampeones": "pentacampeón",
    "pentacloroestearatos": "pentacloroestearato",
    "pentacloroetanos": "pentacloroetano",
    "pentaclorofenatos": "pentaclorofenato",
    "pentaclorofenoles": "pentaclorofenol",
    "pentacloronitrobencenos": "pentacloronitrobenceno",
    "pentacloruros": "pentacloruro",
    "pentacordios": "pentacordio",
    "pentacuspidada": "pentacuspidado",
    "pentacuspidadas": "pentacuspidado",
    "pentacuspidados": "pentacuspidado",
    "pentacíclicas": "pentacíclica",
    "pentadecanoatos": "pentadecanoato",
    "pentadecanoica": "pentadecanoico",
    "pentadecanoicas": "pentadecanoico",
    "pentadecanoicos": "pentadecanoico",
    "pentadecágonos": "pentadecágono",
    "pentadodecaedros": "pentadodecaedro",
    "pentaedros": "pentaedro",
    "pentaeritritas": "pentaeritrita",
    "pentafilares": "pentafilar",
    "pentafluoruros": "pentafluoruro",
    "pentafonías": "pentafonía",
    "pentafásica": "pentafásico",
    "pentafásicas": "pentafásico",
    "pentafásicos": "pentafásico",
    "pentagonales": "pentagonal",
    "pentagramas": "pentagrama",
    "pentalfas": "pentalfa",
    "pentanodioles": "pentanodiol",
    "pentanoles": "pentanol",
    "pentanos": "pentano",
    "pentapolares": "pentapolar",
    "pentapolitana": "pentapolitano",
    "pentapolitanas": "pentapolitano",
    "pentapolitanos": "pentapolitano",
    "pentarcas": "pentarca",
    "pentarquías": "pentarquía",
    "pentas": "penta",
    "pentastómidos": "pentastómido",
    "pentasulfuros": "pentasulfuro",
    "pentasílaba": "pentasílabo",
    "pentasílabas": "pentasílabo",
    "pentasílabos": "pentasílabo",
    "pentasómica": "pentasómico",
    "pentasómicas": "pentasómico",
    "pentasómicos": "pentasómico",
    "pentathlones": "pentathlon",
    "pentathlons": "pentathlon",
    "pentatletas": "pentatleta",
    "pentatlones": "pentatlón",
    "pentatloniana": "pentatloniano",
    "pentatlonianas": "pentatloniano",
    "pentatlonianos": "pentatloniano",
    "pentatriacontanos": "pentatriacontano",
    "pentatónica": "pentatónico",
    "pentatónicas": "pentatónico",
    "pentatónicos": "pentatónico",
    "pentavalentes": "pentavalente",
    "pentilentetrazoles": "pentilentetrazol",
    "pentlanditas": "pentlandita",
    "pentobarbitales": "pentobarbital",
    "pentoda": "pentodo",
    "pentodas": "pentodo",
    "pentodos": "pentodo",
    "pentolitas": "pentolita",
    "pentosas": "pentosa",
    "pentotales": "pentotal",
    "pentágona": "pentágono",
    "pentágonas": "pentágono",
    "pentágonos": "pentágono",
    "pentámera": "pentámero",
    "pentámeras": "pentámero",
    "pentámeros": "pentámero",
    "pentámetros": "pentámetro",
    "pentárquica": "pentárquico",
    "pentárquicas": "pentárquico",
    "pentárquicos": "pentárquico",
    "pentélica": "pentélico",
    "pentélicas": "pentélico",
    "pentélicos": "pentélico",
    "pentóxidos": "pentóxido",
    "penumbras": "penumbra",
    "penumbrosa": "penumbroso",
    "penumbrosas": "penumbroso",
    "penumbrosos": "penumbroso",
    "penurias": "penuria",
    "penábamos": "penar",
    "penáis": "penar",
    "penáramos": "penar",
    "penáremos": "penar",
    "penásemos": "penar",
    "pené": "penar",
    "penéis": "penar",
    "penígera": "penígero",
    "penígeras": "penígero",
    "penígeros": "penígero",
    "penínsulas": "península",
    "penó": "penar",
    "penúltima": "penúltimo",
    "penúltimas": "penúltimo",
    "penúltimos": "penúltimo",
    "peo": "peer",
    "peonadas": "peonada",
    "peonajes": "peonaje",
    "peonerías": "peonería",
    "peones": "peón",
    "peonias": "peonia",
    "peonzas": "peonza",
    "peonías": "peonía",
    "peores": "peor",
    "peorías": "peoría",
    "pepes": "pepe",
    "pepinares": "pepinar",
    "pepinillos": "pepinillo",
    "pepinos": "pepino",
    "pepitas": "pepita",
    "pepitorias": "pepitoria",
    "pepitos": "pepito",
    "pepitosa": "pepitoso",
    "pepitosas": "pepitoso",
    "pepitosos": "pepitoso",
    "peplos": "peplo",
    "peponas": "pepona",
    "pepones": "pepón",
    "pepsinas": "pepsina",
    "peptidasas": "peptidasa",
    "peptizaciones": "peptización",
    "peptonas": "peptona",
    "peptídica": "peptídico",
    "peptídicas": "peptídico",
    "peptídicos": "peptídico",
    "pepónidas": "pepónida",
    "pepónides": "pepónide",
    "peque": "pecar",
    "pequemos": "pecar",
    "pequen": "pecar",
    "peques": "peque",
    "pequeña": "pequeño",
    "pequeñas": "pequeño",
    "pequeñeces": "pequeñez",
    "pequeñoburguesa": "pequeñoburgués",
    "pequeñoburguesas": "pequeñoburgués",
    "pequeñoburgueses": "pequeñoburgués",
    "pequeños": "pequeño",
    "pequines": "pequín",
    "pequinesa": "pequinés",
    "pequinesas": "pequinés",
    "pequineses": "pequinés",
    "pequé": "pecar",
    "pequéis": "pecar",
    "peradas": "perada",
    "peraledas": "peraleda",
    "perales": "peral",
    "peralta": "peraltar",
    "peraltaba": "peraltar",
    "peraltabais": "peraltar",
    "peraltaban": "peraltar",
    "peraltabas": "peraltar",
    "peraltad": "peraltar",
    "peraltada": "peraltar",
    "peraltadas": "peraltar",
    "peraltado": "peraltar",
    "peraltados": "peraltar",
    "peraltamos": "peraltar",
    "peraltan": "peraltar",
    "peraltando": "peraltar",
    "peraltara": "peraltar",
    "peraltarais": "peraltar",
    "peraltaran": "peraltar",
    "peraltaras": "peraltar",
    "peraltare": "peraltar",
    "peraltareis": "peraltar",
    "peraltaremos": "peraltar",
    "peraltaren": "peraltar",
    "peraltares": "peraltar",
    "peraltaron": "peraltar",
    "peraltará": "peraltar",
    "peraltarán": "peraltar",
    "peraltarás": "peraltar",
    "peraltaré": "peraltar",
    "peraltaréis": "peraltar",
    "peraltaría": "peraltar",
    "peraltaríais": "peraltar",
    "peraltaríamos": "peraltar",
    "peraltarían": "peraltar",
    "peraltarías": "peraltar",
    "peraltas": "peraltar",
    "peraltase": "peraltar",
    "peraltaseis": "peraltar",
    "peraltasen": "peraltar",
    "peraltases": "peraltar",
    "peraltaste": "peraltar",
    "peraltasteis": "peraltar",
    "peralte": "peraltar",
    "peraltemos": "peraltar",
    "peralten": "peraltar",
    "peraltes": "peralte",
    "peralto": "peraltar",
    "peraltos": "peralto",
    "peraltábamos": "peraltar",
    "peraltáis": "peraltar",
    "peraltáramos": "peraltar",
    "peraltáremos": "peraltar",
    "peraltásemos": "peraltar",
    "peralté": "peraltar",
    "peraltéis": "peraltar",
    "peraltó": "peraltar",
    "perantones": "perantón",
    "peras": "pera",
    "perbenzoatos": "perbenzoato",
    "perboratos": "perborato",
    "perbrómicos": "perbrómico",
    "percales": "percal",
    "percalinas": "percalina",
    "percances": "percance",
    "percarbonatos": "percarbonato",
    "percas": "perca",
    "percata": "percatar",
    "percataba": "percatar",
    "percatabais": "percatar",
    "percataban": "percatar",
    "percatabas": "percatar",
    "percatad": "percatar",
    "percatada": "percatar",
    "percatadas": "percatar",
    "percatado": "percatar",
    "percatados": "percatar",
    "percatamos": "percatar",
    "percatan": "percatar",
    "percatando": "percatar",
    "percatara": "percatar",
    "percatarais": "percatar",
    "percataran": "percatar",
    "percataras": "percatar",
    "percatare": "percatar",
    "percatareis": "percatar",
    "percataremos": "percatar",
    "percataren": "percatar",
    "percatares": "percatar",
    "percataron": "percatar",
    "percatará": "percatar",
    "percatarán": "percatar",
    "percatarás": "percatar",
    "percataré": "percatar",
    "percataréis": "percatar",
    "percataría": "percatar",
    "percataríais": "percatar",
    "percataríamos": "percatar",
    "percatarían": "percatar",
    "percatarías": "percatar",
    "percatas": "percatar",
    "percatase": "percatar",
    "percataseis": "percatar",
    "percatasen": "percatar",
    "percatases": "percatar",
    "percataste": "percatar",
    "percatasteis": "percatar",
    "percate": "percatar",
    "percatemos": "percatar",
    "percaten": "percatar",
    "percates": "percatar",
    "percato": "percatar",
    "percatábamos": "percatar",
    "percatáis": "percatar",
    "percatáramos": "percatar",
    "percatáremos": "percatar",
    "percatásemos": "percatar",
    "percaté": "percatar",
    "percatéis": "percatar",
    "percató": "percatar",
    "percebes": "percebe",
    "percentilas": "percentila",
    "percentiles": "percentil",
    "percepciones": "percepción",
    "percepciónes": "percepción",
    "perceptibilidades": "perceptibilidad",
    "perceptibles": "perceptible",
    "perceptiva": "perceptivo",
    "perceptivas": "perceptivo",
    "perceptivos": "perceptivo",
    "perceptora": "perceptor",
    "perceptoras": "perceptor",
    "perceptores": "perceptor",
    "percha": "perchar",
    "perchaba": "perchar",
    "perchabais": "perchar",
    "perchaban": "perchar",
    "perchabas": "perchar",
    "perchad": "perchar",
    "perchada": "perchar",
    "perchadas": "perchar",
    "perchado": "perchar",
    "perchadoras": "perchadora",
    "perchados": "perchar",
    "perchamos": "perchar",
    "perchan": "perchar",
    "perchando": "perchar",
    "perchara": "perchar",
    "percharais": "perchar",
    "percharan": "perchar",
    "percharas": "perchar",
    "perchare": "perchar",
    "perchareis": "perchar",
    "percharemos": "perchar",
    "percharen": "perchar",
    "perchares": "perchar",
    "percharon": "perchar",
    "perchará": "perchar",
    "percharán": "perchar",
    "percharás": "perchar",
    "percharé": "perchar",
    "percharéis": "perchar",
    "percharía": "perchar",
    "percharíais": "perchar",
    "percharíamos": "perchar",
    "percharían": "perchar",
    "percharías": "perchar",
    "perchas": "perchar",
    "perchase": "perchar",
    "perchaseis": "perchar",
    "perchasen": "perchar",
    "perchases": "perchar",
    "perchaste": "perchar",
    "perchasteis": "perchar",
    "perche": "perchar",
    "percheles": "perchel",
    "perchemos": "perchar",
    "perchen": "perchar",
    "perchera": "perchero",
    "percheras": "perchero",
    "percherona": "percherón",
    "percheronas": "percherón",
    "percherones": "percherón",
    "percheros": "perchero",
    "perches": "perchar",
    "percho": "perchar",
    "perchones": "perchón",
    "perchábamos": "perchar",
    "percháis": "perchar",
    "percháramos": "perchar",
    "percháremos": "perchar",
    "perchásemos": "perchar",
    "perché": "perchar",
    "perchéis": "perchar",
    "perchó": "perchar",
    "perciba": "percibir",
    "percibamos": "percibir",
    "perciban": "percibir",
    "percibas": "percibir",
    "percibe": "percibir",
    "perciben": "percibir",
    "percibes": "percibir",
    "percibid": "percibir",
    "percibida": "percibir",
    "percibidas": "percibir",
    "percibido": "percibir",
    "percibidos": "percibir",
    "percibiendo": "percibir",
    "percibiera": "percibir",
    "percibierais": "percibir",
    "percibieran": "percibir",
    "percibieras": "percibir",
    "percibiere": "percibir",
    "percibiereis": "percibir",
    "percibieren": "percibir",
    "percibieres": "percibir",
    "percibieron": "percibir",
    "percibiese": "percibir",
    "percibieseis": "percibir",
    "percibiesen": "percibir",
    "percibieses": "percibir",
    "percibimos": "percibir",
    "percibiremos": "percibir",
    "percibirá": "percibir",
    "percibirán": "percibir",
    "percibirás": "percibir",
    "percibiré": "percibir",
    "percibiréis": "percibir",
    "percibiría": "percibir",
    "percibiríais": "percibir",
    "percibiríamos": "percibir",
    "percibirían": "percibir",
    "percibirías": "percibir",
    "percibiste": "percibir",
    "percibisteis": "percibir",
    "percibiéramos": "percibir",
    "percibiéremos": "percibir",
    "percibiésemos": "percibir",
    "percibió": "percibir",
    "percibo": "percibir",
    "percibos": "percibo",
    "percibáis": "percibir",
    "percibí": "percibir",
    "percibía": "percibir",
    "percibíais": "percibir",
    "percibíamos": "percibir",
    "percibían": "percibir",
    "percibías": "percibir",
    "percibís": "percibir",
    "perciformes": "perciforme",
    "percloratos": "perclorato",
    "percloroetilenos": "percloroetileno",
    "percloréteres": "percloréter",
    "perclórica": "perclórico",
    "perclóricas": "perclórico",
    "perclóricos": "perclórico",
    "percoceros": "percocero",
    "percocerías": "percocería",
    "percolaciones": "percolación",
    "percoladora": "percolador",
    "percoladoras": "percolador",
    "percoladores": "percolador",
    "percolantes": "percolante",
    "percrómica": "percrómico",
    "percrómicas": "percrómico",
    "percrómicos": "percrómico",
    "percuda": "percudir",
    "percudamos": "percudir",
    "percudan": "percudir",
    "percudas": "percudir",
    "percude": "percudir",
    "percuden": "percudir",
    "percudes": "percudir",
    "percudid": "percudir",
    "percudida": "percudir",
    "percudidas": "percudir",
    "percudido": "percudir",
    "percudidos": "percudir",
    "percudiendo": "percudir",
    "percudiera": "percudir",
    "percudierais": "percudir",
    "percudieran": "percudir",
    "percudieras": "percudir",
    "percudiere": "percudir",
    "percudiereis": "percudir",
    "percudieren": "percudir",
    "percudieres": "percudir",
    "percudieron": "percudir",
    "percudiese": "percudir",
    "percudieseis": "percudir",
    "percudiesen": "percudir",
    "percudieses": "percudir",
    "percudimos": "percudir",
    "percudiremos": "percudir",
    "percudirá": "percudir",
    "percudirán": "percudir",
    "percudirás": "percudir",
    "percudiré": "percudir",
    "percudiréis": "percudir",
    "percudiría": "percudir",
    "percudiríais": "percudir",
    "percudiríamos": "percudir",
    "percudirían": "percudir",
    "percudirías": "percudir",
    "percudiste": "percudir",
    "percudisteis": "percudir",
    "percudiéramos": "percudir",
    "percudiéremos": "percudir",
    "percudiésemos": "percudir",
    "percudió": "percudir",
    "percudo": "percudir",
    "percudáis": "percudir",
    "percudí": "percudir",
    "percudía": "percudir",
    "percudíais": "percudir",
    "percudíamos": "percudir",
    "percudían": "percudir",
    "percudías": "percudir",
    "percudís": "percudir",
    "percusiones": "percusión",
    "percusionistas": "percusionista",
    "percusiva": "percusivo",
    "percusivas": "percusivo",
    "percusivos": "percusivo",
    "percusora": "percusor",
    "percusoras": "percusor",
    "percusores": "percusor",
    "percuta": "percutir",
    "percutamos": "percutir",
    "percutan": "percutir",
    "percutantes": "percutante",
    "percutas": "percutir",
    "percute": "percutir",
    "percuten": "percutir",
    "percutes": "percutir",
    "percutid": "percutir",
    "percutida": "percutir",
    "percutidas": "percutir",
    "percutido": "percutir",
    "percutidos": "percutir",
    "percutiendo": "percutir",
    "percutiera": "percutir",
    "percutierais": "percutir",
    "percutieran": "percutir",
    "percutieras": "percutir",
    "percutiere": "percutir",
    "percutiereis": "percutir",
    "percutieren": "percutir",
    "percutieres": "percutir",
    "percutieron": "percutir",
    "percutiese": "percutir",
    "percutieseis": "percutir",
    "percutiesen": "percutir",
    "percutieses": "percutir",
    "percutimos": "percutir",
    "percutiremos": "percutir",
    "percutirá": "percutir",
    "percutirán": "percutir",
    "percutirás": "percutir",
    "percutiré": "percutir",
    "percutiréis": "percutir",
    "percutiría": "percutir",
    "percutiríais": "percutir",
    "percutiríamos": "percutir",
    "percutirían": "percutir",
    "percutirías": "percutir",
    "percutiste": "percutir",
    "percutisteis": "percutir",
    "percutiéramos": "percutir",
    "percutiéremos": "percutir",
    "percutiésemos": "percutir",
    "percutió": "percutir",
    "percuto": "percutir",
    "percutora": "percutor",
    "percutoras": "percutor",
    "percutores": "percutor",
    "percutáis": "percutir",
    "percutí": "percutir",
    "percutía": "percutir",
    "percutíais": "percutir",
    "percutíamos": "percutir",
    "percutían": "percutir",
    "percutías": "percutir",
    "percutís": "percutir",
    "perdamos": "perder",
    "perded": "perder",
    "perdederos": "perdedero",
    "perdedora": "perdedor",
    "perdedoras": "perdedor",
    "perdedores": "perdedor",
    "perdemos": "perder",
    "perderemos": "perder",
    "perderá": "perder",
    "perderán": "perder",
    "perderás": "perder",
    "perderé": "perder",
    "perderéis": "perder",
    "perdería": "perder",
    "perderíais": "perder",
    "perderíamos": "perder",
    "perderían": "perder",
    "perderías": "perder",
    "perdices": "perdiz",
    "perdiciones": "perdición",
    "perdida": "perdido",
    "perdidas": "perdido",
    "perdidiza": "perdidizo",
    "perdidizas": "perdidizo",
    "perdidizos": "perdidizo",
    "perdido": "perder",
    "perdidos": "perdido",
    "perdidosa": "perdidoso",
    "perdidosas": "perdidoso",
    "perdidosos": "perdidoso",
    "perdiendo": "perder",
    "perdiera": "perder",
    "perdierais": "perder",
    "perdieran": "perder",
    "perdieras": "perder",
    "perdiere": "perder",
    "perdiereis": "perder",
    "perdieren": "perder",
    "perdieres": "perder",
    "perdieron": "perder",
    "perdiese": "perder",
    "perdieseis": "perder",
    "perdiesen": "perder",
    "perdieses": "perder",
    "perdigonadas": "perdigonada",
    "perdigonazos": "perdigonazo",
    "perdigoneras": "perdigonera",
    "perdigones": "perdigón",
    "perdiguera": "perdiguero",
    "perdigueras": "perdiguero",
    "perdigueros": "perdiguero",
    "perdimentos": "perdimento",
    "perdimos": "perder",
    "perdiste": "perder",
    "perdisteis": "perder",
    "perdiéramos": "perder",
    "perdiéremos": "perder",
    "perdiésemos": "perder",
    "perdió": "perder",
    "perdona": "perdonar",
    "perdonaba": "perdonar",
    "perdonabais": "perdonar",
    "perdonaban": "perdonar",
    "perdonabas": "perdonar",
    "perdonables": "perdonable",
    "perdonad": "perdonar",
    "perdonada": "perdonar",
    "perdonadas": "perdonar",
    "perdonado": "perdonar",
    "perdonadora": "perdonador",
    "perdonadoras": "perdonador",
    "perdonadores": "perdonador",
    "perdonados": "perdonar",
    "perdonamos": "perdonar",
    "perdonan": "perdonar",
    "perdonando": "perdonar",
    "perdonara": "perdonar",
    "perdonarais": "perdonar",
    "perdonaran": "perdonar",
    "perdonaras": "perdonar",
    "perdonare": "perdonar",
    "perdonareis": "perdonar",
    "perdonaremos": "perdonar",
    "perdonaren": "perdonar",
    "perdonares": "perdonar",
    "perdonaron": "perdonar",
    "perdonará": "perdonar",
    "perdonarán": "perdonar",
    "perdonarás": "perdonar",
    "perdonaré": "perdonar",
    "perdonaréis": "perdonar",
    "perdonaría": "perdonar",
    "perdonaríais": "perdonar",
    "perdonaríamos": "perdonar",
    "perdonarían": "perdonar",
    "perdonarías": "perdonar",
    "perdonas": "perdonar",
    "perdonase": "perdonar",
    "perdonaseis": "perdonar",
    "perdonasen": "perdonar",
    "perdonases": "perdonar",
    "perdonaste": "perdonar",
    "perdonasteis": "perdonar",
    "perdone": "perdonar",
    "perdonemos": "perdonar",
    "perdonen": "perdonar",
    "perdones": "perdonar",
    "perdono": "perdonar",
    "perdonábamos": "perdonar",
    "perdonáis": "perdonar",
    "perdonáramos": "perdonar",
    "perdonáremos": "perdonar",
    "perdonásemos": "perdonar",
    "perdoné": "perdonar",
    "perdonéis": "perdonar",
    "perdonó": "perdonar",
    "perdularia": "perdulario",
    "perdularias": "perdulario",
    "perdularios": "perdulario",
    "perdura": "perdurar",
    "perduraba": "perdurar",
    "perdurabais": "perdurar",
    "perduraban": "perdurar",
    "perdurabas": "perdurar",
    "perdurabilidades": "perdurabilidad",
    "perdurables": "perdurable",
    "perduraciones": "perduración",
    "perdurad": "perdurar",
    "perdurada": "perdurar",
    "perduradas": "perdurar",
    "perdurado": "perdurar",
    "perdurados": "perdurar",
    "perduramos": "perdurar",
    "perduran": "perdurar",
    "perdurando": "perdurar",
    "perdurara": "perdurar",
    "perdurarais": "perdurar",
    "perduraran": "perdurar",
    "perduraras": "perdurar",
    "perdurare": "perdurar",
    "perdurareis": "perdurar",
    "perduraremos": "perdurar",
    "perduraren": "perdurar",
    "perdurares": "perdurar",
    "perduraron": "perdurar",
    "perdurará": "perdurar",
    "perdurarán": "perdurar",
    "perdurarás": "perdurar",
    "perduraré": "perdurar",
    "perduraréis": "perdurar",
    "perduraría": "perdurar",
    "perduraríais": "perdurar",
    "perduraríamos": "perdurar",
    "perdurarían": "perdurar",
    "perdurarías": "perdurar",
    "perduras": "perdurar",
    "perdurase": "perdurar",
    "perduraseis": "perdurar",
    "perdurasen": "perdurar",
    "perdurases": "perdurar",
    "perduraste": "perdurar",
    "perdurasteis": "perdurar",
    "perdure": "perdurar",
    "perduremos": "perdurar",
    "perduren": "perdurar",
    "perdures": "perdurar",
    "perduro": "perdurar",
    "perdurábamos": "perdurar",
    "perduráis": "perdurar",
    "perduráramos": "perdurar",
    "perduráremos": "perdurar",
    "perdurásemos": "perdurar",
    "perduré": "perdurar",
    "perduréis": "perdurar",
    "perduró": "perdurar",
    "perdáis": "perder",
    "perdéis": "perder",
    "perdí": "perder",
    "perdía": "perder",
    "perdíais": "perder",
    "perdíamos": "perder",
    "perdían": "perder",
    "perdías": "perder",
    "perece": "perecer",
    "pereced": "perecer",
    "perecedera": "perecedero",
    "perecederas": "perecedero",
    "perecederos": "perecedero",
    "perecemos": "perecer",
    "perecen": "perecer",
    "pereceremos": "perecer",
    "perecerá": "perecer",
    "perecerán": "perecer",
    "perecerás": "perecer",
    "pereceré": "perecer",
    "pereceréis": "perecer",
    "perecería": "perecer",
    "pereceríais": "perecer",
    "pereceríamos": "perecer",
    "perecerían": "perecer",
    "perecerías": "perecer",
    "pereces": "perecer",
    "perecibles": "perecible",
    "perecida": "perecer",
    "perecidas": "perecer",
    "perecido": "perecer",
    "perecidos": "perecer",
    "pereciendo": "perecer",
    "pereciera": "perecer",
    "perecierais": "perecer",
    "perecieran": "perecer",
    "perecieras": "perecer",
    "pereciere": "perecer",
    "pereciereis": "perecer",
    "perecieren": "perecer",
    "perecieres": "perecer",
    "perecieron": "perecer",
    "pereciese": "perecer",
    "perecieseis": "perecer",
    "pereciesen": "perecer",
    "perecieses": "perecer",
    "perecimientos": "perecimiento",
    "perecimos": "perecer",
    "pereciste": "perecer",
    "perecisteis": "perecer",
    "pereciéramos": "perecer",
    "pereciéremos": "perecer",
    "pereciésemos": "perecer",
    "pereció": "perecer",
    "perecéis": "perecer",
    "perecí": "perecer",
    "perecía": "perecer",
    "perecíais": "perecer",
    "perecíamos": "perecer",
    "perecían": "perecer",
    "perecías": "perecer",
    "peredas": "pereda",
    "peregrina": "peregrino",
    "peregrinaba": "peregrinar",
    "peregrinabais": "peregrinar",
    "peregrinaban": "peregrinar",
    "peregrinabas": "peregrinar",
    "peregrinaciones": "peregrinación",
    "peregrinad": "peregrinar",
    "peregrinada": "peregrinar",
    "peregrinadas": "peregrinar",
    "peregrinado": "peregrinar",
    "peregrinados": "peregrinar",
    "peregrinajes": "peregrinaje",
    "peregrinamos": "peregrinar",
    "peregrinan": "peregrinar",
    "peregrinando": "peregrinar",
    "peregrinara": "peregrinar",
    "peregrinarais": "peregrinar",
    "peregrinaran": "peregrinar",
    "peregrinaras": "peregrinar",
    "peregrinare": "peregrinar",
    "peregrinareis": "peregrinar",
    "peregrinaremos": "peregrinar",
    "peregrinaren": "peregrinar",
    "peregrinares": "peregrinar",
    "peregrinaron": "peregrinar",
    "peregrinará": "peregrinar",
    "peregrinarán": "peregrinar",
    "peregrinarás": "peregrinar",
    "peregrinaré": "peregrinar",
    "peregrinaréis": "peregrinar",
    "peregrinaría": "peregrinar",
    "peregrinaríais": "peregrinar",
    "peregrinaríamos": "peregrinar",
    "peregrinarían": "peregrinar",
    "peregrinarías": "peregrinar",
    "peregrinas": "peregrino",
    "peregrinase": "peregrinar",
    "peregrinaseis": "peregrinar",
    "peregrinasen": "peregrinar",
    "peregrinases": "peregrinar",
    "peregrinaste": "peregrinar",
    "peregrinasteis": "peregrinar",
    "peregrine": "peregrinar",
    "peregrinemos": "peregrinar",
    "peregrinen": "peregrinar",
    "peregrines": "peregrinar",
    "peregrinidades": "peregrinidad",
    "peregrino": "peregrinar",
    "peregrinos": "peregrino",
    "peregrinábamos": "peregrinar",
    "peregrináis": "peregrinar",
    "peregrináramos": "peregrinar",
    "peregrináremos": "peregrinar",
    "peregrinásemos": "peregrinar",
    "peregriné": "peregrinar",
    "peregrinéis": "peregrinar",
    "peregrinó": "peregrinar",
    "perejiles": "perejil",
    "perenciones": "perención",
    "perendecas": "perendeca",
    "perendengues": "perendengue",
    "perengana": "perengano",
    "perenganas": "perengano",
    "perenganos": "perengano",
    "perennales": "perennal",
    "perennes": "perenne",
    "perennidades": "perennidad",
    "perentoria": "perentorio",
    "perentorias": "perentorio",
    "perentoriedades": "perentoriedad",
    "perentorios": "perentorio",
    "perestroikas": "perestroika",
    "perezas": "pereza",
    "perezca": "perecer",
    "perezcamos": "perecer",
    "perezcan": "perecer",
    "perezcas": "perecer",
    "perezco": "perecer",
    "perezcáis": "perecer",
    "perezosa": "perezoso",
    "perezosas": "perezoso",
    "perezosos": "perezoso",
    "perfecciona": "perfeccionar",
    "perfeccionaba": "perfeccionar",
    "perfeccionabais": "perfeccionar",
    "perfeccionaban": "perfeccionar",
    "perfeccionabas": "perfeccionar",
    "perfeccionad": "perfeccionar",
    "perfeccionada": "perfeccionar",
    "perfeccionadas": "perfeccionar",
    "perfeccionado": "perfeccionar",
    "perfeccionadora": "perfeccionador",
    "perfeccionadoras": "perfeccionador",
    "perfeccionadores": "perfeccionador",
    "perfeccionados": "perfeccionar",
    "perfeccionamientos": "perfeccionamiento",
    "perfeccionamos": "perfeccionar",
    "perfeccionan": "perfeccionar",
    "perfeccionando": "perfeccionar",
    "perfeccionara": "perfeccionar",
    "perfeccionarais": "perfeccionar",
    "perfeccionaran": "perfeccionar",
    "perfeccionaras": "perfeccionar",
    "perfeccionare": "perfeccionar",
    "perfeccionareis": "perfeccionar",
    "perfeccionaremos": "perfeccionar",
    "perfeccionaren": "perfeccionar",
    "perfeccionares": "perfeccionar",
    "perfeccionaron": "perfeccionar",
    "perfeccionará": "perfeccionar",
    "perfeccionarán": "perfeccionar",
    "perfeccionarás": "perfeccionar",
    "perfeccionaré": "perfeccionar",
    "perfeccionaréis": "perfeccionar",
    "perfeccionaría": "perfeccionar",
    "perfeccionaríais": "perfeccionar",
    "perfeccionaríamos": "perfeccionar",
    "perfeccionarían": "perfeccionar",
    "perfeccionarías": "perfeccionar",
    "perfeccionas": "perfeccionar",
    "perfeccionase": "perfeccionar",
    "perfeccionaseis": "perfeccionar",
    "perfeccionasen": "perfeccionar",
    "perfeccionases": "perfeccionar",
    "perfeccionaste": "perfeccionar",
    "perfeccionasteis": "perfeccionar",
    "perfeccione": "perfeccionar",
    "perfeccionemos": "perfeccionar",
    "perfeccionen": "perfeccionar",
    "perfecciones": "perfeccionar",
    "perfeccionismos": "perfeccionismo",
    "perfeccionistas": "perfeccionista",
    "perfecciono": "perfeccionar",
    "perfeccionábamos": "perfeccionar",
    "perfeccionáis": "perfeccionar",
    "perfeccionáramos": "perfeccionar",
    "perfeccionáremos": "perfeccionar",
    "perfeccionásemos": "perfeccionar",
    "perfeccioné": "perfeccionar",
    "perfeccionéis": "perfeccionar",
    "perfeccionó": "perfeccionar",
    "perfecta": "perfecto",
    "perfectas": "perfecto",
    "perfectibilidades": "perfectibilidad",
    "perfectibles": "perfectible",
    "perfectiva": "perfectivo",
    "perfectivas": "perfectivo",
    "perfectivos": "perfectivo",
    "perfectos": "perfecto",
    "perfenacinas": "perfenacina",
    "perficientes": "perficiente",
    "perfidias": "perfidia",
    "perfila": "perfilar",
    "perfilaba": "perfilar",
    "perfilabais": "perfilar",
    "perfilaban": "perfilar",
    "perfilabas": "perfilar",
    "perfilad": "perfilar",
    "perfilada": "perfilar",
    "perfiladas": "perfilar",
    "perfilado": "perfilar",
    "perfiladores": "perfilador",
    "perfilados": "perfilar",
    "perfiladuras": "perfiladura",
    "perfilamos": "perfilar",
    "perfilan": "perfilar",
    "perfilando": "perfilar",
    "perfilara": "perfilar",
    "perfilarais": "perfilar",
    "perfilaran": "perfilar",
    "perfilaras": "perfilar",
    "perfilare": "perfilar",
    "perfilareis": "perfilar",
    "perfilaremos": "perfilar",
    "perfilaren": "perfilar",
    "perfilares": "perfilar",
    "perfilaron": "perfilar",
    "perfilará": "perfilar",
    "perfilarán": "perfilar",
    "perfilarás": "perfilar",
    "perfilaré": "perfilar",
    "perfilaréis": "perfilar",
    "perfilaría": "perfilar",
    "perfilaríais": "perfilar",
    "perfilaríamos": "perfilar",
    "perfilarían": "perfilar",
    "perfilarías": "perfilar",
    "perfilas": "perfilar",
    "perfilase": "perfilar",
    "perfilaseis": "perfilar",
    "perfilasen": "perfilar",
    "perfilases": "perfilar",
    "perfilaste": "perfilar",
    "perfilasteis": "perfilar",
    "perfile": "perfilar",
    "perfilemos": "perfilar",
    "perfilen": "perfilar",
    "perfiles": "perfilar",
    "perfilo": "perfilar",
    "perfilábamos": "perfilar",
    "perfiláis": "perfilar",
    "perfiláramos": "perfilar",
    "perfiláremos": "perfilar",
    "perfilásemos": "perfilar",
    "perfilé": "perfilar",
    "perfiléis": "perfilar",
    "perfiló": "perfilar",
    "perfilógrafos": "perfilógrafo",
    "perfora": "perforar",
    "perforaba": "perforar",
    "perforabais": "perforar",
    "perforaban": "perforar",
    "perforabas": "perforar",
    "perforables": "perforable",
    "perforaciones": "perforación",
    "perforad": "perforar",
    "perforada": "perforar",
    "perforadas": "perforar",
    "perforado": "perforar",
    "perforadora": "perforador",
    "perforadoras": "perforadora",
    "perforadores": "perforador",
    "perforados": "perforar",
    "perforamos": "perforar",
    "perforan": "perforar",
    "perforando": "perforar",
    "perforantes": "perforante",
    "perforara": "perforar",
    "perforarais": "perforar",
    "perforaran": "perforar",
    "perforaras": "perforar",
    "perforare": "perforar",
    "perforareis": "perforar",
    "perforaremos": "perforar",
    "perforaren": "perforar",
    "perforares": "perforar",
    "perforaron": "perforar",
    "perforará": "perforar",
    "perforarán": "perforar",
    "perforarás": "perforar",
    "perforaré": "perforar",
    "perforaréis": "perforar",
    "perforaría": "perforar",
    "perforaríais": "perforar",
    "perforaríamos": "perforar",
    "perforarían": "perforar",
    "perforarías": "perforar",
    "perforas": "perforar",
    "perforase": "perforar",
    "perforaseis": "perforar",
    "perforasen": "perforar",
    "perforases": "perforar",
    "perforaste": "perforar",
    "perforasteis": "perforar",
    "perfore": "perforar",
    "perforemos": "perforar",
    "perforen": "perforar",
    "perfores": "perforar",
    "perforinas": "perforina",
    "perforistas": "perforista",
    "performances": "performance",
    "performativa": "performativo",
    "performativas": "performativo",
    "performativos": "performativo",
    "perforo": "perforar",
    "perforábamos": "perforar",
    "perforáis": "perforar",
    "perforáramos": "perforar",
    "perforáremos": "perforar",
    "perforásemos": "perforar",
    "perforé": "perforar",
    "perforéis": "perforar",
    "perforó": "perforar",
    "perfuma": "perfumar",
    "perfumaba": "perfumar",
    "perfumabais": "perfumar",
    "perfumaban": "perfumar",
    "perfumabas": "perfumar",
    "perfumad": "perfumar",
    "perfumada": "perfumar",
    "perfumadas": "perfumar",
    "perfumaderos": "perfumadero",
    "perfumado": "perfumar",
    "perfumadora": "perfumador",
    "perfumadoras": "perfumador",
    "perfumadores": "perfumador",
    "perfumados": "perfumar",
    "perfumamos": "perfumar",
    "perfuman": "perfumar",
    "perfumando": "perfumar",
    "perfumara": "perfumar",
    "perfumarais": "perfumar",
    "perfumaran": "perfumar",
    "perfumaras": "perfumar",
    "perfumare": "perfumar",
    "perfumareis": "perfumar",
    "perfumaremos": "perfumar",
    "perfumaren": "perfumar",
    "perfumares": "perfumar",
    "perfumaron": "perfumar",
    "perfumará": "perfumar",
    "perfumarán": "perfumar",
    "perfumarás": "perfumar",
    "perfumaré": "perfumar",
    "perfumaréis": "perfumar",
    "perfumaría": "perfumar",
    "perfumaríais": "perfumar",
    "perfumaríamos": "perfumar",
    "perfumarían": "perfumar",
    "perfumarías": "perfumar",
    "perfumas": "perfumar",
    "perfumase": "perfumar",
    "perfumaseis": "perfumar",
    "perfumasen": "perfumar",
    "perfumases": "perfumar",
    "perfumaste": "perfumar",
    "perfumasteis": "perfumar",
    "perfume": "perfumar",
    "perfumemos": "perfumar",
    "perfumen": "perfumar",
    "perfumera": "perfumero",
    "perfumeras": "perfumero",
    "perfumeros": "perfumero",
    "perfumerías": "perfumería",
    "perfumes": "perfume",
    "perfumistas": "perfumista",
    "perfumo": "perfumar",
    "perfumábamos": "perfumar",
    "perfumáis": "perfumar",
    "perfumáramos": "perfumar",
    "perfumáremos": "perfumar",
    "perfumásemos": "perfumar",
    "perfumé": "perfumar",
    "perfuméis": "perfumar",
    "perfumó": "perfumar",
    "perfusiones": "perfusión",
    "perfórmica": "perfórmico",
    "perfórmicas": "perfórmico",
    "perfórmicos": "perfórmico",
    "pergaminos": "pergamino",
    "pergelisoles": "pergelisol",
    "pergeña": "pergeñar",
    "pergeñaba": "pergeñar",
    "pergeñabais": "pergeñar",
    "pergeñaban": "pergeñar",
    "pergeñabas": "pergeñar",
    "pergeñad": "pergeñar",
    "pergeñada": "pergeñar",
    "pergeñadas": "pergeñar",
    "pergeñado": "pergeñar",
    "pergeñados": "pergeñar",
    "pergeñamos": "pergeñar",
    "pergeñan": "pergeñar",
    "pergeñando": "pergeñar",
    "pergeñara": "pergeñar",
    "pergeñarais": "pergeñar",
    "pergeñaran": "pergeñar",
    "pergeñaras": "pergeñar",
    "pergeñare": "pergeñar",
    "pergeñareis": "pergeñar",
    "pergeñaremos": "pergeñar",
    "pergeñaren": "pergeñar",
    "pergeñares": "pergeñar",
    "pergeñaron": "pergeñar",
    "pergeñará": "pergeñar",
    "pergeñarán": "pergeñar",
    "pergeñarás": "pergeñar",
    "pergeñaré": "pergeñar",
    "pergeñaréis": "pergeñar",
    "pergeñaría": "pergeñar",
    "pergeñaríais": "pergeñar",
    "pergeñaríamos": "pergeñar",
    "pergeñarían": "pergeñar",
    "pergeñarías": "pergeñar",
    "pergeñas": "pergeñar",
    "pergeñase": "pergeñar",
    "pergeñaseis": "pergeñar",
    "pergeñasen": "pergeñar",
    "pergeñases": "pergeñar",
    "pergeñaste": "pergeñar",
    "pergeñasteis": "pergeñar",
    "pergeñe": "pergeñar",
    "pergeñemos": "pergeñar",
    "pergeñen": "pergeñar",
    "pergeñes": "pergeñar",
    "pergeño": "pergeñar",
    "pergeños": "pergeño",
    "pergeñábamos": "pergeñar",
    "pergeñáis": "pergeñar",
    "pergeñáramos": "pergeñar",
    "pergeñáremos": "pergeñar",
    "pergeñásemos": "pergeñar",
    "pergeñé": "pergeñar",
    "pergeñéis": "pergeñar",
    "pergeñó": "pergeñar",
    "perhidroles": "perhidrol",
    "periambos": "periambo",
    "periamigdalina": "periamigdalino",
    "periamigdalinas": "periamigdalino",
    "periamigdalinos": "periamigdalino",
    "periantios": "periantio",
    "periantos": "perianto",
    "periapicales": "periapical",
    "periarteriales": "periarterial",
    "periastros": "periastro",
    "peribucales": "peribucal",
    "perica": "perico",
    "pericardiofrénica": "pericardiofrénico",
    "pericardiofrénicas": "pericardiofrénico",
    "pericardiofrénicos": "pericardiofrénico",
    "pericardios": "pericardio",
    "pericarpios": "pericarpio",
    "pericarpos": "pericarpo",
    "pericas": "perico",
    "pericementales": "pericemental",
    "pericementoclasias": "pericementoclasia",
    "pericementos": "pericemento",
    "pericentriolares": "pericentriolar",
    "periciales": "pericial",
    "pericias": "pericia",
    "periciclos": "periciclo",
    "pericinética": "pericinético",
    "pericinéticas": "pericinético",
    "pericinéticos": "pericinético",
    "periclasas": "periclasa",
    "periclinales": "periclinal",
    "periclita": "periclitar",
    "periclitaba": "periclitar",
    "periclitabais": "periclitar",
    "periclitaban": "periclitar",
    "periclitabas": "periclitar",
    "periclitad": "periclitar",
    "periclitada": "periclitar",
    "periclitadas": "periclitar",
    "periclitado": "periclitar",
    "periclitados": "periclitar",
    "periclitamos": "periclitar",
    "periclitan": "periclitar",
    "periclitando": "periclitar",
    "periclitara": "periclitar",
    "periclitarais": "periclitar",
    "periclitaran": "periclitar",
    "periclitaras": "periclitar",
    "periclitare": "periclitar",
    "periclitareis": "periclitar",
    "periclitaremos": "periclitar",
    "periclitaren": "periclitar",
    "periclitares": "periclitar",
    "periclitaron": "periclitar",
    "periclitará": "periclitar",
    "periclitarán": "periclitar",
    "periclitarás": "periclitar",
    "periclitaré": "periclitar",
    "periclitaréis": "periclitar",
    "periclitaría": "periclitar",
    "periclitaríais": "periclitar",
    "periclitaríamos": "periclitar",
    "periclitarían": "periclitar",
    "periclitarías": "periclitar",
    "periclitas": "periclitar",
    "periclitase": "periclitar",
    "periclitaseis": "periclitar",
    "periclitasen": "periclitar",
    "periclitases": "periclitar",
    "periclitaste": "periclitar",
    "periclitasteis": "periclitar",
    "periclite": "periclitar",
    "periclitemos": "periclitar",
    "pericliten": "periclitar",
    "periclites": "periclitar",
    "periclito": "periclitar",
    "periclitábamos": "periclitar",
    "periclitáis": "periclitar",
    "periclitáramos": "periclitar",
    "periclitáremos": "periclitar",
    "periclitásemos": "periclitar",
    "periclité": "periclitar",
    "periclitéis": "periclitar",
    "periclitó": "periclitar",
    "pericona": "pericón",
    "periconas": "pericón",
    "pericondrios": "pericondrio",
    "pericones": "pericón",
    "pericos": "perico",
    "pericráneos": "pericráneo",
    "pericárdica": "pericárdico",
    "pericárdicas": "pericárdico",
    "pericárdicos": "pericárdico",
    "peridentales": "peridental",
    "peridios": "peridio",
    "peridotitas": "peridotita",
    "peridotos": "peridoto",
    "perieca": "perieco",
    "periecas": "perieco",
    "periecos": "perieco",
    "perifaríngea": "perifaríngeo",
    "perifaríngeas": "perifaríngeo",
    "perifaríngeos": "perifaríngeo",
    "periferias": "periferia",
    "perifolios": "perifolio",
    "perifollos": "perifollo",
    "perifrasea": "perifrasear",
    "perifraseaba": "perifrasear",
    "perifraseabais": "perifrasear",
    "perifraseaban": "perifrasear",
    "perifraseabas": "perifrasear",
    "perifrasead": "perifrasear",
    "perifraseada": "perifrasear",
    "perifraseadas": "perifrasear",
    "perifraseado": "perifrasear",
    "perifraseados": "perifrasear",
    "perifraseamos": "perifrasear",
    "perifrasean": "perifrasear",
    "perifraseando": "perifrasear",
    "perifraseara": "perifrasear",
    "perifrasearais": "perifrasear",
    "perifrasearan": "perifrasear",
    "perifrasearas": "perifrasear",
    "perifraseare": "perifrasear",
    "perifraseareis": "perifrasear",
    "perifrasearemos": "perifrasear",
    "perifrasearen": "perifrasear",
    "perifraseares": "perifrasear",
    "perifrasearon": "perifrasear",
    "perifraseará": "perifrasear",
    "perifrasearán": "perifrasear",
    "perifrasearás": "perifrasear",
    "perifrasearé": "perifrasear",
    "perifrasearéis": "perifrasear",
    "perifrasearía": "perifrasear",
    "perifrasearíais": "perifrasear",
    "perifrasearíamos": "perifrasear",
    "perifrasearían": "perifrasear",
    "perifrasearías": "perifrasear",
    "perifraseas": "perifrasear",
    "perifrasease": "perifrasear",
    "perifraseaseis": "perifrasear",
    "perifraseasen": "perifrasear",
    "perifraseases": "perifrasear",
    "perifraseaste": "perifrasear",
    "perifraseasteis": "perifrasear",
    "perifrasee": "perifrasear",
    "perifraseemos": "perifrasear",
    "perifraseen": "perifrasear",
    "perifrasees": "perifrasear",
    "perifraseo": "perifrasear",
    "perifraseábamos": "perifrasear",
    "perifraseáis": "perifrasear",
    "perifraseáramos": "perifrasear",
    "perifraseáremos": "perifrasear",
    "perifraseásemos": "perifrasear",
    "perifraseé": "perifrasear",
    "perifraseéis": "perifrasear",
    "perifraseó": "perifrasear",
    "perifrástica": "perifrástico",
    "perifrásticas": "perifrástico",
    "perifrásticos": "perifrástico",
    "periférica": "periférico",
    "periféricas": "periférico",
    "periféricos": "periférico",
    "perigallos": "perigallo",
    "perigeos": "perigeo",
    "periglaciares": "periglaciar",
    "perihelios": "perihelio",
    "perilinfas": "perilinfa",
    "perilinfática": "perilinfático",
    "perilinfáticas": "perilinfático",
    "perilinfáticos": "perilinfático",
    "perillana": "perillán",
    "perillanas": "perillán",
    "perillanes": "perillán",
    "perillas": "perilla",
    "perimetrales": "perimetral",
    "perimetrías": "perimetría",
    "perimisios": "perimisio",
    "perimétrica": "perimétrico",
    "perimétricas": "perimétrico",
    "perimétricos": "perimétrico",
    "perinatales": "perinatal",
    "perindolas": "perindola",
    "perineales": "perineal",
    "perineos": "perineo",
    "perineumonías": "perineumonía",
    "perineuros": "perineuro",
    "perinolas": "perinola",
    "perinucleares": "perinuclear",
    "perinés": "periné",
    "periodicidades": "periodicidad",
    "periodicuchos": "periodicucho",
    "periodificaciones": "periodificación",
    "periodismos": "periodismo",
    "periodistas": "periodista",
    "periodogramas": "periodograma",
    "periodoncias": "periodoncia",
    "periodontales": "periodontal",
    "periodontistas": "periodontista",
    "periodontoclasias": "periodontoclasia",
    "periodontologías": "periodontología",
    "periodontos": "periodonto",
    "periodontómetros": "periodontómetro",
    "periodos": "periodo",
    "periodística": "periodístico",
    "periodísticas": "periodístico",
    "periodísticos": "periodístico",
    "periodóntica": "periodóntico",
    "periodónticas": "periodóntico",
    "periodónticos": "periodóntico",
    "periosteótomos": "periosteótomo",
    "periostios": "periostio",
    "peripatetismos": "peripatetismo",
    "peripatos": "peripato",
    "peripatética": "peripatético",
    "peripatéticas": "peripatético",
    "peripatéticos": "peripatético",
    "peripecias": "peripecia",
    "periplasmas": "periplasma",
    "periplasmática": "periplasmático",
    "periplasmáticas": "periplasmático",
    "periplasmáticos": "periplasmático",
    "periplos": "periplo",
    "periplásmica": "periplásmico",
    "periplásmicas": "periplásmico",
    "periplásmicos": "periplásmico",
    "peripolesias": "peripolesia",
    "peripuesta": "peripuesto",
    "peripuestas": "peripuesto",
    "peripuestos": "peripuesto",
    "periquea": "periquear",
    "periqueaba": "periquear",
    "periqueabais": "periquear",
    "periqueaban": "periquear",
    "periqueabas": "periquear",
    "periquead": "periquear",
    "periqueada": "periquear",
    "periqueadas": "periquear",
    "periqueado": "periquear",
    "periqueados": "periquear",
    "periqueamos": "periquear",
    "periquean": "periquear",
    "periqueando": "periquear",
    "periqueara": "periquear",
    "periquearais": "periquear",
    "periquearan": "periquear",
    "periquearas": "periquear",
    "periqueare": "periquear",
    "periqueareis": "periquear",
    "periquearemos": "periquear",
    "periquearen": "periquear",
    "periqueares": "periquear",
    "periquearon": "periquear",
    "periqueará": "periquear",
    "periquearán": "periquear",
    "periquearás": "periquear",
    "periquearé": "periquear",
    "periquearéis": "periquear",
    "periquearía": "periquear",
    "periquearíais": "periquear",
    "periquearíamos": "periquear",
    "periquearían": "periquear",
    "periquearías": "periquear",
    "periqueas": "periquear",
    "periquease": "periquear",
    "periqueaseis": "periquear",
    "periqueasen": "periquear",
    "periqueases": "periquear",
    "periqueaste": "periquear",
    "periqueasteis": "periquear",
    "periquee": "periquear",
    "periqueemos": "periquear",
    "periqueen": "periquear",
    "periquees": "periquear",
    "periqueo": "periquear",
    "periquetes": "periquete",
    "periqueábamos": "periquear",
    "periqueáis": "periquear",
    "periqueáramos": "periquear",
    "periqueáremos": "periquear",
    "periqueásemos": "periquear",
    "periqueé": "periquear",
    "periqueéis": "periquear",
    "periqueó": "periquear",
    "periquitos": "periquito",
    "peris": "peri",
    "periscia": "periscio",
    "periscias": "periscio",
    "periscios": "periscio",
    "periscopios": "periscopio",
    "periscópica": "periscópico",
    "periscópicas": "periscópico",
    "periscópicos": "periscópico",
    "perisodáctilos": "perisodáctilo",
    "perisologías": "perisología",
    "peristaltismos": "peristaltismo",
    "peristeritas": "peristerita",
    "peristilos": "peristilo",
    "peristáltica": "peristáltico",
    "peristálticas": "peristáltico",
    "peristálticos": "peristáltico",
    "perita": "perito",
    "peritaciones": "peritación",
    "peritajes": "peritaje",
    "peritas": "perito",
    "peritecios": "peritecio",
    "peritectomías": "peritectomía",
    "peritendones": "peritendón",
    "peritoneales": "peritoneal",
    "peritoneoperineales": "peritoneoperineal",
    "peritoneos": "peritoneo",
    "peritos": "perito",
    "peritéctica": "peritéctico",
    "peritécticas": "peritéctico",
    "peritécticos": "peritéctico",
    "periventriculares": "periventricular",
    "periápices": "periápice",
    "periódica": "periódico",
    "periódicas": "periódico",
    "periódicos": "periódico",
    "periórbitas": "periórbita",
    "perióstica": "perióstico",
    "periósticas": "perióstico",
    "periósticos": "perióstico",
    "perjudica": "perjudicar",
    "perjudicaba": "perjudicar",
    "perjudicabais": "perjudicar",
    "perjudicaban": "perjudicar",
    "perjudicabas": "perjudicar",
    "perjudicad": "perjudicar",
    "perjudicada": "perjudicar",
    "perjudicadas": "perjudicar",
    "perjudicado": "perjudicar",
    "perjudicadora": "perjudicador",
    "perjudicadoras": "perjudicador",
    "perjudicadores": "perjudicador",
    "perjudicados": "perjudicar",
    "perjudicamos": "perjudicar",
    "perjudican": "perjudicar",
    "perjudicando": "perjudicar",
    "perjudicara": "perjudicar",
    "perjudicarais": "perjudicar",
    "perjudicaran": "perjudicar",
    "perjudicaras": "perjudicar",
    "perjudicare": "perjudicar",
    "perjudicareis": "perjudicar",
    "perjudicaremos": "perjudicar",
    "perjudicaren": "perjudicar",
    "perjudicares": "perjudicar",
    "perjudicaron": "perjudicar",
    "perjudicará": "perjudicar",
    "perjudicarán": "perjudicar",
    "perjudicarás": "perjudicar",
    "perjudicaré": "perjudicar",
    "perjudicaréis": "perjudicar",
    "perjudicaría": "perjudicar",
    "perjudicaríais": "perjudicar",
    "perjudicaríamos": "perjudicar",
    "perjudicarían": "perjudicar",
    "perjudicarías": "perjudicar",
    "perjudicas": "perjudicar",
    "perjudicase": "perjudicar",
    "perjudicaseis": "perjudicar",
    "perjudicasen": "perjudicar",
    "perjudicases": "perjudicar",
    "perjudicaste": "perjudicar",
    "perjudicasteis": "perjudicar",
    "perjudiciales": "perjudicial",
    "perjudico": "perjudicar",
    "perjudicábamos": "perjudicar",
    "perjudicáis": "perjudicar",
    "perjudicáramos": "perjudicar",
    "perjudicáremos": "perjudicar",
    "perjudicásemos": "perjudicar",
    "perjudicó": "perjudicar",
    "perjudique": "perjudicar",
    "perjudiquemos": "perjudicar",
    "perjudiquen": "perjudicar",
    "perjudiques": "perjudicar",
    "perjudiqué": "perjudicar",
    "perjudiquéis": "perjudicar",
    "perjuicios": "perjuicio",
    "perjura": "perjuro",
    "perjuraba": "perjurar",
    "perjurabais": "perjurar",
    "perjuraban": "perjurar",
    "perjurabas": "perjurar",
    "perjurad": "perjurar",
    "perjurada": "perjurar",
    "perjuradas": "perjurar",
    "perjurado": "perjurar",
    "perjurados": "perjurar",
    "perjuramos": "perjurar",
    "perjuran": "perjurar",
    "perjurando": "perjurar",
    "perjurara": "perjurar",
    "perjurarais": "perjurar",
    "perjuraran": "perjurar",
    "perjuraras": "perjurar",
    "perjurare": "perjurar",
    "perjurareis": "perjurar",
    "perjuraremos": "perjurar",
    "perjuraren": "perjurar",
    "perjurares": "perjurar",
    "perjuraron": "perjurar",
    "perjurará": "perjurar",
    "perjurarán": "perjurar",
    "perjurarás": "perjurar",
    "perjuraré": "perjurar",
    "perjuraréis": "perjurar",
    "perjuraría": "perjurar",
    "perjuraríais": "perjurar",
    "perjuraríamos": "perjurar",
    "perjurarían": "perjurar",
    "perjurarías": "perjurar",
    "perjuras": "perjuro",
    "perjurase": "perjurar",
    "perjuraseis": "perjurar",
    "perjurasen": "perjurar",
    "perjurases": "perjurar",
    "perjuraste": "perjurar",
    "perjurasteis": "perjurar",
    "perjure": "perjurar",
    "perjuremos": "perjurar",
    "perjuren": "perjurar",
    "perjures": "perjurar",
    "perjurios": "perjurio",
    "perjuro": "perjurar",
    "perjuros": "perjuro",
    "perjurábamos": "perjurar",
    "perjuráis": "perjurar",
    "perjuráramos": "perjurar",
    "perjuráremos": "perjurar",
    "perjurásemos": "perjurar",
    "perjuré": "perjurar",
    "perjuréis": "perjurar",
    "perjuró": "perjurar",
    "perlada": "perlado",
    "perladas": "perlado",
    "perlados": "perlado",
    "perlas": "perla",
    "perlera": "perlero",
    "perleras": "perlero",
    "perleros": "perlero",
    "perlerías": "perlería",
    "perlesías": "perlesía",
    "perlina": "perlino",
    "perlinas": "perlino",
    "perlinguales": "perlingual",
    "perlinos": "perlino",
    "perlitas": "perlita",
    "perlocutiva": "perlocutivo",
    "perlocutivas": "perlocutivo",
    "perlocutivos": "perlocutivo",
    "perlonga": "perlongar",
    "perlongaba": "perlongar",
    "perlongabais": "perlongar",
    "perlongaban": "perlongar",
    "perlongabas": "perlongar",
    "perlongad": "perlongar",
    "perlongada": "perlongar",
    "perlongadas": "perlongar",
    "perlongado": "perlongar",
    "perlongados": "perlongar",
    "perlongamos": "perlongar",
    "perlongan": "perlongar",
    "perlongando": "perlongar",
    "perlongara": "perlongar",
    "perlongarais": "perlongar",
    "perlongaran": "perlongar",
    "perlongaras": "perlongar",
    "perlongare": "perlongar",
    "perlongareis": "perlongar",
    "perlongaremos": "perlongar",
    "perlongaren": "perlongar",
    "perlongares": "perlongar",
    "perlongaron": "perlongar",
    "perlongará": "perlongar",
    "perlongarán": "perlongar",
    "perlongarás": "perlongar",
    "perlongaré": "perlongar",
    "perlongaréis": "perlongar",
    "perlongaría": "perlongar",
    "perlongaríais": "perlongar",
    "perlongaríamos": "perlongar",
    "perlongarían": "perlongar",
    "perlongarías": "perlongar",
    "perlongas": "perlongar",
    "perlongase": "perlongar",
    "perlongaseis": "perlongar",
    "perlongasen": "perlongar",
    "perlongases": "perlongar",
    "perlongaste": "perlongar",
    "perlongasteis": "perlongar",
    "perlongo": "perlongar",
    "perlongue": "perlongar",
    "perlonguemos": "perlongar",
    "perlonguen": "perlongar",
    "perlongues": "perlongar",
    "perlongué": "perlongar",
    "perlonguéis": "perlongar",
    "perlongábamos": "perlongar",
    "perlongáis": "perlongar",
    "perlongáramos": "perlongar",
    "perlongáremos": "perlongar",
    "perlongásemos": "perlongar",
    "perlongó": "perlongar",
    "perlática": "perlático",
    "perláticas": "perlático",
    "perláticos": "perlático",
    "perlífera": "perlífero",
    "perlíferas": "perlífero",
    "perlíferos": "perlífero",
    "perlítica": "perlítico",
    "perlíticas": "perlítico",
    "perlíticos": "perlítico",
    "perlómetros": "perlómetro",
    "permafrostes": "permafrost",
    "permageles": "permagel",
    "permaloys": "permaloy",
    "permanece": "permanecer",
    "permaneced": "permanecer",
    "permanecemos": "permanecer",
    "permanecen": "permanecer",
    "permaneceremos": "permanecer",
    "permanecerá": "permanecer",
    "permanecerán": "permanecer",
    "permanecerás": "permanecer",
    "permaneceré": "permanecer",
    "permaneceréis": "permanecer",
    "permanecería": "permanecer",
    "permaneceríais": "permanecer",
    "permaneceríamos": "permanecer",
    "permanecerían": "permanecer",
    "permanecerías": "permanecer",
    "permaneces": "permanecer",
    "permanecida": "permanecer",
    "permanecidas": "permanecer",
    "permanecido": "permanecer",
    "permanecidos": "permanecer",
    "permaneciendo": "permanecer",
    "permaneciera": "permanecer",
    "permanecierais": "permanecer",
    "permanecieran": "permanecer",
    "permanecieras": "permanecer",
    "permaneciere": "permanecer",
    "permaneciereis": "permanecer",
    "permanecieren": "permanecer",
    "permanecieres": "permanecer",
    "permanecieron": "permanecer",
    "permaneciese": "permanecer",
    "permanecieseis": "permanecer",
    "permaneciesen": "permanecer",
    "permanecieses": "permanecer",
    "permanecimos": "permanecer",
    "permaneciste": "permanecer",
    "permanecisteis": "permanecer",
    "permaneciéramos": "permanecer",
    "permaneciéremos": "permanecer",
    "permaneciésemos": "permanecer",
    "permaneció": "permanecer",
    "permanecéis": "permanecer",
    "permanecí": "permanecer",
    "permanecía": "permanecer",
    "permanecíais": "permanecer",
    "permanecíamos": "permanecer",
    "permanecían": "permanecer",
    "permanecías": "permanecer",
    "permanencias": "permanencia",
    "permanentes": "permanente",
    "permanezca": "permanecer",
    "permanezcamos": "permanecer",
    "permanezcan": "permanecer",
    "permanezcas": "permanecer",
    "permanezco": "permanecer",
    "permanezcáis": "permanecer",
    "permanganatos": "permanganato",
    "permeabilidades": "permeabilidad",
    "permeables": "permeable",
    "permeaciones": "permeación",
    "permeancias": "permeancia",
    "permeámetros": "permeámetro",
    "permisibilidades": "permisibilidad",
    "permisibles": "permisible",
    "permisionaria": "permisionario",
    "permisionarias": "permisionario",
    "permisionarios": "permisionario",
    "permisiones": "permisión",
    "permisiva": "permisivo",
    "permisivas": "permisivo",
    "permisividades": "permisividad",
    "permisivos": "permisivo",
    "permisos": "permiso",
    "permita": "permitir",
    "permitamos": "permitir",
    "permitan": "permitir",
    "permitas": "permitir",
    "permite": "permitir",
    "permiten": "permitir",
    "permites": "permitir",
    "permitibles": "permitible",
    "permitid": "permitir",
    "permitida": "permitir",
    "permitidas": "permitir",
    "permitido": "permitir",
    "permitidora": "permitidor",
    "permitidoras": "permitidor",
    "permitidores": "permitidor",
    "permitidos": "permitir",
    "permitiendo": "permitir",
    "permitiera": "permitir",
    "permitierais": "permitir",
    "permitieran": "permitir",
    "permitieras": "permitir",
    "permitiere": "permitir",
    "permitiereis": "permitir",
    "permitieren": "permitir",
    "permitieres": "permitir",
    "permitieron": "permitir",
    "permitiese": "permitir",
    "permitieseis": "permitir",
    "permitiesen": "permitir",
    "permitieses": "permitir",
    "permitimos": "permitir",
    "permitiremos": "permitir",
    "permitirá": "permitir",
    "permitirán": "permitir",
    "permitirás": "permitir",
    "permitiré": "permitir",
    "permitiréis": "permitir",
    "permitiría": "permitir",
    "permitiríais": "permitir",
    "permitiríamos": "permitir",
    "permitirían": "permitir",
    "permitirías": "permitir",
    "permitiste": "permitir",
    "permitisteis": "permitir",
    "permitiva": "permitivo",
    "permitivas": "permitivo",
    "permitividades": "permitividad",
    "permitivos": "permitivo",
    "permitiéramos": "permitir",
    "permitiéremos": "permitir",
    "permitiésemos": "permitir",
    "permitió": "permitir",
    "permito": "permitir",
    "permitáis": "permitir",
    "permití": "permitir",
    "permitía": "permitir",
    "permitíais": "permitir",
    "permitíamos": "permitir",
    "permitían": "permitir",
    "permitías": "permitir",
    "permitís": "permitir",
    "permuta": "permutar",
    "permutaba": "permutar",
    "permutabais": "permutar",
    "permutaban": "permutar",
    "permutabas": "permutar",
    "permutabilidades": "permutabilidad",
    "permutables": "permutable",
    "permutaciones": "permutación",
    "permutad": "permutar",
    "permutada": "permutar",
    "permutadas": "permutar",
    "permutado": "permutar",
    "permutados": "permutar",
    "permutamos": "permutar",
    "permutan": "permutar",
    "permutando": "permutar",
    "permutara": "permutar",
    "permutarais": "permutar",
    "permutaran": "permutar",
    "permutaras": "permutar",
    "permutare": "permutar",
    "permutareis": "permutar",
    "permutaremos": "permutar",
    "permutaren": "permutar",
    "permutares": "permutar",
    "permutaron": "permutar",
    "permutará": "permutar",
    "permutarán": "permutar",
    "permutarás": "permutar",
    "permutaré": "permutar",
    "permutaréis": "permutar",
    "permutaría": "permutar",
    "permutaríais": "permutar",
    "permutaríamos": "permutar",
    "permutarían": "permutar",
    "permutarías": "permutar",
    "permutas": "permutar",
    "permutase": "permutar",
    "permutaseis": "permutar",
    "permutasen": "permutar",
    "permutases": "permutar",
    "permutaste": "permutar",
    "permutasteis": "permutar",
    "permutativa": "permutativo",
    "permutativas": "permutativo",
    "permutativos": "permutativo",
    "permute": "permutar",
    "permutemos": "permutar",
    "permuten": "permutar",
    "permutes": "permutar",
    "permuto": "permutar",
    "permutábamos": "permutar",
    "permutáis": "permutar",
    "permutáramos": "permutar",
    "permutáremos": "permutar",
    "permutásemos": "permutar",
    "permuté": "permutar",
    "permutéis": "permutar",
    "permutó": "permutar",
    "pernadas": "pernada",
    "pernea": "pernear",
    "perneaba": "pernear",
    "perneabais": "pernear",
    "perneaban": "pernear",
    "perneabas": "pernear",
    "pernead": "pernear",
    "perneada": "pernear",
    "perneadas": "pernear",
    "perneado": "pernear",
    "perneadora": "perneador",
    "perneadoras": "perneador",
    "perneadores": "perneador",
    "perneados": "pernear",
    "perneamos": "pernear",
    "pernean": "pernear",
    "perneando": "pernear",
    "perneara": "pernear",
    "pernearais": "pernear",
    "pernearan": "pernear",
    "pernearas": "pernear",
    "perneare": "pernear",
    "perneareis": "pernear",
    "pernearemos": "pernear",
    "pernearen": "pernear",
    "perneares": "pernear",
    "pernearon": "pernear",
    "perneará": "pernear",
    "pernearán": "pernear",
    "pernearás": "pernear",
    "pernearé": "pernear",
    "pernearéis": "pernear",
    "pernearía": "pernear",
    "pernearíais": "pernear",
    "pernearíamos": "pernear",
    "pernearían": "pernear",
    "pernearías": "pernear",
    "perneas": "pernear",
    "pernease": "pernear",
    "perneaseis": "pernear",
    "perneasen": "pernear",
    "perneases": "pernear",
    "perneaste": "pernear",
    "perneasteis": "pernear",
    "pernee": "pernear",
    "perneemos": "pernear",
    "perneen": "pernear",
    "pernees": "pernear",
    "perneo": "pernear",
    "perneras": "pernera",
    "perneábamos": "pernear",
    "perneáis": "pernear",
    "perneáramos": "pernear",
    "perneáremos": "pernear",
    "perneásemos": "pernear",
    "perneé": "pernear",
    "perneéis": "pernear",
    "perneó": "pernear",
    "perniabierta": "perniabierto",
    "perniabiertas": "perniabierto",
    "perniabiertos": "perniabierto",
    "perniciosa": "pernicioso",
    "perniciosas": "pernicioso",
    "perniciosos": "pernicioso",
    "pernicorta": "pernicorto",
    "pernicortas": "pernicorto",
    "pernicortos": "pernicorto",
    "perniles": "pernil",
    "pernios": "pernio",
    "perniquebraba": "perniquebrar",
    "perniquebrabais": "perniquebrar",
    "perniquebraban": "perniquebrar",
    "perniquebrabas": "perniquebrar",
    "perniquebrad": "perniquebrar",
    "perniquebrada": "perniquebrar",
    "perniquebradas": "perniquebrar",
    "perniquebrado": "perniquebrar",
    "perniquebrados": "perniquebrar",
    "perniquebramos": "perniquebrar",
    "perniquebrando": "perniquebrar",
    "perniquebrara": "perniquebrar",
    "perniquebrarais": "perniquebrar",
    "perniquebraran": "perniquebrar",
    "perniquebraras": "perniquebrar",
    "perniquebrare": "perniquebrar",
    "perniquebrareis": "perniquebrar",
    "perniquebraremos": "perniquebrar",
    "perniquebraren": "perniquebrar",
    "perniquebrares": "perniquebrar",
    "perniquebraron": "perniquebrar",
    "perniquebrará": "perniquebrar",
    "perniquebrarán": "perniquebrar",
    "perniquebrarás": "perniquebrar",
    "perniquebraré": "perniquebrar",
    "perniquebraréis": "perniquebrar",
    "perniquebraría": "perniquebrar",
    "perniquebraríais": "perniquebrar",
    "perniquebraríamos": "perniquebrar",
    "perniquebrarían": "perniquebrar",
    "perniquebrarías": "perniquebrar",
    "perniquebrase": "perniquebrar",
    "perniquebraseis": "perniquebrar",
    "perniquebrasen": "perniquebrar",
    "perniquebrases": "perniquebrar",
    "perniquebraste": "perniquebrar",
    "perniquebrasteis": "perniquebrar",
    "perniquebremos": "perniquebrar",
    "perniquebrábamos": "perniquebrar",
    "perniquebráis": "perniquebrar",
    "perniquebráramos": "perniquebrar",
    "perniquebráremos": "perniquebrar",
    "perniquebrásemos": "perniquebrar",
    "perniquebré": "perniquebrar",
    "perniquebréis": "perniquebrar",
    "perniquebró": "perniquebrar",
    "perniquiebra": "perniquebrar",
    "perniquiebran": "perniquebrar",
    "perniquiebras": "perniquebrar",
    "perniquiebre": "perniquebrar",
    "perniquiebren": "perniquebrar",
    "perniquiebres": "perniquebrar",
    "perniquiebro": "perniquebrar",
    "pernitos": "pernito",
    "pernituerta": "pernituerto",
    "pernituertas": "pernituerto",
    "pernituertos": "pernituerto",
    "pernocta": "pernoctar",
    "pernoctaba": "pernoctar",
    "pernoctabais": "pernoctar",
    "pernoctaban": "pernoctar",
    "pernoctabas": "pernoctar",
    "pernoctaciones": "pernoctación",
    "pernoctad": "pernoctar",
    "pernoctada": "pernoctar",
    "pernoctadas": "pernoctar",
    "pernoctado": "pernoctar",
    "pernoctados": "pernoctar",
    "pernoctamos": "pernoctar",
    "pernoctan": "pernoctar",
    "pernoctando": "pernoctar",
    "pernoctara": "pernoctar",
    "pernoctarais": "pernoctar",
    "pernoctaran": "pernoctar",
    "pernoctaras": "pernoctar",
    "pernoctare": "pernoctar",
    "pernoctareis": "pernoctar",
    "pernoctaremos": "pernoctar",
    "pernoctaren": "pernoctar",
    "pernoctares": "pernoctar",
    "pernoctaron": "pernoctar",
    "pernoctará": "pernoctar",
    "pernoctarán": "pernoctar",
    "pernoctarás": "pernoctar",
    "pernoctaré": "pernoctar",
    "pernoctaréis": "pernoctar",
    "pernoctaría": "pernoctar",
    "pernoctaríais": "pernoctar",
    "pernoctaríamos": "pernoctar",
    "pernoctarían": "pernoctar",
    "pernoctarías": "pernoctar",
    "pernoctas": "pernoctar",
    "pernoctase": "pernoctar",
    "pernoctaseis": "pernoctar",
    "pernoctasen": "pernoctar",
    "pernoctases": "pernoctar",
    "pernoctaste": "pernoctar",
    "pernoctasteis": "pernoctar",
    "pernocte": "pernoctar",
    "pernoctemos": "pernoctar",
    "pernocten": "pernoctar",
    "pernoctes": "pernoctar",
    "pernocto": "pernoctar",
    "pernoctábamos": "pernoctar",
    "pernoctáis": "pernoctar",
    "pernoctáramos": "pernoctar",
    "pernoctáremos": "pernoctar",
    "pernoctásemos": "pernoctar",
    "pernocté": "pernoctar",
    "pernoctéis": "pernoctar",
    "pernoctó": "pernoctar",
    "pernos": "perno",
    "pernítrica": "pernítrico",
    "pernítricas": "pernítrico",
    "pernítricos": "pernítrico",
    "perogrulladas": "perogrullada",
    "perogrullesca": "perogrullesco",
    "perogrullescas": "perogrullesco",
    "perogrullescos": "perogrullesco",
    "perolas": "perola",
    "peroles": "perol",
    "peronea": "peroneo",
    "peroneales": "peroneal",
    "peroneas": "peroneo",
    "peroneoastragalina": "peroneoastragalino",
    "peroneoastragalinas": "peroneoastragalino",
    "peroneoastragalinos": "peroneoastragalino",
    "peroneos": "peroneo",
    "peronismos": "peronismo",
    "peronistas": "peronista",
    "peronés": "peroné",
    "perora": "perorar",
    "peroraba": "perorar",
    "perorabais": "perorar",
    "peroraban": "perorar",
    "perorabas": "perorar",
    "peroraciones": "peroración",
    "perorad": "perorar",
    "perorada": "perorar",
    "peroradas": "perorar",
    "perorado": "perorar",
    "perorados": "perorar",
    "perorales": "peroral",
    "peroramos": "perorar",
    "peroran": "perorar",
    "perorando": "perorar",
    "perorara": "perorar",
    "perorarais": "perorar",
    "peroraran": "perorar",
    "peroraras": "perorar",
    "perorare": "perorar",
    "perorareis": "perorar",
    "peroraremos": "perorar",
    "peroraren": "perorar",
    "perorares": "perorar",
    "peroraron": "perorar",
    "perorará": "perorar",
    "perorarán": "perorar",
    "perorarás": "perorar",
    "peroraré": "perorar",
    "peroraréis": "perorar",
    "peroraría": "perorar",
    "peroraríais": "perorar",
    "peroraríamos": "perorar",
    "perorarían": "perorar",
    "perorarías": "perorar",
    "peroras": "perorar",
    "perorase": "perorar",
    "peroraseis": "perorar",
    "perorasen": "perorar",
    "perorases": "perorar",
    "peroraste": "perorar",
    "perorasteis": "perorar",
    "peroratas": "perorata",
    "perore": "perorar",
    "peroremos": "perorar",
    "peroren": "perorar",
    "perores": "perorar",
    "peroro": "perorar",
    "perorábamos": "perorar",
    "peroráis": "perorar",
    "peroráramos": "perorar",
    "peroráremos": "perorar",
    "perorásemos": "perorar",
    "peroré": "perorar",
    "peroréis": "perorar",
    "peroró": "perorar",
    "peros": "pero",
    "peroxida": "peroxidar",
    "peroxidaba": "peroxidar",
    "peroxidabais": "peroxidar",
    "peroxidaban": "peroxidar",
    "peroxidabas": "peroxidar",
    "peroxidad": "peroxidar",
    "peroxidada": "peroxidar",
    "peroxidadas": "peroxidar",
    "peroxidado": "peroxidar",
    "peroxidados": "peroxidar",
    "peroxidamos": "peroxidar",
    "peroxidan": "peroxidar",
    "peroxidando": "peroxidar",
    "peroxidara": "peroxidar",
    "peroxidarais": "peroxidar",
    "peroxidaran": "peroxidar",
    "peroxidaras": "peroxidar",
    "peroxidare": "peroxidar",
    "peroxidareis": "peroxidar",
    "peroxidaremos": "peroxidar",
    "peroxidaren": "peroxidar",
    "peroxidares": "peroxidar",
    "peroxidaron": "peroxidar",
    "peroxidará": "peroxidar",
    "peroxidarán": "peroxidar",
    "peroxidarás": "peroxidar",
    "peroxidaré": "peroxidar",
    "peroxidaréis": "peroxidar",
    "peroxidaría": "peroxidar",
    "peroxidaríais": "peroxidar",
    "peroxidaríamos": "peroxidar",
    "peroxidarían": "peroxidar",
    "peroxidarías": "peroxidar",
    "peroxidas": "peroxidar",
    "peroxidasas": "peroxidasa",
    "peroxidase": "peroxidar",
    "peroxidaseis": "peroxidar",
    "peroxidasen": "peroxidar",
    "peroxidases": "peroxidar",
    "peroxidaste": "peroxidar",
    "peroxidasteis": "peroxidar",
    "peroxide": "peroxidar",
    "peroxidemos": "peroxidar",
    "peroxiden": "peroxidar",
    "peroxides": "peroxidar",
    "peroxidicarbonatos": "peroxidicarbonato",
    "peroxido": "peroxidar",
    "peroxidoacetilnitratos": "peroxidoacetilnitrato",
    "peroxidábamos": "peroxidar",
    "peroxidáis": "peroxidar",
    "peroxidáramos": "peroxidar",
    "peroxidáremos": "peroxidar",
    "peroxidásemos": "peroxidar",
    "peroxidé": "peroxidar",
    "peroxidéis": "peroxidar",
    "peroxidó": "peroxidar",
    "peroxisomas": "peroxisoma",
    "perpendiculares": "perpendicular",
    "perpendicularidades": "perpendicularidad",
    "perpetra": "perpetrar",
    "perpetraba": "perpetrar",
    "perpetrabais": "perpetrar",
    "perpetraban": "perpetrar",
    "perpetrabas": "perpetrar",
    "perpetraciones": "perpetración",
    "perpetrad": "perpetrar",
    "perpetrada": "perpetrar",
    "perpetradas": "perpetrar",
    "perpetrado": "perpetrar",
    "perpetradora": "perpetrador",
    "perpetradoras": "perpetrador",
    "perpetradores": "perpetrador",
    "perpetrados": "perpetrar",
    "perpetramos": "perpetrar",
    "perpetran": "perpetrar",
    "perpetrando": "perpetrar",
    "perpetrara": "perpetrar",
    "perpetrarais": "perpetrar",
    "perpetraran": "perpetrar",
    "perpetraras": "perpetrar",
    "perpetrare": "perpetrar",
    "perpetrareis": "perpetrar",
    "perpetraremos": "perpetrar",
    "perpetraren": "perpetrar",
    "perpetrares": "perpetrar",
    "perpetraron": "perpetrar",
    "perpetrará": "perpetrar",
    "perpetrarán": "perpetrar",
    "perpetrarás": "perpetrar",
    "perpetraré": "perpetrar",
    "perpetraréis": "perpetrar",
    "perpetraría": "perpetrar",
    "perpetraríais": "perpetrar",
    "perpetraríamos": "perpetrar",
    "perpetrarían": "perpetrar",
    "perpetrarías": "perpetrar",
    "perpetras": "perpetrar",
    "perpetrase": "perpetrar",
    "perpetraseis": "perpetrar",
    "perpetrasen": "perpetrar",
    "perpetrases": "perpetrar",
    "perpetraste": "perpetrar",
    "perpetrasteis": "perpetrar",
    "perpetre": "perpetrar",
    "perpetremos": "perpetrar",
    "perpetren": "perpetrar",
    "perpetres": "perpetrar",
    "perpetro": "perpetrar",
    "perpetrábamos": "perpetrar",
    "perpetráis": "perpetrar",
    "perpetráramos": "perpetrar",
    "perpetráremos": "perpetrar",
    "perpetrásemos": "perpetrar",
    "perpetré": "perpetrar",
    "perpetréis": "perpetrar",
    "perpetró": "perpetrar",
    "perpetua": "perpetuo",
    "perpetuaba": "perpetuar",
    "perpetuabais": "perpetuar",
    "perpetuaban": "perpetuar",
    "perpetuabas": "perpetuar",
    "perpetuaciones": "perpetuación",
    "perpetuad": "perpetuar",
    "perpetuada": "perpetuar",
    "perpetuadas": "perpetuar",
    "perpetuado": "perpetuar",
    "perpetuados": "perpetuar",
    "perpetuamos": "perpetuar",
    "perpetuando": "perpetuar",
    "perpetuara": "perpetuar",
    "perpetuarais": "perpetuar",
    "perpetuaran": "perpetuar",
    "perpetuaras": "perpetuar",
    "perpetuare": "perpetuar",
    "perpetuareis": "perpetuar",
    "perpetuaremos": "perpetuar",
    "perpetuaren": "perpetuar",
    "perpetuares": "perpetuar",
    "perpetuaron": "perpetuar",
    "perpetuará": "perpetuar",
    "perpetuarán": "perpetuar",
    "perpetuarás": "perpetuar",
    "perpetuaré": "perpetuar",
    "perpetuaréis": "perpetuar",
    "perpetuaría": "perpetuar",
    "perpetuaríais": "perpetuar",
    "perpetuaríamos": "perpetuar",
    "perpetuarían": "perpetuar",
    "perpetuarías": "perpetuar",
    "perpetuas": "perpetuo",
    "perpetuase": "perpetuar",
    "perpetuaseis": "perpetuar",
    "perpetuasen": "perpetuar",
    "perpetuases": "perpetuar",
    "perpetuaste": "perpetuar",
    "perpetuasteis": "perpetuar",
    "perpetuemos": "perpetuar",
    "perpetuidades": "perpetuidad",
    "perpetuos": "perpetuo",
    "perpetuábamos": "perpetuar",
    "perpetuáis": "perpetuar",
    "perpetuáramos": "perpetuar",
    "perpetuáremos": "perpetuar",
    "perpetuásemos": "perpetuar",
    "perpetué": "perpetuar",
    "perpetuéis": "perpetuar",
    "perpetuó": "perpetuar",
    "perpetúa": "perpetuar",
    "perpetúan": "perpetuar",
    "perpetúas": "perpetuar",
    "perpetúe": "perpetuar",
    "perpetúen": "perpetuar",
    "perpetúes": "perpetuar",
    "perpetúo": "perpetuar",
    "perpiaña": "perpiaño",
    "perpiañas": "perpiaño",
    "perpiaños": "perpiaño",
    "perpleja": "perplejo",
    "perplejas": "perplejo",
    "perplejidades": "perplejidad",
    "perplejos": "perplejo",
    "perpuntes": "perpunte",
    "perquiera": "perquirir",
    "perquieran": "perquirir",
    "perquieras": "perquirir",
    "perquiere": "perquirir",
    "perquieren": "perquirir",
    "perquieres": "perquirir",
    "perquiero": "perquirir",
    "perquiramos": "perquirir",
    "perquirid": "perquirir",
    "perquirida": "perquirir",
    "perquiridas": "perquirir",
    "perquirido": "perquirir",
    "perquiridos": "perquirir",
    "perquiriendo": "perquirir",
    "perquiriera": "perquirir",
    "perquirierais": "perquirir",
    "perquirieran": "perquirir",
    "perquirieras": "perquirir",
    "perquiriere": "perquirir",
    "perquiriereis": "perquirir",
    "perquirieren": "perquirir",
    "perquirieres": "perquirir",
    "perquirieron": "perquirir",
    "perquiriese": "perquirir",
    "perquirieseis": "perquirir",
    "perquiriesen": "perquirir",
    "perquirieses": "perquirir",
    "perquirimos": "perquirir",
    "perquiriremos": "perquirir",
    "perquirirá": "perquirir",
    "perquirirán": "perquirir",
    "perquirirás": "perquirir",
    "perquiriré": "perquirir",
    "perquiriréis": "perquirir",
    "perquiriría": "perquirir",
    "perquiriríais": "perquirir",
    "perquiriríamos": "perquirir",
    "perquirirían": "perquirir",
    "perquirirías": "perquirir",
    "perquiriste": "perquirir",
    "perquiristeis": "perquirir",
    "perquiriéramos": "perquirir",
    "perquiriéremos": "perquirir",
    "perquiriésemos": "perquirir",
    "perquirió": "perquirir",
    "perquiráis": "perquirir",
    "perquirí": "perquirir",
    "perquiría": "perquirir",
    "perquiríais": "perquirir",
    "perquiríamos": "perquirir",
    "perquirían": "perquirir",
    "perquirías": "perquirir",
    "perquirís": "perquirir",
    "perquisiciones": "perquisición",
    "perqués": "perqué",
    "perra": "perro",
    "perradas": "perrada",
    "perras": "perro",
    "perrengues": "perrengue",
    "perrera": "perrero",
    "perreras": "perrero",
    "perreros": "perrero",
    "perrerías": "perrería",
    "perrilla": "perrillo",
    "perrillas": "perrillo",
    "perrillos": "perrillo",
    "perrita": "perrito",
    "perritas": "perrito",
    "perritos": "perrito",
    "perros": "perro",
    "perruna": "perruno",
    "perrunas": "perruno",
    "perrunos": "perruno",
    "persas": "persa",
    "persecuciones": "persecución",
    "persecutoria": "persecutorio",
    "persecutorias": "persecutorio",
    "persecutorios": "persecutorio",
    "perseguibilidades": "perseguibilidad",
    "perseguibles": "perseguible",
    "perseguid": "perseguir",
    "perseguida": "perseguir",
    "perseguidas": "perseguir",
    "perseguido": "perseguir",
    "perseguidora": "perseguidor",
    "perseguidoras": "perseguidor",
    "perseguidores": "perseguidor",
    "perseguidos": "perseguir",
    "perseguimos": "perseguir",
    "perseguiremos": "perseguir",
    "perseguirá": "perseguir",
    "perseguirán": "perseguir",
    "perseguirás": "perseguir",
    "perseguiré": "perseguir",
    "perseguiréis": "perseguir",
    "perseguiría": "perseguir",
    "perseguiríais": "perseguir",
    "perseguiríamos": "perseguir",
    "perseguirían": "perseguir",
    "perseguirías": "perseguir",
    "perseguiste": "perseguir",
    "perseguisteis": "perseguir",
    "perseguí": "perseguir",
    "perseguía": "perseguir",
    "perseguíais": "perseguir",
    "perseguíamos": "perseguir",
    "perseguían": "perseguir",
    "perseguías": "perseguir",
    "perseguís": "perseguir",
    "persevantes": "persevante",
    "persevera": "perseverar",
    "perseveraba": "perseverar",
    "perseverabais": "perseverar",
    "perseveraban": "perseverar",
    "perseverabas": "perseverar",
    "perseveraciones": "perseveración",
    "perseverad": "perseverar",
    "perseverada": "perseverar",
    "perseveradas": "perseverar",
    "perseverado": "perseverar",
    "perseverados": "perseverar",
    "perseveramos": "perseverar",
    "perseveran": "perseverar",
    "perseverancias": "perseverancia",
    "perseverando": "perseverar",
    "perseverantes": "perseverante",
    "perseverara": "perseverar",
    "perseverarais": "perseverar",
    "perseveraran": "perseverar",
    "perseveraras": "perseverar",
    "perseverare": "perseverar",
    "perseverareis": "perseverar",
    "perseveraremos": "perseverar",
    "perseveraren": "perseverar",
    "perseverares": "perseverar",
    "perseveraron": "perseverar",
    "perseverará": "perseverar",
    "perseverarán": "perseverar",
    "perseverarás": "perseverar",
    "perseveraré": "perseverar",
    "perseveraréis": "perseverar",
    "perseveraría": "perseverar",
    "perseveraríais": "perseverar",
    "perseveraríamos": "perseverar",
    "perseverarían": "perseverar",
    "perseverarías": "perseverar",
    "perseveras": "perseverar",
    "perseverase": "perseverar",
    "perseveraseis": "perseverar",
    "perseverasen": "perseverar",
    "perseverases": "perseverar",
    "perseveraste": "perseverar",
    "perseverasteis": "perseverar",
    "persevere": "perseverar",
    "perseveremos": "perseverar",
    "perseveren": "perseverar",
    "perseveres": "perseverar",
    "persevero": "perseverar",
    "perseverábamos": "perseverar",
    "perseveráis": "perseverar",
    "perseveráramos": "perseverar",
    "perseveráremos": "perseverar",
    "perseverásemos": "perseverar",
    "perseveré": "perseverar",
    "perseveréis": "perseverar",
    "perseveró": "perseverar",
    "persiana": "persiano",
    "persianas": "persiano",
    "persianistas": "persianista",
    "persianos": "persiano",
    "persiga": "perseguir",
    "persigamos": "perseguir",
    "persigan": "perseguir",
    "persigas": "perseguir",
    "persigna": "persignar",
    "persignaba": "persignar",
    "persignabais": "persignar",
    "persignaban": "persignar",
    "persignabas": "persignar",
    "persignad": "persignar",
    "persignada": "persignar",
    "persignadas": "persignar",
    "persignado": "persignar",
    "persignados": "persignar",
    "persignamos": "persignar",
    "persignan": "persignar",
    "persignando": "persignar",
    "persignara": "persignar",
    "persignarais": "persignar",
    "persignaran": "persignar",
    "persignaras": "persignar",
    "persignare": "persignar",
    "persignareis": "persignar",
    "persignaremos": "persignar",
    "persignaren": "persignar",
    "persignares": "persignar",
    "persignaron": "persignar",
    "persignará": "persignar",
    "persignarán": "persignar",
    "persignarás": "persignar",
    "persignaré": "persignar",
    "persignaréis": "persignar",
    "persignaría": "persignar",
    "persignaríais": "persignar",
    "persignaríamos": "persignar",
    "persignarían": "persignar",
    "persignarías": "persignar",
    "persignas": "persignar",
    "persignase": "persignar",
    "persignaseis": "persignar",
    "persignasen": "persignar",
    "persignases": "persignar",
    "persignaste": "persignar",
    "persignasteis": "persignar",
    "persigne": "persignar",
    "persignemos": "persignar",
    "persignen": "persignar",
    "persignes": "persignar",
    "persigno": "persignar",
    "persignábamos": "persignar",
    "persignáis": "persignar",
    "persignáramos": "persignar",
    "persignáremos": "persignar",
    "persignásemos": "persignar",
    "persigné": "persignar",
    "persignéis": "persignar",
    "persignó": "persignar",
    "persigo": "perseguir",
    "persigue": "perseguir",
    "persiguen": "perseguir",
    "persigues": "perseguir",
    "persiguiendo": "perseguir",
    "persiguiera": "perseguir",
    "persiguierais": "perseguir",
    "persiguieran": "perseguir",
    "persiguieras": "perseguir",
    "persiguiere": "perseguir",
    "persiguiereis": "perseguir",
    "persiguieren": "perseguir",
    "persiguieres": "perseguir",
    "persiguieron": "perseguir",
    "persiguiese": "perseguir",
    "persiguieseis": "perseguir",
    "persiguiesen": "perseguir",
    "persiguieses": "perseguir",
    "persiguiéramos": "perseguir",
    "persiguiéremos": "perseguir",
    "persiguiésemos": "perseguir",
    "persiguió": "perseguir",
    "persigáis": "perseguir",
    "persista": "persistir",
    "persistamos": "persistir",
    "persistan": "persistir",
    "persistas": "persistir",
    "persiste": "persistir",
    "persisten": "persistir",
    "persistenciaas": "persistencia",
    "persistencias": "persistencia",
    "persistentes": "persistente",
    "persistes": "persistir",
    "persistid": "persistir",
    "persistida": "persistir",
    "persistidas": "persistir",
    "persistido": "persistir",
    "persistidos": "persistir",
    "persistiendo": "persistir",
    "persistiera": "persistir",
    "persistierais": "persistir",
    "persistieran": "persistir",
    "persistieras": "persistir",
    "persistiere": "persistir",
    "persistiereis": "persistir",
    "persistieren": "persistir",
    "persistieres": "persistir",
    "persistieron": "persistir",
    "persistiese": "persistir",
    "persistieseis": "persistir",
    "persistiesen": "persistir",
    "persistieses": "persistir",
    "persistimos": "persistir",
    "persistiremos": "persistir",
    "persistirá": "persistir",
    "persistirán": "persistir",
    "persistirás": "persistir",
    "persistiré": "persistir",
    "persistiréis": "persistir",
    "persistiría": "persistir",
    "persistiríais": "persistir",
    "persistiríamos": "persistir",
    "persistirían": "persistir",
    "persistirías": "persistir",
    "persististe": "persistir",
    "persististeis": "persistir",
    "persistiéramos": "persistir",
    "persistiéremos": "persistir",
    "persistiésemos": "persistir",
    "persistió": "persistir",
    "persisto": "persistir",
    "persistáis": "persistir",
    "persistí": "persistir",
    "persistía": "persistir",
    "persistíais": "persistir",
    "persistíamos": "persistir",
    "persistían": "persistir",
    "persistías": "persistir",
    "persistís": "persistir",
    "persona": "personar",
    "personaba": "personar",
    "personabais": "personar",
    "personaban": "personar",
    "personabas": "personar",
    "personaciones": "personación",
    "personad": "personar",
    "personada": "personar",
    "personadas": "personar",
    "personado": "personar",
    "personados": "personar",
    "personajes": "personaje",
    "personales": "personal",
    "personalice": "personalizar",
    "personalicemos": "personalizar",
    "personalicen": "personalizar",
    "personalices": "personalizar",
    "personalicé": "personalizar",
    "personalicéis": "personalizar",
    "personalidades": "personalidad",
    "personalismos": "personalismo",
    "personalistas": "personalista",
    "personaliza": "personalizar",
    "personalizaba": "personalizar",
    "personalizabais": "personalizar",
    "personalizaban": "personalizar",
    "personalizabas": "personalizar",
    "personalizaciones": "personalización",
    "personalizad": "personalizar",
    "personalizada": "personalizar",
    "personalizadas": "personalizar",
    "personalizado": "personalizar",
    "personalizados": "personalizar",
    "personalizamos": "personalizar",
    "personalizan": "personalizar",
    "personalizando": "personalizar",
    "personalizara": "personalizar",
    "personalizarais": "personalizar",
    "personalizaran": "personalizar",
    "personalizaras": "personalizar",
    "personalizare": "personalizar",
    "personalizareis": "personalizar",
    "personalizaremos": "personalizar",
    "personalizaren": "personalizar",
    "personalizares": "personalizar",
    "personalizaron": "personalizar",
    "personalizará": "personalizar",
    "personalizarán": "personalizar",
    "personalizarás": "personalizar",
    "personalizaré": "personalizar",
    "personalizaréis": "personalizar",
    "personalizaría": "personalizar",
    "personalizaríais": "personalizar",
    "personalizaríamos": "personalizar",
    "personalizarían": "personalizar",
    "personalizarías": "personalizar",
    "personalizas": "personalizar",
    "personalizase": "personalizar",
    "personalizaseis": "personalizar",
    "personalizasen": "personalizar",
    "personalizases": "personalizar",
    "personalizaste": "personalizar",
    "personalizasteis": "personalizar",
    "personalizo": "personalizar",
    "personalizábamos": "personalizar",
    "personalizáis": "personalizar",
    "personalizáramos": "personalizar",
    "personalizáremos": "personalizar",
    "personalizásemos": "personalizar",
    "personalizó": "personalizar",
    "personamos": "personar",
    "personan": "personar",
    "personando": "personar",
    "personara": "personar",
    "personarais": "personar",
    "personaran": "personar",
    "personaras": "personar",
    "personare": "personar",
    "personareis": "personar",
    "personaremos": "personar",
    "personaren": "personar",
    "personares": "personar",
    "personaron": "personar",
    "personará": "personar",
    "personarán": "personar",
    "personarás": "personar",
    "personaré": "personar",
    "personaréis": "personar",
    "personaría": "personar",
    "personaríais": "personar",
    "personaríamos": "personar",
    "personarían": "personar",
    "personarías": "personar",
    "personas": "personar",
    "personase": "personar",
    "personaseis": "personar",
    "personasen": "personar",
    "personases": "personar",
    "personaste": "personar",
    "personasteis": "personar",
    "persone": "personar",
    "personemos": "personar",
    "personen": "personar",
    "persones": "personar",
    "personifica": "personificar",
    "personificaba": "personificar",
    "personificabais": "personificar",
    "personificaban": "personificar",
    "personificabas": "personificar",
    "personificaciones": "personificación",
    "personificad": "personificar",
    "personificada": "personificar",
    "personificadas": "personificar",
    "personificado": "personificar",
    "personificados": "personificar",
    "personificamos": "personificar",
    "personifican": "personificar",
    "personificando": "personificar",
    "personificara": "personificar",
    "personificarais": "personificar",
    "personificaran": "personificar",
    "personificaras": "personificar",
    "personificare": "personificar",
    "personificareis": "personificar",
    "personificaremos": "personificar",
    "personificaren": "personificar",
    "personificares": "personificar",
    "personificaron": "personificar",
    "personificará": "personificar",
    "personificarán": "personificar",
    "personificarás": "personificar",
    "personificaré": "personificar",
    "personificaréis": "personificar",
    "personificaría": "personificar",
    "personificaríais": "personificar",
    "personificaríamos": "personificar",
    "personificarían": "personificar",
    "personificarías": "personificar",
    "personificas": "personificar",
    "personificase": "personificar",
    "personificaseis": "personificar",
    "personificasen": "personificar",
    "personificases": "personificar",
    "personificaste": "personificar",
    "personificasteis": "personificar",
    "personifico": "personificar",
    "personificábamos": "personificar",
    "personificáis": "personificar",
    "personificáramos": "personificar",
    "personificáremos": "personificar",
    "personificásemos": "personificar",
    "personificó": "personificar",
    "personifique": "personificar",
    "personifiquemos": "personificar",
    "personifiquen": "personificar",
    "personifiques": "personificar",
    "personifiqué": "personificar",
    "personifiquéis": "personificar",
    "personillas": "personilla",
    "persono": "personar",
    "personábamos": "personar",
    "personáis": "personar",
    "personáramos": "personar",
    "personáremos": "personar",
    "personásemos": "personar",
    "personé": "personar",
    "personéis": "personar",
    "personó": "personar",
    "perspectiva": "perspectivo",
    "perspectivas": "perspectivo",
    "perspectividades": "perspectividad",
    "perspectivismos": "perspectivismo",
    "perspectivos": "perspectivo",
    "perspexes": "perspex",
    "perspicaces": "perspicaz",
    "perspicacias": "perspicacia",
    "perspicazes": "perspicaz",
    "perspicua": "perspicuo",
    "perspicuas": "perspicuo",
    "perspicuidades": "perspicuidad",
    "perspicuos": "perspicuo",
    "persuada": "persuadir",
    "persuadamos": "persuadir",
    "persuadan": "persuadir",
    "persuadas": "persuadir",
    "persuade": "persuadir",
    "persuaden": "persuadir",
    "persuades": "persuadir",
    "persuadid": "persuadir",
    "persuadida": "persuadir",
    "persuadidas": "persuadir",
    "persuadido": "persuadir",
    "persuadidora": "persuadidor",
    "persuadidoras": "persuadidor",
    "persuadidores": "persuadidor",
    "persuadidos": "persuadir",
    "persuadiendo": "persuadir",
    "persuadiera": "persuadir",
    "persuadierais": "persuadir",
    "persuadieran": "persuadir",
    "persuadieras": "persuadir",
    "persuadiere": "persuadir",
    "persuadiereis": "persuadir",
    "persuadieren": "persuadir",
    "persuadieres": "persuadir",
    "persuadieron": "persuadir",
    "persuadiese": "persuadir",
    "persuadieseis": "persuadir",
    "persuadiesen": "persuadir",
    "persuadieses": "persuadir",
    "persuadimos": "persuadir",
    "persuadiremos": "persuadir",
    "persuadirá": "persuadir",
    "persuadirán": "persuadir",
    "persuadirás": "persuadir",
    "persuadiré": "persuadir",
    "persuadiréis": "persuadir",
    "persuadiría": "persuadir",
    "persuadiríais": "persuadir",
    "persuadiríamos": "persuadir",
    "persuadirían": "persuadir",
    "persuadirías": "persuadir",
    "persuadiste": "persuadir",
    "persuadisteis": "persuadir",
    "persuadiéramos": "persuadir",
    "persuadiéremos": "persuadir",
    "persuadiésemos": "persuadir",
    "persuadió": "persuadir",
    "persuado": "persuadir",
    "persuadáis": "persuadir",
    "persuadí": "persuadir",
    "persuadía": "persuadir",
    "persuadíais": "persuadir",
    "persuadíamos": "persuadir",
    "persuadían": "persuadir",
    "persuadías": "persuadir",
    "persuadís": "persuadir",
    "persuasibles": "persuasible",
    "persuasiones": "persuasión",
    "persuasiva": "persuasivo",
    "persuasivas": "persuasivo",
    "persuasivos": "persuasivo",
    "persuasora": "persuasor",
    "persuasoras": "persuasor",
    "persuasores": "persuasor",
    "persuasoria": "persuasorio",
    "persuasorias": "persuasorio",
    "persuasorios": "persuasorio",
    "persulfatos": "persulfato",
    "pertenece": "pertenecer",
    "perteneced": "pertenecer",
    "pertenecemos": "pertenecer",
    "pertenecen": "pertenecer",
    "perteneceremos": "pertenecer",
    "pertenecerá": "pertenecer",
    "pertenecerán": "pertenecer",
    "pertenecerás": "pertenecer",
    "perteneceré": "pertenecer",
    "perteneceréis": "pertenecer",
    "pertenecería": "pertenecer",
    "perteneceríais": "pertenecer",
    "perteneceríamos": "pertenecer",
    "pertenecerían": "pertenecer",
    "pertenecerías": "pertenecer",
    "perteneces": "pertenecer",
    "pertenecida": "pertenecer",
    "pertenecidas": "pertenecer",
    "pertenecido": "pertenecer",
    "pertenecidos": "pertenecer",
    "perteneciendo": "pertenecer",
    "pertenecientes": "perteneciente",
    "perteneciera": "pertenecer",
    "pertenecierais": "pertenecer",
    "pertenecieran": "pertenecer",
    "pertenecieras": "pertenecer",
    "perteneciere": "pertenecer",
    "perteneciereis": "pertenecer",
    "pertenecieren": "pertenecer",
    "pertenecieres": "pertenecer",
    "pertenecieron": "pertenecer",
    "perteneciese": "pertenecer",
    "pertenecieseis": "pertenecer",
    "perteneciesen": "pertenecer",
    "pertenecieses": "pertenecer",
    "pertenecimos": "pertenecer",
    "perteneciste": "pertenecer",
    "pertenecisteis": "pertenecer",
    "perteneciéramos": "pertenecer",
    "perteneciéremos": "pertenecer",
    "perteneciésemos": "pertenecer",
    "perteneció": "pertenecer",
    "pertenecéis": "pertenecer",
    "pertenecí": "pertenecer",
    "pertenecía": "pertenecer",
    "pertenecíais": "pertenecer",
    "pertenecíamos": "pertenecer",
    "pertenecían": "pertenecer",
    "pertenecías": "pertenecer",
    "pertenencias": "pertenencia",
    "pertenezca": "pertenecer",
    "pertenezcamos": "pertenecer",
    "pertenezcan": "pertenecer",
    "pertenezcas": "pertenecer",
    "pertenezco": "pertenecer",
    "pertenezcáis": "pertenecer",
    "perties": "party",
    "pertigales": "pertigal",
    "pertigueros": "pertiguero",
    "pertiguerías": "pertiguería",
    "pertiguistas": "pertiguista",
    "pertinaces": "pertinaz",
    "pertinacias": "pertinacia",
    "pertinazes": "pertinaz",
    "pertinencias": "pertinencia",
    "pertinentes": "pertinente",
    "pertitas": "pertita",
    "pertrecha": "pertrechar",
    "pertrechaba": "pertrechar",
    "pertrechabais": "pertrechar",
    "pertrechaban": "pertrechar",
    "pertrechabas": "pertrechar",
    "pertrechad": "pertrechar",
    "pertrechada": "pertrechar",
    "pertrechadas": "pertrechar",
    "pertrechado": "pertrechar",
    "pertrechados": "pertrechar",
    "pertrechamos": "pertrechar",
    "pertrechan": "pertrechar",
    "pertrechando": "pertrechar",
    "pertrechara": "pertrechar",
    "pertrecharais": "pertrechar",
    "pertrecharan": "pertrechar",
    "pertrecharas": "pertrechar",
    "pertrechare": "pertrechar",
    "pertrechareis": "pertrechar",
    "pertrecharemos": "pertrechar",
    "pertrecharen": "pertrechar",
    "pertrechares": "pertrechar",
    "pertrecharon": "pertrechar",
    "pertrechará": "pertrechar",
    "pertrecharán": "pertrechar",
    "pertrecharás": "pertrechar",
    "pertrecharé": "pertrechar",
    "pertrecharéis": "pertrechar",
    "pertrecharía": "pertrechar",
    "pertrecharíais": "pertrechar",
    "pertrecharíamos": "pertrechar",
    "pertrecharían": "pertrechar",
    "pertrecharías": "pertrechar",
    "pertrechas": "pertrechar",
    "pertrechase": "pertrechar",
    "pertrechaseis": "pertrechar",
    "pertrechasen": "pertrechar",
    "pertrechases": "pertrechar",
    "pertrechaste": "pertrechar",
    "pertrechasteis": "pertrechar",
    "pertreche": "pertrechar",
    "pertrechemos": "pertrechar",
    "pertrechen": "pertrechar",
    "pertreches": "pertrechar",
    "pertrecho": "pertrechar",
    "pertrechábamos": "pertrechar",
    "pertrecháis": "pertrechar",
    "pertrecháramos": "pertrechar",
    "pertrecháremos": "pertrechar",
    "pertrechásemos": "pertrechar",
    "pertreché": "pertrechar",
    "pertrechéis": "pertrechar",
    "pertrechó": "pertrechar",
    "perturba": "perturbar",
    "perturbaba": "perturbar",
    "perturbabais": "perturbar",
    "perturbaban": "perturbar",
    "perturbabas": "perturbar",
    "perturbables": "perturbable",
    "perturbaciones": "perturbación",
    "perturbad": "perturbar",
    "perturbada": "perturbar",
    "perturbadas": "perturbar",
    "perturbado": "perturbar",
    "perturbadora": "perturbador",
    "perturbadoras": "perturbador",
    "perturbadores": "perturbador",
    "perturbados": "perturbar",
    "perturbamos": "perturbar",
    "perturban": "perturbar",
    "perturbando": "perturbar",
    "perturbara": "perturbar",
    "perturbarais": "perturbar",
    "perturbaran": "perturbar",
    "perturbaras": "perturbar",
    "perturbare": "perturbar",
    "perturbareis": "perturbar",
    "perturbaremos": "perturbar",
    "perturbaren": "perturbar",
    "perturbares": "perturbar",
    "perturbaron": "perturbar",
    "perturbará": "perturbar",
    "perturbarán": "perturbar",
    "perturbarás": "perturbar",
    "perturbaré": "perturbar",
    "perturbaréis": "perturbar",
    "perturbaría": "perturbar",
    "perturbaríais": "perturbar",
    "perturbaríamos": "perturbar",
    "perturbarían": "perturbar",
    "perturbarías": "perturbar",
    "perturbas": "perturbar",
    "perturbase": "perturbar",
    "perturbaseis": "perturbar",
    "perturbasen": "perturbar",
    "perturbases": "perturbar",
    "perturbaste": "perturbar",
    "perturbasteis": "perturbar",
    "perturbe": "perturbar",
    "perturbemos": "perturbar",
    "perturben": "perturbar",
    "perturbes": "perturbar",
    "perturbo": "perturbar",
    "perturbábamos": "perturbar",
    "perturbáis": "perturbar",
    "perturbáramos": "perturbar",
    "perturbáremos": "perturbar",
    "perturbásemos": "perturbar",
    "perturbé": "perturbar",
    "perturbéis": "perturbar",
    "perturbó": "perturbar",
    "peruana": "peruano",
    "peruanas": "peruano",
    "peruanismos": "peruanismo",
    "peruanos": "peruano",
    "perulera": "perulero",
    "peruleras": "perulero",
    "peruleros": "perulero",
    "peruviana": "peruviano",
    "peruvianas": "peruviano",
    "peruvianos": "peruviano",
    "perveancias": "perveancia",
    "perversa": "perverso",
    "perversas": "perverso",
    "perversidades": "perversidad",
    "perversiones": "perversión",
    "perversos": "perverso",
    "pervertid": "pervertir",
    "pervertida": "pervertir",
    "pervertidas": "pervertir",
    "pervertido": "pervertir",
    "pervertidos": "pervertir",
    "pervertimos": "pervertir",
    "pervertiremos": "pervertir",
    "pervertirá": "pervertir",
    "pervertirán": "pervertir",
    "pervertirás": "pervertir",
    "pervertiré": "pervertir",
    "pervertiréis": "pervertir",
    "pervertiría": "pervertir",
    "pervertiríais": "pervertir",
    "pervertiríamos": "pervertir",
    "pervertirían": "pervertir",
    "pervertirías": "pervertir",
    "pervertiste": "pervertir",
    "pervertisteis": "pervertir",
    "pervertí": "pervertir",
    "pervertía": "pervertir",
    "pervertíais": "pervertir",
    "pervertíamos": "pervertir",
    "pervertían": "pervertir",
    "pervertías": "pervertir",
    "pervertís": "pervertir",
    "pervibraciones": "pervibración",
    "pervibradores": "pervibrador",
    "pervierta": "pervertir",
    "perviertan": "pervertir",
    "perviertas": "pervertir",
    "pervierte": "pervertir",
    "pervierten": "pervertir",
    "perviertes": "pervertir",
    "pervierto": "pervertir",
    "pervigilios": "pervigilio",
    "pervirtamos": "pervertir",
    "pervirtiendo": "pervertir",
    "pervirtiera": "pervertir",
    "pervirtierais": "pervertir",
    "pervirtieran": "pervertir",
    "pervirtieras": "pervertir",
    "pervirtiere": "pervertir",
    "pervirtiereis": "pervertir",
    "pervirtieren": "pervertir",
    "pervirtieres": "pervertir",
    "pervirtieron": "pervertir",
    "pervirtiese": "pervertir",
    "pervirtieseis": "pervertir",
    "pervirtiesen": "pervertir",
    "pervirtieses": "pervertir",
    "pervirtiéramos": "pervertir",
    "pervirtiéremos": "pervertir",
    "pervirtiésemos": "pervertir",
    "pervirtió": "pervertir",
    "pervirtáis": "pervertir",
    "perviva": "pervivir",
    "pervivamos": "pervivir",
    "pervivan": "pervivir",
    "pervivas": "pervivir",
    "pervive": "pervivir",
    "perviven": "pervivir",
    "pervivencias": "pervivencia",
    "pervives": "pervivir",
    "pervivid": "pervivir",
    "pervivida": "pervivir",
    "pervividas": "pervivir",
    "pervivido": "pervivir",
    "pervividos": "pervivir",
    "perviviendo": "pervivir",
    "pervivientes": "perviviente",
    "perviviera": "pervivir",
    "pervivierais": "pervivir",
    "pervivieran": "pervivir",
    "pervivieras": "pervivir",
    "perviviere": "pervivir",
    "perviviereis": "pervivir",
    "pervivieren": "pervivir",
    "pervivieres": "pervivir",
    "pervivieron": "pervivir",
    "perviviese": "pervivir",
    "pervivieseis": "pervivir",
    "perviviesen": "pervivir",
    "pervivieses": "pervivir",
    "pervivimos": "pervivir",
    "perviviremos": "pervivir",
    "pervivirá": "pervivir",
    "pervivirán": "pervivir",
    "pervivirás": "pervivir",
    "perviviré": "pervivir",
    "perviviréis": "pervivir",
    "perviviría": "pervivir",
    "perviviríais": "pervivir",
    "perviviríamos": "pervivir",
    "pervivirían": "pervivir",
    "pervivirías": "pervivir",
    "perviviste": "pervivir",
    "pervivisteis": "pervivir",
    "perviviéramos": "pervivir",
    "perviviéremos": "pervivir",
    "perviviésemos": "pervivir",
    "pervivió": "pervivir",
    "pervivo": "pervivir",
    "perviváis": "pervivir",
    "perviví": "pervivir",
    "pervivía": "pervivir",
    "pervivíais": "pervivir",
    "pervivíamos": "pervivir",
    "pervivían": "pervivir",
    "pervivías": "pervivir",
    "pervivís": "pervivir",
    "peryodatos": "peryodato",
    "perácidos": "perácido",
    "perímetros": "perímetro",
    "períocas": "períoca",
    "períodos": "período",
    "períptera": "períptero",
    "perípteras": "períptero",
    "perípteros": "períptero",
    "perístoles": "perístole",
    "peróxidos": "peróxido",
    "pesa": "pesar",
    "pesaba": "pesar",
    "pesabais": "pesar",
    "pesaban": "pesar",
    "pesabas": "pesar",
    "pesad": "pesar",
    "pesada": "pesar",
    "pesadas": "pesar",
    "pesadeces": "pesadez",
    "pesadillas": "pesadilla",
    "pesado": "pesar",
    "pesadora": "pesador",
    "pesadoras": "pesador",
    "pesadores": "pesador",
    "pesados": "pesar",
    "pesadumbres": "pesadumbre",
    "pesajes": "pesaje",
    "pesamos": "pesar",
    "pesan": "pesar",
    "pesando": "pesar",
    "pesantes": "pesante",
    "pesara": "pesar",
    "pesarais": "pesar",
    "pesaran": "pesar",
    "pesaras": "pesar",
    "pesare": "pesar",
    "pesareis": "pesar",
    "pesaremos": "pesar",
    "pesaren": "pesar",
    "pesares": "pesar",
    "pesaron": "pesar",
    "pesarosa": "pesaroso",
    "pesarosas": "pesaroso",
    "pesarosos": "pesaroso",
    "pesará": "pesar",
    "pesarán": "pesar",
    "pesarás": "pesar",
    "pesaré": "pesar",
    "pesaréis": "pesar",
    "pesaría": "pesar",
    "pesaríais": "pesar",
    "pesaríamos": "pesar",
    "pesarían": "pesar",
    "pesarías": "pesar",
    "pesas": "pesar",
    "pesase": "pesar",
    "pesaseis": "pesar",
    "pesasen": "pesar",
    "pesases": "pesar",
    "pesaste": "pesar",
    "pesasteis": "pesar",
    "pesca": "pescar",
    "pescaba": "pescar",
    "pescabais": "pescar",
    "pescaban": "pescar",
    "pescabas": "pescar",
    "pescables": "pescable",
    "pescad": "pescar",
    "pescada": "pescar",
    "pescadas": "pescar",
    "pescadera": "pescadero",
    "pescaderas": "pescadero",
    "pescaderos": "pescadero",
    "pescaderías": "pescadería",
    "pescadillas": "pescadilla",
    "pescado": "pescar",
    "pescadora": "pescador",
    "pescadoras": "pescador",
    "pescadores": "pescador",
    "pescados": "pescar",
    "pescamos": "pescar",
    "pescan": "pescar",
    "pescando": "pescar",
    "pescantes": "pescante",
    "pescara": "pescar",
    "pescarais": "pescar",
    "pescaran": "pescar",
    "pescaras": "pescar",
    "pescare": "pescar",
    "pescareis": "pescar",
    "pescaremos": "pescar",
    "pescaren": "pescar",
    "pescares": "pescar",
    "pescaron": "pescar",
    "pescará": "pescar",
    "pescarán": "pescar",
    "pescarás": "pescar",
    "pescaré": "pescar",
    "pescaréis": "pescar",
    "pescaría": "pescar",
    "pescaríais": "pescar",
    "pescaríamos": "pescar",
    "pescarían": "pescar",
    "pescarías": "pescar",
    "pescas": "pescar",
    "pescase": "pescar",
    "pescaseis": "pescar",
    "pescasen": "pescar",
    "pescases": "pescar",
    "pescaste": "pescar",
    "pescasteis": "pescar",
    "pesco": "pescar",
    "pescozadas": "pescozada",
    "pescozones": "pescozón",
    "pescozuda": "pescozudo",
    "pescozudas": "pescozudo",
    "pescozudos": "pescozudo",
    "pescuezos": "pescuezo",
    "pescuños": "pescuño",
    "pescábamos": "pescar",
    "pescáis": "pescar",
    "pescáramos": "pescar",
    "pescáremos": "pescar",
    "pescásemos": "pescar",
    "pescó": "pescar",
    "pese": "pesar",
    "pesebreras": "pesebrera",
    "pesebres": "pesebre",
    "pesebrones": "pesebrón",
    "pesemos": "pesar",
    "pesen": "pesar",
    "peses": "pesar",
    "pesetas": "peseta",
    "pesetera": "pesetero",
    "peseteras": "pesetero",
    "peseteros": "pesetero",
    "peshmergas": "peshmerga",
    "pesillos": "pesillo",
    "pesimismos": "pesimismo",
    "pesimistas": "pesimista",
    "pesistas": "pesista",
    "peso": "pesar",
    "pesos": "peso",
    "pespunta": "pespuntar",
    "pespuntaba": "pespuntar",
    "pespuntabais": "pespuntar",
    "pespuntaban": "pespuntar",
    "pespuntabas": "pespuntar",
    "pespuntad": "pespuntar",
    "pespuntada": "pespuntar",
    "pespuntadas": "pespuntar",
    "pespuntado": "pespuntar",
    "pespuntados": "pespuntar",
    "pespuntamos": "pespuntar",
    "pespuntan": "pespuntar",
    "pespuntando": "pespuntar",
    "pespuntara": "pespuntar",
    "pespuntarais": "pespuntar",
    "pespuntaran": "pespuntar",
    "pespuntaras": "pespuntar",
    "pespuntare": "pespuntar",
    "pespuntareis": "pespuntar",
    "pespuntaremos": "pespuntar",
    "pespuntaren": "pespuntar",
    "pespuntares": "pespuntar",
    "pespuntaron": "pespuntar",
    "pespuntará": "pespuntar",
    "pespuntarán": "pespuntar",
    "pespuntarás": "pespuntar",
    "pespuntaré": "pespuntar",
    "pespuntaréis": "pespuntar",
    "pespuntaría": "pespuntar",
    "pespuntaríais": "pespuntar",
    "pespuntaríamos": "pespuntar",
    "pespuntarían": "pespuntar",
    "pespuntarías": "pespuntar",
    "pespuntas": "pespuntar",
    "pespuntase": "pespuntar",
    "pespuntaseis": "pespuntar",
    "pespuntasen": "pespuntar",
    "pespuntases": "pespuntar",
    "pespuntaste": "pespuntar",
    "pespuntasteis": "pespuntar",
    "pespunte": "pespuntar",
    "pespuntea": "pespuntear",
    "pespunteaba": "pespuntear",
    "pespunteabais": "pespuntear",
    "pespunteaban": "pespuntear",
    "pespunteabas": "pespuntear",
    "pespuntead": "pespuntear",
    "pespunteada": "pespuntear",
    "pespunteadas": "pespuntear",
    "pespunteado": "pespuntear",
    "pespunteados": "pespuntear",
    "pespunteamos": "pespuntear",
    "pespuntean": "pespuntear",
    "pespunteando": "pespuntear",
    "pespunteara": "pespuntear",
    "pespuntearais": "pespuntear",
    "pespuntearan": "pespuntear",
    "pespuntearas": "pespuntear",
    "pespunteare": "pespuntear",
    "pespunteareis": "pespuntear",
    "pespuntearemos": "pespuntear",
    "pespuntearen": "pespuntear",
    "pespunteares": "pespuntear",
    "pespuntearon": "pespuntear",
    "pespunteará": "pespuntear",
    "pespuntearán": "pespuntear",
    "pespuntearás": "pespuntear",
    "pespuntearé": "pespuntear",
    "pespuntearéis": "pespuntear",
    "pespuntearía": "pespuntear",
    "pespuntearíais": "pespuntear",
    "pespuntearíamos": "pespuntear",
    "pespuntearían": "pespuntear",
    "pespuntearías": "pespuntear",
    "pespunteas": "pespuntear",
    "pespuntease": "pespuntear",
    "pespunteaseis": "pespuntear",
    "pespunteasen": "pespuntear",
    "pespunteases": "pespuntear",
    "pespunteaste": "pespuntear",
    "pespunteasteis": "pespuntear",
    "pespuntee": "pespuntear",
    "pespunteemos": "pespuntear",
    "pespunteen": "pespuntear",
    "pespuntees": "pespuntear",
    "pespuntemos": "pespuntar",
    "pespunten": "pespuntar",
    "pespunteo": "pespuntear",
    "pespuntes": "pespunte",
    "pespunteábamos": "pespuntear",
    "pespunteáis": "pespuntear",
    "pespunteáramos": "pespuntear",
    "pespunteáremos": "pespuntear",
    "pespunteásemos": "pespuntear",
    "pespunteé": "pespuntear",
    "pespunteéis": "pespuntear",
    "pespunteó": "pespuntear",
    "pespunto": "pespuntar",
    "pespuntábamos": "pespuntar",
    "pespuntáis": "pespuntar",
    "pespuntáramos": "pespuntar",
    "pespuntáremos": "pespuntar",
    "pespuntásemos": "pespuntar",
    "pespunté": "pespuntar",
    "pespuntéis": "pespuntar",
    "pespuntó": "pespuntar",
    "pesque": "pescar",
    "pesquemos": "pescar",
    "pesquen": "pescar",
    "pesquera": "pesquero",
    "pesqueras": "pesquero",
    "pesqueros": "pesquero",
    "pesquerías": "pesquería",
    "pesques": "pescar",
    "pesquisa": "pesquisar",
    "pesquisaba": "pesquisar",
    "pesquisabais": "pesquisar",
    "pesquisaban": "pesquisar",
    "pesquisabas": "pesquisar",
    "pesquisad": "pesquisar",
    "pesquisada": "pesquisar",
    "pesquisadas": "pesquisar",
    "pesquisado": "pesquisar",
    "pesquisados": "pesquisar",
    "pesquisamos": "pesquisar",
    "pesquisan": "pesquisar",
    "pesquisando": "pesquisar",
    "pesquisara": "pesquisar",
    "pesquisarais": "pesquisar",
    "pesquisaran": "pesquisar",
    "pesquisaras": "pesquisar",
    "pesquisare": "pesquisar",
    "pesquisareis": "pesquisar",
    "pesquisaremos": "pesquisar",
    "pesquisaren": "pesquisar",
    "pesquisares": "pesquisar",
    "pesquisaron": "pesquisar",
    "pesquisará": "pesquisar",
    "pesquisarán": "pesquisar",
    "pesquisarás": "pesquisar",
    "pesquisaré": "pesquisar",
    "pesquisaréis": "pesquisar",
    "pesquisaría": "pesquisar",
    "pesquisaríais": "pesquisar",
    "pesquisaríamos": "pesquisar",
    "pesquisarían": "pesquisar",
    "pesquisarías": "pesquisar",
    "pesquisas": "pesquisar",
    "pesquisase": "pesquisar",
    "pesquisaseis": "pesquisar",
    "pesquisasen": "pesquisar",
    "pesquisases": "pesquisar",
    "pesquisaste": "pesquisar",
    "pesquisasteis": "pesquisar",
    "pesquise": "pesquisar",
    "pesquisemos": "pesquisar",
    "pesquisen": "pesquisar",
    "pesquises": "pesquisar",
    "pesquisidora": "pesquisidor",
    "pesquisidoras": "pesquisidor",
    "pesquisidores": "pesquisidor",
    "pesquiso": "pesquisar",
    "pesquisábamos": "pesquisar",
    "pesquisáis": "pesquisar",
    "pesquisáramos": "pesquisar",
    "pesquisáremos": "pesquisar",
    "pesquisásemos": "pesquisar",
    "pesquisé": "pesquisar",
    "pesquiséis": "pesquisar",
    "pesquisó": "pesquisar",
    "pesqué": "pescar",
    "pesquéis": "pescar",
    "pestañas": "pestaña",
    "pestañea": "pestañear",
    "pestañeaba": "pestañear",
    "pestañeabais": "pestañear",
    "pestañeaban": "pestañear",
    "pestañeabas": "pestañear",
    "pestañead": "pestañear",
    "pestañeada": "pestañear",
    "pestañeadas": "pestañear",
    "pestañeado": "pestañear",
    "pestañeados": "pestañear",
    "pestañeamos": "pestañear",
    "pestañean": "pestañear",
    "pestañeando": "pestañear",
    "pestañeara": "pestañear",
    "pestañearais": "pestañear",
    "pestañearan": "pestañear",
    "pestañearas": "pestañear",
    "pestañeare": "pestañear",
    "pestañeareis": "pestañear",
    "pestañearemos": "pestañear",
    "pestañearen": "pestañear",
    "pestañeares": "pestañear",
    "pestañearon": "pestañear",
    "pestañeará": "pestañear",
    "pestañearán": "pestañear",
    "pestañearás": "pestañear",
    "pestañearé": "pestañear",
    "pestañearéis": "pestañear",
    "pestañearía": "pestañear",
    "pestañearíais": "pestañear",
    "pestañearíamos": "pestañear",
    "pestañearían": "pestañear",
    "pestañearías": "pestañear",
    "pestañeas": "pestañear",
    "pestañease": "pestañear",
    "pestañeaseis": "pestañear",
    "pestañeasen": "pestañear",
    "pestañeases": "pestañear",
    "pestañeaste": "pestañear",
    "pestañeasteis": "pestañear",
    "pestañee": "pestañear",
    "pestañeemos": "pestañear",
    "pestañeen": "pestañear",
    "pestañees": "pestañear",
    "pestañeo": "pestañear",
    "pestañeos": "pestañeo",
    "pestañeábamos": "pestañear",
    "pestañeáis": "pestañear",
    "pestañeáramos": "pestañear",
    "pestañeáremos": "pestañear",
    "pestañeásemos": "pestañear",
    "pestañeé": "pestañear",
    "pestañeéis": "pestañear",
    "pestañeó": "pestañear",
    "pestañosa": "pestañoso",
    "pestañosas": "pestañoso",
    "pestañosos": "pestañoso",
    "pestes": "peste",
    "pesticidas": "pesticida",
    "pestilenciales": "pestilencial",
    "pestilencias": "pestilencia",
    "pestilenciosa": "pestilencioso",
    "pestilenciosas": "pestilencioso",
    "pestilenciosos": "pestilencioso",
    "pestilentes": "pestilente",
    "pestillos": "pestillo",
    "pestiños": "pestiño",
    "pestorejos": "pestorejo",
    "pestífera": "pestífero",
    "pestíferas": "pestífero",
    "pestíferos": "pestífero",
    "pesuñas": "pesuña",
    "pesuños": "pesuño",
    "pesábamos": "pesar",
    "pesáis": "pesar",
    "pesáramos": "pesar",
    "pesáremos": "pesar",
    "pesásemos": "pesar",
    "pesé": "pesar",
    "peséis": "pesar",
    "pesó": "pesar",
    "peta": "petar",
    "petaba": "petar",
    "petabais": "petar",
    "petaban": "petar",
    "petabas": "petar",
    "petabytes": "petabyte",
    "petacas": "petaca",
    "petad": "petar",
    "petada": "petar",
    "petadas": "petar",
    "petado": "petar",
    "petados": "petar",
    "petalismos": "petalismo",
    "petalitas": "petalita",
    "petaloides": "petaloide",
    "petamos": "petar",
    "petan": "petar",
    "petancas": "petanca",
    "petando": "petar",
    "petanques": "petanque",
    "petanquistas": "petanquista",
    "petara": "petar",
    "petarais": "petar",
    "petaran": "petar",
    "petaras": "petar",
    "petardea": "petardear",
    "petardeaba": "petardear",
    "petardeabais": "petardear",
    "petardeaban": "petardear",
    "petardeabas": "petardear",
    "petardead": "petardear",
    "petardeada": "petardear",
    "petardeadas": "petardear",
    "petardeado": "petardear",
    "petardeados": "petardear",
    "petardeamos": "petardear",
    "petardean": "petardear",
    "petardeando": "petardear",
    "petardeara": "petardear",
    "petardearais": "petardear",
    "petardearan": "petardear",
    "petardearas": "petardear",
    "petardeare": "petardear",
    "petardeareis": "petardear",
    "petardearemos": "petardear",
    "petardearen": "petardear",
    "petardeares": "petardear",
    "petardearon": "petardear",
    "petardeará": "petardear",
    "petardearán": "petardear",
    "petardearás": "petardear",
    "petardearé": "petardear",
    "petardearéis": "petardear",
    "petardearía": "petardear",
    "petardearíais": "petardear",
    "petardearíamos": "petardear",
    "petardearían": "petardear",
    "petardearías": "petardear",
    "petardeas": "petardear",
    "petardease": "petardear",
    "petardeaseis": "petardear",
    "petardeasen": "petardear",
    "petardeases": "petardear",
    "petardeaste": "petardear",
    "petardeasteis": "petardear",
    "petardee": "petardear",
    "petardeemos": "petardear",
    "petardeen": "petardear",
    "petardees": "petardear",
    "petardeo": "petardear",
    "petardeos": "petardeo",
    "petardera": "petardero",
    "petarderas": "petardero",
    "petarderos": "petardero",
    "petardeábamos": "petardear",
    "petardeáis": "petardear",
    "petardeáramos": "petardear",
    "petardeáremos": "petardear",
    "petardeásemos": "petardear",
    "petardeé": "petardear",
    "petardeéis": "petardear",
    "petardeó": "petardear",
    "petardistas": "petardista",
    "petardos": "petardo",
    "petare": "petar",
    "petareis": "petar",
    "petaremos": "petar",
    "petaren": "petar",
    "petares": "petar",
    "petaron": "petar",
    "petará": "petar",
    "petarán": "petar",
    "petarás": "petar",
    "petaré": "petar",
    "petaréis": "petar",
    "petaría": "petar",
    "petaríais": "petar",
    "petaríamos": "petar",
    "petarían": "petar",
    "petarías": "petar",
    "petas": "petar",
    "petase": "petar",
    "petaseis": "petar",
    "petasen": "petar",
    "petases": "petar",
    "petaste": "petar",
    "petasteis": "petar",
    "petates": "petate",
    "pete": "petar",
    "petemos": "petar",
    "peten": "petar",
    "peteneras": "petenera",
    "petequias": "petequia",
    "peteras": "petera",
    "peteretes": "peterete",
    "petes": "petar",
    "peticanos": "peticano",
    "petichismos": "petichismo",
    "peticiona": "peticionar",
    "peticionaba": "peticionar",
    "peticionabais": "peticionar",
    "peticionaban": "peticionar",
    "peticionabas": "peticionar",
    "peticionad": "peticionar",
    "peticionada": "peticionar",
    "peticionadas": "peticionar",
    "peticionado": "peticionar",
    "peticionados": "peticionar",
    "peticionamos": "peticionar",
    "peticionan": "peticionar",
    "peticionando": "peticionar",
    "peticionara": "peticionar",
    "peticionarais": "peticionar",
    "peticionaran": "peticionar",
    "peticionaras": "peticionar",
    "peticionare": "peticionar",
    "peticionareis": "peticionar",
    "peticionaremos": "peticionar",
    "peticionaren": "peticionar",
    "peticionares": "peticionar",
    "peticionaria": "peticionario",
    "peticionarias": "peticionario",
    "peticionarios": "peticionario",
    "peticionaron": "peticionar",
    "peticionará": "peticionar",
    "peticionarán": "peticionar",
    "peticionarás": "peticionar",
    "peticionaré": "peticionar",
    "peticionaréis": "peticionar",
    "peticionaría": "peticionar",
    "peticionaríais": "peticionar",
    "peticionaríamos": "peticionar",
    "peticionarían": "peticionar",
    "peticionarías": "peticionar",
    "peticionas": "peticionar",
    "peticionase": "peticionar",
    "peticionaseis": "peticionar",
    "peticionasen": "peticionar",
    "peticionases": "peticionar",
    "peticionaste": "peticionar",
    "peticionasteis": "peticionar",
    "peticione": "peticionar",
    "peticionemos": "peticionar",
    "peticionen": "peticionar",
    "peticiones": "peticionar",
    "peticiono": "peticionar",
    "peticionábamos": "peticionar",
    "peticionáis": "peticionar",
    "peticionáramos": "peticionar",
    "peticionáremos": "peticionar",
    "peticionásemos": "peticionar",
    "peticioné": "peticionar",
    "peticionéis": "peticionar",
    "peticionó": "peticionar",
    "peticánones": "peticanon",
    "petifoques": "petifoque",
    "petigrises": "petigrís",
    "petimetres": "petimetre",
    "petirrojos": "petirrojo",
    "petita": "petitum",
    "petitoria": "petitorio",
    "petitorias": "petitorio",
    "petitorios": "petitorio",
    "peto": "petar",
    "petos": "peto",
    "petrales": "petral",
    "petrarias": "petraria",
    "petrarquesca": "petrarquesco",
    "petrarquescas": "petrarquesco",
    "petrarquescos": "petrarquesco",
    "petrarquismos": "petrarquismo",
    "petrarquistas": "petrarquista",
    "petras": "petra",
    "petreles": "petrel",
    "petrifica": "petrificar",
    "petrificaba": "petrificar",
    "petrificabais": "petrificar",
    "petrificaban": "petrificar",
    "petrificabas": "petrificar",
    "petrificaciones": "petrificación",
    "petrificad": "petrificar",
    "petrificada": "petrificar",
    "petrificadas": "petrificar",
    "petrificado": "petrificar",
    "petrificados": "petrificar",
    "petrificamos": "petrificar",
    "petrifican": "petrificar",
    "petrificando": "petrificar",
    "petrificara": "petrificar",
    "petrificarais": "petrificar",
    "petrificaran": "petrificar",
    "petrificaras": "petrificar",
    "petrificare": "petrificar",
    "petrificareis": "petrificar",
    "petrificaremos": "petrificar",
    "petrificaren": "petrificar",
    "petrificares": "petrificar",
    "petrificaron": "petrificar",
    "petrificará": "petrificar",
    "petrificarán": "petrificar",
    "petrificarás": "petrificar",
    "petrificaré": "petrificar",
    "petrificaréis": "petrificar",
    "petrificaría": "petrificar",
    "petrificaríais": "petrificar",
    "petrificaríamos": "petrificar",
    "petrificarían": "petrificar",
    "petrificarías": "petrificar",
    "petrificas": "petrificar",
    "petrificase": "petrificar",
    "petrificaseis": "petrificar",
    "petrificasen": "petrificar",
    "petrificases": "petrificar",
    "petrificaste": "petrificar",
    "petrificasteis": "petrificar",
    "petrifico": "petrificar",
    "petrificábamos": "petrificar",
    "petrificáis": "petrificar",
    "petrificáramos": "petrificar",
    "petrificáremos": "petrificar",
    "petrificásemos": "petrificar",
    "petrificó": "petrificar",
    "petrifique": "petrificar",
    "petrifiquemos": "petrificar",
    "petrifiquen": "petrificar",
    "petrifiques": "petrificar",
    "petrifiqué": "petrificar",
    "petrifiquéis": "petrificar",
    "petrobonos": "petrobono",
    "petrocálcica": "petrocálcico",
    "petrocálcicas": "petrocálcico",
    "petrocálcicos": "petrocálcico",
    "petrodólares": "petrodólar",
    "petroescamosa": "petroescamoso",
    "petroescamosas": "petroescamoso",
    "petroescamosos": "petroescamoso",
    "petrofaríngea": "petrofaríngeo",
    "petrofaríngeas": "petrofaríngeo",
    "petrofaríngeos": "petrofaríngeo",
    "petroférrica": "petroférrico",
    "petroférricas": "petroférrico",
    "petroférricos": "petroférrico",
    "petroglifos": "petroglifo",
    "petrografías": "petrografía",
    "petrográfica": "petrográfico",
    "petrográficas": "petrográfico",
    "petrográficos": "petrográfico",
    "petrogípsica": "petrogípsico",
    "petrogípsicas": "petrogípsico",
    "petrogípsicos": "petrogípsico",
    "petrolatos": "petrolato",
    "petrolea": "petrolear",
    "petroleaba": "petrolear",
    "petroleabais": "petrolear",
    "petroleaban": "petrolear",
    "petroleabas": "petrolear",
    "petrolead": "petrolear",
    "petroleada": "petrolear",
    "petroleadas": "petrolear",
    "petroleado": "petrolear",
    "petroleados": "petrolear",
    "petroleamos": "petrolear",
    "petrolean": "petrolear",
    "petroleando": "petrolear",
    "petroleara": "petrolear",
    "petrolearais": "petrolear",
    "petrolearan": "petrolear",
    "petrolearas": "petrolear",
    "petroleare": "petrolear",
    "petroleareis": "petrolear",
    "petrolearemos": "petrolear",
    "petrolearen": "petrolear",
    "petroleares": "petrolear",
    "petrolearon": "petrolear",
    "petroleará": "petrolear",
    "petrolearán": "petrolear",
    "petrolearás": "petrolear",
    "petrolearé": "petrolear",
    "petrolearéis": "petrolear",
    "petrolearía": "petrolear",
    "petrolearíais": "petrolear",
    "petrolearíamos": "petrolear",
    "petrolearían": "petrolear",
    "petrolearías": "petrolear",
    "petroleas": "petrolear",
    "petrolease": "petrolear",
    "petroleaseis": "petrolear",
    "petroleasen": "petrolear",
    "petroleases": "petrolear",
    "petroleaste": "petrolear",
    "petroleasteis": "petrolear",
    "petrolee": "petrolear",
    "petroleemos": "petrolear",
    "petroleen": "petrolear",
    "petrolees": "petrolear",
    "petroleo": "petrolear",
    "petrolera": "petrolero",
    "petroleras": "petrolero",
    "petroleros": "petrolero",
    "petroleábamos": "petrolear",
    "petroleáis": "petrolear",
    "petroleáramos": "petrolear",
    "petroleáremos": "petrolear",
    "petroleásemos": "petrolear",
    "petroleé": "petrolear",
    "petroleéis": "petrolear",
    "petroleó": "petrolear",
    "petrologías": "petrología",
    "petrolífera": "petrolífero",
    "petrolíferas": "petrolífero",
    "petrolíferos": "petrolífero",
    "petrolítica": "petrolítico",
    "petrolíticas": "petrolítico",
    "petrolíticos": "petrolítico",
    "petrológica": "petrológico",
    "petrológicas": "petrológico",
    "petrológicos": "petrológico",
    "petromastoidea": "petromastoideo",
    "petromastoideas": "petromastoideo",
    "petromastoideos": "petromastoideo",
    "petrooccipitales": "petrooccipital",
    "petroquímica": "petroquímico",
    "petroquímicas": "petroquímico",
    "petroquímicos": "petroquímico",
    "petrosa": "petroso",
    "petrosas": "petroso",
    "petrosos": "petroso",
    "petrosílea": "petrosíleo",
    "petrosíleas": "petrosíleo",
    "petrosíleos": "petrosíleo",
    "petrotectónicas": "petrotectónica",
    "petrotimpánica": "petrotimpánico",
    "petrotimpánicas": "petrotimpánico",
    "petrotimpánicos": "petrotimpánico",
    "petrífica": "petrífico",
    "petríficas": "petrífico",
    "petríficos": "petrífico",
    "petróleos": "petróleo",
    "petróloga": "petrólogo",
    "petrólogas": "petrólogo",
    "petrólogos": "petrólogo",
    "petulancias": "petulancia",
    "petulantes": "petulante",
    "petunias": "petunia",
    "petábamos": "petar",
    "petáis": "petar",
    "petáramos": "petar",
    "petáremos": "petar",
    "petásemos": "petar",
    "peté": "petar",
    "petéis": "petar",
    "petó": "petar",
    "peyendo": "peer",
    "peyera": "peer",
    "peyerais": "peer",
    "peyeran": "peer",
    "peyeras": "peer",
    "peyere": "peer",
    "peyereis": "peer",
    "peyeren": "peer",
    "peyeres": "peer",
    "peyeron": "peer",
    "peyese": "peer",
    "peyeseis": "peer",
    "peyesen": "peer",
    "peyeses": "peer",
    "peyorativa": "peyorativo",
    "peyorativas": "peyorativo",
    "peyorativos": "peyorativo",
    "peyotes": "peyote",
    "peyotismos": "peyotismo",
    "peyotistas": "peyotista",
    "peyéramos": "peer",
    "peyéremos": "peer",
    "peyésemos": "peer",
    "peyó": "peer",
    "pezizales": "pezizal",
    "pezoneras": "pezonera",
    "pezones": "pezón",
    "pezpalos": "pezpalo",
    "pezuelos": "pezuelo",
    "pezuñas": "pezuña",
    "peáis": "peer",
    "peéis": "peer",
    "peí": "peer",
    "peía": "peer",
    "peíais": "peer",
    "peíamos": "peer",
    "peían": "peer",
    "peías": "peer",
    "peída": "peer",
    "peídas": "peer",
    "peído": "peer",
    "peídos": "peer",
    "peímos": "peer",
    "peíste": "peer",
    "peísteis": "peer",
    "peñarandas": "peñaranda",
    "peñas": "peña",
    "peñascales": "peñascal",
    "peñascos": "peñasco",
    "peñascosa": "peñascoso",
    "peñascosas": "peñascoso",
    "peñascosos": "peñascoso",
    "peñistas": "peñista",
    "peñones": "peñón",
    "peúcos": "peúco",
    "phanatrones": "phanatrón",
    "phanotrones": "phanotrón",
    "phies": "phi",
    "phis": "phi",
    "phons": "phon",
    "phots": "phot",
    "piaba": "piar",
    "piabais": "piar",
    "piaban": "piar",
    "piabas": "piar",
    "piad": "piar",
    "piada": "piar",
    "piadas": "piar",
    "piado": "piar",
    "piadora": "piador",
    "piadoras": "piador",
    "piadores": "piador",
    "piados": "piar",
    "piadosa": "piadoso",
    "piadosas": "piadoso",
    "piadosos": "piadoso",
    "piafa": "piafar",
    "piafaba": "piafar",
    "piafabais": "piafar",
    "piafaban": "piafar",
    "piafabas": "piafar",
    "piafad": "piafar",
    "piafada": "piafar",
    "piafadas": "piafar",
    "piafado": "piafar",
    "piafados": "piafar",
    "piafamos": "piafar",
    "piafan": "piafar",
    "piafando": "piafar",
    "piafara": "piafar",
    "piafarais": "piafar",
    "piafaran": "piafar",
    "piafaras": "piafar",
    "piafare": "piafar",
    "piafareis": "piafar",
    "piafaremos": "piafar",
    "piafaren": "piafar",
    "piafares": "piafar",
    "piafaron": "piafar",
    "piafará": "piafar",
    "piafarán": "piafar",
    "piafarás": "piafar",
    "piafaré": "piafar",
    "piafaréis": "piafar",
    "piafaría": "piafar",
    "piafaríais": "piafar",
    "piafaríamos": "piafar",
    "piafarían": "piafar",
    "piafarías": "piafar",
    "piafas": "piafar",
    "piafase": "piafar",
    "piafaseis": "piafar",
    "piafasen": "piafar",
    "piafases": "piafar",
    "piafaste": "piafar",
    "piafasteis": "piafar",
    "piafe": "piafar",
    "piafemos": "piafar",
    "piafen": "piafar",
    "piafes": "piafar",
    "piafo": "piafar",
    "piafábamos": "piafar",
    "piafáis": "piafar",
    "piafáramos": "piafar",
    "piafáremos": "piafar",
    "piafásemos": "piafar",
    "piafé": "piafar",
    "piaféis": "piafar",
    "piafó": "piafar",
    "piala": "pialar",
    "pialaba": "pialar",
    "pialabais": "pialar",
    "pialaban": "pialar",
    "pialabas": "pialar",
    "pialad": "pialar",
    "pialada": "pialar",
    "pialadas": "pialar",
    "pialado": "pialar",
    "pialados": "pialar",
    "pialamos": "pialar",
    "pialan": "pialar",
    "pialando": "pialar",
    "pialara": "pialar",
    "pialarais": "pialar",
    "pialaran": "pialar",
    "pialaras": "pialar",
    "pialare": "pialar",
    "pialareis": "pialar",
    "pialaremos": "pialar",
    "pialaren": "pialar",
    "pialares": "pialar",
    "pialaron": "pialar",
    "pialará": "pialar",
    "pialarán": "pialar",
    "pialarás": "pialar",
    "pialaré": "pialar",
    "pialaréis": "pialar",
    "pialaría": "pialar",
    "pialaríais": "pialar",
    "pialaríamos": "pialar",
    "pialarían": "pialar",
    "pialarías": "pialar",
    "pialas": "pialar",
    "pialase": "pialar",
    "pialaseis": "pialar",
    "pialasen": "pialar",
    "pialases": "pialar",
    "pialaste": "pialar",
    "pialasteis": "pialar",
    "piale": "pialar",
    "pialemos": "pialar",
    "pialen": "pialar",
    "piales": "pialar",
    "pialo": "pialar",
    "pialábamos": "pialar",
    "pialáis": "pialar",
    "pialáramos": "pialar",
    "pialáremos": "pialar",
    "pialásemos": "pialar",
    "pialé": "pialar",
    "pialéis": "pialar",
    "pialó": "pialar",
    "piamontesa": "piamontés",
    "piamontesas": "piamontés",
    "piamonteses": "piamontés",
    "piamontitas": "piamontita",
    "piamos": "piar",
    "piando": "piar",
    "pianissimos": "pianissimo",
    "pianistas": "pianista",
    "pianofortes": "pianoforte",
    "pianolas": "pianola",
    "pianos": "piano",
    "pianíssimo": "pianissimo",
    "pianíssimos": "pianissimo",
    "pianística": "pianístico",
    "pianísticas": "pianístico",
    "pianísticos": "pianístico",
    "piara": "piar",
    "piarais": "piar",
    "piaran": "piar",
    "piaras": "piara",
    "piare": "piar",
    "piareis": "piar",
    "piaremos": "piar",
    "piaren": "piar",
    "piares": "piar",
    "piariega": "piariego",
    "piariegas": "piariego",
    "piariegos": "piariego",
    "piaron": "piar",
    "piará": "piar",
    "piarán": "piar",
    "piarás": "piar",
    "piaré": "piar",
    "piaréis": "piar",
    "piaría": "piar",
    "piaríais": "piar",
    "piaríamos": "piar",
    "piarían": "piar",
    "piarías": "piar",
    "piase": "piar",
    "piaseis": "piar",
    "piasen": "piar",
    "piases": "piar",
    "piaste": "piar",
    "piasteis": "piar",
    "piastras": "piastra",
    "pica": "picar",
    "picaba": "picar",
    "picabais": "picar",
    "picaban": "picar",
    "picabas": "picar",
    "picacera": "picacero",
    "picaceras": "picacero",
    "picaceros": "picacero",
    "picachos": "picacho",
    "picad": "picar",
    "picada": "picar",
    "picadas": "picar",
    "picaderos": "picadero",
    "picadillos": "picadillo",
    "picado": "picar",
    "picadora": "picador",
    "picadoras": "picadora",
    "picadores": "picador",
    "picados": "picar",
    "picaduras": "picadura",
    "picaflores": "picaflor",
    "picajosa": "picajoso",
    "picajosas": "picajoso",
    "picajosos": "picajoso",
    "picamos": "picar",
    "pican": "picar",
    "picanas": "picana",
    "picando": "picar",
    "picanea": "picanear",
    "picaneaba": "picanear",
    "picaneabais": "picanear",
    "picaneaban": "picanear",
    "picaneabas": "picanear",
    "picanead": "picanear",
    "picaneada": "picanear",
    "picaneadas": "picanear",
    "picaneado": "picanear",
    "picaneados": "picanear",
    "picaneamos": "picanear",
    "picanean": "picanear",
    "picaneando": "picanear",
    "picaneara": "picanear",
    "picanearais": "picanear",
    "picanearan": "picanear",
    "picanearas": "picanear",
    "picaneare": "picanear",
    "picaneareis": "picanear",
    "picanearemos": "picanear",
    "picanearen": "picanear",
    "picaneares": "picanear",
    "picanearon": "picanear",
    "picaneará": "picanear",
    "picanearán": "picanear",
    "picanearás": "picanear",
    "picanearé": "picanear",
    "picanearéis": "picanear",
    "picanearía": "picanear",
    "picanearíais": "picanear",
    "picanearíamos": "picanear",
    "picanearían": "picanear",
    "picanearías": "picanear",
    "picaneas": "picanear",
    "picanease": "picanear",
    "picaneaseis": "picanear",
    "picaneasen": "picanear",
    "picaneases": "picanear",
    "picaneaste": "picanear",
    "picaneasteis": "picanear",
    "picanee": "picanear",
    "picaneemos": "picanear",
    "picaneen": "picanear",
    "picanees": "picanear",
    "picaneo": "picanear",
    "picaneábamos": "picanear",
    "picaneáis": "picanear",
    "picaneáramos": "picanear",
    "picaneáremos": "picanear",
    "picaneásemos": "picanear",
    "picaneé": "picanear",
    "picaneéis": "picanear",
    "picaneó": "picanear",
    "picantes": "picante",
    "picantones": "picantón",
    "picapedrera": "picapedrero",
    "picapedreras": "picapedrero",
    "picapedreros": "picapedrero",
    "picapicas": "picapica",
    "picaportes": "picaporte",
    "picara": "picar",
    "picarais": "picar",
    "picaran": "picar",
    "picaras": "picar",
    "picarda": "picardo",
    "picardas": "picardo",
    "picardea": "picardear",
    "picardeaba": "picardear",
    "picardeabais": "picardear",
    "picardeaban": "picardear",
    "picardeabas": "picardear",
    "picardead": "picardear",
    "picardeada": "picardear",
    "picardeadas": "picardear",
    "picardeado": "picardear",
    "picardeados": "picardear",
    "picardeamos": "picardear",
    "picardean": "picardear",
    "picardeando": "picardear",
    "picardeara": "picardear",
    "picardearais": "picardear",
    "picardearan": "picardear",
    "picardearas": "picardear",
    "picardeare": "picardear",
    "picardeareis": "picardear",
    "picardearemos": "picardear",
    "picardearen": "picardear",
    "picardeares": "picardear",
    "picardearon": "picardear",
    "picardeará": "picardear",
    "picardearán": "picardear",
    "picardearás": "picardear",
    "picardearé": "picardear",
    "picardearéis": "picardear",
    "picardearía": "picardear",
    "picardearíais": "picardear",
    "picardearíamos": "picardear",
    "picardearían": "picardear",
    "picardearías": "picardear",
    "picardeas": "picardear",
    "picardease": "picardear",
    "picardeaseis": "picardear",
    "picardeasen": "picardear",
    "picardeases": "picardear",
    "picardeaste": "picardear",
    "picardeasteis": "picardear",
    "picardee": "picardear",
    "picardeemos": "picardear",
    "picardeen": "picardear",
    "picardees": "picardear",
    "picardeo": "picardear",
    "picardeábamos": "picardear",
    "picardeáis": "picardear",
    "picardeáramos": "picardear",
    "picardeáremos": "picardear",
    "picardeásemos": "picardear",
    "picardeé": "picardear",
    "picardeéis": "picardear",
    "picardeó": "picardear",
    "picardos": "picardo",
    "picardías": "picardía",
    "picare": "picar",
    "picareis": "picar",
    "picaremos": "picar",
    "picaren": "picar",
    "picares": "picar",
    "picaresca": "picaresco",
    "picarescas": "picaresco",
    "picarescos": "picaresco",
    "picariles": "picaril",
    "picaron": "picar",
    "picará": "picar",
    "picarán": "picar",
    "picarás": "picar",
    "picaré": "picar",
    "picaréis": "picar",
    "picaría": "picar",
    "picaríais": "picar",
    "picaríamos": "picar",
    "picarían": "picar",
    "picarías": "picar",
    "picas": "picar",
    "picase": "picar",
    "picaseis": "picar",
    "picasen": "picar",
    "picases": "picar",
    "picassiana": "picassiano",
    "picassianas": "picassiano",
    "picassianos": "picassiano",
    "picaste": "picar",
    "picasteis": "picar",
    "picatostes": "picatoste",
    "picaza": "picazo",
    "picazas": "picazo",
    "picazones": "picazón",
    "picazos": "picazo",
    "pichas": "picha",
    "pichelera": "pichelero",
    "picheleras": "pichelero",
    "picheleros": "pichelero",
    "pichelerías": "pichelería",
    "picheles": "pichel",
    "pichelingues": "pichelingue",
    "pichichis": "pichichi",
    "pichis": "pichi",
    "pichona": "pichón",
    "pichonas": "pichón",
    "pichones": "pichón",
    "pichulas": "pichula",
    "pick-ups": "pick-up",
    "picnics": "picnic",
    "picnometrías": "picnometría",
    "picnómetros": "picnómetro",
    "picnótica": "picnótico",
    "picnóticas": "picnótico",
    "picnóticos": "picnótico",
    "pico": "picar",
    "picogordos": "picogordo",
    "picolas": "picola",
    "picoletes": "picolete",
    "picolinas": "picolina",
    "picona": "picón",
    "piconas": "picón",
    "piconera": "piconero",
    "piconeras": "piconero",
    "piconeros": "piconero",
    "picones": "picón",
    "picores": "picor",
    "picos": "pico",
    "picosa": "picoso",
    "picosas": "picoso",
    "picosegundos": "picosegundo",
    "picosos": "picoso",
    "picotadas": "picotada",
    "picotas": "picota",
    "picotazos": "picotazo",
    "picotea": "picotear",
    "picoteaba": "picotear",
    "picoteabais": "picotear",
    "picoteaban": "picotear",
    "picoteabas": "picotear",
    "picotead": "picotear",
    "picoteada": "picotear",
    "picoteadas": "picotear",
    "picoteado": "picotear",
    "picoteados": "picotear",
    "picoteamos": "picotear",
    "picotean": "picotear",
    "picoteando": "picotear",
    "picoteara": "picotear",
    "picotearais": "picotear",
    "picotearan": "picotear",
    "picotearas": "picotear",
    "picoteare": "picotear",
    "picoteareis": "picotear",
    "picotearemos": "picotear",
    "picotearen": "picotear",
    "picoteares": "picotear",
    "picotearon": "picotear",
    "picoteará": "picotear",
    "picotearán": "picotear",
    "picotearás": "picotear",
    "picotearé": "picotear",
    "picotearéis": "picotear",
    "picotearía": "picotear",
    "picotearíais": "picotear",
    "picotearíamos": "picotear",
    "picotearían": "picotear",
    "picotearías": "picotear",
    "picoteas": "picotear",
    "picotease": "picotear",
    "picoteaseis": "picotear",
    "picoteasen": "picotear",
    "picoteases": "picotear",
    "picoteaste": "picotear",
    "picoteasteis": "picotear",
    "picotee": "picotear",
    "picoteemos": "picotear",
    "picoteen": "picotear",
    "picotees": "picotear",
    "picoteo": "picotear",
    "picoteos": "picoteo",
    "picotera": "picotero",
    "picoteras": "picotero",
    "picoteros": "picotero",
    "picoterías": "picotería",
    "picotes": "picote",
    "picoteábamos": "picotear",
    "picoteáis": "picotear",
    "picoteáramos": "picotear",
    "picoteáremos": "picotear",
    "picoteásemos": "picotear",
    "picoteé": "picotear",
    "picoteéis": "picotear",
    "picoteó": "picotear",
    "picotitas": "picotita",
    "picramatos": "picramato",
    "picratos": "picrato",
    "picrolónica": "picrolónico",
    "picrolónicas": "picrolónico",
    "picrolónicos": "picrolónico",
    "picrotoxinas": "picrotoxina",
    "picrámica": "picrámico",
    "picrámicas": "picrámico",
    "picrámicos": "picrámico",
    "picta": "picto",
    "pictas": "picto",
    "pictografías": "pictografía",
    "pictogramas": "pictograma",
    "pictográfica": "pictográfico",
    "pictográficas": "pictográfico",
    "pictográficos": "pictográfico",
    "pictos": "picto",
    "pictórica": "pictórico",
    "pictóricas": "pictórico",
    "pictóricos": "pictórico",
    "picuda": "picudo",
    "picudas": "picudo",
    "picudos": "picudo",
    "picábamos": "picar",
    "picáis": "picar",
    "picáramos": "picar",
    "picáremos": "picar",
    "picásemos": "picar",
    "picó": "picar",
    "pida": "pedir",
    "pidamos": "pedir",
    "pidan": "pedir",
    "pidas": "pedir",
    "pide": "pedir",
    "piden": "pedir",
    "pides": "pedir",
    "pidiendo": "pedir",
    "pidiera": "pedir",
    "pidierais": "pedir",
    "pidieran": "pedir",
    "pidieras": "pedir",
    "pidiere": "pedir",
    "pidiereis": "pedir",
    "pidieren": "pedir",
    "pidieres": "pedir",
    "pidieron": "pedir",
    "pidiese": "pedir",
    "pidieseis": "pedir",
    "pidiesen": "pedir",
    "pidieses": "pedir",
    "pidiéramos": "pedir",
    "pidiéremos": "pedir",
    "pidiésemos": "pedir",
    "pidió": "pedir",
    "pido": "pedir",
    "pidona": "pidón",
    "pidonas": "pidón",
    "pidones": "pidón",
    "pidáis": "pedir",
    "pied-noires": "pied-noir",
    "pied-noirs": "pied-noir",
    "piedades": "piedad",
    "piedemontes": "piedemonte",
    "piednoires": "piednoir",
    "piednoirs": "piednoir",
    "piedras": "piedra",
    "piedrecitas": "piedrecita",
    "piejos": "piejo",
    "pielera": "pielero",
    "pieleras": "pielero",
    "pieleros": "pielero",
    "pieles": "piel",
    "pielgos": "pielgo",
    "piemontesa": "piemontés",
    "piemontesas": "piemontés",
    "piemonteses": "piemontés",
    "piemos": "piar",
    "piensa": "pensar",
    "piensadora": "piensador",
    "piensadoras": "piensador",
    "piensadores": "piensador",
    "piensan": "pensar",
    "piensas": "pensar",
    "piense": "pensar",
    "piensen": "pensar",
    "pienses": "pensar",
    "pienso": "pensar",
    "piensos": "pienso",
    "pierda": "perder",
    "pierdan": "perder",
    "pierdas": "perder",
    "pierde": "perder",
    "pierden": "perder",
    "pierdes": "perder",
    "pierdo": "perder",
    "pieria": "pierio",
    "pierias": "pierio",
    "pierios": "pierio",
    "piernas": "pierna",
    "piernitendida": "piernitendido",
    "piernitendidas": "piernitendido",
    "piernitendidos": "piernitendido",
    "pies": "pie",
    "pietismos": "pietismo",
    "pietistas": "pietista",
    "piezas": "pieza",
    "piezgos": "piezgo",
    "piezoelectricidades": "piezoelectricidad",
    "piezoeléctrica": "piezoeléctrico",
    "piezoeléctricas": "piezoeléctrico",
    "piezoeléctricos": "piezoeléctrico",
    "piezometrías": "piezometría",
    "piezométrica": "piezométrico",
    "piezométricas": "piezométrico",
    "piezométricos": "piezométrico",
    "piezoquímicas": "piezoquímica",
    "piezógrafos": "piezógrafo",
    "piezómetros": "piezómetro",
    "pifia": "pifiar",
    "pifiaba": "pifiar",
    "pifiabais": "pifiar",
    "pifiaban": "pifiar",
    "pifiabas": "pifiar",
    "pifiad": "pifiar",
    "pifiada": "pifiar",
    "pifiadas": "pifiar",
    "pifiado": "pifiar",
    "pifiados": "pifiar",
    "pifiamos": "pifiar",
    "pifian": "pifiar",
    "pifiando": "pifiar",
    "pifiara": "pifiar",
    "pifiarais": "pifiar",
    "pifiaran": "pifiar",
    "pifiaras": "pifiar",
    "pifiare": "pifiar",
    "pifiareis": "pifiar",
    "pifiaremos": "pifiar",
    "pifiaren": "pifiar",
    "pifiares": "pifiar",
    "pifiaron": "pifiar",
    "pifiará": "pifiar",
    "pifiarán": "pifiar",
    "pifiarás": "pifiar",
    "pifiaré": "pifiar",
    "pifiaréis": "pifiar",
    "pifiaría": "pifiar",
    "pifiaríais": "pifiar",
    "pifiaríamos": "pifiar",
    "pifiarían": "pifiar",
    "pifiarías": "pifiar",
    "pifias": "pifiar",
    "pifiase": "pifiar",
    "pifiaseis": "pifiar",
    "pifiasen": "pifiar",
    "pifiases": "pifiar",
    "pifiaste": "pifiar",
    "pifiasteis": "pifiar",
    "pifie": "pifiar",
    "pifiemos": "pifiar",
    "pifien": "pifiar",
    "pifies": "pifiar",
    "pifio": "pifiar",
    "pifiábamos": "pifiar",
    "pifiáis": "pifiar",
    "pifiáramos": "pifiar",
    "pifiáremos": "pifiar",
    "pifiásemos": "pifiar",
    "pifié": "pifiar",
    "pifiéis": "pifiar",
    "pifió": "pifiar",
    "pigargos": "pigargo",
    "pigmea": "pigmeo",
    "pigmeas": "pigmeo",
    "pigmenta": "pigmentar",
    "pigmentaba": "pigmentar",
    "pigmentabais": "pigmentar",
    "pigmentaban": "pigmentar",
    "pigmentabas": "pigmentar",
    "pigmentaciones": "pigmentación",
    "pigmentad": "pigmentar",
    "pigmentada": "pigmentar",
    "pigmentadas": "pigmentar",
    "pigmentado": "pigmentar",
    "pigmentados": "pigmentar",
    "pigmentamos": "pigmentar",
    "pigmentan": "pigmentar",
    "pigmentando": "pigmentar",
    "pigmentara": "pigmentar",
    "pigmentarais": "pigmentar",
    "pigmentaran": "pigmentar",
    "pigmentaras": "pigmentar",
    "pigmentare": "pigmentar",
    "pigmentareis": "pigmentar",
    "pigmentaremos": "pigmentar",
    "pigmentaren": "pigmentar",
    "pigmentares": "pigmentar",
    "pigmentaria": "pigmentario",
    "pigmentarias": "pigmentario",
    "pigmentarios": "pigmentario",
    "pigmentaron": "pigmentar",
    "pigmentará": "pigmentar",
    "pigmentarán": "pigmentar",
    "pigmentarás": "pigmentar",
    "pigmentaré": "pigmentar",
    "pigmentaréis": "pigmentar",
    "pigmentaría": "pigmentar",
    "pigmentaríais": "pigmentar",
    "pigmentaríamos": "pigmentar",
    "pigmentarían": "pigmentar",
    "pigmentarías": "pigmentar",
    "pigmentas": "pigmentar",
    "pigmentase": "pigmentar",
    "pigmentaseis": "pigmentar",
    "pigmentasen": "pigmentar",
    "pigmentases": "pigmentar",
    "pigmentaste": "pigmentar",
    "pigmentasteis": "pigmentar",
    "pigmente": "pigmentar",
    "pigmentemos": "pigmentar",
    "pigmenten": "pigmentar",
    "pigmentes": "pigmentar",
    "pigmento": "pigmentar",
    "pigmentos": "pigmento",
    "pigmentosa": "pigmentoso",
    "pigmentosas": "pigmentoso",
    "pigmentosos": "pigmentoso",
    "pigmentábamos": "pigmentar",
    "pigmentáis": "pigmentar",
    "pigmentáramos": "pigmentar",
    "pigmentáremos": "pigmentar",
    "pigmentásemos": "pigmentar",
    "pigmenté": "pigmentar",
    "pigmentéis": "pigmentar",
    "pigmentó": "pigmentar",
    "pigmeos": "pigmeo",
    "pignora": "pignorar",
    "pignoraba": "pignorar",
    "pignorabais": "pignorar",
    "pignoraban": "pignorar",
    "pignorabas": "pignorar",
    "pignoraciones": "pignoración",
    "pignorad": "pignorar",
    "pignorada": "pignorar",
    "pignoradas": "pignorar",
    "pignorado": "pignorar",
    "pignoradora": "pignorador",
    "pignoradoras": "pignorador",
    "pignoradores": "pignorador",
    "pignorados": "pignorar",
    "pignoramos": "pignorar",
    "pignoran": "pignorar",
    "pignorando": "pignorar",
    "pignorara": "pignorar",
    "pignorarais": "pignorar",
    "pignoraran": "pignorar",
    "pignoraras": "pignorar",
    "pignorare": "pignorar",
    "pignorareis": "pignorar",
    "pignoraremos": "pignorar",
    "pignoraren": "pignorar",
    "pignorares": "pignorar",
    "pignoraron": "pignorar",
    "pignorará": "pignorar",
    "pignorarán": "pignorar",
    "pignorarás": "pignorar",
    "pignoraré": "pignorar",
    "pignoraréis": "pignorar",
    "pignoraría": "pignorar",
    "pignoraríais": "pignorar",
    "pignoraríamos": "pignorar",
    "pignorarían": "pignorar",
    "pignorarías": "pignorar",
    "pignoras": "pignorar",
    "pignorase": "pignorar",
    "pignoraseis": "pignorar",
    "pignorasen": "pignorar",
    "pignorases": "pignorar",
    "pignoraste": "pignorar",
    "pignorasteis": "pignorar",
    "pignoraticia": "pignoraticio",
    "pignoraticias": "pignoraticio",
    "pignoraticios": "pignoraticio",
    "pignore": "pignorar",
    "pignoremos": "pignorar",
    "pignoren": "pignorar",
    "pignores": "pignorar",
    "pignoro": "pignorar",
    "pignorábamos": "pignorar",
    "pignoráis": "pignorar",
    "pignoráramos": "pignorar",
    "pignoráremos": "pignorar",
    "pignorásemos": "pignorar",
    "pignoré": "pignorar",
    "pignoréis": "pignorar",
    "pignoró": "pignorar",
    "pigres": "pigre",
    "pigricias": "pigricia",
    "pihuelas": "pihuela",
    "pija": "pijo",
    "pijadas": "pijada",
    "pijamas": "pijama",
    "pijas": "pijo",
    "pijos": "pijo",
    "pijotas": "pijota",
    "pijotera": "pijotero",
    "pijoteras": "pijotero",
    "pijoteros": "pijotero",
    "pijoterías": "pijotería",
    "pijotes": "pijote",
    "pila": "pilar",
    "pilaba": "pilar",
    "pilabais": "pilar",
    "pilaban": "pilar",
    "pilabas": "pilar",
    "pilad": "pilar",
    "pilada": "pilar",
    "piladas": "pilar",
    "pilado": "pilar",
    "pilados": "pilar",
    "pilamos": "pilar",
    "pilan": "pilar",
    "pilando": "pilar",
    "pilara": "pilar",
    "pilarais": "pilar",
    "pilaran": "pilar",
    "pilaras": "pilar",
    "pilare": "pilar",
    "pilareis": "pilar",
    "pilaremos": "pilar",
    "pilaren": "pilar",
    "pilares": "pilar",
    "pilaron": "pilar",
    "pilará": "pilar",
    "pilarán": "pilar",
    "pilarás": "pilar",
    "pilaré": "pilar",
    "pilaréis": "pilar",
    "pilaría": "pilar",
    "pilaríais": "pilar",
    "pilaríamos": "pilar",
    "pilarían": "pilar",
    "pilarías": "pilar",
    "pilas": "pilar",
    "pilase": "pilar",
    "pilaseis": "pilar",
    "pilasen": "pilar",
    "pilases": "pilar",
    "pilaste": "pilar",
    "pilasteis": "pilar",
    "pilastras": "pilastra",
    "pilatera": "pilatero",
    "pilateras": "pilatero",
    "pilateros": "pilatero",
    "pilbaritas": "pilbarita",
    "pile": "pilar",
    "pilemos": "pilar",
    "pilen": "pilar",
    "piles": "pilar",
    "piletas": "pileta",
    "pililas": "pilila",
    "pilla": "pillo",
    "pillaba": "pillar",
    "pillabais": "pillar",
    "pillaban": "pillar",
    "pillabas": "pillar",
    "pillad": "pillar",
    "pillada": "pillar",
    "pilladas": "pillar",
    "pillado": "pillar",
    "pillados": "pillar",
    "pillajes": "pillaje",
    "pillamos": "pillar",
    "pillan": "pillar",
    "pillando": "pillar",
    "pillara": "pillar",
    "pillarais": "pillar",
    "pillaran": "pillar",
    "pillaras": "pillar",
    "pillare": "pillar",
    "pillareis": "pillar",
    "pillaremos": "pillar",
    "pillaren": "pillar",
    "pillares": "pillar",
    "pillaron": "pillar",
    "pillará": "pillar",
    "pillarán": "pillar",
    "pillarás": "pillar",
    "pillaré": "pillar",
    "pillaréis": "pillar",
    "pillaría": "pillar",
    "pillaríais": "pillar",
    "pillaríamos": "pillar",
    "pillarían": "pillar",
    "pillarías": "pillar",
    "pillas": "pillo",
    "pillase": "pillar",
    "pillaseis": "pillar",
    "pillasen": "pillar",
    "pillases": "pillar",
    "pillaste": "pillar",
    "pillasteis": "pillar",
    "pillastres": "pillastre",
    "pille": "pillar",
    "pillea": "pillear",
    "pilleaba": "pillear",
    "pilleabais": "pillear",
    "pilleaban": "pillear",
    "pilleabas": "pillear",
    "pillead": "pillear",
    "pilleada": "pillear",
    "pilleadas": "pillear",
    "pilleado": "pillear",
    "pilleados": "pillear",
    "pilleamos": "pillear",
    "pillean": "pillear",
    "pilleando": "pillear",
    "pilleara": "pillear",
    "pillearais": "pillear",
    "pillearan": "pillear",
    "pillearas": "pillear",
    "pilleare": "pillear",
    "pilleareis": "pillear",
    "pillearemos": "pillear",
    "pillearen": "pillear",
    "pilleares": "pillear",
    "pillearon": "pillear",
    "pilleará": "pillear",
    "pillearán": "pillear",
    "pillearás": "pillear",
    "pillearé": "pillear",
    "pillearéis": "pillear",
    "pillearía": "pillear",
    "pillearíais": "pillear",
    "pillearíamos": "pillear",
    "pillearían": "pillear",
    "pillearías": "pillear",
    "pilleas": "pillear",
    "pillease": "pillear",
    "pilleaseis": "pillear",
    "pilleasen": "pillear",
    "pilleases": "pillear",
    "pilleaste": "pillear",
    "pilleasteis": "pillear",
    "pillee": "pillear",
    "pilleemos": "pillear",
    "pilleen": "pillear",
    "pillees": "pillear",
    "pillemos": "pillar",
    "pillen": "pillar",
    "pilleo": "pillear",
    "pillerías": "pillería",
    "pilles": "pillar",
    "pilletes": "pillete",
    "pilleábamos": "pillear",
    "pilleáis": "pillear",
    "pilleáramos": "pillear",
    "pilleáremos": "pillear",
    "pilleásemos": "pillear",
    "pilleé": "pillear",
    "pilleéis": "pillear",
    "pilleó": "pillear",
    "pillo": "pillar",
    "pillos": "pillo",
    "pilluela": "pilluelo",
    "pilluelas": "pilluelo",
    "pilluelos": "pilluelo",
    "pillábamos": "pillar",
    "pilláis": "pillar",
    "pilláramos": "pillar",
    "pilláremos": "pillar",
    "pillásemos": "pillar",
    "pillé": "pillar",
    "pilléis": "pillar",
    "pilló": "pillar",
    "pilo": "pilar",
    "pilocarpinas": "pilocarpina",
    "pilonera": "pilonero",
    "piloneras": "pilonero",
    "piloneros": "pilonero",
    "pilones": "pilón",
    "pilonga": "pilongo",
    "pilongas": "pilongo",
    "pilongos": "pilongo",
    "pilonos": "pilono",
    "pilosa": "piloso",
    "pilosas": "piloso",
    "pilosos": "piloso",
    "pilota": "piloto",
    "pilotaba": "pilotar",
    "pilotabais": "pilotar",
    "pilotaban": "pilotar",
    "pilotabas": "pilotar",
    "pilotad": "pilotar",
    "pilotada": "pilotar",
    "pilotadas": "pilotar",
    "pilotado": "pilotar",
    "pilotados": "pilotar",
    "pilotajes": "pilotaje",
    "pilotamos": "pilotar",
    "pilotan": "pilotar",
    "pilotando": "pilotar",
    "pilotara": "pilotar",
    "pilotarais": "pilotar",
    "pilotaran": "pilotar",
    "pilotaras": "pilotar",
    "pilotare": "pilotar",
    "pilotareis": "pilotar",
    "pilotaremos": "pilotar",
    "pilotaren": "pilotar",
    "pilotares": "pilotar",
    "pilotaron": "pilotar",
    "pilotará": "pilotar",
    "pilotarán": "pilotar",
    "pilotarás": "pilotar",
    "pilotaré": "pilotar",
    "pilotaréis": "pilotar",
    "pilotaría": "pilotar",
    "pilotaríais": "pilotar",
    "pilotaríamos": "pilotar",
    "pilotarían": "pilotar",
    "pilotarías": "pilotar",
    "pilotas": "piloto",
    "pilotase": "pilotar",
    "pilotaseis": "pilotar",
    "pilotasen": "pilotar",
    "pilotases": "pilotar",
    "pilotaste": "pilotar",
    "pilotasteis": "pilotar",
    "pilote": "pilotar",
    "pilotea": "pilotear",
    "piloteaba": "pilotear",
    "piloteabais": "pilotear",
    "piloteaban": "pilotear",
    "piloteabas": "pilotear",
    "pilotead": "pilotear",
    "piloteada": "pilotear",
    "piloteadas": "pilotear",
    "piloteado": "pilotear",
    "piloteados": "pilotear",
    "piloteamos": "pilotear",
    "pilotean": "pilotear",
    "piloteando": "pilotear",
    "piloteara": "pilotear",
    "pilotearais": "pilotear",
    "pilotearan": "pilotear",
    "pilotearas": "pilotear",
    "piloteare": "pilotear",
    "piloteareis": "pilotear",
    "pilotearemos": "pilotear",
    "pilotearen": "pilotear",
    "piloteares": "pilotear",
    "pilotearon": "pilotear",
    "piloteará": "pilotear",
    "pilotearán": "pilotear",
    "pilotearás": "pilotear",
    "pilotearé": "pilotear",
    "pilotearéis": "pilotear",
    "pilotearía": "pilotear",
    "pilotearíais": "pilotear",
    "pilotearíamos": "pilotear",
    "pilotearían": "pilotear",
    "pilotearías": "pilotear",
    "piloteas": "pilotear",
    "pilotease": "pilotear",
    "piloteaseis": "pilotear",
    "piloteasen": "pilotear",
    "piloteases": "pilotear",
    "piloteaste": "pilotear",
    "piloteasteis": "pilotear",
    "pilotee": "pilotear",
    "piloteemos": "pilotear",
    "piloteen": "pilotear",
    "pilotees": "pilotear",
    "pilotemos": "pilotar",
    "piloten": "pilotar",
    "piloteo": "pilotear",
    "pilotes": "pilote",
    "piloteábamos": "pilotear",
    "piloteáis": "pilotear",
    "piloteáramos": "pilotear",
    "piloteáremos": "pilotear",
    "piloteásemos": "pilotear",
    "piloteé": "pilotear",
    "piloteéis": "pilotear",
    "piloteó": "pilotear",
    "piloto": "pilotar",
    "pilotos": "piloto",
    "pilotábamos": "pilotar",
    "pilotáis": "pilotar",
    "pilotáramos": "pilotar",
    "pilotáremos": "pilotar",
    "pilotásemos": "pilotar",
    "piloté": "pilotar",
    "pilotéis": "pilotar",
    "pilotó": "pilotar",
    "pilpiles": "pilpil",
    "piltracas": "piltraca",
    "piltrafas": "piltrafa",
    "pilábamos": "pilar",
    "piláis": "pilar",
    "piláramos": "pilar",
    "piláremos": "pilar",
    "pilásemos": "pilar",
    "pilé": "pilar",
    "piléis": "pilar",
    "piló": "pilar",
    "pilórica": "pilórico",
    "pilóricas": "pilórico",
    "pilóricos": "pilórico",
    "pimaricinas": "pimaricina",
    "pimenteros": "pimentero",
    "pimentones": "pimentón",
    "pimientas": "pimienta",
    "pimientos": "pimiento",
    "pimpinelas": "pimpinela",
    "pimpla": "pimplar",
    "pimplaba": "pimplar",
    "pimplabais": "pimplar",
    "pimplaban": "pimplar",
    "pimplabas": "pimplar",
    "pimplad": "pimplar",
    "pimplada": "pimplar",
    "pimpladas": "pimplar",
    "pimplado": "pimplar",
    "pimplados": "pimplar",
    "pimplamos": "pimplar",
    "pimplan": "pimplar",
    "pimplando": "pimplar",
    "pimplara": "pimplar",
    "pimplarais": "pimplar",
    "pimplaran": "pimplar",
    "pimplaras": "pimplar",
    "pimplare": "pimplar",
    "pimplareis": "pimplar",
    "pimplaremos": "pimplar",
    "pimplaren": "pimplar",
    "pimplares": "pimplar",
    "pimplaron": "pimplar",
    "pimplará": "pimplar",
    "pimplarán": "pimplar",
    "pimplarás": "pimplar",
    "pimplaré": "pimplar",
    "pimplaréis": "pimplar",
    "pimplaría": "pimplar",
    "pimplaríais": "pimplar",
    "pimplaríamos": "pimplar",
    "pimplarían": "pimplar",
    "pimplarías": "pimplar",
    "pimplas": "pimplar",
    "pimplase": "pimplar",
    "pimplaseis": "pimplar",
    "pimplasen": "pimplar",
    "pimplases": "pimplar",
    "pimplaste": "pimplar",
    "pimplasteis": "pimplar",
    "pimple": "pimplar",
    "pimplea": "pimpleo",
    "pimpleas": "pimpleo",
    "pimplemos": "pimplar",
    "pimplen": "pimplar",
    "pimpleos": "pimpleo",
    "pimples": "pimplar",
    "pimplo": "pimplar",
    "pimplábamos": "pimplar",
    "pimpláis": "pimplar",
    "pimpláramos": "pimplar",
    "pimpláremos": "pimplar",
    "pimplásemos": "pimplar",
    "pimplé": "pimplar",
    "pimpléis": "pimplar",
    "pimpló": "pimplar",
    "pimpolladas": "pimpollada",
    "pimpollares": "pimpollar",
    "pimpollea": "pimpollear",
    "pimpolleaba": "pimpollear",
    "pimpolleabais": "pimpollear",
    "pimpolleaban": "pimpollear",
    "pimpolleabas": "pimpollear",
    "pimpollead": "pimpollear",
    "pimpolleada": "pimpollear",
    "pimpolleadas": "pimpollear",
    "pimpolleado": "pimpollear",
    "pimpolleados": "pimpollear",
    "pimpolleamos": "pimpollear",
    "pimpollean": "pimpollear",
    "pimpolleando": "pimpollear",
    "pimpolleara": "pimpollear",
    "pimpollearais": "pimpollear",
    "pimpollearan": "pimpollear",
    "pimpollearas": "pimpollear",
    "pimpolleare": "pimpollear",
    "pimpolleareis": "pimpollear",
    "pimpollearemos": "pimpollear",
    "pimpollearen": "pimpollear",
    "pimpolleares": "pimpollear",
    "pimpollearon": "pimpollear",
    "pimpolleará": "pimpollear",
    "pimpollearán": "pimpollear",
    "pimpollearás": "pimpollear",
    "pimpollearé": "pimpollear",
    "pimpollearéis": "pimpollear",
    "pimpollearía": "pimpollear",
    "pimpollearíais": "pimpollear",
    "pimpollearíamos": "pimpollear",
    "pimpollearían": "pimpollear",
    "pimpollearías": "pimpollear",
    "pimpolleas": "pimpollear",
    "pimpollease": "pimpollear",
    "pimpolleaseis": "pimpollear",
    "pimpolleasen": "pimpollear",
    "pimpolleases": "pimpollear",
    "pimpolleaste": "pimpollear",
    "pimpolleasteis": "pimpollear",
    "pimpollece": "pimpollecer",
    "pimpolleced": "pimpollecer",
    "pimpollecemos": "pimpollecer",
    "pimpollecen": "pimpollecer",
    "pimpolleceremos": "pimpollecer",
    "pimpollecerá": "pimpollecer",
    "pimpollecerán": "pimpollecer",
    "pimpollecerás": "pimpollecer",
    "pimpolleceré": "pimpollecer",
    "pimpolleceréis": "pimpollecer",
    "pimpollecería": "pimpollecer",
    "pimpolleceríais": "pimpollecer",
    "pimpolleceríamos": "pimpollecer",
    "pimpollecerían": "pimpollecer",
    "pimpollecerías": "pimpollecer",
    "pimpolleces": "pimpollecer",
    "pimpollecida": "pimpollecer",
    "pimpollecidas": "pimpollecer",
    "pimpollecido": "pimpollecer",
    "pimpollecidos": "pimpollecer",
    "pimpolleciendo": "pimpollecer",
    "pimpolleciera": "pimpollecer",
    "pimpollecierais": "pimpollecer",
    "pimpollecieran": "pimpollecer",
    "pimpollecieras": "pimpollecer",
    "pimpolleciere": "pimpollecer",
    "pimpolleciereis": "pimpollecer",
    "pimpollecieren": "pimpollecer",
    "pimpollecieres": "pimpollecer",
    "pimpollecieron": "pimpollecer",
    "pimpolleciese": "pimpollecer",
    "pimpollecieseis": "pimpollecer",
    "pimpolleciesen": "pimpollecer",
    "pimpollecieses": "pimpollecer",
    "pimpollecimos": "pimpollecer",
    "pimpolleciste": "pimpollecer",
    "pimpollecisteis": "pimpollecer",
    "pimpolleciéramos": "pimpollecer",
    "pimpolleciéremos": "pimpollecer",
    "pimpolleciésemos": "pimpollecer",
    "pimpolleció": "pimpollecer",
    "pimpollecéis": "pimpollecer",
    "pimpollecí": "pimpollecer",
    "pimpollecía": "pimpollecer",
    "pimpollecíais": "pimpollecer",
    "pimpollecíamos": "pimpollecer",
    "pimpollecían": "pimpollecer",
    "pimpollecías": "pimpollecer",
    "pimpollee": "pimpollear",
    "pimpolleemos": "pimpollear",
    "pimpolleen": "pimpollear",
    "pimpollees": "pimpollear",
    "pimpolleo": "pimpollear",
    "pimpollezca": "pimpollecer",
    "pimpollezcamos": "pimpollecer",
    "pimpollezcan": "pimpollecer",
    "pimpollezcas": "pimpollecer",
    "pimpollezco": "pimpollecer",
    "pimpollezcáis": "pimpollecer",
    "pimpolleábamos": "pimpollear",
    "pimpolleáis": "pimpollear",
    "pimpolleáramos": "pimpollear",
    "pimpolleáremos": "pimpollear",
    "pimpolleásemos": "pimpollear",
    "pimpolleé": "pimpollear",
    "pimpolleéis": "pimpollear",
    "pimpolleó": "pimpollear",
    "pimpollos": "pimpollo",
    "pimpolluda": "pimpolludo",
    "pimpolludas": "pimpolludo",
    "pimpolludos": "pimpolludo",
    "pina": "pino",
    "pinabetes": "pinabete",
    "pinacates": "pinacate",
    "pinacles": "pinacle",
    "pinacotecas": "pinacoteca",
    "pinada": "pinado",
    "pinadas": "pinado",
    "pinados": "pinado",
    "pinares": "pinar",
    "pinariega": "pinariego",
    "pinariegas": "pinariego",
    "pinariegos": "pinariego",
    "pinas": "pino",
    "pinatífida": "pinatífido",
    "pinatífidas": "pinatífido",
    "pinatífidos": "pinatífido",
    "pinazas": "pinaza",
    "pince": "pinzar",
    "pincela": "pincelar",
    "pincelaba": "pincelar",
    "pincelabais": "pincelar",
    "pincelaban": "pincelar",
    "pincelabas": "pincelar",
    "pincelad": "pincelar",
    "pincelada": "pincelar",
    "pinceladas": "pincelar",
    "pincelado": "pincelar",
    "pincelados": "pincelar",
    "pincelamos": "pincelar",
    "pincelan": "pincelar",
    "pincelando": "pincelar",
    "pincelara": "pincelar",
    "pincelarais": "pincelar",
    "pincelaran": "pincelar",
    "pincelaras": "pincelar",
    "pincelare": "pincelar",
    "pincelareis": "pincelar",
    "pincelaremos": "pincelar",
    "pincelaren": "pincelar",
    "pincelares": "pincelar",
    "pincelaron": "pincelar",
    "pincelará": "pincelar",
    "pincelarán": "pincelar",
    "pincelarás": "pincelar",
    "pincelaré": "pincelar",
    "pincelaréis": "pincelar",
    "pincelaría": "pincelar",
    "pincelaríais": "pincelar",
    "pincelaríamos": "pincelar",
    "pincelarían": "pincelar",
    "pincelarías": "pincelar",
    "pincelas": "pincelar",
    "pincelase": "pincelar",
    "pincelaseis": "pincelar",
    "pincelasen": "pincelar",
    "pincelases": "pincelar",
    "pincelaste": "pincelar",
    "pincelasteis": "pincelar",
    "pincele": "pincelar",
    "pincelemos": "pincelar",
    "pincelen": "pincelar",
    "pincelera": "pincelero",
    "pinceleras": "pincelero",
    "pinceleros": "pincelero",
    "pinceles": "pincelar",
    "pincelo": "pincelar",
    "pincelábamos": "pincelar",
    "pinceláis": "pincelar",
    "pinceláramos": "pincelar",
    "pinceláremos": "pincelar",
    "pincelásemos": "pincelar",
    "pincelé": "pincelar",
    "pinceléis": "pincelar",
    "pinceló": "pincelar",
    "pincemos": "pinzar",
    "pincen": "pinzar",
    "pinces": "pinzar",
    "pincha": "pincho",
    "pinchaba": "pinchar",
    "pinchabais": "pinchar",
    "pinchaban": "pinchar",
    "pinchabas": "pinchar",
    "pinchad": "pinchar",
    "pinchada": "pinchar",
    "pinchadas": "pinchar",
    "pinchado": "pinchar",
    "pinchados": "pinchar",
    "pinchaduras": "pinchadura",
    "pinchamos": "pinchar",
    "pinchan": "pinchar",
    "pinchando": "pinchar",
    "pinchara": "pinchar",
    "pincharais": "pinchar",
    "pincharan": "pinchar",
    "pincharas": "pinchar",
    "pinchare": "pinchar",
    "pinchareis": "pinchar",
    "pincharemos": "pinchar",
    "pincharen": "pinchar",
    "pinchares": "pinchar",
    "pincharon": "pinchar",
    "pinchará": "pinchar",
    "pincharán": "pinchar",
    "pincharás": "pinchar",
    "pincharé": "pinchar",
    "pincharéis": "pinchar",
    "pincharía": "pinchar",
    "pincharíais": "pinchar",
    "pincharíamos": "pinchar",
    "pincharían": "pinchar",
    "pincharías": "pinchar",
    "pinchas": "pincho",
    "pinchase": "pinchar",
    "pinchaseis": "pinchar",
    "pinchasen": "pinchar",
    "pinchases": "pinchar",
    "pinchaste": "pinchar",
    "pinchasteis": "pinchar",
    "pinchazos": "pinchazo",
    "pinche": "pinchar",
    "pinchemos": "pinchar",
    "pinchen": "pinchar",
    "pinches": "pinche",
    "pinchitos": "pinchito",
    "pincho": "pinchar",
    "pinchones": "pinchón",
    "pinchos": "pincho",
    "pinchuda": "pinchudo",
    "pinchudas": "pinchudo",
    "pinchudos": "pinchudo",
    "pinchábamos": "pinchar",
    "pincháis": "pinchar",
    "pincháramos": "pinchar",
    "pincháremos": "pinchar",
    "pinchásemos": "pinchar",
    "pinché": "pinchar",
    "pinchéis": "pinchar",
    "pinchés": "pinché",
    "pinchó": "pinchar",
    "pinciana": "pinciano",
    "pincianas": "pinciano",
    "pincianos": "pinciano",
    "pincé": "pinzar",
    "pincéis": "pinzar",
    "pindongas": "pindonga",
    "pindonguea": "pindonguear",
    "pindongueaba": "pindonguear",
    "pindongueabais": "pindonguear",
    "pindongueaban": "pindonguear",
    "pindongueabas": "pindonguear",
    "pindonguead": "pindonguear",
    "pindongueada": "pindonguear",
    "pindongueadas": "pindonguear",
    "pindongueado": "pindonguear",
    "pindongueados": "pindonguear",
    "pindongueamos": "pindonguear",
    "pindonguean": "pindonguear",
    "pindongueando": "pindonguear",
    "pindongueara": "pindonguear",
    "pindonguearais": "pindonguear",
    "pindonguearan": "pindonguear",
    "pindonguearas": "pindonguear",
    "pindongueare": "pindonguear",
    "pindongueareis": "pindonguear",
    "pindonguearemos": "pindonguear",
    "pindonguearen": "pindonguear",
    "pindongueares": "pindonguear",
    "pindonguearon": "pindonguear",
    "pindongueará": "pindonguear",
    "pindonguearán": "pindonguear",
    "pindonguearás": "pindonguear",
    "pindonguearé": "pindonguear",
    "pindonguearéis": "pindonguear",
    "pindonguearía": "pindonguear",
    "pindonguearíais": "pindonguear",
    "pindonguearíamos": "pindonguear",
    "pindonguearían": "pindonguear",
    "pindonguearías": "pindonguear",
    "pindongueas": "pindonguear",
    "pindonguease": "pindonguear",
    "pindongueaseis": "pindonguear",
    "pindongueasen": "pindonguear",
    "pindongueases": "pindonguear",
    "pindongueaste": "pindonguear",
    "pindongueasteis": "pindonguear",
    "pindonguee": "pindonguear",
    "pindongueemos": "pindonguear",
    "pindongueen": "pindonguear",
    "pindonguees": "pindonguear",
    "pindongueo": "pindonguear",
    "pindongueábamos": "pindonguear",
    "pindongueáis": "pindonguear",
    "pindongueáramos": "pindonguear",
    "pindongueáremos": "pindonguear",
    "pindongueásemos": "pindonguear",
    "pindongueé": "pindonguear",
    "pindongueéis": "pindonguear",
    "pindongueó": "pindonguear",
    "pineales": "pineal",
    "pinedas": "pineda",
    "pinedos": "pinedo",
    "pinenos": "pineno",
    "ping-pongs": "ping-pong",
    "pingajos": "pingajo",
    "pingajosa": "pingajoso",
    "pingajosas": "pingajoso",
    "pingajosos": "pingajoso",
    "pinganellos": "pinganello",
    "pingopingos": "pingopingo",
    "pingorotas": "pingorota",
    "pingorotes": "pingorote",
    "pingorotuda": "pingorotudo",
    "pingorotudas": "pingorotudo",
    "pingorotudos": "pingorotudo",
    "pingos": "pingo",
    "pingpongistas": "pingpongista",
    "pingues": "pingue",
    "pingües": "pingüe",
    "pingüinos": "pingüino",
    "pinitos": "pinito",
    "pinjantes": "pinjante",
    "pinnada": "pinnado",
    "pinnadas": "pinnado",
    "pinnados": "pinnado",
    "pinnaticompuesta": "pinnaticompuesto",
    "pinnaticompuestas": "pinnaticompuesto",
    "pinnaticompuestos": "pinnaticompuesto",
    "pinnatinervia": "pinnatinervio",
    "pinnatinervias": "pinnatinervio",
    "pinnatinervios": "pinnatinervio",
    "pinnatipartida": "pinnatipartido",
    "pinnatipartidas": "pinnatipartido",
    "pinnatipartidos": "pinnatipartido",
    "pinnatisecta": "pinnatisecto",
    "pinnatisectas": "pinnatisecto",
    "pinnatisectos": "pinnatisecto",
    "pinnatífida": "pinnatífido",
    "pinnatífidas": "pinnatífido",
    "pinnatífidos": "pinnatífido",
    "pinnípedos": "pinnípedo",
    "pinochas": "pinocha",
    "pinochetistas": "pinochetista",
    "pinochos": "pinocho",
    "pinocitótica": "pinocitótico",
    "pinocitóticas": "pinocitótico",
    "pinocitóticos": "pinocitótico",
    "pinoles": "pinole",
    "pinos": "pino",
    "pinreles": "pinrel",
    "pins": "pin",
    "pinsapos": "pinsapo",
    "pinta": "pintar",
    "pintaba": "pintar",
    "pintabais": "pintar",
    "pintaban": "pintar",
    "pintabas": "pintar",
    "pintad": "pintar",
    "pintada": "pintar",
    "pintadas": "pintar",
    "pintaderas": "pintadera",
    "pintado": "pintar",
    "pintados": "pintar",
    "pintamos": "pintar",
    "pintan": "pintar",
    "pintando": "pintar",
    "pintara": "pintar",
    "pintarais": "pintar",
    "pintaran": "pintar",
    "pintaras": "pintar",
    "pintare": "pintar",
    "pintareis": "pintar",
    "pintaremos": "pintar",
    "pintaren": "pintar",
    "pintares": "pintar",
    "pintaron": "pintar",
    "pintarraja": "pintarrajar",
    "pintarrajaba": "pintarrajar",
    "pintarrajabais": "pintarrajar",
    "pintarrajaban": "pintarrajar",
    "pintarrajabas": "pintarrajar",
    "pintarrajad": "pintarrajar",
    "pintarrajada": "pintarrajar",
    "pintarrajadas": "pintarrajar",
    "pintarrajado": "pintarrajar",
    "pintarrajados": "pintarrajar",
    "pintarrajamos": "pintarrajar",
    "pintarrajan": "pintarrajar",
    "pintarrajando": "pintarrajar",
    "pintarrajara": "pintarrajar",
    "pintarrajarais": "pintarrajar",
    "pintarrajaran": "pintarrajar",
    "pintarrajaras": "pintarrajar",
    "pintarrajare": "pintarrajar",
    "pintarrajareis": "pintarrajar",
    "pintarrajaremos": "pintarrajar",
    "pintarrajaren": "pintarrajar",
    "pintarrajares": "pintarrajar",
    "pintarrajaron": "pintarrajar",
    "pintarrajará": "pintarrajar",
    "pintarrajarán": "pintarrajar",
    "pintarrajarás": "pintarrajar",
    "pintarrajaré": "pintarrajar",
    "pintarrajaréis": "pintarrajar",
    "pintarrajaría": "pintarrajar",
    "pintarrajaríais": "pintarrajar",
    "pintarrajaríamos": "pintarrajar",
    "pintarrajarían": "pintarrajar",
    "pintarrajarías": "pintarrajar",
    "pintarrajas": "pintarrajar",
    "pintarrajase": "pintarrajar",
    "pintarrajaseis": "pintarrajar",
    "pintarrajasen": "pintarrajar",
    "pintarrajases": "pintarrajar",
    "pintarrajaste": "pintarrajar",
    "pintarrajasteis": "pintarrajar",
    "pintarraje": "pintarrajar",
    "pintarrajea": "pintarrajear",
    "pintarrajeaba": "pintarrajear",
    "pintarrajeabais": "pintarrajear",
    "pintarrajeaban": "pintarrajear",
    "pintarrajeabas": "pintarrajear",
    "pintarrajead": "pintarrajear",
    "pintarrajeada": "pintarrajear",
    "pintarrajeadas": "pintarrajear",
    "pintarrajeado": "pintarrajear",
    "pintarrajeados": "pintarrajear",
    "pintarrajeamos": "pintarrajear",
    "pintarrajean": "pintarrajear",
    "pintarrajeando": "pintarrajear",
    "pintarrajeara": "pintarrajear",
    "pintarrajearais": "pintarrajear",
    "pintarrajearan": "pintarrajear",
    "pintarrajearas": "pintarrajear",
    "pintarrajeare": "pintarrajear",
    "pintarrajeareis": "pintarrajear",
    "pintarrajearemos": "pintarrajear",
    "pintarrajearen": "pintarrajear",
    "pintarrajeares": "pintarrajear",
    "pintarrajearon": "pintarrajear",
    "pintarrajeará": "pintarrajear",
    "pintarrajearán": "pintarrajear",
    "pintarrajearás": "pintarrajear",
    "pintarrajearé": "pintarrajear",
    "pintarrajearéis": "pintarrajear",
    "pintarrajearía": "pintarrajear",
    "pintarrajearíais": "pintarrajear",
    "pintarrajearíamos": "pintarrajear",
    "pintarrajearían": "pintarrajear",
    "pintarrajearías": "pintarrajear",
    "pintarrajeas": "pintarrajear",
    "pintarrajease": "pintarrajear",
    "pintarrajeaseis": "pintarrajear",
    "pintarrajeasen": "pintarrajear",
    "pintarrajeases": "pintarrajear",
    "pintarrajeaste": "pintarrajear",
    "pintarrajeasteis": "pintarrajear",
    "pintarrajee": "pintarrajear",
    "pintarrajeemos": "pintarrajear",
    "pintarrajeen": "pintarrajear",
    "pintarrajees": "pintarrajear",
    "pintarrajemos": "pintarrajar",
    "pintarrajen": "pintarrajar",
    "pintarrajeo": "pintarrajear",
    "pintarrajes": "pintarrajar",
    "pintarrajeábamos": "pintarrajear",
    "pintarrajeáis": "pintarrajear",
    "pintarrajeáramos": "pintarrajear",
    "pintarrajeáremos": "pintarrajear",
    "pintarrajeásemos": "pintarrajear",
    "pintarrajeé": "pintarrajear",
    "pintarrajeéis": "pintarrajear",
    "pintarrajeó": "pintarrajear",
    "pintarrajo": "pintarrajar",
    "pintarrajábamos": "pintarrajar",
    "pintarrajáis": "pintarrajar",
    "pintarrajáramos": "pintarrajar",
    "pintarrajáremos": "pintarrajar",
    "pintarrajásemos": "pintarrajar",
    "pintarrajé": "pintarrajar",
    "pintarrajéis": "pintarrajar",
    "pintarrajó": "pintarrajar",
    "pintarrojas": "pintarroja",
    "pintarrojos": "pintarrojo",
    "pintará": "pintar",
    "pintarán": "pintar",
    "pintarás": "pintar",
    "pintaré": "pintar",
    "pintaréis": "pintar",
    "pintaría": "pintar",
    "pintaríais": "pintar",
    "pintaríamos": "pintar",
    "pintarían": "pintar",
    "pintarías": "pintar",
    "pintas": "pintar",
    "pintase": "pintar",
    "pintaseis": "pintar",
    "pintasen": "pintar",
    "pintases": "pintar",
    "pintaste": "pintar",
    "pintasteis": "pintar",
    "pinte": "pintar",
    "pintea": "pintear",
    "pinteaba": "pintear",
    "pinteabais": "pintear",
    "pinteaban": "pintear",
    "pinteabas": "pintear",
    "pintead": "pintear",
    "pinteada": "pintear",
    "pinteadas": "pintear",
    "pinteado": "pintear",
    "pinteados": "pintear",
    "pinteamos": "pintear",
    "pintean": "pintear",
    "pinteando": "pintear",
    "pinteara": "pintear",
    "pintearais": "pintear",
    "pintearan": "pintear",
    "pintearas": "pintear",
    "pinteare": "pintear",
    "pinteareis": "pintear",
    "pintearemos": "pintear",
    "pintearen": "pintear",
    "pinteares": "pintear",
    "pintearon": "pintear",
    "pinteará": "pintear",
    "pintearán": "pintear",
    "pintearás": "pintear",
    "pintearé": "pintear",
    "pintearéis": "pintear",
    "pintearía": "pintear",
    "pintearíais": "pintear",
    "pintearíamos": "pintear",
    "pintearían": "pintear",
    "pintearías": "pintear",
    "pinteas": "pintear",
    "pintease": "pintear",
    "pinteaseis": "pintear",
    "pinteasen": "pintear",
    "pinteases": "pintear",
    "pinteaste": "pintear",
    "pinteasteis": "pintear",
    "pintee": "pintear",
    "pinteemos": "pintear",
    "pinteen": "pintear",
    "pintees": "pintear",
    "pintemos": "pintar",
    "pinten": "pintar",
    "pinteo": "pintear",
    "pintes": "pintar",
    "pinteábamos": "pintear",
    "pinteáis": "pintear",
    "pinteáramos": "pintear",
    "pinteáremos": "pintear",
    "pinteásemos": "pintear",
    "pinteé": "pintear",
    "pinteéis": "pintear",
    "pinteó": "pintear",
    "pintipara": "pintiparar",
    "pintiparaba": "pintiparar",
    "pintiparabais": "pintiparar",
    "pintiparaban": "pintiparar",
    "pintiparabas": "pintiparar",
    "pintiparad": "pintiparar",
    "pintiparada": "pintiparar",
    "pintiparadas": "pintiparar",
    "pintiparado": "pintiparar",
    "pintiparados": "pintiparar",
    "pintiparamos": "pintiparar",
    "pintiparan": "pintiparar",
    "pintiparando": "pintiparar",
    "pintiparara": "pintiparar",
    "pintipararais": "pintiparar",
    "pintipararan": "pintiparar",
    "pintipararas": "pintiparar",
    "pintiparare": "pintiparar",
    "pintiparareis": "pintiparar",
    "pintipararemos": "pintiparar",
    "pintipararen": "pintiparar",
    "pintiparares": "pintiparar",
    "pintipararon": "pintiparar",
    "pintiparará": "pintiparar",
    "pintipararán": "pintiparar",
    "pintipararás": "pintiparar",
    "pintipararé": "pintiparar",
    "pintipararéis": "pintiparar",
    "pintipararía": "pintiparar",
    "pintipararíais": "pintiparar",
    "pintipararíamos": "pintiparar",
    "pintipararían": "pintiparar",
    "pintipararías": "pintiparar",
    "pintiparas": "pintiparar",
    "pintiparase": "pintiparar",
    "pintiparaseis": "pintiparar",
    "pintiparasen": "pintiparar",
    "pintiparases": "pintiparar",
    "pintiparaste": "pintiparar",
    "pintiparasteis": "pintiparar",
    "pintipare": "pintiparar",
    "pintiparemos": "pintiparar",
    "pintiparen": "pintiparar",
    "pintipares": "pintiparar",
    "pintiparo": "pintiparar",
    "pintiparábamos": "pintiparar",
    "pintiparáis": "pintiparar",
    "pintiparáramos": "pintiparar",
    "pintiparáremos": "pintiparar",
    "pintiparásemos": "pintiparar",
    "pintiparé": "pintiparar",
    "pintiparéis": "pintiparar",
    "pintiparó": "pintiparar",
    "pinto": "pintar",
    "pintoja": "pintojo",
    "pintojas": "pintojo",
    "pintojos": "pintojo",
    "pintona": "pintón",
    "pintonas": "pintón",
    "pintonea": "pintonear",
    "pintoneaba": "pintonear",
    "pintoneabais": "pintonear",
    "pintoneaban": "pintonear",
    "pintoneabas": "pintonear",
    "pintonead": "pintonear",
    "pintoneada": "pintonear",
    "pintoneadas": "pintonear",
    "pintoneado": "pintonear",
    "pintoneados": "pintonear",
    "pintoneamos": "pintonear",
    "pintonean": "pintonear",
    "pintoneando": "pintonear",
    "pintoneara": "pintonear",
    "pintonearais": "pintonear",
    "pintonearan": "pintonear",
    "pintonearas": "pintonear",
    "pintoneare": "pintonear",
    "pintoneareis": "pintonear",
    "pintonearemos": "pintonear",
    "pintonearen": "pintonear",
    "pintoneares": "pintonear",
    "pintonearon": "pintonear",
    "pintoneará": "pintonear",
    "pintonearán": "pintonear",
    "pintonearás": "pintonear",
    "pintonearé": "pintonear",
    "pintonearéis": "pintonear",
    "pintonearía": "pintonear",
    "pintonearíais": "pintonear",
    "pintonearíamos": "pintonear",
    "pintonearían": "pintonear",
    "pintonearías": "pintonear",
    "pintoneas": "pintonear",
    "pintonease": "pintonear",
    "pintoneaseis": "pintonear",
    "pintoneasen": "pintonear",
    "pintoneases": "pintonear",
    "pintoneaste": "pintonear",
    "pintoneasteis": "pintonear",
    "pintonee": "pintonear",
    "pintoneemos": "pintonear",
    "pintoneen": "pintonear",
    "pintonees": "pintonear",
    "pintoneo": "pintonear",
    "pintones": "pintón",
    "pintoneábamos": "pintonear",
    "pintoneáis": "pintonear",
    "pintoneáramos": "pintonear",
    "pintoneáremos": "pintonear",
    "pintoneásemos": "pintonear",
    "pintoneé": "pintonear",
    "pintoneéis": "pintonear",
    "pintoneó": "pintonear",
    "pintora": "pintor",
    "pintoras": "pintor",
    "pintores": "pintor",
    "pintoresca": "pintoresco",
    "pintorescas": "pintoresco",
    "pintorescos": "pintoresco",
    "pintoresquismos": "pintoresquismo",
    "pintorrea": "pintorrear",
    "pintorreaba": "pintorrear",
    "pintorreabais": "pintorrear",
    "pintorreaban": "pintorrear",
    "pintorreabas": "pintorrear",
    "pintorread": "pintorrear",
    "pintorreada": "pintorrear",
    "pintorreadas": "pintorrear",
    "pintorreado": "pintorrear",
    "pintorreados": "pintorrear",
    "pintorreamos": "pintorrear",
    "pintorrean": "pintorrear",
    "pintorreando": "pintorrear",
    "pintorreara": "pintorrear",
    "pintorrearais": "pintorrear",
    "pintorrearan": "pintorrear",
    "pintorrearas": "pintorrear",
    "pintorreare": "pintorrear",
    "pintorreareis": "pintorrear",
    "pintorrearemos": "pintorrear",
    "pintorrearen": "pintorrear",
    "pintorreares": "pintorrear",
    "pintorrearon": "pintorrear",
    "pintorreará": "pintorrear",
    "pintorrearán": "pintorrear",
    "pintorrearás": "pintorrear",
    "pintorrearé": "pintorrear",
    "pintorrearéis": "pintorrear",
    "pintorrearía": "pintorrear",
    "pintorrearíais": "pintorrear",
    "pintorrearíamos": "pintorrear",
    "pintorrearían": "pintorrear",
    "pintorrearías": "pintorrear",
    "pintorreas": "pintorrear",
    "pintorrease": "pintorrear",
    "pintorreaseis": "pintorrear",
    "pintorreasen": "pintorrear",
    "pintorreases": "pintorrear",
    "pintorreaste": "pintorrear",
    "pintorreasteis": "pintorrear",
    "pintorree": "pintorrear",
    "pintorreemos": "pintorrear",
    "pintorreen": "pintorrear",
    "pintorrees": "pintorrear",
    "pintorreo": "pintorrear",
    "pintorreábamos": "pintorrear",
    "pintorreáis": "pintorrear",
    "pintorreáramos": "pintorrear",
    "pintorreáremos": "pintorrear",
    "pintorreásemos": "pintorrear",
    "pintorreé": "pintorrear",
    "pintorreéis": "pintorrear",
    "pintorreó": "pintorrear",
    "pinturas": "pintura",
    "pinturera": "pinturero",
    "pintureras": "pinturero",
    "pintureros": "pinturero",
    "pintábamos": "pintar",
    "pintáis": "pintar",
    "pintáramos": "pintar",
    "pintáremos": "pintar",
    "pintásemos": "pintar",
    "pinté": "pintar",
    "pintéis": "pintar",
    "pintó": "pintar",
    "pinza": "pinzar",
    "pinzaba": "pinzar",
    "pinzabais": "pinzar",
    "pinzaban": "pinzar",
    "pinzabas": "pinzar",
    "pinzad": "pinzar",
    "pinzada": "pinzar",
    "pinzadas": "pinzar",
    "pinzado": "pinzar",
    "pinzadora": "pinzador",
    "pinzadoras": "pinzador",
    "pinzadores": "pinzador",
    "pinzados": "pinzar",
    "pinzamientos": "pinzamiento",
    "pinzamos": "pinzar",
    "pinzan": "pinzar",
    "pinzando": "pinzar",
    "pinzara": "pinzar",
    "pinzarais": "pinzar",
    "pinzaran": "pinzar",
    "pinzaras": "pinzar",
    "pinzare": "pinzar",
    "pinzareis": "pinzar",
    "pinzaremos": "pinzar",
    "pinzaren": "pinzar",
    "pinzares": "pinzar",
    "pinzaron": "pinzar",
    "pinzará": "pinzar",
    "pinzarán": "pinzar",
    "pinzarás": "pinzar",
    "pinzaré": "pinzar",
    "pinzaréis": "pinzar",
    "pinzaría": "pinzar",
    "pinzaríais": "pinzar",
    "pinzaríamos": "pinzar",
    "pinzarían": "pinzar",
    "pinzarías": "pinzar",
    "pinzas": "pinzar",
    "pinzase": "pinzar",
    "pinzaseis": "pinzar",
    "pinzasen": "pinzar",
    "pinzases": "pinzar",
    "pinzaste": "pinzar",
    "pinzasteis": "pinzar",
    "pinzo": "pinzar",
    "pinzones": "pinzón",
    "pinzotes": "pinzote",
    "pinzábamos": "pinzar",
    "pinzáis": "pinzar",
    "pinzáramos": "pinzar",
    "pinzáremos": "pinzar",
    "pinzásemos": "pinzar",
    "pinzó": "pinzar",
    "pináceos": "pináceo",
    "pináculos": "pináculo",
    "pinífera": "pinífero",
    "piníferas": "pinífero",
    "piníferos": "pinífero",
    "piochas": "piocha",
    "piofagias": "piofagia",
    "piogenias": "piogenia",
    "piogénica": "piogénico",
    "piogénicas": "piogénico",
    "piogénicos": "piogénico",
    "piojenta": "piojento",
    "piojentas": "piojento",
    "piojentos": "piojento",
    "piojeras": "piojera",
    "piojerías": "piojería",
    "piojos": "piojo",
    "piojosa": "piojoso",
    "piojosas": "piojoso",
    "piojosos": "piojoso",
    "piojuelos": "piojuelo",
    "piola": "piolar",
    "piolaba": "piolar",
    "piolabais": "piolar",
    "piolaban": "piolar",
    "piolabas": "piolar",
    "piolad": "piolar",
    "piolada": "piolar",
    "pioladas": "piolar",
    "piolado": "piolar",
    "piolados": "piolar",
    "piolamos": "piolar",
    "piolan": "piolar",
    "piolando": "piolar",
    "piolara": "piolar",
    "piolarais": "piolar",
    "piolaran": "piolar",
    "piolaras": "piolar",
    "piolare": "piolar",
    "piolareis": "piolar",
    "piolaremos": "piolar",
    "piolaren": "piolar",
    "piolares": "piolar",
    "piolaron": "piolar",
    "piolará": "piolar",
    "piolarán": "piolar",
    "piolarás": "piolar",
    "piolaré": "piolar",
    "piolaréis": "piolar",
    "piolaría": "piolar",
    "piolaríais": "piolar",
    "piolaríamos": "piolar",
    "piolarían": "piolar",
    "piolarías": "piolar",
    "piolas": "piolar",
    "piolase": "piolar",
    "piolaseis": "piolar",
    "piolasen": "piolar",
    "piolases": "piolar",
    "piolaste": "piolar",
    "piolasteis": "piolar",
    "piole": "piolar",
    "piolemos": "piolar",
    "piolen": "piolar",
    "pioles": "piolar",
    "piolets": "piolet",
    "piolo": "piolar",
    "piolábamos": "piolar",
    "pioláis": "piolar",
    "pioláramos": "piolar",
    "pioláremos": "piolar",
    "piolásemos": "piolar",
    "piolé": "piolar",
    "pioléis": "piolar",
    "pioló": "piolar",
    "piona": "pión",
    "pionas": "pión",
    "pionera": "pionero",
    "pioneras": "pionero",
    "pioneros": "pionero",
    "piones": "pión",
    "piornos": "piorno",
    "piorreas": "piorrea",
    "pipas": "pipa",
    "pipelines": "pipeline",
    "piperacinas": "piperacina",
    "piperales": "piperal",
    "piperidinas": "piperidina",
    "piperidinoetanoles": "piperidinoetanol",
    "piperilenos": "piperileno",
    "piperinas": "piperina",
    "piperonales": "piperonal",
    "piperáceos": "piperáceo",
    "piperías": "pipería",
    "pipetas": "pipeta",
    "pipiaba": "pipiar",
    "pipiabais": "pipiar",
    "pipiaban": "pipiar",
    "pipiabas": "pipiar",
    "pipiad": "pipiar",
    "pipiada": "pipiar",
    "pipiadas": "pipiar",
    "pipiado": "pipiar",
    "pipiados": "pipiar",
    "pipiamos": "pipiar",
    "pipiando": "pipiar",
    "pipiara": "pipiar",
    "pipiarais": "pipiar",
    "pipiaran": "pipiar",
    "pipiaras": "pipiar",
    "pipiare": "pipiar",
    "pipiareis": "pipiar",
    "pipiaremos": "pipiar",
    "pipiaren": "pipiar",
    "pipiares": "pipiar",
    "pipiaron": "pipiar",
    "pipiará": "pipiar",
    "pipiarán": "pipiar",
    "pipiarás": "pipiar",
    "pipiaré": "pipiar",
    "pipiaréis": "pipiar",
    "pipiaría": "pipiar",
    "pipiaríais": "pipiar",
    "pipiaríamos": "pipiar",
    "pipiarían": "pipiar",
    "pipiarías": "pipiar",
    "pipiase": "pipiar",
    "pipiaseis": "pipiar",
    "pipiasen": "pipiar",
    "pipiases": "pipiar",
    "pipiaste": "pipiar",
    "pipiasteis": "pipiar",
    "pipiemos": "pipiar",
    "pipies": "pipi",
    "pipiola": "pipiolo",
    "pipiolas": "pipiolo",
    "pipiolos": "pipiolo",
    "pipirigallos": "pipirigallo",
    "pipirijainas": "pipirijaina",
    "pipiripaos": "pipiripao",
    "pipiritañas": "pipiritaña",
    "pipis": "pipi",
    "pipiábamos": "pipiar",
    "pipiáis": "pipiar",
    "pipiáramos": "pipiar",
    "pipiáremos": "pipiar",
    "pipiásemos": "pipiar",
    "pipié": "pipiar",
    "pipiéis": "pipiar",
    "pipió": "pipiar",
    "piporros": "piporro",
    "pipotes": "pipote",
    "pipía": "pipiar",
    "pipían": "pipiar",
    "pipías": "pipiar",
    "pipíe": "pipiar",
    "pipíen": "pipiar",
    "pipíes": "pipiar",
    "pipínidas": "pipínida",
    "pipío": "pipiar",
    "pipís": "pipí",
    "pique": "picar",
    "piquemos": "picar",
    "piquen": "picar",
    "piqueras": "piquera",
    "piques": "pique",
    "piquetas": "piqueta",
    "piquetes": "piquete",
    "piquetillas": "piquetilla",
    "piquillines": "piquillín",
    "piquituertos": "piquituerto",
    "piqué": "picar",
    "piquéis": "picar",
    "piqués": "piqué",
    "pira": "pirar",
    "piraba": "pirar",
    "pirabais": "pirar",
    "piraban": "pirar",
    "pirabas": "pirar",
    "piracinamidas": "piracinamida",
    "pirad": "pirar",
    "pirada": "pirar",
    "piradas": "pirar",
    "pirado": "pirar",
    "pirados": "pirar",
    "piraguas": "piragua",
    "piragüera": "piragüero",
    "piragüeras": "piragüero",
    "piragüeros": "piragüero",
    "piragüismos": "piragüismo",
    "piragüistas": "piragüista",
    "piragüística": "piragüístico",
    "piragüísticas": "piragüístico",
    "piragüísticos": "piragüístico",
    "piralmanditas": "piralmandita",
    "piramidaciones": "piramidación",
    "piramidales": "piramidal",
    "piramos": "pirar",
    "piran": "pirar",
    "pirandinas": "pirandina",
    "pirando": "pirar",
    "piranómetros": "piranómetro",
    "pirara": "pirar",
    "pirarais": "pirar",
    "piraran": "pirar",
    "piraras": "pirar",
    "pirare": "pirar",
    "pirareis": "pirar",
    "piraremos": "pirar",
    "piraren": "pirar",
    "pirares": "pirar",
    "pirargiritas": "pirargirita",
    "piraron": "pirar",
    "pirará": "pirar",
    "pirarán": "pirar",
    "pirarás": "pirar",
    "piraré": "pirar",
    "piraréis": "pirar",
    "piraría": "pirar",
    "piraríais": "pirar",
    "piraríamos": "pirar",
    "pirarían": "pirar",
    "pirarías": "pirar",
    "piras": "pirar",
    "pirase": "pirar",
    "piraseis": "pirar",
    "pirasen": "pirar",
    "pirases": "pirar",
    "piraste": "pirar",
    "pirasteis": "pirar",
    "piratas": "pirata",
    "piratea": "piratear",
    "pirateaba": "piratear",
    "pirateabais": "piratear",
    "pirateaban": "piratear",
    "pirateabas": "piratear",
    "piratead": "piratear",
    "pirateada": "piratear",
    "pirateadas": "piratear",
    "pirateado": "piratear",
    "pirateados": "piratear",
    "pirateamos": "piratear",
    "piratean": "piratear",
    "pirateando": "piratear",
    "pirateara": "piratear",
    "piratearais": "piratear",
    "piratearan": "piratear",
    "piratearas": "piratear",
    "pirateare": "piratear",
    "pirateareis": "piratear",
    "piratearemos": "piratear",
    "piratearen": "piratear",
    "pirateares": "piratear",
    "piratearon": "piratear",
    "pirateará": "piratear",
    "piratearán": "piratear",
    "piratearás": "piratear",
    "piratearé": "piratear",
    "piratearéis": "piratear",
    "piratearía": "piratear",
    "piratearíais": "piratear",
    "piratearíamos": "piratear",
    "piratearían": "piratear",
    "piratearías": "piratear",
    "pirateas": "piratear",
    "piratease": "piratear",
    "pirateaseis": "piratear",
    "pirateasen": "piratear",
    "pirateases": "piratear",
    "pirateaste": "piratear",
    "pirateasteis": "piratear",
    "piratee": "piratear",
    "pirateemos": "piratear",
    "pirateen": "piratear",
    "piratees": "piratear",
    "pirateo": "piratear",
    "pirateos": "pirateo",
    "piraterías": "piratería",
    "pirateábamos": "piratear",
    "pirateáis": "piratear",
    "pirateáramos": "piratear",
    "pirateáremos": "piratear",
    "pirateásemos": "piratear",
    "pirateé": "piratear",
    "pirateéis": "piratear",
    "pirateó": "piratear",
    "piraustas": "pirausta",
    "pirazolonas": "pirazolona",
    "pirañas": "piraña",
    "pirca": "pircar",
    "pircaba": "pircar",
    "pircabais": "pircar",
    "pircaban": "pircar",
    "pircabas": "pircar",
    "pircad": "pircar",
    "pircada": "pircar",
    "pircadas": "pircar",
    "pircado": "pircar",
    "pircados": "pircar",
    "pircamos": "pircar",
    "pircan": "pircar",
    "pircando": "pircar",
    "pircara": "pircar",
    "pircarais": "pircar",
    "pircaran": "pircar",
    "pircaras": "pircar",
    "pircare": "pircar",
    "pircareis": "pircar",
    "pircaremos": "pircar",
    "pircaren": "pircar",
    "pircares": "pircar",
    "pircaron": "pircar",
    "pircará": "pircar",
    "pircarán": "pircar",
    "pircarás": "pircar",
    "pircaré": "pircar",
    "pircaréis": "pircar",
    "pircaría": "pircar",
    "pircaríais": "pircar",
    "pircaríamos": "pircar",
    "pircarían": "pircar",
    "pircarías": "pircar",
    "pircas": "pircar",
    "pircase": "pircar",
    "pircaseis": "pircar",
    "pircasen": "pircar",
    "pircases": "pircar",
    "pircaste": "pircar",
    "pircasteis": "pircar",
    "pirco": "pircar",
    "pircábamos": "pircar",
    "pircáis": "pircar",
    "pircáramos": "pircar",
    "pircáremos": "pircar",
    "pircásemos": "pircar",
    "pircó": "pircar",
    "pire": "pirar",
    "piremos": "pirar",
    "piren": "pirar",
    "pirenaica": "pirenaico",
    "pirenaicas": "pirenaico",
    "pirenaicos": "pirenaico",
    "pires": "pirar",
    "piretologías": "piretología",
    "piretoterapias": "piretoterapia",
    "piretrinas": "piretrina",
    "pirexias": "pirexia",
    "pirheliometrías": "pirheliometría",
    "pirheliómetros": "pirheliómetro",
    "piridiletilacrilatos": "piridiletilacrilato",
    "piridinas": "piridina",
    "piridoxinas": "piridoxina",
    "piriformes": "piriforme",
    "pirimetaminas": "pirimetamina",
    "pirimidinas": "pirimidina",
    "pirinchos": "pirincho",
    "pirinea": "pirineo",
    "pirineas": "pirineo",
    "pirineos": "pirineo",
    "piripis": "piripi",
    "piritas": "pirita",
    "piritoedros": "piritoedro",
    "piritosa": "piritoso",
    "piritosas": "piritoso",
    "piritosos": "piritoso",
    "pirlas": "pirla",
    "piro": "pirar",
    "pirobolistas": "pirobolista",
    "pirocatecoles": "pirocatecol",
    "piroclastos": "piroclasto",
    "pirocloros": "pirocloro",
    "piroclástica": "piroclástico",
    "piroclásticas": "piroclástico",
    "piroclásticos": "piroclástico",
    "piroelectricidades": "piroelectricidad",
    "piroeléctrica": "piroeléctrico",
    "piroeléctricas": "piroeléctrico",
    "piroeléctricos": "piroeléctrico",
    "pirofilacios": "pirofilacio",
    "pirofilitas": "pirofilita",
    "pirofosfatos": "pirofosfato",
    "pirofosfórica": "pirofosfórico",
    "pirofosfóricas": "pirofosfórico",
    "pirofosfóricos": "pirofosfórico",
    "pirofórica": "pirofórico",
    "pirofóricas": "pirofórico",
    "pirofóricos": "pirofórico",
    "pirogaloles": "pirogalol",
    "pirogenaciones": "pirogenación",
    "pirogenética": "pirogenético",
    "pirogenéticas": "pirogenético",
    "pirogenéticos": "pirogenético",
    "piroglobulinas": "piroglobulina",
    "pirograba": "pirograbar",
    "pirogrababa": "pirograbar",
    "pirogrababais": "pirograbar",
    "pirogrababan": "pirograbar",
    "pirogrababas": "pirograbar",
    "pirograbad": "pirograbar",
    "pirograbada": "pirograbar",
    "pirograbadas": "pirograbar",
    "pirograbado": "pirograbar",
    "pirograbados": "pirograbar",
    "pirograbamos": "pirograbar",
    "pirograban": "pirograbar",
    "pirograbando": "pirograbar",
    "pirograbara": "pirograbar",
    "pirograbarais": "pirograbar",
    "pirograbaran": "pirograbar",
    "pirograbaras": "pirograbar",
    "pirograbare": "pirograbar",
    "pirograbareis": "pirograbar",
    "pirograbaremos": "pirograbar",
    "pirograbaren": "pirograbar",
    "pirograbares": "pirograbar",
    "pirograbaron": "pirograbar",
    "pirograbará": "pirograbar",
    "pirograbarán": "pirograbar",
    "pirograbarás": "pirograbar",
    "pirograbaré": "pirograbar",
    "pirograbaréis": "pirograbar",
    "pirograbaría": "pirograbar",
    "pirograbaríais": "pirograbar",
    "pirograbaríamos": "pirograbar",
    "pirograbarían": "pirograbar",
    "pirograbarías": "pirograbar",
    "pirograbas": "pirograbar",
    "pirograbase": "pirograbar",
    "pirograbaseis": "pirograbar",
    "pirograbasen": "pirograbar",
    "pirograbases": "pirograbar",
    "pirograbaste": "pirograbar",
    "pirograbasteis": "pirograbar",
    "pirograbe": "pirograbar",
    "pirograbemos": "pirograbar",
    "pirograben": "pirograbar",
    "pirograbes": "pirograbar",
    "pirograbo": "pirograbar",
    "pirograbábamos": "pirograbar",
    "pirograbáis": "pirograbar",
    "pirograbáramos": "pirograbar",
    "pirograbáremos": "pirograbar",
    "pirograbásemos": "pirograbar",
    "pirograbé": "pirograbar",
    "pirograbéis": "pirograbar",
    "pirograbó": "pirograbar",
    "pirogálica": "pirogálico",
    "pirogálicas": "pirogálico",
    "pirogálicos": "pirogálico",
    "piroleñosa": "piroleñoso",
    "piroleñosas": "piroleñoso",
    "piroleñosos": "piroleñoso",
    "pirologías": "pirología",
    "pirolusitas": "pirolusita",
    "pirolítica": "pirolítico",
    "pirolíticas": "pirolítico",
    "pirolíticos": "pirolítico",
    "piromagmas": "piromagma",
    "piromagnetismos": "piromagnetismo",
    "piromagnética": "piromagnético",
    "piromagnéticas": "piromagnético",
    "piromagnéticos": "piromagnético",
    "piromancias": "piromancia",
    "piromanías": "piromanía",
    "piromelítica": "piromelítico",
    "piromelíticas": "piromelítico",
    "piromelíticos": "piromelítico",
    "pirometamorfismos": "pirometamorfismo",
    "pirometrías": "pirometría",
    "piromorfitas": "piromorfita",
    "piromántica": "piromántico",
    "pirománticas": "piromántico",
    "pirománticos": "piromántico",
    "pirométrica": "pirométrico",
    "pirométricas": "pirométrico",
    "pirométricos": "pirométrico",
    "pironas": "pirona",
    "piropea": "piropear",
    "piropeaba": "piropear",
    "piropeabais": "piropear",
    "piropeaban": "piropear",
    "piropeabas": "piropear",
    "piropead": "piropear",
    "piropeada": "piropear",
    "piropeadas": "piropear",
    "piropeado": "piropear",
    "piropeados": "piropear",
    "piropeamos": "piropear",
    "piropean": "piropear",
    "piropeando": "piropear",
    "piropeara": "piropear",
    "piropearais": "piropear",
    "piropearan": "piropear",
    "piropearas": "piropear",
    "piropeare": "piropear",
    "piropeareis": "piropear",
    "piropearemos": "piropear",
    "piropearen": "piropear",
    "piropeares": "piropear",
    "piropearon": "piropear",
    "piropeará": "piropear",
    "piropearán": "piropear",
    "piropearás": "piropear",
    "piropearé": "piropear",
    "piropearéis": "piropear",
    "piropearía": "piropear",
    "piropearíais": "piropear",
    "piropearíamos": "piropear",
    "piropearían": "piropear",
    "piropearías": "piropear",
    "piropeas": "piropear",
    "piropease": "piropear",
    "piropeaseis": "piropear",
    "piropeasen": "piropear",
    "piropeases": "piropear",
    "piropeaste": "piropear",
    "piropeasteis": "piropear",
    "piropee": "piropear",
    "piropeemos": "piropear",
    "piropeen": "piropear",
    "piropees": "piropear",
    "piropeo": "piropear",
    "piropeos": "piropeo",
    "piropeábamos": "piropear",
    "piropeáis": "piropear",
    "piropeáramos": "piropear",
    "piropeáremos": "piropear",
    "piropeásemos": "piropear",
    "piropeé": "piropear",
    "piropeéis": "piropear",
    "piropeó": "piropear",
    "piropos": "piropo",
    "piroquímica": "piroquímico",
    "piroquímicas": "piroquímico",
    "piroquímicos": "piroquímico",
    "piroscopios": "piroscopio",
    "pirosferas": "pirosfera",
    "pirosulfúricos": "pirosulfúrico",
    "pirotartárica": "pirotartárico",
    "pirotartáricas": "pirotartárico",
    "pirotartáricos": "pirotartárico",
    "pirotecnias": "pirotecnia",
    "pirotécnica": "pirotécnico",
    "pirotécnicas": "pirotécnico",
    "pirotécnicos": "pirotécnico",
    "piroxenitas": "piroxenita",
    "piroxenos": "piroxeno",
    "piroxilada": "piroxilado",
    "piroxiladas": "piroxilado",
    "piroxilados": "piroxilado",
    "piroxilinas": "piroxilina",
    "pirque": "pircar",
    "pirquemos": "pircar",
    "pirquen": "pircar",
    "pirques": "pircar",
    "pirqué": "pircar",
    "pirquéis": "pircar",
    "pirra": "pirrar",
    "pirraba": "pirrar",
    "pirrabais": "pirrar",
    "pirraban": "pirrar",
    "pirrabas": "pirrar",
    "pirrad": "pirrar",
    "pirrada": "pirrar",
    "pirradas": "pirrar",
    "pirrado": "pirrar",
    "pirrados": "pirrar",
    "pirramos": "pirrar",
    "pirran": "pirrar",
    "pirrando": "pirrar",
    "pirrara": "pirrar",
    "pirrarais": "pirrar",
    "pirraran": "pirrar",
    "pirraras": "pirrar",
    "pirrare": "pirrar",
    "pirrareis": "pirrar",
    "pirraremos": "pirrar",
    "pirraren": "pirrar",
    "pirrares": "pirrar",
    "pirraron": "pirrar",
    "pirrará": "pirrar",
    "pirrarán": "pirrar",
    "pirrarás": "pirrar",
    "pirraré": "pirrar",
    "pirraréis": "pirrar",
    "pirraría": "pirrar",
    "pirraríais": "pirrar",
    "pirraríamos": "pirrar",
    "pirrarían": "pirrar",
    "pirrarías": "pirrar",
    "pirras": "pirrar",
    "pirrase": "pirrar",
    "pirraseis": "pirrar",
    "pirrasen": "pirrar",
    "pirrases": "pirrar",
    "pirraste": "pirrar",
    "pirrasteis": "pirrar",
    "pirre": "pirrar",
    "pirremos": "pirrar",
    "pirren": "pirrar",
    "pirres": "pirrar",
    "pirriquios": "pirriquio",
    "pirro": "pirrar",
    "pirroles": "pirrol",
    "pirrolidonas": "pirrolidona",
    "pirroniana": "pirroniano",
    "pirronianas": "pirroniano",
    "pirronianos": "pirroniano",
    "pirronismos": "pirronismo",
    "pirrotinas": "pirrotina",
    "pirrotitas": "pirrotita",
    "pirrábamos": "pirrar",
    "pirráis": "pirrar",
    "pirráramos": "pirrar",
    "pirráremos": "pirrar",
    "pirrásemos": "pirrar",
    "pirré": "pirrar",
    "pirréis": "pirrar",
    "pirró": "pirrar",
    "pirrófitos": "pirrófito",
    "pirrónica": "pirrónico",
    "pirrónicas": "pirrónico",
    "pirrónicos": "pirrónico",
    "piruetas": "pirueta",
    "piruetea": "piruetear",
    "pirueteaba": "piruetear",
    "pirueteabais": "piruetear",
    "pirueteaban": "piruetear",
    "pirueteabas": "piruetear",
    "piruetead": "piruetear",
    "pirueteada": "piruetear",
    "pirueteadas": "piruetear",
    "pirueteado": "piruetear",
    "pirueteados": "piruetear",
    "pirueteamos": "piruetear",
    "piruetean": "piruetear",
    "pirueteando": "piruetear",
    "pirueteara": "piruetear",
    "piruetearais": "piruetear",
    "piruetearan": "piruetear",
    "piruetearas": "piruetear",
    "pirueteare": "piruetear",
    "pirueteareis": "piruetear",
    "piruetearemos": "piruetear",
    "piruetearen": "piruetear",
    "pirueteares": "piruetear",
    "piruetearon": "piruetear",
    "pirueteará": "piruetear",
    "piruetearán": "piruetear",
    "piruetearás": "piruetear",
    "piruetearé": "piruetear",
    "piruetearéis": "piruetear",
    "piruetearía": "piruetear",
    "piruetearíais": "piruetear",
    "piruetearíamos": "piruetear",
    "piruetearían": "piruetear",
    "piruetearías": "piruetear",
    "pirueteas": "piruetear",
    "piruetease": "piruetear",
    "pirueteaseis": "piruetear",
    "pirueteasen": "piruetear",
    "pirueteases": "piruetear",
    "pirueteaste": "piruetear",
    "pirueteasteis": "piruetear",
    "piruetee": "piruetear",
    "pirueteemos": "piruetear",
    "pirueteen": "piruetear",
    "piruetees": "piruetear",
    "pirueteo": "piruetear",
    "pirueteábamos": "piruetear",
    "pirueteáis": "piruetear",
    "pirueteáramos": "piruetear",
    "pirueteáremos": "piruetear",
    "pirueteásemos": "piruetear",
    "pirueteé": "piruetear",
    "pirueteéis": "piruetear",
    "pirueteó": "piruetear",
    "pirujas": "piruja",
    "piruletas": "piruleta",
    "pirulíes": "pirulí",
    "pirulís": "pirulí",
    "pirábamos": "pirar",
    "piráis": "pirar",
    "pirámides": "pirámide",
    "piráramos": "pirar",
    "piráremos": "pirar",
    "pirásemos": "pirar",
    "pirática": "pirático",
    "piráticas": "pirático",
    "piráticos": "pirático",
    "piré": "pirar",
    "piréis": "pirar",
    "pirética": "pirético",
    "piréticas": "pirético",
    "piréticos": "pirético",
    "pirítica": "pirítico",
    "piríticas": "pirítico",
    "piríticos": "pirítico",
    "piró": "pirar",
    "pirófila": "pirófilo",
    "pirófilas": "pirófilo",
    "pirófilos": "pirófilo",
    "pirófitos": "pirófito",
    "piróforos": "piróforo",
    "pirógena": "pirógeno",
    "pirógenas": "pirógeno",
    "pirógenos": "pirógeno",
    "pirómana": "pirómano",
    "pirómanas": "pirómano",
    "pirómanos": "pirómano",
    "pirómetros": "pirómetro",
    "piróscafos": "piróscafo",
    "piróstatos": "piróstato",
    "piróxilos": "piróxilo",
    "pirúvica": "pirúvico",
    "pirúvicas": "pirúvico",
    "pirúvicos": "pirúvico",
    "pis": "pi",
    "pisa": "pisar",
    "pisaba": "pisar",
    "pisabais": "pisar",
    "pisaban": "pisar",
    "pisabas": "pisar",
    "pisables": "pisable",
    "pisad": "pisar",
    "pisada": "pisar",
    "pisadas": "pisar",
    "pisado": "pisar",
    "pisadora": "pisador",
    "pisadoras": "pisador",
    "pisadores": "pisador",
    "pisados": "pisar",
    "pisaduras": "pisadura",
    "pisamos": "pisar",
    "pisan": "pisar",
    "pisana": "pisano",
    "pisanas": "pisano",
    "pisando": "pisar",
    "pisanos": "pisano",
    "pisara": "pisar",
    "pisarais": "pisar",
    "pisaran": "pisar",
    "pisaras": "pisar",
    "pisare": "pisar",
    "pisareis": "pisar",
    "pisaremos": "pisar",
    "pisaren": "pisar",
    "pisares": "pisar",
    "pisaron": "pisar",
    "pisará": "pisar",
    "pisarán": "pisar",
    "pisarás": "pisar",
    "pisaré": "pisar",
    "pisaréis": "pisar",
    "pisaría": "pisar",
    "pisaríais": "pisar",
    "pisaríamos": "pisar",
    "pisarían": "pisar",
    "pisarías": "pisar",
    "pisas": "pisar",
    "pisase": "pisar",
    "pisaseis": "pisar",
    "pisasen": "pisar",
    "pisases": "pisar",
    "pisasfaltos": "pisasfalto",
    "pisaste": "pisar",
    "pisasteis": "pisar",
    "pisaverdes": "pisaverde",
    "piscatores": "piscator",
    "piscatoria": "piscatorio",
    "piscatorias": "piscatorio",
    "piscatorios": "piscatorio",
    "piscicultora": "piscicultor",
    "piscicultoras": "piscicultor",
    "piscicultores": "piscicultor",
    "pisciculturas": "piscicultura",
    "piscifactorías": "piscifactoría",
    "pisciformes": "pisciforme",
    "piscinas": "piscina",
    "piscos": "pisco",
    "piscícolas": "piscícola",
    "piscívora": "piscívoro",
    "piscívoras": "piscívoro",
    "piscívoros": "piscívoro",
    "pise": "pisar",
    "pisemos": "pisar",
    "pisen": "pisar",
    "pises": "pisar",
    "pisiformes": "pisiforme",
    "pisimetacarpiana": "pisimetacarpiano",
    "pisimetacarpianas": "pisimetacarpiano",
    "pisimetacarpianos": "pisimetacarpiano",
    "piso": "pisar",
    "pisolitas": "pisolita",
    "pisonea": "pisonear",
    "pisoneaba": "pisonear",
    "pisoneabais": "pisonear",
    "pisoneaban": "pisonear",
    "pisoneabas": "pisonear",
    "pisonead": "pisonear",
    "pisoneada": "pisonear",
    "pisoneadas": "pisonear",
    "pisoneado": "pisonear",
    "pisoneados": "pisonear",
    "pisoneamos": "pisonear",
    "pisonean": "pisonear",
    "pisoneando": "pisonear",
    "pisoneara": "pisonear",
    "pisonearais": "pisonear",
    "pisonearan": "pisonear",
    "pisonearas": "pisonear",
    "pisoneare": "pisonear",
    "pisoneareis": "pisonear",
    "pisonearemos": "pisonear",
    "pisonearen": "pisonear",
    "pisoneares": "pisonear",
    "pisonearon": "pisonear",
    "pisoneará": "pisonear",
    "pisonearán": "pisonear",
    "pisonearás": "pisonear",
    "pisonearé": "pisonear",
    "pisonearéis": "pisonear",
    "pisonearía": "pisonear",
    "pisonearíais": "pisonear",
    "pisonearíamos": "pisonear",
    "pisonearían": "pisonear",
    "pisonearías": "pisonear",
    "pisoneas": "pisonear",
    "pisonease": "pisonear",
    "pisoneaseis": "pisonear",
    "pisoneasen": "pisonear",
    "pisoneases": "pisonear",
    "pisoneaste": "pisonear",
    "pisoneasteis": "pisonear",
    "pisonee": "pisonear",
    "pisoneemos": "pisonear",
    "pisoneen": "pisonear",
    "pisonees": "pisonear",
    "pisoneo": "pisonear",
    "pisones": "pisón",
    "pisoneábamos": "pisonear",
    "pisoneáis": "pisonear",
    "pisoneáramos": "pisonear",
    "pisoneáremos": "pisonear",
    "pisoneásemos": "pisonear",
    "pisoneé": "pisonear",
    "pisoneéis": "pisonear",
    "pisoneó": "pisonear",
    "pisos": "piso",
    "pisotadas": "pisotada",
    "pisotea": "pisotear",
    "pisoteaba": "pisotear",
    "pisoteabais": "pisotear",
    "pisoteaban": "pisotear",
    "pisoteabas": "pisotear",
    "pisotead": "pisotear",
    "pisoteada": "pisotear",
    "pisoteadas": "pisotear",
    "pisoteado": "pisotear",
    "pisoteados": "pisotear",
    "pisoteamos": "pisotear",
    "pisotean": "pisotear",
    "pisoteando": "pisotear",
    "pisoteara": "pisotear",
    "pisotearais": "pisotear",
    "pisotearan": "pisotear",
    "pisotearas": "pisotear",
    "pisoteare": "pisotear",
    "pisoteareis": "pisotear",
    "pisotearemos": "pisotear",
    "pisotearen": "pisotear",
    "pisoteares": "pisotear",
    "pisotearon": "pisotear",
    "pisoteará": "pisotear",
    "pisotearán": "pisotear",
    "pisotearás": "pisotear",
    "pisotearé": "pisotear",
    "pisotearéis": "pisotear",
    "pisotearía": "pisotear",
    "pisotearíais": "pisotear",
    "pisotearíamos": "pisotear",
    "pisotearían": "pisotear",
    "pisotearías": "pisotear",
    "pisoteas": "pisotear",
    "pisotease": "pisotear",
    "pisoteaseis": "pisotear",
    "pisoteasen": "pisotear",
    "pisoteases": "pisotear",
    "pisoteaste": "pisotear",
    "pisoteasteis": "pisotear",
    "pisotee": "pisotear",
    "pisoteemos": "pisotear",
    "pisoteen": "pisotear",
    "pisotees": "pisotear",
    "pisoteo": "pisotear",
    "pisoteos": "pisoteo",
    "pisoteábamos": "pisotear",
    "pisoteáis": "pisotear",
    "pisoteáramos": "pisotear",
    "pisoteáremos": "pisotear",
    "pisoteásemos": "pisotear",
    "pisoteé": "pisotear",
    "pisoteéis": "pisotear",
    "pisoteó": "pisotear",
    "pisotones": "pisotón",
    "pispa": "pispar",
    "pispaba": "pispar",
    "pispabais": "pispar",
    "pispaban": "pispar",
    "pispabas": "pispar",
    "pispad": "pispar",
    "pispada": "pispar",
    "pispadas": "pispar",
    "pispado": "pispar",
    "pispados": "pispar",
    "pispajos": "pispajo",
    "pispamos": "pispar",
    "pispan": "pispar",
    "pispando": "pispar",
    "pispara": "pispar",
    "pisparais": "pispar",
    "pisparan": "pispar",
    "pisparas": "pispar",
    "pispare": "pispar",
    "pispareis": "pispar",
    "pisparemos": "pispar",
    "pisparen": "pispar",
    "pispares": "pispar",
    "pisparon": "pispar",
    "pispará": "pispar",
    "pisparán": "pispar",
    "pisparás": "pispar",
    "pisparé": "pispar",
    "pisparéis": "pispar",
    "pisparía": "pispar",
    "pisparíais": "pispar",
    "pisparíamos": "pispar",
    "pisparían": "pispar",
    "pisparías": "pispar",
    "pispas": "pispar",
    "pispase": "pispar",
    "pispaseis": "pispar",
    "pispasen": "pispar",
    "pispases": "pispar",
    "pispaste": "pispar",
    "pispasteis": "pispar",
    "pispe": "pispar",
    "pispemos": "pispar",
    "pispen": "pispar",
    "pispes": "pispar",
    "pispo": "pispar",
    "pispábamos": "pispar",
    "pispáis": "pispar",
    "pispáramos": "pispar",
    "pispáremos": "pispar",
    "pispásemos": "pispar",
    "pispé": "pispar",
    "pispéis": "pispar",
    "pispó": "pispar",
    "pista": "pistar",
    "pistaba": "pistar",
    "pistabais": "pistar",
    "pistaban": "pistar",
    "pistabas": "pistar",
    "pistacheros": "pistachero",
    "pistaches": "pistache",
    "pistachos": "pistacho",
    "pistacitas": "pistacita",
    "pistad": "pistar",
    "pistada": "pistar",
    "pistadas": "pistar",
    "pistaderos": "pistadero",
    "pistado": "pistar",
    "pistados": "pistar",
    "pistamos": "pistar",
    "pistan": "pistar",
    "pistando": "pistar",
    "pistara": "pistar",
    "pistarais": "pistar",
    "pistaran": "pistar",
    "pistaras": "pistar",
    "pistare": "pistar",
    "pistareis": "pistar",
    "pistaremos": "pistar",
    "pistaren": "pistar",
    "pistares": "pistar",
    "pistaron": "pistar",
    "pistará": "pistar",
    "pistarán": "pistar",
    "pistarás": "pistar",
    "pistaré": "pistar",
    "pistaréis": "pistar",
    "pistaría": "pistar",
    "pistaríais": "pistar",
    "pistaríamos": "pistar",
    "pistarían": "pistar",
    "pistarías": "pistar",
    "pistas": "pistar",
    "pistase": "pistar",
    "pistaseis": "pistar",
    "pistasen": "pistar",
    "pistases": "pistar",
    "pistaste": "pistar",
    "pistasteis": "pistar",
    "piste": "pistar",
    "pistemos": "pistar",
    "pisten": "pistar",
    "pistera": "pistero",
    "pisteras": "pistero",
    "pisteros": "pistero",
    "pistes": "pistar",
    "pistilos": "pistilo",
    "pisto": "pistar",
    "pistolas": "pistola",
    "pistolera": "pistolero",
    "pistoleras": "pistolero",
    "pistolerismos": "pistolerismo",
    "pistoleros": "pistolero",
    "pistoletazos": "pistoletazo",
    "pistoletes": "pistolete",
    "pistoloquias": "pistoloquia",
    "pistones": "pistón",
    "pistonuda": "pistonudo",
    "pistonudas": "pistonudo",
    "pistonudos": "pistonudo",
    "pistoresas": "pistoresa",
    "pistos": "pisto",
    "pistrajes": "pistraje",
    "pistraques": "pistraque",
    "pisturas": "pistura",
    "pistábamos": "pistar",
    "pistáis": "pistar",
    "pistáramos": "pistar",
    "pistáremos": "pistar",
    "pistásemos": "pistar",
    "pisté": "pistar",
    "pistéis": "pistar",
    "pistó": "pistar",
    "pisábamos": "pisar",
    "pisáis": "pisar",
    "pisáramos": "pisar",
    "pisáremos": "pisar",
    "pisásemos": "pisar",
    "pisé": "pisar",
    "piséis": "pisar",
    "pisó": "pisar",
    "pita": "pitar",
    "pitaba": "pitar",
    "pitabais": "pitar",
    "pitaban": "pitar",
    "pitabas": "pitar",
    "pitacos": "pitaco",
    "pitad": "pitar",
    "pitada": "pitar",
    "pitadas": "pitar",
    "pitado": "pitar",
    "pitados": "pitar",
    "pitagorismos": "pitagorismo",
    "pitagórica": "pitagórico",
    "pitagóricas": "pitagórico",
    "pitagóricos": "pitagórico",
    "pitamos": "pitar",
    "pitan": "pitar",
    "pitancera": "pitancero",
    "pitanceras": "pitancero",
    "pitanceros": "pitancero",
    "pitancerías": "pitancería",
    "pitando": "pitar",
    "pitanzas": "pitanza",
    "pitara": "pitar",
    "pitarais": "pitar",
    "pitaran": "pitar",
    "pitaras": "pitar",
    "pitare": "pitar",
    "pitareis": "pitar",
    "pitaremos": "pitar",
    "pitaren": "pitar",
    "pitares": "pitar",
    "pitaron": "pitar",
    "pitarras": "pitarra",
    "pitarrosa": "pitarroso",
    "pitarrosas": "pitarroso",
    "pitarrosos": "pitarroso",
    "pitará": "pitar",
    "pitarán": "pitar",
    "pitarás": "pitar",
    "pitaré": "pitar",
    "pitaréis": "pitar",
    "pitaría": "pitar",
    "pitaríais": "pitar",
    "pitaríamos": "pitar",
    "pitarían": "pitar",
    "pitarías": "pitar",
    "pitas": "pitar",
    "pitase": "pitar",
    "pitaseis": "pitar",
    "pitasen": "pitar",
    "pitases": "pitar",
    "pitaste": "pitar",
    "pitasteis": "pitar",
    "pitañas": "pitaña",
    "pite": "pitar",
    "pitecántropos": "pitecántropo",
    "pitemos": "pitar",
    "piten": "pitar",
    "pites": "pitar",
    "piteznas": "pitezna",
    "pitia": "pitio",
    "pitias": "pitio",
    "pitidos": "pitido",
    "pitilleras": "pitillera",
    "pitillos": "pitillo",
    "pitiminíes": "pitiminí",
    "pitiminís": "pitiminí",
    "pitios": "pitio",
    "pitipiés": "pitipié",
    "pito": "pitar",
    "pitoflera": "pitoflero",
    "pitofleras": "pitoflero",
    "pitofleros": "pitoflero",
    "pitoneamientos": "pitoneamiento",
    "pitones": "pitón",
    "pitonisas": "pitonisa",
    "pitorrea": "pitorrear",
    "pitorreaba": "pitorrear",
    "pitorreabais": "pitorrear",
    "pitorreaban": "pitorrear",
    "pitorreabas": "pitorrear",
    "pitorread": "pitorrear",
    "pitorreada": "pitorrear",
    "pitorreadas": "pitorrear",
    "pitorreado": "pitorrear",
    "pitorreados": "pitorrear",
    "pitorreamos": "pitorrear",
    "pitorrean": "pitorrear",
    "pitorreando": "pitorrear",
    "pitorreara": "pitorrear",
    "pitorrearais": "pitorrear",
    "pitorrearan": "pitorrear",
    "pitorrearas": "pitorrear",
    "pitorreare": "pitorrear",
    "pitorreareis": "pitorrear",
    "pitorrearemos": "pitorrear",
    "pitorrearen": "pitorrear",
    "pitorreares": "pitorrear",
    "pitorrearon": "pitorrear",
    "pitorreará": "pitorrear",
    "pitorrearán": "pitorrear",
    "pitorrearás": "pitorrear",
    "pitorrearé": "pitorrear",
    "pitorrearéis": "pitorrear",
    "pitorrearía": "pitorrear",
    "pitorrearíais": "pitorrear",
    "pitorrearíamos": "pitorrear",
    "pitorrearían": "pitorrear",
    "pitorrearías": "pitorrear",
    "pitorreas": "pitorrear",
    "pitorrease": "pitorrear",
    "pitorreaseis": "pitorrear",
    "pitorreasen": "pitorrear",
    "pitorreases": "pitorrear",
    "pitorreaste": "pitorrear",
    "pitorreasteis": "pitorrear",
    "pitorree": "pitorrear",
    "pitorreemos": "pitorrear",
    "pitorreen": "pitorrear",
    "pitorrees": "pitorrear",
    "pitorreo": "pitorrear",
    "pitorreos": "pitorreo",
    "pitorreábamos": "pitorrear",
    "pitorreáis": "pitorrear",
    "pitorreáramos": "pitorrear",
    "pitorreáremos": "pitorrear",
    "pitorreásemos": "pitorrear",
    "pitorreé": "pitorrear",
    "pitorreéis": "pitorrear",
    "pitorreó": "pitorrear",
    "pitorros": "pitorro",
    "pitos": "pito",
    "pitosporácea": "pitosporáceo",
    "pitosporáceas": "pitosporáceo",
    "pitosporáceos": "pitosporáceo",
    "pituicitos": "pituicito",
    "pituitaria": "pituitario",
    "pituitarias": "pituitario",
    "pituitarios": "pituitario",
    "pituitas": "pituita",
    "pituitosa": "pituitoso",
    "pituitosas": "pituitoso",
    "pituitosos": "pituitoso",
    "pitusa": "pituso",
    "pitusas": "pituso",
    "pitusos": "pituso",
    "pitábamos": "pitar",
    "pitáis": "pitar",
    "pitáramos": "pitar",
    "pitáremos": "pitar",
    "pitásemos": "pitar",
    "pité": "pitar",
    "pitéis": "pitar",
    "pitó": "pitar",
    "piujares": "piujar",
    "piula": "piular",
    "piulaba": "piular",
    "piulabais": "piular",
    "piulaban": "piular",
    "piulabas": "piular",
    "piulad": "piular",
    "piulada": "piular",
    "piuladas": "piular",
    "piulado": "piular",
    "piulados": "piular",
    "piulamos": "piular",
    "piulan": "piular",
    "piulando": "piular",
    "piulara": "piular",
    "piularais": "piular",
    "piularan": "piular",
    "piularas": "piular",
    "piulare": "piular",
    "piulareis": "piular",
    "piularemos": "piular",
    "piularen": "piular",
    "piulares": "piular",
    "piularon": "piular",
    "piulará": "piular",
    "piularán": "piular",
    "piularás": "piular",
    "piularé": "piular",
    "piularéis": "piular",
    "piularía": "piular",
    "piularíais": "piular",
    "piularíamos": "piular",
    "piularían": "piular",
    "piularías": "piular",
    "piulas": "piular",
    "piulase": "piular",
    "piulaseis": "piular",
    "piulasen": "piular",
    "piulases": "piular",
    "piulaste": "piular",
    "piulasteis": "piular",
    "piule": "piular",
    "piulemos": "piular",
    "piulen": "piular",
    "piules": "piular",
    "piulidos": "piulido",
    "piulo": "piular",
    "piulábamos": "piular",
    "piuláis": "piular",
    "piuláramos": "piular",
    "piuláremos": "piular",
    "piulásemos": "piular",
    "piulé": "piular",
    "piuléis": "piular",
    "piuló": "piular",
    "piunes": "piune",
    "pivota": "pivotar",
    "pivotaba": "pivotar",
    "pivotabais": "pivotar",
    "pivotaban": "pivotar",
    "pivotabas": "pivotar",
    "pivotad": "pivotar",
    "pivotada": "pivotar",
    "pivotadas": "pivotar",
    "pivotado": "pivotar",
    "pivotados": "pivotar",
    "pivotamos": "pivotar",
    "pivotan": "pivotar",
    "pivotando": "pivotar",
    "pivotantes": "pivotante",
    "pivotara": "pivotar",
    "pivotarais": "pivotar",
    "pivotaran": "pivotar",
    "pivotaras": "pivotar",
    "pivotare": "pivotar",
    "pivotareis": "pivotar",
    "pivotaremos": "pivotar",
    "pivotaren": "pivotar",
    "pivotares": "pivotar",
    "pivotaron": "pivotar",
    "pivotará": "pivotar",
    "pivotarán": "pivotar",
    "pivotarás": "pivotar",
    "pivotaré": "pivotar",
    "pivotaréis": "pivotar",
    "pivotaría": "pivotar",
    "pivotaríais": "pivotar",
    "pivotaríamos": "pivotar",
    "pivotarían": "pivotar",
    "pivotarías": "pivotar",
    "pivotas": "pivotar",
    "pivotase": "pivotar",
    "pivotaseis": "pivotar",
    "pivotasen": "pivotar",
    "pivotases": "pivotar",
    "pivotaste": "pivotar",
    "pivotasteis": "pivotar",
    "pivote": "pivotar",
    "pivotemos": "pivotar",
    "pivoten": "pivotar",
    "pivotes": "pivote",
    "pivoto": "pivotar",
    "pivotábamos": "pivotar",
    "pivotáis": "pivotar",
    "pivotáramos": "pivotar",
    "pivotáremos": "pivotar",
    "pivotásemos": "pivotar",
    "pivoté": "pivotar",
    "pivotéis": "pivotar",
    "pivotó": "pivotar",
    "pixeles": "pixel",
    "pixelses": "pixel",
    "pixidios": "pixidio",
    "piyamas": "piyama",
    "pizarrales": "pizarral",
    "pizarras": "pizarra",
    "pizarrera": "pizarrero",
    "pizarreras": "pizarrero",
    "pizarreros": "pizarrero",
    "pizarrerías": "pizarrería",
    "pizarreña": "pizarreño",
    "pizarreñas": "pizarreño",
    "pizarreños": "pizarreño",
    "pizarrines": "pizarrín",
    "pizarrosa": "pizarroso",
    "pizarrosas": "pizarroso",
    "pizarrosos": "pizarroso",
    "pizca": "pizcar",
    "pizcaba": "pizcar",
    "pizcabais": "pizcar",
    "pizcaban": "pizcar",
    "pizcabas": "pizcar",
    "pizcad": "pizcar",
    "pizcada": "pizcar",
    "pizcadas": "pizcar",
    "pizcado": "pizcar",
    "pizcados": "pizcar",
    "pizcamos": "pizcar",
    "pizcan": "pizcar",
    "pizcando": "pizcar",
    "pizcara": "pizcar",
    "pizcarais": "pizcar",
    "pizcaran": "pizcar",
    "pizcaras": "pizcar",
    "pizcare": "pizcar",
    "pizcareis": "pizcar",
    "pizcaremos": "pizcar",
    "pizcaren": "pizcar",
    "pizcares": "pizcar",
    "pizcaron": "pizcar",
    "pizcará": "pizcar",
    "pizcarán": "pizcar",
    "pizcarás": "pizcar",
    "pizcaré": "pizcar",
    "pizcaréis": "pizcar",
    "pizcaría": "pizcar",
    "pizcaríais": "pizcar",
    "pizcaríamos": "pizcar",
    "pizcarían": "pizcar",
    "pizcarías": "pizcar",
    "pizcas": "pizcar",
    "pizcase": "pizcar",
    "pizcaseis": "pizcar",
    "pizcasen": "pizcar",
    "pizcases": "pizcar",
    "pizcaste": "pizcar",
    "pizcasteis": "pizcar",
    "pizco": "pizcar",
    "pizcos": "pizco",
    "pizcábamos": "pizcar",
    "pizcáis": "pizcar",
    "pizcáramos": "pizcar",
    "pizcáremos": "pizcar",
    "pizcásemos": "pizcar",
    "pizcó": "pizcar",
    "pizmienta": "pizmiento",
    "pizmientas": "pizmiento",
    "pizmientos": "pizmiento",
    "pizpiretas": "pizpireta",
    "pizque": "pizcar",
    "pizquemos": "pizcar",
    "pizquen": "pizcar",
    "pizques": "pizcar",
    "pizqué": "pizcar",
    "pizquéis": "pizcar",
    "pizzas": "pizza",
    "pizzerías": "pizzería",
    "pizzicatos": "pizzicato",
    "piábamos": "piar",
    "piáis": "piar",
    "piáramos": "piar",
    "piáremos": "piar",
    "piásemos": "piar",
    "pié": "piar",
    "piéis": "piar",
    "piélagos": "piélago",
    "piérides": "piéride",
    "piñales": "piñal",
    "piñas": "piña",
    "piñatas": "piñata",
    "piñonatas": "piñonata",
    "piñonates": "piñonate",
    "piñonea": "piñonear",
    "piñoneaba": "piñonear",
    "piñoneabais": "piñonear",
    "piñoneaban": "piñonear",
    "piñoneabas": "piñonear",
    "piñonead": "piñonear",
    "piñoneada": "piñonear",
    "piñoneadas": "piñonear",
    "piñoneado": "piñonear",
    "piñoneados": "piñonear",
    "piñoneamos": "piñonear",
    "piñonean": "piñonear",
    "piñoneando": "piñonear",
    "piñoneara": "piñonear",
    "piñonearais": "piñonear",
    "piñonearan": "piñonear",
    "piñonearas": "piñonear",
    "piñoneare": "piñonear",
    "piñoneareis": "piñonear",
    "piñonearemos": "piñonear",
    "piñonearen": "piñonear",
    "piñoneares": "piñonear",
    "piñonearon": "piñonear",
    "piñoneará": "piñonear",
    "piñonearán": "piñonear",
    "piñonearás": "piñonear",
    "piñonearé": "piñonear",
    "piñonearéis": "piñonear",
    "piñonearía": "piñonear",
    "piñonearíais": "piñonear",
    "piñonearíamos": "piñonear",
    "piñonearían": "piñonear",
    "piñonearías": "piñonear",
    "piñoneas": "piñonear",
    "piñonease": "piñonear",
    "piñoneaseis": "piñonear",
    "piñoneasen": "piñonear",
    "piñoneases": "piñonear",
    "piñoneaste": "piñonear",
    "piñoneasteis": "piñonear",
    "piñonee": "piñonear",
    "piñoneemos": "piñonear",
    "piñoneen": "piñonear",
    "piñonees": "piñonear",
    "piñoneo": "piñonear",
    "piñoneos": "piñoneo",
    "piñonera": "piñonero",
    "piñoneras": "piñonero",
    "piñoneros": "piñonero",
    "piñones": "piñón",
    "piñoneábamos": "piñonear",
    "piñoneáis": "piñonear",
    "piñoneáramos": "piñonear",
    "piñoneáremos": "piñonear",
    "piñoneásemos": "piñonear",
    "piñoneé": "piñonear",
    "piñoneéis": "piñonear",
    "piñoneó": "piñonear",
    "piñuelas": "piñuela",
    "piñuelos": "piñuelo",
    "pió": "piar",
    "piógena": "piógeno",
    "piógenas": "piógeno",
    "piógenos": "piógeno",
    "placados": "placado",
    "placajes": "placaje",
    "placas": "placa",
    "place": "placer",
    "placea": "placear",
    "placeaba": "placear",
    "placeabais": "placear",
    "placeaban": "placear",
    "placeabas": "placear",
    "placead": "placear",
    "placeada": "placear",
    "placeadas": "placear",
    "placeado": "placear",
    "placeados": "placear",
    "placeamos": "placear",
    "placean": "placear",
    "placeando": "placear",
    "placeara": "placear",
    "placearais": "placear",
    "placearan": "placear",
    "placearas": "placear",
    "placeare": "placear",
    "placeareis": "placear",
    "placearemos": "placear",
    "placearen": "placear",
    "placeares": "placear",
    "placearon": "placear",
    "placeará": "placear",
    "placearán": "placear",
    "placearás": "placear",
    "placearé": "placear",
    "placearéis": "placear",
    "placearía": "placear",
    "placearíais": "placear",
    "placearíamos": "placear",
    "placearían": "placear",
    "placearías": "placear",
    "placeas": "placear",
    "placease": "placear",
    "placeaseis": "placear",
    "placeasen": "placear",
    "placeases": "placear",
    "placeaste": "placear",
    "placeasteis": "placear",
    "placebos": "placebo",
    "placed": "placer",
    "placee": "placear",
    "placeemos": "placear",
    "placeen": "placear",
    "placees": "placear",
    "placemos": "placer",
    "placen": "placer",
    "placentaciones": "placentación",
    "placentaria": "placentario",
    "placentarias": "placentario",
    "placentarios": "placentario",
    "placentas": "placenta",
    "placentera": "placentero",
    "placenteras": "placentero",
    "placenteros": "placentero",
    "placeo": "placear",
    "placera": "placero",
    "placeras": "placero",
    "placeremos": "placer",
    "placeres": "placer",
    "placeros": "placero",
    "placerá": "placer",
    "placerán": "placer",
    "placerás": "placer",
    "placeré": "placer",
    "placeréis": "placer",
    "placería": "placer",
    "placeríais": "placer",
    "placeríamos": "placer",
    "placerían": "placer",
    "placerías": "placer",
    "places": "placer",
    "placetas": "placeta",
    "placeábamos": "placear",
    "placeáis": "placear",
    "placeáramos": "placear",
    "placeáremos": "placear",
    "placeásemos": "placear",
    "placeé": "placear",
    "placeéis": "placear",
    "placeó": "placear",
    "placibilidades": "placibilidad",
    "placibles": "placible",
    "placida": "placer",
    "placidas": "placer",
    "placideces": "placidez",
    "placido": "placer",
    "placidos": "placer",
    "placiendo": "placer",
    "placiera": "placer",
    "placierais": "placer",
    "placieran": "placer",
    "placieras": "placer",
    "placiere": "placer",
    "placiereis": "placer",
    "placieren": "placer",
    "placieres": "placer",
    "placieron": "placer",
    "placiese": "placer",
    "placieseis": "placer",
    "placiesen": "placer",
    "placieses": "placer",
    "placimos": "placer",
    "placiste": "placer",
    "placisteis": "placer",
    "placiéramos": "placer",
    "placiéremos": "placer",
    "placiésemos": "placer",
    "plació": "placer",
    "placoides": "placoide",
    "placéis": "placer",
    "plací": "placer",
    "placía": "placer",
    "placíais": "placer",
    "placíamos": "placer",
    "placían": "placer",
    "placías": "placer",
    "plafones": "plafón",
    "plaga": "plagar",
    "plagaba": "plagar",
    "plagabais": "plagar",
    "plagaban": "plagar",
    "plagabas": "plagar",
    "plagad": "plagar",
    "plagada": "plagar",
    "plagadas": "plagar",
    "plagado": "plagar",
    "plagados": "plagar",
    "plagamos": "plagar",
    "plagan": "plagar",
    "plagando": "plagar",
    "plagara": "plagar",
    "plagarais": "plagar",
    "plagaran": "plagar",
    "plagaras": "plagar",
    "plagare": "plagar",
    "plagareis": "plagar",
    "plagaremos": "plagar",
    "plagaren": "plagar",
    "plagares": "plagar",
    "plagaron": "plagar",
    "plagará": "plagar",
    "plagarán": "plagar",
    "plagarás": "plagar",
    "plagaré": "plagar",
    "plagaréis": "plagar",
    "plagaría": "plagar",
    "plagaríais": "plagar",
    "plagaríamos": "plagar",
    "plagarían": "plagar",
    "plagarías": "plagar",
    "plagas": "plagar",
    "plagase": "plagar",
    "plagaseis": "plagar",
    "plagasen": "plagar",
    "plagases": "plagar",
    "plagaste": "plagar",
    "plagasteis": "plagar",
    "plagia": "plagiar",
    "plagiaba": "plagiar",
    "plagiabais": "plagiar",
    "plagiaban": "plagiar",
    "plagiabas": "plagiar",
    "plagiad": "plagiar",
    "plagiada": "plagiar",
    "plagiadas": "plagiar",
    "plagiado": "plagiar",
    "plagiados": "plagiar",
    "plagiamos": "plagiar",
    "plagian": "plagiar",
    "plagiando": "plagiar",
    "plagiara": "plagiar",
    "plagiarais": "plagiar",
    "plagiaran": "plagiar",
    "plagiaras": "plagiar",
    "plagiare": "plagiar",
    "plagiareis": "plagiar",
    "plagiaremos": "plagiar",
    "plagiaren": "plagiar",
    "plagiares": "plagiar",
    "plagiaria": "plagiario",
    "plagiarias": "plagiario",
    "plagiarios": "plagiario",
    "plagiaron": "plagiar",
    "plagiará": "plagiar",
    "plagiarán": "plagiar",
    "plagiarás": "plagiar",
    "plagiaré": "plagiar",
    "plagiaréis": "plagiar",
    "plagiaría": "plagiar",
    "plagiaríais": "plagiar",
    "plagiaríamos": "plagiar",
    "plagiarían": "plagiar",
    "plagiarías": "plagiar",
    "plagias": "plagiar",
    "plagiase": "plagiar",
    "plagiaseis": "plagiar",
    "plagiasen": "plagiar",
    "plagiases": "plagiar",
    "plagiaste": "plagiar",
    "plagiasteis": "plagiar",
    "plagie": "plagiar",
    "plagiemos": "plagiar",
    "plagien": "plagiar",
    "plagies": "plagiar",
    "plagio": "plagiar",
    "plagioclasas": "plagioclasa",
    "plagios": "plagio",
    "plagiábamos": "plagiar",
    "plagiáis": "plagiar",
    "plagiáramos": "plagiar",
    "plagiáremos": "plagiar",
    "plagiásemos": "plagiar",
    "plagié": "plagiar",
    "plagiéis": "plagiar",
    "plagió": "plagiar",
    "plago": "plagar",
    "plague": "plagar",
    "plaguemos": "plagar",
    "plaguen": "plagar",
    "plagues": "plagar",
    "plaguicidas": "plaguicida",
    "plagué": "plagar",
    "plaguéis": "plagar",
    "plagábamos": "plagar",
    "plagáis": "plagar",
    "plagáramos": "plagar",
    "plagáremos": "plagar",
    "plagásemos": "plagar",
    "plagó": "plagar",
    "plana": "plano",
    "planadas": "planada",
    "planadora": "planador",
    "planadoras": "planador",
    "planadores": "planador",
    "planares": "planar",
    "planas": "plano",
    "plancha": "planchar",
    "planchaba": "planchar",
    "planchabais": "planchar",
    "planchaban": "planchar",
    "planchabas": "planchar",
    "planchad": "planchar",
    "planchada": "planchar",
    "planchadas": "planchar",
    "planchado": "planchar",
    "planchadora": "planchador",
    "planchadoras": "planchador",
    "planchadores": "planchador",
    "planchados": "planchar",
    "planchamos": "planchar",
    "planchan": "planchar",
    "planchando": "planchar",
    "planchara": "planchar",
    "plancharais": "planchar",
    "plancharan": "planchar",
    "plancharas": "planchar",
    "planchare": "planchar",
    "planchareis": "planchar",
    "plancharemos": "planchar",
    "plancharen": "planchar",
    "planchares": "planchar",
    "plancharon": "planchar",
    "planchará": "planchar",
    "plancharán": "planchar",
    "plancharás": "planchar",
    "plancharé": "planchar",
    "plancharéis": "planchar",
    "plancharía": "planchar",
    "plancharíais": "planchar",
    "plancharíamos": "planchar",
    "plancharían": "planchar",
    "plancharías": "planchar",
    "planchas": "planchar",
    "planchase": "planchar",
    "planchaseis": "planchar",
    "planchasen": "planchar",
    "planchases": "planchar",
    "planchaste": "planchar",
    "planchasteis": "planchar",
    "planche": "planchar",
    "planchemos": "planchar",
    "planchen": "planchar",
    "planches": "planchar",
    "planchetas": "plancheta",
    "planchistas": "planchista",
    "planchisterías": "planchistería",
    "plancho": "planchar",
    "planchábamos": "planchar",
    "plancháis": "planchar",
    "plancháramos": "planchar",
    "plancháremos": "planchar",
    "planchásemos": "planchar",
    "planché": "planchar",
    "planchéis": "planchar",
    "planchó": "planchar",
    "planctófaga": "planctófago",
    "planctófagas": "planctófago",
    "planctófagos": "planctófago",
    "planctónica": "planctónico",
    "planctónicas": "planctónico",
    "planctónicos": "planctónico",
    "planea": "planear",
    "planeaba": "planear",
    "planeabais": "planear",
    "planeaban": "planear",
    "planeabas": "planear",
    "planeaciones": "planeación",
    "planead": "planear",
    "planeada": "planear",
    "planeadas": "planear",
    "planeado": "planear",
    "planeadora": "planeador",
    "planeadoras": "planeador",
    "planeadores": "planeador",
    "planeados": "planear",
    "planeamientos": "planeamiento",
    "planeamos": "planear",
    "planean": "planear",
    "planeando": "planear",
    "planeara": "planear",
    "planearais": "planear",
    "planearan": "planear",
    "planearas": "planear",
    "planeare": "planear",
    "planeareis": "planear",
    "planearemos": "planear",
    "planearen": "planear",
    "planeares": "planear",
    "planearon": "planear",
    "planeará": "planear",
    "planearán": "planear",
    "planearás": "planear",
    "planearé": "planear",
    "planearéis": "planear",
    "planearía": "planear",
    "planearíais": "planear",
    "planearíamos": "planear",
    "planearían": "planear",
    "planearías": "planear",
    "planeas": "planear",
    "planease": "planear",
    "planeaseis": "planear",
    "planeasen": "planear",
    "planeases": "planear",
    "planeaste": "planear",
    "planeasteis": "planear",
    "planee": "planear",
    "planeemos": "planear",
    "planeen": "planear",
    "planees": "planear",
    "planeidades": "planeidad",
    "planeo": "planear",
    "planeos": "planeo",
    "planes": "plan",
    "planetaria": "planetario",
    "planetarias": "planetario",
    "planetarios": "planetario",
    "planetariums": "planetarium",
    "planetas": "planeta",
    "planetoides": "planetoide",
    "planetologías": "planetología",
    "planetáriums": "planetárium",
    "planetícolas": "planetícola",
    "planeábamos": "planear",
    "planeáis": "planear",
    "planeáramos": "planear",
    "planeáremos": "planear",
    "planeásemos": "planear",
    "planeé": "planear",
    "planeéis": "planear",
    "planeó": "planear",
    "planicies": "planicie",
    "planifica": "planificar",
    "planificaba": "planificar",
    "planificabais": "planificar",
    "planificaban": "planificar",
    "planificabas": "planificar",
    "planificaciones": "planificación",
    "planificad": "planificar",
    "planificada": "planificar",
    "planificadas": "planificar",
    "planificado": "planificar",
    "planificadora": "planificador",
    "planificadoras": "planificador",
    "planificadores": "planificador",
    "planificados": "planificar",
    "planificamos": "planificar",
    "planifican": "planificar",
    "planificando": "planificar",
    "planificara": "planificar",
    "planificarais": "planificar",
    "planificaran": "planificar",
    "planificaras": "planificar",
    "planificare": "planificar",
    "planificareis": "planificar",
    "planificaremos": "planificar",
    "planificaren": "planificar",
    "planificares": "planificar",
    "planificaron": "planificar",
    "planificará": "planificar",
    "planificarán": "planificar",
    "planificarás": "planificar",
    "planificaré": "planificar",
    "planificaréis": "planificar",
    "planificaría": "planificar",
    "planificaríais": "planificar",
    "planificaríamos": "planificar",
    "planificarían": "planificar",
    "planificarías": "planificar",
    "planificas": "planificar",
    "planificase": "planificar",
    "planificaseis": "planificar",
    "planificasen": "planificar",
    "planificases": "planificar",
    "planificaste": "planificar",
    "planificasteis": "planificar",
    "planifico": "planificar",
    "planificábamos": "planificar",
    "planificáis": "planificar",
    "planificáramos": "planificar",
    "planificáremos": "planificar",
    "planificásemos": "planificar",
    "planificó": "planificar",
    "planifique": "planificar",
    "planifiquemos": "planificar",
    "planifiquen": "planificar",
    "planifiques": "planificar",
    "planifiqué": "planificar",
    "planifiquéis": "planificar",
    "planifolios": "planifolio",
    "planillas": "planilla",
    "planimetrías": "planimetría",
    "planimétrica": "planimétrico",
    "planimétricas": "planimétrico",
    "planimétricos": "planimétrico",
    "planisferios": "planisferio",
    "plannings": "planning",
    "planoconvexa": "planoconvexo",
    "planoconvexas": "planoconvexo",
    "planoconvexos": "planoconvexo",
    "planocóncava": "planocóncavo",
    "planocóncavas": "planocóncavo",
    "planocóncavos": "planocóncavo",
    "planogametos": "planogameto",
    "planos": "plano",
    "planosoles": "planosol",
    "planta": "plantar",
    "plantaba": "plantar",
    "plantabais": "plantar",
    "plantaban": "plantar",
    "plantabas": "plantar",
    "plantaciones": "plantación",
    "plantad": "plantar",
    "plantada": "plantar",
    "plantadas": "plantar",
    "plantado": "plantar",
    "plantadora": "plantador",
    "plantadoras": "plantadora",
    "plantadores": "plantador",
    "plantados": "plantar",
    "plantaginales": "plantaginal",
    "plantaginácea": "plantagináceo",
    "plantagináceas": "plantagináceo",
    "plantagináceos": "plantagináceo",
    "plantajes": "plantaje",
    "plantamos": "plantar",
    "plantan": "plantar",
    "plantando": "plantar",
    "plantara": "plantar",
    "plantarais": "plantar",
    "plantaran": "plantar",
    "plantaras": "plantar",
    "plantare": "plantar",
    "plantareis": "plantar",
    "plantaremos": "plantar",
    "plantaren": "plantar",
    "plantares": "plantar",
    "plantarios": "plantario",
    "plantaron": "plantar",
    "plantará": "plantar",
    "plantarán": "plantar",
    "plantarás": "plantar",
    "plantaré": "plantar",
    "plantaréis": "plantar",
    "plantaría": "plantar",
    "plantaríais": "plantar",
    "plantaríamos": "plantar",
    "plantarían": "plantar",
    "plantarías": "plantar",
    "plantas": "plantar",
    "plantase": "plantar",
    "plantaseis": "plantar",
    "plantasen": "plantar",
    "plantases": "plantar",
    "plantaste": "plantar",
    "plantasteis": "plantar",
    "plante": "plantar",
    "plantea": "plantear",
    "planteaba": "plantear",
    "planteabais": "plantear",
    "planteaban": "plantear",
    "planteabas": "plantear",
    "plantead": "plantear",
    "planteada": "plantear",
    "planteadas": "plantear",
    "planteado": "plantear",
    "planteados": "plantear",
    "planteamientos": "planteamiento",
    "planteamos": "plantear",
    "plantean": "plantear",
    "planteando": "plantear",
    "planteara": "plantear",
    "plantearais": "plantear",
    "plantearan": "plantear",
    "plantearas": "plantear",
    "planteare": "plantear",
    "planteareis": "plantear",
    "plantearemos": "plantear",
    "plantearen": "plantear",
    "planteares": "plantear",
    "plantearon": "plantear",
    "planteará": "plantear",
    "plantearán": "plantear",
    "plantearás": "plantear",
    "plantearé": "plantear",
    "plantearéis": "plantear",
    "plantearía": "plantear",
    "plantearíais": "plantear",
    "plantearíamos": "plantear",
    "plantearían": "plantear",
    "plantearías": "plantear",
    "planteas": "plantear",
    "plantease": "plantear",
    "planteaseis": "plantear",
    "planteasen": "plantear",
    "planteases": "plantear",
    "planteaste": "plantear",
    "planteasteis": "plantear",
    "plantee": "plantear",
    "planteemos": "plantear",
    "planteen": "plantear",
    "plantees": "plantear",
    "planteles": "plantel",
    "plantemos": "plantar",
    "planten": "plantar",
    "planteo": "plantear",
    "plantes": "plante",
    "planteábamos": "plantear",
    "planteáis": "plantear",
    "planteáramos": "plantear",
    "planteáremos": "plantear",
    "planteásemos": "plantear",
    "planteé": "plantear",
    "planteéis": "plantear",
    "planteó": "plantear",
    "plantifica": "plantificar",
    "plantificaba": "plantificar",
    "plantificabais": "plantificar",
    "plantificaban": "plantificar",
    "plantificabas": "plantificar",
    "plantificad": "plantificar",
    "plantificada": "plantificar",
    "plantificadas": "plantificar",
    "plantificado": "plantificar",
    "plantificados": "plantificar",
    "plantificamos": "plantificar",
    "plantifican": "plantificar",
    "plantificando": "plantificar",
    "plantificara": "plantificar",
    "plantificarais": "plantificar",
    "plantificaran": "plantificar",
    "plantificaras": "plantificar",
    "plantificare": "plantificar",
    "plantificareis": "plantificar",
    "plantificaremos": "plantificar",
    "plantificaren": "plantificar",
    "plantificares": "plantificar",
    "plantificaron": "plantificar",
    "plantificará": "plantificar",
    "plantificarán": "plantificar",
    "plantificarás": "plantificar",
    "plantificaré": "plantificar",
    "plantificaréis": "plantificar",
    "plantificaría": "plantificar",
    "plantificaríais": "plantificar",
    "plantificaríamos": "plantificar",
    "plantificarían": "plantificar",
    "plantificarías": "plantificar",
    "plantificas": "plantificar",
    "plantificase": "plantificar",
    "plantificaseis": "plantificar",
    "plantificasen": "plantificar",
    "plantificases": "plantificar",
    "plantificaste": "plantificar",
    "plantificasteis": "plantificar",
    "plantifico": "plantificar",
    "plantificábamos": "plantificar",
    "plantificáis": "plantificar",
    "plantificáramos": "plantificar",
    "plantificáremos": "plantificar",
    "plantificásemos": "plantificar",
    "plantificó": "plantificar",
    "plantifique": "plantificar",
    "plantifiquemos": "plantificar",
    "plantifiquen": "plantificar",
    "plantifiques": "plantificar",
    "plantifiqué": "plantificar",
    "plantifiquéis": "plantificar",
    "plantillas": "plantilla",
    "plantistas": "plantista",
    "planto": "plantar",
    "plantoneras": "plantonera",
    "plantones": "plantón",
    "plantos": "planto",
    "plantábamos": "plantar",
    "plantáis": "plantar",
    "plantáramos": "plantar",
    "plantáremos": "plantar",
    "plantásemos": "plantar",
    "planté": "plantar",
    "plantéis": "plantar",
    "plantía": "plantío",
    "plantías": "plantío",
    "plantígrada": "plantígrado",
    "plantígradas": "plantígrado",
    "plantígrados": "plantígrado",
    "plantíos": "plantío",
    "plantó": "plantar",
    "planuda": "planudo",
    "planudas": "planudo",
    "planudos": "planudo",
    "planículas": "planícula",
    "planímetros": "planímetro",
    "plaqueados": "plaqueado",
    "plaquetaria": "plaquetario",
    "plaquetarias": "plaquetario",
    "plaquetarios": "plaquetario",
    "plaquetas": "plaqueta",
    "plaquines": "plaquín",
    "plaquitas": "plaquita",
    "plaqués": "plaqué",
    "plasenciana": "plasenciano",
    "plasencianas": "plasenciano",
    "plasencianos": "plasenciano",
    "plasma": "plasmar",
    "plasmaba": "plasmar",
    "plasmabais": "plasmar",
    "plasmaban": "plasmar",
    "plasmabas": "plasmar",
    "plasmaciones": "plasmación",
    "plasmacitos": "plasmacito",
    "plasmacromatinas": "plasmacromatina",
    "plasmad": "plasmar",
    "plasmada": "plasmar",
    "plasmadas": "plasmar",
    "plasmado": "plasmar",
    "plasmados": "plasmar",
    "plasmalemas": "plasmalema",
    "plasmamos": "plasmar",
    "plasman": "plasmar",
    "plasmando": "plasmar",
    "plasmara": "plasmar",
    "plasmarais": "plasmar",
    "plasmaran": "plasmar",
    "plasmaras": "plasmar",
    "plasmare": "plasmar",
    "plasmareis": "plasmar",
    "plasmaremos": "plasmar",
    "plasmaren": "plasmar",
    "plasmares": "plasmar",
    "plasmaron": "plasmar",
    "plasmará": "plasmar",
    "plasmarán": "plasmar",
    "plasmarás": "plasmar",
    "plasmaré": "plasmar",
    "plasmaréis": "plasmar",
    "plasmaría": "plasmar",
    "plasmaríais": "plasmar",
    "plasmaríamos": "plasmar",
    "plasmarían": "plasmar",
    "plasmarías": "plasmar",
    "plasmas": "plasmar",
    "plasmase": "plasmar",
    "plasmaseis": "plasmar",
    "plasmasen": "plasmar",
    "plasmases": "plasmar",
    "plasmasomas": "plasmasoma",
    "plasmaste": "plasmar",
    "plasmasteis": "plasmar",
    "plasmaterapias": "plasmaterapia",
    "plasme": "plasmar",
    "plasmemos": "plasmar",
    "plasmen": "plasmar",
    "plasmes": "plasmar",
    "plasmidios": "plasmidio",
    "plasminas": "plasmina",
    "plasminógenos": "plasminógeno",
    "plasmo": "plasmar",
    "plasmocitomas": "plasmocitoma",
    "plasmocitos": "plasmocito",
    "plasmodesmas": "plasmodesma",
    "plasmodios": "plasmodio",
    "plasmones": "plasmón",
    "plasmosomas": "plasmosoma",
    "plasmábamos": "plasmar",
    "plasmágenes": "plasmagen",
    "plasmáis": "plasmar",
    "plasmáramos": "plasmar",
    "plasmáremos": "plasmar",
    "plasmásemos": "plasmar",
    "plasmática": "plasmático",
    "plasmáticas": "plasmático",
    "plasmáticos": "plasmático",
    "plasmé": "plasmar",
    "plasméis": "plasmar",
    "plasmó": "plasmar",
    "plastas": "plasta",
    "plastecidos": "plastecido",
    "plastes": "plaste",
    "plastias": "plastia",
    "plasticidades": "plasticidad",
    "plastidiales": "plastidial",
    "plastidios": "plastidio",
    "plastifica": "plastificar",
    "plastificaba": "plastificar",
    "plastificabais": "plastificar",
    "plastificaban": "plastificar",
    "plastificabas": "plastificar",
    "plastificaciones": "plastificación",
    "plastificad": "plastificar",
    "plastificada": "plastificar",
    "plastificadas": "plastificar",
    "plastificado": "plastificar",
    "plastificados": "plastificar",
    "plastificamos": "plastificar",
    "plastifican": "plastificar",
    "plastificando": "plastificar",
    "plastificantes": "plastificante",
    "plastificara": "plastificar",
    "plastificarais": "plastificar",
    "plastificaran": "plastificar",
    "plastificaras": "plastificar",
    "plastificare": "plastificar",
    "plastificareis": "plastificar",
    "plastificaremos": "plastificar",
    "plastificaren": "plastificar",
    "plastificares": "plastificar",
    "plastificaron": "plastificar",
    "plastificará": "plastificar",
    "plastificarán": "plastificar",
    "plastificarás": "plastificar",
    "plastificaré": "plastificar",
    "plastificaréis": "plastificar",
    "plastificaría": "plastificar",
    "plastificaríais": "plastificar",
    "plastificaríamos": "plastificar",
    "plastificarían": "plastificar",
    "plastificarías": "plastificar",
    "plastificas": "plastificar",
    "plastificase": "plastificar",
    "plastificaseis": "plastificar",
    "plastificasen": "plastificar",
    "plastificases": "plastificar",
    "plastificaste": "plastificar",
    "plastificasteis": "plastificar",
    "plastifico": "plastificar",
    "plastificábamos": "plastificar",
    "plastificáis": "plastificar",
    "plastificáramos": "plastificar",
    "plastificáremos": "plastificar",
    "plastificásemos": "plastificar",
    "plastificó": "plastificar",
    "plastifique": "plastificar",
    "plastifiquemos": "plastificar",
    "plastifiquen": "plastificar",
    "plastifiques": "plastificar",
    "plastifiqué": "plastificar",
    "plastifiquéis": "plastificar",
    "plastigeles": "plastigel",
    "plastisoles": "plastisol",
    "plastomas": "plastoma",
    "plastorribosomas": "plastorribosoma",
    "plastos": "plasto",
    "plastosomas": "plastosoma",
    "plastrones": "plastrón",
    "plasturgias": "plasturgia",
    "plastómeros": "plastómero",
    "plastómetros": "plastómetro",
    "platabandas": "platabanda",
    "plataformas": "plataforma",
    "platales": "platal",
    "platanales": "platanal",
    "platanares": "platanar",
    "platanera": "platanero",
    "plataneras": "platanero",
    "plataneros": "platanero",
    "platanillos": "platanillo",
    "platanácea": "platanáceo",
    "platanáceas": "platanáceo",
    "platanáceos": "platanáceo",
    "platas": "plata",
    "platea": "platear",
    "plateaba": "platear",
    "plateabais": "platear",
    "plateaban": "platear",
    "plateabas": "platear",
    "platead": "platear",
    "plateada": "platear",
    "plateadas": "platear",
    "plateado": "platear",
    "plateadora": "plateador",
    "plateadoras": "plateador",
    "plateadores": "plateador",
    "plateados": "platear",
    "plateaduras": "plateadura",
    "plateamos": "platear",
    "platean": "platear",
    "plateando": "platear",
    "plateara": "platear",
    "platearais": "platear",
    "platearan": "platear",
    "platearas": "platear",
    "plateare": "platear",
    "plateareis": "platear",
    "platearemos": "platear",
    "platearen": "platear",
    "plateares": "platear",
    "platearon": "platear",
    "plateará": "platear",
    "platearán": "platear",
    "platearás": "platear",
    "platearé": "platear",
    "platearéis": "platear",
    "platearía": "platear",
    "platearíais": "platear",
    "platearíamos": "platear",
    "platearían": "platear",
    "platearías": "platear",
    "plateas": "platear",
    "platease": "platear",
    "plateaseis": "platear",
    "plateasen": "platear",
    "plateases": "platear",
    "plateaste": "platear",
    "plateasteis": "platear",
    "platee": "platear",
    "plateemos": "platear",
    "plateen": "platear",
    "platees": "platear",
    "plateles": "platel",
    "platelmintos": "platelminto",
    "platenses": "platense",
    "plateo": "platear",
    "platera": "platero",
    "plateras": "platero",
    "plateresca": "plateresco",
    "platerescas": "plateresco",
    "platerescos": "plateresco",
    "plateros": "platero",
    "platerías": "platería",
    "plateábamos": "platear",
    "plateáis": "platear",
    "plateáramos": "platear",
    "plateáremos": "platear",
    "plateásemos": "platear",
    "plateé": "platear",
    "plateéis": "platear",
    "plateó": "platear",
    "platica": "platicar",
    "platicaba": "platicar",
    "platicabais": "platicar",
    "platicaban": "platicar",
    "platicabas": "platicar",
    "platicad": "platicar",
    "platicada": "platicar",
    "platicadas": "platicar",
    "platicado": "platicar",
    "platicadora": "platicador",
    "platicadoras": "platicador",
    "platicadores": "platicador",
    "platicados": "platicar",
    "platicamos": "platicar",
    "platican": "platicar",
    "platicando": "platicar",
    "platicara": "platicar",
    "platicarais": "platicar",
    "platicaran": "platicar",
    "platicaras": "platicar",
    "platicare": "platicar",
    "platicareis": "platicar",
    "platicaremos": "platicar",
    "platicaren": "platicar",
    "platicares": "platicar",
    "platicaron": "platicar",
    "platicará": "platicar",
    "platicarán": "platicar",
    "platicarás": "platicar",
    "platicaré": "platicar",
    "platicaréis": "platicar",
    "platicaría": "platicar",
    "platicaríais": "platicar",
    "platicaríamos": "platicar",
    "platicarían": "platicar",
    "platicarías": "platicar",
    "platicas": "platicar",
    "platicase": "platicar",
    "platicaseis": "platicar",
    "platicasen": "platicar",
    "platicases": "platicar",
    "platicaste": "platicar",
    "platicasteis": "platicar",
    "platicefalias": "platicefalia",
    "platico": "platicar",
    "platicultivos": "platicultivo",
    "platicábamos": "platicar",
    "platicáis": "platicar",
    "platicáramos": "platicar",
    "platicáremos": "platicar",
    "platicásemos": "platicar",
    "platicéfala": "platicéfalo",
    "platicéfalas": "platicéfalo",
    "platicéfalos": "platicéfalo",
    "platicó": "platicar",
    "platicúrtica": "platicúrtico",
    "platicúrticas": "platicúrtico",
    "platicúrticos": "platicúrtico",
    "platifónica": "platifónico",
    "platifónicas": "platifónico",
    "platifónicos": "platifónico",
    "platiglosa": "platigloso",
    "platiglosas": "platigloso",
    "platiglosos": "platigloso",
    "platijas": "platija",
    "platillos": "platillo",
    "platimérica": "platimérico",
    "platiméricas": "platimérico",
    "platiméricos": "platimérico",
    "platina": "platinar",
    "platinaba": "platinar",
    "platinabais": "platinar",
    "platinaban": "platinar",
    "platinabas": "platinar",
    "platinad": "platinar",
    "platinada": "platinar",
    "platinadas": "platinar",
    "platinado": "platinar",
    "platinados": "platinar",
    "platinamos": "platinar",
    "platinan": "platinar",
    "platinando": "platinar",
    "platinara": "platinar",
    "platinarais": "platinar",
    "platinaran": "platinar",
    "platinaras": "platinar",
    "platinare": "platinar",
    "platinareis": "platinar",
    "platinaremos": "platinar",
    "platinaren": "platinar",
    "platinares": "platinar",
    "platinaron": "platinar",
    "platinará": "platinar",
    "platinarán": "platinar",
    "platinarás": "platinar",
    "platinaré": "platinar",
    "platinaréis": "platinar",
    "platinaría": "platinar",
    "platinaríais": "platinar",
    "platinaríamos": "platinar",
    "platinarían": "platinar",
    "platinarías": "platinar",
    "platinas": "platinar",
    "platinase": "platinar",
    "platinaseis": "platinar",
    "platinasen": "platinar",
    "platinases": "platinar",
    "platinaste": "platinar",
    "platinasteis": "platinar",
    "platine": "platinar",
    "platinemos": "platinar",
    "platinen": "platinar",
    "platines": "platinar",
    "platinistas": "platinista",
    "platino": "platinar",
    "platinoides": "platinoide",
    "platinos": "platino",
    "platinosa": "platinoso",
    "platinosas": "platinoso",
    "platinosos": "platinoso",
    "platinotipias": "platinotipia",
    "platinábamos": "platinar",
    "platináis": "platinar",
    "platináramos": "platinar",
    "platináremos": "platinar",
    "platinásemos": "platinar",
    "platiné": "platinar",
    "platinéis": "platinar",
    "platinífera": "platinífero",
    "platiníferas": "platinífero",
    "platiníferos": "platinífero",
    "platinó": "platinar",
    "platique": "platicar",
    "platiquemos": "platicar",
    "platiquen": "platicar",
    "platiques": "platicar",
    "platiqué": "platicar",
    "platiquéis": "platicar",
    "platirrina": "platirrino",
    "platirrinas": "platirrino",
    "platirrinias": "platirrinia",
    "platirrinos": "platirrino",
    "platonismos": "platonismo",
    "platos": "plato",
    "platuda": "platudo",
    "platudas": "platudo",
    "platudos": "platudo",
    "platínica": "platínico",
    "platínicas": "platínico",
    "platínicos": "platínico",
    "platónica": "platónico",
    "platónicas": "platónico",
    "platónicos": "platónico",
    "platós": "plató",
    "plausibilidades": "plausibilidad",
    "plausibles": "plausible",
    "plausiva": "plausivo",
    "plausivas": "plausivo",
    "plausivos": "plausivo",
    "plausos": "plauso",
    "plaustros": "plaustro",
    "play-backs": "play-back",
    "play-boys": "play-boy",
    "playada": "playado",
    "playadas": "playado",
    "playados": "playado",
    "playas": "playa",
    "playbacks": "playback",
    "playboys": "playboy",
    "playera": "playero",
    "playeras": "playero",
    "playeros": "playero",
    "plazas": "plaza",
    "plazca": "placer",
    "plazcamos": "placer",
    "plazcan": "placer",
    "plazcas": "placer",
    "plazco": "placer",
    "plazcáis": "placer",
    "plazoletas": "plazoleta",
    "plazos": "plazo",
    "plazuelas": "plazuela",
    "plaña": "plañir",
    "plañamos": "plañir",
    "plañan": "plañir",
    "plañas": "plañir",
    "plañe": "plañir",
    "plañen": "plañir",
    "plañendo": "plañir",
    "plañera": "plañir",
    "plañerais": "plañir",
    "plañeran": "plañir",
    "plañeras": "plañir",
    "plañere": "plañir",
    "plañereis": "plañir",
    "plañeren": "plañir",
    "plañeres": "plañir",
    "plañeron": "plañir",
    "plañes": "plañir",
    "plañese": "plañir",
    "plañeseis": "plañir",
    "plañesen": "plañir",
    "plañeses": "plañir",
    "plañid": "plañir",
    "plañida": "plañir",
    "plañidas": "plañir",
    "plañidera": "plañidero",
    "plañideras": "plañidero",
    "plañideros": "plañidero",
    "plañido": "plañir",
    "plañidos": "plañir",
    "plañimos": "plañir",
    "plañiremos": "plañir",
    "plañirá": "plañir",
    "plañirán": "plañir",
    "plañirás": "plañir",
    "plañiré": "plañir",
    "plañiréis": "plañir",
    "plañiría": "plañir",
    "plañiríais": "plañir",
    "plañiríamos": "plañir",
    "plañirían": "plañir",
    "plañirías": "plañir",
    "plañiste": "plañir",
    "plañisteis": "plañir",
    "plaño": "plañir",
    "plañáis": "plañir",
    "plañéramos": "plañir",
    "plañéremos": "plañir",
    "plañésemos": "plañir",
    "plañí": "plañir",
    "plañía": "plañir",
    "plañíais": "plañir",
    "plañíamos": "plañir",
    "plañían": "plañir",
    "plañías": "plañir",
    "plañís": "plañir",
    "plañó": "plañir",
    "pleamares": "pleamar",
    "plebes": "plebe",
    "plebeya": "plebeyo",
    "plebeyas": "plebeyo",
    "plebeyeces": "plebeyez",
    "plebeyos": "plebeyo",
    "plebiscitares": "plebiscitar",
    "plebiscitaria": "plebiscitario",
    "plebiscitarias": "plebiscitario",
    "plebiscitarios": "plebiscitario",
    "plebiscitos": "plebiscito",
    "plecas": "pleca",
    "plectros": "plectro",
    "plecópteros": "plecóptero",
    "plegaba": "plegar",
    "plegabais": "plegar",
    "plegaban": "plegar",
    "plegabas": "plegar",
    "plegables": "plegable",
    "plegad": "plegar",
    "plegada": "plegar",
    "plegadas": "plegar",
    "plegaderas": "plegadera",
    "plegadiza": "plegadizo",
    "plegadizas": "plegadizo",
    "plegadizos": "plegadizo",
    "plegado": "plegar",
    "plegadora": "plegador",
    "plegadoras": "plegadora",
    "plegadores": "plegador",
    "plegados": "plegar",
    "plegaduras": "plegadura",
    "plegamientos": "plegamiento",
    "plegamos": "plegar",
    "plegando": "plegar",
    "plegara": "plegar",
    "plegarais": "plegar",
    "plegaran": "plegar",
    "plegaras": "plegar",
    "plegare": "plegar",
    "plegareis": "plegar",
    "plegaremos": "plegar",
    "plegaren": "plegar",
    "plegares": "plegar",
    "plegarias": "plegaria",
    "plegaron": "plegar",
    "plegará": "plegar",
    "plegarán": "plegar",
    "plegarás": "plegar",
    "plegaré": "plegar",
    "plegaréis": "plegar",
    "plegaría": "plegar",
    "plegaríais": "plegar",
    "plegaríamos": "plegar",
    "plegarían": "plegar",
    "plegarías": "plegar",
    "plegase": "plegar",
    "plegaseis": "plegar",
    "plegasen": "plegar",
    "plegases": "plegar",
    "plegaste": "plegar",
    "plegasteis": "plegar",
    "plegue": "placer",
    "pleguemos": "plegar",
    "pleguerías": "pleguería",
    "pleguetes": "pleguete",
    "plegué": "plegar",
    "pleguéis": "plegar",
    "plegábamos": "plegar",
    "plegáis": "plegar",
    "plegáramos": "plegar",
    "plegáremos": "plegar",
    "plegásemos": "plegar",
    "plegó": "plegar",
    "pleistocenoa": "pleistoceno",
    "pleistocenoas": "pleistoceno",
    "pleistocenoos": "pleistoceno",
    "pleistocenos": "pleistoceno",
    "pleitas": "pleita",
    "pleitea": "pleitear",
    "pleiteaba": "pleitear",
    "pleiteabais": "pleitear",
    "pleiteaban": "pleitear",
    "pleiteabas": "pleitear",
    "pleitead": "pleitear",
    "pleiteada": "pleitear",
    "pleiteadas": "pleitear",
    "pleiteado": "pleitear",
    "pleiteadora": "pleiteador",
    "pleiteadoras": "pleiteador",
    "pleiteadores": "pleiteador",
    "pleiteados": "pleitear",
    "pleiteamos": "pleitear",
    "pleitean": "pleitear",
    "pleiteando": "pleitear",
    "pleiteara": "pleitear",
    "pleitearais": "pleitear",
    "pleitearan": "pleitear",
    "pleitearas": "pleitear",
    "pleiteare": "pleitear",
    "pleiteareis": "pleitear",
    "pleitearemos": "pleitear",
    "pleitearen": "pleitear",
    "pleiteares": "pleitear",
    "pleitearon": "pleitear",
    "pleiteará": "pleitear",
    "pleitearán": "pleitear",
    "pleitearás": "pleitear",
    "pleitearé": "pleitear",
    "pleitearéis": "pleitear",
    "pleitearía": "pleitear",
    "pleitearíais": "pleitear",
    "pleitearíamos": "pleitear",
    "pleitearían": "pleitear",
    "pleitearías": "pleitear",
    "pleiteas": "pleitear",
    "pleitease": "pleitear",
    "pleiteaseis": "pleitear",
    "pleiteasen": "pleitear",
    "pleiteases": "pleitear",
    "pleiteaste": "pleitear",
    "pleiteasteis": "pleitear",
    "pleitee": "pleitear",
    "pleiteemos": "pleitear",
    "pleiteen": "pleitear",
    "pleitees": "pleitear",
    "pleiteo": "pleitear",
    "pleiteos": "pleiteo",
    "pleitesías": "pleitesía",
    "pleiteábamos": "pleitear",
    "pleiteáis": "pleitear",
    "pleiteáramos": "pleitear",
    "pleiteáremos": "pleitear",
    "pleiteásemos": "pleitear",
    "pleiteé": "pleitear",
    "pleiteéis": "pleitear",
    "pleiteó": "pleitear",
    "pleitistas": "pleitista",
    "pleitos": "pleito",
    "plena": "pleno",
    "plenamares": "plenamar",
    "plenaria": "plenario",
    "plenarias": "plenario",
    "plenarios": "plenario",
    "plenas": "pleno",
    "plenilunios": "plenilunio",
    "plenipotenciaria": "plenipotenciario",
    "plenipotenciarias": "plenipotenciario",
    "plenipotenciarios": "plenipotenciario",
    "plenipotencias": "plenipotencia",
    "plenitudes": "plenitud",
    "plenos": "pleno",
    "pleocroica": "pleocroico",
    "pleocroicas": "pleocroico",
    "pleocroicos": "pleocroico",
    "pleocroísmos": "pleocroísmo",
    "pleonasmos": "pleonasmo",
    "pleonastas": "pleonasta",
    "pleonastos": "pleonasto",
    "pleones": "pleon",
    "pleons": "pleon",
    "pleonástica": "pleonástico",
    "pleonásticas": "pleonástico",
    "pleonásticos": "pleonástico",
    "plepas": "plepa",
    "pleremas": "plerema",
    "pleremáticas": "pleremática",
    "ples": "ple",
    "plesiosauros": "plesiosauro",
    "plesiócrona": "plesiócrono",
    "plesiócronas": "plesiócrono",
    "plesiócronos": "plesiócrono",
    "plesímetros": "plesímetro",
    "pletinas": "pletina",
    "pletórica": "pletórico",
    "pletóricas": "pletórico",
    "pletóricos": "pletórico",
    "pleurales": "pleural",
    "pleuras": "pleura",
    "pleuresías": "pleuresía",
    "pleurodinias": "pleurodinia",
    "pleurodoncias": "pleurodoncia",
    "pleurodontos": "pleurodonto",
    "pleuronectiformes": "pleuronectiforme",
    "pleurítica": "pleurítico",
    "pleuríticas": "pleurítico",
    "pleuríticos": "pleurítico",
    "plexiformes": "plexiforme",
    "plexiglases": "plexiglás",
    "plexos": "plexo",
    "pleópticas": "pleóptica",
    "plicas": "plica",
    "pliega": "plegar",
    "pliegan": "plegar",
    "pliegas": "plegar",
    "pliego": "plegar",
    "pliegos": "pliego",
    "pliegue": "plegar",
    "plieguecillos": "plieguecillo",
    "plieguen": "plegar",
    "pliegues": "pliegue",
    "plintitas": "plintita",
    "plintos": "plinto",
    "plintácuica": "plintácuico",
    "plintácuicas": "plintácuico",
    "plintácuicos": "plintácuico",
    "pliocena": "plioceno",
    "pliocenas": "plioceno",
    "pliocenos": "plioceno",
    "pliopolios": "pliopolio",
    "pliopsonios": "pliopsonio",
    "plisa": "plisar",
    "plisaba": "plisar",
    "plisabais": "plisar",
    "plisaban": "plisar",
    "plisabas": "plisar",
    "plisad": "plisar",
    "plisada": "plisar",
    "plisadas": "plisar",
    "plisado": "plisar",
    "plisadora": "plisador",
    "plisadoras": "plisador",
    "plisadores": "plisador",
    "plisados": "plisar",
    "plisamos": "plisar",
    "plisan": "plisar",
    "plisando": "plisar",
    "plisara": "plisar",
    "plisarais": "plisar",
    "plisaran": "plisar",
    "plisaras": "plisar",
    "plisare": "plisar",
    "plisareis": "plisar",
    "plisaremos": "plisar",
    "plisaren": "plisar",
    "plisares": "plisar",
    "plisaron": "plisar",
    "plisará": "plisar",
    "plisarán": "plisar",
    "plisarás": "plisar",
    "plisaré": "plisar",
    "plisaréis": "plisar",
    "plisaría": "plisar",
    "plisaríais": "plisar",
    "plisaríamos": "plisar",
    "plisarían": "plisar",
    "plisarías": "plisar",
    "plisas": "plisar",
    "plisase": "plisar",
    "plisaseis": "plisar",
    "plisasen": "plisar",
    "plisases": "plisar",
    "plisaste": "plisar",
    "plisasteis": "plisar",
    "plise": "plisar",
    "plisemos": "plisar",
    "plisen": "plisar",
    "plises": "plisar",
    "pliso": "plisar",
    "plisábamos": "plisar",
    "plisáis": "plisar",
    "plisáramos": "plisar",
    "plisáremos": "plisar",
    "plisásemos": "plisar",
    "plisé": "plisar",
    "pliséis": "plisar",
    "plisó": "plisar",
    "ploma": "plomar",
    "plomaba": "plomar",
    "plomabais": "plomar",
    "plomaban": "plomar",
    "plomabas": "plomar",
    "plomad": "plomar",
    "plomada": "plomar",
    "plomadas": "plomar",
    "plomado": "plomar",
    "plomados": "plomar",
    "plomamos": "plomar",
    "ploman": "plomar",
    "plomando": "plomar",
    "plomara": "plomar",
    "plomarais": "plomar",
    "plomaran": "plomar",
    "plomaras": "plomar",
    "plomare": "plomar",
    "plomareis": "plomar",
    "plomaremos": "plomar",
    "plomaren": "plomar",
    "plomares": "plomar",
    "plomaron": "plomar",
    "plomará": "plomar",
    "plomarán": "plomar",
    "plomarás": "plomar",
    "plomaré": "plomar",
    "plomaréis": "plomar",
    "plomaría": "plomar",
    "plomaríais": "plomar",
    "plomaríamos": "plomar",
    "plomarían": "plomar",
    "plomarías": "plomar",
    "plomas": "plomar",
    "plomase": "plomar",
    "plomaseis": "plomar",
    "plomasen": "plomar",
    "plomases": "plomar",
    "plomaste": "plomar",
    "plomasteis": "plomar",
    "plomazos": "plomazo",
    "plombaginas": "plombagina",
    "plombatos": "plombato",
    "plome": "plomar",
    "plomea": "plomear",
    "plomeaba": "plomear",
    "plomeabais": "plomear",
    "plomeaban": "plomear",
    "plomeabas": "plomear",
    "plomead": "plomear",
    "plomeada": "plomear",
    "plomeadas": "plomear",
    "plomeado": "plomear",
    "plomeados": "plomear",
    "plomeamos": "plomear",
    "plomean": "plomear",
    "plomeando": "plomear",
    "plomeara": "plomear",
    "plomearais": "plomear",
    "plomearan": "plomear",
    "plomearas": "plomear",
    "plomeare": "plomear",
    "plomeareis": "plomear",
    "plomearemos": "plomear",
    "plomearen": "plomear",
    "plomeares": "plomear",
    "plomearon": "plomear",
    "plomeará": "plomear",
    "plomearán": "plomear",
    "plomearás": "plomear",
    "plomearé": "plomear",
    "plomearéis": "plomear",
    "plomearía": "plomear",
    "plomearíais": "plomear",
    "plomearíamos": "plomear",
    "plomearían": "plomear",
    "plomearías": "plomear",
    "plomeas": "plomear",
    "plomease": "plomear",
    "plomeaseis": "plomear",
    "plomeasen": "plomear",
    "plomeases": "plomear",
    "plomeaste": "plomear",
    "plomeasteis": "plomear",
    "plomee": "plomear",
    "plomeemos": "plomear",
    "plomeen": "plomear",
    "plomees": "plomear",
    "plomemos": "plomar",
    "plomen": "plomar",
    "plomeo": "plomear",
    "plomeros": "plomero",
    "plomerías": "plomería",
    "plomes": "plomar",
    "plomeábamos": "plomear",
    "plomeáis": "plomear",
    "plomeáramos": "plomear",
    "plomeáremos": "plomear",
    "plomeásemos": "plomear",
    "plomeé": "plomear",
    "plomeéis": "plomear",
    "plomeó": "plomear",
    "plomiza": "plomizo",
    "plomizas": "plomizo",
    "plomizos": "plomizo",
    "plomo": "plomar",
    "plomos": "plomo",
    "plomosa": "plomoso",
    "plomosas": "plomoso",
    "plomosos": "plomoso",
    "plomábamos": "plomar",
    "plomáis": "plomar",
    "plomáramos": "plomar",
    "plomáremos": "plomar",
    "plomásemos": "plomar",
    "plomé": "plomar",
    "ploméis": "plomar",
    "plomífera": "plomífero",
    "plomíferas": "plomífero",
    "plomíferos": "plomífero",
    "plomó": "plomar",
    "plotters": "plotter",
    "plugo": "placer",
    "pluguiera": "placer",
    "pluguiere": "placer",
    "pluguieron": "placer",
    "pluguiese": "placer",
    "plumada": "plumado",
    "plumadas": "plumado",
    "plumados": "plumado",
    "plumajera": "plumajero",
    "plumajeras": "plumajero",
    "plumajeros": "plumajero",
    "plumajerías": "plumajería",
    "plumajes": "plumaje",
    "plumas": "pluma",
    "plumazones": "plumazón",
    "plumazos": "plumazo",
    "plumbaginas": "plumbagina",
    "plumbaginácea": "plumbagináceo",
    "plumbagináceas": "plumbagináceo",
    "plumbagináceos": "plumbagináceo",
    "plumbosolvencias": "plumbosolvencia",
    "plumcakes": "plumcake",
    "plumea": "plumear",
    "plumeaba": "plumear",
    "plumeabais": "plumear",
    "plumeaban": "plumear",
    "plumeabas": "plumear",
    "plumead": "plumear",
    "plumeada": "plumear",
    "plumeadas": "plumear",
    "plumeado": "plumear",
    "plumeados": "plumear",
    "plumeamos": "plumear",
    "plumean": "plumear",
    "plumeando": "plumear",
    "plumeara": "plumear",
    "plumearais": "plumear",
    "plumearan": "plumear",
    "plumearas": "plumear",
    "plumeare": "plumear",
    "plumeareis": "plumear",
    "plumearemos": "plumear",
    "plumearen": "plumear",
    "plumeares": "plumear",
    "plumearon": "plumear",
    "plumeará": "plumear",
    "plumearán": "plumear",
    "plumearás": "plumear",
    "plumearé": "plumear",
    "plumearéis": "plumear",
    "plumearía": "plumear",
    "plumearíais": "plumear",
    "plumearíamos": "plumear",
    "plumearían": "plumear",
    "plumearías": "plumear",
    "plumeas": "plumear",
    "plumease": "plumear",
    "plumeaseis": "plumear",
    "plumeasen": "plumear",
    "plumeases": "plumear",
    "plumeaste": "plumear",
    "plumeasteis": "plumear",
    "plumee": "plumear",
    "plumeemos": "plumear",
    "plumeen": "plumear",
    "plumees": "plumear",
    "plumeo": "plumear",
    "plumeros": "plumero",
    "plumeábamos": "plumear",
    "plumeáis": "plumear",
    "plumeáramos": "plumear",
    "plumeáremos": "plumear",
    "plumeásemos": "plumear",
    "plumeé": "plumear",
    "plumeéis": "plumear",
    "plumeó": "plumear",
    "plumieres": "plumier",
    "plumillas": "plumilla",
    "plumiones": "plumión",
    "plumistas": "plumista",
    "plumones": "plumón",
    "plumosa": "plumoso",
    "plumosas": "plumoso",
    "plumosos": "plumoso",
    "plumulada": "plumulado",
    "plumuladas": "plumulado",
    "plumulados": "plumulado",
    "plumífera": "plumífero",
    "plumíferas": "plumífero",
    "plumíferos": "plumífero",
    "plurales": "plural",
    "pluralice": "pluralizar",
    "pluralicemos": "pluralizar",
    "pluralicen": "pluralizar",
    "pluralices": "pluralizar",
    "pluralicé": "pluralizar",
    "pluralicéis": "pluralizar",
    "pluralidades": "pluralidad",
    "pluralismos": "pluralismo",
    "pluralistas": "pluralista",
    "pluraliza": "pluralizar",
    "pluralizaba": "pluralizar",
    "pluralizabais": "pluralizar",
    "pluralizaban": "pluralizar",
    "pluralizabas": "pluralizar",
    "pluralizaciones": "pluralización",
    "pluralizad": "pluralizar",
    "pluralizada": "pluralizar",
    "pluralizadas": "pluralizar",
    "pluralizado": "pluralizar",
    "pluralizados": "pluralizar",
    "pluralizamos": "pluralizar",
    "pluralizan": "pluralizar",
    "pluralizando": "pluralizar",
    "pluralizaoiónes": "pluralización",
    "pluralizara": "pluralizar",
    "pluralizarais": "pluralizar",
    "pluralizaran": "pluralizar",
    "pluralizaras": "pluralizar",
    "pluralizare": "pluralizar",
    "pluralizareis": "pluralizar",
    "pluralizaremos": "pluralizar",
    "pluralizaren": "pluralizar",
    "pluralizares": "pluralizar",
    "pluralizaron": "pluralizar",
    "pluralizará": "pluralizar",
    "pluralizarán": "pluralizar",
    "pluralizarás": "pluralizar",
    "pluralizaré": "pluralizar",
    "pluralizaréis": "pluralizar",
    "pluralizaría": "pluralizar",
    "pluralizaríais": "pluralizar",
    "pluralizaríamos": "pluralizar",
    "pluralizarían": "pluralizar",
    "pluralizarías": "pluralizar",
    "pluralizas": "pluralizar",
    "pluralizase": "pluralizar",
    "pluralizaseis": "pluralizar",
    "pluralizasen": "pluralizar",
    "pluralizases": "pluralizar",
    "pluralizaste": "pluralizar",
    "pluralizasteis": "pluralizar",
    "pluralizo": "pluralizar",
    "pluralizábamos": "pluralizar",
    "pluralizáis": "pluralizar",
    "pluralizáramos": "pluralizar",
    "pluralizáremos": "pluralizar",
    "pluralizásemos": "pluralizar",
    "pluralizó": "pluralizar",
    "plurianuales": "plurianual",
    "pluricelulares": "pluricelular",
    "pluriculturales": "pluricultural",
    "pluriculturalidades": "pluriculturalidad",
    "pluridimensionales": "pluridimensional",
    "pluridisciplinares": "pluridisciplinar",
    "pluriempleada": "pluriempleado",
    "pluriempleadas": "pluriempleado",
    "pluriempleados": "pluriempleado",
    "pluriempleos": "pluriempleo",
    "pluriestratificada": "pluriestratificado",
    "pluriestratificadas": "pluriestratificado",
    "pluriestratificados": "pluriestratificado",
    "plurifamiliares": "plurifamiliar",
    "plurifuncionales": "plurifuncional",
    "plurilaterales": "plurilateral",
    "plurilegislaciones": "plurilegislación",
    "plurilegislativa": "plurilegislativo",
    "plurilegislativas": "plurilegislativo",
    "plurilegislativos": "plurilegislativo",
    "plurilingües": "plurilingüe",
    "plurilingüismos": "plurilingüismo",
    "pluriloculares": "plurilocular",
    "plurinacionales": "plurinacional",
    "plurinacionalidades": "plurinacionalidad",
    "pluriofensiva": "pluriofensivo",
    "pluriofensivas": "pluriofensivo",
    "pluriofensivos": "pluriofensivo",
    "pluripartidismos": "pluripartidismo",
    "pluripotenciales": "pluripotencial",
    "plurisubjetiva": "plurisubjetivo",
    "plurisubjetivas": "plurisubjetivo",
    "plurisubjetivos": "plurisubjetivo",
    "plurivalencias": "plurivalencia",
    "plurivalentes": "plurivalente",
    "plurivendedora": "plurivendedor",
    "plurivendedoras": "plurivendedor",
    "plurivendedores": "plurivendedor",
    "plurívoca": "plurívoco",
    "plurívocas": "plurívoco",
    "plurívocos": "plurívoco",
    "pluscostos": "pluscosto",
    "pluscuamperfecta": "pluscuamperfecto",
    "pluscuamperfectas": "pluscuamperfecto",
    "pluscuamperfectos": "pluscuamperfecto",
    "pluses": "plus",
    "plusmarcas": "plusmarca",
    "plusmarquistas": "plusmarquista",
    "pluspeticiones": "pluspetición",
    "plustrabajos": "plustrabajo",
    "plusvalías": "plusvalía",
    "plutocracias": "plutocracia",
    "plutocrática": "plutocrático",
    "plutocráticas": "plutocrático",
    "plutocráticos": "plutocrático",
    "plutonios": "plutonio",
    "plutonismos": "plutonismo",
    "plutonistas": "plutonista",
    "plutócratas": "plutócrata",
    "plutónica": "plutónico",
    "plutónicas": "plutónico",
    "plutónicos": "plutónico",
    "pluviales": "pluvial",
    "pluviográfica": "pluviográfico",
    "pluviográficas": "pluviográfico",
    "pluviográficos": "pluviográfico",
    "pluviometrías": "pluviometría",
    "pluviométrica": "pluviométrico",
    "pluviométricas": "pluviométrico",
    "pluviométricos": "pluviométrico",
    "pluviosa": "pluvioso",
    "pluviosas": "pluvioso",
    "pluviosidades": "pluviosidad",
    "pluviosos": "pluvioso",
    "pluviselvas": "pluviselva",
    "pluviógrafos": "pluviógrafo",
    "pluviómetros": "pluviómetro",
    "plácemes": "pláceme",
    "plácetes": "plácet",
    "plácica": "plácico",
    "plácicas": "plácico",
    "plácicos": "plácico",
    "plácida": "plácido",
    "plácidas": "plácido",
    "plácidos": "plácido",
    "plántulas": "plántula",
    "plástica": "plástico",
    "plásticas": "plástico",
    "plásticos": "plástico",
    "plátanos": "plátano",
    "pláticas": "plática",
    "plétoras": "plétora",
    "pléyades": "pléyade",
    "plíntica": "plíntico",
    "plínticas": "plíntico",
    "plínticos": "plíntico",
    "plóteres": "plóter",
    "plúmbea": "plúmbeo",
    "plúmbeas": "plúmbeo",
    "plúmbeos": "plúmbeo",
    "plúmbica": "plúmbico",
    "plúmbicas": "plúmbico",
    "plúmbicos": "plúmbico",
    "plúmea": "plúmeo",
    "plúmeas": "plúmeo",
    "plúmeos": "plúmeo",
    "plúmulas": "plúmula",
    "plúteos": "plúteo",
    "poblaba": "poblar",
    "poblabais": "poblar",
    "poblaban": "poblar",
    "poblabas": "poblar",
    "poblachos": "poblacho",
    "poblacionales": "poblacional",
    "poblaciones": "población",
    "poblad": "poblar",
    "poblada": "poblar",
    "pobladas": "poblar",
    "poblado": "poblar",
    "pobladora": "poblador",
    "pobladoras": "poblador",
    "pobladores": "poblador",
    "poblados": "poblar",
    "poblamientos": "poblamiento",
    "poblamos": "poblar",
    "poblando": "poblar",
    "poblara": "poblar",
    "poblarais": "poblar",
    "poblaran": "poblar",
    "poblaras": "poblar",
    "poblare": "poblar",
    "poblareis": "poblar",
    "poblaremos": "poblar",
    "poblaren": "poblar",
    "poblares": "poblar",
    "poblaron": "poblar",
    "poblará": "poblar",
    "poblarán": "poblar",
    "poblarás": "poblar",
    "poblaré": "poblar",
    "poblaréis": "poblar",
    "poblaría": "poblar",
    "poblaríais": "poblar",
    "poblaríamos": "poblar",
    "poblarían": "poblar",
    "poblarías": "poblar",
    "poblase": "poblar",
    "poblaseis": "poblar",
    "poblasen": "poblar",
    "poblases": "poblar",
    "poblaste": "poblar",
    "poblasteis": "poblar",
    "poblazos": "poblazo",
    "poblemos": "poblar",
    "poblábamos": "poblar",
    "pobláis": "poblar",
    "pobláramos": "poblar",
    "pobláremos": "poblar",
    "poblásemos": "poblar",
    "poblé": "poblar",
    "pobléis": "poblar",
    "pobló": "poblar",
    "pobreros": "pobrero",
    "pobrerías": "pobrería",
    "pobres": "pobre",
    "pobreta": "pobrete",
    "pobretas": "pobrete",
    "pobretea": "pobretear",
    "pobreteaba": "pobretear",
    "pobreteabais": "pobretear",
    "pobreteaban": "pobretear",
    "pobreteabas": "pobretear",
    "pobretead": "pobretear",
    "pobreteada": "pobretear",
    "pobreteadas": "pobretear",
    "pobreteado": "pobretear",
    "pobreteados": "pobretear",
    "pobreteamos": "pobretear",
    "pobretean": "pobretear",
    "pobreteando": "pobretear",
    "pobreteara": "pobretear",
    "pobretearais": "pobretear",
    "pobretearan": "pobretear",
    "pobretearas": "pobretear",
    "pobreteare": "pobretear",
    "pobreteareis": "pobretear",
    "pobretearemos": "pobretear",
    "pobretearen": "pobretear",
    "pobreteares": "pobretear",
    "pobretearon": "pobretear",
    "pobreteará": "pobretear",
    "pobretearán": "pobretear",
    "pobretearás": "pobretear",
    "pobretearé": "pobretear",
    "pobretearéis": "pobretear",
    "pobretearía": "pobretear",
    "pobretearíais": "pobretear",
    "pobretearíamos": "pobretear",
    "pobretearían": "pobretear",
    "pobretearías": "pobretear",
    "pobreteas": "pobretear",
    "pobretease": "pobretear",
    "pobreteaseis": "pobretear",
    "pobreteasen": "pobretear",
    "pobreteases": "pobretear",
    "pobreteaste": "pobretear",
    "pobreteasteis": "pobretear",
    "pobretee": "pobretear",
    "pobreteemos": "pobretear",
    "pobreteen": "pobretear",
    "pobretees": "pobretear",
    "pobreteo": "pobretear",
    "pobreterías": "pobretería",
    "pobretes": "pobrete",
    "pobreteábamos": "pobretear",
    "pobreteáis": "pobretear",
    "pobreteáramos": "pobretear",
    "pobreteáremos": "pobretear",
    "pobreteásemos": "pobretear",
    "pobreteé": "pobretear",
    "pobreteéis": "pobretear",
    "pobreteó": "pobretear",
    "pobretona": "pobretón",
    "pobretonas": "pobretón",
    "pobretones": "pobretón",
    "pobrezas": "pobreza",
    "pobrismos": "pobrismo",
    "poca": "poco",
    "pocas": "poco",
    "poceros": "pocero",
    "pocha": "pocho",
    "pochas": "pocho",
    "pochos": "pocho",
    "pocilgas": "pocilga",
    "pocillos": "pocillo",
    "pociones": "poción",
    "pocos": "poco",
    "poda": "podar",
    "podaba": "podar",
    "podabais": "podar",
    "podaban": "podar",
    "podabas": "podar",
    "podad": "podar",
    "podada": "podar",
    "podadas": "podar",
    "podaderas": "podadera",
    "podado": "podar",
    "podadora": "podador",
    "podadoras": "podadora",
    "podadores": "podador",
    "podados": "podar",
    "podaduras": "podadura",
    "podagras": "podagra",
    "podamos": "poder",
    "podan": "podar",
    "podando": "podar",
    "podara": "podar",
    "podarais": "podar",
    "podaran": "podar",
    "podaras": "podar",
    "podare": "podar",
    "podareis": "podar",
    "podaremos": "podar",
    "podaren": "podar",
    "podares": "podar",
    "podaron": "podar",
    "podará": "podar",
    "podarán": "podar",
    "podarás": "podar",
    "podaré": "podar",
    "podaréis": "podar",
    "podaría": "podar",
    "podaríais": "podar",
    "podaríamos": "podar",
    "podarían": "podar",
    "podarías": "podar",
    "podas": "podar",
    "podase": "podar",
    "podaseis": "podar",
    "podasen": "podar",
    "podases": "podar",
    "podaste": "podar",
    "podasteis": "podar",
    "podazones": "podazón",
    "pode": "podar",
    "poded": "poder",
    "podemos": "poder",
    "poden": "podar",
    "podenca": "podenco",
    "podencas": "podenco",
    "podencos": "podenco",
    "poderdantes": "poderdante",
    "poderes": "poder",
    "poderhabientes": "poderhabiente",
    "poderosa": "poderoso",
    "poderosas": "poderoso",
    "poderosos": "poderoso",
    "poderíos": "poderío",
    "podes": "podar",
    "podestás": "podestá",
    "podida": "poder",
    "podidas": "poder",
    "podido": "poder",
    "podidos": "poder",
    "podios": "podio",
    "podiums": "podium",
    "podo": "podar",
    "podocarpácea": "podocarpáceo",
    "podocarpáceas": "podocarpáceo",
    "podocarpáceos": "podocarpáceo",
    "podologías": "podología",
    "podológica": "podológico",
    "podológicas": "podológico",
    "podológicos": "podológico",
    "podometrías": "podometría",
    "podométrica": "podométrico",
    "podométricas": "podométrico",
    "podométricos": "podométrico",
    "podones": "podón",
    "podostemonácea": "podostemonáceo",
    "podostemonáceas": "podostemonáceo",
    "podostemonáceos": "podostemonáceo",
    "podra": "pudrir",
    "podras": "pudrir",
    "podrece": "podrecer",
    "podreced": "podrecer",
    "podrecemos": "podrecer",
    "podrecen": "podrecer",
    "podreceremos": "podrecer",
    "podrecerá": "podrecer",
    "podrecerán": "podrecer",
    "podrecerás": "podrecer",
    "podreceré": "podrecer",
    "podreceréis": "podrecer",
    "podrecería": "podrecer",
    "podreceríais": "podrecer",
    "podreceríamos": "podrecer",
    "podrecerían": "podrecer",
    "podrecerías": "podrecer",
    "podreces": "podrecer",
    "podrecida": "podrecer",
    "podrecidas": "podrecer",
    "podrecido": "podrecer",
    "podrecidos": "podrecer",
    "podreciendo": "podrecer",
    "podreciera": "podrecer",
    "podrecierais": "podrecer",
    "podrecieran": "podrecer",
    "podrecieras": "podrecer",
    "podreciere": "podrecer",
    "podreciereis": "podrecer",
    "podrecieren": "podrecer",
    "podrecieres": "podrecer",
    "podrecieron": "podrecer",
    "podreciese": "podrecer",
    "podrecieseis": "podrecer",
    "podreciesen": "podrecer",
    "podrecieses": "podrecer",
    "podrecimientos": "podrecimiento",
    "podrecimos": "podrecer",
    "podreciste": "podrecer",
    "podrecisteis": "podrecer",
    "podreciéramos": "podrecer",
    "podreciéremos": "podrecer",
    "podreciésemos": "podrecer",
    "podreció": "podrecer",
    "podrecéis": "podrecer",
    "podrecí": "podrecer",
    "podrecía": "podrecer",
    "podrecíais": "podrecer",
    "podrecíamos": "podrecer",
    "podrecían": "podrecer",
    "podrecías": "podrecer",
    "podredumbres": "podredumbre",
    "podreduras": "podredura",
    "podremos": "poder",
    "podres": "podre",
    "podrezca": "podrecer",
    "podrezcamos": "podrecer",
    "podrezcan": "podrecer",
    "podrezcas": "podrecer",
    "podrezco": "podrecer",
    "podrezcáis": "podrecer",
    "podriciones": "podrición",
    "podrida": "podrir",
    "podridas": "podrir",
    "podrideros": "podridero",
    "podrido": "podrir",
    "podridos": "podrir",
    "podrigorios": "podrigorio",
    "podrimientos": "podrimiento",
    "podriré": "podrir",
    "podriría": "podrir",
    "podro": "pudrir",
    "podros": "pudrir",
    "podrá": "poder",
    "podrán": "poder",
    "podrás": "poder",
    "podré": "poder",
    "podréis": "poder",
    "podrí": "podrir",
    "podría": "poder",
    "podríais": "poder",
    "podríamos": "poder",
    "podrían": "poder",
    "podrías": "poder",
    "podsoles": "podsol",
    "podsolizaciones": "podsolización",
    "podsoluvisoles": "podsoluvisol",
    "podsólica": "podsólico",
    "podsólicas": "podsólico",
    "podsólicos": "podsólico",
    "podzoles": "podzol",
    "podábamos": "podar",
    "podáis": "poder",
    "podáramos": "podar",
    "podáremos": "podar",
    "podásemos": "podar",
    "podé": "podar",
    "podéis": "poder",
    "podía": "poder",
    "podíais": "poder",
    "podíamos": "poder",
    "podían": "poder",
    "podías": "poder",
    "podó": "podar",
    "podóloga": "podólogo",
    "podólogas": "podólogo",
    "podólogos": "podólogo",
    "podómetros": "podómetro",
    "poemarios": "poemario",
    "poemas": "poema",
    "poemática": "poemático",
    "poemáticas": "poemático",
    "poemáticos": "poemático",
    "poesías": "poesía",
    "poetas": "poeta",
    "poetastros": "poetastro",
    "poetice": "poetizar",
    "poeticemos": "poetizar",
    "poeticen": "poetizar",
    "poetices": "poetizar",
    "poeticé": "poetizar",
    "poeticéis": "poetizar",
    "poetisas": "poetisa",
    "poetiza": "poetizar",
    "poetizaba": "poetizar",
    "poetizabais": "poetizar",
    "poetizaban": "poetizar",
    "poetizabas": "poetizar",
    "poetizaciones": "poetización",
    "poetizad": "poetizar",
    "poetizada": "poetizar",
    "poetizadas": "poetizar",
    "poetizado": "poetizar",
    "poetizados": "poetizar",
    "poetizamos": "poetizar",
    "poetizan": "poetizar",
    "poetizando": "poetizar",
    "poetizara": "poetizar",
    "poetizarais": "poetizar",
    "poetizaran": "poetizar",
    "poetizaras": "poetizar",
    "poetizare": "poetizar",
    "poetizareis": "poetizar",
    "poetizaremos": "poetizar",
    "poetizaren": "poetizar",
    "poetizares": "poetizar",
    "poetizaron": "poetizar",
    "poetizará": "poetizar",
    "poetizarán": "poetizar",
    "poetizarás": "poetizar",
    "poetizaré": "poetizar",
    "poetizaréis": "poetizar",
    "poetizaría": "poetizar",
    "poetizaríais": "poetizar",
    "poetizaríamos": "poetizar",
    "poetizarían": "poetizar",
    "poetizarías": "poetizar",
    "poetizas": "poetizar",
    "poetizase": "poetizar",
    "poetizaseis": "poetizar",
    "poetizasen": "poetizar",
    "poetizases": "poetizar",
    "poetizaste": "poetizar",
    "poetizasteis": "poetizar",
    "poetizo": "poetizar",
    "poetizábamos": "poetizar",
    "poetizáis": "poetizar",
    "poetizáramos": "poetizar",
    "poetizáremos": "poetizar",
    "poetizásemos": "poetizar",
    "poetizó": "poetizar",
    "pogonóforos": "pogonóforo",
    "pogromos": "pogromo",
    "pogroms": "pogrom",
    "pointers": "pointer",
    "poiquilocitos": "poiquilocito",
    "poises": "poise",
    "poiseuilles": "poiseuille",
    "pokers": "poker",
    "polaca": "polaco",
    "polacas": "polaco",
    "polacos": "polaco",
    "polacras": "polacra",
    "polainas": "polaina",
    "polares": "polar",
    "polarice": "polarizar",
    "polaricemos": "polarizar",
    "polaricen": "polarizar",
    "polarices": "polarizar",
    "polaricé": "polarizar",
    "polaricéis": "polarizar",
    "polaridades": "polaridad",
    "polarimetrías": "polarimetría",
    "polariscopios": "polariscopio",
    "polariscópica": "polariscópico",
    "polariscópicas": "polariscópico",
    "polariscópicos": "polariscópico",
    "polariza": "polarizar",
    "polarizaba": "polarizar",
    "polarizabais": "polarizar",
    "polarizaban": "polarizar",
    "polarizabas": "polarizar",
    "polarizabilidades": "polarizabilidad",
    "polarizables": "polarizable",
    "polarizaciones": "polarización",
    "polarizad": "polarizar",
    "polarizada": "polarizar",
    "polarizadas": "polarizar",
    "polarizado": "polarizar",
    "polarizadora": "polarizador",
    "polarizadoras": "polarizador",
    "polarizadores": "polarizador",
    "polarizados": "polarizar",
    "polarizamos": "polarizar",
    "polarizan": "polarizar",
    "polarizando": "polarizar",
    "polarizara": "polarizar",
    "polarizarais": "polarizar",
    "polarizaran": "polarizar",
    "polarizaras": "polarizar",
    "polarizare": "polarizar",
    "polarizareis": "polarizar",
    "polarizaremos": "polarizar",
    "polarizaren": "polarizar",
    "polarizares": "polarizar",
    "polarizaron": "polarizar",
    "polarizará": "polarizar",
    "polarizarán": "polarizar",
    "polarizarás": "polarizar",
    "polarizaré": "polarizar",
    "polarizaréis": "polarizar",
    "polarizaría": "polarizar",
    "polarizaríais": "polarizar",
    "polarizaríamos": "polarizar",
    "polarizarían": "polarizar",
    "polarizarías": "polarizar",
    "polarizas": "polarizar",
    "polarizase": "polarizar",
    "polarizaseis": "polarizar",
    "polarizasen": "polarizar",
    "polarizases": "polarizar",
    "polarizaste": "polarizar",
    "polarizasteis": "polarizar",
    "polarizo": "polarizar",
    "polarizábamos": "polarizar",
    "polarizáis": "polarizar",
    "polarizáramos": "polarizar",
    "polarizáremos": "polarizar",
    "polarizásemos": "polarizar",
    "polarizó": "polarizar",
    "polarografías": "polarografía",
    "polarográfica": "polarográfico",
    "polarográficas": "polarográfico",
    "polarográficos": "polarográfico",
    "polaroids": "polaroid",
    "polarones": "polarón",
    "polarímetrica": "polarímetrico",
    "polarímetricas": "polarímetrico",
    "polarímetricos": "polarímetrico",
    "polarímetros": "polarímetro",
    "polarógrafos": "polarógrafo",
    "polca": "polcar",
    "polcaba": "polcar",
    "polcabais": "polcar",
    "polcaban": "polcar",
    "polcabas": "polcar",
    "polcad": "polcar",
    "polcada": "polcar",
    "polcadas": "polcar",
    "polcado": "polcar",
    "polcados": "polcar",
    "polcamos": "polcar",
    "polcan": "polcar",
    "polcando": "polcar",
    "polcara": "polcar",
    "polcarais": "polcar",
    "polcaran": "polcar",
    "polcaras": "polcar",
    "polcare": "polcar",
    "polcareis": "polcar",
    "polcaremos": "polcar",
    "polcaren": "polcar",
    "polcares": "polcar",
    "polcaron": "polcar",
    "polcará": "polcar",
    "polcarán": "polcar",
    "polcarás": "polcar",
    "polcaré": "polcar",
    "polcaréis": "polcar",
    "polcaría": "polcar",
    "polcaríais": "polcar",
    "polcaríamos": "polcar",
    "polcarían": "polcar",
    "polcarías": "polcar",
    "polcas": "polcar",
    "polcase": "polcar",
    "polcaseis": "polcar",
    "polcasen": "polcar",
    "polcases": "polcar",
    "polcaste": "polcar",
    "polcasteis": "polcar",
    "polco": "polcar",
    "polcábamos": "polcar",
    "polcáis": "polcar",
    "polcáramos": "polcar",
    "polcáremos": "polcar",
    "polcásemos": "polcar",
    "polcó": "polcar",
    "poleadas": "poleada",
    "poleames": "poleame",
    "poleas": "polea",
    "polemarcas": "polemarca",
    "polemice": "polemizar",
    "polemicemos": "polemizar",
    "polemicen": "polemizar",
    "polemices": "polemizar",
    "polemicé": "polemizar",
    "polemicéis": "polemizar",
    "polemistas": "polemista",
    "polemiza": "polemizar",
    "polemizaba": "polemizar",
    "polemizabais": "polemizar",
    "polemizaban": "polemizar",
    "polemizabas": "polemizar",
    "polemizad": "polemizar",
    "polemizada": "polemizar",
    "polemizadas": "polemizar",
    "polemizado": "polemizar",
    "polemizados": "polemizar",
    "polemizamos": "polemizar",
    "polemizan": "polemizar",
    "polemizando": "polemizar",
    "polemizara": "polemizar",
    "polemizarais": "polemizar",
    "polemizaran": "polemizar",
    "polemizaras": "polemizar",
    "polemizare": "polemizar",
    "polemizareis": "polemizar",
    "polemizaremos": "polemizar",
    "polemizaren": "polemizar",
    "polemizares": "polemizar",
    "polemizaron": "polemizar",
    "polemizará": "polemizar",
    "polemizarán": "polemizar",
    "polemizarás": "polemizar",
    "polemizaré": "polemizar",
    "polemizaréis": "polemizar",
    "polemizaría": "polemizar",
    "polemizaríais": "polemizar",
    "polemizaríamos": "polemizar",
    "polemizarían": "polemizar",
    "polemizarías": "polemizar",
    "polemizas": "polemizar",
    "polemizase": "polemizar",
    "polemizaseis": "polemizar",
    "polemizasen": "polemizar",
    "polemizases": "polemizar",
    "polemizaste": "polemizar",
    "polemizasteis": "polemizar",
    "polemizo": "polemizar",
    "polemizábamos": "polemizar",
    "polemizáis": "polemizar",
    "polemizáramos": "polemizar",
    "polemizáremos": "polemizar",
    "polemizásemos": "polemizar",
    "polemizó": "polemizar",
    "polemologías": "polemología",
    "polemonios": "polemonio",
    "polemoniácea": "polemoniáceo",
    "polemoniáceas": "polemoniáceo",
    "polemoniáceos": "polemoniáceo",
    "polemóloga": "polemólogo",
    "polemólogas": "polemólogo",
    "polemólogos": "polemólogo",
    "poleos": "poleo",
    "poliacetales": "poliacetal",
    "poliacetatos": "poliacetato",
    "poliacrilamidas": "poliacrilamida",
    "poliacrilatos": "poliacrilato",
    "poliacrilonitrilos": "poliacrilonitrilo",
    "poliacrílica": "poliacrílico",
    "poliacrílicas": "poliacrílico",
    "poliacrílicos": "poliacrílico",
    "poliadiciones": "poliadición",
    "poliaglutinabilidades": "poliaglutinabilidad",
    "poliaglutinaciones": "poliaglutinación",
    "polialcanos": "polialcano",
    "polialcoholes": "polialcohol",
    "polialómeros": "polialómero",
    "poliamidas": "poliamida",
    "poliandrias": "poliandria",
    "polianteas": "poliantea",
    "poliarquías": "poliarquía",
    "poliatómica": "poliatómico",
    "poliatómicas": "poliatómico",
    "poliatómicos": "poliatómico",
    "polibanes": "polibán",
    "polibasitas": "polibasita",
    "polibutadienos": "polibutadieno",
    "polibutenos": "polibuteno",
    "polibutilenos": "polibutileno",
    "polibutirales": "polibutiral",
    "polibásica": "polibásico",
    "polibásicas": "polibásico",
    "polibásicos": "polibásico",
    "policarbonatos": "policarbonato",
    "policarboxílica": "policarboxílico",
    "policarboxílicas": "policarboxílico",
    "policarboxílicos": "policarboxílico",
    "polichinelas": "polichinela",
    "policiaca": "policíaco",
    "policiacas": "policíaco",
    "policiacos": "policíaco",
    "policiales": "policial",
    "policitaciones": "policitación",
    "policitemias": "policitemia",
    "policlonales": "policlonal",
    "policlorada": "policlorado",
    "policloradas": "policlorado",
    "policlorados": "policlorado",
    "policlorobifeniles": "policlorobifenil",
    "policloroprenos": "policloropreno",
    "policloroterfeniles": "policloroterfenil",
    "policlorotrifluoretilenos": "policlorotrifluoretileno",
    "policlorotrifluoroetilenos": "policlorotrifluoroetileno",
    "policloruros": "policloruro",
    "policlínicas": "policlínica",
    "policondensaciones": "policondensación",
    "policondensada": "policondensado",
    "policondensadas": "policondensado",
    "policondensados": "policondensado",
    "policopias": "policopia",
    "policorias": "policoria",
    "policorrientes": "policorriente",
    "policristales": "policristal",
    "policristalina": "policristalino",
    "policristalinas": "policristalino",
    "policristalinos": "policristalino",
    "policroma": "policromo",
    "policromas": "policromo",
    "policromatofilias": "policromatofilia",
    "policromatófila": "policromatófilo",
    "policromatófilas": "policromatófilo",
    "policromatófilos": "policromatófilo",
    "policromos": "policromo",
    "policromías": "policromía",
    "policroísmos": "policroísmo",
    "policultivos": "policultivo",
    "policárpicas": "policárpica",
    "policéntrica": "policéntrico",
    "policéntricas": "policéntrico",
    "policéntricos": "policéntrico",
    "policíaca": "policíaco",
    "policíacas": "policíaco",
    "policíacos": "policíaco",
    "policías": "policía",
    "policíclica": "policíclico",
    "policíclicas": "policíclico",
    "policíclicos": "policíclico",
    "policóndrica": "policóndrico",
    "policóndricas": "policóndrico",
    "policóndricos": "policóndrico",
    "policónica": "policónico",
    "policónicas": "policónico",
    "policónicos": "policónico",
    "polideportiva": "polideportivo",
    "polideportivas": "polideportivo",
    "polideportivos": "polideportivo",
    "polidimensionales": "polidimensional",
    "polidipsias": "polidipsia",
    "polidispersa": "polidisperso",
    "polidispersas": "polidisperso",
    "polidispersos": "polidisperso",
    "poliedros": "poliedro",
    "polienergética": "polienergético",
    "polienergéticas": "polienergético",
    "polienergéticos": "polienergético",
    "poliesterificaciones": "poliesterificación",
    "poliestirenos": "poliestireno",
    "poliestiroles": "poliestirol",
    "poliestérica": "poliestérico",
    "poliestéricas": "poliestérico",
    "poliestéricos": "poliestérico",
    "polietersulfonas": "polietersulfona",
    "polietilenos": "polietileno",
    "polietápica": "polietápico",
    "polietápicas": "polietápico",
    "polietápicos": "polietápico",
    "polifacética": "polifacético",
    "polifacéticas": "polifacético",
    "polifacéticos": "polifacético",
    "polifagias": "polifagia",
    "polifenoles": "polifenol",
    "polifilares": "polifilar",
    "polifiodonta": "polifiodonto",
    "polifiodontas": "polifiodonto",
    "polifiodontias": "polifiodontia",
    "polifiodontos": "polifiodonto",
    "polifluoruros": "polifluoruro",
    "polifonías": "polifonía",
    "poliformaldehídos": "poliformaldehído",
    "poliformales": "poliformal",
    "polifosfatos": "polifosfato",
    "poliftalatos": "poliftalato",
    "polifásica": "polifásico",
    "polifásicas": "polifásico",
    "polifásicos": "polifásico",
    "polifónica": "polifónico",
    "polifónicas": "polifónico",
    "polifónicos": "polifónico",
    "poligalias": "poligalia",
    "poligalácea": "poligaláceo",
    "poligaláceas": "poligaláceo",
    "poligaláceos": "poligaláceo",
    "poligamias": "poligamia",
    "poligenismos": "poligenismo",
    "poligenistas": "poligenista",
    "poligenética": "poligenético",
    "poligenéticas": "poligenético",
    "poligenéticos": "poligenético",
    "poliginias": "poliginia",
    "poliglicoles": "poliglicol",
    "poliglobulias": "poliglobulia",
    "poliglota": "polígloto",
    "poliglotas": "polígloto",
    "poliglotos": "polígloto",
    "poliglotías": "poliglotía",
    "poligonales": "poligonal",
    "poligonizaciones": "poligonización",
    "poligonácea": "poligonáceo",
    "poligonáceas": "poligonáceo",
    "poligonáceos": "poligonáceo",
    "poligrafías": "poligrafía",
    "poligráfica": "poligráfico",
    "poligráficas": "poligráfico",
    "poligráficos": "poligráfico",
    "poligámica": "poligámico",
    "poligámicas": "poligámico",
    "poligámicos": "poligámico",
    "poligénica": "poligénico",
    "poligénicas": "poligénico",
    "poligénicos": "poligénico",
    "poligínica": "poligínico",
    "poligínicas": "poligínico",
    "poligínicos": "poligínico",
    "polihexametilenadipamidas": "polihexametilenadipamida",
    "polihidroxialcanoatos": "polihidroxialcanoato",
    "poliinmunoglobulínica": "poliinmunoglobulínico",
    "poliinmunoglobulínicas": "poliinmunoglobulínico",
    "poliinmunoglobulínicos": "poliinmunoglobulínico",
    "poliinsaturada": "poliinsaturado",
    "poliinsaturadas": "poliinsaturado",
    "poliinsaturados": "poliinsaturado",
    "poliisobutilenos": "poliisobutileno",
    "poliisocianuratos": "poliisocianurato",
    "polillas": "polilla",
    "polimastias": "polimastia",
    "polimatías": "polimatía",
    "polimerasas": "polimerasa",
    "polimerice": "polimerizar",
    "polimericemos": "polimerizar",
    "polimericen": "polimerizar",
    "polimerices": "polimerizar",
    "polimericé": "polimerizar",
    "polimericéis": "polimerizar",
    "polimeriza": "polimerizar",
    "polimerizaba": "polimerizar",
    "polimerizabais": "polimerizar",
    "polimerizaban": "polimerizar",
    "polimerizabas": "polimerizar",
    "polimerizaciones": "polimerización",
    "polimerizad": "polimerizar",
    "polimerizada": "polimerizar",
    "polimerizadas": "polimerizar",
    "polimerizado": "polimerizar",
    "polimerizados": "polimerizar",
    "polimerizamos": "polimerizar",
    "polimerizan": "polimerizar",
    "polimerizando": "polimerizar",
    "polimerizara": "polimerizar",
    "polimerizarais": "polimerizar",
    "polimerizaran": "polimerizar",
    "polimerizaras": "polimerizar",
    "polimerizare": "polimerizar",
    "polimerizareis": "polimerizar",
    "polimerizaremos": "polimerizar",
    "polimerizaren": "polimerizar",
    "polimerizares": "polimerizar",
    "polimerizaron": "polimerizar",
    "polimerizará": "polimerizar",
    "polimerizarán": "polimerizar",
    "polimerizarás": "polimerizar",
    "polimerizaré": "polimerizar",
    "polimerizaréis": "polimerizar",
    "polimerizaría": "polimerizar",
    "polimerizaríais": "polimerizar",
    "polimerizaríamos": "polimerizar",
    "polimerizarían": "polimerizar",
    "polimerizarías": "polimerizar",
    "polimerizas": "polimerizar",
    "polimerizase": "polimerizar",
    "polimerizaseis": "polimerizar",
    "polimerizasen": "polimerizar",
    "polimerizases": "polimerizar",
    "polimerizaste": "polimerizar",
    "polimerizasteis": "polimerizar",
    "polimerizo": "polimerizar",
    "polimerizábamos": "polimerizar",
    "polimerizáis": "polimerizar",
    "polimerizáramos": "polimerizar",
    "polimerizáremos": "polimerizar",
    "polimerizásemos": "polimerizar",
    "polimerizó": "polimerizar",
    "polimerías": "polimería",
    "polimetacrilatos": "polimetacrilato",
    "polimetalismos": "polimetalismo",
    "poliminerales": "polimineral",
    "polimixinas": "polimixina",
    "polimorfa": "polimorfo",
    "polimorfas": "polimorfo",
    "polimorfismos": "polimorfismo",
    "polimorfonucleares": "polimorfonuclear",
    "polimorfos": "polimorfo",
    "polimérica": "polimérico",
    "poliméricas": "polimérico",
    "poliméricos": "polimérico",
    "polimórfica": "polimórfico",
    "polimórficas": "polimórfico",
    "polimórficos": "polimórfico",
    "polines": "polín",
    "polinesia": "polinesio",
    "polinesias": "polinesio",
    "polinesios": "polinesio",
    "polinice": "polinizar",
    "polinicemos": "polinizar",
    "polinicen": "polinizar",
    "polinices": "polinizar",
    "polinicé": "polinizar",
    "polinicéis": "polinizar",
    "poliniza": "polinizar",
    "polinizaba": "polinizar",
    "polinizabais": "polinizar",
    "polinizaban": "polinizar",
    "polinizabas": "polinizar",
    "polinizaciones": "polinización",
    "polinizad": "polinizar",
    "polinizada": "polinizar",
    "polinizadas": "polinizar",
    "polinizado": "polinizar",
    "polinizadores": "polinizador",
    "polinizados": "polinizar",
    "polinizamos": "polinizar",
    "polinizan": "polinizar",
    "polinizando": "polinizar",
    "polinizara": "polinizar",
    "polinizarais": "polinizar",
    "polinizaran": "polinizar",
    "polinizaras": "polinizar",
    "polinizare": "polinizar",
    "polinizareis": "polinizar",
    "polinizaremos": "polinizar",
    "polinizaren": "polinizar",
    "polinizares": "polinizar",
    "polinizaron": "polinizar",
    "polinizará": "polinizar",
    "polinizarán": "polinizar",
    "polinizarás": "polinizar",
    "polinizaré": "polinizar",
    "polinizaréis": "polinizar",
    "polinizaría": "polinizar",
    "polinizaríais": "polinizar",
    "polinizaríamos": "polinizar",
    "polinizarían": "polinizar",
    "polinizarías": "polinizar",
    "polinizas": "polinizar",
    "polinizase": "polinizar",
    "polinizaseis": "polinizar",
    "polinizasen": "polinizar",
    "polinizases": "polinizar",
    "polinizaste": "polinizar",
    "polinizasteis": "polinizar",
    "polinizo": "polinizar",
    "polinizábamos": "polinizar",
    "polinizáis": "polinizar",
    "polinizáramos": "polinizar",
    "polinizáremos": "polinizar",
    "polinizásemos": "polinizar",
    "polinizó": "polinizar",
    "polinomiales": "polinomial",
    "polinomios": "polinomio",
    "polinucleares": "polinuclear",
    "polinésica": "polinésico",
    "polinésicas": "polinésico",
    "polinésicos": "polinésico",
    "polinómica": "polinómico",
    "polinómicas": "polinómico",
    "polinómicos": "polinómico",
    "polinósica": "polinósico",
    "polinósicas": "polinósico",
    "polinósicos": "polinósico",
    "poliodoncias": "poliodoncia",
    "poliolefinas": "poliolefina",
    "poliomielítica": "poliomielítico",
    "poliomielíticas": "poliomielítico",
    "poliomielíticos": "poliomielítico",
    "poliopías": "poliopía",
    "poliorcéticas": "poliorcética",
    "polios": "polio",
    "polioximetilenos": "polioximetileno",
    "polipastos": "polipasto",
    "polipediones": "polipedión",
    "polipeptídica": "polipeptídico",
    "polipeptídicas": "polipeptídico",
    "polipeptídicos": "polipeptídico",
    "poliplacóforos": "poliplacóforo",
    "poliploides": "poliploide",
    "poliploidias": "poliploidia",
    "poliploidías": "poliploidía",
    "polipodios": "polipodio",
    "polipodiácea": "polipodiáceo",
    "polipodiáceas": "polipodiáceo",
    "polipodiáceos": "polipodiáceo",
    "polipolios": "polipolio",
    "poliprenos": "polipreno",
    "polipropilenos": "polipropileno",
    "poliprotodonta": "poliprotodonto",
    "poliprotodontas": "poliprotodonto",
    "poliprotodontos": "poliprotodonto",
    "poliprótica": "poliprótico",
    "polipróticas": "poliprótico",
    "polipróticos": "poliprótico",
    "poliptotons": "poliptoton",
    "poliptótones": "poliptoton",
    "polipéptidos": "polipéptido",
    "polipétala": "polipétalo",
    "polipétalas": "polipétalo",
    "polipétalos": "polipétalo",
    "poliquetos": "poliqueto",
    "polirradiculares": "polirradicular",
    "polirribosomas": "polirribosoma",
    "polirrítmias": "polirrítmia",
    "polisacáridos": "polisacárido",
    "polisarcias": "polisarcia",
    "polisarios": "polisario",
    "poliscopios": "poliscopio",
    "polisemias": "polisemia",
    "polisilogismos": "polisilogismo",
    "polisilábica": "polisilábico",
    "polisilábicas": "polisilábico",
    "polisilábicos": "polisilábico",
    "polisintética": "polisintético",
    "polisintéticas": "polisintético",
    "polisintéticos": "polisintético",
    "polisomas": "polisoma",
    "polisones": "polisón",
    "polisoprenos": "polisopreno",
    "polispastos": "polispasto",
    "polispermias": "polispermia",
    "polistas": "polista",
    "polistila": "polistilo",
    "polistilas": "polistilo",
    "polistilos": "polistilo",
    "polisulfonaciones": "polisulfonación",
    "polisulfonas": "polisulfona",
    "polisulfuros": "polisulfuro",
    "polisémica": "polisémico",
    "polisémicas": "polisémico",
    "polisémicos": "polisémico",
    "polisépala": "polisépalo",
    "polisépalas": "polisépalo",
    "polisépalos": "polisépalo",
    "polisílaba": "polisílabo",
    "polisílabas": "polisílabo",
    "polisílabos": "polisílabo",
    "polisíndetones": "polisíndeton",
    "polisíndetons": "polisíndeton",
    "politburoes": "politburó",
    "politburós": "politburó",
    "politenias": "politenia",
    "politereftalatos": "politereftalato",
    "politetrafluoetilenos": "politetrafluoetileno",
    "politetrafluoroetilenos": "politetrafluoroetileno",
    "politeísmos": "politeísmo",
    "politeístas": "politeísta",
    "politicastra": "politicastro",
    "politicastras": "politicastro",
    "politicastros": "politicastro",
    "politice": "politizar",
    "politicemos": "politizar",
    "politicen": "politizar",
    "politices": "politizar",
    "politicologías": "politicología",
    "politicona": "politicón",
    "politiconas": "politicón",
    "politicones": "politicón",
    "politicé": "politizar",
    "politicéis": "politizar",
    "politicóloga": "politicólogo",
    "politicólogas": "politicólogo",
    "politicólogos": "politicólogo",
    "politipismos": "politipismo",
    "politiquea": "politiquear",
    "politiqueaba": "politiquear",
    "politiqueabais": "politiquear",
    "politiqueaban": "politiquear",
    "politiqueabas": "politiquear",
    "politiquead": "politiquear",
    "politiqueada": "politiquear",
    "politiqueadas": "politiquear",
    "politiqueado": "politiquear",
    "politiqueados": "politiquear",
    "politiqueamos": "politiquear",
    "politiquean": "politiquear",
    "politiqueando": "politiquear",
    "politiqueara": "politiquear",
    "politiquearais": "politiquear",
    "politiquearan": "politiquear",
    "politiquearas": "politiquear",
    "politiqueare": "politiquear",
    "politiqueareis": "politiquear",
    "politiquearemos": "politiquear",
    "politiquearen": "politiquear",
    "politiqueares": "politiquear",
    "politiquearon": "politiquear",
    "politiqueará": "politiquear",
    "politiquearán": "politiquear",
    "politiquearás": "politiquear",
    "politiquearé": "politiquear",
    "politiquearéis": "politiquear",
    "politiquearía": "politiquear",
    "politiquearíais": "politiquear",
    "politiquearíamos": "politiquear",
    "politiquearían": "politiquear",
    "politiquearías": "politiquear",
    "politiqueas": "politiquear",
    "politiquease": "politiquear",
    "politiqueaseis": "politiquear",
    "politiqueasen": "politiquear",
    "politiqueases": "politiquear",
    "politiqueaste": "politiquear",
    "politiqueasteis": "politiquear",
    "politiquee": "politiquear",
    "politiqueemos": "politiquear",
    "politiqueen": "politiquear",
    "politiquees": "politiquear",
    "politiqueo": "politiquear",
    "politiqueos": "politiqueo",
    "politiquera": "politiquero",
    "politiqueras": "politiquero",
    "politiqueros": "politiquero",
    "politiquerías": "politiquería",
    "politiqueábamos": "politiquear",
    "politiqueáis": "politiquear",
    "politiqueáramos": "politiquear",
    "politiqueáremos": "politiquear",
    "politiqueásemos": "politiquear",
    "politiqueé": "politiquear",
    "politiqueéis": "politiquear",
    "politiqueó": "politiquear",
    "politiza": "politizar",
    "politizaba": "politizar",
    "politizabais": "politizar",
    "politizaban": "politizar",
    "politizabas": "politizar",
    "politizaciones": "politización",
    "politizad": "politizar",
    "politizada": "politizar",
    "politizadas": "politizar",
    "politizado": "politizar",
    "politizados": "politizar",
    "politizamos": "politizar",
    "politizan": "politizar",
    "politizando": "politizar",
    "politizara": "politizar",
    "politizarais": "politizar",
    "politizaran": "politizar",
    "politizaras": "politizar",
    "politizare": "politizar",
    "politizareis": "politizar",
    "politizaremos": "politizar",
    "politizaren": "politizar",
    "politizares": "politizar",
    "politizaron": "politizar",
    "politizará": "politizar",
    "politizarán": "politizar",
    "politizarás": "politizar",
    "politizaré": "politizar",
    "politizaréis": "politizar",
    "politizaría": "politizar",
    "politizaríais": "politizar",
    "politizaríamos": "politizar",
    "politizarían": "politizar",
    "politizarías": "politizar",
    "politizas": "politizar",
    "politizase": "politizar",
    "politizaseis": "politizar",
    "politizasen": "politizar",
    "politizases": "politizar",
    "politizaste": "politizar",
    "politizasteis": "politizar",
    "politizo": "politizar",
    "politizábamos": "politizar",
    "politizáis": "politizar",
    "politizáramos": "politizar",
    "politizáremos": "politizar",
    "politizásemos": "politizar",
    "politizó": "politizar",
    "politrópica": "politrópico",
    "politrópicas": "politrópico",
    "politrópicos": "politrópico",
    "politécnica": "politécnico",
    "politécnicas": "politécnico",
    "politécnicos": "politécnico",
    "politénica": "politénico",
    "politénicas": "politénico",
    "politénicos": "politénico",
    "politóloga": "politólogo",
    "politólogas": "politólogo",
    "politólogos": "politólogo",
    "poliuretanos": "poliuretano",
    "poliurias": "poliuria",
    "polivalencias": "polivalencia",
    "polivalentes": "polivalente",
    "polivalva": "polivalvo",
    "polivalvas": "polivalvo",
    "polivalvos": "polivalvo",
    "polivinilidenos": "polivinilideno",
    "polivinilos": "polivinilo",
    "polivinilpirrolidonas": "polivinilpirrolidona",
    "polivinílica": "polivinílico",
    "polivinílicas": "polivinílico",
    "polivinílicos": "polivinílico",
    "polizones": "polizón",
    "polizontes": "polizonte",
    "poliándrica": "poliándrico",
    "poliándricas": "poliándrico",
    "poliándricos": "poliándrico",
    "poliárquica": "poliárquico",
    "poliárquicas": "poliárquico",
    "poliárquicos": "poliárquico",
    "poliédrica": "poliédrico",
    "poliédricas": "poliédrico",
    "poliédricos": "poliédrico",
    "poliésteres": "poliéster",
    "poliéteres": "poliéter",
    "poliétnica": "poliétnico",
    "poliétnicas": "poliétnico",
    "poliétnicos": "poliétnico",
    "polióxidos": "polióxido",
    "poliúrica": "poliúrico",
    "poliúricas": "poliúrico",
    "poliúricos": "poliúrico",
    "poljes": "polje",
    "poljés": "poljé",
    "polladas": "pollada",
    "pollancona": "pollancón",
    "pollanconas": "pollancón",
    "pollancones": "pollancón",
    "pollas": "polla",
    "pollastra": "pollastre",
    "pollastras": "pollastre",
    "pollastres": "pollastre",
    "pollazones": "pollazón",
    "pollea": "pollear",
    "polleaba": "pollear",
    "polleabais": "pollear",
    "polleaban": "pollear",
    "polleabas": "pollear",
    "pollead": "pollear",
    "polleada": "pollear",
    "polleadas": "pollear",
    "polleado": "pollear",
    "polleados": "pollear",
    "polleamos": "pollear",
    "pollean": "pollear",
    "polleando": "pollear",
    "polleara": "pollear",
    "pollearais": "pollear",
    "pollearan": "pollear",
    "pollearas": "pollear",
    "polleare": "pollear",
    "polleareis": "pollear",
    "pollearemos": "pollear",
    "pollearen": "pollear",
    "polleares": "pollear",
    "pollearon": "pollear",
    "polleará": "pollear",
    "pollearán": "pollear",
    "pollearás": "pollear",
    "pollearé": "pollear",
    "pollearéis": "pollear",
    "pollearía": "pollear",
    "pollearíais": "pollear",
    "pollearíamos": "pollear",
    "pollearían": "pollear",
    "pollearías": "pollear",
    "polleas": "pollear",
    "pollease": "pollear",
    "polleaseis": "pollear",
    "polleasen": "pollear",
    "polleases": "pollear",
    "polleaste": "pollear",
    "polleasteis": "pollear",
    "polleces": "pollez",
    "pollee": "pollear",
    "polleemos": "pollear",
    "polleen": "pollear",
    "pollees": "pollear",
    "polleo": "pollear",
    "pollera": "pollero",
    "polleras": "pollero",
    "polleros": "pollero",
    "pollerías": "pollería",
    "polleábamos": "pollear",
    "polleáis": "pollear",
    "polleáramos": "pollear",
    "polleáremos": "pollear",
    "polleásemos": "pollear",
    "polleé": "pollear",
    "polleéis": "pollear",
    "polleó": "pollear",
    "pollina": "pollino",
    "pollinas": "pollino",
    "pollinos": "pollino",
    "pollita": "pollito",
    "pollitas": "pollito",
    "pollitos": "pollito",
    "pollos": "pollo",
    "polluelos": "polluelo",
    "polocitos": "polocito",
    "pololea": "pololear",
    "pololeaba": "pololear",
    "pololeabais": "pololear",
    "pololeaban": "pololear",
    "pololeabas": "pololear",
    "pololead": "pololear",
    "pololeada": "pololear",
    "pololeadas": "pololear",
    "pololeado": "pololear",
    "pololeados": "pololear",
    "pololeamos": "pololear",
    "pololean": "pololear",
    "pololeando": "pololear",
    "pololeara": "pololear",
    "pololearais": "pololear",
    "pololearan": "pololear",
    "pololearas": "pololear",
    "pololeare": "pololear",
    "pololeareis": "pololear",
    "pololearemos": "pololear",
    "pololearen": "pololear",
    "pololeares": "pololear",
    "pololearon": "pololear",
    "pololeará": "pololear",
    "pololearán": "pololear",
    "pololearás": "pololear",
    "pololearé": "pololear",
    "pololearéis": "pololear",
    "pololearía": "pololear",
    "pololearíais": "pololear",
    "pololearíamos": "pololear",
    "pololearían": "pololear",
    "pololearías": "pololear",
    "pololeas": "pololear",
    "pololease": "pololear",
    "pololeaseis": "pololear",
    "pololeasen": "pololear",
    "pololeases": "pololear",
    "pololeaste": "pololear",
    "pololeasteis": "pololear",
    "pololee": "pololear",
    "pololeemos": "pololear",
    "pololeen": "pololear",
    "pololees": "pololear",
    "pololeo": "pololear",
    "pololeábamos": "pololear",
    "pololeáis": "pololear",
    "pololeáramos": "pololear",
    "pololeáremos": "pololear",
    "pololeásemos": "pololear",
    "pololeé": "pololear",
    "pololeéis": "pololear",
    "pololeó": "pololear",
    "pololos": "pololo",
    "polonesa": "polonés",
    "polonesas": "polonés",
    "poloneses": "polonés",
    "polonios": "polonio",
    "polonéses": "polonés",
    "polos": "polo",
    "polque": "polcar",
    "polquemos": "polcar",
    "polquen": "polcar",
    "polques": "polcar",
    "polqué": "polcar",
    "polquéis": "polcar",
    "poltrona": "poltrón",
    "poltronas": "poltrona",
    "poltronerías": "poltronería",
    "poltrones": "poltrón",
    "poltronice": "poltronizar",
    "poltronicemos": "poltronizar",
    "poltronicen": "poltronizar",
    "poltronices": "poltronizar",
    "poltronicé": "poltronizar",
    "poltronicéis": "poltronizar",
    "poltroniza": "poltronizar",
    "poltronizaba": "poltronizar",
    "poltronizabais": "poltronizar",
    "poltronizaban": "poltronizar",
    "poltronizabas": "poltronizar",
    "poltronizad": "poltronizar",
    "poltronizada": "poltronizar",
    "poltronizadas": "poltronizar",
    "poltronizado": "poltronizar",
    "poltronizados": "poltronizar",
    "poltronizamos": "poltronizar",
    "poltronizan": "poltronizar",
    "poltronizando": "poltronizar",
    "poltronizara": "poltronizar",
    "poltronizarais": "poltronizar",
    "poltronizaran": "poltronizar",
    "poltronizaras": "poltronizar",
    "poltronizare": "poltronizar",
    "poltronizareis": "poltronizar",
    "poltronizaremos": "poltronizar",
    "poltronizaren": "poltronizar",
    "poltronizares": "poltronizar",
    "poltronizaron": "poltronizar",
    "poltronizará": "poltronizar",
    "poltronizarán": "poltronizar",
    "poltronizarás": "poltronizar",
    "poltronizaré": "poltronizar",
    "poltronizaréis": "poltronizar",
    "poltronizaría": "poltronizar",
    "poltronizaríais": "poltronizar",
    "poltronizaríamos": "poltronizar",
    "poltronizarían": "poltronizar",
    "poltronizarías": "poltronizar",
    "poltronizas": "poltronizar",
    "poltronizase": "poltronizar",
    "poltronizaseis": "poltronizar",
    "poltronizasen": "poltronizar",
    "poltronizases": "poltronizar",
    "poltronizaste": "poltronizar",
    "poltronizasteis": "poltronizar",
    "poltronizo": "poltronizar",
    "poltronizábamos": "poltronizar",
    "poltronizáis": "poltronizar",
    "poltronizáramos": "poltronizar",
    "poltronizáremos": "poltronizar",
    "poltronizásemos": "poltronizar",
    "poltronizó": "poltronizar",
    "poluciones": "polución",
    "polucitas": "polucita",
    "poluta": "poluto",
    "polutas": "poluto",
    "polutos": "poluto",
    "polvaredas": "polvareda",
    "polveras": "polvera",
    "polvorea": "polvorear",
    "polvoreaba": "polvorear",
    "polvoreabais": "polvorear",
    "polvoreaban": "polvorear",
    "polvoreabas": "polvorear",
    "polvoread": "polvorear",
    "polvoreada": "polvorear",
    "polvoreadas": "polvorear",
    "polvoreado": "polvorear",
    "polvoreados": "polvorear",
    "polvoreamientos": "polvoreamiento",
    "polvoreamos": "polvorear",
    "polvorean": "polvorear",
    "polvoreando": "polvorear",
    "polvoreara": "polvorear",
    "polvorearais": "polvorear",
    "polvorearan": "polvorear",
    "polvorearas": "polvorear",
    "polvoreare": "polvorear",
    "polvoreareis": "polvorear",
    "polvorearemos": "polvorear",
    "polvorearen": "polvorear",
    "polvoreares": "polvorear",
    "polvorearon": "polvorear",
    "polvoreará": "polvorear",
    "polvorearán": "polvorear",
    "polvorearás": "polvorear",
    "polvorearé": "polvorear",
    "polvorearéis": "polvorear",
    "polvorearía": "polvorear",
    "polvorearíais": "polvorear",
    "polvorearíamos": "polvorear",
    "polvorearían": "polvorear",
    "polvorearías": "polvorear",
    "polvoreas": "polvorear",
    "polvorease": "polvorear",
    "polvoreaseis": "polvorear",
    "polvoreasen": "polvorear",
    "polvoreases": "polvorear",
    "polvoreaste": "polvorear",
    "polvoreasteis": "polvorear",
    "polvoree": "polvorear",
    "polvoreemos": "polvorear",
    "polvoreen": "polvorear",
    "polvorees": "polvorear",
    "polvoreo": "polvorear",
    "polvoreábamos": "polvorear",
    "polvoreáis": "polvorear",
    "polvoreáramos": "polvorear",
    "polvoreáremos": "polvorear",
    "polvoreásemos": "polvorear",
    "polvoreé": "polvorear",
    "polvoreéis": "polvorear",
    "polvoreó": "polvorear",
    "polvorice": "polvorizar",
    "polvoricemos": "polvorizar",
    "polvoricen": "polvorizar",
    "polvorices": "polvorizar",
    "polvoricé": "polvorizar",
    "polvoricéis": "polvorizar",
    "polvorienta": "polvoriento",
    "polvorientas": "polvoriento",
    "polvorientos": "polvoriento",
    "polvorillas": "polvorilla",
    "polvorines": "polvorín",
    "polvoristas": "polvorista",
    "polvoriza": "polvorizar",
    "polvorizaba": "polvorizar",
    "polvorizabais": "polvorizar",
    "polvorizaban": "polvorizar",
    "polvorizabas": "polvorizar",
    "polvorizaciones": "polvorización",
    "polvorizad": "polvorizar",
    "polvorizada": "polvorizar",
    "polvorizadas": "polvorizar",
    "polvorizado": "polvorizar",
    "polvorizados": "polvorizar",
    "polvorizamos": "polvorizar",
    "polvorizan": "polvorizar",
    "polvorizando": "polvorizar",
    "polvorizara": "polvorizar",
    "polvorizarais": "polvorizar",
    "polvorizaran": "polvorizar",
    "polvorizaras": "polvorizar",
    "polvorizare": "polvorizar",
    "polvorizareis": "polvorizar",
    "polvorizaremos": "polvorizar",
    "polvorizaren": "polvorizar",
    "polvorizares": "polvorizar",
    "polvorizaron": "polvorizar",
    "polvorizará": "polvorizar",
    "polvorizarán": "polvorizar",
    "polvorizarás": "polvorizar",
    "polvorizaré": "polvorizar",
    "polvorizaréis": "polvorizar",
    "polvorizaría": "polvorizar",
    "polvorizaríais": "polvorizar",
    "polvorizaríamos": "polvorizar",
    "polvorizarían": "polvorizar",
    "polvorizarías": "polvorizar",
    "polvorizas": "polvorizar",
    "polvorizase": "polvorizar",
    "polvorizaseis": "polvorizar",
    "polvorizasen": "polvorizar",
    "polvorizases": "polvorizar",
    "polvorizaste": "polvorizar",
    "polvorizasteis": "polvorizar",
    "polvorizo": "polvorizar",
    "polvorizábamos": "polvorizar",
    "polvorizáis": "polvorizar",
    "polvorizáramos": "polvorizar",
    "polvorizáremos": "polvorizar",
    "polvorizásemos": "polvorizar",
    "polvorizó": "polvorizar",
    "polvorones": "polvorón",
    "polvorosa": "polvoroso",
    "polvorosas": "polvoroso",
    "polvorosos": "polvoroso",
    "polvos": "polvo",
    "polémica": "polémico",
    "polémicas": "polémico",
    "polémicos": "polémico",
    "polícroma": "polícromo",
    "polícromas": "polícromo",
    "polícromos": "polícromo",
    "polífaga": "polífago",
    "polífagas": "polífago",
    "polífagos": "polífago",
    "polífona": "polífono",
    "polífonas": "polífono",
    "polífonos": "polífono",
    "polígalas": "polígala",
    "polígama": "polígamo",
    "polígamas": "polígamo",
    "polígamos": "polígamo",
    "políglota": "polígloto",
    "políglotas": "polígloto",
    "políglotos": "polígloto",
    "polígona": "polígono",
    "polígonas": "polígono",
    "polígonos": "polígono",
    "polígrafa": "polígrafo",
    "polígrafas": "polígrafo",
    "polígrafos": "polígrafo",
    "polímeros": "polímero",
    "polínica": "polínico",
    "polínicas": "polínico",
    "polínicos": "polínico",
    "políperos": "polípero",
    "polípticos": "políptico",
    "política": "político",
    "políticas": "político",
    "político-administrativa": "político-administrativo",
    "político-administrativas": "político-administrativo",
    "político-administrativos": "político-administrativo",
    "político-jurídica": "político-jurídico",
    "político-jurídicas": "político-jurídico",
    "político-jurídicos": "político-jurídico",
    "políticos": "político",
    "pomadas": "pomada",
    "pomaradas": "pomarada",
    "pomares": "pomar",
    "pomarrosas": "pomarrosa",
    "pomas": "poma",
    "pomelos": "pomelo",
    "pomerana": "pomerano",
    "pomeranas": "pomerano",
    "pomerania": "pomeranio",
    "pomeranias": "pomeranio",
    "pomeranios": "pomeranio",
    "pomeranos": "pomerano",
    "pometeada": "pometeado",
    "pometeadas": "pometeado",
    "pometeados": "pometeado",
    "pomicultora": "pomicultor",
    "pomicultoras": "pomicultor",
    "pomicultores": "pomicultor",
    "pomologías": "pomología",
    "pomos": "pomo",
    "pompas": "pompa",
    "pompea": "pompear",
    "pompeaba": "pompear",
    "pompeabais": "pompear",
    "pompeaban": "pompear",
    "pompeabas": "pompear",
    "pompead": "pompear",
    "pompeada": "pompear",
    "pompeadas": "pompear",
    "pompeado": "pompear",
    "pompeados": "pompear",
    "pompeamos": "pompear",
    "pompean": "pompear",
    "pompeando": "pompear",
    "pompeara": "pompear",
    "pompearais": "pompear",
    "pompearan": "pompear",
    "pompearas": "pompear",
    "pompeare": "pompear",
    "pompeareis": "pompear",
    "pompearemos": "pompear",
    "pompearen": "pompear",
    "pompeares": "pompear",
    "pompearon": "pompear",
    "pompeará": "pompear",
    "pompearán": "pompear",
    "pompearás": "pompear",
    "pompearé": "pompear",
    "pompearéis": "pompear",
    "pompearía": "pompear",
    "pompearíais": "pompear",
    "pompearíamos": "pompear",
    "pompearían": "pompear",
    "pompearías": "pompear",
    "pompeas": "pompear",
    "pompease": "pompear",
    "pompeaseis": "pompear",
    "pompeasen": "pompear",
    "pompeases": "pompear",
    "pompeaste": "pompear",
    "pompeasteis": "pompear",
    "pompee": "pompear",
    "pompeemos": "pompear",
    "pompeen": "pompear",
    "pompees": "pompear",
    "pompeo": "pompear",
    "pompeyana": "pompeyano",
    "pompeyanas": "pompeyano",
    "pompeyanos": "pompeyano",
    "pompeábamos": "pompear",
    "pompeáis": "pompear",
    "pompeáramos": "pompear",
    "pompeáremos": "pompear",
    "pompeásemos": "pompear",
    "pompeé": "pompear",
    "pompeéis": "pompear",
    "pompeó": "pompear",
    "pomponea": "pomponear",
    "pomponeaba": "pomponear",
    "pomponeabais": "pomponear",
    "pomponeaban": "pomponear",
    "pomponeabas": "pomponear",
    "pomponead": "pomponear",
    "pomponeada": "pomponear",
    "pomponeadas": "pomponear",
    "pomponeado": "pomponear",
    "pomponeados": "pomponear",
    "pomponeamos": "pomponear",
    "pomponean": "pomponear",
    "pomponeando": "pomponear",
    "pomponeara": "pomponear",
    "pomponearais": "pomponear",
    "pomponearan": "pomponear",
    "pomponearas": "pomponear",
    "pomponeare": "pomponear",
    "pomponeareis": "pomponear",
    "pomponearemos": "pomponear",
    "pomponearen": "pomponear",
    "pomponeares": "pomponear",
    "pomponearon": "pomponear",
    "pomponeará": "pomponear",
    "pomponearán": "pomponear",
    "pomponearás": "pomponear",
    "pomponearé": "pomponear",
    "pomponearéis": "pomponear",
    "pomponearía": "pomponear",
    "pomponearíais": "pomponear",
    "pomponearíamos": "pomponear",
    "pomponearían": "pomponear",
    "pomponearías": "pomponear",
    "pomponeas": "pomponear",
    "pomponease": "pomponear",
    "pomponeaseis": "pomponear",
    "pomponeasen": "pomponear",
    "pomponeases": "pomponear",
    "pomponeaste": "pomponear",
    "pomponeasteis": "pomponear",
    "pomponee": "pomponear",
    "pomponeemos": "pomponear",
    "pomponeen": "pomponear",
    "pomponees": "pomponear",
    "pomponeo": "pomponear",
    "pompones": "pompón",
    "pomponeábamos": "pomponear",
    "pomponeáis": "pomponear",
    "pomponeáramos": "pomponear",
    "pomponeáremos": "pomponear",
    "pomponeásemos": "pomponear",
    "pomponeé": "pomponear",
    "pomponeéis": "pomponear",
    "pomponeó": "pomponear",
    "pomposa": "pomposo",
    "pomposas": "pomposo",
    "pomposidades": "pomposidad",
    "pomposos": "pomposo",
    "pompática": "pompático",
    "pompáticas": "pompático",
    "pompáticos": "pompático",
    "pomácea": "pomáceo",
    "pomáceas": "pomáceo",
    "pomáceos": "pomáceo",
    "pomífera": "pomífero",
    "pomíferas": "pomífero",
    "pomíferos": "pomífero",
    "pon": "poner",
    "poncha": "poncho",
    "ponchadas": "ponchada",
    "ponchas": "poncho",
    "poncheras": "ponchera",
    "ponches": "ponche",
    "ponchos": "poncho",
    "poncidres": "poncidre",
    "ponciles": "poncil",
    "poncíes": "poncí",
    "poncís": "poncí",
    "pondera": "ponderar",
    "ponderaba": "ponderar",
    "ponderabais": "ponderar",
    "ponderaban": "ponderar",
    "ponderabas": "ponderar",
    "ponderables": "ponderable",
    "ponderaciones": "ponderación",
    "ponderad": "ponderar",
    "ponderada": "ponderar",
    "ponderadas": "ponderar",
    "ponderado": "ponderar",
    "ponderadora": "ponderador",
    "ponderadoras": "ponderador",
    "ponderadores": "ponderador",
    "ponderados": "ponderar",
    "ponderales": "ponderal",
    "ponderamos": "ponderar",
    "ponderan": "ponderar",
    "ponderando": "ponderar",
    "ponderara": "ponderar",
    "ponderarais": "ponderar",
    "ponderaran": "ponderar",
    "ponderaras": "ponderar",
    "ponderare": "ponderar",
    "ponderareis": "ponderar",
    "ponderaremos": "ponderar",
    "ponderaren": "ponderar",
    "ponderares": "ponderar",
    "ponderaron": "ponderar",
    "ponderará": "ponderar",
    "ponderarán": "ponderar",
    "ponderarás": "ponderar",
    "ponderaré": "ponderar",
    "ponderaréis": "ponderar",
    "ponderaría": "ponderar",
    "ponderaríais": "ponderar",
    "ponderaríamos": "ponderar",
    "ponderarían": "ponderar",
    "ponderarías": "ponderar",
    "ponderas": "ponderar",
    "ponderase": "ponderar",
    "ponderaseis": "ponderar",
    "ponderasen": "ponderar",
    "ponderases": "ponderar",
    "ponderaste": "ponderar",
    "ponderasteis": "ponderar",
    "ponderativa": "ponderativo",
    "ponderativas": "ponderativo",
    "ponderativos": "ponderativo",
    "pondere": "ponderar",
    "ponderemos": "ponderar",
    "ponderen": "ponderar",
    "ponderes": "ponderar",
    "pondero": "ponderar",
    "ponderosa": "ponderoso",
    "ponderosas": "ponderoso",
    "ponderosos": "ponderoso",
    "ponderábamos": "ponderar",
    "ponderáis": "ponderar",
    "ponderáramos": "ponderar",
    "ponderáremos": "ponderar",
    "ponderásemos": "ponderar",
    "ponderé": "ponderar",
    "ponderéis": "ponderar",
    "ponderó": "ponderar",
    "pondremos": "poner",
    "pondrá": "poner",
    "pondrán": "poner",
    "pondrás": "poner",
    "pondré": "poner",
    "pondréis": "poner",
    "pondría": "poner",
    "pondríais": "poner",
    "pondríamos": "poner",
    "pondrían": "poner",
    "pondrías": "poner",
    "pone": "poner",
    "poned": "poner",
    "ponedera": "ponedero",
    "ponederas": "ponedero",
    "ponederos": "ponedero",
    "ponedora": "ponedor",
    "ponedoras": "ponedor",
    "ponedores": "ponedor",
    "ponemos": "poner",
    "ponen": "poner",
    "ponencias": "ponencia",
    "ponentes": "ponente",
    "pones": "poner",
    "poneys": "poney",
    "ponga": "poner",
    "pongamos": "poner",
    "pongan": "poner",
    "pongas": "poner",
    "pongo": "poner",
    "pongáis": "poner",
    "poniendo": "poner",
    "ponientadas": "ponientada",
    "ponientes": "poniente",
    "pontajes": "pontaje",
    "pontanas": "pontana",
    "pontazgos": "pontazgo",
    "pontazguera": "pontazguero",
    "pontazgueras": "pontazguero",
    "pontazgueros": "pontazguero",
    "pontea": "pontear",
    "ponteaba": "pontear",
    "ponteabais": "pontear",
    "ponteaban": "pontear",
    "ponteabas": "pontear",
    "pontead": "pontear",
    "ponteada": "pontear",
    "ponteadas": "pontear",
    "ponteado": "pontear",
    "ponteados": "pontear",
    "ponteamos": "pontear",
    "pontean": "pontear",
    "ponteando": "pontear",
    "ponteara": "pontear",
    "pontearais": "pontear",
    "pontearan": "pontear",
    "pontearas": "pontear",
    "ponteare": "pontear",
    "ponteareis": "pontear",
    "pontearemos": "pontear",
    "pontearen": "pontear",
    "ponteares": "pontear",
    "pontearon": "pontear",
    "ponteará": "pontear",
    "pontearán": "pontear",
    "pontearás": "pontear",
    "pontearé": "pontear",
    "pontearéis": "pontear",
    "pontearía": "pontear",
    "pontearíais": "pontear",
    "pontearíamos": "pontear",
    "pontearían": "pontear",
    "pontearías": "pontear",
    "ponteas": "pontear",
    "pontease": "pontear",
    "ponteaseis": "pontear",
    "ponteasen": "pontear",
    "ponteases": "pontear",
    "ponteaste": "pontear",
    "ponteasteis": "pontear",
    "pontee": "pontear",
    "ponteemos": "pontear",
    "ponteen": "pontear",
    "pontees": "pontear",
    "ponteo": "pontear",
    "pontevedresa": "pontevedrés",
    "pontevedresas": "pontevedrés",
    "pontevedreses": "pontevedrés",
    "ponteábamos": "pontear",
    "ponteáis": "pontear",
    "ponteáramos": "pontear",
    "ponteáremos": "pontear",
    "ponteásemos": "pontear",
    "ponteé": "pontear",
    "ponteéis": "pontear",
    "ponteó": "pontear",
    "pontifica": "pontificar",
    "pontificaba": "pontificar",
    "pontificabais": "pontificar",
    "pontificaban": "pontificar",
    "pontificabas": "pontificar",
    "pontificad": "pontificar",
    "pontificada": "pontificar",
    "pontificadas": "pontificar",
    "pontificado": "pontificar",
    "pontificados": "pontificar",
    "pontificales": "pontifical",
    "pontificamos": "pontificar",
    "pontifican": "pontificar",
    "pontificando": "pontificar",
    "pontificara": "pontificar",
    "pontificarais": "pontificar",
    "pontificaran": "pontificar",
    "pontificaras": "pontificar",
    "pontificare": "pontificar",
    "pontificareis": "pontificar",
    "pontificaremos": "pontificar",
    "pontificaren": "pontificar",
    "pontificares": "pontificar",
    "pontificaron": "pontificar",
    "pontificará": "pontificar",
    "pontificarán": "pontificar",
    "pontificarás": "pontificar",
    "pontificaré": "pontificar",
    "pontificaréis": "pontificar",
    "pontificaría": "pontificar",
    "pontificaríais": "pontificar",
    "pontificaríamos": "pontificar",
    "pontificarían": "pontificar",
    "pontificarías": "pontificar",
    "pontificas": "pontificar",
    "pontificase": "pontificar",
    "pontificaseis": "pontificar",
    "pontificasen": "pontificar",
    "pontificases": "pontificar",
    "pontificaste": "pontificar",
    "pontificasteis": "pontificar",
    "pontificia": "pontificio",
    "pontificias": "pontificio",
    "pontificios": "pontificio",
    "pontifico": "pontificar",
    "pontificábamos": "pontificar",
    "pontificáis": "pontificar",
    "pontificáramos": "pontificar",
    "pontificáremos": "pontificar",
    "pontificásemos": "pontificar",
    "pontificó": "pontificar",
    "pontifique": "pontificar",
    "pontifiquemos": "pontificar",
    "pontifiquen": "pontificar",
    "pontifiques": "pontificar",
    "pontifiqué": "pontificar",
    "pontifiquéis": "pontificar",
    "pontina": "pontino",
    "pontinas": "pontino",
    "pontinos": "pontino",
    "pontonera": "pontonero",
    "pontoneras": "pontonero",
    "pontoneros": "pontonero",
    "pontones": "pontón",
    "pontos": "ponto",
    "pontífices": "pontífice",
    "ponzoñas": "ponzoña",
    "ponzoñosa": "ponzoñoso",
    "ponzoñosas": "ponzoñoso",
    "ponzoñosos": "ponzoñoso",
    "ponéis": "poner",
    "ponía": "poner",
    "poníais": "poner",
    "poníamos": "poner",
    "ponían": "poner",
    "ponías": "poner",
    "pools": "pool",
    "popas": "popa",
    "popelinas": "popelina",
    "popelines": "popelín",
    "popes": "pope",
    "poplítea": "poplíteo",
    "poplíteas": "poplíteo",
    "poplíteos": "poplíteo",
    "pops": "pop",
    "populachera": "populachero",
    "populacheras": "populachero",
    "populacheros": "populachero",
    "populacherías": "populachería",
    "populachos": "populacho",
    "populares": "popular",
    "popularice": "popularizar",
    "popularicemos": "popularizar",
    "popularicen": "popularizar",
    "popularices": "popularizar",
    "popularicé": "popularizar",
    "popularicéis": "popularizar",
    "popularidades": "popularidad",
    "populariza": "popularizar",
    "popularizaba": "popularizar",
    "popularizabais": "popularizar",
    "popularizaban": "popularizar",
    "popularizabas": "popularizar",
    "popularizaciones": "popularización",
    "popularizad": "popularizar",
    "popularizada": "popularizar",
    "popularizadas": "popularizar",
    "popularizado": "popularizar",
    "popularizados": "popularizar",
    "popularizamos": "popularizar",
    "popularizan": "popularizar",
    "popularizando": "popularizar",
    "popularizara": "popularizar",
    "popularizarais": "popularizar",
    "popularizaran": "popularizar",
    "popularizaras": "popularizar",
    "popularizare": "popularizar",
    "popularizareis": "popularizar",
    "popularizaremos": "popularizar",
    "popularizaren": "popularizar",
    "popularizares": "popularizar",
    "popularizaron": "popularizar",
    "popularizará": "popularizar",
    "popularizarán": "popularizar",
    "popularizarás": "popularizar",
    "popularizaré": "popularizar",
    "popularizaréis": "popularizar",
    "popularizaría": "popularizar",
    "popularizaríais": "popularizar",
    "popularizaríamos": "popularizar",
    "popularizarían": "popularizar",
    "popularizarías": "popularizar",
    "popularizas": "popularizar",
    "popularizase": "popularizar",
    "popularizaseis": "popularizar",
    "popularizasen": "popularizar",
    "popularizases": "popularizar",
    "popularizaste": "popularizar",
    "popularizasteis": "popularizar",
    "popularizo": "popularizar",
    "popularizábamos": "popularizar",
    "popularizáis": "popularizar",
    "popularizáramos": "popularizar",
    "popularizáremos": "popularizar",
    "popularizásemos": "popularizar",
    "popularizó": "popularizar",
    "populismos": "populismo",
    "populistas": "populista",
    "populosa": "populoso",
    "populosas": "populoso",
    "populosos": "populoso",
    "popurris": "popurri",
    "popurrís": "popurrí",
    "poquedades": "poquedad",
    "poquerses": "póquer",
    "poquita": "poquito",
    "poquitas": "poquito",
    "poquitos": "poquito",
    "poquísima": "poquísimo",
    "poquísimas": "poquísimo",
    "poquísimos": "poquísimo",
    "porales": "poral",
    "porcachona": "porcachón",
    "porcachonas": "porcachón",
    "porcachones": "porcachón",
    "porcallona": "porcallón",
    "porcallonas": "porcallón",
    "porcallones": "porcallón",
    "porcelanas": "porcelana",
    "porcentajes": "porcentaje",
    "porcentuales": "porcentual",
    "porches": "porche",
    "porcicultora": "porcicultor",
    "porcicultoras": "porcicultor",
    "porcicultores": "porcicultor",
    "porciculturas": "porcicultura",
    "porcina": "porcino",
    "porcinas": "porcino",
    "porcinos": "porcino",
    "porcionera": "porcionero",
    "porcioneras": "porcionero",
    "porcioneros": "porcionero",
    "porciones": "porción",
    "porcipelos": "porcipelo",
    "porciúnculas": "porciúncula",
    "porcuna": "porcuno",
    "porcunas": "porcuno",
    "porcunos": "porcuno",
    "porcupina": "porcupino",
    "porcupinas": "porcupino",
    "porcupinos": "porcupino",
    "pordiosea": "pordiosear",
    "pordioseaba": "pordiosear",
    "pordioseabais": "pordiosear",
    "pordioseaban": "pordiosear",
    "pordioseabas": "pordiosear",
    "pordiosead": "pordiosear",
    "pordioseada": "pordiosear",
    "pordioseadas": "pordiosear",
    "pordioseado": "pordiosear",
    "pordioseados": "pordiosear",
    "pordioseamos": "pordiosear",
    "pordiosean": "pordiosear",
    "pordioseando": "pordiosear",
    "pordioseara": "pordiosear",
    "pordiosearais": "pordiosear",
    "pordiosearan": "pordiosear",
    "pordiosearas": "pordiosear",
    "pordioseare": "pordiosear",
    "pordioseareis": "pordiosear",
    "pordiosearemos": "pordiosear",
    "pordiosearen": "pordiosear",
    "pordioseares": "pordiosear",
    "pordiosearon": "pordiosear",
    "pordioseará": "pordiosear",
    "pordiosearán": "pordiosear",
    "pordiosearás": "pordiosear",
    "pordiosearé": "pordiosear",
    "pordiosearéis": "pordiosear",
    "pordiosearía": "pordiosear",
    "pordiosearíais": "pordiosear",
    "pordiosearíamos": "pordiosear",
    "pordiosearían": "pordiosear",
    "pordiosearías": "pordiosear",
    "pordioseas": "pordiosear",
    "pordiosease": "pordiosear",
    "pordioseaseis": "pordiosear",
    "pordioseasen": "pordiosear",
    "pordioseases": "pordiosear",
    "pordioseaste": "pordiosear",
    "pordioseasteis": "pordiosear",
    "pordiosee": "pordiosear",
    "pordioseemos": "pordiosear",
    "pordioseen": "pordiosear",
    "pordiosees": "pordiosear",
    "pordioseo": "pordiosear",
    "pordioseos": "pordioseo",
    "pordiosera": "pordiosero",
    "pordioseras": "pordiosero",
    "pordioseros": "pordiosero",
    "pordioserías": "pordiosería",
    "pordioseábamos": "pordiosear",
    "pordioseáis": "pordiosear",
    "pordioseáramos": "pordiosear",
    "pordioseáremos": "pordiosear",
    "pordioseásemos": "pordiosear",
    "pordioseé": "pordiosear",
    "pordioseéis": "pordiosear",
    "pordioseó": "pordiosear",
    "porfiaba": "porfiar",
    "porfiabais": "porfiar",
    "porfiaban": "porfiar",
    "porfiabas": "porfiar",
    "porfiad": "porfiar",
    "porfiada": "porfiar",
    "porfiadas": "porfiar",
    "porfiado": "porfiar",
    "porfiadora": "porfiador",
    "porfiadoras": "porfiador",
    "porfiadores": "porfiador",
    "porfiados": "porfiar",
    "porfiamos": "porfiar",
    "porfiando": "porfiar",
    "porfiara": "porfiar",
    "porfiarais": "porfiar",
    "porfiaran": "porfiar",
    "porfiaras": "porfiar",
    "porfiare": "porfiar",
    "porfiareis": "porfiar",
    "porfiaremos": "porfiar",
    "porfiaren": "porfiar",
    "porfiares": "porfiar",
    "porfiaron": "porfiar",
    "porfiará": "porfiar",
    "porfiarán": "porfiar",
    "porfiarás": "porfiar",
    "porfiaré": "porfiar",
    "porfiaréis": "porfiar",
    "porfiaría": "porfiar",
    "porfiaríais": "porfiar",
    "porfiaríamos": "porfiar",
    "porfiarían": "porfiar",
    "porfiarías": "porfiar",
    "porfiase": "porfiar",
    "porfiaseis": "porfiar",
    "porfiasen": "porfiar",
    "porfiases": "porfiar",
    "porfiaste": "porfiar",
    "porfiasteis": "porfiar",
    "porfiemos": "porfiar",
    "porfiosa": "porfioso",
    "porfiosas": "porfioso",
    "porfiosos": "porfioso",
    "porfirias": "porfiria",
    "porfiriatos": "porfiriato",
    "porfirice": "porfirizar",
    "porfiricemos": "porfirizar",
    "porfiricen": "porfirizar",
    "porfirices": "porfirizar",
    "porfiricé": "porfirizar",
    "porfiricéis": "porfirizar",
    "porfirinas": "porfirina",
    "porfiritas": "porfirita",
    "porfiriza": "porfirizar",
    "porfirizaba": "porfirizar",
    "porfirizabais": "porfirizar",
    "porfirizaban": "porfirizar",
    "porfirizabas": "porfirizar",
    "porfirizad": "porfirizar",
    "porfirizada": "porfirizar",
    "porfirizadas": "porfirizar",
    "porfirizado": "porfirizar",
    "porfirizadores": "porfirizador",
    "porfirizados": "porfirizar",
    "porfirizamos": "porfirizar",
    "porfirizan": "porfirizar",
    "porfirizando": "porfirizar",
    "porfirizara": "porfirizar",
    "porfirizarais": "porfirizar",
    "porfirizaran": "porfirizar",
    "porfirizaras": "porfirizar",
    "porfirizare": "porfirizar",
    "porfirizareis": "porfirizar",
    "porfirizaremos": "porfirizar",
    "porfirizaren": "porfirizar",
    "porfirizares": "porfirizar",
    "porfirizaron": "porfirizar",
    "porfirizará": "porfirizar",
    "porfirizarán": "porfirizar",
    "porfirizarás": "porfirizar",
    "porfirizaré": "porfirizar",
    "porfirizaréis": "porfirizar",
    "porfirizaría": "porfirizar",
    "porfirizaríais": "porfirizar",
    "porfirizaríamos": "porfirizar",
    "porfirizarían": "porfirizar",
    "porfirizarías": "porfirizar",
    "porfirizas": "porfirizar",
    "porfirizase": "porfirizar",
    "porfirizaseis": "porfirizar",
    "porfirizasen": "porfirizar",
    "porfirizases": "porfirizar",
    "porfirizaste": "porfirizar",
    "porfirizasteis": "porfirizar",
    "porfirizo": "porfirizar",
    "porfirizábamos": "porfirizar",
    "porfirizáis": "porfirizar",
    "porfirizáramos": "porfirizar",
    "porfirizáremos": "porfirizar",
    "porfirizásemos": "porfirizar",
    "porfirizó": "porfirizar",
    "porfiábamos": "porfiar",
    "porfiáis": "porfiar",
    "porfiáramos": "porfiar",
    "porfiáremos": "porfiar",
    "porfiásemos": "porfiar",
    "porfié": "porfiar",
    "porfiéis": "porfiar",
    "porfió": "porfiar",
    "porfolios": "porfolio",
    "porfía": "porfiar",
    "porfían": "porfiar",
    "porfías": "porfiar",
    "porfídica": "porfídico",
    "porfídicas": "porfídico",
    "porfídicos": "porfídico",
    "porfíe": "porfiar",
    "porfíen": "porfiar",
    "porfíes": "porfiar",
    "porfío": "porfiar",
    "porismas": "porisma",
    "pormenores": "pormenor",
    "pormenorice": "pormenorizar",
    "pormenoricemos": "pormenorizar",
    "pormenoricen": "pormenorizar",
    "pormenorices": "pormenorizar",
    "pormenoricé": "pormenorizar",
    "pormenoricéis": "pormenorizar",
    "pormenoriza": "pormenorizar",
    "pormenorizaba": "pormenorizar",
    "pormenorizabais": "pormenorizar",
    "pormenorizaban": "pormenorizar",
    "pormenorizabas": "pormenorizar",
    "pormenorizaciones": "pormenorización",
    "pormenorizad": "pormenorizar",
    "pormenorizada": "pormenorizar",
    "pormenorizadas": "pormenorizar",
    "pormenorizado": "pormenorizar",
    "pormenorizados": "pormenorizar",
    "pormenorizamos": "pormenorizar",
    "pormenorizan": "pormenorizar",
    "pormenorizando": "pormenorizar",
    "pormenorizara": "pormenorizar",
    "pormenorizarais": "pormenorizar",
    "pormenorizaran": "pormenorizar",
    "pormenorizaras": "pormenorizar",
    "pormenorizare": "pormenorizar",
    "pormenorizareis": "pormenorizar",
    "pormenorizaremos": "pormenorizar",
    "pormenorizaren": "pormenorizar",
    "pormenorizares": "pormenorizar",
    "pormenorizaron": "pormenorizar",
    "pormenorizará": "pormenorizar",
    "pormenorizarán": "pormenorizar",
    "pormenorizarás": "pormenorizar",
    "pormenorizaré": "pormenorizar",
    "pormenorizaréis": "pormenorizar",
    "pormenorizaría": "pormenorizar",
    "pormenorizaríais": "pormenorizar",
    "pormenorizaríamos": "pormenorizar",
    "pormenorizarían": "pormenorizar",
    "pormenorizarías": "pormenorizar",
    "pormenorizas": "pormenorizar",
    "pormenorizase": "pormenorizar",
    "pormenorizaseis": "pormenorizar",
    "pormenorizasen": "pormenorizar",
    "pormenorizases": "pormenorizar",
    "pormenorizaste": "pormenorizar",
    "pormenorizasteis": "pormenorizar",
    "pormenorizo": "pormenorizar",
    "pormenorizábamos": "pormenorizar",
    "pormenorizáis": "pormenorizar",
    "pormenorizáramos": "pormenorizar",
    "pormenorizáremos": "pormenorizar",
    "pormenorizásemos": "pormenorizar",
    "pormenorizó": "pormenorizar",
    "pornografías": "pornografía",
    "pornográfica": "pornográfico",
    "pornográficas": "pornográfico",
    "pornográficos": "pornográfico",
    "pornógrafa": "pornógrafo",
    "pornógrafas": "pornógrafo",
    "pornógrafos": "pornógrafo",
    "pororóes": "pororó",
    "pororós": "pororó",
    "poros": "poro",
    "porosa": "poroso",
    "porosas": "poroso",
    "porosidades": "porosidad",
    "porosos": "poroso",
    "porosímetros": "porosímetro",
    "porotos": "poroto",
    "porquera": "porquero",
    "porqueras": "porquero",
    "porqueriza": "porquerizo",
    "porquerizas": "porquerizo",
    "porquerizos": "porquerizo",
    "porqueros": "porquero",
    "porquerías": "porquería",
    "porqués": "porqué",
    "porradas": "porrada",
    "porrales": "porral",
    "porras": "porra",
    "porrazos": "porrazo",
    "porrerías": "porrería",
    "porretadas": "porretada",
    "porretas": "porreta",
    "porrillas": "porrilla",
    "porrillos": "porrillo",
    "porrinas": "porrina",
    "porrona": "porrón",
    "porronas": "porrón",
    "porrones": "porrón",
    "porros": "porro",
    "porrácea": "porráceo",
    "porráceas": "porráceo",
    "porráceos": "porráceo",
    "porta": "portar",
    "portaacumuladores": "portaacumulador",
    "portaagujas": "portaaguja",
    "portaalmizcles": "portaalmizcle",
    "portaamalgamas": "portaamalgama",
    "portaaspersores": "portaaspersor",
    "portaba": "portar",
    "portabais": "portar",
    "portaban": "portar",
    "portabanderas": "portabandera",
    "portabas": "portar",
    "portabidones": "portabidón",
    "portabilidades": "portabilidad",
    "portables": "portable",
    "portacajas": "portacaja",
    "portacarabinas": "portacarabina",
    "portachuelos": "portachuelo",
    "portacinchas": "portacincha",
    "portad": "portar",
    "portada": "portar",
    "portadas": "portar",
    "portaderas": "portadera",
    "portadillas": "portadilla",
    "portadiques": "portadique",
    "portado": "portar",
    "portadora": "portador",
    "portadoras": "portador",
    "portadores": "portador",
    "portados": "portar",
    "portaelectrodos": "portaelectrodo",
    "portaestandartes": "portaestandarte",
    "portafusibles": "portafusible",
    "portafusiles": "portafusil",
    "portaguiones": "portaguión",
    "portaimanes": "portaimán",
    "portainjertos": "portainjerto",
    "portajes": "portaje",
    "portaladas": "portalada",
    "portalera": "portalero",
    "portaleras": "portalero",
    "portaleros": "portalero",
    "portales": "portal",
    "portalones": "portalón",
    "portalápices": "portalápiz",
    "portamanteos": "portamanteo",
    "portamatrices": "portamatriz",
    "portamoldes": "portamolde",
    "portamos": "portar",
    "portan": "portar",
    "portanarios": "portanario",
    "portando": "portar",
    "portantes": "portante",
    "portaobjetivos": "portaobjetivo",
    "portapaces": "portapaz",
    "portapulidoras": "portapulidora",
    "portara": "portar",
    "portarais": "portar",
    "portaran": "portar",
    "portaras": "portar",
    "portare": "portar",
    "portareis": "portar",
    "portaremos": "portar",
    "portaren": "portar",
    "portares": "portar",
    "portaron": "portar",
    "portarretratos": "portarretrato",
    "portará": "portar",
    "portarán": "portar",
    "portarás": "portar",
    "portaré": "portar",
    "portaréis": "portar",
    "portaría": "portar",
    "portaríais": "portar",
    "portaríamos": "portar",
    "portarían": "portar",
    "portarías": "portar",
    "portas": "portar",
    "portase": "portar",
    "portaseis": "portar",
    "portasen": "portar",
    "portaservilletas": "portaservilleta",
    "portases": "portar",
    "portaste": "portar",
    "portasteis": "portar",
    "portaventanera": "portaventanero",
    "portaventaneras": "portaventanero",
    "portaventaneros": "portaventanero",
    "portavoces": "portavoz",
    "portazgos": "portazgo",
    "portazguera": "portazguero",
    "portazgueras": "portazguero",
    "portazgueros": "portazguero",
    "portazos": "portazo",
    "portañolas": "portañola",
    "portañuelas": "portañuela",
    "porte": "portar",
    "portea": "portear",
    "porteaba": "portear",
    "porteabais": "portear",
    "porteaban": "portear",
    "porteabas": "portear",
    "portead": "portear",
    "porteada": "portear",
    "porteadas": "portear",
    "porteado": "portear",
    "porteadora": "porteador",
    "porteadoras": "porteador",
    "porteadores": "porteador",
    "porteados": "portear",
    "porteamos": "portear",
    "portean": "portear",
    "porteando": "portear",
    "porteara": "portear",
    "portearais": "portear",
    "portearan": "portear",
    "portearas": "portear",
    "porteare": "portear",
    "porteareis": "portear",
    "portearemos": "portear",
    "portearen": "portear",
    "porteares": "portear",
    "portearon": "portear",
    "porteará": "portear",
    "portearán": "portear",
    "portearás": "portear",
    "portearé": "portear",
    "portearéis": "portear",
    "portearía": "portear",
    "portearíais": "portear",
    "portearíamos": "portear",
    "portearían": "portear",
    "portearías": "portear",
    "porteas": "portear",
    "portease": "portear",
    "porteaseis": "portear",
    "porteasen": "portear",
    "porteases": "portear",
    "porteaste": "portear",
    "porteasteis": "portear",
    "portee": "portear",
    "porteemos": "portear",
    "porteen": "portear",
    "portees": "portear",
    "portemos": "portar",
    "porten": "portar",
    "portentos": "portento",
    "portentosa": "portentoso",
    "portentosas": "portentoso",
    "portentosos": "portentoso",
    "porteo": "portear",
    "porteos": "porteo",
    "portera": "portero",
    "porteras": "portero",
    "porteriles": "porteril",
    "porteros": "portero",
    "porterías": "portería",
    "portes": "porte",
    "portezuelas": "portezuela",
    "porteábamos": "portear",
    "porteáis": "portear",
    "porteáramos": "portear",
    "porteáremos": "portear",
    "porteásemos": "portear",
    "porteé": "portear",
    "porteéis": "portear",
    "porteña": "porteño",
    "porteñas": "porteño",
    "porteños": "porteño",
    "porteó": "portear",
    "portichuelos": "portichuelo",
    "portiers": "portier",
    "portillas": "portilla",
    "portilleras": "portillera",
    "portillos": "portillo",
    "porto": "portar",
    "portones": "portón",
    "portopolipasta": "portopolipasto",
    "portopolipastas": "portopolipasto",
    "portopolipastos": "portopolipasto",
    "portorriqueña": "portorriqueño",
    "portorriqueñas": "portorriqueño",
    "portorriqueños": "portorriqueño",
    "portuaria": "portuario",
    "portuarias": "portuario",
    "portuarios": "portuario",
    "portuguesa": "portugués",
    "portuguesadas": "portuguesada",
    "portuguesas": "portugués",
    "portugueses": "portugués",
    "portuguesismos": "portuguesismo",
    "portulacácea": "portulacáceo",
    "portulacáceas": "portulacáceo",
    "portulacáceos": "portulacáceo",
    "portulanos": "portulano",
    "portábamos": "portar",
    "portáis": "portar",
    "portáramos": "portar",
    "portáremos": "portar",
    "portásemos": "portar",
    "portátiles": "portátil",
    "porté": "portar",
    "portéis": "portar",
    "portó": "portar",
    "porvenires": "porvenir",
    "poríferos": "porífero",
    "porímetros": "porímetro",
    "posa": "posar",
    "posaba": "posar",
    "posabais": "posar",
    "posaban": "posar",
    "posabas": "posar",
    "posad": "posar",
    "posada": "posar",
    "posadas": "posar",
    "posadera": "posadero",
    "posaderas": "posadero",
    "posaderos": "posadero",
    "posado": "posar",
    "posados": "posar",
    "posamos": "posar",
    "posan": "posar",
    "posando": "posar",
    "posantes": "posante",
    "posara": "posar",
    "posarais": "posar",
    "posaran": "posar",
    "posaras": "posar",
    "posare": "posar",
    "posareis": "posar",
    "posaremos": "posar",
    "posaren": "posar",
    "posares": "posar",
    "posaron": "posar",
    "posará": "posar",
    "posarán": "posar",
    "posarás": "posar",
    "posaré": "posar",
    "posaréis": "posar",
    "posaría": "posar",
    "posaríais": "posar",
    "posaríamos": "posar",
    "posarían": "posar",
    "posarías": "posar",
    "posas": "posar",
    "posase": "posar",
    "posaseis": "posar",
    "posasen": "posar",
    "posases": "posar",
    "posaste": "posar",
    "posasteis": "posar",
    "posbélica": "posbélico",
    "posbélicas": "posbélico",
    "posbélicos": "posbélico",
    "poscentrales": "poscentral",
    "poscombustiones": "poscombustión",
    "poscomuniones": "poscomunión",
    "poscontractuales": "poscontractual",
    "posdatas": "posdata",
    "pose": "posar",
    "posea": "poseer",
    "poseamos": "poseer",
    "posean": "poseer",
    "poseas": "poseer",
    "posee": "poseer",
    "poseed": "poseer",
    "poseedora": "poseedor",
    "poseedoras": "poseedor",
    "poseedores": "poseedor",
    "poseemos": "poseer",
    "poseen": "poseer",
    "poseeremos": "poseer",
    "poseerá": "poseer",
    "poseerán": "poseer",
    "poseerás": "poseer",
    "poseeré": "poseer",
    "poseeréis": "poseer",
    "poseería": "poseer",
    "poseeríais": "poseer",
    "poseeríamos": "poseer",
    "poseerían": "poseer",
    "poseerías": "poseer",
    "posees": "poseer",
    "poselectorales": "poselectoral",
    "posemos": "posar",
    "posen": "posar",
    "poseo": "poseer",
    "poses": "pose",
    "posesa": "poseso",
    "posesas": "poseso",
    "posesiona": "posesionar",
    "posesionaba": "posesionar",
    "posesionabais": "posesionar",
    "posesionaban": "posesionar",
    "posesionabas": "posesionar",
    "posesionad": "posesionar",
    "posesionada": "posesionar",
    "posesionadas": "posesionar",
    "posesionado": "posesionar",
    "posesionados": "posesionar",
    "posesionales": "posesional",
    "posesionamos": "posesionar",
    "posesionan": "posesionar",
    "posesionando": "posesionar",
    "posesionara": "posesionar",
    "posesionarais": "posesionar",
    "posesionaran": "posesionar",
    "posesionaras": "posesionar",
    "posesionare": "posesionar",
    "posesionareis": "posesionar",
    "posesionaremos": "posesionar",
    "posesionaren": "posesionar",
    "posesionares": "posesionar",
    "posesionaron": "posesionar",
    "posesionará": "posesionar",
    "posesionarán": "posesionar",
    "posesionarás": "posesionar",
    "posesionaré": "posesionar",
    "posesionaréis": "posesionar",
    "posesionaría": "posesionar",
    "posesionaríais": "posesionar",
    "posesionaríamos": "posesionar",
    "posesionarían": "posesionar",
    "posesionarías": "posesionar",
    "posesionas": "posesionar",
    "posesionase": "posesionar",
    "posesionaseis": "posesionar",
    "posesionasen": "posesionar",
    "posesionases": "posesionar",
    "posesionaste": "posesionar",
    "posesionasteis": "posesionar",
    "posesione": "posesionar",
    "posesionemos": "posesionar",
    "posesionen": "posesionar",
    "posesionera": "posesionero",
    "posesioneras": "posesionero",
    "posesioneros": "posesionero",
    "posesiones": "posesionar",
    "posesiono": "posesionar",
    "posesionábamos": "posesionar",
    "posesionáis": "posesionar",
    "posesionáramos": "posesionar",
    "posesionáremos": "posesionar",
    "posesionásemos": "posesionar",
    "posesioné": "posesionar",
    "posesionéis": "posesionar",
    "posesionó": "posesionar",
    "posesiva": "posesivo",
    "posesivas": "posesivo",
    "posesivos": "posesivo",
    "posesora": "posesor",
    "posesoras": "posesor",
    "posesores": "posesor",
    "posesoria": "posesorio",
    "posesorias": "posesorio",
    "posesorios": "posesorio",
    "posesos": "poseso",
    "poseyendo": "poseer",
    "poseyera": "poseer",
    "poseyerais": "poseer",
    "poseyeran": "poseer",
    "poseyeras": "poseer",
    "poseyere": "poseer",
    "poseyereis": "poseer",
    "poseyeren": "poseer",
    "poseyeres": "poseer",
    "poseyeron": "poseer",
    "poseyese": "poseer",
    "poseyeseis": "poseer",
    "poseyesen": "poseer",
    "poseyeses": "poseer",
    "poseyéramos": "poseer",
    "poseyéremos": "poseer",
    "poseyésemos": "poseer",
    "poseyó": "poseer",
    "poseáis": "poseer",
    "poseéis": "poseer",
    "poseí": "poseer",
    "poseía": "poseer",
    "poseíais": "poseer",
    "poseíamos": "poseer",
    "poseían": "poseer",
    "poseías": "poseer",
    "poseída": "poseído",
    "poseídas": "poseído",
    "poseído": "poseer",
    "poseídos": "poseído",
    "poseímos": "poseer",
    "poseíste": "poseer",
    "poseísteis": "poseer",
    "posfacio": "postfacio",
    "posfacios": "postfacio",
    "posfechas": "posfecha",
    "posfija": "posfijo",
    "posfijas": "posfijo",
    "posfijos": "posfijo",
    "posganglionares": "posganglionar",
    "posgrados": "posgrado",
    "posguerras": "posguerra",
    "posibilidades": "posibilidad",
    "posibilismos": "posibilismo",
    "posibilistas": "posibilista",
    "posibilita": "posibilitar",
    "posibilitaba": "posibilitar",
    "posibilitabais": "posibilitar",
    "posibilitaban": "posibilitar",
    "posibilitabas": "posibilitar",
    "posibilitad": "posibilitar",
    "posibilitada": "posibilitar",
    "posibilitadas": "posibilitar",
    "posibilitado": "posibilitar",
    "posibilitados": "posibilitar",
    "posibilitamos": "posibilitar",
    "posibilitan": "posibilitar",
    "posibilitando": "posibilitar",
    "posibilitara": "posibilitar",
    "posibilitarais": "posibilitar",
    "posibilitaran": "posibilitar",
    "posibilitaras": "posibilitar",
    "posibilitare": "posibilitar",
    "posibilitareis": "posibilitar",
    "posibilitaremos": "posibilitar",
    "posibilitaren": "posibilitar",
    "posibilitares": "posibilitar",
    "posibilitaron": "posibilitar",
    "posibilitará": "posibilitar",
    "posibilitarán": "posibilitar",
    "posibilitarás": "posibilitar",
    "posibilitaré": "posibilitar",
    "posibilitaréis": "posibilitar",
    "posibilitaría": "posibilitar",
    "posibilitaríais": "posibilitar",
    "posibilitaríamos": "posibilitar",
    "posibilitarían": "posibilitar",
    "posibilitarías": "posibilitar",
    "posibilitas": "posibilitar",
    "posibilitase": "posibilitar",
    "posibilitaseis": "posibilitar",
    "posibilitasen": "posibilitar",
    "posibilitases": "posibilitar",
    "posibilitaste": "posibilitar",
    "posibilitasteis": "posibilitar",
    "posibilite": "posibilitar",
    "posibilitemos": "posibilitar",
    "posibiliten": "posibilitar",
    "posibilites": "posibilitar",
    "posibilito": "posibilitar",
    "posibilitábamos": "posibilitar",
    "posibilitáis": "posibilitar",
    "posibilitáramos": "posibilitar",
    "posibilitáremos": "posibilitar",
    "posibilitásemos": "posibilitar",
    "posibilité": "posibilitar",
    "posibilitéis": "posibilitar",
    "posibilitó": "posibilitar",
    "posibles": "posible",
    "posiciona": "posicionar",
    "posicionaba": "posicionar",
    "posicionabais": "posicionar",
    "posicionaban": "posicionar",
    "posicionabas": "posicionar",
    "posicionables": "posicionable",
    "posicionad": "posicionar",
    "posicionada": "posicionar",
    "posicionadas": "posicionar",
    "posicionado": "posicionar",
    "posicionadora": "posicionador",
    "posicionadoras": "posicionador",
    "posicionadores": "posicionador",
    "posicionados": "posicionar",
    "posicionales": "posicional",
    "posicionamientos": "posicionamiento",
    "posicionamos": "posicionar",
    "posicionan": "posicionar",
    "posicionando": "posicionar",
    "posicionara": "posicionar",
    "posicionarais": "posicionar",
    "posicionaran": "posicionar",
    "posicionaras": "posicionar",
    "posicionare": "posicionar",
    "posicionareis": "posicionar",
    "posicionaremos": "posicionar",
    "posicionaren": "posicionar",
    "posicionares": "posicionar",
    "posicionaron": "posicionar",
    "posicionará": "posicionar",
    "posicionarán": "posicionar",
    "posicionarás": "posicionar",
    "posicionaré": "posicionar",
    "posicionaréis": "posicionar",
    "posicionaría": "posicionar",
    "posicionaríais": "posicionar",
    "posicionaríamos": "posicionar",
    "posicionarían": "posicionar",
    "posicionarías": "posicionar",
    "posicionas": "posicionar",
    "posicionase": "posicionar",
    "posicionaseis": "posicionar",
    "posicionasen": "posicionar",
    "posicionases": "posicionar",
    "posicionaste": "posicionar",
    "posicionasteis": "posicionar",
    "posicione": "posicionar",
    "posicionemos": "posicionar",
    "posicionen": "posicionar",
    "posiciones": "posicionar",
    "posiciono": "posicionar",
    "posicionábamos": "posicionar",
    "posicionáis": "posicionar",
    "posicionáramos": "posicionar",
    "posicionáremos": "posicionar",
    "posicionásemos": "posicionar",
    "posicioné": "posicionar",
    "posicionéis": "posicionar",
    "posicionó": "posicionar",
    "posidonias": "posidonia",
    "positiva": "positivo",
    "positivas": "positivo",
    "positivismos": "positivismo",
    "positivistas": "positivista",
    "positivos": "positivo",
    "positones": "positón",
    "positonios": "positonio",
    "positrones": "positrón",
    "positronios": "positronio",
    "posituras": "positura",
    "posmas": "posma",
    "posmeridiana": "posmeridiano",
    "posmeridianas": "posmeridiano",
    "posmeridianos": "posmeridiano",
    "posmodernidades": "posmodernidad",
    "poso": "posar",
    "posologías": "posología",
    "posológica": "posológico",
    "posológicas": "posológico",
    "posológicos": "posológico",
    "posones": "posón",
    "posos": "poso",
    "pospiernas": "pospierna",
    "pospondremos": "posponer",
    "pospondrá": "posponer",
    "pospondrán": "posponer",
    "pospondrás": "posponer",
    "pospondré": "posponer",
    "pospondréis": "posponer",
    "pospondría": "posponer",
    "pospondríais": "posponer",
    "pospondríamos": "posponer",
    "pospondrían": "posponer",
    "pospondrías": "posponer",
    "pospone": "posponer",
    "posponed": "posponer",
    "posponemos": "posponer",
    "posponen": "posponer",
    "pospones": "posponer",
    "posponga": "posponer",
    "pospongamos": "posponer",
    "pospongan": "posponer",
    "pospongas": "posponer",
    "pospongo": "posponer",
    "pospongáis": "posponer",
    "posponiendo": "posponer",
    "posponéis": "posponer",
    "posponía": "posponer",
    "posponíais": "posponer",
    "posponíamos": "posponer",
    "posponían": "posponer",
    "posponías": "posponer",
    "posposiciones": "posposición",
    "pospositiva": "pospositivo",
    "pospositivas": "pospositivo",
    "pospositivos": "pospositivo",
    "posproducción": "postproducción",
    "posproduccióoes": "postproducción",
    "pospuesta": "posponer",
    "pospuestas": "posponer",
    "pospuesto": "posponer",
    "pospuestos": "posponer",
    "pospuse": "posponer",
    "pospusiera": "posponer",
    "pospusierais": "posponer",
    "pospusieran": "posponer",
    "pospusieras": "posponer",
    "pospusiere": "posponer",
    "pospusiereis": "posponer",
    "pospusieren": "posponer",
    "pospusieres": "posponer",
    "pospusieron": "posponer",
    "pospusiese": "posponer",
    "pospusieseis": "posponer",
    "pospusiesen": "posponer",
    "pospusieses": "posponer",
    "pospusimos": "posponer",
    "pospusiste": "posponer",
    "pospusisteis": "posponer",
    "pospusiéramos": "posponer",
    "pospusiéremos": "posponer",
    "pospusiésemos": "posponer",
    "pospuso": "posponer",
    "pospón": "posponer",
    "postales": "postal",
    "postalveolares": "postalveolar",
    "postas": "posta",
    "postbucales": "postbucal",
    "postcondílea": "postcondíleo",
    "postcondíleas": "postcondíleo",
    "postcondíleos": "postcondíleo",
    "postcontracciones": "postcontracción",
    "postcosechas": "postcosecha",
    "postcraniales": "postcranial",
    "postcurados": "postcurado",
    "postdelictualidades": "postdelictualidad",
    "postdiluviana": "postdiluviano",
    "postdiluvianas": "postdiluviano",
    "postdiluvianos": "postdiluviano",
    "postdoctorales": "postdoctoral",
    "postdorsales": "postdorsal",
    "postecualizaciones": "postecualización",
    "postediciones": "postedición",
    "posteditora": "posteditor",
    "posteditoras": "posteditor",
    "posteditores": "posteditor",
    "posteleros": "postelero",
    "postemas": "postema",
    "postemeros": "postemero",
    "posterga": "postergar",
    "postergaba": "postergar",
    "postergabais": "postergar",
    "postergaban": "postergar",
    "postergabas": "postergar",
    "postergaciones": "postergación",
    "postergad": "postergar",
    "postergada": "postergar",
    "postergadas": "postergar",
    "postergado": "postergar",
    "postergados": "postergar",
    "postergamos": "postergar",
    "postergan": "postergar",
    "postergando": "postergar",
    "postergara": "postergar",
    "postergarais": "postergar",
    "postergaran": "postergar",
    "postergaras": "postergar",
    "postergare": "postergar",
    "postergareis": "postergar",
    "postergaremos": "postergar",
    "postergaren": "postergar",
    "postergares": "postergar",
    "postergaron": "postergar",
    "postergará": "postergar",
    "postergarán": "postergar",
    "postergarás": "postergar",
    "postergaré": "postergar",
    "postergaréis": "postergar",
    "postergaría": "postergar",
    "postergaríais": "postergar",
    "postergaríamos": "postergar",
    "postergarían": "postergar",
    "postergarías": "postergar",
    "postergas": "postergar",
    "postergase": "postergar",
    "postergaseis": "postergar",
    "postergasen": "postergar",
    "postergases": "postergar",
    "postergaste": "postergar",
    "postergasteis": "postergar",
    "postergo": "postergar",
    "postergue": "postergar",
    "posterguemos": "postergar",
    "posterguen": "postergar",
    "postergues": "postergar",
    "postergué": "postergar",
    "posterguéis": "postergar",
    "postergábamos": "postergar",
    "postergáis": "postergar",
    "postergáramos": "postergar",
    "postergáremos": "postergar",
    "postergásemos": "postergar",
    "postergó": "postergar",
    "posteridades": "posteridad",
    "posteriores": "posterior",
    "posterioridades": "posterioridad",
    "posterolaterales": "posterolateral",
    "posterotemporales": "posterotemporal",
    "postes": "poste",
    "postesa": "postesar",
    "postesaba": "postesar",
    "postesabais": "postesar",
    "postesaban": "postesar",
    "postesabas": "postesar",
    "postesad": "postesar",
    "postesada": "postesar",
    "postesadas": "postesar",
    "postesado": "postesar",
    "postesados": "postesar",
    "postesamos": "postesar",
    "postesan": "postesar",
    "postesando": "postesar",
    "postesara": "postesar",
    "postesarais": "postesar",
    "postesaran": "postesar",
    "postesaras": "postesar",
    "postesare": "postesar",
    "postesareis": "postesar",
    "postesaremos": "postesar",
    "postesaren": "postesar",
    "postesares": "postesar",
    "postesaron": "postesar",
    "postesará": "postesar",
    "postesarán": "postesar",
    "postesarás": "postesar",
    "postesaré": "postesar",
    "postesaréis": "postesar",
    "postesaría": "postesar",
    "postesaríais": "postesar",
    "postesaríamos": "postesar",
    "postesarían": "postesar",
    "postesarías": "postesar",
    "postesas": "postesar",
    "postesase": "postesar",
    "postesaseis": "postesar",
    "postesasen": "postesar",
    "postesases": "postesar",
    "postesaste": "postesar",
    "postesasteis": "postesar",
    "postese": "postesar",
    "postesemos": "postesar",
    "postesen": "postesar",
    "posteses": "postesar",
    "postesfenoidales": "postesfenoidal",
    "posteso": "postesar",
    "postesábamos": "postesar",
    "postesáis": "postesar",
    "postesáramos": "postesar",
    "postesáremos": "postesar",
    "postesásemos": "postesar",
    "postesé": "postesar",
    "posteséis": "postesar",
    "postesó": "postesar",
    "postetas": "posteta",
    "postfacios": "postfacio",
    "postfechada": "postfechado",
    "postfechadas": "postfechado",
    "postfechados": "postfechado",
    "postfija": "postfijo",
    "postfijas": "postfijo",
    "postfijos": "postfijo",
    "postformados": "postformado",
    "postgrados": "postgrado",
    "postguerras": "postguerra",
    "posthemorrágica": "posthemorrágico",
    "posthemorrágicas": "posthemorrágico",
    "posthemorrágicos": "posthemorrágico",
    "postigos": "postigo",
    "postillas": "postilla",
    "postillones": "postillón",
    "postillosa": "postilloso",
    "postillosas": "postilloso",
    "postillosos": "postilloso",
    "postimpresiones": "postimpresión",
    "postimpresionismos": "postimpresionismo",
    "postimpresionistas": "postimpresionista",
    "postincendios": "postincendio",
    "postindustriales": "postindustrial",
    "postinea": "postinear",
    "postineaba": "postinear",
    "postineabais": "postinear",
    "postineaban": "postinear",
    "postineabas": "postinear",
    "postinead": "postinear",
    "postineada": "postinear",
    "postineadas": "postinear",
    "postineado": "postinear",
    "postineados": "postinear",
    "postineamos": "postinear",
    "postinean": "postinear",
    "postineando": "postinear",
    "postineara": "postinear",
    "postinearais": "postinear",
    "postinearan": "postinear",
    "postinearas": "postinear",
    "postineare": "postinear",
    "postineareis": "postinear",
    "postinearemos": "postinear",
    "postinearen": "postinear",
    "postineares": "postinear",
    "postinearon": "postinear",
    "postineará": "postinear",
    "postinearán": "postinear",
    "postinearás": "postinear",
    "postinearé": "postinear",
    "postinearéis": "postinear",
    "postinearía": "postinear",
    "postinearíais": "postinear",
    "postinearíamos": "postinear",
    "postinearían": "postinear",
    "postinearías": "postinear",
    "postineas": "postinear",
    "postinease": "postinear",
    "postineaseis": "postinear",
    "postineasen": "postinear",
    "postineases": "postinear",
    "postineaste": "postinear",
    "postineasteis": "postinear",
    "postinee": "postinear",
    "postineemos": "postinear",
    "postineen": "postinear",
    "postinees": "postinear",
    "postineo": "postinear",
    "postinera": "postinero",
    "postineras": "postinero",
    "postineros": "postinero",
    "postines": "postín",
    "postineábamos": "postinear",
    "postineáis": "postinear",
    "postineáramos": "postinear",
    "postineáremos": "postinear",
    "postineásemos": "postinear",
    "postineé": "postinear",
    "postineéis": "postinear",
    "postineó": "postinear",
    "postiza": "postizo",
    "postizas": "postizo",
    "postizos": "postizo",
    "postlinguales": "postlingual",
    "postmeridiana": "postmeridiano",
    "postmeridianas": "postmeridiano",
    "postmeridianos": "postmeridiano",
    "postmodernismos": "postmodernismo",
    "postnupciales": "postnupcial",
    "postobligatoria": "postobligatorio",
    "postobligatorias": "postobligatorio",
    "postobligatorios": "postobligatorio",
    "postoperatoria": "postoperatorio",
    "postoperatorias": "postoperatorio",
    "postoperatorios": "postoperatorio",
    "postora": "postor",
    "postoras": "postor",
    "postores": "postor",
    "postpalatales": "postpalatal",
    "postpaíses": "postpaís",
    "postprocesadores": "postprocesador",
    "postprocesos": "postproceso",
    "postprocesuales": "postprocesual",
    "postproducciones": "postproducción",
    "postra": "postrar",
    "postraba": "postrar",
    "postrabais": "postrar",
    "postraban": "postrar",
    "postrabas": "postrar",
    "postraciones": "postración",
    "postrad": "postrar",
    "postrada": "postrar",
    "postradas": "postrar",
    "postrado": "postrar",
    "postradora": "postrador",
    "postradoras": "postrador",
    "postradores": "postrador",
    "postrados": "postrar",
    "postramos": "postrar",
    "postran": "postrar",
    "postrando": "postrar",
    "postransfusionales": "postransfusional",
    "postrara": "postrar",
    "postrarais": "postrar",
    "postraran": "postrar",
    "postraras": "postrar",
    "postrare": "postrar",
    "postrareis": "postrar",
    "postraremos": "postrar",
    "postraren": "postrar",
    "postrares": "postrar",
    "postraron": "postrar",
    "postrará": "postrar",
    "postrarán": "postrar",
    "postrarás": "postrar",
    "postraré": "postrar",
    "postraréis": "postrar",
    "postraría": "postrar",
    "postraríais": "postrar",
    "postraríamos": "postrar",
    "postrarían": "postrar",
    "postrarías": "postrar",
    "postras": "postrar",
    "postrase": "postrar",
    "postraseis": "postrar",
    "postrasen": "postrar",
    "postrases": "postrar",
    "postraste": "postrar",
    "postrasteis": "postrar",
    "postre": "postrar",
    "postrecolecciones": "postrecolección",
    "postrema": "postremo",
    "postremas": "postremo",
    "postremos": "postremo",
    "postren": "postrar",
    "postrera": "postrero",
    "postreras": "postrero",
    "postreros": "postrero",
    "postres": "postre",
    "postrimera": "postrimero",
    "postrimeras": "postrimero",
    "postrimeros": "postrimero",
    "postrimerías": "postrimería",
    "postro": "postrar",
    "postrábamos": "postrar",
    "postráis": "postrar",
    "postráramos": "postrar",
    "postráremos": "postrar",
    "postrásemos": "postrar",
    "postré": "postrar",
    "postréis": "postrar",
    "postró": "postrar",
    "postsecundaria": "postsecundario",
    "postsecundarias": "postsecundario",
    "postsecundarios": "postsecundario",
    "postsincronizaciones": "postsincronización",
    "postula": "postular",
    "postulaba": "postular",
    "postulabais": "postular",
    "postulaban": "postular",
    "postulabas": "postular",
    "postulables": "postulable",
    "postulaciones": "postulación",
    "postulad": "postular",
    "postulada": "postular",
    "postuladas": "postular",
    "postulado": "postular",
    "postuladora": "postulador",
    "postuladoras": "postulador",
    "postuladores": "postulador",
    "postulados": "postular",
    "postulamos": "postular",
    "postulan": "postular",
    "postulando": "postular",
    "postulanta": "postulante",
    "postulantas": "postulante",
    "postulantes": "postulante",
    "postulara": "postular",
    "postularais": "postular",
    "postularan": "postular",
    "postularas": "postular",
    "postulare": "postular",
    "postulareis": "postular",
    "postularemos": "postular",
    "postularen": "postular",
    "postulares": "postular",
    "postularon": "postular",
    "postulará": "postular",
    "postularán": "postular",
    "postularás": "postular",
    "postularé": "postular",
    "postularéis": "postular",
    "postularía": "postular",
    "postularíais": "postular",
    "postularíamos": "postular",
    "postularían": "postular",
    "postularías": "postular",
    "postulas": "postular",
    "postulase": "postular",
    "postulaseis": "postular",
    "postulasen": "postular",
    "postulases": "postular",
    "postulaste": "postular",
    "postulasteis": "postular",
    "postule": "postular",
    "postulemos": "postular",
    "postulen": "postular",
    "postules": "postular",
    "postulo": "postular",
    "postulábamos": "postular",
    "postuláis": "postular",
    "postuláramos": "postular",
    "postuláremos": "postular",
    "postulásemos": "postular",
    "postulé": "postular",
    "postuléis": "postular",
    "postuló": "postular",
    "posturales": "postural",
    "posturas": "postura",
    "postverbales": "postverbal",
    "postámbulos": "postámbulo",
    "postónica": "postónico",
    "postónicas": "postónico",
    "postónicos": "postónico",
    "posverbales": "posverbal",
    "posábamos": "posar",
    "posáis": "posar",
    "posáramos": "posar",
    "posáremos": "posar",
    "posásemos": "posar",
    "posé": "posar",
    "poséis": "posar",
    "posó": "posar",
    "potabilice": "potabilizar",
    "potabilicemos": "potabilizar",
    "potabilicen": "potabilizar",
    "potabilices": "potabilizar",
    "potabilicé": "potabilizar",
    "potabilicéis": "potabilizar",
    "potabilidades": "potabilidad",
    "potabiliza": "potabilizar",
    "potabilizaba": "potabilizar",
    "potabilizabais": "potabilizar",
    "potabilizaban": "potabilizar",
    "potabilizabas": "potabilizar",
    "potabilizaciones": "potabilización",
    "potabilizad": "potabilizar",
    "potabilizada": "potabilizar",
    "potabilizadas": "potabilizar",
    "potabilizado": "potabilizar",
    "potabilizados": "potabilizar",
    "potabilizamos": "potabilizar",
    "potabilizan": "potabilizar",
    "potabilizando": "potabilizar",
    "potabilizara": "potabilizar",
    "potabilizarais": "potabilizar",
    "potabilizaran": "potabilizar",
    "potabilizaras": "potabilizar",
    "potabilizare": "potabilizar",
    "potabilizareis": "potabilizar",
    "potabilizaremos": "potabilizar",
    "potabilizaren": "potabilizar",
    "potabilizares": "potabilizar",
    "potabilizaron": "potabilizar",
    "potabilizará": "potabilizar",
    "potabilizarán": "potabilizar",
    "potabilizarás": "potabilizar",
    "potabilizaré": "potabilizar",
    "potabilizaréis": "potabilizar",
    "potabilizaría": "potabilizar",
    "potabilizaríais": "potabilizar",
    "potabilizaríamos": "potabilizar",
    "potabilizarían": "potabilizar",
    "potabilizarías": "potabilizar",
    "potabilizas": "potabilizar",
    "potabilizase": "potabilizar",
    "potabilizaseis": "potabilizar",
    "potabilizasen": "potabilizar",
    "potabilizases": "potabilizar",
    "potabilizaste": "potabilizar",
    "potabilizasteis": "potabilizar",
    "potabilizo": "potabilizar",
    "potabilizábamos": "potabilizar",
    "potabilizáis": "potabilizar",
    "potabilizáramos": "potabilizar",
    "potabilizáremos": "potabilizar",
    "potabilizásemos": "potabilizar",
    "potabilizó": "potabilizar",
    "potables": "potable",
    "potaciones": "potación",
    "potada": "potado",
    "potadas": "potado",
    "potadora": "potador",
    "potadoras": "potador",
    "potadores": "potador",
    "potados": "potado",
    "potajerías": "potajería",
    "potajes": "potaje",
    "potalas": "potala",
    "potamologías": "potamología",
    "potas": "pota",
    "potasas": "potasa",
    "potasios": "potasio",
    "potencia": "potenciar",
    "potenciaba": "potenciar",
    "potenciabais": "potenciar",
    "potenciaban": "potenciar",
    "potenciabas": "potenciar",
    "potenciaciones": "potenciación",
    "potenciad": "potenciar",
    "potenciada": "potenciar",
    "potenciadas": "potenciar",
    "potenciado": "potenciar",
    "potenciadores": "potenciador",
    "potenciados": "potenciar",
    "potenciales": "potencial",
    "potencialidades": "potencialidad",
    "potenciamos": "potenciar",
    "potencian": "potenciar",
    "potenciando": "potenciar",
    "potenciara": "potenciar",
    "potenciarais": "potenciar",
    "potenciaran": "potenciar",
    "potenciaras": "potenciar",
    "potenciare": "potenciar",
    "potenciareis": "potenciar",
    "potenciaremos": "potenciar",
    "potenciaren": "potenciar",
    "potenciares": "potenciar",
    "potenciaron": "potenciar",
    "potenciará": "potenciar",
    "potenciarán": "potenciar",
    "potenciarás": "potenciar",
    "potenciaré": "potenciar",
    "potenciaréis": "potenciar",
    "potenciaría": "potenciar",
    "potenciaríais": "potenciar",
    "potenciaríamos": "potenciar",
    "potenciarían": "potenciar",
    "potenciarías": "potenciar",
    "potencias": "potenciar",
    "potenciase": "potenciar",
    "potenciaseis": "potenciar",
    "potenciasen": "potenciar",
    "potenciases": "potenciar",
    "potenciaste": "potenciar",
    "potenciasteis": "potenciar",
    "potencie": "potenciar",
    "potenciemos": "potenciar",
    "potencien": "potenciar",
    "potencies": "potenciar",
    "potencio": "potenciar",
    "potenciometrías": "potenciometría",
    "potenciábamos": "potenciar",
    "potenciáis": "potenciar",
    "potenciáramos": "potenciar",
    "potenciáremos": "potenciar",
    "potenciásemos": "potenciar",
    "potencié": "potenciar",
    "potenciéis": "potenciar",
    "potenció": "potenciar",
    "potenciómetros": "potenciómetro",
    "potentada": "potentado",
    "potentadas": "potentado",
    "potentados": "potentado",
    "potentes": "potente",
    "potenzas": "potenza",
    "poteras": "potera",
    "poternas": "poterna",
    "potes": "pote",
    "potestades": "potestad",
    "potestativa": "potestativo",
    "potestativas": "potestativo",
    "potestativos": "potestativo",
    "potingues": "potingue",
    "potos": "poto",
    "potosina": "potosino",
    "potosinas": "potosino",
    "potosinos": "potosino",
    "potosíes": "potosí",
    "potosís": "potosí",
    "potradas": "potrada",
    "potranca": "potranco",
    "potrancas": "potranco",
    "potrancos": "potranco",
    "potras": "potra",
    "potrea": "potrear",
    "potreaba": "potrear",
    "potreabais": "potrear",
    "potreaban": "potrear",
    "potreabas": "potrear",
    "potread": "potrear",
    "potreada": "potrear",
    "potreadas": "potrear",
    "potreado": "potrear",
    "potreados": "potrear",
    "potreamos": "potrear",
    "potrean": "potrear",
    "potreando": "potrear",
    "potreara": "potrear",
    "potrearais": "potrear",
    "potrearan": "potrear",
    "potrearas": "potrear",
    "potreare": "potrear",
    "potreareis": "potrear",
    "potrearemos": "potrear",
    "potrearen": "potrear",
    "potreares": "potrear",
    "potrearon": "potrear",
    "potreará": "potrear",
    "potrearán": "potrear",
    "potrearás": "potrear",
    "potrearé": "potrear",
    "potrearéis": "potrear",
    "potrearía": "potrear",
    "potrearíais": "potrear",
    "potrearíamos": "potrear",
    "potrearían": "potrear",
    "potrearías": "potrear",
    "potreas": "potrear",
    "potrease": "potrear",
    "potreaseis": "potrear",
    "potreasen": "potrear",
    "potreases": "potrear",
    "potreaste": "potrear",
    "potreasteis": "potrear",
    "potree": "potrear",
    "potreemos": "potrear",
    "potreen": "potrear",
    "potrees": "potrear",
    "potreo": "potrear",
    "potrera": "potrero",
    "potreras": "potrero",
    "potreros": "potrero",
    "potreábamos": "potrear",
    "potreáis": "potrear",
    "potreáramos": "potrear",
    "potreáremos": "potrear",
    "potreásemos": "potrear",
    "potreé": "potrear",
    "potreéis": "potrear",
    "potreó": "potrear",
    "potriles": "potril",
    "potrillas": "potrilla",
    "potrillos": "potrillo",
    "potros": "potro",
    "potrosa": "potroso",
    "potrosas": "potroso",
    "potrosos": "potroso",
    "potámides": "potámide",
    "potásica": "potásico",
    "potásicas": "potásico",
    "potásicos": "potásico",
    "poules": "poule",
    "povisas": "povisa",
    "powellitas": "powellita",
    "poyales": "poyal",
    "poyas": "poya",
    "poyatas": "poyata",
    "poyetes": "poyete",
    "poyos": "poyo",
    "pozales": "pozal",
    "pozancos": "pozanco",
    "pozas": "poza",
    "pozos": "pozo",
    "pozuelos": "pozuelo",
    "poética": "poético",
    "poéticas": "poético",
    "poéticos": "poético",
    "poínos": "poíno",
    "pracritos": "pracrito",
    "practica": "practicar",
    "practicaba": "practicar",
    "practicabais": "practicar",
    "practicaban": "practicar",
    "practicabas": "practicar",
    "practicables": "practicable",
    "practicad": "practicar",
    "practicada": "practicar",
    "practicadas": "practicar",
    "practicado": "practicar",
    "practicadora": "practicador",
    "practicadoras": "practicador",
    "practicadores": "practicador",
    "practicados": "practicar",
    "practicajes": "practicaje",
    "practicamos": "practicar",
    "practican": "practicar",
    "practicando": "practicar",
    "practicantas": "practicanta",
    "practicantes": "practicante",
    "practicara": "practicar",
    "practicarais": "practicar",
    "practicaran": "practicar",
    "practicaras": "practicar",
    "practicare": "practicar",
    "practicareis": "practicar",
    "practicaremos": "practicar",
    "practicaren": "practicar",
    "practicares": "practicar",
    "practicaron": "practicar",
    "practicará": "practicar",
    "practicarán": "practicar",
    "practicarás": "practicar",
    "practicaré": "practicar",
    "practicaréis": "practicar",
    "practicaría": "practicar",
    "practicaríais": "practicar",
    "practicaríamos": "practicar",
    "practicarían": "practicar",
    "practicarías": "practicar",
    "practicas": "practicar",
    "practicase": "practicar",
    "practicaseis": "practicar",
    "practicasen": "practicar",
    "practicases": "practicar",
    "practicaste": "practicar",
    "practicasteis": "practicar",
    "practico": "practicar",
    "practicona": "practicón",
    "practiconas": "practicón",
    "practicones": "practicón",
    "practicábamos": "practicar",
    "practicáis": "practicar",
    "practicáramos": "practicar",
    "practicáremos": "practicar",
    "practicásemos": "practicar",
    "practicó": "practicar",
    "practique": "practicar",
    "practiquemos": "practicar",
    "practiquen": "practicar",
    "practiques": "practicar",
    "practiqué": "practicar",
    "practiquéis": "practicar",
    "pradales": "pradal",
    "praderas": "pradera",
    "praderías": "pradería",
    "pradiales": "pradial",
    "prados": "prado",
    "pragmatismos": "pragmatismo",
    "pragmatistas": "pragmatista",
    "pragmática": "pragmático",
    "pragmáticas": "pragmático",
    "pragmáticos": "pragmático",
    "pralinajes": "pralinaje",
    "pralinées": "praliné",
    "pralinés": "praliné",
    "praos": "prao",
    "praseodimios": "praseodimio",
    "prasiolitas": "prasiolita",
    "prasios": "prasio",
    "prasmas": "prasma",
    "pratenses": "pratense",
    "praticultora": "praticultor",
    "praticultoras": "praticultor",
    "praticultores": "praticultor",
    "praticulturas": "praticultura",
    "prava": "pravo",
    "pravas": "pravo",
    "pravedades": "pravedad",
    "pravianas": "praviana",
    "pravos": "pravo",
    "praxeologías": "praxeología",
    "prazicuanteles": "prazicuantel",
    "preacentuaciones": "preacentuación",
    "preadamitas": "preadamita",
    "preadamítica": "preadamítico",
    "preadamíticas": "preadamítico",
    "preadamíticos": "preadamítico",
    "preajustada": "preajustado",
    "preajustadas": "preajustado",
    "preajustados": "preajustado",
    "prealimentaciones": "prealimentación",
    "prealmacenamientos": "prealmacenamiento",
    "preamplificadores": "preamplificador",
    "preanestesias": "preanestesia",
    "preanuncios": "preanuncio",
    "preanómenes": "preanomen",
    "prearmada": "prearmado",
    "prearmadas": "prearmado",
    "prearmados": "prearmado",
    "preauriculares": "preauricular",
    "preavisos": "preaviso",
    "prebenda": "prebendar",
    "prebendaba": "prebendar",
    "prebendabais": "prebendar",
    "prebendaban": "prebendar",
    "prebendabas": "prebendar",
    "prebendad": "prebendar",
    "prebendada": "prebendar",
    "prebendadas": "prebendar",
    "prebendado": "prebendar",
    "prebendados": "prebendar",
    "prebendamos": "prebendar",
    "prebendan": "prebendar",
    "prebendando": "prebendar",
    "prebendara": "prebendar",
    "prebendarais": "prebendar",
    "prebendaran": "prebendar",
    "prebendaras": "prebendar",
    "prebendare": "prebendar",
    "prebendareis": "prebendar",
    "prebendaremos": "prebendar",
    "prebendaren": "prebendar",
    "prebendares": "prebendar",
    "prebendaron": "prebendar",
    "prebendará": "prebendar",
    "prebendarán": "prebendar",
    "prebendarás": "prebendar",
    "prebendaré": "prebendar",
    "prebendaréis": "prebendar",
    "prebendaría": "prebendar",
    "prebendaríais": "prebendar",
    "prebendaríamos": "prebendar",
    "prebendarían": "prebendar",
    "prebendarías": "prebendar",
    "prebendas": "prebendar",
    "prebendase": "prebendar",
    "prebendaseis": "prebendar",
    "prebendasen": "prebendar",
    "prebendases": "prebendar",
    "prebendaste": "prebendar",
    "prebendasteis": "prebendar",
    "prebende": "prebendar",
    "prebendemos": "prebendar",
    "prebenden": "prebendar",
    "prebendes": "prebendar",
    "prebendo": "prebendar",
    "prebendábamos": "prebendar",
    "prebendáis": "prebendar",
    "prebendáramos": "prebendar",
    "prebendáremos": "prebendar",
    "prebendásemos": "prebendar",
    "prebendé": "prebendar",
    "prebendéis": "prebendar",
    "prebendó": "prebendar",
    "prebostazgos": "prebostazgo",
    "prebostes": "preboste",
    "precableados": "precableado",
    "precalefacciones": "precalefacción",
    "precalentaba": "precalentar",
    "precalentabais": "precalentar",
    "precalentaban": "precalentar",
    "precalentabas": "precalentar",
    "precalentad": "precalentar",
    "precalentada": "precalentar",
    "precalentadas": "precalentar",
    "precalentado": "precalentar",
    "precalentadores": "precalentador",
    "precalentados": "precalentar",
    "precalentamientos": "precalentamiento",
    "precalentamos": "precalentar",
    "precalentando": "precalentar",
    "precalentara": "precalentar",
    "precalentarais": "precalentar",
    "precalentaran": "precalentar",
    "precalentaras": "precalentar",
    "precalentare": "precalentar",
    "precalentareis": "precalentar",
    "precalentaremos": "precalentar",
    "precalentaren": "precalentar",
    "precalentares": "precalentar",
    "precalentaron": "precalentar",
    "precalentará": "precalentar",
    "precalentarán": "precalentar",
    "precalentarás": "precalentar",
    "precalentaré": "precalentar",
    "precalentaréis": "precalentar",
    "precalentaría": "precalentar",
    "precalentaríais": "precalentar",
    "precalentaríamos": "precalentar",
    "precalentarían": "precalentar",
    "precalentarías": "precalentar",
    "precalentase": "precalentar",
    "precalentaseis": "precalentar",
    "precalentasen": "precalentar",
    "precalentases": "precalentar",
    "precalentaste": "precalentar",
    "precalentasteis": "precalentar",
    "precalentemos": "precalentar",
    "precalentábamos": "precalentar",
    "precalentáis": "precalentar",
    "precalentáramos": "precalentar",
    "precalentáremos": "precalentar",
    "precalentásemos": "precalentar",
    "precalenté": "precalentar",
    "precalentéis": "precalentar",
    "precalentó": "precalentar",
    "precalienta": "precalentar",
    "precalientan": "precalentar",
    "precalientas": "precalentar",
    "precaliente": "precalentar",
    "precalienten": "precalentar",
    "precalientes": "precalentar",
    "precaliento": "precalentar",
    "precapitalismos": "precapitalismo",
    "precapitalistas": "precapitalista",
    "precargas": "precarga",
    "precaria": "precario",
    "precarias": "precario",
    "precariedades": "precariedad",
    "precarios": "precario",
    "precaristas": "precarista",
    "precarizaciones": "precarización",
    "precauciona": "precaucionar",
    "precaucionaba": "precaucionar",
    "precaucionabais": "precaucionar",
    "precaucionaban": "precaucionar",
    "precaucionabas": "precaucionar",
    "precaucionad": "precaucionar",
    "precaucionada": "precaucionar",
    "precaucionadas": "precaucionar",
    "precaucionado": "precaucionar",
    "precaucionados": "precaucionar",
    "precaucionamos": "precaucionar",
    "precaucionan": "precaucionar",
    "precaucionando": "precaucionar",
    "precaucionara": "precaucionar",
    "precaucionarais": "precaucionar",
    "precaucionaran": "precaucionar",
    "precaucionaras": "precaucionar",
    "precaucionare": "precaucionar",
    "precaucionareis": "precaucionar",
    "precaucionaremos": "precaucionar",
    "precaucionaren": "precaucionar",
    "precaucionares": "precaucionar",
    "precaucionaron": "precaucionar",
    "precaucionará": "precaucionar",
    "precaucionarán": "precaucionar",
    "precaucionarás": "precaucionar",
    "precaucionaré": "precaucionar",
    "precaucionaréis": "precaucionar",
    "precaucionaría": "precaucionar",
    "precaucionaríais": "precaucionar",
    "precaucionaríamos": "precaucionar",
    "precaucionarían": "precaucionar",
    "precaucionarías": "precaucionar",
    "precaucionas": "precaucionar",
    "precaucionase": "precaucionar",
    "precaucionaseis": "precaucionar",
    "precaucionasen": "precaucionar",
    "precaucionases": "precaucionar",
    "precaucionaste": "precaucionar",
    "precaucionasteis": "precaucionar",
    "precaucione": "precaucionar",
    "precaucionemos": "precaucionar",
    "precaucionen": "precaucionar",
    "precauciones": "precaucionar",
    "precauciono": "precaucionar",
    "precaucionábamos": "precaucionar",
    "precaucionáis": "precaucionar",
    "precaucionáramos": "precaucionar",
    "precaucionáremos": "precaucionar",
    "precaucionásemos": "precaucionar",
    "precaucioné": "precaucionar",
    "precaucionéis": "precaucionar",
    "precaucionó": "precaucionar",
    "precautela": "precautelar",
    "precautelaba": "precautelar",
    "precautelabais": "precautelar",
    "precautelaban": "precautelar",
    "precautelabas": "precautelar",
    "precautelad": "precautelar",
    "precautelada": "precautelar",
    "precauteladas": "precautelar",
    "precautelado": "precautelar",
    "precautelados": "precautelar",
    "precautelamos": "precautelar",
    "precautelan": "precautelar",
    "precautelando": "precautelar",
    "precautelara": "precautelar",
    "precautelarais": "precautelar",
    "precautelaran": "precautelar",
    "precautelaras": "precautelar",
    "precautelare": "precautelar",
    "precautelareis": "precautelar",
    "precautelaremos": "precautelar",
    "precautelaren": "precautelar",
    "precautelares": "precautelar",
    "precautelaron": "precautelar",
    "precautelará": "precautelar",
    "precautelarán": "precautelar",
    "precautelarás": "precautelar",
    "precautelaré": "precautelar",
    "precautelaréis": "precautelar",
    "precautelaría": "precautelar",
    "precautelaríais": "precautelar",
    "precautelaríamos": "precautelar",
    "precautelarían": "precautelar",
    "precautelarías": "precautelar",
    "precautelas": "precautelar",
    "precautelase": "precautelar",
    "precautelaseis": "precautelar",
    "precautelasen": "precautelar",
    "precautelases": "precautelar",
    "precautelaste": "precautelar",
    "precautelasteis": "precautelar",
    "precautele": "precautelar",
    "precautelemos": "precautelar",
    "precautelen": "precautelar",
    "precauteles": "precautelar",
    "precautelo": "precautelar",
    "precautelábamos": "precautelar",
    "precauteláis": "precautelar",
    "precauteláramos": "precautelar",
    "precauteláremos": "precautelar",
    "precautelásemos": "precautelar",
    "precautelé": "precautelar",
    "precauteléis": "precautelar",
    "precauteló": "precautelar",
    "precautoria": "precautorio",
    "precautorias": "precautorio",
    "precautorios": "precautorio",
    "precava": "precaver",
    "precavamos": "precaver",
    "precavan": "precaver",
    "precavas": "precaver",
    "precave": "precaver",
    "precaved": "precaver",
    "precavemos": "precaver",
    "precaven": "precaver",
    "precaveremos": "precaver",
    "precaverá": "precaver",
    "precaverán": "precaver",
    "precaverás": "precaver",
    "precaveré": "precaver",
    "precaveréis": "precaver",
    "precavería": "precaver",
    "precaveríais": "precaver",
    "precaveríamos": "precaver",
    "precaverían": "precaver",
    "precaverías": "precaver",
    "precaves": "precaver",
    "precavida": "precaver",
    "precavidas": "precaver",
    "precavido": "precaver",
    "precavidos": "precaver",
    "precaviendo": "precaver",
    "precaviera": "precaver",
    "precavierais": "precaver",
    "precavieran": "precaver",
    "precavieras": "precaver",
    "precaviere": "precaver",
    "precaviereis": "precaver",
    "precavieren": "precaver",
    "precavieres": "precaver",
    "precavieron": "precaver",
    "precaviese": "precaver",
    "precavieseis": "precaver",
    "precaviesen": "precaver",
    "precavieses": "precaver",
    "precavimos": "precaver",
    "precaviste": "precaver",
    "precavisteis": "precaver",
    "precaviéramos": "precaver",
    "precaviéremos": "precaver",
    "precaviésemos": "precaver",
    "precavió": "precaver",
    "precavo": "precaver",
    "precaváis": "precaver",
    "precavéis": "precaver",
    "precaví": "precaver",
    "precavía": "precaver",
    "precavíais": "precaver",
    "precavíamos": "precaver",
    "precavían": "precaver",
    "precavías": "precaver",
    "preceda": "preceder",
    "precedamos": "preceder",
    "precedan": "preceder",
    "precedas": "preceder",
    "precede": "preceder",
    "preceded": "preceder",
    "precedemos": "preceder",
    "preceden": "preceder",
    "precedencias": "precedencia",
    "precedentes": "precedente",
    "precederemos": "preceder",
    "precederá": "preceder",
    "precederán": "preceder",
    "precederás": "preceder",
    "precederé": "preceder",
    "precederéis": "preceder",
    "precedería": "preceder",
    "precederíais": "preceder",
    "precederíamos": "preceder",
    "precederían": "preceder",
    "precederías": "preceder",
    "precedes": "preceder",
    "precedida": "preceder",
    "precedidas": "preceder",
    "precedido": "preceder",
    "precedidos": "preceder",
    "precediendo": "preceder",
    "precediera": "preceder",
    "precedierais": "preceder",
    "precedieran": "preceder",
    "precedieras": "preceder",
    "precediere": "preceder",
    "precediereis": "preceder",
    "precedieren": "preceder",
    "precedieres": "preceder",
    "precedieron": "preceder",
    "precediese": "preceder",
    "precedieseis": "preceder",
    "precediesen": "preceder",
    "precedieses": "preceder",
    "precedimos": "preceder",
    "precediste": "preceder",
    "precedisteis": "preceder",
    "precediéramos": "preceder",
    "precediéremos": "preceder",
    "precediésemos": "preceder",
    "precedió": "preceder",
    "precedo": "preceder",
    "precedáis": "preceder",
    "precedéis": "preceder",
    "precedí": "preceder",
    "precedía": "preceder",
    "precedíais": "preceder",
    "precedíamos": "preceder",
    "precedían": "preceder",
    "precedías": "preceder",
    "precentrales": "precentral",
    "preceptistas": "preceptista",
    "preceptiva": "preceptivo",
    "preceptivas": "preceptivo",
    "preceptivos": "preceptivo",
    "preceptora": "preceptor",
    "preceptorados": "preceptorado",
    "preceptorales": "preceptoral",
    "preceptoras": "preceptor",
    "preceptores": "preceptor",
    "preceptoriles": "preceptoril",
    "preceptos": "precepto",
    "preceptuaba": "preceptuar",
    "preceptuabais": "preceptuar",
    "preceptuaban": "preceptuar",
    "preceptuabas": "preceptuar",
    "preceptuad": "preceptuar",
    "preceptuada": "preceptuar",
    "preceptuadas": "preceptuar",
    "preceptuado": "preceptuar",
    "preceptuados": "preceptuar",
    "preceptuamos": "preceptuar",
    "preceptuando": "preceptuar",
    "preceptuara": "preceptuar",
    "preceptuarais": "preceptuar",
    "preceptuaran": "preceptuar",
    "preceptuaras": "preceptuar",
    "preceptuare": "preceptuar",
    "preceptuareis": "preceptuar",
    "preceptuaremos": "preceptuar",
    "preceptuaren": "preceptuar",
    "preceptuares": "preceptuar",
    "preceptuaron": "preceptuar",
    "preceptuará": "preceptuar",
    "preceptuarán": "preceptuar",
    "preceptuarás": "preceptuar",
    "preceptuaré": "preceptuar",
    "preceptuaréis": "preceptuar",
    "preceptuaría": "preceptuar",
    "preceptuaríais": "preceptuar",
    "preceptuaríamos": "preceptuar",
    "preceptuarían": "preceptuar",
    "preceptuarías": "preceptuar",
    "preceptuase": "preceptuar",
    "preceptuaseis": "preceptuar",
    "preceptuasen": "preceptuar",
    "preceptuases": "preceptuar",
    "preceptuaste": "preceptuar",
    "preceptuasteis": "preceptuar",
    "preceptuemos": "preceptuar",
    "preceptuábamos": "preceptuar",
    "preceptuáis": "preceptuar",
    "preceptuáramos": "preceptuar",
    "preceptuáremos": "preceptuar",
    "preceptuásemos": "preceptuar",
    "preceptué": "preceptuar",
    "preceptuéis": "preceptuar",
    "preceptuó": "preceptuar",
    "preceptúa": "preceptuar",
    "preceptúan": "preceptuar",
    "preceptúas": "preceptuar",
    "preceptúe": "preceptuar",
    "preceptúen": "preceptuar",
    "preceptúes": "preceptuar",
    "preceptúo": "preceptuar",
    "preces": "prez",
    "precesiones": "precesión",
    "precia": "preciar",
    "preciaba": "preciar",
    "preciabais": "preciar",
    "preciaban": "preciar",
    "preciabas": "preciar",
    "preciad": "preciar",
    "preciada": "preciar",
    "preciadas": "preciar",
    "preciado": "preciar",
    "preciadora": "preciador",
    "preciadoras": "preciador",
    "preciadores": "preciador",
    "preciados": "preciar",
    "preciamos": "preciar",
    "precian": "preciar",
    "preciando": "preciar",
    "preciara": "preciar",
    "preciarais": "preciar",
    "preciaran": "preciar",
    "preciaras": "preciar",
    "preciare": "preciar",
    "preciareis": "preciar",
    "preciaremos": "preciar",
    "preciaren": "preciar",
    "preciares": "preciar",
    "preciaron": "preciar",
    "preciará": "preciar",
    "preciarán": "preciar",
    "preciarás": "preciar",
    "preciaré": "preciar",
    "preciaréis": "preciar",
    "preciaría": "preciar",
    "preciaríais": "preciar",
    "preciaríamos": "preciar",
    "preciarían": "preciar",
    "preciarías": "preciar",
    "precias": "preciar",
    "preciase": "preciar",
    "preciaseis": "preciar",
    "preciasen": "preciar",
    "preciases": "preciar",
    "preciaste": "preciar",
    "preciasteis": "preciar",
    "precie": "preciar",
    "preciemos": "preciar",
    "precien": "preciar",
    "precies": "preciar",
    "precinta": "precintar",
    "precintaba": "precintar",
    "precintabais": "precintar",
    "precintaban": "precintar",
    "precintabas": "precintar",
    "precintad": "precintar",
    "precintada": "precintar",
    "precintadas": "precintar",
    "precintado": "precintar",
    "precintados": "precintar",
    "precintamos": "precintar",
    "precintan": "precintar",
    "precintando": "precintar",
    "precintara": "precintar",
    "precintarais": "precintar",
    "precintaran": "precintar",
    "precintaras": "precintar",
    "precintare": "precintar",
    "precintareis": "precintar",
    "precintaremos": "precintar",
    "precintaren": "precintar",
    "precintares": "precintar",
    "precintaron": "precintar",
    "precintará": "precintar",
    "precintarán": "precintar",
    "precintarás": "precintar",
    "precintaré": "precintar",
    "precintaréis": "precintar",
    "precintaría": "precintar",
    "precintaríais": "precintar",
    "precintaríamos": "precintar",
    "precintarían": "precintar",
    "precintarías": "precintar",
    "precintas": "precintar",
    "precintase": "precintar",
    "precintaseis": "precintar",
    "precintasen": "precintar",
    "precintases": "precintar",
    "precintaste": "precintar",
    "precintasteis": "precintar",
    "precinte": "precintar",
    "precintemos": "precintar",
    "precinten": "precintar",
    "precintes": "precintar",
    "precinto": "precintar",
    "precintos": "precinto",
    "precintábamos": "precintar",
    "precintáis": "precintar",
    "precintáramos": "precintar",
    "precintáremos": "precintar",
    "precintásemos": "precintar",
    "precinté": "precintar",
    "precintéis": "precintar",
    "precintó": "precintar",
    "precio": "preciar",
    "precios": "precio",
    "preciosa": "precioso",
    "preciosas": "precioso",
    "preciosidades": "preciosidad",
    "preciosismos": "preciosismo",
    "preciosistas": "preciosista",
    "preciosos": "precioso",
    "preciosuras": "preciosura",
    "precipicios": "precipicio",
    "precipita": "precipitar",
    "precipitaba": "precipitar",
    "precipitabais": "precipitar",
    "precipitaban": "precipitar",
    "precipitabas": "precipitar",
    "precipitables": "precipitable",
    "precipitaciones": "precipitación",
    "precipitad": "precipitar",
    "precipitada": "precipitar",
    "precipitadas": "precipitar",
    "precipitaderos": "precipitadero",
    "precipitado": "precipitar",
    "precipitadores": "precipitador",
    "precipitados": "precipitar",
    "precipitamos": "precipitar",
    "precipitan": "precipitar",
    "precipitando": "precipitar",
    "precipitantes": "precipitante",
    "precipitara": "precipitar",
    "precipitarais": "precipitar",
    "precipitaran": "precipitar",
    "precipitaras": "precipitar",
    "precipitare": "precipitar",
    "precipitareis": "precipitar",
    "precipitaremos": "precipitar",
    "precipitaren": "precipitar",
    "precipitares": "precipitar",
    "precipitaron": "precipitar",
    "precipitará": "precipitar",
    "precipitarán": "precipitar",
    "precipitarás": "precipitar",
    "precipitaré": "precipitar",
    "precipitaréis": "precipitar",
    "precipitaría": "precipitar",
    "precipitaríais": "precipitar",
    "precipitaríamos": "precipitar",
    "precipitarían": "precipitar",
    "precipitarías": "precipitar",
    "precipitas": "precipitar",
    "precipitase": "precipitar",
    "precipitaseis": "precipitar",
    "precipitasen": "precipitar",
    "precipitases": "precipitar",
    "precipitaste": "precipitar",
    "precipitasteis": "precipitar",
    "precipite": "precipitar",
    "precipitemos": "precipitar",
    "precipiten": "precipitar",
    "precipites": "precipitar",
    "precipitinas": "precipitina",
    "precipito": "precipitar",
    "precipitosa": "precipitoso",
    "precipitosas": "precipitoso",
    "precipitosos": "precipitoso",
    "precipitábamos": "precipitar",
    "precipitáis": "precipitar",
    "precipitáramos": "precipitar",
    "precipitáremos": "precipitar",
    "precipitásemos": "precipitar",
    "precipité": "precipitar",
    "precipitéis": "precipitar",
    "precipitó": "precipitar",
    "precipua": "precipuo",
    "precipuas": "precipuo",
    "precipuos": "precipuo",
    "precisa": "preciso",
    "precisaba": "precisar",
    "precisabais": "precisar",
    "precisaban": "precisar",
    "precisabas": "precisar",
    "precisad": "precisar",
    "precisada": "precisar",
    "precisadas": "precisar",
    "precisado": "precisar",
    "precisados": "precisar",
    "precisamos": "precisar",
    "precisan": "precisar",
    "precisando": "precisar",
    "precisara": "precisar",
    "precisarais": "precisar",
    "precisaran": "precisar",
    "precisaras": "precisar",
    "precisare": "precisar",
    "precisareis": "precisar",
    "precisaremos": "precisar",
    "precisaren": "precisar",
    "precisares": "precisar",
    "precisaron": "precisar",
    "precisará": "precisar",
    "precisarán": "precisar",
    "precisarás": "precisar",
    "precisaré": "precisar",
    "precisaréis": "precisar",
    "precisaría": "precisar",
    "precisaríais": "precisar",
    "precisaríamos": "precisar",
    "precisarían": "precisar",
    "precisarías": "precisar",
    "precisas": "preciso",
    "precisase": "precisar",
    "precisaseis": "precisar",
    "precisasen": "precisar",
    "precisases": "precisar",
    "precisaste": "precisar",
    "precisasteis": "precisar",
    "precise": "precisar",
    "precisemos": "precisar",
    "precisen": "precisar",
    "precises": "precisar",
    "precisiones": "precisión",
    "preciso": "precisar",
    "precisos": "preciso",
    "precisábamos": "precisar",
    "precisáis": "precisar",
    "precisáramos": "precisar",
    "precisáremos": "precisar",
    "precisásemos": "precisar",
    "precisé": "precisar",
    "preciséis": "precisar",
    "precisó": "precisar",
    "precita": "precito",
    "precitada": "precitado",
    "precitadas": "precitado",
    "precitados": "precitado",
    "precitas": "precito",
    "precitos": "precito",
    "preciábamos": "preciar",
    "preciáis": "preciar",
    "preciáramos": "preciar",
    "preciáremos": "preciar",
    "preciásemos": "preciar",
    "precié": "preciar",
    "preciéis": "preciar",
    "preció": "preciar",
    "preclara": "preclaro",
    "preclaras": "preclaro",
    "preclaros": "preclaro",
    "preclasificaciones": "preclasificación",
    "preclusiones": "preclusión",
    "preclásica": "preclásico",
    "preclásicas": "preclásico",
    "preclásicos": "preclásico",
    "precoces": "precoz",
    "precocidades": "precocidad",
    "precogniciones": "precognición",
    "precolombina": "precolombino",
    "precolombinas": "precolombino",
    "precolombinos": "precolombino",
    "precoloraciones": "precoloración",
    "precombustiones": "precombustión",
    "precompacta": "precompacto",
    "precompactas": "precompacto",
    "precompactos": "precompacto",
    "precompetitiva": "precompetitivo",
    "precompetitivas": "precompetitivo",
    "precompetitivos": "precompetitivo",
    "precompilaciones": "precompilación",
    "precompiladores": "precompilador",
    "precompresiones": "precompresión",
    "preconcebid": "preconcebir",
    "preconcebida": "preconcebir",
    "preconcebidas": "preconcebir",
    "preconcebido": "preconcebir",
    "preconcebidos": "preconcebir",
    "preconcebimos": "preconcebir",
    "preconcebiremos": "preconcebir",
    "preconcebirá": "preconcebir",
    "preconcebirán": "preconcebir",
    "preconcebirás": "preconcebir",
    "preconcebiré": "preconcebir",
    "preconcebiréis": "preconcebir",
    "preconcebiría": "preconcebir",
    "preconcebiríais": "preconcebir",
    "preconcebiríamos": "preconcebir",
    "preconcebirían": "preconcebir",
    "preconcebirías": "preconcebir",
    "preconcebiste": "preconcebir",
    "preconcebisteis": "preconcebir",
    "preconcebí": "preconcebir",
    "preconcebía": "preconcebir",
    "preconcebíais": "preconcebir",
    "preconcebíamos": "preconcebir",
    "preconcebían": "preconcebir",
    "preconcebías": "preconcebir",
    "preconcebís": "preconcebir",
    "preconciba": "preconcebir",
    "preconcibamos": "preconcebir",
    "preconciban": "preconcebir",
    "preconcibas": "preconcebir",
    "preconcibe": "preconcebir",
    "preconciben": "preconcebir",
    "preconcibes": "preconcebir",
    "preconcibiendo": "preconcebir",
    "preconcibiera": "preconcebir",
    "preconcibierais": "preconcebir",
    "preconcibieran": "preconcebir",
    "preconcibieras": "preconcebir",
    "preconcibiere": "preconcebir",
    "preconcibiereis": "preconcebir",
    "preconcibieren": "preconcebir",
    "preconcibieres": "preconcebir",
    "preconcibieron": "preconcebir",
    "preconcibiese": "preconcebir",
    "preconcibieseis": "preconcebir",
    "preconcibiesen": "preconcebir",
    "preconcibieses": "preconcebir",
    "preconcibiéramos": "preconcebir",
    "preconcibiéremos": "preconcebir",
    "preconcibiésemos": "preconcebir",
    "preconcibió": "preconcebir",
    "preconcibo": "preconcebir",
    "preconcibáis": "preconcebir",
    "precondiciones": "precondición",
    "precongresuales": "precongresual",
    "preconice": "preconizar",
    "preconicemos": "preconizar",
    "preconicen": "preconizar",
    "preconices": "preconizar",
    "preconicé": "preconizar",
    "preconicéis": "preconizar",
    "preconiza": "preconizar",
    "preconizaba": "preconizar",
    "preconizabais": "preconizar",
    "preconizaban": "preconizar",
    "preconizabas": "preconizar",
    "preconizables": "preconizable",
    "preconizaciones": "preconización",
    "preconizad": "preconizar",
    "preconizada": "preconizar",
    "preconizadas": "preconizar",
    "preconizado": "preconizar",
    "preconizadora": "preconizador",
    "preconizadoras": "preconizador",
    "preconizadores": "preconizador",
    "preconizados": "preconizar",
    "preconizamos": "preconizar",
    "preconizan": "preconizar",
    "preconizando": "preconizar",
    "preconizara": "preconizar",
    "preconizarais": "preconizar",
    "preconizaran": "preconizar",
    "preconizaras": "preconizar",
    "preconizare": "preconizar",
    "preconizareis": "preconizar",
    "preconizaremos": "preconizar",
    "preconizaren": "preconizar",
    "preconizares": "preconizar",
    "preconizaron": "preconizar",
    "preconizará": "preconizar",
    "preconizarán": "preconizar",
    "preconizarás": "preconizar",
    "preconizaré": "preconizar",
    "preconizaréis": "preconizar",
    "preconizaría": "preconizar",
    "preconizaríais": "preconizar",
    "preconizaríamos": "preconizar",
    "preconizarían": "preconizar",
    "preconizarías": "preconizar",
    "preconizas": "preconizar",
    "preconizase": "preconizar",
    "preconizaseis": "preconizar",
    "preconizasen": "preconizar",
    "preconizases": "preconizar",
    "preconizaste": "preconizar",
    "preconizasteis": "preconizar",
    "preconizo": "preconizar",
    "preconizábamos": "preconizar",
    "preconizáis": "preconizar",
    "preconizáramos": "preconizar",
    "preconizáremos": "preconizar",
    "preconizásemos": "preconizar",
    "preconizó": "preconizar",
    "preconoce": "preconocer",
    "preconoced": "preconocer",
    "preconocemos": "preconocer",
    "preconocen": "preconocer",
    "preconoceremos": "preconocer",
    "preconocerá": "preconocer",
    "preconocerán": "preconocer",
    "preconocerás": "preconocer",
    "preconoceré": "preconocer",
    "preconoceréis": "preconocer",
    "preconocería": "preconocer",
    "preconoceríais": "preconocer",
    "preconoceríamos": "preconocer",
    "preconocerían": "preconocer",
    "preconocerías": "preconocer",
    "preconoces": "preconocer",
    "preconocida": "preconocer",
    "preconocidas": "preconocer",
    "preconocido": "preconocer",
    "preconocidos": "preconocer",
    "preconociendo": "preconocer",
    "preconociera": "preconocer",
    "preconocierais": "preconocer",
    "preconocieran": "preconocer",
    "preconocieras": "preconocer",
    "preconociere": "preconocer",
    "preconociereis": "preconocer",
    "preconocieren": "preconocer",
    "preconocieres": "preconocer",
    "preconocieron": "preconocer",
    "preconociese": "preconocer",
    "preconocieseis": "preconocer",
    "preconociesen": "preconocer",
    "preconocieses": "preconocer",
    "preconocimos": "preconocer",
    "preconociste": "preconocer",
    "preconocisteis": "preconocer",
    "preconociéramos": "preconocer",
    "preconociéremos": "preconocer",
    "preconociésemos": "preconocer",
    "preconoció": "preconocer",
    "preconocéis": "preconocer",
    "preconocí": "preconocer",
    "preconocía": "preconocer",
    "preconocíais": "preconocer",
    "preconocíamos": "preconocer",
    "preconocían": "preconocer",
    "preconocías": "preconocer",
    "preconozca": "preconocer",
    "preconozcamos": "preconocer",
    "preconozcan": "preconocer",
    "preconozcas": "preconocer",
    "preconozco": "preconocer",
    "preconozcáis": "preconocer",
    "preconscientes": "preconsciente",
    "preconsolidaciones": "preconsolidación",
    "precontinentes": "precontinente",
    "precontractuales": "precontractual",
    "precontratos": "precontrato",
    "precordiales": "precordial",
    "precozes": "precoz",
    "precribadores": "precribador",
    "precursora": "precursor",
    "precursoras": "precursor",
    "precursores": "precursor",
    "precálculos": "precálculo",
    "precámbrica": "precámbrico",
    "precámbricas": "precámbrico",
    "precámbricos": "precámbrico",
    "predaciones": "predación",
    "predadora": "predador",
    "predadoras": "predador",
    "predadores": "predador",
    "predatoria": "predatorio",
    "predatorias": "predatorio",
    "predatorios": "predatorio",
    "predecesora": "predecesor",
    "predecesoras": "predecesor",
    "predecesores": "predecesor",
    "predecibles": "predecible",
    "predecid": "predecir",
    "predecimos": "predecir",
    "predeciremos": "predecir",
    "predecirá": "predecir",
    "predecirán": "predecir",
    "predecirás": "predecir",
    "predeciré": "predecir",
    "predeciréis": "predecir",
    "predeciría": "predecir",
    "predeciríais": "predecir",
    "predeciríamos": "predecir",
    "predecirían": "predecir",
    "predecirías": "predecir",
    "predecía": "predecir",
    "predecíais": "predecir",
    "predecíamos": "predecir",
    "predecían": "predecir",
    "predecías": "predecir",
    "predecís": "predecir",
    "predefina": "predefinir",
    "predefinamos": "predefinir",
    "predefinan": "predefinir",
    "predefinas": "predefinir",
    "predefine": "predefinir",
    "predefinen": "predefinir",
    "predefines": "predefinir",
    "predefiniciones": "predefinición",
    "predefinid": "predefinir",
    "predefinida": "predefinir",
    "predefinidas": "predefinir",
    "predefinido": "predefinir",
    "predefinidos": "predefinir",
    "predefiniendo": "predefinir",
    "predefiniera": "predefinir",
    "predefinierais": "predefinir",
    "predefinieran": "predefinir",
    "predefinieras": "predefinir",
    "predefiniere": "predefinir",
    "predefiniereis": "predefinir",
    "predefinieren": "predefinir",
    "predefinieres": "predefinir",
    "predefinieron": "predefinir",
    "predefiniese": "predefinir",
    "predefinieseis": "predefinir",
    "predefiniesen": "predefinir",
    "predefinieses": "predefinir",
    "predefinimos": "predefinir",
    "predefiniremos": "predefinir",
    "predefinirá": "predefinir",
    "predefinirán": "predefinir",
    "predefinirás": "predefinir",
    "predefiniré": "predefinir",
    "predefiniréis": "predefinir",
    "predefiniría": "predefinir",
    "predefiniríais": "predefinir",
    "predefiniríamos": "predefinir",
    "predefinirían": "predefinir",
    "predefinirías": "predefinir",
    "predefiniste": "predefinir",
    "predefinisteis": "predefinir",
    "predefiniéramos": "predefinir",
    "predefiniéremos": "predefinir",
    "predefiniésemos": "predefinir",
    "predefinió": "predefinir",
    "predefino": "predefinir",
    "predefináis": "predefinir",
    "predefiní": "predefinir",
    "predefinía": "predefinir",
    "predefiníais": "predefinir",
    "predefiníamos": "predefinir",
    "predefinían": "predefinir",
    "predefinías": "predefinir",
    "predefinís": "predefinir",
    "predelas": "predela",
    "predelictuales": "predelictual",
    "predelincuencias": "predelincuencia",
    "predelincuentes": "predelincuente",
    "predentinas": "predentina",
    "predestina": "predestinar",
    "predestinaba": "predestinar",
    "predestinabais": "predestinar",
    "predestinaban": "predestinar",
    "predestinabas": "predestinar",
    "predestinaciones": "predestinación",
    "predestinad": "predestinar",
    "predestinada": "predestinar",
    "predestinadas": "predestinar",
    "predestinado": "predestinar",
    "predestinados": "predestinar",
    "predestinamos": "predestinar",
    "predestinan": "predestinar",
    "predestinando": "predestinar",
    "predestinara": "predestinar",
    "predestinarais": "predestinar",
    "predestinaran": "predestinar",
    "predestinaras": "predestinar",
    "predestinare": "predestinar",
    "predestinareis": "predestinar",
    "predestinaremos": "predestinar",
    "predestinaren": "predestinar",
    "predestinares": "predestinar",
    "predestinaron": "predestinar",
    "predestinará": "predestinar",
    "predestinarán": "predestinar",
    "predestinarás": "predestinar",
    "predestinaré": "predestinar",
    "predestinaréis": "predestinar",
    "predestinaría": "predestinar",
    "predestinaríais": "predestinar",
    "predestinaríamos": "predestinar",
    "predestinarían": "predestinar",
    "predestinarías": "predestinar",
    "predestinas": "predestinar",
    "predestinase": "predestinar",
    "predestinaseis": "predestinar",
    "predestinasen": "predestinar",
    "predestinases": "predestinar",
    "predestinaste": "predestinar",
    "predestinasteis": "predestinar",
    "predestine": "predestinar",
    "predestinemos": "predestinar",
    "predestinen": "predestinar",
    "predestines": "predestinar",
    "predestino": "predestinar",
    "predestinábamos": "predestinar",
    "predestináis": "predestinar",
    "predestináramos": "predestinar",
    "predestináremos": "predestinar",
    "predestinásemos": "predestinar",
    "predestiné": "predestinar",
    "predestinéis": "predestinar",
    "predestinó": "predestinar",
    "predetermina": "predeterminar",
    "predeterminaba": "predeterminar",
    "predeterminabais": "predeterminar",
    "predeterminaban": "predeterminar",
    "predeterminabas": "predeterminar",
    "predeterminaciones": "predeterminación",
    "predeterminad": "predeterminar",
    "predeterminada": "predeterminar",
    "predeterminadas": "predeterminar",
    "predeterminado": "predeterminar",
    "predeterminados": "predeterminar",
    "predeterminamos": "predeterminar",
    "predeterminan": "predeterminar",
    "predeterminando": "predeterminar",
    "predeterminara": "predeterminar",
    "predeterminarais": "predeterminar",
    "predeterminaran": "predeterminar",
    "predeterminaras": "predeterminar",
    "predeterminare": "predeterminar",
    "predeterminareis": "predeterminar",
    "predeterminaremos": "predeterminar",
    "predeterminaren": "predeterminar",
    "predeterminares": "predeterminar",
    "predeterminaron": "predeterminar",
    "predeterminará": "predeterminar",
    "predeterminarán": "predeterminar",
    "predeterminarás": "predeterminar",
    "predeterminaré": "predeterminar",
    "predeterminaréis": "predeterminar",
    "predeterminaría": "predeterminar",
    "predeterminaríais": "predeterminar",
    "predeterminaríamos": "predeterminar",
    "predeterminarían": "predeterminar",
    "predeterminarías": "predeterminar",
    "predeterminas": "predeterminar",
    "predeterminase": "predeterminar",
    "predeterminaseis": "predeterminar",
    "predeterminasen": "predeterminar",
    "predeterminases": "predeterminar",
    "predeterminaste": "predeterminar",
    "predeterminasteis": "predeterminar",
    "predetermine": "predeterminar",
    "predeterminemos": "predeterminar",
    "predeterminen": "predeterminar",
    "predetermines": "predeterminar",
    "predetermino": "predeterminar",
    "predeterminábamos": "predeterminar",
    "predetermináis": "predeterminar",
    "predetermináramos": "predeterminar",
    "predetermináremos": "predeterminar",
    "predeterminásemos": "predeterminar",
    "predeterminé": "predeterminar",
    "predeterminéis": "predeterminar",
    "predeterminó": "predeterminar",
    "prediales": "predial",
    "predica": "predicar",
    "predicaba": "predicar",
    "predicabais": "predicar",
    "predicaban": "predicar",
    "predicabas": "predicar",
    "predicables": "predicable",
    "predicaciones": "predicación",
    "predicad": "predicar",
    "predicada": "predicar",
    "predicadas": "predicar",
    "predicado": "predicar",
    "predicadora": "predicador",
    "predicadoras": "predicador",
    "predicadores": "predicador",
    "predicados": "predicar",
    "predicamentales": "predicamental",
    "predicamentos": "predicamento",
    "predicamos": "predicar",
    "predican": "predicar",
    "predicando": "predicar",
    "predicantes": "predicante",
    "predicara": "predicar",
    "predicarais": "predicar",
    "predicaran": "predicar",
    "predicaras": "predicar",
    "predicare": "predicar",
    "predicareis": "predicar",
    "predicaremos": "predicar",
    "predicaren": "predicar",
    "predicares": "predicar",
    "predicaron": "predicar",
    "predicará": "predicar",
    "predicarán": "predicar",
    "predicarás": "predicar",
    "predicaré": "predicar",
    "predicaréis": "predicar",
    "predicaría": "predicar",
    "predicaríais": "predicar",
    "predicaríamos": "predicar",
    "predicarían": "predicar",
    "predicarías": "predicar",
    "predicas": "predicar",
    "predicase": "predicar",
    "predicaseis": "predicar",
    "predicasen": "predicar",
    "predicases": "predicar",
    "predicaste": "predicar",
    "predicasteis": "predicar",
    "predicativa": "predicativo",
    "predicativas": "predicativo",
    "predicativos": "predicativo",
    "predicciones": "predicción",
    "predice": "predecir",
    "predicen": "predecir",
    "predices": "predecir",
    "predicha": "predecir",
    "predichas": "predecir",
    "predicho": "predecir",
    "predichos": "predecir",
    "prediciendo": "predecir",
    "predico": "predicar",
    "predictibles": "predictible",
    "predictiva": "predictivo",
    "predictivas": "predictivo",
    "predictivos": "predictivo",
    "predictora": "predictor",
    "predictoras": "predictor",
    "predictores": "predictor",
    "predicábamos": "predicar",
    "predicáis": "predicar",
    "predicáramos": "predicar",
    "predicáremos": "predicar",
    "predicásemos": "predicar",
    "predicó": "predicar",
    "prediga": "predecir",
    "predigamos": "predecir",
    "predigan": "predecir",
    "predigas": "predecir",
    "predigo": "predecir",
    "predigáis": "predecir",
    "predije": "predecir",
    "predijera": "predecir",
    "predijerais": "predecir",
    "predijeran": "predecir",
    "predijeras": "predecir",
    "predijere": "predecir",
    "predijereis": "predecir",
    "predijeren": "predecir",
    "predijeres": "predecir",
    "predijeron": "predecir",
    "predijese": "predecir",
    "predijeseis": "predecir",
    "predijesen": "predecir",
    "predijeses": "predecir",
    "predijimos": "predecir",
    "predijiste": "predecir",
    "predijisteis": "predecir",
    "predijo": "predecir",
    "predijéramos": "predecir",
    "predijéremos": "predecir",
    "predijésemos": "predecir",
    "predilecciones": "predilección",
    "predilecta": "predilecto",
    "predilectas": "predilecto",
    "predilectos": "predilecto",
    "predios": "predio",
    "predique": "predicar",
    "prediquemos": "predicar",
    "prediquen": "predicar",
    "prediques": "predicar",
    "prediqué": "predicar",
    "prediquéis": "predicar",
    "predispondremos": "predisponer",
    "predispondrá": "predisponer",
    "predispondrán": "predisponer",
    "predispondrás": "predisponer",
    "predispondré": "predisponer",
    "predispondréis": "predisponer",
    "predispondría": "predisponer",
    "predispondríais": "predisponer",
    "predispondríamos": "predisponer",
    "predispondrían": "predisponer",
    "predispondrías": "predisponer",
    "predispone": "predisponer",
    "predisponed": "predisponer",
    "predisponemos": "predisponer",
    "predisponen": "predisponer",
    "predispones": "predisponer",
    "predisponga": "predisponer",
    "predispongamos": "predisponer",
    "predispongan": "predisponer",
    "predispongas": "predisponer",
    "predispongo": "predisponer",
    "predispongáis": "predisponer",
    "predisponiendo": "predisponer",
    "predisponéis": "predisponer",
    "predisponía": "predisponer",
    "predisponíais": "predisponer",
    "predisponíamos": "predisponer",
    "predisponían": "predisponer",
    "predisponías": "predisponer",
    "predisposiciones": "predisposición",
    "predispuesta": "predisponer",
    "predispuestas": "predisponer",
    "predispuesto": "predisponer",
    "predispuestos": "predisponer",
    "predispuse": "predisponer",
    "predispusiera": "predisponer",
    "predispusierais": "predisponer",
    "predispusieran": "predisponer",
    "predispusieras": "predisponer",
    "predispusiere": "predisponer",
    "predispusiereis": "predisponer",
    "predispusieren": "predisponer",
    "predispusieres": "predisponer",
    "predispusieron": "predisponer",
    "predispusiese": "predisponer",
    "predispusieseis": "predisponer",
    "predispusiesen": "predisponer",
    "predispusieses": "predisponer",
    "predispusimos": "predisponer",
    "predispusiste": "predisponer",
    "predispusisteis": "predisponer",
    "predispusiéramos": "predisponer",
    "predispusiéremos": "predisponer",
    "predispusiésemos": "predisponer",
    "predispuso": "predisponer",
    "predispón": "predisponer",
    "predisruptiva": "predisruptivo",
    "predisruptivas": "predisruptivo",
    "predisruptivos": "predisruptivo",
    "prediálogos": "prediálogo",
    "prednisolonas": "prednisolona",
    "prednisonas": "prednisona",
    "predomina": "predominar",
    "predominaba": "predominar",
    "predominabais": "predominar",
    "predominaban": "predominar",
    "predominabas": "predominar",
    "predominaciones": "predominación",
    "predominad": "predominar",
    "predominada": "predominar",
    "predominadas": "predominar",
    "predominado": "predominar",
    "predominados": "predominar",
    "predominamos": "predominar",
    "predominan": "predominar",
    "predominancias": "predominancia",
    "predominando": "predominar",
    "predominantes": "predominante",
    "predominara": "predominar",
    "predominarais": "predominar",
    "predominaran": "predominar",
    "predominaras": "predominar",
    "predominare": "predominar",
    "predominareis": "predominar",
    "predominaremos": "predominar",
    "predominaren": "predominar",
    "predominares": "predominar",
    "predominaron": "predominar",
    "predominará": "predominar",
    "predominarán": "predominar",
    "predominarás": "predominar",
    "predominaré": "predominar",
    "predominaréis": "predominar",
    "predominaría": "predominar",
    "predominaríais": "predominar",
    "predominaríamos": "predominar",
    "predominarían": "predominar",
    "predominarías": "predominar",
    "predominas": "predominar",
    "predominase": "predominar",
    "predominaseis": "predominar",
    "predominasen": "predominar",
    "predominases": "predominar",
    "predominaste": "predominar",
    "predominasteis": "predominar",
    "predomine": "predominar",
    "predominemos": "predominar",
    "predominen": "predominar",
    "predomines": "predominar",
    "predominios": "predominio",
    "predomino": "predominar",
    "predominábamos": "predominar",
    "predomináis": "predominar",
    "predomináramos": "predominar",
    "predomináremos": "predominar",
    "predominásemos": "predominar",
    "predominé": "predominar",
    "predominéis": "predominar",
    "predominó": "predominar",
    "predorsales": "predorsal",
    "predorsos": "predorso",
    "preecualizaciones": "preecualización",
    "preediciones": "preedición",
    "preeditora": "preeditor",
    "preeditoras": "preeditor",
    "preeditores": "preeditor",
    "preedípica": "preedípico",
    "preedípicas": "preedípico",
    "preedípicos": "preedípico",
    "preelegid": "preelegir",
    "preelegida": "preelegir",
    "preelegidas": "preelegir",
    "preelegido": "preelegir",
    "preelegidos": "preelegir",
    "preelegimos": "preelegir",
    "preelegiremos": "preelegir",
    "preelegirá": "preelegir",
    "preelegirán": "preelegir",
    "preelegirás": "preelegir",
    "preelegiré": "preelegir",
    "preelegiréis": "preelegir",
    "preelegiría": "preelegir",
    "preelegiríais": "preelegir",
    "preelegiríamos": "preelegir",
    "preelegirían": "preelegir",
    "preelegirías": "preelegir",
    "preelegiste": "preelegir",
    "preelegisteis": "preelegir",
    "preelegí": "preelegir",
    "preelegía": "preelegir",
    "preelegíais": "preelegir",
    "preelegíamos": "preelegir",
    "preelegían": "preelegir",
    "preelegías": "preelegir",
    "preelegís": "preelegir",
    "preelige": "preelegir",
    "preeligen": "preelegir",
    "preeliges": "preelegir",
    "preeligiendo": "preelegir",
    "preeligiera": "preelegir",
    "preeligierais": "preelegir",
    "preeligieran": "preelegir",
    "preeligieras": "preelegir",
    "preeligiere": "preelegir",
    "preeligiereis": "preelegir",
    "preeligieren": "preelegir",
    "preeligieres": "preelegir",
    "preeligieron": "preelegir",
    "preeligiese": "preelegir",
    "preeligieseis": "preelegir",
    "preeligiesen": "preelegir",
    "preeligieses": "preelegir",
    "preeligiéramos": "preelegir",
    "preeligiéremos": "preelegir",
    "preeligiésemos": "preelegir",
    "preeligió": "preelegir",
    "preelija": "preelegir",
    "preelijamos": "preelegir",
    "preelijan": "preelegir",
    "preelijas": "preelegir",
    "preelijo": "preelegir",
    "preelijáis": "preelegir",
    "preeminencias": "preeminencia",
    "preeminentes": "preeminente",
    "preenfriadores": "preenfriador",
    "preenvasados": "preenvasado",
    "preenvueltas": "preenvuelta",
    "preepiglótica": "preepiglótico",
    "preepiglóticas": "preepiglótico",
    "preepiglóticos": "preepiglótico",
    "preeruptiva": "preeruptivo",
    "preeruptivas": "preeruptivo",
    "preeruptivos": "preeruptivo",
    "preescolares": "preescolar",
    "preestablece": "preestablecer",
    "preestableced": "preestablecer",
    "preestablecemos": "preestablecer",
    "preestablecen": "preestablecer",
    "preestableceremos": "preestablecer",
    "preestablecerá": "preestablecer",
    "preestablecerán": "preestablecer",
    "preestablecerás": "preestablecer",
    "preestableceré": "preestablecer",
    "preestableceréis": "preestablecer",
    "preestablecería": "preestablecer",
    "preestableceríais": "preestablecer",
    "preestableceríamos": "preestablecer",
    "preestablecerían": "preestablecer",
    "preestablecerías": "preestablecer",
    "preestableces": "preestablecer",
    "preestablecida": "preestablecer",
    "preestablecidas": "preestablecer",
    "preestablecido": "preestablecer",
    "preestablecidos": "preestablecer",
    "preestableciendo": "preestablecer",
    "preestableciera": "preestablecer",
    "preestablecierais": "preestablecer",
    "preestablecieran": "preestablecer",
    "preestablecieras": "preestablecer",
    "preestableciere": "preestablecer",
    "preestableciereis": "preestablecer",
    "preestablecieren": "preestablecer",
    "preestablecieres": "preestablecer",
    "preestablecieron": "preestablecer",
    "preestableciese": "preestablecer",
    "preestablecieseis": "preestablecer",
    "preestableciesen": "preestablecer",
    "preestablecieses": "preestablecer",
    "preestablecimos": "preestablecer",
    "preestableciste": "preestablecer",
    "preestablecisteis": "preestablecer",
    "preestableciéramos": "preestablecer",
    "preestableciéremos": "preestablecer",
    "preestableciésemos": "preestablecer",
    "preestableció": "preestablecer",
    "preestablecéis": "preestablecer",
    "preestablecí": "preestablecer",
    "preestablecía": "preestablecer",
    "preestablecíais": "preestablecer",
    "preestablecíamos": "preestablecer",
    "preestablecían": "preestablecer",
    "preestablecías": "preestablecer",
    "preestablezca": "preestablecer",
    "preestablezcamos": "preestablecer",
    "preestablezcan": "preestablecer",
    "preestablezcas": "preestablecer",
    "preestablezco": "preestablecer",
    "preestablezcáis": "preestablecer",
    "preestatales": "preestatal",
    "preestrenos": "preestreno",
    "preexista": "preexistir",
    "preexistamos": "preexistir",
    "preexistan": "preexistir",
    "preexistas": "preexistir",
    "preexiste": "preexistir",
    "preexisten": "preexistir",
    "preexistencias": "preexistencia",
    "preexistentes": "preexistente",
    "preexistes": "preexistir",
    "preexistid": "preexistir",
    "preexistida": "preexistir",
    "preexistidas": "preexistir",
    "preexistido": "preexistir",
    "preexistidos": "preexistir",
    "preexistiendo": "preexistir",
    "preexistiera": "preexistir",
    "preexistierais": "preexistir",
    "preexistieran": "preexistir",
    "preexistieras": "preexistir",
    "preexistiere": "preexistir",
    "preexistiereis": "preexistir",
    "preexistieren": "preexistir",
    "preexistieres": "preexistir",
    "preexistieron": "preexistir",
    "preexistiese": "preexistir",
    "preexistieseis": "preexistir",
    "preexistiesen": "preexistir",
    "preexistieses": "preexistir",
    "preexistimos": "preexistir",
    "preexistiremos": "preexistir",
    "preexistirá": "preexistir",
    "preexistirán": "preexistir",
    "preexistirás": "preexistir",
    "preexistiré": "preexistir",
    "preexistiréis": "preexistir",
    "preexistiría": "preexistir",
    "preexistiríais": "preexistir",
    "preexistiríamos": "preexistir",
    "preexistirían": "preexistir",
    "preexistirías": "preexistir",
    "preexististe": "preexistir",
    "preexististeis": "preexistir",
    "preexistiéramos": "preexistir",
    "preexistiéremos": "preexistir",
    "preexistiésemos": "preexistir",
    "preexistió": "preexistir",
    "preexisto": "preexistir",
    "preexistáis": "preexistir",
    "preexistí": "preexistir",
    "preexistía": "preexistir",
    "preexistíais": "preexistir",
    "preexistíamos": "preexistir",
    "preexistían": "preexistir",
    "preexistías": "preexistir",
    "preexistís": "preexistir",
    "preexámenes": "preexamen",
    "prefabrica": "prefabricar",
    "prefabricaba": "prefabricar",
    "prefabricabais": "prefabricar",
    "prefabricaban": "prefabricar",
    "prefabricabas": "prefabricar",
    "prefabricaciones": "prefabricación",
    "prefabricad": "prefabricar",
    "prefabricada": "prefabricar",
    "prefabricadas": "prefabricar",
    "prefabricado": "prefabricar",
    "prefabricados": "prefabricar",
    "prefabricamos": "prefabricar",
    "prefabrican": "prefabricar",
    "prefabricando": "prefabricar",
    "prefabricara": "prefabricar",
    "prefabricarais": "prefabricar",
    "prefabricaran": "prefabricar",
    "prefabricaras": "prefabricar",
    "prefabricare": "prefabricar",
    "prefabricareis": "prefabricar",
    "prefabricaremos": "prefabricar",
    "prefabricaren": "prefabricar",
    "prefabricares": "prefabricar",
    "prefabricaron": "prefabricar",
    "prefabricará": "prefabricar",
    "prefabricarán": "prefabricar",
    "prefabricarás": "prefabricar",
    "prefabricaré": "prefabricar",
    "prefabricaréis": "prefabricar",
    "prefabricaría": "prefabricar",
    "prefabricaríais": "prefabricar",
    "prefabricaríamos": "prefabricar",
    "prefabricarían": "prefabricar",
    "prefabricarías": "prefabricar",
    "prefabricas": "prefabricar",
    "prefabricase": "prefabricar",
    "prefabricaseis": "prefabricar",
    "prefabricasen": "prefabricar",
    "prefabricases": "prefabricar",
    "prefabricaste": "prefabricar",
    "prefabricasteis": "prefabricar",
    "prefabrico": "prefabricar",
    "prefabricábamos": "prefabricar",
    "prefabricáis": "prefabricar",
    "prefabricáramos": "prefabricar",
    "prefabricáremos": "prefabricar",
    "prefabricásemos": "prefabricar",
    "prefabricó": "prefabricar",
    "prefabrique": "prefabricar",
    "prefabriquemos": "prefabricar",
    "prefabriquen": "prefabricar",
    "prefabriques": "prefabricar",
    "prefabriqué": "prefabricar",
    "prefabriquéis": "prefabricar",
    "prefaciones": "prefación",
    "prefacios": "prefacio",
    "prefacturaciones": "prefacturación",
    "prefectos": "prefecto",
    "prefecturas": "prefectura",
    "preferenciales": "preferencial",
    "preferencias": "preferencia",
    "preferentes": "preferente",
    "preferibles": "preferible",
    "preferid": "preferir",
    "preferida": "preferir",
    "preferidas": "preferir",
    "preferido": "preferir",
    "preferidos": "preferir",
    "preferimos": "preferir",
    "preferiremos": "preferir",
    "preferirá": "preferir",
    "preferirán": "preferir",
    "preferirás": "preferir",
    "preferiré": "preferir",
    "preferiréis": "preferir",
    "preferiría": "preferir",
    "preferiríais": "preferir",
    "preferiríamos": "preferir",
    "preferirían": "preferir",
    "preferirías": "preferir",
    "preferiste": "preferir",
    "preferisteis": "preferir",
    "preferí": "preferir",
    "prefería": "preferir",
    "preferíais": "preferir",
    "preferíamos": "preferir",
    "preferían": "preferir",
    "preferías": "preferir",
    "preferís": "preferir",
    "prefiera": "preferir",
    "prefieran": "preferir",
    "prefieras": "preferir",
    "prefiere": "preferir",
    "prefieren": "preferir",
    "prefieres": "preferir",
    "prefiero": "preferir",
    "prefigura": "prefigurar",
    "prefiguraba": "prefigurar",
    "prefigurabais": "prefigurar",
    "prefiguraban": "prefigurar",
    "prefigurabas": "prefigurar",
    "prefiguraciones": "prefiguración",
    "prefigurad": "prefigurar",
    "prefigurada": "prefigurar",
    "prefiguradas": "prefigurar",
    "prefigurado": "prefigurar",
    "prefigurados": "prefigurar",
    "prefiguramos": "prefigurar",
    "prefiguran": "prefigurar",
    "prefigurando": "prefigurar",
    "prefigurara": "prefigurar",
    "prefigurarais": "prefigurar",
    "prefiguraran": "prefigurar",
    "prefiguraras": "prefigurar",
    "prefigurare": "prefigurar",
    "prefigurareis": "prefigurar",
    "prefiguraremos": "prefigurar",
    "prefiguraren": "prefigurar",
    "prefigurares": "prefigurar",
    "prefiguraron": "prefigurar",
    "prefigurará": "prefigurar",
    "prefigurarán": "prefigurar",
    "prefigurarás": "prefigurar",
    "prefiguraré": "prefigurar",
    "prefiguraréis": "prefigurar",
    "prefiguraría": "prefigurar",
    "prefiguraríais": "prefigurar",
    "prefiguraríamos": "prefigurar",
    "prefigurarían": "prefigurar",
    "prefigurarías": "prefigurar",
    "prefiguras": "prefigurar",
    "prefigurase": "prefigurar",
    "prefiguraseis": "prefigurar",
    "prefigurasen": "prefigurar",
    "prefigurases": "prefigurar",
    "prefiguraste": "prefigurar",
    "prefigurasteis": "prefigurar",
    "prefigure": "prefigurar",
    "prefiguremos": "prefigurar",
    "prefiguren": "prefigurar",
    "prefigures": "prefigurar",
    "prefiguro": "prefigurar",
    "prefigurábamos": "prefigurar",
    "prefiguráis": "prefigurar",
    "prefiguráramos": "prefigurar",
    "prefiguráremos": "prefigurar",
    "prefigurásemos": "prefigurar",
    "prefiguré": "prefigurar",
    "prefiguréis": "prefigurar",
    "prefiguró": "prefigurar",
    "prefija": "prefijo",
    "prefijaba": "prefijar",
    "prefijabais": "prefijar",
    "prefijaban": "prefijar",
    "prefijabas": "prefijar",
    "prefijables": "prefijable",
    "prefijaciones": "prefijación",
    "prefijad": "prefijar",
    "prefijada": "prefijar",
    "prefijadas": "prefijar",
    "prefijado": "prefijar",
    "prefijados": "prefijar",
    "prefijales": "prefijal",
    "prefijamos": "prefijar",
    "prefijan": "prefijar",
    "prefijando": "prefijar",
    "prefijara": "prefijar",
    "prefijarais": "prefijar",
    "prefijaran": "prefijar",
    "prefijaras": "prefijar",
    "prefijare": "prefijar",
    "prefijareis": "prefijar",
    "prefijaremos": "prefijar",
    "prefijaren": "prefijar",
    "prefijares": "prefijar",
    "prefijaron": "prefijar",
    "prefijará": "prefijar",
    "prefijarán": "prefijar",
    "prefijarás": "prefijar",
    "prefijaré": "prefijar",
    "prefijaréis": "prefijar",
    "prefijaría": "prefijar",
    "prefijaríais": "prefijar",
    "prefijaríamos": "prefijar",
    "prefijarían": "prefijar",
    "prefijarías": "prefijar",
    "prefijas": "prefijo",
    "prefijase": "prefijar",
    "prefijaseis": "prefijar",
    "prefijasen": "prefijar",
    "prefijases": "prefijar",
    "prefijaste": "prefijar",
    "prefijasteis": "prefijar",
    "prefije": "prefijar",
    "prefijemos": "prefijar",
    "prefijen": "prefijar",
    "prefijes": "prefijar",
    "prefijo": "prefijar",
    "prefijos": "prefijo",
    "prefijábamos": "prefijar",
    "prefijáis": "prefijar",
    "prefijáramos": "prefijar",
    "prefijáremos": "prefijar",
    "prefijásemos": "prefijar",
    "prefijé": "prefijar",
    "prefijéis": "prefijar",
    "prefijó": "prefijar",
    "prefiltraciones": "prefiltración",
    "prefinanciaciones": "prefinanciación",
    "prefiniciones": "prefinición",
    "prefiramos": "preferir",
    "prefiriendo": "preferir",
    "prefiriera": "preferir",
    "prefirierais": "preferir",
    "prefirieran": "preferir",
    "prefirieras": "preferir",
    "prefiriere": "preferir",
    "prefiriereis": "preferir",
    "prefirieren": "preferir",
    "prefirieres": "preferir",
    "prefirieron": "preferir",
    "prefiriese": "preferir",
    "prefirieseis": "preferir",
    "prefiriesen": "preferir",
    "prefirieses": "preferir",
    "prefiriéramos": "preferir",
    "prefiriéremos": "preferir",
    "prefiriésemos": "preferir",
    "prefirió": "preferir",
    "prefiráis": "preferir",
    "prefloraciones": "prefloración",
    "prefoliaciones": "prefoliación",
    "preformada": "preformado",
    "preformadas": "preformado",
    "preformados": "preformado",
    "preformas": "preforma",
    "preformulada": "preformulado",
    "preformuladas": "preformulado",
    "preformulados": "preformulado",
    "prefrontales": "prefrontal",
    "prefulgentes": "prefulgente",
    "prefuncionales": "prefuncional",
    "preganglionares": "preganglionar",
    "pregnanodioles": "pregnanodiol",
    "pregnenolonas": "pregnenolona",
    "pregona": "pregonar",
    "pregonaba": "pregonar",
    "pregonabais": "pregonar",
    "pregonaban": "pregonar",
    "pregonabas": "pregonar",
    "pregonad": "pregonar",
    "pregonada": "pregonar",
    "pregonadas": "pregonar",
    "pregonado": "pregonar",
    "pregonados": "pregonar",
    "pregonamos": "pregonar",
    "pregonan": "pregonar",
    "pregonando": "pregonar",
    "pregonara": "pregonar",
    "pregonarais": "pregonar",
    "pregonaran": "pregonar",
    "pregonaras": "pregonar",
    "pregonare": "pregonar",
    "pregonareis": "pregonar",
    "pregonaremos": "pregonar",
    "pregonaren": "pregonar",
    "pregonares": "pregonar",
    "pregonaron": "pregonar",
    "pregonará": "pregonar",
    "pregonarán": "pregonar",
    "pregonarás": "pregonar",
    "pregonaré": "pregonar",
    "pregonaréis": "pregonar",
    "pregonaría": "pregonar",
    "pregonaríais": "pregonar",
    "pregonaríamos": "pregonar",
    "pregonarían": "pregonar",
    "pregonarías": "pregonar",
    "pregonas": "pregonar",
    "pregonase": "pregonar",
    "pregonaseis": "pregonar",
    "pregonasen": "pregonar",
    "pregonases": "pregonar",
    "pregonaste": "pregonar",
    "pregonasteis": "pregonar",
    "pregone": "pregonar",
    "pregonemos": "pregonar",
    "pregonen": "pregonar",
    "pregonera": "pregonero",
    "pregoneras": "pregonero",
    "pregoneros": "pregonero",
    "pregonerías": "pregonería",
    "pregones": "pregonar",
    "pregono": "pregonar",
    "pregonábamos": "pregonar",
    "pregonáis": "pregonar",
    "pregonáramos": "pregonar",
    "pregonáremos": "pregonar",
    "pregonásemos": "pregonar",
    "pregoné": "pregonar",
    "pregonéis": "pregonar",
    "pregonó": "pregonar",
    "pregunta": "preguntar",
    "preguntaba": "preguntar",
    "preguntabais": "preguntar",
    "preguntaban": "preguntar",
    "preguntabas": "preguntar",
    "preguntad": "preguntar",
    "preguntada": "preguntar",
    "preguntadas": "preguntar",
    "preguntado": "preguntar",
    "preguntadora": "preguntador",
    "preguntadoras": "preguntador",
    "preguntadores": "preguntador",
    "preguntados": "preguntar",
    "preguntamos": "preguntar",
    "preguntan": "preguntar",
    "preguntando": "preguntar",
    "preguntara": "preguntar",
    "preguntarais": "preguntar",
    "preguntaran": "preguntar",
    "preguntaras": "preguntar",
    "preguntare": "preguntar",
    "preguntareis": "preguntar",
    "preguntaremos": "preguntar",
    "preguntaren": "preguntar",
    "preguntares": "preguntar",
    "preguntaron": "preguntar",
    "preguntará": "preguntar",
    "preguntarán": "preguntar",
    "preguntarás": "preguntar",
    "preguntaré": "preguntar",
    "preguntaréis": "preguntar",
    "preguntaría": "preguntar",
    "preguntaríais": "preguntar",
    "preguntaríamos": "preguntar",
    "preguntarían": "preguntar",
    "preguntarías": "preguntar",
    "preguntas": "preguntar",
    "preguntase": "preguntar",
    "preguntaseis": "preguntar",
    "preguntasen": "preguntar",
    "preguntases": "preguntar",
    "preguntaste": "preguntar",
    "preguntasteis": "preguntar",
    "pregunte": "preguntar",
    "preguntemos": "preguntar",
    "pregunten": "preguntar",
    "pregunteos": "pregunteo",
    "preguntes": "preguntar",
    "pregunto": "preguntar",
    "preguntona": "preguntón",
    "preguntonas": "preguntón",
    "preguntones": "preguntón",
    "preguntábamos": "preguntar",
    "preguntáis": "preguntar",
    "preguntáramos": "preguntar",
    "preguntáremos": "preguntar",
    "preguntásemos": "preguntar",
    "pregunté": "preguntar",
    "preguntéis": "preguntar",
    "preguntó": "preguntar",
    "pregustaciones": "pregustación",
    "prehelénica": "prehelénico",
    "prehelénicas": "prehelénico",
    "prehelénicos": "prehelénico",
    "prehilbertiana": "prehilbertiano",
    "prehilbertianas": "prehilbertiano",
    "prehilbertianos": "prehilbertiano",
    "prehioidales": "prehioidal",
    "prehispánica": "prehispánico",
    "prehispánicas": "prehispánico",
    "prehispánicos": "prehispánico",
    "prehistoriadora": "prehistoriador",
    "prehistoriadoras": "prehistoriador",
    "prehistoriadores": "prehistoriador",
    "prehistorias": "prehistoria",
    "prehistórica": "prehistórico",
    "prehistóricas": "prehistórico",
    "prehistóricos": "prehistórico",
    "prehnitas": "prehnita",
    "prehnitenos": "prehniteno",
    "preimpresiones": "preimpresión",
    "preimpulsos": "preimpulso",
    "preinmunizaciones": "preinmunización",
    "preinscripciones": "preinscripción",
    "preinserta": "preinserto",
    "preinsertas": "preinserto",
    "preinsertos": "preinserto",
    "preinversiones": "preinversión",
    "preislámica": "preislámico",
    "preislámicas": "preislámico",
    "preislámicos": "preislámico",
    "prejubilaciones": "prejubilación",
    "prejudiciales": "prejudicial",
    "prejudicialidades": "prejudicialidad",
    "prejuicios": "prejuicio",
    "prejuzga": "prejuzgar",
    "prejuzgaba": "prejuzgar",
    "prejuzgabais": "prejuzgar",
    "prejuzgaban": "prejuzgar",
    "prejuzgabas": "prejuzgar",
    "prejuzgad": "prejuzgar",
    "prejuzgada": "prejuzgar",
    "prejuzgadas": "prejuzgar",
    "prejuzgado": "prejuzgar",
    "prejuzgados": "prejuzgar",
    "prejuzgamos": "prejuzgar",
    "prejuzgan": "prejuzgar",
    "prejuzgando": "prejuzgar",
    "prejuzgara": "prejuzgar",
    "prejuzgarais": "prejuzgar",
    "prejuzgaran": "prejuzgar",
    "prejuzgaras": "prejuzgar",
    "prejuzgare": "prejuzgar",
    "prejuzgareis": "prejuzgar",
    "prejuzgaremos": "prejuzgar",
    "prejuzgaren": "prejuzgar",
    "prejuzgares": "prejuzgar",
    "prejuzgaron": "prejuzgar",
    "prejuzgará": "prejuzgar",
    "prejuzgarán": "prejuzgar",
    "prejuzgarás": "prejuzgar",
    "prejuzgaré": "prejuzgar",
    "prejuzgaréis": "prejuzgar",
    "prejuzgaría": "prejuzgar",
    "prejuzgaríais": "prejuzgar",
    "prejuzgaríamos": "prejuzgar",
    "prejuzgarían": "prejuzgar",
    "prejuzgarías": "prejuzgar",
    "prejuzgas": "prejuzgar",
    "prejuzgase": "prejuzgar",
    "prejuzgaseis": "prejuzgar",
    "prejuzgasen": "prejuzgar",
    "prejuzgases": "prejuzgar",
    "prejuzgaste": "prejuzgar",
    "prejuzgasteis": "prejuzgar",
    "prejuzgo": "prejuzgar",
    "prejuzgue": "prejuzgar",
    "prejuzguemos": "prejuzgar",
    "prejuzguen": "prejuzgar",
    "prejuzgues": "prejuzgar",
    "prejuzgué": "prejuzgar",
    "prejuzguéis": "prejuzgar",
    "prejuzgábamos": "prejuzgar",
    "prejuzgáis": "prejuzgar",
    "prejuzgáramos": "prejuzgar",
    "prejuzgáremos": "prejuzgar",
    "prejuzgásemos": "prejuzgar",
    "prejuzgó": "prejuzgar",
    "prelaciones": "prelación",
    "prelacías": "prelacía",
    "preladas": "prelada",
    "prelados": "prelado",
    "prelaríngea": "prelaríngeo",
    "prelaríngeas": "prelaríngeo",
    "prelaríngeos": "prelaríngeo",
    "prelaticia": "prelaticio",
    "prelaticias": "prelaticio",
    "prelaticios": "prelaticio",
    "prelaturas": "prelatura",
    "prelibertades": "prelibertad",
    "preliminares": "preliminar",
    "prelimpiadoras": "prelimpiadora",
    "prelitorales": "prelitoral",
    "preludia": "preludiar",
    "preludiaba": "preludiar",
    "preludiabais": "preludiar",
    "preludiaban": "preludiar",
    "preludiabas": "preludiar",
    "preludiad": "preludiar",
    "preludiada": "preludiar",
    "preludiadas": "preludiar",
    "preludiado": "preludiar",
    "preludiados": "preludiar",
    "preludiamos": "preludiar",
    "preludian": "preludiar",
    "preludiando": "preludiar",
    "preludiara": "preludiar",
    "preludiarais": "preludiar",
    "preludiaran": "preludiar",
    "preludiaras": "preludiar",
    "preludiare": "preludiar",
    "preludiareis": "preludiar",
    "preludiaremos": "preludiar",
    "preludiaren": "preludiar",
    "preludiares": "preludiar",
    "preludiaron": "preludiar",
    "preludiará": "preludiar",
    "preludiarán": "preludiar",
    "preludiarás": "preludiar",
    "preludiaré": "preludiar",
    "preludiaréis": "preludiar",
    "preludiaría": "preludiar",
    "preludiaríais": "preludiar",
    "preludiaríamos": "preludiar",
    "preludiarían": "preludiar",
    "preludiarías": "preludiar",
    "preludias": "preludiar",
    "preludiase": "preludiar",
    "preludiaseis": "preludiar",
    "preludiasen": "preludiar",
    "preludiases": "preludiar",
    "preludiaste": "preludiar",
    "preludiasteis": "preludiar",
    "preludie": "preludiar",
    "preludiemos": "preludiar",
    "preludien": "preludiar",
    "preludies": "preludiar",
    "preludio": "preludiar",
    "preludios": "preludio",
    "preludiábamos": "preludiar",
    "preludiáis": "preludiar",
    "preludiáramos": "preludiar",
    "preludiáremos": "preludiar",
    "preludiásemos": "preludiar",
    "preludié": "preludiar",
    "preludiéis": "preludiar",
    "preludió": "preludiar",
    "premagnetizaciones": "premagnetización",
    "prematrimoniales": "prematrimonial",
    "prematura": "prematuro",
    "prematuras": "prematuro",
    "prematuros": "prematuro",
    "premaxilares": "premaxilar",
    "premaxilas": "premaxila",
    "premedita": "premeditar",
    "premeditaba": "premeditar",
    "premeditabais": "premeditar",
    "premeditaban": "premeditar",
    "premeditabas": "premeditar",
    "premeditaciones": "premeditación",
    "premeditad": "premeditar",
    "premeditada": "premeditar",
    "premeditadas": "premeditar",
    "premeditado": "premeditar",
    "premeditados": "premeditar",
    "premeditamos": "premeditar",
    "premeditan": "premeditar",
    "premeditando": "premeditar",
    "premeditara": "premeditar",
    "premeditarais": "premeditar",
    "premeditaran": "premeditar",
    "premeditaras": "premeditar",
    "premeditare": "premeditar",
    "premeditareis": "premeditar",
    "premeditaremos": "premeditar",
    "premeditaren": "premeditar",
    "premeditares": "premeditar",
    "premeditaron": "premeditar",
    "premeditará": "premeditar",
    "premeditarán": "premeditar",
    "premeditarás": "premeditar",
    "premeditaré": "premeditar",
    "premeditaréis": "premeditar",
    "premeditaría": "premeditar",
    "premeditaríais": "premeditar",
    "premeditaríamos": "premeditar",
    "premeditarían": "premeditar",
    "premeditarías": "premeditar",
    "premeditas": "premeditar",
    "premeditase": "premeditar",
    "premeditaseis": "premeditar",
    "premeditasen": "premeditar",
    "premeditases": "premeditar",
    "premeditaste": "premeditar",
    "premeditasteis": "premeditar",
    "premedite": "premeditar",
    "premeditemos": "premeditar",
    "premediten": "premeditar",
    "premedites": "premeditar",
    "premedito": "premeditar",
    "premeditábamos": "premeditar",
    "premeditáis": "premeditar",
    "premeditáramos": "premeditar",
    "premeditáremos": "premeditar",
    "premeditásemos": "premeditar",
    "premedité": "premeditar",
    "premeditéis": "premeditar",
    "premeditó": "premeditar",
    "premetafases": "premetafase",
    "premezclas": "premezcla",
    "premia": "premiar",
    "premiaba": "premiar",
    "premiabais": "premiar",
    "premiaban": "premiar",
    "premiabas": "premiar",
    "premiad": "premiar",
    "premiada": "premiar",
    "premiadas": "premiar",
    "premiado": "premiar",
    "premiados": "premiar",
    "premiamos": "premiar",
    "premian": "premiar",
    "premiando": "premiar",
    "premiara": "premiar",
    "premiarais": "premiar",
    "premiaran": "premiar",
    "premiaras": "premiar",
    "premiare": "premiar",
    "premiareis": "premiar",
    "premiaremos": "premiar",
    "premiaren": "premiar",
    "premiares": "premiar",
    "premiaron": "premiar",
    "premiará": "premiar",
    "premiarán": "premiar",
    "premiarás": "premiar",
    "premiaré": "premiar",
    "premiaréis": "premiar",
    "premiaría": "premiar",
    "premiaríais": "premiar",
    "premiaríamos": "premiar",
    "premiarían": "premiar",
    "premiarías": "premiar",
    "premias": "premiar",
    "premiase": "premiar",
    "premiaseis": "premiar",
    "premiasen": "premiar",
    "premiases": "premiar",
    "premiaste": "premiar",
    "premiasteis": "premiar",
    "premideras": "premidera",
    "premie": "premiar",
    "premiemos": "premiar",
    "premien": "premiar",
    "premieres": "premiere",
    "premiers": "premier",
    "premies": "premiar",
    "premio": "premiar",
    "premios": "premio",
    "premiosa": "premioso",
    "premiosas": "premioso",
    "premiosidades": "premiosidad",
    "premiosos": "premioso",
    "premisa": "premiso",
    "premisas": "premiso",
    "premisos": "premiso",
    "premiábamos": "premiar",
    "premiáis": "premiar",
    "premiáramos": "premiar",
    "premiáremos": "premiar",
    "premiásemos": "premiar",
    "premié": "premiar",
    "premiéis": "premiar",
    "premió": "premiar",
    "premociones": "premoción",
    "premolares": "premolar",
    "premolarizaciones": "premolarización",
    "premoliendas": "premolienda",
    "premolturaciones": "premolturación",
    "premoniciones": "premonición",
    "premonitora": "premonitor",
    "premonitoras": "premonitor",
    "premonitores": "premonitor",
    "premonitoria": "premonitorio",
    "premonitorias": "premonitorio",
    "premonitorios": "premonitorio",
    "premonstratenses": "premonstratense",
    "premorid": "premorir",
    "premoriencias": "premoriencia",
    "premorimos": "premorir",
    "premoriremos": "premorir",
    "premorirá": "premorir",
    "premorirán": "premorir",
    "premorirás": "premorir",
    "premoriré": "premorir",
    "premoriréis": "premorir",
    "premoriría": "premorir",
    "premoriríais": "premorir",
    "premoriríamos": "premorir",
    "premorirían": "premorir",
    "premorirías": "premorir",
    "premoriste": "premorir",
    "premoristeis": "premorir",
    "premorí": "premorir",
    "premoría": "premorir",
    "premoríais": "premorir",
    "premoríamos": "premorir",
    "premorían": "premorir",
    "premorías": "premorir",
    "premorís": "premorir",
    "premuera": "premorir",
    "premueran": "premorir",
    "premueras": "premorir",
    "premuere": "premorir",
    "premueren": "premorir",
    "premueres": "premorir",
    "premuero": "premorir",
    "premuerta": "premorir",
    "premuertas": "premorir",
    "premuerto": "premorir",
    "premuertos": "premorir",
    "premuniciones": "premunición",
    "premuramos": "premorir",
    "premuras": "premura",
    "premuriendo": "premorir",
    "premuriera": "premorir",
    "premurierais": "premorir",
    "premurieran": "premorir",
    "premurieras": "premorir",
    "premuriere": "premorir",
    "premuriereis": "premorir",
    "premurieren": "premorir",
    "premurieres": "premorir",
    "premurieron": "premorir",
    "premuriese": "premorir",
    "premurieseis": "premorir",
    "premuriesen": "premorir",
    "premurieses": "premorir",
    "premuriéramos": "premorir",
    "premuriéremos": "premorir",
    "premuriésemos": "premorir",
    "premurió": "premorir",
    "premuráis": "premorir",
    "prenatales": "prenatal",
    "prenda": "prender",
    "prendaba": "prendar",
    "prendabais": "prendar",
    "prendaban": "prendar",
    "prendabas": "prendar",
    "prendad": "prendar",
    "prendada": "prendar",
    "prendadas": "prendar",
    "prendado": "prendar",
    "prendadora": "prendador",
    "prendadoras": "prendador",
    "prendadores": "prendador",
    "prendados": "prendar",
    "prendamientos": "prendamiento",
    "prendamos": "prender",
    "prendan": "prender",
    "prendando": "prendar",
    "prendara": "prendar",
    "prendarais": "prendar",
    "prendaran": "prendar",
    "prendaras": "prendar",
    "prendare": "prendar",
    "prendareis": "prendar",
    "prendaremos": "prendar",
    "prendaren": "prendar",
    "prendares": "prendar",
    "prendaria": "prendario",
    "prendarias": "prendario",
    "prendarios": "prendario",
    "prendaron": "prendar",
    "prendará": "prendar",
    "prendarán": "prendar",
    "prendarás": "prendar",
    "prendaré": "prendar",
    "prendaréis": "prendar",
    "prendaría": "prendar",
    "prendaríais": "prendar",
    "prendaríamos": "prendar",
    "prendarían": "prendar",
    "prendarías": "prendar",
    "prendas": "prender",
    "prendase": "prendar",
    "prendaseis": "prendar",
    "prendasen": "prendar",
    "prendases": "prendar",
    "prendaste": "prendar",
    "prendasteis": "prendar",
    "prende": "prender",
    "prended": "prender",
    "prendederos": "prendedero",
    "prendedora": "prendedor",
    "prendedoras": "prendedor",
    "prendedores": "prendedor",
    "prendeduras": "prendedura",
    "prendemos": "prender",
    "prenden": "prender",
    "prendera": "prendero",
    "prenderas": "prendero",
    "prenderemos": "prender",
    "prenderos": "prendero",
    "prenderá": "prender",
    "prenderán": "prender",
    "prenderás": "prender",
    "prenderé": "prender",
    "prenderéis": "prender",
    "prendería": "prender",
    "prenderíais": "prender",
    "prenderíamos": "prender",
    "prenderían": "prender",
    "prenderías": "prendería",
    "prendes": "prender",
    "prendida": "prender",
    "prendidas": "prender",
    "prendido": "prender",
    "prendidos": "prendido",
    "prendiendo": "prender",
    "prendiera": "prender",
    "prendierais": "prender",
    "prendieran": "prender",
    "prendieras": "prender",
    "prendiere": "prender",
    "prendiereis": "prender",
    "prendieren": "prender",
    "prendieres": "prender",
    "prendieron": "prender",
    "prendiese": "prender",
    "prendieseis": "prender",
    "prendiesen": "prender",
    "prendieses": "prender",
    "prendimientos": "prendimiento",
    "prendimos": "prender",
    "prendiste": "prender",
    "prendisteis": "prender",
    "prendiéramos": "prender",
    "prendiéremos": "prender",
    "prendiésemos": "prender",
    "prendió": "prender",
    "prendo": "prender",
    "prendábamos": "prendar",
    "prendáis": "prender",
    "prendáramos": "prendar",
    "prendáremos": "prendar",
    "prendásemos": "prendar",
    "prendé": "prendar",
    "prendéis": "prender",
    "prendí": "prender",
    "prendía": "prender",
    "prendíais": "prender",
    "prendíamos": "prender",
    "prendían": "prender",
    "prendías": "prender",
    "prendó": "prendar",
    "prenociones": "prenoción",
    "prenombres": "prenombre",
    "prensa": "prensar",
    "prensaba": "prensar",
    "prensabais": "prensar",
    "prensabalas": "prensabala",
    "prensaban": "prensar",
    "prensabas": "prensar",
    "prensad": "prensar",
    "prensada": "prensar",
    "prensadas": "prensar",
    "prensado": "prensar",
    "prensadora": "prensador",
    "prensadoras": "prensadora",
    "prensadores": "prensador",
    "prensados": "prensar",
    "prensaduras": "prensadura",
    "prensamos": "prensar",
    "prensan": "prensar",
    "prensando": "prensar",
    "prensara": "prensar",
    "prensarais": "prensar",
    "prensaran": "prensar",
    "prensaras": "prensar",
    "prensare": "prensar",
    "prensareis": "prensar",
    "prensaremos": "prensar",
    "prensaren": "prensar",
    "prensares": "prensar",
    "prensaron": "prensar",
    "prensará": "prensar",
    "prensarán": "prensar",
    "prensarás": "prensar",
    "prensaré": "prensar",
    "prensaréis": "prensar",
    "prensaría": "prensar",
    "prensaríais": "prensar",
    "prensaríamos": "prensar",
    "prensarían": "prensar",
    "prensarías": "prensar",
    "prensas": "prensar",
    "prensase": "prensar",
    "prensaseis": "prensar",
    "prensasen": "prensar",
    "prensases": "prensar",
    "prensaste": "prensar",
    "prensasteis": "prensar",
    "prense": "prensar",
    "prensemos": "prensar",
    "prensen": "prensar",
    "prenses": "prensar",
    "prensiles": "prensil",
    "prensiones": "prensión",
    "prensistas": "prensista",
    "prenso": "prensar",
    "prensoras": "prensora",
    "prensores": "prensor",
    "prensábamos": "prensar",
    "prensáis": "prensar",
    "prensáramos": "prensar",
    "prensáremos": "prensar",
    "prensásemos": "prensar",
    "prensé": "prensar",
    "prenséis": "prensar",
    "prensó": "prensar",
    "prenuncia": "prenunciar",
    "prenunciaba": "prenunciar",
    "prenunciabais": "prenunciar",
    "prenunciaban": "prenunciar",
    "prenunciabas": "prenunciar",
    "prenunciad": "prenunciar",
    "prenunciada": "prenunciar",
    "prenunciadas": "prenunciar",
    "prenunciado": "prenunciar",
    "prenunciados": "prenunciar",
    "prenunciamos": "prenunciar",
    "prenuncian": "prenunciar",
    "prenunciando": "prenunciar",
    "prenunciara": "prenunciar",
    "prenunciarais": "prenunciar",
    "prenunciaran": "prenunciar",
    "prenunciaras": "prenunciar",
    "prenunciare": "prenunciar",
    "prenunciareis": "prenunciar",
    "prenunciaremos": "prenunciar",
    "prenunciaren": "prenunciar",
    "prenunciares": "prenunciar",
    "prenunciaron": "prenunciar",
    "prenunciará": "prenunciar",
    "prenunciarán": "prenunciar",
    "prenunciarás": "prenunciar",
    "prenunciaré": "prenunciar",
    "prenunciaréis": "prenunciar",
    "prenunciaría": "prenunciar",
    "prenunciaríais": "prenunciar",
    "prenunciaríamos": "prenunciar",
    "prenunciarían": "prenunciar",
    "prenunciarías": "prenunciar",
    "prenuncias": "prenunciar",
    "prenunciase": "prenunciar",
    "prenunciaseis": "prenunciar",
    "prenunciasen": "prenunciar",
    "prenunciases": "prenunciar",
    "prenunciaste": "prenunciar",
    "prenunciasteis": "prenunciar",
    "prenuncie": "prenunciar",
    "prenunciemos": "prenunciar",
    "prenuncien": "prenunciar",
    "prenuncies": "prenunciar",
    "prenuncio": "prenunciar",
    "prenuncios": "prenuncio",
    "prenunciábamos": "prenunciar",
    "prenunciáis": "prenunciar",
    "prenunciáramos": "prenunciar",
    "prenunciáremos": "prenunciar",
    "prenunciásemos": "prenunciar",
    "prenuncié": "prenunciar",
    "prenunciéis": "prenunciar",
    "prenunció": "prenunciar",
    "preoccipitales": "preoccipital",
    "preocupa": "preocupar",
    "preocupaba": "preocupar",
    "preocupabais": "preocupar",
    "preocupaban": "preocupar",
    "preocupabas": "preocupar",
    "preocupaciones": "preocupación",
    "preocupad": "preocupar",
    "preocupada": "preocupar",
    "preocupadas": "preocupar",
    "preocupado": "preocupar",
    "preocupados": "preocupar",
    "preocupamos": "preocupar",
    "preocupan": "preocupar",
    "preocupando": "preocupar",
    "preocupantes": "preocupante",
    "preocupara": "preocupar",
    "preocuparais": "preocupar",
    "preocuparan": "preocupar",
    "preocuparas": "preocupar",
    "preocupare": "preocupar",
    "preocupareis": "preocupar",
    "preocuparemos": "preocupar",
    "preocuparen": "preocupar",
    "preocupares": "preocupar",
    "preocuparon": "preocupar",
    "preocupará": "preocupar",
    "preocuparán": "preocupar",
    "preocuparás": "preocupar",
    "preocuparé": "preocupar",
    "preocuparéis": "preocupar",
    "preocuparía": "preocupar",
    "preocuparíais": "preocupar",
    "preocuparíamos": "preocupar",
    "preocuparían": "preocupar",
    "preocuparías": "preocupar",
    "preocupas": "preocupar",
    "preocupase": "preocupar",
    "preocupaseis": "preocupar",
    "preocupasen": "preocupar",
    "preocupases": "preocupar",
    "preocupaste": "preocupar",
    "preocupasteis": "preocupar",
    "preocupe": "preocupar",
    "preocupemos": "preocupar",
    "preocupen": "preocupar",
    "preocupes": "preocupar",
    "preocupo": "preocupar",
    "preocupábamos": "preocupar",
    "preocupáis": "preocupar",
    "preocupáramos": "preocupar",
    "preocupáremos": "preocupar",
    "preocupásemos": "preocupar",
    "preocupé": "preocupar",
    "preocupéis": "preocupar",
    "preocupó": "preocupar",
    "preoperativa": "preoperativo",
    "preoperativas": "preoperativo",
    "preoperativos": "preoperativo",
    "preopinantes": "preopinante",
    "preordinaciones": "preordinación",
    "prepagos": "prepago",
    "prepalatales": "prepalatal",
    "prepara": "preparar",
    "preparaba": "preparar",
    "preparabais": "preparar",
    "preparaban": "preparar",
    "preparabas": "preparar",
    "preparaciones": "preparación",
    "preparad": "preparar",
    "preparada": "preparar",
    "preparadas": "preparar",
    "preparado": "preparar",
    "preparadora": "preparador",
    "preparadoras": "preparador",
    "preparadores": "preparador",
    "preparados": "preparar",
    "preparamos": "preparar",
    "preparan": "preparar",
    "preparando": "preparar",
    "preparara": "preparar",
    "prepararais": "preparar",
    "prepararan": "preparar",
    "prepararas": "preparar",
    "preparare": "preparar",
    "preparareis": "preparar",
    "prepararemos": "preparar",
    "prepararen": "preparar",
    "preparares": "preparar",
    "prepararon": "preparar",
    "preparará": "preparar",
    "prepararán": "preparar",
    "prepararás": "preparar",
    "prepararé": "preparar",
    "prepararéis": "preparar",
    "prepararía": "preparar",
    "prepararíais": "preparar",
    "prepararíamos": "preparar",
    "prepararían": "preparar",
    "prepararías": "preparar",
    "preparas": "preparar",
    "preparase": "preparar",
    "preparaseis": "preparar",
    "preparasen": "preparar",
    "preparases": "preparar",
    "preparaste": "preparar",
    "preparasteis": "preparar",
    "preparativa": "preparativo",
    "preparativas": "preparativo",
    "preparativos": "preparativo",
    "preparatoria": "preparatorio",
    "preparatorias": "preparatorio",
    "preparatorios": "preparatorio",
    "prepare": "preparar",
    "preparemos": "preparar",
    "preparen": "preparar",
    "prepares": "preparar",
    "preparo": "preparar",
    "preparábamos": "preparar",
    "preparáis": "preparar",
    "preparáramos": "preparar",
    "preparáremos": "preparar",
    "preparásemos": "preparar",
    "preparé": "preparar",
    "preparéis": "preparar",
    "preparó": "preparar",
    "preperforaciones": "preperforación",
    "preperforaciónes": "preperforación",
    "preperióstica": "preperióstico",
    "preperiósticas": "preperióstico",
    "preperiósticos": "preperióstico",
    "prepilórica": "prepilórico",
    "prepilóricas": "prepilórico",
    "prepilóricos": "prepilórico",
    "prepodadoras": "prepodadora",
    "prepolímeros": "prepolímero",
    "prepondera": "preponderar",
    "preponderaba": "preponderar",
    "preponderabais": "preponderar",
    "preponderaban": "preponderar",
    "preponderabas": "preponderar",
    "preponderad": "preponderar",
    "preponderada": "preponderar",
    "preponderadas": "preponderar",
    "preponderado": "preponderar",
    "preponderados": "preponderar",
    "preponderamos": "preponderar",
    "preponderan": "preponderar",
    "preponderancias": "preponderancia",
    "preponderando": "preponderar",
    "preponderantes": "preponderante",
    "preponderara": "preponderar",
    "preponderarais": "preponderar",
    "preponderaran": "preponderar",
    "preponderaras": "preponderar",
    "preponderare": "preponderar",
    "preponderareis": "preponderar",
    "preponderaremos": "preponderar",
    "preponderaren": "preponderar",
    "preponderares": "preponderar",
    "preponderaron": "preponderar",
    "preponderará": "preponderar",
    "preponderarán": "preponderar",
    "preponderarás": "preponderar",
    "preponderaré": "preponderar",
    "preponderaréis": "preponderar",
    "preponderaría": "preponderar",
    "preponderaríais": "preponderar",
    "preponderaríamos": "preponderar",
    "preponderarían": "preponderar",
    "preponderarías": "preponderar",
    "preponderas": "preponderar",
    "preponderase": "preponderar",
    "preponderaseis": "preponderar",
    "preponderasen": "preponderar",
    "preponderases": "preponderar",
    "preponderaste": "preponderar",
    "preponderasteis": "preponderar",
    "prepondere": "preponderar",
    "preponderemos": "preponderar",
    "preponderen": "preponderar",
    "preponderes": "preponderar",
    "prepondero": "preponderar",
    "preponderábamos": "preponderar",
    "preponderáis": "preponderar",
    "preponderáramos": "preponderar",
    "preponderáremos": "preponderar",
    "preponderásemos": "preponderar",
    "preponderé": "preponderar",
    "preponderéis": "preponderar",
    "preponderó": "preponderar",
    "prepondremos": "preponer",
    "prepondrá": "preponer",
    "prepondrán": "preponer",
    "prepondrás": "preponer",
    "prepondré": "preponer",
    "prepondréis": "preponer",
    "prepondría": "preponer",
    "prepondríais": "preponer",
    "prepondríamos": "preponer",
    "prepondrían": "preponer",
    "prepondrías": "preponer",
    "prepone": "preponer",
    "preponed": "preponer",
    "preponemos": "preponer",
    "preponen": "preponer",
    "prepones": "preponer",
    "preponga": "preponer",
    "prepongamos": "preponer",
    "prepongan": "preponer",
    "prepongas": "preponer",
    "prepongo": "preponer",
    "prepongáis": "preponer",
    "preponiendo": "preponer",
    "preponéis": "preponer",
    "preponía": "preponer",
    "preponíais": "preponer",
    "preponíamos": "preponer",
    "preponían": "preponer",
    "preponías": "preponer",
    "preposicionales": "preposicional",
    "preposiciones": "preposición",
    "prepositiva": "prepositivo",
    "prepositivas": "prepositivo",
    "prepositivos": "prepositivo",
    "preposituras": "prepositura",
    "preposteraciones": "preposteración",
    "prepotencias": "prepotencia",
    "prepotentes": "prepotente",
    "preprocesadores": "preprocesador",
    "preprocesamientos": "preprocesamiento",
    "preproducciones": "preproducción",
    "preprofagos": "preprofago",
    "preprofásica": "preprofásico",
    "preprofásicas": "preprofásico",
    "preprofásicos": "preprofásico",
    "prepuberales": "prepuberal",
    "prepuciales": "prepucial",
    "prepucios": "prepucio",
    "prepuesta": "preponer",
    "prepuestas": "preponer",
    "prepuesto": "preponer",
    "prepuestos": "preponer",
    "prepuse": "preponer",
    "prepusiera": "preponer",
    "prepusierais": "preponer",
    "prepusieran": "preponer",
    "prepusieras": "preponer",
    "prepusiere": "preponer",
    "prepusiereis": "preponer",
    "prepusieren": "preponer",
    "prepusieres": "preponer",
    "prepusieron": "preponer",
    "prepusiese": "preponer",
    "prepusieseis": "preponer",
    "prepusiesen": "preponer",
    "prepusieses": "preponer",
    "prepusimos": "preponer",
    "prepusiste": "preponer",
    "prepusisteis": "preponer",
    "prepusiéramos": "preponer",
    "prepusiéremos": "preponer",
    "prepusiésemos": "preponer",
    "prepuso": "preponer",
    "prepón": "preponer",
    "prepósitos": "prepósito",
    "prepóstera": "prepóstero",
    "prepósteras": "prepóstero",
    "prepósteros": "prepóstero",
    "prerefrigeraciones": "prerefrigeración",
    "prerrafaelismos": "prerrafaelismo",
    "prerrafaelistas": "prerrafaelista",
    "prerrafaelitas": "prerrafaelita",
    "prerrequisitos": "prerrequisito",
    "prerrogativas": "prerrogativa",
    "prerromana": "prerromano",
    "prerromanas": "prerromano",
    "prerromanos": "prerromano",
    "prerromanticismos": "prerromanticismo",
    "prerrománica": "prerrománico",
    "prerrománicas": "prerrománico",
    "prerrománicos": "prerrománico",
    "prerromántica": "prerromántico",
    "prerrománticas": "prerromántico",
    "prerrománticos": "prerromántico",
    "pres": "pre",
    "presa": "preso",
    "presada": "presado",
    "presadas": "presado",
    "presados": "presado",
    "presaga": "présago",
    "presagas": "présago",
    "presagia": "presagiar",
    "presagiaba": "presagiar",
    "presagiabais": "presagiar",
    "presagiaban": "presagiar",
    "presagiabas": "presagiar",
    "presagiad": "presagiar",
    "presagiada": "presagiar",
    "presagiadas": "presagiar",
    "presagiado": "presagiar",
    "presagiados": "presagiar",
    "presagiamos": "presagiar",
    "presagian": "presagiar",
    "presagiando": "presagiar",
    "presagiara": "presagiar",
    "presagiarais": "presagiar",
    "presagiaran": "presagiar",
    "presagiaras": "presagiar",
    "presagiare": "presagiar",
    "presagiareis": "presagiar",
    "presagiaremos": "presagiar",
    "presagiaren": "presagiar",
    "presagiares": "presagiar",
    "presagiaron": "presagiar",
    "presagiará": "presagiar",
    "presagiarán": "presagiar",
    "presagiarás": "presagiar",
    "presagiaré": "presagiar",
    "presagiaréis": "presagiar",
    "presagiaría": "presagiar",
    "presagiaríais": "presagiar",
    "presagiaríamos": "presagiar",
    "presagiarían": "presagiar",
    "presagiarías": "presagiar",
    "presagias": "presagiar",
    "presagiase": "presagiar",
    "presagiaseis": "presagiar",
    "presagiasen": "presagiar",
    "presagiases": "presagiar",
    "presagiaste": "presagiar",
    "presagiasteis": "presagiar",
    "presagie": "presagiar",
    "presagiemos": "presagiar",
    "presagien": "presagiar",
    "presagies": "presagiar",
    "presagio": "presagiar",
    "presagios": "presagio",
    "presagiosa": "presagioso",
    "presagiosas": "presagioso",
    "presagiosos": "presagioso",
    "presagiábamos": "presagiar",
    "presagiáis": "presagiar",
    "presagiáramos": "presagiar",
    "presagiáremos": "presagiar",
    "presagiásemos": "presagiar",
    "presagié": "presagiar",
    "presagiéis": "presagiar",
    "presagió": "presagiar",
    "presagos": "présago",
    "presas": "preso",
    "presaturada": "presaturado",
    "presaturadas": "presaturado",
    "presaturados": "presaturado",
    "presbiacusias": "presbiacusia",
    "presbicias": "presbicia",
    "presbiopías": "presbiopía",
    "presbiterados": "presbiterado",
    "presbiterales": "presbiteral",
    "presbiterazgos": "presbiterazgo",
    "presbiteriales": "presbiterial",
    "presbiteriana": "presbiteriano",
    "presbiterianas": "presbiteriano",
    "presbiterianismos": "presbiterianismo",
    "presbiterianos": "presbiteriano",
    "presbiterios": "presbiterio",
    "presbivértigos": "presbivértigo",
    "presbíteros": "presbítero",
    "presciencias": "presciencia",
    "prescinda": "prescindir",
    "prescindamos": "prescindir",
    "prescindan": "prescindir",
    "prescindas": "prescindir",
    "prescinde": "prescindir",
    "prescinden": "prescindir",
    "prescindencias": "prescindencia",
    "prescindes": "prescindir",
    "prescindibles": "prescindible",
    "prescindid": "prescindir",
    "prescindida": "prescindir",
    "prescindidas": "prescindir",
    "prescindido": "prescindir",
    "prescindidos": "prescindir",
    "prescindiendo": "prescindir",
    "prescindiera": "prescindir",
    "prescindierais": "prescindir",
    "prescindieran": "prescindir",
    "prescindieras": "prescindir",
    "prescindiere": "prescindir",
    "prescindiereis": "prescindir",
    "prescindieren": "prescindir",
    "prescindieres": "prescindir",
    "prescindieron": "prescindir",
    "prescindiese": "prescindir",
    "prescindieseis": "prescindir",
    "prescindiesen": "prescindir",
    "prescindieses": "prescindir",
    "prescindimos": "prescindir",
    "prescindiremos": "prescindir",
    "prescindirá": "prescindir",
    "prescindirán": "prescindir",
    "prescindirás": "prescindir",
    "prescindiré": "prescindir",
    "prescindiréis": "prescindir",
    "prescindiría": "prescindir",
    "prescindiríais": "prescindir",
    "prescindiríamos": "prescindir",
    "prescindirían": "prescindir",
    "prescindirías": "prescindir",
    "prescindiste": "prescindir",
    "prescindisteis": "prescindir",
    "prescindiéramos": "prescindir",
    "prescindiéremos": "prescindir",
    "prescindiésemos": "prescindir",
    "prescindió": "prescindir",
    "prescindo": "prescindir",
    "prescindáis": "prescindir",
    "prescindí": "prescindir",
    "prescindía": "prescindir",
    "prescindíais": "prescindir",
    "prescindíamos": "prescindir",
    "prescindían": "prescindir",
    "prescindías": "prescindir",
    "prescindís": "prescindir",
    "prescita": "prescito",
    "prescitas": "prescito",
    "prescitos": "prescito",
    "prescriba": "prescribir",
    "prescribamos": "prescribir",
    "prescriban": "prescribir",
    "prescribas": "prescribir",
    "prescribe": "prescribir",
    "prescriben": "prescribir",
    "prescribes": "prescribir",
    "prescribid": "prescribir",
    "prescribiendo": "prescribir",
    "prescribiera": "prescribir",
    "prescribierais": "prescribir",
    "prescribieran": "prescribir",
    "prescribieras": "prescribir",
    "prescribiere": "prescribir",
    "prescribiereis": "prescribir",
    "prescribieren": "prescribir",
    "prescribieres": "prescribir",
    "prescribieron": "prescribir",
    "prescribiese": "prescribir",
    "prescribieseis": "prescribir",
    "prescribiesen": "prescribir",
    "prescribieses": "prescribir",
    "prescribimos": "prescribir",
    "prescribiremos": "prescribir",
    "prescribirá": "prescribir",
    "prescribirán": "prescribir",
    "prescribirás": "prescribir",
    "prescribiré": "prescribir",
    "prescribiréis": "prescribir",
    "prescribiría": "prescribir",
    "prescribiríais": "prescribir",
    "prescribiríamos": "prescribir",
    "prescribirían": "prescribir",
    "prescribirías": "prescribir",
    "prescribiste": "prescribir",
    "prescribisteis": "prescribir",
    "prescribiéramos": "prescribir",
    "prescribiéremos": "prescribir",
    "prescribiésemos": "prescribir",
    "prescribió": "prescribir",
    "prescribo": "prescribir",
    "prescribáis": "prescribir",
    "prescribí": "prescribir",
    "prescribía": "prescribir",
    "prescribíais": "prescribir",
    "prescribíamos": "prescribir",
    "prescribían": "prescribir",
    "prescribías": "prescribir",
    "prescribís": "prescribir",
    "prescripciones": "prescripción",
    "prescripta": "prescribir",
    "prescriptas": "prescribir",
    "prescriptibles": "prescriptible",
    "prescriptiva": "prescriptivo",
    "prescriptivas": "prescriptivo",
    "prescriptivos": "prescriptivo",
    "prescripto": "prescribir",
    "prescriptos": "prescribir",
    "prescrita": "prescribir",
    "prescritas": "prescribir",
    "prescrito": "prescribir",
    "prescritos": "prescribir",
    "preseas": "presea",
    "presecadores": "presecador",
    "preselecciona": "preseleccionar",
    "preseleccionaba": "preseleccionar",
    "preseleccionabais": "preseleccionar",
    "preseleccionaban": "preseleccionar",
    "preseleccionabas": "preseleccionar",
    "preseleccionad": "preseleccionar",
    "preseleccionada": "preseleccionar",
    "preseleccionadas": "preseleccionar",
    "preseleccionado": "preseleccionar",
    "preseleccionados": "preseleccionar",
    "preseleccionamos": "preseleccionar",
    "preseleccionan": "preseleccionar",
    "preseleccionando": "preseleccionar",
    "preseleccionara": "preseleccionar",
    "preseleccionarais": "preseleccionar",
    "preseleccionaran": "preseleccionar",
    "preseleccionaras": "preseleccionar",
    "preseleccionare": "preseleccionar",
    "preseleccionareis": "preseleccionar",
    "preseleccionaremos": "preseleccionar",
    "preseleccionaren": "preseleccionar",
    "preseleccionares": "preseleccionar",
    "preseleccionaron": "preseleccionar",
    "preseleccionará": "preseleccionar",
    "preseleccionarán": "preseleccionar",
    "preseleccionarás": "preseleccionar",
    "preseleccionaré": "preseleccionar",
    "preseleccionaréis": "preseleccionar",
    "preseleccionaría": "preseleccionar",
    "preseleccionaríais": "preseleccionar",
    "preseleccionaríamos": "preseleccionar",
    "preseleccionarían": "preseleccionar",
    "preseleccionarías": "preseleccionar",
    "preseleccionas": "preseleccionar",
    "preseleccionase": "preseleccionar",
    "preseleccionaseis": "preseleccionar",
    "preseleccionasen": "preseleccionar",
    "preseleccionases": "preseleccionar",
    "preseleccionaste": "preseleccionar",
    "preseleccionasteis": "preseleccionar",
    "preseleccione": "preseleccionar",
    "preseleccionemos": "preseleccionar",
    "preseleccionen": "preseleccionar",
    "preselecciones": "preseleccionar",
    "preselecciono": "preseleccionar",
    "preseleccionábamos": "preseleccionar",
    "preseleccionáis": "preseleccionar",
    "preseleccionáramos": "preseleccionar",
    "preseleccionáremos": "preseleccionar",
    "preseleccionásemos": "preseleccionar",
    "preseleccioné": "preseleccionar",
    "preseleccionéis": "preseleccionar",
    "preseleccionó": "preseleccionar",
    "preselectores": "preselector",
    "presencia": "presenciar",
    "presenciaba": "presenciar",
    "presenciabais": "presenciar",
    "presenciaban": "presenciar",
    "presenciabas": "presenciar",
    "presenciad": "presenciar",
    "presenciada": "presenciar",
    "presenciadas": "presenciar",
    "presenciado": "presenciar",
    "presenciados": "presenciar",
    "presenciales": "presencial",
    "presencialidades": "presencialidad",
    "presenciamos": "presenciar",
    "presencian": "presenciar",
    "presenciando": "presenciar",
    "presenciara": "presenciar",
    "presenciarais": "presenciar",
    "presenciaran": "presenciar",
    "presenciaras": "presenciar",
    "presenciare": "presenciar",
    "presenciareis": "presenciar",
    "presenciaremos": "presenciar",
    "presenciaren": "presenciar",
    "presenciares": "presenciar",
    "presenciaron": "presenciar",
    "presenciará": "presenciar",
    "presenciarán": "presenciar",
    "presenciarás": "presenciar",
    "presenciaré": "presenciar",
    "presenciaréis": "presenciar",
    "presenciaría": "presenciar",
    "presenciaríais": "presenciar",
    "presenciaríamos": "presenciar",
    "presenciarían": "presenciar",
    "presenciarías": "presenciar",
    "presencias": "presenciar",
    "presenciase": "presenciar",
    "presenciaseis": "presenciar",
    "presenciasen": "presenciar",
    "presenciases": "presenciar",
    "presenciaste": "presenciar",
    "presenciasteis": "presenciar",
    "presencie": "presenciar",
    "presenciemos": "presenciar",
    "presencien": "presenciar",
    "presencies": "presenciar",
    "presencio": "presenciar",
    "presenciábamos": "presenciar",
    "presenciáis": "presenciar",
    "presenciáramos": "presenciar",
    "presenciáremos": "presenciar",
    "presenciásemos": "presenciar",
    "presencié": "presenciar",
    "presenciéis": "presenciar",
    "presenció": "presenciar",
    "presenta": "presentar",
    "presentaba": "presentar",
    "presentabais": "presentar",
    "presentaban": "presentar",
    "presentabas": "presentar",
    "presentables": "presentable",
    "presentaciones": "presentación",
    "presentad": "presentar",
    "presentada": "presentar",
    "presentadas": "presentar",
    "presentado": "presentar",
    "presentadora": "presentador",
    "presentadoras": "presentador",
    "presentadores": "presentador",
    "presentados": "presentar",
    "presentallas": "presentalla",
    "presentamos": "presentar",
    "presentan": "presentar",
    "presentando": "presentar",
    "presentara": "presentar",
    "presentarais": "presentar",
    "presentaran": "presentar",
    "presentaras": "presentar",
    "presentare": "presentar",
    "presentareis": "presentar",
    "presentaremos": "presentar",
    "presentaren": "presentar",
    "presentares": "presentar",
    "presentaron": "presentar",
    "presentará": "presentar",
    "presentarán": "presentar",
    "presentarás": "presentar",
    "presentaré": "presentar",
    "presentaréis": "presentar",
    "presentaría": "presentar",
    "presentaríais": "presentar",
    "presentaríamos": "presentar",
    "presentarían": "presentar",
    "presentarías": "presentar",
    "presentas": "presentar",
    "presentase": "presentar",
    "presentaseis": "presentar",
    "presentasen": "presentar",
    "presentases": "presentar",
    "presentaste": "presentar",
    "presentasteis": "presentar",
    "presente": "presentar",
    "presentemos": "presentar",
    "presenten": "presentar",
    "presentes": "presente",
    "presentid": "presentir",
    "presentida": "presentir",
    "presentidas": "presentir",
    "presentido": "presentir",
    "presentidos": "presentir",
    "presentimientos": "presentimiento",
    "presentimos": "presentir",
    "presentiremos": "presentir",
    "presentirá": "presentir",
    "presentirán": "presentir",
    "presentirás": "presentir",
    "presentiré": "presentir",
    "presentiréis": "presentir",
    "presentiría": "presentir",
    "presentiríais": "presentir",
    "presentiríamos": "presentir",
    "presentirían": "presentir",
    "presentirías": "presentir",
    "presentiste": "presentir",
    "presentisteis": "presentir",
    "presento": "presentar",
    "presentábamos": "presentar",
    "presentáis": "presentar",
    "presentánea": "presentáneo",
    "presentáneas": "presentáneo",
    "presentáneos": "presentáneo",
    "presentáramos": "presentar",
    "presentáremos": "presentar",
    "presentásemos": "presentar",
    "presenté": "presentar",
    "presentéis": "presentar",
    "presentí": "presentir",
    "presentía": "presentir",
    "presentíais": "presentir",
    "presentíamos": "presentir",
    "presentían": "presentir",
    "presentías": "presentir",
    "presentís": "presentir",
    "presentó": "presentar",
    "preseras": "presera",
    "preseros": "presero",
    "preserva": "preservar",
    "preservaba": "preservar",
    "preservabais": "preservar",
    "preservaban": "preservar",
    "preservabas": "preservar",
    "preservaciones": "preservación",
    "preservad": "preservar",
    "preservada": "preservar",
    "preservadas": "preservar",
    "preservado": "preservar",
    "preservadora": "preservador",
    "preservadoras": "preservador",
    "preservadores": "preservador",
    "preservados": "preservar",
    "preservamos": "preservar",
    "preservan": "preservar",
    "preservando": "preservar",
    "preservara": "preservar",
    "preservarais": "preservar",
    "preservaran": "preservar",
    "preservaras": "preservar",
    "preservare": "preservar",
    "preservareis": "preservar",
    "preservaremos": "preservar",
    "preservaren": "preservar",
    "preservares": "preservar",
    "preservaron": "preservar",
    "preservará": "preservar",
    "preservarán": "preservar",
    "preservarás": "preservar",
    "preservaré": "preservar",
    "preservaréis": "preservar",
    "preservaría": "preservar",
    "preservaríais": "preservar",
    "preservaríamos": "preservar",
    "preservarían": "preservar",
    "preservarías": "preservar",
    "preservas": "preservar",
    "preservase": "preservar",
    "preservaseis": "preservar",
    "preservasen": "preservar",
    "preservases": "preservar",
    "preservaste": "preservar",
    "preservasteis": "preservar",
    "preservativa": "preservativo",
    "preservativas": "preservativo",
    "preservativos": "preservativo",
    "preserve": "preservar",
    "preservemos": "preservar",
    "preserven": "preservar",
    "preserves": "preservar",
    "preservo": "preservar",
    "preservábamos": "preservar",
    "preserváis": "preservar",
    "preserváramos": "preservar",
    "preserváremos": "preservar",
    "preservásemos": "preservar",
    "preservé": "preservar",
    "preservéis": "preservar",
    "preservó": "preservar",
    "preseñalizaciones": "preseñalización",
    "presida": "presidir",
    "presidamos": "presidir",
    "presidan": "presidir",
    "presidas": "presidir",
    "preside": "presidir",
    "presiden": "presidir",
    "presidenciables": "presidenciable",
    "presidenciales": "presidencial",
    "presidencialismos": "presidencialismo",
    "presidencialistas": "presidencialista",
    "presidencias": "presidencia",
    "presidenta": "presidente",
    "presidentas": "presidente",
    "presidentes": "presidente",
    "presides": "presidir",
    "presidiables": "presidiable",
    "presidiaria": "presidiario",
    "presidiarias": "presidiario",
    "presidiarios": "presidiario",
    "presidid": "presidir",
    "presidida": "presidir",
    "presididas": "presidir",
    "presidido": "presidir",
    "presididos": "presidir",
    "presidiendo": "presidir",
    "presidiera": "presidir",
    "presidierais": "presidir",
    "presidieran": "presidir",
    "presidieras": "presidir",
    "presidiere": "presidir",
    "presidiereis": "presidir",
    "presidieren": "presidir",
    "presidieres": "presidir",
    "presidieron": "presidir",
    "presidiese": "presidir",
    "presidieseis": "presidir",
    "presidiesen": "presidir",
    "presidieses": "presidir",
    "presidimos": "presidir",
    "presidios": "presidio",
    "presidiremos": "presidir",
    "presidirá": "presidir",
    "presidirán": "presidir",
    "presidirás": "presidir",
    "presidiré": "presidir",
    "presidiréis": "presidir",
    "presidiría": "presidir",
    "presidiríais": "presidir",
    "presidiríamos": "presidir",
    "presidirían": "presidir",
    "presidirías": "presidir",
    "presidiste": "presidir",
    "presidisteis": "presidir",
    "presidiéramos": "presidir",
    "presidiéremos": "presidir",
    "presidiésemos": "presidir",
    "presidió": "presidir",
    "presido": "presidir",
    "presidáis": "presidir",
    "presidí": "presidir",
    "presidía": "presidir",
    "presidíais": "presidir",
    "presidíamos": "presidir",
    "presidían": "presidir",
    "presidías": "presidir",
    "presidís": "presidir",
    "presienta": "presentir",
    "presientan": "presentir",
    "presientas": "presentir",
    "presiente": "presentir",
    "presienten": "presentir",
    "presientes": "presentir",
    "presiento": "presentir",
    "presillas": "presilla",
    "presintamos": "presentir",
    "presintiendo": "presentir",
    "presintiera": "presentir",
    "presintierais": "presentir",
    "presintieran": "presentir",
    "presintieras": "presentir",
    "presintiere": "presentir",
    "presintiereis": "presentir",
    "presintieren": "presentir",
    "presintieres": "presentir",
    "presintieron": "presentir",
    "presintiese": "presentir",
    "presintieseis": "presentir",
    "presintiesen": "presentir",
    "presintieses": "presentir",
    "presintiéramos": "presentir",
    "presintiéremos": "presentir",
    "presintiésemos": "presentir",
    "presintió": "presentir",
    "presintáis": "presentir",
    "presiona": "presionar",
    "presionaba": "presionar",
    "presionabais": "presionar",
    "presionaban": "presionar",
    "presionabas": "presionar",
    "presionad": "presionar",
    "presionada": "presionar",
    "presionadas": "presionar",
    "presionado": "presionar",
    "presionados": "presionar",
    "presionamos": "presionar",
    "presionan": "presionar",
    "presionando": "presionar",
    "presionantes": "presionante",
    "presionara": "presionar",
    "presionarais": "presionar",
    "presionaran": "presionar",
    "presionaras": "presionar",
    "presionare": "presionar",
    "presionareis": "presionar",
    "presionaremos": "presionar",
    "presionaren": "presionar",
    "presionares": "presionar",
    "presionaron": "presionar",
    "presionará": "presionar",
    "presionarán": "presionar",
    "presionarás": "presionar",
    "presionaré": "presionar",
    "presionaréis": "presionar",
    "presionaría": "presionar",
    "presionaríais": "presionar",
    "presionaríamos": "presionar",
    "presionarían": "presionar",
    "presionarías": "presionar",
    "presionas": "presionar",
    "presionase": "presionar",
    "presionaseis": "presionar",
    "presionasen": "presionar",
    "presionases": "presionar",
    "presionaste": "presionar",
    "presionasteis": "presionar",
    "presione": "presionar",
    "presionemos": "presionar",
    "presionen": "presionar",
    "presiones": "presionar",
    "presiono": "presionar",
    "presionábamos": "presionar",
    "presionáis": "presionar",
    "presionáramos": "presionar",
    "presionáremos": "presionar",
    "presionásemos": "presionar",
    "presioné": "presionar",
    "presionéis": "presionar",
    "presionó": "presionar",
    "presiógrafos": "presiógrafo",
    "presiómetros": "presiómetro",
    "presocrática": "presocrático",
    "presocráticas": "presocrático",
    "presocráticos": "presocrático",
    "presos": "preso",
    "presostatos": "presostato",
    "pressings": "pressing",
    "presta": "presto",
    "prestaba": "prestar",
    "prestabais": "prestar",
    "prestaban": "prestar",
    "prestabas": "prestar",
    "prestables": "prestable",
    "prestacionales": "prestacional",
    "prestaciones": "prestación",
    "prestad": "prestar",
    "prestada": "prestar",
    "prestadas": "prestar",
    "prestado": "prestar",
    "prestadora": "prestador",
    "prestadoras": "prestador",
    "prestadores": "prestador",
    "prestados": "prestar",
    "prestamistas": "prestamista",
    "prestamos": "prestar",
    "prestan": "prestar",
    "prestancias": "prestancia",
    "prestando": "prestar",
    "prestanombres": "prestanombre",
    "prestantes": "prestante",
    "prestara": "prestar",
    "prestarais": "prestar",
    "prestaran": "prestar",
    "prestaras": "prestar",
    "prestare": "prestar",
    "prestareis": "prestar",
    "prestaremos": "prestar",
    "prestaren": "prestar",
    "prestares": "prestar",
    "prestaron": "prestar",
    "prestará": "prestar",
    "prestarán": "prestar",
    "prestarás": "prestar",
    "prestaré": "prestar",
    "prestaréis": "prestar",
    "prestaría": "prestar",
    "prestaríais": "prestar",
    "prestaríamos": "prestar",
    "prestarían": "prestar",
    "prestarías": "prestar",
    "prestas": "presto",
    "prestase": "prestar",
    "prestaseis": "prestar",
    "prestasen": "prestar",
    "prestases": "prestar",
    "prestaste": "prestar",
    "prestasteis": "prestar",
    "prestataria": "prestatario",
    "prestatarias": "prestatario",
    "prestatarios": "prestatario",
    "preste": "prestar",
    "prestemos": "prestar",
    "presten": "prestar",
    "prestes": "preste",
    "prestezas": "presteza",
    "prestidigitaciones": "prestidigitación",
    "prestidigitadora": "prestidigitador",
    "prestidigitadoras": "prestidigitador",
    "prestidigitadores": "prestidigitador",
    "prestigia": "prestigiar",
    "prestigiaba": "prestigiar",
    "prestigiabais": "prestigiar",
    "prestigiaban": "prestigiar",
    "prestigiabas": "prestigiar",
    "prestigiad": "prestigiar",
    "prestigiada": "prestigiar",
    "prestigiadas": "prestigiar",
    "prestigiado": "prestigiar",
    "prestigiadora": "prestigiador",
    "prestigiadoras": "prestigiador",
    "prestigiadores": "prestigiador",
    "prestigiados": "prestigiar",
    "prestigiamos": "prestigiar",
    "prestigian": "prestigiar",
    "prestigiando": "prestigiar",
    "prestigiara": "prestigiar",
    "prestigiarais": "prestigiar",
    "prestigiaran": "prestigiar",
    "prestigiaras": "prestigiar",
    "prestigiare": "prestigiar",
    "prestigiareis": "prestigiar",
    "prestigiaremos": "prestigiar",
    "prestigiaren": "prestigiar",
    "prestigiares": "prestigiar",
    "prestigiaron": "prestigiar",
    "prestigiará": "prestigiar",
    "prestigiarán": "prestigiar",
    "prestigiarás": "prestigiar",
    "prestigiaré": "prestigiar",
    "prestigiaréis": "prestigiar",
    "prestigiaría": "prestigiar",
    "prestigiaríais": "prestigiar",
    "prestigiaríamos": "prestigiar",
    "prestigiarían": "prestigiar",
    "prestigiarías": "prestigiar",
    "prestigias": "prestigiar",
    "prestigiase": "prestigiar",
    "prestigiaseis": "prestigiar",
    "prestigiasen": "prestigiar",
    "prestigiases": "prestigiar",
    "prestigiaste": "prestigiar",
    "prestigiasteis": "prestigiar",
    "prestigie": "prestigiar",
    "prestigiemos": "prestigiar",
    "prestigien": "prestigiar",
    "prestigies": "prestigiar",
    "prestigio": "prestigiar",
    "prestigios": "prestigio",
    "prestigiosa": "prestigioso",
    "prestigiosas": "prestigioso",
    "prestigiosos": "prestigioso",
    "prestigiábamos": "prestigiar",
    "prestigiáis": "prestigiar",
    "prestigiáramos": "prestigiar",
    "prestigiáremos": "prestigiar",
    "prestigiásemos": "prestigiar",
    "prestigié": "prestigiar",
    "prestigiéis": "prestigiar",
    "prestigió": "prestigiar",
    "prestiños": "prestiño",
    "presto": "prestar",
    "prestos": "presto",
    "prestábamos": "prestar",
    "prestáis": "prestar",
    "prestáramos": "prestar",
    "prestáremos": "prestar",
    "prestásemos": "prestar",
    "presté": "prestar",
    "prestéis": "prestar",
    "prestímana": "prestímano",
    "prestímanas": "prestímano",
    "prestímanos": "prestímano",
    "prestó": "prestar",
    "presuma": "presumir",
    "presumamos": "presumir",
    "presuman": "presumir",
    "presumas": "presumir",
    "presume": "presumir",
    "presumen": "presumir",
    "presumes": "presumir",
    "presumibles": "presumible",
    "presumid": "presumir",
    "presumida": "presumir",
    "presumidas": "presumir",
    "presumido": "presumir",
    "presumidos": "presumir",
    "presumiendo": "presumir",
    "presumiera": "presumir",
    "presumierais": "presumir",
    "presumieran": "presumir",
    "presumieras": "presumir",
    "presumiere": "presumir",
    "presumiereis": "presumir",
    "presumieren": "presumir",
    "presumieres": "presumir",
    "presumieron": "presumir",
    "presumiese": "presumir",
    "presumieseis": "presumir",
    "presumiesen": "presumir",
    "presumieses": "presumir",
    "presumimos": "presumir",
    "presumiremos": "presumir",
    "presumirá": "presumir",
    "presumirán": "presumir",
    "presumirás": "presumir",
    "presumiré": "presumir",
    "presumiréis": "presumir",
    "presumiría": "presumir",
    "presumiríais": "presumir",
    "presumiríamos": "presumir",
    "presumirían": "presumir",
    "presumirías": "presumir",
    "presumiste": "presumir",
    "presumisteis": "presumir",
    "presumiéramos": "presumir",
    "presumiéremos": "presumir",
    "presumiésemos": "presumir",
    "presumió": "presumir",
    "presumo": "presumir",
    "presumáis": "presumir",
    "presumí": "presumir",
    "presumía": "presumir",
    "presumíais": "presumir",
    "presumíamos": "presumir",
    "presumían": "presumir",
    "presumías": "presumir",
    "presumís": "presumir",
    "presunciones": "presunción",
    "presunta": "presunto",
    "presuntas": "presunto",
    "presuntiva": "presuntivo",
    "presuntivas": "presuntivo",
    "presuntivos": "presuntivo",
    "presuntos": "presunto",
    "presuntuosa": "presuntuoso",
    "presuntuosas": "presuntuoso",
    "presuntuosidades": "presuntuosidad",
    "presuntuosos": "presuntuoso",
    "presupondremos": "presuponer",
    "presupondrá": "presuponer",
    "presupondrán": "presuponer",
    "presupondrás": "presuponer",
    "presupondré": "presuponer",
    "presupondréis": "presuponer",
    "presupondría": "presuponer",
    "presupondríais": "presuponer",
    "presupondríamos": "presuponer",
    "presupondrían": "presuponer",
    "presupondrías": "presuponer",
    "presupone": "presuponer",
    "presuponed": "presuponer",
    "presuponemos": "presuponer",
    "presuponen": "presuponer",
    "presupones": "presuponer",
    "presuponga": "presuponer",
    "presupongamos": "presuponer",
    "presupongan": "presuponer",
    "presupongas": "presuponer",
    "presupongo": "presuponer",
    "presupongáis": "presuponer",
    "presuponiendo": "presuponer",
    "presuponéis": "presuponer",
    "presuponía": "presuponer",
    "presuponíais": "presuponer",
    "presuponíamos": "presuponer",
    "presuponían": "presuponer",
    "presuponías": "presuponer",
    "presuposiciones": "presuposición",
    "presupuesta": "presupuestar",
    "presupuestaba": "presupuestar",
    "presupuestabais": "presupuestar",
    "presupuestaban": "presupuestar",
    "presupuestabas": "presupuestar",
    "presupuestaciones": "presupuestación",
    "presupuestad": "presupuestar",
    "presupuestada": "presupuestar",
    "presupuestadas": "presupuestar",
    "presupuestado": "presupuestar",
    "presupuestados": "presupuestar",
    "presupuestales": "presupuestal",
    "presupuestamos": "presupuestar",
    "presupuestan": "presupuestar",
    "presupuestando": "presupuestar",
    "presupuestara": "presupuestar",
    "presupuestarais": "presupuestar",
    "presupuestaran": "presupuestar",
    "presupuestaras": "presupuestar",
    "presupuestare": "presupuestar",
    "presupuestareis": "presupuestar",
    "presupuestaremos": "presupuestar",
    "presupuestaren": "presupuestar",
    "presupuestares": "presupuestar",
    "presupuestaria": "presupuestario",
    "presupuestarias": "presupuestario",
    "presupuestarios": "presupuestario",
    "presupuestaron": "presupuestar",
    "presupuestará": "presupuestar",
    "presupuestarán": "presupuestar",
    "presupuestarás": "presupuestar",
    "presupuestaré": "presupuestar",
    "presupuestaréis": "presupuestar",
    "presupuestaría": "presupuestar",
    "presupuestaríais": "presupuestar",
    "presupuestaríamos": "presupuestar",
    "presupuestarían": "presupuestar",
    "presupuestarías": "presupuestar",
    "presupuestas": "presupuestar",
    "presupuestase": "presupuestar",
    "presupuestaseis": "presupuestar",
    "presupuestasen": "presupuestar",
    "presupuestases": "presupuestar",
    "presupuestaste": "presupuestar",
    "presupuestasteis": "presupuestar",
    "presupueste": "presupuestar",
    "presupuestemos": "presupuestar",
    "presupuesten": "presupuestar",
    "presupuestes": "presupuestar",
    "presupuesto": "presupuestar",
    "presupuestos": "presupuesto",
    "presupuestábamos": "presupuestar",
    "presupuestáis": "presupuestar",
    "presupuestáramos": "presupuestar",
    "presupuestáremos": "presupuestar",
    "presupuestásemos": "presupuestar",
    "presupuesté": "presupuestar",
    "presupuestéis": "presupuestar",
    "presupuestó": "presupuestar",
    "presupuse": "presuponer",
    "presupusiera": "presuponer",
    "presupusierais": "presuponer",
    "presupusieran": "presuponer",
    "presupusieras": "presuponer",
    "presupusiere": "presuponer",
    "presupusiereis": "presuponer",
    "presupusieren": "presuponer",
    "presupusieres": "presuponer",
    "presupusieron": "presuponer",
    "presupusiese": "presuponer",
    "presupusieseis": "presuponer",
    "presupusiesen": "presuponer",
    "presupusieses": "presuponer",
    "presupusimos": "presuponer",
    "presupusiste": "presuponer",
    "presupusisteis": "presuponer",
    "presupusiéramos": "presuponer",
    "presupusiéremos": "presuponer",
    "presupusiésemos": "presuponer",
    "presupuso": "presuponer",
    "presupón": "presuponer",
    "presuras": "presura",
    "presurice": "presurizar",
    "presuricemos": "presurizar",
    "presuricen": "presurizar",
    "presurices": "presurizar",
    "presuricé": "presurizar",
    "presuricéis": "presurizar",
    "presuriza": "presurizar",
    "presurizaba": "presurizar",
    "presurizabais": "presurizar",
    "presurizaban": "presurizar",
    "presurizabas": "presurizar",
    "presurizaciones": "presurización",
    "presurizad": "presurizar",
    "presurizada": "presurizar",
    "presurizadas": "presurizar",
    "presurizado": "presurizar",
    "presurizados": "presurizar",
    "presurizamos": "presurizar",
    "presurizan": "presurizar",
    "presurizando": "presurizar",
    "presurizara": "presurizar",
    "presurizarais": "presurizar",
    "presurizaran": "presurizar",
    "presurizaras": "presurizar",
    "presurizare": "presurizar",
    "presurizareis": "presurizar",
    "presurizaremos": "presurizar",
    "presurizaren": "presurizar",
    "presurizares": "presurizar",
    "presurizaron": "presurizar",
    "presurizará": "presurizar",
    "presurizarán": "presurizar",
    "presurizarás": "presurizar",
    "presurizaré": "presurizar",
    "presurizaréis": "presurizar",
    "presurizaría": "presurizar",
    "presurizaríais": "presurizar",
    "presurizaríamos": "presurizar",
    "presurizarían": "presurizar",
    "presurizarías": "presurizar",
    "presurizas": "presurizar",
    "presurizase": "presurizar",
    "presurizaseis": "presurizar",
    "presurizasen": "presurizar",
    "presurizases": "presurizar",
    "presurizaste": "presurizar",
    "presurizasteis": "presurizar",
    "presurizo": "presurizar",
    "presurizábamos": "presurizar",
    "presurizáis": "presurizar",
    "presurizáramos": "presurizar",
    "presurizáremos": "presurizar",
    "presurizásemos": "presurizar",
    "presurizó": "presurizar",
    "presurosa": "presuroso",
    "presurosas": "presuroso",
    "presurosos": "presuroso",
    "presídiums": "presídium",
    "pretales": "pretal",
    "pretemporadas": "pretemporada",
    "pretenciosa": "pretencioso",
    "pretenciosas": "pretencioso",
    "pretenciosos": "pretencioso",
    "pretenda": "pretender",
    "pretendamos": "pretender",
    "pretendan": "pretender",
    "pretendas": "pretender",
    "pretende": "pretender",
    "pretended": "pretender",
    "pretendemos": "pretender",
    "pretenden": "pretender",
    "pretenderemos": "pretender",
    "pretenderá": "pretender",
    "pretenderán": "pretender",
    "pretenderás": "pretender",
    "pretenderé": "pretender",
    "pretenderéis": "pretender",
    "pretendería": "pretender",
    "pretenderíais": "pretender",
    "pretenderíamos": "pretender",
    "pretenderían": "pretender",
    "pretenderías": "pretender",
    "pretendes": "pretender",
    "pretendida": "pretender",
    "pretendidas": "pretender",
    "pretendido": "pretender",
    "pretendidos": "pretender",
    "pretendiendo": "pretender",
    "pretendienta": "pretendiente",
    "pretendientas": "pretendiente",
    "pretendientes": "pretendiente",
    "pretendiera": "pretender",
    "pretendierais": "pretender",
    "pretendieran": "pretender",
    "pretendieras": "pretender",
    "pretendiere": "pretender",
    "pretendiereis": "pretender",
    "pretendieren": "pretender",
    "pretendieres": "pretender",
    "pretendieron": "pretender",
    "pretendiese": "pretender",
    "pretendieseis": "pretender",
    "pretendiesen": "pretender",
    "pretendieses": "pretender",
    "pretendimos": "pretender",
    "pretendiste": "pretender",
    "pretendisteis": "pretender",
    "pretendiéramos": "pretender",
    "pretendiéremos": "pretender",
    "pretendiésemos": "pretender",
    "pretendió": "pretender",
    "pretendo": "pretender",
    "pretendáis": "pretender",
    "pretendéis": "pretender",
    "pretendí": "pretender",
    "pretendía": "pretender",
    "pretendíais": "pretender",
    "pretendíamos": "pretender",
    "pretendían": "pretender",
    "pretendías": "pretender",
    "pretensa": "pretensar",
    "pretensaba": "pretensar",
    "pretensabais": "pretensar",
    "pretensaban": "pretensar",
    "pretensabas": "pretensar",
    "pretensad": "pretensar",
    "pretensada": "pretensar",
    "pretensadas": "pretensar",
    "pretensado": "pretensar",
    "pretensados": "pretensar",
    "pretensamos": "pretensar",
    "pretensan": "pretensar",
    "pretensando": "pretensar",
    "pretensara": "pretensar",
    "pretensarais": "pretensar",
    "pretensaran": "pretensar",
    "pretensaras": "pretensar",
    "pretensare": "pretensar",
    "pretensareis": "pretensar",
    "pretensaremos": "pretensar",
    "pretensaren": "pretensar",
    "pretensares": "pretensar",
    "pretensaron": "pretensar",
    "pretensará": "pretensar",
    "pretensarán": "pretensar",
    "pretensarás": "pretensar",
    "pretensaré": "pretensar",
    "pretensaréis": "pretensar",
    "pretensaría": "pretensar",
    "pretensaríais": "pretensar",
    "pretensaríamos": "pretensar",
    "pretensarían": "pretensar",
    "pretensarías": "pretensar",
    "pretensas": "pretensar",
    "pretensase": "pretensar",
    "pretensaseis": "pretensar",
    "pretensasen": "pretensar",
    "pretensases": "pretensar",
    "pretensaste": "pretensar",
    "pretensasteis": "pretensar",
    "pretense": "pretensar",
    "pretensemos": "pretensar",
    "pretensen": "pretensar",
    "pretenses": "pretensar",
    "pretensiones": "pretensión",
    "pretenso": "pretensar",
    "pretensora": "pretensor",
    "pretensoras": "pretensor",
    "pretensores": "pretensor",
    "pretensábamos": "pretensar",
    "pretensáis": "pretensar",
    "pretensáramos": "pretensar",
    "pretensáremos": "pretensar",
    "pretensásemos": "pretensar",
    "pretensé": "pretensar",
    "pretenséis": "pretensar",
    "pretensó": "pretensar",
    "pretera": "preterir",
    "preteramos": "preterir",
    "preteran": "preterir",
    "preteras": "preterir",
    "pretere": "preterir",
    "preteren": "preterir",
    "preteres": "preterir",
    "pretericiones": "preterición",
    "preterid": "preterir",
    "preterida": "preterir",
    "preteridas": "preterir",
    "preterido": "preterir",
    "preteridos": "preterir",
    "preteriendo": "preterir",
    "preteriera": "preterir",
    "preterierais": "preterir",
    "preterieran": "preterir",
    "preterieras": "preterir",
    "preteriere": "preterir",
    "preteriereis": "preterir",
    "preterieren": "preterir",
    "preterieres": "preterir",
    "preterieron": "preterir",
    "preteriese": "preterir",
    "preterieseis": "preterir",
    "preteriesen": "preterir",
    "preterieses": "preterir",
    "preterimos": "preterir",
    "preterintencionales": "preterintencional",
    "preterintencionalidades": "preterintencionalidad",
    "preteriremos": "preterir",
    "preterirá": "preterir",
    "preterirán": "preterir",
    "preterirás": "preterir",
    "preteriré": "preterir",
    "preteriréis": "preterir",
    "preteriría": "preterir",
    "preteriríais": "preterir",
    "preteriríamos": "preterir",
    "preterirían": "preterir",
    "preterirías": "preterir",
    "preteriste": "preterir",
    "preteristeis": "preterir",
    "preteriéramos": "preterir",
    "preteriéremos": "preterir",
    "preteriésemos": "preterir",
    "preterió": "preterir",
    "pretermisiones": "pretermisión",
    "preternaturales": "preternatural",
    "pretero": "preterir",
    "preteráis": "preterir",
    "preterí": "preterir",
    "pretería": "preterir",
    "preteríais": "preterir",
    "preteríamos": "preterir",
    "preterían": "preterir",
    "preterías": "preterir",
    "preterís": "preterir",
    "pretexta": "pretextar",
    "pretextaba": "pretextar",
    "pretextabais": "pretextar",
    "pretextaban": "pretextar",
    "pretextabas": "pretextar",
    "pretextad": "pretextar",
    "pretextada": "pretextar",
    "pretextadas": "pretextar",
    "pretextado": "pretextar",
    "pretextados": "pretextar",
    "pretextamos": "pretextar",
    "pretextan": "pretextar",
    "pretextando": "pretextar",
    "pretextara": "pretextar",
    "pretextarais": "pretextar",
    "pretextaran": "pretextar",
    "pretextaras": "pretextar",
    "pretextare": "pretextar",
    "pretextareis": "pretextar",
    "pretextaremos": "pretextar",
    "pretextaren": "pretextar",
    "pretextares": "pretextar",
    "pretextaron": "pretextar",
    "pretextará": "pretextar",
    "pretextarán": "pretextar",
    "pretextarás": "pretextar",
    "pretextaré": "pretextar",
    "pretextaréis": "pretextar",
    "pretextaría": "pretextar",
    "pretextaríais": "pretextar",
    "pretextaríamos": "pretextar",
    "pretextarían": "pretextar",
    "pretextarías": "pretextar",
    "pretextas": "pretextar",
    "pretextase": "pretextar",
    "pretextaseis": "pretextar",
    "pretextasen": "pretextar",
    "pretextases": "pretextar",
    "pretextaste": "pretextar",
    "pretextasteis": "pretextar",
    "pretexte": "pretextar",
    "pretextemos": "pretextar",
    "pretexten": "pretextar",
    "pretextes": "pretextar",
    "pretexto": "pretextar",
    "pretextos": "pretexto",
    "pretextábamos": "pretextar",
    "pretextáis": "pretextar",
    "pretextáramos": "pretextar",
    "pretextáremos": "pretextar",
    "pretextásemos": "pretextar",
    "pretexté": "pretextar",
    "pretextéis": "pretextar",
    "pretextó": "pretextar",
    "pretiles": "pretil",
    "pretinas": "pretina",
    "pretinazos": "pretinazo",
    "pretores": "pretor",
    "pretoria": "pretorio",
    "pretoriales": "pretorial",
    "pretoriana": "pretoriano",
    "pretorianas": "pretoriano",
    "pretorianismos": "pretorianismo",
    "pretorianos": "pretoriano",
    "pretorias": "pretorio",
    "pretorienses": "pretoriense",
    "pretorios": "pretorio",
    "pretragiales": "pretragial",
    "pretratamientos": "pretratamiento",
    "pretrituraciones": "pretrituración",
    "pretrituradores": "pretriturador",
    "preturas": "pretura",
    "pretérita": "pretérito",
    "pretéritas": "pretérito",
    "pretéritos": "pretérito",
    "pretónica": "pretónico",
    "pretónicas": "pretónico",
    "pretónicos": "pretónico",
    "prevaldremos": "prevaler",
    "prevaldrá": "prevaler",
    "prevaldrán": "prevaler",
    "prevaldrás": "prevaler",
    "prevaldré": "prevaler",
    "prevaldréis": "prevaler",
    "prevaldría": "prevaler",
    "prevaldríais": "prevaler",
    "prevaldríamos": "prevaler",
    "prevaldrían": "prevaler",
    "prevaldrías": "prevaler",
    "prevale": "prevaler",
    "prevalece": "prevalecer",
    "prevaleced": "prevalecer",
    "prevalecemos": "prevalecer",
    "prevalecen": "prevalecer",
    "prevaleceremos": "prevalecer",
    "prevalecerá": "prevalecer",
    "prevalecerán": "prevalecer",
    "prevalecerás": "prevalecer",
    "prevaleceré": "prevalecer",
    "prevaleceréis": "prevalecer",
    "prevalecería": "prevalecer",
    "prevaleceríais": "prevalecer",
    "prevaleceríamos": "prevalecer",
    "prevalecerían": "prevalecer",
    "prevalecerías": "prevalecer",
    "prevaleces": "prevalecer",
    "prevalecida": "prevalecer",
    "prevalecidas": "prevalecer",
    "prevalecido": "prevalecer",
    "prevalecidos": "prevalecer",
    "prevaleciendo": "prevalecer",
    "prevalecientes": "prevaleciente",
    "prevaleciera": "prevalecer",
    "prevalecierais": "prevalecer",
    "prevalecieran": "prevalecer",
    "prevalecieras": "prevalecer",
    "prevaleciere": "prevalecer",
    "prevaleciereis": "prevalecer",
    "prevalecieren": "prevalecer",
    "prevalecieres": "prevalecer",
    "prevalecieron": "prevalecer",
    "prevaleciese": "prevalecer",
    "prevalecieseis": "prevalecer",
    "prevaleciesen": "prevalecer",
    "prevalecieses": "prevalecer",
    "prevalecimos": "prevalecer",
    "prevaleciste": "prevalecer",
    "prevalecisteis": "prevalecer",
    "prevaleciéramos": "prevalecer",
    "prevaleciéremos": "prevalecer",
    "prevaleciésemos": "prevalecer",
    "prevaleció": "prevalecer",
    "prevalecéis": "prevalecer",
    "prevalecí": "prevalecer",
    "prevalecía": "prevalecer",
    "prevalecíais": "prevalecer",
    "prevalecíamos": "prevalecer",
    "prevalecían": "prevalecer",
    "prevalecías": "prevalecer",
    "prevaled": "prevaler",
    "prevalemos": "prevaler",
    "prevalen": "prevaler",
    "prevalencias": "prevalencia",
    "prevalentes": "prevalente",
    "prevales": "prevaler",
    "prevalezca": "prevalecer",
    "prevalezcamos": "prevalecer",
    "prevalezcan": "prevalecer",
    "prevalezcas": "prevalecer",
    "prevalezco": "prevalecer",
    "prevalezcáis": "prevalecer",
    "prevalga": "prevaler",
    "prevalgamos": "prevaler",
    "prevalgan": "prevaler",
    "prevalgas": "prevaler",
    "prevalgo": "prevaler",
    "prevalgáis": "prevaler",
    "prevalida": "prevaler",
    "prevalidas": "prevaler",
    "prevalido": "prevaler",
    "prevalidos": "prevaler",
    "prevaliendo": "prevaler",
    "prevaliera": "prevaler",
    "prevalierais": "prevaler",
    "prevalieran": "prevaler",
    "prevalieras": "prevaler",
    "prevaliere": "prevaler",
    "prevaliereis": "prevaler",
    "prevalieren": "prevaler",
    "prevalieres": "prevaler",
    "prevalieron": "prevaler",
    "prevaliese": "prevaler",
    "prevalieseis": "prevaler",
    "prevaliesen": "prevaler",
    "prevalieses": "prevaler",
    "prevalimientos": "prevalimiento",
    "prevalimos": "prevaler",
    "prevaliste": "prevaler",
    "prevalisteis": "prevaler",
    "prevaliéramos": "prevaler",
    "prevaliéremos": "prevaler",
    "prevaliésemos": "prevaler",
    "prevalió": "prevaler",
    "prevaléis": "prevaler",
    "prevalí": "prevaler",
    "prevalía": "prevaler",
    "prevalíais": "prevaler",
    "prevalíamos": "prevaler",
    "prevalían": "prevaler",
    "prevalías": "prevaler",
    "prevarica": "prevaricar",
    "prevaricaba": "prevaricar",
    "prevaricabais": "prevaricar",
    "prevaricaban": "prevaricar",
    "prevaricabas": "prevaricar",
    "prevaricaciones": "prevaricación",
    "prevaricad": "prevaricar",
    "prevaricada": "prevaricar",
    "prevaricadas": "prevaricar",
    "prevaricado": "prevaricar",
    "prevaricadora": "prevaricador",
    "prevaricadoras": "prevaricador",
    "prevaricadores": "prevaricador",
    "prevaricados": "prevaricar",
    "prevaricamos": "prevaricar",
    "prevarican": "prevaricar",
    "prevaricando": "prevaricar",
    "prevaricara": "prevaricar",
    "prevaricarais": "prevaricar",
    "prevaricaran": "prevaricar",
    "prevaricaras": "prevaricar",
    "prevaricare": "prevaricar",
    "prevaricareis": "prevaricar",
    "prevaricaremos": "prevaricar",
    "prevaricaren": "prevaricar",
    "prevaricares": "prevaricar",
    "prevaricaron": "prevaricar",
    "prevaricará": "prevaricar",
    "prevaricarán": "prevaricar",
    "prevaricarás": "prevaricar",
    "prevaricaré": "prevaricar",
    "prevaricaréis": "prevaricar",
    "prevaricaría": "prevaricar",
    "prevaricaríais": "prevaricar",
    "prevaricaríamos": "prevaricar",
    "prevaricarían": "prevaricar",
    "prevaricarías": "prevaricar",
    "prevaricas": "prevaricar",
    "prevaricase": "prevaricar",
    "prevaricaseis": "prevaricar",
    "prevaricasen": "prevaricar",
    "prevaricases": "prevaricar",
    "prevaricaste": "prevaricar",
    "prevaricasteis": "prevaricar",
    "prevaricatos": "prevaricato",
    "prevarico": "prevaricar",
    "prevaricábamos": "prevaricar",
    "prevaricáis": "prevaricar",
    "prevaricáramos": "prevaricar",
    "prevaricáremos": "prevaricar",
    "prevaricásemos": "prevaricar",
    "prevaricó": "prevaricar",
    "prevarique": "prevaricar",
    "prevariquemos": "prevaricar",
    "prevariquen": "prevaricar",
    "prevariques": "prevaricar",
    "prevariqué": "prevaricar",
    "prevariquéis": "prevaricar",
    "prevea": "prever",
    "preveamos": "prever",
    "prevean": "prever",
    "preveas": "prever",
    "preved": "prever",
    "prevemos": "prever",
    "preven": "prevenir",
    "prevenciones": "prevención",
    "prevendremos": "prevenir",
    "prevendrá": "prevenir",
    "prevendrán": "prevenir",
    "prevendrás": "prevenir",
    "prevendré": "prevenir",
    "prevendréis": "prevenir",
    "prevendría": "prevenir",
    "prevendríais": "prevenir",
    "prevendríamos": "prevenir",
    "prevendrían": "prevenir",
    "prevendrías": "prevenir",
    "prevenga": "prevenir",
    "prevengamos": "prevenir",
    "prevengan": "prevenir",
    "prevengas": "prevenir",
    "prevengo": "prevenir",
    "prevengáis": "prevenir",
    "prevenid": "prevenir",
    "prevenida": "prevenir",
    "prevenidas": "prevenir",
    "prevenido": "prevenir",
    "prevenidos": "prevenir",
    "prevenimos": "prevenir",
    "preventiva": "preventivo",
    "preventivas": "preventivo",
    "preventivos": "preventivo",
    "preventorios": "preventorio",
    "prevenía": "prevenir",
    "preveníais": "prevenir",
    "preveníamos": "prevenir",
    "prevenían": "prevenir",
    "prevenías": "prevenir",
    "prevenís": "prevenir",
    "preveo": "prever",
    "preveremos": "prever",
    "prevertebrales": "prevertebral",
    "preverá": "prever",
    "preverán": "prever",
    "preverás": "prever",
    "preveré": "prever",
    "preveréis": "prever",
    "prevería": "prever",
    "preveríais": "prever",
    "preveríamos": "prever",
    "preverían": "prever",
    "preverías": "prever",
    "prevesicales": "prevesical",
    "preveáis": "prever",
    "preveía": "prever",
    "preveíais": "prever",
    "preveíamos": "prever",
    "preveían": "prever",
    "preveías": "prever",
    "previa": "previo",
    "previas": "previo",
    "previendo": "prever",
    "previene": "prevenir",
    "previenen": "prevenir",
    "previenes": "prevenir",
    "previera": "prever",
    "previerais": "prever",
    "previeran": "prever",
    "previeras": "prever",
    "previere": "prever",
    "previereis": "prever",
    "previeren": "prever",
    "previeres": "prever",
    "previeron": "prever",
    "previese": "prever",
    "previeseis": "prever",
    "previesen": "prever",
    "previeses": "prever",
    "previmos": "prever",
    "previne": "prevenir",
    "previniendo": "prevenir",
    "previniera": "prevenir",
    "previnierais": "prevenir",
    "previnieran": "prevenir",
    "previnieras": "prevenir",
    "previniere": "prevenir",
    "previniereis": "prevenir",
    "previnieren": "prevenir",
    "previnieres": "prevenir",
    "previnieron": "prevenir",
    "previniese": "prevenir",
    "previnieseis": "prevenir",
    "previniesen": "prevenir",
    "previnieses": "prevenir",
    "previnimos": "prevenir",
    "previniste": "prevenir",
    "previnisteis": "prevenir",
    "previniéramos": "prevenir",
    "previniéremos": "prevenir",
    "previniésemos": "prevenir",
    "previno": "prevenir",
    "previos": "previo",
    "previsibilidades": "previsibilidad",
    "previsibles": "previsible",
    "previsionales": "previsional",
    "previsiones": "previsión",
    "previsora": "previsor",
    "previsoras": "previsor",
    "previsores": "previsor",
    "prevista": "prever",
    "previstas": "prever",
    "previste": "prever",
    "previsteis": "prever",
    "previsto": "prever",
    "previstos": "prever",
    "previéramos": "prever",
    "previéremos": "prever",
    "previésemos": "prever",
    "previó": "prever",
    "prevulcanizaciones": "prevulcanización",
    "prevé": "prever",
    "prevéis": "prever",
    "prevén": "prever",
    "prevés": "prever",
    "preví": "prever",
    "preámbulos": "preámbulo",
    "preña": "preñar",
    "preñaba": "preñar",
    "preñabais": "preñar",
    "preñaban": "preñar",
    "preñabas": "preñar",
    "preñad": "preñar",
    "preñada": "preñar",
    "preñadas": "preñar",
    "preñado": "preñar",
    "preñados": "preñar",
    "preñamos": "preñar",
    "preñan": "preñar",
    "preñando": "preñar",
    "preñara": "preñar",
    "preñarais": "preñar",
    "preñaran": "preñar",
    "preñaras": "preñar",
    "preñare": "preñar",
    "preñareis": "preñar",
    "preñaremos": "preñar",
    "preñaren": "preñar",
    "preñares": "preñar",
    "preñaron": "preñar",
    "preñará": "preñar",
    "preñarán": "preñar",
    "preñarás": "preñar",
    "preñaré": "preñar",
    "preñaréis": "preñar",
    "preñaría": "preñar",
    "preñaríais": "preñar",
    "preñaríamos": "preñar",
    "preñarían": "preñar",
    "preñarías": "preñar",
    "preñas": "preñar",
    "preñase": "preñar",
    "preñaseis": "preñar",
    "preñasen": "preñar",
    "preñases": "preñar",
    "preñaste": "preñar",
    "preñasteis": "preñar",
    "preñe": "preñar",
    "preñeces": "preñez",
    "preñemos": "preñar",
    "preñen": "preñar",
    "preñes": "preñar",
    "preño": "preñar",
    "preñábamos": "preñar",
    "preñáis": "preñar",
    "preñáramos": "preñar",
    "preñáremos": "preñar",
    "preñásemos": "preñar",
    "preñé": "preñar",
    "preñéis": "preñar",
    "preñó": "preñar",
    "preóptica": "preóptico",
    "preópticas": "preóptico",
    "preópticos": "preóptico",
    "priapismos": "priapismo",
    "priapuloideos": "priapuloideo",
    "priapúlidos": "priapúlido",
    "priceitas": "priceita",
    "priesas": "priesa",
    "prieta": "prieto",
    "prietas": "prieto",
    "prietos": "prieto",
    "prima": "primo",
    "primaba": "primar",
    "primabais": "primar",
    "primaban": "primar",
    "primabas": "primar",
    "primacías": "primacía",
    "primad": "primar",
    "primada": "primar",
    "primadas": "primar",
    "primado": "primar",
    "primadonas": "primadona",
    "primados": "primar",
    "primala": "primal",
    "primalas": "primal",
    "primales": "primal",
    "primamos": "primar",
    "priman": "primar",
    "primando": "primar",
    "primara": "primar",
    "primarais": "primar",
    "primaran": "primar",
    "primaras": "primar",
    "primare": "primar",
    "primareis": "primar",
    "primaremos": "primar",
    "primaren": "primar",
    "primares": "primar",
    "primaria": "primario",
    "primarias": "primario",
    "primariedades": "primariedad",
    "primarios": "primario",
    "primaron": "primar",
    "primarosoles": "primarosol",
    "primará": "primar",
    "primarán": "primar",
    "primarás": "primar",
    "primaré": "primar",
    "primaréis": "primar",
    "primaría": "primar",
    "primaríais": "primar",
    "primaríamos": "primar",
    "primarían": "primar",
    "primarías": "primar",
    "primas": "primo",
    "primase": "primar",
    "primaseis": "primar",
    "primasen": "primar",
    "primases": "primar",
    "primaste": "primar",
    "primasteis": "primar",
    "primates": "primate",
    "primaturas": "primatura",
    "primaverales": "primaveral",
    "primaveras": "primavera",
    "primazgos": "primazgo",
    "prime": "primar",
    "primemos": "primar",
    "primen": "primar",
    "primer": "﻿1",
    "primera": "primero",
    "primeras": "primero",
    "primeriza": "primerizo",
    "primerizas": "primerizo",
    "primerizos": "primerizo",
    "primero": "1",
    "primeros": "primero",
    "primerísima": "primerísimo",
    "primerísimas": "primerísimo",
    "primerísimos": "primerísimo",
    "primes": "primar",
    "primeva": "primevo",
    "primevas": "primevo",
    "primevos": "primevo",
    "primiceria": "primicerio",
    "primicerias": "primicerio",
    "primicerios": "primicerio",
    "primichones": "primichón",
    "primiciales": "primicial",
    "primicias": "primicia",
    "primigenia": "primigenio",
    "primigenias": "primigenio",
    "primigenios": "primigenio",
    "primillas": "primilla",
    "primitas": "primita",
    "primitiva": "primitivo",
    "primitivas": "primitivo",
    "primitivismos": "primitivismo",
    "primitivos": "primitivo",
    "primo": "primar",
    "primogenituras": "primogenitura",
    "primogénita": "primogénito",
    "primogénitas": "primogénito",
    "primogénitos": "primogénito",
    "primordiales": "primordial",
    "primordios": "primordio",
    "primorea": "primorear",
    "primoreaba": "primorear",
    "primoreabais": "primorear",
    "primoreaban": "primorear",
    "primoreabas": "primorear",
    "primoread": "primorear",
    "primoreada": "primorear",
    "primoreadas": "primorear",
    "primoreado": "primorear",
    "primoreados": "primorear",
    "primoreamos": "primorear",
    "primorean": "primorear",
    "primoreando": "primorear",
    "primoreara": "primorear",
    "primorearais": "primorear",
    "primorearan": "primorear",
    "primorearas": "primorear",
    "primoreare": "primorear",
    "primoreareis": "primorear",
    "primorearemos": "primorear",
    "primorearen": "primorear",
    "primoreares": "primorear",
    "primorearon": "primorear",
    "primoreará": "primorear",
    "primorearán": "primorear",
    "primorearás": "primorear",
    "primorearé": "primorear",
    "primorearéis": "primorear",
    "primorearía": "primorear",
    "primorearíais": "primorear",
    "primorearíamos": "primorear",
    "primorearían": "primorear",
    "primorearías": "primorear",
    "primoreas": "primorear",
    "primorease": "primorear",
    "primoreaseis": "primorear",
    "primoreasen": "primorear",
    "primoreases": "primorear",
    "primoreaste": "primorear",
    "primoreasteis": "primorear",
    "primoree": "primorear",
    "primoreemos": "primorear",
    "primoreen": "primorear",
    "primorees": "primorear",
    "primoreo": "primorear",
    "primores": "primor",
    "primoreábamos": "primorear",
    "primoreáis": "primorear",
    "primoreáramos": "primorear",
    "primoreáremos": "primorear",
    "primoreásemos": "primorear",
    "primoreé": "primorear",
    "primoreéis": "primorear",
    "primoreó": "primorear",
    "primorosa": "primoroso",
    "primorosas": "primoroso",
    "primorosos": "primoroso",
    "primos": "primo",
    "primulinas": "primulina",
    "primulácea": "primuláceo",
    "primuláceas": "primuláceo",
    "primuláceos": "primuláceo",
    "primábamos": "primar",
    "primáis": "primar",
    "primáramos": "primar",
    "primáremos": "primar",
    "primásemos": "primar",
    "primé": "primar",
    "priméis": "primar",
    "primíparas": "primípara",
    "primó": "primar",
    "princesas": "princesa",
    "principadas": "principada",
    "principados": "principado",
    "principales": "principal",
    "principesca": "principesco",
    "principescas": "principesco",
    "principescos": "principesco",
    "principia": "principiar",
    "principiaba": "principiar",
    "principiabais": "principiar",
    "principiaban": "principiar",
    "principiabas": "principiar",
    "principiad": "principiar",
    "principiada": "principiar",
    "principiadas": "principiar",
    "principiado": "principiar",
    "principiadora": "principiador",
    "principiadoras": "principiador",
    "principiadores": "principiador",
    "principiados": "principiar",
    "principiamos": "principiar",
    "principian": "principiar",
    "principiando": "principiar",
    "principianta": "principiante",
    "principiantas": "principiante",
    "principiantes": "principiante",
    "principiara": "principiar",
    "principiarais": "principiar",
    "principiaran": "principiar",
    "principiaras": "principiar",
    "principiare": "principiar",
    "principiareis": "principiar",
    "principiaremos": "principiar",
    "principiaren": "principiar",
    "principiares": "principiar",
    "principiaron": "principiar",
    "principiará": "principiar",
    "principiarán": "principiar",
    "principiarás": "principiar",
    "principiaré": "principiar",
    "principiaréis": "principiar",
    "principiaría": "principiar",
    "principiaríais": "principiar",
    "principiaríamos": "principiar",
    "principiarían": "principiar",
    "principiarías": "principiar",
    "principias": "principiar",
    "principiase": "principiar",
    "principiaseis": "principiar",
    "principiasen": "principiar",
    "principiases": "principiar",
    "principiaste": "principiar",
    "principiasteis": "principiar",
    "principie": "principiar",
    "principiemos": "principiar",
    "principien": "principiar",
    "principies": "principiar",
    "principio": "principiar",
    "principios": "principio",
    "principiábamos": "principiar",
    "principiáis": "principiar",
    "principiáramos": "principiar",
    "principiáremos": "principiar",
    "principiásemos": "principiar",
    "principié": "principiar",
    "principiéis": "principiar",
    "principió": "principiar",
    "pringa": "pringar",
    "pringaba": "pringar",
    "pringabais": "pringar",
    "pringaban": "pringar",
    "pringabas": "pringar",
    "pringad": "pringar",
    "pringada": "pringar",
    "pringadas": "pringar",
    "pringado": "pringar",
    "pringados": "pringar",
    "pringamos": "pringar",
    "pringan": "pringar",
    "pringando": "pringar",
    "pringara": "pringar",
    "pringarais": "pringar",
    "pringaran": "pringar",
    "pringaras": "pringar",
    "pringare": "pringar",
    "pringareis": "pringar",
    "pringaremos": "pringar",
    "pringaren": "pringar",
    "pringares": "pringar",
    "pringaron": "pringar",
    "pringará": "pringar",
    "pringarán": "pringar",
    "pringarás": "pringar",
    "pringaré": "pringar",
    "pringaréis": "pringar",
    "pringaría": "pringar",
    "pringaríais": "pringar",
    "pringaríamos": "pringar",
    "pringarían": "pringar",
    "pringarías": "pringar",
    "pringas": "pringar",
    "pringase": "pringar",
    "pringaseis": "pringar",
    "pringasen": "pringar",
    "pringases": "pringar",
    "pringaste": "pringar",
    "pringasteis": "pringar",
    "pringo": "pringar",
    "pringona": "pringón",
    "pringonas": "pringón",
    "pringones": "pringón",
    "pringores": "pringor",
    "pringosa": "pringoso",
    "pringosas": "pringoso",
    "pringosos": "pringoso",
    "pringotes": "pringote",
    "pringue": "pringar",
    "pringuemos": "pringar",
    "pringuen": "pringar",
    "pringues": "pringue",
    "pringué": "pringar",
    "pringuéis": "pringar",
    "pringábamos": "pringar",
    "pringáis": "pringar",
    "pringáramos": "pringar",
    "pringáremos": "pringar",
    "pringásemos": "pringar",
    "pringó": "pringar",
    "priora": "prior",
    "priorales": "prioral",
    "prioras": "prior",
    "prioratos": "priorato",
    "priorazgos": "priorazgo",
    "priores": "prior",
    "priorice": "priorizar",
    "prioricemos": "priorizar",
    "prioricen": "priorizar",
    "priorices": "priorizar",
    "prioricé": "priorizar",
    "prioricéis": "priorizar",
    "prioridades": "prioridad",
    "prioritaria": "prioritario",
    "prioritarias": "prioritario",
    "prioritarios": "prioritario",
    "prioriza": "priorizar",
    "priorizaba": "priorizar",
    "priorizabais": "priorizar",
    "priorizaban": "priorizar",
    "priorizabas": "priorizar",
    "priorizaciones": "priorización",
    "priorizad": "priorizar",
    "priorizada": "priorizar",
    "priorizadas": "priorizar",
    "priorizado": "priorizar",
    "priorizados": "priorizar",
    "priorizamos": "priorizar",
    "priorizan": "priorizar",
    "priorizando": "priorizar",
    "priorizara": "priorizar",
    "priorizarais": "priorizar",
    "priorizaran": "priorizar",
    "priorizaras": "priorizar",
    "priorizare": "priorizar",
    "priorizareis": "priorizar",
    "priorizaremos": "priorizar",
    "priorizaren": "priorizar",
    "priorizares": "priorizar",
    "priorizaron": "priorizar",
    "priorizará": "priorizar",
    "priorizarán": "priorizar",
    "priorizarás": "priorizar",
    "priorizaré": "priorizar",
    "priorizaréis": "priorizar",
    "priorizaría": "priorizar",
    "priorizaríais": "priorizar",
    "priorizaríamos": "priorizar",
    "priorizarían": "priorizar",
    "priorizarías": "priorizar",
    "priorizas": "priorizar",
    "priorizase": "priorizar",
    "priorizaseis": "priorizar",
    "priorizasen": "priorizar",
    "priorizases": "priorizar",
    "priorizaste": "priorizar",
    "priorizasteis": "priorizar",
    "priorizo": "priorizar",
    "priorizábamos": "priorizar",
    "priorizáis": "priorizar",
    "priorizáramos": "priorizar",
    "priorizáremos": "priorizar",
    "priorizásemos": "priorizar",
    "priorizó": "priorizar",
    "priostes": "prioste",
    "prisas": "prisa",
    "priscales": "priscal",
    "prisciliana": "prisciliano",
    "priscilianas": "prisciliano",
    "priscilianismos": "priscilianismo",
    "priscilianistas": "priscilianista",
    "priscilianos": "prisciliano",
    "prisionera": "prisionero",
    "prisioneras": "prisionero",
    "prisioneros": "prisionero",
    "prisiones": "prisión",
    "prisionizaciones": "prisionización",
    "prismas": "prisma",
    "prismatoides": "prismatoide",
    "prismoidales": "prismoidal",
    "prismática": "prismático",
    "prismáticas": "prismático",
    "prismáticos": "prismático",
    "prisómetros": "prisómetro",
    "priva": "privar",
    "privaba": "privar",
    "privabais": "privar",
    "privaban": "privar",
    "privabas": "privar",
    "privacidades": "privacidad",
    "privaciones": "privación",
    "privad": "privar",
    "privada": "privar",
    "privadas": "privar",
    "privaderos": "privadero",
    "privado": "privar",
    "privados": "privar",
    "privamos": "privar",
    "privan": "privar",
    "privando": "privar",
    "privanzas": "privanza",
    "privara": "privar",
    "privarais": "privar",
    "privaran": "privar",
    "privaras": "privar",
    "privare": "privar",
    "privareis": "privar",
    "privaremos": "privar",
    "privaren": "privar",
    "privares": "privar",
    "privaron": "privar",
    "privará": "privar",
    "privarán": "privar",
    "privarás": "privar",
    "privaré": "privar",
    "privaréis": "privar",
    "privaría": "privar",
    "privaríais": "privar",
    "privaríamos": "privar",
    "privarían": "privar",
    "privarías": "privar",
    "privas": "privar",
    "privase": "privar",
    "privaseis": "privar",
    "privasen": "privar",
    "privases": "privar",
    "privaste": "privar",
    "privasteis": "privar",
    "privatice": "privatizar",
    "privaticemos": "privatizar",
    "privaticen": "privatizar",
    "privatices": "privatizar",
    "privaticidades": "privaticidad",
    "privaticé": "privatizar",
    "privaticéis": "privatizar",
    "privativa": "privativo",
    "privativas": "privativo",
    "privativos": "privativo",
    "privatiza": "privatizar",
    "privatizaba": "privatizar",
    "privatizabais": "privatizar",
    "privatizaban": "privatizar",
    "privatizabas": "privatizar",
    "privatizaciones": "privatización",
    "privatizad": "privatizar",
    "privatizada": "privatizar",
    "privatizadas": "privatizar",
    "privatizado": "privatizar",
    "privatizadora": "privatizador",
    "privatizadoras": "privatizador",
    "privatizadores": "privatizador",
    "privatizados": "privatizar",
    "privatizamos": "privatizar",
    "privatizan": "privatizar",
    "privatizando": "privatizar",
    "privatizara": "privatizar",
    "privatizarais": "privatizar",
    "privatizaran": "privatizar",
    "privatizaras": "privatizar",
    "privatizare": "privatizar",
    "privatizareis": "privatizar",
    "privatizaremos": "privatizar",
    "privatizaren": "privatizar",
    "privatizares": "privatizar",
    "privatizaron": "privatizar",
    "privatizará": "privatizar",
    "privatizarán": "privatizar",
    "privatizarás": "privatizar",
    "privatizaré": "privatizar",
    "privatizaréis": "privatizar",
    "privatizaría": "privatizar",
    "privatizaríais": "privatizar",
    "privatizaríamos": "privatizar",
    "privatizarían": "privatizar",
    "privatizarías": "privatizar",
    "privatizas": "privatizar",
    "privatizase": "privatizar",
    "privatizaseis": "privatizar",
    "privatizasen": "privatizar",
    "privatizases": "privatizar",
    "privatizaste": "privatizar",
    "privatizasteis": "privatizar",
    "privatizo": "privatizar",
    "privatizábamos": "privatizar",
    "privatizáis": "privatizar",
    "privatizáramos": "privatizar",
    "privatizáremos": "privatizar",
    "privatizásemos": "privatizar",
    "privatizó": "privatizar",
    "prive": "privar",
    "privemos": "privar",
    "priven": "privar",
    "prives": "privar",
    "privilegia": "privilegiar",
    "privilegiaba": "privilegiar",
    "privilegiabais": "privilegiar",
    "privilegiaban": "privilegiar",
    "privilegiabas": "privilegiar",
    "privilegiad": "privilegiar",
    "privilegiada": "privilegiar",
    "privilegiadas": "privilegiar",
    "privilegiado": "privilegiar",
    "privilegiados": "privilegiar",
    "privilegiamos": "privilegiar",
    "privilegian": "privilegiar",
    "privilegiando": "privilegiar",
    "privilegiara": "privilegiar",
    "privilegiarais": "privilegiar",
    "privilegiaran": "privilegiar",
    "privilegiaras": "privilegiar",
    "privilegiare": "privilegiar",
    "privilegiareis": "privilegiar",
    "privilegiaremos": "privilegiar",
    "privilegiaren": "privilegiar",
    "privilegiares": "privilegiar",
    "privilegiaron": "privilegiar",
    "privilegiará": "privilegiar",
    "privilegiarán": "privilegiar",
    "privilegiarás": "privilegiar",
    "privilegiaré": "privilegiar",
    "privilegiaréis": "privilegiar",
    "privilegiaría": "privilegiar",
    "privilegiaríais": "privilegiar",
    "privilegiaríamos": "privilegiar",
    "privilegiarían": "privilegiar",
    "privilegiarías": "privilegiar",
    "privilegias": "privilegiar",
    "privilegiase": "privilegiar",
    "privilegiaseis": "privilegiar",
    "privilegiasen": "privilegiar",
    "privilegiases": "privilegiar",
    "privilegiaste": "privilegiar",
    "privilegiasteis": "privilegiar",
    "privilegiativa": "privilegiativo",
    "privilegiativas": "privilegiativo",
    "privilegiativos": "privilegiativo",
    "privilegie": "privilegiar",
    "privilegiemos": "privilegiar",
    "privilegien": "privilegiar",
    "privilegies": "privilegiar",
    "privilegio": "privilegiar",
    "privilegios": "privilegio",
    "privilegiábamos": "privilegiar",
    "privilegiáis": "privilegiar",
    "privilegiáramos": "privilegiar",
    "privilegiáremos": "privilegiar",
    "privilegiásemos": "privilegiar",
    "privilegié": "privilegiar",
    "privilegiéis": "privilegiar",
    "privilegió": "privilegiar",
    "privo": "privar",
    "privábamos": "privar",
    "priváis": "privar",
    "priváramos": "privar",
    "priváremos": "privar",
    "privásemos": "privar",
    "privé": "privar",
    "privéis": "privar",
    "privó": "privar",
    "proacelerinas": "proacelerina",
    "proales": "proal",
    "proas": "proa",
    "proba": "probo",
    "probaba": "probar",
    "probabais": "probar",
    "probaban": "probar",
    "probabas": "probar",
    "probabilidades": "probabilidad",
    "probabilismos": "probabilismo",
    "probabilistas": "probabilista",
    "probabilística": "probabilístico",
    "probabilísticas": "probabilístico",
    "probabilísticos": "probabilístico",
    "probables": "probable",
    "probaciones": "probación",
    "probad": "probar",
    "probada": "probar",
    "probadas": "probar",
    "probado": "probar",
    "probadora": "probador",
    "probadoras": "probador",
    "probadores": "probador",
    "probados": "probar",
    "probamos": "probar",
    "probando": "probar",
    "probanzas": "probanza",
    "probara": "probar",
    "probarais": "probar",
    "probaran": "probar",
    "probaras": "probar",
    "probare": "probar",
    "probareis": "probar",
    "probaremos": "probar",
    "probaren": "probar",
    "probares": "probar",
    "probaron": "probar",
    "probará": "probar",
    "probarán": "probar",
    "probarás": "probar",
    "probaré": "probar",
    "probaréis": "probar",
    "probaría": "probar",
    "probaríais": "probar",
    "probaríamos": "probar",
    "probarían": "probar",
    "probarías": "probar",
    "probas": "probo",
    "probase": "probar",
    "probaseis": "probar",
    "probasen": "probar",
    "probases": "probar",
    "probaste": "probar",
    "probasteis": "probar",
    "probations": "probation",
    "probatoria": "probatorio",
    "probatorias": "probatorio",
    "probatorios": "probatorio",
    "probaturas": "probatura",
    "probemos": "probar",
    "probetas": "probeta",
    "probidades": "probidad",
    "problemas": "problema",
    "problematice": "problematizar",
    "problematicemos": "problematizar",
    "problematicen": "problematizar",
    "problematices": "problematizar",
    "problematicé": "problematizar",
    "problematicéis": "problematizar",
    "problematismos": "problematismo",
    "problematiza": "problematizar",
    "problematizaba": "problematizar",
    "problematizabais": "problematizar",
    "problematizaban": "problematizar",
    "problematizabas": "problematizar",
    "problematizad": "problematizar",
    "problematizada": "problematizar",
    "problematizadas": "problematizar",
    "problematizado": "problematizar",
    "problematizados": "problematizar",
    "problematizamos": "problematizar",
    "problematizan": "problematizar",
    "problematizando": "problematizar",
    "problematizara": "problematizar",
    "problematizarais": "problematizar",
    "problematizaran": "problematizar",
    "problematizaras": "problematizar",
    "problematizare": "problematizar",
    "problematizareis": "problematizar",
    "problematizaremos": "problematizar",
    "problematizaren": "problematizar",
    "problematizares": "problematizar",
    "problematizaron": "problematizar",
    "problematizará": "problematizar",
    "problematizarán": "problematizar",
    "problematizarás": "problematizar",
    "problematizaré": "problematizar",
    "problematizaréis": "problematizar",
    "problematizaría": "problematizar",
    "problematizaríais": "problematizar",
    "problematizaríamos": "problematizar",
    "problematizarían": "problematizar",
    "problematizarías": "problematizar",
    "problematizas": "problematizar",
    "problematizase": "problematizar",
    "problematizaseis": "problematizar",
    "problematizasen": "problematizar",
    "problematizases": "problematizar",
    "problematizaste": "problematizar",
    "problematizasteis": "problematizar",
    "problematizo": "problematizar",
    "problematizábamos": "problematizar",
    "problematizáis": "problematizar",
    "problematizáramos": "problematizar",
    "problematizáremos": "problematizar",
    "problematizásemos": "problematizar",
    "problematizó": "problematizar",
    "problemática": "problemático",
    "problemáticas": "problemático",
    "problemáticos": "problemático",
    "probos": "probo",
    "proboscídea": "proboscídeo",
    "proboscídeas": "proboscídeo",
    "proboscídeos": "proboscídeo",
    "probábamos": "probar",
    "probáis": "probar",
    "probáramos": "probar",
    "probáremos": "probar",
    "probásemos": "probar",
    "probé": "probar",
    "probéis": "probar",
    "probó": "probar",
    "probóscides": "probóscide",
    "procaces": "procaz",
    "procacidades": "procacidad",
    "procariones": "procarión",
    "procariontes": "procarionte",
    "procariotas": "procariota",
    "procariótica": "procariótico",
    "procarióticas": "procariótico",
    "procarióticos": "procariótico",
    "procazes": "procaz",
    "proceda": "proceder",
    "procedamos": "proceder",
    "procedan": "proceder",
    "procedas": "proceder",
    "procede": "proceder",
    "proceded": "proceder",
    "procedemos": "proceder",
    "proceden": "proceder",
    "procedencias": "procedencia",
    "procedentes": "procedente",
    "procederemos": "proceder",
    "procederes": "proceder",
    "procederá": "proceder",
    "procederán": "proceder",
    "procederás": "proceder",
    "procederé": "proceder",
    "procederéis": "proceder",
    "procedería": "proceder",
    "procederíais": "proceder",
    "procederíamos": "proceder",
    "procederían": "proceder",
    "procederías": "proceder",
    "procedes": "proceder",
    "procedibilidades": "procedibilidad",
    "procedida": "proceder",
    "procedidas": "proceder",
    "procedido": "proceder",
    "procedidos": "proceder",
    "procediendo": "proceder",
    "procediera": "proceder",
    "procedierais": "proceder",
    "procedieran": "proceder",
    "procedieras": "proceder",
    "procediere": "proceder",
    "procediereis": "proceder",
    "procedieren": "proceder",
    "procedieres": "proceder",
    "procedieron": "proceder",
    "procediese": "proceder",
    "procedieseis": "proceder",
    "procediesen": "proceder",
    "procedieses": "proceder",
    "procedimentales": "procedimental",
    "procedimientos": "procedimiento",
    "procedimos": "proceder",
    "procediste": "proceder",
    "procedisteis": "proceder",
    "procediéramos": "proceder",
    "procediéremos": "proceder",
    "procediésemos": "proceder",
    "procedió": "proceder",
    "procedo": "proceder",
    "procedurales": "procedural",
    "procedáis": "proceder",
    "procedéis": "proceder",
    "procedí": "proceder",
    "procedía": "proceder",
    "procedíais": "proceder",
    "procedíamos": "proceder",
    "procedían": "proceder",
    "procedías": "proceder",
    "procelariformes": "procelariforme",
    "procelas": "procela",
    "procelosa": "proceloso",
    "procelosas": "proceloso",
    "procelosos": "proceloso",
    "procera": "prócero",
    "proceras": "prócero",
    "proceratos": "procerato",
    "proceros": "prócero",
    "procesa": "procesar",
    "procesaba": "procesar",
    "procesabais": "procesar",
    "procesaban": "procesar",
    "procesabas": "procesar",
    "procesabilidades": "procesabilidad",
    "procesables": "procesable",
    "procesad": "procesar",
    "procesada": "procesar",
    "procesadas": "procesar",
    "procesado": "procesar",
    "procesadores": "procesador",
    "procesados": "procesar",
    "procesales": "procesal",
    "procesamientos": "procesamiento",
    "procesamos": "procesar",
    "procesan": "procesar",
    "procesando": "procesar",
    "procesara": "procesar",
    "procesarais": "procesar",
    "procesaran": "procesar",
    "procesaras": "procesar",
    "procesare": "procesar",
    "procesareis": "procesar",
    "procesaremos": "procesar",
    "procesaren": "procesar",
    "procesares": "procesar",
    "procesaron": "procesar",
    "procesará": "procesar",
    "procesarán": "procesar",
    "procesarás": "procesar",
    "procesaré": "procesar",
    "procesaréis": "procesar",
    "procesaría": "procesar",
    "procesaríais": "procesar",
    "procesaríamos": "procesar",
    "procesarían": "procesar",
    "procesarías": "procesar",
    "procesas": "procesar",
    "procesase": "procesar",
    "procesaseis": "procesar",
    "procesasen": "procesar",
    "procesases": "procesar",
    "procesaste": "procesar",
    "procesasteis": "procesar",
    "procese": "procesar",
    "procesemos": "procesar",
    "procesen": "procesar",
    "proceses": "procesar",
    "procesionales": "procesional",
    "procesionarias": "procesionaria",
    "procesionarios": "procesionario",
    "procesiones": "procesión",
    "proceso": "procesar",
    "procesos": "proceso",
    "procesuales": "procesual",
    "procesábamos": "procesar",
    "procesáis": "procesar",
    "procesáramos": "procesar",
    "procesáremos": "procesar",
    "procesásemos": "procesar",
    "procesé": "procesar",
    "proceséis": "procesar",
    "procesó": "procesar",
    "proclama": "proclamar",
    "proclamaba": "proclamar",
    "proclamabais": "proclamar",
    "proclamaban": "proclamar",
    "proclamabas": "proclamar",
    "proclamaciones": "proclamación",
    "proclamad": "proclamar",
    "proclamada": "proclamar",
    "proclamadas": "proclamar",
    "proclamado": "proclamar",
    "proclamados": "proclamar",
    "proclamamos": "proclamar",
    "proclaman": "proclamar",
    "proclamando": "proclamar",
    "proclamara": "proclamar",
    "proclamarais": "proclamar",
    "proclamaran": "proclamar",
    "proclamaras": "proclamar",
    "proclamare": "proclamar",
    "proclamareis": "proclamar",
    "proclamaremos": "proclamar",
    "proclamaren": "proclamar",
    "proclamares": "proclamar",
    "proclamaron": "proclamar",
    "proclamará": "proclamar",
    "proclamarán": "proclamar",
    "proclamarás": "proclamar",
    "proclamaré": "proclamar",
    "proclamaréis": "proclamar",
    "proclamaría": "proclamar",
    "proclamaríais": "proclamar",
    "proclamaríamos": "proclamar",
    "proclamarían": "proclamar",
    "proclamarías": "proclamar",
    "proclamas": "proclamar",
    "proclamase": "proclamar",
    "proclamaseis": "proclamar",
    "proclamasen": "proclamar",
    "proclamases": "proclamar",
    "proclamaste": "proclamar",
    "proclamasteis": "proclamar",
    "proclame": "proclamar",
    "proclamemos": "proclamar",
    "proclamen": "proclamar",
    "proclames": "proclamar",
    "proclamo": "proclamar",
    "proclamábamos": "proclamar",
    "proclamáis": "proclamar",
    "proclamáramos": "proclamar",
    "proclamáremos": "proclamar",
    "proclamásemos": "proclamar",
    "proclamé": "proclamar",
    "proclaméis": "proclamar",
    "proclamó": "proclamar",
    "proclives": "proclive",
    "proclividades": "proclividad",
    "proclítica": "proclítico",
    "proclíticas": "proclítico",
    "proclíticos": "proclítico",
    "procomunales": "procomunal",
    "procomunes": "procomún",
    "proconsulada": "proconsulado",
    "proconsuladas": "proconsulado",
    "proconsulados": "proconsulado",
    "proconsulares": "proconsular",
    "procordados": "procordado",
    "procrea": "procrear",
    "procreaba": "procrear",
    "procreabais": "procrear",
    "procreaban": "procrear",
    "procreabas": "procrear",
    "procreaciones": "procreación",
    "procread": "procrear",
    "procreada": "procrear",
    "procreadas": "procrear",
    "procreado": "procrear",
    "procreadora": "procreador",
    "procreadoras": "procreador",
    "procreadores": "procreador",
    "procreados": "procrear",
    "procreamos": "procrear",
    "procrean": "procrear",
    "procreando": "procrear",
    "procreara": "procrear",
    "procrearais": "procrear",
    "procrearan": "procrear",
    "procrearas": "procrear",
    "procreare": "procrear",
    "procreareis": "procrear",
    "procrearemos": "procrear",
    "procrearen": "procrear",
    "procreares": "procrear",
    "procrearon": "procrear",
    "procreará": "procrear",
    "procrearán": "procrear",
    "procrearás": "procrear",
    "procrearé": "procrear",
    "procrearéis": "procrear",
    "procrearía": "procrear",
    "procrearíais": "procrear",
    "procrearíamos": "procrear",
    "procrearían": "procrear",
    "procrearías": "procrear",
    "procreas": "procrear",
    "procrease": "procrear",
    "procreaseis": "procrear",
    "procreasen": "procrear",
    "procreases": "procrear",
    "procreaste": "procrear",
    "procreasteis": "procrear",
    "procree": "procrear",
    "procreemos": "procrear",
    "procreen": "procrear",
    "procrees": "procrear",
    "procreo": "procrear",
    "procreábamos": "procrear",
    "procreáis": "procrear",
    "procreáramos": "procrear",
    "procreáremos": "procrear",
    "procreásemos": "procrear",
    "procreé": "procrear",
    "procreéis": "procrear",
    "procreó": "procrear",
    "procromosomas": "procromosoma",
    "proctologías": "proctología",
    "proctóloga": "proctólogo",
    "proctólogas": "proctólogo",
    "proctólogos": "proctólogo",
    "procura": "procurar",
    "procuraba": "procurar",
    "procurabais": "procurar",
    "procuraban": "procurar",
    "procurabas": "procurar",
    "procuraciones": "procuración",
    "procurad": "procurar",
    "procurada": "procurar",
    "procuradas": "procurar",
    "procurado": "procurar",
    "procuradora": "procurador",
    "procuradoras": "procurador",
    "procuradores": "procurador",
    "procurados": "procurar",
    "procuradurías": "procuraduría",
    "procuramos": "procurar",
    "procuran": "procurar",
    "procurando": "procurar",
    "procurara": "procurar",
    "procurarais": "procurar",
    "procuraran": "procurar",
    "procuraras": "procurar",
    "procurare": "procurar",
    "procurareis": "procurar",
    "procuraremos": "procurar",
    "procuraren": "procurar",
    "procurares": "procurar",
    "procuraron": "procurar",
    "procurará": "procurar",
    "procurarán": "procurar",
    "procurarás": "procurar",
    "procuraré": "procurar",
    "procuraréis": "procurar",
    "procuraría": "procurar",
    "procuraríais": "procurar",
    "procuraríamos": "procurar",
    "procurarían": "procurar",
    "procurarías": "procurar",
    "procuras": "procurar",
    "procurase": "procurar",
    "procuraseis": "procurar",
    "procurasen": "procurar",
    "procurases": "procurar",
    "procuraste": "procurar",
    "procurasteis": "procurar",
    "procure": "procurar",
    "procuremos": "procurar",
    "procuren": "procurar",
    "procures": "procurar",
    "procuro": "procurar",
    "procurábamos": "procurar",
    "procuráis": "procurar",
    "procuráramos": "procurar",
    "procuráremos": "procurar",
    "procurásemos": "procurar",
    "procuré": "procurar",
    "procuréis": "procurar",
    "procuró": "procurar",
    "procámbiums": "procámbium",
    "procáriones": "procarion",
    "procónsules": "procónsul",
    "prodiga": "prodigar",
    "prodigaba": "prodigar",
    "prodigabais": "prodigar",
    "prodigaban": "prodigar",
    "prodigabas": "prodigar",
    "prodigad": "prodigar",
    "prodigada": "prodigar",
    "prodigadas": "prodigar",
    "prodigado": "prodigar",
    "prodigados": "prodigar",
    "prodigalidades": "prodigalidad",
    "prodigamos": "prodigar",
    "prodigan": "prodigar",
    "prodigando": "prodigar",
    "prodigara": "prodigar",
    "prodigarais": "prodigar",
    "prodigaran": "prodigar",
    "prodigaras": "prodigar",
    "prodigare": "prodigar",
    "prodigareis": "prodigar",
    "prodigaremos": "prodigar",
    "prodigaren": "prodigar",
    "prodigares": "prodigar",
    "prodigaron": "prodigar",
    "prodigará": "prodigar",
    "prodigarán": "prodigar",
    "prodigarás": "prodigar",
    "prodigaré": "prodigar",
    "prodigaréis": "prodigar",
    "prodigaría": "prodigar",
    "prodigaríais": "prodigar",
    "prodigaríamos": "prodigar",
    "prodigarían": "prodigar",
    "prodigarías": "prodigar",
    "prodigas": "prodigar",
    "prodigase": "prodigar",
    "prodigaseis": "prodigar",
    "prodigasen": "prodigar",
    "prodigases": "prodigar",
    "prodigaste": "prodigar",
    "prodigasteis": "prodigar",
    "prodigios": "prodigio",
    "prodigiosa": "prodigioso",
    "prodigiosas": "prodigioso",
    "prodigiosos": "prodigioso",
    "prodigo": "prodigar",
    "prodigue": "prodigar",
    "prodiguemos": "prodigar",
    "prodiguen": "prodigar",
    "prodigues": "prodigar",
    "prodigué": "prodigar",
    "prodiguéis": "prodigar",
    "prodigábamos": "prodigar",
    "prodigáis": "prodigar",
    "prodigáramos": "prodigar",
    "prodigáremos": "prodigar",
    "prodigásemos": "prodigar",
    "prodigó": "prodigar",
    "prodrómica": "prodrómico",
    "prodrómicas": "prodrómico",
    "prodrómicos": "prodrómico",
    "producciones": "producción",
    "produce": "producir",
    "producen": "producir",
    "produces": "producir",
    "producibilidades": "producibilidad",
    "producibles": "producible",
    "producid": "producir",
    "producida": "producir",
    "producidas": "producir",
    "producido": "producir",
    "producidos": "producir",
    "produciendo": "producir",
    "producimos": "producir",
    "produciremos": "producir",
    "producirá": "producir",
    "producirán": "producir",
    "producirás": "producir",
    "produciré": "producir",
    "produciréis": "producir",
    "produciría": "producir",
    "produciríais": "producir",
    "produciríamos": "producir",
    "producirían": "producir",
    "producirías": "producir",
    "producta": "producto",
    "productas": "producto",
    "productibilidades": "productibilidad",
    "productibles": "productible",
    "productiva": "productivo",
    "productivas": "productivo",
    "productividades": "productividad",
    "productivismos": "productivismo",
    "productivistas": "productivista",
    "productivos": "productivo",
    "productora": "productor",
    "productoras": "productor",
    "productores": "productor",
    "productos": "producto",
    "producía": "producir",
    "producíais": "producir",
    "producíamos": "producir",
    "producían": "producir",
    "producías": "producir",
    "producís": "producir",
    "produje": "producir",
    "produjera": "producir",
    "produjerais": "producir",
    "produjeran": "producir",
    "produjeras": "producir",
    "produjere": "producir",
    "produjereis": "producir",
    "produjeren": "producir",
    "produjeres": "producir",
    "produjeron": "producir",
    "produjese": "producir",
    "produjeseis": "producir",
    "produjesen": "producir",
    "produjeses": "producir",
    "produjimos": "producir",
    "produjiste": "producir",
    "produjisteis": "producir",
    "produjo": "producir",
    "produjéramos": "producir",
    "produjéremos": "producir",
    "produjésemos": "producir",
    "produzca": "producir",
    "produzcamos": "producir",
    "produzcan": "producir",
    "produzcas": "producir",
    "produzco": "producir",
    "produzcáis": "producir",
    "proeles": "proel",
    "proemiales": "proemial",
    "proemios": "proemio",
    "proenzimas": "proenzima",
    "proeritroblastos": "proeritroblasto",
    "proezas": "proeza",
    "prof": "profesor",
    "profagos": "profago",
    "profana": "profano",
    "profanaba": "profanar",
    "profanabais": "profanar",
    "profanaban": "profanar",
    "profanabas": "profanar",
    "profanaciones": "profanación",
    "profanad": "profanar",
    "profanada": "profanar",
    "profanadas": "profanar",
    "profanado": "profanar",
    "profanadora": "profanador",
    "profanadoras": "profanador",
    "profanadores": "profanador",
    "profanados": "profanar",
    "profanamientos": "profanamiento",
    "profanamos": "profanar",
    "profanan": "profanar",
    "profanando": "profanar",
    "profanara": "profanar",
    "profanarais": "profanar",
    "profanaran": "profanar",
    "profanaras": "profanar",
    "profanare": "profanar",
    "profanareis": "profanar",
    "profanaremos": "profanar",
    "profanaren": "profanar",
    "profanares": "profanar",
    "profanaron": "profanar",
    "profanará": "profanar",
    "profanarán": "profanar",
    "profanarás": "profanar",
    "profanaré": "profanar",
    "profanaréis": "profanar",
    "profanaría": "profanar",
    "profanaríais": "profanar",
    "profanaríamos": "profanar",
    "profanarían": "profanar",
    "profanarías": "profanar",
    "profanas": "profano",
    "profanase": "profanar",
    "profanaseis": "profanar",
    "profanasen": "profanar",
    "profanases": "profanar",
    "profanaste": "profanar",
    "profanasteis": "profanar",
    "profane": "profanar",
    "profanemos": "profanar",
    "profanen": "profanar",
    "profanes": "profanar",
    "profanidades": "profanidad",
    "profano": "profanar",
    "profanos": "profano",
    "profanábamos": "profanar",
    "profanáis": "profanar",
    "profanáramos": "profanar",
    "profanáremos": "profanar",
    "profanásemos": "profanar",
    "profané": "profanar",
    "profanéis": "profanar",
    "profanó": "profanar",
    "profases": "profase",
    "profecticia": "profecticio",
    "profecticias": "profecticio",
    "profecticios": "profecticio",
    "profecías": "profecía",
    "proferid": "proferir",
    "proferida": "proferir",
    "proferidas": "proferir",
    "proferido": "proferir",
    "proferidos": "proferir",
    "proferimos": "proferir",
    "proferiremos": "proferir",
    "proferirá": "proferir",
    "proferirán": "proferir",
    "proferirás": "proferir",
    "proferiré": "proferir",
    "proferiréis": "proferir",
    "proferiría": "proferir",
    "proferiríais": "proferir",
    "proferiríamos": "proferir",
    "proferirían": "proferir",
    "proferirías": "proferir",
    "proferiste": "proferir",
    "proferisteis": "proferir",
    "proferí": "proferir",
    "profería": "proferir",
    "proferíais": "proferir",
    "proferíamos": "proferir",
    "proferían": "proferir",
    "proferías": "proferir",
    "proferís": "proferir",
    "profesa": "profeso",
    "profesaba": "profesar",
    "profesabais": "profesar",
    "profesaban": "profesar",
    "profesabas": "profesar",
    "profesad": "profesar",
    "profesada": "profesar",
    "profesadas": "profesar",
    "profesado": "profesar",
    "profesados": "profesar",
    "profesamos": "profesar",
    "profesan": "profesar",
    "profesando": "profesar",
    "profesara": "profesar",
    "profesarais": "profesar",
    "profesaran": "profesar",
    "profesaras": "profesar",
    "profesare": "profesar",
    "profesareis": "profesar",
    "profesaremos": "profesar",
    "profesaren": "profesar",
    "profesares": "profesar",
    "profesaron": "profesar",
    "profesará": "profesar",
    "profesarán": "profesar",
    "profesarás": "profesar",
    "profesaré": "profesar",
    "profesaréis": "profesar",
    "profesaría": "profesar",
    "profesaríais": "profesar",
    "profesaríamos": "profesar",
    "profesarían": "profesar",
    "profesarías": "profesar",
    "profesas": "profeso",
    "profesase": "profesar",
    "profesaseis": "profesar",
    "profesasen": "profesar",
    "profesases": "profesar",
    "profesaste": "profesar",
    "profesasteis": "profesar",
    "profese": "profesar",
    "profesemos": "profesar",
    "profesen": "profesar",
    "profeses": "profesar",
    "profesionales": "profesional",
    "profesionalice": "profesionalizar",
    "profesionalicemos": "profesionalizar",
    "profesionalicen": "profesionalizar",
    "profesionalices": "profesionalizar",
    "profesionalicé": "profesionalizar",
    "profesionalicéis": "profesionalizar",
    "profesionalidades": "profesionalidad",
    "profesionalismos": "profesionalismo",
    "profesionaliza": "profesionalizar",
    "profesionalizaba": "profesionalizar",
    "profesionalizabais": "profesionalizar",
    "profesionalizaban": "profesionalizar",
    "profesionalizabas": "profesionalizar",
    "profesionalizaciones": "profesionalización",
    "profesionalizad": "profesionalizar",
    "profesionalizada": "profesionalizar",
    "profesionalizadas": "profesionalizar",
    "profesionalizado": "profesionalizar",
    "profesionalizados": "profesionalizar",
    "profesionalizamos": "profesionalizar",
    "profesionalizan": "profesionalizar",
    "profesionalizando": "profesionalizar",
    "profesionalizara": "profesionalizar",
    "profesionalizarais": "profesionalizar",
    "profesionalizaran": "profesionalizar",
    "profesionalizaras": "profesionalizar",
    "profesionalizare": "profesionalizar",
    "profesionalizareis": "profesionalizar",
    "profesionalizaremos": "profesionalizar",
    "profesionalizaren": "profesionalizar",
    "profesionalizares": "profesionalizar",
    "profesionalizaron": "profesionalizar",
    "profesionalizará": "profesionalizar",
    "profesionalizarán": "profesionalizar",
    "profesionalizarás": "profesionalizar",
    "profesionalizaré": "profesionalizar",
    "profesionalizaréis": "profesionalizar",
    "profesionalizaría": "profesionalizar",
    "profesionalizaríais": "profesionalizar",
    "profesionalizaríamos": "profesionalizar",
    "profesionalizarían": "profesionalizar",
    "profesionalizarías": "profesionalizar",
    "profesionalizas": "profesionalizar",
    "profesionalizase": "profesionalizar",
    "profesionalizaseis": "profesionalizar",
    "profesionalizasen": "profesionalizar",
    "profesionalizases": "profesionalizar",
    "profesionalizaste": "profesionalizar",
    "profesionalizasteis": "profesionalizar",
    "profesionalizo": "profesionalizar",
    "profesionalizábamos": "profesionalizar",
    "profesionalizáis": "profesionalizar",
    "profesionalizáramos": "profesionalizar",
    "profesionalizáremos": "profesionalizar",
    "profesionalizásemos": "profesionalizar",
    "profesionalizó": "profesionalizar",
    "profesiones": "profesión",
    "profeso": "profesar",
    "profesora": "profesor",
    "profesorados": "profesorado",
    "profesorales": "profesoral",
    "profesoras": "profesor",
    "profesores": "profesor",
    "profesos": "profeso",
    "professionales": "professional",
    "profesábamos": "profesar",
    "profesáis": "profesar",
    "profesáramos": "profesar",
    "profesáremos": "profesar",
    "profesásemos": "profesar",
    "profesé": "profesar",
    "profeséis": "profesar",
    "profesó": "profesar",
    "profetas": "profeta",
    "profetice": "profetizar",
    "profeticemos": "profetizar",
    "profeticen": "profetizar",
    "profetices": "profetizar",
    "profeticé": "profetizar",
    "profeticéis": "profetizar",
    "profetisas": "profetisa",
    "profetismos": "profetismo",
    "profetiza": "profetizar",
    "profetizaba": "profetizar",
    "profetizabais": "profetizar",
    "profetizaban": "profetizar",
    "profetizabas": "profetizar",
    "profetizad": "profetizar",
    "profetizada": "profetizar",
    "profetizadas": "profetizar",
    "profetizado": "profetizar",
    "profetizadora": "profetizador",
    "profetizadoras": "profetizador",
    "profetizadores": "profetizador",
    "profetizados": "profetizar",
    "profetizamos": "profetizar",
    "profetizan": "profetizar",
    "profetizando": "profetizar",
    "profetizara": "profetizar",
    "profetizarais": "profetizar",
    "profetizaran": "profetizar",
    "profetizaras": "profetizar",
    "profetizare": "profetizar",
    "profetizareis": "profetizar",
    "profetizaremos": "profetizar",
    "profetizaren": "profetizar",
    "profetizares": "profetizar",
    "profetizaron": "profetizar",
    "profetizará": "profetizar",
    "profetizarán": "profetizar",
    "profetizarás": "profetizar",
    "profetizaré": "profetizar",
    "profetizaréis": "profetizar",
    "profetizaría": "profetizar",
    "profetizaríais": "profetizar",
    "profetizaríamos": "profetizar",
    "profetizarían": "profetizar",
    "profetizarías": "profetizar",
    "profetizas": "profetizar",
    "profetizase": "profetizar",
    "profetizaseis": "profetizar",
    "profetizasen": "profetizar",
    "profetizases": "profetizar",
    "profetizaste": "profetizar",
    "profetizasteis": "profetizar",
    "profetizo": "profetizar",
    "profetizábamos": "profetizar",
    "profetizáis": "profetizar",
    "profetizáramos": "profetizar",
    "profetizáremos": "profetizar",
    "profetizásemos": "profetizar",
    "profetizó": "profetizar",
    "profibrinolisinas": "profibrinolisina",
    "proficientes": "proficiente",
    "proficua": "proficuo",
    "proficuas": "proficuo",
    "proficuos": "proficuo",
    "profiera": "proferir",
    "profieran": "proferir",
    "profieras": "proferir",
    "profiere": "proferir",
    "profieren": "proferir",
    "profieres": "proferir",
    "profiero": "proferir",
    "profiláctica": "profiláctico",
    "profilácticas": "profiláctico",
    "profilácticos": "profiláctico",
    "profiramos": "proferir",
    "profiriendo": "proferir",
    "profiriera": "proferir",
    "profirierais": "proferir",
    "profirieran": "proferir",
    "profirieras": "proferir",
    "profiriere": "proferir",
    "profiriereis": "proferir",
    "profirieren": "proferir",
    "profirieres": "proferir",
    "profirieron": "proferir",
    "profiriese": "proferir",
    "profirieseis": "proferir",
    "profiriesen": "proferir",
    "profirieses": "proferir",
    "profiriéramos": "proferir",
    "profiriéremos": "proferir",
    "profiriésemos": "proferir",
    "profirió": "proferir",
    "profiráis": "proferir",
    "profiteroles": "profiterol",
    "proflavinas": "proflavina",
    "profunda": "profundo",
    "profundaba": "profundar",
    "profundabais": "profundar",
    "profundaban": "profundar",
    "profundabas": "profundar",
    "profundad": "profundar",
    "profundada": "profundar",
    "profundadas": "profundar",
    "profundado": "profundar",
    "profundados": "profundar",
    "profundamos": "profundar",
    "profundan": "profundar",
    "profundando": "profundar",
    "profundara": "profundar",
    "profundarais": "profundar",
    "profundaran": "profundar",
    "profundaras": "profundar",
    "profundare": "profundar",
    "profundareis": "profundar",
    "profundaremos": "profundar",
    "profundaren": "profundar",
    "profundares": "profundar",
    "profundaron": "profundar",
    "profundará": "profundar",
    "profundarán": "profundar",
    "profundarás": "profundar",
    "profundaré": "profundar",
    "profundaréis": "profundar",
    "profundaría": "profundar",
    "profundaríais": "profundar",
    "profundaríamos": "profundar",
    "profundarían": "profundar",
    "profundarías": "profundar",
    "profundas": "profundo",
    "profundase": "profundar",
    "profundaseis": "profundar",
    "profundasen": "profundar",
    "profundases": "profundar",
    "profundaste": "profundar",
    "profundasteis": "profundar",
    "profunde": "profundar",
    "profundemos": "profundar",
    "profunden": "profundar",
    "profundes": "profundar",
    "profundice": "profundizar",
    "profundicemos": "profundizar",
    "profundicen": "profundizar",
    "profundices": "profundizar",
    "profundicé": "profundizar",
    "profundicéis": "profundizar",
    "profundidades": "profundidad",
    "profundiza": "profundizar",
    "profundizaba": "profundizar",
    "profundizabais": "profundizar",
    "profundizaban": "profundizar",
    "profundizabas": "profundizar",
    "profundizaciones": "profundización",
    "profundizad": "profundizar",
    "profundizada": "profundizar",
    "profundizadas": "profundizar",
    "profundizado": "profundizar",
    "profundizados": "profundizar",
    "profundizamos": "profundizar",
    "profundizan": "profundizar",
    "profundizando": "profundizar",
    "profundizara": "profundizar",
    "profundizarais": "profundizar",
    "profundizaran": "profundizar",
    "profundizaras": "profundizar",
    "profundizare": "profundizar",
    "profundizareis": "profundizar",
    "profundizaremos": "profundizar",
    "profundizaren": "profundizar",
    "profundizares": "profundizar",
    "profundizaron": "profundizar",
    "profundizará": "profundizar",
    "profundizarán": "profundizar",
    "profundizarás": "profundizar",
    "profundizaré": "profundizar",
    "profundizaréis": "profundizar",
    "profundizaría": "profundizar",
    "profundizaríais": "profundizar",
    "profundizaríamos": "profundizar",
    "profundizarían": "profundizar",
    "profundizarías": "profundizar",
    "profundizas": "profundizar",
    "profundizase": "profundizar",
    "profundizaseis": "profundizar",
    "profundizasen": "profundizar",
    "profundizases": "profundizar",
    "profundizaste": "profundizar",
    "profundizasteis": "profundizar",
    "profundizo": "profundizar",
    "profundizábamos": "profundizar",
    "profundizáis": "profundizar",
    "profundizáramos": "profundizar",
    "profundizáremos": "profundizar",
    "profundizásemos": "profundizar",
    "profundizó": "profundizar",
    "profundo": "profundar",
    "profundos": "profundo",
    "profundábamos": "profundar",
    "profundáis": "profundar",
    "profundáramos": "profundar",
    "profundáremos": "profundar",
    "profundásemos": "profundar",
    "profundé": "profundar",
    "profundéis": "profundar",
    "profundó": "profundar",
    "profusa": "profuso",
    "profusas": "profuso",
    "profusiones": "profusión",
    "profusos": "profuso",
    "profética": "profético",
    "proféticas": "profético",
    "proféticos": "profético",
    "progenies": "progenie",
    "progenitora": "progenitor",
    "progenitoras": "progenitor",
    "progenitores": "progenitor",
    "progenituras": "progenitura",
    "progesteronas": "progesterona",
    "proglosias": "proglosia",
    "prognata": "prognato",
    "prognatas": "prognato",
    "prognatismos": "prognatismo",
    "prognatos": "prognato",
    "prognatómetros": "prognatómetro",
    "prognática": "prognático",
    "prognáticas": "prognático",
    "prognáticos": "prognático",
    "progradaciones": "progradación",
    "progradantes": "progradante",
    "programa": "programar",
    "programaba": "programar",
    "programabais": "programar",
    "programaban": "programar",
    "programabas": "programar",
    "programabilidades": "programabilidad",
    "programables": "programable",
    "programaciones": "programación",
    "programad": "programar",
    "programada": "programar",
    "programadas": "programar",
    "programado": "programar",
    "programadora": "programador",
    "programadoras": "programador",
    "programadores": "programador",
    "programados": "programar",
    "programamos": "programar",
    "programan": "programar",
    "programando": "programar",
    "programara": "programar",
    "programarais": "programar",
    "programaran": "programar",
    "programaras": "programar",
    "programare": "programar",
    "programareis": "programar",
    "programaremos": "programar",
    "programaren": "programar",
    "programares": "programar",
    "programaron": "programar",
    "programará": "programar",
    "programarán": "programar",
    "programarás": "programar",
    "programaré": "programar",
    "programaréis": "programar",
    "programaría": "programar",
    "programaríais": "programar",
    "programaríamos": "programar",
    "programarían": "programar",
    "programarías": "programar",
    "programas": "programar",
    "programase": "programar",
    "programaseis": "programar",
    "programasen": "programar",
    "programases": "programar",
    "programaste": "programar",
    "programasteis": "programar",
    "programe": "programar",
    "programemos": "programar",
    "programen": "programar",
    "programes": "programar",
    "programo": "programar",
    "programábamos": "programar",
    "programáis": "programar",
    "programáramos": "programar",
    "programáremos": "programar",
    "programásemos": "programar",
    "programática": "programático",
    "programáticas": "programático",
    "programáticos": "programático",
    "programé": "programar",
    "programéis": "programar",
    "programó": "programar",
    "progres": "progre",
    "progresa": "progresar",
    "progresaba": "progresar",
    "progresabais": "progresar",
    "progresaban": "progresar",
    "progresabas": "progresar",
    "progresad": "progresar",
    "progresada": "progresar",
    "progresadas": "progresar",
    "progresado": "progresar",
    "progresados": "progresar",
    "progresamos": "progresar",
    "progresan": "progresar",
    "progresando": "progresar",
    "progresara": "progresar",
    "progresarais": "progresar",
    "progresaran": "progresar",
    "progresaras": "progresar",
    "progresare": "progresar",
    "progresareis": "progresar",
    "progresaremos": "progresar",
    "progresaren": "progresar",
    "progresares": "progresar",
    "progresaron": "progresar",
    "progresará": "progresar",
    "progresarán": "progresar",
    "progresarás": "progresar",
    "progresaré": "progresar",
    "progresaréis": "progresar",
    "progresaría": "progresar",
    "progresaríais": "progresar",
    "progresaríamos": "progresar",
    "progresarían": "progresar",
    "progresarías": "progresar",
    "progresas": "progresar",
    "progresase": "progresar",
    "progresaseis": "progresar",
    "progresasen": "progresar",
    "progresases": "progresar",
    "progresaste": "progresar",
    "progresasteis": "progresar",
    "progrese": "progresar",
    "progresemos": "progresar",
    "progresen": "progresar",
    "progreses": "progresar",
    "progresiones": "progresión",
    "progresismos": "progresismo",
    "progresistas": "progresista",
    "progresiva": "progresivo",
    "progresivas": "progresivo",
    "progresividades": "progresividad",
    "progresivos": "progresivo",
    "progreso": "progresar",
    "progresos": "progreso",
    "progresábamos": "progresar",
    "progresáis": "progresar",
    "progresáramos": "progresar",
    "progresáremos": "progresar",
    "progresásemos": "progresar",
    "progresé": "progresar",
    "progreséis": "progresar",
    "progresó": "progresar",
    "prohibamos": "prohibir",
    "prohibiciones": "prohibición",
    "prohibicionismos": "prohibicionismo",
    "prohibicionistas": "prohibicionista",
    "prohibid": "prohibir",
    "prohibida": "prohibir",
    "prohibidas": "prohibir",
    "prohibido": "prohibir",
    "prohibidos": "prohibir",
    "prohibiendo": "prohibir",
    "prohibiera": "prohibir",
    "prohibierais": "prohibir",
    "prohibieran": "prohibir",
    "prohibieras": "prohibir",
    "prohibiere": "prohibir",
    "prohibiereis": "prohibir",
    "prohibieren": "prohibir",
    "prohibieres": "prohibir",
    "prohibieron": "prohibir",
    "prohibiese": "prohibir",
    "prohibieseis": "prohibir",
    "prohibiesen": "prohibir",
    "prohibieses": "prohibir",
    "prohibimos": "prohibir",
    "prohibiremos": "prohibir",
    "prohibirá": "prohibir",
    "prohibirán": "prohibir",
    "prohibirás": "prohibir",
    "prohibiré": "prohibir",
    "prohibiréis": "prohibir",
    "prohibiría": "prohibir",
    "prohibiríais": "prohibir",
    "prohibiríamos": "prohibir",
    "prohibirían": "prohibir",
    "prohibirías": "prohibir",
    "prohibiste": "prohibir",
    "prohibisteis": "prohibir",
    "prohibitiva": "prohibitivo",
    "prohibitivas": "prohibitivo",
    "prohibitivos": "prohibitivo",
    "prohibitoria": "prohibitorio",
    "prohibitorias": "prohibitorio",
    "prohibitorios": "prohibitorio",
    "prohibiéramos": "prohibir",
    "prohibiéremos": "prohibir",
    "prohibiésemos": "prohibir",
    "prohibió": "prohibir",
    "prohibáis": "prohibir",
    "prohibí": "prohibir",
    "prohibía": "prohibir",
    "prohibíais": "prohibir",
    "prohibíamos": "prohibir",
    "prohibían": "prohibir",
    "prohibías": "prohibir",
    "prohibís": "prohibir",
    "prohijaba": "prohijar",
    "prohijabais": "prohijar",
    "prohijaban": "prohijar",
    "prohijabas": "prohijar",
    "prohijad": "prohijar",
    "prohijada": "prohijar",
    "prohijadas": "prohijar",
    "prohijado": "prohijar",
    "prohijadora": "prohijador",
    "prohijadoras": "prohijador",
    "prohijadores": "prohijador",
    "prohijados": "prohijar",
    "prohijamientos": "prohijamiento",
    "prohijamos": "prohijar",
    "prohijando": "prohijar",
    "prohijara": "prohijar",
    "prohijarais": "prohijar",
    "prohijaran": "prohijar",
    "prohijaras": "prohijar",
    "prohijare": "prohijar",
    "prohijareis": "prohijar",
    "prohijaremos": "prohijar",
    "prohijaren": "prohijar",
    "prohijares": "prohijar",
    "prohijaron": "prohijar",
    "prohijará": "prohijar",
    "prohijarán": "prohijar",
    "prohijarás": "prohijar",
    "prohijaré": "prohijar",
    "prohijaréis": "prohijar",
    "prohijaría": "prohijar",
    "prohijaríais": "prohijar",
    "prohijaríamos": "prohijar",
    "prohijarían": "prohijar",
    "prohijarías": "prohijar",
    "prohijase": "prohijar",
    "prohijaseis": "prohijar",
    "prohijasen": "prohijar",
    "prohijases": "prohijar",
    "prohijaste": "prohijar",
    "prohijasteis": "prohijar",
    "prohijemos": "prohijar",
    "prohijábamos": "prohijar",
    "prohijáis": "prohijar",
    "prohijáramos": "prohijar",
    "prohijáremos": "prohijar",
    "prohijásemos": "prohijar",
    "prohijé": "prohijar",
    "prohijéis": "prohijar",
    "prohijó": "prohijar",
    "prohombres": "prohombre",
    "prohíba": "prohibir",
    "prohíban": "prohibir",
    "prohíbas": "prohibir",
    "prohíbe": "prohibir",
    "prohíben": "prohibir",
    "prohíbes": "prohibir",
    "prohíbo": "prohibir",
    "prohíja": "prohijar",
    "prohíjan": "prohijar",
    "prohíjas": "prohijar",
    "prohíje": "prohijar",
    "prohíjen": "prohijar",
    "prohíjes": "prohijar",
    "prohíjo": "prohijar",
    "proindivisiones": "proindivisión",
    "prolactinas": "prolactina",
    "prolamelares": "prolamelar",
    "prolaminas": "prolamina",
    "prolapsos": "prolapso",
    "prolegómenos": "prolegómeno",
    "proles": "prole",
    "proletaria": "proletario",
    "proletariados": "proletariado",
    "proletarias": "proletario",
    "proletarice": "proletarizar",
    "proletaricemos": "proletarizar",
    "proletaricen": "proletarizar",
    "proletarices": "proletarizar",
    "proletaricé": "proletarizar",
    "proletaricéis": "proletarizar",
    "proletarios": "proletario",
    "proletarismos": "proletarismo",
    "proletariza": "proletarizar",
    "proletarizaba": "proletarizar",
    "proletarizabais": "proletarizar",
    "proletarizaban": "proletarizar",
    "proletarizabas": "proletarizar",
    "proletarizaciones": "proletarización",
    "proletarizad": "proletarizar",
    "proletarizada": "proletarizar",
    "proletarizadas": "proletarizar",
    "proletarizado": "proletarizar",
    "proletarizados": "proletarizar",
    "proletarizamos": "proletarizar",
    "proletarizan": "proletarizar",
    "proletarizando": "proletarizar",
    "proletarizara": "proletarizar",
    "proletarizarais": "proletarizar",
    "proletarizaran": "proletarizar",
    "proletarizaras": "proletarizar",
    "proletarizare": "proletarizar",
    "proletarizareis": "proletarizar",
    "proletarizaremos": "proletarizar",
    "proletarizaren": "proletarizar",
    "proletarizares": "proletarizar",
    "proletarizaron": "proletarizar",
    "proletarizará": "proletarizar",
    "proletarizarán": "proletarizar",
    "proletarizarás": "proletarizar",
    "proletarizaré": "proletarizar",
    "proletarizaréis": "proletarizar",
    "proletarizaría": "proletarizar",
    "proletarizaríais": "proletarizar",
    "proletarizaríamos": "proletarizar",
    "proletarizarían": "proletarizar",
    "proletarizarías": "proletarizar",
    "proletarizas": "proletarizar",
    "proletarizase": "proletarizar",
    "proletarizaseis": "proletarizar",
    "proletarizasen": "proletarizar",
    "proletarizases": "proletarizar",
    "proletarizaste": "proletarizar",
    "proletarizasteis": "proletarizar",
    "proletarizo": "proletarizar",
    "proletarizábamos": "proletarizar",
    "proletarizáis": "proletarizar",
    "proletarizáramos": "proletarizar",
    "proletarizáremos": "proletarizar",
    "proletarizásemos": "proletarizar",
    "proletarizó": "proletarizar",
    "prolifera": "proliferar",
    "proliferaba": "proliferar",
    "proliferabais": "proliferar",
    "proliferaban": "proliferar",
    "proliferabas": "proliferar",
    "proliferaciones": "proliferación",
    "proliferad": "proliferar",
    "proliferada": "proliferar",
    "proliferadas": "proliferar",
    "proliferado": "proliferar",
    "proliferados": "proliferar",
    "proliferamos": "proliferar",
    "proliferan": "proliferar",
    "proliferando": "proliferar",
    "proliferara": "proliferar",
    "proliferarais": "proliferar",
    "proliferaran": "proliferar",
    "proliferaras": "proliferar",
    "proliferare": "proliferar",
    "proliferareis": "proliferar",
    "proliferaremos": "proliferar",
    "proliferaren": "proliferar",
    "proliferares": "proliferar",
    "proliferaron": "proliferar",
    "proliferará": "proliferar",
    "proliferarán": "proliferar",
    "proliferarás": "proliferar",
    "proliferaré": "proliferar",
    "proliferaréis": "proliferar",
    "proliferaría": "proliferar",
    "proliferaríais": "proliferar",
    "proliferaríamos": "proliferar",
    "proliferarían": "proliferar",
    "proliferarías": "proliferar",
    "proliferas": "proliferar",
    "proliferase": "proliferar",
    "proliferaseis": "proliferar",
    "proliferasen": "proliferar",
    "proliferases": "proliferar",
    "proliferaste": "proliferar",
    "proliferasteis": "proliferar",
    "proliferativa": "proliferativo",
    "proliferativas": "proliferativo",
    "proliferativos": "proliferativo",
    "prolifere": "proliferar",
    "proliferemos": "proliferar",
    "proliferen": "proliferar",
    "proliferes": "proliferar",
    "prolifero": "proliferar",
    "proliferábamos": "proliferar",
    "proliferáis": "proliferar",
    "proliferáramos": "proliferar",
    "proliferáremos": "proliferar",
    "proliferásemos": "proliferar",
    "proliferé": "proliferar",
    "proliferéis": "proliferar",
    "proliferó": "proliferar",
    "prolificidades": "prolificidad",
    "prolija": "prolijo",
    "prolijas": "prolijo",
    "prolijidades": "prolijidad",
    "prolijos": "prolijo",
    "prolinas": "prolina",
    "prolinfocitos": "prolinfocito",
    "prologa": "prologar",
    "prologaba": "prologar",
    "prologabais": "prologar",
    "prologaban": "prologar",
    "prologabas": "prologar",
    "prologad": "prologar",
    "prologada": "prologar",
    "prologadas": "prologar",
    "prologado": "prologar",
    "prologados": "prologar",
    "prologales": "prologal",
    "prologamos": "prologar",
    "prologan": "prologar",
    "prologando": "prologar",
    "prologara": "prologar",
    "prologarais": "prologar",
    "prologaran": "prologar",
    "prologaras": "prologar",
    "prologare": "prologar",
    "prologareis": "prologar",
    "prologaremos": "prologar",
    "prologaren": "prologar",
    "prologares": "prologar",
    "prologaron": "prologar",
    "prologará": "prologar",
    "prologarán": "prologar",
    "prologarás": "prologar",
    "prologaré": "prologar",
    "prologaréis": "prologar",
    "prologaría": "prologar",
    "prologaríais": "prologar",
    "prologaríamos": "prologar",
    "prologarían": "prologar",
    "prologarías": "prologar",
    "prologas": "prologar",
    "prologase": "prologar",
    "prologaseis": "prologar",
    "prologasen": "prologar",
    "prologases": "prologar",
    "prologaste": "prologar",
    "prologasteis": "prologar",
    "prologo": "prologar",
    "prologue": "prologar",
    "prologuemos": "prologar",
    "prologuen": "prologar",
    "prologues": "prologar",
    "prologuistas": "prologuista",
    "prologué": "prologar",
    "prologuéis": "prologar",
    "prologábamos": "prologar",
    "prologáis": "prologar",
    "prologáramos": "prologar",
    "prologáremos": "prologar",
    "prologásemos": "prologar",
    "prologó": "prologar",
    "prolonga": "prolongar",
    "prolongaba": "prolongar",
    "prolongabais": "prolongar",
    "prolongaban": "prolongar",
    "prolongabas": "prolongar",
    "prolongables": "prolongable",
    "prolongaciones": "prolongación",
    "prolongad": "prolongar",
    "prolongada": "prolongar",
    "prolongadas": "prolongar",
    "prolongado": "prolongar",
    "prolongadora": "prolongador",
    "prolongadoras": "prolongador",
    "prolongadores": "prolongador",
    "prolongados": "prolongar",
    "prolongamientos": "prolongamiento",
    "prolongamos": "prolongar",
    "prolongan": "prolongar",
    "prolongando": "prolongar",
    "prolongara": "prolongar",
    "prolongarais": "prolongar",
    "prolongaran": "prolongar",
    "prolongaras": "prolongar",
    "prolongare": "prolongar",
    "prolongareis": "prolongar",
    "prolongaremos": "prolongar",
    "prolongaren": "prolongar",
    "prolongares": "prolongar",
    "prolongaron": "prolongar",
    "prolongará": "prolongar",
    "prolongarán": "prolongar",
    "prolongarás": "prolongar",
    "prolongaré": "prolongar",
    "prolongaréis": "prolongar",
    "prolongaría": "prolongar",
    "prolongaríais": "prolongar",
    "prolongaríamos": "prolongar",
    "prolongarían": "prolongar",
    "prolongarías": "prolongar",
    "prolongas": "prolongar",
    "prolongase": "prolongar",
    "prolongaseis": "prolongar",
    "prolongasen": "prolongar",
    "prolongases": "prolongar",
    "prolongaste": "prolongar",
    "prolongasteis": "prolongar",
    "prolongo": "prolongar",
    "prolongue": "prolongar",
    "prolonguemos": "prolongar",
    "prolonguen": "prolongar",
    "prolongues": "prolongar",
    "prolongué": "prolongar",
    "prolonguéis": "prolongar",
    "prolongábamos": "prolongar",
    "prolongáis": "prolongar",
    "prolongáramos": "prolongar",
    "prolongáremos": "prolongar",
    "prolongásemos": "prolongar",
    "prolongó": "prolongar",
    "proloquios": "proloquio",
    "prolífera": "prolífero",
    "prolíferas": "prolífero",
    "prolíferos": "prolífero",
    "prolífica": "prolífico",
    "prolíficas": "prolífico",
    "prolíficos": "prolífico",
    "promecios": "promecio",
    "promedia": "promediar",
    "promediaba": "promediar",
    "promediabais": "promediar",
    "promediaban": "promediar",
    "promediabas": "promediar",
    "promediad": "promediar",
    "promediada": "promediar",
    "promediadas": "promediar",
    "promediado": "promediar",
    "promediados": "promediar",
    "promediamos": "promediar",
    "promedian": "promediar",
    "promediando": "promediar",
    "promediara": "promediar",
    "promediarais": "promediar",
    "promediaran": "promediar",
    "promediaras": "promediar",
    "promediare": "promediar",
    "promediareis": "promediar",
    "promediaremos": "promediar",
    "promediaren": "promediar",
    "promediares": "promediar",
    "promediaron": "promediar",
    "promediará": "promediar",
    "promediarán": "promediar",
    "promediarás": "promediar",
    "promediaré": "promediar",
    "promediaréis": "promediar",
    "promediaría": "promediar",
    "promediaríais": "promediar",
    "promediaríamos": "promediar",
    "promediarían": "promediar",
    "promediarías": "promediar",
    "promedias": "promediar",
    "promediase": "promediar",
    "promediaseis": "promediar",
    "promediasen": "promediar",
    "promediases": "promediar",
    "promediaste": "promediar",
    "promediasteis": "promediar",
    "promedie": "promediar",
    "promediemos": "promediar",
    "promedien": "promediar",
    "promedies": "promediar",
    "promedio": "promediar",
    "promedios": "promedio",
    "promediábamos": "promediar",
    "promediáis": "promediar",
    "promediáramos": "promediar",
    "promediáremos": "promediar",
    "promediásemos": "promediar",
    "promedié": "promediar",
    "promediéis": "promediar",
    "promedió": "promediar",
    "promesas": "promesa",
    "prometa": "prometer",
    "prometafases": "prometafase",
    "prometamos": "prometer",
    "prometan": "prometer",
    "prometas": "prometer",
    "promete": "prometer",
    "prometed": "prometer",
    "prometedora": "prometedor",
    "prometedoras": "prometedor",
    "prometedores": "prometedor",
    "prometemos": "prometer",
    "prometen": "prometer",
    "prometeremos": "prometer",
    "prometerá": "prometer",
    "prometerán": "prometer",
    "prometerás": "prometer",
    "prometeré": "prometer",
    "prometeréis": "prometer",
    "prometería": "prometer",
    "prometeríais": "prometer",
    "prometeríamos": "prometer",
    "prometerían": "prometer",
    "prometerías": "prometer",
    "prometes": "prometer",
    "prometida": "prometido",
    "prometidas": "prometido",
    "prometido": "prometer",
    "prometidos": "prometido",
    "prometiendo": "prometer",
    "prometiera": "prometer",
    "prometierais": "prometer",
    "prometieran": "prometer",
    "prometieras": "prometer",
    "prometiere": "prometer",
    "prometiereis": "prometer",
    "prometieren": "prometer",
    "prometieres": "prometer",
    "prometieron": "prometer",
    "prometiese": "prometer",
    "prometieseis": "prometer",
    "prometiesen": "prometer",
    "prometieses": "prometer",
    "prometimos": "prometer",
    "prometiste": "prometer",
    "prometisteis": "prometer",
    "prometiéramos": "prometer",
    "prometiéremos": "prometer",
    "prometiésemos": "prometer",
    "prometió": "prometer",
    "prometo": "prometer",
    "prometáis": "prometer",
    "prometéis": "prometer",
    "prometí": "prometer",
    "prometía": "prometer",
    "prometíais": "prometer",
    "prometíamos": "prometer",
    "prometían": "prometer",
    "prometías": "prometer",
    "promielocitos": "promielocito",
    "prominencias": "prominencia",
    "prominentes": "prominente",
    "promiscua": "promiscuo",
    "promiscuaba": "promiscuar",
    "promiscuabais": "promiscuar",
    "promiscuaban": "promiscuar",
    "promiscuabas": "promiscuar",
    "promiscuaciones": "promiscuación",
    "promiscuad": "promiscuar",
    "promiscuada": "promiscuar",
    "promiscuadas": "promiscuar",
    "promiscuado": "promiscuar",
    "promiscuados": "promiscuar",
    "promiscuamos": "promiscuar",
    "promiscuando": "promiscuar",
    "promiscuara": "promiscuar",
    "promiscuarais": "promiscuar",
    "promiscuaran": "promiscuar",
    "promiscuaras": "promiscuar",
    "promiscuare": "promiscuar",
    "promiscuareis": "promiscuar",
    "promiscuaremos": "promiscuar",
    "promiscuaren": "promiscuar",
    "promiscuares": "promiscuar",
    "promiscuaron": "promiscuar",
    "promiscuará": "promiscuar",
    "promiscuarán": "promiscuar",
    "promiscuarás": "promiscuar",
    "promiscuaré": "promiscuar",
    "promiscuaréis": "promiscuar",
    "promiscuaría": "promiscuar",
    "promiscuaríais": "promiscuar",
    "promiscuaríamos": "promiscuar",
    "promiscuarían": "promiscuar",
    "promiscuarías": "promiscuar",
    "promiscuas": "promiscuo",
    "promiscuase": "promiscuar",
    "promiscuaseis": "promiscuar",
    "promiscuasen": "promiscuar",
    "promiscuases": "promiscuar",
    "promiscuaste": "promiscuar",
    "promiscuasteis": "promiscuar",
    "promiscuemos": "promiscuar",
    "promiscuidades": "promiscuidad",
    "promiscuos": "promiscuo",
    "promiscuábamos": "promiscuar",
    "promiscuáis": "promiscuar",
    "promiscuáramos": "promiscuar",
    "promiscuáremos": "promiscuar",
    "promiscuásemos": "promiscuar",
    "promiscué": "promiscuar",
    "promiscuéis": "promiscuar",
    "promiscuó": "promiscuar",
    "promiscúa": "promiscuar",
    "promiscúan": "promiscuar",
    "promiscúas": "promiscuar",
    "promiscúe": "promiscuar",
    "promiscúen": "promiscuar",
    "promiscúes": "promiscuar",
    "promiscúo": "promiscuar",
    "promisiones": "promisión",
    "promisoria": "promisorio",
    "promisorias": "promisorio",
    "promisorios": "promisorio",
    "promociona": "promocionar",
    "promocionaba": "promocionar",
    "promocionabais": "promocionar",
    "promocionaban": "promocionar",
    "promocionabas": "promocionar",
    "promocionad": "promocionar",
    "promocionada": "promocionar",
    "promocionadas": "promocionar",
    "promocionado": "promocionar",
    "promocionados": "promocionar",
    "promocionales": "promocional",
    "promocionamos": "promocionar",
    "promocionan": "promocionar",
    "promocionando": "promocionar",
    "promocionara": "promocionar",
    "promocionarais": "promocionar",
    "promocionaran": "promocionar",
    "promocionaras": "promocionar",
    "promocionare": "promocionar",
    "promocionareis": "promocionar",
    "promocionaremos": "promocionar",
    "promocionaren": "promocionar",
    "promocionares": "promocionar",
    "promocionaron": "promocionar",
    "promocionará": "promocionar",
    "promocionarán": "promocionar",
    "promocionarás": "promocionar",
    "promocionaré": "promocionar",
    "promocionaréis": "promocionar",
    "promocionaría": "promocionar",
    "promocionaríais": "promocionar",
    "promocionaríamos": "promocionar",
    "promocionarían": "promocionar",
    "promocionarías": "promocionar",
    "promocionas": "promocionar",
    "promocionase": "promocionar",
    "promocionaseis": "promocionar",
    "promocionasen": "promocionar",
    "promocionases": "promocionar",
    "promocionaste": "promocionar",
    "promocionasteis": "promocionar",
    "promocione": "promocionar",
    "promocionemos": "promocionar",
    "promocionen": "promocionar",
    "promocionera": "promocionero",
    "promocioneras": "promocionero",
    "promocioneros": "promocionero",
    "promociones": "promocionar",
    "promociono": "promocionar",
    "promocionábamos": "promocionar",
    "promocionáis": "promocionar",
    "promocionáramos": "promocionar",
    "promocionáremos": "promocionar",
    "promocionásemos": "promocionar",
    "promocioné": "promocionar",
    "promocionéis": "promocionar",
    "promocionó": "promocionar",
    "promonocitos": "promonocito",
    "promontorios": "promontorio",
    "promotora": "promotor",
    "promotoras": "promotor",
    "promotores": "promotor",
    "promovamos": "promover",
    "promoved": "promover",
    "promovedora": "promovedor",
    "promovedoras": "promovedor",
    "promovedores": "promovedor",
    "promovemos": "promover",
    "promoventes": "promovente",
    "promoveremos": "promover",
    "promoverá": "promover",
    "promoverán": "promover",
    "promoverás": "promover",
    "promoveré": "promover",
    "promoveréis": "promover",
    "promovería": "promover",
    "promoveríais": "promover",
    "promoveríamos": "promover",
    "promoverían": "promover",
    "promoverías": "promover",
    "promovida": "promover",
    "promovidas": "promover",
    "promovido": "promover",
    "promovidos": "promover",
    "promoviendo": "promover",
    "promoviera": "promover",
    "promovierais": "promover",
    "promovieran": "promover",
    "promovieras": "promover",
    "promoviere": "promover",
    "promoviereis": "promover",
    "promovieren": "promover",
    "promovieres": "promover",
    "promovieron": "promover",
    "promoviese": "promover",
    "promovieseis": "promover",
    "promoviesen": "promover",
    "promovieses": "promover",
    "promovimos": "promover",
    "promoviste": "promover",
    "promovisteis": "promover",
    "promoviéramos": "promover",
    "promoviéremos": "promover",
    "promoviésemos": "promover",
    "promovió": "promover",
    "promováis": "promover",
    "promovéis": "promover",
    "promoví": "promover",
    "promovía": "promover",
    "promovíais": "promover",
    "promovíamos": "promover",
    "promovían": "promover",
    "promovías": "promover",
    "promueva": "promover",
    "promuevan": "promover",
    "promuevas": "promover",
    "promueve": "promover",
    "promueven": "promover",
    "promueves": "promover",
    "promuevo": "promover",
    "promulga": "promulgar",
    "promulgaba": "promulgar",
    "promulgabais": "promulgar",
    "promulgaban": "promulgar",
    "promulgabas": "promulgar",
    "promulgaciones": "promulgación",
    "promulgad": "promulgar",
    "promulgada": "promulgar",
    "promulgadas": "promulgar",
    "promulgado": "promulgar",
    "promulgadora": "promulgador",
    "promulgadoras": "promulgador",
    "promulgadores": "promulgador",
    "promulgados": "promulgar",
    "promulgamos": "promulgar",
    "promulgan": "promulgar",
    "promulgando": "promulgar",
    "promulgara": "promulgar",
    "promulgarais": "promulgar",
    "promulgaran": "promulgar",
    "promulgaras": "promulgar",
    "promulgare": "promulgar",
    "promulgareis": "promulgar",
    "promulgaremos": "promulgar",
    "promulgaren": "promulgar",
    "promulgares": "promulgar",
    "promulgaron": "promulgar",
    "promulgará": "promulgar",
    "promulgarán": "promulgar",
    "promulgarás": "promulgar",
    "promulgaré": "promulgar",
    "promulgaréis": "promulgar",
    "promulgaría": "promulgar",
    "promulgaríais": "promulgar",
    "promulgaríamos": "promulgar",
    "promulgarían": "promulgar",
    "promulgarías": "promulgar",
    "promulgas": "promulgar",
    "promulgase": "promulgar",
    "promulgaseis": "promulgar",
    "promulgasen": "promulgar",
    "promulgases": "promulgar",
    "promulgaste": "promulgar",
    "promulgasteis": "promulgar",
    "promulgo": "promulgar",
    "promulgue": "promulgar",
    "promulguemos": "promulgar",
    "promulguen": "promulgar",
    "promulgues": "promulgar",
    "promulgué": "promulgar",
    "promulguéis": "promulgar",
    "promulgábamos": "promulgar",
    "promulgáis": "promulgar",
    "promulgáramos": "promulgar",
    "promulgáremos": "promulgar",
    "promulgásemos": "promulgar",
    "promulgó": "promulgar",
    "prona": "prono",
    "pronaciones": "pronación",
    "pronas": "prono",
    "pronombres": "pronombre",
    "pronominales": "pronominal",
    "pronominalizaciones": "pronominalización",
    "pronos": "prono",
    "pronostica": "pronosticar",
    "pronosticaba": "pronosticar",
    "pronosticabais": "pronosticar",
    "pronosticaban": "pronosticar",
    "pronosticabas": "pronosticar",
    "pronosticaciones": "pronosticación",
    "pronosticad": "pronosticar",
    "pronosticada": "pronosticar",
    "pronosticadas": "pronosticar",
    "pronosticado": "pronosticar",
    "pronosticadora": "pronosticador",
    "pronosticadoras": "pronosticador",
    "pronosticadores": "pronosticador",
    "pronosticados": "pronosticar",
    "pronosticamos": "pronosticar",
    "pronostican": "pronosticar",
    "pronosticando": "pronosticar",
    "pronosticara": "pronosticar",
    "pronosticarais": "pronosticar",
    "pronosticaran": "pronosticar",
    "pronosticaras": "pronosticar",
    "pronosticare": "pronosticar",
    "pronosticareis": "pronosticar",
    "pronosticaremos": "pronosticar",
    "pronosticaren": "pronosticar",
    "pronosticares": "pronosticar",
    "pronosticaron": "pronosticar",
    "pronosticará": "pronosticar",
    "pronosticarán": "pronosticar",
    "pronosticarás": "pronosticar",
    "pronosticaré": "pronosticar",
    "pronosticaréis": "pronosticar",
    "pronosticaría": "pronosticar",
    "pronosticaríais": "pronosticar",
    "pronosticaríamos": "pronosticar",
    "pronosticarían": "pronosticar",
    "pronosticarías": "pronosticar",
    "pronosticas": "pronosticar",
    "pronosticase": "pronosticar",
    "pronosticaseis": "pronosticar",
    "pronosticasen": "pronosticar",
    "pronosticases": "pronosticar",
    "pronosticaste": "pronosticar",
    "pronosticasteis": "pronosticar",
    "pronostico": "pronosticar",
    "pronosticábamos": "pronosticar",
    "pronosticáis": "pronosticar",
    "pronosticáramos": "pronosticar",
    "pronosticáremos": "pronosticar",
    "pronosticásemos": "pronosticar",
    "pronosticó": "pronosticar",
    "pronostique": "pronosticar",
    "pronostiquemos": "pronosticar",
    "pronostiquen": "pronosticar",
    "pronostiques": "pronosticar",
    "pronostiqué": "pronosticar",
    "pronostiquéis": "pronosticar",
    "pronta": "pronto",
    "prontas": "pronto",
    "prontitudes": "prontitud",
    "prontos": "pronto",
    "prontuarios": "prontuario",
    "pronucléolos": "pronucléolo",
    "pronuncia": "pronunciar",
    "pronunciaba": "pronunciar",
    "pronunciabais": "pronunciar",
    "pronunciaban": "pronunciar",
    "pronunciabas": "pronunciar",
    "pronunciables": "pronunciable",
    "pronunciaciones": "pronunciación",
    "pronunciad": "pronunciar",
    "pronunciada": "pronunciar",
    "pronunciadas": "pronunciar",
    "pronunciado": "pronunciar",
    "pronunciadora": "pronunciador",
    "pronunciadoras": "pronunciador",
    "pronunciadores": "pronunciador",
    "pronunciados": "pronunciar",
    "pronunciamientos": "pronunciamiento",
    "pronunciamos": "pronunciar",
    "pronuncian": "pronunciar",
    "pronunciando": "pronunciar",
    "pronunciara": "pronunciar",
    "pronunciarais": "pronunciar",
    "pronunciaran": "pronunciar",
    "pronunciaras": "pronunciar",
    "pronunciare": "pronunciar",
    "pronunciareis": "pronunciar",
    "pronunciaremos": "pronunciar",
    "pronunciaren": "pronunciar",
    "pronunciares": "pronunciar",
    "pronunciaron": "pronunciar",
    "pronunciará": "pronunciar",
    "pronunciarán": "pronunciar",
    "pronunciarás": "pronunciar",
    "pronunciaré": "pronunciar",
    "pronunciaréis": "pronunciar",
    "pronunciaría": "pronunciar",
    "pronunciaríais": "pronunciar",
    "pronunciaríamos": "pronunciar",
    "pronunciarían": "pronunciar",
    "pronunciarías": "pronunciar",
    "pronuncias": "pronunciar",
    "pronunciase": "pronunciar",
    "pronunciaseis": "pronunciar",
    "pronunciasen": "pronunciar",
    "pronunciases": "pronunciar",
    "pronunciaste": "pronunciar",
    "pronunciasteis": "pronunciar",
    "pronuncie": "pronunciar",
    "pronunciemos": "pronunciar",
    "pronuncien": "pronunciar",
    "pronuncies": "pronunciar",
    "pronuncio": "pronunciar",
    "pronuncios": "pronuncio",
    "pronunciábamos": "pronunciar",
    "pronunciáis": "pronunciar",
    "pronunciáramos": "pronunciar",
    "pronunciáremos": "pronunciar",
    "pronunciásemos": "pronunciar",
    "pronuncié": "pronunciar",
    "pronunciéis": "pronunciar",
    "pronunció": "pronunciar",
    "pronósticos": "pronóstico",
    "pronúcleos": "pronúcleo",
    "propaga": "propagar",
    "propagaba": "propagar",
    "propagabais": "propagar",
    "propagaban": "propagar",
    "propagabas": "propagar",
    "propagaciones": "propagación",
    "propagad": "propagar",
    "propagada": "propagar",
    "propagadas": "propagar",
    "propagado": "propagar",
    "propagadora": "propagador",
    "propagadoras": "propagador",
    "propagadores": "propagador",
    "propagados": "propagar",
    "propagamos": "propagar",
    "propagan": "propagar",
    "propagandas": "propaganda",
    "propagandistas": "propagandista",
    "propagando": "propagar",
    "propagandística": "propagandístico",
    "propagandísticas": "propagandístico",
    "propagandísticos": "propagandístico",
    "propagara": "propagar",
    "propagarais": "propagar",
    "propagaran": "propagar",
    "propagaras": "propagar",
    "propagare": "propagar",
    "propagareis": "propagar",
    "propagaremos": "propagar",
    "propagaren": "propagar",
    "propagares": "propagar",
    "propagaron": "propagar",
    "propagará": "propagar",
    "propagarán": "propagar",
    "propagarás": "propagar",
    "propagaré": "propagar",
    "propagaréis": "propagar",
    "propagaría": "propagar",
    "propagaríais": "propagar",
    "propagaríamos": "propagar",
    "propagarían": "propagar",
    "propagarías": "propagar",
    "propagas": "propagar",
    "propagase": "propagar",
    "propagaseis": "propagar",
    "propagasen": "propagar",
    "propagases": "propagar",
    "propagaste": "propagar",
    "propagasteis": "propagar",
    "propagativa": "propagativo",
    "propagativas": "propagativo",
    "propagativos": "propagativo",
    "propago": "propagar",
    "propague": "propagar",
    "propaguemos": "propagar",
    "propaguen": "propagar",
    "propagues": "propagar",
    "propagué": "propagar",
    "propaguéis": "propagar",
    "propagábamos": "propagar",
    "propagáis": "propagar",
    "propagáramos": "propagar",
    "propagáremos": "propagar",
    "propagásemos": "propagar",
    "propagó": "propagar",
    "propala": "propalar",
    "propalaba": "propalar",
    "propalabais": "propalar",
    "propalaban": "propalar",
    "propalabas": "propalar",
    "propalad": "propalar",
    "propalada": "propalar",
    "propaladas": "propalar",
    "propalado": "propalar",
    "propaladora": "propalador",
    "propaladoras": "propalador",
    "propaladores": "propalador",
    "propalados": "propalar",
    "propalamos": "propalar",
    "propalan": "propalar",
    "propalando": "propalar",
    "propalara": "propalar",
    "propalarais": "propalar",
    "propalaran": "propalar",
    "propalaras": "propalar",
    "propalare": "propalar",
    "propalareis": "propalar",
    "propalaremos": "propalar",
    "propalaren": "propalar",
    "propalares": "propalar",
    "propalaron": "propalar",
    "propalará": "propalar",
    "propalarán": "propalar",
    "propalarás": "propalar",
    "propalaré": "propalar",
    "propalaréis": "propalar",
    "propalaría": "propalar",
    "propalaríais": "propalar",
    "propalaríamos": "propalar",
    "propalarían": "propalar",
    "propalarías": "propalar",
    "propalas": "propalar",
    "propalase": "propalar",
    "propalaseis": "propalar",
    "propalasen": "propalar",
    "propalases": "propalar",
    "propalaste": "propalar",
    "propalasteis": "propalar",
    "propale": "propalar",
    "propalemos": "propalar",
    "propalen": "propalar",
    "propales": "propalar",
    "propalo": "propalar",
    "propalábamos": "propalar",
    "propaláis": "propalar",
    "propaláramos": "propalar",
    "propaláremos": "propalar",
    "propalásemos": "propalar",
    "propalé": "propalar",
    "propaléis": "propalar",
    "propaló": "propalar",
    "propanales": "propanal",
    "propanoles": "propanol",
    "propanololes": "propanolol",
    "propanonas": "propanona",
    "propanos": "propano",
    "propaos": "propao",
    "proparoxítona": "proparoxítono",
    "proparoxítonas": "proparoxítono",
    "proparoxítonos": "proparoxítono",
    "propasa": "propasar",
    "propasaba": "propasar",
    "propasabais": "propasar",
    "propasaban": "propasar",
    "propasabas": "propasar",
    "propasad": "propasar",
    "propasada": "propasar",
    "propasadas": "propasar",
    "propasado": "propasar",
    "propasados": "propasar",
    "propasamos": "propasar",
    "propasan": "propasar",
    "propasando": "propasar",
    "propasara": "propasar",
    "propasarais": "propasar",
    "propasaran": "propasar",
    "propasaras": "propasar",
    "propasare": "propasar",
    "propasareis": "propasar",
    "propasaremos": "propasar",
    "propasaren": "propasar",
    "propasares": "propasar",
    "propasaron": "propasar",
    "propasará": "propasar",
    "propasarán": "propasar",
    "propasarás": "propasar",
    "propasaré": "propasar",
    "propasaréis": "propasar",
    "propasaría": "propasar",
    "propasaríais": "propasar",
    "propasaríamos": "propasar",
    "propasarían": "propasar",
    "propasarías": "propasar",
    "propasas": "propasar",
    "propasase": "propasar",
    "propasaseis": "propasar",
    "propasasen": "propasar",
    "propasases": "propasar",
    "propasaste": "propasar",
    "propasasteis": "propasar",
    "propase": "propasar",
    "propasemos": "propasar",
    "propasen": "propasar",
    "propases": "propasar",
    "propaso": "propasar",
    "propasábamos": "propasar",
    "propasáis": "propasar",
    "propasáramos": "propasar",
    "propasáremos": "propasar",
    "propasásemos": "propasar",
    "propasé": "propasar",
    "propaséis": "propasar",
    "propasó": "propasar",
    "propedéutica": "propedéutico",
    "propedéuticas": "propedéutico",
    "propedéuticos": "propedéutico",
    "propenales": "propenal",
    "propenda": "propender",
    "propendamos": "propender",
    "propendan": "propender",
    "propendas": "propender",
    "propende": "propender",
    "propended": "propender",
    "propendemos": "propender",
    "propenden": "propender",
    "propenderemos": "propender",
    "propenderá": "propender",
    "propenderán": "propender",
    "propenderás": "propender",
    "propenderé": "propender",
    "propenderéis": "propender",
    "propendería": "propender",
    "propenderíais": "propender",
    "propenderíamos": "propender",
    "propenderían": "propender",
    "propenderías": "propender",
    "propendes": "propender",
    "propendida": "propender",
    "propendidas": "propender",
    "propendido": "propender",
    "propendidos": "propender",
    "propendiendo": "propender",
    "propendiera": "propender",
    "propendierais": "propender",
    "propendieran": "propender",
    "propendieras": "propender",
    "propendiere": "propender",
    "propendiereis": "propender",
    "propendieren": "propender",
    "propendieres": "propender",
    "propendieron": "propender",
    "propendiese": "propender",
    "propendieseis": "propender",
    "propendiesen": "propender",
    "propendieses": "propender",
    "propendimos": "propender",
    "propendiste": "propender",
    "propendisteis": "propender",
    "propendiéramos": "propender",
    "propendiéremos": "propender",
    "propendiésemos": "propender",
    "propendió": "propender",
    "propendo": "propender",
    "propendáis": "propender",
    "propendéis": "propender",
    "propendí": "propender",
    "propendía": "propender",
    "propendíais": "propender",
    "propendíamos": "propender",
    "propendían": "propender",
    "propendías": "propender",
    "propenoles": "propenol",
    "propenos": "propeno",
    "propensa": "propenso",
    "propensas": "propenso",
    "propensidades": "propensidad",
    "propensiones": "propensión",
    "propensos": "propenso",
    "properdinas": "properdina",
    "propergoles": "propergol",
    "propia": "propio",
    "propias": "propio",
    "propicia": "propicio",
    "propiciaba": "propiciar",
    "propiciabais": "propiciar",
    "propiciaban": "propiciar",
    "propiciabas": "propiciar",
    "propiciaciones": "propiciación",
    "propiciad": "propiciar",
    "propiciada": "propiciar",
    "propiciadas": "propiciar",
    "propiciado": "propiciar",
    "propiciadora": "propiciador",
    "propiciadoras": "propiciador",
    "propiciadores": "propiciador",
    "propiciados": "propiciar",
    "propiciamos": "propiciar",
    "propician": "propiciar",
    "propiciando": "propiciar",
    "propiciara": "propiciar",
    "propiciarais": "propiciar",
    "propiciaran": "propiciar",
    "propiciaras": "propiciar",
    "propiciare": "propiciar",
    "propiciareis": "propiciar",
    "propiciaremos": "propiciar",
    "propiciaren": "propiciar",
    "propiciares": "propiciar",
    "propiciaron": "propiciar",
    "propiciará": "propiciar",
    "propiciarán": "propiciar",
    "propiciarás": "propiciar",
    "propiciaré": "propiciar",
    "propiciaréis": "propiciar",
    "propiciaría": "propiciar",
    "propiciaríais": "propiciar",
    "propiciaríamos": "propiciar",
    "propiciarían": "propiciar",
    "propiciarías": "propiciar",
    "propicias": "propicio",
    "propiciase": "propiciar",
    "propiciaseis": "propiciar",
    "propiciasen": "propiciar",
    "propiciases": "propiciar",
    "propiciaste": "propiciar",
    "propiciasteis": "propiciar",
    "propiciatoria": "propiciatorio",
    "propiciatorias": "propiciatorio",
    "propiciatorios": "propiciatorio",
    "propicie": "propiciar",
    "propiciemos": "propiciar",
    "propicien": "propiciar",
    "propicies": "propiciar",
    "propicio": "propiciar",
    "propicios": "propicio",
    "propiciábamos": "propiciar",
    "propiciáis": "propiciar",
    "propiciáramos": "propiciar",
    "propiciáremos": "propiciar",
    "propiciásemos": "propiciar",
    "propicié": "propiciar",
    "propiciéis": "propiciar",
    "propició": "propiciar",
    "propiedades": "propiedad",
    "propiendas": "propienda",
    "propietaria": "propietario",
    "propietarias": "propietario",
    "propietarios": "propietario",
    "propilacroleínas": "propilacroleína",
    "propilaminas": "propilamina",
    "propilendiaminas": "propilendiamina",
    "propilenglicoles": "propilenglicol",
    "propilenos": "propileno",
    "propileos": "propileo",
    "propilhexedrinas": "propilhexedrina",
    "propiltiouracilos": "propiltiouracilo",
    "propina": "propinar",
    "propinaba": "propinar",
    "propinabais": "propinar",
    "propinaban": "propinar",
    "propinabas": "propinar",
    "propinad": "propinar",
    "propinada": "propinar",
    "propinadas": "propinar",
    "propinado": "propinar",
    "propinados": "propinar",
    "propinamos": "propinar",
    "propinan": "propinar",
    "propinando": "propinar",
    "propinara": "propinar",
    "propinarais": "propinar",
    "propinaran": "propinar",
    "propinaras": "propinar",
    "propinare": "propinar",
    "propinareis": "propinar",
    "propinaremos": "propinar",
    "propinaren": "propinar",
    "propinares": "propinar",
    "propinaron": "propinar",
    "propinará": "propinar",
    "propinarán": "propinar",
    "propinarás": "propinar",
    "propinaré": "propinar",
    "propinaréis": "propinar",
    "propinaría": "propinar",
    "propinaríais": "propinar",
    "propinaríamos": "propinar",
    "propinarían": "propinar",
    "propinarías": "propinar",
    "propinas": "propinar",
    "propinase": "propinar",
    "propinaseis": "propinar",
    "propinasen": "propinar",
    "propinases": "propinar",
    "propinaste": "propinar",
    "propinasteis": "propinar",
    "propincua": "propincuo",
    "propincuas": "propincuo",
    "propincuidades": "propincuidad",
    "propincuos": "propincuo",
    "propine": "propinar",
    "propinemos": "propinar",
    "propinen": "propinar",
    "propines": "propinar",
    "propino": "propinar",
    "propinábamos": "propinar",
    "propináis": "propinar",
    "propináramos": "propinar",
    "propináremos": "propinar",
    "propinásemos": "propinar",
    "propiné": "propinar",
    "propinéis": "propinar",
    "propinó": "propinar",
    "propioceptiva": "propioceptivo",
    "propioceptivas": "propioceptivo",
    "propioceptivos": "propioceptivo",
    "propiofenonas": "propiofenona",
    "propiolactonas": "propiolactona",
    "propionaldehídos": "propionaldehído",
    "propionatos": "propionato",
    "propios": "propio",
    "propiónica": "propiónico",
    "propiónicas": "propiónico",
    "propiónicos": "propiónico",
    "propondremos": "proponer",
    "propondrá": "proponer",
    "propondrán": "proponer",
    "propondrás": "proponer",
    "propondré": "proponer",
    "propondréis": "proponer",
    "propondría": "proponer",
    "propondríais": "proponer",
    "propondríamos": "proponer",
    "propondrían": "proponer",
    "propondrías": "proponer",
    "propone": "proponer",
    "proponed": "proponer",
    "proponedora": "proponedor",
    "proponedoras": "proponedor",
    "proponedores": "proponedor",
    "proponemos": "proponer",
    "proponen": "proponer",
    "proponentes": "proponente",
    "propones": "proponer",
    "proponga": "proponer",
    "propongamos": "proponer",
    "propongan": "proponer",
    "propongas": "proponer",
    "propongo": "proponer",
    "propongáis": "proponer",
    "proponiendo": "proponer",
    "proponéis": "proponer",
    "proponía": "proponer",
    "proponíais": "proponer",
    "proponíamos": "proponer",
    "proponían": "proponer",
    "proponías": "proponer",
    "proporciona": "proporcionar",
    "proporcionaba": "proporcionar",
    "proporcionabais": "proporcionar",
    "proporcionaban": "proporcionar",
    "proporcionabas": "proporcionar",
    "proporcionables": "proporcionable",
    "proporcionad": "proporcionar",
    "proporcionada": "proporcionar",
    "proporcionadas": "proporcionar",
    "proporcionado": "proporcionar",
    "proporcionados": "proporcionar",
    "proporcionales": "proporcional",
    "proporcionalidades": "proporcionalidad",
    "proporcionamos": "proporcionar",
    "proporcionan": "proporcionar",
    "proporcionando": "proporcionar",
    "proporcionara": "proporcionar",
    "proporcionarais": "proporcionar",
    "proporcionaran": "proporcionar",
    "proporcionaras": "proporcionar",
    "proporcionare": "proporcionar",
    "proporcionareis": "proporcionar",
    "proporcionaremos": "proporcionar",
    "proporcionaren": "proporcionar",
    "proporcionares": "proporcionar",
    "proporcionaron": "proporcionar",
    "proporcionará": "proporcionar",
    "proporcionarán": "proporcionar",
    "proporcionarás": "proporcionar",
    "proporcionaré": "proporcionar",
    "proporcionaréis": "proporcionar",
    "proporcionaría": "proporcionar",
    "proporcionaríais": "proporcionar",
    "proporcionaríamos": "proporcionar",
    "proporcionarían": "proporcionar",
    "proporcionarías": "proporcionar",
    "proporcionas": "proporcionar",
    "proporcionase": "proporcionar",
    "proporcionaseis": "proporcionar",
    "proporcionasen": "proporcionar",
    "proporcionases": "proporcionar",
    "proporcionaste": "proporcionar",
    "proporcionasteis": "proporcionar",
    "proporcione": "proporcionar",
    "proporcionemos": "proporcionar",
    "proporcionen": "proporcionar",
    "proporciones": "proporcionar",
    "proporciono": "proporcionar",
    "proporcionábamos": "proporcionar",
    "proporcionáis": "proporcionar",
    "proporcionáramos": "proporcionar",
    "proporcionáremos": "proporcionar",
    "proporcionásemos": "proporcionar",
    "proporcioné": "proporcionar",
    "proporcionéis": "proporcionar",
    "proporcionó": "proporcionar",
    "proposicionales": "proposicional",
    "proposiciones": "proposición",
    "propretores": "propretor",
    "propreturas": "propretura",
    "propuesta": "proponer",
    "propuestas": "propuesta",
    "propuesto": "proponer",
    "propuestos": "proponer",
    "propugna": "propugnar",
    "propugnaba": "propugnar",
    "propugnabais": "propugnar",
    "propugnaban": "propugnar",
    "propugnabas": "propugnar",
    "propugnaciones": "propugnación",
    "propugnad": "propugnar",
    "propugnada": "propugnar",
    "propugnadas": "propugnar",
    "propugnado": "propugnar",
    "propugnados": "propugnar",
    "propugnamos": "propugnar",
    "propugnan": "propugnar",
    "propugnando": "propugnar",
    "propugnara": "propugnar",
    "propugnarais": "propugnar",
    "propugnaran": "propugnar",
    "propugnaras": "propugnar",
    "propugnare": "propugnar",
    "propugnareis": "propugnar",
    "propugnaremos": "propugnar",
    "propugnaren": "propugnar",
    "propugnares": "propugnar",
    "propugnaron": "propugnar",
    "propugnará": "propugnar",
    "propugnarán": "propugnar",
    "propugnarás": "propugnar",
    "propugnaré": "propugnar",
    "propugnaréis": "propugnar",
    "propugnaría": "propugnar",
    "propugnaríais": "propugnar",
    "propugnaríamos": "propugnar",
    "propugnarían": "propugnar",
    "propugnarías": "propugnar",
    "propugnas": "propugnar",
    "propugnase": "propugnar",
    "propugnaseis": "propugnar",
    "propugnasen": "propugnar",
    "propugnases": "propugnar",
    "propugnaste": "propugnar",
    "propugnasteis": "propugnar",
    "propugne": "propugnar",
    "propugnemos": "propugnar",
    "propugnen": "propugnar",
    "propugnes": "propugnar",
    "propugno": "propugnar",
    "propugnábamos": "propugnar",
    "propugnáis": "propugnar",
    "propugnáramos": "propugnar",
    "propugnáremos": "propugnar",
    "propugnásemos": "propugnar",
    "propugné": "propugnar",
    "propugnéis": "propugnar",
    "propugnó": "propugnar",
    "propulsa": "propulsar",
    "propulsaba": "propulsar",
    "propulsabais": "propulsar",
    "propulsaban": "propulsar",
    "propulsabas": "propulsar",
    "propulsad": "propulsar",
    "propulsada": "propulsar",
    "propulsadas": "propulsar",
    "propulsado": "propulsar",
    "propulsados": "propulsar",
    "propulsamos": "propulsar",
    "propulsan": "propulsar",
    "propulsando": "propulsar",
    "propulsantes": "propulsante",
    "propulsara": "propulsar",
    "propulsarais": "propulsar",
    "propulsaran": "propulsar",
    "propulsaras": "propulsar",
    "propulsare": "propulsar",
    "propulsareis": "propulsar",
    "propulsaremos": "propulsar",
    "propulsaren": "propulsar",
    "propulsares": "propulsar",
    "propulsaron": "propulsar",
    "propulsará": "propulsar",
    "propulsarán": "propulsar",
    "propulsarás": "propulsar",
    "propulsaré": "propulsar",
    "propulsaréis": "propulsar",
    "propulsaría": "propulsar",
    "propulsaríais": "propulsar",
    "propulsaríamos": "propulsar",
    "propulsarían": "propulsar",
    "propulsarías": "propulsar",
    "propulsas": "propulsar",
    "propulsase": "propulsar",
    "propulsaseis": "propulsar",
    "propulsasen": "propulsar",
    "propulsases": "propulsar",
    "propulsaste": "propulsar",
    "propulsasteis": "propulsar",
    "propulse": "propulsar",
    "propulsemos": "propulsar",
    "propulsen": "propulsar",
    "propulses": "propulsar",
    "propulsiones": "propulsión",
    "propulsiva": "propulsivo",
    "propulsivas": "propulsivo",
    "propulsivos": "propulsivo",
    "propulso": "propulsar",
    "propulsora": "propulsor",
    "propulsoras": "propulsor",
    "propulsores": "propulsor",
    "propulsábamos": "propulsar",
    "propulsáis": "propulsar",
    "propulsáramos": "propulsar",
    "propulsáremos": "propulsar",
    "propulsásemos": "propulsar",
    "propulsé": "propulsar",
    "propulséis": "propulsar",
    "propulsó": "propulsar",
    "propuse": "proponer",
    "propusiera": "proponer",
    "propusierais": "proponer",
    "propusieran": "proponer",
    "propusieras": "proponer",
    "propusiere": "proponer",
    "propusiereis": "proponer",
    "propusieren": "proponer",
    "propusieres": "proponer",
    "propusieron": "proponer",
    "propusiese": "proponer",
    "propusieseis": "proponer",
    "propusiesen": "proponer",
    "propusieses": "proponer",
    "propusimos": "proponer",
    "propusiste": "proponer",
    "propusisteis": "proponer",
    "propusiéramos": "proponer",
    "propusiéremos": "proponer",
    "propusiésemos": "proponer",
    "propuso": "proponer",
    "propágulos": "propágulo",
    "propón": "proponer",
    "propósitos": "propósito",
    "prorratas": "prorrata",
    "prorratea": "prorratear",
    "prorrateaba": "prorratear",
    "prorrateabais": "prorratear",
    "prorrateaban": "prorratear",
    "prorrateabas": "prorratear",
    "prorratead": "prorratear",
    "prorrateada": "prorratear",
    "prorrateadas": "prorratear",
    "prorrateado": "prorratear",
    "prorrateados": "prorratear",
    "prorrateamos": "prorratear",
    "prorratean": "prorratear",
    "prorrateando": "prorratear",
    "prorrateara": "prorratear",
    "prorratearais": "prorratear",
    "prorratearan": "prorratear",
    "prorratearas": "prorratear",
    "prorrateare": "prorratear",
    "prorrateareis": "prorratear",
    "prorratearemos": "prorratear",
    "prorratearen": "prorratear",
    "prorrateares": "prorratear",
    "prorratearon": "prorratear",
    "prorrateará": "prorratear",
    "prorratearán": "prorratear",
    "prorratearás": "prorratear",
    "prorratearé": "prorratear",
    "prorratearéis": "prorratear",
    "prorratearía": "prorratear",
    "prorratearíais": "prorratear",
    "prorratearíamos": "prorratear",
    "prorratearían": "prorratear",
    "prorratearías": "prorratear",
    "prorrateas": "prorratear",
    "prorratease": "prorratear",
    "prorrateaseis": "prorratear",
    "prorrateasen": "prorratear",
    "prorrateases": "prorratear",
    "prorrateaste": "prorratear",
    "prorrateasteis": "prorratear",
    "prorratee": "prorratear",
    "prorrateemos": "prorratear",
    "prorrateen": "prorratear",
    "prorratees": "prorratear",
    "prorrateo": "prorratear",
    "prorrateos": "prorrateo",
    "prorrateábamos": "prorratear",
    "prorrateáis": "prorratear",
    "prorrateáramos": "prorratear",
    "prorrateáremos": "prorratear",
    "prorrateásemos": "prorratear",
    "prorrateé": "prorratear",
    "prorrateéis": "prorratear",
    "prorrateó": "prorratear",
    "prorroga": "prorrogar",
    "prorrogaba": "prorrogar",
    "prorrogabais": "prorrogar",
    "prorrogaban": "prorrogar",
    "prorrogabas": "prorrogar",
    "prorrogables": "prorrogable",
    "prorrogaciones": "prorrogación",
    "prorrogad": "prorrogar",
    "prorrogada": "prorrogar",
    "prorrogadas": "prorrogar",
    "prorrogado": "prorrogar",
    "prorrogados": "prorrogar",
    "prorrogamos": "prorrogar",
    "prorrogan": "prorrogar",
    "prorrogando": "prorrogar",
    "prorrogara": "prorrogar",
    "prorrogarais": "prorrogar",
    "prorrogaran": "prorrogar",
    "prorrogaras": "prorrogar",
    "prorrogare": "prorrogar",
    "prorrogareis": "prorrogar",
    "prorrogaremos": "prorrogar",
    "prorrogaren": "prorrogar",
    "prorrogares": "prorrogar",
    "prorrogaron": "prorrogar",
    "prorrogará": "prorrogar",
    "prorrogarán": "prorrogar",
    "prorrogarás": "prorrogar",
    "prorrogaré": "prorrogar",
    "prorrogaréis": "prorrogar",
    "prorrogaría": "prorrogar",
    "prorrogaríais": "prorrogar",
    "prorrogaríamos": "prorrogar",
    "prorrogarían": "prorrogar",
    "prorrogarías": "prorrogar",
    "prorrogas": "prorrogar",
    "prorrogase": "prorrogar",
    "prorrogaseis": "prorrogar",
    "prorrogasen": "prorrogar",
    "prorrogases": "prorrogar",
    "prorrogaste": "prorrogar",
    "prorrogasteis": "prorrogar",
    "prorrogativa": "prorrogativo",
    "prorrogativas": "prorrogativo",
    "prorrogativos": "prorrogativo",
    "prorrogo": "prorrogar",
    "prorrogue": "prorrogar",
    "prorroguemos": "prorrogar",
    "prorroguen": "prorrogar",
    "prorrogues": "prorrogar",
    "prorrogué": "prorrogar",
    "prorroguéis": "prorrogar",
    "prorrogábamos": "prorrogar",
    "prorrogáis": "prorrogar",
    "prorrogáramos": "prorrogar",
    "prorrogáremos": "prorrogar",
    "prorrogásemos": "prorrogar",
    "prorrogó": "prorrogar",
    "prorrumpa": "prorrumpir",
    "prorrumpamos": "prorrumpir",
    "prorrumpan": "prorrumpir",
    "prorrumpas": "prorrumpir",
    "prorrumpe": "prorrumpir",
    "prorrumpen": "prorrumpir",
    "prorrumpes": "prorrumpir",
    "prorrumpid": "prorrumpir",
    "prorrumpida": "prorrumpir",
    "prorrumpidas": "prorrumpir",
    "prorrumpido": "prorrumpir",
    "prorrumpidos": "prorrumpir",
    "prorrumpiendo": "prorrumpir",
    "prorrumpiera": "prorrumpir",
    "prorrumpierais": "prorrumpir",
    "prorrumpieran": "prorrumpir",
    "prorrumpieras": "prorrumpir",
    "prorrumpiere": "prorrumpir",
    "prorrumpiereis": "prorrumpir",
    "prorrumpieren": "prorrumpir",
    "prorrumpieres": "prorrumpir",
    "prorrumpieron": "prorrumpir",
    "prorrumpiese": "prorrumpir",
    "prorrumpieseis": "prorrumpir",
    "prorrumpiesen": "prorrumpir",
    "prorrumpieses": "prorrumpir",
    "prorrumpimos": "prorrumpir",
    "prorrumpiremos": "prorrumpir",
    "prorrumpirá": "prorrumpir",
    "prorrumpirán": "prorrumpir",
    "prorrumpirás": "prorrumpir",
    "prorrumpiré": "prorrumpir",
    "prorrumpiréis": "prorrumpir",
    "prorrumpiría": "prorrumpir",
    "prorrumpiríais": "prorrumpir",
    "prorrumpiríamos": "prorrumpir",
    "prorrumpirían": "prorrumpir",
    "prorrumpirías": "prorrumpir",
    "prorrumpiste": "prorrumpir",
    "prorrumpisteis": "prorrumpir",
    "prorrumpiéramos": "prorrumpir",
    "prorrumpiéremos": "prorrumpir",
    "prorrumpiésemos": "prorrumpir",
    "prorrumpió": "prorrumpir",
    "prorrumpo": "prorrumpir",
    "prorrumpáis": "prorrumpir",
    "prorrumpí": "prorrumpir",
    "prorrumpía": "prorrumpir",
    "prorrumpíais": "prorrumpir",
    "prorrumpíamos": "prorrumpir",
    "prorrumpían": "prorrumpir",
    "prorrumpías": "prorrumpir",
    "prorrumpís": "prorrumpir",
    "pros": "pro",
    "prosada": "prosado",
    "prosadas": "prosado",
    "prosados": "prosado",
    "prosaica": "prosaico",
    "prosaicas": "prosaico",
    "prosaicos": "prosaico",
    "prosapias": "prosapia",
    "prosas": "prosa",
    "prosaísmos": "prosaísmo",
    "proscenios": "proscenio",
    "proscriba": "proscribir",
    "proscribamos": "proscribir",
    "proscriban": "proscribir",
    "proscribas": "proscribir",
    "proscribe": "proscribir",
    "proscriben": "proscribir",
    "proscribes": "proscribir",
    "proscribid": "proscribir",
    "proscribiendo": "proscribir",
    "proscribiera": "proscribir",
    "proscribierais": "proscribir",
    "proscribieran": "proscribir",
    "proscribieras": "proscribir",
    "proscribiere": "proscribir",
    "proscribiereis": "proscribir",
    "proscribieren": "proscribir",
    "proscribieres": "proscribir",
    "proscribieron": "proscribir",
    "proscribiese": "proscribir",
    "proscribieseis": "proscribir",
    "proscribiesen": "proscribir",
    "proscribieses": "proscribir",
    "proscribimos": "proscribir",
    "proscribiremos": "proscribir",
    "proscribirá": "proscribir",
    "proscribirán": "proscribir",
    "proscribirás": "proscribir",
    "proscribiré": "proscribir",
    "proscribiréis": "proscribir",
    "proscribiría": "proscribir",
    "proscribiríais": "proscribir",
    "proscribiríamos": "proscribir",
    "proscribirían": "proscribir",
    "proscribirías": "proscribir",
    "proscribiste": "proscribir",
    "proscribisteis": "proscribir",
    "proscribiéramos": "proscribir",
    "proscribiéremos": "proscribir",
    "proscribiésemos": "proscribir",
    "proscribió": "proscribir",
    "proscribo": "proscribir",
    "proscribáis": "proscribir",
    "proscribí": "proscribir",
    "proscribía": "proscribir",
    "proscribíais": "proscribir",
    "proscribíamos": "proscribir",
    "proscribían": "proscribir",
    "proscribías": "proscribir",
    "proscribís": "proscribir",
    "proscripciones": "proscripción",
    "proscriptora": "proscriptor",
    "proscriptoras": "proscriptor",
    "proscriptores": "proscriptor",
    "proscrita": "proscribir",
    "proscritas": "proscribir",
    "proscrito": "proscribir",
    "proscritos": "proscribir",
    "proscénica": "proscénico",
    "proscénicas": "proscénico",
    "proscénicos": "proscénico",
    "prosecuciones": "prosecución",
    "proseguibles": "proseguible",
    "proseguid": "proseguir",
    "proseguida": "proseguir",
    "proseguidas": "proseguir",
    "proseguido": "proseguir",
    "proseguidos": "proseguir",
    "proseguimientos": "proseguimiento",
    "proseguimos": "proseguir",
    "proseguiremos": "proseguir",
    "proseguirá": "proseguir",
    "proseguirán": "proseguir",
    "proseguirás": "proseguir",
    "proseguiré": "proseguir",
    "proseguiréis": "proseguir",
    "proseguiría": "proseguir",
    "proseguiríais": "proseguir",
    "proseguiríamos": "proseguir",
    "proseguirían": "proseguir",
    "proseguirías": "proseguir",
    "proseguiste": "proseguir",
    "proseguisteis": "proseguir",
    "proseguí": "proseguir",
    "proseguía": "proseguir",
    "proseguíais": "proseguir",
    "proseguíamos": "proseguir",
    "proseguían": "proseguir",
    "proseguías": "proseguir",
    "proseguís": "proseguir",
    "proselitismos": "proselitismo",
    "proselitistas": "proselitista",
    "prosencéfalos": "prosencéfalo",
    "prosifica": "prosificar",
    "prosificaba": "prosificar",
    "prosificabais": "prosificar",
    "prosificaban": "prosificar",
    "prosificabas": "prosificar",
    "prosificaciones": "prosificación",
    "prosificad": "prosificar",
    "prosificada": "prosificar",
    "prosificadas": "prosificar",
    "prosificado": "prosificar",
    "prosificadora": "prosificador",
    "prosificadoras": "prosificador",
    "prosificadores": "prosificador",
    "prosificados": "prosificar",
    "prosificamos": "prosificar",
    "prosifican": "prosificar",
    "prosificando": "prosificar",
    "prosificara": "prosificar",
    "prosificarais": "prosificar",
    "prosificaran": "prosificar",
    "prosificaras": "prosificar",
    "prosificare": "prosificar",
    "prosificareis": "prosificar",
    "prosificaremos": "prosificar",
    "prosificaren": "prosificar",
    "prosificares": "prosificar",
    "prosificaron": "prosificar",
    "prosificará": "prosificar",
    "prosificarán": "prosificar",
    "prosificarás": "prosificar",
    "prosificaré": "prosificar",
    "prosificaréis": "prosificar",
    "prosificaría": "prosificar",
    "prosificaríais": "prosificar",
    "prosificaríamos": "prosificar",
    "prosificarían": "prosificar",
    "prosificarías": "prosificar",
    "prosificas": "prosificar",
    "prosificase": "prosificar",
    "prosificaseis": "prosificar",
    "prosificasen": "prosificar",
    "prosificases": "prosificar",
    "prosificaste": "prosificar",
    "prosificasteis": "prosificar",
    "prosifico": "prosificar",
    "prosificábamos": "prosificar",
    "prosificáis": "prosificar",
    "prosificáramos": "prosificar",
    "prosificáremos": "prosificar",
    "prosificásemos": "prosificar",
    "prosificó": "prosificar",
    "prosifique": "prosificar",
    "prosifiquemos": "prosificar",
    "prosifiquen": "prosificar",
    "prosifiques": "prosificar",
    "prosifiqué": "prosificar",
    "prosifiquéis": "prosificar",
    "prosiga": "proseguir",
    "prosigamos": "proseguir",
    "prosigan": "proseguir",
    "prosigas": "proseguir",
    "prosigo": "proseguir",
    "prosigue": "proseguir",
    "prosiguen": "proseguir",
    "prosigues": "proseguir",
    "prosiguiendo": "proseguir",
    "prosiguiera": "proseguir",
    "prosiguierais": "proseguir",
    "prosiguieran": "proseguir",
    "prosiguieras": "proseguir",
    "prosiguiere": "proseguir",
    "prosiguiereis": "proseguir",
    "prosiguieren": "proseguir",
    "prosiguieres": "proseguir",
    "prosiguieron": "proseguir",
    "prosiguiese": "proseguir",
    "prosiguieseis": "proseguir",
    "prosiguiesen": "proseguir",
    "prosiguieses": "proseguir",
    "prosiguiéramos": "proseguir",
    "prosiguiéremos": "proseguir",
    "prosiguiésemos": "proseguir",
    "prosiguió": "proseguir",
    "prosigáis": "proseguir",
    "prosistas": "prosista",
    "prosobranquios": "prosobranquio",
    "prosodias": "prosodia",
    "prosopalgias": "prosopalgia",
    "prosopitas": "prosopita",
    "prosopografías": "prosopografía",
    "prosopopeyas": "prosopopeya",
    "prosopómetros": "prosopómetro",
    "prospecciones": "prospección",
    "prospecta": "prospectar",
    "prospectaba": "prospectar",
    "prospectabais": "prospectar",
    "prospectaban": "prospectar",
    "prospectabas": "prospectar",
    "prospectad": "prospectar",
    "prospectada": "prospectar",
    "prospectadas": "prospectar",
    "prospectado": "prospectar",
    "prospectados": "prospectar",
    "prospectamos": "prospectar",
    "prospectan": "prospectar",
    "prospectando": "prospectar",
    "prospectara": "prospectar",
    "prospectarais": "prospectar",
    "prospectaran": "prospectar",
    "prospectaras": "prospectar",
    "prospectare": "prospectar",
    "prospectareis": "prospectar",
    "prospectaremos": "prospectar",
    "prospectaren": "prospectar",
    "prospectares": "prospectar",
    "prospectaron": "prospectar",
    "prospectará": "prospectar",
    "prospectarán": "prospectar",
    "prospectarás": "prospectar",
    "prospectaré": "prospectar",
    "prospectaréis": "prospectar",
    "prospectaría": "prospectar",
    "prospectaríais": "prospectar",
    "prospectaríamos": "prospectar",
    "prospectarían": "prospectar",
    "prospectarías": "prospectar",
    "prospectas": "prospectar",
    "prospectase": "prospectar",
    "prospectaseis": "prospectar",
    "prospectasen": "prospectar",
    "prospectases": "prospectar",
    "prospectaste": "prospectar",
    "prospectasteis": "prospectar",
    "prospecte": "prospectar",
    "prospectemos": "prospectar",
    "prospecten": "prospectar",
    "prospectes": "prospectar",
    "prospectiva": "prospectivo",
    "prospectivas": "prospectivo",
    "prospectivos": "prospectivo",
    "prospecto": "prospectar",
    "prospectora": "prospector",
    "prospectoras": "prospector",
    "prospectores": "prospector",
    "prospectos": "prospecto",
    "prospectábamos": "prospectar",
    "prospectáis": "prospectar",
    "prospectáramos": "prospectar",
    "prospectáremos": "prospectar",
    "prospectásemos": "prospectar",
    "prospecté": "prospectar",
    "prospectéis": "prospectar",
    "prospectó": "prospectar",
    "prospera": "prosperar",
    "prosperaba": "prosperar",
    "prosperabais": "prosperar",
    "prosperaban": "prosperar",
    "prosperabas": "prosperar",
    "prosperad": "prosperar",
    "prosperada": "prosperar",
    "prosperadas": "prosperar",
    "prosperado": "prosperar",
    "prosperados": "prosperar",
    "prosperamos": "prosperar",
    "prosperan": "prosperar",
    "prosperando": "prosperar",
    "prosperara": "prosperar",
    "prosperarais": "prosperar",
    "prosperaran": "prosperar",
    "prosperaras": "prosperar",
    "prosperare": "prosperar",
    "prosperareis": "prosperar",
    "prosperaremos": "prosperar",
    "prosperaren": "prosperar",
    "prosperares": "prosperar",
    "prosperaron": "prosperar",
    "prosperará": "prosperar",
    "prosperarán": "prosperar",
    "prosperarás": "prosperar",
    "prosperaré": "prosperar",
    "prosperaréis": "prosperar",
    "prosperaría": "prosperar",
    "prosperaríais": "prosperar",
    "prosperaríamos": "prosperar",
    "prosperarían": "prosperar",
    "prosperarías": "prosperar",
    "prosperas": "prosperar",
    "prosperase": "prosperar",
    "prosperaseis": "prosperar",
    "prosperasen": "prosperar",
    "prosperases": "prosperar",
    "prosperaste": "prosperar",
    "prosperasteis": "prosperar",
    "prospere": "prosperar",
    "prosperemos": "prosperar",
    "prosperen": "prosperar",
    "prosperes": "prosperar",
    "prosperidades": "prosperidad",
    "prospero": "prosperar",
    "prosperábamos": "prosperar",
    "prosperáis": "prosperar",
    "prosperáramos": "prosperar",
    "prosperáremos": "prosperar",
    "prosperásemos": "prosperar",
    "prosperé": "prosperar",
    "prosperéis": "prosperar",
    "prosperó": "prosperar",
    "prostaglandinas": "prostaglandina",
    "prosterna": "prosternar",
    "prosternaba": "prosternar",
    "prosternabais": "prosternar",
    "prosternaban": "prosternar",
    "prosternabas": "prosternar",
    "prosternaciones": "prosternación",
    "prosternad": "prosternar",
    "prosternada": "prosternar",
    "prosternadas": "prosternar",
    "prosternado": "prosternar",
    "prosternados": "prosternar",
    "prosternamos": "prosternar",
    "prosternan": "prosternar",
    "prosternando": "prosternar",
    "prosternara": "prosternar",
    "prosternarais": "prosternar",
    "prosternaran": "prosternar",
    "prosternaras": "prosternar",
    "prosternare": "prosternar",
    "prosternareis": "prosternar",
    "prosternaremos": "prosternar",
    "prosternaren": "prosternar",
    "prosternares": "prosternar",
    "prosternaron": "prosternar",
    "prosternará": "prosternar",
    "prosternarán": "prosternar",
    "prosternarás": "prosternar",
    "prosternaré": "prosternar",
    "prosternaréis": "prosternar",
    "prosternaría": "prosternar",
    "prosternaríais": "prosternar",
    "prosternaríamos": "prosternar",
    "prosternarían": "prosternar",
    "prosternarías": "prosternar",
    "prosternas": "prosternar",
    "prosternase": "prosternar",
    "prosternaseis": "prosternar",
    "prosternasen": "prosternar",
    "prosternases": "prosternar",
    "prosternaste": "prosternar",
    "prosternasteis": "prosternar",
    "prosterne": "prosternar",
    "prosternemos": "prosternar",
    "prosternen": "prosternar",
    "prosternes": "prosternar",
    "prosterno": "prosternar",
    "prosternábamos": "prosternar",
    "prosternáis": "prosternar",
    "prosternáramos": "prosternar",
    "prosternáremos": "prosternar",
    "prosternásemos": "prosternar",
    "prosterné": "prosternar",
    "prosternéis": "prosternar",
    "prosternó": "prosternar",
    "prostibularia": "prostibulario",
    "prostibularias": "prostibulario",
    "prostibularios": "prostibulario",
    "prostituciones": "prostitución",
    "prostituid": "prostituir",
    "prostituida": "prostituir",
    "prostituidas": "prostituir",
    "prostituido": "prostituir",
    "prostituidos": "prostituir",
    "prostituimos": "prostituir",
    "prostituiremos": "prostituir",
    "prostituirá": "prostituir",
    "prostituirán": "prostituir",
    "prostituirás": "prostituir",
    "prostituiré": "prostituir",
    "prostituiréis": "prostituir",
    "prostituiría": "prostituir",
    "prostituiríais": "prostituir",
    "prostituiríamos": "prostituir",
    "prostituirían": "prostituir",
    "prostituirías": "prostituir",
    "prostituiste": "prostituir",
    "prostituisteis": "prostituir",
    "prostituta": "prostituir",
    "prostitutas": "prostituta",
    "prostituto": "prostituir",
    "prostitutos": "prostituto",
    "prostituya": "prostituir",
    "prostituyamos": "prostituir",
    "prostituyan": "prostituir",
    "prostituyas": "prostituir",
    "prostituye": "prostituir",
    "prostituyen": "prostituir",
    "prostituyendo": "prostituir",
    "prostituyera": "prostituir",
    "prostituyerais": "prostituir",
    "prostituyeran": "prostituir",
    "prostituyeras": "prostituir",
    "prostituyere": "prostituir",
    "prostituyereis": "prostituir",
    "prostituyeren": "prostituir",
    "prostituyeres": "prostituir",
    "prostituyeron": "prostituir",
    "prostituyes": "prostituir",
    "prostituyese": "prostituir",
    "prostituyeseis": "prostituir",
    "prostituyesen": "prostituir",
    "prostituyeses": "prostituir",
    "prostituyo": "prostituir",
    "prostituyáis": "prostituir",
    "prostituyéramos": "prostituir",
    "prostituyéremos": "prostituir",
    "prostituyésemos": "prostituir",
    "prostituyó": "prostituir",
    "prostituí": "prostituir",
    "prostituía": "prostituir",
    "prostituíais": "prostituir",
    "prostituíamos": "prostituir",
    "prostituían": "prostituir",
    "prostituías": "prostituir",
    "prostituís": "prostituir",
    "prostodoncias": "prostodoncia",
    "prostraciones": "prostración",
    "prostática": "prostático",
    "prostáticas": "prostático",
    "prostáticos": "prostático",
    "prostética": "prostético",
    "prostéticas": "prostético",
    "prostéticos": "prostético",
    "prostíbulos": "prostíbulo",
    "prosélitos": "prosélito",
    "prosémicas": "prosémica",
    "prosénquimas": "prosénquima",
    "prosística": "prosístico",
    "prosísticas": "prosístico",
    "prosísticos": "prosístico",
    "prosódica": "prosódico",
    "prosódicas": "prosódico",
    "prosódicos": "prosódico",
    "protactinios": "protactinio",
    "protagonice": "protagonizar",
    "protagonicemos": "protagonizar",
    "protagonicen": "protagonizar",
    "protagonices": "protagonizar",
    "protagonicé": "protagonizar",
    "protagonicéis": "protagonizar",
    "protagonismos": "protagonismo",
    "protagonistas": "protagonista",
    "protagoniza": "protagonizar",
    "protagonizaba": "protagonizar",
    "protagonizabais": "protagonizar",
    "protagonizaban": "protagonizar",
    "protagonizabas": "protagonizar",
    "protagonizad": "protagonizar",
    "protagonizada": "protagonizar",
    "protagonizadas": "protagonizar",
    "protagonizado": "protagonizar",
    "protagonizados": "protagonizar",
    "protagonizamos": "protagonizar",
    "protagonizan": "protagonizar",
    "protagonizando": "protagonizar",
    "protagonizara": "protagonizar",
    "protagonizarais": "protagonizar",
    "protagonizaran": "protagonizar",
    "protagonizaras": "protagonizar",
    "protagonizare": "protagonizar",
    "protagonizareis": "protagonizar",
    "protagonizaremos": "protagonizar",
    "protagonizaren": "protagonizar",
    "protagonizares": "protagonizar",
    "protagonizaron": "protagonizar",
    "protagonizará": "protagonizar",
    "protagonizarán": "protagonizar",
    "protagonizarás": "protagonizar",
    "protagonizaré": "protagonizar",
    "protagonizaréis": "protagonizar",
    "protagonizaría": "protagonizar",
    "protagonizaríais": "protagonizar",
    "protagonizaríamos": "protagonizar",
    "protagonizarían": "protagonizar",
    "protagonizarías": "protagonizar",
    "protagonizas": "protagonizar",
    "protagonizase": "protagonizar",
    "protagonizaseis": "protagonizar",
    "protagonizasen": "protagonizar",
    "protagonizases": "protagonizar",
    "protagonizaste": "protagonizar",
    "protagonizasteis": "protagonizar",
    "protagonizo": "protagonizar",
    "protagonizábamos": "protagonizar",
    "protagonizáis": "protagonizar",
    "protagonizáramos": "protagonizar",
    "protagonizáremos": "protagonizar",
    "protagonizásemos": "protagonizar",
    "protagonizó": "protagonizar",
    "protaminas": "protamina",
    "protanomalopsias": "protanomalopsia",
    "protanomalopías": "protanomalopía",
    "protanopsias": "protanopsia",
    "protanopías": "protanopía",
    "proteasas": "proteasa",
    "protecciones": "protección",
    "proteccionismos": "proteccionismo",
    "proteccionistas": "proteccionista",
    "protectora": "protector",
    "protectorados": "protectorado",
    "protectoras": "protector",
    "protectores": "protector",
    "protectoria": "protectorio",
    "protectorias": "protectorio",
    "protectorios": "protectorio",
    "protectorías": "protectoría",
    "protege": "proteger",
    "proteged": "proteger",
    "protegemos": "proteger",
    "protegen": "proteger",
    "protegeremos": "proteger",
    "protegerá": "proteger",
    "protegerán": "proteger",
    "protegerás": "proteger",
    "protegeré": "proteger",
    "protegeréis": "proteger",
    "protegería": "proteger",
    "protegeríais": "proteger",
    "protegeríamos": "proteger",
    "protegerían": "proteger",
    "protegerías": "proteger",
    "proteges": "proteger",
    "protegida": "protegido",
    "protegidas": "protegido",
    "protegido": "proteger",
    "protegidos": "protegido",
    "protegiendo": "proteger",
    "protegiera": "proteger",
    "protegierais": "proteger",
    "protegieran": "proteger",
    "protegieras": "proteger",
    "protegiere": "proteger",
    "protegiereis": "proteger",
    "protegieren": "proteger",
    "protegieres": "proteger",
    "protegieron": "proteger",
    "protegiese": "proteger",
    "protegieseis": "proteger",
    "protegiesen": "proteger",
    "protegieses": "proteger",
    "protegimos": "proteger",
    "protegiste": "proteger",
    "protegisteis": "proteger",
    "protegiéramos": "proteger",
    "protegiéremos": "proteger",
    "protegiésemos": "proteger",
    "protegió": "proteger",
    "protegéis": "proteger",
    "protegí": "proteger",
    "protegía": "proteger",
    "protegíais": "proteger",
    "protegíamos": "proteger",
    "protegían": "proteger",
    "protegías": "proteger",
    "proteica": "proteico",
    "proteicas": "proteico",
    "proteicos": "proteico",
    "proteidos": "proteido",
    "proteinemias": "proteinemia",
    "proteinogramas": "proteinograma",
    "proteinurias": "proteinuria",
    "proteja": "proteger",
    "protejamos": "proteger",
    "protejan": "proteger",
    "protejas": "proteger",
    "protejo": "proteger",
    "protejáis": "proteger",
    "proteolítica": "proteolítico",
    "proteolíticas": "proteolítico",
    "proteolíticos": "proteolítico",
    "proteos": "proteo",
    "proterva": "protervo",
    "protervas": "protervo",
    "protervias": "protervia",
    "protervidades": "protervidad",
    "protervos": "protervo",
    "protesistas": "protesista",
    "protesta": "protestar",
    "protestaba": "protestar",
    "protestabais": "protestar",
    "protestaban": "protestar",
    "protestabas": "protestar",
    "protestaciones": "protestación",
    "protestad": "protestar",
    "protestada": "protestar",
    "protestadas": "protestar",
    "protestado": "protestar",
    "protestados": "protestar",
    "protestamos": "protestar",
    "protestan": "protestar",
    "protestando": "protestar",
    "protestantes": "protestante",
    "protestantismos": "protestantismo",
    "protestara": "protestar",
    "protestarais": "protestar",
    "protestaran": "protestar",
    "protestaras": "protestar",
    "protestare": "protestar",
    "protestareis": "protestar",
    "protestaremos": "protestar",
    "protestaren": "protestar",
    "protestares": "protestar",
    "protestaron": "protestar",
    "protestará": "protestar",
    "protestarán": "protestar",
    "protestarás": "protestar",
    "protestaré": "protestar",
    "protestaréis": "protestar",
    "protestaría": "protestar",
    "protestaríais": "protestar",
    "protestaríamos": "protestar",
    "protestarían": "protestar",
    "protestarías": "protestar",
    "protestas": "protestar",
    "protestase": "protestar",
    "protestaseis": "protestar",
    "protestasen": "protestar",
    "protestases": "protestar",
    "protestaste": "protestar",
    "protestasteis": "protestar",
    "protestataria": "protestatario",
    "protestatarias": "protestatario",
    "protestatarios": "protestatario",
    "proteste": "protestar",
    "protestemos": "protestar",
    "protesten": "protestar",
    "protestes": "protestar",
    "protesto": "protestar",
    "protestos": "protesto",
    "protestábamos": "protestar",
    "protestáis": "protestar",
    "protestáramos": "protestar",
    "protestáremos": "protestar",
    "protestásemos": "protestar",
    "protesté": "protestar",
    "protestéis": "protestar",
    "protestó": "protestar",
    "proteácea": "proteáceo",
    "proteáceas": "proteáceo",
    "proteáceos": "proteáceo",
    "proteínas": "proteína",
    "proteínica": "proteínico",
    "proteínicas": "proteínico",
    "proteínicos": "proteínico",
    "protios": "protio",
    "protoactinios": "protoactinio",
    "protoblastos": "protoblasto",
    "protocola": "protocolar",
    "protocolaba": "protocolar",
    "protocolabais": "protocolar",
    "protocolaban": "protocolar",
    "protocolabas": "protocolar",
    "protocolad": "protocolar",
    "protocolada": "protocolar",
    "protocoladas": "protocolar",
    "protocolado": "protocolar",
    "protocolados": "protocolar",
    "protocolamos": "protocolar",
    "protocolan": "protocolar",
    "protocolando": "protocolar",
    "protocolara": "protocolar",
    "protocolarais": "protocolar",
    "protocolaran": "protocolar",
    "protocolaras": "protocolar",
    "protocolare": "protocolar",
    "protocolareis": "protocolar",
    "protocolaremos": "protocolar",
    "protocolaren": "protocolar",
    "protocolares": "protocolar",
    "protocolaria": "protocolario",
    "protocolarias": "protocolario",
    "protocolarios": "protocolario",
    "protocolaron": "protocolar",
    "protocolará": "protocolar",
    "protocolarán": "protocolar",
    "protocolarás": "protocolar",
    "protocolaré": "protocolar",
    "protocolaréis": "protocolar",
    "protocolaría": "protocolar",
    "protocolaríais": "protocolar",
    "protocolaríamos": "protocolar",
    "protocolarían": "protocolar",
    "protocolarías": "protocolar",
    "protocolas": "protocolar",
    "protocolase": "protocolar",
    "protocolaseis": "protocolar",
    "protocolasen": "protocolar",
    "protocolases": "protocolar",
    "protocolaste": "protocolar",
    "protocolasteis": "protocolar",
    "protocole": "protocolar",
    "protocolemos": "protocolar",
    "protocolen": "protocolar",
    "protocoles": "protocolar",
    "protocolice": "protocolizar",
    "protocolicemos": "protocolizar",
    "protocolicen": "protocolizar",
    "protocolices": "protocolizar",
    "protocolicé": "protocolizar",
    "protocolicéis": "protocolizar",
    "protocoliza": "protocolizar",
    "protocolizaba": "protocolizar",
    "protocolizabais": "protocolizar",
    "protocolizaban": "protocolizar",
    "protocolizabas": "protocolizar",
    "protocolizaciones": "protocolización",
    "protocolizad": "protocolizar",
    "protocolizada": "protocolizar",
    "protocolizadas": "protocolizar",
    "protocolizado": "protocolizar",
    "protocolizados": "protocolizar",
    "protocolizamos": "protocolizar",
    "protocolizan": "protocolizar",
    "protocolizando": "protocolizar",
    "protocolizara": "protocolizar",
    "protocolizarais": "protocolizar",
    "protocolizaran": "protocolizar",
    "protocolizaras": "protocolizar",
    "protocolizare": "protocolizar",
    "protocolizareis": "protocolizar",
    "protocolizaremos": "protocolizar",
    "protocolizaren": "protocolizar",
    "protocolizares": "protocolizar",
    "protocolizaron": "protocolizar",
    "protocolizará": "protocolizar",
    "protocolizarán": "protocolizar",
    "protocolizarás": "protocolizar",
    "protocolizaré": "protocolizar",
    "protocolizaréis": "protocolizar",
    "protocolizaría": "protocolizar",
    "protocolizaríais": "protocolizar",
    "protocolizaríamos": "protocolizar",
    "protocolizarían": "protocolizar",
    "protocolizarías": "protocolizar",
    "protocolizas": "protocolizar",
    "protocolizase": "protocolizar",
    "protocolizaseis": "protocolizar",
    "protocolizasen": "protocolizar",
    "protocolizases": "protocolizar",
    "protocolizaste": "protocolizar",
    "protocolizasteis": "protocolizar",
    "protocolizo": "protocolizar",
    "protocolizábamos": "protocolizar",
    "protocolizáis": "protocolizar",
    "protocolizáramos": "protocolizar",
    "protocolizáremos": "protocolizar",
    "protocolizásemos": "protocolizar",
    "protocolizó": "protocolizar",
    "protocolo": "protocolar",
    "protocolos": "protocolo",
    "protocolábamos": "protocolar",
    "protocoláis": "protocolar",
    "protocoláramos": "protocolar",
    "protocoláremos": "protocolar",
    "protocolásemos": "protocolar",
    "protocolé": "protocolar",
    "protocoléis": "protocolar",
    "protocoló": "protocolar",
    "protoevangelios": "protoevangelio",
    "protofilamentos": "protofilamento",
    "protogenética": "protogenético",
    "protogenéticas": "protogenético",
    "protogenéticos": "protogenético",
    "protohistorias": "protohistoria",
    "protohistórica": "protohistórico",
    "protohistóricas": "protohistórico",
    "protohistóricos": "protohistórico",
    "protomedicatos": "protomedicato",
    "protomártires": "protomártir",
    "protomédicos": "protomédico",
    "protones": "protón",
    "protonotaria": "protonotario",
    "protonotarias": "protonotario",
    "protonotarios": "protonotario",
    "protoplasmas": "protoplasma",
    "protoplasmática": "protoplasmático",
    "protoplasmáticas": "protoplasmático",
    "protoplasmáticos": "protoplasmático",
    "protoplastos": "protoplasto",
    "protoplásmica": "protoplásmico",
    "protoplásmicas": "protoplásmico",
    "protoplásmicos": "protoplásmico",
    "protosoles": "protosol",
    "prototipa": "prototipar",
    "prototipaba": "prototipar",
    "prototipabais": "prototipar",
    "prototipaban": "prototipar",
    "prototipabas": "prototipar",
    "prototipad": "prototipar",
    "prototipada": "prototipar",
    "prototipadas": "prototipar",
    "prototipado": "prototipar",
    "prototipados": "prototipar",
    "prototipamos": "prototipar",
    "prototipan": "prototipar",
    "prototipando": "prototipar",
    "prototipara": "prototipar",
    "prototiparais": "prototipar",
    "prototiparan": "prototipar",
    "prototiparas": "prototipar",
    "prototipare": "prototipar",
    "prototipareis": "prototipar",
    "prototiparemos": "prototipar",
    "prototiparen": "prototipar",
    "prototipares": "prototipar",
    "prototiparon": "prototipar",
    "prototipará": "prototipar",
    "prototiparán": "prototipar",
    "prototiparás": "prototipar",
    "prototiparé": "prototipar",
    "prototiparéis": "prototipar",
    "prototiparía": "prototipar",
    "prototiparíais": "prototipar",
    "prototiparíamos": "prototipar",
    "prototiparían": "prototipar",
    "prototiparías": "prototipar",
    "prototipas": "prototipar",
    "prototipase": "prototipar",
    "prototipaseis": "prototipar",
    "prototipasen": "prototipar",
    "prototipases": "prototipar",
    "prototipaste": "prototipar",
    "prototipasteis": "prototipar",
    "prototipe": "prototipar",
    "prototipemos": "prototipar",
    "prototipen": "prototipar",
    "prototipes": "prototipar",
    "prototipo": "prototipar",
    "prototipos": "prototipo",
    "prototipábamos": "prototipar",
    "prototipáis": "prototipar",
    "prototipáramos": "prototipar",
    "prototipáremos": "prototipar",
    "prototipásemos": "prototipar",
    "prototipé": "prototipar",
    "prototipéis": "prototipar",
    "prototipó": "prototipar",
    "prototípica": "prototípico",
    "prototípicas": "prototípico",
    "prototípicos": "prototípico",
    "protoveratrinas": "protoveratrina",
    "protozoologías": "protozoología",
    "protozoos": "protozoo",
    "protracciones": "protracción",
    "protrombinas": "protrombina",
    "protrusiones": "protrusión",
    "protrusiva": "protrusivo",
    "protrusivas": "protrusivo",
    "protrusivos": "protrusivo",
    "protráctiles": "protráctil",
    "protrómbica": "protrómbico",
    "protrómbicas": "protrómbico",
    "protrómbicos": "protrómbico",
    "protuberanciales": "protuberancial",
    "protuberancias": "protuberancia",
    "protuberantes": "protuberante",
    "protutora": "protutor",
    "protutoras": "protutor",
    "protutores": "protutor",
    "protánopes": "protánope",
    "protática": "protático",
    "protáticas": "protático",
    "protáticos": "protático",
    "protésica": "protésico",
    "protésicas": "protésico",
    "protésicos": "protésico",
    "protética": "protético",
    "protéticas": "protético",
    "protéticos": "protético",
    "protófitos": "protófito",
    "protónica": "protónico",
    "protónicas": "protónico",
    "protónicos": "protónico",
    "protóraxes": "protórax",
    "protótrofa": "protótrofo",
    "protótrofas": "protótrofo",
    "protótrofos": "protótrofo",
    "proustitas": "proustita",
    "prov": "provincia",
    "prova": "provo",
    "provas": "provo",
    "provea": "proveer",
    "proveamos": "proveer",
    "provean": "proveer",
    "proveas": "proveer",
    "provechos": "provecho",
    "provechosa": "provechoso",
    "provechosas": "provechoso",
    "provechosos": "provechoso",
    "provecta": "provecto",
    "provectas": "provecto",
    "provectos": "provecto",
    "provee": "proveer",
    "proveed": "proveer",
    "proveedora": "proveedor",
    "proveedoras": "proveedor",
    "proveedores": "proveedor",
    "proveedurías": "proveeduría",
    "proveemos": "proveer",
    "proveen": "proveer",
    "proveeremos": "proveer",
    "proveerá": "proveer",
    "proveerán": "proveer",
    "proveerás": "proveer",
    "proveeré": "proveer",
    "proveeréis": "proveer",
    "proveería": "proveer",
    "proveeríais": "proveer",
    "proveeríamos": "proveer",
    "proveerían": "proveer",
    "proveerías": "proveer",
    "provees": "proveer",
    "proveimientos": "proveimiento",
    "proven": "provenir",
    "provenas": "provena",
    "provendremos": "provenir",
    "provendrá": "provenir",
    "provendrán": "provenir",
    "provendrás": "provenir",
    "provendré": "provenir",
    "provendréis": "provenir",
    "provendría": "provenir",
    "provendríais": "provenir",
    "provendríamos": "provenir",
    "provendrían": "provenir",
    "provendrías": "provenir",
    "provenga": "provenir",
    "provengamos": "provenir",
    "provengan": "provenir",
    "provengas": "provenir",
    "provengo": "provenir",
    "provengáis": "provenir",
    "provenid": "provenir",
    "provenida": "provenir",
    "provenidas": "provenir",
    "provenido": "provenir",
    "provenidos": "provenir",
    "proveniencias": "proveniencia",
    "provenientes": "proveniente",
    "provenimos": "provenir",
    "provenzalas": "provenzal",
    "provenzales": "provenzal",
    "provenzalismos": "provenzalismo",
    "provenzalistas": "provenzalista",
    "provenía": "provenir",
    "proveníais": "provenir",
    "proveníamos": "provenir",
    "provenían": "provenir",
    "provenías": "provenir",
    "provenís": "provenir",
    "proveo": "proveer",
    "proverbiadores": "proverbiador",
    "proverbiales": "proverbial",
    "proverbios": "proverbio",
    "proverbistas": "proverbista",
    "proveyendo": "proveer",
    "proveyera": "proveer",
    "proveyerais": "proveer",
    "proveyeran": "proveer",
    "proveyeras": "proveer",
    "proveyere": "proveer",
    "proveyereis": "proveer",
    "proveyeren": "proveer",
    "proveyeres": "proveer",
    "proveyeron": "proveer",
    "proveyese": "proveer",
    "proveyeseis": "proveer",
    "proveyesen": "proveer",
    "proveyeses": "proveer",
    "proveyéramos": "proveer",
    "proveyéremos": "proveer",
    "proveyésemos": "proveer",
    "proveyó": "proveer",
    "proveáis": "proveer",
    "proveéis": "proveer",
    "proveí": "proveer",
    "proveía": "proveer",
    "proveíais": "proveer",
    "proveíamos": "proveer",
    "proveían": "proveer",
    "proveías": "proveer",
    "proveída": "proveer",
    "proveídas": "proveer",
    "proveído": "proveer",
    "proveídos": "proveído",
    "proveímos": "proveer",
    "proveíste": "proveer",
    "proveísteis": "proveer",
    "providencia": "providenciar",
    "providenciaba": "providenciar",
    "providenciabais": "providenciar",
    "providenciaban": "providenciar",
    "providenciabas": "providenciar",
    "providenciad": "providenciar",
    "providenciada": "providenciar",
    "providenciadas": "providenciar",
    "providenciado": "providenciar",
    "providenciados": "providenciar",
    "providenciales": "providencial",
    "providencialismos": "providencialismo",
    "providencialistas": "providencialista",
    "providenciamos": "providenciar",
    "providencian": "providenciar",
    "providenciando": "providenciar",
    "providenciara": "providenciar",
    "providenciarais": "providenciar",
    "providenciaran": "providenciar",
    "providenciaras": "providenciar",
    "providenciare": "providenciar",
    "providenciareis": "providenciar",
    "providenciaremos": "providenciar",
    "providenciaren": "providenciar",
    "providenciares": "providenciar",
    "providenciaron": "providenciar",
    "providenciará": "providenciar",
    "providenciarán": "providenciar",
    "providenciarás": "providenciar",
    "providenciaré": "providenciar",
    "providenciaréis": "providenciar",
    "providenciaría": "providenciar",
    "providenciaríais": "providenciar",
    "providenciaríamos": "providenciar",
    "providenciarían": "providenciar",
    "providenciarías": "providenciar",
    "providencias": "providenciar",
    "providenciase": "providenciar",
    "providenciaseis": "providenciar",
    "providenciasen": "providenciar",
    "providenciases": "providenciar",
    "providenciaste": "providenciar",
    "providenciasteis": "providenciar",
    "providencie": "providenciar",
    "providenciemos": "providenciar",
    "providencien": "providenciar",
    "providencies": "providenciar",
    "providencio": "providenciar",
    "providenciábamos": "providenciar",
    "providenciáis": "providenciar",
    "providenciáramos": "providenciar",
    "providenciáremos": "providenciar",
    "providenciásemos": "providenciar",
    "providencié": "providenciar",
    "providenciéis": "providenciar",
    "providenció": "providenciar",
    "providentes": "providente",
    "proviene": "provenir",
    "provienen": "provenir",
    "provienes": "provenir",
    "provincialas": "provinciala",
    "provincialatos": "provincialato",
    "provinciales": "provincial",
    "provincialismos": "provincialismo",
    "provincializaciones": "provincialización",
    "provinciana": "provinciano",
    "provincianas": "provinciano",
    "provincianismos": "provincianismo",
    "provincianos": "provinciano",
    "provincias": "provincia",
    "provine": "provenir",
    "proviniendo": "provenir",
    "proviniera": "provenir",
    "provinierais": "provenir",
    "provinieran": "provenir",
    "provinieras": "provenir",
    "proviniere": "provenir",
    "proviniereis": "provenir",
    "provinieren": "provenir",
    "provinieres": "provenir",
    "provinieron": "provenir",
    "proviniese": "provenir",
    "provinieseis": "provenir",
    "proviniesen": "provenir",
    "provinieses": "provenir",
    "provinimos": "provenir",
    "proviniste": "provenir",
    "provinisteis": "provenir",
    "proviniéramos": "provenir",
    "proviniéremos": "provenir",
    "proviniésemos": "provenir",
    "provino": "provenir",
    "provisionales": "provisional",
    "provisionalidades": "provisionalidad",
    "provisiones": "provisión",
    "provisora": "provisor",
    "provisoras": "provisor",
    "provisoratos": "provisorato",
    "provisores": "provisor",
    "provisoria": "provisorio",
    "provisorias": "provisorio",
    "provisorios": "provisorio",
    "provisorías": "provisoría",
    "provista": "proveer",
    "provistas": "proveer",
    "provisto": "proveer",
    "provistos": "proveer",
    "provitaminas": "provitamina",
    "provoca": "provocar",
    "provocaba": "provocar",
    "provocabais": "provocar",
    "provocaban": "provocar",
    "provocabas": "provocar",
    "provocaciones": "provocación",
    "provocad": "provocar",
    "provocada": "provocar",
    "provocadas": "provocar",
    "provocado": "provocar",
    "provocadora": "provocador",
    "provocadoras": "provocador",
    "provocadores": "provocador",
    "provocados": "provocar",
    "provocamos": "provocar",
    "provocan": "provocar",
    "provocando": "provocar",
    "provocara": "provocar",
    "provocarais": "provocar",
    "provocaran": "provocar",
    "provocaras": "provocar",
    "provocare": "provocar",
    "provocareis": "provocar",
    "provocaremos": "provocar",
    "provocaren": "provocar",
    "provocares": "provocar",
    "provocaron": "provocar",
    "provocará": "provocar",
    "provocarán": "provocar",
    "provocarás": "provocar",
    "provocaré": "provocar",
    "provocaréis": "provocar",
    "provocaría": "provocar",
    "provocaríais": "provocar",
    "provocaríamos": "provocar",
    "provocarían": "provocar",
    "provocarías": "provocar",
    "provocas": "provocar",
    "provocase": "provocar",
    "provocaseis": "provocar",
    "provocasen": "provocar",
    "provocases": "provocar",
    "provocaste": "provocar",
    "provocasteis": "provocar",
    "provocativa": "provocativo",
    "provocativas": "provocativo",
    "provocativos": "provocativo",
    "provoco": "provocar",
    "provocábamos": "provocar",
    "provocáis": "provocar",
    "provocáramos": "provocar",
    "provocáremos": "provocar",
    "provocásemos": "provocar",
    "provocó": "provocar",
    "provoque": "provocar",
    "provoquemos": "provocar",
    "provoquen": "provocar",
    "provoques": "provocar",
    "provoqué": "provocar",
    "provoquéis": "provocar",
    "provos": "provo",
    "proxemiologías": "proxemiología",
    "proxenetas": "proxeneta",
    "proxenetismos": "proxenetismo",
    "proxenitismos": "proxenitismo",
    "proxenética": "proxenético",
    "proxenéticas": "proxenético",
    "proxenéticos": "proxenético",
    "proximales": "proximal",
    "proximidades": "proximidad",
    "proyecciones": "proyección",
    "proyecta": "proyectar",
    "proyectaba": "proyectar",
    "proyectabais": "proyectar",
    "proyectaban": "proyectar",
    "proyectabas": "proyectar",
    "proyectables": "proyectable",
    "proyectad": "proyectar",
    "proyectada": "proyectar",
    "proyectadas": "proyectar",
    "proyectado": "proyectar",
    "proyectados": "proyectar",
    "proyectamos": "proyectar",
    "proyectan": "proyectar",
    "proyectando": "proyectar",
    "proyectantes": "proyectante",
    "proyectara": "proyectar",
    "proyectarais": "proyectar",
    "proyectaran": "proyectar",
    "proyectaras": "proyectar",
    "proyectare": "proyectar",
    "proyectareis": "proyectar",
    "proyectaremos": "proyectar",
    "proyectaren": "proyectar",
    "proyectares": "proyectar",
    "proyectaron": "proyectar",
    "proyectará": "proyectar",
    "proyectarán": "proyectar",
    "proyectarás": "proyectar",
    "proyectaré": "proyectar",
    "proyectaréis": "proyectar",
    "proyectaría": "proyectar",
    "proyectaríais": "proyectar",
    "proyectaríamos": "proyectar",
    "proyectarían": "proyectar",
    "proyectarías": "proyectar",
    "proyectas": "proyectar",
    "proyectase": "proyectar",
    "proyectaseis": "proyectar",
    "proyectasen": "proyectar",
    "proyectases": "proyectar",
    "proyectaste": "proyectar",
    "proyectasteis": "proyectar",
    "proyecte": "proyectar",
    "proyectemos": "proyectar",
    "proyecten": "proyectar",
    "proyectes": "proyectar",
    "proyectiles": "proyectil",
    "proyectistas": "proyectista",
    "proyectiva": "proyectivo",
    "proyectivas": "proyectivo",
    "proyectividades": "proyectividad",
    "proyectivos": "proyectivo",
    "proyecto": "proyectar",
    "proyectores": "proyector",
    "proyectos": "proyecto",
    "proyecturas": "proyectura",
    "proyectábamos": "proyectar",
    "proyectáis": "proyectar",
    "proyectáramos": "proyectar",
    "proyectáremos": "proyectar",
    "proyectásemos": "proyectar",
    "proyecté": "proyectar",
    "proyectéis": "proyectar",
    "proyectó": "proyectar",
    "proíces": "proíz",
    "proíses": "proís",
    "prudenciales": "prudencial",
    "prudencias": "prudencia",
    "prudentes": "prudente",
    "prueba": "probar",
    "prueban": "probar",
    "pruebas": "prueba",
    "pruebe": "probar",
    "prueben": "probar",
    "pruebes": "probar",
    "pruebo": "probar",
    "pruinas": "pruina",
    "pruinosa": "pruinoso",
    "pruinosas": "pruinoso",
    "pruinosos": "pruinoso",
    "pruriginosa": "pruriginoso",
    "pruriginosas": "pruriginoso",
    "pruriginosos": "pruriginoso",
    "prurigos": "prurigo",
    "pruritos": "prurito",
    "prusiana": "prusiano",
    "prusianas": "prusiano",
    "prusianos": "prusiano",
    "prácritos": "prácrito",
    "práctica": "práctico",
    "prácticas": "práctico",
    "prácticos": "práctico",
    "prédicas": "prédica",
    "présaga": "présago",
    "présagas": "présago",
    "présagos": "présago",
    "présbitas": "présbita",
    "présbites": "présbite",
    "préstamos": "préstamo",
    "prímulas": "prímula",
    "príncipes": "príncipe",
    "prístina": "prístino",
    "prístinas": "prístino",
    "prístinos": "prístino",
    "prócera": "prócero",
    "próceras": "prócero",
    "próceres": "prócer",
    "próceros": "prócero",
    "pródiga": "pródigo",
    "pródigas": "pródigo",
    "pródigos": "pródigo",
    "pródromos": "pródromo",
    "prófuga": "prófugo",
    "prófugas": "prófugo",
    "prófugos": "prófugo",
    "prójima": "prójimo",
    "prójimas": "prójimo",
    "prójimos": "prójimo",
    "prólogos": "prólogo",
    "prórrogas": "prórroga",
    "próspera": "próspero",
    "prósperas": "próspero",
    "prósperos": "próspero",
    "próstatas": "próstata",
    "próstila": "próstilo",
    "próstilas": "próstilo",
    "próstilos": "próstilo",
    "prótidos": "prótido",
    "próvida": "próvido",
    "próvidas": "próvido",
    "próvidos": "próvido",
    "próxima": "próximo",
    "próximas": "próximo",
    "próximos": "próximo",
    "prúsica": "prúsico",
    "prúsicas": "prúsico",
    "prúsicos": "prúsico",
    "psalidodoncias": "psalidodoncia",
    "psammitas": "psammita",
    "psamófila": "psamófilo",
    "psamófilas": "psamófilo",
    "psamófilos": "psamófilo",
    "psefitas": "psefita",
    "pseudoaccidentales": "pseudoaccidental",
    "pseudoacrosomas": "pseudoacrosoma",
    "pseudoaleatoria": "pseudoaleatorio",
    "pseudoaleatorias": "pseudoaleatorio",
    "pseudoaleatorios": "pseudoaleatorio",
    "pseudoalveolares": "pseudoalveolar",
    "pseudobases": "pseudobase",
    "pseudociencias": "pseudociencia",
    "pseudocientífica": "pseudocientífico",
    "pseudocientíficas": "pseudocientífico",
    "pseudocientíficos": "pseudocientífico",
    "pseudocilíndrica": "pseudocilíndrico",
    "pseudocilíndricas": "pseudocilíndrico",
    "pseudocilíndricos": "pseudocilíndrico",
    "pseudocumenos": "pseudocumeno",
    "pseudocódigos": "pseudocódigo",
    "pseudoesferas": "pseudoesfera",
    "pseudoesférica": "pseudoesférico",
    "pseudoesféricas": "pseudoesférico",
    "pseudoesféricos": "pseudoesférico",
    "pseudofitas": "pseudofita",
    "pseudogenes": "pseudogén",
    "pseudologías": "pseudología",
    "pseudomorfa": "pseudomorfo",
    "pseudomorfas": "pseudomorfo",
    "pseudomorfos": "pseudomorfo",
    "pseudoptialismos": "pseudoptialismo",
    "pseudoracémica": "pseudoracémico",
    "pseudoracémicas": "pseudoracémico",
    "pseudoracémicos": "pseudoracémico",
    "pseudotopológica": "pseudotopológico",
    "pseudotopológicas": "pseudotopológico",
    "pseudotopológicos": "pseudotopológico",
    "pseudovariedades": "pseudovariedad",
    "pseudoácidos": "pseudoácido",
    "pseudónimos": "pseudónimo",
    "pseudópodos": "pseudópodo",
    "psicagogias": "psicagogia",
    "psicastenias": "psicastenia",
    "psicasténica": "psicasténico",
    "psicasténicas": "psicasténico",
    "psicasténicos": "psicasténico",
    "psicoanalistas": "psicoanalista",
    "psicoanalítica": "psicoanalítico",
    "psicoanalíticas": "psicoanalítico",
    "psicoanalíticos": "psicoanalítico",
    "psicodelias": "psicodelia",
    "psicodepresora": "psicodepresor",
    "psicodepresoras": "psicodepresor",
    "psicodepresores": "psicodepresor",
    "psicodiagnósticos": "psicodiagnóstico",
    "psicodramas": "psicodrama",
    "psicodélica": "psicodélico",
    "psicodélicas": "psicodélico",
    "psicodélicos": "psicodélico",
    "psicoesferas": "psicoesfera",
    "psicofarmacologias": "psicofarmacologia",
    "psicofarmacologías": "psicofarmacología",
    "psicofarmacológica": "psicofarmacológico",
    "psicofarmacológicas": "psicofarmacológico",
    "psicofarmacológicos": "psicofarmacológico",
    "psicofisiologías": "psicofisiología",
    "psicofisiológica": "psicofisiológico",
    "psicofisiológicas": "psicofisiológico",
    "psicofisiológicos": "psicofisiológico",
    "psicofísica": "psicofísico",
    "psicofísicas": "psicofísico",
    "psicofísicos": "psicofísico",
    "psicogramas": "psicograma",
    "psicogénica": "psicogénico",
    "psicogénicas": "psicogénico",
    "psicogénicos": "psicogénico",
    "psicolingüísticas": "psicolingüística",
    "psicologismos": "psicologismo",
    "psicologistas": "psicologista",
    "psicologías": "psicología",
    "psicológica": "psicológico",
    "psicológicas": "psicológico",
    "psicológicos": "psicológico",
    "psicometrías": "psicometría",
    "psicomotrices": "psicomotriz",
    "psicomotricidades": "psicomotricidad",
    "psicométrica": "psicométrico",
    "psicométricas": "psicométrico",
    "psicométricos": "psicométrico",
    "psicopatologías": "psicopatología",
    "psicopatológica": "psicopatológico",
    "psicopatológicas": "psicopatológico",
    "psicopatológicos": "psicopatológico",
    "psicopatías": "psicopatía",
    "psicopatóloga": "psicopatólogo",
    "psicopatólogas": "psicopatólogo",
    "psicopatólogos": "psicopatólogo",
    "psicopedagogos": "psicopedagogo",
    "psicopedagogías": "psicopedagogía",
    "psicopedagógica": "psicopedagógico",
    "psicopedagógicas": "psicopedagógico",
    "psicopedagógicos": "psicopedagógico",
    "psicopática": "psicopático",
    "psicopáticas": "psicopático",
    "psicopáticos": "psicopático",
    "psicoquímicas": "psicoquímica",
    "psicosexuales": "psicosexual",
    "psicosociales": "psicosocial",
    "psicosociológica": "psicosociológico",
    "psicosociológicas": "psicosociológico",
    "psicosociológicos": "psicosociológico",
    "psicosomática": "psicosomático",
    "psicosomáticas": "psicosomático",
    "psicosomáticos": "psicosomático",
    "psicotecnias": "psicotecnia",
    "psicoterapias": "psicoterapia",
    "psicoterápica": "psicoterápico",
    "psicoterápicas": "psicoterápico",
    "psicoterápicos": "psicoterápico",
    "psicotrópica": "psicotrópico",
    "psicotrópicas": "psicotrópico",
    "psicotrópicos": "psicotrópico",
    "psicotécnica": "psicotécnico",
    "psicotécnicas": "psicotécnico",
    "psicotécnicos": "psicotécnico",
    "psicrometrías": "psicrometría",
    "psicrométrica": "psicrométrico",
    "psicrométricas": "psicrométrico",
    "psicrométricos": "psicrométrico",
    "psicrómetros": "psicrómetro",
    "psicógena": "psicógeno",
    "psicógenas": "psicógeno",
    "psicógenos": "psicógeno",
    "psicóloga": "psicólogo",
    "psicólogas": "psicólogo",
    "psicólogos": "psicólogo",
    "psicómetros": "psicómetro",
    "psicópatas": "psicópata",
    "psicótica": "psicótico",
    "psicóticas": "psicótico",
    "psicóticos": "psicótico",
    "psilomelanas": "psilomelana",
    "psilomelanos": "psilomelano",
    "psiques": "psique",
    "psiquiatras": "psiquiatra",
    "psiquiatrías": "psiquiatría",
    "psiquismos": "psiquismo",
    "psiquiátrica": "psiquiátrico",
    "psiquiátricas": "psiquiátrico",
    "psiquiátricos": "psiquiátrico",
    "psis": "psi",
    "psitaciformes": "psitaciforme",
    "psitacismos": "psitacismo",
    "psofométrica": "psofométrico",
    "psofométricas": "psofométrico",
    "psofométricos": "psofométrico",
    "psofómetros": "psofómetro",
    "psoriácica": "psoriácico",
    "psoriácicas": "psoriácico",
    "psoriácicos": "psoriácico",
    "psíquica": "psíquico",
    "psíquicas": "psíquico",
    "psíquicos": "psíquico",
    "psórica": "psórico",
    "psóricas": "psórico",
    "psóricos": "psórico",
    "pta": "peseta",
    "ptas": "peseta",
    "pteridófitos": "pteridófito",
    "pterigios": "pterigio",
    "pterigoespinosa": "pterigoespinoso",
    "pterigoespinosas": "pterigoespinoso",
    "pterigoespinosos": "pterigoespinoso",
    "pterigofaríngea": "pterigofaríngeo",
    "pterigofaríngeas": "pterigofaríngeo",
    "pterigofaríngeos": "pterigofaríngeo",
    "pterigoidea": "pterigoideo",
    "pterigoideas": "pterigoideo",
    "pterigoideos": "pterigoideo",
    "pterigomandibulares": "pterigomandibular",
    "pterigomaxilares": "pterigomaxilar",
    "pterigopalatina": "pterigopalatino",
    "pterigopalatinas": "pterigopalatino",
    "pterigopalatinos": "pterigopalatino",
    "ptialinas": "ptialina",
    "ptialismos": "ptialismo",
    "ptialografías": "ptialografía",
    "ptialogénica": "ptialogénico",
    "ptialogénicas": "ptialogénico",
    "ptialogénicos": "ptialogénico",
    "ptialolitotomías": "ptialolitotomía",
    "ptialorreas": "ptialorrea",
    "ptomaínas": "ptomaína",
    "pts": "peseta",
    "puberales": "puberal",
    "pubertades": "pubertad",
    "pubescencias": "pubescencia",
    "pubescentes": "pubescente",
    "pubiana": "pubiano",
    "pubianas": "pubiano",
    "pubianos": "pubiano",
    "pubillajes": "pubillaje",
    "pubillas": "pubilla",
    "publica": "publicar",
    "publicaba": "publicar",
    "publicabais": "publicar",
    "publicaban": "publicar",
    "publicabas": "publicar",
    "publicables": "publicable",
    "publicaciones": "publicación",
    "publicad": "publicar",
    "publicada": "publicar",
    "publicadas": "publicar",
    "publicado": "publicar",
    "publicadora": "publicador",
    "publicadoras": "publicador",
    "publicadores": "publicador",
    "publicados": "publicar",
    "publicamos": "publicar",
    "publican": "publicar",
    "publicando": "publicar",
    "publicanos": "publicano",
    "publicara": "publicar",
    "publicarais": "publicar",
    "publicaran": "publicar",
    "publicaras": "publicar",
    "publicare": "publicar",
    "publicareis": "publicar",
    "publicaremos": "publicar",
    "publicaren": "publicar",
    "publicares": "publicar",
    "publicaron": "publicar",
    "publicará": "publicar",
    "publicarán": "publicar",
    "publicarás": "publicar",
    "publicaré": "publicar",
    "publicaréis": "publicar",
    "publicaría": "publicar",
    "publicaríais": "publicar",
    "publicaríamos": "publicar",
    "publicarían": "publicar",
    "publicarías": "publicar",
    "publicas": "publicar",
    "publicase": "publicar",
    "publicaseis": "publicar",
    "publicasen": "publicar",
    "publicases": "publicar",
    "publicaste": "publicar",
    "publicasteis": "publicar",
    "publicidades": "publicidad",
    "publicismos": "publicismo",
    "publicistas": "publicista",
    "publicita": "publicitar",
    "publicitaba": "publicitar",
    "publicitabais": "publicitar",
    "publicitaban": "publicitar",
    "publicitabas": "publicitar",
    "publicitad": "publicitar",
    "publicitada": "publicitar",
    "publicitadas": "publicitar",
    "publicitado": "publicitar",
    "publicitados": "publicitar",
    "publicitamos": "publicitar",
    "publicitan": "publicitar",
    "publicitando": "publicitar",
    "publicitara": "publicitar",
    "publicitarais": "publicitar",
    "publicitaran": "publicitar",
    "publicitaras": "publicitar",
    "publicitare": "publicitar",
    "publicitareis": "publicitar",
    "publicitaremos": "publicitar",
    "publicitaren": "publicitar",
    "publicitares": "publicitar",
    "publicitaria": "publicitario",
    "publicitarias": "publicitario",
    "publicitarios": "publicitario",
    "publicitaron": "publicitar",
    "publicitará": "publicitar",
    "publicitarán": "publicitar",
    "publicitarás": "publicitar",
    "publicitaré": "publicitar",
    "publicitaréis": "publicitar",
    "publicitaría": "publicitar",
    "publicitaríais": "publicitar",
    "publicitaríamos": "publicitar",
    "publicitarían": "publicitar",
    "publicitarías": "publicitar",
    "publicitas": "publicitar",
    "publicitase": "publicitar",
    "publicitaseis": "publicitar",
    "publicitasen": "publicitar",
    "publicitases": "publicitar",
    "publicitaste": "publicitar",
    "publicitasteis": "publicitar",
    "publicite": "publicitar",
    "publicitemos": "publicitar",
    "publiciten": "publicitar",
    "publicites": "publicitar",
    "publicito": "publicitar",
    "publicitábamos": "publicitar",
    "publicitáis": "publicitar",
    "publicitáramos": "publicitar",
    "publicitáremos": "publicitar",
    "publicitásemos": "publicitar",
    "publicité": "publicitar",
    "publicitéis": "publicitar",
    "publicitó": "publicitar",
    "publico": "publicar",
    "publicábamos": "publicar",
    "publicáis": "publicar",
    "publicáramos": "publicar",
    "publicáremos": "publicar",
    "publicásemos": "publicar",
    "publicó": "publicar",
    "publienvíos": "publienvío",
    "publique": "publicar",
    "publiquemos": "publicar",
    "publiquen": "publicar",
    "publiques": "publicar",
    "publiqué": "publicar",
    "publiquéis": "publicar",
    "pubococcígea": "pubococcígeo",
    "pubococcígeas": "pubococcígeo",
    "pubococcígeos": "pubococcígeo",
    "pubofemorales": "pubofemoral",
    "puboprostática": "puboprostático",
    "puboprostáticas": "puboprostático",
    "puboprostáticos": "puboprostático",
    "puborrectales": "puborrectal",
    "pubovesicales": "pubovesical",
    "pubs": "pub",
    "pucelana": "pucelano",
    "pucelanas": "pucelano",
    "pucelanos": "pucelano",
    "puchadas": "puchada",
    "puchera": "puchero",
    "pucheras": "puchero",
    "pucherazos": "pucherazo",
    "pucheros": "puchero",
    "puchos": "pucho",
    "pude": "poder",
    "pudela": "pudelar",
    "pudelaba": "pudelar",
    "pudelabais": "pudelar",
    "pudelaban": "pudelar",
    "pudelabas": "pudelar",
    "pudelaciones": "pudelación",
    "pudelad": "pudelar",
    "pudelada": "pudelar",
    "pudeladas": "pudelar",
    "pudelado": "pudelar",
    "pudeladora": "pudelador",
    "pudeladoras": "pudelador",
    "pudeladores": "pudelador",
    "pudelados": "pudelar",
    "pudelamos": "pudelar",
    "pudelan": "pudelar",
    "pudelando": "pudelar",
    "pudelara": "pudelar",
    "pudelarais": "pudelar",
    "pudelaran": "pudelar",
    "pudelaras": "pudelar",
    "pudelare": "pudelar",
    "pudelareis": "pudelar",
    "pudelaremos": "pudelar",
    "pudelaren": "pudelar",
    "pudelares": "pudelar",
    "pudelaron": "pudelar",
    "pudelará": "pudelar",
    "pudelarán": "pudelar",
    "pudelarás": "pudelar",
    "pudelaré": "pudelar",
    "pudelaréis": "pudelar",
    "pudelaría": "pudelar",
    "pudelaríais": "pudelar",
    "pudelaríamos": "pudelar",
    "pudelarían": "pudelar",
    "pudelarías": "pudelar",
    "pudelas": "pudelar",
    "pudelase": "pudelar",
    "pudelaseis": "pudelar",
    "pudelasen": "pudelar",
    "pudelases": "pudelar",
    "pudelaste": "pudelar",
    "pudelasteis": "pudelar",
    "pudele": "pudelar",
    "pudelemos": "pudelar",
    "pudelen": "pudelar",
    "pudeles": "pudelar",
    "pudelo": "pudelar",
    "pudelábamos": "pudelar",
    "pudeláis": "pudelar",
    "pudeláramos": "pudelar",
    "pudeláremos": "pudelar",
    "pudelásemos": "pudelar",
    "pudelé": "pudelar",
    "pudeléis": "pudelar",
    "pudeló": "pudelar",
    "pudenda": "pudendo",
    "pudendas": "pudendo",
    "pudendos": "pudendo",
    "pudibunda": "pudibundo",
    "pudibundas": "pudibundo",
    "pudibundeces": "pudibundez",
    "pudibundos": "pudibundo",
    "pudicicias": "pudicicia",
    "pudiendo": "poder",
    "pudientes": "pudiente",
    "pudiera": "poder",
    "pudierais": "poder",
    "pudieran": "poder",
    "pudieras": "poder",
    "pudiere": "poder",
    "pudiereis": "poder",
    "pudieren": "poder",
    "pudieres": "poder",
    "pudieron": "poder",
    "pudiese": "poder",
    "pudieseis": "poder",
    "pudiesen": "poder",
    "pudieses": "poder",
    "pudimos": "poder",
    "pudines": "pudín",
    "pudingas": "pudinga",
    "pudings": "puding",
    "pudiste": "poder",
    "pudisteis": "poder",
    "pudiéramos": "poder",
    "pudiéremos": "poder",
    "pudiésemos": "poder",
    "pudo": "poder",
    "pudores": "pudor",
    "pudorosa": "pudoroso",
    "pudorosas": "pudoroso",
    "pudorosos": "pudoroso",
    "pudra": "pudrir",
    "pudramos": "pudrir",
    "pudran": "pudrir",
    "pudras": "pudrir",
    "pudre": "pudrir",
    "pudren": "pudrir",
    "pudres": "pudrir",
    "pudriciones": "pudrición",
    "pudriciónes": "pudrición",
    "pudrid": "pudrir",
    "pudrideros": "pudridero",
    "pudriendo": "pudrir",
    "pudriera": "pudrir",
    "pudrierais": "pudrir",
    "pudrieran": "pudrir",
    "pudrieras": "pudrir",
    "pudriere": "pudrir",
    "pudriereis": "pudrir",
    "pudrieren": "pudrir",
    "pudrieres": "pudrir",
    "pudrieron": "pudrir",
    "pudriese": "pudrir",
    "pudrieseis": "pudrir",
    "pudriesen": "pudrir",
    "pudrieses": "pudrir",
    "pudrimientos": "pudrimiento",
    "pudrimos": "pudrir",
    "pudrir": "podrir",
    "pudriremos": "pudrir",
    "pudrirá": "pudrir",
    "pudrirán": "pudrir",
    "pudrirás": "pudrir",
    "pudriré": "pudrir",
    "pudriréis": "pudrir",
    "pudriría": "pudrir",
    "pudriríais": "pudrir",
    "pudriríamos": "pudrir",
    "pudrirían": "pudrir",
    "pudrirías": "pudrir",
    "pudriste": "pudrir",
    "pudristeis": "pudrir",
    "pudriéramos": "pudrir",
    "pudriéremos": "pudrir",
    "pudriésemos": "pudrir",
    "pudrió": "pudrir",
    "pudro": "pudrir",
    "pudráis": "pudrir",
    "pudrí": "pudrir",
    "pudría": "pudrir",
    "pudríais": "pudrir",
    "pudríamos": "pudrir",
    "pudrían": "pudrir",
    "pudrías": "pudrir",
    "pudrís": "pudrir",
    "puebla": "poblar",
    "pueblan": "poblar",
    "pueblas": "poblar",
    "pueble": "poblar",
    "pueblecitos": "pueblecito",
    "pueblen": "poblar",
    "pueblerina": "pueblerino",
    "pueblerinas": "pueblerino",
    "pueblerinos": "pueblerino",
    "puebles": "pueble",
    "pueblo": "poblar",
    "pueblos": "pueblo",
    "puebluchos": "pueblucho",
    "pueda": "poder",
    "puedan": "poder",
    "puedas": "poder",
    "puede": "poder",
    "pueden": "poder",
    "puedes": "poder",
    "puedo": "poder",
    "puenteos": "puenteo",
    "puentes": "puente",
    "puentings": "puenting",
    "puerca": "puerco",
    "puercas": "puerco",
    "puercoespines": "puercoespín",
    "puercos": "puerco",
    "puericias": "puericia",
    "puericultora": "puericultor",
    "puericultoras": "puericultor",
    "puericultores": "puericultor",
    "puericulturas": "puericultura",
    "pueriles": "pueril",
    "puerilidades": "puerilidad",
    "puerperales": "puerperal",
    "puerperios": "puerperio",
    "puerros": "puerro",
    "puertas": "puerta",
    "puertaventanas": "puertaventana",
    "puertollanenses": "puertollanense",
    "puertorriqueña": "puertorriqueño",
    "puertorriqueñas": "puertorriqueño",
    "puertorriqueñismos": "puertorriqueñismo",
    "puertorriqueños": "puertorriqueño",
    "puertos": "puerto",
    "puesta": "poner",
    "puestas": "puesta",
    "puestera": "puestero",
    "puesteras": "puestero",
    "puesteros": "puestero",
    "puesto": "poner",
    "puestos": "puesto",
    "pufes": "puf",
    "pufos": "pufo",
    "pufs": "puf",
    "pugilatos": "pugilato",
    "pugilismos": "pugilismo",
    "pugilistas": "pugilista",
    "pugilística": "pugilístico",
    "pugilísticas": "pugilístico",
    "pugilísticos": "pugilístico",
    "pugna": "pugnar",
    "pugnaba": "pugnar",
    "pugnabais": "pugnar",
    "pugnaban": "pugnar",
    "pugnabas": "pugnar",
    "pugnaces": "pugnaz",
    "pugnacidades": "pugnacidad",
    "pugnad": "pugnar",
    "pugnada": "pugnar",
    "pugnadas": "pugnar",
    "pugnado": "pugnar",
    "pugnados": "pugnar",
    "pugnamos": "pugnar",
    "pugnan": "pugnar",
    "pugnando": "pugnar",
    "pugnantes": "pugnante",
    "pugnara": "pugnar",
    "pugnarais": "pugnar",
    "pugnaran": "pugnar",
    "pugnaras": "pugnar",
    "pugnare": "pugnar",
    "pugnareis": "pugnar",
    "pugnaremos": "pugnar",
    "pugnaren": "pugnar",
    "pugnares": "pugnar",
    "pugnaron": "pugnar",
    "pugnará": "pugnar",
    "pugnarán": "pugnar",
    "pugnarás": "pugnar",
    "pugnaré": "pugnar",
    "pugnaréis": "pugnar",
    "pugnaría": "pugnar",
    "pugnaríais": "pugnar",
    "pugnaríamos": "pugnar",
    "pugnarían": "pugnar",
    "pugnarías": "pugnar",
    "pugnas": "pugnar",
    "pugnase": "pugnar",
    "pugnaseis": "pugnar",
    "pugnasen": "pugnar",
    "pugnases": "pugnar",
    "pugnaste": "pugnar",
    "pugnasteis": "pugnar",
    "pugne": "pugnar",
    "pugnemos": "pugnar",
    "pugnen": "pugnar",
    "pugnes": "pugnar",
    "pugno": "pugnar",
    "pugnábamos": "pugnar",
    "pugnáis": "pugnar",
    "pugnáramos": "pugnar",
    "pugnáremos": "pugnar",
    "pugnásemos": "pugnar",
    "pugné": "pugnar",
    "pugnéis": "pugnar",
    "pugnó": "pugnar",
    "puja": "pujar",
    "pujaba": "pujar",
    "pujabais": "pujar",
    "pujaban": "pujar",
    "pujabas": "pujar",
    "pujad": "pujar",
    "pujada": "pujar",
    "pujadas": "pujar",
    "pujado": "pujar",
    "pujadora": "pujador",
    "pujadoras": "pujador",
    "pujadores": "pujador",
    "pujados": "pujar",
    "pujames": "pujame",
    "pujamientos": "pujamiento",
    "pujamos": "pujar",
    "pujan": "pujar",
    "pujando": "pujar",
    "pujantes": "pujante",
    "pujanzas": "pujanza",
    "pujara": "pujar",
    "pujarais": "pujar",
    "pujaran": "pujar",
    "pujaras": "pujar",
    "pujare": "pujar",
    "pujareis": "pujar",
    "pujaremos": "pujar",
    "pujaren": "pujar",
    "pujares": "pujar",
    "pujaron": "pujar",
    "pujará": "pujar",
    "pujarán": "pujar",
    "pujarás": "pujar",
    "pujaré": "pujar",
    "pujaréis": "pujar",
    "pujaría": "pujar",
    "pujaríais": "pujar",
    "pujaríamos": "pujar",
    "pujarían": "pujar",
    "pujarías": "pujar",
    "pujas": "pujar",
    "pujase": "pujar",
    "pujaseis": "pujar",
    "pujasen": "pujar",
    "pujases": "pujar",
    "pujaste": "pujar",
    "pujasteis": "pujar",
    "pujavantes": "pujavante",
    "puje": "pujar",
    "pujemos": "pujar",
    "pujen": "pujar",
    "pujes": "pujar",
    "pujo": "pujar",
    "pujolismos": "pujolismo",
    "pujolistas": "pujolista",
    "pujos": "pujo",
    "pujábamos": "pujar",
    "pujáis": "pujar",
    "pujámenes": "pujamen",
    "pujáramos": "pujar",
    "pujáremos": "pujar",
    "pujásemos": "pujar",
    "pujé": "pujar",
    "pujéis": "pujar",
    "pujó": "pujar",
    "pula": "pulir",
    "pulamos": "pulir",
    "pulan": "pulir",
    "pulardas": "pularda",
    "pulas": "pulir",
    "pulchinelas": "pulchinela",
    "pulcra": "pulcro",
    "pulcras": "pulcro",
    "pulcritudes": "pulcritud",
    "pulcros": "pulcro",
    "pule": "pulir",
    "pulen": "pulir",
    "pules": "pulir",
    "pulgadas": "pulgada",
    "pulgaradas": "pulgarada",
    "pulgares": "pulgar",
    "pulgas": "pulga",
    "pulgones": "pulgón",
    "pulgosa": "pulgoso",
    "pulgosas": "pulgoso",
    "pulgosos": "pulgoso",
    "pulgueras": "pulguera",
    "pulicanes": "pulicán",
    "pulid": "pulir",
    "pulida": "pulir",
    "pulidas": "pulir",
    "pulideces": "pulidez",
    "pulideros": "pulidero",
    "pulido": "pulir",
    "pulidora": "pulidor",
    "pulidoras": "pulidora",
    "pulidores": "pulidor",
    "pulidos": "pulir",
    "puliendo": "pulir",
    "puliera": "pulir",
    "pulierais": "pulir",
    "pulieran": "pulir",
    "pulieras": "pulir",
    "puliere": "pulir",
    "puliereis": "pulir",
    "pulieren": "pulir",
    "pulieres": "pulir",
    "pulieron": "pulir",
    "puliese": "pulir",
    "pulieseis": "pulir",
    "puliesen": "pulir",
    "pulieses": "pulir",
    "pulimenta": "pulimentar",
    "pulimentaba": "pulimentar",
    "pulimentabais": "pulimentar",
    "pulimentaban": "pulimentar",
    "pulimentabas": "pulimentar",
    "pulimentad": "pulimentar",
    "pulimentada": "pulimentar",
    "pulimentadas": "pulimentar",
    "pulimentado": "pulimentar",
    "pulimentados": "pulimentar",
    "pulimentamos": "pulimentar",
    "pulimentan": "pulimentar",
    "pulimentando": "pulimentar",
    "pulimentara": "pulimentar",
    "pulimentarais": "pulimentar",
    "pulimentaran": "pulimentar",
    "pulimentaras": "pulimentar",
    "pulimentare": "pulimentar",
    "pulimentareis": "pulimentar",
    "pulimentaremos": "pulimentar",
    "pulimentaren": "pulimentar",
    "pulimentares": "pulimentar",
    "pulimentaron": "pulimentar",
    "pulimentará": "pulimentar",
    "pulimentarán": "pulimentar",
    "pulimentarás": "pulimentar",
    "pulimentaré": "pulimentar",
    "pulimentaréis": "pulimentar",
    "pulimentaría": "pulimentar",
    "pulimentaríais": "pulimentar",
    "pulimentaríamos": "pulimentar",
    "pulimentarían": "pulimentar",
    "pulimentarías": "pulimentar",
    "pulimentas": "pulimentar",
    "pulimentase": "pulimentar",
    "pulimentaseis": "pulimentar",
    "pulimentasen": "pulimentar",
    "pulimentases": "pulimentar",
    "pulimentaste": "pulimentar",
    "pulimentasteis": "pulimentar",
    "pulimente": "pulimentar",
    "pulimentemos": "pulimentar",
    "pulimenten": "pulimentar",
    "pulimentes": "pulimentar",
    "pulimento": "pulimentar",
    "pulimentos": "pulimento",
    "pulimentábamos": "pulimentar",
    "pulimentáis": "pulimentar",
    "pulimentáramos": "pulimentar",
    "pulimentáremos": "pulimentar",
    "pulimentásemos": "pulimentar",
    "pulimenté": "pulimentar",
    "pulimentéis": "pulimentar",
    "pulimentó": "pulimentar",
    "pulimos": "pulir",
    "puliremos": "pulir",
    "pulirá": "pulir",
    "pulirán": "pulir",
    "pulirás": "pulir",
    "puliré": "pulir",
    "puliréis": "pulir",
    "puliría": "pulir",
    "puliríais": "pulir",
    "puliríamos": "pulir",
    "pulirían": "pulir",
    "pulirías": "pulir",
    "puliste": "pulir",
    "pulisteis": "pulir",
    "puliéramos": "pulir",
    "puliéremos": "pulir",
    "puliésemos": "pulir",
    "pulió": "pulir",
    "pullas": "pulla",
    "pullistas": "pullista",
    "pullmans": "pullman",
    "pullovers": "pullover",
    "pullóveres": "pullóver",
    "pulmonares": "pulmonar",
    "pulmonarias": "pulmonaria",
    "pulmones": "pulmón",
    "pulmoniaca": "pulmoníaco",
    "pulmoniacas": "pulmoníaco",
    "pulmoniacos": "pulmoníaco",
    "pulmoníaca": "pulmoníaco",
    "pulmoníacas": "pulmoníaco",
    "pulmoníacos": "pulmoníaco",
    "pulmonías": "pulmonía",
    "pulo": "pulir",
    "pulpalgias": "pulpalgia",
    "pulpares": "pulpar",
    "pulpas": "pulpa",
    "pulpectomías": "pulpectomía",
    "pulpejos": "pulpejo",
    "pulpetas": "pulpeta",
    "pulpitos": "pulpito",
    "pulpolitos": "pulpolito",
    "pulpos": "pulpo",
    "pulposa": "pulposo",
    "pulposas": "pulposo",
    "pulposos": "pulposo",
    "pulpotomías": "pulpotomía",
    "pulquerías": "pulquería",
    "pulques": "pulque",
    "pulquérrima": "pulquérrimo",
    "pulquérrimas": "pulquérrimo",
    "pulquérrimos": "pulquérrimo",
    "pulsa": "pulsar",
    "pulsaba": "pulsar",
    "pulsabais": "pulsar",
    "pulsaban": "pulsar",
    "pulsabas": "pulsar",
    "pulsaciones": "pulsación",
    "pulsad": "pulsar",
    "pulsada": "pulsar",
    "pulsadas": "pulsar",
    "pulsado": "pulsar",
    "pulsadora": "pulsador",
    "pulsadoras": "pulsador",
    "pulsadores": "pulsador",
    "pulsados": "pulsar",
    "pulsamos": "pulsar",
    "pulsan": "pulsar",
    "pulsando": "pulsar",
    "pulsantes": "pulsante",
    "pulsara": "pulsar",
    "pulsarais": "pulsar",
    "pulsaran": "pulsar",
    "pulsaras": "pulsar",
    "pulsare": "pulsar",
    "pulsareis": "pulsar",
    "pulsaremos": "pulsar",
    "pulsaren": "pulsar",
    "pulsares": "pulsar",
    "pulsaron": "pulsar",
    "pulsará": "pulsar",
    "pulsarán": "pulsar",
    "pulsarás": "pulsar",
    "pulsaré": "pulsar",
    "pulsaréis": "pulsar",
    "pulsaría": "pulsar",
    "pulsaríais": "pulsar",
    "pulsaríamos": "pulsar",
    "pulsarían": "pulsar",
    "pulsarías": "pulsar",
    "pulsas": "pulsar",
    "pulsase": "pulsar",
    "pulsaseis": "pulsar",
    "pulsasen": "pulsar",
    "pulsases": "pulsar",
    "pulsaste": "pulsar",
    "pulsasteis": "pulsar",
    "pulsativa": "pulsativo",
    "pulsativas": "pulsativo",
    "pulsativos": "pulsativo",
    "pulsatoria": "pulsatorio",
    "pulsatorias": "pulsatorio",
    "pulsatorios": "pulsatorio",
    "pulse": "pulsar",
    "pulsea": "pulsear",
    "pulseaba": "pulsear",
    "pulseabais": "pulsear",
    "pulseaban": "pulsear",
    "pulseabas": "pulsear",
    "pulsead": "pulsear",
    "pulseada": "pulsear",
    "pulseadas": "pulsear",
    "pulseado": "pulsear",
    "pulseados": "pulsear",
    "pulseamos": "pulsear",
    "pulsean": "pulsear",
    "pulseando": "pulsear",
    "pulseara": "pulsear",
    "pulsearais": "pulsear",
    "pulsearan": "pulsear",
    "pulsearas": "pulsear",
    "pulseare": "pulsear",
    "pulseareis": "pulsear",
    "pulsearemos": "pulsear",
    "pulsearen": "pulsear",
    "pulseares": "pulsear",
    "pulsearon": "pulsear",
    "pulseará": "pulsear",
    "pulsearán": "pulsear",
    "pulsearás": "pulsear",
    "pulsearé": "pulsear",
    "pulsearéis": "pulsear",
    "pulsearía": "pulsear",
    "pulsearíais": "pulsear",
    "pulsearíamos": "pulsear",
    "pulsearían": "pulsear",
    "pulsearías": "pulsear",
    "pulseas": "pulsear",
    "pulsease": "pulsear",
    "pulseaseis": "pulsear",
    "pulseasen": "pulsear",
    "pulseases": "pulsear",
    "pulseaste": "pulsear",
    "pulseasteis": "pulsear",
    "pulsee": "pulsear",
    "pulseemos": "pulsear",
    "pulseen": "pulsear",
    "pulsees": "pulsear",
    "pulsemos": "pulsar",
    "pulsen": "pulsar",
    "pulseo": "pulsear",
    "pulseras": "pulsera",
    "pulses": "pulsar",
    "pulseábamos": "pulsear",
    "pulseáis": "pulsear",
    "pulseáramos": "pulsear",
    "pulseáremos": "pulsear",
    "pulseásemos": "pulsear",
    "pulseé": "pulsear",
    "pulseéis": "pulsear",
    "pulseó": "pulsear",
    "pulsiones": "pulsión",
    "pulso": "pulsar",
    "pulsos": "pulso",
    "pulsábamos": "pulsar",
    "pulsáis": "pulsar",
    "pulsáramos": "pulsar",
    "pulsáremos": "pulsar",
    "pulsásemos": "pulsar",
    "pulsátiles": "pulsátil",
    "pulsé": "pulsar",
    "pulséis": "pulsar",
    "pulsímetros": "pulsímetro",
    "pulsó": "pulsar",
    "pulsómetros": "pulsómetro",
    "pultácea": "pultáceo",
    "pultáceas": "pultáceo",
    "pultáceos": "pultáceo",
    "pulula": "pulular",
    "pululaba": "pulular",
    "pululabais": "pulular",
    "pululaban": "pulular",
    "pululabas": "pulular",
    "pululad": "pulular",
    "pululada": "pulular",
    "pululadas": "pulular",
    "pululado": "pulular",
    "pululados": "pulular",
    "pululamos": "pulular",
    "pululan": "pulular",
    "pululando": "pulular",
    "pululara": "pulular",
    "pulularais": "pulular",
    "pulularan": "pulular",
    "pulularas": "pulular",
    "pululare": "pulular",
    "pululareis": "pulular",
    "pulularemos": "pulular",
    "pulularen": "pulular",
    "pululares": "pulular",
    "pulularon": "pulular",
    "pululará": "pulular",
    "pulularán": "pulular",
    "pulularás": "pulular",
    "pulularé": "pulular",
    "pulularéis": "pulular",
    "pulularía": "pulular",
    "pulularíais": "pulular",
    "pulularíamos": "pulular",
    "pulularían": "pulular",
    "pulularías": "pulular",
    "pululas": "pulular",
    "pululase": "pulular",
    "pululaseis": "pulular",
    "pululasen": "pulular",
    "pululases": "pulular",
    "pululaste": "pulular",
    "pululasteis": "pulular",
    "pulule": "pulular",
    "pululemos": "pulular",
    "pululen": "pulular",
    "pulules": "pulular",
    "pululo": "pulular",
    "pululábamos": "pulular",
    "pululáis": "pulular",
    "pululáramos": "pulular",
    "pululáremos": "pulular",
    "pululásemos": "pulular",
    "pululé": "pulular",
    "pululéis": "pulular",
    "pululó": "pulular",
    "pulverice": "pulverizar",
    "pulvericemos": "pulverizar",
    "pulvericen": "pulverizar",
    "pulverices": "pulverizar",
    "pulvericé": "pulverizar",
    "pulvericéis": "pulverizar",
    "pulveriza": "pulverizar",
    "pulverizaba": "pulverizar",
    "pulverizabais": "pulverizar",
    "pulverizaban": "pulverizar",
    "pulverizabas": "pulverizar",
    "pulverizables": "pulverizable",
    "pulverizaciones": "pulverización",
    "pulverizad": "pulverizar",
    "pulverizada": "pulverizar",
    "pulverizadas": "pulverizar",
    "pulverizado": "pulverizar",
    "pulverizadora": "pulverizador",
    "pulverizadoras": "pulverizador",
    "pulverizadores": "pulverizador",
    "pulverizados": "pulverizar",
    "pulverizamos": "pulverizar",
    "pulverizan": "pulverizar",
    "pulverizando": "pulverizar",
    "pulverizara": "pulverizar",
    "pulverizarais": "pulverizar",
    "pulverizaran": "pulverizar",
    "pulverizaras": "pulverizar",
    "pulverizare": "pulverizar",
    "pulverizareis": "pulverizar",
    "pulverizaremos": "pulverizar",
    "pulverizaren": "pulverizar",
    "pulverizares": "pulverizar",
    "pulverizaron": "pulverizar",
    "pulverizará": "pulverizar",
    "pulverizarán": "pulverizar",
    "pulverizarás": "pulverizar",
    "pulverizaré": "pulverizar",
    "pulverizaréis": "pulverizar",
    "pulverizaría": "pulverizar",
    "pulverizaríais": "pulverizar",
    "pulverizaríamos": "pulverizar",
    "pulverizarían": "pulverizar",
    "pulverizarías": "pulverizar",
    "pulverizas": "pulverizar",
    "pulverizase": "pulverizar",
    "pulverizaseis": "pulverizar",
    "pulverizasen": "pulverizar",
    "pulverizases": "pulverizar",
    "pulverizaste": "pulverizar",
    "pulverizasteis": "pulverizar",
    "pulverizo": "pulverizar",
    "pulverizábamos": "pulverizar",
    "pulverizáis": "pulverizar",
    "pulverizáramos": "pulverizar",
    "pulverizáremos": "pulverizar",
    "pulverizásemos": "pulverizar",
    "pulverizó": "pulverizar",
    "pulverulencias": "pulverulencia",
    "pulverulenta": "pulverulento",
    "pulverulentas": "pulverulento",
    "pulverulentos": "pulverulento",
    "pulvimetalurgias": "pulvimetalurgia",
    "puláis": "pulir",
    "pulí": "pulir",
    "pulía": "pulir",
    "pulíais": "pulir",
    "pulíamos": "pulir",
    "pulían": "pulir",
    "pulías": "pulir",
    "pulís": "pulir",
    "pumas": "puma",
    "pumitas": "pumita",
    "puna": "punir",
    "punamos": "punir",
    "punan": "punir",
    "punas": "punir",
    "punce": "punzar",
    "puncemos": "punzar",
    "puncen": "punzar",
    "punces": "punzar",
    "punchas": "puncha",
    "punching-balls": "punching-ball",
    "punchingballs": "punchingball",
    "punciona": "puncionar",
    "puncionaba": "puncionar",
    "puncionabais": "puncionar",
    "puncionaban": "puncionar",
    "puncionabas": "puncionar",
    "puncionad": "puncionar",
    "puncionada": "puncionar",
    "puncionadas": "puncionar",
    "puncionado": "puncionar",
    "puncionados": "puncionar",
    "puncionamos": "puncionar",
    "puncionan": "puncionar",
    "puncionando": "puncionar",
    "puncionara": "puncionar",
    "puncionarais": "puncionar",
    "puncionaran": "puncionar",
    "puncionaras": "puncionar",
    "puncionare": "puncionar",
    "puncionareis": "puncionar",
    "puncionaremos": "puncionar",
    "puncionaren": "puncionar",
    "puncionares": "puncionar",
    "puncionaron": "puncionar",
    "puncionará": "puncionar",
    "puncionarán": "puncionar",
    "puncionarás": "puncionar",
    "puncionaré": "puncionar",
    "puncionaréis": "puncionar",
    "puncionaría": "puncionar",
    "puncionaríais": "puncionar",
    "puncionaríamos": "puncionar",
    "puncionarían": "puncionar",
    "puncionarías": "puncionar",
    "puncionas": "puncionar",
    "puncionase": "puncionar",
    "puncionaseis": "puncionar",
    "puncionasen": "puncionar",
    "puncionases": "puncionar",
    "puncionaste": "puncionar",
    "puncionasteis": "puncionar",
    "puncione": "puncionar",
    "puncionemos": "puncionar",
    "puncionen": "puncionar",
    "punciones": "puncionar",
    "punciono": "puncionar",
    "puncionábamos": "puncionar",
    "puncionáis": "puncionar",
    "puncionáramos": "puncionar",
    "puncionáremos": "puncionar",
    "puncionásemos": "puncionar",
    "puncioné": "puncionar",
    "puncionéis": "puncionar",
    "puncionó": "puncionar",
    "puncé": "punzar",
    "puncéis": "punzar",
    "pundonores": "pundonor",
    "pundonorosa": "pundonoroso",
    "pundonorosas": "pundonoroso",
    "pundonorosos": "pundonoroso",
    "pune": "punir",
    "punen": "punir",
    "punes": "punir",
    "punge": "pungir",
    "pungen": "pungir",
    "punges": "pungir",
    "pungid": "pungir",
    "pungida": "pungir",
    "pungidas": "pungir",
    "pungido": "pungir",
    "pungidos": "pungir",
    "pungiendo": "pungir",
    "pungiera": "pungir",
    "pungierais": "pungir",
    "pungieran": "pungir",
    "pungieras": "pungir",
    "pungiere": "pungir",
    "pungiereis": "pungir",
    "pungieren": "pungir",
    "pungieres": "pungir",
    "pungieron": "pungir",
    "pungiese": "pungir",
    "pungieseis": "pungir",
    "pungiesen": "pungir",
    "pungieses": "pungir",
    "pungimientos": "pungimiento",
    "pungimos": "pungir",
    "pungiremos": "pungir",
    "pungirá": "pungir",
    "pungirán": "pungir",
    "pungirás": "pungir",
    "pungiré": "pungir",
    "pungiréis": "pungir",
    "pungiría": "pungir",
    "pungiríais": "pungir",
    "pungiríamos": "pungir",
    "pungirían": "pungir",
    "pungirías": "pungir",
    "pungiste": "pungir",
    "pungisteis": "pungir",
    "pungiéramos": "pungir",
    "pungiéremos": "pungir",
    "pungiésemos": "pungir",
    "pungió": "pungir",
    "pungí": "pungir",
    "pungía": "pungir",
    "pungíais": "pungir",
    "pungíamos": "pungir",
    "pungían": "pungir",
    "pungías": "pungir",
    "pungís": "pungir",
    "punibilidades": "punibilidad",
    "punibles": "punible",
    "puniciones": "punición",
    "punid": "punir",
    "punida": "punir",
    "punidas": "punir",
    "punido": "punir",
    "punidos": "punir",
    "puniendo": "punir",
    "puniera": "punir",
    "punierais": "punir",
    "punieran": "punir",
    "punieras": "punir",
    "puniere": "punir",
    "puniereis": "punir",
    "punieren": "punir",
    "punieres": "punir",
    "punieron": "punir",
    "puniese": "punir",
    "punieseis": "punir",
    "puniesen": "punir",
    "punieses": "punir",
    "punimos": "punir",
    "puniremos": "punir",
    "punirá": "punir",
    "punirán": "punir",
    "punirás": "punir",
    "puniré": "punir",
    "puniréis": "punir",
    "puniría": "punir",
    "puniríais": "punir",
    "puniríamos": "punir",
    "punirían": "punir",
    "punirías": "punir",
    "puniste": "punir",
    "punisteis": "punir",
    "punitiva": "punitivo",
    "punitivas": "punitivo",
    "punitivos": "punitivo",
    "puniéramos": "punir",
    "puniéremos": "punir",
    "puniésemos": "punir",
    "punió": "punir",
    "punja": "pungir",
    "punjamos": "pungir",
    "punjan": "pungir",
    "punjas": "pungir",
    "punjo": "pungir",
    "punjáis": "pungir",
    "punks": "punk",
    "puno": "punir",
    "puntadas": "puntada",
    "puntales": "puntal",
    "puntapiés": "puntapié",
    "puntas": "punta",
    "puntazos": "puntazo",
    "puntea": "puntear",
    "punteaba": "puntear",
    "punteabais": "puntear",
    "punteaban": "puntear",
    "punteabas": "puntear",
    "puntead": "puntear",
    "punteada": "puntear",
    "punteadas": "puntear",
    "punteado": "puntear",
    "punteadoras": "punteadora",
    "punteados": "puntear",
    "punteamos": "puntear",
    "puntean": "puntear",
    "punteando": "puntear",
    "punteara": "puntear",
    "puntearais": "puntear",
    "puntearan": "puntear",
    "puntearas": "puntear",
    "punteare": "puntear",
    "punteareis": "puntear",
    "puntearemos": "puntear",
    "puntearen": "puntear",
    "punteares": "puntear",
    "puntearon": "puntear",
    "punteará": "puntear",
    "puntearán": "puntear",
    "puntearás": "puntear",
    "puntearé": "puntear",
    "puntearéis": "puntear",
    "puntearía": "puntear",
    "puntearíais": "puntear",
    "puntearíamos": "puntear",
    "puntearían": "puntear",
    "puntearías": "puntear",
    "punteas": "puntear",
    "puntease": "puntear",
    "punteaseis": "puntear",
    "punteasen": "puntear",
    "punteases": "puntear",
    "punteaste": "puntear",
    "punteasteis": "puntear",
    "puntee": "puntear",
    "punteemos": "puntear",
    "punteen": "puntear",
    "puntees": "puntear",
    "punteles": "puntel",
    "punteo": "puntear",
    "punteos": "punteo",
    "puntera": "puntero",
    "punteras": "puntero",
    "punterazos": "punterazo",
    "punterolas": "punterola",
    "punteros": "puntero",
    "punterías": "puntería",
    "punteábamos": "puntear",
    "punteáis": "puntear",
    "punteáramos": "puntear",
    "punteáremos": "puntear",
    "punteásemos": "puntear",
    "punteé": "puntear",
    "punteéis": "puntear",
    "punteó": "puntear",
    "puntiaguda": "puntiagudo",
    "puntiagudas": "puntiagudo",
    "puntiagudos": "puntiagudo",
    "puntiformes": "puntiforme",
    "puntillada": "puntillado",
    "puntilladas": "puntillado",
    "puntillados": "puntillado",
    "puntillas": "puntilla",
    "puntillazos": "puntillazo",
    "puntillera": "puntillero",
    "puntilleras": "puntillero",
    "puntilleros": "puntillero",
    "puntillismos": "puntillismo",
    "puntillistas": "puntillista",
    "puntillones": "puntillón",
    "puntillos": "puntillo",
    "puntillosa": "puntilloso",
    "puntillosas": "puntilloso",
    "puntillosos": "puntilloso",
    "puntiseca": "puntiseco",
    "puntisecas": "puntiseco",
    "puntisecos": "puntiseco",
    "puntistas": "puntista",
    "puntitos": "puntito",
    "puntizones": "puntizón",
    "puntos": "punto",
    "puntosa": "puntoso",
    "puntosas": "puntoso",
    "puntosos": "puntoso",
    "puntuaba": "puntuar",
    "puntuabais": "puntuar",
    "puntuaban": "puntuar",
    "puntuabas": "puntuar",
    "puntuables": "puntuable",
    "puntuaciones": "puntuación",
    "puntuad": "puntuar",
    "puntuada": "puntuar",
    "puntuadas": "puntuar",
    "puntuado": "puntuar",
    "puntuados": "puntuar",
    "puntuales": "puntual",
    "puntualice": "puntualizar",
    "puntualicemos": "puntualizar",
    "puntualicen": "puntualizar",
    "puntualices": "puntualizar",
    "puntualicé": "puntualizar",
    "puntualicéis": "puntualizar",
    "puntualidades": "puntualidad",
    "puntualiza": "puntualizar",
    "puntualizaba": "puntualizar",
    "puntualizabais": "puntualizar",
    "puntualizaban": "puntualizar",
    "puntualizabas": "puntualizar",
    "puntualizaciones": "puntualización",
    "puntualizad": "puntualizar",
    "puntualizada": "puntualizar",
    "puntualizadas": "puntualizar",
    "puntualizado": "puntualizar",
    "puntualizados": "puntualizar",
    "puntualizamos": "puntualizar",
    "puntualizan": "puntualizar",
    "puntualizando": "puntualizar",
    "puntualizara": "puntualizar",
    "puntualizarais": "puntualizar",
    "puntualizaran": "puntualizar",
    "puntualizaras": "puntualizar",
    "puntualizare": "puntualizar",
    "puntualizareis": "puntualizar",
    "puntualizaremos": "puntualizar",
    "puntualizaren": "puntualizar",
    "puntualizares": "puntualizar",
    "puntualizaron": "puntualizar",
    "puntualizará": "puntualizar",
    "puntualizarán": "puntualizar",
    "puntualizarás": "puntualizar",
    "puntualizaré": "puntualizar",
    "puntualizaréis": "puntualizar",
    "puntualizaría": "puntualizar",
    "puntualizaríais": "puntualizar",
    "puntualizaríamos": "puntualizar",
    "puntualizarían": "puntualizar",
    "puntualizarías": "puntualizar",
    "puntualizas": "puntualizar",
    "puntualizase": "puntualizar",
    "puntualizaseis": "puntualizar",
    "puntualizasen": "puntualizar",
    "puntualizases": "puntualizar",
    "puntualizaste": "puntualizar",
    "puntualizasteis": "puntualizar",
    "puntualizo": "puntualizar",
    "puntualizábamos": "puntualizar",
    "puntualizáis": "puntualizar",
    "puntualizáramos": "puntualizar",
    "puntualizáremos": "puntualizar",
    "puntualizásemos": "puntualizar",
    "puntualizó": "puntualizar",
    "puntuamos": "puntuar",
    "puntuando": "puntuar",
    "puntuara": "puntuar",
    "puntuarais": "puntuar",
    "puntuaran": "puntuar",
    "puntuaras": "puntuar",
    "puntuare": "puntuar",
    "puntuareis": "puntuar",
    "puntuaremos": "puntuar",
    "puntuaren": "puntuar",
    "puntuares": "puntuar",
    "puntuaron": "puntuar",
    "puntuará": "puntuar",
    "puntuarán": "puntuar",
    "puntuarás": "puntuar",
    "puntuaré": "puntuar",
    "puntuaréis": "puntuar",
    "puntuaría": "puntuar",
    "puntuaríais": "puntuar",
    "puntuaríamos": "puntuar",
    "puntuarían": "puntuar",
    "puntuarías": "puntuar",
    "puntuase": "puntuar",
    "puntuaseis": "puntuar",
    "puntuasen": "puntuar",
    "puntuases": "puntuar",
    "puntuaste": "puntuar",
    "puntuasteis": "puntuar",
    "puntuemos": "puntuar",
    "puntuosa": "puntuoso",
    "puntuosas": "puntuoso",
    "puntuosos": "puntuoso",
    "punturas": "puntura",
    "puntuábamos": "puntuar",
    "puntuáis": "puntuar",
    "puntuáramos": "puntuar",
    "puntuáremos": "puntuar",
    "puntuásemos": "puntuar",
    "puntué": "puntuar",
    "puntuéis": "puntuar",
    "puntuó": "puntuar",
    "puntúa": "puntuar",
    "puntúan": "puntuar",
    "puntúas": "puntuar",
    "puntúe": "puntuar",
    "puntúen": "puntuar",
    "puntúes": "puntuar",
    "puntúo": "puntuar",
    "punza": "punzar",
    "punzaba": "punzar",
    "punzabais": "punzar",
    "punzaban": "punzar",
    "punzabas": "punzar",
    "punzad": "punzar",
    "punzada": "punzar",
    "punzadas": "punzar",
    "punzado": "punzar",
    "punzados": "punzar",
    "punzaduras": "punzadura",
    "punzamos": "punzar",
    "punzan": "punzar",
    "punzando": "punzar",
    "punzantes": "punzante",
    "punzara": "punzar",
    "punzarais": "punzar",
    "punzaran": "punzar",
    "punzaras": "punzar",
    "punzare": "punzar",
    "punzareis": "punzar",
    "punzaremos": "punzar",
    "punzaren": "punzar",
    "punzares": "punzar",
    "punzaron": "punzar",
    "punzará": "punzar",
    "punzarán": "punzar",
    "punzarás": "punzar",
    "punzaré": "punzar",
    "punzaréis": "punzar",
    "punzaría": "punzar",
    "punzaríais": "punzar",
    "punzaríamos": "punzar",
    "punzarían": "punzar",
    "punzarías": "punzar",
    "punzas": "punzar",
    "punzase": "punzar",
    "punzaseis": "punzar",
    "punzasen": "punzar",
    "punzases": "punzar",
    "punzaste": "punzar",
    "punzasteis": "punzar",
    "punzo": "punzar",
    "punzona": "punzonar",
    "punzonaba": "punzonar",
    "punzonabais": "punzonar",
    "punzonaban": "punzonar",
    "punzonabas": "punzonar",
    "punzonad": "punzonar",
    "punzonada": "punzonar",
    "punzonadas": "punzonar",
    "punzonado": "punzonar",
    "punzonadoras": "punzonadora",
    "punzonados": "punzonar",
    "punzonamos": "punzonar",
    "punzonan": "punzonar",
    "punzonando": "punzonar",
    "punzonara": "punzonar",
    "punzonarais": "punzonar",
    "punzonaran": "punzonar",
    "punzonaras": "punzonar",
    "punzonare": "punzonar",
    "punzonareis": "punzonar",
    "punzonaremos": "punzonar",
    "punzonaren": "punzonar",
    "punzonares": "punzonar",
    "punzonaron": "punzonar",
    "punzonará": "punzonar",
    "punzonarán": "punzonar",
    "punzonarás": "punzonar",
    "punzonaré": "punzonar",
    "punzonaréis": "punzonar",
    "punzonaría": "punzonar",
    "punzonaríais": "punzonar",
    "punzonaríamos": "punzonar",
    "punzonarían": "punzonar",
    "punzonarías": "punzonar",
    "punzonas": "punzonar",
    "punzonase": "punzonar",
    "punzonaseis": "punzonar",
    "punzonasen": "punzonar",
    "punzonases": "punzonar",
    "punzonaste": "punzonar",
    "punzonasteis": "punzonar",
    "punzone": "punzonar",
    "punzonemos": "punzonar",
    "punzonen": "punzonar",
    "punzones": "punzonar",
    "punzono": "punzonar",
    "punzonábamos": "punzonar",
    "punzonáis": "punzonar",
    "punzonáramos": "punzonar",
    "punzonáremos": "punzonar",
    "punzonásemos": "punzonar",
    "punzoné": "punzonar",
    "punzonéis": "punzonar",
    "punzonó": "punzonar",
    "punzábamos": "punzar",
    "punzáis": "punzar",
    "punzáramos": "punzar",
    "punzáremos": "punzar",
    "punzásemos": "punzar",
    "punzó": "punzar",
    "punzós": "punzó",
    "punáis": "punir",
    "puní": "punir",
    "punía": "punir",
    "puníais": "punir",
    "puníamos": "punir",
    "punían": "punir",
    "punías": "punir",
    "punís": "punir",
    "pupas": "pupa",
    "pupila": "pupilo",
    "pupilajes": "pupilaje",
    "pupilares": "pupilar",
    "pupilas": "pupilo",
    "pupilatonías": "pupilatonía",
    "pupilera": "pupilero",
    "pupileras": "pupilero",
    "pupileros": "pupilero",
    "pupilometrías": "pupilometría",
    "pupilos": "pupilo",
    "pupiloscopios": "pupiloscopio",
    "pupilostatómetros": "pupilostatómetro",
    "pupilógrafos": "pupilógrafo",
    "pupilómetros": "pupilómetro",
    "pupinizaciones": "pupinización",
    "pupitres": "pupitre",
    "puposa": "puposo",
    "puposas": "puposo",
    "puposos": "puposo",
    "pupurris": "pupurri",
    "puquíos": "puquío",
    "pura": "puro",
    "puranas": "purana",
    "puras": "puro",
    "pureras": "purera",
    "purezas": "pureza",
    "purga": "purgar",
    "purgaba": "purgar",
    "purgabais": "purgar",
    "purgaban": "purgar",
    "purgabas": "purgar",
    "purgaciones": "purgación",
    "purgad": "purgar",
    "purgada": "purgar",
    "purgadas": "purgar",
    "purgado": "purgar",
    "purgadora": "purgador",
    "purgadoras": "purgador",
    "purgadores": "purgador",
    "purgados": "purgar",
    "purgamientos": "purgamiento",
    "purgamos": "purgar",
    "purgan": "purgar",
    "purgando": "purgar",
    "purgantes": "purgante",
    "purgara": "purgar",
    "purgarais": "purgar",
    "purgaran": "purgar",
    "purgaras": "purgar",
    "purgare": "purgar",
    "purgareis": "purgar",
    "purgaremos": "purgar",
    "purgaren": "purgar",
    "purgares": "purgar",
    "purgaron": "purgar",
    "purgará": "purgar",
    "purgarán": "purgar",
    "purgarás": "purgar",
    "purgaré": "purgar",
    "purgaréis": "purgar",
    "purgaría": "purgar",
    "purgaríais": "purgar",
    "purgaríamos": "purgar",
    "purgarían": "purgar",
    "purgarías": "purgar",
    "purgas": "purgar",
    "purgase": "purgar",
    "purgaseis": "purgar",
    "purgasen": "purgar",
    "purgases": "purgar",
    "purgaste": "purgar",
    "purgasteis": "purgar",
    "purgativa": "purgativo",
    "purgativas": "purgativo",
    "purgativos": "purgativo",
    "purgatorios": "purgatorio",
    "purgo": "purgar",
    "purgue": "purgar",
    "purguemos": "purgar",
    "purguen": "purgar",
    "purgues": "purgar",
    "purgué": "purgar",
    "purguéis": "purgar",
    "purgábamos": "purgar",
    "purgáis": "purgar",
    "purgáramos": "purgar",
    "purgáremos": "purgar",
    "purgásemos": "purgar",
    "purgó": "purgar",
    "puridades": "puridad",
    "purifica": "purificar",
    "purificaba": "purificar",
    "purificabais": "purificar",
    "purificaban": "purificar",
    "purificabas": "purificar",
    "purificaciones": "purificación",
    "purificad": "purificar",
    "purificada": "purificar",
    "purificadas": "purificar",
    "purificado": "purificar",
    "purificadora": "purificador",
    "purificadoras": "purificador",
    "purificadores": "purificador",
    "purificados": "purificar",
    "purificamos": "purificar",
    "purifican": "purificar",
    "purificando": "purificar",
    "purificara": "purificar",
    "purificarais": "purificar",
    "purificaran": "purificar",
    "purificaras": "purificar",
    "purificare": "purificar",
    "purificareis": "purificar",
    "purificaremos": "purificar",
    "purificaren": "purificar",
    "purificares": "purificar",
    "purificaron": "purificar",
    "purificará": "purificar",
    "purificarán": "purificar",
    "purificarás": "purificar",
    "purificaré": "purificar",
    "purificaréis": "purificar",
    "purificaría": "purificar",
    "purificaríais": "purificar",
    "purificaríamos": "purificar",
    "purificarían": "purificar",
    "purificarías": "purificar",
    "purificas": "purificar",
    "purificase": "purificar",
    "purificaseis": "purificar",
    "purificasen": "purificar",
    "purificases": "purificar",
    "purificaste": "purificar",
    "purificasteis": "purificar",
    "purificatoria": "purificatorio",
    "purificatorias": "purificatorio",
    "purificatorios": "purificatorio",
    "purifico": "purificar",
    "purificábamos": "purificar",
    "purificáis": "purificar",
    "purificáramos": "purificar",
    "purificáremos": "purificar",
    "purificásemos": "purificar",
    "purificó": "purificar",
    "purifique": "purificar",
    "purifiquemos": "purificar",
    "purifiquen": "purificar",
    "purifiques": "purificar",
    "purifiqué": "purificar",
    "purifiquéis": "purificar",
    "purinas": "purina",
    "purines": "purín",
    "purismos": "purismo",
    "puristas": "purista",
    "puritana": "puritano",
    "puritanas": "puritano",
    "puritanismos": "puritanismo",
    "puritanos": "puritano",
    "puritos": "purito",
    "puromicinas": "puromicina",
    "puros": "puro",
    "purpura": "purpurar",
    "purpuraba": "purpurar",
    "purpurabais": "purpurar",
    "purpuraban": "purpurar",
    "purpurabas": "purpurar",
    "purpurad": "purpurar",
    "purpurada": "purpurar",
    "purpuradas": "purpurar",
    "purpurado": "purpurar",
    "purpurados": "purpurar",
    "purpuramos": "purpurar",
    "purpuran": "purpurar",
    "purpurando": "purpurar",
    "purpurara": "purpurar",
    "purpurarais": "purpurar",
    "purpuraran": "purpurar",
    "purpuraras": "purpurar",
    "purpurare": "purpurar",
    "purpurareis": "purpurar",
    "purpuraremos": "purpurar",
    "purpuraren": "purpurar",
    "purpurares": "purpurar",
    "purpuraron": "purpurar",
    "purpurará": "purpurar",
    "purpurarán": "purpurar",
    "purpurarás": "purpurar",
    "purpuraré": "purpurar",
    "purpuraréis": "purpurar",
    "purpuraría": "purpurar",
    "purpuraríais": "purpurar",
    "purpuraríamos": "purpurar",
    "purpurarían": "purpurar",
    "purpurarías": "purpurar",
    "purpuras": "purpurar",
    "purpurase": "purpurar",
    "purpuraseis": "purpurar",
    "purpurasen": "purpurar",
    "purpurases": "purpurar",
    "purpuraste": "purpurar",
    "purpurasteis": "purpurar",
    "purpure": "purpurar",
    "purpurea": "purpurear",
    "purpureaba": "purpurear",
    "purpureabais": "purpurear",
    "purpureaban": "purpurear",
    "purpureabas": "purpurear",
    "purpuread": "purpurear",
    "purpureada": "purpurear",
    "purpureadas": "purpurear",
    "purpureado": "purpurear",
    "purpureados": "purpurear",
    "purpureamos": "purpurear",
    "purpurean": "purpurear",
    "purpureando": "purpurear",
    "purpureara": "purpurear",
    "purpurearais": "purpurear",
    "purpurearan": "purpurear",
    "purpurearas": "purpurear",
    "purpureare": "purpurear",
    "purpureareis": "purpurear",
    "purpurearemos": "purpurear",
    "purpurearen": "purpurear",
    "purpureares": "purpurear",
    "purpurearon": "purpurear",
    "purpureará": "purpurear",
    "purpurearán": "purpurear",
    "purpurearás": "purpurear",
    "purpurearé": "purpurear",
    "purpurearéis": "purpurear",
    "purpurearía": "purpurear",
    "purpurearíais": "purpurear",
    "purpurearíamos": "purpurear",
    "purpurearían": "purpurear",
    "purpurearías": "purpurear",
    "purpureas": "purpurear",
    "purpurease": "purpurear",
    "purpureaseis": "purpurear",
    "purpureasen": "purpurear",
    "purpureases": "purpurear",
    "purpureaste": "purpurear",
    "purpureasteis": "purpurear",
    "purpuree": "purpurear",
    "purpureemos": "purpurear",
    "purpureen": "purpurear",
    "purpurees": "purpurear",
    "purpuremos": "purpurar",
    "purpuren": "purpurar",
    "purpureo": "purpurear",
    "purpures": "purpurar",
    "purpureábamos": "purpurear",
    "purpureáis": "purpurear",
    "purpureáramos": "purpurear",
    "purpureáremos": "purpurear",
    "purpureásemos": "purpurear",
    "purpureé": "purpurear",
    "purpureéis": "purpurear",
    "purpureó": "purpurear",
    "purpurina": "purpurino",
    "purpurinas": "purpurino",
    "purpurinos": "purpurino",
    "purpuro": "purpurar",
    "purpurábamos": "purpurar",
    "purpuráis": "purpurar",
    "purpuráramos": "purpurar",
    "purpuráremos": "purpurar",
    "purpurásemos": "purpurar",
    "purpuré": "purpurar",
    "purpuréis": "purpurar",
    "purpuró": "purpurar",
    "purpúrea": "purpúreo",
    "purpúreas": "purpúreo",
    "purpúreos": "purpúreo",
    "purrelas": "purrela",
    "purrias": "purria",
    "purrielas": "purriela",
    "purulencias": "purulencia",
    "purulenta": "purulento",
    "purulentas": "purulento",
    "purulentos": "purulento",
    "purés": "puré",
    "puse": "poner",
    "puses": "pus",
    "pushes": "push",
    "pusiera": "poner",
    "pusierais": "poner",
    "pusieran": "poner",
    "pusieras": "poner",
    "pusiere": "poner",
    "pusiereis": "poner",
    "pusieren": "poner",
    "pusieres": "poner",
    "pusieron": "poner",
    "pusiese": "poner",
    "pusieseis": "poner",
    "pusiesen": "poner",
    "pusieses": "poner",
    "pusilanimidades": "pusilanimidad",
    "pusilánimes": "pusilánime",
    "pusimos": "poner",
    "pusinesca": "pusinesco",
    "pusinescas": "pusinesco",
    "pusinescos": "pusinesco",
    "pusiste": "poner",
    "pusisteis": "poner",
    "pusiéramos": "poner",
    "pusiéremos": "poner",
    "pusiésemos": "poner",
    "puso": "poner",
    "pustulosa": "pustuloso",
    "pustulosas": "pustuloso",
    "pustulosos": "pustuloso",
    "puta": "puto",
    "putanismos": "putanismo",
    "putas": "puto",
    "putativa": "putativo",
    "putativas": "putativo",
    "putativos": "putativo",
    "putaísmos": "putaísmo",
    "putañea": "putañear",
    "putañeaba": "putañear",
    "putañeabais": "putañear",
    "putañeaban": "putañear",
    "putañeabas": "putañear",
    "putañead": "putañear",
    "putañeada": "putañear",
    "putañeadas": "putañear",
    "putañeado": "putañear",
    "putañeados": "putañear",
    "putañeamos": "putañear",
    "putañean": "putañear",
    "putañeando": "putañear",
    "putañeara": "putañear",
    "putañearais": "putañear",
    "putañearan": "putañear",
    "putañearas": "putañear",
    "putañeare": "putañear",
    "putañeareis": "putañear",
    "putañearemos": "putañear",
    "putañearen": "putañear",
    "putañeares": "putañear",
    "putañearon": "putañear",
    "putañeará": "putañear",
    "putañearán": "putañear",
    "putañearás": "putañear",
    "putañearé": "putañear",
    "putañearéis": "putañear",
    "putañearía": "putañear",
    "putañearíais": "putañear",
    "putañearíamos": "putañear",
    "putañearían": "putañear",
    "putañearías": "putañear",
    "putañeas": "putañear",
    "putañease": "putañear",
    "putañeaseis": "putañear",
    "putañeasen": "putañear",
    "putañeases": "putañear",
    "putañeaste": "putañear",
    "putañeasteis": "putañear",
    "putañee": "putañear",
    "putañeemos": "putañear",
    "putañeen": "putañear",
    "putañees": "putañear",
    "putañeo": "putañear",
    "putañeábamos": "putañear",
    "putañeáis": "putañear",
    "putañeáramos": "putañear",
    "putañeáremos": "putañear",
    "putañeásemos": "putañear",
    "putañeé": "putañear",
    "putañeéis": "putañear",
    "putañeó": "putañear",
    "putea": "putear",
    "puteaba": "putear",
    "puteabais": "putear",
    "puteaban": "putear",
    "puteabas": "putear",
    "putead": "putear",
    "puteada": "putear",
    "puteadas": "putear",
    "puteado": "putear",
    "puteados": "putear",
    "puteamos": "putear",
    "putean": "putear",
    "puteando": "putear",
    "puteara": "putear",
    "putearais": "putear",
    "putearan": "putear",
    "putearas": "putear",
    "puteare": "putear",
    "puteareis": "putear",
    "putearemos": "putear",
    "putearen": "putear",
    "puteares": "putear",
    "putearon": "putear",
    "puteará": "putear",
    "putearán": "putear",
    "putearás": "putear",
    "putearé": "putear",
    "putearéis": "putear",
    "putearía": "putear",
    "putearíais": "putear",
    "putearíamos": "putear",
    "putearían": "putear",
    "putearías": "putear",
    "puteas": "putear",
    "putease": "putear",
    "puteaseis": "putear",
    "puteasen": "putear",
    "puteases": "putear",
    "puteaste": "putear",
    "puteasteis": "putear",
    "putee": "putear",
    "puteemos": "putear",
    "puteen": "putear",
    "putees": "putear",
    "puteo": "putear",
    "putera": "putero",
    "puteras": "putero",
    "puteros": "putero",
    "puterías": "putería",
    "putesca": "putesco",
    "putescas": "putesco",
    "putescos": "putesco",
    "puteábamos": "putear",
    "puteáis": "putear",
    "puteáramos": "putear",
    "puteáremos": "putear",
    "puteásemos": "putear",
    "puteé": "putear",
    "puteéis": "putear",
    "puteó": "putear",
    "puticlubes": "puticlub",
    "puticlubs": "puticlub",
    "putos": "puto",
    "putrefacciones": "putrefacción",
    "putrefacta": "putrefacto",
    "putrefactas": "putrefacto",
    "putrefactiva": "putrefactivo",
    "putrefactivas": "putrefactivo",
    "putrefactivos": "putrefactivo",
    "putrefactos": "putrefacto",
    "putrescencias": "putrescencia",
    "putrescentes": "putrescente",
    "putrescibles": "putrescible",
    "putrideces": "putridez",
    "putts": "putt",
    "putámenes": "putamen",
    "puyas": "puya",
    "puyazos": "puyazo",
    "puzolanas": "puzolana",
    "puzoles": "puzol",
    "puzolánica": "puzolánico",
    "puzolánicas": "puzolánico",
    "puzolánicos": "puzolánico",
    "puzzles": "puzzle",
    "puérperas": "puérpera",
    "puñadas": "puñada",
    "puñados": "puñado",
    "puñaladas": "puñalada",
    "puñalera": "puñalero",
    "puñaleras": "puñalero",
    "puñaleros": "puñalero",
    "puñales": "puñal",
    "puñeras": "puñera",
    "puñetas": "puñeta",
    "puñetazos": "puñetazo",
    "puñetera": "puñetero",
    "puñeteras": "puñetero",
    "puñeteros": "puñetero",
    "puñetes": "puñete",
    "puños": "puño",
    "pymes": "pyme",
    "pábilos": "pábilo",
    "pábulos": "pábulo",
    "pádica": "pádico",
    "pádicas": "pádico",
    "pádicos": "pádico",
    "pág": "página",
    "págalos": "págalo",
    "páginas": "página",
    "págs": "página",
    "pájaras": "pájara",
    "pájaros": "pájaro",
    "pálida": "pálido",
    "pálidas": "pálido",
    "pálidos": "pálido",
    "pállidums": "pállidum",
    "pálmeres": "pálmer",
    "pálmers": "pálmer",
    "pálpebras": "pálpebra",
    "pálpitos": "pálpito",
    "pámpanas": "pámpana",
    "pámpanos": "pámpano",
    "pánaces": "pánace",
    "pánfila": "pánfilo",
    "pánfilas": "pánfilo",
    "pánfilos": "pánfilo",
    "pánica": "pánico",
    "pánicas": "pánico",
    "pánicos": "pánico",
    "pápara": "páparo",
    "páparas": "páparo",
    "páparos": "páparo",
    "pápiros": "pápiro",
    "pápulas": "pápula",
    "páquica": "páquico",
    "páquicas": "páquico",
    "páquicos": "páquico",
    "páramos": "páramo",
    "páridos": "párido",
    "párolis": "pároli",
    "párpados": "párpado",
    "párquings": "párquing",
    "párrafos": "párrafo",
    "párrocos": "párroco",
    "párvula": "párvulo",
    "párvulas": "párvulo",
    "párvulos": "párvulo",
    "páteras": "pátera",
    "pátinas": "pátina",
    "pávida": "pávido",
    "pávidas": "pávido",
    "pávidos": "pávido",
    "pécaries": "pécari",
    "pécaris": "pécari",
    "pécoras": "pécora",
    "pélvica": "pélvico",
    "pélvicas": "pélvico",
    "pélvicos": "pélvico",
    "péndolas": "péndola",
    "péndula": "péndulo",
    "péndulas": "péndulo",
    "péndulos": "péndulo",
    "pénfigos": "pénfigo",
    "pénsiles": "pénsil",
    "pépsica": "pépsico",
    "pépsicas": "pépsico",
    "pépsicos": "pépsico",
    "péptica": "péptico",
    "pépticas": "péptico",
    "pépticos": "péptico",
    "péptidos": "péptido",
    "pérdidas": "pérdida",
    "pérfida": "pérfido",
    "pérfidas": "pérfido",
    "pérfidos": "pérfido",
    "pérgolas": "pérgola",
    "pérmica": "pérmico",
    "pérmicas": "pérmico",
    "pérmicos": "pérmico",
    "pérsica": "pérsico",
    "pérsicas": "pérsico",
    "pérsicos": "pérsico",
    "pértigas": "pértiga",
    "pértigos": "pértigo",
    "pésames": "pésame",
    "pésima": "malo",
    "pésimas": "malo",
    "pésimo": "malo",
    "pésimos": "malo",
    "pétalos": "pétalo",
    "pétrea": "pétreo",
    "pétreas": "pétreo",
    "pétreos": "pétreo",
    "péñolas": "péñola",
    "pía": "pío",
    "pían": "piar",
    "pías": "pío",
    "pícara": "pícaro",
    "pícaras": "pícaro",
    "pícaros": "pícaro",
    "pícea": "píceo",
    "píceas": "píceo",
    "píceos": "píceo",
    "pícnica": "pícnico",
    "pícnicas": "pícnico",
    "pícnicos": "pícnico",
    "pícnics": "pícnic",
    "pícniques": "picnic",
    "pícrica": "pícrico",
    "pícricas": "pícrico",
    "pícricos": "pícrico",
    "pídolas": "pídola",
    "píe": "piar",
    "píen": "piar",
    "píes": "piar",
    "pífanos": "pífano",
    "píldoras": "píldora",
    "píleos": "píleo",
    "píloros": "píloro",
    "pínulas": "pínula",
    "pío": "piar",
    "píos": "pío",
    "pírica": "pírico",
    "píricas": "pírico",
    "píricos": "pírico",
    "pírrica": "pírrico",
    "pírricas": "pírrico",
    "pírricos": "pírrico",
    "pítica": "pítico",
    "píticas": "pítico",
    "píticos": "pítico",
    "pítimas": "pítima",
    "pívots": "pívot",
    "píxeles": "píxel",
    "píxidas": "píxida",
    "píxides": "píxide",
    "pócimas": "pócima",
    "pódiums": "pódium",
    "pókers": "póker",
    "pólderes": "pólder",
    "pólenes": "polen",
    "póleos": "póleo",
    "pólipos": "pólipo",
    "pólizas": "póliza",
    "pólvoras": "pólvora",
    "pómeces": "pómez",
    "pómulos": "pómulo",
    "póngidos": "póngido",
    "póntica": "póntico",
    "pónticas": "póntico",
    "pónticos": "póntico",
    "póqueres": "póquer",
    "pórfidos": "pórfido",
    "pórtica": "pórtico",
    "pórticas": "pórtico",
    "pórticos": "pórtico",
    "pósitos": "pósito",
    "pósteres": "póster",
    "pósterses": "póster",
    "póstulas": "póstula",
    "póstuma": "póstumo",
    "póstumas": "póstumo",
    "póstumos": "póstumo",
    "púas": "púa",
    "púberes": "púber",
    "púbica": "púbico",
    "púbicas": "púbico",
    "púbicos": "púbico",
    "pública": "público",
    "públicas": "público",
    "públicos": "público",
    "púdica": "púdico",
    "púdicas": "púdico",
    "púdicos": "púdico",
    "púgiles": "púgil",
    "púlpitos": "púlpito",
    "púlsares": "púlsar",
    "púnica": "púnico",
    "púnicas": "púnico",
    "púnicos": "púnico",
    "púrpuras": "púrpura",
    "pústulas": "pústula",
    "pútrida": "pútrido",
    "pútridas": "pútrido",
    "pútridos": "pútrido",
    "qasbas": "qasba",
    "qasidas": "qasida",
    "qataríes": "qatarí",
    "qatarís": "qatarí",
    "quadrimestrales": "quadrimestral",
    "quadroxalatos": "quadroxalato",
    "quaestios": "quaestio",
    "quanta": "quántum",
    "quantas": "quantum",
    "quantos": "quanto",
    "quarks": "quark",
    "quasars": "quásar",
    "quattrocentos": "quattrocento",
    "quebequesa": "quebequés",
    "quebequesas": "quebequés",
    "quebequeses": "quebequés",
    "quebraba": "quebrar",
    "quebrabais": "quebrar",
    "quebraban": "quebrar",
    "quebrabas": "quebrar",
    "quebrachitoles": "quebrachitol",
    "quebrachos": "quebracho",
    "quebrad": "quebrar",
    "quebrada": "quebrar",
    "quebradas": "quebrar",
    "quebraderos": "quebradero",
    "quebradiza": "quebradizo",
    "quebradizas": "quebradizo",
    "quebradizos": "quebradizo",
    "quebrado": "quebrar",
    "quebradora": "quebrador",
    "quebradoras": "quebrador",
    "quebradores": "quebrador",
    "quebrados": "quebrar",
    "quebraduras": "quebradura",
    "quebraja": "quebrajar",
    "quebrajaba": "quebrajar",
    "quebrajabais": "quebrajar",
    "quebrajaban": "quebrajar",
    "quebrajabas": "quebrajar",
    "quebrajad": "quebrajar",
    "quebrajada": "quebrajar",
    "quebrajadas": "quebrajar",
    "quebrajado": "quebrajar",
    "quebrajados": "quebrajar",
    "quebrajamos": "quebrajar",
    "quebrajan": "quebrajar",
    "quebrajando": "quebrajar",
    "quebrajara": "quebrajar",
    "quebrajarais": "quebrajar",
    "quebrajaran": "quebrajar",
    "quebrajaras": "quebrajar",
    "quebrajare": "quebrajar",
    "quebrajareis": "quebrajar",
    "quebrajaremos": "quebrajar",
    "quebrajaren": "quebrajar",
    "quebrajares": "quebrajar",
    "quebrajaron": "quebrajar",
    "quebrajará": "quebrajar",
    "quebrajarán": "quebrajar",
    "quebrajarás": "quebrajar",
    "quebrajaré": "quebrajar",
    "quebrajaréis": "quebrajar",
    "quebrajaría": "quebrajar",
    "quebrajaríais": "quebrajar",
    "quebrajaríamos": "quebrajar",
    "quebrajarían": "quebrajar",
    "quebrajarías": "quebrajar",
    "quebrajas": "quebrajar",
    "quebrajase": "quebrajar",
    "quebrajaseis": "quebrajar",
    "quebrajasen": "quebrajar",
    "quebrajases": "quebrajar",
    "quebrajaste": "quebrajar",
    "quebrajasteis": "quebrajar",
    "quebraje": "quebrajar",
    "quebrajemos": "quebrajar",
    "quebrajen": "quebrajar",
    "quebrajes": "quebrajar",
    "quebrajo": "quebrajar",
    "quebrajosa": "quebrajoso",
    "quebrajosas": "quebrajoso",
    "quebrajosos": "quebrajoso",
    "quebrajábamos": "quebrajar",
    "quebrajáis": "quebrajar",
    "quebrajáramos": "quebrajar",
    "quebrajáremos": "quebrajar",
    "quebrajásemos": "quebrajar",
    "quebrajé": "quebrajar",
    "quebrajéis": "quebrajar",
    "quebrajó": "quebrajar",
    "quebramientos": "quebramiento",
    "quebramos": "quebrar",
    "quebrando": "quebrar",
    "quebranta": "quebrantar",
    "quebrantaba": "quebrantar",
    "quebrantabais": "quebrantar",
    "quebrantaban": "quebrantar",
    "quebrantabas": "quebrantar",
    "quebrantables": "quebrantable",
    "quebrantad": "quebrantar",
    "quebrantada": "quebrantar",
    "quebrantadas": "quebrantar",
    "quebrantado": "quebrantar",
    "quebrantadora": "quebrantador",
    "quebrantadoras": "quebrantador",
    "quebrantadores": "quebrantador",
    "quebrantados": "quebrantar",
    "quebrantaduras": "quebrantadura",
    "quebrantamientos": "quebrantamiento",
    "quebrantamos": "quebrantar",
    "quebrantan": "quebrantar",
    "quebrantando": "quebrantar",
    "quebrantara": "quebrantar",
    "quebrantarais": "quebrantar",
    "quebrantaran": "quebrantar",
    "quebrantaras": "quebrantar",
    "quebrantare": "quebrantar",
    "quebrantareis": "quebrantar",
    "quebrantaremos": "quebrantar",
    "quebrantaren": "quebrantar",
    "quebrantares": "quebrantar",
    "quebrantaron": "quebrantar",
    "quebrantará": "quebrantar",
    "quebrantarán": "quebrantar",
    "quebrantarás": "quebrantar",
    "quebrantaré": "quebrantar",
    "quebrantaréis": "quebrantar",
    "quebrantaría": "quebrantar",
    "quebrantaríais": "quebrantar",
    "quebrantaríamos": "quebrantar",
    "quebrantarían": "quebrantar",
    "quebrantarías": "quebrantar",
    "quebrantas": "quebrantar",
    "quebrantase": "quebrantar",
    "quebrantaseis": "quebrantar",
    "quebrantasen": "quebrantar",
    "quebrantases": "quebrantar",
    "quebrantaste": "quebrantar",
    "quebrantasteis": "quebrantar",
    "quebrante": "quebrantar",
    "quebrantemos": "quebrantar",
    "quebranten": "quebrantar",
    "quebrantes": "quebrantar",
    "quebranto": "quebrantar",
    "quebrantos": "quebranto",
    "quebrantábamos": "quebrantar",
    "quebrantáis": "quebrantar",
    "quebrantáramos": "quebrantar",
    "quebrantáremos": "quebrantar",
    "quebrantásemos": "quebrantar",
    "quebranté": "quebrantar",
    "quebrantéis": "quebrantar",
    "quebrantó": "quebrantar",
    "quebrara": "quebrar",
    "quebrarais": "quebrar",
    "quebraran": "quebrar",
    "quebraras": "quebrar",
    "quebrare": "quebrar",
    "quebrareis": "quebrar",
    "quebraremos": "quebrar",
    "quebraren": "quebrar",
    "quebrares": "quebrar",
    "quebraron": "quebrar",
    "quebrará": "quebrar",
    "quebrarán": "quebrar",
    "quebrarás": "quebrar",
    "quebraré": "quebrar",
    "quebraréis": "quebrar",
    "quebraría": "quebrar",
    "quebraríais": "quebrar",
    "quebraríamos": "quebrar",
    "quebrarían": "quebrar",
    "quebrarías": "quebrar",
    "quebrase": "quebrar",
    "quebraseis": "quebrar",
    "quebrasen": "quebrar",
    "quebrases": "quebrar",
    "quebraste": "quebrar",
    "quebrasteis": "quebrar",
    "quebrazas": "quebraza",
    "quebrazones": "quebrazón",
    "quebremos": "quebrar",
    "quebrábamos": "quebrar",
    "quebráis": "quebrar",
    "quebráramos": "quebrar",
    "quebráremos": "quebrar",
    "quebrásemos": "quebrar",
    "quebré": "quebrar",
    "quebréis": "quebrar",
    "quebró": "quebrar",
    "quechemarines": "quechemarín",
    "queches": "queche",
    "quechuas": "quechua",
    "quechuismos": "quechuismo",
    "queda": "quedo",
    "quedaba": "quedar",
    "quedabais": "quedar",
    "quedaban": "quedar",
    "quedabas": "quedar",
    "quedad": "quedar",
    "quedada": "quedar",
    "quedadas": "quedar",
    "quedado": "quedar",
    "quedados": "quedar",
    "quedamos": "quedar",
    "quedan": "quedar",
    "quedando": "quedar",
    "quedara": "quedar",
    "quedarais": "quedar",
    "quedaran": "quedar",
    "quedaras": "quedar",
    "quedare": "quedar",
    "quedareis": "quedar",
    "quedaremos": "quedar",
    "quedaren": "quedar",
    "quedares": "quedar",
    "quedaron": "quedar",
    "quedará": "quedar",
    "quedarán": "quedar",
    "quedarás": "quedar",
    "quedaré": "quedar",
    "quedaréis": "quedar",
    "quedaría": "quedar",
    "quedaríais": "quedar",
    "quedaríamos": "quedar",
    "quedarían": "quedar",
    "quedarías": "quedar",
    "quedas": "quedo",
    "quedase": "quedar",
    "quedaseis": "quedar",
    "quedasen": "quedar",
    "quedases": "quedar",
    "quedaste": "quedar",
    "quedasteis": "quedar",
    "quede": "quedar",
    "quedemos": "quedar",
    "queden": "quedar",
    "quedes": "quedar",
    "quedo": "quedar",
    "quedos": "quedo",
    "quedábamos": "quedar",
    "quedáis": "quedar",
    "quedáramos": "quedar",
    "quedáremos": "quedar",
    "quedásemos": "quedar",
    "quedé": "quedar",
    "quedéis": "quedar",
    "quedó": "quedar",
    "quehaceres": "quehacer",
    "queilofagias": "queilofagia",
    "queiloplastias": "queiloplastia",
    "queja": "quejar",
    "quejaba": "quejar",
    "quejabais": "quejar",
    "quejaban": "quejar",
    "quejabas": "quejar",
    "quejad": "quejar",
    "quejada": "quejar",
    "quejadas": "quejar",
    "quejado": "quejar",
    "quejados": "quejar",
    "quejamos": "quejar",
    "quejan": "quejar",
    "quejando": "quejar",
    "quejara": "quejar",
    "quejarais": "quejar",
    "quejaran": "quejar",
    "quejaras": "quejar",
    "quejare": "quejar",
    "quejareis": "quejar",
    "quejaremos": "quejar",
    "quejaren": "quejar",
    "quejares": "quejar",
    "quejaron": "quejar",
    "quejará": "quejar",
    "quejarán": "quejar",
    "quejarás": "quejar",
    "quejaré": "quejar",
    "quejaréis": "quejar",
    "quejaría": "quejar",
    "quejaríais": "quejar",
    "quejaríamos": "quejar",
    "quejarían": "quejar",
    "quejarías": "quejar",
    "quejas": "quejar",
    "quejase": "quejar",
    "quejaseis": "quejar",
    "quejasen": "quejar",
    "quejases": "quejar",
    "quejaste": "quejar",
    "quejasteis": "quejar",
    "queje": "quejar",
    "quejemos": "quejar",
    "quejen": "quejar",
    "quejes": "quejar",
    "quejicas": "quejica",
    "quejicosa": "quejicoso",
    "quejicosas": "quejicoso",
    "quejicosos": "quejicoso",
    "quejidos": "quejido",
    "quejigales": "quejigal",
    "quejigos": "quejigo",
    "quejo": "quejar",
    "quejona": "quejón",
    "quejonas": "quejón",
    "quejones": "quejón",
    "quejos": "quejo",
    "quejosa": "quejoso",
    "quejosas": "quejoso",
    "quejosos": "quejoso",
    "quejumbres": "quejumbre",
    "quejumbrosa": "quejumbroso",
    "quejumbrosas": "quejumbroso",
    "quejumbrosos": "quejumbroso",
    "quejábamos": "quejar",
    "quejáis": "quejar",
    "quejáramos": "quejar",
    "quejáremos": "quejar",
    "quejásemos": "quejar",
    "quejé": "quejar",
    "quejéis": "quejar",
    "quejó": "quejar",
    "quelaciones": "quelación",
    "quelantes": "quelante",
    "quelas": "quela",
    "quelatos": "quelato",
    "quelonia": "quelonio",
    "quelonias": "quelonio",
    "quelonios": "quelonio",
    "quelpos": "quelpo",
    "queluviaciones": "queluviación",
    "quelvachos": "quelvacho",
    "quelíceros": "quelícero",
    "quema": "quemar",
    "quemaba": "quemar",
    "quemabais": "quemar",
    "quemaban": "quemar",
    "quemabas": "quemar",
    "quemad": "quemar",
    "quemada": "quemar",
    "quemadas": "quemar",
    "quemado": "quemar",
    "quemadora": "quemador",
    "quemadoras": "quemador",
    "quemadores": "quemador",
    "quemados": "quemar",
    "quemaduras": "quemadura",
    "quemajosa": "quemajoso",
    "quemajosas": "quemajoso",
    "quemajosos": "quemajoso",
    "quemamos": "quemar",
    "queman": "quemar",
    "quemando": "quemar",
    "quemara": "quemar",
    "quemarais": "quemar",
    "quemaran": "quemar",
    "quemaras": "quemar",
    "quemare": "quemar",
    "quemareis": "quemar",
    "quemaremos": "quemar",
    "quemaren": "quemar",
    "quemares": "quemar",
    "quemaron": "quemar",
    "quemará": "quemar",
    "quemarán": "quemar",
    "quemarás": "quemar",
    "quemaré": "quemar",
    "quemaréis": "quemar",
    "quemaría": "quemar",
    "quemaríais": "quemar",
    "quemaríamos": "quemar",
    "quemarían": "quemar",
    "quemarías": "quemar",
    "quemas": "quemar",
    "quemase": "quemar",
    "quemaseis": "quemar",
    "quemasen": "quemar",
    "quemases": "quemar",
    "quemaste": "quemar",
    "quemasteis": "quemar",
    "quemazones": "quemazón",
    "queme": "quemar",
    "quememos": "quemar",
    "quemen": "quemar",
    "quemes": "quemar",
    "quemo": "quemar",
    "quemodectomas": "quemodectoma",
    "quemábamos": "quemar",
    "quemáis": "quemar",
    "quemáramos": "quemar",
    "quemáremos": "quemar",
    "quemásemos": "quemar",
    "quemé": "quemar",
    "queméis": "quemar",
    "quemó": "quemar",
    "quenas": "quena",
    "quenopodiácea": "quenopodiáceo",
    "quenopodiáceas": "quenopodiáceo",
    "quenopodiáceos": "quenopodiáceo",
    "quepa": "caber",
    "quepamos": "caber",
    "quepan": "caber",
    "quepas": "caber",
    "quepo": "caber",
    "quepáis": "caber",
    "queramos": "querer",
    "querargiritas": "querargirita",
    "queratalgias": "queratalgia",
    "queratectomías": "queratectomía",
    "queratinas": "queratina",
    "queratoceles": "queratocele",
    "queratoconos": "queratocono",
    "queratofaquias": "queratofaquia",
    "queratoglobos": "queratoglobo",
    "queratohemias": "queratohemia",
    "queratomalacias": "queratomalacia",
    "queratopatías": "queratopatía",
    "queratoplastias": "queratoplastia",
    "queratorréxises": "queratorrexis",
    "queratoscopias": "queratoscopia",
    "queratoscopios": "queratoscopio",
    "queratotomías": "queratotomía",
    "queratómetros": "queratómetro",
    "quercetinas": "quercetina",
    "quercitoles": "quercitol",
    "quered": "querer",
    "querella": "querellar",
    "querellaba": "querellar",
    "querellabais": "querellar",
    "querellaban": "querellar",
    "querellabas": "querellar",
    "querellad": "querellar",
    "querellada": "querellar",
    "querelladas": "querellar",
    "querellado": "querellar",
    "querelladora": "querellador",
    "querelladoras": "querellador",
    "querelladores": "querellador",
    "querellados": "querellar",
    "querellamos": "querellar",
    "querellan": "querellar",
    "querellando": "querellar",
    "querellantes": "querellante",
    "querellara": "querellar",
    "querellarais": "querellar",
    "querellaran": "querellar",
    "querellaras": "querellar",
    "querellare": "querellar",
    "querellareis": "querellar",
    "querellaremos": "querellar",
    "querellaren": "querellar",
    "querellares": "querellar",
    "querellaron": "querellar",
    "querellará": "querellar",
    "querellarán": "querellar",
    "querellarás": "querellar",
    "querellaré": "querellar",
    "querellaréis": "querellar",
    "querellaría": "querellar",
    "querellaríais": "querellar",
    "querellaríamos": "querellar",
    "querellarían": "querellar",
    "querellarías": "querellar",
    "querellas": "querellar",
    "querellase": "querellar",
    "querellaseis": "querellar",
    "querellasen": "querellar",
    "querellases": "querellar",
    "querellaste": "querellar",
    "querellasteis": "querellar",
    "querelle": "querellar",
    "querellemos": "querellar",
    "querellen": "querellar",
    "querelles": "querellar",
    "querello": "querellar",
    "querellosa": "querelloso",
    "querellosas": "querelloso",
    "querellosos": "querelloso",
    "querellábamos": "querellar",
    "querelláis": "querellar",
    "querelláramos": "querellar",
    "querelláremos": "querellar",
    "querellásemos": "querellar",
    "querellé": "querellar",
    "querelléis": "querellar",
    "querelló": "querellar",
    "queremos": "querer",
    "querencias": "querencia",
    "querenciosa": "querencioso",
    "querenciosas": "querencioso",
    "querenciosos": "querencioso",
    "querendona": "querendón",
    "querendonas": "querendón",
    "querendones": "querendón",
    "quereres": "querer",
    "queresas": "queresa",
    "queretana": "queretano",
    "queretanas": "queretano",
    "queretanos": "queretano",
    "querida": "querido",
    "queridas": "querido",
    "querido": "querer",
    "queridos": "querido",
    "queriendo": "querer",
    "querindanga": "querindango",
    "querindangas": "querindango",
    "querindangos": "querindango",
    "quermesitas": "quermesita",
    "querocha": "querochar",
    "querochaba": "querochar",
    "querochabais": "querochar",
    "querochaban": "querochar",
    "querochabas": "querochar",
    "querochad": "querochar",
    "querochada": "querochar",
    "querochadas": "querochar",
    "querochado": "querochar",
    "querochados": "querochar",
    "querochamos": "querochar",
    "querochan": "querochar",
    "querochando": "querochar",
    "querochara": "querochar",
    "querocharais": "querochar",
    "querocharan": "querochar",
    "querocharas": "querochar",
    "querochare": "querochar",
    "querochareis": "querochar",
    "querocharemos": "querochar",
    "querocharen": "querochar",
    "querochares": "querochar",
    "querocharon": "querochar",
    "querochará": "querochar",
    "querocharán": "querochar",
    "querocharás": "querochar",
    "querocharé": "querochar",
    "querocharéis": "querochar",
    "querocharía": "querochar",
    "querocharíais": "querochar",
    "querocharíamos": "querochar",
    "querocharían": "querochar",
    "querocharías": "querochar",
    "querochas": "querochar",
    "querochase": "querochar",
    "querochaseis": "querochar",
    "querochasen": "querochar",
    "querochases": "querochar",
    "querochaste": "querochar",
    "querochasteis": "querochar",
    "queroche": "querochar",
    "querochemos": "querochar",
    "querochen": "querochar",
    "queroches": "querochar",
    "querocho": "querochar",
    "querochábamos": "querochar",
    "querocháis": "querochar",
    "querocháramos": "querochar",
    "querocháremos": "querochar",
    "querochásemos": "querochar",
    "queroché": "querochar",
    "querochéis": "querochar",
    "querochó": "querochar",
    "querosenos": "queroseno",
    "querototomías": "querototomía",
    "querremos": "querer",
    "querrá": "querer",
    "querrán": "querer",
    "querrás": "querer",
    "querré": "querer",
    "querréis": "querer",
    "querría": "querer",
    "querríais": "querer",
    "querríamos": "querer",
    "querrían": "querer",
    "querrías": "querer",
    "quersonesos": "quersoneso",
    "querubes": "querube",
    "querubines": "querubín",
    "queruscos": "querusco",
    "queráis": "querer",
    "queréis": "querer",
    "quería": "querer",
    "queríais": "querer",
    "queríamos": "querer",
    "querían": "querer",
    "querías": "querer",
    "querógenos": "querógeno",
    "querúbica": "querúbico",
    "querúbicas": "querúbico",
    "querúbicos": "querúbico",
    "quesadillas": "quesadilla",
    "quesera": "quesero",
    "queseras": "quesero",
    "queseros": "quesero",
    "queserías": "quesería",
    "quesnelismos": "quesnelismo",
    "quesos": "queso",
    "questora": "questor",
    "questoras": "questor",
    "questores": "questor",
    "quetoforales": "quetoforal",
    "quetzales": "quetzal",
    "quevedesca": "quevedesco",
    "quevedescas": "quevedesco",
    "quevedescos": "quevedesco",
    "quiantis": "quianti",
    "quiasmas": "quiasma",
    "quiasmos": "quiasmo",
    "quiasmática": "quiasmático",
    "quiasmáticas": "quiasmático",
    "quiasmáticos": "quiasmático",
    "quiastolitas": "quiastolita",
    "quibeis": "quibey",
    "quibeyes": "quibey",
    "quibeys": "quibey",
    "quibinaria": "quibinario",
    "quibinarias": "quibinario",
    "quibinarios": "quibinario",
    "quichuas": "quichua",
    "quichés": "quiché",
    "quiciales": "quicial",
    "quicios": "quicio",
    "quididades": "quididad",
    "quiebra": "quebrar",
    "quiebran": "quebrar",
    "quiebras": "quiebra",
    "quiebre": "quebrar",
    "quiebren": "quebrar",
    "quiebres": "quebrar",
    "quiebro": "quebrar",
    "quiebros": "quiebro",
    "quienes": "quien",
    "quienesquiera": "quienquiera",
    "quiera": "querer",
    "quieran": "querer",
    "quieras": "querer",
    "quiere": "querer",
    "quieren": "querer",
    "quieres": "querer",
    "quiero": "querer",
    "quiescencias": "quiescencia",
    "quiescentes": "quiescente",
    "quieta": "quieto",
    "quietaciones": "quietación",
    "quietas": "quieto",
    "quietes": "quiete",
    "quietismos": "quietismo",
    "quietistas": "quietista",
    "quietos": "quieto",
    "quietudes": "quietud",
    "quijadas": "quijada",
    "quijales": "quijal",
    "quijares": "quijar",
    "quijeras": "quijera",
    "quijeros": "quijero",
    "quijos": "quijo",
    "quijotadas": "quijotada",
    "quijoterías": "quijotería",
    "quijotes": "quijote",
    "quijotesca": "quijotesco",
    "quijotescas": "quijotesco",
    "quijotescos": "quijotesco",
    "quijotismos": "quijotismo",
    "quilas": "quila",
    "quilata": "quilatar",
    "quilataba": "quilatar",
    "quilatabais": "quilatar",
    "quilataban": "quilatar",
    "quilatabas": "quilatar",
    "quilatad": "quilatar",
    "quilatada": "quilatar",
    "quilatadas": "quilatar",
    "quilatado": "quilatar",
    "quilatadora": "quilatador",
    "quilatadoras": "quilatador",
    "quilatadores": "quilatador",
    "quilatados": "quilatar",
    "quilatamos": "quilatar",
    "quilatan": "quilatar",
    "quilatando": "quilatar",
    "quilatara": "quilatar",
    "quilatarais": "quilatar",
    "quilataran": "quilatar",
    "quilataras": "quilatar",
    "quilatare": "quilatar",
    "quilatareis": "quilatar",
    "quilataremos": "quilatar",
    "quilataren": "quilatar",
    "quilatares": "quilatar",
    "quilataron": "quilatar",
    "quilatará": "quilatar",
    "quilatarán": "quilatar",
    "quilatarás": "quilatar",
    "quilataré": "quilatar",
    "quilataréis": "quilatar",
    "quilataría": "quilatar",
    "quilataríais": "quilatar",
    "quilataríamos": "quilatar",
    "quilatarían": "quilatar",
    "quilatarías": "quilatar",
    "quilatas": "quilatar",
    "quilatase": "quilatar",
    "quilataseis": "quilatar",
    "quilatasen": "quilatar",
    "quilatases": "quilatar",
    "quilataste": "quilatar",
    "quilatasteis": "quilatar",
    "quilate": "quilatar",
    "quilatemos": "quilatar",
    "quilaten": "quilatar",
    "quilateros": "quilatero",
    "quilates": "quilate",
    "quilato": "quilatar",
    "quilatábamos": "quilatar",
    "quilatáis": "quilatar",
    "quilatáramos": "quilatar",
    "quilatáremos": "quilatar",
    "quilatásemos": "quilatar",
    "quilaté": "quilatar",
    "quilatéis": "quilatar",
    "quilató": "quilatar",
    "quilifica": "quilificar",
    "quilificaba": "quilificar",
    "quilificabais": "quilificar",
    "quilificaban": "quilificar",
    "quilificabas": "quilificar",
    "quilificaciones": "quilificación",
    "quilificad": "quilificar",
    "quilificada": "quilificar",
    "quilificadas": "quilificar",
    "quilificado": "quilificar",
    "quilificados": "quilificar",
    "quilificamos": "quilificar",
    "quilifican": "quilificar",
    "quilificando": "quilificar",
    "quilificara": "quilificar",
    "quilificarais": "quilificar",
    "quilificaran": "quilificar",
    "quilificaras": "quilificar",
    "quilificare": "quilificar",
    "quilificareis": "quilificar",
    "quilificaremos": "quilificar",
    "quilificaren": "quilificar",
    "quilificares": "quilificar",
    "quilificaron": "quilificar",
    "quilificará": "quilificar",
    "quilificarán": "quilificar",
    "quilificarás": "quilificar",
    "quilificaré": "quilificar",
    "quilificaréis": "quilificar",
    "quilificaría": "quilificar",
    "quilificaríais": "quilificar",
    "quilificaríamos": "quilificar",
    "quilificarían": "quilificar",
    "quilificarías": "quilificar",
    "quilificas": "quilificar",
    "quilificase": "quilificar",
    "quilificaseis": "quilificar",
    "quilificasen": "quilificar",
    "quilificases": "quilificar",
    "quilificaste": "quilificar",
    "quilificasteis": "quilificar",
    "quilifico": "quilificar",
    "quilificábamos": "quilificar",
    "quilificáis": "quilificar",
    "quilificáramos": "quilificar",
    "quilificáremos": "quilificar",
    "quilificásemos": "quilificar",
    "quilificó": "quilificar",
    "quilifique": "quilificar",
    "quilifiquemos": "quilificar",
    "quilifiquen": "quilificar",
    "quilifiques": "quilificar",
    "quilifiqué": "quilificar",
    "quilifiquéis": "quilificar",
    "quillas": "quilla",
    "quillotros": "quillotro",
    "quilmas": "quilma",
    "quilodinias": "quilodinia",
    "quilogramos": "quilogramo",
    "quilolitros": "quilolitro",
    "quilométrica": "quilométrico",
    "quilométricas": "quilométrico",
    "quilométricos": "quilométrico",
    "quilos": "quilo",
    "quilosa": "quiloso",
    "quilosas": "quiloso",
    "quilosos": "quiloso",
    "quilovatios": "quilovatio",
    "quilífera": "quilífero",
    "quilíferas": "quilífero",
    "quilíferos": "quilífero",
    "quilómetros": "quilómetro",
    "quimbomboes": "quimbombó",
    "quimbombós": "quimbombó",
    "quimeras": "quimera",
    "quimerice": "quimerizar",
    "quimericemos": "quimerizar",
    "quimericen": "quimerizar",
    "quimerices": "quimerizar",
    "quimericé": "quimerizar",
    "quimericéis": "quimerizar",
    "quimerismos": "quimerismo",
    "quimeristas": "quimerista",
    "quimeriza": "quimerizar",
    "quimerizaba": "quimerizar",
    "quimerizabais": "quimerizar",
    "quimerizaban": "quimerizar",
    "quimerizabas": "quimerizar",
    "quimerizad": "quimerizar",
    "quimerizada": "quimerizar",
    "quimerizadas": "quimerizar",
    "quimerizado": "quimerizar",
    "quimerizados": "quimerizar",
    "quimerizamos": "quimerizar",
    "quimerizan": "quimerizar",
    "quimerizando": "quimerizar",
    "quimerizara": "quimerizar",
    "quimerizarais": "quimerizar",
    "quimerizaran": "quimerizar",
    "quimerizaras": "quimerizar",
    "quimerizare": "quimerizar",
    "quimerizareis": "quimerizar",
    "quimerizaremos": "quimerizar",
    "quimerizaren": "quimerizar",
    "quimerizares": "quimerizar",
    "quimerizaron": "quimerizar",
    "quimerizará": "quimerizar",
    "quimerizarán": "quimerizar",
    "quimerizarás": "quimerizar",
    "quimerizaré": "quimerizar",
    "quimerizaréis": "quimerizar",
    "quimerizaría": "quimerizar",
    "quimerizaríais": "quimerizar",
    "quimerizaríamos": "quimerizar",
    "quimerizarían": "quimerizar",
    "quimerizarías": "quimerizar",
    "quimerizas": "quimerizar",
    "quimerizase": "quimerizar",
    "quimerizaseis": "quimerizar",
    "quimerizasen": "quimerizar",
    "quimerizases": "quimerizar",
    "quimerizaste": "quimerizar",
    "quimerizasteis": "quimerizar",
    "quimerizo": "quimerizar",
    "quimerizábamos": "quimerizar",
    "quimerizáis": "quimerizar",
    "quimerizáramos": "quimerizar",
    "quimerizáremos": "quimerizar",
    "quimerizásemos": "quimerizar",
    "quimerizó": "quimerizar",
    "quimiadsorciones": "quimiadsorción",
    "quimicofarmacéutica": "quimicofarmacéutico",
    "quimicofarmacéuticas": "quimicofarmacéutico",
    "quimicofarmacéuticos": "quimicofarmacéutico",
    "quimifica": "quimificar",
    "quimificaba": "quimificar",
    "quimificabais": "quimificar",
    "quimificaban": "quimificar",
    "quimificabas": "quimificar",
    "quimificaciones": "quimificación",
    "quimificad": "quimificar",
    "quimificada": "quimificar",
    "quimificadas": "quimificar",
    "quimificado": "quimificar",
    "quimificados": "quimificar",
    "quimificamos": "quimificar",
    "quimifican": "quimificar",
    "quimificando": "quimificar",
    "quimificara": "quimificar",
    "quimificarais": "quimificar",
    "quimificaran": "quimificar",
    "quimificaras": "quimificar",
    "quimificare": "quimificar",
    "quimificareis": "quimificar",
    "quimificaremos": "quimificar",
    "quimificaren": "quimificar",
    "quimificares": "quimificar",
    "quimificaron": "quimificar",
    "quimificará": "quimificar",
    "quimificarán": "quimificar",
    "quimificarás": "quimificar",
    "quimificaré": "quimificar",
    "quimificaréis": "quimificar",
    "quimificaría": "quimificar",
    "quimificaríais": "quimificar",
    "quimificaríamos": "quimificar",
    "quimificarían": "quimificar",
    "quimificarías": "quimificar",
    "quimificas": "quimificar",
    "quimificase": "quimificar",
    "quimificaseis": "quimificar",
    "quimificasen": "quimificar",
    "quimificases": "quimificar",
    "quimificaste": "quimificar",
    "quimificasteis": "quimificar",
    "quimifico": "quimificar",
    "quimificábamos": "quimificar",
    "quimificáis": "quimificar",
    "quimificáramos": "quimificar",
    "quimificáremos": "quimificar",
    "quimificásemos": "quimificar",
    "quimificó": "quimificar",
    "quimifique": "quimificar",
    "quimifiquemos": "quimificar",
    "quimifiquen": "quimificar",
    "quimifiques": "quimificar",
    "quimifiqué": "quimificar",
    "quimifiquéis": "quimificar",
    "quimiluminiscencias": "quimiluminiscencia",
    "quimioluminiscencias": "quimioluminiscencia",
    "quimionucleares": "quimionuclear",
    "quimiorreceptores": "quimiorreceptor",
    "quimiorresistencias": "quimiorresistencia",
    "quimiosintética": "quimiosintético",
    "quimiosintéticas": "quimiosintético",
    "quimiosintéticos": "quimiosintético",
    "quimioterapias": "quimioterapia",
    "quimioterapéutica": "quimioterapéutico",
    "quimioterapéuticas": "quimioterapéutico",
    "quimioterapéuticos": "quimioterapéutico",
    "quimioterápica": "quimioterápico",
    "quimioterápicas": "quimioterápico",
    "quimioterápicos": "quimioterápico",
    "quimiotipos": "quimiotipo",
    "quimiotáctica": "quimiotáctico",
    "quimiotácticas": "quimiotáctico",
    "quimiotácticos": "quimiotáctico",
    "quimiotáxises": "quimiotaxis",
    "quimisorciones": "quimisorción",
    "quimogramas": "quimograma",
    "quimográfica": "quimográfico",
    "quimográficas": "quimográfico",
    "quimográficos": "quimográfico",
    "quimonos": "quimono",
    "quimos": "quimo",
    "quimosinas": "quimosina",
    "quimotripsinas": "quimotripsina",
    "quimérica": "quimérico",
    "quiméricas": "quimérico",
    "quiméricos": "quimérico",
    "quimógrafos": "quimógrafo",
    "quinada": "quinado",
    "quinadas": "quinado",
    "quinados": "quinado",
    "quinaldinas": "quinaldina",
    "quinales": "quinal",
    "quinalizarinas": "quinalizarina",
    "quinaquinas": "quinaquina",
    "quinaria": "quinario",
    "quinarias": "quinario",
    "quinarios": "quinario",
    "quinas": "quina",
    "quinasas": "quinasa",
    "quincallas": "quincalla",
    "quincallera": "quincallero",
    "quincalleras": "quincallero",
    "quincalleros": "quincallero",
    "quincallerías": "quincallería",
    "quinceava": "quinceavo",
    "quinceavas": "quinceavo",
    "quinceavo": "15",
    "quinceavos": "quinceavo",
    "quinceañera": "quinceañero",
    "quinceañeras": "quinceañero",
    "quinceañeros": "quinceañero",
    "quincenales": "quincenal",
    "quincenaria": "quincenario",
    "quincenarias": "quincenario",
    "quincenarios": "quincenario",
    "quincenas": "quincena",
    "quincenos": "quinceno",
    "quinchihues": "quinchihue",
    "quinchihués": "quinchihué",
    "quinchonchillos": "quinchonchillo",
    "quincuagenaria": "quincuagenario",
    "quincuagenarias": "quincuagenario",
    "quincuagenarios": "quincuagenario",
    "quincuagenas": "quincuagena",
    "quincuagesimoctava": "58",
    "quincuagesimoctavas": "58",
    "quincuagesimoctavo": "58",
    "quincuagesimoctavos": "58",
    "quincuagesimocuarta": "54",
    "quincuagesimocuartas": "54",
    "quincuagesimocuarto": "54",
    "quincuagesimocuartos": "54",
    "quincuagesimonona": "59",
    "quincuagesimononas": "59",
    "quincuagesimonono": "59",
    "quincuagesimononos": "59",
    "quincuagesimonovena": "59",
    "quincuagesimonovenas": "59",
    "quincuagesimonoveno": "59",
    "quincuagesimonovenos": "59",
    "quincuagesimoprimera": "51",
    "quincuagesimoprimeras": "51",
    "quincuagesimoprimero": "51",
    "quincuagesimoprimeros": "51",
    "quincuagesimoquinta": "55",
    "quincuagesimoquintas": "55",
    "quincuagesimoquinto": "55",
    "quincuagesimoquintos": "55",
    "quincuagesimosegunda": "52",
    "quincuagesimosegundas": "52",
    "quincuagesimosegundo": "52",
    "quincuagesimosegundos": "52",
    "quincuagesimosexta": "56",
    "quincuagesimosextas": "56",
    "quincuagesimosexto": "56",
    "quincuagesimosextos": "56",
    "quincuagesimoséptima": "57",
    "quincuagesimoséptimas": "57",
    "quincuagesimoséptimo": "57",
    "quincuagesimoséptimos": "57",
    "quincuagesimotercera": "53",
    "quincuagesimoterceras": "53",
    "quincuagesimotercero": "53",
    "quincuagesimoterceros": "53",
    "quincuagésima": "quincuagésimo",
    "quincuagésimas": "quincuagésimo",
    "quincuagésimo": "50",
    "quincuagésimos": "quincuagésimo",
    "quindecemviros": "quindecemviro",
    "quindecenviros": "quindecenviro",
    "quindeniales": "quindenial",
    "quindenios": "quindenio",
    "quindécima": "quindécimo",
    "quindécimas": "quindécimo",
    "quindécimos": "quindécimo",
    "quingentésima": "quingentésimo",
    "quingentésimas": "quingentésimo",
    "quingentésimo": "500",
    "quingentésimos": "quingentésimo",
    "quingomboes": "quingombó",
    "quingombós": "quingombó",
    "quinhidronas": "quinhidrona",
    "quinidinas": "quinidina",
    "quinielas": "quiniela",
    "quinielistas": "quinielista",
    "quinientistas": "quinientista",
    "quinientosava": "quinientosavo",
    "quinientosavas": "quinientosavo",
    "quinientosavo": "500",
    "quinientosavos": "quinientosavo",
    "quininas": "quinina",
    "quinismos": "quinismo",
    "quinoleínas": "quinoleína",
    "quinolinas": "quinolina",
    "quinonas": "quinona",
    "quinonógenos": "quinonógeno",
    "quinos": "quino",
    "quinoxalinas": "quinoxalina",
    "quinquefolios": "quinquefolio",
    "quinquenales": "quinquenal",
    "quinquenios": "quinquenio",
    "quinquis": "quinqui",
    "quinqués": "quinqué",
    "quinta": "quinto",
    "quintaba": "quintar",
    "quintabais": "quintar",
    "quintaban": "quintar",
    "quintabas": "quintar",
    "quintacolumnistas": "quintacolumnista",
    "quintad": "quintar",
    "quintada": "quintar",
    "quintadas": "quintar",
    "quintado": "quintar",
    "quintadora": "quintador",
    "quintadoras": "quintador",
    "quintadores": "quintador",
    "quintados": "quintar",
    "quintaesencia": "quintaesenciar",
    "quintaesenciaba": "quintaesenciar",
    "quintaesenciabais": "quintaesenciar",
    "quintaesenciaban": "quintaesenciar",
    "quintaesenciabas": "quintaesenciar",
    "quintaesenciad": "quintaesenciar",
    "quintaesenciada": "quintaesenciar",
    "quintaesenciadas": "quintaesenciar",
    "quintaesenciado": "quintaesenciar",
    "quintaesenciados": "quintaesenciar",
    "quintaesenciamos": "quintaesenciar",
    "quintaesencian": "quintaesenciar",
    "quintaesenciando": "quintaesenciar",
    "quintaesenciara": "quintaesenciar",
    "quintaesenciarais": "quintaesenciar",
    "quintaesenciaran": "quintaesenciar",
    "quintaesenciaras": "quintaesenciar",
    "quintaesenciare": "quintaesenciar",
    "quintaesenciareis": "quintaesenciar",
    "quintaesenciaremos": "quintaesenciar",
    "quintaesenciaren": "quintaesenciar",
    "quintaesenciares": "quintaesenciar",
    "quintaesenciaron": "quintaesenciar",
    "quintaesenciará": "quintaesenciar",
    "quintaesenciarán": "quintaesenciar",
    "quintaesenciarás": "quintaesenciar",
    "quintaesenciaré": "quintaesenciar",
    "quintaesenciaréis": "quintaesenciar",
    "quintaesenciaría": "quintaesenciar",
    "quintaesenciaríais": "quintaesenciar",
    "quintaesenciaríamos": "quintaesenciar",
    "quintaesenciarían": "quintaesenciar",
    "quintaesenciarías": "quintaesenciar",
    "quintaesencias": "quintaesenciar",
    "quintaesenciase": "quintaesenciar",
    "quintaesenciaseis": "quintaesenciar",
    "quintaesenciasen": "quintaesenciar",
    "quintaesenciases": "quintaesenciar",
    "quintaesenciaste": "quintaesenciar",
    "quintaesenciasteis": "quintaesenciar",
    "quintaesencie": "quintaesenciar",
    "quintaesenciemos": "quintaesenciar",
    "quintaesencien": "quintaesenciar",
    "quintaesencies": "quintaesenciar",
    "quintaesencio": "quintaesenciar",
    "quintaesenciábamos": "quintaesenciar",
    "quintaesenciáis": "quintaesenciar",
    "quintaesenciáramos": "quintaesenciar",
    "quintaesenciáremos": "quintaesenciar",
    "quintaesenciásemos": "quintaesenciar",
    "quintaesencié": "quintaesenciar",
    "quintaesenciéis": "quintaesenciar",
    "quintaesenció": "quintaesenciar",
    "quintalea": "quintaleo",
    "quintaleas": "quintaleo",
    "quintaleos": "quintaleo",
    "quintalera": "quintalero",
    "quintaleras": "quintalero",
    "quintaleros": "quintalero",
    "quintales": "quintal",
    "quintamos": "quintar",
    "quintan": "quintar",
    "quintanas": "quintana",
    "quintando": "quintar",
    "quintara": "quintar",
    "quintarais": "quintar",
    "quintaran": "quintar",
    "quintaras": "quintar",
    "quintare": "quintar",
    "quintareis": "quintar",
    "quintaremos": "quintar",
    "quintaren": "quintar",
    "quintares": "quintar",
    "quintaron": "quintar",
    "quintará": "quintar",
    "quintarán": "quintar",
    "quintarás": "quintar",
    "quintaré": "quintar",
    "quintaréis": "quintar",
    "quintaría": "quintar",
    "quintaríais": "quintar",
    "quintaríamos": "quintar",
    "quintarían": "quintar",
    "quintarías": "quintar",
    "quintas": "quinto",
    "quintase": "quintar",
    "quintaseis": "quintar",
    "quintasen": "quintar",
    "quintases": "quintar",
    "quintaste": "quintar",
    "quintasteis": "quintar",
    "quintañona": "quintañón",
    "quintañonas": "quintañón",
    "quintañones": "quintañón",
    "quinte": "quintar",
    "quintemos": "quintar",
    "quinten": "quintar",
    "quinternos": "quinterno",
    "quinteros": "quintero",
    "quinterías": "quintería",
    "quintes": "quintar",
    "quintetos": "quinteto",
    "quintiles": "quintil",
    "quintillas": "quintilla",
    "quintilliza": "quintillizo",
    "quintillizas": "quintillizo",
    "quintillizos": "quintillizo",
    "quintillones": "quintillón",
    "quintillos": "quintillo",
    "quinto": "quintar",
    "quintos": "quinto",
    "quintuplica": "quintuplicar",
    "quintuplicaba": "quintuplicar",
    "quintuplicabais": "quintuplicar",
    "quintuplicaban": "quintuplicar",
    "quintuplicabas": "quintuplicar",
    "quintuplicaciones": "quintuplicación",
    "quintuplicad": "quintuplicar",
    "quintuplicada": "quintuplicar",
    "quintuplicadas": "quintuplicar",
    "quintuplicado": "quintuplicar",
    "quintuplicados": "quintuplicar",
    "quintuplicamos": "quintuplicar",
    "quintuplican": "quintuplicar",
    "quintuplicando": "quintuplicar",
    "quintuplicara": "quintuplicar",
    "quintuplicarais": "quintuplicar",
    "quintuplicaran": "quintuplicar",
    "quintuplicaras": "quintuplicar",
    "quintuplicare": "quintuplicar",
    "quintuplicareis": "quintuplicar",
    "quintuplicaremos": "quintuplicar",
    "quintuplicaren": "quintuplicar",
    "quintuplicares": "quintuplicar",
    "quintuplicaron": "quintuplicar",
    "quintuplicará": "quintuplicar",
    "quintuplicarán": "quintuplicar",
    "quintuplicarás": "quintuplicar",
    "quintuplicaré": "quintuplicar",
    "quintuplicaréis": "quintuplicar",
    "quintuplicaría": "quintuplicar",
    "quintuplicaríais": "quintuplicar",
    "quintuplicaríamos": "quintuplicar",
    "quintuplicarían": "quintuplicar",
    "quintuplicarías": "quintuplicar",
    "quintuplicas": "quintuplicar",
    "quintuplicase": "quintuplicar",
    "quintuplicaseis": "quintuplicar",
    "quintuplicasen": "quintuplicar",
    "quintuplicases": "quintuplicar",
    "quintuplicaste": "quintuplicar",
    "quintuplicasteis": "quintuplicar",
    "quintuplico": "quintuplicar",
    "quintuplicábamos": "quintuplicar",
    "quintuplicáis": "quintuplicar",
    "quintuplicáramos": "quintuplicar",
    "quintuplicáremos": "quintuplicar",
    "quintuplicásemos": "quintuplicar",
    "quintuplicó": "quintuplicar",
    "quintuplique": "quintuplicar",
    "quintupliquemos": "quintuplicar",
    "quintupliquen": "quintuplicar",
    "quintupliques": "quintuplicar",
    "quintupliqué": "quintuplicar",
    "quintupliquéis": "quintuplicar",
    "quintábamos": "quintar",
    "quintáis": "quintar",
    "quintáramos": "quintar",
    "quintáremos": "quintar",
    "quintásemos": "quintar",
    "quinté": "quintar",
    "quintéis": "quintar",
    "quintó": "quintar",
    "quinzava": "quinzavo",
    "quinzavas": "quinzavo",
    "quinzavos": "quinzavo",
    "quioscos": "quiosco",
    "quiosquera": "quiosquero",
    "quiosqueras": "quiosquero",
    "quiosqueros": "quiosquero",
    "quipos": "quipo",
    "quiquiriquíes": "quiquiriquí",
    "quiquiriquís": "quiquiriquí",
    "quiragras": "quiragra",
    "quiries": "quirie",
    "quirinales": "quirinal",
    "quiritaria": "quiritario",
    "quiritarias": "quiritario",
    "quiritarios": "quiritario",
    "quirites": "quirite",
    "quirografaria": "quirografario",
    "quirografarias": "quirografario",
    "quirografarios": "quirografario",
    "quiromancias": "quiromancia",
    "quiromancías": "quiromancía",
    "quiromasajes": "quiromasaje",
    "quiromántica": "quiromántico",
    "quirománticas": "quiromántico",
    "quirománticos": "quiromántico",
    "quiropodias": "quiropodia",
    "quiropodistas": "quiropodista",
    "quiropráctica": "quiropráctico",
    "quiroprácticas": "quiropráctico",
    "quiroprácticos": "quiropráctico",
    "quiroscopias": "quiroscopia",
    "quirotecas": "quiroteca",
    "quirquinchos": "quirquincho",
    "quirófanos": "quirófano",
    "quirógrafa": "quirógrafo",
    "quirógrafas": "quirógrafo",
    "quirógrafos": "quirógrafo",
    "quirópteros": "quiróptero",
    "quirúrgica": "quirúrgico",
    "quirúrgicas": "quirúrgico",
    "quirúrgicos": "quirúrgico",
    "quise": "querer",
    "quisicosas": "quisicosa",
    "quisiera": "querer",
    "quisierais": "querer",
    "quisieran": "querer",
    "quisieras": "querer",
    "quisiere": "querer",
    "quisiereis": "querer",
    "quisieren": "querer",
    "quisieres": "querer",
    "quisieron": "querer",
    "quisiese": "querer",
    "quisieseis": "querer",
    "quisiesen": "querer",
    "quisieses": "querer",
    "quisimos": "querer",
    "quisiste": "querer",
    "quisisteis": "querer",
    "quisiéramos": "querer",
    "quisiéremos": "querer",
    "quisiésemos": "querer",
    "quiso": "querer",
    "quisquillas": "quisquilla",
    "quisquillosa": "quisquilloso",
    "quisquillosas": "quisquilloso",
    "quisquillosidades": "quisquillosidad",
    "quisquillosos": "quisquilloso",
    "quistaminas": "quistamina",
    "quistes": "quiste",
    "quita": "quito",
    "quitaba": "quitar",
    "quitabais": "quitar",
    "quitaban": "quitar",
    "quitabas": "quitar",
    "quitaciones": "quitación",
    "quitad": "quitar",
    "quitada": "quitar",
    "quitadas": "quitar",
    "quitado": "quitar",
    "quitados": "quitar",
    "quitaesmaltes": "quitaesmalte",
    "quitaipones": "quitaipón",
    "quitamientos": "quitamiento",
    "quitamos": "quitar",
    "quitan": "quitar",
    "quitando": "quitar",
    "quitanzas": "quitanza",
    "quitapones": "quitapón",
    "quitara": "quitar",
    "quitarais": "quitar",
    "quitaran": "quitar",
    "quitaras": "quitar",
    "quitare": "quitar",
    "quitareis": "quitar",
    "quitaremos": "quitar",
    "quitaren": "quitar",
    "quitares": "quitar",
    "quitaron": "quitar",
    "quitará": "quitar",
    "quitarán": "quitar",
    "quitarás": "quitar",
    "quitaré": "quitar",
    "quitaréis": "quitar",
    "quitaría": "quitar",
    "quitaríais": "quitar",
    "quitaríamos": "quitar",
    "quitarían": "quitar",
    "quitarías": "quitar",
    "quitas": "quito",
    "quitase": "quitar",
    "quitaseis": "quitar",
    "quitasen": "quitar",
    "quitases": "quitar",
    "quitasoles": "quitasol",
    "quitasolillos": "quitasolillo",
    "quitaste": "quitar",
    "quitasteis": "quitar",
    "quitasueños": "quitasueño",
    "quite": "quitar",
    "quitemos": "quitar",
    "quiten": "quitar",
    "quites": "quite",
    "quiteña": "quiteño",
    "quiteñas": "quiteño",
    "quiteños": "quiteño",
    "quitinas": "quitina",
    "quitinosa": "quitinoso",
    "quitinosas": "quitinoso",
    "quitinosos": "quitinoso",
    "quito": "quitar",
    "quitos": "quito",
    "quitridiales": "quitridial",
    "quitrines": "quitrín",
    "quitábamos": "quitar",
    "quitáis": "quitar",
    "quitáramos": "quitar",
    "quitáremos": "quitar",
    "quitásemos": "quitar",
    "quité": "quitar",
    "quitéis": "quitar",
    "quitó": "quitar",
    "quiénes": "quién",
    "quiñonera": "quiñonero",
    "quiñoneras": "quiñonero",
    "quiñoneros": "quiñonero",
    "quiñones": "quiñón",
    "qurayshíes": "qurayshí",
    "qurayshís": "qurayshí",
    "quántums": "quántum",
    "química": "químico",
    "químicas": "químico",
    "químicos": "químico",
    "quínica": "quínico",
    "quínicas": "quínico",
    "quínicos": "quínico",
    "quínolas": "quínola",
    "quíntica": "quíntico",
    "quínticas": "quíntico",
    "quínticos": "quíntico",
    "quíntupla": "quíntuplo",
    "quíntuplas": "quíntuplo",
    "quíntuples": "quíntuple",
    "quíntuplos": "quíntuplo",
    "quística": "quístico",
    "quísticas": "quístico",
    "quísticos": "quístico",
    "rabadanes": "rabadán",
    "rabadas": "rabada",
    "rabadillas": "rabadilla",
    "rabanales": "rabanal",
    "rabanera": "rabanero",
    "rabaneras": "rabanero",
    "rabaneros": "rabanero",
    "rabanillos": "rabanillo",
    "rabanizas": "rabaniza",
    "rabas": "raba",
    "rabassaires": "rabassaire",
    "rabdomancias": "rabdomancia",
    "rabea": "rabear",
    "rabeaba": "rabear",
    "rabeabais": "rabear",
    "rabeaban": "rabear",
    "rabeabas": "rabear",
    "rabead": "rabear",
    "rabeada": "rabear",
    "rabeadas": "rabear",
    "rabeado": "rabear",
    "rabeados": "rabear",
    "rabeamos": "rabear",
    "rabean": "rabear",
    "rabeando": "rabear",
    "rabeara": "rabear",
    "rabearais": "rabear",
    "rabearan": "rabear",
    "rabearas": "rabear",
    "rabeare": "rabear",
    "rabeareis": "rabear",
    "rabearemos": "rabear",
    "rabearen": "rabear",
    "rabeares": "rabear",
    "rabearon": "rabear",
    "rabeará": "rabear",
    "rabearán": "rabear",
    "rabearás": "rabear",
    "rabearé": "rabear",
    "rabearéis": "rabear",
    "rabearía": "rabear",
    "rabearíais": "rabear",
    "rabearíamos": "rabear",
    "rabearían": "rabear",
    "rabearías": "rabear",
    "rabeas": "rabear",
    "rabease": "rabear",
    "rabeaseis": "rabear",
    "rabeasen": "rabear",
    "rabeases": "rabear",
    "rabeaste": "rabear",
    "rabeasteis": "rabear",
    "rabee": "rabear",
    "rabeemos": "rabear",
    "rabeen": "rabear",
    "rabees": "rabear",
    "rabelera": "rabelero",
    "rabeleras": "rabelero",
    "rabeleros": "rabelero",
    "rabeles": "rabel",
    "rabeo": "rabear",
    "rabeos": "rabeo",
    "raberas": "rabera",
    "raberones": "raberón",
    "rabeábamos": "rabear",
    "rabeáis": "rabear",
    "rabeáramos": "rabear",
    "rabeáremos": "rabear",
    "rabeásemos": "rabear",
    "rabeé": "rabear",
    "rabeéis": "rabear",
    "rabeó": "rabear",
    "rabia": "rabiar",
    "rabiaba": "rabiar",
    "rabiabais": "rabiar",
    "rabiaban": "rabiar",
    "rabiabas": "rabiar",
    "rabiad": "rabiar",
    "rabiada": "rabiar",
    "rabiadas": "rabiar",
    "rabiado": "rabiar",
    "rabiados": "rabiar",
    "rabiamos": "rabiar",
    "rabian": "rabiar",
    "rabiando": "rabiar",
    "rabiara": "rabiar",
    "rabiarais": "rabiar",
    "rabiaran": "rabiar",
    "rabiaras": "rabiar",
    "rabiare": "rabiar",
    "rabiareis": "rabiar",
    "rabiaremos": "rabiar",
    "rabiaren": "rabiar",
    "rabiares": "rabiar",
    "rabiaron": "rabiar",
    "rabiará": "rabiar",
    "rabiarán": "rabiar",
    "rabiarás": "rabiar",
    "rabiaré": "rabiar",
    "rabiaréis": "rabiar",
    "rabiaría": "rabiar",
    "rabiaríais": "rabiar",
    "rabiaríamos": "rabiar",
    "rabiarían": "rabiar",
    "rabiarías": "rabiar",
    "rabias": "rabiar",
    "rabiase": "rabiar",
    "rabiaseis": "rabiar",
    "rabiasen": "rabiar",
    "rabiases": "rabiar",
    "rabiaste": "rabiar",
    "rabiasteis": "rabiar",
    "rabiata": "rabiatar",
    "rabiataba": "rabiatar",
    "rabiatabais": "rabiatar",
    "rabiataban": "rabiatar",
    "rabiatabas": "rabiatar",
    "rabiatad": "rabiatar",
    "rabiatada": "rabiatar",
    "rabiatadas": "rabiatar",
    "rabiatado": "rabiatar",
    "rabiatados": "rabiatar",
    "rabiatamos": "rabiatar",
    "rabiatan": "rabiatar",
    "rabiatando": "rabiatar",
    "rabiatara": "rabiatar",
    "rabiatarais": "rabiatar",
    "rabiataran": "rabiatar",
    "rabiataras": "rabiatar",
    "rabiatare": "rabiatar",
    "rabiatareis": "rabiatar",
    "rabiataremos": "rabiatar",
    "rabiataren": "rabiatar",
    "rabiatares": "rabiatar",
    "rabiataron": "rabiatar",
    "rabiatará": "rabiatar",
    "rabiatarán": "rabiatar",
    "rabiatarás": "rabiatar",
    "rabiataré": "rabiatar",
    "rabiataréis": "rabiatar",
    "rabiataría": "rabiatar",
    "rabiataríais": "rabiatar",
    "rabiataríamos": "rabiatar",
    "rabiatarían": "rabiatar",
    "rabiatarías": "rabiatar",
    "rabiatas": "rabiatar",
    "rabiatase": "rabiatar",
    "rabiataseis": "rabiatar",
    "rabiatasen": "rabiatar",
    "rabiatases": "rabiatar",
    "rabiataste": "rabiatar",
    "rabiatasteis": "rabiatar",
    "rabiate": "rabiatar",
    "rabiatemos": "rabiatar",
    "rabiaten": "rabiatar",
    "rabiates": "rabiatar",
    "rabiato": "rabiatar",
    "rabiatábamos": "rabiatar",
    "rabiatáis": "rabiatar",
    "rabiatáramos": "rabiatar",
    "rabiatáremos": "rabiatar",
    "rabiatásemos": "rabiatar",
    "rabiaté": "rabiatar",
    "rabiatéis": "rabiatar",
    "rabiató": "rabiatar",
    "rabicana": "rabicano",
    "rabicanas": "rabicano",
    "rabicanos": "rabicano",
    "rabicona": "rabicón",
    "rabiconas": "rabicón",
    "rabicones": "rabicón",
    "rabicorta": "rabicorto",
    "rabicortas": "rabicorto",
    "rabicortos": "rabicorto",
    "rabie": "rabiar",
    "rabiemos": "rabiar",
    "rabien": "rabiar",
    "rabies": "rabiar",
    "rabietas": "rabieta",
    "rabihorcados": "rabihorcado",
    "rabijuncos": "rabijunco",
    "rabilarga": "rabilargo",
    "rabilargas": "rabilargo",
    "rabilargos": "rabilargo",
    "rabiles": "rabil",
    "rabillos": "rabillo",
    "rabinismos": "rabinismo",
    "rabinistas": "rabinista",
    "rabinos": "rabino",
    "rabio": "rabiar",
    "rabiones": "rabión",
    "rabiosa": "rabioso",
    "rabiosas": "rabioso",
    "rabiosos": "rabioso",
    "rabisalseras": "rabisalsera",
    "rabizas": "rabiza",
    "rabiábamos": "rabiar",
    "rabiáis": "rabiar",
    "rabiáramos": "rabiar",
    "rabiáremos": "rabiar",
    "rabiásemos": "rabiar",
    "rabié": "rabiar",
    "rabiéis": "rabiar",
    "rabió": "rabiar",
    "rabona": "rabón",
    "rabonas": "rabón",
    "rabones": "rabón",
    "rabos": "rabo",
    "rabosa": "raboso",
    "rabosas": "raboso",
    "rabosea": "rabosear",
    "raboseaba": "rabosear",
    "raboseabais": "rabosear",
    "raboseaban": "rabosear",
    "raboseabas": "rabosear",
    "rabosead": "rabosear",
    "raboseada": "rabosear",
    "raboseadas": "rabosear",
    "raboseado": "rabosear",
    "raboseados": "rabosear",
    "raboseaduras": "raboseadura",
    "raboseamos": "rabosear",
    "rabosean": "rabosear",
    "raboseando": "rabosear",
    "raboseara": "rabosear",
    "rabosearais": "rabosear",
    "rabosearan": "rabosear",
    "rabosearas": "rabosear",
    "raboseare": "rabosear",
    "raboseareis": "rabosear",
    "rabosearemos": "rabosear",
    "rabosearen": "rabosear",
    "raboseares": "rabosear",
    "rabosearon": "rabosear",
    "raboseará": "rabosear",
    "rabosearán": "rabosear",
    "rabosearás": "rabosear",
    "rabosearé": "rabosear",
    "rabosearéis": "rabosear",
    "rabosearía": "rabosear",
    "rabosearíais": "rabosear",
    "rabosearíamos": "rabosear",
    "rabosearían": "rabosear",
    "rabosearías": "rabosear",
    "raboseas": "rabosear",
    "rabosease": "rabosear",
    "raboseaseis": "rabosear",
    "raboseasen": "rabosear",
    "raboseases": "rabosear",
    "raboseaste": "rabosear",
    "raboseasteis": "rabosear",
    "rabosee": "rabosear",
    "raboseemos": "rabosear",
    "raboseen": "rabosear",
    "rabosees": "rabosear",
    "raboseo": "rabosear",
    "raboseábamos": "rabosear",
    "raboseáis": "rabosear",
    "raboseáramos": "rabosear",
    "raboseáremos": "rabosear",
    "raboseásemos": "rabosear",
    "raboseé": "rabosear",
    "raboseéis": "rabosear",
    "raboseó": "rabosear",
    "rabosos": "raboso",
    "rabotadas": "rabotada",
    "rabotea": "rabotear",
    "raboteaba": "rabotear",
    "raboteabais": "rabotear",
    "raboteaban": "rabotear",
    "raboteabas": "rabotear",
    "rabotead": "rabotear",
    "raboteada": "rabotear",
    "raboteadas": "rabotear",
    "raboteado": "rabotear",
    "raboteados": "rabotear",
    "raboteamos": "rabotear",
    "rabotean": "rabotear",
    "raboteando": "rabotear",
    "raboteara": "rabotear",
    "rabotearais": "rabotear",
    "rabotearan": "rabotear",
    "rabotearas": "rabotear",
    "raboteare": "rabotear",
    "raboteareis": "rabotear",
    "rabotearemos": "rabotear",
    "rabotearen": "rabotear",
    "raboteares": "rabotear",
    "rabotearon": "rabotear",
    "raboteará": "rabotear",
    "rabotearán": "rabotear",
    "rabotearás": "rabotear",
    "rabotearé": "rabotear",
    "rabotearéis": "rabotear",
    "rabotearía": "rabotear",
    "rabotearíais": "rabotear",
    "rabotearíamos": "rabotear",
    "rabotearían": "rabotear",
    "rabotearías": "rabotear",
    "raboteas": "rabotear",
    "rabotease": "rabotear",
    "raboteaseis": "rabotear",
    "raboteasen": "rabotear",
    "raboteases": "rabotear",
    "raboteaste": "rabotear",
    "raboteasteis": "rabotear",
    "rabotee": "rabotear",
    "raboteemos": "rabotear",
    "raboteen": "rabotear",
    "rabotees": "rabotear",
    "raboteo": "rabotear",
    "raboteos": "raboteo",
    "raboteábamos": "rabotear",
    "raboteáis": "rabotear",
    "raboteáramos": "rabotear",
    "raboteáremos": "rabotear",
    "raboteásemos": "rabotear",
    "raboteé": "rabotear",
    "raboteéis": "rabotear",
    "raboteó": "rabotear",
    "rabuda": "rabudo",
    "rabudas": "rabudo",
    "rabudos": "rabudo",
    "rabíes": "rabí",
    "rabínica": "rabínico",
    "rabínicas": "rabínico",
    "rabínicos": "rabínico",
    "rabís": "rabí",
    "racamentas": "racamenta",
    "racamentos": "racamento",
    "racanerías": "racanería",
    "raceles": "racel",
    "racemizaciones": "racemización",
    "racemosa": "racemoso",
    "racemosas": "racemoso",
    "racemosos": "racemoso",
    "raceres": "racer",
    "rachas": "racha",
    "racheada": "racheado",
    "racheadas": "racheado",
    "racheados": "racheado",
    "raciales": "racial",
    "racima": "racimar",
    "racimaba": "racimar",
    "racimabais": "racimar",
    "racimaban": "racimar",
    "racimabas": "racimar",
    "racimad": "racimar",
    "racimada": "racimar",
    "racimadas": "racimar",
    "racimado": "racimar",
    "racimados": "racimar",
    "racimamos": "racimar",
    "raciman": "racimar",
    "racimando": "racimar",
    "racimara": "racimar",
    "racimarais": "racimar",
    "racimaran": "racimar",
    "racimaras": "racimar",
    "racimare": "racimar",
    "racimareis": "racimar",
    "racimaremos": "racimar",
    "racimaren": "racimar",
    "racimares": "racimar",
    "racimaron": "racimar",
    "racimará": "racimar",
    "racimarán": "racimar",
    "racimarás": "racimar",
    "racimaré": "racimar",
    "racimaréis": "racimar",
    "racimaría": "racimar",
    "racimaríais": "racimar",
    "racimaríamos": "racimar",
    "racimarían": "racimar",
    "racimarías": "racimar",
    "racimas": "racimar",
    "racimase": "racimar",
    "racimaseis": "racimar",
    "racimasen": "racimar",
    "racimases": "racimar",
    "racimaste": "racimar",
    "racimasteis": "racimar",
    "racime": "racimar",
    "racimemos": "racimar",
    "racimen": "racimar",
    "racimes": "racimar",
    "racimo": "racimar",
    "racimos": "racimo",
    "racimosa": "racimoso",
    "racimosas": "racimoso",
    "racimosos": "racimoso",
    "racimuda": "racimudo",
    "racimudas": "racimudo",
    "racimudos": "racimudo",
    "racimábamos": "racimar",
    "racimáis": "racimar",
    "racimáramos": "racimar",
    "racimáremos": "racimar",
    "racimásemos": "racimar",
    "racimé": "racimar",
    "raciméis": "racimar",
    "racimó": "racimar",
    "raciocina": "raciocinar",
    "raciocinaba": "raciocinar",
    "raciocinabais": "raciocinar",
    "raciocinaban": "raciocinar",
    "raciocinabas": "raciocinar",
    "raciocinaciones": "raciocinación",
    "raciocinad": "raciocinar",
    "raciocinada": "raciocinar",
    "raciocinadas": "raciocinar",
    "raciocinado": "raciocinar",
    "raciocinados": "raciocinar",
    "raciocinamos": "raciocinar",
    "raciocinan": "raciocinar",
    "raciocinando": "raciocinar",
    "raciocinara": "raciocinar",
    "raciocinarais": "raciocinar",
    "raciocinaran": "raciocinar",
    "raciocinaras": "raciocinar",
    "raciocinare": "raciocinar",
    "raciocinareis": "raciocinar",
    "raciocinaremos": "raciocinar",
    "raciocinaren": "raciocinar",
    "raciocinares": "raciocinar",
    "raciocinaron": "raciocinar",
    "raciocinará": "raciocinar",
    "raciocinarán": "raciocinar",
    "raciocinarás": "raciocinar",
    "raciocinaré": "raciocinar",
    "raciocinaréis": "raciocinar",
    "raciocinaría": "raciocinar",
    "raciocinaríais": "raciocinar",
    "raciocinaríamos": "raciocinar",
    "raciocinarían": "raciocinar",
    "raciocinarías": "raciocinar",
    "raciocinas": "raciocinar",
    "raciocinase": "raciocinar",
    "raciocinaseis": "raciocinar",
    "raciocinasen": "raciocinar",
    "raciocinases": "raciocinar",
    "raciocinaste": "raciocinar",
    "raciocinasteis": "raciocinar",
    "raciocine": "raciocinar",
    "raciocinemos": "raciocinar",
    "raciocinen": "raciocinar",
    "raciocines": "raciocinar",
    "raciocinios": "raciocinio",
    "raciocino": "raciocinar",
    "raciocinábamos": "raciocinar",
    "raciocináis": "raciocinar",
    "raciocináramos": "raciocinar",
    "raciocináremos": "raciocinar",
    "raciocinásemos": "raciocinar",
    "raciociné": "raciocinar",
    "raciocinéis": "raciocinar",
    "raciocinó": "raciocinar",
    "raciona": "racionar",
    "racionaba": "racionar",
    "racionabais": "racionar",
    "racionaban": "racionar",
    "racionabas": "racionar",
    "racionabilidades": "racionabilidad",
    "racionad": "racionar",
    "racionada": "racionar",
    "racionadas": "racionar",
    "racionado": "racionar",
    "racionados": "racionar",
    "racionales": "racional",
    "racionalice": "racionalizar",
    "racionalicemos": "racionalizar",
    "racionalicen": "racionalizar",
    "racionalices": "racionalizar",
    "racionalicé": "racionalizar",
    "racionalicéis": "racionalizar",
    "racionalidades": "racionalidad",
    "racionalismos": "racionalismo",
    "racionalistas": "racionalista",
    "racionaliza": "racionalizar",
    "racionalizaba": "racionalizar",
    "racionalizabais": "racionalizar",
    "racionalizaban": "racionalizar",
    "racionalizabas": "racionalizar",
    "racionalizaciones": "racionalización",
    "racionalizad": "racionalizar",
    "racionalizada": "racionalizar",
    "racionalizadas": "racionalizar",
    "racionalizado": "racionalizar",
    "racionalizados": "racionalizar",
    "racionalizamos": "racionalizar",
    "racionalizan": "racionalizar",
    "racionalizando": "racionalizar",
    "racionalizara": "racionalizar",
    "racionalizarais": "racionalizar",
    "racionalizaran": "racionalizar",
    "racionalizaras": "racionalizar",
    "racionalizare": "racionalizar",
    "racionalizareis": "racionalizar",
    "racionalizaremos": "racionalizar",
    "racionalizaren": "racionalizar",
    "racionalizares": "racionalizar",
    "racionalizaron": "racionalizar",
    "racionalizará": "racionalizar",
    "racionalizarán": "racionalizar",
    "racionalizarás": "racionalizar",
    "racionalizaré": "racionalizar",
    "racionalizaréis": "racionalizar",
    "racionalizaría": "racionalizar",
    "racionalizaríais": "racionalizar",
    "racionalizaríamos": "racionalizar",
    "racionalizarían": "racionalizar",
    "racionalizarías": "racionalizar",
    "racionalizas": "racionalizar",
    "racionalizase": "racionalizar",
    "racionalizaseis": "racionalizar",
    "racionalizasen": "racionalizar",
    "racionalizases": "racionalizar",
    "racionalizaste": "racionalizar",
    "racionalizasteis": "racionalizar",
    "racionalizo": "racionalizar",
    "racionalizábamos": "racionalizar",
    "racionalizáis": "racionalizar",
    "racionalizáramos": "racionalizar",
    "racionalizáremos": "racionalizar",
    "racionalizásemos": "racionalizar",
    "racionalizó": "racionalizar",
    "racionamientos": "racionamiento",
    "racionamos": "racionar",
    "racionan": "racionar",
    "racionando": "racionar",
    "racionara": "racionar",
    "racionarais": "racionar",
    "racionaran": "racionar",
    "racionaras": "racionar",
    "racionare": "racionar",
    "racionareis": "racionar",
    "racionaremos": "racionar",
    "racionaren": "racionar",
    "racionares": "racionar",
    "racionaron": "racionar",
    "racionará": "racionar",
    "racionarán": "racionar",
    "racionarás": "racionar",
    "racionaré": "racionar",
    "racionaréis": "racionar",
    "racionaría": "racionar",
    "racionaríais": "racionar",
    "racionaríamos": "racionar",
    "racionarían": "racionar",
    "racionarías": "racionar",
    "racionas": "racionar",
    "racionase": "racionar",
    "racionaseis": "racionar",
    "racionasen": "racionar",
    "racionases": "racionar",
    "racionaste": "racionar",
    "racionasteis": "racionar",
    "racione": "racionar",
    "racionemos": "racionar",
    "racionen": "racionar",
    "raciones": "racionar",
    "racionistas": "racionista",
    "raciono": "racionar",
    "racionábamos": "racionar",
    "racionáis": "racionar",
    "racionáramos": "racionar",
    "racionáremos": "racionar",
    "racionásemos": "racionar",
    "racioné": "racionar",
    "racionéis": "racionar",
    "racionó": "racionar",
    "raciovitalismos": "raciovitalismo",
    "racismos": "racismo",
    "racistas": "racista",
    "racores": "racor",
    "racémica": "racémico",
    "racémicas": "racémico",
    "racémicos": "racémico",
    "rad": "radián",
    "radappertizaciones": "radappertización",
    "radarastronomías": "radarastronomía",
    "radares": "radar",
    "radarfaros": "radarfaro",
    "radaristas": "radarista",
    "radas": "rada",
    "radectomías": "radectomía",
    "rades": "rad",
    "radia": "radiar",
    "radiaba": "radiar",
    "radiabais": "radiar",
    "radiaban": "radiar",
    "radiabas": "radiar",
    "radiaciones": "radiación",
    "radiactinios": "radiactinio",
    "radiactiva": "radiactivo",
    "radiactivas": "radiactivo",
    "radiactividades": "radiactividad",
    "radiactivos": "radiactivo",
    "radiad": "radiar",
    "radiada": "radiar",
    "radiadas": "radiar",
    "radiado": "radiar",
    "radiadores": "radiador",
    "radiados": "radiar",
    "radiales": "radial",
    "radiamos": "radiar",
    "radian": "radiar",
    "radiancias": "radiancia",
    "radiando": "radiar",
    "radianes": "radián",
    "radiantes": "radiante",
    "radiara": "radiar",
    "radiarais": "radiar",
    "radiaran": "radiar",
    "radiaras": "radiar",
    "radiare": "radiar",
    "radiareis": "radiar",
    "radiaremos": "radiar",
    "radiaren": "radiar",
    "radiares": "radiar",
    "radiaron": "radiar",
    "radiará": "radiar",
    "radiarán": "radiar",
    "radiarás": "radiar",
    "radiaré": "radiar",
    "radiaréis": "radiar",
    "radiaría": "radiar",
    "radiaríais": "radiar",
    "radiaríamos": "radiar",
    "radiarían": "radiar",
    "radiarías": "radiar",
    "radias": "radiar",
    "radiase": "radiar",
    "radiaseis": "radiar",
    "radiasen": "radiar",
    "radiases": "radiar",
    "radiaste": "radiar",
    "radiasteis": "radiar",
    "radiativa": "radiativo",
    "radiativas": "radiativo",
    "radiativos": "radiativo",
    "radica": "radicar",
    "radicaba": "radicar",
    "radicabais": "radicar",
    "radicaban": "radicar",
    "radicabas": "radicar",
    "radicaciones": "radicación",
    "radicad": "radicar",
    "radicada": "radicar",
    "radicadas": "radicar",
    "radicado": "radicar",
    "radicadores": "radicador",
    "radicados": "radicar",
    "radicales": "radical",
    "radicalice": "radicalizar",
    "radicalicemos": "radicalizar",
    "radicalicen": "radicalizar",
    "radicalices": "radicalizar",
    "radicalicé": "radicalizar",
    "radicalicéis": "radicalizar",
    "radicalidades": "radicalidad",
    "radicalismos": "radicalismo",
    "radicaliza": "radicalizar",
    "radicalizaba": "radicalizar",
    "radicalizabais": "radicalizar",
    "radicalizaban": "radicalizar",
    "radicalizabas": "radicalizar",
    "radicalizaciones": "radicalización",
    "radicalizad": "radicalizar",
    "radicalizada": "radicalizar",
    "radicalizadas": "radicalizar",
    "radicalizado": "radicalizar",
    "radicalizados": "radicalizar",
    "radicalizamos": "radicalizar",
    "radicalizan": "radicalizar",
    "radicalizando": "radicalizar",
    "radicalizara": "radicalizar",
    "radicalizarais": "radicalizar",
    "radicalizaran": "radicalizar",
    "radicalizaras": "radicalizar",
    "radicalizare": "radicalizar",
    "radicalizareis": "radicalizar",
    "radicalizaremos": "radicalizar",
    "radicalizaren": "radicalizar",
    "radicalizares": "radicalizar",
    "radicalizaron": "radicalizar",
    "radicalizará": "radicalizar",
    "radicalizarán": "radicalizar",
    "radicalizarás": "radicalizar",
    "radicalizaré": "radicalizar",
    "radicalizaréis": "radicalizar",
    "radicalizaría": "radicalizar",
    "radicalizaríais": "radicalizar",
    "radicalizaríamos": "radicalizar",
    "radicalizarían": "radicalizar",
    "radicalizarías": "radicalizar",
    "radicalizas": "radicalizar",
    "radicalizase": "radicalizar",
    "radicalizaseis": "radicalizar",
    "radicalizasen": "radicalizar",
    "radicalizases": "radicalizar",
    "radicalizaste": "radicalizar",
    "radicalizasteis": "radicalizar",
    "radicalizo": "radicalizar",
    "radicalizábamos": "radicalizar",
    "radicalizáis": "radicalizar",
    "radicalizáramos": "radicalizar",
    "radicalizáremos": "radicalizar",
    "radicalizásemos": "radicalizar",
    "radicalizó": "radicalizar",
    "radicalsocialismos": "radicalsocialismo",
    "radicalsocialistas": "radicalsocialista",
    "radicamos": "radicar",
    "radican": "radicar",
    "radicando": "radicar",
    "radicandos": "radicando",
    "radicara": "radicar",
    "radicarais": "radicar",
    "radicaran": "radicar",
    "radicaras": "radicar",
    "radicare": "radicar",
    "radicareis": "radicar",
    "radicaremos": "radicar",
    "radicaren": "radicar",
    "radicares": "radicar",
    "radicaron": "radicar",
    "radicará": "radicar",
    "radicarán": "radicar",
    "radicarás": "radicar",
    "radicaré": "radicar",
    "radicaréis": "radicar",
    "radicaría": "radicar",
    "radicaríais": "radicar",
    "radicaríamos": "radicar",
    "radicarían": "radicar",
    "radicarías": "radicar",
    "radicas": "radicar",
    "radicase": "radicar",
    "radicaseis": "radicar",
    "radicasen": "radicar",
    "radicases": "radicar",
    "radicaste": "radicar",
    "radicasteis": "radicar",
    "radicectomías": "radicectomía",
    "radicidaciones": "radicidación",
    "radico": "radicar",
    "radicomunicaciones": "radicomunicación",
    "radicosa": "radicoso",
    "radicosas": "radicoso",
    "radicosos": "radicoso",
    "radiculares": "radicular",
    "radicábamos": "radicar",
    "radicáis": "radicar",
    "radicáramos": "radicar",
    "radicáremos": "radicar",
    "radicásemos": "radicar",
    "radicícolas": "radicícola",
    "radicó": "radicar",
    "radie": "radiar",
    "radiemos": "radiar",
    "radien": "radiar",
    "radies": "radiar",
    "radiestesias": "radiestesia",
    "radiestesistas": "radiestesista",
    "radio": "radiar",
    "radioactiva": "radioactivo",
    "radioactivas": "radioactivo",
    "radioactividades": "radioactividad",
    "radioactivos": "radioactivo",
    "radioacústicas": "radioacústica",
    "radioaficionada": "radioaficionado",
    "radioaficionadas": "radioaficionado",
    "radioaficionados": "radioaficionado",
    "radioalineaciones": "radioalineación",
    "radioaltímetros": "radioaltímetro",
    "radioastronomías": "radioastronomía",
    "radioastrónoma": "radioastrónomo",
    "radioastrónomas": "radioastrónomo",
    "radioastrónomos": "radioastrónomo",
    "radioautografías": "radioautografía",
    "radiobalizas": "radiobaliza",
    "radiobúsquedas": "radiobúsqueda",
    "radiocarbonos": "radiocarbono",
    "radiocarpiana": "radiocarpiano",
    "radiocarpianas": "radiocarpiano",
    "radiocarpianos": "radiocarpiano",
    "radiocasetes": "radiocasete",
    "radioclimática": "radioclimático",
    "radioclimáticas": "radioclimático",
    "radioclimáticos": "radioclimático",
    "radiocompases": "radiocompás",
    "radiocomunicaciones": "radiocomunicación",
    "radioconcéntrica": "radioconcéntrico",
    "radioconcéntricas": "radioconcéntrico",
    "radioconcéntricos": "radioconcéntrico",
    "radioconductores": "radioconductor",
    "radiocontroles": "radiocontrol",
    "radiocronologías": "radiocronología",
    "radiocroísmos": "radiocroísmo",
    "radiocubitales": "radiocubital",
    "radiodespertadores": "radiodespertador",
    "radiodifundida": "radiodifundido",
    "radiodifundidas": "radiodifundido",
    "radiodifundidos": "radiodifundido",
    "radiodifusiones": "radiodifusión",
    "radiodifusora": "radiodifusor",
    "radiodifusoras": "radiodifusor",
    "radiodifusores": "radiodifusor",
    "radiodinámica": "radiodinámico",
    "radiodinámicas": "radiodinámico",
    "radiodinámicos": "radiodinámico",
    "radioelectricidades": "radioelectricidad",
    "radioelementos": "radioelemento",
    "radioeléctrica": "radioeléctrico",
    "radioeléctricas": "radioeléctrico",
    "radioeléctricos": "radioeléctrico",
    "radioemanaciones": "radioemanación",
    "radioemisora": "radioemisor",
    "radioemisoras": "radioemisora",
    "radioemisores": "radioemisor",
    "radioenlaces": "radioenlace",
    "radioescuchas": "radioescucha",
    "radioespectrógrafos": "radioespectrógrafo",
    "radiofaros": "radiofaro",
    "radiofonistas": "radiofonista",
    "radiofonías": "radiofonía",
    "radiofotografías": "radiofotografía",
    "radiofotogramas": "radiofotograma",
    "radiofrecuencias": "radiofrecuencia",
    "radiofuentes": "radiofuente",
    "radiofónica": "radiofónico",
    "radiofónicas": "radiofónico",
    "radiofónicos": "radiofónico",
    "radiofórmulas": "radiofórmula",
    "radiogalaxias": "radiogalaxia",
    "radiogoniometrías": "radiogoniometría",
    "radiogoniómetrica": "radiogoniómetrico",
    "radiogoniómetricas": "radiogoniómetrico",
    "radiogoniómetricos": "radiogoniómetrico",
    "radiogoniómetros": "radiogoniómetro",
    "radiografiaba": "radiografiar",
    "radiografiabais": "radiografiar",
    "radiografiaban": "radiografiar",
    "radiografiabas": "radiografiar",
    "radiografiad": "radiografiar",
    "radiografiada": "radiografiar",
    "radiografiadas": "radiografiar",
    "radiografiado": "radiografiar",
    "radiografiados": "radiografiar",
    "radiografiamos": "radiografiar",
    "radiografiando": "radiografiar",
    "radiografiara": "radiografiar",
    "radiografiarais": "radiografiar",
    "radiografiaran": "radiografiar",
    "radiografiaras": "radiografiar",
    "radiografiare": "radiografiar",
    "radiografiareis": "radiografiar",
    "radiografiaremos": "radiografiar",
    "radiografiaren": "radiografiar",
    "radiografiares": "radiografiar",
    "radiografiaron": "radiografiar",
    "radiografiará": "radiografiar",
    "radiografiarán": "radiografiar",
    "radiografiarás": "radiografiar",
    "radiografiaré": "radiografiar",
    "radiografiaréis": "radiografiar",
    "radiografiaría": "radiografiar",
    "radiografiaríais": "radiografiar",
    "radiografiaríamos": "radiografiar",
    "radiografiarían": "radiografiar",
    "radiografiarías": "radiografiar",
    "radiografiase": "radiografiar",
    "radiografiaseis": "radiografiar",
    "radiografiasen": "radiografiar",
    "radiografiases": "radiografiar",
    "radiografiaste": "radiografiar",
    "radiografiasteis": "radiografiar",
    "radiografiemos": "radiografiar",
    "radiografiábamos": "radiografiar",
    "radiografiáis": "radiografiar",
    "radiografiáramos": "radiografiar",
    "radiografiáremos": "radiografiar",
    "radiografiásemos": "radiografiar",
    "radiografié": "radiografiar",
    "radiografiéis": "radiografiar",
    "radiografió": "radiografiar",
    "radiografía": "radiografiar",
    "radiografían": "radiografiar",
    "radiografías": "radiografiar",
    "radiografíe": "radiografiar",
    "radiografíen": "radiografiar",
    "radiografíes": "radiografiar",
    "radiografío": "radiografiar",
    "radiogramas": "radiograma",
    "radiogramolas": "radiogramola",
    "radiográfica": "radiográfico",
    "radiográficas": "radiográfico",
    "radiográficos": "radiográfico",
    "radioguiada": "radioguiado",
    "radioguiadas": "radioguiado",
    "radioguiados": "radioguiado",
    "radioguías": "radioguía",
    "radiogénica": "radiogénico",
    "radiogénicas": "radiogénico",
    "radiogénicos": "radiogénico",
    "radioinmunodifusiones": "radioinmunodifusión",
    "radioinmunoensayos": "radioinmunoensayo",
    "radioinmunovaloraciones": "radioinmunovaloración",
    "radiointerferómetros": "radiointerferómetro",
    "radioisótopos": "radioisótopo",
    "radiolarios": "radiolario",
    "radiolaritas": "radiolarita",
    "radiolocalizaciones": "radiolocalización",
    "radiologías": "radiología",
    "radioluminiscencias": "radioluminiscencia",
    "radiológica": "radiológico",
    "radiológicas": "radiológico",
    "radiológicos": "radiológico",
    "radiolúcida": "radiolúcido",
    "radiolúcidas": "radiolúcido",
    "radiolúcidos": "radiolúcido",
    "radiomandos": "radiomando",
    "radiomarcaciones": "radiomarcación",
    "radiomensajes": "radiomensaje",
    "radiometalografías": "radiometalografía",
    "radiometeorologías": "radiometeorología",
    "radiometrías": "radiometría",
    "radiomicrómetros": "radiomicrómetro",
    "radiométrica": "radiométrico",
    "radiométricas": "radiométrico",
    "radiométricos": "radiométrico",
    "radiomóviles": "radiomóvil",
    "radionavegaciones": "radionavegación",
    "radionavegantes": "radionavegante",
    "radiononiómetros": "radiononiómetro",
    "radionúclidos": "radionúclido",
    "radioondas": "radioonda",
    "radioopaca": "radioopaco",
    "radioopacas": "radioopaco",
    "radioopacidades": "radioopacidad",
    "radioopacos": "radioopaco",
    "radiopatologías": "radiopatología",
    "radiopatrullas": "radiopatrulla",
    "radioquímica": "radioquímico",
    "radioquímicas": "radioquímico",
    "radioquímicos": "radioquímico",
    "radiorreceptores": "radiorreceptor",
    "radios": "radio",
    "radiosa": "radioso",
    "radiosas": "radioso",
    "radioscopias": "radioscopia",
    "radioscópica": "radioscópico",
    "radioscópicas": "radioscópico",
    "radioscópicos": "radioscópico",
    "radiosondas": "radiosonda",
    "radiosondeos": "radiosondeo",
    "radiosos": "radioso",
    "radiosónica": "radiosónico",
    "radiosónicas": "radiosónico",
    "radiosónicos": "radiosónico",
    "radioteatros": "radioteatro",
    "radiotecnias": "radiotecnia",
    "radiotelecomunicaciones": "radiotelecomunicación",
    "radiotelefonistas": "radiotelefonista",
    "radiotelefonías": "radiotelefonía",
    "radiotelefónica": "radiotelefónico",
    "radiotelefónicas": "radiotelefónico",
    "radiotelefónicos": "radiotelefónico",
    "radiotelegrafiaba": "radiotelegrafiar",
    "radiotelegrafiabais": "radiotelegrafiar",
    "radiotelegrafiaban": "radiotelegrafiar",
    "radiotelegrafiabas": "radiotelegrafiar",
    "radiotelegrafiad": "radiotelegrafiar",
    "radiotelegrafiada": "radiotelegrafiar",
    "radiotelegrafiadas": "radiotelegrafiar",
    "radiotelegrafiado": "radiotelegrafiar",
    "radiotelegrafiados": "radiotelegrafiar",
    "radiotelegrafiamos": "radiotelegrafiar",
    "radiotelegrafiando": "radiotelegrafiar",
    "radiotelegrafiara": "radiotelegrafiar",
    "radiotelegrafiarais": "radiotelegrafiar",
    "radiotelegrafiaran": "radiotelegrafiar",
    "radiotelegrafiaras": "radiotelegrafiar",
    "radiotelegrafiare": "radiotelegrafiar",
    "radiotelegrafiareis": "radiotelegrafiar",
    "radiotelegrafiaremos": "radiotelegrafiar",
    "radiotelegrafiaren": "radiotelegrafiar",
    "radiotelegrafiares": "radiotelegrafiar",
    "radiotelegrafiaron": "radiotelegrafiar",
    "radiotelegrafiará": "radiotelegrafiar",
    "radiotelegrafiarán": "radiotelegrafiar",
    "radiotelegrafiarás": "radiotelegrafiar",
    "radiotelegrafiaré": "radiotelegrafiar",
    "radiotelegrafiaréis": "radiotelegrafiar",
    "radiotelegrafiaría": "radiotelegrafiar",
    "radiotelegrafiaríais": "radiotelegrafiar",
    "radiotelegrafiaríamos": "radiotelegrafiar",
    "radiotelegrafiarían": "radiotelegrafiar",
    "radiotelegrafiarías": "radiotelegrafiar",
    "radiotelegrafiase": "radiotelegrafiar",
    "radiotelegrafiaseis": "radiotelegrafiar",
    "radiotelegrafiasen": "radiotelegrafiar",
    "radiotelegrafiases": "radiotelegrafiar",
    "radiotelegrafiaste": "radiotelegrafiar",
    "radiotelegrafiasteis": "radiotelegrafiar",
    "radiotelegrafiemos": "radiotelegrafiar",
    "radiotelegrafistas": "radiotelegrafista",
    "radiotelegrafiábamos": "radiotelegrafiar",
    "radiotelegrafiáis": "radiotelegrafiar",
    "radiotelegrafiáramos": "radiotelegrafiar",
    "radiotelegrafiáremos": "radiotelegrafiar",
    "radiotelegrafiásemos": "radiotelegrafiar",
    "radiotelegrafié": "radiotelegrafiar",
    "radiotelegrafiéis": "radiotelegrafiar",
    "radiotelegrafió": "radiotelegrafiar",
    "radiotelegrafía": "radiotelegrafiar",
    "radiotelegrafían": "radiotelegrafiar",
    "radiotelegrafías": "radiotelegrafiar",
    "radiotelegrafíe": "radiotelegrafiar",
    "radiotelegrafíen": "radiotelegrafiar",
    "radiotelegrafíes": "radiotelegrafiar",
    "radiotelegrafío": "radiotelegrafiar",
    "radiotelegramas": "radiotelegrama",
    "radiotelegráfica": "radiotelegráfico",
    "radiotelegráficas": "radiotelegráfico",
    "radiotelegráficos": "radiotelegráfico",
    "radiotelemetrías": "radiotelemetría",
    "radiotelescopios": "radiotelescopio",
    "radiotelevisiones": "radiotelevisión",
    "radioteléfonos": "radioteléfono",
    "radiotelégrafos": "radiotelégrafo",
    "radiotelémetros": "radiotelémetro",
    "radioterapias": "radioterapia",
    "radioterapistas": "radioterapista",
    "radioterápica": "radioterápico",
    "radioterápicas": "radioterápico",
    "radioterápicos": "radioterápico",
    "radiotorios": "radiotorio",
    "radiotransmisiones": "radiotransmisión",
    "radiotransmisores": "radiotransmisor",
    "radiotransparencias": "radiotransparencia",
    "radiotransparentes": "radiotransparente",
    "radiotécnica": "radiotécnico",
    "radiotécnicas": "radiotécnico",
    "radiotécnicos": "radiotécnico",
    "radioyentes": "radioyente",
    "radique": "radicar",
    "radiquemos": "radicar",
    "radiquen": "radicar",
    "radiques": "radicar",
    "radiqué": "radicar",
    "radiquéis": "radicar",
    "radiábamos": "radiar",
    "radiáis": "radiar",
    "radiáramos": "radiar",
    "radiáremos": "radiar",
    "radiásemos": "radiar",
    "radié": "radiar",
    "radiéis": "radiar",
    "radió": "radiar",
    "radiófonos": "radiófono",
    "radiógena": "radiógeno",
    "radiógenas": "radiógeno",
    "radiógenos": "radiógeno",
    "radiógrafos": "radiógrafo",
    "radióloga": "radiólogo",
    "radiólogas": "radiólogo",
    "radiólogos": "radiólogo",
    "radiómetros": "radiómetro",
    "radones": "radón",
    "rads": "rad",
    "radurizaciones": "radurización",
    "radículas": "radícula",
    "rae": "raer",
    "raed": "raer",
    "raederas": "raedera",
    "raediza": "raedizo",
    "raedizas": "raedizo",
    "raedizos": "raedizo",
    "raedora": "raedor",
    "raedoras": "raedor",
    "raedores": "raedor",
    "raeduras": "raedura",
    "raemos": "raer",
    "raen": "raer",
    "raeremos": "raer",
    "raerá": "raer",
    "raerán": "raer",
    "raerás": "raer",
    "raeré": "raer",
    "raeréis": "raer",
    "raería": "raer",
    "raeríais": "raer",
    "raeríamos": "raer",
    "raerían": "raer",
    "raerías": "raer",
    "raes": "raer",
    "rafaelesca": "rafaelesco",
    "rafaelescas": "rafaelesco",
    "rafaelescos": "rafaelesco",
    "rafaelitas": "rafaelita",
    "rafas": "rafa",
    "rafes": "rafe",
    "rafias": "rafia",
    "rafinosas": "rafinosa",
    "raflesiácea": "raflesiáceo",
    "raflesiáceas": "raflesiáceo",
    "raflesiáceos": "raflesiáceo",
    "raftings": "rafting",
    "raglanes": "raglán",
    "ragtimes": "ragtime",
    "raguas": "ragua",
    "ragúes": "ragú",
    "ragús": "ragú",
    "ragúses": "ragú",
    "raheces": "rahez",
    "raides": "raid",
    "raids": "raid",
    "raiga": "raer",
    "raigales": "raigal",
    "raigambres": "raigambre",
    "raigamos": "raer",
    "raigan": "raer",
    "raigas": "raer",
    "raigo": "raer",
    "raigones": "raigón",
    "raigáis": "raer",
    "railes": "rail",
    "raimientos": "raimiento",
    "raines": "rain",
    "rains": "rain",
    "raises": "rais",
    "raja": "rajar",
    "rajaba": "rajar",
    "rajabais": "rajar",
    "rajaban": "rajar",
    "rajabas": "rajar",
    "rajables": "rajable",
    "rajad": "rajar",
    "rajada": "rajar",
    "rajadas": "rajar",
    "rajadiza": "rajadizo",
    "rajadizas": "rajadizo",
    "rajadizos": "rajadizo",
    "rajado": "rajar",
    "rajadora": "rajador",
    "rajadoras": "rajador",
    "rajadores": "rajador",
    "rajados": "rajar",
    "rajaduras": "rajadura",
    "rajamos": "rajar",
    "rajan": "rajar",
    "rajando": "rajar",
    "rajara": "rajar",
    "rajarais": "rajar",
    "rajaran": "rajar",
    "rajaras": "rajar",
    "rajare": "rajar",
    "rajareis": "rajar",
    "rajaremos": "rajar",
    "rajaren": "rajar",
    "rajares": "rajar",
    "rajaron": "rajar",
    "rajará": "rajar",
    "rajarán": "rajar",
    "rajarás": "rajar",
    "rajaré": "rajar",
    "rajaréis": "rajar",
    "rajaría": "rajar",
    "rajaríais": "rajar",
    "rajaríamos": "rajar",
    "rajarían": "rajar",
    "rajarías": "rajar",
    "rajas": "rajar",
    "rajase": "rajar",
    "rajaseis": "rajar",
    "rajasen": "rajar",
    "rajases": "rajar",
    "rajaste": "rajar",
    "rajasteis": "rajar",
    "raje": "rajar",
    "rajemos": "rajar",
    "rajen": "rajar",
    "rajes": "rajar",
    "rajo": "rajar",
    "rajos": "rajo",
    "rajuelas": "rajuela",
    "rajábamos": "rajar",
    "rajáes": "rajá",
    "rajáis": "rajar",
    "rajáramos": "rajar",
    "rajáremos": "rajar",
    "rajás": "rajá",
    "rajásemos": "rajar",
    "rajé": "rajar",
    "rajéis": "rajar",
    "rajó": "rajar",
    "rala": "ralo",
    "ralas": "ralo",
    "ralea": "ralear",
    "raleaba": "ralear",
    "raleabais": "ralear",
    "raleaban": "ralear",
    "raleabas": "ralear",
    "ralead": "ralear",
    "raleada": "ralear",
    "raleadas": "ralear",
    "raleado": "ralear",
    "raleados": "ralear",
    "raleamos": "ralear",
    "ralean": "ralear",
    "raleando": "ralear",
    "raleara": "ralear",
    "ralearais": "ralear",
    "ralearan": "ralear",
    "ralearas": "ralear",
    "raleare": "ralear",
    "raleareis": "ralear",
    "ralearemos": "ralear",
    "ralearen": "ralear",
    "raleares": "ralear",
    "ralearon": "ralear",
    "raleará": "ralear",
    "ralearán": "ralear",
    "ralearás": "ralear",
    "ralearé": "ralear",
    "ralearéis": "ralear",
    "ralearía": "ralear",
    "ralearíais": "ralear",
    "ralearíamos": "ralear",
    "ralearían": "ralear",
    "ralearías": "ralear",
    "raleas": "ralear",
    "ralease": "ralear",
    "raleaseis": "ralear",
    "raleasen": "ralear",
    "raleases": "ralear",
    "raleaste": "ralear",
    "raleasteis": "ralear",
    "ralee": "ralear",
    "raleemos": "ralear",
    "raleen": "ralear",
    "ralees": "ralear",
    "ralentice": "ralentizar",
    "ralenticemos": "ralentizar",
    "ralenticen": "ralentizar",
    "ralentices": "ralentizar",
    "ralenticé": "ralentizar",
    "ralenticéis": "ralentizar",
    "ralentis": "ralenti",
    "ralentiza": "ralentizar",
    "ralentizaba": "ralentizar",
    "ralentizabais": "ralentizar",
    "ralentizaban": "ralentizar",
    "ralentizabas": "ralentizar",
    "ralentizaciones": "ralentización",
    "ralentizad": "ralentizar",
    "ralentizada": "ralentizar",
    "ralentizadas": "ralentizar",
    "ralentizado": "ralentizar",
    "ralentizados": "ralentizar",
    "ralentizamos": "ralentizar",
    "ralentizan": "ralentizar",
    "ralentizando": "ralentizar",
    "ralentizara": "ralentizar",
    "ralentizarais": "ralentizar",
    "ralentizaran": "ralentizar",
    "ralentizaras": "ralentizar",
    "ralentizare": "ralentizar",
    "ralentizareis": "ralentizar",
    "ralentizaremos": "ralentizar",
    "ralentizaren": "ralentizar",
    "ralentizares": "ralentizar",
    "ralentizaron": "ralentizar",
    "ralentizará": "ralentizar",
    "ralentizarán": "ralentizar",
    "ralentizarás": "ralentizar",
    "ralentizaré": "ralentizar",
    "ralentizaréis": "ralentizar",
    "ralentizaría": "ralentizar",
    "ralentizaríais": "ralentizar",
    "ralentizaríamos": "ralentizar",
    "ralentizarían": "ralentizar",
    "ralentizarías": "ralentizar",
    "ralentizas": "ralentizar",
    "ralentizase": "ralentizar",
    "ralentizaseis": "ralentizar",
    "ralentizasen": "ralentizar",
    "ralentizases": "ralentizar",
    "ralentizaste": "ralentizar",
    "ralentizasteis": "ralentizar",
    "ralentizo": "ralentizar",
    "ralentizábamos": "ralentizar",
    "ralentizáis": "ralentizar",
    "ralentizáramos": "ralentizar",
    "ralentizáremos": "ralentizar",
    "ralentizásemos": "ralentizar",
    "ralentizó": "ralentizar",
    "ralentíes": "ralentí",
    "ralentís": "ralentí",
    "raleo": "ralear",
    "raleona": "raleón",
    "raleonas": "raleón",
    "raleones": "raleón",
    "ralezas": "raleza",
    "raleábamos": "ralear",
    "raleáis": "ralear",
    "raleáramos": "ralear",
    "raleáremos": "ralear",
    "raleásemos": "ralear",
    "raleé": "ralear",
    "raleéis": "ralear",
    "raleó": "ralear",
    "ralla": "rallar",
    "rallaba": "rallar",
    "rallabais": "rallar",
    "rallaban": "rallar",
    "rallabas": "rallar",
    "rallad": "rallar",
    "rallada": "rallar",
    "ralladas": "rallar",
    "rallado": "rallar",
    "ralladores": "rallador",
    "rallados": "rallar",
    "ralladuras": "ralladura",
    "rallamos": "rallar",
    "rallan": "rallar",
    "rallando": "rallar",
    "rallara": "rallar",
    "rallarais": "rallar",
    "rallaran": "rallar",
    "rallaras": "rallar",
    "rallare": "rallar",
    "rallareis": "rallar",
    "rallaremos": "rallar",
    "rallaren": "rallar",
    "rallares": "rallar",
    "rallaron": "rallar",
    "rallará": "rallar",
    "rallarán": "rallar",
    "rallarás": "rallar",
    "rallaré": "rallar",
    "rallaréis": "rallar",
    "rallaría": "rallar",
    "rallaríais": "rallar",
    "rallaríamos": "rallar",
    "rallarían": "rallar",
    "rallarías": "rallar",
    "rallas": "rallar",
    "rallase": "rallar",
    "rallaseis": "rallar",
    "rallasen": "rallar",
    "rallases": "rallar",
    "rallaste": "rallar",
    "rallasteis": "rallar",
    "ralle": "rallar",
    "rallemos": "rallar",
    "rallen": "rallar",
    "ralles": "rallar",
    "rallies": "rally",
    "ralliformes": "ralliforme",
    "rallo": "rallar",
    "rallos": "rallo",
    "rallyes": "rallye",
    "rallys": "rally",
    "rallábamos": "rallar",
    "ralláis": "rallar",
    "ralláramos": "rallar",
    "ralláremos": "rallar",
    "rallásemos": "rallar",
    "rallé": "rallar",
    "ralléis": "rallar",
    "ralló": "rallar",
    "ralos": "ralo",
    "ramadanes": "ramadán",
    "ramadas": "ramada",
    "ramajes": "ramaje",
    "ramalazos": "ramalazo",
    "ramales": "ramal",
    "ramas": "rama",
    "ramascos": "ramasco",
    "ramazones": "ramazón",
    "ramblares": "ramblar",
    "ramblas": "rambla",
    "ramblazos": "ramblazo",
    "ramblizos": "ramblizo",
    "rambutanes": "rambután",
    "rameada": "rameado",
    "rameadas": "rameado",
    "rameados": "rameado",
    "rameales": "rameal",
    "rameras": "ramera",
    "ramerías": "ramería",
    "ramifica": "ramificar",
    "ramificaba": "ramificar",
    "ramificabais": "ramificar",
    "ramificaban": "ramificar",
    "ramificabas": "ramificar",
    "ramificaciones": "ramificación",
    "ramificad": "ramificar",
    "ramificada": "ramificar",
    "ramificadas": "ramificar",
    "ramificado": "ramificar",
    "ramificados": "ramificar",
    "ramificamos": "ramificar",
    "ramifican": "ramificar",
    "ramificando": "ramificar",
    "ramificara": "ramificar",
    "ramificarais": "ramificar",
    "ramificaran": "ramificar",
    "ramificaras": "ramificar",
    "ramificare": "ramificar",
    "ramificareis": "ramificar",
    "ramificaremos": "ramificar",
    "ramificaren": "ramificar",
    "ramificares": "ramificar",
    "ramificaron": "ramificar",
    "ramificará": "ramificar",
    "ramificarán": "ramificar",
    "ramificarás": "ramificar",
    "ramificaré": "ramificar",
    "ramificaréis": "ramificar",
    "ramificaría": "ramificar",
    "ramificaríais": "ramificar",
    "ramificaríamos": "ramificar",
    "ramificarían": "ramificar",
    "ramificarías": "ramificar",
    "ramificas": "ramificar",
    "ramificase": "ramificar",
    "ramificaseis": "ramificar",
    "ramificasen": "ramificar",
    "ramificases": "ramificar",
    "ramificaste": "ramificar",
    "ramificasteis": "ramificar",
    "ramifico": "ramificar",
    "ramificábamos": "ramificar",
    "ramificáis": "ramificar",
    "ramificáramos": "ramificar",
    "ramificáremos": "ramificar",
    "ramificásemos": "ramificar",
    "ramificó": "ramificar",
    "ramifique": "ramificar",
    "ramifiquemos": "ramificar",
    "ramifiquen": "ramificar",
    "ramifiques": "ramificar",
    "ramifiqué": "ramificar",
    "ramifiquéis": "ramificar",
    "ramillas": "ramilla",
    "ramilletera": "ramilletero",
    "ramilleteras": "ramilletero",
    "ramilleteros": "ramilletero",
    "ramilletes": "ramillete",
    "ramios": "ramio",
    "ramizas": "ramiza",
    "ramnácea": "ramnáceo",
    "ramnáceas": "ramnáceo",
    "ramnáceos": "ramnáceo",
    "ramojos": "ramojo",
    "ramonea": "ramonear",
    "ramoneaba": "ramonear",
    "ramoneabais": "ramonear",
    "ramoneaban": "ramonear",
    "ramoneabas": "ramonear",
    "ramonead": "ramonear",
    "ramoneada": "ramonear",
    "ramoneadas": "ramonear",
    "ramoneado": "ramonear",
    "ramoneados": "ramonear",
    "ramoneamos": "ramonear",
    "ramonean": "ramonear",
    "ramoneando": "ramonear",
    "ramoneara": "ramonear",
    "ramonearais": "ramonear",
    "ramonearan": "ramonear",
    "ramonearas": "ramonear",
    "ramoneare": "ramonear",
    "ramoneareis": "ramonear",
    "ramonearemos": "ramonear",
    "ramonearen": "ramonear",
    "ramoneares": "ramonear",
    "ramonearon": "ramonear",
    "ramoneará": "ramonear",
    "ramonearán": "ramonear",
    "ramonearás": "ramonear",
    "ramonearé": "ramonear",
    "ramonearéis": "ramonear",
    "ramonearía": "ramonear",
    "ramonearíais": "ramonear",
    "ramonearíamos": "ramonear",
    "ramonearían": "ramonear",
    "ramonearías": "ramonear",
    "ramoneas": "ramonear",
    "ramonease": "ramonear",
    "ramoneaseis": "ramonear",
    "ramoneasen": "ramonear",
    "ramoneases": "ramonear",
    "ramoneaste": "ramonear",
    "ramoneasteis": "ramonear",
    "ramonee": "ramonear",
    "ramoneemos": "ramonear",
    "ramoneen": "ramonear",
    "ramonees": "ramonear",
    "ramoneo": "ramonear",
    "ramoneos": "ramoneo",
    "ramoneábamos": "ramonear",
    "ramoneáis": "ramonear",
    "ramoneáramos": "ramonear",
    "ramoneáremos": "ramonear",
    "ramoneásemos": "ramonear",
    "ramoneé": "ramonear",
    "ramoneéis": "ramonear",
    "ramoneó": "ramonear",
    "ramos": "ramo",
    "ramosa": "ramoso",
    "ramosas": "ramoso",
    "ramosos": "ramoso",
    "rampantes": "rampante",
    "rampas": "rampa",
    "ramplona": "ramplón",
    "ramplonas": "ramplón",
    "ramplonerías": "ramplonería",
    "ramplones": "ramplón",
    "rampojos": "rampojo",
    "rampollos": "rampollo",
    "ranales": "ranal",
    "ranas": "rana",
    "rancajada": "rancajado",
    "rancajadas": "rancajado",
    "rancajados": "rancajado",
    "rancajos": "rancajo",
    "ranchea": "ranchear",
    "rancheaba": "ranchear",
    "rancheabais": "ranchear",
    "rancheaban": "ranchear",
    "rancheabas": "ranchear",
    "ranchead": "ranchear",
    "rancheada": "ranchear",
    "rancheadas": "ranchear",
    "rancheaderos": "rancheadero",
    "rancheado": "ranchear",
    "rancheados": "ranchear",
    "rancheamos": "ranchear",
    "ranchean": "ranchear",
    "rancheando": "ranchear",
    "rancheara": "ranchear",
    "ranchearais": "ranchear",
    "ranchearan": "ranchear",
    "ranchearas": "ranchear",
    "rancheare": "ranchear",
    "rancheareis": "ranchear",
    "ranchearemos": "ranchear",
    "ranchearen": "ranchear",
    "rancheares": "ranchear",
    "ranchearon": "ranchear",
    "rancheará": "ranchear",
    "ranchearán": "ranchear",
    "ranchearás": "ranchear",
    "ranchearé": "ranchear",
    "ranchearéis": "ranchear",
    "ranchearía": "ranchear",
    "ranchearíais": "ranchear",
    "ranchearíamos": "ranchear",
    "ranchearían": "ranchear",
    "ranchearías": "ranchear",
    "rancheas": "ranchear",
    "ranchease": "ranchear",
    "rancheaseis": "ranchear",
    "rancheasen": "ranchear",
    "rancheases": "ranchear",
    "rancheaste": "ranchear",
    "rancheasteis": "ranchear",
    "ranchee": "ranchear",
    "rancheemos": "ranchear",
    "rancheen": "ranchear",
    "ranchees": "ranchear",
    "rancheo": "ranchear",
    "ranchera": "ranchero",
    "rancheras": "ranchero",
    "rancheros": "ranchero",
    "rancherías": "ranchería",
    "rancheábamos": "ranchear",
    "rancheáis": "ranchear",
    "rancheáramos": "ranchear",
    "rancheáremos": "ranchear",
    "rancheásemos": "ranchear",
    "rancheé": "ranchear",
    "rancheéis": "ranchear",
    "rancheó": "ranchear",
    "ranchos": "rancho",
    "rancia": "rancio",
    "ranciaba": "ranciar",
    "ranciabais": "ranciar",
    "ranciaban": "ranciar",
    "ranciabas": "ranciar",
    "ranciad": "ranciar",
    "ranciada": "ranciar",
    "ranciadas": "ranciar",
    "ranciado": "ranciar",
    "ranciados": "ranciar",
    "ranciamos": "ranciar",
    "rancian": "ranciar",
    "ranciando": "ranciar",
    "ranciara": "ranciar",
    "ranciarais": "ranciar",
    "ranciaran": "ranciar",
    "ranciaras": "ranciar",
    "ranciare": "ranciar",
    "ranciareis": "ranciar",
    "ranciaremos": "ranciar",
    "ranciaren": "ranciar",
    "ranciares": "ranciar",
    "ranciaron": "ranciar",
    "ranciará": "ranciar",
    "ranciarán": "ranciar",
    "ranciarás": "ranciar",
    "ranciaré": "ranciar",
    "ranciaréis": "ranciar",
    "ranciaría": "ranciar",
    "ranciaríais": "ranciar",
    "ranciaríamos": "ranciar",
    "ranciarían": "ranciar",
    "ranciarías": "ranciar",
    "rancias": "rancio",
    "ranciase": "ranciar",
    "ranciaseis": "ranciar",
    "ranciasen": "ranciar",
    "ranciases": "ranciar",
    "ranciaste": "ranciar",
    "ranciasteis": "ranciar",
    "rancideces": "rancidez",
    "rancie": "ranciar",
    "ranciedades": "ranciedad",
    "ranciemos": "ranciar",
    "rancien": "ranciar",
    "rancies": "ranciar",
    "rancio": "ranciar",
    "rancios": "rancio",
    "ranciosa": "rancioso",
    "ranciosas": "rancioso",
    "ranciosos": "rancioso",
    "ranciábamos": "ranciar",
    "ranciáis": "ranciar",
    "ranciáramos": "ranciar",
    "ranciáremos": "ranciar",
    "ranciásemos": "ranciar",
    "rancié": "ranciar",
    "ranciéis": "ranciar",
    "ranció": "ranciar",
    "randada": "randado",
    "randadas": "randado",
    "randados": "randado",
    "randas": "randa",
    "randomizaciones": "randomización",
    "rands": "rand",
    "ranglanes": "ranglán",
    "ranglánes": "ranglán",
    "rangos": "rango",
    "ranguas": "rangua",
    "rangíferos": "rangífero",
    "ranillas": "ranilla",
    "ranina": "ranino",
    "raninas": "ranino",
    "raninos": "ranino",
    "ranitas": "ranita",
    "rankings": "ranking",
    "ranunculácea": "ranunculáceo",
    "ranunculáceas": "ranunculáceo",
    "ranunculáceos": "ranunculáceo",
    "ranura": "ranurar",
    "ranuraba": "ranurar",
    "ranurabais": "ranurar",
    "ranuraban": "ranurar",
    "ranurabas": "ranurar",
    "ranurad": "ranurar",
    "ranurada": "ranurar",
    "ranuradas": "ranurar",
    "ranurado": "ranurar",
    "ranuradoras": "ranuradora",
    "ranurados": "ranurar",
    "ranuramos": "ranurar",
    "ranuran": "ranurar",
    "ranurando": "ranurar",
    "ranurara": "ranurar",
    "ranurarais": "ranurar",
    "ranuraran": "ranurar",
    "ranuraras": "ranurar",
    "ranurare": "ranurar",
    "ranurareis": "ranurar",
    "ranuraremos": "ranurar",
    "ranuraren": "ranurar",
    "ranurares": "ranurar",
    "ranuraron": "ranurar",
    "ranurará": "ranurar",
    "ranurarán": "ranurar",
    "ranurarás": "ranurar",
    "ranuraré": "ranurar",
    "ranuraréis": "ranurar",
    "ranuraría": "ranurar",
    "ranuraríais": "ranurar",
    "ranuraríamos": "ranurar",
    "ranurarían": "ranurar",
    "ranurarías": "ranurar",
    "ranuras": "ranurar",
    "ranurase": "ranurar",
    "ranuraseis": "ranurar",
    "ranurasen": "ranurar",
    "ranurases": "ranurar",
    "ranuraste": "ranurar",
    "ranurasteis": "ranurar",
    "ranure": "ranurar",
    "ranuremos": "ranurar",
    "ranuren": "ranurar",
    "ranures": "ranurar",
    "ranuro": "ranurar",
    "ranurábamos": "ranurar",
    "ranuráis": "ranurar",
    "ranuráramos": "ranurar",
    "ranuráremos": "ranurar",
    "ranurásemos": "ranurar",
    "ranuré": "ranurar",
    "ranuréis": "ranurar",
    "ranuró": "ranurar",
    "ranúnculos": "ranúnculo",
    "rao": "raer",
    "rapa": "rapar",
    "rapaba": "rapar",
    "rapabais": "rapar",
    "rapaban": "rapar",
    "rapabas": "rapar",
    "rapacejos": "rapacejo",
    "rapacerías": "rapacería",
    "rapaces": "rapaz",
    "rapacidades": "rapacidad",
    "rapad": "rapar",
    "rapada": "rapar",
    "rapadas": "rapar",
    "rapado": "rapar",
    "rapadora": "rapador",
    "rapadoras": "rapador",
    "rapadores": "rapador",
    "rapados": "rapar",
    "rapaduras": "rapadura",
    "rapamientos": "rapamiento",
    "rapamos": "rapar",
    "rapan": "rapar",
    "rapando": "rapar",
    "rapapolvos": "rapapolvo",
    "rapara": "rapar",
    "raparais": "rapar",
    "raparan": "rapar",
    "raparas": "rapar",
    "rapare": "rapar",
    "rapareis": "rapar",
    "raparemos": "rapar",
    "raparen": "rapar",
    "rapares": "rapar",
    "raparon": "rapar",
    "rapará": "rapar",
    "raparán": "rapar",
    "raparás": "rapar",
    "raparé": "rapar",
    "raparéis": "rapar",
    "raparía": "rapar",
    "raparíais": "rapar",
    "raparíamos": "rapar",
    "raparían": "rapar",
    "raparías": "rapar",
    "rapas": "rapar",
    "rapase": "rapar",
    "rapaseis": "rapar",
    "rapasen": "rapar",
    "rapases": "rapar",
    "rapaste": "rapar",
    "rapasteis": "rapar",
    "rapaza": "rapaz",
    "rapazas": "rapaz",
    "rapazes": "rapaz",
    "rape": "rapar",
    "rapeladores": "rapelador",
    "rapemos": "rapar",
    "rapen": "rapar",
    "rapera": "rapero",
    "raperas": "rapero",
    "raperos": "rapero",
    "rapes": "rape",
    "rapideces": "rapidez",
    "rapiña": "rapiñar",
    "rapiñaba": "rapiñar",
    "rapiñabais": "rapiñar",
    "rapiñaban": "rapiñar",
    "rapiñabas": "rapiñar",
    "rapiñad": "rapiñar",
    "rapiñada": "rapiñar",
    "rapiñadas": "rapiñar",
    "rapiñado": "rapiñar",
    "rapiñadora": "rapiñador",
    "rapiñadoras": "rapiñador",
    "rapiñadores": "rapiñador",
    "rapiñados": "rapiñar",
    "rapiñamos": "rapiñar",
    "rapiñan": "rapiñar",
    "rapiñando": "rapiñar",
    "rapiñara": "rapiñar",
    "rapiñarais": "rapiñar",
    "rapiñaran": "rapiñar",
    "rapiñaras": "rapiñar",
    "rapiñare": "rapiñar",
    "rapiñareis": "rapiñar",
    "rapiñaremos": "rapiñar",
    "rapiñaren": "rapiñar",
    "rapiñares": "rapiñar",
    "rapiñaron": "rapiñar",
    "rapiñará": "rapiñar",
    "rapiñarán": "rapiñar",
    "rapiñarás": "rapiñar",
    "rapiñaré": "rapiñar",
    "rapiñaréis": "rapiñar",
    "rapiñaría": "rapiñar",
    "rapiñaríais": "rapiñar",
    "rapiñaríamos": "rapiñar",
    "rapiñarían": "rapiñar",
    "rapiñarías": "rapiñar",
    "rapiñas": "rapiñar",
    "rapiñase": "rapiñar",
    "rapiñaseis": "rapiñar",
    "rapiñasen": "rapiñar",
    "rapiñases": "rapiñar",
    "rapiñaste": "rapiñar",
    "rapiñasteis": "rapiñar",
    "rapiñe": "rapiñar",
    "rapiñemos": "rapiñar",
    "rapiñen": "rapiñar",
    "rapiñes": "rapiñar",
    "rapiño": "rapiñar",
    "rapiñábamos": "rapiñar",
    "rapiñáis": "rapiñar",
    "rapiñáramos": "rapiñar",
    "rapiñáremos": "rapiñar",
    "rapiñásemos": "rapiñar",
    "rapiñé": "rapiñar",
    "rapiñéis": "rapiñar",
    "rapiñó": "rapiñar",
    "rapo": "rapar",
    "rapos": "rapo",
    "raposas": "raposa",
    "raposea": "raposear",
    "raposeaba": "raposear",
    "raposeabais": "raposear",
    "raposeaban": "raposear",
    "raposeabas": "raposear",
    "raposead": "raposear",
    "raposeada": "raposear",
    "raposeadas": "raposear",
    "raposeado": "raposear",
    "raposeados": "raposear",
    "raposeamos": "raposear",
    "raposean": "raposear",
    "raposeando": "raposear",
    "raposeara": "raposear",
    "raposearais": "raposear",
    "raposearan": "raposear",
    "raposearas": "raposear",
    "raposeare": "raposear",
    "raposeareis": "raposear",
    "raposearemos": "raposear",
    "raposearen": "raposear",
    "raposeares": "raposear",
    "raposearon": "raposear",
    "raposeará": "raposear",
    "raposearán": "raposear",
    "raposearás": "raposear",
    "raposearé": "raposear",
    "raposearéis": "raposear",
    "raposearía": "raposear",
    "raposearíais": "raposear",
    "raposearíamos": "raposear",
    "raposearían": "raposear",
    "raposearías": "raposear",
    "raposeas": "raposear",
    "raposease": "raposear",
    "raposeaseis": "raposear",
    "raposeasen": "raposear",
    "raposeases": "raposear",
    "raposeaste": "raposear",
    "raposeasteis": "raposear",
    "raposee": "raposear",
    "raposeemos": "raposear",
    "raposeen": "raposear",
    "raposees": "raposear",
    "raposeo": "raposear",
    "raposeos": "raposeo",
    "raposera": "raposero",
    "raposeras": "raposero",
    "raposeros": "raposero",
    "raposerías": "raposería",
    "raposeábamos": "raposear",
    "raposeáis": "raposear",
    "raposeáramos": "raposear",
    "raposeáremos": "raposear",
    "raposeásemos": "raposear",
    "raposeé": "raposear",
    "raposeéis": "raposear",
    "raposeó": "raposear",
    "raposina": "raposino",
    "raposinas": "raposino",
    "raposinos": "raposino",
    "raposos": "raposo",
    "raposuna": "raposuno",
    "raposunas": "raposuno",
    "raposunos": "raposuno",
    "raposías": "raposía",
    "rappeles": "rappel",
    "rappels": "rappel",
    "rapsodas": "rapsoda",
    "rapsodias": "rapsodia",
    "rapta": "raptar",
    "raptaba": "raptar",
    "raptabais": "raptar",
    "raptaban": "raptar",
    "raptabas": "raptar",
    "raptad": "raptar",
    "raptada": "raptar",
    "raptadas": "raptar",
    "raptado": "raptar",
    "raptados": "raptar",
    "raptamos": "raptar",
    "raptan": "raptar",
    "raptando": "raptar",
    "raptara": "raptar",
    "raptarais": "raptar",
    "raptaran": "raptar",
    "raptaras": "raptar",
    "raptare": "raptar",
    "raptareis": "raptar",
    "raptaremos": "raptar",
    "raptaren": "raptar",
    "raptares": "raptar",
    "raptaron": "raptar",
    "raptará": "raptar",
    "raptarán": "raptar",
    "raptarás": "raptar",
    "raptaré": "raptar",
    "raptaréis": "raptar",
    "raptaría": "raptar",
    "raptaríais": "raptar",
    "raptaríamos": "raptar",
    "raptarían": "raptar",
    "raptarías": "raptar",
    "raptas": "raptar",
    "raptase": "raptar",
    "raptaseis": "raptar",
    "raptasen": "raptar",
    "raptases": "raptar",
    "raptaste": "raptar",
    "raptasteis": "raptar",
    "rapte": "raptar",
    "raptemos": "raptar",
    "rapten": "raptar",
    "raptes": "raptar",
    "rapto": "raptar",
    "raptora": "raptor",
    "raptoras": "raptor",
    "raptores": "raptor",
    "raptos": "rapto",
    "raptábamos": "raptar",
    "raptáis": "raptar",
    "raptáramos": "raptar",
    "raptáremos": "raptar",
    "raptásemos": "raptar",
    "rapté": "raptar",
    "raptéis": "raptar",
    "raptó": "raptar",
    "rapábamos": "rapar",
    "rapáis": "rapar",
    "rapáramos": "rapar",
    "rapáremos": "rapar",
    "rapásemos": "rapar",
    "rapé": "rapar",
    "rapéis": "rapar",
    "rapés": "rapé",
    "rapó": "rapar",
    "rapónchigos": "rapónchigo",
    "raquea": "raquear",
    "raqueaba": "raquear",
    "raqueabais": "raquear",
    "raqueaban": "raquear",
    "raqueabas": "raquear",
    "raquead": "raquear",
    "raqueada": "raquear",
    "raqueadas": "raquear",
    "raqueado": "raquear",
    "raqueados": "raquear",
    "raqueamos": "raquear",
    "raquean": "raquear",
    "raqueando": "raquear",
    "raqueara": "raquear",
    "raquearais": "raquear",
    "raquearan": "raquear",
    "raquearas": "raquear",
    "raqueare": "raquear",
    "raqueareis": "raquear",
    "raquearemos": "raquear",
    "raquearen": "raquear",
    "raqueares": "raquear",
    "raquearon": "raquear",
    "raqueará": "raquear",
    "raquearán": "raquear",
    "raquearás": "raquear",
    "raquearé": "raquear",
    "raquearéis": "raquear",
    "raquearía": "raquear",
    "raquearíais": "raquear",
    "raquearíamos": "raquear",
    "raquearían": "raquear",
    "raquearías": "raquear",
    "raqueas": "raquear",
    "raquease": "raquear",
    "raqueaseis": "raquear",
    "raqueasen": "raquear",
    "raqueases": "raquear",
    "raqueaste": "raquear",
    "raqueasteis": "raquear",
    "raquee": "raquear",
    "raqueemos": "raquear",
    "raqueen": "raquear",
    "raquees": "raquear",
    "raqueo": "raquear",
    "raquera": "raquero",
    "raqueras": "raquero",
    "raqueros": "raquero",
    "raques": "raque",
    "raquetas": "raqueta",
    "raquetazos": "raquetazo",
    "raquetera": "raquetero",
    "raqueteras": "raquetero",
    "raqueteros": "raquetero",
    "raquetistas": "raquetista",
    "raqueábamos": "raquear",
    "raqueáis": "raquear",
    "raqueáramos": "raquear",
    "raqueáremos": "raquear",
    "raqueásemos": "raquear",
    "raqueé": "raquear",
    "raqueéis": "raquear",
    "raqueó": "raquear",
    "raquialgias": "raquialgia",
    "raquitismos": "raquitismo",
    "raquídea": "raquídeo",
    "raquídeas": "raquídeo",
    "raquídeos": "raquídeo",
    "raquítica": "raquítico",
    "raquíticas": "raquítico",
    "raquíticos": "raquítico",
    "rara": "raro",
    "raras": "raro",
    "rarefacciones": "rarefacción",
    "rareface": "rarefacer",
    "rarefaced": "rarefacer",
    "rarefacemos": "rarefacer",
    "rarefacen": "rarefacer",
    "rarefaces": "rarefacer",
    "rarefaciendo": "rarefacer",
    "rarefacta": "rarefacer",
    "rarefactas": "rarefacer",
    "rarefacto": "rarefacer",
    "rarefactos": "rarefacer",
    "rarefacéis": "rarefacer",
    "rarefacía": "rarefacer",
    "rarefacíais": "rarefacer",
    "rarefacíamos": "rarefacer",
    "rarefacían": "rarefacer",
    "rarefacías": "rarefacer",
    "rarefaga": "rarefacer",
    "rarefagamos": "rarefacer",
    "rarefagan": "rarefacer",
    "rarefagas": "rarefacer",
    "rarefago": "rarefacer",
    "rarefagáis": "rarefacer",
    "rarefaremos": "rarefacer",
    "rarefará": "rarefacer",
    "rarefarán": "rarefacer",
    "rarefarás": "rarefacer",
    "rarefaré": "rarefacer",
    "rarefaréis": "rarefacer",
    "rarefaría": "rarefacer",
    "rarefaríais": "rarefacer",
    "rarefaríamos": "rarefacer",
    "rarefarían": "rarefacer",
    "rarefarías": "rarefacer",
    "rarefaz": "rarefacer",
    "rarefecha": "rarefacer",
    "rarefechas": "rarefacer",
    "rarefecho": "rarefacer",
    "rarefechos": "rarefacer",
    "rarefice": "rarefacer",
    "rareficiera": "rarefacer",
    "rareficierais": "rarefacer",
    "rareficieran": "rarefacer",
    "rareficieras": "rarefacer",
    "rareficiere": "rarefacer",
    "rareficiereis": "rarefacer",
    "rareficieren": "rarefacer",
    "rareficieres": "rarefacer",
    "rareficieron": "rarefacer",
    "rareficiese": "rarefacer",
    "rareficieseis": "rarefacer",
    "rareficiesen": "rarefacer",
    "rareficieses": "rarefacer",
    "rareficimos": "rarefacer",
    "rareficiste": "rarefacer",
    "rareficisteis": "rarefacer",
    "rareficiéramos": "rarefacer",
    "rareficiéremos": "rarefacer",
    "rareficiésemos": "rarefacer",
    "rarefizo": "rarefacer",
    "rarezas": "rareza",
    "raridades": "raridad",
    "rarifica": "rarificar",
    "rarificaba": "rarificar",
    "rarificabais": "rarificar",
    "rarificaban": "rarificar",
    "rarificabas": "rarificar",
    "rarificad": "rarificar",
    "rarificada": "rarificar",
    "rarificadas": "rarificar",
    "rarificado": "rarificar",
    "rarificados": "rarificar",
    "rarificamos": "rarificar",
    "rarifican": "rarificar",
    "rarificando": "rarificar",
    "rarificara": "rarificar",
    "rarificarais": "rarificar",
    "rarificaran": "rarificar",
    "rarificaras": "rarificar",
    "rarificare": "rarificar",
    "rarificareis": "rarificar",
    "rarificaremos": "rarificar",
    "rarificaren": "rarificar",
    "rarificares": "rarificar",
    "rarificaron": "rarificar",
    "rarificará": "rarificar",
    "rarificarán": "rarificar",
    "rarificarás": "rarificar",
    "rarificaré": "rarificar",
    "rarificaréis": "rarificar",
    "rarificaría": "rarificar",
    "rarificaríais": "rarificar",
    "rarificaríamos": "rarificar",
    "rarificarían": "rarificar",
    "rarificarías": "rarificar",
    "rarificas": "rarificar",
    "rarificase": "rarificar",
    "rarificaseis": "rarificar",
    "rarificasen": "rarificar",
    "rarificases": "rarificar",
    "rarificaste": "rarificar",
    "rarificasteis": "rarificar",
    "rarificativa": "rarificativo",
    "rarificativas": "rarificativo",
    "rarificativos": "rarificativo",
    "rarifico": "rarificar",
    "rarificábamos": "rarificar",
    "rarificáis": "rarificar",
    "rarificáramos": "rarificar",
    "rarificáremos": "rarificar",
    "rarificásemos": "rarificar",
    "rarificó": "rarificar",
    "rarifique": "rarificar",
    "rarifiquemos": "rarificar",
    "rarifiquen": "rarificar",
    "rarifiques": "rarificar",
    "rarifiqué": "rarificar",
    "rarifiquéis": "rarificar",
    "raros": "raro",
    "rasa": "raso",
    "rasaba": "rasar",
    "rasabais": "rasar",
    "rasaban": "rasar",
    "rasabas": "rasar",
    "rasad": "rasar",
    "rasada": "rasar",
    "rasadas": "rasar",
    "rasado": "rasar",
    "rasados": "rasar",
    "rasamos": "rasar",
    "rasan": "rasar",
    "rasando": "rasar",
    "rasantes": "rasante",
    "rasara": "rasar",
    "rasarais": "rasar",
    "rasaran": "rasar",
    "rasaras": "rasar",
    "rasare": "rasar",
    "rasareis": "rasar",
    "rasaremos": "rasar",
    "rasaren": "rasar",
    "rasares": "rasar",
    "rasaron": "rasar",
    "rasará": "rasar",
    "rasarán": "rasar",
    "rasarás": "rasar",
    "rasaré": "rasar",
    "rasaréis": "rasar",
    "rasaría": "rasar",
    "rasaríais": "rasar",
    "rasaríamos": "rasar",
    "rasarían": "rasar",
    "rasarías": "rasar",
    "rasas": "raso",
    "rasase": "rasar",
    "rasaseis": "rasar",
    "rasasen": "rasar",
    "rasases": "rasar",
    "rasaste": "rasar",
    "rasasteis": "rasar",
    "rasca": "rascar",
    "rascaba": "rascar",
    "rascabais": "rascar",
    "rascaban": "rascar",
    "rascabas": "rascar",
    "rascacios": "rascacio",
    "rascad": "rascar",
    "rascada": "rascar",
    "rascadas": "rascar",
    "rascaderas": "rascadera",
    "rascado": "rascar",
    "rascadora": "rascador",
    "rascadoras": "rascador",
    "rascadores": "rascador",
    "rascados": "rascar",
    "rascaduras": "rascadura",
    "rascamientos": "rascamiento",
    "rascamos": "rascar",
    "rascamoños": "rascamoño",
    "rascan": "rascar",
    "rascando": "rascar",
    "rascara": "rascar",
    "rascarais": "rascar",
    "rascaran": "rascar",
    "rascaras": "rascar",
    "rascare": "rascar",
    "rascareis": "rascar",
    "rascaremos": "rascar",
    "rascaren": "rascar",
    "rascares": "rascar",
    "rascaron": "rascar",
    "rascará": "rascar",
    "rascarán": "rascar",
    "rascarás": "rascar",
    "rascaré": "rascar",
    "rascaréis": "rascar",
    "rascaría": "rascar",
    "rascaríais": "rascar",
    "rascaríamos": "rascar",
    "rascarían": "rascar",
    "rascarías": "rascar",
    "rascas": "rascar",
    "rascase": "rascar",
    "rascaseis": "rascar",
    "rascasen": "rascar",
    "rascases": "rascar",
    "rascaste": "rascar",
    "rascasteis": "rascar",
    "rascazones": "rascazón",
    "rascles": "rascle",
    "rasco": "rascar",
    "rascona": "rascón",
    "rasconas": "rascón",
    "rascones": "rascón",
    "rascábamos": "rascar",
    "rascáis": "rascar",
    "rascáramos": "rascar",
    "rascáremos": "rascar",
    "rascásemos": "rascar",
    "rascó": "rascar",
    "rase": "rasar",
    "rasea": "rasear",
    "raseaba": "rasear",
    "raseabais": "rasear",
    "raseaban": "rasear",
    "raseabas": "rasear",
    "rasead": "rasear",
    "raseada": "rasear",
    "raseadas": "rasear",
    "raseado": "rasear",
    "raseados": "rasear",
    "raseamos": "rasear",
    "rasean": "rasear",
    "raseando": "rasear",
    "raseara": "rasear",
    "rasearais": "rasear",
    "rasearan": "rasear",
    "rasearas": "rasear",
    "raseare": "rasear",
    "raseareis": "rasear",
    "rasearemos": "rasear",
    "rasearen": "rasear",
    "raseares": "rasear",
    "rasearon": "rasear",
    "raseará": "rasear",
    "rasearán": "rasear",
    "rasearás": "rasear",
    "rasearé": "rasear",
    "rasearéis": "rasear",
    "rasearía": "rasear",
    "rasearíais": "rasear",
    "rasearíamos": "rasear",
    "rasearían": "rasear",
    "rasearías": "rasear",
    "raseas": "rasear",
    "rasease": "rasear",
    "raseaseis": "rasear",
    "raseasen": "rasear",
    "raseases": "rasear",
    "raseaste": "rasear",
    "raseasteis": "rasear",
    "rasee": "rasear",
    "raseemos": "rasear",
    "raseen": "rasear",
    "rasees": "rasear",
    "raseles": "rasel",
    "rasemos": "rasar",
    "rasen": "rasar",
    "raseo": "rasear",
    "rasera": "rasero",
    "raseras": "rasero",
    "raseros": "rasero",
    "rases": "rasar",
    "rasetes": "rasete",
    "raseábamos": "rasear",
    "raseáis": "rasear",
    "raseáramos": "rasear",
    "raseáremos": "rasear",
    "raseásemos": "rasear",
    "raseé": "rasear",
    "raseéis": "rasear",
    "raseó": "rasear",
    "rasga": "rasgar",
    "rasgaba": "rasgar",
    "rasgabais": "rasgar",
    "rasgaban": "rasgar",
    "rasgabas": "rasgar",
    "rasgad": "rasgar",
    "rasgada": "rasgar",
    "rasgadas": "rasgar",
    "rasgado": "rasgar",
    "rasgadora": "rasgador",
    "rasgadoras": "rasgador",
    "rasgadores": "rasgador",
    "rasgados": "rasgar",
    "rasgaduras": "rasgadura",
    "rasgamos": "rasgar",
    "rasgan": "rasgar",
    "rasgando": "rasgar",
    "rasgara": "rasgar",
    "rasgarais": "rasgar",
    "rasgaran": "rasgar",
    "rasgaras": "rasgar",
    "rasgare": "rasgar",
    "rasgareis": "rasgar",
    "rasgaremos": "rasgar",
    "rasgaren": "rasgar",
    "rasgares": "rasgar",
    "rasgaron": "rasgar",
    "rasgará": "rasgar",
    "rasgarán": "rasgar",
    "rasgarás": "rasgar",
    "rasgaré": "rasgar",
    "rasgaréis": "rasgar",
    "rasgaría": "rasgar",
    "rasgaríais": "rasgar",
    "rasgaríamos": "rasgar",
    "rasgarían": "rasgar",
    "rasgarías": "rasgar",
    "rasgas": "rasgar",
    "rasgase": "rasgar",
    "rasgaseis": "rasgar",
    "rasgasen": "rasgar",
    "rasgases": "rasgar",
    "rasgaste": "rasgar",
    "rasgasteis": "rasgar",
    "rasgo": "rasgar",
    "rasgones": "rasgón",
    "rasgos": "rasgo",
    "rasgue": "rasgar",
    "rasguea": "rasguear",
    "rasgueaba": "rasguear",
    "rasgueabais": "rasguear",
    "rasgueaban": "rasguear",
    "rasgueabas": "rasguear",
    "rasguead": "rasguear",
    "rasgueada": "rasguear",
    "rasgueadas": "rasguear",
    "rasgueado": "rasguear",
    "rasgueadora": "rasgueador",
    "rasgueadoras": "rasgueador",
    "rasgueadores": "rasgueador",
    "rasgueados": "rasguear",
    "rasgueamos": "rasguear",
    "rasguean": "rasguear",
    "rasgueando": "rasguear",
    "rasgueara": "rasguear",
    "rasguearais": "rasguear",
    "rasguearan": "rasguear",
    "rasguearas": "rasguear",
    "rasgueare": "rasguear",
    "rasgueareis": "rasguear",
    "rasguearemos": "rasguear",
    "rasguearen": "rasguear",
    "rasgueares": "rasguear",
    "rasguearon": "rasguear",
    "rasgueará": "rasguear",
    "rasguearán": "rasguear",
    "rasguearás": "rasguear",
    "rasguearé": "rasguear",
    "rasguearéis": "rasguear",
    "rasguearía": "rasguear",
    "rasguearíais": "rasguear",
    "rasguearíamos": "rasguear",
    "rasguearían": "rasguear",
    "rasguearías": "rasguear",
    "rasgueas": "rasguear",
    "rasguease": "rasguear",
    "rasgueaseis": "rasguear",
    "rasgueasen": "rasguear",
    "rasgueases": "rasguear",
    "rasgueaste": "rasguear",
    "rasgueasteis": "rasguear",
    "rasguee": "rasguear",
    "rasgueemos": "rasguear",
    "rasgueen": "rasguear",
    "rasguees": "rasguear",
    "rasguemos": "rasgar",
    "rasguen": "rasgar",
    "rasgueo": "rasguear",
    "rasgueos": "rasgueo",
    "rasgues": "rasgar",
    "rasgueábamos": "rasguear",
    "rasgueáis": "rasguear",
    "rasgueáramos": "rasguear",
    "rasgueáremos": "rasguear",
    "rasgueásemos": "rasguear",
    "rasgueé": "rasguear",
    "rasgueéis": "rasguear",
    "rasgueó": "rasguear",
    "rasgué": "rasgar",
    "rasguéis": "rasgar",
    "rasguña": "rasguñar",
    "rasguñaba": "rasguñar",
    "rasguñabais": "rasguñar",
    "rasguñaban": "rasguñar",
    "rasguñabas": "rasguñar",
    "rasguñad": "rasguñar",
    "rasguñada": "rasguñar",
    "rasguñadas": "rasguñar",
    "rasguñado": "rasguñar",
    "rasguñados": "rasguñar",
    "rasguñamos": "rasguñar",
    "rasguñan": "rasguñar",
    "rasguñando": "rasguñar",
    "rasguñara": "rasguñar",
    "rasguñarais": "rasguñar",
    "rasguñaran": "rasguñar",
    "rasguñaras": "rasguñar",
    "rasguñare": "rasguñar",
    "rasguñareis": "rasguñar",
    "rasguñaremos": "rasguñar",
    "rasguñaren": "rasguñar",
    "rasguñares": "rasguñar",
    "rasguñaron": "rasguñar",
    "rasguñará": "rasguñar",
    "rasguñarán": "rasguñar",
    "rasguñarás": "rasguñar",
    "rasguñaré": "rasguñar",
    "rasguñaréis": "rasguñar",
    "rasguñaría": "rasguñar",
    "rasguñaríais": "rasguñar",
    "rasguñaríamos": "rasguñar",
    "rasguñarían": "rasguñar",
    "rasguñarías": "rasguñar",
    "rasguñas": "rasguñar",
    "rasguñase": "rasguñar",
    "rasguñaseis": "rasguñar",
    "rasguñasen": "rasguñar",
    "rasguñases": "rasguñar",
    "rasguñaste": "rasguñar",
    "rasguñasteis": "rasguñar",
    "rasguñe": "rasguñar",
    "rasguñemos": "rasguñar",
    "rasguñen": "rasguñar",
    "rasguñes": "rasguñar",
    "rasguño": "rasguñar",
    "rasguños": "rasguño",
    "rasguñábamos": "rasguñar",
    "rasguñáis": "rasguñar",
    "rasguñáramos": "rasguñar",
    "rasguñáremos": "rasguñar",
    "rasguñásemos": "rasguñar",
    "rasguñé": "rasguñar",
    "rasguñéis": "rasguñar",
    "rasguñó": "rasguñar",
    "rasgábamos": "rasgar",
    "rasgáis": "rasgar",
    "rasgáramos": "rasgar",
    "rasgáremos": "rasgar",
    "rasgásemos": "rasgar",
    "rasgó": "rasgar",
    "rasillas": "rasilla",
    "raso": "rasar",
    "rasos": "raso",
    "raspa": "raspar",
    "raspaba": "raspar",
    "raspabais": "raspar",
    "raspaban": "raspar",
    "raspabas": "raspar",
    "raspad": "raspar",
    "raspada": "raspar",
    "raspadas": "raspar",
    "raspado": "raspar",
    "raspadores": "raspador",
    "raspados": "raspar",
    "raspaduras": "raspadura",
    "raspajos": "raspajo",
    "raspallones": "raspallón",
    "raspamientos": "raspamiento",
    "raspamos": "raspar",
    "raspan": "raspar",
    "raspando": "raspar",
    "raspantes": "raspante",
    "raspara": "raspar",
    "rasparais": "raspar",
    "rasparan": "raspar",
    "rasparas": "raspar",
    "raspare": "raspar",
    "raspareis": "raspar",
    "rasparemos": "raspar",
    "rasparen": "raspar",
    "raspares": "raspar",
    "rasparon": "raspar",
    "raspará": "raspar",
    "rasparán": "raspar",
    "rasparás": "raspar",
    "rasparé": "raspar",
    "rasparéis": "raspar",
    "rasparía": "raspar",
    "rasparíais": "raspar",
    "rasparíamos": "raspar",
    "rasparían": "raspar",
    "rasparías": "raspar",
    "raspas": "raspar",
    "raspase": "raspar",
    "raspaseis": "raspar",
    "raspasen": "raspar",
    "raspases": "raspar",
    "raspaste": "raspar",
    "raspasteis": "raspar",
    "raspe": "raspar",
    "raspea": "raspear",
    "raspeaba": "raspear",
    "raspeabais": "raspear",
    "raspeaban": "raspear",
    "raspeabas": "raspear",
    "raspead": "raspear",
    "raspeada": "raspear",
    "raspeadas": "raspear",
    "raspeado": "raspear",
    "raspeados": "raspear",
    "raspeamos": "raspear",
    "raspean": "raspear",
    "raspeando": "raspear",
    "raspeara": "raspear",
    "raspearais": "raspear",
    "raspearan": "raspear",
    "raspearas": "raspear",
    "raspeare": "raspear",
    "raspeareis": "raspear",
    "raspearemos": "raspear",
    "raspearen": "raspear",
    "raspeares": "raspear",
    "raspearon": "raspear",
    "raspeará": "raspear",
    "raspearán": "raspear",
    "raspearás": "raspear",
    "raspearé": "raspear",
    "raspearéis": "raspear",
    "raspearía": "raspear",
    "raspearíais": "raspear",
    "raspearíamos": "raspear",
    "raspearían": "raspear",
    "raspearías": "raspear",
    "raspeas": "raspear",
    "raspease": "raspear",
    "raspeaseis": "raspear",
    "raspeasen": "raspear",
    "raspeases": "raspear",
    "raspeaste": "raspear",
    "raspeasteis": "raspear",
    "raspee": "raspear",
    "raspeemos": "raspear",
    "raspeen": "raspear",
    "raspees": "raspear",
    "raspemos": "raspar",
    "raspen": "raspar",
    "raspeo": "raspear",
    "raspes": "raspar",
    "raspeábamos": "raspear",
    "raspeáis": "raspear",
    "raspeáramos": "raspear",
    "raspeáremos": "raspear",
    "raspeásemos": "raspear",
    "raspeé": "raspear",
    "raspeéis": "raspear",
    "raspeó": "raspear",
    "raspillas": "raspilla",
    "raspo": "raspar",
    "rasponazos": "rasponazo",
    "rasposa": "rasposo",
    "rasposas": "rasposo",
    "rasposos": "rasposo",
    "raspuda": "raspudo",
    "raspudas": "raspudo",
    "raspudos": "raspudo",
    "raspábamos": "raspar",
    "raspáis": "raspar",
    "raspáramos": "raspar",
    "raspáremos": "raspar",
    "raspásemos": "raspar",
    "raspé": "raspar",
    "raspéis": "raspar",
    "raspó": "raspar",
    "rasque": "rascar",
    "rasquemos": "rascar",
    "rasquen": "rascar",
    "rasques": "rascar",
    "rasquetas": "rasqueta",
    "rasquetea": "rasquetear",
    "rasqueteaba": "rasquetear",
    "rasqueteabais": "rasquetear",
    "rasqueteaban": "rasquetear",
    "rasqueteabas": "rasquetear",
    "rasquetead": "rasquetear",
    "rasqueteada": "rasquetear",
    "rasqueteadas": "rasquetear",
    "rasqueteado": "rasquetear",
    "rasqueteados": "rasquetear",
    "rasqueteamos": "rasquetear",
    "rasquetean": "rasquetear",
    "rasqueteando": "rasquetear",
    "rasqueteara": "rasquetear",
    "rasquetearais": "rasquetear",
    "rasquetearan": "rasquetear",
    "rasquetearas": "rasquetear",
    "rasqueteare": "rasquetear",
    "rasqueteareis": "rasquetear",
    "rasquetearemos": "rasquetear",
    "rasquetearen": "rasquetear",
    "rasqueteares": "rasquetear",
    "rasquetearon": "rasquetear",
    "rasqueteará": "rasquetear",
    "rasquetearán": "rasquetear",
    "rasquetearás": "rasquetear",
    "rasquetearé": "rasquetear",
    "rasquetearéis": "rasquetear",
    "rasquetearía": "rasquetear",
    "rasquetearíais": "rasquetear",
    "rasquetearíamos": "rasquetear",
    "rasquetearían": "rasquetear",
    "rasquetearías": "rasquetear",
    "rasqueteas": "rasquetear",
    "rasquetease": "rasquetear",
    "rasqueteaseis": "rasquetear",
    "rasqueteasen": "rasquetear",
    "rasqueteases": "rasquetear",
    "rasqueteaste": "rasquetear",
    "rasqueteasteis": "rasquetear",
    "rasquetee": "rasquetear",
    "rasqueteemos": "rasquetear",
    "rasqueteen": "rasquetear",
    "rasquetees": "rasquetear",
    "rasqueteo": "rasquetear",
    "rasquetes": "rasquete",
    "rasqueteábamos": "rasquetear",
    "rasqueteáis": "rasquetear",
    "rasqueteáramos": "rasquetear",
    "rasqueteáremos": "rasquetear",
    "rasqueteásemos": "rasquetear",
    "rasqueteé": "rasquetear",
    "rasqueteéis": "rasquetear",
    "rasqueteó": "rasquetear",
    "rasqué": "rascar",
    "rasquéis": "rascar",
    "rasteles": "rastel",
    "rasteres": "raster",
    "rastralla": "rastrallar",
    "rastrallaba": "rastrallar",
    "rastrallabais": "rastrallar",
    "rastrallaban": "rastrallar",
    "rastrallabas": "rastrallar",
    "rastrallad": "rastrallar",
    "rastrallada": "rastrallar",
    "rastralladas": "rastrallar",
    "rastrallado": "rastrallar",
    "rastrallados": "rastrallar",
    "rastrallamos": "rastrallar",
    "rastrallan": "rastrallar",
    "rastrallando": "rastrallar",
    "rastrallara": "rastrallar",
    "rastrallarais": "rastrallar",
    "rastrallaran": "rastrallar",
    "rastrallaras": "rastrallar",
    "rastrallare": "rastrallar",
    "rastrallareis": "rastrallar",
    "rastrallaremos": "rastrallar",
    "rastrallaren": "rastrallar",
    "rastrallares": "rastrallar",
    "rastrallaron": "rastrallar",
    "rastrallará": "rastrallar",
    "rastrallarán": "rastrallar",
    "rastrallarás": "rastrallar",
    "rastrallaré": "rastrallar",
    "rastrallaréis": "rastrallar",
    "rastrallaría": "rastrallar",
    "rastrallaríais": "rastrallar",
    "rastrallaríamos": "rastrallar",
    "rastrallarían": "rastrallar",
    "rastrallarías": "rastrallar",
    "rastrallas": "rastrallar",
    "rastrallase": "rastrallar",
    "rastrallaseis": "rastrallar",
    "rastrallasen": "rastrallar",
    "rastrallases": "rastrallar",
    "rastrallaste": "rastrallar",
    "rastrallasteis": "rastrallar",
    "rastralle": "rastrallar",
    "rastrallemos": "rastrallar",
    "rastrallen": "rastrallar",
    "rastralles": "rastrallar",
    "rastrallo": "rastrallar",
    "rastrallábamos": "rastrallar",
    "rastralláis": "rastrallar",
    "rastralláramos": "rastrallar",
    "rastralláremos": "rastrallar",
    "rastrallásemos": "rastrallar",
    "rastrallé": "rastrallar",
    "rastralléis": "rastrallar",
    "rastralló": "rastrallar",
    "rastras": "rastra",
    "rastrea": "rastrear",
    "rastreaba": "rastrear",
    "rastreabais": "rastrear",
    "rastreaban": "rastrear",
    "rastreabas": "rastrear",
    "rastreabilidades": "rastreabilidad",
    "rastreables": "rastreable",
    "rastread": "rastrear",
    "rastreada": "rastrear",
    "rastreadas": "rastrear",
    "rastreado": "rastrear",
    "rastreadora": "rastreador",
    "rastreadoras": "rastreador",
    "rastreadores": "rastreador",
    "rastreados": "rastrear",
    "rastreamos": "rastrear",
    "rastrean": "rastrear",
    "rastreando": "rastrear",
    "rastreara": "rastrear",
    "rastrearais": "rastrear",
    "rastrearan": "rastrear",
    "rastrearas": "rastrear",
    "rastreare": "rastrear",
    "rastreareis": "rastrear",
    "rastrearemos": "rastrear",
    "rastrearen": "rastrear",
    "rastreares": "rastrear",
    "rastrearon": "rastrear",
    "rastreará": "rastrear",
    "rastrearán": "rastrear",
    "rastrearás": "rastrear",
    "rastrearé": "rastrear",
    "rastrearéis": "rastrear",
    "rastrearía": "rastrear",
    "rastrearíais": "rastrear",
    "rastrearíamos": "rastrear",
    "rastrearían": "rastrear",
    "rastrearías": "rastrear",
    "rastreas": "rastrear",
    "rastrease": "rastrear",
    "rastreaseis": "rastrear",
    "rastreasen": "rastrear",
    "rastreases": "rastrear",
    "rastreaste": "rastrear",
    "rastreasteis": "rastrear",
    "rastree": "rastrear",
    "rastreemos": "rastrear",
    "rastreen": "rastrear",
    "rastrees": "rastrear",
    "rastreles": "rastrel",
    "rastreo": "rastrear",
    "rastreos": "rastreo",
    "rastrera": "rastrero",
    "rastreras": "rastrero",
    "rastreros": "rastrero",
    "rastreábamos": "rastrear",
    "rastreáis": "rastrear",
    "rastreáramos": "rastrear",
    "rastreáremos": "rastrear",
    "rastreásemos": "rastrear",
    "rastreé": "rastrear",
    "rastreéis": "rastrear",
    "rastreó": "rastrear",
    "rastrilla": "rastrillar",
    "rastrillaba": "rastrillar",
    "rastrillabais": "rastrillar",
    "rastrillaban": "rastrillar",
    "rastrillabas": "rastrillar",
    "rastrillad": "rastrillar",
    "rastrillada": "rastrillar",
    "rastrilladas": "rastrillar",
    "rastrillado": "rastrillar",
    "rastrilladora": "rastrillador",
    "rastrilladoras": "rastrillador",
    "rastrilladores": "rastrillador",
    "rastrillados": "rastrillar",
    "rastrillajes": "rastrillaje",
    "rastrillamos": "rastrillar",
    "rastrillan": "rastrillar",
    "rastrillando": "rastrillar",
    "rastrillara": "rastrillar",
    "rastrillarais": "rastrillar",
    "rastrillaran": "rastrillar",
    "rastrillaras": "rastrillar",
    "rastrillare": "rastrillar",
    "rastrillareis": "rastrillar",
    "rastrillaremos": "rastrillar",
    "rastrillaren": "rastrillar",
    "rastrillares": "rastrillar",
    "rastrillaron": "rastrillar",
    "rastrillará": "rastrillar",
    "rastrillarán": "rastrillar",
    "rastrillarás": "rastrillar",
    "rastrillaré": "rastrillar",
    "rastrillaréis": "rastrillar",
    "rastrillaría": "rastrillar",
    "rastrillaríais": "rastrillar",
    "rastrillaríamos": "rastrillar",
    "rastrillarían": "rastrillar",
    "rastrillarías": "rastrillar",
    "rastrillas": "rastrillar",
    "rastrillase": "rastrillar",
    "rastrillaseis": "rastrillar",
    "rastrillasen": "rastrillar",
    "rastrillases": "rastrillar",
    "rastrillaste": "rastrillar",
    "rastrillasteis": "rastrillar",
    "rastrille": "rastrillar",
    "rastrillemos": "rastrillar",
    "rastrillen": "rastrillar",
    "rastrilles": "rastrillar",
    "rastrillo": "rastrillar",
    "rastrillos": "rastrillo",
    "rastrillábamos": "rastrillar",
    "rastrilláis": "rastrillar",
    "rastrilláramos": "rastrillar",
    "rastrilláremos": "rastrillar",
    "rastrillásemos": "rastrillar",
    "rastrillé": "rastrillar",
    "rastrilléis": "rastrillar",
    "rastrilló": "rastrillar",
    "rastroja": "rastrojar",
    "rastrojaba": "rastrojar",
    "rastrojabais": "rastrojar",
    "rastrojaban": "rastrojar",
    "rastrojabas": "rastrojar",
    "rastrojad": "rastrojar",
    "rastrojada": "rastrojar",
    "rastrojadas": "rastrojar",
    "rastrojado": "rastrojar",
    "rastrojados": "rastrojar",
    "rastrojales": "rastrojal",
    "rastrojamos": "rastrojar",
    "rastrojan": "rastrojar",
    "rastrojando": "rastrojar",
    "rastrojara": "rastrojar",
    "rastrojarais": "rastrojar",
    "rastrojaran": "rastrojar",
    "rastrojaras": "rastrojar",
    "rastrojare": "rastrojar",
    "rastrojareis": "rastrojar",
    "rastrojaremos": "rastrojar",
    "rastrojaren": "rastrojar",
    "rastrojares": "rastrojar",
    "rastrojaron": "rastrojar",
    "rastrojará": "rastrojar",
    "rastrojarán": "rastrojar",
    "rastrojarás": "rastrojar",
    "rastrojaré": "rastrojar",
    "rastrojaréis": "rastrojar",
    "rastrojaría": "rastrojar",
    "rastrojaríais": "rastrojar",
    "rastrojaríamos": "rastrojar",
    "rastrojarían": "rastrojar",
    "rastrojarías": "rastrojar",
    "rastrojas": "rastrojar",
    "rastrojase": "rastrojar",
    "rastrojaseis": "rastrojar",
    "rastrojasen": "rastrojar",
    "rastrojases": "rastrojar",
    "rastrojaste": "rastrojar",
    "rastrojasteis": "rastrojar",
    "rastroje": "rastrojar",
    "rastrojemos": "rastrojar",
    "rastrojen": "rastrojar",
    "rastrojeras": "rastrojera",
    "rastrojes": "rastrojar",
    "rastrojo": "rastrojar",
    "rastrojos": "rastrojo",
    "rastrojábamos": "rastrojar",
    "rastrojáis": "rastrojar",
    "rastrojáramos": "rastrojar",
    "rastrojáremos": "rastrojar",
    "rastrojásemos": "rastrojar",
    "rastrojé": "rastrojar",
    "rastrojéis": "rastrojar",
    "rastrojó": "rastrojar",
    "rastros": "rastro",
    "rasura": "rasurar",
    "rasuraba": "rasurar",
    "rasurabais": "rasurar",
    "rasuraban": "rasurar",
    "rasurabas": "rasurar",
    "rasuraciones": "rasuración",
    "rasurad": "rasurar",
    "rasurada": "rasurar",
    "rasuradas": "rasurar",
    "rasurado": "rasurar",
    "rasurados": "rasurar",
    "rasuramos": "rasurar",
    "rasuran": "rasurar",
    "rasurando": "rasurar",
    "rasurara": "rasurar",
    "rasurarais": "rasurar",
    "rasuraran": "rasurar",
    "rasuraras": "rasurar",
    "rasurare": "rasurar",
    "rasurareis": "rasurar",
    "rasuraremos": "rasurar",
    "rasuraren": "rasurar",
    "rasurares": "rasurar",
    "rasuraron": "rasurar",
    "rasurará": "rasurar",
    "rasurarán": "rasurar",
    "rasurarás": "rasurar",
    "rasuraré": "rasurar",
    "rasuraréis": "rasurar",
    "rasuraría": "rasurar",
    "rasuraríais": "rasurar",
    "rasuraríamos": "rasurar",
    "rasurarían": "rasurar",
    "rasurarías": "rasurar",
    "rasuras": "rasurar",
    "rasurase": "rasurar",
    "rasuraseis": "rasurar",
    "rasurasen": "rasurar",
    "rasurases": "rasurar",
    "rasuraste": "rasurar",
    "rasurasteis": "rasurar",
    "rasure": "rasurar",
    "rasuremos": "rasurar",
    "rasuren": "rasurar",
    "rasures": "rasurar",
    "rasuro": "rasurar",
    "rasurábamos": "rasurar",
    "rasuráis": "rasurar",
    "rasuráramos": "rasurar",
    "rasuráremos": "rasurar",
    "rasurásemos": "rasurar",
    "rasuré": "rasurar",
    "rasuréis": "rasurar",
    "rasuró": "rasurar",
    "rasábamos": "rasar",
    "rasáis": "rasar",
    "rasáramos": "rasar",
    "rasáremos": "rasar",
    "rasásemos": "rasar",
    "rasé": "rasar",
    "raséis": "rasar",
    "rasó": "rasar",
    "ratafías": "ratafía",
    "ratanias": "ratania",
    "rataplanes": "rataplán",
    "ratas": "rata",
    "ratea": "ratear",
    "rateaba": "ratear",
    "rateabais": "ratear",
    "rateaban": "ratear",
    "rateabas": "ratear",
    "ratead": "ratear",
    "rateada": "ratear",
    "rateadas": "ratear",
    "rateado": "ratear",
    "rateados": "ratear",
    "rateamos": "ratear",
    "ratean": "ratear",
    "rateando": "ratear",
    "rateara": "ratear",
    "ratearais": "ratear",
    "ratearan": "ratear",
    "ratearas": "ratear",
    "rateare": "ratear",
    "rateareis": "ratear",
    "ratearemos": "ratear",
    "ratearen": "ratear",
    "rateares": "ratear",
    "ratearon": "ratear",
    "rateará": "ratear",
    "ratearán": "ratear",
    "ratearás": "ratear",
    "ratearé": "ratear",
    "ratearéis": "ratear",
    "ratearía": "ratear",
    "ratearíais": "ratear",
    "ratearíamos": "ratear",
    "ratearían": "ratear",
    "ratearías": "ratear",
    "rateas": "ratear",
    "ratease": "ratear",
    "rateaseis": "ratear",
    "rateasen": "ratear",
    "rateases": "ratear",
    "rateaste": "ratear",
    "rateasteis": "ratear",
    "ratee": "ratear",
    "rateemos": "ratear",
    "rateen": "ratear",
    "ratees": "ratear",
    "rateles": "ratel",
    "rateo": "ratear",
    "ratera": "ratero",
    "rateras": "ratero",
    "rateros": "ratero",
    "raterías": "ratería",
    "rateábamos": "ratear",
    "rateáis": "ratear",
    "rateáramos": "ratear",
    "rateáremos": "ratear",
    "rateásemos": "ratear",
    "rateé": "ratear",
    "rateéis": "ratear",
    "rateó": "ratear",
    "raticidas": "raticida",
    "ratifica": "ratificar",
    "ratificaba": "ratificar",
    "ratificabais": "ratificar",
    "ratificaban": "ratificar",
    "ratificabas": "ratificar",
    "ratificaciones": "ratificación",
    "ratificad": "ratificar",
    "ratificada": "ratificar",
    "ratificadas": "ratificar",
    "ratificado": "ratificar",
    "ratificados": "ratificar",
    "ratificamos": "ratificar",
    "ratifican": "ratificar",
    "ratificando": "ratificar",
    "ratificara": "ratificar",
    "ratificarais": "ratificar",
    "ratificaran": "ratificar",
    "ratificaras": "ratificar",
    "ratificare": "ratificar",
    "ratificareis": "ratificar",
    "ratificaremos": "ratificar",
    "ratificaren": "ratificar",
    "ratificares": "ratificar",
    "ratificaron": "ratificar",
    "ratificará": "ratificar",
    "ratificarán": "ratificar",
    "ratificarás": "ratificar",
    "ratificaré": "ratificar",
    "ratificaréis": "ratificar",
    "ratificaría": "ratificar",
    "ratificaríais": "ratificar",
    "ratificaríamos": "ratificar",
    "ratificarían": "ratificar",
    "ratificarías": "ratificar",
    "ratificas": "ratificar",
    "ratificase": "ratificar",
    "ratificaseis": "ratificar",
    "ratificasen": "ratificar",
    "ratificases": "ratificar",
    "ratificaste": "ratificar",
    "ratificasteis": "ratificar",
    "ratificatoria": "ratificatorio",
    "ratificatorias": "ratificatorio",
    "ratificatorios": "ratificatorio",
    "ratifico": "ratificar",
    "ratificábamos": "ratificar",
    "ratificáis": "ratificar",
    "ratificáramos": "ratificar",
    "ratificáremos": "ratificar",
    "ratificásemos": "ratificar",
    "ratificó": "ratificar",
    "ratifique": "ratificar",
    "ratifiquemos": "ratificar",
    "ratifiquen": "ratificar",
    "ratifiques": "ratificar",
    "ratifiqué": "ratificar",
    "ratifiquéis": "ratificar",
    "ratihabiciones": "ratihabición",
    "ratinados": "ratinado",
    "ratinas": "ratina",
    "ratings": "rating",
    "ratios": "ratio",
    "ratonas": "ratona",
    "ratoncillos": "ratoncillo",
    "ratonera": "ratonero",
    "ratoneras": "ratonero",
    "ratoneros": "ratonero",
    "ratones": "ratón",
    "ratonesca": "ratonesco",
    "ratonescas": "ratonesco",
    "ratonescos": "ratonesco",
    "ratoniles": "ratonil",
    "ratos": "rato",
    "ratímetros": "ratímetro",
    "rauca": "rauco",
    "raucas": "rauco",
    "raucos": "rauco",
    "rauda": "raudo",
    "raudales": "raudal",
    "raudas": "raudo",
    "raudos": "raudo",
    "raulíes": "raulí",
    "raulís": "raulí",
    "rauwolfias": "rauwolfia",
    "raveles": "ravel",
    "raya": "rayar",
    "rayaba": "rayar",
    "rayabais": "rayar",
    "rayaban": "rayar",
    "rayabas": "rayar",
    "rayad": "rayar",
    "rayada": "rayar",
    "rayadas": "rayar",
    "rayadillos": "rayadillo",
    "rayado": "rayar",
    "rayadoras": "rayadora",
    "rayadors": "rayador",
    "rayados": "rayar",
    "rayaduras": "rayadura",
    "rayamos": "rayar",
    "rayan": "rayar",
    "rayana": "rayano",
    "rayanas": "rayano",
    "rayando": "rayar",
    "rayanos": "rayano",
    "rayara": "rayar",
    "rayarais": "rayar",
    "rayaran": "rayar",
    "rayaras": "rayar",
    "rayare": "rayar",
    "rayareis": "rayar",
    "rayaremos": "rayar",
    "rayaren": "rayar",
    "rayares": "rayar",
    "rayaron": "rayar",
    "rayará": "rayar",
    "rayarán": "rayar",
    "rayarás": "rayar",
    "rayaré": "rayar",
    "rayaréis": "rayar",
    "rayaría": "rayar",
    "rayaríais": "rayar",
    "rayaríamos": "rayar",
    "rayarían": "rayar",
    "rayarías": "rayar",
    "rayas": "rayar",
    "rayase": "rayar",
    "rayaseis": "rayar",
    "rayasen": "rayar",
    "rayases": "rayar",
    "rayaste": "rayar",
    "rayasteis": "rayar",
    "raye": "rayar",
    "rayemos": "rayar",
    "rayen": "rayar",
    "rayendo": "raer",
    "rayera": "raer",
    "rayerais": "raer",
    "rayeran": "raer",
    "rayeras": "raer",
    "rayere": "raer",
    "rayereis": "raer",
    "rayeren": "raer",
    "rayeres": "raer",
    "rayeron": "raer",
    "rayes": "rayar",
    "rayese": "raer",
    "rayeseis": "raer",
    "rayesen": "raer",
    "rayeses": "raer",
    "rayo": "rayar",
    "rayones": "rayón",
    "rayos": "rayo",
    "rayosa": "rayoso",
    "rayosas": "rayoso",
    "rayosos": "rayoso",
    "rayuelas": "rayuela",
    "rayábamos": "rayar",
    "rayáis": "rayar",
    "rayáramos": "rayar",
    "rayáremos": "rayar",
    "rayásemos": "rayar",
    "rayé": "rayar",
    "rayéis": "rayar",
    "rayéramos": "raer",
    "rayéremos": "raer",
    "rayésemos": "raer",
    "rayó": "rayar",
    "razas": "raza",
    "razias": "razia",
    "razona": "razonar",
    "razonaba": "razonar",
    "razonabais": "razonar",
    "razonaban": "razonar",
    "razonabas": "razonar",
    "razonables": "razonable",
    "razonad": "razonar",
    "razonada": "razonar",
    "razonadas": "razonar",
    "razonado": "razonar",
    "razonadora": "razonador",
    "razonadoras": "razonador",
    "razonadores": "razonador",
    "razonados": "razonar",
    "razonamientos": "razonamiento",
    "razonamos": "razonar",
    "razonan": "razonar",
    "razonando": "razonar",
    "razonara": "razonar",
    "razonarais": "razonar",
    "razonaran": "razonar",
    "razonaras": "razonar",
    "razonare": "razonar",
    "razonareis": "razonar",
    "razonaremos": "razonar",
    "razonaren": "razonar",
    "razonares": "razonar",
    "razonaron": "razonar",
    "razonará": "razonar",
    "razonarán": "razonar",
    "razonarás": "razonar",
    "razonaré": "razonar",
    "razonaréis": "razonar",
    "razonaría": "razonar",
    "razonaríais": "razonar",
    "razonaríamos": "razonar",
    "razonarían": "razonar",
    "razonarías": "razonar",
    "razonas": "razonar",
    "razonase": "razonar",
    "razonaseis": "razonar",
    "razonasen": "razonar",
    "razonases": "razonar",
    "razonaste": "razonar",
    "razonasteis": "razonar",
    "razone": "razonar",
    "razonemos": "razonar",
    "razonen": "razonar",
    "razones": "razonar",
    "razono": "razonar",
    "razonábamos": "razonar",
    "razonáis": "razonar",
    "razonáramos": "razonar",
    "razonáremos": "razonar",
    "razonásemos": "razonar",
    "razoné": "razonar",
    "razonéis": "razonar",
    "razonó": "razonar",
    "razzias": "razzia",
    "raéis": "raer",
    "raí": "raer",
    "raía": "raer",
    "raíais": "raer",
    "raíamos": "raer",
    "raían": "raer",
    "raías": "raer",
    "raíbles": "raíble",
    "raíces": "raíz",
    "raída": "raer",
    "raídas": "raer",
    "raído": "raer",
    "raídos": "raer",
    "raíles": "raíl",
    "raímos": "raer",
    "raíste": "raer",
    "raísteis": "raer",
    "rañas": "raña",
    "raños": "raño",
    "rea": "reo",
    "reabierta": "reabrir",
    "reabiertas": "reabrir",
    "reabierto": "reabrir",
    "reabiertos": "reabrir",
    "reabra": "reabrir",
    "reabramos": "reabrir",
    "reabran": "reabrir",
    "reabras": "reabrir",
    "reabre": "reabrir",
    "reabren": "reabrir",
    "reabres": "reabrir",
    "reabrid": "reabrir",
    "reabriendo": "reabrir",
    "reabriera": "reabrir",
    "reabrierais": "reabrir",
    "reabrieran": "reabrir",
    "reabrieras": "reabrir",
    "reabriere": "reabrir",
    "reabriereis": "reabrir",
    "reabrieren": "reabrir",
    "reabrieres": "reabrir",
    "reabrieron": "reabrir",
    "reabriese": "reabrir",
    "reabrieseis": "reabrir",
    "reabriesen": "reabrir",
    "reabrieses": "reabrir",
    "reabrimos": "reabrir",
    "reabriremos": "reabrir",
    "reabrirá": "reabrir",
    "reabrirán": "reabrir",
    "reabrirás": "reabrir",
    "reabriré": "reabrir",
    "reabriréis": "reabrir",
    "reabriría": "reabrir",
    "reabriríais": "reabrir",
    "reabriríamos": "reabrir",
    "reabrirían": "reabrir",
    "reabrirías": "reabrir",
    "reabriste": "reabrir",
    "reabristeis": "reabrir",
    "reabriéramos": "reabrir",
    "reabriéremos": "reabrir",
    "reabriésemos": "reabrir",
    "reabrió": "reabrir",
    "reabro": "reabrir",
    "reabráis": "reabrir",
    "reabrí": "reabrir",
    "reabría": "reabrir",
    "reabríais": "reabrir",
    "reabríamos": "reabrir",
    "reabrían": "reabrir",
    "reabrías": "reabrir",
    "reabrís": "reabrir",
    "reabsorba": "reabsorber",
    "reabsorbamos": "reabsorber",
    "reabsorban": "reabsorber",
    "reabsorbas": "reabsorber",
    "reabsorbe": "reabsorber",
    "reabsorbed": "reabsorber",
    "reabsorbemos": "reabsorber",
    "reabsorben": "reabsorber",
    "reabsorberemos": "reabsorber",
    "reabsorberá": "reabsorber",
    "reabsorberán": "reabsorber",
    "reabsorberás": "reabsorber",
    "reabsorberé": "reabsorber",
    "reabsorberéis": "reabsorber",
    "reabsorbería": "reabsorber",
    "reabsorberíais": "reabsorber",
    "reabsorberíamos": "reabsorber",
    "reabsorberían": "reabsorber",
    "reabsorberías": "reabsorber",
    "reabsorbes": "reabsorber",
    "reabsorbida": "reabsorber",
    "reabsorbidas": "reabsorber",
    "reabsorbido": "reabsorber",
    "reabsorbidos": "reabsorber",
    "reabsorbiendo": "reabsorber",
    "reabsorbiera": "reabsorber",
    "reabsorbierais": "reabsorber",
    "reabsorbieran": "reabsorber",
    "reabsorbieras": "reabsorber",
    "reabsorbiere": "reabsorber",
    "reabsorbiereis": "reabsorber",
    "reabsorbieren": "reabsorber",
    "reabsorbieres": "reabsorber",
    "reabsorbieron": "reabsorber",
    "reabsorbiese": "reabsorber",
    "reabsorbieseis": "reabsorber",
    "reabsorbiesen": "reabsorber",
    "reabsorbieses": "reabsorber",
    "reabsorbimos": "reabsorber",
    "reabsorbiste": "reabsorber",
    "reabsorbisteis": "reabsorber",
    "reabsorbiéramos": "reabsorber",
    "reabsorbiéremos": "reabsorber",
    "reabsorbiésemos": "reabsorber",
    "reabsorbió": "reabsorber",
    "reabsorbo": "reabsorber",
    "reabsorbáis": "reabsorber",
    "reabsorbéis": "reabsorber",
    "reabsorbí": "reabsorber",
    "reabsorbía": "reabsorber",
    "reabsorbíais": "reabsorber",
    "reabsorbíamos": "reabsorber",
    "reabsorbían": "reabsorber",
    "reabsorbías": "reabsorber",
    "reabsorciones": "reabsorción",
    "reacciona": "reaccionar",
    "reaccionaba": "reaccionar",
    "reaccionabais": "reaccionar",
    "reaccionaban": "reaccionar",
    "reaccionabas": "reaccionar",
    "reaccionad": "reaccionar",
    "reaccionada": "reaccionar",
    "reaccionadas": "reaccionar",
    "reaccionado": "reaccionar",
    "reaccionados": "reaccionar",
    "reaccionamos": "reaccionar",
    "reaccionan": "reaccionar",
    "reaccionando": "reaccionar",
    "reaccionara": "reaccionar",
    "reaccionarais": "reaccionar",
    "reaccionaran": "reaccionar",
    "reaccionaras": "reaccionar",
    "reaccionare": "reaccionar",
    "reaccionareis": "reaccionar",
    "reaccionaremos": "reaccionar",
    "reaccionaren": "reaccionar",
    "reaccionares": "reaccionar",
    "reaccionaria": "reaccionario",
    "reaccionarias": "reaccionario",
    "reaccionarios": "reaccionario",
    "reaccionaron": "reaccionar",
    "reaccionará": "reaccionar",
    "reaccionarán": "reaccionar",
    "reaccionarás": "reaccionar",
    "reaccionaré": "reaccionar",
    "reaccionaréis": "reaccionar",
    "reaccionaría": "reaccionar",
    "reaccionaríais": "reaccionar",
    "reaccionaríamos": "reaccionar",
    "reaccionarían": "reaccionar",
    "reaccionarías": "reaccionar",
    "reaccionas": "reaccionar",
    "reaccionase": "reaccionar",
    "reaccionaseis": "reaccionar",
    "reaccionasen": "reaccionar",
    "reaccionases": "reaccionar",
    "reaccionaste": "reaccionar",
    "reaccionasteis": "reaccionar",
    "reaccione": "reaccionar",
    "reaccionemos": "reaccionar",
    "reaccionen": "reaccionar",
    "reacciones": "reaccionar",
    "reacciono": "reaccionar",
    "reaccionábamos": "reaccionar",
    "reaccionáis": "reaccionar",
    "reaccionáramos": "reaccionar",
    "reaccionáremos": "reaccionar",
    "reaccionásemos": "reaccionar",
    "reaccioné": "reaccionar",
    "reaccionéis": "reaccionar",
    "reaccionó": "reaccionar",
    "reaceleraciones": "reaceleración",
    "reaceptaciones": "reaceptación",
    "reacia": "reacio",
    "reacias": "reacio",
    "reacios": "reacio",
    "reacondicionadores": "reacondicionador",
    "reacondicionamientos": "reacondicionamiento",
    "reactancias": "reactancia",
    "reactiva": "reactivo",
    "reactivaba": "reactivar",
    "reactivabais": "reactivar",
    "reactivaban": "reactivar",
    "reactivabas": "reactivar",
    "reactivables": "reactivable",
    "reactivaciones": "reactivación",
    "reactivad": "reactivar",
    "reactivada": "reactivar",
    "reactivadas": "reactivar",
    "reactivado": "reactivar",
    "reactivados": "reactivar",
    "reactivamos": "reactivar",
    "reactivan": "reactivar",
    "reactivando": "reactivar",
    "reactivara": "reactivar",
    "reactivarais": "reactivar",
    "reactivaran": "reactivar",
    "reactivaras": "reactivar",
    "reactivare": "reactivar",
    "reactivareis": "reactivar",
    "reactivaremos": "reactivar",
    "reactivaren": "reactivar",
    "reactivares": "reactivar",
    "reactivaron": "reactivar",
    "reactivará": "reactivar",
    "reactivarán": "reactivar",
    "reactivarás": "reactivar",
    "reactivaré": "reactivar",
    "reactivaréis": "reactivar",
    "reactivaría": "reactivar",
    "reactivaríais": "reactivar",
    "reactivaríamos": "reactivar",
    "reactivarían": "reactivar",
    "reactivarías": "reactivar",
    "reactivas": "reactivo",
    "reactivase": "reactivar",
    "reactivaseis": "reactivar",
    "reactivasen": "reactivar",
    "reactivases": "reactivar",
    "reactivaste": "reactivar",
    "reactivasteis": "reactivar",
    "reactive": "reactivar",
    "reactivemos": "reactivar",
    "reactiven": "reactivar",
    "reactives": "reactivar",
    "reactividades": "reactividad",
    "reactivo": "reactivar",
    "reactivos": "reactivo",
    "reactivábamos": "reactivar",
    "reactiváis": "reactivar",
    "reactiváramos": "reactivar",
    "reactiváremos": "reactivar",
    "reactivásemos": "reactivar",
    "reactivé": "reactivar",
    "reactivéis": "reactivar",
    "reactivó": "reactivar",
    "reactores": "reactor",
    "reacuña": "reacuñar",
    "reacuñaba": "reacuñar",
    "reacuñabais": "reacuñar",
    "reacuñaban": "reacuñar",
    "reacuñabas": "reacuñar",
    "reacuñad": "reacuñar",
    "reacuñada": "reacuñar",
    "reacuñadas": "reacuñar",
    "reacuñado": "reacuñar",
    "reacuñados": "reacuñar",
    "reacuñamos": "reacuñar",
    "reacuñan": "reacuñar",
    "reacuñando": "reacuñar",
    "reacuñara": "reacuñar",
    "reacuñarais": "reacuñar",
    "reacuñaran": "reacuñar",
    "reacuñaras": "reacuñar",
    "reacuñare": "reacuñar",
    "reacuñareis": "reacuñar",
    "reacuñaremos": "reacuñar",
    "reacuñaren": "reacuñar",
    "reacuñares": "reacuñar",
    "reacuñaron": "reacuñar",
    "reacuñará": "reacuñar",
    "reacuñarán": "reacuñar",
    "reacuñarás": "reacuñar",
    "reacuñaré": "reacuñar",
    "reacuñaréis": "reacuñar",
    "reacuñaría": "reacuñar",
    "reacuñaríais": "reacuñar",
    "reacuñaríamos": "reacuñar",
    "reacuñarían": "reacuñar",
    "reacuñarías": "reacuñar",
    "reacuñas": "reacuñar",
    "reacuñase": "reacuñar",
    "reacuñaseis": "reacuñar",
    "reacuñasen": "reacuñar",
    "reacuñases": "reacuñar",
    "reacuñaste": "reacuñar",
    "reacuñasteis": "reacuñar",
    "reacuñe": "reacuñar",
    "reacuñemos": "reacuñar",
    "reacuñen": "reacuñar",
    "reacuñes": "reacuñar",
    "reacuño": "reacuñar",
    "reacuñábamos": "reacuñar",
    "reacuñáis": "reacuñar",
    "reacuñáramos": "reacuñar",
    "reacuñáremos": "reacuñar",
    "reacuñásemos": "reacuñar",
    "reacuñé": "reacuñar",
    "reacuñéis": "reacuñar",
    "reacuñó": "reacuñar",
    "readapta": "readaptar",
    "readaptaba": "readaptar",
    "readaptabais": "readaptar",
    "readaptaban": "readaptar",
    "readaptabas": "readaptar",
    "readaptaciones": "readaptación",
    "readaptad": "readaptar",
    "readaptada": "readaptar",
    "readaptadas": "readaptar",
    "readaptado": "readaptar",
    "readaptados": "readaptar",
    "readaptamos": "readaptar",
    "readaptan": "readaptar",
    "readaptando": "readaptar",
    "readaptara": "readaptar",
    "readaptarais": "readaptar",
    "readaptaran": "readaptar",
    "readaptaras": "readaptar",
    "readaptare": "readaptar",
    "readaptareis": "readaptar",
    "readaptaremos": "readaptar",
    "readaptaren": "readaptar",
    "readaptares": "readaptar",
    "readaptaron": "readaptar",
    "readaptará": "readaptar",
    "readaptarán": "readaptar",
    "readaptarás": "readaptar",
    "readaptaré": "readaptar",
    "readaptaréis": "readaptar",
    "readaptaría": "readaptar",
    "readaptaríais": "readaptar",
    "readaptaríamos": "readaptar",
    "readaptarían": "readaptar",
    "readaptarías": "readaptar",
    "readaptas": "readaptar",
    "readaptase": "readaptar",
    "readaptaseis": "readaptar",
    "readaptasen": "readaptar",
    "readaptases": "readaptar",
    "readaptaste": "readaptar",
    "readaptasteis": "readaptar",
    "readapte": "readaptar",
    "readaptemos": "readaptar",
    "readapten": "readaptar",
    "readaptes": "readaptar",
    "readapto": "readaptar",
    "readaptábamos": "readaptar",
    "readaptáis": "readaptar",
    "readaptáramos": "readaptar",
    "readaptáremos": "readaptar",
    "readaptásemos": "readaptar",
    "readapté": "readaptar",
    "readaptéis": "readaptar",
    "readaptó": "readaptar",
    "readmisiones": "readmisión",
    "readmita": "readmitir",
    "readmitamos": "readmitir",
    "readmitan": "readmitir",
    "readmitas": "readmitir",
    "readmite": "readmitir",
    "readmiten": "readmitir",
    "readmites": "readmitir",
    "readmitid": "readmitir",
    "readmitida": "readmitir",
    "readmitidas": "readmitir",
    "readmitido": "readmitir",
    "readmitidos": "readmitir",
    "readmitiendo": "readmitir",
    "readmitiera": "readmitir",
    "readmitierais": "readmitir",
    "readmitieran": "readmitir",
    "readmitieras": "readmitir",
    "readmitiere": "readmitir",
    "readmitiereis": "readmitir",
    "readmitieren": "readmitir",
    "readmitieres": "readmitir",
    "readmitieron": "readmitir",
    "readmitiese": "readmitir",
    "readmitieseis": "readmitir",
    "readmitiesen": "readmitir",
    "readmitieses": "readmitir",
    "readmitimos": "readmitir",
    "readmitiremos": "readmitir",
    "readmitirá": "readmitir",
    "readmitirán": "readmitir",
    "readmitirás": "readmitir",
    "readmitiré": "readmitir",
    "readmitiréis": "readmitir",
    "readmitiría": "readmitir",
    "readmitiríais": "readmitir",
    "readmitiríamos": "readmitir",
    "readmitirían": "readmitir",
    "readmitirías": "readmitir",
    "readmitiste": "readmitir",
    "readmitisteis": "readmitir",
    "readmitiéramos": "readmitir",
    "readmitiéremos": "readmitir",
    "readmitiésemos": "readmitir",
    "readmitió": "readmitir",
    "readmito": "readmitir",
    "readmitáis": "readmitir",
    "readmití": "readmitir",
    "readmitía": "readmitir",
    "readmitíais": "readmitir",
    "readmitíamos": "readmitir",
    "readmitían": "readmitir",
    "readmitías": "readmitir",
    "readmitís": "readmitir",
    "readopta": "readoptar",
    "readoptaba": "readoptar",
    "readoptabais": "readoptar",
    "readoptaban": "readoptar",
    "readoptabas": "readoptar",
    "readoptad": "readoptar",
    "readoptada": "readoptar",
    "readoptadas": "readoptar",
    "readoptado": "readoptar",
    "readoptados": "readoptar",
    "readoptamos": "readoptar",
    "readoptan": "readoptar",
    "readoptando": "readoptar",
    "readoptara": "readoptar",
    "readoptarais": "readoptar",
    "readoptaran": "readoptar",
    "readoptaras": "readoptar",
    "readoptare": "readoptar",
    "readoptareis": "readoptar",
    "readoptaremos": "readoptar",
    "readoptaren": "readoptar",
    "readoptares": "readoptar",
    "readoptaron": "readoptar",
    "readoptará": "readoptar",
    "readoptarán": "readoptar",
    "readoptarás": "readoptar",
    "readoptaré": "readoptar",
    "readoptaréis": "readoptar",
    "readoptaría": "readoptar",
    "readoptaríais": "readoptar",
    "readoptaríamos": "readoptar",
    "readoptarían": "readoptar",
    "readoptarías": "readoptar",
    "readoptas": "readoptar",
    "readoptase": "readoptar",
    "readoptaseis": "readoptar",
    "readoptasen": "readoptar",
    "readoptases": "readoptar",
    "readoptaste": "readoptar",
    "readoptasteis": "readoptar",
    "readopte": "readoptar",
    "readoptemos": "readoptar",
    "readopten": "readoptar",
    "readoptes": "readoptar",
    "readopto": "readoptar",
    "readoptábamos": "readoptar",
    "readoptáis": "readoptar",
    "readoptáramos": "readoptar",
    "readoptáremos": "readoptar",
    "readoptásemos": "readoptar",
    "readopté": "readoptar",
    "readoptéis": "readoptar",
    "readoptó": "readoptar",
    "readquiera": "readquirir",
    "readquieran": "readquirir",
    "readquieras": "readquirir",
    "readquiere": "readquirir",
    "readquieren": "readquirir",
    "readquieres": "readquirir",
    "readquiero": "readquirir",
    "readquiramos": "readquirir",
    "readquirid": "readquirir",
    "readquirida": "readquirir",
    "readquiridas": "readquirir",
    "readquirido": "readquirir",
    "readquiridos": "readquirir",
    "readquiriendo": "readquirir",
    "readquiriera": "readquirir",
    "readquirierais": "readquirir",
    "readquirieran": "readquirir",
    "readquirieras": "readquirir",
    "readquiriere": "readquirir",
    "readquiriereis": "readquirir",
    "readquirieren": "readquirir",
    "readquirieres": "readquirir",
    "readquirieron": "readquirir",
    "readquiriese": "readquirir",
    "readquirieseis": "readquirir",
    "readquiriesen": "readquirir",
    "readquirieses": "readquirir",
    "readquirimos": "readquirir",
    "readquiriremos": "readquirir",
    "readquirirá": "readquirir",
    "readquirirán": "readquirir",
    "readquirirás": "readquirir",
    "readquiriré": "readquirir",
    "readquiriréis": "readquirir",
    "readquiriría": "readquirir",
    "readquiriríais": "readquirir",
    "readquiriríamos": "readquirir",
    "readquirirían": "readquirir",
    "readquirirías": "readquirir",
    "readquiriste": "readquirir",
    "readquiristeis": "readquirir",
    "readquiriéramos": "readquirir",
    "readquiriéremos": "readquirir",
    "readquiriésemos": "readquirir",
    "readquirió": "readquirir",
    "readquiráis": "readquirir",
    "readquirí": "readquirir",
    "readquiría": "readquirir",
    "readquiríais": "readquirir",
    "readquiríamos": "readquirir",
    "readquirían": "readquirir",
    "readquirías": "readquirir",
    "readquirís": "readquirir",
    "reafirma": "reafirmar",
    "reafirmaba": "reafirmar",
    "reafirmabais": "reafirmar",
    "reafirmaban": "reafirmar",
    "reafirmabas": "reafirmar",
    "reafirmaciones": "reafirmación",
    "reafirmad": "reafirmar",
    "reafirmada": "reafirmar",
    "reafirmadas": "reafirmar",
    "reafirmado": "reafirmar",
    "reafirmados": "reafirmar",
    "reafirmamos": "reafirmar",
    "reafirman": "reafirmar",
    "reafirmando": "reafirmar",
    "reafirmara": "reafirmar",
    "reafirmarais": "reafirmar",
    "reafirmaran": "reafirmar",
    "reafirmaras": "reafirmar",
    "reafirmare": "reafirmar",
    "reafirmareis": "reafirmar",
    "reafirmaremos": "reafirmar",
    "reafirmaren": "reafirmar",
    "reafirmares": "reafirmar",
    "reafirmaron": "reafirmar",
    "reafirmará": "reafirmar",
    "reafirmarán": "reafirmar",
    "reafirmarás": "reafirmar",
    "reafirmaré": "reafirmar",
    "reafirmaréis": "reafirmar",
    "reafirmaría": "reafirmar",
    "reafirmaríais": "reafirmar",
    "reafirmaríamos": "reafirmar",
    "reafirmarían": "reafirmar",
    "reafirmarías": "reafirmar",
    "reafirmas": "reafirmar",
    "reafirmase": "reafirmar",
    "reafirmaseis": "reafirmar",
    "reafirmasen": "reafirmar",
    "reafirmases": "reafirmar",
    "reafirmaste": "reafirmar",
    "reafirmasteis": "reafirmar",
    "reafirme": "reafirmar",
    "reafirmemos": "reafirmar",
    "reafirmen": "reafirmar",
    "reafirmes": "reafirmar",
    "reafirmo": "reafirmar",
    "reafirmábamos": "reafirmar",
    "reafirmáis": "reafirmar",
    "reafirmáramos": "reafirmar",
    "reafirmáremos": "reafirmar",
    "reafirmásemos": "reafirmar",
    "reafirmé": "reafirmar",
    "reafirméis": "reafirmar",
    "reafirmó": "reafirmar",
    "reagalizas": "reagaliza",
    "reaginas": "reagina",
    "reagravaciones": "reagravación",
    "reagrupa": "reagrupar",
    "reagrupaba": "reagrupar",
    "reagrupabais": "reagrupar",
    "reagrupaban": "reagrupar",
    "reagrupabas": "reagrupar",
    "reagrupaciones": "reagrupación",
    "reagrupad": "reagrupar",
    "reagrupada": "reagrupar",
    "reagrupadas": "reagrupar",
    "reagrupado": "reagrupar",
    "reagrupados": "reagrupar",
    "reagrupamientos": "reagrupamiento",
    "reagrupamos": "reagrupar",
    "reagrupan": "reagrupar",
    "reagrupando": "reagrupar",
    "reagrupara": "reagrupar",
    "reagruparais": "reagrupar",
    "reagruparan": "reagrupar",
    "reagruparas": "reagrupar",
    "reagrupare": "reagrupar",
    "reagrupareis": "reagrupar",
    "reagruparemos": "reagrupar",
    "reagruparen": "reagrupar",
    "reagrupares": "reagrupar",
    "reagruparon": "reagrupar",
    "reagrupará": "reagrupar",
    "reagruparán": "reagrupar",
    "reagruparás": "reagrupar",
    "reagruparé": "reagrupar",
    "reagruparéis": "reagrupar",
    "reagruparía": "reagrupar",
    "reagruparíais": "reagrupar",
    "reagruparíamos": "reagrupar",
    "reagruparían": "reagrupar",
    "reagruparías": "reagrupar",
    "reagrupas": "reagrupar",
    "reagrupase": "reagrupar",
    "reagrupaseis": "reagrupar",
    "reagrupasen": "reagrupar",
    "reagrupases": "reagrupar",
    "reagrupaste": "reagrupar",
    "reagrupasteis": "reagrupar",
    "reagrupe": "reagrupar",
    "reagrupemos": "reagrupar",
    "reagrupen": "reagrupar",
    "reagrupes": "reagrupar",
    "reagrupo": "reagrupar",
    "reagrupábamos": "reagrupar",
    "reagrupáis": "reagrupar",
    "reagrupáramos": "reagrupar",
    "reagrupáremos": "reagrupar",
    "reagrupásemos": "reagrupar",
    "reagrupé": "reagrupar",
    "reagrupéis": "reagrupar",
    "reagrupó": "reagrupar",
    "reajusta": "reajustar",
    "reajustaba": "reajustar",
    "reajustabais": "reajustar",
    "reajustaban": "reajustar",
    "reajustabas": "reajustar",
    "reajustables": "reajustable",
    "reajustad": "reajustar",
    "reajustada": "reajustar",
    "reajustadas": "reajustar",
    "reajustado": "reajustar",
    "reajustados": "reajustar",
    "reajustamos": "reajustar",
    "reajustan": "reajustar",
    "reajustando": "reajustar",
    "reajustara": "reajustar",
    "reajustarais": "reajustar",
    "reajustaran": "reajustar",
    "reajustaras": "reajustar",
    "reajustare": "reajustar",
    "reajustareis": "reajustar",
    "reajustaremos": "reajustar",
    "reajustaren": "reajustar",
    "reajustares": "reajustar",
    "reajustaron": "reajustar",
    "reajustará": "reajustar",
    "reajustarán": "reajustar",
    "reajustarás": "reajustar",
    "reajustaré": "reajustar",
    "reajustaréis": "reajustar",
    "reajustaría": "reajustar",
    "reajustaríais": "reajustar",
    "reajustaríamos": "reajustar",
    "reajustarían": "reajustar",
    "reajustarías": "reajustar",
    "reajustas": "reajustar",
    "reajustase": "reajustar",
    "reajustaseis": "reajustar",
    "reajustasen": "reajustar",
    "reajustases": "reajustar",
    "reajustaste": "reajustar",
    "reajustasteis": "reajustar",
    "reajuste": "reajustar",
    "reajustemos": "reajustar",
    "reajusten": "reajustar",
    "reajustes": "reajuste",
    "reajusto": "reajustar",
    "reajustábamos": "reajustar",
    "reajustáis": "reajustar",
    "reajustáramos": "reajustar",
    "reajustáremos": "reajustar",
    "reajustásemos": "reajustar",
    "reajusté": "reajustar",
    "reajustéis": "reajustar",
    "reajustó": "reajustar",
    "realas": "reala",
    "realce": "realzar",
    "realcemos": "realzar",
    "realcen": "realzar",
    "realces": "realzar",
    "realcé": "realzar",
    "realcéis": "realzar",
    "realenga": "realengo",
    "realengas": "realengo",
    "realengos": "realengo",
    "realeras": "realera",
    "reales": "real",
    "realezas": "realeza",
    "realice": "realizar",
    "realicemos": "realizar",
    "realicen": "realizar",
    "realices": "realizar",
    "realicé": "realizar",
    "realicéis": "realizar",
    "realidades": "realidad",
    "realimentaciones": "realimentación",
    "realineamientos": "realineamiento",
    "realismos": "realismo",
    "realistas": "realista",
    "realiza": "realizar",
    "realizaba": "realizar",
    "realizabais": "realizar",
    "realizaban": "realizar",
    "realizabas": "realizar",
    "realizables": "realizable",
    "realizaciones": "realización",
    "realizad": "realizar",
    "realizada": "realizar",
    "realizadas": "realizar",
    "realizado": "realizar",
    "realizadora": "realizador",
    "realizadoras": "realizador",
    "realizadores": "realizador",
    "realizados": "realizar",
    "realizamos": "realizar",
    "realizan": "realizar",
    "realizando": "realizar",
    "realizara": "realizar",
    "realizarais": "realizar",
    "realizaran": "realizar",
    "realizaras": "realizar",
    "realizare": "realizar",
    "realizareis": "realizar",
    "realizaremos": "realizar",
    "realizaren": "realizar",
    "realizares": "realizar",
    "realizaron": "realizar",
    "realizará": "realizar",
    "realizarán": "realizar",
    "realizarás": "realizar",
    "realizaré": "realizar",
    "realizaréis": "realizar",
    "realizaría": "realizar",
    "realizaríais": "realizar",
    "realizaríamos": "realizar",
    "realizarían": "realizar",
    "realizarías": "realizar",
    "realizas": "realizar",
    "realizase": "realizar",
    "realizaseis": "realizar",
    "realizasen": "realizar",
    "realizases": "realizar",
    "realizaste": "realizar",
    "realizasteis": "realizar",
    "realizo": "realizar",
    "realizábamos": "realizar",
    "realizáis": "realizar",
    "realizáramos": "realizar",
    "realizáremos": "realizar",
    "realizásemos": "realizar",
    "realizó": "realizar",
    "realmacenables": "realmacenable",
    "realmacenamientos": "realmacenamiento",
    "realoja": "realojar",
    "realojaba": "realojar",
    "realojabais": "realojar",
    "realojaban": "realojar",
    "realojabas": "realojar",
    "realojad": "realojar",
    "realojada": "realojar",
    "realojadas": "realojar",
    "realojado": "realojar",
    "realojados": "realojar",
    "realojamientos": "realojamiento",
    "realojamos": "realojar",
    "realojan": "realojar",
    "realojando": "realojar",
    "realojara": "realojar",
    "realojarais": "realojar",
    "realojaran": "realojar",
    "realojaras": "realojar",
    "realojare": "realojar",
    "realojareis": "realojar",
    "realojaremos": "realojar",
    "realojaren": "realojar",
    "realojares": "realojar",
    "realojaron": "realojar",
    "realojará": "realojar",
    "realojarán": "realojar",
    "realojarás": "realojar",
    "realojaré": "realojar",
    "realojaréis": "realojar",
    "realojaría": "realojar",
    "realojaríais": "realojar",
    "realojaríamos": "realojar",
    "realojarían": "realojar",
    "realojarías": "realojar",
    "realojas": "realojar",
    "realojase": "realojar",
    "realojaseis": "realojar",
    "realojasen": "realojar",
    "realojases": "realojar",
    "realojaste": "realojar",
    "realojasteis": "realojar",
    "realoje": "realojar",
    "realojemos": "realojar",
    "realojen": "realojar",
    "realojes": "realojar",
    "realojo": "realojar",
    "realojábamos": "realojar",
    "realojáis": "realojar",
    "realojáramos": "realojar",
    "realojáremos": "realojar",
    "realojásemos": "realojar",
    "realojé": "realojar",
    "realojéis": "realojar",
    "realojó": "realojar",
    "realquila": "realquilar",
    "realquilaba": "realquilar",
    "realquilabais": "realquilar",
    "realquilaban": "realquilar",
    "realquilabas": "realquilar",
    "realquilad": "realquilar",
    "realquilada": "realquilar",
    "realquiladas": "realquilar",
    "realquilado": "realquilar",
    "realquilados": "realquilar",
    "realquilamos": "realquilar",
    "realquilan": "realquilar",
    "realquilando": "realquilar",
    "realquilara": "realquilar",
    "realquilarais": "realquilar",
    "realquilaran": "realquilar",
    "realquilaras": "realquilar",
    "realquilare": "realquilar",
    "realquilareis": "realquilar",
    "realquilaremos": "realquilar",
    "realquilaren": "realquilar",
    "realquilares": "realquilar",
    "realquilaron": "realquilar",
    "realquilará": "realquilar",
    "realquilarán": "realquilar",
    "realquilarás": "realquilar",
    "realquilaré": "realquilar",
    "realquilaréis": "realquilar",
    "realquilaría": "realquilar",
    "realquilaríais": "realquilar",
    "realquilaríamos": "realquilar",
    "realquilarían": "realquilar",
    "realquilarías": "realquilar",
    "realquilas": "realquilar",
    "realquilase": "realquilar",
    "realquilaseis": "realquilar",
    "realquilasen": "realquilar",
    "realquilases": "realquilar",
    "realquilaste": "realquilar",
    "realquilasteis": "realquilar",
    "realquile": "realquilar",
    "realquilemos": "realquilar",
    "realquilen": "realquilar",
    "realquiles": "realquilar",
    "realquilo": "realquilar",
    "realquilábamos": "realquilar",
    "realquiláis": "realquilar",
    "realquiláramos": "realquilar",
    "realquiláremos": "realquilar",
    "realquilásemos": "realquilar",
    "realquilé": "realquilar",
    "realquiléis": "realquilar",
    "realquiló": "realquilar",
    "realza": "realzar",
    "realzaba": "realzar",
    "realzabais": "realzar",
    "realzaban": "realzar",
    "realzabas": "realzar",
    "realzad": "realzar",
    "realzada": "realzar",
    "realzadas": "realzar",
    "realzado": "realzar",
    "realzados": "realzar",
    "realzamos": "realzar",
    "realzan": "realzar",
    "realzando": "realzar",
    "realzara": "realzar",
    "realzarais": "realzar",
    "realzaran": "realzar",
    "realzaras": "realzar",
    "realzare": "realzar",
    "realzareis": "realzar",
    "realzaremos": "realzar",
    "realzaren": "realzar",
    "realzares": "realzar",
    "realzaron": "realzar",
    "realzará": "realzar",
    "realzarán": "realzar",
    "realzarás": "realzar",
    "realzaré": "realzar",
    "realzaréis": "realzar",
    "realzaría": "realzar",
    "realzaríais": "realzar",
    "realzaríamos": "realzar",
    "realzarían": "realzar",
    "realzarías": "realzar",
    "realzas": "realzar",
    "realzase": "realzar",
    "realzaseis": "realzar",
    "realzasen": "realzar",
    "realzases": "realzar",
    "realzaste": "realzar",
    "realzasteis": "realzar",
    "realzo": "realzar",
    "realzábamos": "realzar",
    "realzáis": "realzar",
    "realzáramos": "realzar",
    "realzáremos": "realzar",
    "realzásemos": "realzar",
    "realzó": "realzar",
    "reanima": "reanimar",
    "reanimaba": "reanimar",
    "reanimabais": "reanimar",
    "reanimaban": "reanimar",
    "reanimabas": "reanimar",
    "reanimaciones": "reanimación",
    "reanimad": "reanimar",
    "reanimada": "reanimar",
    "reanimadas": "reanimar",
    "reanimado": "reanimar",
    "reanimadora": "reanimador",
    "reanimadoras": "reanimador",
    "reanimadores": "reanimador",
    "reanimados": "reanimar",
    "reanimamos": "reanimar",
    "reaniman": "reanimar",
    "reanimando": "reanimar",
    "reanimara": "reanimar",
    "reanimarais": "reanimar",
    "reanimaran": "reanimar",
    "reanimaras": "reanimar",
    "reanimare": "reanimar",
    "reanimareis": "reanimar",
    "reanimaremos": "reanimar",
    "reanimaren": "reanimar",
    "reanimares": "reanimar",
    "reanimaron": "reanimar",
    "reanimará": "reanimar",
    "reanimarán": "reanimar",
    "reanimarás": "reanimar",
    "reanimaré": "reanimar",
    "reanimaréis": "reanimar",
    "reanimaría": "reanimar",
    "reanimaríais": "reanimar",
    "reanimaríamos": "reanimar",
    "reanimarían": "reanimar",
    "reanimarías": "reanimar",
    "reanimas": "reanimar",
    "reanimase": "reanimar",
    "reanimaseis": "reanimar",
    "reanimasen": "reanimar",
    "reanimases": "reanimar",
    "reanimaste": "reanimar",
    "reanimasteis": "reanimar",
    "reanime": "reanimar",
    "reanimemos": "reanimar",
    "reanimen": "reanimar",
    "reanimes": "reanimar",
    "reanimo": "reanimar",
    "reanimábamos": "reanimar",
    "reanimáis": "reanimar",
    "reanimáramos": "reanimar",
    "reanimáremos": "reanimar",
    "reanimásemos": "reanimar",
    "reanimé": "reanimar",
    "reaniméis": "reanimar",
    "reanimó": "reanimar",
    "reanuda": "reanudar",
    "reanudaba": "reanudar",
    "reanudabais": "reanudar",
    "reanudaban": "reanudar",
    "reanudabas": "reanudar",
    "reanudaciones": "reanudación",
    "reanudad": "reanudar",
    "reanudada": "reanudar",
    "reanudadas": "reanudar",
    "reanudado": "reanudar",
    "reanudados": "reanudar",
    "reanudamos": "reanudar",
    "reanudan": "reanudar",
    "reanudando": "reanudar",
    "reanudara": "reanudar",
    "reanudarais": "reanudar",
    "reanudaran": "reanudar",
    "reanudaras": "reanudar",
    "reanudare": "reanudar",
    "reanudareis": "reanudar",
    "reanudaremos": "reanudar",
    "reanudaren": "reanudar",
    "reanudares": "reanudar",
    "reanudaron": "reanudar",
    "reanudará": "reanudar",
    "reanudarán": "reanudar",
    "reanudarás": "reanudar",
    "reanudaré": "reanudar",
    "reanudaréis": "reanudar",
    "reanudaría": "reanudar",
    "reanudaríais": "reanudar",
    "reanudaríamos": "reanudar",
    "reanudarían": "reanudar",
    "reanudarías": "reanudar",
    "reanudas": "reanudar",
    "reanudase": "reanudar",
    "reanudaseis": "reanudar",
    "reanudasen": "reanudar",
    "reanudases": "reanudar",
    "reanudaste": "reanudar",
    "reanudasteis": "reanudar",
    "reanude": "reanudar",
    "reanudemos": "reanudar",
    "reanuden": "reanudar",
    "reanudes": "reanudar",
    "reanudo": "reanudar",
    "reanudábamos": "reanudar",
    "reanudáis": "reanudar",
    "reanudáramos": "reanudar",
    "reanudáremos": "reanudar",
    "reanudásemos": "reanudar",
    "reanudé": "reanudar",
    "reanudéis": "reanudar",
    "reanudó": "reanudar",
    "reaparece": "reaparecer",
    "reapareced": "reaparecer",
    "reaparecemos": "reaparecer",
    "reaparecen": "reaparecer",
    "reapareceremos": "reaparecer",
    "reaparecerá": "reaparecer",
    "reaparecerán": "reaparecer",
    "reaparecerás": "reaparecer",
    "reapareceré": "reaparecer",
    "reapareceréis": "reaparecer",
    "reaparecería": "reaparecer",
    "reapareceríais": "reaparecer",
    "reapareceríamos": "reaparecer",
    "reaparecerían": "reaparecer",
    "reaparecerías": "reaparecer",
    "reapareces": "reaparecer",
    "reaparecida": "reaparecer",
    "reaparecidas": "reaparecer",
    "reaparecido": "reaparecer",
    "reaparecidos": "reaparecer",
    "reapareciendo": "reaparecer",
    "reapareciera": "reaparecer",
    "reaparecierais": "reaparecer",
    "reaparecieran": "reaparecer",
    "reaparecieras": "reaparecer",
    "reapareciere": "reaparecer",
    "reapareciereis": "reaparecer",
    "reaparecieren": "reaparecer",
    "reaparecieres": "reaparecer",
    "reaparecieron": "reaparecer",
    "reapareciese": "reaparecer",
    "reaparecieseis": "reaparecer",
    "reapareciesen": "reaparecer",
    "reaparecieses": "reaparecer",
    "reaparecimos": "reaparecer",
    "reapareciste": "reaparecer",
    "reaparecisteis": "reaparecer",
    "reapareciéramos": "reaparecer",
    "reapareciéremos": "reaparecer",
    "reapareciésemos": "reaparecer",
    "reapareció": "reaparecer",
    "reaparecéis": "reaparecer",
    "reaparecí": "reaparecer",
    "reaparecía": "reaparecer",
    "reaparecíais": "reaparecer",
    "reaparecíamos": "reaparecer",
    "reaparecían": "reaparecer",
    "reaparecías": "reaparecer",
    "reaparezca": "reaparecer",
    "reaparezcamos": "reaparecer",
    "reaparezcan": "reaparecer",
    "reaparezcas": "reaparecer",
    "reaparezco": "reaparecer",
    "reaparezcáis": "reaparecer",
    "reapariciones": "reaparición",
    "reaperturas": "reapertura",
    "reaplicabilidades": "reaplicabilidad",
    "reaplicaciones": "reaplicación",
    "reaprendizajes": "reaprendizaje",
    "reaprovechables": "reaprovechable",
    "reaprovisionamientos": "reaprovisionamiento",
    "rearguya": "reargüir",
    "rearguyamos": "reargüir",
    "rearguyan": "reargüir",
    "rearguyas": "reargüir",
    "rearguye": "reargüir",
    "rearguyen": "reargüir",
    "rearguyendo": "reargüir",
    "rearguyera": "reargüir",
    "rearguyerais": "reargüir",
    "rearguyeran": "reargüir",
    "rearguyeras": "reargüir",
    "rearguyere": "reargüir",
    "rearguyereis": "reargüir",
    "rearguyeren": "reargüir",
    "rearguyeres": "reargüir",
    "rearguyeron": "reargüir",
    "rearguyes": "reargüir",
    "rearguyese": "reargüir",
    "rearguyeseis": "reargüir",
    "rearguyesen": "reargüir",
    "rearguyeses": "reargüir",
    "rearguyo": "reargüir",
    "rearguyáis": "reargüir",
    "rearguyéramos": "reargüir",
    "rearguyéremos": "reargüir",
    "rearguyésemos": "reargüir",
    "rearguyó": "reargüir",
    "reargüid": "reargüir",
    "reargüida": "reargüir",
    "reargüidas": "reargüir",
    "reargüido": "reargüir",
    "reargüidos": "reargüir",
    "reargüimos": "reargüir",
    "reargüiremos": "reargüir",
    "reargüirá": "reargüir",
    "reargüirán": "reargüir",
    "reargüirás": "reargüir",
    "reargüiré": "reargüir",
    "reargüiréis": "reargüir",
    "reargüiría": "reargüir",
    "reargüiríais": "reargüir",
    "reargüiríamos": "reargüir",
    "reargüirían": "reargüir",
    "reargüirías": "reargüir",
    "reargüiste": "reargüir",
    "reargüisteis": "reargüir",
    "reargüí": "reargüir",
    "reargüía": "reargüir",
    "reargüíais": "reargüir",
    "reargüíamos": "reargüir",
    "reargüían": "reargüir",
    "reargüías": "reargüir",
    "reargüís": "reargüir",
    "rearma": "rearmar",
    "rearmaba": "rearmar",
    "rearmabais": "rearmar",
    "rearmaban": "rearmar",
    "rearmabas": "rearmar",
    "rearmables": "rearmable",
    "rearmad": "rearmar",
    "rearmada": "rearmar",
    "rearmadas": "rearmar",
    "rearmado": "rearmar",
    "rearmados": "rearmar",
    "rearmamos": "rearmar",
    "rearman": "rearmar",
    "rearmando": "rearmar",
    "rearmara": "rearmar",
    "rearmarais": "rearmar",
    "rearmaran": "rearmar",
    "rearmaras": "rearmar",
    "rearmare": "rearmar",
    "rearmareis": "rearmar",
    "rearmaremos": "rearmar",
    "rearmaren": "rearmar",
    "rearmares": "rearmar",
    "rearmaron": "rearmar",
    "rearmará": "rearmar",
    "rearmarán": "rearmar",
    "rearmarás": "rearmar",
    "rearmaré": "rearmar",
    "rearmaréis": "rearmar",
    "rearmaría": "rearmar",
    "rearmaríais": "rearmar",
    "rearmaríamos": "rearmar",
    "rearmarían": "rearmar",
    "rearmarías": "rearmar",
    "rearmas": "rearmar",
    "rearmase": "rearmar",
    "rearmaseis": "rearmar",
    "rearmasen": "rearmar",
    "rearmases": "rearmar",
    "rearmaste": "rearmar",
    "rearmasteis": "rearmar",
    "rearme": "rearmar",
    "rearmemos": "rearmar",
    "rearmen": "rearmar",
    "rearmes": "rearme",
    "rearmo": "rearmar",
    "rearmábamos": "rearmar",
    "rearmáis": "rearmar",
    "rearmáramos": "rearmar",
    "rearmáremos": "rearmar",
    "rearmásemos": "rearmar",
    "rearmé": "rearmar",
    "rearméis": "rearmar",
    "rearmó": "rearmar",
    "rearranques": "rearranque",
    "rearrendamientos": "rearrendamiento",
    "reas": "reo",
    "reasegura": "reasegurar",
    "reaseguraba": "reasegurar",
    "reasegurabais": "reasegurar",
    "reaseguraban": "reasegurar",
    "reasegurabas": "reasegurar",
    "reasegurad": "reasegurar",
    "reasegurada": "reasegurar",
    "reaseguradas": "reasegurar",
    "reasegurado": "reasegurar",
    "reaseguradora": "reasegurador",
    "reaseguradoras": "reasegurador",
    "reaseguradores": "reasegurador",
    "reasegurados": "reasegurar",
    "reaseguramos": "reasegurar",
    "reaseguran": "reasegurar",
    "reasegurando": "reasegurar",
    "reasegurara": "reasegurar",
    "reasegurarais": "reasegurar",
    "reaseguraran": "reasegurar",
    "reaseguraras": "reasegurar",
    "reasegurare": "reasegurar",
    "reasegurareis": "reasegurar",
    "reaseguraremos": "reasegurar",
    "reaseguraren": "reasegurar",
    "reasegurares": "reasegurar",
    "reaseguraron": "reasegurar",
    "reasegurará": "reasegurar",
    "reasegurarán": "reasegurar",
    "reasegurarás": "reasegurar",
    "reaseguraré": "reasegurar",
    "reaseguraréis": "reasegurar",
    "reaseguraría": "reasegurar",
    "reaseguraríais": "reasegurar",
    "reaseguraríamos": "reasegurar",
    "reasegurarían": "reasegurar",
    "reasegurarías": "reasegurar",
    "reaseguras": "reasegurar",
    "reasegurase": "reasegurar",
    "reaseguraseis": "reasegurar",
    "reasegurasen": "reasegurar",
    "reasegurases": "reasegurar",
    "reaseguraste": "reasegurar",
    "reasegurasteis": "reasegurar",
    "reasegure": "reasegurar",
    "reaseguremos": "reasegurar",
    "reaseguren": "reasegurar",
    "reasegures": "reasegurar",
    "reaseguro": "reasegurar",
    "reaseguros": "reaseguro",
    "reasegurábamos": "reasegurar",
    "reaseguráis": "reasegurar",
    "reaseguráramos": "reasegurar",
    "reaseguráremos": "reasegurar",
    "reasegurásemos": "reasegurar",
    "reaseguré": "reasegurar",
    "reaseguréis": "reasegurar",
    "reaseguró": "reasegurar",
    "reasentaba": "reasentar",
    "reasentabais": "reasentar",
    "reasentaban": "reasentar",
    "reasentabas": "reasentar",
    "reasentad": "reasentar",
    "reasentada": "reasentar",
    "reasentadas": "reasentar",
    "reasentado": "reasentar",
    "reasentados": "reasentar",
    "reasentamientos": "reasentamiento",
    "reasentamos": "reasentar",
    "reasentando": "reasentar",
    "reasentara": "reasentar",
    "reasentarais": "reasentar",
    "reasentaran": "reasentar",
    "reasentaras": "reasentar",
    "reasentare": "reasentar",
    "reasentareis": "reasentar",
    "reasentaremos": "reasentar",
    "reasentaren": "reasentar",
    "reasentares": "reasentar",
    "reasentaron": "reasentar",
    "reasentará": "reasentar",
    "reasentarán": "reasentar",
    "reasentarás": "reasentar",
    "reasentaré": "reasentar",
    "reasentaréis": "reasentar",
    "reasentaría": "reasentar",
    "reasentaríais": "reasentar",
    "reasentaríamos": "reasentar",
    "reasentarían": "reasentar",
    "reasentarías": "reasentar",
    "reasentase": "reasentar",
    "reasentaseis": "reasentar",
    "reasentasen": "reasentar",
    "reasentases": "reasentar",
    "reasentaste": "reasentar",
    "reasentasteis": "reasentar",
    "reasentemos": "reasentar",
    "reasentábamos": "reasentar",
    "reasentáis": "reasentar",
    "reasentáramos": "reasentar",
    "reasentáremos": "reasentar",
    "reasentásemos": "reasentar",
    "reasenté": "reasentar",
    "reasentéis": "reasentar",
    "reasentó": "reasentar",
    "reasienta": "reasentar",
    "reasientan": "reasentar",
    "reasientas": "reasentar",
    "reasiente": "reasentar",
    "reasienten": "reasentar",
    "reasientes": "reasentar",
    "reasiento": "reasentar",
    "reasigna": "reasignar",
    "reasignaba": "reasignar",
    "reasignabais": "reasignar",
    "reasignaban": "reasignar",
    "reasignabas": "reasignar",
    "reasignaciones": "reasignación",
    "reasignad": "reasignar",
    "reasignada": "reasignar",
    "reasignadas": "reasignar",
    "reasignado": "reasignar",
    "reasignados": "reasignar",
    "reasignamos": "reasignar",
    "reasignan": "reasignar",
    "reasignando": "reasignar",
    "reasignara": "reasignar",
    "reasignarais": "reasignar",
    "reasignaran": "reasignar",
    "reasignaras": "reasignar",
    "reasignare": "reasignar",
    "reasignareis": "reasignar",
    "reasignaremos": "reasignar",
    "reasignaren": "reasignar",
    "reasignares": "reasignar",
    "reasignaron": "reasignar",
    "reasignará": "reasignar",
    "reasignarán": "reasignar",
    "reasignarás": "reasignar",
    "reasignaré": "reasignar",
    "reasignaréis": "reasignar",
    "reasignaría": "reasignar",
    "reasignaríais": "reasignar",
    "reasignaríamos": "reasignar",
    "reasignarían": "reasignar",
    "reasignarías": "reasignar",
    "reasignas": "reasignar",
    "reasignase": "reasignar",
    "reasignaseis": "reasignar",
    "reasignasen": "reasignar",
    "reasignases": "reasignar",
    "reasignaste": "reasignar",
    "reasignasteis": "reasignar",
    "reasigne": "reasignar",
    "reasignemos": "reasignar",
    "reasignen": "reasignar",
    "reasignes": "reasignar",
    "reasigno": "reasignar",
    "reasignábamos": "reasignar",
    "reasignáis": "reasignar",
    "reasignáramos": "reasignar",
    "reasignáremos": "reasignar",
    "reasignásemos": "reasignar",
    "reasigné": "reasignar",
    "reasignéis": "reasignar",
    "reasignó": "reasignar",
    "reasuma": "reasumir",
    "reasumamos": "reasumir",
    "reasuman": "reasumir",
    "reasumas": "reasumir",
    "reasume": "reasumir",
    "reasumen": "reasumir",
    "reasumes": "reasumir",
    "reasumid": "reasumir",
    "reasumida": "reasumir",
    "reasumidas": "reasumir",
    "reasumido": "reasumir",
    "reasumidos": "reasumir",
    "reasumiendo": "reasumir",
    "reasumiera": "reasumir",
    "reasumierais": "reasumir",
    "reasumieran": "reasumir",
    "reasumieras": "reasumir",
    "reasumiere": "reasumir",
    "reasumiereis": "reasumir",
    "reasumieren": "reasumir",
    "reasumieres": "reasumir",
    "reasumieron": "reasumir",
    "reasumiese": "reasumir",
    "reasumieseis": "reasumir",
    "reasumiesen": "reasumir",
    "reasumieses": "reasumir",
    "reasumimos": "reasumir",
    "reasumiremos": "reasumir",
    "reasumirá": "reasumir",
    "reasumirán": "reasumir",
    "reasumirás": "reasumir",
    "reasumiré": "reasumir",
    "reasumiréis": "reasumir",
    "reasumiría": "reasumir",
    "reasumiríais": "reasumir",
    "reasumiríamos": "reasumir",
    "reasumirían": "reasumir",
    "reasumirías": "reasumir",
    "reasumiste": "reasumir",
    "reasumisteis": "reasumir",
    "reasumiéramos": "reasumir",
    "reasumiéremos": "reasumir",
    "reasumiésemos": "reasumir",
    "reasumió": "reasumir",
    "reasumo": "reasumir",
    "reasumáis": "reasumir",
    "reasumí": "reasumir",
    "reasumía": "reasumir",
    "reasumíais": "reasumir",
    "reasumíamos": "reasumir",
    "reasumían": "reasumir",
    "reasumías": "reasumir",
    "reasumís": "reasumir",
    "reasunciones": "reasunción",
    "reasunta": "reasumir",
    "reasuntas": "reasumir",
    "reasunto": "reasumir",
    "reasuntos": "reasumir",
    "reata": "reatar",
    "reataba": "reatar",
    "reatabais": "reatar",
    "reataban": "reatar",
    "reatabas": "reatar",
    "reatad": "reatar",
    "reatada": "reatar",
    "reatadas": "reatar",
    "reatado": "reatar",
    "reatados": "reatar",
    "reatamos": "reatar",
    "reatan": "reatar",
    "reatando": "reatar",
    "reatara": "reatar",
    "reatarais": "reatar",
    "reataran": "reatar",
    "reataras": "reatar",
    "reatare": "reatar",
    "reatareis": "reatar",
    "reataremos": "reatar",
    "reataren": "reatar",
    "reatares": "reatar",
    "reataron": "reatar",
    "reatará": "reatar",
    "reatarán": "reatar",
    "reatarás": "reatar",
    "reataré": "reatar",
    "reataréis": "reatar",
    "reataría": "reatar",
    "reataríais": "reatar",
    "reataríamos": "reatar",
    "reatarían": "reatar",
    "reatarías": "reatar",
    "reatas": "reatar",
    "reatase": "reatar",
    "reataseis": "reatar",
    "reatasen": "reatar",
    "reatases": "reatar",
    "reataste": "reatar",
    "reatasteis": "reatar",
    "reate": "reatar",
    "reatemos": "reatar",
    "reaten": "reatar",
    "reates": "reatar",
    "reato": "reatar",
    "reatos": "reato",
    "reatábamos": "reatar",
    "reatáis": "reatar",
    "reatáramos": "reatar",
    "reatáremos": "reatar",
    "reatásemos": "reatar",
    "reaté": "reatar",
    "reatéis": "reatar",
    "reató": "reatar",
    "reaviva": "reavivar",
    "reavivaba": "reavivar",
    "reavivabais": "reavivar",
    "reavivaban": "reavivar",
    "reavivabas": "reavivar",
    "reavivaciones": "reavivación",
    "reavivad": "reavivar",
    "reavivada": "reavivar",
    "reavivadas": "reavivar",
    "reavivado": "reavivar",
    "reavivados": "reavivar",
    "reavivamientos": "reavivamiento",
    "reavivamos": "reavivar",
    "reavivan": "reavivar",
    "reavivando": "reavivar",
    "reavivara": "reavivar",
    "reavivarais": "reavivar",
    "reavivaran": "reavivar",
    "reavivaras": "reavivar",
    "reavivare": "reavivar",
    "reavivareis": "reavivar",
    "reavivaremos": "reavivar",
    "reavivaren": "reavivar",
    "reavivares": "reavivar",
    "reavivaron": "reavivar",
    "reavivará": "reavivar",
    "reavivarán": "reavivar",
    "reavivarás": "reavivar",
    "reavivaré": "reavivar",
    "reavivaréis": "reavivar",
    "reavivaría": "reavivar",
    "reavivaríais": "reavivar",
    "reavivaríamos": "reavivar",
    "reavivarían": "reavivar",
    "reavivarías": "reavivar",
    "reavivas": "reavivar",
    "reavivase": "reavivar",
    "reavivaseis": "reavivar",
    "reavivasen": "reavivar",
    "reavivases": "reavivar",
    "reavivaste": "reavivar",
    "reavivasteis": "reavivar",
    "reavive": "reavivar",
    "reavivemos": "reavivar",
    "reaviven": "reavivar",
    "reavives": "reavivar",
    "reavivo": "reavivar",
    "reavivábamos": "reavivar",
    "reaviváis": "reavivar",
    "reaviváramos": "reavivar",
    "reaviváremos": "reavivar",
    "reavivásemos": "reavivar",
    "reavivé": "reavivar",
    "reavivéis": "reavivar",
    "reavivó": "reavivar",
    "rebabadora": "rebabador",
    "rebabadoras": "rebabador",
    "rebabadores": "rebabador",
    "rebabas": "rebaba",
    "rebaja": "rebajar",
    "rebajaba": "rebajar",
    "rebajabais": "rebajar",
    "rebajaban": "rebajar",
    "rebajabas": "rebajar",
    "rebajad": "rebajar",
    "rebajada": "rebajar",
    "rebajadas": "rebajar",
    "rebajado": "rebajar",
    "rebajados": "rebajar",
    "rebajamos": "rebajar",
    "rebajan": "rebajar",
    "rebajando": "rebajar",
    "rebajara": "rebajar",
    "rebajarais": "rebajar",
    "rebajaran": "rebajar",
    "rebajaras": "rebajar",
    "rebajare": "rebajar",
    "rebajareis": "rebajar",
    "rebajaremos": "rebajar",
    "rebajaren": "rebajar",
    "rebajares": "rebajar",
    "rebajaron": "rebajar",
    "rebajará": "rebajar",
    "rebajarán": "rebajar",
    "rebajarás": "rebajar",
    "rebajaré": "rebajar",
    "rebajaréis": "rebajar",
    "rebajaría": "rebajar",
    "rebajaríais": "rebajar",
    "rebajaríamos": "rebajar",
    "rebajarían": "rebajar",
    "rebajarías": "rebajar",
    "rebajas": "rebajar",
    "rebajase": "rebajar",
    "rebajaseis": "rebajar",
    "rebajasen": "rebajar",
    "rebajases": "rebajar",
    "rebajaste": "rebajar",
    "rebajasteis": "rebajar",
    "rebaje": "rebajar",
    "rebajemos": "rebajar",
    "rebajen": "rebajar",
    "rebajes": "rebaje",
    "rebajo": "rebajar",
    "rebajos": "rebajo",
    "rebajábamos": "rebajar",
    "rebajáis": "rebajar",
    "rebajáramos": "rebajar",
    "rebajáremos": "rebajar",
    "rebajásemos": "rebajar",
    "rebajé": "rebajar",
    "rebajéis": "rebajar",
    "rebajó": "rebajar",
    "rebalajes": "rebalaje",
    "rebalsa": "rebalsar",
    "rebalsaba": "rebalsar",
    "rebalsabais": "rebalsar",
    "rebalsaban": "rebalsar",
    "rebalsabas": "rebalsar",
    "rebalsad": "rebalsar",
    "rebalsada": "rebalsar",
    "rebalsadas": "rebalsar",
    "rebalsado": "rebalsar",
    "rebalsados": "rebalsar",
    "rebalsamos": "rebalsar",
    "rebalsan": "rebalsar",
    "rebalsando": "rebalsar",
    "rebalsara": "rebalsar",
    "rebalsarais": "rebalsar",
    "rebalsaran": "rebalsar",
    "rebalsaras": "rebalsar",
    "rebalsare": "rebalsar",
    "rebalsareis": "rebalsar",
    "rebalsaremos": "rebalsar",
    "rebalsaren": "rebalsar",
    "rebalsares": "rebalsar",
    "rebalsaron": "rebalsar",
    "rebalsará": "rebalsar",
    "rebalsarán": "rebalsar",
    "rebalsarás": "rebalsar",
    "rebalsaré": "rebalsar",
    "rebalsaréis": "rebalsar",
    "rebalsaría": "rebalsar",
    "rebalsaríais": "rebalsar",
    "rebalsaríamos": "rebalsar",
    "rebalsarían": "rebalsar",
    "rebalsarías": "rebalsar",
    "rebalsas": "rebalsar",
    "rebalsase": "rebalsar",
    "rebalsaseis": "rebalsar",
    "rebalsasen": "rebalsar",
    "rebalsases": "rebalsar",
    "rebalsaste": "rebalsar",
    "rebalsasteis": "rebalsar",
    "rebalse": "rebalsar",
    "rebalsemos": "rebalsar",
    "rebalsen": "rebalsar",
    "rebalses": "rebalse",
    "rebalso": "rebalsar",
    "rebalsábamos": "rebalsar",
    "rebalsáis": "rebalsar",
    "rebalsáramos": "rebalsar",
    "rebalsáremos": "rebalsar",
    "rebalsásemos": "rebalsar",
    "rebalsé": "rebalsar",
    "rebalséis": "rebalsar",
    "rebalsó": "rebalsar",
    "rebana": "rebanar",
    "rebanaba": "rebanar",
    "rebanabais": "rebanar",
    "rebanaban": "rebanar",
    "rebanabas": "rebanar",
    "rebanad": "rebanar",
    "rebanada": "rebanar",
    "rebanadas": "rebanar",
    "rebanado": "rebanar",
    "rebanados": "rebanar",
    "rebanamos": "rebanar",
    "rebanan": "rebanar",
    "rebanando": "rebanar",
    "rebanara": "rebanar",
    "rebanarais": "rebanar",
    "rebanaran": "rebanar",
    "rebanaras": "rebanar",
    "rebanare": "rebanar",
    "rebanareis": "rebanar",
    "rebanaremos": "rebanar",
    "rebanaren": "rebanar",
    "rebanares": "rebanar",
    "rebanaron": "rebanar",
    "rebanará": "rebanar",
    "rebanarán": "rebanar",
    "rebanarás": "rebanar",
    "rebanaré": "rebanar",
    "rebanaréis": "rebanar",
    "rebanaría": "rebanar",
    "rebanaríais": "rebanar",
    "rebanaríamos": "rebanar",
    "rebanarían": "rebanar",
    "rebanarías": "rebanar",
    "rebanas": "rebanar",
    "rebanase": "rebanar",
    "rebanaseis": "rebanar",
    "rebanasen": "rebanar",
    "rebanases": "rebanar",
    "rebanaste": "rebanar",
    "rebanasteis": "rebanar",
    "rebane": "rebanar",
    "rebanemos": "rebanar",
    "rebanen": "rebanar",
    "rebanes": "rebanar",
    "rebanitas": "rebanita",
    "rebano": "rebanar",
    "rebanábamos": "rebanar",
    "rebanáis": "rebanar",
    "rebanáramos": "rebanar",
    "rebanáremos": "rebanar",
    "rebanásemos": "rebanar",
    "rebané": "rebanar",
    "rebanéis": "rebanar",
    "rebanó": "rebanar",
    "rebarbada": "rebarbado",
    "rebarbadas": "rebarbado",
    "rebarbadora": "rebarbador",
    "rebarbadoras": "rebarbador",
    "rebarbadores": "rebarbador",
    "rebarbados": "rebarbado",
    "rebasa": "rebasar",
    "rebasaba": "rebasar",
    "rebasabais": "rebasar",
    "rebasaban": "rebasar",
    "rebasabas": "rebasar",
    "rebasad": "rebasar",
    "rebasada": "rebasar",
    "rebasadas": "rebasar",
    "rebasaderos": "rebasadero",
    "rebasado": "rebasar",
    "rebasados": "rebasar",
    "rebasamientos": "rebasamiento",
    "rebasamos": "rebasar",
    "rebasan": "rebasar",
    "rebasando": "rebasar",
    "rebasara": "rebasar",
    "rebasarais": "rebasar",
    "rebasaran": "rebasar",
    "rebasaras": "rebasar",
    "rebasare": "rebasar",
    "rebasareis": "rebasar",
    "rebasaremos": "rebasar",
    "rebasaren": "rebasar",
    "rebasares": "rebasar",
    "rebasaron": "rebasar",
    "rebasará": "rebasar",
    "rebasarán": "rebasar",
    "rebasarás": "rebasar",
    "rebasaré": "rebasar",
    "rebasaréis": "rebasar",
    "rebasaría": "rebasar",
    "rebasaríais": "rebasar",
    "rebasaríamos": "rebasar",
    "rebasarían": "rebasar",
    "rebasarías": "rebasar",
    "rebasas": "rebasar",
    "rebasase": "rebasar",
    "rebasaseis": "rebasar",
    "rebasasen": "rebasar",
    "rebasases": "rebasar",
    "rebasaste": "rebasar",
    "rebasasteis": "rebasar",
    "rebase": "rebasar",
    "rebasemos": "rebasar",
    "rebasen": "rebasar",
    "rebases": "rebase",
    "rebaso": "rebasar",
    "rebasábamos": "rebasar",
    "rebasáis": "rebasar",
    "rebasáramos": "rebasar",
    "rebasáremos": "rebasar",
    "rebasásemos": "rebasar",
    "rebasé": "rebasar",
    "rebaséis": "rebasar",
    "rebasó": "rebasar",
    "rebata": "rebatir",
    "rebatamos": "rebatir",
    "rebatan": "rebatir",
    "rebatas": "rebatir",
    "rebate": "rebatir",
    "rebaten": "rebatir",
    "rebates": "rebatir",
    "rebatibles": "rebatible",
    "rebatid": "rebatir",
    "rebatida": "rebatir",
    "rebatidas": "rebatir",
    "rebatido": "rebatir",
    "rebatidos": "rebatir",
    "rebatiendo": "rebatir",
    "rebatiera": "rebatir",
    "rebatierais": "rebatir",
    "rebatieran": "rebatir",
    "rebatieras": "rebatir",
    "rebatiere": "rebatir",
    "rebatiereis": "rebatir",
    "rebatieren": "rebatir",
    "rebatieres": "rebatir",
    "rebatieron": "rebatir",
    "rebatiese": "rebatir",
    "rebatieseis": "rebatir",
    "rebatiesen": "rebatir",
    "rebatieses": "rebatir",
    "rebatimientos": "rebatimiento",
    "rebatimos": "rebatir",
    "rebatiremos": "rebatir",
    "rebatirá": "rebatir",
    "rebatirán": "rebatir",
    "rebatirás": "rebatir",
    "rebatiré": "rebatir",
    "rebatiréis": "rebatir",
    "rebatiría": "rebatir",
    "rebatiríais": "rebatir",
    "rebatiríamos": "rebatir",
    "rebatirían": "rebatir",
    "rebatirías": "rebatir",
    "rebatiste": "rebatir",
    "rebatisteis": "rebatir",
    "rebatiéramos": "rebatir",
    "rebatiéremos": "rebatir",
    "rebatiésemos": "rebatir",
    "rebatiñas": "rebatiña",
    "rebatió": "rebatir",
    "rebato": "rebatir",
    "rebatos": "rebato",
    "rebatáis": "rebatir",
    "rebatí": "rebatir",
    "rebatía": "rebatir",
    "rebatíais": "rebatir",
    "rebatíamos": "rebatir",
    "rebatían": "rebatir",
    "rebatías": "rebatir",
    "rebatís": "rebatir",
    "rebautice": "rebautizar",
    "rebauticemos": "rebautizar",
    "rebauticen": "rebautizar",
    "rebautices": "rebautizar",
    "rebauticé": "rebautizar",
    "rebauticéis": "rebautizar",
    "rebautiza": "rebautizar",
    "rebautizaba": "rebautizar",
    "rebautizabais": "rebautizar",
    "rebautizaban": "rebautizar",
    "rebautizabas": "rebautizar",
    "rebautizad": "rebautizar",
    "rebautizada": "rebautizar",
    "rebautizadas": "rebautizar",
    "rebautizado": "rebautizar",
    "rebautizados": "rebautizar",
    "rebautizamos": "rebautizar",
    "rebautizan": "rebautizar",
    "rebautizando": "rebautizar",
    "rebautizara": "rebautizar",
    "rebautizarais": "rebautizar",
    "rebautizaran": "rebautizar",
    "rebautizaras": "rebautizar",
    "rebautizare": "rebautizar",
    "rebautizareis": "rebautizar",
    "rebautizaremos": "rebautizar",
    "rebautizaren": "rebautizar",
    "rebautizares": "rebautizar",
    "rebautizaron": "rebautizar",
    "rebautizará": "rebautizar",
    "rebautizarán": "rebautizar",
    "rebautizarás": "rebautizar",
    "rebautizaré": "rebautizar",
    "rebautizaréis": "rebautizar",
    "rebautizaría": "rebautizar",
    "rebautizaríais": "rebautizar",
    "rebautizaríamos": "rebautizar",
    "rebautizarían": "rebautizar",
    "rebautizarías": "rebautizar",
    "rebautizas": "rebautizar",
    "rebautizase": "rebautizar",
    "rebautizaseis": "rebautizar",
    "rebautizasen": "rebautizar",
    "rebautizases": "rebautizar",
    "rebautizaste": "rebautizar",
    "rebautizasteis": "rebautizar",
    "rebautizo": "rebautizar",
    "rebautizábamos": "rebautizar",
    "rebautizáis": "rebautizar",
    "rebautizáramos": "rebautizar",
    "rebautizáremos": "rebautizar",
    "rebautizásemos": "rebautizar",
    "rebautizó": "rebautizar",
    "rebaña": "rebañar",
    "rebañaba": "rebañar",
    "rebañabais": "rebañar",
    "rebañaban": "rebañar",
    "rebañabas": "rebañar",
    "rebañad": "rebañar",
    "rebañada": "rebañar",
    "rebañadas": "rebañar",
    "rebañaderas": "rebañadera",
    "rebañado": "rebañar",
    "rebañadora": "rebañador",
    "rebañadoras": "rebañador",
    "rebañadores": "rebañador",
    "rebañados": "rebañar",
    "rebañaduras": "rebañadura",
    "rebañamos": "rebañar",
    "rebañan": "rebañar",
    "rebañando": "rebañar",
    "rebañara": "rebañar",
    "rebañarais": "rebañar",
    "rebañaran": "rebañar",
    "rebañaras": "rebañar",
    "rebañare": "rebañar",
    "rebañareis": "rebañar",
    "rebañaremos": "rebañar",
    "rebañaren": "rebañar",
    "rebañares": "rebañar",
    "rebañaron": "rebañar",
    "rebañará": "rebañar",
    "rebañarán": "rebañar",
    "rebañarás": "rebañar",
    "rebañaré": "rebañar",
    "rebañaréis": "rebañar",
    "rebañaría": "rebañar",
    "rebañaríais": "rebañar",
    "rebañaríamos": "rebañar",
    "rebañarían": "rebañar",
    "rebañarías": "rebañar",
    "rebañas": "rebañar",
    "rebañase": "rebañar",
    "rebañaseis": "rebañar",
    "rebañasen": "rebañar",
    "rebañases": "rebañar",
    "rebañaste": "rebañar",
    "rebañasteis": "rebañar",
    "rebañe": "rebañar",
    "rebañega": "rebañego",
    "rebañegas": "rebañego",
    "rebañegos": "rebañego",
    "rebañemos": "rebañar",
    "rebañen": "rebañar",
    "rebañes": "rebañar",
    "rebaño": "rebañar",
    "rebaños": "rebaño",
    "rebañábamos": "rebañar",
    "rebañáis": "rebañar",
    "rebañáramos": "rebañar",
    "rebañáremos": "rebañar",
    "rebañásemos": "rebañar",
    "rebañé": "rebañar",
    "rebañéis": "rebañar",
    "rebañó": "rebañar",
    "rebecas": "rebeca",
    "rebecos": "rebeco",
    "rebela": "rebelar",
    "rebelaba": "rebelar",
    "rebelabais": "rebelar",
    "rebelaban": "rebelar",
    "rebelabas": "rebelar",
    "rebelad": "rebelar",
    "rebelada": "rebelar",
    "rebeladas": "rebelar",
    "rebelado": "rebelar",
    "rebelados": "rebelar",
    "rebelamos": "rebelar",
    "rebelan": "rebelar",
    "rebelando": "rebelar",
    "rebelara": "rebelar",
    "rebelarais": "rebelar",
    "rebelaran": "rebelar",
    "rebelaras": "rebelar",
    "rebelare": "rebelar",
    "rebelareis": "rebelar",
    "rebelaremos": "rebelar",
    "rebelaren": "rebelar",
    "rebelares": "rebelar",
    "rebelaron": "rebelar",
    "rebelará": "rebelar",
    "rebelarán": "rebelar",
    "rebelarás": "rebelar",
    "rebelaré": "rebelar",
    "rebelaréis": "rebelar",
    "rebelaría": "rebelar",
    "rebelaríais": "rebelar",
    "rebelaríamos": "rebelar",
    "rebelarían": "rebelar",
    "rebelarías": "rebelar",
    "rebelas": "rebelar",
    "rebelase": "rebelar",
    "rebelaseis": "rebelar",
    "rebelasen": "rebelar",
    "rebelases": "rebelar",
    "rebelaste": "rebelar",
    "rebelasteis": "rebelar",
    "rebeldes": "rebelde",
    "rebeldías": "rebeldía",
    "rebele": "rebelar",
    "rebelemos": "rebelar",
    "rebelen": "rebelar",
    "rebeles": "rebelar",
    "rebeliones": "rebelión",
    "rebelo": "rebelar",
    "rebelona": "rebelón",
    "rebelonas": "rebelón",
    "rebelones": "rebelón",
    "rebelábamos": "rebelar",
    "rebeláis": "rebelar",
    "rebeláramos": "rebelar",
    "rebeláremos": "rebelar",
    "rebelásemos": "rebelar",
    "rebelé": "rebelar",
    "rebeléis": "rebelar",
    "rebeló": "rebelar",
    "rebencazos": "rebencazo",
    "rebenques": "rebenque",
    "rebina": "rebinar",
    "rebinaba": "rebinar",
    "rebinabais": "rebinar",
    "rebinaban": "rebinar",
    "rebinabas": "rebinar",
    "rebinad": "rebinar",
    "rebinada": "rebinar",
    "rebinadas": "rebinar",
    "rebinado": "rebinar",
    "rebinados": "rebinar",
    "rebinamos": "rebinar",
    "rebinan": "rebinar",
    "rebinando": "rebinar",
    "rebinara": "rebinar",
    "rebinarais": "rebinar",
    "rebinaran": "rebinar",
    "rebinaras": "rebinar",
    "rebinare": "rebinar",
    "rebinareis": "rebinar",
    "rebinaremos": "rebinar",
    "rebinaren": "rebinar",
    "rebinares": "rebinar",
    "rebinaron": "rebinar",
    "rebinará": "rebinar",
    "rebinarán": "rebinar",
    "rebinarás": "rebinar",
    "rebinaré": "rebinar",
    "rebinaréis": "rebinar",
    "rebinaría": "rebinar",
    "rebinaríais": "rebinar",
    "rebinaríamos": "rebinar",
    "rebinarían": "rebinar",
    "rebinarías": "rebinar",
    "rebinas": "rebinar",
    "rebinase": "rebinar",
    "rebinaseis": "rebinar",
    "rebinasen": "rebinar",
    "rebinases": "rebinar",
    "rebinaste": "rebinar",
    "rebinasteis": "rebinar",
    "rebine": "rebinar",
    "rebinemos": "rebinar",
    "rebinen": "rebinar",
    "rebines": "rebinar",
    "rebino": "rebinar",
    "rebinábamos": "rebinar",
    "rebináis": "rebinar",
    "rebináramos": "rebinar",
    "rebináremos": "rebinar",
    "rebinásemos": "rebinar",
    "rebiné": "rebinar",
    "rebinéis": "rebinar",
    "rebinó": "rebinar",
    "rebisabuela": "rebisabuelo",
    "rebisabuelas": "rebisabuelo",
    "rebisabuelos": "rebisabuelo",
    "rebisnieta": "rebisnieto",
    "rebisnietas": "rebisnieto",
    "rebisnietos": "rebisnieto",
    "rebites": "rebite",
    "reblandece": "reblandecer",
    "reblandeced": "reblandecer",
    "reblandecedores": "reblandecedor",
    "reblandecemos": "reblandecer",
    "reblandecen": "reblandecer",
    "reblandeceremos": "reblandecer",
    "reblandecerá": "reblandecer",
    "reblandecerán": "reblandecer",
    "reblandecerás": "reblandecer",
    "reblandeceré": "reblandecer",
    "reblandeceréis": "reblandecer",
    "reblandecería": "reblandecer",
    "reblandeceríais": "reblandecer",
    "reblandeceríamos": "reblandecer",
    "reblandecerían": "reblandecer",
    "reblandecerías": "reblandecer",
    "reblandeces": "reblandecer",
    "reblandecida": "reblandecer",
    "reblandecidas": "reblandecer",
    "reblandecido": "reblandecer",
    "reblandecidos": "reblandecer",
    "reblandeciendo": "reblandecer",
    "reblandeciera": "reblandecer",
    "reblandecierais": "reblandecer",
    "reblandecieran": "reblandecer",
    "reblandecieras": "reblandecer",
    "reblandeciere": "reblandecer",
    "reblandeciereis": "reblandecer",
    "reblandecieren": "reblandecer",
    "reblandecieres": "reblandecer",
    "reblandecieron": "reblandecer",
    "reblandeciese": "reblandecer",
    "reblandecieseis": "reblandecer",
    "reblandeciesen": "reblandecer",
    "reblandecieses": "reblandecer",
    "reblandecimientos": "reblandecimiento",
    "reblandecimos": "reblandecer",
    "reblandeciste": "reblandecer",
    "reblandecisteis": "reblandecer",
    "reblandeciéramos": "reblandecer",
    "reblandeciéremos": "reblandecer",
    "reblandeciésemos": "reblandecer",
    "reblandeció": "reblandecer",
    "reblandecéis": "reblandecer",
    "reblandecí": "reblandecer",
    "reblandecía": "reblandecer",
    "reblandecíais": "reblandecer",
    "reblandecíamos": "reblandecer",
    "reblandecían": "reblandecer",
    "reblandecías": "reblandecer",
    "reblandezca": "reblandecer",
    "reblandezcamos": "reblandecer",
    "reblandezcan": "reblandecer",
    "reblandezcas": "reblandecer",
    "reblandezco": "reblandecer",
    "reblandezcáis": "reblandecer",
    "rebobina": "rebobinar",
    "rebobinaba": "rebobinar",
    "rebobinabais": "rebobinar",
    "rebobinaban": "rebobinar",
    "rebobinabas": "rebobinar",
    "rebobinad": "rebobinar",
    "rebobinada": "rebobinar",
    "rebobinadas": "rebobinar",
    "rebobinado": "rebobinar",
    "rebobinados": "rebobinar",
    "rebobinamos": "rebobinar",
    "rebobinan": "rebobinar",
    "rebobinando": "rebobinar",
    "rebobinara": "rebobinar",
    "rebobinarais": "rebobinar",
    "rebobinaran": "rebobinar",
    "rebobinaras": "rebobinar",
    "rebobinare": "rebobinar",
    "rebobinareis": "rebobinar",
    "rebobinaremos": "rebobinar",
    "rebobinaren": "rebobinar",
    "rebobinares": "rebobinar",
    "rebobinaron": "rebobinar",
    "rebobinará": "rebobinar",
    "rebobinarán": "rebobinar",
    "rebobinarás": "rebobinar",
    "rebobinaré": "rebobinar",
    "rebobinaréis": "rebobinar",
    "rebobinaría": "rebobinar",
    "rebobinaríais": "rebobinar",
    "rebobinaríamos": "rebobinar",
    "rebobinarían": "rebobinar",
    "rebobinarías": "rebobinar",
    "rebobinas": "rebobinar",
    "rebobinase": "rebobinar",
    "rebobinaseis": "rebobinar",
    "rebobinasen": "rebobinar",
    "rebobinases": "rebobinar",
    "rebobinaste": "rebobinar",
    "rebobinasteis": "rebobinar",
    "rebobine": "rebobinar",
    "rebobinemos": "rebobinar",
    "rebobinen": "rebobinar",
    "rebobines": "rebobinar",
    "rebobino": "rebobinar",
    "rebobinábamos": "rebobinar",
    "rebobináis": "rebobinar",
    "rebobináramos": "rebobinar",
    "rebobináremos": "rebobinar",
    "rebobinásemos": "rebobinar",
    "rebobiné": "rebobinar",
    "rebobinéis": "rebobinar",
    "rebobinó": "rebobinar",
    "reboce": "rebozar",
    "rebocemos": "rebozar",
    "rebocen": "rebozar",
    "reboces": "rebozar",
    "rebocillos": "rebocillo",
    "rebociños": "rebociño",
    "rebocé": "rebozar",
    "rebocéis": "rebozar",
    "rebojos": "rebojo",
    "rebollares": "rebollar",
    "rebolledos": "rebolledo",
    "rebollos": "rebollo",
    "rebolluda": "rebolludo",
    "rebolludas": "rebolludo",
    "rebolludos": "rebolludo",
    "rebordea": "rebordear",
    "rebordeaba": "rebordear",
    "rebordeabais": "rebordear",
    "rebordeaban": "rebordear",
    "rebordeabas": "rebordear",
    "rebordead": "rebordear",
    "rebordeada": "rebordear",
    "rebordeadas": "rebordear",
    "rebordeado": "rebordear",
    "rebordeados": "rebordear",
    "rebordeamos": "rebordear",
    "rebordean": "rebordear",
    "rebordeando": "rebordear",
    "rebordeara": "rebordear",
    "rebordearais": "rebordear",
    "rebordearan": "rebordear",
    "rebordearas": "rebordear",
    "rebordeare": "rebordear",
    "rebordeareis": "rebordear",
    "rebordearemos": "rebordear",
    "rebordearen": "rebordear",
    "rebordeares": "rebordear",
    "rebordearon": "rebordear",
    "rebordeará": "rebordear",
    "rebordearán": "rebordear",
    "rebordearás": "rebordear",
    "rebordearé": "rebordear",
    "rebordearéis": "rebordear",
    "rebordearía": "rebordear",
    "rebordearíais": "rebordear",
    "rebordearíamos": "rebordear",
    "rebordearían": "rebordear",
    "rebordearías": "rebordear",
    "rebordeas": "rebordear",
    "rebordease": "rebordear",
    "rebordeaseis": "rebordear",
    "rebordeasen": "rebordear",
    "rebordeases": "rebordear",
    "rebordeaste": "rebordear",
    "rebordeasteis": "rebordear",
    "rebordee": "rebordear",
    "rebordeemos": "rebordear",
    "rebordeen": "rebordear",
    "rebordees": "rebordear",
    "rebordeo": "rebordear",
    "rebordeos": "rebordeo",
    "rebordes": "reborde",
    "rebordeábamos": "rebordear",
    "rebordeáis": "rebordear",
    "rebordeáramos": "rebordear",
    "rebordeáremos": "rebordear",
    "rebordeásemos": "rebordear",
    "rebordeé": "rebordear",
    "rebordeéis": "rebordear",
    "rebordeó": "rebordear",
    "rebosa": "rebosar",
    "rebosaba": "rebosar",
    "rebosabais": "rebosar",
    "rebosaban": "rebosar",
    "rebosabas": "rebosar",
    "rebosad": "rebosar",
    "rebosada": "rebosar",
    "rebosadas": "rebosar",
    "rebosaderos": "rebosadero",
    "rebosado": "rebosar",
    "rebosados": "rebosar",
    "rebosaduras": "rebosadura",
    "rebosamientos": "rebosamiento",
    "rebosamos": "rebosar",
    "rebosan": "rebosar",
    "rebosando": "rebosar",
    "rebosantes": "rebosante",
    "rebosara": "rebosar",
    "rebosarais": "rebosar",
    "rebosaran": "rebosar",
    "rebosaras": "rebosar",
    "rebosare": "rebosar",
    "rebosareis": "rebosar",
    "rebosaremos": "rebosar",
    "rebosaren": "rebosar",
    "rebosares": "rebosar",
    "rebosaron": "rebosar",
    "rebosará": "rebosar",
    "rebosarán": "rebosar",
    "rebosarás": "rebosar",
    "rebosaré": "rebosar",
    "rebosaréis": "rebosar",
    "rebosaría": "rebosar",
    "rebosaríais": "rebosar",
    "rebosaríamos": "rebosar",
    "rebosarían": "rebosar",
    "rebosarías": "rebosar",
    "rebosas": "rebosar",
    "rebosase": "rebosar",
    "rebosaseis": "rebosar",
    "rebosasen": "rebosar",
    "rebosases": "rebosar",
    "rebosaste": "rebosar",
    "rebosasteis": "rebosar",
    "rebose": "rebosar",
    "rebosemos": "rebosar",
    "rebosen": "rebosar",
    "reboses": "rebose",
    "reboso": "rebosar",
    "rebosábamos": "rebosar",
    "rebosáis": "rebosar",
    "rebosáramos": "rebosar",
    "rebosáremos": "rebosar",
    "rebosásemos": "rebosar",
    "rebosé": "rebosar",
    "reboséis": "rebosar",
    "rebosó": "rebosar",
    "rebota": "rebotar",
    "rebotaba": "rebotar",
    "rebotabais": "rebotar",
    "rebotaban": "rebotar",
    "rebotabas": "rebotar",
    "rebotad": "rebotar",
    "rebotada": "rebotar",
    "rebotadas": "rebotar",
    "rebotaderas": "rebotadera",
    "rebotado": "rebotar",
    "rebotadora": "rebotador",
    "rebotadoras": "rebotador",
    "rebotadores": "rebotador",
    "rebotados": "rebotar",
    "rebotaduras": "rebotadura",
    "rebotamos": "rebotar",
    "rebotan": "rebotar",
    "rebotando": "rebotar",
    "rebotara": "rebotar",
    "rebotarais": "rebotar",
    "rebotaran": "rebotar",
    "rebotaras": "rebotar",
    "rebotare": "rebotar",
    "rebotareis": "rebotar",
    "rebotaremos": "rebotar",
    "rebotaren": "rebotar",
    "rebotares": "rebotar",
    "rebotaron": "rebotar",
    "rebotará": "rebotar",
    "rebotarán": "rebotar",
    "rebotarás": "rebotar",
    "rebotaré": "rebotar",
    "rebotaréis": "rebotar",
    "rebotaría": "rebotar",
    "rebotaríais": "rebotar",
    "rebotaríamos": "rebotar",
    "rebotarían": "rebotar",
    "rebotarías": "rebotar",
    "rebotas": "rebotar",
    "rebotase": "rebotar",
    "rebotaseis": "rebotar",
    "rebotasen": "rebotar",
    "rebotases": "rebotar",
    "rebotaste": "rebotar",
    "rebotasteis": "rebotar",
    "rebote": "rebotar",
    "rebotea": "rebotear",
    "reboteaba": "rebotear",
    "reboteabais": "rebotear",
    "reboteaban": "rebotear",
    "reboteabas": "rebotear",
    "rebotead": "rebotear",
    "reboteada": "rebotear",
    "reboteadas": "rebotear",
    "reboteado": "rebotear",
    "reboteadores": "reboteador",
    "reboteados": "rebotear",
    "reboteamos": "rebotear",
    "rebotean": "rebotear",
    "reboteando": "rebotear",
    "reboteara": "rebotear",
    "rebotearais": "rebotear",
    "rebotearan": "rebotear",
    "rebotearas": "rebotear",
    "reboteare": "rebotear",
    "reboteareis": "rebotear",
    "rebotearemos": "rebotear",
    "rebotearen": "rebotear",
    "reboteares": "rebotear",
    "rebotearon": "rebotear",
    "reboteará": "rebotear",
    "rebotearán": "rebotear",
    "rebotearás": "rebotear",
    "rebotearé": "rebotear",
    "rebotearéis": "rebotear",
    "rebotearía": "rebotear",
    "rebotearíais": "rebotear",
    "rebotearíamos": "rebotear",
    "rebotearían": "rebotear",
    "rebotearías": "rebotear",
    "reboteas": "rebotear",
    "rebotease": "rebotear",
    "reboteaseis": "rebotear",
    "reboteasen": "rebotear",
    "reboteases": "rebotear",
    "reboteaste": "rebotear",
    "reboteasteis": "rebotear",
    "rebotee": "rebotear",
    "reboteemos": "rebotear",
    "reboteen": "rebotear",
    "rebotees": "rebotear",
    "rebotemos": "rebotar",
    "reboten": "rebotar",
    "reboteo": "rebotear",
    "rebotes": "rebote",
    "reboteábamos": "rebotear",
    "reboteáis": "rebotear",
    "reboteáramos": "rebotear",
    "reboteáremos": "rebotear",
    "reboteásemos": "rebotear",
    "reboteé": "rebotear",
    "reboteéis": "rebotear",
    "reboteó": "rebotear",
    "reboticas": "rebotica",
    "reboto": "rebotar",
    "rebotábamos": "rebotar",
    "rebotáis": "rebotar",
    "rebotáramos": "rebotar",
    "rebotáremos": "rebotar",
    "rebotásemos": "rebotar",
    "reboté": "rebotar",
    "rebotéis": "rebotar",
    "rebotó": "rebotar",
    "reboza": "rebozar",
    "rebozaba": "rebozar",
    "rebozabais": "rebozar",
    "rebozaban": "rebozar",
    "rebozabas": "rebozar",
    "rebozad": "rebozar",
    "rebozada": "rebozar",
    "rebozadas": "rebozar",
    "rebozado": "rebozar",
    "rebozados": "rebozar",
    "rebozamos": "rebozar",
    "rebozan": "rebozar",
    "rebozando": "rebozar",
    "rebozara": "rebozar",
    "rebozarais": "rebozar",
    "rebozaran": "rebozar",
    "rebozaras": "rebozar",
    "rebozare": "rebozar",
    "rebozareis": "rebozar",
    "rebozaremos": "rebozar",
    "rebozaren": "rebozar",
    "rebozares": "rebozar",
    "rebozaron": "rebozar",
    "rebozará": "rebozar",
    "rebozarán": "rebozar",
    "rebozarás": "rebozar",
    "rebozaré": "rebozar",
    "rebozaréis": "rebozar",
    "rebozaría": "rebozar",
    "rebozaríais": "rebozar",
    "rebozaríamos": "rebozar",
    "rebozarían": "rebozar",
    "rebozarías": "rebozar",
    "rebozas": "rebozar",
    "rebozase": "rebozar",
    "rebozaseis": "rebozar",
    "rebozasen": "rebozar",
    "rebozases": "rebozar",
    "rebozaste": "rebozar",
    "rebozasteis": "rebozar",
    "rebozo": "rebozar",
    "rebozos": "rebozo",
    "rebozábamos": "rebozar",
    "rebozáis": "rebozar",
    "rebozáramos": "rebozar",
    "rebozáremos": "rebozar",
    "rebozásemos": "rebozar",
    "rebozó": "rebozar",
    "reboños": "reboño",
    "rebrama": "rebramar",
    "rebramaba": "rebramar",
    "rebramabais": "rebramar",
    "rebramaban": "rebramar",
    "rebramabas": "rebramar",
    "rebramad": "rebramar",
    "rebramada": "rebramar",
    "rebramadas": "rebramar",
    "rebramado": "rebramar",
    "rebramados": "rebramar",
    "rebramamos": "rebramar",
    "rebraman": "rebramar",
    "rebramando": "rebramar",
    "rebramara": "rebramar",
    "rebramarais": "rebramar",
    "rebramaran": "rebramar",
    "rebramaras": "rebramar",
    "rebramare": "rebramar",
    "rebramareis": "rebramar",
    "rebramaremos": "rebramar",
    "rebramaren": "rebramar",
    "rebramares": "rebramar",
    "rebramaron": "rebramar",
    "rebramará": "rebramar",
    "rebramarán": "rebramar",
    "rebramarás": "rebramar",
    "rebramaré": "rebramar",
    "rebramaréis": "rebramar",
    "rebramaría": "rebramar",
    "rebramaríais": "rebramar",
    "rebramaríamos": "rebramar",
    "rebramarían": "rebramar",
    "rebramarías": "rebramar",
    "rebramas": "rebramar",
    "rebramase": "rebramar",
    "rebramaseis": "rebramar",
    "rebramasen": "rebramar",
    "rebramases": "rebramar",
    "rebramaste": "rebramar",
    "rebramasteis": "rebramar",
    "rebrame": "rebramar",
    "rebramemos": "rebramar",
    "rebramen": "rebramar",
    "rebrames": "rebramar",
    "rebramo": "rebramar",
    "rebramábamos": "rebramar",
    "rebramáis": "rebramar",
    "rebramáramos": "rebramar",
    "rebramáremos": "rebramar",
    "rebramásemos": "rebramar",
    "rebramé": "rebramar",
    "rebraméis": "rebramar",
    "rebramó": "rebramar",
    "rebrinca": "rebrincar",
    "rebrincaba": "rebrincar",
    "rebrincabais": "rebrincar",
    "rebrincaban": "rebrincar",
    "rebrincabas": "rebrincar",
    "rebrincad": "rebrincar",
    "rebrincada": "rebrincar",
    "rebrincadas": "rebrincar",
    "rebrincado": "rebrincar",
    "rebrincados": "rebrincar",
    "rebrincamos": "rebrincar",
    "rebrincan": "rebrincar",
    "rebrincando": "rebrincar",
    "rebrincara": "rebrincar",
    "rebrincarais": "rebrincar",
    "rebrincaran": "rebrincar",
    "rebrincaras": "rebrincar",
    "rebrincare": "rebrincar",
    "rebrincareis": "rebrincar",
    "rebrincaremos": "rebrincar",
    "rebrincaren": "rebrincar",
    "rebrincares": "rebrincar",
    "rebrincaron": "rebrincar",
    "rebrincará": "rebrincar",
    "rebrincarán": "rebrincar",
    "rebrincarás": "rebrincar",
    "rebrincaré": "rebrincar",
    "rebrincaréis": "rebrincar",
    "rebrincaría": "rebrincar",
    "rebrincaríais": "rebrincar",
    "rebrincaríamos": "rebrincar",
    "rebrincarían": "rebrincar",
    "rebrincarías": "rebrincar",
    "rebrincas": "rebrincar",
    "rebrincase": "rebrincar",
    "rebrincaseis": "rebrincar",
    "rebrincasen": "rebrincar",
    "rebrincases": "rebrincar",
    "rebrincaste": "rebrincar",
    "rebrincasteis": "rebrincar",
    "rebrinco": "rebrincar",
    "rebrincábamos": "rebrincar",
    "rebrincáis": "rebrincar",
    "rebrincáramos": "rebrincar",
    "rebrincáremos": "rebrincar",
    "rebrincásemos": "rebrincar",
    "rebrincó": "rebrincar",
    "rebrinque": "rebrincar",
    "rebrinquemos": "rebrincar",
    "rebrinquen": "rebrincar",
    "rebrinques": "rebrincar",
    "rebrinqué": "rebrincar",
    "rebrinquéis": "rebrincar",
    "rebrota": "rebrotar",
    "rebrotaba": "rebrotar",
    "rebrotabais": "rebrotar",
    "rebrotaban": "rebrotar",
    "rebrotabas": "rebrotar",
    "rebrotad": "rebrotar",
    "rebrotada": "rebrotar",
    "rebrotadas": "rebrotar",
    "rebrotado": "rebrotar",
    "rebrotados": "rebrotar",
    "rebrotamos": "rebrotar",
    "rebrotan": "rebrotar",
    "rebrotando": "rebrotar",
    "rebrotara": "rebrotar",
    "rebrotarais": "rebrotar",
    "rebrotaran": "rebrotar",
    "rebrotaras": "rebrotar",
    "rebrotare": "rebrotar",
    "rebrotareis": "rebrotar",
    "rebrotaremos": "rebrotar",
    "rebrotaren": "rebrotar",
    "rebrotares": "rebrotar",
    "rebrotaron": "rebrotar",
    "rebrotará": "rebrotar",
    "rebrotarán": "rebrotar",
    "rebrotarás": "rebrotar",
    "rebrotaré": "rebrotar",
    "rebrotaréis": "rebrotar",
    "rebrotaría": "rebrotar",
    "rebrotaríais": "rebrotar",
    "rebrotaríamos": "rebrotar",
    "rebrotarían": "rebrotar",
    "rebrotarías": "rebrotar",
    "rebrotas": "rebrotar",
    "rebrotase": "rebrotar",
    "rebrotaseis": "rebrotar",
    "rebrotasen": "rebrotar",
    "rebrotases": "rebrotar",
    "rebrotaste": "rebrotar",
    "rebrotasteis": "rebrotar",
    "rebrote": "rebrotar",
    "rebrotemos": "rebrotar",
    "rebroten": "rebrotar",
    "rebrotes": "rebrote",
    "rebroto": "rebrotar",
    "rebrotábamos": "rebrotar",
    "rebrotáis": "rebrotar",
    "rebrotáramos": "rebrotar",
    "rebrotáremos": "rebrotar",
    "rebrotásemos": "rebrotar",
    "rebroté": "rebrotar",
    "rebrotéis": "rebrotar",
    "rebrotó": "rebrotar",
    "rebudia": "rebudiar",
    "rebudiaba": "rebudiar",
    "rebudiabais": "rebudiar",
    "rebudiaban": "rebudiar",
    "rebudiabas": "rebudiar",
    "rebudiad": "rebudiar",
    "rebudiada": "rebudiar",
    "rebudiadas": "rebudiar",
    "rebudiado": "rebudiar",
    "rebudiados": "rebudiar",
    "rebudiamos": "rebudiar",
    "rebudian": "rebudiar",
    "rebudiando": "rebudiar",
    "rebudiara": "rebudiar",
    "rebudiarais": "rebudiar",
    "rebudiaran": "rebudiar",
    "rebudiaras": "rebudiar",
    "rebudiare": "rebudiar",
    "rebudiareis": "rebudiar",
    "rebudiaremos": "rebudiar",
    "rebudiaren": "rebudiar",
    "rebudiares": "rebudiar",
    "rebudiaron": "rebudiar",
    "rebudiará": "rebudiar",
    "rebudiarán": "rebudiar",
    "rebudiarás": "rebudiar",
    "rebudiaré": "rebudiar",
    "rebudiaréis": "rebudiar",
    "rebudiaría": "rebudiar",
    "rebudiaríais": "rebudiar",
    "rebudiaríamos": "rebudiar",
    "rebudiarían": "rebudiar",
    "rebudiarías": "rebudiar",
    "rebudias": "rebudiar",
    "rebudiase": "rebudiar",
    "rebudiaseis": "rebudiar",
    "rebudiasen": "rebudiar",
    "rebudiases": "rebudiar",
    "rebudiaste": "rebudiar",
    "rebudiasteis": "rebudiar",
    "rebudie": "rebudiar",
    "rebudiemos": "rebudiar",
    "rebudien": "rebudiar",
    "rebudies": "rebudiar",
    "rebudio": "rebudiar",
    "rebudios": "rebudio",
    "rebudiábamos": "rebudiar",
    "rebudiáis": "rebudiar",
    "rebudiáramos": "rebudiar",
    "rebudiáremos": "rebudiar",
    "rebudiásemos": "rebudiar",
    "rebudié": "rebudiar",
    "rebudiéis": "rebudiar",
    "rebudió": "rebudiar",
    "rebufa": "rebufar",
    "rebufaba": "rebufar",
    "rebufabais": "rebufar",
    "rebufaban": "rebufar",
    "rebufabas": "rebufar",
    "rebufad": "rebufar",
    "rebufada": "rebufar",
    "rebufadas": "rebufar",
    "rebufado": "rebufar",
    "rebufados": "rebufar",
    "rebufamos": "rebufar",
    "rebufan": "rebufar",
    "rebufando": "rebufar",
    "rebufara": "rebufar",
    "rebufarais": "rebufar",
    "rebufaran": "rebufar",
    "rebufaras": "rebufar",
    "rebufare": "rebufar",
    "rebufareis": "rebufar",
    "rebufaremos": "rebufar",
    "rebufaren": "rebufar",
    "rebufares": "rebufar",
    "rebufaron": "rebufar",
    "rebufará": "rebufar",
    "rebufarán": "rebufar",
    "rebufarás": "rebufar",
    "rebufaré": "rebufar",
    "rebufaréis": "rebufar",
    "rebufaría": "rebufar",
    "rebufaríais": "rebufar",
    "rebufaríamos": "rebufar",
    "rebufarían": "rebufar",
    "rebufarías": "rebufar",
    "rebufas": "rebufar",
    "rebufase": "rebufar",
    "rebufaseis": "rebufar",
    "rebufasen": "rebufar",
    "rebufases": "rebufar",
    "rebufaste": "rebufar",
    "rebufasteis": "rebufar",
    "rebufe": "rebufar",
    "rebufemos": "rebufar",
    "rebufen": "rebufar",
    "rebufes": "rebufe",
    "rebufo": "rebufar",
    "rebufos": "rebufo",
    "rebufábamos": "rebufar",
    "rebufáis": "rebufar",
    "rebufáramos": "rebufar",
    "rebufáremos": "rebufar",
    "rebufásemos": "rebufar",
    "rebufé": "rebufar",
    "rebuféis": "rebufar",
    "rebufó": "rebufar",
    "rebuja": "rebujar",
    "rebujaba": "rebujar",
    "rebujabais": "rebujar",
    "rebujaban": "rebujar",
    "rebujabas": "rebujar",
    "rebujad": "rebujar",
    "rebujada": "rebujar",
    "rebujadas": "rebujar",
    "rebujado": "rebujar",
    "rebujados": "rebujar",
    "rebujales": "rebujal",
    "rebujamos": "rebujar",
    "rebujan": "rebujar",
    "rebujando": "rebujar",
    "rebujara": "rebujar",
    "rebujarais": "rebujar",
    "rebujaran": "rebujar",
    "rebujaras": "rebujar",
    "rebujare": "rebujar",
    "rebujareis": "rebujar",
    "rebujaremos": "rebujar",
    "rebujaren": "rebujar",
    "rebujares": "rebujar",
    "rebujaron": "rebujar",
    "rebujará": "rebujar",
    "rebujarán": "rebujar",
    "rebujarás": "rebujar",
    "rebujaré": "rebujar",
    "rebujaréis": "rebujar",
    "rebujaría": "rebujar",
    "rebujaríais": "rebujar",
    "rebujaríamos": "rebujar",
    "rebujarían": "rebujar",
    "rebujarías": "rebujar",
    "rebujas": "rebujar",
    "rebujase": "rebujar",
    "rebujaseis": "rebujar",
    "rebujasen": "rebujar",
    "rebujases": "rebujar",
    "rebujaste": "rebujar",
    "rebujasteis": "rebujar",
    "rebuje": "rebujar",
    "rebujemos": "rebujar",
    "rebujen": "rebujar",
    "rebujes": "rebujar",
    "rebujinas": "rebujina",
    "rebujiñas": "rebujiña",
    "rebujo": "rebujar",
    "rebujos": "rebujo",
    "rebujábamos": "rebujar",
    "rebujáis": "rebujar",
    "rebujáramos": "rebujar",
    "rebujáremos": "rebujar",
    "rebujásemos": "rebujar",
    "rebujé": "rebujar",
    "rebujéis": "rebujar",
    "rebujó": "rebujar",
    "rebulla": "rebullir",
    "rebullamos": "rebullir",
    "rebullan": "rebullir",
    "rebullas": "rebullir",
    "rebulle": "rebullir",
    "rebullen": "rebullir",
    "rebullendo": "rebullir",
    "rebullera": "rebullir",
    "rebullerais": "rebullir",
    "rebulleran": "rebullir",
    "rebulleras": "rebullir",
    "rebullere": "rebullir",
    "rebullereis": "rebullir",
    "rebulleren": "rebullir",
    "rebulleres": "rebullir",
    "rebulleron": "rebullir",
    "rebulles": "rebullir",
    "rebullese": "rebullir",
    "rebulleseis": "rebullir",
    "rebullesen": "rebullir",
    "rebulleses": "rebullir",
    "rebullicios": "rebullicio",
    "rebullid": "rebullir",
    "rebullida": "rebullir",
    "rebullidas": "rebullir",
    "rebullido": "rebullir",
    "rebullidos": "rebullir",
    "rebullimos": "rebullir",
    "rebulliremos": "rebullir",
    "rebullirá": "rebullir",
    "rebullirán": "rebullir",
    "rebullirás": "rebullir",
    "rebulliré": "rebullir",
    "rebulliréis": "rebullir",
    "rebulliría": "rebullir",
    "rebulliríais": "rebullir",
    "rebulliríamos": "rebullir",
    "rebullirían": "rebullir",
    "rebullirías": "rebullir",
    "rebulliste": "rebullir",
    "rebullisteis": "rebullir",
    "rebullo": "rebullir",
    "rebulláis": "rebullir",
    "rebulléramos": "rebullir",
    "rebulléremos": "rebullir",
    "rebullésemos": "rebullir",
    "rebullí": "rebullir",
    "rebullía": "rebullir",
    "rebullíais": "rebullir",
    "rebullíamos": "rebullir",
    "rebullían": "rebullir",
    "rebullías": "rebullir",
    "rebullís": "rebullir",
    "rebulló": "rebullir",
    "rebumbios": "rebumbio",
    "reburuja": "reburujar",
    "reburujaba": "reburujar",
    "reburujabais": "reburujar",
    "reburujaban": "reburujar",
    "reburujabas": "reburujar",
    "reburujad": "reburujar",
    "reburujada": "reburujar",
    "reburujadas": "reburujar",
    "reburujado": "reburujar",
    "reburujados": "reburujar",
    "reburujamos": "reburujar",
    "reburujan": "reburujar",
    "reburujando": "reburujar",
    "reburujara": "reburujar",
    "reburujarais": "reburujar",
    "reburujaran": "reburujar",
    "reburujaras": "reburujar",
    "reburujare": "reburujar",
    "reburujareis": "reburujar",
    "reburujaremos": "reburujar",
    "reburujaren": "reburujar",
    "reburujares": "reburujar",
    "reburujaron": "reburujar",
    "reburujará": "reburujar",
    "reburujarán": "reburujar",
    "reburujarás": "reburujar",
    "reburujaré": "reburujar",
    "reburujaréis": "reburujar",
    "reburujaría": "reburujar",
    "reburujaríais": "reburujar",
    "reburujaríamos": "reburujar",
    "reburujarían": "reburujar",
    "reburujarías": "reburujar",
    "reburujas": "reburujar",
    "reburujase": "reburujar",
    "reburujaseis": "reburujar",
    "reburujasen": "reburujar",
    "reburujases": "reburujar",
    "reburujaste": "reburujar",
    "reburujasteis": "reburujar",
    "reburuje": "reburujar",
    "reburujemos": "reburujar",
    "reburujen": "reburujar",
    "reburujes": "reburujar",
    "reburujo": "reburujar",
    "reburujones": "reburujón",
    "reburujábamos": "reburujar",
    "reburujáis": "reburujar",
    "reburujáramos": "reburujar",
    "reburujáremos": "reburujar",
    "reburujásemos": "reburujar",
    "reburujé": "reburujar",
    "reburujéis": "reburujar",
    "reburujó": "reburujar",
    "rebusca": "rebuscar",
    "rebuscaba": "rebuscar",
    "rebuscabais": "rebuscar",
    "rebuscaban": "rebuscar",
    "rebuscabas": "rebuscar",
    "rebuscad": "rebuscar",
    "rebuscada": "rebuscar",
    "rebuscadas": "rebuscar",
    "rebuscado": "rebuscar",
    "rebuscadora": "rebuscador",
    "rebuscadoras": "rebuscador",
    "rebuscadores": "rebuscador",
    "rebuscados": "rebuscar",
    "rebuscamientos": "rebuscamiento",
    "rebuscamos": "rebuscar",
    "rebuscan": "rebuscar",
    "rebuscando": "rebuscar",
    "rebuscara": "rebuscar",
    "rebuscarais": "rebuscar",
    "rebuscaran": "rebuscar",
    "rebuscaras": "rebuscar",
    "rebuscare": "rebuscar",
    "rebuscareis": "rebuscar",
    "rebuscaremos": "rebuscar",
    "rebuscaren": "rebuscar",
    "rebuscares": "rebuscar",
    "rebuscaron": "rebuscar",
    "rebuscará": "rebuscar",
    "rebuscarán": "rebuscar",
    "rebuscarás": "rebuscar",
    "rebuscaré": "rebuscar",
    "rebuscaréis": "rebuscar",
    "rebuscaría": "rebuscar",
    "rebuscaríais": "rebuscar",
    "rebuscaríamos": "rebuscar",
    "rebuscarían": "rebuscar",
    "rebuscarías": "rebuscar",
    "rebuscas": "rebuscar",
    "rebuscase": "rebuscar",
    "rebuscaseis": "rebuscar",
    "rebuscasen": "rebuscar",
    "rebuscases": "rebuscar",
    "rebuscaste": "rebuscar",
    "rebuscasteis": "rebuscar",
    "rebusco": "rebuscar",
    "rebuscos": "rebusco",
    "rebuscábamos": "rebuscar",
    "rebuscáis": "rebuscar",
    "rebuscáramos": "rebuscar",
    "rebuscáremos": "rebuscar",
    "rebuscásemos": "rebuscar",
    "rebuscó": "rebuscar",
    "rebusque": "rebuscar",
    "rebusquemos": "rebuscar",
    "rebusquen": "rebuscar",
    "rebusques": "rebuscar",
    "rebusqué": "rebuscar",
    "rebusquéis": "rebuscar",
    "rebuzna": "rebuznar",
    "rebuznaba": "rebuznar",
    "rebuznabais": "rebuznar",
    "rebuznaban": "rebuznar",
    "rebuznabas": "rebuznar",
    "rebuznad": "rebuznar",
    "rebuznada": "rebuznar",
    "rebuznadas": "rebuznar",
    "rebuznado": "rebuznar",
    "rebuznados": "rebuznar",
    "rebuznamos": "rebuznar",
    "rebuznan": "rebuznar",
    "rebuznando": "rebuznar",
    "rebuznara": "rebuznar",
    "rebuznarais": "rebuznar",
    "rebuznaran": "rebuznar",
    "rebuznaras": "rebuznar",
    "rebuznare": "rebuznar",
    "rebuznareis": "rebuznar",
    "rebuznaremos": "rebuznar",
    "rebuznaren": "rebuznar",
    "rebuznares": "rebuznar",
    "rebuznaron": "rebuznar",
    "rebuznará": "rebuznar",
    "rebuznarán": "rebuznar",
    "rebuznarás": "rebuznar",
    "rebuznaré": "rebuznar",
    "rebuznaréis": "rebuznar",
    "rebuznaría": "rebuznar",
    "rebuznaríais": "rebuznar",
    "rebuznaríamos": "rebuznar",
    "rebuznarían": "rebuznar",
    "rebuznarías": "rebuznar",
    "rebuznas": "rebuznar",
    "rebuznase": "rebuznar",
    "rebuznaseis": "rebuznar",
    "rebuznasen": "rebuznar",
    "rebuznases": "rebuznar",
    "rebuznaste": "rebuznar",
    "rebuznasteis": "rebuznar",
    "rebuzne": "rebuznar",
    "rebuznemos": "rebuznar",
    "rebuznen": "rebuznar",
    "rebuznes": "rebuznar",
    "rebuzno": "rebuznar",
    "rebuznos": "rebuzno",
    "rebuznábamos": "rebuznar",
    "rebuznáis": "rebuznar",
    "rebuznáramos": "rebuznar",
    "rebuznáremos": "rebuznar",
    "rebuznásemos": "rebuznar",
    "rebuzné": "rebuznar",
    "rebuznéis": "rebuznar",
    "rebuznó": "rebuznar",
    "recaba": "recabar",
    "recababa": "recabar",
    "recababais": "recabar",
    "recababan": "recabar",
    "recababas": "recabar",
    "recabad": "recabar",
    "recabada": "recabar",
    "recabadas": "recabar",
    "recabado": "recabar",
    "recabados": "recabar",
    "recabamos": "recabar",
    "recaban": "recabar",
    "recabando": "recabar",
    "recabara": "recabar",
    "recabarais": "recabar",
    "recabaran": "recabar",
    "recabaras": "recabar",
    "recabare": "recabar",
    "recabareis": "recabar",
    "recabaremos": "recabar",
    "recabaren": "recabar",
    "recabares": "recabar",
    "recabaron": "recabar",
    "recabará": "recabar",
    "recabarán": "recabar",
    "recabarás": "recabar",
    "recabaré": "recabar",
    "recabaréis": "recabar",
    "recabaría": "recabar",
    "recabaríais": "recabar",
    "recabaríamos": "recabar",
    "recabarían": "recabar",
    "recabarías": "recabar",
    "recabas": "recabar",
    "recabase": "recabar",
    "recabaseis": "recabar",
    "recabasen": "recabar",
    "recabases": "recabar",
    "recabaste": "recabar",
    "recabasteis": "recabar",
    "recabe": "recabar",
    "recabemos": "recabar",
    "recaben": "recabar",
    "recabes": "recabar",
    "recableados": "recableado",
    "recabo": "recabar",
    "recabábamos": "recabar",
    "recabáis": "recabar",
    "recabáramos": "recabar",
    "recabáremos": "recabar",
    "recabásemos": "recabar",
    "recabé": "recabar",
    "recabéis": "recabar",
    "recabó": "recabar",
    "recadera": "recadero",
    "recaderas": "recadero",
    "recaderos": "recadero",
    "recadistas": "recadista",
    "recados": "recado",
    "recae": "recaer",
    "recaed": "recaer",
    "recaemos": "recaer",
    "recaen": "recaer",
    "recaeremos": "recaer",
    "recaerá": "recaer",
    "recaerán": "recaer",
    "recaerás": "recaer",
    "recaeré": "recaer",
    "recaeréis": "recaer",
    "recaería": "recaer",
    "recaeríais": "recaer",
    "recaeríamos": "recaer",
    "recaerían": "recaer",
    "recaerías": "recaer",
    "recaes": "recaer",
    "recaiga": "recaer",
    "recaigamos": "recaer",
    "recaigan": "recaer",
    "recaigas": "recaer",
    "recaigo": "recaer",
    "recaigáis": "recaer",
    "recala": "recalar",
    "recalaba": "recalar",
    "recalabais": "recalar",
    "recalaban": "recalar",
    "recalabas": "recalar",
    "recalad": "recalar",
    "recalada": "recalar",
    "recaladas": "recalar",
    "recalado": "recalar",
    "recalados": "recalar",
    "recalamos": "recalar",
    "recalan": "recalar",
    "recalando": "recalar",
    "recalara": "recalar",
    "recalarais": "recalar",
    "recalaran": "recalar",
    "recalaras": "recalar",
    "recalare": "recalar",
    "recalareis": "recalar",
    "recalaremos": "recalar",
    "recalaren": "recalar",
    "recalares": "recalar",
    "recalaron": "recalar",
    "recalará": "recalar",
    "recalarán": "recalar",
    "recalarás": "recalar",
    "recalaré": "recalar",
    "recalaréis": "recalar",
    "recalaría": "recalar",
    "recalaríais": "recalar",
    "recalaríamos": "recalar",
    "recalarían": "recalar",
    "recalarías": "recalar",
    "recalas": "recalar",
    "recalase": "recalar",
    "recalaseis": "recalar",
    "recalasen": "recalar",
    "recalases": "recalar",
    "recalaste": "recalar",
    "recalasteis": "recalar",
    "recalca": "recalcar",
    "recalcaba": "recalcar",
    "recalcabais": "recalcar",
    "recalcaban": "recalcar",
    "recalcabas": "recalcar",
    "recalcad": "recalcar",
    "recalcada": "recalcar",
    "recalcadas": "recalcar",
    "recalcado": "recalcar",
    "recalcadora": "recalcador",
    "recalcadoras": "recalcadora",
    "recalcadores": "recalcador",
    "recalcados": "recalcar",
    "recalcaduras": "recalcadura",
    "recalcamos": "recalcar",
    "recalcan": "recalcar",
    "recalcando": "recalcar",
    "recalcara": "recalcar",
    "recalcarais": "recalcar",
    "recalcaran": "recalcar",
    "recalcaras": "recalcar",
    "recalcare": "recalcar",
    "recalcareis": "recalcar",
    "recalcaremos": "recalcar",
    "recalcaren": "recalcar",
    "recalcares": "recalcar",
    "recalcaron": "recalcar",
    "recalcará": "recalcar",
    "recalcarán": "recalcar",
    "recalcarás": "recalcar",
    "recalcaré": "recalcar",
    "recalcaréis": "recalcar",
    "recalcaría": "recalcar",
    "recalcaríais": "recalcar",
    "recalcaríamos": "recalcar",
    "recalcarían": "recalcar",
    "recalcarías": "recalcar",
    "recalcas": "recalcar",
    "recalcase": "recalcar",
    "recalcaseis": "recalcar",
    "recalcasen": "recalcar",
    "recalcases": "recalcar",
    "recalcaste": "recalcar",
    "recalcasteis": "recalcar",
    "recalce": "recalzar",
    "recalcemos": "recalzar",
    "recalcen": "recalzar",
    "recalces": "recalzar",
    "recalcificaciones": "recalcificación",
    "recalcitra": "recalcitrar",
    "recalcitraba": "recalcitrar",
    "recalcitrabais": "recalcitrar",
    "recalcitraban": "recalcitrar",
    "recalcitrabas": "recalcitrar",
    "recalcitrad": "recalcitrar",
    "recalcitrada": "recalcitrar",
    "recalcitradas": "recalcitrar",
    "recalcitrado": "recalcitrar",
    "recalcitrados": "recalcitrar",
    "recalcitramos": "recalcitrar",
    "recalcitran": "recalcitrar",
    "recalcitrando": "recalcitrar",
    "recalcitrantes": "recalcitrante",
    "recalcitrara": "recalcitrar",
    "recalcitrarais": "recalcitrar",
    "recalcitraran": "recalcitrar",
    "recalcitraras": "recalcitrar",
    "recalcitrare": "recalcitrar",
    "recalcitrareis": "recalcitrar",
    "recalcitraremos": "recalcitrar",
    "recalcitraren": "recalcitrar",
    "recalcitrares": "recalcitrar",
    "recalcitraron": "recalcitrar",
    "recalcitrará": "recalcitrar",
    "recalcitrarán": "recalcitrar",
    "recalcitrarás": "recalcitrar",
    "recalcitraré": "recalcitrar",
    "recalcitraréis": "recalcitrar",
    "recalcitraría": "recalcitrar",
    "recalcitraríais": "recalcitrar",
    "recalcitraríamos": "recalcitrar",
    "recalcitrarían": "recalcitrar",
    "recalcitrarías": "recalcitrar",
    "recalcitras": "recalcitrar",
    "recalcitrase": "recalcitrar",
    "recalcitraseis": "recalcitrar",
    "recalcitrasen": "recalcitrar",
    "recalcitrases": "recalcitrar",
    "recalcitraste": "recalcitrar",
    "recalcitrasteis": "recalcitrar",
    "recalcitre": "recalcitrar",
    "recalcitremos": "recalcitrar",
    "recalcitren": "recalcitrar",
    "recalcitres": "recalcitrar",
    "recalcitro": "recalcitrar",
    "recalcitrábamos": "recalcitrar",
    "recalcitráis": "recalcitrar",
    "recalcitráramos": "recalcitrar",
    "recalcitráremos": "recalcitrar",
    "recalcitrásemos": "recalcitrar",
    "recalcitré": "recalcitrar",
    "recalcitréis": "recalcitrar",
    "recalcitró": "recalcitrar",
    "recalco": "recalcar",
    "recalcábamos": "recalcar",
    "recalcáis": "recalcar",
    "recalcáramos": "recalcar",
    "recalcáremos": "recalcar",
    "recalcásemos": "recalcar",
    "recalcé": "recalzar",
    "recalcéis": "recalzar",
    "recalcó": "recalcar",
    "recale": "recalar",
    "recalemos": "recalar",
    "recalen": "recalar",
    "recalentaba": "recalentar",
    "recalentabais": "recalentar",
    "recalentaban": "recalentar",
    "recalentabas": "recalentar",
    "recalentad": "recalentar",
    "recalentada": "recalentar",
    "recalentadas": "recalentar",
    "recalentado": "recalentar",
    "recalentadora": "recalentador",
    "recalentadoras": "recalentador",
    "recalentadores": "recalentador",
    "recalentados": "recalentar",
    "recalentamientos": "recalentamiento",
    "recalentamos": "recalentar",
    "recalentando": "recalentar",
    "recalentara": "recalentar",
    "recalentarais": "recalentar",
    "recalentaran": "recalentar",
    "recalentaras": "recalentar",
    "recalentare": "recalentar",
    "recalentareis": "recalentar",
    "recalentaremos": "recalentar",
    "recalentaren": "recalentar",
    "recalentares": "recalentar",
    "recalentaron": "recalentar",
    "recalentará": "recalentar",
    "recalentarán": "recalentar",
    "recalentarás": "recalentar",
    "recalentaré": "recalentar",
    "recalentaréis": "recalentar",
    "recalentaría": "recalentar",
    "recalentaríais": "recalentar",
    "recalentaríamos": "recalentar",
    "recalentarían": "recalentar",
    "recalentarías": "recalentar",
    "recalentase": "recalentar",
    "recalentaseis": "recalentar",
    "recalentasen": "recalentar",
    "recalentases": "recalentar",
    "recalentaste": "recalentar",
    "recalentasteis": "recalentar",
    "recalentemos": "recalentar",
    "recalentones": "recalentón",
    "recalentábamos": "recalentar",
    "recalentáis": "recalentar",
    "recalentáramos": "recalentar",
    "recalentáremos": "recalentar",
    "recalentásemos": "recalentar",
    "recalenté": "recalentar",
    "recalentéis": "recalentar",
    "recalentó": "recalentar",
    "recales": "recalar",
    "recalescencias": "recalescencia",
    "recalescentes": "recalescente",
    "recalienta": "recalentar",
    "recalientan": "recalentar",
    "recalientas": "recalentar",
    "recaliente": "recalentar",
    "recalienten": "recalentar",
    "recalientes": "recalentar",
    "recaliento": "recalentar",
    "recalificaciones": "recalificación",
    "recalmones": "recalmón",
    "recalo": "recalar",
    "recalque": "recalcar",
    "recalquemos": "recalcar",
    "recalquen": "recalcar",
    "recalques": "recalcar",
    "recalqué": "recalcar",
    "recalquéis": "recalcar",
    "recalza": "recalzar",
    "recalzaba": "recalzar",
    "recalzabais": "recalzar",
    "recalzaban": "recalzar",
    "recalzabas": "recalzar",
    "recalzad": "recalzar",
    "recalzada": "recalzar",
    "recalzadas": "recalzar",
    "recalzado": "recalzar",
    "recalzados": "recalzar",
    "recalzamos": "recalzar",
    "recalzan": "recalzar",
    "recalzando": "recalzar",
    "recalzara": "recalzar",
    "recalzarais": "recalzar",
    "recalzaran": "recalzar",
    "recalzaras": "recalzar",
    "recalzare": "recalzar",
    "recalzareis": "recalzar",
    "recalzaremos": "recalzar",
    "recalzaren": "recalzar",
    "recalzares": "recalzar",
    "recalzaron": "recalzar",
    "recalzará": "recalzar",
    "recalzarán": "recalzar",
    "recalzarás": "recalzar",
    "recalzaré": "recalzar",
    "recalzaréis": "recalzar",
    "recalzaría": "recalzar",
    "recalzaríais": "recalzar",
    "recalzaríamos": "recalzar",
    "recalzarían": "recalzar",
    "recalzarías": "recalzar",
    "recalzas": "recalzar",
    "recalzase": "recalzar",
    "recalzaseis": "recalzar",
    "recalzasen": "recalzar",
    "recalzases": "recalzar",
    "recalzaste": "recalzar",
    "recalzasteis": "recalzar",
    "recalzo": "recalzar",
    "recalzones": "recalzón",
    "recalzos": "recalzo",
    "recalzábamos": "recalzar",
    "recalzáis": "recalzar",
    "recalzáramos": "recalzar",
    "recalzáremos": "recalzar",
    "recalzásemos": "recalzar",
    "recalzó": "recalzar",
    "recalábamos": "recalar",
    "recaláis": "recalar",
    "recaláramos": "recalar",
    "recaláremos": "recalar",
    "recalásemos": "recalar",
    "recalé": "recalar",
    "recaléis": "recalar",
    "recaló": "recalar",
    "recama": "recamar",
    "recamaba": "recamar",
    "recamabais": "recamar",
    "recamaban": "recamar",
    "recamabas": "recamar",
    "recamad": "recamar",
    "recamada": "recamar",
    "recamadas": "recamar",
    "recamado": "recamar",
    "recamadora": "recamador",
    "recamadoras": "recamador",
    "recamadores": "recamador",
    "recamados": "recamar",
    "recamamos": "recamar",
    "recaman": "recamar",
    "recamando": "recamar",
    "recamara": "recamar",
    "recamarais": "recamar",
    "recamaran": "recamar",
    "recamaras": "recamar",
    "recamare": "recamar",
    "recamareis": "recamar",
    "recamaremos": "recamar",
    "recamaren": "recamar",
    "recamares": "recamar",
    "recamaron": "recamar",
    "recamará": "recamar",
    "recamarán": "recamar",
    "recamarás": "recamar",
    "recamaré": "recamar",
    "recamaréis": "recamar",
    "recamaría": "recamar",
    "recamaríais": "recamar",
    "recamaríamos": "recamar",
    "recamarían": "recamar",
    "recamarías": "recamar",
    "recamas": "recamar",
    "recamase": "recamar",
    "recamaseis": "recamar",
    "recamasen": "recamar",
    "recamases": "recamar",
    "recamaste": "recamar",
    "recamasteis": "recamar",
    "recambia": "recambiar",
    "recambiaba": "recambiar",
    "recambiabais": "recambiar",
    "recambiaban": "recambiar",
    "recambiabas": "recambiar",
    "recambiables": "recambiable",
    "recambiad": "recambiar",
    "recambiada": "recambiar",
    "recambiadas": "recambiar",
    "recambiado": "recambiar",
    "recambiados": "recambiar",
    "recambiamos": "recambiar",
    "recambian": "recambiar",
    "recambiando": "recambiar",
    "recambiara": "recambiar",
    "recambiarais": "recambiar",
    "recambiaran": "recambiar",
    "recambiaras": "recambiar",
    "recambiare": "recambiar",
    "recambiareis": "recambiar",
    "recambiaremos": "recambiar",
    "recambiaren": "recambiar",
    "recambiares": "recambiar",
    "recambiaron": "recambiar",
    "recambiará": "recambiar",
    "recambiarán": "recambiar",
    "recambiarás": "recambiar",
    "recambiaré": "recambiar",
    "recambiaréis": "recambiar",
    "recambiaría": "recambiar",
    "recambiaríais": "recambiar",
    "recambiaríamos": "recambiar",
    "recambiarían": "recambiar",
    "recambiarías": "recambiar",
    "recambias": "recambiar",
    "recambiase": "recambiar",
    "recambiaseis": "recambiar",
    "recambiasen": "recambiar",
    "recambiases": "recambiar",
    "recambiaste": "recambiar",
    "recambiasteis": "recambiar",
    "recambie": "recambiar",
    "recambiemos": "recambiar",
    "recambien": "recambiar",
    "recambies": "recambiar",
    "recambio": "recambiar",
    "recambios": "recambio",
    "recambiábamos": "recambiar",
    "recambiáis": "recambiar",
    "recambiáramos": "recambiar",
    "recambiáremos": "recambiar",
    "recambiásemos": "recambiar",
    "recambié": "recambiar",
    "recambiéis": "recambiar",
    "recambió": "recambiar",
    "recame": "recamar",
    "recamemos": "recamar",
    "recamen": "recamar",
    "recames": "recamar",
    "recamo": "recamar",
    "recamábamos": "recamar",
    "recamáis": "recamar",
    "recamáramos": "recamar",
    "recamáremos": "recamar",
    "recamásemos": "recamar",
    "recamé": "recamar",
    "recaméis": "recamar",
    "recamó": "recamar",
    "recancamusas": "recancamusa",
    "recancanillas": "recancanilla",
    "recantaciones": "recantación",
    "recantones": "recantón",
    "recapacita": "recapacitar",
    "recapacitaba": "recapacitar",
    "recapacitabais": "recapacitar",
    "recapacitaban": "recapacitar",
    "recapacitabas": "recapacitar",
    "recapacitad": "recapacitar",
    "recapacitada": "recapacitar",
    "recapacitadas": "recapacitar",
    "recapacitado": "recapacitar",
    "recapacitados": "recapacitar",
    "recapacitamos": "recapacitar",
    "recapacitan": "recapacitar",
    "recapacitando": "recapacitar",
    "recapacitara": "recapacitar",
    "recapacitarais": "recapacitar",
    "recapacitaran": "recapacitar",
    "recapacitaras": "recapacitar",
    "recapacitare": "recapacitar",
    "recapacitareis": "recapacitar",
    "recapacitaremos": "recapacitar",
    "recapacitaren": "recapacitar",
    "recapacitares": "recapacitar",
    "recapacitaron": "recapacitar",
    "recapacitará": "recapacitar",
    "recapacitarán": "recapacitar",
    "recapacitarás": "recapacitar",
    "recapacitaré": "recapacitar",
    "recapacitaréis": "recapacitar",
    "recapacitaría": "recapacitar",
    "recapacitaríais": "recapacitar",
    "recapacitaríamos": "recapacitar",
    "recapacitarían": "recapacitar",
    "recapacitarías": "recapacitar",
    "recapacitas": "recapacitar",
    "recapacitase": "recapacitar",
    "recapacitaseis": "recapacitar",
    "recapacitasen": "recapacitar",
    "recapacitases": "recapacitar",
    "recapacitaste": "recapacitar",
    "recapacitasteis": "recapacitar",
    "recapacite": "recapacitar",
    "recapacitemos": "recapacitar",
    "recapaciten": "recapacitar",
    "recapacites": "recapacitar",
    "recapacito": "recapacitar",
    "recapacitábamos": "recapacitar",
    "recapacitáis": "recapacitar",
    "recapacitáramos": "recapacitar",
    "recapacitáremos": "recapacitar",
    "recapacitásemos": "recapacitar",
    "recapacité": "recapacitar",
    "recapacitéis": "recapacitar",
    "recapacitó": "recapacitar",
    "recapitalizaciones": "recapitalización",
    "recapitula": "recapitular",
    "recapitulaba": "recapitular",
    "recapitulabais": "recapitular",
    "recapitulaban": "recapitular",
    "recapitulabas": "recapitular",
    "recapitulaciones": "recapitulación",
    "recapitulad": "recapitular",
    "recapitulada": "recapitular",
    "recapituladas": "recapitular",
    "recapitulado": "recapitular",
    "recapitulados": "recapitular",
    "recapitulamos": "recapitular",
    "recapitulan": "recapitular",
    "recapitulando": "recapitular",
    "recapitulara": "recapitular",
    "recapitularais": "recapitular",
    "recapitularan": "recapitular",
    "recapitularas": "recapitular",
    "recapitulare": "recapitular",
    "recapitulareis": "recapitular",
    "recapitularemos": "recapitular",
    "recapitularen": "recapitular",
    "recapitulares": "recapitular",
    "recapitularon": "recapitular",
    "recapitulará": "recapitular",
    "recapitularán": "recapitular",
    "recapitularás": "recapitular",
    "recapitularé": "recapitular",
    "recapitularéis": "recapitular",
    "recapitularía": "recapitular",
    "recapitularíais": "recapitular",
    "recapitularíamos": "recapitular",
    "recapitularían": "recapitular",
    "recapitularías": "recapitular",
    "recapitulas": "recapitular",
    "recapitulase": "recapitular",
    "recapitulaseis": "recapitular",
    "recapitulasen": "recapitular",
    "recapitulases": "recapitular",
    "recapitulaste": "recapitular",
    "recapitulasteis": "recapitular",
    "recapitule": "recapitular",
    "recapitulemos": "recapitular",
    "recapitulen": "recapitular",
    "recapitules": "recapitular",
    "recapitulo": "recapitular",
    "recapitulábamos": "recapitular",
    "recapituláis": "recapitular",
    "recapituláramos": "recapitular",
    "recapituláremos": "recapitular",
    "recapitulásemos": "recapitular",
    "recapitulé": "recapitular",
    "recapituléis": "recapitular",
    "recapituló": "recapitular",
    "recaptaciones": "recaptación",
    "recarburaciones": "recarburación",
    "recarga": "recargar",
    "recargaba": "recargar",
    "recargabais": "recargar",
    "recargaban": "recargar",
    "recargabas": "recargar",
    "recargables": "recargable",
    "recargad": "recargar",
    "recargada": "recargar",
    "recargadas": "recargar",
    "recargado": "recargar",
    "recargadores": "recargador",
    "recargados": "recargar",
    "recargamos": "recargar",
    "recargan": "recargar",
    "recargando": "recargar",
    "recargara": "recargar",
    "recargarais": "recargar",
    "recargaran": "recargar",
    "recargaras": "recargar",
    "recargare": "recargar",
    "recargareis": "recargar",
    "recargaremos": "recargar",
    "recargaren": "recargar",
    "recargares": "recargar",
    "recargaron": "recargar",
    "recargará": "recargar",
    "recargarán": "recargar",
    "recargarás": "recargar",
    "recargaré": "recargar",
    "recargaréis": "recargar",
    "recargaría": "recargar",
    "recargaríais": "recargar",
    "recargaríamos": "recargar",
    "recargarían": "recargar",
    "recargarías": "recargar",
    "recargas": "recargar",
    "recargase": "recargar",
    "recargaseis": "recargar",
    "recargasen": "recargar",
    "recargases": "recargar",
    "recargaste": "recargar",
    "recargasteis": "recargar",
    "recargo": "recargar",
    "recargos": "recargo",
    "recargue": "recargar",
    "recarguemos": "recargar",
    "recarguen": "recargar",
    "recargues": "recargar",
    "recargué": "recargar",
    "recarguéis": "recargar",
    "recargábamos": "recargar",
    "recargáis": "recargar",
    "recargáramos": "recargar",
    "recargáremos": "recargar",
    "recargásemos": "recargar",
    "recargó": "recargar",
    "recata": "recatar",
    "recataba": "recatar",
    "recatabais": "recatar",
    "recataban": "recatar",
    "recatabas": "recatar",
    "recatad": "recatar",
    "recatada": "recatar",
    "recatadas": "recatar",
    "recatado": "recatar",
    "recatados": "recatar",
    "recatamos": "recatar",
    "recatan": "recatar",
    "recatando": "recatar",
    "recatara": "recatar",
    "recatarais": "recatar",
    "recataran": "recatar",
    "recataras": "recatar",
    "recatare": "recatar",
    "recatareis": "recatar",
    "recataremos": "recatar",
    "recataren": "recatar",
    "recatares": "recatar",
    "recataron": "recatar",
    "recatará": "recatar",
    "recatarán": "recatar",
    "recatarás": "recatar",
    "recataré": "recatar",
    "recataréis": "recatar",
    "recataría": "recatar",
    "recataríais": "recatar",
    "recataríamos": "recatar",
    "recatarían": "recatar",
    "recatarías": "recatar",
    "recatas": "recatar",
    "recatase": "recatar",
    "recataseis": "recatar",
    "recatasen": "recatar",
    "recatases": "recatar",
    "recataste": "recatar",
    "recatasteis": "recatar",
    "recate": "recatar",
    "recatemos": "recatar",
    "recaten": "recatar",
    "recates": "recatar",
    "recato": "recatar",
    "recatos": "recato",
    "recatábamos": "recatar",
    "recatáis": "recatar",
    "recatáramos": "recatar",
    "recatáremos": "recatar",
    "recatásemos": "recatar",
    "recaté": "recatar",
    "recatéis": "recatar",
    "recató": "recatar",
    "recauchuta": "recauchutar",
    "recauchutaba": "recauchutar",
    "recauchutabais": "recauchutar",
    "recauchutaban": "recauchutar",
    "recauchutabas": "recauchutar",
    "recauchutad": "recauchutar",
    "recauchutada": "recauchutar",
    "recauchutadas": "recauchutar",
    "recauchutado": "recauchutar",
    "recauchutados": "recauchutar",
    "recauchutamos": "recauchutar",
    "recauchutan": "recauchutar",
    "recauchutando": "recauchutar",
    "recauchutara": "recauchutar",
    "recauchutarais": "recauchutar",
    "recauchutaran": "recauchutar",
    "recauchutaras": "recauchutar",
    "recauchutare": "recauchutar",
    "recauchutareis": "recauchutar",
    "recauchutaremos": "recauchutar",
    "recauchutaren": "recauchutar",
    "recauchutares": "recauchutar",
    "recauchutaron": "recauchutar",
    "recauchutará": "recauchutar",
    "recauchutarán": "recauchutar",
    "recauchutarás": "recauchutar",
    "recauchutaré": "recauchutar",
    "recauchutaréis": "recauchutar",
    "recauchutaría": "recauchutar",
    "recauchutaríais": "recauchutar",
    "recauchutaríamos": "recauchutar",
    "recauchutarían": "recauchutar",
    "recauchutarías": "recauchutar",
    "recauchutas": "recauchutar",
    "recauchutase": "recauchutar",
    "recauchutaseis": "recauchutar",
    "recauchutasen": "recauchutar",
    "recauchutases": "recauchutar",
    "recauchutaste": "recauchutar",
    "recauchutasteis": "recauchutar",
    "recauchute": "recauchutar",
    "recauchutemos": "recauchutar",
    "recauchuten": "recauchutar",
    "recauchutes": "recauchutar",
    "recauchuto": "recauchutar",
    "recauchutábamos": "recauchutar",
    "recauchutáis": "recauchutar",
    "recauchutáramos": "recauchutar",
    "recauchutáremos": "recauchutar",
    "recauchutásemos": "recauchutar",
    "recauchuté": "recauchutar",
    "recauchutéis": "recauchutar",
    "recauchutó": "recauchutar",
    "recauda": "recaudar",
    "recaudaba": "recaudar",
    "recaudabais": "recaudar",
    "recaudaban": "recaudar",
    "recaudabas": "recaudar",
    "recaudaciones": "recaudación",
    "recaudad": "recaudar",
    "recaudada": "recaudar",
    "recaudadas": "recaudar",
    "recaudado": "recaudar",
    "recaudadora": "recaudador",
    "recaudadoras": "recaudador",
    "recaudadores": "recaudador",
    "recaudados": "recaudar",
    "recaudamientos": "recaudamiento",
    "recaudamos": "recaudar",
    "recaudan": "recaudar",
    "recaudando": "recaudar",
    "recaudara": "recaudar",
    "recaudarais": "recaudar",
    "recaudaran": "recaudar",
    "recaudaras": "recaudar",
    "recaudare": "recaudar",
    "recaudareis": "recaudar",
    "recaudaremos": "recaudar",
    "recaudaren": "recaudar",
    "recaudares": "recaudar",
    "recaudaron": "recaudar",
    "recaudará": "recaudar",
    "recaudarán": "recaudar",
    "recaudarás": "recaudar",
    "recaudaré": "recaudar",
    "recaudaréis": "recaudar",
    "recaudaría": "recaudar",
    "recaudaríais": "recaudar",
    "recaudaríamos": "recaudar",
    "recaudarían": "recaudar",
    "recaudarías": "recaudar",
    "recaudas": "recaudar",
    "recaudase": "recaudar",
    "recaudaseis": "recaudar",
    "recaudasen": "recaudar",
    "recaudases": "recaudar",
    "recaudaste": "recaudar",
    "recaudasteis": "recaudar",
    "recaudatoria": "recaudatorio",
    "recaudatorias": "recaudatorio",
    "recaudatorios": "recaudatorio",
    "recaude": "recaudar",
    "recaudemos": "recaudar",
    "recauden": "recaudar",
    "recaudes": "recaudar",
    "recaudo": "recaudar",
    "recaudos": "recaudo",
    "recaudábamos": "recaudar",
    "recaudáis": "recaudar",
    "recaudáramos": "recaudar",
    "recaudáremos": "recaudar",
    "recaudásemos": "recaudar",
    "recaudé": "recaudar",
    "recaudéis": "recaudar",
    "recaudó": "recaudar",
    "recava": "recavar",
    "recavaba": "recavar",
    "recavabais": "recavar",
    "recavaban": "recavar",
    "recavabas": "recavar",
    "recavad": "recavar",
    "recavada": "recavar",
    "recavadas": "recavar",
    "recavado": "recavar",
    "recavados": "recavar",
    "recavamos": "recavar",
    "recavan": "recavar",
    "recavando": "recavar",
    "recavara": "recavar",
    "recavarais": "recavar",
    "recavaran": "recavar",
    "recavaras": "recavar",
    "recavare": "recavar",
    "recavareis": "recavar",
    "recavaremos": "recavar",
    "recavaren": "recavar",
    "recavares": "recavar",
    "recavaron": "recavar",
    "recavará": "recavar",
    "recavarán": "recavar",
    "recavarás": "recavar",
    "recavaré": "recavar",
    "recavaréis": "recavar",
    "recavaría": "recavar",
    "recavaríais": "recavar",
    "recavaríamos": "recavar",
    "recavarían": "recavar",
    "recavarías": "recavar",
    "recavas": "recavar",
    "recavase": "recavar",
    "recavaseis": "recavar",
    "recavasen": "recavar",
    "recavases": "recavar",
    "recavaste": "recavar",
    "recavasteis": "recavar",
    "recave": "recavar",
    "recavemos": "recavar",
    "recaven": "recavar",
    "recaves": "recavar",
    "recavo": "recavar",
    "recavábamos": "recavar",
    "recaváis": "recavar",
    "recaváramos": "recavar",
    "recaváremos": "recavar",
    "recavásemos": "recavar",
    "recavé": "recavar",
    "recavéis": "recavar",
    "recavó": "recavar",
    "recayendo": "recaer",
    "recayera": "recaer",
    "recayerais": "recaer",
    "recayeran": "recaer",
    "recayeras": "recaer",
    "recayere": "recaer",
    "recayereis": "recaer",
    "recayeren": "recaer",
    "recayeres": "recaer",
    "recayeron": "recaer",
    "recayese": "recaer",
    "recayeseis": "recaer",
    "recayesen": "recaer",
    "recayeses": "recaer",
    "recayéramos": "recaer",
    "recayéremos": "recaer",
    "recayésemos": "recaer",
    "recayó": "recaer",
    "recazos": "recazo",
    "recaéis": "recaer",
    "recaí": "recaer",
    "recaía": "recaer",
    "recaíais": "recaer",
    "recaíamos": "recaer",
    "recaían": "recaer",
    "recaías": "recaer",
    "recaída": "recaer",
    "recaídas": "recaída",
    "recaído": "recaer",
    "recaídos": "recaer",
    "recaímos": "recaer",
    "recaíste": "recaer",
    "recaísteis": "recaer",
    "rece": "rezar",
    "receba": "recebar",
    "recebaba": "recebar",
    "recebabais": "recebar",
    "recebaban": "recebar",
    "recebabas": "recebar",
    "recebad": "recebar",
    "recebada": "recebar",
    "recebadas": "recebar",
    "recebado": "recebar",
    "recebados": "recebar",
    "recebamos": "recebar",
    "receban": "recebar",
    "recebando": "recebar",
    "recebara": "recebar",
    "recebarais": "recebar",
    "recebaran": "recebar",
    "recebaras": "recebar",
    "recebare": "recebar",
    "recebareis": "recebar",
    "recebaremos": "recebar",
    "recebaren": "recebar",
    "recebares": "recebar",
    "recebaron": "recebar",
    "recebará": "recebar",
    "recebarán": "recebar",
    "recebarás": "recebar",
    "recebaré": "recebar",
    "recebaréis": "recebar",
    "recebaría": "recebar",
    "recebaríais": "recebar",
    "recebaríamos": "recebar",
    "recebarían": "recebar",
    "recebarías": "recebar",
    "recebas": "recebar",
    "recebase": "recebar",
    "recebaseis": "recebar",
    "recebasen": "recebar",
    "recebases": "recebar",
    "recebaste": "recebar",
    "recebasteis": "recebar",
    "recebe": "recebar",
    "recebemos": "recebar",
    "receben": "recebar",
    "recebes": "recebar",
    "recebo": "recebar",
    "recebos": "recebo",
    "recebábamos": "recebar",
    "recebáis": "recebar",
    "recebáramos": "recebar",
    "recebáremos": "recebar",
    "recebásemos": "recebar",
    "recebé": "recebar",
    "recebéis": "recebar",
    "recebó": "recebar",
    "recechos": "rececho",
    "recela": "recelar",
    "recelaba": "recelar",
    "recelabais": "recelar",
    "recelaban": "recelar",
    "recelabas": "recelar",
    "recelad": "recelar",
    "recelada": "recelar",
    "receladas": "recelar",
    "recelado": "recelar",
    "recelados": "recelar",
    "recelamientos": "recelamiento",
    "recelamos": "recelar",
    "recelan": "recelar",
    "recelando": "recelar",
    "recelara": "recelar",
    "recelarais": "recelar",
    "recelaran": "recelar",
    "recelaras": "recelar",
    "recelare": "recelar",
    "recelareis": "recelar",
    "recelaremos": "recelar",
    "recelaren": "recelar",
    "recelares": "recelar",
    "recelaron": "recelar",
    "recelará": "recelar",
    "recelarán": "recelar",
    "recelarás": "recelar",
    "recelaré": "recelar",
    "recelaréis": "recelar",
    "recelaría": "recelar",
    "recelaríais": "recelar",
    "recelaríamos": "recelar",
    "recelarían": "recelar",
    "recelarías": "recelar",
    "recelas": "recelar",
    "recelase": "recelar",
    "recelaseis": "recelar",
    "recelasen": "recelar",
    "recelases": "recelar",
    "recelaste": "recelar",
    "recelasteis": "recelar",
    "recele": "recelar",
    "recelemos": "recelar",
    "recelen": "recelar",
    "receles": "recelar",
    "recelo": "recelar",
    "recelos": "recelo",
    "recelosa": "receloso",
    "recelosas": "receloso",
    "recelosos": "receloso",
    "recelábamos": "recelar",
    "receláis": "recelar",
    "receláramos": "recelar",
    "receláremos": "recelar",
    "recelásemos": "recelar",
    "recelé": "recelar",
    "receléis": "recelar",
    "receló": "recelar",
    "recemos": "rezar",
    "recen": "rezar",
    "recensiones": "recensión",
    "recensora": "recensor",
    "recensoras": "recensor",
    "recensores": "recensor",
    "recentaba": "recentar",
    "recentabais": "recentar",
    "recentaban": "recentar",
    "recentabas": "recentar",
    "recentad": "recentar",
    "recentada": "recentar",
    "recentadas": "recentar",
    "recentado": "recentar",
    "recentados": "recentar",
    "recentaduras": "recentadura",
    "recentales": "recental",
    "recentamos": "recentar",
    "recentando": "recentar",
    "recentara": "recentar",
    "recentarais": "recentar",
    "recentaran": "recentar",
    "recentaras": "recentar",
    "recentare": "recentar",
    "recentareis": "recentar",
    "recentaremos": "recentar",
    "recentaren": "recentar",
    "recentares": "recentar",
    "recentaron": "recentar",
    "recentará": "recentar",
    "recentarán": "recentar",
    "recentarás": "recentar",
    "recentaré": "recentar",
    "recentaréis": "recentar",
    "recentaría": "recentar",
    "recentaríais": "recentar",
    "recentaríamos": "recentar",
    "recentarían": "recentar",
    "recentarías": "recentar",
    "recentase": "recentar",
    "recentaseis": "recentar",
    "recentasen": "recentar",
    "recentases": "recentar",
    "recentaste": "recentar",
    "recentasteis": "recentar",
    "recentemos": "recentar",
    "recentines": "recentín",
    "recentábamos": "recentar",
    "recentáis": "recentar",
    "recentáramos": "recentar",
    "recentáremos": "recentar",
    "recentásemos": "recentar",
    "recenté": "recentar",
    "recentéis": "recentar",
    "recentó": "recentar",
    "recepciona": "recepcionar",
    "recepcionaba": "recepcionar",
    "recepcionabais": "recepcionar",
    "recepcionaban": "recepcionar",
    "recepcionabas": "recepcionar",
    "recepcionad": "recepcionar",
    "recepcionada": "recepcionar",
    "recepcionadas": "recepcionar",
    "recepcionado": "recepcionar",
    "recepcionados": "recepcionar",
    "recepcionamos": "recepcionar",
    "recepcionan": "recepcionar",
    "recepcionando": "recepcionar",
    "recepcionara": "recepcionar",
    "recepcionarais": "recepcionar",
    "recepcionaran": "recepcionar",
    "recepcionaras": "recepcionar",
    "recepcionare": "recepcionar",
    "recepcionareis": "recepcionar",
    "recepcionaremos": "recepcionar",
    "recepcionaren": "recepcionar",
    "recepcionares": "recepcionar",
    "recepcionaron": "recepcionar",
    "recepcionará": "recepcionar",
    "recepcionarán": "recepcionar",
    "recepcionarás": "recepcionar",
    "recepcionaré": "recepcionar",
    "recepcionaréis": "recepcionar",
    "recepcionaría": "recepcionar",
    "recepcionaríais": "recepcionar",
    "recepcionaríamos": "recepcionar",
    "recepcionarían": "recepcionar",
    "recepcionarías": "recepcionar",
    "recepcionas": "recepcionar",
    "recepcionase": "recepcionar",
    "recepcionaseis": "recepcionar",
    "recepcionasen": "recepcionar",
    "recepcionases": "recepcionar",
    "recepcionaste": "recepcionar",
    "recepcionasteis": "recepcionar",
    "recepcione": "recepcionar",
    "recepcionemos": "recepcionar",
    "recepcionen": "recepcionar",
    "recepciones": "recepcionar",
    "recepcionistas": "recepcionista",
    "recepciono": "recepcionar",
    "recepcionábamos": "recepcionar",
    "recepcionáis": "recepcionar",
    "recepcionáramos": "recepcionar",
    "recepcionáremos": "recepcionar",
    "recepcionásemos": "recepcionar",
    "recepcioné": "recepcionar",
    "recepcionéis": "recepcionar",
    "recepcionó": "recepcionar",
    "recepta": "receptar",
    "receptaba": "receptar",
    "receptabais": "receptar",
    "receptaban": "receptar",
    "receptabas": "receptar",
    "receptaciones": "receptación",
    "receptad": "receptar",
    "receptada": "receptar",
    "receptadas": "receptar",
    "receptado": "receptar",
    "receptadora": "receptador",
    "receptadoras": "receptador",
    "receptadores": "receptador",
    "receptados": "receptar",
    "receptamos": "receptar",
    "receptan": "receptar",
    "receptando": "receptar",
    "receptara": "receptar",
    "receptarais": "receptar",
    "receptaran": "receptar",
    "receptaras": "receptar",
    "receptare": "receptar",
    "receptareis": "receptar",
    "receptaremos": "receptar",
    "receptaren": "receptar",
    "receptares": "receptar",
    "receptaron": "receptar",
    "receptará": "receptar",
    "receptarán": "receptar",
    "receptarás": "receptar",
    "receptaré": "receptar",
    "receptaréis": "receptar",
    "receptaría": "receptar",
    "receptaríais": "receptar",
    "receptaríamos": "receptar",
    "receptarían": "receptar",
    "receptarías": "receptar",
    "receptas": "receptar",
    "receptase": "receptar",
    "receptaseis": "receptar",
    "receptasen": "receptar",
    "receptases": "receptar",
    "receptaste": "receptar",
    "receptasteis": "receptar",
    "recepte": "receptar",
    "receptemos": "receptar",
    "recepten": "receptar",
    "receptes": "receptar",
    "receptiva": "receptivo",
    "receptivas": "receptivo",
    "receptividades": "receptividad",
    "receptivos": "receptivo",
    "recepto": "receptar",
    "receptora": "receptor",
    "receptoras": "receptor",
    "receptores": "receptor",
    "receptorías": "receptoría",
    "receptábamos": "receptar",
    "receptáculos": "receptáculo",
    "receptáis": "receptar",
    "receptáramos": "receptar",
    "receptáremos": "receptar",
    "receptásemos": "receptar",
    "recepté": "receptar",
    "receptéis": "receptar",
    "receptó": "receptar",
    "recertificaciones": "recertificación",
    "reces": "rezar",
    "recesiones": "recesión",
    "recesiva": "recesivo",
    "recesivas": "recesivo",
    "recesividades": "recesividad",
    "recesivos": "recesivo",
    "recesos": "receso",
    "receta": "recetar",
    "recetaba": "recetar",
    "recetabais": "recetar",
    "recetaban": "recetar",
    "recetabas": "recetar",
    "recetad": "recetar",
    "recetada": "recetar",
    "recetadas": "recetar",
    "recetado": "recetar",
    "recetados": "recetar",
    "recetamos": "recetar",
    "recetan": "recetar",
    "recetando": "recetar",
    "recetara": "recetar",
    "recetarais": "recetar",
    "recetaran": "recetar",
    "recetaras": "recetar",
    "recetare": "recetar",
    "recetareis": "recetar",
    "recetaremos": "recetar",
    "recetaren": "recetar",
    "recetares": "recetar",
    "recetarios": "recetario",
    "recetaron": "recetar",
    "recetará": "recetar",
    "recetarán": "recetar",
    "recetarás": "recetar",
    "recetaré": "recetar",
    "recetaréis": "recetar",
    "recetaría": "recetar",
    "recetaríais": "recetar",
    "recetaríamos": "recetar",
    "recetarían": "recetar",
    "recetarías": "recetar",
    "recetas": "recetar",
    "recetase": "recetar",
    "recetaseis": "recetar",
    "recetasen": "recetar",
    "recetases": "recetar",
    "recetaste": "recetar",
    "recetasteis": "recetar",
    "recete": "recetar",
    "recetemos": "recetar",
    "receten": "recetar",
    "recetes": "recetar",
    "receto": "recetar",
    "recetábamos": "recetar",
    "recetáis": "recetar",
    "recetáramos": "recetar",
    "recetáremos": "recetar",
    "recetásemos": "recetar",
    "receté": "recetar",
    "recetéis": "recetar",
    "recetó": "recetar",
    "rechace": "rechazar",
    "rechacemos": "rechazar",
    "rechacen": "rechazar",
    "rechaces": "rechazar",
    "rechacé": "rechazar",
    "rechacéis": "rechazar",
    "rechaza": "rechazar",
    "rechazaba": "rechazar",
    "rechazabais": "rechazar",
    "rechazaban": "rechazar",
    "rechazabas": "rechazar",
    "rechazables": "rechazable",
    "rechazad": "rechazar",
    "rechazada": "rechazar",
    "rechazadas": "rechazar",
    "rechazado": "rechazar",
    "rechazados": "rechazar",
    "rechazamientos": "rechazamiento",
    "rechazamos": "rechazar",
    "rechazan": "rechazar",
    "rechazando": "rechazar",
    "rechazara": "rechazar",
    "rechazarais": "rechazar",
    "rechazaran": "rechazar",
    "rechazaras": "rechazar",
    "rechazare": "rechazar",
    "rechazareis": "rechazar",
    "rechazaremos": "rechazar",
    "rechazaren": "rechazar",
    "rechazares": "rechazar",
    "rechazaron": "rechazar",
    "rechazará": "rechazar",
    "rechazarán": "rechazar",
    "rechazarás": "rechazar",
    "rechazaré": "rechazar",
    "rechazaréis": "rechazar",
    "rechazaría": "rechazar",
    "rechazaríais": "rechazar",
    "rechazaríamos": "rechazar",
    "rechazarían": "rechazar",
    "rechazarías": "rechazar",
    "rechazas": "rechazar",
    "rechazase": "rechazar",
    "rechazaseis": "rechazar",
    "rechazasen": "rechazar",
    "rechazases": "rechazar",
    "rechazaste": "rechazar",
    "rechazasteis": "rechazar",
    "rechazo": "rechazar",
    "rechazos": "rechazo",
    "rechazábamos": "rechazar",
    "rechazáis": "rechazar",
    "rechazáramos": "rechazar",
    "rechazáremos": "rechazar",
    "rechazásemos": "rechazar",
    "rechazó": "rechazar",
    "rechifla": "rechiflar",
    "rechiflaba": "rechiflar",
    "rechiflabais": "rechiflar",
    "rechiflaban": "rechiflar",
    "rechiflabas": "rechiflar",
    "rechiflad": "rechiflar",
    "rechiflada": "rechiflar",
    "rechifladas": "rechiflar",
    "rechiflado": "rechiflar",
    "rechiflados": "rechiflar",
    "rechiflamos": "rechiflar",
    "rechiflan": "rechiflar",
    "rechiflando": "rechiflar",
    "rechiflara": "rechiflar",
    "rechiflarais": "rechiflar",
    "rechiflaran": "rechiflar",
    "rechiflaras": "rechiflar",
    "rechiflare": "rechiflar",
    "rechiflareis": "rechiflar",
    "rechiflaremos": "rechiflar",
    "rechiflaren": "rechiflar",
    "rechiflares": "rechiflar",
    "rechiflaron": "rechiflar",
    "rechiflará": "rechiflar",
    "rechiflarán": "rechiflar",
    "rechiflarás": "rechiflar",
    "rechiflaré": "rechiflar",
    "rechiflaréis": "rechiflar",
    "rechiflaría": "rechiflar",
    "rechiflaríais": "rechiflar",
    "rechiflaríamos": "rechiflar",
    "rechiflarían": "rechiflar",
    "rechiflarías": "rechiflar",
    "rechiflas": "rechiflar",
    "rechiflase": "rechiflar",
    "rechiflaseis": "rechiflar",
    "rechiflasen": "rechiflar",
    "rechiflases": "rechiflar",
    "rechiflaste": "rechiflar",
    "rechiflasteis": "rechiflar",
    "rechifle": "rechiflar",
    "rechiflemos": "rechiflar",
    "rechiflen": "rechiflar",
    "rechifles": "rechiflar",
    "rechiflo": "rechiflar",
    "rechiflábamos": "rechiflar",
    "rechifláis": "rechiflar",
    "rechifláramos": "rechiflar",
    "rechifláremos": "rechiflar",
    "rechiflásemos": "rechiflar",
    "rechiflé": "rechiflar",
    "rechifléis": "rechiflar",
    "rechifló": "rechiflar",
    "rechina": "rechinar",
    "rechinaba": "rechinar",
    "rechinabais": "rechinar",
    "rechinaban": "rechinar",
    "rechinabas": "rechinar",
    "rechinad": "rechinar",
    "rechinada": "rechinar",
    "rechinadas": "rechinar",
    "rechinado": "rechinar",
    "rechinadora": "rechinador",
    "rechinadoras": "rechinador",
    "rechinadores": "rechinador",
    "rechinados": "rechinar",
    "rechinamientos": "rechinamiento",
    "rechinamos": "rechinar",
    "rechinan": "rechinar",
    "rechinando": "rechinar",
    "rechinara": "rechinar",
    "rechinarais": "rechinar",
    "rechinaran": "rechinar",
    "rechinaras": "rechinar",
    "rechinare": "rechinar",
    "rechinareis": "rechinar",
    "rechinaremos": "rechinar",
    "rechinaren": "rechinar",
    "rechinares": "rechinar",
    "rechinaron": "rechinar",
    "rechinará": "rechinar",
    "rechinarán": "rechinar",
    "rechinarás": "rechinar",
    "rechinaré": "rechinar",
    "rechinaréis": "rechinar",
    "rechinaría": "rechinar",
    "rechinaríais": "rechinar",
    "rechinaríamos": "rechinar",
    "rechinarían": "rechinar",
    "rechinarías": "rechinar",
    "rechinas": "rechinar",
    "rechinase": "rechinar",
    "rechinaseis": "rechinar",
    "rechinasen": "rechinar",
    "rechinases": "rechinar",
    "rechinaste": "rechinar",
    "rechinasteis": "rechinar",
    "rechine": "rechinar",
    "rechinemos": "rechinar",
    "rechinen": "rechinar",
    "rechines": "rechinar",
    "rechinidos": "rechinido",
    "rechino": "rechinar",
    "rechinos": "rechino",
    "rechinábamos": "rechinar",
    "rechináis": "rechinar",
    "rechináramos": "rechinar",
    "rechináremos": "rechinar",
    "rechinásemos": "rechinar",
    "rechiné": "rechinar",
    "rechinéis": "rechinar",
    "rechinó": "rechinar",
    "rechista": "rechistar",
    "rechistaba": "rechistar",
    "rechistabais": "rechistar",
    "rechistaban": "rechistar",
    "rechistabas": "rechistar",
    "rechistad": "rechistar",
    "rechistada": "rechistar",
    "rechistadas": "rechistar",
    "rechistado": "rechistar",
    "rechistados": "rechistar",
    "rechistamos": "rechistar",
    "rechistan": "rechistar",
    "rechistando": "rechistar",
    "rechistara": "rechistar",
    "rechistarais": "rechistar",
    "rechistaran": "rechistar",
    "rechistaras": "rechistar",
    "rechistare": "rechistar",
    "rechistareis": "rechistar",
    "rechistaremos": "rechistar",
    "rechistaren": "rechistar",
    "rechistares": "rechistar",
    "rechistaron": "rechistar",
    "rechistará": "rechistar",
    "rechistarán": "rechistar",
    "rechistarás": "rechistar",
    "rechistaré": "rechistar",
    "rechistaréis": "rechistar",
    "rechistaría": "rechistar",
    "rechistaríais": "rechistar",
    "rechistaríamos": "rechistar",
    "rechistarían": "rechistar",
    "rechistarías": "rechistar",
    "rechistas": "rechistar",
    "rechistase": "rechistar",
    "rechistaseis": "rechistar",
    "rechistasen": "rechistar",
    "rechistases": "rechistar",
    "rechistaste": "rechistar",
    "rechistasteis": "rechistar",
    "rechiste": "rechistar",
    "rechistemos": "rechistar",
    "rechisten": "rechistar",
    "rechistes": "rechistar",
    "rechisto": "rechistar",
    "rechistábamos": "rechistar",
    "rechistáis": "rechistar",
    "rechistáramos": "rechistar",
    "rechistáremos": "rechistar",
    "rechistásemos": "rechistar",
    "rechisté": "rechistar",
    "rechistéis": "rechistar",
    "rechistó": "rechistar",
    "rechoncha": "rechoncho",
    "rechonchas": "rechoncho",
    "rechonchos": "rechoncho",
    "rechupes": "rechupe",
    "recia": "recio",
    "reciales": "recial",
    "reciarios": "reciario",
    "recias": "recio",
    "reciba": "recibir",
    "recibamos": "recibir",
    "reciban": "recibir",
    "recibas": "recibir",
    "recibe": "recibir",
    "reciben": "recibir",
    "recibes": "recibir",
    "recibid": "recibir",
    "recibida": "recibir",
    "recibidas": "recibir",
    "recibido": "recibir",
    "recibidora": "recibidor",
    "recibidoras": "recibidor",
    "recibidores": "recibidor",
    "recibidos": "recibir",
    "recibiendo": "recibir",
    "recibiera": "recibir",
    "recibierais": "recibir",
    "recibieran": "recibir",
    "recibieras": "recibir",
    "recibiere": "recibir",
    "recibiereis": "recibir",
    "recibieren": "recibir",
    "recibieres": "recibir",
    "recibieron": "recibir",
    "recibiese": "recibir",
    "recibieseis": "recibir",
    "recibiesen": "recibir",
    "recibieses": "recibir",
    "recibimientos": "recibimiento",
    "recibimos": "recibir",
    "recibiremos": "recibir",
    "recibirá": "recibir",
    "recibirán": "recibir",
    "recibirás": "recibir",
    "recibiré": "recibir",
    "recibiréis": "recibir",
    "recibiría": "recibir",
    "recibiríais": "recibir",
    "recibiríamos": "recibir",
    "recibirían": "recibir",
    "recibirías": "recibir",
    "recibiste": "recibir",
    "recibisteis": "recibir",
    "recibiéramos": "recibir",
    "recibiéremos": "recibir",
    "recibiésemos": "recibir",
    "recibió": "recibir",
    "recibo": "recibir",
    "recibos": "recibo",
    "recibáis": "recibir",
    "recibí": "recibir",
    "recibía": "recibir",
    "recibíais": "recibir",
    "recibíamos": "recibir",
    "recibían": "recibir",
    "recibías": "recibir",
    "recibíes": "recibí",
    "recibís": "recibir",
    "recicla": "reciclar",
    "reciclaba": "reciclar",
    "reciclabais": "reciclar",
    "reciclaban": "reciclar",
    "reciclabas": "reciclar",
    "reciclabilidades": "reciclabilidad",
    "reciclables": "reciclable",
    "reciclad": "reciclar",
    "reciclada": "reciclar",
    "recicladas": "reciclar",
    "reciclado": "reciclar",
    "reciclados": "reciclar",
    "reciclajes": "reciclaje",
    "reciclamos": "reciclar",
    "reciclan": "reciclar",
    "reciclando": "reciclar",
    "reciclara": "reciclar",
    "reciclarais": "reciclar",
    "reciclaran": "reciclar",
    "reciclaras": "reciclar",
    "reciclare": "reciclar",
    "reciclareis": "reciclar",
    "reciclaremos": "reciclar",
    "reciclaren": "reciclar",
    "reciclares": "reciclar",
    "reciclaron": "reciclar",
    "reciclará": "reciclar",
    "reciclarán": "reciclar",
    "reciclarás": "reciclar",
    "reciclaré": "reciclar",
    "reciclaréis": "reciclar",
    "reciclaría": "reciclar",
    "reciclaríais": "reciclar",
    "reciclaríamos": "reciclar",
    "reciclarían": "reciclar",
    "reciclarías": "reciclar",
    "reciclas": "reciclar",
    "reciclase": "reciclar",
    "reciclaseis": "reciclar",
    "reciclasen": "reciclar",
    "reciclases": "reciclar",
    "reciclaste": "reciclar",
    "reciclasteis": "reciclar",
    "recicle": "reciclar",
    "reciclemos": "reciclar",
    "reciclen": "reciclar",
    "recicles": "reciclar",
    "reciclo": "reciclar",
    "reciclábamos": "reciclar",
    "recicláis": "reciclar",
    "recicláramos": "reciclar",
    "recicláremos": "reciclar",
    "reciclásemos": "reciclar",
    "reciclé": "reciclar",
    "recicléis": "reciclar",
    "recicló": "reciclar",
    "recidiva": "recidivar",
    "recidivaba": "recidivar",
    "recidivabais": "recidivar",
    "recidivaban": "recidivar",
    "recidivabas": "recidivar",
    "recidivad": "recidivar",
    "recidivada": "recidivar",
    "recidivadas": "recidivar",
    "recidivado": "recidivar",
    "recidivados": "recidivar",
    "recidivamos": "recidivar",
    "recidivan": "recidivar",
    "recidivando": "recidivar",
    "recidivara": "recidivar",
    "recidivarais": "recidivar",
    "recidivaran": "recidivar",
    "recidivaras": "recidivar",
    "recidivare": "recidivar",
    "recidivareis": "recidivar",
    "recidivaremos": "recidivar",
    "recidivaren": "recidivar",
    "recidivares": "recidivar",
    "recidivaron": "recidivar",
    "recidivará": "recidivar",
    "recidivarán": "recidivar",
    "recidivarás": "recidivar",
    "recidivaré": "recidivar",
    "recidivaréis": "recidivar",
    "recidivaría": "recidivar",
    "recidivaríais": "recidivar",
    "recidivaríamos": "recidivar",
    "recidivarían": "recidivar",
    "recidivarías": "recidivar",
    "recidivas": "recidivar",
    "recidivase": "recidivar",
    "recidivaseis": "recidivar",
    "recidivasen": "recidivar",
    "recidivases": "recidivar",
    "recidivaste": "recidivar",
    "recidivasteis": "recidivar",
    "recidive": "recidivar",
    "recidivemos": "recidivar",
    "recidiven": "recidivar",
    "recidives": "recidivar",
    "recidivo": "recidivar",
    "recidivábamos": "recidivar",
    "recidiváis": "recidivar",
    "recidiváramos": "recidivar",
    "recidiváremos": "recidivar",
    "recidivásemos": "recidivar",
    "recidivé": "recidivar",
    "recidivéis": "recidivar",
    "recidivó": "recidivar",
    "recienta": "recentar",
    "recientan": "recentar",
    "recientas": "recentar",
    "reciente": "recentar",
    "recienten": "recentar",
    "recientes": "reciente",
    "reciento": "recentar",
    "recintos": "recinto",
    "recios": "recio",
    "recipiendaria": "recipiendario",
    "recipiendarias": "recipiendario",
    "recipiendarios": "recipiendario",
    "recipientes": "recipiente",
    "reciproca": "reciprocar",
    "reciprocaba": "reciprocar",
    "reciprocabais": "reciprocar",
    "reciprocaban": "reciprocar",
    "reciprocabas": "reciprocar",
    "reciprocaciones": "reciprocación",
    "reciprocad": "reciprocar",
    "reciprocada": "reciprocar",
    "reciprocadas": "reciprocar",
    "reciprocado": "reciprocar",
    "reciprocados": "reciprocar",
    "reciprocamos": "reciprocar",
    "reciprocan": "reciprocar",
    "reciprocando": "reciprocar",
    "reciprocara": "reciprocar",
    "reciprocarais": "reciprocar",
    "reciprocaran": "reciprocar",
    "reciprocaras": "reciprocar",
    "reciprocare": "reciprocar",
    "reciprocareis": "reciprocar",
    "reciprocaremos": "reciprocar",
    "reciprocaren": "reciprocar",
    "reciprocares": "reciprocar",
    "reciprocaron": "reciprocar",
    "reciprocará": "reciprocar",
    "reciprocarán": "reciprocar",
    "reciprocarás": "reciprocar",
    "reciprocaré": "reciprocar",
    "reciprocaréis": "reciprocar",
    "reciprocaría": "reciprocar",
    "reciprocaríais": "reciprocar",
    "reciprocaríamos": "reciprocar",
    "reciprocarían": "reciprocar",
    "reciprocarías": "reciprocar",
    "reciprocas": "reciprocar",
    "reciprocase": "reciprocar",
    "reciprocaseis": "reciprocar",
    "reciprocasen": "reciprocar",
    "reciprocases": "reciprocar",
    "reciprocaste": "reciprocar",
    "reciprocasteis": "reciprocar",
    "reciprocidades": "reciprocidad",
    "reciproco": "reciprocar",
    "reciprocábamos": "reciprocar",
    "reciprocáis": "reciprocar",
    "reciprocáramos": "reciprocar",
    "reciprocáremos": "reciprocar",
    "reciprocásemos": "reciprocar",
    "reciprocó": "reciprocar",
    "reciproque": "reciprocar",
    "reciproquemos": "reciprocar",
    "reciproquen": "reciprocar",
    "reciproques": "reciprocar",
    "reciproqué": "reciprocar",
    "reciproquéis": "reciprocar",
    "recirculaciones": "recirculación",
    "recita": "recitar",
    "recitaba": "recitar",
    "recitabais": "recitar",
    "recitaban": "recitar",
    "recitabas": "recitar",
    "recitaciones": "recitación",
    "recitad": "recitar",
    "recitada": "recitar",
    "recitadas": "recitar",
    "recitado": "recitar",
    "recitadora": "recitador",
    "recitadoras": "recitador",
    "recitadores": "recitador",
    "recitados": "recitar",
    "recitales": "recital",
    "recitamos": "recitar",
    "recitan": "recitar",
    "recitando": "recitar",
    "recitara": "recitar",
    "recitarais": "recitar",
    "recitaran": "recitar",
    "recitaras": "recitar",
    "recitare": "recitar",
    "recitareis": "recitar",
    "recitaremos": "recitar",
    "recitaren": "recitar",
    "recitares": "recitar",
    "recitaron": "recitar",
    "recitará": "recitar",
    "recitarán": "recitar",
    "recitarás": "recitar",
    "recitaré": "recitar",
    "recitaréis": "recitar",
    "recitaría": "recitar",
    "recitaríais": "recitar",
    "recitaríamos": "recitar",
    "recitarían": "recitar",
    "recitarías": "recitar",
    "recitas": "recitar",
    "recitase": "recitar",
    "recitaseis": "recitar",
    "recitasen": "recitar",
    "recitases": "recitar",
    "recitaste": "recitar",
    "recitasteis": "recitar",
    "recitativa": "recitativo",
    "recitativas": "recitativo",
    "recitativos": "recitativo",
    "recite": "recitar",
    "recitemos": "recitar",
    "reciten": "recitar",
    "recites": "recitar",
    "recito": "recitar",
    "recitábamos": "recitar",
    "recitáculos": "recitáculo",
    "recitáis": "recitar",
    "recitáramos": "recitar",
    "recitáremos": "recitar",
    "recitásemos": "recitar",
    "recité": "recitar",
    "recitéis": "recitar",
    "recitó": "recitar",
    "reciuras": "reciura",
    "reciéna": "recién",
    "reciénas": "recién",
    "reciéno": "recién",
    "reciénos": "recién",
    "reclama": "reclamar",
    "reclamaba": "reclamar",
    "reclamabais": "reclamar",
    "reclamaban": "reclamar",
    "reclamabas": "reclamar",
    "reclamables": "reclamable",
    "reclamaciones": "reclamación",
    "reclamad": "reclamar",
    "reclamada": "reclamar",
    "reclamadas": "reclamar",
    "reclamado": "reclamar",
    "reclamadora": "reclamador",
    "reclamadoras": "reclamador",
    "reclamadores": "reclamador",
    "reclamados": "reclamar",
    "reclamamos": "reclamar",
    "reclaman": "reclamar",
    "reclamando": "reclamar",
    "reclamantes": "reclamante",
    "reclamara": "reclamar",
    "reclamarais": "reclamar",
    "reclamaran": "reclamar",
    "reclamaras": "reclamar",
    "reclamare": "reclamar",
    "reclamareis": "reclamar",
    "reclamaremos": "reclamar",
    "reclamaren": "reclamar",
    "reclamares": "reclamar",
    "reclamaron": "reclamar",
    "reclamará": "reclamar",
    "reclamarán": "reclamar",
    "reclamarás": "reclamar",
    "reclamaré": "reclamar",
    "reclamaréis": "reclamar",
    "reclamaría": "reclamar",
    "reclamaríais": "reclamar",
    "reclamaríamos": "reclamar",
    "reclamarían": "reclamar",
    "reclamarías": "reclamar",
    "reclamas": "reclamar",
    "reclamase": "reclamar",
    "reclamaseis": "reclamar",
    "reclamasen": "reclamar",
    "reclamases": "reclamar",
    "reclamaste": "reclamar",
    "reclamasteis": "reclamar",
    "reclame": "reclamar",
    "reclamemos": "reclamar",
    "reclamen": "reclamar",
    "reclames": "reclame",
    "reclamo": "reclamar",
    "reclamos": "reclamo",
    "reclamábamos": "reclamar",
    "reclamáis": "reclamar",
    "reclamáramos": "reclamar",
    "reclamáremos": "reclamar",
    "reclamásemos": "reclamar",
    "reclamé": "reclamar",
    "reclaméis": "reclamar",
    "reclamó": "reclamar",
    "reclasificaciones": "reclasificación",
    "recles": "recle",
    "reclina": "reclinar",
    "reclinaba": "reclinar",
    "reclinabais": "reclinar",
    "reclinaban": "reclinar",
    "reclinabas": "reclinar",
    "reclinables": "reclinable",
    "reclinaciones": "reclinación",
    "reclinad": "reclinar",
    "reclinada": "reclinar",
    "reclinadas": "reclinar",
    "reclinado": "reclinar",
    "reclinados": "reclinar",
    "reclinamos": "reclinar",
    "reclinan": "reclinar",
    "reclinando": "reclinar",
    "reclinara": "reclinar",
    "reclinarais": "reclinar",
    "reclinaran": "reclinar",
    "reclinaras": "reclinar",
    "reclinare": "reclinar",
    "reclinareis": "reclinar",
    "reclinaremos": "reclinar",
    "reclinaren": "reclinar",
    "reclinares": "reclinar",
    "reclinaron": "reclinar",
    "reclinará": "reclinar",
    "reclinarán": "reclinar",
    "reclinarás": "reclinar",
    "reclinaré": "reclinar",
    "reclinaréis": "reclinar",
    "reclinaría": "reclinar",
    "reclinaríais": "reclinar",
    "reclinaríamos": "reclinar",
    "reclinarían": "reclinar",
    "reclinarías": "reclinar",
    "reclinas": "reclinar",
    "reclinase": "reclinar",
    "reclinaseis": "reclinar",
    "reclinasen": "reclinar",
    "reclinases": "reclinar",
    "reclinaste": "reclinar",
    "reclinasteis": "reclinar",
    "reclinatorios": "reclinatorio",
    "recline": "reclinar",
    "reclinemos": "reclinar",
    "reclinen": "reclinar",
    "reclines": "reclinar",
    "reclino": "reclinar",
    "reclinábamos": "reclinar",
    "reclináis": "reclinar",
    "reclináramos": "reclinar",
    "reclináremos": "reclinar",
    "reclinásemos": "reclinar",
    "recliné": "reclinar",
    "reclinéis": "reclinar",
    "reclinó": "reclinar",
    "recluid": "recluir",
    "recluida": "recluir",
    "recluidas": "recluir",
    "recluido": "recluir",
    "recluidos": "recluir",
    "recluimos": "recluir",
    "recluiremos": "recluir",
    "recluirá": "recluir",
    "recluirán": "recluir",
    "recluirás": "recluir",
    "recluiré": "recluir",
    "recluiréis": "recluir",
    "recluiría": "recluir",
    "recluiríais": "recluir",
    "recluiríamos": "recluir",
    "recluirían": "recluir",
    "recluirías": "recluir",
    "recluiste": "recluir",
    "recluisteis": "recluir",
    "reclusa": "recluso",
    "reclusas": "recluso",
    "reclusiones": "reclusión",
    "reclusos": "recluso",
    "recluta": "reclutar",
    "reclutaba": "reclutar",
    "reclutabais": "reclutar",
    "reclutaban": "reclutar",
    "reclutabas": "reclutar",
    "reclutad": "reclutar",
    "reclutada": "reclutar",
    "reclutadas": "reclutar",
    "reclutado": "reclutar",
    "reclutadora": "reclutador",
    "reclutadoras": "reclutador",
    "reclutadores": "reclutador",
    "reclutados": "reclutar",
    "reclutamientos": "reclutamiento",
    "reclutamos": "reclutar",
    "reclutan": "reclutar",
    "reclutando": "reclutar",
    "reclutara": "reclutar",
    "reclutarais": "reclutar",
    "reclutaran": "reclutar",
    "reclutaras": "reclutar",
    "reclutare": "reclutar",
    "reclutareis": "reclutar",
    "reclutaremos": "reclutar",
    "reclutaren": "reclutar",
    "reclutares": "reclutar",
    "reclutaron": "reclutar",
    "reclutará": "reclutar",
    "reclutarán": "reclutar",
    "reclutarás": "reclutar",
    "reclutaré": "reclutar",
    "reclutaréis": "reclutar",
    "reclutaría": "reclutar",
    "reclutaríais": "reclutar",
    "reclutaríamos": "reclutar",
    "reclutarían": "reclutar",
    "reclutarías": "reclutar",
    "reclutas": "reclutar",
    "reclutase": "reclutar",
    "reclutaseis": "reclutar",
    "reclutasen": "reclutar",
    "reclutases": "reclutar",
    "reclutaste": "reclutar",
    "reclutasteis": "reclutar",
    "reclute": "reclutar",
    "reclutemos": "reclutar",
    "recluten": "reclutar",
    "reclutes": "reclutar",
    "recluto": "reclutar",
    "reclutábamos": "reclutar",
    "reclutáis": "reclutar",
    "reclutáramos": "reclutar",
    "reclutáremos": "reclutar",
    "reclutásemos": "reclutar",
    "recluté": "reclutar",
    "reclutéis": "reclutar",
    "reclutó": "reclutar",
    "recluya": "recluir",
    "recluyamos": "recluir",
    "recluyan": "recluir",
    "recluyas": "recluir",
    "recluye": "recluir",
    "recluyen": "recluir",
    "recluyendo": "recluir",
    "recluyera": "recluir",
    "recluyerais": "recluir",
    "recluyeran": "recluir",
    "recluyeras": "recluir",
    "recluyere": "recluir",
    "recluyereis": "recluir",
    "recluyeren": "recluir",
    "recluyeres": "recluir",
    "recluyeron": "recluir",
    "recluyes": "recluir",
    "recluyese": "recluir",
    "recluyeseis": "recluir",
    "recluyesen": "recluir",
    "recluyeses": "recluir",
    "recluyo": "recluir",
    "recluyáis": "recluir",
    "recluyéramos": "recluir",
    "recluyéremos": "recluir",
    "recluyésemos": "recluir",
    "recluyó": "recluir",
    "recluí": "recluir",
    "recluía": "recluir",
    "recluíais": "recluir",
    "recluíamos": "recluir",
    "recluían": "recluir",
    "recluías": "recluir",
    "recluís": "recluir",
    "recobra": "recobrar",
    "recobraba": "recobrar",
    "recobrabais": "recobrar",
    "recobraban": "recobrar",
    "recobrabas": "recobrar",
    "recobrad": "recobrar",
    "recobrada": "recobrar",
    "recobradas": "recobrar",
    "recobrado": "recobrar",
    "recobrados": "recobrar",
    "recobramos": "recobrar",
    "recobran": "recobrar",
    "recobrando": "recobrar",
    "recobrara": "recobrar",
    "recobrarais": "recobrar",
    "recobraran": "recobrar",
    "recobraras": "recobrar",
    "recobrare": "recobrar",
    "recobrareis": "recobrar",
    "recobraremos": "recobrar",
    "recobraren": "recobrar",
    "recobrares": "recobrar",
    "recobraron": "recobrar",
    "recobrará": "recobrar",
    "recobrarán": "recobrar",
    "recobrarás": "recobrar",
    "recobraré": "recobrar",
    "recobraréis": "recobrar",
    "recobraría": "recobrar",
    "recobraríais": "recobrar",
    "recobraríamos": "recobrar",
    "recobrarían": "recobrar",
    "recobrarías": "recobrar",
    "recobras": "recobrar",
    "recobrase": "recobrar",
    "recobraseis": "recobrar",
    "recobrasen": "recobrar",
    "recobrases": "recobrar",
    "recobraste": "recobrar",
    "recobrasteis": "recobrar",
    "recobre": "recobrar",
    "recobremos": "recobrar",
    "recobren": "recobrar",
    "recobres": "recobrar",
    "recobro": "recobrar",
    "recobros": "recobro",
    "recobrábamos": "recobrar",
    "recobráis": "recobrar",
    "recobráramos": "recobrar",
    "recobráremos": "recobrar",
    "recobrásemos": "recobrar",
    "recobré": "recobrar",
    "recobréis": "recobrar",
    "recobró": "recobrar",
    "recocciones": "recocción",
    "recoced": "recocer",
    "recocemos": "recocer",
    "recoceremos": "recocer",
    "recocerá": "recocer",
    "recocerán": "recocer",
    "recocerás": "recocer",
    "recoceré": "recocer",
    "recoceréis": "recocer",
    "recocería": "recocer",
    "recoceríais": "recocer",
    "recoceríamos": "recocer",
    "recocerían": "recocer",
    "recocerías": "recocer",
    "recocha": "recocho",
    "recochas": "recocho",
    "recochinea": "recochinear",
    "recochineaba": "recochinear",
    "recochineabais": "recochinear",
    "recochineaban": "recochinear",
    "recochineabas": "recochinear",
    "recochinead": "recochinear",
    "recochineada": "recochinear",
    "recochineadas": "recochinear",
    "recochineado": "recochinear",
    "recochineados": "recochinear",
    "recochineamos": "recochinear",
    "recochinean": "recochinear",
    "recochineando": "recochinear",
    "recochineara": "recochinear",
    "recochinearais": "recochinear",
    "recochinearan": "recochinear",
    "recochinearas": "recochinear",
    "recochineare": "recochinear",
    "recochineareis": "recochinear",
    "recochinearemos": "recochinear",
    "recochinearen": "recochinear",
    "recochineares": "recochinear",
    "recochinearon": "recochinear",
    "recochineará": "recochinear",
    "recochinearán": "recochinear",
    "recochinearás": "recochinear",
    "recochinearé": "recochinear",
    "recochinearéis": "recochinear",
    "recochinearía": "recochinear",
    "recochinearíais": "recochinear",
    "recochinearíamos": "recochinear",
    "recochinearían": "recochinear",
    "recochinearías": "recochinear",
    "recochineas": "recochinear",
    "recochinease": "recochinear",
    "recochineaseis": "recochinear",
    "recochineasen": "recochinear",
    "recochineases": "recochinear",
    "recochineaste": "recochinear",
    "recochineasteis": "recochinear",
    "recochinee": "recochinear",
    "recochineemos": "recochinear",
    "recochineen": "recochinear",
    "recochinees": "recochinear",
    "recochineo": "recochinear",
    "recochineos": "recochineo",
    "recochineábamos": "recochinear",
    "recochineáis": "recochinear",
    "recochineáramos": "recochinear",
    "recochineáremos": "recochinear",
    "recochineásemos": "recochinear",
    "recochineé": "recochinear",
    "recochineéis": "recochinear",
    "recochineó": "recochinear",
    "recochos": "recocho",
    "recocida": "recocer",
    "recocidas": "recocida",
    "recocido": "recocer",
    "recocidos": "recocido",
    "recociendo": "recocer",
    "recociera": "recocer",
    "recocierais": "recocer",
    "recocieran": "recocer",
    "recocieras": "recocer",
    "recociere": "recocer",
    "recociereis": "recocer",
    "recocieren": "recocer",
    "recocieres": "recocer",
    "recocieron": "recocer",
    "recociese": "recocer",
    "recocieseis": "recocer",
    "recociesen": "recocer",
    "recocieses": "recocer",
    "recocimos": "recocer",
    "recocinas": "recocina",
    "recociste": "recocer",
    "recocisteis": "recocer",
    "recociéramos": "recocer",
    "recociéremos": "recocer",
    "recociésemos": "recocer",
    "recoció": "recocer",
    "recocéis": "recocer",
    "recocí": "recocer",
    "recocía": "recocer",
    "recocíais": "recocer",
    "recocíamos": "recocer",
    "recocían": "recocer",
    "recocías": "recocer",
    "recoda": "recodar",
    "recodaba": "recodar",
    "recodabais": "recodar",
    "recodaban": "recodar",
    "recodabas": "recodar",
    "recodad": "recodar",
    "recodada": "recodar",
    "recodadas": "recodar",
    "recodaderos": "recodadero",
    "recodado": "recodar",
    "recodados": "recodar",
    "recodamos": "recodar",
    "recodan": "recodar",
    "recodando": "recodar",
    "recodara": "recodar",
    "recodarais": "recodar",
    "recodaran": "recodar",
    "recodaras": "recodar",
    "recodare": "recodar",
    "recodareis": "recodar",
    "recodaremos": "recodar",
    "recodaren": "recodar",
    "recodares": "recodar",
    "recodaron": "recodar",
    "recodará": "recodar",
    "recodarán": "recodar",
    "recodarás": "recodar",
    "recodaré": "recodar",
    "recodaréis": "recodar",
    "recodaría": "recodar",
    "recodaríais": "recodar",
    "recodaríamos": "recodar",
    "recodarían": "recodar",
    "recodarías": "recodar",
    "recodas": "recodar",
    "recodase": "recodar",
    "recodaseis": "recodar",
    "recodasen": "recodar",
    "recodases": "recodar",
    "recodaste": "recodar",
    "recodasteis": "recodar",
    "recode": "recodar",
    "recodemos": "recodar",
    "recoden": "recodar",
    "recodes": "recodar",
    "recodificaciones": "recodificación",
    "recodificadores": "recodificador",
    "recodo": "recodar",
    "recodos": "recodo",
    "recodábamos": "recodar",
    "recodáis": "recodar",
    "recodáramos": "recodar",
    "recodáremos": "recodar",
    "recodásemos": "recodar",
    "recodé": "recodar",
    "recodéis": "recodar",
    "recodó": "recodar",
    "recoge": "recoger",
    "recoged": "recoger",
    "recogederos": "recogedero",
    "recogedora": "recogedor",
    "recogedoras": "recogedor",
    "recogedores": "recogedor",
    "recogemos": "recoger",
    "recogen": "recoger",
    "recogeremos": "recoger",
    "recogerá": "recoger",
    "recogerán": "recoger",
    "recogerás": "recoger",
    "recogeré": "recoger",
    "recogeréis": "recoger",
    "recogería": "recoger",
    "recogeríais": "recoger",
    "recogeríamos": "recoger",
    "recogerían": "recoger",
    "recogerías": "recoger",
    "recoges": "recoger",
    "recogida": "recoger",
    "recogidas": "recogida",
    "recogido": "recoger",
    "recogidos": "recoger",
    "recogiendo": "recoger",
    "recogiera": "recoger",
    "recogierais": "recoger",
    "recogieran": "recoger",
    "recogieras": "recoger",
    "recogiere": "recoger",
    "recogiereis": "recoger",
    "recogieren": "recoger",
    "recogieres": "recoger",
    "recogieron": "recoger",
    "recogiese": "recoger",
    "recogieseis": "recoger",
    "recogiesen": "recoger",
    "recogieses": "recoger",
    "recogimientos": "recogimiento",
    "recogimos": "recoger",
    "recogiste": "recoger",
    "recogisteis": "recoger",
    "recogiéramos": "recoger",
    "recogiéremos": "recoger",
    "recogiésemos": "recoger",
    "recogió": "recoger",
    "recogéis": "recoger",
    "recogí": "recoger",
    "recogía": "recoger",
    "recogíais": "recoger",
    "recogíamos": "recoger",
    "recogían": "recoger",
    "recogías": "recoger",
    "recoja": "recoger",
    "recojamos": "recoger",
    "recojan": "recoger",
    "recojas": "recoger",
    "recojo": "recoger",
    "recojáis": "recoger",
    "recolaba": "recolar",
    "recolabais": "recolar",
    "recolaban": "recolar",
    "recolabas": "recolar",
    "recolad": "recolar",
    "recolada": "recolar",
    "recoladas": "recolar",
    "recolado": "recolar",
    "recolados": "recolar",
    "recolamos": "recolar",
    "recolando": "recolar",
    "recolara": "recolar",
    "recolarais": "recolar",
    "recolaran": "recolar",
    "recolaras": "recolar",
    "recolare": "recolar",
    "recolareis": "recolar",
    "recolaremos": "recolar",
    "recolaren": "recolar",
    "recolares": "recolar",
    "recolaron": "recolar",
    "recolará": "recolar",
    "recolarán": "recolar",
    "recolarás": "recolar",
    "recolaré": "recolar",
    "recolaréis": "recolar",
    "recolaría": "recolar",
    "recolaríais": "recolar",
    "recolaríamos": "recolar",
    "recolarían": "recolar",
    "recolarías": "recolar",
    "recolase": "recolar",
    "recolaseis": "recolar",
    "recolasen": "recolar",
    "recolases": "recolar",
    "recolaste": "recolar",
    "recolasteis": "recolar",
    "recolecciones": "recolección",
    "recolecta": "recolectar",
    "recolectaba": "recolectar",
    "recolectabais": "recolectar",
    "recolectaban": "recolectar",
    "recolectabas": "recolectar",
    "recolectad": "recolectar",
    "recolectada": "recolectar",
    "recolectadas": "recolectar",
    "recolectado": "recolectar",
    "recolectados": "recolectar",
    "recolectamos": "recolectar",
    "recolectan": "recolectar",
    "recolectando": "recolectar",
    "recolectara": "recolectar",
    "recolectarais": "recolectar",
    "recolectaran": "recolectar",
    "recolectaras": "recolectar",
    "recolectare": "recolectar",
    "recolectareis": "recolectar",
    "recolectaremos": "recolectar",
    "recolectaren": "recolectar",
    "recolectares": "recolectar",
    "recolectaron": "recolectar",
    "recolectará": "recolectar",
    "recolectarán": "recolectar",
    "recolectarás": "recolectar",
    "recolectaré": "recolectar",
    "recolectaréis": "recolectar",
    "recolectaría": "recolectar",
    "recolectaríais": "recolectar",
    "recolectaríamos": "recolectar",
    "recolectarían": "recolectar",
    "recolectarías": "recolectar",
    "recolectas": "recolectar",
    "recolectase": "recolectar",
    "recolectaseis": "recolectar",
    "recolectasen": "recolectar",
    "recolectases": "recolectar",
    "recolectaste": "recolectar",
    "recolectasteis": "recolectar",
    "recolecte": "recolectar",
    "recolectemos": "recolectar",
    "recolecten": "recolectar",
    "recolectes": "recolectar",
    "recolecto": "recolectar",
    "recolectora": "recolector",
    "recolectoras": "recolector",
    "recolectores": "recolector",
    "recolectábamos": "recolectar",
    "recolectáis": "recolectar",
    "recolectáramos": "recolectar",
    "recolectáremos": "recolectar",
    "recolectásemos": "recolectar",
    "recolecté": "recolectar",
    "recolectéis": "recolectar",
    "recolectó": "recolectar",
    "recolemos": "recolar",
    "recoleta": "recoleto",
    "recoletas": "recoleto",
    "recoletos": "recoleto",
    "recolocaciones": "recolocación",
    "recolábamos": "recolar",
    "recoláis": "recolar",
    "recoláramos": "recolar",
    "recoláremos": "recolar",
    "recolásemos": "recolar",
    "recolé": "recolar",
    "recoléis": "recolar",
    "recoló": "recolar",
    "recoma": "recomer",
    "recomamos": "recomer",
    "recoman": "recomer",
    "recomas": "recomer",
    "recombinaciones": "recombinación",
    "recombinantes": "recombinante",
    "recome": "recomer",
    "recomed": "recomer",
    "recomemos": "recomer",
    "recomen": "recomer",
    "recomencemos": "recomenzar",
    "recomencé": "recomenzar",
    "recomencéis": "recomenzar",
    "recomendaba": "recomendar",
    "recomendabais": "recomendar",
    "recomendaban": "recomendar",
    "recomendabas": "recomendar",
    "recomendables": "recomendable",
    "recomendaciones": "recomendación",
    "recomendad": "recomendar",
    "recomendada": "recomendar",
    "recomendadas": "recomendar",
    "recomendado": "recomendar",
    "recomendados": "recomendar",
    "recomendamos": "recomendar",
    "recomendando": "recomendar",
    "recomendara": "recomendar",
    "recomendarais": "recomendar",
    "recomendaran": "recomendar",
    "recomendaras": "recomendar",
    "recomendare": "recomendar",
    "recomendareis": "recomendar",
    "recomendaremos": "recomendar",
    "recomendaren": "recomendar",
    "recomendares": "recomendar",
    "recomendaron": "recomendar",
    "recomendará": "recomendar",
    "recomendarán": "recomendar",
    "recomendarás": "recomendar",
    "recomendaré": "recomendar",
    "recomendaréis": "recomendar",
    "recomendaría": "recomendar",
    "recomendaríais": "recomendar",
    "recomendaríamos": "recomendar",
    "recomendarían": "recomendar",
    "recomendarías": "recomendar",
    "recomendase": "recomendar",
    "recomendaseis": "recomendar",
    "recomendasen": "recomendar",
    "recomendases": "recomendar",
    "recomendaste": "recomendar",
    "recomendasteis": "recomendar",
    "recomendatoria": "recomendatorio",
    "recomendatorias": "recomendatorio",
    "recomendatorios": "recomendatorio",
    "recomendemos": "recomendar",
    "recomendábamos": "recomendar",
    "recomendáis": "recomendar",
    "recomendáramos": "recomendar",
    "recomendáremos": "recomendar",
    "recomendásemos": "recomendar",
    "recomendé": "recomendar",
    "recomendéis": "recomendar",
    "recomendó": "recomendar",
    "recomenzaba": "recomenzar",
    "recomenzabais": "recomenzar",
    "recomenzaban": "recomenzar",
    "recomenzabas": "recomenzar",
    "recomenzad": "recomenzar",
    "recomenzada": "recomenzar",
    "recomenzadas": "recomenzar",
    "recomenzado": "recomenzar",
    "recomenzados": "recomenzar",
    "recomenzamos": "recomenzar",
    "recomenzando": "recomenzar",
    "recomenzara": "recomenzar",
    "recomenzarais": "recomenzar",
    "recomenzaran": "recomenzar",
    "recomenzaras": "recomenzar",
    "recomenzare": "recomenzar",
    "recomenzareis": "recomenzar",
    "recomenzaremos": "recomenzar",
    "recomenzaren": "recomenzar",
    "recomenzares": "recomenzar",
    "recomenzaron": "recomenzar",
    "recomenzará": "recomenzar",
    "recomenzarán": "recomenzar",
    "recomenzarás": "recomenzar",
    "recomenzaré": "recomenzar",
    "recomenzaréis": "recomenzar",
    "recomenzaría": "recomenzar",
    "recomenzaríais": "recomenzar",
    "recomenzaríamos": "recomenzar",
    "recomenzarían": "recomenzar",
    "recomenzarías": "recomenzar",
    "recomenzase": "recomenzar",
    "recomenzaseis": "recomenzar",
    "recomenzasen": "recomenzar",
    "recomenzases": "recomenzar",
    "recomenzaste": "recomenzar",
    "recomenzasteis": "recomenzar",
    "recomenzábamos": "recomenzar",
    "recomenzáis": "recomenzar",
    "recomenzáramos": "recomenzar",
    "recomenzáremos": "recomenzar",
    "recomenzásemos": "recomenzar",
    "recomenzó": "recomenzar",
    "recomeremos": "recomer",
    "recomerá": "recomer",
    "recomerán": "recomer",
    "recomerás": "recomer",
    "recomeré": "recomer",
    "recomeréis": "recomer",
    "recomería": "recomer",
    "recomeríais": "recomer",
    "recomeríamos": "recomer",
    "recomerían": "recomer",
    "recomerías": "recomer",
    "recomes": "recomer",
    "recomida": "recomer",
    "recomidas": "recomer",
    "recomido": "recomer",
    "recomidos": "recomer",
    "recomience": "recomenzar",
    "recomiencen": "recomenzar",
    "recomiences": "recomenzar",
    "recomienda": "recomendar",
    "recomiendan": "recomendar",
    "recomiendas": "recomendar",
    "recomiende": "recomendar",
    "recomienden": "recomendar",
    "recomiendes": "recomendar",
    "recomiendo": "recomer",
    "recomienza": "recomenzar",
    "recomienzan": "recomenzar",
    "recomienzas": "recomenzar",
    "recomienzo": "recomenzar",
    "recomiera": "recomer",
    "recomierais": "recomer",
    "recomieran": "recomer",
    "recomieras": "recomer",
    "recomiere": "recomer",
    "recomiereis": "recomer",
    "recomieren": "recomer",
    "recomieres": "recomer",
    "recomieron": "recomer",
    "recomiese": "recomer",
    "recomieseis": "recomer",
    "recomiesen": "recomer",
    "recomieses": "recomer",
    "recomimos": "recomer",
    "recomiste": "recomer",
    "recomisteis": "recomer",
    "recomiéramos": "recomer",
    "recomiéremos": "recomer",
    "recomiésemos": "recomer",
    "recomió": "recomer",
    "recomo": "recomer",
    "recompactaciones": "recompactación",
    "recompensa": "recompensar",
    "recompensaba": "recompensar",
    "recompensabais": "recompensar",
    "recompensaban": "recompensar",
    "recompensabas": "recompensar",
    "recompensables": "recompensable",
    "recompensad": "recompensar",
    "recompensada": "recompensar",
    "recompensadas": "recompensar",
    "recompensado": "recompensar",
    "recompensados": "recompensar",
    "recompensamos": "recompensar",
    "recompensan": "recompensar",
    "recompensando": "recompensar",
    "recompensara": "recompensar",
    "recompensarais": "recompensar",
    "recompensaran": "recompensar",
    "recompensaras": "recompensar",
    "recompensare": "recompensar",
    "recompensareis": "recompensar",
    "recompensaremos": "recompensar",
    "recompensaren": "recompensar",
    "recompensares": "recompensar",
    "recompensaron": "recompensar",
    "recompensará": "recompensar",
    "recompensarán": "recompensar",
    "recompensarás": "recompensar",
    "recompensaré": "recompensar",
    "recompensaréis": "recompensar",
    "recompensaría": "recompensar",
    "recompensaríais": "recompensar",
    "recompensaríamos": "recompensar",
    "recompensarían": "recompensar",
    "recompensarías": "recompensar",
    "recompensas": "recompensar",
    "recompensase": "recompensar",
    "recompensaseis": "recompensar",
    "recompensasen": "recompensar",
    "recompensases": "recompensar",
    "recompensaste": "recompensar",
    "recompensasteis": "recompensar",
    "recompensatoria": "recompensatorio",
    "recompensatorias": "recompensatorio",
    "recompensatorios": "recompensatorio",
    "recompense": "recompensar",
    "recompensemos": "recompensar",
    "recompensen": "recompensar",
    "recompenses": "recompensar",
    "recompenso": "recompensar",
    "recompensábamos": "recompensar",
    "recompensáis": "recompensar",
    "recompensáramos": "recompensar",
    "recompensáremos": "recompensar",
    "recompensásemos": "recompensar",
    "recompensé": "recompensar",
    "recompenséis": "recompensar",
    "recompensó": "recompensar",
    "recompila": "recompilar",
    "recompilaba": "recompilar",
    "recompilabais": "recompilar",
    "recompilaban": "recompilar",
    "recompilabas": "recompilar",
    "recompilaciones": "recompilación",
    "recompilad": "recompilar",
    "recompilada": "recompilar",
    "recompiladas": "recompilar",
    "recompilado": "recompilar",
    "recompilados": "recompilar",
    "recompilamos": "recompilar",
    "recompilan": "recompilar",
    "recompilando": "recompilar",
    "recompilara": "recompilar",
    "recompilarais": "recompilar",
    "recompilaran": "recompilar",
    "recompilaras": "recompilar",
    "recompilare": "recompilar",
    "recompilareis": "recompilar",
    "recompilaremos": "recompilar",
    "recompilaren": "recompilar",
    "recompilares": "recompilar",
    "recompilaron": "recompilar",
    "recompilará": "recompilar",
    "recompilarán": "recompilar",
    "recompilarás": "recompilar",
    "recompilaré": "recompilar",
    "recompilaréis": "recompilar",
    "recompilaría": "recompilar",
    "recompilaríais": "recompilar",
    "recompilaríamos": "recompilar",
    "recompilarían": "recompilar",
    "recompilarías": "recompilar",
    "recompilas": "recompilar",
    "recompilase": "recompilar",
    "recompilaseis": "recompilar",
    "recompilasen": "recompilar",
    "recompilases": "recompilar",
    "recompilaste": "recompilar",
    "recompilasteis": "recompilar",
    "recompile": "recompilar",
    "recompilemos": "recompilar",
    "recompilen": "recompilar",
    "recompiles": "recompilar",
    "recompilo": "recompilar",
    "recompilábamos": "recompilar",
    "recompiláis": "recompilar",
    "recompiláramos": "recompilar",
    "recompiláremos": "recompilar",
    "recompilásemos": "recompilar",
    "recompilé": "recompilar",
    "recompiléis": "recompilar",
    "recompiló": "recompilar",
    "recomplementaciones": "recomplementación",
    "recompondremos": "recomponer",
    "recompondrá": "recomponer",
    "recompondrán": "recomponer",
    "recompondrás": "recomponer",
    "recompondré": "recomponer",
    "recompondréis": "recomponer",
    "recompondría": "recomponer",
    "recompondríais": "recomponer",
    "recompondríamos": "recomponer",
    "recompondrían": "recomponer",
    "recompondrías": "recomponer",
    "recompone": "recomponer",
    "recomponed": "recomponer",
    "recomponemos": "recomponer",
    "recomponen": "recomponer",
    "recompones": "recomponer",
    "recomponga": "recomponer",
    "recompongamos": "recomponer",
    "recompongan": "recomponer",
    "recompongas": "recomponer",
    "recompongo": "recomponer",
    "recompongáis": "recomponer",
    "recomponiendo": "recomponer",
    "recomponéis": "recomponer",
    "recomponía": "recomponer",
    "recomponíais": "recomponer",
    "recomponíamos": "recomponer",
    "recomponían": "recomponer",
    "recomponías": "recomponer",
    "recomposiciones": "recomposición",
    "recompras": "recompra",
    "recompresiones": "recompresión",
    "recompuesta": "recomponer",
    "recompuestas": "recomponer",
    "recompuesto": "recomponer",
    "recompuestos": "recomponer",
    "recompuse": "recomponer",
    "recompusiera": "recomponer",
    "recompusierais": "recomponer",
    "recompusieran": "recomponer",
    "recompusieras": "recomponer",
    "recompusiere": "recomponer",
    "recompusiereis": "recomponer",
    "recompusieren": "recomponer",
    "recompusieres": "recomponer",
    "recompusieron": "recomponer",
    "recompusiese": "recomponer",
    "recompusieseis": "recomponer",
    "recompusiesen": "recomponer",
    "recompusieses": "recomponer",
    "recompusimos": "recomponer",
    "recompusiste": "recomponer",
    "recompusisteis": "recomponer",
    "recompusiéramos": "recomponer",
    "recompusiéremos": "recomponer",
    "recompusiésemos": "recomponer",
    "recompuso": "recomponer",
    "recompón": "recomponer",
    "recomáis": "recomer",
    "recoméis": "recomer",
    "recomí": "recomer",
    "recomía": "recomer",
    "recomíais": "recomer",
    "recomíamos": "recomer",
    "recomían": "recomer",
    "recomías": "recomer",
    "reconcentra": "reconcentrar",
    "reconcentraba": "reconcentrar",
    "reconcentrabais": "reconcentrar",
    "reconcentraban": "reconcentrar",
    "reconcentrabas": "reconcentrar",
    "reconcentraciones": "reconcentración",
    "reconcentrad": "reconcentrar",
    "reconcentrada": "reconcentrar",
    "reconcentradas": "reconcentrar",
    "reconcentrado": "reconcentrar",
    "reconcentrados": "reconcentrar",
    "reconcentramientos": "reconcentramiento",
    "reconcentramos": "reconcentrar",
    "reconcentran": "reconcentrar",
    "reconcentrando": "reconcentrar",
    "reconcentrara": "reconcentrar",
    "reconcentrarais": "reconcentrar",
    "reconcentraran": "reconcentrar",
    "reconcentraras": "reconcentrar",
    "reconcentrare": "reconcentrar",
    "reconcentrareis": "reconcentrar",
    "reconcentraremos": "reconcentrar",
    "reconcentraren": "reconcentrar",
    "reconcentrares": "reconcentrar",
    "reconcentraron": "reconcentrar",
    "reconcentrará": "reconcentrar",
    "reconcentrarán": "reconcentrar",
    "reconcentrarás": "reconcentrar",
    "reconcentraré": "reconcentrar",
    "reconcentraréis": "reconcentrar",
    "reconcentraría": "reconcentrar",
    "reconcentraríais": "reconcentrar",
    "reconcentraríamos": "reconcentrar",
    "reconcentrarían": "reconcentrar",
    "reconcentrarías": "reconcentrar",
    "reconcentras": "reconcentrar",
    "reconcentrase": "reconcentrar",
    "reconcentraseis": "reconcentrar",
    "reconcentrasen": "reconcentrar",
    "reconcentrases": "reconcentrar",
    "reconcentraste": "reconcentrar",
    "reconcentrasteis": "reconcentrar",
    "reconcentre": "reconcentrar",
    "reconcentremos": "reconcentrar",
    "reconcentren": "reconcentrar",
    "reconcentres": "reconcentrar",
    "reconcentro": "reconcentrar",
    "reconcentrábamos": "reconcentrar",
    "reconcentráis": "reconcentrar",
    "reconcentráramos": "reconcentrar",
    "reconcentráremos": "reconcentrar",
    "reconcentrásemos": "reconcentrar",
    "reconcentré": "reconcentrar",
    "reconcentréis": "reconcentrar",
    "reconcentró": "reconcentrar",
    "reconcilia": "reconciliar",
    "reconciliaba": "reconciliar",
    "reconciliabais": "reconciliar",
    "reconciliaban": "reconciliar",
    "reconciliabas": "reconciliar",
    "reconciliaciones": "reconciliación",
    "reconciliad": "reconciliar",
    "reconciliada": "reconciliar",
    "reconciliadas": "reconciliar",
    "reconciliado": "reconciliar",
    "reconciliadora": "reconciliador",
    "reconciliadoras": "reconciliador",
    "reconciliadores": "reconciliador",
    "reconciliados": "reconciliar",
    "reconciliamos": "reconciliar",
    "reconcilian": "reconciliar",
    "reconciliando": "reconciliar",
    "reconciliara": "reconciliar",
    "reconciliarais": "reconciliar",
    "reconciliaran": "reconciliar",
    "reconciliaras": "reconciliar",
    "reconciliare": "reconciliar",
    "reconciliareis": "reconciliar",
    "reconciliaremos": "reconciliar",
    "reconciliaren": "reconciliar",
    "reconciliares": "reconciliar",
    "reconciliaron": "reconciliar",
    "reconciliará": "reconciliar",
    "reconciliarán": "reconciliar",
    "reconciliarás": "reconciliar",
    "reconciliaré": "reconciliar",
    "reconciliaréis": "reconciliar",
    "reconciliaría": "reconciliar",
    "reconciliaríais": "reconciliar",
    "reconciliaríamos": "reconciliar",
    "reconciliarían": "reconciliar",
    "reconciliarías": "reconciliar",
    "reconcilias": "reconciliar",
    "reconciliase": "reconciliar",
    "reconciliaseis": "reconciliar",
    "reconciliasen": "reconciliar",
    "reconciliases": "reconciliar",
    "reconciliaste": "reconciliar",
    "reconciliasteis": "reconciliar",
    "reconcilie": "reconciliar",
    "reconciliemos": "reconciliar",
    "reconcilien": "reconciliar",
    "reconcilies": "reconciliar",
    "reconcilio": "reconciliar",
    "reconciliábamos": "reconciliar",
    "reconciliáis": "reconciliar",
    "reconciliáramos": "reconciliar",
    "reconciliáremos": "reconciliar",
    "reconciliásemos": "reconciliar",
    "reconcilié": "reconciliar",
    "reconciliéis": "reconciliar",
    "reconcilió": "reconciliar",
    "reconcoma": "reconcomer",
    "reconcomamos": "reconcomer",
    "reconcoman": "reconcomer",
    "reconcomas": "reconcomer",
    "reconcome": "reconcomer",
    "reconcomed": "reconcomer",
    "reconcomemos": "reconcomer",
    "reconcomen": "reconcomer",
    "reconcomeremos": "reconcomer",
    "reconcomerá": "reconcomer",
    "reconcomerán": "reconcomer",
    "reconcomerás": "reconcomer",
    "reconcomeré": "reconcomer",
    "reconcomeréis": "reconcomer",
    "reconcomería": "reconcomer",
    "reconcomeríais": "reconcomer",
    "reconcomeríamos": "reconcomer",
    "reconcomerían": "reconcomer",
    "reconcomerías": "reconcomer",
    "reconcomes": "reconcomer",
    "reconcomida": "reconcomer",
    "reconcomidas": "reconcomer",
    "reconcomido": "reconcomer",
    "reconcomidos": "reconcomer",
    "reconcomiendo": "reconcomer",
    "reconcomiera": "reconcomer",
    "reconcomierais": "reconcomer",
    "reconcomieran": "reconcomer",
    "reconcomieras": "reconcomer",
    "reconcomiere": "reconcomer",
    "reconcomiereis": "reconcomer",
    "reconcomieren": "reconcomer",
    "reconcomieres": "reconcomer",
    "reconcomieron": "reconcomer",
    "reconcomiese": "reconcomer",
    "reconcomieseis": "reconcomer",
    "reconcomiesen": "reconcomer",
    "reconcomieses": "reconcomer",
    "reconcomimos": "reconcomer",
    "reconcomios": "reconcomio",
    "reconcomiste": "reconcomer",
    "reconcomisteis": "reconcomer",
    "reconcomiéramos": "reconcomer",
    "reconcomiéremos": "reconcomer",
    "reconcomiésemos": "reconcomer",
    "reconcomió": "reconcomer",
    "reconcomo": "reconcomer",
    "reconcomáis": "reconcomer",
    "reconcoméis": "reconcomer",
    "reconcomí": "reconcomer",
    "reconcomía": "reconcomer",
    "reconcomíais": "reconcomer",
    "reconcomíamos": "reconcomer",
    "reconcomían": "reconcomer",
    "reconcomías": "reconcomer",
    "recondena": "recondenar",
    "recondenaba": "recondenar",
    "recondenabais": "recondenar",
    "recondenaban": "recondenar",
    "recondenabas": "recondenar",
    "recondenad": "recondenar",
    "recondenada": "recondenar",
    "recondenadas": "recondenar",
    "recondenado": "recondenar",
    "recondenados": "recondenar",
    "recondenamos": "recondenar",
    "recondenan": "recondenar",
    "recondenando": "recondenar",
    "recondenara": "recondenar",
    "recondenarais": "recondenar",
    "recondenaran": "recondenar",
    "recondenaras": "recondenar",
    "recondenare": "recondenar",
    "recondenareis": "recondenar",
    "recondenaremos": "recondenar",
    "recondenaren": "recondenar",
    "recondenares": "recondenar",
    "recondenaron": "recondenar",
    "recondenará": "recondenar",
    "recondenarán": "recondenar",
    "recondenarás": "recondenar",
    "recondenaré": "recondenar",
    "recondenaréis": "recondenar",
    "recondenaría": "recondenar",
    "recondenaríais": "recondenar",
    "recondenaríamos": "recondenar",
    "recondenarían": "recondenar",
    "recondenarías": "recondenar",
    "recondenas": "recondenar",
    "recondenase": "recondenar",
    "recondenaseis": "recondenar",
    "recondenasen": "recondenar",
    "recondenases": "recondenar",
    "recondenaste": "recondenar",
    "recondenasteis": "recondenar",
    "recondene": "recondenar",
    "recondenemos": "recondenar",
    "recondenen": "recondenar",
    "recondenes": "recondenar",
    "recondeno": "recondenar",
    "recondenábamos": "recondenar",
    "recondenáis": "recondenar",
    "recondenáramos": "recondenar",
    "recondenáremos": "recondenar",
    "recondenásemos": "recondenar",
    "recondené": "recondenar",
    "recondenéis": "recondenar",
    "recondenó": "recondenar",
    "reconditeces": "reconditez",
    "reconducciones": "reconducción",
    "reconduce": "reconducir",
    "reconducen": "reconducir",
    "reconduces": "reconducir",
    "reconducid": "reconducir",
    "reconducida": "reconducir",
    "reconducidas": "reconducir",
    "reconducido": "reconducir",
    "reconducidos": "reconducir",
    "reconduciendo": "reconducir",
    "reconducimos": "reconducir",
    "reconduciremos": "reconducir",
    "reconducirá": "reconducir",
    "reconducirán": "reconducir",
    "reconducirás": "reconducir",
    "reconduciré": "reconducir",
    "reconduciréis": "reconducir",
    "reconduciría": "reconducir",
    "reconduciríais": "reconducir",
    "reconduciríamos": "reconducir",
    "reconducirían": "reconducir",
    "reconducirías": "reconducir",
    "reconducía": "reconducir",
    "reconducíais": "reconducir",
    "reconducíamos": "reconducir",
    "reconducían": "reconducir",
    "reconducías": "reconducir",
    "reconducís": "reconducir",
    "reconduje": "reconducir",
    "recondujera": "reconducir",
    "recondujerais": "reconducir",
    "recondujeran": "reconducir",
    "recondujeras": "reconducir",
    "recondujere": "reconducir",
    "recondujereis": "reconducir",
    "recondujeren": "reconducir",
    "recondujeres": "reconducir",
    "recondujeron": "reconducir",
    "recondujese": "reconducir",
    "recondujeseis": "reconducir",
    "recondujesen": "reconducir",
    "recondujeses": "reconducir",
    "recondujimos": "reconducir",
    "recondujiste": "reconducir",
    "recondujisteis": "reconducir",
    "recondujo": "reconducir",
    "recondujéramos": "reconducir",
    "recondujéremos": "reconducir",
    "recondujésemos": "reconducir",
    "reconduzca": "reconducir",
    "reconduzcamos": "reconducir",
    "reconduzcan": "reconducir",
    "reconduzcas": "reconducir",
    "reconduzco": "reconducir",
    "reconduzcáis": "reconducir",
    "reconexiones": "reconexión",
    "reconfiguraciones": "reconfiguración",
    "reconforta": "reconfortar",
    "reconfortaba": "reconfortar",
    "reconfortabais": "reconfortar",
    "reconfortaban": "reconfortar",
    "reconfortabas": "reconfortar",
    "reconfortad": "reconfortar",
    "reconfortada": "reconfortar",
    "reconfortadas": "reconfortar",
    "reconfortado": "reconfortar",
    "reconfortados": "reconfortar",
    "reconfortamos": "reconfortar",
    "reconfortan": "reconfortar",
    "reconfortando": "reconfortar",
    "reconfortantes": "reconfortante",
    "reconfortara": "reconfortar",
    "reconfortarais": "reconfortar",
    "reconfortaran": "reconfortar",
    "reconfortaras": "reconfortar",
    "reconfortare": "reconfortar",
    "reconfortareis": "reconfortar",
    "reconfortaremos": "reconfortar",
    "reconfortaren": "reconfortar",
    "reconfortares": "reconfortar",
    "reconfortaron": "reconfortar",
    "reconfortará": "reconfortar",
    "reconfortarán": "reconfortar",
    "reconfortarás": "reconfortar",
    "reconfortaré": "reconfortar",
    "reconfortaréis": "reconfortar",
    "reconfortaría": "reconfortar",
    "reconfortaríais": "reconfortar",
    "reconfortaríamos": "reconfortar",
    "reconfortarían": "reconfortar",
    "reconfortarías": "reconfortar",
    "reconfortas": "reconfortar",
    "reconfortase": "reconfortar",
    "reconfortaseis": "reconfortar",
    "reconfortasen": "reconfortar",
    "reconfortases": "reconfortar",
    "reconfortaste": "reconfortar",
    "reconfortasteis": "reconfortar",
    "reconforte": "reconfortar",
    "reconfortemos": "reconfortar",
    "reconforten": "reconfortar",
    "reconfortes": "reconfortar",
    "reconforto": "reconfortar",
    "reconfortábamos": "reconfortar",
    "reconfortáis": "reconfortar",
    "reconfortáramos": "reconfortar",
    "reconfortáremos": "reconfortar",
    "reconfortásemos": "reconfortar",
    "reconforté": "reconfortar",
    "reconfortéis": "reconfortar",
    "reconfortó": "reconfortar",
    "recongelaciones": "recongelación",
    "reconoce": "reconocer",
    "reconoced": "reconocer",
    "reconocedora": "reconocedor",
    "reconocedoras": "reconocedor",
    "reconocedores": "reconocedor",
    "reconocemos": "reconocer",
    "reconocen": "reconocer",
    "reconoceremos": "reconocer",
    "reconocerá": "reconocer",
    "reconocerán": "reconocer",
    "reconocerás": "reconocer",
    "reconoceré": "reconocer",
    "reconoceréis": "reconocer",
    "reconocería": "reconocer",
    "reconoceríais": "reconocer",
    "reconoceríamos": "reconocer",
    "reconocerían": "reconocer",
    "reconocerías": "reconocer",
    "reconoces": "reconocer",
    "reconocibles": "reconocible",
    "reconocida": "reconocer",
    "reconocidas": "reconocer",
    "reconocido": "reconocer",
    "reconocidos": "reconocer",
    "reconociendo": "reconocer",
    "reconociera": "reconocer",
    "reconocierais": "reconocer",
    "reconocieran": "reconocer",
    "reconocieras": "reconocer",
    "reconociere": "reconocer",
    "reconociereis": "reconocer",
    "reconocieren": "reconocer",
    "reconocieres": "reconocer",
    "reconocieron": "reconocer",
    "reconociese": "reconocer",
    "reconocieseis": "reconocer",
    "reconociesen": "reconocer",
    "reconocieses": "reconocer",
    "reconocimientos": "reconocimiento",
    "reconocimos": "reconocer",
    "reconociste": "reconocer",
    "reconocisteis": "reconocer",
    "reconociéramos": "reconocer",
    "reconociéremos": "reconocer",
    "reconociésemos": "reconocer",
    "reconoció": "reconocer",
    "reconocéis": "reconocer",
    "reconocí": "reconocer",
    "reconocía": "reconocer",
    "reconocíais": "reconocer",
    "reconocíamos": "reconocer",
    "reconocían": "reconocer",
    "reconocías": "reconocer",
    "reconozca": "reconocer",
    "reconozcamos": "reconocer",
    "reconozcan": "reconocer",
    "reconozcas": "reconocer",
    "reconozco": "reconocer",
    "reconozcáis": "reconocer",
    "reconquista": "reconquistar",
    "reconquistaba": "reconquistar",
    "reconquistabais": "reconquistar",
    "reconquistaban": "reconquistar",
    "reconquistabas": "reconquistar",
    "reconquistad": "reconquistar",
    "reconquistada": "reconquistar",
    "reconquistadas": "reconquistar",
    "reconquistado": "reconquistar",
    "reconquistadora": "reconquistador",
    "reconquistadoras": "reconquistador",
    "reconquistadores": "reconquistador",
    "reconquistados": "reconquistar",
    "reconquistamos": "reconquistar",
    "reconquistan": "reconquistar",
    "reconquistando": "reconquistar",
    "reconquistara": "reconquistar",
    "reconquistarais": "reconquistar",
    "reconquistaran": "reconquistar",
    "reconquistaras": "reconquistar",
    "reconquistare": "reconquistar",
    "reconquistareis": "reconquistar",
    "reconquistaremos": "reconquistar",
    "reconquistaren": "reconquistar",
    "reconquistares": "reconquistar",
    "reconquistaron": "reconquistar",
    "reconquistará": "reconquistar",
    "reconquistarán": "reconquistar",
    "reconquistarás": "reconquistar",
    "reconquistaré": "reconquistar",
    "reconquistaréis": "reconquistar",
    "reconquistaría": "reconquistar",
    "reconquistaríais": "reconquistar",
    "reconquistaríamos": "reconquistar",
    "reconquistarían": "reconquistar",
    "reconquistarías": "reconquistar",
    "reconquistas": "reconquistar",
    "reconquistase": "reconquistar",
    "reconquistaseis": "reconquistar",
    "reconquistasen": "reconquistar",
    "reconquistases": "reconquistar",
    "reconquistaste": "reconquistar",
    "reconquistasteis": "reconquistar",
    "reconquiste": "reconquistar",
    "reconquistemos": "reconquistar",
    "reconquisten": "reconquistar",
    "reconquistes": "reconquistar",
    "reconquisto": "reconquistar",
    "reconquistábamos": "reconquistar",
    "reconquistáis": "reconquistar",
    "reconquistáramos": "reconquistar",
    "reconquistáremos": "reconquistar",
    "reconquistásemos": "reconquistar",
    "reconquisté": "reconquistar",
    "reconquistéis": "reconquistar",
    "reconquistó": "reconquistar",
    "reconsidera": "reconsiderar",
    "reconsideraba": "reconsiderar",
    "reconsiderabais": "reconsiderar",
    "reconsideraban": "reconsiderar",
    "reconsiderabas": "reconsiderar",
    "reconsideraciones": "reconsideración",
    "reconsiderad": "reconsiderar",
    "reconsiderada": "reconsiderar",
    "reconsideradas": "reconsiderar",
    "reconsiderado": "reconsiderar",
    "reconsiderados": "reconsiderar",
    "reconsideramos": "reconsiderar",
    "reconsideran": "reconsiderar",
    "reconsiderando": "reconsiderar",
    "reconsiderara": "reconsiderar",
    "reconsiderarais": "reconsiderar",
    "reconsideraran": "reconsiderar",
    "reconsideraras": "reconsiderar",
    "reconsiderare": "reconsiderar",
    "reconsiderareis": "reconsiderar",
    "reconsideraremos": "reconsiderar",
    "reconsideraren": "reconsiderar",
    "reconsiderares": "reconsiderar",
    "reconsideraron": "reconsiderar",
    "reconsiderará": "reconsiderar",
    "reconsiderarán": "reconsiderar",
    "reconsiderarás": "reconsiderar",
    "reconsideraré": "reconsiderar",
    "reconsideraréis": "reconsiderar",
    "reconsideraría": "reconsiderar",
    "reconsideraríais": "reconsiderar",
    "reconsideraríamos": "reconsiderar",
    "reconsiderarían": "reconsiderar",
    "reconsiderarías": "reconsiderar",
    "reconsideras": "reconsiderar",
    "reconsiderase": "reconsiderar",
    "reconsideraseis": "reconsiderar",
    "reconsiderasen": "reconsiderar",
    "reconsiderases": "reconsiderar",
    "reconsideraste": "reconsiderar",
    "reconsiderasteis": "reconsiderar",
    "reconsidere": "reconsiderar",
    "reconsideremos": "reconsiderar",
    "reconsideren": "reconsiderar",
    "reconsideres": "reconsiderar",
    "reconsidero": "reconsiderar",
    "reconsiderábamos": "reconsiderar",
    "reconsideráis": "reconsiderar",
    "reconsideráramos": "reconsiderar",
    "reconsideráremos": "reconsiderar",
    "reconsiderásemos": "reconsiderar",
    "reconsideré": "reconsiderar",
    "reconsideréis": "reconsiderar",
    "reconsideró": "reconsiderar",
    "reconsignaciones": "reconsignación",
    "reconstituciones": "reconstitución",
    "reconstituid": "reconstituir",
    "reconstituida": "reconstituir",
    "reconstituidas": "reconstituir",
    "reconstituido": "reconstituir",
    "reconstituidos": "reconstituir",
    "reconstituimos": "reconstituir",
    "reconstituiremos": "reconstituir",
    "reconstituirá": "reconstituir",
    "reconstituirán": "reconstituir",
    "reconstituirás": "reconstituir",
    "reconstituiré": "reconstituir",
    "reconstituiréis": "reconstituir",
    "reconstituiría": "reconstituir",
    "reconstituiríais": "reconstituir",
    "reconstituiríamos": "reconstituir",
    "reconstituirían": "reconstituir",
    "reconstituirías": "reconstituir",
    "reconstituiste": "reconstituir",
    "reconstituisteis": "reconstituir",
    "reconstituya": "reconstituir",
    "reconstituyamos": "reconstituir",
    "reconstituyan": "reconstituir",
    "reconstituyas": "reconstituir",
    "reconstituye": "reconstituir",
    "reconstituyen": "reconstituir",
    "reconstituyendo": "reconstituir",
    "reconstituyentes": "reconstituyente",
    "reconstituyera": "reconstituir",
    "reconstituyerais": "reconstituir",
    "reconstituyeran": "reconstituir",
    "reconstituyeras": "reconstituir",
    "reconstituyere": "reconstituir",
    "reconstituyereis": "reconstituir",
    "reconstituyeren": "reconstituir",
    "reconstituyeres": "reconstituir",
    "reconstituyeron": "reconstituir",
    "reconstituyes": "reconstituir",
    "reconstituyese": "reconstituir",
    "reconstituyeseis": "reconstituir",
    "reconstituyesen": "reconstituir",
    "reconstituyeses": "reconstituir",
    "reconstituyo": "reconstituir",
    "reconstituyáis": "reconstituir",
    "reconstituyéramos": "reconstituir",
    "reconstituyéremos": "reconstituir",
    "reconstituyésemos": "reconstituir",
    "reconstituyó": "reconstituir",
    "reconstituí": "reconstituir",
    "reconstituía": "reconstituir",
    "reconstituíais": "reconstituir",
    "reconstituíamos": "reconstituir",
    "reconstituían": "reconstituir",
    "reconstituías": "reconstituir",
    "reconstituís": "reconstituir",
    "reconstrucciones": "reconstrucción",
    "reconstructiva": "reconstructivo",
    "reconstructivas": "reconstructivo",
    "reconstructivos": "reconstructivo",
    "reconstruid": "reconstruir",
    "reconstruida": "reconstruir",
    "reconstruidas": "reconstruir",
    "reconstruido": "reconstruir",
    "reconstruidos": "reconstruir",
    "reconstruimos": "reconstruir",
    "reconstruiremos": "reconstruir",
    "reconstruirá": "reconstruir",
    "reconstruirán": "reconstruir",
    "reconstruirás": "reconstruir",
    "reconstruiré": "reconstruir",
    "reconstruiréis": "reconstruir",
    "reconstruiría": "reconstruir",
    "reconstruiríais": "reconstruir",
    "reconstruiríamos": "reconstruir",
    "reconstruirían": "reconstruir",
    "reconstruirías": "reconstruir",
    "reconstruiste": "reconstruir",
    "reconstruisteis": "reconstruir",
    "reconstruya": "reconstruir",
    "reconstruyamos": "reconstruir",
    "reconstruyan": "reconstruir",
    "reconstruyas": "reconstruir",
    "reconstruye": "reconstruir",
    "reconstruyen": "reconstruir",
    "reconstruyendo": "reconstruir",
    "reconstruyera": "reconstruir",
    "reconstruyerais": "reconstruir",
    "reconstruyeran": "reconstruir",
    "reconstruyeras": "reconstruir",
    "reconstruyere": "reconstruir",
    "reconstruyereis": "reconstruir",
    "reconstruyeren": "reconstruir",
    "reconstruyeres": "reconstruir",
    "reconstruyeron": "reconstruir",
    "reconstruyes": "reconstruir",
    "reconstruyese": "reconstruir",
    "reconstruyeseis": "reconstruir",
    "reconstruyesen": "reconstruir",
    "reconstruyeses": "reconstruir",
    "reconstruyo": "reconstruir",
    "reconstruyáis": "reconstruir",
    "reconstruyéramos": "reconstruir",
    "reconstruyéremos": "reconstruir",
    "reconstruyésemos": "reconstruir",
    "reconstruyó": "reconstruir",
    "reconstruí": "reconstruir",
    "reconstruía": "reconstruir",
    "reconstruíais": "reconstruir",
    "reconstruíamos": "reconstruir",
    "reconstruían": "reconstruir",
    "reconstruías": "reconstruir",
    "reconstruís": "reconstruir",
    "recontaba": "recontar",
    "recontabais": "recontar",
    "recontaban": "recontar",
    "recontabas": "recontar",
    "recontad": "recontar",
    "recontada": "recontar",
    "recontadas": "recontar",
    "recontado": "recontar",
    "recontados": "recontar",
    "recontamos": "recontar",
    "recontando": "recontar",
    "recontara": "recontar",
    "recontarais": "recontar",
    "recontaran": "recontar",
    "recontaras": "recontar",
    "recontare": "recontar",
    "recontareis": "recontar",
    "recontaremos": "recontar",
    "recontaren": "recontar",
    "recontares": "recontar",
    "recontaron": "recontar",
    "recontará": "recontar",
    "recontarán": "recontar",
    "recontarás": "recontar",
    "recontaré": "recontar",
    "recontaréis": "recontar",
    "recontaría": "recontar",
    "recontaríais": "recontar",
    "recontaríamos": "recontar",
    "recontarían": "recontar",
    "recontarías": "recontar",
    "recontase": "recontar",
    "recontaseis": "recontar",
    "recontasen": "recontar",
    "recontases": "recontar",
    "recontaste": "recontar",
    "recontasteis": "recontar",
    "recontemos": "recontar",
    "recontábamos": "recontar",
    "recontáis": "recontar",
    "recontáramos": "recontar",
    "recontáremos": "recontar",
    "recontásemos": "recontar",
    "reconté": "recontar",
    "recontéis": "recontar",
    "recontó": "recontar",
    "reconvalece": "reconvalecer",
    "reconvaleced": "reconvalecer",
    "reconvalecemos": "reconvalecer",
    "reconvalecen": "reconvalecer",
    "reconvaleceremos": "reconvalecer",
    "reconvalecerá": "reconvalecer",
    "reconvalecerán": "reconvalecer",
    "reconvalecerás": "reconvalecer",
    "reconvaleceré": "reconvalecer",
    "reconvaleceréis": "reconvalecer",
    "reconvalecería": "reconvalecer",
    "reconvaleceríais": "reconvalecer",
    "reconvaleceríamos": "reconvalecer",
    "reconvalecerían": "reconvalecer",
    "reconvalecerías": "reconvalecer",
    "reconvaleces": "reconvalecer",
    "reconvalecida": "reconvalecer",
    "reconvalecidas": "reconvalecer",
    "reconvalecido": "reconvalecer",
    "reconvalecidos": "reconvalecer",
    "reconvaleciendo": "reconvalecer",
    "reconvaleciera": "reconvalecer",
    "reconvalecierais": "reconvalecer",
    "reconvalecieran": "reconvalecer",
    "reconvalecieras": "reconvalecer",
    "reconvaleciere": "reconvalecer",
    "reconvaleciereis": "reconvalecer",
    "reconvalecieren": "reconvalecer",
    "reconvalecieres": "reconvalecer",
    "reconvalecieron": "reconvalecer",
    "reconvaleciese": "reconvalecer",
    "reconvalecieseis": "reconvalecer",
    "reconvaleciesen": "reconvalecer",
    "reconvalecieses": "reconvalecer",
    "reconvalecimos": "reconvalecer",
    "reconvaleciste": "reconvalecer",
    "reconvalecisteis": "reconvalecer",
    "reconvaleciéramos": "reconvalecer",
    "reconvaleciéremos": "reconvalecer",
    "reconvaleciésemos": "reconvalecer",
    "reconvaleció": "reconvalecer",
    "reconvalecéis": "reconvalecer",
    "reconvalecí": "reconvalecer",
    "reconvalecía": "reconvalecer",
    "reconvalecíais": "reconvalecer",
    "reconvalecíamos": "reconvalecer",
    "reconvalecían": "reconvalecer",
    "reconvalecías": "reconvalecer",
    "reconvalezca": "reconvalecer",
    "reconvalezcamos": "reconvalecer",
    "reconvalezcan": "reconvalecer",
    "reconvalezcas": "reconvalecer",
    "reconvalezco": "reconvalecer",
    "reconvalezcáis": "reconvalecer",
    "reconven": "reconvenir",
    "reconvenciones": "reconvención",
    "reconvendremos": "reconvenir",
    "reconvendrá": "reconvenir",
    "reconvendrán": "reconvenir",
    "reconvendrás": "reconvenir",
    "reconvendré": "reconvenir",
    "reconvendréis": "reconvenir",
    "reconvendría": "reconvenir",
    "reconvendríais": "reconvenir",
    "reconvendríamos": "reconvenir",
    "reconvendrían": "reconvenir",
    "reconvendrías": "reconvenir",
    "reconvenga": "reconvenir",
    "reconvengamos": "reconvenir",
    "reconvengan": "reconvenir",
    "reconvengas": "reconvenir",
    "reconvengo": "reconvenir",
    "reconvengáis": "reconvenir",
    "reconvenid": "reconvenir",
    "reconvenida": "reconvenir",
    "reconvenidas": "reconvenir",
    "reconvenido": "reconvenir",
    "reconvenidos": "reconvenir",
    "reconvenimos": "reconvenir",
    "reconvenía": "reconvenir",
    "reconveníais": "reconvenir",
    "reconveníamos": "reconvenir",
    "reconvenían": "reconvenir",
    "reconvenías": "reconvenir",
    "reconvenís": "reconvenir",
    "reconversiones": "reconversión",
    "reconvertibles": "reconvertible",
    "reconvertid": "reconvertir",
    "reconvertida": "reconvertir",
    "reconvertidas": "reconvertir",
    "reconvertido": "reconvertir",
    "reconvertidos": "reconvertir",
    "reconvertimos": "reconvertir",
    "reconvertiremos": "reconvertir",
    "reconvertirá": "reconvertir",
    "reconvertirán": "reconvertir",
    "reconvertirás": "reconvertir",
    "reconvertiré": "reconvertir",
    "reconvertiréis": "reconvertir",
    "reconvertiría": "reconvertir",
    "reconvertiríais": "reconvertir",
    "reconvertiríamos": "reconvertir",
    "reconvertirían": "reconvertir",
    "reconvertirías": "reconvertir",
    "reconvertiste": "reconvertir",
    "reconvertisteis": "reconvertir",
    "reconvertí": "reconvertir",
    "reconvertía": "reconvertir",
    "reconvertíais": "reconvertir",
    "reconvertíamos": "reconvertir",
    "reconvertían": "reconvertir",
    "reconvertías": "reconvertir",
    "reconvertís": "reconvertir",
    "reconviene": "reconvenir",
    "reconvienen": "reconvenir",
    "reconvienes": "reconvenir",
    "reconvierta": "reconvertir",
    "reconviertan": "reconvertir",
    "reconviertas": "reconvertir",
    "reconvierte": "reconvertir",
    "reconvierten": "reconvertir",
    "reconviertes": "reconvertir",
    "reconvierto": "reconvertir",
    "reconvine": "reconvenir",
    "reconviniendo": "reconvenir",
    "reconviniera": "reconvenir",
    "reconvinierais": "reconvenir",
    "reconvinieran": "reconvenir",
    "reconvinieras": "reconvenir",
    "reconviniere": "reconvenir",
    "reconviniereis": "reconvenir",
    "reconvinieren": "reconvenir",
    "reconvinieres": "reconvenir",
    "reconvinieron": "reconvenir",
    "reconviniese": "reconvenir",
    "reconvinieseis": "reconvenir",
    "reconviniesen": "reconvenir",
    "reconvinieses": "reconvenir",
    "reconvinimos": "reconvenir",
    "reconviniste": "reconvenir",
    "reconvinisteis": "reconvenir",
    "reconviniéramos": "reconvenir",
    "reconviniéremos": "reconvenir",
    "reconviniésemos": "reconvenir",
    "reconvino": "reconvenir",
    "reconvirtamos": "reconvertir",
    "reconvirtiendo": "reconvertir",
    "reconvirtiera": "reconvertir",
    "reconvirtierais": "reconvertir",
    "reconvirtieran": "reconvertir",
    "reconvirtieras": "reconvertir",
    "reconvirtiere": "reconvertir",
    "reconvirtiereis": "reconvertir",
    "reconvirtieren": "reconvertir",
    "reconvirtieres": "reconvertir",
    "reconvirtieron": "reconvertir",
    "reconvirtiese": "reconvertir",
    "reconvirtieseis": "reconvertir",
    "reconvirtiesen": "reconvertir",
    "reconvirtieses": "reconvertir",
    "reconvirtiéramos": "reconvertir",
    "reconvirtiéremos": "reconvertir",
    "reconvirtiésemos": "reconvertir",
    "reconvirtió": "reconvertir",
    "reconvirtáis": "reconvertir",
    "recoordinaciones": "recoordinación",
    "recopila": "recopilar",
    "recopilaba": "recopilar",
    "recopilabais": "recopilar",
    "recopilaban": "recopilar",
    "recopilabas": "recopilar",
    "recopilaciones": "recopilación",
    "recopilad": "recopilar",
    "recopilada": "recopilar",
    "recopiladas": "recopilar",
    "recopilado": "recopilar",
    "recopiladora": "recopilador",
    "recopiladoras": "recopilador",
    "recopiladores": "recopilador",
    "recopilados": "recopilar",
    "recopilamos": "recopilar",
    "recopilan": "recopilar",
    "recopilando": "recopilar",
    "recopilara": "recopilar",
    "recopilarais": "recopilar",
    "recopilaran": "recopilar",
    "recopilaras": "recopilar",
    "recopilare": "recopilar",
    "recopilareis": "recopilar",
    "recopilaremos": "recopilar",
    "recopilaren": "recopilar",
    "recopilares": "recopilar",
    "recopilaron": "recopilar",
    "recopilará": "recopilar",
    "recopilarán": "recopilar",
    "recopilarás": "recopilar",
    "recopilaré": "recopilar",
    "recopilaréis": "recopilar",
    "recopilaría": "recopilar",
    "recopilaríais": "recopilar",
    "recopilaríamos": "recopilar",
    "recopilarían": "recopilar",
    "recopilarías": "recopilar",
    "recopilas": "recopilar",
    "recopilase": "recopilar",
    "recopilaseis": "recopilar",
    "recopilasen": "recopilar",
    "recopilases": "recopilar",
    "recopilaste": "recopilar",
    "recopilasteis": "recopilar",
    "recopilativa": "recopilativo",
    "recopilativas": "recopilativo",
    "recopilativos": "recopilativo",
    "recopilatorios": "recopilatorio",
    "recopile": "recopilar",
    "recopilemos": "recopilar",
    "recopilen": "recopilar",
    "recopiles": "recopilar",
    "recopilo": "recopilar",
    "recopilábamos": "recopilar",
    "recopiláis": "recopilar",
    "recopiláramos": "recopilar",
    "recopiláremos": "recopilar",
    "recopilásemos": "recopilar",
    "recopilé": "recopilar",
    "recopiléis": "recopilar",
    "recopiló": "recopilar",
    "recordaba": "recordar",
    "recordabais": "recordar",
    "recordaban": "recordar",
    "recordabas": "recordar",
    "recordables": "recordable",
    "recordaciones": "recordación",
    "recordad": "recordar",
    "recordada": "recordar",
    "recordadas": "recordar",
    "recordado": "recordar",
    "recordados": "recordar",
    "recordamos": "recordar",
    "recordando": "recordar",
    "recordara": "recordar",
    "recordarais": "recordar",
    "recordaran": "recordar",
    "recordaras": "recordar",
    "recordare": "recordar",
    "recordareis": "recordar",
    "recordaremos": "recordar",
    "recordaren": "recordar",
    "recordares": "recordar",
    "recordaron": "recordar",
    "recordará": "recordar",
    "recordarán": "recordar",
    "recordarás": "recordar",
    "recordaré": "recordar",
    "recordaréis": "recordar",
    "recordaría": "recordar",
    "recordaríais": "recordar",
    "recordaríamos": "recordar",
    "recordarían": "recordar",
    "recordarías": "recordar",
    "recordase": "recordar",
    "recordaseis": "recordar",
    "recordasen": "recordar",
    "recordases": "recordar",
    "recordaste": "recordar",
    "recordasteis": "recordar",
    "recordativa": "recordativo",
    "recordativas": "recordativo",
    "recordativos": "recordativo",
    "recordatoria": "recordatorio",
    "recordatorias": "recordatorio",
    "recordatorios": "recordatorio",
    "recordemos": "recordar",
    "recordistas": "recordista",
    "recordmans": "recordman",
    "recordábamos": "recordar",
    "recordáis": "recordar",
    "recordáramos": "recordar",
    "recordáremos": "recordar",
    "recordásemos": "recordar",
    "recordé": "recordar",
    "recordéis": "recordar",
    "recordó": "recordar",
    "recorra": "recorrer",
    "recorramos": "recorrer",
    "recorran": "recorrer",
    "recorras": "recorrer",
    "recorre": "recorrer",
    "recorred": "recorrer",
    "recorremos": "recorrer",
    "recorren": "recorrer",
    "recorreremos": "recorrer",
    "recorrerá": "recorrer",
    "recorrerán": "recorrer",
    "recorrerás": "recorrer",
    "recorreré": "recorrer",
    "recorreréis": "recorrer",
    "recorrería": "recorrer",
    "recorreríais": "recorrer",
    "recorreríamos": "recorrer",
    "recorrerían": "recorrer",
    "recorrerías": "recorrer",
    "recorres": "recorrer",
    "recorrida": "recorrer",
    "recorridas": "recorrer",
    "recorrido": "recorrer",
    "recorridos": "recorrido",
    "recorriendo": "recorrer",
    "recorriera": "recorrer",
    "recorrierais": "recorrer",
    "recorrieran": "recorrer",
    "recorrieras": "recorrer",
    "recorriere": "recorrer",
    "recorriereis": "recorrer",
    "recorrieren": "recorrer",
    "recorrieres": "recorrer",
    "recorrieron": "recorrer",
    "recorriese": "recorrer",
    "recorrieseis": "recorrer",
    "recorriesen": "recorrer",
    "recorrieses": "recorrer",
    "recorrimos": "recorrer",
    "recorriste": "recorrer",
    "recorristeis": "recorrer",
    "recorriéramos": "recorrer",
    "recorriéremos": "recorrer",
    "recorriésemos": "recorrer",
    "recorrió": "recorrer",
    "recorro": "recorrer",
    "recorráis": "recorrer",
    "recorréis": "recorrer",
    "recorrí": "recorrer",
    "recorría": "recorrer",
    "recorríais": "recorrer",
    "recorríamos": "recorrer",
    "recorrían": "recorrer",
    "recorrías": "recorrer",
    "recorta": "recortar",
    "recortaba": "recortar",
    "recortabais": "recortar",
    "recortaban": "recortar",
    "recortabas": "recortar",
    "recortables": "recortable",
    "recortad": "recortar",
    "recortada": "recortar",
    "recortadas": "recortar",
    "recortado": "recortar",
    "recortadora": "recortador",
    "recortadoras": "recortadora",
    "recortadores": "recortador",
    "recortados": "recortar",
    "recortaduras": "recortadura",
    "recortamos": "recortar",
    "recortan": "recortar",
    "recortando": "recortar",
    "recortara": "recortar",
    "recortarais": "recortar",
    "recortaran": "recortar",
    "recortaras": "recortar",
    "recortare": "recortar",
    "recortareis": "recortar",
    "recortaremos": "recortar",
    "recortaren": "recortar",
    "recortares": "recortar",
    "recortaron": "recortar",
    "recortará": "recortar",
    "recortarán": "recortar",
    "recortarás": "recortar",
    "recortaré": "recortar",
    "recortaréis": "recortar",
    "recortaría": "recortar",
    "recortaríais": "recortar",
    "recortaríamos": "recortar",
    "recortarían": "recortar",
    "recortarías": "recortar",
    "recortas": "recortar",
    "recortase": "recortar",
    "recortaseis": "recortar",
    "recortasen": "recortar",
    "recortases": "recortar",
    "recortaste": "recortar",
    "recortasteis": "recortar",
    "recorte": "recortar",
    "recortemos": "recortar",
    "recorten": "recortar",
    "recortes": "recorte",
    "recorto": "recortar",
    "recortábamos": "recortar",
    "recortáis": "recortar",
    "recortáramos": "recortar",
    "recortáremos": "recortar",
    "recortásemos": "recortar",
    "recorté": "recortar",
    "recortéis": "recortar",
    "recortó": "recortar",
    "recorva": "recorvo",
    "recorvaba": "recorvar",
    "recorvabais": "recorvar",
    "recorvaban": "recorvar",
    "recorvabas": "recorvar",
    "recorvad": "recorvar",
    "recorvada": "recorvar",
    "recorvadas": "recorvar",
    "recorvado": "recorvar",
    "recorvados": "recorvar",
    "recorvamos": "recorvar",
    "recorvan": "recorvar",
    "recorvando": "recorvar",
    "recorvara": "recorvar",
    "recorvarais": "recorvar",
    "recorvaran": "recorvar",
    "recorvaras": "recorvar",
    "recorvare": "recorvar",
    "recorvareis": "recorvar",
    "recorvaremos": "recorvar",
    "recorvaren": "recorvar",
    "recorvares": "recorvar",
    "recorvaron": "recorvar",
    "recorvará": "recorvar",
    "recorvarán": "recorvar",
    "recorvarás": "recorvar",
    "recorvaré": "recorvar",
    "recorvaréis": "recorvar",
    "recorvaría": "recorvar",
    "recorvaríais": "recorvar",
    "recorvaríamos": "recorvar",
    "recorvarían": "recorvar",
    "recorvarías": "recorvar",
    "recorvas": "recorvo",
    "recorvase": "recorvar",
    "recorvaseis": "recorvar",
    "recorvasen": "recorvar",
    "recorvases": "recorvar",
    "recorvaste": "recorvar",
    "recorvasteis": "recorvar",
    "recorve": "recorvar",
    "recorvemos": "recorvar",
    "recorven": "recorvar",
    "recorves": "recorvar",
    "recorvo": "recorvar",
    "recorvos": "recorvo",
    "recorvábamos": "recorvar",
    "recorváis": "recorvar",
    "recorváramos": "recorvar",
    "recorváremos": "recorvar",
    "recorvásemos": "recorvar",
    "recorvé": "recorvar",
    "recorvéis": "recorvar",
    "recorvó": "recorvar",
    "recosa": "recoser",
    "recosamos": "recoser",
    "recosan": "recoser",
    "recosas": "recoser",
    "recose": "recoser",
    "recosed": "recoser",
    "recosemos": "recoser",
    "recosen": "recoser",
    "recoseremos": "recoser",
    "recoserá": "recoser",
    "recoserán": "recoser",
    "recoserás": "recoser",
    "recoseré": "recoser",
    "recoseréis": "recoser",
    "recosería": "recoser",
    "recoseríais": "recoser",
    "recoseríamos": "recoser",
    "recoserían": "recoser",
    "recoserías": "recoser",
    "recoses": "recoser",
    "recosida": "recoser",
    "recosidas": "recoser",
    "recosido": "recoser",
    "recosidos": "recosido",
    "recosiendo": "recoser",
    "recosiera": "recoser",
    "recosierais": "recoser",
    "recosieran": "recoser",
    "recosieras": "recoser",
    "recosiere": "recoser",
    "recosiereis": "recoser",
    "recosieren": "recoser",
    "recosieres": "recoser",
    "recosieron": "recoser",
    "recosiese": "recoser",
    "recosieseis": "recoser",
    "recosiesen": "recoser",
    "recosieses": "recoser",
    "recosimos": "recoser",
    "recosiste": "recoser",
    "recosisteis": "recoser",
    "recosiéramos": "recoser",
    "recosiéremos": "recoser",
    "recosiésemos": "recoser",
    "recosió": "recoser",
    "recoso": "recoser",
    "recostaba": "recostar",
    "recostabais": "recostar",
    "recostaban": "recostar",
    "recostabas": "recostar",
    "recostad": "recostar",
    "recostada": "recostar",
    "recostadas": "recostar",
    "recostado": "recostar",
    "recostados": "recostar",
    "recostamos": "recostar",
    "recostando": "recostar",
    "recostara": "recostar",
    "recostarais": "recostar",
    "recostaran": "recostar",
    "recostaras": "recostar",
    "recostare": "recostar",
    "recostareis": "recostar",
    "recostaremos": "recostar",
    "recostaren": "recostar",
    "recostares": "recostar",
    "recostaron": "recostar",
    "recostará": "recostar",
    "recostarán": "recostar",
    "recostarás": "recostar",
    "recostaré": "recostar",
    "recostaréis": "recostar",
    "recostaría": "recostar",
    "recostaríais": "recostar",
    "recostaríamos": "recostar",
    "recostarían": "recostar",
    "recostarías": "recostar",
    "recostase": "recostar",
    "recostaseis": "recostar",
    "recostasen": "recostar",
    "recostases": "recostar",
    "recostaste": "recostar",
    "recostasteis": "recostar",
    "recostemos": "recostar",
    "recostábamos": "recostar",
    "recostáis": "recostar",
    "recostáramos": "recostar",
    "recostáremos": "recostar",
    "recostásemos": "recostar",
    "recosté": "recostar",
    "recostéis": "recostar",
    "recostó": "recostar",
    "recosáis": "recoser",
    "recoséis": "recoser",
    "recosí": "recoser",
    "recosía": "recoser",
    "recosíais": "recoser",
    "recosíamos": "recoser",
    "recosían": "recoser",
    "recosías": "recoser",
    "recova": "recovar",
    "recovaba": "recovar",
    "recovabais": "recovar",
    "recovaban": "recovar",
    "recovabas": "recovar",
    "recovad": "recovar",
    "recovada": "recovar",
    "recovadas": "recovar",
    "recovado": "recovar",
    "recovados": "recovar",
    "recovamos": "recovar",
    "recovan": "recovar",
    "recovando": "recovar",
    "recovara": "recovar",
    "recovarais": "recovar",
    "recovaran": "recovar",
    "recovaras": "recovar",
    "recovare": "recovar",
    "recovareis": "recovar",
    "recovaremos": "recovar",
    "recovaren": "recovar",
    "recovares": "recovar",
    "recovaron": "recovar",
    "recovará": "recovar",
    "recovarán": "recovar",
    "recovarás": "recovar",
    "recovaré": "recovar",
    "recovaréis": "recovar",
    "recovaría": "recovar",
    "recovaríais": "recovar",
    "recovaríamos": "recovar",
    "recovarían": "recovar",
    "recovarías": "recovar",
    "recovas": "recovar",
    "recovase": "recovar",
    "recovaseis": "recovar",
    "recovasen": "recovar",
    "recovases": "recovar",
    "recovaste": "recovar",
    "recovasteis": "recovar",
    "recove": "recovar",
    "recovecos": "recoveco",
    "recovemos": "recovar",
    "recoven": "recovar",
    "recovera": "recovero",
    "recoveras": "recovero",
    "recoveros": "recovero",
    "recoves": "recovar",
    "recovo": "recovar",
    "recovábamos": "recovar",
    "recováis": "recovar",
    "recováramos": "recovar",
    "recováremos": "recovar",
    "recovásemos": "recovar",
    "recové": "recovar",
    "recovéis": "recovar",
    "recovó": "recovar",
    "recozamos": "recocer",
    "recozáis": "recocer",
    "recrea": "recrear",
    "recreaba": "recrear",
    "recreabais": "recrear",
    "recreaban": "recrear",
    "recreabas": "recrear",
    "recreaciones": "recreación",
    "recread": "recrear",
    "recreada": "recrear",
    "recreadas": "recrear",
    "recreado": "recrear",
    "recreados": "recrear",
    "recreamos": "recrear",
    "recrean": "recrear",
    "recreando": "recrear",
    "recreara": "recrear",
    "recrearais": "recrear",
    "recrearan": "recrear",
    "recrearas": "recrear",
    "recreare": "recrear",
    "recreareis": "recrear",
    "recrearemos": "recrear",
    "recrearen": "recrear",
    "recreares": "recrear",
    "recrearon": "recrear",
    "recreará": "recrear",
    "recrearán": "recrear",
    "recrearás": "recrear",
    "recrearé": "recrear",
    "recrearéis": "recrear",
    "recrearía": "recrear",
    "recrearíais": "recrear",
    "recrearíamos": "recrear",
    "recrearían": "recrear",
    "recrearías": "recrear",
    "recreas": "recrear",
    "recrease": "recrear",
    "recreaseis": "recrear",
    "recreasen": "recrear",
    "recreases": "recrear",
    "recreaste": "recrear",
    "recreasteis": "recrear",
    "recreativa": "recreativo",
    "recreativas": "recreativo",
    "recreativos": "recreativo",
    "recrece": "recrecer",
    "recreced": "recrecer",
    "recrecemos": "recrecer",
    "recrecen": "recrecer",
    "recreceremos": "recrecer",
    "recrecerá": "recrecer",
    "recrecerán": "recrecer",
    "recrecerás": "recrecer",
    "recreceré": "recrecer",
    "recreceréis": "recrecer",
    "recrecería": "recrecer",
    "recreceríais": "recrecer",
    "recreceríamos": "recrecer",
    "recrecerían": "recrecer",
    "recrecerías": "recrecer",
    "recreces": "recrecer",
    "recrecida": "recrecer",
    "recrecidas": "recrecer",
    "recrecido": "recrecer",
    "recrecidos": "recrecer",
    "recreciendo": "recrecer",
    "recreciera": "recrecer",
    "recrecierais": "recrecer",
    "recrecieran": "recrecer",
    "recrecieras": "recrecer",
    "recreciere": "recrecer",
    "recreciereis": "recrecer",
    "recrecieren": "recrecer",
    "recrecieres": "recrecer",
    "recrecieron": "recrecer",
    "recreciese": "recrecer",
    "recrecieseis": "recrecer",
    "recreciesen": "recrecer",
    "recrecieses": "recrecer",
    "recrecimientos": "recrecimiento",
    "recrecimos": "recrecer",
    "recreciste": "recrecer",
    "recrecisteis": "recrecer",
    "recreciéramos": "recrecer",
    "recreciéremos": "recrecer",
    "recreciésemos": "recrecer",
    "recreció": "recrecer",
    "recrecéis": "recrecer",
    "recrecí": "recrecer",
    "recrecía": "recrecer",
    "recrecíais": "recrecer",
    "recrecíamos": "recrecer",
    "recrecían": "recrecer",
    "recrecías": "recrecer",
    "recree": "recrear",
    "recreemos": "recrear",
    "recreen": "recrear",
    "recrees": "recrear",
    "recreo": "recrear",
    "recreos": "recreo",
    "recrezca": "recrecer",
    "recrezcamos": "recrecer",
    "recrezcan": "recrecer",
    "recrezcas": "recrecer",
    "recrezco": "recrecer",
    "recrezcáis": "recrecer",
    "recreábamos": "recrear",
    "recreáis": "recrear",
    "recreáramos": "recrear",
    "recreáremos": "recrear",
    "recreásemos": "recrear",
    "recreé": "recrear",
    "recreéis": "recrear",
    "recreó": "recrear",
    "recriaba": "recriar",
    "recriabais": "recriar",
    "recriaban": "recriar",
    "recriabas": "recriar",
    "recriad": "recriar",
    "recriada": "recriar",
    "recriadas": "recriar",
    "recriado": "recriar",
    "recriadora": "recriador",
    "recriadoras": "recriador",
    "recriadores": "recriador",
    "recriados": "recriar",
    "recriamos": "recriar",
    "recriando": "recriar",
    "recriara": "recriar",
    "recriarais": "recriar",
    "recriaran": "recriar",
    "recriaras": "recriar",
    "recriare": "recriar",
    "recriareis": "recriar",
    "recriaremos": "recriar",
    "recriaren": "recriar",
    "recriares": "recriar",
    "recriaron": "recriar",
    "recriará": "recriar",
    "recriarán": "recriar",
    "recriarás": "recriar",
    "recriaré": "recriar",
    "recriaréis": "recriar",
    "recriaría": "recriar",
    "recriaríais": "recriar",
    "recriaríamos": "recriar",
    "recriarían": "recriar",
    "recriarías": "recriar",
    "recriase": "recriar",
    "recriaseis": "recriar",
    "recriasen": "recriar",
    "recriases": "recriar",
    "recriaste": "recriar",
    "recriasteis": "recriar",
    "recriemos": "recriar",
    "recrimina": "recriminar",
    "recriminaba": "recriminar",
    "recriminabais": "recriminar",
    "recriminaban": "recriminar",
    "recriminabas": "recriminar",
    "recriminaciones": "recriminación",
    "recriminad": "recriminar",
    "recriminada": "recriminar",
    "recriminadas": "recriminar",
    "recriminado": "recriminar",
    "recriminadora": "recriminador",
    "recriminadoras": "recriminador",
    "recriminadores": "recriminador",
    "recriminados": "recriminar",
    "recriminamos": "recriminar",
    "recriminan": "recriminar",
    "recriminando": "recriminar",
    "recriminara": "recriminar",
    "recriminarais": "recriminar",
    "recriminaran": "recriminar",
    "recriminaras": "recriminar",
    "recriminare": "recriminar",
    "recriminareis": "recriminar",
    "recriminaremos": "recriminar",
    "recriminaren": "recriminar",
    "recriminares": "recriminar",
    "recriminaron": "recriminar",
    "recriminará": "recriminar",
    "recriminarán": "recriminar",
    "recriminarás": "recriminar",
    "recriminaré": "recriminar",
    "recriminaréis": "recriminar",
    "recriminaría": "recriminar",
    "recriminaríais": "recriminar",
    "recriminaríamos": "recriminar",
    "recriminarían": "recriminar",
    "recriminarías": "recriminar",
    "recriminas": "recriminar",
    "recriminase": "recriminar",
    "recriminaseis": "recriminar",
    "recriminasen": "recriminar",
    "recriminases": "recriminar",
    "recriminaste": "recriminar",
    "recriminasteis": "recriminar",
    "recriminatoria": "recriminatorio",
    "recriminatorias": "recriminatorio",
    "recriminatorios": "recriminatorio",
    "recrimine": "recriminar",
    "recriminemos": "recriminar",
    "recriminen": "recriminar",
    "recrimines": "recriminar",
    "recrimino": "recriminar",
    "recriminábamos": "recriminar",
    "recrimináis": "recriminar",
    "recrimináramos": "recriminar",
    "recrimináremos": "recriminar",
    "recriminásemos": "recriminar",
    "recriminé": "recriminar",
    "recriminéis": "recriminar",
    "recriminó": "recriminar",
    "recristalice": "recristalizar",
    "recristalicemos": "recristalizar",
    "recristalicen": "recristalizar",
    "recristalices": "recristalizar",
    "recristalicé": "recristalizar",
    "recristalicéis": "recristalizar",
    "recristaliza": "recristalizar",
    "recristalizaba": "recristalizar",
    "recristalizabais": "recristalizar",
    "recristalizaban": "recristalizar",
    "recristalizabas": "recristalizar",
    "recristalizaciones": "recristalización",
    "recristalizad": "recristalizar",
    "recristalizada": "recristalizar",
    "recristalizadas": "recristalizar",
    "recristalizado": "recristalizar",
    "recristalizados": "recristalizar",
    "recristalizamos": "recristalizar",
    "recristalizan": "recristalizar",
    "recristalizando": "recristalizar",
    "recristalizara": "recristalizar",
    "recristalizarais": "recristalizar",
    "recristalizaran": "recristalizar",
    "recristalizaras": "recristalizar",
    "recristalizare": "recristalizar",
    "recristalizareis": "recristalizar",
    "recristalizaremos": "recristalizar",
    "recristalizaren": "recristalizar",
    "recristalizares": "recristalizar",
    "recristalizaron": "recristalizar",
    "recristalizará": "recristalizar",
    "recristalizarán": "recristalizar",
    "recristalizarás": "recristalizar",
    "recristalizaré": "recristalizar",
    "recristalizaréis": "recristalizar",
    "recristalizaría": "recristalizar",
    "recristalizaríais": "recristalizar",
    "recristalizaríamos": "recristalizar",
    "recristalizarían": "recristalizar",
    "recristalizarías": "recristalizar",
    "recristalizas": "recristalizar",
    "recristalizase": "recristalizar",
    "recristalizaseis": "recristalizar",
    "recristalizasen": "recristalizar",
    "recristalizases": "recristalizar",
    "recristalizaste": "recristalizar",
    "recristalizasteis": "recristalizar",
    "recristalizo": "recristalizar",
    "recristalizábamos": "recristalizar",
    "recristalizáis": "recristalizar",
    "recristalizáramos": "recristalizar",
    "recristalizáremos": "recristalizar",
    "recristalizásemos": "recristalizar",
    "recristalizó": "recristalizar",
    "recriábamos": "recriar",
    "recriáis": "recriar",
    "recriáramos": "recriar",
    "recriáremos": "recriar",
    "recriásemos": "recriar",
    "recrié": "recriar",
    "recriéis": "recriar",
    "recrió": "recriar",
    "recrucetada": "recrucetado",
    "recrucetadas": "recrucetado",
    "recrucetados": "recrucetado",
    "recrudece": "recrudecer",
    "recrudeced": "recrudecer",
    "recrudecemos": "recrudecer",
    "recrudecen": "recrudecer",
    "recrudeceremos": "recrudecer",
    "recrudecerá": "recrudecer",
    "recrudecerán": "recrudecer",
    "recrudecerás": "recrudecer",
    "recrudeceré": "recrudecer",
    "recrudeceréis": "recrudecer",
    "recrudecería": "recrudecer",
    "recrudeceríais": "recrudecer",
    "recrudeceríamos": "recrudecer",
    "recrudecerían": "recrudecer",
    "recrudecerías": "recrudecer",
    "recrudeces": "recrudecer",
    "recrudecida": "recrudecer",
    "recrudecidas": "recrudecer",
    "recrudecido": "recrudecer",
    "recrudecidos": "recrudecer",
    "recrudeciendo": "recrudecer",
    "recrudeciera": "recrudecer",
    "recrudecierais": "recrudecer",
    "recrudecieran": "recrudecer",
    "recrudecieras": "recrudecer",
    "recrudeciere": "recrudecer",
    "recrudeciereis": "recrudecer",
    "recrudecieren": "recrudecer",
    "recrudecieres": "recrudecer",
    "recrudecieron": "recrudecer",
    "recrudeciese": "recrudecer",
    "recrudecieseis": "recrudecer",
    "recrudeciesen": "recrudecer",
    "recrudecieses": "recrudecer",
    "recrudecimientos": "recrudecimiento",
    "recrudecimos": "recrudecer",
    "recrudeciste": "recrudecer",
    "recrudecisteis": "recrudecer",
    "recrudeciéramos": "recrudecer",
    "recrudeciéremos": "recrudecer",
    "recrudeciésemos": "recrudecer",
    "recrudeció": "recrudecer",
    "recrudecéis": "recrudecer",
    "recrudecí": "recrudecer",
    "recrudecía": "recrudecer",
    "recrudecíais": "recrudecer",
    "recrudecíamos": "recrudecer",
    "recrudecían": "recrudecer",
    "recrudecías": "recrudecer",
    "recrudescencias": "recrudescencia",
    "recrudezca": "recrudecer",
    "recrudezcamos": "recrudecer",
    "recrudezcan": "recrudecer",
    "recrudezcas": "recrudecer",
    "recrudezco": "recrudecer",
    "recrudezcáis": "recrudecer",
    "recría": "recriar",
    "recrían": "recriar",
    "recrías": "recriar",
    "recríe": "recriar",
    "recríen": "recriar",
    "recríes": "recriar",
    "recrío": "recriar",
    "recta": "recto",
    "rectales": "rectal",
    "rectangulares": "rectangular",
    "rectangularidades": "rectangularidad",
    "rectas": "recto",
    "rectifica": "rectificar",
    "rectificaba": "rectificar",
    "rectificabais": "rectificar",
    "rectificaban": "rectificar",
    "rectificabas": "rectificar",
    "rectificables": "rectificable",
    "rectificaciones": "rectificación",
    "rectificad": "rectificar",
    "rectificada": "rectificar",
    "rectificadas": "rectificar",
    "rectificado": "rectificar",
    "rectificadora": "rectificador",
    "rectificadoras": "rectificadora",
    "rectificadores": "rectificador",
    "rectificados": "rectificar",
    "rectificamos": "rectificar",
    "rectifican": "rectificar",
    "rectificando": "rectificar",
    "rectificantes": "rectificante",
    "rectificara": "rectificar",
    "rectificarais": "rectificar",
    "rectificaran": "rectificar",
    "rectificaras": "rectificar",
    "rectificare": "rectificar",
    "rectificareis": "rectificar",
    "rectificaremos": "rectificar",
    "rectificaren": "rectificar",
    "rectificares": "rectificar",
    "rectificaron": "rectificar",
    "rectificará": "rectificar",
    "rectificarán": "rectificar",
    "rectificarás": "rectificar",
    "rectificaré": "rectificar",
    "rectificaréis": "rectificar",
    "rectificaría": "rectificar",
    "rectificaríais": "rectificar",
    "rectificaríamos": "rectificar",
    "rectificarían": "rectificar",
    "rectificarías": "rectificar",
    "rectificas": "rectificar",
    "rectificase": "rectificar",
    "rectificaseis": "rectificar",
    "rectificasen": "rectificar",
    "rectificases": "rectificar",
    "rectificaste": "rectificar",
    "rectificasteis": "rectificar",
    "rectificativa": "rectificativo",
    "rectificativas": "rectificativo",
    "rectificativos": "rectificativo",
    "rectifico": "rectificar",
    "rectificábamos": "rectificar",
    "rectificáis": "rectificar",
    "rectificáramos": "rectificar",
    "rectificáremos": "rectificar",
    "rectificásemos": "rectificar",
    "rectificó": "rectificar",
    "rectifique": "rectificar",
    "rectifiquemos": "rectificar",
    "rectifiquen": "rectificar",
    "rectifiques": "rectificar",
    "rectifiqué": "rectificar",
    "rectifiquéis": "rectificar",
    "rectilínea": "rectilíneo",
    "rectilíneas": "rectilíneo",
    "rectilíneos": "rectilíneo",
    "rectitudes": "rectitud",
    "rectococcígea": "rectococcígeo",
    "rectococcígeas": "rectococcígeo",
    "rectococcígeos": "rectococcígeo",
    "rectora": "rector",
    "rectorados": "rectorado",
    "rectorales": "rectoral",
    "rectoras": "rector",
    "rectores": "rector",
    "rectorragias": "rectorragia",
    "rectorías": "rectoría",
    "rectos": "recto",
    "rectoscopias": "rectoscopia",
    "rectoscopios": "rectoscopio",
    "rectouretrales": "rectouretral",
    "rectouterina": "rectouterino",
    "rectouterinas": "rectouterino",
    "rectouterinos": "rectouterino",
    "rectovaginales": "rectovaginal",
    "rectovesicales": "rectovesical",
    "rectángula": "rectángulo",
    "rectángulas": "rectángulo",
    "rectángulos": "rectángulo",
    "rectímetros": "rectímetro",
    "recuadra": "recuadrar",
    "recuadraba": "recuadrar",
    "recuadrabais": "recuadrar",
    "recuadraban": "recuadrar",
    "recuadrabas": "recuadrar",
    "recuadrad": "recuadrar",
    "recuadrada": "recuadrar",
    "recuadradas": "recuadrar",
    "recuadrado": "recuadrar",
    "recuadrados": "recuadrar",
    "recuadramos": "recuadrar",
    "recuadran": "recuadrar",
    "recuadrando": "recuadrar",
    "recuadrara": "recuadrar",
    "recuadrarais": "recuadrar",
    "recuadraran": "recuadrar",
    "recuadraras": "recuadrar",
    "recuadrare": "recuadrar",
    "recuadrareis": "recuadrar",
    "recuadraremos": "recuadrar",
    "recuadraren": "recuadrar",
    "recuadrares": "recuadrar",
    "recuadraron": "recuadrar",
    "recuadrará": "recuadrar",
    "recuadrarán": "recuadrar",
    "recuadrarás": "recuadrar",
    "recuadraré": "recuadrar",
    "recuadraréis": "recuadrar",
    "recuadraría": "recuadrar",
    "recuadraríais": "recuadrar",
    "recuadraríamos": "recuadrar",
    "recuadrarían": "recuadrar",
    "recuadrarías": "recuadrar",
    "recuadras": "recuadrar",
    "recuadrase": "recuadrar",
    "recuadraseis": "recuadrar",
    "recuadrasen": "recuadrar",
    "recuadrases": "recuadrar",
    "recuadraste": "recuadrar",
    "recuadrasteis": "recuadrar",
    "recuadre": "recuadrar",
    "recuadremos": "recuadrar",
    "recuadren": "recuadrar",
    "recuadres": "recuadrar",
    "recuadro": "recuadrar",
    "recuadros": "recuadro",
    "recuadrábamos": "recuadrar",
    "recuadráis": "recuadrar",
    "recuadráramos": "recuadrar",
    "recuadráremos": "recuadrar",
    "recuadrásemos": "recuadrar",
    "recuadré": "recuadrar",
    "recuadréis": "recuadrar",
    "recuadró": "recuadrar",
    "recuartas": "recuarta",
    "recuas": "recua",
    "recubierta": "recubrir",
    "recubiertas": "recubrir",
    "recubierto": "recubrir",
    "recubiertos": "recubrir",
    "recubra": "recubrir",
    "recubramos": "recubrir",
    "recubran": "recubrir",
    "recubras": "recubrir",
    "recubre": "recubrir",
    "recubren": "recubrir",
    "recubres": "recubrir",
    "recubrid": "recubrir",
    "recubridora": "recubridor",
    "recubridoras": "recubridor",
    "recubridores": "recubridor",
    "recubriendo": "recubrir",
    "recubriera": "recubrir",
    "recubrierais": "recubrir",
    "recubrieran": "recubrir",
    "recubrieras": "recubrir",
    "recubriere": "recubrir",
    "recubriereis": "recubrir",
    "recubrieren": "recubrir",
    "recubrieres": "recubrir",
    "recubrieron": "recubrir",
    "recubriese": "recubrir",
    "recubrieseis": "recubrir",
    "recubriesen": "recubrir",
    "recubrieses": "recubrir",
    "recubrimientos": "recubrimiento",
    "recubrimos": "recubrir",
    "recubriremos": "recubrir",
    "recubrirá": "recubrir",
    "recubrirán": "recubrir",
    "recubrirás": "recubrir",
    "recubriré": "recubrir",
    "recubriréis": "recubrir",
    "recubriría": "recubrir",
    "recubriríais": "recubrir",
    "recubriríamos": "recubrir",
    "recubrirían": "recubrir",
    "recubrirías": "recubrir",
    "recubriste": "recubrir",
    "recubristeis": "recubrir",
    "recubriéramos": "recubrir",
    "recubriéremos": "recubrir",
    "recubriésemos": "recubrir",
    "recubrió": "recubrir",
    "recubro": "recubrir",
    "recubráis": "recubrir",
    "recubrí": "recubrir",
    "recubría": "recubrir",
    "recubríais": "recubrir",
    "recubríamos": "recubrir",
    "recubrían": "recubrir",
    "recubrías": "recubrir",
    "recubrís": "recubrir",
    "recuece": "recocer",
    "recuecen": "recocer",
    "recueces": "recocer",
    "recuela": "recolar",
    "recuelan": "recolar",
    "recuelas": "recolar",
    "recuele": "recolar",
    "recuelen": "recolar",
    "recueles": "recolar",
    "recuelo": "recolar",
    "recuelos": "recuelo",
    "recuenta": "recontar",
    "recuentan": "recontar",
    "recuentas": "recontar",
    "recuente": "recontar",
    "recuenten": "recontar",
    "recuentes": "recontar",
    "recuento": "recontar",
    "recuentos": "recuento",
    "recuerda": "recordar",
    "recuerdan": "recordar",
    "recuerdas": "recordar",
    "recuerde": "recordar",
    "recuerden": "recordar",
    "recuerdes": "recordar",
    "recuerdo": "recordar",
    "recuerdos": "recuerdo",
    "recueros": "recuero",
    "recuesta": "recuestar",
    "recuestaba": "recuestar",
    "recuestabais": "recuestar",
    "recuestaban": "recuestar",
    "recuestabas": "recuestar",
    "recuestad": "recuestar",
    "recuestada": "recuestar",
    "recuestadas": "recuestar",
    "recuestado": "recuestar",
    "recuestados": "recuestar",
    "recuestamos": "recuestar",
    "recuestan": "recuestar",
    "recuestando": "recuestar",
    "recuestara": "recuestar",
    "recuestarais": "recuestar",
    "recuestaran": "recuestar",
    "recuestaras": "recuestar",
    "recuestare": "recuestar",
    "recuestareis": "recuestar",
    "recuestaremos": "recuestar",
    "recuestaren": "recuestar",
    "recuestares": "recuestar",
    "recuestaron": "recuestar",
    "recuestará": "recuestar",
    "recuestarán": "recuestar",
    "recuestarás": "recuestar",
    "recuestaré": "recuestar",
    "recuestaréis": "recuestar",
    "recuestaría": "recuestar",
    "recuestaríais": "recuestar",
    "recuestaríamos": "recuestar",
    "recuestarían": "recuestar",
    "recuestarías": "recuestar",
    "recuestas": "recuestar",
    "recuestase": "recuestar",
    "recuestaseis": "recuestar",
    "recuestasen": "recuestar",
    "recuestases": "recuestar",
    "recuestaste": "recuestar",
    "recuestasteis": "recuestar",
    "recueste": "recuestar",
    "recuestemos": "recuestar",
    "recuesten": "recuestar",
    "recuestes": "recuestar",
    "recuesto": "recuestar",
    "recuestos": "recuesto",
    "recuestábamos": "recuestar",
    "recuestáis": "recuestar",
    "recuestáramos": "recuestar",
    "recuestáremos": "recuestar",
    "recuestásemos": "recuestar",
    "recuesté": "recuestar",
    "recuestéis": "recuestar",
    "recuestó": "recuestar",
    "recueza": "recocer",
    "recuezan": "recocer",
    "recuezas": "recocer",
    "recuezo": "recocer",
    "recula": "reculo",
    "reculaba": "recular",
    "reculabais": "recular",
    "reculaban": "recular",
    "reculabas": "recular",
    "reculad": "recular",
    "reculada": "recular",
    "reculadas": "recular",
    "reculado": "recular",
    "reculados": "recular",
    "reculamos": "recular",
    "reculan": "recular",
    "reculando": "recular",
    "reculara": "recular",
    "recularais": "recular",
    "recularan": "recular",
    "recularas": "recular",
    "reculare": "recular",
    "reculareis": "recular",
    "recularemos": "recular",
    "recularen": "recular",
    "reculares": "recular",
    "recularon": "recular",
    "reculará": "recular",
    "recularán": "recular",
    "recularás": "recular",
    "recularé": "recular",
    "recularéis": "recular",
    "recularía": "recular",
    "recularíais": "recular",
    "recularíamos": "recular",
    "recularían": "recular",
    "recularías": "recular",
    "reculas": "reculo",
    "reculase": "recular",
    "reculaseis": "recular",
    "reculasen": "recular",
    "reculases": "recular",
    "reculaste": "recular",
    "reculasteis": "recular",
    "recule": "recular",
    "reculemos": "recular",
    "reculen": "recular",
    "recules": "recular",
    "reculo": "recular",
    "reculos": "reculo",
    "reculábamos": "recular",
    "reculáis": "recular",
    "reculáramos": "recular",
    "reculáremos": "recular",
    "reculásemos": "recular",
    "reculé": "recular",
    "reculéis": "recular",
    "reculó": "recular",
    "recupera": "recuperar",
    "recuperaba": "recuperar",
    "recuperabais": "recuperar",
    "recuperaban": "recuperar",
    "recuperabas": "recuperar",
    "recuperables": "recuperable",
    "recuperaciones": "recuperación",
    "recuperad": "recuperar",
    "recuperada": "recuperar",
    "recuperadas": "recuperar",
    "recuperado": "recuperar",
    "recuperadora": "recuperador",
    "recuperadoras": "recuperador",
    "recuperadores": "recuperador",
    "recuperados": "recuperar",
    "recuperamos": "recuperar",
    "recuperan": "recuperar",
    "recuperando": "recuperar",
    "recuperara": "recuperar",
    "recuperarais": "recuperar",
    "recuperaran": "recuperar",
    "recuperaras": "recuperar",
    "recuperare": "recuperar",
    "recuperareis": "recuperar",
    "recuperaremos": "recuperar",
    "recuperaren": "recuperar",
    "recuperares": "recuperar",
    "recuperaron": "recuperar",
    "recuperará": "recuperar",
    "recuperarán": "recuperar",
    "recuperarás": "recuperar",
    "recuperaré": "recuperar",
    "recuperaréis": "recuperar",
    "recuperaría": "recuperar",
    "recuperaríais": "recuperar",
    "recuperaríamos": "recuperar",
    "recuperarían": "recuperar",
    "recuperarías": "recuperar",
    "recuperas": "recuperar",
    "recuperase": "recuperar",
    "recuperaseis": "recuperar",
    "recuperasen": "recuperar",
    "recuperases": "recuperar",
    "recuperaste": "recuperar",
    "recuperasteis": "recuperar",
    "recuperativa": "recuperativo",
    "recuperativas": "recuperativo",
    "recuperativos": "recuperativo",
    "recupere": "recuperar",
    "recuperemos": "recuperar",
    "recuperen": "recuperar",
    "recuperes": "recuperar",
    "recupero": "recuperar",
    "recuperábamos": "recuperar",
    "recuperáis": "recuperar",
    "recuperáramos": "recuperar",
    "recuperáremos": "recuperar",
    "recuperásemos": "recuperar",
    "recuperé": "recuperar",
    "recuperéis": "recuperar",
    "recuperó": "recuperar",
    "recuras": "recura",
    "recurra": "recurrir",
    "recurramos": "recurrir",
    "recurran": "recurrir",
    "recurras": "recurrir",
    "recurre": "recurrir",
    "recurren": "recurrir",
    "recurrencias": "recurrencia",
    "recurrentes": "recurrente",
    "recurres": "recurrir",
    "recurribilidades": "recurribilidad",
    "recurribles": "recurrible",
    "recurrid": "recurrir",
    "recurrida": "recurrir",
    "recurridas": "recurrir",
    "recurrido": "recurrir",
    "recurridos": "recurrir",
    "recurriendo": "recurrir",
    "recurriera": "recurrir",
    "recurrierais": "recurrir",
    "recurrieran": "recurrir",
    "recurrieras": "recurrir",
    "recurriere": "recurrir",
    "recurriereis": "recurrir",
    "recurrieren": "recurrir",
    "recurrieres": "recurrir",
    "recurrieron": "recurrir",
    "recurriese": "recurrir",
    "recurrieseis": "recurrir",
    "recurriesen": "recurrir",
    "recurrieses": "recurrir",
    "recurrimos": "recurrir",
    "recurriremos": "recurrir",
    "recurrirá": "recurrir",
    "recurrirán": "recurrir",
    "recurrirás": "recurrir",
    "recurriré": "recurrir",
    "recurriréis": "recurrir",
    "recurriría": "recurrir",
    "recurriríais": "recurrir",
    "recurriríamos": "recurrir",
    "recurrirían": "recurrir",
    "recurrirías": "recurrir",
    "recurriste": "recurrir",
    "recurristeis": "recurrir",
    "recurriéramos": "recurrir",
    "recurriéremos": "recurrir",
    "recurriésemos": "recurrir",
    "recurrió": "recurrir",
    "recurro": "recurrir",
    "recurráis": "recurrir",
    "recurrí": "recurrir",
    "recurría": "recurrir",
    "recurríais": "recurrir",
    "recurríamos": "recurrir",
    "recurrían": "recurrir",
    "recurrías": "recurrir",
    "recurrís": "recurrir",
    "recursiones": "recursión",
    "recursiva": "recursivo",
    "recursivas": "recursivo",
    "recursividades": "recursividad",
    "recursivos": "recursivo",
    "recursos": "recurso",
    "recurvada": "recurvado",
    "recurvadas": "recurvado",
    "recurvados": "recurvado",
    "recurvirróstridos": "recurvirróstrido",
    "recusa": "recusar",
    "recusaba": "recusar",
    "recusabais": "recusar",
    "recusaban": "recusar",
    "recusabas": "recusar",
    "recusables": "recusable",
    "recusaciones": "recusación",
    "recusad": "recusar",
    "recusada": "recusar",
    "recusadas": "recusar",
    "recusado": "recusar",
    "recusados": "recusar",
    "recusamos": "recusar",
    "recusan": "recusar",
    "recusando": "recusar",
    "recusantes": "recusante",
    "recusara": "recusar",
    "recusarais": "recusar",
    "recusaran": "recusar",
    "recusaras": "recusar",
    "recusare": "recusar",
    "recusareis": "recusar",
    "recusaremos": "recusar",
    "recusaren": "recusar",
    "recusares": "recusar",
    "recusaron": "recusar",
    "recusará": "recusar",
    "recusarán": "recusar",
    "recusarás": "recusar",
    "recusaré": "recusar",
    "recusaréis": "recusar",
    "recusaría": "recusar",
    "recusaríais": "recusar",
    "recusaríamos": "recusar",
    "recusarían": "recusar",
    "recusarías": "recusar",
    "recusas": "recusar",
    "recusase": "recusar",
    "recusaseis": "recusar",
    "recusasen": "recusar",
    "recusases": "recusar",
    "recusaste": "recusar",
    "recusasteis": "recusar",
    "recuse": "recusar",
    "recusemos": "recusar",
    "recusen": "recusar",
    "recuses": "recusar",
    "recuso": "recusar",
    "recusábamos": "recusar",
    "recusáis": "recusar",
    "recusáramos": "recusar",
    "recusáremos": "recusar",
    "recusásemos": "recusar",
    "recusé": "recusar",
    "recuséis": "recusar",
    "recusó": "recusar",
    "recuña": "recuñar",
    "recuñaba": "recuñar",
    "recuñabais": "recuñar",
    "recuñaban": "recuñar",
    "recuñabas": "recuñar",
    "recuñad": "recuñar",
    "recuñada": "recuñar",
    "recuñadas": "recuñar",
    "recuñado": "recuñar",
    "recuñados": "recuñar",
    "recuñamos": "recuñar",
    "recuñan": "recuñar",
    "recuñando": "recuñar",
    "recuñara": "recuñar",
    "recuñarais": "recuñar",
    "recuñaran": "recuñar",
    "recuñaras": "recuñar",
    "recuñare": "recuñar",
    "recuñareis": "recuñar",
    "recuñaremos": "recuñar",
    "recuñaren": "recuñar",
    "recuñares": "recuñar",
    "recuñaron": "recuñar",
    "recuñará": "recuñar",
    "recuñarán": "recuñar",
    "recuñarás": "recuñar",
    "recuñaré": "recuñar",
    "recuñaréis": "recuñar",
    "recuñaría": "recuñar",
    "recuñaríais": "recuñar",
    "recuñaríamos": "recuñar",
    "recuñarían": "recuñar",
    "recuñarías": "recuñar",
    "recuñas": "recuñar",
    "recuñase": "recuñar",
    "recuñaseis": "recuñar",
    "recuñasen": "recuñar",
    "recuñases": "recuñar",
    "recuñaste": "recuñar",
    "recuñasteis": "recuñar",
    "recuñe": "recuñar",
    "recuñemos": "recuñar",
    "recuñen": "recuñar",
    "recuñes": "recuñar",
    "recuño": "recuñar",
    "recuñábamos": "recuñar",
    "recuñáis": "recuñar",
    "recuñáramos": "recuñar",
    "recuñáremos": "recuñar",
    "recuñásemos": "recuñar",
    "recuñé": "recuñar",
    "recuñéis": "recuñar",
    "recuñó": "recuñar",
    "recálculos": "recálculo",
    "recámaras": "recámara",
    "recé": "rezar",
    "recéis": "rezar",
    "recíproca": "recíproco",
    "recíprocas": "recíproco",
    "recíprocos": "recíproco",
    "recóndita": "recóndito",
    "recónditas": "recóndito",
    "recónditos": "recóndito",
    "recórdmans": "recordman",
    "reda": "redar",
    "redaba": "redar",
    "redabais": "redar",
    "redaban": "redar",
    "redabas": "redar",
    "redaccionales": "redaccional",
    "redacciones": "redacción",
    "redacta": "redactar",
    "redactaba": "redactar",
    "redactabais": "redactar",
    "redactaban": "redactar",
    "redactabas": "redactar",
    "redactad": "redactar",
    "redactada": "redactar",
    "redactadas": "redactar",
    "redactado": "redactar",
    "redactados": "redactar",
    "redactamos": "redactar",
    "redactan": "redactar",
    "redactando": "redactar",
    "redactara": "redactar",
    "redactarais": "redactar",
    "redactaran": "redactar",
    "redactaras": "redactar",
    "redactare": "redactar",
    "redactareis": "redactar",
    "redactaremos": "redactar",
    "redactaren": "redactar",
    "redactares": "redactar",
    "redactaron": "redactar",
    "redactará": "redactar",
    "redactarán": "redactar",
    "redactarás": "redactar",
    "redactaré": "redactar",
    "redactaréis": "redactar",
    "redactaría": "redactar",
    "redactaríais": "redactar",
    "redactaríamos": "redactar",
    "redactarían": "redactar",
    "redactarías": "redactar",
    "redactas": "redactar",
    "redactase": "redactar",
    "redactaseis": "redactar",
    "redactasen": "redactar",
    "redactases": "redactar",
    "redactaste": "redactar",
    "redactasteis": "redactar",
    "redacte": "redactar",
    "redactemos": "redactar",
    "redacten": "redactar",
    "redactes": "redactar",
    "redacto": "redactar",
    "redactora": "redactor",
    "redactoras": "redactor",
    "redactores": "redactor",
    "redactábamos": "redactar",
    "redactáis": "redactar",
    "redactáramos": "redactar",
    "redactáremos": "redactar",
    "redactásemos": "redactar",
    "redacté": "redactar",
    "redactéis": "redactar",
    "redactó": "redactar",
    "redad": "redar",
    "redada": "redar",
    "redadas": "redar",
    "redado": "redar",
    "redados": "redar",
    "redamos": "redar",
    "redan": "redar",
    "redando": "redar",
    "redara": "redar",
    "redarais": "redar",
    "redaran": "redar",
    "redaras": "redar",
    "redare": "redar",
    "redareis": "redar",
    "redaremos": "redar",
    "redaren": "redar",
    "redares": "redar",
    "redarguya": "redargüir",
    "redarguyamos": "redargüir",
    "redarguyan": "redargüir",
    "redarguyas": "redargüir",
    "redarguye": "redargüir",
    "redarguyen": "redargüir",
    "redarguyendo": "redargüir",
    "redarguyera": "redargüir",
    "redarguyerais": "redargüir",
    "redarguyeran": "redargüir",
    "redarguyeras": "redargüir",
    "redarguyere": "redargüir",
    "redarguyereis": "redargüir",
    "redarguyeren": "redargüir",
    "redarguyeres": "redargüir",
    "redarguyeron": "redargüir",
    "redarguyes": "redargüir",
    "redarguyese": "redargüir",
    "redarguyeseis": "redargüir",
    "redarguyesen": "redargüir",
    "redarguyeses": "redargüir",
    "redarguyo": "redargüir",
    "redarguyáis": "redargüir",
    "redarguyéramos": "redargüir",
    "redarguyéremos": "redargüir",
    "redarguyésemos": "redargüir",
    "redarguyó": "redargüir",
    "redargüid": "redargüir",
    "redargüida": "redargüir",
    "redargüidas": "redargüir",
    "redargüido": "redargüir",
    "redargüidos": "redargüir",
    "redargüimos": "redargüir",
    "redargüiremos": "redargüir",
    "redargüirá": "redargüir",
    "redargüirán": "redargüir",
    "redargüirás": "redargüir",
    "redargüiré": "redargüir",
    "redargüiréis": "redargüir",
    "redargüiría": "redargüir",
    "redargüiríais": "redargüir",
    "redargüiríamos": "redargüir",
    "redargüirían": "redargüir",
    "redargüirías": "redargüir",
    "redargüiste": "redargüir",
    "redargüisteis": "redargüir",
    "redargüí": "redargüir",
    "redargüía": "redargüir",
    "redargüíais": "redargüir",
    "redargüíamos": "redargüir",
    "redargüían": "redargüir",
    "redargüías": "redargüir",
    "redargüís": "redargüir",
    "redaron": "redar",
    "redará": "redar",
    "redarán": "redar",
    "redarás": "redar",
    "redaré": "redar",
    "redaréis": "redar",
    "redaría": "redar",
    "redaríais": "redar",
    "redaríamos": "redar",
    "redarían": "redar",
    "redarías": "redar",
    "redas": "redar",
    "redase": "redar",
    "redaseis": "redar",
    "redasen": "redar",
    "redases": "redar",
    "redaste": "redar",
    "redasteis": "redar",
    "redayas": "redaya",
    "redaños": "redaño",
    "rede": "redar",
    "redecid": "redecir",
    "redecillas": "redecilla",
    "redecimos": "redecir",
    "redeciremos": "redecir",
    "redecirá": "redecir",
    "redecirán": "redecir",
    "redecirás": "redecir",
    "redeciré": "redecir",
    "redeciréis": "redecir",
    "redeciría": "redecir",
    "redeciríais": "redecir",
    "redeciríamos": "redecir",
    "redecirían": "redecir",
    "redecirías": "redecir",
    "redeclaraciones": "redeclaración",
    "redecoraciones": "redecoración",
    "redecía": "redecir",
    "redecíais": "redecir",
    "redecíamos": "redecir",
    "redecían": "redecir",
    "redecías": "redecir",
    "redecís": "redecir",
    "rededores": "rededor",
    "redefina": "redefinir",
    "redefinamos": "redefinir",
    "redefinan": "redefinir",
    "redefinas": "redefinir",
    "redefine": "redefinir",
    "redefinen": "redefinir",
    "redefines": "redefinir",
    "redefiniciones": "redefinición",
    "redefinid": "redefinir",
    "redefinida": "redefinir",
    "redefinidas": "redefinir",
    "redefinido": "redefinir",
    "redefinidos": "redefinir",
    "redefiniendo": "redefinir",
    "redefiniera": "redefinir",
    "redefinierais": "redefinir",
    "redefinieran": "redefinir",
    "redefinieras": "redefinir",
    "redefiniere": "redefinir",
    "redefiniereis": "redefinir",
    "redefinieren": "redefinir",
    "redefinieres": "redefinir",
    "redefinieron": "redefinir",
    "redefiniese": "redefinir",
    "redefinieseis": "redefinir",
    "redefiniesen": "redefinir",
    "redefinieses": "redefinir",
    "redefinimos": "redefinir",
    "redefiniremos": "redefinir",
    "redefinirá": "redefinir",
    "redefinirán": "redefinir",
    "redefinirás": "redefinir",
    "redefiniré": "redefinir",
    "redefiniréis": "redefinir",
    "redefiniría": "redefinir",
    "redefiniríais": "redefinir",
    "redefiniríamos": "redefinir",
    "redefinirían": "redefinir",
    "redefinirías": "redefinir",
    "redefiniste": "redefinir",
    "redefinisteis": "redefinir",
    "redefiniéramos": "redefinir",
    "redefiniéremos": "redefinir",
    "redefiniésemos": "redefinir",
    "redefinió": "redefinir",
    "redefino": "redefinir",
    "redefináis": "redefinir",
    "redefiní": "redefinir",
    "redefinía": "redefinir",
    "redefiníais": "redefinir",
    "redefiníamos": "redefinir",
    "redefinían": "redefinir",
    "redefinías": "redefinir",
    "redefinís": "redefinir",
    "redejones": "redejón",
    "redeles": "redel",
    "redemos": "redar",
    "reden": "redar",
    "redenciones": "redención",
    "redentora": "redentor",
    "redentoras": "redentor",
    "redentores": "redentor",
    "redentoristas": "redentorista",
    "redera": "redero",
    "rederas": "redero",
    "rederos": "redero",
    "redes": "redar",
    "redescubierta": "redescubrir",
    "redescubiertas": "redescubrir",
    "redescubierto": "redescubrir",
    "redescubiertos": "redescubrir",
    "redescubra": "redescubrir",
    "redescubramos": "redescubrir",
    "redescubran": "redescubrir",
    "redescubras": "redescubrir",
    "redescubre": "redescubrir",
    "redescubren": "redescubrir",
    "redescubres": "redescubrir",
    "redescubrid": "redescubrir",
    "redescubriendo": "redescubrir",
    "redescubriera": "redescubrir",
    "redescubrierais": "redescubrir",
    "redescubrieran": "redescubrir",
    "redescubrieras": "redescubrir",
    "redescubriere": "redescubrir",
    "redescubriereis": "redescubrir",
    "redescubrieren": "redescubrir",
    "redescubrieres": "redescubrir",
    "redescubrieron": "redescubrir",
    "redescubriese": "redescubrir",
    "redescubrieseis": "redescubrir",
    "redescubriesen": "redescubrir",
    "redescubrieses": "redescubrir",
    "redescubrimientos": "redescubrimiento",
    "redescubrimos": "redescubrir",
    "redescubriremos": "redescubrir",
    "redescubrirá": "redescubrir",
    "redescubrirán": "redescubrir",
    "redescubrirás": "redescubrir",
    "redescubriré": "redescubrir",
    "redescubriréis": "redescubrir",
    "redescubriría": "redescubrir",
    "redescubriríais": "redescubrir",
    "redescubriríamos": "redescubrir",
    "redescubrirían": "redescubrir",
    "redescubrirías": "redescubrir",
    "redescubriste": "redescubrir",
    "redescubristeis": "redescubrir",
    "redescubriéramos": "redescubrir",
    "redescubriéremos": "redescubrir",
    "redescubriésemos": "redescubrir",
    "redescubrió": "redescubrir",
    "redescubro": "redescubrir",
    "redescubráis": "redescubrir",
    "redescubrí": "redescubrir",
    "redescubría": "redescubrir",
    "redescubríais": "redescubrir",
    "redescubríamos": "redescubrir",
    "redescubrían": "redescubrir",
    "redescubrías": "redescubrir",
    "redescubrís": "redescubrir",
    "redescuentos": "redescuento",
    "redhiba": "redhibir",
    "redhibamos": "redhibir",
    "redhiban": "redhibir",
    "redhibas": "redhibir",
    "redhibe": "redhibir",
    "redhiben": "redhibir",
    "redhibes": "redhibir",
    "redhibiciones": "redhibición",
    "redhibid": "redhibir",
    "redhibida": "redhibir",
    "redhibidas": "redhibir",
    "redhibido": "redhibir",
    "redhibidos": "redhibir",
    "redhibiendo": "redhibir",
    "redhibiera": "redhibir",
    "redhibierais": "redhibir",
    "redhibieran": "redhibir",
    "redhibieras": "redhibir",
    "redhibiere": "redhibir",
    "redhibiereis": "redhibir",
    "redhibieren": "redhibir",
    "redhibieres": "redhibir",
    "redhibieron": "redhibir",
    "redhibiese": "redhibir",
    "redhibieseis": "redhibir",
    "redhibiesen": "redhibir",
    "redhibieses": "redhibir",
    "redhibimos": "redhibir",
    "redhibiremos": "redhibir",
    "redhibirá": "redhibir",
    "redhibirán": "redhibir",
    "redhibirás": "redhibir",
    "redhibiré": "redhibir",
    "redhibiréis": "redhibir",
    "redhibiría": "redhibir",
    "redhibiríais": "redhibir",
    "redhibiríamos": "redhibir",
    "redhibirían": "redhibir",
    "redhibirías": "redhibir",
    "redhibiste": "redhibir",
    "redhibisteis": "redhibir",
    "redhibitoria": "redhibitorio",
    "redhibitorias": "redhibitorio",
    "redhibitorios": "redhibitorio",
    "redhibiéramos": "redhibir",
    "redhibiéremos": "redhibir",
    "redhibiésemos": "redhibir",
    "redhibió": "redhibir",
    "redhibo": "redhibir",
    "redhibáis": "redhibir",
    "redhibí": "redhibir",
    "redhibía": "redhibir",
    "redhibíais": "redhibir",
    "redhibíamos": "redhibir",
    "redhibían": "redhibir",
    "redhibías": "redhibir",
    "redhibís": "redhibir",
    "redice": "redecir",
    "redicen": "redecir",
    "redices": "redecir",
    "redicha": "redecir",
    "redichas": "redecir",
    "redicho": "redecir",
    "redichos": "redecir",
    "rediciendo": "redecir",
    "rediciones": "redición",
    "redientes": "rediente",
    "rediezma": "rediezmar",
    "rediezmaba": "rediezmar",
    "rediezmabais": "rediezmar",
    "rediezmaban": "rediezmar",
    "rediezmabas": "rediezmar",
    "rediezmad": "rediezmar",
    "rediezmada": "rediezmar",
    "rediezmadas": "rediezmar",
    "rediezmado": "rediezmar",
    "rediezmados": "rediezmar",
    "rediezmamos": "rediezmar",
    "rediezman": "rediezmar",
    "rediezmando": "rediezmar",
    "rediezmara": "rediezmar",
    "rediezmarais": "rediezmar",
    "rediezmaran": "rediezmar",
    "rediezmaras": "rediezmar",
    "rediezmare": "rediezmar",
    "rediezmareis": "rediezmar",
    "rediezmaremos": "rediezmar",
    "rediezmaren": "rediezmar",
    "rediezmares": "rediezmar",
    "rediezmaron": "rediezmar",
    "rediezmará": "rediezmar",
    "rediezmarán": "rediezmar",
    "rediezmarás": "rediezmar",
    "rediezmaré": "rediezmar",
    "rediezmaréis": "rediezmar",
    "rediezmaría": "rediezmar",
    "rediezmaríais": "rediezmar",
    "rediezmaríamos": "rediezmar",
    "rediezmarían": "rediezmar",
    "rediezmarías": "rediezmar",
    "rediezmas": "rediezmar",
    "rediezmase": "rediezmar",
    "rediezmaseis": "rediezmar",
    "rediezmasen": "rediezmar",
    "rediezmases": "rediezmar",
    "rediezmaste": "rediezmar",
    "rediezmasteis": "rediezmar",
    "rediezme": "rediezmar",
    "rediezmemos": "rediezmar",
    "rediezmen": "rediezmar",
    "rediezmes": "rediezmar",
    "rediezmo": "rediezmar",
    "rediezmábamos": "rediezmar",
    "rediezmáis": "rediezmar",
    "rediezmáramos": "rediezmar",
    "rediezmáremos": "rediezmar",
    "rediezmásemos": "rediezmar",
    "rediezmé": "rediezmar",
    "rediezméis": "rediezmar",
    "rediezmó": "rediezmar",
    "redifusiones": "redifusión",
    "rediga": "redecir",
    "redigamos": "redecir",
    "redigan": "redecir",
    "redigas": "redecir",
    "redigo": "redecir",
    "redigáis": "redecir",
    "redije": "redecir",
    "redijera": "redecir",
    "redijerais": "redecir",
    "redijeran": "redecir",
    "redijeras": "redecir",
    "redijere": "redecir",
    "redijereis": "redecir",
    "redijeren": "redecir",
    "redijeres": "redecir",
    "redijeron": "redecir",
    "redijese": "redecir",
    "redijeseis": "redecir",
    "redijesen": "redecir",
    "redijeses": "redecir",
    "redijimos": "redecir",
    "redijiste": "redecir",
    "redijisteis": "redecir",
    "redijo": "redecir",
    "redijéramos": "redecir",
    "redijéremos": "redecir",
    "redijésemos": "redecir",
    "redila": "redilar",
    "redilaba": "redilar",
    "redilabais": "redilar",
    "redilaban": "redilar",
    "redilabas": "redilar",
    "redilad": "redilar",
    "redilada": "redilar",
    "rediladas": "redilar",
    "redilado": "redilar",
    "redilados": "redilar",
    "redilamos": "redilar",
    "redilan": "redilar",
    "redilando": "redilar",
    "redilara": "redilar",
    "redilarais": "redilar",
    "redilaran": "redilar",
    "redilaras": "redilar",
    "redilare": "redilar",
    "redilareis": "redilar",
    "redilaremos": "redilar",
    "redilaren": "redilar",
    "redilares": "redilar",
    "redilaron": "redilar",
    "redilará": "redilar",
    "redilarán": "redilar",
    "redilarás": "redilar",
    "redilaré": "redilar",
    "redilaréis": "redilar",
    "redilaría": "redilar",
    "redilaríais": "redilar",
    "redilaríamos": "redilar",
    "redilarían": "redilar",
    "redilarías": "redilar",
    "redilas": "redilar",
    "redilase": "redilar",
    "redilaseis": "redilar",
    "redilasen": "redilar",
    "redilases": "redilar",
    "redilaste": "redilar",
    "redilasteis": "redilar",
    "redile": "redilar",
    "redilea": "redilear",
    "redileaba": "redilear",
    "redileabais": "redilear",
    "redileaban": "redilear",
    "redileabas": "redilear",
    "redilead": "redilear",
    "redileada": "redilear",
    "redileadas": "redilear",
    "redileado": "redilear",
    "redileados": "redilear",
    "redileamos": "redilear",
    "redilean": "redilear",
    "redileando": "redilear",
    "redileara": "redilear",
    "redilearais": "redilear",
    "redilearan": "redilear",
    "redilearas": "redilear",
    "redileare": "redilear",
    "redileareis": "redilear",
    "redilearemos": "redilear",
    "redilearen": "redilear",
    "redileares": "redilear",
    "redilearon": "redilear",
    "redileará": "redilear",
    "redilearán": "redilear",
    "redilearás": "redilear",
    "redilearé": "redilear",
    "redilearéis": "redilear",
    "redilearía": "redilear",
    "redilearíais": "redilear",
    "redilearíamos": "redilear",
    "redilearían": "redilear",
    "redilearías": "redilear",
    "redileas": "redilear",
    "redilease": "redilear",
    "redileaseis": "redilear",
    "redileasen": "redilear",
    "redileases": "redilear",
    "redileaste": "redilear",
    "redileasteis": "redilear",
    "redilee": "redilear",
    "redileemos": "redilear",
    "redileen": "redilear",
    "redilees": "redilear",
    "redilemos": "redilar",
    "redilen": "redilar",
    "redileo": "redilear",
    "redileos": "redileo",
    "rediles": "redilar",
    "redileábamos": "redilear",
    "redileáis": "redilear",
    "redileáramos": "redilear",
    "redileáremos": "redilear",
    "redileásemos": "redilear",
    "redileé": "redilear",
    "redileéis": "redilear",
    "redileó": "redilear",
    "redilo": "redilar",
    "redilábamos": "redilar",
    "rediláis": "redilar",
    "rediláramos": "redilar",
    "rediláremos": "redilar",
    "redilásemos": "redilar",
    "redilé": "redilar",
    "rediléis": "redilar",
    "rediló": "redilar",
    "redima": "redimir",
    "redimamos": "redimir",
    "rediman": "redimir",
    "redimas": "redimir",
    "redime": "redimir",
    "redimen": "redimir",
    "redimensionamientos": "redimensionamiento",
    "redimes": "redimir",
    "redimibles": "redimible",
    "redimid": "redimir",
    "redimida": "redimir",
    "redimidas": "redimir",
    "redimido": "redimir",
    "redimidos": "redimir",
    "redimiendo": "redimir",
    "redimiera": "redimir",
    "redimierais": "redimir",
    "redimieran": "redimir",
    "redimieras": "redimir",
    "redimiere": "redimir",
    "redimiereis": "redimir",
    "redimieren": "redimir",
    "redimieres": "redimir",
    "redimieron": "redimir",
    "redimiese": "redimir",
    "redimieseis": "redimir",
    "redimiesen": "redimir",
    "redimieses": "redimir",
    "redimimos": "redimir",
    "redimiremos": "redimir",
    "redimirá": "redimir",
    "redimirán": "redimir",
    "redimirás": "redimir",
    "redimiré": "redimir",
    "redimiréis": "redimir",
    "redimiría": "redimir",
    "redimiríais": "redimir",
    "redimiríamos": "redimir",
    "redimirían": "redimir",
    "redimirías": "redimir",
    "redimiste": "redimir",
    "redimisteis": "redimir",
    "redimiéramos": "redimir",
    "redimiéremos": "redimir",
    "redimiésemos": "redimir",
    "redimió": "redimir",
    "redimo": "redimir",
    "redimáis": "redimir",
    "redimí": "redimir",
    "redimía": "redimir",
    "redimíais": "redimir",
    "redimíamos": "redimir",
    "redimían": "redimir",
    "redimías": "redimir",
    "redimís": "redimir",
    "redingotes": "redingote",
    "redireccionamientos": "redireccionamiento",
    "rediseña": "rediseñar",
    "rediseñaba": "rediseñar",
    "rediseñabais": "rediseñar",
    "rediseñaban": "rediseñar",
    "rediseñabas": "rediseñar",
    "rediseñad": "rediseñar",
    "rediseñada": "rediseñar",
    "rediseñadas": "rediseñar",
    "rediseñado": "rediseñar",
    "rediseñados": "rediseñar",
    "rediseñamos": "rediseñar",
    "rediseñan": "rediseñar",
    "rediseñando": "rediseñar",
    "rediseñara": "rediseñar",
    "rediseñarais": "rediseñar",
    "rediseñaran": "rediseñar",
    "rediseñaras": "rediseñar",
    "rediseñare": "rediseñar",
    "rediseñareis": "rediseñar",
    "rediseñaremos": "rediseñar",
    "rediseñaren": "rediseñar",
    "rediseñares": "rediseñar",
    "rediseñaron": "rediseñar",
    "rediseñará": "rediseñar",
    "rediseñarán": "rediseñar",
    "rediseñarás": "rediseñar",
    "rediseñaré": "rediseñar",
    "rediseñaréis": "rediseñar",
    "rediseñaría": "rediseñar",
    "rediseñaríais": "rediseñar",
    "rediseñaríamos": "rediseñar",
    "rediseñarían": "rediseñar",
    "rediseñarías": "rediseñar",
    "rediseñas": "rediseñar",
    "rediseñase": "rediseñar",
    "rediseñaseis": "rediseñar",
    "rediseñasen": "rediseñar",
    "rediseñases": "rediseñar",
    "rediseñaste": "rediseñar",
    "rediseñasteis": "rediseñar",
    "rediseñe": "rediseñar",
    "rediseñemos": "rediseñar",
    "rediseñen": "rediseñar",
    "rediseñes": "rediseñar",
    "rediseño": "rediseñar",
    "rediseñábamos": "rediseñar",
    "rediseñáis": "rediseñar",
    "rediseñáramos": "rediseñar",
    "rediseñáremos": "rediseñar",
    "rediseñásemos": "rediseñar",
    "rediseñé": "rediseñar",
    "rediseñéis": "rediseñar",
    "rediseñó": "rediseñar",
    "redisoluciones": "redisolución",
    "redisposiciones": "redisposición",
    "redistribuciones": "redistribución",
    "redistribuid": "redistribuir",
    "redistribuida": "redistribuir",
    "redistribuidas": "redistribuir",
    "redistribuido": "redistribuir",
    "redistribuidos": "redistribuir",
    "redistribuimos": "redistribuir",
    "redistribuiremos": "redistribuir",
    "redistribuirá": "redistribuir",
    "redistribuirán": "redistribuir",
    "redistribuirás": "redistribuir",
    "redistribuiré": "redistribuir",
    "redistribuiréis": "redistribuir",
    "redistribuiría": "redistribuir",
    "redistribuiríais": "redistribuir",
    "redistribuiríamos": "redistribuir",
    "redistribuirían": "redistribuir",
    "redistribuirías": "redistribuir",
    "redistribuiste": "redistribuir",
    "redistribuisteis": "redistribuir",
    "redistributiva": "redistributivo",
    "redistributivas": "redistributivo",
    "redistributivos": "redistributivo",
    "redistribuya": "redistribuir",
    "redistribuyamos": "redistribuir",
    "redistribuyan": "redistribuir",
    "redistribuyas": "redistribuir",
    "redistribuye": "redistribuir",
    "redistribuyen": "redistribuir",
    "redistribuyendo": "redistribuir",
    "redistribuyera": "redistribuir",
    "redistribuyerais": "redistribuir",
    "redistribuyeran": "redistribuir",
    "redistribuyeras": "redistribuir",
    "redistribuyere": "redistribuir",
    "redistribuyereis": "redistribuir",
    "redistribuyeren": "redistribuir",
    "redistribuyeres": "redistribuir",
    "redistribuyeron": "redistribuir",
    "redistribuyes": "redistribuir",
    "redistribuyese": "redistribuir",
    "redistribuyeseis": "redistribuir",
    "redistribuyesen": "redistribuir",
    "redistribuyeses": "redistribuir",
    "redistribuyo": "redistribuir",
    "redistribuyáis": "redistribuir",
    "redistribuyéramos": "redistribuir",
    "redistribuyéremos": "redistribuir",
    "redistribuyésemos": "redistribuir",
    "redistribuyó": "redistribuir",
    "redistribuí": "redistribuir",
    "redistribuía": "redistribuir",
    "redistribuíais": "redistribuir",
    "redistribuíamos": "redistribuir",
    "redistribuían": "redistribuir",
    "redistribuías": "redistribuir",
    "redistribuís": "redistribuir",
    "redituaba": "redituar",
    "redituabais": "redituar",
    "redituaban": "redituar",
    "redituabas": "redituar",
    "redituables": "redituable",
    "redituad": "redituar",
    "redituada": "redituar",
    "redituadas": "redituar",
    "redituado": "redituar",
    "redituados": "redituar",
    "redituales": "reditual",
    "redituamos": "redituar",
    "redituando": "redituar",
    "redituara": "redituar",
    "redituarais": "redituar",
    "redituaran": "redituar",
    "redituaras": "redituar",
    "redituare": "redituar",
    "redituareis": "redituar",
    "redituaremos": "redituar",
    "redituaren": "redituar",
    "redituares": "redituar",
    "redituaron": "redituar",
    "redituará": "redituar",
    "redituarán": "redituar",
    "redituarás": "redituar",
    "redituaré": "redituar",
    "redituaréis": "redituar",
    "redituaría": "redituar",
    "redituaríais": "redituar",
    "redituaríamos": "redituar",
    "redituarían": "redituar",
    "redituarías": "redituar",
    "redituase": "redituar",
    "redituaseis": "redituar",
    "redituasen": "redituar",
    "redituases": "redituar",
    "redituaste": "redituar",
    "redituasteis": "redituar",
    "redituemos": "redituar",
    "redituábamos": "redituar",
    "redituáis": "redituar",
    "redituáramos": "redituar",
    "redituáremos": "redituar",
    "redituásemos": "redituar",
    "reditué": "redituar",
    "redituéis": "redituar",
    "redituó": "redituar",
    "reditúa": "redituar",
    "reditúan": "redituar",
    "reditúas": "redituar",
    "reditúe": "redituar",
    "reditúen": "redituar",
    "reditúes": "redituar",
    "reditúo": "redituar",
    "rediviva": "redivivo",
    "redivivas": "redivivo",
    "redivivos": "redivivo",
    "redo": "redar",
    "redobla": "redoblar",
    "redoblaba": "redoblar",
    "redoblabais": "redoblar",
    "redoblaban": "redoblar",
    "redoblabas": "redoblar",
    "redoblad": "redoblar",
    "redoblada": "redoblar",
    "redobladas": "redoblar",
    "redoblado": "redoblar",
    "redoblados": "redoblar",
    "redobladuras": "redobladura",
    "redoblamientos": "redoblamiento",
    "redoblamos": "redoblar",
    "redoblan": "redoblar",
    "redoblando": "redoblar",
    "redoblantes": "redoblante",
    "redoblara": "redoblar",
    "redoblarais": "redoblar",
    "redoblaran": "redoblar",
    "redoblaras": "redoblar",
    "redoblare": "redoblar",
    "redoblareis": "redoblar",
    "redoblaremos": "redoblar",
    "redoblaren": "redoblar",
    "redoblares": "redoblar",
    "redoblaron": "redoblar",
    "redoblará": "redoblar",
    "redoblarán": "redoblar",
    "redoblarás": "redoblar",
    "redoblaré": "redoblar",
    "redoblaréis": "redoblar",
    "redoblaría": "redoblar",
    "redoblaríais": "redoblar",
    "redoblaríamos": "redoblar",
    "redoblarían": "redoblar",
    "redoblarías": "redoblar",
    "redoblas": "redoblar",
    "redoblase": "redoblar",
    "redoblaseis": "redoblar",
    "redoblasen": "redoblar",
    "redoblases": "redoblar",
    "redoblaste": "redoblar",
    "redoblasteis": "redoblar",
    "redoble": "redoblar",
    "redoblemos": "redoblar",
    "redoblen": "redoblar",
    "redobles": "redoble",
    "redoblo": "redoblar",
    "redoblones": "redoblón",
    "redoblábamos": "redoblar",
    "redobláis": "redoblar",
    "redobláramos": "redoblar",
    "redobláremos": "redoblar",
    "redoblásemos": "redoblar",
    "redoblé": "redoblar",
    "redobléis": "redoblar",
    "redobló": "redoblar",
    "redoladas": "redolada",
    "redolas": "redola",
    "redoles": "redol",
    "redolores": "redolor",
    "redomada": "redomado",
    "redomadas": "redomado",
    "redomados": "redomado",
    "redomas": "redoma",
    "redonda": "redondo",
    "redondas": "redondo",
    "redondea": "redondear",
    "redondeaba": "redondear",
    "redondeabais": "redondear",
    "redondeaban": "redondear",
    "redondeabas": "redondear",
    "redondead": "redondear",
    "redondeada": "redondear",
    "redondeadas": "redondear",
    "redondeado": "redondear",
    "redondeados": "redondear",
    "redondeamos": "redondear",
    "redondean": "redondear",
    "redondeando": "redondear",
    "redondeara": "redondear",
    "redondearais": "redondear",
    "redondearan": "redondear",
    "redondearas": "redondear",
    "redondeare": "redondear",
    "redondeareis": "redondear",
    "redondearemos": "redondear",
    "redondearen": "redondear",
    "redondeares": "redondear",
    "redondearon": "redondear",
    "redondeará": "redondear",
    "redondearán": "redondear",
    "redondearás": "redondear",
    "redondearé": "redondear",
    "redondearéis": "redondear",
    "redondearía": "redondear",
    "redondearíais": "redondear",
    "redondearíamos": "redondear",
    "redondearían": "redondear",
    "redondearías": "redondear",
    "redondeas": "redondear",
    "redondease": "redondear",
    "redondeaseis": "redondear",
    "redondeasen": "redondear",
    "redondeases": "redondear",
    "redondeaste": "redondear",
    "redondeasteis": "redondear",
    "redondeces": "redondez",
    "redondee": "redondear",
    "redondeemos": "redondear",
    "redondeen": "redondear",
    "redondees": "redondear",
    "redondeles": "redondel",
    "redondeo": "redondear",
    "redondeos": "redondeo",
    "redondeábamos": "redondear",
    "redondeáis": "redondear",
    "redondeáramos": "redondear",
    "redondeáremos": "redondear",
    "redondeásemos": "redondear",
    "redondeé": "redondear",
    "redondeéis": "redondear",
    "redondeó": "redondear",
    "redondos": "redondo",
    "redopelos": "redopelo",
    "redora": "redorar",
    "redoraba": "redorar",
    "redorabais": "redorar",
    "redoraban": "redorar",
    "redorabas": "redorar",
    "redorad": "redorar",
    "redorada": "redorar",
    "redoradas": "redorar",
    "redorado": "redorar",
    "redorados": "redorar",
    "redoramos": "redorar",
    "redoran": "redorar",
    "redorando": "redorar",
    "redorara": "redorar",
    "redorarais": "redorar",
    "redoraran": "redorar",
    "redoraras": "redorar",
    "redorare": "redorar",
    "redorareis": "redorar",
    "redoraremos": "redorar",
    "redoraren": "redorar",
    "redorares": "redorar",
    "redoraron": "redorar",
    "redorará": "redorar",
    "redorarán": "redorar",
    "redorarás": "redorar",
    "redoraré": "redorar",
    "redoraréis": "redorar",
    "redoraría": "redorar",
    "redoraríais": "redorar",
    "redoraríamos": "redorar",
    "redorarían": "redorar",
    "redorarías": "redorar",
    "redoras": "redorar",
    "redorase": "redorar",
    "redoraseis": "redorar",
    "redorasen": "redorar",
    "redorases": "redorar",
    "redoraste": "redorar",
    "redorasteis": "redorar",
    "redore": "redorar",
    "redoremos": "redorar",
    "redoren": "redorar",
    "redores": "redorar",
    "redoro": "redorar",
    "redorábamos": "redorar",
    "redoráis": "redorar",
    "redoráramos": "redorar",
    "redoráremos": "redorar",
    "redorásemos": "redorar",
    "redoré": "redorar",
    "redoréis": "redorar",
    "redoró": "redorar",
    "redovas": "redova",
    "redra": "redrar",
    "redraba": "redrar",
    "redrabais": "redrar",
    "redraban": "redrar",
    "redrabas": "redrar",
    "redrad": "redrar",
    "redrada": "redrar",
    "redradas": "redrar",
    "redrado": "redrar",
    "redrados": "redrar",
    "redramos": "redrar",
    "redran": "redrar",
    "redrando": "redrar",
    "redrara": "redrar",
    "redrarais": "redrar",
    "redraran": "redrar",
    "redraras": "redrar",
    "redrare": "redrar",
    "redrareis": "redrar",
    "redraremos": "redrar",
    "redraren": "redrar",
    "redrares": "redrar",
    "redraron": "redrar",
    "redrará": "redrar",
    "redrarán": "redrar",
    "redrarás": "redrar",
    "redraré": "redrar",
    "redraréis": "redrar",
    "redraría": "redrar",
    "redraríais": "redrar",
    "redraríamos": "redrar",
    "redrarían": "redrar",
    "redrarías": "redrar",
    "redras": "redrar",
    "redrase": "redrar",
    "redraseis": "redrar",
    "redrasen": "redrar",
    "redrases": "redrar",
    "redraste": "redrar",
    "redrasteis": "redrar",
    "redre": "redrar",
    "redremos": "redrar",
    "redren": "redrar",
    "redres": "redrar",
    "redro": "redrar",
    "redrojos": "redrojo",
    "redropelos": "redropelo",
    "redrovientos": "redroviento",
    "redrutitas": "redrutita",
    "redrábamos": "redrar",
    "redráis": "redrar",
    "redráramos": "redrar",
    "redráremos": "redrar",
    "redrásemos": "redrar",
    "redré": "redrar",
    "redréis": "redrar",
    "redró": "redrar",
    "reducciones": "reducción",
    "reduccionistas": "reduccionista",
    "reduce": "reducir",
    "reducen": "reducir",
    "reduces": "reducir",
    "reducibilidades": "reducibilidad",
    "reducibles": "reducible",
    "reducid": "reducir",
    "reducida": "reducir",
    "reducidas": "reducir",
    "reducido": "reducir",
    "reducidos": "reducir",
    "reduciendo": "reducir",
    "reducimos": "reducir",
    "reduciremos": "reducir",
    "reducirá": "reducir",
    "reducirán": "reducir",
    "reducirás": "reducir",
    "reduciré": "reducir",
    "reduciréis": "reducir",
    "reduciría": "reducir",
    "reduciríais": "reducir",
    "reduciríamos": "reducir",
    "reducirían": "reducir",
    "reducirías": "reducir",
    "reductasas": "reductasa",
    "reductibilidades": "reductibilidad",
    "reductibles": "reductible",
    "reductiva": "reductivo",
    "reductivas": "reductivo",
    "reductivos": "reductivo",
    "reductora": "reductor",
    "reductoras": "reductor",
    "reductores": "reductor",
    "reductos": "reducto",
    "reducía": "reducir",
    "reducíais": "reducir",
    "reducíamos": "reducir",
    "reducían": "reducir",
    "reducías": "reducir",
    "reducís": "reducir",
    "reduje": "reducir",
    "redujera": "reducir",
    "redujerais": "reducir",
    "redujeran": "reducir",
    "redujeras": "reducir",
    "redujere": "reducir",
    "redujereis": "reducir",
    "redujeren": "reducir",
    "redujeres": "reducir",
    "redujeron": "reducir",
    "redujese": "reducir",
    "redujeseis": "reducir",
    "redujesen": "reducir",
    "redujeses": "reducir",
    "redujimos": "reducir",
    "redujiste": "reducir",
    "redujisteis": "reducir",
    "redujo": "reducir",
    "redujéramos": "reducir",
    "redujéremos": "reducir",
    "redujésemos": "reducir",
    "redunda": "redundar",
    "redundaba": "redundar",
    "redundabais": "redundar",
    "redundaban": "redundar",
    "redundabas": "redundar",
    "redundad": "redundar",
    "redundada": "redundar",
    "redundadas": "redundar",
    "redundado": "redundar",
    "redundados": "redundar",
    "redundamos": "redundar",
    "redundan": "redundar",
    "redundancias": "redundancia",
    "redundando": "redundar",
    "redundantes": "redundante",
    "redundara": "redundar",
    "redundarais": "redundar",
    "redundaran": "redundar",
    "redundaras": "redundar",
    "redundare": "redundar",
    "redundareis": "redundar",
    "redundaremos": "redundar",
    "redundaren": "redundar",
    "redundares": "redundar",
    "redundaron": "redundar",
    "redundará": "redundar",
    "redundarán": "redundar",
    "redundarás": "redundar",
    "redundaré": "redundar",
    "redundaréis": "redundar",
    "redundaría": "redundar",
    "redundaríais": "redundar",
    "redundaríamos": "redundar",
    "redundarían": "redundar",
    "redundarías": "redundar",
    "redundas": "redundar",
    "redundase": "redundar",
    "redundaseis": "redundar",
    "redundasen": "redundar",
    "redundases": "redundar",
    "redundaste": "redundar",
    "redundasteis": "redundar",
    "redunde": "redundar",
    "redundemos": "redundar",
    "redunden": "redundar",
    "redundes": "redundar",
    "redundo": "redundar",
    "redundábamos": "redundar",
    "redundáis": "redundar",
    "redundáramos": "redundar",
    "redundáremos": "redundar",
    "redundásemos": "redundar",
    "redundé": "redundar",
    "redundéis": "redundar",
    "redundó": "redundar",
    "reduplica": "reduplicar",
    "reduplicaba": "reduplicar",
    "reduplicabais": "reduplicar",
    "reduplicaban": "reduplicar",
    "reduplicabas": "reduplicar",
    "reduplicaciones": "reduplicación",
    "reduplicad": "reduplicar",
    "reduplicada": "reduplicar",
    "reduplicadas": "reduplicar",
    "reduplicado": "reduplicar",
    "reduplicados": "reduplicar",
    "reduplicamos": "reduplicar",
    "reduplican": "reduplicar",
    "reduplicando": "reduplicar",
    "reduplicara": "reduplicar",
    "reduplicarais": "reduplicar",
    "reduplicaran": "reduplicar",
    "reduplicaras": "reduplicar",
    "reduplicare": "reduplicar",
    "reduplicareis": "reduplicar",
    "reduplicaremos": "reduplicar",
    "reduplicaren": "reduplicar",
    "reduplicares": "reduplicar",
    "reduplicaron": "reduplicar",
    "reduplicará": "reduplicar",
    "reduplicarán": "reduplicar",
    "reduplicarás": "reduplicar",
    "reduplicaré": "reduplicar",
    "reduplicaréis": "reduplicar",
    "reduplicaría": "reduplicar",
    "reduplicaríais": "reduplicar",
    "reduplicaríamos": "reduplicar",
    "reduplicarían": "reduplicar",
    "reduplicarías": "reduplicar",
    "reduplicas": "reduplicar",
    "reduplicase": "reduplicar",
    "reduplicaseis": "reduplicar",
    "reduplicasen": "reduplicar",
    "reduplicases": "reduplicar",
    "reduplicaste": "reduplicar",
    "reduplicasteis": "reduplicar",
    "reduplicativa": "reduplicativo",
    "reduplicativas": "reduplicativo",
    "reduplicativos": "reduplicativo",
    "reduplico": "reduplicar",
    "reduplicábamos": "reduplicar",
    "reduplicáis": "reduplicar",
    "reduplicáramos": "reduplicar",
    "reduplicáremos": "reduplicar",
    "reduplicásemos": "reduplicar",
    "reduplicó": "reduplicar",
    "reduplique": "reduplicar",
    "redupliquemos": "reduplicar",
    "redupliquen": "reduplicar",
    "redupliques": "reduplicar",
    "redupliqué": "reduplicar",
    "redupliquéis": "reduplicar",
    "reduzca": "reducir",
    "reduzcamos": "reducir",
    "reduzcan": "reducir",
    "reduzcas": "reducir",
    "reduzco": "reducir",
    "reduzcáis": "reducir",
    "redábamos": "redar",
    "redáis": "redar",
    "redáramos": "redar",
    "redáremos": "redar",
    "redásemos": "redar",
    "redé": "redar",
    "redéis": "redar",
    "redó": "redar",
    "reediciones": "reedición",
    "reedifica": "reedificar",
    "reedificaba": "reedificar",
    "reedificabais": "reedificar",
    "reedificaban": "reedificar",
    "reedificabas": "reedificar",
    "reedificaciones": "reedificación",
    "reedificad": "reedificar",
    "reedificada": "reedificar",
    "reedificadas": "reedificar",
    "reedificado": "reedificar",
    "reedificadora": "reedificador",
    "reedificadoras": "reedificador",
    "reedificadores": "reedificador",
    "reedificados": "reedificar",
    "reedificamos": "reedificar",
    "reedifican": "reedificar",
    "reedificando": "reedificar",
    "reedificara": "reedificar",
    "reedificarais": "reedificar",
    "reedificaran": "reedificar",
    "reedificaras": "reedificar",
    "reedificare": "reedificar",
    "reedificareis": "reedificar",
    "reedificaremos": "reedificar",
    "reedificaren": "reedificar",
    "reedificares": "reedificar",
    "reedificaron": "reedificar",
    "reedificará": "reedificar",
    "reedificarán": "reedificar",
    "reedificarás": "reedificar",
    "reedificaré": "reedificar",
    "reedificaréis": "reedificar",
    "reedificaría": "reedificar",
    "reedificaríais": "reedificar",
    "reedificaríamos": "reedificar",
    "reedificarían": "reedificar",
    "reedificarías": "reedificar",
    "reedificas": "reedificar",
    "reedificase": "reedificar",
    "reedificaseis": "reedificar",
    "reedificasen": "reedificar",
    "reedificases": "reedificar",
    "reedificaste": "reedificar",
    "reedificasteis": "reedificar",
    "reedifico": "reedificar",
    "reedificábamos": "reedificar",
    "reedificáis": "reedificar",
    "reedificáramos": "reedificar",
    "reedificáremos": "reedificar",
    "reedificásemos": "reedificar",
    "reedificó": "reedificar",
    "reedifique": "reedificar",
    "reedifiquemos": "reedificar",
    "reedifiquen": "reedificar",
    "reedifiques": "reedificar",
    "reedifiqué": "reedificar",
    "reedifiquéis": "reedificar",
    "reedita": "reeditar",
    "reeditaba": "reeditar",
    "reeditabais": "reeditar",
    "reeditaban": "reeditar",
    "reeditabas": "reeditar",
    "reeditad": "reeditar",
    "reeditada": "reeditar",
    "reeditadas": "reeditar",
    "reeditado": "reeditar",
    "reeditados": "reeditar",
    "reeditamos": "reeditar",
    "reeditan": "reeditar",
    "reeditando": "reeditar",
    "reeditara": "reeditar",
    "reeditarais": "reeditar",
    "reeditaran": "reeditar",
    "reeditaras": "reeditar",
    "reeditare": "reeditar",
    "reeditareis": "reeditar",
    "reeditaremos": "reeditar",
    "reeditaren": "reeditar",
    "reeditares": "reeditar",
    "reeditaron": "reeditar",
    "reeditará": "reeditar",
    "reeditarán": "reeditar",
    "reeditarás": "reeditar",
    "reeditaré": "reeditar",
    "reeditaréis": "reeditar",
    "reeditaría": "reeditar",
    "reeditaríais": "reeditar",
    "reeditaríamos": "reeditar",
    "reeditarían": "reeditar",
    "reeditarías": "reeditar",
    "reeditas": "reeditar",
    "reeditase": "reeditar",
    "reeditaseis": "reeditar",
    "reeditasen": "reeditar",
    "reeditases": "reeditar",
    "reeditaste": "reeditar",
    "reeditasteis": "reeditar",
    "reedite": "reeditar",
    "reeditemos": "reeditar",
    "reediten": "reeditar",
    "reedites": "reeditar",
    "reedito": "reeditar",
    "reeditábamos": "reeditar",
    "reeditáis": "reeditar",
    "reeditáramos": "reeditar",
    "reeditáremos": "reeditar",
    "reeditásemos": "reeditar",
    "reedité": "reeditar",
    "reeditéis": "reeditar",
    "reeditó": "reeditar",
    "reeduca": "reeducar",
    "reeducaba": "reeducar",
    "reeducabais": "reeducar",
    "reeducaban": "reeducar",
    "reeducabas": "reeducar",
    "reeducaciones": "reeducación",
    "reeducad": "reeducar",
    "reeducada": "reeducar",
    "reeducadas": "reeducar",
    "reeducado": "reeducar",
    "reeducados": "reeducar",
    "reeducamos": "reeducar",
    "reeducan": "reeducar",
    "reeducando": "reeducar",
    "reeducara": "reeducar",
    "reeducarais": "reeducar",
    "reeducaran": "reeducar",
    "reeducaras": "reeducar",
    "reeducare": "reeducar",
    "reeducareis": "reeducar",
    "reeducaremos": "reeducar",
    "reeducaren": "reeducar",
    "reeducares": "reeducar",
    "reeducaron": "reeducar",
    "reeducará": "reeducar",
    "reeducarán": "reeducar",
    "reeducarás": "reeducar",
    "reeducaré": "reeducar",
    "reeducaréis": "reeducar",
    "reeducaría": "reeducar",
    "reeducaríais": "reeducar",
    "reeducaríamos": "reeducar",
    "reeducarían": "reeducar",
    "reeducarías": "reeducar",
    "reeducas": "reeducar",
    "reeducase": "reeducar",
    "reeducaseis": "reeducar",
    "reeducasen": "reeducar",
    "reeducases": "reeducar",
    "reeducaste": "reeducar",
    "reeducasteis": "reeducar",
    "reeduco": "reeducar",
    "reeducábamos": "reeducar",
    "reeducáis": "reeducar",
    "reeducáramos": "reeducar",
    "reeducáremos": "reeducar",
    "reeducásemos": "reeducar",
    "reeducó": "reeducar",
    "reeduque": "reeducar",
    "reeduquemos": "reeducar",
    "reeduquen": "reeducar",
    "reeduques": "reeducar",
    "reeduqué": "reeducar",
    "reeduquéis": "reeducar",
    "reejecuciones": "reejecución",
    "reelecciones": "reelección",
    "reelecta": "reelegir",
    "reelectas": "reelegir",
    "reelecto": "reelegir",
    "reelectos": "reelegir",
    "reelegibles": "reelegible",
    "reelegid": "reelegir",
    "reelegida": "reelegir",
    "reelegidas": "reelegir",
    "reelegido": "reelegir",
    "reelegidos": "reelegir",
    "reelegimos": "reelegir",
    "reelegiremos": "reelegir",
    "reelegirá": "reelegir",
    "reelegirán": "reelegir",
    "reelegirás": "reelegir",
    "reelegiré": "reelegir",
    "reelegiréis": "reelegir",
    "reelegiría": "reelegir",
    "reelegiríais": "reelegir",
    "reelegiríamos": "reelegir",
    "reelegirían": "reelegir",
    "reelegirías": "reelegir",
    "reelegiste": "reelegir",
    "reelegisteis": "reelegir",
    "reelegí": "reelegir",
    "reelegía": "reelegir",
    "reelegíais": "reelegir",
    "reelegíamos": "reelegir",
    "reelegían": "reelegir",
    "reelegías": "reelegir",
    "reelegís": "reelegir",
    "reelige": "reelegir",
    "reeligen": "reelegir",
    "reeliges": "reelegir",
    "reeligiendo": "reelegir",
    "reeligiera": "reelegir",
    "reeligierais": "reelegir",
    "reeligieran": "reelegir",
    "reeligieras": "reelegir",
    "reeligiere": "reelegir",
    "reeligiereis": "reelegir",
    "reeligieren": "reelegir",
    "reeligieres": "reelegir",
    "reeligieron": "reelegir",
    "reeligiese": "reelegir",
    "reeligieseis": "reelegir",
    "reeligiesen": "reelegir",
    "reeligieses": "reelegir",
    "reeligiéramos": "reelegir",
    "reeligiéremos": "reelegir",
    "reeligiésemos": "reelegir",
    "reeligió": "reelegir",
    "reelija": "reelegir",
    "reelijamos": "reelegir",
    "reelijan": "reelegir",
    "reelijas": "reelegir",
    "reelijo": "reelegir",
    "reelijáis": "reelegir",
    "reembarca": "reembarcar",
    "reembarcaba": "reembarcar",
    "reembarcabais": "reembarcar",
    "reembarcaban": "reembarcar",
    "reembarcabas": "reembarcar",
    "reembarcad": "reembarcar",
    "reembarcada": "reembarcar",
    "reembarcadas": "reembarcar",
    "reembarcado": "reembarcar",
    "reembarcados": "reembarcar",
    "reembarcamos": "reembarcar",
    "reembarcan": "reembarcar",
    "reembarcando": "reembarcar",
    "reembarcara": "reembarcar",
    "reembarcarais": "reembarcar",
    "reembarcaran": "reembarcar",
    "reembarcaras": "reembarcar",
    "reembarcare": "reembarcar",
    "reembarcareis": "reembarcar",
    "reembarcaremos": "reembarcar",
    "reembarcaren": "reembarcar",
    "reembarcares": "reembarcar",
    "reembarcaron": "reembarcar",
    "reembarcará": "reembarcar",
    "reembarcarán": "reembarcar",
    "reembarcarás": "reembarcar",
    "reembarcaré": "reembarcar",
    "reembarcaréis": "reembarcar",
    "reembarcaría": "reembarcar",
    "reembarcaríais": "reembarcar",
    "reembarcaríamos": "reembarcar",
    "reembarcarían": "reembarcar",
    "reembarcarías": "reembarcar",
    "reembarcas": "reembarcar",
    "reembarcase": "reembarcar",
    "reembarcaseis": "reembarcar",
    "reembarcasen": "reembarcar",
    "reembarcases": "reembarcar",
    "reembarcaste": "reembarcar",
    "reembarcasteis": "reembarcar",
    "reembarco": "reembarcar",
    "reembarcábamos": "reembarcar",
    "reembarcáis": "reembarcar",
    "reembarcáramos": "reembarcar",
    "reembarcáremos": "reembarcar",
    "reembarcásemos": "reembarcar",
    "reembarcó": "reembarcar",
    "reembargos": "reembargo",
    "reembarque": "reembarcar",
    "reembarquemos": "reembarcar",
    "reembarquen": "reembarcar",
    "reembarques": "reembarque",
    "reembarqué": "reembarcar",
    "reembarquéis": "reembarcar",
    "reembolsa": "reembolsar",
    "reembolsaba": "reembolsar",
    "reembolsabais": "reembolsar",
    "reembolsaban": "reembolsar",
    "reembolsabas": "reembolsar",
    "reembolsables": "reembolsable",
    "reembolsad": "reembolsar",
    "reembolsada": "reembolsar",
    "reembolsadas": "reembolsar",
    "reembolsado": "reembolsar",
    "reembolsados": "reembolsar",
    "reembolsamos": "reembolsar",
    "reembolsan": "reembolsar",
    "reembolsando": "reembolsar",
    "reembolsara": "reembolsar",
    "reembolsarais": "reembolsar",
    "reembolsaran": "reembolsar",
    "reembolsaras": "reembolsar",
    "reembolsare": "reembolsar",
    "reembolsareis": "reembolsar",
    "reembolsaremos": "reembolsar",
    "reembolsaren": "reembolsar",
    "reembolsares": "reembolsar",
    "reembolsaron": "reembolsar",
    "reembolsará": "reembolsar",
    "reembolsarán": "reembolsar",
    "reembolsarás": "reembolsar",
    "reembolsaré": "reembolsar",
    "reembolsaréis": "reembolsar",
    "reembolsaría": "reembolsar",
    "reembolsaríais": "reembolsar",
    "reembolsaríamos": "reembolsar",
    "reembolsarían": "reembolsar",
    "reembolsarías": "reembolsar",
    "reembolsas": "reembolsar",
    "reembolsase": "reembolsar",
    "reembolsaseis": "reembolsar",
    "reembolsasen": "reembolsar",
    "reembolsases": "reembolsar",
    "reembolsaste": "reembolsar",
    "reembolsasteis": "reembolsar",
    "reembolse": "reembolsar",
    "reembolsemos": "reembolsar",
    "reembolsen": "reembolsar",
    "reembolses": "reembolsar",
    "reembolso": "reembolsar",
    "reembolsos": "reembolso",
    "reembolsábamos": "reembolsar",
    "reembolsáis": "reembolsar",
    "reembolsáramos": "reembolsar",
    "reembolsáremos": "reembolsar",
    "reembolsásemos": "reembolsar",
    "reembolsé": "reembolsar",
    "reembolséis": "reembolsar",
    "reembolsó": "reembolsar",
    "reembragues": "reembrague",
    "reemisores": "reemisor",
    "reempeños": "reempeño",
    "reemplace": "reemplazar",
    "reemplacemos": "reemplazar",
    "reemplacen": "reemplazar",
    "reemplaces": "reemplazar",
    "reemplacé": "reemplazar",
    "reemplacéis": "reemplazar",
    "reemplaza": "reemplazar",
    "reemplazaba": "reemplazar",
    "reemplazabais": "reemplazar",
    "reemplazaban": "reemplazar",
    "reemplazabas": "reemplazar",
    "reemplazables": "reemplazable",
    "reemplazad": "reemplazar",
    "reemplazada": "reemplazar",
    "reemplazadas": "reemplazar",
    "reemplazado": "reemplazar",
    "reemplazados": "reemplazar",
    "reemplazamos": "reemplazar",
    "reemplazan": "reemplazar",
    "reemplazando": "reemplazar",
    "reemplazantes": "reemplazante",
    "reemplazara": "reemplazar",
    "reemplazarais": "reemplazar",
    "reemplazaran": "reemplazar",
    "reemplazaras": "reemplazar",
    "reemplazare": "reemplazar",
    "reemplazareis": "reemplazar",
    "reemplazaremos": "reemplazar",
    "reemplazaren": "reemplazar",
    "reemplazares": "reemplazar",
    "reemplazaron": "reemplazar",
    "reemplazará": "reemplazar",
    "reemplazarán": "reemplazar",
    "reemplazarás": "reemplazar",
    "reemplazaré": "reemplazar",
    "reemplazaréis": "reemplazar",
    "reemplazaría": "reemplazar",
    "reemplazaríais": "reemplazar",
    "reemplazaríamos": "reemplazar",
    "reemplazarían": "reemplazar",
    "reemplazarías": "reemplazar",
    "reemplazas": "reemplazar",
    "reemplazase": "reemplazar",
    "reemplazaseis": "reemplazar",
    "reemplazasen": "reemplazar",
    "reemplazases": "reemplazar",
    "reemplazaste": "reemplazar",
    "reemplazasteis": "reemplazar",
    "reemplazo": "reemplazar",
    "reemplazos": "reemplazo",
    "reemplazábamos": "reemplazar",
    "reemplazáis": "reemplazar",
    "reemplazáramos": "reemplazar",
    "reemplazáremos": "reemplazar",
    "reemplazásemos": "reemplazar",
    "reemplazó": "reemplazar",
    "reempleos": "reempleo",
    "reemprenda": "reemprender",
    "reemprendamos": "reemprender",
    "reemprendan": "reemprender",
    "reemprendas": "reemprender",
    "reemprende": "reemprender",
    "reemprended": "reemprender",
    "reemprendemos": "reemprender",
    "reemprenden": "reemprender",
    "reemprenderemos": "reemprender",
    "reemprenderá": "reemprender",
    "reemprenderán": "reemprender",
    "reemprenderás": "reemprender",
    "reemprenderé": "reemprender",
    "reemprenderéis": "reemprender",
    "reemprendería": "reemprender",
    "reemprenderíais": "reemprender",
    "reemprenderíamos": "reemprender",
    "reemprenderían": "reemprender",
    "reemprenderías": "reemprender",
    "reemprendes": "reemprender",
    "reemprendida": "reemprender",
    "reemprendidas": "reemprender",
    "reemprendido": "reemprender",
    "reemprendidos": "reemprender",
    "reemprendiendo": "reemprender",
    "reemprendiera": "reemprender",
    "reemprendierais": "reemprender",
    "reemprendieran": "reemprender",
    "reemprendieras": "reemprender",
    "reemprendiere": "reemprender",
    "reemprendiereis": "reemprender",
    "reemprendieren": "reemprender",
    "reemprendieres": "reemprender",
    "reemprendieron": "reemprender",
    "reemprendiese": "reemprender",
    "reemprendieseis": "reemprender",
    "reemprendiesen": "reemprender",
    "reemprendieses": "reemprender",
    "reemprendimos": "reemprender",
    "reemprendiste": "reemprender",
    "reemprendisteis": "reemprender",
    "reemprendiéramos": "reemprender",
    "reemprendiéremos": "reemprender",
    "reemprendiésemos": "reemprender",
    "reemprendió": "reemprender",
    "reemprendo": "reemprender",
    "reemprendáis": "reemprender",
    "reemprendéis": "reemprender",
    "reemprendí": "reemprender",
    "reemprendía": "reemprender",
    "reemprendíais": "reemprender",
    "reemprendíamos": "reemprender",
    "reemprendían": "reemprender",
    "reemprendías": "reemprender",
    "reencaminamientos": "reencaminamiento",
    "reencarna": "reencarnar",
    "reencarnaba": "reencarnar",
    "reencarnabais": "reencarnar",
    "reencarnaban": "reencarnar",
    "reencarnabas": "reencarnar",
    "reencarnaciones": "reencarnación",
    "reencarnad": "reencarnar",
    "reencarnada": "reencarnar",
    "reencarnadas": "reencarnar",
    "reencarnado": "reencarnar",
    "reencarnados": "reencarnar",
    "reencarnamos": "reencarnar",
    "reencarnan": "reencarnar",
    "reencarnando": "reencarnar",
    "reencarnara": "reencarnar",
    "reencarnarais": "reencarnar",
    "reencarnaran": "reencarnar",
    "reencarnaras": "reencarnar",
    "reencarnare": "reencarnar",
    "reencarnareis": "reencarnar",
    "reencarnaremos": "reencarnar",
    "reencarnaren": "reencarnar",
    "reencarnares": "reencarnar",
    "reencarnaron": "reencarnar",
    "reencarnará": "reencarnar",
    "reencarnarán": "reencarnar",
    "reencarnarás": "reencarnar",
    "reencarnaré": "reencarnar",
    "reencarnaréis": "reencarnar",
    "reencarnaría": "reencarnar",
    "reencarnaríais": "reencarnar",
    "reencarnaríamos": "reencarnar",
    "reencarnarían": "reencarnar",
    "reencarnarías": "reencarnar",
    "reencarnas": "reencarnar",
    "reencarnase": "reencarnar",
    "reencarnaseis": "reencarnar",
    "reencarnasen": "reencarnar",
    "reencarnases": "reencarnar",
    "reencarnaste": "reencarnar",
    "reencarnasteis": "reencarnar",
    "reencarne": "reencarnar",
    "reencarnemos": "reencarnar",
    "reencarnen": "reencarnar",
    "reencarnes": "reencarnar",
    "reencarno": "reencarnar",
    "reencarnábamos": "reencarnar",
    "reencarnáis": "reencarnar",
    "reencarnáramos": "reencarnar",
    "reencarnáremos": "reencarnar",
    "reencarnásemos": "reencarnar",
    "reencarné": "reencarnar",
    "reencarnéis": "reencarnar",
    "reencarnó": "reencarnar",
    "reencendidos": "reencendido",
    "reencontraba": "reencontrar",
    "reencontrabais": "reencontrar",
    "reencontraban": "reencontrar",
    "reencontrabas": "reencontrar",
    "reencontrad": "reencontrar",
    "reencontrada": "reencontrar",
    "reencontradas": "reencontrar",
    "reencontrado": "reencontrar",
    "reencontrados": "reencontrar",
    "reencontramos": "reencontrar",
    "reencontrando": "reencontrar",
    "reencontrara": "reencontrar",
    "reencontrarais": "reencontrar",
    "reencontraran": "reencontrar",
    "reencontraras": "reencontrar",
    "reencontrare": "reencontrar",
    "reencontrareis": "reencontrar",
    "reencontraremos": "reencontrar",
    "reencontraren": "reencontrar",
    "reencontrares": "reencontrar",
    "reencontraron": "reencontrar",
    "reencontrará": "reencontrar",
    "reencontrarán": "reencontrar",
    "reencontrarás": "reencontrar",
    "reencontraré": "reencontrar",
    "reencontraréis": "reencontrar",
    "reencontraría": "reencontrar",
    "reencontraríais": "reencontrar",
    "reencontraríamos": "reencontrar",
    "reencontrarían": "reencontrar",
    "reencontrarías": "reencontrar",
    "reencontrase": "reencontrar",
    "reencontraseis": "reencontrar",
    "reencontrasen": "reencontrar",
    "reencontrases": "reencontrar",
    "reencontraste": "reencontrar",
    "reencontrasteis": "reencontrar",
    "reencontremos": "reencontrar",
    "reencontrábamos": "reencontrar",
    "reencontráis": "reencontrar",
    "reencontráramos": "reencontrar",
    "reencontráremos": "reencontrar",
    "reencontrásemos": "reencontrar",
    "reencontré": "reencontrar",
    "reencontréis": "reencontrar",
    "reencontró": "reencontrar",
    "reencuaderna": "reencuadernar",
    "reencuadernaba": "reencuadernar",
    "reencuadernabais": "reencuadernar",
    "reencuadernaban": "reencuadernar",
    "reencuadernabas": "reencuadernar",
    "reencuadernaciones": "reencuadernación",
    "reencuadernad": "reencuadernar",
    "reencuadernada": "reencuadernar",
    "reencuadernadas": "reencuadernar",
    "reencuadernado": "reencuadernar",
    "reencuadernados": "reencuadernar",
    "reencuadernamos": "reencuadernar",
    "reencuadernan": "reencuadernar",
    "reencuadernando": "reencuadernar",
    "reencuadernara": "reencuadernar",
    "reencuadernarais": "reencuadernar",
    "reencuadernaran": "reencuadernar",
    "reencuadernaras": "reencuadernar",
    "reencuadernare": "reencuadernar",
    "reencuadernareis": "reencuadernar",
    "reencuadernaremos": "reencuadernar",
    "reencuadernaren": "reencuadernar",
    "reencuadernares": "reencuadernar",
    "reencuadernaron": "reencuadernar",
    "reencuadernará": "reencuadernar",
    "reencuadernarán": "reencuadernar",
    "reencuadernarás": "reencuadernar",
    "reencuadernaré": "reencuadernar",
    "reencuadernaréis": "reencuadernar",
    "reencuadernaría": "reencuadernar",
    "reencuadernaríais": "reencuadernar",
    "reencuadernaríamos": "reencuadernar",
    "reencuadernarían": "reencuadernar",
    "reencuadernarías": "reencuadernar",
    "reencuadernas": "reencuadernar",
    "reencuadernase": "reencuadernar",
    "reencuadernaseis": "reencuadernar",
    "reencuadernasen": "reencuadernar",
    "reencuadernases": "reencuadernar",
    "reencuadernaste": "reencuadernar",
    "reencuadernasteis": "reencuadernar",
    "reencuaderne": "reencuadernar",
    "reencuadernemos": "reencuadernar",
    "reencuadernen": "reencuadernar",
    "reencuadernes": "reencuadernar",
    "reencuaderno": "reencuadernar",
    "reencuadernábamos": "reencuadernar",
    "reencuadernáis": "reencuadernar",
    "reencuadernáramos": "reencuadernar",
    "reencuadernáremos": "reencuadernar",
    "reencuadernásemos": "reencuadernar",
    "reencuaderné": "reencuadernar",
    "reencuadernéis": "reencuadernar",
    "reencuadernó": "reencuadernar",
    "reencuentra": "reencontrar",
    "reencuentran": "reencontrar",
    "reencuentras": "reencontrar",
    "reencuentre": "reencontrar",
    "reencuentren": "reencontrar",
    "reencuentres": "reencontrar",
    "reencuentro": "reencontrar",
    "reencuentros": "reencuentro",
    "reenfoques": "reenfoque",
    "reenfriadores": "reenfriador",
    "reengancha": "reenganchar",
    "reenganchaba": "reenganchar",
    "reenganchabais": "reenganchar",
    "reenganchaban": "reenganchar",
    "reenganchabas": "reenganchar",
    "reenganchad": "reenganchar",
    "reenganchada": "reenganchar",
    "reenganchadas": "reenganchar",
    "reenganchado": "reenganchar",
    "reenganchados": "reenganchar",
    "reenganchamientos": "reenganchamiento",
    "reenganchamos": "reenganchar",
    "reenganchan": "reenganchar",
    "reenganchando": "reenganchar",
    "reenganchara": "reenganchar",
    "reengancharais": "reenganchar",
    "reengancharan": "reenganchar",
    "reengancharas": "reenganchar",
    "reenganchare": "reenganchar",
    "reenganchareis": "reenganchar",
    "reengancharemos": "reenganchar",
    "reengancharen": "reenganchar",
    "reenganchares": "reenganchar",
    "reengancharon": "reenganchar",
    "reenganchará": "reenganchar",
    "reengancharán": "reenganchar",
    "reengancharás": "reenganchar",
    "reengancharé": "reenganchar",
    "reengancharéis": "reenganchar",
    "reengancharía": "reenganchar",
    "reengancharíais": "reenganchar",
    "reengancharíamos": "reenganchar",
    "reengancharían": "reenganchar",
    "reengancharías": "reenganchar",
    "reenganchas": "reenganchar",
    "reenganchase": "reenganchar",
    "reenganchaseis": "reenganchar",
    "reenganchasen": "reenganchar",
    "reenganchases": "reenganchar",
    "reenganchaste": "reenganchar",
    "reenganchasteis": "reenganchar",
    "reenganche": "reenganchar",
    "reenganchemos": "reenganchar",
    "reenganchen": "reenganchar",
    "reenganches": "reenganche",
    "reengancho": "reenganchar",
    "reenganchábamos": "reenganchar",
    "reengancháis": "reenganchar",
    "reengancháramos": "reenganchar",
    "reengancháremos": "reenganchar",
    "reenganchásemos": "reenganchar",
    "reenganché": "reenganchar",
    "reenganchéis": "reenganchar",
    "reenganchó": "reenganchar",
    "reengendra": "reengendrar",
    "reengendraba": "reengendrar",
    "reengendrabais": "reengendrar",
    "reengendraban": "reengendrar",
    "reengendrabas": "reengendrar",
    "reengendrad": "reengendrar",
    "reengendrada": "reengendrar",
    "reengendradas": "reengendrar",
    "reengendrado": "reengendrar",
    "reengendradora": "reengendrador",
    "reengendradoras": "reengendrador",
    "reengendradores": "reengendrador",
    "reengendrados": "reengendrar",
    "reengendramos": "reengendrar",
    "reengendran": "reengendrar",
    "reengendrando": "reengendrar",
    "reengendrara": "reengendrar",
    "reengendrarais": "reengendrar",
    "reengendraran": "reengendrar",
    "reengendraras": "reengendrar",
    "reengendrare": "reengendrar",
    "reengendrareis": "reengendrar",
    "reengendraremos": "reengendrar",
    "reengendraren": "reengendrar",
    "reengendrares": "reengendrar",
    "reengendraron": "reengendrar",
    "reengendrará": "reengendrar",
    "reengendrarán": "reengendrar",
    "reengendrarás": "reengendrar",
    "reengendraré": "reengendrar",
    "reengendraréis": "reengendrar",
    "reengendraría": "reengendrar",
    "reengendraríais": "reengendrar",
    "reengendraríamos": "reengendrar",
    "reengendrarían": "reengendrar",
    "reengendrarías": "reengendrar",
    "reengendras": "reengendrar",
    "reengendrase": "reengendrar",
    "reengendraseis": "reengendrar",
    "reengendrasen": "reengendrar",
    "reengendrases": "reengendrar",
    "reengendraste": "reengendrar",
    "reengendrasteis": "reengendrar",
    "reengendre": "reengendrar",
    "reengendremos": "reengendrar",
    "reengendren": "reengendrar",
    "reengendres": "reengendrar",
    "reengendro": "reengendrar",
    "reengendrábamos": "reengendrar",
    "reengendráis": "reengendrar",
    "reengendráramos": "reengendrar",
    "reengendráremos": "reengendrar",
    "reengendrásemos": "reengendrar",
    "reengendré": "reengendrar",
    "reengendréis": "reengendrar",
    "reengendró": "reengendrar",
    "reenmacetados": "reenmacetado",
    "reensamblajes": "reensamblaje",
    "reensaya": "reensayar",
    "reensayaba": "reensayar",
    "reensayabais": "reensayar",
    "reensayaban": "reensayar",
    "reensayabas": "reensayar",
    "reensayad": "reensayar",
    "reensayada": "reensayar",
    "reensayadas": "reensayar",
    "reensayado": "reensayar",
    "reensayados": "reensayar",
    "reensayamos": "reensayar",
    "reensayan": "reensayar",
    "reensayando": "reensayar",
    "reensayara": "reensayar",
    "reensayarais": "reensayar",
    "reensayaran": "reensayar",
    "reensayaras": "reensayar",
    "reensayare": "reensayar",
    "reensayareis": "reensayar",
    "reensayaremos": "reensayar",
    "reensayaren": "reensayar",
    "reensayares": "reensayar",
    "reensayaron": "reensayar",
    "reensayará": "reensayar",
    "reensayarán": "reensayar",
    "reensayarás": "reensayar",
    "reensayaré": "reensayar",
    "reensayaréis": "reensayar",
    "reensayaría": "reensayar",
    "reensayaríais": "reensayar",
    "reensayaríamos": "reensayar",
    "reensayarían": "reensayar",
    "reensayarías": "reensayar",
    "reensayas": "reensayar",
    "reensayase": "reensayar",
    "reensayaseis": "reensayar",
    "reensayasen": "reensayar",
    "reensayases": "reensayar",
    "reensayaste": "reensayar",
    "reensayasteis": "reensayar",
    "reensaye": "reensayar",
    "reensayemos": "reensayar",
    "reensayen": "reensayar",
    "reensayes": "reensaye",
    "reensayo": "reensayar",
    "reensayos": "reensayo",
    "reensayábamos": "reensayar",
    "reensayáis": "reensayar",
    "reensayáramos": "reensayar",
    "reensayáremos": "reensayar",
    "reensayásemos": "reensayar",
    "reensayé": "reensayar",
    "reensayéis": "reensayar",
    "reensayó": "reensayar",
    "reentrables": "reentrable",
    "reentradas": "reentrada",
    "reentrantes": "reentrante",
    "reentregas": "reentrega",
    "reenviaba": "reenviar",
    "reenviabais": "reenviar",
    "reenviaban": "reenviar",
    "reenviabas": "reenviar",
    "reenviad": "reenviar",
    "reenviada": "reenviar",
    "reenviadas": "reenviar",
    "reenviado": "reenviar",
    "reenviados": "reenviar",
    "reenviamos": "reenviar",
    "reenviando": "reenviar",
    "reenviara": "reenviar",
    "reenviarais": "reenviar",
    "reenviaran": "reenviar",
    "reenviaras": "reenviar",
    "reenviare": "reenviar",
    "reenviareis": "reenviar",
    "reenviaremos": "reenviar",
    "reenviaren": "reenviar",
    "reenviares": "reenviar",
    "reenviaron": "reenviar",
    "reenviará": "reenviar",
    "reenviarán": "reenviar",
    "reenviarás": "reenviar",
    "reenviaré": "reenviar",
    "reenviaréis": "reenviar",
    "reenviaría": "reenviar",
    "reenviaríais": "reenviar",
    "reenviaríamos": "reenviar",
    "reenviarían": "reenviar",
    "reenviarías": "reenviar",
    "reenviase": "reenviar",
    "reenviaseis": "reenviar",
    "reenviasen": "reenviar",
    "reenviases": "reenviar",
    "reenviaste": "reenviar",
    "reenviasteis": "reenviar",
    "reenvida": "reenvidar",
    "reenvidaba": "reenvidar",
    "reenvidabais": "reenvidar",
    "reenvidaban": "reenvidar",
    "reenvidabas": "reenvidar",
    "reenvidad": "reenvidar",
    "reenvidada": "reenvidar",
    "reenvidadas": "reenvidar",
    "reenvidado": "reenvidar",
    "reenvidados": "reenvidar",
    "reenvidamos": "reenvidar",
    "reenvidan": "reenvidar",
    "reenvidando": "reenvidar",
    "reenvidara": "reenvidar",
    "reenvidarais": "reenvidar",
    "reenvidaran": "reenvidar",
    "reenvidaras": "reenvidar",
    "reenvidare": "reenvidar",
    "reenvidareis": "reenvidar",
    "reenvidaremos": "reenvidar",
    "reenvidaren": "reenvidar",
    "reenvidares": "reenvidar",
    "reenvidaron": "reenvidar",
    "reenvidará": "reenvidar",
    "reenvidarán": "reenvidar",
    "reenvidarás": "reenvidar",
    "reenvidaré": "reenvidar",
    "reenvidaréis": "reenvidar",
    "reenvidaría": "reenvidar",
    "reenvidaríais": "reenvidar",
    "reenvidaríamos": "reenvidar",
    "reenvidarían": "reenvidar",
    "reenvidarías": "reenvidar",
    "reenvidas": "reenvidar",
    "reenvidase": "reenvidar",
    "reenvidaseis": "reenvidar",
    "reenvidasen": "reenvidar",
    "reenvidases": "reenvidar",
    "reenvidaste": "reenvidar",
    "reenvidasteis": "reenvidar",
    "reenvide": "reenvidar",
    "reenvidemos": "reenvidar",
    "reenviden": "reenvidar",
    "reenvides": "reenvidar",
    "reenvido": "reenvidar",
    "reenvidábamos": "reenvidar",
    "reenvidáis": "reenvidar",
    "reenvidáramos": "reenvidar",
    "reenvidáremos": "reenvidar",
    "reenvidásemos": "reenvidar",
    "reenvidé": "reenvidar",
    "reenvidéis": "reenvidar",
    "reenvidó": "reenvidar",
    "reenviemos": "reenviar",
    "reenvites": "reenvite",
    "reenviábamos": "reenviar",
    "reenviáis": "reenviar",
    "reenviáramos": "reenviar",
    "reenviáremos": "reenviar",
    "reenviásemos": "reenviar",
    "reenvié": "reenviar",
    "reenviéis": "reenviar",
    "reenvió": "reenviar",
    "reenvía": "reenviar",
    "reenvían": "reenviar",
    "reenvías": "reenviar",
    "reenvíe": "reenviar",
    "reenvíen": "reenviar",
    "reenvíes": "reenviar",
    "reenvío": "reenviar",
    "reenvíos": "reenvío",
    "reequilibra": "reequilibrar",
    "reequilibraba": "reequilibrar",
    "reequilibrabais": "reequilibrar",
    "reequilibraban": "reequilibrar",
    "reequilibrabas": "reequilibrar",
    "reequilibrad": "reequilibrar",
    "reequilibrada": "reequilibrar",
    "reequilibradas": "reequilibrar",
    "reequilibrado": "reequilibrar",
    "reequilibrados": "reequilibrar",
    "reequilibramos": "reequilibrar",
    "reequilibran": "reequilibrar",
    "reequilibrando": "reequilibrar",
    "reequilibrara": "reequilibrar",
    "reequilibrarais": "reequilibrar",
    "reequilibraran": "reequilibrar",
    "reequilibraras": "reequilibrar",
    "reequilibrare": "reequilibrar",
    "reequilibrareis": "reequilibrar",
    "reequilibraremos": "reequilibrar",
    "reequilibraren": "reequilibrar",
    "reequilibrares": "reequilibrar",
    "reequilibraron": "reequilibrar",
    "reequilibrará": "reequilibrar",
    "reequilibrarán": "reequilibrar",
    "reequilibrarás": "reequilibrar",
    "reequilibraré": "reequilibrar",
    "reequilibraréis": "reequilibrar",
    "reequilibraría": "reequilibrar",
    "reequilibraríais": "reequilibrar",
    "reequilibraríamos": "reequilibrar",
    "reequilibrarían": "reequilibrar",
    "reequilibrarías": "reequilibrar",
    "reequilibras": "reequilibrar",
    "reequilibrase": "reequilibrar",
    "reequilibraseis": "reequilibrar",
    "reequilibrasen": "reequilibrar",
    "reequilibrases": "reequilibrar",
    "reequilibraste": "reequilibrar",
    "reequilibrasteis": "reequilibrar",
    "reequilibre": "reequilibrar",
    "reequilibremos": "reequilibrar",
    "reequilibren": "reequilibrar",
    "reequilibres": "reequilibrar",
    "reequilibrios": "reequilibrio",
    "reequilibro": "reequilibrar",
    "reequilibrábamos": "reequilibrar",
    "reequilibráis": "reequilibrar",
    "reequilibráramos": "reequilibrar",
    "reequilibráremos": "reequilibrar",
    "reequilibrásemos": "reequilibrar",
    "reequilibré": "reequilibrar",
    "reequilibréis": "reequilibrar",
    "reequilibró": "reequilibrar",
    "reequipamientos": "reequipamiento",
    "reerecciones": "reerección",
    "reescriba": "reescribir",
    "reescribamos": "reescribir",
    "reescriban": "reescribir",
    "reescribas": "reescribir",
    "reescribe": "reescribir",
    "reescriben": "reescribir",
    "reescribes": "reescribir",
    "reescribid": "reescribir",
    "reescribiendo": "reescribir",
    "reescribiera": "reescribir",
    "reescribierais": "reescribir",
    "reescribieran": "reescribir",
    "reescribieras": "reescribir",
    "reescribiere": "reescribir",
    "reescribiereis": "reescribir",
    "reescribieren": "reescribir",
    "reescribieres": "reescribir",
    "reescribieron": "reescribir",
    "reescribiese": "reescribir",
    "reescribieseis": "reescribir",
    "reescribiesen": "reescribir",
    "reescribieses": "reescribir",
    "reescribimos": "reescribir",
    "reescribiremos": "reescribir",
    "reescribirá": "reescribir",
    "reescribirán": "reescribir",
    "reescribirás": "reescribir",
    "reescribiré": "reescribir",
    "reescribiréis": "reescribir",
    "reescribiría": "reescribir",
    "reescribiríais": "reescribir",
    "reescribiríamos": "reescribir",
    "reescribirían": "reescribir",
    "reescribirías": "reescribir",
    "reescribiste": "reescribir",
    "reescribisteis": "reescribir",
    "reescribiéramos": "reescribir",
    "reescribiéremos": "reescribir",
    "reescribiésemos": "reescribir",
    "reescribió": "reescribir",
    "reescribo": "reescribir",
    "reescribáis": "reescribir",
    "reescribí": "reescribir",
    "reescribía": "reescribir",
    "reescribíais": "reescribir",
    "reescribíamos": "reescribir",
    "reescribían": "reescribir",
    "reescribías": "reescribir",
    "reescribís": "reescribir",
    "reescrita": "reescribir",
    "reescritas": "reescribir",
    "reescrito": "reescribir",
    "reescritos": "reescribir",
    "reescrituras": "reescritura",
    "reestablece": "reestablecer",
    "reestableced": "reestablecer",
    "reestablecemos": "reestablecer",
    "reestablecen": "reestablecer",
    "reestableceremos": "reestablecer",
    "reestablecerá": "reestablecer",
    "reestablecerán": "reestablecer",
    "reestablecerás": "reestablecer",
    "reestableceré": "reestablecer",
    "reestableceréis": "reestablecer",
    "reestablecería": "reestablecer",
    "reestableceríais": "reestablecer",
    "reestableceríamos": "reestablecer",
    "reestablecerían": "reestablecer",
    "reestablecerías": "reestablecer",
    "reestableces": "reestablecer",
    "reestablecida": "reestablecer",
    "reestablecidas": "reestablecer",
    "reestablecido": "reestablecer",
    "reestablecidos": "reestablecer",
    "reestableciendo": "reestablecer",
    "reestableciera": "reestablecer",
    "reestablecierais": "reestablecer",
    "reestablecieran": "reestablecer",
    "reestablecieras": "reestablecer",
    "reestableciere": "reestablecer",
    "reestableciereis": "reestablecer",
    "reestablecieren": "reestablecer",
    "reestablecieres": "reestablecer",
    "reestablecieron": "reestablecer",
    "reestableciese": "reestablecer",
    "reestablecieseis": "reestablecer",
    "reestableciesen": "reestablecer",
    "reestablecieses": "reestablecer",
    "reestablecimos": "reestablecer",
    "reestableciste": "reestablecer",
    "reestablecisteis": "reestablecer",
    "reestableciéramos": "reestablecer",
    "reestableciéremos": "reestablecer",
    "reestableciésemos": "reestablecer",
    "reestableció": "reestablecer",
    "reestablecéis": "reestablecer",
    "reestablecí": "reestablecer",
    "reestablecía": "reestablecer",
    "reestablecíais": "reestablecer",
    "reestablecíamos": "reestablecer",
    "reestablecían": "reestablecer",
    "reestablecías": "reestablecer",
    "reestablezca": "reestablecer",
    "reestablezcamos": "reestablecer",
    "reestablezcan": "reestablecer",
    "reestablezcas": "reestablecer",
    "reestablezco": "reestablecer",
    "reestablezcáis": "reestablecer",
    "reestrena": "reestrenar",
    "reestrenaba": "reestrenar",
    "reestrenabais": "reestrenar",
    "reestrenaban": "reestrenar",
    "reestrenabas": "reestrenar",
    "reestrenad": "reestrenar",
    "reestrenada": "reestrenar",
    "reestrenadas": "reestrenar",
    "reestrenado": "reestrenar",
    "reestrenados": "reestrenar",
    "reestrenamos": "reestrenar",
    "reestrenan": "reestrenar",
    "reestrenando": "reestrenar",
    "reestrenara": "reestrenar",
    "reestrenarais": "reestrenar",
    "reestrenaran": "reestrenar",
    "reestrenaras": "reestrenar",
    "reestrenare": "reestrenar",
    "reestrenareis": "reestrenar",
    "reestrenaremos": "reestrenar",
    "reestrenaren": "reestrenar",
    "reestrenares": "reestrenar",
    "reestrenaron": "reestrenar",
    "reestrenará": "reestrenar",
    "reestrenarán": "reestrenar",
    "reestrenarás": "reestrenar",
    "reestrenaré": "reestrenar",
    "reestrenaréis": "reestrenar",
    "reestrenaría": "reestrenar",
    "reestrenaríais": "reestrenar",
    "reestrenaríamos": "reestrenar",
    "reestrenarían": "reestrenar",
    "reestrenarías": "reestrenar",
    "reestrenas": "reestrenar",
    "reestrenase": "reestrenar",
    "reestrenaseis": "reestrenar",
    "reestrenasen": "reestrenar",
    "reestrenases": "reestrenar",
    "reestrenaste": "reestrenar",
    "reestrenasteis": "reestrenar",
    "reestrene": "reestrenar",
    "reestrenemos": "reestrenar",
    "reestrenen": "reestrenar",
    "reestrenes": "reestrenar",
    "reestreno": "reestrenar",
    "reestrenos": "reestreno",
    "reestrenábamos": "reestrenar",
    "reestrenáis": "reestrenar",
    "reestrenáramos": "reestrenar",
    "reestrenáremos": "reestrenar",
    "reestrenásemos": "reestrenar",
    "reestrené": "reestrenar",
    "reestrenéis": "reestrenar",
    "reestrenó": "reestrenar",
    "reestructura": "reestructurar",
    "reestructuraba": "reestructurar",
    "reestructurabais": "reestructurar",
    "reestructuraban": "reestructurar",
    "reestructurabas": "reestructurar",
    "reestructuraciones": "reestructuración",
    "reestructurad": "reestructurar",
    "reestructurada": "reestructurar",
    "reestructuradas": "reestructurar",
    "reestructurado": "reestructurar",
    "reestructurados": "reestructurar",
    "reestructuramos": "reestructurar",
    "reestructuran": "reestructurar",
    "reestructurando": "reestructurar",
    "reestructurara": "reestructurar",
    "reestructurarais": "reestructurar",
    "reestructuraran": "reestructurar",
    "reestructuraras": "reestructurar",
    "reestructurare": "reestructurar",
    "reestructurareis": "reestructurar",
    "reestructuraremos": "reestructurar",
    "reestructuraren": "reestructurar",
    "reestructurares": "reestructurar",
    "reestructuraron": "reestructurar",
    "reestructurará": "reestructurar",
    "reestructurarán": "reestructurar",
    "reestructurarás": "reestructurar",
    "reestructuraré": "reestructurar",
    "reestructuraréis": "reestructurar",
    "reestructuraría": "reestructurar",
    "reestructuraríais": "reestructurar",
    "reestructuraríamos": "reestructurar",
    "reestructurarían": "reestructurar",
    "reestructurarías": "reestructurar",
    "reestructuras": "reestructurar",
    "reestructurase": "reestructurar",
    "reestructuraseis": "reestructurar",
    "reestructurasen": "reestructurar",
    "reestructurases": "reestructurar",
    "reestructuraste": "reestructurar",
    "reestructurasteis": "reestructurar",
    "reestructure": "reestructurar",
    "reestructuremos": "reestructurar",
    "reestructuren": "reestructurar",
    "reestructures": "reestructurar",
    "reestructuro": "reestructurar",
    "reestructurábamos": "reestructurar",
    "reestructuráis": "reestructurar",
    "reestructuráramos": "reestructurar",
    "reestructuráremos": "reestructurar",
    "reestructurásemos": "reestructurar",
    "reestructuré": "reestructurar",
    "reestructuréis": "reestructurar",
    "reestructuró": "reestructurar",
    "reevaluaciones": "reevaluación",
    "reexamina": "reexaminar",
    "reexaminaba": "reexaminar",
    "reexaminabais": "reexaminar",
    "reexaminaban": "reexaminar",
    "reexaminabas": "reexaminar",
    "reexaminaciones": "reexaminación",
    "reexaminad": "reexaminar",
    "reexaminada": "reexaminar",
    "reexaminadas": "reexaminar",
    "reexaminado": "reexaminar",
    "reexaminados": "reexaminar",
    "reexaminamos": "reexaminar",
    "reexaminan": "reexaminar",
    "reexaminando": "reexaminar",
    "reexaminara": "reexaminar",
    "reexaminarais": "reexaminar",
    "reexaminaran": "reexaminar",
    "reexaminaras": "reexaminar",
    "reexaminare": "reexaminar",
    "reexaminareis": "reexaminar",
    "reexaminaremos": "reexaminar",
    "reexaminaren": "reexaminar",
    "reexaminares": "reexaminar",
    "reexaminaron": "reexaminar",
    "reexaminará": "reexaminar",
    "reexaminarán": "reexaminar",
    "reexaminarás": "reexaminar",
    "reexaminaré": "reexaminar",
    "reexaminaréis": "reexaminar",
    "reexaminaría": "reexaminar",
    "reexaminaríais": "reexaminar",
    "reexaminaríamos": "reexaminar",
    "reexaminarían": "reexaminar",
    "reexaminarías": "reexaminar",
    "reexaminas": "reexaminar",
    "reexaminase": "reexaminar",
    "reexaminaseis": "reexaminar",
    "reexaminasen": "reexaminar",
    "reexaminases": "reexaminar",
    "reexaminaste": "reexaminar",
    "reexaminasteis": "reexaminar",
    "reexamine": "reexaminar",
    "reexaminemos": "reexaminar",
    "reexaminen": "reexaminar",
    "reexamines": "reexaminar",
    "reexamino": "reexaminar",
    "reexaminábamos": "reexaminar",
    "reexamináis": "reexaminar",
    "reexamináramos": "reexaminar",
    "reexamináremos": "reexaminar",
    "reexaminásemos": "reexaminar",
    "reexaminé": "reexaminar",
    "reexaminéis": "reexaminar",
    "reexaminó": "reexaminar",
    "reexpediciones": "reexpedición",
    "reexpedid": "reexpedir",
    "reexpedida": "reexpedir",
    "reexpedidas": "reexpedir",
    "reexpedido": "reexpedir",
    "reexpedidos": "reexpedir",
    "reexpedimos": "reexpedir",
    "reexpediremos": "reexpedir",
    "reexpedirá": "reexpedir",
    "reexpedirán": "reexpedir",
    "reexpedirás": "reexpedir",
    "reexpediré": "reexpedir",
    "reexpediréis": "reexpedir",
    "reexpediría": "reexpedir",
    "reexpediríais": "reexpedir",
    "reexpediríamos": "reexpedir",
    "reexpedirían": "reexpedir",
    "reexpedirías": "reexpedir",
    "reexpediste": "reexpedir",
    "reexpedisteis": "reexpedir",
    "reexpedí": "reexpedir",
    "reexpedía": "reexpedir",
    "reexpedíais": "reexpedir",
    "reexpedíamos": "reexpedir",
    "reexpedían": "reexpedir",
    "reexpedías": "reexpedir",
    "reexpedís": "reexpedir",
    "reexpida": "reexpedir",
    "reexpidamos": "reexpedir",
    "reexpidan": "reexpedir",
    "reexpidas": "reexpedir",
    "reexpide": "reexpedir",
    "reexpiden": "reexpedir",
    "reexpides": "reexpedir",
    "reexpidiendo": "reexpedir",
    "reexpidiera": "reexpedir",
    "reexpidierais": "reexpedir",
    "reexpidieran": "reexpedir",
    "reexpidieras": "reexpedir",
    "reexpidiere": "reexpedir",
    "reexpidiereis": "reexpedir",
    "reexpidieren": "reexpedir",
    "reexpidieres": "reexpedir",
    "reexpidieron": "reexpedir",
    "reexpidiese": "reexpedir",
    "reexpidieseis": "reexpedir",
    "reexpidiesen": "reexpedir",
    "reexpidieses": "reexpedir",
    "reexpidiéramos": "reexpedir",
    "reexpidiéremos": "reexpedir",
    "reexpidiésemos": "reexpedir",
    "reexpidió": "reexpedir",
    "reexpido": "reexpedir",
    "reexpidáis": "reexpedir",
    "reexporta": "reexportar",
    "reexportaba": "reexportar",
    "reexportabais": "reexportar",
    "reexportaban": "reexportar",
    "reexportabas": "reexportar",
    "reexportaciones": "reexportación",
    "reexportad": "reexportar",
    "reexportada": "reexportar",
    "reexportadas": "reexportar",
    "reexportado": "reexportar",
    "reexportados": "reexportar",
    "reexportamos": "reexportar",
    "reexportan": "reexportar",
    "reexportando": "reexportar",
    "reexportara": "reexportar",
    "reexportarais": "reexportar",
    "reexportaran": "reexportar",
    "reexportaras": "reexportar",
    "reexportare": "reexportar",
    "reexportareis": "reexportar",
    "reexportaremos": "reexportar",
    "reexportaren": "reexportar",
    "reexportares": "reexportar",
    "reexportaron": "reexportar",
    "reexportará": "reexportar",
    "reexportarán": "reexportar",
    "reexportarás": "reexportar",
    "reexportaré": "reexportar",
    "reexportaréis": "reexportar",
    "reexportaría": "reexportar",
    "reexportaríais": "reexportar",
    "reexportaríamos": "reexportar",
    "reexportarían": "reexportar",
    "reexportarías": "reexportar",
    "reexportas": "reexportar",
    "reexportase": "reexportar",
    "reexportaseis": "reexportar",
    "reexportasen": "reexportar",
    "reexportases": "reexportar",
    "reexportaste": "reexportar",
    "reexportasteis": "reexportar",
    "reexporte": "reexportar",
    "reexportemos": "reexportar",
    "reexporten": "reexportar",
    "reexportes": "reexportar",
    "reexporto": "reexportar",
    "reexportábamos": "reexportar",
    "reexportáis": "reexportar",
    "reexportáramos": "reexportar",
    "reexportáremos": "reexportar",
    "reexportásemos": "reexportar",
    "reexporté": "reexportar",
    "reexportéis": "reexportar",
    "reexportó": "reexportar",
    "reextracciones": "reextracción",
    "reexámenes": "reexamen",
    "refaccionaria": "refaccionario",
    "refaccionarias": "refaccionario",
    "refaccionarios": "refaccionario",
    "refacciones": "refacción",
    "refajos": "refajo",
    "refectorios": "refectorio",
    "referencia": "referenciar",
    "referenciaba": "referenciar",
    "referenciabais": "referenciar",
    "referenciaban": "referenciar",
    "referenciabas": "referenciar",
    "referenciad": "referenciar",
    "referenciada": "referenciar",
    "referenciadas": "referenciar",
    "referenciado": "referenciar",
    "referenciados": "referenciar",
    "referenciales": "referencial",
    "referenciamos": "referenciar",
    "referencian": "referenciar",
    "referenciando": "referenciar",
    "referenciara": "referenciar",
    "referenciarais": "referenciar",
    "referenciaran": "referenciar",
    "referenciaras": "referenciar",
    "referenciare": "referenciar",
    "referenciareis": "referenciar",
    "referenciaremos": "referenciar",
    "referenciaren": "referenciar",
    "referenciares": "referenciar",
    "referenciaron": "referenciar",
    "referenciará": "referenciar",
    "referenciarán": "referenciar",
    "referenciarás": "referenciar",
    "referenciaré": "referenciar",
    "referenciaréis": "referenciar",
    "referenciaría": "referenciar",
    "referenciaríais": "referenciar",
    "referenciaríamos": "referenciar",
    "referenciarían": "referenciar",
    "referenciarías": "referenciar",
    "referencias": "referenciar",
    "referenciase": "referenciar",
    "referenciaseis": "referenciar",
    "referenciasen": "referenciar",
    "referenciases": "referenciar",
    "referenciaste": "referenciar",
    "referenciasteis": "referenciar",
    "referencie": "referenciar",
    "referenciemos": "referenciar",
    "referencien": "referenciar",
    "referencies": "referenciar",
    "referencio": "referenciar",
    "referenciábamos": "referenciar",
    "referenciáis": "referenciar",
    "referenciáramos": "referenciar",
    "referenciáremos": "referenciar",
    "referenciásemos": "referenciar",
    "referencié": "referenciar",
    "referenciéis": "referenciar",
    "referenció": "referenciar",
    "referendarios": "referendario",
    "referendos": "referendo",
    "referentes": "referente",
    "referibles": "referible",
    "referid": "referir",
    "referida": "referir",
    "referidas": "referir",
    "referido": "referir",
    "referidos": "referir",
    "referimos": "referir",
    "referiremos": "referir",
    "referirá": "referir",
    "referirán": "referir",
    "referirás": "referir",
    "referiré": "referir",
    "referiréis": "referir",
    "referiría": "referir",
    "referiríais": "referir",
    "referiríamos": "referir",
    "referirían": "referir",
    "referirías": "referir",
    "referiste": "referir",
    "referisteis": "referir",
    "referéndums": "referéndum",
    "referí": "referir",
    "refería": "referir",
    "referíais": "referir",
    "referíamos": "referir",
    "referían": "referir",
    "referías": "referir",
    "referíes": "referí",
    "referís": "referir",
    "refiera": "referir",
    "refieran": "referir",
    "refieras": "referir",
    "refiere": "referir",
    "refieren": "referir",
    "refieres": "referir",
    "refiero": "referir",
    "refina": "refino",
    "refinaba": "refinar",
    "refinabais": "refinar",
    "refinaban": "refinar",
    "refinabas": "refinar",
    "refinaciones": "refinación",
    "refinad": "refinar",
    "refinada": "refinar",
    "refinadas": "refinar",
    "refinaderas": "refinadera",
    "refinado": "refinar",
    "refinadora": "refinador",
    "refinadoras": "refinador",
    "refinadores": "refinador",
    "refinados": "refinar",
    "refinaduras": "refinadura",
    "refinamientos": "refinamiento",
    "refinamos": "refinar",
    "refinan": "refinar",
    "refinancia": "refinanciar",
    "refinanciaba": "refinanciar",
    "refinanciabais": "refinanciar",
    "refinanciaban": "refinanciar",
    "refinanciabas": "refinanciar",
    "refinanciables": "refinanciable",
    "refinanciaciones": "refinanciación",
    "refinanciad": "refinanciar",
    "refinanciada": "refinanciar",
    "refinanciadas": "refinanciar",
    "refinanciado": "refinanciar",
    "refinanciados": "refinanciar",
    "refinanciamientos": "refinanciamiento",
    "refinanciamos": "refinanciar",
    "refinancian": "refinanciar",
    "refinanciando": "refinanciar",
    "refinanciara": "refinanciar",
    "refinanciarais": "refinanciar",
    "refinanciaran": "refinanciar",
    "refinanciaras": "refinanciar",
    "refinanciare": "refinanciar",
    "refinanciareis": "refinanciar",
    "refinanciaremos": "refinanciar",
    "refinanciaren": "refinanciar",
    "refinanciares": "refinanciar",
    "refinanciaron": "refinanciar",
    "refinanciará": "refinanciar",
    "refinanciarán": "refinanciar",
    "refinanciarás": "refinanciar",
    "refinanciaré": "refinanciar",
    "refinanciaréis": "refinanciar",
    "refinanciaría": "refinanciar",
    "refinanciaríais": "refinanciar",
    "refinanciaríamos": "refinanciar",
    "refinanciarían": "refinanciar",
    "refinanciarías": "refinanciar",
    "refinancias": "refinanciar",
    "refinanciase": "refinanciar",
    "refinanciaseis": "refinanciar",
    "refinanciasen": "refinanciar",
    "refinanciases": "refinanciar",
    "refinanciaste": "refinanciar",
    "refinanciasteis": "refinanciar",
    "refinancie": "refinanciar",
    "refinanciemos": "refinanciar",
    "refinancien": "refinanciar",
    "refinancies": "refinanciar",
    "refinancio": "refinanciar",
    "refinanciábamos": "refinanciar",
    "refinanciáis": "refinanciar",
    "refinanciáramos": "refinanciar",
    "refinanciáremos": "refinanciar",
    "refinanciásemos": "refinanciar",
    "refinancié": "refinanciar",
    "refinanciéis": "refinanciar",
    "refinanció": "refinanciar",
    "refinando": "refinar",
    "refinara": "refinar",
    "refinarais": "refinar",
    "refinaran": "refinar",
    "refinaras": "refinar",
    "refinare": "refinar",
    "refinareis": "refinar",
    "refinaremos": "refinar",
    "refinaren": "refinar",
    "refinares": "refinar",
    "refinaron": "refinar",
    "refinará": "refinar",
    "refinarán": "refinar",
    "refinarás": "refinar",
    "refinaré": "refinar",
    "refinaréis": "refinar",
    "refinaría": "refinar",
    "refinaríais": "refinar",
    "refinaríamos": "refinar",
    "refinarían": "refinar",
    "refinarías": "refinar",
    "refinas": "refino",
    "refinase": "refinar",
    "refinaseis": "refinar",
    "refinasen": "refinar",
    "refinases": "refinar",
    "refinaste": "refinar",
    "refinasteis": "refinar",
    "refine": "refinar",
    "refinemos": "refinar",
    "refinen": "refinar",
    "refinerías": "refinería",
    "refines": "refinar",
    "refino": "refinar",
    "refinos": "refino",
    "refinábamos": "refinar",
    "refináis": "refinar",
    "refináramos": "refinar",
    "refináremos": "refinar",
    "refinásemos": "refinar",
    "refiné": "refinar",
    "refinéis": "refinar",
    "refinó": "refinar",
    "refiramos": "referir",
    "refiriendo": "referir",
    "refiriera": "referir",
    "refirierais": "referir",
    "refirieran": "referir",
    "refirieras": "referir",
    "refiriere": "referir",
    "refiriereis": "referir",
    "refirieren": "referir",
    "refirieres": "referir",
    "refirieron": "referir",
    "refiriese": "referir",
    "refirieseis": "referir",
    "refiriesen": "referir",
    "refirieses": "referir",
    "refiriéramos": "referir",
    "refiriéremos": "referir",
    "refiriésemos": "referir",
    "refirió": "referir",
    "refiráis": "referir",
    "reflaciones": "reflación",
    "reflecta": "reflectar",
    "reflectaba": "reflectar",
    "reflectabais": "reflectar",
    "reflectaban": "reflectar",
    "reflectabas": "reflectar",
    "reflectad": "reflectar",
    "reflectada": "reflectar",
    "reflectadas": "reflectar",
    "reflectado": "reflectar",
    "reflectados": "reflectar",
    "reflectamos": "reflectar",
    "reflectan": "reflectar",
    "reflectancias": "reflectancia",
    "reflectando": "reflectar",
    "reflectantes": "reflectante",
    "reflectara": "reflectar",
    "reflectarais": "reflectar",
    "reflectaran": "reflectar",
    "reflectaras": "reflectar",
    "reflectare": "reflectar",
    "reflectareis": "reflectar",
    "reflectaremos": "reflectar",
    "reflectaren": "reflectar",
    "reflectares": "reflectar",
    "reflectaron": "reflectar",
    "reflectará": "reflectar",
    "reflectarán": "reflectar",
    "reflectarás": "reflectar",
    "reflectaré": "reflectar",
    "reflectaréis": "reflectar",
    "reflectaría": "reflectar",
    "reflectaríais": "reflectar",
    "reflectaríamos": "reflectar",
    "reflectarían": "reflectar",
    "reflectarías": "reflectar",
    "reflectas": "reflectar",
    "reflectase": "reflectar",
    "reflectaseis": "reflectar",
    "reflectasen": "reflectar",
    "reflectases": "reflectar",
    "reflectaste": "reflectar",
    "reflectasteis": "reflectar",
    "reflecte": "reflectar",
    "reflectemos": "reflectar",
    "reflecten": "reflectar",
    "reflectes": "reflectar",
    "reflectividades": "reflectividad",
    "reflecto": "reflectar",
    "reflectora": "reflector",
    "reflectoras": "reflector",
    "reflectores": "reflector",
    "reflectábamos": "reflectar",
    "reflectáis": "reflectar",
    "reflectáramos": "reflectar",
    "reflectáremos": "reflectar",
    "reflectásemos": "reflectar",
    "reflecté": "reflectar",
    "reflectéis": "reflectar",
    "reflectó": "reflectar",
    "reflectómetros": "reflectómetro",
    "refleja": "reflejo",
    "reflejaba": "reflejar",
    "reflejabais": "reflejar",
    "reflejaban": "reflejar",
    "reflejabas": "reflejar",
    "reflejables": "reflejable",
    "reflejad": "reflejar",
    "reflejada": "reflejar",
    "reflejadas": "reflejar",
    "reflejado": "reflejar",
    "reflejados": "reflejar",
    "reflejamos": "reflejar",
    "reflejan": "reflejar",
    "reflejando": "reflejar",
    "reflejara": "reflejar",
    "reflejarais": "reflejar",
    "reflejaran": "reflejar",
    "reflejaras": "reflejar",
    "reflejare": "reflejar",
    "reflejareis": "reflejar",
    "reflejaremos": "reflejar",
    "reflejaren": "reflejar",
    "reflejares": "reflejar",
    "reflejaron": "reflejar",
    "reflejará": "reflejar",
    "reflejarán": "reflejar",
    "reflejarás": "reflejar",
    "reflejaré": "reflejar",
    "reflejaréis": "reflejar",
    "reflejaría": "reflejar",
    "reflejaríais": "reflejar",
    "reflejaríamos": "reflejar",
    "reflejarían": "reflejar",
    "reflejarías": "reflejar",
    "reflejas": "reflejo",
    "reflejase": "reflejar",
    "reflejaseis": "reflejar",
    "reflejasen": "reflejar",
    "reflejases": "reflejar",
    "reflejaste": "reflejar",
    "reflejasteis": "reflejar",
    "refleje": "reflejar",
    "reflejemos": "reflejar",
    "reflejen": "reflejar",
    "reflejes": "reflejar",
    "reflejo": "reflejar",
    "reflejos": "reflejo",
    "reflejábamos": "reflejar",
    "reflejáis": "reflejar",
    "reflejáramos": "reflejar",
    "reflejáremos": "reflejar",
    "reflejásemos": "reflejar",
    "reflejé": "reflejar",
    "reflejéis": "reflejar",
    "reflejó": "reflejar",
    "reflexibilidades": "reflexibilidad",
    "reflexibles": "reflexible",
    "reflexiona": "reflexionar",
    "reflexionaba": "reflexionar",
    "reflexionabais": "reflexionar",
    "reflexionaban": "reflexionar",
    "reflexionabas": "reflexionar",
    "reflexionad": "reflexionar",
    "reflexionada": "reflexionar",
    "reflexionadas": "reflexionar",
    "reflexionado": "reflexionar",
    "reflexionados": "reflexionar",
    "reflexionamos": "reflexionar",
    "reflexionan": "reflexionar",
    "reflexionando": "reflexionar",
    "reflexionara": "reflexionar",
    "reflexionarais": "reflexionar",
    "reflexionaran": "reflexionar",
    "reflexionaras": "reflexionar",
    "reflexionare": "reflexionar",
    "reflexionareis": "reflexionar",
    "reflexionaremos": "reflexionar",
    "reflexionaren": "reflexionar",
    "reflexionares": "reflexionar",
    "reflexionaron": "reflexionar",
    "reflexionará": "reflexionar",
    "reflexionarán": "reflexionar",
    "reflexionarás": "reflexionar",
    "reflexionaré": "reflexionar",
    "reflexionaréis": "reflexionar",
    "reflexionaría": "reflexionar",
    "reflexionaríais": "reflexionar",
    "reflexionaríamos": "reflexionar",
    "reflexionarían": "reflexionar",
    "reflexionarías": "reflexionar",
    "reflexionas": "reflexionar",
    "reflexionase": "reflexionar",
    "reflexionaseis": "reflexionar",
    "reflexionasen": "reflexionar",
    "reflexionases": "reflexionar",
    "reflexionaste": "reflexionar",
    "reflexionasteis": "reflexionar",
    "reflexione": "reflexionar",
    "reflexionemos": "reflexionar",
    "reflexionen": "reflexionar",
    "reflexiones": "reflexionar",
    "reflexiono": "reflexionar",
    "reflexionábamos": "reflexionar",
    "reflexionáis": "reflexionar",
    "reflexionáramos": "reflexionar",
    "reflexionáremos": "reflexionar",
    "reflexionásemos": "reflexionar",
    "reflexioné": "reflexionar",
    "reflexionéis": "reflexionar",
    "reflexionó": "reflexionar",
    "reflexiva": "reflexivo",
    "reflexivas": "reflexivo",
    "reflexividades": "reflexividad",
    "reflexivos": "reflexivo",
    "reflexologías": "reflexología",
    "reflorece": "reflorecer",
    "refloreced": "reflorecer",
    "reflorecemos": "reflorecer",
    "reflorecen": "reflorecer",
    "refloreceremos": "reflorecer",
    "reflorecerá": "reflorecer",
    "reflorecerán": "reflorecer",
    "reflorecerás": "reflorecer",
    "refloreceré": "reflorecer",
    "refloreceréis": "reflorecer",
    "reflorecería": "reflorecer",
    "refloreceríais": "reflorecer",
    "refloreceríamos": "reflorecer",
    "reflorecerían": "reflorecer",
    "reflorecerías": "reflorecer",
    "refloreces": "reflorecer",
    "reflorecida": "reflorecer",
    "reflorecidas": "reflorecer",
    "reflorecido": "reflorecer",
    "reflorecidos": "reflorecer",
    "refloreciendo": "reflorecer",
    "refloreciera": "reflorecer",
    "reflorecierais": "reflorecer",
    "reflorecieran": "reflorecer",
    "reflorecieras": "reflorecer",
    "refloreciere": "reflorecer",
    "refloreciereis": "reflorecer",
    "reflorecieren": "reflorecer",
    "reflorecieres": "reflorecer",
    "reflorecieron": "reflorecer",
    "refloreciese": "reflorecer",
    "reflorecieseis": "reflorecer",
    "refloreciesen": "reflorecer",
    "reflorecieses": "reflorecer",
    "reflorecimientos": "reflorecimiento",
    "reflorecimos": "reflorecer",
    "refloreciste": "reflorecer",
    "reflorecisteis": "reflorecer",
    "refloreciéramos": "reflorecer",
    "refloreciéremos": "reflorecer",
    "refloreciésemos": "reflorecer",
    "refloreció": "reflorecer",
    "reflorecéis": "reflorecer",
    "reflorecí": "reflorecer",
    "reflorecía": "reflorecer",
    "reflorecíais": "reflorecer",
    "reflorecíamos": "reflorecer",
    "reflorecían": "reflorecer",
    "reflorecías": "reflorecer",
    "reflorezca": "reflorecer",
    "reflorezcamos": "reflorecer",
    "reflorezcan": "reflorecer",
    "reflorezcas": "reflorecer",
    "reflorezco": "reflorecer",
    "reflorezcáis": "reflorecer",
    "reflotes": "reflote",
    "refluid": "refluir",
    "refluida": "refluir",
    "refluidas": "refluir",
    "refluido": "refluir",
    "refluidos": "refluir",
    "refluimos": "refluir",
    "refluiremos": "refluir",
    "refluirá": "refluir",
    "refluirán": "refluir",
    "refluirás": "refluir",
    "refluiré": "refluir",
    "refluiréis": "refluir",
    "refluiría": "refluir",
    "refluiríais": "refluir",
    "refluiríamos": "refluir",
    "refluirían": "refluir",
    "refluirías": "refluir",
    "refluiste": "refluir",
    "refluisteis": "refluir",
    "reflujos": "reflujo",
    "refluya": "refluir",
    "refluyamos": "refluir",
    "refluyan": "refluir",
    "refluyas": "refluir",
    "refluye": "refluir",
    "refluyen": "refluir",
    "refluyendo": "refluir",
    "refluyera": "refluir",
    "refluyerais": "refluir",
    "refluyeran": "refluir",
    "refluyeras": "refluir",
    "refluyere": "refluir",
    "refluyereis": "refluir",
    "refluyeren": "refluir",
    "refluyeres": "refluir",
    "refluyeron": "refluir",
    "refluyes": "refluir",
    "refluyese": "refluir",
    "refluyeseis": "refluir",
    "refluyesen": "refluir",
    "refluyeses": "refluir",
    "refluyo": "refluir",
    "refluyáis": "refluir",
    "refluyéramos": "refluir",
    "refluyéremos": "refluir",
    "refluyésemos": "refluir",
    "refluyó": "refluir",
    "refluí": "refluir",
    "refluía": "refluir",
    "refluíais": "refluir",
    "refluíamos": "refluir",
    "refluían": "refluir",
    "refluías": "refluir",
    "refluís": "refluir",
    "refocila": "refocilar",
    "refocilaba": "refocilar",
    "refocilabais": "refocilar",
    "refocilaban": "refocilar",
    "refocilabas": "refocilar",
    "refocilaciones": "refocilación",
    "refocilad": "refocilar",
    "refocilada": "refocilar",
    "refociladas": "refocilar",
    "refocilado": "refocilar",
    "refocilados": "refocilar",
    "refocilamos": "refocilar",
    "refocilan": "refocilar",
    "refocilando": "refocilar",
    "refocilara": "refocilar",
    "refocilarais": "refocilar",
    "refocilaran": "refocilar",
    "refocilaras": "refocilar",
    "refocilare": "refocilar",
    "refocilareis": "refocilar",
    "refocilaremos": "refocilar",
    "refocilaren": "refocilar",
    "refocilares": "refocilar",
    "refocilaron": "refocilar",
    "refocilará": "refocilar",
    "refocilarán": "refocilar",
    "refocilarás": "refocilar",
    "refocilaré": "refocilar",
    "refocilaréis": "refocilar",
    "refocilaría": "refocilar",
    "refocilaríais": "refocilar",
    "refocilaríamos": "refocilar",
    "refocilarían": "refocilar",
    "refocilarías": "refocilar",
    "refocilas": "refocilar",
    "refocilase": "refocilar",
    "refocilaseis": "refocilar",
    "refocilasen": "refocilar",
    "refocilases": "refocilar",
    "refocilaste": "refocilar",
    "refocilasteis": "refocilar",
    "refocile": "refocilar",
    "refocilemos": "refocilar",
    "refocilen": "refocilar",
    "refociles": "refocilar",
    "refocilo": "refocilar",
    "refocilos": "refocilo",
    "refocilábamos": "refocilar",
    "refociláis": "refocilar",
    "refociláramos": "refocilar",
    "refociláremos": "refocilar",
    "refocilásemos": "refocilar",
    "refocilé": "refocilar",
    "refociléis": "refocilar",
    "refociló": "refocilar",
    "refonologizaciones": "refonologización",
    "reforcemos": "reforzar",
    "reforcé": "reforzar",
    "reforcéis": "reforzar",
    "reforestaciones": "reforestación",
    "reforma": "reformar",
    "reformaba": "reformar",
    "reformabais": "reformar",
    "reformaban": "reformar",
    "reformabas": "reformar",
    "reformables": "reformable",
    "reformaciones": "reformación",
    "reformad": "reformar",
    "reformada": "reformar",
    "reformadas": "reformar",
    "reformado": "reformar",
    "reformadora": "reformador",
    "reformadoras": "reformador",
    "reformadores": "reformador",
    "reformados": "reformar",
    "reformamientos": "reformamiento",
    "reformamos": "reformar",
    "reforman": "reformar",
    "reformando": "reformar",
    "reformara": "reformar",
    "reformarais": "reformar",
    "reformaran": "reformar",
    "reformaras": "reformar",
    "reformare": "reformar",
    "reformareis": "reformar",
    "reformaremos": "reformar",
    "reformaren": "reformar",
    "reformares": "reformar",
    "reformaron": "reformar",
    "reformará": "reformar",
    "reformarán": "reformar",
    "reformarás": "reformar",
    "reformaré": "reformar",
    "reformaréis": "reformar",
    "reformaría": "reformar",
    "reformaríais": "reformar",
    "reformaríamos": "reformar",
    "reformarían": "reformar",
    "reformarías": "reformar",
    "reformas": "reformar",
    "reformase": "reformar",
    "reformaseis": "reformar",
    "reformasen": "reformar",
    "reformases": "reformar",
    "reformaste": "reformar",
    "reformasteis": "reformar",
    "reformatea": "reformatear",
    "reformateaba": "reformatear",
    "reformateabais": "reformatear",
    "reformateaban": "reformatear",
    "reformateabas": "reformatear",
    "reformatead": "reformatear",
    "reformateada": "reformatear",
    "reformateadas": "reformatear",
    "reformateado": "reformatear",
    "reformateadores": "reformateador",
    "reformateados": "reformatear",
    "reformateamos": "reformatear",
    "reformatean": "reformatear",
    "reformateando": "reformatear",
    "reformateara": "reformatear",
    "reformatearais": "reformatear",
    "reformatearan": "reformatear",
    "reformatearas": "reformatear",
    "reformateare": "reformatear",
    "reformateareis": "reformatear",
    "reformatearemos": "reformatear",
    "reformatearen": "reformatear",
    "reformateares": "reformatear",
    "reformatearon": "reformatear",
    "reformateará": "reformatear",
    "reformatearán": "reformatear",
    "reformatearás": "reformatear",
    "reformatearé": "reformatear",
    "reformatearéis": "reformatear",
    "reformatearía": "reformatear",
    "reformatearíais": "reformatear",
    "reformatearíamos": "reformatear",
    "reformatearían": "reformatear",
    "reformatearías": "reformatear",
    "reformateas": "reformatear",
    "reformatease": "reformatear",
    "reformateaseis": "reformatear",
    "reformateasen": "reformatear",
    "reformateases": "reformatear",
    "reformateaste": "reformatear",
    "reformateasteis": "reformatear",
    "reformatee": "reformatear",
    "reformateemos": "reformatear",
    "reformateen": "reformatear",
    "reformatees": "reformatear",
    "reformateo": "reformatear",
    "reformateos": "reformateo",
    "reformateábamos": "reformatear",
    "reformateáis": "reformatear",
    "reformateáramos": "reformatear",
    "reformateáremos": "reformatear",
    "reformateásemos": "reformatear",
    "reformateé": "reformatear",
    "reformateéis": "reformatear",
    "reformateó": "reformatear",
    "reformativa": "reformativo",
    "reformativas": "reformativo",
    "reformativos": "reformativo",
    "reformatoria": "reformatorio",
    "reformatorias": "reformatorio",
    "reformatorios": "reformatorio",
    "reformatos": "reformato",
    "reforme": "reformar",
    "reformemos": "reformar",
    "reformen": "reformar",
    "reformes": "reformar",
    "reformismos": "reformismo",
    "reformistas": "reformista",
    "reformo": "reformar",
    "reformulaciones": "reformulación",
    "reformábamos": "reformar",
    "reformáis": "reformar",
    "reformáramos": "reformar",
    "reformáremos": "reformar",
    "reformásemos": "reformar",
    "reformé": "reformar",
    "reforméis": "reformar",
    "reformó": "reformar",
    "reforzaba": "reforzar",
    "reforzabais": "reforzar",
    "reforzaban": "reforzar",
    "reforzabas": "reforzar",
    "reforzad": "reforzar",
    "reforzada": "reforzar",
    "reforzadas": "reforzar",
    "reforzado": "reforzar",
    "reforzadora": "reforzador",
    "reforzadoras": "reforzador",
    "reforzadores": "reforzador",
    "reforzados": "reforzar",
    "reforzamientos": "reforzamiento",
    "reforzamos": "reforzar",
    "reforzando": "reforzar",
    "reforzantes": "reforzante",
    "reforzara": "reforzar",
    "reforzarais": "reforzar",
    "reforzaran": "reforzar",
    "reforzaras": "reforzar",
    "reforzare": "reforzar",
    "reforzareis": "reforzar",
    "reforzaremos": "reforzar",
    "reforzaren": "reforzar",
    "reforzares": "reforzar",
    "reforzaron": "reforzar",
    "reforzará": "reforzar",
    "reforzarán": "reforzar",
    "reforzarás": "reforzar",
    "reforzaré": "reforzar",
    "reforzaréis": "reforzar",
    "reforzaría": "reforzar",
    "reforzaríais": "reforzar",
    "reforzaríamos": "reforzar",
    "reforzarían": "reforzar",
    "reforzarías": "reforzar",
    "reforzase": "reforzar",
    "reforzaseis": "reforzar",
    "reforzasen": "reforzar",
    "reforzases": "reforzar",
    "reforzaste": "reforzar",
    "reforzasteis": "reforzar",
    "reforzábamos": "reforzar",
    "reforzáis": "reforzar",
    "reforzáramos": "reforzar",
    "reforzáremos": "reforzar",
    "reforzásemos": "reforzar",
    "reforzó": "reforzar",
    "refracciones": "refracción",
    "refracta": "refractar",
    "refractaba": "refractar",
    "refractabais": "refractar",
    "refractaban": "refractar",
    "refractabas": "refractar",
    "refractad": "refractar",
    "refractada": "refractar",
    "refractadas": "refractar",
    "refractado": "refractar",
    "refractados": "refractar",
    "refractamos": "refractar",
    "refractan": "refractar",
    "refractando": "refractar",
    "refractara": "refractar",
    "refractarais": "refractar",
    "refractaran": "refractar",
    "refractaras": "refractar",
    "refractare": "refractar",
    "refractareis": "refractar",
    "refractaremos": "refractar",
    "refractaren": "refractar",
    "refractares": "refractar",
    "refractaria": "refractario",
    "refractarias": "refractario",
    "refractarios": "refractario",
    "refractaron": "refractar",
    "refractará": "refractar",
    "refractarán": "refractar",
    "refractarás": "refractar",
    "refractaré": "refractar",
    "refractaréis": "refractar",
    "refractaría": "refractar",
    "refractaríais": "refractar",
    "refractaríamos": "refractar",
    "refractarían": "refractar",
    "refractarías": "refractar",
    "refractas": "refractar",
    "refractase": "refractar",
    "refractaseis": "refractar",
    "refractasen": "refractar",
    "refractases": "refractar",
    "refractaste": "refractar",
    "refractasteis": "refractar",
    "refracte": "refractar",
    "refractemos": "refractar",
    "refracten": "refractar",
    "refractes": "refractar",
    "refractiva": "refractivo",
    "refractivas": "refractivo",
    "refractividades": "refractividad",
    "refractivos": "refractivo",
    "refracto": "refractar",
    "refractometrías": "refractometría",
    "refractométrica": "refractométrico",
    "refractométricas": "refractométrico",
    "refractométricos": "refractométrico",
    "refractora": "refractor",
    "refractoras": "refractor",
    "refractores": "refractor",
    "refractábamos": "refractar",
    "refractáis": "refractar",
    "refractáramos": "refractar",
    "refractáremos": "refractar",
    "refractásemos": "refractar",
    "refracté": "refractar",
    "refractéis": "refractar",
    "refractó": "refractar",
    "refractómetros": "refractómetro",
    "refraneros": "refranero",
    "refranes": "refrán",
    "refranesca": "refranesco",
    "refranescas": "refranesco",
    "refranescos": "refranesco",
    "refrangibilidades": "refrangibilidad",
    "refrangibles": "refrangible",
    "refregaba": "refregar",
    "refregabais": "refregar",
    "refregaban": "refregar",
    "refregabas": "refregar",
    "refregad": "refregar",
    "refregada": "refregar",
    "refregadas": "refregar",
    "refregado": "refregar",
    "refregados": "refregar",
    "refregaduras": "refregadura",
    "refregamientos": "refregamiento",
    "refregamos": "refregar",
    "refregando": "refregar",
    "refregara": "refregar",
    "refregarais": "refregar",
    "refregaran": "refregar",
    "refregaras": "refregar",
    "refregare": "refregar",
    "refregareis": "refregar",
    "refregaremos": "refregar",
    "refregaren": "refregar",
    "refregares": "refregar",
    "refregaron": "refregar",
    "refregará": "refregar",
    "refregarán": "refregar",
    "refregarás": "refregar",
    "refregaré": "refregar",
    "refregaréis": "refregar",
    "refregaría": "refregar",
    "refregaríais": "refregar",
    "refregaríamos": "refregar",
    "refregarían": "refregar",
    "refregarías": "refregar",
    "refregase": "refregar",
    "refregaseis": "refregar",
    "refregasen": "refregar",
    "refregases": "refregar",
    "refregaste": "refregar",
    "refregasteis": "refregar",
    "refreguemos": "refregar",
    "refregué": "refregar",
    "refreguéis": "refregar",
    "refregábamos": "refregar",
    "refregáis": "refregar",
    "refregáramos": "refregar",
    "refregáremos": "refregar",
    "refregásemos": "refregar",
    "refregó": "refregar",
    "refreiremos": "refreír",
    "refreirá": "refreír",
    "refreirán": "refreír",
    "refreirás": "refreír",
    "refreiré": "refreír",
    "refreiréis": "refreír",
    "refreiría": "refreír",
    "refreiríais": "refreír",
    "refreiríamos": "refreír",
    "refreirían": "refreír",
    "refreirías": "refreír",
    "refrena": "refrenar",
    "refrenaba": "refrenar",
    "refrenabais": "refrenar",
    "refrenaban": "refrenar",
    "refrenabas": "refrenar",
    "refrenables": "refrenable",
    "refrenad": "refrenar",
    "refrenada": "refrenar",
    "refrenadas": "refrenar",
    "refrenado": "refrenar",
    "refrenados": "refrenar",
    "refrenamientos": "refrenamiento",
    "refrenamos": "refrenar",
    "refrenan": "refrenar",
    "refrenando": "refrenar",
    "refrenara": "refrenar",
    "refrenarais": "refrenar",
    "refrenaran": "refrenar",
    "refrenaras": "refrenar",
    "refrenare": "refrenar",
    "refrenareis": "refrenar",
    "refrenaremos": "refrenar",
    "refrenaren": "refrenar",
    "refrenares": "refrenar",
    "refrenaron": "refrenar",
    "refrenará": "refrenar",
    "refrenarán": "refrenar",
    "refrenarás": "refrenar",
    "refrenaré": "refrenar",
    "refrenaréis": "refrenar",
    "refrenaría": "refrenar",
    "refrenaríais": "refrenar",
    "refrenaríamos": "refrenar",
    "refrenarían": "refrenar",
    "refrenarías": "refrenar",
    "refrenas": "refrenar",
    "refrenase": "refrenar",
    "refrenaseis": "refrenar",
    "refrenasen": "refrenar",
    "refrenases": "refrenar",
    "refrenaste": "refrenar",
    "refrenasteis": "refrenar",
    "refrenda": "refrendar",
    "refrendaba": "refrendar",
    "refrendabais": "refrendar",
    "refrendaban": "refrendar",
    "refrendabas": "refrendar",
    "refrendaciones": "refrendación",
    "refrendad": "refrendar",
    "refrendada": "refrendar",
    "refrendadas": "refrendar",
    "refrendado": "refrendar",
    "refrendados": "refrendar",
    "refrendamos": "refrendar",
    "refrendan": "refrendar",
    "refrendando": "refrendar",
    "refrendara": "refrendar",
    "refrendarais": "refrendar",
    "refrendaran": "refrendar",
    "refrendaras": "refrendar",
    "refrendare": "refrendar",
    "refrendareis": "refrendar",
    "refrendaremos": "refrendar",
    "refrendaren": "refrendar",
    "refrendares": "refrendar",
    "refrendaria": "refrendario",
    "refrendarias": "refrendario",
    "refrendarios": "refrendario",
    "refrendaron": "refrendar",
    "refrendará": "refrendar",
    "refrendarán": "refrendar",
    "refrendarás": "refrendar",
    "refrendaré": "refrendar",
    "refrendaréis": "refrendar",
    "refrendaría": "refrendar",
    "refrendaríais": "refrendar",
    "refrendaríamos": "refrendar",
    "refrendarían": "refrendar",
    "refrendarías": "refrendar",
    "refrendas": "refrendar",
    "refrendase": "refrendar",
    "refrendaseis": "refrendar",
    "refrendasen": "refrendar",
    "refrendases": "refrendar",
    "refrendaste": "refrendar",
    "refrendasteis": "refrendar",
    "refrendatas": "refrendata",
    "refrende": "refrendar",
    "refrendemos": "refrendar",
    "refrenden": "refrendar",
    "refrendes": "refrendar",
    "refrendo": "refrendar",
    "refrendos": "refrendo",
    "refrendábamos": "refrendar",
    "refrendáis": "refrendar",
    "refrendáramos": "refrendar",
    "refrendáremos": "refrendar",
    "refrendásemos": "refrendar",
    "refrendé": "refrendar",
    "refrendéis": "refrendar",
    "refrendó": "refrendar",
    "refrene": "refrenar",
    "refrenemos": "refrenar",
    "refrenen": "refrenar",
    "refrenes": "refrenar",
    "refreno": "refrenar",
    "refrentada": "refrentado",
    "refrentadas": "refrentado",
    "refrentados": "refrentado",
    "refrenábamos": "refrenar",
    "refrenáis": "refrenar",
    "refrenáramos": "refrenar",
    "refrenáremos": "refrenar",
    "refrenásemos": "refrenar",
    "refrené": "refrenar",
    "refrenéis": "refrenar",
    "refrenó": "refrenar",
    "refresca": "refrescar",
    "refrescaba": "refrescar",
    "refrescabais": "refrescar",
    "refrescaban": "refrescar",
    "refrescabas": "refrescar",
    "refrescad": "refrescar",
    "refrescada": "refrescar",
    "refrescadas": "refrescar",
    "refrescado": "refrescar",
    "refrescadora": "refrescador",
    "refrescadoras": "refrescador",
    "refrescadores": "refrescador",
    "refrescados": "refrescar",
    "refrescaduras": "refrescadura",
    "refrescamientos": "refrescamiento",
    "refrescamos": "refrescar",
    "refrescan": "refrescar",
    "refrescando": "refrescar",
    "refrescantes": "refrescante",
    "refrescara": "refrescar",
    "refrescarais": "refrescar",
    "refrescaran": "refrescar",
    "refrescaras": "refrescar",
    "refrescare": "refrescar",
    "refrescareis": "refrescar",
    "refrescaremos": "refrescar",
    "refrescaren": "refrescar",
    "refrescares": "refrescar",
    "refrescaron": "refrescar",
    "refrescará": "refrescar",
    "refrescarán": "refrescar",
    "refrescarás": "refrescar",
    "refrescaré": "refrescar",
    "refrescaréis": "refrescar",
    "refrescaría": "refrescar",
    "refrescaríais": "refrescar",
    "refrescaríamos": "refrescar",
    "refrescarían": "refrescar",
    "refrescarías": "refrescar",
    "refrescas": "refrescar",
    "refrescase": "refrescar",
    "refrescaseis": "refrescar",
    "refrescasen": "refrescar",
    "refrescases": "refrescar",
    "refrescaste": "refrescar",
    "refrescasteis": "refrescar",
    "refresco": "refrescar",
    "refrescos": "refresco",
    "refrescábamos": "refrescar",
    "refrescáis": "refrescar",
    "refrescáramos": "refrescar",
    "refrescáremos": "refrescar",
    "refrescásemos": "refrescar",
    "refrescó": "refrescar",
    "refresque": "refrescar",
    "refresquemos": "refrescar",
    "refresquen": "refrescar",
    "refresques": "refrescar",
    "refresqué": "refrescar",
    "refresquéis": "refrescar",
    "refreí": "refreír",
    "refreía": "refreír",
    "refreíais": "refreír",
    "refreíamos": "refreír",
    "refreían": "refreír",
    "refreías": "refreír",
    "refreíd": "refreír",
    "refreída": "refreír",
    "refreídas": "refreír",
    "refreído": "refreír",
    "refreídos": "refreír",
    "refreímos": "refreír",
    "refreís": "refreír",
    "refreíste": "refreír",
    "refreísteis": "refreír",
    "refriamos": "refreír",
    "refriega": "refregar",
    "refriegan": "refregar",
    "refriegas": "refriega",
    "refriego": "refregar",
    "refriegue": "refregar",
    "refrieguen": "refregar",
    "refriegues": "refregar",
    "refriendo": "refreír",
    "refriera": "refreír",
    "refrierais": "refreír",
    "refrieran": "refreír",
    "refrieras": "refreír",
    "refriere": "refreír",
    "refriereis": "refreír",
    "refrieren": "refreír",
    "refrieres": "refreír",
    "refrieron": "refreír",
    "refriese": "refreír",
    "refrieseis": "refreír",
    "refriesen": "refreír",
    "refrieses": "refreír",
    "refrigera": "refrigerar",
    "refrigeraba": "refrigerar",
    "refrigerabais": "refrigerar",
    "refrigeraban": "refrigerar",
    "refrigerabas": "refrigerar",
    "refrigerables": "refrigerable",
    "refrigeraciones": "refrigeración",
    "refrigerad": "refrigerar",
    "refrigerada": "refrigerar",
    "refrigeradas": "refrigerar",
    "refrigerado": "refrigerar",
    "refrigeradora": "refrigerador",
    "refrigeradoras": "refrigerador",
    "refrigeradores": "refrigerador",
    "refrigerados": "refrigerar",
    "refrigeramos": "refrigerar",
    "refrigeran": "refrigerar",
    "refrigerando": "refrigerar",
    "refrigerantes": "refrigerante",
    "refrigerara": "refrigerar",
    "refrigerarais": "refrigerar",
    "refrigeraran": "refrigerar",
    "refrigeraras": "refrigerar",
    "refrigerare": "refrigerar",
    "refrigerareis": "refrigerar",
    "refrigeraremos": "refrigerar",
    "refrigeraren": "refrigerar",
    "refrigerares": "refrigerar",
    "refrigeraron": "refrigerar",
    "refrigerará": "refrigerar",
    "refrigerarán": "refrigerar",
    "refrigerarás": "refrigerar",
    "refrigeraré": "refrigerar",
    "refrigeraréis": "refrigerar",
    "refrigeraría": "refrigerar",
    "refrigeraríais": "refrigerar",
    "refrigeraríamos": "refrigerar",
    "refrigerarían": "refrigerar",
    "refrigerarías": "refrigerar",
    "refrigeras": "refrigerar",
    "refrigerase": "refrigerar",
    "refrigeraseis": "refrigerar",
    "refrigerasen": "refrigerar",
    "refrigerases": "refrigerar",
    "refrigeraste": "refrigerar",
    "refrigerasteis": "refrigerar",
    "refrigerativa": "refrigerativo",
    "refrigerativas": "refrigerativo",
    "refrigerativos": "refrigerativo",
    "refrigere": "refrigerar",
    "refrigeremos": "refrigerar",
    "refrigeren": "refrigerar",
    "refrigeres": "refrigerar",
    "refrigerios": "refrigerio",
    "refrigero": "refrigerar",
    "refrigerábamos": "refrigerar",
    "refrigeráis": "refrigerar",
    "refrigeráramos": "refrigerar",
    "refrigeráremos": "refrigerar",
    "refrigerásemos": "refrigerar",
    "refrigeré": "refrigerar",
    "refrigeréis": "refrigerar",
    "refrigeró": "refrigerar",
    "refringe": "refringir",
    "refringen": "refringir",
    "refringencias": "refringencia",
    "refringentes": "refringente",
    "refringes": "refringir",
    "refringid": "refringir",
    "refringida": "refringir",
    "refringidas": "refringir",
    "refringido": "refringir",
    "refringidos": "refringir",
    "refringiendo": "refringir",
    "refringiera": "refringir",
    "refringierais": "refringir",
    "refringieran": "refringir",
    "refringieras": "refringir",
    "refringiere": "refringir",
    "refringiereis": "refringir",
    "refringieren": "refringir",
    "refringieres": "refringir",
    "refringieron": "refringir",
    "refringiese": "refringir",
    "refringieseis": "refringir",
    "refringiesen": "refringir",
    "refringieses": "refringir",
    "refringimos": "refringir",
    "refringiremos": "refringir",
    "refringirá": "refringir",
    "refringirán": "refringir",
    "refringirás": "refringir",
    "refringiré": "refringir",
    "refringiréis": "refringir",
    "refringiría": "refringir",
    "refringiríais": "refringir",
    "refringiríamos": "refringir",
    "refringirían": "refringir",
    "refringirías": "refringir",
    "refringiste": "refringir",
    "refringisteis": "refringir",
    "refringiéramos": "refringir",
    "refringiéremos": "refringir",
    "refringiésemos": "refringir",
    "refringió": "refringir",
    "refringí": "refringir",
    "refringía": "refringir",
    "refringíais": "refringir",
    "refringíamos": "refringir",
    "refringían": "refringir",
    "refringías": "refringir",
    "refringís": "refringir",
    "refrinja": "refringir",
    "refrinjamos": "refringir",
    "refrinjan": "refringir",
    "refrinjas": "refringir",
    "refrinjo": "refringir",
    "refrinjáis": "refringir",
    "refrita": "refreír",
    "refritas": "refreír",
    "refrito": "refreír",
    "refritos": "refrito",
    "refriáis": "refreír",
    "refriéramos": "refreír",
    "refriéremos": "refreír",
    "refriésemos": "refreír",
    "refrió": "refreír",
    "refría": "refreír",
    "refrían": "refreír",
    "refrías": "refreír",
    "refríe": "refreír",
    "refríen": "refreír",
    "refríes": "refreír",
    "refrío": "refreír",
    "refucilos": "refucilo",
    "refuerce": "reforzar",
    "refuercen": "reforzar",
    "refuerces": "reforzar",
    "refuerza": "reforzar",
    "refuerzan": "reforzar",
    "refuerzas": "reforzar",
    "refuerzo": "reforzar",
    "refuerzos": "refuerzo",
    "refugia": "refugiar",
    "refugiaba": "refugiar",
    "refugiabais": "refugiar",
    "refugiaban": "refugiar",
    "refugiabas": "refugiar",
    "refugiad": "refugiar",
    "refugiada": "refugiar",
    "refugiadas": "refugiar",
    "refugiado": "refugiar",
    "refugiados": "refugiar",
    "refugiamos": "refugiar",
    "refugian": "refugiar",
    "refugiando": "refugiar",
    "refugiara": "refugiar",
    "refugiarais": "refugiar",
    "refugiaran": "refugiar",
    "refugiaras": "refugiar",
    "refugiare": "refugiar",
    "refugiareis": "refugiar",
    "refugiaremos": "refugiar",
    "refugiaren": "refugiar",
    "refugiares": "refugiar",
    "refugiaron": "refugiar",
    "refugiará": "refugiar",
    "refugiarán": "refugiar",
    "refugiarás": "refugiar",
    "refugiaré": "refugiar",
    "refugiaréis": "refugiar",
    "refugiaría": "refugiar",
    "refugiaríais": "refugiar",
    "refugiaríamos": "refugiar",
    "refugiarían": "refugiar",
    "refugiarías": "refugiar",
    "refugias": "refugiar",
    "refugiase": "refugiar",
    "refugiaseis": "refugiar",
    "refugiasen": "refugiar",
    "refugiases": "refugiar",
    "refugiaste": "refugiar",
    "refugiasteis": "refugiar",
    "refugie": "refugiar",
    "refugiemos": "refugiar",
    "refugien": "refugiar",
    "refugies": "refugiar",
    "refugio": "refugiar",
    "refugios": "refugio",
    "refugiábamos": "refugiar",
    "refugiáis": "refugiar",
    "refugiáramos": "refugiar",
    "refugiáremos": "refugiar",
    "refugiásemos": "refugiar",
    "refugié": "refugiar",
    "refugiéis": "refugiar",
    "refugió": "refugiar",
    "refulge": "refulgir",
    "refulgen": "refulgir",
    "refulgencias": "refulgencia",
    "refulgentes": "refulgente",
    "refulges": "refulgir",
    "refulgid": "refulgir",
    "refulgida": "refulgir",
    "refulgidas": "refulgir",
    "refulgido": "refulgir",
    "refulgidos": "refulgir",
    "refulgiendo": "refulgir",
    "refulgiera": "refulgir",
    "refulgierais": "refulgir",
    "refulgieran": "refulgir",
    "refulgieras": "refulgir",
    "refulgiere": "refulgir",
    "refulgiereis": "refulgir",
    "refulgieren": "refulgir",
    "refulgieres": "refulgir",
    "refulgieron": "refulgir",
    "refulgiese": "refulgir",
    "refulgieseis": "refulgir",
    "refulgiesen": "refulgir",
    "refulgieses": "refulgir",
    "refulgimos": "refulgir",
    "refulgiremos": "refulgir",
    "refulgirá": "refulgir",
    "refulgirán": "refulgir",
    "refulgirás": "refulgir",
    "refulgiré": "refulgir",
    "refulgiréis": "refulgir",
    "refulgiría": "refulgir",
    "refulgiríais": "refulgir",
    "refulgiríamos": "refulgir",
    "refulgirían": "refulgir",
    "refulgirías": "refulgir",
    "refulgiste": "refulgir",
    "refulgisteis": "refulgir",
    "refulgiéramos": "refulgir",
    "refulgiéremos": "refulgir",
    "refulgiésemos": "refulgir",
    "refulgió": "refulgir",
    "refulgí": "refulgir",
    "refulgía": "refulgir",
    "refulgíais": "refulgir",
    "refulgíamos": "refulgir",
    "refulgían": "refulgir",
    "refulgías": "refulgir",
    "refulgís": "refulgir",
    "refulja": "refulgir",
    "refuljamos": "refulgir",
    "refuljan": "refulgir",
    "refuljas": "refulgir",
    "refuljo": "refulgir",
    "refuljáis": "refulgir",
    "refunda": "refundir",
    "refundaba": "refundar",
    "refundabais": "refundar",
    "refundaban": "refundar",
    "refundabas": "refundar",
    "refundad": "refundar",
    "refundada": "refundar",
    "refundadas": "refundar",
    "refundado": "refundar",
    "refundados": "refundar",
    "refundamos": "refundir",
    "refundan": "refundir",
    "refundando": "refundar",
    "refundara": "refundar",
    "refundarais": "refundar",
    "refundaran": "refundar",
    "refundaras": "refundar",
    "refundare": "refundar",
    "refundareis": "refundar",
    "refundaremos": "refundar",
    "refundaren": "refundar",
    "refundares": "refundar",
    "refundaron": "refundar",
    "refundará": "refundar",
    "refundarán": "refundar",
    "refundarás": "refundar",
    "refundaré": "refundar",
    "refundaréis": "refundar",
    "refundaría": "refundar",
    "refundaríais": "refundar",
    "refundaríamos": "refundar",
    "refundarían": "refundar",
    "refundarías": "refundar",
    "refundas": "refundir",
    "refundase": "refundar",
    "refundaseis": "refundar",
    "refundasen": "refundar",
    "refundases": "refundar",
    "refundaste": "refundar",
    "refundasteis": "refundar",
    "refunde": "refundir",
    "refundemos": "refundar",
    "refunden": "refundir",
    "refundes": "refundir",
    "refundiciones": "refundición",
    "refundid": "refundir",
    "refundida": "refundir",
    "refundidas": "refundir",
    "refundido": "refundir",
    "refundidos": "refundir",
    "refundiendo": "refundir",
    "refundiera": "refundir",
    "refundierais": "refundir",
    "refundieran": "refundir",
    "refundieras": "refundir",
    "refundiere": "refundir",
    "refundiereis": "refundir",
    "refundieren": "refundir",
    "refundieres": "refundir",
    "refundieron": "refundir",
    "refundiese": "refundir",
    "refundieseis": "refundir",
    "refundiesen": "refundir",
    "refundieses": "refundir",
    "refundimos": "refundir",
    "refundiremos": "refundir",
    "refundirá": "refundir",
    "refundirán": "refundir",
    "refundirás": "refundir",
    "refundiré": "refundir",
    "refundiréis": "refundir",
    "refundiría": "refundir",
    "refundiríais": "refundir",
    "refundiríamos": "refundir",
    "refundirían": "refundir",
    "refundirías": "refundir",
    "refundiste": "refundir",
    "refundisteis": "refundir",
    "refundiéramos": "refundir",
    "refundiéremos": "refundir",
    "refundiésemos": "refundir",
    "refundió": "refundir",
    "refundo": "refundir",
    "refundábamos": "refundar",
    "refundáis": "refundir",
    "refundáramos": "refundar",
    "refundáremos": "refundar",
    "refundásemos": "refundar",
    "refundé": "refundar",
    "refundéis": "refundar",
    "refundí": "refundir",
    "refundía": "refundir",
    "refundíais": "refundir",
    "refundíamos": "refundir",
    "refundían": "refundir",
    "refundías": "refundir",
    "refundís": "refundir",
    "refundó": "refundar",
    "refunfuña": "refunfuñar",
    "refunfuñaba": "refunfuñar",
    "refunfuñabais": "refunfuñar",
    "refunfuñaban": "refunfuñar",
    "refunfuñabas": "refunfuñar",
    "refunfuñad": "refunfuñar",
    "refunfuñada": "refunfuñar",
    "refunfuñadas": "refunfuñar",
    "refunfuñado": "refunfuñar",
    "refunfuñadora": "refunfuñador",
    "refunfuñadoras": "refunfuñador",
    "refunfuñadores": "refunfuñador",
    "refunfuñados": "refunfuñar",
    "refunfuñaduras": "refunfuñadura",
    "refunfuñamos": "refunfuñar",
    "refunfuñan": "refunfuñar",
    "refunfuñando": "refunfuñar",
    "refunfuñara": "refunfuñar",
    "refunfuñarais": "refunfuñar",
    "refunfuñaran": "refunfuñar",
    "refunfuñaras": "refunfuñar",
    "refunfuñare": "refunfuñar",
    "refunfuñareis": "refunfuñar",
    "refunfuñaremos": "refunfuñar",
    "refunfuñaren": "refunfuñar",
    "refunfuñares": "refunfuñar",
    "refunfuñaron": "refunfuñar",
    "refunfuñará": "refunfuñar",
    "refunfuñarán": "refunfuñar",
    "refunfuñarás": "refunfuñar",
    "refunfuñaré": "refunfuñar",
    "refunfuñaréis": "refunfuñar",
    "refunfuñaría": "refunfuñar",
    "refunfuñaríais": "refunfuñar",
    "refunfuñaríamos": "refunfuñar",
    "refunfuñarían": "refunfuñar",
    "refunfuñarías": "refunfuñar",
    "refunfuñas": "refunfuñar",
    "refunfuñase": "refunfuñar",
    "refunfuñaseis": "refunfuñar",
    "refunfuñasen": "refunfuñar",
    "refunfuñases": "refunfuñar",
    "refunfuñaste": "refunfuñar",
    "refunfuñasteis": "refunfuñar",
    "refunfuñe": "refunfuñar",
    "refunfuñemos": "refunfuñar",
    "refunfuñen": "refunfuñar",
    "refunfuñes": "refunfuñar",
    "refunfuño": "refunfuñar",
    "refunfuñona": "refunfuñón",
    "refunfuñonas": "refunfuñón",
    "refunfuñones": "refunfuñón",
    "refunfuños": "refunfuño",
    "refunfuñábamos": "refunfuñar",
    "refunfuñáis": "refunfuñar",
    "refunfuñáramos": "refunfuñar",
    "refunfuñáremos": "refunfuñar",
    "refunfuñásemos": "refunfuñar",
    "refunfuñé": "refunfuñar",
    "refunfuñéis": "refunfuñar",
    "refunfuñó": "refunfuñar",
    "refusiones": "refusión",
    "refuta": "refutar",
    "refutaba": "refutar",
    "refutabais": "refutar",
    "refutaban": "refutar",
    "refutabas": "refutar",
    "refutables": "refutable",
    "refutaciones": "refutación",
    "refutacionismos": "refutacionismo",
    "refutad": "refutar",
    "refutada": "refutar",
    "refutadas": "refutar",
    "refutado": "refutar",
    "refutados": "refutar",
    "refutamos": "refutar",
    "refutan": "refutar",
    "refutando": "refutar",
    "refutara": "refutar",
    "refutarais": "refutar",
    "refutaran": "refutar",
    "refutaras": "refutar",
    "refutare": "refutar",
    "refutareis": "refutar",
    "refutaremos": "refutar",
    "refutaren": "refutar",
    "refutares": "refutar",
    "refutaron": "refutar",
    "refutará": "refutar",
    "refutarán": "refutar",
    "refutarás": "refutar",
    "refutaré": "refutar",
    "refutaréis": "refutar",
    "refutaría": "refutar",
    "refutaríais": "refutar",
    "refutaríamos": "refutar",
    "refutarían": "refutar",
    "refutarías": "refutar",
    "refutas": "refutar",
    "refutase": "refutar",
    "refutaseis": "refutar",
    "refutasen": "refutar",
    "refutases": "refutar",
    "refutaste": "refutar",
    "refutasteis": "refutar",
    "refutatoria": "refutatorio",
    "refutatorias": "refutatorio",
    "refutatorios": "refutatorio",
    "refute": "refutar",
    "refutemos": "refutar",
    "refuten": "refutar",
    "refutes": "refutar",
    "refuto": "refutar",
    "refutábamos": "refutar",
    "refutáis": "refutar",
    "refutáramos": "refutar",
    "refutáremos": "refutar",
    "refutásemos": "refutar",
    "refuté": "refutar",
    "refutéis": "refutar",
    "refutó": "refutar",
    "regaba": "regar",
    "regabais": "regar",
    "regaban": "regar",
    "regabas": "regar",
    "regables": "regable",
    "regad": "regar",
    "regada": "regar",
    "regadas": "regar",
    "regaderas": "regadera",
    "regado": "regar",
    "regadora": "regador",
    "regadoras": "regadora",
    "regadores": "regador",
    "regados": "regar",
    "regaduras": "regadura",
    "regadía": "regadío",
    "regadías": "regadío",
    "regadíos": "regadío",
    "regaifas": "regaifa",
    "regajales": "regajal",
    "regajos": "regajo",
    "regala": "regalar",
    "regalaba": "regalar",
    "regalabais": "regalar",
    "regalaban": "regalar",
    "regalabas": "regalar",
    "regalad": "regalar",
    "regalada": "regalar",
    "regaladas": "regalar",
    "regalado": "regalar",
    "regaladora": "regalador",
    "regaladoras": "regalador",
    "regaladores": "regalador",
    "regalados": "regalar",
    "regalamos": "regalar",
    "regalan": "regalar",
    "regalando": "regalar",
    "regalara": "regalar",
    "regalarais": "regalar",
    "regalaran": "regalar",
    "regalaras": "regalar",
    "regalare": "regalar",
    "regalareis": "regalar",
    "regalaremos": "regalar",
    "regalaren": "regalar",
    "regalares": "regalar",
    "regalaron": "regalar",
    "regalará": "regalar",
    "regalarán": "regalar",
    "regalarás": "regalar",
    "regalaré": "regalar",
    "regalaréis": "regalar",
    "regalaría": "regalar",
    "regalaríais": "regalar",
    "regalaríamos": "regalar",
    "regalarían": "regalar",
    "regalarías": "regalar",
    "regalas": "regalar",
    "regalase": "regalar",
    "regalaseis": "regalar",
    "regalasen": "regalar",
    "regalases": "regalar",
    "regalaste": "regalar",
    "regalasteis": "regalar",
    "regale": "regalar",
    "regalemos": "regalar",
    "regalen": "regalar",
    "regales": "regalar",
    "regalices": "regaliz",
    "regalicias": "regalicia",
    "regalismos": "regalismo",
    "regalistas": "regalista",
    "regalitos": "regalito",
    "regalizas": "regaliza",
    "regalo": "regalar",
    "regalos": "regalo",
    "regalábamos": "regalar",
    "regaláis": "regalar",
    "regaláramos": "regalar",
    "regaláremos": "regalar",
    "regalásemos": "regalar",
    "regalé": "regalar",
    "regaléis": "regalar",
    "regalías": "regalía",
    "regaló": "regalar",
    "regamos": "regar",
    "regando": "regar",
    "regantes": "regante",
    "regara": "regar",
    "regarais": "regar",
    "regaran": "regar",
    "regaras": "regar",
    "regare": "regar",
    "regareis": "regar",
    "regaremos": "regar",
    "regaren": "regar",
    "regares": "regar",
    "regaron": "regar",
    "regará": "regar",
    "regarán": "regar",
    "regarás": "regar",
    "regaré": "regar",
    "regaréis": "regar",
    "regaría": "regar",
    "regaríais": "regar",
    "regaríamos": "regar",
    "regarían": "regar",
    "regarías": "regar",
    "regase": "regar",
    "regaseis": "regar",
    "regasen": "regar",
    "regases": "regar",
    "regasificaciones": "regasificación",
    "regaste": "regar",
    "regasteis": "regar",
    "regatas": "regata",
    "regatea": "regatear",
    "regateaba": "regatear",
    "regateabais": "regatear",
    "regateaban": "regatear",
    "regateabas": "regatear",
    "regatead": "regatear",
    "regateada": "regatear",
    "regateadas": "regatear",
    "regateado": "regatear",
    "regateadora": "regateador",
    "regateadoras": "regateador",
    "regateadores": "regateador",
    "regateados": "regatear",
    "regateamos": "regatear",
    "regatean": "regatear",
    "regateando": "regatear",
    "regateara": "regatear",
    "regatearais": "regatear",
    "regatearan": "regatear",
    "regatearas": "regatear",
    "regateare": "regatear",
    "regateareis": "regatear",
    "regatearemos": "regatear",
    "regatearen": "regatear",
    "regateares": "regatear",
    "regatearon": "regatear",
    "regateará": "regatear",
    "regatearán": "regatear",
    "regatearás": "regatear",
    "regatearé": "regatear",
    "regatearéis": "regatear",
    "regatearía": "regatear",
    "regatearíais": "regatear",
    "regatearíamos": "regatear",
    "regatearían": "regatear",
    "regatearías": "regatear",
    "regateas": "regatear",
    "regatease": "regatear",
    "regateaseis": "regatear",
    "regateasen": "regatear",
    "regateases": "regatear",
    "regateaste": "regatear",
    "regateasteis": "regatear",
    "regatee": "regatear",
    "regateemos": "regatear",
    "regateen": "regatear",
    "regatees": "regatear",
    "regateo": "regatear",
    "regateos": "regateo",
    "regates": "regate",
    "regateábamos": "regatear",
    "regateáis": "regatear",
    "regateáramos": "regatear",
    "regateáremos": "regatear",
    "regateásemos": "regatear",
    "regateé": "regatear",
    "regateéis": "regatear",
    "regateó": "regatear",
    "regatistas": "regatista",
    "regatona": "regatón",
    "regatonas": "regatón",
    "regatones": "regatón",
    "regatos": "regato",
    "regazos": "regazo",
    "regaña": "regañar",
    "regañaba": "regañar",
    "regañabais": "regañar",
    "regañaban": "regañar",
    "regañabas": "regañar",
    "regañad": "regañar",
    "regañada": "regañar",
    "regañadas": "regañar",
    "regañado": "regañar",
    "regañados": "regañar",
    "regañamos": "regañar",
    "regañan": "regañar",
    "regañando": "regañar",
    "regañara": "regañar",
    "regañarais": "regañar",
    "regañaran": "regañar",
    "regañaras": "regañar",
    "regañare": "regañar",
    "regañareis": "regañar",
    "regañaremos": "regañar",
    "regañaren": "regañar",
    "regañares": "regañar",
    "regañaron": "regañar",
    "regañará": "regañar",
    "regañarán": "regañar",
    "regañarás": "regañar",
    "regañaré": "regañar",
    "regañaréis": "regañar",
    "regañaría": "regañar",
    "regañaríais": "regañar",
    "regañaríamos": "regañar",
    "regañarían": "regañar",
    "regañarías": "regañar",
    "regañas": "regañar",
    "regañase": "regañar",
    "regañaseis": "regañar",
    "regañasen": "regañar",
    "regañases": "regañar",
    "regañaste": "regañar",
    "regañasteis": "regañar",
    "regañe": "regañar",
    "regañemos": "regañar",
    "regañen": "regañar",
    "regañeras": "regañera",
    "regañes": "regañar",
    "regañinas": "regañina",
    "regaño": "regañar",
    "regañona": "regañón",
    "regañonas": "regañón",
    "regañones": "regañón",
    "regaños": "regaño",
    "regañábamos": "regañar",
    "regañáis": "regañar",
    "regañáramos": "regañar",
    "regañáremos": "regañar",
    "regañásemos": "regañar",
    "regañé": "regañar",
    "regañéis": "regañar",
    "regañó": "regañar",
    "regencias": "regencia",
    "regenera": "regenerar",
    "regeneraba": "regenerar",
    "regenerabais": "regenerar",
    "regeneraban": "regenerar",
    "regenerabas": "regenerar",
    "regeneraciones": "regeneración",
    "regeneracionismos": "regeneracionismo",
    "regeneracionistas": "regeneracionista",
    "regenerad": "regenerar",
    "regenerada": "regenerar",
    "regeneradas": "regenerar",
    "regenerado": "regenerar",
    "regeneradora": "regenerador",
    "regeneradoras": "regenerador",
    "regeneradores": "regenerador",
    "regenerados": "regenerar",
    "regeneramos": "regenerar",
    "regeneran": "regenerar",
    "regenerando": "regenerar",
    "regenerara": "regenerar",
    "regenerarais": "regenerar",
    "regeneraran": "regenerar",
    "regeneraras": "regenerar",
    "regenerare": "regenerar",
    "regenerareis": "regenerar",
    "regeneraremos": "regenerar",
    "regeneraren": "regenerar",
    "regenerares": "regenerar",
    "regeneraron": "regenerar",
    "regenerará": "regenerar",
    "regenerarán": "regenerar",
    "regenerarás": "regenerar",
    "regeneraré": "regenerar",
    "regeneraréis": "regenerar",
    "regeneraría": "regenerar",
    "regeneraríais": "regenerar",
    "regeneraríamos": "regenerar",
    "regenerarían": "regenerar",
    "regenerarías": "regenerar",
    "regeneras": "regenerar",
    "regenerase": "regenerar",
    "regeneraseis": "regenerar",
    "regenerasen": "regenerar",
    "regenerases": "regenerar",
    "regeneraste": "regenerar",
    "regenerasteis": "regenerar",
    "regenerativa": "regenerativo",
    "regenerativas": "regenerativo",
    "regenerativos": "regenerativo",
    "regenere": "regenerar",
    "regeneremos": "regenerar",
    "regeneren": "regenerar",
    "regeneres": "regenerar",
    "regenero": "regenerar",
    "regenerábamos": "regenerar",
    "regeneráis": "regenerar",
    "regeneráramos": "regenerar",
    "regeneráremos": "regenerar",
    "regenerásemos": "regenerar",
    "regeneré": "regenerar",
    "regeneréis": "regenerar",
    "regeneró": "regenerar",
    "regenta": "regentar",
    "regentaba": "regentar",
    "regentabais": "regentar",
    "regentaban": "regentar",
    "regentabas": "regentar",
    "regentad": "regentar",
    "regentada": "regentar",
    "regentadas": "regentar",
    "regentado": "regentar",
    "regentados": "regentar",
    "regentamos": "regentar",
    "regentan": "regentar",
    "regentando": "regentar",
    "regentara": "regentar",
    "regentarais": "regentar",
    "regentaran": "regentar",
    "regentaras": "regentar",
    "regentare": "regentar",
    "regentareis": "regentar",
    "regentaremos": "regentar",
    "regentaren": "regentar",
    "regentares": "regentar",
    "regentaron": "regentar",
    "regentará": "regentar",
    "regentarán": "regentar",
    "regentarás": "regentar",
    "regentaré": "regentar",
    "regentaréis": "regentar",
    "regentaría": "regentar",
    "regentaríais": "regentar",
    "regentaríamos": "regentar",
    "regentarían": "regentar",
    "regentarías": "regentar",
    "regentas": "regentar",
    "regentase": "regentar",
    "regentaseis": "regentar",
    "regentasen": "regentar",
    "regentases": "regentar",
    "regentaste": "regentar",
    "regentasteis": "regentar",
    "regente": "regentar",
    "regentea": "regentear",
    "regenteaba": "regentear",
    "regenteabais": "regentear",
    "regenteaban": "regentear",
    "regenteabas": "regentear",
    "regentead": "regentear",
    "regenteada": "regentear",
    "regenteadas": "regentear",
    "regenteado": "regentear",
    "regenteados": "regentear",
    "regenteamos": "regentear",
    "regentean": "regentear",
    "regenteando": "regentear",
    "regenteara": "regentear",
    "regentearais": "regentear",
    "regentearan": "regentear",
    "regentearas": "regentear",
    "regenteare": "regentear",
    "regenteareis": "regentear",
    "regentearemos": "regentear",
    "regentearen": "regentear",
    "regenteares": "regentear",
    "regentearon": "regentear",
    "regenteará": "regentear",
    "regentearán": "regentear",
    "regentearás": "regentear",
    "regentearé": "regentear",
    "regentearéis": "regentear",
    "regentearía": "regentear",
    "regentearíais": "regentear",
    "regentearíamos": "regentear",
    "regentearían": "regentear",
    "regentearías": "regentear",
    "regenteas": "regentear",
    "regentease": "regentear",
    "regenteaseis": "regentear",
    "regenteasen": "regentear",
    "regenteases": "regentear",
    "regenteaste": "regentear",
    "regenteasteis": "regentear",
    "regentee": "regentear",
    "regenteemos": "regentear",
    "regenteen": "regentear",
    "regentees": "regentear",
    "regentemos": "regentar",
    "regenten": "regentar",
    "regenteo": "regentear",
    "regentes": "regente",
    "regenteábamos": "regentear",
    "regenteáis": "regentear",
    "regenteáramos": "regentear",
    "regenteáremos": "regentear",
    "regenteásemos": "regentear",
    "regenteé": "regentear",
    "regenteéis": "regentear",
    "regenteó": "regentear",
    "regento": "regentar",
    "regentábamos": "regentar",
    "regentáis": "regentar",
    "regentáramos": "regentar",
    "regentáremos": "regentar",
    "regentásemos": "regentar",
    "regenté": "regentar",
    "regentéis": "regentar",
    "regentó": "regentar",
    "reggaes": "reggae",
    "regia": "regio",
    "regias": "regio",
    "regicidas": "regicida",
    "regicidios": "regicidio",
    "regid": "regir",
    "regida": "regir",
    "regidas": "regir",
    "regido": "regir",
    "regidora": "regidor",
    "regidoras": "regidor",
    "regidores": "regidor",
    "regidorías": "regidoría",
    "regidos": "regir",
    "regidurías": "regiduría",
    "regimentaba": "regimentar",
    "regimentabais": "regimentar",
    "regimentaban": "regimentar",
    "regimentabas": "regimentar",
    "regimentad": "regimentar",
    "regimentada": "regimentar",
    "regimentadas": "regimentar",
    "regimentado": "regimentar",
    "regimentados": "regimentar",
    "regimentales": "regimental",
    "regimentamos": "regimentar",
    "regimentando": "regimentar",
    "regimentara": "regimentar",
    "regimentarais": "regimentar",
    "regimentaran": "regimentar",
    "regimentaras": "regimentar",
    "regimentare": "regimentar",
    "regimentareis": "regimentar",
    "regimentaremos": "regimentar",
    "regimentaren": "regimentar",
    "regimentares": "regimentar",
    "regimentaron": "regimentar",
    "regimentará": "regimentar",
    "regimentarán": "regimentar",
    "regimentarás": "regimentar",
    "regimentaré": "regimentar",
    "regimentaréis": "regimentar",
    "regimentaría": "regimentar",
    "regimentaríais": "regimentar",
    "regimentaríamos": "regimentar",
    "regimentarían": "regimentar",
    "regimentarías": "regimentar",
    "regimentase": "regimentar",
    "regimentaseis": "regimentar",
    "regimentasen": "regimentar",
    "regimentases": "regimentar",
    "regimentaste": "regimentar",
    "regimentasteis": "regimentar",
    "regimentemos": "regimentar",
    "regimentábamos": "regimentar",
    "regimentáis": "regimentar",
    "regimentáramos": "regimentar",
    "regimentáremos": "regimentar",
    "regimentásemos": "regimentar",
    "regimenté": "regimentar",
    "regimentéis": "regimentar",
    "regimentó": "regimentar",
    "regimienta": "regimentar",
    "regimientan": "regimentar",
    "regimientas": "regimentar",
    "regimiente": "regimentar",
    "regimienten": "regimentar",
    "regimientes": "regimentar",
    "regimiento": "regimentar",
    "regimientos": "regimiento",
    "regimos": "regir",
    "regionales": "regional",
    "regionalice": "regionalizar",
    "regionalicemos": "regionalizar",
    "regionalicen": "regionalizar",
    "regionalices": "regionalizar",
    "regionalicé": "regionalizar",
    "regionalicéis": "regionalizar",
    "regionalidades": "regionalidad",
    "regionalismos": "regionalismo",
    "regionalistas": "regionalista",
    "regionaliza": "regionalizar",
    "regionalizaba": "regionalizar",
    "regionalizabais": "regionalizar",
    "regionalizaban": "regionalizar",
    "regionalizabas": "regionalizar",
    "regionalizaciones": "regionalización",
    "regionalizad": "regionalizar",
    "regionalizada": "regionalizar",
    "regionalizadas": "regionalizar",
    "regionalizado": "regionalizar",
    "regionalizados": "regionalizar",
    "regionalizamos": "regionalizar",
    "regionalizan": "regionalizar",
    "regionalizando": "regionalizar",
    "regionalizara": "regionalizar",
    "regionalizarais": "regionalizar",
    "regionalizaran": "regionalizar",
    "regionalizaras": "regionalizar",
    "regionalizare": "regionalizar",
    "regionalizareis": "regionalizar",
    "regionalizaremos": "regionalizar",
    "regionalizaren": "regionalizar",
    "regionalizares": "regionalizar",
    "regionalizaron": "regionalizar",
    "regionalizará": "regionalizar",
    "regionalizarán": "regionalizar",
    "regionalizarás": "regionalizar",
    "regionalizaré": "regionalizar",
    "regionalizaréis": "regionalizar",
    "regionalizaría": "regionalizar",
    "regionalizaríais": "regionalizar",
    "regionalizaríamos": "regionalizar",
    "regionalizarían": "regionalizar",
    "regionalizarías": "regionalizar",
    "regionalizas": "regionalizar",
    "regionalizase": "regionalizar",
    "regionalizaseis": "regionalizar",
    "regionalizasen": "regionalizar",
    "regionalizases": "regionalizar",
    "regionalizaste": "regionalizar",
    "regionalizasteis": "regionalizar",
    "regionalizo": "regionalizar",
    "regionalizábamos": "regionalizar",
    "regionalizáis": "regionalizar",
    "regionalizáramos": "regionalizar",
    "regionalizáremos": "regionalizar",
    "regionalizásemos": "regionalizar",
    "regionalizó": "regionalizar",
    "regiones": "región",
    "regios": "regio",
    "regiremos": "regir",
    "regirá": "regir",
    "regirán": "regir",
    "regirás": "regir",
    "regiré": "regir",
    "regiréis": "regir",
    "regiría": "regir",
    "regiríais": "regir",
    "regiríamos": "regir",
    "regirían": "regir",
    "regirías": "regir",
    "registe": "regir",
    "registeis": "regir",
    "registra": "registrar",
    "registraba": "registrar",
    "registrabais": "registrar",
    "registraban": "registrar",
    "registrabas": "registrar",
    "registrables": "registrable",
    "registrad": "registrar",
    "registrada": "registrar",
    "registradas": "registrar",
    "registrado": "registrar",
    "registradora": "registrador",
    "registradoras": "registrador",
    "registradores": "registrador",
    "registrados": "registrar",
    "registrales": "registral",
    "registramos": "registrar",
    "registran": "registrar",
    "registrando": "registrar",
    "registrantes": "registrante",
    "registrara": "registrar",
    "registrarais": "registrar",
    "registraran": "registrar",
    "registraras": "registrar",
    "registrare": "registrar",
    "registrareis": "registrar",
    "registraremos": "registrar",
    "registraren": "registrar",
    "registrares": "registrar",
    "registraron": "registrar",
    "registrará": "registrar",
    "registrarán": "registrar",
    "registrarás": "registrar",
    "registraré": "registrar",
    "registraréis": "registrar",
    "registraría": "registrar",
    "registraríais": "registrar",
    "registraríamos": "registrar",
    "registrarían": "registrar",
    "registrarías": "registrar",
    "registras": "registrar",
    "registrase": "registrar",
    "registraseis": "registrar",
    "registrasen": "registrar",
    "registrases": "registrar",
    "registraste": "registrar",
    "registrasteis": "registrar",
    "registre": "registrar",
    "registremos": "registrar",
    "registren": "registrar",
    "registres": "registrar",
    "registro": "registrar",
    "registros": "registro",
    "registrábamos": "registrar",
    "registráis": "registrar",
    "registráramos": "registrar",
    "registráremos": "registrar",
    "registrásemos": "registrar",
    "registré": "registrar",
    "registréis": "registrar",
    "registró": "registrar",
    "regla": "reglar",
    "reglaba": "reglar",
    "reglabais": "reglar",
    "reglaban": "reglar",
    "reglabas": "reglar",
    "reglables": "reglable",
    "reglad": "reglar",
    "reglada": "reglar",
    "regladas": "reglar",
    "reglado": "reglar",
    "reglados": "reglar",
    "reglajes": "reglaje",
    "reglamenta": "reglamentar",
    "reglamentaba": "reglamentar",
    "reglamentabais": "reglamentar",
    "reglamentaban": "reglamentar",
    "reglamentabas": "reglamentar",
    "reglamentaciones": "reglamentación",
    "reglamentad": "reglamentar",
    "reglamentada": "reglamentar",
    "reglamentadas": "reglamentar",
    "reglamentado": "reglamentar",
    "reglamentadora": "reglamentador",
    "reglamentadoras": "reglamentador",
    "reglamentadores": "reglamentador",
    "reglamentados": "reglamentar",
    "reglamentamos": "reglamentar",
    "reglamentan": "reglamentar",
    "reglamentando": "reglamentar",
    "reglamentara": "reglamentar",
    "reglamentarais": "reglamentar",
    "reglamentaran": "reglamentar",
    "reglamentaras": "reglamentar",
    "reglamentare": "reglamentar",
    "reglamentareis": "reglamentar",
    "reglamentaremos": "reglamentar",
    "reglamentaren": "reglamentar",
    "reglamentares": "reglamentar",
    "reglamentaria": "reglamentario",
    "reglamentarias": "reglamentario",
    "reglamentarios": "reglamentario",
    "reglamentaron": "reglamentar",
    "reglamentará": "reglamentar",
    "reglamentarán": "reglamentar",
    "reglamentarás": "reglamentar",
    "reglamentaré": "reglamentar",
    "reglamentaréis": "reglamentar",
    "reglamentaría": "reglamentar",
    "reglamentaríais": "reglamentar",
    "reglamentaríamos": "reglamentar",
    "reglamentarían": "reglamentar",
    "reglamentarías": "reglamentar",
    "reglamentas": "reglamentar",
    "reglamentase": "reglamentar",
    "reglamentaseis": "reglamentar",
    "reglamentasen": "reglamentar",
    "reglamentases": "reglamentar",
    "reglamentaste": "reglamentar",
    "reglamentasteis": "reglamentar",
    "reglamente": "reglamentar",
    "reglamentemos": "reglamentar",
    "reglamenten": "reglamentar",
    "reglamentes": "reglamentar",
    "reglamento": "reglamentar",
    "reglamentos": "reglamento",
    "reglamentábamos": "reglamentar",
    "reglamentáis": "reglamentar",
    "reglamentáramos": "reglamentar",
    "reglamentáremos": "reglamentar",
    "reglamentásemos": "reglamentar",
    "reglamenté": "reglamentar",
    "reglamentéis": "reglamentar",
    "reglamentó": "reglamentar",
    "reglamos": "reglar",
    "reglan": "reglar",
    "reglando": "reglar",
    "reglara": "reglar",
    "reglarais": "reglar",
    "reglaran": "reglar",
    "reglaras": "reglar",
    "reglare": "reglar",
    "reglareis": "reglar",
    "reglaremos": "reglar",
    "reglaren": "reglar",
    "reglares": "reglar",
    "reglaron": "reglar",
    "reglará": "reglar",
    "reglarán": "reglar",
    "reglarás": "reglar",
    "reglaré": "reglar",
    "reglaréis": "reglar",
    "reglaría": "reglar",
    "reglaríais": "reglar",
    "reglaríamos": "reglar",
    "reglarían": "reglar",
    "reglarías": "reglar",
    "reglas": "reglar",
    "reglase": "reglar",
    "reglaseis": "reglar",
    "reglasen": "reglar",
    "reglases": "reglar",
    "reglaste": "reglar",
    "reglasteis": "reglar",
    "regle": "reglar",
    "reglemos": "reglar",
    "reglen": "reglar",
    "regles": "reglar",
    "regletas": "regleta",
    "regletea": "regletear",
    "regleteaba": "regletear",
    "regleteabais": "regletear",
    "regleteaban": "regletear",
    "regleteabas": "regletear",
    "regletead": "regletear",
    "regleteada": "regletear",
    "regleteadas": "regletear",
    "regleteado": "regletear",
    "regleteados": "regletear",
    "regleteamos": "regletear",
    "regletean": "regletear",
    "regleteando": "regletear",
    "regleteara": "regletear",
    "regletearais": "regletear",
    "regletearan": "regletear",
    "regletearas": "regletear",
    "regleteare": "regletear",
    "regleteareis": "regletear",
    "regletearemos": "regletear",
    "regletearen": "regletear",
    "regleteares": "regletear",
    "regletearon": "regletear",
    "regleteará": "regletear",
    "regletearán": "regletear",
    "regletearás": "regletear",
    "regletearé": "regletear",
    "regletearéis": "regletear",
    "regletearía": "regletear",
    "regletearíais": "regletear",
    "regletearíamos": "regletear",
    "regletearían": "regletear",
    "regletearías": "regletear",
    "regleteas": "regletear",
    "regletease": "regletear",
    "regleteaseis": "regletear",
    "regleteasen": "regletear",
    "regleteases": "regletear",
    "regleteaste": "regletear",
    "regleteasteis": "regletear",
    "regletee": "regletear",
    "regleteemos": "regletear",
    "regleteen": "regletear",
    "regletees": "regletear",
    "regleteo": "regletear",
    "regleteábamos": "regletear",
    "regleteáis": "regletear",
    "regleteáramos": "regletear",
    "regleteáremos": "regletear",
    "regleteásemos": "regletear",
    "regleteé": "regletear",
    "regleteéis": "regletear",
    "regleteó": "regletear",
    "reglo": "reglar",
    "reglones": "reglón",
    "reglábamos": "reglar",
    "regláis": "reglar",
    "regláramos": "reglar",
    "regláremos": "reglar",
    "reglásemos": "reglar",
    "reglé": "reglar",
    "regléis": "reglar",
    "regló": "reglar",
    "regnícolas": "regnícola",
    "regocija": "regocijar",
    "regocijaba": "regocijar",
    "regocijabais": "regocijar",
    "regocijaban": "regocijar",
    "regocijabas": "regocijar",
    "regocijad": "regocijar",
    "regocijada": "regocijar",
    "regocijadas": "regocijar",
    "regocijado": "regocijar",
    "regocijadora": "regocijador",
    "regocijadoras": "regocijador",
    "regocijadores": "regocijador",
    "regocijados": "regocijar",
    "regocijamos": "regocijar",
    "regocijan": "regocijar",
    "regocijando": "regocijar",
    "regocijara": "regocijar",
    "regocijarais": "regocijar",
    "regocijaran": "regocijar",
    "regocijaras": "regocijar",
    "regocijare": "regocijar",
    "regocijareis": "regocijar",
    "regocijaremos": "regocijar",
    "regocijaren": "regocijar",
    "regocijares": "regocijar",
    "regocijaron": "regocijar",
    "regocijará": "regocijar",
    "regocijarán": "regocijar",
    "regocijarás": "regocijar",
    "regocijaré": "regocijar",
    "regocijaréis": "regocijar",
    "regocijaría": "regocijar",
    "regocijaríais": "regocijar",
    "regocijaríamos": "regocijar",
    "regocijarían": "regocijar",
    "regocijarías": "regocijar",
    "regocijas": "regocijar",
    "regocijase": "regocijar",
    "regocijaseis": "regocijar",
    "regocijasen": "regocijar",
    "regocijases": "regocijar",
    "regocijaste": "regocijar",
    "regocijasteis": "regocijar",
    "regocije": "regocijar",
    "regocijemos": "regocijar",
    "regocijen": "regocijar",
    "regocijes": "regocijar",
    "regocijo": "regocijar",
    "regocijos": "regocijo",
    "regocijábamos": "regocijar",
    "regocijáis": "regocijar",
    "regocijáramos": "regocijar",
    "regocijáremos": "regocijar",
    "regocijásemos": "regocijar",
    "regocijé": "regocijar",
    "regocijéis": "regocijar",
    "regocijó": "regocijar",
    "regodea": "regodear",
    "regodeaba": "regodear",
    "regodeabais": "regodear",
    "regodeaban": "regodear",
    "regodeabas": "regodear",
    "regodead": "regodear",
    "regodeada": "regodear",
    "regodeadas": "regodear",
    "regodeado": "regodear",
    "regodeados": "regodear",
    "regodeamos": "regodear",
    "regodean": "regodear",
    "regodeando": "regodear",
    "regodeara": "regodear",
    "regodearais": "regodear",
    "regodearan": "regodear",
    "regodearas": "regodear",
    "regodeare": "regodear",
    "regodeareis": "regodear",
    "regodearemos": "regodear",
    "regodearen": "regodear",
    "regodeares": "regodear",
    "regodearon": "regodear",
    "regodeará": "regodear",
    "regodearán": "regodear",
    "regodearás": "regodear",
    "regodearé": "regodear",
    "regodearéis": "regodear",
    "regodearía": "regodear",
    "regodearíais": "regodear",
    "regodearíamos": "regodear",
    "regodearían": "regodear",
    "regodearías": "regodear",
    "regodeas": "regodear",
    "regodease": "regodear",
    "regodeaseis": "regodear",
    "regodeasen": "regodear",
    "regodeases": "regodear",
    "regodeaste": "regodear",
    "regodeasteis": "regodear",
    "regodee": "regodear",
    "regodeemos": "regodear",
    "regodeen": "regodear",
    "regodees": "regodear",
    "regodeo": "regodear",
    "regodeos": "regodeo",
    "regodeábamos": "regodear",
    "regodeáis": "regodear",
    "regodeáramos": "regodear",
    "regodeáremos": "regodear",
    "regodeásemos": "regodear",
    "regodeé": "regodear",
    "regodeéis": "regodear",
    "regodeó": "regodear",
    "regojos": "regojo",
    "regoldaba": "regoldar",
    "regoldabais": "regoldar",
    "regoldaban": "regoldar",
    "regoldabas": "regoldar",
    "regoldad": "regoldar",
    "regoldada": "regoldar",
    "regoldadas": "regoldar",
    "regoldado": "regoldar",
    "regoldados": "regoldar",
    "regoldamos": "regoldar",
    "regoldana": "regoldano",
    "regoldanas": "regoldano",
    "regoldando": "regoldar",
    "regoldanos": "regoldano",
    "regoldara": "regoldar",
    "regoldarais": "regoldar",
    "regoldaran": "regoldar",
    "regoldaras": "regoldar",
    "regoldare": "regoldar",
    "regoldareis": "regoldar",
    "regoldaremos": "regoldar",
    "regoldaren": "regoldar",
    "regoldares": "regoldar",
    "regoldaron": "regoldar",
    "regoldará": "regoldar",
    "regoldarán": "regoldar",
    "regoldarás": "regoldar",
    "regoldaré": "regoldar",
    "regoldaréis": "regoldar",
    "regoldaría": "regoldar",
    "regoldaríais": "regoldar",
    "regoldaríamos": "regoldar",
    "regoldarían": "regoldar",
    "regoldarías": "regoldar",
    "regoldase": "regoldar",
    "regoldaseis": "regoldar",
    "regoldasen": "regoldar",
    "regoldases": "regoldar",
    "regoldaste": "regoldar",
    "regoldasteis": "regoldar",
    "regoldemos": "regoldar",
    "regoldos": "regoldo",
    "regoldábamos": "regoldar",
    "regoldáis": "regoldar",
    "regoldáramos": "regoldar",
    "regoldáremos": "regoldar",
    "regoldásemos": "regoldar",
    "regoldé": "regoldar",
    "regoldéis": "regoldar",
    "regoldó": "regoldar",
    "regolfos": "regolfo",
    "regolitas": "regolita",
    "regolitos": "regolito",
    "regonas": "regona",
    "regordeta": "regordete",
    "regordetas": "regordete",
    "regordetes": "regordete",
    "regosoles": "regosol",
    "regostos": "regosto",
    "regosuelos": "regosuelo",
    "regrabaciones": "regrabación",
    "regresa": "regresar",
    "regresaba": "regresar",
    "regresabais": "regresar",
    "regresaban": "regresar",
    "regresabas": "regresar",
    "regresad": "regresar",
    "regresada": "regresar",
    "regresadas": "regresar",
    "regresado": "regresar",
    "regresados": "regresar",
    "regresamos": "regresar",
    "regresan": "regresar",
    "regresando": "regresar",
    "regresara": "regresar",
    "regresarais": "regresar",
    "regresaran": "regresar",
    "regresaras": "regresar",
    "regresare": "regresar",
    "regresareis": "regresar",
    "regresaremos": "regresar",
    "regresaren": "regresar",
    "regresares": "regresar",
    "regresaron": "regresar",
    "regresará": "regresar",
    "regresarán": "regresar",
    "regresarás": "regresar",
    "regresaré": "regresar",
    "regresaréis": "regresar",
    "regresaría": "regresar",
    "regresaríais": "regresar",
    "regresaríamos": "regresar",
    "regresarían": "regresar",
    "regresarías": "regresar",
    "regresas": "regresar",
    "regresase": "regresar",
    "regresaseis": "regresar",
    "regresasen": "regresar",
    "regresases": "regresar",
    "regresaste": "regresar",
    "regresasteis": "regresar",
    "regrese": "regresar",
    "regresemos": "regresar",
    "regresen": "regresar",
    "regreses": "regresar",
    "regresiones": "regresión",
    "regresiva": "regresivo",
    "regresivas": "regresivo",
    "regresividades": "regresividad",
    "regresivos": "regresivo",
    "regreso": "regresar",
    "regresora": "regresor",
    "regresoras": "regresor",
    "regresores": "regresor",
    "regresos": "regreso",
    "regresábamos": "regresar",
    "regresáis": "regresar",
    "regresáramos": "regresar",
    "regresáremos": "regresar",
    "regresásemos": "regresar",
    "regresé": "regresar",
    "regreséis": "regresar",
    "regresó": "regresar",
    "regruña": "regruñir",
    "regruñamos": "regruñir",
    "regruñan": "regruñir",
    "regruñas": "regruñir",
    "regruñe": "regruñir",
    "regruñen": "regruñir",
    "regruñendo": "regruñir",
    "regruñera": "regruñir",
    "regruñerais": "regruñir",
    "regruñeran": "regruñir",
    "regruñeras": "regruñir",
    "regruñere": "regruñir",
    "regruñereis": "regruñir",
    "regruñeren": "regruñir",
    "regruñeres": "regruñir",
    "regruñeron": "regruñir",
    "regruñes": "regruñir",
    "regruñese": "regruñir",
    "regruñeseis": "regruñir",
    "regruñesen": "regruñir",
    "regruñeses": "regruñir",
    "regruñid": "regruñir",
    "regruñida": "regruñir",
    "regruñidas": "regruñir",
    "regruñido": "regruñir",
    "regruñidos": "regruñir",
    "regruñimos": "regruñir",
    "regruñiremos": "regruñir",
    "regruñirá": "regruñir",
    "regruñirán": "regruñir",
    "regruñirás": "regruñir",
    "regruñiré": "regruñir",
    "regruñiréis": "regruñir",
    "regruñiría": "regruñir",
    "regruñiríais": "regruñir",
    "regruñiríamos": "regruñir",
    "regruñirían": "regruñir",
    "regruñirías": "regruñir",
    "regruñiste": "regruñir",
    "regruñisteis": "regruñir",
    "regruño": "regruñir",
    "regruñáis": "regruñir",
    "regruñéramos": "regruñir",
    "regruñéremos": "regruñir",
    "regruñésemos": "regruñir",
    "regruñí": "regruñir",
    "regruñía": "regruñir",
    "regruñíais": "regruñir",
    "regruñíamos": "regruñir",
    "regruñían": "regruñir",
    "regruñías": "regruñir",
    "regruñís": "regruñir",
    "regruñó": "regruñir",
    "reguelda": "regoldar",
    "regueldan": "regoldar",
    "regueldas": "regoldar",
    "reguelde": "regoldar",
    "reguelden": "regoldar",
    "regueldes": "regoldar",
    "regueldo": "regoldar",
    "reguemos": "regar",
    "regueras": "reguera",
    "regueros": "reguero",
    "regula": "regular",
    "regulaba": "regular",
    "regulabais": "regular",
    "regulaban": "regular",
    "regulabas": "regular",
    "regulables": "regulable",
    "regulaciones": "regulación",
    "regulad": "regular",
    "regulada": "regular",
    "reguladas": "regular",
    "regulado": "regular",
    "reguladora": "regulador",
    "reguladoras": "regulador",
    "reguladores": "regulador",
    "regulados": "regular",
    "regulamos": "regular",
    "regulan": "regular",
    "regulando": "regular",
    "regulara": "regular",
    "regularais": "regular",
    "regularan": "regular",
    "regularas": "regular",
    "regulare": "regular",
    "regulareis": "regular",
    "regularemos": "regular",
    "regularen": "regular",
    "regulares": "regular",
    "regularice": "regularizar",
    "regularicemos": "regularizar",
    "regularicen": "regularizar",
    "regularices": "regularizar",
    "regularicé": "regularizar",
    "regularicéis": "regularizar",
    "regularidades": "regularidad",
    "regulariza": "regularizar",
    "regularizaba": "regularizar",
    "regularizabais": "regularizar",
    "regularizaban": "regularizar",
    "regularizabas": "regularizar",
    "regularizables": "regularizable",
    "regularizaciones": "regularización",
    "regularizad": "regularizar",
    "regularizada": "regularizar",
    "regularizadas": "regularizar",
    "regularizado": "regularizar",
    "regularizadora": "regularizador",
    "regularizadoras": "regularizador",
    "regularizadores": "regularizador",
    "regularizados": "regularizar",
    "regularizamos": "regularizar",
    "regularizan": "regularizar",
    "regularizando": "regularizar",
    "regularizara": "regularizar",
    "regularizarais": "regularizar",
    "regularizaran": "regularizar",
    "regularizaras": "regularizar",
    "regularizare": "regularizar",
    "regularizareis": "regularizar",
    "regularizaremos": "regularizar",
    "regularizaren": "regularizar",
    "regularizares": "regularizar",
    "regularizaron": "regularizar",
    "regularizará": "regularizar",
    "regularizarán": "regularizar",
    "regularizarás": "regularizar",
    "regularizaré": "regularizar",
    "regularizaréis": "regularizar",
    "regularizaría": "regularizar",
    "regularizaríais": "regularizar",
    "regularizaríamos": "regularizar",
    "regularizarían": "regularizar",
    "regularizarías": "regularizar",
    "regularizas": "regularizar",
    "regularizase": "regularizar",
    "regularizaseis": "regularizar",
    "regularizasen": "regularizar",
    "regularizases": "regularizar",
    "regularizaste": "regularizar",
    "regularizasteis": "regularizar",
    "regularizo": "regularizar",
    "regularizábamos": "regularizar",
    "regularizáis": "regularizar",
    "regularizáramos": "regularizar",
    "regularizáremos": "regularizar",
    "regularizásemos": "regularizar",
    "regularizó": "regularizar",
    "regularon": "regular",
    "regulará": "regular",
    "regularán": "regular",
    "regularás": "regular",
    "regularé": "regular",
    "regularéis": "regular",
    "regularía": "regular",
    "regularíais": "regular",
    "regularíamos": "regular",
    "regularían": "regular",
    "regularías": "regular",
    "regularímetros": "regularímetro",
    "regulas": "regular",
    "regulase": "regular",
    "regulaseis": "regular",
    "regulasen": "regular",
    "regulases": "regular",
    "regulaste": "regular",
    "regulasteis": "regular",
    "regulativa": "regulativo",
    "regulativas": "regulativo",
    "regulativos": "regulativo",
    "regule": "regular",
    "regulemos": "regular",
    "regulen": "regular",
    "regules": "regular",
    "regulo": "regular",
    "regulábamos": "regular",
    "reguláis": "regular",
    "reguláramos": "regular",
    "reguláremos": "regular",
    "regulásemos": "regular",
    "regulé": "regular",
    "reguléis": "regular",
    "reguló": "regular",
    "regurgita": "regurgitar",
    "regurgitaba": "regurgitar",
    "regurgitabais": "regurgitar",
    "regurgitaban": "regurgitar",
    "regurgitabas": "regurgitar",
    "regurgitaciones": "regurgitación",
    "regurgitad": "regurgitar",
    "regurgitada": "regurgitar",
    "regurgitadas": "regurgitar",
    "regurgitado": "regurgitar",
    "regurgitados": "regurgitar",
    "regurgitamos": "regurgitar",
    "regurgitan": "regurgitar",
    "regurgitando": "regurgitar",
    "regurgitara": "regurgitar",
    "regurgitarais": "regurgitar",
    "regurgitaran": "regurgitar",
    "regurgitaras": "regurgitar",
    "regurgitare": "regurgitar",
    "regurgitareis": "regurgitar",
    "regurgitaremos": "regurgitar",
    "regurgitaren": "regurgitar",
    "regurgitares": "regurgitar",
    "regurgitaron": "regurgitar",
    "regurgitará": "regurgitar",
    "regurgitarán": "regurgitar",
    "regurgitarás": "regurgitar",
    "regurgitaré": "regurgitar",
    "regurgitaréis": "regurgitar",
    "regurgitaría": "regurgitar",
    "regurgitaríais": "regurgitar",
    "regurgitaríamos": "regurgitar",
    "regurgitarían": "regurgitar",
    "regurgitarías": "regurgitar",
    "regurgitas": "regurgitar",
    "regurgitase": "regurgitar",
    "regurgitaseis": "regurgitar",
    "regurgitasen": "regurgitar",
    "regurgitases": "regurgitar",
    "regurgitaste": "regurgitar",
    "regurgitasteis": "regurgitar",
    "regurgite": "regurgitar",
    "regurgitemos": "regurgitar",
    "regurgiten": "regurgitar",
    "regurgites": "regurgitar",
    "regurgito": "regurgitar",
    "regurgitábamos": "regurgitar",
    "regurgitáis": "regurgitar",
    "regurgitáramos": "regurgitar",
    "regurgitáremos": "regurgitar",
    "regurgitásemos": "regurgitar",
    "regurgité": "regurgitar",
    "regurgitéis": "regurgitar",
    "regurgitó": "regurgitar",
    "regustos": "regusto",
    "regué": "regar",
    "reguéis": "regar",
    "regábamos": "regar",
    "regáis": "regar",
    "regáramos": "regar",
    "regáremos": "regar",
    "regásemos": "regar",
    "regí": "regir",
    "regía": "regir",
    "regíais": "regir",
    "regíamos": "regir",
    "regían": "regir",
    "regías": "regir",
    "regímenes": "régimen",
    "regís": "regir",
    "regó": "regar",
    "regüeldos": "regüeldo",
    "rehabilita": "rehabilitar",
    "rehabilitaba": "rehabilitar",
    "rehabilitabais": "rehabilitar",
    "rehabilitaban": "rehabilitar",
    "rehabilitabas": "rehabilitar",
    "rehabilitaciones": "rehabilitación",
    "rehabilitad": "rehabilitar",
    "rehabilitada": "rehabilitar",
    "rehabilitadas": "rehabilitar",
    "rehabilitado": "rehabilitar",
    "rehabilitados": "rehabilitar",
    "rehabilitamos": "rehabilitar",
    "rehabilitan": "rehabilitar",
    "rehabilitando": "rehabilitar",
    "rehabilitara": "rehabilitar",
    "rehabilitarais": "rehabilitar",
    "rehabilitaran": "rehabilitar",
    "rehabilitaras": "rehabilitar",
    "rehabilitare": "rehabilitar",
    "rehabilitareis": "rehabilitar",
    "rehabilitaremos": "rehabilitar",
    "rehabilitaren": "rehabilitar",
    "rehabilitares": "rehabilitar",
    "rehabilitaron": "rehabilitar",
    "rehabilitará": "rehabilitar",
    "rehabilitarán": "rehabilitar",
    "rehabilitarás": "rehabilitar",
    "rehabilitaré": "rehabilitar",
    "rehabilitaréis": "rehabilitar",
    "rehabilitaría": "rehabilitar",
    "rehabilitaríais": "rehabilitar",
    "rehabilitaríamos": "rehabilitar",
    "rehabilitarían": "rehabilitar",
    "rehabilitarías": "rehabilitar",
    "rehabilitas": "rehabilitar",
    "rehabilitase": "rehabilitar",
    "rehabilitaseis": "rehabilitar",
    "rehabilitasen": "rehabilitar",
    "rehabilitases": "rehabilitar",
    "rehabilitaste": "rehabilitar",
    "rehabilitasteis": "rehabilitar",
    "rehabilite": "rehabilitar",
    "rehabilitemos": "rehabilitar",
    "rehabiliten": "rehabilitar",
    "rehabilites": "rehabilitar",
    "rehabilito": "rehabilitar",
    "rehabilitábamos": "rehabilitar",
    "rehabilitáis": "rehabilitar",
    "rehabilitáramos": "rehabilitar",
    "rehabilitáremos": "rehabilitar",
    "rehabilitásemos": "rehabilitar",
    "rehabilité": "rehabilitar",
    "rehabilitéis": "rehabilitar",
    "rehabilitó": "rehabilitar",
    "rehace": "rehacer",
    "rehaced": "rehacer",
    "rehacemos": "rehacer",
    "rehacen": "rehacer",
    "rehaces": "rehacer",
    "rehaciendo": "rehacer",
    "rehacimientos": "rehacimiento",
    "rehacéis": "rehacer",
    "rehacía": "rehacer",
    "rehacíais": "rehacer",
    "rehacíamos": "rehacer",
    "rehacían": "rehacer",
    "rehacías": "rehacer",
    "rehaga": "rehacer",
    "rehagamos": "rehacer",
    "rehagan": "rehacer",
    "rehagas": "rehacer",
    "rehago": "rehacer",
    "rehagáis": "rehacer",
    "rehalas": "rehala",
    "rehaleros": "rehalero",
    "reharemos": "rehacer",
    "rehará": "rehacer",
    "reharán": "rehacer",
    "reharás": "rehacer",
    "reharé": "rehacer",
    "reharéis": "rehacer",
    "reharía": "rehacer",
    "reharíais": "rehacer",
    "reharíamos": "rehacer",
    "reharían": "rehacer",
    "reharías": "rehacer",
    "rehaz": "rehacer",
    "rehecha": "rehacer",
    "rehechas": "rehacer",
    "rehecho": "rehacer",
    "rehechos": "rehacer",
    "rehenchid": "rehenchir",
    "rehenchida": "rehenchir",
    "rehenchidas": "rehenchir",
    "rehenchido": "rehenchir",
    "rehenchidos": "rehenchir",
    "rehenchimientos": "rehenchimiento",
    "rehenchimos": "rehenchir",
    "rehenchiremos": "rehenchir",
    "rehenchirá": "rehenchir",
    "rehenchirán": "rehenchir",
    "rehenchirás": "rehenchir",
    "rehenchiré": "rehenchir",
    "rehenchiréis": "rehenchir",
    "rehenchiría": "rehenchir",
    "rehenchiríais": "rehenchir",
    "rehenchiríamos": "rehenchir",
    "rehenchirían": "rehenchir",
    "rehenchirías": "rehenchir",
    "rehenchiste": "rehenchir",
    "rehenchisteis": "rehenchir",
    "rehenchí": "rehenchir",
    "rehenchía": "rehenchir",
    "rehenchíais": "rehenchir",
    "rehenchíamos": "rehenchir",
    "rehenchían": "rehenchir",
    "rehenchías": "rehenchir",
    "rehenchís": "rehenchir",
    "rehenes": "rehén",
    "rehervid": "rehervir",
    "rehervida": "rehervir",
    "rehervidas": "rehervir",
    "rehervido": "rehervir",
    "rehervidos": "rehervir",
    "rehervimos": "rehervir",
    "reherviremos": "rehervir",
    "rehervirá": "rehervir",
    "rehervirán": "rehervir",
    "rehervirás": "rehervir",
    "reherviré": "rehervir",
    "reherviréis": "rehervir",
    "reherviría": "rehervir",
    "reherviríais": "rehervir",
    "reherviríamos": "rehervir",
    "rehervirían": "rehervir",
    "rehervirías": "rehervir",
    "reherviste": "rehervir",
    "rehervisteis": "rehervir",
    "reherví": "rehervir",
    "rehervía": "rehervir",
    "rehervíais": "rehervir",
    "rehervíamos": "rehervir",
    "rehervían": "rehervir",
    "rehervías": "rehervir",
    "rehervís": "rehervir",
    "rehice": "rehacer",
    "rehiciera": "rehacer",
    "rehicierais": "rehacer",
    "rehicieran": "rehacer",
    "rehicieras": "rehacer",
    "rehiciere": "rehacer",
    "rehiciereis": "rehacer",
    "rehicieren": "rehacer",
    "rehicieres": "rehacer",
    "rehicieron": "rehacer",
    "rehiciese": "rehacer",
    "rehicieseis": "rehacer",
    "rehiciesen": "rehacer",
    "rehicieses": "rehacer",
    "rehicimos": "rehacer",
    "rehiciste": "rehacer",
    "rehicisteis": "rehacer",
    "rehiciéramos": "rehacer",
    "rehiciéremos": "rehacer",
    "rehiciésemos": "rehacer",
    "rehidrataciones": "rehidratación",
    "rehielos": "rehielo",
    "rehierva": "rehervir",
    "rehiervan": "rehervir",
    "rehiervas": "rehervir",
    "rehierve": "rehervir",
    "rehierven": "rehervir",
    "rehierves": "rehervir",
    "rehiervo": "rehervir",
    "rehilaba": "rehilar",
    "rehilabais": "rehilar",
    "rehilaban": "rehilar",
    "rehilabas": "rehilar",
    "rehilad": "rehilar",
    "rehilada": "rehilar",
    "rehiladas": "rehilar",
    "rehiladillos": "rehiladillo",
    "rehilado": "rehilar",
    "rehilados": "rehilar",
    "rehilamientos": "rehilamiento",
    "rehilamos": "rehilar",
    "rehilanderas": "rehilandera",
    "rehilando": "rehilar",
    "rehilara": "rehilar",
    "rehilarais": "rehilar",
    "rehilaran": "rehilar",
    "rehilaras": "rehilar",
    "rehilare": "rehilar",
    "rehilareis": "rehilar",
    "rehilaremos": "rehilar",
    "rehilaren": "rehilar",
    "rehilares": "rehilar",
    "rehilaron": "rehilar",
    "rehilará": "rehilar",
    "rehilarán": "rehilar",
    "rehilarás": "rehilar",
    "rehilaré": "rehilar",
    "rehilaréis": "rehilar",
    "rehilaría": "rehilar",
    "rehilaríais": "rehilar",
    "rehilaríamos": "rehilar",
    "rehilarían": "rehilar",
    "rehilarías": "rehilar",
    "rehilase": "rehilar",
    "rehilaseis": "rehilar",
    "rehilasen": "rehilar",
    "rehilases": "rehilar",
    "rehilaste": "rehilar",
    "rehilasteis": "rehilar",
    "rehilemos": "rehilar",
    "rehiletera": "rehiletero",
    "rehileteras": "rehiletero",
    "rehileteros": "rehiletero",
    "rehiletes": "rehilete",
    "rehilábamos": "rehilar",
    "rehiláis": "rehilar",
    "rehiláramos": "rehilar",
    "rehiláremos": "rehilar",
    "rehilásemos": "rehilar",
    "rehilé": "rehilar",
    "rehiléis": "rehilar",
    "rehiló": "rehilar",
    "rehincha": "rehenchir",
    "rehinchamos": "rehenchir",
    "rehinchan": "rehenchir",
    "rehinchas": "rehenchir",
    "rehinche": "rehenchir",
    "rehinchen": "rehenchir",
    "rehinches": "rehenchir",
    "rehinchiendo": "rehenchir",
    "rehinchiera": "rehenchir",
    "rehinchierais": "rehenchir",
    "rehinchieran": "rehenchir",
    "rehinchieras": "rehenchir",
    "rehinchiere": "rehenchir",
    "rehinchiereis": "rehenchir",
    "rehinchieren": "rehenchir",
    "rehinchieres": "rehenchir",
    "rehinchieron": "rehenchir",
    "rehinchiese": "rehenchir",
    "rehinchieseis": "rehenchir",
    "rehinchiesen": "rehenchir",
    "rehinchieses": "rehenchir",
    "rehinchiéramos": "rehenchir",
    "rehinchiéremos": "rehenchir",
    "rehinchiésemos": "rehenchir",
    "rehinchió": "rehenchir",
    "rehincho": "rehenchir",
    "rehincháis": "rehenchir",
    "rehirvamos": "rehervir",
    "rehirviendo": "rehervir",
    "rehirviera": "rehervir",
    "rehirvierais": "rehervir",
    "rehirvieran": "rehervir",
    "rehirvieras": "rehervir",
    "rehirviere": "rehervir",
    "rehirviereis": "rehervir",
    "rehirvieren": "rehervir",
    "rehirvieres": "rehervir",
    "rehirvieron": "rehervir",
    "rehirviese": "rehervir",
    "rehirvieseis": "rehervir",
    "rehirviesen": "rehervir",
    "rehirvieses": "rehervir",
    "rehirviéramos": "rehervir",
    "rehirviéremos": "rehervir",
    "rehirviésemos": "rehervir",
    "rehirvió": "rehervir",
    "rehirváis": "rehervir",
    "rehizo": "rehacer",
    "rehoga": "rehogar",
    "rehogaba": "rehogar",
    "rehogabais": "rehogar",
    "rehogaban": "rehogar",
    "rehogabas": "rehogar",
    "rehogad": "rehogar",
    "rehogada": "rehogar",
    "rehogadas": "rehogar",
    "rehogado": "rehogar",
    "rehogados": "rehogar",
    "rehogamos": "rehogar",
    "rehogan": "rehogar",
    "rehogando": "rehogar",
    "rehogara": "rehogar",
    "rehogarais": "rehogar",
    "rehogaran": "rehogar",
    "rehogaras": "rehogar",
    "rehogare": "rehogar",
    "rehogareis": "rehogar",
    "rehogaremos": "rehogar",
    "rehogaren": "rehogar",
    "rehogares": "rehogar",
    "rehogaron": "rehogar",
    "rehogará": "rehogar",
    "rehogarán": "rehogar",
    "rehogarás": "rehogar",
    "rehogaré": "rehogar",
    "rehogaréis": "rehogar",
    "rehogaría": "rehogar",
    "rehogaríais": "rehogar",
    "rehogaríamos": "rehogar",
    "rehogarían": "rehogar",
    "rehogarías": "rehogar",
    "rehogas": "rehogar",
    "rehogase": "rehogar",
    "rehogaseis": "rehogar",
    "rehogasen": "rehogar",
    "rehogases": "rehogar",
    "rehogaste": "rehogar",
    "rehogasteis": "rehogar",
    "rehogo": "rehogar",
    "rehogue": "rehogar",
    "rehoguemos": "rehogar",
    "rehoguen": "rehogar",
    "rehogues": "rehogar",
    "rehogué": "rehogar",
    "rehoguéis": "rehogar",
    "rehogábamos": "rehogar",
    "rehogáis": "rehogar",
    "rehogáramos": "rehogar",
    "rehogáremos": "rehogar",
    "rehogásemos": "rehogar",
    "rehogó": "rehogar",
    "rehollaba": "rehollar",
    "rehollabais": "rehollar",
    "rehollaban": "rehollar",
    "rehollabas": "rehollar",
    "rehollad": "rehollar",
    "rehollada": "rehollar",
    "reholladas": "rehollar",
    "rehollado": "rehollar",
    "rehollados": "rehollar",
    "rehollamos": "rehollar",
    "rehollando": "rehollar",
    "rehollara": "rehollar",
    "rehollarais": "rehollar",
    "rehollaran": "rehollar",
    "rehollaras": "rehollar",
    "rehollare": "rehollar",
    "rehollareis": "rehollar",
    "rehollaremos": "rehollar",
    "rehollaren": "rehollar",
    "rehollares": "rehollar",
    "rehollaron": "rehollar",
    "rehollará": "rehollar",
    "rehollarán": "rehollar",
    "rehollarás": "rehollar",
    "rehollaré": "rehollar",
    "rehollaréis": "rehollar",
    "rehollaría": "rehollar",
    "rehollaríais": "rehollar",
    "rehollaríamos": "rehollar",
    "rehollarían": "rehollar",
    "rehollarías": "rehollar",
    "rehollase": "rehollar",
    "rehollaseis": "rehollar",
    "rehollasen": "rehollar",
    "rehollases": "rehollar",
    "rehollaste": "rehollar",
    "rehollasteis": "rehollar",
    "rehollemos": "rehollar",
    "rehollábamos": "rehollar",
    "reholláis": "rehollar",
    "reholláramos": "rehollar",
    "reholláremos": "rehollar",
    "rehollásemos": "rehollar",
    "rehollé": "rehollar",
    "reholléis": "rehollar",
    "reholló": "rehollar",
    "rehoyas": "rehoya",
    "rehoyos": "rehoyo",
    "rehuella": "rehollar",
    "rehuellan": "rehollar",
    "rehuellas": "rehollar",
    "rehuelle": "rehollar",
    "rehuellen": "rehollar",
    "rehuelles": "rehollar",
    "rehuello": "rehollar",
    "rehuid": "rehuir",
    "rehuida": "rehuir",
    "rehuidas": "rehuir",
    "rehuido": "rehuir",
    "rehuidos": "rehuir",
    "rehuimos": "rehuir",
    "rehuiremos": "rehuir",
    "rehuirá": "rehuir",
    "rehuirán": "rehuir",
    "rehuirás": "rehuir",
    "rehuiré": "rehuir",
    "rehuiréis": "rehuir",
    "rehuiría": "rehuir",
    "rehuiríais": "rehuir",
    "rehuiríamos": "rehuir",
    "rehuirían": "rehuir",
    "rehuirías": "rehuir",
    "rehuiste": "rehuir",
    "rehuisteis": "rehuir",
    "rehumedece": "rehumedecer",
    "rehumedeced": "rehumedecer",
    "rehumedecemos": "rehumedecer",
    "rehumedecen": "rehumedecer",
    "rehumedeceremos": "rehumedecer",
    "rehumedecerá": "rehumedecer",
    "rehumedecerán": "rehumedecer",
    "rehumedecerás": "rehumedecer",
    "rehumedeceré": "rehumedecer",
    "rehumedeceréis": "rehumedecer",
    "rehumedecería": "rehumedecer",
    "rehumedeceríais": "rehumedecer",
    "rehumedeceríamos": "rehumedecer",
    "rehumedecerían": "rehumedecer",
    "rehumedecerías": "rehumedecer",
    "rehumedeces": "rehumedecer",
    "rehumedecida": "rehumedecer",
    "rehumedecidas": "rehumedecer",
    "rehumedecido": "rehumedecer",
    "rehumedecidos": "rehumedecer",
    "rehumedeciendo": "rehumedecer",
    "rehumedeciera": "rehumedecer",
    "rehumedecierais": "rehumedecer",
    "rehumedecieran": "rehumedecer",
    "rehumedecieras": "rehumedecer",
    "rehumedeciere": "rehumedecer",
    "rehumedeciereis": "rehumedecer",
    "rehumedecieren": "rehumedecer",
    "rehumedecieres": "rehumedecer",
    "rehumedecieron": "rehumedecer",
    "rehumedeciese": "rehumedecer",
    "rehumedecieseis": "rehumedecer",
    "rehumedeciesen": "rehumedecer",
    "rehumedecieses": "rehumedecer",
    "rehumedecimos": "rehumedecer",
    "rehumedeciste": "rehumedecer",
    "rehumedecisteis": "rehumedecer",
    "rehumedeciéramos": "rehumedecer",
    "rehumedeciéremos": "rehumedecer",
    "rehumedeciésemos": "rehumedecer",
    "rehumedeció": "rehumedecer",
    "rehumedecéis": "rehumedecer",
    "rehumedecí": "rehumedecer",
    "rehumedecía": "rehumedecer",
    "rehumedecíais": "rehumedecer",
    "rehumedecíamos": "rehumedecer",
    "rehumedecían": "rehumedecer",
    "rehumedecías": "rehumedecer",
    "rehumedezca": "rehumedecer",
    "rehumedezcamos": "rehumedecer",
    "rehumedezcan": "rehumedecer",
    "rehumedezcas": "rehumedecer",
    "rehumedezco": "rehumedecer",
    "rehumedezcáis": "rehumedecer",
    "rehunda": "rehundir",
    "rehundamos": "rehundir",
    "rehundan": "rehundir",
    "rehundas": "rehundir",
    "rehunde": "rehundir",
    "rehunden": "rehundir",
    "rehundes": "rehundir",
    "rehundid": "rehundir",
    "rehundida": "rehundir",
    "rehundidas": "rehundir",
    "rehundido": "rehundir",
    "rehundidos": "rehundir",
    "rehundiendo": "rehundir",
    "rehundiera": "rehundir",
    "rehundierais": "rehundir",
    "rehundieran": "rehundir",
    "rehundieras": "rehundir",
    "rehundiere": "rehundir",
    "rehundiereis": "rehundir",
    "rehundieren": "rehundir",
    "rehundieres": "rehundir",
    "rehundieron": "rehundir",
    "rehundiese": "rehundir",
    "rehundieseis": "rehundir",
    "rehundiesen": "rehundir",
    "rehundieses": "rehundir",
    "rehundimos": "rehundir",
    "rehundiremos": "rehundir",
    "rehundirá": "rehundir",
    "rehundirán": "rehundir",
    "rehundirás": "rehundir",
    "rehundiré": "rehundir",
    "rehundiréis": "rehundir",
    "rehundiría": "rehundir",
    "rehundiríais": "rehundir",
    "rehundiríamos": "rehundir",
    "rehundirían": "rehundir",
    "rehundirías": "rehundir",
    "rehundiste": "rehundir",
    "rehundisteis": "rehundir",
    "rehundiéramos": "rehundir",
    "rehundiéremos": "rehundir",
    "rehundiésemos": "rehundir",
    "rehundió": "rehundir",
    "rehundo": "rehundir",
    "rehundáis": "rehundir",
    "rehundí": "rehundir",
    "rehundía": "rehundir",
    "rehundíais": "rehundir",
    "rehundíamos": "rehundir",
    "rehundían": "rehundir",
    "rehundías": "rehundir",
    "rehundís": "rehundir",
    "rehusaba": "rehusar",
    "rehusabais": "rehusar",
    "rehusaban": "rehusar",
    "rehusabas": "rehusar",
    "rehusad": "rehusar",
    "rehusada": "rehusar",
    "rehusadas": "rehusar",
    "rehusado": "rehusar",
    "rehusados": "rehusar",
    "rehusamos": "rehusar",
    "rehusando": "rehusar",
    "rehusara": "rehusar",
    "rehusarais": "rehusar",
    "rehusaran": "rehusar",
    "rehusaras": "rehusar",
    "rehusare": "rehusar",
    "rehusareis": "rehusar",
    "rehusaremos": "rehusar",
    "rehusaren": "rehusar",
    "rehusares": "rehusar",
    "rehusaron": "rehusar",
    "rehusará": "rehusar",
    "rehusarán": "rehusar",
    "rehusarás": "rehusar",
    "rehusaré": "rehusar",
    "rehusaréis": "rehusar",
    "rehusaría": "rehusar",
    "rehusaríais": "rehusar",
    "rehusaríamos": "rehusar",
    "rehusarían": "rehusar",
    "rehusarías": "rehusar",
    "rehusase": "rehusar",
    "rehusaseis": "rehusar",
    "rehusasen": "rehusar",
    "rehusases": "rehusar",
    "rehusaste": "rehusar",
    "rehusasteis": "rehusar",
    "rehuse": "rehúse",
    "rehusemos": "rehusar",
    "rehuses": "rehúse",
    "rehusábamos": "rehusar",
    "rehusáis": "rehusar",
    "rehusáramos": "rehusar",
    "rehusáremos": "rehusar",
    "rehusásemos": "rehusar",
    "rehusé": "rehusar",
    "rehuséis": "rehusar",
    "rehusó": "rehusar",
    "rehuya": "rehuir",
    "rehuyamos": "rehuir",
    "rehuyan": "rehuir",
    "rehuyas": "rehuir",
    "rehuye": "rehuir",
    "rehuyen": "rehuir",
    "rehuyendo": "rehuir",
    "rehuyera": "rehuir",
    "rehuyerais": "rehuir",
    "rehuyeran": "rehuir",
    "rehuyeras": "rehuir",
    "rehuyere": "rehuir",
    "rehuyereis": "rehuir",
    "rehuyeren": "rehuir",
    "rehuyeres": "rehuir",
    "rehuyeron": "rehuir",
    "rehuyes": "rehuir",
    "rehuyese": "rehuir",
    "rehuyeseis": "rehuir",
    "rehuyesen": "rehuir",
    "rehuyeses": "rehuir",
    "rehuyo": "rehuir",
    "rehuyáis": "rehuir",
    "rehuyéramos": "rehuir",
    "rehuyéremos": "rehuir",
    "rehuyésemos": "rehuir",
    "rehuyó": "rehuir",
    "rehuí": "rehuir",
    "rehuía": "rehuir",
    "rehuíais": "rehuir",
    "rehuíamos": "rehuir",
    "rehuían": "rehuir",
    "rehuías": "rehuir",
    "rehuís": "rehuir",
    "rehíla": "rehilar",
    "rehílan": "rehilar",
    "rehílas": "rehilar",
    "rehíle": "rehilar",
    "rehílen": "rehilar",
    "rehíles": "rehilar",
    "rehílo": "rehilar",
    "rehúsa": "rehusar",
    "rehúsan": "rehusar",
    "rehúsas": "rehusar",
    "rehúse": "rehusar",
    "rehúsen": "rehusar",
    "rehúses": "rehusar",
    "rehúso": "rehusar",
    "rehúye": "rehuir",
    "rehúyen": "rehuir",
    "rehúyes": "rehuir",
    "rehúyo": "rehuir",
    "reidora": "reidor",
    "reidoras": "reidor",
    "reidores": "reidor",
    "reificaciones": "reificación",
    "reificaciónes": "reificación",
    "reiformes": "reiforme",
    "reimanaciones": "reimanación",
    "reimantaciones": "reimantación",
    "reimplantaciones": "reimplantación",
    "reimplementaciones": "reimplementación",
    "reimporta": "reimportar",
    "reimportaba": "reimportar",
    "reimportabais": "reimportar",
    "reimportaban": "reimportar",
    "reimportabas": "reimportar",
    "reimportaciones": "reimportación",
    "reimportad": "reimportar",
    "reimportada": "reimportar",
    "reimportadas": "reimportar",
    "reimportado": "reimportar",
    "reimportados": "reimportar",
    "reimportamos": "reimportar",
    "reimportan": "reimportar",
    "reimportando": "reimportar",
    "reimportara": "reimportar",
    "reimportarais": "reimportar",
    "reimportaran": "reimportar",
    "reimportaras": "reimportar",
    "reimportare": "reimportar",
    "reimportareis": "reimportar",
    "reimportaremos": "reimportar",
    "reimportaren": "reimportar",
    "reimportares": "reimportar",
    "reimportaron": "reimportar",
    "reimportará": "reimportar",
    "reimportarán": "reimportar",
    "reimportarás": "reimportar",
    "reimportaré": "reimportar",
    "reimportaréis": "reimportar",
    "reimportaría": "reimportar",
    "reimportaríais": "reimportar",
    "reimportaríamos": "reimportar",
    "reimportarían": "reimportar",
    "reimportarías": "reimportar",
    "reimportas": "reimportar",
    "reimportase": "reimportar",
    "reimportaseis": "reimportar",
    "reimportasen": "reimportar",
    "reimportases": "reimportar",
    "reimportaste": "reimportar",
    "reimportasteis": "reimportar",
    "reimporte": "reimportar",
    "reimportemos": "reimportar",
    "reimporten": "reimportar",
    "reimportes": "reimportar",
    "reimporto": "reimportar",
    "reimportábamos": "reimportar",
    "reimportáis": "reimportar",
    "reimportáramos": "reimportar",
    "reimportáremos": "reimportar",
    "reimportásemos": "reimportar",
    "reimporté": "reimportar",
    "reimportéis": "reimportar",
    "reimportó": "reimportar",
    "reimpresa": "reimprimir",
    "reimpresas": "reimprimir",
    "reimpresiones": "reimpresión",
    "reimpreso": "reimprimir",
    "reimpresos": "reimprimir",
    "reimprima": "reimprimir",
    "reimprimamos": "reimprimir",
    "reimpriman": "reimprimir",
    "reimprimas": "reimprimir",
    "reimprime": "reimprimir",
    "reimprimen": "reimprimir",
    "reimprimes": "reimprimir",
    "reimprimid": "reimprimir",
    "reimprimida": "reimprimir",
    "reimprimidas": "reimprimir",
    "reimprimido": "reimprimir",
    "reimprimidos": "reimprimir",
    "reimprimiendo": "reimprimir",
    "reimprimiera": "reimprimir",
    "reimprimierais": "reimprimir",
    "reimprimieran": "reimprimir",
    "reimprimieras": "reimprimir",
    "reimprimiere": "reimprimir",
    "reimprimiereis": "reimprimir",
    "reimprimieren": "reimprimir",
    "reimprimieres": "reimprimir",
    "reimprimieron": "reimprimir",
    "reimprimiese": "reimprimir",
    "reimprimieseis": "reimprimir",
    "reimprimiesen": "reimprimir",
    "reimprimieses": "reimprimir",
    "reimprimimos": "reimprimir",
    "reimprimiremos": "reimprimir",
    "reimprimirá": "reimprimir",
    "reimprimirán": "reimprimir",
    "reimprimirás": "reimprimir",
    "reimprimiré": "reimprimir",
    "reimprimiréis": "reimprimir",
    "reimprimiría": "reimprimir",
    "reimprimiríais": "reimprimir",
    "reimprimiríamos": "reimprimir",
    "reimprimirían": "reimprimir",
    "reimprimirías": "reimprimir",
    "reimprimiste": "reimprimir",
    "reimprimisteis": "reimprimir",
    "reimprimiéramos": "reimprimir",
    "reimprimiéremos": "reimprimir",
    "reimprimiésemos": "reimprimir",
    "reimprimió": "reimprimir",
    "reimprimo": "reimprimir",
    "reimprimáis": "reimprimir",
    "reimprimí": "reimprimir",
    "reimprimía": "reimprimir",
    "reimprimíais": "reimprimir",
    "reimprimíamos": "reimprimir",
    "reimprimían": "reimprimir",
    "reimprimías": "reimprimir",
    "reimprimís": "reimprimir",
    "reina": "reinar",
    "reinaba": "reinar",
    "reinabais": "reinar",
    "reinaban": "reinar",
    "reinabas": "reinar",
    "reinad": "reinar",
    "reinada": "reinar",
    "reinadas": "reinar",
    "reinado": "reinar",
    "reinados": "reinar",
    "reinales": "reinal",
    "reinamos": "reinar",
    "reinan": "reinar",
    "reinando": "reinar",
    "reinantes": "reinante",
    "reinara": "reinar",
    "reinarais": "reinar",
    "reinaran": "reinar",
    "reinaras": "reinar",
    "reinare": "reinar",
    "reinareis": "reinar",
    "reinaremos": "reinar",
    "reinaren": "reinar",
    "reinares": "reinar",
    "reinaron": "reinar",
    "reinará": "reinar",
    "reinarán": "reinar",
    "reinarás": "reinar",
    "reinaré": "reinar",
    "reinaréis": "reinar",
    "reinaría": "reinar",
    "reinaríais": "reinar",
    "reinaríamos": "reinar",
    "reinarían": "reinar",
    "reinarías": "reinar",
    "reinas": "reinar",
    "reinase": "reinar",
    "reinaseis": "reinar",
    "reinasen": "reinar",
    "reinases": "reinar",
    "reinaste": "reinar",
    "reinasteis": "reinar",
    "reincida": "reincidir",
    "reincidamos": "reincidir",
    "reincidan": "reincidir",
    "reincidas": "reincidir",
    "reincide": "reincidir",
    "reinciden": "reincidir",
    "reincidencias": "reincidencia",
    "reincidentes": "reincidente",
    "reincides": "reincidir",
    "reincidid": "reincidir",
    "reincidida": "reincidir",
    "reincididas": "reincidir",
    "reincidido": "reincidir",
    "reincididos": "reincidir",
    "reincidiendo": "reincidir",
    "reincidiera": "reincidir",
    "reincidierais": "reincidir",
    "reincidieran": "reincidir",
    "reincidieras": "reincidir",
    "reincidiere": "reincidir",
    "reincidiereis": "reincidir",
    "reincidieren": "reincidir",
    "reincidieres": "reincidir",
    "reincidieron": "reincidir",
    "reincidiese": "reincidir",
    "reincidieseis": "reincidir",
    "reincidiesen": "reincidir",
    "reincidieses": "reincidir",
    "reincidimos": "reincidir",
    "reincidiremos": "reincidir",
    "reincidirá": "reincidir",
    "reincidirán": "reincidir",
    "reincidirás": "reincidir",
    "reincidiré": "reincidir",
    "reincidiréis": "reincidir",
    "reincidiría": "reincidir",
    "reincidiríais": "reincidir",
    "reincidiríamos": "reincidir",
    "reincidirían": "reincidir",
    "reincidirías": "reincidir",
    "reincidiste": "reincidir",
    "reincidisteis": "reincidir",
    "reincidiéramos": "reincidir",
    "reincidiéremos": "reincidir",
    "reincidiésemos": "reincidir",
    "reincidió": "reincidir",
    "reincido": "reincidir",
    "reincidáis": "reincidir",
    "reincidí": "reincidir",
    "reincidía": "reincidir",
    "reincidíais": "reincidir",
    "reincidíamos": "reincidir",
    "reincidían": "reincidir",
    "reincidías": "reincidir",
    "reincidís": "reincidir",
    "reincorpora": "reincorporar",
    "reincorporaba": "reincorporar",
    "reincorporabais": "reincorporar",
    "reincorporaban": "reincorporar",
    "reincorporabas": "reincorporar",
    "reincorporaciones": "reincorporación",
    "reincorporad": "reincorporar",
    "reincorporada": "reincorporar",
    "reincorporadas": "reincorporar",
    "reincorporado": "reincorporar",
    "reincorporados": "reincorporar",
    "reincorporamos": "reincorporar",
    "reincorporan": "reincorporar",
    "reincorporando": "reincorporar",
    "reincorporara": "reincorporar",
    "reincorporarais": "reincorporar",
    "reincorporaran": "reincorporar",
    "reincorporaras": "reincorporar",
    "reincorporare": "reincorporar",
    "reincorporareis": "reincorporar",
    "reincorporaremos": "reincorporar",
    "reincorporaren": "reincorporar",
    "reincorporares": "reincorporar",
    "reincorporaron": "reincorporar",
    "reincorporará": "reincorporar",
    "reincorporarán": "reincorporar",
    "reincorporarás": "reincorporar",
    "reincorporaré": "reincorporar",
    "reincorporaréis": "reincorporar",
    "reincorporaría": "reincorporar",
    "reincorporaríais": "reincorporar",
    "reincorporaríamos": "reincorporar",
    "reincorporarían": "reincorporar",
    "reincorporarías": "reincorporar",
    "reincorporas": "reincorporar",
    "reincorporase": "reincorporar",
    "reincorporaseis": "reincorporar",
    "reincorporasen": "reincorporar",
    "reincorporases": "reincorporar",
    "reincorporaste": "reincorporar",
    "reincorporasteis": "reincorporar",
    "reincorpore": "reincorporar",
    "reincorporemos": "reincorporar",
    "reincorporen": "reincorporar",
    "reincorpores": "reincorporar",
    "reincorporo": "reincorporar",
    "reincorporábamos": "reincorporar",
    "reincorporáis": "reincorporar",
    "reincorporáramos": "reincorporar",
    "reincorporáremos": "reincorporar",
    "reincorporásemos": "reincorporar",
    "reincorporé": "reincorporar",
    "reincorporéis": "reincorporar",
    "reincorporó": "reincorporar",
    "reindustrializaciones": "reindustrialización",
    "reine": "reinar",
    "reinemos": "reinar",
    "reinen": "reinar",
    "reines": "reinar",
    "reinetas": "reineta",
    "reinfecciones": "reinfección",
    "reingenierías": "reingeniería",
    "reingresa": "reingresar",
    "reingresaba": "reingresar",
    "reingresabais": "reingresar",
    "reingresaban": "reingresar",
    "reingresabas": "reingresar",
    "reingresad": "reingresar",
    "reingresada": "reingresar",
    "reingresadas": "reingresar",
    "reingresado": "reingresar",
    "reingresados": "reingresar",
    "reingresamos": "reingresar",
    "reingresan": "reingresar",
    "reingresando": "reingresar",
    "reingresara": "reingresar",
    "reingresarais": "reingresar",
    "reingresaran": "reingresar",
    "reingresaras": "reingresar",
    "reingresare": "reingresar",
    "reingresareis": "reingresar",
    "reingresaremos": "reingresar",
    "reingresaren": "reingresar",
    "reingresares": "reingresar",
    "reingresaron": "reingresar",
    "reingresará": "reingresar",
    "reingresarán": "reingresar",
    "reingresarás": "reingresar",
    "reingresaré": "reingresar",
    "reingresaréis": "reingresar",
    "reingresaría": "reingresar",
    "reingresaríais": "reingresar",
    "reingresaríamos": "reingresar",
    "reingresarían": "reingresar",
    "reingresarías": "reingresar",
    "reingresas": "reingresar",
    "reingresase": "reingresar",
    "reingresaseis": "reingresar",
    "reingresasen": "reingresar",
    "reingresases": "reingresar",
    "reingresaste": "reingresar",
    "reingresasteis": "reingresar",
    "reingrese": "reingresar",
    "reingresemos": "reingresar",
    "reingresen": "reingresar",
    "reingreses": "reingresar",
    "reingreso": "reingresar",
    "reingresos": "reingreso",
    "reingresábamos": "reingresar",
    "reingresáis": "reingresar",
    "reingresáramos": "reingresar",
    "reingresáremos": "reingresar",
    "reingresásemos": "reingresar",
    "reingresé": "reingresar",
    "reingreséis": "reingresar",
    "reingresó": "reingresar",
    "reinicializaciones": "reinicialización",
    "reinicios": "reinicio",
    "reinmunizaciones": "reinmunización",
    "reino": "reinar",
    "reinos": "reino",
    "reinserciones": "reinserción",
    "reinserta": "reinsertar",
    "reinsertaba": "reinsertar",
    "reinsertabais": "reinsertar",
    "reinsertaban": "reinsertar",
    "reinsertabas": "reinsertar",
    "reinsertad": "reinsertar",
    "reinsertada": "reinsertar",
    "reinsertadas": "reinsertar",
    "reinsertado": "reinsertar",
    "reinsertados": "reinsertar",
    "reinsertamos": "reinsertar",
    "reinsertan": "reinsertar",
    "reinsertando": "reinsertar",
    "reinsertara": "reinsertar",
    "reinsertarais": "reinsertar",
    "reinsertaran": "reinsertar",
    "reinsertaras": "reinsertar",
    "reinsertare": "reinsertar",
    "reinsertareis": "reinsertar",
    "reinsertaremos": "reinsertar",
    "reinsertaren": "reinsertar",
    "reinsertares": "reinsertar",
    "reinsertaron": "reinsertar",
    "reinsertará": "reinsertar",
    "reinsertarán": "reinsertar",
    "reinsertarás": "reinsertar",
    "reinsertaré": "reinsertar",
    "reinsertaréis": "reinsertar",
    "reinsertaría": "reinsertar",
    "reinsertaríais": "reinsertar",
    "reinsertaríamos": "reinsertar",
    "reinsertarían": "reinsertar",
    "reinsertarías": "reinsertar",
    "reinsertas": "reinsertar",
    "reinsertase": "reinsertar",
    "reinsertaseis": "reinsertar",
    "reinsertasen": "reinsertar",
    "reinsertases": "reinsertar",
    "reinsertaste": "reinsertar",
    "reinsertasteis": "reinsertar",
    "reinserte": "reinsertar",
    "reinsertemos": "reinsertar",
    "reinserten": "reinsertar",
    "reinsertes": "reinsertar",
    "reinserto": "reinsertar",
    "reinsertábamos": "reinsertar",
    "reinsertáis": "reinsertar",
    "reinsertáramos": "reinsertar",
    "reinsertáremos": "reinsertar",
    "reinsertásemos": "reinsertar",
    "reinserté": "reinsertar",
    "reinsertéis": "reinsertar",
    "reinsertó": "reinsertar",
    "reinstala": "reinstalar",
    "reinstalaba": "reinstalar",
    "reinstalabais": "reinstalar",
    "reinstalaban": "reinstalar",
    "reinstalabas": "reinstalar",
    "reinstalaciones": "reinstalación",
    "reinstalad": "reinstalar",
    "reinstalada": "reinstalar",
    "reinstaladas": "reinstalar",
    "reinstalado": "reinstalar",
    "reinstalados": "reinstalar",
    "reinstalamos": "reinstalar",
    "reinstalan": "reinstalar",
    "reinstalando": "reinstalar",
    "reinstalara": "reinstalar",
    "reinstalarais": "reinstalar",
    "reinstalaran": "reinstalar",
    "reinstalaras": "reinstalar",
    "reinstalare": "reinstalar",
    "reinstalareis": "reinstalar",
    "reinstalaremos": "reinstalar",
    "reinstalaren": "reinstalar",
    "reinstalares": "reinstalar",
    "reinstalaron": "reinstalar",
    "reinstalará": "reinstalar",
    "reinstalarán": "reinstalar",
    "reinstalarás": "reinstalar",
    "reinstalaré": "reinstalar",
    "reinstalaréis": "reinstalar",
    "reinstalaría": "reinstalar",
    "reinstalaríais": "reinstalar",
    "reinstalaríamos": "reinstalar",
    "reinstalarían": "reinstalar",
    "reinstalarías": "reinstalar",
    "reinstalas": "reinstalar",
    "reinstalase": "reinstalar",
    "reinstalaseis": "reinstalar",
    "reinstalasen": "reinstalar",
    "reinstalases": "reinstalar",
    "reinstalaste": "reinstalar",
    "reinstalasteis": "reinstalar",
    "reinstale": "reinstalar",
    "reinstalemos": "reinstalar",
    "reinstalen": "reinstalar",
    "reinstales": "reinstalar",
    "reinstalo": "reinstalar",
    "reinstalábamos": "reinstalar",
    "reinstaláis": "reinstalar",
    "reinstaláramos": "reinstalar",
    "reinstaláremos": "reinstalar",
    "reinstalásemos": "reinstalar",
    "reinstalé": "reinstalar",
    "reinstaléis": "reinstalar",
    "reinstaló": "reinstalar",
    "reinstaura": "reinstaurar",
    "reinstauraba": "reinstaurar",
    "reinstaurabais": "reinstaurar",
    "reinstauraban": "reinstaurar",
    "reinstaurabas": "reinstaurar",
    "reinstauraciones": "reinstauración",
    "reinstaurad": "reinstaurar",
    "reinstaurada": "reinstaurar",
    "reinstauradas": "reinstaurar",
    "reinstaurado": "reinstaurar",
    "reinstaurados": "reinstaurar",
    "reinstauramos": "reinstaurar",
    "reinstauran": "reinstaurar",
    "reinstaurando": "reinstaurar",
    "reinstaurara": "reinstaurar",
    "reinstaurarais": "reinstaurar",
    "reinstauraran": "reinstaurar",
    "reinstauraras": "reinstaurar",
    "reinstaurare": "reinstaurar",
    "reinstaurareis": "reinstaurar",
    "reinstauraremos": "reinstaurar",
    "reinstauraren": "reinstaurar",
    "reinstaurares": "reinstaurar",
    "reinstauraron": "reinstaurar",
    "reinstaurará": "reinstaurar",
    "reinstaurarán": "reinstaurar",
    "reinstaurarás": "reinstaurar",
    "reinstauraré": "reinstaurar",
    "reinstauraréis": "reinstaurar",
    "reinstauraría": "reinstaurar",
    "reinstauraríais": "reinstaurar",
    "reinstauraríamos": "reinstaurar",
    "reinstaurarían": "reinstaurar",
    "reinstaurarías": "reinstaurar",
    "reinstauras": "reinstaurar",
    "reinstaurase": "reinstaurar",
    "reinstauraseis": "reinstaurar",
    "reinstaurasen": "reinstaurar",
    "reinstaurases": "reinstaurar",
    "reinstauraste": "reinstaurar",
    "reinstaurasteis": "reinstaurar",
    "reinstaure": "reinstaurar",
    "reinstauremos": "reinstaurar",
    "reinstauren": "reinstaurar",
    "reinstaures": "reinstaurar",
    "reinstauro": "reinstaurar",
    "reinstaurábamos": "reinstaurar",
    "reinstauráis": "reinstaurar",
    "reinstauráramos": "reinstaurar",
    "reinstauráremos": "reinstaurar",
    "reinstaurásemos": "reinstaurar",
    "reinstauré": "reinstaurar",
    "reinstauréis": "reinstaurar",
    "reinstauró": "reinstaurar",
    "reintegra": "reintegrar",
    "reintegraba": "reintegrar",
    "reintegrabais": "reintegrar",
    "reintegraban": "reintegrar",
    "reintegrabas": "reintegrar",
    "reintegrables": "reintegrable",
    "reintegraciones": "reintegración",
    "reintegrad": "reintegrar",
    "reintegrada": "reintegrar",
    "reintegradas": "reintegrar",
    "reintegrado": "reintegrar",
    "reintegrados": "reintegrar",
    "reintegramos": "reintegrar",
    "reintegran": "reintegrar",
    "reintegrando": "reintegrar",
    "reintegrara": "reintegrar",
    "reintegrarais": "reintegrar",
    "reintegraran": "reintegrar",
    "reintegraras": "reintegrar",
    "reintegrare": "reintegrar",
    "reintegrareis": "reintegrar",
    "reintegraremos": "reintegrar",
    "reintegraren": "reintegrar",
    "reintegrares": "reintegrar",
    "reintegraron": "reintegrar",
    "reintegrará": "reintegrar",
    "reintegrarán": "reintegrar",
    "reintegrarás": "reintegrar",
    "reintegraré": "reintegrar",
    "reintegraréis": "reintegrar",
    "reintegraría": "reintegrar",
    "reintegraríais": "reintegrar",
    "reintegraríamos": "reintegrar",
    "reintegrarían": "reintegrar",
    "reintegrarías": "reintegrar",
    "reintegras": "reintegrar",
    "reintegrase": "reintegrar",
    "reintegraseis": "reintegrar",
    "reintegrasen": "reintegrar",
    "reintegrases": "reintegrar",
    "reintegraste": "reintegrar",
    "reintegrasteis": "reintegrar",
    "reintegre": "reintegrar",
    "reintegremos": "reintegrar",
    "reintegren": "reintegrar",
    "reintegres": "reintegrar",
    "reintegro": "reintegrar",
    "reintegros": "reintegro",
    "reintegrábamos": "reintegrar",
    "reintegráis": "reintegrar",
    "reintegráramos": "reintegrar",
    "reintegráremos": "reintegrar",
    "reintegrásemos": "reintegrar",
    "reintegré": "reintegrar",
    "reintegréis": "reintegrar",
    "reintegró": "reintegrar",
    "reintentos": "reintento",
    "reinterpretaciones": "reinterpretación",
    "reinventa": "reinventar",
    "reinventaba": "reinventar",
    "reinventabais": "reinventar",
    "reinventaban": "reinventar",
    "reinventabas": "reinventar",
    "reinventad": "reinventar",
    "reinventada": "reinventar",
    "reinventadas": "reinventar",
    "reinventado": "reinventar",
    "reinventados": "reinventar",
    "reinventamos": "reinventar",
    "reinventan": "reinventar",
    "reinventando": "reinventar",
    "reinventara": "reinventar",
    "reinventarais": "reinventar",
    "reinventaran": "reinventar",
    "reinventaras": "reinventar",
    "reinventare": "reinventar",
    "reinventareis": "reinventar",
    "reinventaremos": "reinventar",
    "reinventaren": "reinventar",
    "reinventares": "reinventar",
    "reinventaron": "reinventar",
    "reinventará": "reinventar",
    "reinventarán": "reinventar",
    "reinventarás": "reinventar",
    "reinventaré": "reinventar",
    "reinventaréis": "reinventar",
    "reinventaría": "reinventar",
    "reinventaríais": "reinventar",
    "reinventaríamos": "reinventar",
    "reinventarían": "reinventar",
    "reinventarías": "reinventar",
    "reinventas": "reinventar",
    "reinventase": "reinventar",
    "reinventaseis": "reinventar",
    "reinventasen": "reinventar",
    "reinventases": "reinventar",
    "reinventaste": "reinventar",
    "reinventasteis": "reinventar",
    "reinvente": "reinventar",
    "reinventemos": "reinventar",
    "reinventen": "reinventar",
    "reinventes": "reinventar",
    "reinvento": "reinventar",
    "reinventos": "reinvento",
    "reinventábamos": "reinventar",
    "reinventáis": "reinventar",
    "reinventáramos": "reinventar",
    "reinventáremos": "reinventar",
    "reinventásemos": "reinventar",
    "reinventé": "reinventar",
    "reinventéis": "reinventar",
    "reinventó": "reinventar",
    "reinversiones": "reinversión",
    "reinábamos": "reinar",
    "reináis": "reinar",
    "reináramos": "reinar",
    "reináremos": "reinar",
    "reinásemos": "reinar",
    "reiné": "reinar",
    "reinéis": "reinar",
    "reinó": "reinar",
    "reiremos": "reír",
    "reirá": "reír",
    "reirán": "reír",
    "reirás": "reír",
    "reiré": "reír",
    "reiréis": "reír",
    "reiría": "reír",
    "reiríais": "reír",
    "reiríamos": "reír",
    "reirían": "reír",
    "reirías": "reír",
    "reitera": "reiterar",
    "reiteraba": "reiterar",
    "reiterabais": "reiterar",
    "reiteraban": "reiterar",
    "reiterabas": "reiterar",
    "reiteraciones": "reiteración",
    "reiterad": "reiterar",
    "reiterada": "reiterar",
    "reiteradas": "reiterar",
    "reiterado": "reiterar",
    "reiterados": "reiterar",
    "reiteramos": "reiterar",
    "reiteran": "reiterar",
    "reiterando": "reiterar",
    "reiterara": "reiterar",
    "reiterarais": "reiterar",
    "reiteraran": "reiterar",
    "reiteraras": "reiterar",
    "reiterare": "reiterar",
    "reiterareis": "reiterar",
    "reiteraremos": "reiterar",
    "reiteraren": "reiterar",
    "reiterares": "reiterar",
    "reiteraron": "reiterar",
    "reiterará": "reiterar",
    "reiterarán": "reiterar",
    "reiterarás": "reiterar",
    "reiteraré": "reiterar",
    "reiteraréis": "reiterar",
    "reiteraría": "reiterar",
    "reiteraríais": "reiterar",
    "reiteraríamos": "reiterar",
    "reiterarían": "reiterar",
    "reiterarías": "reiterar",
    "reiteras": "reiterar",
    "reiterase": "reiterar",
    "reiteraseis": "reiterar",
    "reiterasen": "reiterar",
    "reiterases": "reiterar",
    "reiteraste": "reiterar",
    "reiterasteis": "reiterar",
    "reiterativa": "reiterativo",
    "reiterativas": "reiterativo",
    "reiterativos": "reiterativo",
    "reitere": "reiterar",
    "reiteremos": "reiterar",
    "reiteren": "reiterar",
    "reiteres": "reiterar",
    "reitero": "reiterar",
    "reiterábamos": "reiterar",
    "reiteráis": "reiterar",
    "reiteráramos": "reiterar",
    "reiteráremos": "reiterar",
    "reiterásemos": "reiterar",
    "reiteré": "reiterar",
    "reiteréis": "reiterar",
    "reiteró": "reiterar",
    "reitres": "reitre",
    "reivindica": "reivindicar",
    "reivindicaba": "reivindicar",
    "reivindicabais": "reivindicar",
    "reivindicaban": "reivindicar",
    "reivindicabas": "reivindicar",
    "reivindicables": "reivindicable",
    "reivindicaciones": "reivindicación",
    "reivindicad": "reivindicar",
    "reivindicada": "reivindicar",
    "reivindicadas": "reivindicar",
    "reivindicado": "reivindicar",
    "reivindicados": "reivindicar",
    "reivindicamos": "reivindicar",
    "reivindican": "reivindicar",
    "reivindicando": "reivindicar",
    "reivindicara": "reivindicar",
    "reivindicarais": "reivindicar",
    "reivindicaran": "reivindicar",
    "reivindicaras": "reivindicar",
    "reivindicare": "reivindicar",
    "reivindicareis": "reivindicar",
    "reivindicaremos": "reivindicar",
    "reivindicaren": "reivindicar",
    "reivindicares": "reivindicar",
    "reivindicaron": "reivindicar",
    "reivindicará": "reivindicar",
    "reivindicarán": "reivindicar",
    "reivindicarás": "reivindicar",
    "reivindicaré": "reivindicar",
    "reivindicaréis": "reivindicar",
    "reivindicaría": "reivindicar",
    "reivindicaríais": "reivindicar",
    "reivindicaríamos": "reivindicar",
    "reivindicarían": "reivindicar",
    "reivindicarías": "reivindicar",
    "reivindicas": "reivindicar",
    "reivindicase": "reivindicar",
    "reivindicaseis": "reivindicar",
    "reivindicasen": "reivindicar",
    "reivindicases": "reivindicar",
    "reivindicaste": "reivindicar",
    "reivindicasteis": "reivindicar",
    "reivindicativa": "reivindicativo",
    "reivindicativas": "reivindicativo",
    "reivindicativos": "reivindicativo",
    "reivindicatoria": "reivindicatorio",
    "reivindicatorias": "reivindicatorio",
    "reivindicatorios": "reivindicatorio",
    "reivindico": "reivindicar",
    "reivindicábamos": "reivindicar",
    "reivindicáis": "reivindicar",
    "reivindicáramos": "reivindicar",
    "reivindicáremos": "reivindicar",
    "reivindicásemos": "reivindicar",
    "reivindicó": "reivindicar",
    "reivindique": "reivindicar",
    "reivindiquemos": "reivindicar",
    "reivindiquen": "reivindicar",
    "reivindiques": "reivindicar",
    "reivindiqué": "reivindicar",
    "reivindiquéis": "reivindicar",
    "rejaca": "rejacar",
    "rejacaba": "rejacar",
    "rejacabais": "rejacar",
    "rejacaban": "rejacar",
    "rejacabas": "rejacar",
    "rejacad": "rejacar",
    "rejacada": "rejacar",
    "rejacadas": "rejacar",
    "rejacado": "rejacar",
    "rejacados": "rejacar",
    "rejacamos": "rejacar",
    "rejacan": "rejacar",
    "rejacando": "rejacar",
    "rejacara": "rejacar",
    "rejacarais": "rejacar",
    "rejacaran": "rejacar",
    "rejacaras": "rejacar",
    "rejacare": "rejacar",
    "rejacareis": "rejacar",
    "rejacaremos": "rejacar",
    "rejacaren": "rejacar",
    "rejacares": "rejacar",
    "rejacaron": "rejacar",
    "rejacará": "rejacar",
    "rejacarán": "rejacar",
    "rejacarás": "rejacar",
    "rejacaré": "rejacar",
    "rejacaréis": "rejacar",
    "rejacaría": "rejacar",
    "rejacaríais": "rejacar",
    "rejacaríamos": "rejacar",
    "rejacarían": "rejacar",
    "rejacarías": "rejacar",
    "rejacas": "rejacar",
    "rejacase": "rejacar",
    "rejacaseis": "rejacar",
    "rejacasen": "rejacar",
    "rejacases": "rejacar",
    "rejacaste": "rejacar",
    "rejacasteis": "rejacar",
    "rejaco": "rejacar",
    "rejacábamos": "rejacar",
    "rejacáis": "rejacar",
    "rejacáramos": "rejacar",
    "rejacáremos": "rejacar",
    "rejacásemos": "rejacar",
    "rejacó": "rejacar",
    "rejadas": "rejada",
    "rejados": "rejado",
    "rejales": "rejal",
    "rejalgares": "rejalgar",
    "rejaque": "rejacar",
    "rejaquemos": "rejacar",
    "rejaquen": "rejacar",
    "rejaques": "rejacar",
    "rejaqué": "rejacar",
    "rejaquéis": "rejacar",
    "rejas": "reja",
    "rejera": "rejero",
    "rejeras": "rejero",
    "rejeros": "rejero",
    "rejerías": "rejería",
    "rejillas": "rejilla",
    "rejonazos": "rejonazo",
    "rejonea": "rejonear",
    "rejoneaba": "rejonear",
    "rejoneabais": "rejonear",
    "rejoneaban": "rejonear",
    "rejoneabas": "rejonear",
    "rejonead": "rejonear",
    "rejoneada": "rejonear",
    "rejoneadas": "rejonear",
    "rejoneado": "rejonear",
    "rejoneadora": "rejoneador",
    "rejoneadoras": "rejoneador",
    "rejoneadores": "rejoneador",
    "rejoneados": "rejonear",
    "rejoneamos": "rejonear",
    "rejonean": "rejonear",
    "rejoneando": "rejonear",
    "rejoneara": "rejonear",
    "rejonearais": "rejonear",
    "rejonearan": "rejonear",
    "rejonearas": "rejonear",
    "rejoneare": "rejonear",
    "rejoneareis": "rejonear",
    "rejonearemos": "rejonear",
    "rejonearen": "rejonear",
    "rejoneares": "rejonear",
    "rejonearon": "rejonear",
    "rejoneará": "rejonear",
    "rejonearán": "rejonear",
    "rejonearás": "rejonear",
    "rejonearé": "rejonear",
    "rejonearéis": "rejonear",
    "rejonearía": "rejonear",
    "rejonearíais": "rejonear",
    "rejonearíamos": "rejonear",
    "rejonearían": "rejonear",
    "rejonearías": "rejonear",
    "rejoneas": "rejonear",
    "rejonease": "rejonear",
    "rejoneaseis": "rejonear",
    "rejoneasen": "rejonear",
    "rejoneases": "rejonear",
    "rejoneaste": "rejonear",
    "rejoneasteis": "rejonear",
    "rejonee": "rejonear",
    "rejoneemos": "rejonear",
    "rejoneen": "rejonear",
    "rejonees": "rejonear",
    "rejoneo": "rejonear",
    "rejoneos": "rejoneo",
    "rejones": "rejón",
    "rejoneábamos": "rejonear",
    "rejoneáis": "rejonear",
    "rejoneáramos": "rejonear",
    "rejoneáremos": "rejonear",
    "rejoneásemos": "rejonear",
    "rejoneé": "rejonear",
    "rejoneéis": "rejonear",
    "rejoneó": "rejonear",
    "rejos": "rejo",
    "rejuelas": "rejuela",
    "rejunta": "rejuntar",
    "rejuntaba": "rejuntar",
    "rejuntabais": "rejuntar",
    "rejuntaban": "rejuntar",
    "rejuntabas": "rejuntar",
    "rejuntad": "rejuntar",
    "rejuntada": "rejuntar",
    "rejuntadas": "rejuntar",
    "rejuntado": "rejuntar",
    "rejuntados": "rejuntar",
    "rejuntamos": "rejuntar",
    "rejuntan": "rejuntar",
    "rejuntando": "rejuntar",
    "rejuntara": "rejuntar",
    "rejuntarais": "rejuntar",
    "rejuntaran": "rejuntar",
    "rejuntaras": "rejuntar",
    "rejuntare": "rejuntar",
    "rejuntareis": "rejuntar",
    "rejuntaremos": "rejuntar",
    "rejuntaren": "rejuntar",
    "rejuntares": "rejuntar",
    "rejuntaron": "rejuntar",
    "rejuntará": "rejuntar",
    "rejuntarán": "rejuntar",
    "rejuntarás": "rejuntar",
    "rejuntaré": "rejuntar",
    "rejuntaréis": "rejuntar",
    "rejuntaría": "rejuntar",
    "rejuntaríais": "rejuntar",
    "rejuntaríamos": "rejuntar",
    "rejuntarían": "rejuntar",
    "rejuntarías": "rejuntar",
    "rejuntas": "rejuntar",
    "rejuntase": "rejuntar",
    "rejuntaseis": "rejuntar",
    "rejuntasen": "rejuntar",
    "rejuntases": "rejuntar",
    "rejuntaste": "rejuntar",
    "rejuntasteis": "rejuntar",
    "rejunte": "rejuntar",
    "rejuntemos": "rejuntar",
    "rejunten": "rejuntar",
    "rejuntes": "rejuntar",
    "rejunto": "rejuntar",
    "rejuntábamos": "rejuntar",
    "rejuntáis": "rejuntar",
    "rejuntáramos": "rejuntar",
    "rejuntáremos": "rejuntar",
    "rejuntásemos": "rejuntar",
    "rejunté": "rejuntar",
    "rejuntéis": "rejuntar",
    "rejuntó": "rejuntar",
    "rejuvenece": "rejuvenecer",
    "rejuveneced": "rejuvenecer",
    "rejuvenecemos": "rejuvenecer",
    "rejuvenecen": "rejuvenecer",
    "rejuveneceremos": "rejuvenecer",
    "rejuvenecerá": "rejuvenecer",
    "rejuvenecerán": "rejuvenecer",
    "rejuvenecerás": "rejuvenecer",
    "rejuveneceré": "rejuvenecer",
    "rejuveneceréis": "rejuvenecer",
    "rejuvenecería": "rejuvenecer",
    "rejuveneceríais": "rejuvenecer",
    "rejuveneceríamos": "rejuvenecer",
    "rejuvenecerían": "rejuvenecer",
    "rejuvenecerías": "rejuvenecer",
    "rejuveneces": "rejuvenecer",
    "rejuvenecida": "rejuvenecer",
    "rejuvenecidas": "rejuvenecer",
    "rejuvenecido": "rejuvenecer",
    "rejuvenecidos": "rejuvenecer",
    "rejuveneciendo": "rejuvenecer",
    "rejuveneciera": "rejuvenecer",
    "rejuvenecierais": "rejuvenecer",
    "rejuvenecieran": "rejuvenecer",
    "rejuvenecieras": "rejuvenecer",
    "rejuveneciere": "rejuvenecer",
    "rejuveneciereis": "rejuvenecer",
    "rejuvenecieren": "rejuvenecer",
    "rejuvenecieres": "rejuvenecer",
    "rejuvenecieron": "rejuvenecer",
    "rejuveneciese": "rejuvenecer",
    "rejuvenecieseis": "rejuvenecer",
    "rejuveneciesen": "rejuvenecer",
    "rejuvenecieses": "rejuvenecer",
    "rejuvenecimientos": "rejuvenecimiento",
    "rejuvenecimos": "rejuvenecer",
    "rejuveneciste": "rejuvenecer",
    "rejuvenecisteis": "rejuvenecer",
    "rejuveneciéramos": "rejuvenecer",
    "rejuveneciéremos": "rejuvenecer",
    "rejuveneciésemos": "rejuvenecer",
    "rejuveneció": "rejuvenecer",
    "rejuvenecéis": "rejuvenecer",
    "rejuvenecí": "rejuvenecer",
    "rejuvenecía": "rejuvenecer",
    "rejuvenecíais": "rejuvenecer",
    "rejuvenecíamos": "rejuvenecer",
    "rejuvenecían": "rejuvenecer",
    "rejuvenecías": "rejuvenecer",
    "rejuvenezca": "rejuvenecer",
    "rejuvenezcamos": "rejuvenecer",
    "rejuvenezcan": "rejuvenecer",
    "rejuvenezcas": "rejuvenecer",
    "rejuvenezco": "rejuvenecer",
    "rejuvenezcáis": "rejuvenecer",
    "relaciona": "relacionar",
    "relacionaba": "relacionar",
    "relacionabais": "relacionar",
    "relacionaban": "relacionar",
    "relacionabas": "relacionar",
    "relacionad": "relacionar",
    "relacionada": "relacionar",
    "relacionadas": "relacionar",
    "relacionado": "relacionar",
    "relacionados": "relacionar",
    "relacionales": "relacional",
    "relacionamos": "relacionar",
    "relacionan": "relacionar",
    "relacionando": "relacionar",
    "relacionara": "relacionar",
    "relacionarais": "relacionar",
    "relacionaran": "relacionar",
    "relacionaras": "relacionar",
    "relacionare": "relacionar",
    "relacionareis": "relacionar",
    "relacionaremos": "relacionar",
    "relacionaren": "relacionar",
    "relacionares": "relacionar",
    "relacionaron": "relacionar",
    "relacionará": "relacionar",
    "relacionarán": "relacionar",
    "relacionarás": "relacionar",
    "relacionaré": "relacionar",
    "relacionaréis": "relacionar",
    "relacionaría": "relacionar",
    "relacionaríais": "relacionar",
    "relacionaríamos": "relacionar",
    "relacionarían": "relacionar",
    "relacionarías": "relacionar",
    "relacionas": "relacionar",
    "relacionase": "relacionar",
    "relacionaseis": "relacionar",
    "relacionasen": "relacionar",
    "relacionases": "relacionar",
    "relacionaste": "relacionar",
    "relacionasteis": "relacionar",
    "relacione": "relacionar",
    "relacionemos": "relacionar",
    "relacionen": "relacionar",
    "relaciones": "relacionar",
    "relaciono": "relacionar",
    "relacionábamos": "relacionar",
    "relacionáis": "relacionar",
    "relacionáramos": "relacionar",
    "relacionáremos": "relacionar",
    "relacionásemos": "relacionar",
    "relacioné": "relacionar",
    "relacionéis": "relacionar",
    "relacionó": "relacionar",
    "relaja": "relajar",
    "relajaba": "relajar",
    "relajabais": "relajar",
    "relajaban": "relajar",
    "relajabas": "relajar",
    "relajaciones": "relajación",
    "relajad": "relajar",
    "relajada": "relajar",
    "relajadas": "relajar",
    "relajado": "relajar",
    "relajadora": "relajador",
    "relajadoras": "relajador",
    "relajadores": "relajador",
    "relajados": "relajar",
    "relajamientos": "relajamiento",
    "relajamos": "relajar",
    "relajan": "relajar",
    "relajando": "relajar",
    "relajantes": "relajante",
    "relajara": "relajar",
    "relajarais": "relajar",
    "relajaran": "relajar",
    "relajaras": "relajar",
    "relajare": "relajar",
    "relajareis": "relajar",
    "relajaremos": "relajar",
    "relajaren": "relajar",
    "relajares": "relajar",
    "relajaron": "relajar",
    "relajará": "relajar",
    "relajarán": "relajar",
    "relajarás": "relajar",
    "relajaré": "relajar",
    "relajaréis": "relajar",
    "relajaría": "relajar",
    "relajaríais": "relajar",
    "relajaríamos": "relajar",
    "relajarían": "relajar",
    "relajarías": "relajar",
    "relajas": "relajar",
    "relajase": "relajar",
    "relajaseis": "relajar",
    "relajasen": "relajar",
    "relajases": "relajar",
    "relajaste": "relajar",
    "relajasteis": "relajar",
    "relaje": "relajar",
    "relajemos": "relajar",
    "relajen": "relajar",
    "relajes": "relajar",
    "relajo": "relajar",
    "relajos": "relajo",
    "relajábamos": "relajar",
    "relajáis": "relajar",
    "relajáramos": "relajar",
    "relajáremos": "relajar",
    "relajásemos": "relajar",
    "relajé": "relajar",
    "relajéis": "relajar",
    "relajó": "relajar",
    "relama": "relamer",
    "relamamos": "relamer",
    "relaman": "relamer",
    "relamas": "relamer",
    "relame": "relamer",
    "relamed": "relamer",
    "relamemos": "relamer",
    "relamen": "relamer",
    "relameremos": "relamer",
    "relamerá": "relamer",
    "relamerán": "relamer",
    "relamerás": "relamer",
    "relameré": "relamer",
    "relameréis": "relamer",
    "relamería": "relamer",
    "relameríais": "relamer",
    "relameríamos": "relamer",
    "relamerían": "relamer",
    "relamerías": "relamer",
    "relames": "relamer",
    "relamida": "relamido",
    "relamidas": "relamido",
    "relamido": "relamer",
    "relamidos": "relamido",
    "relamiendo": "relamer",
    "relamiera": "relamer",
    "relamierais": "relamer",
    "relamieran": "relamer",
    "relamieras": "relamer",
    "relamiere": "relamer",
    "relamiereis": "relamer",
    "relamieren": "relamer",
    "relamieres": "relamer",
    "relamieron": "relamer",
    "relamiese": "relamer",
    "relamieseis": "relamer",
    "relamiesen": "relamer",
    "relamieses": "relamer",
    "relamimos": "relamer",
    "relamiste": "relamer",
    "relamisteis": "relamer",
    "relamiéramos": "relamer",
    "relamiéremos": "relamer",
    "relamiésemos": "relamer",
    "relamió": "relamer",
    "relamo": "relamer",
    "relampaguea": "relampaguear",
    "relampagueaba": "relampaguear",
    "relampagueabais": "relampaguear",
    "relampagueaban": "relampaguear",
    "relampagueabas": "relampaguear",
    "relampaguead": "relampaguear",
    "relampagueada": "relampaguear",
    "relampagueadas": "relampaguear",
    "relampagueado": "relampaguear",
    "relampagueados": "relampaguear",
    "relampagueamos": "relampaguear",
    "relampaguean": "relampaguear",
    "relampagueando": "relampaguear",
    "relampagueantes": "relampagueante",
    "relampagueara": "relampaguear",
    "relampaguearais": "relampaguear",
    "relampaguearan": "relampaguear",
    "relampaguearas": "relampaguear",
    "relampagueare": "relampaguear",
    "relampagueareis": "relampaguear",
    "relampaguearemos": "relampaguear",
    "relampaguearen": "relampaguear",
    "relampagueares": "relampaguear",
    "relampaguearon": "relampaguear",
    "relampagueará": "relampaguear",
    "relampaguearán": "relampaguear",
    "relampaguearás": "relampaguear",
    "relampaguearé": "relampaguear",
    "relampaguearéis": "relampaguear",
    "relampaguearía": "relampaguear",
    "relampaguearíais": "relampaguear",
    "relampaguearíamos": "relampaguear",
    "relampaguearían": "relampaguear",
    "relampaguearías": "relampaguear",
    "relampagueas": "relampaguear",
    "relampaguease": "relampaguear",
    "relampagueaseis": "relampaguear",
    "relampagueasen": "relampaguear",
    "relampagueases": "relampaguear",
    "relampagueaste": "relampaguear",
    "relampagueasteis": "relampaguear",
    "relampaguee": "relampaguear",
    "relampagueemos": "relampaguear",
    "relampagueen": "relampaguear",
    "relampaguees": "relampaguear",
    "relampagueo": "relampaguear",
    "relampagueos": "relampagueo",
    "relampagueábamos": "relampaguear",
    "relampagueáis": "relampaguear",
    "relampagueáramos": "relampaguear",
    "relampagueáremos": "relampaguear",
    "relampagueásemos": "relampaguear",
    "relampagueé": "relampaguear",
    "relampagueéis": "relampaguear",
    "relampagueó": "relampaguear",
    "relamáis": "relamer",
    "relaméis": "relamer",
    "relamí": "relamer",
    "relamía": "relamer",
    "relamíais": "relamer",
    "relamíamos": "relamer",
    "relamían": "relamer",
    "relamías": "relamer",
    "relance": "relanzar",
    "relancemos": "relanzar",
    "relancen": "relanzar",
    "relances": "relanzar",
    "relancé": "relanzar",
    "relancéis": "relanzar",
    "relanza": "relanzar",
    "relanzaba": "relanzar",
    "relanzabais": "relanzar",
    "relanzaban": "relanzar",
    "relanzabas": "relanzar",
    "relanzad": "relanzar",
    "relanzada": "relanzar",
    "relanzadas": "relanzar",
    "relanzado": "relanzar",
    "relanzados": "relanzar",
    "relanzamientos": "relanzamiento",
    "relanzamos": "relanzar",
    "relanzan": "relanzar",
    "relanzando": "relanzar",
    "relanzara": "relanzar",
    "relanzarais": "relanzar",
    "relanzaran": "relanzar",
    "relanzaras": "relanzar",
    "relanzare": "relanzar",
    "relanzareis": "relanzar",
    "relanzaremos": "relanzar",
    "relanzaren": "relanzar",
    "relanzares": "relanzar",
    "relanzaron": "relanzar",
    "relanzará": "relanzar",
    "relanzarán": "relanzar",
    "relanzarás": "relanzar",
    "relanzaré": "relanzar",
    "relanzaréis": "relanzar",
    "relanzaría": "relanzar",
    "relanzaríais": "relanzar",
    "relanzaríamos": "relanzar",
    "relanzarían": "relanzar",
    "relanzarías": "relanzar",
    "relanzas": "relanzar",
    "relanzase": "relanzar",
    "relanzaseis": "relanzar",
    "relanzasen": "relanzar",
    "relanzases": "relanzar",
    "relanzaste": "relanzar",
    "relanzasteis": "relanzar",
    "relanzo": "relanzar",
    "relanzábamos": "relanzar",
    "relanzáis": "relanzar",
    "relanzáramos": "relanzar",
    "relanzáremos": "relanzar",
    "relanzásemos": "relanzar",
    "relanzó": "relanzar",
    "relapsa": "relapso",
    "relapsas": "relapso",
    "relapsos": "relapso",
    "relata": "relatar",
    "relataba": "relatar",
    "relatabais": "relatar",
    "relataban": "relatar",
    "relatabas": "relatar",
    "relatad": "relatar",
    "relatada": "relatar",
    "relatadas": "relatar",
    "relatado": "relatar",
    "relatadora": "relatador",
    "relatadoras": "relatador",
    "relatadores": "relatador",
    "relatados": "relatar",
    "relatamos": "relatar",
    "relatan": "relatar",
    "relatando": "relatar",
    "relatara": "relatar",
    "relatarais": "relatar",
    "relataran": "relatar",
    "relataras": "relatar",
    "relatare": "relatar",
    "relatareis": "relatar",
    "relataremos": "relatar",
    "relataren": "relatar",
    "relatares": "relatar",
    "relataron": "relatar",
    "relatará": "relatar",
    "relatarán": "relatar",
    "relatarás": "relatar",
    "relataré": "relatar",
    "relataréis": "relatar",
    "relataría": "relatar",
    "relataríais": "relatar",
    "relataríamos": "relatar",
    "relatarían": "relatar",
    "relatarías": "relatar",
    "relatas": "relatar",
    "relatase": "relatar",
    "relataseis": "relatar",
    "relatasen": "relatar",
    "relatases": "relatar",
    "relataste": "relatar",
    "relatasteis": "relatar",
    "relate": "relatar",
    "relatemos": "relatar",
    "relaten": "relatar",
    "relates": "relatar",
    "relativa": "relativo",
    "relativas": "relativo",
    "relativice": "relativizar",
    "relativicemos": "relativizar",
    "relativicen": "relativizar",
    "relativices": "relativizar",
    "relativicé": "relativizar",
    "relativicéis": "relativizar",
    "relatividades": "relatividad",
    "relativismos": "relativismo",
    "relativistas": "relativista",
    "relativiza": "relativizar",
    "relativizaba": "relativizar",
    "relativizabais": "relativizar",
    "relativizaban": "relativizar",
    "relativizabas": "relativizar",
    "relativizaciones": "relativización",
    "relativizad": "relativizar",
    "relativizada": "relativizar",
    "relativizadas": "relativizar",
    "relativizado": "relativizar",
    "relativizados": "relativizar",
    "relativizamos": "relativizar",
    "relativizan": "relativizar",
    "relativizando": "relativizar",
    "relativizara": "relativizar",
    "relativizarais": "relativizar",
    "relativizaran": "relativizar",
    "relativizaras": "relativizar",
    "relativizare": "relativizar",
    "relativizareis": "relativizar",
    "relativizaremos": "relativizar",
    "relativizaren": "relativizar",
    "relativizares": "relativizar",
    "relativizaron": "relativizar",
    "relativizará": "relativizar",
    "relativizarán": "relativizar",
    "relativizarás": "relativizar",
    "relativizaré": "relativizar",
    "relativizaréis": "relativizar",
    "relativizaría": "relativizar",
    "relativizaríais": "relativizar",
    "relativizaríamos": "relativizar",
    "relativizarían": "relativizar",
    "relativizarías": "relativizar",
    "relativizas": "relativizar",
    "relativizase": "relativizar",
    "relativizaseis": "relativizar",
    "relativizasen": "relativizar",
    "relativizases": "relativizar",
    "relativizaste": "relativizar",
    "relativizasteis": "relativizar",
    "relativizo": "relativizar",
    "relativizábamos": "relativizar",
    "relativizáis": "relativizar",
    "relativizáramos": "relativizar",
    "relativizáremos": "relativizar",
    "relativizásemos": "relativizar",
    "relativizó": "relativizar",
    "relativos": "relativo",
    "relato": "relatar",
    "relatora": "relator",
    "relatoras": "relator",
    "relatores": "relator",
    "relatorías": "relatoría",
    "relatos": "relato",
    "relatábamos": "relatar",
    "relatáis": "relatar",
    "relatáramos": "relatar",
    "relatáremos": "relatar",
    "relatásemos": "relatar",
    "relaté": "relatar",
    "relatéis": "relatar",
    "relató": "relatar",
    "relaxes": "relax",
    "relea": "releer",
    "releamos": "releer",
    "relean": "releer",
    "releas": "releer",
    "releases": "release",
    "relee": "releer",
    "releed": "releer",
    "releemos": "releer",
    "releen": "releer",
    "releeremos": "releer",
    "releerá": "releer",
    "releerán": "releer",
    "releerás": "releer",
    "releeré": "releer",
    "releeréis": "releer",
    "releería": "releer",
    "releeríais": "releer",
    "releeríamos": "releer",
    "releerían": "releer",
    "releerías": "releer",
    "relees": "releer",
    "relega": "relegar",
    "relegaba": "relegar",
    "relegabais": "relegar",
    "relegaban": "relegar",
    "relegabas": "relegar",
    "relegaciones": "relegación",
    "relegad": "relegar",
    "relegada": "relegar",
    "relegadas": "relegar",
    "relegado": "relegar",
    "relegados": "relegar",
    "relegamos": "relegar",
    "relegan": "relegar",
    "relegando": "relegar",
    "relegara": "relegar",
    "relegarais": "relegar",
    "relegaran": "relegar",
    "relegaras": "relegar",
    "relegare": "relegar",
    "relegareis": "relegar",
    "relegaremos": "relegar",
    "relegaren": "relegar",
    "relegares": "relegar",
    "relegaron": "relegar",
    "relegará": "relegar",
    "relegarán": "relegar",
    "relegarás": "relegar",
    "relegaré": "relegar",
    "relegaréis": "relegar",
    "relegaría": "relegar",
    "relegaríais": "relegar",
    "relegaríamos": "relegar",
    "relegarían": "relegar",
    "relegarías": "relegar",
    "relegas": "relegar",
    "relegase": "relegar",
    "relegaseis": "relegar",
    "relegasen": "relegar",
    "relegases": "relegar",
    "relegaste": "relegar",
    "relegasteis": "relegar",
    "relego": "relegar",
    "relegue": "relegar",
    "releguemos": "relegar",
    "releguen": "relegar",
    "relegues": "relegar",
    "relegué": "relegar",
    "releguéis": "relegar",
    "relegábamos": "relegar",
    "relegáis": "relegar",
    "relegáramos": "relegar",
    "relegáremos": "relegar",
    "relegásemos": "relegar",
    "relegó": "relegar",
    "relejes": "releje",
    "relentes": "relente",
    "relentizaciones": "relentización",
    "releo": "releer",
    "releva": "relevar",
    "relevaba": "relevar",
    "relevabais": "relevar",
    "relevaban": "relevar",
    "relevabas": "relevar",
    "relevaciones": "relevación",
    "relevad": "relevar",
    "relevada": "relevar",
    "relevadas": "relevar",
    "relevado": "relevar",
    "relevadores": "relevador",
    "relevados": "relevar",
    "relevamos": "relevar",
    "relevan": "relevar",
    "relevancias": "relevancia",
    "relevando": "relevar",
    "relevantes": "relevante",
    "relevara": "relevar",
    "relevarais": "relevar",
    "relevaran": "relevar",
    "relevaras": "relevar",
    "relevare": "relevar",
    "relevareis": "relevar",
    "relevaremos": "relevar",
    "relevaren": "relevar",
    "relevares": "relevar",
    "relevaron": "relevar",
    "relevará": "relevar",
    "relevarán": "relevar",
    "relevarás": "relevar",
    "relevaré": "relevar",
    "relevaréis": "relevar",
    "relevaría": "relevar",
    "relevaríais": "relevar",
    "relevaríamos": "relevar",
    "relevarían": "relevar",
    "relevarías": "relevar",
    "relevas": "relevar",
    "relevase": "relevar",
    "relevaseis": "relevar",
    "relevasen": "relevar",
    "relevases": "relevar",
    "relevaste": "relevar",
    "relevasteis": "relevar",
    "releve": "relevar",
    "relevemos": "relevar",
    "releven": "relevar",
    "releves": "relevar",
    "relevistas": "relevista",
    "relevo": "relevar",
    "relevos": "relevo",
    "relevábamos": "relevar",
    "releváis": "relevar",
    "releváramos": "relevar",
    "releváremos": "relevar",
    "relevásemos": "relevar",
    "relevé": "relevar",
    "relevéis": "relevar",
    "relevó": "relevar",
    "releyendo": "releer",
    "releyera": "releer",
    "releyerais": "releer",
    "releyeran": "releer",
    "releyeras": "releer",
    "releyere": "releer",
    "releyereis": "releer",
    "releyeren": "releer",
    "releyeres": "releer",
    "releyeron": "releer",
    "releyese": "releer",
    "releyeseis": "releer",
    "releyesen": "releer",
    "releyeses": "releer",
    "releyéramos": "releer",
    "releyéremos": "releer",
    "releyésemos": "releer",
    "releyó": "releer",
    "releáis": "releer",
    "releéis": "releer",
    "releí": "releer",
    "releía": "releer",
    "releíais": "releer",
    "releíamos": "releer",
    "releían": "releer",
    "releías": "releer",
    "releída": "releer",
    "releídas": "releer",
    "releído": "releer",
    "releídos": "releer",
    "releímos": "releer",
    "releíste": "releer",
    "releísteis": "releer",
    "relicarios": "relicario",
    "relicta": "relicto",
    "relictas": "relicto",
    "relictos": "relicto",
    "relieves": "relieve",
    "religa": "religar",
    "religaba": "religar",
    "religabais": "religar",
    "religaban": "religar",
    "religabas": "religar",
    "religaciones": "religación",
    "religad": "religar",
    "religada": "religar",
    "religadas": "religar",
    "religado": "religar",
    "religados": "religar",
    "religamos": "religar",
    "religan": "religar",
    "religando": "religar",
    "religara": "religar",
    "religarais": "religar",
    "religaran": "religar",
    "religaras": "religar",
    "religare": "religar",
    "religareis": "religar",
    "religaremos": "religar",
    "religaren": "religar",
    "religares": "religar",
    "religaron": "religar",
    "religará": "religar",
    "religarán": "religar",
    "religarás": "religar",
    "religaré": "religar",
    "religaréis": "religar",
    "religaría": "religar",
    "religaríais": "religar",
    "religaríamos": "religar",
    "religarían": "religar",
    "religarías": "religar",
    "religas": "religar",
    "religase": "religar",
    "religaseis": "religar",
    "religasen": "religar",
    "religases": "religar",
    "religaste": "religar",
    "religasteis": "religar",
    "religiones": "religión",
    "religiosa": "religioso",
    "religiosas": "religioso",
    "religiosidades": "religiosidad",
    "religiosos": "religioso",
    "religo": "religar",
    "religue": "religar",
    "religuemos": "religar",
    "religuen": "religar",
    "religues": "religar",
    "religué": "religar",
    "religuéis": "religar",
    "religábamos": "religar",
    "religáis": "religar",
    "religáramos": "religar",
    "religáremos": "religar",
    "religásemos": "religar",
    "religó": "religar",
    "relimpia": "relimpio",
    "relimpias": "relimpio",
    "relimpios": "relimpio",
    "relincha": "relinchar",
    "relinchaba": "relinchar",
    "relinchabais": "relinchar",
    "relinchaban": "relinchar",
    "relinchabas": "relinchar",
    "relinchad": "relinchar",
    "relinchada": "relinchar",
    "relinchadas": "relinchar",
    "relinchado": "relinchar",
    "relinchados": "relinchar",
    "relinchamos": "relinchar",
    "relinchan": "relinchar",
    "relinchando": "relinchar",
    "relinchara": "relinchar",
    "relincharais": "relinchar",
    "relincharan": "relinchar",
    "relincharas": "relinchar",
    "relinchare": "relinchar",
    "relinchareis": "relinchar",
    "relincharemos": "relinchar",
    "relincharen": "relinchar",
    "relinchares": "relinchar",
    "relincharon": "relinchar",
    "relinchará": "relinchar",
    "relincharán": "relinchar",
    "relincharás": "relinchar",
    "relincharé": "relinchar",
    "relincharéis": "relinchar",
    "relincharía": "relinchar",
    "relincharíais": "relinchar",
    "relincharíamos": "relinchar",
    "relincharían": "relinchar",
    "relincharías": "relinchar",
    "relinchas": "relinchar",
    "relinchase": "relinchar",
    "relinchaseis": "relinchar",
    "relinchasen": "relinchar",
    "relinchases": "relinchar",
    "relinchaste": "relinchar",
    "relinchasteis": "relinchar",
    "relinche": "relinchar",
    "relinchemos": "relinchar",
    "relinchen": "relinchar",
    "relinches": "relinchar",
    "relinchidos": "relinchido",
    "relincho": "relinchar",
    "relinchos": "relincho",
    "relinchábamos": "relinchar",
    "relincháis": "relinchar",
    "relincháramos": "relinchar",
    "relincháremos": "relinchar",
    "relinchásemos": "relinchar",
    "relinché": "relinchar",
    "relinchéis": "relinchar",
    "relinchó": "relinchar",
    "relinga": "relingar",
    "relingaba": "relingar",
    "relingabais": "relingar",
    "relingaban": "relingar",
    "relingabas": "relingar",
    "relingad": "relingar",
    "relingada": "relingar",
    "relingadas": "relingar",
    "relingado": "relingar",
    "relingados": "relingar",
    "relingamos": "relingar",
    "relingan": "relingar",
    "relingando": "relingar",
    "relingara": "relingar",
    "relingarais": "relingar",
    "relingaran": "relingar",
    "relingaras": "relingar",
    "relingare": "relingar",
    "relingareis": "relingar",
    "relingaremos": "relingar",
    "relingaren": "relingar",
    "relingares": "relingar",
    "relingaron": "relingar",
    "relingará": "relingar",
    "relingarán": "relingar",
    "relingarás": "relingar",
    "relingaré": "relingar",
    "relingaréis": "relingar",
    "relingaría": "relingar",
    "relingaríais": "relingar",
    "relingaríamos": "relingar",
    "relingarían": "relingar",
    "relingarías": "relingar",
    "relingas": "relingar",
    "relingase": "relingar",
    "relingaseis": "relingar",
    "relingasen": "relingar",
    "relingases": "relingar",
    "relingaste": "relingar",
    "relingasteis": "relingar",
    "relingo": "relingar",
    "relingue": "relingar",
    "relinguemos": "relingar",
    "relinguen": "relingar",
    "relingues": "relingar",
    "relingué": "relingar",
    "relinguéis": "relingar",
    "relingábamos": "relingar",
    "relingáis": "relingar",
    "relingáramos": "relingar",
    "relingáremos": "relingar",
    "relingásemos": "relingar",
    "relingó": "relingar",
    "reliquias": "reliquia",
    "rellamadas": "rellamada",
    "rellana": "rellanar",
    "rellanaba": "rellanar",
    "rellanabais": "rellanar",
    "rellanaban": "rellanar",
    "rellanabas": "rellanar",
    "rellanad": "rellanar",
    "rellanada": "rellanar",
    "rellanadas": "rellanar",
    "rellanado": "rellanar",
    "rellanados": "rellanar",
    "rellanamos": "rellanar",
    "rellanan": "rellanar",
    "rellanando": "rellanar",
    "rellanara": "rellanar",
    "rellanarais": "rellanar",
    "rellanaran": "rellanar",
    "rellanaras": "rellanar",
    "rellanare": "rellanar",
    "rellanareis": "rellanar",
    "rellanaremos": "rellanar",
    "rellanaren": "rellanar",
    "rellanares": "rellanar",
    "rellanaron": "rellanar",
    "rellanará": "rellanar",
    "rellanarán": "rellanar",
    "rellanarás": "rellanar",
    "rellanaré": "rellanar",
    "rellanaréis": "rellanar",
    "rellanaría": "rellanar",
    "rellanaríais": "rellanar",
    "rellanaríamos": "rellanar",
    "rellanarían": "rellanar",
    "rellanarías": "rellanar",
    "rellanas": "rellanar",
    "rellanase": "rellanar",
    "rellanaseis": "rellanar",
    "rellanasen": "rellanar",
    "rellanases": "rellanar",
    "rellanaste": "rellanar",
    "rellanasteis": "rellanar",
    "rellane": "rellanar",
    "rellanemos": "rellanar",
    "rellanen": "rellanar",
    "rellanes": "rellanar",
    "rellano": "rellanar",
    "rellanos": "rellano",
    "rellanábamos": "rellanar",
    "rellanáis": "rellanar",
    "rellanáramos": "rellanar",
    "rellanáremos": "rellanar",
    "rellanásemos": "rellanar",
    "rellané": "rellanar",
    "rellanéis": "rellanar",
    "rellanó": "rellanar",
    "rellena": "relleno",
    "rellenaba": "rellenar",
    "rellenabais": "rellenar",
    "rellenaban": "rellenar",
    "rellenabas": "rellenar",
    "rellenables": "rellenable",
    "rellenad": "rellenar",
    "rellenada": "rellenar",
    "rellenadas": "rellenar",
    "rellenado": "rellenar",
    "rellenadores": "rellenador",
    "rellenados": "rellenar",
    "rellenamos": "rellenar",
    "rellenan": "rellenar",
    "rellenando": "rellenar",
    "rellenara": "rellenar",
    "rellenarais": "rellenar",
    "rellenaran": "rellenar",
    "rellenaras": "rellenar",
    "rellenare": "rellenar",
    "rellenareis": "rellenar",
    "rellenaremos": "rellenar",
    "rellenaren": "rellenar",
    "rellenares": "rellenar",
    "rellenaron": "rellenar",
    "rellenará": "rellenar",
    "rellenarán": "rellenar",
    "rellenarás": "rellenar",
    "rellenaré": "rellenar",
    "rellenaréis": "rellenar",
    "rellenaría": "rellenar",
    "rellenaríais": "rellenar",
    "rellenaríamos": "rellenar",
    "rellenarían": "rellenar",
    "rellenarías": "rellenar",
    "rellenas": "relleno",
    "rellenase": "rellenar",
    "rellenaseis": "rellenar",
    "rellenasen": "rellenar",
    "rellenases": "rellenar",
    "rellenaste": "rellenar",
    "rellenasteis": "rellenar",
    "rellene": "rellenar",
    "rellenemos": "rellenar",
    "rellenen": "rellenar",
    "rellenes": "rellenar",
    "relleno": "rellenar",
    "rellenos": "relleno",
    "rellenábamos": "rellenar",
    "rellenáis": "rellenar",
    "rellenáramos": "rellenar",
    "rellenáremos": "rellenar",
    "rellenásemos": "rellenar",
    "rellené": "rellenar",
    "rellenéis": "rellenar",
    "rellenó": "rellenar",
    "relocalizables": "relocalizable",
    "relocalizaciones": "relocalización",
    "relojera": "relojero",
    "relojeras": "relojero",
    "relojeros": "relojero",
    "relojerías": "relojería",
    "relojes": "reloj",
    "reluce": "relucir",
    "relucen": "relucir",
    "reluces": "relucir",
    "relucid": "relucir",
    "relucida": "relucir",
    "relucidas": "relucir",
    "relucido": "relucir",
    "relucidos": "relucir",
    "reluciendo": "relucir",
    "relucientes": "reluciente",
    "reluciera": "relucir",
    "relucierais": "relucir",
    "relucieran": "relucir",
    "relucieras": "relucir",
    "reluciere": "relucir",
    "reluciereis": "relucir",
    "relucieren": "relucir",
    "relucieres": "relucir",
    "relucieron": "relucir",
    "reluciese": "relucir",
    "relucieseis": "relucir",
    "reluciesen": "relucir",
    "relucieses": "relucir",
    "relucimos": "relucir",
    "reluciremos": "relucir",
    "relucirá": "relucir",
    "relucirán": "relucir",
    "relucirás": "relucir",
    "reluciré": "relucir",
    "reluciréis": "relucir",
    "reluciría": "relucir",
    "reluciríais": "relucir",
    "reluciríamos": "relucir",
    "relucirían": "relucir",
    "relucirías": "relucir",
    "reluciste": "relucir",
    "relucisteis": "relucir",
    "reluciéramos": "relucir",
    "reluciéremos": "relucir",
    "reluciésemos": "relucir",
    "relució": "relucir",
    "reluctancias": "reluctancia",
    "reluctantes": "reluctante",
    "reluctividades": "reluctividad",
    "relucí": "relucir",
    "relucía": "relucir",
    "relucíais": "relucir",
    "relucíamos": "relucir",
    "relucían": "relucir",
    "relucías": "relucir",
    "relucís": "relucir",
    "relumbra": "relumbrar",
    "relumbraba": "relumbrar",
    "relumbrabais": "relumbrar",
    "relumbraban": "relumbrar",
    "relumbrabas": "relumbrar",
    "relumbrad": "relumbrar",
    "relumbrada": "relumbrar",
    "relumbradas": "relumbrar",
    "relumbrado": "relumbrar",
    "relumbrados": "relumbrar",
    "relumbramos": "relumbrar",
    "relumbran": "relumbrar",
    "relumbrando": "relumbrar",
    "relumbrara": "relumbrar",
    "relumbrarais": "relumbrar",
    "relumbraran": "relumbrar",
    "relumbraras": "relumbrar",
    "relumbrare": "relumbrar",
    "relumbrareis": "relumbrar",
    "relumbraremos": "relumbrar",
    "relumbraren": "relumbrar",
    "relumbrares": "relumbrar",
    "relumbraron": "relumbrar",
    "relumbrará": "relumbrar",
    "relumbrarán": "relumbrar",
    "relumbrarás": "relumbrar",
    "relumbraré": "relumbrar",
    "relumbraréis": "relumbrar",
    "relumbraría": "relumbrar",
    "relumbraríais": "relumbrar",
    "relumbraríamos": "relumbrar",
    "relumbrarían": "relumbrar",
    "relumbrarías": "relumbrar",
    "relumbras": "relumbrar",
    "relumbrase": "relumbrar",
    "relumbraseis": "relumbrar",
    "relumbrasen": "relumbrar",
    "relumbrases": "relumbrar",
    "relumbraste": "relumbrar",
    "relumbrasteis": "relumbrar",
    "relumbre": "relumbrar",
    "relumbremos": "relumbrar",
    "relumbren": "relumbrar",
    "relumbres": "relumbre",
    "relumbro": "relumbrar",
    "relumbrones": "relumbrón",
    "relumbros": "relumbro",
    "relumbrosa": "relumbroso",
    "relumbrosas": "relumbroso",
    "relumbrosos": "relumbroso",
    "relumbrábamos": "relumbrar",
    "relumbráis": "relumbrar",
    "relumbráramos": "relumbrar",
    "relumbráremos": "relumbrar",
    "relumbrásemos": "relumbrar",
    "relumbré": "relumbrar",
    "relumbréis": "relumbrar",
    "relumbró": "relumbrar",
    "reluzca": "relucir",
    "reluzcamos": "relucir",
    "reluzcan": "relucir",
    "reluzcas": "relucir",
    "reluzco": "relucir",
    "reluzcáis": "relucir",
    "relvas": "relva",
    "relámpagos": "relámpago",
    "relés": "relé",
    "relíctica": "relíctico",
    "relícticas": "relíctico",
    "relícticos": "relíctico",
    "rema": "remar",
    "remaba": "remar",
    "remabais": "remar",
    "remaban": "remar",
    "remabas": "remar",
    "remacha": "remachar",
    "remachaba": "remachar",
    "remachabais": "remachar",
    "remachaban": "remachar",
    "remachabas": "remachar",
    "remachad": "remachar",
    "remachada": "remachar",
    "remachadas": "remachar",
    "remachado": "remachar",
    "remachadora": "remachador",
    "remachadoras": "remachadora",
    "remachadores": "remachador",
    "remachados": "remachar",
    "remachamos": "remachar",
    "remachan": "remachar",
    "remachando": "remachar",
    "remachara": "remachar",
    "remacharais": "remachar",
    "remacharan": "remachar",
    "remacharas": "remachar",
    "remachare": "remachar",
    "remachareis": "remachar",
    "remacharemos": "remachar",
    "remacharen": "remachar",
    "remachares": "remachar",
    "remacharon": "remachar",
    "remachará": "remachar",
    "remacharán": "remachar",
    "remacharás": "remachar",
    "remacharé": "remachar",
    "remacharéis": "remachar",
    "remacharía": "remachar",
    "remacharíais": "remachar",
    "remacharíamos": "remachar",
    "remacharían": "remachar",
    "remacharías": "remachar",
    "remachas": "remachar",
    "remachase": "remachar",
    "remachaseis": "remachar",
    "remachasen": "remachar",
    "remachases": "remachar",
    "remachaste": "remachar",
    "remachasteis": "remachar",
    "remache": "remachar",
    "remachemos": "remachar",
    "remachen": "remachar",
    "remaches": "remache",
    "remacho": "remachar",
    "remachábamos": "remachar",
    "remacháis": "remachar",
    "remacháramos": "remachar",
    "remacháremos": "remachar",
    "remachásemos": "remachar",
    "remaché": "remachar",
    "remachéis": "remachar",
    "remachó": "remachar",
    "remad": "remar",
    "remada": "remar",
    "remadas": "remar",
    "remado": "remar",
    "remadora": "remador",
    "remadoras": "remador",
    "remadores": "remador",
    "remados": "remar",
    "remaduras": "remadura",
    "remagnetizaciones": "remagnetización",
    "remakes": "remake",
    "remalla": "remallar",
    "remallaba": "remallar",
    "remallabais": "remallar",
    "remallaban": "remallar",
    "remallabas": "remallar",
    "remallad": "remallar",
    "remallada": "remallar",
    "remalladas": "remallar",
    "remallado": "remallar",
    "remallados": "remallar",
    "remallamos": "remallar",
    "remallan": "remallar",
    "remallando": "remallar",
    "remallara": "remallar",
    "remallarais": "remallar",
    "remallaran": "remallar",
    "remallaras": "remallar",
    "remallare": "remallar",
    "remallareis": "remallar",
    "remallaremos": "remallar",
    "remallaren": "remallar",
    "remallares": "remallar",
    "remallaron": "remallar",
    "remallará": "remallar",
    "remallarán": "remallar",
    "remallarás": "remallar",
    "remallaré": "remallar",
    "remallaréis": "remallar",
    "remallaría": "remallar",
    "remallaríais": "remallar",
    "remallaríamos": "remallar",
    "remallarían": "remallar",
    "remallarías": "remallar",
    "remallas": "remallar",
    "remallase": "remallar",
    "remallaseis": "remallar",
    "remallasen": "remallar",
    "remallases": "remallar",
    "remallaste": "remallar",
    "remallasteis": "remallar",
    "remalle": "remallar",
    "remallemos": "remallar",
    "remallen": "remallar",
    "remalles": "remallar",
    "remallo": "remallar",
    "remallábamos": "remallar",
    "remalláis": "remallar",
    "remalláramos": "remallar",
    "remalláremos": "remallar",
    "remallásemos": "remallar",
    "remallé": "remallar",
    "remalléis": "remallar",
    "remalló": "remallar",
    "remamientos": "remamiento",
    "remamos": "remar",
    "reman": "remar",
    "remando": "remar",
    "remanencias": "remanencia",
    "remanentes": "remanente",
    "remanga": "remangar",
    "remangaba": "remangar",
    "remangabais": "remangar",
    "remangaban": "remangar",
    "remangabas": "remangar",
    "remangad": "remangar",
    "remangada": "remangar",
    "remangadas": "remangar",
    "remangado": "remangar",
    "remangados": "remangar",
    "remangamos": "remangar",
    "remangan": "remangar",
    "remangando": "remangar",
    "remangara": "remangar",
    "remangarais": "remangar",
    "remangaran": "remangar",
    "remangaras": "remangar",
    "remangare": "remangar",
    "remangareis": "remangar",
    "remangaremos": "remangar",
    "remangaren": "remangar",
    "remangares": "remangar",
    "remangaron": "remangar",
    "remangará": "remangar",
    "remangarán": "remangar",
    "remangarás": "remangar",
    "remangaré": "remangar",
    "remangaréis": "remangar",
    "remangaría": "remangar",
    "remangaríais": "remangar",
    "remangaríamos": "remangar",
    "remangarían": "remangar",
    "remangarías": "remangar",
    "remangas": "remangar",
    "remangase": "remangar",
    "remangaseis": "remangar",
    "remangasen": "remangar",
    "remangases": "remangar",
    "remangaste": "remangar",
    "remangasteis": "remangar",
    "remango": "remangar",
    "remangos": "remango",
    "remangue": "remangar",
    "remanguemos": "remangar",
    "remanguen": "remangar",
    "remangues": "remangar",
    "remangué": "remangar",
    "remanguéis": "remangar",
    "remangábamos": "remangar",
    "remangáis": "remangar",
    "remangáramos": "remangar",
    "remangáremos": "remangar",
    "remangásemos": "remangar",
    "remangó": "remangar",
    "remansa": "remansar",
    "remansaba": "remansar",
    "remansabais": "remansar",
    "remansaban": "remansar",
    "remansabas": "remansar",
    "remansad": "remansar",
    "remansada": "remansar",
    "remansadas": "remansar",
    "remansado": "remansar",
    "remansados": "remansar",
    "remansamos": "remansar",
    "remansan": "remansar",
    "remansando": "remansar",
    "remansara": "remansar",
    "remansarais": "remansar",
    "remansaran": "remansar",
    "remansaras": "remansar",
    "remansare": "remansar",
    "remansareis": "remansar",
    "remansaremos": "remansar",
    "remansaren": "remansar",
    "remansares": "remansar",
    "remansaron": "remansar",
    "remansará": "remansar",
    "remansarán": "remansar",
    "remansarás": "remansar",
    "remansaré": "remansar",
    "remansaréis": "remansar",
    "remansaría": "remansar",
    "remansaríais": "remansar",
    "remansaríamos": "remansar",
    "remansarían": "remansar",
    "remansarías": "remansar",
    "remansas": "remansar",
    "remansase": "remansar",
    "remansaseis": "remansar",
    "remansasen": "remansar",
    "remansases": "remansar",
    "remansaste": "remansar",
    "remansasteis": "remansar",
    "remanse": "remansar",
    "remansemos": "remansar",
    "remansen": "remansar",
    "remanses": "remansar",
    "remanso": "remansar",
    "remansos": "remanso",
    "remansábamos": "remansar",
    "remansáis": "remansar",
    "remansáramos": "remansar",
    "remansáremos": "remansar",
    "remansásemos": "remansar",
    "remansé": "remansar",
    "remanséis": "remansar",
    "remansó": "remansar",
    "remara": "remar",
    "remarais": "remar",
    "remaran": "remar",
    "remaras": "remar",
    "remarca": "remarcar",
    "remarcaba": "remarcar",
    "remarcabais": "remarcar",
    "remarcaban": "remarcar",
    "remarcabas": "remarcar",
    "remarcables": "remarcable",
    "remarcad": "remarcar",
    "remarcada": "remarcar",
    "remarcadas": "remarcar",
    "remarcado": "remarcar",
    "remarcados": "remarcar",
    "remarcamos": "remarcar",
    "remarcan": "remarcar",
    "remarcando": "remarcar",
    "remarcara": "remarcar",
    "remarcarais": "remarcar",
    "remarcaran": "remarcar",
    "remarcaras": "remarcar",
    "remarcare": "remarcar",
    "remarcareis": "remarcar",
    "remarcaremos": "remarcar",
    "remarcaren": "remarcar",
    "remarcares": "remarcar",
    "remarcaron": "remarcar",
    "remarcará": "remarcar",
    "remarcarán": "remarcar",
    "remarcarás": "remarcar",
    "remarcaré": "remarcar",
    "remarcaréis": "remarcar",
    "remarcaría": "remarcar",
    "remarcaríais": "remarcar",
    "remarcaríamos": "remarcar",
    "remarcarían": "remarcar",
    "remarcarías": "remarcar",
    "remarcas": "remarcar",
    "remarcase": "remarcar",
    "remarcaseis": "remarcar",
    "remarcasen": "remarcar",
    "remarcases": "remarcar",
    "remarcaste": "remarcar",
    "remarcasteis": "remarcar",
    "remarco": "remarcar",
    "remarcábamos": "remarcar",
    "remarcáis": "remarcar",
    "remarcáramos": "remarcar",
    "remarcáremos": "remarcar",
    "remarcásemos": "remarcar",
    "remarcó": "remarcar",
    "remare": "remar",
    "remareis": "remar",
    "remaremos": "remar",
    "remaren": "remar",
    "remares": "remar",
    "remaron": "remar",
    "remarque": "remarcar",
    "remarquemos": "remarcar",
    "remarquen": "remarcar",
    "remarques": "remarcar",
    "remarqué": "remarcar",
    "remarquéis": "remarcar",
    "remará": "remar",
    "remarán": "remar",
    "remarás": "remar",
    "remaré": "remar",
    "remaréis": "remar",
    "remaría": "remar",
    "remaríais": "remar",
    "remaríamos": "remar",
    "remarían": "remar",
    "remarías": "remar",
    "remas": "remar",
    "remasas": "remasa",
    "remase": "remar",
    "remaseis": "remar",
    "remasen": "remar",
    "remases": "remar",
    "remaste": "remar",
    "remasteis": "remar",
    "remata": "rematar",
    "remataba": "rematar",
    "rematabais": "rematar",
    "remataban": "rematar",
    "rematabas": "rematar",
    "rematad": "rematar",
    "rematada": "rematar",
    "rematadas": "rematar",
    "rematado": "rematar",
    "rematadora": "rematador",
    "rematadoras": "rematador",
    "rematadores": "rematador",
    "rematados": "rematar",
    "rematamos": "rematar",
    "rematan": "rematar",
    "rematando": "rematar",
    "rematara": "rematar",
    "rematarais": "rematar",
    "remataran": "rematar",
    "remataras": "rematar",
    "rematare": "rematar",
    "rematareis": "rematar",
    "remataremos": "rematar",
    "remataren": "rematar",
    "rematares": "rematar",
    "remataron": "rematar",
    "rematará": "rematar",
    "rematarán": "rematar",
    "rematarás": "rematar",
    "remataré": "rematar",
    "remataréis": "rematar",
    "remataría": "rematar",
    "remataríais": "rematar",
    "remataríamos": "rematar",
    "rematarían": "rematar",
    "rematarías": "rematar",
    "rematas": "rematar",
    "rematase": "rematar",
    "remataseis": "rematar",
    "rematasen": "rematar",
    "rematases": "rematar",
    "remataste": "rematar",
    "rematasteis": "rematar",
    "remate": "rematar",
    "rematemos": "rematar",
    "rematen": "rematar",
    "remates": "remate",
    "remato": "rematar",
    "rematábamos": "rematar",
    "rematáis": "rematar",
    "rematáramos": "rematar",
    "rematáremos": "rematar",
    "rematásemos": "rematar",
    "rematé": "rematar",
    "rematéis": "rematar",
    "remató": "rematar",
    "rembolsa": "rembolsar",
    "rembolsaba": "rembolsar",
    "rembolsabais": "rembolsar",
    "rembolsaban": "rembolsar",
    "rembolsabas": "rembolsar",
    "rembolsad": "rembolsar",
    "rembolsada": "rembolsar",
    "rembolsadas": "rembolsar",
    "rembolsado": "rembolsar",
    "rembolsados": "rembolsar",
    "rembolsamos": "rembolsar",
    "rembolsan": "rembolsar",
    "rembolsando": "rembolsar",
    "rembolsara": "rembolsar",
    "rembolsarais": "rembolsar",
    "rembolsaran": "rembolsar",
    "rembolsaras": "rembolsar",
    "rembolsare": "rembolsar",
    "rembolsareis": "rembolsar",
    "rembolsaremos": "rembolsar",
    "rembolsaren": "rembolsar",
    "rembolsares": "rembolsar",
    "rembolsaron": "rembolsar",
    "rembolsará": "rembolsar",
    "rembolsarán": "rembolsar",
    "rembolsarás": "rembolsar",
    "rembolsaré": "rembolsar",
    "rembolsaréis": "rembolsar",
    "rembolsaría": "rembolsar",
    "rembolsaríais": "rembolsar",
    "rembolsaríamos": "rembolsar",
    "rembolsarían": "rembolsar",
    "rembolsarías": "rembolsar",
    "rembolsas": "rembolsar",
    "rembolsase": "rembolsar",
    "rembolsaseis": "rembolsar",
    "rembolsasen": "rembolsar",
    "rembolsases": "rembolsar",
    "rembolsaste": "rembolsar",
    "rembolsasteis": "rembolsar",
    "rembolse": "rembolsar",
    "rembolsemos": "rembolsar",
    "rembolsen": "rembolsar",
    "rembolses": "rembolsar",
    "rembolso": "rembolsar",
    "rembolsábamos": "rembolsar",
    "rembolsáis": "rembolsar",
    "rembolsáramos": "rembolsar",
    "rembolsáremos": "rembolsar",
    "rembolsásemos": "rembolsar",
    "rembolsé": "rembolsar",
    "rembolséis": "rembolsar",
    "rembolsó": "rembolsar",
    "reme": "remar",
    "remece": "remecer",
    "remeced": "remecer",
    "remecedora": "remecedor",
    "remecedoras": "remecedor",
    "remecedores": "remecedor",
    "remecemos": "remecer",
    "remecen": "remecer",
    "remeceremos": "remecer",
    "remecerá": "remecer",
    "remecerán": "remecer",
    "remecerás": "remecer",
    "remeceré": "remecer",
    "remeceréis": "remecer",
    "remecería": "remecer",
    "remeceríais": "remecer",
    "remeceríamos": "remecer",
    "remecerían": "remecer",
    "remecerías": "remecer",
    "remeces": "remecer",
    "remecida": "remecer",
    "remecidas": "remecer",
    "remecido": "remecer",
    "remecidos": "remecer",
    "remeciendo": "remecer",
    "remeciera": "remecer",
    "remecierais": "remecer",
    "remecieran": "remecer",
    "remecieras": "remecer",
    "remeciere": "remecer",
    "remeciereis": "remecer",
    "remecieren": "remecer",
    "remecieres": "remecer",
    "remecieron": "remecer",
    "remeciese": "remecer",
    "remecieseis": "remecer",
    "remeciesen": "remecer",
    "remecieses": "remecer",
    "remecimos": "remecer",
    "remeciste": "remecer",
    "remecisteis": "remecer",
    "remeciéramos": "remecer",
    "remeciéremos": "remecer",
    "remeciésemos": "remecer",
    "remeció": "remecer",
    "remecéis": "remecer",
    "remecí": "remecer",
    "remecía": "remecer",
    "remecíais": "remecer",
    "remecíamos": "remecer",
    "remecían": "remecer",
    "remecías": "remecer",
    "remeda": "remedar",
    "remedaba": "remedar",
    "remedabais": "remedar",
    "remedaban": "remedar",
    "remedabas": "remedar",
    "remedables": "remedable",
    "remedad": "remedar",
    "remedada": "remedar",
    "remedadas": "remedar",
    "remedado": "remedar",
    "remedadora": "remedador",
    "remedadoras": "remedador",
    "remedadores": "remedador",
    "remedados": "remedar",
    "remedamos": "remedar",
    "remedan": "remedar",
    "remedando": "remedar",
    "remedara": "remedar",
    "remedarais": "remedar",
    "remedaran": "remedar",
    "remedaras": "remedar",
    "remedare": "remedar",
    "remedareis": "remedar",
    "remedaremos": "remedar",
    "remedaren": "remedar",
    "remedares": "remedar",
    "remedaron": "remedar",
    "remedará": "remedar",
    "remedarán": "remedar",
    "remedarás": "remedar",
    "remedaré": "remedar",
    "remedaréis": "remedar",
    "remedaría": "remedar",
    "remedaríais": "remedar",
    "remedaríamos": "remedar",
    "remedarían": "remedar",
    "remedarías": "remedar",
    "remedas": "remedar",
    "remedase": "remedar",
    "remedaseis": "remedar",
    "remedasen": "remedar",
    "remedases": "remedar",
    "remedaste": "remedar",
    "remedasteis": "remedar",
    "remede": "remedar",
    "remedemos": "remedar",
    "remeden": "remedar",
    "remedes": "remedar",
    "remedia": "remediar",
    "remediaba": "remediar",
    "remediabais": "remediar",
    "remediaban": "remediar",
    "remediabas": "remediar",
    "remediables": "remediable",
    "remediad": "remediar",
    "remediada": "remediar",
    "remediadas": "remediar",
    "remediado": "remediar",
    "remediadora": "remediador",
    "remediadoras": "remediador",
    "remediadores": "remediador",
    "remediados": "remediar",
    "remediamos": "remediar",
    "remedian": "remediar",
    "remediando": "remediar",
    "remediara": "remediar",
    "remediarais": "remediar",
    "remediaran": "remediar",
    "remediaras": "remediar",
    "remediare": "remediar",
    "remediareis": "remediar",
    "remediaremos": "remediar",
    "remediaren": "remediar",
    "remediares": "remediar",
    "remediaron": "remediar",
    "remediará": "remediar",
    "remediarán": "remediar",
    "remediarás": "remediar",
    "remediaré": "remediar",
    "remediaréis": "remediar",
    "remediaría": "remediar",
    "remediaríais": "remediar",
    "remediaríamos": "remediar",
    "remediarían": "remediar",
    "remediarías": "remediar",
    "remedias": "remediar",
    "remediase": "remediar",
    "remediaseis": "remediar",
    "remediasen": "remediar",
    "remediases": "remediar",
    "remediaste": "remediar",
    "remediasteis": "remediar",
    "remediciones": "remedición",
    "remedid": "remedir",
    "remedida": "remedir",
    "remedidas": "remedir",
    "remedido": "remedir",
    "remedidos": "remedir",
    "remedie": "remediar",
    "remediemos": "remediar",
    "remedien": "remediar",
    "remedies": "remediar",
    "remedimos": "remedir",
    "remedio": "remediar",
    "remediones": "remedión",
    "remedios": "remedio",
    "remediremos": "remedir",
    "remedirá": "remedir",
    "remedirán": "remedir",
    "remedirás": "remedir",
    "remediré": "remedir",
    "remediréis": "remedir",
    "remediría": "remedir",
    "remediríais": "remedir",
    "remediríamos": "remedir",
    "remedirían": "remedir",
    "remedirías": "remedir",
    "remediste": "remedir",
    "remedisteis": "remedir",
    "remediábamos": "remediar",
    "remediáis": "remediar",
    "remediáramos": "remediar",
    "remediáremos": "remediar",
    "remediásemos": "remediar",
    "remedié": "remediar",
    "remediéis": "remediar",
    "remedió": "remediar",
    "remedo": "remedar",
    "remedos": "remedo",
    "remedábamos": "remedar",
    "remedáis": "remedar",
    "remedáramos": "remedar",
    "remedáremos": "remedar",
    "remedásemos": "remedar",
    "remedé": "remedar",
    "remedéis": "remedar",
    "remedí": "remedir",
    "remedía": "remedir",
    "remedíais": "remedir",
    "remedíamos": "remedir",
    "remedían": "remedir",
    "remedías": "remedir",
    "remedís": "remedir",
    "remedó": "remedar",
    "remella": "remellar",
    "remellaba": "remellar",
    "remellabais": "remellar",
    "remellaban": "remellar",
    "remellabas": "remellar",
    "remellad": "remellar",
    "remellada": "remellar",
    "remelladas": "remellar",
    "remellado": "remellar",
    "remellados": "remellar",
    "remellamos": "remellar",
    "remellan": "remellar",
    "remellando": "remellar",
    "remellara": "remellar",
    "remellarais": "remellar",
    "remellaran": "remellar",
    "remellaras": "remellar",
    "remellare": "remellar",
    "remellareis": "remellar",
    "remellaremos": "remellar",
    "remellaren": "remellar",
    "remellares": "remellar",
    "remellaron": "remellar",
    "remellará": "remellar",
    "remellarán": "remellar",
    "remellarás": "remellar",
    "remellaré": "remellar",
    "remellaréis": "remellar",
    "remellaría": "remellar",
    "remellaríais": "remellar",
    "remellaríamos": "remellar",
    "remellarían": "remellar",
    "remellarías": "remellar",
    "remellas": "remellar",
    "remellase": "remellar",
    "remellaseis": "remellar",
    "remellasen": "remellar",
    "remellases": "remellar",
    "remellaste": "remellar",
    "remellasteis": "remellar",
    "remelle": "remellar",
    "remellemos": "remellar",
    "remellen": "remellar",
    "remelles": "remellar",
    "remello": "remellar",
    "remellona": "remellón",
    "remellonas": "remellón",
    "remellones": "remellón",
    "remellábamos": "remellar",
    "remelláis": "remellar",
    "remelláramos": "remellar",
    "remelláremos": "remellar",
    "remellásemos": "remellar",
    "remellé": "remellar",
    "remelléis": "remellar",
    "remelló": "remellar",
    "remembranzas": "remembranza",
    "rememora": "rememorar",
    "rememoraba": "rememorar",
    "rememorabais": "rememorar",
    "rememoraban": "rememorar",
    "rememorabas": "rememorar",
    "rememoraciones": "rememoración",
    "rememorad": "rememorar",
    "rememorada": "rememorar",
    "rememoradas": "rememorar",
    "rememorado": "rememorar",
    "rememorados": "rememorar",
    "rememoramos": "rememorar",
    "rememoran": "rememorar",
    "rememorando": "rememorar",
    "rememorara": "rememorar",
    "rememorarais": "rememorar",
    "rememoraran": "rememorar",
    "rememoraras": "rememorar",
    "rememorare": "rememorar",
    "rememorareis": "rememorar",
    "rememoraremos": "rememorar",
    "rememoraren": "rememorar",
    "rememorares": "rememorar",
    "rememoraron": "rememorar",
    "rememorará": "rememorar",
    "rememorarán": "rememorar",
    "rememorarás": "rememorar",
    "rememoraré": "rememorar",
    "rememoraréis": "rememorar",
    "rememoraría": "rememorar",
    "rememoraríais": "rememorar",
    "rememoraríamos": "rememorar",
    "rememorarían": "rememorar",
    "rememorarías": "rememorar",
    "rememoras": "rememorar",
    "rememorase": "rememorar",
    "rememoraseis": "rememorar",
    "rememorasen": "rememorar",
    "rememorases": "rememorar",
    "rememoraste": "rememorar",
    "rememorasteis": "rememorar",
    "rememorativa": "rememorativo",
    "rememorativas": "rememorativo",
    "rememorativos": "rememorativo",
    "rememore": "rememorar",
    "rememoremos": "rememorar",
    "rememoren": "rememorar",
    "rememores": "rememorar",
    "rememoro": "rememorar",
    "rememorábamos": "rememorar",
    "rememoráis": "rememorar",
    "rememoráramos": "rememorar",
    "rememoráremos": "rememorar",
    "rememorásemos": "rememorar",
    "rememoré": "rememorar",
    "rememoréis": "rememorar",
    "rememoró": "rememorar",
    "rememos": "remar",
    "remen": "remar",
    "remendaba": "remendar",
    "remendabais": "remendar",
    "remendaban": "remendar",
    "remendabas": "remendar",
    "remendad": "remendar",
    "remendada": "remendar",
    "remendadas": "remendar",
    "remendado": "remendar",
    "remendados": "remendar",
    "remendamos": "remendar",
    "remendando": "remendar",
    "remendara": "remendar",
    "remendarais": "remendar",
    "remendaran": "remendar",
    "remendaras": "remendar",
    "remendare": "remendar",
    "remendareis": "remendar",
    "remendaremos": "remendar",
    "remendaren": "remendar",
    "remendares": "remendar",
    "remendaron": "remendar",
    "remendará": "remendar",
    "remendarán": "remendar",
    "remendarás": "remendar",
    "remendaré": "remendar",
    "remendaréis": "remendar",
    "remendaría": "remendar",
    "remendaríais": "remendar",
    "remendaríamos": "remendar",
    "remendarían": "remendar",
    "remendarías": "remendar",
    "remendase": "remendar",
    "remendaseis": "remendar",
    "remendasen": "remendar",
    "remendases": "remendar",
    "remendaste": "remendar",
    "remendasteis": "remendar",
    "remendemos": "remendar",
    "remendona": "remendón",
    "remendonas": "remendón",
    "remendones": "remendón",
    "remendábamos": "remendar",
    "remendáis": "remendar",
    "remendáramos": "remendar",
    "remendáremos": "remendar",
    "remendásemos": "remendar",
    "remendé": "remendar",
    "remendéis": "remendar",
    "remendó": "remendar",
    "remeneos": "remeneo",
    "remensas": "remensa",
    "remera": "remero",
    "remeras": "remero",
    "remeros": "remero",
    "remes": "remar",
    "remesa": "remesar",
    "remesaba": "remesar",
    "remesabais": "remesar",
    "remesaban": "remesar",
    "remesabas": "remesar",
    "remesad": "remesar",
    "remesada": "remesar",
    "remesadas": "remesar",
    "remesado": "remesar",
    "remesados": "remesar",
    "remesamos": "remesar",
    "remesan": "remesar",
    "remesando": "remesar",
    "remesara": "remesar",
    "remesarais": "remesar",
    "remesaran": "remesar",
    "remesaras": "remesar",
    "remesare": "remesar",
    "remesareis": "remesar",
    "remesaremos": "remesar",
    "remesaren": "remesar",
    "remesares": "remesar",
    "remesaron": "remesar",
    "remesará": "remesar",
    "remesarán": "remesar",
    "remesarás": "remesar",
    "remesaré": "remesar",
    "remesaréis": "remesar",
    "remesaría": "remesar",
    "remesaríais": "remesar",
    "remesaríamos": "remesar",
    "remesarían": "remesar",
    "remesarías": "remesar",
    "remesas": "remesar",
    "remesase": "remesar",
    "remesaseis": "remesar",
    "remesasen": "remesar",
    "remesases": "remesar",
    "remesaste": "remesar",
    "remesasteis": "remesar",
    "remese": "remesar",
    "remesemos": "remesar",
    "remesen": "remesar",
    "remeses": "remesar",
    "remeso": "remesar",
    "remesones": "remesón",
    "remesábamos": "remesar",
    "remesáis": "remesar",
    "remesáramos": "remesar",
    "remesáremos": "remesar",
    "remesásemos": "remesar",
    "remesé": "remesar",
    "remeséis": "remesar",
    "remesó": "remesar",
    "remeta": "remeter",
    "remetamos": "remeter",
    "remetan": "remeter",
    "remetas": "remeter",
    "remete": "remeter",
    "remeted": "remeter",
    "remetemos": "remeter",
    "remeten": "remeter",
    "remeteremos": "remeter",
    "remeterá": "remeter",
    "remeterán": "remeter",
    "remeterás": "remeter",
    "remeteré": "remeter",
    "remeteréis": "remeter",
    "remetería": "remeter",
    "remeteríais": "remeter",
    "remeteríamos": "remeter",
    "remeterían": "remeter",
    "remeterías": "remeter",
    "remetes": "remeter",
    "remetida": "remeter",
    "remetidas": "remeter",
    "remetido": "remeter",
    "remetidos": "remeter",
    "remetiendo": "remeter",
    "remetiera": "remeter",
    "remetierais": "remeter",
    "remetieran": "remeter",
    "remetieras": "remeter",
    "remetiere": "remeter",
    "remetiereis": "remeter",
    "remetieren": "remeter",
    "remetieres": "remeter",
    "remetieron": "remeter",
    "remetiese": "remeter",
    "remetieseis": "remeter",
    "remetiesen": "remeter",
    "remetieses": "remeter",
    "remetimos": "remeter",
    "remetiste": "remeter",
    "remetisteis": "remeter",
    "remetiéramos": "remeter",
    "remetiéremos": "remeter",
    "remetiésemos": "remeter",
    "remetió": "remeter",
    "remeto": "remeter",
    "remetáis": "remeter",
    "remetéis": "remeter",
    "remetí": "remeter",
    "remetía": "remeter",
    "remetíais": "remeter",
    "remetíamos": "remeter",
    "remetían": "remeter",
    "remetías": "remeter",
    "remeza": "remecer",
    "remezamos": "remecer",
    "remezan": "remecer",
    "remezas": "remecer",
    "remezo": "remecer",
    "remezones": "remezón",
    "remezáis": "remecer",
    "remida": "remedir",
    "remidamos": "remedir",
    "remidan": "remedir",
    "remidas": "remedir",
    "remide": "remedir",
    "remiden": "remedir",
    "remides": "remedir",
    "remidiendo": "remedir",
    "remidiera": "remedir",
    "remidierais": "remedir",
    "remidieran": "remedir",
    "remidieras": "remedir",
    "remidiere": "remedir",
    "remidiereis": "remedir",
    "remidieren": "remedir",
    "remidieres": "remedir",
    "remidieron": "remedir",
    "remidiese": "remedir",
    "remidieseis": "remedir",
    "remidiesen": "remedir",
    "remidieses": "remedir",
    "remidiéramos": "remedir",
    "remidiéremos": "remedir",
    "remidiésemos": "remedir",
    "remidió": "remedir",
    "remido": "remedir",
    "remidáis": "remedir",
    "remieles": "remiel",
    "remienda": "remendar",
    "remiendan": "remendar",
    "remiendas": "remendar",
    "remiende": "remendar",
    "remienden": "remendar",
    "remiendes": "remendar",
    "remiendo": "remendar",
    "remiendos": "remiendo",
    "remilga": "remilgar",
    "remilgaba": "remilgar",
    "remilgabais": "remilgar",
    "remilgaban": "remilgar",
    "remilgabas": "remilgar",
    "remilgad": "remilgar",
    "remilgada": "remilgar",
    "remilgadas": "remilgar",
    "remilgado": "remilgar",
    "remilgados": "remilgar",
    "remilgamos": "remilgar",
    "remilgan": "remilgar",
    "remilgando": "remilgar",
    "remilgara": "remilgar",
    "remilgarais": "remilgar",
    "remilgaran": "remilgar",
    "remilgaras": "remilgar",
    "remilgare": "remilgar",
    "remilgareis": "remilgar",
    "remilgaremos": "remilgar",
    "remilgaren": "remilgar",
    "remilgares": "remilgar",
    "remilgaron": "remilgar",
    "remilgará": "remilgar",
    "remilgarán": "remilgar",
    "remilgarás": "remilgar",
    "remilgaré": "remilgar",
    "remilgaréis": "remilgar",
    "remilgaría": "remilgar",
    "remilgaríais": "remilgar",
    "remilgaríamos": "remilgar",
    "remilgarían": "remilgar",
    "remilgarías": "remilgar",
    "remilgas": "remilgar",
    "remilgase": "remilgar",
    "remilgaseis": "remilgar",
    "remilgasen": "remilgar",
    "remilgases": "remilgar",
    "remilgaste": "remilgar",
    "remilgasteis": "remilgar",
    "remilgo": "remilgar",
    "remilgos": "remilgo",
    "remilgue": "remilgar",
    "remilguemos": "remilgar",
    "remilguen": "remilgar",
    "remilgues": "remilgar",
    "remilgué": "remilgar",
    "remilguéis": "remilgar",
    "remilgábamos": "remilgar",
    "remilgáis": "remilgar",
    "remilgáramos": "remilgar",
    "remilgáremos": "remilgar",
    "remilgásemos": "remilgar",
    "remilgó": "remilgar",
    "remineralizaciones": "remineralización",
    "remingtones": "rémington",
    "reminiscencias": "reminiscencia",
    "remira": "remirar",
    "remiraba": "remirar",
    "remirabais": "remirar",
    "remiraban": "remirar",
    "remirabas": "remirar",
    "remirad": "remirar",
    "remirada": "remirar",
    "remiradas": "remirar",
    "remirado": "remirar",
    "remirados": "remirar",
    "remiramos": "remirar",
    "remiran": "remirar",
    "remirando": "remirar",
    "remirara": "remirar",
    "remirarais": "remirar",
    "remiraran": "remirar",
    "remiraras": "remirar",
    "remirare": "remirar",
    "remirareis": "remirar",
    "remiraremos": "remirar",
    "remiraren": "remirar",
    "remirares": "remirar",
    "remiraron": "remirar",
    "remirará": "remirar",
    "remirarán": "remirar",
    "remirarás": "remirar",
    "remiraré": "remirar",
    "remiraréis": "remirar",
    "remiraría": "remirar",
    "remiraríais": "remirar",
    "remiraríamos": "remirar",
    "remirarían": "remirar",
    "remirarías": "remirar",
    "remiras": "remirar",
    "remirase": "remirar",
    "remiraseis": "remirar",
    "remirasen": "remirar",
    "remirases": "remirar",
    "remiraste": "remirar",
    "remirasteis": "remirar",
    "remire": "remirar",
    "remiremos": "remirar",
    "remiren": "remirar",
    "remires": "remirar",
    "remiro": "remirar",
    "remirábamos": "remirar",
    "remiráis": "remirar",
    "remiráramos": "remirar",
    "remiráremos": "remirar",
    "remirásemos": "remirar",
    "remiré": "remirar",
    "remiréis": "remirar",
    "remiró": "remirar",
    "remisa": "remiso",
    "remisas": "remiso",
    "remisibles": "remisible",
    "remisiones": "remisión",
    "remisiva": "remisivo",
    "remisivas": "remisivo",
    "remisivos": "remisivo",
    "remisora": "remisor",
    "remisoras": "remisor",
    "remisores": "remisor",
    "remisoria": "remisorio",
    "remisorias": "remisorio",
    "remisorios": "remisorio",
    "remisos": "remiso",
    "remita": "remitir",
    "remitamos": "remitir",
    "remitan": "remitir",
    "remitancias": "remitancia",
    "remitas": "remitir",
    "remite": "remitir",
    "remiten": "remitir",
    "remitentes": "remitente",
    "remites": "remitir",
    "remitid": "remitir",
    "remitida": "remitir",
    "remitidas": "remitir",
    "remitido": "remitir",
    "remitidos": "remitir",
    "remitiendo": "remitir",
    "remitiera": "remitir",
    "remitierais": "remitir",
    "remitieran": "remitir",
    "remitieras": "remitir",
    "remitiere": "remitir",
    "remitiereis": "remitir",
    "remitieren": "remitir",
    "remitieres": "remitir",
    "remitieron": "remitir",
    "remitiese": "remitir",
    "remitieseis": "remitir",
    "remitiesen": "remitir",
    "remitieses": "remitir",
    "remitimos": "remitir",
    "remitiremos": "remitir",
    "remitirá": "remitir",
    "remitirán": "remitir",
    "remitirás": "remitir",
    "remitiré": "remitir",
    "remitiréis": "remitir",
    "remitiría": "remitir",
    "remitiríais": "remitir",
    "remitiríamos": "remitir",
    "remitirían": "remitir",
    "remitirías": "remitir",
    "remitiste": "remitir",
    "remitisteis": "remitir",
    "remitiéramos": "remitir",
    "remitiéremos": "remitir",
    "remitiésemos": "remitir",
    "remitió": "remitir",
    "remito": "remitir",
    "remitáis": "remitir",
    "remití": "remitir",
    "remitía": "remitir",
    "remitíais": "remitir",
    "remitíamos": "remitir",
    "remitían": "remitir",
    "remitías": "remitir",
    "remitís": "remitir",
    "remo": "remar",
    "remoce": "remozar",
    "remocemos": "remozar",
    "remocen": "remozar",
    "remoces": "remozar",
    "remociones": "remoción",
    "remocé": "remozar",
    "remocéis": "remozar",
    "remodela": "remodelar",
    "remodelaba": "remodelar",
    "remodelabais": "remodelar",
    "remodelaban": "remodelar",
    "remodelabas": "remodelar",
    "remodelaciones": "remodelación",
    "remodelad": "remodelar",
    "remodelada": "remodelar",
    "remodeladas": "remodelar",
    "remodelado": "remodelar",
    "remodelados": "remodelar",
    "remodelamos": "remodelar",
    "remodelan": "remodelar",
    "remodelando": "remodelar",
    "remodelara": "remodelar",
    "remodelarais": "remodelar",
    "remodelaran": "remodelar",
    "remodelaras": "remodelar",
    "remodelare": "remodelar",
    "remodelareis": "remodelar",
    "remodelaremos": "remodelar",
    "remodelaren": "remodelar",
    "remodelares": "remodelar",
    "remodelaron": "remodelar",
    "remodelará": "remodelar",
    "remodelarán": "remodelar",
    "remodelarás": "remodelar",
    "remodelaré": "remodelar",
    "remodelaréis": "remodelar",
    "remodelaría": "remodelar",
    "remodelaríais": "remodelar",
    "remodelaríamos": "remodelar",
    "remodelarían": "remodelar",
    "remodelarías": "remodelar",
    "remodelas": "remodelar",
    "remodelase": "remodelar",
    "remodelaseis": "remodelar",
    "remodelasen": "remodelar",
    "remodelases": "remodelar",
    "remodelaste": "remodelar",
    "remodelasteis": "remodelar",
    "remodele": "remodelar",
    "remodelemos": "remodelar",
    "remodelen": "remodelar",
    "remodeles": "remodelar",
    "remodelo": "remodelar",
    "remodelábamos": "remodelar",
    "remodeláis": "remodelar",
    "remodeláramos": "remodelar",
    "remodeláremos": "remodelar",
    "remodelásemos": "remodelar",
    "remodelé": "remodelar",
    "remodeléis": "remodelar",
    "remodeló": "remodelar",
    "remoduladora": "remodulador",
    "remoduladoras": "remodulador",
    "remoduladores": "remodulador",
    "remoja": "remojar",
    "remojaba": "remojar",
    "remojabais": "remojar",
    "remojaban": "remojar",
    "remojabas": "remojar",
    "remojad": "remojar",
    "remojada": "remojar",
    "remojadas": "remojar",
    "remojado": "remojar",
    "remojados": "remojar",
    "remojamos": "remojar",
    "remojan": "remojar",
    "remojando": "remojar",
    "remojara": "remojar",
    "remojarais": "remojar",
    "remojaran": "remojar",
    "remojaras": "remojar",
    "remojare": "remojar",
    "remojareis": "remojar",
    "remojaremos": "remojar",
    "remojaren": "remojar",
    "remojares": "remojar",
    "remojaron": "remojar",
    "remojará": "remojar",
    "remojarán": "remojar",
    "remojarás": "remojar",
    "remojaré": "remojar",
    "remojaréis": "remojar",
    "remojaría": "remojar",
    "remojaríais": "remojar",
    "remojaríamos": "remojar",
    "remojarían": "remojar",
    "remojarías": "remojar",
    "remojas": "remojar",
    "remojase": "remojar",
    "remojaseis": "remojar",
    "remojasen": "remojar",
    "remojases": "remojar",
    "remojaste": "remojar",
    "remojasteis": "remojar",
    "remoje": "remojar",
    "remojemos": "remojar",
    "remojen": "remojar",
    "remojes": "remojar",
    "remojo": "remojar",
    "remojones": "remojón",
    "remojos": "remojo",
    "remojábamos": "remojar",
    "remojáis": "remojar",
    "remojáramos": "remojar",
    "remojáremos": "remojar",
    "remojásemos": "remojar",
    "remojé": "remojar",
    "remojéis": "remojar",
    "remojó": "remojar",
    "remolachas": "remolacha",
    "remolachera": "remolachero",
    "remolacheras": "remolachero",
    "remolacheros": "remolachero",
    "remolamos": "remoler",
    "remolares": "remolar",
    "remolca": "remolcar",
    "remolcaba": "remolcar",
    "remolcabais": "remolcar",
    "remolcaban": "remolcar",
    "remolcabas": "remolcar",
    "remolcables": "remolcable",
    "remolcad": "remolcar",
    "remolcada": "remolcar",
    "remolcadas": "remolcar",
    "remolcado": "remolcar",
    "remolcadora": "remolcador",
    "remolcadoras": "remolcador",
    "remolcadores": "remolcador",
    "remolcados": "remolcar",
    "remolcamos": "remolcar",
    "remolcan": "remolcar",
    "remolcando": "remolcar",
    "remolcara": "remolcar",
    "remolcarais": "remolcar",
    "remolcaran": "remolcar",
    "remolcaras": "remolcar",
    "remolcare": "remolcar",
    "remolcareis": "remolcar",
    "remolcaremos": "remolcar",
    "remolcaren": "remolcar",
    "remolcares": "remolcar",
    "remolcaron": "remolcar",
    "remolcará": "remolcar",
    "remolcarán": "remolcar",
    "remolcarás": "remolcar",
    "remolcaré": "remolcar",
    "remolcaréis": "remolcar",
    "remolcaría": "remolcar",
    "remolcaríais": "remolcar",
    "remolcaríamos": "remolcar",
    "remolcarían": "remolcar",
    "remolcarías": "remolcar",
    "remolcas": "remolcar",
    "remolcase": "remolcar",
    "remolcaseis": "remolcar",
    "remolcasen": "remolcar",
    "remolcases": "remolcar",
    "remolcaste": "remolcar",
    "remolcasteis": "remolcar",
    "remolco": "remolcar",
    "remolcábamos": "remolcar",
    "remolcáis": "remolcar",
    "remolcáramos": "remolcar",
    "remolcáremos": "remolcar",
    "remolcásemos": "remolcar",
    "remolcó": "remolcar",
    "remoldeados": "remoldeado",
    "remoled": "remoler",
    "remolemos": "remoler",
    "remoleremos": "remoler",
    "remolerá": "remoler",
    "remolerán": "remoler",
    "remolerás": "remoler",
    "remoleré": "remoler",
    "remoleréis": "remoler",
    "remolería": "remoler",
    "remoleríais": "remoler",
    "remoleríamos": "remoler",
    "remolerían": "remoler",
    "remolerías": "remoler",
    "remolida": "remoler",
    "remolidas": "remoler",
    "remolido": "remoler",
    "remolidos": "remolido",
    "remoliendo": "remoler",
    "remoliera": "remoler",
    "remolierais": "remoler",
    "remolieran": "remoler",
    "remolieras": "remoler",
    "remoliere": "remoler",
    "remoliereis": "remoler",
    "remolieren": "remoler",
    "remolieres": "remoler",
    "remolieron": "remoler",
    "remoliese": "remoler",
    "remolieseis": "remoler",
    "remoliesen": "remoler",
    "remolieses": "remoler",
    "remolimientos": "remolimiento",
    "remolimos": "remoler",
    "remolina": "remolinar",
    "remolinaba": "remolinar",
    "remolinabais": "remolinar",
    "remolinaban": "remolinar",
    "remolinabas": "remolinar",
    "remolinad": "remolinar",
    "remolinada": "remolinar",
    "remolinadas": "remolinar",
    "remolinado": "remolinar",
    "remolinados": "remolinar",
    "remolinamos": "remolinar",
    "remolinan": "remolinar",
    "remolinando": "remolinar",
    "remolinara": "remolinar",
    "remolinarais": "remolinar",
    "remolinaran": "remolinar",
    "remolinaras": "remolinar",
    "remolinare": "remolinar",
    "remolinareis": "remolinar",
    "remolinaremos": "remolinar",
    "remolinaren": "remolinar",
    "remolinares": "remolinar",
    "remolinaron": "remolinar",
    "remolinará": "remolinar",
    "remolinarán": "remolinar",
    "remolinarás": "remolinar",
    "remolinaré": "remolinar",
    "remolinaréis": "remolinar",
    "remolinaría": "remolinar",
    "remolinaríais": "remolinar",
    "remolinaríamos": "remolinar",
    "remolinarían": "remolinar",
    "remolinarías": "remolinar",
    "remolinas": "remolinar",
    "remolinase": "remolinar",
    "remolinaseis": "remolinar",
    "remolinasen": "remolinar",
    "remolinases": "remolinar",
    "remolinaste": "remolinar",
    "remolinasteis": "remolinar",
    "remoline": "remolinar",
    "remolinea": "remolinear",
    "remolineaba": "remolinear",
    "remolineabais": "remolinear",
    "remolineaban": "remolinear",
    "remolineabas": "remolinear",
    "remolinead": "remolinear",
    "remolineada": "remolinear",
    "remolineadas": "remolinear",
    "remolineado": "remolinear",
    "remolineados": "remolinear",
    "remolineamos": "remolinear",
    "remolinean": "remolinear",
    "remolineando": "remolinear",
    "remolineara": "remolinear",
    "remolinearais": "remolinear",
    "remolinearan": "remolinear",
    "remolinearas": "remolinear",
    "remolineare": "remolinear",
    "remolineareis": "remolinear",
    "remolinearemos": "remolinear",
    "remolinearen": "remolinear",
    "remolineares": "remolinear",
    "remolinearon": "remolinear",
    "remolineará": "remolinear",
    "remolinearán": "remolinear",
    "remolinearás": "remolinear",
    "remolinearé": "remolinear",
    "remolinearéis": "remolinear",
    "remolinearía": "remolinear",
    "remolinearíais": "remolinear",
    "remolinearíamos": "remolinear",
    "remolinearían": "remolinear",
    "remolinearías": "remolinear",
    "remolineas": "remolinear",
    "remolinease": "remolinear",
    "remolineaseis": "remolinear",
    "remolineasen": "remolinear",
    "remolineases": "remolinear",
    "remolineaste": "remolinear",
    "remolineasteis": "remolinear",
    "remolinee": "remolinear",
    "remolineemos": "remolinear",
    "remolineen": "remolinear",
    "remolinees": "remolinear",
    "remolinemos": "remolinar",
    "remolinen": "remolinar",
    "remolineo": "remolinear",
    "remolines": "remolinar",
    "remolineábamos": "remolinear",
    "remolineáis": "remolinear",
    "remolineáramos": "remolinear",
    "remolineáremos": "remolinear",
    "remolineásemos": "remolinear",
    "remolineé": "remolinear",
    "remolineéis": "remolinear",
    "remolineó": "remolinear",
    "remolino": "remolinar",
    "remolinos": "remolino",
    "remolinábamos": "remolinar",
    "remolináis": "remolinar",
    "remolináramos": "remolinar",
    "remolináremos": "remolinar",
    "remolinásemos": "remolinar",
    "remoliné": "remolinar",
    "remolinéis": "remolinar",
    "remolinó": "remolinar",
    "remoliste": "remoler",
    "remolisteis": "remoler",
    "remoliéramos": "remoler",
    "remoliéremos": "remoler",
    "remoliésemos": "remoler",
    "remolió": "remoler",
    "remolona": "remolón",
    "remolonas": "remolón",
    "remolonea": "remolonear",
    "remoloneaba": "remolonear",
    "remoloneabais": "remolonear",
    "remoloneaban": "remolonear",
    "remoloneabas": "remolonear",
    "remolonead": "remolonear",
    "remoloneada": "remolonear",
    "remoloneadas": "remolonear",
    "remoloneado": "remolonear",
    "remoloneados": "remolonear",
    "remoloneamos": "remolonear",
    "remolonean": "remolonear",
    "remoloneando": "remolonear",
    "remoloneara": "remolonear",
    "remolonearais": "remolonear",
    "remolonearan": "remolonear",
    "remolonearas": "remolonear",
    "remoloneare": "remolonear",
    "remoloneareis": "remolonear",
    "remolonearemos": "remolonear",
    "remolonearen": "remolonear",
    "remoloneares": "remolonear",
    "remolonearon": "remolonear",
    "remoloneará": "remolonear",
    "remolonearán": "remolonear",
    "remolonearás": "remolonear",
    "remolonearé": "remolonear",
    "remolonearéis": "remolonear",
    "remolonearía": "remolonear",
    "remolonearíais": "remolonear",
    "remolonearíamos": "remolonear",
    "remolonearían": "remolonear",
    "remolonearías": "remolonear",
    "remoloneas": "remolonear",
    "remolonease": "remolonear",
    "remoloneaseis": "remolonear",
    "remoloneasen": "remolonear",
    "remoloneases": "remolonear",
    "remoloneaste": "remolonear",
    "remoloneasteis": "remolonear",
    "remolonee": "remolonear",
    "remoloneemos": "remolonear",
    "remoloneen": "remolonear",
    "remolonees": "remolonear",
    "remoloneo": "remolonear",
    "remolones": "remolón",
    "remoloneábamos": "remolonear",
    "remoloneáis": "remolonear",
    "remoloneáramos": "remolonear",
    "remoloneáremos": "remolonear",
    "remoloneásemos": "remolonear",
    "remoloneé": "remolonear",
    "remoloneéis": "remolonear",
    "remoloneó": "remolonear",
    "remolque": "remolcar",
    "remolquemos": "remolcar",
    "remolquen": "remolcar",
    "remolques": "remolque",
    "remolqué": "remolcar",
    "remolquéis": "remolcar",
    "remolturaciones": "remolturación",
    "remoláis": "remoler",
    "remoléis": "remoler",
    "remolí": "remoler",
    "remolía": "remoler",
    "remolíais": "remoler",
    "remolíamos": "remoler",
    "remolían": "remoler",
    "remolías": "remoler",
    "remonetizaciones": "remonetización",
    "remonta": "remontar",
    "remontaba": "remontar",
    "remontabais": "remontar",
    "remontaban": "remontar",
    "remontabas": "remontar",
    "remontad": "remontar",
    "remontada": "remontar",
    "remontadas": "remontar",
    "remontado": "remontar",
    "remontadores": "remontador",
    "remontados": "remontar",
    "remontajes": "remontaje",
    "remontamientos": "remontamiento",
    "remontamos": "remontar",
    "remontan": "remontar",
    "remontando": "remontar",
    "remontantes": "remontante",
    "remontara": "remontar",
    "remontarais": "remontar",
    "remontaran": "remontar",
    "remontaras": "remontar",
    "remontare": "remontar",
    "remontareis": "remontar",
    "remontaremos": "remontar",
    "remontaren": "remontar",
    "remontares": "remontar",
    "remontaron": "remontar",
    "remontará": "remontar",
    "remontarán": "remontar",
    "remontarás": "remontar",
    "remontaré": "remontar",
    "remontaréis": "remontar",
    "remontaría": "remontar",
    "remontaríais": "remontar",
    "remontaríamos": "remontar",
    "remontarían": "remontar",
    "remontarías": "remontar",
    "remontas": "remontar",
    "remontase": "remontar",
    "remontaseis": "remontar",
    "remontasen": "remontar",
    "remontases": "remontar",
    "remontaste": "remontar",
    "remontasteis": "remontar",
    "remonte": "remontar",
    "remontemos": "remontar",
    "remonten": "remontar",
    "remontes": "remonte",
    "remontistas": "remontista",
    "remonto": "remontar",
    "remontábamos": "remontar",
    "remontáis": "remontar",
    "remontáramos": "remontar",
    "remontáremos": "remontar",
    "remontásemos": "remontar",
    "remonté": "remontar",
    "remontéis": "remontar",
    "remontó": "remontar",
    "remoques": "remoque",
    "remoquetes": "remoquete",
    "remordamos": "remorder",
    "remorded": "remorder",
    "remordemos": "remorder",
    "remorderemos": "remorder",
    "remorderá": "remorder",
    "remorderán": "remorder",
    "remorderás": "remorder",
    "remorderé": "remorder",
    "remorderéis": "remorder",
    "remordería": "remorder",
    "remorderíais": "remorder",
    "remorderíamos": "remorder",
    "remorderían": "remorder",
    "remorderías": "remorder",
    "remordida": "remorder",
    "remordidas": "remorder",
    "remordido": "remorder",
    "remordidos": "remorder",
    "remordiendo": "remorder",
    "remordiera": "remorder",
    "remordierais": "remorder",
    "remordieran": "remorder",
    "remordieras": "remorder",
    "remordiere": "remorder",
    "remordiereis": "remorder",
    "remordieren": "remorder",
    "remordieres": "remorder",
    "remordieron": "remorder",
    "remordiese": "remorder",
    "remordieseis": "remorder",
    "remordiesen": "remorder",
    "remordieses": "remorder",
    "remordimientos": "remordimiento",
    "remordimos": "remorder",
    "remordiste": "remorder",
    "remordisteis": "remorder",
    "remordiéramos": "remorder",
    "remordiéremos": "remorder",
    "remordiésemos": "remorder",
    "remordió": "remorder",
    "remordáis": "remorder",
    "remordéis": "remorder",
    "remordí": "remorder",
    "remordía": "remorder",
    "remordíais": "remorder",
    "remordíamos": "remorder",
    "remordían": "remorder",
    "remordías": "remorder",
    "remos": "remo",
    "remosquea": "remosquear",
    "remosqueaba": "remosquear",
    "remosqueabais": "remosquear",
    "remosqueaban": "remosquear",
    "remosqueabas": "remosquear",
    "remosquead": "remosquear",
    "remosqueada": "remosquear",
    "remosqueadas": "remosquear",
    "remosqueado": "remosquear",
    "remosqueados": "remosquear",
    "remosqueamos": "remosquear",
    "remosquean": "remosquear",
    "remosqueando": "remosquear",
    "remosqueara": "remosquear",
    "remosquearais": "remosquear",
    "remosquearan": "remosquear",
    "remosquearas": "remosquear",
    "remosqueare": "remosquear",
    "remosqueareis": "remosquear",
    "remosquearemos": "remosquear",
    "remosquearen": "remosquear",
    "remosqueares": "remosquear",
    "remosquearon": "remosquear",
    "remosqueará": "remosquear",
    "remosquearán": "remosquear",
    "remosquearás": "remosquear",
    "remosquearé": "remosquear",
    "remosquearéis": "remosquear",
    "remosquearía": "remosquear",
    "remosquearíais": "remosquear",
    "remosquearíamos": "remosquear",
    "remosquearían": "remosquear",
    "remosquearías": "remosquear",
    "remosqueas": "remosquear",
    "remosquease": "remosquear",
    "remosqueaseis": "remosquear",
    "remosqueasen": "remosquear",
    "remosqueases": "remosquear",
    "remosqueaste": "remosquear",
    "remosqueasteis": "remosquear",
    "remosquee": "remosquear",
    "remosqueemos": "remosquear",
    "remosqueen": "remosquear",
    "remosquees": "remosquear",
    "remosqueo": "remosquear",
    "remosqueos": "remosqueo",
    "remosqueábamos": "remosquear",
    "remosqueáis": "remosquear",
    "remosqueáramos": "remosquear",
    "remosqueáremos": "remosquear",
    "remosqueásemos": "remosquear",
    "remosqueé": "remosquear",
    "remosqueéis": "remosquear",
    "remosqueó": "remosquear",
    "remosta": "remostar",
    "remostaba": "remostar",
    "remostabais": "remostar",
    "remostaban": "remostar",
    "remostabas": "remostar",
    "remostad": "remostar",
    "remostada": "remostar",
    "remostadas": "remostar",
    "remostado": "remostar",
    "remostados": "remostar",
    "remostamos": "remostar",
    "remostan": "remostar",
    "remostando": "remostar",
    "remostara": "remostar",
    "remostarais": "remostar",
    "remostaran": "remostar",
    "remostaras": "remostar",
    "remostare": "remostar",
    "remostareis": "remostar",
    "remostaremos": "remostar",
    "remostaren": "remostar",
    "remostares": "remostar",
    "remostaron": "remostar",
    "remostará": "remostar",
    "remostarán": "remostar",
    "remostarás": "remostar",
    "remostaré": "remostar",
    "remostaréis": "remostar",
    "remostaría": "remostar",
    "remostaríais": "remostar",
    "remostaríamos": "remostar",
    "remostarían": "remostar",
    "remostarías": "remostar",
    "remostas": "remostar",
    "remostase": "remostar",
    "remostaseis": "remostar",
    "remostasen": "remostar",
    "remostases": "remostar",
    "remostaste": "remostar",
    "remostasteis": "remostar",
    "remoste": "remostar",
    "remostemos": "remostar",
    "remosten": "remostar",
    "remostes": "remostar",
    "remosto": "remostar",
    "remostábamos": "remostar",
    "remostáis": "remostar",
    "remostáramos": "remostar",
    "remostáremos": "remostar",
    "remostásemos": "remostar",
    "remosté": "remostar",
    "remostéis": "remostar",
    "remostó": "remostar",
    "remota": "remoto",
    "remotas": "remoto",
    "remotos": "remoto",
    "removamos": "remover",
    "removed": "remover",
    "removemos": "remover",
    "removeremos": "remover",
    "removerá": "remover",
    "removerán": "remover",
    "removerás": "remover",
    "removeré": "remover",
    "removeréis": "remover",
    "removería": "remover",
    "removeríais": "remover",
    "removeríamos": "remover",
    "removerían": "remover",
    "removerías": "remover",
    "removibles": "removible",
    "removida": "remover",
    "removidas": "remover",
    "removido": "remover",
    "removidos": "remover",
    "removiendo": "remover",
    "removiera": "remover",
    "removierais": "remover",
    "removieran": "remover",
    "removieras": "remover",
    "removiere": "remover",
    "removiereis": "remover",
    "removieren": "remover",
    "removieres": "remover",
    "removieron": "remover",
    "removiese": "remover",
    "removieseis": "remover",
    "removiesen": "remover",
    "removieses": "remover",
    "removimientos": "removimiento",
    "removimos": "remover",
    "removiste": "remover",
    "removisteis": "remover",
    "removiéramos": "remover",
    "removiéremos": "remover",
    "removiésemos": "remover",
    "removió": "remover",
    "remováis": "remover",
    "removéis": "remover",
    "removí": "remover",
    "removía": "remover",
    "removíais": "remover",
    "removíamos": "remover",
    "removían": "remover",
    "removías": "remover",
    "remoza": "remozar",
    "remozaba": "remozar",
    "remozabais": "remozar",
    "remozaban": "remozar",
    "remozabas": "remozar",
    "remozad": "remozar",
    "remozada": "remozar",
    "remozadas": "remozar",
    "remozado": "remozar",
    "remozados": "remozar",
    "remozamientos": "remozamiento",
    "remozamos": "remozar",
    "remozan": "remozar",
    "remozando": "remozar",
    "remozara": "remozar",
    "remozarais": "remozar",
    "remozaran": "remozar",
    "remozaras": "remozar",
    "remozare": "remozar",
    "remozareis": "remozar",
    "remozaremos": "remozar",
    "remozaren": "remozar",
    "remozares": "remozar",
    "remozaron": "remozar",
    "remozará": "remozar",
    "remozarán": "remozar",
    "remozarás": "remozar",
    "remozaré": "remozar",
    "remozaréis": "remozar",
    "remozaría": "remozar",
    "remozaríais": "remozar",
    "remozaríamos": "remozar",
    "remozarían": "remozar",
    "remozarías": "remozar",
    "remozas": "remozar",
    "remozase": "remozar",
    "remozaseis": "remozar",
    "remozasen": "remozar",
    "remozases": "remozar",
    "remozaste": "remozar",
    "remozasteis": "remozar",
    "remozo": "remozar",
    "remozábamos": "remozar",
    "remozáis": "remozar",
    "remozáramos": "remozar",
    "remozáremos": "remozar",
    "remozásemos": "remozar",
    "remozó": "remozar",
    "remplace": "remplazar",
    "remplacemos": "remplazar",
    "remplacen": "remplazar",
    "remplaces": "remplazar",
    "remplacé": "remplazar",
    "remplacéis": "remplazar",
    "remplaza": "remplazar",
    "remplazaba": "remplazar",
    "remplazabais": "remplazar",
    "remplazaban": "remplazar",
    "remplazabas": "remplazar",
    "remplazad": "remplazar",
    "remplazada": "remplazar",
    "remplazadas": "remplazar",
    "remplazado": "remplazar",
    "remplazados": "remplazar",
    "remplazamos": "remplazar",
    "remplazan": "remplazar",
    "remplazando": "remplazar",
    "remplazara": "remplazar",
    "remplazarais": "remplazar",
    "remplazaran": "remplazar",
    "remplazaras": "remplazar",
    "remplazare": "remplazar",
    "remplazareis": "remplazar",
    "remplazaremos": "remplazar",
    "remplazaren": "remplazar",
    "remplazares": "remplazar",
    "remplazaron": "remplazar",
    "remplazará": "remplazar",
    "remplazarán": "remplazar",
    "remplazarás": "remplazar",
    "remplazaré": "remplazar",
    "remplazaréis": "remplazar",
    "remplazaría": "remplazar",
    "remplazaríais": "remplazar",
    "remplazaríamos": "remplazar",
    "remplazarían": "remplazar",
    "remplazarías": "remplazar",
    "remplazas": "remplazar",
    "remplazase": "remplazar",
    "remplazaseis": "remplazar",
    "remplazasen": "remplazar",
    "remplazases": "remplazar",
    "remplazaste": "remplazar",
    "remplazasteis": "remplazar",
    "remplazo": "remplazar",
    "remplazos": "remplazo",
    "remplazábamos": "remplazar",
    "remplazáis": "remplazar",
    "remplazáramos": "remplazar",
    "remplazáremos": "remplazar",
    "remplazásemos": "remplazar",
    "remplazó": "remplazar",
    "rempuja": "rempujar",
    "rempujaba": "rempujar",
    "rempujabais": "rempujar",
    "rempujaban": "rempujar",
    "rempujabas": "rempujar",
    "rempujad": "rempujar",
    "rempujada": "rempujar",
    "rempujadas": "rempujar",
    "rempujado": "rempujar",
    "rempujados": "rempujar",
    "rempujamos": "rempujar",
    "rempujan": "rempujar",
    "rempujando": "rempujar",
    "rempujara": "rempujar",
    "rempujarais": "rempujar",
    "rempujaran": "rempujar",
    "rempujaras": "rempujar",
    "rempujare": "rempujar",
    "rempujareis": "rempujar",
    "rempujaremos": "rempujar",
    "rempujaren": "rempujar",
    "rempujares": "rempujar",
    "rempujaron": "rempujar",
    "rempujará": "rempujar",
    "rempujarán": "rempujar",
    "rempujarás": "rempujar",
    "rempujaré": "rempujar",
    "rempujaréis": "rempujar",
    "rempujaría": "rempujar",
    "rempujaríais": "rempujar",
    "rempujaríamos": "rempujar",
    "rempujarían": "rempujar",
    "rempujarías": "rempujar",
    "rempujas": "rempujar",
    "rempujase": "rempujar",
    "rempujaseis": "rempujar",
    "rempujasen": "rempujar",
    "rempujases": "rempujar",
    "rempujaste": "rempujar",
    "rempujasteis": "rempujar",
    "rempuje": "rempujar",
    "rempujemos": "rempujar",
    "rempujen": "rempujar",
    "rempujes": "rempujar",
    "rempujo": "rempujar",
    "rempujones": "rempujón",
    "rempujos": "rempujo",
    "rempujábamos": "rempujar",
    "rempujáis": "rempujar",
    "rempujáramos": "rempujar",
    "rempujáremos": "rempujar",
    "rempujásemos": "rempujar",
    "rempujé": "rempujar",
    "rempujéis": "rempujar",
    "rempujó": "rempujar",
    "rems": "rem",
    "remuda": "remudar",
    "remudaba": "remudar",
    "remudabais": "remudar",
    "remudaban": "remudar",
    "remudabas": "remudar",
    "remudad": "remudar",
    "remudada": "remudar",
    "remudadas": "remudar",
    "remudado": "remudar",
    "remudados": "remudar",
    "remudamientos": "remudamiento",
    "remudamos": "remudar",
    "remudan": "remudar",
    "remudando": "remudar",
    "remudara": "remudar",
    "remudarais": "remudar",
    "remudaran": "remudar",
    "remudaras": "remudar",
    "remudare": "remudar",
    "remudareis": "remudar",
    "remudaremos": "remudar",
    "remudaren": "remudar",
    "remudares": "remudar",
    "remudaron": "remudar",
    "remudará": "remudar",
    "remudarán": "remudar",
    "remudarás": "remudar",
    "remudaré": "remudar",
    "remudaréis": "remudar",
    "remudaría": "remudar",
    "remudaríais": "remudar",
    "remudaríamos": "remudar",
    "remudarían": "remudar",
    "remudarías": "remudar",
    "remudas": "remudar",
    "remudase": "remudar",
    "remudaseis": "remudar",
    "remudasen": "remudar",
    "remudases": "remudar",
    "remudaste": "remudar",
    "remudasteis": "remudar",
    "remude": "remudar",
    "remudemos": "remudar",
    "remuden": "remudar",
    "remudes": "remudar",
    "remudo": "remudar",
    "remudábamos": "remudar",
    "remudáis": "remudar",
    "remudáramos": "remudar",
    "remudáremos": "remudar",
    "remudásemos": "remudar",
    "remudé": "remudar",
    "remudéis": "remudar",
    "remudó": "remudar",
    "remuela": "remoler",
    "remuelan": "remoler",
    "remuelas": "remoler",
    "remuele": "remoler",
    "remuelen": "remoler",
    "remueles": "remoler",
    "remuelo": "remoler",
    "remuerda": "remorder",
    "remuerdan": "remorder",
    "remuerdas": "remorder",
    "remuerde": "remorder",
    "remuerden": "remorder",
    "remuerdes": "remorder",
    "remuerdo": "remorder",
    "remueva": "remover",
    "remuevan": "remover",
    "remuevas": "remover",
    "remueve": "remover",
    "remueven": "remover",
    "remueves": "remover",
    "remuevo": "remover",
    "remulla": "remullir",
    "remullamos": "remullir",
    "remullan": "remullir",
    "remullas": "remullir",
    "remulle": "remullir",
    "remullen": "remullir",
    "remullendo": "remullir",
    "remullera": "remullir",
    "remullerais": "remullir",
    "remulleran": "remullir",
    "remulleras": "remullir",
    "remullere": "remullir",
    "remullereis": "remullir",
    "remulleren": "remullir",
    "remulleres": "remullir",
    "remulleron": "remullir",
    "remulles": "remullir",
    "remullese": "remullir",
    "remulleseis": "remullir",
    "remullesen": "remullir",
    "remulleses": "remullir",
    "remullid": "remullir",
    "remullida": "remullir",
    "remullidas": "remullir",
    "remullido": "remullir",
    "remullidos": "remullir",
    "remullimos": "remullir",
    "remulliremos": "remullir",
    "remullirá": "remullir",
    "remullirán": "remullir",
    "remullirás": "remullir",
    "remulliré": "remullir",
    "remulliréis": "remullir",
    "remulliría": "remullir",
    "remulliríais": "remullir",
    "remulliríamos": "remullir",
    "remullirían": "remullir",
    "remullirías": "remullir",
    "remulliste": "remullir",
    "remullisteis": "remullir",
    "remullo": "remullir",
    "remulláis": "remullir",
    "remulléramos": "remullir",
    "remulléremos": "remullir",
    "remullésemos": "remullir",
    "remullí": "remullir",
    "remullía": "remullir",
    "remullíais": "remullir",
    "remullíamos": "remullir",
    "remullían": "remullir",
    "remullías": "remullir",
    "remullís": "remullir",
    "remulló": "remullir",
    "remunera": "remunerar",
    "remuneraba": "remunerar",
    "remunerabais": "remunerar",
    "remuneraban": "remunerar",
    "remunerabas": "remunerar",
    "remuneraciones": "remuneración",
    "remuneraciónes": "remuneración",
    "remunerad": "remunerar",
    "remunerada": "remunerar",
    "remuneradas": "remunerar",
    "remunerado": "remunerar",
    "remuneradora": "remunerador",
    "remuneradoras": "remunerador",
    "remuneradores": "remunerador",
    "remunerados": "remunerar",
    "remuneramos": "remunerar",
    "remuneran": "remunerar",
    "remunerando": "remunerar",
    "remunerara": "remunerar",
    "remunerarais": "remunerar",
    "remuneraran": "remunerar",
    "remuneraras": "remunerar",
    "remunerare": "remunerar",
    "remunerareis": "remunerar",
    "remuneraremos": "remunerar",
    "remuneraren": "remunerar",
    "remunerares": "remunerar",
    "remuneraron": "remunerar",
    "remunerará": "remunerar",
    "remunerarán": "remunerar",
    "remunerarás": "remunerar",
    "remuneraré": "remunerar",
    "remuneraréis": "remunerar",
    "remuneraría": "remunerar",
    "remuneraríais": "remunerar",
    "remuneraríamos": "remunerar",
    "remunerarían": "remunerar",
    "remunerarías": "remunerar",
    "remuneras": "remunerar",
    "remunerase": "remunerar",
    "remuneraseis": "remunerar",
    "remunerasen": "remunerar",
    "remunerases": "remunerar",
    "remuneraste": "remunerar",
    "remunerasteis": "remunerar",
    "remunerativa": "remunerativo",
    "remunerativas": "remunerativo",
    "remunerativos": "remunerativo",
    "remuneratoria": "remuneratorio",
    "remuneratorias": "remuneratorio",
    "remuneratorios": "remuneratorio",
    "remunere": "remunerar",
    "remuneremos": "remunerar",
    "remuneren": "remunerar",
    "remuneres": "remunerar",
    "remunero": "remunerar",
    "remunerábamos": "remunerar",
    "remuneráis": "remunerar",
    "remuneráramos": "remunerar",
    "remuneráremos": "remunerar",
    "remunerásemos": "remunerar",
    "remuneré": "remunerar",
    "remuneréis": "remunerar",
    "remuneró": "remunerar",
    "remusga": "remusgar",
    "remusgaba": "remusgar",
    "remusgabais": "remusgar",
    "remusgaban": "remusgar",
    "remusgabas": "remusgar",
    "remusgad": "remusgar",
    "remusgada": "remusgar",
    "remusgadas": "remusgar",
    "remusgado": "remusgar",
    "remusgados": "remusgar",
    "remusgamos": "remusgar",
    "remusgan": "remusgar",
    "remusgando": "remusgar",
    "remusgara": "remusgar",
    "remusgarais": "remusgar",
    "remusgaran": "remusgar",
    "remusgaras": "remusgar",
    "remusgare": "remusgar",
    "remusgareis": "remusgar",
    "remusgaremos": "remusgar",
    "remusgaren": "remusgar",
    "remusgares": "remusgar",
    "remusgaron": "remusgar",
    "remusgará": "remusgar",
    "remusgarán": "remusgar",
    "remusgarás": "remusgar",
    "remusgaré": "remusgar",
    "remusgaréis": "remusgar",
    "remusgaría": "remusgar",
    "remusgaríais": "remusgar",
    "remusgaríamos": "remusgar",
    "remusgarían": "remusgar",
    "remusgarías": "remusgar",
    "remusgas": "remusgar",
    "remusgase": "remusgar",
    "remusgaseis": "remusgar",
    "remusgasen": "remusgar",
    "remusgases": "remusgar",
    "remusgaste": "remusgar",
    "remusgasteis": "remusgar",
    "remusgo": "remusgar",
    "remusgos": "remusgo",
    "remusgue": "remusgar",
    "remusguemos": "remusgar",
    "remusguen": "remusgar",
    "remusgues": "remusgar",
    "remusgué": "remusgar",
    "remusguéis": "remusgar",
    "remusgábamos": "remusgar",
    "remusgáis": "remusgar",
    "remusgáramos": "remusgar",
    "remusgáremos": "remusgar",
    "remusgásemos": "remusgar",
    "remusgó": "remusgar",
    "remábamos": "remar",
    "remáis": "remar",
    "remáramos": "remar",
    "remáremos": "remar",
    "remásemos": "remar",
    "remé": "remar",
    "reméis": "remar",
    "remó": "remar",
    "renace": "renacer",
    "renaced": "renacer",
    "renacemos": "renacer",
    "renacen": "renacer",
    "renacentistas": "renacentista",
    "renaceremos": "renacer",
    "renacerá": "renacer",
    "renacerán": "renacer",
    "renacerás": "renacer",
    "renaceré": "renacer",
    "renaceréis": "renacer",
    "renacería": "renacer",
    "renaceríais": "renacer",
    "renaceríamos": "renacer",
    "renacerían": "renacer",
    "renacerías": "renacer",
    "renaces": "renacer",
    "renacida": "renacer",
    "renacidas": "renacer",
    "renacido": "renacer",
    "renacidos": "renacer",
    "renaciendo": "renacer",
    "renacientes": "renaciente",
    "renaciera": "renacer",
    "renacierais": "renacer",
    "renacieran": "renacer",
    "renacieras": "renacer",
    "renaciere": "renacer",
    "renaciereis": "renacer",
    "renacieren": "renacer",
    "renacieres": "renacer",
    "renacieron": "renacer",
    "renaciese": "renacer",
    "renacieseis": "renacer",
    "renaciesen": "renacer",
    "renacieses": "renacer",
    "renacimientos": "renacimiento",
    "renacimos": "renacer",
    "renaciste": "renacer",
    "renacisteis": "renacer",
    "renaciéramos": "renacer",
    "renaciéremos": "renacer",
    "renaciésemos": "renacer",
    "renació": "renacer",
    "renacuajos": "renacuajo",
    "renacéis": "renacer",
    "renací": "renacer",
    "renacía": "renacer",
    "renacíais": "renacer",
    "renacíamos": "renacer",
    "renacían": "renacer",
    "renacías": "renacer",
    "renadíos": "renadío",
    "renaixenças": "renaixença",
    "renales": "renal",
    "renana": "renano",
    "renanas": "renano",
    "renanos": "renano",
    "renards": "renard",
    "renazca": "renacer",
    "renazcamos": "renacer",
    "renazcan": "renacer",
    "renazcas": "renacer",
    "renazco": "renacer",
    "renazcáis": "renacer",
    "renca": "renco",
    "rencas": "renco",
    "rencillas": "rencilla",
    "rencillosa": "rencilloso",
    "rencillosas": "rencilloso",
    "rencillosos": "rencilloso",
    "rencores": "rencor",
    "rencorosa": "rencoroso",
    "rencorosas": "rencoroso",
    "rencorosos": "rencoroso",
    "rencos": "renco",
    "rencosa": "rencoso",
    "rencosas": "rencoso",
    "rencosos": "rencoso",
    "rendajes": "rendaje",
    "rendas": "renda",
    "rendibúes": "rendibú",
    "rendibús": "rendibú",
    "rendiciones": "rendición",
    "rendid": "rendir",
    "rendida": "rendir",
    "rendidas": "rendir",
    "rendido": "rendir",
    "rendidos": "rendir",
    "rendijas": "rendija",
    "rendimientos": "rendimiento",
    "rendimos": "rendir",
    "rendiremos": "rendir",
    "rendirá": "rendir",
    "rendirán": "rendir",
    "rendirás": "rendir",
    "rendiré": "rendir",
    "rendiréis": "rendir",
    "rendiría": "rendir",
    "rendiríais": "rendir",
    "rendiríamos": "rendir",
    "rendirían": "rendir",
    "rendirías": "rendir",
    "rendiste": "rendir",
    "rendisteis": "rendir",
    "rendsinas": "rendsina",
    "rendzinas": "rendzina",
    "rendí": "rendir",
    "rendía": "rendir",
    "rendíais": "rendir",
    "rendíamos": "rendir",
    "rendían": "rendir",
    "rendías": "rendir",
    "rendís": "rendir",
    "renegaba": "renegar",
    "renegabais": "renegar",
    "renegaban": "renegar",
    "renegabas": "renegar",
    "renegad": "renegar",
    "renegada": "renegar",
    "renegadas": "renegar",
    "renegado": "renegar",
    "renegadora": "renegador",
    "renegadoras": "renegador",
    "renegadores": "renegador",
    "renegados": "renegar",
    "renegamos": "renegar",
    "renegando": "renegar",
    "renegara": "renegar",
    "renegarais": "renegar",
    "renegaran": "renegar",
    "renegaras": "renegar",
    "renegare": "renegar",
    "renegareis": "renegar",
    "renegaremos": "renegar",
    "renegaren": "renegar",
    "renegares": "renegar",
    "renegaron": "renegar",
    "renegará": "renegar",
    "renegarán": "renegar",
    "renegarás": "renegar",
    "renegaré": "renegar",
    "renegaréis": "renegar",
    "renegaría": "renegar",
    "renegaríais": "renegar",
    "renegaríamos": "renegar",
    "renegarían": "renegar",
    "renegarías": "renegar",
    "renegase": "renegar",
    "renegaseis": "renegar",
    "renegasen": "renegar",
    "renegases": "renegar",
    "renegaste": "renegar",
    "renegasteis": "renegar",
    "renegocia": "renegociar",
    "renegociaba": "renegociar",
    "renegociabais": "renegociar",
    "renegociaban": "renegociar",
    "renegociabas": "renegociar",
    "renegociaciones": "renegociación",
    "renegociad": "renegociar",
    "renegociada": "renegociar",
    "renegociadas": "renegociar",
    "renegociado": "renegociar",
    "renegociados": "renegociar",
    "renegociamos": "renegociar",
    "renegocian": "renegociar",
    "renegociando": "renegociar",
    "renegociara": "renegociar",
    "renegociarais": "renegociar",
    "renegociaran": "renegociar",
    "renegociaras": "renegociar",
    "renegociare": "renegociar",
    "renegociareis": "renegociar",
    "renegociaremos": "renegociar",
    "renegociaren": "renegociar",
    "renegociares": "renegociar",
    "renegociaron": "renegociar",
    "renegociará": "renegociar",
    "renegociarán": "renegociar",
    "renegociarás": "renegociar",
    "renegociaré": "renegociar",
    "renegociaréis": "renegociar",
    "renegociaría": "renegociar",
    "renegociaríais": "renegociar",
    "renegociaríamos": "renegociar",
    "renegociarían": "renegociar",
    "renegociarías": "renegociar",
    "renegocias": "renegociar",
    "renegociase": "renegociar",
    "renegociaseis": "renegociar",
    "renegociasen": "renegociar",
    "renegociases": "renegociar",
    "renegociaste": "renegociar",
    "renegociasteis": "renegociar",
    "renegocie": "renegociar",
    "renegociemos": "renegociar",
    "renegocien": "renegociar",
    "renegocies": "renegociar",
    "renegocio": "renegociar",
    "renegociábamos": "renegociar",
    "renegociáis": "renegociar",
    "renegociáramos": "renegociar",
    "renegociáremos": "renegociar",
    "renegociásemos": "renegociar",
    "renegocié": "renegociar",
    "renegociéis": "renegociar",
    "renegoció": "renegociar",
    "renegona": "renegón",
    "renegonas": "renegón",
    "renegones": "renegón",
    "renegrida": "renegrido",
    "renegridas": "renegrido",
    "renegridos": "renegrido",
    "reneguemos": "renegar",
    "renegué": "renegar",
    "reneguéis": "renegar",
    "renegábamos": "renegar",
    "renegáis": "renegar",
    "renegáramos": "renegar",
    "renegáremos": "renegar",
    "renegásemos": "renegar",
    "renegó": "renegar",
    "renga": "rengo",
    "rengas": "rengo",
    "renglonaduras": "renglonadura",
    "renglones": "renglón",
    "rengos": "rengo",
    "renguea": "renguear",
    "rengueaba": "renguear",
    "rengueabais": "renguear",
    "rengueaban": "renguear",
    "rengueabas": "renguear",
    "renguead": "renguear",
    "rengueada": "renguear",
    "rengueadas": "renguear",
    "rengueado": "renguear",
    "rengueados": "renguear",
    "rengueamos": "renguear",
    "renguean": "renguear",
    "rengueando": "renguear",
    "rengueara": "renguear",
    "renguearais": "renguear",
    "renguearan": "renguear",
    "renguearas": "renguear",
    "rengueare": "renguear",
    "rengueareis": "renguear",
    "renguearemos": "renguear",
    "renguearen": "renguear",
    "rengueares": "renguear",
    "renguearon": "renguear",
    "rengueará": "renguear",
    "renguearán": "renguear",
    "renguearás": "renguear",
    "renguearé": "renguear",
    "renguearéis": "renguear",
    "renguearía": "renguear",
    "renguearíais": "renguear",
    "renguearíamos": "renguear",
    "renguearían": "renguear",
    "renguearías": "renguear",
    "rengueas": "renguear",
    "renguease": "renguear",
    "rengueaseis": "renguear",
    "rengueasen": "renguear",
    "rengueases": "renguear",
    "rengueaste": "renguear",
    "rengueasteis": "renguear",
    "renguee": "renguear",
    "rengueemos": "renguear",
    "rengueen": "renguear",
    "renguees": "renguear",
    "rengueo": "renguear",
    "rengueábamos": "renguear",
    "rengueáis": "renguear",
    "rengueáramos": "renguear",
    "rengueáremos": "renguear",
    "rengueásemos": "renguear",
    "rengueé": "renguear",
    "rengueéis": "renguear",
    "rengueó": "renguear",
    "rengíferos": "rengífero",
    "reniega": "renegar",
    "reniegan": "renegar",
    "reniegas": "renegar",
    "reniego": "renegar",
    "reniegos": "reniego",
    "reniegue": "renegar",
    "renieguen": "renegar",
    "reniegues": "renegar",
    "reniformes": "reniforme",
    "renios": "renio",
    "renitencias": "renitencia",
    "renitentes": "renitente",
    "renombrada": "renombrado",
    "renombradas": "renombrado",
    "renombrados": "renombrado",
    "renombramientos": "renombramiento",
    "renombres": "renombre",
    "renos": "reno",
    "renovaba": "renovar",
    "renovabais": "renovar",
    "renovaban": "renovar",
    "renovabas": "renovar",
    "renovables": "renovable",
    "renovaciones": "renovación",
    "renovad": "renovar",
    "renovada": "renovar",
    "renovadas": "renovar",
    "renovado": "renovar",
    "renovadora": "renovador",
    "renovadoras": "renovador",
    "renovadores": "renovador",
    "renovados": "renovar",
    "renovales": "renoval",
    "renovamos": "renovar",
    "renovando": "renovar",
    "renovara": "renovar",
    "renovarais": "renovar",
    "renovaran": "renovar",
    "renovaras": "renovar",
    "renovare": "renovar",
    "renovareis": "renovar",
    "renovaremos": "renovar",
    "renovaren": "renovar",
    "renovares": "renovar",
    "renovaron": "renovar",
    "renovará": "renovar",
    "renovarán": "renovar",
    "renovarás": "renovar",
    "renovaré": "renovar",
    "renovaréis": "renovar",
    "renovaría": "renovar",
    "renovaríais": "renovar",
    "renovaríamos": "renovar",
    "renovarían": "renovar",
    "renovarías": "renovar",
    "renovase": "renovar",
    "renovaseis": "renovar",
    "renovasen": "renovar",
    "renovases": "renovar",
    "renovaste": "renovar",
    "renovasteis": "renovar",
    "renovemos": "renovar",
    "renovábamos": "renovar",
    "renováis": "renovar",
    "renováramos": "renovar",
    "renováremos": "renovar",
    "renovásemos": "renovar",
    "renové": "renovar",
    "renovéis": "renovar",
    "renovó": "renovar",
    "renquea": "renquear",
    "renqueaba": "renquear",
    "renqueabais": "renquear",
    "renqueaban": "renquear",
    "renqueabas": "renquear",
    "renquead": "renquear",
    "renqueada": "renquear",
    "renqueadas": "renquear",
    "renqueado": "renquear",
    "renqueados": "renquear",
    "renqueamos": "renquear",
    "renquean": "renquear",
    "renqueando": "renquear",
    "renqueantes": "renqueante",
    "renqueara": "renquear",
    "renquearais": "renquear",
    "renquearan": "renquear",
    "renquearas": "renquear",
    "renqueare": "renquear",
    "renqueareis": "renquear",
    "renquearemos": "renquear",
    "renquearen": "renquear",
    "renqueares": "renquear",
    "renquearon": "renquear",
    "renqueará": "renquear",
    "renquearán": "renquear",
    "renquearás": "renquear",
    "renquearé": "renquear",
    "renquearéis": "renquear",
    "renquearía": "renquear",
    "renquearíais": "renquear",
    "renquearíamos": "renquear",
    "renquearían": "renquear",
    "renquearías": "renquear",
    "renqueas": "renquear",
    "renquease": "renquear",
    "renqueaseis": "renquear",
    "renqueasen": "renquear",
    "renqueases": "renquear",
    "renqueaste": "renquear",
    "renqueasteis": "renquear",
    "renquee": "renquear",
    "renqueemos": "renquear",
    "renqueen": "renquear",
    "renquees": "renquear",
    "renqueo": "renquear",
    "renqueos": "renqueo",
    "renqueábamos": "renquear",
    "renqueáis": "renquear",
    "renqueáramos": "renquear",
    "renqueáremos": "renquear",
    "renqueásemos": "renquear",
    "renqueé": "renquear",
    "renqueéis": "renquear",
    "renqueó": "renquear",
    "renta": "rentar",
    "rentaba": "rentar",
    "rentabais": "rentar",
    "rentaban": "rentar",
    "rentabas": "rentar",
    "rentabilice": "rentabilizar",
    "rentabilicemos": "rentabilizar",
    "rentabilicen": "rentabilizar",
    "rentabilices": "rentabilizar",
    "rentabilicé": "rentabilizar",
    "rentabilicéis": "rentabilizar",
    "rentabilidades": "rentabilidad",
    "rentabiliza": "rentabilizar",
    "rentabilizaba": "rentabilizar",
    "rentabilizabais": "rentabilizar",
    "rentabilizaban": "rentabilizar",
    "rentabilizabas": "rentabilizar",
    "rentabilizaciones": "rentabilización",
    "rentabilizaciónes": "rentabilización",
    "rentabilizad": "rentabilizar",
    "rentabilizada": "rentabilizar",
    "rentabilizadas": "rentabilizar",
    "rentabilizado": "rentabilizar",
    "rentabilizados": "rentabilizar",
    "rentabilizamos": "rentabilizar",
    "rentabilizan": "rentabilizar",
    "rentabilizando": "rentabilizar",
    "rentabilizara": "rentabilizar",
    "rentabilizarais": "rentabilizar",
    "rentabilizaran": "rentabilizar",
    "rentabilizaras": "rentabilizar",
    "rentabilizare": "rentabilizar",
    "rentabilizareis": "rentabilizar",
    "rentabilizaremos": "rentabilizar",
    "rentabilizaren": "rentabilizar",
    "rentabilizares": "rentabilizar",
    "rentabilizaron": "rentabilizar",
    "rentabilizará": "rentabilizar",
    "rentabilizarán": "rentabilizar",
    "rentabilizarás": "rentabilizar",
    "rentabilizaré": "rentabilizar",
    "rentabilizaréis": "rentabilizar",
    "rentabilizaría": "rentabilizar",
    "rentabilizaríais": "rentabilizar",
    "rentabilizaríamos": "rentabilizar",
    "rentabilizarían": "rentabilizar",
    "rentabilizarías": "rentabilizar",
    "rentabilizas": "rentabilizar",
    "rentabilizase": "rentabilizar",
    "rentabilizaseis": "rentabilizar",
    "rentabilizasen": "rentabilizar",
    "rentabilizases": "rentabilizar",
    "rentabilizaste": "rentabilizar",
    "rentabilizasteis": "rentabilizar",
    "rentabilizo": "rentabilizar",
    "rentabilizábamos": "rentabilizar",
    "rentabilizáis": "rentabilizar",
    "rentabilizáramos": "rentabilizar",
    "rentabilizáremos": "rentabilizar",
    "rentabilizásemos": "rentabilizar",
    "rentabilizó": "rentabilizar",
    "rentables": "rentable",
    "rentad": "rentar",
    "rentada": "rentar",
    "rentadas": "rentar",
    "rentado": "rentar",
    "rentados": "rentar",
    "rentamos": "rentar",
    "rentan": "rentar",
    "rentando": "rentar",
    "rentara": "rentar",
    "rentarais": "rentar",
    "rentaran": "rentar",
    "rentaras": "rentar",
    "rentare": "rentar",
    "rentareis": "rentar",
    "rentaremos": "rentar",
    "rentaren": "rentar",
    "rentares": "rentar",
    "rentaron": "rentar",
    "rentará": "rentar",
    "rentarán": "rentar",
    "rentarás": "rentar",
    "rentaré": "rentar",
    "rentaréis": "rentar",
    "rentaría": "rentar",
    "rentaríais": "rentar",
    "rentaríamos": "rentar",
    "rentarían": "rentar",
    "rentarías": "rentar",
    "rentas": "rentar",
    "rentase": "rentar",
    "rentaseis": "rentar",
    "rentasen": "rentar",
    "rentases": "rentar",
    "rentaste": "rentar",
    "rentasteis": "rentar",
    "rente": "rentar",
    "rentemos": "rentar",
    "renten": "rentar",
    "rentera": "rentero",
    "renteras": "rentero",
    "renteros": "rentero",
    "rentes": "rentar",
    "rentings": "renting",
    "rentistas": "rentista",
    "rento": "rentar",
    "rentos": "rento",
    "rentosa": "rentoso",
    "rentosas": "rentoso",
    "rentosos": "rentoso",
    "rentoyes": "rentoy",
    "rentábamos": "rentar",
    "rentáis": "rentar",
    "rentáramos": "rentar",
    "rentáremos": "rentar",
    "rentásemos": "rentar",
    "renté": "rentar",
    "rentéis": "rentar",
    "rentística": "rentístico",
    "rentísticas": "rentístico",
    "rentísticos": "rentístico",
    "rentó": "rentar",
    "rentóis": "rentoy",
    "renuencias": "renuencia",
    "renuentes": "renuente",
    "renueva": "renovar",
    "renuevan": "renovar",
    "renuevas": "renovar",
    "renueve": "renovar",
    "renueven": "renovar",
    "renueves": "renovar",
    "renuevo": "renovar",
    "renuevos": "renuevo",
    "renuncia": "renunciar",
    "renunciaba": "renunciar",
    "renunciabais": "renunciar",
    "renunciaban": "renunciar",
    "renunciabas": "renunciar",
    "renunciables": "renunciable",
    "renunciaciones": "renunciación",
    "renunciad": "renunciar",
    "renunciada": "renunciar",
    "renunciadas": "renunciar",
    "renunciado": "renunciar",
    "renunciados": "renunciar",
    "renunciamientos": "renunciamiento",
    "renunciamos": "renunciar",
    "renuncian": "renunciar",
    "renunciando": "renunciar",
    "renunciantes": "renunciante",
    "renunciara": "renunciar",
    "renunciarais": "renunciar",
    "renunciaran": "renunciar",
    "renunciaras": "renunciar",
    "renunciare": "renunciar",
    "renunciareis": "renunciar",
    "renunciaremos": "renunciar",
    "renunciaren": "renunciar",
    "renunciares": "renunciar",
    "renunciaron": "renunciar",
    "renunciará": "renunciar",
    "renunciarán": "renunciar",
    "renunciarás": "renunciar",
    "renunciaré": "renunciar",
    "renunciaréis": "renunciar",
    "renunciaría": "renunciar",
    "renunciaríais": "renunciar",
    "renunciaríamos": "renunciar",
    "renunciarían": "renunciar",
    "renunciarías": "renunciar",
    "renuncias": "renunciar",
    "renunciase": "renunciar",
    "renunciaseis": "renunciar",
    "renunciasen": "renunciar",
    "renunciases": "renunciar",
    "renunciaste": "renunciar",
    "renunciasteis": "renunciar",
    "renunciataria": "renunciatario",
    "renunciatarias": "renunciatario",
    "renunciatarios": "renunciatario",
    "renuncie": "renunciar",
    "renunciemos": "renunciar",
    "renuncien": "renunciar",
    "renuncies": "renunciar",
    "renuncio": "renunciar",
    "renunciábamos": "renunciar",
    "renunciáis": "renunciar",
    "renunciáramos": "renunciar",
    "renunciáremos": "renunciar",
    "renunciásemos": "renunciar",
    "renuncié": "renunciar",
    "renunciéis": "renunciar",
    "renunció": "renunciar",
    "renvalsa": "renvalsar",
    "renvalsaba": "renvalsar",
    "renvalsabais": "renvalsar",
    "renvalsaban": "renvalsar",
    "renvalsabas": "renvalsar",
    "renvalsad": "renvalsar",
    "renvalsada": "renvalsar",
    "renvalsadas": "renvalsar",
    "renvalsado": "renvalsar",
    "renvalsados": "renvalsar",
    "renvalsamos": "renvalsar",
    "renvalsan": "renvalsar",
    "renvalsando": "renvalsar",
    "renvalsara": "renvalsar",
    "renvalsarais": "renvalsar",
    "renvalsaran": "renvalsar",
    "renvalsaras": "renvalsar",
    "renvalsare": "renvalsar",
    "renvalsareis": "renvalsar",
    "renvalsaremos": "renvalsar",
    "renvalsaren": "renvalsar",
    "renvalsares": "renvalsar",
    "renvalsaron": "renvalsar",
    "renvalsará": "renvalsar",
    "renvalsarán": "renvalsar",
    "renvalsarás": "renvalsar",
    "renvalsaré": "renvalsar",
    "renvalsaréis": "renvalsar",
    "renvalsaría": "renvalsar",
    "renvalsaríais": "renvalsar",
    "renvalsaríamos": "renvalsar",
    "renvalsarían": "renvalsar",
    "renvalsarías": "renvalsar",
    "renvalsas": "renvalsar",
    "renvalsase": "renvalsar",
    "renvalsaseis": "renvalsar",
    "renvalsasen": "renvalsar",
    "renvalsases": "renvalsar",
    "renvalsaste": "renvalsar",
    "renvalsasteis": "renvalsar",
    "renvalse": "renvalsar",
    "renvalsemos": "renvalsar",
    "renvalsen": "renvalsar",
    "renvalses": "renvalsar",
    "renvalso": "renvalsar",
    "renvalsos": "renvalso",
    "renvalsábamos": "renvalsar",
    "renvalsáis": "renvalsar",
    "renvalsáramos": "renvalsar",
    "renvalsáremos": "renvalsar",
    "renvalsásemos": "renvalsar",
    "renvalsé": "renvalsar",
    "renvalséis": "renvalsar",
    "renvalsó": "renvalsar",
    "reográfica": "reográfico",
    "reográficas": "reográfico",
    "reográficos": "reográfico",
    "reologías": "reología",
    "reológica": "reológico",
    "reológicas": "reológico",
    "reológicos": "reológico",
    "reopexias": "reopexia",
    "reopéctica": "reopéctico",
    "reopécticas": "reopéctico",
    "reopécticos": "reopéctico",
    "reordena": "reordenar",
    "reordenaba": "reordenar",
    "reordenabais": "reordenar",
    "reordenaban": "reordenar",
    "reordenabas": "reordenar",
    "reordenaciones": "reordenación",
    "reordenad": "reordenar",
    "reordenada": "reordenar",
    "reordenadas": "reordenar",
    "reordenado": "reordenar",
    "reordenados": "reordenar",
    "reordenamientos": "reordenamiento",
    "reordenamos": "reordenar",
    "reordenan": "reordenar",
    "reordenando": "reordenar",
    "reordenara": "reordenar",
    "reordenarais": "reordenar",
    "reordenaran": "reordenar",
    "reordenaras": "reordenar",
    "reordenare": "reordenar",
    "reordenareis": "reordenar",
    "reordenaremos": "reordenar",
    "reordenaren": "reordenar",
    "reordenares": "reordenar",
    "reordenaron": "reordenar",
    "reordenará": "reordenar",
    "reordenarán": "reordenar",
    "reordenarás": "reordenar",
    "reordenaré": "reordenar",
    "reordenaréis": "reordenar",
    "reordenaría": "reordenar",
    "reordenaríais": "reordenar",
    "reordenaríamos": "reordenar",
    "reordenarían": "reordenar",
    "reordenarías": "reordenar",
    "reordenas": "reordenar",
    "reordenase": "reordenar",
    "reordenaseis": "reordenar",
    "reordenasen": "reordenar",
    "reordenases": "reordenar",
    "reordenaste": "reordenar",
    "reordenasteis": "reordenar",
    "reordene": "reordenar",
    "reordenemos": "reordenar",
    "reordenen": "reordenar",
    "reordenes": "reordenar",
    "reordeno": "reordenar",
    "reordenábamos": "reordenar",
    "reordenáis": "reordenar",
    "reordenáramos": "reordenar",
    "reordenáremos": "reordenar",
    "reordenásemos": "reordenar",
    "reordené": "reordenar",
    "reordenéis": "reordenar",
    "reordenó": "reordenar",
    "reorganice": "reorganizar",
    "reorganicemos": "reorganizar",
    "reorganicen": "reorganizar",
    "reorganices": "reorganizar",
    "reorganicé": "reorganizar",
    "reorganicéis": "reorganizar",
    "reorganiza": "reorganizar",
    "reorganizaba": "reorganizar",
    "reorganizabais": "reorganizar",
    "reorganizaban": "reorganizar",
    "reorganizabas": "reorganizar",
    "reorganizaciones": "reorganización",
    "reorganizad": "reorganizar",
    "reorganizada": "reorganizar",
    "reorganizadas": "reorganizar",
    "reorganizado": "reorganizar",
    "reorganizadora": "reorganizador",
    "reorganizadoras": "reorganizador",
    "reorganizadores": "reorganizador",
    "reorganizados": "reorganizar",
    "reorganizamos": "reorganizar",
    "reorganizan": "reorganizar",
    "reorganizando": "reorganizar",
    "reorganizara": "reorganizar",
    "reorganizarais": "reorganizar",
    "reorganizaran": "reorganizar",
    "reorganizaras": "reorganizar",
    "reorganizare": "reorganizar",
    "reorganizareis": "reorganizar",
    "reorganizaremos": "reorganizar",
    "reorganizaren": "reorganizar",
    "reorganizares": "reorganizar",
    "reorganizaron": "reorganizar",
    "reorganizará": "reorganizar",
    "reorganizarán": "reorganizar",
    "reorganizarás": "reorganizar",
    "reorganizaré": "reorganizar",
    "reorganizaréis": "reorganizar",
    "reorganizaría": "reorganizar",
    "reorganizaríais": "reorganizar",
    "reorganizaríamos": "reorganizar",
    "reorganizarían": "reorganizar",
    "reorganizarías": "reorganizar",
    "reorganizas": "reorganizar",
    "reorganizase": "reorganizar",
    "reorganizaseis": "reorganizar",
    "reorganizasen": "reorganizar",
    "reorganizases": "reorganizar",
    "reorganizaste": "reorganizar",
    "reorganizasteis": "reorganizar",
    "reorganizo": "reorganizar",
    "reorganizábamos": "reorganizar",
    "reorganizáis": "reorganizar",
    "reorganizáramos": "reorganizar",
    "reorganizáremos": "reorganizar",
    "reorganizásemos": "reorganizar",
    "reorganizó": "reorganizar",
    "reorienta": "reorientar",
    "reorientaba": "reorientar",
    "reorientabais": "reorientar",
    "reorientaban": "reorientar",
    "reorientabas": "reorientar",
    "reorientaciones": "reorientación",
    "reorientad": "reorientar",
    "reorientada": "reorientar",
    "reorientadas": "reorientar",
    "reorientado": "reorientar",
    "reorientados": "reorientar",
    "reorientamos": "reorientar",
    "reorientan": "reorientar",
    "reorientando": "reorientar",
    "reorientara": "reorientar",
    "reorientarais": "reorientar",
    "reorientaran": "reorientar",
    "reorientaras": "reorientar",
    "reorientare": "reorientar",
    "reorientareis": "reorientar",
    "reorientaremos": "reorientar",
    "reorientaren": "reorientar",
    "reorientares": "reorientar",
    "reorientaron": "reorientar",
    "reorientará": "reorientar",
    "reorientarán": "reorientar",
    "reorientarás": "reorientar",
    "reorientaré": "reorientar",
    "reorientaréis": "reorientar",
    "reorientaría": "reorientar",
    "reorientaríais": "reorientar",
    "reorientaríamos": "reorientar",
    "reorientarían": "reorientar",
    "reorientarías": "reorientar",
    "reorientas": "reorientar",
    "reorientase": "reorientar",
    "reorientaseis": "reorientar",
    "reorientasen": "reorientar",
    "reorientases": "reorientar",
    "reorientaste": "reorientar",
    "reorientasteis": "reorientar",
    "reoriente": "reorientar",
    "reorientemos": "reorientar",
    "reorienten": "reorientar",
    "reorientes": "reorientar",
    "reoriento": "reorientar",
    "reorientábamos": "reorientar",
    "reorientáis": "reorientar",
    "reorientáramos": "reorientar",
    "reorientáremos": "reorientar",
    "reorientásemos": "reorientar",
    "reorienté": "reorientar",
    "reorientéis": "reorientar",
    "reorientó": "reorientar",
    "reos": "reo",
    "reoscopios": "reoscopio",
    "reostatos": "reostato",
    "reostricciones": "reostricción",
    "reostática": "reostático",
    "reostáticas": "reostático",
    "reostáticos": "reostático",
    "reoxidaciones": "reoxidación",
    "repajos": "repajo",
    "repaleos": "repaleo",
    "repanchiga": "repanchigar",
    "repanchigaba": "repanchigar",
    "repanchigabais": "repanchigar",
    "repanchigaban": "repanchigar",
    "repanchigabas": "repanchigar",
    "repanchigad": "repanchigar",
    "repanchigada": "repanchigar",
    "repanchigadas": "repanchigar",
    "repanchigado": "repanchigar",
    "repanchigados": "repanchigar",
    "repanchigamos": "repanchigar",
    "repanchigan": "repanchigar",
    "repanchigando": "repanchigar",
    "repanchigara": "repanchigar",
    "repanchigarais": "repanchigar",
    "repanchigaran": "repanchigar",
    "repanchigaras": "repanchigar",
    "repanchigare": "repanchigar",
    "repanchigareis": "repanchigar",
    "repanchigaremos": "repanchigar",
    "repanchigaren": "repanchigar",
    "repanchigares": "repanchigar",
    "repanchigaron": "repanchigar",
    "repanchigará": "repanchigar",
    "repanchigarán": "repanchigar",
    "repanchigarás": "repanchigar",
    "repanchigaré": "repanchigar",
    "repanchigaréis": "repanchigar",
    "repanchigaría": "repanchigar",
    "repanchigaríais": "repanchigar",
    "repanchigaríamos": "repanchigar",
    "repanchigarían": "repanchigar",
    "repanchigarías": "repanchigar",
    "repanchigas": "repanchigar",
    "repanchigase": "repanchigar",
    "repanchigaseis": "repanchigar",
    "repanchigasen": "repanchigar",
    "repanchigases": "repanchigar",
    "repanchigaste": "repanchigar",
    "repanchigasteis": "repanchigar",
    "repanchigo": "repanchigar",
    "repanchigue": "repanchigar",
    "repanchiguemos": "repanchigar",
    "repanchiguen": "repanchigar",
    "repanchigues": "repanchigar",
    "repanchigué": "repanchigar",
    "repanchiguéis": "repanchigar",
    "repanchigábamos": "repanchigar",
    "repanchigáis": "repanchigar",
    "repanchigáramos": "repanchigar",
    "repanchigáremos": "repanchigar",
    "repanchigásemos": "repanchigar",
    "repanchigó": "repanchigar",
    "repantes": "repante",
    "repantiga": "repantigar",
    "repantigaba": "repantigar",
    "repantigabais": "repantigar",
    "repantigaban": "repantigar",
    "repantigabas": "repantigar",
    "repantigad": "repantigar",
    "repantigada": "repantigar",
    "repantigadas": "repantigar",
    "repantigado": "repantigar",
    "repantigados": "repantigar",
    "repantigamos": "repantigar",
    "repantigan": "repantigar",
    "repantigando": "repantigar",
    "repantigara": "repantigar",
    "repantigarais": "repantigar",
    "repantigaran": "repantigar",
    "repantigaras": "repantigar",
    "repantigare": "repantigar",
    "repantigareis": "repantigar",
    "repantigaremos": "repantigar",
    "repantigaren": "repantigar",
    "repantigares": "repantigar",
    "repantigaron": "repantigar",
    "repantigará": "repantigar",
    "repantigarán": "repantigar",
    "repantigarás": "repantigar",
    "repantigaré": "repantigar",
    "repantigaréis": "repantigar",
    "repantigaría": "repantigar",
    "repantigaríais": "repantigar",
    "repantigaríamos": "repantigar",
    "repantigarían": "repantigar",
    "repantigarías": "repantigar",
    "repantigas": "repantigar",
    "repantigase": "repantigar",
    "repantigaseis": "repantigar",
    "repantigasen": "repantigar",
    "repantigases": "repantigar",
    "repantigaste": "repantigar",
    "repantigasteis": "repantigar",
    "repantigo": "repantigar",
    "repantigue": "repantigar",
    "repantiguemos": "repantigar",
    "repantiguen": "repantigar",
    "repantigues": "repantigar",
    "repantigué": "repantigar",
    "repantiguéis": "repantigar",
    "repantigábamos": "repantigar",
    "repantigáis": "repantigar",
    "repantigáramos": "repantigar",
    "repantigáremos": "repantigar",
    "repantigásemos": "repantigar",
    "repantigó": "repantigar",
    "repara": "reparar",
    "reparaba": "reparar",
    "reparabais": "reparar",
    "reparaban": "reparar",
    "reparabas": "reparar",
    "reparables": "reparable",
    "reparaciones": "reparación",
    "reparad": "reparar",
    "reparada": "reparar",
    "reparadas": "reparar",
    "reparado": "reparar",
    "reparadora": "reparador",
    "reparadoras": "reparador",
    "reparadores": "reparador",
    "reparados": "reparar",
    "reparamos": "reparar",
    "reparan": "reparar",
    "reparando": "reparar",
    "reparara": "reparar",
    "repararais": "reparar",
    "repararan": "reparar",
    "repararas": "reparar",
    "reparare": "reparar",
    "reparareis": "reparar",
    "repararemos": "reparar",
    "repararen": "reparar",
    "reparares": "reparar",
    "repararon": "reparar",
    "reparará": "reparar",
    "repararán": "reparar",
    "repararás": "reparar",
    "repararé": "reparar",
    "repararéis": "reparar",
    "repararía": "reparar",
    "repararíais": "reparar",
    "repararíamos": "reparar",
    "repararían": "reparar",
    "repararías": "reparar",
    "reparas": "reparar",
    "reparase": "reparar",
    "reparaseis": "reparar",
    "reparasen": "reparar",
    "reparases": "reparar",
    "reparaste": "reparar",
    "reparasteis": "reparar",
    "reparativa": "reparativo",
    "reparativas": "reparativo",
    "reparativos": "reparativo",
    "reparatoria": "reparatorio",
    "reparatorias": "reparatorio",
    "reparatorios": "reparatorio",
    "repare": "reparar",
    "reparemos": "reparar",
    "reparen": "reparar",
    "repares": "reparar",
    "reparo": "reparar",
    "reparona": "reparón",
    "reparonas": "reparón",
    "reparones": "reparón",
    "reparos": "reparo",
    "reparta": "repartir",
    "repartamos": "repartir",
    "repartan": "repartir",
    "repartas": "repartir",
    "reparte": "repartir",
    "reparten": "repartir",
    "repartes": "repartir",
    "repartibles": "repartible",
    "reparticiones": "repartición",
    "repartid": "repartir",
    "repartida": "repartir",
    "repartidas": "repartir",
    "repartidera": "repartidero",
    "repartideras": "repartidero",
    "repartideros": "repartidero",
    "repartido": "repartir",
    "repartidora": "repartidor",
    "repartidoras": "repartidor",
    "repartidores": "repartidor",
    "repartidos": "repartir",
    "repartiendo": "repartir",
    "repartiera": "repartir",
    "repartierais": "repartir",
    "repartieran": "repartir",
    "repartieras": "repartir",
    "repartiere": "repartir",
    "repartiereis": "repartir",
    "repartieren": "repartir",
    "repartieres": "repartir",
    "repartieron": "repartir",
    "repartiese": "repartir",
    "repartieseis": "repartir",
    "repartiesen": "repartir",
    "repartieses": "repartir",
    "repartimientos": "repartimiento",
    "repartimos": "repartir",
    "repartiremos": "repartir",
    "repartirá": "repartir",
    "repartirán": "repartir",
    "repartirás": "repartir",
    "repartiré": "repartir",
    "repartiréis": "repartir",
    "repartiría": "repartir",
    "repartiríais": "repartir",
    "repartiríamos": "repartir",
    "repartirían": "repartir",
    "repartirías": "repartir",
    "repartiste": "repartir",
    "repartisteis": "repartir",
    "repartiéramos": "repartir",
    "repartiéremos": "repartir",
    "repartiésemos": "repartir",
    "repartió": "repartir",
    "reparto": "repartir",
    "repartos": "reparto",
    "repartáis": "repartir",
    "repartí": "repartir",
    "repartía": "repartir",
    "repartíais": "repartir",
    "repartíamos": "repartir",
    "repartían": "repartir",
    "repartías": "repartir",
    "repartís": "repartir",
    "reparábamos": "reparar",
    "reparáis": "reparar",
    "reparáramos": "reparar",
    "reparáremos": "reparar",
    "reparásemos": "reparar",
    "reparé": "reparar",
    "reparéis": "reparar",
    "reparó": "reparar",
    "repasa": "repasar",
    "repasaba": "repasar",
    "repasabais": "repasar",
    "repasaban": "repasar",
    "repasabas": "repasar",
    "repasad": "repasar",
    "repasada": "repasar",
    "repasadas": "repasar",
    "repasaderas": "repasadera",
    "repasado": "repasar",
    "repasadora": "repasador",
    "repasadoras": "repasadora",
    "repasadores": "repasador",
    "repasados": "repasar",
    "repasamos": "repasar",
    "repasan": "repasar",
    "repasando": "repasar",
    "repasara": "repasar",
    "repasarais": "repasar",
    "repasaran": "repasar",
    "repasaras": "repasar",
    "repasare": "repasar",
    "repasareis": "repasar",
    "repasaremos": "repasar",
    "repasaren": "repasar",
    "repasares": "repasar",
    "repasaron": "repasar",
    "repasará": "repasar",
    "repasarán": "repasar",
    "repasarás": "repasar",
    "repasaré": "repasar",
    "repasaréis": "repasar",
    "repasaría": "repasar",
    "repasaríais": "repasar",
    "repasaríamos": "repasar",
    "repasarían": "repasar",
    "repasarías": "repasar",
    "repasas": "repasar",
    "repasase": "repasar",
    "repasaseis": "repasar",
    "repasasen": "repasar",
    "repasases": "repasar",
    "repasaste": "repasar",
    "repasasteis": "repasar",
    "repasatas": "repasata",
    "repase": "repasar",
    "repasemos": "repasar",
    "repasen": "repasar",
    "repases": "repasar",
    "repaso": "repasar",
    "repasos": "repaso",
    "repasta": "repastar",
    "repastaba": "repastar",
    "repastabais": "repastar",
    "repastaban": "repastar",
    "repastabas": "repastar",
    "repastad": "repastar",
    "repastada": "repastar",
    "repastadas": "repastar",
    "repastado": "repastar",
    "repastados": "repastar",
    "repastamos": "repastar",
    "repastan": "repastar",
    "repastando": "repastar",
    "repastara": "repastar",
    "repastarais": "repastar",
    "repastaran": "repastar",
    "repastaras": "repastar",
    "repastare": "repastar",
    "repastareis": "repastar",
    "repastaremos": "repastar",
    "repastaren": "repastar",
    "repastares": "repastar",
    "repastaron": "repastar",
    "repastará": "repastar",
    "repastarán": "repastar",
    "repastarás": "repastar",
    "repastaré": "repastar",
    "repastaréis": "repastar",
    "repastaría": "repastar",
    "repastaríais": "repastar",
    "repastaríamos": "repastar",
    "repastarían": "repastar",
    "repastarías": "repastar",
    "repastas": "repastar",
    "repastase": "repastar",
    "repastaseis": "repastar",
    "repastasen": "repastar",
    "repastases": "repastar",
    "repastaste": "repastar",
    "repastasteis": "repastar",
    "repaste": "repastar",
    "repastemos": "repastar",
    "repasten": "repastar",
    "repastes": "repastar",
    "repasto": "repastar",
    "repastos": "repasto",
    "repastábamos": "repastar",
    "repastáis": "repastar",
    "repastáramos": "repastar",
    "repastáremos": "repastar",
    "repastásemos": "repastar",
    "repasté": "repastar",
    "repastéis": "repastar",
    "repastó": "repastar",
    "repasábamos": "repasar",
    "repasáis": "repasar",
    "repasáramos": "repasar",
    "repasáremos": "repasar",
    "repasásemos": "repasar",
    "repasé": "repasar",
    "repaséis": "repasar",
    "repasó": "repasar",
    "repatriaba": "repatriar",
    "repatriabais": "repatriar",
    "repatriaban": "repatriar",
    "repatriabas": "repatriar",
    "repatriaciones": "repatriación",
    "repatriad": "repatriar",
    "repatriada": "repatriar",
    "repatriadas": "repatriar",
    "repatriado": "repatriar",
    "repatriados": "repatriar",
    "repatriamos": "repatriar",
    "repatriando": "repatriar",
    "repatriara": "repatriar",
    "repatriarais": "repatriar",
    "repatriaran": "repatriar",
    "repatriaras": "repatriar",
    "repatriare": "repatriar",
    "repatriareis": "repatriar",
    "repatriaremos": "repatriar",
    "repatriaren": "repatriar",
    "repatriares": "repatriar",
    "repatriaron": "repatriar",
    "repatriará": "repatriar",
    "repatriarán": "repatriar",
    "repatriarás": "repatriar",
    "repatriaré": "repatriar",
    "repatriaréis": "repatriar",
    "repatriaría": "repatriar",
    "repatriaríais": "repatriar",
    "repatriaríamos": "repatriar",
    "repatriarían": "repatriar",
    "repatriarías": "repatriar",
    "repatriase": "repatriar",
    "repatriaseis": "repatriar",
    "repatriasen": "repatriar",
    "repatriases": "repatriar",
    "repatriaste": "repatriar",
    "repatriasteis": "repatriar",
    "repatriemos": "repatriar",
    "repatriábamos": "repatriar",
    "repatriáis": "repatriar",
    "repatriáramos": "repatriar",
    "repatriáremos": "repatriar",
    "repatriásemos": "repatriar",
    "repatrié": "repatriar",
    "repatriéis": "repatriar",
    "repatrió": "repatriar",
    "repatría": "repatriar",
    "repatrían": "repatriar",
    "repatrías": "repatriar",
    "repatríe": "repatriar",
    "repatríen": "repatriar",
    "repatríes": "repatriar",
    "repatrío": "repatriar",
    "repecha": "repechar",
    "repechaba": "repechar",
    "repechabais": "repechar",
    "repechaban": "repechar",
    "repechabas": "repechar",
    "repechad": "repechar",
    "repechada": "repechar",
    "repechadas": "repechar",
    "repechado": "repechar",
    "repechados": "repechar",
    "repechamos": "repechar",
    "repechan": "repechar",
    "repechando": "repechar",
    "repechara": "repechar",
    "repecharais": "repechar",
    "repecharan": "repechar",
    "repecharas": "repechar",
    "repechare": "repechar",
    "repechareis": "repechar",
    "repecharemos": "repechar",
    "repecharen": "repechar",
    "repechares": "repechar",
    "repecharon": "repechar",
    "repechará": "repechar",
    "repecharán": "repechar",
    "repecharás": "repechar",
    "repecharé": "repechar",
    "repecharéis": "repechar",
    "repecharía": "repechar",
    "repecharíais": "repechar",
    "repecharíamos": "repechar",
    "repecharían": "repechar",
    "repecharías": "repechar",
    "repechas": "repechar",
    "repechase": "repechar",
    "repechaseis": "repechar",
    "repechasen": "repechar",
    "repechases": "repechar",
    "repechaste": "repechar",
    "repechasteis": "repechar",
    "repeche": "repechar",
    "repechemos": "repechar",
    "repechen": "repechar",
    "repeches": "repechar",
    "repecho": "repechar",
    "repechos": "repecho",
    "repechábamos": "repechar",
    "repecháis": "repechar",
    "repecháramos": "repechar",
    "repecháremos": "repechar",
    "repechásemos": "repechar",
    "repeché": "repechar",
    "repechéis": "repechar",
    "repechó": "repechar",
    "repeina": "repeinar",
    "repeinaba": "repeinar",
    "repeinabais": "repeinar",
    "repeinaban": "repeinar",
    "repeinabas": "repeinar",
    "repeinad": "repeinar",
    "repeinada": "repeinar",
    "repeinadas": "repeinar",
    "repeinado": "repeinar",
    "repeinados": "repeinar",
    "repeinamos": "repeinar",
    "repeinan": "repeinar",
    "repeinando": "repeinar",
    "repeinara": "repeinar",
    "repeinarais": "repeinar",
    "repeinaran": "repeinar",
    "repeinaras": "repeinar",
    "repeinare": "repeinar",
    "repeinareis": "repeinar",
    "repeinaremos": "repeinar",
    "repeinaren": "repeinar",
    "repeinares": "repeinar",
    "repeinaron": "repeinar",
    "repeinará": "repeinar",
    "repeinarán": "repeinar",
    "repeinarás": "repeinar",
    "repeinaré": "repeinar",
    "repeinaréis": "repeinar",
    "repeinaría": "repeinar",
    "repeinaríais": "repeinar",
    "repeinaríamos": "repeinar",
    "repeinarían": "repeinar",
    "repeinarías": "repeinar",
    "repeinas": "repeinar",
    "repeinase": "repeinar",
    "repeinaseis": "repeinar",
    "repeinasen": "repeinar",
    "repeinases": "repeinar",
    "repeinaste": "repeinar",
    "repeinasteis": "repeinar",
    "repeine": "repeinar",
    "repeinemos": "repeinar",
    "repeinen": "repeinar",
    "repeines": "repeinar",
    "repeino": "repeinar",
    "repeinábamos": "repeinar",
    "repeináis": "repeinar",
    "repeináramos": "repeinar",
    "repeináremos": "repeinar",
    "repeinásemos": "repeinar",
    "repeiné": "repeinar",
    "repeinéis": "repeinar",
    "repeinó": "repeinar",
    "repela": "repeler",
    "repelaba": "repelar",
    "repelabais": "repelar",
    "repelaban": "repelar",
    "repelabas": "repelar",
    "repelad": "repelar",
    "repelada": "repelar",
    "repeladas": "repelar",
    "repelado": "repelar",
    "repelados": "repelar",
    "repeladuras": "repeladura",
    "repelamos": "repeler",
    "repelan": "repeler",
    "repelando": "repelar",
    "repelara": "repelar",
    "repelarais": "repelar",
    "repelaran": "repelar",
    "repelaras": "repelar",
    "repelare": "repelar",
    "repelareis": "repelar",
    "repelaremos": "repelar",
    "repelaren": "repelar",
    "repelares": "repelar",
    "repelaron": "repelar",
    "repelará": "repelar",
    "repelarán": "repelar",
    "repelarás": "repelar",
    "repelaré": "repelar",
    "repelaréis": "repelar",
    "repelaría": "repelar",
    "repelaríais": "repelar",
    "repelaríamos": "repelar",
    "repelarían": "repelar",
    "repelarías": "repelar",
    "repelas": "repeler",
    "repelase": "repelar",
    "repelaseis": "repelar",
    "repelasen": "repelar",
    "repelases": "repelar",
    "repelaste": "repelar",
    "repelasteis": "repelar",
    "repele": "repeler",
    "repeled": "repeler",
    "repelemos": "repeler",
    "repelen": "repeler",
    "repelencias": "repelencia",
    "repelentes": "repelente",
    "repeleremos": "repeler",
    "repelerá": "repeler",
    "repelerán": "repeler",
    "repelerás": "repeler",
    "repeleré": "repeler",
    "repeleréis": "repeler",
    "repelería": "repeler",
    "repeleríais": "repeler",
    "repeleríamos": "repeler",
    "repelerían": "repeler",
    "repelerías": "repeler",
    "repeles": "repeler",
    "repelida": "repeler",
    "repelidas": "repeler",
    "repelido": "repeler",
    "repelidos": "repeler",
    "repeliendo": "repeler",
    "repeliera": "repeler",
    "repelierais": "repeler",
    "repelieran": "repeler",
    "repelieras": "repeler",
    "repeliere": "repeler",
    "repeliereis": "repeler",
    "repelieren": "repeler",
    "repelieres": "repeler",
    "repelieron": "repeler",
    "repeliese": "repeler",
    "repelieseis": "repeler",
    "repeliesen": "repeler",
    "repelieses": "repeler",
    "repelimos": "repeler",
    "repeliste": "repeler",
    "repelisteis": "repeler",
    "repeliéramos": "repeler",
    "repeliéremos": "repeler",
    "repeliésemos": "repeler",
    "repelió": "repeler",
    "repella": "repellar",
    "repellaba": "repellar",
    "repellabais": "repellar",
    "repellaban": "repellar",
    "repellabas": "repellar",
    "repellad": "repellar",
    "repellada": "repellar",
    "repelladas": "repellar",
    "repellado": "repellar",
    "repellados": "repellar",
    "repellamos": "repellar",
    "repellan": "repellar",
    "repellando": "repellar",
    "repellara": "repellar",
    "repellarais": "repellar",
    "repellaran": "repellar",
    "repellaras": "repellar",
    "repellare": "repellar",
    "repellareis": "repellar",
    "repellaremos": "repellar",
    "repellaren": "repellar",
    "repellares": "repellar",
    "repellaron": "repellar",
    "repellará": "repellar",
    "repellarán": "repellar",
    "repellarás": "repellar",
    "repellaré": "repellar",
    "repellaréis": "repellar",
    "repellaría": "repellar",
    "repellaríais": "repellar",
    "repellaríamos": "repellar",
    "repellarían": "repellar",
    "repellarías": "repellar",
    "repellas": "repellar",
    "repellase": "repellar",
    "repellaseis": "repellar",
    "repellasen": "repellar",
    "repellases": "repellar",
    "repellaste": "repellar",
    "repellasteis": "repellar",
    "repelle": "repellar",
    "repellemos": "repellar",
    "repellen": "repellar",
    "repelles": "repellar",
    "repello": "repellar",
    "repellos": "repello",
    "repellábamos": "repellar",
    "repelláis": "repellar",
    "repelláramos": "repellar",
    "repelláremos": "repellar",
    "repellásemos": "repellar",
    "repellé": "repellar",
    "repelléis": "repellar",
    "repelló": "repellar",
    "repelo": "repeler",
    "repelones": "repelón",
    "repelos": "repelo",
    "repelosa": "repeloso",
    "repelosas": "repeloso",
    "repelosos": "repeloso",
    "repeluses": "repelús",
    "repeluznos": "repeluzno",
    "repelábamos": "repelar",
    "repeláis": "repeler",
    "repeláramos": "repelar",
    "repeláremos": "repelar",
    "repelásemos": "repelar",
    "repelé": "repelar",
    "repeléis": "repeler",
    "repelí": "repeler",
    "repelía": "repeler",
    "repelíais": "repeler",
    "repelíamos": "repeler",
    "repelían": "repeler",
    "repelías": "repeler",
    "repeló": "repelar",
    "repensaba": "repensar",
    "repensabais": "repensar",
    "repensaban": "repensar",
    "repensabas": "repensar",
    "repensad": "repensar",
    "repensada": "repensar",
    "repensadas": "repensar",
    "repensado": "repensar",
    "repensados": "repensar",
    "repensamos": "repensar",
    "repensando": "repensar",
    "repensara": "repensar",
    "repensarais": "repensar",
    "repensaran": "repensar",
    "repensaras": "repensar",
    "repensare": "repensar",
    "repensareis": "repensar",
    "repensaremos": "repensar",
    "repensaren": "repensar",
    "repensares": "repensar",
    "repensaron": "repensar",
    "repensará": "repensar",
    "repensarán": "repensar",
    "repensarás": "repensar",
    "repensaré": "repensar",
    "repensaréis": "repensar",
    "repensaría": "repensar",
    "repensaríais": "repensar",
    "repensaríamos": "repensar",
    "repensarían": "repensar",
    "repensarías": "repensar",
    "repensase": "repensar",
    "repensaseis": "repensar",
    "repensasen": "repensar",
    "repensases": "repensar",
    "repensaste": "repensar",
    "repensasteis": "repensar",
    "repensemos": "repensar",
    "repensábamos": "repensar",
    "repensáis": "repensar",
    "repensáramos": "repensar",
    "repensáremos": "repensar",
    "repensásemos": "repensar",
    "repensé": "repensar",
    "repenséis": "repensar",
    "repensó": "repensar",
    "repentes": "repente",
    "repentice": "repentizar",
    "repenticemos": "repentizar",
    "repenticen": "repentizar",
    "repentices": "repentizar",
    "repenticé": "repentizar",
    "repenticéis": "repentizar",
    "repentina": "repentino",
    "repentinas": "repentino",
    "repentinos": "repentino",
    "repentistas": "repentista",
    "repentiza": "repentizar",
    "repentizaba": "repentizar",
    "repentizabais": "repentizar",
    "repentizaban": "repentizar",
    "repentizabas": "repentizar",
    "repentizaciones": "repentización",
    "repentizad": "repentizar",
    "repentizada": "repentizar",
    "repentizadas": "repentizar",
    "repentizado": "repentizar",
    "repentizados": "repentizar",
    "repentizamos": "repentizar",
    "repentizan": "repentizar",
    "repentizando": "repentizar",
    "repentizara": "repentizar",
    "repentizarais": "repentizar",
    "repentizaran": "repentizar",
    "repentizaras": "repentizar",
    "repentizare": "repentizar",
    "repentizareis": "repentizar",
    "repentizaremos": "repentizar",
    "repentizaren": "repentizar",
    "repentizares": "repentizar",
    "repentizaron": "repentizar",
    "repentizará": "repentizar",
    "repentizarán": "repentizar",
    "repentizarás": "repentizar",
    "repentizaré": "repentizar",
    "repentizaréis": "repentizar",
    "repentizaría": "repentizar",
    "repentizaríais": "repentizar",
    "repentizaríamos": "repentizar",
    "repentizarían": "repentizar",
    "repentizarías": "repentizar",
    "repentizas": "repentizar",
    "repentizase": "repentizar",
    "repentizaseis": "repentizar",
    "repentizasen": "repentizar",
    "repentizases": "repentizar",
    "repentizaste": "repentizar",
    "repentizasteis": "repentizar",
    "repentizo": "repentizar",
    "repentizábamos": "repentizar",
    "repentizáis": "repentizar",
    "repentizáramos": "repentizar",
    "repentizáremos": "repentizar",
    "repentizásemos": "repentizar",
    "repentizó": "repentizar",
    "repeores": "repeor",
    "repercudidas": "repercudida",
    "repercusiones": "repercusión",
    "repercusiva": "repercusivo",
    "repercusivas": "repercusivo",
    "repercusivos": "repercusivo",
    "repercuta": "repercutir",
    "repercutamos": "repercutir",
    "repercutan": "repercutir",
    "repercutas": "repercutir",
    "repercute": "repercutir",
    "repercuten": "repercutir",
    "repercutes": "repercutir",
    "repercutibles": "repercutible",
    "repercutid": "repercutir",
    "repercutida": "repercutir",
    "repercutidas": "repercutir",
    "repercutido": "repercutir",
    "repercutidos": "repercutir",
    "repercutiendo": "repercutir",
    "repercutiera": "repercutir",
    "repercutierais": "repercutir",
    "repercutieran": "repercutir",
    "repercutieras": "repercutir",
    "repercutiere": "repercutir",
    "repercutiereis": "repercutir",
    "repercutieren": "repercutir",
    "repercutieres": "repercutir",
    "repercutieron": "repercutir",
    "repercutiese": "repercutir",
    "repercutieseis": "repercutir",
    "repercutiesen": "repercutir",
    "repercutieses": "repercutir",
    "repercutimos": "repercutir",
    "repercutiremos": "repercutir",
    "repercutirá": "repercutir",
    "repercutirán": "repercutir",
    "repercutirás": "repercutir",
    "repercutiré": "repercutir",
    "repercutiréis": "repercutir",
    "repercutiría": "repercutir",
    "repercutiríais": "repercutir",
    "repercutiríamos": "repercutir",
    "repercutirían": "repercutir",
    "repercutirías": "repercutir",
    "repercutiste": "repercutir",
    "repercutisteis": "repercutir",
    "repercutiéramos": "repercutir",
    "repercutiéremos": "repercutir",
    "repercutiésemos": "repercutir",
    "repercutió": "repercutir",
    "repercuto": "repercutir",
    "repercutáis": "repercutir",
    "repercutí": "repercutir",
    "repercutía": "repercutir",
    "repercutíais": "repercutir",
    "repercutíamos": "repercutir",
    "repercutían": "repercutir",
    "repercutías": "repercutir",
    "repercutís": "repercutir",
    "reperforaciones": "reperforación",
    "repertorios": "repertorio",
    "repertoristas": "repertorista",
    "repertorizaciones": "repertorización",
    "repesa": "repesar",
    "repesaba": "repesar",
    "repesabais": "repesar",
    "repesaban": "repesar",
    "repesabas": "repesar",
    "repesad": "repesar",
    "repesada": "repesar",
    "repesadas": "repesar",
    "repesado": "repesar",
    "repesados": "repesar",
    "repesamos": "repesar",
    "repesan": "repesar",
    "repesando": "repesar",
    "repesara": "repesar",
    "repesarais": "repesar",
    "repesaran": "repesar",
    "repesaras": "repesar",
    "repesare": "repesar",
    "repesareis": "repesar",
    "repesaremos": "repesar",
    "repesaren": "repesar",
    "repesares": "repesar",
    "repesaron": "repesar",
    "repesará": "repesar",
    "repesarán": "repesar",
    "repesarás": "repesar",
    "repesaré": "repesar",
    "repesaréis": "repesar",
    "repesaría": "repesar",
    "repesaríais": "repesar",
    "repesaríamos": "repesar",
    "repesarían": "repesar",
    "repesarías": "repesar",
    "repesas": "repesar",
    "repesase": "repesar",
    "repesaseis": "repesar",
    "repesasen": "repesar",
    "repesases": "repesar",
    "repesaste": "repesar",
    "repesasteis": "repesar",
    "repesca": "repescar",
    "repescaba": "repescar",
    "repescabais": "repescar",
    "repescaban": "repescar",
    "repescabas": "repescar",
    "repescad": "repescar",
    "repescada": "repescar",
    "repescadas": "repescar",
    "repescado": "repescar",
    "repescados": "repescar",
    "repescamos": "repescar",
    "repescan": "repescar",
    "repescando": "repescar",
    "repescara": "repescar",
    "repescarais": "repescar",
    "repescaran": "repescar",
    "repescaras": "repescar",
    "repescare": "repescar",
    "repescareis": "repescar",
    "repescaremos": "repescar",
    "repescaren": "repescar",
    "repescares": "repescar",
    "repescaron": "repescar",
    "repescará": "repescar",
    "repescarán": "repescar",
    "repescarás": "repescar",
    "repescaré": "repescar",
    "repescaréis": "repescar",
    "repescaría": "repescar",
    "repescaríais": "repescar",
    "repescaríamos": "repescar",
    "repescarían": "repescar",
    "repescarías": "repescar",
    "repescas": "repescar",
    "repescase": "repescar",
    "repescaseis": "repescar",
    "repescasen": "repescar",
    "repescases": "repescar",
    "repescaste": "repescar",
    "repescasteis": "repescar",
    "repesco": "repescar",
    "repescábamos": "repescar",
    "repescáis": "repescar",
    "repescáramos": "repescar",
    "repescáremos": "repescar",
    "repescásemos": "repescar",
    "repescó": "repescar",
    "repese": "repesar",
    "repesemos": "repesar",
    "repesen": "repesar",
    "repeses": "repesar",
    "repeso": "repesar",
    "repesos": "repeso",
    "repesque": "repescar",
    "repesquemos": "repescar",
    "repesquen": "repescar",
    "repesques": "repescar",
    "repesqué": "repescar",
    "repesquéis": "repescar",
    "repesábamos": "repesar",
    "repesáis": "repesar",
    "repesáramos": "repesar",
    "repesáremos": "repesar",
    "repesásemos": "repesar",
    "repesé": "repesar",
    "repeséis": "repesar",
    "repesó": "repesar",
    "repetibilidades": "repetibilidad",
    "repetibles": "repetible",
    "repeticiones": "repetición",
    "repetid": "repetir",
    "repetida": "repetir",
    "repetidas": "repetir",
    "repetido": "repetir",
    "repetidora": "repetidor",
    "repetidoras": "repetidor",
    "repetidores": "repetidor",
    "repetidos": "repetir",
    "repetimos": "repetir",
    "repetiremos": "repetir",
    "repetirá": "repetir",
    "repetirán": "repetir",
    "repetirás": "repetir",
    "repetiré": "repetir",
    "repetiréis": "repetir",
    "repetiría": "repetir",
    "repetiríais": "repetir",
    "repetiríamos": "repetir",
    "repetirían": "repetir",
    "repetirías": "repetir",
    "repetiste": "repetir",
    "repetisteis": "repetir",
    "repetitiva": "repetitivo",
    "repetitivas": "repetitivo",
    "repetitividades": "repetitividad",
    "repetitivos": "repetitivo",
    "repetí": "repetir",
    "repetía": "repetir",
    "repetíais": "repetir",
    "repetíamos": "repetir",
    "repetían": "repetir",
    "repetías": "repetir",
    "repetís": "repetir",
    "repica": "repicar",
    "repicaba": "repicar",
    "repicabais": "repicar",
    "repicaban": "repicar",
    "repicabas": "repicar",
    "repicad": "repicar",
    "repicada": "repicar",
    "repicadas": "repicar",
    "repicado": "repicar",
    "repicados": "repicar",
    "repicamos": "repicar",
    "repican": "repicar",
    "repicando": "repicar",
    "repicara": "repicar",
    "repicarais": "repicar",
    "repicaran": "repicar",
    "repicaras": "repicar",
    "repicare": "repicar",
    "repicareis": "repicar",
    "repicaremos": "repicar",
    "repicaren": "repicar",
    "repicares": "repicar",
    "repicaron": "repicar",
    "repicará": "repicar",
    "repicarán": "repicar",
    "repicarás": "repicar",
    "repicaré": "repicar",
    "repicaréis": "repicar",
    "repicaría": "repicar",
    "repicaríais": "repicar",
    "repicaríamos": "repicar",
    "repicarían": "repicar",
    "repicarías": "repicar",
    "repicas": "repicar",
    "repicase": "repicar",
    "repicaseis": "repicar",
    "repicasen": "repicar",
    "repicases": "repicar",
    "repicaste": "repicar",
    "repicasteis": "repicar",
    "repico": "repicar",
    "repicoteada": "repicoteado",
    "repicoteadas": "repicoteado",
    "repicoteados": "repicoteado",
    "repicábamos": "repicar",
    "repicáis": "repicar",
    "repicáramos": "repicar",
    "repicáremos": "repicar",
    "repicásemos": "repicar",
    "repicó": "repicar",
    "repiensa": "repensar",
    "repiensan": "repensar",
    "repiensas": "repensar",
    "repiense": "repensar",
    "repiensen": "repensar",
    "repienses": "repensar",
    "repienso": "repensar",
    "repinta": "repintar",
    "repintaba": "repintar",
    "repintabais": "repintar",
    "repintaban": "repintar",
    "repintabas": "repintar",
    "repintad": "repintar",
    "repintada": "repintar",
    "repintadas": "repintar",
    "repintado": "repintar",
    "repintados": "repintar",
    "repintamos": "repintar",
    "repintan": "repintar",
    "repintando": "repintar",
    "repintara": "repintar",
    "repintarais": "repintar",
    "repintaran": "repintar",
    "repintaras": "repintar",
    "repintare": "repintar",
    "repintareis": "repintar",
    "repintaremos": "repintar",
    "repintaren": "repintar",
    "repintares": "repintar",
    "repintaron": "repintar",
    "repintará": "repintar",
    "repintarán": "repintar",
    "repintarás": "repintar",
    "repintaré": "repintar",
    "repintaréis": "repintar",
    "repintaría": "repintar",
    "repintaríais": "repintar",
    "repintaríamos": "repintar",
    "repintarían": "repintar",
    "repintarías": "repintar",
    "repintas": "repintar",
    "repintase": "repintar",
    "repintaseis": "repintar",
    "repintasen": "repintar",
    "repintases": "repintar",
    "repintaste": "repintar",
    "repintasteis": "repintar",
    "repinte": "repintar",
    "repintemos": "repintar",
    "repinten": "repintar",
    "repintes": "repinte",
    "repinto": "repintar",
    "repintábamos": "repintar",
    "repintáis": "repintar",
    "repintáramos": "repintar",
    "repintáremos": "repintar",
    "repintásemos": "repintar",
    "repinté": "repintar",
    "repintéis": "repintar",
    "repintó": "repintar",
    "repipis": "repipi",
    "repique": "repicar",
    "repiquemos": "repicar",
    "repiquen": "repicar",
    "repiques": "repique",
    "repiquetea": "repiquetear",
    "repiqueteaba": "repiquetear",
    "repiqueteabais": "repiquetear",
    "repiqueteaban": "repiquetear",
    "repiqueteabas": "repiquetear",
    "repiquetead": "repiquetear",
    "repiqueteada": "repiquetear",
    "repiqueteadas": "repiquetear",
    "repiqueteado": "repiquetear",
    "repiqueteados": "repiquetear",
    "repiqueteamos": "repiquetear",
    "repiquetean": "repiquetear",
    "repiqueteando": "repiquetear",
    "repiqueteara": "repiquetear",
    "repiquetearais": "repiquetear",
    "repiquetearan": "repiquetear",
    "repiquetearas": "repiquetear",
    "repiqueteare": "repiquetear",
    "repiqueteareis": "repiquetear",
    "repiquetearemos": "repiquetear",
    "repiquetearen": "repiquetear",
    "repiqueteares": "repiquetear",
    "repiquetearon": "repiquetear",
    "repiqueteará": "repiquetear",
    "repiquetearán": "repiquetear",
    "repiquetearás": "repiquetear",
    "repiquetearé": "repiquetear",
    "repiquetearéis": "repiquetear",
    "repiquetearía": "repiquetear",
    "repiquetearíais": "repiquetear",
    "repiquetearíamos": "repiquetear",
    "repiquetearían": "repiquetear",
    "repiquetearías": "repiquetear",
    "repiqueteas": "repiquetear",
    "repiquetease": "repiquetear",
    "repiqueteaseis": "repiquetear",
    "repiqueteasen": "repiquetear",
    "repiqueteases": "repiquetear",
    "repiqueteaste": "repiquetear",
    "repiqueteasteis": "repiquetear",
    "repiquetee": "repiquetear",
    "repiqueteemos": "repiquetear",
    "repiqueteen": "repiquetear",
    "repiquetees": "repiquetear",
    "repiqueteo": "repiquetear",
    "repiqueteos": "repiqueteo",
    "repiquetes": "repiquete",
    "repiqueteábamos": "repiquetear",
    "repiqueteáis": "repiquetear",
    "repiqueteáramos": "repiquetear",
    "repiqueteáremos": "repiquetear",
    "repiqueteásemos": "repiquetear",
    "repiqueteé": "repiquetear",
    "repiqueteéis": "repiquetear",
    "repiqueteó": "repiquetear",
    "repiqué": "repicar",
    "repiquéis": "repicar",
    "repisas": "repisa",
    "repisos": "repiso",
    "repita": "repetir",
    "repitamos": "repetir",
    "repitan": "repetir",
    "repitas": "repetir",
    "repite": "repetir",
    "repiten": "repetir",
    "repites": "repetir",
    "repitiendo": "repetir",
    "repitiera": "repetir",
    "repitierais": "repetir",
    "repitieran": "repetir",
    "repitieras": "repetir",
    "repitiere": "repetir",
    "repitiereis": "repetir",
    "repitieren": "repetir",
    "repitieres": "repetir",
    "repitieron": "repetir",
    "repitiese": "repetir",
    "repitieseis": "repetir",
    "repitiesen": "repetir",
    "repitieses": "repetir",
    "repitiéramos": "repetir",
    "repitiéremos": "repetir",
    "repitiésemos": "repetir",
    "repitió": "repetir",
    "repito": "repetir",
    "repitáis": "repetir",
    "repizcos": "repizco",
    "replanificaciones": "replanificación",
    "replanta": "replantar",
    "replantaba": "replantar",
    "replantabais": "replantar",
    "replantaban": "replantar",
    "replantabas": "replantar",
    "replantaciones": "replantación",
    "replantad": "replantar",
    "replantada": "replantar",
    "replantadas": "replantar",
    "replantado": "replantar",
    "replantados": "replantar",
    "replantamos": "replantar",
    "replantan": "replantar",
    "replantando": "replantar",
    "replantara": "replantar",
    "replantarais": "replantar",
    "replantaran": "replantar",
    "replantaras": "replantar",
    "replantare": "replantar",
    "replantareis": "replantar",
    "replantaremos": "replantar",
    "replantaren": "replantar",
    "replantares": "replantar",
    "replantaron": "replantar",
    "replantará": "replantar",
    "replantarán": "replantar",
    "replantarás": "replantar",
    "replantaré": "replantar",
    "replantaréis": "replantar",
    "replantaría": "replantar",
    "replantaríais": "replantar",
    "replantaríamos": "replantar",
    "replantarían": "replantar",
    "replantarías": "replantar",
    "replantas": "replantar",
    "replantase": "replantar",
    "replantaseis": "replantar",
    "replantasen": "replantar",
    "replantases": "replantar",
    "replantaste": "replantar",
    "replantasteis": "replantar",
    "replante": "replantar",
    "replantea": "replantear",
    "replanteaba": "replantear",
    "replanteabais": "replantear",
    "replanteaban": "replantear",
    "replanteabas": "replantear",
    "replantead": "replantear",
    "replanteada": "replantear",
    "replanteadas": "replantear",
    "replanteado": "replantear",
    "replanteados": "replantear",
    "replanteamientos": "replanteamiento",
    "replanteamos": "replantear",
    "replantean": "replantear",
    "replanteando": "replantear",
    "replanteara": "replantear",
    "replantearais": "replantear",
    "replantearan": "replantear",
    "replantearas": "replantear",
    "replanteare": "replantear",
    "replanteareis": "replantear",
    "replantearemos": "replantear",
    "replantearen": "replantear",
    "replanteares": "replantear",
    "replantearon": "replantear",
    "replanteará": "replantear",
    "replantearán": "replantear",
    "replantearás": "replantear",
    "replantearé": "replantear",
    "replantearéis": "replantear",
    "replantearía": "replantear",
    "replantearíais": "replantear",
    "replantearíamos": "replantear",
    "replantearían": "replantear",
    "replantearías": "replantear",
    "replanteas": "replantear",
    "replantease": "replantear",
    "replanteaseis": "replantear",
    "replanteasen": "replantear",
    "replanteases": "replantear",
    "replanteaste": "replantear",
    "replanteasteis": "replantear",
    "replantee": "replantear",
    "replanteemos": "replantear",
    "replanteen": "replantear",
    "replantees": "replantear",
    "replantemos": "replantar",
    "replanten": "replantar",
    "replanteo": "replantear",
    "replanteos": "replanteo",
    "replantes": "replantar",
    "replanteábamos": "replantear",
    "replanteáis": "replantear",
    "replanteáramos": "replantear",
    "replanteáremos": "replantear",
    "replanteásemos": "replantear",
    "replanteé": "replantear",
    "replanteéis": "replantear",
    "replanteó": "replantear",
    "replanto": "replantar",
    "replantábamos": "replantar",
    "replantáis": "replantar",
    "replantáramos": "replantar",
    "replantáremos": "replantar",
    "replantásemos": "replantar",
    "replanté": "replantar",
    "replantéis": "replantar",
    "replantó": "replantar",
    "repleciones": "repleción",
    "replegaba": "replegar",
    "replegabais": "replegar",
    "replegaban": "replegar",
    "replegabas": "replegar",
    "replegad": "replegar",
    "replegada": "replegar",
    "replegadas": "replegar",
    "replegado": "replegar",
    "replegados": "replegar",
    "replegamos": "replegar",
    "replegando": "replegar",
    "replegara": "replegar",
    "replegarais": "replegar",
    "replegaran": "replegar",
    "replegaras": "replegar",
    "replegare": "replegar",
    "replegareis": "replegar",
    "replegaremos": "replegar",
    "replegaren": "replegar",
    "replegares": "replegar",
    "replegaron": "replegar",
    "replegará": "replegar",
    "replegarán": "replegar",
    "replegarás": "replegar",
    "replegaré": "replegar",
    "replegaréis": "replegar",
    "replegaría": "replegar",
    "replegaríais": "replegar",
    "replegaríamos": "replegar",
    "replegarían": "replegar",
    "replegarías": "replegar",
    "replegase": "replegar",
    "replegaseis": "replegar",
    "replegasen": "replegar",
    "replegases": "replegar",
    "replegaste": "replegar",
    "replegasteis": "replegar",
    "repleguemos": "replegar",
    "replegué": "replegar",
    "repleguéis": "replegar",
    "replegábamos": "replegar",
    "replegáis": "replegar",
    "replegáramos": "replegar",
    "replegáremos": "replegar",
    "replegásemos": "replegar",
    "replegó": "replegar",
    "repleta": "repleto",
    "repletas": "repleto",
    "repletos": "repleto",
    "replica": "replicar",
    "replicaba": "replicar",
    "replicabais": "replicar",
    "replicaban": "replicar",
    "replicabas": "replicar",
    "replicaciones": "replicación",
    "replicad": "replicar",
    "replicada": "replicar",
    "replicadas": "replicar",
    "replicado": "replicar",
    "replicadora": "replicador",
    "replicadoras": "replicador",
    "replicadores": "replicador",
    "replicados": "replicar",
    "replicamos": "replicar",
    "replican": "replicar",
    "replicando": "replicar",
    "replicantes": "replicante",
    "replicara": "replicar",
    "replicarais": "replicar",
    "replicaran": "replicar",
    "replicaras": "replicar",
    "replicare": "replicar",
    "replicareis": "replicar",
    "replicaremos": "replicar",
    "replicaren": "replicar",
    "replicares": "replicar",
    "replicaron": "replicar",
    "replicará": "replicar",
    "replicarán": "replicar",
    "replicarás": "replicar",
    "replicaré": "replicar",
    "replicaréis": "replicar",
    "replicaría": "replicar",
    "replicaríais": "replicar",
    "replicaríamos": "replicar",
    "replicarían": "replicar",
    "replicarías": "replicar",
    "replicas": "replicar",
    "replicase": "replicar",
    "replicaseis": "replicar",
    "replicasen": "replicar",
    "replicases": "replicar",
    "replicaste": "replicar",
    "replicasteis": "replicar",
    "replicatos": "replicato",
    "replico": "replicar",
    "replicona": "replicón",
    "repliconas": "replicón",
    "replicones": "replicón",
    "replicábamos": "replicar",
    "replicáis": "replicar",
    "replicáramos": "replicar",
    "replicáremos": "replicar",
    "replicásemos": "replicar",
    "replicó": "replicar",
    "repliega": "replegar",
    "repliegan": "replegar",
    "repliegas": "replegar",
    "repliego": "replegar",
    "repliegue": "replegar",
    "replieguen": "replegar",
    "repliegues": "repliegue",
    "replique": "replicar",
    "repliquemos": "replicar",
    "repliquen": "replicar",
    "repliques": "replicar",
    "repliqué": "replicar",
    "repliquéis": "replicar",
    "repoblaba": "repoblar",
    "repoblabais": "repoblar",
    "repoblaban": "repoblar",
    "repoblabas": "repoblar",
    "repoblaciones": "repoblación",
    "repoblad": "repoblar",
    "repoblada": "repoblar",
    "repobladas": "repoblar",
    "repoblado": "repoblar",
    "repoblados": "repoblar",
    "repoblamos": "repoblar",
    "repoblando": "repoblar",
    "repoblara": "repoblar",
    "repoblarais": "repoblar",
    "repoblaran": "repoblar",
    "repoblaras": "repoblar",
    "repoblare": "repoblar",
    "repoblareis": "repoblar",
    "repoblaremos": "repoblar",
    "repoblaren": "repoblar",
    "repoblares": "repoblar",
    "repoblaron": "repoblar",
    "repoblará": "repoblar",
    "repoblarán": "repoblar",
    "repoblarás": "repoblar",
    "repoblaré": "repoblar",
    "repoblaréis": "repoblar",
    "repoblaría": "repoblar",
    "repoblaríais": "repoblar",
    "repoblaríamos": "repoblar",
    "repoblarían": "repoblar",
    "repoblarías": "repoblar",
    "repoblase": "repoblar",
    "repoblaseis": "repoblar",
    "repoblasen": "repoblar",
    "repoblases": "repoblar",
    "repoblaste": "repoblar",
    "repoblasteis": "repoblar",
    "repoblemos": "repoblar",
    "repoblábamos": "repoblar",
    "repobláis": "repoblar",
    "repobláramos": "repoblar",
    "repobláremos": "repoblar",
    "repoblásemos": "repoblar",
    "repoblé": "repoblar",
    "repobléis": "repoblar",
    "repobló": "repoblar",
    "repoda": "repodar",
    "repodaba": "repodar",
    "repodabais": "repodar",
    "repodaban": "repodar",
    "repodabas": "repodar",
    "repodad": "repodar",
    "repodada": "repodar",
    "repodadas": "repodar",
    "repodado": "repodar",
    "repodados": "repodar",
    "repodamos": "repodar",
    "repodan": "repodar",
    "repodando": "repodar",
    "repodara": "repodar",
    "repodarais": "repodar",
    "repodaran": "repodar",
    "repodaras": "repodar",
    "repodare": "repodar",
    "repodareis": "repodar",
    "repodaremos": "repodar",
    "repodaren": "repodar",
    "repodares": "repodar",
    "repodaron": "repodar",
    "repodará": "repodar",
    "repodarán": "repodar",
    "repodarás": "repodar",
    "repodaré": "repodar",
    "repodaréis": "repodar",
    "repodaría": "repodar",
    "repodaríais": "repodar",
    "repodaríamos": "repodar",
    "repodarían": "repodar",
    "repodarías": "repodar",
    "repodas": "repodar",
    "repodase": "repodar",
    "repodaseis": "repodar",
    "repodasen": "repodar",
    "repodases": "repodar",
    "repodaste": "repodar",
    "repodasteis": "repodar",
    "repode": "repodar",
    "repodemos": "repodar",
    "repoden": "repodar",
    "repodes": "repodar",
    "repodo": "repodar",
    "repodrida": "repudrir",
    "repodridas": "repudrir",
    "repodrido": "repudrir",
    "repodridos": "repudrir",
    "repodriré": "repodrir",
    "repodriría": "repodrir",
    "repodrí": "repodrir",
    "repodábamos": "repodar",
    "repodáis": "repodar",
    "repodáramos": "repodar",
    "repodáremos": "repodar",
    "repodásemos": "repodar",
    "repodé": "repodar",
    "repodéis": "repodar",
    "repodó": "repodar",
    "repolla": "repollar",
    "repollaba": "repollar",
    "repollabais": "repollar",
    "repollaban": "repollar",
    "repollabas": "repollar",
    "repollad": "repollar",
    "repollada": "repollar",
    "repolladas": "repollar",
    "repollado": "repollar",
    "repollados": "repollar",
    "repollamos": "repollar",
    "repollan": "repollar",
    "repollando": "repollar",
    "repollara": "repollar",
    "repollarais": "repollar",
    "repollaran": "repollar",
    "repollaras": "repollar",
    "repollare": "repollar",
    "repollareis": "repollar",
    "repollaremos": "repollar",
    "repollaren": "repollar",
    "repollares": "repollar",
    "repollaron": "repollar",
    "repollará": "repollar",
    "repollarán": "repollar",
    "repollarás": "repollar",
    "repollaré": "repollar",
    "repollaréis": "repollar",
    "repollaría": "repollar",
    "repollaríais": "repollar",
    "repollaríamos": "repollar",
    "repollarían": "repollar",
    "repollarías": "repollar",
    "repollas": "repollar",
    "repollase": "repollar",
    "repollaseis": "repollar",
    "repollasen": "repollar",
    "repollases": "repollar",
    "repollaste": "repollar",
    "repollasteis": "repollar",
    "repolle": "repollar",
    "repollemos": "repollar",
    "repollen": "repollar",
    "repolles": "repollar",
    "repollo": "repollar",
    "repollos": "repollo",
    "repolluda": "repolludo",
    "repolludas": "repolludo",
    "repolludos": "repolludo",
    "repollábamos": "repollar",
    "repolláis": "repollar",
    "repolláramos": "repollar",
    "repolláremos": "repollar",
    "repollásemos": "repollar",
    "repollé": "repollar",
    "repolléis": "repollar",
    "repolló": "repollar",
    "repondremos": "reponer",
    "repondrá": "reponer",
    "repondrán": "reponer",
    "repondrás": "reponer",
    "repondré": "reponer",
    "repondréis": "reponer",
    "repondría": "reponer",
    "repondríais": "reponer",
    "repondríamos": "reponer",
    "repondrían": "reponer",
    "repondrías": "reponer",
    "repone": "reponer",
    "reponed": "reponer",
    "reponemos": "reponer",
    "reponen": "reponer",
    "repones": "reponer",
    "reponga": "reponer",
    "repongamos": "reponer",
    "repongan": "reponer",
    "repongas": "reponer",
    "repongo": "reponer",
    "repongáis": "reponer",
    "reponiendo": "reponer",
    "reponéis": "reponer",
    "reponía": "reponer",
    "reponíais": "reponer",
    "reponíamos": "reponer",
    "reponían": "reponer",
    "reponías": "reponer",
    "reporta": "reportar",
    "reportaba": "reportar",
    "reportabais": "reportar",
    "reportaban": "reportar",
    "reportabas": "reportar",
    "reportad": "reportar",
    "reportada": "reportar",
    "reportadas": "reportar",
    "reportado": "reportar",
    "reportados": "reportar",
    "reportajes": "reportaje",
    "reportamientos": "reportamiento",
    "reportamos": "reportar",
    "reportan": "reportar",
    "reportando": "reportar",
    "reportara": "reportar",
    "reportarais": "reportar",
    "reportaran": "reportar",
    "reportaras": "reportar",
    "reportare": "reportar",
    "reportareis": "reportar",
    "reportaremos": "reportar",
    "reportaren": "reportar",
    "reportares": "reportar",
    "reportaron": "reportar",
    "reportará": "reportar",
    "reportarán": "reportar",
    "reportarás": "reportar",
    "reportaré": "reportar",
    "reportaréis": "reportar",
    "reportaría": "reportar",
    "reportaríais": "reportar",
    "reportaríamos": "reportar",
    "reportarían": "reportar",
    "reportarías": "reportar",
    "reportas": "reportar",
    "reportase": "reportar",
    "reportaseis": "reportar",
    "reportasen": "reportar",
    "reportases": "reportar",
    "reportaste": "reportar",
    "reportasteis": "reportar",
    "reporte": "reportar",
    "reportemos": "reportar",
    "reporten": "reportar",
    "reportera": "reportero",
    "reporteras": "reportero",
    "reporteriles": "reporteril",
    "reporterismos": "reporterismo",
    "reporteros": "reportero",
    "reporters": "repórter",
    "reportes": "reporte",
    "reportistas": "reportista",
    "reporto": "reportar",
    "reportábamos": "reportar",
    "reportáis": "reportar",
    "reportáramos": "reportar",
    "reportáremos": "reportar",
    "reportásemos": "reportar",
    "reporté": "reportar",
    "reportéis": "reportar",
    "reportó": "reportar",
    "reposa": "reposar",
    "reposaba": "reposar",
    "reposabais": "reposar",
    "reposaban": "reposar",
    "reposabas": "reposar",
    "reposad": "reposar",
    "reposada": "reposar",
    "reposadas": "reposar",
    "reposaderos": "reposadero",
    "reposado": "reposar",
    "reposados": "reposar",
    "reposamos": "reposar",
    "reposan": "reposar",
    "reposando": "reposar",
    "reposara": "reposar",
    "reposarais": "reposar",
    "reposaran": "reposar",
    "reposaras": "reposar",
    "reposare": "reposar",
    "reposareis": "reposar",
    "reposaremos": "reposar",
    "reposaren": "reposar",
    "reposares": "reposar",
    "reposaron": "reposar",
    "reposará": "reposar",
    "reposarán": "reposar",
    "reposarás": "reposar",
    "reposaré": "reposar",
    "reposaréis": "reposar",
    "reposaría": "reposar",
    "reposaríais": "reposar",
    "reposaríamos": "reposar",
    "reposarían": "reposar",
    "reposarías": "reposar",
    "reposas": "reposar",
    "reposase": "reposar",
    "reposaseis": "reposar",
    "reposasen": "reposar",
    "reposases": "reposar",
    "reposaste": "reposar",
    "reposasteis": "reposar",
    "repose": "reposar",
    "reposemos": "reposar",
    "reposen": "reposar",
    "reposes": "reposar",
    "reposicionamientos": "reposicionamiento",
    "reposiciones": "reposición",
    "repositorios": "repositorio",
    "reposo": "reposar",
    "reposos": "reposo",
    "reposta": "repostar",
    "repostaba": "repostar",
    "repostabais": "repostar",
    "repostaban": "repostar",
    "repostabas": "repostar",
    "repostad": "repostar",
    "repostada": "repostar",
    "repostadas": "repostar",
    "repostado": "repostar",
    "repostados": "repostar",
    "repostamos": "repostar",
    "repostan": "repostar",
    "repostando": "repostar",
    "repostara": "repostar",
    "repostarais": "repostar",
    "repostaran": "repostar",
    "repostaras": "repostar",
    "repostare": "repostar",
    "repostareis": "repostar",
    "repostaremos": "repostar",
    "repostaren": "repostar",
    "repostares": "repostar",
    "repostaron": "repostar",
    "repostará": "repostar",
    "repostarán": "repostar",
    "repostarás": "repostar",
    "repostaré": "repostar",
    "repostaréis": "repostar",
    "repostaría": "repostar",
    "repostaríais": "repostar",
    "repostaríamos": "repostar",
    "repostarían": "repostar",
    "repostarías": "repostar",
    "repostas": "repostar",
    "repostase": "repostar",
    "repostaseis": "repostar",
    "repostasen": "repostar",
    "repostases": "repostar",
    "repostaste": "repostar",
    "repostasteis": "repostar",
    "reposte": "repostar",
    "repostemos": "repostar",
    "reposten": "repostar",
    "repostera": "repostero",
    "reposteras": "repostero",
    "reposteros": "repostero",
    "reposterías": "repostería",
    "repostes": "repostar",
    "reposto": "repostar",
    "repostábamos": "repostar",
    "repostáis": "repostar",
    "repostáramos": "repostar",
    "repostáremos": "repostar",
    "repostásemos": "repostar",
    "reposté": "repostar",
    "repostéis": "repostar",
    "repostó": "repostar",
    "reposábamos": "reposar",
    "reposáis": "reposar",
    "reposáramos": "reposar",
    "reposáremos": "reposar",
    "reposásemos": "reposar",
    "reposé": "reposar",
    "reposéis": "reposar",
    "reposó": "reposar",
    "reprecipitaciones": "reprecipitación",
    "repreguntas": "repregunta",
    "reprehensibles": "reprehensible",
    "reprehensiones": "reprehensión",
    "reprenda": "reprender",
    "reprendamos": "reprender",
    "reprendan": "reprender",
    "reprendas": "reprender",
    "reprende": "reprender",
    "reprended": "reprender",
    "reprendemos": "reprender",
    "reprenden": "reprender",
    "reprenderemos": "reprender",
    "reprenderá": "reprender",
    "reprenderán": "reprender",
    "reprenderás": "reprender",
    "reprenderé": "reprender",
    "reprenderéis": "reprender",
    "reprendería": "reprender",
    "reprenderíais": "reprender",
    "reprenderíamos": "reprender",
    "reprenderían": "reprender",
    "reprenderías": "reprender",
    "reprendes": "reprender",
    "reprendida": "reprender",
    "reprendidas": "reprender",
    "reprendido": "reprender",
    "reprendidos": "reprender",
    "reprendiendo": "reprender",
    "reprendiera": "reprender",
    "reprendierais": "reprender",
    "reprendieran": "reprender",
    "reprendieras": "reprender",
    "reprendiere": "reprender",
    "reprendiereis": "reprender",
    "reprendieren": "reprender",
    "reprendieres": "reprender",
    "reprendieron": "reprender",
    "reprendiese": "reprender",
    "reprendieseis": "reprender",
    "reprendiesen": "reprender",
    "reprendieses": "reprender",
    "reprendimos": "reprender",
    "reprendiste": "reprender",
    "reprendisteis": "reprender",
    "reprendiéramos": "reprender",
    "reprendiéremos": "reprender",
    "reprendiésemos": "reprender",
    "reprendió": "reprender",
    "reprendo": "reprender",
    "reprendáis": "reprender",
    "reprendéis": "reprender",
    "reprendí": "reprender",
    "reprendía": "reprender",
    "reprendíais": "reprender",
    "reprendíamos": "reprender",
    "reprendían": "reprender",
    "reprendías": "reprender",
    "reprensibles": "reprensible",
    "reprensiones": "reprensión",
    "reprensora": "reprensor",
    "reprensoras": "reprensor",
    "reprensores": "reprensor",
    "represa": "represar",
    "represaba": "represar",
    "represabais": "represar",
    "represaban": "represar",
    "represabas": "represar",
    "represad": "represar",
    "represada": "represar",
    "represadas": "represar",
    "represado": "represar",
    "represados": "represar",
    "represaliada": "represaliado",
    "represaliadas": "represaliado",
    "represaliados": "represaliado",
    "represalias": "represalia",
    "represamos": "represar",
    "represan": "represar",
    "represando": "represar",
    "represara": "represar",
    "represarais": "represar",
    "represaran": "represar",
    "represaras": "represar",
    "represare": "represar",
    "represareis": "represar",
    "represaremos": "represar",
    "represaren": "represar",
    "represares": "represar",
    "represaron": "represar",
    "represará": "represar",
    "represarán": "represar",
    "represarás": "represar",
    "represaré": "represar",
    "represaréis": "represar",
    "represaría": "represar",
    "represaríais": "represar",
    "represaríamos": "represar",
    "represarían": "represar",
    "represarías": "represar",
    "represas": "represar",
    "represase": "represar",
    "represaseis": "represar",
    "represasen": "represar",
    "represases": "represar",
    "represaste": "represar",
    "represasteis": "represar",
    "represe": "represar",
    "represemos": "represar",
    "represen": "represar",
    "representa": "representar",
    "representaba": "representar",
    "representabais": "representar",
    "representaban": "representar",
    "representabas": "representar",
    "representables": "representable",
    "representaciones": "representación",
    "representad": "representar",
    "representada": "representar",
    "representadas": "representar",
    "representado": "representar",
    "representadora": "representador",
    "representadoras": "representador",
    "representadores": "representador",
    "representados": "representar",
    "representamos": "representar",
    "representan": "representar",
    "representando": "representar",
    "representantes": "representante",
    "representara": "representar",
    "representarais": "representar",
    "representaran": "representar",
    "representaras": "representar",
    "representare": "representar",
    "representareis": "representar",
    "representaremos": "representar",
    "representaren": "representar",
    "representares": "representar",
    "representaron": "representar",
    "representará": "representar",
    "representarán": "representar",
    "representarás": "representar",
    "representaré": "representar",
    "representaréis": "representar",
    "representaría": "representar",
    "representaríais": "representar",
    "representaríamos": "representar",
    "representarían": "representar",
    "representarías": "representar",
    "representas": "representar",
    "representase": "representar",
    "representaseis": "representar",
    "representasen": "representar",
    "representases": "representar",
    "representaste": "representar",
    "representasteis": "representar",
    "representativa": "representativo",
    "representativas": "representativo",
    "representatividades": "representatividad",
    "representativos": "representativo",
    "represente": "representar",
    "representemos": "representar",
    "representen": "representar",
    "representes": "representar",
    "represento": "representar",
    "representábamos": "representar",
    "representáis": "representar",
    "representáramos": "representar",
    "representáremos": "representar",
    "representásemos": "representar",
    "representé": "representar",
    "representéis": "representar",
    "representó": "representar",
    "represes": "represar",
    "represiones": "represión",
    "represiva": "represivo",
    "represivas": "represivo",
    "represivos": "represivo",
    "represo": "represar",
    "represora": "represor",
    "represoras": "represor",
    "represores": "represor",
    "represábamos": "represar",
    "represáis": "represar",
    "represáramos": "represar",
    "represáremos": "represar",
    "represásemos": "represar",
    "represé": "represar",
    "represéis": "represar",
    "represó": "represar",
    "reprima": "reprimir",
    "reprimamos": "reprimir",
    "repriman": "reprimir",
    "reprimas": "reprimir",
    "reprime": "reprimir",
    "reprimen": "reprimir",
    "reprimendas": "reprimenda",
    "reprimes": "reprimir",
    "reprimid": "reprimir",
    "reprimida": "reprimir",
    "reprimidas": "reprimir",
    "reprimido": "reprimir",
    "reprimidos": "reprimir",
    "reprimiendo": "reprimir",
    "reprimiera": "reprimir",
    "reprimierais": "reprimir",
    "reprimieran": "reprimir",
    "reprimieras": "reprimir",
    "reprimiere": "reprimir",
    "reprimiereis": "reprimir",
    "reprimieren": "reprimir",
    "reprimieres": "reprimir",
    "reprimieron": "reprimir",
    "reprimiese": "reprimir",
    "reprimieseis": "reprimir",
    "reprimiesen": "reprimir",
    "reprimieses": "reprimir",
    "reprimimos": "reprimir",
    "reprimiremos": "reprimir",
    "reprimirá": "reprimir",
    "reprimirán": "reprimir",
    "reprimirás": "reprimir",
    "reprimiré": "reprimir",
    "reprimiréis": "reprimir",
    "reprimiría": "reprimir",
    "reprimiríais": "reprimir",
    "reprimiríamos": "reprimir",
    "reprimirían": "reprimir",
    "reprimirías": "reprimir",
    "reprimiste": "reprimir",
    "reprimisteis": "reprimir",
    "reprimiéramos": "reprimir",
    "reprimiéremos": "reprimir",
    "reprimiésemos": "reprimir",
    "reprimió": "reprimir",
    "reprimo": "reprimir",
    "reprimáis": "reprimir",
    "reprimí": "reprimir",
    "reprimía": "reprimir",
    "reprimíais": "reprimir",
    "reprimíamos": "reprimir",
    "reprimían": "reprimir",
    "reprimías": "reprimir",
    "reprimís": "reprimir",
    "reprise": "reprís",
    "reprises": "reprís",
    "reprivatizaciones": "reprivatización",
    "reprobaba": "reprobar",
    "reprobabais": "reprobar",
    "reprobaban": "reprobar",
    "reprobabas": "reprobar",
    "reprobables": "reprobable",
    "reprobaciones": "reprobación",
    "reprobad": "reprobar",
    "reprobada": "reprobar",
    "reprobadas": "reprobar",
    "reprobado": "reprobar",
    "reprobadora": "reprobador",
    "reprobadoras": "reprobador",
    "reprobadores": "reprobador",
    "reprobados": "reprobar",
    "reprobamos": "reprobar",
    "reprobando": "reprobar",
    "reprobara": "reprobar",
    "reprobarais": "reprobar",
    "reprobaran": "reprobar",
    "reprobaras": "reprobar",
    "reprobare": "reprobar",
    "reprobareis": "reprobar",
    "reprobaremos": "reprobar",
    "reprobaren": "reprobar",
    "reprobares": "reprobar",
    "reprobaron": "reprobar",
    "reprobará": "reprobar",
    "reprobarán": "reprobar",
    "reprobarás": "reprobar",
    "reprobaré": "reprobar",
    "reprobaréis": "reprobar",
    "reprobaría": "reprobar",
    "reprobaríais": "reprobar",
    "reprobaríamos": "reprobar",
    "reprobarían": "reprobar",
    "reprobarías": "reprobar",
    "reprobase": "reprobar",
    "reprobaseis": "reprobar",
    "reprobasen": "reprobar",
    "reprobases": "reprobar",
    "reprobaste": "reprobar",
    "reprobasteis": "reprobar",
    "reprobatoria": "reprobatorio",
    "reprobatorias": "reprobatorio",
    "reprobatorios": "reprobatorio",
    "reprobemos": "reprobar",
    "reprobábamos": "reprobar",
    "reprobáis": "reprobar",
    "reprobáramos": "reprobar",
    "reprobáremos": "reprobar",
    "reprobásemos": "reprobar",
    "reprobé": "reprobar",
    "reprobéis": "reprobar",
    "reprobó": "reprobar",
    "reprocesamientos": "reprocesamiento",
    "reprocesos": "reproceso",
    "reprocha": "reprochar",
    "reprochaba": "reprochar",
    "reprochabais": "reprochar",
    "reprochaban": "reprochar",
    "reprochabas": "reprochar",
    "reprochabilidades": "reprochabilidad",
    "reprochables": "reprochable",
    "reprochad": "reprochar",
    "reprochada": "reprochar",
    "reprochadas": "reprochar",
    "reprochado": "reprochar",
    "reprochadora": "reprochador",
    "reprochadoras": "reprochador",
    "reprochadores": "reprochador",
    "reprochados": "reprochar",
    "reprochamos": "reprochar",
    "reprochan": "reprochar",
    "reprochando": "reprochar",
    "reprochara": "reprochar",
    "reprocharais": "reprochar",
    "reprocharan": "reprochar",
    "reprocharas": "reprochar",
    "reprochare": "reprochar",
    "reprochareis": "reprochar",
    "reprocharemos": "reprochar",
    "reprocharen": "reprochar",
    "reprochares": "reprochar",
    "reprocharon": "reprochar",
    "reprochará": "reprochar",
    "reprocharán": "reprochar",
    "reprocharás": "reprochar",
    "reprocharé": "reprochar",
    "reprocharéis": "reprochar",
    "reprocharía": "reprochar",
    "reprocharíais": "reprochar",
    "reprocharíamos": "reprochar",
    "reprocharían": "reprochar",
    "reprocharías": "reprochar",
    "reprochas": "reprochar",
    "reprochase": "reprochar",
    "reprochaseis": "reprochar",
    "reprochasen": "reprochar",
    "reprochases": "reprochar",
    "reprochaste": "reprochar",
    "reprochasteis": "reprochar",
    "reproche": "reprochar",
    "reprochemos": "reprochar",
    "reprochen": "reprochar",
    "reproches": "reproche",
    "reprocho": "reprochar",
    "reprochábamos": "reprochar",
    "reprocháis": "reprochar",
    "reprocháramos": "reprochar",
    "reprocháremos": "reprochar",
    "reprochásemos": "reprochar",
    "reproché": "reprochar",
    "reprochéis": "reprochar",
    "reprochó": "reprochar",
    "reproducciones": "reproducción",
    "reproduce": "reproducir",
    "reproducen": "reproducir",
    "reproduces": "reproducir",
    "reproducibilidades": "reproducibilidad",
    "reproducibles": "reproducible",
    "reproducid": "reproducir",
    "reproducida": "reproducir",
    "reproducidas": "reproducir",
    "reproducido": "reproducir",
    "reproducidos": "reproducir",
    "reproduciendo": "reproducir",
    "reproducimos": "reproducir",
    "reproduciremos": "reproducir",
    "reproducirá": "reproducir",
    "reproducirán": "reproducir",
    "reproducirás": "reproducir",
    "reproduciré": "reproducir",
    "reproduciréis": "reproducir",
    "reproduciría": "reproducir",
    "reproduciríais": "reproducir",
    "reproduciríamos": "reproducir",
    "reproducirían": "reproducir",
    "reproducirías": "reproducir",
    "reproductibilidades": "reproductibilidad",
    "reproductiva": "reproductivo",
    "reproductivas": "reproductivo",
    "reproductivos": "reproductivo",
    "reproductora": "reproductor",
    "reproductoras": "reproductora",
    "reproductores": "reproductor",
    "reproducía": "reproducir",
    "reproducíais": "reproducir",
    "reproducíamos": "reproducir",
    "reproducían": "reproducir",
    "reproducías": "reproducir",
    "reproducís": "reproducir",
    "reproduje": "reproducir",
    "reprodujera": "reproducir",
    "reprodujerais": "reproducir",
    "reprodujeran": "reproducir",
    "reprodujeras": "reproducir",
    "reprodujere": "reproducir",
    "reprodujereis": "reproducir",
    "reprodujeren": "reproducir",
    "reprodujeres": "reproducir",
    "reprodujeron": "reproducir",
    "reprodujese": "reproducir",
    "reprodujeseis": "reproducir",
    "reprodujesen": "reproducir",
    "reprodujeses": "reproducir",
    "reprodujimos": "reproducir",
    "reprodujiste": "reproducir",
    "reprodujisteis": "reproducir",
    "reprodujo": "reproducir",
    "reprodujéramos": "reproducir",
    "reprodujéremos": "reproducir",
    "reprodujésemos": "reproducir",
    "reproduzca": "reproducir",
    "reproduzcamos": "reproducir",
    "reproduzcan": "reproducir",
    "reproduzcas": "reproducir",
    "reproduzco": "reproducir",
    "reproduzcáis": "reproducir",
    "reprografías": "reprografía",
    "reprogramables": "reprogramable",
    "reprogramaciones": "reprogramación",
    "reprográfica": "reprográfico",
    "reprográficas": "reprográfico",
    "reprográficos": "reprográfico",
    "repropia": "repropio",
    "repropias": "repropio",
    "repropios": "repropio",
    "reprueba": "reprobar",
    "reprueban": "reprobar",
    "repruebas": "reprobar",
    "repruebe": "reprobar",
    "reprueben": "reprobar",
    "repruebes": "reprobar",
    "repruebo": "reprobar",
    "reprógrafa": "reprógrafo",
    "reprógrafas": "reprógrafo",
    "reprógrafos": "reprógrafo",
    "repta": "reptar",
    "reptaba": "reptar",
    "reptabais": "reptar",
    "reptaban": "reptar",
    "reptabas": "reptar",
    "reptaciones": "reptación",
    "reptad": "reptar",
    "reptada": "reptar",
    "reptadas": "reptar",
    "reptado": "reptar",
    "reptadora": "reptador",
    "reptadoras": "reptador",
    "reptadores": "reptador",
    "reptados": "reptar",
    "reptamos": "reptar",
    "reptan": "reptar",
    "reptando": "reptar",
    "reptantes": "reptante",
    "reptara": "reptar",
    "reptarais": "reptar",
    "reptaran": "reptar",
    "reptaras": "reptar",
    "reptare": "reptar",
    "reptareis": "reptar",
    "reptaremos": "reptar",
    "reptaren": "reptar",
    "reptares": "reptar",
    "reptaron": "reptar",
    "reptará": "reptar",
    "reptarán": "reptar",
    "reptarás": "reptar",
    "reptaré": "reptar",
    "reptaréis": "reptar",
    "reptaría": "reptar",
    "reptaríais": "reptar",
    "reptaríamos": "reptar",
    "reptarían": "reptar",
    "reptarías": "reptar",
    "reptas": "reptar",
    "reptase": "reptar",
    "reptaseis": "reptar",
    "reptasen": "reptar",
    "reptases": "reptar",
    "reptaste": "reptar",
    "reptasteis": "reptar",
    "repte": "reptar",
    "reptemos": "reptar",
    "repten": "reptar",
    "reptes": "reptar",
    "reptiles": "reptil",
    "repto": "reptar",
    "reptábamos": "reptar",
    "reptáis": "reptar",
    "reptáramos": "reptar",
    "reptáremos": "reptar",
    "reptásemos": "reptar",
    "repté": "reptar",
    "reptéis": "reptar",
    "reptó": "reptar",
    "republicana": "republicano",
    "republicanas": "republicano",
    "republicanismos": "republicanismo",
    "republicanos": "republicano",
    "repudia": "repudiar",
    "repudiaba": "repudiar",
    "repudiabais": "repudiar",
    "repudiaban": "repudiar",
    "repudiabas": "repudiar",
    "repudiaciones": "repudiación",
    "repudiad": "repudiar",
    "repudiada": "repudiar",
    "repudiadas": "repudiar",
    "repudiado": "repudiar",
    "repudiados": "repudiar",
    "repudiamos": "repudiar",
    "repudian": "repudiar",
    "repudiando": "repudiar",
    "repudiara": "repudiar",
    "repudiarais": "repudiar",
    "repudiaran": "repudiar",
    "repudiaras": "repudiar",
    "repudiare": "repudiar",
    "repudiareis": "repudiar",
    "repudiaremos": "repudiar",
    "repudiaren": "repudiar",
    "repudiares": "repudiar",
    "repudiaron": "repudiar",
    "repudiará": "repudiar",
    "repudiarán": "repudiar",
    "repudiarás": "repudiar",
    "repudiaré": "repudiar",
    "repudiaréis": "repudiar",
    "repudiaría": "repudiar",
    "repudiaríais": "repudiar",
    "repudiaríamos": "repudiar",
    "repudiarían": "repudiar",
    "repudiarías": "repudiar",
    "repudias": "repudiar",
    "repudiase": "repudiar",
    "repudiaseis": "repudiar",
    "repudiasen": "repudiar",
    "repudiases": "repudiar",
    "repudiaste": "repudiar",
    "repudiasteis": "repudiar",
    "repudie": "repudiar",
    "repudiemos": "repudiar",
    "repudien": "repudiar",
    "repudies": "repudiar",
    "repudio": "repudiar",
    "repudios": "repudio",
    "repudiábamos": "repudiar",
    "repudiáis": "repudiar",
    "repudiáramos": "repudiar",
    "repudiáremos": "repudiar",
    "repudiásemos": "repudiar",
    "repudié": "repudiar",
    "repudiéis": "repudiar",
    "repudió": "repudiar",
    "repudra": "repudrir",
    "repudramos": "repudrir",
    "repudran": "repudrir",
    "repudras": "repudrir",
    "repudre": "repudrir",
    "repudren": "repudrir",
    "repudres": "repudrir",
    "repudrid": "repudrir",
    "repudriendo": "repudrir",
    "repudriera": "repudrir",
    "repudrierais": "repudrir",
    "repudrieran": "repudrir",
    "repudrieras": "repudrir",
    "repudriere": "repudrir",
    "repudriereis": "repudrir",
    "repudrieren": "repudrir",
    "repudrieres": "repudrir",
    "repudrieron": "repudrir",
    "repudriese": "repudrir",
    "repudrieseis": "repudrir",
    "repudriesen": "repudrir",
    "repudrieses": "repudrir",
    "repudrimos": "repudrir",
    "repudrir": "repodrir",
    "repudriremos": "repudrir",
    "repudrirá": "repudrir",
    "repudrirán": "repudrir",
    "repudrirás": "repudrir",
    "repudriré": "repudrir",
    "repudriréis": "repudrir",
    "repudriría": "repudrir",
    "repudriríais": "repudrir",
    "repudriríamos": "repudrir",
    "repudrirían": "repudrir",
    "repudrirías": "repudrir",
    "repudriste": "repudrir",
    "repudristeis": "repudrir",
    "repudriéramos": "repudrir",
    "repudriéremos": "repudrir",
    "repudriésemos": "repudrir",
    "repudrió": "repudrir",
    "repudro": "repudrir",
    "repudráis": "repudrir",
    "repudrí": "repudrir",
    "repudría": "repudrir",
    "repudríais": "repudrir",
    "repudríamos": "repudrir",
    "repudrían": "repudrir",
    "repudrías": "repudrir",
    "repudrís": "repudrir",
    "repuebla": "repoblar",
    "repueblan": "repoblar",
    "repueblas": "repoblar",
    "repueble": "repoblar",
    "repueblen": "repoblar",
    "repuebles": "repoblar",
    "repueblo": "repoblar",
    "repuesta": "reponer",
    "repuestas": "reponer",
    "repuesto": "reponer",
    "repuestos": "repuesto",
    "repugna": "repugnar",
    "repugnaba": "repugnar",
    "repugnabais": "repugnar",
    "repugnaban": "repugnar",
    "repugnabas": "repugnar",
    "repugnad": "repugnar",
    "repugnada": "repugnar",
    "repugnadas": "repugnar",
    "repugnado": "repugnar",
    "repugnados": "repugnar",
    "repugnamos": "repugnar",
    "repugnan": "repugnar",
    "repugnancias": "repugnancia",
    "repugnando": "repugnar",
    "repugnantes": "repugnante",
    "repugnara": "repugnar",
    "repugnarais": "repugnar",
    "repugnaran": "repugnar",
    "repugnaras": "repugnar",
    "repugnare": "repugnar",
    "repugnareis": "repugnar",
    "repugnaremos": "repugnar",
    "repugnaren": "repugnar",
    "repugnares": "repugnar",
    "repugnaron": "repugnar",
    "repugnará": "repugnar",
    "repugnarán": "repugnar",
    "repugnarás": "repugnar",
    "repugnaré": "repugnar",
    "repugnaréis": "repugnar",
    "repugnaría": "repugnar",
    "repugnaríais": "repugnar",
    "repugnaríamos": "repugnar",
    "repugnarían": "repugnar",
    "repugnarías": "repugnar",
    "repugnas": "repugnar",
    "repugnase": "repugnar",
    "repugnaseis": "repugnar",
    "repugnasen": "repugnar",
    "repugnases": "repugnar",
    "repugnaste": "repugnar",
    "repugnasteis": "repugnar",
    "repugne": "repugnar",
    "repugnemos": "repugnar",
    "repugnen": "repugnar",
    "repugnes": "repugnar",
    "repugno": "repugnar",
    "repugnábamos": "repugnar",
    "repugnáis": "repugnar",
    "repugnáramos": "repugnar",
    "repugnáremos": "repugnar",
    "repugnásemos": "repugnar",
    "repugné": "repugnar",
    "repugnéis": "repugnar",
    "repugnó": "repugnar",
    "repuja": "repujar",
    "repujaba": "repujar",
    "repujabais": "repujar",
    "repujaban": "repujar",
    "repujabas": "repujar",
    "repujad": "repujar",
    "repujada": "repujar",
    "repujadas": "repujar",
    "repujado": "repujar",
    "repujadora": "repujador",
    "repujadoras": "repujador",
    "repujadores": "repujador",
    "repujados": "repujar",
    "repujamos": "repujar",
    "repujan": "repujar",
    "repujando": "repujar",
    "repujara": "repujar",
    "repujarais": "repujar",
    "repujaran": "repujar",
    "repujaras": "repujar",
    "repujare": "repujar",
    "repujareis": "repujar",
    "repujaremos": "repujar",
    "repujaren": "repujar",
    "repujares": "repujar",
    "repujaron": "repujar",
    "repujará": "repujar",
    "repujarán": "repujar",
    "repujarás": "repujar",
    "repujaré": "repujar",
    "repujaréis": "repujar",
    "repujaría": "repujar",
    "repujaríais": "repujar",
    "repujaríamos": "repujar",
    "repujarían": "repujar",
    "repujarías": "repujar",
    "repujas": "repujar",
    "repujase": "repujar",
    "repujaseis": "repujar",
    "repujasen": "repujar",
    "repujases": "repujar",
    "repujaste": "repujar",
    "repujasteis": "repujar",
    "repuje": "repujar",
    "repujemos": "repujar",
    "repujen": "repujar",
    "repujes": "repujar",
    "repujo": "repujar",
    "repujábamos": "repujar",
    "repujáis": "repujar",
    "repujáramos": "repujar",
    "repujáremos": "repujar",
    "repujásemos": "repujar",
    "repujé": "repujar",
    "repujéis": "repujar",
    "repujó": "repujar",
    "repula": "repulir",
    "repulamos": "repulir",
    "repulan": "repulir",
    "repulas": "repulir",
    "repule": "repulir",
    "repulen": "repulir",
    "repules": "repulir",
    "repulga": "repulgar",
    "repulgaba": "repulgar",
    "repulgabais": "repulgar",
    "repulgaban": "repulgar",
    "repulgabas": "repulgar",
    "repulgad": "repulgar",
    "repulgada": "repulgar",
    "repulgadas": "repulgar",
    "repulgado": "repulgar",
    "repulgados": "repulgar",
    "repulgamos": "repulgar",
    "repulgan": "repulgar",
    "repulgando": "repulgar",
    "repulgara": "repulgar",
    "repulgarais": "repulgar",
    "repulgaran": "repulgar",
    "repulgaras": "repulgar",
    "repulgare": "repulgar",
    "repulgareis": "repulgar",
    "repulgaremos": "repulgar",
    "repulgaren": "repulgar",
    "repulgares": "repulgar",
    "repulgaron": "repulgar",
    "repulgará": "repulgar",
    "repulgarán": "repulgar",
    "repulgarás": "repulgar",
    "repulgaré": "repulgar",
    "repulgaréis": "repulgar",
    "repulgaría": "repulgar",
    "repulgaríais": "repulgar",
    "repulgaríamos": "repulgar",
    "repulgarían": "repulgar",
    "repulgarías": "repulgar",
    "repulgas": "repulgar",
    "repulgase": "repulgar",
    "repulgaseis": "repulgar",
    "repulgasen": "repulgar",
    "repulgases": "repulgar",
    "repulgaste": "repulgar",
    "repulgasteis": "repulgar",
    "repulgo": "repulgar",
    "repulgos": "repulgo",
    "repulgue": "repulgar",
    "repulguemos": "repulgar",
    "repulguen": "repulgar",
    "repulgues": "repulgar",
    "repulgué": "repulgar",
    "repulguéis": "repulgar",
    "repulgábamos": "repulgar",
    "repulgáis": "repulgar",
    "repulgáramos": "repulgar",
    "repulgáremos": "repulgar",
    "repulgásemos": "repulgar",
    "repulgó": "repulgar",
    "repulid": "repulir",
    "repulida": "repulir",
    "repulidas": "repulir",
    "repulido": "repulir",
    "repulidos": "repulir",
    "repuliendo": "repulir",
    "repuliera": "repulir",
    "repulierais": "repulir",
    "repulieran": "repulir",
    "repulieras": "repulir",
    "repuliere": "repulir",
    "repuliereis": "repulir",
    "repulieren": "repulir",
    "repulieres": "repulir",
    "repulieron": "repulir",
    "repuliese": "repulir",
    "repulieseis": "repulir",
    "repuliesen": "repulir",
    "repulieses": "repulir",
    "repulimos": "repulir",
    "repuliremos": "repulir",
    "repulirá": "repulir",
    "repulirán": "repulir",
    "repulirás": "repulir",
    "repuliré": "repulir",
    "repuliréis": "repulir",
    "repuliría": "repulir",
    "repuliríais": "repulir",
    "repuliríamos": "repulir",
    "repulirían": "repulir",
    "repulirías": "repulir",
    "repuliste": "repulir",
    "repulisteis": "repulir",
    "repuliéramos": "repulir",
    "repuliéremos": "repulir",
    "repuliésemos": "repulir",
    "repulió": "repulir",
    "repullos": "repullo",
    "repulo": "repulir",
    "repulsa": "repulsar",
    "repulsaba": "repulsar",
    "repulsabais": "repulsar",
    "repulsaban": "repulsar",
    "repulsabas": "repulsar",
    "repulsad": "repulsar",
    "repulsada": "repulsar",
    "repulsadas": "repulsar",
    "repulsado": "repulsar",
    "repulsados": "repulsar",
    "repulsamos": "repulsar",
    "repulsan": "repulsar",
    "repulsando": "repulsar",
    "repulsara": "repulsar",
    "repulsarais": "repulsar",
    "repulsaran": "repulsar",
    "repulsaras": "repulsar",
    "repulsare": "repulsar",
    "repulsareis": "repulsar",
    "repulsaremos": "repulsar",
    "repulsaren": "repulsar",
    "repulsares": "repulsar",
    "repulsaron": "repulsar",
    "repulsará": "repulsar",
    "repulsarán": "repulsar",
    "repulsarás": "repulsar",
    "repulsaré": "repulsar",
    "repulsaréis": "repulsar",
    "repulsaría": "repulsar",
    "repulsaríais": "repulsar",
    "repulsaríamos": "repulsar",
    "repulsarían": "repulsar",
    "repulsarías": "repulsar",
    "repulsas": "repulsar",
    "repulsase": "repulsar",
    "repulsaseis": "repulsar",
    "repulsasen": "repulsar",
    "repulsases": "repulsar",
    "repulsaste": "repulsar",
    "repulsasteis": "repulsar",
    "repulse": "repulsar",
    "repulsemos": "repulsar",
    "repulsen": "repulsar",
    "repulses": "repulsar",
    "repulsiones": "repulsión",
    "repulsiva": "repulsivo",
    "repulsivas": "repulsivo",
    "repulsivos": "repulsivo",
    "repulso": "repulsar",
    "repulsábamos": "repulsar",
    "repulsáis": "repulsar",
    "repulsáramos": "repulsar",
    "repulsáremos": "repulsar",
    "repulsásemos": "repulsar",
    "repulsé": "repulsar",
    "repulséis": "repulsar",
    "repulsó": "repulsar",
    "repuláis": "repulir",
    "repulí": "repulir",
    "repulía": "repulir",
    "repulíais": "repulir",
    "repulíamos": "repulir",
    "repulían": "repulir",
    "repulías": "repulir",
    "repulís": "repulir",
    "repunta": "repuntar",
    "repuntaba": "repuntar",
    "repuntabais": "repuntar",
    "repuntaban": "repuntar",
    "repuntabas": "repuntar",
    "repuntad": "repuntar",
    "repuntada": "repuntar",
    "repuntadas": "repuntar",
    "repuntado": "repuntar",
    "repuntados": "repuntar",
    "repuntamos": "repuntar",
    "repuntan": "repuntar",
    "repuntando": "repuntar",
    "repuntara": "repuntar",
    "repuntarais": "repuntar",
    "repuntaran": "repuntar",
    "repuntaras": "repuntar",
    "repuntare": "repuntar",
    "repuntareis": "repuntar",
    "repuntaremos": "repuntar",
    "repuntaren": "repuntar",
    "repuntares": "repuntar",
    "repuntaron": "repuntar",
    "repuntará": "repuntar",
    "repuntarán": "repuntar",
    "repuntarás": "repuntar",
    "repuntaré": "repuntar",
    "repuntaréis": "repuntar",
    "repuntaría": "repuntar",
    "repuntaríais": "repuntar",
    "repuntaríamos": "repuntar",
    "repuntarían": "repuntar",
    "repuntarías": "repuntar",
    "repuntas": "repuntar",
    "repuntase": "repuntar",
    "repuntaseis": "repuntar",
    "repuntasen": "repuntar",
    "repuntases": "repuntar",
    "repuntaste": "repuntar",
    "repuntasteis": "repuntar",
    "repunte": "repuntar",
    "repuntemos": "repuntar",
    "repunten": "repuntar",
    "repuntes": "repunte",
    "repunto": "repuntar",
    "repuntábamos": "repuntar",
    "repuntáis": "repuntar",
    "repuntáramos": "repuntar",
    "repuntáremos": "repuntar",
    "repuntásemos": "repuntar",
    "repunté": "repuntar",
    "repuntéis": "repuntar",
    "repuntó": "repuntar",
    "repurga": "repurgar",
    "repurgaba": "repurgar",
    "repurgabais": "repurgar",
    "repurgaban": "repurgar",
    "repurgabas": "repurgar",
    "repurgad": "repurgar",
    "repurgada": "repurgar",
    "repurgadas": "repurgar",
    "repurgado": "repurgar",
    "repurgados": "repurgar",
    "repurgamos": "repurgar",
    "repurgan": "repurgar",
    "repurgando": "repurgar",
    "repurgara": "repurgar",
    "repurgarais": "repurgar",
    "repurgaran": "repurgar",
    "repurgaras": "repurgar",
    "repurgare": "repurgar",
    "repurgareis": "repurgar",
    "repurgaremos": "repurgar",
    "repurgaren": "repurgar",
    "repurgares": "repurgar",
    "repurgaron": "repurgar",
    "repurgará": "repurgar",
    "repurgarán": "repurgar",
    "repurgarás": "repurgar",
    "repurgaré": "repurgar",
    "repurgaréis": "repurgar",
    "repurgaría": "repurgar",
    "repurgaríais": "repurgar",
    "repurgaríamos": "repurgar",
    "repurgarían": "repurgar",
    "repurgarías": "repurgar",
    "repurgas": "repurgar",
    "repurgase": "repurgar",
    "repurgaseis": "repurgar",
    "repurgasen": "repurgar",
    "repurgases": "repurgar",
    "repurgaste": "repurgar",
    "repurgasteis": "repurgar",
    "repurgo": "repurgar",
    "repurgue": "repurgar",
    "repurguemos": "repurgar",
    "repurguen": "repurgar",
    "repurgues": "repurgar",
    "repurgué": "repurgar",
    "repurguéis": "repurgar",
    "repurgábamos": "repurgar",
    "repurgáis": "repurgar",
    "repurgáramos": "repurgar",
    "repurgáremos": "repurgar",
    "repurgásemos": "repurgar",
    "repurgó": "repurgar",
    "repuse": "reponer",
    "repusiera": "reponer",
    "repusierais": "reponer",
    "repusieran": "reponer",
    "repusieras": "reponer",
    "repusiere": "reponer",
    "repusiereis": "reponer",
    "repusieren": "reponer",
    "repusieres": "reponer",
    "repusieron": "reponer",
    "repusiese": "reponer",
    "repusieseis": "reponer",
    "repusiesen": "reponer",
    "repusieses": "reponer",
    "repusimos": "reponer",
    "repusiste": "reponer",
    "repusisteis": "reponer",
    "repusiéramos": "reponer",
    "repusiéremos": "reponer",
    "repusiésemos": "reponer",
    "repuso": "reponer",
    "reputa": "reputar",
    "reputaba": "reputar",
    "reputabais": "reputar",
    "reputaban": "reputar",
    "reputabas": "reputar",
    "reputaciones": "reputación",
    "reputad": "reputar",
    "reputada": "reputar",
    "reputadas": "reputar",
    "reputado": "reputar",
    "reputados": "reputar",
    "reputamos": "reputar",
    "reputan": "reputar",
    "reputando": "reputar",
    "reputara": "reputar",
    "reputarais": "reputar",
    "reputaran": "reputar",
    "reputaras": "reputar",
    "reputare": "reputar",
    "reputareis": "reputar",
    "reputaremos": "reputar",
    "reputaren": "reputar",
    "reputares": "reputar",
    "reputaron": "reputar",
    "reputará": "reputar",
    "reputarán": "reputar",
    "reputarás": "reputar",
    "reputaré": "reputar",
    "reputaréis": "reputar",
    "reputaría": "reputar",
    "reputaríais": "reputar",
    "reputaríamos": "reputar",
    "reputarían": "reputar",
    "reputarías": "reputar",
    "reputas": "reputar",
    "reputase": "reputar",
    "reputaseis": "reputar",
    "reputasen": "reputar",
    "reputases": "reputar",
    "reputaste": "reputar",
    "reputasteis": "reputar",
    "repute": "reputar",
    "reputemos": "reputar",
    "reputen": "reputar",
    "reputes": "reputar",
    "reputo": "reputar",
    "reputábamos": "reputar",
    "reputáis": "reputar",
    "reputáramos": "reputar",
    "reputáremos": "reputar",
    "reputásemos": "reputar",
    "reputé": "reputar",
    "reputéis": "reputar",
    "reputó": "reputar",
    "repón": "reponer",
    "repórteres": "repórter",
    "repórters": "repórter",
    "repúblicas": "república",
    "repúblicos": "repúblico",
    "requebraba": "requebrar",
    "requebrabais": "requebrar",
    "requebraban": "requebrar",
    "requebrabas": "requebrar",
    "requebrad": "requebrar",
    "requebrada": "requebrar",
    "requebradas": "requebrar",
    "requebrado": "requebrar",
    "requebradora": "requebrador",
    "requebradoras": "requebrador",
    "requebradores": "requebrador",
    "requebrados": "requebrar",
    "requebraduras": "requebradura",
    "requebramos": "requebrar",
    "requebrando": "requebrar",
    "requebrara": "requebrar",
    "requebrarais": "requebrar",
    "requebraran": "requebrar",
    "requebraras": "requebrar",
    "requebrare": "requebrar",
    "requebrareis": "requebrar",
    "requebraremos": "requebrar",
    "requebraren": "requebrar",
    "requebrares": "requebrar",
    "requebraron": "requebrar",
    "requebrará": "requebrar",
    "requebrarán": "requebrar",
    "requebrarás": "requebrar",
    "requebraré": "requebrar",
    "requebraréis": "requebrar",
    "requebraría": "requebrar",
    "requebraríais": "requebrar",
    "requebraríamos": "requebrar",
    "requebrarían": "requebrar",
    "requebrarías": "requebrar",
    "requebrase": "requebrar",
    "requebraseis": "requebrar",
    "requebrasen": "requebrar",
    "requebrases": "requebrar",
    "requebraste": "requebrar",
    "requebrasteis": "requebrar",
    "requebremos": "requebrar",
    "requebrábamos": "requebrar",
    "requebráis": "requebrar",
    "requebráramos": "requebrar",
    "requebráremos": "requebrar",
    "requebrásemos": "requebrar",
    "requebré": "requebrar",
    "requebréis": "requebrar",
    "requebró": "requebrar",
    "requema": "requemar",
    "requemaba": "requemar",
    "requemabais": "requemar",
    "requemaban": "requemar",
    "requemabas": "requemar",
    "requemad": "requemar",
    "requemada": "requemar",
    "requemadas": "requemar",
    "requemado": "requemar",
    "requemados": "requemar",
    "requemamientos": "requemamiento",
    "requemamos": "requemar",
    "requeman": "requemar",
    "requemando": "requemar",
    "requemara": "requemar",
    "requemarais": "requemar",
    "requemaran": "requemar",
    "requemaras": "requemar",
    "requemare": "requemar",
    "requemareis": "requemar",
    "requemaremos": "requemar",
    "requemaren": "requemar",
    "requemares": "requemar",
    "requemaron": "requemar",
    "requemará": "requemar",
    "requemarán": "requemar",
    "requemarás": "requemar",
    "requemaré": "requemar",
    "requemaréis": "requemar",
    "requemaría": "requemar",
    "requemaríais": "requemar",
    "requemaríamos": "requemar",
    "requemarían": "requemar",
    "requemarías": "requemar",
    "requemas": "requemar",
    "requemase": "requemar",
    "requemaseis": "requemar",
    "requemasen": "requemar",
    "requemases": "requemar",
    "requemaste": "requemar",
    "requemasteis": "requemar",
    "requeme": "requemar",
    "requememos": "requemar",
    "requemen": "requemar",
    "requemes": "requemar",
    "requemo": "requemar",
    "requemábamos": "requemar",
    "requemáis": "requemar",
    "requemáramos": "requemar",
    "requemáremos": "requemar",
    "requemásemos": "requemar",
    "requemé": "requemar",
    "requeméis": "requemar",
    "requemó": "requemar",
    "requerid": "requerir",
    "requerida": "requerir",
    "requeridas": "requerir",
    "requerido": "requerir",
    "requeridora": "requeridor",
    "requeridoras": "requeridor",
    "requeridores": "requeridor",
    "requeridos": "requerir",
    "requerimientos": "requerimiento",
    "requerimos": "requerir",
    "requeriremos": "requerir",
    "requerirá": "requerir",
    "requerirán": "requerir",
    "requerirás": "requerir",
    "requeriré": "requerir",
    "requeriréis": "requerir",
    "requeriría": "requerir",
    "requeriríais": "requerir",
    "requeriríamos": "requerir",
    "requerirían": "requerir",
    "requerirías": "requerir",
    "requeriste": "requerir",
    "requeristeis": "requerir",
    "requerí": "requerir",
    "requería": "requerir",
    "requeríais": "requerir",
    "requeríamos": "requerir",
    "requerían": "requerir",
    "requerías": "requerir",
    "requerís": "requerir",
    "requesonera": "requesonero",
    "requesoneras": "requesonero",
    "requesoneros": "requesonero",
    "requesones": "requesón",
    "requestas": "requesta",
    "requetés": "requeté",
    "requiebra": "requebrar",
    "requiebran": "requebrar",
    "requiebras": "requebrar",
    "requiebre": "requebrar",
    "requiebren": "requebrar",
    "requiebres": "requebrar",
    "requiebro": "requebrar",
    "requiebros": "requiebro",
    "requiera": "requerir",
    "requieran": "requerir",
    "requieras": "requerir",
    "requiere": "requerir",
    "requieren": "requerir",
    "requieres": "requerir",
    "requiero": "requerir",
    "requilorios": "requilorio",
    "requinta": "requintar",
    "requintaba": "requintar",
    "requintabais": "requintar",
    "requintaban": "requintar",
    "requintabas": "requintar",
    "requintad": "requintar",
    "requintada": "requintar",
    "requintadas": "requintar",
    "requintado": "requintar",
    "requintadora": "requintador",
    "requintadoras": "requintador",
    "requintadores": "requintador",
    "requintados": "requintar",
    "requintamos": "requintar",
    "requintan": "requintar",
    "requintando": "requintar",
    "requintara": "requintar",
    "requintarais": "requintar",
    "requintaran": "requintar",
    "requintaras": "requintar",
    "requintare": "requintar",
    "requintareis": "requintar",
    "requintaremos": "requintar",
    "requintaren": "requintar",
    "requintares": "requintar",
    "requintaron": "requintar",
    "requintará": "requintar",
    "requintarán": "requintar",
    "requintarás": "requintar",
    "requintaré": "requintar",
    "requintaréis": "requintar",
    "requintaría": "requintar",
    "requintaríais": "requintar",
    "requintaríamos": "requintar",
    "requintarían": "requintar",
    "requintarías": "requintar",
    "requintas": "requintar",
    "requintase": "requintar",
    "requintaseis": "requintar",
    "requintasen": "requintar",
    "requintases": "requintar",
    "requintaste": "requintar",
    "requintasteis": "requintar",
    "requinte": "requintar",
    "requintemos": "requintar",
    "requinten": "requintar",
    "requintes": "requintar",
    "requinto": "requintar",
    "requintos": "requinto",
    "requintábamos": "requintar",
    "requintáis": "requintar",
    "requintáramos": "requintar",
    "requintáremos": "requintar",
    "requintásemos": "requintar",
    "requinté": "requintar",
    "requintéis": "requintar",
    "requintó": "requintar",
    "requiramos": "requerir",
    "requirentes": "requirente",
    "requiriendo": "requerir",
    "requirientes": "requiriente",
    "requiriera": "requerir",
    "requirierais": "requerir",
    "requirieran": "requerir",
    "requirieras": "requerir",
    "requiriere": "requerir",
    "requiriereis": "requerir",
    "requirieren": "requerir",
    "requirieres": "requerir",
    "requirieron": "requerir",
    "requiriese": "requerir",
    "requirieseis": "requerir",
    "requiriesen": "requerir",
    "requirieses": "requerir",
    "requiriéramos": "requerir",
    "requiriéremos": "requerir",
    "requiriésemos": "requerir",
    "requirió": "requerir",
    "requiráis": "requerir",
    "requisa": "requisar",
    "requisaba": "requisar",
    "requisabais": "requisar",
    "requisaban": "requisar",
    "requisabas": "requisar",
    "requisad": "requisar",
    "requisada": "requisar",
    "requisadas": "requisar",
    "requisado": "requisar",
    "requisados": "requisar",
    "requisamos": "requisar",
    "requisan": "requisar",
    "requisando": "requisar",
    "requisara": "requisar",
    "requisarais": "requisar",
    "requisaran": "requisar",
    "requisaras": "requisar",
    "requisare": "requisar",
    "requisareis": "requisar",
    "requisaremos": "requisar",
    "requisaren": "requisar",
    "requisares": "requisar",
    "requisaron": "requisar",
    "requisará": "requisar",
    "requisarán": "requisar",
    "requisarás": "requisar",
    "requisaré": "requisar",
    "requisaréis": "requisar",
    "requisaría": "requisar",
    "requisaríais": "requisar",
    "requisaríamos": "requisar",
    "requisarían": "requisar",
    "requisarías": "requisar",
    "requisas": "requisar",
    "requisase": "requisar",
    "requisaseis": "requisar",
    "requisasen": "requisar",
    "requisases": "requisar",
    "requisaste": "requisar",
    "requisasteis": "requisar",
    "requise": "requisar",
    "requisemos": "requisar",
    "requisen": "requisar",
    "requises": "requisar",
    "requisiciones": "requisición",
    "requisita": "requerir",
    "requisitas": "requerir",
    "requisito": "requerir",
    "requisitoria": "requisitorio",
    "requisitorias": "requisitorio",
    "requisitorios": "requisitorio",
    "requisitos": "requisito",
    "requiso": "requisar",
    "requisábamos": "requisar",
    "requisáis": "requisar",
    "requisáramos": "requisar",
    "requisáremos": "requisar",
    "requisásemos": "requisar",
    "requisé": "requisar",
    "requiséis": "requisar",
    "requisó": "requisar",
    "reradiaciones": "reradiación",
    "res": "re",
    "resabia": "resabiar",
    "resabiaba": "resabiar",
    "resabiabais": "resabiar",
    "resabiaban": "resabiar",
    "resabiabas": "resabiar",
    "resabiad": "resabiar",
    "resabiada": "resabiar",
    "resabiadas": "resabiar",
    "resabiado": "resabiar",
    "resabiados": "resabiar",
    "resabiamos": "resabiar",
    "resabian": "resabiar",
    "resabiando": "resabiar",
    "resabiara": "resabiar",
    "resabiarais": "resabiar",
    "resabiaran": "resabiar",
    "resabiaras": "resabiar",
    "resabiare": "resabiar",
    "resabiareis": "resabiar",
    "resabiaremos": "resabiar",
    "resabiaren": "resabiar",
    "resabiares": "resabiar",
    "resabiaron": "resabiar",
    "resabiará": "resabiar",
    "resabiarán": "resabiar",
    "resabiarás": "resabiar",
    "resabiaré": "resabiar",
    "resabiaréis": "resabiar",
    "resabiaría": "resabiar",
    "resabiaríais": "resabiar",
    "resabiaríamos": "resabiar",
    "resabiarían": "resabiar",
    "resabiarías": "resabiar",
    "resabias": "resabiar",
    "resabiase": "resabiar",
    "resabiaseis": "resabiar",
    "resabiasen": "resabiar",
    "resabiases": "resabiar",
    "resabiaste": "resabiar",
    "resabiasteis": "resabiar",
    "resabida": "resabido",
    "resabidas": "resabido",
    "resabidos": "resabido",
    "resabie": "resabiar",
    "resabiemos": "resabiar",
    "resabien": "resabiar",
    "resabies": "resabiar",
    "resabio": "resabiar",
    "resabios": "resabio",
    "resabiábamos": "resabiar",
    "resabiáis": "resabiar",
    "resabiáramos": "resabiar",
    "resabiáremos": "resabiar",
    "resabiásemos": "resabiar",
    "resabié": "resabiar",
    "resabiéis": "resabiar",
    "resabió": "resabiar",
    "resaca": "resacar",
    "resacaba": "resacar",
    "resacabais": "resacar",
    "resacaban": "resacar",
    "resacabas": "resacar",
    "resacad": "resacar",
    "resacada": "resacar",
    "resacadas": "resacar",
    "resacado": "resacar",
    "resacados": "resacar",
    "resacamos": "resacar",
    "resacan": "resacar",
    "resacando": "resacar",
    "resacara": "resacar",
    "resacarais": "resacar",
    "resacaran": "resacar",
    "resacaras": "resacar",
    "resacare": "resacar",
    "resacareis": "resacar",
    "resacaremos": "resacar",
    "resacaren": "resacar",
    "resacares": "resacar",
    "resacaron": "resacar",
    "resacará": "resacar",
    "resacarán": "resacar",
    "resacarás": "resacar",
    "resacaré": "resacar",
    "resacaréis": "resacar",
    "resacaría": "resacar",
    "resacaríais": "resacar",
    "resacaríamos": "resacar",
    "resacarían": "resacar",
    "resacarías": "resacar",
    "resacas": "resacar",
    "resacase": "resacar",
    "resacaseis": "resacar",
    "resacasen": "resacar",
    "resacases": "resacar",
    "resacaste": "resacar",
    "resacasteis": "resacar",
    "resaco": "resacar",
    "resacábamos": "resacar",
    "resacáis": "resacar",
    "resacáramos": "resacar",
    "resacáremos": "resacar",
    "resacásemos": "resacar",
    "resacó": "resacar",
    "resal": "resalir",
    "resalada": "resalado",
    "resaladas": "resalado",
    "resalados": "resalado",
    "resaldremos": "resalir",
    "resaldrá": "resalir",
    "resaldrán": "resalir",
    "resaldrás": "resalir",
    "resaldré": "resalir",
    "resaldréis": "resalir",
    "resaldría": "resalir",
    "resaldríais": "resalir",
    "resaldríamos": "resalir",
    "resaldrían": "resalir",
    "resaldrías": "resalir",
    "resale": "resalir",
    "resalen": "resalir",
    "resales": "resalir",
    "resalga": "resalir",
    "resalgamos": "resalir",
    "resalgan": "resalir",
    "resalgas": "resalir",
    "resalgo": "resalir",
    "resalgáis": "resalir",
    "resalid": "resalir",
    "resalida": "resalir",
    "resalidas": "resalir",
    "resalido": "resalir",
    "resalidos": "resalir",
    "resaliendo": "resalir",
    "resaliera": "resalir",
    "resalierais": "resalir",
    "resalieran": "resalir",
    "resalieras": "resalir",
    "resaliere": "resalir",
    "resaliereis": "resalir",
    "resalieren": "resalir",
    "resalieres": "resalir",
    "resalieron": "resalir",
    "resaliese": "resalir",
    "resalieseis": "resalir",
    "resaliesen": "resalir",
    "resalieses": "resalir",
    "resalimos": "resalir",
    "resaliste": "resalir",
    "resalisteis": "resalir",
    "resaliéramos": "resalir",
    "resaliéremos": "resalir",
    "resaliésemos": "resalir",
    "resalió": "resalir",
    "resalla": "resallar",
    "resallaba": "resallar",
    "resallabais": "resallar",
    "resallaban": "resallar",
    "resallabas": "resallar",
    "resallad": "resallar",
    "resallada": "resallar",
    "resalladas": "resallar",
    "resallado": "resallar",
    "resallados": "resallar",
    "resallamos": "resallar",
    "resallan": "resallar",
    "resallando": "resallar",
    "resallara": "resallar",
    "resallarais": "resallar",
    "resallaran": "resallar",
    "resallaras": "resallar",
    "resallare": "resallar",
    "resallareis": "resallar",
    "resallaremos": "resallar",
    "resallaren": "resallar",
    "resallares": "resallar",
    "resallaron": "resallar",
    "resallará": "resallar",
    "resallarán": "resallar",
    "resallarás": "resallar",
    "resallaré": "resallar",
    "resallaréis": "resallar",
    "resallaría": "resallar",
    "resallaríais": "resallar",
    "resallaríamos": "resallar",
    "resallarían": "resallar",
    "resallarías": "resallar",
    "resallas": "resallar",
    "resallase": "resallar",
    "resallaseis": "resallar",
    "resallasen": "resallar",
    "resallases": "resallar",
    "resallaste": "resallar",
    "resallasteis": "resallar",
    "resalle": "resallar",
    "resallemos": "resallar",
    "resallen": "resallar",
    "resalles": "resallar",
    "resallo": "resallar",
    "resallábamos": "resallar",
    "resalláis": "resallar",
    "resalláramos": "resallar",
    "resalláremos": "resallar",
    "resallásemos": "resallar",
    "resallé": "resallar",
    "resalléis": "resallar",
    "resalló": "resallar",
    "resalta": "resaltar",
    "resaltaba": "resaltar",
    "resaltabais": "resaltar",
    "resaltaban": "resaltar",
    "resaltabas": "resaltar",
    "resaltad": "resaltar",
    "resaltada": "resaltar",
    "resaltadas": "resaltar",
    "resaltado": "resaltar",
    "resaltados": "resaltar",
    "resaltamos": "resaltar",
    "resaltan": "resaltar",
    "resaltando": "resaltar",
    "resaltara": "resaltar",
    "resaltarais": "resaltar",
    "resaltaran": "resaltar",
    "resaltaras": "resaltar",
    "resaltare": "resaltar",
    "resaltareis": "resaltar",
    "resaltaremos": "resaltar",
    "resaltaren": "resaltar",
    "resaltares": "resaltar",
    "resaltaron": "resaltar",
    "resaltará": "resaltar",
    "resaltarán": "resaltar",
    "resaltarás": "resaltar",
    "resaltaré": "resaltar",
    "resaltaréis": "resaltar",
    "resaltaría": "resaltar",
    "resaltaríais": "resaltar",
    "resaltaríamos": "resaltar",
    "resaltarían": "resaltar",
    "resaltarías": "resaltar",
    "resaltas": "resaltar",
    "resaltase": "resaltar",
    "resaltaseis": "resaltar",
    "resaltasen": "resaltar",
    "resaltases": "resaltar",
    "resaltaste": "resaltar",
    "resaltasteis": "resaltar",
    "resalte": "resaltar",
    "resaltemos": "resaltar",
    "resalten": "resaltar",
    "resaltes": "resalte",
    "resalto": "resaltar",
    "resaltos": "resalto",
    "resaltábamos": "resaltar",
    "resaltáis": "resaltar",
    "resaltáramos": "resaltar",
    "resaltáremos": "resaltar",
    "resaltásemos": "resaltar",
    "resalté": "resaltar",
    "resaltéis": "resaltar",
    "resaltó": "resaltar",
    "resalutaciones": "resalutación",
    "resalvos": "resalvo",
    "resalí": "resalir",
    "resalía": "resalir",
    "resalíais": "resalir",
    "resalíamos": "resalir",
    "resalían": "resalir",
    "resalías": "resalir",
    "resalís": "resalir",
    "resana": "resanar",
    "resanaba": "resanar",
    "resanabais": "resanar",
    "resanaban": "resanar",
    "resanabas": "resanar",
    "resanad": "resanar",
    "resanada": "resanar",
    "resanadas": "resanar",
    "resanado": "resanar",
    "resanados": "resanar",
    "resanamos": "resanar",
    "resanan": "resanar",
    "resanando": "resanar",
    "resanara": "resanar",
    "resanarais": "resanar",
    "resanaran": "resanar",
    "resanaras": "resanar",
    "resanare": "resanar",
    "resanareis": "resanar",
    "resanaremos": "resanar",
    "resanaren": "resanar",
    "resanares": "resanar",
    "resanaron": "resanar",
    "resanará": "resanar",
    "resanarán": "resanar",
    "resanarás": "resanar",
    "resanaré": "resanar",
    "resanaréis": "resanar",
    "resanaría": "resanar",
    "resanaríais": "resanar",
    "resanaríamos": "resanar",
    "resanarían": "resanar",
    "resanarías": "resanar",
    "resanas": "resanar",
    "resanase": "resanar",
    "resanaseis": "resanar",
    "resanasen": "resanar",
    "resanases": "resanar",
    "resanaste": "resanar",
    "resanasteis": "resanar",
    "resane": "resanar",
    "resanemos": "resanar",
    "resanen": "resanar",
    "resanes": "resanar",
    "resano": "resanar",
    "resanábamos": "resanar",
    "resanáis": "resanar",
    "resanáramos": "resanar",
    "resanáremos": "resanar",
    "resanásemos": "resanar",
    "resané": "resanar",
    "resanéis": "resanar",
    "resanó": "resanar",
    "resaque": "resacar",
    "resaquemos": "resacar",
    "resaquen": "resacar",
    "resaques": "resacar",
    "resaqué": "resacar",
    "resaquéis": "resacar",
    "resarce": "resarcir",
    "resarcen": "resarcir",
    "resarces": "resarcir",
    "resarcibles": "resarcible",
    "resarcid": "resarcir",
    "resarcida": "resarcir",
    "resarcidas": "resarcir",
    "resarcido": "resarcir",
    "resarcidos": "resarcir",
    "resarciendo": "resarcir",
    "resarciera": "resarcir",
    "resarcierais": "resarcir",
    "resarcieran": "resarcir",
    "resarcieras": "resarcir",
    "resarciere": "resarcir",
    "resarciereis": "resarcir",
    "resarcieren": "resarcir",
    "resarcieres": "resarcir",
    "resarcieron": "resarcir",
    "resarciese": "resarcir",
    "resarcieseis": "resarcir",
    "resarciesen": "resarcir",
    "resarcieses": "resarcir",
    "resarcimientos": "resarcimiento",
    "resarcimos": "resarcir",
    "resarciremos": "resarcir",
    "resarcirá": "resarcir",
    "resarcirán": "resarcir",
    "resarcirás": "resarcir",
    "resarciré": "resarcir",
    "resarciréis": "resarcir",
    "resarciría": "resarcir",
    "resarciríais": "resarcir",
    "resarciríamos": "resarcir",
    "resarcirían": "resarcir",
    "resarcirías": "resarcir",
    "resarciste": "resarcir",
    "resarcisteis": "resarcir",
    "resarciéramos": "resarcir",
    "resarciéremos": "resarcir",
    "resarciésemos": "resarcir",
    "resarció": "resarcir",
    "resarcí": "resarcir",
    "resarcía": "resarcir",
    "resarcíais": "resarcir",
    "resarcíamos": "resarcir",
    "resarcían": "resarcir",
    "resarcías": "resarcir",
    "resarcís": "resarcir",
    "resarza": "resarcir",
    "resarzamos": "resarcir",
    "resarzan": "resarcir",
    "resarzas": "resarcir",
    "resarzo": "resarcir",
    "resarzáis": "resarcir",
    "resbala": "resbalar",
    "resbalaba": "resbalar",
    "resbalabais": "resbalar",
    "resbalaban": "resbalar",
    "resbalabas": "resbalar",
    "resbalad": "resbalar",
    "resbalada": "resbalar",
    "resbaladas": "resbalar",
    "resbaladera": "resbaladero",
    "resbaladeras": "resbaladero",
    "resbaladeros": "resbaladero",
    "resbaladiza": "resbaladizo",
    "resbaladizas": "resbaladizo",
    "resbaladizos": "resbaladizo",
    "resbalado": "resbalar",
    "resbaladora": "resbalador",
    "resbaladoras": "resbalador",
    "resbaladores": "resbalador",
    "resbalados": "resbalar",
    "resbaladuras": "resbaladura",
    "resbalamientos": "resbalamiento",
    "resbalamos": "resbalar",
    "resbalan": "resbalar",
    "resbalando": "resbalar",
    "resbalara": "resbalar",
    "resbalarais": "resbalar",
    "resbalaran": "resbalar",
    "resbalaras": "resbalar",
    "resbalare": "resbalar",
    "resbalareis": "resbalar",
    "resbalaremos": "resbalar",
    "resbalaren": "resbalar",
    "resbalares": "resbalar",
    "resbalaron": "resbalar",
    "resbalará": "resbalar",
    "resbalarán": "resbalar",
    "resbalarás": "resbalar",
    "resbalaré": "resbalar",
    "resbalaréis": "resbalar",
    "resbalaría": "resbalar",
    "resbalaríais": "resbalar",
    "resbalaríamos": "resbalar",
    "resbalarían": "resbalar",
    "resbalarías": "resbalar",
    "resbalas": "resbalar",
    "resbalase": "resbalar",
    "resbalaseis": "resbalar",
    "resbalasen": "resbalar",
    "resbalases": "resbalar",
    "resbalaste": "resbalar",
    "resbalasteis": "resbalar",
    "resbale": "resbalar",
    "resbalemos": "resbalar",
    "resbalen": "resbalar",
    "resbales": "resbalar",
    "resbalo": "resbalar",
    "resbalones": "resbalón",
    "resbalosa": "resbaloso",
    "resbalosas": "resbaloso",
    "resbalosos": "resbaloso",
    "resbalábamos": "resbalar",
    "resbaláis": "resbalar",
    "resbaláramos": "resbalar",
    "resbaláremos": "resbalar",
    "resbalásemos": "resbalar",
    "resbalé": "resbalar",
    "resbaléis": "resbalar",
    "resbaló": "resbalar",
    "rescalda": "rescaldar",
    "rescaldaba": "rescaldar",
    "rescaldabais": "rescaldar",
    "rescaldaban": "rescaldar",
    "rescaldabas": "rescaldar",
    "rescaldad": "rescaldar",
    "rescaldada": "rescaldar",
    "rescaldadas": "rescaldar",
    "rescaldado": "rescaldar",
    "rescaldados": "rescaldar",
    "rescaldamos": "rescaldar",
    "rescaldan": "rescaldar",
    "rescaldando": "rescaldar",
    "rescaldara": "rescaldar",
    "rescaldarais": "rescaldar",
    "rescaldaran": "rescaldar",
    "rescaldaras": "rescaldar",
    "rescaldare": "rescaldar",
    "rescaldareis": "rescaldar",
    "rescaldaremos": "rescaldar",
    "rescaldaren": "rescaldar",
    "rescaldares": "rescaldar",
    "rescaldaron": "rescaldar",
    "rescaldará": "rescaldar",
    "rescaldarán": "rescaldar",
    "rescaldarás": "rescaldar",
    "rescaldaré": "rescaldar",
    "rescaldaréis": "rescaldar",
    "rescaldaría": "rescaldar",
    "rescaldaríais": "rescaldar",
    "rescaldaríamos": "rescaldar",
    "rescaldarían": "rescaldar",
    "rescaldarías": "rescaldar",
    "rescaldas": "rescaldar",
    "rescaldase": "rescaldar",
    "rescaldaseis": "rescaldar",
    "rescaldasen": "rescaldar",
    "rescaldases": "rescaldar",
    "rescaldaste": "rescaldar",
    "rescaldasteis": "rescaldar",
    "rescalde": "rescaldar",
    "rescaldemos": "rescaldar",
    "rescalden": "rescaldar",
    "rescaldes": "rescaldar",
    "rescaldo": "rescaldar",
    "rescaldábamos": "rescaldar",
    "rescaldáis": "rescaldar",
    "rescaldáramos": "rescaldar",
    "rescaldáremos": "rescaldar",
    "rescaldásemos": "rescaldar",
    "rescaldé": "rescaldar",
    "rescaldéis": "rescaldar",
    "rescaldó": "rescaldar",
    "rescata": "rescatar",
    "rescataba": "rescatar",
    "rescatabais": "rescatar",
    "rescataban": "rescatar",
    "rescatabas": "rescatar",
    "rescatables": "rescatable",
    "rescatad": "rescatar",
    "rescatada": "rescatar",
    "rescatadas": "rescatar",
    "rescatado": "rescatar",
    "rescatadora": "rescatador",
    "rescatadoras": "rescatador",
    "rescatadores": "rescatador",
    "rescatados": "rescatar",
    "rescatamos": "rescatar",
    "rescatan": "rescatar",
    "rescatando": "rescatar",
    "rescatara": "rescatar",
    "rescatarais": "rescatar",
    "rescataran": "rescatar",
    "rescataras": "rescatar",
    "rescatare": "rescatar",
    "rescatareis": "rescatar",
    "rescataremos": "rescatar",
    "rescataren": "rescatar",
    "rescatares": "rescatar",
    "rescataron": "rescatar",
    "rescatará": "rescatar",
    "rescatarán": "rescatar",
    "rescatarás": "rescatar",
    "rescataré": "rescatar",
    "rescataréis": "rescatar",
    "rescataría": "rescatar",
    "rescataríais": "rescatar",
    "rescataríamos": "rescatar",
    "rescatarían": "rescatar",
    "rescatarías": "rescatar",
    "rescatas": "rescatar",
    "rescatase": "rescatar",
    "rescataseis": "rescatar",
    "rescatasen": "rescatar",
    "rescatases": "rescatar",
    "rescataste": "rescatar",
    "rescatasteis": "rescatar",
    "rescate": "rescatar",
    "rescatemos": "rescatar",
    "rescaten": "rescatar",
    "rescates": "rescate",
    "rescato": "rescatar",
    "rescatábamos": "rescatar",
    "rescatáis": "rescatar",
    "rescatáramos": "rescatar",
    "rescatáremos": "rescatar",
    "rescatásemos": "rescatar",
    "rescaté": "rescatar",
    "rescatéis": "rescatar",
    "rescató": "rescatar",
    "rescaños": "rescaño",
    "rescinaminas": "rescinamina",
    "rescinda": "rescindir",
    "rescindamos": "rescindir",
    "rescindan": "rescindir",
    "rescindas": "rescindir",
    "rescinde": "rescindir",
    "rescinden": "rescindir",
    "rescindes": "rescindir",
    "rescindibles": "rescindible",
    "rescindid": "rescindir",
    "rescindida": "rescindir",
    "rescindidas": "rescindir",
    "rescindido": "rescindir",
    "rescindidos": "rescindir",
    "rescindiendo": "rescindir",
    "rescindiera": "rescindir",
    "rescindierais": "rescindir",
    "rescindieran": "rescindir",
    "rescindieras": "rescindir",
    "rescindiere": "rescindir",
    "rescindiereis": "rescindir",
    "rescindieren": "rescindir",
    "rescindieres": "rescindir",
    "rescindieron": "rescindir",
    "rescindiese": "rescindir",
    "rescindieseis": "rescindir",
    "rescindiesen": "rescindir",
    "rescindieses": "rescindir",
    "rescindimos": "rescindir",
    "rescindiremos": "rescindir",
    "rescindirá": "rescindir",
    "rescindirán": "rescindir",
    "rescindirás": "rescindir",
    "rescindiré": "rescindir",
    "rescindiréis": "rescindir",
    "rescindiría": "rescindir",
    "rescindiríais": "rescindir",
    "rescindiríamos": "rescindir",
    "rescindirían": "rescindir",
    "rescindirías": "rescindir",
    "rescindiste": "rescindir",
    "rescindisteis": "rescindir",
    "rescindiéramos": "rescindir",
    "rescindiéremos": "rescindir",
    "rescindiésemos": "rescindir",
    "rescindió": "rescindir",
    "rescindo": "rescindir",
    "rescindáis": "rescindir",
    "rescindí": "rescindir",
    "rescindía": "rescindir",
    "rescindíais": "rescindir",
    "rescindíamos": "rescindir",
    "rescindían": "rescindir",
    "rescindías": "rescindir",
    "rescindís": "rescindir",
    "rescisiones": "rescisión",
    "rescisoria": "rescisorio",
    "rescisorias": "rescisorio",
    "rescisorios": "rescisorio",
    "rescolderas": "rescoldera",
    "rescoldos": "rescoldo",
    "rescriba": "rescribir",
    "rescribamos": "rescribir",
    "rescriban": "rescribir",
    "rescribas": "rescribir",
    "rescribe": "rescribir",
    "rescriben": "rescribir",
    "rescribes": "rescribir",
    "rescribid": "rescribir",
    "rescribiendo": "rescribir",
    "rescribiera": "rescribir",
    "rescribierais": "rescribir",
    "rescribieran": "rescribir",
    "rescribieras": "rescribir",
    "rescribiere": "rescribir",
    "rescribiereis": "rescribir",
    "rescribieren": "rescribir",
    "rescribieres": "rescribir",
    "rescribieron": "rescribir",
    "rescribiese": "rescribir",
    "rescribieseis": "rescribir",
    "rescribiesen": "rescribir",
    "rescribieses": "rescribir",
    "rescribimos": "rescribir",
    "rescribiremos": "rescribir",
    "rescribirá": "rescribir",
    "rescribirán": "rescribir",
    "rescribirás": "rescribir",
    "rescribiré": "rescribir",
    "rescribiréis": "rescribir",
    "rescribiría": "rescribir",
    "rescribiríais": "rescribir",
    "rescribiríamos": "rescribir",
    "rescribirían": "rescribir",
    "rescribirías": "rescribir",
    "rescribiste": "rescribir",
    "rescribisteis": "rescribir",
    "rescribiéramos": "rescribir",
    "rescribiéremos": "rescribir",
    "rescribiésemos": "rescribir",
    "rescribió": "rescribir",
    "rescribo": "rescribir",
    "rescribáis": "rescribir",
    "rescribí": "rescribir",
    "rescribía": "rescribir",
    "rescribíais": "rescribir",
    "rescribíamos": "rescribir",
    "rescribían": "rescribir",
    "rescribías": "rescribir",
    "rescribís": "rescribir",
    "rescriptoria": "rescriptorio",
    "rescriptorias": "rescriptorio",
    "rescriptorios": "rescriptorio",
    "rescriptos": "rescripto",
    "rescrita": "rescribir",
    "rescritas": "rescribir",
    "rescrito": "rescribir",
    "rescritos": "rescribir",
    "rescuentros": "rescuentro",
    "reseca": "reseco",
    "resecaba": "resecar",
    "resecabais": "resecar",
    "resecaban": "resecar",
    "resecabas": "resecar",
    "resecaciones": "resecación",
    "resecad": "resecar",
    "resecada": "resecar",
    "resecadas": "resecar",
    "resecado": "resecar",
    "resecados": "resecar",
    "resecamos": "resecar",
    "resecan": "resecar",
    "resecando": "resecar",
    "resecara": "resecar",
    "resecarais": "resecar",
    "resecaran": "resecar",
    "resecaras": "resecar",
    "resecare": "resecar",
    "resecareis": "resecar",
    "resecaremos": "resecar",
    "resecaren": "resecar",
    "resecares": "resecar",
    "resecaron": "resecar",
    "resecará": "resecar",
    "resecarán": "resecar",
    "resecarás": "resecar",
    "resecaré": "resecar",
    "resecaréis": "resecar",
    "resecaría": "resecar",
    "resecaríais": "resecar",
    "resecaríamos": "resecar",
    "resecarían": "resecar",
    "resecarías": "resecar",
    "resecas": "reseco",
    "resecase": "resecar",
    "resecaseis": "resecar",
    "resecasen": "resecar",
    "resecases": "resecar",
    "resecaste": "resecar",
    "resecasteis": "resecar",
    "resecciones": "resección",
    "reseco": "resecar",
    "resecos": "reseco",
    "resecuencias": "resecuencia",
    "resecábamos": "resecar",
    "resecáis": "resecar",
    "resecáramos": "resecar",
    "resecáremos": "resecar",
    "resecásemos": "resecar",
    "resecó": "resecar",
    "resedas": "reseda",
    "resedimentaciones": "resedimentación",
    "resedácea": "resedáceo",
    "resedáceas": "resedáceo",
    "resedáceos": "resedáceo",
    "resegaba": "resegar",
    "resegabais": "resegar",
    "resegaban": "resegar",
    "resegabas": "resegar",
    "resegad": "resegar",
    "resegada": "resegar",
    "resegadas": "resegar",
    "resegado": "resegar",
    "resegados": "resegar",
    "resegamos": "resegar",
    "resegando": "resegar",
    "resegara": "resegar",
    "resegarais": "resegar",
    "resegaran": "resegar",
    "resegaras": "resegar",
    "resegare": "resegar",
    "resegareis": "resegar",
    "resegaremos": "resegar",
    "resegaren": "resegar",
    "resegares": "resegar",
    "resegaron": "resegar",
    "resegará": "resegar",
    "resegarán": "resegar",
    "resegarás": "resegar",
    "resegaré": "resegar",
    "resegaréis": "resegar",
    "resegaría": "resegar",
    "resegaríais": "resegar",
    "resegaríamos": "resegar",
    "resegarían": "resegar",
    "resegarías": "resegar",
    "resegase": "resegar",
    "resegaseis": "resegar",
    "resegasen": "resegar",
    "resegases": "resegar",
    "resegaste": "resegar",
    "resegasteis": "resegar",
    "reseguemos": "resegar",
    "reseguid": "reseguir",
    "reseguida": "reseguir",
    "reseguidas": "reseguir",
    "reseguido": "reseguir",
    "reseguidos": "reseguir",
    "reseguimos": "reseguir",
    "reseguiremos": "reseguir",
    "reseguirá": "reseguir",
    "reseguirán": "reseguir",
    "reseguirás": "reseguir",
    "reseguiré": "reseguir",
    "reseguiréis": "reseguir",
    "reseguiría": "reseguir",
    "reseguiríais": "reseguir",
    "reseguiríamos": "reseguir",
    "reseguirían": "reseguir",
    "reseguirías": "reseguir",
    "reseguiste": "reseguir",
    "reseguisteis": "reseguir",
    "resegué": "resegar",
    "reseguéis": "resegar",
    "reseguí": "reseguir",
    "reseguía": "reseguir",
    "reseguíais": "reseguir",
    "reseguíamos": "reseguir",
    "reseguían": "reseguir",
    "reseguías": "reseguir",
    "reseguís": "reseguir",
    "resegábamos": "resegar",
    "resegáis": "resegar",
    "resegáramos": "resegar",
    "resegáremos": "resegar",
    "resegásemos": "resegar",
    "resegó": "resegar",
    "resella": "resellar",
    "resellaba": "resellar",
    "resellabais": "resellar",
    "resellaban": "resellar",
    "resellabas": "resellar",
    "resellad": "resellar",
    "resellada": "resellar",
    "reselladas": "resellar",
    "resellado": "resellar",
    "resellados": "resellar",
    "resellamos": "resellar",
    "resellan": "resellar",
    "resellando": "resellar",
    "resellara": "resellar",
    "resellarais": "resellar",
    "resellaran": "resellar",
    "resellaras": "resellar",
    "resellare": "resellar",
    "resellareis": "resellar",
    "resellaremos": "resellar",
    "resellaren": "resellar",
    "resellares": "resellar",
    "resellaron": "resellar",
    "resellará": "resellar",
    "resellarán": "resellar",
    "resellarás": "resellar",
    "resellaré": "resellar",
    "resellaréis": "resellar",
    "resellaría": "resellar",
    "resellaríais": "resellar",
    "resellaríamos": "resellar",
    "resellarían": "resellar",
    "resellarías": "resellar",
    "resellas": "resellar",
    "resellase": "resellar",
    "resellaseis": "resellar",
    "resellasen": "resellar",
    "resellases": "resellar",
    "resellaste": "resellar",
    "resellasteis": "resellar",
    "reselle": "resellar",
    "resellemos": "resellar",
    "resellen": "resellar",
    "reselles": "resellar",
    "resello": "resellar",
    "resellos": "resello",
    "resellábamos": "resellar",
    "reselláis": "resellar",
    "reselláramos": "resellar",
    "reselláremos": "resellar",
    "resellásemos": "resellar",
    "resellé": "resellar",
    "reselléis": "resellar",
    "reselló": "resellar",
    "resembraba": "resembrar",
    "resembrabais": "resembrar",
    "resembraban": "resembrar",
    "resembrabas": "resembrar",
    "resembrad": "resembrar",
    "resembrada": "resembrar",
    "resembradas": "resembrar",
    "resembrado": "resembrar",
    "resembrados": "resembrar",
    "resembramos": "resembrar",
    "resembrando": "resembrar",
    "resembrara": "resembrar",
    "resembrarais": "resembrar",
    "resembraran": "resembrar",
    "resembraras": "resembrar",
    "resembrare": "resembrar",
    "resembrareis": "resembrar",
    "resembraremos": "resembrar",
    "resembraren": "resembrar",
    "resembrares": "resembrar",
    "resembraron": "resembrar",
    "resembrará": "resembrar",
    "resembrarán": "resembrar",
    "resembrarás": "resembrar",
    "resembraré": "resembrar",
    "resembraréis": "resembrar",
    "resembraría": "resembrar",
    "resembraríais": "resembrar",
    "resembraríamos": "resembrar",
    "resembrarían": "resembrar",
    "resembrarías": "resembrar",
    "resembrase": "resembrar",
    "resembraseis": "resembrar",
    "resembrasen": "resembrar",
    "resembrases": "resembrar",
    "resembraste": "resembrar",
    "resembrasteis": "resembrar",
    "resembremos": "resembrar",
    "resembrábamos": "resembrar",
    "resembráis": "resembrar",
    "resembráramos": "resembrar",
    "resembráremos": "resembrar",
    "resembrásemos": "resembrar",
    "resembré": "resembrar",
    "resembréis": "resembrar",
    "resembró": "resembrar",
    "resentid": "resentir",
    "resentida": "resentir",
    "resentidas": "resentir",
    "resentido": "resentir",
    "resentidos": "resentir",
    "resentimientos": "resentimiento",
    "resentimos": "resentir",
    "resentiremos": "resentir",
    "resentirá": "resentir",
    "resentirán": "resentir",
    "resentirás": "resentir",
    "resentiré": "resentir",
    "resentiréis": "resentir",
    "resentiría": "resentir",
    "resentiríais": "resentir",
    "resentiríamos": "resentir",
    "resentirían": "resentir",
    "resentirías": "resentir",
    "resentiste": "resentir",
    "resentisteis": "resentir",
    "resentí": "resentir",
    "resentía": "resentir",
    "resentíais": "resentir",
    "resentíamos": "resentir",
    "resentían": "resentir",
    "resentías": "resentir",
    "resentís": "resentir",
    "reseque": "resecar",
    "resequemos": "resecar",
    "resequen": "resecar",
    "reseques": "resecar",
    "resequida": "resequido",
    "resequidas": "resequido",
    "resequidos": "resequido",
    "resequé": "resecar",
    "resequéis": "resecar",
    "resera": "resero",
    "reseras": "resero",
    "reseros": "resero",
    "reserpinas": "reserpina",
    "reserva": "reservar",
    "reservaba": "reservar",
    "reservabais": "reservar",
    "reservaban": "reservar",
    "reservabas": "reservar",
    "reservables": "reservable",
    "reservaciones": "reservación",
    "reservad": "reservar",
    "reservada": "reservar",
    "reservadas": "reservar",
    "reservado": "reservar",
    "reservados": "reservar",
    "reservamos": "reservar",
    "reservan": "reservar",
    "reservando": "reservar",
    "reservara": "reservar",
    "reservarais": "reservar",
    "reservaran": "reservar",
    "reservaras": "reservar",
    "reservare": "reservar",
    "reservareis": "reservar",
    "reservaremos": "reservar",
    "reservaren": "reservar",
    "reservares": "reservar",
    "reservaron": "reservar",
    "reservará": "reservar",
    "reservarán": "reservar",
    "reservarás": "reservar",
    "reservaré": "reservar",
    "reservaréis": "reservar",
    "reservaría": "reservar",
    "reservaríais": "reservar",
    "reservaríamos": "reservar",
    "reservarían": "reservar",
    "reservarías": "reservar",
    "reservas": "reservar",
    "reservase": "reservar",
    "reservaseis": "reservar",
    "reservasen": "reservar",
    "reservases": "reservar",
    "reservaste": "reservar",
    "reservasteis": "reservar",
    "reservativa": "reservativo",
    "reservativas": "reservativo",
    "reservativos": "reservativo",
    "reserve": "reservar",
    "reservemos": "reservar",
    "reserven": "reservar",
    "reserves": "reservar",
    "reservistas": "reservista",
    "reservo": "reservar",
    "reservona": "reservón",
    "reservonas": "reservón",
    "reservones": "reservón",
    "reservorios": "reservorio",
    "reservábamos": "reservar",
    "reserváis": "reservar",
    "reserváramos": "reservar",
    "reserváremos": "reservar",
    "reservásemos": "reservar",
    "reservé": "reservar",
    "reservéis": "reservar",
    "reservó": "reservar",
    "reses": "res",
    "reseña": "reseñar",
    "reseñaba": "reseñar",
    "reseñabais": "reseñar",
    "reseñaban": "reseñar",
    "reseñabas": "reseñar",
    "reseñables": "reseñable",
    "reseñad": "reseñar",
    "reseñada": "reseñar",
    "reseñadas": "reseñar",
    "reseñado": "reseñar",
    "reseñados": "reseñar",
    "reseñamos": "reseñar",
    "reseñan": "reseñar",
    "reseñando": "reseñar",
    "reseñara": "reseñar",
    "reseñarais": "reseñar",
    "reseñaran": "reseñar",
    "reseñaras": "reseñar",
    "reseñare": "reseñar",
    "reseñareis": "reseñar",
    "reseñaremos": "reseñar",
    "reseñaren": "reseñar",
    "reseñares": "reseñar",
    "reseñaron": "reseñar",
    "reseñará": "reseñar",
    "reseñarán": "reseñar",
    "reseñarás": "reseñar",
    "reseñaré": "reseñar",
    "reseñaréis": "reseñar",
    "reseñaría": "reseñar",
    "reseñaríais": "reseñar",
    "reseñaríamos": "reseñar",
    "reseñarían": "reseñar",
    "reseñarías": "reseñar",
    "reseñas": "reseñar",
    "reseñase": "reseñar",
    "reseñaseis": "reseñar",
    "reseñasen": "reseñar",
    "reseñases": "reseñar",
    "reseñaste": "reseñar",
    "reseñasteis": "reseñar",
    "reseñe": "reseñar",
    "reseñemos": "reseñar",
    "reseñen": "reseñar",
    "reseñes": "reseñar",
    "reseño": "reseñar",
    "reseñábamos": "reseñar",
    "reseñáis": "reseñar",
    "reseñáramos": "reseñar",
    "reseñáremos": "reseñar",
    "reseñásemos": "reseñar",
    "reseñé": "reseñar",
    "reseñéis": "reseñar",
    "reseñó": "reseñar",
    "resfriaba": "resfriar",
    "resfriabais": "resfriar",
    "resfriaban": "resfriar",
    "resfriabas": "resfriar",
    "resfriad": "resfriar",
    "resfriada": "resfriar",
    "resfriadas": "resfriar",
    "resfriado": "resfriar",
    "resfriados": "resfriar",
    "resfriaduras": "resfriadura",
    "resfriamientos": "resfriamiento",
    "resfriamos": "resfriar",
    "resfriando": "resfriar",
    "resfriantes": "resfriante",
    "resfriara": "resfriar",
    "resfriarais": "resfriar",
    "resfriaran": "resfriar",
    "resfriaras": "resfriar",
    "resfriare": "resfriar",
    "resfriareis": "resfriar",
    "resfriaremos": "resfriar",
    "resfriaren": "resfriar",
    "resfriares": "resfriar",
    "resfriaron": "resfriar",
    "resfriará": "resfriar",
    "resfriarán": "resfriar",
    "resfriarás": "resfriar",
    "resfriaré": "resfriar",
    "resfriaréis": "resfriar",
    "resfriaría": "resfriar",
    "resfriaríais": "resfriar",
    "resfriaríamos": "resfriar",
    "resfriarían": "resfriar",
    "resfriarías": "resfriar",
    "resfriase": "resfriar",
    "resfriaseis": "resfriar",
    "resfriasen": "resfriar",
    "resfriases": "resfriar",
    "resfriaste": "resfriar",
    "resfriasteis": "resfriar",
    "resfriemos": "resfriar",
    "resfriábamos": "resfriar",
    "resfriáis": "resfriar",
    "resfriáramos": "resfriar",
    "resfriáremos": "resfriar",
    "resfriásemos": "resfriar",
    "resfrié": "resfriar",
    "resfriéis": "resfriar",
    "resfrió": "resfriar",
    "resfría": "resfriar",
    "resfrían": "resfriar",
    "resfrías": "resfriar",
    "resfríe": "resfriar",
    "resfríen": "resfriar",
    "resfríes": "resfriar",
    "resfrío": "resfriar",
    "resfríos": "resfrío",
    "resguarda": "resguardar",
    "resguardaba": "resguardar",
    "resguardabais": "resguardar",
    "resguardaban": "resguardar",
    "resguardabas": "resguardar",
    "resguardad": "resguardar",
    "resguardada": "resguardar",
    "resguardadas": "resguardar",
    "resguardado": "resguardar",
    "resguardados": "resguardar",
    "resguardamos": "resguardar",
    "resguardan": "resguardar",
    "resguardando": "resguardar",
    "resguardara": "resguardar",
    "resguardarais": "resguardar",
    "resguardaran": "resguardar",
    "resguardaras": "resguardar",
    "resguardare": "resguardar",
    "resguardareis": "resguardar",
    "resguardaremos": "resguardar",
    "resguardaren": "resguardar",
    "resguardares": "resguardar",
    "resguardaron": "resguardar",
    "resguardará": "resguardar",
    "resguardarán": "resguardar",
    "resguardarás": "resguardar",
    "resguardaré": "resguardar",
    "resguardaréis": "resguardar",
    "resguardaría": "resguardar",
    "resguardaríais": "resguardar",
    "resguardaríamos": "resguardar",
    "resguardarían": "resguardar",
    "resguardarías": "resguardar",
    "resguardas": "resguardar",
    "resguardase": "resguardar",
    "resguardaseis": "resguardar",
    "resguardasen": "resguardar",
    "resguardases": "resguardar",
    "resguardaste": "resguardar",
    "resguardasteis": "resguardar",
    "resguarde": "resguardar",
    "resguardemos": "resguardar",
    "resguarden": "resguardar",
    "resguardes": "resguardar",
    "resguardo": "resguardar",
    "resguardos": "resguardo",
    "resguardábamos": "resguardar",
    "resguardáis": "resguardar",
    "resguardáramos": "resguardar",
    "resguardáremos": "resguardar",
    "resguardásemos": "resguardar",
    "resguardé": "resguardar",
    "resguardéis": "resguardar",
    "resguardó": "resguardar",
    "resida": "residir",
    "residamos": "residir",
    "residan": "residir",
    "residas": "residir",
    "reside": "residir",
    "residen": "residir",
    "residencia": "residenciar",
    "residenciaba": "residenciar",
    "residenciabais": "residenciar",
    "residenciaban": "residenciar",
    "residenciabas": "residenciar",
    "residenciad": "residenciar",
    "residenciada": "residenciar",
    "residenciadas": "residenciar",
    "residenciado": "residenciar",
    "residenciados": "residenciar",
    "residenciales": "residencial",
    "residenciamos": "residenciar",
    "residencian": "residenciar",
    "residenciando": "residenciar",
    "residenciara": "residenciar",
    "residenciarais": "residenciar",
    "residenciaran": "residenciar",
    "residenciaras": "residenciar",
    "residenciare": "residenciar",
    "residenciareis": "residenciar",
    "residenciaremos": "residenciar",
    "residenciaren": "residenciar",
    "residenciares": "residenciar",
    "residenciaron": "residenciar",
    "residenciará": "residenciar",
    "residenciarán": "residenciar",
    "residenciarás": "residenciar",
    "residenciaré": "residenciar",
    "residenciaréis": "residenciar",
    "residenciaría": "residenciar",
    "residenciaríais": "residenciar",
    "residenciaríamos": "residenciar",
    "residenciarían": "residenciar",
    "residenciarías": "residenciar",
    "residencias": "residenciar",
    "residenciase": "residenciar",
    "residenciaseis": "residenciar",
    "residenciasen": "residenciar",
    "residenciases": "residenciar",
    "residenciaste": "residenciar",
    "residenciasteis": "residenciar",
    "residencie": "residenciar",
    "residenciemos": "residenciar",
    "residencien": "residenciar",
    "residencies": "residenciar",
    "residencio": "residenciar",
    "residenciábamos": "residenciar",
    "residenciáis": "residenciar",
    "residenciáramos": "residenciar",
    "residenciáremos": "residenciar",
    "residenciásemos": "residenciar",
    "residencié": "residenciar",
    "residenciéis": "residenciar",
    "residenció": "residenciar",
    "residentes": "residente",
    "resides": "residir",
    "residid": "residir",
    "residida": "residir",
    "resididas": "residir",
    "residido": "residir",
    "resididos": "residir",
    "residiendo": "residir",
    "residiera": "residir",
    "residierais": "residir",
    "residieran": "residir",
    "residieras": "residir",
    "residiere": "residir",
    "residiereis": "residir",
    "residieren": "residir",
    "residieres": "residir",
    "residieron": "residir",
    "residiese": "residir",
    "residieseis": "residir",
    "residiesen": "residir",
    "residieses": "residir",
    "residimos": "residir",
    "residiremos": "residir",
    "residirá": "residir",
    "residirán": "residir",
    "residirás": "residir",
    "residiré": "residir",
    "residiréis": "residir",
    "residiría": "residir",
    "residiríais": "residir",
    "residiríamos": "residir",
    "residirían": "residir",
    "residirías": "residir",
    "residiste": "residir",
    "residisteis": "residir",
    "residiéramos": "residir",
    "residiéremos": "residir",
    "residiésemos": "residir",
    "residió": "residir",
    "resido": "residir",
    "residuales": "residual",
    "residuaria": "residuario",
    "residuarias": "residuario",
    "residuarios": "residuario",
    "residuos": "residuo",
    "residáis": "residir",
    "residí": "residir",
    "residía": "residir",
    "residíais": "residir",
    "residíamos": "residir",
    "residían": "residir",
    "residías": "residir",
    "residís": "residir",
    "resiega": "resegar",
    "resiegan": "resegar",
    "resiegas": "resegar",
    "resiego": "resegar",
    "resiegue": "resegar",
    "resieguen": "resegar",
    "resiegues": "resegar",
    "resiembra": "resembrar",
    "resiembran": "resembrar",
    "resiembras": "resiembra",
    "resiembre": "resembrar",
    "resiembren": "resembrar",
    "resiembres": "resembrar",
    "resiembro": "resembrar",
    "resienta": "resentir",
    "resientan": "resentir",
    "resientas": "resentir",
    "resiente": "resentir",
    "resienten": "resentir",
    "resientes": "resentir",
    "resiento": "resentir",
    "resiga": "reseguir",
    "resigamos": "reseguir",
    "resigan": "reseguir",
    "resigas": "reseguir",
    "resigna": "resignar",
    "resignaba": "resignar",
    "resignabais": "resignar",
    "resignaban": "resignar",
    "resignabas": "resignar",
    "resignaciones": "resignación",
    "resignad": "resignar",
    "resignada": "resignar",
    "resignadas": "resignar",
    "resignado": "resignar",
    "resignados": "resignar",
    "resignamos": "resignar",
    "resignan": "resignar",
    "resignando": "resignar",
    "resignara": "resignar",
    "resignarais": "resignar",
    "resignaran": "resignar",
    "resignaras": "resignar",
    "resignare": "resignar",
    "resignareis": "resignar",
    "resignaremos": "resignar",
    "resignaren": "resignar",
    "resignares": "resignar",
    "resignaron": "resignar",
    "resignará": "resignar",
    "resignarán": "resignar",
    "resignarás": "resignar",
    "resignaré": "resignar",
    "resignaréis": "resignar",
    "resignaría": "resignar",
    "resignaríais": "resignar",
    "resignaríamos": "resignar",
    "resignarían": "resignar",
    "resignarías": "resignar",
    "resignas": "resignar",
    "resignase": "resignar",
    "resignaseis": "resignar",
    "resignasen": "resignar",
    "resignases": "resignar",
    "resignaste": "resignar",
    "resignasteis": "resignar",
    "resignatarios": "resignatario",
    "resigne": "resignar",
    "resignemos": "resignar",
    "resignen": "resignar",
    "resignes": "resignar",
    "resigno": "resignar",
    "resignábamos": "resignar",
    "resignáis": "resignar",
    "resignáramos": "resignar",
    "resignáremos": "resignar",
    "resignásemos": "resignar",
    "resigné": "resignar",
    "resignéis": "resignar",
    "resignó": "resignar",
    "resigo": "reseguir",
    "resigue": "reseguir",
    "resiguen": "reseguir",
    "resigues": "reseguir",
    "resiguiendo": "reseguir",
    "resiguiera": "reseguir",
    "resiguierais": "reseguir",
    "resiguieran": "reseguir",
    "resiguieras": "reseguir",
    "resiguiere": "reseguir",
    "resiguiereis": "reseguir",
    "resiguieren": "reseguir",
    "resiguieres": "reseguir",
    "resiguieron": "reseguir",
    "resiguiese": "reseguir",
    "resiguieseis": "reseguir",
    "resiguiesen": "reseguir",
    "resiguieses": "reseguir",
    "resiguiéramos": "reseguir",
    "resiguiéremos": "reseguir",
    "resiguiésemos": "reseguir",
    "resiguió": "reseguir",
    "resigáis": "reseguir",
    "resiliencias": "resiliencia",
    "resina": "resinar",
    "resinaba": "resinar",
    "resinabais": "resinar",
    "resinaban": "resinar",
    "resinabas": "resinar",
    "resinaciones": "resinación",
    "resinad": "resinar",
    "resinada": "resinar",
    "resinadas": "resinar",
    "resinado": "resinar",
    "resinados": "resinar",
    "resinamos": "resinar",
    "resinan": "resinar",
    "resinando": "resinar",
    "resinara": "resinar",
    "resinarais": "resinar",
    "resinaran": "resinar",
    "resinaras": "resinar",
    "resinare": "resinar",
    "resinareis": "resinar",
    "resinaremos": "resinar",
    "resinaren": "resinar",
    "resinares": "resinar",
    "resinaron": "resinar",
    "resinará": "resinar",
    "resinarán": "resinar",
    "resinarás": "resinar",
    "resinaré": "resinar",
    "resinaréis": "resinar",
    "resinaría": "resinar",
    "resinaríais": "resinar",
    "resinaríamos": "resinar",
    "resinarían": "resinar",
    "resinarías": "resinar",
    "resinas": "resinar",
    "resinase": "resinar",
    "resinaseis": "resinar",
    "resinasen": "resinar",
    "resinases": "resinar",
    "resinaste": "resinar",
    "resinasteis": "resinar",
    "resinatos": "resinato",
    "resincronizaciones": "resincronización",
    "resine": "resinar",
    "resinemos": "resinar",
    "resinen": "resinar",
    "resinera": "resinero",
    "resineras": "resinero",
    "resineros": "resinero",
    "resines": "resinar",
    "resinifica": "resinificar",
    "resinificaba": "resinificar",
    "resinificabais": "resinificar",
    "resinificaban": "resinificar",
    "resinificabas": "resinificar",
    "resinificad": "resinificar",
    "resinificada": "resinificar",
    "resinificadas": "resinificar",
    "resinificado": "resinificar",
    "resinificados": "resinificar",
    "resinificamos": "resinificar",
    "resinifican": "resinificar",
    "resinificando": "resinificar",
    "resinificara": "resinificar",
    "resinificarais": "resinificar",
    "resinificaran": "resinificar",
    "resinificaras": "resinificar",
    "resinificare": "resinificar",
    "resinificareis": "resinificar",
    "resinificaremos": "resinificar",
    "resinificaren": "resinificar",
    "resinificares": "resinificar",
    "resinificaron": "resinificar",
    "resinificará": "resinificar",
    "resinificarán": "resinificar",
    "resinificarás": "resinificar",
    "resinificaré": "resinificar",
    "resinificaréis": "resinificar",
    "resinificaría": "resinificar",
    "resinificaríais": "resinificar",
    "resinificaríamos": "resinificar",
    "resinificarían": "resinificar",
    "resinificarías": "resinificar",
    "resinificas": "resinificar",
    "resinificase": "resinificar",
    "resinificaseis": "resinificar",
    "resinificasen": "resinificar",
    "resinificases": "resinificar",
    "resinificaste": "resinificar",
    "resinificasteis": "resinificar",
    "resinifico": "resinificar",
    "resinificábamos": "resinificar",
    "resinificáis": "resinificar",
    "resinificáramos": "resinificar",
    "resinificáremos": "resinificar",
    "resinificásemos": "resinificar",
    "resinificó": "resinificar",
    "resinifique": "resinificar",
    "resinifiquemos": "resinificar",
    "resinifiquen": "resinificar",
    "resinifiques": "resinificar",
    "resinifiqué": "resinificar",
    "resinifiquéis": "resinificar",
    "resino": "resinar",
    "resinoles": "resinol",
    "resinosa": "resinoso",
    "resinosas": "resinoso",
    "resinosos": "resinoso",
    "resintamos": "resentir",
    "resintiendo": "resentir",
    "resintiera": "resentir",
    "resintierais": "resentir",
    "resintieran": "resentir",
    "resintieras": "resentir",
    "resintiere": "resentir",
    "resintiereis": "resentir",
    "resintieren": "resentir",
    "resintieres": "resentir",
    "resintieron": "resentir",
    "resintiese": "resentir",
    "resintieseis": "resentir",
    "resintiesen": "resentir",
    "resintieses": "resentir",
    "resintiéramos": "resentir",
    "resintiéremos": "resentir",
    "resintiésemos": "resentir",
    "resintió": "resentir",
    "resintonizaciones": "resintonización",
    "resintáis": "resentir",
    "resinábamos": "resinar",
    "resináis": "resinar",
    "resináramos": "resinar",
    "resináremos": "resinar",
    "resinásemos": "resinar",
    "resiné": "resinar",
    "resinéis": "resinar",
    "resinífera": "resinífero",
    "resiníferas": "resinífero",
    "resiníferos": "resinífero",
    "resinó": "resinar",
    "resisas": "resisa",
    "resista": "resistir",
    "resistamos": "resistir",
    "resistan": "resistir",
    "resistas": "resistir",
    "resiste": "resistir",
    "resisten": "resistir",
    "resistencialismos": "resistencialismo",
    "resistencialistas": "resistencialista",
    "resistencias": "resistencia",
    "resistentes": "resistente",
    "resisteros": "resistero",
    "resistes": "resistir",
    "resistibles": "resistible",
    "resistid": "resistir",
    "resistida": "resistir",
    "resistidas": "resistir",
    "resistideros": "resistidero",
    "resistido": "resistir",
    "resistidora": "resistidor",
    "resistidoras": "resistidor",
    "resistidores": "resistidor",
    "resistidos": "resistir",
    "resistiendo": "resistir",
    "resistiera": "resistir",
    "resistierais": "resistir",
    "resistieran": "resistir",
    "resistieras": "resistir",
    "resistiere": "resistir",
    "resistiereis": "resistir",
    "resistieren": "resistir",
    "resistieres": "resistir",
    "resistieron": "resistir",
    "resistiese": "resistir",
    "resistieseis": "resistir",
    "resistiesen": "resistir",
    "resistieses": "resistir",
    "resistimos": "resistir",
    "resistiremos": "resistir",
    "resistirá": "resistir",
    "resistirán": "resistir",
    "resistirás": "resistir",
    "resistiré": "resistir",
    "resistiréis": "resistir",
    "resistiría": "resistir",
    "resistiríais": "resistir",
    "resistiríamos": "resistir",
    "resistirían": "resistir",
    "resistirías": "resistir",
    "resististe": "resistir",
    "resististeis": "resistir",
    "resistiva": "resistivo",
    "resistivas": "resistivo",
    "resistividades": "resistividad",
    "resistivos": "resistivo",
    "resistivímetros": "resistivímetro",
    "resistiéramos": "resistir",
    "resistiéremos": "resistir",
    "resistiésemos": "resistir",
    "resistió": "resistir",
    "resisto": "resistir",
    "resistores": "resistor",
    "resistáis": "resistir",
    "resistí": "resistir",
    "resistía": "resistir",
    "resistíais": "resistir",
    "resistíamos": "resistir",
    "resistían": "resistir",
    "resistías": "resistir",
    "resistís": "resistir",
    "resitas": "resita",
    "resitoles": "resitol",
    "resmas": "resma",
    "resmillas": "resmilla",
    "resobada": "resobado",
    "resobadas": "resobado",
    "resobados": "resobado",
    "resobrina": "resobrino",
    "resobrinas": "resobrino",
    "resobrinos": "resobrino",
    "resolana": "resolano",
    "resolanas": "resolano",
    "resolanos": "resolano",
    "resoles": "resol",
    "resolidificaciones": "resolidificación",
    "resollaba": "resollar",
    "resollabais": "resollar",
    "resollaban": "resollar",
    "resollabas": "resollar",
    "resollad": "resollar",
    "resollada": "resollar",
    "resolladas": "resollar",
    "resollado": "resollar",
    "resollados": "resollar",
    "resollamos": "resollar",
    "resollando": "resollar",
    "resollara": "resollar",
    "resollarais": "resollar",
    "resollaran": "resollar",
    "resollaras": "resollar",
    "resollare": "resollar",
    "resollareis": "resollar",
    "resollaremos": "resollar",
    "resollaren": "resollar",
    "resollares": "resollar",
    "resollaron": "resollar",
    "resollará": "resollar",
    "resollarán": "resollar",
    "resollarás": "resollar",
    "resollaré": "resollar",
    "resollaréis": "resollar",
    "resollaría": "resollar",
    "resollaríais": "resollar",
    "resollaríamos": "resollar",
    "resollarían": "resollar",
    "resollarías": "resollar",
    "resollase": "resollar",
    "resollaseis": "resollar",
    "resollasen": "resollar",
    "resollases": "resollar",
    "resollaste": "resollar",
    "resollasteis": "resollar",
    "resollemos": "resollar",
    "resollábamos": "resollar",
    "resolláis": "resollar",
    "resolláramos": "resollar",
    "resolláremos": "resollar",
    "resollásemos": "resollar",
    "resollé": "resollar",
    "resolléis": "resollar",
    "resolló": "resollar",
    "resolubilidades": "resolubilidad",
    "resolubles": "resoluble",
    "resoluciones": "resolución",
    "resoluta": "resoluto",
    "resolutas": "resoluto",
    "resolutiva": "resolutivo",
    "resolutivas": "resolutivo",
    "resolutivos": "resolutivo",
    "resolutoria": "resolutorio",
    "resolutorias": "resolutorio",
    "resolutorios": "resolutorio",
    "resolutos": "resoluto",
    "resolvamos": "resolver",
    "resolved": "resolver",
    "resolvedores": "resolvedor",
    "resolvemos": "resolver",
    "resolventes": "resolvente",
    "resolveremos": "resolver",
    "resolverá": "resolver",
    "resolverán": "resolver",
    "resolverás": "resolver",
    "resolveré": "resolver",
    "resolveréis": "resolver",
    "resolvería": "resolver",
    "resolveríais": "resolver",
    "resolveríamos": "resolver",
    "resolverían": "resolver",
    "resolverías": "resolver",
    "resolviendo": "resolver",
    "resolviera": "resolver",
    "resolvierais": "resolver",
    "resolvieran": "resolver",
    "resolvieras": "resolver",
    "resolviere": "resolver",
    "resolviereis": "resolver",
    "resolvieren": "resolver",
    "resolvieres": "resolver",
    "resolvieron": "resolver",
    "resolviese": "resolver",
    "resolvieseis": "resolver",
    "resolviesen": "resolver",
    "resolvieses": "resolver",
    "resolvimos": "resolver",
    "resolviste": "resolver",
    "resolvisteis": "resolver",
    "resolviéramos": "resolver",
    "resolviéremos": "resolver",
    "resolviésemos": "resolver",
    "resolvió": "resolver",
    "resolváis": "resolver",
    "resolvéis": "resolver",
    "resolví": "resolver",
    "resolvía": "resolver",
    "resolvíais": "resolver",
    "resolvíamos": "resolver",
    "resolvían": "resolver",
    "resolvías": "resolver",
    "resonaba": "resonar",
    "resonabais": "resonar",
    "resonaban": "resonar",
    "resonabas": "resonar",
    "resonad": "resonar",
    "resonada": "resonar",
    "resonadas": "resonar",
    "resonado": "resonar",
    "resonadora": "resonador",
    "resonadoras": "resonador",
    "resonadores": "resonador",
    "resonados": "resonar",
    "resonamos": "resonar",
    "resonancias": "resonancia",
    "resonando": "resonar",
    "resonantes": "resonante",
    "resonara": "resonar",
    "resonarais": "resonar",
    "resonaran": "resonar",
    "resonaras": "resonar",
    "resonare": "resonar",
    "resonareis": "resonar",
    "resonaremos": "resonar",
    "resonaren": "resonar",
    "resonares": "resonar",
    "resonaron": "resonar",
    "resonará": "resonar",
    "resonarán": "resonar",
    "resonarás": "resonar",
    "resonaré": "resonar",
    "resonaréis": "resonar",
    "resonaría": "resonar",
    "resonaríais": "resonar",
    "resonaríamos": "resonar",
    "resonarían": "resonar",
    "resonarías": "resonar",
    "resonase": "resonar",
    "resonaseis": "resonar",
    "resonasen": "resonar",
    "resonases": "resonar",
    "resonaste": "resonar",
    "resonasteis": "resonar",
    "resonemos": "resonar",
    "resonábamos": "resonar",
    "resonáis": "resonar",
    "resonáramos": "resonar",
    "resonáremos": "resonar",
    "resonásemos": "resonar",
    "resoné": "resonar",
    "resonéis": "resonar",
    "resonó": "resonar",
    "resopla": "resoplar",
    "resoplaba": "resoplar",
    "resoplabais": "resoplar",
    "resoplaban": "resoplar",
    "resoplabas": "resoplar",
    "resoplad": "resoplar",
    "resoplada": "resoplar",
    "resopladas": "resoplar",
    "resoplado": "resoplar",
    "resoplados": "resoplar",
    "resoplamos": "resoplar",
    "resoplan": "resoplar",
    "resoplando": "resoplar",
    "resoplara": "resoplar",
    "resoplarais": "resoplar",
    "resoplaran": "resoplar",
    "resoplaras": "resoplar",
    "resoplare": "resoplar",
    "resoplareis": "resoplar",
    "resoplaremos": "resoplar",
    "resoplaren": "resoplar",
    "resoplares": "resoplar",
    "resoplaron": "resoplar",
    "resoplará": "resoplar",
    "resoplarán": "resoplar",
    "resoplarás": "resoplar",
    "resoplaré": "resoplar",
    "resoplaréis": "resoplar",
    "resoplaría": "resoplar",
    "resoplaríais": "resoplar",
    "resoplaríamos": "resoplar",
    "resoplarían": "resoplar",
    "resoplarías": "resoplar",
    "resoplas": "resoplar",
    "resoplase": "resoplar",
    "resoplaseis": "resoplar",
    "resoplasen": "resoplar",
    "resoplases": "resoplar",
    "resoplaste": "resoplar",
    "resoplasteis": "resoplar",
    "resople": "resoplar",
    "resoplemos": "resoplar",
    "resoplen": "resoplar",
    "resoples": "resoplar",
    "resoplidos": "resoplido",
    "resoplo": "resoplar",
    "resoplos": "resoplo",
    "resoplábamos": "resoplar",
    "resopláis": "resoplar",
    "resopláramos": "resoplar",
    "resopláremos": "resoplar",
    "resoplásemos": "resoplar",
    "resoplé": "resoplar",
    "resopléis": "resoplar",
    "resopló": "resoplar",
    "resorcinas": "resorcina",
    "resorciones": "resorción",
    "resorcílica": "resorcílico",
    "resorcílicas": "resorcílico",
    "resorcílicos": "resorcílico",
    "resortes": "resorte",
    "respalda": "respaldar",
    "respaldaba": "respaldar",
    "respaldabais": "respaldar",
    "respaldaban": "respaldar",
    "respaldabas": "respaldar",
    "respaldad": "respaldar",
    "respaldada": "respaldar",
    "respaldadas": "respaldar",
    "respaldado": "respaldar",
    "respaldados": "respaldar",
    "respaldamos": "respaldar",
    "respaldan": "respaldar",
    "respaldando": "respaldar",
    "respaldara": "respaldar",
    "respaldarais": "respaldar",
    "respaldaran": "respaldar",
    "respaldaras": "respaldar",
    "respaldare": "respaldar",
    "respaldareis": "respaldar",
    "respaldaremos": "respaldar",
    "respaldaren": "respaldar",
    "respaldares": "respaldar",
    "respaldaron": "respaldar",
    "respaldará": "respaldar",
    "respaldarán": "respaldar",
    "respaldarás": "respaldar",
    "respaldaré": "respaldar",
    "respaldaréis": "respaldar",
    "respaldaría": "respaldar",
    "respaldaríais": "respaldar",
    "respaldaríamos": "respaldar",
    "respaldarían": "respaldar",
    "respaldarías": "respaldar",
    "respaldas": "respaldar",
    "respaldase": "respaldar",
    "respaldaseis": "respaldar",
    "respaldasen": "respaldar",
    "respaldases": "respaldar",
    "respaldaste": "respaldar",
    "respaldasteis": "respaldar",
    "respalde": "respaldar",
    "respaldemos": "respaldar",
    "respalden": "respaldar",
    "respaldes": "respaldar",
    "respaldo": "respaldar",
    "respaldos": "respaldo",
    "respaldábamos": "respaldar",
    "respaldáis": "respaldar",
    "respaldáramos": "respaldar",
    "respaldáremos": "respaldar",
    "respaldásemos": "respaldar",
    "respaldé": "respaldar",
    "respaldéis": "respaldar",
    "respaldó": "respaldar",
    "respecta": "respectar",
    "respectaba": "respectar",
    "respectabais": "respectar",
    "respectaban": "respectar",
    "respectabas": "respectar",
    "respectad": "respectar",
    "respectada": "respectar",
    "respectadas": "respectar",
    "respectado": "respectar",
    "respectados": "respectar",
    "respectamos": "respectar",
    "respectan": "respectar",
    "respectando": "respectar",
    "respectara": "respectar",
    "respectarais": "respectar",
    "respectaran": "respectar",
    "respectaras": "respectar",
    "respectare": "respectar",
    "respectareis": "respectar",
    "respectaremos": "respectar",
    "respectaren": "respectar",
    "respectares": "respectar",
    "respectaron": "respectar",
    "respectará": "respectar",
    "respectarán": "respectar",
    "respectarás": "respectar",
    "respectaré": "respectar",
    "respectaréis": "respectar",
    "respectaría": "respectar",
    "respectaríais": "respectar",
    "respectaríamos": "respectar",
    "respectarían": "respectar",
    "respectarías": "respectar",
    "respectas": "respectar",
    "respectase": "respectar",
    "respectaseis": "respectar",
    "respectasen": "respectar",
    "respectases": "respectar",
    "respectaste": "respectar",
    "respectasteis": "respectar",
    "respecte": "respectar",
    "respectemos": "respectar",
    "respecten": "respectar",
    "respectes": "respectar",
    "respectiva": "respectivo",
    "respectivas": "respectivo",
    "respectivos": "respectivo",
    "respecto": "respectar",
    "respectos": "respecto",
    "respectábamos": "respectar",
    "respectáis": "respectar",
    "respectáramos": "respectar",
    "respectáremos": "respectar",
    "respectásemos": "respectar",
    "respecté": "respectar",
    "respectéis": "respectar",
    "respectó": "respectar",
    "respeds": "résped",
    "respes": "respe",
    "respeta": "respetar",
    "respetaba": "respetar",
    "respetabais": "respetar",
    "respetaban": "respetar",
    "respetabas": "respetar",
    "respetabilidades": "respetabilidad",
    "respetables": "respetable",
    "respetad": "respetar",
    "respetada": "respetar",
    "respetadas": "respetar",
    "respetado": "respetar",
    "respetados": "respetar",
    "respetamos": "respetar",
    "respetan": "respetar",
    "respetando": "respetar",
    "respetara": "respetar",
    "respetarais": "respetar",
    "respetaran": "respetar",
    "respetaras": "respetar",
    "respetare": "respetar",
    "respetareis": "respetar",
    "respetaremos": "respetar",
    "respetaren": "respetar",
    "respetares": "respetar",
    "respetaron": "respetar",
    "respetará": "respetar",
    "respetarán": "respetar",
    "respetarás": "respetar",
    "respetaré": "respetar",
    "respetaréis": "respetar",
    "respetaría": "respetar",
    "respetaríais": "respetar",
    "respetaríamos": "respetar",
    "respetarían": "respetar",
    "respetarías": "respetar",
    "respetas": "respetar",
    "respetase": "respetar",
    "respetaseis": "respetar",
    "respetasen": "respetar",
    "respetases": "respetar",
    "respetaste": "respetar",
    "respetasteis": "respetar",
    "respete": "respetar",
    "respetemos": "respetar",
    "respeten": "respetar",
    "respetes": "respetar",
    "respeto": "respetar",
    "respetos": "respeto",
    "respetuosa": "respetuoso",
    "respetuosas": "respetuoso",
    "respetuosidades": "respetuosidad",
    "respetuosos": "respetuoso",
    "respetábamos": "respetar",
    "respetáis": "respetar",
    "respetáramos": "respetar",
    "respetáremos": "respetar",
    "respetásemos": "respetar",
    "respeté": "respetar",
    "respetéis": "respetar",
    "respetó": "respetar",
    "respiga": "respigar",
    "respigaba": "respigar",
    "respigabais": "respigar",
    "respigaban": "respigar",
    "respigabas": "respigar",
    "respigad": "respigar",
    "respigada": "respigar",
    "respigadas": "respigar",
    "respigado": "respigar",
    "respigadora": "respigador",
    "respigadoras": "respigador",
    "respigadores": "respigador",
    "respigados": "respigar",
    "respigamos": "respigar",
    "respigan": "respigar",
    "respigando": "respigar",
    "respigara": "respigar",
    "respigarais": "respigar",
    "respigaran": "respigar",
    "respigaras": "respigar",
    "respigare": "respigar",
    "respigareis": "respigar",
    "respigaremos": "respigar",
    "respigaren": "respigar",
    "respigares": "respigar",
    "respigaron": "respigar",
    "respigará": "respigar",
    "respigarán": "respigar",
    "respigarás": "respigar",
    "respigaré": "respigar",
    "respigaréis": "respigar",
    "respigaría": "respigar",
    "respigaríais": "respigar",
    "respigaríamos": "respigar",
    "respigarían": "respigar",
    "respigarías": "respigar",
    "respigas": "respigar",
    "respigase": "respigar",
    "respigaseis": "respigar",
    "respigasen": "respigar",
    "respigases": "respigar",
    "respigaste": "respigar",
    "respigasteis": "respigar",
    "respigo": "respigar",
    "respigones": "respigón",
    "respigue": "respigar",
    "respiguemos": "respigar",
    "respiguen": "respigar",
    "respigues": "respigar",
    "respigué": "respigar",
    "respiguéis": "respigar",
    "respigábamos": "respigar",
    "respigáis": "respigar",
    "respigáramos": "respigar",
    "respigáremos": "respigar",
    "respigásemos": "respigar",
    "respigó": "respigar",
    "respinga": "respingar",
    "respingaba": "respingar",
    "respingabais": "respingar",
    "respingaban": "respingar",
    "respingabas": "respingar",
    "respingad": "respingar",
    "respingada": "respingar",
    "respingadas": "respingar",
    "respingado": "respingar",
    "respingados": "respingar",
    "respingamos": "respingar",
    "respingan": "respingar",
    "respingando": "respingar",
    "respingara": "respingar",
    "respingarais": "respingar",
    "respingaran": "respingar",
    "respingaras": "respingar",
    "respingare": "respingar",
    "respingareis": "respingar",
    "respingaremos": "respingar",
    "respingaren": "respingar",
    "respingares": "respingar",
    "respingaron": "respingar",
    "respingará": "respingar",
    "respingarán": "respingar",
    "respingarás": "respingar",
    "respingaré": "respingar",
    "respingaréis": "respingar",
    "respingaría": "respingar",
    "respingaríais": "respingar",
    "respingaríamos": "respingar",
    "respingarían": "respingar",
    "respingarías": "respingar",
    "respingas": "respingar",
    "respingase": "respingar",
    "respingaseis": "respingar",
    "respingasen": "respingar",
    "respingases": "respingar",
    "respingaste": "respingar",
    "respingasteis": "respingar",
    "respingo": "respingar",
    "respingona": "respingón",
    "respingonas": "respingón",
    "respingones": "respingón",
    "respingos": "respingo",
    "respingue": "respingar",
    "respinguemos": "respingar",
    "respinguen": "respingar",
    "respingues": "respingar",
    "respingué": "respingar",
    "respinguéis": "respingar",
    "respingábamos": "respingar",
    "respingáis": "respingar",
    "respingáramos": "respingar",
    "respingáremos": "respingar",
    "respingásemos": "respingar",
    "respingó": "respingar",
    "respira": "respirar",
    "respiraba": "respirar",
    "respirabais": "respirar",
    "respiraban": "respirar",
    "respirabas": "respirar",
    "respirables": "respirable",
    "respiraciones": "respiración",
    "respirad": "respirar",
    "respirada": "respirar",
    "respiradas": "respirar",
    "respiraderos": "respiradero",
    "respirado": "respirar",
    "respiradora": "respirador",
    "respiradoras": "respirador",
    "respiradores": "respirador",
    "respirados": "respirar",
    "respiramos": "respirar",
    "respiran": "respirar",
    "respirando": "respirar",
    "respirara": "respirar",
    "respirarais": "respirar",
    "respiraran": "respirar",
    "respiraras": "respirar",
    "respirare": "respirar",
    "respirareis": "respirar",
    "respiraremos": "respirar",
    "respiraren": "respirar",
    "respirares": "respirar",
    "respiraron": "respirar",
    "respirará": "respirar",
    "respirarán": "respirar",
    "respirarás": "respirar",
    "respiraré": "respirar",
    "respiraréis": "respirar",
    "respiraría": "respirar",
    "respiraríais": "respirar",
    "respiraríamos": "respirar",
    "respirarían": "respirar",
    "respirarías": "respirar",
    "respiras": "respirar",
    "respirase": "respirar",
    "respiraseis": "respirar",
    "respirasen": "respirar",
    "respirases": "respirar",
    "respiraste": "respirar",
    "respirasteis": "respirar",
    "respiratoria": "respiratorio",
    "respiratorias": "respiratorio",
    "respiratorios": "respiratorio",
    "respire": "respirar",
    "respiremos": "respirar",
    "respiren": "respirar",
    "respires": "respirar",
    "respiro": "respirar",
    "respiros": "respiro",
    "respirábamos": "respirar",
    "respiráis": "respirar",
    "respiráramos": "respirar",
    "respiráremos": "respirar",
    "respirásemos": "respirar",
    "respiré": "respirar",
    "respiréis": "respirar",
    "respiró": "respirar",
    "resplandece": "resplandecer",
    "resplandeced": "resplandecer",
    "resplandecemos": "resplandecer",
    "resplandecen": "resplandecer",
    "resplandeceremos": "resplandecer",
    "resplandecerá": "resplandecer",
    "resplandecerán": "resplandecer",
    "resplandecerás": "resplandecer",
    "resplandeceré": "resplandecer",
    "resplandeceréis": "resplandecer",
    "resplandecería": "resplandecer",
    "resplandeceríais": "resplandecer",
    "resplandeceríamos": "resplandecer",
    "resplandecerían": "resplandecer",
    "resplandecerías": "resplandecer",
    "resplandeces": "resplandecer",
    "resplandecida": "resplandecer",
    "resplandecidas": "resplandecer",
    "resplandecido": "resplandecer",
    "resplandecidos": "resplandecer",
    "resplandeciendo": "resplandecer",
    "resplandecientes": "resplandeciente",
    "resplandeciera": "resplandecer",
    "resplandecierais": "resplandecer",
    "resplandecieran": "resplandecer",
    "resplandecieras": "resplandecer",
    "resplandeciere": "resplandecer",
    "resplandeciereis": "resplandecer",
    "resplandecieren": "resplandecer",
    "resplandecieres": "resplandecer",
    "resplandecieron": "resplandecer",
    "resplandeciese": "resplandecer",
    "resplandecieseis": "resplandecer",
    "resplandeciesen": "resplandecer",
    "resplandecieses": "resplandecer",
    "resplandecimientos": "resplandecimiento",
    "resplandecimos": "resplandecer",
    "resplandeciste": "resplandecer",
    "resplandecisteis": "resplandecer",
    "resplandeciéramos": "resplandecer",
    "resplandeciéremos": "resplandecer",
    "resplandeciésemos": "resplandecer",
    "resplandeció": "resplandecer",
    "resplandecéis": "resplandecer",
    "resplandecí": "resplandecer",
    "resplandecía": "resplandecer",
    "resplandecíais": "resplandecer",
    "resplandecíamos": "resplandecer",
    "resplandecían": "resplandecer",
    "resplandecías": "resplandecer",
    "resplandezca": "resplandecer",
    "resplandezcamos": "resplandecer",
    "resplandezcan": "resplandecer",
    "resplandezcas": "resplandecer",
    "resplandezco": "resplandecer",
    "resplandezcáis": "resplandecer",
    "resplandinas": "resplandina",
    "resplandores": "resplandor",
    "responda": "responder",
    "respondamos": "responder",
    "respondan": "responder",
    "respondas": "responder",
    "responde": "responder",
    "responded": "responder",
    "respondedora": "respondedor",
    "respondedoras": "respondedor",
    "respondedores": "respondedor",
    "respondemos": "responder",
    "responden": "responder",
    "responderemos": "responder",
    "responderá": "responder",
    "responderán": "responder",
    "responderás": "responder",
    "responderé": "responder",
    "responderéis": "responder",
    "respondería": "responder",
    "responderíais": "responder",
    "responderíamos": "responder",
    "responderían": "responder",
    "responderías": "responder",
    "respondes": "responder",
    "respondida": "responder",
    "respondidas": "responder",
    "respondido": "responder",
    "respondidos": "responder",
    "respondiendo": "responder",
    "respondiera": "responder",
    "respondierais": "responder",
    "respondieran": "responder",
    "respondieras": "responder",
    "respondiere": "responder",
    "respondiereis": "responder",
    "respondieren": "responder",
    "respondieres": "responder",
    "respondieron": "responder",
    "respondiese": "responder",
    "respondieseis": "responder",
    "respondiesen": "responder",
    "respondieses": "responder",
    "respondimos": "responder",
    "respondiste": "responder",
    "respondisteis": "responder",
    "respondiéramos": "responder",
    "respondiéremos": "responder",
    "respondiésemos": "responder",
    "respondió": "responder",
    "respondo": "responder",
    "respondona": "respondón",
    "respondonas": "respondón",
    "respondones": "respondón",
    "respondáis": "responder",
    "respondéis": "responder",
    "respondí": "responder",
    "respondía": "responder",
    "respondíais": "responder",
    "respondíamos": "responder",
    "respondían": "responder",
    "respondías": "responder",
    "responsa": "responsar",
    "responsaba": "responsar",
    "responsabais": "responsar",
    "responsaban": "responsar",
    "responsabas": "responsar",
    "responsabilice": "responsabilizar",
    "responsabilicemos": "responsabilizar",
    "responsabilicen": "responsabilizar",
    "responsabilices": "responsabilizar",
    "responsabilicé": "responsabilizar",
    "responsabilicéis": "responsabilizar",
    "responsabilidades": "responsabilidad",
    "responsabiliza": "responsabilizar",
    "responsabilizaba": "responsabilizar",
    "responsabilizabais": "responsabilizar",
    "responsabilizaban": "responsabilizar",
    "responsabilizabas": "responsabilizar",
    "responsabilizaciones": "responsabilización",
    "responsabilizad": "responsabilizar",
    "responsabilizada": "responsabilizar",
    "responsabilizadas": "responsabilizar",
    "responsabilizado": "responsabilizar",
    "responsabilizados": "responsabilizar",
    "responsabilizamos": "responsabilizar",
    "responsabilizan": "responsabilizar",
    "responsabilizando": "responsabilizar",
    "responsabilizara": "responsabilizar",
    "responsabilizarais": "responsabilizar",
    "responsabilizaran": "responsabilizar",
    "responsabilizaras": "responsabilizar",
    "responsabilizare": "responsabilizar",
    "responsabilizareis": "responsabilizar",
    "responsabilizaremos": "responsabilizar",
    "responsabilizaren": "responsabilizar",
    "responsabilizares": "responsabilizar",
    "responsabilizaron": "responsabilizar",
    "responsabilizará": "responsabilizar",
    "responsabilizarán": "responsabilizar",
    "responsabilizarás": "responsabilizar",
    "responsabilizaré": "responsabilizar",
    "responsabilizaréis": "responsabilizar",
    "responsabilizaría": "responsabilizar",
    "responsabilizaríais": "responsabilizar",
    "responsabilizaríamos": "responsabilizar",
    "responsabilizarían": "responsabilizar",
    "responsabilizarías": "responsabilizar",
    "responsabilizas": "responsabilizar",
    "responsabilizase": "responsabilizar",
    "responsabilizaseis": "responsabilizar",
    "responsabilizasen": "responsabilizar",
    "responsabilizases": "responsabilizar",
    "responsabilizaste": "responsabilizar",
    "responsabilizasteis": "responsabilizar",
    "responsabilizo": "responsabilizar",
    "responsabilizábamos": "responsabilizar",
    "responsabilizáis": "responsabilizar",
    "responsabilizáramos": "responsabilizar",
    "responsabilizáremos": "responsabilizar",
    "responsabilizásemos": "responsabilizar",
    "responsabilizó": "responsabilizar",
    "responsables": "responsable",
    "responsad": "responsar",
    "responsada": "responsar",
    "responsadas": "responsar",
    "responsado": "responsar",
    "responsados": "responsar",
    "responsamos": "responsar",
    "responsan": "responsar",
    "responsando": "responsar",
    "responsara": "responsar",
    "responsarais": "responsar",
    "responsaran": "responsar",
    "responsaras": "responsar",
    "responsare": "responsar",
    "responsareis": "responsar",
    "responsaremos": "responsar",
    "responsaren": "responsar",
    "responsares": "responsar",
    "responsaron": "responsar",
    "responsará": "responsar",
    "responsarán": "responsar",
    "responsarás": "responsar",
    "responsaré": "responsar",
    "responsaréis": "responsar",
    "responsaría": "responsar",
    "responsaríais": "responsar",
    "responsaríamos": "responsar",
    "responsarían": "responsar",
    "responsarías": "responsar",
    "responsas": "responsar",
    "responsase": "responsar",
    "responsaseis": "responsar",
    "responsasen": "responsar",
    "responsases": "responsar",
    "responsaste": "responsar",
    "responsasteis": "responsar",
    "response": "responsar",
    "responsea": "responsear",
    "responseaba": "responsear",
    "responseabais": "responsear",
    "responseaban": "responsear",
    "responseabas": "responsear",
    "responsead": "responsear",
    "responseada": "responsear",
    "responseadas": "responsear",
    "responseado": "responsear",
    "responseados": "responsear",
    "responseamos": "responsear",
    "responsean": "responsear",
    "responseando": "responsear",
    "responseara": "responsear",
    "responsearais": "responsear",
    "responsearan": "responsear",
    "responsearas": "responsear",
    "responseare": "responsear",
    "responseareis": "responsear",
    "responsearemos": "responsear",
    "responsearen": "responsear",
    "responseares": "responsear",
    "responsearon": "responsear",
    "responseará": "responsear",
    "responsearán": "responsear",
    "responsearás": "responsear",
    "responsearé": "responsear",
    "responsearéis": "responsear",
    "responsearía": "responsear",
    "responsearíais": "responsear",
    "responsearíamos": "responsear",
    "responsearían": "responsear",
    "responsearías": "responsear",
    "responseas": "responsear",
    "responsease": "responsear",
    "responseaseis": "responsear",
    "responseasen": "responsear",
    "responseases": "responsear",
    "responseaste": "responsear",
    "responseasteis": "responsear",
    "responsee": "responsear",
    "responseemos": "responsear",
    "responseen": "responsear",
    "responsees": "responsear",
    "responsemos": "responsar",
    "responsen": "responsar",
    "responseo": "responsear",
    "responses": "responsar",
    "responseábamos": "responsear",
    "responseáis": "responsear",
    "responseáramos": "responsear",
    "responseáremos": "responsear",
    "responseásemos": "responsear",
    "responseé": "responsear",
    "responseéis": "responsear",
    "responseó": "responsear",
    "responsiva": "responsivo",
    "responsivas": "responsivo",
    "responsivos": "responsivo",
    "responso": "responsar",
    "responsorios": "responsorio",
    "responsos": "responso",
    "responsábamos": "responsar",
    "responsáis": "responsar",
    "responsáramos": "responsar",
    "responsáremos": "responsar",
    "responsásemos": "responsar",
    "responsé": "responsar",
    "responséis": "responsar",
    "responsó": "responsar",
    "respuestas": "respuesta",
    "resquebraba": "resquebrar",
    "resquebrabais": "resquebrar",
    "resquebraban": "resquebrar",
    "resquebrabas": "resquebrar",
    "resquebrad": "resquebrar",
    "resquebrada": "resquebrar",
    "resquebradas": "resquebrar",
    "resquebrado": "resquebrar",
    "resquebrados": "resquebrar",
    "resquebraduras": "resquebradura",
    "resquebraja": "resquebrajar",
    "resquebrajaba": "resquebrajar",
    "resquebrajabais": "resquebrajar",
    "resquebrajaban": "resquebrajar",
    "resquebrajabas": "resquebrajar",
    "resquebrajad": "resquebrajar",
    "resquebrajada": "resquebrajar",
    "resquebrajadas": "resquebrajar",
    "resquebrajado": "resquebrajar",
    "resquebrajados": "resquebrajar",
    "resquebrajaduras": "resquebrajadura",
    "resquebrajamientos": "resquebrajamiento",
    "resquebrajamos": "resquebrajar",
    "resquebrajan": "resquebrajar",
    "resquebrajando": "resquebrajar",
    "resquebrajara": "resquebrajar",
    "resquebrajarais": "resquebrajar",
    "resquebrajaran": "resquebrajar",
    "resquebrajaras": "resquebrajar",
    "resquebrajare": "resquebrajar",
    "resquebrajareis": "resquebrajar",
    "resquebrajaremos": "resquebrajar",
    "resquebrajaren": "resquebrajar",
    "resquebrajares": "resquebrajar",
    "resquebrajaron": "resquebrajar",
    "resquebrajará": "resquebrajar",
    "resquebrajarán": "resquebrajar",
    "resquebrajarás": "resquebrajar",
    "resquebrajaré": "resquebrajar",
    "resquebrajaréis": "resquebrajar",
    "resquebrajaría": "resquebrajar",
    "resquebrajaríais": "resquebrajar",
    "resquebrajaríamos": "resquebrajar",
    "resquebrajarían": "resquebrajar",
    "resquebrajarías": "resquebrajar",
    "resquebrajas": "resquebrajar",
    "resquebrajase": "resquebrajar",
    "resquebrajaseis": "resquebrajar",
    "resquebrajasen": "resquebrajar",
    "resquebrajases": "resquebrajar",
    "resquebrajaste": "resquebrajar",
    "resquebrajasteis": "resquebrajar",
    "resquebraje": "resquebrajar",
    "resquebrajemos": "resquebrajar",
    "resquebrajen": "resquebrajar",
    "resquebrajes": "resquebrajar",
    "resquebrajo": "resquebrajar",
    "resquebrajos": "resquebrajo",
    "resquebrajosa": "resquebrajoso",
    "resquebrajosas": "resquebrajoso",
    "resquebrajosos": "resquebrajoso",
    "resquebrajábamos": "resquebrajar",
    "resquebrajáis": "resquebrajar",
    "resquebrajáramos": "resquebrajar",
    "resquebrajáremos": "resquebrajar",
    "resquebrajásemos": "resquebrajar",
    "resquebrajé": "resquebrajar",
    "resquebrajéis": "resquebrajar",
    "resquebrajó": "resquebrajar",
    "resquebramos": "resquebrar",
    "resquebrando": "resquebrar",
    "resquebrara": "resquebrar",
    "resquebrarais": "resquebrar",
    "resquebraran": "resquebrar",
    "resquebraras": "resquebrar",
    "resquebrare": "resquebrar",
    "resquebrareis": "resquebrar",
    "resquebraremos": "resquebrar",
    "resquebraren": "resquebrar",
    "resquebrares": "resquebrar",
    "resquebraron": "resquebrar",
    "resquebrará": "resquebrar",
    "resquebrarán": "resquebrar",
    "resquebrarás": "resquebrar",
    "resquebraré": "resquebrar",
    "resquebraréis": "resquebrar",
    "resquebraría": "resquebrar",
    "resquebraríais": "resquebrar",
    "resquebraríamos": "resquebrar",
    "resquebrarían": "resquebrar",
    "resquebrarías": "resquebrar",
    "resquebrase": "resquebrar",
    "resquebraseis": "resquebrar",
    "resquebrasen": "resquebrar",
    "resquebrases": "resquebrar",
    "resquebraste": "resquebrar",
    "resquebrasteis": "resquebrar",
    "resquebremos": "resquebrar",
    "resquebrábamos": "resquebrar",
    "resquebráis": "resquebrar",
    "resquebráramos": "resquebrar",
    "resquebráremos": "resquebrar",
    "resquebrásemos": "resquebrar",
    "resquebré": "resquebrar",
    "resquebréis": "resquebrar",
    "resquebró": "resquebrar",
    "resquema": "resquemar",
    "resquemaba": "resquemar",
    "resquemabais": "resquemar",
    "resquemaban": "resquemar",
    "resquemabas": "resquemar",
    "resquemad": "resquemar",
    "resquemada": "resquemar",
    "resquemadas": "resquemar",
    "resquemado": "resquemar",
    "resquemados": "resquemar",
    "resquemamos": "resquemar",
    "resqueman": "resquemar",
    "resquemando": "resquemar",
    "resquemara": "resquemar",
    "resquemarais": "resquemar",
    "resquemaran": "resquemar",
    "resquemaras": "resquemar",
    "resquemare": "resquemar",
    "resquemareis": "resquemar",
    "resquemaremos": "resquemar",
    "resquemaren": "resquemar",
    "resquemares": "resquemar",
    "resquemaron": "resquemar",
    "resquemará": "resquemar",
    "resquemarán": "resquemar",
    "resquemarás": "resquemar",
    "resquemaré": "resquemar",
    "resquemaréis": "resquemar",
    "resquemaría": "resquemar",
    "resquemaríais": "resquemar",
    "resquemaríamos": "resquemar",
    "resquemarían": "resquemar",
    "resquemarías": "resquemar",
    "resquemas": "resquemar",
    "resquemase": "resquemar",
    "resquemaseis": "resquemar",
    "resquemasen": "resquemar",
    "resquemases": "resquemar",
    "resquemaste": "resquemar",
    "resquemasteis": "resquemar",
    "resquemazones": "resquemazón",
    "resqueme": "resquemar",
    "resquememos": "resquemar",
    "resquemen": "resquemar",
    "resquemes": "resquemar",
    "resquemo": "resquemar",
    "resquemores": "resquemor",
    "resquemos": "resquemo",
    "resquemábamos": "resquemar",
    "resquemáis": "resquemar",
    "resquemáramos": "resquemar",
    "resquemáremos": "resquemar",
    "resquemásemos": "resquemar",
    "resquemé": "resquemar",
    "resqueméis": "resquemar",
    "resquemó": "resquemar",
    "resquicios": "resquicio",
    "resquiebra": "resquebrar",
    "resquiebran": "resquebrar",
    "resquiebras": "resquebrar",
    "resquiebre": "resquebrar",
    "resquiebren": "resquebrar",
    "resquiebres": "resquebrar",
    "resquiebro": "resquebrar",
    "resta": "restar",
    "restaba": "restar",
    "restabais": "restar",
    "restaban": "restar",
    "restabas": "restar",
    "restablece": "restablecer",
    "restableced": "restablecer",
    "restablecemos": "restablecer",
    "restablecen": "restablecer",
    "restableceremos": "restablecer",
    "restablecerá": "restablecer",
    "restablecerán": "restablecer",
    "restablecerás": "restablecer",
    "restableceré": "restablecer",
    "restableceréis": "restablecer",
    "restablecería": "restablecer",
    "restableceríais": "restablecer",
    "restableceríamos": "restablecer",
    "restablecerían": "restablecer",
    "restablecerías": "restablecer",
    "restableces": "restablecer",
    "restablecida": "restablecer",
    "restablecidas": "restablecer",
    "restablecido": "restablecer",
    "restablecidos": "restablecer",
    "restableciendo": "restablecer",
    "restableciera": "restablecer",
    "restablecierais": "restablecer",
    "restablecieran": "restablecer",
    "restablecieras": "restablecer",
    "restableciere": "restablecer",
    "restableciereis": "restablecer",
    "restablecieren": "restablecer",
    "restablecieres": "restablecer",
    "restablecieron": "restablecer",
    "restableciese": "restablecer",
    "restablecieseis": "restablecer",
    "restableciesen": "restablecer",
    "restablecieses": "restablecer",
    "restablecimientos": "restablecimiento",
    "restablecimos": "restablecer",
    "restableciste": "restablecer",
    "restablecisteis": "restablecer",
    "restableciéramos": "restablecer",
    "restableciéremos": "restablecer",
    "restableciésemos": "restablecer",
    "restableció": "restablecer",
    "restablecéis": "restablecer",
    "restablecí": "restablecer",
    "restablecía": "restablecer",
    "restablecíais": "restablecer",
    "restablecíamos": "restablecer",
    "restablecían": "restablecer",
    "restablecías": "restablecer",
    "restablezca": "restablecer",
    "restablezcamos": "restablecer",
    "restablezcan": "restablecer",
    "restablezcas": "restablecer",
    "restablezco": "restablecer",
    "restablezcáis": "restablecer",
    "restad": "restar",
    "restada": "restar",
    "restadas": "restar",
    "restado": "restar",
    "restadora": "restador",
    "restadoras": "restador",
    "restadores": "restador",
    "restados": "restar",
    "restalla": "restallar",
    "restallaba": "restallar",
    "restallabais": "restallar",
    "restallaban": "restallar",
    "restallabas": "restallar",
    "restallad": "restallar",
    "restallada": "restallar",
    "restalladas": "restallar",
    "restallado": "restallar",
    "restallados": "restallar",
    "restallamos": "restallar",
    "restallan": "restallar",
    "restallando": "restallar",
    "restallara": "restallar",
    "restallarais": "restallar",
    "restallaran": "restallar",
    "restallaras": "restallar",
    "restallare": "restallar",
    "restallareis": "restallar",
    "restallaremos": "restallar",
    "restallaren": "restallar",
    "restallares": "restallar",
    "restallaron": "restallar",
    "restallará": "restallar",
    "restallarán": "restallar",
    "restallarás": "restallar",
    "restallaré": "restallar",
    "restallaréis": "restallar",
    "restallaría": "restallar",
    "restallaríais": "restallar",
    "restallaríamos": "restallar",
    "restallarían": "restallar",
    "restallarías": "restallar",
    "restallas": "restallar",
    "restallase": "restallar",
    "restallaseis": "restallar",
    "restallasen": "restallar",
    "restallases": "restallar",
    "restallaste": "restallar",
    "restallasteis": "restallar",
    "restalle": "restallar",
    "restallemos": "restallar",
    "restallen": "restallar",
    "restalles": "restallar",
    "restallidos": "restallido",
    "restallo": "restallar",
    "restallábamos": "restallar",
    "restalláis": "restallar",
    "restalláramos": "restallar",
    "restalláremos": "restallar",
    "restallásemos": "restallar",
    "restallé": "restallar",
    "restalléis": "restallar",
    "restalló": "restallar",
    "restamos": "restar",
    "restan": "restar",
    "restando": "restar",
    "restantes": "restante",
    "restara": "restar",
    "restarais": "restar",
    "restaran": "restar",
    "restaras": "restar",
    "restare": "restar",
    "restareis": "restar",
    "restaremos": "restar",
    "restaren": "restar",
    "restares": "restar",
    "restaron": "restar",
    "restará": "restar",
    "restarán": "restar",
    "restarás": "restar",
    "restaré": "restar",
    "restaréis": "restar",
    "restaría": "restar",
    "restaríais": "restar",
    "restaríamos": "restar",
    "restarían": "restar",
    "restarías": "restar",
    "restas": "restar",
    "restase": "restar",
    "restaseis": "restar",
    "restasen": "restar",
    "restases": "restar",
    "restaste": "restar",
    "restasteis": "restar",
    "restaura": "restaurar",
    "restauraba": "restaurar",
    "restaurabais": "restaurar",
    "restauraban": "restaurar",
    "restaurabas": "restaurar",
    "restauraciones": "restauración",
    "restaurad": "restaurar",
    "restaurada": "restaurar",
    "restauradas": "restaurar",
    "restaurado": "restaurar",
    "restauradora": "restaurador",
    "restauradoras": "restaurador",
    "restauradores": "restaurador",
    "restaurados": "restaurar",
    "restauramos": "restaurar",
    "restauran": "restaurar",
    "restaurando": "restaurar",
    "restaurantes": "restaurante",
    "restaurara": "restaurar",
    "restaurarais": "restaurar",
    "restauraran": "restaurar",
    "restauraras": "restaurar",
    "restaurare": "restaurar",
    "restaurareis": "restaurar",
    "restauraremos": "restaurar",
    "restauraren": "restaurar",
    "restaurares": "restaurar",
    "restauraron": "restaurar",
    "restaurará": "restaurar",
    "restaurarán": "restaurar",
    "restaurarás": "restaurar",
    "restauraré": "restaurar",
    "restauraréis": "restaurar",
    "restauraría": "restaurar",
    "restauraríais": "restaurar",
    "restauraríamos": "restaurar",
    "restaurarían": "restaurar",
    "restaurarías": "restaurar",
    "restauras": "restaurar",
    "restaurase": "restaurar",
    "restauraseis": "restaurar",
    "restaurasen": "restaurar",
    "restaurases": "restaurar",
    "restauraste": "restaurar",
    "restaurasteis": "restaurar",
    "restaurativa": "restaurativo",
    "restaurativas": "restaurativo",
    "restaurativos": "restaurativo",
    "restaure": "restaurar",
    "restauremos": "restaurar",
    "restauren": "restaurar",
    "restaures": "restaurar",
    "restauro": "restaurar",
    "restaurábamos": "restaurar",
    "restauráis": "restaurar",
    "restauráramos": "restaurar",
    "restauráremos": "restaurar",
    "restaurásemos": "restaurar",
    "restauré": "restaurar",
    "restauréis": "restaurar",
    "restauró": "restaurar",
    "restaña": "restañar",
    "restañaba": "restañar",
    "restañabais": "restañar",
    "restañaban": "restañar",
    "restañabas": "restañar",
    "restañad": "restañar",
    "restañada": "restañar",
    "restañadas": "restañar",
    "restañaderos": "restañadero",
    "restañado": "restañar",
    "restañados": "restañar",
    "restañaduras": "restañadura",
    "restañamos": "restañar",
    "restañan": "restañar",
    "restañando": "restañar",
    "restañara": "restañar",
    "restañarais": "restañar",
    "restañaran": "restañar",
    "restañaras": "restañar",
    "restañare": "restañar",
    "restañareis": "restañar",
    "restañaremos": "restañar",
    "restañaren": "restañar",
    "restañares": "restañar",
    "restañaron": "restañar",
    "restañará": "restañar",
    "restañarán": "restañar",
    "restañarás": "restañar",
    "restañaré": "restañar",
    "restañaréis": "restañar",
    "restañaría": "restañar",
    "restañaríais": "restañar",
    "restañaríamos": "restañar",
    "restañarían": "restañar",
    "restañarías": "restañar",
    "restañas": "restañar",
    "restañasangres": "restañasangre",
    "restañase": "restañar",
    "restañaseis": "restañar",
    "restañasen": "restañar",
    "restañases": "restañar",
    "restañaste": "restañar",
    "restañasteis": "restañar",
    "restañe": "restañar",
    "restañemos": "restañar",
    "restañen": "restañar",
    "restañes": "restañar",
    "restaño": "restañar",
    "restaños": "restaño",
    "restañábamos": "restañar",
    "restañáis": "restañar",
    "restañáramos": "restañar",
    "restañáremos": "restañar",
    "restañásemos": "restañar",
    "restañé": "restañar",
    "restañéis": "restañar",
    "restañó": "restañar",
    "reste": "restar",
    "restemos": "restar",
    "resten": "restar",
    "restes": "restar",
    "restingares": "restingar",
    "restingas": "restinga",
    "restitopos": "restitopo",
    "restituciones": "restitución",
    "restituibles": "restituible",
    "restituid": "restituir",
    "restituida": "restituir",
    "restituidas": "restituir",
    "restituido": "restituir",
    "restituidora": "restituidor",
    "restituidoras": "restituidor",
    "restituidores": "restituidor",
    "restituidos": "restituir",
    "restituimos": "restituir",
    "restituiremos": "restituir",
    "restituirá": "restituir",
    "restituirán": "restituir",
    "restituirás": "restituir",
    "restituiré": "restituir",
    "restituiréis": "restituir",
    "restituiría": "restituir",
    "restituiríais": "restituir",
    "restituiríamos": "restituir",
    "restituirían": "restituir",
    "restituirías": "restituir",
    "restituiste": "restituir",
    "restituisteis": "restituir",
    "restitutoria": "restitutorio",
    "restitutorias": "restitutorio",
    "restitutorios": "restitutorio",
    "restituya": "restituir",
    "restituyamos": "restituir",
    "restituyan": "restituir",
    "restituyas": "restituir",
    "restituye": "restituir",
    "restituyen": "restituir",
    "restituyendo": "restituir",
    "restituyera": "restituir",
    "restituyerais": "restituir",
    "restituyeran": "restituir",
    "restituyeras": "restituir",
    "restituyere": "restituir",
    "restituyereis": "restituir",
    "restituyeren": "restituir",
    "restituyeres": "restituir",
    "restituyeron": "restituir",
    "restituyes": "restituir",
    "restituyese": "restituir",
    "restituyeseis": "restituir",
    "restituyesen": "restituir",
    "restituyeses": "restituir",
    "restituyo": "restituir",
    "restituyáis": "restituir",
    "restituyéramos": "restituir",
    "restituyéremos": "restituir",
    "restituyésemos": "restituir",
    "restituyó": "restituir",
    "restituí": "restituir",
    "restituía": "restituir",
    "restituíais": "restituir",
    "restituíamos": "restituir",
    "restituían": "restituir",
    "restituías": "restituir",
    "restituís": "restituir",
    "resto": "restar",
    "restojos": "restojo",
    "restos": "resto",
    "restregaba": "restregar",
    "restregabais": "restregar",
    "restregaban": "restregar",
    "restregabas": "restregar",
    "restregad": "restregar",
    "restregada": "restregar",
    "restregadas": "restregar",
    "restregado": "restregar",
    "restregados": "restregar",
    "restregaduras": "restregadura",
    "restregamientos": "restregamiento",
    "restregamos": "restregar",
    "restregando": "restregar",
    "restregara": "restregar",
    "restregarais": "restregar",
    "restregaran": "restregar",
    "restregaras": "restregar",
    "restregare": "restregar",
    "restregareis": "restregar",
    "restregaremos": "restregar",
    "restregaren": "restregar",
    "restregares": "restregar",
    "restregaron": "restregar",
    "restregará": "restregar",
    "restregarán": "restregar",
    "restregarás": "restregar",
    "restregaré": "restregar",
    "restregaréis": "restregar",
    "restregaría": "restregar",
    "restregaríais": "restregar",
    "restregaríamos": "restregar",
    "restregarían": "restregar",
    "restregarías": "restregar",
    "restregase": "restregar",
    "restregaseis": "restregar",
    "restregasen": "restregar",
    "restregases": "restregar",
    "restregaste": "restregar",
    "restregasteis": "restregar",
    "restregones": "restregón",
    "restreguemos": "restregar",
    "restregué": "restregar",
    "restreguéis": "restregar",
    "restregábamos": "restregar",
    "restregáis": "restregar",
    "restregáramos": "restregar",
    "restregáremos": "restregar",
    "restregásemos": "restregar",
    "restregó": "restregar",
    "restricciones": "restricción",
    "restricta": "restricto",
    "restrictas": "restricto",
    "restrictiva": "restrictivo",
    "restrictivas": "restrictivo",
    "restrictivos": "restrictivo",
    "restrictos": "restricto",
    "restriega": "restregar",
    "restriegan": "restregar",
    "restriegas": "restregar",
    "restriego": "restregar",
    "restriegue": "restregar",
    "restrieguen": "restregar",
    "restriegues": "restregar",
    "restringas": "restringa",
    "restringe": "restringir",
    "restringen": "restringir",
    "restringes": "restringir",
    "restringibles": "restringible",
    "restringid": "restringir",
    "restringida": "restringir",
    "restringidas": "restringir",
    "restringido": "restringir",
    "restringidos": "restringir",
    "restringiendo": "restringir",
    "restringiera": "restringir",
    "restringierais": "restringir",
    "restringieran": "restringir",
    "restringieras": "restringir",
    "restringiere": "restringir",
    "restringiereis": "restringir",
    "restringieren": "restringir",
    "restringieres": "restringir",
    "restringieron": "restringir",
    "restringiese": "restringir",
    "restringieseis": "restringir",
    "restringiesen": "restringir",
    "restringieses": "restringir",
    "restringimos": "restringir",
    "restringiremos": "restringir",
    "restringirá": "restringir",
    "restringirán": "restringir",
    "restringirás": "restringir",
    "restringiré": "restringir",
    "restringiréis": "restringir",
    "restringiría": "restringir",
    "restringiríais": "restringir",
    "restringiríamos": "restringir",
    "restringirían": "restringir",
    "restringirías": "restringir",
    "restringiste": "restringir",
    "restringisteis": "restringir",
    "restringiéramos": "restringir",
    "restringiéremos": "restringir",
    "restringiésemos": "restringir",
    "restringió": "restringir",
    "restringí": "restringir",
    "restringía": "restringir",
    "restringíais": "restringir",
    "restringíamos": "restringir",
    "restringían": "restringir",
    "restringías": "restringir",
    "restringís": "restringir",
    "restrinja": "restringir",
    "restrinjamos": "restringir",
    "restrinjan": "restringir",
    "restrinjas": "restringir",
    "restrinjo": "restringir",
    "restrinjáis": "restringir",
    "restriña": "restriñir",
    "restriñamos": "restriñir",
    "restriñan": "restriñir",
    "restriñas": "restriñir",
    "restriñe": "restriñir",
    "restriñen": "restriñir",
    "restriñendo": "restriñir",
    "restriñera": "restriñir",
    "restriñerais": "restriñir",
    "restriñeran": "restriñir",
    "restriñeras": "restriñir",
    "restriñere": "restriñir",
    "restriñereis": "restriñir",
    "restriñeren": "restriñir",
    "restriñeres": "restriñir",
    "restriñeron": "restriñir",
    "restriñes": "restriñir",
    "restriñese": "restriñir",
    "restriñeseis": "restriñir",
    "restriñesen": "restriñir",
    "restriñeses": "restriñir",
    "restriñid": "restriñir",
    "restriñida": "restriñir",
    "restriñidas": "restriñir",
    "restriñido": "restriñir",
    "restriñidora": "restriñidor",
    "restriñidoras": "restriñidor",
    "restriñidores": "restriñidor",
    "restriñidos": "restriñir",
    "restriñimientos": "restriñimiento",
    "restriñimos": "restriñir",
    "restriñiremos": "restriñir",
    "restriñirá": "restriñir",
    "restriñirán": "restriñir",
    "restriñirás": "restriñir",
    "restriñiré": "restriñir",
    "restriñiréis": "restriñir",
    "restriñiría": "restriñir",
    "restriñiríais": "restriñir",
    "restriñiríamos": "restriñir",
    "restriñirían": "restriñir",
    "restriñirías": "restriñir",
    "restriñiste": "restriñir",
    "restriñisteis": "restriñir",
    "restriño": "restriñir",
    "restriñáis": "restriñir",
    "restriñéramos": "restriñir",
    "restriñéremos": "restriñir",
    "restriñésemos": "restriñir",
    "restriñí": "restriñir",
    "restriñía": "restriñir",
    "restriñíais": "restriñir",
    "restriñíamos": "restriñir",
    "restriñían": "restriñir",
    "restriñías": "restriñir",
    "restriñís": "restriñir",
    "restriñó": "restriñir",
    "restrojos": "restrojo",
    "restructuraciones": "restructuración",
    "restábamos": "restar",
    "restáis": "restar",
    "restáramos": "restar",
    "restáremos": "restar",
    "restásemos": "restar",
    "resté": "restar",
    "restéis": "restar",
    "restó": "restar",
    "resucita": "resucitar",
    "resucitaba": "resucitar",
    "resucitabais": "resucitar",
    "resucitaban": "resucitar",
    "resucitabas": "resucitar",
    "resucitaciones": "resucitación",
    "resucitad": "resucitar",
    "resucitada": "resucitar",
    "resucitadas": "resucitar",
    "resucitado": "resucitar",
    "resucitadora": "resucitador",
    "resucitadoras": "resucitador",
    "resucitadores": "resucitador",
    "resucitados": "resucitar",
    "resucitamos": "resucitar",
    "resucitan": "resucitar",
    "resucitando": "resucitar",
    "resucitara": "resucitar",
    "resucitarais": "resucitar",
    "resucitaran": "resucitar",
    "resucitaras": "resucitar",
    "resucitare": "resucitar",
    "resucitareis": "resucitar",
    "resucitaremos": "resucitar",
    "resucitaren": "resucitar",
    "resucitares": "resucitar",
    "resucitaron": "resucitar",
    "resucitará": "resucitar",
    "resucitarán": "resucitar",
    "resucitarás": "resucitar",
    "resucitaré": "resucitar",
    "resucitaréis": "resucitar",
    "resucitaría": "resucitar",
    "resucitaríais": "resucitar",
    "resucitaríamos": "resucitar",
    "resucitarían": "resucitar",
    "resucitarías": "resucitar",
    "resucitas": "resucitar",
    "resucitase": "resucitar",
    "resucitaseis": "resucitar",
    "resucitasen": "resucitar",
    "resucitases": "resucitar",
    "resucitaste": "resucitar",
    "resucitasteis": "resucitar",
    "resucite": "resucitar",
    "resucitemos": "resucitar",
    "resuciten": "resucitar",
    "resucites": "resucitar",
    "resucito": "resucitar",
    "resucitábamos": "resucitar",
    "resucitáis": "resucitar",
    "resucitáramos": "resucitar",
    "resucitáremos": "resucitar",
    "resucitásemos": "resucitar",
    "resucité": "resucitar",
    "resucitéis": "resucitar",
    "resucitó": "resucitar",
    "resuda": "resudar",
    "resudaba": "resudar",
    "resudabais": "resudar",
    "resudaban": "resudar",
    "resudabas": "resudar",
    "resudaciones": "resudación",
    "resudad": "resudar",
    "resudada": "resudar",
    "resudadas": "resudar",
    "resudado": "resudar",
    "resudados": "resudar",
    "resudamos": "resudar",
    "resudan": "resudar",
    "resudando": "resudar",
    "resudara": "resudar",
    "resudarais": "resudar",
    "resudaran": "resudar",
    "resudaras": "resudar",
    "resudare": "resudar",
    "resudareis": "resudar",
    "resudaremos": "resudar",
    "resudaren": "resudar",
    "resudares": "resudar",
    "resudaron": "resudar",
    "resudará": "resudar",
    "resudarán": "resudar",
    "resudarás": "resudar",
    "resudaré": "resudar",
    "resudaréis": "resudar",
    "resudaría": "resudar",
    "resudaríais": "resudar",
    "resudaríamos": "resudar",
    "resudarían": "resudar",
    "resudarías": "resudar",
    "resudas": "resudar",
    "resudase": "resudar",
    "resudaseis": "resudar",
    "resudasen": "resudar",
    "resudases": "resudar",
    "resudaste": "resudar",
    "resudasteis": "resudar",
    "resude": "resudar",
    "resudemos": "resudar",
    "resuden": "resudar",
    "resudes": "resudar",
    "resudo": "resudar",
    "resudores": "resudor",
    "resudábamos": "resudar",
    "resudáis": "resudar",
    "resudáramos": "resudar",
    "resudáremos": "resudar",
    "resudásemos": "resudar",
    "resudé": "resudar",
    "resudéis": "resudar",
    "resudó": "resudar",
    "resuella": "resollar",
    "resuellan": "resollar",
    "resuellas": "resollar",
    "resuelle": "resollar",
    "resuellen": "resollar",
    "resuelles": "resollar",
    "resuello": "resollar",
    "resuellos": "resuello",
    "resuelta": "resolver",
    "resueltas": "resolver",
    "resuelto": "resolver",
    "resueltos": "resolver",
    "resuelva": "resolver",
    "resuelvan": "resolver",
    "resuelvas": "resolver",
    "resuelve": "resolver",
    "resuelven": "resolver",
    "resuelves": "resolver",
    "resuelvo": "resolver",
    "resuena": "resonar",
    "resuenan": "resonar",
    "resuenas": "resonar",
    "resuene": "resonar",
    "resuenen": "resonar",
    "resuenes": "resonar",
    "resueno": "resonar",
    "resulta": "resultar",
    "resultaba": "resultar",
    "resultabais": "resultar",
    "resultaban": "resultar",
    "resultabas": "resultar",
    "resultad": "resultar",
    "resultada": "resultar",
    "resultadas": "resultar",
    "resultado": "resultar",
    "resultados": "resultar",
    "resultamos": "resultar",
    "resultan": "resultar",
    "resultancias": "resultancia",
    "resultando": "resultar",
    "resultandos": "resultando",
    "resultantes": "resultante",
    "resultara": "resultar",
    "resultarais": "resultar",
    "resultaran": "resultar",
    "resultaras": "resultar",
    "resultare": "resultar",
    "resultareis": "resultar",
    "resultaremos": "resultar",
    "resultaren": "resultar",
    "resultares": "resultar",
    "resultaron": "resultar",
    "resultará": "resultar",
    "resultarán": "resultar",
    "resultarás": "resultar",
    "resultaré": "resultar",
    "resultaréis": "resultar",
    "resultaría": "resultar",
    "resultaríais": "resultar",
    "resultaríamos": "resultar",
    "resultarían": "resultar",
    "resultarías": "resultar",
    "resultas": "resultar",
    "resultase": "resultar",
    "resultaseis": "resultar",
    "resultasen": "resultar",
    "resultases": "resultar",
    "resultaste": "resultar",
    "resultasteis": "resultar",
    "resultativa": "resultativo",
    "resultativas": "resultativo",
    "resultativos": "resultativo",
    "resulte": "resultar",
    "resultemos": "resultar",
    "resulten": "resultar",
    "resultes": "resultar",
    "resulto": "resultar",
    "resultona": "resultón",
    "resultonas": "resultón",
    "resultones": "resultón",
    "resultábamos": "resultar",
    "resultáis": "resultar",
    "resultáramos": "resultar",
    "resultáremos": "resultar",
    "resultásemos": "resultar",
    "resulté": "resultar",
    "resultéis": "resultar",
    "resultó": "resultar",
    "resuma": "resumir",
    "resumamos": "resumir",
    "resuman": "resumir",
    "resumas": "resumir",
    "resume": "resumir",
    "resumen": "resumir",
    "resumes": "resumir",
    "resumid": "resumir",
    "resumida": "resumir",
    "resumidas": "resumir",
    "resumido": "resumir",
    "resumidos": "resumir",
    "resumiendo": "resumir",
    "resumiera": "resumir",
    "resumierais": "resumir",
    "resumieran": "resumir",
    "resumieras": "resumir",
    "resumiere": "resumir",
    "resumiereis": "resumir",
    "resumieren": "resumir",
    "resumieres": "resumir",
    "resumieron": "resumir",
    "resumiese": "resumir",
    "resumieseis": "resumir",
    "resumiesen": "resumir",
    "resumieses": "resumir",
    "resumimos": "resumir",
    "resumiremos": "resumir",
    "resumirá": "resumir",
    "resumirán": "resumir",
    "resumirás": "resumir",
    "resumiré": "resumir",
    "resumiréis": "resumir",
    "resumiría": "resumir",
    "resumiríais": "resumir",
    "resumiríamos": "resumir",
    "resumirían": "resumir",
    "resumirías": "resumir",
    "resumiste": "resumir",
    "resumisteis": "resumir",
    "resumiéramos": "resumir",
    "resumiéremos": "resumir",
    "resumiésemos": "resumir",
    "resumió": "resumir",
    "resumo": "resumir",
    "resumáis": "resumir",
    "resumí": "resumir",
    "resumía": "resumir",
    "resumíais": "resumir",
    "resumíamos": "resumir",
    "resumían": "resumir",
    "resumías": "resumir",
    "resumís": "resumir",
    "resurge": "resurgir",
    "resurgen": "resurgir",
    "resurgencias": "resurgencia",
    "resurges": "resurgir",
    "resurgid": "resurgir",
    "resurgida": "resurgir",
    "resurgidas": "resurgir",
    "resurgido": "resurgir",
    "resurgidos": "resurgir",
    "resurgiendo": "resurgir",
    "resurgiera": "resurgir",
    "resurgierais": "resurgir",
    "resurgieran": "resurgir",
    "resurgieras": "resurgir",
    "resurgiere": "resurgir",
    "resurgiereis": "resurgir",
    "resurgieren": "resurgir",
    "resurgieres": "resurgir",
    "resurgieron": "resurgir",
    "resurgiese": "resurgir",
    "resurgieseis": "resurgir",
    "resurgiesen": "resurgir",
    "resurgieses": "resurgir",
    "resurgimientos": "resurgimiento",
    "resurgimos": "resurgir",
    "resurgiremos": "resurgir",
    "resurgirá": "resurgir",
    "resurgirán": "resurgir",
    "resurgirás": "resurgir",
    "resurgiré": "resurgir",
    "resurgiréis": "resurgir",
    "resurgiría": "resurgir",
    "resurgiríais": "resurgir",
    "resurgiríamos": "resurgir",
    "resurgirían": "resurgir",
    "resurgirías": "resurgir",
    "resurgiste": "resurgir",
    "resurgisteis": "resurgir",
    "resurgiéramos": "resurgir",
    "resurgiéremos": "resurgir",
    "resurgiésemos": "resurgir",
    "resurgió": "resurgir",
    "resurgí": "resurgir",
    "resurgía": "resurgir",
    "resurgíais": "resurgir",
    "resurgíamos": "resurgir",
    "resurgían": "resurgir",
    "resurgías": "resurgir",
    "resurgís": "resurgir",
    "resurja": "resurgir",
    "resurjamos": "resurgir",
    "resurjan": "resurgir",
    "resurjas": "resurgir",
    "resurjo": "resurgir",
    "resurjáis": "resurgir",
    "resurrecciones": "resurrección",
    "resurtidas": "resurtida",
    "resurtiva": "resurtivo",
    "resurtivas": "resurtivo",
    "resurtivos": "resurtivo",
    "resúmenes": "resumen",
    "reta": "retar",
    "retaba": "retar",
    "retabais": "retar",
    "retaban": "retar",
    "retabas": "retar",
    "retablos": "retablo",
    "retaca": "retacar",
    "retacaba": "retacar",
    "retacabais": "retacar",
    "retacaban": "retacar",
    "retacabas": "retacar",
    "retacad": "retacar",
    "retacada": "retacar",
    "retacadas": "retacar",
    "retacado": "retacar",
    "retacados": "retacar",
    "retacamos": "retacar",
    "retacan": "retacar",
    "retacando": "retacar",
    "retacara": "retacar",
    "retacarais": "retacar",
    "retacaran": "retacar",
    "retacaras": "retacar",
    "retacare": "retacar",
    "retacareis": "retacar",
    "retacaremos": "retacar",
    "retacaren": "retacar",
    "retacares": "retacar",
    "retacaron": "retacar",
    "retacará": "retacar",
    "retacarán": "retacar",
    "retacarás": "retacar",
    "retacaré": "retacar",
    "retacaréis": "retacar",
    "retacaría": "retacar",
    "retacaríais": "retacar",
    "retacaríamos": "retacar",
    "retacarían": "retacar",
    "retacarías": "retacar",
    "retacas": "retacar",
    "retacase": "retacar",
    "retacaseis": "retacar",
    "retacasen": "retacar",
    "retacases": "retacar",
    "retacaste": "retacar",
    "retacasteis": "retacar",
    "retace": "retazar",
    "retacemos": "retazar",
    "retacen": "retazar",
    "retaceos": "retaceo",
    "retacerías": "retacería",
    "retaces": "retazar",
    "retaco": "retacar",
    "retacos": "retaco",
    "retacábamos": "retacar",
    "retacáis": "retacar",
    "retacáramos": "retacar",
    "retacáremos": "retacar",
    "retacásemos": "retacar",
    "retacé": "retazar",
    "retacéis": "retazar",
    "retacó": "retacar",
    "retad": "retar",
    "retada": "retar",
    "retadas": "retar",
    "retado": "retar",
    "retadora": "retador",
    "retadoras": "retador",
    "retadores": "retador",
    "retados": "retar",
    "retaguardias": "retaguardia",
    "retahílas": "retahíla",
    "retaja": "retajar",
    "retajaba": "retajar",
    "retajabais": "retajar",
    "retajaban": "retajar",
    "retajabas": "retajar",
    "retajad": "retajar",
    "retajada": "retajar",
    "retajadas": "retajar",
    "retajado": "retajar",
    "retajados": "retajar",
    "retajamos": "retajar",
    "retajan": "retajar",
    "retajando": "retajar",
    "retajara": "retajar",
    "retajarais": "retajar",
    "retajaran": "retajar",
    "retajaras": "retajar",
    "retajare": "retajar",
    "retajareis": "retajar",
    "retajaremos": "retajar",
    "retajaren": "retajar",
    "retajares": "retajar",
    "retajaron": "retajar",
    "retajará": "retajar",
    "retajarán": "retajar",
    "retajarás": "retajar",
    "retajaré": "retajar",
    "retajaréis": "retajar",
    "retajaría": "retajar",
    "retajaríais": "retajar",
    "retajaríamos": "retajar",
    "retajarían": "retajar",
    "retajarías": "retajar",
    "retajas": "retajar",
    "retajase": "retajar",
    "retajaseis": "retajar",
    "retajasen": "retajar",
    "retajases": "retajar",
    "retajaste": "retajar",
    "retajasteis": "retajar",
    "retaje": "retajar",
    "retajemos": "retajar",
    "retajen": "retajar",
    "retajes": "retajar",
    "retajo": "retajar",
    "retajos": "retajo",
    "retajábamos": "retajar",
    "retajáis": "retajar",
    "retajáramos": "retajar",
    "retajáremos": "retajar",
    "retajásemos": "retajar",
    "retajé": "retajar",
    "retajéis": "retajar",
    "retajó": "retajar",
    "retales": "retal",
    "retalla": "retallar",
    "retallaba": "retallar",
    "retallabais": "retallar",
    "retallaban": "retallar",
    "retallabas": "retallar",
    "retallad": "retallar",
    "retallada": "retallar",
    "retalladas": "retallar",
    "retallado": "retallar",
    "retallados": "retallar",
    "retallamos": "retallar",
    "retallan": "retallar",
    "retallando": "retallar",
    "retallara": "retallar",
    "retallarais": "retallar",
    "retallaran": "retallar",
    "retallaras": "retallar",
    "retallare": "retallar",
    "retallareis": "retallar",
    "retallaremos": "retallar",
    "retallaren": "retallar",
    "retallares": "retallar",
    "retallaron": "retallar",
    "retallará": "retallar",
    "retallarán": "retallar",
    "retallarás": "retallar",
    "retallaré": "retallar",
    "retallaréis": "retallar",
    "retallaría": "retallar",
    "retallaríais": "retallar",
    "retallaríamos": "retallar",
    "retallarían": "retallar",
    "retallarías": "retallar",
    "retallas": "retallar",
    "retallase": "retallar",
    "retallaseis": "retallar",
    "retallasen": "retallar",
    "retallases": "retallar",
    "retallaste": "retallar",
    "retallasteis": "retallar",
    "retalle": "retallar",
    "retallece": "retallecer",
    "retalleced": "retallecer",
    "retallecemos": "retallecer",
    "retallecen": "retallecer",
    "retalleceremos": "retallecer",
    "retallecerá": "retallecer",
    "retallecerán": "retallecer",
    "retallecerás": "retallecer",
    "retalleceré": "retallecer",
    "retalleceréis": "retallecer",
    "retallecería": "retallecer",
    "retalleceríais": "retallecer",
    "retalleceríamos": "retallecer",
    "retallecerían": "retallecer",
    "retallecerías": "retallecer",
    "retalleces": "retallecer",
    "retallecida": "retallecer",
    "retallecidas": "retallecer",
    "retallecido": "retallecer",
    "retallecidos": "retallecer",
    "retalleciendo": "retallecer",
    "retalleciera": "retallecer",
    "retallecierais": "retallecer",
    "retallecieran": "retallecer",
    "retallecieras": "retallecer",
    "retalleciere": "retallecer",
    "retalleciereis": "retallecer",
    "retallecieren": "retallecer",
    "retallecieres": "retallecer",
    "retallecieron": "retallecer",
    "retalleciese": "retallecer",
    "retallecieseis": "retallecer",
    "retalleciesen": "retallecer",
    "retallecieses": "retallecer",
    "retallecimos": "retallecer",
    "retalleciste": "retallecer",
    "retallecisteis": "retallecer",
    "retalleciéramos": "retallecer",
    "retalleciéremos": "retallecer",
    "retalleciésemos": "retallecer",
    "retalleció": "retallecer",
    "retallecéis": "retallecer",
    "retallecí": "retallecer",
    "retallecía": "retallecer",
    "retallecíais": "retallecer",
    "retallecíamos": "retallecer",
    "retallecían": "retallecer",
    "retallecías": "retallecer",
    "retallemos": "retallar",
    "retallen": "retallar",
    "retalles": "retallar",
    "retallezca": "retallecer",
    "retallezcamos": "retallecer",
    "retallezcan": "retallecer",
    "retallezcas": "retallecer",
    "retallezco": "retallecer",
    "retallezcáis": "retallecer",
    "retallo": "retallar",
    "retallos": "retallo",
    "retallábamos": "retallar",
    "retalláis": "retallar",
    "retalláramos": "retallar",
    "retalláremos": "retallar",
    "retallásemos": "retallar",
    "retallé": "retallar",
    "retalléis": "retallar",
    "retalló": "retallar",
    "retamales": "retamal",
    "retamares": "retamar",
    "retamas": "retama",
    "retamera": "retamero",
    "retameras": "retamero",
    "retameros": "retamero",
    "retamos": "retar",
    "retan": "retar",
    "retando": "retar",
    "retaque": "retacar",
    "retaquemos": "retacar",
    "retaquen": "retacar",
    "retaques": "retacar",
    "retaqué": "retacar",
    "retaquéis": "retacar",
    "retara": "retar",
    "retarais": "retar",
    "retaran": "retar",
    "retaras": "retar",
    "retarda": "retardar",
    "retardaba": "retardar",
    "retardabais": "retardar",
    "retardaban": "retardar",
    "retardabas": "retardar",
    "retardaciones": "retardación",
    "retardad": "retardar",
    "retardada": "retardar",
    "retardadas": "retardar",
    "retardado": "retardar",
    "retardadora": "retardador",
    "retardadoras": "retardador",
    "retardadores": "retardador",
    "retardados": "retardar",
    "retardamos": "retardar",
    "retardan": "retardar",
    "retardando": "retardar",
    "retardantes": "retardante",
    "retardara": "retardar",
    "retardarais": "retardar",
    "retardaran": "retardar",
    "retardaras": "retardar",
    "retardare": "retardar",
    "retardareis": "retardar",
    "retardaremos": "retardar",
    "retardaren": "retardar",
    "retardares": "retardar",
    "retardaron": "retardar",
    "retardará": "retardar",
    "retardarán": "retardar",
    "retardarás": "retardar",
    "retardaré": "retardar",
    "retardaréis": "retardar",
    "retardaría": "retardar",
    "retardaríais": "retardar",
    "retardaríamos": "retardar",
    "retardarían": "retardar",
    "retardarías": "retardar",
    "retardas": "retardar",
    "retardase": "retardar",
    "retardaseis": "retardar",
    "retardasen": "retardar",
    "retardases": "retardar",
    "retardaste": "retardar",
    "retardasteis": "retardar",
    "retardataria": "retardatario",
    "retardatarias": "retardatario",
    "retardatarios": "retardatario",
    "retardativa": "retardativo",
    "retardativas": "retardativo",
    "retardativos": "retardativo",
    "retardatoria": "retardatorio",
    "retardatorias": "retardatorio",
    "retardatorios": "retardatorio",
    "retardatrices": "retardatriz",
    "retarde": "retardar",
    "retardemos": "retardar",
    "retarden": "retardar",
    "retardes": "retardar",
    "retardo": "retardar",
    "retardos": "retardo",
    "retardábamos": "retardar",
    "retardáis": "retardar",
    "retardáramos": "retardar",
    "retardáremos": "retardar",
    "retardásemos": "retardar",
    "retardé": "retardar",
    "retardéis": "retardar",
    "retardó": "retardar",
    "retare": "retar",
    "retareis": "retar",
    "retaremos": "retar",
    "retaren": "retar",
    "retares": "retar",
    "retaron": "retar",
    "retará": "retar",
    "retarán": "retar",
    "retarás": "retar",
    "retaré": "retar",
    "retaréis": "retar",
    "retaría": "retar",
    "retaríais": "retar",
    "retaríamos": "retar",
    "retarían": "retar",
    "retarías": "retar",
    "retas": "retar",
    "retasa": "retasar",
    "retasaba": "retasar",
    "retasabais": "retasar",
    "retasaban": "retasar",
    "retasabas": "retasar",
    "retasaciones": "retasación",
    "retasad": "retasar",
    "retasada": "retasar",
    "retasadas": "retasar",
    "retasado": "retasar",
    "retasados": "retasar",
    "retasamos": "retasar",
    "retasan": "retasar",
    "retasando": "retasar",
    "retasara": "retasar",
    "retasarais": "retasar",
    "retasaran": "retasar",
    "retasaras": "retasar",
    "retasare": "retasar",
    "retasareis": "retasar",
    "retasaremos": "retasar",
    "retasaren": "retasar",
    "retasares": "retasar",
    "retasaron": "retasar",
    "retasará": "retasar",
    "retasarán": "retasar",
    "retasarás": "retasar",
    "retasaré": "retasar",
    "retasaréis": "retasar",
    "retasaría": "retasar",
    "retasaríais": "retasar",
    "retasaríamos": "retasar",
    "retasarían": "retasar",
    "retasarías": "retasar",
    "retasas": "retasar",
    "retasase": "retasar",
    "retasaseis": "retasar",
    "retasasen": "retasar",
    "retasases": "retasar",
    "retasaste": "retasar",
    "retasasteis": "retasar",
    "retase": "retasar",
    "retaseis": "retar",
    "retasemos": "retasar",
    "retasen": "retasar",
    "retases": "retasar",
    "retaso": "retasar",
    "retaste": "retar",
    "retasteis": "retar",
    "retasábamos": "retasar",
    "retasáis": "retasar",
    "retasáramos": "retasar",
    "retasáremos": "retasar",
    "retasásemos": "retasar",
    "retasé": "retasar",
    "retaséis": "retasar",
    "retasó": "retasar",
    "retaza": "retazar",
    "retazaba": "retazar",
    "retazabais": "retazar",
    "retazaban": "retazar",
    "retazabas": "retazar",
    "retazad": "retazar",
    "retazada": "retazar",
    "retazadas": "retazar",
    "retazado": "retazar",
    "retazados": "retazar",
    "retazamos": "retazar",
    "retazan": "retazar",
    "retazando": "retazar",
    "retazara": "retazar",
    "retazarais": "retazar",
    "retazaran": "retazar",
    "retazaras": "retazar",
    "retazare": "retazar",
    "retazareis": "retazar",
    "retazaremos": "retazar",
    "retazaren": "retazar",
    "retazares": "retazar",
    "retazaron": "retazar",
    "retazará": "retazar",
    "retazarán": "retazar",
    "retazarás": "retazar",
    "retazaré": "retazar",
    "retazaréis": "retazar",
    "retazaría": "retazar",
    "retazaríais": "retazar",
    "retazaríamos": "retazar",
    "retazarían": "retazar",
    "retazarías": "retazar",
    "retazas": "retazar",
    "retazase": "retazar",
    "retazaseis": "retazar",
    "retazasen": "retazar",
    "retazases": "retazar",
    "retazaste": "retazar",
    "retazasteis": "retazar",
    "retazo": "retazar",
    "retazos": "retazo",
    "retazábamos": "retazar",
    "retazáis": "retazar",
    "retazáramos": "retazar",
    "retazáremos": "retazar",
    "retazásemos": "retazar",
    "retazó": "retazar",
    "rete": "retar",
    "retecleados": "retecleado",
    "reteja": "retejer",
    "retejaba": "retejar",
    "retejabais": "retejar",
    "retejaban": "retejar",
    "retejabas": "retejar",
    "retejad": "retejar",
    "retejada": "retejar",
    "retejadas": "retejar",
    "retejado": "retejar",
    "retejadores": "retejador",
    "retejados": "retejar",
    "retejamos": "retejer",
    "retejan": "retejer",
    "retejando": "retejar",
    "retejara": "retejar",
    "retejarais": "retejar",
    "retejaran": "retejar",
    "retejaras": "retejar",
    "retejare": "retejar",
    "retejareis": "retejar",
    "retejaremos": "retejar",
    "retejaren": "retejar",
    "retejares": "retejar",
    "retejaron": "retejar",
    "retejará": "retejar",
    "retejarán": "retejar",
    "retejarás": "retejar",
    "retejaré": "retejar",
    "retejaréis": "retejar",
    "retejaría": "retejar",
    "retejaríais": "retejar",
    "retejaríamos": "retejar",
    "retejarían": "retejar",
    "retejarías": "retejar",
    "retejas": "retejer",
    "retejase": "retejar",
    "retejaseis": "retejar",
    "retejasen": "retejar",
    "retejases": "retejar",
    "retejaste": "retejar",
    "retejasteis": "retejar",
    "reteje": "retejer",
    "retejed": "retejer",
    "retejemos": "retejer",
    "retejen": "retejer",
    "retejeremos": "retejer",
    "retejerá": "retejer",
    "retejerán": "retejer",
    "retejerás": "retejer",
    "retejeré": "retejer",
    "retejeréis": "retejer",
    "retejería": "retejer",
    "retejeríais": "retejer",
    "retejeríamos": "retejer",
    "retejerían": "retejer",
    "retejerías": "retejer",
    "retejes": "retejer",
    "retejida": "retejer",
    "retejidas": "retejer",
    "retejido": "retejer",
    "retejidos": "retejer",
    "retejiendo": "retejer",
    "retejiera": "retejer",
    "retejierais": "retejer",
    "retejieran": "retejer",
    "retejieras": "retejer",
    "retejiere": "retejer",
    "retejiereis": "retejer",
    "retejieren": "retejer",
    "retejieres": "retejer",
    "retejieron": "retejer",
    "retejiese": "retejer",
    "retejieseis": "retejer",
    "retejiesen": "retejer",
    "retejieses": "retejer",
    "retejimos": "retejer",
    "retejiste": "retejer",
    "retejisteis": "retejer",
    "retejiéramos": "retejer",
    "retejiéremos": "retejer",
    "retejiésemos": "retejer",
    "retejió": "retejer",
    "retejo": "retejer",
    "retejos": "retejo",
    "retejábamos": "retejar",
    "retejáis": "retejer",
    "retejáramos": "retejar",
    "retejáremos": "retejar",
    "retejásemos": "retejar",
    "retejé": "retejar",
    "retejéis": "retejer",
    "retejí": "retejer",
    "retejía": "retejer",
    "retejíais": "retejer",
    "retejíamos": "retejer",
    "retejían": "retejer",
    "retejías": "retejer",
    "retejó": "retejar",
    "reteles": "retel",
    "retemblaba": "retemblar",
    "retemblabais": "retemblar",
    "retemblaban": "retemblar",
    "retemblabas": "retemblar",
    "retemblad": "retemblar",
    "retemblada": "retemblar",
    "retembladas": "retemblar",
    "retemblado": "retemblar",
    "retemblados": "retemblar",
    "retemblamos": "retemblar",
    "retemblando": "retemblar",
    "retemblara": "retemblar",
    "retemblarais": "retemblar",
    "retemblaran": "retemblar",
    "retemblaras": "retemblar",
    "retemblare": "retemblar",
    "retemblareis": "retemblar",
    "retemblaremos": "retemblar",
    "retemblaren": "retemblar",
    "retemblares": "retemblar",
    "retemblaron": "retemblar",
    "retemblará": "retemblar",
    "retemblarán": "retemblar",
    "retemblarás": "retemblar",
    "retemblaré": "retemblar",
    "retemblaréis": "retemblar",
    "retemblaría": "retemblar",
    "retemblaríais": "retemblar",
    "retemblaríamos": "retemblar",
    "retemblarían": "retemblar",
    "retemblarías": "retemblar",
    "retemblase": "retemblar",
    "retemblaseis": "retemblar",
    "retemblasen": "retemblar",
    "retemblases": "retemblar",
    "retemblaste": "retemblar",
    "retemblasteis": "retemblar",
    "retemblemos": "retemblar",
    "retemblábamos": "retemblar",
    "retembláis": "retemblar",
    "retembláramos": "retemblar",
    "retembláremos": "retemblar",
    "retemblásemos": "retemblar",
    "retemblé": "retemblar",
    "retembléis": "retemblar",
    "retembló": "retemblar",
    "retemos": "retar",
    "reten": "retar",
    "retenciones": "retención",
    "retendremos": "retener",
    "retendrá": "retener",
    "retendrán": "retener",
    "retendrás": "retener",
    "retendré": "retener",
    "retendréis": "retener",
    "retendría": "retener",
    "retendríais": "retener",
    "retendríamos": "retener",
    "retendrían": "retener",
    "retendrías": "retener",
    "retened": "retener",
    "retenedora": "retenedor",
    "retenedoras": "retenedor",
    "retenedores": "retenedor",
    "retenemos": "retener",
    "retenes": "retén",
    "retenga": "retener",
    "retengamos": "retener",
    "retengan": "retener",
    "retengas": "retener",
    "retengo": "retener",
    "retengáis": "retener",
    "retenida": "retener",
    "retenidas": "retenida",
    "retenido": "retener",
    "retenidos": "retener",
    "reteniendo": "retener",
    "retenimientos": "retenimiento",
    "retentiva": "retentivo",
    "retentivas": "retentivo",
    "retentivos": "retentivo",
    "retenéis": "retener",
    "retenía": "retener",
    "reteníais": "retener",
    "reteníamos": "retener",
    "retenían": "retener",
    "retenías": "retener",
    "retes": "retar",
    "retesa": "retesar",
    "retesaba": "retesar",
    "retesabais": "retesar",
    "retesaban": "retesar",
    "retesabas": "retesar",
    "retesad": "retesar",
    "retesada": "retesar",
    "retesadas": "retesar",
    "retesado": "retesar",
    "retesados": "retesar",
    "retesamientos": "retesamiento",
    "retesamos": "retesar",
    "retesan": "retesar",
    "retesando": "retesar",
    "retesara": "retesar",
    "retesarais": "retesar",
    "retesaran": "retesar",
    "retesaras": "retesar",
    "retesare": "retesar",
    "retesareis": "retesar",
    "retesaremos": "retesar",
    "retesaren": "retesar",
    "retesares": "retesar",
    "retesaron": "retesar",
    "retesará": "retesar",
    "retesarán": "retesar",
    "retesarás": "retesar",
    "retesaré": "retesar",
    "retesaréis": "retesar",
    "retesaría": "retesar",
    "retesaríais": "retesar",
    "retesaríamos": "retesar",
    "retesarían": "retesar",
    "retesarías": "retesar",
    "retesas": "retesar",
    "retesase": "retesar",
    "retesaseis": "retesar",
    "retesasen": "retesar",
    "retesases": "retesar",
    "retesaste": "retesar",
    "retesasteis": "retesar",
    "retese": "retesar",
    "retesemos": "retesar",
    "retesen": "retesar",
    "reteses": "retesar",
    "reteso": "retesar",
    "retesos": "reteso",
    "retesábamos": "retesar",
    "retesáis": "retesar",
    "retesáramos": "retesar",
    "retesáremos": "retesar",
    "retesásemos": "retesar",
    "retesé": "retesar",
    "reteséis": "retesar",
    "retesó": "retesar",
    "reteñid": "reteñir",
    "reteñida": "reteñir",
    "reteñidas": "reteñir",
    "reteñido": "reteñir",
    "reteñidos": "reteñir",
    "reteñimos": "reteñir",
    "reteñiremos": "reteñir",
    "reteñirá": "reteñir",
    "reteñirán": "reteñir",
    "reteñirás": "reteñir",
    "reteñiré": "reteñir",
    "reteñiréis": "reteñir",
    "reteñiría": "reteñir",
    "reteñiríais": "reteñir",
    "reteñiríamos": "reteñir",
    "reteñirían": "reteñir",
    "reteñirías": "reteñir",
    "reteñiste": "reteñir",
    "reteñisteis": "reteñir",
    "reteñí": "reteñir",
    "reteñía": "reteñir",
    "reteñíais": "reteñir",
    "reteñíamos": "reteñir",
    "reteñían": "reteñir",
    "reteñías": "reteñir",
    "reteñís": "reteñir",
    "reticencias": "reticencia",
    "reticentes": "reticente",
    "reticulada": "reticulado",
    "reticuladas": "reticulado",
    "reticulados": "reticulado",
    "reticulares": "reticular",
    "reticulinas": "reticulina",
    "reticulocitopenias": "reticulocitopenia",
    "reticulocitos": "reticulocito",
    "reticuloendoteliales": "reticuloendotelial",
    "retiembla": "retemblar",
    "retiemblan": "retemblar",
    "retiemblas": "retemblar",
    "retiemble": "retemblar",
    "retiemblen": "retemblar",
    "retiembles": "retemblar",
    "retiemblo": "retemblar",
    "retiene": "retener",
    "retienen": "retener",
    "retienes": "retener",
    "retientas": "retienta",
    "retinas": "retina",
    "retines": "retín",
    "retiniana": "retiniano",
    "retinianas": "retiniano",
    "retinianos": "retiniano",
    "retinoblastomas": "retinoblastoma",
    "retinocoroidea": "retinocoroideo",
    "retinocoroideas": "retinocoroideo",
    "retinocoroideos": "retinocoroideo",
    "retinografías": "retinografía",
    "retinoles": "retinol",
    "retinomalacias": "retinomalacia",
    "retinopatías": "retinopatía",
    "retinoscopias": "retinoscopia",
    "retinoscopios": "retinoscopio",
    "retintes": "retinte",
    "retintines": "retintín",
    "retinturas": "retintura",
    "retináculos": "retináculo",
    "retira": "retirar",
    "retiraba": "retirar",
    "retirabais": "retirar",
    "retiraban": "retirar",
    "retirabas": "retirar",
    "retiraciones": "retiración",
    "retirad": "retirar",
    "retirada": "retirar",
    "retiradas": "retirar",
    "retirado": "retirar",
    "retirados": "retirar",
    "retiramientos": "retiramiento",
    "retiramos": "retirar",
    "retiran": "retirar",
    "retirando": "retirar",
    "retirara": "retirar",
    "retirarais": "retirar",
    "retiraran": "retirar",
    "retiraras": "retirar",
    "retirare": "retirar",
    "retirareis": "retirar",
    "retiraremos": "retirar",
    "retiraren": "retirar",
    "retirares": "retirar",
    "retiraron": "retirar",
    "retirará": "retirar",
    "retirarán": "retirar",
    "retirarás": "retirar",
    "retiraré": "retirar",
    "retiraréis": "retirar",
    "retiraría": "retirar",
    "retiraríais": "retirar",
    "retiraríamos": "retirar",
    "retirarían": "retirar",
    "retirarías": "retirar",
    "retiras": "retirar",
    "retirase": "retirar",
    "retiraseis": "retirar",
    "retirasen": "retirar",
    "retirases": "retirar",
    "retiraste": "retirar",
    "retirasteis": "retirar",
    "retire": "retirar",
    "retiremos": "retirar",
    "retiren": "retirar",
    "retires": "retirar",
    "retiro": "retirar",
    "retiros": "retiro",
    "retirábamos": "retirar",
    "retiráis": "retirar",
    "retiráramos": "retirar",
    "retiráremos": "retirar",
    "retirásemos": "retirar",
    "retiré": "retirar",
    "retiréis": "retirar",
    "retiró": "retirar",
    "retiña": "retiñir",
    "retiñamos": "retiñir",
    "retiñan": "retiñir",
    "retiñas": "retiñir",
    "retiñe": "retiñir",
    "retiñen": "retiñir",
    "retiñendo": "retiñir",
    "retiñera": "retiñir",
    "retiñerais": "retiñir",
    "retiñeran": "retiñir",
    "retiñeras": "retiñir",
    "retiñere": "retiñir",
    "retiñereis": "retiñir",
    "retiñeren": "retiñir",
    "retiñeres": "retiñir",
    "retiñeron": "retiñir",
    "retiñes": "retiñir",
    "retiñese": "retiñir",
    "retiñeseis": "retiñir",
    "retiñesen": "retiñir",
    "retiñeses": "retiñir",
    "retiñid": "retiñir",
    "retiñida": "retiñir",
    "retiñidas": "retiñir",
    "retiñido": "retiñir",
    "retiñidos": "retiñir",
    "retiñimos": "retiñir",
    "retiñiremos": "retiñir",
    "retiñirá": "retiñir",
    "retiñirán": "retiñir",
    "retiñirás": "retiñir",
    "retiñiré": "retiñir",
    "retiñiréis": "retiñir",
    "retiñiría": "retiñir",
    "retiñiríais": "retiñir",
    "retiñiríamos": "retiñir",
    "retiñirían": "retiñir",
    "retiñirías": "retiñir",
    "retiñiste": "retiñir",
    "retiñisteis": "retiñir",
    "retiño": "retiñir",
    "retiñáis": "retiñir",
    "retiñéramos": "retiñir",
    "retiñéremos": "retiñir",
    "retiñésemos": "retiñir",
    "retiñí": "retiñir",
    "retiñía": "retiñir",
    "retiñíais": "retiñir",
    "retiñíamos": "retiñir",
    "retiñían": "retiñir",
    "retiñías": "retiñir",
    "retiñís": "retiñir",
    "retiñó": "retiñir",
    "reto": "retar",
    "retoba": "retobar",
    "retobaba": "retobar",
    "retobabais": "retobar",
    "retobaban": "retobar",
    "retobabas": "retobar",
    "retobad": "retobar",
    "retobada": "retobar",
    "retobadas": "retobar",
    "retobado": "retobar",
    "retobados": "retobar",
    "retobamos": "retobar",
    "retoban": "retobar",
    "retobando": "retobar",
    "retobara": "retobar",
    "retobarais": "retobar",
    "retobaran": "retobar",
    "retobaras": "retobar",
    "retobare": "retobar",
    "retobareis": "retobar",
    "retobaremos": "retobar",
    "retobaren": "retobar",
    "retobares": "retobar",
    "retobaron": "retobar",
    "retobará": "retobar",
    "retobarán": "retobar",
    "retobarás": "retobar",
    "retobaré": "retobar",
    "retobaréis": "retobar",
    "retobaría": "retobar",
    "retobaríais": "retobar",
    "retobaríamos": "retobar",
    "retobarían": "retobar",
    "retobarías": "retobar",
    "retobas": "retobar",
    "retobase": "retobar",
    "retobaseis": "retobar",
    "retobasen": "retobar",
    "retobases": "retobar",
    "retobaste": "retobar",
    "retobasteis": "retobar",
    "retobe": "retobar",
    "retobemos": "retobar",
    "retoben": "retobar",
    "retobes": "retobar",
    "retobo": "retobar",
    "retobábamos": "retobar",
    "retobáis": "retobar",
    "retobáramos": "retobar",
    "retobáremos": "retobar",
    "retobásemos": "retobar",
    "retobé": "retobar",
    "retobéis": "retobar",
    "retobó": "retobar",
    "retoca": "retocar",
    "retocaba": "retocar",
    "retocabais": "retocar",
    "retocaban": "retocar",
    "retocabas": "retocar",
    "retocad": "retocar",
    "retocada": "retocar",
    "retocadas": "retocar",
    "retocado": "retocar",
    "retocadora": "retocador",
    "retocadoras": "retocador",
    "retocadores": "retocador",
    "retocados": "retocar",
    "retocamos": "retocar",
    "retocan": "retocar",
    "retocando": "retocar",
    "retocara": "retocar",
    "retocarais": "retocar",
    "retocaran": "retocar",
    "retocaras": "retocar",
    "retocare": "retocar",
    "retocareis": "retocar",
    "retocaremos": "retocar",
    "retocaren": "retocar",
    "retocares": "retocar",
    "retocaron": "retocar",
    "retocará": "retocar",
    "retocarán": "retocar",
    "retocarás": "retocar",
    "retocaré": "retocar",
    "retocaréis": "retocar",
    "retocaría": "retocar",
    "retocaríais": "retocar",
    "retocaríamos": "retocar",
    "retocarían": "retocar",
    "retocarías": "retocar",
    "retocas": "retocar",
    "retocase": "retocar",
    "retocaseis": "retocar",
    "retocasen": "retocar",
    "retocases": "retocar",
    "retocaste": "retocar",
    "retocasteis": "retocar",
    "retoce": "retozar",
    "retocemos": "retozar",
    "retocen": "retozar",
    "retoces": "retozar",
    "retoco": "retocar",
    "retocábamos": "retocar",
    "retocáis": "retocar",
    "retocáramos": "retocar",
    "retocáremos": "retocar",
    "retocásemos": "retocar",
    "retocé": "retozar",
    "retocéis": "retozar",
    "retocó": "retocar",
    "retoma": "retomar",
    "retomaba": "retomar",
    "retomabais": "retomar",
    "retomaban": "retomar",
    "retomabas": "retomar",
    "retomad": "retomar",
    "retomada": "retomar",
    "retomadas": "retomar",
    "retomado": "retomar",
    "retomados": "retomar",
    "retomamos": "retomar",
    "retoman": "retomar",
    "retomando": "retomar",
    "retomara": "retomar",
    "retomarais": "retomar",
    "retomaran": "retomar",
    "retomaras": "retomar",
    "retomare": "retomar",
    "retomareis": "retomar",
    "retomaremos": "retomar",
    "retomaren": "retomar",
    "retomares": "retomar",
    "retomaron": "retomar",
    "retomará": "retomar",
    "retomarán": "retomar",
    "retomarás": "retomar",
    "retomaré": "retomar",
    "retomaréis": "retomar",
    "retomaría": "retomar",
    "retomaríais": "retomar",
    "retomaríamos": "retomar",
    "retomarían": "retomar",
    "retomarías": "retomar",
    "retomas": "retomar",
    "retomase": "retomar",
    "retomaseis": "retomar",
    "retomasen": "retomar",
    "retomases": "retomar",
    "retomaste": "retomar",
    "retomasteis": "retomar",
    "retome": "retomar",
    "retomedos": "retomedo",
    "retomemos": "retomar",
    "retomen": "retomar",
    "retomes": "retomar",
    "retomo": "retomar",
    "retomábamos": "retomar",
    "retomáis": "retomar",
    "retomáramos": "retomar",
    "retomáremos": "retomar",
    "retomásemos": "retomar",
    "retomé": "retomar",
    "retoméis": "retomar",
    "retomó": "retomar",
    "retoque": "retocar",
    "retoquemos": "retocar",
    "retoquen": "retocar",
    "retoques": "retoque",
    "retoqué": "retocar",
    "retoquéis": "retocar",
    "retorced": "retorcer",
    "retorceduras": "retorcedura",
    "retorcemos": "retorcer",
    "retorceremos": "retorcer",
    "retorcerá": "retorcer",
    "retorcerán": "retorcer",
    "retorcerás": "retorcer",
    "retorceré": "retorcer",
    "retorceréis": "retorcer",
    "retorcería": "retorcer",
    "retorceríais": "retorcer",
    "retorceríamos": "retorcer",
    "retorcerían": "retorcer",
    "retorcerías": "retorcer",
    "retorcida": "retorcer",
    "retorcidas": "retorcer",
    "retorcido": "retorcer",
    "retorcidos": "retorcer",
    "retorciendo": "retorcer",
    "retorciera": "retorcer",
    "retorcierais": "retorcer",
    "retorcieran": "retorcer",
    "retorcieras": "retorcer",
    "retorciere": "retorcer",
    "retorciereis": "retorcer",
    "retorcieren": "retorcer",
    "retorcieres": "retorcer",
    "retorcieron": "retorcer",
    "retorciese": "retorcer",
    "retorcieseis": "retorcer",
    "retorciesen": "retorcer",
    "retorcieses": "retorcer",
    "retorcimientos": "retorcimiento",
    "retorcimos": "retorcer",
    "retorciste": "retorcer",
    "retorcisteis": "retorcer",
    "retorciéramos": "retorcer",
    "retorciéremos": "retorcer",
    "retorciésemos": "retorcer",
    "retorció": "retorcer",
    "retorcéis": "retorcer",
    "retorcí": "retorcer",
    "retorcía": "retorcer",
    "retorcíais": "retorcer",
    "retorcíamos": "retorcer",
    "retorcían": "retorcer",
    "retorcías": "retorcer",
    "retores": "retor",
    "retorna": "retornar",
    "retornaba": "retornar",
    "retornabais": "retornar",
    "retornaban": "retornar",
    "retornabas": "retornar",
    "retornables": "retornable",
    "retornad": "retornar",
    "retornada": "retornar",
    "retornadas": "retornar",
    "retornado": "retornar",
    "retornados": "retornar",
    "retornamientos": "retornamiento",
    "retornamos": "retornar",
    "retornan": "retornar",
    "retornando": "retornar",
    "retornara": "retornar",
    "retornarais": "retornar",
    "retornaran": "retornar",
    "retornaras": "retornar",
    "retornare": "retornar",
    "retornareis": "retornar",
    "retornaremos": "retornar",
    "retornaren": "retornar",
    "retornares": "retornar",
    "retornaron": "retornar",
    "retornará": "retornar",
    "retornarán": "retornar",
    "retornarás": "retornar",
    "retornaré": "retornar",
    "retornaréis": "retornar",
    "retornaría": "retornar",
    "retornaríais": "retornar",
    "retornaríamos": "retornar",
    "retornarían": "retornar",
    "retornarías": "retornar",
    "retornas": "retornar",
    "retornase": "retornar",
    "retornaseis": "retornar",
    "retornasen": "retornar",
    "retornases": "retornar",
    "retornaste": "retornar",
    "retornasteis": "retornar",
    "retorne": "retornar",
    "retornelos": "retornelo",
    "retornemos": "retornar",
    "retornen": "retornar",
    "retornes": "retornar",
    "retorno": "retornar",
    "retornos": "retorno",
    "retornábamos": "retornar",
    "retornáis": "retornar",
    "retornáramos": "retornar",
    "retornáremos": "retornar",
    "retornásemos": "retornar",
    "retorné": "retornar",
    "retornéis": "retornar",
    "retornó": "retornar",
    "retorromana": "retorromano",
    "retorromanas": "retorromano",
    "retorromanos": "retorromano",
    "retorrománica": "retorrománico",
    "retorrománicas": "retorrománico",
    "retorrománicos": "retorrománico",
    "retorsiones": "retorsión",
    "retorsiva": "retorsivo",
    "retorsivas": "retorsivo",
    "retorsivos": "retorsivo",
    "retortas": "retorta",
    "retorteros": "retortero",
    "retortijones": "retortijón",
    "retorzamos": "retorcer",
    "retorzáis": "retorcer",
    "retos": "reto",
    "retostaba": "retostar",
    "retostabais": "retostar",
    "retostaban": "retostar",
    "retostabas": "retostar",
    "retostad": "retostar",
    "retostada": "retostar",
    "retostadas": "retostar",
    "retostado": "retostar",
    "retostados": "retostar",
    "retostamos": "retostar",
    "retostando": "retostar",
    "retostara": "retostar",
    "retostarais": "retostar",
    "retostaran": "retostar",
    "retostaras": "retostar",
    "retostare": "retostar",
    "retostareis": "retostar",
    "retostaremos": "retostar",
    "retostaren": "retostar",
    "retostares": "retostar",
    "retostaron": "retostar",
    "retostará": "retostar",
    "retostarán": "retostar",
    "retostarás": "retostar",
    "retostaré": "retostar",
    "retostaréis": "retostar",
    "retostaría": "retostar",
    "retostaríais": "retostar",
    "retostaríamos": "retostar",
    "retostarían": "retostar",
    "retostarías": "retostar",
    "retostase": "retostar",
    "retostaseis": "retostar",
    "retostasen": "retostar",
    "retostases": "retostar",
    "retostaste": "retostar",
    "retostasteis": "retostar",
    "retostemos": "retostar",
    "retostábamos": "retostar",
    "retostáis": "retostar",
    "retostáramos": "retostar",
    "retostáremos": "retostar",
    "retostásemos": "retostar",
    "retosté": "retostar",
    "retostéis": "retostar",
    "retostó": "retostar",
    "retoza": "retozar",
    "retozaba": "retozar",
    "retozabais": "retozar",
    "retozaban": "retozar",
    "retozabas": "retozar",
    "retozad": "retozar",
    "retozada": "retozar",
    "retozadas": "retozar",
    "retozado": "retozar",
    "retozadora": "retozador",
    "retozadoras": "retozador",
    "retozadores": "retozador",
    "retozados": "retozar",
    "retozaduras": "retozadura",
    "retozamos": "retozar",
    "retozan": "retozar",
    "retozando": "retozar",
    "retozara": "retozar",
    "retozarais": "retozar",
    "retozaran": "retozar",
    "retozaras": "retozar",
    "retozare": "retozar",
    "retozareis": "retozar",
    "retozaremos": "retozar",
    "retozaren": "retozar",
    "retozares": "retozar",
    "retozaron": "retozar",
    "retozará": "retozar",
    "retozarán": "retozar",
    "retozarás": "retozar",
    "retozaré": "retozar",
    "retozaréis": "retozar",
    "retozaría": "retozar",
    "retozaríais": "retozar",
    "retozaríamos": "retozar",
    "retozarían": "retozar",
    "retozarías": "retozar",
    "retozas": "retozar",
    "retozase": "retozar",
    "retozaseis": "retozar",
    "retozasen": "retozar",
    "retozases": "retozar",
    "retozaste": "retozar",
    "retozasteis": "retozar",
    "retozo": "retozar",
    "retozona": "retozón",
    "retozonas": "retozón",
    "retozones": "retozón",
    "retozos": "retozo",
    "retozábamos": "retozar",
    "retozáis": "retozar",
    "retozáramos": "retozar",
    "retozáremos": "retozar",
    "retozásemos": "retozar",
    "retozó": "retozar",
    "retoña": "retoñar",
    "retoñaba": "retoñar",
    "retoñabais": "retoñar",
    "retoñaban": "retoñar",
    "retoñabas": "retoñar",
    "retoñad": "retoñar",
    "retoñada": "retoñar",
    "retoñadas": "retoñar",
    "retoñado": "retoñar",
    "retoñados": "retoñar",
    "retoñamos": "retoñar",
    "retoñan": "retoñar",
    "retoñando": "retoñar",
    "retoñara": "retoñar",
    "retoñarais": "retoñar",
    "retoñaran": "retoñar",
    "retoñaras": "retoñar",
    "retoñare": "retoñar",
    "retoñareis": "retoñar",
    "retoñaremos": "retoñar",
    "retoñaren": "retoñar",
    "retoñares": "retoñar",
    "retoñaron": "retoñar",
    "retoñará": "retoñar",
    "retoñarán": "retoñar",
    "retoñarás": "retoñar",
    "retoñaré": "retoñar",
    "retoñaréis": "retoñar",
    "retoñaría": "retoñar",
    "retoñaríais": "retoñar",
    "retoñaríamos": "retoñar",
    "retoñarían": "retoñar",
    "retoñarías": "retoñar",
    "retoñas": "retoñar",
    "retoñase": "retoñar",
    "retoñaseis": "retoñar",
    "retoñasen": "retoñar",
    "retoñases": "retoñar",
    "retoñaste": "retoñar",
    "retoñasteis": "retoñar",
    "retoñe": "retoñar",
    "retoñece": "retoñecer",
    "retoñeced": "retoñecer",
    "retoñecemos": "retoñecer",
    "retoñecen": "retoñecer",
    "retoñeceremos": "retoñecer",
    "retoñecerá": "retoñecer",
    "retoñecerán": "retoñecer",
    "retoñecerás": "retoñecer",
    "retoñeceré": "retoñecer",
    "retoñeceréis": "retoñecer",
    "retoñecería": "retoñecer",
    "retoñeceríais": "retoñecer",
    "retoñeceríamos": "retoñecer",
    "retoñecerían": "retoñecer",
    "retoñecerías": "retoñecer",
    "retoñeces": "retoñecer",
    "retoñecida": "retoñecer",
    "retoñecidas": "retoñecer",
    "retoñecido": "retoñecer",
    "retoñecidos": "retoñecer",
    "retoñeciendo": "retoñecer",
    "retoñeciera": "retoñecer",
    "retoñecierais": "retoñecer",
    "retoñecieran": "retoñecer",
    "retoñecieras": "retoñecer",
    "retoñeciere": "retoñecer",
    "retoñeciereis": "retoñecer",
    "retoñecieren": "retoñecer",
    "retoñecieres": "retoñecer",
    "retoñecieron": "retoñecer",
    "retoñeciese": "retoñecer",
    "retoñecieseis": "retoñecer",
    "retoñeciesen": "retoñecer",
    "retoñecieses": "retoñecer",
    "retoñecimos": "retoñecer",
    "retoñeciste": "retoñecer",
    "retoñecisteis": "retoñecer",
    "retoñeciéramos": "retoñecer",
    "retoñeciéremos": "retoñecer",
    "retoñeciésemos": "retoñecer",
    "retoñeció": "retoñecer",
    "retoñecéis": "retoñecer",
    "retoñecí": "retoñecer",
    "retoñecía": "retoñecer",
    "retoñecíais": "retoñecer",
    "retoñecíamos": "retoñecer",
    "retoñecían": "retoñecer",
    "retoñecías": "retoñecer",
    "retoñemos": "retoñar",
    "retoñen": "retoñar",
    "retoñes": "retoñar",
    "retoñezca": "retoñecer",
    "retoñezcamos": "retoñecer",
    "retoñezcan": "retoñecer",
    "retoñezcas": "retoñecer",
    "retoñezco": "retoñecer",
    "retoñezcáis": "retoñecer",
    "retoño": "retoñar",
    "retoños": "retoño",
    "retoñábamos": "retoñar",
    "retoñáis": "retoñar",
    "retoñáramos": "retoñar",
    "retoñáremos": "retoñar",
    "retoñásemos": "retoñar",
    "retoñé": "retoñar",
    "retoñéis": "retoñar",
    "retoñó": "retoñar",
    "retracciones": "retracción",
    "retracta": "retractar",
    "retractaba": "retractar",
    "retractabais": "retractar",
    "retractaban": "retractar",
    "retractabas": "retractar",
    "retractables": "retractable",
    "retractaciones": "retractación",
    "retractad": "retractar",
    "retractada": "retractar",
    "retractadas": "retractar",
    "retractado": "retractar",
    "retractados": "retractar",
    "retractamos": "retractar",
    "retractan": "retractar",
    "retractando": "retractar",
    "retractara": "retractar",
    "retractarais": "retractar",
    "retractaran": "retractar",
    "retractaras": "retractar",
    "retractare": "retractar",
    "retractareis": "retractar",
    "retractaremos": "retractar",
    "retractaren": "retractar",
    "retractares": "retractar",
    "retractaron": "retractar",
    "retractará": "retractar",
    "retractarán": "retractar",
    "retractarás": "retractar",
    "retractaré": "retractar",
    "retractaréis": "retractar",
    "retractaría": "retractar",
    "retractaríais": "retractar",
    "retractaríamos": "retractar",
    "retractarían": "retractar",
    "retractarías": "retractar",
    "retractas": "retractar",
    "retractase": "retractar",
    "retractaseis": "retractar",
    "retractasen": "retractar",
    "retractases": "retractar",
    "retractaste": "retractar",
    "retractasteis": "retractar",
    "retracte": "retractar",
    "retractemos": "retractar",
    "retracten": "retractar",
    "retractes": "retractar",
    "retractibilidades": "retractibilidad",
    "retractilada": "retractilado",
    "retractiladas": "retractilado",
    "retractilados": "retractilado",
    "retractilidades": "retractilidad",
    "retractiva": "retractivo",
    "retractivas": "retractivo",
    "retractivos": "retractivo",
    "retracto": "retractar",
    "retractora": "retractor",
    "retractoras": "retractor",
    "retractores": "retractor",
    "retractos": "retracto",
    "retractábamos": "retractar",
    "retractáis": "retractar",
    "retractáramos": "retractar",
    "retractáremos": "retractar",
    "retractásemos": "retractar",
    "retracté": "retractar",
    "retractéis": "retractar",
    "retractó": "retractar",
    "retrae": "retraer",
    "retraed": "retraer",
    "retraemos": "retraer",
    "retraen": "retraer",
    "retraeremos": "retraer",
    "retraerá": "retraer",
    "retraerán": "retraer",
    "retraerás": "retraer",
    "retraeré": "retraer",
    "retraeréis": "retraer",
    "retraería": "retraer",
    "retraeríais": "retraer",
    "retraeríamos": "retraer",
    "retraerían": "retraer",
    "retraerías": "retraer",
    "retraes": "retraer",
    "retraiga": "retraer",
    "retraigamos": "retraer",
    "retraigan": "retraer",
    "retraigas": "retraer",
    "retraigo": "retraer",
    "retraigáis": "retraer",
    "retraimientos": "retraimiento",
    "retraje": "retraer",
    "retrajera": "retraer",
    "retrajerais": "retraer",
    "retrajeran": "retraer",
    "retrajeras": "retraer",
    "retrajere": "retraer",
    "retrajereis": "retraer",
    "retrajeren": "retraer",
    "retrajeres": "retraer",
    "retrajeron": "retraer",
    "retrajese": "retraer",
    "retrajeseis": "retraer",
    "retrajesen": "retraer",
    "retrajeses": "retraer",
    "retrajimos": "retraer",
    "retrajiste": "retraer",
    "retrajisteis": "retraer",
    "retrajo": "retraer",
    "retrajéramos": "retraer",
    "retrajéremos": "retraer",
    "retrajésemos": "retraer",
    "retrancas": "retranca",
    "retranquea": "retranquear",
    "retranqueaba": "retranquear",
    "retranqueabais": "retranquear",
    "retranqueaban": "retranquear",
    "retranqueabas": "retranquear",
    "retranquead": "retranquear",
    "retranqueada": "retranquear",
    "retranqueadas": "retranquear",
    "retranqueado": "retranquear",
    "retranqueados": "retranquear",
    "retranqueamos": "retranquear",
    "retranquean": "retranquear",
    "retranqueando": "retranquear",
    "retranqueara": "retranquear",
    "retranquearais": "retranquear",
    "retranquearan": "retranquear",
    "retranquearas": "retranquear",
    "retranqueare": "retranquear",
    "retranqueareis": "retranquear",
    "retranquearemos": "retranquear",
    "retranquearen": "retranquear",
    "retranqueares": "retranquear",
    "retranquearon": "retranquear",
    "retranqueará": "retranquear",
    "retranquearán": "retranquear",
    "retranquearás": "retranquear",
    "retranquearé": "retranquear",
    "retranquearéis": "retranquear",
    "retranquearía": "retranquear",
    "retranquearíais": "retranquear",
    "retranquearíamos": "retranquear",
    "retranquearían": "retranquear",
    "retranquearías": "retranquear",
    "retranqueas": "retranquear",
    "retranquease": "retranquear",
    "retranqueaseis": "retranquear",
    "retranqueasen": "retranquear",
    "retranqueases": "retranquear",
    "retranqueaste": "retranquear",
    "retranqueasteis": "retranquear",
    "retranquee": "retranquear",
    "retranqueemos": "retranquear",
    "retranqueen": "retranquear",
    "retranquees": "retranquear",
    "retranqueo": "retranquear",
    "retranqueos": "retranqueo",
    "retranqueábamos": "retranquear",
    "retranqueáis": "retranquear",
    "retranqueáramos": "retranquear",
    "retranqueáremos": "retranquear",
    "retranqueásemos": "retranquear",
    "retranqueé": "retranquear",
    "retranqueéis": "retranquear",
    "retranqueó": "retranquear",
    "retransmisiones": "retransmisión",
    "retransmisores": "retransmisor",
    "retransmita": "retransmitir",
    "retransmitamos": "retransmitir",
    "retransmitan": "retransmitir",
    "retransmitas": "retransmitir",
    "retransmite": "retransmitir",
    "retransmiten": "retransmitir",
    "retransmites": "retransmitir",
    "retransmitid": "retransmitir",
    "retransmitida": "retransmitir",
    "retransmitidas": "retransmitir",
    "retransmitido": "retransmitir",
    "retransmitidos": "retransmitir",
    "retransmitiendo": "retransmitir",
    "retransmitiera": "retransmitir",
    "retransmitierais": "retransmitir",
    "retransmitieran": "retransmitir",
    "retransmitieras": "retransmitir",
    "retransmitiere": "retransmitir",
    "retransmitiereis": "retransmitir",
    "retransmitieren": "retransmitir",
    "retransmitieres": "retransmitir",
    "retransmitieron": "retransmitir",
    "retransmitiese": "retransmitir",
    "retransmitieseis": "retransmitir",
    "retransmitiesen": "retransmitir",
    "retransmitieses": "retransmitir",
    "retransmitimos": "retransmitir",
    "retransmitiremos": "retransmitir",
    "retransmitirá": "retransmitir",
    "retransmitirán": "retransmitir",
    "retransmitirás": "retransmitir",
    "retransmitiré": "retransmitir",
    "retransmitiréis": "retransmitir",
    "retransmitiría": "retransmitir",
    "retransmitiríais": "retransmitir",
    "retransmitiríamos": "retransmitir",
    "retransmitirían": "retransmitir",
    "retransmitirías": "retransmitir",
    "retransmitiste": "retransmitir",
    "retransmitisteis": "retransmitir",
    "retransmitiéramos": "retransmitir",
    "retransmitiéremos": "retransmitir",
    "retransmitiésemos": "retransmitir",
    "retransmitió": "retransmitir",
    "retransmito": "retransmitir",
    "retransmitáis": "retransmitir",
    "retransmití": "retransmitir",
    "retransmitía": "retransmitir",
    "retransmitíais": "retransmitir",
    "retransmitíamos": "retransmitir",
    "retransmitían": "retransmitir",
    "retransmitías": "retransmitir",
    "retransmitís": "retransmitir",
    "retrasa": "retrasar",
    "retrasaba": "retrasar",
    "retrasabais": "retrasar",
    "retrasaban": "retrasar",
    "retrasabas": "retrasar",
    "retrasad": "retrasar",
    "retrasada": "retrasar",
    "retrasadas": "retrasar",
    "retrasado": "retrasar",
    "retrasados": "retrasar",
    "retrasamos": "retrasar",
    "retrasan": "retrasar",
    "retrasando": "retrasar",
    "retrasara": "retrasar",
    "retrasarais": "retrasar",
    "retrasaran": "retrasar",
    "retrasaras": "retrasar",
    "retrasare": "retrasar",
    "retrasareis": "retrasar",
    "retrasaremos": "retrasar",
    "retrasaren": "retrasar",
    "retrasares": "retrasar",
    "retrasaron": "retrasar",
    "retrasará": "retrasar",
    "retrasarán": "retrasar",
    "retrasarás": "retrasar",
    "retrasaré": "retrasar",
    "retrasaréis": "retrasar",
    "retrasaría": "retrasar",
    "retrasaríais": "retrasar",
    "retrasaríamos": "retrasar",
    "retrasarían": "retrasar",
    "retrasarías": "retrasar",
    "retrasas": "retrasar",
    "retrasase": "retrasar",
    "retrasaseis": "retrasar",
    "retrasasen": "retrasar",
    "retrasases": "retrasar",
    "retrasaste": "retrasar",
    "retrasasteis": "retrasar",
    "retrase": "retrasar",
    "retrasemos": "retrasar",
    "retrasen": "retrasar",
    "retrases": "retrasar",
    "retraso": "retrasar",
    "retrasos": "retraso",
    "retrasábamos": "retrasar",
    "retrasáis": "retrasar",
    "retrasáramos": "retrasar",
    "retrasáremos": "retrasar",
    "retrasásemos": "retrasar",
    "retrasé": "retrasar",
    "retraséis": "retrasar",
    "retrasó": "retrasar",
    "retrata": "retratar",
    "retrataba": "retratar",
    "retratabais": "retratar",
    "retrataban": "retratar",
    "retratabas": "retratar",
    "retratad": "retratar",
    "retratada": "retratar",
    "retratadas": "retratar",
    "retratado": "retratar",
    "retratados": "retratar",
    "retratamos": "retratar",
    "retratan": "retratar",
    "retratando": "retratar",
    "retratara": "retratar",
    "retratarais": "retratar",
    "retrataran": "retratar",
    "retrataras": "retratar",
    "retratare": "retratar",
    "retratareis": "retratar",
    "retrataremos": "retratar",
    "retrataren": "retratar",
    "retratares": "retratar",
    "retrataron": "retratar",
    "retratará": "retratar",
    "retratarán": "retratar",
    "retratarás": "retratar",
    "retrataré": "retratar",
    "retrataréis": "retratar",
    "retrataría": "retratar",
    "retrataríais": "retratar",
    "retrataríamos": "retratar",
    "retratarían": "retratar",
    "retratarías": "retratar",
    "retratas": "retratar",
    "retratase": "retratar",
    "retrataseis": "retratar",
    "retratasen": "retratar",
    "retratases": "retratar",
    "retrataste": "retratar",
    "retratasteis": "retratar",
    "retrate": "retratar",
    "retratemos": "retratar",
    "retraten": "retratar",
    "retrates": "retratar",
    "retratistas": "retratista",
    "retrato": "retratar",
    "retratos": "retrato",
    "retratábamos": "retratar",
    "retratáis": "retratar",
    "retratáramos": "retratar",
    "retratáremos": "retratar",
    "retratásemos": "retratar",
    "retraté": "retratar",
    "retratéis": "retratar",
    "retrató": "retratar",
    "retrayendo": "retraer",
    "retraéis": "retraer",
    "retraía": "retraer",
    "retraíais": "retraer",
    "retraíamos": "retraer",
    "retraían": "retraer",
    "retraías": "retraer",
    "retraída": "retraer",
    "retraídas": "retraer",
    "retraído": "retraer",
    "retraídos": "retraer",
    "retrecha": "retrechar",
    "retrechaba": "retrechar",
    "retrechabais": "retrechar",
    "retrechaban": "retrechar",
    "retrechabas": "retrechar",
    "retrechad": "retrechar",
    "retrechada": "retrechar",
    "retrechadas": "retrechar",
    "retrechado": "retrechar",
    "retrechados": "retrechar",
    "retrechamos": "retrechar",
    "retrechan": "retrechar",
    "retrechando": "retrechar",
    "retrechara": "retrechar",
    "retrecharais": "retrechar",
    "retrecharan": "retrechar",
    "retrecharas": "retrechar",
    "retrechare": "retrechar",
    "retrechareis": "retrechar",
    "retrecharemos": "retrechar",
    "retrecharen": "retrechar",
    "retrechares": "retrechar",
    "retrecharon": "retrechar",
    "retrechará": "retrechar",
    "retrecharán": "retrechar",
    "retrecharás": "retrechar",
    "retrecharé": "retrechar",
    "retrecharéis": "retrechar",
    "retrecharía": "retrechar",
    "retrecharíais": "retrechar",
    "retrecharíamos": "retrechar",
    "retrecharían": "retrechar",
    "retrecharías": "retrechar",
    "retrechas": "retrechar",
    "retrechase": "retrechar",
    "retrechaseis": "retrechar",
    "retrechasen": "retrechar",
    "retrechases": "retrechar",
    "retrechaste": "retrechar",
    "retrechasteis": "retrechar",
    "retreche": "retrechar",
    "retrechemos": "retrechar",
    "retrechen": "retrechar",
    "retrechera": "retrechero",
    "retrecheras": "retrechero",
    "retrecheros": "retrechero",
    "retrecherías": "retrechería",
    "retreches": "retrechar",
    "retrecho": "retrechar",
    "retrechábamos": "retrechar",
    "retrecháis": "retrechar",
    "retrecháramos": "retrechar",
    "retrecháremos": "retrechar",
    "retrechásemos": "retrechar",
    "retreché": "retrechar",
    "retrechéis": "retrechar",
    "retrechó": "retrechar",
    "retrepa": "retrepar",
    "retrepaba": "retrepar",
    "retrepabais": "retrepar",
    "retrepaban": "retrepar",
    "retrepabas": "retrepar",
    "retrepad": "retrepar",
    "retrepada": "retrepar",
    "retrepadas": "retrepar",
    "retrepado": "retrepar",
    "retrepados": "retrepar",
    "retrepamos": "retrepar",
    "retrepan": "retrepar",
    "retrepando": "retrepar",
    "retrepara": "retrepar",
    "retreparais": "retrepar",
    "retreparan": "retrepar",
    "retreparas": "retrepar",
    "retrepare": "retrepar",
    "retrepareis": "retrepar",
    "retreparemos": "retrepar",
    "retreparen": "retrepar",
    "retrepares": "retrepar",
    "retreparon": "retrepar",
    "retrepará": "retrepar",
    "retreparán": "retrepar",
    "retreparás": "retrepar",
    "retreparé": "retrepar",
    "retreparéis": "retrepar",
    "retreparía": "retrepar",
    "retreparíais": "retrepar",
    "retreparíamos": "retrepar",
    "retreparían": "retrepar",
    "retreparías": "retrepar",
    "retrepas": "retrepar",
    "retrepase": "retrepar",
    "retrepaseis": "retrepar",
    "retrepasen": "retrepar",
    "retrepases": "retrepar",
    "retrepaste": "retrepar",
    "retrepasteis": "retrepar",
    "retrepe": "retrepar",
    "retrepemos": "retrepar",
    "retrepen": "retrepar",
    "retrepes": "retrepar",
    "retrepo": "retrepar",
    "retrepábamos": "retrepar",
    "retrepáis": "retrepar",
    "retrepáramos": "retrepar",
    "retrepáremos": "retrepar",
    "retrepásemos": "retrepar",
    "retrepé": "retrepar",
    "retrepéis": "retrepar",
    "retrepó": "retrepar",
    "retretas": "retreta",
    "retretes": "retrete",
    "retribuciones": "retribución",
    "retribuibles": "retribuible",
    "retribuid": "retribuir",
    "retribuida": "retribuir",
    "retribuidas": "retribuir",
    "retribuido": "retribuir",
    "retribuidos": "retribuir",
    "retribuimos": "retribuir",
    "retribuiremos": "retribuir",
    "retribuirá": "retribuir",
    "retribuirán": "retribuir",
    "retribuirás": "retribuir",
    "retribuiré": "retribuir",
    "retribuiréis": "retribuir",
    "retribuiría": "retribuir",
    "retribuiríais": "retribuir",
    "retribuiríamos": "retribuir",
    "retribuirían": "retribuir",
    "retribuirías": "retribuir",
    "retribuiste": "retribuir",
    "retribuisteis": "retribuir",
    "retributiva": "retributivo",
    "retributivas": "retributivo",
    "retributivos": "retributivo",
    "retribuya": "retribuir",
    "retribuyamos": "retribuir",
    "retribuyan": "retribuir",
    "retribuyas": "retribuir",
    "retribuye": "retribuir",
    "retribuyen": "retribuir",
    "retribuyendo": "retribuir",
    "retribuyera": "retribuir",
    "retribuyerais": "retribuir",
    "retribuyeran": "retribuir",
    "retribuyeras": "retribuir",
    "retribuyere": "retribuir",
    "retribuyereis": "retribuir",
    "retribuyeren": "retribuir",
    "retribuyeres": "retribuir",
    "retribuyeron": "retribuir",
    "retribuyes": "retribuir",
    "retribuyese": "retribuir",
    "retribuyeseis": "retribuir",
    "retribuyesen": "retribuir",
    "retribuyeses": "retribuir",
    "retribuyo": "retribuir",
    "retribuyáis": "retribuir",
    "retribuyéramos": "retribuir",
    "retribuyéremos": "retribuir",
    "retribuyésemos": "retribuir",
    "retribuyó": "retribuir",
    "retribuí": "retribuir",
    "retribuía": "retribuir",
    "retribuíais": "retribuir",
    "retribuíamos": "retribuir",
    "retribuían": "retribuir",
    "retribuías": "retribuir",
    "retribuís": "retribuir",
    "retrilla": "retrillar",
    "retrillaba": "retrillar",
    "retrillabais": "retrillar",
    "retrillaban": "retrillar",
    "retrillabas": "retrillar",
    "retrillad": "retrillar",
    "retrillada": "retrillar",
    "retrilladas": "retrillar",
    "retrillado": "retrillar",
    "retrillados": "retrillar",
    "retrillamos": "retrillar",
    "retrillan": "retrillar",
    "retrillando": "retrillar",
    "retrillara": "retrillar",
    "retrillarais": "retrillar",
    "retrillaran": "retrillar",
    "retrillaras": "retrillar",
    "retrillare": "retrillar",
    "retrillareis": "retrillar",
    "retrillaremos": "retrillar",
    "retrillaren": "retrillar",
    "retrillares": "retrillar",
    "retrillaron": "retrillar",
    "retrillará": "retrillar",
    "retrillarán": "retrillar",
    "retrillarás": "retrillar",
    "retrillaré": "retrillar",
    "retrillaréis": "retrillar",
    "retrillaría": "retrillar",
    "retrillaríais": "retrillar",
    "retrillaríamos": "retrillar",
    "retrillarían": "retrillar",
    "retrillarías": "retrillar",
    "retrillas": "retrillar",
    "retrillase": "retrillar",
    "retrillaseis": "retrillar",
    "retrillasen": "retrillar",
    "retrillases": "retrillar",
    "retrillaste": "retrillar",
    "retrillasteis": "retrillar",
    "retrille": "retrillar",
    "retrillemos": "retrillar",
    "retrillen": "retrillar",
    "retrilles": "retrillar",
    "retrillo": "retrillar",
    "retrillábamos": "retrillar",
    "retrilláis": "retrillar",
    "retrilláramos": "retrillar",
    "retrilláremos": "retrillar",
    "retrillásemos": "retrillar",
    "retrillé": "retrillar",
    "retrilléis": "retrillar",
    "retrilló": "retrillar",
    "retroacciones": "retroacción",
    "retroactiva": "retroactivo",
    "retroactivas": "retroactivo",
    "retroactividades": "retroactividad",
    "retroactivos": "retroactivo",
    "retroalimentaciones": "retroalimentación",
    "retroaurales": "retroaural",
    "retroauriculares": "retroauricular",
    "retrobucales": "retrobucal",
    "retrobulbares": "retrobulbar",
    "retrocargadores": "retrocargador",
    "retrocatadióptrica": "retrocatadióptrico",
    "retrocatadióptricas": "retrocatadióptrico",
    "retrocatadióptricos": "retrocatadióptrico",
    "retrocecales": "retrocecal",
    "retroceda": "retroceder",
    "retrocedamos": "retroceder",
    "retrocedan": "retroceder",
    "retrocedas": "retroceder",
    "retrocede": "retroceder",
    "retroceded": "retroceder",
    "retrocedemos": "retroceder",
    "retroceden": "retroceder",
    "retrocederemos": "retroceder",
    "retrocederá": "retroceder",
    "retrocederán": "retroceder",
    "retrocederás": "retroceder",
    "retrocederé": "retroceder",
    "retrocederéis": "retroceder",
    "retrocedería": "retroceder",
    "retrocederíais": "retroceder",
    "retrocederíamos": "retroceder",
    "retrocederían": "retroceder",
    "retrocederías": "retroceder",
    "retrocedes": "retroceder",
    "retrocedida": "retroceder",
    "retrocedidas": "retroceder",
    "retrocedido": "retroceder",
    "retrocedidos": "retroceder",
    "retrocediendo": "retroceder",
    "retrocediera": "retroceder",
    "retrocedierais": "retroceder",
    "retrocedieran": "retroceder",
    "retrocedieras": "retroceder",
    "retrocediere": "retroceder",
    "retrocediereis": "retroceder",
    "retrocedieren": "retroceder",
    "retrocedieres": "retroceder",
    "retrocedieron": "retroceder",
    "retrocediese": "retroceder",
    "retrocedieseis": "retroceder",
    "retrocediesen": "retroceder",
    "retrocedieses": "retroceder",
    "retrocedimos": "retroceder",
    "retrocediste": "retroceder",
    "retrocedisteis": "retroceder",
    "retrocediéramos": "retroceder",
    "retrocediéremos": "retroceder",
    "retrocediésemos": "retroceder",
    "retrocedió": "retroceder",
    "retrocedo": "retroceder",
    "retrocedáis": "retroceder",
    "retrocedéis": "retroceder",
    "retrocedí": "retroceder",
    "retrocedía": "retroceder",
    "retrocedíais": "retroceder",
    "retrocedíamos": "retroceder",
    "retrocedían": "retroceder",
    "retrocedías": "retroceder",
    "retrocesiones": "retrocesión",
    "retrocesos": "retroceso",
    "retrococleares": "retrococlear",
    "retrocohetes": "retrocohete",
    "retroconversiones": "retroconversión",
    "retrodifusiones": "retrodifusión",
    "retrodispersiones": "retrodispersión",
    "retrodonaciones": "retrodonación",
    "retroduodenales": "retroduodenal",
    "retroensambladores": "retroensamblador",
    "retroexcavadoras": "retroexcavadora",
    "retrofaringes": "retrofaringe",
    "retrofaríngea": "retrofaríngeo",
    "retrofaríngeas": "retrofaríngeo",
    "retrofaríngeos": "retrofaríngeo",
    "retrofleja": "retroflejo",
    "retroflejas": "retroflejo",
    "retroflejos": "retroflejo",
    "retrofocales": "retrofocal",
    "retrogenias": "retrogenia",
    "retrognática": "retrognático",
    "retrognáticas": "retrognático",
    "retrognáticos": "retrognático",
    "retrograda": "retrogradar",
    "retrogradaba": "retrogradar",
    "retrogradabais": "retrogradar",
    "retrogradaban": "retrogradar",
    "retrogradabas": "retrogradar",
    "retrogradaciones": "retrogradación",
    "retrogradad": "retrogradar",
    "retrogradada": "retrogradar",
    "retrogradadas": "retrogradar",
    "retrogradado": "retrogradar",
    "retrogradados": "retrogradar",
    "retrogradamos": "retrogradar",
    "retrogradan": "retrogradar",
    "retrogradando": "retrogradar",
    "retrogradara": "retrogradar",
    "retrogradarais": "retrogradar",
    "retrogradaran": "retrogradar",
    "retrogradaras": "retrogradar",
    "retrogradare": "retrogradar",
    "retrogradareis": "retrogradar",
    "retrogradaremos": "retrogradar",
    "retrogradaren": "retrogradar",
    "retrogradares": "retrogradar",
    "retrogradaron": "retrogradar",
    "retrogradará": "retrogradar",
    "retrogradarán": "retrogradar",
    "retrogradarás": "retrogradar",
    "retrogradaré": "retrogradar",
    "retrogradaréis": "retrogradar",
    "retrogradaría": "retrogradar",
    "retrogradaríais": "retrogradar",
    "retrogradaríamos": "retrogradar",
    "retrogradarían": "retrogradar",
    "retrogradarías": "retrogradar",
    "retrogradas": "retrogradar",
    "retrogradase": "retrogradar",
    "retrogradaseis": "retrogradar",
    "retrogradasen": "retrogradar",
    "retrogradases": "retrogradar",
    "retrogradaste": "retrogradar",
    "retrogradasteis": "retrogradar",
    "retrograde": "retrogradar",
    "retrogrademos": "retrogradar",
    "retrograden": "retrogradar",
    "retrogrades": "retrogradar",
    "retrogrado": "retrogradar",
    "retrogradábamos": "retrogradar",
    "retrogradáis": "retrogradar",
    "retrogradáramos": "retrogradar",
    "retrogradáremos": "retrogradar",
    "retrogradásemos": "retrogradar",
    "retrogradé": "retrogradar",
    "retrogradéis": "retrogradar",
    "retrogradó": "retrogradar",
    "retrogresiones": "retrogresión",
    "retrohioidea": "retrohioideo",
    "retrohioideas": "retrohioideo",
    "retrohioideos": "retrohioideo",
    "retroincisiva": "retroincisivo",
    "retroincisivas": "retroincisivo",
    "retroincisivos": "retroincisivo",
    "retrolecturas": "retrolectura",
    "retrolenticulares": "retrolenticular",
    "retrolinguales": "retrolingual",
    "retromandibulares": "retromandibular",
    "retromaxilares": "retromaxilar",
    "retromolares": "retromolar",
    "retronasales": "retronasal",
    "retrooclusiones": "retrooclusión",
    "retroperitoneales": "retroperitoneal",
    "retropilastras": "retropilastra",
    "retropropulsiones": "retropropulsión",
    "retroproyecciones": "retroproyección",
    "retroproyectores": "retroproyector",
    "retropubiana": "retropubiano",
    "retropubianas": "retropubiano",
    "retropubianos": "retropubiano",
    "retropulsiones": "retropulsión",
    "retrorreflectora": "retrorreflector",
    "retrorreflectoras": "retrorreflector",
    "retrorreflectores": "retrorreflector",
    "retrorreflexiones": "retrorreflexión",
    "retrospecciones": "retrospección",
    "retrospectiva": "retrospectivo",
    "retrospectivas": "retrospectivo",
    "retrospectivos": "retrospectivo",
    "retrotracciones": "retrotracción",
    "retrotrae": "retrotraer",
    "retrotraed": "retrotraer",
    "retrotraemos": "retrotraer",
    "retrotraen": "retrotraer",
    "retrotraeremos": "retrotraer",
    "retrotraerá": "retrotraer",
    "retrotraerán": "retrotraer",
    "retrotraerás": "retrotraer",
    "retrotraeré": "retrotraer",
    "retrotraeréis": "retrotraer",
    "retrotraería": "retrotraer",
    "retrotraeríais": "retrotraer",
    "retrotraeríamos": "retrotraer",
    "retrotraerían": "retrotraer",
    "retrotraerías": "retrotraer",
    "retrotraes": "retrotraer",
    "retrotraiga": "retrotraer",
    "retrotraigamos": "retrotraer",
    "retrotraigan": "retrotraer",
    "retrotraigas": "retrotraer",
    "retrotraigo": "retrotraer",
    "retrotraigáis": "retrotraer",
    "retrotraje": "retrotraer",
    "retrotrajera": "retrotraer",
    "retrotrajerais": "retrotraer",
    "retrotrajeran": "retrotraer",
    "retrotrajeras": "retrotraer",
    "retrotrajere": "retrotraer",
    "retrotrajereis": "retrotraer",
    "retrotrajeren": "retrotraer",
    "retrotrajeres": "retrotraer",
    "retrotrajeron": "retrotraer",
    "retrotrajese": "retrotraer",
    "retrotrajeseis": "retrotraer",
    "retrotrajesen": "retrotraer",
    "retrotrajeses": "retrotraer",
    "retrotrajimos": "retrotraer",
    "retrotrajiste": "retrotraer",
    "retrotrajisteis": "retrotraer",
    "retrotrajo": "retrotraer",
    "retrotrajéramos": "retrotraer",
    "retrotrajéremos": "retrotraer",
    "retrotrajésemos": "retrotraer",
    "retrotrayendo": "retrotraer",
    "retrotraéis": "retrotraer",
    "retrotraía": "retrotraer",
    "retrotraíais": "retrotraer",
    "retrotraíamos": "retrotraer",
    "retrotraían": "retrotraer",
    "retrotraías": "retrotraer",
    "retrotraída": "retrotraer",
    "retrotraídas": "retrotraer",
    "retrotraído": "retrotraer",
    "retrotraídos": "retrotraer",
    "retrovenda": "retrovender",
    "retrovendamos": "retrovender",
    "retrovendan": "retrovender",
    "retrovendas": "retrovender",
    "retrovende": "retrovender",
    "retrovended": "retrovender",
    "retrovendemos": "retrovender",
    "retrovenden": "retrovender",
    "retrovenderemos": "retrovender",
    "retrovenderá": "retrovender",
    "retrovenderán": "retrovender",
    "retrovenderás": "retrovender",
    "retrovenderé": "retrovender",
    "retrovenderéis": "retrovender",
    "retrovendería": "retrovender",
    "retrovenderíais": "retrovender",
    "retrovenderíamos": "retrovender",
    "retrovenderían": "retrovender",
    "retrovenderías": "retrovender",
    "retrovendes": "retrovender",
    "retrovendiciones": "retrovendición",
    "retrovendida": "retrovender",
    "retrovendidas": "retrovender",
    "retrovendido": "retrovender",
    "retrovendidos": "retrovender",
    "retrovendiendo": "retrovender",
    "retrovendiera": "retrovender",
    "retrovendierais": "retrovender",
    "retrovendieran": "retrovender",
    "retrovendieras": "retrovender",
    "retrovendiere": "retrovender",
    "retrovendiereis": "retrovender",
    "retrovendieren": "retrovender",
    "retrovendieres": "retrovender",
    "retrovendieron": "retrovender",
    "retrovendiese": "retrovender",
    "retrovendieseis": "retrovender",
    "retrovendiesen": "retrovender",
    "retrovendieses": "retrovender",
    "retrovendimos": "retrovender",
    "retrovendiste": "retrovender",
    "retrovendisteis": "retrovender",
    "retrovendiéramos": "retrovender",
    "retrovendiéremos": "retrovender",
    "retrovendiésemos": "retrovender",
    "retrovendió": "retrovender",
    "retrovendo": "retrovender",
    "retrovendáis": "retrovender",
    "retrovendéis": "retrovender",
    "retrovendí": "retrovender",
    "retrovendía": "retrovender",
    "retrovendíais": "retrovender",
    "retrovendíamos": "retrovender",
    "retrovendían": "retrovender",
    "retrovendías": "retrovender",
    "retroventas": "retroventa",
    "retroversiones": "retroversión",
    "retrovisora": "retrovisor",
    "retrovisoras": "retrovisor",
    "retrovisores": "retrovisor",
    "retrovídeos": "retrovídeo",
    "retruca": "retrucar",
    "retrucaba": "retrucar",
    "retrucabais": "retrucar",
    "retrucaban": "retrucar",
    "retrucabas": "retrucar",
    "retrucad": "retrucar",
    "retrucada": "retrucar",
    "retrucadas": "retrucar",
    "retrucado": "retrucar",
    "retrucados": "retrucar",
    "retrucamos": "retrucar",
    "retrucan": "retrucar",
    "retrucando": "retrucar",
    "retrucara": "retrucar",
    "retrucarais": "retrucar",
    "retrucaran": "retrucar",
    "retrucaras": "retrucar",
    "retrucare": "retrucar",
    "retrucareis": "retrucar",
    "retrucaremos": "retrucar",
    "retrucaren": "retrucar",
    "retrucares": "retrucar",
    "retrucaron": "retrucar",
    "retrucará": "retrucar",
    "retrucarán": "retrucar",
    "retrucarás": "retrucar",
    "retrucaré": "retrucar",
    "retrucaréis": "retrucar",
    "retrucaría": "retrucar",
    "retrucaríais": "retrucar",
    "retrucaríamos": "retrucar",
    "retrucarían": "retrucar",
    "retrucarías": "retrucar",
    "retrucas": "retrucar",
    "retrucase": "retrucar",
    "retrucaseis": "retrucar",
    "retrucasen": "retrucar",
    "retrucases": "retrucar",
    "retrucaste": "retrucar",
    "retrucasteis": "retrucar",
    "retruco": "retrucar",
    "retrucos": "retruco",
    "retrucábamos": "retrucar",
    "retrucáis": "retrucar",
    "retrucáramos": "retrucar",
    "retrucáremos": "retrucar",
    "retrucásemos": "retrucar",
    "retrucó": "retrucar",
    "retruque": "retrucar",
    "retruquemos": "retrucar",
    "retruquen": "retrucar",
    "retruques": "retruque",
    "retruqué": "retrucar",
    "retruquéis": "retrucar",
    "retrusiones": "retrusión",
    "retrusiva": "retrusivo",
    "retrusivas": "retrusivo",
    "retrusivos": "retrusivo",
    "retruécanos": "retruécano",
    "retráctiles": "retráctil",
    "retrógrada": "retrógrado",
    "retrógradas": "retrógrado",
    "retrógrados": "retrógrado",
    "retrónicas": "retrónica",
    "retuerce": "retorcer",
    "retuercen": "retorcer",
    "retuerces": "retorcer",
    "retuerza": "retorcer",
    "retuerzan": "retorcer",
    "retuerzas": "retorcer",
    "retuerzo": "retorcer",
    "retuesta": "retostar",
    "retuestan": "retostar",
    "retuestas": "retostar",
    "retueste": "retostar",
    "retuesten": "retostar",
    "retuestes": "retostar",
    "retuesto": "retostar",
    "retumba": "retumbar",
    "retumbaba": "retumbar",
    "retumbabais": "retumbar",
    "retumbaban": "retumbar",
    "retumbabas": "retumbar",
    "retumbad": "retumbar",
    "retumbada": "retumbar",
    "retumbadas": "retumbar",
    "retumbado": "retumbar",
    "retumbados": "retumbar",
    "retumbamos": "retumbar",
    "retumban": "retumbar",
    "retumbando": "retumbar",
    "retumbantes": "retumbante",
    "retumbara": "retumbar",
    "retumbarais": "retumbar",
    "retumbaran": "retumbar",
    "retumbaras": "retumbar",
    "retumbare": "retumbar",
    "retumbareis": "retumbar",
    "retumbaremos": "retumbar",
    "retumbaren": "retumbar",
    "retumbares": "retumbar",
    "retumbaron": "retumbar",
    "retumbará": "retumbar",
    "retumbarán": "retumbar",
    "retumbarás": "retumbar",
    "retumbaré": "retumbar",
    "retumbaréis": "retumbar",
    "retumbaría": "retumbar",
    "retumbaríais": "retumbar",
    "retumbaríamos": "retumbar",
    "retumbarían": "retumbar",
    "retumbarías": "retumbar",
    "retumbas": "retumbar",
    "retumbase": "retumbar",
    "retumbaseis": "retumbar",
    "retumbasen": "retumbar",
    "retumbases": "retumbar",
    "retumbaste": "retumbar",
    "retumbasteis": "retumbar",
    "retumbe": "retumbar",
    "retumbemos": "retumbar",
    "retumben": "retumbar",
    "retumbes": "retumbar",
    "retumbo": "retumbar",
    "retumbos": "retumbo",
    "retumbábamos": "retumbar",
    "retumbáis": "retumbar",
    "retumbáramos": "retumbar",
    "retumbáremos": "retumbar",
    "retumbásemos": "retumbar",
    "retumbé": "retumbar",
    "retumbéis": "retumbar",
    "retumbó": "retumbar",
    "retuve": "retener",
    "retuviera": "retener",
    "retuvierais": "retener",
    "retuvieran": "retener",
    "retuvieras": "retener",
    "retuviere": "retener",
    "retuviereis": "retener",
    "retuvieren": "retener",
    "retuvieres": "retener",
    "retuvieron": "retener",
    "retuviese": "retener",
    "retuvieseis": "retener",
    "retuviesen": "retener",
    "retuvieses": "retener",
    "retuvimos": "retener",
    "retuviste": "retener",
    "retuvisteis": "retener",
    "retuviéramos": "retener",
    "retuviéremos": "retener",
    "retuviésemos": "retener",
    "retuvo": "retener",
    "retábamos": "retar",
    "retáis": "retar",
    "retáramos": "retar",
    "retáremos": "retar",
    "retásemos": "retar",
    "reté": "retar",
    "retéis": "retar",
    "retén": "retener",
    "retículas": "retícula",
    "retículos": "retículo",
    "retó": "retar",
    "retórica": "retórico",
    "retóricas": "retórico",
    "retóricos": "retórico",
    "reubica": "reubicar",
    "reubicaba": "reubicar",
    "reubicabais": "reubicar",
    "reubicaban": "reubicar",
    "reubicabas": "reubicar",
    "reubicabilidades": "reubicabilidad",
    "reubicables": "reubicable",
    "reubicaciones": "reubicación",
    "reubicad": "reubicar",
    "reubicada": "reubicar",
    "reubicadas": "reubicar",
    "reubicado": "reubicar",
    "reubicados": "reubicar",
    "reubicamos": "reubicar",
    "reubican": "reubicar",
    "reubicando": "reubicar",
    "reubicara": "reubicar",
    "reubicarais": "reubicar",
    "reubicaran": "reubicar",
    "reubicaras": "reubicar",
    "reubicare": "reubicar",
    "reubicareis": "reubicar",
    "reubicaremos": "reubicar",
    "reubicaren": "reubicar",
    "reubicares": "reubicar",
    "reubicaron": "reubicar",
    "reubicará": "reubicar",
    "reubicarán": "reubicar",
    "reubicarás": "reubicar",
    "reubicaré": "reubicar",
    "reubicaréis": "reubicar",
    "reubicaría": "reubicar",
    "reubicaríais": "reubicar",
    "reubicaríamos": "reubicar",
    "reubicarían": "reubicar",
    "reubicarías": "reubicar",
    "reubicas": "reubicar",
    "reubicase": "reubicar",
    "reubicaseis": "reubicar",
    "reubicasen": "reubicar",
    "reubicases": "reubicar",
    "reubicaste": "reubicar",
    "reubicasteis": "reubicar",
    "reubico": "reubicar",
    "reubicábamos": "reubicar",
    "reubicáis": "reubicar",
    "reubicáramos": "reubicar",
    "reubicáremos": "reubicar",
    "reubicásemos": "reubicar",
    "reubicó": "reubicar",
    "reubique": "reubicar",
    "reubiquemos": "reubicar",
    "reubiquen": "reubicar",
    "reubiques": "reubicar",
    "reubiqué": "reubicar",
    "reubiquéis": "reubicar",
    "reucliniana": "reucliniano",
    "reuclinianas": "reucliniano",
    "reuclinianos": "reucliniano",
    "reumas": "reuma",
    "reumatismos": "reumatismo",
    "reumatoidea": "reumatoideo",
    "reumatoideas": "reumatoideo",
    "reumatoideos": "reumatoideo",
    "reumatoides": "reumatoide",
    "reumatologías": "reumatología",
    "reumatóloga": "reumatólogo",
    "reumatólogas": "reumatólogo",
    "reumatólogos": "reumatólogo",
    "reumática": "reumático",
    "reumáticas": "reumático",
    "reumáticos": "reumático",
    "reumátides": "reumátide",
    "reunamos": "reunir",
    "reunid": "reunir",
    "reunida": "reunir",
    "reunidas": "reunir",
    "reunido": "reunir",
    "reunidoras": "reunidora",
    "reunidos": "reunir",
    "reuniendo": "reunir",
    "reuniera": "reunir",
    "reunierais": "reunir",
    "reunieran": "reunir",
    "reunieras": "reunir",
    "reuniere": "reunir",
    "reuniereis": "reunir",
    "reunieren": "reunir",
    "reunieres": "reunir",
    "reunieron": "reunir",
    "reuniese": "reunir",
    "reunieseis": "reunir",
    "reuniesen": "reunir",
    "reunieses": "reunir",
    "reunificaciones": "reunificación",
    "reunimos": "reunir",
    "reuniones": "reunión",
    "reuniremos": "reunir",
    "reunirá": "reunir",
    "reunirán": "reunir",
    "reunirás": "reunir",
    "reuniré": "reunir",
    "reuniréis": "reunir",
    "reuniría": "reunir",
    "reuniríais": "reunir",
    "reuniríamos": "reunir",
    "reunirían": "reunir",
    "reunirías": "reunir",
    "reuniste": "reunir",
    "reunisteis": "reunir",
    "reuniéramos": "reunir",
    "reuniéremos": "reunir",
    "reuniésemos": "reunir",
    "reunió": "reunir",
    "reunáis": "reunir",
    "reuní": "reunir",
    "reunía": "reunir",
    "reuníais": "reunir",
    "reuníamos": "reunir",
    "reunían": "reunir",
    "reunías": "reunir",
    "reunís": "reunir",
    "reurbanizaciones": "reurbanización",
    "reusabilidades": "reusabilidad",
    "reusenses": "reusense",
    "reutilice": "reutilizar",
    "reutilicemos": "reutilizar",
    "reutilicen": "reutilizar",
    "reutilices": "reutilizar",
    "reutilicé": "reutilizar",
    "reutilicéis": "reutilizar",
    "reutilidades": "reutilidad",
    "reutiliza": "reutilizar",
    "reutilizaba": "reutilizar",
    "reutilizabais": "reutilizar",
    "reutilizaban": "reutilizar",
    "reutilizabas": "reutilizar",
    "reutilizables": "reutilizable",
    "reutilizaciones": "reutilización",
    "reutilizad": "reutilizar",
    "reutilizada": "reutilizar",
    "reutilizadas": "reutilizar",
    "reutilizado": "reutilizar",
    "reutilizados": "reutilizar",
    "reutilizamos": "reutilizar",
    "reutilizan": "reutilizar",
    "reutilizando": "reutilizar",
    "reutilizara": "reutilizar",
    "reutilizarais": "reutilizar",
    "reutilizaran": "reutilizar",
    "reutilizaras": "reutilizar",
    "reutilizare": "reutilizar",
    "reutilizareis": "reutilizar",
    "reutilizaremos": "reutilizar",
    "reutilizaren": "reutilizar",
    "reutilizares": "reutilizar",
    "reutilizaron": "reutilizar",
    "reutilizará": "reutilizar",
    "reutilizarán": "reutilizar",
    "reutilizarás": "reutilizar",
    "reutilizaré": "reutilizar",
    "reutilizaréis": "reutilizar",
    "reutilizaría": "reutilizar",
    "reutilizaríais": "reutilizar",
    "reutilizaríamos": "reutilizar",
    "reutilizarían": "reutilizar",
    "reutilizarías": "reutilizar",
    "reutilizas": "reutilizar",
    "reutilizase": "reutilizar",
    "reutilizaseis": "reutilizar",
    "reutilizasen": "reutilizar",
    "reutilizases": "reutilizar",
    "reutilizaste": "reutilizar",
    "reutilizasteis": "reutilizar",
    "reutilizo": "reutilizar",
    "reutilizábamos": "reutilizar",
    "reutilizáis": "reutilizar",
    "reutilizáramos": "reutilizar",
    "reutilizáremos": "reutilizar",
    "reutilizásemos": "reutilizar",
    "reutilizó": "reutilizar",
    "rev": "revolución",
    "revacuna": "revacunar",
    "revacunaba": "revacunar",
    "revacunabais": "revacunar",
    "revacunaban": "revacunar",
    "revacunabas": "revacunar",
    "revacunaciones": "revacunación",
    "revacunad": "revacunar",
    "revacunada": "revacunar",
    "revacunadas": "revacunar",
    "revacunado": "revacunar",
    "revacunados": "revacunar",
    "revacunamos": "revacunar",
    "revacunan": "revacunar",
    "revacunando": "revacunar",
    "revacunara": "revacunar",
    "revacunarais": "revacunar",
    "revacunaran": "revacunar",
    "revacunaras": "revacunar",
    "revacunare": "revacunar",
    "revacunareis": "revacunar",
    "revacunaremos": "revacunar",
    "revacunaren": "revacunar",
    "revacunares": "revacunar",
    "revacunaron": "revacunar",
    "revacunará": "revacunar",
    "revacunarán": "revacunar",
    "revacunarás": "revacunar",
    "revacunaré": "revacunar",
    "revacunaréis": "revacunar",
    "revacunaría": "revacunar",
    "revacunaríais": "revacunar",
    "revacunaríamos": "revacunar",
    "revacunarían": "revacunar",
    "revacunarías": "revacunar",
    "revacunas": "revacunar",
    "revacunase": "revacunar",
    "revacunaseis": "revacunar",
    "revacunasen": "revacunar",
    "revacunases": "revacunar",
    "revacunaste": "revacunar",
    "revacunasteis": "revacunar",
    "revacune": "revacunar",
    "revacunemos": "revacunar",
    "revacunen": "revacunar",
    "revacunes": "revacunar",
    "revacuno": "revacunar",
    "revacunábamos": "revacunar",
    "revacunáis": "revacunar",
    "revacunáramos": "revacunar",
    "revacunáremos": "revacunar",
    "revacunásemos": "revacunar",
    "revacuné": "revacunar",
    "revacunéis": "revacunar",
    "revacunó": "revacunar",
    "revalida": "revalidar",
    "revalidaba": "revalidar",
    "revalidabais": "revalidar",
    "revalidaban": "revalidar",
    "revalidabas": "revalidar",
    "revalidaciones": "revalidación",
    "revalidad": "revalidar",
    "revalidada": "revalidar",
    "revalidadas": "revalidar",
    "revalidado": "revalidar",
    "revalidados": "revalidar",
    "revalidamos": "revalidar",
    "revalidan": "revalidar",
    "revalidando": "revalidar",
    "revalidara": "revalidar",
    "revalidarais": "revalidar",
    "revalidaran": "revalidar",
    "revalidaras": "revalidar",
    "revalidare": "revalidar",
    "revalidareis": "revalidar",
    "revalidaremos": "revalidar",
    "revalidaren": "revalidar",
    "revalidares": "revalidar",
    "revalidaron": "revalidar",
    "revalidará": "revalidar",
    "revalidarán": "revalidar",
    "revalidarás": "revalidar",
    "revalidaré": "revalidar",
    "revalidaréis": "revalidar",
    "revalidaría": "revalidar",
    "revalidaríais": "revalidar",
    "revalidaríamos": "revalidar",
    "revalidarían": "revalidar",
    "revalidarías": "revalidar",
    "revalidas": "revalidar",
    "revalidase": "revalidar",
    "revalidaseis": "revalidar",
    "revalidasen": "revalidar",
    "revalidases": "revalidar",
    "revalidaste": "revalidar",
    "revalidasteis": "revalidar",
    "revalide": "revalidar",
    "revalidemos": "revalidar",
    "revaliden": "revalidar",
    "revalides": "revalidar",
    "revalido": "revalidar",
    "revalidábamos": "revalidar",
    "revalidáis": "revalidar",
    "revalidáramos": "revalidar",
    "revalidáremos": "revalidar",
    "revalidásemos": "revalidar",
    "revalidé": "revalidar",
    "revalidéis": "revalidar",
    "revalidó": "revalidar",
    "revalora": "revalorar",
    "revaloraba": "revalorar",
    "revalorabais": "revalorar",
    "revaloraban": "revalorar",
    "revalorabas": "revalorar",
    "revalorad": "revalorar",
    "revalorada": "revalorar",
    "revaloradas": "revalorar",
    "revalorado": "revalorar",
    "revalorados": "revalorar",
    "revaloramos": "revalorar",
    "revaloran": "revalorar",
    "revalorando": "revalorar",
    "revalorara": "revalorar",
    "revalorarais": "revalorar",
    "revaloraran": "revalorar",
    "revaloraras": "revalorar",
    "revalorare": "revalorar",
    "revalorareis": "revalorar",
    "revaloraremos": "revalorar",
    "revaloraren": "revalorar",
    "revalorares": "revalorar",
    "revaloraron": "revalorar",
    "revalorará": "revalorar",
    "revalorarán": "revalorar",
    "revalorarás": "revalorar",
    "revaloraré": "revalorar",
    "revaloraréis": "revalorar",
    "revaloraría": "revalorar",
    "revaloraríais": "revalorar",
    "revaloraríamos": "revalorar",
    "revalorarían": "revalorar",
    "revalorarías": "revalorar",
    "revaloras": "revalorar",
    "revalorase": "revalorar",
    "revaloraseis": "revalorar",
    "revalorasen": "revalorar",
    "revalorases": "revalorar",
    "revaloraste": "revalorar",
    "revalorasteis": "revalorar",
    "revalore": "revalorar",
    "revaloremos": "revalorar",
    "revaloren": "revalorar",
    "revalores": "revalorar",
    "revalorice": "revalorizar",
    "revaloricemos": "revalorizar",
    "revaloricen": "revalorizar",
    "revalorices": "revalorizar",
    "revaloricé": "revalorizar",
    "revaloricéis": "revalorizar",
    "revaloriza": "revalorizar",
    "revalorizaba": "revalorizar",
    "revalorizabais": "revalorizar",
    "revalorizaban": "revalorizar",
    "revalorizabas": "revalorizar",
    "revalorizaciones": "revalorización",
    "revalorizad": "revalorizar",
    "revalorizada": "revalorizar",
    "revalorizadas": "revalorizar",
    "revalorizado": "revalorizar",
    "revalorizadora": "revalorizador",
    "revalorizadoras": "revalorizador",
    "revalorizadores": "revalorizador",
    "revalorizados": "revalorizar",
    "revalorizamos": "revalorizar",
    "revalorizan": "revalorizar",
    "revalorizando": "revalorizar",
    "revalorizara": "revalorizar",
    "revalorizarais": "revalorizar",
    "revalorizaran": "revalorizar",
    "revalorizaras": "revalorizar",
    "revalorizare": "revalorizar",
    "revalorizareis": "revalorizar",
    "revalorizaremos": "revalorizar",
    "revalorizaren": "revalorizar",
    "revalorizares": "revalorizar",
    "revalorizaron": "revalorizar",
    "revalorizará": "revalorizar",
    "revalorizarán": "revalorizar",
    "revalorizarás": "revalorizar",
    "revalorizaré": "revalorizar",
    "revalorizaréis": "revalorizar",
    "revalorizaría": "revalorizar",
    "revalorizaríais": "revalorizar",
    "revalorizaríamos": "revalorizar",
    "revalorizarían": "revalorizar",
    "revalorizarías": "revalorizar",
    "revalorizas": "revalorizar",
    "revalorizase": "revalorizar",
    "revalorizaseis": "revalorizar",
    "revalorizasen": "revalorizar",
    "revalorizases": "revalorizar",
    "revalorizaste": "revalorizar",
    "revalorizasteis": "revalorizar",
    "revalorizo": "revalorizar",
    "revalorizábamos": "revalorizar",
    "revalorizáis": "revalorizar",
    "revalorizáramos": "revalorizar",
    "revalorizáremos": "revalorizar",
    "revalorizásemos": "revalorizar",
    "revalorizó": "revalorizar",
    "revaloro": "revalorar",
    "revalorábamos": "revalorar",
    "revaloráis": "revalorar",
    "revaloráramos": "revalorar",
    "revaloráremos": "revalorar",
    "revalorásemos": "revalorar",
    "revaloré": "revalorar",
    "revaloréis": "revalorar",
    "revaloró": "revalorar",
    "revaluaba": "revaluar",
    "revaluabais": "revaluar",
    "revaluaban": "revaluar",
    "revaluabas": "revaluar",
    "revaluaciones": "revaluación",
    "revaluad": "revaluar",
    "revaluada": "revaluar",
    "revaluadas": "revaluar",
    "revaluado": "revaluar",
    "revaluados": "revaluar",
    "revaluamos": "revaluar",
    "revaluando": "revaluar",
    "revaluara": "revaluar",
    "revaluarais": "revaluar",
    "revaluaran": "revaluar",
    "revaluaras": "revaluar",
    "revaluare": "revaluar",
    "revaluareis": "revaluar",
    "revaluaremos": "revaluar",
    "revaluaren": "revaluar",
    "revaluares": "revaluar",
    "revaluaron": "revaluar",
    "revaluará": "revaluar",
    "revaluarán": "revaluar",
    "revaluarás": "revaluar",
    "revaluaré": "revaluar",
    "revaluaréis": "revaluar",
    "revaluaría": "revaluar",
    "revaluaríais": "revaluar",
    "revaluaríamos": "revaluar",
    "revaluarían": "revaluar",
    "revaluarías": "revaluar",
    "revaluase": "revaluar",
    "revaluaseis": "revaluar",
    "revaluasen": "revaluar",
    "revaluases": "revaluar",
    "revaluaste": "revaluar",
    "revaluasteis": "revaluar",
    "revaluemos": "revaluar",
    "revaluábamos": "revaluar",
    "revaluáis": "revaluar",
    "revaluáramos": "revaluar",
    "revaluáremos": "revaluar",
    "revaluásemos": "revaluar",
    "revalué": "revaluar",
    "revaluéis": "revaluar",
    "revaluó": "revaluar",
    "revalúa": "revaluar",
    "revalúan": "revaluar",
    "revalúas": "revaluar",
    "revalúe": "revaluar",
    "revalúen": "revaluar",
    "revalúes": "revaluar",
    "revalúo": "revaluar",
    "revanchas": "revancha",
    "revanchismos": "revanchismo",
    "revanchistas": "revanchista",
    "revascularizaciones": "revascularización",
    "revea": "rever",
    "reveamos": "rever",
    "revean": "rever",
    "reveas": "rever",
    "revecera": "revecero",
    "reveceras": "revecero",
    "reveceros": "revecero",
    "revectorizaciones": "revectorización",
    "reved": "rever",
    "revegetaciones": "revegetación",
    "revejece": "revejecer",
    "revejeced": "revejecer",
    "revejecemos": "revejecer",
    "revejecen": "revejecer",
    "revejeceremos": "revejecer",
    "revejecerá": "revejecer",
    "revejecerán": "revejecer",
    "revejecerás": "revejecer",
    "revejeceré": "revejecer",
    "revejeceréis": "revejecer",
    "revejecería": "revejecer",
    "revejeceríais": "revejecer",
    "revejeceríamos": "revejecer",
    "revejecerían": "revejecer",
    "revejecerías": "revejecer",
    "revejeces": "revejecer",
    "revejecida": "revejecer",
    "revejecidas": "revejecer",
    "revejecido": "revejecer",
    "revejecidos": "revejecer",
    "revejeciendo": "revejecer",
    "revejeciera": "revejecer",
    "revejecierais": "revejecer",
    "revejecieran": "revejecer",
    "revejecieras": "revejecer",
    "revejeciere": "revejecer",
    "revejeciereis": "revejecer",
    "revejecieren": "revejecer",
    "revejecieres": "revejecer",
    "revejecieron": "revejecer",
    "revejeciese": "revejecer",
    "revejecieseis": "revejecer",
    "revejeciesen": "revejecer",
    "revejecieses": "revejecer",
    "revejecimos": "revejecer",
    "revejeciste": "revejecer",
    "revejecisteis": "revejecer",
    "revejeciéramos": "revejecer",
    "revejeciéremos": "revejecer",
    "revejeciésemos": "revejecer",
    "revejeció": "revejecer",
    "revejecéis": "revejecer",
    "revejecí": "revejecer",
    "revejecía": "revejecer",
    "revejecíais": "revejecer",
    "revejecíamos": "revejecer",
    "revejecían": "revejecer",
    "revejecías": "revejecer",
    "revejezca": "revejecer",
    "revejezcamos": "revejecer",
    "revejezcan": "revejecer",
    "revejezcas": "revejecer",
    "revejezco": "revejecer",
    "revejezcáis": "revejecer",
    "revejida": "revejido",
    "revejidas": "revejido",
    "revejidos": "revejido",
    "revela": "revelar",
    "revelaba": "revelar",
    "revelabais": "revelar",
    "revelaban": "revelar",
    "revelabas": "revelar",
    "revelaciones": "revelación",
    "revelad": "revelar",
    "revelada": "revelar",
    "reveladas": "revelar",
    "revelado": "revelar",
    "reveladora": "revelador",
    "reveladoras": "revelador",
    "reveladores": "revelador",
    "revelados": "revelar",
    "revelamientos": "revelamiento",
    "revelamos": "revelar",
    "revelan": "revelar",
    "revelandera": "revelandero",
    "revelanderas": "revelandero",
    "revelanderos": "revelandero",
    "revelando": "revelar",
    "revelara": "revelar",
    "revelarais": "revelar",
    "revelaran": "revelar",
    "revelaras": "revelar",
    "revelare": "revelar",
    "revelareis": "revelar",
    "revelaremos": "revelar",
    "revelaren": "revelar",
    "revelares": "revelar",
    "revelaron": "revelar",
    "revelará": "revelar",
    "revelarán": "revelar",
    "revelarás": "revelar",
    "revelaré": "revelar",
    "revelaréis": "revelar",
    "revelaría": "revelar",
    "revelaríais": "revelar",
    "revelaríamos": "revelar",
    "revelarían": "revelar",
    "revelarías": "revelar",
    "revelas": "revelar",
    "revelase": "revelar",
    "revelaseis": "revelar",
    "revelasen": "revelar",
    "revelases": "revelar",
    "revelaste": "revelar",
    "revelasteis": "revelar",
    "revele": "revelar",
    "revelemos": "revelar",
    "revelen": "revelar",
    "reveles": "revelar",
    "revellines": "revellín",
    "revellones": "revellón",
    "revelo": "revelar",
    "revelábamos": "revelar",
    "reveláis": "revelar",
    "reveláramos": "revelar",
    "reveláremos": "revelar",
    "revelásemos": "revelar",
    "revelé": "revelar",
    "reveléis": "revelar",
    "reveló": "revelar",
    "revemos": "rever",
    "reven": "revenir",
    "revena": "revenar",
    "revenaba": "revenar",
    "revenabais": "revenar",
    "revenaban": "revenar",
    "revenabas": "revenar",
    "revenad": "revenar",
    "revenada": "revenar",
    "revenadas": "revenar",
    "revenado": "revenar",
    "revenados": "revenar",
    "revenamos": "revenar",
    "revenan": "revenar",
    "revenando": "revenar",
    "revenara": "revenar",
    "revenarais": "revenar",
    "revenaran": "revenar",
    "revenaras": "revenar",
    "revenare": "revenar",
    "revenareis": "revenar",
    "revenaremos": "revenar",
    "revenaren": "revenar",
    "revenares": "revenar",
    "revenaron": "revenar",
    "revenará": "revenar",
    "revenarán": "revenar",
    "revenarás": "revenar",
    "revenaré": "revenar",
    "revenaréis": "revenar",
    "revenaría": "revenar",
    "revenaríais": "revenar",
    "revenaríamos": "revenar",
    "revenarían": "revenar",
    "revenarías": "revenar",
    "revenas": "revenar",
    "revenase": "revenar",
    "revenaseis": "revenar",
    "revenasen": "revenar",
    "revenases": "revenar",
    "revenaste": "revenar",
    "revenasteis": "revenar",
    "revenda": "revender",
    "revendamos": "revender",
    "revendan": "revender",
    "revendas": "revender",
    "revende": "revender",
    "revended": "revender",
    "revendedora": "revendedor",
    "revendedoras": "revendedor",
    "revendedores": "revendedor",
    "revendemos": "revender",
    "revenden": "revender",
    "revenderemos": "revender",
    "revenderá": "revender",
    "revenderán": "revender",
    "revenderás": "revender",
    "revenderé": "revender",
    "revenderéis": "revender",
    "revendería": "revender",
    "revenderíais": "revender",
    "revenderíamos": "revender",
    "revenderían": "revender",
    "revenderías": "revender",
    "revendes": "revender",
    "revendida": "revender",
    "revendidas": "revender",
    "revendido": "revender",
    "revendidos": "revender",
    "revendiendo": "revender",
    "revendiera": "revender",
    "revendierais": "revender",
    "revendieran": "revender",
    "revendieras": "revender",
    "revendiere": "revender",
    "revendiereis": "revender",
    "revendieren": "revender",
    "revendieres": "revender",
    "revendieron": "revender",
    "revendiese": "revender",
    "revendieseis": "revender",
    "revendiesen": "revender",
    "revendieses": "revender",
    "revendimos": "revender",
    "revendiste": "revender",
    "revendisteis": "revender",
    "revendiéramos": "revender",
    "revendiéremos": "revender",
    "revendiésemos": "revender",
    "revendió": "revender",
    "revendo": "revender",
    "revendremos": "revenir",
    "revendrá": "revenir",
    "revendrán": "revenir",
    "revendrás": "revenir",
    "revendré": "revenir",
    "revendréis": "revenir",
    "revendría": "revenir",
    "revendríais": "revenir",
    "revendríamos": "revenir",
    "revendrían": "revenir",
    "revendrías": "revenir",
    "revendáis": "revender",
    "revendéis": "revender",
    "revendí": "revender",
    "revendía": "revender",
    "revendíais": "revender",
    "revendíamos": "revender",
    "revendían": "revender",
    "revendías": "revender",
    "revene": "revenar",
    "revenemos": "revenar",
    "revenen": "revenar",
    "revenes": "revenar",
    "revenga": "revenir",
    "revengamos": "revenir",
    "revengan": "revenir",
    "revengas": "revenir",
    "revengo": "revenir",
    "revengáis": "revenir",
    "revenibles": "revenible",
    "revenid": "revenir",
    "revenida": "revenir",
    "revenidas": "revenir",
    "revenido": "revenir",
    "revenidos": "revenir",
    "revenimientos": "revenimiento",
    "revenimos": "revenir",
    "reveno": "revenar",
    "revenos": "reveno",
    "reventaba": "reventar",
    "reventabais": "reventar",
    "reventaban": "reventar",
    "reventabas": "reventar",
    "reventad": "reventar",
    "reventada": "reventar",
    "reventadas": "reventar",
    "reventaderos": "reventadero",
    "reventado": "reventar",
    "reventadora": "reventador",
    "reventadoras": "reventador",
    "reventadores": "reventador",
    "reventados": "reventar",
    "reventamos": "reventar",
    "reventando": "reventar",
    "reventara": "reventar",
    "reventarais": "reventar",
    "reventaran": "reventar",
    "reventaras": "reventar",
    "reventare": "reventar",
    "reventareis": "reventar",
    "reventaremos": "reventar",
    "reventaren": "reventar",
    "reventares": "reventar",
    "reventaron": "reventar",
    "reventará": "reventar",
    "reventarán": "reventar",
    "reventarás": "reventar",
    "reventaré": "reventar",
    "reventaréis": "reventar",
    "reventaría": "reventar",
    "reventaríais": "reventar",
    "reventaríamos": "reventar",
    "reventarían": "reventar",
    "reventarías": "reventar",
    "reventas": "reventa",
    "reventase": "reventar",
    "reventaseis": "reventar",
    "reventasen": "reventar",
    "reventases": "reventar",
    "reventaste": "reventar",
    "reventasteis": "reventar",
    "reventazones": "reventazón",
    "reventemos": "reventar",
    "reventona": "reventón",
    "reventonas": "reventón",
    "reventones": "reventón",
    "reventábamos": "reventar",
    "reventáis": "reventar",
    "reventáramos": "reventar",
    "reventáremos": "reventar",
    "reventásemos": "reventar",
    "reventé": "reventar",
    "reventéis": "reventar",
    "reventó": "reventar",
    "revenábamos": "revenar",
    "revenáis": "revenar",
    "revenáramos": "revenar",
    "revenáremos": "revenar",
    "revenásemos": "revenar",
    "revené": "revenar",
    "revenéis": "revenar",
    "revenía": "revenir",
    "reveníais": "revenir",
    "reveníamos": "revenir",
    "revenían": "revenir",
    "revenías": "revenir",
    "revenís": "revenir",
    "revenó": "revenar",
    "reveo": "rever",
    "reverbera": "reverberar",
    "reverberaba": "reverberar",
    "reverberabais": "reverberar",
    "reverberaban": "reverberar",
    "reverberabas": "reverberar",
    "reverberaciones": "reverberación",
    "reverberad": "reverberar",
    "reverberada": "reverberar",
    "reverberadas": "reverberar",
    "reverberado": "reverberar",
    "reverberados": "reverberar",
    "reverberamos": "reverberar",
    "reverberan": "reverberar",
    "reverberando": "reverberar",
    "reverberantes": "reverberante",
    "reverberara": "reverberar",
    "reverberarais": "reverberar",
    "reverberaran": "reverberar",
    "reverberaras": "reverberar",
    "reverberare": "reverberar",
    "reverberareis": "reverberar",
    "reverberaremos": "reverberar",
    "reverberaren": "reverberar",
    "reverberares": "reverberar",
    "reverberaron": "reverberar",
    "reverberará": "reverberar",
    "reverberarán": "reverberar",
    "reverberarás": "reverberar",
    "reverberaré": "reverberar",
    "reverberaréis": "reverberar",
    "reverberaría": "reverberar",
    "reverberaríais": "reverberar",
    "reverberaríamos": "reverberar",
    "reverberarían": "reverberar",
    "reverberarías": "reverberar",
    "reverberas": "reverberar",
    "reverberase": "reverberar",
    "reverberaseis": "reverberar",
    "reverberasen": "reverberar",
    "reverberases": "reverberar",
    "reverberaste": "reverberar",
    "reverberasteis": "reverberar",
    "reverbere": "reverberar",
    "reverberemos": "reverberar",
    "reverberen": "reverberar",
    "reverberes": "reverberar",
    "reverbero": "reverberar",
    "reverberos": "reverbero",
    "reverberábamos": "reverberar",
    "reverberáis": "reverberar",
    "reverberáramos": "reverberar",
    "reverberáremos": "reverberar",
    "reverberásemos": "reverberar",
    "reverberé": "reverberar",
    "reverberéis": "reverberar",
    "reverberó": "reverberar",
    "reverdece": "reverdecer",
    "reverdeced": "reverdecer",
    "reverdecemos": "reverdecer",
    "reverdecen": "reverdecer",
    "reverdeceremos": "reverdecer",
    "reverdecerá": "reverdecer",
    "reverdecerán": "reverdecer",
    "reverdecerás": "reverdecer",
    "reverdeceré": "reverdecer",
    "reverdeceréis": "reverdecer",
    "reverdecería": "reverdecer",
    "reverdeceríais": "reverdecer",
    "reverdeceríamos": "reverdecer",
    "reverdecerían": "reverdecer",
    "reverdecerías": "reverdecer",
    "reverdeces": "reverdecer",
    "reverdecida": "reverdecer",
    "reverdecidas": "reverdecer",
    "reverdecido": "reverdecer",
    "reverdecidos": "reverdecer",
    "reverdeciendo": "reverdecer",
    "reverdeciera": "reverdecer",
    "reverdecierais": "reverdecer",
    "reverdecieran": "reverdecer",
    "reverdecieras": "reverdecer",
    "reverdeciere": "reverdecer",
    "reverdeciereis": "reverdecer",
    "reverdecieren": "reverdecer",
    "reverdecieres": "reverdecer",
    "reverdecieron": "reverdecer",
    "reverdeciese": "reverdecer",
    "reverdecieseis": "reverdecer",
    "reverdeciesen": "reverdecer",
    "reverdecieses": "reverdecer",
    "reverdecimientos": "reverdecimiento",
    "reverdecimos": "reverdecer",
    "reverdeciste": "reverdecer",
    "reverdecisteis": "reverdecer",
    "reverdeciéramos": "reverdecer",
    "reverdeciéremos": "reverdecer",
    "reverdeciésemos": "reverdecer",
    "reverdeció": "reverdecer",
    "reverdecéis": "reverdecer",
    "reverdecí": "reverdecer",
    "reverdecía": "reverdecer",
    "reverdecíais": "reverdecer",
    "reverdecíamos": "reverdecer",
    "reverdecían": "reverdecer",
    "reverdecías": "reverdecer",
    "reverdezca": "reverdecer",
    "reverdezcamos": "reverdecer",
    "reverdezcan": "reverdecer",
    "reverdezcas": "reverdecer",
    "reverdezco": "reverdecer",
    "reverdezcáis": "reverdecer",
    "reveremos": "rever",
    "reverencia": "reverenciar",
    "reverenciaba": "reverenciar",
    "reverenciabais": "reverenciar",
    "reverenciaban": "reverenciar",
    "reverenciabas": "reverenciar",
    "reverenciables": "reverenciable",
    "reverenciad": "reverenciar",
    "reverenciada": "reverenciar",
    "reverenciadas": "reverenciar",
    "reverenciado": "reverenciar",
    "reverenciadora": "reverenciador",
    "reverenciadoras": "reverenciador",
    "reverenciadores": "reverenciador",
    "reverenciados": "reverenciar",
    "reverenciales": "reverencial",
    "reverenciamos": "reverenciar",
    "reverencian": "reverenciar",
    "reverenciando": "reverenciar",
    "reverenciara": "reverenciar",
    "reverenciarais": "reverenciar",
    "reverenciaran": "reverenciar",
    "reverenciaras": "reverenciar",
    "reverenciare": "reverenciar",
    "reverenciareis": "reverenciar",
    "reverenciaremos": "reverenciar",
    "reverenciaren": "reverenciar",
    "reverenciares": "reverenciar",
    "reverenciaron": "reverenciar",
    "reverenciará": "reverenciar",
    "reverenciarán": "reverenciar",
    "reverenciarás": "reverenciar",
    "reverenciaré": "reverenciar",
    "reverenciaréis": "reverenciar",
    "reverenciaría": "reverenciar",
    "reverenciaríais": "reverenciar",
    "reverenciaríamos": "reverenciar",
    "reverenciarían": "reverenciar",
    "reverenciarías": "reverenciar",
    "reverencias": "reverenciar",
    "reverenciase": "reverenciar",
    "reverenciaseis": "reverenciar",
    "reverenciasen": "reverenciar",
    "reverenciases": "reverenciar",
    "reverenciaste": "reverenciar",
    "reverenciasteis": "reverenciar",
    "reverencie": "reverenciar",
    "reverenciemos": "reverenciar",
    "reverencien": "reverenciar",
    "reverencies": "reverenciar",
    "reverencio": "reverenciar",
    "reverenciosa": "reverencioso",
    "reverenciosas": "reverencioso",
    "reverenciosos": "reverencioso",
    "reverenciábamos": "reverenciar",
    "reverenciáis": "reverenciar",
    "reverenciáramos": "reverenciar",
    "reverenciáremos": "reverenciar",
    "reverenciásemos": "reverenciar",
    "reverencié": "reverenciar",
    "reverenciéis": "reverenciar",
    "reverenció": "reverenciar",
    "reverenda": "reverendo",
    "reverendas": "reverendo",
    "reverendos": "reverendo",
    "reverendísima": "reverendísimo",
    "reverendísimas": "reverendísimo",
    "reverendísimos": "reverendísimo",
    "reverentes": "reverente",
    "reversibilidades": "reversibilidad",
    "reversibles": "reversible",
    "reversionales": "reversional",
    "reversiones": "reversión",
    "reversos": "reverso",
    "revertibles": "revertible",
    "revertid": "revertir",
    "revertida": "revertir",
    "revertidas": "revertir",
    "revertido": "revertir",
    "revertidos": "revertir",
    "revertimos": "revertir",
    "revertiremos": "revertir",
    "revertirá": "revertir",
    "revertirán": "revertir",
    "revertirás": "revertir",
    "revertiré": "revertir",
    "revertiréis": "revertir",
    "revertiría": "revertir",
    "revertiríais": "revertir",
    "revertiríamos": "revertir",
    "revertirían": "revertir",
    "revertirías": "revertir",
    "revertiste": "revertir",
    "revertisteis": "revertir",
    "revertí": "revertir",
    "revertía": "revertir",
    "revertíais": "revertir",
    "revertíamos": "revertir",
    "revertían": "revertir",
    "revertías": "revertir",
    "revertís": "revertir",
    "reverá": "rever",
    "reverán": "rever",
    "reverás": "rever",
    "reveré": "rever",
    "reveréis": "rever",
    "revería": "rever",
    "reveríais": "rever",
    "reveríamos": "rever",
    "reverían": "rever",
    "reverías": "rever",
    "revesada": "revesado",
    "revesadas": "revesado",
    "revesados": "revesado",
    "revesas": "revesa",
    "reveses": "revés",
    "revesinos": "revesino",
    "revestid": "revestir",
    "revestida": "revestir",
    "revestidas": "revestir",
    "revestido": "revestir",
    "revestidos": "revestir",
    "revestimientos": "revestimiento",
    "revestimos": "revestir",
    "revestiremos": "revestir",
    "revestirá": "revestir",
    "revestirán": "revestir",
    "revestirás": "revestir",
    "revestiré": "revestir",
    "revestiréis": "revestir",
    "revestiría": "revestir",
    "revestiríais": "revestir",
    "revestiríamos": "revestir",
    "revestirían": "revestir",
    "revestirías": "revestir",
    "revestiste": "revestir",
    "revestisteis": "revestir",
    "revestí": "revestir",
    "revestía": "revestir",
    "revestíais": "revestir",
    "revestíamos": "revestir",
    "revestían": "revestir",
    "revestías": "revestir",
    "revestís": "revestir",
    "revezas": "reveza",
    "revezos": "revezo",
    "reveáis": "rever",
    "reveía": "rever",
    "reveíais": "rever",
    "reveíamos": "rever",
    "reveían": "rever",
    "reveías": "rever",
    "revieja": "reviejo",
    "reviejas": "reviejo",
    "reviejos": "reviejo",
    "reviendo": "rever",
    "reviene": "revenir",
    "revienen": "revenir",
    "revienes": "revenir",
    "revienta": "reventar",
    "revientan": "reventar",
    "revientas": "reventar",
    "reviente": "reventar",
    "revienten": "reventar",
    "revientes": "reventar",
    "reviento": "reventar",
    "reviera": "rever",
    "revierais": "rever",
    "revieran": "rever",
    "revieras": "rever",
    "reviere": "rever",
    "reviereis": "rever",
    "revieren": "rever",
    "revieres": "rever",
    "revieron": "rever",
    "revierta": "revertir",
    "reviertan": "revertir",
    "reviertas": "revertir",
    "revierte": "revertir",
    "revierten": "revertir",
    "reviertes": "revertir",
    "revierto": "revertir",
    "reviese": "rever",
    "revieseis": "rever",
    "reviesen": "rever",
    "revieses": "rever",
    "revigorice": "revigorizar",
    "revigoricemos": "revigorizar",
    "revigoricen": "revigorizar",
    "revigorices": "revigorizar",
    "revigoricé": "revigorizar",
    "revigoricéis": "revigorizar",
    "revigoriza": "revigorizar",
    "revigorizaba": "revigorizar",
    "revigorizabais": "revigorizar",
    "revigorizaban": "revigorizar",
    "revigorizabas": "revigorizar",
    "revigorizad": "revigorizar",
    "revigorizada": "revigorizar",
    "revigorizadas": "revigorizar",
    "revigorizado": "revigorizar",
    "revigorizados": "revigorizar",
    "revigorizamos": "revigorizar",
    "revigorizan": "revigorizar",
    "revigorizando": "revigorizar",
    "revigorizara": "revigorizar",
    "revigorizarais": "revigorizar",
    "revigorizaran": "revigorizar",
    "revigorizaras": "revigorizar",
    "revigorizare": "revigorizar",
    "revigorizareis": "revigorizar",
    "revigorizaremos": "revigorizar",
    "revigorizaren": "revigorizar",
    "revigorizares": "revigorizar",
    "revigorizaron": "revigorizar",
    "revigorizará": "revigorizar",
    "revigorizarán": "revigorizar",
    "revigorizarás": "revigorizar",
    "revigorizaré": "revigorizar",
    "revigorizaréis": "revigorizar",
    "revigorizaría": "revigorizar",
    "revigorizaríais": "revigorizar",
    "revigorizaríamos": "revigorizar",
    "revigorizarían": "revigorizar",
    "revigorizarías": "revigorizar",
    "revigorizas": "revigorizar",
    "revigorizase": "revigorizar",
    "revigorizaseis": "revigorizar",
    "revigorizasen": "revigorizar",
    "revigorizases": "revigorizar",
    "revigorizaste": "revigorizar",
    "revigorizasteis": "revigorizar",
    "revigorizo": "revigorizar",
    "revigorizábamos": "revigorizar",
    "revigorizáis": "revigorizar",
    "revigorizáramos": "revigorizar",
    "revigorizáremos": "revigorizar",
    "revigorizásemos": "revigorizar",
    "revigorizó": "revigorizar",
    "revimos": "rever",
    "revine": "revenir",
    "reviniendo": "revenir",
    "reviniera": "revenir",
    "revinierais": "revenir",
    "revinieran": "revenir",
    "revinieras": "revenir",
    "reviniere": "revenir",
    "reviniereis": "revenir",
    "revinieren": "revenir",
    "revinieres": "revenir",
    "revinieron": "revenir",
    "reviniese": "revenir",
    "revinieseis": "revenir",
    "reviniesen": "revenir",
    "revinieses": "revenir",
    "revinimos": "revenir",
    "reviniste": "revenir",
    "revinisteis": "revenir",
    "reviniéramos": "revenir",
    "reviniéremos": "revenir",
    "reviniésemos": "revenir",
    "revino": "revenir",
    "revirada": "revirado",
    "reviradas": "revirado",
    "revirados": "revirado",
    "revirtamos": "revertir",
    "revirtiendo": "revertir",
    "revirtiera": "revertir",
    "revirtierais": "revertir",
    "revirtieran": "revertir",
    "revirtieras": "revertir",
    "revirtiere": "revertir",
    "revirtiereis": "revertir",
    "revirtieren": "revertir",
    "revirtieres": "revertir",
    "revirtieron": "revertir",
    "revirtiese": "revertir",
    "revirtieseis": "revertir",
    "revirtiesen": "revertir",
    "revirtieses": "revertir",
    "revirtiéramos": "revertir",
    "revirtiéremos": "revertir",
    "revirtiésemos": "revertir",
    "revirtió": "revertir",
    "revirtáis": "revertir",
    "revisa": "revisar",
    "revisaba": "revisar",
    "revisabais": "revisar",
    "revisaban": "revisar",
    "revisabas": "revisar",
    "revisables": "revisable",
    "revisad": "revisar",
    "revisada": "revisar",
    "revisadas": "revisar",
    "revisado": "revisar",
    "revisados": "revisar",
    "revisamos": "revisar",
    "revisan": "revisar",
    "revisando": "revisar",
    "revisara": "revisar",
    "revisarais": "revisar",
    "revisaran": "revisar",
    "revisaras": "revisar",
    "revisare": "revisar",
    "revisareis": "revisar",
    "revisaremos": "revisar",
    "revisaren": "revisar",
    "revisares": "revisar",
    "revisaron": "revisar",
    "revisará": "revisar",
    "revisarán": "revisar",
    "revisarás": "revisar",
    "revisaré": "revisar",
    "revisaréis": "revisar",
    "revisaría": "revisar",
    "revisaríais": "revisar",
    "revisaríamos": "revisar",
    "revisarían": "revisar",
    "revisarías": "revisar",
    "revisas": "revisar",
    "revisase": "revisar",
    "revisaseis": "revisar",
    "revisasen": "revisar",
    "revisases": "revisar",
    "revisaste": "revisar",
    "revisasteis": "revisar",
    "revise": "revisar",
    "revisemos": "revisar",
    "revisen": "revisar",
    "revises": "revisar",
    "revisiones": "revisión",
    "revisionismos": "revisionismo",
    "revisionistas": "revisionista",
    "reviso": "revisar",
    "revisora": "revisor",
    "revisoras": "revisor",
    "revisores": "revisor",
    "revisorías": "revisoría",
    "revista": "revistar",
    "revistaba": "revistar",
    "revistabais": "revistar",
    "revistaban": "revistar",
    "revistabas": "revistar",
    "revistad": "revistar",
    "revistada": "revistar",
    "revistadas": "revistar",
    "revistado": "revistar",
    "revistados": "revistar",
    "revistamos": "revistar",
    "revistan": "revistar",
    "revistando": "revistar",
    "revistara": "revistar",
    "revistarais": "revistar",
    "revistaran": "revistar",
    "revistaras": "revistar",
    "revistare": "revistar",
    "revistareis": "revistar",
    "revistaremos": "revistar",
    "revistaren": "revistar",
    "revistares": "revistar",
    "revistaron": "revistar",
    "revistará": "revistar",
    "revistarán": "revistar",
    "revistarás": "revistar",
    "revistaré": "revistar",
    "revistaréis": "revistar",
    "revistaría": "revistar",
    "revistaríais": "revistar",
    "revistaríamos": "revistar",
    "revistarían": "revistar",
    "revistarías": "revistar",
    "revistas": "revistar",
    "revistase": "revistar",
    "revistaseis": "revistar",
    "revistasen": "revistar",
    "revistases": "revistar",
    "revistaste": "revistar",
    "revistasteis": "revistar",
    "reviste": "revistar",
    "revisteis": "rever",
    "revistemos": "revistar",
    "revisten": "revistar",
    "revisteros": "revistero",
    "revistes": "revistar",
    "revistiendo": "revestir",
    "revistiera": "revestir",
    "revistierais": "revestir",
    "revistieran": "revestir",
    "revistieras": "revestir",
    "revistiere": "revestir",
    "revistiereis": "revestir",
    "revistieren": "revestir",
    "revistieres": "revestir",
    "revistieron": "revestir",
    "revistiese": "revestir",
    "revistieseis": "revestir",
    "revistiesen": "revestir",
    "revistieses": "revestir",
    "revistiéramos": "revestir",
    "revistiéremos": "revestir",
    "revistiésemos": "revestir",
    "revistió": "revestir",
    "revisto": "revistar",
    "revistos": "rever",
    "revistábamos": "revistar",
    "revistáis": "revistar",
    "revistáramos": "revistar",
    "revistáremos": "revistar",
    "revistásemos": "revistar",
    "revisté": "revistar",
    "revistéis": "revistar",
    "revistó": "revistar",
    "revisábamos": "revisar",
    "revisáis": "revisar",
    "revisáramos": "revisar",
    "revisáremos": "revisar",
    "revisásemos": "revisar",
    "revisé": "revisar",
    "reviséis": "revisar",
    "revisó": "revisar",
    "revitalice": "revitalizar",
    "revitalicemos": "revitalizar",
    "revitalicen": "revitalizar",
    "revitalices": "revitalizar",
    "revitalicé": "revitalizar",
    "revitalicéis": "revitalizar",
    "revitaliza": "revitalizar",
    "revitalizaba": "revitalizar",
    "revitalizabais": "revitalizar",
    "revitalizaban": "revitalizar",
    "revitalizabas": "revitalizar",
    "revitalizaciones": "revitalización",
    "revitalizad": "revitalizar",
    "revitalizada": "revitalizar",
    "revitalizadas": "revitalizar",
    "revitalizado": "revitalizar",
    "revitalizadores": "revitalizador",
    "revitalizados": "revitalizar",
    "revitalizamos": "revitalizar",
    "revitalizan": "revitalizar",
    "revitalizando": "revitalizar",
    "revitalizara": "revitalizar",
    "revitalizarais": "revitalizar",
    "revitalizaran": "revitalizar",
    "revitalizaras": "revitalizar",
    "revitalizare": "revitalizar",
    "revitalizareis": "revitalizar",
    "revitalizaremos": "revitalizar",
    "revitalizaren": "revitalizar",
    "revitalizares": "revitalizar",
    "revitalizaron": "revitalizar",
    "revitalizará": "revitalizar",
    "revitalizarán": "revitalizar",
    "revitalizarás": "revitalizar",
    "revitalizaré": "revitalizar",
    "revitalizaréis": "revitalizar",
    "revitalizaría": "revitalizar",
    "revitalizaríais": "revitalizar",
    "revitalizaríamos": "revitalizar",
    "revitalizarían": "revitalizar",
    "revitalizarías": "revitalizar",
    "revitalizas": "revitalizar",
    "revitalizase": "revitalizar",
    "revitalizaseis": "revitalizar",
    "revitalizasen": "revitalizar",
    "revitalizases": "revitalizar",
    "revitalizaste": "revitalizar",
    "revitalizasteis": "revitalizar",
    "revitalizo": "revitalizar",
    "revitalizábamos": "revitalizar",
    "revitalizáis": "revitalizar",
    "revitalizáramos": "revitalizar",
    "revitalizáremos": "revitalizar",
    "revitalizásemos": "revitalizar",
    "revitalizó": "revitalizar",
    "reviva": "revivir",
    "revivales": "revival",
    "revivals": "revival",
    "revivamos": "revivir",
    "revivan": "revivir",
    "revivas": "revivir",
    "revive": "revivir",
    "reviven": "revivir",
    "revives": "revivir",
    "revivid": "revivir",
    "revivida": "revivir",
    "revividas": "revivir",
    "revivideros": "revividero",
    "revivido": "revivir",
    "revividos": "revivir",
    "reviviendo": "revivir",
    "reviviera": "revivir",
    "revivierais": "revivir",
    "revivieran": "revivir",
    "revivieras": "revivir",
    "reviviere": "revivir",
    "reviviereis": "revivir",
    "revivieren": "revivir",
    "revivieres": "revivir",
    "revivieron": "revivir",
    "reviviese": "revivir",
    "revivieseis": "revivir",
    "reviviesen": "revivir",
    "revivieses": "revivir",
    "revivifica": "revivificar",
    "revivificaba": "revivificar",
    "revivificabais": "revivificar",
    "revivificaban": "revivificar",
    "revivificabas": "revivificar",
    "revivificaciones": "revivificación",
    "revivificad": "revivificar",
    "revivificada": "revivificar",
    "revivificadas": "revivificar",
    "revivificado": "revivificar",
    "revivificados": "revivificar",
    "revivificamos": "revivificar",
    "revivifican": "revivificar",
    "revivificando": "revivificar",
    "revivificara": "revivificar",
    "revivificarais": "revivificar",
    "revivificaran": "revivificar",
    "revivificaras": "revivificar",
    "revivificare": "revivificar",
    "revivificareis": "revivificar",
    "revivificaremos": "revivificar",
    "revivificaren": "revivificar",
    "revivificares": "revivificar",
    "revivificaron": "revivificar",
    "revivificará": "revivificar",
    "revivificarán": "revivificar",
    "revivificarás": "revivificar",
    "revivificaré": "revivificar",
    "revivificaréis": "revivificar",
    "revivificaría": "revivificar",
    "revivificaríais": "revivificar",
    "revivificaríamos": "revivificar",
    "revivificarían": "revivificar",
    "revivificarías": "revivificar",
    "revivificas": "revivificar",
    "revivificase": "revivificar",
    "revivificaseis": "revivificar",
    "revivificasen": "revivificar",
    "revivificases": "revivificar",
    "revivificaste": "revivificar",
    "revivificasteis": "revivificar",
    "revivifico": "revivificar",
    "revivificábamos": "revivificar",
    "revivificáis": "revivificar",
    "revivificáramos": "revivificar",
    "revivificáremos": "revivificar",
    "revivificásemos": "revivificar",
    "revivificó": "revivificar",
    "revivifique": "revivificar",
    "revivifiquemos": "revivificar",
    "revivifiquen": "revivificar",
    "revivifiques": "revivificar",
    "revivifiqué": "revivificar",
    "revivifiquéis": "revivificar",
    "revivimos": "revivir",
    "reviviremos": "revivir",
    "revivirá": "revivir",
    "revivirán": "revivir",
    "revivirás": "revivir",
    "reviviré": "revivir",
    "reviviréis": "revivir",
    "reviviría": "revivir",
    "reviviríais": "revivir",
    "reviviríamos": "revivir",
    "revivirían": "revivir",
    "revivirías": "revivir",
    "reviviscencias": "reviviscencia",
    "reviviste": "revivir",
    "revivisteis": "revivir",
    "reviviéramos": "revivir",
    "reviviéremos": "revivir",
    "reviviésemos": "revivir",
    "revivió": "revivir",
    "revivo": "revivir",
    "reviváis": "revivir",
    "reviví": "revivir",
    "revivía": "revivir",
    "revivíais": "revivir",
    "revivíamos": "revivir",
    "revivían": "revivir",
    "revivías": "revivir",
    "revivís": "revivir",
    "reviéramos": "rever",
    "reviéremos": "rever",
    "reviésemos": "rever",
    "revió": "rever",
    "revoca": "revocar",
    "revocaba": "revocar",
    "revocabais": "revocar",
    "revocaban": "revocar",
    "revocabas": "revocar",
    "revocabilidades": "revocabilidad",
    "revocables": "revocable",
    "revocaciones": "revocación",
    "revocad": "revocar",
    "revocada": "revocar",
    "revocadas": "revocar",
    "revocado": "revocar",
    "revocadora": "revocador",
    "revocadoras": "revocador",
    "revocadores": "revocador",
    "revocados": "revocar",
    "revocaduras": "revocadura",
    "revocamos": "revocar",
    "revocan": "revocar",
    "revocando": "revocar",
    "revocara": "revocar",
    "revocarais": "revocar",
    "revocaran": "revocar",
    "revocaras": "revocar",
    "revocare": "revocar",
    "revocareis": "revocar",
    "revocaremos": "revocar",
    "revocaren": "revocar",
    "revocares": "revocar",
    "revocaron": "revocar",
    "revocará": "revocar",
    "revocarán": "revocar",
    "revocarás": "revocar",
    "revocaré": "revocar",
    "revocaréis": "revocar",
    "revocaría": "revocar",
    "revocaríais": "revocar",
    "revocaríamos": "revocar",
    "revocarían": "revocar",
    "revocarías": "revocar",
    "revocas": "revocar",
    "revocase": "revocar",
    "revocaseis": "revocar",
    "revocasen": "revocar",
    "revocases": "revocar",
    "revocaste": "revocar",
    "revocasteis": "revocar",
    "revocatoria": "revocatorio",
    "revocatorias": "revocatorio",
    "revocatorios": "revocatorio",
    "revoco": "revocar",
    "revocos": "revoco",
    "revocábamos": "revocar",
    "revocáis": "revocar",
    "revocáramos": "revocar",
    "revocáremos": "revocar",
    "revocásemos": "revocar",
    "revocó": "revocar",
    "revolcaba": "revolcar",
    "revolcabais": "revolcar",
    "revolcaban": "revolcar",
    "revolcabas": "revolcar",
    "revolcad": "revolcar",
    "revolcada": "revolcar",
    "revolcadas": "revolcar",
    "revolcaderos": "revolcadero",
    "revolcado": "revolcar",
    "revolcados": "revolcar",
    "revolcamos": "revolcar",
    "revolcando": "revolcar",
    "revolcara": "revolcar",
    "revolcarais": "revolcar",
    "revolcaran": "revolcar",
    "revolcaras": "revolcar",
    "revolcare": "revolcar",
    "revolcareis": "revolcar",
    "revolcaremos": "revolcar",
    "revolcaren": "revolcar",
    "revolcares": "revolcar",
    "revolcaron": "revolcar",
    "revolcará": "revolcar",
    "revolcarán": "revolcar",
    "revolcarás": "revolcar",
    "revolcaré": "revolcar",
    "revolcaréis": "revolcar",
    "revolcaría": "revolcar",
    "revolcaríais": "revolcar",
    "revolcaríamos": "revolcar",
    "revolcarían": "revolcar",
    "revolcarías": "revolcar",
    "revolcase": "revolcar",
    "revolcaseis": "revolcar",
    "revolcasen": "revolcar",
    "revolcases": "revolcar",
    "revolcaste": "revolcar",
    "revolcasteis": "revolcar",
    "revolcones": "revolcón",
    "revolcábamos": "revolcar",
    "revolcáis": "revolcar",
    "revolcáramos": "revolcar",
    "revolcáremos": "revolcar",
    "revolcásemos": "revolcar",
    "revolcó": "revolcar",
    "revolea": "revolear",
    "revoleaba": "revolear",
    "revoleabais": "revolear",
    "revoleaban": "revolear",
    "revoleabas": "revolear",
    "revolead": "revolear",
    "revoleada": "revolear",
    "revoleadas": "revolear",
    "revoleado": "revolear",
    "revoleados": "revolear",
    "revoleamos": "revolear",
    "revolean": "revolear",
    "revoleando": "revolear",
    "revoleara": "revolear",
    "revolearais": "revolear",
    "revolearan": "revolear",
    "revolearas": "revolear",
    "revoleare": "revolear",
    "revoleareis": "revolear",
    "revolearemos": "revolear",
    "revolearen": "revolear",
    "revoleares": "revolear",
    "revolearon": "revolear",
    "revoleará": "revolear",
    "revolearán": "revolear",
    "revolearás": "revolear",
    "revolearé": "revolear",
    "revolearéis": "revolear",
    "revolearía": "revolear",
    "revolearíais": "revolear",
    "revolearíamos": "revolear",
    "revolearían": "revolear",
    "revolearías": "revolear",
    "revoleas": "revolear",
    "revolease": "revolear",
    "revoleaseis": "revolear",
    "revoleasen": "revolear",
    "revoleases": "revolear",
    "revoleaste": "revolear",
    "revoleasteis": "revolear",
    "revolee": "revolear",
    "revoleemos": "revolear",
    "revoleen": "revolear",
    "revolees": "revolear",
    "revoleo": "revolear",
    "revoleábamos": "revolear",
    "revoleáis": "revolear",
    "revoleáramos": "revolear",
    "revoleáremos": "revolear",
    "revoleásemos": "revolear",
    "revoleé": "revolear",
    "revoleéis": "revolear",
    "revoleó": "revolear",
    "revolotea": "revolotear",
    "revoloteaba": "revolotear",
    "revoloteabais": "revolotear",
    "revoloteaban": "revolotear",
    "revoloteabas": "revolotear",
    "revolotead": "revolotear",
    "revoloteada": "revolotear",
    "revoloteadas": "revolotear",
    "revoloteado": "revolotear",
    "revoloteados": "revolotear",
    "revoloteamos": "revolotear",
    "revolotean": "revolotear",
    "revoloteando": "revolotear",
    "revoloteara": "revolotear",
    "revolotearais": "revolotear",
    "revolotearan": "revolotear",
    "revolotearas": "revolotear",
    "revoloteare": "revolotear",
    "revoloteareis": "revolotear",
    "revolotearemos": "revolotear",
    "revolotearen": "revolotear",
    "revoloteares": "revolotear",
    "revolotearon": "revolotear",
    "revoloteará": "revolotear",
    "revolotearán": "revolotear",
    "revolotearás": "revolotear",
    "revolotearé": "revolotear",
    "revolotearéis": "revolotear",
    "revolotearía": "revolotear",
    "revolotearíais": "revolotear",
    "revolotearíamos": "revolotear",
    "revolotearían": "revolotear",
    "revolotearías": "revolotear",
    "revoloteas": "revolotear",
    "revolotease": "revolotear",
    "revoloteaseis": "revolotear",
    "revoloteasen": "revolotear",
    "revoloteases": "revolotear",
    "revoloteaste": "revolotear",
    "revoloteasteis": "revolotear",
    "revolotee": "revolotear",
    "revoloteemos": "revolotear",
    "revoloteen": "revolotear",
    "revolotees": "revolotear",
    "revoloteo": "revolotear",
    "revoloteos": "revoloteo",
    "revoloteábamos": "revolotear",
    "revoloteáis": "revolotear",
    "revoloteáramos": "revolotear",
    "revoloteáremos": "revolotear",
    "revoloteásemos": "revolotear",
    "revoloteé": "revolotear",
    "revoloteéis": "revolotear",
    "revoloteó": "revolotear",
    "revolquemos": "revolcar",
    "revolqué": "revolcar",
    "revolquéis": "revolcar",
    "revoltijos": "revoltijo",
    "revoltillos": "revoltillo",
    "revoltones": "revoltón",
    "revoltosa": "revoltoso",
    "revoltosas": "revoltoso",
    "revoltosos": "revoltoso",
    "revoluciona": "revolucionar",
    "revolucionaba": "revolucionar",
    "revolucionabais": "revolucionar",
    "revolucionaban": "revolucionar",
    "revolucionabas": "revolucionar",
    "revolucionad": "revolucionar",
    "revolucionada": "revolucionar",
    "revolucionadas": "revolucionar",
    "revolucionado": "revolucionar",
    "revolucionados": "revolucionar",
    "revolucionamos": "revolucionar",
    "revolucionan": "revolucionar",
    "revolucionando": "revolucionar",
    "revolucionara": "revolucionar",
    "revolucionarais": "revolucionar",
    "revolucionaran": "revolucionar",
    "revolucionaras": "revolucionar",
    "revolucionare": "revolucionar",
    "revolucionareis": "revolucionar",
    "revolucionaremos": "revolucionar",
    "revolucionaren": "revolucionar",
    "revolucionares": "revolucionar",
    "revolucionaria": "revolucionario",
    "revolucionarias": "revolucionario",
    "revolucionarios": "revolucionario",
    "revolucionaron": "revolucionar",
    "revolucionará": "revolucionar",
    "revolucionarán": "revolucionar",
    "revolucionarás": "revolucionar",
    "revolucionaré": "revolucionar",
    "revolucionaréis": "revolucionar",
    "revolucionaría": "revolucionar",
    "revolucionaríais": "revolucionar",
    "revolucionaríamos": "revolucionar",
    "revolucionarían": "revolucionar",
    "revolucionarías": "revolucionar",
    "revolucionas": "revolucionar",
    "revolucionase": "revolucionar",
    "revolucionaseis": "revolucionar",
    "revolucionasen": "revolucionar",
    "revolucionases": "revolucionar",
    "revolucionaste": "revolucionar",
    "revolucionasteis": "revolucionar",
    "revolucione": "revolucionar",
    "revolucionemos": "revolucionar",
    "revolucionen": "revolucionar",
    "revoluciones": "revolucionar",
    "revolucionistas": "revolucionista",
    "revoluciono": "revolucionar",
    "revolucionábamos": "revolucionar",
    "revolucionáis": "revolucionar",
    "revolucionáramos": "revolucionar",
    "revolucionáremos": "revolucionar",
    "revolucionásemos": "revolucionar",
    "revolucioné": "revolucionar",
    "revolucionéis": "revolucionar",
    "revolucionó": "revolucionar",
    "revoluciónes": "revolución",
    "revolutiva": "revolutivo",
    "revolutivas": "revolutivo",
    "revolutivos": "revolutivo",
    "revolvamos": "revolver",
    "revolved": "revolver",
    "revolvederos": "revolvedero",
    "revolvemos": "revolver",
    "revolveremos": "revolver",
    "revolverá": "revolver",
    "revolverán": "revolver",
    "revolverás": "revolver",
    "revolveré": "revolver",
    "revolveréis": "revolver",
    "revolvería": "revolver",
    "revolveríais": "revolver",
    "revolveríamos": "revolver",
    "revolverían": "revolver",
    "revolverías": "revolver",
    "revolviendo": "revolver",
    "revolviera": "revolver",
    "revolvierais": "revolver",
    "revolvieran": "revolver",
    "revolvieras": "revolver",
    "revolviere": "revolver",
    "revolviereis": "revolver",
    "revolvieren": "revolver",
    "revolvieres": "revolver",
    "revolvieron": "revolver",
    "revolviese": "revolver",
    "revolvieseis": "revolver",
    "revolviesen": "revolver",
    "revolvieses": "revolver",
    "revolvimientos": "revolvimiento",
    "revolvimos": "revolver",
    "revolvings": "revolving",
    "revolviste": "revolver",
    "revolvisteis": "revolver",
    "revolviéramos": "revolver",
    "revolviéremos": "revolver",
    "revolviésemos": "revolver",
    "revolvió": "revolver",
    "revolváis": "revolver",
    "revolvéis": "revolver",
    "revolví": "revolver",
    "revolvía": "revolver",
    "revolvíais": "revolver",
    "revolvíamos": "revolver",
    "revolvían": "revolver",
    "revolvías": "revolver",
    "revoque": "revocar",
    "revoquemos": "revocar",
    "revoquen": "revocar",
    "revoques": "revoque",
    "revoqué": "revocar",
    "revoquéis": "revocar",
    "revuelca": "revolcar",
    "revuelcan": "revolcar",
    "revuelcas": "revolcar",
    "revuelco": "revolcar",
    "revuelcos": "revuelco",
    "revuelos": "revuelo",
    "revuelque": "revolcar",
    "revuelquen": "revolcar",
    "revuelques": "revolcar",
    "revuelta": "revolver",
    "revueltas": "revuelta",
    "revuelto": "revolver",
    "revueltos": "revuelto",
    "revuelva": "revolver",
    "revuelvan": "revolver",
    "revuelvas": "revolver",
    "revuelve": "revolver",
    "revuelven": "revolver",
    "revuelves": "revolver",
    "revuelvo": "revolver",
    "revulsiones": "revulsión",
    "revulsiva": "revulsivo",
    "revulsivas": "revulsivo",
    "revulsivos": "revulsivo",
    "revulsoria": "revulsorio",
    "revulsorias": "revulsorio",
    "revulsorios": "revulsorio",
    "reválidas": "reválida",
    "revé": "rever",
    "revéis": "rever",
    "revén": "rever",
    "revés": "rever",
    "reví": "rever",
    "revólveres": "revólver",
    "rexistasias": "rexistasia",
    "reyertas": "reyerta",
    "reyes": "rey",
    "reyezuelos": "reyezuelo",
    "reza": "rezar",
    "rezaba": "rezar",
    "rezabais": "rezar",
    "rezaban": "rezar",
    "rezabas": "rezar",
    "rezad": "rezar",
    "rezada": "rezar",
    "rezadas": "rezar",
    "rezado": "rezar",
    "rezadora": "rezador",
    "rezadoras": "rezador",
    "rezadores": "rezador",
    "rezados": "rezar",
    "rezaga": "rezagar",
    "rezagaba": "rezagar",
    "rezagabais": "rezagar",
    "rezagaban": "rezagar",
    "rezagabas": "rezagar",
    "rezagad": "rezagar",
    "rezagada": "rezagar",
    "rezagadas": "rezagar",
    "rezagado": "rezagar",
    "rezagados": "rezagar",
    "rezagamos": "rezagar",
    "rezagan": "rezagar",
    "rezagando": "rezagar",
    "rezagara": "rezagar",
    "rezagarais": "rezagar",
    "rezagaran": "rezagar",
    "rezagaras": "rezagar",
    "rezagare": "rezagar",
    "rezagareis": "rezagar",
    "rezagaremos": "rezagar",
    "rezagaren": "rezagar",
    "rezagares": "rezagar",
    "rezagaron": "rezagar",
    "rezagará": "rezagar",
    "rezagarán": "rezagar",
    "rezagarás": "rezagar",
    "rezagaré": "rezagar",
    "rezagaréis": "rezagar",
    "rezagaría": "rezagar",
    "rezagaríais": "rezagar",
    "rezagaríamos": "rezagar",
    "rezagarían": "rezagar",
    "rezagarías": "rezagar",
    "rezagas": "rezagar",
    "rezagase": "rezagar",
    "rezagaseis": "rezagar",
    "rezagasen": "rezagar",
    "rezagases": "rezagar",
    "rezagaste": "rezagar",
    "rezagasteis": "rezagar",
    "rezago": "rezagar",
    "rezague": "rezagar",
    "rezaguemos": "rezagar",
    "rezaguen": "rezagar",
    "rezagues": "rezagar",
    "rezagué": "rezagar",
    "rezaguéis": "rezagar",
    "rezagábamos": "rezagar",
    "rezagáis": "rezagar",
    "rezagáramos": "rezagar",
    "rezagáremos": "rezagar",
    "rezagásemos": "rezagar",
    "rezagó": "rezagar",
    "rezamos": "rezar",
    "rezan": "rezar",
    "rezando": "rezar",
    "rezara": "rezar",
    "rezarais": "rezar",
    "rezaran": "rezar",
    "rezaras": "rezar",
    "rezare": "rezar",
    "rezareis": "rezar",
    "rezaremos": "rezar",
    "rezaren": "rezar",
    "rezares": "rezar",
    "rezaron": "rezar",
    "rezará": "rezar",
    "rezarán": "rezar",
    "rezarás": "rezar",
    "rezaré": "rezar",
    "rezaréis": "rezar",
    "rezaría": "rezar",
    "rezaríais": "rezar",
    "rezaríamos": "rezar",
    "rezarían": "rezar",
    "rezarías": "rezar",
    "rezas": "rezar",
    "rezase": "rezar",
    "rezaseis": "rezar",
    "rezasen": "rezar",
    "rezases": "rezar",
    "rezaste": "rezar",
    "rezasteis": "rezar",
    "reznos": "rezno",
    "rezo": "rezar",
    "rezones": "rezón",
    "rezonga": "rezongar",
    "rezongaba": "rezongar",
    "rezongabais": "rezongar",
    "rezongaban": "rezongar",
    "rezongabas": "rezongar",
    "rezongad": "rezongar",
    "rezongada": "rezongar",
    "rezongadas": "rezongar",
    "rezongado": "rezongar",
    "rezongadora": "rezongador",
    "rezongadoras": "rezongador",
    "rezongadores": "rezongador",
    "rezongados": "rezongar",
    "rezongamos": "rezongar",
    "rezongan": "rezongar",
    "rezongando": "rezongar",
    "rezongara": "rezongar",
    "rezongarais": "rezongar",
    "rezongaran": "rezongar",
    "rezongaras": "rezongar",
    "rezongare": "rezongar",
    "rezongareis": "rezongar",
    "rezongaremos": "rezongar",
    "rezongaren": "rezongar",
    "rezongares": "rezongar",
    "rezongaron": "rezongar",
    "rezongará": "rezongar",
    "rezongarán": "rezongar",
    "rezongarás": "rezongar",
    "rezongaré": "rezongar",
    "rezongaréis": "rezongar",
    "rezongaría": "rezongar",
    "rezongaríais": "rezongar",
    "rezongaríamos": "rezongar",
    "rezongarían": "rezongar",
    "rezongarías": "rezongar",
    "rezongas": "rezongar",
    "rezongase": "rezongar",
    "rezongaseis": "rezongar",
    "rezongasen": "rezongar",
    "rezongases": "rezongar",
    "rezongaste": "rezongar",
    "rezongasteis": "rezongar",
    "rezonglona": "rezonglón",
    "rezonglonas": "rezonglón",
    "rezonglones": "rezonglón",
    "rezongo": "rezongar",
    "rezongona": "rezongón",
    "rezongonas": "rezongón",
    "rezongones": "rezongón",
    "rezongos": "rezongo",
    "rezongue": "rezongar",
    "rezonguemos": "rezongar",
    "rezonguen": "rezongar",
    "rezongues": "rezongar",
    "rezongué": "rezongar",
    "rezonguéis": "rezongar",
    "rezongábamos": "rezongar",
    "rezongáis": "rezongar",
    "rezongáramos": "rezongar",
    "rezongáremos": "rezongar",
    "rezongásemos": "rezongar",
    "rezongó": "rezongar",
    "rezos": "rezo",
    "rezuma": "rezumar",
    "rezumaba": "rezumar",
    "rezumabais": "rezumar",
    "rezumaban": "rezumar",
    "rezumabas": "rezumar",
    "rezumad": "rezumar",
    "rezumada": "rezumar",
    "rezumadas": "rezumar",
    "rezumaderos": "rezumadero",
    "rezumado": "rezumar",
    "rezumados": "rezumar",
    "rezumamientos": "rezumamiento",
    "rezumamos": "rezumar",
    "rezuman": "rezumar",
    "rezumando": "rezumar",
    "rezumantes": "rezumante",
    "rezumara": "rezumar",
    "rezumarais": "rezumar",
    "rezumaran": "rezumar",
    "rezumaras": "rezumar",
    "rezumare": "rezumar",
    "rezumareis": "rezumar",
    "rezumaremos": "rezumar",
    "rezumaren": "rezumar",
    "rezumares": "rezumar",
    "rezumaron": "rezumar",
    "rezumará": "rezumar",
    "rezumarán": "rezumar",
    "rezumarás": "rezumar",
    "rezumaré": "rezumar",
    "rezumaréis": "rezumar",
    "rezumaría": "rezumar",
    "rezumaríais": "rezumar",
    "rezumaríamos": "rezumar",
    "rezumarían": "rezumar",
    "rezumarías": "rezumar",
    "rezumas": "rezumar",
    "rezumase": "rezumar",
    "rezumaseis": "rezumar",
    "rezumasen": "rezumar",
    "rezumases": "rezumar",
    "rezumaste": "rezumar",
    "rezumasteis": "rezumar",
    "rezume": "rezumar",
    "rezumemos": "rezumar",
    "rezumen": "rezumar",
    "rezumes": "rezumar",
    "rezumo": "rezumar",
    "rezumábamos": "rezumar",
    "rezumáis": "rezumar",
    "rezumáramos": "rezumar",
    "rezumáremos": "rezumar",
    "rezumásemos": "rezumar",
    "rezumé": "rezumar",
    "rezuméis": "rezumar",
    "rezumó": "rezumar",
    "rezábamos": "rezar",
    "rezáis": "rezar",
    "rezáramos": "rezar",
    "rezáremos": "rezar",
    "rezásemos": "rezar",
    "rezó": "rezar",
    "reí": "reír",
    "reía": "reír",
    "reíais": "reír",
    "reíamos": "reír",
    "reían": "reír",
    "reías": "reír",
    "reíd": "reír",
    "reída": "reír",
    "reídas": "reír",
    "reído": "reír",
    "reídos": "reír",
    "reímos": "reír",
    "reís": "reír",
    "reísmos": "reísmo",
    "reíste": "reír",
    "reísteis": "reír",
    "reñid": "reñir",
    "reñida": "reñir",
    "reñidas": "reñir",
    "reñideros": "reñidero",
    "reñido": "reñir",
    "reñidora": "reñidor",
    "reñidoras": "reñidor",
    "reñidores": "reñidor",
    "reñidos": "reñir",
    "reñiduras": "reñidura",
    "reñimos": "reñir",
    "reñiremos": "reñir",
    "reñirá": "reñir",
    "reñirán": "reñir",
    "reñirás": "reñir",
    "reñiré": "reñir",
    "reñiréis": "reñir",
    "reñiría": "reñir",
    "reñiríais": "reñir",
    "reñiríamos": "reñir",
    "reñirían": "reñir",
    "reñirías": "reñir",
    "reñiste": "reñir",
    "reñisteis": "reñir",
    "reñí": "reñir",
    "reñía": "reñir",
    "reñíais": "reñir",
    "reñíamos": "reñir",
    "reñían": "reñir",
    "reñías": "reñir",
    "reñís": "reñir",
    "reófila": "reófilo",
    "reófilas": "reófilo",
    "reófilos": "reófilo",
    "reóforos": "reóforo",
    "reógrafos": "reógrafo",
    "reómetros": "reómetro",
    "reóstatos": "reóstato",
    "reúmas": "reúma",
    "reúna": "reunir",
    "reúnan": "reunir",
    "reúnas": "reunir",
    "reúne": "reunir",
    "reúnen": "reunir",
    "reúnes": "reunir",
    "reúno": "reunir",
    "rhodoides": "rhodoid",
    "rhos": "rho",
    "riachos": "riacho",
    "riachuelos": "riachuelo",
    "riadas": "riada",
    "riales": "rial",
    "riamos": "reír",
    "ribadoquines": "ribadoquín",
    "ribalda": "ribaldo",
    "ribaldas": "ribaldo",
    "ribalderías": "ribaldería",
    "ribaldos": "ribaldo",
    "ribas": "riba",
    "ribazones": "ribazón",
    "ribazos": "ribazo",
    "ribeiros": "ribeiro",
    "riberas": "ribera",
    "ribereña": "ribereño",
    "ribereñas": "ribereño",
    "ribereños": "ribereño",
    "riberiega": "riberiego",
    "riberiegas": "riberiego",
    "riberiegos": "riberiego",
    "riberos": "ribero",
    "ribetea": "ribetear",
    "ribeteaba": "ribetear",
    "ribeteabais": "ribetear",
    "ribeteaban": "ribetear",
    "ribeteabas": "ribetear",
    "ribetead": "ribetear",
    "ribeteada": "ribetear",
    "ribeteadas": "ribetear",
    "ribeteado": "ribetear",
    "ribeteados": "ribetear",
    "ribeteamos": "ribetear",
    "ribetean": "ribetear",
    "ribeteando": "ribetear",
    "ribeteara": "ribetear",
    "ribetearais": "ribetear",
    "ribetearan": "ribetear",
    "ribetearas": "ribetear",
    "ribeteare": "ribetear",
    "ribeteareis": "ribetear",
    "ribetearemos": "ribetear",
    "ribetearen": "ribetear",
    "ribeteares": "ribetear",
    "ribetearon": "ribetear",
    "ribeteará": "ribetear",
    "ribetearán": "ribetear",
    "ribetearás": "ribetear",
    "ribetearé": "ribetear",
    "ribetearéis": "ribetear",
    "ribetearía": "ribetear",
    "ribetearíais": "ribetear",
    "ribetearíamos": "ribetear",
    "ribetearían": "ribetear",
    "ribetearías": "ribetear",
    "ribeteas": "ribetear",
    "ribetease": "ribetear",
    "ribeteaseis": "ribetear",
    "ribeteasen": "ribetear",
    "ribeteases": "ribetear",
    "ribeteaste": "ribetear",
    "ribeteasteis": "ribetear",
    "ribetee": "ribetear",
    "ribeteemos": "ribetear",
    "ribeteen": "ribetear",
    "ribetees": "ribetear",
    "ribeteo": "ribetear",
    "ribetes": "ribete",
    "ribeteábamos": "ribetear",
    "ribeteáis": "ribetear",
    "ribeteáramos": "ribetear",
    "ribeteáremos": "ribetear",
    "ribeteásemos": "ribetear",
    "ribeteé": "ribetear",
    "ribeteéis": "ribetear",
    "ribeteó": "ribetear",
    "riboflavinas": "riboflavina",
    "riboforinas": "riboforina",
    "ribonucleasas": "ribonucleasa",
    "ribonucleica": "ribonucleico",
    "ribonucleicas": "ribonucleico",
    "ribonucleicos": "ribonucleico",
    "ribosas": "ribosa",
    "ribosomales": "ribosomal",
    "ribosomas": "ribosoma",
    "ribosómica": "ribosómico",
    "ribosómicas": "ribosómico",
    "ribosómicos": "ribosómico",
    "rica": "rico",
    "ricacha": "ricacho",
    "ricachas": "ricacho",
    "ricachona": "ricachón",
    "ricachonas": "ricachón",
    "ricachones": "ricachón",
    "ricachos": "ricacho",
    "ricadueñas": "ricadueña",
    "ricahembras": "ricahembra",
    "ricahombrías": "ricahombría",
    "ricas": "rico",
    "rice": "rizar",
    "ricemos": "rizar",
    "ricen": "rizar",
    "rices": "rizar",
    "riciales": "ricial",
    "ricias": "ricia",
    "ricinas": "ricina",
    "ricinoleatos": "ricinoleato",
    "ricinoleica": "ricinoleico",
    "ricinoleicas": "ricinoleico",
    "ricinoleicos": "ricinoleico",
    "ricinos": "ricino",
    "rickettsias": "rickettsia",
    "ricohombres": "ricohombre",
    "ricos": "rico",
    "rictuses": "rictus",
    "ricuras": "ricura",
    "ricé": "rizar",
    "ricéis": "rizar",
    "ridectomías": "ridectomía",
    "ridiculeces": "ridiculez",
    "ridiculice": "ridiculizar",
    "ridiculicemos": "ridiculizar",
    "ridiculicen": "ridiculizar",
    "ridiculices": "ridiculizar",
    "ridiculicé": "ridiculizar",
    "ridiculicéis": "ridiculizar",
    "ridiculiza": "ridiculizar",
    "ridiculizaba": "ridiculizar",
    "ridiculizabais": "ridiculizar",
    "ridiculizaban": "ridiculizar",
    "ridiculizabas": "ridiculizar",
    "ridiculizad": "ridiculizar",
    "ridiculizada": "ridiculizar",
    "ridiculizadas": "ridiculizar",
    "ridiculizado": "ridiculizar",
    "ridiculizados": "ridiculizar",
    "ridiculizamos": "ridiculizar",
    "ridiculizan": "ridiculizar",
    "ridiculizando": "ridiculizar",
    "ridiculizara": "ridiculizar",
    "ridiculizarais": "ridiculizar",
    "ridiculizaran": "ridiculizar",
    "ridiculizaras": "ridiculizar",
    "ridiculizare": "ridiculizar",
    "ridiculizareis": "ridiculizar",
    "ridiculizaremos": "ridiculizar",
    "ridiculizaren": "ridiculizar",
    "ridiculizares": "ridiculizar",
    "ridiculizaron": "ridiculizar",
    "ridiculizará": "ridiculizar",
    "ridiculizarán": "ridiculizar",
    "ridiculizarás": "ridiculizar",
    "ridiculizaré": "ridiculizar",
    "ridiculizaréis": "ridiculizar",
    "ridiculizaría": "ridiculizar",
    "ridiculizaríais": "ridiculizar",
    "ridiculizaríamos": "ridiculizar",
    "ridiculizarían": "ridiculizar",
    "ridiculizarías": "ridiculizar",
    "ridiculizas": "ridiculizar",
    "ridiculizase": "ridiculizar",
    "ridiculizaseis": "ridiculizar",
    "ridiculizasen": "ridiculizar",
    "ridiculizases": "ridiculizar",
    "ridiculizaste": "ridiculizar",
    "ridiculizasteis": "ridiculizar",
    "ridiculizo": "ridiculizar",
    "ridiculizábamos": "ridiculizar",
    "ridiculizáis": "ridiculizar",
    "ridiculizáramos": "ridiculizar",
    "ridiculizáremos": "ridiculizar",
    "ridiculizásemos": "ridiculizar",
    "ridiculizó": "ridiculizar",
    "ridícula": "ridículo",
    "ridículas": "ridículo",
    "ridículos": "ridículo",
    "riebeckitas": "riebeckita",
    "riega": "regar",
    "riegan": "regar",
    "riegas": "regar",
    "riego": "regar",
    "riegos": "riego",
    "riegue": "regar",
    "rieguen": "regar",
    "riegues": "regar",
    "riela": "rielar",
    "rielaba": "rielar",
    "rielabais": "rielar",
    "rielaban": "rielar",
    "rielabas": "rielar",
    "rielad": "rielar",
    "rielada": "rielar",
    "rieladas": "rielar",
    "rielado": "rielar",
    "rielados": "rielar",
    "rielamos": "rielar",
    "rielan": "rielar",
    "rielando": "rielar",
    "rielara": "rielar",
    "rielarais": "rielar",
    "rielaran": "rielar",
    "rielaras": "rielar",
    "rielare": "rielar",
    "rielareis": "rielar",
    "rielaremos": "rielar",
    "rielaren": "rielar",
    "rielares": "rielar",
    "rielaron": "rielar",
    "rielará": "rielar",
    "rielarán": "rielar",
    "rielarás": "rielar",
    "rielaré": "rielar",
    "rielaréis": "rielar",
    "rielaría": "rielar",
    "rielaríais": "rielar",
    "rielaríamos": "rielar",
    "rielarían": "rielar",
    "rielarías": "rielar",
    "rielas": "rielar",
    "rielase": "rielar",
    "rielaseis": "rielar",
    "rielasen": "rielar",
    "rielases": "rielar",
    "rielaste": "rielar",
    "rielasteis": "rielar",
    "riele": "rielar",
    "rielemos": "rielar",
    "rielen": "rielar",
    "rieleras": "rielera",
    "rieles": "rielar",
    "rielo": "rielar",
    "rielábamos": "rielar",
    "rieláis": "rielar",
    "rieláramos": "rielar",
    "rieláremos": "rielar",
    "rielásemos": "rielar",
    "rielé": "rielar",
    "rieléis": "rielar",
    "rieló": "rielar",
    "riendas": "rienda",
    "riendo": "reír",
    "rientes": "riente",
    "riera": "reír",
    "rierais": "reír",
    "rieran": "reír",
    "rieras": "riera",
    "riere": "reír",
    "riereis": "reír",
    "rieren": "reír",
    "rieres": "reír",
    "rieron": "reír",
    "riese": "reír",
    "rieseis": "reír",
    "riesen": "reír",
    "rieses": "reír",
    "riesgos": "riesgo",
    "riesgosa": "riesgoso",
    "riesgosas": "riesgoso",
    "riesgosos": "riesgoso",
    "rifa": "rifar",
    "rifaba": "rifar",
    "rifabais": "rifar",
    "rifaban": "rifar",
    "rifabas": "rifar",
    "rifad": "rifar",
    "rifada": "rifar",
    "rifadas": "rifar",
    "rifado": "rifar",
    "rifadora": "rifador",
    "rifadoras": "rifador",
    "rifadores": "rifador",
    "rifados": "rifar",
    "rifaduras": "rifadura",
    "rifamos": "rifar",
    "rifampicinas": "rifampicina",
    "rifan": "rifar",
    "rifando": "rifar",
    "rifara": "rifar",
    "rifarais": "rifar",
    "rifaran": "rifar",
    "rifaras": "rifar",
    "rifare": "rifar",
    "rifareis": "rifar",
    "rifaremos": "rifar",
    "rifaren": "rifar",
    "rifares": "rifar",
    "rifaron": "rifar",
    "rifará": "rifar",
    "rifarán": "rifar",
    "rifarás": "rifar",
    "rifaré": "rifar",
    "rifaréis": "rifar",
    "rifaría": "rifar",
    "rifaríais": "rifar",
    "rifaríamos": "rifar",
    "rifarían": "rifar",
    "rifarías": "rifar",
    "rifas": "rifar",
    "rifase": "rifar",
    "rifaseis": "rifar",
    "rifasen": "rifar",
    "rifases": "rifar",
    "rifaste": "rifar",
    "rifasteis": "rifar",
    "rife": "rifar",
    "rifemos": "rifar",
    "rifen": "rifar",
    "rifes": "rifar",
    "rifeña": "rifeño",
    "rifeñas": "rifeño",
    "rifeños": "rifeño",
    "rifirrafes": "rifirrafe",
    "rifles": "rifle",
    "rifo": "rifar",
    "rifts": "rift",
    "rifábamos": "rifar",
    "rifáis": "rifar",
    "rifáramos": "rifar",
    "rifáremos": "rifar",
    "rifásemos": "rifar",
    "rifé": "rifar",
    "riféis": "rifar",
    "rifó": "rifar",
    "rige": "regir",
    "rigen": "regir",
    "rigentes": "rigente",
    "riges": "regir",
    "rigideces": "rigidez",
    "rigidizadores": "rigidizador",
    "rigiendo": "regir",
    "rigiera": "regir",
    "rigierais": "regir",
    "rigieran": "regir",
    "rigieras": "regir",
    "rigiere": "regir",
    "rigiereis": "regir",
    "rigieren": "regir",
    "rigieres": "regir",
    "rigieron": "regir",
    "rigiese": "regir",
    "rigieseis": "regir",
    "rigiesen": "regir",
    "rigieses": "regir",
    "rigiéramos": "regir",
    "rigiéremos": "regir",
    "rigiésemos": "regir",
    "rigió": "regir",
    "rigodones": "rigodón",
    "rigolas": "rigola",
    "rigores": "rigor",
    "rigorismos": "rigorismo",
    "rigoristas": "rigorista",
    "rigorosa": "rigoroso",
    "rigorosas": "rigoroso",
    "rigorosos": "rigoroso",
    "rigurosa": "riguroso",
    "rigurosas": "riguroso",
    "rigurosidades": "rigurosidad",
    "rigurosos": "riguroso",
    "rija": "regir",
    "rijadora": "rijador",
    "rijadoras": "rijador",
    "rijadores": "rijador",
    "rijamos": "regir",
    "rijan": "regir",
    "rijas": "rija",
    "rijo": "regir",
    "rijos": "rijo",
    "rijosa": "rijoso",
    "rijosas": "rijoso",
    "rijosidades": "rijosidad",
    "rijosos": "rijoso",
    "rijáis": "regir",
    "rila": "rilar",
    "rilaba": "rilar",
    "rilabais": "rilar",
    "rilaban": "rilar",
    "rilabas": "rilar",
    "rilad": "rilar",
    "rilada": "rilar",
    "riladas": "rilar",
    "rilado": "rilar",
    "rilados": "rilar",
    "rilamos": "rilar",
    "rilan": "rilar",
    "rilando": "rilar",
    "rilara": "rilar",
    "rilarais": "rilar",
    "rilaran": "rilar",
    "rilaras": "rilar",
    "rilare": "rilar",
    "rilareis": "rilar",
    "rilaremos": "rilar",
    "rilaren": "rilar",
    "rilares": "rilar",
    "rilaron": "rilar",
    "rilará": "rilar",
    "rilarán": "rilar",
    "rilarás": "rilar",
    "rilaré": "rilar",
    "rilaréis": "rilar",
    "rilaría": "rilar",
    "rilaríais": "rilar",
    "rilaríamos": "rilar",
    "rilarían": "rilar",
    "rilarías": "rilar",
    "rilas": "rilar",
    "rilase": "rilar",
    "rilaseis": "rilar",
    "rilasen": "rilar",
    "rilases": "rilar",
    "rilaste": "rilar",
    "rilasteis": "rilar",
    "rile": "rilar",
    "rilemos": "rilar",
    "rilen": "rilar",
    "riles": "rilar",
    "rilo": "rilar",
    "rilábamos": "rilar",
    "riláis": "rilar",
    "riláramos": "rilar",
    "riláremos": "rilar",
    "rilásemos": "rilar",
    "rilé": "rilar",
    "riléis": "rilar",
    "riló": "rilar",
    "rima": "rimar",
    "rimaba": "rimar",
    "rimabais": "rimar",
    "rimaban": "rimar",
    "rimabas": "rimar",
    "rimad": "rimar",
    "rimada": "rimar",
    "rimadas": "rimar",
    "rimado": "rimar",
    "rimadora": "rimador",
    "rimadoras": "rimador",
    "rimadores": "rimador",
    "rimados": "rimar",
    "rimamos": "rimar",
    "riman": "rimar",
    "rimando": "rimar",
    "rimara": "rimar",
    "rimarais": "rimar",
    "rimaran": "rimar",
    "rimaras": "rimar",
    "rimare": "rimar",
    "rimareis": "rimar",
    "rimaremos": "rimar",
    "rimaren": "rimar",
    "rimares": "rimar",
    "rimaron": "rimar",
    "rimará": "rimar",
    "rimarán": "rimar",
    "rimarás": "rimar",
    "rimaré": "rimar",
    "rimaréis": "rimar",
    "rimaría": "rimar",
    "rimaríais": "rimar",
    "rimaríamos": "rimar",
    "rimarían": "rimar",
    "rimarías": "rimar",
    "rimas": "rimar",
    "rimase": "rimar",
    "rimaseis": "rimar",
    "rimasen": "rimar",
    "rimases": "rimar",
    "rimaste": "rimar",
    "rimasteis": "rimar",
    "rimayas": "rimaya",
    "rimbombancias": "rimbombancia",
    "rimbombantes": "rimbombante",
    "rimbombes": "rimbombe",
    "rimbombos": "rimbombo",
    "rime": "rimar",
    "rimels": "rimel",
    "rimemos": "rimar",
    "rimen": "rimar",
    "rimeros": "rimero",
    "rimes": "rimar",
    "rimmeles": "rimmel",
    "rimo": "rimar",
    "rimábamos": "rimar",
    "rimáis": "rimar",
    "rimáramos": "rimar",
    "rimáremos": "rimar",
    "rimásemos": "rimar",
    "rimé": "rimar",
    "riméis": "rimar",
    "rimó": "rimar",
    "rinales": "rinal",
    "rinalgias": "rinalgia",
    "rinconadas": "rinconada",
    "rinconeras": "rinconera",
    "rincones": "rincón",
    "rinda": "rendir",
    "rindamos": "rendir",
    "rindan": "rendir",
    "rindas": "rendir",
    "rinde": "rendir",
    "rinden": "rendir",
    "rindes": "rendir",
    "rindiendo": "rendir",
    "rindiera": "rendir",
    "rindierais": "rendir",
    "rindieran": "rendir",
    "rindieras": "rendir",
    "rindiere": "rendir",
    "rindiereis": "rendir",
    "rindieren": "rendir",
    "rindieres": "rendir",
    "rindieron": "rendir",
    "rindiese": "rendir",
    "rindieseis": "rendir",
    "rindiesen": "rendir",
    "rindieses": "rendir",
    "rindiéramos": "rendir",
    "rindiéremos": "rendir",
    "rindiésemos": "rendir",
    "rindió": "rendir",
    "rindo": "rendir",
    "rindáis": "rendir",
    "rinencéfalos": "rinencéfalo",
    "ringgits": "ringgit",
    "ringlas": "ringla",
    "ringleras": "ringlera",
    "ringleros": "ringlero",
    "ringles": "ringle",
    "ringorrangos": "ringorrango",
    "rings": "ring",
    "rinocerontes": "rinoceronte",
    "rinofaringes": "rinofaringe",
    "rinofaringoscopias": "rinofaringoscopia",
    "rinofaríngea": "rinofaríngeo",
    "rinofaríngeas": "rinofaríngeo",
    "rinofaríngeos": "rinofaríngeo",
    "rinofimas": "rinofima",
    "rinofonías": "rinofonía",
    "rinolalias": "rinolalia",
    "rinolitos": "rinolito",
    "rinologías": "rinología",
    "rinomanometrías": "rinomanometría",
    "rinomanómetros": "rinomanómetro",
    "rinometrías": "rinometría",
    "rinopatías": "rinopatía",
    "rinoplastias": "rinoplastia",
    "rinorreas": "rinorrea",
    "rinoscleromas": "rinoscleroma",
    "rinoscopias": "rinoscopia",
    "rinoscopios": "rinoscopio",
    "rinotomías": "rinotomía",
    "rinóloga": "rinólogo",
    "rinólogas": "rinólogo",
    "rinólogos": "rinólogo",
    "rinómetros": "rinómetro",
    "riodacitas": "riodacita",
    "riojana": "riojano",
    "riojanas": "riojano",
    "riojanos": "riojano",
    "rioladas": "riolada",
    "riolitas": "riolita",
    "rioplatenses": "rioplatense",
    "riostras": "riostra",
    "ripabilidades": "ripabilidad",
    "ripia": "ripiar",
    "ripiaba": "ripiar",
    "ripiabais": "ripiar",
    "ripiaban": "ripiar",
    "ripiabas": "ripiar",
    "ripiad": "ripiar",
    "ripiada": "ripiar",
    "ripiadas": "ripiar",
    "ripiado": "ripiar",
    "ripiados": "ripiar",
    "ripiamos": "ripiar",
    "ripian": "ripiar",
    "ripiando": "ripiar",
    "ripiara": "ripiar",
    "ripiarais": "ripiar",
    "ripiaran": "ripiar",
    "ripiaras": "ripiar",
    "ripiare": "ripiar",
    "ripiareis": "ripiar",
    "ripiaremos": "ripiar",
    "ripiaren": "ripiar",
    "ripiares": "ripiar",
    "ripiaron": "ripiar",
    "ripiará": "ripiar",
    "ripiarán": "ripiar",
    "ripiarás": "ripiar",
    "ripiaré": "ripiar",
    "ripiaréis": "ripiar",
    "ripiaría": "ripiar",
    "ripiaríais": "ripiar",
    "ripiaríamos": "ripiar",
    "ripiarían": "ripiar",
    "ripiarías": "ripiar",
    "ripias": "ripiar",
    "ripiase": "ripiar",
    "ripiaseis": "ripiar",
    "ripiasen": "ripiar",
    "ripiases": "ripiar",
    "ripiaste": "ripiar",
    "ripiasteis": "ripiar",
    "ripidolitas": "ripidolita",
    "ripie": "ripiar",
    "ripiemos": "ripiar",
    "ripien": "ripiar",
    "ripies": "ripiar",
    "ripio": "ripiar",
    "ripios": "ripio",
    "ripiosa": "ripioso",
    "ripiosas": "ripioso",
    "ripiosos": "ripioso",
    "ripiábamos": "ripiar",
    "ripiáis": "ripiar",
    "ripiáramos": "ripiar",
    "ripiáremos": "ripiar",
    "ripiásemos": "ripiar",
    "ripié": "ripiar",
    "ripiéis": "ripiar",
    "ripió": "ripiar",
    "riquezas": "riqueza",
    "riquísima": "riquísimo",
    "riquísimas": "riquísimo",
    "riquísimos": "riquísimo",
    "risadas": "risada",
    "risas": "risa",
    "riscales": "riscal",
    "riscos": "risco",
    "riscosa": "riscoso",
    "riscosas": "riscoso",
    "riscosos": "riscoso",
    "risibilidades": "risibilidad",
    "risibles": "risible",
    "risiones": "risión",
    "risitas": "risita",
    "risorgimentos": "risorgimento",
    "risoria": "risorio",
    "risorias": "risorio",
    "risorios": "risorio",
    "risotadas": "risotada",
    "risoteos": "risoteo",
    "risottos": "risotto",
    "rispa": "rispo",
    "rispas": "rispo",
    "rispos": "rispo",
    "ristocetinas": "ristocetina",
    "ristras": "ristra",
    "ristreles": "ristrel",
    "ristres": "ristre",
    "risueña": "risueño",
    "risueñas": "risueño",
    "risueños": "risueño",
    "ritardandos": "ritardando",
    "ritma": "ritmar",
    "ritmaba": "ritmar",
    "ritmabais": "ritmar",
    "ritmaban": "ritmar",
    "ritmabas": "ritmar",
    "ritmad": "ritmar",
    "ritmada": "ritmar",
    "ritmadas": "ritmar",
    "ritmado": "ritmar",
    "ritmados": "ritmar",
    "ritmamos": "ritmar",
    "ritman": "ritmar",
    "ritmando": "ritmar",
    "ritmara": "ritmar",
    "ritmarais": "ritmar",
    "ritmaran": "ritmar",
    "ritmaras": "ritmar",
    "ritmare": "ritmar",
    "ritmareis": "ritmar",
    "ritmaremos": "ritmar",
    "ritmaren": "ritmar",
    "ritmares": "ritmar",
    "ritmaron": "ritmar",
    "ritmará": "ritmar",
    "ritmarán": "ritmar",
    "ritmarás": "ritmar",
    "ritmaré": "ritmar",
    "ritmaréis": "ritmar",
    "ritmaría": "ritmar",
    "ritmaríais": "ritmar",
    "ritmaríamos": "ritmar",
    "ritmarían": "ritmar",
    "ritmarías": "ritmar",
    "ritmas": "ritmar",
    "ritmase": "ritmar",
    "ritmaseis": "ritmar",
    "ritmasen": "ritmar",
    "ritmases": "ritmar",
    "ritmaste": "ritmar",
    "ritmasteis": "ritmar",
    "ritme": "ritmar",
    "ritmemos": "ritmar",
    "ritmen": "ritmar",
    "ritmes": "ritmar",
    "ritmo": "ritmar",
    "ritmos": "ritmo",
    "ritmábamos": "ritmar",
    "ritmáis": "ritmar",
    "ritmáramos": "ritmar",
    "ritmáremos": "ritmar",
    "ritmásemos": "ritmar",
    "ritmé": "ritmar",
    "ritméis": "ritmar",
    "ritmó": "ritmar",
    "ritornelos": "ritornelo",
    "ritos": "rito",
    "rituales": "ritual",
    "ritualidades": "ritualidad",
    "ritualismos": "ritualismo",
    "ritualistas": "ritualista",
    "rivales": "rival",
    "rivalice": "rivalizar",
    "rivalicemos": "rivalizar",
    "rivalicen": "rivalizar",
    "rivalices": "rivalizar",
    "rivalicé": "rivalizar",
    "rivalicéis": "rivalizar",
    "rivalidades": "rivalidad",
    "rivaliza": "rivalizar",
    "rivalizaba": "rivalizar",
    "rivalizabais": "rivalizar",
    "rivalizaban": "rivalizar",
    "rivalizabas": "rivalizar",
    "rivalizad": "rivalizar",
    "rivalizada": "rivalizar",
    "rivalizadas": "rivalizar",
    "rivalizado": "rivalizar",
    "rivalizados": "rivalizar",
    "rivalizamos": "rivalizar",
    "rivalizan": "rivalizar",
    "rivalizando": "rivalizar",
    "rivalizara": "rivalizar",
    "rivalizarais": "rivalizar",
    "rivalizaran": "rivalizar",
    "rivalizaras": "rivalizar",
    "rivalizare": "rivalizar",
    "rivalizareis": "rivalizar",
    "rivalizaremos": "rivalizar",
    "rivalizaren": "rivalizar",
    "rivalizares": "rivalizar",
    "rivalizaron": "rivalizar",
    "rivalizará": "rivalizar",
    "rivalizarán": "rivalizar",
    "rivalizarás": "rivalizar",
    "rivalizaré": "rivalizar",
    "rivalizaréis": "rivalizar",
    "rivalizaría": "rivalizar",
    "rivalizaríais": "rivalizar",
    "rivalizaríamos": "rivalizar",
    "rivalizarían": "rivalizar",
    "rivalizarías": "rivalizar",
    "rivalizas": "rivalizar",
    "rivalizase": "rivalizar",
    "rivalizaseis": "rivalizar",
    "rivalizasen": "rivalizar",
    "rivalizases": "rivalizar",
    "rivalizaste": "rivalizar",
    "rivalizasteis": "rivalizar",
    "rivalizo": "rivalizar",
    "rivalizábamos": "rivalizar",
    "rivalizáis": "rivalizar",
    "rivalizáramos": "rivalizar",
    "rivalizáremos": "rivalizar",
    "rivalizásemos": "rivalizar",
    "rivalizó": "rivalizar",
    "rivanoles": "rivanol",
    "river-skis": "river-ski",
    "riyales": "riyal",
    "riza": "rizo",
    "rizaba": "rizar",
    "rizabais": "rizar",
    "rizaban": "rizar",
    "rizabas": "rizar",
    "rizad": "rizar",
    "rizada": "rizar",
    "rizadas": "rizar",
    "rizado": "rizar",
    "rizadora": "rizador",
    "rizadoras": "rizador",
    "rizadores": "rizador",
    "rizados": "rizar",
    "rizaduras": "rizadura",
    "rizamos": "rizar",
    "rizan": "rizar",
    "rizando": "rizar",
    "rizara": "rizar",
    "rizarais": "rizar",
    "rizaran": "rizar",
    "rizaras": "rizar",
    "rizare": "rizar",
    "rizareis": "rizar",
    "rizaremos": "rizar",
    "rizaren": "rizar",
    "rizares": "rizar",
    "rizaron": "rizar",
    "rizará": "rizar",
    "rizarán": "rizar",
    "rizarás": "rizar",
    "rizaré": "rizar",
    "rizaréis": "rizar",
    "rizaría": "rizar",
    "rizaríais": "rizar",
    "rizaríamos": "rizar",
    "rizarían": "rizar",
    "rizarías": "rizar",
    "rizas": "rizo",
    "rizase": "rizar",
    "rizaseis": "rizar",
    "rizasen": "rizar",
    "rizases": "rizar",
    "rizaste": "rizar",
    "rizasteis": "rizar",
    "rizo": "rizar",
    "rizobios": "rizobio",
    "rizoforácea": "rizoforáceo",
    "rizoforáceas": "rizoforáceo",
    "rizoforáceos": "rizoforáceo",
    "rizofóreos": "rizofóreo",
    "rizoides": "rizoide",
    "rizomas": "rizoma",
    "rizos": "rizo",
    "rizosa": "rizoso",
    "rizosas": "rizoso",
    "rizosferas": "rizosfera",
    "rizosos": "rizoso",
    "rizábamos": "rizar",
    "rizáis": "rizar",
    "rizáramos": "rizar",
    "rizáremos": "rizar",
    "rizásemos": "rizar",
    "rizó": "rizar",
    "rizófaga": "rizófago",
    "rizófagas": "rizófago",
    "rizófagos": "rizófago",
    "rizópodos": "rizópodo",
    "riáis": "reír",
    "riéramos": "reír",
    "riéremos": "reír",
    "riésemos": "reír",
    "riña": "reñir",
    "riñamos": "reñir",
    "riñan": "reñir",
    "riñas": "riña",
    "riñe": "reñir",
    "riñen": "reñir",
    "riñendo": "reñir",
    "riñera": "reñir",
    "riñerais": "reñir",
    "riñeran": "reñir",
    "riñeras": "reñir",
    "riñere": "reñir",
    "riñereis": "reñir",
    "riñeren": "reñir",
    "riñeres": "reñir",
    "riñeron": "reñir",
    "riñes": "reñir",
    "riñese": "reñir",
    "riñeseis": "reñir",
    "riñesen": "reñir",
    "riñeses": "reñir",
    "riño": "reñir",
    "riñonadas": "riñonada",
    "riñoneras": "riñonera",
    "riñones": "riñón",
    "riñáis": "reñir",
    "riñéramos": "reñir",
    "riñéremos": "reñir",
    "riñésemos": "reñir",
    "riñó": "reñir",
    "rió": "reír",
    "roa": "roer",
    "roamos": "roer",
    "roan": "roer",
    "roana": "roano",
    "roanas": "roano",
    "roanos": "roano",
    "roas": "roer",
    "roastbeefs": "roastbeef",
    "roba": "robar",
    "robaba": "robar",
    "robabais": "robar",
    "robaban": "robar",
    "robabas": "robar",
    "robad": "robar",
    "robada": "robar",
    "robadas": "robar",
    "robaderas": "robadera",
    "robadizos": "robadizo",
    "robado": "robar",
    "robadora": "robador",
    "robadoras": "robador",
    "robadores": "robador",
    "robados": "robar",
    "robalizas": "robaliza",
    "robalos": "robalo",
    "robamos": "robar",
    "roban": "robar",
    "robando": "robar",
    "robara": "robar",
    "robarais": "robar",
    "robaran": "robar",
    "robaras": "robar",
    "robare": "robar",
    "robareis": "robar",
    "robaremos": "robar",
    "robaren": "robar",
    "robares": "robar",
    "robaron": "robar",
    "robará": "robar",
    "robarán": "robar",
    "robarás": "robar",
    "robaré": "robar",
    "robaréis": "robar",
    "robaría": "robar",
    "robaríais": "robar",
    "robaríamos": "robar",
    "robarían": "robar",
    "robarías": "robar",
    "robas": "robar",
    "robase": "robar",
    "robaseis": "robar",
    "robasen": "robar",
    "robases": "robar",
    "robaste": "robar",
    "robasteis": "robar",
    "robe": "robar",
    "robellones": "robellón",
    "robemos": "robar",
    "roben": "robar",
    "robes": "robar",
    "robezos": "robezo",
    "robines": "robín",
    "robineterías": "robinetería",
    "robinias": "robinia",
    "robinsones": "robinsón",
    "robinsoniana": "robinsoniano",
    "robinsonianas": "robinsoniano",
    "robinsonianos": "robinsoniano",
    "robla": "roblar",
    "roblaba": "roblar",
    "roblabais": "roblar",
    "roblaban": "roblar",
    "roblabas": "roblar",
    "roblad": "roblar",
    "roblada": "roblar",
    "robladas": "roblar",
    "robladera": "robladero",
    "robladeras": "robladero",
    "robladeros": "robladero",
    "roblado": "roblar",
    "roblados": "roblar",
    "robladuras": "robladura",
    "roblamos": "roblar",
    "roblan": "roblar",
    "roblando": "roblar",
    "roblara": "roblar",
    "roblarais": "roblar",
    "roblaran": "roblar",
    "roblaras": "roblar",
    "roblare": "roblar",
    "roblareis": "roblar",
    "roblaremos": "roblar",
    "roblaren": "roblar",
    "roblares": "roblar",
    "roblaron": "roblar",
    "roblará": "roblar",
    "roblarán": "roblar",
    "roblarás": "roblar",
    "roblaré": "roblar",
    "roblaréis": "roblar",
    "roblaría": "roblar",
    "roblaríais": "roblar",
    "roblaríamos": "roblar",
    "roblarían": "roblar",
    "roblarías": "roblar",
    "roblas": "roblar",
    "roblase": "roblar",
    "roblaseis": "roblar",
    "roblasen": "roblar",
    "roblases": "roblar",
    "roblaste": "roblar",
    "roblasteis": "roblar",
    "roble": "roblar",
    "robledales": "robledal",
    "robledares": "robledar",
    "robledas": "robleda",
    "robledos": "robledo",
    "roblemos": "roblar",
    "roblen": "roblar",
    "robles": "roble",
    "robliza": "roblizo",
    "roblizas": "roblizo",
    "roblizos": "roblizo",
    "roblo": "roblar",
    "roblona": "roblonar",
    "roblonaba": "roblonar",
    "roblonabais": "roblonar",
    "roblonaban": "roblonar",
    "roblonabas": "roblonar",
    "roblonad": "roblonar",
    "roblonada": "roblonar",
    "roblonadas": "roblonar",
    "roblonado": "roblonar",
    "roblonadoras": "roblonadora",
    "roblonados": "roblonar",
    "roblonamos": "roblonar",
    "roblonan": "roblonar",
    "roblonando": "roblonar",
    "roblonara": "roblonar",
    "roblonarais": "roblonar",
    "roblonaran": "roblonar",
    "roblonaras": "roblonar",
    "roblonare": "roblonar",
    "roblonareis": "roblonar",
    "roblonaremos": "roblonar",
    "roblonaren": "roblonar",
    "roblonares": "roblonar",
    "roblonaron": "roblonar",
    "roblonará": "roblonar",
    "roblonarán": "roblonar",
    "roblonarás": "roblonar",
    "roblonaré": "roblonar",
    "roblonaréis": "roblonar",
    "roblonaría": "roblonar",
    "roblonaríais": "roblonar",
    "roblonaríamos": "roblonar",
    "roblonarían": "roblonar",
    "roblonarías": "roblonar",
    "roblonas": "roblonar",
    "roblonase": "roblonar",
    "roblonaseis": "roblonar",
    "roblonasen": "roblonar",
    "roblonases": "roblonar",
    "roblonaste": "roblonar",
    "roblonasteis": "roblonar",
    "roblone": "roblonar",
    "roblonemos": "roblonar",
    "roblonen": "roblonar",
    "roblones": "roblonar",
    "roblono": "roblonar",
    "roblonábamos": "roblonar",
    "roblonáis": "roblonar",
    "roblonáramos": "roblonar",
    "roblonáremos": "roblonar",
    "roblonásemos": "roblonar",
    "robloné": "roblonar",
    "roblonéis": "roblonar",
    "roblonó": "roblonar",
    "roblábamos": "roblar",
    "robláis": "roblar",
    "robláramos": "roblar",
    "robláremos": "roblar",
    "roblásemos": "roblar",
    "roblé": "roblar",
    "robléis": "roblar",
    "robló": "roblar",
    "robo": "robar",
    "robora": "roborar",
    "roboraba": "roborar",
    "roborabais": "roborar",
    "roboraban": "roborar",
    "roborabas": "roborar",
    "roboraciones": "roboración",
    "roborad": "roborar",
    "roborada": "roborar",
    "roboradas": "roborar",
    "roborado": "roborar",
    "roborados": "roborar",
    "roboramos": "roborar",
    "roboran": "roborar",
    "roborando": "roborar",
    "roborantes": "roborante",
    "roborara": "roborar",
    "roborarais": "roborar",
    "roboraran": "roborar",
    "roboraras": "roborar",
    "roborare": "roborar",
    "roborareis": "roborar",
    "roboraremos": "roborar",
    "roboraren": "roborar",
    "roborares": "roborar",
    "roboraron": "roborar",
    "roborará": "roborar",
    "roborarán": "roborar",
    "roborarás": "roborar",
    "roboraré": "roborar",
    "roboraréis": "roborar",
    "roboraría": "roborar",
    "roboraríais": "roborar",
    "roboraríamos": "roborar",
    "roborarían": "roborar",
    "roborarías": "roborar",
    "roboras": "roborar",
    "roborase": "roborar",
    "roboraseis": "roborar",
    "roborasen": "roborar",
    "roborases": "roborar",
    "roboraste": "roborar",
    "roborasteis": "roborar",
    "roborativa": "roborativo",
    "roborativas": "roborativo",
    "roborativos": "roborativo",
    "robore": "roborar",
    "roboremos": "roborar",
    "roboren": "roborar",
    "robores": "roborar",
    "roboro": "roborar",
    "roborábamos": "roborar",
    "roboráis": "roborar",
    "roboráramos": "roborar",
    "roboráremos": "roborar",
    "roborásemos": "roborar",
    "roboré": "roborar",
    "roboréis": "roborar",
    "roboró": "roborar",
    "robos": "robo",
    "robotizaciones": "robotización",
    "robotizaciónes": "robotización",
    "robotizada": "robotizado",
    "robotizadas": "robotizado",
    "robotizados": "robotizado",
    "robots": "robot",
    "robras": "robra",
    "robredales": "robredal",
    "robredos": "robredo",
    "robres": "robre",
    "robusta": "robusto",
    "robustas": "robusto",
    "robustece": "robustecer",
    "robusteced": "robustecer",
    "robustecedora": "robustecedor",
    "robustecedoras": "robustecedor",
    "robustecedores": "robustecedor",
    "robustecemos": "robustecer",
    "robustecen": "robustecer",
    "robusteceremos": "robustecer",
    "robustecerá": "robustecer",
    "robustecerán": "robustecer",
    "robustecerás": "robustecer",
    "robusteceré": "robustecer",
    "robusteceréis": "robustecer",
    "robustecería": "robustecer",
    "robusteceríais": "robustecer",
    "robusteceríamos": "robustecer",
    "robustecerían": "robustecer",
    "robustecerías": "robustecer",
    "robusteces": "robustez",
    "robustecida": "robustecer",
    "robustecidas": "robustecer",
    "robustecido": "robustecer",
    "robustecidos": "robustecer",
    "robusteciendo": "robustecer",
    "robusteciera": "robustecer",
    "robustecierais": "robustecer",
    "robustecieran": "robustecer",
    "robustecieras": "robustecer",
    "robusteciere": "robustecer",
    "robusteciereis": "robustecer",
    "robustecieren": "robustecer",
    "robustecieres": "robustecer",
    "robustecieron": "robustecer",
    "robusteciese": "robustecer",
    "robustecieseis": "robustecer",
    "robusteciesen": "robustecer",
    "robustecieses": "robustecer",
    "robustecimientos": "robustecimiento",
    "robustecimos": "robustecer",
    "robusteciste": "robustecer",
    "robustecisteis": "robustecer",
    "robusteciéramos": "robustecer",
    "robusteciéremos": "robustecer",
    "robusteciésemos": "robustecer",
    "robusteció": "robustecer",
    "robustecéis": "robustecer",
    "robustecí": "robustecer",
    "robustecía": "robustecer",
    "robustecíais": "robustecer",
    "robustecíamos": "robustecer",
    "robustecían": "robustecer",
    "robustecías": "robustecer",
    "robustezas": "robusteza",
    "robustezca": "robustecer",
    "robustezcamos": "robustecer",
    "robustezcan": "robustecer",
    "robustezcas": "robustecer",
    "robustezco": "robustecer",
    "robustezcáis": "robustecer",
    "robustos": "robusto",
    "robábamos": "robar",
    "robáis": "robar",
    "robáramos": "robar",
    "robáremos": "robar",
    "robásemos": "robar",
    "robé": "robar",
    "robéis": "robar",
    "robó": "robar",
    "robótica": "robótico",
    "robóticas": "robótico",
    "robóticos": "robótico",
    "rocadas": "rocada",
    "rocaderos": "rocadero",
    "rocadores": "rocador",
    "rocallas": "rocalla",
    "rocallosa": "rocalloso",
    "rocallosas": "rocalloso",
    "rocallosos": "rocalloso",
    "rocambolesca": "rocambolesco",
    "rocambolescas": "rocambolesco",
    "rocambolescos": "rocambolesco",
    "rocanrolera": "rocanrolero",
    "rocanroleras": "rocanrolero",
    "rocanroleros": "rocanrolero",
    "rocas": "roca",
    "roce": "rozar",
    "rocemos": "rozar",
    "rocen": "rozar",
    "roces": "rozar",
    "rochas": "rocha",
    "rociaba": "rociar",
    "rociabais": "rociar",
    "rociaban": "rociar",
    "rociabas": "rociar",
    "rociad": "rociar",
    "rociada": "rociar",
    "rociadas": "rociar",
    "rociado": "rociar",
    "rociadora": "rociador",
    "rociadoras": "rociador",
    "rociadores": "rociador",
    "rociados": "rociar",
    "rociaduras": "rociadura",
    "rociamientos": "rociamiento",
    "rociamos": "rociar",
    "rociando": "rociar",
    "rociara": "rociar",
    "rociarais": "rociar",
    "rociaran": "rociar",
    "rociaras": "rociar",
    "rociare": "rociar",
    "rociareis": "rociar",
    "rociaremos": "rociar",
    "rociaren": "rociar",
    "rociares": "rociar",
    "rociaron": "rociar",
    "rociará": "rociar",
    "rociarán": "rociar",
    "rociarás": "rociar",
    "rociaré": "rociar",
    "rociaréis": "rociar",
    "rociaría": "rociar",
    "rociaríais": "rociar",
    "rociaríamos": "rociar",
    "rociarían": "rociar",
    "rociarías": "rociar",
    "rociase": "rociar",
    "rociaseis": "rociar",
    "rociasen": "rociar",
    "rociases": "rociar",
    "rociaste": "rociar",
    "rociasteis": "rociar",
    "rociemos": "rociar",
    "rociera": "rociero",
    "rocieras": "rociero",
    "rocieros": "rociero",
    "rocinales": "rocinal",
    "rocinantes": "rocinante",
    "rocines": "rocín",
    "rociniega": "rociniego",
    "rociniegas": "rociniego",
    "rociniegos": "rociniego",
    "rocinos": "rocino",
    "rociones": "roción",
    "rociábamos": "rociar",
    "rociáis": "rociar",
    "rociáramos": "rociar",
    "rociáremos": "rociar",
    "rociásemos": "rociar",
    "rocié": "rociar",
    "rociéis": "rociar",
    "roció": "rociar",
    "rockabillys": "rockabilly",
    "rockandrolles": "rockandroll",
    "rockandrolls": "rockandroll",
    "rockera": "rockero",
    "rockeras": "rockero",
    "rockeros": "rockero",
    "rocks": "rock",
    "rococós": "rococó",
    "rocosa": "rocoso",
    "rocosas": "rocoso",
    "rocosos": "rocoso",
    "rocé": "rozar",
    "rocéis": "rozar",
    "rocía": "rociar",
    "rocían": "rociar",
    "rocías": "rociar",
    "rocíe": "rociar",
    "rocíen": "rociar",
    "rocíes": "rociar",
    "rocío": "rociar",
    "rocíos": "rocío",
    "rocódromos": "rocódromo",
    "rodaba": "rodar",
    "rodabais": "rodar",
    "rodaballos": "rodaballo",
    "rodaban": "rodar",
    "rodabas": "rodar",
    "rodad": "rodar",
    "rodada": "rodar",
    "rodadas": "rodar",
    "rodadera": "rodadero",
    "rodaderas": "rodadero",
    "rodaderos": "rodadero",
    "rodadiza": "rodadizo",
    "rodadizas": "rodadizo",
    "rodadizos": "rodadizo",
    "rodado": "rodar",
    "rodadora": "rodador",
    "rodadoras": "rodador",
    "rodadores": "rodador",
    "rodados": "rodar",
    "rodaduras": "rodadura",
    "rodajas": "rodaja",
    "rodajes": "rodaje",
    "rodales": "rodal",
    "rodamientos": "rodamiento",
    "rodaminas": "rodamina",
    "rodamos": "rodar",
    "rodando": "rodar",
    "rodaninas": "rodanina",
    "rodantes": "rodante",
    "rodapelos": "rodapelo",
    "rodapiés": "rodapié",
    "rodaplanchas": "rodaplancha",
    "rodara": "rodar",
    "rodarais": "rodar",
    "rodaran": "rodar",
    "rodaras": "rodar",
    "rodare": "rodar",
    "rodareis": "rodar",
    "rodaremos": "rodar",
    "rodaren": "rodar",
    "rodares": "rodar",
    "rodaron": "rodar",
    "rodará": "rodar",
    "rodarán": "rodar",
    "rodarás": "rodar",
    "rodaré": "rodar",
    "rodaréis": "rodar",
    "rodaría": "rodar",
    "rodaríais": "rodar",
    "rodaríamos": "rodar",
    "rodarían": "rodar",
    "rodarías": "rodar",
    "rodas": "roda",
    "rodase": "rodar",
    "rodaseis": "rodar",
    "rodasen": "rodar",
    "rodases": "rodar",
    "rodaste": "rodar",
    "rodasteis": "rodar",
    "rodea": "rodear",
    "rodeaba": "rodear",
    "rodeabais": "rodear",
    "rodeaban": "rodear",
    "rodeabas": "rodear",
    "rodead": "rodear",
    "rodeada": "rodear",
    "rodeadas": "rodear",
    "rodeado": "rodear",
    "rodeadora": "rodeador",
    "rodeadoras": "rodeador",
    "rodeadores": "rodeador",
    "rodeados": "rodear",
    "rodeamos": "rodear",
    "rodean": "rodear",
    "rodeando": "rodear",
    "rodeara": "rodear",
    "rodearais": "rodear",
    "rodearan": "rodear",
    "rodearas": "rodear",
    "rodeare": "rodear",
    "rodeareis": "rodear",
    "rodearemos": "rodear",
    "rodearen": "rodear",
    "rodeares": "rodear",
    "rodearon": "rodear",
    "rodeará": "rodear",
    "rodearán": "rodear",
    "rodearás": "rodear",
    "rodearé": "rodear",
    "rodearéis": "rodear",
    "rodearía": "rodear",
    "rodearíais": "rodear",
    "rodearíamos": "rodear",
    "rodearían": "rodear",
    "rodearías": "rodear",
    "rodeas": "rodear",
    "rodease": "rodear",
    "rodeaseis": "rodear",
    "rodeasen": "rodear",
    "rodeases": "rodear",
    "rodeaste": "rodear",
    "rodeasteis": "rodear",
    "rodee": "rodear",
    "rodeemos": "rodear",
    "rodeen": "rodear",
    "rodees": "rodear",
    "rodelas": "rodela",
    "rodeleros": "rodelero",
    "rodemos": "rodar",
    "rodena": "rodeno",
    "rodenales": "rodenal",
    "rodenas": "rodeno",
    "rodenos": "rodeno",
    "rodeo": "rodear",
    "rodeos": "rodeo",
    "rodera": "rodero",
    "roderas": "rodero",
    "roderos": "rodero",
    "rodesia": "rodesio",
    "rodesiana": "rodesiano",
    "rodesianas": "rodesiano",
    "rodesianos": "rodesiano",
    "rodesias": "rodesio",
    "rodesios": "rodesio",
    "rodetes": "rodete",
    "rodeznos": "rodezno",
    "rodeábamos": "rodear",
    "rodeáis": "rodear",
    "rodeáramos": "rodear",
    "rodeáremos": "rodear",
    "rodeásemos": "rodear",
    "rodeé": "rodear",
    "rodeéis": "rodear",
    "rodeó": "rodear",
    "rodia": "rodio",
    "rodiados": "rodiado",
    "rodias": "rodio",
    "rodicitas": "rodicita",
    "rodilladas": "rodillada",
    "rodillas": "rodilla",
    "rodillazos": "rodillazo",
    "rodillera": "rodillero",
    "rodilleras": "rodillero",
    "rodilleros": "rodillero",
    "rodillos": "rodillo",
    "rodilluda": "rodilludo",
    "rodilludas": "rodilludo",
    "rodilludos": "rodilludo",
    "rodinoles": "rodinol",
    "rodios": "rodio",
    "rodiotas": "rodiota",
    "rodocrositas": "rodocrosita",
    "rododafnes": "rododafne",
    "rododendros": "rododendro",
    "rodolitas": "rodolita",
    "rodomieles": "rodomiel",
    "rodonitas": "rodonita",
    "rodopsinas": "rodopsina",
    "rodos": "rodo",
    "rodriga": "rodrigar",
    "rodrigaba": "rodrigar",
    "rodrigabais": "rodrigar",
    "rodrigaban": "rodrigar",
    "rodrigabas": "rodrigar",
    "rodrigad": "rodrigar",
    "rodrigada": "rodrigar",
    "rodrigadas": "rodrigar",
    "rodrigado": "rodrigar",
    "rodrigados": "rodrigar",
    "rodrigamos": "rodrigar",
    "rodrigan": "rodrigar",
    "rodrigando": "rodrigar",
    "rodrigara": "rodrigar",
    "rodrigarais": "rodrigar",
    "rodrigaran": "rodrigar",
    "rodrigaras": "rodrigar",
    "rodrigare": "rodrigar",
    "rodrigareis": "rodrigar",
    "rodrigaremos": "rodrigar",
    "rodrigaren": "rodrigar",
    "rodrigares": "rodrigar",
    "rodrigaron": "rodrigar",
    "rodrigará": "rodrigar",
    "rodrigarán": "rodrigar",
    "rodrigarás": "rodrigar",
    "rodrigaré": "rodrigar",
    "rodrigaréis": "rodrigar",
    "rodrigaría": "rodrigar",
    "rodrigaríais": "rodrigar",
    "rodrigaríamos": "rodrigar",
    "rodrigarían": "rodrigar",
    "rodrigarías": "rodrigar",
    "rodrigas": "rodrigar",
    "rodrigase": "rodrigar",
    "rodrigaseis": "rodrigar",
    "rodrigasen": "rodrigar",
    "rodrigases": "rodrigar",
    "rodrigaste": "rodrigar",
    "rodrigasteis": "rodrigar",
    "rodrigazones": "rodrigazón",
    "rodrigo": "rodrigar",
    "rodrigones": "rodrigón",
    "rodrigue": "rodrigar",
    "rodriguemos": "rodrigar",
    "rodriguen": "rodrigar",
    "rodrigues": "rodrigar",
    "rodrigué": "rodrigar",
    "rodriguéis": "rodrigar",
    "rodrigábamos": "rodrigar",
    "rodrigáis": "rodrigar",
    "rodrigáramos": "rodrigar",
    "rodrigáremos": "rodrigar",
    "rodrigásemos": "rodrigar",
    "rodrigó": "rodrigar",
    "rodábamos": "rodar",
    "rodáis": "rodar",
    "rodánica": "rodánico",
    "rodánicas": "rodánico",
    "rodánicos": "rodánico",
    "rodáramos": "rodar",
    "rodáremos": "rodar",
    "rodásemos": "rodar",
    "rodé": "rodar",
    "rodéis": "rodar",
    "rodó": "rodar",
    "rodófitos": "rodófito",
    "roe": "roer",
    "roed": "roer",
    "roedora": "roedor",
    "roedoras": "roedor",
    "roedores": "roedor",
    "roeduras": "roedura",
    "roelas": "roela",
    "roeles": "roel",
    "roemos": "roer",
    "roen": "roer",
    "roentgenes": "roentgen",
    "roentgenográfica": "roentgenográfico",
    "roentgenográficas": "roentgenográfico",
    "roentgenográficos": "roentgenográfico",
    "roentgens": "roentgen",
    "roeremos": "roer",
    "roerá": "roer",
    "roerán": "roer",
    "roerás": "roer",
    "roeré": "roer",
    "roeréis": "roer",
    "roería": "roer",
    "roeríais": "roer",
    "roeríamos": "roer",
    "roerían": "roer",
    "roerías": "roer",
    "roes": "roer",
    "rogaba": "rogar",
    "rogabais": "rogar",
    "rogaban": "rogar",
    "rogabas": "rogar",
    "rogaciones": "rogación",
    "rogad": "rogar",
    "rogada": "rogar",
    "rogadas": "rogar",
    "rogado": "rogar",
    "rogadora": "rogador",
    "rogadoras": "rogador",
    "rogadores": "rogador",
    "rogados": "rogar",
    "rogamos": "rogar",
    "rogando": "rogar",
    "rogara": "rogar",
    "rogarais": "rogar",
    "rogaran": "rogar",
    "rogaras": "rogar",
    "rogare": "rogar",
    "rogareis": "rogar",
    "rogaremos": "rogar",
    "rogaren": "rogar",
    "rogares": "rogar",
    "rogaron": "rogar",
    "rogará": "rogar",
    "rogarán": "rogar",
    "rogarás": "rogar",
    "rogaré": "rogar",
    "rogaréis": "rogar",
    "rogaría": "rogar",
    "rogaríais": "rogar",
    "rogaríamos": "rogar",
    "rogarían": "rogar",
    "rogarías": "rogar",
    "rogase": "rogar",
    "rogaseis": "rogar",
    "rogasen": "rogar",
    "rogases": "rogar",
    "rogaste": "rogar",
    "rogasteis": "rogar",
    "rogativa": "rogativo",
    "rogativas": "rogativo",
    "rogativos": "rogativo",
    "rogatoria": "rogatorio",
    "rogatorias": "rogatorio",
    "rogatorios": "rogatorio",
    "roguemos": "rogar",
    "rogué": "rogar",
    "roguéis": "rogar",
    "rogábamos": "rogar",
    "rogáis": "rogar",
    "rogáramos": "rogar",
    "rogáremos": "rogar",
    "rogásemos": "rogar",
    "rogó": "rogar",
    "roiga": "roer",
    "roigamos": "roer",
    "roigan": "roer",
    "roigas": "roer",
    "roigo": "roer",
    "roigáis": "roer",
    "roja": "rojo",
    "rojales": "rojal",
    "rojas": "rojo",
    "rojea": "rojear",
    "rojeaba": "rojear",
    "rojeabais": "rojear",
    "rojeaban": "rojear",
    "rojeabas": "rojear",
    "rojead": "rojear",
    "rojeada": "rojear",
    "rojeadas": "rojear",
    "rojeado": "rojear",
    "rojeados": "rojear",
    "rojeamos": "rojear",
    "rojean": "rojear",
    "rojeando": "rojear",
    "rojeara": "rojear",
    "rojearais": "rojear",
    "rojearan": "rojear",
    "rojearas": "rojear",
    "rojeare": "rojear",
    "rojeareis": "rojear",
    "rojearemos": "rojear",
    "rojearen": "rojear",
    "rojeares": "rojear",
    "rojearon": "rojear",
    "rojeará": "rojear",
    "rojearán": "rojear",
    "rojearás": "rojear",
    "rojearé": "rojear",
    "rojearéis": "rojear",
    "rojearía": "rojear",
    "rojearíais": "rojear",
    "rojearíamos": "rojear",
    "rojearían": "rojear",
    "rojearías": "rojear",
    "rojeas": "rojear",
    "rojease": "rojear",
    "rojeaseis": "rojear",
    "rojeasen": "rojear",
    "rojeases": "rojear",
    "rojeaste": "rojear",
    "rojeasteis": "rojear",
    "rojeces": "rojez",
    "rojee": "rojear",
    "rojeemos": "rojear",
    "rojeen": "rojear",
    "rojees": "rojear",
    "rojeo": "rojear",
    "rojetes": "rojete",
    "rojeábamos": "rojear",
    "rojeáis": "rojear",
    "rojeáramos": "rojear",
    "rojeáremos": "rojear",
    "rojeásemos": "rojear",
    "rojeé": "rojear",
    "rojeéis": "rojear",
    "rojeó": "rojear",
    "rojiblanca": "rojiblanco",
    "rojiblancas": "rojiblanco",
    "rojiblancos": "rojiblanco",
    "rojiza": "rojizo",
    "rojizas": "rojizo",
    "rojizos": "rojizo",
    "rojos": "rojo",
    "rojuras": "rojura",
    "rola": "rolar",
    "rolaba": "rolar",
    "rolabais": "rolar",
    "rolaban": "rolar",
    "rolabas": "rolar",
    "rolad": "rolar",
    "rolada": "rolar",
    "roladas": "rolar",
    "rolado": "rolar",
    "rolados": "rolar",
    "rolamos": "rolar",
    "rolan": "rolar",
    "rolando": "rolar",
    "rolara": "rolar",
    "rolarais": "rolar",
    "rolaran": "rolar",
    "rolaras": "rolar",
    "rolare": "rolar",
    "rolareis": "rolar",
    "rolaremos": "rolar",
    "rolaren": "rolar",
    "rolares": "rolar",
    "rolaron": "rolar",
    "rolará": "rolar",
    "rolarán": "rolar",
    "rolarás": "rolar",
    "rolaré": "rolar",
    "rolaréis": "rolar",
    "rolaría": "rolar",
    "rolaríais": "rolar",
    "rolaríamos": "rolar",
    "rolarían": "rolar",
    "rolarías": "rolar",
    "rolas": "rolar",
    "rolase": "rolar",
    "rolaseis": "rolar",
    "rolasen": "rolar",
    "rolases": "rolar",
    "rolaste": "rolar",
    "rolasteis": "rolar",
    "roldanas": "roldana",
    "roldes": "rolde",
    "role": "rolar",
    "rolemos": "rolar",
    "rolen": "rolar",
    "roleos": "roleo",
    "roles": "rolar",
    "rollas": "rolla",
    "rollistas": "rollista",
    "rolliza": "rollizo",
    "rollizas": "rollizo",
    "rollizos": "rollizo",
    "rollones": "rollón",
    "rollos": "rollo",
    "rolo": "rolar",
    "rolones": "rolón",
    "rolábamos": "rolar",
    "roláis": "rolar",
    "roláramos": "rolar",
    "roláremos": "rolar",
    "rolásemos": "rolar",
    "rolé": "rolar",
    "roléis": "rolar",
    "roló": "rolar",
    "roma": "romo",
    "romadizos": "romadizo",
    "romaica": "romaico",
    "romaicas": "romaico",
    "romaicos": "romaico",
    "romana": "romano",
    "romanadores": "romanador",
    "romanas": "romano",
    "romanatos": "romanato",
    "romanceadora": "romanceador",
    "romanceadoras": "romanceador",
    "romanceadores": "romanceador",
    "romanceristas": "romancerista",
    "romanceros": "romancero",
    "romances": "romance",
    "romancesca": "romancesco",
    "romancescas": "romancesco",
    "romancescos": "romancesco",
    "romanches": "romanche",
    "romancistas": "romancista",
    "romancística": "romancístico",
    "romancísticas": "romancístico",
    "romancísticos": "romancístico",
    "romaneos": "romaneo",
    "romaneros": "romanero",
    "romanesca": "romanesco",
    "romanescas": "romanesco",
    "romanescos": "romanesco",
    "romanice": "romanizar",
    "romanicemos": "romanizar",
    "romanicen": "romanizar",
    "romanices": "romanizar",
    "romanicé": "romanizar",
    "romanicéis": "romanizar",
    "romanidades": "romanidad",
    "romanismos": "romanismo",
    "romanistas": "romanista",
    "romaniza": "romanizar",
    "romanizaba": "romanizar",
    "romanizabais": "romanizar",
    "romanizaban": "romanizar",
    "romanizabas": "romanizar",
    "romanizaciones": "romanización",
    "romanizad": "romanizar",
    "romanizada": "romanizar",
    "romanizadas": "romanizar",
    "romanizado": "romanizar",
    "romanizados": "romanizar",
    "romanizamos": "romanizar",
    "romanizan": "romanizar",
    "romanizando": "romanizar",
    "romanizara": "romanizar",
    "romanizarais": "romanizar",
    "romanizaran": "romanizar",
    "romanizaras": "romanizar",
    "romanizare": "romanizar",
    "romanizareis": "romanizar",
    "romanizaremos": "romanizar",
    "romanizaren": "romanizar",
    "romanizares": "romanizar",
    "romanizaron": "romanizar",
    "romanizará": "romanizar",
    "romanizarán": "romanizar",
    "romanizarás": "romanizar",
    "romanizaré": "romanizar",
    "romanizaréis": "romanizar",
    "romanizaría": "romanizar",
    "romanizaríais": "romanizar",
    "romanizaríamos": "romanizar",
    "romanizarían": "romanizar",
    "romanizarías": "romanizar",
    "romanizas": "romanizar",
    "romanizase": "romanizar",
    "romanizaseis": "romanizar",
    "romanizasen": "romanizar",
    "romanizases": "romanizar",
    "romanizaste": "romanizar",
    "romanizasteis": "romanizar",
    "romanizo": "romanizar",
    "romanizábamos": "romanizar",
    "romanizáis": "romanizar",
    "romanizáramos": "romanizar",
    "romanizáremos": "romanizar",
    "romanizásemos": "romanizar",
    "romanizó": "romanizar",
    "romanos": "romano",
    "romanticismos": "romanticismo",
    "romanzas": "romanza",
    "romaníes": "romaní",
    "romanís": "romaní",
    "romanísticas": "romanística",
    "romas": "romo",
    "romazas": "romaza",
    "rombales": "rombal",
    "rombencéfalos": "rombencéfalo",
    "romboedros": "romboedro",
    "romboidales": "romboidal",
    "romboidea": "romboideo",
    "romboideas": "romboideo",
    "romboideos": "romboideo",
    "romboides": "romboide",
    "rombos": "rombo",
    "romboédrica": "romboédrico",
    "romboédricas": "romboédrico",
    "romboédricos": "romboédrico",
    "romea": "romeo",
    "romeas": "romeo",
    "romeos": "romeo",
    "romera": "romero",
    "romerajes": "romeraje",
    "romerales": "romeral",
    "romeras": "romero",
    "romerillos": "romerillo",
    "romeritas": "romerita",
    "romeros": "romero",
    "romerías": "romería",
    "romescos": "romesco",
    "romos": "romo",
    "rompa": "romper",
    "rompamos": "romper",
    "rompan": "romper",
    "rompas": "romper",
    "rompe": "romper",
    "romped": "romper",
    "rompedera": "rompedero",
    "rompederas": "rompedero",
    "rompederos": "rompedero",
    "rompedora": "rompedor",
    "rompedoras": "rompedor",
    "rompedores": "rompedor",
    "rompeduras": "rompedura",
    "rompemos": "romper",
    "rompen": "romper",
    "romperemos": "romper",
    "romperá": "romper",
    "romperán": "romper",
    "romperás": "romper",
    "romperé": "romper",
    "romperéis": "romper",
    "rompería": "romper",
    "romperíais": "romper",
    "romperíamos": "romper",
    "romperían": "romper",
    "romperías": "romper",
    "rompes": "romper",
    "rompibles": "rompible",
    "rompida": "romper",
    "rompidas": "romper",
    "rompido": "romper",
    "rompidos": "rompido",
    "rompiendo": "romper",
    "rompientes": "rompiente",
    "rompiera": "romper",
    "rompierais": "romper",
    "rompieran": "romper",
    "rompieras": "romper",
    "rompiere": "romper",
    "rompiereis": "romper",
    "rompieren": "romper",
    "rompieres": "romper",
    "rompieron": "romper",
    "rompiese": "romper",
    "rompieseis": "romper",
    "rompiesen": "romper",
    "rompieses": "romper",
    "rompimientos": "rompimiento",
    "rompimos": "romper",
    "rompiste": "romper",
    "rompisteis": "romper",
    "rompiéramos": "romper",
    "rompiéremos": "romper",
    "rompiésemos": "romper",
    "rompió": "romper",
    "rompo": "romper",
    "rompáis": "romper",
    "rompéis": "romper",
    "rompí": "romper",
    "rompía": "romper",
    "rompíais": "romper",
    "rompíamos": "romper",
    "rompían": "romper",
    "rompías": "romper",
    "románica": "románico",
    "románicas": "románico",
    "románicos": "románico",
    "romántica": "romántico",
    "románticas": "romántico",
    "románticos": "romántico",
    "ronca": "ronco",
    "roncaba": "roncar",
    "roncabais": "roncar",
    "roncaban": "roncar",
    "roncabas": "roncar",
    "roncad": "roncar",
    "roncada": "roncar",
    "roncadas": "roncar",
    "roncado": "roncar",
    "roncadora": "roncador",
    "roncadoras": "roncador",
    "roncadores": "roncador",
    "roncados": "roncar",
    "roncamos": "roncar",
    "roncan": "roncar",
    "roncando": "roncar",
    "roncara": "roncar",
    "roncarais": "roncar",
    "roncaran": "roncar",
    "roncaras": "roncar",
    "roncare": "roncar",
    "roncareis": "roncar",
    "roncaremos": "roncar",
    "roncaren": "roncar",
    "roncares": "roncar",
    "roncaron": "roncar",
    "roncará": "roncar",
    "roncarán": "roncar",
    "roncarás": "roncar",
    "roncaré": "roncar",
    "roncaréis": "roncar",
    "roncaría": "roncar",
    "roncaríais": "roncar",
    "roncaríamos": "roncar",
    "roncarían": "roncar",
    "roncarías": "roncar",
    "roncas": "ronco",
    "roncase": "roncar",
    "roncaseis": "roncar",
    "roncasen": "roncar",
    "roncases": "roncar",
    "roncaste": "roncar",
    "roncasteis": "roncar",
    "ronce": "ronzar",
    "roncea": "roncear",
    "ronceaba": "roncear",
    "ronceabais": "roncear",
    "ronceaban": "roncear",
    "ronceabas": "roncear",
    "roncead": "roncear",
    "ronceada": "roncear",
    "ronceadas": "roncear",
    "ronceado": "roncear",
    "ronceados": "roncear",
    "ronceamos": "roncear",
    "roncean": "roncear",
    "ronceando": "roncear",
    "ronceara": "roncear",
    "roncearais": "roncear",
    "roncearan": "roncear",
    "roncearas": "roncear",
    "ronceare": "roncear",
    "ronceareis": "roncear",
    "roncearemos": "roncear",
    "roncearen": "roncear",
    "ronceares": "roncear",
    "roncearon": "roncear",
    "ronceará": "roncear",
    "roncearán": "roncear",
    "roncearás": "roncear",
    "roncearé": "roncear",
    "roncearéis": "roncear",
    "roncearía": "roncear",
    "roncearíais": "roncear",
    "roncearíamos": "roncear",
    "roncearían": "roncear",
    "roncearías": "roncear",
    "ronceas": "roncear",
    "roncease": "roncear",
    "ronceaseis": "roncear",
    "ronceasen": "roncear",
    "ronceases": "roncear",
    "ronceaste": "roncear",
    "ronceasteis": "roncear",
    "roncee": "roncear",
    "ronceemos": "roncear",
    "ronceen": "roncear",
    "roncees": "roncear",
    "roncemos": "ronzar",
    "roncen": "ronzar",
    "ronceo": "roncear",
    "roncera": "roncero",
    "ronceras": "roncero",
    "ronceros": "roncero",
    "roncerías": "roncería",
    "ronces": "ronzar",
    "ronceábamos": "roncear",
    "ronceáis": "roncear",
    "ronceáramos": "roncear",
    "ronceáremos": "roncear",
    "ronceásemos": "roncear",
    "ronceé": "roncear",
    "ronceéis": "roncear",
    "ronceó": "roncear",
    "ronchas": "roncha",
    "ronchones": "ronchón",
    "ronco": "roncar",
    "roncones": "roncón",
    "roncos": "ronco",
    "roncábamos": "roncar",
    "roncáis": "roncar",
    "roncáramos": "roncar",
    "roncáremos": "roncar",
    "roncásemos": "roncar",
    "roncé": "ronzar",
    "roncéis": "ronzar",
    "roncó": "roncar",
    "ronda": "rondar",
    "rondaba": "rondar",
    "rondabais": "rondar",
    "rondaban": "rondar",
    "rondabas": "rondar",
    "rondad": "rondar",
    "rondada": "rondar",
    "rondadas": "rondar",
    "rondado": "rondar",
    "rondadora": "rondador",
    "rondadoras": "rondador",
    "rondadores": "rondador",
    "rondados": "rondar",
    "rondallas": "rondalla",
    "rondamos": "rondar",
    "rondan": "rondar",
    "rondanas": "rondana",
    "rondando": "rondar",
    "rondara": "rondar",
    "rondarais": "rondar",
    "rondaran": "rondar",
    "rondaras": "rondar",
    "rondare": "rondar",
    "rondareis": "rondar",
    "rondaremos": "rondar",
    "rondaren": "rondar",
    "rondares": "rondar",
    "rondaron": "rondar",
    "rondará": "rondar",
    "rondarán": "rondar",
    "rondarás": "rondar",
    "rondaré": "rondar",
    "rondaréis": "rondar",
    "rondaría": "rondar",
    "rondaríais": "rondar",
    "rondaríamos": "rondar",
    "rondarían": "rondar",
    "rondarías": "rondar",
    "rondas": "rondar",
    "rondase": "rondar",
    "rondaseis": "rondar",
    "rondasen": "rondar",
    "rondases": "rondar",
    "rondaste": "rondar",
    "rondasteis": "rondar",
    "ronde": "rondar",
    "rondeles": "rondel",
    "rondemos": "rondar",
    "ronden": "rondar",
    "rondes": "rondar",
    "rondeña": "rondeño",
    "rondeñas": "rondeño",
    "rondeños": "rondeño",
    "rondices": "rondiz",
    "rondines": "rondín",
    "rondises": "rondís",
    "rondo": "rondar",
    "rondábamos": "rondar",
    "rondáis": "rondar",
    "rondáramos": "rondar",
    "rondáremos": "rondar",
    "rondásemos": "rondar",
    "rondé": "rondar",
    "rondéis": "rondar",
    "rondó": "rondar",
    "rondós": "rondó",
    "rones": "ron",
    "ronque": "roncar",
    "ronquea": "ronquear",
    "ronqueaba": "ronquear",
    "ronqueabais": "ronquear",
    "ronqueaban": "ronquear",
    "ronqueabas": "ronquear",
    "ronquead": "ronquear",
    "ronqueada": "ronquear",
    "ronqueadas": "ronquear",
    "ronqueado": "ronquear",
    "ronqueadora": "ronqueador",
    "ronqueadoras": "ronqueador",
    "ronqueadores": "ronqueador",
    "ronqueados": "ronquear",
    "ronqueamos": "ronquear",
    "ronquean": "ronquear",
    "ronqueando": "ronquear",
    "ronqueara": "ronquear",
    "ronquearais": "ronquear",
    "ronquearan": "ronquear",
    "ronquearas": "ronquear",
    "ronqueare": "ronquear",
    "ronqueareis": "ronquear",
    "ronquearemos": "ronquear",
    "ronquearen": "ronquear",
    "ronqueares": "ronquear",
    "ronquearon": "ronquear",
    "ronqueará": "ronquear",
    "ronquearán": "ronquear",
    "ronquearás": "ronquear",
    "ronquearé": "ronquear",
    "ronquearéis": "ronquear",
    "ronquearía": "ronquear",
    "ronquearíais": "ronquear",
    "ronquearíamos": "ronquear",
    "ronquearían": "ronquear",
    "ronquearías": "ronquear",
    "ronqueas": "ronquear",
    "ronquease": "ronquear",
    "ronqueaseis": "ronquear",
    "ronqueasen": "ronquear",
    "ronqueases": "ronquear",
    "ronqueaste": "ronquear",
    "ronqueasteis": "ronquear",
    "ronquedades": "ronquedad",
    "ronquee": "ronquear",
    "ronqueemos": "ronquear",
    "ronqueen": "ronquear",
    "ronquees": "ronquear",
    "ronquemos": "roncar",
    "ronquen": "roncar",
    "ronqueo": "ronquear",
    "ronqueras": "ronquera",
    "ronques": "roncar",
    "ronqueábamos": "ronquear",
    "ronqueáis": "ronquear",
    "ronqueáramos": "ronquear",
    "ronqueáremos": "ronquear",
    "ronqueásemos": "ronquear",
    "ronqueé": "ronquear",
    "ronqueéis": "ronquear",
    "ronqueó": "ronquear",
    "ronquidos": "ronquido",
    "ronqué": "roncar",
    "ronquéis": "roncar",
    "ronronea": "ronronear",
    "ronroneaba": "ronronear",
    "ronroneabais": "ronronear",
    "ronroneaban": "ronronear",
    "ronroneabas": "ronronear",
    "ronronead": "ronronear",
    "ronroneada": "ronronear",
    "ronroneadas": "ronronear",
    "ronroneado": "ronronear",
    "ronroneados": "ronronear",
    "ronroneamos": "ronronear",
    "ronronean": "ronronear",
    "ronroneando": "ronronear",
    "ronroneara": "ronronear",
    "ronronearais": "ronronear",
    "ronronearan": "ronronear",
    "ronronearas": "ronronear",
    "ronroneare": "ronronear",
    "ronroneareis": "ronronear",
    "ronronearemos": "ronronear",
    "ronronearen": "ronronear",
    "ronroneares": "ronronear",
    "ronronearon": "ronronear",
    "ronroneará": "ronronear",
    "ronronearán": "ronronear",
    "ronronearás": "ronronear",
    "ronronearé": "ronronear",
    "ronronearéis": "ronronear",
    "ronronearía": "ronronear",
    "ronronearíais": "ronronear",
    "ronronearíamos": "ronronear",
    "ronronearían": "ronronear",
    "ronronearías": "ronronear",
    "ronroneas": "ronronear",
    "ronronease": "ronronear",
    "ronroneaseis": "ronronear",
    "ronroneasen": "ronronear",
    "ronroneases": "ronronear",
    "ronroneaste": "ronronear",
    "ronroneasteis": "ronronear",
    "ronronee": "ronronear",
    "ronroneemos": "ronronear",
    "ronroneen": "ronronear",
    "ronronees": "ronronear",
    "ronroneo": "ronronear",
    "ronroneos": "ronroneo",
    "ronroneábamos": "ronronear",
    "ronroneáis": "ronronear",
    "ronroneáramos": "ronronear",
    "ronroneáremos": "ronronear",
    "ronroneásemos": "ronronear",
    "ronroneé": "ronronear",
    "ronroneéis": "ronronear",
    "ronroneó": "ronronear",
    "ronza": "ronzar",
    "ronzaba": "ronzar",
    "ronzabais": "ronzar",
    "ronzaban": "ronzar",
    "ronzabas": "ronzar",
    "ronzad": "ronzar",
    "ronzada": "ronzar",
    "ronzadas": "ronzar",
    "ronzado": "ronzar",
    "ronzados": "ronzar",
    "ronzales": "ronzal",
    "ronzamos": "ronzar",
    "ronzan": "ronzar",
    "ronzando": "ronzar",
    "ronzara": "ronzar",
    "ronzarais": "ronzar",
    "ronzaran": "ronzar",
    "ronzaras": "ronzar",
    "ronzare": "ronzar",
    "ronzareis": "ronzar",
    "ronzaremos": "ronzar",
    "ronzaren": "ronzar",
    "ronzares": "ronzar",
    "ronzaron": "ronzar",
    "ronzará": "ronzar",
    "ronzarán": "ronzar",
    "ronzarás": "ronzar",
    "ronzaré": "ronzar",
    "ronzaréis": "ronzar",
    "ronzaría": "ronzar",
    "ronzaríais": "ronzar",
    "ronzaríamos": "ronzar",
    "ronzarían": "ronzar",
    "ronzarías": "ronzar",
    "ronzas": "ronzar",
    "ronzase": "ronzar",
    "ronzaseis": "ronzar",
    "ronzasen": "ronzar",
    "ronzases": "ronzar",
    "ronzaste": "ronzar",
    "ronzasteis": "ronzar",
    "ronzo": "ronzar",
    "ronzábamos": "ronzar",
    "ronzáis": "ronzar",
    "ronzáramos": "ronzar",
    "ronzáremos": "ronzar",
    "ronzásemos": "ronzar",
    "ronzó": "ronzar",
    "roo": "roer",
    "ropajes": "ropaje",
    "ropas": "ropa",
    "ropavejera": "ropavejero",
    "ropavejeras": "ropavejero",
    "ropavejeros": "ropavejero",
    "ropavejerías": "ropavejería",
    "roperos": "ropero",
    "roperías": "ropería",
    "ropetas": "ropeta",
    "ropillas": "ropilla",
    "ropones": "ropón",
    "roquedales": "roquedal",
    "roquedas": "roqueda",
    "roquedos": "roquedo",
    "roqueforts": "roquefort",
    "roquera": "roquero",
    "roqueras": "roquero",
    "roqueros": "roquero",
    "roques": "roque",
    "roquetes": "roquete",
    "roqueña": "roqueño",
    "roqueñas": "roqueño",
    "roqueños": "roqueño",
    "rorcuales": "rorcual",
    "rorros": "rorro",
    "ros": "ro",
    "rosada": "rosado",
    "rosadas": "rosado",
    "rosados": "rosado",
    "rosaledas": "rosaleda",
    "rosales": "rosal",
    "rosanilinas": "rosanilina",
    "rosariera": "rosariero",
    "rosarieras": "rosariero",
    "rosarieros": "rosariero",
    "rosarina": "rosarino",
    "rosarinas": "rosarino",
    "rosarinos": "rosarino",
    "rosarios": "rosario",
    "rosas": "rosa",
    "rosbifs": "rosbif",
    "rosca": "roscar",
    "roscaba": "roscar",
    "roscabais": "roscar",
    "roscaban": "roscar",
    "roscabas": "roscar",
    "roscad": "roscar",
    "roscada": "roscar",
    "roscadas": "roscar",
    "roscado": "roscar",
    "roscadoras": "roscadora",
    "roscados": "roscar",
    "roscamos": "roscar",
    "roscan": "roscar",
    "roscando": "roscar",
    "roscara": "roscar",
    "roscarais": "roscar",
    "roscaran": "roscar",
    "roscaras": "roscar",
    "roscare": "roscar",
    "roscareis": "roscar",
    "roscaremos": "roscar",
    "roscaren": "roscar",
    "roscares": "roscar",
    "roscaron": "roscar",
    "roscará": "roscar",
    "roscarán": "roscar",
    "roscarás": "roscar",
    "roscaré": "roscar",
    "roscaréis": "roscar",
    "roscaría": "roscar",
    "roscaríais": "roscar",
    "roscaríamos": "roscar",
    "roscarían": "roscar",
    "roscarías": "roscar",
    "roscas": "roscar",
    "roscase": "roscar",
    "roscaseis": "roscar",
    "roscasen": "roscar",
    "roscases": "roscar",
    "roscaste": "roscar",
    "roscasteis": "roscar",
    "rosco": "roscar",
    "roscoelitas": "roscoelita",
    "roscones": "roscón",
    "roscos": "rosco",
    "roscábamos": "roscar",
    "roscáis": "roscar",
    "roscáramos": "roscar",
    "roscáremos": "roscar",
    "roscásemos": "roscar",
    "roscó": "roscar",
    "rosellonesa": "rosellonés",
    "rosellonesas": "rosellonés",
    "roselloneses": "rosellonés",
    "rosera": "rosero",
    "roseras": "rosero",
    "roseros": "rosero",
    "roses": "ros",
    "rosetas": "roseta",
    "rosetones": "rosetón",
    "rosicleres": "rosicler",
    "rosilla": "rosillo",
    "rosillas": "rosillo",
    "rosillos": "rosillo",
    "rositas": "rosita",
    "rosmarina": "rosmarino",
    "rosmarinas": "rosmarino",
    "rosmarinos": "rosmarino",
    "rosolies": "rosoli",
    "rosolis": "rosoli",
    "rosolíes": "rosolí",
    "rosolís": "rosolí",
    "rosones": "rosón",
    "rosos": "roso",
    "rosque": "roscar",
    "rosqueada": "rosqueado",
    "rosqueadas": "rosqueado",
    "rosqueados": "rosqueado",
    "rosquemos": "roscar",
    "rosquen": "roscar",
    "rosques": "roscar",
    "rosquetes": "rosquete",
    "rosquillas": "rosquilla",
    "rosquilleras": "rosquillera",
    "rosqué": "roscar",
    "rosquéis": "roscar",
    "rosticerías": "rosticería",
    "rostrada": "rostrado",
    "rostradas": "rostrado",
    "rostrados": "rostrado",
    "rostrales": "rostral",
    "rostritorcida": "rostritorcido",
    "rostritorcidas": "rostritorcido",
    "rostritorcidos": "rostritorcido",
    "rostrituerta": "rostrituerto",
    "rostrituertas": "rostrituerto",
    "rostrituertos": "rostrituerto",
    "rostros": "rostro",
    "rosácea": "rosáceo",
    "rosáceas": "rosáceo",
    "rosáceos": "rosáceo",
    "roséolas": "roséola",
    "rota": "rotar",
    "rotaba": "rotar",
    "rotabais": "rotar",
    "rotaban": "rotar",
    "rotabas": "rotar",
    "rotabilidades": "rotabilidad",
    "rotacionales": "rotacional",
    "rotaciones": "rotación",
    "rotacismos": "rotacismo",
    "rotad": "rotar",
    "rotada": "rotar",
    "rotadas": "rotar",
    "rotado": "rotar",
    "rotadora": "rotador",
    "rotadoras": "rotador",
    "rotadores": "rotador",
    "rotados": "rotar",
    "rotafrotaciones": "rotafrotación",
    "rotamos": "rotar",
    "rotan": "rotar",
    "rotando": "rotar",
    "rotantes": "rotante",
    "rotara": "rotar",
    "rotarais": "rotar",
    "rotaran": "rotar",
    "rotaras": "rotar",
    "rotare": "rotar",
    "rotareis": "rotar",
    "rotaremos": "rotar",
    "rotaren": "rotar",
    "rotares": "rotar",
    "rotaria": "rotario",
    "rotarias": "rotario",
    "rotarios": "rotario",
    "rotaron": "rotar",
    "rotará": "rotar",
    "rotarán": "rotar",
    "rotarás": "rotar",
    "rotaré": "rotar",
    "rotaréis": "rotar",
    "rotaría": "rotar",
    "rotaríais": "rotar",
    "rotaríamos": "rotar",
    "rotarían": "rotar",
    "rotarías": "rotar",
    "rotas": "rotar",
    "rotase": "rotar",
    "rotaseis": "rotar",
    "rotasen": "rotar",
    "rotases": "rotar",
    "rotaste": "rotar",
    "rotasteis": "rotar",
    "rotativa": "rotativo",
    "rotativas": "rotativo",
    "rotativos": "rotativo",
    "rotatoria": "rotatorio",
    "rotatorias": "rotatorio",
    "rotatorios": "rotatorio",
    "rotaturas": "rotatura",
    "rote": "rotar",
    "rotemos": "rotar",
    "roten": "rotar",
    "rotenonas": "rotenona",
    "rotens": "roten",
    "roterdes": "roterde",
    "roterodamenses": "roterodamense",
    "rotes": "rotar",
    "rotlehmes": "rotlehm",
    "roto": "rotar",
    "rotoarados": "rotoarado",
    "rotocultivadora": "rotocultivador",
    "rotocultivadoras": "rotocultivador",
    "rotocultivadores": "rotocultivador",
    "rotocultores": "rotocultor",
    "rotoempacadoras": "rotoempacadora",
    "rotograbada": "rotograbado",
    "rotograbadas": "rotograbado",
    "rotograbados": "rotograbado",
    "rotoinversiones": "rotoinversión",
    "rotondas": "rotonda",
    "rotores": "rotor",
    "rotos": "roto",
    "rotula": "rotular",
    "rotulaba": "rotular",
    "rotulabais": "rotular",
    "rotulaban": "rotular",
    "rotulabas": "rotular",
    "rotulaciones": "rotulación",
    "rotulad": "rotular",
    "rotulada": "rotular",
    "rotuladas": "rotular",
    "rotulado": "rotular",
    "rotuladora": "rotulador",
    "rotuladoras": "rotulador",
    "rotuladores": "rotulador",
    "rotulados": "rotular",
    "rotulamos": "rotular",
    "rotulan": "rotular",
    "rotulando": "rotular",
    "rotulara": "rotular",
    "rotularais": "rotular",
    "rotularan": "rotular",
    "rotularas": "rotular",
    "rotulare": "rotular",
    "rotulareis": "rotular",
    "rotularemos": "rotular",
    "rotularen": "rotular",
    "rotulares": "rotular",
    "rotularon": "rotular",
    "rotulará": "rotular",
    "rotularán": "rotular",
    "rotularás": "rotular",
    "rotularé": "rotular",
    "rotularéis": "rotular",
    "rotularía": "rotular",
    "rotularíais": "rotular",
    "rotularíamos": "rotular",
    "rotularían": "rotular",
    "rotularías": "rotular",
    "rotulas": "rotular",
    "rotulase": "rotular",
    "rotulaseis": "rotular",
    "rotulasen": "rotular",
    "rotulases": "rotular",
    "rotulaste": "rotular",
    "rotulasteis": "rotular",
    "rotulatas": "rotulata",
    "rotule": "rotular",
    "rotulemos": "rotular",
    "rotulen": "rotular",
    "rotules": "rotular",
    "rotuliana": "rotuliano",
    "rotulianas": "rotuliano",
    "rotulianos": "rotuliano",
    "rotulistas": "rotulista",
    "rotulo": "rotular",
    "rotulábamos": "rotular",
    "rotuláis": "rotular",
    "rotuláramos": "rotular",
    "rotuláremos": "rotular",
    "rotulásemos": "rotular",
    "rotulé": "rotular",
    "rotuléis": "rotular",
    "rotuló": "rotular",
    "rotunda": "rotundo",
    "rotundas": "rotundo",
    "rotundeces": "rotundez",
    "rotundidades": "rotundidad",
    "rotundideces": "rotundidez",
    "rotundos": "rotundo",
    "rotura": "roturar",
    "roturaba": "roturar",
    "roturabais": "roturar",
    "roturaban": "roturar",
    "roturabas": "roturar",
    "roturaciones": "roturación",
    "roturad": "roturar",
    "roturada": "roturar",
    "roturadas": "roturar",
    "roturado": "roturar",
    "roturadora": "roturador",
    "roturadoras": "roturadora",
    "roturadores": "roturador",
    "roturados": "roturar",
    "roturamos": "roturar",
    "roturan": "roturar",
    "roturando": "roturar",
    "roturara": "roturar",
    "roturarais": "roturar",
    "roturaran": "roturar",
    "roturaras": "roturar",
    "roturare": "roturar",
    "roturareis": "roturar",
    "roturaremos": "roturar",
    "roturaren": "roturar",
    "roturares": "roturar",
    "roturaron": "roturar",
    "roturará": "roturar",
    "roturarán": "roturar",
    "roturarás": "roturar",
    "roturaré": "roturar",
    "roturaréis": "roturar",
    "roturaría": "roturar",
    "roturaríais": "roturar",
    "roturaríamos": "roturar",
    "roturarían": "roturar",
    "roturarías": "roturar",
    "roturas": "roturar",
    "roturase": "roturar",
    "roturaseis": "roturar",
    "roturasen": "roturar",
    "roturases": "roturar",
    "roturaste": "roturar",
    "roturasteis": "roturar",
    "roture": "roturar",
    "roturemos": "roturar",
    "roturen": "roturar",
    "rotures": "roturar",
    "roturo": "roturar",
    "roturábamos": "roturar",
    "roturáis": "roturar",
    "roturáramos": "roturar",
    "roturáremos": "roturar",
    "roturásemos": "roturar",
    "roturé": "roturar",
    "roturéis": "roturar",
    "roturó": "roturar",
    "rotábamos": "rotar",
    "rotáis": "rotar",
    "rotámetros": "rotámetro",
    "rotáramos": "rotar",
    "rotáremos": "rotar",
    "rotásemos": "rotar",
    "roté": "rotar",
    "rotéis": "rotar",
    "rotíferos": "rotífero",
    "rotó": "rotar",
    "rotórica": "rotórico",
    "rotóricas": "rotórico",
    "rotóricos": "rotórico",
    "roulottes": "roulotte",
    "rounds": "round",
    "rovellones": "rovellón",
    "roya": "royo",
    "royalties": "royalty",
    "royamos": "roer",
    "royan": "roer",
    "royas": "royo",
    "royendo": "roer",
    "royera": "roer",
    "royerais": "roer",
    "royeran": "roer",
    "royeras": "roer",
    "royere": "roer",
    "royereis": "roer",
    "royeren": "roer",
    "royeres": "roer",
    "royeron": "roer",
    "royese": "roer",
    "royeseis": "roer",
    "royesen": "roer",
    "royeses": "roer",
    "royo": "roer",
    "royos": "royo",
    "royáis": "roer",
    "royéramos": "roer",
    "royéremos": "roer",
    "royésemos": "roer",
    "royó": "roer",
    "roza": "rozar",
    "rozaba": "rozar",
    "rozabais": "rozar",
    "rozaban": "rozar",
    "rozabas": "rozar",
    "rozables": "rozable",
    "rozad": "rozar",
    "rozada": "rozar",
    "rozadas": "rozar",
    "rozaderas": "rozadera",
    "rozado": "rozar",
    "rozadora": "rozador",
    "rozadoras": "rozador",
    "rozadores": "rozador",
    "rozados": "rozar",
    "rozaduras": "rozadura",
    "rozagantes": "rozagante",
    "rozamientos": "rozamiento",
    "rozamos": "rozar",
    "rozan": "rozar",
    "rozando": "rozar",
    "rozara": "rozar",
    "rozarais": "rozar",
    "rozaran": "rozar",
    "rozaras": "rozar",
    "rozare": "rozar",
    "rozareis": "rozar",
    "rozaremos": "rozar",
    "rozaren": "rozar",
    "rozares": "rozar",
    "rozaron": "rozar",
    "rozará": "rozar",
    "rozarán": "rozar",
    "rozarás": "rozar",
    "rozaré": "rozar",
    "rozaréis": "rozar",
    "rozaría": "rozar",
    "rozaríais": "rozar",
    "rozaríamos": "rozar",
    "rozarían": "rozar",
    "rozarías": "rozar",
    "rozas": "rozar",
    "rozase": "rozar",
    "rozaseis": "rozar",
    "rozasen": "rozar",
    "rozases": "rozar",
    "rozaste": "rozar",
    "rozasteis": "rozar",
    "roznidos": "roznido",
    "roznos": "rozno",
    "rozo": "rozar",
    "rozones": "rozón",
    "rozos": "rozo",
    "rozábamos": "rozar",
    "rozáis": "rozar",
    "rozáramos": "rozar",
    "rozáremos": "rozar",
    "rozásemos": "rozar",
    "rozó": "rozar",
    "rozónes": "rozón",
    "roáis": "roer",
    "roéis": "roer",
    "roí": "roer",
    "roía": "roer",
    "roíais": "roer",
    "roíamos": "roer",
    "roían": "roer",
    "roías": "roer",
    "roída": "roer",
    "roídas": "roer",
    "roído": "roer",
    "roídos": "roer",
    "roímos": "roer",
    "roíste": "roer",
    "roísteis": "roer",
    "roñas": "roña",
    "roñerías": "roñería",
    "roñicas": "roñica",
    "roñosa": "roñoso",
    "roñosas": "roñoso",
    "roñoserías": "roñosería",
    "roñosos": "roñoso",
    "rpm": "r.p.m.",
    "ruana": "ruano",
    "ruanas": "ruano",
    "ruandas": "ruanda",
    "ruandesa": "rwandés",
    "ruandesas": "rwandés",
    "ruandeses": "rwandés",
    "ruandés": "rwandés",
    "ruanos": "ruano",
    "ruantes": "ruante",
    "rubefacciones": "rubefacción",
    "rubefacientes": "rubefaciente",
    "rubelitas": "rubelita",
    "rubeolas": "rubeola",
    "rubescentes": "rubescente",
    "rubeánica": "rubeánico",
    "rubeánicas": "rubeánico",
    "rubeánicos": "rubeánico",
    "rubia": "rubio",
    "rubiales": "rubial",
    "rubias": "rubio",
    "rubicana": "rubicán",
    "rubicanas": "rubicán",
    "rubicanes": "rubicán",
    "rubicelas": "rubicela",
    "rubicunda": "rubicundo",
    "rubicundas": "rubicundo",
    "rubicundeces": "rubicundez",
    "rubicundos": "rubicundo",
    "rubidios": "rubidio",
    "rubifica": "rubificar",
    "rubificaba": "rubificar",
    "rubificabais": "rubificar",
    "rubificaban": "rubificar",
    "rubificabas": "rubificar",
    "rubificad": "rubificar",
    "rubificada": "rubificar",
    "rubificadas": "rubificar",
    "rubificado": "rubificar",
    "rubificados": "rubificar",
    "rubificamos": "rubificar",
    "rubifican": "rubificar",
    "rubificando": "rubificar",
    "rubificara": "rubificar",
    "rubificarais": "rubificar",
    "rubificaran": "rubificar",
    "rubificaras": "rubificar",
    "rubificare": "rubificar",
    "rubificareis": "rubificar",
    "rubificaremos": "rubificar",
    "rubificaren": "rubificar",
    "rubificares": "rubificar",
    "rubificaron": "rubificar",
    "rubificará": "rubificar",
    "rubificarán": "rubificar",
    "rubificarás": "rubificar",
    "rubificaré": "rubificar",
    "rubificaréis": "rubificar",
    "rubificaría": "rubificar",
    "rubificaríais": "rubificar",
    "rubificaríamos": "rubificar",
    "rubificarían": "rubificar",
    "rubificarías": "rubificar",
    "rubificas": "rubificar",
    "rubificase": "rubificar",
    "rubificaseis": "rubificar",
    "rubificasen": "rubificar",
    "rubificases": "rubificar",
    "rubificaste": "rubificar",
    "rubificasteis": "rubificar",
    "rubifico": "rubificar",
    "rubificábamos": "rubificar",
    "rubificáis": "rubificar",
    "rubificáramos": "rubificar",
    "rubificáremos": "rubificar",
    "rubificásemos": "rubificar",
    "rubificó": "rubificar",
    "rubifique": "rubificar",
    "rubifiquemos": "rubificar",
    "rubifiquen": "rubificar",
    "rubifiques": "rubificar",
    "rubifiqué": "rubificar",
    "rubifiquéis": "rubificar",
    "rubines": "rubín",
    "rubiona": "rubión",
    "rubionas": "rubión",
    "rubiones": "rubión",
    "rubios": "rubio",
    "rubiácea": "rubiáceo",
    "rubiáceas": "rubiáceo",
    "rubiáceos": "rubiáceo",
    "rublos": "rublo",
    "rubores": "rubor",
    "ruborice": "ruborizar",
    "ruboricemos": "ruborizar",
    "ruboricen": "ruborizar",
    "ruborices": "ruborizar",
    "ruboricé": "ruborizar",
    "ruboricéis": "ruborizar",
    "ruboriza": "ruborizar",
    "ruborizaba": "ruborizar",
    "ruborizabais": "ruborizar",
    "ruborizaban": "ruborizar",
    "ruborizabas": "ruborizar",
    "ruborizad": "ruborizar",
    "ruborizada": "ruborizar",
    "ruborizadas": "ruborizar",
    "ruborizado": "ruborizar",
    "ruborizados": "ruborizar",
    "ruborizamos": "ruborizar",
    "ruborizan": "ruborizar",
    "ruborizando": "ruborizar",
    "ruborizara": "ruborizar",
    "ruborizarais": "ruborizar",
    "ruborizaran": "ruborizar",
    "ruborizaras": "ruborizar",
    "ruborizare": "ruborizar",
    "ruborizareis": "ruborizar",
    "ruborizaremos": "ruborizar",
    "ruborizaren": "ruborizar",
    "ruborizares": "ruborizar",
    "ruborizaron": "ruborizar",
    "ruborizará": "ruborizar",
    "ruborizarán": "ruborizar",
    "ruborizarás": "ruborizar",
    "ruborizaré": "ruborizar",
    "ruborizaréis": "ruborizar",
    "ruborizaría": "ruborizar",
    "ruborizaríais": "ruborizar",
    "ruborizaríamos": "ruborizar",
    "ruborizarían": "ruborizar",
    "ruborizarías": "ruborizar",
    "ruborizas": "ruborizar",
    "ruborizase": "ruborizar",
    "ruborizaseis": "ruborizar",
    "ruborizasen": "ruborizar",
    "ruborizases": "ruborizar",
    "ruborizaste": "ruborizar",
    "ruborizasteis": "ruborizar",
    "ruborizo": "ruborizar",
    "ruborizábamos": "ruborizar",
    "ruborizáis": "ruborizar",
    "ruborizáramos": "ruborizar",
    "ruborizáremos": "ruborizar",
    "ruborizásemos": "ruborizar",
    "ruborizó": "ruborizar",
    "ruborosa": "ruboroso",
    "ruborosas": "ruboroso",
    "ruborosos": "ruboroso",
    "rubra": "rubro",
    "rubras": "rubro",
    "rubrica": "rubricar",
    "rubricaba": "rubricar",
    "rubricabais": "rubricar",
    "rubricaban": "rubricar",
    "rubricabas": "rubricar",
    "rubricad": "rubricar",
    "rubricada": "rubricar",
    "rubricadas": "rubricar",
    "rubricado": "rubricar",
    "rubricados": "rubricar",
    "rubricamos": "rubricar",
    "rubrican": "rubricar",
    "rubricando": "rubricar",
    "rubricantes": "rubricante",
    "rubricara": "rubricar",
    "rubricarais": "rubricar",
    "rubricaran": "rubricar",
    "rubricaras": "rubricar",
    "rubricare": "rubricar",
    "rubricareis": "rubricar",
    "rubricaremos": "rubricar",
    "rubricaren": "rubricar",
    "rubricares": "rubricar",
    "rubricaron": "rubricar",
    "rubricará": "rubricar",
    "rubricarán": "rubricar",
    "rubricarás": "rubricar",
    "rubricaré": "rubricar",
    "rubricaréis": "rubricar",
    "rubricaría": "rubricar",
    "rubricaríais": "rubricar",
    "rubricaríamos": "rubricar",
    "rubricarían": "rubricar",
    "rubricarías": "rubricar",
    "rubricas": "rubricar",
    "rubricase": "rubricar",
    "rubricaseis": "rubricar",
    "rubricasen": "rubricar",
    "rubricases": "rubricar",
    "rubricaste": "rubricar",
    "rubricasteis": "rubricar",
    "rubricistas": "rubricista",
    "rubrico": "rubricar",
    "rubricábamos": "rubricar",
    "rubricáis": "rubricar",
    "rubricáramos": "rubricar",
    "rubricáremos": "rubricar",
    "rubricásemos": "rubricar",
    "rubricó": "rubricar",
    "rubrique": "rubricar",
    "rubriquemos": "rubricar",
    "rubriquen": "rubricar",
    "rubriques": "rubricar",
    "rubriquistas": "rubriquista",
    "rubriqué": "rubricar",
    "rubriquéis": "rubricar",
    "rubros": "rubro",
    "rubíes": "rubí",
    "rubís": "rubí",
    "rubíses": "rubí",
    "ruches": "ruche",
    "rucia": "rucio",
    "rucias": "rucio",
    "rucios": "rucio",
    "ruda": "rudo",
    "rudas": "rudo",
    "ruderales": "ruderal",
    "rudezas": "rudeza",
    "rudimentales": "rudimental",
    "rudimentaria": "rudimentario",
    "rudimentarias": "rudimentario",
    "rudimentarios": "rudimentario",
    "rudimentos": "rudimento",
    "rudos": "rudo",
    "ruecas": "rueca",
    "rueda": "rodar",
    "ruedan": "rodar",
    "ruedas": "rueda",
    "ruede": "rodar",
    "ruedecillas": "ruedecilla",
    "rueden": "rodar",
    "ruedera": "ruedero",
    "ruederas": "ruedero",
    "ruederos": "ruedero",
    "ruedes": "rodar",
    "ruedo": "rodar",
    "ruedos": "ruedo",
    "ruega": "rogar",
    "ruegan": "rogar",
    "ruegas": "rogar",
    "ruego": "rogar",
    "ruegos": "ruego",
    "ruegue": "rogar",
    "rueguen": "rogar",
    "ruegues": "rogar",
    "ruejos": "ruejo",
    "rueznos": "ruezno",
    "rufa": "rufo",
    "rufas": "rufo",
    "rufiana": "rufián",
    "rufianas": "rufián",
    "rufianea": "rufianear",
    "rufianeaba": "rufianear",
    "rufianeabais": "rufianear",
    "rufianeaban": "rufianear",
    "rufianeabas": "rufianear",
    "rufianead": "rufianear",
    "rufianeada": "rufianear",
    "rufianeadas": "rufianear",
    "rufianeado": "rufianear",
    "rufianeados": "rufianear",
    "rufianeamos": "rufianear",
    "rufianean": "rufianear",
    "rufianeando": "rufianear",
    "rufianeara": "rufianear",
    "rufianearais": "rufianear",
    "rufianearan": "rufianear",
    "rufianearas": "rufianear",
    "rufianeare": "rufianear",
    "rufianeareis": "rufianear",
    "rufianearemos": "rufianear",
    "rufianearen": "rufianear",
    "rufianeares": "rufianear",
    "rufianearon": "rufianear",
    "rufianeará": "rufianear",
    "rufianearán": "rufianear",
    "rufianearás": "rufianear",
    "rufianearé": "rufianear",
    "rufianearéis": "rufianear",
    "rufianearía": "rufianear",
    "rufianearíais": "rufianear",
    "rufianearíamos": "rufianear",
    "rufianearían": "rufianear",
    "rufianearías": "rufianear",
    "rufianeas": "rufianear",
    "rufianease": "rufianear",
    "rufianeaseis": "rufianear",
    "rufianeasen": "rufianear",
    "rufianeases": "rufianear",
    "rufianeaste": "rufianear",
    "rufianeasteis": "rufianear",
    "rufianee": "rufianear",
    "rufianeemos": "rufianear",
    "rufianeen": "rufianear",
    "rufianees": "rufianear",
    "rufianeo": "rufianear",
    "rufianerías": "rufianería",
    "rufianes": "rufián",
    "rufianesca": "rufianesco",
    "rufianescas": "rufianesco",
    "rufianescos": "rufianesco",
    "rufianeábamos": "rufianear",
    "rufianeáis": "rufianear",
    "rufianeáramos": "rufianear",
    "rufianeáremos": "rufianear",
    "rufianeásemos": "rufianear",
    "rufianeé": "rufianear",
    "rufianeéis": "rufianear",
    "rufianeó": "rufianear",
    "rufianismos": "rufianismo",
    "rufos": "rufo",
    "rugas": "ruga",
    "ruge": "rugir",
    "rugen": "rugir",
    "ruges": "rugir",
    "rugid": "rugir",
    "rugida": "rugir",
    "rugidas": "rugir",
    "rugido": "rugir",
    "rugidora": "rugidor",
    "rugidoras": "rugidor",
    "rugidores": "rugidor",
    "rugidos": "rugir",
    "rugiendo": "rugir",
    "rugientes": "rugiente",
    "rugiera": "rugir",
    "rugierais": "rugir",
    "rugieran": "rugir",
    "rugieras": "rugir",
    "rugiere": "rugir",
    "rugiereis": "rugir",
    "rugieren": "rugir",
    "rugieres": "rugir",
    "rugieron": "rugir",
    "rugiese": "rugir",
    "rugieseis": "rugir",
    "rugiesen": "rugir",
    "rugieses": "rugir",
    "rugimos": "rugir",
    "ruginosa": "ruginoso",
    "ruginosas": "ruginoso",
    "ruginosos": "ruginoso",
    "rugiremos": "rugir",
    "rugirá": "rugir",
    "rugirán": "rugir",
    "rugirás": "rugir",
    "rugiré": "rugir",
    "rugiréis": "rugir",
    "rugiría": "rugir",
    "rugiríais": "rugir",
    "rugiríamos": "rugir",
    "rugirían": "rugir",
    "rugirías": "rugir",
    "rugiste": "rugir",
    "rugisteis": "rugir",
    "rugiéramos": "rugir",
    "rugiéremos": "rugir",
    "rugiésemos": "rugir",
    "rugió": "rugir",
    "rugosa": "rugoso",
    "rugosas": "rugoso",
    "rugosidades": "rugosidad",
    "rugosos": "rugoso",
    "rugosímetros": "rugosímetro",
    "rugí": "rugir",
    "rugía": "rugir",
    "rugíais": "rugir",
    "rugíamos": "rugir",
    "rugían": "rugir",
    "rugías": "rugir",
    "rugís": "rugir",
    "rugómetros": "rugómetro",
    "ruibarbos": "ruibarbo",
    "ruidos": "ruido",
    "ruidosa": "ruidoso",
    "ruidosas": "ruidoso",
    "ruidosos": "ruidoso",
    "ruinas": "ruina",
    "ruindades": "ruindad",
    "ruines": "ruin",
    "ruinosa": "ruinoso",
    "ruinosas": "ruinoso",
    "ruinosos": "ruinoso",
    "ruipónticos": "ruipóntico",
    "ruiseñores": "ruiseñor",
    "ruja": "rugir",
    "rujamos": "rugir",
    "rujan": "rugir",
    "rujas": "rugir",
    "rujo": "rugir",
    "rujáis": "rugir",
    "ruletas": "ruleta",
    "ruletera": "ruletero",
    "ruleteras": "ruletero",
    "ruleteros": "ruletero",
    "rulos": "rulo",
    "rulotes": "rulot",
    "rulés": "rulé",
    "rumana": "rumano",
    "rumanas": "rumano",
    "rumanitas": "rumanita",
    "rumanos": "rumano",
    "rumbas": "rumba",
    "rumbea": "rumbear",
    "rumbeaba": "rumbear",
    "rumbeabais": "rumbear",
    "rumbeaban": "rumbear",
    "rumbeabas": "rumbear",
    "rumbead": "rumbear",
    "rumbeada": "rumbear",
    "rumbeadas": "rumbear",
    "rumbeado": "rumbear",
    "rumbeados": "rumbear",
    "rumbeamos": "rumbear",
    "rumbean": "rumbear",
    "rumbeando": "rumbear",
    "rumbeara": "rumbear",
    "rumbearais": "rumbear",
    "rumbearan": "rumbear",
    "rumbearas": "rumbear",
    "rumbeare": "rumbear",
    "rumbeareis": "rumbear",
    "rumbearemos": "rumbear",
    "rumbearen": "rumbear",
    "rumbeares": "rumbear",
    "rumbearon": "rumbear",
    "rumbeará": "rumbear",
    "rumbearán": "rumbear",
    "rumbearás": "rumbear",
    "rumbearé": "rumbear",
    "rumbearéis": "rumbear",
    "rumbearía": "rumbear",
    "rumbearíais": "rumbear",
    "rumbearíamos": "rumbear",
    "rumbearían": "rumbear",
    "rumbearías": "rumbear",
    "rumbeas": "rumbear",
    "rumbease": "rumbear",
    "rumbeaseis": "rumbear",
    "rumbeasen": "rumbear",
    "rumbeases": "rumbear",
    "rumbeaste": "rumbear",
    "rumbeasteis": "rumbear",
    "rumbee": "rumbear",
    "rumbeemos": "rumbear",
    "rumbeen": "rumbear",
    "rumbees": "rumbear",
    "rumbeo": "rumbear",
    "rumbera": "rumbero",
    "rumberas": "rumbero",
    "rumberos": "rumbero",
    "rumbeábamos": "rumbear",
    "rumbeáis": "rumbear",
    "rumbeáramos": "rumbear",
    "rumbeáremos": "rumbear",
    "rumbeásemos": "rumbear",
    "rumbeé": "rumbear",
    "rumbeéis": "rumbear",
    "rumbeó": "rumbear",
    "rumbona": "rumbón",
    "rumbonas": "rumbón",
    "rumbones": "rumbón",
    "rumbos": "rumbo",
    "rumbosa": "rumboso",
    "rumbosas": "rumboso",
    "rumbosos": "rumboso",
    "rumbática": "rumbático",
    "rumbáticas": "rumbático",
    "rumbáticos": "rumbático",
    "rumens": "rumen",
    "rumia": "rumiar",
    "rumiaba": "rumiar",
    "rumiabais": "rumiar",
    "rumiaban": "rumiar",
    "rumiabas": "rumiar",
    "rumiad": "rumiar",
    "rumiada": "rumiar",
    "rumiadas": "rumiar",
    "rumiado": "rumiar",
    "rumiadora": "rumiador",
    "rumiadoras": "rumiador",
    "rumiadores": "rumiador",
    "rumiados": "rumiar",
    "rumiaduras": "rumiadura",
    "rumiajos": "rumiajo",
    "rumiamos": "rumiar",
    "rumian": "rumiar",
    "rumiando": "rumiar",
    "rumiantes": "rumiante",
    "rumiara": "rumiar",
    "rumiarais": "rumiar",
    "rumiaran": "rumiar",
    "rumiaras": "rumiar",
    "rumiare": "rumiar",
    "rumiareis": "rumiar",
    "rumiaremos": "rumiar",
    "rumiaren": "rumiar",
    "rumiares": "rumiar",
    "rumiaron": "rumiar",
    "rumiará": "rumiar",
    "rumiarán": "rumiar",
    "rumiarás": "rumiar",
    "rumiaré": "rumiar",
    "rumiaréis": "rumiar",
    "rumiaría": "rumiar",
    "rumiaríais": "rumiar",
    "rumiaríamos": "rumiar",
    "rumiarían": "rumiar",
    "rumiarías": "rumiar",
    "rumias": "rumiar",
    "rumiase": "rumiar",
    "rumiaseis": "rumiar",
    "rumiasen": "rumiar",
    "rumiases": "rumiar",
    "rumiaste": "rumiar",
    "rumiasteis": "rumiar",
    "rumie": "rumiar",
    "rumiemos": "rumiar",
    "rumien": "rumiar",
    "rumies": "rumiar",
    "rumio": "rumiar",
    "rumiona": "rumión",
    "rumionas": "rumión",
    "rumiones": "rumión",
    "rumiábamos": "rumiar",
    "rumiáis": "rumiar",
    "rumiáramos": "rumiar",
    "rumiáremos": "rumiar",
    "rumiásemos": "rumiar",
    "rumié": "rumiar",
    "rumiéis": "rumiar",
    "rumió": "rumiar",
    "rumora": "rumorar",
    "rumoraba": "rumorar",
    "rumorabais": "rumorar",
    "rumoraban": "rumorar",
    "rumorabas": "rumorar",
    "rumorad": "rumorar",
    "rumorada": "rumorar",
    "rumoradas": "rumorar",
    "rumorado": "rumorar",
    "rumorados": "rumorar",
    "rumoramos": "rumorar",
    "rumoran": "rumorar",
    "rumorando": "rumorar",
    "rumorara": "rumorar",
    "rumorarais": "rumorar",
    "rumoraran": "rumorar",
    "rumoraras": "rumorar",
    "rumorare": "rumorar",
    "rumorareis": "rumorar",
    "rumoraremos": "rumorar",
    "rumoraren": "rumorar",
    "rumorares": "rumorar",
    "rumoraron": "rumorar",
    "rumorará": "rumorar",
    "rumorarán": "rumorar",
    "rumorarás": "rumorar",
    "rumoraré": "rumorar",
    "rumoraréis": "rumorar",
    "rumoraría": "rumorar",
    "rumoraríais": "rumorar",
    "rumoraríamos": "rumorar",
    "rumorarían": "rumorar",
    "rumorarías": "rumorar",
    "rumoras": "rumorar",
    "rumorase": "rumorar",
    "rumoraseis": "rumorar",
    "rumorasen": "rumorar",
    "rumorases": "rumorar",
    "rumoraste": "rumorar",
    "rumorasteis": "rumorar",
    "rumore": "rumorar",
    "rumorea": "rumorear",
    "rumoreaba": "rumorear",
    "rumoreabais": "rumorear",
    "rumoreaban": "rumorear",
    "rumoreabas": "rumorear",
    "rumoread": "rumorear",
    "rumoreada": "rumorear",
    "rumoreadas": "rumorear",
    "rumoreado": "rumorear",
    "rumoreados": "rumorear",
    "rumoreamos": "rumorear",
    "rumorean": "rumorear",
    "rumoreando": "rumorear",
    "rumoreara": "rumorear",
    "rumorearais": "rumorear",
    "rumorearan": "rumorear",
    "rumorearas": "rumorear",
    "rumoreare": "rumorear",
    "rumoreareis": "rumorear",
    "rumorearemos": "rumorear",
    "rumorearen": "rumorear",
    "rumoreares": "rumorear",
    "rumorearon": "rumorear",
    "rumoreará": "rumorear",
    "rumorearán": "rumorear",
    "rumorearás": "rumorear",
    "rumorearé": "rumorear",
    "rumorearéis": "rumorear",
    "rumorearía": "rumorear",
    "rumorearíais": "rumorear",
    "rumorearíamos": "rumorear",
    "rumorearían": "rumorear",
    "rumorearías": "rumorear",
    "rumoreas": "rumorear",
    "rumorease": "rumorear",
    "rumoreaseis": "rumorear",
    "rumoreasen": "rumorear",
    "rumoreases": "rumorear",
    "rumoreaste": "rumorear",
    "rumoreasteis": "rumorear",
    "rumoree": "rumorear",
    "rumoreemos": "rumorear",
    "rumoreen": "rumorear",
    "rumorees": "rumorear",
    "rumoremos": "rumorar",
    "rumoren": "rumorar",
    "rumoreo": "rumorear",
    "rumoreos": "rumoreo",
    "rumores": "rumorar",
    "rumoreábamos": "rumorear",
    "rumoreáis": "rumorear",
    "rumoreáramos": "rumorear",
    "rumoreáremos": "rumorear",
    "rumoreásemos": "rumorear",
    "rumoreé": "rumorear",
    "rumoreéis": "rumorear",
    "rumoreó": "rumorear",
    "rumoro": "rumorar",
    "rumorosa": "rumoroso",
    "rumorosas": "rumoroso",
    "rumorosos": "rumoroso",
    "rumorábamos": "rumorar",
    "rumoráis": "rumorar",
    "rumoráramos": "rumorar",
    "rumoráremos": "rumorar",
    "rumorásemos": "rumorar",
    "rumoré": "rumorar",
    "rumoréis": "rumorar",
    "rumoró": "rumorar",
    "rumos": "rumo",
    "rumíes": "rumí",
    "rumís": "rumí",
    "runa": "runo",
    "runas": "runo",
    "runfladas": "runflada",
    "runflas": "runfla",
    "runos": "runo",
    "runrunea": "runrunear",
    "runruneaba": "runrunear",
    "runruneabais": "runrunear",
    "runruneaban": "runrunear",
    "runruneabas": "runrunear",
    "runrunead": "runrunear",
    "runruneada": "runrunear",
    "runruneadas": "runrunear",
    "runruneado": "runrunear",
    "runruneados": "runrunear",
    "runruneamos": "runrunear",
    "runrunean": "runrunear",
    "runruneando": "runrunear",
    "runruneara": "runrunear",
    "runrunearais": "runrunear",
    "runrunearan": "runrunear",
    "runrunearas": "runrunear",
    "runruneare": "runrunear",
    "runruneareis": "runrunear",
    "runrunearemos": "runrunear",
    "runrunearen": "runrunear",
    "runruneares": "runrunear",
    "runrunearon": "runrunear",
    "runruneará": "runrunear",
    "runrunearán": "runrunear",
    "runrunearás": "runrunear",
    "runrunearé": "runrunear",
    "runrunearéis": "runrunear",
    "runrunearía": "runrunear",
    "runrunearíais": "runrunear",
    "runrunearíamos": "runrunear",
    "runrunearían": "runrunear",
    "runrunearías": "runrunear",
    "runruneas": "runrunear",
    "runrunease": "runrunear",
    "runruneaseis": "runrunear",
    "runruneasen": "runrunear",
    "runruneases": "runrunear",
    "runruneaste": "runrunear",
    "runruneasteis": "runrunear",
    "runrunee": "runrunear",
    "runruneemos": "runrunear",
    "runruneen": "runrunear",
    "runrunees": "runrunear",
    "runruneo": "runrunear",
    "runruneos": "runruneo",
    "runrunes": "runrún",
    "runruneábamos": "runrunear",
    "runruneáis": "runrunear",
    "runruneáramos": "runrunear",
    "runruneáremos": "runrunear",
    "runruneásemos": "runrunear",
    "runruneé": "runrunear",
    "runruneéis": "runrunear",
    "runruneó": "runrunear",
    "rupestres": "rupestre",
    "rupias": "rupia",
    "ruptores": "ruptor",
    "rupturas": "ruptura",
    "rupturistas": "rupturista",
    "rupícolas": "rupícola",
    "rurales": "rural",
    "ruralismos": "ruralismo",
    "ruralizaciones": "ruralización",
    "ruralizaciónes": "ruralización",
    "rusa": "ruso",
    "rusalcas": "rusalca",
    "rusalkas": "rusalka",
    "rusas": "ruso",
    "ruscos": "rusco",
    "ruseles": "rusel",
    "rusias": "rusia",
    "rusifica": "rusificar",
    "rusificaba": "rusificar",
    "rusificabais": "rusificar",
    "rusificaban": "rusificar",
    "rusificabas": "rusificar",
    "rusificaciones": "rusificación",
    "rusificad": "rusificar",
    "rusificada": "rusificar",
    "rusificadas": "rusificar",
    "rusificado": "rusificar",
    "rusificados": "rusificar",
    "rusificamos": "rusificar",
    "rusifican": "rusificar",
    "rusificando": "rusificar",
    "rusificara": "rusificar",
    "rusificarais": "rusificar",
    "rusificaran": "rusificar",
    "rusificaras": "rusificar",
    "rusificare": "rusificar",
    "rusificareis": "rusificar",
    "rusificaremos": "rusificar",
    "rusificaren": "rusificar",
    "rusificares": "rusificar",
    "rusificaron": "rusificar",
    "rusificará": "rusificar",
    "rusificarán": "rusificar",
    "rusificarás": "rusificar",
    "rusificaré": "rusificar",
    "rusificaréis": "rusificar",
    "rusificaría": "rusificar",
    "rusificaríais": "rusificar",
    "rusificaríamos": "rusificar",
    "rusificarían": "rusificar",
    "rusificarías": "rusificar",
    "rusificas": "rusificar",
    "rusificase": "rusificar",
    "rusificaseis": "rusificar",
    "rusificasen": "rusificar",
    "rusificases": "rusificar",
    "rusificaste": "rusificar",
    "rusificasteis": "rusificar",
    "rusifico": "rusificar",
    "rusificábamos": "rusificar",
    "rusificáis": "rusificar",
    "rusificáramos": "rusificar",
    "rusificáremos": "rusificar",
    "rusificásemos": "rusificar",
    "rusificó": "rusificar",
    "rusifique": "rusificar",
    "rusifiquemos": "rusificar",
    "rusifiquen": "rusificar",
    "rusifiques": "rusificar",
    "rusifiqué": "rusificar",
    "rusifiquéis": "rusificar",
    "rusos": "ruso",
    "rusticana": "rusticano",
    "rusticanas": "rusticano",
    "rusticanos": "rusticano",
    "rusticidades": "rusticidad",
    "rustiqueces": "rustiquez",
    "rustiquezas": "rustiqueza",
    "rustros": "rustro",
    "rusófila": "rusófilo",
    "rusófilas": "rusófilo",
    "rusófilos": "rusófilo",
    "rutas": "ruta",
    "rutena": "ruteno",
    "rutenas": "ruteno",
    "rutenios": "rutenio",
    "rutenos": "ruteno",
    "rutherfordes": "rutherford",
    "rutherfords": "rutherford",
    "rutila": "rutilar",
    "rutilaba": "rutilar",
    "rutilabais": "rutilar",
    "rutilaban": "rutilar",
    "rutilabas": "rutilar",
    "rutilad": "rutilar",
    "rutilada": "rutilar",
    "rutiladas": "rutilar",
    "rutilado": "rutilar",
    "rutilados": "rutilar",
    "rutilamos": "rutilar",
    "rutilan": "rutilar",
    "rutilancias": "rutilancia",
    "rutilando": "rutilar",
    "rutilantes": "rutilante",
    "rutilara": "rutilar",
    "rutilarais": "rutilar",
    "rutilaran": "rutilar",
    "rutilaras": "rutilar",
    "rutilare": "rutilar",
    "rutilareis": "rutilar",
    "rutilaremos": "rutilar",
    "rutilaren": "rutilar",
    "rutilares": "rutilar",
    "rutilaron": "rutilar",
    "rutilará": "rutilar",
    "rutilarán": "rutilar",
    "rutilarás": "rutilar",
    "rutilaré": "rutilar",
    "rutilaréis": "rutilar",
    "rutilaría": "rutilar",
    "rutilaríais": "rutilar",
    "rutilaríamos": "rutilar",
    "rutilarían": "rutilar",
    "rutilarías": "rutilar",
    "rutilas": "rutilar",
    "rutilase": "rutilar",
    "rutilaseis": "rutilar",
    "rutilasen": "rutilar",
    "rutilases": "rutilar",
    "rutilaste": "rutilar",
    "rutilasteis": "rutilar",
    "rutile": "rutilar",
    "rutilemos": "rutilar",
    "rutilen": "rutilar",
    "rutiles": "rutilar",
    "rutilo": "rutilar",
    "rutilos": "rutilo",
    "rutilábamos": "rutilar",
    "rutiláis": "rutilar",
    "rutiláramos": "rutilar",
    "rutiláremos": "rutilar",
    "rutilásemos": "rutilar",
    "rutilé": "rutilar",
    "rutiléis": "rutilar",
    "rutiló": "rutilar",
    "rutinaria": "rutinario",
    "rutinarias": "rutinario",
    "rutinarios": "rutinario",
    "rutinas": "rutina",
    "rutinera": "rutinero",
    "rutineras": "rutinero",
    "rutineros": "rutinero",
    "rutinizaciones": "rutinización",
    "rutinizaciónes": "rutinización",
    "rutácea": "rutáceo",
    "rutáceas": "rutáceo",
    "rutáceos": "rutáceo",
    "ruña": "ruñar",
    "ruñaba": "ruñar",
    "ruñabais": "ruñar",
    "ruñaban": "ruñar",
    "ruñabas": "ruñar",
    "ruñad": "ruñar",
    "ruñada": "ruñar",
    "ruñadas": "ruñar",
    "ruñado": "ruñar",
    "ruñados": "ruñar",
    "ruñamos": "ruñar",
    "ruñan": "ruñar",
    "ruñando": "ruñar",
    "ruñara": "ruñar",
    "ruñarais": "ruñar",
    "ruñaran": "ruñar",
    "ruñaras": "ruñar",
    "ruñare": "ruñar",
    "ruñareis": "ruñar",
    "ruñaremos": "ruñar",
    "ruñaren": "ruñar",
    "ruñares": "ruñar",
    "ruñaron": "ruñar",
    "ruñará": "ruñar",
    "ruñarán": "ruñar",
    "ruñarás": "ruñar",
    "ruñaré": "ruñar",
    "ruñaréis": "ruñar",
    "ruñaría": "ruñar",
    "ruñaríais": "ruñar",
    "ruñaríamos": "ruñar",
    "ruñarían": "ruñar",
    "ruñarías": "ruñar",
    "ruñas": "ruñar",
    "ruñase": "ruñar",
    "ruñaseis": "ruñar",
    "ruñasen": "ruñar",
    "ruñases": "ruñar",
    "ruñaste": "ruñar",
    "ruñasteis": "ruñar",
    "ruñe": "ruñar",
    "ruñemos": "ruñar",
    "ruñen": "ruñar",
    "ruñes": "ruñar",
    "ruño": "ruñar",
    "ruñábamos": "ruñar",
    "ruñáis": "ruñar",
    "ruñáramos": "ruñar",
    "ruñáremos": "ruñar",
    "ruñásemos": "ruñar",
    "ruñé": "ruñar",
    "ruñéis": "ruñar",
    "ruñó": "ruñar",
    "rwandesa": "rwandés",
    "rwandesas": "rwandés",
    "rwandeses": "rwandés",
    "rábanos": "rábano",
    "rábica": "rábico",
    "rábicas": "rábico",
    "rábicos": "rábico",
    "rábidas": "rábida",
    "rábulas": "rábula",
    "rácana": "rácano",
    "rácanas": "rácano",
    "rácanos": "rácano",
    "ráfagas": "ráfaga",
    "ráftings": "ráfting",
    "rámea": "rámeo",
    "rámeas": "rámeo",
    "rámeos": "rámeo",
    "ránglanes": "ranglán",
    "ránkings": "ránking",
    "ránulas": "ránula",
    "rápeles": "rápel",
    "rápels": "rápel",
    "rápida": "rápido",
    "rápidas": "rápido",
    "rápidos": "rápido",
    "rápitas": "rápita",
    "ráseres": "ráser",
    "rátigos": "rátigo",
    "récipes": "récipe",
    "récords": "récord",
    "réditos": "rédito",
    "régulos": "régulo",
    "rémiges": "rémige",
    "rémoles": "rémol",
    "rémoras": "rémora",
    "réplicas": "réplica",
    "réproba": "réprobo",
    "réprobas": "réprobo",
    "réprobos": "réprobo",
    "réptiles": "réptil",
    "réquiems": "réquiem",
    "réspedes": "réspede",
    "réspices": "réspice",
    "rética": "rético",
    "réticas": "rético",
    "réticos": "rético",
    "réveillones": "réveillon",
    "ría": "reír",
    "rían": "reír",
    "rías": "ría",
    "ríctuses": "rictus",
    "ríe": "reír",
    "ríen": "reír",
    "ríes": "reír",
    "rígida": "rígido",
    "rígidas": "rígido",
    "rígidos": "rígido",
    "rímeles": "rímel",
    "rímels": "rímel",
    "rímmels": "rímmel",
    "río": "reír",
    "ríos": "río",
    "ríspida": "ríspido",
    "ríspidas": "ríspido",
    "ríspidos": "ríspido",
    "rítmica": "rítmico",
    "rítmicas": "rítmico",
    "rítmicos": "rítmico",
    "róbalos": "róbalo",
    "ródica": "ródico",
    "ródicas": "ródico",
    "ródicos": "ródico",
    "rómbica": "rómbico",
    "rómbicas": "rómbico",
    "rómbicos": "rómbico",
    "rósea": "róseo",
    "róseas": "róseo",
    "róseos": "róseo",
    "rótenes": "roten",
    "rótulas": "rótula",
    "rótulos": "rótulo",
    "rúas": "rúa",
    "rúbea": "rúbeo",
    "rúbeas": "rúbeo",
    "rúbeos": "rúbeo",
    "rúbricas": "rúbrica",
    "rúmenes": "rumen",
    "rúnica": "rúnico",
    "rúnicas": "rúnico",
    "rúnicos": "rúnico",
    "rústica": "rústico",
    "rústicas": "rústico",
    "rústicos": "rústico",
    "rútila": "rútilo",
    "rútilas": "rútilo",
    "rútilos": "rútilo",
    "s": "segundo",
    "sabadellenses": "sabadellense",
    "sabadillas": "sabadilla",
    "sabalares": "sabalar",
    "sabalera": "sabalero",
    "sabaleras": "sabalero",
    "sabaleros": "sabalero",
    "sabanas": "sabana",
    "sabandijas": "sabandija",
    "sabanea": "sabanear",
    "sabaneaba": "sabanear",
    "sabaneabais": "sabanear",
    "sabaneaban": "sabanear",
    "sabaneabas": "sabanear",
    "sabanead": "sabanear",
    "sabaneada": "sabanear",
    "sabaneadas": "sabanear",
    "sabaneado": "sabanear",
    "sabaneados": "sabanear",
    "sabaneamos": "sabanear",
    "sabanean": "sabanear",
    "sabaneando": "sabanear",
    "sabaneara": "sabanear",
    "sabanearais": "sabanear",
    "sabanearan": "sabanear",
    "sabanearas": "sabanear",
    "sabaneare": "sabanear",
    "sabaneareis": "sabanear",
    "sabanearemos": "sabanear",
    "sabanearen": "sabanear",
    "sabaneares": "sabanear",
    "sabanearon": "sabanear",
    "sabaneará": "sabanear",
    "sabanearán": "sabanear",
    "sabanearás": "sabanear",
    "sabanearé": "sabanear",
    "sabanearéis": "sabanear",
    "sabanearía": "sabanear",
    "sabanearíais": "sabanear",
    "sabanearíamos": "sabanear",
    "sabanearían": "sabanear",
    "sabanearías": "sabanear",
    "sabaneas": "sabanear",
    "sabanease": "sabanear",
    "sabaneaseis": "sabanear",
    "sabaneasen": "sabanear",
    "sabaneases": "sabanear",
    "sabaneaste": "sabanear",
    "sabaneasteis": "sabanear",
    "sabanee": "sabanear",
    "sabaneemos": "sabanear",
    "sabaneen": "sabanear",
    "sabanees": "sabanear",
    "sabaneo": "sabanear",
    "sabanera": "sabanero",
    "sabaneras": "sabanero",
    "sabaneros": "sabanero",
    "sabaneábamos": "sabanear",
    "sabaneáis": "sabanear",
    "sabaneáramos": "sabanear",
    "sabaneáremos": "sabanear",
    "sabaneásemos": "sabanear",
    "sabaneé": "sabanear",
    "sabaneéis": "sabanear",
    "sabaneó": "sabanear",
    "sabanillas": "sabanilla",
    "sabatina": "sabatino",
    "sabatinas": "sabatino",
    "sabatinos": "sabatino",
    "sabatismos": "sabatismo",
    "sabañones": "sabañón",
    "sabbathes": "sabbath",
    "sabe": "saber",
    "sabea": "sabeo",
    "sabeas": "sabeo",
    "sabed": "saber",
    "sabedora": "sabedor",
    "sabedoras": "sabedor",
    "sabedores": "sabedor",
    "sabelas": "sabela",
    "sabeliana": "sabeliano",
    "sabelianas": "sabeliano",
    "sabelianismos": "sabelianismo",
    "sabelianos": "sabeliano",
    "sabelotodos": "sabelotodo",
    "sabemos": "saber",
    "saben": "saber",
    "sabeos": "sabeo",
    "saberes": "saber",
    "sabes": "saber",
    "sabia": "sabio",
    "sabias": "sabio",
    "sabida": "saber",
    "sabidas": "saber",
    "sabidilla": "sabidillo",
    "sabidillas": "sabidillo",
    "sabidillos": "sabidillo",
    "sabido": "saber",
    "sabidos": "saber",
    "sabidurías": "sabiduría",
    "sabiendo": "saber",
    "sabihonda": "sabihondo",
    "sabihondas": "sabihondo",
    "sabihondeces": "sabihondez",
    "sabihondos": "sabihondo",
    "sabina": "sabino",
    "sabinas": "sabino",
    "sabinos": "sabino",
    "sabionda": "sabiondo",
    "sabiondas": "sabiondo",
    "sabiondeces": "sabiondez",
    "sabiondos": "sabiondo",
    "sabios": "sabio",
    "sablazos": "sablazo",
    "sablea": "sablear",
    "sableaba": "sablear",
    "sableabais": "sablear",
    "sableaban": "sablear",
    "sableabas": "sablear",
    "sablead": "sablear",
    "sableada": "sablear",
    "sableadas": "sablear",
    "sableado": "sablear",
    "sableadora": "sableador",
    "sableadoras": "sableador",
    "sableadores": "sableador",
    "sableados": "sablear",
    "sableamos": "sablear",
    "sablean": "sablear",
    "sableando": "sablear",
    "sableara": "sablear",
    "sablearais": "sablear",
    "sablearan": "sablear",
    "sablearas": "sablear",
    "sableare": "sablear",
    "sableareis": "sablear",
    "sablearemos": "sablear",
    "sablearen": "sablear",
    "sableares": "sablear",
    "sablearon": "sablear",
    "sableará": "sablear",
    "sablearán": "sablear",
    "sablearás": "sablear",
    "sablearé": "sablear",
    "sablearéis": "sablear",
    "sablearía": "sablear",
    "sablearíais": "sablear",
    "sablearíamos": "sablear",
    "sablearían": "sablear",
    "sablearías": "sablear",
    "sableas": "sablear",
    "sablease": "sablear",
    "sableaseis": "sablear",
    "sableasen": "sablear",
    "sableases": "sablear",
    "sableaste": "sablear",
    "sableasteis": "sablear",
    "sablee": "sablear",
    "sableemos": "sablear",
    "sableen": "sablear",
    "sablees": "sablear",
    "sableo": "sablear",
    "sables": "sable",
    "sableábamos": "sablear",
    "sableáis": "sablear",
    "sableáramos": "sablear",
    "sableáremos": "sablear",
    "sableásemos": "sablear",
    "sableé": "sablear",
    "sableéis": "sablear",
    "sableó": "sablear",
    "sablistas": "sablista",
    "sabogales": "sabogal",
    "sabogas": "saboga",
    "saborea": "saborear",
    "saboreaba": "saborear",
    "saboreabais": "saborear",
    "saboreaban": "saborear",
    "saboreabas": "saborear",
    "saboread": "saborear",
    "saboreada": "saborear",
    "saboreadas": "saborear",
    "saboreado": "saborear",
    "saboreadora": "saboreador",
    "saboreadoras": "saboreador",
    "saboreadores": "saboreador",
    "saboreados": "saborear",
    "saboreamientos": "saboreamiento",
    "saboreamos": "saborear",
    "saborean": "saborear",
    "saboreando": "saborear",
    "saboreara": "saborear",
    "saborearais": "saborear",
    "saborearan": "saborear",
    "saborearas": "saborear",
    "saboreare": "saborear",
    "saboreareis": "saborear",
    "saborearemos": "saborear",
    "saborearen": "saborear",
    "saboreares": "saborear",
    "saborearon": "saborear",
    "saboreará": "saborear",
    "saborearán": "saborear",
    "saborearás": "saborear",
    "saborearé": "saborear",
    "saborearéis": "saborear",
    "saborearía": "saborear",
    "saborearíais": "saborear",
    "saborearíamos": "saborear",
    "saborearían": "saborear",
    "saborearías": "saborear",
    "saboreas": "saborear",
    "saborease": "saborear",
    "saboreaseis": "saborear",
    "saboreasen": "saborear",
    "saboreases": "saborear",
    "saboreaste": "saborear",
    "saboreasteis": "saborear",
    "saboree": "saborear",
    "saboreemos": "saborear",
    "saboreen": "saborear",
    "saborees": "saborear",
    "saboreo": "saborear",
    "saboreos": "saboreo",
    "sabores": "sabor",
    "saboreábamos": "saborear",
    "saboreáis": "saborear",
    "saboreáramos": "saborear",
    "saboreáremos": "saborear",
    "saboreásemos": "saborear",
    "saboreé": "saborear",
    "saboreéis": "saborear",
    "saboreó": "saborear",
    "sabotajes": "sabotaje",
    "sabotea": "sabotear",
    "saboteaba": "sabotear",
    "saboteabais": "sabotear",
    "saboteaban": "sabotear",
    "saboteabas": "sabotear",
    "sabotead": "sabotear",
    "saboteada": "sabotear",
    "saboteadas": "sabotear",
    "saboteado": "sabotear",
    "saboteadora": "saboteador",
    "saboteadoras": "saboteador",
    "saboteadores": "saboteador",
    "saboteados": "sabotear",
    "saboteamos": "sabotear",
    "sabotean": "sabotear",
    "saboteando": "sabotear",
    "saboteara": "sabotear",
    "sabotearais": "sabotear",
    "sabotearan": "sabotear",
    "sabotearas": "sabotear",
    "saboteare": "sabotear",
    "saboteareis": "sabotear",
    "sabotearemos": "sabotear",
    "sabotearen": "sabotear",
    "saboteares": "sabotear",
    "sabotearon": "sabotear",
    "saboteará": "sabotear",
    "sabotearán": "sabotear",
    "sabotearás": "sabotear",
    "sabotearé": "sabotear",
    "sabotearéis": "sabotear",
    "sabotearía": "sabotear",
    "sabotearíais": "sabotear",
    "sabotearíamos": "sabotear",
    "sabotearían": "sabotear",
    "sabotearías": "sabotear",
    "saboteas": "sabotear",
    "sabotease": "sabotear",
    "saboteaseis": "sabotear",
    "saboteasen": "sabotear",
    "saboteases": "sabotear",
    "saboteaste": "sabotear",
    "saboteasteis": "sabotear",
    "sabotee": "sabotear",
    "saboteemos": "sabotear",
    "saboteen": "sabotear",
    "sabotees": "sabotear",
    "saboteo": "sabotear",
    "saboteábamos": "sabotear",
    "saboteáis": "sabotear",
    "saboteáramos": "sabotear",
    "saboteáremos": "sabotear",
    "saboteásemos": "sabotear",
    "saboteé": "sabotear",
    "saboteéis": "sabotear",
    "saboteó": "sabotear",
    "saboyana": "saboyano",
    "saboyanas": "saboyano",
    "saboyanos": "saboyano",
    "sabremos": "saber",
    "sabrosa": "sabroso",
    "sabrosas": "sabroso",
    "sabrosos": "sabroso",
    "sabrá": "saber",
    "sabrán": "saber",
    "sabrás": "saber",
    "sabré": "saber",
    "sabréis": "saber",
    "sabría": "saber",
    "sabríais": "saber",
    "sabríamos": "saber",
    "sabrían": "saber",
    "sabrías": "saber",
    "sabuesa": "sabueso",
    "sabuesas": "sabueso",
    "sabuesos": "sabueso",
    "sabulosa": "sabuloso",
    "sabulosas": "sabuloso",
    "sabulosos": "sabuloso",
    "saburrales": "saburral",
    "saburras": "saburra",
    "sabática": "sabático",
    "sabáticas": "sabático",
    "sabáticos": "sabático",
    "sabéis": "saber",
    "sabélica": "sabélico",
    "sabélicas": "sabélico",
    "sabélicos": "sabélico",
    "sabía": "saber",
    "sabíais": "saber",
    "sabíamos": "saber",
    "sabían": "saber",
    "sabías": "saber",
    "saca": "sacar",
    "sacaba": "sacar",
    "sacabais": "sacar",
    "sacaban": "sacar",
    "sacabas": "sacar",
    "sacabuches": "sacabuche",
    "sacad": "sacar",
    "sacada": "sacar",
    "sacadas": "sacar",
    "sacadillas": "sacadilla",
    "sacadineros": "sacadinero",
    "sacado": "sacar",
    "sacadora": "sacador",
    "sacadoras": "sacador",
    "sacadores": "sacador",
    "sacados": "sacar",
    "sacaduras": "sacadura",
    "sacaliñas": "sacaliña",
    "sacamientos": "sacamiento",
    "sacamolersacamolera": "sacamolero",
    "sacamolersacamoleras": "sacamolero",
    "sacamolersacamoleros": "sacamolero",
    "sacamos": "sacar",
    "sacan": "sacar",
    "sacando": "sacar",
    "sacara": "sacar",
    "sacarais": "sacar",
    "sacaran": "sacar",
    "sacaras": "sacar",
    "sacarasas": "sacarasa",
    "sacare": "sacar",
    "sacareis": "sacar",
    "sacaremos": "sacar",
    "sacaren": "sacar",
    "sacares": "sacar",
    "sacarifica": "sacarificar",
    "sacarificaba": "sacarificar",
    "sacarificabais": "sacarificar",
    "sacarificaban": "sacarificar",
    "sacarificabas": "sacarificar",
    "sacarificaciones": "sacarificación",
    "sacarificad": "sacarificar",
    "sacarificada": "sacarificar",
    "sacarificadas": "sacarificar",
    "sacarificado": "sacarificar",
    "sacarificadores": "sacarificador",
    "sacarificados": "sacarificar",
    "sacarificamos": "sacarificar",
    "sacarifican": "sacarificar",
    "sacarificando": "sacarificar",
    "sacarificara": "sacarificar",
    "sacarificarais": "sacarificar",
    "sacarificaran": "sacarificar",
    "sacarificaras": "sacarificar",
    "sacarificare": "sacarificar",
    "sacarificareis": "sacarificar",
    "sacarificaremos": "sacarificar",
    "sacarificaren": "sacarificar",
    "sacarificares": "sacarificar",
    "sacarificaron": "sacarificar",
    "sacarificará": "sacarificar",
    "sacarificarán": "sacarificar",
    "sacarificarás": "sacarificar",
    "sacarificaré": "sacarificar",
    "sacarificaréis": "sacarificar",
    "sacarificaría": "sacarificar",
    "sacarificaríais": "sacarificar",
    "sacarificaríamos": "sacarificar",
    "sacarificarían": "sacarificar",
    "sacarificarías": "sacarificar",
    "sacarificas": "sacarificar",
    "sacarificase": "sacarificar",
    "sacarificaseis": "sacarificar",
    "sacarificasen": "sacarificar",
    "sacarificases": "sacarificar",
    "sacarificaste": "sacarificar",
    "sacarificasteis": "sacarificar",
    "sacarifico": "sacarificar",
    "sacarificábamos": "sacarificar",
    "sacarificáis": "sacarificar",
    "sacarificáramos": "sacarificar",
    "sacarificáremos": "sacarificar",
    "sacarificásemos": "sacarificar",
    "sacarificó": "sacarificar",
    "sacarifique": "sacarificar",
    "sacarifiquemos": "sacarificar",
    "sacarifiquen": "sacarificar",
    "sacarifiques": "sacarificar",
    "sacarifiqué": "sacarificar",
    "sacarifiquéis": "sacarificar",
    "sacarimetrías": "sacarimetría",
    "sacarimétrica": "sacarimétrico",
    "sacarimétricas": "sacarimétrico",
    "sacarimétricos": "sacarimétrico",
    "sacarina": "sacarino",
    "sacarinas": "sacarino",
    "sacarinos": "sacarino",
    "sacaron": "sacar",
    "sacarosas": "sacarosa",
    "sacará": "sacar",
    "sacarán": "sacar",
    "sacarás": "sacar",
    "sacaré": "sacar",
    "sacaréis": "sacar",
    "sacaría": "sacar",
    "sacaríais": "sacar",
    "sacaríamos": "sacar",
    "sacarían": "sacar",
    "sacarías": "sacar",
    "sacarífera": "sacarífero",
    "sacaríferas": "sacarífero",
    "sacaríferos": "sacarífero",
    "sacarígena": "sacarígeno",
    "sacarígenas": "sacarígeno",
    "sacarígenos": "sacarígeno",
    "sacarímetros": "sacarímetro",
    "sacarómetros": "sacarómetro",
    "sacas": "sacar",
    "sacase": "sacar",
    "sacaseis": "sacar",
    "sacasen": "sacar",
    "sacases": "sacar",
    "sacaste": "sacar",
    "sacasteis": "sacar",
    "sacatapones": "sacatapón",
    "sacatintas": "sacatinta",
    "sacciformes": "sacciforme",
    "sacerdocios": "sacerdocio",
    "sacerdotales": "sacerdotal",
    "sacerdotes": "sacerdote",
    "sacerdotisas": "sacerdotisa",
    "saces": "saz",
    "sacha": "sachar",
    "sachaba": "sachar",
    "sachabais": "sachar",
    "sachaban": "sachar",
    "sachabas": "sachar",
    "sachad": "sachar",
    "sachada": "sachar",
    "sachadas": "sachar",
    "sachado": "sachar",
    "sachados": "sachar",
    "sachaduras": "sachadura",
    "sachamos": "sachar",
    "sachan": "sachar",
    "sachando": "sachar",
    "sachara": "sachar",
    "sacharais": "sachar",
    "sacharan": "sachar",
    "sacharas": "sachar",
    "sachare": "sachar",
    "sachareis": "sachar",
    "sacharemos": "sachar",
    "sacharen": "sachar",
    "sachares": "sachar",
    "sacharon": "sachar",
    "sachará": "sachar",
    "sacharán": "sachar",
    "sacharás": "sachar",
    "sacharé": "sachar",
    "sacharéis": "sachar",
    "sacharía": "sachar",
    "sacharíais": "sachar",
    "sacharíamos": "sachar",
    "sacharían": "sachar",
    "sacharías": "sachar",
    "sachas": "sachar",
    "sachase": "sachar",
    "sachaseis": "sachar",
    "sachasen": "sachar",
    "sachases": "sachar",
    "sachaste": "sachar",
    "sachasteis": "sachar",
    "sache": "sachar",
    "sachemos": "sachar",
    "sachen": "sachar",
    "saches": "sachar",
    "sachetes": "sachet",
    "sachets": "sachet",
    "sacho": "sachar",
    "sachos": "sacho",
    "sachábamos": "sachar",
    "sacháis": "sachar",
    "sacháramos": "sachar",
    "sacháremos": "sachar",
    "sachásemos": "sachar",
    "saché": "sachar",
    "sachéis": "sachar",
    "sachó": "sachar",
    "sacia": "saciar",
    "saciaba": "saciar",
    "saciabais": "saciar",
    "saciaban": "saciar",
    "saciabas": "saciar",
    "saciables": "saciable",
    "saciad": "saciar",
    "saciada": "saciar",
    "saciadas": "saciar",
    "saciado": "saciar",
    "saciados": "saciar",
    "saciamos": "saciar",
    "sacian": "saciar",
    "saciando": "saciar",
    "saciara": "saciar",
    "saciarais": "saciar",
    "saciaran": "saciar",
    "saciaras": "saciar",
    "saciare": "saciar",
    "saciareis": "saciar",
    "saciaremos": "saciar",
    "saciaren": "saciar",
    "saciares": "saciar",
    "saciaron": "saciar",
    "saciará": "saciar",
    "saciarán": "saciar",
    "saciarás": "saciar",
    "saciaré": "saciar",
    "saciaréis": "saciar",
    "saciaría": "saciar",
    "saciaríais": "saciar",
    "saciaríamos": "saciar",
    "saciarían": "saciar",
    "saciarías": "saciar",
    "sacias": "saciar",
    "saciase": "saciar",
    "saciaseis": "saciar",
    "saciasen": "saciar",
    "saciases": "saciar",
    "saciaste": "saciar",
    "saciasteis": "saciar",
    "sacie": "saciar",
    "saciedades": "saciedad",
    "saciemos": "saciar",
    "sacien": "saciar",
    "sacies": "saciar",
    "sacio": "saciar",
    "saciábamos": "saciar",
    "saciáis": "saciar",
    "saciáramos": "saciar",
    "saciáremos": "saciar",
    "saciásemos": "saciar",
    "sacié": "saciar",
    "saciéis": "saciar",
    "sació": "saciar",
    "saco": "sacar",
    "sacos": "saco",
    "sacra": "sacro",
    "sacrales": "sacral",
    "sacralice": "sacralizar",
    "sacralicemos": "sacralizar",
    "sacralicen": "sacralizar",
    "sacralices": "sacralizar",
    "sacralicé": "sacralizar",
    "sacralicéis": "sacralizar",
    "sacraliza": "sacralizar",
    "sacralizaba": "sacralizar",
    "sacralizabais": "sacralizar",
    "sacralizaban": "sacralizar",
    "sacralizabas": "sacralizar",
    "sacralizaciones": "sacralización",
    "sacralizad": "sacralizar",
    "sacralizada": "sacralizar",
    "sacralizadas": "sacralizar",
    "sacralizado": "sacralizar",
    "sacralizados": "sacralizar",
    "sacralizamos": "sacralizar",
    "sacralizan": "sacralizar",
    "sacralizando": "sacralizar",
    "sacralizara": "sacralizar",
    "sacralizarais": "sacralizar",
    "sacralizaran": "sacralizar",
    "sacralizaras": "sacralizar",
    "sacralizare": "sacralizar",
    "sacralizareis": "sacralizar",
    "sacralizaremos": "sacralizar",
    "sacralizaren": "sacralizar",
    "sacralizares": "sacralizar",
    "sacralizaron": "sacralizar",
    "sacralizará": "sacralizar",
    "sacralizarán": "sacralizar",
    "sacralizarás": "sacralizar",
    "sacralizaré": "sacralizar",
    "sacralizaréis": "sacralizar",
    "sacralizaría": "sacralizar",
    "sacralizaríais": "sacralizar",
    "sacralizaríamos": "sacralizar",
    "sacralizarían": "sacralizar",
    "sacralizarías": "sacralizar",
    "sacralizas": "sacralizar",
    "sacralizase": "sacralizar",
    "sacralizaseis": "sacralizar",
    "sacralizasen": "sacralizar",
    "sacralizases": "sacralizar",
    "sacralizaste": "sacralizar",
    "sacralizasteis": "sacralizar",
    "sacralizo": "sacralizar",
    "sacralizábamos": "sacralizar",
    "sacralizáis": "sacralizar",
    "sacralizáramos": "sacralizar",
    "sacralizáremos": "sacralizar",
    "sacralizásemos": "sacralizar",
    "sacralizó": "sacralizar",
    "sacramenta": "sacramentar",
    "sacramentaba": "sacramentar",
    "sacramentabais": "sacramentar",
    "sacramentaban": "sacramentar",
    "sacramentabas": "sacramentar",
    "sacramentaciones": "sacramentación",
    "sacramentad": "sacramentar",
    "sacramentada": "sacramentar",
    "sacramentadas": "sacramentar",
    "sacramentado": "sacramentar",
    "sacramentados": "sacramentar",
    "sacramentales": "sacramental",
    "sacramentamos": "sacramentar",
    "sacramentan": "sacramentar",
    "sacramentando": "sacramentar",
    "sacramentara": "sacramentar",
    "sacramentarais": "sacramentar",
    "sacramentaran": "sacramentar",
    "sacramentaras": "sacramentar",
    "sacramentare": "sacramentar",
    "sacramentareis": "sacramentar",
    "sacramentaremos": "sacramentar",
    "sacramentaren": "sacramentar",
    "sacramentares": "sacramentar",
    "sacramentaron": "sacramentar",
    "sacramentará": "sacramentar",
    "sacramentarán": "sacramentar",
    "sacramentarás": "sacramentar",
    "sacramentaré": "sacramentar",
    "sacramentaréis": "sacramentar",
    "sacramentaría": "sacramentar",
    "sacramentaríais": "sacramentar",
    "sacramentaríamos": "sacramentar",
    "sacramentarían": "sacramentar",
    "sacramentarías": "sacramentar",
    "sacramentas": "sacramentar",
    "sacramentase": "sacramentar",
    "sacramentaseis": "sacramentar",
    "sacramentasen": "sacramentar",
    "sacramentases": "sacramentar",
    "sacramentaste": "sacramentar",
    "sacramentasteis": "sacramentar",
    "sacramente": "sacramentar",
    "sacramentemos": "sacramentar",
    "sacramenten": "sacramentar",
    "sacramentes": "sacramentar",
    "sacramento": "sacramentar",
    "sacramentos": "sacramento",
    "sacramentábamos": "sacramentar",
    "sacramentáis": "sacramentar",
    "sacramentáramos": "sacramentar",
    "sacramentáremos": "sacramentar",
    "sacramentásemos": "sacramentar",
    "sacramenté": "sacramentar",
    "sacramentéis": "sacramentar",
    "sacramentó": "sacramentar",
    "sacras": "sacro",
    "sacratísima": "sacratísimo",
    "sacratísimas": "sacratísimo",
    "sacratísimos": "sacratísimo",
    "sacrifica": "sacrificar",
    "sacrificaba": "sacrificar",
    "sacrificabais": "sacrificar",
    "sacrificaban": "sacrificar",
    "sacrificabas": "sacrificar",
    "sacrificad": "sacrificar",
    "sacrificada": "sacrificar",
    "sacrificadas": "sacrificar",
    "sacrificaderos": "sacrificadero",
    "sacrificado": "sacrificar",
    "sacrificados": "sacrificar",
    "sacrificamos": "sacrificar",
    "sacrifican": "sacrificar",
    "sacrificando": "sacrificar",
    "sacrificara": "sacrificar",
    "sacrificarais": "sacrificar",
    "sacrificaran": "sacrificar",
    "sacrificaras": "sacrificar",
    "sacrificare": "sacrificar",
    "sacrificareis": "sacrificar",
    "sacrificaremos": "sacrificar",
    "sacrificaren": "sacrificar",
    "sacrificares": "sacrificar",
    "sacrificaron": "sacrificar",
    "sacrificará": "sacrificar",
    "sacrificarán": "sacrificar",
    "sacrificarás": "sacrificar",
    "sacrificaré": "sacrificar",
    "sacrificaréis": "sacrificar",
    "sacrificaría": "sacrificar",
    "sacrificaríais": "sacrificar",
    "sacrificaríamos": "sacrificar",
    "sacrificarían": "sacrificar",
    "sacrificarías": "sacrificar",
    "sacrificas": "sacrificar",
    "sacrificase": "sacrificar",
    "sacrificaseis": "sacrificar",
    "sacrificasen": "sacrificar",
    "sacrificases": "sacrificar",
    "sacrificaste": "sacrificar",
    "sacrificasteis": "sacrificar",
    "sacrificios": "sacrificio",
    "sacrifico": "sacrificar",
    "sacrificábamos": "sacrificar",
    "sacrificáis": "sacrificar",
    "sacrificáramos": "sacrificar",
    "sacrificáremos": "sacrificar",
    "sacrificásemos": "sacrificar",
    "sacrificó": "sacrificar",
    "sacrifique": "sacrificar",
    "sacrifiquemos": "sacrificar",
    "sacrifiquen": "sacrificar",
    "sacrifiques": "sacrificar",
    "sacrifiqué": "sacrificar",
    "sacrifiquéis": "sacrificar",
    "sacrilegios": "sacrilegio",
    "sacrismoches": "sacrismoche",
    "sacrismochos": "sacrismocho",
    "sacristana": "sacristán",
    "sacristanas": "sacristana",
    "sacristanes": "sacristán",
    "sacristanos": "sacristán",
    "sacristanías": "sacristanía",
    "sacristías": "sacristía",
    "sacrociática": "sacrociático",
    "sacrociáticas": "sacrociático",
    "sacrociáticos": "sacrociático",
    "sacrococcígea": "sacrococcígeo",
    "sacrococcígeas": "sacrococcígeo",
    "sacrococcígeos": "sacrococcígeo",
    "sacrocoxalgias": "sacrocoxalgia",
    "sacroespinales": "sacroespinal",
    "sacroespinosa": "sacroespinoso",
    "sacroespinosas": "sacroespinoso",
    "sacroespinosos": "sacroespinoso",
    "sacroilíaca": "sacroilíaco",
    "sacroilíacas": "sacroilíaco",
    "sacroilíacos": "sacroilíaco",
    "sacros": "sacro",
    "sacrosanta": "sacrosanto",
    "sacrosantas": "sacrosanto",
    "sacrosantos": "sacrosanto",
    "sacrotuberosa": "sacrotuberoso",
    "sacrotuberosas": "sacrotuberoso",
    "sacrotuberosos": "sacrotuberoso",
    "sacrílega": "sacrílego",
    "sacrílegas": "sacrílego",
    "sacrílegos": "sacrílego",
    "sacuda": "sacudir",
    "sacudamos": "sacudir",
    "sacudan": "sacudir",
    "sacudas": "sacudir",
    "sacude": "sacudir",
    "sacuden": "sacudir",
    "sacudes": "sacudir",
    "sacudid": "sacudir",
    "sacudida": "sacudir",
    "sacudidas": "sacudir",
    "sacudido": "sacudir",
    "sacudidora": "sacudidor",
    "sacudidoras": "sacudidor",
    "sacudidores": "sacudidor",
    "sacudidos": "sacudir",
    "sacudiduras": "sacudidura",
    "sacudiendo": "sacudir",
    "sacudiera": "sacudir",
    "sacudierais": "sacudir",
    "sacudieran": "sacudir",
    "sacudieras": "sacudir",
    "sacudiere": "sacudir",
    "sacudiereis": "sacudir",
    "sacudieren": "sacudir",
    "sacudieres": "sacudir",
    "sacudieron": "sacudir",
    "sacudiese": "sacudir",
    "sacudieseis": "sacudir",
    "sacudiesen": "sacudir",
    "sacudieses": "sacudir",
    "sacudimientos": "sacudimiento",
    "sacudimos": "sacudir",
    "sacudiremos": "sacudir",
    "sacudirá": "sacudir",
    "sacudirán": "sacudir",
    "sacudirás": "sacudir",
    "sacudiré": "sacudir",
    "sacudiréis": "sacudir",
    "sacudiría": "sacudir",
    "sacudiríais": "sacudir",
    "sacudiríamos": "sacudir",
    "sacudirían": "sacudir",
    "sacudirías": "sacudir",
    "sacudiste": "sacudir",
    "sacudisteis": "sacudir",
    "sacudiéramos": "sacudir",
    "sacudiéremos": "sacudir",
    "sacudiésemos": "sacudir",
    "sacudió": "sacudir",
    "sacudo": "sacudir",
    "sacudáis": "sacudir",
    "sacudí": "sacudir",
    "sacudía": "sacudir",
    "sacudíais": "sacudir",
    "sacudíamos": "sacudir",
    "sacudían": "sacudir",
    "sacudías": "sacudir",
    "sacudís": "sacudir",
    "sacábamos": "sacar",
    "sacádica": "sacádico",
    "sacádicas": "sacádico",
    "sacádicos": "sacádico",
    "sacáis": "sacar",
    "sacáramos": "sacar",
    "sacáremos": "sacar",
    "sacáridos": "sacárido",
    "sacásemos": "sacar",
    "sacó": "sacar",
    "sadis": "sadi",
    "sadismos": "sadismo",
    "sadomasoquismos": "sadomasoquismo",
    "sadomasoquistas": "sadomasoquista",
    "saducea": "saduceo",
    "saduceas": "saduceo",
    "saduceos": "saduceo",
    "saduceísmos": "saduceísmo",
    "sadíes": "sadi",
    "saetadas": "saetada",
    "saetas": "saeta",
    "saetazos": "saetazo",
    "saetea": "saetear",
    "saeteaba": "saetear",
    "saeteabais": "saetear",
    "saeteaban": "saetear",
    "saeteabas": "saetear",
    "saetead": "saetear",
    "saeteada": "saetear",
    "saeteadas": "saetear",
    "saeteado": "saetear",
    "saeteados": "saetear",
    "saeteamos": "saetear",
    "saetean": "saetear",
    "saeteando": "saetear",
    "saeteara": "saetear",
    "saetearais": "saetear",
    "saetearan": "saetear",
    "saetearas": "saetear",
    "saeteare": "saetear",
    "saeteareis": "saetear",
    "saetearemos": "saetear",
    "saetearen": "saetear",
    "saeteares": "saetear",
    "saetearon": "saetear",
    "saeteará": "saetear",
    "saetearán": "saetear",
    "saetearás": "saetear",
    "saetearé": "saetear",
    "saetearéis": "saetear",
    "saetearía": "saetear",
    "saetearíais": "saetear",
    "saetearíamos": "saetear",
    "saetearían": "saetear",
    "saetearías": "saetear",
    "saeteas": "saetear",
    "saetease": "saetear",
    "saeteaseis": "saetear",
    "saeteasen": "saetear",
    "saeteases": "saetear",
    "saeteaste": "saetear",
    "saeteasteis": "saetear",
    "saetee": "saetear",
    "saeteemos": "saetear",
    "saeteen": "saetear",
    "saetees": "saetear",
    "saeteo": "saetear",
    "saetera": "saetero",
    "saeteras": "saetero",
    "saeteros": "saetero",
    "saeteábamos": "saetear",
    "saeteáis": "saetear",
    "saeteáramos": "saetear",
    "saeteáremos": "saetear",
    "saeteásemos": "saetear",
    "saeteé": "saetear",
    "saeteéis": "saetear",
    "saeteó": "saetear",
    "saetines": "saetín",
    "safaris": "safari",
    "safawíes": "safawí",
    "safawís": "safawí",
    "safena": "safeno",
    "safenas": "safeno",
    "safenos": "safeno",
    "saffaris": "saffari",
    "safismos": "safismo",
    "safraninas": "safranina",
    "safroles": "safrol",
    "sagaces": "sagaz",
    "sagacidades": "sagacidad",
    "sagas": "saga",
    "sagazes": "sagaz",
    "sagitada": "sagitado",
    "sagitadas": "sagitado",
    "sagitados": "sagitado",
    "sagitales": "sagital",
    "sagitarios": "sagitario",
    "sagitas": "sagita",
    "sagitiformes": "sagitiforme",
    "sagrada": "sagrado",
    "sagradas": "sagrado",
    "sagrados": "sagrado",
    "sagrarios": "sagrario",
    "saguntina": "saguntino",
    "saguntinas": "saguntino",
    "saguntinos": "saguntino",
    "saharauis": "saharaui",
    "saharawis": "saharawi",
    "sahariana": "sahariano",
    "saharianas": "sahariano",
    "saharianos": "sahariano",
    "sahibes": "sahib",
    "sahibs": "sahib",
    "sahorna": "sahornar",
    "sahornaba": "sahornar",
    "sahornabais": "sahornar",
    "sahornaban": "sahornar",
    "sahornabas": "sahornar",
    "sahornad": "sahornar",
    "sahornada": "sahornar",
    "sahornadas": "sahornar",
    "sahornado": "sahornar",
    "sahornados": "sahornar",
    "sahornamos": "sahornar",
    "sahornan": "sahornar",
    "sahornando": "sahornar",
    "sahornara": "sahornar",
    "sahornarais": "sahornar",
    "sahornaran": "sahornar",
    "sahornaras": "sahornar",
    "sahornare": "sahornar",
    "sahornareis": "sahornar",
    "sahornaremos": "sahornar",
    "sahornaren": "sahornar",
    "sahornares": "sahornar",
    "sahornaron": "sahornar",
    "sahornará": "sahornar",
    "sahornarán": "sahornar",
    "sahornarás": "sahornar",
    "sahornaré": "sahornar",
    "sahornaréis": "sahornar",
    "sahornaría": "sahornar",
    "sahornaríais": "sahornar",
    "sahornaríamos": "sahornar",
    "sahornarían": "sahornar",
    "sahornarías": "sahornar",
    "sahornas": "sahornar",
    "sahornase": "sahornar",
    "sahornaseis": "sahornar",
    "sahornasen": "sahornar",
    "sahornases": "sahornar",
    "sahornaste": "sahornar",
    "sahornasteis": "sahornar",
    "sahorne": "sahornar",
    "sahornemos": "sahornar",
    "sahornen": "sahornar",
    "sahornes": "sahornar",
    "sahorno": "sahornar",
    "sahornos": "sahorno",
    "sahornábamos": "sahornar",
    "sahornáis": "sahornar",
    "sahornáramos": "sahornar",
    "sahornáremos": "sahornar",
    "sahornásemos": "sahornar",
    "sahorné": "sahornar",
    "sahornéis": "sahornar",
    "sahornó": "sahornar",
    "sahumada": "sahumado",
    "sahumadas": "sahumado",
    "sahumadores": "sahumador",
    "sahumados": "sahumado",
    "sahumaduras": "sahumadura",
    "sahumerios": "sahumerio",
    "saigas": "saiga",
    "sainaba": "sainar",
    "sainabais": "sainar",
    "sainaban": "sainar",
    "sainabas": "sainar",
    "sainad": "sainar",
    "sainada": "sainar",
    "sainadas": "sainar",
    "sainado": "sainar",
    "sainados": "sainar",
    "sainamos": "sainar",
    "sainando": "sainar",
    "sainara": "sainar",
    "sainarais": "sainar",
    "sainaran": "sainar",
    "sainaras": "sainar",
    "sainare": "sainar",
    "sainareis": "sainar",
    "sainaremos": "sainar",
    "sainaren": "sainar",
    "sainares": "sainar",
    "sainaron": "sainar",
    "sainará": "sainar",
    "sainarán": "sainar",
    "sainarás": "sainar",
    "sainaré": "sainar",
    "sainaréis": "sainar",
    "sainaría": "sainar",
    "sainaríais": "sainar",
    "sainaríamos": "sainar",
    "sainarían": "sainar",
    "sainarías": "sainar",
    "sainase": "sainar",
    "sainaseis": "sainar",
    "sainasen": "sainar",
    "sainases": "sainar",
    "sainaste": "sainar",
    "sainasteis": "sainar",
    "sainemos": "sainar",
    "saines": "saín",
    "sainetea": "sainetear",
    "saineteaba": "sainetear",
    "saineteabais": "sainetear",
    "saineteaban": "sainetear",
    "saineteabas": "sainetear",
    "sainetead": "sainetear",
    "saineteada": "sainetear",
    "saineteadas": "sainetear",
    "saineteado": "sainetear",
    "saineteados": "sainetear",
    "saineteamos": "sainetear",
    "sainetean": "sainetear",
    "saineteando": "sainetear",
    "saineteara": "sainetear",
    "sainetearais": "sainetear",
    "sainetearan": "sainetear",
    "sainetearas": "sainetear",
    "saineteare": "sainetear",
    "saineteareis": "sainetear",
    "sainetearemos": "sainetear",
    "sainetearen": "sainetear",
    "saineteares": "sainetear",
    "sainetearon": "sainetear",
    "saineteará": "sainetear",
    "sainetearán": "sainetear",
    "sainetearás": "sainetear",
    "sainetearé": "sainetear",
    "sainetearéis": "sainetear",
    "sainetearía": "sainetear",
    "sainetearíais": "sainetear",
    "sainetearíamos": "sainetear",
    "sainetearían": "sainetear",
    "sainetearías": "sainetear",
    "saineteas": "sainetear",
    "sainetease": "sainetear",
    "saineteaseis": "sainetear",
    "saineteasen": "sainetear",
    "saineteases": "sainetear",
    "saineteaste": "sainetear",
    "saineteasteis": "sainetear",
    "sainetee": "sainetear",
    "saineteemos": "sainetear",
    "saineteen": "sainetear",
    "sainetees": "sainetear",
    "saineteo": "sainetear",
    "sainetera": "sainetero",
    "saineteras": "sainetero",
    "saineteros": "sainetero",
    "sainetes": "sainete",
    "sainetesca": "sainetesco",
    "sainetescas": "sainetesco",
    "sainetescos": "sainetesco",
    "saineteábamos": "sainetear",
    "saineteáis": "sainetear",
    "saineteáramos": "sainetear",
    "saineteáremos": "sainetear",
    "saineteásemos": "sainetear",
    "saineteé": "sainetear",
    "saineteéis": "sainetear",
    "saineteó": "sainetear",
    "sainetistas": "sainetista",
    "saintsimoniana": "saintsimoniano",
    "saintsimonianas": "saintsimoniano",
    "saintsimonianos": "saintsimoniano",
    "sainábamos": "sainar",
    "saináis": "sainar",
    "saináramos": "sainar",
    "saináremos": "sainar",
    "sainásemos": "sainar",
    "sainé": "sainar",
    "sainéis": "sainar",
    "sainó": "sainar",
    "saja": "sajar",
    "sajaba": "sajar",
    "sajabais": "sajar",
    "sajaban": "sajar",
    "sajabas": "sajar",
    "sajad": "sajar",
    "sajada": "sajar",
    "sajadas": "sajar",
    "sajado": "sajar",
    "sajadores": "sajador",
    "sajados": "sajar",
    "sajaduras": "sajadura",
    "sajamos": "sajar",
    "sajan": "sajar",
    "sajando": "sajar",
    "sajara": "sajar",
    "sajarais": "sajar",
    "sajaran": "sajar",
    "sajaras": "sajar",
    "sajare": "sajar",
    "sajareis": "sajar",
    "sajaremos": "sajar",
    "sajaren": "sajar",
    "sajares": "sajar",
    "sajaron": "sajar",
    "sajará": "sajar",
    "sajarán": "sajar",
    "sajarás": "sajar",
    "sajaré": "sajar",
    "sajaréis": "sajar",
    "sajaría": "sajar",
    "sajaríais": "sajar",
    "sajaríamos": "sajar",
    "sajarían": "sajar",
    "sajarías": "sajar",
    "sajas": "sajar",
    "sajase": "sajar",
    "sajaseis": "sajar",
    "sajasen": "sajar",
    "sajases": "sajar",
    "sajaste": "sajar",
    "sajasteis": "sajar",
    "saje": "sajar",
    "sajemos": "sajar",
    "sajen": "sajar",
    "sajes": "sajar",
    "sajo": "sajar",
    "sajona": "sajón",
    "sajonas": "sajón",
    "sajones": "sajón",
    "sajábamos": "sajar",
    "sajáis": "sajar",
    "sajáramos": "sajar",
    "sajáremos": "sajar",
    "sajásemos": "sajar",
    "sajé": "sajar",
    "sajéis": "sajar",
    "sajó": "sajar",
    "sajóna": "sajón",
    "sajónas": "sajón",
    "sajónes": "sajón",
    "sakes": "sake",
    "sal": "salir",
    "sala": "salar",
    "salaba": "salar",
    "salabais": "salar",
    "salaban": "salar",
    "salabardos": "salabardo",
    "salabas": "salar",
    "salabres": "salabre",
    "salaces": "salaz",
    "salacidades": "salacidad",
    "salacots": "salacot",
    "salad": "salar",
    "salada": "salar",
    "saladares": "saladar",
    "saladas": "salar",
    "saladeros": "saladero",
    "saladerías": "saladería",
    "salado": "salar",
    "saladora": "salador",
    "saladoras": "salador",
    "saladores": "salador",
    "salados": "salar",
    "saladuras": "saladura",
    "salamandras": "salamandra",
    "salamandrina": "salamandrino",
    "salamandrinas": "salamandrino",
    "salamandrinos": "salamandrino",
    "salamanquesa": "salamanqués",
    "salamanquesas": "salamanquesa",
    "salamanqueses": "salamanqués",
    "salamanquina": "salamanquino",
    "salamanquinas": "salamanquino",
    "salamanquinos": "salamanquino",
    "salamis": "salami",
    "salamos": "salar",
    "salan": "salar",
    "salando": "salar",
    "salanganas": "salangana",
    "salara": "salar",
    "salarais": "salar",
    "salaran": "salar",
    "salaras": "salar",
    "salare": "salar",
    "salareis": "salar",
    "salaremos": "salar",
    "salaren": "salar",
    "salares": "salar",
    "salaria": "salariar",
    "salariaba": "salariar",
    "salariabais": "salariar",
    "salariaban": "salariar",
    "salariabas": "salariar",
    "salariad": "salariar",
    "salariada": "salariar",
    "salariadas": "salariar",
    "salariado": "salariar",
    "salariados": "salariar",
    "salariales": "salarial",
    "salariamos": "salariar",
    "salarian": "salariar",
    "salariando": "salariar",
    "salariara": "salariar",
    "salariarais": "salariar",
    "salariaran": "salariar",
    "salariaras": "salariar",
    "salariare": "salariar",
    "salariareis": "salariar",
    "salariaremos": "salariar",
    "salariaren": "salariar",
    "salariares": "salariar",
    "salariaron": "salariar",
    "salariará": "salariar",
    "salariarán": "salariar",
    "salariarás": "salariar",
    "salariaré": "salariar",
    "salariaréis": "salariar",
    "salariaría": "salariar",
    "salariaríais": "salariar",
    "salariaríamos": "salariar",
    "salariarían": "salariar",
    "salariarías": "salariar",
    "salarias": "salariar",
    "salariase": "salariar",
    "salariaseis": "salariar",
    "salariasen": "salariar",
    "salariases": "salariar",
    "salariaste": "salariar",
    "salariasteis": "salariar",
    "salarie": "salariar",
    "salariemos": "salariar",
    "salarien": "salariar",
    "salaries": "salariar",
    "salario": "salariar",
    "salarios": "salario",
    "salariábamos": "salariar",
    "salariáis": "salariar",
    "salariáramos": "salariar",
    "salariáremos": "salariar",
    "salariásemos": "salariar",
    "salarié": "salariar",
    "salariéis": "salariar",
    "salarió": "salariar",
    "salaron": "salar",
    "salará": "salar",
    "salarán": "salar",
    "salarás": "salar",
    "salaré": "salar",
    "salaréis": "salar",
    "salaría": "salar",
    "salaríais": "salar",
    "salaríamos": "salar",
    "salarían": "salar",
    "salarías": "salar",
    "salas": "salar",
    "salase": "salar",
    "salaseis": "salar",
    "salasen": "salar",
    "salases": "salar",
    "salaste": "salar",
    "salasteis": "salar",
    "salazonera": "salazonero",
    "salazoneras": "salazonero",
    "salazoneros": "salazonero",
    "salazones": "salazón",
    "salcedas": "salceda",
    "salcedos": "salcedo",
    "salces": "salce",
    "salchichas": "salchicha",
    "salchichera": "salchichero",
    "salchicheras": "salchichero",
    "salchicheros": "salchichero",
    "salchichones": "salchichón",
    "salcocha": "salcochar",
    "salcochaba": "salcochar",
    "salcochabais": "salcochar",
    "salcochaban": "salcochar",
    "salcochabas": "salcochar",
    "salcochad": "salcochar",
    "salcochada": "salcochar",
    "salcochadas": "salcochar",
    "salcochado": "salcochar",
    "salcochados": "salcochar",
    "salcochamos": "salcochar",
    "salcochan": "salcochar",
    "salcochando": "salcochar",
    "salcochara": "salcochar",
    "salcocharais": "salcochar",
    "salcocharan": "salcochar",
    "salcocharas": "salcochar",
    "salcochare": "salcochar",
    "salcochareis": "salcochar",
    "salcocharemos": "salcochar",
    "salcocharen": "salcochar",
    "salcochares": "salcochar",
    "salcocharon": "salcochar",
    "salcochará": "salcochar",
    "salcocharán": "salcochar",
    "salcocharás": "salcochar",
    "salcocharé": "salcochar",
    "salcocharéis": "salcochar",
    "salcocharía": "salcochar",
    "salcocharíais": "salcochar",
    "salcocharíamos": "salcochar",
    "salcocharían": "salcochar",
    "salcocharías": "salcochar",
    "salcochas": "salcochar",
    "salcochase": "salcochar",
    "salcochaseis": "salcochar",
    "salcochasen": "salcochar",
    "salcochases": "salcochar",
    "salcochaste": "salcochar",
    "salcochasteis": "salcochar",
    "salcoche": "salcochar",
    "salcochemos": "salcochar",
    "salcochen": "salcochar",
    "salcoches": "salcochar",
    "salcocho": "salcochar",
    "salcochos": "salcocho",
    "salcochábamos": "salcochar",
    "salcocháis": "salcochar",
    "salcocháramos": "salcochar",
    "salcocháremos": "salcochar",
    "salcochásemos": "salcochar",
    "salcoché": "salcochar",
    "salcochéis": "salcochar",
    "salcochó": "salcochar",
    "salda": "saldar",
    "saldaba": "saldar",
    "saldabais": "saldar",
    "saldaban": "saldar",
    "saldabas": "saldar",
    "saldad": "saldar",
    "saldada": "saldar",
    "saldadas": "saldar",
    "saldado": "saldar",
    "saldados": "saldar",
    "saldamos": "saldar",
    "saldan": "saldar",
    "saldando": "saldar",
    "saldara": "saldar",
    "saldarais": "saldar",
    "saldaran": "saldar",
    "saldaras": "saldar",
    "saldare": "saldar",
    "saldareis": "saldar",
    "saldaremos": "saldar",
    "saldaren": "saldar",
    "saldares": "saldar",
    "saldaron": "saldar",
    "saldará": "saldar",
    "saldarán": "saldar",
    "saldarás": "saldar",
    "saldaré": "saldar",
    "saldaréis": "saldar",
    "saldaría": "saldar",
    "saldaríais": "saldar",
    "saldaríamos": "saldar",
    "saldarían": "saldar",
    "saldarías": "saldar",
    "saldas": "saldar",
    "saldase": "saldar",
    "saldaseis": "saldar",
    "saldasen": "saldar",
    "saldases": "saldar",
    "saldaste": "saldar",
    "saldasteis": "saldar",
    "salde": "saldar",
    "saldemos": "saldar",
    "salden": "saldar",
    "saldes": "saldar",
    "saldistas": "saldista",
    "saldo": "saldar",
    "saldos": "saldo",
    "saldremos": "salir",
    "saldrá": "salir",
    "saldrán": "salir",
    "saldrás": "salir",
    "saldré": "salir",
    "saldréis": "salir",
    "saldría": "salir",
    "saldríais": "salir",
    "saldríamos": "salir",
    "saldrían": "salir",
    "saldrías": "salir",
    "saldábamos": "saldar",
    "saldáis": "saldar",
    "saldáramos": "saldar",
    "saldáremos": "saldar",
    "saldásemos": "saldar",
    "saldé": "saldar",
    "saldéis": "saldar",
    "saldó": "saldar",
    "sale": "salir",
    "saleas": "salea",
    "salediza": "saledizo",
    "saledizas": "saledizo",
    "saledizos": "saledizo",
    "salegares": "salegar",
    "salegas": "salega",
    "salemas": "salema",
    "salemos": "salar",
    "salen": "salir",
    "salepes": "salep",
    "saleps": "salep",
    "saleros": "salero",
    "salerosa": "saleroso",
    "salerosas": "saleroso",
    "salerosos": "saleroso",
    "sales": "salir",
    "salesas": "salesa",
    "salesiana": "salesiano",
    "salesianas": "salesiano",
    "salesianos": "salesiano",
    "salga": "salir",
    "salgaba": "salgar",
    "salgabais": "salgar",
    "salgaban": "salgar",
    "salgabas": "salgar",
    "salgad": "salgar",
    "salgada": "salgar",
    "salgadas": "salgar",
    "salgado": "salgar",
    "salgados": "salgar",
    "salgamos": "salir",
    "salgan": "salir",
    "salgando": "salgar",
    "salgara": "salgar",
    "salgarais": "salgar",
    "salgaran": "salgar",
    "salgaras": "salgar",
    "salgare": "salgar",
    "salgareis": "salgar",
    "salgaremos": "salgar",
    "salgaren": "salgar",
    "salgares": "salgar",
    "salgareña": "salgareño",
    "salgareñas": "salgareño",
    "salgareños": "salgareño",
    "salgaron": "salgar",
    "salgará": "salgar",
    "salgarán": "salgar",
    "salgarás": "salgar",
    "salgaré": "salgar",
    "salgaréis": "salgar",
    "salgaría": "salgar",
    "salgaríais": "salgar",
    "salgaríamos": "salgar",
    "salgarían": "salgar",
    "salgarías": "salgar",
    "salgas": "salir",
    "salgase": "salgar",
    "salgaseis": "salgar",
    "salgasen": "salgar",
    "salgases": "salgar",
    "salgaste": "salgar",
    "salgasteis": "salgar",
    "salgo": "salir",
    "salgue": "salgar",
    "salguemos": "salgar",
    "salguen": "salgar",
    "salgueras": "salguera",
    "salgueros": "salguero",
    "salgues": "salgar",
    "salgué": "salgar",
    "salguéis": "salgar",
    "salgábamos": "salgar",
    "salgáis": "salir",
    "salgáramos": "salgar",
    "salgáremos": "salgar",
    "salgásemos": "salgar",
    "salgó": "salgar",
    "salia": "salio",
    "salias": "salio",
    "salicales": "salical",
    "salicarias": "salicaria",
    "salicilaldehídos": "salicilaldehído",
    "salicilamidas": "salicilamida",
    "salicilanilidas": "salicilanilida",
    "salicilatos": "salicilato",
    "salicilazosulfapiridinas": "salicilazosulfapiridina",
    "salicinas": "salicina",
    "saliculturas": "salicultura",
    "salicácea": "salicáceo",
    "salicáceas": "salicáceo",
    "salicáceos": "salicáceo",
    "salicílica": "salicílico",
    "salicílicas": "salicílico",
    "salicílicos": "salicílico",
    "salid": "salir",
    "salida": "salir",
    "salidas": "salir",
    "salidera": "salidero",
    "salideras": "salidero",
    "salideros": "salidero",
    "salidizos": "salidizo",
    "salido": "salir",
    "salidos": "salir",
    "saliendo": "salir",
    "salientes": "saliente",
    "saliera": "salir",
    "salierais": "salir",
    "salieran": "salir",
    "salieras": "salir",
    "saliere": "salir",
    "saliereis": "salir",
    "salieren": "salir",
    "salieres": "salir",
    "salieron": "salir",
    "saliese": "salir",
    "salieseis": "salir",
    "saliesen": "salir",
    "salieses": "salir",
    "salifica": "salificar",
    "salificaba": "salificar",
    "salificabais": "salificar",
    "salificaban": "salificar",
    "salificabas": "salificar",
    "salificables": "salificable",
    "salificaciones": "salificación",
    "salificad": "salificar",
    "salificada": "salificar",
    "salificadas": "salificar",
    "salificado": "salificar",
    "salificados": "salificar",
    "salificamos": "salificar",
    "salifican": "salificar",
    "salificando": "salificar",
    "salificara": "salificar",
    "salificarais": "salificar",
    "salificaran": "salificar",
    "salificaras": "salificar",
    "salificare": "salificar",
    "salificareis": "salificar",
    "salificaremos": "salificar",
    "salificaren": "salificar",
    "salificares": "salificar",
    "salificaron": "salificar",
    "salificará": "salificar",
    "salificarán": "salificar",
    "salificarás": "salificar",
    "salificaré": "salificar",
    "salificaréis": "salificar",
    "salificaría": "salificar",
    "salificaríais": "salificar",
    "salificaríamos": "salificar",
    "salificarían": "salificar",
    "salificarías": "salificar",
    "salificas": "salificar",
    "salificase": "salificar",
    "salificaseis": "salificar",
    "salificasen": "salificar",
    "salificases": "salificar",
    "salificaste": "salificar",
    "salificasteis": "salificar",
    "salifico": "salificar",
    "salificábamos": "salificar",
    "salificáis": "salificar",
    "salificáramos": "salificar",
    "salificáremos": "salificar",
    "salificásemos": "salificar",
    "salificó": "salificar",
    "salifique": "salificar",
    "salifiquemos": "salificar",
    "salifiquen": "salificar",
    "salifiques": "salificar",
    "salifiqué": "salificar",
    "salifiquéis": "salificar",
    "salimientos": "salimiento",
    "salimos": "salir",
    "salina": "salino",
    "salinas": "salino",
    "salinera": "salinero",
    "salineras": "salinero",
    "salineros": "salinero",
    "salinidades": "salinidad",
    "salinizaciones": "salinización",
    "salinos": "salino",
    "salinómetros": "salinómetro",
    "salios": "salio",
    "saliste": "salir",
    "salisteis": "salir",
    "salitrada": "salitrado",
    "salitradas": "salitrado",
    "salitrados": "salitrado",
    "salitrales": "salitral",
    "salitrera": "salitrero",
    "salitreras": "salitrero",
    "salitreros": "salitrero",
    "salitrerías": "salitrería",
    "salitres": "salitre",
    "salitrosa": "salitroso",
    "salitrosas": "salitroso",
    "salitrosos": "salitroso",
    "saliva": "salivar",
    "salivaba": "salivar",
    "salivabais": "salivar",
    "salivaban": "salivar",
    "salivabas": "salivar",
    "salivaciones": "salivación",
    "salivad": "salivar",
    "salivada": "salivar",
    "salivadas": "salivar",
    "salivado": "salivar",
    "salivados": "salivar",
    "salivajos": "salivajo",
    "salivales": "salival",
    "salivamos": "salivar",
    "salivan": "salivar",
    "salivando": "salivar",
    "salivara": "salivar",
    "salivarais": "salivar",
    "salivaran": "salivar",
    "salivaras": "salivar",
    "salivare": "salivar",
    "salivareis": "salivar",
    "salivaremos": "salivar",
    "salivaren": "salivar",
    "salivares": "salivar",
    "salivaron": "salivar",
    "salivará": "salivar",
    "salivarán": "salivar",
    "salivarás": "salivar",
    "salivaré": "salivar",
    "salivaréis": "salivar",
    "salivaría": "salivar",
    "salivaríais": "salivar",
    "salivaríamos": "salivar",
    "salivarían": "salivar",
    "salivarías": "salivar",
    "salivas": "salivar",
    "salivase": "salivar",
    "salivaseis": "salivar",
    "salivasen": "salivar",
    "salivases": "salivar",
    "salivaste": "salivar",
    "salivasteis": "salivar",
    "salivatoria": "salivatorio",
    "salivatorias": "salivatorio",
    "salivatorios": "salivatorio",
    "salivazos": "salivazo",
    "salive": "salivar",
    "salivemos": "salivar",
    "saliven": "salivar",
    "saliveras": "salivera",
    "salives": "salivar",
    "salivinas": "salivina",
    "salivo": "salivar",
    "salivosa": "salivoso",
    "salivosas": "salivoso",
    "salivosos": "salivoso",
    "salivábamos": "salivar",
    "saliváis": "salivar",
    "saliváramos": "salivar",
    "saliváremos": "salivar",
    "salivásemos": "salivar",
    "salivé": "salivar",
    "salivéis": "salivar",
    "salivó": "salivar",
    "saliéramos": "salir",
    "saliéremos": "salir",
    "saliésemos": "salir",
    "salió": "salir",
    "salmanticenses": "salmanticense",
    "salmantina": "salmantino",
    "salmantinas": "salmantino",
    "salmantinos": "salmantino",
    "salmea": "salmear",
    "salmeaba": "salmear",
    "salmeabais": "salmear",
    "salmeaban": "salmear",
    "salmeabas": "salmear",
    "salmead": "salmear",
    "salmeada": "salmear",
    "salmeadas": "salmear",
    "salmeado": "salmear",
    "salmeados": "salmear",
    "salmeamos": "salmear",
    "salmean": "salmear",
    "salmeando": "salmear",
    "salmeara": "salmear",
    "salmearais": "salmear",
    "salmearan": "salmear",
    "salmearas": "salmear",
    "salmeare": "salmear",
    "salmeareis": "salmear",
    "salmearemos": "salmear",
    "salmearen": "salmear",
    "salmeares": "salmear",
    "salmearon": "salmear",
    "salmeará": "salmear",
    "salmearán": "salmear",
    "salmearás": "salmear",
    "salmearé": "salmear",
    "salmearéis": "salmear",
    "salmearía": "salmear",
    "salmearíais": "salmear",
    "salmearíamos": "salmear",
    "salmearían": "salmear",
    "salmearías": "salmear",
    "salmeas": "salmear",
    "salmease": "salmear",
    "salmeaseis": "salmear",
    "salmeasen": "salmear",
    "salmeases": "salmear",
    "salmeaste": "salmear",
    "salmeasteis": "salmear",
    "salmee": "salmear",
    "salmeemos": "salmear",
    "salmeen": "salmear",
    "salmees": "salmear",
    "salmeo": "salmear",
    "salmeres": "salmer",
    "salmeábamos": "salmear",
    "salmeáis": "salmear",
    "salmeáramos": "salmear",
    "salmeáremos": "salmear",
    "salmeásemos": "salmear",
    "salmeé": "salmear",
    "salmeéis": "salmear",
    "salmeó": "salmear",
    "salmiaces": "salmiac",
    "salmiacs": "salmiac",
    "salmistas": "salmista",
    "salmodia": "salmodiar",
    "salmodiaba": "salmodiar",
    "salmodiabais": "salmodiar",
    "salmodiaban": "salmodiar",
    "salmodiabas": "salmodiar",
    "salmodiad": "salmodiar",
    "salmodiada": "salmodiar",
    "salmodiadas": "salmodiar",
    "salmodiado": "salmodiar",
    "salmodiados": "salmodiar",
    "salmodiamos": "salmodiar",
    "salmodian": "salmodiar",
    "salmodiando": "salmodiar",
    "salmodiara": "salmodiar",
    "salmodiarais": "salmodiar",
    "salmodiaran": "salmodiar",
    "salmodiaras": "salmodiar",
    "salmodiare": "salmodiar",
    "salmodiareis": "salmodiar",
    "salmodiaremos": "salmodiar",
    "salmodiaren": "salmodiar",
    "salmodiares": "salmodiar",
    "salmodiaron": "salmodiar",
    "salmodiará": "salmodiar",
    "salmodiarán": "salmodiar",
    "salmodiarás": "salmodiar",
    "salmodiaré": "salmodiar",
    "salmodiaréis": "salmodiar",
    "salmodiaría": "salmodiar",
    "salmodiaríais": "salmodiar",
    "salmodiaríamos": "salmodiar",
    "salmodiarían": "salmodiar",
    "salmodiarías": "salmodiar",
    "salmodias": "salmodiar",
    "salmodiase": "salmodiar",
    "salmodiaseis": "salmodiar",
    "salmodiasen": "salmodiar",
    "salmodiases": "salmodiar",
    "salmodiaste": "salmodiar",
    "salmodiasteis": "salmodiar",
    "salmodie": "salmodiar",
    "salmodiemos": "salmodiar",
    "salmodien": "salmodiar",
    "salmodies": "salmodiar",
    "salmodio": "salmodiar",
    "salmodiábamos": "salmodiar",
    "salmodiáis": "salmodiar",
    "salmodiáramos": "salmodiar",
    "salmodiáremos": "salmodiar",
    "salmodiásemos": "salmodiar",
    "salmodié": "salmodiar",
    "salmodiéis": "salmodiar",
    "salmodió": "salmodiar",
    "salmonada": "salmonado",
    "salmonadas": "salmonado",
    "salmonados": "salmonado",
    "salmonelas": "salmonela",
    "salmoneras": "salmonera",
    "salmones": "salmón",
    "salmonetes": "salmonete",
    "salmoniculturas": "salmonicultura",
    "salmorejos": "salmorejo",
    "salmos": "salmo",
    "salmueras": "salmuera",
    "salmónidos": "salmónido",
    "salo": "salar",
    "salobrales": "salobral",
    "salobres": "salobre",
    "salobreña": "salobreño",
    "salobreñas": "salobreño",
    "salobreños": "salobreño",
    "salobridades": "salobridad",
    "saloles": "salol",
    "salomas": "saloma",
    "salomónica": "salomónico",
    "salomónicas": "salomónico",
    "salomónicos": "salomónico",
    "saloncillos": "saloncillo",
    "salones": "salón",
    "salpas": "salpa",
    "salpica": "salpicar",
    "salpicaba": "salpicar",
    "salpicabais": "salpicar",
    "salpicaban": "salpicar",
    "salpicabas": "salpicar",
    "salpicad": "salpicar",
    "salpicada": "salpicar",
    "salpicadas": "salpicar",
    "salpicaderos": "salpicadero",
    "salpicado": "salpicar",
    "salpicados": "salpicar",
    "salpicaduras": "salpicadura",
    "salpicamos": "salpicar",
    "salpican": "salpicar",
    "salpicando": "salpicar",
    "salpicara": "salpicar",
    "salpicarais": "salpicar",
    "salpicaran": "salpicar",
    "salpicaras": "salpicar",
    "salpicare": "salpicar",
    "salpicareis": "salpicar",
    "salpicaremos": "salpicar",
    "salpicaren": "salpicar",
    "salpicares": "salpicar",
    "salpicaron": "salpicar",
    "salpicará": "salpicar",
    "salpicarán": "salpicar",
    "salpicarás": "salpicar",
    "salpicaré": "salpicar",
    "salpicaréis": "salpicar",
    "salpicaría": "salpicar",
    "salpicaríais": "salpicar",
    "salpicaríamos": "salpicar",
    "salpicarían": "salpicar",
    "salpicarías": "salpicar",
    "salpicas": "salpicar",
    "salpicase": "salpicar",
    "salpicaseis": "salpicar",
    "salpicasen": "salpicar",
    "salpicases": "salpicar",
    "salpicaste": "salpicar",
    "salpicasteis": "salpicar",
    "salpico": "salpicar",
    "salpicones": "salpicón",
    "salpicábamos": "salpicar",
    "salpicáis": "salpicar",
    "salpicáramos": "salpicar",
    "salpicáremos": "salpicar",
    "salpicásemos": "salpicar",
    "salpicó": "salpicar",
    "salpimentaba": "salpimentar",
    "salpimentabais": "salpimentar",
    "salpimentaban": "salpimentar",
    "salpimentabas": "salpimentar",
    "salpimentad": "salpimentar",
    "salpimentada": "salpimentar",
    "salpimentadas": "salpimentar",
    "salpimentado": "salpimentar",
    "salpimentados": "salpimentar",
    "salpimentamos": "salpimentar",
    "salpimentando": "salpimentar",
    "salpimentara": "salpimentar",
    "salpimentarais": "salpimentar",
    "salpimentaran": "salpimentar",
    "salpimentaras": "salpimentar",
    "salpimentare": "salpimentar",
    "salpimentareis": "salpimentar",
    "salpimentaremos": "salpimentar",
    "salpimentaren": "salpimentar",
    "salpimentares": "salpimentar",
    "salpimentaron": "salpimentar",
    "salpimentará": "salpimentar",
    "salpimentarán": "salpimentar",
    "salpimentarás": "salpimentar",
    "salpimentaré": "salpimentar",
    "salpimentaréis": "salpimentar",
    "salpimentaría": "salpimentar",
    "salpimentaríais": "salpimentar",
    "salpimentaríamos": "salpimentar",
    "salpimentarían": "salpimentar",
    "salpimentarías": "salpimentar",
    "salpimentase": "salpimentar",
    "salpimentaseis": "salpimentar",
    "salpimentasen": "salpimentar",
    "salpimentases": "salpimentar",
    "salpimentaste": "salpimentar",
    "salpimentasteis": "salpimentar",
    "salpimentemos": "salpimentar",
    "salpimentábamos": "salpimentar",
    "salpimentáis": "salpimentar",
    "salpimentáramos": "salpimentar",
    "salpimentáremos": "salpimentar",
    "salpimentásemos": "salpimentar",
    "salpimenté": "salpimentar",
    "salpimentéis": "salpimentar",
    "salpimentó": "salpimentar",
    "salpimienta": "salpimentar",
    "salpimientan": "salpimentar",
    "salpimientas": "salpimienta",
    "salpimiente": "salpimentar",
    "salpimienten": "salpimentar",
    "salpimientes": "salpimentar",
    "salpimiento": "salpimentar",
    "salpingofaríngea": "salpingofaríngeo",
    "salpingofaríngeas": "salpingofaríngeo",
    "salpingofaríngeos": "salpingofaríngeo",
    "salpingopalatina": "salpingopalatino",
    "salpingopalatinas": "salpingopalatino",
    "salpingopalatinos": "salpingopalatino",
    "salpique": "salpicar",
    "salpiquemos": "salpicar",
    "salpiquen": "salpicar",
    "salpiques": "salpicar",
    "salpiqué": "salpicar",
    "salpiquéis": "salpicar",
    "salpresa": "salpresar",
    "salpresaba": "salpresar",
    "salpresabais": "salpresar",
    "salpresaban": "salpresar",
    "salpresabas": "salpresar",
    "salpresad": "salpresar",
    "salpresada": "salpresar",
    "salpresadas": "salpresar",
    "salpresado": "salpresar",
    "salpresados": "salpresar",
    "salpresamientos": "salpresamiento",
    "salpresamos": "salpresar",
    "salpresan": "salpresar",
    "salpresando": "salpresar",
    "salpresara": "salpresar",
    "salpresarais": "salpresar",
    "salpresaran": "salpresar",
    "salpresaras": "salpresar",
    "salpresare": "salpresar",
    "salpresareis": "salpresar",
    "salpresaremos": "salpresar",
    "salpresaren": "salpresar",
    "salpresares": "salpresar",
    "salpresaron": "salpresar",
    "salpresará": "salpresar",
    "salpresarán": "salpresar",
    "salpresarás": "salpresar",
    "salpresaré": "salpresar",
    "salpresaréis": "salpresar",
    "salpresaría": "salpresar",
    "salpresaríais": "salpresar",
    "salpresaríamos": "salpresar",
    "salpresarían": "salpresar",
    "salpresarías": "salpresar",
    "salpresas": "salpresar",
    "salpresase": "salpresar",
    "salpresaseis": "salpresar",
    "salpresasen": "salpresar",
    "salpresases": "salpresar",
    "salpresaste": "salpresar",
    "salpresasteis": "salpresar",
    "salprese": "salpresar",
    "salpresemos": "salpresar",
    "salpresen": "salpresar",
    "salpreses": "salpresar",
    "salpreso": "salpresar",
    "salpresábamos": "salpresar",
    "salpresáis": "salpresar",
    "salpresáramos": "salpresar",
    "salpresáremos": "salpresar",
    "salpresásemos": "salpresar",
    "salpresé": "salpresar",
    "salpreséis": "salpresar",
    "salpresó": "salpresar",
    "salpullidos": "salpullido",
    "salsas": "salsa",
    "salseras": "salsera",
    "salseretas": "salsereta",
    "salserillas": "salserilla",
    "salseruelas": "salseruela",
    "salsichones": "salsichón",
    "salsifises": "salsifí",
    "salsifíes": "salsifí",
    "salsifís": "salsifí",
    "salta": "saltar",
    "saltaba": "saltar",
    "saltabais": "saltar",
    "saltaban": "saltar",
    "saltabancos": "saltabanco",
    "saltabas": "saltar",
    "saltacaballos": "saltacaballo",
    "saltaciones": "saltación",
    "saltad": "saltar",
    "saltada": "saltar",
    "saltadas": "saltar",
    "saltaderos": "saltadero",
    "saltadiza": "saltadizo",
    "saltadizas": "saltadizo",
    "saltadizos": "saltadizo",
    "saltado": "saltar",
    "saltadora": "saltador",
    "saltadoras": "saltador",
    "saltadores": "saltador",
    "saltados": "saltar",
    "saltamos": "saltar",
    "saltan": "saltar",
    "saltando": "saltar",
    "saltantes": "saltante",
    "saltara": "saltar",
    "saltarais": "saltar",
    "saltaran": "saltar",
    "saltaras": "saltar",
    "saltare": "saltar",
    "saltaregla": "saltarregla",
    "saltareglas": "saltarregla",
    "saltareis": "saltar",
    "saltaremos": "saltar",
    "saltaren": "saltar",
    "saltares": "saltar",
    "saltarina": "saltarín",
    "saltarinas": "saltarín",
    "saltarines": "saltarín",
    "saltaron": "saltar",
    "saltará": "saltar",
    "saltarán": "saltar",
    "saltarás": "saltar",
    "saltaré": "saltar",
    "saltaréis": "saltar",
    "saltaría": "saltar",
    "saltaríais": "saltar",
    "saltaríamos": "saltar",
    "saltarían": "saltar",
    "saltarías": "saltar",
    "saltas": "saltar",
    "saltase": "saltar",
    "saltaseis": "saltar",
    "saltasen": "saltar",
    "saltases": "saltar",
    "saltaste": "saltar",
    "saltasteis": "saltar",
    "salte": "saltar",
    "saltea": "saltear",
    "salteaba": "saltear",
    "salteabais": "saltear",
    "salteaban": "saltear",
    "salteabas": "saltear",
    "saltead": "saltear",
    "salteada": "saltear",
    "salteadas": "saltear",
    "salteado": "saltear",
    "salteadora": "salteador",
    "salteadoras": "salteador",
    "salteadores": "salteador",
    "salteados": "saltear",
    "salteamientos": "salteamiento",
    "salteamos": "saltear",
    "saltean": "saltear",
    "salteando": "saltear",
    "salteara": "saltear",
    "saltearais": "saltear",
    "saltearan": "saltear",
    "saltearas": "saltear",
    "salteare": "saltear",
    "salteareis": "saltear",
    "saltearemos": "saltear",
    "saltearen": "saltear",
    "salteares": "saltear",
    "saltearon": "saltear",
    "salteará": "saltear",
    "saltearán": "saltear",
    "saltearás": "saltear",
    "saltearé": "saltear",
    "saltearéis": "saltear",
    "saltearía": "saltear",
    "saltearíais": "saltear",
    "saltearíamos": "saltear",
    "saltearían": "saltear",
    "saltearías": "saltear",
    "salteas": "saltear",
    "saltease": "saltear",
    "salteaseis": "saltear",
    "salteasen": "saltear",
    "salteases": "saltear",
    "salteaste": "saltear",
    "salteasteis": "saltear",
    "saltee": "saltear",
    "salteemos": "saltear",
    "salteen": "saltear",
    "saltees": "saltear",
    "saltemos": "saltar",
    "salten": "saltar",
    "salteo": "saltear",
    "salteos": "salteo",
    "saltera": "saltero",
    "salteras": "saltero",
    "salterios": "salterio",
    "salteros": "saltero",
    "saltes": "saltar",
    "salteábamos": "saltear",
    "salteáis": "saltear",
    "salteáramos": "saltear",
    "salteáremos": "saltear",
    "salteásemos": "saltear",
    "salteé": "saltear",
    "salteéis": "saltear",
    "salteó": "saltear",
    "saltillos": "saltillo",
    "salto": "saltar",
    "saltona": "saltón",
    "saltonas": "saltón",
    "saltones": "saltón",
    "saltos": "salto",
    "saltábamos": "saltar",
    "saltáis": "saltar",
    "saltáramos": "saltar",
    "saltáremos": "saltar",
    "saltásemos": "saltar",
    "salté": "saltar",
    "saltéis": "saltar",
    "saltó": "saltar",
    "saltómetros": "saltómetro",
    "salubres": "salubre",
    "salubridades": "salubridad",
    "saluda": "saludar",
    "saludaba": "saludar",
    "saludabais": "saludar",
    "saludaban": "saludar",
    "saludabas": "saludar",
    "saludabilidades": "saludabilidad",
    "saludables": "saludable",
    "saludad": "saludar",
    "saludada": "saludar",
    "saludadas": "saludar",
    "saludado": "saludar",
    "saludadora": "saludador",
    "saludadoras": "saludador",
    "saludadores": "saludador",
    "saludados": "saludar",
    "saludamos": "saludar",
    "saludan": "saludar",
    "saludando": "saludar",
    "saludara": "saludar",
    "saludarais": "saludar",
    "saludaran": "saludar",
    "saludaras": "saludar",
    "saludare": "saludar",
    "saludareis": "saludar",
    "saludaremos": "saludar",
    "saludaren": "saludar",
    "saludares": "saludar",
    "saludaron": "saludar",
    "saludará": "saludar",
    "saludarán": "saludar",
    "saludarás": "saludar",
    "saludaré": "saludar",
    "saludaréis": "saludar",
    "saludaría": "saludar",
    "saludaríais": "saludar",
    "saludaríamos": "saludar",
    "saludarían": "saludar",
    "saludarías": "saludar",
    "saludas": "saludar",
    "saludase": "saludar",
    "saludaseis": "saludar",
    "saludasen": "saludar",
    "saludases": "saludar",
    "saludaste": "saludar",
    "saludasteis": "saludar",
    "salude": "saludar",
    "saludemos": "saludar",
    "saluden": "saludar",
    "saludes": "saludar",
    "saludo": "saludar",
    "saludos": "saludo",
    "saludábamos": "saludar",
    "saludáis": "saludar",
    "saludáramos": "saludar",
    "saludáremos": "saludar",
    "saludásemos": "saludar",
    "saludé": "saludar",
    "saludéis": "saludar",
    "saludó": "saludar",
    "salutaciones": "salutación",
    "salva": "salvo",
    "salvaba": "salvar",
    "salvabais": "salvar",
    "salvaban": "salvar",
    "salvabas": "salvar",
    "salvaciones": "salvación",
    "salvad": "salvar",
    "salvada": "salvar",
    "salvadas": "salvar",
    "salvaderas": "salvadera",
    "salvado": "salvar",
    "salvadora": "salvador",
    "salvadoras": "salvador",
    "salvadores": "salvador",
    "salvadoreña": "salvadoreño",
    "salvadoreñas": "salvadoreño",
    "salvadoreños": "salvadoreño",
    "salvados": "salvar",
    "salvaguarda": "salvaguardar",
    "salvaguardaba": "salvaguardar",
    "salvaguardabais": "salvaguardar",
    "salvaguardaban": "salvaguardar",
    "salvaguardabas": "salvaguardar",
    "salvaguardad": "salvaguardar",
    "salvaguardada": "salvaguardar",
    "salvaguardadas": "salvaguardar",
    "salvaguardado": "salvaguardar",
    "salvaguardados": "salvaguardar",
    "salvaguardamos": "salvaguardar",
    "salvaguardan": "salvaguardar",
    "salvaguardando": "salvaguardar",
    "salvaguardara": "salvaguardar",
    "salvaguardarais": "salvaguardar",
    "salvaguardaran": "salvaguardar",
    "salvaguardaras": "salvaguardar",
    "salvaguardare": "salvaguardar",
    "salvaguardareis": "salvaguardar",
    "salvaguardaremos": "salvaguardar",
    "salvaguardaren": "salvaguardar",
    "salvaguardares": "salvaguardar",
    "salvaguardaron": "salvaguardar",
    "salvaguardará": "salvaguardar",
    "salvaguardarán": "salvaguardar",
    "salvaguardarás": "salvaguardar",
    "salvaguardaré": "salvaguardar",
    "salvaguardaréis": "salvaguardar",
    "salvaguardaría": "salvaguardar",
    "salvaguardaríais": "salvaguardar",
    "salvaguardaríamos": "salvaguardar",
    "salvaguardarían": "salvaguardar",
    "salvaguardarías": "salvaguardar",
    "salvaguardas": "salvaguardar",
    "salvaguardase": "salvaguardar",
    "salvaguardaseis": "salvaguardar",
    "salvaguardasen": "salvaguardar",
    "salvaguardases": "salvaguardar",
    "salvaguardaste": "salvaguardar",
    "salvaguardasteis": "salvaguardar",
    "salvaguarde": "salvaguardar",
    "salvaguardemos": "salvaguardar",
    "salvaguarden": "salvaguardar",
    "salvaguardes": "salvaguardar",
    "salvaguardias": "salvaguardia",
    "salvaguardo": "salvaguardar",
    "salvaguardábamos": "salvaguardar",
    "salvaguardáis": "salvaguardar",
    "salvaguardáramos": "salvaguardar",
    "salvaguardáremos": "salvaguardar",
    "salvaguardásemos": "salvaguardar",
    "salvaguardé": "salvaguardar",
    "salvaguardéis": "salvaguardar",
    "salvaguardó": "salvaguardar",
    "salvajadas": "salvajada",
    "salvajerías": "salvajería",
    "salvajes": "salvaje",
    "salvajinas": "salvajina",
    "salvajismos": "salvajismo",
    "salvamanos": "salvamano",
    "salvamentos": "salvamento",
    "salvamientos": "salvamiento",
    "salvamos": "salvar",
    "salvan": "salvar",
    "salvando": "salvar",
    "salvara": "salvar",
    "salvarais": "salvar",
    "salvaran": "salvar",
    "salvaras": "salvar",
    "salvare": "salvar",
    "salvareis": "salvar",
    "salvaremos": "salvar",
    "salvaren": "salvar",
    "salvares": "salvar",
    "salvariegos": "salvariego",
    "salvaron": "salvar",
    "salvará": "salvar",
    "salvarán": "salvar",
    "salvarás": "salvar",
    "salvaré": "salvar",
    "salvaréis": "salvar",
    "salvaría": "salvar",
    "salvaríais": "salvar",
    "salvaríamos": "salvar",
    "salvarían": "salvar",
    "salvarías": "salvar",
    "salvas": "salvo",
    "salvase": "salvar",
    "salvaseis": "salvar",
    "salvasen": "salvar",
    "salvases": "salvar",
    "salvaste": "salvar",
    "salvasteis": "salvar",
    "salvatajes": "salvataje",
    "salve": "salvar",
    "salvedades": "salvedad",
    "salvemos": "salvar",
    "salven": "salvar",
    "salves": "salve",
    "salvias": "salvia",
    "salvillas": "salvilla",
    "salviniácea": "salviniáceo",
    "salviniáceas": "salviniáceo",
    "salviniáceos": "salviniáceo",
    "salvo": "salvar",
    "salvoconductos": "salvoconducto",
    "salvohonores": "salvohonor",
    "salvos": "salvo",
    "salvábamos": "salvar",
    "salváis": "salvar",
    "salváramos": "salvar",
    "salváremos": "salvar",
    "salvásemos": "salvar",
    "salvé": "salvar",
    "salvéis": "salvar",
    "salvó": "salvar",
    "salábamos": "salar",
    "saláis": "salar",
    "saláramos": "salar",
    "saláremos": "salar",
    "salásemos": "salar",
    "salé": "salar",
    "saléis": "salar",
    "salí": "salir",
    "salía": "salir",
    "salíais": "salir",
    "salíamos": "salir",
    "salían": "salir",
    "salías": "salir",
    "salífera": "salífero",
    "salíferas": "salífero",
    "salíferos": "salífero",
    "salís": "salir",
    "saló": "salar",
    "samaníes": "samaní",
    "samanís": "samaní",
    "samarios": "samario",
    "samaritana": "samaritano",
    "samaritanas": "samaritano",
    "samaritanos": "samaritano",
    "samarskitas": "samarskita",
    "samarugos": "samarugo",
    "samas": "sama",
    "sambares": "sambar",
    "sambas": "samba",
    "sambenitos": "sambenito",
    "sambucas": "sambuca",
    "samia": "samio",
    "samias": "samio",
    "samios": "samio",
    "samnitas": "samnita",
    "samnítica": "samnítico",
    "samníticas": "samnítico",
    "samníticos": "samnítico",
    "samoana": "samoano",
    "samoanas": "samoano",
    "samoanos": "samoano",
    "samovares": "samovar",
    "samoyeda": "samoyedo",
    "samoyedas": "samoyedo",
    "samoyedos": "samoyedo",
    "sampanes": "sampán",
    "sampedreña": "sampedreño",
    "sampedreñas": "sampedreño",
    "sampedreños": "sampedreño",
    "samurais": "samurai",
    "san": "santo",
    "sana": "sano",
    "sanaba": "sanar",
    "sanabais": "sanar",
    "sanaban": "sanar",
    "sanabas": "sanar",
    "sanables": "sanable",
    "sanad": "sanar",
    "sanada": "sanar",
    "sanadas": "sanar",
    "sanado": "sanar",
    "sanadora": "sanador",
    "sanadoras": "sanador",
    "sanadores": "sanador",
    "sanados": "sanar",
    "sanamos": "sanar",
    "sanan": "sanar",
    "sanando": "sanar",
    "sanara": "sanar",
    "sanarais": "sanar",
    "sanaran": "sanar",
    "sanaras": "sanar",
    "sanare": "sanar",
    "sanareis": "sanar",
    "sanaremos": "sanar",
    "sanaren": "sanar",
    "sanares": "sanar",
    "sanaron": "sanar",
    "sanará": "sanar",
    "sanarán": "sanar",
    "sanarás": "sanar",
    "sanaré": "sanar",
    "sanaréis": "sanar",
    "sanaría": "sanar",
    "sanaríais": "sanar",
    "sanaríamos": "sanar",
    "sanarían": "sanar",
    "sanarías": "sanar",
    "sanas": "sano",
    "sanase": "sanar",
    "sanaseis": "sanar",
    "sanasen": "sanar",
    "sanases": "sanar",
    "sanaste": "sanar",
    "sanasteis": "sanar",
    "sanativa": "sanativo",
    "sanativas": "sanativo",
    "sanativos": "sanativo",
    "sanatorios": "sanatorio",
    "sanbernardos": "sanbernardo",
    "sanchopancesca": "sanchopancesco",
    "sanchopancescas": "sanchopancesco",
    "sanchopancescos": "sanchopancesco",
    "sanciona": "sancionar",
    "sancionaba": "sancionar",
    "sancionabais": "sancionar",
    "sancionaban": "sancionar",
    "sancionabas": "sancionar",
    "sancionables": "sancionable",
    "sancionad": "sancionar",
    "sancionada": "sancionar",
    "sancionadas": "sancionar",
    "sancionado": "sancionar",
    "sancionadora": "sancionador",
    "sancionadoras": "sancionador",
    "sancionadores": "sancionador",
    "sancionados": "sancionar",
    "sancionamos": "sancionar",
    "sancionan": "sancionar",
    "sancionando": "sancionar",
    "sancionara": "sancionar",
    "sancionarais": "sancionar",
    "sancionaran": "sancionar",
    "sancionaras": "sancionar",
    "sancionare": "sancionar",
    "sancionareis": "sancionar",
    "sancionaremos": "sancionar",
    "sancionaren": "sancionar",
    "sancionares": "sancionar",
    "sancionaron": "sancionar",
    "sancionará": "sancionar",
    "sancionarán": "sancionar",
    "sancionarás": "sancionar",
    "sancionaré": "sancionar",
    "sancionaréis": "sancionar",
    "sancionaría": "sancionar",
    "sancionaríais": "sancionar",
    "sancionaríamos": "sancionar",
    "sancionarían": "sancionar",
    "sancionarías": "sancionar",
    "sancionas": "sancionar",
    "sancionase": "sancionar",
    "sancionaseis": "sancionar",
    "sancionasen": "sancionar",
    "sancionases": "sancionar",
    "sancionaste": "sancionar",
    "sancionasteis": "sancionar",
    "sancionatoria": "sancionatorio",
    "sancionatorias": "sancionatorio",
    "sancionatorios": "sancionatorio",
    "sancione": "sancionar",
    "sancionemos": "sancionar",
    "sancionen": "sancionar",
    "sanciones": "sancionar",
    "sanciono": "sancionar",
    "sancionábamos": "sancionar",
    "sancionáis": "sancionar",
    "sancionáramos": "sancionar",
    "sancionáremos": "sancionar",
    "sancionásemos": "sancionar",
    "sancioné": "sancionar",
    "sancionéis": "sancionar",
    "sancionó": "sancionar",
    "sancocha": "sancochar",
    "sancochaba": "sancochar",
    "sancochabais": "sancochar",
    "sancochaban": "sancochar",
    "sancochabas": "sancochar",
    "sancochad": "sancochar",
    "sancochada": "sancochar",
    "sancochadas": "sancochar",
    "sancochado": "sancochar",
    "sancochados": "sancochar",
    "sancochamos": "sancochar",
    "sancochan": "sancochar",
    "sancochando": "sancochar",
    "sancochara": "sancochar",
    "sancocharais": "sancochar",
    "sancocharan": "sancochar",
    "sancocharas": "sancochar",
    "sancochare": "sancochar",
    "sancochareis": "sancochar",
    "sancocharemos": "sancochar",
    "sancocharen": "sancochar",
    "sancochares": "sancochar",
    "sancocharon": "sancochar",
    "sancochará": "sancochar",
    "sancocharán": "sancochar",
    "sancocharás": "sancochar",
    "sancocharé": "sancochar",
    "sancocharéis": "sancochar",
    "sancocharía": "sancochar",
    "sancocharíais": "sancochar",
    "sancocharíamos": "sancochar",
    "sancocharían": "sancochar",
    "sancocharías": "sancochar",
    "sancochas": "sancochar",
    "sancochase": "sancochar",
    "sancochaseis": "sancochar",
    "sancochasen": "sancochar",
    "sancochases": "sancochar",
    "sancochaste": "sancochar",
    "sancochasteis": "sancochar",
    "sancoche": "sancochar",
    "sancochemos": "sancochar",
    "sancochen": "sancochar",
    "sancoches": "sancochar",
    "sancocho": "sancochar",
    "sancochos": "sancocho",
    "sancochábamos": "sancochar",
    "sancocháis": "sancochar",
    "sancocháramos": "sancochar",
    "sancocháremos": "sancochar",
    "sancochásemos": "sancochar",
    "sancoché": "sancochar",
    "sancochéis": "sancochar",
    "sancochó": "sancochar",
    "sandalias": "sandalia",
    "sandalina": "sandalino",
    "sandalinas": "sandalino",
    "sandalinos": "sandalino",
    "sandeces": "sandez",
    "sandia": "sandio",
    "sandiales": "sandial",
    "sandiares": "sandiar",
    "sandias": "sandio",
    "sandinistas": "sandinista",
    "sandios": "sandio",
    "sandungas": "sandunga",
    "sandunguera": "sandunguero",
    "sandungueras": "sandunguero",
    "sandungueros": "sandunguero",
    "sandwicherías": "sandwichería",
    "sandwiches": "sandwich",
    "sandáracas": "sandáraca",
    "sandáracos": "sandáraco",
    "sandías": "sandía",
    "sane": "sanar",
    "sanea": "sanear",
    "saneaba": "sanear",
    "saneabais": "sanear",
    "saneaban": "sanear",
    "saneabas": "sanear",
    "sanead": "sanear",
    "saneada": "sanear",
    "saneadas": "sanear",
    "saneado": "sanear",
    "saneados": "sanear",
    "saneamientos": "saneamiento",
    "saneamos": "sanear",
    "sanean": "sanear",
    "saneando": "sanear",
    "saneara": "sanear",
    "sanearais": "sanear",
    "sanearan": "sanear",
    "sanearas": "sanear",
    "saneare": "sanear",
    "saneareis": "sanear",
    "sanearemos": "sanear",
    "sanearen": "sanear",
    "saneares": "sanear",
    "sanearon": "sanear",
    "saneará": "sanear",
    "sanearán": "sanear",
    "sanearás": "sanear",
    "sanearé": "sanear",
    "sanearéis": "sanear",
    "sanearía": "sanear",
    "sanearíais": "sanear",
    "sanearíamos": "sanear",
    "sanearían": "sanear",
    "sanearías": "sanear",
    "saneas": "sanear",
    "sanease": "sanear",
    "saneaseis": "sanear",
    "saneasen": "sanear",
    "saneases": "sanear",
    "saneaste": "sanear",
    "saneasteis": "sanear",
    "sanedrines": "sanedrín",
    "sanee": "sanear",
    "saneemos": "sanear",
    "saneen": "sanear",
    "sanees": "sanear",
    "sanemos": "sanar",
    "sanen": "sanar",
    "saneo": "sanear",
    "sanes": "sanar",
    "saneábamos": "sanear",
    "saneáis": "sanear",
    "saneáramos": "sanear",
    "saneáremos": "sanear",
    "saneásemos": "sanear",
    "saneé": "sanear",
    "saneéis": "sanear",
    "saneó": "sanear",
    "sanforizados": "sanforizado",
    "sangachos": "sangacho",
    "sangra": "sangrar",
    "sangraba": "sangrar",
    "sangrabais": "sangrar",
    "sangraban": "sangrar",
    "sangrabas": "sangrar",
    "sangrad": "sangrar",
    "sangrada": "sangrar",
    "sangradas": "sangrar",
    "sangraderas": "sangradera",
    "sangrado": "sangrar",
    "sangradora": "sangrador",
    "sangradoras": "sangrador",
    "sangradores": "sangrador",
    "sangrados": "sangrar",
    "sangraduras": "sangradura",
    "sangramos": "sangrar",
    "sangran": "sangrar",
    "sangrando": "sangrar",
    "sangrantes": "sangrante",
    "sangrara": "sangrar",
    "sangrarais": "sangrar",
    "sangraran": "sangrar",
    "sangraras": "sangrar",
    "sangrare": "sangrar",
    "sangrareis": "sangrar",
    "sangraremos": "sangrar",
    "sangraren": "sangrar",
    "sangrares": "sangrar",
    "sangraron": "sangrar",
    "sangrará": "sangrar",
    "sangrarán": "sangrar",
    "sangrarás": "sangrar",
    "sangraré": "sangrar",
    "sangraréis": "sangrar",
    "sangraría": "sangrar",
    "sangraríais": "sangrar",
    "sangraríamos": "sangrar",
    "sangrarían": "sangrar",
    "sangrarías": "sangrar",
    "sangras": "sangrar",
    "sangrase": "sangrar",
    "sangraseis": "sangrar",
    "sangrasen": "sangrar",
    "sangrases": "sangrar",
    "sangraste": "sangrar",
    "sangrasteis": "sangrar",
    "sangrazas": "sangraza",
    "sangre": "sangrar",
    "sangregordas": "sangregorda",
    "sangremos": "sangrar",
    "sangren": "sangrar",
    "sangres": "sangre",
    "sangrienta": "sangriento",
    "sangrientas": "sangriento",
    "sangrientos": "sangriento",
    "sangrizas": "sangriza",
    "sangro": "sangrar",
    "sangrábamos": "sangrar",
    "sangráis": "sangrar",
    "sangráramos": "sangrar",
    "sangráremos": "sangrar",
    "sangrásemos": "sangrar",
    "sangré": "sangrar",
    "sangréis": "sangrar",
    "sangrías": "sangría",
    "sangró": "sangrar",
    "sanguazas": "sanguaza",
    "sanguifica": "sanguificar",
    "sanguificaba": "sanguificar",
    "sanguificabais": "sanguificar",
    "sanguificaban": "sanguificar",
    "sanguificabas": "sanguificar",
    "sanguificaciones": "sanguificación",
    "sanguificad": "sanguificar",
    "sanguificada": "sanguificar",
    "sanguificadas": "sanguificar",
    "sanguificado": "sanguificar",
    "sanguificados": "sanguificar",
    "sanguificamos": "sanguificar",
    "sanguifican": "sanguificar",
    "sanguificando": "sanguificar",
    "sanguificara": "sanguificar",
    "sanguificarais": "sanguificar",
    "sanguificaran": "sanguificar",
    "sanguificaras": "sanguificar",
    "sanguificare": "sanguificar",
    "sanguificareis": "sanguificar",
    "sanguificaremos": "sanguificar",
    "sanguificaren": "sanguificar",
    "sanguificares": "sanguificar",
    "sanguificaron": "sanguificar",
    "sanguificará": "sanguificar",
    "sanguificarán": "sanguificar",
    "sanguificarás": "sanguificar",
    "sanguificaré": "sanguificar",
    "sanguificaréis": "sanguificar",
    "sanguificaría": "sanguificar",
    "sanguificaríais": "sanguificar",
    "sanguificaríamos": "sanguificar",
    "sanguificarían": "sanguificar",
    "sanguificarías": "sanguificar",
    "sanguificas": "sanguificar",
    "sanguificase": "sanguificar",
    "sanguificaseis": "sanguificar",
    "sanguificasen": "sanguificar",
    "sanguificases": "sanguificar",
    "sanguificaste": "sanguificar",
    "sanguificasteis": "sanguificar",
    "sanguifico": "sanguificar",
    "sanguificábamos": "sanguificar",
    "sanguificáis": "sanguificar",
    "sanguificáramos": "sanguificar",
    "sanguificáremos": "sanguificar",
    "sanguificásemos": "sanguificar",
    "sanguificó": "sanguificar",
    "sanguifique": "sanguificar",
    "sanguifiquemos": "sanguificar",
    "sanguifiquen": "sanguificar",
    "sanguifiques": "sanguificar",
    "sanguifiqué": "sanguificar",
    "sanguifiquéis": "sanguificar",
    "sanguijuelas": "sanguijuela",
    "sanguina": "sanguino",
    "sanguinaria": "sanguinario",
    "sanguinarias": "sanguinario",
    "sanguinarios": "sanguinario",
    "sanguinas": "sanguino",
    "sanguinolencias": "sanguinolencia",
    "sanguinolenta": "sanguinolento",
    "sanguinolentas": "sanguinolento",
    "sanguinolentos": "sanguinolento",
    "sanguinos": "sanguino",
    "sanguinosa": "sanguinoso",
    "sanguinosas": "sanguinoso",
    "sanguinosos": "sanguinoso",
    "sanguisorbas": "sanguisorba",
    "sanguífera": "sanguífero",
    "sanguíferas": "sanguífero",
    "sanguíferos": "sanguífero",
    "sanguínea": "sanguíneo",
    "sanguíneas": "sanguíneo",
    "sanguíneos": "sanguíneo",
    "sangüesas": "sangüesa",
    "sangüesos": "sangüeso",
    "sanidades": "sanidad",
    "sanidinas": "sanidina",
    "sanies": "sanie",
    "sanitaria": "sanitario",
    "sanitarias": "sanitario",
    "sanitarios": "sanitario",
    "sanjacados": "sanjacado",
    "sanjacatos": "sanjacato",
    "sanjacos": "sanjaco",
    "sanjuanadas": "sanjuanada",
    "sanjuanera": "sanjuanero",
    "sanjuaneras": "sanjuanero",
    "sanjuaneros": "sanjuanero",
    "sanjuaneña": "sanjuaneño",
    "sanjuaneñas": "sanjuaneño",
    "sanjuaneños": "sanjuaneño",
    "sanjuanina": "sanjuanino",
    "sanjuaninas": "sanjuanino",
    "sanjuaninos": "sanjuanino",
    "sanluisera": "sanluisero",
    "sanluiseras": "sanluisero",
    "sanluiseros": "sanluisero",
    "sanluiseña": "sanluiseño",
    "sanluiseñas": "sanluiseño",
    "sanluiseños": "sanluiseño",
    "sanmartines": "sanmartín",
    "sanmartiniana": "sanmartiniano",
    "sanmartinianas": "sanmartiniano",
    "sanmartinianos": "sanmartiniano",
    "sano": "sanar",
    "sanos": "sano",
    "sanpedros": "sanpedro",
    "sanscritismos": "sanscritismo",
    "sanscritistas": "sanscritista",
    "sansculottes": "sansculotte",
    "sansimoniana": "sansimoniano",
    "sansimonianas": "sansimoniano",
    "sansimonianos": "sansimoniano",
    "sansimonismos": "sansimonismo",
    "sansirolés": "sansirolé",
    "sansones": "sansón",
    "santa": "santo",
    "santabárbaras": "santabárbara",
    "santacruceña": "santacruceño",
    "santacruceñas": "santacruceño",
    "santacruceños": "santacruceño",
    "santafesina": "santafesino",
    "santafesinas": "santafesino",
    "santafesinos": "santafesino",
    "santalales": "santalal",
    "santaloles": "santalol",
    "santaláceas": "santalácea",
    "santandereana": "santandereano",
    "santandereanas": "santandereano",
    "santandereanos": "santandereano",
    "santanderienses": "santanderiense",
    "santanderina": "santanderino",
    "santanderinas": "santanderino",
    "santanderinos": "santanderino",
    "santas": "santo",
    "santateresas": "santateresa",
    "santelmos": "santelmo",
    "santera": "santero",
    "santeras": "santero",
    "santeros": "santero",
    "santerías": "santería",
    "santiaguera": "santiaguero",
    "santiagueras": "santiaguero",
    "santiagueros": "santiaguero",
    "santiaguesa": "santiagués",
    "santiaguesas": "santiagués",
    "santiagueses": "santiagués",
    "santiagueña": "santiagueño",
    "santiagueñas": "santiagueño",
    "santiagueños": "santiagueño",
    "santiaguina": "santiaguino",
    "santiaguinas": "santiaguino",
    "santiaguinos": "santiaguino",
    "santiaguistas": "santiaguista",
    "santidades": "santidad",
    "santifica": "santificar",
    "santificaba": "santificar",
    "santificabais": "santificar",
    "santificaban": "santificar",
    "santificabas": "santificar",
    "santificables": "santificable",
    "santificaciones": "santificación",
    "santificaciónes": "santificación",
    "santificad": "santificar",
    "santificada": "santificar",
    "santificadas": "santificar",
    "santificado": "santificar",
    "santificados": "santificar",
    "santificamos": "santificar",
    "santifican": "santificar",
    "santificando": "santificar",
    "santificara": "santificar",
    "santificarais": "santificar",
    "santificaran": "santificar",
    "santificaras": "santificar",
    "santificare": "santificar",
    "santificareis": "santificar",
    "santificaremos": "santificar",
    "santificaren": "santificar",
    "santificares": "santificar",
    "santificaron": "santificar",
    "santificará": "santificar",
    "santificarán": "santificar",
    "santificarás": "santificar",
    "santificaré": "santificar",
    "santificaréis": "santificar",
    "santificaría": "santificar",
    "santificaríais": "santificar",
    "santificaríamos": "santificar",
    "santificarían": "santificar",
    "santificarías": "santificar",
    "santificas": "santificar",
    "santificase": "santificar",
    "santificaseis": "santificar",
    "santificasen": "santificar",
    "santificases": "santificar",
    "santificaste": "santificar",
    "santificasteis": "santificar",
    "santifico": "santificar",
    "santificábamos": "santificar",
    "santificáis": "santificar",
    "santificáramos": "santificar",
    "santificáremos": "santificar",
    "santificásemos": "santificar",
    "santificó": "santificar",
    "santifique": "santificar",
    "santifiquemos": "santificar",
    "santifiquen": "santificar",
    "santifiques": "santificar",
    "santifiqué": "santificar",
    "santifiquéis": "santificar",
    "santigua": "santiguar",
    "santiguaba": "santiguar",
    "santiguabais": "santiguar",
    "santiguaban": "santiguar",
    "santiguabas": "santiguar",
    "santiguad": "santiguar",
    "santiguada": "santiguar",
    "santiguadas": "santiguar",
    "santiguaderas": "santiguadera",
    "santiguado": "santiguar",
    "santiguadora": "santiguador",
    "santiguadoras": "santiguador",
    "santiguadores": "santiguador",
    "santiguados": "santiguar",
    "santiguamientos": "santiguamiento",
    "santiguamos": "santiguar",
    "santiguan": "santiguar",
    "santiguando": "santiguar",
    "santiguara": "santiguar",
    "santiguarais": "santiguar",
    "santiguaran": "santiguar",
    "santiguaras": "santiguar",
    "santiguare": "santiguar",
    "santiguareis": "santiguar",
    "santiguaremos": "santiguar",
    "santiguaren": "santiguar",
    "santiguares": "santiguar",
    "santiguaron": "santiguar",
    "santiguará": "santiguar",
    "santiguarán": "santiguar",
    "santiguarás": "santiguar",
    "santiguaré": "santiguar",
    "santiguaréis": "santiguar",
    "santiguaría": "santiguar",
    "santiguaríais": "santiguar",
    "santiguaríamos": "santiguar",
    "santiguarían": "santiguar",
    "santiguarías": "santiguar",
    "santiguas": "santiguar",
    "santiguase": "santiguar",
    "santiguaseis": "santiguar",
    "santiguasen": "santiguar",
    "santiguases": "santiguar",
    "santiguaste": "santiguar",
    "santiguasteis": "santiguar",
    "santiguo": "santiguar",
    "santiguos": "santiguo",
    "santiguábamos": "santiguar",
    "santiguáis": "santiguar",
    "santiguáramos": "santiguar",
    "santiguáremos": "santiguar",
    "santiguásemos": "santiguar",
    "santiguó": "santiguar",
    "santigüe": "santiguar",
    "santigüemos": "santiguar",
    "santigüen": "santiguar",
    "santigües": "santiguar",
    "santigüé": "santiguar",
    "santigüéis": "santiguar",
    "santiscarios": "santiscario",
    "santolios": "santolio",
    "santona": "santón",
    "santonas": "santón",
    "santones": "santón",
    "santoninas": "santonina",
    "santorales": "santoral",
    "santos": "santo",
    "santuarios": "santuario",
    "santucha": "santucho",
    "santuchas": "santucho",
    "santuchos": "santucho",
    "santurrona": "santurrón",
    "santurronas": "santurrón",
    "santurronerías": "santurronería",
    "santurrones": "santurrón",
    "santísima": "santísimo",
    "santísimas": "santísimo",
    "santísimos": "santísimo",
    "santónica": "santónico",
    "santónicas": "santónico",
    "santónicos": "santónico",
    "sanábamos": "sanar",
    "sanáis": "sanar",
    "sanáramos": "sanar",
    "sanáremos": "sanar",
    "sanásemos": "sanar",
    "sané": "sanar",
    "sanéis": "sanar",
    "sanículas": "sanícula",
    "sanó": "sanar",
    "sapas": "sapa",
    "sapellis": "sapelli",
    "sapideces": "sapidez",
    "sapienciales": "sapiencial",
    "sapiencias": "sapiencia",
    "sapientes": "sapiente",
    "sapillos": "sapillo",
    "sapindáceas": "sapindácea",
    "saponifica": "saponificar",
    "saponificaba": "saponificar",
    "saponificabais": "saponificar",
    "saponificaban": "saponificar",
    "saponificabas": "saponificar",
    "saponificables": "saponificable",
    "saponificaciones": "saponificación",
    "saponificad": "saponificar",
    "saponificada": "saponificar",
    "saponificadas": "saponificar",
    "saponificado": "saponificar",
    "saponificados": "saponificar",
    "saponificamos": "saponificar",
    "saponifican": "saponificar",
    "saponificando": "saponificar",
    "saponificara": "saponificar",
    "saponificarais": "saponificar",
    "saponificaran": "saponificar",
    "saponificaras": "saponificar",
    "saponificare": "saponificar",
    "saponificareis": "saponificar",
    "saponificaremos": "saponificar",
    "saponificaren": "saponificar",
    "saponificares": "saponificar",
    "saponificaron": "saponificar",
    "saponificará": "saponificar",
    "saponificarán": "saponificar",
    "saponificarás": "saponificar",
    "saponificaré": "saponificar",
    "saponificaréis": "saponificar",
    "saponificaría": "saponificar",
    "saponificaríais": "saponificar",
    "saponificaríamos": "saponificar",
    "saponificarían": "saponificar",
    "saponificarías": "saponificar",
    "saponificas": "saponificar",
    "saponificase": "saponificar",
    "saponificaseis": "saponificar",
    "saponificasen": "saponificar",
    "saponificases": "saponificar",
    "saponificaste": "saponificar",
    "saponificasteis": "saponificar",
    "saponifico": "saponificar",
    "saponificábamos": "saponificar",
    "saponificáis": "saponificar",
    "saponificáramos": "saponificar",
    "saponificáremos": "saponificar",
    "saponificásemos": "saponificar",
    "saponificó": "saponificar",
    "saponifique": "saponificar",
    "saponifiquemos": "saponificar",
    "saponifiquen": "saponificar",
    "saponifiques": "saponificar",
    "saponifiqué": "saponificar",
    "saponifiquéis": "saponificar",
    "saponinas": "saponina",
    "saponitas": "saponita",
    "saponácea": "saponáceo",
    "saponáceas": "saponáceo",
    "saponáceos": "saponáceo",
    "saporífera": "saporífero",
    "saporíferas": "saporífero",
    "saporíferos": "saporífero",
    "sapos": "sapo",
    "sapotáceas": "sapotácea",
    "saprobios": "saprobio",
    "saprofita": "saprófito",
    "saprofitas": "saprófito",
    "saprofitos": "saprófito",
    "saprofítica": "saprofítico",
    "saprofíticas": "saprofítico",
    "saprofíticos": "saprofítico",
    "saprolitas": "saprolita",
    "sapropeles": "sapropel",
    "saprófaga": "saprófago",
    "saprófagas": "saprófago",
    "saprófagos": "saprófago",
    "saprófila": "saprófilo",
    "saprófilas": "saprófilo",
    "saprófilos": "saprófilo",
    "saprófita": "saprófito",
    "saprófitas": "saprófito",
    "saprófitos": "saprófito",
    "saque": "sacar",
    "saquea": "saquear",
    "saqueaba": "saquear",
    "saqueabais": "saquear",
    "saqueaban": "saquear",
    "saqueabas": "saquear",
    "saquead": "saquear",
    "saqueada": "saquear",
    "saqueadas": "saquear",
    "saqueado": "saquear",
    "saqueadora": "saqueador",
    "saqueadoras": "saqueador",
    "saqueadores": "saqueador",
    "saqueados": "saquear",
    "saqueamientos": "saqueamiento",
    "saqueamos": "saquear",
    "saquean": "saquear",
    "saqueando": "saquear",
    "saqueara": "saquear",
    "saquearais": "saquear",
    "saquearan": "saquear",
    "saquearas": "saquear",
    "saqueare": "saquear",
    "saqueareis": "saquear",
    "saquearemos": "saquear",
    "saquearen": "saquear",
    "saqueares": "saquear",
    "saquearon": "saquear",
    "saqueará": "saquear",
    "saquearán": "saquear",
    "saquearás": "saquear",
    "saquearé": "saquear",
    "saquearéis": "saquear",
    "saquearía": "saquear",
    "saquearíais": "saquear",
    "saquearíamos": "saquear",
    "saquearían": "saquear",
    "saquearías": "saquear",
    "saqueas": "saquear",
    "saquease": "saquear",
    "saqueaseis": "saquear",
    "saqueasen": "saquear",
    "saqueases": "saquear",
    "saqueaste": "saquear",
    "saqueasteis": "saquear",
    "saquee": "saquear",
    "saqueemos": "saquear",
    "saqueen": "saquear",
    "saquees": "saquear",
    "saquemos": "sacar",
    "saquen": "sacar",
    "saqueo": "saquear",
    "saqueos": "saqueo",
    "saquera": "saquero",
    "saqueras": "saquero",
    "saqueros": "saquero",
    "saquerías": "saquería",
    "saqueríos": "saquerío",
    "saques": "saque",
    "saqueábamos": "saquear",
    "saqueáis": "saquear",
    "saqueáramos": "saquear",
    "saqueáremos": "saquear",
    "saqueásemos": "saquear",
    "saqueé": "saquear",
    "saqueéis": "saquear",
    "saqueó": "saquear",
    "saquiladas": "saquilada",
    "saqué": "sacar",
    "saquéis": "sacar",
    "saragüetes": "saragüete",
    "sarahules": "sarahule",
    "sarajevitas": "sarajevita",
    "sarampiones": "sarampión",
    "sarandíes": "sarandí",
    "sarandís": "sarandí",
    "saraos": "sarao",
    "sarapes": "sarape",
    "sarasas": "sarasa",
    "sarcasmos": "sarcasmo",
    "sarcias": "sarcia",
    "sarcocarpios": "sarcocarpio",
    "sarcocarpos": "sarcocarpo",
    "sarcoceles": "sarcocele",
    "sarcolemas": "sarcolema",
    "sarcomas": "sarcoma",
    "sarcomatosa": "sarcomatoso",
    "sarcomatosas": "sarcomatoso",
    "sarcomatosos": "sarcomatoso",
    "sarcoplasmas": "sarcoplasma",
    "sarcoplasmática": "sarcoplasmático",
    "sarcoplasmáticas": "sarcoplasmático",
    "sarcoplasmáticos": "sarcoplasmático",
    "sarcoplásmica": "sarcoplásmico",
    "sarcoplásmicas": "sarcoplásmico",
    "sarcoplásmicos": "sarcoplásmico",
    "sarcosinas": "sarcosina",
    "sarcosinatos": "sarcosinato",
    "sarcosomas": "sarcosoma",
    "sarcástica": "sarcástico",
    "sarcásticas": "sarcástico",
    "sarcásticos": "sarcástico",
    "sarcófagos": "sarcófago",
    "sarcómeros": "sarcómero",
    "sarda": "sardo",
    "sardanas": "sardana",
    "sardanistas": "sardanista",
    "sardas": "sardo",
    "sardesca": "sardesco",
    "sardescas": "sardesco",
    "sardescos": "sardesco",
    "sardicenses": "sardicense",
    "sardinales": "sardinal",
    "sardinas": "sardina",
    "sardineles": "sardinel",
    "sardinera": "sardinero",
    "sardineras": "sardinero",
    "sardineros": "sardinero",
    "sardinetas": "sardineta",
    "sardios": "sardio",
    "sardones": "sardón",
    "sardonias": "sardonia",
    "sardonixes": "sardonix",
    "sardos": "sardo",
    "sardónica": "sardónico",
    "sardónicas": "sardónico",
    "sardónices": "sardónice",
    "sardónicos": "sardónico",
    "sardóniques": "sardónique",
    "sargada": "sargado",
    "sargadas": "sargado",
    "sargados": "sargado",
    "sargales": "sargal",
    "sargas": "sarga",
    "sargazos": "sargazo",
    "sargenta": "sargento",
    "sargentas": "sargento",
    "sargentea": "sargentear",
    "sargenteaba": "sargentear",
    "sargenteabais": "sargentear",
    "sargenteaban": "sargentear",
    "sargenteabas": "sargentear",
    "sargentead": "sargentear",
    "sargenteada": "sargentear",
    "sargenteadas": "sargentear",
    "sargenteado": "sargentear",
    "sargenteados": "sargentear",
    "sargenteamos": "sargentear",
    "sargentean": "sargentear",
    "sargenteando": "sargentear",
    "sargenteara": "sargentear",
    "sargentearais": "sargentear",
    "sargentearan": "sargentear",
    "sargentearas": "sargentear",
    "sargenteare": "sargentear",
    "sargenteareis": "sargentear",
    "sargentearemos": "sargentear",
    "sargentearen": "sargentear",
    "sargenteares": "sargentear",
    "sargentearon": "sargentear",
    "sargenteará": "sargentear",
    "sargentearán": "sargentear",
    "sargentearás": "sargentear",
    "sargentearé": "sargentear",
    "sargentearéis": "sargentear",
    "sargentearía": "sargentear",
    "sargentearíais": "sargentear",
    "sargentearíamos": "sargentear",
    "sargentearían": "sargentear",
    "sargentearías": "sargentear",
    "sargenteas": "sargentear",
    "sargentease": "sargentear",
    "sargenteaseis": "sargentear",
    "sargenteasen": "sargentear",
    "sargenteases": "sargentear",
    "sargenteaste": "sargentear",
    "sargenteasteis": "sargentear",
    "sargentee": "sargentear",
    "sargenteemos": "sargentear",
    "sargenteen": "sargentear",
    "sargentees": "sargentear",
    "sargenteo": "sargentear",
    "sargenterías": "sargentería",
    "sargenteábamos": "sargentear",
    "sargenteáis": "sargentear",
    "sargenteáramos": "sargentear",
    "sargenteáremos": "sargentear",
    "sargenteásemos": "sargentear",
    "sargenteé": "sargentear",
    "sargenteéis": "sargentear",
    "sargenteó": "sargentear",
    "sargentonas": "sargentona",
    "sargentos": "sargento",
    "sargentías": "sargentía",
    "sargos": "sargo",
    "sarguera": "sarguero",
    "sargueras": "sarguero",
    "sargueros": "sarguero",
    "sargónidas": "sargónida",
    "saris": "sari",
    "saristas": "sarista",
    "sarmentaba": "sarmentar",
    "sarmentabais": "sarmentar",
    "sarmentaban": "sarmentar",
    "sarmentabas": "sarmentar",
    "sarmentad": "sarmentar",
    "sarmentada": "sarmentar",
    "sarmentadas": "sarmentar",
    "sarmentado": "sarmentar",
    "sarmentadora": "sarmentador",
    "sarmentadoras": "sarmentador",
    "sarmentadores": "sarmentador",
    "sarmentados": "sarmentar",
    "sarmentamos": "sarmentar",
    "sarmentando": "sarmentar",
    "sarmentara": "sarmentar",
    "sarmentarais": "sarmentar",
    "sarmentaran": "sarmentar",
    "sarmentaras": "sarmentar",
    "sarmentare": "sarmentar",
    "sarmentareis": "sarmentar",
    "sarmentaremos": "sarmentar",
    "sarmentaren": "sarmentar",
    "sarmentares": "sarmentar",
    "sarmentaron": "sarmentar",
    "sarmentará": "sarmentar",
    "sarmentarán": "sarmentar",
    "sarmentarás": "sarmentar",
    "sarmentaré": "sarmentar",
    "sarmentaréis": "sarmentar",
    "sarmentaría": "sarmentar",
    "sarmentaríais": "sarmentar",
    "sarmentaríamos": "sarmentar",
    "sarmentarían": "sarmentar",
    "sarmentarías": "sarmentar",
    "sarmentase": "sarmentar",
    "sarmentaseis": "sarmentar",
    "sarmentasen": "sarmentar",
    "sarmentases": "sarmentar",
    "sarmentaste": "sarmentar",
    "sarmentasteis": "sarmentar",
    "sarmentazos": "sarmentazo",
    "sarmentemos": "sarmentar",
    "sarmenteras": "sarmentera",
    "sarmenticia": "sarmenticio",
    "sarmenticias": "sarmenticio",
    "sarmenticios": "sarmenticio",
    "sarmentosa": "sarmentoso",
    "sarmentosas": "sarmentoso",
    "sarmentosos": "sarmentoso",
    "sarmentábamos": "sarmentar",
    "sarmentáis": "sarmentar",
    "sarmentáramos": "sarmentar",
    "sarmentáremos": "sarmentar",
    "sarmentásemos": "sarmentar",
    "sarmenté": "sarmentar",
    "sarmentéis": "sarmentar",
    "sarmentó": "sarmentar",
    "sarmienta": "sarmentar",
    "sarmientan": "sarmentar",
    "sarmientas": "sarmentar",
    "sarmiente": "sarmentar",
    "sarmienten": "sarmentar",
    "sarmientes": "sarmentar",
    "sarmiento": "sarmentar",
    "sarmientos": "sarmiento",
    "sarmática": "sarmático",
    "sarmáticas": "sarmático",
    "sarmáticos": "sarmático",
    "sarnas": "sarna",
    "sarnosa": "sarnoso",
    "sarnosas": "sarnoso",
    "sarnosos": "sarnoso",
    "sarpullidos": "sarpullido",
    "sarracena": "sarraceno",
    "sarracenas": "sarraceno",
    "sarraceniácea": "sarraceniáceo",
    "sarraceniáceas": "sarraceniáceo",
    "sarraceniáceos": "sarraceniáceo",
    "sarracenos": "sarraceno",
    "sarracina": "sarracino",
    "sarracinas": "sarracino",
    "sarracines": "sarracín",
    "sarracinos": "sarracino",
    "sarracénica": "sarracénico",
    "sarracénicas": "sarracénico",
    "sarracénicos": "sarracénico",
    "sarrias": "sarria",
    "sarrietas": "sarrieta",
    "sarrillos": "sarrillo",
    "sarros": "sarro",
    "sarrosa": "sarroso",
    "sarrosas": "sarroso",
    "sarrosos": "sarroso",
    "sartales": "sartal",
    "sartas": "sarta",
    "sartenadas": "sartenada",
    "sartenazos": "sartenazo",
    "sartenera": "sartenero",
    "sarteneras": "sartenero",
    "sarteneros": "sartenero",
    "sartenes": "sartén",
    "sartoria": "sartorio",
    "sartorias": "sartorio",
    "sartorios": "sartorio",
    "sasabis": "sasabi",
    "sasafrases": "sasafrás",
    "sasolitas": "sasolita",
    "sastra": "sastre",
    "sastras": "sastre",
    "sastrerías": "sastrería",
    "sastres": "sastre",
    "sasánidas": "sasánida",
    "satanismos": "satanismo",
    "satanistas": "satanista",
    "satelice": "satelizar",
    "satelicemos": "satelizar",
    "satelicen": "satelizar",
    "satelices": "satelizar",
    "satelicé": "satelizar",
    "satelicéis": "satelizar",
    "sateliza": "satelizar",
    "satelizaba": "satelizar",
    "satelizabais": "satelizar",
    "satelizaban": "satelizar",
    "satelizabas": "satelizar",
    "satelizaciones": "satelización",
    "satelizad": "satelizar",
    "satelizada": "satelizar",
    "satelizadas": "satelizar",
    "satelizado": "satelizar",
    "satelizados": "satelizar",
    "satelizamos": "satelizar",
    "satelizan": "satelizar",
    "satelizando": "satelizar",
    "satelizara": "satelizar",
    "satelizarais": "satelizar",
    "satelizaran": "satelizar",
    "satelizaras": "satelizar",
    "satelizare": "satelizar",
    "satelizareis": "satelizar",
    "satelizaremos": "satelizar",
    "satelizaren": "satelizar",
    "satelizares": "satelizar",
    "satelizaron": "satelizar",
    "satelizará": "satelizar",
    "satelizarán": "satelizar",
    "satelizarás": "satelizar",
    "satelizaré": "satelizar",
    "satelizaréis": "satelizar",
    "satelizaría": "satelizar",
    "satelizaríais": "satelizar",
    "satelizaríamos": "satelizar",
    "satelizarían": "satelizar",
    "satelizarías": "satelizar",
    "satelizas": "satelizar",
    "satelizase": "satelizar",
    "satelizaseis": "satelizar",
    "satelizasen": "satelizar",
    "satelizases": "satelizar",
    "satelizaste": "satelizar",
    "satelizasteis": "satelizar",
    "satelizo": "satelizar",
    "satelizábamos": "satelizar",
    "satelizáis": "satelizar",
    "satelizáramos": "satelizar",
    "satelizáremos": "satelizar",
    "satelizásemos": "satelizar",
    "satelizó": "satelizar",
    "satenes": "satén",
    "satina": "satinar",
    "satinaba": "satinar",
    "satinabais": "satinar",
    "satinaban": "satinar",
    "satinabas": "satinar",
    "satinad": "satinar",
    "satinada": "satinar",
    "satinadas": "satinar",
    "satinado": "satinar",
    "satinadora": "satinador",
    "satinadoras": "satinador",
    "satinadores": "satinador",
    "satinados": "satinar",
    "satinamos": "satinar",
    "satinan": "satinar",
    "satinando": "satinar",
    "satinara": "satinar",
    "satinarais": "satinar",
    "satinaran": "satinar",
    "satinaras": "satinar",
    "satinare": "satinar",
    "satinareis": "satinar",
    "satinaremos": "satinar",
    "satinaren": "satinar",
    "satinares": "satinar",
    "satinaron": "satinar",
    "satinará": "satinar",
    "satinarán": "satinar",
    "satinarás": "satinar",
    "satinaré": "satinar",
    "satinaréis": "satinar",
    "satinaría": "satinar",
    "satinaríais": "satinar",
    "satinaríamos": "satinar",
    "satinarían": "satinar",
    "satinarías": "satinar",
    "satinas": "satinar",
    "satinase": "satinar",
    "satinaseis": "satinar",
    "satinasen": "satinar",
    "satinases": "satinar",
    "satinaste": "satinar",
    "satinasteis": "satinar",
    "satine": "satinar",
    "satinemos": "satinar",
    "satinen": "satinar",
    "satines": "satinar",
    "satino": "satinar",
    "satinábamos": "satinar",
    "satináis": "satinar",
    "satináramos": "satinar",
    "satináremos": "satinar",
    "satinásemos": "satinar",
    "satiné": "satinar",
    "satinéis": "satinar",
    "satinó": "satinar",
    "satirice": "satirizar",
    "satiricemos": "satirizar",
    "satiricen": "satirizar",
    "satirices": "satirizar",
    "satiricé": "satirizar",
    "satiricéis": "satirizar",
    "satiriones": "satirión",
    "satiriza": "satirizar",
    "satirizaba": "satirizar",
    "satirizabais": "satirizar",
    "satirizaban": "satirizar",
    "satirizabas": "satirizar",
    "satirizad": "satirizar",
    "satirizada": "satirizar",
    "satirizadas": "satirizar",
    "satirizado": "satirizar",
    "satirizados": "satirizar",
    "satirizamos": "satirizar",
    "satirizan": "satirizar",
    "satirizando": "satirizar",
    "satirizara": "satirizar",
    "satirizarais": "satirizar",
    "satirizaran": "satirizar",
    "satirizaras": "satirizar",
    "satirizare": "satirizar",
    "satirizareis": "satirizar",
    "satirizaremos": "satirizar",
    "satirizaren": "satirizar",
    "satirizares": "satirizar",
    "satirizaron": "satirizar",
    "satirizará": "satirizar",
    "satirizarán": "satirizar",
    "satirizarás": "satirizar",
    "satirizaré": "satirizar",
    "satirizaréis": "satirizar",
    "satirizaría": "satirizar",
    "satirizaríais": "satirizar",
    "satirizaríamos": "satirizar",
    "satirizarían": "satirizar",
    "satirizarías": "satirizar",
    "satirizas": "satirizar",
    "satirizase": "satirizar",
    "satirizaseis": "satirizar",
    "satirizasen": "satirizar",
    "satirizases": "satirizar",
    "satirizaste": "satirizar",
    "satirizasteis": "satirizar",
    "satirizo": "satirizar",
    "satirizábamos": "satirizar",
    "satirizáis": "satirizar",
    "satirizáramos": "satirizar",
    "satirizáremos": "satirizar",
    "satirizásemos": "satirizar",
    "satirizó": "satirizar",
    "satisdaciones": "satisdación",
    "satisfacciones": "satisfacción",
    "satisface": "satisfacer",
    "satisfaced": "satisfacer",
    "satisfacemos": "satisfacer",
    "satisfacen": "satisfacer",
    "satisfaces": "satisfacer",
    "satisfaciendo": "satisfacer",
    "satisfactoria": "satisfactorio",
    "satisfactorias": "satisfactorio",
    "satisfactorios": "satisfactorio",
    "satisfacéis": "satisfacer",
    "satisfacía": "satisfacer",
    "satisfacíais": "satisfacer",
    "satisfacíamos": "satisfacer",
    "satisfacían": "satisfacer",
    "satisfacías": "satisfacer",
    "satisfaga": "satisfacer",
    "satisfagamos": "satisfacer",
    "satisfagan": "satisfacer",
    "satisfagas": "satisfacer",
    "satisfago": "satisfacer",
    "satisfagáis": "satisfacer",
    "satisfaremos": "satisfacer",
    "satisfará": "satisfacer",
    "satisfarán": "satisfacer",
    "satisfarás": "satisfacer",
    "satisfaré": "satisfacer",
    "satisfaréis": "satisfacer",
    "satisfaría": "satisfacer",
    "satisfaríais": "satisfacer",
    "satisfaríamos": "satisfacer",
    "satisfarían": "satisfacer",
    "satisfarías": "satisfacer",
    "satisfaz": "satisfacer",
    "satisfecha": "satisfacer",
    "satisfechas": "satisfacer",
    "satisfecho": "satisfacer",
    "satisfechos": "satisfacer",
    "satisfice": "satisfacer",
    "satisficiera": "satisfacer",
    "satisficierais": "satisfacer",
    "satisficieran": "satisfacer",
    "satisficieras": "satisfacer",
    "satisficiere": "satisfacer",
    "satisficiereis": "satisfacer",
    "satisficieren": "satisfacer",
    "satisficieres": "satisfacer",
    "satisficieron": "satisfacer",
    "satisficiese": "satisfacer",
    "satisficieseis": "satisfacer",
    "satisficiesen": "satisfacer",
    "satisficieses": "satisfacer",
    "satisficimos": "satisfacer",
    "satisficiste": "satisfacer",
    "satisficisteis": "satisfacer",
    "satisficiéramos": "satisfacer",
    "satisficiéremos": "satisfacer",
    "satisficiésemos": "satisfacer",
    "satisfizo": "satisfacer",
    "sativa": "sativo",
    "sativas": "sativo",
    "sativos": "sativo",
    "satrapías": "satrapía",
    "satura": "saturar",
    "saturaba": "saturar",
    "saturabais": "saturar",
    "saturaban": "saturar",
    "saturabas": "saturar",
    "saturables": "saturable",
    "saturaciones": "saturación",
    "saturad": "saturar",
    "saturada": "saturar",
    "saturadas": "saturar",
    "saturado": "saturar",
    "saturadores": "saturador",
    "saturados": "saturar",
    "saturamos": "saturar",
    "saturan": "saturar",
    "saturando": "saturar",
    "saturantes": "saturante",
    "saturara": "saturar",
    "saturarais": "saturar",
    "saturaran": "saturar",
    "saturaras": "saturar",
    "saturare": "saturar",
    "saturareis": "saturar",
    "saturaremos": "saturar",
    "saturaren": "saturar",
    "saturares": "saturar",
    "saturaron": "saturar",
    "saturará": "saturar",
    "saturarán": "saturar",
    "saturarás": "saturar",
    "saturaré": "saturar",
    "saturaréis": "saturar",
    "saturaría": "saturar",
    "saturaríais": "saturar",
    "saturaríamos": "saturar",
    "saturarían": "saturar",
    "saturarías": "saturar",
    "saturas": "saturar",
    "saturase": "saturar",
    "saturaseis": "saturar",
    "saturasen": "saturar",
    "saturases": "saturar",
    "saturaste": "saturar",
    "saturasteis": "saturar",
    "sature": "saturar",
    "saturemos": "saturar",
    "saturen": "saturar",
    "satures": "saturar",
    "saturnales": "saturnal",
    "saturnia": "saturnio",
    "saturnias": "saturnio",
    "saturnina": "saturnino",
    "saturninas": "saturnino",
    "saturninos": "saturnino",
    "saturnios": "saturnio",
    "saturnismos": "saturnismo",
    "saturo": "saturar",
    "saturábamos": "saturar",
    "saturáis": "saturar",
    "saturáramos": "saturar",
    "saturáremos": "saturar",
    "saturásemos": "saturar",
    "saturé": "saturar",
    "saturéis": "saturar",
    "saturó": "saturar",
    "satánica": "satánico",
    "satánicas": "satánico",
    "satánicos": "satánico",
    "satélites": "satélite",
    "satírica": "satírico",
    "satíricas": "satírico",
    "satíricos": "satírico",
    "saucedales": "saucedal",
    "saucedas": "sauceda",
    "sauceras": "saucera",
    "sauces": "sauce",
    "saudades": "saudade",
    "saudies": "saudí",
    "saudis": "saudí",
    "sauditas": "saudita",
    "saudíes": "saudí",
    "saudís": "saudí",
    "saunas": "sauna",
    "saureles": "saurel",
    "sauria": "saurio",
    "saurias": "saurio",
    "saurios": "saurio",
    "sauserías": "sausería",
    "saussuritas": "saussurita",
    "sautores": "sautor",
    "sauz": "sauce",
    "sauzales": "sauzal",
    "sauzgatillos": "sauzgatillo",
    "savias": "savia",
    "saxifragácea": "saxifragáceo",
    "saxifragáceas": "saxifragáceo",
    "saxifragáceos": "saxifragáceo",
    "saxofones": "saxofón",
    "saxofonistas": "saxofonista",
    "saxos": "saxo",
    "saxátiles": "saxátil",
    "saxícola": "saxícolo",
    "saxícolas": "saxícolo",
    "saxícolos": "saxícolo",
    "saxífragas": "saxífraga",
    "saxófonos": "saxófono",
    "sayales": "sayal",
    "sayaletes": "sayalete",
    "sayas": "saya",
    "sayonas": "sayona",
    "sayones": "sayón",
    "sayos": "sayo",
    "sazona": "sazonar",
    "sazonaba": "sazonar",
    "sazonabais": "sazonar",
    "sazonaban": "sazonar",
    "sazonabas": "sazonar",
    "sazonad": "sazonar",
    "sazonada": "sazonar",
    "sazonadas": "sazonar",
    "sazonado": "sazonar",
    "sazonadora": "sazonador",
    "sazonadoras": "sazonador",
    "sazonadores": "sazonador",
    "sazonados": "sazonar",
    "sazonamos": "sazonar",
    "sazonan": "sazonar",
    "sazonando": "sazonar",
    "sazonara": "sazonar",
    "sazonarais": "sazonar",
    "sazonaran": "sazonar",
    "sazonaras": "sazonar",
    "sazonare": "sazonar",
    "sazonareis": "sazonar",
    "sazonaremos": "sazonar",
    "sazonaren": "sazonar",
    "sazonares": "sazonar",
    "sazonaron": "sazonar",
    "sazonará": "sazonar",
    "sazonarán": "sazonar",
    "sazonarás": "sazonar",
    "sazonaré": "sazonar",
    "sazonaréis": "sazonar",
    "sazonaría": "sazonar",
    "sazonaríais": "sazonar",
    "sazonaríamos": "sazonar",
    "sazonarían": "sazonar",
    "sazonarías": "sazonar",
    "sazonas": "sazonar",
    "sazonase": "sazonar",
    "sazonaseis": "sazonar",
    "sazonasen": "sazonar",
    "sazonases": "sazonar",
    "sazonaste": "sazonar",
    "sazonasteis": "sazonar",
    "sazone": "sazonar",
    "sazonemos": "sazonar",
    "sazonen": "sazonar",
    "sazones": "sazonar",
    "sazono": "sazonar",
    "sazonábamos": "sazonar",
    "sazonáis": "sazonar",
    "sazonáramos": "sazonar",
    "sazonáremos": "sazonar",
    "sazonásemos": "sazonar",
    "sazoné": "sazonar",
    "sazonéis": "sazonar",
    "sazonó": "sazonar",
    "saína": "sainar",
    "saínan": "sainar",
    "saínas": "sainar",
    "saíne": "sainar",
    "saínen": "sainar",
    "saínes": "sainar",
    "saíno": "sainar",
    "sañas": "saña",
    "sañosa": "sañoso",
    "sañosas": "sañoso",
    "sañosos": "sañoso",
    "sañuda": "sañudo",
    "sañudas": "sañudo",
    "sañudos": "sañudo",
    "saúcos": "saúco",
    "scandios": "scandio",
    "scannerses": "scanner",
    "schedulers": "scheduler",
    "scheelitas": "scheelita",
    "scherzos": "scherzo",
    "schillings": "schilling",
    "schoepitas": "schoepita",
    "schroeckingeritas": "schroeckingerita",
    "schubertiana": "schubertiano",
    "schubertianas": "schubertiano",
    "schubertianos": "schubertiano",
    "scoops": "scoop",
    "scooteres": "scooter",
    "scooters": "scooter",
    "scorings": "scoring",
    "scotches": "scotch",
    "scotchs": "scotch",
    "scraperes": "scraper",
    "scrapers": "scraper",
    "scriptgirls": "scriptgirl",
    "scripts": "script",
    "sculls": "scull",
    "sea": "ser",
    "seamos": "ser",
    "sean": "ser",
    "seas": "ser",
    "sebacatos": "sebacato",
    "sebaconitrilos": "sebaconitrilo",
    "sebastianismos": "sebastianismo",
    "sebes": "sebe",
    "sebestenes": "sebestén",
    "sebillos": "sebillo",
    "seborreas": "seborrea",
    "seborreica": "seborreico",
    "seborreicas": "seborreico",
    "seborreicos": "seborreico",
    "sebos": "sebo",
    "sebosa": "seboso",
    "sebosas": "seboso",
    "sebosos": "seboso",
    "sebácea": "sebáceo",
    "sebáceas": "sebáceo",
    "sebáceos": "sebáceo",
    "sebácica": "sebácico",
    "sebácicas": "sebácico",
    "sebácicos": "sebácico",
    "seca": "seco",
    "secaba": "secar",
    "secabais": "secar",
    "secaban": "secar",
    "secabas": "secar",
    "secad": "secar",
    "secada": "secar",
    "secadales": "secadal",
    "secadas": "secar",
    "secadera": "secadero",
    "secaderas": "secadero",
    "secaderos": "secadero",
    "secadillos": "secadillo",
    "secado": "secar",
    "secadora": "secador",
    "secadoras": "secadora",
    "secadores": "secador",
    "secados": "secar",
    "secadía": "secadío",
    "secadías": "secadío",
    "secadíos": "secadío",
    "secamientos": "secamiento",
    "secamos": "secar",
    "secan": "secar",
    "secando": "secar",
    "secanos": "secano",
    "secansas": "secansa",
    "secantes": "secante",
    "secara": "secar",
    "secarais": "secar",
    "secarales": "secaral",
    "secaran": "secar",
    "secaras": "secar",
    "secare": "secar",
    "secareis": "secar",
    "secaremos": "secar",
    "secaren": "secar",
    "secares": "secar",
    "secaron": "secar",
    "secará": "secar",
    "secarán": "secar",
    "secarás": "secar",
    "secaré": "secar",
    "secaréis": "secar",
    "secaría": "secar",
    "secaríais": "secar",
    "secaríamos": "secar",
    "secarían": "secar",
    "secarías": "secar",
    "secas": "seco",
    "secase": "secar",
    "secaseis": "secar",
    "secasen": "secar",
    "secases": "secar",
    "secaste": "secar",
    "secasteis": "secar",
    "secatona": "secatón",
    "secatonas": "secatón",
    "secatones": "secatón",
    "secaturas": "secatura",
    "secciona": "seccionar",
    "seccionaba": "seccionar",
    "seccionabais": "seccionar",
    "seccionaban": "seccionar",
    "seccionabas": "seccionar",
    "seccionad": "seccionar",
    "seccionada": "seccionar",
    "seccionadas": "seccionar",
    "seccionado": "seccionar",
    "seccionadora": "seccionador",
    "seccionadoras": "seccionador",
    "seccionadores": "seccionador",
    "seccionados": "seccionar",
    "seccionales": "seccional",
    "seccionamos": "seccionar",
    "seccionan": "seccionar",
    "seccionando": "seccionar",
    "seccionara": "seccionar",
    "seccionarais": "seccionar",
    "seccionaran": "seccionar",
    "seccionaras": "seccionar",
    "seccionare": "seccionar",
    "seccionareis": "seccionar",
    "seccionaremos": "seccionar",
    "seccionaren": "seccionar",
    "seccionares": "seccionar",
    "seccionaron": "seccionar",
    "seccionará": "seccionar",
    "seccionarán": "seccionar",
    "seccionarás": "seccionar",
    "seccionaré": "seccionar",
    "seccionaréis": "seccionar",
    "seccionaría": "seccionar",
    "seccionaríais": "seccionar",
    "seccionaríamos": "seccionar",
    "seccionarían": "seccionar",
    "seccionarías": "seccionar",
    "seccionas": "seccionar",
    "seccionase": "seccionar",
    "seccionaseis": "seccionar",
    "seccionasen": "seccionar",
    "seccionases": "seccionar",
    "seccionaste": "seccionar",
    "seccionasteis": "seccionar",
    "seccione": "seccionar",
    "seccionemos": "seccionar",
    "seccionen": "seccionar",
    "secciones": "seccionar",
    "secciono": "seccionar",
    "seccionábamos": "seccionar",
    "seccionáis": "seccionar",
    "seccionáramos": "seccionar",
    "seccionáremos": "seccionar",
    "seccionásemos": "seccionar",
    "seccioné": "seccionar",
    "seccionéis": "seccionar",
    "seccionó": "seccionar",
    "secesiones": "secesión",
    "secesionismos": "secesionismo",
    "secesionistas": "secesionista",
    "secesos": "seceso",
    "seco": "secar",
    "secobarbitales": "secobarbital",
    "secos": "seco",
    "secoyas": "secoya",
    "secreciones": "secreción",
    "secreta": "secreto",
    "secretaba": "secretar",
    "secretabais": "secretar",
    "secretaban": "secretar",
    "secretabas": "secretar",
    "secretad": "secretar",
    "secretada": "secretar",
    "secretadas": "secretar",
    "secretado": "secretar",
    "secretados": "secretar",
    "secretamos": "secretar",
    "secretan": "secretar",
    "secretando": "secretar",
    "secretara": "secretar",
    "secretarais": "secretar",
    "secretaran": "secretar",
    "secretaras": "secretar",
    "secretare": "secretar",
    "secretareis": "secretar",
    "secretaremos": "secretar",
    "secretaren": "secretar",
    "secretares": "secretar",
    "secretaria": "secretario",
    "secretariados": "secretariado",
    "secretarias": "secretario",
    "secretarios": "secretario",
    "secretaron": "secretar",
    "secretará": "secretar",
    "secretarán": "secretar",
    "secretarás": "secretar",
    "secretaré": "secretar",
    "secretaréis": "secretar",
    "secretaría": "secretar",
    "secretaríais": "secretar",
    "secretaríamos": "secretar",
    "secretarían": "secretar",
    "secretarías": "secretaría",
    "secretas": "secreto",
    "secretase": "secretar",
    "secretaseis": "secretar",
    "secretasen": "secretar",
    "secretases": "secretar",
    "secretaste": "secretar",
    "secretasteis": "secretar",
    "secrete": "secretar",
    "secretea": "secretear",
    "secreteaba": "secretear",
    "secreteabais": "secretear",
    "secreteaban": "secretear",
    "secreteabas": "secretear",
    "secretead": "secretear",
    "secreteada": "secretear",
    "secreteadas": "secretear",
    "secreteado": "secretear",
    "secreteados": "secretear",
    "secreteamos": "secretear",
    "secretean": "secretear",
    "secreteando": "secretear",
    "secreteara": "secretear",
    "secretearais": "secretear",
    "secretearan": "secretear",
    "secretearas": "secretear",
    "secreteare": "secretear",
    "secreteareis": "secretear",
    "secretearemos": "secretear",
    "secretearen": "secretear",
    "secreteares": "secretear",
    "secretearon": "secretear",
    "secreteará": "secretear",
    "secretearán": "secretear",
    "secretearás": "secretear",
    "secretearé": "secretear",
    "secretearéis": "secretear",
    "secretearía": "secretear",
    "secretearíais": "secretear",
    "secretearíamos": "secretear",
    "secretearían": "secretear",
    "secretearías": "secretear",
    "secreteas": "secretear",
    "secretease": "secretear",
    "secreteaseis": "secretear",
    "secreteasen": "secretear",
    "secreteases": "secretear",
    "secreteaste": "secretear",
    "secreteasteis": "secretear",
    "secretee": "secretear",
    "secreteemos": "secretear",
    "secreteen": "secretear",
    "secretees": "secretear",
    "secretemos": "secretar",
    "secreten": "secretar",
    "secreteo": "secretear",
    "secreteos": "secreteo",
    "secreteres": "secreter",
    "secretes": "secretar",
    "secreteábamos": "secretear",
    "secreteáis": "secretear",
    "secreteáramos": "secretear",
    "secreteáremos": "secretear",
    "secreteásemos": "secretear",
    "secreteé": "secretear",
    "secreteéis": "secretear",
    "secreteó": "secretear",
    "secretismos": "secretismo",
    "secretistas": "secretista",
    "secreto": "secretar",
    "secretora": "secretor",
    "secretoras": "secretor",
    "secretores": "secretor",
    "secretoria": "secretorio",
    "secretorias": "secretorio",
    "secretorios": "secretorio",
    "secretos": "secreto",
    "secretábamos": "secretar",
    "secretáis": "secretar",
    "secretáramos": "secretar",
    "secretáremos": "secretar",
    "secretásemos": "secretar",
    "secreté": "secretar",
    "secretéis": "secretar",
    "secretó": "secretar",
    "sectadora": "sectador",
    "sectadoras": "sectador",
    "sectadores": "sectador",
    "sectaria": "sectario",
    "sectarias": "sectario",
    "sectarios": "sectario",
    "sectarismos": "sectarismo",
    "sectas": "secta",
    "sectiles": "sectil",
    "sectores": "sector",
    "sectoriales": "sectorial",
    "sectorizaciones": "sectorización",
    "sectorizaciónes": "sectorización",
    "secuaces": "secuaz",
    "secuana": "secuano",
    "secuanas": "secuano",
    "secuanos": "secuano",
    "secuelas": "secuela",
    "secuencia": "secuenciar",
    "secuenciaba": "secuenciar",
    "secuenciabais": "secuenciar",
    "secuenciaban": "secuenciar",
    "secuenciabas": "secuenciar",
    "secuenciaciones": "secuenciación",
    "secuenciad": "secuenciar",
    "secuenciada": "secuenciar",
    "secuenciadas": "secuenciar",
    "secuenciado": "secuenciar",
    "secuenciadores": "secuenciador",
    "secuenciados": "secuenciar",
    "secuenciales": "secuencial",
    "secuencializaciones": "secuencialización",
    "secuenciamos": "secuenciar",
    "secuencian": "secuenciar",
    "secuenciando": "secuenciar",
    "secuenciara": "secuenciar",
    "secuenciarais": "secuenciar",
    "secuenciaran": "secuenciar",
    "secuenciaras": "secuenciar",
    "secuenciare": "secuenciar",
    "secuenciareis": "secuenciar",
    "secuenciaremos": "secuenciar",
    "secuenciaren": "secuenciar",
    "secuenciares": "secuenciar",
    "secuenciaron": "secuenciar",
    "secuenciará": "secuenciar",
    "secuenciarán": "secuenciar",
    "secuenciarás": "secuenciar",
    "secuenciaré": "secuenciar",
    "secuenciaréis": "secuenciar",
    "secuenciaría": "secuenciar",
    "secuenciaríais": "secuenciar",
    "secuenciaríamos": "secuenciar",
    "secuenciarían": "secuenciar",
    "secuenciarías": "secuenciar",
    "secuencias": "secuenciar",
    "secuenciase": "secuenciar",
    "secuenciaseis": "secuenciar",
    "secuenciasen": "secuenciar",
    "secuenciases": "secuenciar",
    "secuenciaste": "secuenciar",
    "secuenciasteis": "secuenciar",
    "secuencie": "secuenciar",
    "secuenciemos": "secuenciar",
    "secuencien": "secuenciar",
    "secuencies": "secuenciar",
    "secuencio": "secuenciar",
    "secuenciábamos": "secuenciar",
    "secuenciáis": "secuenciar",
    "secuenciáramos": "secuenciar",
    "secuenciáremos": "secuenciar",
    "secuenciásemos": "secuenciar",
    "secuencié": "secuenciar",
    "secuenciéis": "secuenciar",
    "secuenció": "secuenciar",
    "secuestra": "secuestrar",
    "secuestraba": "secuestrar",
    "secuestrabais": "secuestrar",
    "secuestraban": "secuestrar",
    "secuestrabas": "secuestrar",
    "secuestraciones": "secuestración",
    "secuestrad": "secuestrar",
    "secuestrada": "secuestrar",
    "secuestradas": "secuestrar",
    "secuestrado": "secuestrar",
    "secuestradora": "secuestrador",
    "secuestradoras": "secuestrador",
    "secuestradores": "secuestrador",
    "secuestrados": "secuestrar",
    "secuestramos": "secuestrar",
    "secuestran": "secuestrar",
    "secuestrando": "secuestrar",
    "secuestrantes": "secuestrante",
    "secuestrara": "secuestrar",
    "secuestrarais": "secuestrar",
    "secuestraran": "secuestrar",
    "secuestraras": "secuestrar",
    "secuestrare": "secuestrar",
    "secuestrareis": "secuestrar",
    "secuestraremos": "secuestrar",
    "secuestraren": "secuestrar",
    "secuestrares": "secuestrar",
    "secuestraria": "secuestrario",
    "secuestrarias": "secuestrario",
    "secuestrarios": "secuestrario",
    "secuestraron": "secuestrar",
    "secuestrará": "secuestrar",
    "secuestrarán": "secuestrar",
    "secuestrarás": "secuestrar",
    "secuestraré": "secuestrar",
    "secuestraréis": "secuestrar",
    "secuestraría": "secuestrar",
    "secuestraríais": "secuestrar",
    "secuestraríamos": "secuestrar",
    "secuestrarían": "secuestrar",
    "secuestrarías": "secuestrar",
    "secuestras": "secuestrar",
    "secuestrase": "secuestrar",
    "secuestraseis": "secuestrar",
    "secuestrasen": "secuestrar",
    "secuestrases": "secuestrar",
    "secuestraste": "secuestrar",
    "secuestrasteis": "secuestrar",
    "secuestre": "secuestrar",
    "secuestremos": "secuestrar",
    "secuestren": "secuestrar",
    "secuestres": "secuestrar",
    "secuestro": "secuestrar",
    "secuestros": "secuestro",
    "secuestrábamos": "secuestrar",
    "secuestráis": "secuestrar",
    "secuestráramos": "secuestrar",
    "secuestráremos": "secuestrar",
    "secuestrásemos": "secuestrar",
    "secuestré": "secuestrar",
    "secuestréis": "secuestrar",
    "secuestró": "secuestrar",
    "seculares": "secular",
    "secularice": "secularizar",
    "secularicemos": "secularizar",
    "secularicen": "secularizar",
    "secularices": "secularizar",
    "secularicé": "secularizar",
    "secularicéis": "secularizar",
    "seculariza": "secularizar",
    "secularizaba": "secularizar",
    "secularizabais": "secularizar",
    "secularizaban": "secularizar",
    "secularizabas": "secularizar",
    "secularizaciones": "secularización",
    "secularizad": "secularizar",
    "secularizada": "secularizar",
    "secularizadas": "secularizar",
    "secularizado": "secularizar",
    "secularizados": "secularizar",
    "secularizamos": "secularizar",
    "secularizan": "secularizar",
    "secularizando": "secularizar",
    "secularizara": "secularizar",
    "secularizarais": "secularizar",
    "secularizaran": "secularizar",
    "secularizaras": "secularizar",
    "secularizare": "secularizar",
    "secularizareis": "secularizar",
    "secularizaremos": "secularizar",
    "secularizaren": "secularizar",
    "secularizares": "secularizar",
    "secularizaron": "secularizar",
    "secularizará": "secularizar",
    "secularizarán": "secularizar",
    "secularizarás": "secularizar",
    "secularizaré": "secularizar",
    "secularizaréis": "secularizar",
    "secularizaría": "secularizar",
    "secularizaríais": "secularizar",
    "secularizaríamos": "secularizar",
    "secularizarían": "secularizar",
    "secularizarías": "secularizar",
    "secularizas": "secularizar",
    "secularizase": "secularizar",
    "secularizaseis": "secularizar",
    "secularizasen": "secularizar",
    "secularizases": "secularizar",
    "secularizaste": "secularizar",
    "secularizasteis": "secularizar",
    "secularizo": "secularizar",
    "secularizábamos": "secularizar",
    "secularizáis": "secularizar",
    "secularizáramos": "secularizar",
    "secularizáremos": "secularizar",
    "secularizásemos": "secularizar",
    "secularizó": "secularizar",
    "seculars": "secular",
    "secunda": "secundar",
    "secundaba": "secundar",
    "secundabais": "secundar",
    "secundaban": "secundar",
    "secundabas": "secundar",
    "secundad": "secundar",
    "secundada": "secundar",
    "secundadas": "secundar",
    "secundado": "secundar",
    "secundados": "secundar",
    "secundamos": "secundar",
    "secundan": "secundar",
    "secundando": "secundar",
    "secundara": "secundar",
    "secundarais": "secundar",
    "secundaran": "secundar",
    "secundaras": "secundar",
    "secundare": "secundar",
    "secundareis": "secundar",
    "secundaremos": "secundar",
    "secundaren": "secundar",
    "secundares": "secundar",
    "secundaria": "secundario",
    "secundarias": "secundario",
    "secundarios": "secundario",
    "secundaron": "secundar",
    "secundará": "secundar",
    "secundarán": "secundar",
    "secundarás": "secundar",
    "secundaré": "secundar",
    "secundaréis": "secundar",
    "secundaría": "secundar",
    "secundaríais": "secundar",
    "secundaríamos": "secundar",
    "secundarían": "secundar",
    "secundarías": "secundar",
    "secundas": "secundar",
    "secundase": "secundar",
    "secundaseis": "secundar",
    "secundasen": "secundar",
    "secundases": "secundar",
    "secundaste": "secundar",
    "secundasteis": "secundar",
    "secunde": "secundar",
    "secundemos": "secundar",
    "secunden": "secundar",
    "secundes": "secundar",
    "secundo": "secundar",
    "secundábamos": "secundar",
    "secundáis": "secundar",
    "secundáramos": "secundar",
    "secundáremos": "secundar",
    "secundásemos": "secundar",
    "secundé": "secundar",
    "secundéis": "secundar",
    "secundíparas": "secundípara",
    "secundó": "secundar",
    "secuoyas": "secuoya",
    "securas": "secura",
    "securitizaciones": "securitización",
    "secábamos": "secar",
    "secáis": "secar",
    "secáramos": "secar",
    "secáremos": "secar",
    "secásemos": "secar",
    "secó": "secar",
    "sed": "ser",
    "seda": "sedar",
    "sedaba": "sedar",
    "sedabais": "sedar",
    "sedaban": "sedar",
    "sedabas": "sedar",
    "sedaciones": "sedación",
    "sedad": "sedar",
    "sedada": "sedar",
    "sedadas": "sedar",
    "sedaderas": "sedadera",
    "sedado": "sedar",
    "sedados": "sedar",
    "sedales": "sedal",
    "sedalinas": "sedalina",
    "sedamos": "sedar",
    "sedan": "sedar",
    "sedando": "sedar",
    "sedanes": "sedán",
    "sedantes": "sedante",
    "sedara": "sedar",
    "sedarais": "sedar",
    "sedaran": "sedar",
    "sedaras": "sedar",
    "sedare": "sedar",
    "sedareis": "sedar",
    "sedaremos": "sedar",
    "sedaren": "sedar",
    "sedares": "sedar",
    "sedaron": "sedar",
    "sedará": "sedar",
    "sedarán": "sedar",
    "sedarás": "sedar",
    "sedaré": "sedar",
    "sedaréis": "sedar",
    "sedaría": "sedar",
    "sedaríais": "sedar",
    "sedaríamos": "sedar",
    "sedarían": "sedar",
    "sedarías": "sedar",
    "sedas": "sedar",
    "sedase": "sedar",
    "sedaseis": "sedar",
    "sedasen": "sedar",
    "sedases": "sedar",
    "sedaste": "sedar",
    "sedasteis": "sedar",
    "sedativa": "sedativo",
    "sedativas": "sedativo",
    "sedativos": "sedativo",
    "sede": "sedar",
    "sedea": "sedear",
    "sedeaba": "sedear",
    "sedeabais": "sedear",
    "sedeaban": "sedear",
    "sedeabas": "sedear",
    "sedead": "sedear",
    "sedeada": "sedear",
    "sedeadas": "sedear",
    "sedeado": "sedear",
    "sedeados": "sedear",
    "sedeamos": "sedear",
    "sedean": "sedear",
    "sedeando": "sedear",
    "sedeara": "sedear",
    "sedearais": "sedear",
    "sedearan": "sedear",
    "sedearas": "sedear",
    "sedeare": "sedear",
    "sedeareis": "sedear",
    "sedearemos": "sedear",
    "sedearen": "sedear",
    "sedeares": "sedear",
    "sedearon": "sedear",
    "sedeará": "sedear",
    "sedearán": "sedear",
    "sedearás": "sedear",
    "sedearé": "sedear",
    "sedearéis": "sedear",
    "sedearía": "sedear",
    "sedearíais": "sedear",
    "sedearíamos": "sedear",
    "sedearían": "sedear",
    "sedearías": "sedear",
    "sedeas": "sedear",
    "sedease": "sedear",
    "sedeaseis": "sedear",
    "sedeasen": "sedear",
    "sedeases": "sedear",
    "sedeaste": "sedear",
    "sedeasteis": "sedear",
    "sedee": "sedear",
    "sedeemos": "sedear",
    "sedeen": "sedear",
    "sedees": "sedear",
    "sedemos": "sedar",
    "seden": "sedar",
    "sedentaria": "sedentario",
    "sedentarias": "sedentario",
    "sedentarios": "sedentario",
    "sedentarismos": "sedentarismo",
    "sedentes": "sedente",
    "sedeo": "sedear",
    "sedera": "sedero",
    "sederas": "sedero",
    "sederos": "sedero",
    "sederías": "sedería",
    "sedes": "sede",
    "sedeábamos": "sedear",
    "sedeáis": "sedear",
    "sedeáramos": "sedear",
    "sedeáremos": "sedear",
    "sedeásemos": "sedear",
    "sedeé": "sedear",
    "sedeéis": "sedear",
    "sedeña": "sedeño",
    "sedeñas": "sedeño",
    "sedeños": "sedeño",
    "sedeó": "sedear",
    "sedicentes": "sedicente",
    "sediciones": "sedición",
    "sediciosa": "sedicioso",
    "sediciosas": "sedicioso",
    "sediciosos": "sedicioso",
    "sedienta": "sediento",
    "sedientas": "sediento",
    "sedientos": "sediento",
    "sedimenta": "sedimentar",
    "sedimentaba": "sedimentar",
    "sedimentabais": "sedimentar",
    "sedimentaban": "sedimentar",
    "sedimentabas": "sedimentar",
    "sedimentables": "sedimentable",
    "sedimentaciones": "sedimentación",
    "sedimentad": "sedimentar",
    "sedimentada": "sedimentar",
    "sedimentadas": "sedimentar",
    "sedimentado": "sedimentar",
    "sedimentados": "sedimentar",
    "sedimentamos": "sedimentar",
    "sedimentan": "sedimentar",
    "sedimentando": "sedimentar",
    "sedimentara": "sedimentar",
    "sedimentarais": "sedimentar",
    "sedimentaran": "sedimentar",
    "sedimentaras": "sedimentar",
    "sedimentare": "sedimentar",
    "sedimentareis": "sedimentar",
    "sedimentaremos": "sedimentar",
    "sedimentaren": "sedimentar",
    "sedimentares": "sedimentar",
    "sedimentaria": "sedimentario",
    "sedimentarias": "sedimentario",
    "sedimentarios": "sedimentario",
    "sedimentaron": "sedimentar",
    "sedimentará": "sedimentar",
    "sedimentarán": "sedimentar",
    "sedimentarás": "sedimentar",
    "sedimentaré": "sedimentar",
    "sedimentaréis": "sedimentar",
    "sedimentaría": "sedimentar",
    "sedimentaríais": "sedimentar",
    "sedimentaríamos": "sedimentar",
    "sedimentarían": "sedimentar",
    "sedimentarías": "sedimentar",
    "sedimentas": "sedimentar",
    "sedimentase": "sedimentar",
    "sedimentaseis": "sedimentar",
    "sedimentasen": "sedimentar",
    "sedimentases": "sedimentar",
    "sedimentaste": "sedimentar",
    "sedimentasteis": "sedimentar",
    "sedimente": "sedimentar",
    "sedimentemos": "sedimentar",
    "sedimenten": "sedimentar",
    "sedimentes": "sedimentar",
    "sedimento": "sedimentar",
    "sedimentologías": "sedimentología",
    "sedimentológica": "sedimentológico",
    "sedimentológicas": "sedimentológico",
    "sedimentológicos": "sedimentológico",
    "sedimentos": "sedimento",
    "sedimentábamos": "sedimentar",
    "sedimentáis": "sedimentar",
    "sedimentáramos": "sedimentar",
    "sedimentáremos": "sedimentar",
    "sedimentásemos": "sedimentar",
    "sedimenté": "sedimentar",
    "sedimentéis": "sedimentar",
    "sedimentó": "sedimentar",
    "sedimentóloga": "sedimentólogo",
    "sedimentólogas": "sedimentólogo",
    "sedimentólogos": "sedimentólogo",
    "sedo": "sedar",
    "sedosa": "sedoso",
    "sedosas": "sedoso",
    "sedosos": "sedoso",
    "seducciones": "seducción",
    "seduce": "seducir",
    "seducen": "seducir",
    "seduces": "seducir",
    "seducibles": "seducible",
    "seducid": "seducir",
    "seducida": "seducir",
    "seducidas": "seducir",
    "seducido": "seducir",
    "seducidos": "seducir",
    "seduciendo": "seducir",
    "seducimos": "seducir",
    "seduciremos": "seducir",
    "seducirá": "seducir",
    "seducirán": "seducir",
    "seducirás": "seducir",
    "seduciré": "seducir",
    "seduciréis": "seducir",
    "seduciría": "seducir",
    "seduciríais": "seducir",
    "seduciríamos": "seducir",
    "seducirían": "seducir",
    "seducirías": "seducir",
    "seductiva": "seductivo",
    "seductivas": "seductivo",
    "seductivos": "seductivo",
    "seductora": "seductor",
    "seductoras": "seductor",
    "seductores": "seductor",
    "seducía": "seducir",
    "seducíais": "seducir",
    "seducíamos": "seducir",
    "seducían": "seducir",
    "seducías": "seducir",
    "seducís": "seducir",
    "seduje": "seducir",
    "sedujera": "seducir",
    "sedujerais": "seducir",
    "sedujeran": "seducir",
    "sedujeras": "seducir",
    "sedujere": "seducir",
    "sedujereis": "seducir",
    "sedujeren": "seducir",
    "sedujeres": "seducir",
    "sedujeron": "seducir",
    "sedujese": "seducir",
    "sedujeseis": "seducir",
    "sedujesen": "seducir",
    "sedujeses": "seducir",
    "sedujimos": "seducir",
    "sedujiste": "seducir",
    "sedujisteis": "seducir",
    "sedujo": "seducir",
    "sedujéramos": "seducir",
    "sedujéremos": "seducir",
    "sedujésemos": "seducir",
    "seduzca": "seducir",
    "seduzcamos": "seducir",
    "seduzcan": "seducir",
    "seduzcas": "seducir",
    "seduzco": "seducir",
    "seduzcáis": "seducir",
    "sedábamos": "sedar",
    "sedáis": "sedar",
    "sedáramos": "sedar",
    "sedáremos": "sedar",
    "sedásemos": "sedar",
    "sedé": "sedar",
    "sedéis": "sedar",
    "sedó": "sedar",
    "sefarditas": "sefardita",
    "sefardíes": "sefardí",
    "sefardís": "sefardí",
    "seg": "segundo",
    "segaba": "segar",
    "segabais": "segar",
    "segaban": "segar",
    "segabas": "segar",
    "segables": "segable",
    "segad": "segar",
    "segada": "segar",
    "segadas": "segar",
    "segadera": "segadero",
    "segaderas": "segadero",
    "segaderos": "segadero",
    "segado": "segar",
    "segadora": "segador",
    "segadoras": "segadora",
    "segadores": "segador",
    "segados": "segar",
    "segamos": "segar",
    "segando": "segar",
    "segara": "segar",
    "segarais": "segar",
    "segaran": "segar",
    "segaras": "segar",
    "segare": "segar",
    "segareis": "segar",
    "segaremos": "segar",
    "segaren": "segar",
    "segares": "segar",
    "segaron": "segar",
    "segará": "segar",
    "segarán": "segar",
    "segarás": "segar",
    "segaré": "segar",
    "segaréis": "segar",
    "segaría": "segar",
    "segaríais": "segar",
    "segaríamos": "segar",
    "segarían": "segar",
    "segarías": "segar",
    "segase": "segar",
    "segaseis": "segar",
    "segasen": "segar",
    "segases": "segar",
    "segaste": "segar",
    "segasteis": "segar",
    "segazones": "segazón",
    "seglara": "seglar",
    "seglaras": "seglar",
    "seglares": "seglar",
    "segmenta": "segmentar",
    "segmentaba": "segmentar",
    "segmentabais": "segmentar",
    "segmentaban": "segmentar",
    "segmentabas": "segmentar",
    "segmentaciones": "segmentación",
    "segmentad": "segmentar",
    "segmentada": "segmentar",
    "segmentadas": "segmentar",
    "segmentado": "segmentar",
    "segmentados": "segmentar",
    "segmentales": "segmental",
    "segmentamos": "segmentar",
    "segmentan": "segmentar",
    "segmentando": "segmentar",
    "segmentara": "segmentar",
    "segmentarais": "segmentar",
    "segmentaran": "segmentar",
    "segmentaras": "segmentar",
    "segmentare": "segmentar",
    "segmentareis": "segmentar",
    "segmentaremos": "segmentar",
    "segmentaren": "segmentar",
    "segmentares": "segmentar",
    "segmentaria": "segmentario",
    "segmentarias": "segmentario",
    "segmentarios": "segmentario",
    "segmentaron": "segmentar",
    "segmentará": "segmentar",
    "segmentarán": "segmentar",
    "segmentarás": "segmentar",
    "segmentaré": "segmentar",
    "segmentaréis": "segmentar",
    "segmentaría": "segmentar",
    "segmentaríais": "segmentar",
    "segmentaríamos": "segmentar",
    "segmentarían": "segmentar",
    "segmentarías": "segmentar",
    "segmentas": "segmentar",
    "segmentase": "segmentar",
    "segmentaseis": "segmentar",
    "segmentasen": "segmentar",
    "segmentases": "segmentar",
    "segmentaste": "segmentar",
    "segmentasteis": "segmentar",
    "segmente": "segmentar",
    "segmentemos": "segmentar",
    "segmenten": "segmentar",
    "segmentes": "segmentar",
    "segmento": "segmentar",
    "segmentos": "segmento",
    "segmentábamos": "segmentar",
    "segmentáis": "segmentar",
    "segmentáramos": "segmentar",
    "segmentáremos": "segmentar",
    "segmentásemos": "segmentar",
    "segmenté": "segmentar",
    "segmentéis": "segmentar",
    "segmentó": "segmentar",
    "segoviana": "segoviano",
    "segovianas": "segoviano",
    "segovianos": "segoviano",
    "segovienses": "segoviense",
    "segrega": "segregar",
    "segregaba": "segregar",
    "segregabais": "segregar",
    "segregaban": "segregar",
    "segregabas": "segregar",
    "segregables": "segregable",
    "segregaciones": "segregación",
    "segregacionismos": "segregacionismo",
    "segregacionistas": "segregacionista",
    "segregad": "segregar",
    "segregada": "segregar",
    "segregadas": "segregar",
    "segregado": "segregar",
    "segregadora": "segregador",
    "segregadoras": "segregador",
    "segregadores": "segregador",
    "segregados": "segregar",
    "segregamos": "segregar",
    "segregan": "segregar",
    "segregando": "segregar",
    "segregantes": "segregante",
    "segregara": "segregar",
    "segregarais": "segregar",
    "segregaran": "segregar",
    "segregaras": "segregar",
    "segregare": "segregar",
    "segregareis": "segregar",
    "segregaremos": "segregar",
    "segregaren": "segregar",
    "segregares": "segregar",
    "segregaron": "segregar",
    "segregará": "segregar",
    "segregarán": "segregar",
    "segregarás": "segregar",
    "segregaré": "segregar",
    "segregaréis": "segregar",
    "segregaría": "segregar",
    "segregaríais": "segregar",
    "segregaríamos": "segregar",
    "segregarían": "segregar",
    "segregarías": "segregar",
    "segregas": "segregar",
    "segregase": "segregar",
    "segregaseis": "segregar",
    "segregasen": "segregar",
    "segregases": "segregar",
    "segregaste": "segregar",
    "segregasteis": "segregar",
    "segregativa": "segregativo",
    "segregativas": "segregativo",
    "segregativos": "segregativo",
    "segrego": "segregar",
    "segregue": "segregar",
    "segreguemos": "segregar",
    "segreguen": "segregar",
    "segregues": "segregar",
    "segregué": "segregar",
    "segreguéis": "segregar",
    "segregábamos": "segregar",
    "segregáis": "segregar",
    "segregáramos": "segregar",
    "segregáremos": "segregar",
    "segregásemos": "segregar",
    "segregó": "segregar",
    "seguemos": "segar",
    "seguetas": "segueta",
    "seguid": "seguir",
    "seguida": "seguir",
    "seguidas": "seguir",
    "seguideros": "seguidero",
    "seguidillas": "seguidilla",
    "seguidismos": "seguidismo",
    "seguido": "seguir",
    "seguidora": "seguidor",
    "seguidoras": "seguidor",
    "seguidores": "seguidor",
    "seguidos": "seguir",
    "seguimientos": "seguimiento",
    "seguimos": "seguir",
    "seguiremos": "seguir",
    "seguirá": "seguir",
    "seguirán": "seguir",
    "seguirás": "seguir",
    "seguiré": "seguir",
    "seguiréis": "seguir",
    "seguiría": "seguir",
    "seguiríais": "seguir",
    "seguiríamos": "seguir",
    "seguirían": "seguir",
    "seguirías": "seguir",
    "seguiste": "seguir",
    "seguisteis": "seguir",
    "segunda": "segundar",
    "segundaba": "segundar",
    "segundabais": "segundar",
    "segundaban": "segundar",
    "segundabas": "segundar",
    "segundad": "segundar",
    "segundada": "segundar",
    "segundadas": "segundar",
    "segundado": "segundar",
    "segundados": "segundar",
    "segundamos": "segundar",
    "segundan": "segundar",
    "segundando": "segundar",
    "segundara": "segundar",
    "segundarais": "segundar",
    "segundaran": "segundar",
    "segundaras": "segundar",
    "segundare": "segundar",
    "segundareis": "segundar",
    "segundaremos": "segundar",
    "segundaren": "segundar",
    "segundares": "segundar",
    "segundaria": "segundario",
    "segundarias": "segundario",
    "segundarios": "segundario",
    "segundaron": "segundar",
    "segundará": "segundar",
    "segundarán": "segundar",
    "segundarás": "segundar",
    "segundaré": "segundar",
    "segundaréis": "segundar",
    "segundaría": "segundar",
    "segundaríais": "segundar",
    "segundaríamos": "segundar",
    "segundarían": "segundar",
    "segundarías": "segundar",
    "segundas": "segundar",
    "segundase": "segundar",
    "segundaseis": "segundar",
    "segundasen": "segundar",
    "segundases": "segundar",
    "segundaste": "segundar",
    "segundasteis": "segundar",
    "segunde": "segundar",
    "segundemos": "segundar",
    "segunden": "segundar",
    "segundera": "segundero",
    "segunderas": "segundero",
    "segunderos": "segundero",
    "segundes": "segundar",
    "segundillas": "segundilla",
    "segundillos": "segundillo",
    "segundo": "segundar",
    "segundogenituras": "segundogenitura",
    "segundogénita": "segundogénito",
    "segundogénitas": "segundogénito",
    "segundogénitos": "segundogénito",
    "segundones": "segundón",
    "segundos": "segundo",
    "segundábamos": "segundar",
    "segundáis": "segundar",
    "segundáramos": "segundar",
    "segundáremos": "segundar",
    "segundásemos": "segundar",
    "segundé": "segundar",
    "segundéis": "segundar",
    "segundó": "segundar",
    "segura": "seguro",
    "seguras": "seguro",
    "segures": "segur",
    "seguridades": "seguridad",
    "seguros": "seguro",
    "segué": "segar",
    "seguéis": "segar",
    "seguí": "seguir",
    "seguía": "seguir",
    "seguíais": "seguir",
    "seguíamos": "seguir",
    "seguían": "seguir",
    "seguías": "seguir",
    "seguís": "seguir",
    "segábamos": "segar",
    "segáis": "segar",
    "segáramos": "segar",
    "segáremos": "segar",
    "segásemos": "segar",
    "segó": "segar",
    "seisava": "seisavo",
    "seisavas": "seisavo",
    "seisavo": "6",
    "seisavos": "seisavo",
    "seiscientosava": "seiscientosavo",
    "seiscientosavas": "seiscientosavo",
    "seiscientosavo": "600",
    "seiscientosavos": "seiscientosavo",
    "seisillos": "seisillo",
    "sejes": "seje",
    "selacios": "selacio",
    "selecciona": "seleccionar",
    "seleccionaba": "seleccionar",
    "seleccionabais": "seleccionar",
    "seleccionaban": "seleccionar",
    "seleccionabas": "seleccionar",
    "seleccionables": "seleccionable",
    "seleccionad": "seleccionar",
    "seleccionada": "seleccionar",
    "seleccionadas": "seleccionar",
    "seleccionado": "seleccionar",
    "seleccionadora": "seleccionador",
    "seleccionadoras": "seleccionador",
    "seleccionadores": "seleccionador",
    "seleccionados": "seleccionar",
    "seleccionamos": "seleccionar",
    "seleccionan": "seleccionar",
    "seleccionando": "seleccionar",
    "seleccionara": "seleccionar",
    "seleccionarais": "seleccionar",
    "seleccionaran": "seleccionar",
    "seleccionaras": "seleccionar",
    "seleccionare": "seleccionar",
    "seleccionareis": "seleccionar",
    "seleccionaremos": "seleccionar",
    "seleccionaren": "seleccionar",
    "seleccionares": "seleccionar",
    "seleccionaron": "seleccionar",
    "seleccionará": "seleccionar",
    "seleccionarán": "seleccionar",
    "seleccionarás": "seleccionar",
    "seleccionaré": "seleccionar",
    "seleccionaréis": "seleccionar",
    "seleccionaría": "seleccionar",
    "seleccionaríais": "seleccionar",
    "seleccionaríamos": "seleccionar",
    "seleccionarían": "seleccionar",
    "seleccionarías": "seleccionar",
    "seleccionas": "seleccionar",
    "seleccionase": "seleccionar",
    "seleccionaseis": "seleccionar",
    "seleccionasen": "seleccionar",
    "seleccionases": "seleccionar",
    "seleccionaste": "seleccionar",
    "seleccionasteis": "seleccionar",
    "seleccione": "seleccionar",
    "seleccionemos": "seleccionar",
    "seleccionen": "seleccionar",
    "selecciones": "seleccionar",
    "seleccionistas": "seleccionista",
    "selecciono": "seleccionar",
    "seleccionábamos": "seleccionar",
    "seleccionáis": "seleccionar",
    "seleccionáramos": "seleccionar",
    "seleccionáremos": "seleccionar",
    "seleccionásemos": "seleccionar",
    "seleccioné": "seleccionar",
    "seleccionéis": "seleccionar",
    "seleccionó": "seleccionar",
    "selecta": "selecto",
    "selectas": "selecto",
    "selectinas": "selectina",
    "selectiva": "selectivo",
    "selectivas": "selectivo",
    "selectividades": "selectividad",
    "selectivos": "selectivo",
    "selectora": "selector",
    "selectoras": "selector",
    "selectores": "selector",
    "selectos": "selecto",
    "seleniatos": "seleniato",
    "selenios": "selenio",
    "seleniosa": "selenioso",
    "seleniosas": "selenioso",
    "seleniosos": "selenioso",
    "selenitas": "selenita",
    "selenitosa": "selenitoso",
    "selenitosas": "selenitoso",
    "selenitosos": "selenitoso",
    "seleniuros": "seleniuro",
    "selenocéntrica": "selenocéntrico",
    "selenocéntricas": "selenocéntrico",
    "selenocéntricos": "selenocéntrico",
    "selenodonta": "selenodonto",
    "selenodontas": "selenodonto",
    "selenodontos": "selenodonto",
    "selenografías": "selenografía",
    "selenográfica": "selenográfico",
    "selenográficas": "selenográfico",
    "selenográficos": "selenográfico",
    "selenostatos": "selenostato",
    "selenógrafa": "selenógrafo",
    "selenógrafas": "selenógrafo",
    "selenógrafos": "selenógrafo",
    "selfactinas": "selfactina",
    "sella": "sellar",
    "sellaba": "sellar",
    "sellabais": "sellar",
    "sellaban": "sellar",
    "sellabas": "sellar",
    "sellad": "sellar",
    "sellada": "sellar",
    "selladas": "sellar",
    "sellado": "sellar",
    "selladora": "sellador",
    "selladoras": "sellador",
    "selladores": "sellador",
    "sellados": "sellar",
    "selladuras": "selladura",
    "sellamos": "sellar",
    "sellan": "sellar",
    "sellando": "sellar",
    "sellara": "sellar",
    "sellarais": "sellar",
    "sellaran": "sellar",
    "sellaras": "sellar",
    "sellare": "sellar",
    "sellareis": "sellar",
    "sellaremos": "sellar",
    "sellaren": "sellar",
    "sellares": "sellar",
    "sellaron": "sellar",
    "sellará": "sellar",
    "sellarán": "sellar",
    "sellarás": "sellar",
    "sellaré": "sellar",
    "sellaréis": "sellar",
    "sellaría": "sellar",
    "sellaríais": "sellar",
    "sellaríamos": "sellar",
    "sellarían": "sellar",
    "sellarías": "sellar",
    "sellas": "sellar",
    "sellase": "sellar",
    "sellaseis": "sellar",
    "sellasen": "sellar",
    "sellases": "sellar",
    "sellaste": "sellar",
    "sellasteis": "sellar",
    "selle": "sellar",
    "sellemos": "sellar",
    "sellen": "sellar",
    "selles": "sellar",
    "sello": "sellar",
    "sellos": "sello",
    "sellábamos": "sellar",
    "selláis": "sellar",
    "selláramos": "sellar",
    "selláremos": "sellar",
    "sellásemos": "sellar",
    "sellé": "sellar",
    "selléis": "sellar",
    "selló": "sellar",
    "selsoles": "selsol",
    "selvas": "selva",
    "selvatiqueces": "selvatiquez",
    "selviculturas": "selvicultura",
    "selvosa": "selvoso",
    "selvosas": "selvoso",
    "selvosos": "selvoso",
    "selvática": "selvático",
    "selváticas": "selvático",
    "selváticos": "selvático",
    "selyuqíes": "selyuqí",
    "selyuqís": "selyuqí",
    "seláceos": "seláceo",
    "selénica": "selénico",
    "selénicas": "selénico",
    "selénicos": "selénico",
    "seléucidas": "seléucida",
    "semafórica": "semafórico",
    "semafóricas": "semafórico",
    "semafóricos": "semafórico",
    "semanadas": "semanada",
    "semanales": "semanal",
    "semanaria": "semanario",
    "semanarias": "semanario",
    "semanarios": "semanario",
    "semanas": "semana",
    "semanera": "semanero",
    "semaneras": "semanero",
    "semaneros": "semanero",
    "semanerías": "semanería",
    "semantemas": "semantema",
    "semantistas": "semantista",
    "semasiologías": "semasiología",
    "semasiológica": "semasiológico",
    "semasiológicas": "semasiológico",
    "semasiológicos": "semasiológico",
    "semblantes": "semblante",
    "semblanzas": "semblanza",
    "sembraba": "sembrar",
    "sembrabais": "sembrar",
    "sembraban": "sembrar",
    "sembrabas": "sembrar",
    "sembrad": "sembrar",
    "sembrada": "sembrar",
    "sembradas": "sembrar",
    "sembraderas": "sembradera",
    "sembrado": "sembrar",
    "sembradora": "sembrador",
    "sembradoras": "sembradora",
    "sembradores": "sembrador",
    "sembrados": "sembrar",
    "sembraduras": "sembradura",
    "sembradía": "sembradío",
    "sembradías": "sembradío",
    "sembradíos": "sembradío",
    "sembramos": "sembrar",
    "sembrando": "sembrar",
    "sembrara": "sembrar",
    "sembrarais": "sembrar",
    "sembraran": "sembrar",
    "sembraras": "sembrar",
    "sembrare": "sembrar",
    "sembrareis": "sembrar",
    "sembraremos": "sembrar",
    "sembraren": "sembrar",
    "sembrares": "sembrar",
    "sembraron": "sembrar",
    "sembrará": "sembrar",
    "sembrarán": "sembrar",
    "sembrarás": "sembrar",
    "sembraré": "sembrar",
    "sembraréis": "sembrar",
    "sembraría": "sembrar",
    "sembraríais": "sembrar",
    "sembraríamos": "sembrar",
    "sembrarían": "sembrar",
    "sembrarías": "sembrar",
    "sembrase": "sembrar",
    "sembraseis": "sembrar",
    "sembrasen": "sembrar",
    "sembrases": "sembrar",
    "sembraste": "sembrar",
    "sembrasteis": "sembrar",
    "sembremos": "sembrar",
    "sembrábamos": "sembrar",
    "sembráis": "sembrar",
    "sembráramos": "sembrar",
    "sembráremos": "sembrar",
    "sembrásemos": "sembrar",
    "sembré": "sembrar",
    "sembréis": "sembrar",
    "sembró": "sembrar",
    "semeja": "semejar",
    "semejaba": "semejar",
    "semejabais": "semejar",
    "semejaban": "semejar",
    "semejabas": "semejar",
    "semejables": "semejable",
    "semejad": "semejar",
    "semejada": "semejar",
    "semejadas": "semejar",
    "semejado": "semejar",
    "semejados": "semejar",
    "semejamos": "semejar",
    "semejan": "semejar",
    "semejando": "semejar",
    "semejantes": "semejante",
    "semejanzas": "semejanza",
    "semejara": "semejar",
    "semejarais": "semejar",
    "semejaran": "semejar",
    "semejaras": "semejar",
    "semejare": "semejar",
    "semejareis": "semejar",
    "semejaremos": "semejar",
    "semejaren": "semejar",
    "semejares": "semejar",
    "semejaron": "semejar",
    "semejará": "semejar",
    "semejarán": "semejar",
    "semejarás": "semejar",
    "semejaré": "semejar",
    "semejaréis": "semejar",
    "semejaría": "semejar",
    "semejaríais": "semejar",
    "semejaríamos": "semejar",
    "semejarían": "semejar",
    "semejarías": "semejar",
    "semejas": "semejar",
    "semejase": "semejar",
    "semejaseis": "semejar",
    "semejasen": "semejar",
    "semejases": "semejar",
    "semejaste": "semejar",
    "semejasteis": "semejar",
    "semeje": "semejar",
    "semejemos": "semejar",
    "semejen": "semejar",
    "semejes": "semejar",
    "semejo": "semejar",
    "semejábamos": "semejar",
    "semejáis": "semejar",
    "semejáramos": "semejar",
    "semejáremos": "semejar",
    "semejásemos": "semejar",
    "semejé": "semejar",
    "semejéis": "semejar",
    "semejó": "semejar",
    "sememas": "semema",
    "semenceras": "semencera",
    "sementales": "semental",
    "sementeras": "sementera",
    "sementeros": "sementero",
    "sementina": "sementino",
    "sementinas": "sementino",
    "sementinos": "sementino",
    "semestrales": "semestral",
    "semestralizaciones": "semestralización",
    "semestres": "semestre",
    "semi-invertida": "semi-invertido",
    "semi-invertidas": "semi-invertido",
    "semi-invertidos": "semi-invertido",
    "semiabierta": "semiabierto",
    "semiabiertas": "semiabierto",
    "semiabiertos": "semiabierto",
    "semiacera": "semiacero",
    "semiaceras": "semiacero",
    "semiaceros": "semiacero",
    "semiajustables": "semiajustable",
    "semialta": "semialto",
    "semialtas": "semialto",
    "semialtos": "semialto",
    "semianticadencias": "semianticadencia",
    "semianuales": "semianual",
    "semiautomática": "semiautomático",
    "semiautomáticas": "semiautomático",
    "semiautomáticos": "semiautomático",
    "semiautómatas": "semiautómata",
    "semibaterías": "semibatería",
    "semiblindada": "semiblindado",
    "semiblindadas": "semiblindado",
    "semiblindados": "semiblindado",
    "semibreves": "semibreve",
    "semicabrones": "semicabrón",
    "semicadencias": "semicadencia",
    "semicajas": "semicaja",
    "semicapros": "semicapro",
    "semicarbacidas": "semicarbacida",
    "semicarbazonas": "semicarbazona",
    "semicariótipos": "semicariótipo",
    "semicerraba": "semicerrar",
    "semicerrabais": "semicerrar",
    "semicerraban": "semicerrar",
    "semicerrabas": "semicerrar",
    "semicerrad": "semicerrar",
    "semicerrada": "semicerrar",
    "semicerradas": "semicerrar",
    "semicerrado": "semicerrar",
    "semicerrados": "semicerrar",
    "semicerramos": "semicerrar",
    "semicerrando": "semicerrar",
    "semicerrara": "semicerrar",
    "semicerrarais": "semicerrar",
    "semicerraran": "semicerrar",
    "semicerraras": "semicerrar",
    "semicerrare": "semicerrar",
    "semicerrareis": "semicerrar",
    "semicerraremos": "semicerrar",
    "semicerraren": "semicerrar",
    "semicerrares": "semicerrar",
    "semicerraron": "semicerrar",
    "semicerrará": "semicerrar",
    "semicerrarán": "semicerrar",
    "semicerrarás": "semicerrar",
    "semicerraré": "semicerrar",
    "semicerraréis": "semicerrar",
    "semicerraría": "semicerrar",
    "semicerraríais": "semicerrar",
    "semicerraríamos": "semicerrar",
    "semicerrarían": "semicerrar",
    "semicerrarías": "semicerrar",
    "semicerrase": "semicerrar",
    "semicerraseis": "semicerrar",
    "semicerrasen": "semicerrar",
    "semicerrases": "semicerrar",
    "semicerraste": "semicerrar",
    "semicerrasteis": "semicerrar",
    "semicerremos": "semicerrar",
    "semicerrábamos": "semicerrar",
    "semicerráis": "semicerrar",
    "semicerráramos": "semicerrar",
    "semicerráremos": "semicerrar",
    "semicerrásemos": "semicerrar",
    "semicerré": "semicerrar",
    "semicerréis": "semicerrar",
    "semicerró": "semicerrar",
    "semicientífica": "semicientífico",
    "semicientíficas": "semicientífico",
    "semicientíficos": "semicientífico",
    "semicierra": "semicerrar",
    "semicierran": "semicerrar",
    "semicierras": "semicerrar",
    "semicierre": "semicerrar",
    "semicierren": "semicerrar",
    "semicierres": "semicerrar",
    "semicierro": "semicerrar",
    "semicilindros": "semicilindro",
    "semicilíndrica": "semicilíndrico",
    "semicilíndricas": "semicilíndrico",
    "semicilíndricos": "semicilíndrico",
    "semicirculares": "semicircular",
    "semicircunferencias": "semicircunferencia",
    "semiclara": "semiclaro",
    "semiclaras": "semiclaro",
    "semiclaros": "semiclaro",
    "semiconductividades": "semiconductividad",
    "semiconductora": "semiconductor",
    "semiconductoras": "semiconductor",
    "semiconductores": "semiconductor",
    "semiconservas": "semiconserva",
    "semiconsonantes": "semiconsonante",
    "semiconsonántica": "semiconsonántico",
    "semiconsonánticas": "semiconsonántico",
    "semiconsonánticos": "semiconsonántico",
    "semicontinua": "semicontinuo",
    "semicontinuas": "semicontinuo",
    "semicontinuidades": "semicontinuidad",
    "semicontinuos": "semicontinuo",
    "semicoques": "semicoque",
    "semicorcheas": "semicorchea",
    "semicristalina": "semicristalino",
    "semicristalinas": "semicristalino",
    "semicristalinos": "semicristalino",
    "semicromática": "semicromático",
    "semicromáticas": "semicromático",
    "semicromáticos": "semicromático",
    "semicruzada": "semicruzado",
    "semicruzadas": "semicruzado",
    "semicruzados": "semicruzado",
    "semicultismos": "semicultismo",
    "semicírculos": "semicírculo",
    "semicúbica": "semicúbico",
    "semicúbicas": "semicúbico",
    "semicúbicos": "semicúbico",
    "semidesconocida": "semidesconocido",
    "semidesconocidas": "semidesconocido",
    "semidesconocidos": "semidesconocido",
    "semidesiertos": "semidesierto",
    "semidesnatada": "semidesnatado",
    "semidesnatadas": "semidesnatado",
    "semidesnatados": "semidesnatado",
    "semidespoblada": "semidespoblado",
    "semidespobladas": "semidespoblado",
    "semidespoblados": "semidespoblado",
    "semidieseles": "semidiesel",
    "semidifunta": "semidifunto",
    "semidifuntas": "semidifunto",
    "semidifuntos": "semidifunto",
    "semidiosa": "semidiós",
    "semidiosas": "semidiós",
    "semidioses": "semidiós",
    "semidirecta": "semidirecto",
    "semidirectas": "semidirecto",
    "semidirectos": "semidirecto",
    "semidiurna": "semidiurno",
    "semidiurnas": "semidiurno",
    "semidiurnos": "semidiurno",
    "semidiámetros": "semidiámetro",
    "semidormida": "semidormido",
    "semidormidas": "semidormido",
    "semidormidos": "semidormido",
    "semidura": "semiduro",
    "semiduras": "semiduro",
    "semiduros": "semiduro",
    "semidítonos": "semidítono",
    "semidúplexes": "semidúplex",
    "semiejes": "semieje",
    "semielaborada": "semielaborado",
    "semielaboradas": "semielaborado",
    "semielaborados": "semielaborado",
    "semielectrónicas": "semielectrónica",
    "semielementos": "semielemento",
    "semielíptica": "semielíptico",
    "semielípticas": "semielíptico",
    "semielípticos": "semielíptico",
    "semiempírica": "semiempírico",
    "semiempíricas": "semiempírico",
    "semiempíricos": "semiempírico",
    "semiesferas": "semiesfera",
    "semiesférica": "semiesférico",
    "semiesféricas": "semiesférico",
    "semiesféricos": "semiesférico",
    "semiespacios": "semiespacio",
    "semiespinosa": "semiespinoso",
    "semiespinosas": "semiespinoso",
    "semiespinosos": "semiespinoso",
    "semiestatales": "semiestatal",
    "semifaltas": "semifalta",
    "semifija": "semifijo",
    "semifijas": "semifijo",
    "semifijos": "semifijo",
    "semifinales": "semifinal",
    "semifinalistas": "semifinalista",
    "semifluida": "semifluido",
    "semifluidas": "semifluido",
    "semifluidos": "semifluido",
    "semiflósculos": "semiflósculo",
    "semiformes": "semiforme",
    "semifosforosa": "semifosforoso",
    "semifosforosas": "semifosforoso",
    "semifosforosos": "semifosforoso",
    "semifusas": "semifusa",
    "semiganchos": "semigancho",
    "semigrasa": "semigraso",
    "semigrasas": "semigraso",
    "semigrasos": "semigraso",
    "semigrupos": "semigrupo",
    "semiheterotípica": "semiheterotípico",
    "semiheterotípicas": "semiheterotípico",
    "semiheterotípicos": "semiheterotípico",
    "semihidratos": "semihidrato",
    "semihombres": "semihombre",
    "semiindirecta": "semiindirecto",
    "semiindirectas": "semiindirecto",
    "semiindirectos": "semiindirecto",
    "semiinscrita": "semiinscrito",
    "semiinscritas": "semiinscrito",
    "semiinscritos": "semiinscrito",
    "semiletales": "semiletal",
    "semilibertades": "semilibertad",
    "semilibres": "semilibre",
    "semiligera": "semiligero",
    "semiligeras": "semiligero",
    "semiligeros": "semiligero",
    "semilineales": "semilineal",
    "semilisa": "semiliso",
    "semilisas": "semiliso",
    "semilisos": "semiliso",
    "semillas": "semilla",
    "semillera": "semillero",
    "semilleras": "semillero",
    "semilleros": "semillero",
    "semilogarítmica": "semilogarítmico",
    "semilogarítmicas": "semilogarítmico",
    "semilogarítmicos": "semilogarítmico",
    "semilunares": "semilunar",
    "semilunios": "semilunio",
    "semimagnética": "semimagnético",
    "semimagnéticas": "semimagnético",
    "semimagnéticos": "semimagnético",
    "semimanufacturada": "semimanufacturado",
    "semimanufacturadas": "semimanufacturado",
    "semimanufacturados": "semimanufacturado",
    "semimembranosa": "semimembranoso",
    "semimembranosas": "semimembranoso",
    "semimembranosos": "semimembranoso",
    "semimetales": "semimetal",
    "semimundos": "semimundo",
    "semimódulos": "semimódulo",
    "seminales": "seminal",
    "seminarios": "seminario",
    "seminaristas": "seminarista",
    "seminomadismos": "seminomadismo",
    "seminomas": "seminoma",
    "seminormas": "seminorma",
    "seminternados": "seminternado",
    "seminífera": "seminífero",
    "seminíferas": "seminífero",
    "seminíferos": "seminífero",
    "seminívora": "seminívoro",
    "seminívoras": "seminívoro",
    "seminívoros": "seminívoro",
    "seminómadas": "seminómada",
    "semiobstruida": "semiobstruido",
    "semiobstruidas": "semiobstruido",
    "semiobstruidos": "semiobstruido",
    "semioctetos": "semiocteto",
    "semioficiales": "semioficial",
    "semiologías": "semiología",
    "semiológica": "semiológico",
    "semiológicas": "semiológico",
    "semiológicos": "semiológico",
    "semiondas": "semionda",
    "semiorugas": "semioruga",
    "semiotecnias": "semiotecnia",
    "semiotizaciones": "semiotización",
    "semiovales": "semioval",
    "semiparadas": "semiparada",
    "semiparásitos": "semiparásito",
    "semipatinadora": "semipatinador",
    "semipatinadoras": "semipatinador",
    "semipatinadores": "semipatinador",
    "semipedales": "semipedal",
    "semipelagiana": "semipelagiano",
    "semipelagianas": "semipelagiano",
    "semipelagianismos": "semipelagianismo",
    "semipelagianos": "semipelagiano",
    "semipermanentes": "semipermanente",
    "semipermeables": "semipermeable",
    "semiperímetros": "semiperímetro",
    "semiperíodos": "semiperíodo",
    "semipesada": "semipesado",
    "semipesadas": "semipesado",
    "semipesados": "semipesado",
    "semiplanos": "semiplano",
    "semipolítica": "semipolítico",
    "semipolíticas": "semipolítico",
    "semipolíticos": "semipolítico",
    "semiportátiles": "semiportátil",
    "semipresidencialismos": "semipresidencialismo",
    "semipresidencialistas": "semipresidencialista",
    "semiprocesada": "semiprocesado",
    "semiprocesadas": "semiprocesado",
    "semiprocesados": "semiprocesado",
    "semipulsos": "semipulso",
    "semiquiasmas": "semiquiasma",
    "semiquímica": "semiquímico",
    "semiquímicas": "semiquímico",
    "semiquímicos": "semiquímico",
    "semirrectas": "semirrecta",
    "semirredonda": "semirredondo",
    "semirredondas": "semirredondo",
    "semirredondos": "semirredondo",
    "semirremolques": "semirremolque",
    "semirrápida": "semirrápido",
    "semirrápidas": "semirrápido",
    "semirrápidos": "semirrápido",
    "semirrígida": "semirrígido",
    "semirrígidas": "semirrígido",
    "semirrígidos": "semirrígido",
    "semiseca": "semiseco",
    "semisecas": "semiseco",
    "semisecos": "semiseco",
    "semisimples": "semisimple",
    "semisumadores": "semisumador",
    "semisumas": "semisuma",
    "semisílabas": "semisílaba",
    "semisólida": "semisólido",
    "semisólidas": "semisólido",
    "semisólidos": "semisólido",
    "semisótanos": "semisótano",
    "semitas": "semita",
    "semiterminada": "semiterminado",
    "semiterminadas": "semiterminado",
    "semiterminados": "semiterminado",
    "semitismos": "semitismo",
    "semitistas": "semitista",
    "semitonos": "semitono",
    "semitrinos": "semitrino",
    "semiunivalentes": "semiunivalente",
    "semividas": "semivida",
    "semiviva": "semivivo",
    "semivivas": "semivivo",
    "semivivos": "semivivo",
    "semivocales": "semivocal",
    "semivocálica": "semivocálico",
    "semivocálicas": "semivocálico",
    "semivocálicos": "semivocálico",
    "semiángulos": "semiángulo",
    "semióloga": "semiólogo",
    "semiólogas": "semiólogo",
    "semiólogos": "semiólogo",
    "semiótica": "semiótico",
    "semióticas": "semiótico",
    "semióticos": "semiótico",
    "semovientes": "semoviente",
    "sempervirentes": "sempervirente",
    "sempiterna": "sempiterno",
    "sempiternas": "sempiterno",
    "sempiternos": "sempiterno",
    "semáforos": "semáforo",
    "semántica": "semántico",
    "semánticas": "semántico",
    "semánticos": "semántico",
    "semínimas": "semínima",
    "semínolas": "semínola",
    "semítica": "semítico",
    "semíticas": "semítico",
    "semíticos": "semítico",
    "senadas": "senada",
    "senadoconsultos": "senadoconsulto",
    "senadora": "senador",
    "senadoras": "senador",
    "senadores": "senador",
    "senados": "senado",
    "senadurías": "senaduría",
    "senaras": "senara",
    "senaria": "senario",
    "senarias": "senario",
    "senarios": "senario",
    "senarmontitas": "senarmontita",
    "senatoria": "senatorio",
    "senatoriales": "senatorial",
    "senatorias": "senatorio",
    "senatorios": "senatorio",
    "sencilla": "sencillo",
    "sencillas": "sencillo",
    "sencilleces": "sencillez",
    "sencillos": "sencillo",
    "sendas": "sendos",
    "senderea": "senderear",
    "sendereaba": "senderear",
    "sendereabais": "senderear",
    "sendereaban": "senderear",
    "sendereabas": "senderear",
    "senderead": "senderear",
    "sendereada": "senderear",
    "sendereadas": "senderear",
    "sendereado": "senderear",
    "sendereados": "senderear",
    "sendereamos": "senderear",
    "senderean": "senderear",
    "sendereando": "senderear",
    "sendereara": "senderear",
    "senderearais": "senderear",
    "senderearan": "senderear",
    "senderearas": "senderear",
    "sendereare": "senderear",
    "sendereareis": "senderear",
    "senderearemos": "senderear",
    "senderearen": "senderear",
    "sendereares": "senderear",
    "senderearon": "senderear",
    "sendereará": "senderear",
    "senderearán": "senderear",
    "senderearás": "senderear",
    "senderearé": "senderear",
    "senderearéis": "senderear",
    "senderearía": "senderear",
    "senderearíais": "senderear",
    "senderearíamos": "senderear",
    "senderearían": "senderear",
    "senderearías": "senderear",
    "sendereas": "senderear",
    "senderease": "senderear",
    "sendereaseis": "senderear",
    "sendereasen": "senderear",
    "sendereases": "senderear",
    "sendereaste": "senderear",
    "sendereasteis": "senderear",
    "senderee": "senderear",
    "sendereemos": "senderear",
    "sendereen": "senderear",
    "senderees": "senderear",
    "sendereo": "senderear",
    "sendereábamos": "senderear",
    "sendereáis": "senderear",
    "sendereáramos": "senderear",
    "sendereáremos": "senderear",
    "sendereásemos": "senderear",
    "sendereé": "senderear",
    "sendereéis": "senderear",
    "sendereó": "senderear",
    "senderismos": "senderismo",
    "senderistas": "senderista",
    "senderos": "sendero",
    "senectudes": "senectud",
    "senegalesa": "senegalés",
    "senegalesas": "senegalés",
    "senegaleses": "senegalés",
    "senegas": "senega",
    "senequismos": "senequismo",
    "senequistas": "senequista",
    "senes": "sen",
    "senescales": "senescal",
    "senescencias": "senescencia",
    "senescentes": "senescente",
    "seniles": "senil",
    "senilidades": "senilidad",
    "seniors": "senior",
    "senoidales": "senoidal",
    "senos": "seno",
    "sensacionales": "sensacional",
    "sensacionalismos": "sensacionalismo",
    "sensacionalistas": "sensacionalista",
    "sensaciones": "sensación",
    "sensata": "sensato",
    "sensatas": "sensato",
    "sensateces": "sensatez",
    "sensatos": "sensato",
    "sensibilice": "sensibilizar",
    "sensibilicemos": "sensibilizar",
    "sensibilicen": "sensibilizar",
    "sensibilices": "sensibilizar",
    "sensibilicé": "sensibilizar",
    "sensibilicéis": "sensibilizar",
    "sensibilidades": "sensibilidad",
    "sensibiliza": "sensibilizar",
    "sensibilizaba": "sensibilizar",
    "sensibilizabais": "sensibilizar",
    "sensibilizaban": "sensibilizar",
    "sensibilizabas": "sensibilizar",
    "sensibilizaciones": "sensibilización",
    "sensibilizad": "sensibilizar",
    "sensibilizada": "sensibilizar",
    "sensibilizadas": "sensibilizar",
    "sensibilizado": "sensibilizar",
    "sensibilizadora": "sensibilizador",
    "sensibilizadoras": "sensibilizador",
    "sensibilizadores": "sensibilizador",
    "sensibilizados": "sensibilizar",
    "sensibilizamos": "sensibilizar",
    "sensibilizan": "sensibilizar",
    "sensibilizando": "sensibilizar",
    "sensibilizara": "sensibilizar",
    "sensibilizarais": "sensibilizar",
    "sensibilizaran": "sensibilizar",
    "sensibilizaras": "sensibilizar",
    "sensibilizare": "sensibilizar",
    "sensibilizareis": "sensibilizar",
    "sensibilizaremos": "sensibilizar",
    "sensibilizaren": "sensibilizar",
    "sensibilizares": "sensibilizar",
    "sensibilizaron": "sensibilizar",
    "sensibilizará": "sensibilizar",
    "sensibilizarán": "sensibilizar",
    "sensibilizarás": "sensibilizar",
    "sensibilizaré": "sensibilizar",
    "sensibilizaréis": "sensibilizar",
    "sensibilizaría": "sensibilizar",
    "sensibilizaríais": "sensibilizar",
    "sensibilizaríamos": "sensibilizar",
    "sensibilizarían": "sensibilizar",
    "sensibilizarías": "sensibilizar",
    "sensibilizas": "sensibilizar",
    "sensibilizase": "sensibilizar",
    "sensibilizaseis": "sensibilizar",
    "sensibilizasen": "sensibilizar",
    "sensibilizases": "sensibilizar",
    "sensibilizaste": "sensibilizar",
    "sensibilizasteis": "sensibilizar",
    "sensibilizo": "sensibilizar",
    "sensibilizábamos": "sensibilizar",
    "sensibilizáis": "sensibilizar",
    "sensibilizáramos": "sensibilizar",
    "sensibilizáremos": "sensibilizar",
    "sensibilizásemos": "sensibilizar",
    "sensibilizó": "sensibilizar",
    "sensiblera": "sensiblero",
    "sensibleras": "sensiblero",
    "sensibleros": "sensiblero",
    "sensiblerías": "sensiblería",
    "sensibles": "sensible",
    "sensitiva": "sensitivo",
    "sensitivas": "sensitivo",
    "sensitivos": "sensitivo",
    "sensitometrías": "sensitometría",
    "sensitométrica": "sensitométrico",
    "sensitométricas": "sensitométrico",
    "sensitométricos": "sensitométrico",
    "sensitómetros": "sensitómetro",
    "sensores": "sensor",
    "sensoria": "sensorio",
    "sensoriales": "sensorial",
    "sensorias": "sensorio",
    "sensorios": "sensorio",
    "sensuales": "sensual",
    "sensualidades": "sensualidad",
    "sensualismos": "sensualismo",
    "sensualistas": "sensualista",
    "sentaba": "sentar",
    "sentabais": "sentar",
    "sentaban": "sentar",
    "sentabas": "sentar",
    "sentad": "sentar",
    "sentada": "sentar",
    "sentadas": "sentar",
    "sentaderos": "sentadero",
    "sentado": "sentar",
    "sentados": "sentar",
    "sentamos": "sentar",
    "sentando": "sentar",
    "sentara": "sentar",
    "sentarais": "sentar",
    "sentaran": "sentar",
    "sentaras": "sentar",
    "sentare": "sentar",
    "sentareis": "sentar",
    "sentaremos": "sentar",
    "sentaren": "sentar",
    "sentares": "sentar",
    "sentaron": "sentar",
    "sentará": "sentar",
    "sentarán": "sentar",
    "sentarás": "sentar",
    "sentaré": "sentar",
    "sentaréis": "sentar",
    "sentaría": "sentar",
    "sentaríais": "sentar",
    "sentaríamos": "sentar",
    "sentarían": "sentar",
    "sentarías": "sentar",
    "sentase": "sentar",
    "sentaseis": "sentar",
    "sentasen": "sentar",
    "sentases": "sentar",
    "sentaste": "sentar",
    "sentasteis": "sentar",
    "sentemos": "sentar",
    "sentencia": "sentenciar",
    "sentenciaba": "sentenciar",
    "sentenciabais": "sentenciar",
    "sentenciaban": "sentenciar",
    "sentenciabas": "sentenciar",
    "sentenciad": "sentenciar",
    "sentenciada": "sentenciar",
    "sentenciadas": "sentenciar",
    "sentenciado": "sentenciar",
    "sentenciadora": "sentenciador",
    "sentenciadoras": "sentenciador",
    "sentenciadores": "sentenciador",
    "sentenciados": "sentenciar",
    "sentenciales": "sentencial",
    "sentenciamos": "sentenciar",
    "sentencian": "sentenciar",
    "sentenciando": "sentenciar",
    "sentenciara": "sentenciar",
    "sentenciarais": "sentenciar",
    "sentenciaran": "sentenciar",
    "sentenciaras": "sentenciar",
    "sentenciare": "sentenciar",
    "sentenciareis": "sentenciar",
    "sentenciaremos": "sentenciar",
    "sentenciaren": "sentenciar",
    "sentenciares": "sentenciar",
    "sentenciaron": "sentenciar",
    "sentenciará": "sentenciar",
    "sentenciarán": "sentenciar",
    "sentenciarás": "sentenciar",
    "sentenciaré": "sentenciar",
    "sentenciaréis": "sentenciar",
    "sentenciaría": "sentenciar",
    "sentenciaríais": "sentenciar",
    "sentenciaríamos": "sentenciar",
    "sentenciarían": "sentenciar",
    "sentenciarías": "sentenciar",
    "sentencias": "sentenciar",
    "sentenciase": "sentenciar",
    "sentenciaseis": "sentenciar",
    "sentenciasen": "sentenciar",
    "sentenciases": "sentenciar",
    "sentenciaste": "sentenciar",
    "sentenciasteis": "sentenciar",
    "sentencie": "sentenciar",
    "sentenciemos": "sentenciar",
    "sentencien": "sentenciar",
    "sentencies": "sentenciar",
    "sentencio": "sentenciar",
    "sentenciosa": "sentencioso",
    "sentenciosas": "sentencioso",
    "sentenciosos": "sentencioso",
    "sentenciábamos": "sentenciar",
    "sentenciáis": "sentenciar",
    "sentenciáramos": "sentenciar",
    "sentenciáremos": "sentenciar",
    "sentenciásemos": "sentenciar",
    "sentencié": "sentenciar",
    "sentenciéis": "sentenciar",
    "sentenció": "sentenciar",
    "senticares": "senticar",
    "sentid": "sentir",
    "sentida": "sentir",
    "sentidas": "sentir",
    "sentido": "sentir",
    "sentidos": "sentir",
    "sentimentales": "sentimental",
    "sentimentalismos": "sentimentalismo",
    "sentimientos": "sentimiento",
    "sentimos": "sentir",
    "sentinas": "sentina",
    "sentiremos": "sentir",
    "sentires": "sentir",
    "sentirá": "sentir",
    "sentirán": "sentir",
    "sentirás": "sentir",
    "sentiré": "sentir",
    "sentiréis": "sentir",
    "sentiría": "sentir",
    "sentiríais": "sentir",
    "sentiríamos": "sentir",
    "sentirían": "sentir",
    "sentirías": "sentir",
    "sentiste": "sentir",
    "sentisteis": "sentir",
    "sentábamos": "sentar",
    "sentáis": "sentar",
    "sentáramos": "sentar",
    "sentáremos": "sentar",
    "sentásemos": "sentar",
    "senté": "sentar",
    "sentéis": "sentar",
    "sentí": "sentir",
    "sentía": "sentir",
    "sentíais": "sentir",
    "sentíamos": "sentir",
    "sentían": "sentir",
    "sentías": "sentir",
    "sentís": "sentir",
    "sentó": "sentar",
    "seora": "seor",
    "seoras": "seor",
    "seores": "seor",
    "seos": "seo",
    "sep.": "septiembre",
    "sepa": "saber",
    "sepamos": "saber",
    "sepan": "saber",
    "separa": "separar",
    "separaba": "separar",
    "separabais": "separar",
    "separaban": "separar",
    "separabas": "separar",
    "separabilidades": "separabilidad",
    "separables": "separable",
    "separaciones": "separación",
    "separad": "separar",
    "separada": "separar",
    "separadas": "separar",
    "separado": "separar",
    "separadora": "separador",
    "separadoras": "separadora",
    "separadores": "separador",
    "separados": "separar",
    "separamos": "separar",
    "separan": "separar",
    "separando": "separar",
    "separara": "separar",
    "separarais": "separar",
    "separaran": "separar",
    "separaras": "separar",
    "separare": "separar",
    "separareis": "separar",
    "separaremos": "separar",
    "separaren": "separar",
    "separares": "separar",
    "separaron": "separar",
    "separará": "separar",
    "separarán": "separar",
    "separarás": "separar",
    "separaré": "separar",
    "separaréis": "separar",
    "separaría": "separar",
    "separaríais": "separar",
    "separaríamos": "separar",
    "separarían": "separar",
    "separarías": "separar",
    "separas": "separar",
    "separase": "separar",
    "separaseis": "separar",
    "separasen": "separar",
    "separases": "separar",
    "separaste": "separar",
    "separasteis": "separar",
    "separatas": "separata",
    "separatismos": "separatismo",
    "separatistas": "separatista",
    "separativa": "separativo",
    "separativas": "separativo",
    "separativos": "separativo",
    "separatrices": "separatriz",
    "separe": "separar",
    "separemos": "separar",
    "separen": "separar",
    "separes": "separar",
    "separo": "separar",
    "separábamos": "separar",
    "separáis": "separar",
    "separáramos": "separar",
    "separáremos": "separar",
    "separásemos": "separar",
    "separé": "separar",
    "separéis": "separar",
    "separó": "separar",
    "sepas": "saber",
    "sepedones": "sepedón",
    "sepelios": "sepelio",
    "sepias": "sepia",
    "sepiolas": "sepiola",
    "sepiolitas": "sepiolita",
    "septada": "septado",
    "septadas": "septado",
    "septados": "septado",
    "septena": "septeno",
    "septenaria": "septenario",
    "septenarias": "septenario",
    "septenarios": "septenario",
    "septenas": "septeno",
    "septenatos": "septenato",
    "septenios": "septenio",
    "septenos": "septeno",
    "septentrionales": "septentrional",
    "septentriones": "septentrión",
    "septetos": "septeto",
    "septicemias": "septicemia",
    "septicidades": "septicidad",
    "septicidas": "septicida",
    "septicémica": "septicémico",
    "septicémicas": "septicémico",
    "septicémicos": "septicémico",
    "septiembres": "septiembre",
    "septillos": "septillo",
    "septingentésima": "700",
    "septingentésimas": "700",
    "septingentésimo": "700",
    "septingentésimos": "700",
    "septisílaba": "septisílabo",
    "septisílabas": "septisílabo",
    "septisílabos": "septisílabo",
    "septisómica": "septisómico",
    "septisómicas": "septisómico",
    "septisómicos": "septisómico",
    "septoplastias": "septoplastia",
    "septos": "septo",
    "septuagenaria": "septuagenario",
    "septuagenarias": "septuagenario",
    "septuagenarios": "septuagenario",
    "septuagesimoctava": "78",
    "septuagesimoctavas": "78",
    "septuagesimoctavo": "78",
    "septuagesimoctavos": "78",
    "septuagesimocuarta": "74",
    "septuagesimocuartas": "74",
    "septuagesimocuarto": "74",
    "septuagesimocuartos": "74",
    "septuagesimonona": "79",
    "septuagesimononas": "79",
    "septuagesimonono": "79",
    "septuagesimononos": "79",
    "septuagesimonovena": "79",
    "septuagesimonovenas": "79",
    "septuagesimonoveno": "79",
    "septuagesimonovenos": "79",
    "septuagesimoprimera": "71",
    "septuagesimoprimeras": "71",
    "septuagesimoprimero": "71",
    "septuagesimoprimeros": "71",
    "septuagesimoquinta": "75",
    "septuagesimoquintas": "75",
    "septuagesimoquinto": "75",
    "septuagesimoquintos": "75",
    "septuagesimosegunda": "72",
    "septuagesimosegundas": "72",
    "septuagesimosegundo": "72",
    "septuagesimosegundos": "72",
    "septuagesimosexta": "76",
    "septuagesimosextas": "76",
    "septuagesimosexto": "76",
    "septuagesimosextos": "76",
    "septuagesimoséptima": "77",
    "septuagesimoséptimas": "77",
    "septuagesimoséptimo": "77",
    "septuagesimoséptimos": "77",
    "septuagesimotercera": "73",
    "septuagesimoterceras": "73",
    "septuagesimotercero": "73",
    "septuagesimoterceros": "73",
    "septuagésima": "70",
    "septuagésimas": "70",
    "septuagésimo": "70",
    "septuagésimos": "70",
    "septuplica": "septuplicar",
    "septuplicaba": "septuplicar",
    "septuplicabais": "septuplicar",
    "septuplicaban": "septuplicar",
    "septuplicabas": "septuplicar",
    "septuplicaciones": "septuplicación",
    "septuplicad": "septuplicar",
    "septuplicada": "septuplicar",
    "septuplicadas": "septuplicar",
    "septuplicado": "septuplicar",
    "septuplicados": "septuplicar",
    "septuplicamos": "septuplicar",
    "septuplican": "septuplicar",
    "septuplicando": "septuplicar",
    "septuplicara": "septuplicar",
    "septuplicarais": "septuplicar",
    "septuplicaran": "septuplicar",
    "septuplicaras": "septuplicar",
    "septuplicare": "septuplicar",
    "septuplicareis": "septuplicar",
    "septuplicaremos": "septuplicar",
    "septuplicaren": "septuplicar",
    "septuplicares": "septuplicar",
    "septuplicaron": "septuplicar",
    "septuplicará": "septuplicar",
    "septuplicarán": "septuplicar",
    "septuplicarás": "septuplicar",
    "septuplicaré": "septuplicar",
    "septuplicaréis": "septuplicar",
    "septuplicaría": "septuplicar",
    "septuplicaríais": "septuplicar",
    "septuplicaríamos": "septuplicar",
    "septuplicarían": "septuplicar",
    "septuplicarías": "septuplicar",
    "septuplicas": "septuplicar",
    "septuplicase": "septuplicar",
    "septuplicaseis": "septuplicar",
    "septuplicasen": "septuplicar",
    "septuplicases": "septuplicar",
    "septuplicaste": "septuplicar",
    "septuplicasteis": "septuplicar",
    "septuplico": "septuplicar",
    "septuplicábamos": "septuplicar",
    "septuplicáis": "septuplicar",
    "septuplicáramos": "septuplicar",
    "septuplicáremos": "septuplicar",
    "septuplicásemos": "septuplicar",
    "septuplicó": "septuplicar",
    "septuplique": "septuplicar",
    "septupliquemos": "septuplicar",
    "septupliquen": "septuplicar",
    "septupliques": "septuplicar",
    "septupliqué": "septuplicar",
    "septupliquéis": "septuplicar",
    "sepulcrales": "sepulcral",
    "sepulcros": "sepulcro",
    "sepulta": "sepulto",
    "sepultaba": "sepultar",
    "sepultabais": "sepultar",
    "sepultaban": "sepultar",
    "sepultabas": "sepultar",
    "sepultad": "sepultar",
    "sepultada": "sepultar",
    "sepultadas": "sepultar",
    "sepultado": "sepultar",
    "sepultadora": "sepultador",
    "sepultadoras": "sepultador",
    "sepultadores": "sepultador",
    "sepultados": "sepultar",
    "sepultamos": "sepultar",
    "sepultan": "sepultar",
    "sepultando": "sepultar",
    "sepultara": "sepultar",
    "sepultarais": "sepultar",
    "sepultaran": "sepultar",
    "sepultaras": "sepultar",
    "sepultare": "sepultar",
    "sepultareis": "sepultar",
    "sepultaremos": "sepultar",
    "sepultaren": "sepultar",
    "sepultares": "sepultar",
    "sepultaron": "sepultar",
    "sepultará": "sepultar",
    "sepultarán": "sepultar",
    "sepultarás": "sepultar",
    "sepultaré": "sepultar",
    "sepultaréis": "sepultar",
    "sepultaría": "sepultar",
    "sepultaríais": "sepultar",
    "sepultaríamos": "sepultar",
    "sepultarían": "sepultar",
    "sepultarías": "sepultar",
    "sepultas": "sepulto",
    "sepultase": "sepultar",
    "sepultaseis": "sepultar",
    "sepultasen": "sepultar",
    "sepultases": "sepultar",
    "sepultaste": "sepultar",
    "sepultasteis": "sepultar",
    "sepulte": "sepultar",
    "sepultemos": "sepultar",
    "sepulten": "sepultar",
    "sepultes": "sepultar",
    "sepulto": "sepultar",
    "sepultos": "sepulto",
    "sepulturas": "sepultura",
    "sepulturera": "sepulturero",
    "sepultureras": "sepulturero",
    "sepultureros": "sepulturero",
    "sepultábamos": "sepultar",
    "sepultáis": "sepultar",
    "sepultáramos": "sepultar",
    "sepultáremos": "sepultar",
    "sepultásemos": "sepultar",
    "sepulté": "sepultar",
    "sepultéis": "sepultar",
    "sepultó": "sepultar",
    "sepáis": "saber",
    "seque": "secar",
    "sequedades": "sequedad",
    "sequedales": "sequedal",
    "sequemos": "secar",
    "sequen": "secar",
    "sequerales": "sequeral",
    "sequeros": "sequero",
    "sequerosa": "sequeroso",
    "sequerosas": "sequeroso",
    "sequerosos": "sequeroso",
    "sequerías": "sequería",
    "seques": "secar",
    "sequetes": "sequete",
    "sequillos": "sequillo",
    "sequiza": "sequizo",
    "sequizas": "sequizo",
    "sequizos": "sequizo",
    "sequoyas": "sequoya",
    "sequé": "secar",
    "sequéis": "secar",
    "sequías": "sequía",
    "sequíos": "sequío",
    "serados": "serado",
    "serafines": "serafín",
    "serajes": "seraje",
    "seras": "sera",
    "serasoides": "serasoide",
    "serasquieratos": "serasquierato",
    "serasquieres": "serasquier",
    "serbales": "serbal",
    "serbas": "serba",
    "serbia": "serbio",
    "serbias": "serbio",
    "serbios": "serbio",
    "serbobosnia": "serbobosnio",
    "serbobosnias": "serbobosnio",
    "serbobosnios": "serbobosnio",
    "serbocroatas": "serbocroata",
    "seremos": "ser",
    "serena": "sereno",
    "serenaba": "serenar",
    "serenabais": "serenar",
    "serenaban": "serenar",
    "serenabas": "serenar",
    "serenad": "serenar",
    "serenada": "serenar",
    "serenadas": "serenar",
    "serenado": "serenar",
    "serenados": "serenar",
    "serenamos": "serenar",
    "serenan": "serenar",
    "serenando": "serenar",
    "serenara": "serenar",
    "serenarais": "serenar",
    "serenaran": "serenar",
    "serenaras": "serenar",
    "serenare": "serenar",
    "serenareis": "serenar",
    "serenaremos": "serenar",
    "serenaren": "serenar",
    "serenares": "serenar",
    "serenaron": "serenar",
    "serenará": "serenar",
    "serenarán": "serenar",
    "serenarás": "serenar",
    "serenaré": "serenar",
    "serenaréis": "serenar",
    "serenaría": "serenar",
    "serenaríais": "serenar",
    "serenaríamos": "serenar",
    "serenarían": "serenar",
    "serenarías": "serenar",
    "serenas": "sereno",
    "serenase": "serenar",
    "serenaseis": "serenar",
    "serenasen": "serenar",
    "serenases": "serenar",
    "serenaste": "serenar",
    "serenasteis": "serenar",
    "serenatas": "serenata",
    "serene": "serenar",
    "serenemos": "serenar",
    "serenen": "serenar",
    "serenes": "serenar",
    "serenidades": "serenidad",
    "sereno": "serenar",
    "serenos": "sereno",
    "serenábamos": "serenar",
    "serenáis": "serenar",
    "serenáramos": "serenar",
    "serenáremos": "serenar",
    "serenásemos": "serenar",
    "serené": "serenar",
    "serenéis": "serenar",
    "serenó": "serenar",
    "seres": "ser",
    "seretes": "serete",
    "sergentas": "sergenta",
    "seria": "serio",
    "seriaba": "seriar",
    "seriabais": "seriar",
    "seriaban": "seriar",
    "seriabas": "seriar",
    "seriaciones": "seriación",
    "seriad": "seriar",
    "seriada": "seriar",
    "seriadas": "seriar",
    "seriado": "seriar",
    "seriados": "seriar",
    "seriales": "serial",
    "serializaciones": "serialización",
    "seriamos": "seriar",
    "serian": "seriar",
    "seriando": "seriar",
    "seriara": "seriar",
    "seriarais": "seriar",
    "seriaran": "seriar",
    "seriaras": "seriar",
    "seriare": "seriar",
    "seriareis": "seriar",
    "seriaremos": "seriar",
    "seriaren": "seriar",
    "seriares": "seriar",
    "seriaron": "seriar",
    "seriará": "seriar",
    "seriarán": "seriar",
    "seriarás": "seriar",
    "seriaré": "seriar",
    "seriaréis": "seriar",
    "seriaría": "seriar",
    "seriaríais": "seriar",
    "seriaríamos": "seriar",
    "seriarían": "seriar",
    "seriarías": "seriar",
    "serias": "serio",
    "seriase": "seriar",
    "seriaseis": "seriar",
    "seriasen": "seriar",
    "seriases": "seriar",
    "seriaste": "seriar",
    "seriasteis": "seriar",
    "sericicultora": "sericicultor",
    "sericicultoras": "sericicultor",
    "sericicultores": "sericicultor",
    "sericiculturas": "sericicultura",
    "sericitas": "sericita",
    "sericulturas": "sericultura",
    "sericícolas": "sericícola",
    "serie": "seriar",
    "seriedades": "seriedad",
    "seriemos": "seriar",
    "serien": "seriar",
    "series": "serie",
    "serigrafías": "serigrafía",
    "serigráfica": "serigráfico",
    "serigráficas": "serigráfico",
    "serigráficos": "serigráfico",
    "serijos": "serijo",
    "serinas": "serina",
    "serio": "seriar",
    "seriolas": "seriola",
    "serios": "serio",
    "seriábamos": "seriar",
    "seriáis": "seriar",
    "seriáramos": "seriar",
    "seriáremos": "seriar",
    "seriásemos": "seriar",
    "serié": "seriar",
    "seriéis": "seriar",
    "serió": "seriar",
    "sermonaria": "sermonario",
    "sermonarias": "sermonario",
    "sermonarios": "sermonario",
    "sermonea": "sermonear",
    "sermoneaba": "sermonear",
    "sermoneabais": "sermonear",
    "sermoneaban": "sermonear",
    "sermoneabas": "sermonear",
    "sermonead": "sermonear",
    "sermoneada": "sermonear",
    "sermoneadas": "sermonear",
    "sermoneado": "sermonear",
    "sermoneadora": "sermoneador",
    "sermoneadoras": "sermoneador",
    "sermoneadores": "sermoneador",
    "sermoneados": "sermonear",
    "sermoneamos": "sermonear",
    "sermonean": "sermonear",
    "sermoneando": "sermonear",
    "sermoneara": "sermonear",
    "sermonearais": "sermonear",
    "sermonearan": "sermonear",
    "sermonearas": "sermonear",
    "sermoneare": "sermonear",
    "sermoneareis": "sermonear",
    "sermonearemos": "sermonear",
    "sermonearen": "sermonear",
    "sermoneares": "sermonear",
    "sermonearon": "sermonear",
    "sermoneará": "sermonear",
    "sermonearán": "sermonear",
    "sermonearás": "sermonear",
    "sermonearé": "sermonear",
    "sermonearéis": "sermonear",
    "sermonearía": "sermonear",
    "sermonearíais": "sermonear",
    "sermonearíamos": "sermonear",
    "sermonearían": "sermonear",
    "sermonearías": "sermonear",
    "sermoneas": "sermonear",
    "sermonease": "sermonear",
    "sermoneaseis": "sermonear",
    "sermoneasen": "sermonear",
    "sermoneases": "sermonear",
    "sermoneaste": "sermonear",
    "sermoneasteis": "sermonear",
    "sermonee": "sermonear",
    "sermoneemos": "sermonear",
    "sermoneen": "sermonear",
    "sermonees": "sermonear",
    "sermoneo": "sermonear",
    "sermoneos": "sermoneo",
    "sermones": "sermón",
    "sermoneábamos": "sermonear",
    "sermoneáis": "sermonear",
    "sermoneáramos": "sermonear",
    "sermoneáremos": "sermonear",
    "sermoneásemos": "sermonear",
    "sermoneé": "sermonear",
    "sermoneéis": "sermonear",
    "sermoneó": "sermonear",
    "sernas": "serna",
    "seroaglutinaciones": "seroaglutinación",
    "seroalbúminas": "seroalbúmina",
    "seroconversiones": "seroconversión",
    "serocultivos": "serocultivo",
    "serodiagnósticos": "serodiagnóstico",
    "serojas": "seroja",
    "serojos": "serojo",
    "serologías": "serología",
    "serológica": "serológico",
    "serológicas": "serológico",
    "serológicos": "serológico",
    "seromucosa": "seromucoso",
    "seromucosas": "seromucoso",
    "seromucosos": "seromucoso",
    "seronda": "serondo",
    "serondas": "serondo",
    "serondos": "serondo",
    "seronera": "seronero",
    "seroneras": "seronero",
    "seroneros": "seronero",
    "serones": "serón",
    "seropositiva": "seropositivo",
    "seropositivas": "seropositivo",
    "seropositividades": "seropositividad",
    "seropositivos": "seropositivo",
    "serosa": "seroso",
    "serosas": "seroso",
    "serosidades": "serosidad",
    "serosos": "seroso",
    "seroterapias": "seroterapia",
    "serotipos": "serotipo",
    "serotoninas": "serotonina",
    "serovacunaciones": "serovacunación",
    "serpas": "serpa",
    "serpentea": "serpentear",
    "serpenteaba": "serpentear",
    "serpenteabais": "serpentear",
    "serpenteaban": "serpentear",
    "serpenteabas": "serpentear",
    "serpentead": "serpentear",
    "serpenteada": "serpentear",
    "serpenteadas": "serpentear",
    "serpenteado": "serpentear",
    "serpenteados": "serpentear",
    "serpenteamos": "serpentear",
    "serpentean": "serpentear",
    "serpenteando": "serpentear",
    "serpenteantes": "serpenteante",
    "serpenteara": "serpentear",
    "serpentearais": "serpentear",
    "serpentearan": "serpentear",
    "serpentearas": "serpentear",
    "serpenteare": "serpentear",
    "serpenteareis": "serpentear",
    "serpentearemos": "serpentear",
    "serpentearen": "serpentear",
    "serpenteares": "serpentear",
    "serpentearon": "serpentear",
    "serpenteará": "serpentear",
    "serpentearán": "serpentear",
    "serpentearás": "serpentear",
    "serpentearé": "serpentear",
    "serpentearéis": "serpentear",
    "serpentearía": "serpentear",
    "serpentearíais": "serpentear",
    "serpentearíamos": "serpentear",
    "serpentearían": "serpentear",
    "serpentearías": "serpentear",
    "serpenteas": "serpentear",
    "serpentease": "serpentear",
    "serpenteaseis": "serpentear",
    "serpenteasen": "serpentear",
    "serpenteases": "serpentear",
    "serpenteaste": "serpentear",
    "serpenteasteis": "serpentear",
    "serpentee": "serpentear",
    "serpenteemos": "serpentear",
    "serpenteen": "serpentear",
    "serpentees": "serpentear",
    "serpenteo": "serpentear",
    "serpenteos": "serpenteo",
    "serpenteábamos": "serpentear",
    "serpenteáis": "serpentear",
    "serpenteáramos": "serpentear",
    "serpenteáremos": "serpentear",
    "serpenteásemos": "serpentear",
    "serpenteé": "serpentear",
    "serpenteéis": "serpentear",
    "serpenteó": "serpentear",
    "serpentina": "serpentino",
    "serpentinas": "serpentino",
    "serpentines": "serpentín",
    "serpentinos": "serpentino",
    "serpentones": "serpentón",
    "serpentígera": "serpentígero",
    "serpentígeras": "serpentígero",
    "serpentígeros": "serpentígero",
    "serpias": "serpia",
    "serpientes": "serpiente",
    "serpiginosa": "serpiginoso",
    "serpiginosas": "serpiginoso",
    "serpiginosos": "serpiginoso",
    "serpigos": "serpigo",
    "serpinas": "serpina",
    "serpoles": "serpol",
    "serpolla": "serpollar",
    "serpollaba": "serpollar",
    "serpollabais": "serpollar",
    "serpollaban": "serpollar",
    "serpollabas": "serpollar",
    "serpollad": "serpollar",
    "serpollada": "serpollar",
    "serpolladas": "serpollar",
    "serpollado": "serpollar",
    "serpollados": "serpollar",
    "serpollamos": "serpollar",
    "serpollan": "serpollar",
    "serpollando": "serpollar",
    "serpollara": "serpollar",
    "serpollarais": "serpollar",
    "serpollaran": "serpollar",
    "serpollaras": "serpollar",
    "serpollare": "serpollar",
    "serpollareis": "serpollar",
    "serpollaremos": "serpollar",
    "serpollaren": "serpollar",
    "serpollares": "serpollar",
    "serpollaron": "serpollar",
    "serpollará": "serpollar",
    "serpollarán": "serpollar",
    "serpollarás": "serpollar",
    "serpollaré": "serpollar",
    "serpollaréis": "serpollar",
    "serpollaría": "serpollar",
    "serpollaríais": "serpollar",
    "serpollaríamos": "serpollar",
    "serpollarían": "serpollar",
    "serpollarías": "serpollar",
    "serpollas": "serpollar",
    "serpollase": "serpollar",
    "serpollaseis": "serpollar",
    "serpollasen": "serpollar",
    "serpollases": "serpollar",
    "serpollaste": "serpollar",
    "serpollasteis": "serpollar",
    "serpolle": "serpollar",
    "serpollemos": "serpollar",
    "serpollen": "serpollar",
    "serpolles": "serpollar",
    "serpollo": "serpollar",
    "serpollos": "serpollo",
    "serpollábamos": "serpollar",
    "serpolláis": "serpollar",
    "serpolláramos": "serpollar",
    "serpolláremos": "serpollar",
    "serpollásemos": "serpollar",
    "serpollé": "serpollar",
    "serpolléis": "serpollar",
    "serpolló": "serpollar",
    "serraba": "serrar",
    "serrabais": "serrar",
    "serraban": "serrar",
    "serrabas": "serrar",
    "serrad": "serrar",
    "serrada": "serrar",
    "serradas": "serrar",
    "serradiza": "serradizo",
    "serradizas": "serradizo",
    "serradizos": "serradizo",
    "serrado": "serrar",
    "serradora": "serrador",
    "serradoras": "serradora",
    "serradores": "serrador",
    "serrados": "serrar",
    "serrallos": "serrallo",
    "serramos": "serrar",
    "serrana": "serrano",
    "serranas": "serrano",
    "serrandeles": "serrandel",
    "serrando": "serrar",
    "serraniega": "serraniego",
    "serraniegas": "serraniego",
    "serraniegos": "serraniego",
    "serraniles": "serranil",
    "serranillas": "serranilla",
    "serranos": "serrano",
    "serranías": "serranía",
    "serrara": "serrar",
    "serrarais": "serrar",
    "serraran": "serrar",
    "serraras": "serrar",
    "serrare": "serrar",
    "serrareis": "serrar",
    "serraremos": "serrar",
    "serraren": "serrar",
    "serrares": "serrar",
    "serraron": "serrar",
    "serrará": "serrar",
    "serrarán": "serrar",
    "serrarás": "serrar",
    "serraré": "serrar",
    "serraréis": "serrar",
    "serraría": "serrar",
    "serraríais": "serrar",
    "serraríamos": "serrar",
    "serrarían": "serrar",
    "serrarías": "serrar",
    "serrase": "serrar",
    "serraseis": "serrar",
    "serrasen": "serrar",
    "serrases": "serrar",
    "serraste": "serrar",
    "serrasteis": "serrar",
    "serratas": "serrata",
    "serratos": "serrato",
    "serremos": "serrar",
    "serrerías": "serrería",
    "serretas": "serreta",
    "serretazos": "serretazo",
    "serrijones": "serrijón",
    "serrina": "serrino",
    "serrinas": "serrino",
    "serrines": "serrín",
    "serrinos": "serrino",
    "serrones": "serrón",
    "serruchos": "serrucho",
    "serrábamos": "serrar",
    "serráis": "serrar",
    "serráramos": "serrar",
    "serráremos": "serrar",
    "serrásemos": "serrar",
    "serré": "serrar",
    "serréis": "serrar",
    "serró": "serrar",
    "servadores": "servador",
    "servales": "serval",
    "servatos": "servato",
    "serventesios": "serventesio",
    "servia": "servio",
    "servias": "servio",
    "servibles": "servible",
    "serviciales": "servicial",
    "servicios": "servicio",
    "servid": "servir",
    "servida": "servir",
    "servidas": "servir",
    "servidera": "servidero",
    "servideras": "servidero",
    "servideros": "servidero",
    "servido": "servir",
    "servidora": "servidor",
    "servidoras": "servidor",
    "servidores": "servidor",
    "servidos": "servir",
    "servidumbres": "servidumbre",
    "serviles": "servil",
    "servilismos": "servilismo",
    "servilletas": "servilleta",
    "servilleteros": "servilletero",
    "servilona": "servilón",
    "servilonas": "servilón",
    "servilones": "servilón",
    "servimos": "servir",
    "serviolas": "serviola",
    "servios": "servio",
    "serviremos": "servir",
    "servirá": "servir",
    "servirán": "servir",
    "servirás": "servir",
    "serviré": "servir",
    "serviréis": "servir",
    "serviría": "servir",
    "serviríais": "servir",
    "serviríamos": "servir",
    "servirían": "servir",
    "servirías": "servir",
    "serviste": "servir",
    "servisteis": "servir",
    "servitudes": "servitud",
    "servoamplificadores": "servoamplificador",
    "servoconducciones": "servoconducción",
    "servocontrolada": "servocontrolado",
    "servocontroladas": "servocontrolado",
    "servocontrolados": "servocontrolado",
    "servocontroles": "servocontrol",
    "servocroatas": "servocroata",
    "servodirecciones": "servodirección",
    "servofrenos": "servofreno",
    "servomandos": "servomando",
    "servomecanismos": "servomecanismo",
    "servomotores": "servomotor",
    "servoo": "servo",
    "servopistas": "servopista",
    "servosistemas": "servosistema",
    "servoválvulas": "servoválvula",
    "servozonas": "servozona",
    "serví": "servir",
    "servía": "servir",
    "servíais": "servir",
    "servíamos": "servir",
    "servían": "servir",
    "servías": "servir",
    "servís": "servir",
    "será": "ser",
    "seráfica": "seráfico",
    "seráficas": "seráfico",
    "seráficos": "seráfico",
    "serán": "ser",
    "serás": "ser",
    "seré": "ser",
    "seréis": "ser",
    "sería": "ser",
    "seríais": "ser",
    "seríamos": "ser",
    "serían": "ser",
    "serías": "ser",
    "serícea": "seríceo",
    "seríceas": "seríceo",
    "seríceos": "seríceo",
    "serótina": "serótino",
    "serótinas": "serótino",
    "serótinos": "serótino",
    "sesadas": "sesada",
    "sesamoidea": "sesamoideo",
    "sesamoideas": "sesamoideo",
    "sesamoideos": "sesamoideo",
    "sesamolinas": "sesamolina",
    "sesea": "sesear",
    "seseaba": "sesear",
    "seseabais": "sesear",
    "seseaban": "sesear",
    "seseabas": "sesear",
    "sesead": "sesear",
    "seseada": "sesear",
    "seseadas": "sesear",
    "seseado": "sesear",
    "seseados": "sesear",
    "seseamos": "sesear",
    "sesean": "sesear",
    "seseando": "sesear",
    "seseara": "sesear",
    "sesearais": "sesear",
    "sesearan": "sesear",
    "sesearas": "sesear",
    "seseare": "sesear",
    "seseareis": "sesear",
    "sesearemos": "sesear",
    "sesearen": "sesear",
    "seseares": "sesear",
    "sesearon": "sesear",
    "seseará": "sesear",
    "sesearán": "sesear",
    "sesearás": "sesear",
    "sesearé": "sesear",
    "sesearéis": "sesear",
    "sesearía": "sesear",
    "sesearíais": "sesear",
    "sesearíamos": "sesear",
    "sesearían": "sesear",
    "sesearías": "sesear",
    "seseas": "sesear",
    "sesease": "sesear",
    "seseaseis": "sesear",
    "seseasen": "sesear",
    "seseases": "sesear",
    "seseaste": "sesear",
    "seseasteis": "sesear",
    "sesee": "sesear",
    "seseemos": "sesear",
    "seseen": "sesear",
    "sesees": "sesear",
    "sesentaicincoava": "sesentaicincoavo",
    "sesentaicincoavas": "sesentaicincoavo",
    "sesentaicincoavo": "65",
    "sesentaicincoavos": "sesentaicincoavo",
    "sesentaicuatroava": "sesentaicuatroavo",
    "sesentaicuatroavas": "sesentaicuatroavo",
    "sesentaicuatroavo": "64",
    "sesentaicuatroavos": "sesentaicuatroavo",
    "sesentaidosava": "sesentaidosavo",
    "sesentaidosavas": "sesentaidosavo",
    "sesentaidosavo": "62",
    "sesentaidosavos": "sesentaidosavo",
    "sesentainueveava": "sesentainueveavo",
    "sesentainueveavas": "sesentainueveavo",
    "sesentainueveavo": "69",
    "sesentainueveavos": "sesentainueveavo",
    "sesentaiochava": "sesentaiochoavo",
    "sesentaiochavas": "sesentaiochoavo",
    "sesentaiochavo": "sesentaiochoavo",
    "sesentaiochavos": "sesentaiochoavo",
    "sesentaiochoava": "sesentaiochoavo",
    "sesentaiochoavas": "sesentaiochoavo",
    "sesentaiochoavo": "68",
    "sesentaiochoavos": "sesentaiochoavo",
    "sesentaiseisava": "sesentaiseisavo",
    "sesentaiseisavas": "sesentaiseisavo",
    "sesentaiseisavo": "66",
    "sesentaiseisavos": "sesentaiseisavo",
    "sesentaisieteava": "sesentaisieteavo",
    "sesentaisieteavas": "sesentaisieteavo",
    "sesentaisieteavo": "67",
    "sesentaisieteavos": "sesentaisieteavo",
    "sesentaitresava": "sesentaitresavo",
    "sesentaitresavas": "sesentaitresavo",
    "sesentaitresavo": "63",
    "sesentaitresavos": "sesentaitresavo",
    "sesentaiunava": "sesentaiunavo",
    "sesentaiunavas": "sesentaiunavo",
    "sesentaiunavo": "61",
    "sesentaiunavos": "sesentaiunavo",
    "sesentava": "sesentavo",
    "sesentavas": "sesentavo",
    "sesentavo": "60",
    "sesentavos": "sesentavo",
    "sesentona": "sesentón",
    "sesentonas": "sesentón",
    "sesentones": "sesentón",
    "seseo": "sesear",
    "seseos": "seseo",
    "seseras": "sesera",
    "seseábamos": "sesear",
    "seseáis": "sesear",
    "seseáramos": "sesear",
    "seseáremos": "sesear",
    "seseásemos": "sesear",
    "seseé": "sesear",
    "seseéis": "sesear",
    "seseó": "sesear",
    "sesga": "sesgo",
    "sesgaba": "sesgar",
    "sesgabais": "sesgar",
    "sesgaban": "sesgar",
    "sesgabas": "sesgar",
    "sesgad": "sesgar",
    "sesgada": "sesgar",
    "sesgadas": "sesgar",
    "sesgado": "sesgar",
    "sesgados": "sesgar",
    "sesgaduras": "sesgadura",
    "sesgamos": "sesgar",
    "sesgan": "sesgar",
    "sesgando": "sesgar",
    "sesgara": "sesgar",
    "sesgarais": "sesgar",
    "sesgaran": "sesgar",
    "sesgaras": "sesgar",
    "sesgare": "sesgar",
    "sesgareis": "sesgar",
    "sesgaremos": "sesgar",
    "sesgaren": "sesgar",
    "sesgares": "sesgar",
    "sesgaron": "sesgar",
    "sesgará": "sesgar",
    "sesgarán": "sesgar",
    "sesgarás": "sesgar",
    "sesgaré": "sesgar",
    "sesgaréis": "sesgar",
    "sesgaría": "sesgar",
    "sesgaríais": "sesgar",
    "sesgaríamos": "sesgar",
    "sesgarían": "sesgar",
    "sesgarías": "sesgar",
    "sesgas": "sesgo",
    "sesgase": "sesgar",
    "sesgaseis": "sesgar",
    "sesgasen": "sesgar",
    "sesgases": "sesgar",
    "sesgaste": "sesgar",
    "sesgasteis": "sesgar",
    "sesgo": "sesgar",
    "sesgos": "sesgo",
    "sesgue": "sesgar",
    "sesguemos": "sesgar",
    "sesguen": "sesgar",
    "sesgues": "sesgar",
    "sesgué": "sesgar",
    "sesguéis": "sesgar",
    "sesgábamos": "sesgar",
    "sesgáis": "sesgar",
    "sesgáramos": "sesgar",
    "sesgáremos": "sesgar",
    "sesgásemos": "sesgar",
    "sesgó": "sesgar",
    "sesiones": "sesión",
    "sesos": "seso",
    "sesquibromuros": "sesquibromuro",
    "sesquicarbonatos": "sesquicarbonato",
    "sesquidiploidias": "sesquidiploidia",
    "sesquilineales": "sesquilineal",
    "sesquipedales": "sesquipedal",
    "sesquisilicatos": "sesquisilicato",
    "sesquisulfuros": "sesquisulfuro",
    "sesquiáltera": "sesquiáltero",
    "sesquiálteras": "sesquiáltero",
    "sesquiálteros": "sesquiáltero",
    "sesquióxidos": "sesquióxido",
    "sestea": "sestear",
    "sesteaba": "sestear",
    "sesteabais": "sestear",
    "sesteaban": "sestear",
    "sesteabas": "sestear",
    "sestead": "sestear",
    "sesteada": "sestear",
    "sesteadas": "sestear",
    "sesteaderos": "sesteadero",
    "sesteado": "sestear",
    "sesteados": "sestear",
    "sesteamos": "sestear",
    "sestean": "sestear",
    "sesteando": "sestear",
    "sesteara": "sestear",
    "sestearais": "sestear",
    "sestearan": "sestear",
    "sestearas": "sestear",
    "sesteare": "sestear",
    "sesteareis": "sestear",
    "sestearemos": "sestear",
    "sestearen": "sestear",
    "sesteares": "sestear",
    "sestearon": "sestear",
    "sesteará": "sestear",
    "sestearán": "sestear",
    "sestearás": "sestear",
    "sestearé": "sestear",
    "sestearéis": "sestear",
    "sestearía": "sestear",
    "sestearíais": "sestear",
    "sestearíamos": "sestear",
    "sestearían": "sestear",
    "sestearías": "sestear",
    "sesteas": "sestear",
    "sestease": "sestear",
    "sesteaseis": "sestear",
    "sesteasen": "sestear",
    "sesteases": "sestear",
    "sesteaste": "sestear",
    "sesteasteis": "sestear",
    "sestee": "sestear",
    "sesteemos": "sestear",
    "sesteen": "sestear",
    "sestees": "sestear",
    "sesteo": "sestear",
    "sesteos": "sesteo",
    "sestercios": "sestercio",
    "sesteros": "sestero",
    "sesteábamos": "sestear",
    "sesteáis": "sestear",
    "sesteáramos": "sestear",
    "sesteáremos": "sestear",
    "sesteásemos": "sestear",
    "sesteé": "sestear",
    "sesteéis": "sestear",
    "sesteó": "sestear",
    "sestiles": "sestil",
    "sestones": "seston",
    "sesuda": "sesudo",
    "sesudas": "sesudo",
    "sesudeces": "sesudez",
    "sesudos": "sesudo",
    "sesámea": "sesámeo",
    "sesámeas": "sesámeo",
    "sesámeos": "sesámeo",
    "setales": "setal",
    "setas": "seta",
    "setecientosava": "setecientosavo",
    "setecientosavas": "setecientosavo",
    "setecientosavo": "700",
    "setecientosavos": "setecientosavo",
    "setenada": "setenado",
    "setenadas": "setenado",
    "setenados": "setenado",
    "setentaicincoava": "setentaicincoavo",
    "setentaicincoavas": "setentaicincoavo",
    "setentaicincoavo": "75",
    "setentaicincoavos": "setentaicincoavo",
    "setentaicuatroava": "setentaicuatroavo",
    "setentaicuatroavas": "setentaicuatroavo",
    "setentaicuatroavo": "74",
    "setentaicuatroavos": "setentaicuatroavo",
    "setentaidosava": "setentaidosavo",
    "setentaidosavas": "setentaidosavo",
    "setentaidosavo": "72",
    "setentaidosavos": "setentaidosavo",
    "setentainueveava": "setentainueveavo",
    "setentainueveavas": "setentainueveavo",
    "setentainueveavo": "79",
    "setentainueveavos": "setentainueveavo",
    "setentaiochava": "setentaiochoavo",
    "setentaiochavas": "setentaiochoavo",
    "setentaiochavo": "setentaiochoavo",
    "setentaiochavos": "setentaiochoavo",
    "setentaiochoava": "setentaiochoavo",
    "setentaiochoavas": "setentaiochoavo",
    "setentaiochoavo": "78",
    "setentaiochoavos": "setentaiochoavo",
    "setentaiseisava": "setentaiseisavo",
    "setentaiseisavas": "setentaiseisavo",
    "setentaiseisavo": "76",
    "setentaiseisavos": "setentaiseisavo",
    "setentaisieteava": "setentaisieteavo",
    "setentaisieteavas": "setentaisieteavo",
    "setentaisieteavo": "77",
    "setentaisieteavos": "setentaisieteavo",
    "setentaitresava": "setentaitresavo",
    "setentaitresavas": "setentaitresavo",
    "setentaitresavo": "73",
    "setentaitresavos": "setentaitresavo",
    "setentaiunava": "setentaiunavo",
    "setentaiunavas": "setentaiunavo",
    "setentaiunavo": "71",
    "setentaiunavos": "setentaiunavo",
    "setentava": "setentavo",
    "setentavas": "setentavo",
    "setentavo": "70",
    "setentavos": "setentavo",
    "setentona": "setentón",
    "setentonas": "setentón",
    "setentones": "setentón",
    "setiembres": "setiembre",
    "setos": "seto",
    "sets": "set",
    "settecentos": "settecento",
    "setters": "setter",
    "settleres": "settler",
    "seudoaleaciones": "seudoaleación",
    "seudoanodoncias": "seudoanodoncia",
    "seudoartículos": "seudoartículo",
    "seudobolsas": "seudobolsa",
    "seudocarpos": "seudocarpo",
    "seudoceles": "seudocele",
    "seudocromosomas": "seudocromosoma",
    "seudocrítica": "seudocrítico",
    "seudocríticas": "seudocrítico",
    "seudocríticos": "seudocrítico",
    "seudogleyes": "seudogley",
    "seudoglobulinas": "seudoglobulina",
    "seudohalógenos": "seudohalógeno",
    "seudohemaglutinaciones": "seudohemaglutinación",
    "seudohermafroditas": "seudohermafrodita",
    "seudohermafroditismos": "seudohermafroditismo",
    "seudologías": "seudología",
    "seudomembranas": "seudomembrana",
    "seudopasividades": "seudopasividad",
    "seudoperiódica": "seudoperiódico",
    "seudoperiódicas": "seudoperiódico",
    "seudoperiódicos": "seudoperiódico",
    "seudoperíodos": "seudoperíodo",
    "seudoplasticidades": "seudoplasticidad",
    "seudoptialismo": "pseudoptialismo",
    "seudoptialismos": "pseudoptialismo",
    "seudoscopias": "seudoscopia",
    "seudoscopios": "seudoscopio",
    "seudoscorpiones": "seudoscorpión",
    "seudoscópica": "seudoscópico",
    "seudoscópicas": "seudoscópico",
    "seudoscópicos": "seudoscópico",
    "seudónima": "seudónimo",
    "seudónimas": "seudónimo",
    "seudónimos": "seudónimo",
    "seudópodos": "seudópodo",
    "severa": "severo",
    "severas": "severo",
    "severidades": "severidad",
    "severos": "severo",
    "sevicias": "sevicia",
    "sevillana": "sevillano",
    "sevillanas": "sevillano",
    "sevillanos": "sevillano",
    "sevillistas": "sevillista",
    "sex-shops": "sex-shop",
    "sexaciones": "sexación",
    "sexadora": "sexador",
    "sexadoras": "sexador",
    "sexadores": "sexador",
    "sexagenaria": "sexagenario",
    "sexagenarias": "sexagenario",
    "sexagenarios": "sexagenario",
    "sexagesimales": "sexagesimal",
    "sexagesimoctava": "68",
    "sexagesimoctavas": "68",
    "sexagesimoctavo": "68",
    "sexagesimoctavos": "68",
    "sexagesimocuarta": "64",
    "sexagesimocuartas": "64",
    "sexagesimocuarto": "64",
    "sexagesimocuartos": "64",
    "sexagesimonona": "69",
    "sexagesimononas": "69",
    "sexagesimonono": "69",
    "sexagesimononos": "69",
    "sexagesimonovena": "69",
    "sexagesimonovenas": "69",
    "sexagesimonoveno": "69",
    "sexagesimonovenos": "69",
    "sexagesimoprimera": "61",
    "sexagesimoprimeras": "61",
    "sexagesimoprimero": "61",
    "sexagesimoprimeros": "61",
    "sexagesimoquinta": "65",
    "sexagesimoquintas": "65",
    "sexagesimoquinto": "65",
    "sexagesimoquintos": "65",
    "sexagesimosegunda": "62",
    "sexagesimosegundas": "62",
    "sexagesimosegundo": "62",
    "sexagesimosegundos": "62",
    "sexagesimosexta": "66",
    "sexagesimosextas": "66",
    "sexagesimosexto": "66",
    "sexagesimosextos": "66",
    "sexagesimoséptima": "67",
    "sexagesimoséptimas": "67",
    "sexagesimoséptimo": "67",
    "sexagesimoséptimos": "67",
    "sexagesimotercera": "63",
    "sexagesimoterceras": "63",
    "sexagesimotercero": "63",
    "sexagesimoterceros": "63",
    "sexagonales": "sexagonal",
    "sexagésima": "60",
    "sexagésimas": "60",
    "sexagésimo": "60",
    "sexagésimos": "60",
    "sexajes": "sexaje",
    "sexcentésima": "600",
    "sexcentésimas": "600",
    "sexcentésimo": "600",
    "sexcentésimos": "600",
    "sexenios": "sexenio",
    "sexis": "sexi",
    "sexismos": "sexismo",
    "sexistas": "sexista",
    "sexma": "sexmo",
    "sexmas": "sexmo",
    "sexmeros": "sexmero",
    "sexmos": "sexmo",
    "sexologías": "sexología",
    "sexos": "sexo",
    "sexshops": "sexshop",
    "sexsymbols": "sexsymbol",
    "sexta": "sexto",
    "sextantes": "sextante",
    "sextas": "sexto",
    "sextava": "sextavar",
    "sextavaba": "sextavar",
    "sextavabais": "sextavar",
    "sextavaban": "sextavar",
    "sextavabas": "sextavar",
    "sextavad": "sextavar",
    "sextavada": "sextavar",
    "sextavadas": "sextavar",
    "sextavado": "sextavar",
    "sextavados": "sextavar",
    "sextavamos": "sextavar",
    "sextavan": "sextavar",
    "sextavando": "sextavar",
    "sextavara": "sextavar",
    "sextavarais": "sextavar",
    "sextavaran": "sextavar",
    "sextavaras": "sextavar",
    "sextavare": "sextavar",
    "sextavareis": "sextavar",
    "sextavaremos": "sextavar",
    "sextavaren": "sextavar",
    "sextavares": "sextavar",
    "sextavaron": "sextavar",
    "sextavará": "sextavar",
    "sextavarán": "sextavar",
    "sextavarás": "sextavar",
    "sextavaré": "sextavar",
    "sextavaréis": "sextavar",
    "sextavaría": "sextavar",
    "sextavaríais": "sextavar",
    "sextavaríamos": "sextavar",
    "sextavarían": "sextavar",
    "sextavarías": "sextavar",
    "sextavas": "sextavar",
    "sextavase": "sextavar",
    "sextavaseis": "sextavar",
    "sextavasen": "sextavar",
    "sextavases": "sextavar",
    "sextavaste": "sextavar",
    "sextavasteis": "sextavar",
    "sextave": "sextavar",
    "sextavemos": "sextavar",
    "sextaven": "sextavar",
    "sextaves": "sextavar",
    "sextavo": "sextavar",
    "sextavábamos": "sextavar",
    "sextaváis": "sextavar",
    "sextaváramos": "sextavar",
    "sextaváremos": "sextavar",
    "sextavásemos": "sextavar",
    "sextavé": "sextavar",
    "sextavéis": "sextavar",
    "sextavó": "sextavar",
    "sextetos": "sexteto",
    "sextillas": "sextilla",
    "sextillos": "sextillo",
    "sextinas": "sextina",
    "sexto": "6",
    "sextos": "sexto",
    "sextuplica": "sextuplicar",
    "sextuplicaba": "sextuplicar",
    "sextuplicabais": "sextuplicar",
    "sextuplicaban": "sextuplicar",
    "sextuplicabas": "sextuplicar",
    "sextuplicaciones": "sextuplicación",
    "sextuplicad": "sextuplicar",
    "sextuplicada": "sextuplicar",
    "sextuplicadas": "sextuplicar",
    "sextuplicado": "sextuplicar",
    "sextuplicados": "sextuplicar",
    "sextuplicamos": "sextuplicar",
    "sextuplican": "sextuplicar",
    "sextuplicando": "sextuplicar",
    "sextuplicara": "sextuplicar",
    "sextuplicarais": "sextuplicar",
    "sextuplicaran": "sextuplicar",
    "sextuplicaras": "sextuplicar",
    "sextuplicare": "sextuplicar",
    "sextuplicareis": "sextuplicar",
    "sextuplicaremos": "sextuplicar",
    "sextuplicaren": "sextuplicar",
    "sextuplicares": "sextuplicar",
    "sextuplicaron": "sextuplicar",
    "sextuplicará": "sextuplicar",
    "sextuplicarán": "sextuplicar",
    "sextuplicarás": "sextuplicar",
    "sextuplicaré": "sextuplicar",
    "sextuplicaréis": "sextuplicar",
    "sextuplicaría": "sextuplicar",
    "sextuplicaríais": "sextuplicar",
    "sextuplicaríamos": "sextuplicar",
    "sextuplicarían": "sextuplicar",
    "sextuplicarías": "sextuplicar",
    "sextuplicas": "sextuplicar",
    "sextuplicase": "sextuplicar",
    "sextuplicaseis": "sextuplicar",
    "sextuplicasen": "sextuplicar",
    "sextuplicases": "sextuplicar",
    "sextuplicaste": "sextuplicar",
    "sextuplicasteis": "sextuplicar",
    "sextuplico": "sextuplicar",
    "sextuplicábamos": "sextuplicar",
    "sextuplicáis": "sextuplicar",
    "sextuplicáramos": "sextuplicar",
    "sextuplicáremos": "sextuplicar",
    "sextuplicásemos": "sextuplicar",
    "sextuplicó": "sextuplicar",
    "sextuplique": "sextuplicar",
    "sextupliquemos": "sextuplicar",
    "sextupliquen": "sextuplicar",
    "sextupliques": "sextuplicar",
    "sextupliqué": "sextuplicar",
    "sextupliquéis": "sextuplicar",
    "sexuada": "sexuado",
    "sexuadas": "sexuado",
    "sexuados": "sexuado",
    "sexuales": "sexual",
    "sexualidades": "sexualidad",
    "sexys": "sexy",
    "sexóloga": "sexólogo",
    "sexólogas": "sexólogo",
    "sexólogos": "sexólogo",
    "seáis": "ser",
    "seísmos": "seísmo",
    "señala": "señalar",
    "señalaba": "señalar",
    "señalabais": "señalar",
    "señalaban": "señalar",
    "señalabas": "señalar",
    "señalad": "señalar",
    "señalada": "señalar",
    "señaladas": "señalar",
    "señalado": "señalar",
    "señalados": "señalar",
    "señalamientos": "señalamiento",
    "señalamos": "señalar",
    "señalan": "señalar",
    "señalando": "señalar",
    "señalara": "señalar",
    "señalarais": "señalar",
    "señalaran": "señalar",
    "señalaras": "señalar",
    "señalare": "señalar",
    "señalareis": "señalar",
    "señalaremos": "señalar",
    "señalaren": "señalar",
    "señalares": "señalar",
    "señalaron": "señalar",
    "señalará": "señalar",
    "señalarán": "señalar",
    "señalarás": "señalar",
    "señalaré": "señalar",
    "señalaréis": "señalar",
    "señalaría": "señalar",
    "señalaríais": "señalar",
    "señalaríamos": "señalar",
    "señalarían": "señalar",
    "señalarías": "señalar",
    "señalas": "señalar",
    "señalase": "señalar",
    "señalaseis": "señalar",
    "señalasen": "señalar",
    "señalases": "señalar",
    "señalaste": "señalar",
    "señalasteis": "señalar",
    "señale": "señalar",
    "señalemos": "señalar",
    "señalen": "señalar",
    "señales": "señalar",
    "señalice": "señalizar",
    "señalicemos": "señalizar",
    "señalicen": "señalizar",
    "señalices": "señalizar",
    "señalicé": "señalizar",
    "señalicéis": "señalizar",
    "señaliza": "señalizar",
    "señalizaba": "señalizar",
    "señalizabais": "señalizar",
    "señalizaban": "señalizar",
    "señalizabas": "señalizar",
    "señalizaciones": "señalización",
    "señalizad": "señalizar",
    "señalizada": "señalizar",
    "señalizadas": "señalizar",
    "señalizado": "señalizar",
    "señalizadores": "señalizador",
    "señalizados": "señalizar",
    "señalizamos": "señalizar",
    "señalizan": "señalizar",
    "señalizando": "señalizar",
    "señalizara": "señalizar",
    "señalizarais": "señalizar",
    "señalizaran": "señalizar",
    "señalizaras": "señalizar",
    "señalizare": "señalizar",
    "señalizareis": "señalizar",
    "señalizaremos": "señalizar",
    "señalizaren": "señalizar",
    "señalizares": "señalizar",
    "señalizaron": "señalizar",
    "señalizará": "señalizar",
    "señalizarán": "señalizar",
    "señalizarás": "señalizar",
    "señalizaré": "señalizar",
    "señalizaréis": "señalizar",
    "señalizaría": "señalizar",
    "señalizaríais": "señalizar",
    "señalizaríamos": "señalizar",
    "señalizarían": "señalizar",
    "señalizarías": "señalizar",
    "señalizas": "señalizar",
    "señalizase": "señalizar",
    "señalizaseis": "señalizar",
    "señalizasen": "señalizar",
    "señalizases": "señalizar",
    "señalizaste": "señalizar",
    "señalizasteis": "señalizar",
    "señalizo": "señalizar",
    "señalizábamos": "señalizar",
    "señalizáis": "señalizar",
    "señalizáramos": "señalizar",
    "señalizáremos": "señalizar",
    "señalizásemos": "señalizar",
    "señalizó": "señalizar",
    "señalo": "señalar",
    "señalábamos": "señalar",
    "señaláis": "señalar",
    "señaláramos": "señalar",
    "señaláremos": "señalar",
    "señalásemos": "señalar",
    "señalé": "señalar",
    "señaléis": "señalar",
    "señaló": "señalar",
    "señas": "seña",
    "señera": "señero",
    "señeras": "señero",
    "señeros": "señero",
    "señora": "señor",
    "señoradas": "señorada",
    "señorajes": "señoraje",
    "señoras": "señor",
    "señorea": "señorear",
    "señoreaba": "señorear",
    "señoreabais": "señorear",
    "señoreaban": "señorear",
    "señoreabas": "señorear",
    "señoread": "señorear",
    "señoreada": "señorear",
    "señoreadas": "señorear",
    "señoreado": "señorear",
    "señoreadora": "señoreador",
    "señoreadoras": "señoreador",
    "señoreadores": "señoreador",
    "señoreados": "señorear",
    "señoreamos": "señorear",
    "señorean": "señorear",
    "señoreando": "señorear",
    "señoreara": "señorear",
    "señorearais": "señorear",
    "señorearan": "señorear",
    "señorearas": "señorear",
    "señoreare": "señorear",
    "señoreareis": "señorear",
    "señorearemos": "señorear",
    "señorearen": "señorear",
    "señoreares": "señorear",
    "señorearon": "señorear",
    "señoreará": "señorear",
    "señorearán": "señorear",
    "señorearás": "señorear",
    "señorearé": "señorear",
    "señorearéis": "señorear",
    "señorearía": "señorear",
    "señorearíais": "señorear",
    "señorearíamos": "señorear",
    "señorearían": "señorear",
    "señorearías": "señorear",
    "señoreas": "señorear",
    "señorease": "señorear",
    "señoreaseis": "señorear",
    "señoreasen": "señorear",
    "señoreases": "señorear",
    "señoreaste": "señorear",
    "señoreasteis": "señorear",
    "señoree": "señorear",
    "señoreemos": "señorear",
    "señoreen": "señorear",
    "señorees": "señorear",
    "señoreo": "señorear",
    "señores": "señor",
    "señoreábamos": "señorear",
    "señoreáis": "señorear",
    "señoreáramos": "señorear",
    "señoreáremos": "señorear",
    "señoreásemos": "señorear",
    "señoreé": "señorear",
    "señoreéis": "señorear",
    "señoreó": "señorear",
    "señoriales": "señorial",
    "señoriles": "señoril",
    "señorita": "señorito",
    "señoritas": "señorito",
    "señoritinga": "señoritingo",
    "señoritingas": "señoritingo",
    "señoritingos": "señoritingo",
    "señoritismos": "señoritismo",
    "señoritos": "señorito",
    "señorona": "señorón",
    "señoronas": "señorón",
    "señorones": "señorón",
    "señorías": "señoría",
    "señoríos": "señorío",
    "señuelos": "señuelo",
    "seós": "seó",
    "shakespeariana": "shakespeariano",
    "shakespearianas": "shakespeariano",
    "shakespearianos": "shakespeariano",
    "shantungs": "shantung",
    "shas": "sha",
    "shekeles": "shekel",
    "shekels": "shekel",
    "sheriffs": "sheriff",
    "sherpas": "sherpa",
    "sherries": "sherry",
    "shetlands": "shetland",
    "shiis": "shií",
    "shiíes": "shií",
    "shiís": "shií",
    "shocks": "shock",
    "shoguns": "shogun",
    "shorts": "short",
    "showmans": "showman",
    "shows": "show",
    "shunts": "shunt",
    "sialagogos": "sialagogo",
    "siales": "sial",
    "sialismos": "sialismo",
    "sialitas": "sialita",
    "sialitizacoónes": "sialitización",
    "sialoadenectomías": "sialoadenectomía",
    "sialoadenotomías": "sialoadenotomía",
    "sialoaerofagias": "sialoaerofagia",
    "sialoangiografías": "sialoangiografía",
    "sialoceles": "sialocele",
    "sialodocoplastias": "sialodocoplastia",
    "sialografías": "sialografía",
    "sialogramas": "sialograma",
    "sialolitos": "sialolito",
    "sialometaplasias": "sialometaplasia",
    "sialorreas": "sialorrea",
    "sialógena": "sialógeno",
    "sialógenas": "sialógeno",
    "sialógenos": "sialógeno",
    "siamangs": "siamang",
    "siamesa": "siamés",
    "siamesas": "siamés",
    "siameses": "siamés",
    "sibaritas": "sibarita",
    "sibaritismos": "sibaritismo",
    "sibarítica": "sibarítico",
    "sibaríticas": "sibarítico",
    "sibaríticos": "sibarítico",
    "siberiana": "siberiano",
    "siberianas": "siberiano",
    "siberianos": "siberiano",
    "siberitas": "siberita",
    "sibilantes": "sibilante",
    "sibiles": "sibil",
    "sibilina": "sibilino",
    "sibilinas": "sibilino",
    "sibilinos": "sibilino",
    "siboneyes": "siboney",
    "sibucaos": "sibucao",
    "sicalíptica": "sicalíptico",
    "sicalípticas": "sicalíptico",
    "sicalípticos": "sicalíptico",
    "sicambra": "sicambro",
    "sicambras": "sicambro",
    "sicambrios": "sicambrio",
    "sicambros": "sicambro",
    "sicamores": "sicamor",
    "sicana": "sicano",
    "sicanas": "sicano",
    "sicanos": "sicano",
    "sicarios": "sicario",
    "sicatividades": "sicatividad",
    "sicigias": "sicigia",
    "siciliana": "siciliano",
    "sicilianas": "siciliano",
    "sicilianos": "siciliano",
    "sicoanalistas": "sicoanalista",
    "sicoanalítica": "sicoanalítico",
    "sicoanalíticas": "sicoanalítico",
    "sicoanalíticos": "sicoanalítico",
    "sicofantas": "sicofanta",
    "sicofantes": "sicofante",
    "sicologías": "sicología",
    "sicológica": "sicológico",
    "sicológicas": "sicológico",
    "sicológicos": "sicológico",
    "sicomoros": "sicomoro",
    "sicopatías": "sicopatía",
    "sicoquímica": "psicoquímica",
    "sicoquímicas": "psicoquímica",
    "sicosis": "psicosis",
    "sicosomática": "sicosomático",
    "sicosomáticas": "sicosomático",
    "sicosomáticos": "sicosomático",
    "sicotecnias": "sicotecnia",
    "sicoterapias": "sicoterapia",
    "sicotécnica": "sicotécnico",
    "sicotécnicas": "sicotécnico",
    "sicotécnicos": "sicotécnico",
    "sicrómetro": "psicrómetro",
    "sicrómetros": "psicrómetro",
    "sicóloga": "sicólogo",
    "sicólogas": "sicólogo",
    "sicólogos": "sicólogo",
    "sicópatas": "sicópata",
    "sicósica": "sicósico",
    "sicósicas": "sicósico",
    "sicósicos": "sicósico",
    "sidas": "sida",
    "sidecares": "sidecar",
    "sidecars": "sidecar",
    "sideraciones": "sideración",
    "siderales": "sideral",
    "sideremias": "sideremia",
    "sideritas": "siderita",
    "sideritos": "siderito",
    "sideroblástica": "sideroblástico",
    "sideroblásticas": "sideroblástico",
    "sideroblásticos": "sideroblástico",
    "siderocitos": "siderocito",
    "siderofilinas": "siderofilina",
    "siderolitos": "siderolito",
    "siderometalúrgica": "siderometalúrgico",
    "siderometalúrgicas": "siderometalúrgico",
    "siderometalúrgicos": "siderometalúrgico",
    "sideropenias": "sideropenia",
    "sideropénica": "sideropénico",
    "sideropénicas": "sideropénico",
    "sideropénicos": "sideropénico",
    "siderostatos": "siderostato",
    "siderurgias": "siderurgia",
    "siderófila": "siderófilo",
    "siderófilas": "siderófilo",
    "siderófilos": "siderófilo",
    "siderúrgica": "siderúrgico",
    "siderúrgicas": "siderúrgico",
    "siderúrgicos": "siderúrgico",
    "sidis": "sidi",
    "sido": "ser",
    "sidonia": "sidonio",
    "sidonias": "sidonio",
    "sidonios": "sidonio",
    "sidras": "sidra",
    "sidé-spins": "side-spin",
    "sidérea": "sidéreo",
    "sidéreas": "sidéreo",
    "sidéreos": "sidéreo",
    "sidérica": "sidérico",
    "sidéricas": "sidérico",
    "sidéricos": "sidérico",
    "siega": "segar",
    "siegan": "segar",
    "siegas": "siega",
    "siego": "segar",
    "siegue": "segar",
    "sieguen": "segar",
    "siegues": "segar",
    "siembra": "sembrar",
    "siembran": "sembrar",
    "siembras": "siembra",
    "siembre": "sembrar",
    "siembren": "sembrar",
    "siembres": "sembrar",
    "siembro": "sembrar",
    "siempretiesos": "siempretieso",
    "siemprevivas": "siempreviva",
    "siendo": "ser",
    "sienes": "sien",
    "sienesa": "sienés",
    "sienesas": "sienés",
    "sieneses": "sienés",
    "sienitas": "sienita",
    "sienta": "sentir",
    "sientan": "sentir",
    "sientas": "sentir",
    "siente": "sentir",
    "sienten": "sentir",
    "sientes": "sentir",
    "siento": "sentir",
    "sienítica": "sienítico",
    "sieníticas": "sienítico",
    "sieníticos": "sienítico",
    "sierosemes": "sierosem",
    "sierpes": "sierpe",
    "sierra": "serrar",
    "sierraleonesa": "sierraleonés",
    "sierraleonesas": "sierraleonés",
    "sierraleoneses": "sierraleonés",
    "sierran": "serrar",
    "sierras": "sierra",
    "sierre": "serrar",
    "sierren": "serrar",
    "sierres": "serrar",
    "sierro": "serrar",
    "sierva": "siervo",
    "siervas": "siervo",
    "siervos": "siervo",
    "siesos": "sieso",
    "siestas": "siesta",
    "sieteenramas": "sieteenrama",
    "sietemesina": "sietemesino",
    "sietemesinas": "sietemesino",
    "sietemesinos": "sietemesino",
    "sietenramas": "sietenrama",
    "sieteñales": "sieteñal",
    "sifilografías": "sifilografía",
    "sifilográfica": "sifilográfico",
    "sifilográficas": "sifilográfico",
    "sifilográficos": "sifilográfico",
    "sifilomas": "sifiloma",
    "sifilítica": "sifilítico",
    "sifilíticas": "sifilítico",
    "sifilíticos": "sifilítico",
    "sifilógrafa": "sifilógrafo",
    "sifilógrafas": "sifilógrafo",
    "sifilógrafos": "sifilógrafo",
    "sifonales": "sifonal",
    "sifones": "sifón",
    "sifonocladales": "sifonocladal",
    "sifonóforos": "sifonóforo",
    "sifonógamos": "sifonógamo",
    "sifués": "sifué",
    "sifílides": "sifílide",
    "siga": "seguir",
    "sigamos": "seguir",
    "sigan": "seguir",
    "sigas": "seguir",
    "sigila": "sigilar",
    "sigilaba": "sigilar",
    "sigilabais": "sigilar",
    "sigilaban": "sigilar",
    "sigilabas": "sigilar",
    "sigilaciones": "sigilación",
    "sigilad": "sigilar",
    "sigilada": "sigilar",
    "sigiladas": "sigilar",
    "sigilado": "sigilar",
    "sigilados": "sigilar",
    "sigilamos": "sigilar",
    "sigilan": "sigilar",
    "sigilando": "sigilar",
    "sigilara": "sigilar",
    "sigilarais": "sigilar",
    "sigilaran": "sigilar",
    "sigilaras": "sigilar",
    "sigilare": "sigilar",
    "sigilareis": "sigilar",
    "sigilaremos": "sigilar",
    "sigilaren": "sigilar",
    "sigilares": "sigilar",
    "sigilaron": "sigilar",
    "sigilará": "sigilar",
    "sigilarán": "sigilar",
    "sigilarás": "sigilar",
    "sigilaré": "sigilar",
    "sigilaréis": "sigilar",
    "sigilaría": "sigilar",
    "sigilaríais": "sigilar",
    "sigilaríamos": "sigilar",
    "sigilarían": "sigilar",
    "sigilarías": "sigilar",
    "sigilas": "sigilar",
    "sigilase": "sigilar",
    "sigilaseis": "sigilar",
    "sigilasen": "sigilar",
    "sigilases": "sigilar",
    "sigilaste": "sigilar",
    "sigilasteis": "sigilar",
    "sigile": "sigilar",
    "sigilemos": "sigilar",
    "sigilen": "sigilar",
    "sigiles": "sigilar",
    "sigilo": "sigilar",
    "sigilografías": "sigilografía",
    "sigilos": "sigilo",
    "sigilosa": "sigiloso",
    "sigilosas": "sigiloso",
    "sigilosos": "sigiloso",
    "sigilábamos": "sigilar",
    "sigiláis": "sigilar",
    "sigiláramos": "sigilar",
    "sigiláremos": "sigilar",
    "sigilásemos": "sigilar",
    "sigilé": "sigilar",
    "sigiléis": "sigilar",
    "sigiló": "sigilar",
    "siglas": "sigla",
    "siglos": "siglo",
    "sigmas": "sigma",
    "sigmatismos": "sigmatismo",
    "sigmoidea": "sigmoideo",
    "sigmoideas": "sigmoideo",
    "sigmoideos": "sigmoideo",
    "sigmoides": "sigmoide",
    "signa": "signar",
    "signaba": "signar",
    "signabais": "signar",
    "signaban": "signar",
    "signabas": "signar",
    "signad": "signar",
    "signada": "signar",
    "signadas": "signar",
    "signado": "signar",
    "signados": "signar",
    "signamos": "signar",
    "signan": "signar",
    "signando": "signar",
    "signara": "signar",
    "signarais": "signar",
    "signaran": "signar",
    "signaras": "signar",
    "signare": "signar",
    "signareis": "signar",
    "signaremos": "signar",
    "signaren": "signar",
    "signares": "signar",
    "signaron": "signar",
    "signará": "signar",
    "signarán": "signar",
    "signarás": "signar",
    "signaré": "signar",
    "signaréis": "signar",
    "signaría": "signar",
    "signaríais": "signar",
    "signaríamos": "signar",
    "signarían": "signar",
    "signarías": "signar",
    "signas": "signar",
    "signase": "signar",
    "signaseis": "signar",
    "signasen": "signar",
    "signases": "signar",
    "signaste": "signar",
    "signasteis": "signar",
    "signataria": "signatario",
    "signatarias": "signatario",
    "signatarios": "signatario",
    "signaturas": "signatura",
    "signe": "signar",
    "signemos": "signar",
    "signen": "signar",
    "signes": "signar",
    "significa": "significar",
    "significaba": "significar",
    "significabais": "significar",
    "significaban": "significar",
    "significabas": "significar",
    "significaciones": "significación",
    "significad": "significar",
    "significada": "significar",
    "significadas": "significar",
    "significado": "significar",
    "significados": "significar",
    "significamos": "significar",
    "significan": "significar",
    "significancias": "significancia",
    "significando": "significar",
    "significantes": "significante",
    "significara": "significar",
    "significarais": "significar",
    "significaran": "significar",
    "significaras": "significar",
    "significare": "significar",
    "significareis": "significar",
    "significaremos": "significar",
    "significaren": "significar",
    "significares": "significar",
    "significaron": "significar",
    "significará": "significar",
    "significarán": "significar",
    "significarás": "significar",
    "significaré": "significar",
    "significaréis": "significar",
    "significaría": "significar",
    "significaríais": "significar",
    "significaríamos": "significar",
    "significarían": "significar",
    "significarías": "significar",
    "significas": "significar",
    "significase": "significar",
    "significaseis": "significar",
    "significasen": "significar",
    "significases": "significar",
    "significaste": "significar",
    "significasteis": "significar",
    "significativa": "significativo",
    "significativas": "significativo",
    "significativos": "significativo",
    "significo": "significar",
    "significábamos": "significar",
    "significáis": "significar",
    "significáramos": "significar",
    "significáremos": "significar",
    "significásemos": "significar",
    "significó": "significar",
    "signifique": "significar",
    "signifiquemos": "significar",
    "signifiquen": "significar",
    "signifiques": "significar",
    "signifiqué": "significar",
    "signifiquéis": "significar",
    "signo": "signar",
    "signos": "signo",
    "signábamos": "signar",
    "signáculos": "signáculo",
    "signáis": "signar",
    "signáramos": "signar",
    "signáremos": "signar",
    "signásemos": "signar",
    "signé": "signar",
    "signéis": "signar",
    "signífera": "signífero",
    "signíferas": "signífero",
    "signíferos": "signífero",
    "signó": "signar",
    "sigo": "seguir",
    "sigue": "seguir",
    "siguen": "seguir",
    "sigues": "seguir",
    "siguiendo": "seguir",
    "siguientes": "siguiente",
    "siguiera": "seguir",
    "siguierais": "seguir",
    "siguieran": "seguir",
    "siguieras": "seguir",
    "siguiere": "seguir",
    "siguiereis": "seguir",
    "siguieren": "seguir",
    "siguieres": "seguir",
    "siguieron": "seguir",
    "siguiese": "seguir",
    "siguieseis": "seguir",
    "siguiesen": "seguir",
    "siguieses": "seguir",
    "siguiéramos": "seguir",
    "siguiéremos": "seguir",
    "siguiésemos": "seguir",
    "siguió": "seguir",
    "sigáis": "seguir",
    "silaba": "silabar",
    "silababa": "silabar",
    "silababais": "silabar",
    "silababan": "silabar",
    "silababas": "silabar",
    "silabad": "silabar",
    "silabada": "silabar",
    "silabadas": "silabar",
    "silabado": "silabar",
    "silabados": "silabar",
    "silabamos": "silabar",
    "silaban": "silabar",
    "silabando": "silabar",
    "silabara": "silabar",
    "silabarais": "silabar",
    "silabaran": "silabar",
    "silabaras": "silabar",
    "silabare": "silabar",
    "silabareis": "silabar",
    "silabaremos": "silabar",
    "silabaren": "silabar",
    "silabares": "silabar",
    "silabarios": "silabario",
    "silabaron": "silabar",
    "silabará": "silabar",
    "silabarán": "silabar",
    "silabarás": "silabar",
    "silabaré": "silabar",
    "silabaréis": "silabar",
    "silabaría": "silabar",
    "silabaríais": "silabar",
    "silabaríamos": "silabar",
    "silabarían": "silabar",
    "silabarías": "silabar",
    "silabas": "silabar",
    "silabase": "silabar",
    "silabaseis": "silabar",
    "silabasen": "silabar",
    "silabases": "silabar",
    "silabaste": "silabar",
    "silabasteis": "silabar",
    "silabe": "silabar",
    "silabea": "silabear",
    "silabeaba": "silabear",
    "silabeabais": "silabear",
    "silabeaban": "silabear",
    "silabeabas": "silabear",
    "silabead": "silabear",
    "silabeada": "silabear",
    "silabeadas": "silabear",
    "silabeado": "silabear",
    "silabeados": "silabear",
    "silabeamos": "silabear",
    "silabean": "silabear",
    "silabeando": "silabear",
    "silabeara": "silabear",
    "silabearais": "silabear",
    "silabearan": "silabear",
    "silabearas": "silabear",
    "silabeare": "silabear",
    "silabeareis": "silabear",
    "silabearemos": "silabear",
    "silabearen": "silabear",
    "silabeares": "silabear",
    "silabearon": "silabear",
    "silabeará": "silabear",
    "silabearán": "silabear",
    "silabearás": "silabear",
    "silabearé": "silabear",
    "silabearéis": "silabear",
    "silabearía": "silabear",
    "silabearíais": "silabear",
    "silabearíamos": "silabear",
    "silabearían": "silabear",
    "silabearías": "silabear",
    "silabeas": "silabear",
    "silabease": "silabear",
    "silabeaseis": "silabear",
    "silabeasen": "silabear",
    "silabeases": "silabear",
    "silabeaste": "silabear",
    "silabeasteis": "silabear",
    "silabee": "silabear",
    "silabeemos": "silabear",
    "silabeen": "silabear",
    "silabees": "silabear",
    "silabemos": "silabar",
    "silaben": "silabar",
    "silabeo": "silabear",
    "silabeos": "silabeo",
    "silabes": "silabar",
    "silabeábamos": "silabear",
    "silabeáis": "silabear",
    "silabeáramos": "silabear",
    "silabeáremos": "silabear",
    "silabeásemos": "silabear",
    "silabeé": "silabear",
    "silabeéis": "silabear",
    "silabeó": "silabear",
    "silabo": "silabar",
    "silabábamos": "silabar",
    "silabáis": "silabar",
    "silabáramos": "silabar",
    "silabáremos": "silabar",
    "silabásemos": "silabar",
    "silabé": "silabar",
    "silabéis": "silabar",
    "silabó": "silabar",
    "silanos": "silano",
    "silba": "silbar",
    "silbaba": "silbar",
    "silbabais": "silbar",
    "silbaban": "silbar",
    "silbabas": "silbar",
    "silbad": "silbar",
    "silbada": "silbar",
    "silbadas": "silbar",
    "silbado": "silbar",
    "silbadora": "silbador",
    "silbadoras": "silbador",
    "silbadores": "silbador",
    "silbados": "silbar",
    "silbamos": "silbar",
    "silban": "silbar",
    "silbando": "silbar",
    "silbara": "silbar",
    "silbarais": "silbar",
    "silbaran": "silbar",
    "silbaras": "silbar",
    "silbare": "silbar",
    "silbareis": "silbar",
    "silbaremos": "silbar",
    "silbaren": "silbar",
    "silbares": "silbar",
    "silbaron": "silbar",
    "silbará": "silbar",
    "silbarán": "silbar",
    "silbarás": "silbar",
    "silbaré": "silbar",
    "silbaréis": "silbar",
    "silbaría": "silbar",
    "silbaríais": "silbar",
    "silbaríamos": "silbar",
    "silbarían": "silbar",
    "silbarías": "silbar",
    "silbas": "silbar",
    "silbase": "silbar",
    "silbaseis": "silbar",
    "silbasen": "silbar",
    "silbases": "silbar",
    "silbaste": "silbar",
    "silbasteis": "silbar",
    "silbatos": "silbato",
    "silbe": "silbar",
    "silbemos": "silbar",
    "silben": "silbar",
    "silbes": "silbar",
    "silbidos": "silbido",
    "silbo": "silbar",
    "silbos": "silbo",
    "silbosa": "silboso",
    "silbosas": "silboso",
    "silbosos": "silboso",
    "silbábamos": "silbar",
    "silbáis": "silbar",
    "silbáramos": "silbar",
    "silbáremos": "silbar",
    "silbásemos": "silbar",
    "silbé": "silbar",
    "silbéis": "silbar",
    "silbó": "silbar",
    "silencia": "silenciar",
    "silenciaba": "silenciar",
    "silenciabais": "silenciar",
    "silenciaban": "silenciar",
    "silenciabas": "silenciar",
    "silenciad": "silenciar",
    "silenciada": "silenciar",
    "silenciadas": "silenciar",
    "silenciado": "silenciar",
    "silenciadores": "silenciador",
    "silenciados": "silenciar",
    "silenciamos": "silenciar",
    "silencian": "silenciar",
    "silenciando": "silenciar",
    "silenciara": "silenciar",
    "silenciarais": "silenciar",
    "silenciaran": "silenciar",
    "silenciaras": "silenciar",
    "silenciare": "silenciar",
    "silenciareis": "silenciar",
    "silenciaremos": "silenciar",
    "silenciaren": "silenciar",
    "silenciares": "silenciar",
    "silenciaria": "silenciario",
    "silenciarias": "silenciario",
    "silenciarios": "silenciario",
    "silenciaron": "silenciar",
    "silenciará": "silenciar",
    "silenciarán": "silenciar",
    "silenciarás": "silenciar",
    "silenciaré": "silenciar",
    "silenciaréis": "silenciar",
    "silenciaría": "silenciar",
    "silenciaríais": "silenciar",
    "silenciaríamos": "silenciar",
    "silenciarían": "silenciar",
    "silenciarías": "silenciar",
    "silencias": "silenciar",
    "silenciase": "silenciar",
    "silenciaseis": "silenciar",
    "silenciasen": "silenciar",
    "silenciases": "silenciar",
    "silenciaste": "silenciar",
    "silenciasteis": "silenciar",
    "silencie": "silenciar",
    "silenciemos": "silenciar",
    "silencien": "silenciar",
    "silenciera": "silenciero",
    "silencieras": "silenciero",
    "silencieros": "silenciero",
    "silencies": "silenciar",
    "silencio": "silenciar",
    "silencios": "silencio",
    "silenciosa": "silencioso",
    "silenciosas": "silencioso",
    "silenciosos": "silencioso",
    "silenciábamos": "silenciar",
    "silenciáis": "silenciar",
    "silenciáramos": "silenciar",
    "silenciáremos": "silenciar",
    "silenciásemos": "silenciar",
    "silencié": "silenciar",
    "silenciéis": "silenciar",
    "silenció": "silenciar",
    "silenos": "sileno",
    "silentes": "silente",
    "sileros": "silero",
    "silerías": "silería",
    "siles": "sil",
    "silesia": "silesio",
    "silesiana": "silesiano",
    "silesianas": "silesiano",
    "silesianos": "silesiano",
    "silesias": "silesio",
    "silesios": "silesio",
    "silfos": "silfo",
    "silgas": "silga",
    "silicataciones": "silicatación",
    "silicatos": "silicato",
    "silicificaciones": "silicificación",
    "silicios": "silicio",
    "siliciuraciones": "siliciuración",
    "siliciuros": "siliciuro",
    "silicoaluminatos": "silicoaluminato",
    "siliconas": "silicona",
    "silicotúngstica": "silicotúngstico",
    "silicotúngsticas": "silicotúngstico",
    "silicotúngsticos": "silicotúngstico",
    "silicuas": "silicua",
    "silicuosa": "silicuoso",
    "silicuosas": "silicuoso",
    "silicuosos": "silicuoso",
    "silicícolas": "silicícola",
    "silicótica": "silicótico",
    "silicóticas": "silicótico",
    "silicóticos": "silicótico",
    "silimanitas": "silimanita",
    "silladas": "sillada",
    "sillarejos": "sillarejo",
    "sillares": "sillar",
    "sillas": "silla",
    "sillera": "sillero",
    "silleras": "sillero",
    "silleros": "sillero",
    "sillerías": "sillería",
    "silletas": "silleta",
    "silletazos": "silletazo",
    "silleteros": "silletero",
    "sillicos": "sillico",
    "sillimanitas": "sillimanita",
    "sillines": "sillín",
    "sillones": "sillón",
    "silogice": "silogizar",
    "silogicemos": "silogizar",
    "silogicen": "silogizar",
    "silogices": "silogizar",
    "silogicé": "silogizar",
    "silogicéis": "silogizar",
    "silogismos": "silogismo",
    "silogiza": "silogizar",
    "silogizaba": "silogizar",
    "silogizabais": "silogizar",
    "silogizaban": "silogizar",
    "silogizabas": "silogizar",
    "silogizad": "silogizar",
    "silogizada": "silogizar",
    "silogizadas": "silogizar",
    "silogizado": "silogizar",
    "silogizados": "silogizar",
    "silogizamos": "silogizar",
    "silogizan": "silogizar",
    "silogizando": "silogizar",
    "silogizara": "silogizar",
    "silogizarais": "silogizar",
    "silogizaran": "silogizar",
    "silogizaras": "silogizar",
    "silogizare": "silogizar",
    "silogizareis": "silogizar",
    "silogizaremos": "silogizar",
    "silogizaren": "silogizar",
    "silogizares": "silogizar",
    "silogizaron": "silogizar",
    "silogizará": "silogizar",
    "silogizarán": "silogizar",
    "silogizarás": "silogizar",
    "silogizaré": "silogizar",
    "silogizaréis": "silogizar",
    "silogizaría": "silogizar",
    "silogizaríais": "silogizar",
    "silogizaríamos": "silogizar",
    "silogizarían": "silogizar",
    "silogizarías": "silogizar",
    "silogizas": "silogizar",
    "silogizase": "silogizar",
    "silogizaseis": "silogizar",
    "silogizasen": "silogizar",
    "silogizases": "silogizar",
    "silogizaste": "silogizar",
    "silogizasteis": "silogizar",
    "silogizo": "silogizar",
    "silogizábamos": "silogizar",
    "silogizáis": "silogizar",
    "silogizáramos": "silogizar",
    "silogizáremos": "silogizar",
    "silogizásemos": "silogizar",
    "silogizó": "silogizar",
    "silogística": "silogístico",
    "silogísticas": "silogístico",
    "silogísticos": "silogístico",
    "silos": "silo",
    "siloxanos": "siloxano",
    "siluetas": "silueta",
    "silueteada": "silueteado",
    "silueteadas": "silueteado",
    "silueteados": "silueteado",
    "siluriana": "siluriano",
    "silurianas": "siluriano",
    "silurianos": "siluriano",
    "siluros": "siluro",
    "silvanitas": "silvanita",
    "silvas": "silva",
    "silvestres": "silvestre",
    "silvicultora": "silvicultor",
    "silvicultoras": "silvicultor",
    "silvicultores": "silvicultor",
    "silviculturas": "silvicultura",
    "silvinas": "silvina",
    "silvinitas": "silvinita",
    "silvitas": "silvita",
    "silvícolas": "silvícola",
    "silábica": "silábico",
    "silábicas": "silábico",
    "silábicos": "silábico",
    "silícea": "silíceo",
    "silíceas": "silíceo",
    "silíceos": "silíceo",
    "silícica": "silícico",
    "silícicas": "silícico",
    "silícicos": "silícico",
    "silículas": "silícula",
    "silúrica": "silúrico",
    "silúricas": "silúrico",
    "silúricos": "silúrico",
    "simarubas": "simaruba",
    "simarubácea": "simarubáceo",
    "simarubáceas": "simarubáceo",
    "simarubáceos": "simarubáceo",
    "simas": "sima",
    "simbiontes": "simbionte",
    "simbiótica": "simbiótico",
    "simbióticas": "simbiótico",
    "simbióticos": "simbiótico",
    "simbolice": "simbolizar",
    "simbolicemos": "simbolizar",
    "simbolicen": "simbolizar",
    "simbolices": "simbolizar",
    "simbolicé": "simbolizar",
    "simbolicéis": "simbolizar",
    "simbolismos": "simbolismo",
    "simbolistas": "simbolista",
    "simboliza": "simbolizar",
    "simbolizaba": "simbolizar",
    "simbolizabais": "simbolizar",
    "simbolizaban": "simbolizar",
    "simbolizabas": "simbolizar",
    "simbolizables": "simbolizable",
    "simbolizaciones": "simbolización",
    "simbolizad": "simbolizar",
    "simbolizada": "simbolizar",
    "simbolizadas": "simbolizar",
    "simbolizado": "simbolizar",
    "simbolizados": "simbolizar",
    "simbolizamos": "simbolizar",
    "simbolizan": "simbolizar",
    "simbolizando": "simbolizar",
    "simbolizara": "simbolizar",
    "simbolizarais": "simbolizar",
    "simbolizaran": "simbolizar",
    "simbolizaras": "simbolizar",
    "simbolizare": "simbolizar",
    "simbolizareis": "simbolizar",
    "simbolizaremos": "simbolizar",
    "simbolizaren": "simbolizar",
    "simbolizares": "simbolizar",
    "simbolizaron": "simbolizar",
    "simbolizará": "simbolizar",
    "simbolizarán": "simbolizar",
    "simbolizarás": "simbolizar",
    "simbolizaré": "simbolizar",
    "simbolizaréis": "simbolizar",
    "simbolizaría": "simbolizar",
    "simbolizaríais": "simbolizar",
    "simbolizaríamos": "simbolizar",
    "simbolizarían": "simbolizar",
    "simbolizarías": "simbolizar",
    "simbolizas": "simbolizar",
    "simbolizase": "simbolizar",
    "simbolizaseis": "simbolizar",
    "simbolizasen": "simbolizar",
    "simbolizases": "simbolizar",
    "simbolizaste": "simbolizar",
    "simbolizasteis": "simbolizar",
    "simbolizo": "simbolizar",
    "simbolizábamos": "simbolizar",
    "simbolizáis": "simbolizar",
    "simbolizáramos": "simbolizar",
    "simbolizáremos": "simbolizar",
    "simbolizásemos": "simbolizar",
    "simbolizó": "simbolizar",
    "simbologías": "simbología",
    "simbólica": "simbólico",
    "simbólicas": "simbólico",
    "simbólicos": "simbólico",
    "simetitas": "simetita",
    "simetrías": "simetría",
    "simia": "simio",
    "simias": "simio",
    "simientes": "simiente",
    "simiesca": "simiesco",
    "simiescas": "simiesco",
    "simiescos": "simiesco",
    "similares": "similar",
    "similigrabados": "similigrabado",
    "similitudes": "similitud",
    "similitudinaria": "similitudinario",
    "similitudinarias": "similitudinario",
    "similitudinarios": "similitudinario",
    "simios": "simio",
    "simones": "simón",
    "simoniaca": "simoníaco",
    "simoniacas": "simoníaco",
    "simoniacos": "simoníaco",
    "simoniática": "simoniático",
    "simoniáticas": "simoniático",
    "simoniáticos": "simoniático",
    "simoníaca": "simoníaco",
    "simoníacas": "simoníaco",
    "simoníacos": "simoníaco",
    "simonías": "simonía",
    "simpatectomías": "simpatectomía",
    "simpatice": "simpatizar",
    "simpaticemos": "simpatizar",
    "simpaticen": "simpatizar",
    "simpatices": "simpatizar",
    "simpaticolítica": "simpaticolítico",
    "simpaticolíticas": "simpaticolítico",
    "simpaticolíticos": "simpaticolítico",
    "simpaticomimética": "simpaticomimético",
    "simpaticomiméticas": "simpaticomimético",
    "simpaticomiméticos": "simpaticomimético",
    "simpaticé": "simpatizar",
    "simpaticéis": "simpatizar",
    "simpatiza": "simpatizar",
    "simpatizaba": "simpatizar",
    "simpatizabais": "simpatizar",
    "simpatizaban": "simpatizar",
    "simpatizabas": "simpatizar",
    "simpatizad": "simpatizar",
    "simpatizada": "simpatizar",
    "simpatizadas": "simpatizar",
    "simpatizado": "simpatizar",
    "simpatizadora": "simpatizador",
    "simpatizadoras": "simpatizador",
    "simpatizadores": "simpatizador",
    "simpatizados": "simpatizar",
    "simpatizamos": "simpatizar",
    "simpatizan": "simpatizar",
    "simpatizando": "simpatizar",
    "simpatizantes": "simpatizante",
    "simpatizara": "simpatizar",
    "simpatizarais": "simpatizar",
    "simpatizaran": "simpatizar",
    "simpatizaras": "simpatizar",
    "simpatizare": "simpatizar",
    "simpatizareis": "simpatizar",
    "simpatizaremos": "simpatizar",
    "simpatizaren": "simpatizar",
    "simpatizares": "simpatizar",
    "simpatizaron": "simpatizar",
    "simpatizará": "simpatizar",
    "simpatizarán": "simpatizar",
    "simpatizarás": "simpatizar",
    "simpatizaré": "simpatizar",
    "simpatizaréis": "simpatizar",
    "simpatizaría": "simpatizar",
    "simpatizaríais": "simpatizar",
    "simpatizaríamos": "simpatizar",
    "simpatizarían": "simpatizar",
    "simpatizarías": "simpatizar",
    "simpatizas": "simpatizar",
    "simpatizase": "simpatizar",
    "simpatizaseis": "simpatizar",
    "simpatizasen": "simpatizar",
    "simpatizases": "simpatizar",
    "simpatizaste": "simpatizar",
    "simpatizasteis": "simpatizar",
    "simpatizo": "simpatizar",
    "simpatizábamos": "simpatizar",
    "simpatizáis": "simpatizar",
    "simpatizáramos": "simpatizar",
    "simpatizáremos": "simpatizar",
    "simpatizásemos": "simpatizar",
    "simpatizó": "simpatizar",
    "simpatolítica": "simpatolítico",
    "simpatolíticas": "simpatolítico",
    "simpatolíticos": "simpatolítico",
    "simpatomas": "simpatoma",
    "simpatías": "simpatía",
    "simplastos": "simplasto",
    "simples": "simple",
    "simplezas": "simpleza",
    "simpliciales": "simplicial",
    "simplicidades": "simplicidad",
    "simplicistas": "simplicista",
    "simplifica": "simplificar",
    "simplificaba": "simplificar",
    "simplificabais": "simplificar",
    "simplificaban": "simplificar",
    "simplificabas": "simplificar",
    "simplificables": "simplificable",
    "simplificaciones": "simplificación",
    "simplificad": "simplificar",
    "simplificada": "simplificar",
    "simplificadas": "simplificar",
    "simplificado": "simplificar",
    "simplificadora": "simplificador",
    "simplificadoras": "simplificador",
    "simplificadores": "simplificador",
    "simplificados": "simplificar",
    "simplificamos": "simplificar",
    "simplifican": "simplificar",
    "simplificando": "simplificar",
    "simplificara": "simplificar",
    "simplificarais": "simplificar",
    "simplificaran": "simplificar",
    "simplificaras": "simplificar",
    "simplificare": "simplificar",
    "simplificareis": "simplificar",
    "simplificaremos": "simplificar",
    "simplificaren": "simplificar",
    "simplificares": "simplificar",
    "simplificaron": "simplificar",
    "simplificará": "simplificar",
    "simplificarán": "simplificar",
    "simplificarás": "simplificar",
    "simplificaré": "simplificar",
    "simplificaréis": "simplificar",
    "simplificaría": "simplificar",
    "simplificaríais": "simplificar",
    "simplificaríamos": "simplificar",
    "simplificarían": "simplificar",
    "simplificarías": "simplificar",
    "simplificas": "simplificar",
    "simplificase": "simplificar",
    "simplificaseis": "simplificar",
    "simplificasen": "simplificar",
    "simplificases": "simplificar",
    "simplificaste": "simplificar",
    "simplificasteis": "simplificar",
    "simplifico": "simplificar",
    "simplificábamos": "simplificar",
    "simplificáis": "simplificar",
    "simplificáramos": "simplificar",
    "simplificáremos": "simplificar",
    "simplificásemos": "simplificar",
    "simplificó": "simplificar",
    "simplifique": "simplificar",
    "simplifiquemos": "simplificar",
    "simplifiquen": "simplificar",
    "simplifiques": "simplificar",
    "simplifiqué": "simplificar",
    "simplifiquéis": "simplificar",
    "simplistas": "simplista",
    "simplocáceas": "simplocácea",
    "simplona": "simplón",
    "simplonas": "simplón",
    "simplones": "simplón",
    "simplécticas": "simpléctica",
    "simpodios": "simpodio",
    "simposios": "simposio",
    "simposiums": "simposium",
    "simpática": "simpático",
    "simpáticas": "simpático",
    "simpáticos": "simpático",
    "simpétala": "simpétalo",
    "simpétalas": "simpétalo",
    "simpétalos": "simpétalo",
    "simula": "simular",
    "simulaba": "simular",
    "simulabais": "simular",
    "simulaban": "simular",
    "simulabas": "simular",
    "simulaciones": "simulación",
    "simulacros": "simulacro",
    "simulad": "simular",
    "simulada": "simular",
    "simuladas": "simular",
    "simulado": "simular",
    "simuladora": "simulador",
    "simuladoras": "simulador",
    "simuladores": "simulador",
    "simulados": "simular",
    "simulamos": "simular",
    "simulan": "simular",
    "simulando": "simular",
    "simulara": "simular",
    "simularais": "simular",
    "simularan": "simular",
    "simularas": "simular",
    "simulare": "simular",
    "simulareis": "simular",
    "simularemos": "simular",
    "simularen": "simular",
    "simulares": "simular",
    "simularon": "simular",
    "simulará": "simular",
    "simularán": "simular",
    "simularás": "simular",
    "simularé": "simular",
    "simularéis": "simular",
    "simularía": "simular",
    "simularíais": "simular",
    "simularíamos": "simular",
    "simularían": "simular",
    "simularías": "simular",
    "simulas": "simular",
    "simulase": "simular",
    "simulaseis": "simular",
    "simulasen": "simular",
    "simulases": "simular",
    "simulaste": "simular",
    "simulasteis": "simular",
    "simule": "simular",
    "simulemos": "simular",
    "simulen": "simular",
    "simules": "simular",
    "simulo": "simular",
    "simultanea": "simultanear",
    "simultaneaba": "simultanear",
    "simultaneabais": "simultanear",
    "simultaneaban": "simultanear",
    "simultaneabas": "simultanear",
    "simultanead": "simultanear",
    "simultaneada": "simultanear",
    "simultaneadas": "simultanear",
    "simultaneado": "simultanear",
    "simultaneados": "simultanear",
    "simultaneamos": "simultanear",
    "simultanean": "simultanear",
    "simultaneando": "simultanear",
    "simultaneara": "simultanear",
    "simultanearais": "simultanear",
    "simultanearan": "simultanear",
    "simultanearas": "simultanear",
    "simultaneare": "simultanear",
    "simultaneareis": "simultanear",
    "simultanearemos": "simultanear",
    "simultanearen": "simultanear",
    "simultaneares": "simultanear",
    "simultanearon": "simultanear",
    "simultaneará": "simultanear",
    "simultanearán": "simultanear",
    "simultanearás": "simultanear",
    "simultanearé": "simultanear",
    "simultanearéis": "simultanear",
    "simultanearía": "simultanear",
    "simultanearíais": "simultanear",
    "simultanearíamos": "simultanear",
    "simultanearían": "simultanear",
    "simultanearías": "simultanear",
    "simultaneas": "simultanear",
    "simultanease": "simultanear",
    "simultaneaseis": "simultanear",
    "simultaneasen": "simultanear",
    "simultaneases": "simultanear",
    "simultaneaste": "simultanear",
    "simultaneasteis": "simultanear",
    "simultanee": "simultanear",
    "simultaneemos": "simultanear",
    "simultaneen": "simultanear",
    "simultanees": "simultanear",
    "simultaneidades": "simultaneidad",
    "simultaneo": "simultanear",
    "simultaneábamos": "simultanear",
    "simultaneáis": "simultanear",
    "simultaneáramos": "simultanear",
    "simultaneáremos": "simultanear",
    "simultaneásemos": "simultanear",
    "simultaneé": "simultanear",
    "simultaneéis": "simultanear",
    "simultaneó": "simultanear",
    "simultánea": "simultáneo",
    "simultáneas": "simultáneo",
    "simultáneos": "simultáneo",
    "simulábamos": "simular",
    "simuláis": "simular",
    "simuláramos": "simular",
    "simuláremos": "simular",
    "simulásemos": "simular",
    "simulé": "simular",
    "simuléis": "simular",
    "simuló": "simular",
    "simunes": "simún",
    "simétrica": "simétrico",
    "simétricas": "simétrico",
    "simétricos": "simétrico",
    "sinagogas": "sinagoga",
    "sinalagmática": "sinalagmático",
    "sinalagmáticas": "sinalagmático",
    "sinalagmáticos": "sinalagmático",
    "sinalefas": "sinalefa",
    "sinalgias": "sinalgia",
    "sinandras": "sinandra",
    "sinapismos": "sinapismo",
    "sinaptinémica": "sinaptinémico",
    "sinaptinémicas": "sinaptinémico",
    "sinaptinémicos": "sinaptinémico",
    "sincarpos": "sincarpo",
    "sincategoremática": "sincategoremático",
    "sincategoremáticas": "sincategoremático",
    "sincategoremáticos": "sincategoremático",
    "sincera": "sincero",
    "sinceraba": "sincerar",
    "sincerabais": "sincerar",
    "sinceraban": "sincerar",
    "sincerabas": "sincerar",
    "sincerad": "sincerar",
    "sincerada": "sincerar",
    "sinceradas": "sincerar",
    "sincerado": "sincerar",
    "sincerados": "sincerar",
    "sinceramos": "sincerar",
    "sinceran": "sincerar",
    "sincerando": "sincerar",
    "sincerara": "sincerar",
    "sincerarais": "sincerar",
    "sinceraran": "sincerar",
    "sinceraras": "sincerar",
    "sincerare": "sincerar",
    "sincerareis": "sincerar",
    "sinceraremos": "sincerar",
    "sinceraren": "sincerar",
    "sincerares": "sincerar",
    "sinceraron": "sincerar",
    "sincerará": "sincerar",
    "sincerarán": "sincerar",
    "sincerarás": "sincerar",
    "sinceraré": "sincerar",
    "sinceraréis": "sincerar",
    "sinceraría": "sincerar",
    "sinceraríais": "sincerar",
    "sinceraríamos": "sincerar",
    "sincerarían": "sincerar",
    "sincerarías": "sincerar",
    "sinceras": "sincero",
    "sincerase": "sincerar",
    "sinceraseis": "sincerar",
    "sincerasen": "sincerar",
    "sincerases": "sincerar",
    "sinceraste": "sincerar",
    "sincerasteis": "sincerar",
    "sincere": "sincerar",
    "sinceremos": "sincerar",
    "sinceren": "sincerar",
    "sinceres": "sincerar",
    "sinceridades": "sinceridad",
    "sincero": "sincerar",
    "sinceros": "sincero",
    "sincerábamos": "sincerar",
    "sinceráis": "sincerar",
    "sinceráramos": "sincerar",
    "sinceráremos": "sincerar",
    "sincerásemos": "sincerar",
    "sinceré": "sincerar",
    "sinceréis": "sincerar",
    "sinceró": "sincerar",
    "sincitios": "sincitio",
    "sinclinales": "sinclinal",
    "sincopa": "sincopar",
    "sincopaba": "sincopar",
    "sincopabais": "sincopar",
    "sincopaban": "sincopar",
    "sincopabas": "sincopar",
    "sincopad": "sincopar",
    "sincopada": "sincopar",
    "sincopadas": "sincopar",
    "sincopado": "sincopar",
    "sincopados": "sincopar",
    "sincopales": "sincopal",
    "sincopamos": "sincopar",
    "sincopan": "sincopar",
    "sincopando": "sincopar",
    "sincopara": "sincopar",
    "sincoparais": "sincopar",
    "sincoparan": "sincopar",
    "sincoparas": "sincopar",
    "sincopare": "sincopar",
    "sincopareis": "sincopar",
    "sincoparemos": "sincopar",
    "sincoparen": "sincopar",
    "sincopares": "sincopar",
    "sincoparon": "sincopar",
    "sincopará": "sincopar",
    "sincoparán": "sincopar",
    "sincoparás": "sincopar",
    "sincoparé": "sincopar",
    "sincoparéis": "sincopar",
    "sincoparía": "sincopar",
    "sincoparíais": "sincopar",
    "sincoparíamos": "sincopar",
    "sincoparían": "sincopar",
    "sincoparías": "sincopar",
    "sincopas": "sincopar",
    "sincopase": "sincopar",
    "sincopaseis": "sincopar",
    "sincopasen": "sincopar",
    "sincopases": "sincopar",
    "sincopaste": "sincopar",
    "sincopasteis": "sincopar",
    "sincope": "sincopar",
    "sincopemos": "sincopar",
    "sincopen": "sincopar",
    "sincopes": "sincopar",
    "sincopice": "sincopizar",
    "sincopicemos": "sincopizar",
    "sincopicen": "sincopizar",
    "sincopices": "sincopizar",
    "sincopicé": "sincopizar",
    "sincopicéis": "sincopizar",
    "sincopiza": "sincopizar",
    "sincopizaba": "sincopizar",
    "sincopizabais": "sincopizar",
    "sincopizaban": "sincopizar",
    "sincopizabas": "sincopizar",
    "sincopizad": "sincopizar",
    "sincopizada": "sincopizar",
    "sincopizadas": "sincopizar",
    "sincopizado": "sincopizar",
    "sincopizados": "sincopizar",
    "sincopizamos": "sincopizar",
    "sincopizan": "sincopizar",
    "sincopizando": "sincopizar",
    "sincopizara": "sincopizar",
    "sincopizarais": "sincopizar",
    "sincopizaran": "sincopizar",
    "sincopizaras": "sincopizar",
    "sincopizare": "sincopizar",
    "sincopizareis": "sincopizar",
    "sincopizaremos": "sincopizar",
    "sincopizaren": "sincopizar",
    "sincopizares": "sincopizar",
    "sincopizaron": "sincopizar",
    "sincopizará": "sincopizar",
    "sincopizarán": "sincopizar",
    "sincopizarás": "sincopizar",
    "sincopizaré": "sincopizar",
    "sincopizaréis": "sincopizar",
    "sincopizaría": "sincopizar",
    "sincopizaríais": "sincopizar",
    "sincopizaríamos": "sincopizar",
    "sincopizarían": "sincopizar",
    "sincopizarías": "sincopizar",
    "sincopizas": "sincopizar",
    "sincopizase": "sincopizar",
    "sincopizaseis": "sincopizar",
    "sincopizasen": "sincopizar",
    "sincopizases": "sincopizar",
    "sincopizaste": "sincopizar",
    "sincopizasteis": "sincopizar",
    "sincopizo": "sincopizar",
    "sincopizábamos": "sincopizar",
    "sincopizáis": "sincopizar",
    "sincopizáramos": "sincopizar",
    "sincopizáremos": "sincopizar",
    "sincopizásemos": "sincopizar",
    "sincopizó": "sincopizar",
    "sincopo": "sincopar",
    "sincopábamos": "sincopar",
    "sincopáis": "sincopar",
    "sincopáramos": "sincopar",
    "sincopáremos": "sincopar",
    "sincopásemos": "sincopar",
    "sincopé": "sincopar",
    "sincopéis": "sincopar",
    "sincopó": "sincopar",
    "sincorologías": "sincorología",
    "sincretismos": "sincretismo",
    "sincrociclotrones": "sincrociclotrón",
    "sincroflashes": "sincroflash",
    "sincrogeneradores": "sincrogenerador",
    "sincronice": "sincronizar",
    "sincronicemos": "sincronizar",
    "sincronicen": "sincronizar",
    "sincronices": "sincronizar",
    "sincronicé": "sincronizar",
    "sincronicéis": "sincronizar",
    "sincronismos": "sincronismo",
    "sincroniza": "sincronizar",
    "sincronizaba": "sincronizar",
    "sincronizabais": "sincronizar",
    "sincronizaban": "sincronizar",
    "sincronizabas": "sincronizar",
    "sincronizaciones": "sincronización",
    "sincronizad": "sincronizar",
    "sincronizada": "sincronizar",
    "sincronizadas": "sincronizar",
    "sincronizado": "sincronizar",
    "sincronizadores": "sincronizador",
    "sincronizados": "sincronizar",
    "sincronizamos": "sincronizar",
    "sincronizan": "sincronizar",
    "sincronizando": "sincronizar",
    "sincronizara": "sincronizar",
    "sincronizarais": "sincronizar",
    "sincronizaran": "sincronizar",
    "sincronizaras": "sincronizar",
    "sincronizare": "sincronizar",
    "sincronizareis": "sincronizar",
    "sincronizaremos": "sincronizar",
    "sincronizaren": "sincronizar",
    "sincronizares": "sincronizar",
    "sincronizaron": "sincronizar",
    "sincronizará": "sincronizar",
    "sincronizarán": "sincronizar",
    "sincronizarás": "sincronizar",
    "sincronizaré": "sincronizar",
    "sincronizaréis": "sincronizar",
    "sincronizaría": "sincronizar",
    "sincronizaríais": "sincronizar",
    "sincronizaríamos": "sincronizar",
    "sincronizarían": "sincronizar",
    "sincronizarías": "sincronizar",
    "sincronizas": "sincronizar",
    "sincronizase": "sincronizar",
    "sincronizaseis": "sincronizar",
    "sincronizasen": "sincronizar",
    "sincronizases": "sincronizar",
    "sincronizaste": "sincronizar",
    "sincronizasteis": "sincronizar",
    "sincronizo": "sincronizar",
    "sincronizábamos": "sincronizar",
    "sincronizáis": "sincronizar",
    "sincronizáramos": "sincronizar",
    "sincronizáremos": "sincronizar",
    "sincronizásemos": "sincronizar",
    "sincronizó": "sincronizar",
    "sincronoscopios": "sincronoscopio",
    "sincronías": "sincronía",
    "sincrotransmisores": "sincrotransmisor",
    "sincrotrones": "sincrotrón",
    "sincrética": "sincrético",
    "sincréticas": "sincrético",
    "sincréticos": "sincrético",
    "sincrónica": "sincrónico",
    "sincrónicas": "sincrónico",
    "sincrónicos": "sincrónico",
    "sincárpica": "sincárpico",
    "sincárpicas": "sincárpico",
    "sincárpicos": "sincárpico",
    "sindesmologías": "sindesmología",
    "sindesmopexias": "sindesmopexia",
    "sindesmotomías": "sindesmotomía",
    "sindhis": "sindhi",
    "sindica": "sindicar",
    "sindicaba": "sindicar",
    "sindicabais": "sindicar",
    "sindicaban": "sindicar",
    "sindicabas": "sindicar",
    "sindicables": "sindicable",
    "sindicaciones": "sindicación",
    "sindicad": "sindicar",
    "sindicada": "sindicar",
    "sindicadas": "sindicar",
    "sindicado": "sindicar",
    "sindicadora": "sindicador",
    "sindicadoras": "sindicador",
    "sindicadores": "sindicador",
    "sindicados": "sindicar",
    "sindicales": "sindical",
    "sindicalismos": "sindicalismo",
    "sindicalistas": "sindicalista",
    "sindicalizada": "sindicalizado",
    "sindicalizadas": "sindicalizado",
    "sindicalizados": "sindicalizado",
    "sindicamos": "sindicar",
    "sindican": "sindicar",
    "sindicando": "sindicar",
    "sindicara": "sindicar",
    "sindicarais": "sindicar",
    "sindicaran": "sindicar",
    "sindicaras": "sindicar",
    "sindicare": "sindicar",
    "sindicareis": "sindicar",
    "sindicaremos": "sindicar",
    "sindicaren": "sindicar",
    "sindicares": "sindicar",
    "sindicaron": "sindicar",
    "sindicará": "sindicar",
    "sindicarán": "sindicar",
    "sindicarás": "sindicar",
    "sindicaré": "sindicar",
    "sindicaréis": "sindicar",
    "sindicaría": "sindicar",
    "sindicaríais": "sindicar",
    "sindicaríamos": "sindicar",
    "sindicarían": "sindicar",
    "sindicarías": "sindicar",
    "sindicas": "sindicar",
    "sindicase": "sindicar",
    "sindicaseis": "sindicar",
    "sindicasen": "sindicar",
    "sindicases": "sindicar",
    "sindicaste": "sindicar",
    "sindicasteis": "sindicar",
    "sindicatos": "sindicato",
    "sindicaturas": "sindicatura",
    "sindico": "sindicar",
    "sindicábamos": "sindicar",
    "sindicáis": "sindicar",
    "sindicáramos": "sindicar",
    "sindicáremos": "sindicar",
    "sindicásemos": "sindicar",
    "sindicó": "sindicar",
    "sindiotáctica": "sindiotáctico",
    "sindiotácticas": "sindiotáctico",
    "sindiotácticos": "sindiotáctico",
    "sindique": "sindicar",
    "sindiquemos": "sindicar",
    "sindiquen": "sindicar",
    "sindiques": "sindicar",
    "sindiqué": "sindicar",
    "sindiquéis": "sindicar",
    "sindáctila": "sindáctilo",
    "sindáctilas": "sindáctilo",
    "sindáctilos": "sindáctilo",
    "sinecologías": "sinecología",
    "sinecuras": "sinecura",
    "sinequias": "sinequia",
    "sinergias": "sinergia",
    "sinergismos": "sinergismo",
    "sinergética": "sinergético",
    "sinergéticas": "sinergético",
    "sinergéticos": "sinergético",
    "sinestesias": "sinestesia",
    "sinfines": "sinfín",
    "sinfonismos": "sinfonismo",
    "sinfonistas": "sinfonista",
    "sinfonolas": "sinfonola",
    "sinfonías": "sinfonía",
    "sinfónica": "sinfónico",
    "sinfónicas": "sinfónico",
    "sinfónico-corales": "sinfónico-coral",
    "sinfónicos": "sinfónico",
    "singa": "singar",
    "singaba": "singar",
    "singabais": "singar",
    "singaban": "singar",
    "singabas": "singar",
    "singad": "singar",
    "singada": "singar",
    "singadas": "singar",
    "singado": "singar",
    "singados": "singar",
    "singamos": "singar",
    "singan": "singar",
    "singando": "singar",
    "singapureña": "singapureño",
    "singapureñas": "singapureño",
    "singapureños": "singapureño",
    "singara": "singar",
    "singarais": "singar",
    "singaran": "singar",
    "singaras": "singar",
    "singare": "singar",
    "singareis": "singar",
    "singaremos": "singar",
    "singaren": "singar",
    "singares": "singar",
    "singaron": "singar",
    "singará": "singar",
    "singarán": "singar",
    "singarás": "singar",
    "singaré": "singar",
    "singaréis": "singar",
    "singaría": "singar",
    "singaríais": "singar",
    "singaríamos": "singar",
    "singarían": "singar",
    "singarías": "singar",
    "singas": "singar",
    "singase": "singar",
    "singaseis": "singar",
    "singasen": "singar",
    "singases": "singar",
    "singaste": "singar",
    "singasteis": "singar",
    "singenética": "singenético",
    "singenéticas": "singenético",
    "singenéticos": "singenético",
    "singla": "singlar",
    "singlaba": "singlar",
    "singlabais": "singlar",
    "singlaban": "singlar",
    "singlabas": "singlar",
    "singlad": "singlar",
    "singlada": "singlar",
    "singladas": "singlar",
    "singlado": "singlar",
    "singlados": "singlar",
    "singladuras": "singladura",
    "singlamos": "singlar",
    "singlan": "singlar",
    "singlando": "singlar",
    "singlara": "singlar",
    "singlarais": "singlar",
    "singlaran": "singlar",
    "singlaras": "singlar",
    "singlare": "singlar",
    "singlareis": "singlar",
    "singlaremos": "singlar",
    "singlaren": "singlar",
    "singlares": "singlar",
    "singlaron": "singlar",
    "singlará": "singlar",
    "singlarán": "singlar",
    "singlarás": "singlar",
    "singlaré": "singlar",
    "singlaréis": "singlar",
    "singlaría": "singlar",
    "singlaríais": "singlar",
    "singlaríamos": "singlar",
    "singlarían": "singlar",
    "singlarías": "singlar",
    "singlas": "singlar",
    "singlase": "singlar",
    "singlaseis": "singlar",
    "singlasen": "singlar",
    "singlases": "singlar",
    "singlaste": "singlar",
    "singlasteis": "singlar",
    "single": "singlar",
    "singlemos": "singlar",
    "singlen": "singlar",
    "singles": "single",
    "singlo": "singlar",
    "singlábamos": "singlar",
    "singláis": "singlar",
    "singláramos": "singlar",
    "singláremos": "singlar",
    "singlásemos": "singlar",
    "singlé": "singlar",
    "singléis": "singlar",
    "singló": "singlar",
    "singo": "singar",
    "singue": "singar",
    "singuemos": "singar",
    "singuen": "singar",
    "singues": "singar",
    "singulares": "singular",
    "singularice": "singularizar",
    "singularicemos": "singularizar",
    "singularicen": "singularizar",
    "singularices": "singularizar",
    "singularicé": "singularizar",
    "singularicéis": "singularizar",
    "singularidades": "singularidad",
    "singulariza": "singularizar",
    "singularizaba": "singularizar",
    "singularizabais": "singularizar",
    "singularizaban": "singularizar",
    "singularizabas": "singularizar",
    "singularizad": "singularizar",
    "singularizada": "singularizar",
    "singularizadas": "singularizar",
    "singularizado": "singularizar",
    "singularizados": "singularizar",
    "singularizamos": "singularizar",
    "singularizan": "singularizar",
    "singularizando": "singularizar",
    "singularizara": "singularizar",
    "singularizarais": "singularizar",
    "singularizaran": "singularizar",
    "singularizaras": "singularizar",
    "singularizare": "singularizar",
    "singularizareis": "singularizar",
    "singularizaremos": "singularizar",
    "singularizaren": "singularizar",
    "singularizares": "singularizar",
    "singularizaron": "singularizar",
    "singularizará": "singularizar",
    "singularizarán": "singularizar",
    "singularizarás": "singularizar",
    "singularizaré": "singularizar",
    "singularizaréis": "singularizar",
    "singularizaría": "singularizar",
    "singularizaríais": "singularizar",
    "singularizaríamos": "singularizar",
    "singularizarían": "singularizar",
    "singularizarías": "singularizar",
    "singularizas": "singularizar",
    "singularizase": "singularizar",
    "singularizaseis": "singularizar",
    "singularizasen": "singularizar",
    "singularizases": "singularizar",
    "singularizaste": "singularizar",
    "singularizasteis": "singularizar",
    "singularizo": "singularizar",
    "singularizábamos": "singularizar",
    "singularizáis": "singularizar",
    "singularizáramos": "singularizar",
    "singularizáremos": "singularizar",
    "singularizásemos": "singularizar",
    "singularizó": "singularizar",
    "singué": "singar",
    "singuéis": "singar",
    "singábamos": "singar",
    "singáis": "singar",
    "singáramos": "singar",
    "singáremos": "singar",
    "singásemos": "singar",
    "singénica": "singénico",
    "singénicas": "singénico",
    "singénicos": "singénico",
    "singó": "singar",
    "sinhalitas": "sinhalita",
    "sinhuesos": "sinhueso",
    "siniestra": "siniestro",
    "siniestrada": "siniestrado",
    "siniestradas": "siniestrado",
    "siniestrados": "siniestrado",
    "siniestralidades": "siniestralidad",
    "siniestras": "siniestro",
    "siniestros": "siniestro",
    "sinistrodelta": "sinistrodelto",
    "sinistrodeltas": "sinistrodelto",
    "sinistrodeltos": "sinistrodelto",
    "sinnúmeros": "sinnúmero",
    "sinoatriales": "sinoatrial",
    "sinoauriculares": "sinoauricular",
    "sinodales": "sinodal",
    "sinoftalmías": "sinoftalmía",
    "sinologías": "sinología",
    "sinonimias": "sinonimia",
    "sinoples": "sinople",
    "sinoptóforos": "sinoptóforo",
    "sinos": "sino",
    "sinostósica": "sinostósico",
    "sinostósicas": "sinostósico",
    "sinostósicos": "sinostósico",
    "sinoviales": "sinovial",
    "sinovias": "sinovia",
    "sinoístas": "sinoísta",
    "sinquelias": "sinquelia",
    "sinrazones": "sinrazón",
    "sinsabores": "sinsabor",
    "sinsustancias": "sinsustancia",
    "sintagmas": "sintagma",
    "sintagmática": "sintagmático",
    "sintagmáticas": "sintagmático",
    "sintagmáticos": "sintagmático",
    "sintamos": "sentir",
    "sinterice": "sinterizar",
    "sintericemos": "sinterizar",
    "sintericen": "sinterizar",
    "sinterices": "sinterizar",
    "sintericé": "sinterizar",
    "sintericéis": "sinterizar",
    "sinteriza": "sinterizar",
    "sinterizaba": "sinterizar",
    "sinterizabais": "sinterizar",
    "sinterizaban": "sinterizar",
    "sinterizabas": "sinterizar",
    "sinterizaciones": "sinterización",
    "sinterizad": "sinterizar",
    "sinterizada": "sinterizar",
    "sinterizadas": "sinterizar",
    "sinterizado": "sinterizar",
    "sinterizados": "sinterizar",
    "sinterizamos": "sinterizar",
    "sinterizan": "sinterizar",
    "sinterizando": "sinterizar",
    "sinterizara": "sinterizar",
    "sinterizarais": "sinterizar",
    "sinterizaran": "sinterizar",
    "sinterizaras": "sinterizar",
    "sinterizare": "sinterizar",
    "sinterizareis": "sinterizar",
    "sinterizaremos": "sinterizar",
    "sinterizaren": "sinterizar",
    "sinterizares": "sinterizar",
    "sinterizaron": "sinterizar",
    "sinterizará": "sinterizar",
    "sinterizarán": "sinterizar",
    "sinterizarás": "sinterizar",
    "sinterizaré": "sinterizar",
    "sinterizaréis": "sinterizar",
    "sinterizaría": "sinterizar",
    "sinterizaríais": "sinterizar",
    "sinterizaríamos": "sinterizar",
    "sinterizarían": "sinterizar",
    "sinterizarías": "sinterizar",
    "sinterizas": "sinterizar",
    "sinterizase": "sinterizar",
    "sinterizaseis": "sinterizar",
    "sinterizasen": "sinterizar",
    "sinterizases": "sinterizar",
    "sinterizaste": "sinterizar",
    "sinterizasteis": "sinterizar",
    "sinterizo": "sinterizar",
    "sinterizábamos": "sinterizar",
    "sinterizáis": "sinterizar",
    "sinterizáramos": "sinterizar",
    "sinterizáremos": "sinterizar",
    "sinterizásemos": "sinterizar",
    "sinterizó": "sinterizar",
    "sintetice": "sintetizar",
    "sinteticemos": "sintetizar",
    "sinteticen": "sintetizar",
    "sintetices": "sintetizar",
    "sinteticé": "sintetizar",
    "sinteticéis": "sintetizar",
    "sintetismos": "sintetismo",
    "sintetiza": "sintetizar",
    "sintetizaba": "sintetizar",
    "sintetizabais": "sintetizar",
    "sintetizaban": "sintetizar",
    "sintetizabas": "sintetizar",
    "sintetizables": "sintetizable",
    "sintetizad": "sintetizar",
    "sintetizada": "sintetizar",
    "sintetizadas": "sintetizar",
    "sintetizado": "sintetizar",
    "sintetizadores": "sintetizador",
    "sintetizados": "sintetizar",
    "sintetizamos": "sintetizar",
    "sintetizan": "sintetizar",
    "sintetizando": "sintetizar",
    "sintetizara": "sintetizar",
    "sintetizarais": "sintetizar",
    "sintetizaran": "sintetizar",
    "sintetizaras": "sintetizar",
    "sintetizare": "sintetizar",
    "sintetizareis": "sintetizar",
    "sintetizaremos": "sintetizar",
    "sintetizaren": "sintetizar",
    "sintetizares": "sintetizar",
    "sintetizaron": "sintetizar",
    "sintetizará": "sintetizar",
    "sintetizarán": "sintetizar",
    "sintetizarás": "sintetizar",
    "sintetizaré": "sintetizar",
    "sintetizaréis": "sintetizar",
    "sintetizaría": "sintetizar",
    "sintetizaríais": "sintetizar",
    "sintetizaríamos": "sintetizar",
    "sintetizarían": "sintetizar",
    "sintetizarías": "sintetizar",
    "sintetizas": "sintetizar",
    "sintetizase": "sintetizar",
    "sintetizaseis": "sintetizar",
    "sintetizasen": "sintetizar",
    "sintetizases": "sintetizar",
    "sintetizaste": "sintetizar",
    "sintetizasteis": "sintetizar",
    "sintetizo": "sintetizar",
    "sintetizábamos": "sintetizar",
    "sintetizáis": "sintetizar",
    "sintetizáramos": "sintetizar",
    "sintetizáremos": "sintetizar",
    "sintetizásemos": "sintetizar",
    "sintetizó": "sintetizar",
    "sintiendo": "sentir",
    "sintiera": "sentir",
    "sintierais": "sentir",
    "sintieran": "sentir",
    "sintieras": "sentir",
    "sintiere": "sentir",
    "sintiereis": "sentir",
    "sintieren": "sentir",
    "sintieres": "sentir",
    "sintieron": "sentir",
    "sintiese": "sentir",
    "sintieseis": "sentir",
    "sintiesen": "sentir",
    "sintieses": "sentir",
    "sintiéramos": "sentir",
    "sintiéremos": "sentir",
    "sintiésemos": "sentir",
    "sintió": "sentir",
    "sintomatologías": "sintomatología",
    "sintomática": "sintomático",
    "sintomáticas": "sintomático",
    "sintomáticos": "sintomático",
    "sintonice": "sintonizar",
    "sintonicemos": "sintonizar",
    "sintonicen": "sintonizar",
    "sintonices": "sintonizar",
    "sintonicé": "sintonizar",
    "sintonicéis": "sintonizar",
    "sintonismos": "sintonismo",
    "sintoniza": "sintonizar",
    "sintonizaba": "sintonizar",
    "sintonizabais": "sintonizar",
    "sintonizaban": "sintonizar",
    "sintonizabas": "sintonizar",
    "sintonizabilidades": "sintonizabilidad",
    "sintonizables": "sintonizable",
    "sintonizaciones": "sintonización",
    "sintonizad": "sintonizar",
    "sintonizada": "sintonizar",
    "sintonizadas": "sintonizar",
    "sintonizado": "sintonizar",
    "sintonizadores": "sintonizador",
    "sintonizados": "sintonizar",
    "sintonizamos": "sintonizar",
    "sintonizan": "sintonizar",
    "sintonizando": "sintonizar",
    "sintonizara": "sintonizar",
    "sintonizarais": "sintonizar",
    "sintonizaran": "sintonizar",
    "sintonizaras": "sintonizar",
    "sintonizare": "sintonizar",
    "sintonizareis": "sintonizar",
    "sintonizaremos": "sintonizar",
    "sintonizaren": "sintonizar",
    "sintonizares": "sintonizar",
    "sintonizaron": "sintonizar",
    "sintonizará": "sintonizar",
    "sintonizarán": "sintonizar",
    "sintonizarás": "sintonizar",
    "sintonizaré": "sintonizar",
    "sintonizaréis": "sintonizar",
    "sintonizaría": "sintonizar",
    "sintonizaríais": "sintonizar",
    "sintonizaríamos": "sintonizar",
    "sintonizarían": "sintonizar",
    "sintonizarías": "sintonizar",
    "sintonizas": "sintonizar",
    "sintonizase": "sintonizar",
    "sintonizaseis": "sintonizar",
    "sintonizasen": "sintonizar",
    "sintonizases": "sintonizar",
    "sintonizaste": "sintonizar",
    "sintonizasteis": "sintonizar",
    "sintonizo": "sintonizar",
    "sintonizábamos": "sintonizar",
    "sintonizáis": "sintonizar",
    "sintonizáramos": "sintonizar",
    "sintonizáremos": "sintonizar",
    "sintonizásemos": "sintonizar",
    "sintonizó": "sintonizar",
    "sintonías": "sintonía",
    "sintoísmos": "sintoísmo",
    "sintoístas": "sintoísta",
    "sintáctica": "sintáctico",
    "sintácticas": "sintáctico",
    "sintácticos": "sintáctico",
    "sintáis": "sentir",
    "sintética": "sintético",
    "sintéticas": "sintético",
    "sintéticos": "sintético",
    "sintónica": "sintónico",
    "sintónicas": "sintónico",
    "sintónicos": "sintónico",
    "sinuosa": "sinuoso",
    "sinuosas": "sinuoso",
    "sinuosidades": "sinuosidad",
    "sinuosos": "sinuoso",
    "sinusectomías": "sinusectomía",
    "sinusias": "sinusia",
    "sinusoidales": "sinusoidal",
    "sinusoides": "sinusoide",
    "sinvergonzonerías": "sinvergonzonería",
    "sinvergüencerías": "sinvergüencería",
    "sinvergüenzas": "sinvergüenza",
    "sinántropos": "sinántropo",
    "sináptica": "sináptico",
    "sinápticas": "sináptico",
    "sinápticos": "sináptico",
    "sinécdoques": "sinécdoque",
    "sinécticas": "sinéctica",
    "sinérgica": "sinérgico",
    "sinérgicas": "sinérgico",
    "sinérgicos": "sinérgico",
    "sinódica": "sinódico",
    "sinódicas": "sinódico",
    "sinódicos": "sinódico",
    "sinóloga": "sinólogo",
    "sinólogas": "sinólogo",
    "sinólogos": "sinólogo",
    "sinónima": "sinónimo",
    "sinónimas": "sinónimo",
    "sinónimos": "sinónimo",
    "sinóptica": "sinóptico",
    "sinópticas": "sinóptico",
    "sinópticos": "sinóptico",
    "sionismos": "sionismo",
    "sionistas": "sionista",
    "siquiatras": "siquiatra",
    "siquiatrías": "siquiatría",
    "siquier": "siquiera",
    "siquiátrica": "siquiátrico",
    "siquiátricas": "siquiátrico",
    "siquiátricos": "siquiátrico",
    "sira": "siro",
    "siracusana": "siracusano",
    "siracusanas": "siracusano",
    "siracusanos": "siracusano",
    "siras": "siro",
    "sirenas": "sirena",
    "sirenios": "sirenio",
    "sires": "sir",
    "sirga": "sirgar",
    "sirgaba": "sirgar",
    "sirgabais": "sirgar",
    "sirgaban": "sirgar",
    "sirgabas": "sirgar",
    "sirgad": "sirgar",
    "sirgada": "sirgar",
    "sirgadas": "sirgar",
    "sirgado": "sirgar",
    "sirgados": "sirgar",
    "sirgamos": "sirgar",
    "sirgan": "sirgar",
    "sirgando": "sirgar",
    "sirgara": "sirgar",
    "sirgarais": "sirgar",
    "sirgaran": "sirgar",
    "sirgaras": "sirgar",
    "sirgare": "sirgar",
    "sirgareis": "sirgar",
    "sirgaremos": "sirgar",
    "sirgaren": "sirgar",
    "sirgares": "sirgar",
    "sirgaron": "sirgar",
    "sirgará": "sirgar",
    "sirgarán": "sirgar",
    "sirgarás": "sirgar",
    "sirgaré": "sirgar",
    "sirgaréis": "sirgar",
    "sirgaría": "sirgar",
    "sirgaríais": "sirgar",
    "sirgaríamos": "sirgar",
    "sirgarían": "sirgar",
    "sirgarías": "sirgar",
    "sirgas": "sirgar",
    "sirgase": "sirgar",
    "sirgaseis": "sirgar",
    "sirgasen": "sirgar",
    "sirgases": "sirgar",
    "sirgaste": "sirgar",
    "sirgasteis": "sirgar",
    "sirgo": "sirgar",
    "sirgos": "sirgo",
    "sirgue": "sirgar",
    "sirguemos": "sirgar",
    "sirguen": "sirgar",
    "sirgues": "sirgar",
    "sirgué": "sirgar",
    "sirguéis": "sirgar",
    "sirgábamos": "sirgar",
    "sirgáis": "sirgar",
    "sirgáramos": "sirgar",
    "sirgáremos": "sirgar",
    "sirgásemos": "sirgar",
    "sirgó": "sirgar",
    "siria": "sirio",
    "siriaca": "siríaco",
    "siriacas": "siríaco",
    "siriacos": "siríaco",
    "sirias": "sirio",
    "siringas": "siringa",
    "siringes": "siringe",
    "siringomielias": "siringomielia",
    "sirios": "sirio",
    "sirles": "sirle",
    "sirocos": "siroco",
    "siropes": "sirope",
    "siros": "siro",
    "sirosingopinas": "sirosingopina",
    "sirrias": "sirria",
    "sirtes": "sirte",
    "sirva": "servir",
    "sirvamos": "servir",
    "sirvan": "servir",
    "sirvas": "servir",
    "sirve": "servir",
    "sirven": "servir",
    "sirventeses": "sirventés",
    "sirves": "servir",
    "sirviendo": "servir",
    "sirvienta": "sirviente",
    "sirvientas": "sirviente",
    "sirvientes": "sirviente",
    "sirviera": "servir",
    "sirvierais": "servir",
    "sirvieran": "servir",
    "sirvieras": "servir",
    "sirviere": "servir",
    "sirviereis": "servir",
    "sirvieren": "servir",
    "sirvieres": "servir",
    "sirvieron": "servir",
    "sirviese": "servir",
    "sirvieseis": "servir",
    "sirviesen": "servir",
    "sirvieses": "servir",
    "sirviéramos": "servir",
    "sirviéremos": "servir",
    "sirviésemos": "servir",
    "sirvió": "servir",
    "sirvo": "servir",
    "sirváis": "servir",
    "siríaca": "siríaco",
    "siríacas": "siríaco",
    "siríacos": "siríaco",
    "sis": "si",
    "sisa": "sisar",
    "sisaba": "sisar",
    "sisabais": "sisar",
    "sisaban": "sisar",
    "sisabas": "sisar",
    "sisad": "sisar",
    "sisada": "sisar",
    "sisadas": "sisar",
    "sisado": "sisar",
    "sisadora": "sisador",
    "sisadoras": "sisador",
    "sisadores": "sisador",
    "sisados": "sisar",
    "sisales": "sisal",
    "sisamos": "sisar",
    "sisan": "sisar",
    "sisando": "sisar",
    "sisara": "sisar",
    "sisarais": "sisar",
    "sisaran": "sisar",
    "sisaras": "sisar",
    "sisare": "sisar",
    "sisareis": "sisar",
    "sisaremos": "sisar",
    "sisaren": "sisar",
    "sisares": "sisar",
    "sisaron": "sisar",
    "sisará": "sisar",
    "sisarán": "sisar",
    "sisarás": "sisar",
    "sisaré": "sisar",
    "sisaréis": "sisar",
    "sisaría": "sisar",
    "sisaríais": "sisar",
    "sisaríamos": "sisar",
    "sisarían": "sisar",
    "sisarías": "sisar",
    "sisas": "sisar",
    "sisase": "sisar",
    "sisaseis": "sisar",
    "sisasen": "sisar",
    "sisases": "sisar",
    "sisaste": "sisar",
    "sisasteis": "sisar",
    "sise": "sisar",
    "sisea": "sisear",
    "siseaba": "sisear",
    "siseabais": "sisear",
    "siseaban": "sisear",
    "siseabas": "sisear",
    "sisead": "sisear",
    "siseada": "sisear",
    "siseadas": "sisear",
    "siseado": "sisear",
    "siseados": "sisear",
    "siseamos": "sisear",
    "sisean": "sisear",
    "siseando": "sisear",
    "siseara": "sisear",
    "sisearais": "sisear",
    "sisearan": "sisear",
    "sisearas": "sisear",
    "siseare": "sisear",
    "siseareis": "sisear",
    "sisearemos": "sisear",
    "sisearen": "sisear",
    "siseares": "sisear",
    "sisearon": "sisear",
    "siseará": "sisear",
    "sisearán": "sisear",
    "sisearás": "sisear",
    "sisearé": "sisear",
    "sisearéis": "sisear",
    "sisearía": "sisear",
    "sisearíais": "sisear",
    "sisearíamos": "sisear",
    "sisearían": "sisear",
    "sisearías": "sisear",
    "siseas": "sisear",
    "sisease": "sisear",
    "siseaseis": "sisear",
    "siseasen": "sisear",
    "siseases": "sisear",
    "siseaste": "sisear",
    "siseasteis": "sisear",
    "sisee": "sisear",
    "siseemos": "sisear",
    "siseen": "sisear",
    "sisees": "sisear",
    "sisemos": "sisar",
    "sisen": "sisar",
    "siseo": "sisear",
    "siseos": "siseo",
    "sises": "sisar",
    "siseábamos": "sisear",
    "siseáis": "sisear",
    "siseáramos": "sisear",
    "siseáremos": "sisear",
    "siseásemos": "sisear",
    "siseé": "sisear",
    "siseéis": "sisear",
    "siseó": "sisear",
    "sismicidades": "sismicidad",
    "sismografías": "sismografía",
    "sismogramas": "sismograma",
    "sismográfica": "sismográfico",
    "sismográficas": "sismográfico",
    "sismográficos": "sismográfico",
    "sismologías": "sismología",
    "sismológica": "sismológico",
    "sismológicas": "sismológico",
    "sismológicos": "sismológico",
    "sismos": "sismo",
    "sismoscopios": "sismoscopio",
    "sismógrafos": "sismógrafo",
    "sismóloga": "sismólogo",
    "sismólogas": "sismólogo",
    "sismólogos": "sismólogo",
    "sismómetros": "sismómetro",
    "siso": "sisar",
    "sisona": "sisón",
    "sisonas": "sisón",
    "sisones": "sisón",
    "sistemas": "sistema",
    "sistematice": "sistematizar",
    "sistematicemos": "sistematizar",
    "sistematicen": "sistematizar",
    "sistematices": "sistematizar",
    "sistematicé": "sistematizar",
    "sistematicéis": "sistematizar",
    "sistematiza": "sistematizar",
    "sistematizaba": "sistematizar",
    "sistematizabais": "sistematizar",
    "sistematizaban": "sistematizar",
    "sistematizabas": "sistematizar",
    "sistematizaciones": "sistematización",
    "sistematizad": "sistematizar",
    "sistematizada": "sistematizar",
    "sistematizadas": "sistematizar",
    "sistematizado": "sistematizar",
    "sistematizados": "sistematizar",
    "sistematizamos": "sistematizar",
    "sistematizan": "sistematizar",
    "sistematizando": "sistematizar",
    "sistematizara": "sistematizar",
    "sistematizarais": "sistematizar",
    "sistematizaran": "sistematizar",
    "sistematizaras": "sistematizar",
    "sistematizare": "sistematizar",
    "sistematizareis": "sistematizar",
    "sistematizaremos": "sistematizar",
    "sistematizaren": "sistematizar",
    "sistematizares": "sistematizar",
    "sistematizaron": "sistematizar",
    "sistematizará": "sistematizar",
    "sistematizarán": "sistematizar",
    "sistematizarás": "sistematizar",
    "sistematizaré": "sistematizar",
    "sistematizaréis": "sistematizar",
    "sistematizaría": "sistematizar",
    "sistematizaríais": "sistematizar",
    "sistematizaríamos": "sistematizar",
    "sistematizarían": "sistematizar",
    "sistematizarías": "sistematizar",
    "sistematizas": "sistematizar",
    "sistematizase": "sistematizar",
    "sistematizaseis": "sistematizar",
    "sistematizasen": "sistematizar",
    "sistematizases": "sistematizar",
    "sistematizaste": "sistematizar",
    "sistematizasteis": "sistematizar",
    "sistematizo": "sistematizar",
    "sistematizábamos": "sistematizar",
    "sistematizáis": "sistematizar",
    "sistematizáramos": "sistematizar",
    "sistematizáremos": "sistematizar",
    "sistematizásemos": "sistematizar",
    "sistematizó": "sistematizar",
    "sistemática": "sistemático",
    "sistemáticas": "sistemático",
    "sistemáticos": "sistemático",
    "sistémica": "sistémico",
    "sistémicas": "sistémico",
    "sistémicos": "sistémico",
    "sistólica": "sistólico",
    "sistólicas": "sistólico",
    "sistólicos": "sistólico",
    "sisábamos": "sisar",
    "sisáis": "sisar",
    "sisáramos": "sisar",
    "sisáremos": "sisar",
    "sisásemos": "sisar",
    "sisé": "sisar",
    "siséis": "sisar",
    "sisó": "sisar",
    "sita": "sito",
    "sitares": "sitar",
    "sitas": "sito",
    "sitia": "sitiar",
    "sitiaba": "sitiar",
    "sitiabais": "sitiar",
    "sitiaban": "sitiar",
    "sitiabas": "sitiar",
    "sitiad": "sitiar",
    "sitiada": "sitiar",
    "sitiadas": "sitiar",
    "sitiado": "sitiar",
    "sitiadora": "sitiador",
    "sitiadoras": "sitiador",
    "sitiadores": "sitiador",
    "sitiados": "sitiar",
    "sitiales": "sitial",
    "sitiamos": "sitiar",
    "sitian": "sitiar",
    "sitiando": "sitiar",
    "sitiara": "sitiar",
    "sitiarais": "sitiar",
    "sitiaran": "sitiar",
    "sitiaras": "sitiar",
    "sitiare": "sitiar",
    "sitiareis": "sitiar",
    "sitiaremos": "sitiar",
    "sitiaren": "sitiar",
    "sitiares": "sitiar",
    "sitiaron": "sitiar",
    "sitiará": "sitiar",
    "sitiarán": "sitiar",
    "sitiarás": "sitiar",
    "sitiaré": "sitiar",
    "sitiaréis": "sitiar",
    "sitiaría": "sitiar",
    "sitiaríais": "sitiar",
    "sitiaríamos": "sitiar",
    "sitiarían": "sitiar",
    "sitiarías": "sitiar",
    "sitias": "sitiar",
    "sitiase": "sitiar",
    "sitiaseis": "sitiar",
    "sitiasen": "sitiar",
    "sitiases": "sitiar",
    "sitiaste": "sitiar",
    "sitiasteis": "sitiar",
    "sitie": "sitiar",
    "sitiemos": "sitiar",
    "sitien": "sitiar",
    "sities": "sitiar",
    "sitio": "sitiar",
    "sitios": "sitio",
    "sitiábamos": "sitiar",
    "sitiáis": "sitiar",
    "sitiáramos": "sitiar",
    "sitiáremos": "sitiar",
    "sitiásemos": "sitiar",
    "sitié": "sitiar",
    "sitiéis": "sitiar",
    "sitió": "sitiar",
    "sitomanías": "sitomanía",
    "sitos": "sito",
    "sitosteroles": "sitosterol",
    "situaba": "situar",
    "situabais": "situar",
    "situaban": "situar",
    "situabas": "situar",
    "situables": "situable",
    "situacionales": "situacional",
    "situaciones": "situación",
    "situad": "situar",
    "situada": "situar",
    "situadas": "situar",
    "situado": "situar",
    "situados": "situar",
    "situamos": "situar",
    "situando": "situar",
    "situara": "situar",
    "situarais": "situar",
    "situaran": "situar",
    "situaras": "situar",
    "situare": "situar",
    "situareis": "situar",
    "situaremos": "situar",
    "situaren": "situar",
    "situares": "situar",
    "situaron": "situar",
    "situará": "situar",
    "situarán": "situar",
    "situarás": "situar",
    "situaré": "situar",
    "situaréis": "situar",
    "situaría": "situar",
    "situaríais": "situar",
    "situaríamos": "situar",
    "situarían": "situar",
    "situarías": "situar",
    "situase": "situar",
    "situaseis": "situar",
    "situasen": "situar",
    "situases": "situar",
    "situaste": "situar",
    "situasteis": "situar",
    "situemos": "situar",
    "situábamos": "situar",
    "situáis": "situar",
    "situáramos": "situar",
    "situáremos": "situar",
    "situásemos": "situar",
    "situé": "situar",
    "situéis": "situar",
    "situó": "situar",
    "sitúa": "situar",
    "sitúan": "situar",
    "sitúas": "situar",
    "sitúe": "situar",
    "sitúen": "situar",
    "sitúes": "situar",
    "sitúo": "situar",
    "sizigias": "sizigia",
    "siálica": "siálico",
    "siálicas": "siálico",
    "siálicos": "siálico",
    "skas": "ska",
    "skateboards": "skateboard",
    "sketches": "sketch",
    "ski": "esquí",
    "ski-biscuits": "ski-biscuit",
    "skiffistas": "skiffista",
    "skiffs": "skiff",
    "skins": "skin",
    "skips": "skip",
    "skis": "esquí",
    "skuteruditas": "skuterudita",
    "skíes": "esquí",
    "slalom": "eslálom",
    "slaloms": "eslálom",
    "slangs": "slang",
    "slap-shots": "slap-shot",
    "slips": "slip",
    "slogan": "eslogan",
    "slogans": "eslogan",
    "sloops": "sloop",
    "slálom": "eslálom",
    "sláloms": "eslálom",
    "smithsoniana": "smithsoniano",
    "smithsonianas": "smithsoniano",
    "smithsonianos": "smithsoniano",
    "smithsonitas": "smithsonita",
    "smokings": "smoking",
    "snackbars": "snackbar",
    "snacks": "snack",
    "snobismos": "snobismo",
    "snobs": "snob",
    "snowboards": "snowboard",
    "soasa": "soasar",
    "soasaba": "soasar",
    "soasabais": "soasar",
    "soasaban": "soasar",
    "soasabas": "soasar",
    "soasad": "soasar",
    "soasada": "soasar",
    "soasadas": "soasar",
    "soasado": "soasar",
    "soasados": "soasar",
    "soasamos": "soasar",
    "soasan": "soasar",
    "soasando": "soasar",
    "soasara": "soasar",
    "soasarais": "soasar",
    "soasaran": "soasar",
    "soasaras": "soasar",
    "soasare": "soasar",
    "soasareis": "soasar",
    "soasaremos": "soasar",
    "soasaren": "soasar",
    "soasares": "soasar",
    "soasaron": "soasar",
    "soasará": "soasar",
    "soasarán": "soasar",
    "soasarás": "soasar",
    "soasaré": "soasar",
    "soasaréis": "soasar",
    "soasaría": "soasar",
    "soasaríais": "soasar",
    "soasaríamos": "soasar",
    "soasarían": "soasar",
    "soasarías": "soasar",
    "soasas": "soasar",
    "soasase": "soasar",
    "soasaseis": "soasar",
    "soasasen": "soasar",
    "soasases": "soasar",
    "soasaste": "soasar",
    "soasasteis": "soasar",
    "soase": "soasar",
    "soasemos": "soasar",
    "soasen": "soasar",
    "soases": "soasar",
    "soaso": "soasar",
    "soasábamos": "soasar",
    "soasáis": "soasar",
    "soasáramos": "soasar",
    "soasáremos": "soasar",
    "soasásemos": "soasar",
    "soasé": "soasar",
    "soaséis": "soasar",
    "soasó": "soasar",
    "soba": "sobar",
    "sobaba": "sobar",
    "sobabais": "sobar",
    "sobaban": "sobar",
    "sobabas": "sobar",
    "sobacos": "sobaco",
    "sobacuna": "sobacuno",
    "sobacunas": "sobacuno",
    "sobacunos": "sobacuno",
    "sobad": "sobar",
    "sobada": "sobar",
    "sobadas": "sobar",
    "sobadera": "sobadero",
    "sobaderas": "sobadero",
    "sobaderos": "sobadero",
    "sobado": "sobar",
    "sobados": "sobar",
    "sobaduras": "sobadura",
    "sobaja": "sobajar",
    "sobajaba": "sobajar",
    "sobajabais": "sobajar",
    "sobajaban": "sobajar",
    "sobajabas": "sobajar",
    "sobajad": "sobajar",
    "sobajada": "sobajar",
    "sobajadas": "sobajar",
    "sobajado": "sobajar",
    "sobajados": "sobajar",
    "sobajamos": "sobajar",
    "sobajan": "sobajar",
    "sobajando": "sobajar",
    "sobajara": "sobajar",
    "sobajarais": "sobajar",
    "sobajaran": "sobajar",
    "sobajaras": "sobajar",
    "sobajare": "sobajar",
    "sobajareis": "sobajar",
    "sobajaremos": "sobajar",
    "sobajaren": "sobajar",
    "sobajares": "sobajar",
    "sobajaron": "sobajar",
    "sobajará": "sobajar",
    "sobajarán": "sobajar",
    "sobajarás": "sobajar",
    "sobajaré": "sobajar",
    "sobajaréis": "sobajar",
    "sobajaría": "sobajar",
    "sobajaríais": "sobajar",
    "sobajaríamos": "sobajar",
    "sobajarían": "sobajar",
    "sobajarías": "sobajar",
    "sobajas": "sobajar",
    "sobajase": "sobajar",
    "sobajaseis": "sobajar",
    "sobajasen": "sobajar",
    "sobajases": "sobajar",
    "sobajaste": "sobajar",
    "sobajasteis": "sobajar",
    "sobaje": "sobajar",
    "sobajea": "sobajear",
    "sobajeaba": "sobajear",
    "sobajeabais": "sobajear",
    "sobajeaban": "sobajear",
    "sobajeabas": "sobajear",
    "sobajead": "sobajear",
    "sobajeada": "sobajear",
    "sobajeadas": "sobajear",
    "sobajeado": "sobajear",
    "sobajeados": "sobajear",
    "sobajeamos": "sobajear",
    "sobajean": "sobajear",
    "sobajeando": "sobajear",
    "sobajeara": "sobajear",
    "sobajearais": "sobajear",
    "sobajearan": "sobajear",
    "sobajearas": "sobajear",
    "sobajeare": "sobajear",
    "sobajeareis": "sobajear",
    "sobajearemos": "sobajear",
    "sobajearen": "sobajear",
    "sobajeares": "sobajear",
    "sobajearon": "sobajear",
    "sobajeará": "sobajear",
    "sobajearán": "sobajear",
    "sobajearás": "sobajear",
    "sobajearé": "sobajear",
    "sobajearéis": "sobajear",
    "sobajearía": "sobajear",
    "sobajearíais": "sobajear",
    "sobajearíamos": "sobajear",
    "sobajearían": "sobajear",
    "sobajearías": "sobajear",
    "sobajeas": "sobajear",
    "sobajease": "sobajear",
    "sobajeaseis": "sobajear",
    "sobajeasen": "sobajear",
    "sobajeases": "sobajear",
    "sobajeaste": "sobajear",
    "sobajeasteis": "sobajear",
    "sobajee": "sobajear",
    "sobajeemos": "sobajear",
    "sobajeen": "sobajear",
    "sobajees": "sobajear",
    "sobajemos": "sobajar",
    "sobajen": "sobajar",
    "sobajeo": "sobajear",
    "sobajes": "sobajar",
    "sobajeábamos": "sobajear",
    "sobajeáis": "sobajear",
    "sobajeáramos": "sobajear",
    "sobajeáremos": "sobajear",
    "sobajeásemos": "sobajear",
    "sobajeé": "sobajear",
    "sobajeéis": "sobajear",
    "sobajeó": "sobajear",
    "sobajo": "sobajar",
    "sobajábamos": "sobajar",
    "sobajáis": "sobajar",
    "sobajáramos": "sobajar",
    "sobajáremos": "sobajar",
    "sobajásemos": "sobajar",
    "sobajé": "sobajar",
    "sobajéis": "sobajar",
    "sobajó": "sobajar",
    "sobamos": "sobar",
    "soban": "sobar",
    "sobandas": "sobanda",
    "sobando": "sobar",
    "sobaqueras": "sobaquera",
    "sobaquinas": "sobaquina",
    "sobara": "sobar",
    "sobarais": "sobar",
    "sobaran": "sobar",
    "sobaras": "sobar",
    "sobarbadas": "sobarbada",
    "sobarbas": "sobarba",
    "sobarbos": "sobarbo",
    "sobare": "sobar",
    "sobareis": "sobar",
    "sobaremos": "sobar",
    "sobaren": "sobar",
    "sobares": "sobar",
    "sobaron": "sobar",
    "sobará": "sobar",
    "sobarán": "sobar",
    "sobarás": "sobar",
    "sobaré": "sobar",
    "sobaréis": "sobar",
    "sobaría": "sobar",
    "sobaríais": "sobar",
    "sobaríamos": "sobar",
    "sobarían": "sobar",
    "sobarías": "sobar",
    "sobas": "sobar",
    "sobase": "sobar",
    "sobaseis": "sobar",
    "sobasen": "sobar",
    "sobases": "sobar",
    "sobaste": "sobar",
    "sobasteis": "sobar",
    "sobe": "sobar",
    "sobemos": "sobar",
    "soben": "sobar",
    "sobeos": "sobeo",
    "soberana": "soberano",
    "soberanas": "soberano",
    "soberanea": "soberanear",
    "soberaneaba": "soberanear",
    "soberaneabais": "soberanear",
    "soberaneaban": "soberanear",
    "soberaneabas": "soberanear",
    "soberanead": "soberanear",
    "soberaneada": "soberanear",
    "soberaneadas": "soberanear",
    "soberaneado": "soberanear",
    "soberaneados": "soberanear",
    "soberaneamos": "soberanear",
    "soberanean": "soberanear",
    "soberaneando": "soberanear",
    "soberaneara": "soberanear",
    "soberanearais": "soberanear",
    "soberanearan": "soberanear",
    "soberanearas": "soberanear",
    "soberaneare": "soberanear",
    "soberaneareis": "soberanear",
    "soberanearemos": "soberanear",
    "soberanearen": "soberanear",
    "soberaneares": "soberanear",
    "soberanearon": "soberanear",
    "soberaneará": "soberanear",
    "soberanearán": "soberanear",
    "soberanearás": "soberanear",
    "soberanearé": "soberanear",
    "soberanearéis": "soberanear",
    "soberanearía": "soberanear",
    "soberanearíais": "soberanear",
    "soberanearíamos": "soberanear",
    "soberanearían": "soberanear",
    "soberanearías": "soberanear",
    "soberaneas": "soberanear",
    "soberanease": "soberanear",
    "soberaneaseis": "soberanear",
    "soberaneasen": "soberanear",
    "soberaneases": "soberanear",
    "soberaneaste": "soberanear",
    "soberaneasteis": "soberanear",
    "soberanee": "soberanear",
    "soberaneemos": "soberanear",
    "soberaneen": "soberanear",
    "soberanees": "soberanear",
    "soberaneo": "soberanear",
    "soberaneábamos": "soberanear",
    "soberaneáis": "soberanear",
    "soberaneáramos": "soberanear",
    "soberaneáremos": "soberanear",
    "soberaneásemos": "soberanear",
    "soberaneé": "soberanear",
    "soberaneéis": "soberanear",
    "soberaneó": "soberanear",
    "soberanos": "soberano",
    "soberanías": "soberanía",
    "soberbia": "soberbio",
    "soberbias": "soberbio",
    "soberbios": "soberbio",
    "sobermeja": "sobermejo",
    "sobermejas": "sobermejo",
    "sobermejos": "sobermejo",
    "sobes": "sobar",
    "sobo": "sobar",
    "sobona": "sobón",
    "sobonas": "sobón",
    "sobones": "sobón",
    "sobordos": "sobordo",
    "soborna": "sobornar",
    "sobornaba": "sobornar",
    "sobornabais": "sobornar",
    "sobornaban": "sobornar",
    "sobornabas": "sobornar",
    "sobornables": "sobornable",
    "sobornad": "sobornar",
    "sobornada": "sobornar",
    "sobornadas": "sobornar",
    "sobornado": "sobornar",
    "sobornadora": "sobornador",
    "sobornadoras": "sobornador",
    "sobornadores": "sobornador",
    "sobornados": "sobornar",
    "sobornales": "sobornal",
    "sobornamos": "sobornar",
    "sobornan": "sobornar",
    "sobornando": "sobornar",
    "sobornara": "sobornar",
    "sobornarais": "sobornar",
    "sobornaran": "sobornar",
    "sobornaras": "sobornar",
    "sobornare": "sobornar",
    "sobornareis": "sobornar",
    "sobornaremos": "sobornar",
    "sobornaren": "sobornar",
    "sobornares": "sobornar",
    "sobornaron": "sobornar",
    "sobornará": "sobornar",
    "sobornarán": "sobornar",
    "sobornarás": "sobornar",
    "sobornaré": "sobornar",
    "sobornaréis": "sobornar",
    "sobornaría": "sobornar",
    "sobornaríais": "sobornar",
    "sobornaríamos": "sobornar",
    "sobornarían": "sobornar",
    "sobornarías": "sobornar",
    "sobornas": "sobornar",
    "sobornase": "sobornar",
    "sobornaseis": "sobornar",
    "sobornasen": "sobornar",
    "sobornases": "sobornar",
    "sobornaste": "sobornar",
    "sobornasteis": "sobornar",
    "soborne": "sobornar",
    "sobornemos": "sobornar",
    "sobornen": "sobornar",
    "sobornes": "sobornar",
    "soborno": "sobornar",
    "sobornos": "soborno",
    "sobornábamos": "sobornar",
    "sobornáis": "sobornar",
    "sobornáramos": "sobornar",
    "sobornáremos": "sobornar",
    "sobornásemos": "sobornar",
    "soborné": "sobornar",
    "sobornéis": "sobornar",
    "sobornó": "sobornar",
    "sobra": "sobrar",
    "sobraba": "sobrar",
    "sobrabais": "sobrar",
    "sobraban": "sobrar",
    "sobrabas": "sobrar",
    "sobrad": "sobrar",
    "sobrada": "sobrar",
    "sobradas": "sobrar",
    "sobradillos": "sobradillo",
    "sobrado": "sobrar",
    "sobrados": "sobrar",
    "sobramos": "sobrar",
    "sobran": "sobrar",
    "sobrando": "sobrar",
    "sobrantes": "sobrante",
    "sobrara": "sobrar",
    "sobrarais": "sobrar",
    "sobraran": "sobrar",
    "sobraras": "sobrar",
    "sobrare": "sobrar",
    "sobrareis": "sobrar",
    "sobraremos": "sobrar",
    "sobraren": "sobrar",
    "sobrares": "sobrar",
    "sobraron": "sobrar",
    "sobrará": "sobrar",
    "sobrarán": "sobrar",
    "sobrarás": "sobrar",
    "sobraré": "sobrar",
    "sobraréis": "sobrar",
    "sobraría": "sobrar",
    "sobraríais": "sobrar",
    "sobraríamos": "sobrar",
    "sobrarían": "sobrar",
    "sobrarías": "sobrar",
    "sobras": "sobrar",
    "sobrasa": "sobrasar",
    "sobrasaba": "sobrasar",
    "sobrasabais": "sobrasar",
    "sobrasaban": "sobrasar",
    "sobrasabas": "sobrasar",
    "sobrasad": "sobrasar",
    "sobrasada": "sobrasar",
    "sobrasadas": "sobrasar",
    "sobrasado": "sobrasar",
    "sobrasados": "sobrasar",
    "sobrasamos": "sobrasar",
    "sobrasan": "sobrasar",
    "sobrasando": "sobrasar",
    "sobrasara": "sobrasar",
    "sobrasarais": "sobrasar",
    "sobrasaran": "sobrasar",
    "sobrasaras": "sobrasar",
    "sobrasare": "sobrasar",
    "sobrasareis": "sobrasar",
    "sobrasaremos": "sobrasar",
    "sobrasaren": "sobrasar",
    "sobrasares": "sobrasar",
    "sobrasaron": "sobrasar",
    "sobrasará": "sobrasar",
    "sobrasarán": "sobrasar",
    "sobrasarás": "sobrasar",
    "sobrasaré": "sobrasar",
    "sobrasaréis": "sobrasar",
    "sobrasaría": "sobrasar",
    "sobrasaríais": "sobrasar",
    "sobrasaríamos": "sobrasar",
    "sobrasarían": "sobrasar",
    "sobrasarías": "sobrasar",
    "sobrasas": "sobrasar",
    "sobrasase": "sobrasar",
    "sobrasaseis": "sobrasar",
    "sobrasasen": "sobrasar",
    "sobrasases": "sobrasar",
    "sobrasaste": "sobrasar",
    "sobrasasteis": "sobrasar",
    "sobrase": "sobrasar",
    "sobraseis": "sobrar",
    "sobrasemos": "sobrasar",
    "sobrasen": "sobrasar",
    "sobrases": "sobrasar",
    "sobraso": "sobrasar",
    "sobraste": "sobrar",
    "sobrasteis": "sobrar",
    "sobrasábamos": "sobrasar",
    "sobrasáis": "sobrasar",
    "sobrasáramos": "sobrasar",
    "sobrasáremos": "sobrasar",
    "sobrasásemos": "sobrasar",
    "sobrasé": "sobrasar",
    "sobraséis": "sobrasar",
    "sobrasó": "sobrasar",
    "sobre": "sobrar",
    "sobreabunda": "sobreabundar",
    "sobreabundaba": "sobreabundar",
    "sobreabundabais": "sobreabundar",
    "sobreabundaban": "sobreabundar",
    "sobreabundabas": "sobreabundar",
    "sobreabundad": "sobreabundar",
    "sobreabundada": "sobreabundar",
    "sobreabundadas": "sobreabundar",
    "sobreabundado": "sobreabundar",
    "sobreabundados": "sobreabundar",
    "sobreabundamos": "sobreabundar",
    "sobreabundan": "sobreabundar",
    "sobreabundancias": "sobreabundancia",
    "sobreabundando": "sobreabundar",
    "sobreabundara": "sobreabundar",
    "sobreabundarais": "sobreabundar",
    "sobreabundaran": "sobreabundar",
    "sobreabundaras": "sobreabundar",
    "sobreabundare": "sobreabundar",
    "sobreabundareis": "sobreabundar",
    "sobreabundaremos": "sobreabundar",
    "sobreabundaren": "sobreabundar",
    "sobreabundares": "sobreabundar",
    "sobreabundaron": "sobreabundar",
    "sobreabundará": "sobreabundar",
    "sobreabundarán": "sobreabundar",
    "sobreabundarás": "sobreabundar",
    "sobreabundaré": "sobreabundar",
    "sobreabundaréis": "sobreabundar",
    "sobreabundaría": "sobreabundar",
    "sobreabundaríais": "sobreabundar",
    "sobreabundaríamos": "sobreabundar",
    "sobreabundarían": "sobreabundar",
    "sobreabundarías": "sobreabundar",
    "sobreabundas": "sobreabundar",
    "sobreabundase": "sobreabundar",
    "sobreabundaseis": "sobreabundar",
    "sobreabundasen": "sobreabundar",
    "sobreabundases": "sobreabundar",
    "sobreabundaste": "sobreabundar",
    "sobreabundasteis": "sobreabundar",
    "sobreabunde": "sobreabundar",
    "sobreabundemos": "sobreabundar",
    "sobreabunden": "sobreabundar",
    "sobreabundes": "sobreabundar",
    "sobreabundo": "sobreabundar",
    "sobreabundábamos": "sobreabundar",
    "sobreabundáis": "sobreabundar",
    "sobreabundáramos": "sobreabundar",
    "sobreabundáremos": "sobreabundar",
    "sobreabundásemos": "sobreabundar",
    "sobreabundé": "sobreabundar",
    "sobreabundéis": "sobreabundar",
    "sobreabundó": "sobreabundar",
    "sobreaguda": "sobreagudo",
    "sobreagudas": "sobreagudo",
    "sobreagudos": "sobreagudo",
    "sobrealce": "sobrealzar",
    "sobrealcemos": "sobrealzar",
    "sobrealcen": "sobrealzar",
    "sobrealces": "sobrealzar",
    "sobrealcé": "sobrealzar",
    "sobrealcéis": "sobrealzar",
    "sobrealientos": "sobrealiento",
    "sobrealimenta": "sobrealimentar",
    "sobrealimentaba": "sobrealimentar",
    "sobrealimentabais": "sobrealimentar",
    "sobrealimentaban": "sobrealimentar",
    "sobrealimentabas": "sobrealimentar",
    "sobrealimentaciones": "sobrealimentación",
    "sobrealimentad": "sobrealimentar",
    "sobrealimentada": "sobrealimentar",
    "sobrealimentadas": "sobrealimentar",
    "sobrealimentado": "sobrealimentar",
    "sobrealimentadores": "sobrealimentador",
    "sobrealimentados": "sobrealimentar",
    "sobrealimentamos": "sobrealimentar",
    "sobrealimentan": "sobrealimentar",
    "sobrealimentando": "sobrealimentar",
    "sobrealimentara": "sobrealimentar",
    "sobrealimentarais": "sobrealimentar",
    "sobrealimentaran": "sobrealimentar",
    "sobrealimentaras": "sobrealimentar",
    "sobrealimentare": "sobrealimentar",
    "sobrealimentareis": "sobrealimentar",
    "sobrealimentaremos": "sobrealimentar",
    "sobrealimentaren": "sobrealimentar",
    "sobrealimentares": "sobrealimentar",
    "sobrealimentaron": "sobrealimentar",
    "sobrealimentará": "sobrealimentar",
    "sobrealimentarán": "sobrealimentar",
    "sobrealimentarás": "sobrealimentar",
    "sobrealimentaré": "sobrealimentar",
    "sobrealimentaréis": "sobrealimentar",
    "sobrealimentaría": "sobrealimentar",
    "sobrealimentaríais": "sobrealimentar",
    "sobrealimentaríamos": "sobrealimentar",
    "sobrealimentarían": "sobrealimentar",
    "sobrealimentarías": "sobrealimentar",
    "sobrealimentas": "sobrealimentar",
    "sobrealimentase": "sobrealimentar",
    "sobrealimentaseis": "sobrealimentar",
    "sobrealimentasen": "sobrealimentar",
    "sobrealimentases": "sobrealimentar",
    "sobrealimentaste": "sobrealimentar",
    "sobrealimentasteis": "sobrealimentar",
    "sobrealimente": "sobrealimentar",
    "sobrealimentemos": "sobrealimentar",
    "sobrealimenten": "sobrealimentar",
    "sobrealimentes": "sobrealimentar",
    "sobrealimento": "sobrealimentar",
    "sobrealimentábamos": "sobrealimentar",
    "sobrealimentáis": "sobrealimentar",
    "sobrealimentáramos": "sobrealimentar",
    "sobrealimentáremos": "sobrealimentar",
    "sobrealimentásemos": "sobrealimentar",
    "sobrealimenté": "sobrealimentar",
    "sobrealimentéis": "sobrealimentar",
    "sobrealimentó": "sobrealimentar",
    "sobrealmacenamientos": "sobrealmacenamiento",
    "sobrealza": "sobrealzar",
    "sobrealzaba": "sobrealzar",
    "sobrealzabais": "sobrealzar",
    "sobrealzaban": "sobrealzar",
    "sobrealzabas": "sobrealzar",
    "sobrealzad": "sobrealzar",
    "sobrealzada": "sobrealzar",
    "sobrealzadas": "sobrealzar",
    "sobrealzado": "sobrealzar",
    "sobrealzados": "sobrealzar",
    "sobrealzamos": "sobrealzar",
    "sobrealzan": "sobrealzar",
    "sobrealzando": "sobrealzar",
    "sobrealzara": "sobrealzar",
    "sobrealzarais": "sobrealzar",
    "sobrealzaran": "sobrealzar",
    "sobrealzaras": "sobrealzar",
    "sobrealzare": "sobrealzar",
    "sobrealzareis": "sobrealzar",
    "sobrealzaremos": "sobrealzar",
    "sobrealzaren": "sobrealzar",
    "sobrealzares": "sobrealzar",
    "sobrealzaron": "sobrealzar",
    "sobrealzará": "sobrealzar",
    "sobrealzarán": "sobrealzar",
    "sobrealzarás": "sobrealzar",
    "sobrealzaré": "sobrealzar",
    "sobrealzaréis": "sobrealzar",
    "sobrealzaría": "sobrealzar",
    "sobrealzaríais": "sobrealzar",
    "sobrealzaríamos": "sobrealzar",
    "sobrealzarían": "sobrealzar",
    "sobrealzarías": "sobrealzar",
    "sobrealzas": "sobrealzar",
    "sobrealzase": "sobrealzar",
    "sobrealzaseis": "sobrealzar",
    "sobrealzasen": "sobrealzar",
    "sobrealzases": "sobrealzar",
    "sobrealzaste": "sobrealzar",
    "sobrealzasteis": "sobrealzar",
    "sobrealzo": "sobrealzar",
    "sobrealzábamos": "sobrealzar",
    "sobrealzáis": "sobrealzar",
    "sobrealzáramos": "sobrealzar",
    "sobrealzáremos": "sobrealzar",
    "sobrealzásemos": "sobrealzar",
    "sobrealzó": "sobrealzar",
    "sobreamortiguada": "sobreamortiguado",
    "sobreamortiguadas": "sobreamortiguado",
    "sobreamortiguados": "sobreamortiguado",
    "sobreanchos": "sobreancho",
    "sobreaprendizajes": "sobreaprendizaje",
    "sobreara": "sobrearar",
    "sobrearaba": "sobrearar",
    "sobrearabais": "sobrearar",
    "sobrearaban": "sobrearar",
    "sobrearabas": "sobrearar",
    "sobrearad": "sobrearar",
    "sobrearada": "sobrearar",
    "sobrearadas": "sobrearar",
    "sobrearado": "sobrearar",
    "sobrearados": "sobrearar",
    "sobrearamos": "sobrearar",
    "sobrearan": "sobrearar",
    "sobrearando": "sobrearar",
    "sobrearara": "sobrearar",
    "sobreararais": "sobrearar",
    "sobreararan": "sobrearar",
    "sobreararas": "sobrearar",
    "sobrearare": "sobrearar",
    "sobrearareis": "sobrearar",
    "sobreararemos": "sobrearar",
    "sobreararen": "sobrearar",
    "sobrearares": "sobrearar",
    "sobreararon": "sobrearar",
    "sobrearará": "sobrearar",
    "sobreararán": "sobrearar",
    "sobreararás": "sobrearar",
    "sobreararé": "sobrearar",
    "sobreararéis": "sobrearar",
    "sobreararía": "sobrearar",
    "sobreararíais": "sobrearar",
    "sobreararíamos": "sobrearar",
    "sobreararían": "sobrearar",
    "sobreararías": "sobrearar",
    "sobrearas": "sobrearar",
    "sobrearase": "sobrearar",
    "sobrearaseis": "sobrearar",
    "sobrearasen": "sobrearar",
    "sobrearases": "sobrearar",
    "sobrearaste": "sobrearar",
    "sobrearasteis": "sobrearar",
    "sobrearcos": "sobrearco",
    "sobreare": "sobrearar",
    "sobrearemos": "sobrearar",
    "sobrearen": "sobrearar",
    "sobreares": "sobrearar",
    "sobrearo": "sobrearar",
    "sobrearábamos": "sobrearar",
    "sobrearáis": "sobrearar",
    "sobrearáramos": "sobrearar",
    "sobrearáremos": "sobrearar",
    "sobrearásemos": "sobrearar",
    "sobrearé": "sobrearar",
    "sobrearéis": "sobrearar",
    "sobrearó": "sobrearar",
    "sobreasadas": "sobreasada",
    "sobreañada": "sobreañadir",
    "sobreañadamos": "sobreañadir",
    "sobreañadan": "sobreañadir",
    "sobreañadas": "sobreañadir",
    "sobreañade": "sobreañadir",
    "sobreañaden": "sobreañadir",
    "sobreañades": "sobreañadir",
    "sobreañadid": "sobreañadir",
    "sobreañadida": "sobreañadir",
    "sobreañadidas": "sobreañadir",
    "sobreañadido": "sobreañadir",
    "sobreañadidos": "sobreañadir",
    "sobreañadiendo": "sobreañadir",
    "sobreañadiera": "sobreañadir",
    "sobreañadierais": "sobreañadir",
    "sobreañadieran": "sobreañadir",
    "sobreañadieras": "sobreañadir",
    "sobreañadiere": "sobreañadir",
    "sobreañadiereis": "sobreañadir",
    "sobreañadieren": "sobreañadir",
    "sobreañadieres": "sobreañadir",
    "sobreañadieron": "sobreañadir",
    "sobreañadiese": "sobreañadir",
    "sobreañadieseis": "sobreañadir",
    "sobreañadiesen": "sobreañadir",
    "sobreañadieses": "sobreañadir",
    "sobreañadimos": "sobreañadir",
    "sobreañadiremos": "sobreañadir",
    "sobreañadirá": "sobreañadir",
    "sobreañadirán": "sobreañadir",
    "sobreañadirás": "sobreañadir",
    "sobreañadiré": "sobreañadir",
    "sobreañadiréis": "sobreañadir",
    "sobreañadiría": "sobreañadir",
    "sobreañadiríais": "sobreañadir",
    "sobreañadiríamos": "sobreañadir",
    "sobreañadirían": "sobreañadir",
    "sobreañadirías": "sobreañadir",
    "sobreañadiste": "sobreañadir",
    "sobreañadisteis": "sobreañadir",
    "sobreañadiéramos": "sobreañadir",
    "sobreañadiéremos": "sobreañadir",
    "sobreañadiésemos": "sobreañadir",
    "sobreañadió": "sobreañadir",
    "sobreañado": "sobreañadir",
    "sobreañadáis": "sobreañadir",
    "sobreañadí": "sobreañadir",
    "sobreañadía": "sobreañadir",
    "sobreañadíais": "sobreañadir",
    "sobreañadíamos": "sobreañadir",
    "sobreañadían": "sobreañadir",
    "sobreañadías": "sobreañadir",
    "sobreañadís": "sobreañadir",
    "sobreañales": "sobreañal",
    "sobrebarra": "sobrebarrer",
    "sobrebarramos": "sobrebarrer",
    "sobrebarran": "sobrebarrer",
    "sobrebarras": "sobrebarrer",
    "sobrebarre": "sobrebarrer",
    "sobrebarred": "sobrebarrer",
    "sobrebarremos": "sobrebarrer",
    "sobrebarren": "sobrebarrer",
    "sobrebarreremos": "sobrebarrer",
    "sobrebarrerá": "sobrebarrer",
    "sobrebarrerán": "sobrebarrer",
    "sobrebarrerás": "sobrebarrer",
    "sobrebarreré": "sobrebarrer",
    "sobrebarreréis": "sobrebarrer",
    "sobrebarrería": "sobrebarrer",
    "sobrebarreríais": "sobrebarrer",
    "sobrebarreríamos": "sobrebarrer",
    "sobrebarrerían": "sobrebarrer",
    "sobrebarrerías": "sobrebarrer",
    "sobrebarres": "sobrebarrer",
    "sobrebarrida": "sobrebarrer",
    "sobrebarridas": "sobrebarrer",
    "sobrebarrido": "sobrebarrer",
    "sobrebarridos": "sobrebarrer",
    "sobrebarriendo": "sobrebarrer",
    "sobrebarriera": "sobrebarrer",
    "sobrebarrierais": "sobrebarrer",
    "sobrebarrieran": "sobrebarrer",
    "sobrebarrieras": "sobrebarrer",
    "sobrebarriere": "sobrebarrer",
    "sobrebarriereis": "sobrebarrer",
    "sobrebarrieren": "sobrebarrer",
    "sobrebarrieres": "sobrebarrer",
    "sobrebarrieron": "sobrebarrer",
    "sobrebarriese": "sobrebarrer",
    "sobrebarrieseis": "sobrebarrer",
    "sobrebarriesen": "sobrebarrer",
    "sobrebarrieses": "sobrebarrer",
    "sobrebarrimos": "sobrebarrer",
    "sobrebarriste": "sobrebarrer",
    "sobrebarristeis": "sobrebarrer",
    "sobrebarriéramos": "sobrebarrer",
    "sobrebarriéremos": "sobrebarrer",
    "sobrebarriésemos": "sobrebarrer",
    "sobrebarrió": "sobrebarrer",
    "sobrebarro": "sobrebarrer",
    "sobrebarráis": "sobrebarrer",
    "sobrebarréis": "sobrebarrer",
    "sobrebarrí": "sobrebarrer",
    "sobrebarría": "sobrebarrer",
    "sobrebarríais": "sobrebarrer",
    "sobrebarríamos": "sobrebarrer",
    "sobrebarrían": "sobrebarrer",
    "sobrebarrías": "sobrebarrer",
    "sobrebeba": "sobrebeber",
    "sobrebebamos": "sobrebeber",
    "sobrebeban": "sobrebeber",
    "sobrebebas": "sobrebeber",
    "sobrebebe": "sobrebeber",
    "sobrebebed": "sobrebeber",
    "sobrebebemos": "sobrebeber",
    "sobrebeben": "sobrebeber",
    "sobrebeberemos": "sobrebeber",
    "sobrebeberá": "sobrebeber",
    "sobrebeberán": "sobrebeber",
    "sobrebeberás": "sobrebeber",
    "sobrebeberé": "sobrebeber",
    "sobrebeberéis": "sobrebeber",
    "sobrebebería": "sobrebeber",
    "sobrebeberíais": "sobrebeber",
    "sobrebeberíamos": "sobrebeber",
    "sobrebeberían": "sobrebeber",
    "sobrebeberías": "sobrebeber",
    "sobrebebes": "sobrebeber",
    "sobrebebida": "sobrebeber",
    "sobrebebidas": "sobrebeber",
    "sobrebebido": "sobrebeber",
    "sobrebebidos": "sobrebeber",
    "sobrebebiendo": "sobrebeber",
    "sobrebebiera": "sobrebeber",
    "sobrebebierais": "sobrebeber",
    "sobrebebieran": "sobrebeber",
    "sobrebebieras": "sobrebeber",
    "sobrebebiere": "sobrebeber",
    "sobrebebiereis": "sobrebeber",
    "sobrebebieren": "sobrebeber",
    "sobrebebieres": "sobrebeber",
    "sobrebebieron": "sobrebeber",
    "sobrebebiese": "sobrebeber",
    "sobrebebieseis": "sobrebeber",
    "sobrebebiesen": "sobrebeber",
    "sobrebebieses": "sobrebeber",
    "sobrebebimos": "sobrebeber",
    "sobrebebiste": "sobrebeber",
    "sobrebebisteis": "sobrebeber",
    "sobrebebiéramos": "sobrebeber",
    "sobrebebiéremos": "sobrebeber",
    "sobrebebiésemos": "sobrebeber",
    "sobrebebió": "sobrebeber",
    "sobrebebo": "sobrebeber",
    "sobrebebáis": "sobrebeber",
    "sobrebebéis": "sobrebeber",
    "sobrebebí": "sobrebeber",
    "sobrebebía": "sobrebeber",
    "sobrebebíais": "sobrebeber",
    "sobrebebíamos": "sobrebeber",
    "sobrebebían": "sobrebeber",
    "sobrebebías": "sobrebeber",
    "sobrecalentaba": "sobrecalentar",
    "sobrecalentabais": "sobrecalentar",
    "sobrecalentaban": "sobrecalentar",
    "sobrecalentabas": "sobrecalentar",
    "sobrecalentad": "sobrecalentar",
    "sobrecalentada": "sobrecalentar",
    "sobrecalentadas": "sobrecalentar",
    "sobrecalentado": "sobrecalentar",
    "sobrecalentadores": "sobrecalentador",
    "sobrecalentados": "sobrecalentar",
    "sobrecalentamientos": "sobrecalentamiento",
    "sobrecalentamos": "sobrecalentar",
    "sobrecalentando": "sobrecalentar",
    "sobrecalentara": "sobrecalentar",
    "sobrecalentarais": "sobrecalentar",
    "sobrecalentaran": "sobrecalentar",
    "sobrecalentaras": "sobrecalentar",
    "sobrecalentare": "sobrecalentar",
    "sobrecalentareis": "sobrecalentar",
    "sobrecalentaremos": "sobrecalentar",
    "sobrecalentaren": "sobrecalentar",
    "sobrecalentares": "sobrecalentar",
    "sobrecalentaron": "sobrecalentar",
    "sobrecalentará": "sobrecalentar",
    "sobrecalentarán": "sobrecalentar",
    "sobrecalentarás": "sobrecalentar",
    "sobrecalentaré": "sobrecalentar",
    "sobrecalentaréis": "sobrecalentar",
    "sobrecalentaría": "sobrecalentar",
    "sobrecalentaríais": "sobrecalentar",
    "sobrecalentaríamos": "sobrecalentar",
    "sobrecalentarían": "sobrecalentar",
    "sobrecalentarías": "sobrecalentar",
    "sobrecalentase": "sobrecalentar",
    "sobrecalentaseis": "sobrecalentar",
    "sobrecalentasen": "sobrecalentar",
    "sobrecalentases": "sobrecalentar",
    "sobrecalentaste": "sobrecalentar",
    "sobrecalentasteis": "sobrecalentar",
    "sobrecalentemos": "sobrecalentar",
    "sobrecalentábamos": "sobrecalentar",
    "sobrecalentáis": "sobrecalentar",
    "sobrecalentáramos": "sobrecalentar",
    "sobrecalentáremos": "sobrecalentar",
    "sobrecalentásemos": "sobrecalentar",
    "sobrecalenté": "sobrecalentar",
    "sobrecalentéis": "sobrecalentar",
    "sobrecalentó": "sobrecalentar",
    "sobrecalienta": "sobrecalentar",
    "sobrecalientan": "sobrecalentar",
    "sobrecalientas": "sobrecalentar",
    "sobrecaliente": "sobrecalentar",
    "sobrecalienten": "sobrecalentar",
    "sobrecalientes": "sobrecalentar",
    "sobrecaliento": "sobrecalentar",
    "sobrecalzas": "sobrecalza",
    "sobrecamas": "sobrecama",
    "sobrecamisas": "sobrecamisa",
    "sobrecapitalizaciones": "sobrecapitalización",
    "sobrecarga": "sobrecargar",
    "sobrecargaba": "sobrecargar",
    "sobrecargabais": "sobrecargar",
    "sobrecargaban": "sobrecargar",
    "sobrecargabas": "sobrecargar",
    "sobrecargad": "sobrecargar",
    "sobrecargada": "sobrecargar",
    "sobrecargadas": "sobrecargar",
    "sobrecargado": "sobrecargar",
    "sobrecargados": "sobrecargar",
    "sobrecargamos": "sobrecargar",
    "sobrecargan": "sobrecargar",
    "sobrecargando": "sobrecargar",
    "sobrecargara": "sobrecargar",
    "sobrecargarais": "sobrecargar",
    "sobrecargaran": "sobrecargar",
    "sobrecargaras": "sobrecargar",
    "sobrecargare": "sobrecargar",
    "sobrecargareis": "sobrecargar",
    "sobrecargaremos": "sobrecargar",
    "sobrecargaren": "sobrecargar",
    "sobrecargares": "sobrecargar",
    "sobrecargaron": "sobrecargar",
    "sobrecargará": "sobrecargar",
    "sobrecargarán": "sobrecargar",
    "sobrecargarás": "sobrecargar",
    "sobrecargaré": "sobrecargar",
    "sobrecargaréis": "sobrecargar",
    "sobrecargaría": "sobrecargar",
    "sobrecargaríais": "sobrecargar",
    "sobrecargaríamos": "sobrecargar",
    "sobrecargarían": "sobrecargar",
    "sobrecargarías": "sobrecargar",
    "sobrecargas": "sobrecargar",
    "sobrecargase": "sobrecargar",
    "sobrecargaseis": "sobrecargar",
    "sobrecargasen": "sobrecargar",
    "sobrecargases": "sobrecargar",
    "sobrecargaste": "sobrecargar",
    "sobrecargasteis": "sobrecargar",
    "sobrecargo": "sobrecargar",
    "sobrecargos": "sobrecargo",
    "sobrecargue": "sobrecargar",
    "sobrecarguemos": "sobrecargar",
    "sobrecarguen": "sobrecargar",
    "sobrecargues": "sobrecargar",
    "sobrecargué": "sobrecargar",
    "sobrecarguéis": "sobrecargar",
    "sobrecargábamos": "sobrecargar",
    "sobrecargáis": "sobrecargar",
    "sobrecargáramos": "sobrecargar",
    "sobrecargáremos": "sobrecargar",
    "sobrecargásemos": "sobrecargar",
    "sobrecargó": "sobrecargar",
    "sobrecarta": "sobrecartar",
    "sobrecartaba": "sobrecartar",
    "sobrecartabais": "sobrecartar",
    "sobrecartaban": "sobrecartar",
    "sobrecartabas": "sobrecartar",
    "sobrecartad": "sobrecartar",
    "sobrecartada": "sobrecartar",
    "sobrecartadas": "sobrecartar",
    "sobrecartado": "sobrecartar",
    "sobrecartados": "sobrecartar",
    "sobrecartamos": "sobrecartar",
    "sobrecartan": "sobrecartar",
    "sobrecartando": "sobrecartar",
    "sobrecartara": "sobrecartar",
    "sobrecartarais": "sobrecartar",
    "sobrecartaran": "sobrecartar",
    "sobrecartaras": "sobrecartar",
    "sobrecartare": "sobrecartar",
    "sobrecartareis": "sobrecartar",
    "sobrecartaremos": "sobrecartar",
    "sobrecartaren": "sobrecartar",
    "sobrecartares": "sobrecartar",
    "sobrecartaron": "sobrecartar",
    "sobrecartará": "sobrecartar",
    "sobrecartarán": "sobrecartar",
    "sobrecartarás": "sobrecartar",
    "sobrecartaré": "sobrecartar",
    "sobrecartaréis": "sobrecartar",
    "sobrecartaría": "sobrecartar",
    "sobrecartaríais": "sobrecartar",
    "sobrecartaríamos": "sobrecartar",
    "sobrecartarían": "sobrecartar",
    "sobrecartarías": "sobrecartar",
    "sobrecartas": "sobrecartar",
    "sobrecartase": "sobrecartar",
    "sobrecartaseis": "sobrecartar",
    "sobrecartasen": "sobrecartar",
    "sobrecartases": "sobrecartar",
    "sobrecartaste": "sobrecartar",
    "sobrecartasteis": "sobrecartar",
    "sobrecarte": "sobrecartar",
    "sobrecartemos": "sobrecartar",
    "sobrecarten": "sobrecartar",
    "sobrecartes": "sobrecartar",
    "sobrecarto": "sobrecartar",
    "sobrecartábamos": "sobrecartar",
    "sobrecartáis": "sobrecartar",
    "sobrecartáramos": "sobrecartar",
    "sobrecartáremos": "sobrecartar",
    "sobrecartásemos": "sobrecartar",
    "sobrecarté": "sobrecartar",
    "sobrecartéis": "sobrecartar",
    "sobrecartó": "sobrecartar",
    "sobrecejas": "sobreceja",
    "sobrecejos": "sobrecejo",
    "sobrecena": "sobrecenar",
    "sobrecenaba": "sobrecenar",
    "sobrecenabais": "sobrecenar",
    "sobrecenaban": "sobrecenar",
    "sobrecenabas": "sobrecenar",
    "sobrecenad": "sobrecenar",
    "sobrecenada": "sobrecenar",
    "sobrecenadas": "sobrecenar",
    "sobrecenado": "sobrecenar",
    "sobrecenados": "sobrecenar",
    "sobrecenamos": "sobrecenar",
    "sobrecenan": "sobrecenar",
    "sobrecenando": "sobrecenar",
    "sobrecenara": "sobrecenar",
    "sobrecenarais": "sobrecenar",
    "sobrecenaran": "sobrecenar",
    "sobrecenaras": "sobrecenar",
    "sobrecenare": "sobrecenar",
    "sobrecenareis": "sobrecenar",
    "sobrecenaremos": "sobrecenar",
    "sobrecenaren": "sobrecenar",
    "sobrecenares": "sobrecenar",
    "sobrecenaron": "sobrecenar",
    "sobrecenará": "sobrecenar",
    "sobrecenarán": "sobrecenar",
    "sobrecenarás": "sobrecenar",
    "sobrecenaré": "sobrecenar",
    "sobrecenaréis": "sobrecenar",
    "sobrecenaría": "sobrecenar",
    "sobrecenaríais": "sobrecenar",
    "sobrecenaríamos": "sobrecenar",
    "sobrecenarían": "sobrecenar",
    "sobrecenarías": "sobrecenar",
    "sobrecenas": "sobrecenar",
    "sobrecenase": "sobrecenar",
    "sobrecenaseis": "sobrecenar",
    "sobrecenasen": "sobrecenar",
    "sobrecenases": "sobrecenar",
    "sobrecenaste": "sobrecenar",
    "sobrecenasteis": "sobrecenar",
    "sobrecene": "sobrecenar",
    "sobrecenemos": "sobrecenar",
    "sobrecenen": "sobrecenar",
    "sobrecenes": "sobrecenar",
    "sobreceno": "sobrecenar",
    "sobrecenábamos": "sobrecenar",
    "sobrecenáis": "sobrecenar",
    "sobrecenáramos": "sobrecenar",
    "sobrecenáremos": "sobrecenar",
    "sobrecenásemos": "sobrecenar",
    "sobrecené": "sobrecenar",
    "sobrecenéis": "sobrecenar",
    "sobrecenó": "sobrecenar",
    "sobrecercos": "sobrecerco",
    "sobreceños": "sobreceño",
    "sobrecielos": "sobrecielo",
    "sobrecinchas": "sobrecincha",
    "sobrecinchos": "sobrecincho",
    "sobrecoge": "sobrecoger",
    "sobrecoged": "sobrecoger",
    "sobrecogedora": "sobrecogedor",
    "sobrecogedoras": "sobrecogedor",
    "sobrecogedores": "sobrecogedor",
    "sobrecogemos": "sobrecoger",
    "sobrecogen": "sobrecoger",
    "sobrecogeremos": "sobrecoger",
    "sobrecogerá": "sobrecoger",
    "sobrecogerán": "sobrecoger",
    "sobrecogerás": "sobrecoger",
    "sobrecogeré": "sobrecoger",
    "sobrecogeréis": "sobrecoger",
    "sobrecogería": "sobrecoger",
    "sobrecogeríais": "sobrecoger",
    "sobrecogeríamos": "sobrecoger",
    "sobrecogerían": "sobrecoger",
    "sobrecogerías": "sobrecoger",
    "sobrecoges": "sobrecoger",
    "sobrecogida": "sobrecoger",
    "sobrecogidas": "sobrecoger",
    "sobrecogido": "sobrecoger",
    "sobrecogidos": "sobrecoger",
    "sobrecogiendo": "sobrecoger",
    "sobrecogiera": "sobrecoger",
    "sobrecogierais": "sobrecoger",
    "sobrecogieran": "sobrecoger",
    "sobrecogieras": "sobrecoger",
    "sobrecogiere": "sobrecoger",
    "sobrecogiereis": "sobrecoger",
    "sobrecogieren": "sobrecoger",
    "sobrecogieres": "sobrecoger",
    "sobrecogieron": "sobrecoger",
    "sobrecogiese": "sobrecoger",
    "sobrecogieseis": "sobrecoger",
    "sobrecogiesen": "sobrecoger",
    "sobrecogieses": "sobrecoger",
    "sobrecogimientos": "sobrecogimiento",
    "sobrecogimos": "sobrecoger",
    "sobrecogiste": "sobrecoger",
    "sobrecogisteis": "sobrecoger",
    "sobrecogiéramos": "sobrecoger",
    "sobrecogiéremos": "sobrecoger",
    "sobrecogiésemos": "sobrecoger",
    "sobrecogió": "sobrecoger",
    "sobrecogéis": "sobrecoger",
    "sobrecogí": "sobrecoger",
    "sobrecogía": "sobrecoger",
    "sobrecogíais": "sobrecoger",
    "sobrecogíamos": "sobrecoger",
    "sobrecogían": "sobrecoger",
    "sobrecogías": "sobrecoger",
    "sobrecoja": "sobrecoger",
    "sobrecojamos": "sobrecoger",
    "sobrecojan": "sobrecoger",
    "sobrecojas": "sobrecoger",
    "sobrecojo": "sobrecoger",
    "sobrecojáis": "sobrecoger",
    "sobrecomercios": "sobrecomercio",
    "sobrecomidas": "sobrecomida",
    "sobrecomisiones": "sobrecomisión",
    "sobrecompresiones": "sobrecompresión",
    "sobrecontrataciones": "sobrecontratación",
    "sobrecorrientes": "sobrecorriente",
    "sobrecrece": "sobrecrecer",
    "sobrecreced": "sobrecrecer",
    "sobrecrecemos": "sobrecrecer",
    "sobrecrecen": "sobrecrecer",
    "sobrecreceremos": "sobrecrecer",
    "sobrecrecerá": "sobrecrecer",
    "sobrecrecerán": "sobrecrecer",
    "sobrecrecerás": "sobrecrecer",
    "sobrecreceré": "sobrecrecer",
    "sobrecreceréis": "sobrecrecer",
    "sobrecrecería": "sobrecrecer",
    "sobrecreceríais": "sobrecrecer",
    "sobrecreceríamos": "sobrecrecer",
    "sobrecrecerían": "sobrecrecer",
    "sobrecrecerías": "sobrecrecer",
    "sobrecreces": "sobrecrecer",
    "sobrecrecida": "sobrecrecer",
    "sobrecrecidas": "sobrecrecer",
    "sobrecrecido": "sobrecrecer",
    "sobrecrecidos": "sobrecrecer",
    "sobrecreciendo": "sobrecrecer",
    "sobrecreciera": "sobrecrecer",
    "sobrecrecierais": "sobrecrecer",
    "sobrecrecieran": "sobrecrecer",
    "sobrecrecieras": "sobrecrecer",
    "sobrecreciere": "sobrecrecer",
    "sobrecreciereis": "sobrecrecer",
    "sobrecrecieren": "sobrecrecer",
    "sobrecrecieres": "sobrecrecer",
    "sobrecrecieron": "sobrecrecer",
    "sobrecreciese": "sobrecrecer",
    "sobrecrecieseis": "sobrecrecer",
    "sobrecreciesen": "sobrecrecer",
    "sobrecrecieses": "sobrecrecer",
    "sobrecrecimos": "sobrecrecer",
    "sobrecreciste": "sobrecrecer",
    "sobrecrecisteis": "sobrecrecer",
    "sobrecreciéramos": "sobrecrecer",
    "sobrecreciéremos": "sobrecrecer",
    "sobrecreciésemos": "sobrecrecer",
    "sobrecreció": "sobrecrecer",
    "sobrecrecéis": "sobrecrecer",
    "sobrecrecí": "sobrecrecer",
    "sobrecrecía": "sobrecrecer",
    "sobrecrecíais": "sobrecrecer",
    "sobrecrecíamos": "sobrecrecer",
    "sobrecrecían": "sobrecrecer",
    "sobrecrecías": "sobrecrecer",
    "sobrecrezca": "sobrecrecer",
    "sobrecrezcamos": "sobrecrecer",
    "sobrecrezcan": "sobrecrecer",
    "sobrecrezcas": "sobrecrecer",
    "sobrecrezco": "sobrecrecer",
    "sobrecrezcáis": "sobrecrecer",
    "sobrecubiertas": "sobrecubierta",
    "sobrecuellos": "sobrecuello",
    "sobrecurados": "sobrecurado",
    "sobredepreciaciones": "sobredepreciación",
    "sobredeterminaciones": "sobredeterminación",
    "sobredicha": "sobredicho",
    "sobredichas": "sobredicho",
    "sobredichos": "sobredicho",
    "sobredientes": "sobrediente",
    "sobredimensiona": "sobredimensionar",
    "sobredimensionaba": "sobredimensionar",
    "sobredimensionabais": "sobredimensionar",
    "sobredimensionaban": "sobredimensionar",
    "sobredimensionabas": "sobredimensionar",
    "sobredimensionad": "sobredimensionar",
    "sobredimensionada": "sobredimensionar",
    "sobredimensionadas": "sobredimensionar",
    "sobredimensionado": "sobredimensionar",
    "sobredimensionados": "sobredimensionar",
    "sobredimensionamientos": "sobredimensionamiento",
    "sobredimensionamos": "sobredimensionar",
    "sobredimensionan": "sobredimensionar",
    "sobredimensionando": "sobredimensionar",
    "sobredimensionara": "sobredimensionar",
    "sobredimensionarais": "sobredimensionar",
    "sobredimensionaran": "sobredimensionar",
    "sobredimensionaras": "sobredimensionar",
    "sobredimensionare": "sobredimensionar",
    "sobredimensionareis": "sobredimensionar",
    "sobredimensionaremos": "sobredimensionar",
    "sobredimensionaren": "sobredimensionar",
    "sobredimensionares": "sobredimensionar",
    "sobredimensionaron": "sobredimensionar",
    "sobredimensionará": "sobredimensionar",
    "sobredimensionarán": "sobredimensionar",
    "sobredimensionarás": "sobredimensionar",
    "sobredimensionaré": "sobredimensionar",
    "sobredimensionaréis": "sobredimensionar",
    "sobredimensionaría": "sobredimensionar",
    "sobredimensionaríais": "sobredimensionar",
    "sobredimensionaríamos": "sobredimensionar",
    "sobredimensionarían": "sobredimensionar",
    "sobredimensionarías": "sobredimensionar",
    "sobredimensionas": "sobredimensionar",
    "sobredimensionase": "sobredimensionar",
    "sobredimensionaseis": "sobredimensionar",
    "sobredimensionasen": "sobredimensionar",
    "sobredimensionases": "sobredimensionar",
    "sobredimensionaste": "sobredimensionar",
    "sobredimensionasteis": "sobredimensionar",
    "sobredimensione": "sobredimensionar",
    "sobredimensionemos": "sobredimensionar",
    "sobredimensionen": "sobredimensionar",
    "sobredimensiones": "sobredimensionar",
    "sobredimensiono": "sobredimensionar",
    "sobredimensionábamos": "sobredimensionar",
    "sobredimensionáis": "sobredimensionar",
    "sobredimensionáramos": "sobredimensionar",
    "sobredimensionáremos": "sobredimensionar",
    "sobredimensionásemos": "sobredimensionar",
    "sobredimensioné": "sobredimensionar",
    "sobredimensionéis": "sobredimensionar",
    "sobredimensionó": "sobredimensionar",
    "sobredora": "sobredorar",
    "sobredoraba": "sobredorar",
    "sobredorabais": "sobredorar",
    "sobredoraban": "sobredorar",
    "sobredorabas": "sobredorar",
    "sobredorad": "sobredorar",
    "sobredorada": "sobredorar",
    "sobredoradas": "sobredorar",
    "sobredorado": "sobredorar",
    "sobredorados": "sobredorar",
    "sobredoramos": "sobredorar",
    "sobredoran": "sobredorar",
    "sobredorando": "sobredorar",
    "sobredorara": "sobredorar",
    "sobredorarais": "sobredorar",
    "sobredoraran": "sobredorar",
    "sobredoraras": "sobredorar",
    "sobredorare": "sobredorar",
    "sobredorareis": "sobredorar",
    "sobredoraremos": "sobredorar",
    "sobredoraren": "sobredorar",
    "sobredorares": "sobredorar",
    "sobredoraron": "sobredorar",
    "sobredorará": "sobredorar",
    "sobredorarán": "sobredorar",
    "sobredorarás": "sobredorar",
    "sobredoraré": "sobredorar",
    "sobredoraréis": "sobredorar",
    "sobredoraría": "sobredorar",
    "sobredoraríais": "sobredorar",
    "sobredoraríamos": "sobredorar",
    "sobredorarían": "sobredorar",
    "sobredorarías": "sobredorar",
    "sobredoras": "sobredorar",
    "sobredorase": "sobredorar",
    "sobredoraseis": "sobredorar",
    "sobredorasen": "sobredorar",
    "sobredorases": "sobredorar",
    "sobredoraste": "sobredorar",
    "sobredorasteis": "sobredorar",
    "sobredore": "sobredorar",
    "sobredoremos": "sobredorar",
    "sobredoren": "sobredorar",
    "sobredores": "sobredorar",
    "sobredoro": "sobredorar",
    "sobredorábamos": "sobredorar",
    "sobredoráis": "sobredorar",
    "sobredoráramos": "sobredorar",
    "sobredoráremos": "sobredorar",
    "sobredorásemos": "sobredorar",
    "sobredoré": "sobredorar",
    "sobredoréis": "sobredorar",
    "sobredoró": "sobredorar",
    "sobreebulliciones": "sobreebullición",
    "sobreedifica": "sobreedificar",
    "sobreedificaba": "sobreedificar",
    "sobreedificabais": "sobreedificar",
    "sobreedificaban": "sobreedificar",
    "sobreedificabas": "sobreedificar",
    "sobreedificad": "sobreedificar",
    "sobreedificada": "sobreedificar",
    "sobreedificadas": "sobreedificar",
    "sobreedificado": "sobreedificar",
    "sobreedificados": "sobreedificar",
    "sobreedificamos": "sobreedificar",
    "sobreedifican": "sobreedificar",
    "sobreedificando": "sobreedificar",
    "sobreedificara": "sobreedificar",
    "sobreedificarais": "sobreedificar",
    "sobreedificaran": "sobreedificar",
    "sobreedificaras": "sobreedificar",
    "sobreedificare": "sobreedificar",
    "sobreedificareis": "sobreedificar",
    "sobreedificaremos": "sobreedificar",
    "sobreedificaren": "sobreedificar",
    "sobreedificares": "sobreedificar",
    "sobreedificaron": "sobreedificar",
    "sobreedificará": "sobreedificar",
    "sobreedificarán": "sobreedificar",
    "sobreedificarás": "sobreedificar",
    "sobreedificaré": "sobreedificar",
    "sobreedificaréis": "sobreedificar",
    "sobreedificaría": "sobreedificar",
    "sobreedificaríais": "sobreedificar",
    "sobreedificaríamos": "sobreedificar",
    "sobreedificarían": "sobreedificar",
    "sobreedificarías": "sobreedificar",
    "sobreedificas": "sobreedificar",
    "sobreedificase": "sobreedificar",
    "sobreedificaseis": "sobreedificar",
    "sobreedificasen": "sobreedificar",
    "sobreedificases": "sobreedificar",
    "sobreedificaste": "sobreedificar",
    "sobreedificasteis": "sobreedificar",
    "sobreedifico": "sobreedificar",
    "sobreedificábamos": "sobreedificar",
    "sobreedificáis": "sobreedificar",
    "sobreedificáramos": "sobreedificar",
    "sobreedificáremos": "sobreedificar",
    "sobreedificásemos": "sobreedificar",
    "sobreedificó": "sobreedificar",
    "sobreedifique": "sobreedificar",
    "sobreedifiquemos": "sobreedificar",
    "sobreedifiquen": "sobreedificar",
    "sobreedifiques": "sobreedificar",
    "sobreedifiqué": "sobreedificar",
    "sobreedifiquéis": "sobreedificar",
    "sobreelevaciones": "sobreelevación",
    "sobreempeños": "sobreempeño",
    "sobreempleos": "sobreempleo",
    "sobreentendamos": "sobreentender",
    "sobreentended": "sobreentender",
    "sobreentendemos": "sobreentender",
    "sobreentenderemos": "sobreentender",
    "sobreentenderá": "sobreentender",
    "sobreentenderán": "sobreentender",
    "sobreentenderás": "sobreentender",
    "sobreentenderé": "sobreentender",
    "sobreentenderéis": "sobreentender",
    "sobreentendería": "sobreentender",
    "sobreentenderíais": "sobreentender",
    "sobreentenderíamos": "sobreentender",
    "sobreentenderían": "sobreentender",
    "sobreentenderías": "sobreentender",
    "sobreentendida": "sobreentender",
    "sobreentendidas": "sobreentender",
    "sobreentendido": "sobreentender",
    "sobreentendidos": "sobreentender",
    "sobreentendiendo": "sobreentender",
    "sobreentendiera": "sobreentender",
    "sobreentendierais": "sobreentender",
    "sobreentendieran": "sobreentender",
    "sobreentendieras": "sobreentender",
    "sobreentendiere": "sobreentender",
    "sobreentendiereis": "sobreentender",
    "sobreentendieren": "sobreentender",
    "sobreentendieres": "sobreentender",
    "sobreentendieron": "sobreentender",
    "sobreentendiese": "sobreentender",
    "sobreentendieseis": "sobreentender",
    "sobreentendiesen": "sobreentender",
    "sobreentendieses": "sobreentender",
    "sobreentendimos": "sobreentender",
    "sobreentendiste": "sobreentender",
    "sobreentendisteis": "sobreentender",
    "sobreentendiéramos": "sobreentender",
    "sobreentendiéremos": "sobreentender",
    "sobreentendiésemos": "sobreentender",
    "sobreentendió": "sobreentender",
    "sobreentendáis": "sobreentender",
    "sobreentendéis": "sobreentender",
    "sobreentendí": "sobreentender",
    "sobreentendía": "sobreentender",
    "sobreentendíais": "sobreentender",
    "sobreentendíamos": "sobreentender",
    "sobreentendían": "sobreentender",
    "sobreentendías": "sobreentender",
    "sobreentienda": "sobreentender",
    "sobreentiendan": "sobreentender",
    "sobreentiendas": "sobreentender",
    "sobreentiende": "sobreentender",
    "sobreentienden": "sobreentender",
    "sobreentiendes": "sobreentender",
    "sobreentiendo": "sobreentender",
    "sobreentrenamientos": "sobreentrenamiento",
    "sobreenvejecimientos": "sobreenvejecimiento",
    "sobreerupciones": "sobreerupción",
    "sobreescriba": "sobreescribir",
    "sobreescribamos": "sobreescribir",
    "sobreescriban": "sobreescribir",
    "sobreescribas": "sobreescribir",
    "sobreescribe": "sobreescribir",
    "sobreescriben": "sobreescribir",
    "sobreescribes": "sobreescribir",
    "sobreescribibles": "sobreescribible",
    "sobreescribid": "sobreescribir",
    "sobreescribiendo": "sobreescribir",
    "sobreescribiera": "sobreescribir",
    "sobreescribierais": "sobreescribir",
    "sobreescribieran": "sobreescribir",
    "sobreescribieras": "sobreescribir",
    "sobreescribiere": "sobreescribir",
    "sobreescribiereis": "sobreescribir",
    "sobreescribieren": "sobreescribir",
    "sobreescribieres": "sobreescribir",
    "sobreescribieron": "sobreescribir",
    "sobreescribiese": "sobreescribir",
    "sobreescribieseis": "sobreescribir",
    "sobreescribiesen": "sobreescribir",
    "sobreescribieses": "sobreescribir",
    "sobreescribimos": "sobreescribir",
    "sobreescribiremos": "sobreescribir",
    "sobreescribirá": "sobreescribir",
    "sobreescribirán": "sobreescribir",
    "sobreescribirás": "sobreescribir",
    "sobreescribiré": "sobreescribir",
    "sobreescribiréis": "sobreescribir",
    "sobreescribiría": "sobreescribir",
    "sobreescribiríais": "sobreescribir",
    "sobreescribiríamos": "sobreescribir",
    "sobreescribirían": "sobreescribir",
    "sobreescribirías": "sobreescribir",
    "sobreescribiste": "sobreescribir",
    "sobreescribisteis": "sobreescribir",
    "sobreescribiéramos": "sobreescribir",
    "sobreescribiéremos": "sobreescribir",
    "sobreescribiésemos": "sobreescribir",
    "sobreescribió": "sobreescribir",
    "sobreescribo": "sobreescribir",
    "sobreescribáis": "sobreescribir",
    "sobreescribí": "sobreescribir",
    "sobreescribía": "sobreescribir",
    "sobreescribíais": "sobreescribir",
    "sobreescribíamos": "sobreescribir",
    "sobreescribían": "sobreescribir",
    "sobreescribías": "sobreescribir",
    "sobreescribís": "sobreescribir",
    "sobreescrita": "sobreescribir",
    "sobreescritas": "sobreescribir",
    "sobreescrito": "sobreescribir",
    "sobreescritos": "sobreescribir",
    "sobreesdrújula": "sobreesdrújulo",
    "sobreesdrújulas": "sobreesdrújulo",
    "sobreesdrújulos": "sobreesdrújulo",
    "sobreestima": "sobreestimar",
    "sobreestimaba": "sobreestimar",
    "sobreestimabais": "sobreestimar",
    "sobreestimaban": "sobreestimar",
    "sobreestimabas": "sobreestimar",
    "sobreestimaciones": "sobreestimación",
    "sobreestimad": "sobreestimar",
    "sobreestimada": "sobreestimar",
    "sobreestimadas": "sobreestimar",
    "sobreestimado": "sobreestimar",
    "sobreestimados": "sobreestimar",
    "sobreestimamos": "sobreestimar",
    "sobreestiman": "sobreestimar",
    "sobreestimando": "sobreestimar",
    "sobreestimara": "sobreestimar",
    "sobreestimarais": "sobreestimar",
    "sobreestimaran": "sobreestimar",
    "sobreestimaras": "sobreestimar",
    "sobreestimare": "sobreestimar",
    "sobreestimareis": "sobreestimar",
    "sobreestimaremos": "sobreestimar",
    "sobreestimaren": "sobreestimar",
    "sobreestimares": "sobreestimar",
    "sobreestimaron": "sobreestimar",
    "sobreestimará": "sobreestimar",
    "sobreestimarán": "sobreestimar",
    "sobreestimarás": "sobreestimar",
    "sobreestimaré": "sobreestimar",
    "sobreestimaréis": "sobreestimar",
    "sobreestimaría": "sobreestimar",
    "sobreestimaríais": "sobreestimar",
    "sobreestimaríamos": "sobreestimar",
    "sobreestimarían": "sobreestimar",
    "sobreestimarías": "sobreestimar",
    "sobreestimas": "sobreestimar",
    "sobreestimase": "sobreestimar",
    "sobreestimaseis": "sobreestimar",
    "sobreestimasen": "sobreestimar",
    "sobreestimases": "sobreestimar",
    "sobreestimaste": "sobreestimar",
    "sobreestimasteis": "sobreestimar",
    "sobreestime": "sobreestimar",
    "sobreestimemos": "sobreestimar",
    "sobreestimen": "sobreestimar",
    "sobreestimes": "sobreestimar",
    "sobreestimo": "sobreestimar",
    "sobreestimábamos": "sobreestimar",
    "sobreestimáis": "sobreestimar",
    "sobreestimáramos": "sobreestimar",
    "sobreestimáremos": "sobreestimar",
    "sobreestimásemos": "sobreestimar",
    "sobreestimé": "sobreestimar",
    "sobreestiméis": "sobreestimar",
    "sobreestimó": "sobreestimar",
    "sobreestructuras": "sobreestructura",
    "sobreexcavaciones": "sobreexcavación",
    "sobreexceda": "sobreexceder",
    "sobreexcedamos": "sobreexceder",
    "sobreexcedan": "sobreexceder",
    "sobreexcedas": "sobreexceder",
    "sobreexcede": "sobreexceder",
    "sobreexceded": "sobreexceder",
    "sobreexcedemos": "sobreexceder",
    "sobreexceden": "sobreexceder",
    "sobreexcederemos": "sobreexceder",
    "sobreexcederá": "sobreexceder",
    "sobreexcederán": "sobreexceder",
    "sobreexcederás": "sobreexceder",
    "sobreexcederé": "sobreexceder",
    "sobreexcederéis": "sobreexceder",
    "sobreexcedería": "sobreexceder",
    "sobreexcederíais": "sobreexceder",
    "sobreexcederíamos": "sobreexceder",
    "sobreexcederían": "sobreexceder",
    "sobreexcederías": "sobreexceder",
    "sobreexcedes": "sobreexceder",
    "sobreexcedida": "sobreexceder",
    "sobreexcedidas": "sobreexceder",
    "sobreexcedido": "sobreexceder",
    "sobreexcedidos": "sobreexceder",
    "sobreexcediendo": "sobreexceder",
    "sobreexcediera": "sobreexceder",
    "sobreexcedierais": "sobreexceder",
    "sobreexcedieran": "sobreexceder",
    "sobreexcedieras": "sobreexceder",
    "sobreexcediere": "sobreexceder",
    "sobreexcediereis": "sobreexceder",
    "sobreexcedieren": "sobreexceder",
    "sobreexcedieres": "sobreexceder",
    "sobreexcedieron": "sobreexceder",
    "sobreexcediese": "sobreexceder",
    "sobreexcedieseis": "sobreexceder",
    "sobreexcediesen": "sobreexceder",
    "sobreexcedieses": "sobreexceder",
    "sobreexcedimos": "sobreexceder",
    "sobreexcediste": "sobreexceder",
    "sobreexcedisteis": "sobreexceder",
    "sobreexcediéramos": "sobreexceder",
    "sobreexcediéremos": "sobreexceder",
    "sobreexcediésemos": "sobreexceder",
    "sobreexcedió": "sobreexceder",
    "sobreexcedo": "sobreexceder",
    "sobreexcedáis": "sobreexceder",
    "sobreexcedéis": "sobreexceder",
    "sobreexcedí": "sobreexceder",
    "sobreexcedía": "sobreexceder",
    "sobreexcedíais": "sobreexceder",
    "sobreexcedíamos": "sobreexceder",
    "sobreexcedían": "sobreexceder",
    "sobreexcedías": "sobreexceder",
    "sobreexcita": "sobreexcitar",
    "sobreexcitaba": "sobreexcitar",
    "sobreexcitabais": "sobreexcitar",
    "sobreexcitaban": "sobreexcitar",
    "sobreexcitabas": "sobreexcitar",
    "sobreexcitaciones": "sobreexcitación",
    "sobreexcitacoónes": "sobreexcitación",
    "sobreexcitad": "sobreexcitar",
    "sobreexcitada": "sobreexcitar",
    "sobreexcitadas": "sobreexcitar",
    "sobreexcitado": "sobreexcitar",
    "sobreexcitados": "sobreexcitar",
    "sobreexcitamos": "sobreexcitar",
    "sobreexcitan": "sobreexcitar",
    "sobreexcitando": "sobreexcitar",
    "sobreexcitara": "sobreexcitar",
    "sobreexcitarais": "sobreexcitar",
    "sobreexcitaran": "sobreexcitar",
    "sobreexcitaras": "sobreexcitar",
    "sobreexcitare": "sobreexcitar",
    "sobreexcitareis": "sobreexcitar",
    "sobreexcitaremos": "sobreexcitar",
    "sobreexcitaren": "sobreexcitar",
    "sobreexcitares": "sobreexcitar",
    "sobreexcitaron": "sobreexcitar",
    "sobreexcitará": "sobreexcitar",
    "sobreexcitarán": "sobreexcitar",
    "sobreexcitarás": "sobreexcitar",
    "sobreexcitaré": "sobreexcitar",
    "sobreexcitaréis": "sobreexcitar",
    "sobreexcitaría": "sobreexcitar",
    "sobreexcitaríais": "sobreexcitar",
    "sobreexcitaríamos": "sobreexcitar",
    "sobreexcitarían": "sobreexcitar",
    "sobreexcitarías": "sobreexcitar",
    "sobreexcitas": "sobreexcitar",
    "sobreexcitase": "sobreexcitar",
    "sobreexcitaseis": "sobreexcitar",
    "sobreexcitasen": "sobreexcitar",
    "sobreexcitases": "sobreexcitar",
    "sobreexcitaste": "sobreexcitar",
    "sobreexcitasteis": "sobreexcitar",
    "sobreexcite": "sobreexcitar",
    "sobreexcitemos": "sobreexcitar",
    "sobreexciten": "sobreexcitar",
    "sobreexcites": "sobreexcitar",
    "sobreexcito": "sobreexcitar",
    "sobreexcitábamos": "sobreexcitar",
    "sobreexcitáis": "sobreexcitar",
    "sobreexcitáramos": "sobreexcitar",
    "sobreexcitáremos": "sobreexcitar",
    "sobreexcitásemos": "sobreexcitar",
    "sobreexcité": "sobreexcitar",
    "sobreexcitéis": "sobreexcitar",
    "sobreexcitó": "sobreexcitar",
    "sobreexpansiones": "sobreexpansión",
    "sobreexplotaciones": "sobreexplotación",
    "sobreexpondremos": "sobreexponer",
    "sobreexpondrá": "sobreexponer",
    "sobreexpondrán": "sobreexponer",
    "sobreexpondrás": "sobreexponer",
    "sobreexpondré": "sobreexponer",
    "sobreexpondréis": "sobreexponer",
    "sobreexpondría": "sobreexponer",
    "sobreexpondríais": "sobreexponer",
    "sobreexpondríamos": "sobreexponer",
    "sobreexpondrían": "sobreexponer",
    "sobreexpondrías": "sobreexponer",
    "sobreexpone": "sobreexponer",
    "sobreexponed": "sobreexponer",
    "sobreexponemos": "sobreexponer",
    "sobreexponen": "sobreexponer",
    "sobreexpones": "sobreexponer",
    "sobreexponga": "sobreexponer",
    "sobreexpongamos": "sobreexponer",
    "sobreexpongan": "sobreexponer",
    "sobreexpongas": "sobreexponer",
    "sobreexpongo": "sobreexponer",
    "sobreexpongáis": "sobreexponer",
    "sobreexponiendo": "sobreexponer",
    "sobreexponéis": "sobreexponer",
    "sobreexponía": "sobreexponer",
    "sobreexponíais": "sobreexponer",
    "sobreexponíamos": "sobreexponer",
    "sobreexponían": "sobreexponer",
    "sobreexponías": "sobreexponer",
    "sobreexposiciones": "sobreexposición",
    "sobreexpuesta": "sobreexponer",
    "sobreexpuestas": "sobreexponer",
    "sobreexpuesto": "sobreexponer",
    "sobreexpuestos": "sobreexponer",
    "sobreexpuse": "sobreexponer",
    "sobreexpusiera": "sobreexponer",
    "sobreexpusierais": "sobreexponer",
    "sobreexpusieran": "sobreexponer",
    "sobreexpusieras": "sobreexponer",
    "sobreexpusiere": "sobreexponer",
    "sobreexpusiereis": "sobreexponer",
    "sobreexpusieren": "sobreexponer",
    "sobreexpusieres": "sobreexponer",
    "sobreexpusieron": "sobreexponer",
    "sobreexpusiese": "sobreexponer",
    "sobreexpusieseis": "sobreexponer",
    "sobreexpusiesen": "sobreexponer",
    "sobreexpusieses": "sobreexponer",
    "sobreexpusimos": "sobreexponer",
    "sobreexpusiste": "sobreexponer",
    "sobreexpusisteis": "sobreexponer",
    "sobreexpusiéramos": "sobreexponer",
    "sobreexpusiéremos": "sobreexponer",
    "sobreexpusiésemos": "sobreexponer",
    "sobreexpuso": "sobreexponer",
    "sobreexpón": "sobreexponer",
    "sobrefaces": "sobrefaz",
    "sobrefacturaciones": "sobrefacturación",
    "sobrefaldas": "sobrefalda",
    "sobreflores": "sobreflor",
    "sobrefusiones": "sobrefusión",
    "sobregiros": "sobregiro",
    "sobreguardas": "sobreguarda",
    "sobrehaces": "sobrehaz",
    "sobreherida": "sobreherido",
    "sobreheridas": "sobreherido",
    "sobreheridos": "sobreherido",
    "sobrehilaba": "sobrehilar",
    "sobrehilabais": "sobrehilar",
    "sobrehilaban": "sobrehilar",
    "sobrehilabas": "sobrehilar",
    "sobrehilad": "sobrehilar",
    "sobrehilada": "sobrehilar",
    "sobrehiladas": "sobrehilar",
    "sobrehilado": "sobrehilar",
    "sobrehilados": "sobrehilar",
    "sobrehilamos": "sobrehilar",
    "sobrehilando": "sobrehilar",
    "sobrehilara": "sobrehilar",
    "sobrehilarais": "sobrehilar",
    "sobrehilaran": "sobrehilar",
    "sobrehilaras": "sobrehilar",
    "sobrehilare": "sobrehilar",
    "sobrehilareis": "sobrehilar",
    "sobrehilaremos": "sobrehilar",
    "sobrehilaren": "sobrehilar",
    "sobrehilares": "sobrehilar",
    "sobrehilaron": "sobrehilar",
    "sobrehilará": "sobrehilar",
    "sobrehilarán": "sobrehilar",
    "sobrehilarás": "sobrehilar",
    "sobrehilaré": "sobrehilar",
    "sobrehilaréis": "sobrehilar",
    "sobrehilaría": "sobrehilar",
    "sobrehilaríais": "sobrehilar",
    "sobrehilaríamos": "sobrehilar",
    "sobrehilarían": "sobrehilar",
    "sobrehilarías": "sobrehilar",
    "sobrehilase": "sobrehilar",
    "sobrehilaseis": "sobrehilar",
    "sobrehilasen": "sobrehilar",
    "sobrehilases": "sobrehilar",
    "sobrehilaste": "sobrehilar",
    "sobrehilasteis": "sobrehilar",
    "sobrehilemos": "sobrehilar",
    "sobrehilábamos": "sobrehilar",
    "sobrehiláis": "sobrehilar",
    "sobrehiláramos": "sobrehilar",
    "sobrehiláremos": "sobrehilar",
    "sobrehilásemos": "sobrehilar",
    "sobrehilé": "sobrehilar",
    "sobrehiléis": "sobrehilar",
    "sobrehiló": "sobrehilar",
    "sobrehuesos": "sobrehueso",
    "sobrehumana": "sobrehumano",
    "sobrehumanas": "sobrehumano",
    "sobrehumanos": "sobrehumano",
    "sobrehíla": "sobrehilar",
    "sobrehílan": "sobrehilar",
    "sobrehílas": "sobrehilar",
    "sobrehíle": "sobrehilar",
    "sobrehílen": "sobrehilar",
    "sobrehíles": "sobrehilar",
    "sobrehílo": "sobrehilar",
    "sobrehílos": "sobrehílo",
    "sobreimposiciones": "sobreimposición",
    "sobreimpresiones": "sobreimpresión",
    "sobreimpuestos": "sobreimpuesto",
    "sobreinscripciones": "sobreinscripción",
    "sobreintensidades": "sobreintensidad",
    "sobreinterpretaciones": "sobreinterpretación",
    "sobreinversiones": "sobreinversión",
    "sobrejuanetes": "sobrejuanete",
    "sobrelechos": "sobrelecho",
    "sobrellaves": "sobrellave",
    "sobrellena": "sobrelleno",
    "sobrellenaba": "sobrellenar",
    "sobrellenabais": "sobrellenar",
    "sobrellenaban": "sobrellenar",
    "sobrellenabas": "sobrellenar",
    "sobrellenad": "sobrellenar",
    "sobrellenada": "sobrellenar",
    "sobrellenadas": "sobrellenar",
    "sobrellenado": "sobrellenar",
    "sobrellenados": "sobrellenar",
    "sobrellenamos": "sobrellenar",
    "sobrellenan": "sobrellenar",
    "sobrellenando": "sobrellenar",
    "sobrellenara": "sobrellenar",
    "sobrellenarais": "sobrellenar",
    "sobrellenaran": "sobrellenar",
    "sobrellenaras": "sobrellenar",
    "sobrellenare": "sobrellenar",
    "sobrellenareis": "sobrellenar",
    "sobrellenaremos": "sobrellenar",
    "sobrellenaren": "sobrellenar",
    "sobrellenares": "sobrellenar",
    "sobrellenaron": "sobrellenar",
    "sobrellenará": "sobrellenar",
    "sobrellenarán": "sobrellenar",
    "sobrellenarás": "sobrellenar",
    "sobrellenaré": "sobrellenar",
    "sobrellenaréis": "sobrellenar",
    "sobrellenaría": "sobrellenar",
    "sobrellenaríais": "sobrellenar",
    "sobrellenaríamos": "sobrellenar",
    "sobrellenarían": "sobrellenar",
    "sobrellenarías": "sobrellenar",
    "sobrellenas": "sobrelleno",
    "sobrellenase": "sobrellenar",
    "sobrellenaseis": "sobrellenar",
    "sobrellenasen": "sobrellenar",
    "sobrellenases": "sobrellenar",
    "sobrellenaste": "sobrellenar",
    "sobrellenasteis": "sobrellenar",
    "sobrellene": "sobrellenar",
    "sobrellenemos": "sobrellenar",
    "sobrellenen": "sobrellenar",
    "sobrellenes": "sobrellenar",
    "sobrelleno": "sobrellenar",
    "sobrellenos": "sobrelleno",
    "sobrellenábamos": "sobrellenar",
    "sobrellenáis": "sobrellenar",
    "sobrellenáramos": "sobrellenar",
    "sobrellenáremos": "sobrellenar",
    "sobrellenásemos": "sobrellenar",
    "sobrellené": "sobrellenar",
    "sobrellenéis": "sobrellenar",
    "sobrellenó": "sobrellenar",
    "sobrelleva": "sobrellevar",
    "sobrellevaba": "sobrellevar",
    "sobrellevabais": "sobrellevar",
    "sobrellevaban": "sobrellevar",
    "sobrellevabas": "sobrellevar",
    "sobrellevad": "sobrellevar",
    "sobrellevada": "sobrellevar",
    "sobrellevadas": "sobrellevar",
    "sobrellevado": "sobrellevar",
    "sobrellevados": "sobrellevar",
    "sobrellevamos": "sobrellevar",
    "sobrellevan": "sobrellevar",
    "sobrellevando": "sobrellevar",
    "sobrellevara": "sobrellevar",
    "sobrellevarais": "sobrellevar",
    "sobrellevaran": "sobrellevar",
    "sobrellevaras": "sobrellevar",
    "sobrellevare": "sobrellevar",
    "sobrellevareis": "sobrellevar",
    "sobrellevaremos": "sobrellevar",
    "sobrellevaren": "sobrellevar",
    "sobrellevares": "sobrellevar",
    "sobrellevaron": "sobrellevar",
    "sobrellevará": "sobrellevar",
    "sobrellevarán": "sobrellevar",
    "sobrellevarás": "sobrellevar",
    "sobrellevaré": "sobrellevar",
    "sobrellevaréis": "sobrellevar",
    "sobrellevaría": "sobrellevar",
    "sobrellevaríais": "sobrellevar",
    "sobrellevaríamos": "sobrellevar",
    "sobrellevarían": "sobrellevar",
    "sobrellevarías": "sobrellevar",
    "sobrellevas": "sobrellevar",
    "sobrellevase": "sobrellevar",
    "sobrellevaseis": "sobrellevar",
    "sobrellevasen": "sobrellevar",
    "sobrellevases": "sobrellevar",
    "sobrellevaste": "sobrellevar",
    "sobrellevasteis": "sobrellevar",
    "sobrelleve": "sobrellevar",
    "sobrellevemos": "sobrellevar",
    "sobrelleven": "sobrellevar",
    "sobrelleves": "sobrellevar",
    "sobrellevo": "sobrellevar",
    "sobrellevábamos": "sobrellevar",
    "sobrelleváis": "sobrellevar",
    "sobrelleváramos": "sobrellevar",
    "sobrelleváremos": "sobrellevar",
    "sobrellevásemos": "sobrellevar",
    "sobrellevé": "sobrellevar",
    "sobrellevéis": "sobrellevar",
    "sobrellevó": "sobrellevar",
    "sobrelongitudes": "sobrelongitud",
    "sobremanos": "sobremano",
    "sobremasticada": "sobremasticado",
    "sobremasticadas": "sobremasticado",
    "sobremasticados": "sobremasticado",
    "sobremedias": "sobremedia",
    "sobremedidas": "sobremedida",
    "sobremesanas": "sobremesana",
    "sobremesas": "sobremesa",
    "sobremodulaciones": "sobremodulación",
    "sobremoliendas": "sobremolienda",
    "sobremordidas": "sobremordida",
    "sobremos": "sobrar",
    "sobren": "sobrar",
    "sobrenada": "sobrenadar",
    "sobrenadaba": "sobrenadar",
    "sobrenadabais": "sobrenadar",
    "sobrenadaban": "sobrenadar",
    "sobrenadabas": "sobrenadar",
    "sobrenadad": "sobrenadar",
    "sobrenadada": "sobrenadar",
    "sobrenadadas": "sobrenadar",
    "sobrenadado": "sobrenadar",
    "sobrenadados": "sobrenadar",
    "sobrenadamos": "sobrenadar",
    "sobrenadan": "sobrenadar",
    "sobrenadando": "sobrenadar",
    "sobrenadara": "sobrenadar",
    "sobrenadarais": "sobrenadar",
    "sobrenadaran": "sobrenadar",
    "sobrenadaras": "sobrenadar",
    "sobrenadare": "sobrenadar",
    "sobrenadareis": "sobrenadar",
    "sobrenadaremos": "sobrenadar",
    "sobrenadaren": "sobrenadar",
    "sobrenadares": "sobrenadar",
    "sobrenadaron": "sobrenadar",
    "sobrenadará": "sobrenadar",
    "sobrenadarán": "sobrenadar",
    "sobrenadarás": "sobrenadar",
    "sobrenadaré": "sobrenadar",
    "sobrenadaréis": "sobrenadar",
    "sobrenadaría": "sobrenadar",
    "sobrenadaríais": "sobrenadar",
    "sobrenadaríamos": "sobrenadar",
    "sobrenadarían": "sobrenadar",
    "sobrenadarías": "sobrenadar",
    "sobrenadas": "sobrenadar",
    "sobrenadase": "sobrenadar",
    "sobrenadaseis": "sobrenadar",
    "sobrenadasen": "sobrenadar",
    "sobrenadases": "sobrenadar",
    "sobrenadaste": "sobrenadar",
    "sobrenadasteis": "sobrenadar",
    "sobrenade": "sobrenadar",
    "sobrenademos": "sobrenadar",
    "sobrenaden": "sobrenadar",
    "sobrenades": "sobrenadar",
    "sobrenado": "sobrenadar",
    "sobrenadábamos": "sobrenadar",
    "sobrenadáis": "sobrenadar",
    "sobrenadáramos": "sobrenadar",
    "sobrenadáremos": "sobrenadar",
    "sobrenadásemos": "sobrenadar",
    "sobrenadé": "sobrenadar",
    "sobrenadéis": "sobrenadar",
    "sobrenadó": "sobrenadar",
    "sobrenaturales": "sobrenatural",
    "sobrenombres": "sobrenombre",
    "sobrentendamos": "sobrentender",
    "sobrentended": "sobrentender",
    "sobrentendemos": "sobrentender",
    "sobrentenderemos": "sobrentender",
    "sobrentenderá": "sobrentender",
    "sobrentenderán": "sobrentender",
    "sobrentenderás": "sobrentender",
    "sobrentenderé": "sobrentender",
    "sobrentenderéis": "sobrentender",
    "sobrentendería": "sobrentender",
    "sobrentenderíais": "sobrentender",
    "sobrentenderíamos": "sobrentender",
    "sobrentenderían": "sobrentender",
    "sobrentenderías": "sobrentender",
    "sobrentendida": "sobrentender",
    "sobrentendidas": "sobrentender",
    "sobrentendido": "sobrentender",
    "sobrentendidos": "sobrentender",
    "sobrentendiendo": "sobrentender",
    "sobrentendiera": "sobrentender",
    "sobrentendierais": "sobrentender",
    "sobrentendieran": "sobrentender",
    "sobrentendieras": "sobrentender",
    "sobrentendiere": "sobrentender",
    "sobrentendiereis": "sobrentender",
    "sobrentendieren": "sobrentender",
    "sobrentendieres": "sobrentender",
    "sobrentendieron": "sobrentender",
    "sobrentendiese": "sobrentender",
    "sobrentendieseis": "sobrentender",
    "sobrentendiesen": "sobrentender",
    "sobrentendieses": "sobrentender",
    "sobrentendimos": "sobrentender",
    "sobrentendiste": "sobrentender",
    "sobrentendisteis": "sobrentender",
    "sobrentendiéramos": "sobrentender",
    "sobrentendiéremos": "sobrentender",
    "sobrentendiésemos": "sobrentender",
    "sobrentendió": "sobrentender",
    "sobrentendáis": "sobrentender",
    "sobrentendéis": "sobrentender",
    "sobrentendí": "sobrentender",
    "sobrentendía": "sobrentender",
    "sobrentendíais": "sobrentender",
    "sobrentendíamos": "sobrentender",
    "sobrentendían": "sobrentender",
    "sobrentendías": "sobrentender",
    "sobrentienda": "sobrentender",
    "sobrentiendan": "sobrentender",
    "sobrentiendas": "sobrentender",
    "sobrentiende": "sobrentender",
    "sobrentienden": "sobrentender",
    "sobrentiendes": "sobrentender",
    "sobrentiendo": "sobrentender",
    "sobreocupaciones": "sobreocupación",
    "sobreofertas": "sobreoferta",
    "sobrepagas": "sobrepaga",
    "sobrepartos": "sobreparto",
    "sobrepasa": "sobrepasar",
    "sobrepasaba": "sobrepasar",
    "sobrepasabais": "sobrepasar",
    "sobrepasaban": "sobrepasar",
    "sobrepasabas": "sobrepasar",
    "sobrepasables": "sobrepasable",
    "sobrepasad": "sobrepasar",
    "sobrepasada": "sobrepasar",
    "sobrepasadas": "sobrepasar",
    "sobrepasado": "sobrepasar",
    "sobrepasados": "sobrepasar",
    "sobrepasamos": "sobrepasar",
    "sobrepasan": "sobrepasar",
    "sobrepasando": "sobrepasar",
    "sobrepasara": "sobrepasar",
    "sobrepasarais": "sobrepasar",
    "sobrepasaran": "sobrepasar",
    "sobrepasaras": "sobrepasar",
    "sobrepasare": "sobrepasar",
    "sobrepasareis": "sobrepasar",
    "sobrepasaremos": "sobrepasar",
    "sobrepasaren": "sobrepasar",
    "sobrepasares": "sobrepasar",
    "sobrepasaron": "sobrepasar",
    "sobrepasará": "sobrepasar",
    "sobrepasarán": "sobrepasar",
    "sobrepasarás": "sobrepasar",
    "sobrepasaré": "sobrepasar",
    "sobrepasaréis": "sobrepasar",
    "sobrepasaría": "sobrepasar",
    "sobrepasaríais": "sobrepasar",
    "sobrepasaríamos": "sobrepasar",
    "sobrepasarían": "sobrepasar",
    "sobrepasarías": "sobrepasar",
    "sobrepasas": "sobrepasar",
    "sobrepasase": "sobrepasar",
    "sobrepasaseis": "sobrepasar",
    "sobrepasasen": "sobrepasar",
    "sobrepasases": "sobrepasar",
    "sobrepasaste": "sobrepasar",
    "sobrepasasteis": "sobrepasar",
    "sobrepase": "sobrepasar",
    "sobrepasemos": "sobrepasar",
    "sobrepasen": "sobrepasar",
    "sobrepases": "sobrepasar",
    "sobrepaso": "sobrepasar",
    "sobrepasábamos": "sobrepasar",
    "sobrepasáis": "sobrepasar",
    "sobrepasáramos": "sobrepasar",
    "sobrepasáremos": "sobrepasar",
    "sobrepasásemos": "sobrepasar",
    "sobrepasé": "sobrepasar",
    "sobrepaséis": "sobrepasar",
    "sobrepasó": "sobrepasar",
    "sobrepaños": "sobrepaño",
    "sobrepellices": "sobrepelliz",
    "sobreperforaciones": "sobreperforación",
    "sobrepescas": "sobrepesca",
    "sobrepesos": "sobrepeso",
    "sobrepinta": "sobrepintar",
    "sobrepintaba": "sobrepintar",
    "sobrepintabais": "sobrepintar",
    "sobrepintaban": "sobrepintar",
    "sobrepintabas": "sobrepintar",
    "sobrepintad": "sobrepintar",
    "sobrepintada": "sobrepintar",
    "sobrepintadas": "sobrepintar",
    "sobrepintado": "sobrepintar",
    "sobrepintados": "sobrepintar",
    "sobrepintamos": "sobrepintar",
    "sobrepintan": "sobrepintar",
    "sobrepintando": "sobrepintar",
    "sobrepintara": "sobrepintar",
    "sobrepintarais": "sobrepintar",
    "sobrepintaran": "sobrepintar",
    "sobrepintaras": "sobrepintar",
    "sobrepintare": "sobrepintar",
    "sobrepintareis": "sobrepintar",
    "sobrepintaremos": "sobrepintar",
    "sobrepintaren": "sobrepintar",
    "sobrepintares": "sobrepintar",
    "sobrepintaron": "sobrepintar",
    "sobrepintará": "sobrepintar",
    "sobrepintarán": "sobrepintar",
    "sobrepintarás": "sobrepintar",
    "sobrepintaré": "sobrepintar",
    "sobrepintaréis": "sobrepintar",
    "sobrepintaría": "sobrepintar",
    "sobrepintaríais": "sobrepintar",
    "sobrepintaríamos": "sobrepintar",
    "sobrepintarían": "sobrepintar",
    "sobrepintarías": "sobrepintar",
    "sobrepintas": "sobrepintar",
    "sobrepintase": "sobrepintar",
    "sobrepintaseis": "sobrepintar",
    "sobrepintasen": "sobrepintar",
    "sobrepintases": "sobrepintar",
    "sobrepintaste": "sobrepintar",
    "sobrepintasteis": "sobrepintar",
    "sobrepinte": "sobrepintar",
    "sobrepintemos": "sobrepintar",
    "sobrepinten": "sobrepintar",
    "sobrepintes": "sobrepintar",
    "sobrepinto": "sobrepintar",
    "sobrepintábamos": "sobrepintar",
    "sobrepintáis": "sobrepintar",
    "sobrepintáramos": "sobrepintar",
    "sobrepintáremos": "sobrepintar",
    "sobrepintásemos": "sobrepintar",
    "sobrepinté": "sobrepintar",
    "sobrepintéis": "sobrepintar",
    "sobrepintó": "sobrepintar",
    "sobrepiés": "sobrepié",
    "sobreplanes": "sobreplán",
    "sobrepondremos": "sobreponer",
    "sobrepondrá": "sobreponer",
    "sobrepondrán": "sobreponer",
    "sobrepondrás": "sobreponer",
    "sobrepondré": "sobreponer",
    "sobrepondréis": "sobreponer",
    "sobrepondría": "sobreponer",
    "sobrepondríais": "sobreponer",
    "sobrepondríamos": "sobreponer",
    "sobrepondrían": "sobreponer",
    "sobrepondrías": "sobreponer",
    "sobrepone": "sobreponer",
    "sobreponed": "sobreponer",
    "sobreponemos": "sobreponer",
    "sobreponen": "sobreponer",
    "sobrepones": "sobreponer",
    "sobreponga": "sobreponer",
    "sobrepongamos": "sobreponer",
    "sobrepongan": "sobreponer",
    "sobrepongas": "sobreponer",
    "sobrepongo": "sobreponer",
    "sobrepongáis": "sobreponer",
    "sobreponiendo": "sobreponer",
    "sobreponéis": "sobreponer",
    "sobreponía": "sobreponer",
    "sobreponíais": "sobreponer",
    "sobreponíamos": "sobreponer",
    "sobreponían": "sobreponer",
    "sobreponías": "sobreponer",
    "sobrepotencias": "sobrepotencia",
    "sobreprecios": "sobreprecio",
    "sobrepresiones": "sobrepresión",
    "sobreprimas": "sobreprima",
    "sobreproducciones": "sobreproducción",
    "sobreproporcionada": "sobreproporcionado",
    "sobreproporcionadas": "sobreproporcionado",
    "sobreproporcionados": "sobreproporcionado",
    "sobrepuertas": "sobrepuerta",
    "sobrepuesta": "sobreponer",
    "sobrepuestas": "sobreponer",
    "sobrepuesto": "sobreponer",
    "sobrepuestos": "sobreponer",
    "sobrepuja": "sobrepujar",
    "sobrepujaba": "sobrepujar",
    "sobrepujabais": "sobrepujar",
    "sobrepujaban": "sobrepujar",
    "sobrepujabas": "sobrepujar",
    "sobrepujad": "sobrepujar",
    "sobrepujada": "sobrepujar",
    "sobrepujadas": "sobrepujar",
    "sobrepujado": "sobrepujar",
    "sobrepujados": "sobrepujar",
    "sobrepujamientos": "sobrepujamiento",
    "sobrepujamos": "sobrepujar",
    "sobrepujan": "sobrepujar",
    "sobrepujando": "sobrepujar",
    "sobrepujanzas": "sobrepujanza",
    "sobrepujara": "sobrepujar",
    "sobrepujarais": "sobrepujar",
    "sobrepujaran": "sobrepujar",
    "sobrepujaras": "sobrepujar",
    "sobrepujare": "sobrepujar",
    "sobrepujareis": "sobrepujar",
    "sobrepujaremos": "sobrepujar",
    "sobrepujaren": "sobrepujar",
    "sobrepujares": "sobrepujar",
    "sobrepujaron": "sobrepujar",
    "sobrepujará": "sobrepujar",
    "sobrepujarán": "sobrepujar",
    "sobrepujarás": "sobrepujar",
    "sobrepujaré": "sobrepujar",
    "sobrepujaréis": "sobrepujar",
    "sobrepujaría": "sobrepujar",
    "sobrepujaríais": "sobrepujar",
    "sobrepujaríamos": "sobrepujar",
    "sobrepujarían": "sobrepujar",
    "sobrepujarías": "sobrepujar",
    "sobrepujas": "sobrepujar",
    "sobrepujase": "sobrepujar",
    "sobrepujaseis": "sobrepujar",
    "sobrepujasen": "sobrepujar",
    "sobrepujases": "sobrepujar",
    "sobrepujaste": "sobrepujar",
    "sobrepujasteis": "sobrepujar",
    "sobrepuje": "sobrepujar",
    "sobrepujemos": "sobrepujar",
    "sobrepujen": "sobrepujar",
    "sobrepujes": "sobrepujar",
    "sobrepujo": "sobrepujar",
    "sobrepujábamos": "sobrepujar",
    "sobrepujáis": "sobrepujar",
    "sobrepujáramos": "sobrepujar",
    "sobrepujáremos": "sobrepujar",
    "sobrepujásemos": "sobrepujar",
    "sobrepujé": "sobrepujar",
    "sobrepujéis": "sobrepujar",
    "sobrepujó": "sobrepujar",
    "sobrepuse": "sobreponer",
    "sobrepusiera": "sobreponer",
    "sobrepusierais": "sobreponer",
    "sobrepusieran": "sobreponer",
    "sobrepusieras": "sobreponer",
    "sobrepusiere": "sobreponer",
    "sobrepusiereis": "sobreponer",
    "sobrepusieren": "sobreponer",
    "sobrepusieres": "sobreponer",
    "sobrepusieron": "sobreponer",
    "sobrepusiese": "sobreponer",
    "sobrepusieseis": "sobreponer",
    "sobrepusiesen": "sobreponer",
    "sobrepusieses": "sobreponer",
    "sobrepusimos": "sobreponer",
    "sobrepusiste": "sobreponer",
    "sobrepusisteis": "sobreponer",
    "sobrepusiéramos": "sobreponer",
    "sobrepusiéremos": "sobreponer",
    "sobrepusiésemos": "sobreponer",
    "sobrepuso": "sobreponer",
    "sobrepón": "sobreponer",
    "sobrequillas": "sobrequilla",
    "sobrera": "sobrero",
    "sobreras": "sobrero",
    "sobreros": "sobrero",
    "sobrerrondas": "sobrerronda",
    "sobres": "sobre",
    "sobresal": "sobresalir",
    "sobresaldremos": "sobresalir",
    "sobresaldrá": "sobresalir",
    "sobresaldrán": "sobresalir",
    "sobresaldrás": "sobresalir",
    "sobresaldré": "sobresalir",
    "sobresaldréis": "sobresalir",
    "sobresaldría": "sobresalir",
    "sobresaldríais": "sobresalir",
    "sobresaldríamos": "sobresalir",
    "sobresaldrían": "sobresalir",
    "sobresaldrías": "sobresalir",
    "sobresale": "sobresalir",
    "sobresalen": "sobresalir",
    "sobresales": "sobresalir",
    "sobresalga": "sobresalir",
    "sobresalgamos": "sobresalir",
    "sobresalgan": "sobresalir",
    "sobresalgas": "sobresalir",
    "sobresalgo": "sobresalir",
    "sobresalgáis": "sobresalir",
    "sobresalid": "sobresalir",
    "sobresalida": "sobresalir",
    "sobresalidas": "sobresalir",
    "sobresalido": "sobresalir",
    "sobresalidos": "sobresalir",
    "sobresaliendo": "sobresalir",
    "sobresalientas": "sobresalienta",
    "sobresalientes": "sobresaliente",
    "sobresaliera": "sobresalir",
    "sobresalierais": "sobresalir",
    "sobresalieran": "sobresalir",
    "sobresalieras": "sobresalir",
    "sobresaliere": "sobresalir",
    "sobresaliereis": "sobresalir",
    "sobresalieren": "sobresalir",
    "sobresalieres": "sobresalir",
    "sobresalieron": "sobresalir",
    "sobresaliese": "sobresalir",
    "sobresalieseis": "sobresalir",
    "sobresaliesen": "sobresalir",
    "sobresalieses": "sobresalir",
    "sobresalimos": "sobresalir",
    "sobresaliste": "sobresalir",
    "sobresalisteis": "sobresalir",
    "sobresaliéramos": "sobresalir",
    "sobresaliéremos": "sobresalir",
    "sobresaliésemos": "sobresalir",
    "sobresalió": "sobresalir",
    "sobresalta": "sobresaltar",
    "sobresaltaba": "sobresaltar",
    "sobresaltabais": "sobresaltar",
    "sobresaltaban": "sobresaltar",
    "sobresaltabas": "sobresaltar",
    "sobresaltad": "sobresaltar",
    "sobresaltada": "sobresaltar",
    "sobresaltadas": "sobresaltar",
    "sobresaltado": "sobresaltar",
    "sobresaltados": "sobresaltar",
    "sobresaltamos": "sobresaltar",
    "sobresaltan": "sobresaltar",
    "sobresaltando": "sobresaltar",
    "sobresaltara": "sobresaltar",
    "sobresaltarais": "sobresaltar",
    "sobresaltaran": "sobresaltar",
    "sobresaltaras": "sobresaltar",
    "sobresaltare": "sobresaltar",
    "sobresaltareis": "sobresaltar",
    "sobresaltaremos": "sobresaltar",
    "sobresaltaren": "sobresaltar",
    "sobresaltares": "sobresaltar",
    "sobresaltaron": "sobresaltar",
    "sobresaltará": "sobresaltar",
    "sobresaltarán": "sobresaltar",
    "sobresaltarás": "sobresaltar",
    "sobresaltaré": "sobresaltar",
    "sobresaltaréis": "sobresaltar",
    "sobresaltaría": "sobresaltar",
    "sobresaltaríais": "sobresaltar",
    "sobresaltaríamos": "sobresaltar",
    "sobresaltarían": "sobresaltar",
    "sobresaltarías": "sobresaltar",
    "sobresaltas": "sobresaltar",
    "sobresaltase": "sobresaltar",
    "sobresaltaseis": "sobresaltar",
    "sobresaltasen": "sobresaltar",
    "sobresaltases": "sobresaltar",
    "sobresaltaste": "sobresaltar",
    "sobresaltasteis": "sobresaltar",
    "sobresalte": "sobresaltar",
    "sobresaltemos": "sobresaltar",
    "sobresalten": "sobresaltar",
    "sobresaltes": "sobresaltar",
    "sobresalto": "sobresaltar",
    "sobresaltos": "sobresalto",
    "sobresaltábamos": "sobresaltar",
    "sobresaltáis": "sobresaltar",
    "sobresaltáramos": "sobresaltar",
    "sobresaltáremos": "sobresaltar",
    "sobresaltásemos": "sobresaltar",
    "sobresalté": "sobresaltar",
    "sobresaltéis": "sobresaltar",
    "sobresaltó": "sobresaltar",
    "sobresalí": "sobresalir",
    "sobresalía": "sobresalir",
    "sobresalíais": "sobresalir",
    "sobresalíamos": "sobresalir",
    "sobresalían": "sobresalir",
    "sobresalías": "sobresalir",
    "sobresalís": "sobresalir",
    "sobresatura": "sobresaturar",
    "sobresaturaba": "sobresaturar",
    "sobresaturabais": "sobresaturar",
    "sobresaturaban": "sobresaturar",
    "sobresaturabas": "sobresaturar",
    "sobresaturaciones": "sobresaturación",
    "sobresaturad": "sobresaturar",
    "sobresaturada": "sobresaturar",
    "sobresaturadas": "sobresaturar",
    "sobresaturado": "sobresaturar",
    "sobresaturados": "sobresaturar",
    "sobresaturamos": "sobresaturar",
    "sobresaturan": "sobresaturar",
    "sobresaturando": "sobresaturar",
    "sobresaturara": "sobresaturar",
    "sobresaturarais": "sobresaturar",
    "sobresaturaran": "sobresaturar",
    "sobresaturaras": "sobresaturar",
    "sobresaturare": "sobresaturar",
    "sobresaturareis": "sobresaturar",
    "sobresaturaremos": "sobresaturar",
    "sobresaturaren": "sobresaturar",
    "sobresaturares": "sobresaturar",
    "sobresaturaron": "sobresaturar",
    "sobresaturará": "sobresaturar",
    "sobresaturarán": "sobresaturar",
    "sobresaturarás": "sobresaturar",
    "sobresaturaré": "sobresaturar",
    "sobresaturaréis": "sobresaturar",
    "sobresaturaría": "sobresaturar",
    "sobresaturaríais": "sobresaturar",
    "sobresaturaríamos": "sobresaturar",
    "sobresaturarían": "sobresaturar",
    "sobresaturarías": "sobresaturar",
    "sobresaturas": "sobresaturar",
    "sobresaturase": "sobresaturar",
    "sobresaturaseis": "sobresaturar",
    "sobresaturasen": "sobresaturar",
    "sobresaturases": "sobresaturar",
    "sobresaturaste": "sobresaturar",
    "sobresaturasteis": "sobresaturar",
    "sobresature": "sobresaturar",
    "sobresaturemos": "sobresaturar",
    "sobresaturen": "sobresaturar",
    "sobresatures": "sobresaturar",
    "sobresaturo": "sobresaturar",
    "sobresaturábamos": "sobresaturar",
    "sobresaturáis": "sobresaturar",
    "sobresaturáramos": "sobresaturar",
    "sobresaturáremos": "sobresaturar",
    "sobresaturásemos": "sobresaturar",
    "sobresaturé": "sobresaturar",
    "sobresaturéis": "sobresaturar",
    "sobresaturó": "sobresaturar",
    "sobrescrituras": "sobrescritura",
    "sobresdrújula": "sobresdrújulo",
    "sobresdrújulas": "sobresdrújulo",
    "sobresdrújulos": "sobresdrújulo",
    "sobresea": "sobreseer",
    "sobreseamos": "sobreseer",
    "sobresean": "sobreseer",
    "sobreseas": "sobreseer",
    "sobresee": "sobreseer",
    "sobreseed": "sobreseer",
    "sobreseemos": "sobreseer",
    "sobreseen": "sobreseer",
    "sobreseeremos": "sobreseer",
    "sobreseerá": "sobreseer",
    "sobreseerán": "sobreseer",
    "sobreseerás": "sobreseer",
    "sobreseeré": "sobreseer",
    "sobreseeréis": "sobreseer",
    "sobreseería": "sobreseer",
    "sobreseeríais": "sobreseer",
    "sobreseeríamos": "sobreseer",
    "sobreseerían": "sobreseer",
    "sobreseerías": "sobreseer",
    "sobresees": "sobreseer",
    "sobreseguros": "sobreseguro",
    "sobreseimientos": "sobreseimiento",
    "sobresellos": "sobresello",
    "sobresembraba": "sobresembrar",
    "sobresembrabais": "sobresembrar",
    "sobresembraban": "sobresembrar",
    "sobresembrabas": "sobresembrar",
    "sobresembrad": "sobresembrar",
    "sobresembrada": "sobresembrar",
    "sobresembradas": "sobresembrar",
    "sobresembrado": "sobresembrar",
    "sobresembrados": "sobresembrar",
    "sobresembramos": "sobresembrar",
    "sobresembrando": "sobresembrar",
    "sobresembrara": "sobresembrar",
    "sobresembrarais": "sobresembrar",
    "sobresembraran": "sobresembrar",
    "sobresembraras": "sobresembrar",
    "sobresembrare": "sobresembrar",
    "sobresembrareis": "sobresembrar",
    "sobresembraremos": "sobresembrar",
    "sobresembraren": "sobresembrar",
    "sobresembrares": "sobresembrar",
    "sobresembraron": "sobresembrar",
    "sobresembrará": "sobresembrar",
    "sobresembrarán": "sobresembrar",
    "sobresembrarás": "sobresembrar",
    "sobresembraré": "sobresembrar",
    "sobresembraréis": "sobresembrar",
    "sobresembraría": "sobresembrar",
    "sobresembraríais": "sobresembrar",
    "sobresembraríamos": "sobresembrar",
    "sobresembrarían": "sobresembrar",
    "sobresembrarías": "sobresembrar",
    "sobresembrase": "sobresembrar",
    "sobresembraseis": "sobresembrar",
    "sobresembrasen": "sobresembrar",
    "sobresembrases": "sobresembrar",
    "sobresembraste": "sobresembrar",
    "sobresembrasteis": "sobresembrar",
    "sobresembremos": "sobresembrar",
    "sobresembrábamos": "sobresembrar",
    "sobresembráis": "sobresembrar",
    "sobresembráramos": "sobresembrar",
    "sobresembráremos": "sobresembrar",
    "sobresembrásemos": "sobresembrar",
    "sobresembré": "sobresembrar",
    "sobresembréis": "sobresembrar",
    "sobresembró": "sobresembrar",
    "sobreseo": "sobreseer",
    "sobreseyendo": "sobreseer",
    "sobreseyera": "sobreseer",
    "sobreseyerais": "sobreseer",
    "sobreseyeran": "sobreseer",
    "sobreseyeras": "sobreseer",
    "sobreseyere": "sobreseer",
    "sobreseyereis": "sobreseer",
    "sobreseyeren": "sobreseer",
    "sobreseyeres": "sobreseer",
    "sobreseyeron": "sobreseer",
    "sobreseyese": "sobreseer",
    "sobreseyeseis": "sobreseer",
    "sobreseyesen": "sobreseer",
    "sobreseyeses": "sobreseer",
    "sobreseyéramos": "sobreseer",
    "sobreseyéremos": "sobreseer",
    "sobreseyésemos": "sobreseer",
    "sobreseyó": "sobreseer",
    "sobreseáis": "sobreseer",
    "sobreseéis": "sobreseer",
    "sobreseí": "sobreseer",
    "sobreseía": "sobreseer",
    "sobreseíais": "sobreseer",
    "sobreseíamos": "sobreseer",
    "sobreseían": "sobreseer",
    "sobreseías": "sobreseer",
    "sobreseída": "sobreseer",
    "sobreseídas": "sobreseer",
    "sobreseído": "sobreseer",
    "sobreseídos": "sobreseer",
    "sobreseímos": "sobreseer",
    "sobreseíste": "sobreseer",
    "sobreseísteis": "sobreseer",
    "sobreseñales": "sobreseñal",
    "sobresiembra": "sobresembrar",
    "sobresiembran": "sobresembrar",
    "sobresiembras": "sobresembrar",
    "sobresiembre": "sobresembrar",
    "sobresiembren": "sobresembrar",
    "sobresiembres": "sobresembrar",
    "sobresiembro": "sobresembrar",
    "sobrestadías": "sobrestadía",
    "sobrestantes": "sobrestante",
    "sobrestantías": "sobrestantía",
    "sobrestima": "sobrestimar",
    "sobrestimaba": "sobrestimar",
    "sobrestimabais": "sobrestimar",
    "sobrestimaban": "sobrestimar",
    "sobrestimabas": "sobrestimar",
    "sobrestimad": "sobrestimar",
    "sobrestimada": "sobrestimar",
    "sobrestimadas": "sobrestimar",
    "sobrestimado": "sobrestimar",
    "sobrestimados": "sobrestimar",
    "sobrestimamos": "sobrestimar",
    "sobrestiman": "sobrestimar",
    "sobrestimando": "sobrestimar",
    "sobrestimara": "sobrestimar",
    "sobrestimarais": "sobrestimar",
    "sobrestimaran": "sobrestimar",
    "sobrestimaras": "sobrestimar",
    "sobrestimare": "sobrestimar",
    "sobrestimareis": "sobrestimar",
    "sobrestimaremos": "sobrestimar",
    "sobrestimaren": "sobrestimar",
    "sobrestimares": "sobrestimar",
    "sobrestimaron": "sobrestimar",
    "sobrestimará": "sobrestimar",
    "sobrestimarán": "sobrestimar",
    "sobrestimarás": "sobrestimar",
    "sobrestimaré": "sobrestimar",
    "sobrestimaréis": "sobrestimar",
    "sobrestimaría": "sobrestimar",
    "sobrestimaríais": "sobrestimar",
    "sobrestimaríamos": "sobrestimar",
    "sobrestimarían": "sobrestimar",
    "sobrestimarías": "sobrestimar",
    "sobrestimas": "sobrestimar",
    "sobrestimase": "sobrestimar",
    "sobrestimaseis": "sobrestimar",
    "sobrestimasen": "sobrestimar",
    "sobrestimases": "sobrestimar",
    "sobrestimaste": "sobrestimar",
    "sobrestimasteis": "sobrestimar",
    "sobrestime": "sobrestimar",
    "sobrestimemos": "sobrestimar",
    "sobrestimen": "sobrestimar",
    "sobrestimes": "sobrestimar",
    "sobrestimo": "sobrestimar",
    "sobrestimábamos": "sobrestimar",
    "sobrestimáis": "sobrestimar",
    "sobrestimáramos": "sobrestimar",
    "sobrestimáremos": "sobrestimar",
    "sobrestimásemos": "sobrestimar",
    "sobrestimé": "sobrestimar",
    "sobrestiméis": "sobrestimar",
    "sobrestimó": "sobrestimar",
    "sobresubvenciones": "sobresubvención",
    "sobresueldos": "sobresueldo",
    "sobresuelos": "sobresuelo",
    "sobresuscripciones": "sobresuscripción",
    "sobretardes": "sobretarde",
    "sobretasas": "sobretasa",
    "sobretendones": "sobretendón",
    "sobretensiones": "sobretensión",
    "sobretodos": "sobretodo",
    "sobretonos": "sobretono",
    "sobretítulos": "sobretítulo",
    "sobrevalora": "sobrevalorar",
    "sobrevaloraba": "sobrevalorar",
    "sobrevalorabais": "sobrevalorar",
    "sobrevaloraban": "sobrevalorar",
    "sobrevalorabas": "sobrevalorar",
    "sobrevalorad": "sobrevalorar",
    "sobrevalorada": "sobrevalorar",
    "sobrevaloradas": "sobrevalorar",
    "sobrevalorado": "sobrevalorar",
    "sobrevalorados": "sobrevalorar",
    "sobrevaloramos": "sobrevalorar",
    "sobrevaloran": "sobrevalorar",
    "sobrevalorando": "sobrevalorar",
    "sobrevalorara": "sobrevalorar",
    "sobrevalorarais": "sobrevalorar",
    "sobrevaloraran": "sobrevalorar",
    "sobrevaloraras": "sobrevalorar",
    "sobrevalorare": "sobrevalorar",
    "sobrevalorareis": "sobrevalorar",
    "sobrevaloraremos": "sobrevalorar",
    "sobrevaloraren": "sobrevalorar",
    "sobrevalorares": "sobrevalorar",
    "sobrevaloraron": "sobrevalorar",
    "sobrevalorará": "sobrevalorar",
    "sobrevalorarán": "sobrevalorar",
    "sobrevalorarás": "sobrevalorar",
    "sobrevaloraré": "sobrevalorar",
    "sobrevaloraréis": "sobrevalorar",
    "sobrevaloraría": "sobrevalorar",
    "sobrevaloraríais": "sobrevalorar",
    "sobrevaloraríamos": "sobrevalorar",
    "sobrevalorarían": "sobrevalorar",
    "sobrevalorarías": "sobrevalorar",
    "sobrevaloras": "sobrevalorar",
    "sobrevalorase": "sobrevalorar",
    "sobrevaloraseis": "sobrevalorar",
    "sobrevalorasen": "sobrevalorar",
    "sobrevalorases": "sobrevalorar",
    "sobrevaloraste": "sobrevalorar",
    "sobrevalorasteis": "sobrevalorar",
    "sobrevalore": "sobrevalorar",
    "sobrevaloremos": "sobrevalorar",
    "sobrevaloren": "sobrevalorar",
    "sobrevalores": "sobrevalorar",
    "sobrevaloro": "sobrevalorar",
    "sobrevalorábamos": "sobrevalorar",
    "sobrevaloráis": "sobrevalorar",
    "sobrevaloráramos": "sobrevalorar",
    "sobrevaloráremos": "sobrevalorar",
    "sobrevalorásemos": "sobrevalorar",
    "sobrevaloré": "sobrevalorar",
    "sobrevaloréis": "sobrevalorar",
    "sobrevaloró": "sobrevalorar",
    "sobrevelocidades": "sobrevelocidad",
    "sobreven": "sobrevenir",
    "sobrevendremos": "sobrevenir",
    "sobrevendrá": "sobrevenir",
    "sobrevendrán": "sobrevenir",
    "sobrevendrás": "sobrevenir",
    "sobrevendré": "sobrevenir",
    "sobrevendréis": "sobrevenir",
    "sobrevendría": "sobrevenir",
    "sobrevendríais": "sobrevenir",
    "sobrevendríamos": "sobrevenir",
    "sobrevendrían": "sobrevenir",
    "sobrevendrías": "sobrevenir",
    "sobrevenga": "sobrevenir",
    "sobrevengamos": "sobrevenir",
    "sobrevengan": "sobrevenir",
    "sobrevengas": "sobrevenir",
    "sobrevengo": "sobrevenir",
    "sobrevengáis": "sobrevenir",
    "sobrevenid": "sobrevenir",
    "sobrevenida": "sobrevenir",
    "sobrevenidas": "sobrevenir",
    "sobrevenido": "sobrevenir",
    "sobrevenidos": "sobrevenir",
    "sobrevenimos": "sobrevenir",
    "sobreventas": "sobreventa",
    "sobrevenía": "sobrevenir",
    "sobreveníais": "sobrevenir",
    "sobreveníamos": "sobrevenir",
    "sobrevenían": "sobrevenir",
    "sobrevenías": "sobrevenir",
    "sobrevenís": "sobrevenir",
    "sobrevidrieras": "sobrevidriera",
    "sobreviene": "sobrevenir",
    "sobrevienen": "sobrevenir",
    "sobrevienes": "sobrevenir",
    "sobrevientas": "sobrevienta",
    "sobrevientos": "sobreviento",
    "sobrevine": "sobrevenir",
    "sobreviniendo": "sobrevenir",
    "sobreviniera": "sobrevenir",
    "sobrevinierais": "sobrevenir",
    "sobrevinieran": "sobrevenir",
    "sobrevinieras": "sobrevenir",
    "sobreviniere": "sobrevenir",
    "sobreviniereis": "sobrevenir",
    "sobrevinieren": "sobrevenir",
    "sobrevinieres": "sobrevenir",
    "sobrevinieron": "sobrevenir",
    "sobreviniese": "sobrevenir",
    "sobrevinieseis": "sobrevenir",
    "sobreviniesen": "sobrevenir",
    "sobrevinieses": "sobrevenir",
    "sobrevinimos": "sobrevenir",
    "sobreviniste": "sobrevenir",
    "sobrevinisteis": "sobrevenir",
    "sobreviniéramos": "sobrevenir",
    "sobreviniéremos": "sobrevenir",
    "sobreviniésemos": "sobrevenir",
    "sobrevino": "sobrevenir",
    "sobrevistas": "sobrevista",
    "sobreviva": "sobrevivir",
    "sobrevivamos": "sobrevivir",
    "sobrevivan": "sobrevivir",
    "sobrevivas": "sobrevivir",
    "sobrevive": "sobrevivir",
    "sobreviven": "sobrevivir",
    "sobrevivencias": "sobrevivencia",
    "sobrevives": "sobrevivir",
    "sobrevivid": "sobrevivir",
    "sobrevivida": "sobrevivir",
    "sobrevividas": "sobrevivir",
    "sobrevivido": "sobrevivir",
    "sobrevividos": "sobrevivir",
    "sobreviviendo": "sobrevivir",
    "sobrevivientes": "sobreviviente",
    "sobreviviera": "sobrevivir",
    "sobrevivierais": "sobrevivir",
    "sobrevivieran": "sobrevivir",
    "sobrevivieras": "sobrevivir",
    "sobreviviere": "sobrevivir",
    "sobreviviereis": "sobrevivir",
    "sobrevivieren": "sobrevivir",
    "sobrevivieres": "sobrevivir",
    "sobrevivieron": "sobrevivir",
    "sobreviviese": "sobrevivir",
    "sobrevivieseis": "sobrevivir",
    "sobreviviesen": "sobrevivir",
    "sobrevivieses": "sobrevivir",
    "sobrevivimos": "sobrevivir",
    "sobreviviremos": "sobrevivir",
    "sobrevivirá": "sobrevivir",
    "sobrevivirán": "sobrevivir",
    "sobrevivirás": "sobrevivir",
    "sobreviviré": "sobrevivir",
    "sobreviviréis": "sobrevivir",
    "sobreviviría": "sobrevivir",
    "sobreviviríais": "sobrevivir",
    "sobreviviríamos": "sobrevivir",
    "sobrevivirían": "sobrevivir",
    "sobrevivirías": "sobrevivir",
    "sobreviviste": "sobrevivir",
    "sobrevivisteis": "sobrevivir",
    "sobreviviéramos": "sobrevivir",
    "sobreviviéremos": "sobrevivir",
    "sobreviviésemos": "sobrevivir",
    "sobrevivió": "sobrevivir",
    "sobrevivo": "sobrevivir",
    "sobreviváis": "sobrevivir",
    "sobreviví": "sobrevivir",
    "sobrevivía": "sobrevivir",
    "sobrevivíais": "sobrevivir",
    "sobrevivíamos": "sobrevivir",
    "sobrevivían": "sobrevivir",
    "sobrevivías": "sobrevivir",
    "sobrevivís": "sobrevivir",
    "sobrevolaba": "sobrevolar",
    "sobrevolabais": "sobrevolar",
    "sobrevolaban": "sobrevolar",
    "sobrevolabas": "sobrevolar",
    "sobrevolad": "sobrevolar",
    "sobrevolada": "sobrevolar",
    "sobrevoladas": "sobrevolar",
    "sobrevolado": "sobrevolar",
    "sobrevolados": "sobrevolar",
    "sobrevolamos": "sobrevolar",
    "sobrevolando": "sobrevolar",
    "sobrevolara": "sobrevolar",
    "sobrevolarais": "sobrevolar",
    "sobrevolaran": "sobrevolar",
    "sobrevolaras": "sobrevolar",
    "sobrevolare": "sobrevolar",
    "sobrevolareis": "sobrevolar",
    "sobrevolaremos": "sobrevolar",
    "sobrevolaren": "sobrevolar",
    "sobrevolares": "sobrevolar",
    "sobrevolaron": "sobrevolar",
    "sobrevolará": "sobrevolar",
    "sobrevolarán": "sobrevolar",
    "sobrevolarás": "sobrevolar",
    "sobrevolaré": "sobrevolar",
    "sobrevolaréis": "sobrevolar",
    "sobrevolaría": "sobrevolar",
    "sobrevolaríais": "sobrevolar",
    "sobrevolaríamos": "sobrevolar",
    "sobrevolarían": "sobrevolar",
    "sobrevolarías": "sobrevolar",
    "sobrevolase": "sobrevolar",
    "sobrevolaseis": "sobrevolar",
    "sobrevolasen": "sobrevolar",
    "sobrevolases": "sobrevolar",
    "sobrevolaste": "sobrevolar",
    "sobrevolasteis": "sobrevolar",
    "sobrevolemos": "sobrevolar",
    "sobrevolábamos": "sobrevolar",
    "sobrevoláis": "sobrevolar",
    "sobrevoláramos": "sobrevolar",
    "sobrevoláremos": "sobrevolar",
    "sobrevolásemos": "sobrevolar",
    "sobrevolé": "sobrevolar",
    "sobrevoléis": "sobrevolar",
    "sobrevoló": "sobrevolar",
    "sobrevuela": "sobrevolar",
    "sobrevuelan": "sobrevolar",
    "sobrevuelas": "sobrevolar",
    "sobrevuele": "sobrevolar",
    "sobrevuelen": "sobrevolar",
    "sobrevueles": "sobrevolar",
    "sobrevuelo": "sobrevolar",
    "sobrexceda": "sobrexceder",
    "sobrexcedamos": "sobrexceder",
    "sobrexcedan": "sobrexceder",
    "sobrexcedas": "sobrexceder",
    "sobrexcede": "sobrexceder",
    "sobrexceded": "sobrexceder",
    "sobrexcedemos": "sobrexceder",
    "sobrexceden": "sobrexceder",
    "sobrexcederemos": "sobrexceder",
    "sobrexcederá": "sobrexceder",
    "sobrexcederán": "sobrexceder",
    "sobrexcederás": "sobrexceder",
    "sobrexcederé": "sobrexceder",
    "sobrexcederéis": "sobrexceder",
    "sobrexcedería": "sobrexceder",
    "sobrexcederíais": "sobrexceder",
    "sobrexcederíamos": "sobrexceder",
    "sobrexcederían": "sobrexceder",
    "sobrexcederías": "sobrexceder",
    "sobrexcedes": "sobrexceder",
    "sobrexcedida": "sobrexceder",
    "sobrexcedidas": "sobrexceder",
    "sobrexcedido": "sobrexceder",
    "sobrexcedidos": "sobrexceder",
    "sobrexcediendo": "sobrexceder",
    "sobrexcediera": "sobrexceder",
    "sobrexcedierais": "sobrexceder",
    "sobrexcedieran": "sobrexceder",
    "sobrexcedieras": "sobrexceder",
    "sobrexcediere": "sobrexceder",
    "sobrexcediereis": "sobrexceder",
    "sobrexcedieren": "sobrexceder",
    "sobrexcedieres": "sobrexceder",
    "sobrexcedieron": "sobrexceder",
    "sobrexcediese": "sobrexceder",
    "sobrexcedieseis": "sobrexceder",
    "sobrexcediesen": "sobrexceder",
    "sobrexcedieses": "sobrexceder",
    "sobrexcedimos": "sobrexceder",
    "sobrexcediste": "sobrexceder",
    "sobrexcedisteis": "sobrexceder",
    "sobrexcediéramos": "sobrexceder",
    "sobrexcediéremos": "sobrexceder",
    "sobrexcediésemos": "sobrexceder",
    "sobrexcedió": "sobrexceder",
    "sobrexcedo": "sobrexceder",
    "sobrexcedáis": "sobrexceder",
    "sobrexcedéis": "sobrexceder",
    "sobrexcedí": "sobrexceder",
    "sobrexcedía": "sobrexceder",
    "sobrexcedíais": "sobrexceder",
    "sobrexcedíamos": "sobrexceder",
    "sobrexcedían": "sobrexceder",
    "sobrexcedías": "sobrexceder",
    "sobrexcita": "sobrexcitar",
    "sobrexcitaba": "sobrexcitar",
    "sobrexcitabais": "sobrexcitar",
    "sobrexcitaban": "sobrexcitar",
    "sobrexcitabas": "sobrexcitar",
    "sobrexcitad": "sobrexcitar",
    "sobrexcitada": "sobrexcitar",
    "sobrexcitadas": "sobrexcitar",
    "sobrexcitado": "sobrexcitar",
    "sobrexcitados": "sobrexcitar",
    "sobrexcitamos": "sobrexcitar",
    "sobrexcitan": "sobrexcitar",
    "sobrexcitando": "sobrexcitar",
    "sobrexcitara": "sobrexcitar",
    "sobrexcitarais": "sobrexcitar",
    "sobrexcitaran": "sobrexcitar",
    "sobrexcitaras": "sobrexcitar",
    "sobrexcitare": "sobrexcitar",
    "sobrexcitareis": "sobrexcitar",
    "sobrexcitaremos": "sobrexcitar",
    "sobrexcitaren": "sobrexcitar",
    "sobrexcitares": "sobrexcitar",
    "sobrexcitaron": "sobrexcitar",
    "sobrexcitará": "sobrexcitar",
    "sobrexcitarán": "sobrexcitar",
    "sobrexcitarás": "sobrexcitar",
    "sobrexcitaré": "sobrexcitar",
    "sobrexcitaréis": "sobrexcitar",
    "sobrexcitaría": "sobrexcitar",
    "sobrexcitaríais": "sobrexcitar",
    "sobrexcitaríamos": "sobrexcitar",
    "sobrexcitarían": "sobrexcitar",
    "sobrexcitarías": "sobrexcitar",
    "sobrexcitas": "sobrexcitar",
    "sobrexcitase": "sobrexcitar",
    "sobrexcitaseis": "sobrexcitar",
    "sobrexcitasen": "sobrexcitar",
    "sobrexcitases": "sobrexcitar",
    "sobrexcitaste": "sobrexcitar",
    "sobrexcitasteis": "sobrexcitar",
    "sobrexcite": "sobrexcitar",
    "sobrexcitemos": "sobrexcitar",
    "sobrexciten": "sobrexcitar",
    "sobrexcites": "sobrexcitar",
    "sobrexcito": "sobrexcitar",
    "sobrexcitábamos": "sobrexcitar",
    "sobrexcitáis": "sobrexcitar",
    "sobrexcitáramos": "sobrexcitar",
    "sobrexcitáremos": "sobrexcitar",
    "sobrexcitásemos": "sobrexcitar",
    "sobrexcité": "sobrexcitar",
    "sobrexcitéis": "sobrexcitar",
    "sobrexcitó": "sobrexcitar",
    "sobreáticos": "sobreático",
    "sobria": "sobrio",
    "sobrias": "sobrio",
    "sobriedades": "sobriedad",
    "sobrina": "sobrino",
    "sobrinas": "sobrino",
    "sobrinazgos": "sobrinazgo",
    "sobrinos": "sobrino",
    "sobrios": "sobrio",
    "sobro": "sobrar",
    "sobrábamos": "sobrar",
    "sobráis": "sobrar",
    "sobráramos": "sobrar",
    "sobráremos": "sobrar",
    "sobrásemos": "sobrar",
    "sobré": "sobrar",
    "sobréis": "sobrar",
    "sobró": "sobrar",
    "sobábamos": "sobar",
    "sobáis": "sobar",
    "sobáramos": "sobar",
    "sobáremos": "sobar",
    "sobásemos": "sobar",
    "sobé": "sobar",
    "sobéis": "sobar",
    "sobó": "sobar",
    "socaces": "socaz",
    "socaireros": "socairero",
    "socaires": "socaire",
    "socalce": "socalzar",
    "socalcemos": "socalzar",
    "socalcen": "socalzar",
    "socalces": "socalzar",
    "socalcé": "socalzar",
    "socalcéis": "socalzar",
    "socaliña": "socaliñar",
    "socaliñaba": "socaliñar",
    "socaliñabais": "socaliñar",
    "socaliñaban": "socaliñar",
    "socaliñabas": "socaliñar",
    "socaliñad": "socaliñar",
    "socaliñada": "socaliñar",
    "socaliñadas": "socaliñar",
    "socaliñado": "socaliñar",
    "socaliñados": "socaliñar",
    "socaliñamos": "socaliñar",
    "socaliñan": "socaliñar",
    "socaliñando": "socaliñar",
    "socaliñara": "socaliñar",
    "socaliñarais": "socaliñar",
    "socaliñaran": "socaliñar",
    "socaliñaras": "socaliñar",
    "socaliñare": "socaliñar",
    "socaliñareis": "socaliñar",
    "socaliñaremos": "socaliñar",
    "socaliñaren": "socaliñar",
    "socaliñares": "socaliñar",
    "socaliñaron": "socaliñar",
    "socaliñará": "socaliñar",
    "socaliñarán": "socaliñar",
    "socaliñarás": "socaliñar",
    "socaliñaré": "socaliñar",
    "socaliñaréis": "socaliñar",
    "socaliñaría": "socaliñar",
    "socaliñaríais": "socaliñar",
    "socaliñaríamos": "socaliñar",
    "socaliñarían": "socaliñar",
    "socaliñarías": "socaliñar",
    "socaliñas": "socaliñar",
    "socaliñase": "socaliñar",
    "socaliñaseis": "socaliñar",
    "socaliñasen": "socaliñar",
    "socaliñases": "socaliñar",
    "socaliñaste": "socaliñar",
    "socaliñasteis": "socaliñar",
    "socaliñe": "socaliñar",
    "socaliñemos": "socaliñar",
    "socaliñen": "socaliñar",
    "socaliñera": "socaliñero",
    "socaliñeras": "socaliñero",
    "socaliñeros": "socaliñero",
    "socaliñes": "socaliñar",
    "socaliño": "socaliñar",
    "socaliñábamos": "socaliñar",
    "socaliñáis": "socaliñar",
    "socaliñáramos": "socaliñar",
    "socaliñáremos": "socaliñar",
    "socaliñásemos": "socaliñar",
    "socaliñé": "socaliñar",
    "socaliñéis": "socaliñar",
    "socaliñó": "socaliñar",
    "socalza": "socalzar",
    "socalzaba": "socalzar",
    "socalzabais": "socalzar",
    "socalzaban": "socalzar",
    "socalzabas": "socalzar",
    "socalzad": "socalzar",
    "socalzada": "socalzar",
    "socalzadas": "socalzar",
    "socalzado": "socalzar",
    "socalzados": "socalzar",
    "socalzamos": "socalzar",
    "socalzan": "socalzar",
    "socalzando": "socalzar",
    "socalzara": "socalzar",
    "socalzarais": "socalzar",
    "socalzaran": "socalzar",
    "socalzaras": "socalzar",
    "socalzare": "socalzar",
    "socalzareis": "socalzar",
    "socalzaremos": "socalzar",
    "socalzaren": "socalzar",
    "socalzares": "socalzar",
    "socalzaron": "socalzar",
    "socalzará": "socalzar",
    "socalzarán": "socalzar",
    "socalzarás": "socalzar",
    "socalzaré": "socalzar",
    "socalzaréis": "socalzar",
    "socalzaría": "socalzar",
    "socalzaríais": "socalzar",
    "socalzaríamos": "socalzar",
    "socalzarían": "socalzar",
    "socalzarías": "socalzar",
    "socalzas": "socalzar",
    "socalzase": "socalzar",
    "socalzaseis": "socalzar",
    "socalzasen": "socalzar",
    "socalzases": "socalzar",
    "socalzaste": "socalzar",
    "socalzasteis": "socalzar",
    "socalzo": "socalzar",
    "socalzábamos": "socalzar",
    "socalzáis": "socalzar",
    "socalzáramos": "socalzar",
    "socalzáremos": "socalzar",
    "socalzásemos": "socalzar",
    "socalzó": "socalzar",
    "socapa": "socapar",
    "socapaba": "socapar",
    "socapabais": "socapar",
    "socapaban": "socapar",
    "socapabas": "socapar",
    "socapad": "socapar",
    "socapada": "socapar",
    "socapadas": "socapar",
    "socapado": "socapar",
    "socapados": "socapar",
    "socapamos": "socapar",
    "socapan": "socapar",
    "socapando": "socapar",
    "socapara": "socapar",
    "socaparais": "socapar",
    "socaparan": "socapar",
    "socaparas": "socapar",
    "socapare": "socapar",
    "socapareis": "socapar",
    "socaparemos": "socapar",
    "socaparen": "socapar",
    "socapares": "socapar",
    "socaparon": "socapar",
    "socapará": "socapar",
    "socaparán": "socapar",
    "socaparás": "socapar",
    "socaparé": "socapar",
    "socaparéis": "socapar",
    "socaparía": "socapar",
    "socaparíais": "socapar",
    "socaparíamos": "socapar",
    "socaparían": "socapar",
    "socaparías": "socapar",
    "socapas": "socapar",
    "socapase": "socapar",
    "socapaseis": "socapar",
    "socapasen": "socapar",
    "socapases": "socapar",
    "socapaste": "socapar",
    "socapasteis": "socapar",
    "socape": "socapar",
    "socapemos": "socapar",
    "socapen": "socapar",
    "socapes": "socapar",
    "socapo": "socapar",
    "socapábamos": "socapar",
    "socapáis": "socapar",
    "socapáramos": "socapar",
    "socapáremos": "socapar",
    "socapásemos": "socapar",
    "socapé": "socapar",
    "socapéis": "socapar",
    "socapó": "socapar",
    "socarra": "socarrar",
    "socarraba": "socarrar",
    "socarrabais": "socarrar",
    "socarraban": "socarrar",
    "socarrabas": "socarrar",
    "socarrad": "socarrar",
    "socarrada": "socarrar",
    "socarradas": "socarrar",
    "socarrado": "socarrar",
    "socarrados": "socarrar",
    "socarramos": "socarrar",
    "socarran": "socarrar",
    "socarrando": "socarrar",
    "socarrara": "socarrar",
    "socarrarais": "socarrar",
    "socarraran": "socarrar",
    "socarraras": "socarrar",
    "socarrare": "socarrar",
    "socarrareis": "socarrar",
    "socarraremos": "socarrar",
    "socarraren": "socarrar",
    "socarrares": "socarrar",
    "socarraron": "socarrar",
    "socarrará": "socarrar",
    "socarrarán": "socarrar",
    "socarrarás": "socarrar",
    "socarraré": "socarrar",
    "socarraréis": "socarrar",
    "socarraría": "socarrar",
    "socarraríais": "socarrar",
    "socarraríamos": "socarrar",
    "socarrarían": "socarrar",
    "socarrarías": "socarrar",
    "socarras": "socarrar",
    "socarrase": "socarrar",
    "socarraseis": "socarrar",
    "socarrasen": "socarrar",
    "socarrases": "socarrar",
    "socarraste": "socarrar",
    "socarrasteis": "socarrar",
    "socarre": "socarrar",
    "socarremos": "socarrar",
    "socarren": "socarrar",
    "socarrenas": "socarrena",
    "socarrenes": "socarrén",
    "socarres": "socarrar",
    "socarrinas": "socarrina",
    "socarro": "socarrar",
    "socarrona": "socarrón",
    "socarronas": "socarrón",
    "socarronerías": "socarronería",
    "socarrones": "socarrón",
    "socarrábamos": "socarrar",
    "socarráis": "socarrar",
    "socarráramos": "socarrar",
    "socarráremos": "socarrar",
    "socarrásemos": "socarrar",
    "socarré": "socarrar",
    "socarréis": "socarrar",
    "socarró": "socarrar",
    "socas": "soca",
    "socava": "socavar",
    "socavaba": "socavar",
    "socavabais": "socavar",
    "socavaban": "socavar",
    "socavabas": "socavar",
    "socavaciones": "socavación",
    "socavad": "socavar",
    "socavada": "socavar",
    "socavadas": "socavar",
    "socavado": "socavar",
    "socavados": "socavar",
    "socavamos": "socavar",
    "socavan": "socavar",
    "socavando": "socavar",
    "socavara": "socavar",
    "socavarais": "socavar",
    "socavaran": "socavar",
    "socavaras": "socavar",
    "socavare": "socavar",
    "socavareis": "socavar",
    "socavaremos": "socavar",
    "socavaren": "socavar",
    "socavares": "socavar",
    "socavaron": "socavar",
    "socavará": "socavar",
    "socavarán": "socavar",
    "socavarás": "socavar",
    "socavaré": "socavar",
    "socavaréis": "socavar",
    "socavaría": "socavar",
    "socavaríais": "socavar",
    "socavaríamos": "socavar",
    "socavarían": "socavar",
    "socavarías": "socavar",
    "socavas": "socavar",
    "socavase": "socavar",
    "socavaseis": "socavar",
    "socavasen": "socavar",
    "socavases": "socavar",
    "socavaste": "socavar",
    "socavasteis": "socavar",
    "socave": "socavar",
    "socavemos": "socavar",
    "socaven": "socavar",
    "socaves": "socavar",
    "socavo": "socavar",
    "socavones": "socavón",
    "socavábamos": "socavar",
    "socaváis": "socavar",
    "socaváramos": "socavar",
    "socaváremos": "socavar",
    "socavásemos": "socavar",
    "socavé": "socavar",
    "socavéis": "socavar",
    "socavó": "socavar",
    "sochantres": "sochantre",
    "socia": "socio",
    "sociabilidades": "sociabilidad",
    "sociables": "sociable",
    "socialcristiana": "socialcristiano",
    "socialcristianas": "socialcristiano",
    "socialcristianos": "socialcristiano",
    "socialdemocracias": "socialdemocracia",
    "socialdemocrática": "socialdemocrático",
    "socialdemocráticas": "socialdemocrático",
    "socialdemocráticos": "socialdemocrático",
    "socialdemócratas": "socialdemócrata",
    "sociales": "social",
    "socialice": "socializar",
    "socialicemos": "socializar",
    "socialicen": "socializar",
    "socialices": "socializar",
    "socialicé": "socializar",
    "socialicéis": "socializar",
    "socialismos": "socialismo",
    "socialistas": "socialista",
    "socializa": "socializar",
    "socializaba": "socializar",
    "socializabais": "socializar",
    "socializaban": "socializar",
    "socializabas": "socializar",
    "socializaciones": "socialización",
    "socializad": "socializar",
    "socializada": "socializar",
    "socializadas": "socializar",
    "socializado": "socializar",
    "socializadora": "socializador",
    "socializadoras": "socializador",
    "socializadores": "socializador",
    "socializados": "socializar",
    "socializamos": "socializar",
    "socializan": "socializar",
    "socializando": "socializar",
    "socializantes": "socializante",
    "socializara": "socializar",
    "socializarais": "socializar",
    "socializaran": "socializar",
    "socializaras": "socializar",
    "socializare": "socializar",
    "socializareis": "socializar",
    "socializaremos": "socializar",
    "socializaren": "socializar",
    "socializares": "socializar",
    "socializaron": "socializar",
    "socializará": "socializar",
    "socializarán": "socializar",
    "socializarás": "socializar",
    "socializaré": "socializar",
    "socializaréis": "socializar",
    "socializaría": "socializar",
    "socializaríais": "socializar",
    "socializaríamos": "socializar",
    "socializarían": "socializar",
    "socializarías": "socializar",
    "socializas": "socializar",
    "socializase": "socializar",
    "socializaseis": "socializar",
    "socializasen": "socializar",
    "socializases": "socializar",
    "socializaste": "socializar",
    "socializasteis": "socializar",
    "socializo": "socializar",
    "socializábamos": "socializar",
    "socializáis": "socializar",
    "socializáramos": "socializar",
    "socializáremos": "socializar",
    "socializásemos": "socializar",
    "socializó": "socializar",
    "socias": "socio",
    "sociedades": "sociedad",
    "societaria": "societario",
    "societarias": "societario",
    "societarios": "societario",
    "societarismos": "societarismo",
    "sociniana": "sociniano",
    "socinianas": "sociniano",
    "socinianismos": "socinianismo",
    "socinianos": "sociniano",
    "sociobiologías": "sociobiología",
    "socioculturales": "sociocultural",
    "sociodramas": "sociodrama",
    "socioeconómica": "socioeconómico",
    "socioeconómicas": "socioeconómico",
    "socioeconómicos": "socioeconómico",
    "sociografías": "sociografía",
    "sociogramas": "sociograma",
    "sociolaborales": "sociolaboral",
    "sociolingüistas": "sociolingüista",
    "sociolingüística": "sociolingüístico",
    "sociolingüísticas": "sociolingüístico",
    "sociolingüísticos": "sociolingüístico",
    "sociologismos": "sociologismo",
    "sociologías": "sociología",
    "sociológica": "sociológico",
    "sociológicas": "sociológico",
    "sociológicos": "sociológico",
    "sociometrías": "sociometría",
    "sociométrica": "sociométrico",
    "sociométricas": "sociométrico",
    "sociométricos": "sociométrico",
    "sociopolítica": "sociopolítico",
    "sociopolíticas": "sociopolítico",
    "sociopolíticos": "sociopolítico",
    "socioprofesionales": "socioprofesional",
    "sociopática": "sociopático",
    "sociopáticas": "sociopático",
    "sociopáticos": "sociopático",
    "socios": "socio",
    "sociosanitaria": "sociosanitario",
    "sociosanitarias": "sociosanitario",
    "sociosanitarios": "sociosanitario",
    "socióloga": "sociólogo",
    "sociólogas": "sociólogo",
    "sociólogos": "sociólogo",
    "socolladas": "socollada",
    "socollares": "socollar",
    "socolores": "socolor",
    "socoros": "socoro",
    "socorra": "socorrer",
    "socorramos": "socorrer",
    "socorran": "socorrer",
    "socorras": "socorrer",
    "socorre": "socorrer",
    "socorred": "socorrer",
    "socorredora": "socorredor",
    "socorredoras": "socorredor",
    "socorredores": "socorredor",
    "socorremos": "socorrer",
    "socorren": "socorrer",
    "socorreremos": "socorrer",
    "socorrerá": "socorrer",
    "socorrerán": "socorrer",
    "socorrerás": "socorrer",
    "socorreré": "socorrer",
    "socorreréis": "socorrer",
    "socorrería": "socorrer",
    "socorreríais": "socorrer",
    "socorreríamos": "socorrer",
    "socorrerían": "socorrer",
    "socorrerías": "socorrer",
    "socorres": "socorrer",
    "socorrida": "socorrer",
    "socorridas": "socorrer",
    "socorrido": "socorrer",
    "socorridos": "socorrer",
    "socorriendo": "socorrer",
    "socorriera": "socorrer",
    "socorrierais": "socorrer",
    "socorrieran": "socorrer",
    "socorrieras": "socorrer",
    "socorriere": "socorrer",
    "socorriereis": "socorrer",
    "socorrieren": "socorrer",
    "socorrieres": "socorrer",
    "socorrieron": "socorrer",
    "socorriese": "socorrer",
    "socorrieseis": "socorrer",
    "socorriesen": "socorrer",
    "socorrieses": "socorrer",
    "socorrimos": "socorrer",
    "socorrismos": "socorrismo",
    "socorristas": "socorrista",
    "socorriste": "socorrer",
    "socorristeis": "socorrer",
    "socorriéramos": "socorrer",
    "socorriéremos": "socorrer",
    "socorriésemos": "socorrer",
    "socorrió": "socorrer",
    "socorro": "socorrer",
    "socorros": "socorro",
    "socorráis": "socorrer",
    "socorréis": "socorrer",
    "socorrí": "socorrer",
    "socorría": "socorrer",
    "socorríais": "socorrer",
    "socorríamos": "socorrer",
    "socorrían": "socorrer",
    "socorrías": "socorrer",
    "socrocios": "socrocio",
    "socrática": "socrático",
    "socráticas": "socrático",
    "socráticos": "socrático",
    "socópteros": "socóptero",
    "sodalitas": "sodalita",
    "sodas": "soda",
    "sodificaciones": "sodificación",
    "sodioacetatos": "sodioacetato",
    "sodios": "sodio",
    "sodokus": "sodoku",
    "sodomitas": "sodomita",
    "sodomías": "sodomía",
    "sodomítica": "sodomítico",
    "sodomíticas": "sodomítico",
    "sodomíticos": "sodomítico",
    "soeces": "soez",
    "soezes": "soez",
    "sofalda": "sofaldar",
    "sofaldaba": "sofaldar",
    "sofaldabais": "sofaldar",
    "sofaldaban": "sofaldar",
    "sofaldabas": "sofaldar",
    "sofaldad": "sofaldar",
    "sofaldada": "sofaldar",
    "sofaldadas": "sofaldar",
    "sofaldado": "sofaldar",
    "sofaldados": "sofaldar",
    "sofaldamos": "sofaldar",
    "sofaldan": "sofaldar",
    "sofaldando": "sofaldar",
    "sofaldara": "sofaldar",
    "sofaldarais": "sofaldar",
    "sofaldaran": "sofaldar",
    "sofaldaras": "sofaldar",
    "sofaldare": "sofaldar",
    "sofaldareis": "sofaldar",
    "sofaldaremos": "sofaldar",
    "sofaldaren": "sofaldar",
    "sofaldares": "sofaldar",
    "sofaldaron": "sofaldar",
    "sofaldará": "sofaldar",
    "sofaldarán": "sofaldar",
    "sofaldarás": "sofaldar",
    "sofaldaré": "sofaldar",
    "sofaldaréis": "sofaldar",
    "sofaldaría": "sofaldar",
    "sofaldaríais": "sofaldar",
    "sofaldaríamos": "sofaldar",
    "sofaldarían": "sofaldar",
    "sofaldarías": "sofaldar",
    "sofaldas": "sofaldar",
    "sofaldase": "sofaldar",
    "sofaldaseis": "sofaldar",
    "sofaldasen": "sofaldar",
    "sofaldases": "sofaldar",
    "sofaldaste": "sofaldar",
    "sofaldasteis": "sofaldar",
    "sofalde": "sofaldar",
    "sofaldemos": "sofaldar",
    "sofalden": "sofaldar",
    "sofaldes": "sofaldar",
    "sofaldo": "sofaldar",
    "sofaldos": "sofaldo",
    "sofaldábamos": "sofaldar",
    "sofaldáis": "sofaldar",
    "sofaldáramos": "sofaldar",
    "sofaldáremos": "sofaldar",
    "sofaldásemos": "sofaldar",
    "sofaldé": "sofaldar",
    "sofaldéis": "sofaldar",
    "sofaldó": "sofaldar",
    "sofiana": "sofiano",
    "sofianas": "sofiano",
    "sofianos": "sofiano",
    "sofiones": "sofión",
    "sofismas": "sofisma",
    "sofismos": "sofismo",
    "sofistas": "sofista",
    "sofisterías": "sofistería",
    "sofistica": "sofisticar",
    "sofisticaba": "sofisticar",
    "sofisticabais": "sofisticar",
    "sofisticaban": "sofisticar",
    "sofisticabas": "sofisticar",
    "sofisticaciones": "sofisticación",
    "sofisticad": "sofisticar",
    "sofisticada": "sofisticar",
    "sofisticadas": "sofisticar",
    "sofisticado": "sofisticar",
    "sofisticados": "sofisticar",
    "sofisticamos": "sofisticar",
    "sofistican": "sofisticar",
    "sofisticando": "sofisticar",
    "sofisticara": "sofisticar",
    "sofisticarais": "sofisticar",
    "sofisticaran": "sofisticar",
    "sofisticaras": "sofisticar",
    "sofisticare": "sofisticar",
    "sofisticareis": "sofisticar",
    "sofisticaremos": "sofisticar",
    "sofisticaren": "sofisticar",
    "sofisticares": "sofisticar",
    "sofisticaron": "sofisticar",
    "sofisticará": "sofisticar",
    "sofisticarán": "sofisticar",
    "sofisticarás": "sofisticar",
    "sofisticaré": "sofisticar",
    "sofisticaréis": "sofisticar",
    "sofisticaría": "sofisticar",
    "sofisticaríais": "sofisticar",
    "sofisticaríamos": "sofisticar",
    "sofisticarían": "sofisticar",
    "sofisticarías": "sofisticar",
    "sofisticas": "sofisticar",
    "sofisticase": "sofisticar",
    "sofisticaseis": "sofisticar",
    "sofisticasen": "sofisticar",
    "sofisticases": "sofisticar",
    "sofisticaste": "sofisticar",
    "sofisticasteis": "sofisticar",
    "sofistico": "sofisticar",
    "sofisticábamos": "sofisticar",
    "sofisticáis": "sofisticar",
    "sofisticáramos": "sofisticar",
    "sofisticáremos": "sofisticar",
    "sofisticásemos": "sofisticar",
    "sofisticó": "sofisticar",
    "sofistique": "sofisticar",
    "sofistiqueces": "sofistiquez",
    "sofistiquemos": "sofisticar",
    "sofistiquen": "sofisticar",
    "sofistiques": "sofisticar",
    "sofistiqué": "sofisticar",
    "sofistiquéis": "sofisticar",
    "sofitos": "sofito",
    "soflama": "soflamar",
    "soflamaba": "soflamar",
    "soflamabais": "soflamar",
    "soflamaban": "soflamar",
    "soflamabas": "soflamar",
    "soflamad": "soflamar",
    "soflamada": "soflamar",
    "soflamadas": "soflamar",
    "soflamado": "soflamar",
    "soflamados": "soflamar",
    "soflamamos": "soflamar",
    "soflaman": "soflamar",
    "soflamando": "soflamar",
    "soflamara": "soflamar",
    "soflamarais": "soflamar",
    "soflamaran": "soflamar",
    "soflamaras": "soflamar",
    "soflamare": "soflamar",
    "soflamareis": "soflamar",
    "soflamaremos": "soflamar",
    "soflamaren": "soflamar",
    "soflamares": "soflamar",
    "soflamaron": "soflamar",
    "soflamará": "soflamar",
    "soflamarán": "soflamar",
    "soflamarás": "soflamar",
    "soflamaré": "soflamar",
    "soflamaréis": "soflamar",
    "soflamaría": "soflamar",
    "soflamaríais": "soflamar",
    "soflamaríamos": "soflamar",
    "soflamarían": "soflamar",
    "soflamarías": "soflamar",
    "soflamas": "soflamar",
    "soflamase": "soflamar",
    "soflamaseis": "soflamar",
    "soflamasen": "soflamar",
    "soflamases": "soflamar",
    "soflamaste": "soflamar",
    "soflamasteis": "soflamar",
    "soflame": "soflamar",
    "soflamemos": "soflamar",
    "soflamen": "soflamar",
    "soflamera": "soflamero",
    "soflameras": "soflamero",
    "soflameros": "soflamero",
    "soflames": "soflamar",
    "soflamo": "soflamar",
    "soflamábamos": "soflamar",
    "soflamáis": "soflamar",
    "soflamáramos": "soflamar",
    "soflamáremos": "soflamar",
    "soflamásemos": "soflamar",
    "soflamé": "soflamar",
    "soflaméis": "soflamar",
    "soflamó": "soflamar",
    "sofoca": "sofocar",
    "sofocaba": "sofocar",
    "sofocabais": "sofocar",
    "sofocaban": "sofocar",
    "sofocabas": "sofocar",
    "sofocaciones": "sofocación",
    "sofocad": "sofocar",
    "sofocada": "sofocar",
    "sofocadas": "sofocar",
    "sofocado": "sofocar",
    "sofocadora": "sofocador",
    "sofocadoras": "sofocador",
    "sofocadores": "sofocador",
    "sofocados": "sofocar",
    "sofocamos": "sofocar",
    "sofocan": "sofocar",
    "sofocando": "sofocar",
    "sofocantes": "sofocante",
    "sofocara": "sofocar",
    "sofocarais": "sofocar",
    "sofocaran": "sofocar",
    "sofocaras": "sofocar",
    "sofocare": "sofocar",
    "sofocareis": "sofocar",
    "sofocaremos": "sofocar",
    "sofocaren": "sofocar",
    "sofocares": "sofocar",
    "sofocaron": "sofocar",
    "sofocará": "sofocar",
    "sofocarán": "sofocar",
    "sofocarás": "sofocar",
    "sofocaré": "sofocar",
    "sofocaréis": "sofocar",
    "sofocaría": "sofocar",
    "sofocaríais": "sofocar",
    "sofocaríamos": "sofocar",
    "sofocarían": "sofocar",
    "sofocarías": "sofocar",
    "sofocas": "sofocar",
    "sofocase": "sofocar",
    "sofocaseis": "sofocar",
    "sofocasen": "sofocar",
    "sofocases": "sofocar",
    "sofocaste": "sofocar",
    "sofocasteis": "sofocar",
    "sofoclea": "sofocleo",
    "sofocleas": "sofocleo",
    "sofocleos": "sofocleo",
    "sofoco": "sofocar",
    "sofocones": "sofocón",
    "sofocos": "sofoco",
    "sofocábamos": "sofocar",
    "sofocáis": "sofocar",
    "sofocáramos": "sofocar",
    "sofocáremos": "sofocar",
    "sofocásemos": "sofocar",
    "sofocó": "sofocar",
    "sofometrías": "sofometría",
    "sofoque": "sofocar",
    "sofoquemos": "sofocar",
    "sofoquen": "sofocar",
    "sofoques": "sofocar",
    "sofoquinas": "sofoquina",
    "sofoqué": "sofocar",
    "sofoquéis": "sofocar",
    "sofreiremos": "sofreír",
    "sofreirá": "sofreír",
    "sofreirán": "sofreír",
    "sofreirás": "sofreír",
    "sofreiré": "sofreír",
    "sofreiréis": "sofreír",
    "sofreiría": "sofreír",
    "sofreiríais": "sofreír",
    "sofreiríamos": "sofreír",
    "sofreirían": "sofreír",
    "sofreirías": "sofreír",
    "sofrena": "sofrenar",
    "sofrenaba": "sofrenar",
    "sofrenabais": "sofrenar",
    "sofrenaban": "sofrenar",
    "sofrenabas": "sofrenar",
    "sofrenad": "sofrenar",
    "sofrenada": "sofrenar",
    "sofrenadas": "sofrenar",
    "sofrenado": "sofrenar",
    "sofrenados": "sofrenar",
    "sofrenamos": "sofrenar",
    "sofrenan": "sofrenar",
    "sofrenando": "sofrenar",
    "sofrenara": "sofrenar",
    "sofrenarais": "sofrenar",
    "sofrenaran": "sofrenar",
    "sofrenaras": "sofrenar",
    "sofrenare": "sofrenar",
    "sofrenareis": "sofrenar",
    "sofrenaremos": "sofrenar",
    "sofrenaren": "sofrenar",
    "sofrenares": "sofrenar",
    "sofrenaron": "sofrenar",
    "sofrenará": "sofrenar",
    "sofrenarán": "sofrenar",
    "sofrenarás": "sofrenar",
    "sofrenaré": "sofrenar",
    "sofrenaréis": "sofrenar",
    "sofrenaría": "sofrenar",
    "sofrenaríais": "sofrenar",
    "sofrenaríamos": "sofrenar",
    "sofrenarían": "sofrenar",
    "sofrenarías": "sofrenar",
    "sofrenas": "sofrenar",
    "sofrenase": "sofrenar",
    "sofrenaseis": "sofrenar",
    "sofrenasen": "sofrenar",
    "sofrenases": "sofrenar",
    "sofrenaste": "sofrenar",
    "sofrenasteis": "sofrenar",
    "sofrene": "sofrenar",
    "sofrenemos": "sofrenar",
    "sofrenen": "sofrenar",
    "sofrenes": "sofrenar",
    "sofreno": "sofrenar",
    "sofrenábamos": "sofrenar",
    "sofrenáis": "sofrenar",
    "sofrenáramos": "sofrenar",
    "sofrenáremos": "sofrenar",
    "sofrenásemos": "sofrenar",
    "sofrené": "sofrenar",
    "sofrenéis": "sofrenar",
    "sofrenó": "sofrenar",
    "sofreí": "sofreír",
    "sofreía": "sofreír",
    "sofreíais": "sofreír",
    "sofreíamos": "sofreír",
    "sofreían": "sofreír",
    "sofreías": "sofreír",
    "sofreíd": "sofreír",
    "sofreída": "sofreír",
    "sofreídas": "sofreír",
    "sofreído": "sofreír",
    "sofreídos": "sofreír",
    "sofreímos": "sofreír",
    "sofreís": "sofreír",
    "sofreíste": "sofreír",
    "sofreísteis": "sofreír",
    "sofriamos": "sofreír",
    "sofriendo": "sofreír",
    "sofriera": "sofreír",
    "sofrierais": "sofreír",
    "sofrieran": "sofreír",
    "sofrieras": "sofreír",
    "sofriere": "sofreír",
    "sofriereis": "sofreír",
    "sofrieren": "sofreír",
    "sofrieres": "sofreír",
    "sofrieron": "sofreír",
    "sofriese": "sofreír",
    "sofrieseis": "sofreír",
    "sofriesen": "sofreír",
    "sofrieses": "sofreír",
    "sofrita": "sofreír",
    "sofritas": "sofreír",
    "sofrito": "sofreír",
    "sofritos": "sofrito",
    "sofriáis": "sofreír",
    "sofriéramos": "sofreír",
    "sofriéremos": "sofreír",
    "sofriésemos": "sofreír",
    "sofrió": "sofreír",
    "sofrologías": "sofrología",
    "sofría": "sofreír",
    "sofrían": "sofreír",
    "sofrías": "sofreír",
    "sofríe": "sofreír",
    "sofríen": "sofreír",
    "sofríes": "sofreír",
    "sofrío": "sofreír",
    "softballs": "softball",
    "sofácamas": "sofácama",
    "sofás": "sofá",
    "sofáscamas": "sofáscama",
    "sofíes": "sofí",
    "sofís": "sofí",
    "sofística": "sofístico",
    "sofísticas": "sofístico",
    "sofísticos": "sofístico",
    "sofómetros": "sofómetro",
    "sogas": "soga",
    "soguera": "soguero",
    "sogueras": "soguero",
    "sogueros": "soguero",
    "soguerías": "soguería",
    "soguillas": "soguilla",
    "sogúnes": "sogún",
    "soirées": "soirée",
    "sois": "ser",
    "sojas": "soja",
    "sojuzga": "sojuzgar",
    "sojuzgaba": "sojuzgar",
    "sojuzgabais": "sojuzgar",
    "sojuzgaban": "sojuzgar",
    "sojuzgabas": "sojuzgar",
    "sojuzgad": "sojuzgar",
    "sojuzgada": "sojuzgar",
    "sojuzgadas": "sojuzgar",
    "sojuzgado": "sojuzgar",
    "sojuzgadora": "sojuzgador",
    "sojuzgadoras": "sojuzgador",
    "sojuzgadores": "sojuzgador",
    "sojuzgados": "sojuzgar",
    "sojuzgamos": "sojuzgar",
    "sojuzgan": "sojuzgar",
    "sojuzgando": "sojuzgar",
    "sojuzgara": "sojuzgar",
    "sojuzgarais": "sojuzgar",
    "sojuzgaran": "sojuzgar",
    "sojuzgaras": "sojuzgar",
    "sojuzgare": "sojuzgar",
    "sojuzgareis": "sojuzgar",
    "sojuzgaremos": "sojuzgar",
    "sojuzgaren": "sojuzgar",
    "sojuzgares": "sojuzgar",
    "sojuzgaron": "sojuzgar",
    "sojuzgará": "sojuzgar",
    "sojuzgarán": "sojuzgar",
    "sojuzgarás": "sojuzgar",
    "sojuzgaré": "sojuzgar",
    "sojuzgaréis": "sojuzgar",
    "sojuzgaría": "sojuzgar",
    "sojuzgaríais": "sojuzgar",
    "sojuzgaríamos": "sojuzgar",
    "sojuzgarían": "sojuzgar",
    "sojuzgarías": "sojuzgar",
    "sojuzgas": "sojuzgar",
    "sojuzgase": "sojuzgar",
    "sojuzgaseis": "sojuzgar",
    "sojuzgasen": "sojuzgar",
    "sojuzgases": "sojuzgar",
    "sojuzgaste": "sojuzgar",
    "sojuzgasteis": "sojuzgar",
    "sojuzgo": "sojuzgar",
    "sojuzgue": "sojuzgar",
    "sojuzguemos": "sojuzgar",
    "sojuzguen": "sojuzgar",
    "sojuzgues": "sojuzgar",
    "sojuzgué": "sojuzgar",
    "sojuzguéis": "sojuzgar",
    "sojuzgábamos": "sojuzgar",
    "sojuzgáis": "sojuzgar",
    "sojuzgáramos": "sojuzgar",
    "sojuzgáremos": "sojuzgar",
    "sojuzgásemos": "sojuzgar",
    "sojuzgó": "sojuzgar",
    "sola": "solo",
    "solaba": "solar",
    "solabais": "solar",
    "solaban": "solar",
    "solabas": "solar",
    "solace": "solazar",
    "solacemos": "solazar",
    "solacen": "solazar",
    "solaces": "solazar",
    "solaciones": "solación",
    "solacé": "solazar",
    "solacéis": "solazar",
    "solad": "solar",
    "solada": "solar",
    "soladas": "solar",
    "solado": "solar",
    "soladora": "solador",
    "soladoras": "solador",
    "soladores": "solador",
    "solados": "solar",
    "soladuras": "soladura",
    "solamos": "soler",
    "solanas": "solana",
    "solando": "solar",
    "solaneras": "solanera",
    "solaninas": "solanina",
    "solanos": "solano",
    "solanácea": "solanáceo",
    "solanáceas": "solanáceo",
    "solanáceos": "solanáceo",
    "solapa": "solapar",
    "solapaba": "solapar",
    "solapabais": "solapar",
    "solapaban": "solapar",
    "solapabas": "solapar",
    "solapad": "solapar",
    "solapada": "solapar",
    "solapadas": "solapar",
    "solapado": "solapar",
    "solapados": "solapar",
    "solapamientos": "solapamiento",
    "solapamos": "solapar",
    "solapan": "solapar",
    "solapando": "solapar",
    "solapara": "solapar",
    "solaparais": "solapar",
    "solaparan": "solapar",
    "solaparas": "solapar",
    "solapare": "solapar",
    "solapareis": "solapar",
    "solaparemos": "solapar",
    "solaparen": "solapar",
    "solapares": "solapar",
    "solaparon": "solapar",
    "solapará": "solapar",
    "solaparán": "solapar",
    "solaparás": "solapar",
    "solaparé": "solapar",
    "solaparéis": "solapar",
    "solaparía": "solapar",
    "solaparíais": "solapar",
    "solaparíamos": "solapar",
    "solaparían": "solapar",
    "solaparías": "solapar",
    "solapas": "solapar",
    "solapase": "solapar",
    "solapaseis": "solapar",
    "solapasen": "solapar",
    "solapases": "solapar",
    "solapaste": "solapar",
    "solapasteis": "solapar",
    "solape": "solapar",
    "solapemos": "solapar",
    "solapen": "solapar",
    "solapes": "solape",
    "solapo": "solapar",
    "solapos": "solapo",
    "solapábamos": "solapar",
    "solapáis": "solapar",
    "solapáramos": "solapar",
    "solapáremos": "solapar",
    "solapásemos": "solapar",
    "solapé": "solapar",
    "solapéis": "solapar",
    "solapó": "solapar",
    "solara": "solar",
    "solarais": "solar",
    "solaran": "solar",
    "solaras": "solar",
    "solare": "solar",
    "solareis": "solar",
    "solaremos": "solar",
    "solaren": "solar",
    "solares": "solar",
    "solariega": "solariego",
    "solariegas": "solariego",
    "solariegos": "solariego",
    "solarios": "solario",
    "solariums": "solarium",
    "solarizaciones": "solarización",
    "solaron": "solar",
    "solará": "solar",
    "solarán": "solar",
    "solarás": "solar",
    "solaré": "solar",
    "solaréis": "solar",
    "solaría": "solar",
    "solaríais": "solar",
    "solaríamos": "solar",
    "solarían": "solar",
    "solarías": "solar",
    "solarímetros": "solarímetro",
    "solas": "solo",
    "solase": "solar",
    "solaseis": "solar",
    "solasen": "solar",
    "solases": "solar",
    "solaste": "solar",
    "solasteis": "solar",
    "solaza": "solazar",
    "solazaba": "solazar",
    "solazabais": "solazar",
    "solazaban": "solazar",
    "solazabas": "solazar",
    "solazad": "solazar",
    "solazada": "solazar",
    "solazadas": "solazar",
    "solazado": "solazar",
    "solazados": "solazar",
    "solazamos": "solazar",
    "solazan": "solazar",
    "solazando": "solazar",
    "solazara": "solazar",
    "solazarais": "solazar",
    "solazaran": "solazar",
    "solazaras": "solazar",
    "solazare": "solazar",
    "solazareis": "solazar",
    "solazaremos": "solazar",
    "solazaren": "solazar",
    "solazares": "solazar",
    "solazaron": "solazar",
    "solazará": "solazar",
    "solazarán": "solazar",
    "solazarás": "solazar",
    "solazaré": "solazar",
    "solazaréis": "solazar",
    "solazaría": "solazar",
    "solazaríais": "solazar",
    "solazaríamos": "solazar",
    "solazarían": "solazar",
    "solazarías": "solazar",
    "solazas": "solazar",
    "solazase": "solazar",
    "solazaseis": "solazar",
    "solazasen": "solazar",
    "solazases": "solazar",
    "solazaste": "solazar",
    "solazasteis": "solazar",
    "solazo": "solazar",
    "solazos": "solazo",
    "solazosa": "solazoso",
    "solazosas": "solazoso",
    "solazosos": "solazoso",
    "solazábamos": "solazar",
    "solazáis": "solazar",
    "solazáramos": "solazar",
    "solazáremos": "solazar",
    "solazásemos": "solazar",
    "solazó": "solazar",
    "soldaba": "soldar",
    "soldabais": "soldar",
    "soldaban": "soldar",
    "soldabas": "soldar",
    "soldabilidades": "soldabilidad",
    "soldables": "soldable",
    "soldad": "soldar",
    "soldada": "soldar",
    "soldadas": "soldar",
    "soldadesca": "soldadesco",
    "soldadescas": "soldadesco",
    "soldadescos": "soldadesco",
    "soldado": "soldar",
    "soldadora": "soldador",
    "soldadoras": "soldadora",
    "soldadores": "soldador",
    "soldados": "soldar",
    "soldaduras": "soldadura",
    "soldamos": "soldar",
    "soldando": "soldar",
    "soldara": "soldar",
    "soldarais": "soldar",
    "soldaran": "soldar",
    "soldaras": "soldar",
    "soldare": "soldar",
    "soldareis": "soldar",
    "soldaremos": "soldar",
    "soldaren": "soldar",
    "soldares": "soldar",
    "soldaron": "soldar",
    "soldará": "soldar",
    "soldarán": "soldar",
    "soldarás": "soldar",
    "soldaré": "soldar",
    "soldaréis": "soldar",
    "soldaría": "soldar",
    "soldaríais": "soldar",
    "soldaríamos": "soldar",
    "soldarían": "soldar",
    "soldarías": "soldar",
    "soldase": "soldar",
    "soldaseis": "soldar",
    "soldasen": "soldar",
    "soldases": "soldar",
    "soldaste": "soldar",
    "soldasteis": "soldar",
    "soldemos": "soldar",
    "soldeos": "soldeo",
    "soldábamos": "soldar",
    "soldáis": "soldar",
    "soldáramos": "soldar",
    "soldáremos": "soldar",
    "soldásemos": "soldar",
    "soldé": "soldar",
    "soldéis": "soldar",
    "soldó": "soldar",
    "solea": "solear",
    "soleaba": "solear",
    "soleabais": "solear",
    "soleaban": "solear",
    "soleabas": "solear",
    "solead": "solear",
    "soleada": "solear",
    "soleadas": "solear",
    "soleado": "solear",
    "soleados": "solear",
    "soleamientos": "soleamiento",
    "soleamos": "solear",
    "solean": "solear",
    "soleando": "solear",
    "soleara": "solear",
    "solearais": "solear",
    "solearan": "solear",
    "solearas": "solear",
    "soleare": "solear",
    "soleareis": "solear",
    "solearemos": "solear",
    "solearen": "solear",
    "soleares": "solear",
    "solearon": "solear",
    "soleará": "solear",
    "solearán": "solear",
    "solearás": "solear",
    "solearé": "solear",
    "solearéis": "solear",
    "solearía": "solear",
    "solearíais": "solear",
    "solearíamos": "solear",
    "solearían": "solear",
    "solearías": "solear",
    "soleas": "solear",
    "solease": "solear",
    "soleaseis": "solear",
    "soleasen": "solear",
    "soleases": "solear",
    "soleaste": "solear",
    "soleasteis": "solear",
    "solecismos": "solecismo",
    "soledades": "soledad",
    "soledosa": "soledoso",
    "soledosas": "soledoso",
    "soledosos": "soledoso",
    "solee": "solear",
    "soleemos": "solear",
    "soleen": "solear",
    "solees": "solear",
    "solejares": "solejar",
    "solemnes": "solemne",
    "solemnice": "solemnizar",
    "solemnicemos": "solemnizar",
    "solemnicen": "solemnizar",
    "solemnices": "solemnizar",
    "solemnicé": "solemnizar",
    "solemnicéis": "solemnizar",
    "solemnidades": "solemnidad",
    "solemniza": "solemnizar",
    "solemnizaba": "solemnizar",
    "solemnizabais": "solemnizar",
    "solemnizaban": "solemnizar",
    "solemnizabas": "solemnizar",
    "solemnizad": "solemnizar",
    "solemnizada": "solemnizar",
    "solemnizadas": "solemnizar",
    "solemnizado": "solemnizar",
    "solemnizados": "solemnizar",
    "solemnizamos": "solemnizar",
    "solemnizan": "solemnizar",
    "solemnizando": "solemnizar",
    "solemnizara": "solemnizar",
    "solemnizarais": "solemnizar",
    "solemnizaran": "solemnizar",
    "solemnizaras": "solemnizar",
    "solemnizare": "solemnizar",
    "solemnizareis": "solemnizar",
    "solemnizaremos": "solemnizar",
    "solemnizaren": "solemnizar",
    "solemnizares": "solemnizar",
    "solemnizaron": "solemnizar",
    "solemnizará": "solemnizar",
    "solemnizarán": "solemnizar",
    "solemnizarás": "solemnizar",
    "solemnizaré": "solemnizar",
    "solemnizaréis": "solemnizar",
    "solemnizaría": "solemnizar",
    "solemnizaríais": "solemnizar",
    "solemnizaríamos": "solemnizar",
    "solemnizarían": "solemnizar",
    "solemnizarías": "solemnizar",
    "solemnizas": "solemnizar",
    "solemnizase": "solemnizar",
    "solemnizaseis": "solemnizar",
    "solemnizasen": "solemnizar",
    "solemnizases": "solemnizar",
    "solemnizaste": "solemnizar",
    "solemnizasteis": "solemnizar",
    "solemnizo": "solemnizar",
    "solemnizábamos": "solemnizar",
    "solemnizáis": "solemnizar",
    "solemnizáramos": "solemnizar",
    "solemnizáremos": "solemnizar",
    "solemnizásemos": "solemnizar",
    "solemnizó": "solemnizar",
    "solemos": "soler",
    "solenoglifa": "solenoglifo",
    "solenoglifas": "solenoglifo",
    "solenoglifos": "solenoglifo",
    "solenoidales": "solenoidal",
    "solenoides": "solenoide",
    "soleo": "solear",
    "soleos": "soleo",
    "soleras": "solera",
    "solercias": "solercia",
    "soleres": "soler",
    "solertes": "solerte",
    "solerías": "solería",
    "soles": "sol",
    "soletas": "soleta",
    "soleva": "solevar",
    "solevaba": "solevar",
    "solevabais": "solevar",
    "solevaban": "solevar",
    "solevabas": "solevar",
    "solevaciones": "solevación",
    "solevad": "solevar",
    "solevada": "solevar",
    "solevadas": "solevar",
    "solevado": "solevar",
    "solevados": "solevar",
    "solevamientos": "solevamiento",
    "solevamos": "solevar",
    "solevan": "solevar",
    "solevando": "solevar",
    "solevanta": "solevantar",
    "solevantaba": "solevantar",
    "solevantabais": "solevantar",
    "solevantaban": "solevantar",
    "solevantabas": "solevantar",
    "solevantad": "solevantar",
    "solevantada": "solevantar",
    "solevantadas": "solevantar",
    "solevantado": "solevantar",
    "solevantados": "solevantar",
    "solevantamientos": "solevantamiento",
    "solevantamos": "solevantar",
    "solevantan": "solevantar",
    "solevantando": "solevantar",
    "solevantara": "solevantar",
    "solevantarais": "solevantar",
    "solevantaran": "solevantar",
    "solevantaras": "solevantar",
    "solevantare": "solevantar",
    "solevantareis": "solevantar",
    "solevantaremos": "solevantar",
    "solevantaren": "solevantar",
    "solevantares": "solevantar",
    "solevantaron": "solevantar",
    "solevantará": "solevantar",
    "solevantarán": "solevantar",
    "solevantarás": "solevantar",
    "solevantaré": "solevantar",
    "solevantaréis": "solevantar",
    "solevantaría": "solevantar",
    "solevantaríais": "solevantar",
    "solevantaríamos": "solevantar",
    "solevantarían": "solevantar",
    "solevantarías": "solevantar",
    "solevantas": "solevantar",
    "solevantase": "solevantar",
    "solevantaseis": "solevantar",
    "solevantasen": "solevantar",
    "solevantases": "solevantar",
    "solevantaste": "solevantar",
    "solevantasteis": "solevantar",
    "solevante": "solevantar",
    "solevantemos": "solevantar",
    "solevanten": "solevantar",
    "solevantes": "solevantar",
    "solevanto": "solevantar",
    "solevantábamos": "solevantar",
    "solevantáis": "solevantar",
    "solevantáramos": "solevantar",
    "solevantáremos": "solevantar",
    "solevantásemos": "solevantar",
    "solevanté": "solevantar",
    "solevantéis": "solevantar",
    "solevantó": "solevantar",
    "solevara": "solevar",
    "solevarais": "solevar",
    "solevaran": "solevar",
    "solevaras": "solevar",
    "solevare": "solevar",
    "solevareis": "solevar",
    "solevaremos": "solevar",
    "solevaren": "solevar",
    "solevares": "solevar",
    "solevaron": "solevar",
    "solevará": "solevar",
    "solevarán": "solevar",
    "solevarás": "solevar",
    "solevaré": "solevar",
    "solevaréis": "solevar",
    "solevaría": "solevar",
    "solevaríais": "solevar",
    "solevaríamos": "solevar",
    "solevarían": "solevar",
    "solevarías": "solevar",
    "solevas": "solevar",
    "solevase": "solevar",
    "solevaseis": "solevar",
    "solevasen": "solevar",
    "solevases": "solevar",
    "solevaste": "solevar",
    "solevasteis": "solevar",
    "soleve": "solevar",
    "solevemos": "solevar",
    "soleven": "solevar",
    "soleves": "solevar",
    "solevo": "solevar",
    "solevábamos": "solevar",
    "soleváis": "solevar",
    "soleváramos": "solevar",
    "soleváremos": "solevar",
    "solevásemos": "solevar",
    "solevé": "solevar",
    "solevéis": "solevar",
    "solevó": "solevar",
    "soleábamos": "solear",
    "soleáis": "solear",
    "soleáramos": "solear",
    "soleáremos": "solear",
    "soleás": "soleá",
    "soleásemos": "solear",
    "soleé": "solear",
    "soleéis": "solear",
    "soleó": "solear",
    "solfas": "solfa",
    "solfataras": "solfatara",
    "solfea": "solfear",
    "solfeaba": "solfear",
    "solfeabais": "solfear",
    "solfeaban": "solfear",
    "solfeabas": "solfear",
    "solfead": "solfear",
    "solfeada": "solfear",
    "solfeadas": "solfear",
    "solfeado": "solfear",
    "solfeadora": "solfeador",
    "solfeadoras": "solfeador",
    "solfeadores": "solfeador",
    "solfeados": "solfear",
    "solfeamos": "solfear",
    "solfean": "solfear",
    "solfeando": "solfear",
    "solfeara": "solfear",
    "solfearais": "solfear",
    "solfearan": "solfear",
    "solfearas": "solfear",
    "solfeare": "solfear",
    "solfeareis": "solfear",
    "solfearemos": "solfear",
    "solfearen": "solfear",
    "solfeares": "solfear",
    "solfearon": "solfear",
    "solfeará": "solfear",
    "solfearán": "solfear",
    "solfearás": "solfear",
    "solfearé": "solfear",
    "solfearéis": "solfear",
    "solfearía": "solfear",
    "solfearíais": "solfear",
    "solfearíamos": "solfear",
    "solfearían": "solfear",
    "solfearías": "solfear",
    "solfeas": "solfear",
    "solfease": "solfear",
    "solfeaseis": "solfear",
    "solfeasen": "solfear",
    "solfeases": "solfear",
    "solfeaste": "solfear",
    "solfeasteis": "solfear",
    "solfee": "solfear",
    "solfeemos": "solfear",
    "solfeen": "solfear",
    "solfees": "solfear",
    "solfeo": "solfear",
    "solfeos": "solfeo",
    "solfeábamos": "solfear",
    "solfeáis": "solfear",
    "solfeáramos": "solfear",
    "solfeáremos": "solfear",
    "solfeásemos": "solfear",
    "solfeé": "solfear",
    "solfeéis": "solfear",
    "solfeó": "solfear",
    "solfistas": "solfista",
    "solicita": "solicitar",
    "solicitaba": "solicitar",
    "solicitabais": "solicitar",
    "solicitaban": "solicitar",
    "solicitabas": "solicitar",
    "solicitaciones": "solicitación",
    "solicitad": "solicitar",
    "solicitada": "solicitar",
    "solicitadas": "solicitar",
    "solicitado": "solicitar",
    "solicitadora": "solicitador",
    "solicitadoras": "solicitador",
    "solicitadores": "solicitador",
    "solicitados": "solicitar",
    "solicitamos": "solicitar",
    "solicitan": "solicitar",
    "solicitando": "solicitar",
    "solicitantes": "solicitante",
    "solicitara": "solicitar",
    "solicitarais": "solicitar",
    "solicitaran": "solicitar",
    "solicitaras": "solicitar",
    "solicitare": "solicitar",
    "solicitareis": "solicitar",
    "solicitaremos": "solicitar",
    "solicitaren": "solicitar",
    "solicitares": "solicitar",
    "solicitaron": "solicitar",
    "solicitará": "solicitar",
    "solicitarán": "solicitar",
    "solicitarás": "solicitar",
    "solicitaré": "solicitar",
    "solicitaréis": "solicitar",
    "solicitaría": "solicitar",
    "solicitaríais": "solicitar",
    "solicitaríamos": "solicitar",
    "solicitarían": "solicitar",
    "solicitarías": "solicitar",
    "solicitas": "solicitar",
    "solicitase": "solicitar",
    "solicitaseis": "solicitar",
    "solicitasen": "solicitar",
    "solicitases": "solicitar",
    "solicitaste": "solicitar",
    "solicitasteis": "solicitar",
    "solicite": "solicitar",
    "solicitemos": "solicitar",
    "soliciten": "solicitar",
    "solicites": "solicitar",
    "solicito": "solicitar",
    "solicitudes": "solicitud",
    "solicitábamos": "solicitar",
    "solicitáis": "solicitar",
    "solicitáramos": "solicitar",
    "solicitáremos": "solicitar",
    "solicitásemos": "solicitar",
    "solicité": "solicitar",
    "solicitéis": "solicitar",
    "solicitó": "solicitar",
    "solida": "solidar",
    "solidaba": "solidar",
    "solidabais": "solidar",
    "solidaban": "solidar",
    "solidabas": "solidar",
    "solidad": "solidar",
    "solidada": "solidar",
    "solidadas": "solidar",
    "solidado": "solidar",
    "solidados": "solidar",
    "solidamos": "solidar",
    "solidan": "solidar",
    "solidando": "solidar",
    "solidara": "solidar",
    "solidarais": "solidar",
    "solidaran": "solidar",
    "solidaras": "solidar",
    "solidare": "solidar",
    "solidareis": "solidar",
    "solidaremos": "solidar",
    "solidaren": "solidar",
    "solidares": "solidar",
    "solidaria": "solidario",
    "solidarias": "solidario",
    "solidarice": "solidarizar",
    "solidaricemos": "solidarizar",
    "solidaricen": "solidarizar",
    "solidarices": "solidarizar",
    "solidaricé": "solidarizar",
    "solidaricéis": "solidarizar",
    "solidaridades": "solidaridad",
    "solidarios": "solidario",
    "solidariza": "solidarizar",
    "solidarizaba": "solidarizar",
    "solidarizabais": "solidarizar",
    "solidarizaban": "solidarizar",
    "solidarizabas": "solidarizar",
    "solidarizaciones": "solidarización",
    "solidarizad": "solidarizar",
    "solidarizada": "solidarizar",
    "solidarizadas": "solidarizar",
    "solidarizado": "solidarizar",
    "solidarizados": "solidarizar",
    "solidarizamos": "solidarizar",
    "solidarizan": "solidarizar",
    "solidarizando": "solidarizar",
    "solidarizara": "solidarizar",
    "solidarizarais": "solidarizar",
    "solidarizaran": "solidarizar",
    "solidarizaras": "solidarizar",
    "solidarizare": "solidarizar",
    "solidarizareis": "solidarizar",
    "solidarizaremos": "solidarizar",
    "solidarizaren": "solidarizar",
    "solidarizares": "solidarizar",
    "solidarizaron": "solidarizar",
    "solidarizará": "solidarizar",
    "solidarizarán": "solidarizar",
    "solidarizarás": "solidarizar",
    "solidarizaré": "solidarizar",
    "solidarizaréis": "solidarizar",
    "solidarizaría": "solidarizar",
    "solidarizaríais": "solidarizar",
    "solidarizaríamos": "solidarizar",
    "solidarizarían": "solidarizar",
    "solidarizarías": "solidarizar",
    "solidarizas": "solidarizar",
    "solidarizase": "solidarizar",
    "solidarizaseis": "solidarizar",
    "solidarizasen": "solidarizar",
    "solidarizases": "solidarizar",
    "solidarizaste": "solidarizar",
    "solidarizasteis": "solidarizar",
    "solidarizo": "solidarizar",
    "solidarizábamos": "solidarizar",
    "solidarizáis": "solidarizar",
    "solidarizáramos": "solidarizar",
    "solidarizáremos": "solidarizar",
    "solidarizásemos": "solidarizar",
    "solidarizó": "solidarizar",
    "solidaron": "solidar",
    "solidará": "solidar",
    "solidarán": "solidar",
    "solidarás": "solidar",
    "solidaré": "solidar",
    "solidaréis": "solidar",
    "solidaría": "solidar",
    "solidaríais": "solidar",
    "solidaríamos": "solidar",
    "solidarían": "solidar",
    "solidarías": "solidar",
    "solidas": "solidar",
    "solidase": "solidar",
    "solidaseis": "solidar",
    "solidasen": "solidar",
    "solidases": "solidar",
    "solidaste": "solidar",
    "solidasteis": "solidar",
    "solide": "solidar",
    "solideces": "solidez",
    "solidemos": "solidar",
    "soliden": "solidar",
    "solideos": "solideo",
    "solides": "solidar",
    "solidifica": "solidificar",
    "solidificaba": "solidificar",
    "solidificabais": "solidificar",
    "solidificaban": "solidificar",
    "solidificabas": "solidificar",
    "solidificaciones": "solidificación",
    "solidificad": "solidificar",
    "solidificada": "solidificar",
    "solidificadas": "solidificar",
    "solidificado": "solidificar",
    "solidificados": "solidificar",
    "solidificamos": "solidificar",
    "solidifican": "solidificar",
    "solidificando": "solidificar",
    "solidificara": "solidificar",
    "solidificarais": "solidificar",
    "solidificaran": "solidificar",
    "solidificaras": "solidificar",
    "solidificare": "solidificar",
    "solidificareis": "solidificar",
    "solidificaremos": "solidificar",
    "solidificaren": "solidificar",
    "solidificares": "solidificar",
    "solidificaron": "solidificar",
    "solidificará": "solidificar",
    "solidificarán": "solidificar",
    "solidificarás": "solidificar",
    "solidificaré": "solidificar",
    "solidificaréis": "solidificar",
    "solidificaría": "solidificar",
    "solidificaríais": "solidificar",
    "solidificaríamos": "solidificar",
    "solidificarían": "solidificar",
    "solidificarías": "solidificar",
    "solidificas": "solidificar",
    "solidificase": "solidificar",
    "solidificaseis": "solidificar",
    "solidificasen": "solidificar",
    "solidificases": "solidificar",
    "solidificaste": "solidificar",
    "solidificasteis": "solidificar",
    "solidifico": "solidificar",
    "solidificábamos": "solidificar",
    "solidificáis": "solidificar",
    "solidificáramos": "solidificar",
    "solidificáremos": "solidificar",
    "solidificásemos": "solidificar",
    "solidificó": "solidificar",
    "solidifique": "solidificar",
    "solidifiquemos": "solidificar",
    "solidifiquen": "solidificar",
    "solidifiques": "solidificar",
    "solidifiqué": "solidificar",
    "solidifiquéis": "solidificar",
    "solido": "solidar",
    "solidábamos": "solidar",
    "solidáis": "solidar",
    "solidáramos": "solidar",
    "solidáremos": "solidar",
    "solidásemos": "solidar",
    "solidé": "solidar",
    "solidéis": "solidar",
    "solidó": "solidar",
    "soliera": "soler",
    "solierais": "soler",
    "solieran": "soler",
    "solieras": "soler",
    "solieron": "soler",
    "soliese": "soler",
    "solieseis": "soler",
    "soliesen": "soler",
    "solieses": "soler",
    "solifluxiones": "solifluxión",
    "soliloquia": "soliloquiar",
    "soliloquiaba": "soliloquiar",
    "soliloquiabais": "soliloquiar",
    "soliloquiaban": "soliloquiar",
    "soliloquiabas": "soliloquiar",
    "soliloquiad": "soliloquiar",
    "soliloquiada": "soliloquiar",
    "soliloquiadas": "soliloquiar",
    "soliloquiado": "soliloquiar",
    "soliloquiados": "soliloquiar",
    "soliloquiamos": "soliloquiar",
    "soliloquian": "soliloquiar",
    "soliloquiando": "soliloquiar",
    "soliloquiara": "soliloquiar",
    "soliloquiarais": "soliloquiar",
    "soliloquiaran": "soliloquiar",
    "soliloquiaras": "soliloquiar",
    "soliloquiare": "soliloquiar",
    "soliloquiareis": "soliloquiar",
    "soliloquiaremos": "soliloquiar",
    "soliloquiaren": "soliloquiar",
    "soliloquiares": "soliloquiar",
    "soliloquiaron": "soliloquiar",
    "soliloquiará": "soliloquiar",
    "soliloquiarán": "soliloquiar",
    "soliloquiarás": "soliloquiar",
    "soliloquiaré": "soliloquiar",
    "soliloquiaréis": "soliloquiar",
    "soliloquiaría": "soliloquiar",
    "soliloquiaríais": "soliloquiar",
    "soliloquiaríamos": "soliloquiar",
    "soliloquiarían": "soliloquiar",
    "soliloquiarías": "soliloquiar",
    "soliloquias": "soliloquiar",
    "soliloquiase": "soliloquiar",
    "soliloquiaseis": "soliloquiar",
    "soliloquiasen": "soliloquiar",
    "soliloquiases": "soliloquiar",
    "soliloquiaste": "soliloquiar",
    "soliloquiasteis": "soliloquiar",
    "soliloquie": "soliloquiar",
    "soliloquiemos": "soliloquiar",
    "soliloquien": "soliloquiar",
    "soliloquies": "soliloquiar",
    "soliloquio": "soliloquiar",
    "soliloquios": "soliloquio",
    "soliloquiábamos": "soliloquiar",
    "soliloquiáis": "soliloquiar",
    "soliloquiáramos": "soliloquiar",
    "soliloquiáremos": "soliloquiar",
    "soliloquiásemos": "soliloquiar",
    "soliloquié": "soliloquiar",
    "soliloquiéis": "soliloquiar",
    "soliloquió": "soliloquiar",
    "solimos": "soler",
    "solios": "solio",
    "solipsismos": "solipsismo",
    "solistas": "solista",
    "soliste": "soler",
    "solisteis": "soler",
    "solitaria": "solitario",
    "solitarias": "solitario",
    "solitarios": "solitario",
    "solitones": "solitón",
    "solivia": "soliviar",
    "soliviaba": "soliviar",
    "soliviabais": "soliviar",
    "soliviaban": "soliviar",
    "soliviabas": "soliviar",
    "soliviad": "soliviar",
    "soliviada": "soliviar",
    "soliviadas": "soliviar",
    "soliviado": "soliviar",
    "soliviados": "soliviar",
    "soliviaduras": "soliviadura",
    "soliviamos": "soliviar",
    "solivian": "soliviar",
    "soliviando": "soliviar",
    "solivianta": "soliviantar",
    "soliviantaba": "soliviantar",
    "soliviantabais": "soliviantar",
    "soliviantaban": "soliviantar",
    "soliviantabas": "soliviantar",
    "soliviantad": "soliviantar",
    "soliviantada": "soliviantar",
    "soliviantadas": "soliviantar",
    "soliviantado": "soliviantar",
    "soliviantados": "soliviantar",
    "soliviantamos": "soliviantar",
    "soliviantan": "soliviantar",
    "soliviantando": "soliviantar",
    "soliviantara": "soliviantar",
    "soliviantarais": "soliviantar",
    "soliviantaran": "soliviantar",
    "soliviantaras": "soliviantar",
    "soliviantare": "soliviantar",
    "soliviantareis": "soliviantar",
    "soliviantaremos": "soliviantar",
    "soliviantaren": "soliviantar",
    "soliviantares": "soliviantar",
    "soliviantaron": "soliviantar",
    "soliviantará": "soliviantar",
    "soliviantarán": "soliviantar",
    "soliviantarás": "soliviantar",
    "soliviantaré": "soliviantar",
    "soliviantaréis": "soliviantar",
    "soliviantaría": "soliviantar",
    "soliviantaríais": "soliviantar",
    "soliviantaríamos": "soliviantar",
    "soliviantarían": "soliviantar",
    "soliviantarías": "soliviantar",
    "soliviantas": "soliviantar",
    "soliviantase": "soliviantar",
    "soliviantaseis": "soliviantar",
    "soliviantasen": "soliviantar",
    "soliviantases": "soliviantar",
    "soliviantaste": "soliviantar",
    "soliviantasteis": "soliviantar",
    "soliviante": "soliviantar",
    "soliviantemos": "soliviantar",
    "solivianten": "soliviantar",
    "soliviantes": "soliviantar",
    "solivianto": "soliviantar",
    "soliviantábamos": "soliviantar",
    "soliviantáis": "soliviantar",
    "soliviantáramos": "soliviantar",
    "soliviantáremos": "soliviantar",
    "soliviantásemos": "soliviantar",
    "solivianté": "soliviantar",
    "soliviantéis": "soliviantar",
    "soliviantó": "soliviantar",
    "soliviara": "soliviar",
    "soliviarais": "soliviar",
    "soliviaran": "soliviar",
    "soliviaras": "soliviar",
    "soliviare": "soliviar",
    "soliviareis": "soliviar",
    "soliviaremos": "soliviar",
    "soliviaren": "soliviar",
    "soliviares": "soliviar",
    "soliviaron": "soliviar",
    "soliviará": "soliviar",
    "soliviarán": "soliviar",
    "soliviarás": "soliviar",
    "soliviaré": "soliviar",
    "soliviaréis": "soliviar",
    "soliviaría": "soliviar",
    "soliviaríais": "soliviar",
    "soliviaríamos": "soliviar",
    "soliviarían": "soliviar",
    "soliviarías": "soliviar",
    "solivias": "soliviar",
    "soliviase": "soliviar",
    "soliviaseis": "soliviar",
    "soliviasen": "soliviar",
    "soliviases": "soliviar",
    "soliviaste": "soliviar",
    "soliviasteis": "soliviar",
    "solivie": "soliviar",
    "soliviemos": "soliviar",
    "solivien": "soliviar",
    "solivies": "soliviar",
    "solivio": "soliviar",
    "soliviones": "solivión",
    "solivios": "solivio",
    "soliviábamos": "soliviar",
    "soliviáis": "soliviar",
    "soliviáramos": "soliviar",
    "soliviáremos": "soliviar",
    "soliviásemos": "soliviar",
    "solivié": "soliviar",
    "soliviéis": "soliviar",
    "solivió": "soliviar",
    "soliéramos": "soler",
    "soliésemos": "soler",
    "solió": "soler",
    "sollada": "sollado",
    "solladas": "sollado",
    "sollados": "sollado",
    "sollama": "sollamar",
    "sollamaba": "sollamar",
    "sollamabais": "sollamar",
    "sollamaban": "sollamar",
    "sollamabas": "sollamar",
    "sollamad": "sollamar",
    "sollamada": "sollamar",
    "sollamadas": "sollamar",
    "sollamado": "sollamar",
    "sollamados": "sollamar",
    "sollamamos": "sollamar",
    "sollaman": "sollamar",
    "sollamando": "sollamar",
    "sollamara": "sollamar",
    "sollamarais": "sollamar",
    "sollamaran": "sollamar",
    "sollamaras": "sollamar",
    "sollamare": "sollamar",
    "sollamareis": "sollamar",
    "sollamaremos": "sollamar",
    "sollamaren": "sollamar",
    "sollamares": "sollamar",
    "sollamaron": "sollamar",
    "sollamará": "sollamar",
    "sollamarán": "sollamar",
    "sollamarás": "sollamar",
    "sollamaré": "sollamar",
    "sollamaréis": "sollamar",
    "sollamaría": "sollamar",
    "sollamaríais": "sollamar",
    "sollamaríamos": "sollamar",
    "sollamarían": "sollamar",
    "sollamarías": "sollamar",
    "sollamas": "sollamar",
    "sollamase": "sollamar",
    "sollamaseis": "sollamar",
    "sollamasen": "sollamar",
    "sollamases": "sollamar",
    "sollamaste": "sollamar",
    "sollamasteis": "sollamar",
    "sollame": "sollamar",
    "sollamemos": "sollamar",
    "sollamen": "sollamar",
    "sollames": "sollamar",
    "sollamo": "sollamar",
    "sollamábamos": "sollamar",
    "sollamáis": "sollamar",
    "sollamáramos": "sollamar",
    "sollamáremos": "sollamar",
    "sollamásemos": "sollamar",
    "sollamé": "sollamar",
    "sollaméis": "sollamar",
    "sollamó": "sollamar",
    "sollas": "solla",
    "sollastres": "sollastre",
    "sollastrías": "sollastría",
    "solletas": "solleta",
    "solloce": "sollozar",
    "sollocemos": "sollozar",
    "sollocen": "sollozar",
    "solloces": "sollozar",
    "sollocé": "sollozar",
    "sollocéis": "sollozar",
    "solloza": "sollozar",
    "sollozaba": "sollozar",
    "sollozabais": "sollozar",
    "sollozaban": "sollozar",
    "sollozabas": "sollozar",
    "sollozad": "sollozar",
    "sollozada": "sollozar",
    "sollozadas": "sollozar",
    "sollozado": "sollozar",
    "sollozados": "sollozar",
    "sollozamos": "sollozar",
    "sollozan": "sollozar",
    "sollozando": "sollozar",
    "sollozara": "sollozar",
    "sollozarais": "sollozar",
    "sollozaran": "sollozar",
    "sollozaras": "sollozar",
    "sollozare": "sollozar",
    "sollozareis": "sollozar",
    "sollozaremos": "sollozar",
    "sollozaren": "sollozar",
    "sollozares": "sollozar",
    "sollozaron": "sollozar",
    "sollozará": "sollozar",
    "sollozarán": "sollozar",
    "sollozarás": "sollozar",
    "sollozaré": "sollozar",
    "sollozaréis": "sollozar",
    "sollozaría": "sollozar",
    "sollozaríais": "sollozar",
    "sollozaríamos": "sollozar",
    "sollozarían": "sollozar",
    "sollozarías": "sollozar",
    "sollozas": "sollozar",
    "sollozase": "sollozar",
    "sollozaseis": "sollozar",
    "sollozasen": "sollozar",
    "sollozases": "sollozar",
    "sollozaste": "sollozar",
    "sollozasteis": "sollozar",
    "sollozo": "sollozar",
    "sollozos": "sollozo",
    "sollozábamos": "sollozar",
    "sollozáis": "sollozar",
    "sollozáramos": "sollozar",
    "sollozáremos": "sollozar",
    "sollozásemos": "sollozar",
    "sollozó": "sollozar",
    "solodizaciones": "solodización",
    "solomillos": "solomillo",
    "solomos": "solomo",
    "solos": "solo",
    "sols": "sol",
    "solsticios": "solsticio",
    "soltaba": "soltar",
    "soltabais": "soltar",
    "soltaban": "soltar",
    "soltabas": "soltar",
    "soltad": "soltar",
    "soltada": "soltar",
    "soltadas": "soltar",
    "soltado": "soltar",
    "soltadora": "soltador",
    "soltadoras": "soltador",
    "soltadores": "soltador",
    "soltados": "soltar",
    "soltamos": "soltar",
    "soltando": "soltar",
    "soltara": "soltar",
    "soltarais": "soltar",
    "soltaran": "soltar",
    "soltaras": "soltar",
    "soltare": "soltar",
    "soltareis": "soltar",
    "soltaremos": "soltar",
    "soltaren": "soltar",
    "soltares": "soltar",
    "soltaron": "soltar",
    "soltará": "soltar",
    "soltarán": "soltar",
    "soltarás": "soltar",
    "soltaré": "soltar",
    "soltaréis": "soltar",
    "soltaría": "soltar",
    "soltaríais": "soltar",
    "soltaríamos": "soltar",
    "soltarían": "soltar",
    "soltarías": "soltar",
    "soltase": "soltar",
    "soltaseis": "soltar",
    "soltasen": "soltar",
    "soltases": "soltar",
    "soltaste": "soltar",
    "soltasteis": "soltar",
    "soltemos": "soltar",
    "soltera": "soltero",
    "solteras": "soltero",
    "solterona": "solterón",
    "solteronas": "solterón",
    "solterones": "solterón",
    "solteros": "soltero",
    "solterías": "soltería",
    "solturas": "soltura",
    "soltábamos": "soltar",
    "soltáis": "soltar",
    "soltáramos": "soltar",
    "soltáremos": "soltar",
    "soltásemos": "soltar",
    "solté": "soltar",
    "soltéis": "soltar",
    "soltó": "soltar",
    "solubilice": "solubilizar",
    "solubilicemos": "solubilizar",
    "solubilicen": "solubilizar",
    "solubilices": "solubilizar",
    "solubilicé": "solubilizar",
    "solubilicéis": "solubilizar",
    "solubilidades": "solubilidad",
    "solubiliza": "solubilizar",
    "solubilizaba": "solubilizar",
    "solubilizabais": "solubilizar",
    "solubilizaban": "solubilizar",
    "solubilizabas": "solubilizar",
    "solubilizaciones": "solubilización",
    "solubilizad": "solubilizar",
    "solubilizada": "solubilizar",
    "solubilizadas": "solubilizar",
    "solubilizado": "solubilizar",
    "solubilizados": "solubilizar",
    "solubilizamos": "solubilizar",
    "solubilizan": "solubilizar",
    "solubilizando": "solubilizar",
    "solubilizara": "solubilizar",
    "solubilizarais": "solubilizar",
    "solubilizaran": "solubilizar",
    "solubilizaras": "solubilizar",
    "solubilizare": "solubilizar",
    "solubilizareis": "solubilizar",
    "solubilizaremos": "solubilizar",
    "solubilizaren": "solubilizar",
    "solubilizares": "solubilizar",
    "solubilizaron": "solubilizar",
    "solubilizará": "solubilizar",
    "solubilizarán": "solubilizar",
    "solubilizarás": "solubilizar",
    "solubilizaré": "solubilizar",
    "solubilizaréis": "solubilizar",
    "solubilizaría": "solubilizar",
    "solubilizaríais": "solubilizar",
    "solubilizaríamos": "solubilizar",
    "solubilizarían": "solubilizar",
    "solubilizarías": "solubilizar",
    "solubilizas": "solubilizar",
    "solubilizase": "solubilizar",
    "solubilizaseis": "solubilizar",
    "solubilizasen": "solubilizar",
    "solubilizases": "solubilizar",
    "solubilizaste": "solubilizar",
    "solubilizasteis": "solubilizar",
    "solubilizo": "solubilizar",
    "solubilizábamos": "solubilizar",
    "solubilizáis": "solubilizar",
    "solubilizáramos": "solubilizar",
    "solubilizáremos": "solubilizar",
    "solubilizásemos": "solubilizar",
    "solubilizó": "solubilizar",
    "solubles": "soluble",
    "soluciona": "solucionar",
    "solucionaba": "solucionar",
    "solucionabais": "solucionar",
    "solucionaban": "solucionar",
    "solucionabas": "solucionar",
    "solucionables": "solucionable",
    "solucionad": "solucionar",
    "solucionada": "solucionar",
    "solucionadas": "solucionar",
    "solucionado": "solucionar",
    "solucionados": "solucionar",
    "solucionamos": "solucionar",
    "solucionan": "solucionar",
    "solucionando": "solucionar",
    "solucionara": "solucionar",
    "solucionarais": "solucionar",
    "solucionaran": "solucionar",
    "solucionaras": "solucionar",
    "solucionare": "solucionar",
    "solucionareis": "solucionar",
    "solucionaremos": "solucionar",
    "solucionaren": "solucionar",
    "solucionares": "solucionar",
    "solucionaron": "solucionar",
    "solucionará": "solucionar",
    "solucionarán": "solucionar",
    "solucionarás": "solucionar",
    "solucionaré": "solucionar",
    "solucionaréis": "solucionar",
    "solucionaría": "solucionar",
    "solucionaríais": "solucionar",
    "solucionaríamos": "solucionar",
    "solucionarían": "solucionar",
    "solucionarías": "solucionar",
    "solucionas": "solucionar",
    "solucionase": "solucionar",
    "solucionaseis": "solucionar",
    "solucionasen": "solucionar",
    "solucionases": "solucionar",
    "solucionaste": "solucionar",
    "solucionasteis": "solucionar",
    "solucione": "solucionar",
    "solucionemos": "solucionar",
    "solucionen": "solucionar",
    "soluciones": "solucionar",
    "soluciono": "solucionar",
    "solucionábamos": "solucionar",
    "solucionáis": "solucionar",
    "solucionáramos": "solucionar",
    "solucionáremos": "solucionar",
    "solucionásemos": "solucionar",
    "solucioné": "solucionar",
    "solucionéis": "solucionar",
    "solucionó": "solucionar",
    "solumes": "solum",
    "solutiva": "solutivo",
    "solutivas": "solutivo",
    "solutivos": "solutivo",
    "solutos": "soluto",
    "solutrenses": "solutrense",
    "solvaciones": "solvación",
    "solvataciones": "solvatación",
    "solvatos": "solvato",
    "solvencias": "solvencia",
    "solventa": "solventar",
    "solventaba": "solventar",
    "solventabais": "solventar",
    "solventaban": "solventar",
    "solventabas": "solventar",
    "solventad": "solventar",
    "solventada": "solventar",
    "solventadas": "solventar",
    "solventado": "solventar",
    "solventados": "solventar",
    "solventamos": "solventar",
    "solventan": "solventar",
    "solventando": "solventar",
    "solventara": "solventar",
    "solventarais": "solventar",
    "solventaran": "solventar",
    "solventaras": "solventar",
    "solventare": "solventar",
    "solventareis": "solventar",
    "solventaremos": "solventar",
    "solventaren": "solventar",
    "solventares": "solventar",
    "solventaron": "solventar",
    "solventará": "solventar",
    "solventarán": "solventar",
    "solventarás": "solventar",
    "solventaré": "solventar",
    "solventaréis": "solventar",
    "solventaría": "solventar",
    "solventaríais": "solventar",
    "solventaríamos": "solventar",
    "solventarían": "solventar",
    "solventarías": "solventar",
    "solventas": "solventar",
    "solventase": "solventar",
    "solventaseis": "solventar",
    "solventasen": "solventar",
    "solventases": "solventar",
    "solventaste": "solventar",
    "solventasteis": "solventar",
    "solvente": "solventar",
    "solventemos": "solventar",
    "solventen": "solventar",
    "solventes": "solvente",
    "solvento": "solventar",
    "solventábamos": "solventar",
    "solventáis": "solventar",
    "solventáramos": "solventar",
    "solventáremos": "solventar",
    "solventásemos": "solventar",
    "solventé": "solventar",
    "solventéis": "solventar",
    "solventó": "solventar",
    "solábamos": "solar",
    "soláis": "soler",
    "soláramos": "solar",
    "soláremos": "solar",
    "soláriums": "solárium",
    "solásemos": "solar",
    "solé": "solar",
    "soléis": "soler",
    "solénquimas": "solénquima",
    "solí": "soler",
    "solía": "soler",
    "solíais": "soler",
    "solíamos": "soler",
    "solían": "soler",
    "solías": "soler",
    "solícita": "solícito",
    "solícitas": "solícito",
    "solícitos": "solícito",
    "solífugos": "solífugo",
    "solípeda": "solípedo",
    "solípedas": "solípedo",
    "solípedos": "solípedo",
    "soló": "solar",
    "somalíes": "somalí",
    "somalís": "somalí",
    "somalíses": "somalí",
    "somantas": "somanta",
    "somas": "soma",
    "somatenes": "somatén",
    "somatizaciones": "somatización",
    "somatologías": "somatología",
    "somatometrías": "somatometría",
    "somatoplástica": "somatoplástico",
    "somatoplásticas": "somatoplástico",
    "somatoplásticos": "somatoplástico",
    "somatopsíquica": "somatopsíquico",
    "somatopsíquicas": "somatopsíquico",
    "somatopsíquicos": "somatopsíquico",
    "somatostatinas": "somatostatina",
    "somatotropinas": "somatotropina",
    "sombra": "sombrar",
    "sombraba": "sombrar",
    "sombrabais": "sombrar",
    "sombraban": "sombrar",
    "sombrabas": "sombrar",
    "sombrad": "sombrar",
    "sombrada": "sombrar",
    "sombradas": "sombrar",
    "sombrado": "sombrar",
    "sombrados": "sombrar",
    "sombrajes": "sombraje",
    "sombrajos": "sombrajo",
    "sombramos": "sombrar",
    "sombran": "sombrar",
    "sombrando": "sombrar",
    "sombrara": "sombrar",
    "sombrarais": "sombrar",
    "sombraran": "sombrar",
    "sombraras": "sombrar",
    "sombrare": "sombrar",
    "sombrareis": "sombrar",
    "sombraremos": "sombrar",
    "sombraren": "sombrar",
    "sombrares": "sombrar",
    "sombraron": "sombrar",
    "sombrará": "sombrar",
    "sombrarán": "sombrar",
    "sombrarás": "sombrar",
    "sombraré": "sombrar",
    "sombraréis": "sombrar",
    "sombraría": "sombrar",
    "sombraríais": "sombrar",
    "sombraríamos": "sombrar",
    "sombrarían": "sombrar",
    "sombrarías": "sombrar",
    "sombras": "sombrar",
    "sombrase": "sombrar",
    "sombraseis": "sombrar",
    "sombrasen": "sombrar",
    "sombrases": "sombrar",
    "sombraste": "sombrar",
    "sombrasteis": "sombrar",
    "sombre": "sombrar",
    "sombrea": "sombrear",
    "sombreaba": "sombrear",
    "sombreabais": "sombrear",
    "sombreaban": "sombrear",
    "sombreabas": "sombrear",
    "sombread": "sombrear",
    "sombreada": "sombrear",
    "sombreadas": "sombrear",
    "sombreado": "sombrear",
    "sombreadores": "sombreador",
    "sombreados": "sombrear",
    "sombreamos": "sombrear",
    "sombrean": "sombrear",
    "sombreando": "sombrear",
    "sombreara": "sombrear",
    "sombrearais": "sombrear",
    "sombrearan": "sombrear",
    "sombrearas": "sombrear",
    "sombreare": "sombrear",
    "sombreareis": "sombrear",
    "sombrearemos": "sombrear",
    "sombrearen": "sombrear",
    "sombreares": "sombrear",
    "sombrearon": "sombrear",
    "sombreará": "sombrear",
    "sombrearán": "sombrear",
    "sombrearás": "sombrear",
    "sombrearé": "sombrear",
    "sombrearéis": "sombrear",
    "sombrearía": "sombrear",
    "sombrearíais": "sombrear",
    "sombrearíamos": "sombrear",
    "sombrearían": "sombrear",
    "sombrearías": "sombrear",
    "sombreas": "sombrear",
    "sombrease": "sombrear",
    "sombreaseis": "sombrear",
    "sombreasen": "sombrear",
    "sombreases": "sombrear",
    "sombreaste": "sombrear",
    "sombreasteis": "sombrear",
    "sombree": "sombrear",
    "sombreemos": "sombrear",
    "sombreen": "sombrear",
    "sombrees": "sombrear",
    "sombremos": "sombrar",
    "sombren": "sombrar",
    "sombreo": "sombrear",
    "sombreradas": "sombrerada",
    "sombreras": "sombrera",
    "sombrerazos": "sombrerazo",
    "sombrerera": "sombrerero",
    "sombrereras": "sombrerero",
    "sombrereros": "sombrerero",
    "sombrererías": "sombrerería",
    "sombreretes": "sombrerete",
    "sombrerillos": "sombrerillo",
    "sombreros": "sombrero",
    "sombres": "sombrar",
    "sombreábamos": "sombrear",
    "sombreáis": "sombrear",
    "sombreáramos": "sombrear",
    "sombreáremos": "sombrear",
    "sombreásemos": "sombrear",
    "sombreé": "sombrear",
    "sombreéis": "sombrear",
    "sombreó": "sombrear",
    "sombrillas": "sombrilla",
    "sombrillazos": "sombrillazo",
    "sombro": "sombrar",
    "sombrosa": "sombroso",
    "sombrosas": "sombroso",
    "sombrosos": "sombroso",
    "sombrábamos": "sombrar",
    "sombráis": "sombrar",
    "sombráramos": "sombrar",
    "sombráremos": "sombrar",
    "sombrásemos": "sombrar",
    "sombré": "sombrar",
    "sombréis": "sombrar",
    "sombría": "sombrío",
    "sombrías": "sombrío",
    "sombríos": "sombrío",
    "sombró": "sombrar",
    "somera": "somero",
    "someras": "somero",
    "someros": "somero",
    "someta": "someter",
    "sometamos": "someter",
    "sometan": "someter",
    "sometas": "someter",
    "somete": "someter",
    "someted": "someter",
    "sometemos": "someter",
    "someten": "someter",
    "someteremos": "someter",
    "someterá": "someter",
    "someterán": "someter",
    "someterás": "someter",
    "someteré": "someter",
    "someteréis": "someter",
    "sometería": "someter",
    "someteríais": "someter",
    "someteríamos": "someter",
    "someterían": "someter",
    "someterías": "someter",
    "sometes": "someter",
    "sometida": "someter",
    "sometidas": "someter",
    "sometido": "someter",
    "sometidos": "someter",
    "sometiendo": "someter",
    "sometiera": "someter",
    "sometierais": "someter",
    "sometieran": "someter",
    "sometieras": "someter",
    "sometiere": "someter",
    "sometiereis": "someter",
    "sometieren": "someter",
    "sometieres": "someter",
    "sometieron": "someter",
    "sometiese": "someter",
    "sometieseis": "someter",
    "sometiesen": "someter",
    "sometieses": "someter",
    "sometimientos": "sometimiento",
    "sometimos": "someter",
    "sometiste": "someter",
    "sometisteis": "someter",
    "sometiéramos": "someter",
    "sometiéremos": "someter",
    "sometiésemos": "someter",
    "sometió": "someter",
    "someto": "someter",
    "sometáis": "someter",
    "sometéis": "someter",
    "sometí": "someter",
    "sometía": "someter",
    "sometíais": "someter",
    "sometíamos": "someter",
    "sometían": "someter",
    "sometías": "someter",
    "somieres": "somier",
    "somiers": "somier",
    "sommeliers": "sommelier",
    "somnambulismos": "somnambulismo",
    "somnolencias": "somnolencia",
    "somnolienta": "somnoliento",
    "somnolientas": "somnoliento",
    "somnolientos": "somnoliento",
    "somnámbula": "somnámbulo",
    "somnámbulas": "somnámbulo",
    "somnámbulos": "somnámbulo",
    "somnífera": "somnífero",
    "somníferas": "somnífero",
    "somníferos": "somnífero",
    "somnílocua": "somnílocuo",
    "somnílocuas": "somnílocuo",
    "somnílocuos": "somnílocuo",
    "somontana": "somontano",
    "somontanas": "somontano",
    "somontanos": "somontano",
    "somontes": "somonte",
    "somorguja": "somorgujar",
    "somorgujaba": "somorgujar",
    "somorgujabais": "somorgujar",
    "somorgujaban": "somorgujar",
    "somorgujabas": "somorgujar",
    "somorgujad": "somorgujar",
    "somorgujada": "somorgujar",
    "somorgujadas": "somorgujar",
    "somorgujado": "somorgujar",
    "somorgujadores": "somorgujador",
    "somorgujados": "somorgujar",
    "somorgujamos": "somorgujar",
    "somorgujan": "somorgujar",
    "somorgujando": "somorgujar",
    "somorgujara": "somorgujar",
    "somorgujarais": "somorgujar",
    "somorgujaran": "somorgujar",
    "somorgujaras": "somorgujar",
    "somorgujare": "somorgujar",
    "somorgujareis": "somorgujar",
    "somorgujaremos": "somorgujar",
    "somorgujaren": "somorgujar",
    "somorgujares": "somorgujar",
    "somorgujaron": "somorgujar",
    "somorgujará": "somorgujar",
    "somorgujarán": "somorgujar",
    "somorgujarás": "somorgujar",
    "somorgujaré": "somorgujar",
    "somorgujaréis": "somorgujar",
    "somorgujaría": "somorgujar",
    "somorgujaríais": "somorgujar",
    "somorgujaríamos": "somorgujar",
    "somorgujarían": "somorgujar",
    "somorgujarías": "somorgujar",
    "somorgujas": "somorgujar",
    "somorgujase": "somorgujar",
    "somorgujaseis": "somorgujar",
    "somorgujasen": "somorgujar",
    "somorgujases": "somorgujar",
    "somorgujaste": "somorgujar",
    "somorgujasteis": "somorgujar",
    "somorguje": "somorgujar",
    "somorgujemos": "somorgujar",
    "somorgujen": "somorgujar",
    "somorgujes": "somorgujar",
    "somorgujo": "somorgujar",
    "somorgujábamos": "somorgujar",
    "somorgujáis": "somorgujar",
    "somorgujáramos": "somorgujar",
    "somorgujáremos": "somorgujar",
    "somorgujásemos": "somorgujar",
    "somorgujé": "somorgujar",
    "somorgujéis": "somorgujar",
    "somorgujó": "somorgujar",
    "somormuja": "somormujar",
    "somormujaba": "somormujar",
    "somormujabais": "somormujar",
    "somormujaban": "somormujar",
    "somormujabas": "somormujar",
    "somormujad": "somormujar",
    "somormujada": "somormujar",
    "somormujadas": "somormujar",
    "somormujado": "somormujar",
    "somormujados": "somormujar",
    "somormujamos": "somormujar",
    "somormujan": "somormujar",
    "somormujando": "somormujar",
    "somormujara": "somormujar",
    "somormujarais": "somormujar",
    "somormujaran": "somormujar",
    "somormujaras": "somormujar",
    "somormujare": "somormujar",
    "somormujareis": "somormujar",
    "somormujaremos": "somormujar",
    "somormujaren": "somormujar",
    "somormujares": "somormujar",
    "somormujaron": "somormujar",
    "somormujará": "somormujar",
    "somormujarán": "somormujar",
    "somormujarás": "somormujar",
    "somormujaré": "somormujar",
    "somormujaréis": "somormujar",
    "somormujaría": "somormujar",
    "somormujaríais": "somormujar",
    "somormujaríamos": "somormujar",
    "somormujarían": "somormujar",
    "somormujarías": "somormujar",
    "somormujas": "somormujar",
    "somormujase": "somormujar",
    "somormujaseis": "somormujar",
    "somormujasen": "somormujar",
    "somormujases": "somormujar",
    "somormujaste": "somormujar",
    "somormujasteis": "somormujar",
    "somormuje": "somormujar",
    "somormujemos": "somormujar",
    "somormujen": "somormujar",
    "somormujes": "somormujar",
    "somormujo": "somormujar",
    "somormujos": "somormujo",
    "somormujábamos": "somormujar",
    "somormujáis": "somormujar",
    "somormujáramos": "somormujar",
    "somormujáremos": "somormujar",
    "somormujásemos": "somormujar",
    "somormujé": "somormujar",
    "somormujéis": "somormujar",
    "somormujó": "somormujar",
    "somos": "ser",
    "somática": "somático",
    "somáticas": "somático",
    "somáticos": "somático",
    "son": "ser",
    "sonaba": "sonar",
    "sonabais": "sonar",
    "sonaban": "sonar",
    "sonabas": "sonar",
    "sonad": "sonar",
    "sonada": "sonar",
    "sonadas": "sonar",
    "sonaderas": "sonadera",
    "sonaderos": "sonadero",
    "sonado": "sonar",
    "sonadora": "sonador",
    "sonadoras": "sonador",
    "sonadores": "sonador",
    "sonados": "sonar",
    "sonajas": "sonaja",
    "sonajeros": "sonajero",
    "sonambulismos": "sonambulismo",
    "sonamos": "sonar",
    "sonando": "sonar",
    "sonantes": "sonante",
    "sonara": "sonar",
    "sonarais": "sonar",
    "sonaran": "sonar",
    "sonaras": "sonar",
    "sonare": "sonar",
    "sonareis": "sonar",
    "sonaremos": "sonar",
    "sonaren": "sonar",
    "sonares": "sonar",
    "sonaron": "sonar",
    "sonará": "sonar",
    "sonarán": "sonar",
    "sonarás": "sonar",
    "sonaré": "sonar",
    "sonaréis": "sonar",
    "sonaría": "sonar",
    "sonaríais": "sonar",
    "sonaríamos": "sonar",
    "sonarían": "sonar",
    "sonarías": "sonar",
    "sonase": "sonar",
    "sonaseis": "sonar",
    "sonasen": "sonar",
    "sonases": "sonar",
    "sonaste": "sonar",
    "sonasteis": "sonar",
    "sonatas": "sonata",
    "sonatinas": "sonatina",
    "sonda": "sondar",
    "sondaba": "sondar",
    "sondabais": "sondar",
    "sondaban": "sondar",
    "sondabas": "sondar",
    "sondables": "sondable",
    "sondad": "sondar",
    "sondada": "sondar",
    "sondadas": "sondar",
    "sondado": "sondar",
    "sondadores": "sondador",
    "sondados": "sondar",
    "sondajes": "sondaje",
    "sondalezas": "sondaleza",
    "sondamos": "sondar",
    "sondan": "sondar",
    "sondando": "sondar",
    "sondara": "sondar",
    "sondarais": "sondar",
    "sondaran": "sondar",
    "sondaras": "sondar",
    "sondare": "sondar",
    "sondareis": "sondar",
    "sondaremos": "sondar",
    "sondaren": "sondar",
    "sondares": "sondar",
    "sondaron": "sondar",
    "sondará": "sondar",
    "sondarán": "sondar",
    "sondarás": "sondar",
    "sondaré": "sondar",
    "sondaréis": "sondar",
    "sondaría": "sondar",
    "sondaríais": "sondar",
    "sondaríamos": "sondar",
    "sondarían": "sondar",
    "sondarías": "sondar",
    "sondas": "sondar",
    "sondase": "sondar",
    "sondaseis": "sondar",
    "sondasen": "sondar",
    "sondases": "sondar",
    "sondaste": "sondar",
    "sondasteis": "sondar",
    "sonde": "sondar",
    "sondea": "sondear",
    "sondeaba": "sondear",
    "sondeabais": "sondear",
    "sondeaban": "sondear",
    "sondeabas": "sondear",
    "sondead": "sondear",
    "sondeada": "sondear",
    "sondeadas": "sondear",
    "sondeado": "sondear",
    "sondeadores": "sondeador",
    "sondeados": "sondear",
    "sondeamos": "sondear",
    "sondean": "sondear",
    "sondeando": "sondear",
    "sondeara": "sondear",
    "sondearais": "sondear",
    "sondearan": "sondear",
    "sondearas": "sondear",
    "sondeare": "sondear",
    "sondeareis": "sondear",
    "sondearemos": "sondear",
    "sondearen": "sondear",
    "sondeares": "sondear",
    "sondearon": "sondear",
    "sondeará": "sondear",
    "sondearán": "sondear",
    "sondearás": "sondear",
    "sondearé": "sondear",
    "sondearéis": "sondear",
    "sondearía": "sondear",
    "sondearíais": "sondear",
    "sondearíamos": "sondear",
    "sondearían": "sondear",
    "sondearías": "sondear",
    "sondeas": "sondear",
    "sondease": "sondear",
    "sondeaseis": "sondear",
    "sondeasen": "sondear",
    "sondeases": "sondear",
    "sondeaste": "sondear",
    "sondeasteis": "sondear",
    "sondee": "sondear",
    "sondeemos": "sondear",
    "sondeen": "sondear",
    "sondees": "sondear",
    "sondemos": "sondar",
    "sonden": "sondar",
    "sondeo": "sondear",
    "sondeos": "sondeo",
    "sondes": "sondar",
    "sondeábamos": "sondear",
    "sondeáis": "sondear",
    "sondeáramos": "sondear",
    "sondeáremos": "sondear",
    "sondeásemos": "sondear",
    "sondeé": "sondear",
    "sondeéis": "sondear",
    "sondeó": "sondear",
    "sondistas": "sondista",
    "sondo": "sondar",
    "sondábamos": "sondar",
    "sondáis": "sondar",
    "sondáramos": "sondar",
    "sondáremos": "sondar",
    "sondásemos": "sondar",
    "sondé": "sondar",
    "sondéis": "sondar",
    "sondó": "sondar",
    "sonemos": "sonar",
    "sones": "son",
    "soneticos": "sonetico",
    "sonetillos": "sonetillo",
    "sonetistas": "sonetista",
    "sonetos": "soneto",
    "songhayes": "songhay",
    "songhays": "songhay",
    "sonidos": "sonido",
    "soniquetes": "soniquete",
    "sonlocada": "sonlocado",
    "sonlocadas": "sonlocado",
    "sonlocados": "sonlocado",
    "sonochadas": "sonochada",
    "sonometrías": "sonometría",
    "sonométrica": "sonométrico",
    "sonométricas": "sonométrico",
    "sonométricos": "sonométrico",
    "sonora": "sonoro",
    "sonoras": "sonoro",
    "sonorenses": "sonorense",
    "sonorice": "sonorizar",
    "sonoricemos": "sonorizar",
    "sonoricen": "sonorizar",
    "sonorices": "sonorizar",
    "sonoricé": "sonorizar",
    "sonoricéis": "sonorizar",
    "sonoridades": "sonoridad",
    "sonoriza": "sonorizar",
    "sonorizaba": "sonorizar",
    "sonorizabais": "sonorizar",
    "sonorizaban": "sonorizar",
    "sonorizabas": "sonorizar",
    "sonorizaciones": "sonorización",
    "sonorizad": "sonorizar",
    "sonorizada": "sonorizar",
    "sonorizadas": "sonorizar",
    "sonorizado": "sonorizar",
    "sonorizados": "sonorizar",
    "sonorizamos": "sonorizar",
    "sonorizan": "sonorizar",
    "sonorizando": "sonorizar",
    "sonorizara": "sonorizar",
    "sonorizarais": "sonorizar",
    "sonorizaran": "sonorizar",
    "sonorizaras": "sonorizar",
    "sonorizare": "sonorizar",
    "sonorizareis": "sonorizar",
    "sonorizaremos": "sonorizar",
    "sonorizaren": "sonorizar",
    "sonorizares": "sonorizar",
    "sonorizaron": "sonorizar",
    "sonorizará": "sonorizar",
    "sonorizarán": "sonorizar",
    "sonorizarás": "sonorizar",
    "sonorizaré": "sonorizar",
    "sonorizaréis": "sonorizar",
    "sonorizaría": "sonorizar",
    "sonorizaríais": "sonorizar",
    "sonorizaríamos": "sonorizar",
    "sonorizarían": "sonorizar",
    "sonorizarías": "sonorizar",
    "sonorizas": "sonorizar",
    "sonorizase": "sonorizar",
    "sonorizaseis": "sonorizar",
    "sonorizasen": "sonorizar",
    "sonorizases": "sonorizar",
    "sonorizaste": "sonorizar",
    "sonorizasteis": "sonorizar",
    "sonorizo": "sonorizar",
    "sonorizábamos": "sonorizar",
    "sonorizáis": "sonorizar",
    "sonorizáramos": "sonorizar",
    "sonorizáremos": "sonorizar",
    "sonorizásemos": "sonorizar",
    "sonorizó": "sonorizar",
    "sonoros": "sonoro",
    "sonreiremos": "sonreír",
    "sonreirá": "sonreír",
    "sonreirán": "sonreír",
    "sonreirás": "sonreír",
    "sonreiré": "sonreír",
    "sonreiréis": "sonreír",
    "sonreiría": "sonreír",
    "sonreiríais": "sonreír",
    "sonreiríamos": "sonreír",
    "sonreirían": "sonreír",
    "sonreirías": "sonreír",
    "sonreí": "sonreír",
    "sonreía": "sonreír",
    "sonreíais": "sonreír",
    "sonreíamos": "sonreír",
    "sonreían": "sonreír",
    "sonreías": "sonreír",
    "sonreíd": "sonreír",
    "sonreída": "sonreír",
    "sonreídas": "sonreír",
    "sonreído": "sonreír",
    "sonreídos": "sonreír",
    "sonreímos": "sonreír",
    "sonreís": "sonreír",
    "sonreíste": "sonreír",
    "sonreísteis": "sonreír",
    "sonriamos": "sonreír",
    "sonriendo": "sonreír",
    "sonrientes": "sonriente",
    "sonriera": "sonreír",
    "sonrierais": "sonreír",
    "sonrieran": "sonreír",
    "sonrieras": "sonreír",
    "sonriere": "sonreír",
    "sonriereis": "sonreír",
    "sonrieren": "sonreír",
    "sonrieres": "sonreír",
    "sonrieron": "sonreír",
    "sonriese": "sonreír",
    "sonrieseis": "sonreír",
    "sonriesen": "sonreír",
    "sonrieses": "sonreír",
    "sonrisas": "sonrisa",
    "sonriáis": "sonreír",
    "sonriéramos": "sonreír",
    "sonriéremos": "sonreír",
    "sonriésemos": "sonreír",
    "sonrió": "sonreír",
    "sonroja": "sonrojar",
    "sonrojaba": "sonrojar",
    "sonrojabais": "sonrojar",
    "sonrojaban": "sonrojar",
    "sonrojabas": "sonrojar",
    "sonrojad": "sonrojar",
    "sonrojada": "sonrojar",
    "sonrojadas": "sonrojar",
    "sonrojado": "sonrojar",
    "sonrojados": "sonrojar",
    "sonrojamos": "sonrojar",
    "sonrojan": "sonrojar",
    "sonrojando": "sonrojar",
    "sonrojara": "sonrojar",
    "sonrojarais": "sonrojar",
    "sonrojaran": "sonrojar",
    "sonrojaras": "sonrojar",
    "sonrojare": "sonrojar",
    "sonrojareis": "sonrojar",
    "sonrojaremos": "sonrojar",
    "sonrojaren": "sonrojar",
    "sonrojares": "sonrojar",
    "sonrojaron": "sonrojar",
    "sonrojará": "sonrojar",
    "sonrojarán": "sonrojar",
    "sonrojarás": "sonrojar",
    "sonrojaré": "sonrojar",
    "sonrojaréis": "sonrojar",
    "sonrojaría": "sonrojar",
    "sonrojaríais": "sonrojar",
    "sonrojaríamos": "sonrojar",
    "sonrojarían": "sonrojar",
    "sonrojarías": "sonrojar",
    "sonrojas": "sonrojar",
    "sonrojase": "sonrojar",
    "sonrojaseis": "sonrojar",
    "sonrojasen": "sonrojar",
    "sonrojases": "sonrojar",
    "sonrojaste": "sonrojar",
    "sonrojasteis": "sonrojar",
    "sonroje": "sonrojar",
    "sonrojemos": "sonrojar",
    "sonrojen": "sonrojar",
    "sonrojes": "sonrojar",
    "sonrojo": "sonrojar",
    "sonrojos": "sonrojo",
    "sonrojábamos": "sonrojar",
    "sonrojáis": "sonrojar",
    "sonrojáramos": "sonrojar",
    "sonrojáremos": "sonrojar",
    "sonrojásemos": "sonrojar",
    "sonrojé": "sonrojar",
    "sonrojéis": "sonrojar",
    "sonrojó": "sonrojar",
    "sonrosa": "sonrosar",
    "sonrosaba": "sonrosar",
    "sonrosabais": "sonrosar",
    "sonrosaban": "sonrosar",
    "sonrosabas": "sonrosar",
    "sonrosad": "sonrosar",
    "sonrosada": "sonrosar",
    "sonrosadas": "sonrosar",
    "sonrosado": "sonrosar",
    "sonrosados": "sonrosar",
    "sonrosamos": "sonrosar",
    "sonrosan": "sonrosar",
    "sonrosando": "sonrosar",
    "sonrosara": "sonrosar",
    "sonrosarais": "sonrosar",
    "sonrosaran": "sonrosar",
    "sonrosaras": "sonrosar",
    "sonrosare": "sonrosar",
    "sonrosareis": "sonrosar",
    "sonrosaremos": "sonrosar",
    "sonrosaren": "sonrosar",
    "sonrosares": "sonrosar",
    "sonrosaron": "sonrosar",
    "sonrosará": "sonrosar",
    "sonrosarán": "sonrosar",
    "sonrosarás": "sonrosar",
    "sonrosaré": "sonrosar",
    "sonrosaréis": "sonrosar",
    "sonrosaría": "sonrosar",
    "sonrosaríais": "sonrosar",
    "sonrosaríamos": "sonrosar",
    "sonrosarían": "sonrosar",
    "sonrosarías": "sonrosar",
    "sonrosas": "sonrosar",
    "sonrosase": "sonrosar",
    "sonrosaseis": "sonrosar",
    "sonrosasen": "sonrosar",
    "sonrosases": "sonrosar",
    "sonrosaste": "sonrosar",
    "sonrosasteis": "sonrosar",
    "sonrose": "sonrosar",
    "sonrosea": "sonrosear",
    "sonroseaba": "sonrosear",
    "sonroseabais": "sonrosear",
    "sonroseaban": "sonrosear",
    "sonroseabas": "sonrosear",
    "sonrosead": "sonrosear",
    "sonroseada": "sonrosear",
    "sonroseadas": "sonrosear",
    "sonroseado": "sonrosear",
    "sonroseados": "sonrosear",
    "sonroseamos": "sonrosear",
    "sonrosean": "sonrosear",
    "sonroseando": "sonrosear",
    "sonroseara": "sonrosear",
    "sonrosearais": "sonrosear",
    "sonrosearan": "sonrosear",
    "sonrosearas": "sonrosear",
    "sonroseare": "sonrosear",
    "sonroseareis": "sonrosear",
    "sonrosearemos": "sonrosear",
    "sonrosearen": "sonrosear",
    "sonroseares": "sonrosear",
    "sonrosearon": "sonrosear",
    "sonroseará": "sonrosear",
    "sonrosearán": "sonrosear",
    "sonrosearás": "sonrosear",
    "sonrosearé": "sonrosear",
    "sonrosearéis": "sonrosear",
    "sonrosearía": "sonrosear",
    "sonrosearíais": "sonrosear",
    "sonrosearíamos": "sonrosear",
    "sonrosearían": "sonrosear",
    "sonrosearías": "sonrosear",
    "sonroseas": "sonrosear",
    "sonrosease": "sonrosear",
    "sonroseaseis": "sonrosear",
    "sonroseasen": "sonrosear",
    "sonroseases": "sonrosear",
    "sonroseaste": "sonrosear",
    "sonroseasteis": "sonrosear",
    "sonrosee": "sonrosear",
    "sonroseemos": "sonrosear",
    "sonroseen": "sonrosear",
    "sonrosees": "sonrosear",
    "sonrosemos": "sonrosar",
    "sonrosen": "sonrosar",
    "sonroseo": "sonrosear",
    "sonroseos": "sonroseo",
    "sonroses": "sonrosar",
    "sonroseábamos": "sonrosear",
    "sonroseáis": "sonrosear",
    "sonroseáramos": "sonrosear",
    "sonroseáremos": "sonrosear",
    "sonroseásemos": "sonrosear",
    "sonroseé": "sonrosear",
    "sonroseéis": "sonrosear",
    "sonroseó": "sonrosear",
    "sonroso": "sonrosar",
    "sonrosábamos": "sonrosar",
    "sonrosáis": "sonrosar",
    "sonrosáramos": "sonrosar",
    "sonrosáremos": "sonrosar",
    "sonrosásemos": "sonrosar",
    "sonrosé": "sonrosar",
    "sonroséis": "sonrosar",
    "sonrosó": "sonrosar",
    "sonría": "sonreír",
    "sonrían": "sonreír",
    "sonrías": "sonreír",
    "sonríe": "sonreír",
    "sonríen": "sonreír",
    "sonríes": "sonreír",
    "sonrío": "sonreír",
    "sons": "son",
    "sonsaca": "sonsacar",
    "sonsacaba": "sonsacar",
    "sonsacabais": "sonsacar",
    "sonsacaban": "sonsacar",
    "sonsacabas": "sonsacar",
    "sonsacad": "sonsacar",
    "sonsacada": "sonsacar",
    "sonsacadas": "sonsacar",
    "sonsacado": "sonsacar",
    "sonsacadora": "sonsacador",
    "sonsacadoras": "sonsacador",
    "sonsacadores": "sonsacador",
    "sonsacados": "sonsacar",
    "sonsacamientos": "sonsacamiento",
    "sonsacamos": "sonsacar",
    "sonsacan": "sonsacar",
    "sonsacando": "sonsacar",
    "sonsacara": "sonsacar",
    "sonsacarais": "sonsacar",
    "sonsacaran": "sonsacar",
    "sonsacaras": "sonsacar",
    "sonsacare": "sonsacar",
    "sonsacareis": "sonsacar",
    "sonsacaremos": "sonsacar",
    "sonsacaren": "sonsacar",
    "sonsacares": "sonsacar",
    "sonsacaron": "sonsacar",
    "sonsacará": "sonsacar",
    "sonsacarán": "sonsacar",
    "sonsacarás": "sonsacar",
    "sonsacaré": "sonsacar",
    "sonsacaréis": "sonsacar",
    "sonsacaría": "sonsacar",
    "sonsacaríais": "sonsacar",
    "sonsacaríamos": "sonsacar",
    "sonsacarían": "sonsacar",
    "sonsacarías": "sonsacar",
    "sonsacas": "sonsacar",
    "sonsacase": "sonsacar",
    "sonsacaseis": "sonsacar",
    "sonsacasen": "sonsacar",
    "sonsacases": "sonsacar",
    "sonsacaste": "sonsacar",
    "sonsacasteis": "sonsacar",
    "sonsaco": "sonsacar",
    "sonsacábamos": "sonsacar",
    "sonsacáis": "sonsacar",
    "sonsacáramos": "sonsacar",
    "sonsacáremos": "sonsacar",
    "sonsacásemos": "sonsacar",
    "sonsacó": "sonsacar",
    "sonsaque": "sonsacar",
    "sonsaquemos": "sonsacar",
    "sonsaquen": "sonsacar",
    "sonsaques": "sonsaque",
    "sonsaqué": "sonsacar",
    "sonsaquéis": "sonsacar",
    "sonsonetes": "sonsonete",
    "sonsos": "sonso",
    "sonábamos": "sonar",
    "sonáis": "sonar",
    "sonámbula": "sonámbulo",
    "sonámbulas": "sonámbulo",
    "sonámbulos": "sonámbulo",
    "sonántica": "sonántico",
    "sonánticas": "sonántico",
    "sonánticos": "sonántico",
    "sonáramos": "sonar",
    "sonáremos": "sonar",
    "sonásemos": "sonar",
    "soné": "sonar",
    "sonéis": "sonar",
    "sonó": "sonar",
    "sonógrafos": "sonógrafo",
    "sonómetros": "sonómetro",
    "sopa": "sopar",
    "sopaba": "sopar",
    "sopabais": "sopar",
    "sopaban": "sopar",
    "sopabas": "sopar",
    "sopad": "sopar",
    "sopada": "sopar",
    "sopadas": "sopar",
    "sopado": "sopar",
    "sopados": "sopar",
    "sopamos": "sopar",
    "sopan": "sopar",
    "sopandas": "sopanda",
    "sopando": "sopar",
    "sopapea": "sopapear",
    "sopapeaba": "sopapear",
    "sopapeabais": "sopapear",
    "sopapeaban": "sopapear",
    "sopapeabas": "sopapear",
    "sopapead": "sopapear",
    "sopapeada": "sopapear",
    "sopapeadas": "sopapear",
    "sopapeado": "sopapear",
    "sopapeados": "sopapear",
    "sopapeamos": "sopapear",
    "sopapean": "sopapear",
    "sopapeando": "sopapear",
    "sopapeara": "sopapear",
    "sopapearais": "sopapear",
    "sopapearan": "sopapear",
    "sopapearas": "sopapear",
    "sopapeare": "sopapear",
    "sopapeareis": "sopapear",
    "sopapearemos": "sopapear",
    "sopapearen": "sopapear",
    "sopapeares": "sopapear",
    "sopapearon": "sopapear",
    "sopapeará": "sopapear",
    "sopapearán": "sopapear",
    "sopapearás": "sopapear",
    "sopapearé": "sopapear",
    "sopapearéis": "sopapear",
    "sopapearía": "sopapear",
    "sopapearíais": "sopapear",
    "sopapearíamos": "sopapear",
    "sopapearían": "sopapear",
    "sopapearías": "sopapear",
    "sopapeas": "sopapear",
    "sopapease": "sopapear",
    "sopapeaseis": "sopapear",
    "sopapeasen": "sopapear",
    "sopapeases": "sopapear",
    "sopapeaste": "sopapear",
    "sopapeasteis": "sopapear",
    "sopapee": "sopapear",
    "sopapeemos": "sopapear",
    "sopapeen": "sopapear",
    "sopapees": "sopapear",
    "sopapeo": "sopapear",
    "sopapeábamos": "sopapear",
    "sopapeáis": "sopapear",
    "sopapeáramos": "sopapear",
    "sopapeáremos": "sopapear",
    "sopapeásemos": "sopapear",
    "sopapeé": "sopapear",
    "sopapeéis": "sopapear",
    "sopapeó": "sopapear",
    "sopapinas": "sopapina",
    "sopapos": "sopapo",
    "sopara": "sopar",
    "soparais": "sopar",
    "soparan": "sopar",
    "soparas": "sopar",
    "sopare": "sopar",
    "sopareis": "sopar",
    "soparemos": "sopar",
    "soparen": "sopar",
    "sopares": "sopar",
    "soparon": "sopar",
    "sopará": "sopar",
    "soparán": "sopar",
    "soparás": "sopar",
    "soparé": "sopar",
    "soparéis": "sopar",
    "soparía": "sopar",
    "soparíais": "sopar",
    "soparíamos": "sopar",
    "soparían": "sopar",
    "soparías": "sopar",
    "sopas": "sopar",
    "sopase": "sopar",
    "sopaseis": "sopar",
    "sopasen": "sopar",
    "sopases": "sopar",
    "sopaste": "sopar",
    "sopasteis": "sopar",
    "sope": "sopar",
    "sopemos": "sopar",
    "sopen": "sopar",
    "sopera": "sopero",
    "soperas": "sopero",
    "soperos": "sopero",
    "sopes": "sopar",
    "sopesa": "sopesar",
    "sopesaba": "sopesar",
    "sopesabais": "sopesar",
    "sopesaban": "sopesar",
    "sopesabas": "sopesar",
    "sopesad": "sopesar",
    "sopesada": "sopesar",
    "sopesadas": "sopesar",
    "sopesado": "sopesar",
    "sopesados": "sopesar",
    "sopesamos": "sopesar",
    "sopesan": "sopesar",
    "sopesando": "sopesar",
    "sopesara": "sopesar",
    "sopesarais": "sopesar",
    "sopesaran": "sopesar",
    "sopesaras": "sopesar",
    "sopesare": "sopesar",
    "sopesareis": "sopesar",
    "sopesaremos": "sopesar",
    "sopesaren": "sopesar",
    "sopesares": "sopesar",
    "sopesaron": "sopesar",
    "sopesará": "sopesar",
    "sopesarán": "sopesar",
    "sopesarás": "sopesar",
    "sopesaré": "sopesar",
    "sopesaréis": "sopesar",
    "sopesaría": "sopesar",
    "sopesaríais": "sopesar",
    "sopesaríamos": "sopesar",
    "sopesarían": "sopesar",
    "sopesarías": "sopesar",
    "sopesas": "sopesar",
    "sopesase": "sopesar",
    "sopesaseis": "sopesar",
    "sopesasen": "sopesar",
    "sopesases": "sopesar",
    "sopesaste": "sopesar",
    "sopesasteis": "sopesar",
    "sopese": "sopesar",
    "sopesemos": "sopesar",
    "sopesen": "sopesar",
    "sopeses": "sopesar",
    "sopeso": "sopesar",
    "sopesábamos": "sopesar",
    "sopesáis": "sopesar",
    "sopesáramos": "sopesar",
    "sopesáremos": "sopesar",
    "sopesásemos": "sopesar",
    "sopesé": "sopesar",
    "sopeséis": "sopesar",
    "sopesó": "sopesar",
    "sopetea": "sopetear",
    "sopeteaba": "sopetear",
    "sopeteabais": "sopetear",
    "sopeteaban": "sopetear",
    "sopeteabas": "sopetear",
    "sopetead": "sopetear",
    "sopeteada": "sopetear",
    "sopeteadas": "sopetear",
    "sopeteado": "sopetear",
    "sopeteados": "sopetear",
    "sopeteamos": "sopetear",
    "sopetean": "sopetear",
    "sopeteando": "sopetear",
    "sopeteara": "sopetear",
    "sopetearais": "sopetear",
    "sopetearan": "sopetear",
    "sopetearas": "sopetear",
    "sopeteare": "sopetear",
    "sopeteareis": "sopetear",
    "sopetearemos": "sopetear",
    "sopetearen": "sopetear",
    "sopeteares": "sopetear",
    "sopetearon": "sopetear",
    "sopeteará": "sopetear",
    "sopetearán": "sopetear",
    "sopetearás": "sopetear",
    "sopetearé": "sopetear",
    "sopetearéis": "sopetear",
    "sopetearía": "sopetear",
    "sopetearíais": "sopetear",
    "sopetearíamos": "sopetear",
    "sopetearían": "sopetear",
    "sopetearías": "sopetear",
    "sopeteas": "sopetear",
    "sopetease": "sopetear",
    "sopeteaseis": "sopetear",
    "sopeteasen": "sopetear",
    "sopeteases": "sopetear",
    "sopeteaste": "sopetear",
    "sopeteasteis": "sopetear",
    "sopetee": "sopetear",
    "sopeteemos": "sopetear",
    "sopeteen": "sopetear",
    "sopetees": "sopetear",
    "sopeteo": "sopetear",
    "sopeteos": "sopeteo",
    "sopeteábamos": "sopetear",
    "sopeteáis": "sopetear",
    "sopeteáramos": "sopetear",
    "sopeteáremos": "sopetear",
    "sopeteásemos": "sopetear",
    "sopeteé": "sopetear",
    "sopeteéis": "sopetear",
    "sopeteó": "sopetear",
    "sopetones": "sopetón",
    "sopeñas": "sopeña",
    "sopicaldos": "sopicaldo",
    "sopitipandos": "sopitipando",
    "sopla": "soplar",
    "soplaba": "soplar",
    "soplabais": "soplar",
    "soplaban": "soplar",
    "soplabas": "soplar",
    "soplad": "soplar",
    "soplada": "soplar",
    "sopladas": "soplar",
    "sopladera": "sopladero",
    "sopladeras": "sopladero",
    "sopladeros": "sopladero",
    "soplado": "soplar",
    "sopladora": "soplador",
    "sopladoras": "soplador",
    "sopladores": "soplador",
    "soplados": "soplar",
    "sopladuras": "sopladura",
    "soplamos": "soplar",
    "soplan": "soplar",
    "soplando": "soplar",
    "soplantes": "soplante",
    "soplara": "soplar",
    "soplarais": "soplar",
    "soplaran": "soplar",
    "soplaras": "soplar",
    "soplare": "soplar",
    "soplareis": "soplar",
    "soplaremos": "soplar",
    "soplaren": "soplar",
    "soplares": "soplar",
    "soplaron": "soplar",
    "soplará": "soplar",
    "soplarán": "soplar",
    "soplarás": "soplar",
    "soplaré": "soplar",
    "soplaréis": "soplar",
    "soplaría": "soplar",
    "soplaríais": "soplar",
    "soplaríamos": "soplar",
    "soplarían": "soplar",
    "soplarías": "soplar",
    "soplas": "soplar",
    "soplase": "soplar",
    "soplaseis": "soplar",
    "soplasen": "soplar",
    "soplases": "soplar",
    "soplaste": "soplar",
    "soplasteis": "soplar",
    "sople": "soplar",
    "soplemos": "soplar",
    "soplen": "soplar",
    "soples": "soplar",
    "sopletes": "soplete",
    "soplidos": "soplido",
    "soplillos": "soplillo",
    "soplo": "soplar",
    "soplona": "soplón",
    "soplonas": "soplón",
    "soplonea": "soplonear",
    "soploneaba": "soplonear",
    "soploneabais": "soplonear",
    "soploneaban": "soplonear",
    "soploneabas": "soplonear",
    "soplonead": "soplonear",
    "soploneada": "soplonear",
    "soploneadas": "soplonear",
    "soploneado": "soplonear",
    "soploneados": "soplonear",
    "soploneamos": "soplonear",
    "soplonean": "soplonear",
    "soploneando": "soplonear",
    "soploneara": "soplonear",
    "soplonearais": "soplonear",
    "soplonearan": "soplonear",
    "soplonearas": "soplonear",
    "soploneare": "soplonear",
    "soploneareis": "soplonear",
    "soplonearemos": "soplonear",
    "soplonearen": "soplonear",
    "soploneares": "soplonear",
    "soplonearon": "soplonear",
    "soploneará": "soplonear",
    "soplonearán": "soplonear",
    "soplonearás": "soplonear",
    "soplonearé": "soplonear",
    "soplonearéis": "soplonear",
    "soplonearía": "soplonear",
    "soplonearíais": "soplonear",
    "soplonearíamos": "soplonear",
    "soplonearían": "soplonear",
    "soplonearías": "soplonear",
    "soploneas": "soplonear",
    "soplonease": "soplonear",
    "soploneaseis": "soplonear",
    "soploneasen": "soplonear",
    "soploneases": "soplonear",
    "soploneaste": "soplonear",
    "soploneasteis": "soplonear",
    "soplonee": "soplonear",
    "soploneemos": "soplonear",
    "soploneen": "soplonear",
    "soplonees": "soplonear",
    "soploneo": "soplonear",
    "soplonerías": "soplonería",
    "soplones": "soplón",
    "soploneábamos": "soplonear",
    "soploneáis": "soplonear",
    "soploneáramos": "soplonear",
    "soploneáremos": "soplonear",
    "soploneásemos": "soplonear",
    "soploneé": "soplonear",
    "soploneéis": "soplonear",
    "soploneó": "soplonear",
    "soplos": "soplo",
    "soplábamos": "soplar",
    "sopláis": "soplar",
    "sopláramos": "soplar",
    "sopláremos": "soplar",
    "soplásemos": "soplar",
    "soplé": "soplar",
    "sopléis": "soplar",
    "sopló": "soplar",
    "sopo": "sopar",
    "soponcios": "soponcio",
    "sopores": "sopor",
    "soporosa": "soporoso",
    "soporosas": "soporoso",
    "soporosos": "soporoso",
    "soporta": "soportar",
    "soportaba": "soportar",
    "soportabais": "soportar",
    "soportaban": "soportar",
    "soportabas": "soportar",
    "soportables": "soportable",
    "soportad": "soportar",
    "soportada": "soportar",
    "soportadas": "soportar",
    "soportado": "soportar",
    "soportadora": "soportador",
    "soportadoras": "soportador",
    "soportadores": "soportador",
    "soportados": "soportar",
    "soportales": "soportal",
    "soportamos": "soportar",
    "soportan": "soportar",
    "soportando": "soportar",
    "soportara": "soportar",
    "soportarais": "soportar",
    "soportaran": "soportar",
    "soportaras": "soportar",
    "soportare": "soportar",
    "soportareis": "soportar",
    "soportaremos": "soportar",
    "soportaren": "soportar",
    "soportares": "soportar",
    "soportaron": "soportar",
    "soportará": "soportar",
    "soportarán": "soportar",
    "soportarás": "soportar",
    "soportaré": "soportar",
    "soportaréis": "soportar",
    "soportaría": "soportar",
    "soportaríais": "soportar",
    "soportaríamos": "soportar",
    "soportarían": "soportar",
    "soportarías": "soportar",
    "soportas": "soportar",
    "soportase": "soportar",
    "soportaseis": "soportar",
    "soportasen": "soportar",
    "soportases": "soportar",
    "soportaste": "soportar",
    "soportasteis": "soportar",
    "soporte": "soportar",
    "soportemos": "soportar",
    "soporten": "soportar",
    "soportes": "soporte",
    "soporto": "soportar",
    "soportábamos": "soportar",
    "soportáis": "soportar",
    "soportáramos": "soportar",
    "soportáremos": "soportar",
    "soportásemos": "soportar",
    "soporté": "soportar",
    "soportéis": "soportar",
    "soportó": "soportar",
    "soporífera": "soporífero",
    "soporíferas": "soporífero",
    "soporíferos": "soporífero",
    "sopranos": "soprano",
    "sopábamos": "sopar",
    "sopáis": "sopar",
    "sopáramos": "sopar",
    "sopáremos": "sopar",
    "sopásemos": "sopar",
    "sopé": "sopar",
    "sopéis": "sopar",
    "sopó": "sopar",
    "sorba": "sorber",
    "sorbamos": "sorber",
    "sorban": "sorber",
    "sorbas": "sorber",
    "sorbe": "sorber",
    "sorbed": "sorber",
    "sorbemos": "sorber",
    "sorben": "sorber",
    "sorberemos": "sorber",
    "sorberá": "sorber",
    "sorberán": "sorber",
    "sorberás": "sorber",
    "sorberé": "sorber",
    "sorberéis": "sorber",
    "sorbería": "sorber",
    "sorberíais": "sorber",
    "sorberíamos": "sorber",
    "sorberían": "sorber",
    "sorberías": "sorber",
    "sorbes": "sorber",
    "sorbetes": "sorbete",
    "sorbibles": "sorbible",
    "sorbida": "sorber",
    "sorbidas": "sorber",
    "sorbido": "sorber",
    "sorbidos": "sorber",
    "sorbiendo": "sorber",
    "sorbiera": "sorber",
    "sorbierais": "sorber",
    "sorbieran": "sorber",
    "sorbieras": "sorber",
    "sorbiere": "sorber",
    "sorbiereis": "sorber",
    "sorbieren": "sorber",
    "sorbieres": "sorber",
    "sorbieron": "sorber",
    "sorbiese": "sorber",
    "sorbieseis": "sorber",
    "sorbiesen": "sorber",
    "sorbieses": "sorber",
    "sorbimos": "sorber",
    "sorbiste": "sorber",
    "sorbisteis": "sorber",
    "sorbitas": "sorbita",
    "sorbitoles": "sorbitol",
    "sorbiéramos": "sorber",
    "sorbiéremos": "sorber",
    "sorbiésemos": "sorber",
    "sorbió": "sorber",
    "sorbo": "sorber",
    "sorbos": "sorbo",
    "sorbosas": "sorbosa",
    "sorbáis": "sorber",
    "sorbéis": "sorber",
    "sorbí": "sorber",
    "sorbía": "sorber",
    "sorbíais": "sorber",
    "sorbíamos": "sorber",
    "sorbían": "sorber",
    "sorbías": "sorber",
    "sorbítica": "sorbítico",
    "sorbíticas": "sorbítico",
    "sorbíticos": "sorbítico",
    "sorciones": "sorción",
    "sorda": "sordo",
    "sordas": "sordo",
    "sordeces": "sordez",
    "sorderas": "sordera",
    "sordideces": "sordidez",
    "sordinas": "sordina",
    "sordinos": "sordino",
    "sordomuda": "sordomudo",
    "sordomudas": "sordomudo",
    "sordomudeces": "sordomudez",
    "sordomudos": "sordomudo",
    "sordos": "sordo",
    "sores": "sor",
    "sorgos": "sorgo",
    "soriana": "soriano",
    "sorianas": "soriano",
    "sorianos": "soriano",
    "sornas": "sorna",
    "soroches": "soroche",
    "sororales": "sororal",
    "soros": "soro",
    "sorosilicatos": "sorosilicato",
    "sorprenda": "sorprender",
    "sorprendamos": "sorprender",
    "sorprendan": "sorprender",
    "sorprendas": "sorprender",
    "sorprende": "sorprender",
    "sorprended": "sorprender",
    "sorprendemos": "sorprender",
    "sorprenden": "sorprender",
    "sorprendentes": "sorprendente",
    "sorprenderemos": "sorprender",
    "sorprenderá": "sorprender",
    "sorprenderán": "sorprender",
    "sorprenderás": "sorprender",
    "sorprenderé": "sorprender",
    "sorprenderéis": "sorprender",
    "sorprendería": "sorprender",
    "sorprenderíais": "sorprender",
    "sorprenderíamos": "sorprender",
    "sorprenderían": "sorprender",
    "sorprenderías": "sorprender",
    "sorprendes": "sorprender",
    "sorprendida": "sorprender",
    "sorprendidas": "sorprender",
    "sorprendido": "sorprender",
    "sorprendidos": "sorprender",
    "sorprendiendo": "sorprender",
    "sorprendiera": "sorprender",
    "sorprendierais": "sorprender",
    "sorprendieran": "sorprender",
    "sorprendieras": "sorprender",
    "sorprendiere": "sorprender",
    "sorprendiereis": "sorprender",
    "sorprendieren": "sorprender",
    "sorprendieres": "sorprender",
    "sorprendieron": "sorprender",
    "sorprendiese": "sorprender",
    "sorprendieseis": "sorprender",
    "sorprendiesen": "sorprender",
    "sorprendieses": "sorprender",
    "sorprendimos": "sorprender",
    "sorprendiste": "sorprender",
    "sorprendisteis": "sorprender",
    "sorprendiéramos": "sorprender",
    "sorprendiéremos": "sorprender",
    "sorprendiésemos": "sorprender",
    "sorprendió": "sorprender",
    "sorprendo": "sorprender",
    "sorprendáis": "sorprender",
    "sorprendéis": "sorprender",
    "sorprendí": "sorprender",
    "sorprendía": "sorprender",
    "sorprendíais": "sorprender",
    "sorprendíamos": "sorprender",
    "sorprendían": "sorprender",
    "sorprendías": "sorprender",
    "sorpresas": "sorpresa",
    "sorpresiva": "sorpresivo",
    "sorpresivas": "sorpresivo",
    "sorpresivos": "sorpresivo",
    "sorras": "sorra",
    "sorregaba": "sorregar",
    "sorregabais": "sorregar",
    "sorregaban": "sorregar",
    "sorregabas": "sorregar",
    "sorregad": "sorregar",
    "sorregada": "sorregar",
    "sorregadas": "sorregar",
    "sorregado": "sorregar",
    "sorregados": "sorregar",
    "sorregamos": "sorregar",
    "sorregando": "sorregar",
    "sorregara": "sorregar",
    "sorregarais": "sorregar",
    "sorregaran": "sorregar",
    "sorregaras": "sorregar",
    "sorregare": "sorregar",
    "sorregareis": "sorregar",
    "sorregaremos": "sorregar",
    "sorregaren": "sorregar",
    "sorregares": "sorregar",
    "sorregaron": "sorregar",
    "sorregará": "sorregar",
    "sorregarán": "sorregar",
    "sorregarás": "sorregar",
    "sorregaré": "sorregar",
    "sorregaréis": "sorregar",
    "sorregaría": "sorregar",
    "sorregaríais": "sorregar",
    "sorregaríamos": "sorregar",
    "sorregarían": "sorregar",
    "sorregarías": "sorregar",
    "sorregase": "sorregar",
    "sorregaseis": "sorregar",
    "sorregasen": "sorregar",
    "sorregases": "sorregar",
    "sorregaste": "sorregar",
    "sorregasteis": "sorregar",
    "sorreguemos": "sorregar",
    "sorregué": "sorregar",
    "sorreguéis": "sorregar",
    "sorregábamos": "sorregar",
    "sorregáis": "sorregar",
    "sorregáramos": "sorregar",
    "sorregáremos": "sorregar",
    "sorregásemos": "sorregar",
    "sorregó": "sorregar",
    "sorriega": "sorregar",
    "sorriegan": "sorregar",
    "sorriegas": "sorregar",
    "sorriego": "sorregar",
    "sorriegos": "sorriego",
    "sorriegue": "sorregar",
    "sorrieguen": "sorregar",
    "sorriegues": "sorregar",
    "sorrostradas": "sorrostrada",
    "sortea": "sortear",
    "sorteaba": "sortear",
    "sorteabais": "sortear",
    "sorteaban": "sortear",
    "sorteabas": "sortear",
    "sorteables": "sorteable",
    "sortead": "sortear",
    "sorteada": "sortear",
    "sorteadas": "sortear",
    "sorteado": "sortear",
    "sorteadora": "sorteador",
    "sorteadoras": "sorteador",
    "sorteadores": "sorteador",
    "sorteados": "sortear",
    "sorteamientos": "sorteamiento",
    "sorteamos": "sortear",
    "sortean": "sortear",
    "sorteando": "sortear",
    "sorteara": "sortear",
    "sortearais": "sortear",
    "sortearan": "sortear",
    "sortearas": "sortear",
    "sorteare": "sortear",
    "sorteareis": "sortear",
    "sortearemos": "sortear",
    "sortearen": "sortear",
    "sorteares": "sortear",
    "sortearon": "sortear",
    "sorteará": "sortear",
    "sortearán": "sortear",
    "sortearás": "sortear",
    "sortearé": "sortear",
    "sortearéis": "sortear",
    "sortearía": "sortear",
    "sortearíais": "sortear",
    "sortearíamos": "sortear",
    "sortearían": "sortear",
    "sortearías": "sortear",
    "sorteas": "sortear",
    "sortease": "sortear",
    "sorteaseis": "sortear",
    "sorteasen": "sortear",
    "sorteases": "sortear",
    "sorteaste": "sortear",
    "sorteasteis": "sortear",
    "sortee": "sortear",
    "sorteemos": "sortear",
    "sorteen": "sortear",
    "sortees": "sortear",
    "sorteo": "sortear",
    "sorteos": "sorteo",
    "sortera": "sortero",
    "sorteras": "sortero",
    "sorteros": "sortero",
    "sorteábamos": "sortear",
    "sorteáis": "sortear",
    "sorteáramos": "sortear",
    "sorteáremos": "sortear",
    "sorteásemos": "sortear",
    "sorteé": "sortear",
    "sorteéis": "sortear",
    "sorteó": "sortear",
    "sortiarias": "sortiaria",
    "sortijas": "sortija",
    "sortijeros": "sortijero",
    "sortilegios": "sortilegio",
    "sortílega": "sortílego",
    "sortílegas": "sortílego",
    "sortílegos": "sortílego",
    "sosa": "soso",
    "sosainas": "sosaina",
    "sosas": "soso",
    "soscavaciones": "soscavación",
    "sosegaba": "sosegar",
    "sosegabais": "sosegar",
    "sosegaban": "sosegar",
    "sosegabas": "sosegar",
    "sosegad": "sosegar",
    "sosegada": "sosegar",
    "sosegadas": "sosegar",
    "sosegado": "sosegar",
    "sosegadora": "sosegador",
    "sosegadoras": "sosegador",
    "sosegadores": "sosegador",
    "sosegados": "sosegar",
    "sosegamos": "sosegar",
    "sosegando": "sosegar",
    "sosegara": "sosegar",
    "sosegarais": "sosegar",
    "sosegaran": "sosegar",
    "sosegaras": "sosegar",
    "sosegare": "sosegar",
    "sosegareis": "sosegar",
    "sosegaremos": "sosegar",
    "sosegaren": "sosegar",
    "sosegares": "sosegar",
    "sosegaron": "sosegar",
    "sosegará": "sosegar",
    "sosegarán": "sosegar",
    "sosegarás": "sosegar",
    "sosegaré": "sosegar",
    "sosegaréis": "sosegar",
    "sosegaría": "sosegar",
    "sosegaríais": "sosegar",
    "sosegaríamos": "sosegar",
    "sosegarían": "sosegar",
    "sosegarías": "sosegar",
    "sosegase": "sosegar",
    "sosegaseis": "sosegar",
    "sosegasen": "sosegar",
    "sosegases": "sosegar",
    "sosegaste": "sosegar",
    "sosegasteis": "sosegar",
    "soseguemos": "sosegar",
    "sosegué": "sosegar",
    "soseguéis": "sosegar",
    "sosegábamos": "sosegar",
    "sosegáis": "sosegar",
    "sosegáramos": "sosegar",
    "sosegáremos": "sosegar",
    "sosegásemos": "sosegar",
    "sosegó": "sosegar",
    "sosera": "sosero",
    "soseras": "sosero",
    "soseros": "sosero",
    "soserías": "sosería",
    "sosias": "sosia",
    "sosiega": "sosegar",
    "sosiegan": "sosegar",
    "sosiegas": "sosiega",
    "sosiego": "sosegar",
    "sosiegos": "sosiego",
    "sosiegue": "sosegar",
    "sosieguen": "sosegar",
    "sosiegues": "sosegar",
    "soslaya": "soslayo",
    "soslayaba": "soslayar",
    "soslayabais": "soslayar",
    "soslayaban": "soslayar",
    "soslayabas": "soslayar",
    "soslayad": "soslayar",
    "soslayada": "soslayar",
    "soslayadas": "soslayar",
    "soslayado": "soslayar",
    "soslayados": "soslayar",
    "soslayamos": "soslayar",
    "soslayan": "soslayar",
    "soslayando": "soslayar",
    "soslayara": "soslayar",
    "soslayarais": "soslayar",
    "soslayaran": "soslayar",
    "soslayaras": "soslayar",
    "soslayare": "soslayar",
    "soslayareis": "soslayar",
    "soslayaremos": "soslayar",
    "soslayaren": "soslayar",
    "soslayares": "soslayar",
    "soslayaron": "soslayar",
    "soslayará": "soslayar",
    "soslayarán": "soslayar",
    "soslayarás": "soslayar",
    "soslayaré": "soslayar",
    "soslayaréis": "soslayar",
    "soslayaría": "soslayar",
    "soslayaríais": "soslayar",
    "soslayaríamos": "soslayar",
    "soslayarían": "soslayar",
    "soslayarías": "soslayar",
    "soslayas": "soslayo",
    "soslayase": "soslayar",
    "soslayaseis": "soslayar",
    "soslayasen": "soslayar",
    "soslayases": "soslayar",
    "soslayaste": "soslayar",
    "soslayasteis": "soslayar",
    "soslaye": "soslayar",
    "soslayemos": "soslayar",
    "soslayen": "soslayar",
    "soslayes": "soslayar",
    "soslayo": "soslayar",
    "soslayos": "soslayo",
    "soslayábamos": "soslayar",
    "soslayáis": "soslayar",
    "soslayáramos": "soslayar",
    "soslayáremos": "soslayar",
    "soslayásemos": "soslayar",
    "soslayé": "soslayar",
    "soslayéis": "soslayar",
    "soslayó": "soslayar",
    "sosos": "soso",
    "sospecha": "sospechar",
    "sospechaba": "sospechar",
    "sospechabais": "sospechar",
    "sospechaban": "sospechar",
    "sospechabas": "sospechar",
    "sospechables": "sospechable",
    "sospechad": "sospechar",
    "sospechada": "sospechar",
    "sospechadas": "sospechar",
    "sospechado": "sospechar",
    "sospechados": "sospechar",
    "sospechamos": "sospechar",
    "sospechan": "sospechar",
    "sospechando": "sospechar",
    "sospechara": "sospechar",
    "sospecharais": "sospechar",
    "sospecharan": "sospechar",
    "sospecharas": "sospechar",
    "sospechare": "sospechar",
    "sospechareis": "sospechar",
    "sospecharemos": "sospechar",
    "sospecharen": "sospechar",
    "sospechares": "sospechar",
    "sospecharon": "sospechar",
    "sospechará": "sospechar",
    "sospecharán": "sospechar",
    "sospecharás": "sospechar",
    "sospecharé": "sospechar",
    "sospecharéis": "sospechar",
    "sospecharía": "sospechar",
    "sospecharíais": "sospechar",
    "sospecharíamos": "sospechar",
    "sospecharían": "sospechar",
    "sospecharías": "sospechar",
    "sospechas": "sospechar",
    "sospechase": "sospechar",
    "sospechaseis": "sospechar",
    "sospechasen": "sospechar",
    "sospechases": "sospechar",
    "sospechaste": "sospechar",
    "sospechasteis": "sospechar",
    "sospeche": "sospechar",
    "sospechemos": "sospechar",
    "sospechen": "sospechar",
    "sospeches": "sospechar",
    "sospecho": "sospechar",
    "sospechosa": "sospechoso",
    "sospechosas": "sospechoso",
    "sospechosos": "sospechoso",
    "sospechábamos": "sospechar",
    "sospecháis": "sospechar",
    "sospecháramos": "sospechar",
    "sospecháremos": "sospechar",
    "sospechásemos": "sospechar",
    "sospeché": "sospechar",
    "sospechéis": "sospechar",
    "sospechó": "sospechar",
    "sosquines": "sosquín",
    "sostendremos": "sostener",
    "sostendrá": "sostener",
    "sostendrán": "sostener",
    "sostendrás": "sostener",
    "sostendré": "sostener",
    "sostendréis": "sostener",
    "sostendría": "sostener",
    "sostendríais": "sostener",
    "sostendríamos": "sostener",
    "sostendrían": "sostener",
    "sostendrías": "sostener",
    "sostened": "sostener",
    "sostenedora": "sostenedor",
    "sostenedoras": "sostenedor",
    "sostenedores": "sostenedor",
    "sostenemos": "sostener",
    "sostenes": "sostén",
    "sostenga": "sostener",
    "sostengamos": "sostener",
    "sostengan": "sostener",
    "sostengas": "sostener",
    "sostengo": "sostener",
    "sostengáis": "sostener",
    "sostenibilidades": "sostenibilidad",
    "sostenibles": "sostenible",
    "sostenida": "sostener",
    "sostenidas": "sostener",
    "sostenido": "sostener",
    "sostenidos": "sostenido",
    "sosteniendo": "sostener",
    "sostenimientos": "sostenimiento",
    "sostenéis": "sostener",
    "sostenía": "sostener",
    "sosteníais": "sostener",
    "sosteníamos": "sostener",
    "sostenían": "sostener",
    "sostenías": "sostener",
    "sostiene": "sostener",
    "sostienen": "sostener",
    "sostienes": "sostener",
    "sostuve": "sostener",
    "sostuviera": "sostener",
    "sostuvierais": "sostener",
    "sostuvieran": "sostener",
    "sostuvieras": "sostener",
    "sostuviere": "sostener",
    "sostuviereis": "sostener",
    "sostuvieren": "sostener",
    "sostuvieres": "sostener",
    "sostuvieron": "sostener",
    "sostuviese": "sostener",
    "sostuvieseis": "sostener",
    "sostuviesen": "sostener",
    "sostuvieses": "sostener",
    "sostuvimos": "sostener",
    "sostuviste": "sostener",
    "sostuvisteis": "sostener",
    "sostuviéramos": "sostener",
    "sostuviéremos": "sostener",
    "sostuviésemos": "sostener",
    "sostuvo": "sostener",
    "sostén": "sostener",
    "sotabancos": "sotabanco",
    "sotabarbas": "sotabarba",
    "sotabatanes": "sotabatán",
    "sotacardas": "sotacarda",
    "sotacolas": "sotacola",
    "sotacoros": "sotacoro",
    "sotacuras": "sotacura",
    "sotalias": "sotalia",
    "sotalugos": "sotalugo",
    "sotanas": "sotana",
    "sotanea": "sotanear",
    "sotaneaba": "sotanear",
    "sotaneabais": "sotanear",
    "sotaneaban": "sotanear",
    "sotaneabas": "sotanear",
    "sotanead": "sotanear",
    "sotaneada": "sotanear",
    "sotaneadas": "sotanear",
    "sotaneado": "sotanear",
    "sotaneados": "sotanear",
    "sotaneamos": "sotanear",
    "sotanean": "sotanear",
    "sotaneando": "sotanear",
    "sotaneara": "sotanear",
    "sotanearais": "sotanear",
    "sotanearan": "sotanear",
    "sotanearas": "sotanear",
    "sotaneare": "sotanear",
    "sotaneareis": "sotanear",
    "sotanearemos": "sotanear",
    "sotanearen": "sotanear",
    "sotaneares": "sotanear",
    "sotanearon": "sotanear",
    "sotaneará": "sotanear",
    "sotanearán": "sotanear",
    "sotanearás": "sotanear",
    "sotanearé": "sotanear",
    "sotanearéis": "sotanear",
    "sotanearía": "sotanear",
    "sotanearíais": "sotanear",
    "sotanearíamos": "sotanear",
    "sotanearían": "sotanear",
    "sotanearías": "sotanear",
    "sotaneas": "sotanear",
    "sotanease": "sotanear",
    "sotaneaseis": "sotanear",
    "sotaneasen": "sotanear",
    "sotaneases": "sotanear",
    "sotaneaste": "sotanear",
    "sotaneasteis": "sotanear",
    "sotanee": "sotanear",
    "sotaneemos": "sotanear",
    "sotaneen": "sotanear",
    "sotanees": "sotanear",
    "sotaneo": "sotanear",
    "sotaneábamos": "sotanear",
    "sotaneáis": "sotanear",
    "sotaneáramos": "sotanear",
    "sotaneáremos": "sotanear",
    "sotaneásemos": "sotanear",
    "sotaneé": "sotanear",
    "sotaneéis": "sotanear",
    "sotaneó": "sotanear",
    "sotas": "sota",
    "sotaventa": "sotaventar",
    "sotaventaba": "sotaventar",
    "sotaventabais": "sotaventar",
    "sotaventaban": "sotaventar",
    "sotaventabas": "sotaventar",
    "sotaventad": "sotaventar",
    "sotaventada": "sotaventar",
    "sotaventadas": "sotaventar",
    "sotaventado": "sotaventar",
    "sotaventados": "sotaventar",
    "sotaventamos": "sotaventar",
    "sotaventan": "sotaventar",
    "sotaventando": "sotaventar",
    "sotaventara": "sotaventar",
    "sotaventarais": "sotaventar",
    "sotaventaran": "sotaventar",
    "sotaventaras": "sotaventar",
    "sotaventare": "sotaventar",
    "sotaventareis": "sotaventar",
    "sotaventaremos": "sotaventar",
    "sotaventaren": "sotaventar",
    "sotaventares": "sotaventar",
    "sotaventaron": "sotaventar",
    "sotaventará": "sotaventar",
    "sotaventarán": "sotaventar",
    "sotaventarás": "sotaventar",
    "sotaventaré": "sotaventar",
    "sotaventaréis": "sotaventar",
    "sotaventaría": "sotaventar",
    "sotaventaríais": "sotaventar",
    "sotaventaríamos": "sotaventar",
    "sotaventarían": "sotaventar",
    "sotaventarías": "sotaventar",
    "sotaventas": "sotaventar",
    "sotaventase": "sotaventar",
    "sotaventaseis": "sotaventar",
    "sotaventasen": "sotaventar",
    "sotaventases": "sotaventar",
    "sotaventaste": "sotaventar",
    "sotaventasteis": "sotaventar",
    "sotavente": "sotaventar",
    "sotaventemos": "sotaventar",
    "sotaventen": "sotaventar",
    "sotaventes": "sotaventar",
    "sotavento": "sotaventar",
    "sotaventos": "sotavento",
    "sotaventábamos": "sotaventar",
    "sotaventáis": "sotaventar",
    "sotaventáramos": "sotaventar",
    "sotaventáremos": "sotaventar",
    "sotaventásemos": "sotaventar",
    "sotaventé": "sotaventar",
    "sotaventéis": "sotaventar",
    "sotaventó": "sotaventar",
    "sotechados": "sotechado",
    "soteriologías": "soteriología",
    "soterra": "soterrar",
    "soterraba": "soterrar",
    "soterrabais": "soterrar",
    "soterraban": "soterrar",
    "soterrabas": "soterrar",
    "soterrad": "soterrar",
    "soterrada": "soterrar",
    "soterradas": "soterrar",
    "soterrado": "soterrar",
    "soterrados": "soterrar",
    "soterramientos": "soterramiento",
    "soterramos": "soterrar",
    "soterran": "soterrar",
    "soterrando": "soterrar",
    "soterrara": "soterrar",
    "soterrarais": "soterrar",
    "soterraran": "soterrar",
    "soterraras": "soterrar",
    "soterrare": "soterrar",
    "soterrareis": "soterrar",
    "soterraremos": "soterrar",
    "soterraren": "soterrar",
    "soterrares": "soterrar",
    "soterraron": "soterrar",
    "soterrará": "soterrar",
    "soterrarán": "soterrar",
    "soterrarás": "soterrar",
    "soterraré": "soterrar",
    "soterraréis": "soterrar",
    "soterraría": "soterrar",
    "soterraríais": "soterrar",
    "soterraríamos": "soterrar",
    "soterrarían": "soterrar",
    "soterrarías": "soterrar",
    "soterras": "soterrar",
    "soterrase": "soterrar",
    "soterraseis": "soterrar",
    "soterrasen": "soterrar",
    "soterrases": "soterrar",
    "soterraste": "soterrar",
    "soterrasteis": "soterrar",
    "soterraña": "soterraño",
    "soterrañas": "soterraño",
    "soterraños": "soterraño",
    "soterre": "soterrar",
    "soterremos": "soterrar",
    "soterren": "soterrar",
    "soterres": "soterrar",
    "soterro": "soterrar",
    "soterrábamos": "soterrar",
    "soterráis": "soterrar",
    "soterráramos": "soterrar",
    "soterráremos": "soterrar",
    "soterrásemos": "soterrar",
    "soterré": "soterrar",
    "soterréis": "soterrar",
    "soterró": "soterrar",
    "soteña": "soteño",
    "soteñas": "soteño",
    "soteños": "soteño",
    "sotilezas": "sotileza",
    "sotobosques": "sotobosque",
    "sotomontes": "sotomonte",
    "sotos": "soto",
    "sotrozos": "sotrozo",
    "sotueres": "sotuer",
    "soufflés": "soufflé",
    "sovietice": "sovietizar",
    "sovieticemos": "sovietizar",
    "sovieticen": "sovietizar",
    "sovietices": "sovietizar",
    "sovieticé": "sovietizar",
    "sovieticéis": "sovietizar",
    "sovietiza": "sovietizar",
    "sovietizaba": "sovietizar",
    "sovietizabais": "sovietizar",
    "sovietizaban": "sovietizar",
    "sovietizabas": "sovietizar",
    "sovietizaciones": "sovietización",
    "sovietizad": "sovietizar",
    "sovietizada": "sovietizar",
    "sovietizadas": "sovietizar",
    "sovietizado": "sovietizar",
    "sovietizados": "sovietizar",
    "sovietizamos": "sovietizar",
    "sovietizan": "sovietizar",
    "sovietizando": "sovietizar",
    "sovietizara": "sovietizar",
    "sovietizarais": "sovietizar",
    "sovietizaran": "sovietizar",
    "sovietizaras": "sovietizar",
    "sovietizare": "sovietizar",
    "sovietizareis": "sovietizar",
    "sovietizaremos": "sovietizar",
    "sovietizaren": "sovietizar",
    "sovietizares": "sovietizar",
    "sovietizaron": "sovietizar",
    "sovietizará": "sovietizar",
    "sovietizarán": "sovietizar",
    "sovietizarás": "sovietizar",
    "sovietizaré": "sovietizar",
    "sovietizaréis": "sovietizar",
    "sovietizaría": "sovietizar",
    "sovietizaríais": "sovietizar",
    "sovietizaríamos": "sovietizar",
    "sovietizarían": "sovietizar",
    "sovietizarías": "sovietizar",
    "sovietizas": "sovietizar",
    "sovietizase": "sovietizar",
    "sovietizaseis": "sovietizar",
    "sovietizasen": "sovietizar",
    "sovietizases": "sovietizar",
    "sovietizaste": "sovietizar",
    "sovietizasteis": "sovietizar",
    "sovietizo": "sovietizar",
    "sovietizábamos": "sovietizar",
    "sovietizáis": "sovietizar",
    "sovietizáramos": "sovietizar",
    "sovietizáremos": "sovietizar",
    "sovietizásemos": "sovietizar",
    "sovietizó": "sovietizar",
    "soviets": "soviet",
    "soviética": "soviético",
    "soviéticas": "soviético",
    "soviéticos": "soviético",
    "sovjoses": "sovjós",
    "soy": "ser",
    "soñaba": "soñar",
    "soñabais": "soñar",
    "soñaban": "soñar",
    "soñabas": "soñar",
    "soñad": "soñar",
    "soñada": "soñar",
    "soñadas": "soñar",
    "soñado": "soñar",
    "soñadora": "soñador",
    "soñadoras": "soñador",
    "soñadores": "soñador",
    "soñados": "soñar",
    "soñamos": "soñar",
    "soñando": "soñar",
    "soñara": "soñar",
    "soñarais": "soñar",
    "soñaran": "soñar",
    "soñaras": "soñar",
    "soñare": "soñar",
    "soñareis": "soñar",
    "soñaremos": "soñar",
    "soñaren": "soñar",
    "soñares": "soñar",
    "soñaron": "soñar",
    "soñarreras": "soñarrera",
    "soñará": "soñar",
    "soñarán": "soñar",
    "soñarás": "soñar",
    "soñaré": "soñar",
    "soñaréis": "soñar",
    "soñaría": "soñar",
    "soñaríais": "soñar",
    "soñaríamos": "soñar",
    "soñarían": "soñar",
    "soñarías": "soñar",
    "soñase": "soñar",
    "soñaseis": "soñar",
    "soñasen": "soñar",
    "soñases": "soñar",
    "soñaste": "soñar",
    "soñasteis": "soñar",
    "soñemos": "soñar",
    "soñeras": "soñera",
    "soñolencias": "soñolencia",
    "soñolienta": "soñoliento",
    "soñolientas": "soñoliento",
    "soñolientos": "soñoliento",
    "soñábamos": "soñar",
    "soñáis": "soñar",
    "soñáramos": "soñar",
    "soñáremos": "soñar",
    "soñásemos": "soñar",
    "soñé": "soñar",
    "soñéis": "soñar",
    "soñó": "soñar",
    "spagats": "spagat",
    "spaghettis": "spaghetti",
    "sparrings": "sparring",
    "speakers": "speaker",
    "spinnakers": "spinnaker",
    "spins": "spin",
    "spis": "spí",
    "sponsors": "sponsor",
    "spools": "spool",
    "sports": "sport",
    "spots": "spot",
    "sprays": "spray",
    "springs": "spring",
    "sprinters": "sprinter",
    "sprints": "sprint",
    "squatteres": "squatter",
    "squatters": "squatter",
    "staccatos": "staccato",
    "stages": "stage",
    "stagflations": "stagflation",
    "stajanovismos": "stajanovismo",
    "stalinismos": "stalinismo",
    "standards": "standard",
    "stands": "stand",
    "star-systems": "star-system",
    "stars": "star",
    "starters": "starter",
    "stayers": "stayer",
    "steps": "step",
    "stichtitas": "stichtita",
    "sticks": "stick",
    "stigmómetros": "stigmómetro",
    "stilbs": "stilb",
    "stocks": "stock",
    "stops": "stop",
    "stripteasees": "striptease",
    "stripteases": "striptease",
    "stupas": "stupa",
    "suaba": "suabo",
    "suabas": "suabo",
    "suabia": "suabio",
    "suabias": "suabio",
    "suabios": "suabio",
    "suabos": "suabo",
    "suahilis": "suahili",
    "suardas": "suarda",
    "suarismos": "suarismo",
    "suaristas": "suarista",
    "suasoria": "suasorio",
    "suasorias": "suasorio",
    "suasorios": "suasorio",
    "suaves": "suave",
    "suavice": "suavizar",
    "suavicemos": "suavizar",
    "suavicen": "suavizar",
    "suavices": "suavizar",
    "suavicé": "suavizar",
    "suavicéis": "suavizar",
    "suavidades": "suavidad",
    "suaviza": "suavizar",
    "suavizaba": "suavizar",
    "suavizabais": "suavizar",
    "suavizaban": "suavizar",
    "suavizabas": "suavizar",
    "suavizaciones": "suavización",
    "suavizad": "suavizar",
    "suavizada": "suavizar",
    "suavizadas": "suavizar",
    "suavizado": "suavizar",
    "suavizadora": "suavizador",
    "suavizadoras": "suavizador",
    "suavizadores": "suavizador",
    "suavizados": "suavizar",
    "suavizamos": "suavizar",
    "suavizan": "suavizar",
    "suavizando": "suavizar",
    "suavizantes": "suavizante",
    "suavizara": "suavizar",
    "suavizarais": "suavizar",
    "suavizaran": "suavizar",
    "suavizaras": "suavizar",
    "suavizare": "suavizar",
    "suavizareis": "suavizar",
    "suavizaremos": "suavizar",
    "suavizaren": "suavizar",
    "suavizares": "suavizar",
    "suavizaron": "suavizar",
    "suavizará": "suavizar",
    "suavizarán": "suavizar",
    "suavizarás": "suavizar",
    "suavizaré": "suavizar",
    "suavizaréis": "suavizar",
    "suavizaría": "suavizar",
    "suavizaríais": "suavizar",
    "suavizaríamos": "suavizar",
    "suavizarían": "suavizar",
    "suavizarías": "suavizar",
    "suavizas": "suavizar",
    "suavizase": "suavizar",
    "suavizaseis": "suavizar",
    "suavizasen": "suavizar",
    "suavizases": "suavizar",
    "suavizaste": "suavizar",
    "suavizasteis": "suavizar",
    "suavizo": "suavizar",
    "suavizábamos": "suavizar",
    "suavizáis": "suavizar",
    "suavizáramos": "suavizar",
    "suavizáremos": "suavizar",
    "suavizásemos": "suavizar",
    "suavizó": "suavizar",
    "suba": "subir",
    "subacromiales": "subacromial",
    "subacuática": "subacuático",
    "subacuáticas": "subacuático",
    "subacuáticos": "subacuático",
    "subafluentes": "subafluente",
    "subagentes": "subagente",
    "subalimentaciones": "subalimentación",
    "subalinas": "subalina",
    "subalpina": "subalpino",
    "subalpinas": "subalpino",
    "subalpinos": "subalpino",
    "subalterna": "subalterno",
    "subalternas": "subalterno",
    "subalternos": "subalterno",
    "subamos": "subir",
    "suban": "subir",
    "subanillos": "subanillo",
    "subaracnoidea": "subaracnoideo",
    "subaracnoideas": "subaracnoideo",
    "subaracnoideos": "subaracnoideo",
    "subarmónica": "subarmónico",
    "subarmónicas": "subarmónico",
    "subarmónicos": "subarmónico",
    "subarrendaba": "subarrendar",
    "subarrendabais": "subarrendar",
    "subarrendaban": "subarrendar",
    "subarrendabas": "subarrendar",
    "subarrendad": "subarrendar",
    "subarrendada": "subarrendar",
    "subarrendadas": "subarrendar",
    "subarrendado": "subarrendar",
    "subarrendadora": "subarrendador",
    "subarrendadoras": "subarrendador",
    "subarrendadores": "subarrendador",
    "subarrendados": "subarrendar",
    "subarrendamientos": "subarrendamiento",
    "subarrendamos": "subarrendar",
    "subarrendando": "subarrendar",
    "subarrendara": "subarrendar",
    "subarrendarais": "subarrendar",
    "subarrendaran": "subarrendar",
    "subarrendaras": "subarrendar",
    "subarrendare": "subarrendar",
    "subarrendareis": "subarrendar",
    "subarrendaremos": "subarrendar",
    "subarrendaren": "subarrendar",
    "subarrendares": "subarrendar",
    "subarrendaron": "subarrendar",
    "subarrendará": "subarrendar",
    "subarrendarán": "subarrendar",
    "subarrendarás": "subarrendar",
    "subarrendaré": "subarrendar",
    "subarrendaréis": "subarrendar",
    "subarrendaría": "subarrendar",
    "subarrendaríais": "subarrendar",
    "subarrendaríamos": "subarrendar",
    "subarrendarían": "subarrendar",
    "subarrendarías": "subarrendar",
    "subarrendase": "subarrendar",
    "subarrendaseis": "subarrendar",
    "subarrendasen": "subarrendar",
    "subarrendases": "subarrendar",
    "subarrendaste": "subarrendar",
    "subarrendasteis": "subarrendar",
    "subarrendataria": "subarrendatario",
    "subarrendatarias": "subarrendatario",
    "subarrendatarios": "subarrendatario",
    "subarrendemos": "subarrendar",
    "subarrendábamos": "subarrendar",
    "subarrendáis": "subarrendar",
    "subarrendáramos": "subarrendar",
    "subarrendáremos": "subarrendar",
    "subarrendásemos": "subarrendar",
    "subarrendé": "subarrendar",
    "subarrendéis": "subarrendar",
    "subarrendó": "subarrendar",
    "subarrienda": "subarrendar",
    "subarriendan": "subarrendar",
    "subarriendas": "subarrendar",
    "subarriende": "subarrendar",
    "subarrienden": "subarrendar",
    "subarriendes": "subarrendar",
    "subarriendo": "subarrendar",
    "subarriendos": "subarriendo",
    "subas": "subir",
    "subasociaciones": "subasociación",
    "subasta": "subastar",
    "subastaba": "subastar",
    "subastabais": "subastar",
    "subastaban": "subastar",
    "subastabas": "subastar",
    "subastad": "subastar",
    "subastada": "subastar",
    "subastadas": "subastar",
    "subastado": "subastar",
    "subastadora": "subastador",
    "subastadoras": "subastador",
    "subastadores": "subastador",
    "subastados": "subastar",
    "subastamos": "subastar",
    "subastan": "subastar",
    "subastando": "subastar",
    "subastara": "subastar",
    "subastarais": "subastar",
    "subastaran": "subastar",
    "subastaras": "subastar",
    "subastare": "subastar",
    "subastareis": "subastar",
    "subastaremos": "subastar",
    "subastaren": "subastar",
    "subastares": "subastar",
    "subastaron": "subastar",
    "subastará": "subastar",
    "subastarán": "subastar",
    "subastarás": "subastar",
    "subastaré": "subastar",
    "subastaréis": "subastar",
    "subastaría": "subastar",
    "subastaríais": "subastar",
    "subastaríamos": "subastar",
    "subastarían": "subastar",
    "subastarías": "subastar",
    "subastas": "subastar",
    "subastase": "subastar",
    "subastaseis": "subastar",
    "subastasen": "subastar",
    "subastases": "subastar",
    "subastaste": "subastar",
    "subastasteis": "subastar",
    "subaste": "subastar",
    "subastemos": "subastar",
    "subasten": "subastar",
    "subastera": "subastero",
    "subasteras": "subastero",
    "subasteros": "subastero",
    "subastes": "subastar",
    "subasto": "subastar",
    "subastragalina": "subastragalino",
    "subastragalinas": "subastragalino",
    "subastragalinos": "subastragalino",
    "subastábamos": "subastar",
    "subastáis": "subastar",
    "subastáramos": "subastar",
    "subastáremos": "subastar",
    "subastásemos": "subastar",
    "subasté": "subastar",
    "subastéis": "subastar",
    "subastó": "subastar",
    "subatmosférica": "subatmosférico",
    "subatmosféricas": "subatmosférico",
    "subatmosféricos": "subatmosférico",
    "subatributos": "subatributo",
    "subatómica": "subatómico",
    "subatómicas": "subatómico",
    "subatómicos": "subatómico",
    "subbases": "subbase",
    "subbucles": "subbucle",
    "subcadenas": "subcadena",
    "subcallosa": "subcalloso",
    "subcallosas": "subcalloso",
    "subcallosos": "subcalloso",
    "subcampeona": "subcampeón",
    "subcampeonas": "subcampeón",
    "subcampeonatos": "subcampeonato",
    "subcampeones": "subcampeón",
    "subcampos": "subcampo",
    "subcanales": "subcanal",
    "subcapilares": "subcapilar",
    "subcapsulares": "subcapsular",
    "subcarbonatos": "subcarbonato",
    "subcategorizaciones": "subcategorización",
    "subcategorías": "subcategoría",
    "subcatálogos": "subcatálogo",
    "subceros": "subcero",
    "subclanes": "subclán",
    "subclases": "subclase",
    "subclaves": "subclave",
    "subclavia": "subclavio",
    "subclavias": "subclavio",
    "subclavios": "subclavio",
    "subclímaxes": "subclímax",
    "subcolas": "subcola",
    "subcomandos": "subcomando",
    "subcomisarios": "subcomisario",
    "subcomisiones": "subcomisión",
    "subcomisurales": "subcomisural",
    "subcomites": "subcomité",
    "subcomponentes": "subcomponente",
    "subconciencias": "subconciencia",
    "subcondiciones": "subcondición",
    "subcondílea": "subcondíleo",
    "subcondíleas": "subcondíleo",
    "subcondíleos": "subcondíleo",
    "subconjuntos": "subconjunto",
    "subconsciencias": "subconsciencia",
    "subconscientes": "subconsciente",
    "subconsolas": "subconsola",
    "subconsolidada": "subconsolidado",
    "subconsolidadas": "subconsolidado",
    "subconsolidados": "subconsolidado",
    "subconsumos": "subconsumo",
    "subcontrata": "subcontratar",
    "subcontrataba": "subcontratar",
    "subcontratabais": "subcontratar",
    "subcontrataban": "subcontratar",
    "subcontratabas": "subcontratar",
    "subcontrataciones": "subcontratación",
    "subcontratad": "subcontratar",
    "subcontratada": "subcontratar",
    "subcontratadas": "subcontratar",
    "subcontratado": "subcontratar",
    "subcontratados": "subcontratar",
    "subcontratamos": "subcontratar",
    "subcontratan": "subcontratar",
    "subcontratando": "subcontratar",
    "subcontratantes": "subcontratante",
    "subcontratara": "subcontratar",
    "subcontratarais": "subcontratar",
    "subcontrataran": "subcontratar",
    "subcontrataras": "subcontratar",
    "subcontratare": "subcontratar",
    "subcontratareis": "subcontratar",
    "subcontrataremos": "subcontratar",
    "subcontrataren": "subcontratar",
    "subcontratares": "subcontratar",
    "subcontrataron": "subcontratar",
    "subcontratará": "subcontratar",
    "subcontratarán": "subcontratar",
    "subcontratarás": "subcontratar",
    "subcontrataré": "subcontratar",
    "subcontrataréis": "subcontratar",
    "subcontrataría": "subcontratar",
    "subcontrataríais": "subcontratar",
    "subcontrataríamos": "subcontratar",
    "subcontratarían": "subcontratar",
    "subcontratarías": "subcontratar",
    "subcontratas": "subcontratar",
    "subcontratase": "subcontratar",
    "subcontrataseis": "subcontratar",
    "subcontratasen": "subcontratar",
    "subcontratases": "subcontratar",
    "subcontrataste": "subcontratar",
    "subcontratasteis": "subcontratar",
    "subcontrate": "subcontratar",
    "subcontratemos": "subcontratar",
    "subcontraten": "subcontratar",
    "subcontrates": "subcontratar",
    "subcontratistas": "subcontratista",
    "subcontrato": "subcontratar",
    "subcontratos": "subcontrato",
    "subcontratábamos": "subcontratar",
    "subcontratáis": "subcontratar",
    "subcontratáramos": "subcontratar",
    "subcontratáremos": "subcontratar",
    "subcontratásemos": "subcontratar",
    "subcontraté": "subcontratar",
    "subcontratéis": "subcontratar",
    "subcontrató": "subcontratar",
    "subcorticales": "subcortical",
    "subcostales": "subcostal",
    "subcrítica": "subcrítico",
    "subcríticas": "subcrítico",
    "subcríticos": "subcrítico",
    "subcuentas": "subcuenta",
    "subcuerpos": "subcuerpo",
    "subcultivos": "subcultivo",
    "subculturas": "subcultura",
    "subcutánea": "subcutáneo",
    "subcutáneas": "subcutáneo",
    "subcutáneos": "subcutáneo",
    "subdelega": "subdelegar",
    "subdelegaba": "subdelegar",
    "subdelegabais": "subdelegar",
    "subdelegaban": "subdelegar",
    "subdelegabas": "subdelegar",
    "subdelegables": "subdelegable",
    "subdelegaciones": "subdelegación",
    "subdelegad": "subdelegar",
    "subdelegada": "subdelegar",
    "subdelegadas": "subdelegar",
    "subdelegado": "subdelegar",
    "subdelegados": "subdelegar",
    "subdelegamos": "subdelegar",
    "subdelegan": "subdelegar",
    "subdelegando": "subdelegar",
    "subdelegara": "subdelegar",
    "subdelegarais": "subdelegar",
    "subdelegaran": "subdelegar",
    "subdelegaras": "subdelegar",
    "subdelegare": "subdelegar",
    "subdelegareis": "subdelegar",
    "subdelegaremos": "subdelegar",
    "subdelegaren": "subdelegar",
    "subdelegares": "subdelegar",
    "subdelegaron": "subdelegar",
    "subdelegará": "subdelegar",
    "subdelegarán": "subdelegar",
    "subdelegarás": "subdelegar",
    "subdelegaré": "subdelegar",
    "subdelegaréis": "subdelegar",
    "subdelegaría": "subdelegar",
    "subdelegaríais": "subdelegar",
    "subdelegaríamos": "subdelegar",
    "subdelegarían": "subdelegar",
    "subdelegarías": "subdelegar",
    "subdelegas": "subdelegar",
    "subdelegase": "subdelegar",
    "subdelegaseis": "subdelegar",
    "subdelegasen": "subdelegar",
    "subdelegases": "subdelegar",
    "subdelegaste": "subdelegar",
    "subdelegasteis": "subdelegar",
    "subdelego": "subdelegar",
    "subdelegue": "subdelegar",
    "subdeleguemos": "subdelegar",
    "subdeleguen": "subdelegar",
    "subdelegues": "subdelegar",
    "subdelegué": "subdelegar",
    "subdeleguéis": "subdelegar",
    "subdelegábamos": "subdelegar",
    "subdelegáis": "subdelegar",
    "subdelegáramos": "subdelegar",
    "subdelegáremos": "subdelegar",
    "subdelegásemos": "subdelegar",
    "subdelegó": "subdelegar",
    "subdelirios": "subdelirio",
    "subdeltoidea": "subdeltoideo",
    "subdeltoideas": "subdeltoideo",
    "subdeltoideos": "subdeltoideo",
    "subdentaria": "subdentario",
    "subdentarias": "subdentario",
    "subdentarios": "subdentario",
    "subdesarrollada": "subdesarrollado",
    "subdesarrolladas": "subdesarrollado",
    "subdesarrollados": "subdesarrollado",
    "subdesarrollos": "subdesarrollo",
    "subdesértica": "subdesértico",
    "subdesérticas": "subdesértico",
    "subdesérticos": "subdesértico",
    "subdiaconados": "subdiaconado",
    "subdiaconales": "subdiaconal",
    "subdiaconatos": "subdiaconato",
    "subdialectos": "subdialecto",
    "subdirecciones": "subdirección",
    "subdirectora": "subdirector",
    "subdirectoras": "subdirector",
    "subdirectores": "subdirector",
    "subdirectorios": "subdirectorio",
    "subdistinciones": "subdistinción",
    "subdivida": "subdividir",
    "subdividamos": "subdividir",
    "subdividan": "subdividir",
    "subdividas": "subdividir",
    "subdivide": "subdividir",
    "subdividen": "subdividir",
    "subdivides": "subdividir",
    "subdividid": "subdividir",
    "subdividida": "subdividir",
    "subdivididas": "subdividir",
    "subdividido": "subdividir",
    "subdivididos": "subdividir",
    "subdividiendo": "subdividir",
    "subdividiera": "subdividir",
    "subdividierais": "subdividir",
    "subdividieran": "subdividir",
    "subdividieras": "subdividir",
    "subdividiere": "subdividir",
    "subdividiereis": "subdividir",
    "subdividieren": "subdividir",
    "subdividieres": "subdividir",
    "subdividieron": "subdividir",
    "subdividiese": "subdividir",
    "subdividieseis": "subdividir",
    "subdividiesen": "subdividir",
    "subdividieses": "subdividir",
    "subdividimos": "subdividir",
    "subdividiremos": "subdividir",
    "subdividirá": "subdividir",
    "subdividirán": "subdividir",
    "subdividirás": "subdividir",
    "subdividiré": "subdividir",
    "subdividiréis": "subdividir",
    "subdividiría": "subdividir",
    "subdividiríais": "subdividir",
    "subdividiríamos": "subdividir",
    "subdividirían": "subdividir",
    "subdividirías": "subdividir",
    "subdividiste": "subdividir",
    "subdividisteis": "subdividir",
    "subdividiéramos": "subdividir",
    "subdividiéremos": "subdividir",
    "subdividiésemos": "subdividir",
    "subdividió": "subdividir",
    "subdivido": "subdividir",
    "subdividáis": "subdividir",
    "subdividí": "subdividir",
    "subdividía": "subdividir",
    "subdividíais": "subdividir",
    "subdividíamos": "subdividir",
    "subdividían": "subdividir",
    "subdividías": "subdividir",
    "subdividís": "subdividir",
    "subdivisiones": "subdivisión",
    "subdiáconos": "subdiácono",
    "subdominantes": "subdominante",
    "subdominios": "subdominio",
    "subducciones": "subducción",
    "subdupla": "subduplo",
    "subduplas": "subduplo",
    "subduplos": "subduplo",
    "subdurales": "subdural",
    "sube": "subir",
    "subejecuciones": "subejecución",
    "subempleada": "subempleado",
    "subempleadas": "subempleado",
    "subempleados": "subempleado",
    "subempleos": "subempleo",
    "suben": "subir",
    "subenfriada": "subenfriado",
    "subenfriadas": "subenfriado",
    "subenfriados": "subenfriado",
    "subeos": "subeo",
    "suberificaciones": "suberificación",
    "suberinas": "suberina",
    "suberizaciones": "suberización",
    "suberosa": "suberoso",
    "suberosas": "suberoso",
    "suberosos": "suberoso",
    "subes": "subir",
    "subescapulares": "subescapular",
    "subespacios": "subespacio",
    "subespecies": "subespecie",
    "subespinales": "subespinal",
    "subesquemas": "subesquema",
    "subestaciones": "subestación",
    "subestatales": "subestatal",
    "subestima": "subestimar",
    "subestimaba": "subestimar",
    "subestimabais": "subestimar",
    "subestimaban": "subestimar",
    "subestimabas": "subestimar",
    "subestimaciones": "subestimación",
    "subestimad": "subestimar",
    "subestimada": "subestimar",
    "subestimadas": "subestimar",
    "subestimado": "subestimar",
    "subestimados": "subestimar",
    "subestimamos": "subestimar",
    "subestiman": "subestimar",
    "subestimando": "subestimar",
    "subestimara": "subestimar",
    "subestimarais": "subestimar",
    "subestimaran": "subestimar",
    "subestimaras": "subestimar",
    "subestimare": "subestimar",
    "subestimareis": "subestimar",
    "subestimaremos": "subestimar",
    "subestimaren": "subestimar",
    "subestimares": "subestimar",
    "subestimaron": "subestimar",
    "subestimará": "subestimar",
    "subestimarán": "subestimar",
    "subestimarás": "subestimar",
    "subestimaré": "subestimar",
    "subestimaréis": "subestimar",
    "subestimaría": "subestimar",
    "subestimaríais": "subestimar",
    "subestimaríamos": "subestimar",
    "subestimarían": "subestimar",
    "subestimarías": "subestimar",
    "subestimas": "subestimar",
    "subestimase": "subestimar",
    "subestimaseis": "subestimar",
    "subestimasen": "subestimar",
    "subestimases": "subestimar",
    "subestimaste": "subestimar",
    "subestimasteis": "subestimar",
    "subestime": "subestimar",
    "subestimemos": "subestimar",
    "subestimen": "subestimar",
    "subestimes": "subestimar",
    "subestimo": "subestimar",
    "subestimábamos": "subestimar",
    "subestimáis": "subestimar",
    "subestimáramos": "subestimar",
    "subestimáremos": "subestimar",
    "subestimásemos": "subestimar",
    "subestimé": "subestimar",
    "subestiméis": "subestimar",
    "subestimó": "subestimar",
    "subexposiciones": "subexposición",
    "subfacturaciones": "subfacturación",
    "subfamilias": "subfamilia",
    "subfebriles": "subfebril",
    "subfiadora": "subfiador",
    "subfiadoras": "subfiador",
    "subfiadores": "subfiador",
    "subfibras": "subfibra",
    "subfletadores": "subfletador",
    "subfletamientos": "subfletamiento",
    "subfonémica": "subfonémico",
    "subfonémicas": "subfonémico",
    "subfonémicos": "subfonémico",
    "subfrénica": "subfrénico",
    "subfrénicas": "subfrénico",
    "subfrénicos": "subfrénico",
    "subfunciones": "subfunción",
    "subfusiles": "subfusil",
    "subfusiones": "subfusión",
    "subfósiles": "subfósil",
    "subgalatos": "subgalato",
    "subgarantes": "subgarante",
    "subgingivales": "subgingival",
    "subglótica": "subglótico",
    "subglóticas": "subglótico",
    "subglóticos": "subglótico",
    "subgobernadora": "subgobernador",
    "subgobernadoras": "subgobernador",
    "subgobernadores": "subgobernador",
    "subgranos": "subgrano",
    "subgrupos": "subgrupo",
    "subgéneros": "subgénero",
    "subhepática": "subhepático",
    "subhepáticas": "subhepático",
    "subhepáticos": "subhepático",
    "subid": "subir",
    "subida": "subir",
    "subidas": "subir",
    "subidentificaciones": "subidentificación",
    "subidera": "subidero",
    "subideras": "subidero",
    "subideros": "subidero",
    "subido": "subir",
    "subidos": "subir",
    "subiendo": "subir",
    "subiera": "subir",
    "subierais": "subir",
    "subieran": "subir",
    "subieras": "subir",
    "subiere": "subir",
    "subiereis": "subir",
    "subieren": "subir",
    "subieres": "subir",
    "subieron": "subir",
    "subiese": "subir",
    "subieseis": "subir",
    "subiesen": "subir",
    "subieses": "subir",
    "subilarcs": "subilarc",
    "subimientos": "subimiento",
    "subimos": "subir",
    "subincisiones": "subincisión",
    "subincisos": "subinciso",
    "subinfeudaciones": "subinfeudación",
    "subinquilinos": "subinquilino",
    "subinspecciones": "subinspección",
    "subinspectora": "subinspector",
    "subinspectoras": "subinspector",
    "subinspectores": "subinspector",
    "subintendencias": "subintendencia",
    "subintendentes": "subintendente",
    "subintra": "subintrar",
    "subintraba": "subintrar",
    "subintrabais": "subintrar",
    "subintraban": "subintrar",
    "subintrabas": "subintrar",
    "subintraciones": "subintración",
    "subintrad": "subintrar",
    "subintrada": "subintrar",
    "subintradas": "subintrar",
    "subintrado": "subintrar",
    "subintrados": "subintrar",
    "subintramos": "subintrar",
    "subintran": "subintrar",
    "subintrando": "subintrar",
    "subintrantes": "subintrante",
    "subintrara": "subintrar",
    "subintrarais": "subintrar",
    "subintraran": "subintrar",
    "subintraras": "subintrar",
    "subintrare": "subintrar",
    "subintrareis": "subintrar",
    "subintraremos": "subintrar",
    "subintraren": "subintrar",
    "subintrares": "subintrar",
    "subintraron": "subintrar",
    "subintrará": "subintrar",
    "subintrarán": "subintrar",
    "subintrarás": "subintrar",
    "subintraré": "subintrar",
    "subintraréis": "subintrar",
    "subintraría": "subintrar",
    "subintraríais": "subintrar",
    "subintraríamos": "subintrar",
    "subintrarían": "subintrar",
    "subintrarías": "subintrar",
    "subintras": "subintrar",
    "subintrase": "subintrar",
    "subintraseis": "subintrar",
    "subintrasen": "subintrar",
    "subintrases": "subintrar",
    "subintraste": "subintrar",
    "subintrasteis": "subintrar",
    "subintre": "subintrar",
    "subintremos": "subintrar",
    "subintren": "subintrar",
    "subintres": "subintrar",
    "subintro": "subintrar",
    "subintrábamos": "subintrar",
    "subintráis": "subintrar",
    "subintráramos": "subintrar",
    "subintráremos": "subintrar",
    "subintrásemos": "subintrar",
    "subintré": "subintrar",
    "subintréis": "subintrar",
    "subintró": "subintrar",
    "subinversiones": "subinversión",
    "subiremos": "subir",
    "subirrigaciones": "subirrigación",
    "subirá": "subir",
    "subirán": "subir",
    "subirás": "subir",
    "subiré": "subir",
    "subiréis": "subir",
    "subiría": "subir",
    "subiríais": "subir",
    "subiríamos": "subir",
    "subirían": "subir",
    "subirías": "subir",
    "subiste": "subir",
    "subisteis": "subir",
    "subitánea": "subitáneo",
    "subitáneas": "subitáneo",
    "subitáneos": "subitáneo",
    "subiéramos": "subir",
    "subiéremos": "subir",
    "subiésemos": "subir",
    "subió": "subir",
    "subjefes": "subjefe",
    "subjetiva": "subjetivo",
    "subjetivas": "subjetivo",
    "subjetividades": "subjetividad",
    "subjetivismos": "subjetivismo",
    "subjetivistas": "subjetivista",
    "subjetivos": "subjetivo",
    "subjunciones": "subjunción",
    "subjuntas": "subjunta",
    "subjuntiva": "subjuntivo",
    "subjuntivas": "subjuntivo",
    "subjuntivos": "subjuntivo",
    "sublegados": "sublegado",
    "subleva": "sublevar",
    "sublevaba": "sublevar",
    "sublevabais": "sublevar",
    "sublevaban": "sublevar",
    "sublevabas": "sublevar",
    "sublevaciones": "sublevación",
    "sublevad": "sublevar",
    "sublevada": "sublevar",
    "sublevadas": "sublevar",
    "sublevado": "sublevar",
    "sublevados": "sublevar",
    "sublevamos": "sublevar",
    "sublevan": "sublevar",
    "sublevando": "sublevar",
    "sublevara": "sublevar",
    "sublevarais": "sublevar",
    "sublevaran": "sublevar",
    "sublevaras": "sublevar",
    "sublevare": "sublevar",
    "sublevareis": "sublevar",
    "sublevaremos": "sublevar",
    "sublevaren": "sublevar",
    "sublevares": "sublevar",
    "sublevaron": "sublevar",
    "sublevará": "sublevar",
    "sublevarán": "sublevar",
    "sublevarás": "sublevar",
    "sublevaré": "sublevar",
    "sublevaréis": "sublevar",
    "sublevaría": "sublevar",
    "sublevaríais": "sublevar",
    "sublevaríamos": "sublevar",
    "sublevarían": "sublevar",
    "sublevarías": "sublevar",
    "sublevas": "sublevar",
    "sublevase": "sublevar",
    "sublevaseis": "sublevar",
    "sublevasen": "sublevar",
    "sublevases": "sublevar",
    "sublevaste": "sublevar",
    "sublevasteis": "sublevar",
    "subleve": "sublevar",
    "sublevemos": "sublevar",
    "subleven": "sublevar",
    "subleves": "sublevar",
    "sublevo": "sublevar",
    "sublevábamos": "sublevar",
    "subleváis": "sublevar",
    "subleváramos": "sublevar",
    "subleváremos": "sublevar",
    "sublevásemos": "sublevar",
    "sublevé": "sublevar",
    "sublevéis": "sublevar",
    "sublevó": "sublevar",
    "sublicencias": "sublicencia",
    "sublima": "sublimar",
    "sublimaba": "sublimar",
    "sublimabais": "sublimar",
    "sublimaban": "sublimar",
    "sublimabas": "sublimar",
    "sublimaciones": "sublimación",
    "sublimad": "sublimar",
    "sublimada": "sublimar",
    "sublimadas": "sublimar",
    "sublimado": "sublimar",
    "sublimadores": "sublimador",
    "sublimados": "sublimar",
    "sublimamos": "sublimar",
    "subliman": "sublimar",
    "sublimando": "sublimar",
    "sublimara": "sublimar",
    "sublimarais": "sublimar",
    "sublimaran": "sublimar",
    "sublimaras": "sublimar",
    "sublimare": "sublimar",
    "sublimareis": "sublimar",
    "sublimaremos": "sublimar",
    "sublimaren": "sublimar",
    "sublimares": "sublimar",
    "sublimaron": "sublimar",
    "sublimará": "sublimar",
    "sublimarán": "sublimar",
    "sublimarás": "sublimar",
    "sublimaré": "sublimar",
    "sublimaréis": "sublimar",
    "sublimaría": "sublimar",
    "sublimaríais": "sublimar",
    "sublimaríamos": "sublimar",
    "sublimarían": "sublimar",
    "sublimarías": "sublimar",
    "sublimas": "sublimar",
    "sublimase": "sublimar",
    "sublimaseis": "sublimar",
    "sublimasen": "sublimar",
    "sublimases": "sublimar",
    "sublimaste": "sublimar",
    "sublimasteis": "sublimar",
    "sublime": "sublimar",
    "sublimemos": "sublimar",
    "sublimen": "sublimar",
    "sublimes": "sublime",
    "sublimidades": "sublimidad",
    "subliminales": "subliminal",
    "subliminares": "subliminar",
    "sublimo": "sublimar",
    "sublimábamos": "sublimar",
    "sublimáis": "sublimar",
    "sublimáramos": "sublimar",
    "sublimáremos": "sublimar",
    "sublimásemos": "sublimar",
    "sublimé": "sublimar",
    "subliméis": "sublimar",
    "sublimó": "sublimar",
    "sublinguales": "sublingual",
    "sublistas": "sublista",
    "sublunares": "sublunar",
    "subluxaciones": "subluxación",
    "sublímites": "sublímite",
    "submacros": "submacro",
    "submandibulares": "submandibular",
    "submarina": "submarino",
    "submarinas": "submarino",
    "submarinismos": "submarinismo",
    "submarinistas": "submarinista",
    "submarinos": "submarino",
    "submatrices": "submatriz",
    "submaxilares": "submaxilar",
    "submensajes": "submensaje",
    "submentoniana": "submentoniano",
    "submentonianas": "submentoniano",
    "submentonianos": "submentoniano",
    "submenus": "submenú",
    "submetacéntrica": "submetacéntrico",
    "submetacéntricas": "submetacéntrico",
    "submetacéntricos": "submetacéntrico",
    "submicroestructuras": "submicroestructura",
    "submicrométrica": "submicrométrico",
    "submicrométricas": "submicrométrico",
    "submicrométricos": "submicrométrico",
    "submicrones": "submicrón",
    "submicrosomas": "submicrosoma",
    "submoléculas": "submolécula",
    "submucosa": "submucoso",
    "submucosas": "submucoso",
    "submucosos": "submucoso",
    "submuestras": "submuestra",
    "submundos": "submundo",
    "submódulos": "submódulo",
    "submúltipla": "submúltiplo",
    "submúltiplas": "submúltiplo",
    "submúltiplos": "submúltiplo",
    "subnasales": "subnasal",
    "subnitratos": "subnitrato",
    "subniveles": "subnivel",
    "subnormales": "subnormal",
    "subnormalidades": "subnormalidad",
    "subnotas": "subnota",
    "subnúmeros": "subnúmero",
    "subo": "subir",
    "subobjetivos": "subobjetivo",
    "suboccipitales": "suboccipital",
    "subocupaciones": "subocupación",
    "suboficiales": "suboficial",
    "suboptimizaciones": "suboptimización",
    "suborbitaria": "suborbitario",
    "suborbitarias": "suborbitario",
    "suborbitarios": "suborbitario",
    "subordina": "subordinar",
    "subordinaba": "subordinar",
    "subordinabais": "subordinar",
    "subordinaban": "subordinar",
    "subordinabas": "subordinar",
    "subordinaciones": "subordinación",
    "subordinad": "subordinar",
    "subordinada": "subordinar",
    "subordinadas": "subordinar",
    "subordinado": "subordinar",
    "subordinados": "subordinar",
    "subordinamos": "subordinar",
    "subordinan": "subordinar",
    "subordinando": "subordinar",
    "subordinantes": "subordinante",
    "subordinara": "subordinar",
    "subordinarais": "subordinar",
    "subordinaran": "subordinar",
    "subordinaras": "subordinar",
    "subordinare": "subordinar",
    "subordinareis": "subordinar",
    "subordinaremos": "subordinar",
    "subordinaren": "subordinar",
    "subordinares": "subordinar",
    "subordinaron": "subordinar",
    "subordinará": "subordinar",
    "subordinarán": "subordinar",
    "subordinarás": "subordinar",
    "subordinaré": "subordinar",
    "subordinaréis": "subordinar",
    "subordinaría": "subordinar",
    "subordinaríais": "subordinar",
    "subordinaríamos": "subordinar",
    "subordinarían": "subordinar",
    "subordinarías": "subordinar",
    "subordinas": "subordinar",
    "subordinase": "subordinar",
    "subordinaseis": "subordinar",
    "subordinasen": "subordinar",
    "subordinases": "subordinar",
    "subordinaste": "subordinar",
    "subordinasteis": "subordinar",
    "subordine": "subordinar",
    "subordinemos": "subordinar",
    "subordinen": "subordinar",
    "subordines": "subordinar",
    "subordino": "subordinar",
    "subordinábamos": "subordinar",
    "subordináis": "subordinar",
    "subordináramos": "subordinar",
    "subordináremos": "subordinar",
    "subordinásemos": "subordinar",
    "subordiné": "subordinar",
    "subordinéis": "subordinar",
    "subordinó": "subordinar",
    "subparietales": "subparietal",
    "subperforaciones": "subperforación",
    "subperitoneales": "subperitoneal",
    "subperióstica": "subperióstico",
    "subperiósticas": "subperióstico",
    "subperiósticos": "subperióstico",
    "subpilórica": "subpilórico",
    "subpilóricas": "subpilórico",
    "subpilóricos": "subpilórico",
    "subpoblaciones": "subpoblación",
    "subpolares": "subpolar",
    "subpoplítea": "subpoplíteo",
    "subpoplíteas": "subpoplíteo",
    "subpoplíteos": "subpoplíteo",
    "subportadoras": "subportadora",
    "subposiciones": "subposición",
    "subprefectos": "subprefecto",
    "subprefecturas": "subprefectura",
    "subpresiones": "subpresión",
    "subprestatarios": "subprestatario",
    "subproblemas": "subproblema",
    "subproductos": "subproducto",
    "subprogramas": "subprograma",
    "subproletariados": "subproletariado",
    "subproyectos": "subproyecto",
    "subpréstamos": "subpréstamo",
    "subpubiana": "subpubiano",
    "subpubianas": "subpubiano",
    "subpubianos": "subpubiano",
    "subraya": "subrayar",
    "subrayaba": "subrayar",
    "subrayabais": "subrayar",
    "subrayaban": "subrayar",
    "subrayabas": "subrayar",
    "subrayables": "subrayable",
    "subrayad": "subrayar",
    "subrayada": "subrayar",
    "subrayadas": "subrayar",
    "subrayado": "subrayar",
    "subrayados": "subrayar",
    "subrayamos": "subrayar",
    "subrayan": "subrayar",
    "subrayando": "subrayar",
    "subrayara": "subrayar",
    "subrayarais": "subrayar",
    "subrayaran": "subrayar",
    "subrayaras": "subrayar",
    "subrayare": "subrayar",
    "subrayareis": "subrayar",
    "subrayaremos": "subrayar",
    "subrayaren": "subrayar",
    "subrayares": "subrayar",
    "subrayaron": "subrayar",
    "subrayará": "subrayar",
    "subrayarán": "subrayar",
    "subrayarás": "subrayar",
    "subrayaré": "subrayar",
    "subrayaréis": "subrayar",
    "subrayaría": "subrayar",
    "subrayaríais": "subrayar",
    "subrayaríamos": "subrayar",
    "subrayarían": "subrayar",
    "subrayarías": "subrayar",
    "subrayas": "subrayar",
    "subrayase": "subrayar",
    "subrayaseis": "subrayar",
    "subrayasen": "subrayar",
    "subrayases": "subrayar",
    "subrayaste": "subrayar",
    "subrayasteis": "subrayar",
    "subraye": "subrayar",
    "subrayemos": "subrayar",
    "subrayen": "subrayar",
    "subrayes": "subrayar",
    "subrayo": "subrayar",
    "subrayábamos": "subrayar",
    "subrayáis": "subrayar",
    "subrayáramos": "subrayar",
    "subrayáremos": "subrayar",
    "subrayásemos": "subrayar",
    "subrayé": "subrayar",
    "subrayéis": "subrayar",
    "subrayó": "subrayar",
    "subredes": "subred",
    "subregionales": "subregional",
    "subregiones": "subregión",
    "subreinos": "subreino",
    "subrepciones": "subrepción",
    "subrepresentada": "subrepresentado",
    "subrepresentadas": "subrepresentado",
    "subrepresentados": "subrepresentado",
    "subrepticia": "subrepticio",
    "subrepticias": "subrepticio",
    "subrepticios": "subrepticio",
    "subroga": "subrogar",
    "subrogaba": "subrogar",
    "subrogabais": "subrogar",
    "subrogaban": "subrogar",
    "subrogabas": "subrogar",
    "subrogaciones": "subrogación",
    "subrogad": "subrogar",
    "subrogada": "subrogar",
    "subrogadas": "subrogar",
    "subrogado": "subrogar",
    "subrogados": "subrogar",
    "subrogamos": "subrogar",
    "subrogan": "subrogar",
    "subrogando": "subrogar",
    "subrogara": "subrogar",
    "subrogarais": "subrogar",
    "subrogaran": "subrogar",
    "subrogaras": "subrogar",
    "subrogare": "subrogar",
    "subrogareis": "subrogar",
    "subrogaremos": "subrogar",
    "subrogaren": "subrogar",
    "subrogares": "subrogar",
    "subrogaron": "subrogar",
    "subrogará": "subrogar",
    "subrogarán": "subrogar",
    "subrogarás": "subrogar",
    "subrogaré": "subrogar",
    "subrogaréis": "subrogar",
    "subrogaría": "subrogar",
    "subrogaríais": "subrogar",
    "subrogaríamos": "subrogar",
    "subrogarían": "subrogar",
    "subrogarías": "subrogar",
    "subrogas": "subrogar",
    "subrogase": "subrogar",
    "subrogaseis": "subrogar",
    "subrogasen": "subrogar",
    "subrogases": "subrogar",
    "subrogaste": "subrogar",
    "subrogasteis": "subrogar",
    "subrogo": "subrogar",
    "subrogue": "subrogar",
    "subroguemos": "subrogar",
    "subroguen": "subrogar",
    "subrogues": "subrogar",
    "subrogué": "subrogar",
    "subroguéis": "subrogar",
    "subrogábamos": "subrogar",
    "subrogáis": "subrogar",
    "subrogáramos": "subrogar",
    "subrogáremos": "subrogar",
    "subrogásemos": "subrogar",
    "subrogó": "subrogar",
    "subrubros": "subrubro",
    "subrutinas": "subrutina",
    "subsahariana": "subsahariano",
    "subsaharianas": "subsahariano",
    "subsaharianos": "subsahariano",
    "subsana": "subsanar",
    "subsanaba": "subsanar",
    "subsanabais": "subsanar",
    "subsanaban": "subsanar",
    "subsanabas": "subsanar",
    "subsanables": "subsanable",
    "subsanaciones": "subsanación",
    "subsanad": "subsanar",
    "subsanada": "subsanar",
    "subsanadas": "subsanar",
    "subsanado": "subsanar",
    "subsanados": "subsanar",
    "subsanamos": "subsanar",
    "subsanan": "subsanar",
    "subsanando": "subsanar",
    "subsanara": "subsanar",
    "subsanarais": "subsanar",
    "subsanaran": "subsanar",
    "subsanaras": "subsanar",
    "subsanare": "subsanar",
    "subsanareis": "subsanar",
    "subsanaremos": "subsanar",
    "subsanaren": "subsanar",
    "subsanares": "subsanar",
    "subsanaron": "subsanar",
    "subsanará": "subsanar",
    "subsanarán": "subsanar",
    "subsanarás": "subsanar",
    "subsanaré": "subsanar",
    "subsanaréis": "subsanar",
    "subsanaría": "subsanar",
    "subsanaríais": "subsanar",
    "subsanaríamos": "subsanar",
    "subsanarían": "subsanar",
    "subsanarías": "subsanar",
    "subsanas": "subsanar",
    "subsanase": "subsanar",
    "subsanaseis": "subsanar",
    "subsanasen": "subsanar",
    "subsanases": "subsanar",
    "subsanaste": "subsanar",
    "subsanasteis": "subsanar",
    "subsane": "subsanar",
    "subsanemos": "subsanar",
    "subsanen": "subsanar",
    "subsanes": "subsanar",
    "subsano": "subsanar",
    "subsanábamos": "subsanar",
    "subsanáis": "subsanar",
    "subsanáramos": "subsanar",
    "subsanáremos": "subsanar",
    "subsanásemos": "subsanar",
    "subsané": "subsanar",
    "subsanéis": "subsanar",
    "subsanó": "subsanar",
    "subscriba": "subscribir",
    "subscribamos": "subscribir",
    "subscriban": "subscribir",
    "subscribas": "subscribir",
    "subscribe": "subscribir",
    "subscriben": "subscribir",
    "subscribes": "subscribir",
    "subscribid": "subscribir",
    "subscribiendo": "subscribir",
    "subscribiera": "subscribir",
    "subscribierais": "subscribir",
    "subscribieran": "subscribir",
    "subscribieras": "subscribir",
    "subscribiere": "subscribir",
    "subscribiereis": "subscribir",
    "subscribieren": "subscribir",
    "subscribieres": "subscribir",
    "subscribieron": "subscribir",
    "subscribiese": "subscribir",
    "subscribieseis": "subscribir",
    "subscribiesen": "subscribir",
    "subscribieses": "subscribir",
    "subscribimos": "subscribir",
    "subscribiremos": "subscribir",
    "subscribirá": "subscribir",
    "subscribirán": "subscribir",
    "subscribirás": "subscribir",
    "subscribiré": "subscribir",
    "subscribiréis": "subscribir",
    "subscribiría": "subscribir",
    "subscribiríais": "subscribir",
    "subscribiríamos": "subscribir",
    "subscribirían": "subscribir",
    "subscribirías": "subscribir",
    "subscribiste": "subscribir",
    "subscribisteis": "subscribir",
    "subscribiéramos": "subscribir",
    "subscribiéremos": "subscribir",
    "subscribiésemos": "subscribir",
    "subscribió": "subscribir",
    "subscribo": "subscribir",
    "subscribáis": "subscribir",
    "subscribí": "subscribir",
    "subscribía": "subscribir",
    "subscribíais": "subscribir",
    "subscribíamos": "subscribir",
    "subscribían": "subscribir",
    "subscribías": "subscribir",
    "subscribís": "subscribir",
    "subscripciones": "subscripción",
    "subscriptora": "subscriptor",
    "subscriptoras": "subscriptor",
    "subscriptores": "subscriptor",
    "subscrita": "subscribir",
    "subscritas": "subscribir",
    "subscrito": "subscribir",
    "subscritora": "subscritor",
    "subscritoras": "subscritor",
    "subscritores": "subscritor",
    "subscritos": "subscribir",
    "subsecciones": "subsección",
    "subsecretaria": "subsecretario",
    "subsecretarias": "subsecretario",
    "subsecretarios": "subsecretario",
    "subsecretarías": "subsecretaría",
    "subsectores": "subsector",
    "subsecuentes": "subsecuente",
    "subseguid": "subseguir",
    "subseguida": "subseguir",
    "subseguidas": "subseguir",
    "subseguido": "subseguir",
    "subseguidos": "subseguir",
    "subseguimos": "subseguir",
    "subseguiremos": "subseguir",
    "subseguirá": "subseguir",
    "subseguirán": "subseguir",
    "subseguirás": "subseguir",
    "subseguiré": "subseguir",
    "subseguiréis": "subseguir",
    "subseguiría": "subseguir",
    "subseguiríais": "subseguir",
    "subseguiríamos": "subseguir",
    "subseguirían": "subseguir",
    "subseguirías": "subseguir",
    "subseguiste": "subseguir",
    "subseguisteis": "subseguir",
    "subseguí": "subseguir",
    "subseguía": "subseguir",
    "subseguíais": "subseguir",
    "subseguíamos": "subseguir",
    "subseguían": "subseguir",
    "subseguías": "subseguir",
    "subseguís": "subseguir",
    "subserosa": "subseroso",
    "subserosas": "subseroso",
    "subserosos": "subseroso",
    "subsidencias": "subsidencia",
    "subsidia": "subsidiar",
    "subsidiaba": "subsidiar",
    "subsidiabais": "subsidiar",
    "subsidiaban": "subsidiar",
    "subsidiabas": "subsidiar",
    "subsidiad": "subsidiar",
    "subsidiada": "subsidiar",
    "subsidiadas": "subsidiar",
    "subsidiado": "subsidiar",
    "subsidiados": "subsidiar",
    "subsidiamos": "subsidiar",
    "subsidian": "subsidiar",
    "subsidiando": "subsidiar",
    "subsidiara": "subsidiar",
    "subsidiarais": "subsidiar",
    "subsidiaran": "subsidiar",
    "subsidiaras": "subsidiar",
    "subsidiare": "subsidiar",
    "subsidiareis": "subsidiar",
    "subsidiaremos": "subsidiar",
    "subsidiaren": "subsidiar",
    "subsidiares": "subsidiar",
    "subsidiaria": "subsidiario",
    "subsidiarias": "subsidiario",
    "subsidiariedades": "subsidiariedad",
    "subsidiarios": "subsidiario",
    "subsidiaron": "subsidiar",
    "subsidiará": "subsidiar",
    "subsidiarán": "subsidiar",
    "subsidiarás": "subsidiar",
    "subsidiaré": "subsidiar",
    "subsidiaréis": "subsidiar",
    "subsidiaría": "subsidiar",
    "subsidiaríais": "subsidiar",
    "subsidiaríamos": "subsidiar",
    "subsidiarían": "subsidiar",
    "subsidiarías": "subsidiar",
    "subsidias": "subsidiar",
    "subsidiase": "subsidiar",
    "subsidiaseis": "subsidiar",
    "subsidiasen": "subsidiar",
    "subsidiases": "subsidiar",
    "subsidiaste": "subsidiar",
    "subsidiasteis": "subsidiar",
    "subsidie": "subsidiar",
    "subsidiemos": "subsidiar",
    "subsidien": "subsidiar",
    "subsidies": "subsidiar",
    "subsidio": "subsidiar",
    "subsidios": "subsidio",
    "subsidiábamos": "subsidiar",
    "subsidiáis": "subsidiar",
    "subsidiáramos": "subsidiar",
    "subsidiáremos": "subsidiar",
    "subsidiásemos": "subsidiar",
    "subsidié": "subsidiar",
    "subsidiéis": "subsidiar",
    "subsidió": "subsidiar",
    "subsiga": "subseguir",
    "subsigamos": "subseguir",
    "subsigan": "subseguir",
    "subsigas": "subseguir",
    "subsigo": "subseguir",
    "subsigue": "subseguir",
    "subsiguen": "subseguir",
    "subsigues": "subseguir",
    "subsiguiendo": "subseguir",
    "subsiguientes": "subsiguiente",
    "subsiguiera": "subseguir",
    "subsiguierais": "subseguir",
    "subsiguieran": "subseguir",
    "subsiguieras": "subseguir",
    "subsiguiere": "subseguir",
    "subsiguiereis": "subseguir",
    "subsiguieren": "subseguir",
    "subsiguieres": "subseguir",
    "subsiguieron": "subseguir",
    "subsiguiese": "subseguir",
    "subsiguieseis": "subseguir",
    "subsiguiesen": "subseguir",
    "subsiguieses": "subseguir",
    "subsiguiéramos": "subseguir",
    "subsiguiéremos": "subseguir",
    "subsiguiésemos": "subseguir",
    "subsiguió": "subseguir",
    "subsigáis": "subseguir",
    "subsista": "subsistir",
    "subsistamos": "subsistir",
    "subsistan": "subsistir",
    "subsistas": "subsistir",
    "subsiste": "subsistir",
    "subsistemas": "subsistema",
    "subsisten": "subsistir",
    "subsistencias": "subsistencia",
    "subsistentes": "subsistente",
    "subsistes": "subsistir",
    "subsistid": "subsistir",
    "subsistida": "subsistir",
    "subsistidas": "subsistir",
    "subsistido": "subsistir",
    "subsistidos": "subsistir",
    "subsistiendo": "subsistir",
    "subsistiera": "subsistir",
    "subsistierais": "subsistir",
    "subsistieran": "subsistir",
    "subsistieras": "subsistir",
    "subsistiere": "subsistir",
    "subsistiereis": "subsistir",
    "subsistieren": "subsistir",
    "subsistieres": "subsistir",
    "subsistieron": "subsistir",
    "subsistiese": "subsistir",
    "subsistieseis": "subsistir",
    "subsistiesen": "subsistir",
    "subsistieses": "subsistir",
    "subsistimos": "subsistir",
    "subsistiremos": "subsistir",
    "subsistirá": "subsistir",
    "subsistirán": "subsistir",
    "subsistirás": "subsistir",
    "subsistiré": "subsistir",
    "subsistiréis": "subsistir",
    "subsistiría": "subsistir",
    "subsistiríais": "subsistir",
    "subsistiríamos": "subsistir",
    "subsistirían": "subsistir",
    "subsistirías": "subsistir",
    "subsististe": "subsistir",
    "subsististeis": "subsistir",
    "subsistiéramos": "subsistir",
    "subsistiéremos": "subsistir",
    "subsistiésemos": "subsistir",
    "subsistió": "subsistir",
    "subsisto": "subsistir",
    "subsistáis": "subsistir",
    "subsistí": "subsistir",
    "subsistía": "subsistir",
    "subsistíais": "subsistir",
    "subsistíamos": "subsistir",
    "subsistían": "subsistir",
    "subsistías": "subsistir",
    "subsistís": "subsistir",
    "subsoladores": "subsolador",
    "subsolanos": "subsolano",
    "substancia": "substanciar",
    "substanciaba": "substanciar",
    "substanciabais": "substanciar",
    "substanciaban": "substanciar",
    "substanciabas": "substanciar",
    "substanciaciones": "substanciación",
    "substanciad": "substanciar",
    "substanciada": "substanciar",
    "substanciadas": "substanciar",
    "substanciado": "substanciar",
    "substanciados": "substanciar",
    "substanciales": "substancial",
    "substancialidades": "substancialidad",
    "substanciamos": "substanciar",
    "substancian": "substanciar",
    "substanciando": "substanciar",
    "substanciara": "substanciar",
    "substanciarais": "substanciar",
    "substanciaran": "substanciar",
    "substanciaras": "substanciar",
    "substanciare": "substanciar",
    "substanciareis": "substanciar",
    "substanciaremos": "substanciar",
    "substanciaren": "substanciar",
    "substanciares": "substanciar",
    "substanciaron": "substanciar",
    "substanciará": "substanciar",
    "substanciarán": "substanciar",
    "substanciarás": "substanciar",
    "substanciaré": "substanciar",
    "substanciaréis": "substanciar",
    "substanciaría": "substanciar",
    "substanciaríais": "substanciar",
    "substanciaríamos": "substanciar",
    "substanciarían": "substanciar",
    "substanciarías": "substanciar",
    "substancias": "substanciar",
    "substanciase": "substanciar",
    "substanciaseis": "substanciar",
    "substanciasen": "substanciar",
    "substanciases": "substanciar",
    "substanciaste": "substanciar",
    "substanciasteis": "substanciar",
    "substancie": "substanciar",
    "substanciemos": "substanciar",
    "substancien": "substanciar",
    "substancies": "substanciar",
    "substancio": "substanciar",
    "substanciosa": "substancioso",
    "substanciosas": "substancioso",
    "substanciosos": "substancioso",
    "substanciábamos": "substanciar",
    "substanciáis": "substanciar",
    "substanciáramos": "substanciar",
    "substanciáremos": "substanciar",
    "substanciásemos": "substanciar",
    "substancié": "substanciar",
    "substanciéis": "substanciar",
    "substanció": "substanciar",
    "substantiva": "substantivo",
    "substantivaba": "substantivar",
    "substantivabais": "substantivar",
    "substantivaban": "substantivar",
    "substantivabas": "substantivar",
    "substantivaciones": "substantivación",
    "substantivad": "substantivar",
    "substantivada": "substantivar",
    "substantivadas": "substantivar",
    "substantivado": "substantivar",
    "substantivados": "substantivar",
    "substantivamos": "substantivar",
    "substantivan": "substantivar",
    "substantivando": "substantivar",
    "substantivara": "substantivar",
    "substantivarais": "substantivar",
    "substantivaran": "substantivar",
    "substantivaras": "substantivar",
    "substantivare": "substantivar",
    "substantivareis": "substantivar",
    "substantivaremos": "substantivar",
    "substantivaren": "substantivar",
    "substantivares": "substantivar",
    "substantivaron": "substantivar",
    "substantivará": "substantivar",
    "substantivarán": "substantivar",
    "substantivarás": "substantivar",
    "substantivaré": "substantivar",
    "substantivaréis": "substantivar",
    "substantivaría": "substantivar",
    "substantivaríais": "substantivar",
    "substantivaríamos": "substantivar",
    "substantivarían": "substantivar",
    "substantivarías": "substantivar",
    "substantivas": "substantivo",
    "substantivase": "substantivar",
    "substantivaseis": "substantivar",
    "substantivasen": "substantivar",
    "substantivases": "substantivar",
    "substantivaste": "substantivar",
    "substantivasteis": "substantivar",
    "substantive": "substantivar",
    "substantivemos": "substantivar",
    "substantiven": "substantivar",
    "substantives": "substantivar",
    "substantividades": "substantividad",
    "substantivo": "substantivar",
    "substantivos": "substantivo",
    "substantivábamos": "substantivar",
    "substantiváis": "substantivar",
    "substantiváramos": "substantivar",
    "substantiváremos": "substantivar",
    "substantivásemos": "substantivar",
    "substantivé": "substantivar",
    "substantivéis": "substantivar",
    "substantivó": "substantivar",
    "substituciones": "substitución",
    "substituibles": "substituible",
    "substituid": "substituir",
    "substituida": "substituir",
    "substituidas": "substituir",
    "substituido": "substituir",
    "substituidora": "substituidor",
    "substituidoras": "substituidor",
    "substituidores": "substituidor",
    "substituidos": "substituir",
    "substituimos": "substituir",
    "substituiremos": "substituir",
    "substituirá": "substituir",
    "substituirán": "substituir",
    "substituirás": "substituir",
    "substituiré": "substituir",
    "substituiréis": "substituir",
    "substituiría": "substituir",
    "substituiríais": "substituir",
    "substituiríamos": "substituir",
    "substituirían": "substituir",
    "substituirías": "substituir",
    "substituiste": "substituir",
    "substituisteis": "substituir",
    "substituta": "substituto",
    "substitutas": "substituto",
    "substitutiva": "substitutivo",
    "substitutivas": "substitutivo",
    "substitutivos": "substitutivo",
    "substitutoria": "substitutorio",
    "substitutorias": "substitutorio",
    "substitutorios": "substitutorio",
    "substitutos": "substituto",
    "substituya": "substituir",
    "substituyamos": "substituir",
    "substituyan": "substituir",
    "substituyas": "substituir",
    "substituye": "substituir",
    "substituyen": "substituir",
    "substituyendo": "substituir",
    "substituyera": "substituir",
    "substituyerais": "substituir",
    "substituyeran": "substituir",
    "substituyeras": "substituir",
    "substituyere": "substituir",
    "substituyereis": "substituir",
    "substituyeren": "substituir",
    "substituyeres": "substituir",
    "substituyeron": "substituir",
    "substituyes": "substituir",
    "substituyese": "substituir",
    "substituyeseis": "substituir",
    "substituyesen": "substituir",
    "substituyeses": "substituir",
    "substituyo": "substituir",
    "substituyáis": "substituir",
    "substituyéramos": "substituir",
    "substituyéremos": "substituir",
    "substituyésemos": "substituir",
    "substituyó": "substituir",
    "substituí": "substituir",
    "substituía": "substituir",
    "substituíais": "substituir",
    "substituíamos": "substituir",
    "substituían": "substituir",
    "substituías": "substituir",
    "substituís": "substituir",
    "substracciones": "substracción",
    "substractiva": "substractivo",
    "substractivas": "substractivo",
    "substractivos": "substractivo",
    "substrae": "substraer",
    "substraed": "substraer",
    "substraemos": "substraer",
    "substraen": "substraer",
    "substraendos": "substraendo",
    "substraeremos": "substraer",
    "substraerá": "substraer",
    "substraerán": "substraer",
    "substraerás": "substraer",
    "substraeré": "substraer",
    "substraeréis": "substraer",
    "substraería": "substraer",
    "substraeríais": "substraer",
    "substraeríamos": "substraer",
    "substraerían": "substraer",
    "substraerías": "substraer",
    "substraes": "substraer",
    "substraiga": "substraer",
    "substraigamos": "substraer",
    "substraigan": "substraer",
    "substraigas": "substraer",
    "substraigo": "substraer",
    "substraigáis": "substraer",
    "substraje": "substraer",
    "substrajera": "substraer",
    "substrajerais": "substraer",
    "substrajeran": "substraer",
    "substrajeras": "substraer",
    "substrajere": "substraer",
    "substrajereis": "substraer",
    "substrajeren": "substraer",
    "substrajeres": "substraer",
    "substrajeron": "substraer",
    "substrajese": "substraer",
    "substrajeseis": "substraer",
    "substrajesen": "substraer",
    "substrajeses": "substraer",
    "substrajimos": "substraer",
    "substrajiste": "substraer",
    "substrajisteis": "substraer",
    "substrajo": "substraer",
    "substrajéramos": "substraer",
    "substrajéremos": "substraer",
    "substrajésemos": "substraer",
    "substratos": "substrato",
    "substrayendo": "substraer",
    "substraéis": "substraer",
    "substraía": "substraer",
    "substraíais": "substraer",
    "substraíamos": "substraer",
    "substraían": "substraer",
    "substraías": "substraer",
    "substraída": "substraer",
    "substraídas": "substraer",
    "substraído": "substraer",
    "substraídos": "substraer",
    "subsucesiones": "subsucesión",
    "subsuelos": "subsuelo",
    "subsunciones": "subsunción",
    "subsuperficiales": "subsuperficial",
    "subsíncrona": "subsíncrono",
    "subsíncronas": "subsíncrono",
    "subsíncronos": "subsíncrono",
    "subsónica": "subsónico",
    "subsónicas": "subsónico",
    "subsónicos": "subsónico",
    "subtalámica": "subtalámico",
    "subtalámicas": "subtalámico",
    "subtalámicos": "subtalámico",
    "subtangentes": "subtangente",
    "subtareas": "subtarea",
    "subtechumbres": "subtechumbre",
    "subtendamos": "subtender",
    "subtended": "subtender",
    "subtendemos": "subtender",
    "subtenderemos": "subtender",
    "subtenderá": "subtender",
    "subtenderán": "subtender",
    "subtenderás": "subtender",
    "subtenderé": "subtender",
    "subtenderéis": "subtender",
    "subtendería": "subtender",
    "subtenderíais": "subtender",
    "subtenderíamos": "subtender",
    "subtenderían": "subtender",
    "subtenderías": "subtender",
    "subtendida": "subtender",
    "subtendidas": "subtender",
    "subtendido": "subtender",
    "subtendidos": "subtender",
    "subtendiendo": "subtender",
    "subtendiera": "subtender",
    "subtendierais": "subtender",
    "subtendieran": "subtender",
    "subtendieras": "subtender",
    "subtendiere": "subtender",
    "subtendiereis": "subtender",
    "subtendieren": "subtender",
    "subtendieres": "subtender",
    "subtendieron": "subtender",
    "subtendiese": "subtender",
    "subtendieseis": "subtender",
    "subtendiesen": "subtender",
    "subtendieses": "subtender",
    "subtendimos": "subtender",
    "subtendinosa": "subtendinoso",
    "subtendinosas": "subtendinoso",
    "subtendinosos": "subtendinoso",
    "subtendiste": "subtender",
    "subtendisteis": "subtender",
    "subtendiéramos": "subtender",
    "subtendiéremos": "subtender",
    "subtendiésemos": "subtender",
    "subtendió": "subtender",
    "subtendáis": "subtender",
    "subtendéis": "subtender",
    "subtendí": "subtender",
    "subtendía": "subtender",
    "subtendíais": "subtender",
    "subtendíamos": "subtender",
    "subtendían": "subtender",
    "subtendías": "subtender",
    "subtenencias": "subtenencia",
    "subtenientes": "subteniente",
    "subtensa": "subtenso",
    "subtensas": "subtenso",
    "subtensiones": "subtensión",
    "subtensos": "subtenso",
    "subteorías": "subteoría",
    "subterfugios": "subterfugio",
    "subterránea": "subterráneo",
    "subterráneas": "subterráneo",
    "subterráneos": "subterráneo",
    "subtienda": "subtender",
    "subtiendan": "subtender",
    "subtiendas": "subtender",
    "subtiende": "subtender",
    "subtienden": "subtender",
    "subtiendes": "subtender",
    "subtiendo": "subtender",
    "subtilinas": "subtilina",
    "subtipos": "subtipo",
    "subtitula": "subtitular",
    "subtitulaba": "subtitular",
    "subtitulabais": "subtitular",
    "subtitulaban": "subtitular",
    "subtitulabas": "subtitular",
    "subtitulad": "subtitular",
    "subtitulada": "subtitular",
    "subtituladas": "subtitular",
    "subtitulado": "subtitular",
    "subtitulados": "subtitular",
    "subtitulamos": "subtitular",
    "subtitulan": "subtitular",
    "subtitulando": "subtitular",
    "subtitulara": "subtitular",
    "subtitularais": "subtitular",
    "subtitularan": "subtitular",
    "subtitularas": "subtitular",
    "subtitulare": "subtitular",
    "subtitulareis": "subtitular",
    "subtitularemos": "subtitular",
    "subtitularen": "subtitular",
    "subtitulares": "subtitular",
    "subtitularon": "subtitular",
    "subtitulará": "subtitular",
    "subtitularán": "subtitular",
    "subtitularás": "subtitular",
    "subtitularé": "subtitular",
    "subtitularéis": "subtitular",
    "subtitularía": "subtitular",
    "subtitularíais": "subtitular",
    "subtitularíamos": "subtitular",
    "subtitularían": "subtitular",
    "subtitularías": "subtitular",
    "subtitulas": "subtitular",
    "subtitulase": "subtitular",
    "subtitulaseis": "subtitular",
    "subtitulasen": "subtitular",
    "subtitulases": "subtitular",
    "subtitulaste": "subtitular",
    "subtitulasteis": "subtitular",
    "subtitule": "subtitular",
    "subtitulemos": "subtitular",
    "subtitulen": "subtitular",
    "subtitules": "subtitular",
    "subtitulo": "subtitular",
    "subtitulábamos": "subtitular",
    "subtituláis": "subtitular",
    "subtituláramos": "subtitular",
    "subtituláremos": "subtitular",
    "subtitulásemos": "subtitular",
    "subtitulé": "subtitular",
    "subtituléis": "subtitular",
    "subtituló": "subtitular",
    "subtonos": "subtono",
    "subtotales": "subtotal",
    "subtracciones": "subtracción",
    "subtractores": "subtractor",
    "subtransitoria": "subtransitorio",
    "subtransitorias": "subtransitorio",
    "subtransitorios": "subtransitorio",
    "subtropicales": "subtropical",
    "subtroquiniana": "subtroquiniano",
    "subtroquinianas": "subtroquiniano",
    "subtroquinianos": "subtroquiniano",
    "subtroquiteriana": "subtroquiteriano",
    "subtroquiterianas": "subtroquiteriano",
    "subtroquiterianos": "subtroquiteriano",
    "subtítulos": "subtítulo",
    "subungueales": "subungueal",
    "subunidades": "subunidad",
    "subuniversos": "subuniverso",
    "suburbana": "suburbano",
    "suburbanas": "suburbano",
    "suburbanos": "suburbano",
    "suburbiales": "suburbial",
    "suburbicaria": "suburbicario",
    "suburbicarias": "suburbicario",
    "suburbicarios": "suburbicario",
    "suburbios": "suburbio",
    "subvalora": "subvalorar",
    "subvaloraba": "subvalorar",
    "subvalorabais": "subvalorar",
    "subvaloraban": "subvalorar",
    "subvalorabas": "subvalorar",
    "subvalorad": "subvalorar",
    "subvalorada": "subvalorar",
    "subvaloradas": "subvalorar",
    "subvalorado": "subvalorar",
    "subvalorados": "subvalorar",
    "subvaloramos": "subvalorar",
    "subvaloran": "subvalorar",
    "subvalorando": "subvalorar",
    "subvalorara": "subvalorar",
    "subvalorarais": "subvalorar",
    "subvaloraran": "subvalorar",
    "subvaloraras": "subvalorar",
    "subvalorare": "subvalorar",
    "subvalorareis": "subvalorar",
    "subvaloraremos": "subvalorar",
    "subvaloraren": "subvalorar",
    "subvalorares": "subvalorar",
    "subvaloraron": "subvalorar",
    "subvalorará": "subvalorar",
    "subvalorarán": "subvalorar",
    "subvalorarás": "subvalorar",
    "subvaloraré": "subvalorar",
    "subvaloraréis": "subvalorar",
    "subvaloraría": "subvalorar",
    "subvaloraríais": "subvalorar",
    "subvaloraríamos": "subvalorar",
    "subvalorarían": "subvalorar",
    "subvalorarías": "subvalorar",
    "subvaloras": "subvalorar",
    "subvalorase": "subvalorar",
    "subvaloraseis": "subvalorar",
    "subvalorasen": "subvalorar",
    "subvalorases": "subvalorar",
    "subvaloraste": "subvalorar",
    "subvalorasteis": "subvalorar",
    "subvalore": "subvalorar",
    "subvaloremos": "subvalorar",
    "subvaloren": "subvalorar",
    "subvalores": "subvalorar",
    "subvaloro": "subvalorar",
    "subvalorábamos": "subvalorar",
    "subvaloráis": "subvalorar",
    "subvaloráramos": "subvalorar",
    "subvaloráremos": "subvalorar",
    "subvalorásemos": "subvalorar",
    "subvaloré": "subvalorar",
    "subvaloréis": "subvalorar",
    "subvaloró": "subvalorar",
    "subvariantes": "subvariante",
    "subvariedades": "subvariedad",
    "subven": "subvenir",
    "subvenciona": "subvencionar",
    "subvencionaba": "subvencionar",
    "subvencionabais": "subvencionar",
    "subvencionaban": "subvencionar",
    "subvencionabas": "subvencionar",
    "subvencionables": "subvencionable",
    "subvencionad": "subvencionar",
    "subvencionada": "subvencionar",
    "subvencionadas": "subvencionar",
    "subvencionado": "subvencionar",
    "subvencionados": "subvencionar",
    "subvencionamos": "subvencionar",
    "subvencionan": "subvencionar",
    "subvencionando": "subvencionar",
    "subvencionara": "subvencionar",
    "subvencionarais": "subvencionar",
    "subvencionaran": "subvencionar",
    "subvencionaras": "subvencionar",
    "subvencionare": "subvencionar",
    "subvencionareis": "subvencionar",
    "subvencionaremos": "subvencionar",
    "subvencionaren": "subvencionar",
    "subvencionares": "subvencionar",
    "subvencionaron": "subvencionar",
    "subvencionará": "subvencionar",
    "subvencionarán": "subvencionar",
    "subvencionarás": "subvencionar",
    "subvencionaré": "subvencionar",
    "subvencionaréis": "subvencionar",
    "subvencionaría": "subvencionar",
    "subvencionaríais": "subvencionar",
    "subvencionaríamos": "subvencionar",
    "subvencionarían": "subvencionar",
    "subvencionarías": "subvencionar",
    "subvencionas": "subvencionar",
    "subvencionase": "subvencionar",
    "subvencionaseis": "subvencionar",
    "subvencionasen": "subvencionar",
    "subvencionases": "subvencionar",
    "subvencionaste": "subvencionar",
    "subvencionasteis": "subvencionar",
    "subvencione": "subvencionar",
    "subvencionemos": "subvencionar",
    "subvencionen": "subvencionar",
    "subvenciones": "subvencionar",
    "subvenciono": "subvencionar",
    "subvencionábamos": "subvencionar",
    "subvencionáis": "subvencionar",
    "subvencionáramos": "subvencionar",
    "subvencionáremos": "subvencionar",
    "subvencionásemos": "subvencionar",
    "subvencioné": "subvencionar",
    "subvencionéis": "subvencionar",
    "subvencionó": "subvencionar",
    "subvendremos": "subvenir",
    "subvendrá": "subvenir",
    "subvendrán": "subvenir",
    "subvendrás": "subvenir",
    "subvendré": "subvenir",
    "subvendréis": "subvenir",
    "subvendría": "subvenir",
    "subvendríais": "subvenir",
    "subvendríamos": "subvenir",
    "subvendrían": "subvenir",
    "subvendrías": "subvenir",
    "subvenga": "subvenir",
    "subvengamos": "subvenir",
    "subvengan": "subvenir",
    "subvengas": "subvenir",
    "subvengo": "subvenir",
    "subvengáis": "subvenir",
    "subvenid": "subvenir",
    "subvenida": "subvenir",
    "subvenidas": "subvenir",
    "subvenido": "subvenir",
    "subvenidos": "subvenir",
    "subvenimos": "subvenir",
    "subventanas": "subventana",
    "subvenía": "subvenir",
    "subveníais": "subvenir",
    "subveníamos": "subvenir",
    "subvenían": "subvenir",
    "subvenías": "subvenir",
    "subvenís": "subvenir",
    "subversiones": "subversión",
    "subversiva": "subversivo",
    "subversivas": "subversivo",
    "subversivos": "subversivo",
    "subversora": "subversor",
    "subversoras": "subversor",
    "subversores": "subversor",
    "subvertid": "subvertir",
    "subvertida": "subvertir",
    "subvertidas": "subvertir",
    "subvertido": "subvertir",
    "subvertidos": "subvertir",
    "subvertimos": "subvertir",
    "subvertiremos": "subvertir",
    "subvertirá": "subvertir",
    "subvertirán": "subvertir",
    "subvertirás": "subvertir",
    "subvertiré": "subvertir",
    "subvertiréis": "subvertir",
    "subvertiría": "subvertir",
    "subvertiríais": "subvertir",
    "subvertiríamos": "subvertir",
    "subvertirían": "subvertir",
    "subvertirías": "subvertir",
    "subvertiste": "subvertir",
    "subvertisteis": "subvertir",
    "subvertí": "subvertir",
    "subvertía": "subvertir",
    "subvertíais": "subvertir",
    "subvertíamos": "subvertir",
    "subvertían": "subvertir",
    "subvertías": "subvertir",
    "subvertís": "subvertir",
    "subviene": "subvenir",
    "subvienen": "subvenir",
    "subvienes": "subvenir",
    "subvierta": "subvertir",
    "subviertan": "subvertir",
    "subviertas": "subvertir",
    "subvierte": "subvertir",
    "subvierten": "subvertir",
    "subviertes": "subvertir",
    "subvierto": "subvertir",
    "subvine": "subvenir",
    "subviniendo": "subvenir",
    "subviniera": "subvenir",
    "subvinierais": "subvenir",
    "subvinieran": "subvenir",
    "subvinieras": "subvenir",
    "subviniere": "subvenir",
    "subviniereis": "subvenir",
    "subvinieren": "subvenir",
    "subvinieres": "subvenir",
    "subvinieron": "subvenir",
    "subviniese": "subvenir",
    "subvinieseis": "subvenir",
    "subviniesen": "subvenir",
    "subvinieses": "subvenir",
    "subvinimos": "subvenir",
    "subviniste": "subvenir",
    "subvinisteis": "subvenir",
    "subviniéramos": "subvenir",
    "subviniéremos": "subvenir",
    "subviniésemos": "subvenir",
    "subvino": "subvenir",
    "subvirtamos": "subvertir",
    "subvirtiendo": "subvertir",
    "subvirtiera": "subvertir",
    "subvirtierais": "subvertir",
    "subvirtieran": "subvertir",
    "subvirtieras": "subvertir",
    "subvirtiere": "subvertir",
    "subvirtiereis": "subvertir",
    "subvirtieren": "subvertir",
    "subvirtieres": "subvertir",
    "subvirtieron": "subvertir",
    "subvirtiese": "subvertir",
    "subvirtieseis": "subvertir",
    "subvirtiesen": "subvertir",
    "subvirtieses": "subvertir",
    "subvirtiéramos": "subvertir",
    "subvirtiéremos": "subvertir",
    "subvirtiésemos": "subvertir",
    "subvirtió": "subvertir",
    "subvirtáis": "subvertir",
    "subvoltajes": "subvoltaje",
    "subyace": "subyacer",
    "subyaced": "subyacer",
    "subyacemos": "subyacer",
    "subyacen": "subyacer",
    "subyacentes": "subyacente",
    "subyaceremos": "subyacer",
    "subyacerá": "subyacer",
    "subyacerán": "subyacer",
    "subyacerás": "subyacer",
    "subyaceré": "subyacer",
    "subyaceréis": "subyacer",
    "subyacería": "subyacer",
    "subyaceríais": "subyacer",
    "subyaceríamos": "subyacer",
    "subyacerían": "subyacer",
    "subyacerías": "subyacer",
    "subyaces": "subyacer",
    "subyacida": "subyacer",
    "subyacidas": "subyacer",
    "subyacido": "subyacer",
    "subyacidos": "subyacer",
    "subyaciendo": "subyacer",
    "subyaciera": "subyacer",
    "subyacierais": "subyacer",
    "subyacieran": "subyacer",
    "subyacieras": "subyacer",
    "subyaciere": "subyacer",
    "subyaciereis": "subyacer",
    "subyacieren": "subyacer",
    "subyacieres": "subyacer",
    "subyacieron": "subyacer",
    "subyaciese": "subyacer",
    "subyacieseis": "subyacer",
    "subyaciesen": "subyacer",
    "subyacieses": "subyacer",
    "subyacimos": "subyacer",
    "subyaciste": "subyacer",
    "subyacisteis": "subyacer",
    "subyaciéramos": "subyacer",
    "subyaciéremos": "subyacer",
    "subyaciésemos": "subyacer",
    "subyació": "subyacer",
    "subyacéis": "subyacer",
    "subyací": "subyacer",
    "subyacía": "subyacer",
    "subyacíais": "subyacer",
    "subyacíamos": "subyacer",
    "subyacían": "subyacer",
    "subyacías": "subyacer",
    "subyaga": "subyacer",
    "subyagamos": "subyacer",
    "subyagan": "subyacer",
    "subyagas": "subyacer",
    "subyago": "subyacer",
    "subyagáis": "subyacer",
    "subyaz": "subyacer",
    "subyazca": "subyacer",
    "subyazcamos": "subyacer",
    "subyazcan": "subyacer",
    "subyazcas": "subyacer",
    "subyazco": "subyacer",
    "subyazcáis": "subyacer",
    "subyazga": "subyacer",
    "subyazgamos": "subyacer",
    "subyazgan": "subyacer",
    "subyazgas": "subyacer",
    "subyazgo": "subyacer",
    "subyazgáis": "subyacer",
    "subyuga": "subyugar",
    "subyugaba": "subyugar",
    "subyugabais": "subyugar",
    "subyugaban": "subyugar",
    "subyugabas": "subyugar",
    "subyugables": "subyugable",
    "subyugaciones": "subyugación",
    "subyugad": "subyugar",
    "subyugada": "subyugar",
    "subyugadas": "subyugar",
    "subyugado": "subyugar",
    "subyugadora": "subyugador",
    "subyugadoras": "subyugador",
    "subyugadores": "subyugador",
    "subyugados": "subyugar",
    "subyugamos": "subyugar",
    "subyugan": "subyugar",
    "subyugando": "subyugar",
    "subyugantes": "subyugante",
    "subyugara": "subyugar",
    "subyugarais": "subyugar",
    "subyugaran": "subyugar",
    "subyugaras": "subyugar",
    "subyugare": "subyugar",
    "subyugareis": "subyugar",
    "subyugaremos": "subyugar",
    "subyugaren": "subyugar",
    "subyugares": "subyugar",
    "subyugaron": "subyugar",
    "subyugará": "subyugar",
    "subyugarán": "subyugar",
    "subyugarás": "subyugar",
    "subyugaré": "subyugar",
    "subyugaréis": "subyugar",
    "subyugaría": "subyugar",
    "subyugaríais": "subyugar",
    "subyugaríamos": "subyugar",
    "subyugarían": "subyugar",
    "subyugarías": "subyugar",
    "subyugas": "subyugar",
    "subyugase": "subyugar",
    "subyugaseis": "subyugar",
    "subyugasen": "subyugar",
    "subyugases": "subyugar",
    "subyugaste": "subyugar",
    "subyugasteis": "subyugar",
    "subyugo": "subyugar",
    "subyugue": "subyugar",
    "subyuguemos": "subyugar",
    "subyuguen": "subyugar",
    "subyugues": "subyugar",
    "subyugué": "subyugar",
    "subyuguéis": "subyugar",
    "subyugábamos": "subyugar",
    "subyugáis": "subyugar",
    "subyugáramos": "subyugar",
    "subyugáremos": "subyugar",
    "subyugásemos": "subyugar",
    "subyugó": "subyugar",
    "subyunciones": "subyunción",
    "subáis": "subir",
    "subálgebras": "subálgebra",
    "subálvea": "subálveo",
    "subálveas": "subálveo",
    "subálveos": "subálveo",
    "subárboles": "subárbol",
    "subáreas": "subárea",
    "subérica": "subérico",
    "subéricas": "subérico",
    "subéricos": "subérico",
    "subí": "subir",
    "subía": "subir",
    "subíais": "subir",
    "subíamos": "subir",
    "subían": "subir",
    "subías": "subir",
    "subíndices": "subíndice",
    "subís": "subir",
    "subóptima": "subóptimo",
    "subóptimas": "subóptimo",
    "subóptimos": "subóptimo",
    "subórdenes": "suborden",
    "subóxidos": "subóxido",
    "sucas": "suca",
    "succinatos": "succinato",
    "succinilsulfatiazoles": "succinilsulfatiazol",
    "succinimidas": "succinimida",
    "succinitas": "succinita",
    "succinos": "succino",
    "succiones": "succión",
    "succínica": "succínico",
    "succínicas": "succínico",
    "succínicos": "succínico",
    "suceda": "suceder",
    "sucedamos": "suceder",
    "sucedan": "suceder",
    "sucedas": "suceder",
    "sucede": "suceder",
    "suceded": "suceder",
    "sucedemos": "suceder",
    "suceden": "suceder",
    "sucederemos": "suceder",
    "sucederá": "suceder",
    "sucederán": "suceder",
    "sucederás": "suceder",
    "sucederé": "suceder",
    "sucederéis": "suceder",
    "sucedería": "suceder",
    "sucederíais": "suceder",
    "sucederíamos": "suceder",
    "sucederían": "suceder",
    "sucederías": "suceder",
    "sucedes": "suceder",
    "sucedida": "suceder",
    "sucedidas": "suceder",
    "sucedido": "suceder",
    "sucedidos": "sucedido",
    "sucediendo": "suceder",
    "sucediera": "suceder",
    "sucedierais": "suceder",
    "sucedieran": "suceder",
    "sucedieras": "suceder",
    "sucediere": "suceder",
    "sucediereis": "suceder",
    "sucedieren": "suceder",
    "sucedieres": "suceder",
    "sucedieron": "suceder",
    "sucediese": "suceder",
    "sucedieseis": "suceder",
    "sucediesen": "suceder",
    "sucedieses": "suceder",
    "sucedimos": "suceder",
    "sucediste": "suceder",
    "sucedisteis": "suceder",
    "sucediéramos": "suceder",
    "sucediéremos": "suceder",
    "sucediésemos": "suceder",
    "sucedió": "suceder",
    "sucedo": "suceder",
    "sucedáis": "suceder",
    "sucedánea": "sucedáneo",
    "sucedáneas": "sucedáneo",
    "sucedáneos": "sucedáneo",
    "sucedéis": "suceder",
    "sucedí": "suceder",
    "sucedía": "suceder",
    "sucedíais": "suceder",
    "sucedíamos": "suceder",
    "sucedían": "suceder",
    "sucedías": "suceder",
    "sucesibles": "sucesible",
    "sucesionales": "sucesional",
    "sucesiones": "sucesión",
    "sucesiva": "sucesivo",
    "sucesivas": "sucesivo",
    "sucesivos": "sucesivo",
    "sucesora": "sucesor",
    "sucesorales": "sucesoral",
    "sucesoras": "sucesor",
    "sucesores": "sucesor",
    "sucesoria": "sucesorio",
    "sucesorias": "sucesorio",
    "sucesorios": "sucesorio",
    "sucesos": "suceso",
    "sucia": "sucio",
    "sucias": "sucio",
    "suciedades": "suciedad",
    "sucinta": "sucinto",
    "sucintas": "sucinto",
    "sucintos": "sucinto",
    "sucios": "sucio",
    "sucosa": "sucoso",
    "sucosas": "sucoso",
    "sucosos": "sucoso",
    "sucrenses": "sucrense",
    "sucres": "sucre",
    "sucreña": "sucreño",
    "sucreñas": "sucreño",
    "sucreños": "sucreño",
    "sucuchos": "sucucho",
    "suculencias": "suculencia",
    "suculenta": "suculento",
    "suculentas": "suculento",
    "suculentos": "suculento",
    "sucumba": "sucumbir",
    "sucumbamos": "sucumbir",
    "sucumban": "sucumbir",
    "sucumbas": "sucumbir",
    "sucumbe": "sucumbir",
    "sucumben": "sucumbir",
    "sucumbes": "sucumbir",
    "sucumbid": "sucumbir",
    "sucumbida": "sucumbir",
    "sucumbidas": "sucumbir",
    "sucumbido": "sucumbir",
    "sucumbidos": "sucumbir",
    "sucumbiendo": "sucumbir",
    "sucumbiera": "sucumbir",
    "sucumbierais": "sucumbir",
    "sucumbieran": "sucumbir",
    "sucumbieras": "sucumbir",
    "sucumbiere": "sucumbir",
    "sucumbiereis": "sucumbir",
    "sucumbieren": "sucumbir",
    "sucumbieres": "sucumbir",
    "sucumbieron": "sucumbir",
    "sucumbiese": "sucumbir",
    "sucumbieseis": "sucumbir",
    "sucumbiesen": "sucumbir",
    "sucumbieses": "sucumbir",
    "sucumbimos": "sucumbir",
    "sucumbiremos": "sucumbir",
    "sucumbirá": "sucumbir",
    "sucumbirán": "sucumbir",
    "sucumbirás": "sucumbir",
    "sucumbiré": "sucumbir",
    "sucumbiréis": "sucumbir",
    "sucumbiría": "sucumbir",
    "sucumbiríais": "sucumbir",
    "sucumbiríamos": "sucumbir",
    "sucumbirían": "sucumbir",
    "sucumbirías": "sucumbir",
    "sucumbiste": "sucumbir",
    "sucumbisteis": "sucumbir",
    "sucumbiéramos": "sucumbir",
    "sucumbiéremos": "sucumbir",
    "sucumbiésemos": "sucumbir",
    "sucumbió": "sucumbir",
    "sucumbo": "sucumbir",
    "sucumbáis": "sucumbir",
    "sucumbí": "sucumbir",
    "sucumbía": "sucumbir",
    "sucumbíais": "sucumbir",
    "sucumbíamos": "sucumbir",
    "sucumbían": "sucumbir",
    "sucumbías": "sucumbir",
    "sucumbís": "sucumbir",
    "sucursales": "sucursal",
    "sucursalistas": "sucursalista",
    "sucursalizaciones": "sucursalización",
    "sucusiones": "sucusión",
    "suda": "sudar",
    "sudaba": "sudar",
    "sudabais": "sudar",
    "sudaban": "sudar",
    "sudabas": "sudar",
    "sudacas": "sudaca",
    "sudaciones": "sudación",
    "sudad": "sudar",
    "sudada": "sudar",
    "sudadas": "sudar",
    "sudaderas": "sudadera",
    "sudaderos": "sudadero",
    "sudado": "sudar",
    "sudados": "sudar",
    "sudafricana": "sudafricano",
    "sudafricanas": "sudafricano",
    "sudafricanos": "sudafricano",
    "sudamericana": "sudamericano",
    "sudamericanas": "sudamericano",
    "sudamericanos": "sudamericano",
    "sudaminas": "sudamina",
    "sudamos": "sudar",
    "sudan": "sudar",
    "sudando": "sudar",
    "sudanesa": "sudanés",
    "sudanesas": "sudanés",
    "sudaneses": "sudanés",
    "sudara": "sudar",
    "sudarais": "sudar",
    "sudaran": "sudar",
    "sudaras": "sudar",
    "sudare": "sudar",
    "sudareis": "sudar",
    "sudaremos": "sudar",
    "sudaren": "sudar",
    "sudares": "sudar",
    "sudarios": "sudario",
    "sudaron": "sudar",
    "sudará": "sudar",
    "sudarán": "sudar",
    "sudarás": "sudar",
    "sudaré": "sudar",
    "sudaréis": "sudar",
    "sudaría": "sudar",
    "sudaríais": "sudar",
    "sudaríamos": "sudar",
    "sudarían": "sudar",
    "sudarías": "sudar",
    "sudas": "sudar",
    "sudase": "sudar",
    "sudaseis": "sudar",
    "sudasen": "sudar",
    "sudases": "sudar",
    "sudaste": "sudar",
    "sudasteis": "sudar",
    "sudatoria": "sudatorio",
    "sudatorias": "sudatorio",
    "sudatorios": "sudatorio",
    "sudcoreana": "sudcoreano",
    "sudcoreanas": "sudcoreano",
    "sudcoreanos": "sudcoreano",
    "sude": "sudar",
    "sudemos": "sudar",
    "suden": "sudar",
    "sudes": "sudar",
    "sudestes": "sudeste",
    "sudetes": "sudete",
    "sudistas": "sudista",
    "sudmongólidas": "sudmongólida",
    "sudo": "sudar",
    "sudoccidentales": "sudoccidental",
    "sudoestes": "sudoeste",
    "sudoraciones": "sudoración",
    "sudoraciónes": "sudoración",
    "sudorales": "sudoral",
    "sudores": "sudor",
    "sudorienta": "sudoriento",
    "sudorientales": "sudoriental",
    "sudorientas": "sudoriento",
    "sudorientos": "sudoriento",
    "sudorosa": "sudoroso",
    "sudorosas": "sudoroso",
    "sudorosos": "sudoroso",
    "sudorífera": "sudorífero",
    "sudoríferas": "sudorífero",
    "sudoríferos": "sudorífero",
    "sudorífica": "sudorífico",
    "sudoríficas": "sudorífico",
    "sudoríficos": "sudorífico",
    "sudorípara": "sudoríparo",
    "sudoríparas": "sudoríparo",
    "sudoríparos": "sudoríparo",
    "sudosa": "sudoso",
    "sudosas": "sudoso",
    "sudosos": "sudoso",
    "sudsudestes": "sudsudeste",
    "sudsudoestes": "sudsudoeste",
    "sudábamos": "sudar",
    "sudáis": "sudar",
    "sudáramos": "sudar",
    "sudáremos": "sudar",
    "sudásemos": "sudar",
    "sudé": "sudar",
    "sudéis": "sudar",
    "sudó": "sudar",
    "sueca": "sueco",
    "suecas": "sueco",
    "suecos": "sueco",
    "suegra": "suegro",
    "suegras": "suegro",
    "suegros": "suegro",
    "suela": "soler",
    "suelan": "soler",
    "suelas": "suela",
    "suelda": "soldar",
    "sueldan": "soldar",
    "sueldas": "soldar",
    "suelde": "soldar",
    "suelden": "soldar",
    "sueldes": "soldar",
    "sueldo": "soldar",
    "sueldos": "sueldo",
    "suele": "soler",
    "suelen": "soler",
    "sueles": "soler",
    "suelo": "soler",
    "suelos": "suelo",
    "suelta": "suelto",
    "sueltan": "soltar",
    "sueltas": "suelto",
    "suelte": "soltar",
    "suelten": "soltar",
    "sueltes": "soltar",
    "suelto": "soltar",
    "sueltos": "suelto",
    "suena": "sonar",
    "suenan": "sonar",
    "suenas": "sonar",
    "suene": "sonar",
    "suenen": "sonar",
    "suenes": "sonar",
    "sueno": "sonar",
    "sueros": "suero",
    "suerosa": "sueroso",
    "suerosas": "sueroso",
    "suerosos": "sueroso",
    "sueroterapias": "sueroterapia",
    "suertes": "suerte",
    "suestes": "sueste",
    "sueva": "suevo",
    "suevas": "suevo",
    "suevos": "suevo",
    "sueña": "soñar",
    "sueñan": "soñar",
    "sueñas": "soñar",
    "sueñe": "soñar",
    "sueñen": "soñar",
    "sueñes": "soñar",
    "sueño": "soñar",
    "sueños": "sueño",
    "sufetes": "sufete",
    "suficiencias": "suficiencia",
    "suficientes": "suficiente",
    "sufija": "sufijo",
    "sufijaba": "sufijar",
    "sufijabais": "sufijar",
    "sufijaban": "sufijar",
    "sufijabas": "sufijar",
    "sufijaciones": "sufijación",
    "sufijad": "sufijar",
    "sufijada": "sufijar",
    "sufijadas": "sufijar",
    "sufijado": "sufijar",
    "sufijados": "sufijar",
    "sufijales": "sufijal",
    "sufijamos": "sufijar",
    "sufijan": "sufijar",
    "sufijando": "sufijar",
    "sufijara": "sufijar",
    "sufijarais": "sufijar",
    "sufijaran": "sufijar",
    "sufijaras": "sufijar",
    "sufijare": "sufijar",
    "sufijareis": "sufijar",
    "sufijaremos": "sufijar",
    "sufijaren": "sufijar",
    "sufijares": "sufijar",
    "sufijaron": "sufijar",
    "sufijará": "sufijar",
    "sufijarán": "sufijar",
    "sufijarás": "sufijar",
    "sufijaré": "sufijar",
    "sufijaréis": "sufijar",
    "sufijaría": "sufijar",
    "sufijaríais": "sufijar",
    "sufijaríamos": "sufijar",
    "sufijarían": "sufijar",
    "sufijarías": "sufijar",
    "sufijas": "sufijo",
    "sufijase": "sufijar",
    "sufijaseis": "sufijar",
    "sufijasen": "sufijar",
    "sufijases": "sufijar",
    "sufijaste": "sufijar",
    "sufijasteis": "sufijar",
    "sufije": "sufijar",
    "sufijemos": "sufijar",
    "sufijen": "sufijar",
    "sufijes": "sufijar",
    "sufijo": "sufijar",
    "sufijos": "sufijo",
    "sufijábamos": "sufijar",
    "sufijáis": "sufijar",
    "sufijáramos": "sufijar",
    "sufijáremos": "sufijar",
    "sufijásemos": "sufijar",
    "sufijé": "sufijar",
    "sufijéis": "sufijar",
    "sufijó": "sufijar",
    "sufismos": "sufismo",
    "sufistas": "sufista",
    "sufocaciones": "sufocación",
    "sufocadora": "sufocador",
    "sufocadoras": "sufocador",
    "sufocadores": "sufocador",
    "sufra": "sufrir",
    "sufraga": "sufragar",
    "sufragaba": "sufragar",
    "sufragabais": "sufragar",
    "sufragaban": "sufragar",
    "sufragabas": "sufragar",
    "sufragad": "sufragar",
    "sufragada": "sufragar",
    "sufragadas": "sufragar",
    "sufragado": "sufragar",
    "sufragados": "sufragar",
    "sufragamos": "sufragar",
    "sufragan": "sufragar",
    "sufragando": "sufragar",
    "sufragara": "sufragar",
    "sufragarais": "sufragar",
    "sufragaran": "sufragar",
    "sufragaras": "sufragar",
    "sufragare": "sufragar",
    "sufragareis": "sufragar",
    "sufragaremos": "sufragar",
    "sufragaren": "sufragar",
    "sufragares": "sufragar",
    "sufragaron": "sufragar",
    "sufragará": "sufragar",
    "sufragarán": "sufragar",
    "sufragarás": "sufragar",
    "sufragaré": "sufragar",
    "sufragaréis": "sufragar",
    "sufragaría": "sufragar",
    "sufragaríais": "sufragar",
    "sufragaríamos": "sufragar",
    "sufragarían": "sufragar",
    "sufragarías": "sufragar",
    "sufragas": "sufragar",
    "sufragase": "sufragar",
    "sufragaseis": "sufragar",
    "sufragasen": "sufragar",
    "sufragases": "sufragar",
    "sufragaste": "sufragar",
    "sufragasteis": "sufragar",
    "sufragios": "sufragio",
    "sufragismos": "sufragismo",
    "sufragistas": "sufragista",
    "sufrago": "sufragar",
    "sufrague": "sufragar",
    "sufraguemos": "sufragar",
    "sufraguen": "sufragar",
    "sufragues": "sufragar",
    "sufragué": "sufragar",
    "sufraguéis": "sufragar",
    "sufragábamos": "sufragar",
    "sufragáis": "sufragar",
    "sufragánea": "sufragáneo",
    "sufragáneas": "sufragáneo",
    "sufragáneos": "sufragáneo",
    "sufragáramos": "sufragar",
    "sufragáremos": "sufragar",
    "sufragásemos": "sufragar",
    "sufragó": "sufragar",
    "suframos": "sufrir",
    "sufran": "sufrir",
    "sufras": "sufrir",
    "sufre": "sufrir",
    "sufren": "sufrir",
    "sufres": "sufrir",
    "sufribles": "sufrible",
    "sufrid": "sufrir",
    "sufrida": "sufrir",
    "sufridas": "sufrir",
    "sufridera": "sufridero",
    "sufrideras": "sufridero",
    "sufrideros": "sufridero",
    "sufrido": "sufrir",
    "sufridora": "sufridor",
    "sufridoras": "sufridor",
    "sufridores": "sufridor",
    "sufridos": "sufrir",
    "sufriendo": "sufrir",
    "sufriera": "sufrir",
    "sufrierais": "sufrir",
    "sufrieran": "sufrir",
    "sufrieras": "sufrir",
    "sufriere": "sufrir",
    "sufriereis": "sufrir",
    "sufrieren": "sufrir",
    "sufrieres": "sufrir",
    "sufrieron": "sufrir",
    "sufriese": "sufrir",
    "sufrieseis": "sufrir",
    "sufriesen": "sufrir",
    "sufrieses": "sufrir",
    "sufrimientos": "sufrimiento",
    "sufrimos": "sufrir",
    "sufriremos": "sufrir",
    "sufrirá": "sufrir",
    "sufrirán": "sufrir",
    "sufrirás": "sufrir",
    "sufriré": "sufrir",
    "sufriréis": "sufrir",
    "sufriría": "sufrir",
    "sufriríais": "sufrir",
    "sufriríamos": "sufrir",
    "sufrirían": "sufrir",
    "sufrirías": "sufrir",
    "sufriste": "sufrir",
    "sufristeis": "sufrir",
    "sufriéramos": "sufrir",
    "sufriéremos": "sufrir",
    "sufriésemos": "sufrir",
    "sufrió": "sufrir",
    "sufro": "sufrir",
    "sufráis": "sufrir",
    "sufrí": "sufrir",
    "sufría": "sufrir",
    "sufríais": "sufrir",
    "sufríamos": "sufrir",
    "sufrían": "sufrir",
    "sufrías": "sufrir",
    "sufrís": "sufrir",
    "sufusiones": "sufusión",
    "sufíes": "sufí",
    "sufís": "sufí",
    "sugerencias": "sugerencia",
    "sugerentes": "sugerente",
    "sugerid": "sugerir",
    "sugerida": "sugerir",
    "sugeridas": "sugerir",
    "sugerido": "sugerir",
    "sugeridora": "sugeridor",
    "sugeridoras": "sugeridor",
    "sugeridores": "sugeridor",
    "sugeridos": "sugerir",
    "sugerimos": "sugerir",
    "sugeriremos": "sugerir",
    "sugerirá": "sugerir",
    "sugerirán": "sugerir",
    "sugerirás": "sugerir",
    "sugeriré": "sugerir",
    "sugeriréis": "sugerir",
    "sugeriría": "sugerir",
    "sugeriríais": "sugerir",
    "sugeriríamos": "sugerir",
    "sugerirían": "sugerir",
    "sugerirías": "sugerir",
    "sugeriste": "sugerir",
    "sugeristeis": "sugerir",
    "sugerí": "sugerir",
    "sugería": "sugerir",
    "sugeríais": "sugerir",
    "sugeríamos": "sugerir",
    "sugerían": "sugerir",
    "sugerías": "sugerir",
    "sugerís": "sugerir",
    "sugestibilidades": "sugestibilidad",
    "sugestiona": "sugestionar",
    "sugestionaba": "sugestionar",
    "sugestionabais": "sugestionar",
    "sugestionaban": "sugestionar",
    "sugestionabas": "sugestionar",
    "sugestionables": "sugestionable",
    "sugestionad": "sugestionar",
    "sugestionada": "sugestionar",
    "sugestionadas": "sugestionar",
    "sugestionado": "sugestionar",
    "sugestionadora": "sugestionador",
    "sugestionadoras": "sugestionador",
    "sugestionadores": "sugestionador",
    "sugestionados": "sugestionar",
    "sugestionamos": "sugestionar",
    "sugestionan": "sugestionar",
    "sugestionando": "sugestionar",
    "sugestionara": "sugestionar",
    "sugestionarais": "sugestionar",
    "sugestionaran": "sugestionar",
    "sugestionaras": "sugestionar",
    "sugestionare": "sugestionar",
    "sugestionareis": "sugestionar",
    "sugestionaremos": "sugestionar",
    "sugestionaren": "sugestionar",
    "sugestionares": "sugestionar",
    "sugestionaron": "sugestionar",
    "sugestionará": "sugestionar",
    "sugestionarán": "sugestionar",
    "sugestionarás": "sugestionar",
    "sugestionaré": "sugestionar",
    "sugestionaréis": "sugestionar",
    "sugestionaría": "sugestionar",
    "sugestionaríais": "sugestionar",
    "sugestionaríamos": "sugestionar",
    "sugestionarían": "sugestionar",
    "sugestionarías": "sugestionar",
    "sugestionas": "sugestionar",
    "sugestionase": "sugestionar",
    "sugestionaseis": "sugestionar",
    "sugestionasen": "sugestionar",
    "sugestionases": "sugestionar",
    "sugestionaste": "sugestionar",
    "sugestionasteis": "sugestionar",
    "sugestione": "sugestionar",
    "sugestionemos": "sugestionar",
    "sugestionen": "sugestionar",
    "sugestiones": "sugestionar",
    "sugestiono": "sugestionar",
    "sugestionábamos": "sugestionar",
    "sugestionáis": "sugestionar",
    "sugestionáramos": "sugestionar",
    "sugestionáremos": "sugestionar",
    "sugestionásemos": "sugestionar",
    "sugestioné": "sugestionar",
    "sugestionéis": "sugestionar",
    "sugestionó": "sugestionar",
    "sugestiva": "sugestivo",
    "sugestivas": "sugestivo",
    "sugestivos": "sugestivo",
    "sugiera": "sugerir",
    "sugieran": "sugerir",
    "sugieras": "sugerir",
    "sugiere": "sugerir",
    "sugieren": "sugerir",
    "sugieres": "sugerir",
    "sugiero": "sugerir",
    "sugiramos": "sugerir",
    "sugiriendo": "sugerir",
    "sugiriera": "sugerir",
    "sugirierais": "sugerir",
    "sugirieran": "sugerir",
    "sugirieras": "sugerir",
    "sugiriere": "sugerir",
    "sugiriereis": "sugerir",
    "sugirieren": "sugerir",
    "sugirieres": "sugerir",
    "sugirieron": "sugerir",
    "sugiriese": "sugerir",
    "sugirieseis": "sugerir",
    "sugiriesen": "sugerir",
    "sugirieses": "sugerir",
    "sugiriéramos": "sugerir",
    "sugiriéremos": "sugerir",
    "sugiriésemos": "sugerir",
    "sugirió": "sugerir",
    "sugiráis": "sugerir",
    "suicida": "suicidar",
    "suicidaba": "suicidar",
    "suicidabais": "suicidar",
    "suicidaban": "suicidar",
    "suicidabas": "suicidar",
    "suicidad": "suicidar",
    "suicidada": "suicidar",
    "suicidadas": "suicidar",
    "suicidado": "suicidar",
    "suicidados": "suicidar",
    "suicidamos": "suicidar",
    "suicidan": "suicidar",
    "suicidando": "suicidar",
    "suicidara": "suicidar",
    "suicidarais": "suicidar",
    "suicidaran": "suicidar",
    "suicidaras": "suicidar",
    "suicidare": "suicidar",
    "suicidareis": "suicidar",
    "suicidaremos": "suicidar",
    "suicidaren": "suicidar",
    "suicidares": "suicidar",
    "suicidaron": "suicidar",
    "suicidará": "suicidar",
    "suicidarán": "suicidar",
    "suicidarás": "suicidar",
    "suicidaré": "suicidar",
    "suicidaréis": "suicidar",
    "suicidaría": "suicidar",
    "suicidaríais": "suicidar",
    "suicidaríamos": "suicidar",
    "suicidarían": "suicidar",
    "suicidarías": "suicidar",
    "suicidas": "suicidar",
    "suicidase": "suicidar",
    "suicidaseis": "suicidar",
    "suicidasen": "suicidar",
    "suicidases": "suicidar",
    "suicidaste": "suicidar",
    "suicidasteis": "suicidar",
    "suicide": "suicidar",
    "suicidemos": "suicidar",
    "suiciden": "suicidar",
    "suicides": "suicidar",
    "suicidios": "suicidio",
    "suicido": "suicidar",
    "suicidábamos": "suicidar",
    "suicidáis": "suicidar",
    "suicidáramos": "suicidar",
    "suicidáremos": "suicidar",
    "suicidásemos": "suicidar",
    "suicidé": "suicidar",
    "suicidéis": "suicidar",
    "suicidó": "suicidar",
    "suidos": "suido",
    "suites": "suite",
    "suiza": "suizo",
    "suizas": "suizo",
    "suizos": "suizo",
    "sujeciones": "sujeción",
    "sujeta": "sujeto",
    "sujetaba": "sujetar",
    "sujetabais": "sujetar",
    "sujetaban": "sujetar",
    "sujetabas": "sujetar",
    "sujetad": "sujetar",
    "sujetada": "sujetar",
    "sujetadas": "sujetar",
    "sujetado": "sujetar",
    "sujetadora": "sujetador",
    "sujetadoras": "sujetador",
    "sujetadores": "sujetador",
    "sujetados": "sujetar",
    "sujetamos": "sujetar",
    "sujetan": "sujetar",
    "sujetando": "sujetar",
    "sujetara": "sujetar",
    "sujetarais": "sujetar",
    "sujetaran": "sujetar",
    "sujetaras": "sujetar",
    "sujetare": "sujetar",
    "sujetareis": "sujetar",
    "sujetaremos": "sujetar",
    "sujetaren": "sujetar",
    "sujetares": "sujetar",
    "sujetaron": "sujetar",
    "sujetará": "sujetar",
    "sujetarán": "sujetar",
    "sujetarás": "sujetar",
    "sujetaré": "sujetar",
    "sujetaréis": "sujetar",
    "sujetaría": "sujetar",
    "sujetaríais": "sujetar",
    "sujetaríamos": "sujetar",
    "sujetarían": "sujetar",
    "sujetarías": "sujetar",
    "sujetas": "sujeto",
    "sujetase": "sujetar",
    "sujetaseis": "sujetar",
    "sujetasen": "sujetar",
    "sujetases": "sujetar",
    "sujetaste": "sujetar",
    "sujetasteis": "sujetar",
    "sujete": "sujetar",
    "sujetemos": "sujetar",
    "sujeten": "sujetar",
    "sujetes": "sujetar",
    "sujeto": "sujetar",
    "sujetos": "sujeto",
    "sujetábamos": "sujetar",
    "sujetáis": "sujetar",
    "sujetáramos": "sujetar",
    "sujetáremos": "sujetar",
    "sujetásemos": "sujetar",
    "sujeté": "sujetar",
    "sujetéis": "sujetar",
    "sujetó": "sujetar",
    "sulciformes": "sulciforme",
    "suletinos": "suletino",
    "sulfacetamidas": "sulfacetamida",
    "sulfadiacinas": "sulfadiacina",
    "sulfadimetoxinas": "sulfadimetoxina",
    "sulfaguanidinas": "sulfaguanidina",
    "sulfamatos": "sulfamato",
    "sulfameracinas": "sulfameracina",
    "sulfametacinas": "sulfametacina",
    "sulfametiazoles": "sulfametiazol",
    "sulfametoxipiridacinas": "sulfametoxipiridacina",
    "sulfamidas": "sulfamida",
    "sulfanilamidas": "sulfanilamida",
    "sulfanilatos": "sulfanilato",
    "sulfanílica": "sulfanílico",
    "sulfanílicas": "sulfanílico",
    "sulfanílicos": "sulfanílico",
    "sulfapiridinas": "sulfapiridina",
    "sulfaquinoxalinas": "sulfaquinoxalina",
    "sulfarsfenaminas": "sulfarsfenamina",
    "sulfata": "sulfatar",
    "sulfataba": "sulfatar",
    "sulfatabais": "sulfatar",
    "sulfataban": "sulfatar",
    "sulfatabas": "sulfatar",
    "sulfataciones": "sulfatación",
    "sulfatad": "sulfatar",
    "sulfatada": "sulfatar",
    "sulfatadas": "sulfatar",
    "sulfatado": "sulfatar",
    "sulfatadoras": "sulfatadora",
    "sulfatados": "sulfatar",
    "sulfatamos": "sulfatar",
    "sulfatan": "sulfatar",
    "sulfatando": "sulfatar",
    "sulfatara": "sulfatar",
    "sulfatarais": "sulfatar",
    "sulfataran": "sulfatar",
    "sulfataras": "sulfatar",
    "sulfatare": "sulfatar",
    "sulfatareis": "sulfatar",
    "sulfataremos": "sulfatar",
    "sulfataren": "sulfatar",
    "sulfatares": "sulfatar",
    "sulfataron": "sulfatar",
    "sulfatará": "sulfatar",
    "sulfatarán": "sulfatar",
    "sulfatarás": "sulfatar",
    "sulfataré": "sulfatar",
    "sulfataréis": "sulfatar",
    "sulfataría": "sulfatar",
    "sulfataríais": "sulfatar",
    "sulfataríamos": "sulfatar",
    "sulfatarían": "sulfatar",
    "sulfatarías": "sulfatar",
    "sulfatas": "sulfatar",
    "sulfatase": "sulfatar",
    "sulfataseis": "sulfatar",
    "sulfatasen": "sulfatar",
    "sulfatases": "sulfatar",
    "sulfataste": "sulfatar",
    "sulfatasteis": "sulfatar",
    "sulfate": "sulfatar",
    "sulfatemos": "sulfatar",
    "sulfaten": "sulfatar",
    "sulfates": "sulfatar",
    "sulfatiazoles": "sulfatiazol",
    "sulfato": "sulfatar",
    "sulfatos": "sulfato",
    "sulfatábamos": "sulfatar",
    "sulfatáis": "sulfatar",
    "sulfatáramos": "sulfatar",
    "sulfatáremos": "sulfatar",
    "sulfatásemos": "sulfatar",
    "sulfaté": "sulfatar",
    "sulfatéis": "sulfatar",
    "sulfató": "sulfatar",
    "sulfhidrilos": "sulfhidrilo",
    "sulfhidrometrías": "sulfhidrometría",
    "sulfinizaciones": "sulfinización",
    "sulfinpirazonas": "sulfinpirazona",
    "sulfitaciones": "sulfitación",
    "sulfitados": "sulfitado",
    "sulfitos": "sulfito",
    "sulfobenzoica": "sulfobenzoico",
    "sulfobenzoicas": "sulfobenzoico",
    "sulfobenzoicos": "sulfobenzoico",
    "sulfocarbonatos": "sulfocarbonato",
    "sulfocianuros": "sulfocianuro",
    "sulfoftálica": "sulfoftálico",
    "sulfoftálicas": "sulfoftálico",
    "sulfoftálicos": "sulfoftálico",
    "sulfonaciones": "sulfonación",
    "sulfonada": "sulfonado",
    "sulfonadas": "sulfonado",
    "sulfonados": "sulfonado",
    "sulfonales": "sulfonal",
    "sulfonamidas": "sulfonamida",
    "sulfonas": "sulfona",
    "sulfonatos": "sulfonato",
    "sulfonetilmetanos": "sulfonetilmetano",
    "sulfonildianilinas": "sulfonildianilina",
    "sulfonmetanos": "sulfonmetano",
    "sulforricinoleatos": "sulforricinoleato",
    "sulfosalicílica": "sulfosalicílico",
    "sulfosalicílicas": "sulfosalicílico",
    "sulfosalicílicos": "sulfosalicílico",
    "sulfura": "sulfurar",
    "sulfuraba": "sulfurar",
    "sulfurabais": "sulfurar",
    "sulfuraban": "sulfurar",
    "sulfurabas": "sulfurar",
    "sulfuraciones": "sulfuración",
    "sulfurad": "sulfurar",
    "sulfurada": "sulfurar",
    "sulfuradas": "sulfurar",
    "sulfurado": "sulfurar",
    "sulfurados": "sulfurar",
    "sulfuramos": "sulfurar",
    "sulfuran": "sulfurar",
    "sulfurando": "sulfurar",
    "sulfurara": "sulfurar",
    "sulfurarais": "sulfurar",
    "sulfuraran": "sulfurar",
    "sulfuraras": "sulfurar",
    "sulfurare": "sulfurar",
    "sulfurareis": "sulfurar",
    "sulfuraremos": "sulfurar",
    "sulfuraren": "sulfurar",
    "sulfurares": "sulfurar",
    "sulfuraron": "sulfurar",
    "sulfurará": "sulfurar",
    "sulfurarán": "sulfurar",
    "sulfurarás": "sulfurar",
    "sulfuraré": "sulfurar",
    "sulfuraréis": "sulfurar",
    "sulfuraría": "sulfurar",
    "sulfuraríais": "sulfurar",
    "sulfuraríamos": "sulfurar",
    "sulfurarían": "sulfurar",
    "sulfurarías": "sulfurar",
    "sulfuras": "sulfurar",
    "sulfurase": "sulfurar",
    "sulfuraseis": "sulfurar",
    "sulfurasen": "sulfurar",
    "sulfurases": "sulfurar",
    "sulfuraste": "sulfurar",
    "sulfurasteis": "sulfurar",
    "sulfure": "sulfurar",
    "sulfuremos": "sulfurar",
    "sulfuren": "sulfurar",
    "sulfures": "sulfurar",
    "sulfurilos": "sulfurilo",
    "sulfuro": "sulfurar",
    "sulfuros": "sulfuro",
    "sulfurosa": "sulfuroso",
    "sulfurosas": "sulfuroso",
    "sulfurosos": "sulfuroso",
    "sulfurábamos": "sulfurar",
    "sulfuráis": "sulfurar",
    "sulfuráramos": "sulfurar",
    "sulfuráremos": "sulfurar",
    "sulfurásemos": "sulfurar",
    "sulfuré": "sulfurar",
    "sulfuréis": "sulfurar",
    "sulfuró": "sulfurar",
    "sulfámica": "sulfámico",
    "sulfámicas": "sulfámico",
    "sulfámicos": "sulfámico",
    "sulfónica": "sulfónico",
    "sulfónicas": "sulfónico",
    "sulfónicos": "sulfónico",
    "sulfóxidos": "sulfóxido",
    "sulfúrea": "sulfúreo",
    "sulfúreas": "sulfúreo",
    "sulfúreos": "sulfúreo",
    "sulfúrica": "sulfúrico",
    "sulfúricas": "sulfúrico",
    "sulfúricos": "sulfúrico",
    "sultana": "sultano",
    "sultanas": "sultano",
    "sultanatos": "sultanato",
    "sultanes": "sultán",
    "sultanos": "sultano",
    "sultanías": "sultanía",
    "sultánica": "sultánico",
    "sultánicas": "sultánico",
    "sultánicos": "sultánico",
    "suma": "sumo",
    "sumaba": "sumar",
    "sumabais": "sumar",
    "sumaban": "sumar",
    "sumabas": "sumar",
    "sumabilidades": "sumabilidad",
    "sumables": "sumable",
    "sumacas": "sumaca",
    "sumad": "sumar",
    "sumada": "sumar",
    "sumadas": "sumar",
    "sumado": "sumar",
    "sumadora": "sumador",
    "sumadoras": "sumador",
    "sumadores": "sumador",
    "sumados": "sumar",
    "sumamos": "sumir",
    "suman": "sumir",
    "sumando": "sumar",
    "sumandos": "sumando",
    "sumara": "sumar",
    "sumarais": "sumar",
    "sumaran": "sumar",
    "sumaras": "sumar",
    "sumare": "sumar",
    "sumareis": "sumar",
    "sumaremos": "sumar",
    "sumaren": "sumar",
    "sumares": "sumar",
    "sumaria": "sumario",
    "sumariaba": "sumariar",
    "sumariabais": "sumariar",
    "sumariaban": "sumariar",
    "sumariabas": "sumariar",
    "sumariad": "sumariar",
    "sumariada": "sumariar",
    "sumariadas": "sumariar",
    "sumariado": "sumariar",
    "sumariados": "sumariar",
    "sumariales": "sumarial",
    "sumariamos": "sumariar",
    "sumarian": "sumariar",
    "sumariando": "sumariar",
    "sumariara": "sumariar",
    "sumariarais": "sumariar",
    "sumariaran": "sumariar",
    "sumariaras": "sumariar",
    "sumariare": "sumariar",
    "sumariareis": "sumariar",
    "sumariaremos": "sumariar",
    "sumariaren": "sumariar",
    "sumariares": "sumariar",
    "sumariaron": "sumariar",
    "sumariará": "sumariar",
    "sumariarán": "sumariar",
    "sumariarás": "sumariar",
    "sumariaré": "sumariar",
    "sumariaréis": "sumariar",
    "sumariaría": "sumariar",
    "sumariaríais": "sumariar",
    "sumariaríamos": "sumariar",
    "sumariarían": "sumariar",
    "sumariarías": "sumariar",
    "sumarias": "sumario",
    "sumariase": "sumariar",
    "sumariaseis": "sumariar",
    "sumariasen": "sumariar",
    "sumariases": "sumariar",
    "sumariaste": "sumariar",
    "sumariasteis": "sumariar",
    "sumarie": "sumariar",
    "sumariedades": "sumariedad",
    "sumariemos": "sumariar",
    "sumarien": "sumariar",
    "sumaries": "sumariar",
    "sumario": "sumariar",
    "sumarios": "sumario",
    "sumariábamos": "sumariar",
    "sumariáis": "sumariar",
    "sumariáramos": "sumariar",
    "sumariáremos": "sumariar",
    "sumariásemos": "sumariar",
    "sumarié": "sumariar",
    "sumariéis": "sumariar",
    "sumarió": "sumariar",
    "sumaron": "sumar",
    "sumará": "sumar",
    "sumarán": "sumar",
    "sumarás": "sumar",
    "sumaré": "sumar",
    "sumaréis": "sumar",
    "sumaría": "sumar",
    "sumaríais": "sumar",
    "sumaríamos": "sumar",
    "sumarían": "sumar",
    "sumarías": "sumar",
    "sumas": "sumo",
    "sumase": "sumar",
    "sumaseis": "sumar",
    "sumasen": "sumar",
    "sumases": "sumar",
    "sumaste": "sumar",
    "sumasteis": "sumar",
    "sumativa": "sumativo",
    "sumativas": "sumativo",
    "sumativos": "sumativo",
    "sumatoria": "sumatorio",
    "sumatorias": "sumatorio",
    "sumatorios": "sumatorio",
    "sume": "sumir",
    "sumemos": "sumar",
    "sumen": "sumir",
    "sumerge": "sumergir",
    "sumergen": "sumergir",
    "sumerges": "sumergir",
    "sumergibles": "sumergible",
    "sumergid": "sumergir",
    "sumergida": "sumergir",
    "sumergidas": "sumergir",
    "sumergido": "sumergir",
    "sumergidos": "sumergir",
    "sumergiendo": "sumergir",
    "sumergiera": "sumergir",
    "sumergierais": "sumergir",
    "sumergieran": "sumergir",
    "sumergieras": "sumergir",
    "sumergiere": "sumergir",
    "sumergiereis": "sumergir",
    "sumergieren": "sumergir",
    "sumergieres": "sumergir",
    "sumergieron": "sumergir",
    "sumergiese": "sumergir",
    "sumergieseis": "sumergir",
    "sumergiesen": "sumergir",
    "sumergieses": "sumergir",
    "sumergimientos": "sumergimiento",
    "sumergimos": "sumergir",
    "sumergiremos": "sumergir",
    "sumergirá": "sumergir",
    "sumergirán": "sumergir",
    "sumergirás": "sumergir",
    "sumergiré": "sumergir",
    "sumergiréis": "sumergir",
    "sumergiría": "sumergir",
    "sumergiríais": "sumergir",
    "sumergiríamos": "sumergir",
    "sumergirían": "sumergir",
    "sumergirías": "sumergir",
    "sumergiste": "sumergir",
    "sumergisteis": "sumergir",
    "sumergiéramos": "sumergir",
    "sumergiéremos": "sumergir",
    "sumergiésemos": "sumergir",
    "sumergió": "sumergir",
    "sumergí": "sumergir",
    "sumergía": "sumergir",
    "sumergíais": "sumergir",
    "sumergíamos": "sumergir",
    "sumergían": "sumergir",
    "sumergías": "sumergir",
    "sumergís": "sumergir",
    "sumeria": "sumerio",
    "sumerias": "sumerio",
    "sumerios": "sumerio",
    "sumerja": "sumergir",
    "sumerjamos": "sumergir",
    "sumerjan": "sumergir",
    "sumerjas": "sumergir",
    "sumerjo": "sumergir",
    "sumerjáis": "sumergir",
    "sumersiones": "sumersión",
    "sumes": "sumir",
    "sumid": "sumir",
    "sumida": "sumir",
    "sumidades": "sumidad",
    "sumidas": "sumir",
    "sumideros": "sumidero",
    "sumido": "sumir",
    "sumidos": "sumir",
    "sumiendo": "sumir",
    "sumiera": "sumir",
    "sumierais": "sumir",
    "sumieran": "sumir",
    "sumieras": "sumir",
    "sumiere": "sumir",
    "sumiereis": "sumir",
    "sumieren": "sumir",
    "sumieres": "sumir",
    "sumieron": "sumir",
    "sumiese": "sumir",
    "sumieseis": "sumir",
    "sumiesen": "sumir",
    "sumieses": "sumir",
    "sumilleres": "sumiller",
    "sumimos": "sumir",
    "suministra": "suministrar",
    "suministraba": "suministrar",
    "suministrabais": "suministrar",
    "suministraban": "suministrar",
    "suministrabas": "suministrar",
    "suministrables": "suministrable",
    "suministraciones": "suministración",
    "suministrad": "suministrar",
    "suministrada": "suministrar",
    "suministradas": "suministrar",
    "suministrado": "suministrar",
    "suministradora": "suministrador",
    "suministradoras": "suministrador",
    "suministradores": "suministrador",
    "suministrados": "suministrar",
    "suministramos": "suministrar",
    "suministran": "suministrar",
    "suministrando": "suministrar",
    "suministrara": "suministrar",
    "suministrarais": "suministrar",
    "suministraran": "suministrar",
    "suministraras": "suministrar",
    "suministrare": "suministrar",
    "suministrareis": "suministrar",
    "suministraremos": "suministrar",
    "suministraren": "suministrar",
    "suministrares": "suministrar",
    "suministraron": "suministrar",
    "suministrará": "suministrar",
    "suministrarán": "suministrar",
    "suministrarás": "suministrar",
    "suministraré": "suministrar",
    "suministraréis": "suministrar",
    "suministraría": "suministrar",
    "suministraríais": "suministrar",
    "suministraríamos": "suministrar",
    "suministrarían": "suministrar",
    "suministrarías": "suministrar",
    "suministras": "suministrar",
    "suministrase": "suministrar",
    "suministraseis": "suministrar",
    "suministrasen": "suministrar",
    "suministrases": "suministrar",
    "suministraste": "suministrar",
    "suministrasteis": "suministrar",
    "suministre": "suministrar",
    "suministremos": "suministrar",
    "suministren": "suministrar",
    "suministres": "suministrar",
    "suministro": "suministrar",
    "suministros": "suministro",
    "suministrábamos": "suministrar",
    "suministráis": "suministrar",
    "suministráramos": "suministrar",
    "suministráremos": "suministrar",
    "suministrásemos": "suministrar",
    "suministré": "suministrar",
    "suministréis": "suministrar",
    "suministró": "suministrar",
    "sumiremos": "sumir",
    "sumirá": "sumir",
    "sumirán": "sumir",
    "sumirás": "sumir",
    "sumiré": "sumir",
    "sumiréis": "sumir",
    "sumiría": "sumir",
    "sumiríais": "sumir",
    "sumiríamos": "sumir",
    "sumirían": "sumir",
    "sumirías": "sumir",
    "sumisa": "sumiso",
    "sumisas": "sumiso",
    "sumisiones": "sumisión",
    "sumisos": "sumiso",
    "sumiste": "sumir",
    "sumisteis": "sumir",
    "sumiéramos": "sumir",
    "sumiéremos": "sumir",
    "sumiésemos": "sumir",
    "sumió": "sumir",
    "sumo": "sumir",
    "sumos": "sumo",
    "sumoscapos": "sumoscapo",
    "sumulística": "sumulístico",
    "sumulísticas": "sumulístico",
    "sumulísticos": "sumulístico",
    "sumábamos": "sumar",
    "sumáis": "sumir",
    "sumáramos": "sumar",
    "sumáremos": "sumar",
    "sumásemos": "sumar",
    "sumé": "sumar",
    "suméis": "sumar",
    "sumí": "sumir",
    "sumía": "sumir",
    "sumíais": "sumir",
    "sumíamos": "sumir",
    "sumían": "sumir",
    "sumías": "sumir",
    "sumís": "sumir",
    "sumó": "sumar",
    "sunchos": "suncho",
    "sunciones": "sunción",
    "sunis": "suní",
    "sunnas": "sunna",
    "sunníes": "sunní",
    "sunnís": "sunní",
    "suntuaria": "suntuario",
    "suntuarias": "suntuario",
    "suntuarios": "suntuario",
    "suntuosa": "suntuoso",
    "suntuosas": "suntuoso",
    "suntuosidades": "suntuosidad",
    "suntuosos": "suntuoso",
    "supe": "saber",
    "supedita": "supeditar",
    "supeditaba": "supeditar",
    "supeditabais": "supeditar",
    "supeditaban": "supeditar",
    "supeditabas": "supeditar",
    "supeditaciones": "supeditación",
    "supeditad": "supeditar",
    "supeditada": "supeditar",
    "supeditadas": "supeditar",
    "supeditado": "supeditar",
    "supeditados": "supeditar",
    "supeditamos": "supeditar",
    "supeditan": "supeditar",
    "supeditando": "supeditar",
    "supeditara": "supeditar",
    "supeditarais": "supeditar",
    "supeditaran": "supeditar",
    "supeditaras": "supeditar",
    "supeditare": "supeditar",
    "supeditareis": "supeditar",
    "supeditaremos": "supeditar",
    "supeditaren": "supeditar",
    "supeditares": "supeditar",
    "supeditaron": "supeditar",
    "supeditará": "supeditar",
    "supeditarán": "supeditar",
    "supeditarás": "supeditar",
    "supeditaré": "supeditar",
    "supeditaréis": "supeditar",
    "supeditaría": "supeditar",
    "supeditaríais": "supeditar",
    "supeditaríamos": "supeditar",
    "supeditarían": "supeditar",
    "supeditarías": "supeditar",
    "supeditas": "supeditar",
    "supeditase": "supeditar",
    "supeditaseis": "supeditar",
    "supeditasen": "supeditar",
    "supeditases": "supeditar",
    "supeditaste": "supeditar",
    "supeditasteis": "supeditar",
    "supedite": "supeditar",
    "supeditemos": "supeditar",
    "supediten": "supeditar",
    "supedites": "supeditar",
    "supedito": "supeditar",
    "supeditábamos": "supeditar",
    "supeditáis": "supeditar",
    "supeditáramos": "supeditar",
    "supeditáremos": "supeditar",
    "supeditásemos": "supeditar",
    "supedité": "supeditar",
    "supeditéis": "supeditar",
    "supeditó": "supeditar",
    "supedáneos": "supedáneo",
    "supera": "superar",
    "superaba": "superar",
    "superabais": "superar",
    "superaban": "superar",
    "superabas": "superar",
    "superables": "superable",
    "superabunda": "superabundar",
    "superabundaba": "superabundar",
    "superabundabais": "superabundar",
    "superabundaban": "superabundar",
    "superabundabas": "superabundar",
    "superabundad": "superabundar",
    "superabundada": "superabundar",
    "superabundadas": "superabundar",
    "superabundado": "superabundar",
    "superabundados": "superabundar",
    "superabundamos": "superabundar",
    "superabundan": "superabundar",
    "superabundancias": "superabundancia",
    "superabundando": "superabundar",
    "superabundantes": "superabundante",
    "superabundara": "superabundar",
    "superabundarais": "superabundar",
    "superabundaran": "superabundar",
    "superabundaras": "superabundar",
    "superabundare": "superabundar",
    "superabundareis": "superabundar",
    "superabundaremos": "superabundar",
    "superabundaren": "superabundar",
    "superabundares": "superabundar",
    "superabundaron": "superabundar",
    "superabundará": "superabundar",
    "superabundarán": "superabundar",
    "superabundarás": "superabundar",
    "superabundaré": "superabundar",
    "superabundaréis": "superabundar",
    "superabundaría": "superabundar",
    "superabundaríais": "superabundar",
    "superabundaríamos": "superabundar",
    "superabundarían": "superabundar",
    "superabundarías": "superabundar",
    "superabundas": "superabundar",
    "superabundase": "superabundar",
    "superabundaseis": "superabundar",
    "superabundasen": "superabundar",
    "superabundases": "superabundar",
    "superabundaste": "superabundar",
    "superabundasteis": "superabundar",
    "superabunde": "superabundar",
    "superabundemos": "superabundar",
    "superabunden": "superabundar",
    "superabundes": "superabundar",
    "superabundo": "superabundar",
    "superabundábamos": "superabundar",
    "superabundáis": "superabundar",
    "superabundáramos": "superabundar",
    "superabundáremos": "superabundar",
    "superabundásemos": "superabundar",
    "superabundé": "superabundar",
    "superabundéis": "superabundar",
    "superabundó": "superabundar",
    "superaciones": "superación",
    "superactiva": "superactivo",
    "superactivas": "superactivo",
    "superactividades": "superactividad",
    "superactivos": "superactivo",
    "superad": "superar",
    "superada": "superar",
    "superadas": "superar",
    "superado": "superar",
    "superadora": "superador",
    "superadoras": "superador",
    "superadores": "superador",
    "superados": "superar",
    "superaerodinámicas": "superaerodinámica",
    "superagentes": "superagente",
    "superaleaciones": "superaleación",
    "superalimenta": "superalimentar",
    "superalimentaba": "superalimentar",
    "superalimentabais": "superalimentar",
    "superalimentaban": "superalimentar",
    "superalimentabas": "superalimentar",
    "superalimentad": "superalimentar",
    "superalimentada": "superalimentar",
    "superalimentadas": "superalimentar",
    "superalimentado": "superalimentar",
    "superalimentados": "superalimentar",
    "superalimentamos": "superalimentar",
    "superalimentan": "superalimentar",
    "superalimentando": "superalimentar",
    "superalimentara": "superalimentar",
    "superalimentarais": "superalimentar",
    "superalimentaran": "superalimentar",
    "superalimentaras": "superalimentar",
    "superalimentare": "superalimentar",
    "superalimentareis": "superalimentar",
    "superalimentaremos": "superalimentar",
    "superalimentaren": "superalimentar",
    "superalimentares": "superalimentar",
    "superalimentaron": "superalimentar",
    "superalimentará": "superalimentar",
    "superalimentarán": "superalimentar",
    "superalimentarás": "superalimentar",
    "superalimentaré": "superalimentar",
    "superalimentaréis": "superalimentar",
    "superalimentaría": "superalimentar",
    "superalimentaríais": "superalimentar",
    "superalimentaríamos": "superalimentar",
    "superalimentarían": "superalimentar",
    "superalimentarías": "superalimentar",
    "superalimentas": "superalimentar",
    "superalimentase": "superalimentar",
    "superalimentaseis": "superalimentar",
    "superalimentasen": "superalimentar",
    "superalimentases": "superalimentar",
    "superalimentaste": "superalimentar",
    "superalimentasteis": "superalimentar",
    "superalimente": "superalimentar",
    "superalimentemos": "superalimentar",
    "superalimenten": "superalimentar",
    "superalimentes": "superalimentar",
    "superalimento": "superalimentar",
    "superalimentábamos": "superalimentar",
    "superalimentáis": "superalimentar",
    "superalimentáramos": "superalimentar",
    "superalimentáremos": "superalimentar",
    "superalimentásemos": "superalimentar",
    "superalimenté": "superalimentar",
    "superalimentéis": "superalimentar",
    "superalimentó": "superalimentar",
    "superamos": "superar",
    "superan": "superar",
    "superando": "superar",
    "superantígenos": "superantígeno",
    "superara": "superar",
    "superarais": "superar",
    "superaran": "superar",
    "superaras": "superar",
    "superare": "superar",
    "superareis": "superar",
    "superaremos": "superar",
    "superaren": "superar",
    "superares": "superar",
    "superaron": "superar",
    "superará": "superar",
    "superarán": "superar",
    "superarás": "superar",
    "superaré": "superar",
    "superaréis": "superar",
    "superaría": "superar",
    "superaríais": "superar",
    "superaríamos": "superar",
    "superarían": "superar",
    "superarías": "superar",
    "superas": "superar",
    "superase": "superar",
    "superaseis": "superar",
    "superasen": "superar",
    "superases": "superar",
    "superaste": "superar",
    "superasteis": "superar",
    "superavits": "superávit",
    "superbikes": "superbike",
    "superbombarderos": "superbombardero",
    "superchera": "superchero",
    "supercheras": "superchero",
    "supercheros": "superchero",
    "supercherías": "superchería",
    "superchips": "superchip",
    "superciliares": "superciliar",
    "superclases": "superclase",
    "superconducciones": "superconducción",
    "superconducentes": "superconducente",
    "superconductiva": "superconductivo",
    "superconductivas": "superconductivo",
    "superconductividades": "superconductividad",
    "superconductivos": "superconductivo",
    "superconductora": "superconductor",
    "superconductoras": "superconductor",
    "superconductores": "superconductor",
    "superconjuntos": "superconjunto",
    "supercrítica": "supercrítico",
    "supercríticas": "supercrítico",
    "supercríticos": "supercrítico",
    "supercuentas": "supercuenta",
    "superdislocaciones": "superdislocación",
    "superdominantes": "superdominante",
    "superdotada": "superdotado",
    "superdotadas": "superdotado",
    "superdotados": "superdotado",
    "supere": "superar",
    "superegos": "superego",
    "supereminencias": "supereminencia",
    "supereminentes": "supereminente",
    "superemos": "superar",
    "superen": "superar",
    "supererogaciones": "supererogación",
    "supererogatoria": "supererogatorio",
    "supererogatorias": "supererogatorio",
    "supererogatorios": "supererogatorio",
    "superes": "superar",
    "superescisiones": "superescisión",
    "superestratos": "superestrato",
    "superestrellas": "superestrella",
    "superestructuras": "superestructura",
    "superferolítica": "superferolítico",
    "superferolíticas": "superferolítico",
    "superferolíticos": "superferolítico",
    "superfetaciones": "superfetación",
    "superficiales": "superficial",
    "superficialidades": "superficialidad",
    "superficiaria": "superficiario",
    "superficiarias": "superficiario",
    "superficiarios": "superficiario",
    "superficies": "superficie",
    "superfina": "superfino",
    "superfinas": "superfino",
    "superfinos": "superfino",
    "superflua": "superfluo",
    "superfluas": "superfluo",
    "superfluencias": "superfluencia",
    "superfluida": "superfluido",
    "superfluidades": "superfluidad",
    "superfluidas": "superfluido",
    "superfluidos": "superfluido",
    "superfluos": "superfluo",
    "superfondos": "superfondo",
    "superfosfatos": "superfosfato",
    "supergrupos": "supergrupo",
    "superheterodina": "superheterodino",
    "superheterodinas": "superheterodino",
    "superheterodinos": "superheterodino",
    "superhombres": "superhombre",
    "superhumerales": "superhumeral",
    "supericonoscopios": "supericonoscopio",
    "superintendencias": "superintendencia",
    "superintendentes": "superintendente",
    "superiora": "superior",
    "superioras": "superiora",
    "superioratos": "superiorato",
    "superiores": "superior",
    "superioridades": "superioridad",
    "superlaciones": "superlación",
    "superlativa": "superlativo",
    "superlativas": "superlativo",
    "superlativos": "superlativo",
    "superlíquida": "superlíquido",
    "superlíquidas": "superlíquido",
    "superlíquidos": "superlíquido",
    "supermarca": "supermarcar",
    "supermarcaba": "supermarcar",
    "supermarcabais": "supermarcar",
    "supermarcaban": "supermarcar",
    "supermarcabas": "supermarcar",
    "supermarcad": "supermarcar",
    "supermarcada": "supermarcar",
    "supermarcadas": "supermarcar",
    "supermarcado": "supermarcar",
    "supermarcados": "supermarcar",
    "supermarcamos": "supermarcar",
    "supermarcan": "supermarcar",
    "supermarcando": "supermarcar",
    "supermarcara": "supermarcar",
    "supermarcarais": "supermarcar",
    "supermarcaran": "supermarcar",
    "supermarcaras": "supermarcar",
    "supermarcare": "supermarcar",
    "supermarcareis": "supermarcar",
    "supermarcaremos": "supermarcar",
    "supermarcaren": "supermarcar",
    "supermarcares": "supermarcar",
    "supermarcaron": "supermarcar",
    "supermarcará": "supermarcar",
    "supermarcarán": "supermarcar",
    "supermarcarás": "supermarcar",
    "supermarcaré": "supermarcar",
    "supermarcaréis": "supermarcar",
    "supermarcaría": "supermarcar",
    "supermarcaríais": "supermarcar",
    "supermarcaríamos": "supermarcar",
    "supermarcarían": "supermarcar",
    "supermarcarías": "supermarcar",
    "supermarcas": "supermarcar",
    "supermarcase": "supermarcar",
    "supermarcaseis": "supermarcar",
    "supermarcasen": "supermarcar",
    "supermarcases": "supermarcar",
    "supermarcaste": "supermarcar",
    "supermarcasteis": "supermarcar",
    "supermarco": "supermarcar",
    "supermarcábamos": "supermarcar",
    "supermarcáis": "supermarcar",
    "supermarcáramos": "supermarcar",
    "supermarcáremos": "supermarcar",
    "supermarcásemos": "supermarcar",
    "supermarcó": "supermarcar",
    "supermarque": "supermarcar",
    "supermarquemos": "supermarcar",
    "supermarquen": "supermarcar",
    "supermarques": "supermarcar",
    "supermarqué": "supermarcar",
    "supermarquéis": "supermarcar",
    "supermercados": "supermercado",
    "supermicroordenadores": "supermicroordenador",
    "superminiordenadores": "superminiordenador",
    "supernovas": "supernova",
    "supernumeraria": "supernumerario",
    "supernumerarias": "supernumerario",
    "supernumerarios": "supernumerario",
    "supero": "superar",
    "superordenadores": "superordenador",
    "superosculaciones": "superosculación",
    "superovulaciones": "superovulación",
    "superpesada": "superpesado",
    "superpesadas": "superpesado",
    "superpesados": "superpesado",
    "superpetroleros": "superpetrolero",
    "superpoblaba": "superpoblar",
    "superpoblabais": "superpoblar",
    "superpoblaban": "superpoblar",
    "superpoblabas": "superpoblar",
    "superpoblaciones": "superpoblación",
    "superpoblad": "superpoblar",
    "superpoblada": "superpoblar",
    "superpobladas": "superpoblar",
    "superpoblado": "superpoblar",
    "superpoblados": "superpoblar",
    "superpoblamos": "superpoblar",
    "superpoblando": "superpoblar",
    "superpoblara": "superpoblar",
    "superpoblarais": "superpoblar",
    "superpoblaran": "superpoblar",
    "superpoblaras": "superpoblar",
    "superpoblare": "superpoblar",
    "superpoblareis": "superpoblar",
    "superpoblaremos": "superpoblar",
    "superpoblaren": "superpoblar",
    "superpoblares": "superpoblar",
    "superpoblaron": "superpoblar",
    "superpoblará": "superpoblar",
    "superpoblarán": "superpoblar",
    "superpoblarás": "superpoblar",
    "superpoblaré": "superpoblar",
    "superpoblaréis": "superpoblar",
    "superpoblaría": "superpoblar",
    "superpoblaríais": "superpoblar",
    "superpoblaríamos": "superpoblar",
    "superpoblarían": "superpoblar",
    "superpoblarías": "superpoblar",
    "superpoblase": "superpoblar",
    "superpoblaseis": "superpoblar",
    "superpoblasen": "superpoblar",
    "superpoblases": "superpoblar",
    "superpoblaste": "superpoblar",
    "superpoblasteis": "superpoblar",
    "superpoblemos": "superpoblar",
    "superpoblábamos": "superpoblar",
    "superpobláis": "superpoblar",
    "superpobláramos": "superpoblar",
    "superpobláremos": "superpoblar",
    "superpoblásemos": "superpoblar",
    "superpoblé": "superpoblar",
    "superpobléis": "superpoblar",
    "superpobló": "superpoblar",
    "superpolímeros": "superpolímero",
    "superpondremos": "superponer",
    "superpondrá": "superponer",
    "superpondrán": "superponer",
    "superpondrás": "superponer",
    "superpondré": "superponer",
    "superpondréis": "superponer",
    "superpondría": "superponer",
    "superpondríais": "superponer",
    "superpondríamos": "superponer",
    "superpondrían": "superponer",
    "superpondrías": "superponer",
    "superpone": "superponer",
    "superponed": "superponer",
    "superponemos": "superponer",
    "superponen": "superponer",
    "superpones": "superponer",
    "superponga": "superponer",
    "superpongamos": "superponer",
    "superpongan": "superponer",
    "superpongas": "superponer",
    "superpongo": "superponer",
    "superpongáis": "superponer",
    "superponibles": "superponible",
    "superponiendo": "superponer",
    "superponéis": "superponer",
    "superponía": "superponer",
    "superponíais": "superponer",
    "superponíamos": "superponer",
    "superponían": "superponer",
    "superponías": "superponer",
    "superposiciones": "superposición",
    "superpotencias": "superpotencia",
    "superproducciones": "superproducción",
    "superprofesionales": "superprofesional",
    "superpuebla": "superpoblar",
    "superpueblan": "superpoblar",
    "superpueblas": "superpoblar",
    "superpueble": "superpoblar",
    "superpueblen": "superpoblar",
    "superpuebles": "superpoblar",
    "superpueblo": "superpoblar",
    "superpuesta": "superponer",
    "superpuestas": "superponer",
    "superpuesto": "superponer",
    "superpuestos": "superponer",
    "superpuse": "superponer",
    "superpusiera": "superponer",
    "superpusierais": "superponer",
    "superpusieran": "superponer",
    "superpusieras": "superponer",
    "superpusiere": "superponer",
    "superpusiereis": "superponer",
    "superpusieren": "superponer",
    "superpusieres": "superponer",
    "superpusieron": "superponer",
    "superpusiese": "superponer",
    "superpusieseis": "superponer",
    "superpusiesen": "superponer",
    "superpusieses": "superponer",
    "superpusimos": "superponer",
    "superpusiste": "superponer",
    "superpusisteis": "superponer",
    "superpusiéramos": "superponer",
    "superpusiéremos": "superponer",
    "superpusiésemos": "superponer",
    "superpuso": "superponer",
    "superpón": "superponer",
    "superrealismos": "superrealismo",
    "superrealistas": "superrealista",
    "superregeneraciones": "superregeneración",
    "supersatura": "supersaturar",
    "supersaturaba": "supersaturar",
    "supersaturabais": "supersaturar",
    "supersaturaban": "supersaturar",
    "supersaturabas": "supersaturar",
    "supersaturaciones": "supersaturación",
    "supersaturad": "supersaturar",
    "supersaturada": "supersaturar",
    "supersaturadas": "supersaturar",
    "supersaturado": "supersaturar",
    "supersaturados": "supersaturar",
    "supersaturamos": "supersaturar",
    "supersaturan": "supersaturar",
    "supersaturando": "supersaturar",
    "supersaturara": "supersaturar",
    "supersaturarais": "supersaturar",
    "supersaturaran": "supersaturar",
    "supersaturaras": "supersaturar",
    "supersaturare": "supersaturar",
    "supersaturareis": "supersaturar",
    "supersaturaremos": "supersaturar",
    "supersaturaren": "supersaturar",
    "supersaturares": "supersaturar",
    "supersaturaron": "supersaturar",
    "supersaturará": "supersaturar",
    "supersaturarán": "supersaturar",
    "supersaturarás": "supersaturar",
    "supersaturaré": "supersaturar",
    "supersaturaréis": "supersaturar",
    "supersaturaría": "supersaturar",
    "supersaturaríais": "supersaturar",
    "supersaturaríamos": "supersaturar",
    "supersaturarían": "supersaturar",
    "supersaturarías": "supersaturar",
    "supersaturas": "supersaturar",
    "supersaturase": "supersaturar",
    "supersaturaseis": "supersaturar",
    "supersaturasen": "supersaturar",
    "supersaturases": "supersaturar",
    "supersaturaste": "supersaturar",
    "supersaturasteis": "supersaturar",
    "supersature": "supersaturar",
    "supersaturemos": "supersaturar",
    "supersaturen": "supersaturar",
    "supersatures": "supersaturar",
    "supersaturo": "supersaturar",
    "supersaturábamos": "supersaturar",
    "supersaturáis": "supersaturar",
    "supersaturáramos": "supersaturar",
    "supersaturáremos": "supersaturar",
    "supersaturásemos": "supersaturar",
    "supersaturé": "supersaturar",
    "supersaturéis": "supersaturar",
    "supersaturó": "supersaturar",
    "supersensibles": "supersensible",
    "superservicios": "superservicio",
    "supersincrónica": "supersincrónico",
    "supersincrónicas": "supersincrónico",
    "supersincrónicos": "supersincrónico",
    "supersticiones": "superstición",
    "supersticiosa": "supersticioso",
    "supersticiosas": "supersticioso",
    "supersticiosos": "supersticioso",
    "supersímbolos": "supersímbolo",
    "supersónica": "supersónico",
    "supersónicas": "supersónico",
    "supersónicos": "supersónico",
    "supertónicas": "supertónica",
    "superusuarios": "superusuario",
    "supervalora": "supervalorar",
    "supervaloraba": "supervalorar",
    "supervalorabais": "supervalorar",
    "supervaloraban": "supervalorar",
    "supervalorabas": "supervalorar",
    "supervaloraciones": "supervaloración",
    "supervalorad": "supervalorar",
    "supervalorada": "supervalorar",
    "supervaloradas": "supervalorar",
    "supervalorado": "supervalorar",
    "supervalorados": "supervalorar",
    "supervaloramos": "supervalorar",
    "supervaloran": "supervalorar",
    "supervalorando": "supervalorar",
    "supervalorara": "supervalorar",
    "supervalorarais": "supervalorar",
    "supervaloraran": "supervalorar",
    "supervaloraras": "supervalorar",
    "supervalorare": "supervalorar",
    "supervalorareis": "supervalorar",
    "supervaloraremos": "supervalorar",
    "supervaloraren": "supervalorar",
    "supervalorares": "supervalorar",
    "supervaloraron": "supervalorar",
    "supervalorará": "supervalorar",
    "supervalorarán": "supervalorar",
    "supervalorarás": "supervalorar",
    "supervaloraré": "supervalorar",
    "supervaloraréis": "supervalorar",
    "supervaloraría": "supervalorar",
    "supervaloraríais": "supervalorar",
    "supervaloraríamos": "supervalorar",
    "supervalorarían": "supervalorar",
    "supervalorarías": "supervalorar",
    "supervaloras": "supervalorar",
    "supervalorase": "supervalorar",
    "supervaloraseis": "supervalorar",
    "supervalorasen": "supervalorar",
    "supervalorases": "supervalorar",
    "supervaloraste": "supervalorar",
    "supervalorasteis": "supervalorar",
    "supervalore": "supervalorar",
    "supervaloremos": "supervalorar",
    "supervaloren": "supervalorar",
    "supervalores": "supervalorar",
    "supervaloro": "supervalorar",
    "supervalorábamos": "supervalorar",
    "supervaloráis": "supervalorar",
    "supervaloráramos": "supervalorar",
    "supervaloráremos": "supervalorar",
    "supervalorásemos": "supervalorar",
    "supervaloré": "supervalorar",
    "supervaloréis": "supervalorar",
    "supervaloró": "supervalorar",
    "superven": "supervenir",
    "supervenciones": "supervención",
    "supervendremos": "supervenir",
    "supervendrá": "supervenir",
    "supervendrán": "supervenir",
    "supervendrás": "supervenir",
    "supervendré": "supervenir",
    "supervendréis": "supervenir",
    "supervendría": "supervenir",
    "supervendríais": "supervenir",
    "supervendríamos": "supervenir",
    "supervendrían": "supervenir",
    "supervendrías": "supervenir",
    "supervenga": "supervenir",
    "supervengamos": "supervenir",
    "supervengan": "supervenir",
    "supervengas": "supervenir",
    "supervengo": "supervenir",
    "supervengáis": "supervenir",
    "supervenid": "supervenir",
    "supervenida": "supervenir",
    "supervenidas": "supervenir",
    "supervenido": "supervenir",
    "supervenidos": "supervenir",
    "superveniencias": "superveniencia",
    "supervenimos": "supervenir",
    "supervenía": "supervenir",
    "superveníais": "supervenir",
    "superveníamos": "supervenir",
    "supervenían": "supervenir",
    "supervenías": "supervenir",
    "supervenís": "supervenir",
    "superviene": "supervenir",
    "supervienen": "supervenir",
    "supervienes": "supervenir",
    "supervine": "supervenir",
    "superviniendo": "supervenir",
    "superviniera": "supervenir",
    "supervinierais": "supervenir",
    "supervinieran": "supervenir",
    "supervinieras": "supervenir",
    "superviniere": "supervenir",
    "superviniereis": "supervenir",
    "supervinieren": "supervenir",
    "supervinieres": "supervenir",
    "supervinieron": "supervenir",
    "superviniese": "supervenir",
    "supervinieseis": "supervenir",
    "superviniesen": "supervenir",
    "supervinieses": "supervenir",
    "supervinimos": "supervenir",
    "superviniste": "supervenir",
    "supervinisteis": "supervenir",
    "superviniéramos": "supervenir",
    "superviniéremos": "supervenir",
    "superviniésemos": "supervenir",
    "supervino": "supervenir",
    "supervisa": "supervisar",
    "supervisaba": "supervisar",
    "supervisabais": "supervisar",
    "supervisaban": "supervisar",
    "supervisabas": "supervisar",
    "supervisad": "supervisar",
    "supervisada": "supervisar",
    "supervisadas": "supervisar",
    "supervisado": "supervisar",
    "supervisados": "supervisar",
    "supervisamos": "supervisar",
    "supervisan": "supervisar",
    "supervisando": "supervisar",
    "supervisara": "supervisar",
    "supervisarais": "supervisar",
    "supervisaran": "supervisar",
    "supervisaras": "supervisar",
    "supervisare": "supervisar",
    "supervisareis": "supervisar",
    "supervisaremos": "supervisar",
    "supervisaren": "supervisar",
    "supervisares": "supervisar",
    "supervisaron": "supervisar",
    "supervisará": "supervisar",
    "supervisarán": "supervisar",
    "supervisarás": "supervisar",
    "supervisaré": "supervisar",
    "supervisaréis": "supervisar",
    "supervisaría": "supervisar",
    "supervisaríais": "supervisar",
    "supervisaríamos": "supervisar",
    "supervisarían": "supervisar",
    "supervisarías": "supervisar",
    "supervisas": "supervisar",
    "supervisase": "supervisar",
    "supervisaseis": "supervisar",
    "supervisasen": "supervisar",
    "supervisases": "supervisar",
    "supervisaste": "supervisar",
    "supervisasteis": "supervisar",
    "supervise": "supervisar",
    "supervisemos": "supervisar",
    "supervisen": "supervisar",
    "supervises": "supervisar",
    "supervisiones": "supervisión",
    "superviso": "supervisar",
    "supervisora": "supervisor",
    "supervisoras": "supervisor",
    "supervisores": "supervisor",
    "supervisábamos": "supervisar",
    "supervisáis": "supervisar",
    "supervisáramos": "supervisar",
    "supervisáremos": "supervisar",
    "supervisásemos": "supervisar",
    "supervisé": "supervisar",
    "superviséis": "supervisar",
    "supervisó": "supervisar",
    "superviva": "supervivir",
    "supervivamos": "supervivir",
    "supervivan": "supervivir",
    "supervivas": "supervivir",
    "supervive": "supervivir",
    "superviven": "supervivir",
    "supervivencias": "supervivencia",
    "supervives": "supervivir",
    "supervivid": "supervivir",
    "supervivida": "supervivir",
    "supervividas": "supervivir",
    "supervivido": "supervivir",
    "supervividos": "supervivir",
    "superviviendo": "supervivir",
    "supervivientes": "superviviente",
    "superviviera": "supervivir",
    "supervivierais": "supervivir",
    "supervivieran": "supervivir",
    "supervivieras": "supervivir",
    "superviviere": "supervivir",
    "superviviereis": "supervivir",
    "supervivieren": "supervivir",
    "supervivieres": "supervivir",
    "supervivieron": "supervivir",
    "superviviese": "supervivir",
    "supervivieseis": "supervivir",
    "superviviesen": "supervivir",
    "supervivieses": "supervivir",
    "supervivimos": "supervivir",
    "superviviremos": "supervivir",
    "supervivirá": "supervivir",
    "supervivirán": "supervivir",
    "supervivirás": "supervivir",
    "superviviré": "supervivir",
    "superviviréis": "supervivir",
    "superviviría": "supervivir",
    "superviviríais": "supervivir",
    "superviviríamos": "supervivir",
    "supervivirían": "supervivir",
    "supervivirías": "supervivir",
    "superviviste": "supervivir",
    "supervivisteis": "supervivir",
    "superviviéramos": "supervivir",
    "superviviéremos": "supervivir",
    "superviviésemos": "supervivir",
    "supervivió": "supervivir",
    "supervivo": "supervivir",
    "superviváis": "supervivir",
    "superviví": "supervivir",
    "supervivía": "supervivir",
    "supervivíais": "supervivir",
    "supervivíamos": "supervivir",
    "supervivían": "supervivir",
    "supervivías": "supervivir",
    "supervivís": "supervivir",
    "superyacentes": "superyacente",
    "superyecciones": "superyección",
    "superyós": "superyó",
    "superábamos": "superar",
    "superádita": "superádito",
    "superáditas": "superádito",
    "superáditos": "superádito",
    "superágiles": "superágil",
    "superáis": "superar",
    "superáramos": "superar",
    "superáremos": "superar",
    "superásemos": "superar",
    "superávits": "superávit",
    "superé": "superar",
    "superéis": "superar",
    "superíndices": "superíndice",
    "superó": "superar",
    "superóhmica": "superóhmico",
    "superóhmicas": "superóhmico",
    "superóhmicos": "superóhmico",
    "superórdenes": "superorden",
    "superóxidos": "superóxido",
    "supes": "supe",
    "supiera": "saber",
    "supierais": "saber",
    "supieran": "saber",
    "supieras": "saber",
    "supiere": "saber",
    "supiereis": "saber",
    "supieren": "saber",
    "supieres": "saber",
    "supieron": "saber",
    "supiese": "saber",
    "supieseis": "saber",
    "supiesen": "saber",
    "supieses": "saber",
    "supimos": "saber",
    "supina": "supino",
    "supinaciones": "supinación",
    "supinadores": "supinador",
    "supinas": "supino",
    "supinos": "supino",
    "supiste": "saber",
    "supisteis": "saber",
    "supiéramos": "saber",
    "supiéremos": "saber",
    "supiésemos": "saber",
    "supla": "suplir",
    "suplamos": "suplir",
    "suplan": "suplir",
    "suplanta": "suplantar",
    "suplantaba": "suplantar",
    "suplantabais": "suplantar",
    "suplantaban": "suplantar",
    "suplantabas": "suplantar",
    "suplantables": "suplantable",
    "suplantaciones": "suplantación",
    "suplantad": "suplantar",
    "suplantada": "suplantar",
    "suplantadas": "suplantar",
    "suplantado": "suplantar",
    "suplantadora": "suplantador",
    "suplantadoras": "suplantador",
    "suplantadores": "suplantador",
    "suplantados": "suplantar",
    "suplantamos": "suplantar",
    "suplantan": "suplantar",
    "suplantando": "suplantar",
    "suplantara": "suplantar",
    "suplantarais": "suplantar",
    "suplantaran": "suplantar",
    "suplantaras": "suplantar",
    "suplantare": "suplantar",
    "suplantareis": "suplantar",
    "suplantaremos": "suplantar",
    "suplantaren": "suplantar",
    "suplantares": "suplantar",
    "suplantaron": "suplantar",
    "suplantará": "suplantar",
    "suplantarán": "suplantar",
    "suplantarás": "suplantar",
    "suplantaré": "suplantar",
    "suplantaréis": "suplantar",
    "suplantaría": "suplantar",
    "suplantaríais": "suplantar",
    "suplantaríamos": "suplantar",
    "suplantarían": "suplantar",
    "suplantarías": "suplantar",
    "suplantas": "suplantar",
    "suplantase": "suplantar",
    "suplantaseis": "suplantar",
    "suplantasen": "suplantar",
    "suplantases": "suplantar",
    "suplantaste": "suplantar",
    "suplantasteis": "suplantar",
    "suplante": "suplantar",
    "suplantemos": "suplantar",
    "suplanten": "suplantar",
    "suplantes": "suplantar",
    "suplanto": "suplantar",
    "suplantábamos": "suplantar",
    "suplantáis": "suplantar",
    "suplantáramos": "suplantar",
    "suplantáremos": "suplantar",
    "suplantásemos": "suplantar",
    "suplanté": "suplantar",
    "suplantéis": "suplantar",
    "suplantó": "suplantar",
    "suplas": "suplir",
    "suple": "suplir",
    "supleciones": "supleción",
    "suplementa": "suplementar",
    "suplementaba": "suplementar",
    "suplementabais": "suplementar",
    "suplementaban": "suplementar",
    "suplementabas": "suplementar",
    "suplementad": "suplementar",
    "suplementada": "suplementar",
    "suplementadas": "suplementar",
    "suplementado": "suplementar",
    "suplementados": "suplementar",
    "suplementales": "suplemental",
    "suplementamos": "suplementar",
    "suplementan": "suplementar",
    "suplementando": "suplementar",
    "suplementara": "suplementar",
    "suplementarais": "suplementar",
    "suplementaran": "suplementar",
    "suplementaras": "suplementar",
    "suplementare": "suplementar",
    "suplementareis": "suplementar",
    "suplementaremos": "suplementar",
    "suplementaren": "suplementar",
    "suplementares": "suplementar",
    "suplementaria": "suplementario",
    "suplementarias": "suplementario",
    "suplementarios": "suplementario",
    "suplementaron": "suplementar",
    "suplementará": "suplementar",
    "suplementarán": "suplementar",
    "suplementarás": "suplementar",
    "suplementaré": "suplementar",
    "suplementaréis": "suplementar",
    "suplementaría": "suplementar",
    "suplementaríais": "suplementar",
    "suplementaríamos": "suplementar",
    "suplementarían": "suplementar",
    "suplementarías": "suplementar",
    "suplementas": "suplementar",
    "suplementase": "suplementar",
    "suplementaseis": "suplementar",
    "suplementasen": "suplementar",
    "suplementases": "suplementar",
    "suplementaste": "suplementar",
    "suplementasteis": "suplementar",
    "suplemente": "suplementar",
    "suplementemos": "suplementar",
    "suplementen": "suplementar",
    "suplementes": "suplementar",
    "suplemento": "suplementar",
    "suplementos": "suplemento",
    "suplementábamos": "suplementar",
    "suplementáis": "suplementar",
    "suplementáramos": "suplementar",
    "suplementáremos": "suplementar",
    "suplementásemos": "suplementar",
    "suplementé": "suplementar",
    "suplementéis": "suplementar",
    "suplementó": "suplementar",
    "suplen": "suplir",
    "suplencias": "suplencia",
    "suplentes": "suplente",
    "suples": "suplir",
    "supletiva": "supletivo",
    "supletivas": "supletivo",
    "supletivos": "supletivo",
    "supletoria": "supletorio",
    "supletorias": "supletorio",
    "supletorios": "supletorio",
    "suplica": "suplicar",
    "suplicaba": "suplicar",
    "suplicabais": "suplicar",
    "suplicaban": "suplicar",
    "suplicabas": "suplicar",
    "suplicaciones": "suplicación",
    "suplicad": "suplicar",
    "suplicada": "suplicar",
    "suplicadas": "suplicar",
    "suplicado": "suplicar",
    "suplicados": "suplicar",
    "suplicamos": "suplicar",
    "suplican": "suplicar",
    "suplicando": "suplicar",
    "suplicantes": "suplicante",
    "suplicara": "suplicar",
    "suplicarais": "suplicar",
    "suplicaran": "suplicar",
    "suplicaras": "suplicar",
    "suplicare": "suplicar",
    "suplicareis": "suplicar",
    "suplicaremos": "suplicar",
    "suplicaren": "suplicar",
    "suplicares": "suplicar",
    "suplicaron": "suplicar",
    "suplicará": "suplicar",
    "suplicarán": "suplicar",
    "suplicarás": "suplicar",
    "suplicaré": "suplicar",
    "suplicaréis": "suplicar",
    "suplicaría": "suplicar",
    "suplicaríais": "suplicar",
    "suplicaríamos": "suplicar",
    "suplicarían": "suplicar",
    "suplicarías": "suplicar",
    "suplicas": "suplicar",
    "suplicase": "suplicar",
    "suplicaseis": "suplicar",
    "suplicasen": "suplicar",
    "suplicases": "suplicar",
    "suplicaste": "suplicar",
    "suplicasteis": "suplicar",
    "suplicatoria": "suplicatorio",
    "suplicatorias": "suplicatorio",
    "suplicatorios": "suplicatorio",
    "suplicios": "suplicio",
    "suplico": "suplicar",
    "suplicos": "suplico",
    "suplicábamos": "suplicar",
    "suplicáis": "suplicar",
    "suplicáramos": "suplicar",
    "suplicáremos": "suplicar",
    "suplicásemos": "suplicar",
    "suplicó": "suplicar",
    "suplid": "suplir",
    "suplida": "suplir",
    "suplidas": "suplir",
    "suplido": "suplir",
    "suplidora": "suplidor",
    "suplidoras": "suplidor",
    "suplidores": "suplidor",
    "suplidos": "suplir",
    "supliendo": "suplir",
    "supliera": "suplir",
    "suplierais": "suplir",
    "suplieran": "suplir",
    "suplieras": "suplir",
    "supliere": "suplir",
    "supliereis": "suplir",
    "suplieren": "suplir",
    "suplieres": "suplir",
    "suplieron": "suplir",
    "supliese": "suplir",
    "suplieseis": "suplir",
    "supliesen": "suplir",
    "suplieses": "suplir",
    "suplimos": "suplir",
    "suplique": "suplicar",
    "supliquemos": "suplicar",
    "supliquen": "suplicar",
    "supliques": "suplicar",
    "supliqué": "suplicar",
    "supliquéis": "suplicar",
    "supliremos": "suplir",
    "suplirá": "suplir",
    "suplirán": "suplir",
    "suplirás": "suplir",
    "supliré": "suplir",
    "supliréis": "suplir",
    "supliría": "suplir",
    "supliríais": "suplir",
    "supliríamos": "suplir",
    "suplirían": "suplir",
    "suplirías": "suplir",
    "supliste": "suplir",
    "suplisteis": "suplir",
    "supliéramos": "suplir",
    "supliéremos": "suplir",
    "supliésemos": "suplir",
    "suplió": "suplir",
    "suplo": "suplir",
    "supláis": "suplir",
    "suplí": "suplir",
    "suplía": "suplir",
    "suplíais": "suplir",
    "suplíamos": "suplir",
    "suplían": "suplir",
    "suplías": "suplir",
    "suplís": "suplir",
    "supo": "saber",
    "supondremos": "suponer",
    "supondrá": "suponer",
    "supondrán": "suponer",
    "supondrás": "suponer",
    "supondré": "suponer",
    "supondréis": "suponer",
    "supondría": "suponer",
    "supondríais": "suponer",
    "supondríamos": "suponer",
    "supondrían": "suponer",
    "supondrías": "suponer",
    "supone": "suponer",
    "suponed": "suponer",
    "suponedora": "suponedor",
    "suponedoras": "suponedor",
    "suponedores": "suponedor",
    "suponemos": "suponer",
    "suponen": "suponer",
    "supones": "suponer",
    "suponga": "suponer",
    "supongamos": "suponer",
    "supongan": "suponer",
    "supongas": "suponer",
    "supongo": "suponer",
    "supongáis": "suponer",
    "suponiendo": "suponer",
    "suponéis": "suponer",
    "suponía": "suponer",
    "suponíais": "suponer",
    "suponíamos": "suponer",
    "suponían": "suponer",
    "suponías": "suponer",
    "suposiciones": "suposición",
    "supositicia": "supositicio",
    "supositicias": "supositicio",
    "supositicios": "supositicio",
    "supositiva": "supositivo",
    "supositivas": "supositivo",
    "supositivos": "supositivo",
    "supositorios": "supositorio",
    "supraalveolares": "supraalveolar",
    "supraamigdalina": "supraamigdalino",
    "supraamigdalinas": "supraamigdalino",
    "supraamigdalinos": "supraamigdalino",
    "supraaudibles": "supraaudible",
    "suprabucales": "suprabucal",
    "supracallosa": "supracalloso",
    "supracallosas": "supracalloso",
    "supracallosos": "supracalloso",
    "supraclara": "supraclaro",
    "supraclaras": "supraclaro",
    "supraclaros": "supraclaro",
    "supraclaviculares": "supraclavicular",
    "supraconductividades": "supraconductividad",
    "supraconductora": "supraconductor",
    "supraconductoras": "supraconductor",
    "supraconductores": "supraconductor",
    "supracondílea": "supracondíleo",
    "supracondíleas": "supracondíleo",
    "supracondíleos": "supracondíleo",
    "supraconvergentes": "supraconvergente",
    "supracoroidea": "supracoroideo",
    "supracoroideas": "supracoroideo",
    "supracoroideos": "supracoroideo",
    "supradivergentes": "supradivergente",
    "supraescapulares": "supraescapular",
    "supraespinosa": "supraespinoso",
    "supraespinosas": "supraespinoso",
    "supraespinosos": "supraespinoso",
    "supraesternales": "supraesternal",
    "suprafluida": "suprafluido",
    "suprafluidas": "suprafluido",
    "suprafluidos": "suprafluido",
    "supraglenoidea": "supraglenoideo",
    "supraglenoideas": "supraglenoideo",
    "supraglenoideos": "supraglenoideo",
    "supraglótica": "supraglótico",
    "supraglóticas": "supraglótico",
    "supraglóticos": "supraglótico",
    "supragnacias": "supragnacia",
    "suprahioidea": "suprahioideo",
    "suprahioideas": "suprahioideo",
    "suprahioideos": "suprahioideo",
    "supralegalidades": "supralegalidad",
    "supraliminares": "supraliminar",
    "supralíquida": "supralíquido",
    "supralíquidas": "supralíquido",
    "supralíquidos": "supralíquido",
    "supramandibulares": "supramandibular",
    "supramarginales": "supramarginal",
    "supramastoidea": "supramastoideo",
    "supramastoideas": "supramastoideo",
    "supramastoideos": "supramastoideo",
    "supramaxilares": "supramaxilar",
    "suprameática": "suprameático",
    "suprameáticas": "suprameático",
    "suprameáticos": "suprameático",
    "supramunicipales": "supramunicipal",
    "supranacionales": "supranacional",
    "supranaturales": "supranatural",
    "supraoclusiones": "supraoclusión",
    "supraorbitaria": "supraorbitario",
    "supraorbitarias": "supraorbitario",
    "supraorbitarios": "supraorbitario",
    "supraperióstica": "supraperióstico",
    "supraperiósticas": "supraperióstico",
    "supraperiósticos": "supraperióstico",
    "suprapineales": "suprapineal",
    "suprapleurales": "suprapleural",
    "supraprotestos": "supraprotesto",
    "suprarrenales": "suprarrenal",
    "suprarrotuliana": "suprarrotuliano",
    "suprarrotulianas": "suprarrotuliano",
    "suprarrotulianos": "suprarrotuliano",
    "suprasegmentales": "suprasegmental",
    "supraseguros": "supraseguro",
    "supratonsilares": "supratonsilar",
    "supratrocleares": "supratroclear",
    "supravesicales": "supravesical",
    "suprayacentes": "suprayacente",
    "suprayectiva": "suprayectivo",
    "suprayectivas": "suprayectivo",
    "suprayectivos": "suprayectivo",
    "supraóptica": "supraóptico",
    "supraópticas": "supraóptico",
    "supraópticos": "supraóptico",
    "suprema": "supremo",
    "supremacías": "supremacía",
    "supremas": "supremo",
    "suprematismos": "suprematismo",
    "supremos": "supremo",
    "supresibles": "supresible",
    "supresiones": "supresión",
    "supresora": "supresor",
    "supresoras": "supresor",
    "supresores": "supresor",
    "suprima": "suprimir",
    "suprimamos": "suprimir",
    "supriman": "suprimir",
    "suprimas": "suprimir",
    "suprime": "suprimir",
    "suprimen": "suprimir",
    "suprimes": "suprimir",
    "suprimid": "suprimir",
    "suprimida": "suprimir",
    "suprimidas": "suprimir",
    "suprimido": "suprimir",
    "suprimidos": "suprimir",
    "suprimiendo": "suprimir",
    "suprimiera": "suprimir",
    "suprimierais": "suprimir",
    "suprimieran": "suprimir",
    "suprimieras": "suprimir",
    "suprimiere": "suprimir",
    "suprimiereis": "suprimir",
    "suprimieren": "suprimir",
    "suprimieres": "suprimir",
    "suprimieron": "suprimir",
    "suprimiese": "suprimir",
    "suprimieseis": "suprimir",
    "suprimiesen": "suprimir",
    "suprimieses": "suprimir",
    "suprimimos": "suprimir",
    "suprimiremos": "suprimir",
    "suprimirá": "suprimir",
    "suprimirán": "suprimir",
    "suprimirás": "suprimir",
    "suprimiré": "suprimir",
    "suprimiréis": "suprimir",
    "suprimiría": "suprimir",
    "suprimiríais": "suprimir",
    "suprimiríamos": "suprimir",
    "suprimirían": "suprimir",
    "suprimirías": "suprimir",
    "suprimiste": "suprimir",
    "suprimisteis": "suprimir",
    "suprimiéramos": "suprimir",
    "suprimiéremos": "suprimir",
    "suprimiésemos": "suprimir",
    "suprimió": "suprimir",
    "suprimo": "suprimir",
    "suprimáis": "suprimir",
    "suprimí": "suprimir",
    "suprimía": "suprimir",
    "suprimíais": "suprimir",
    "suprimíamos": "suprimir",
    "suprimían": "suprimir",
    "suprimías": "suprimir",
    "suprimís": "suprimir",
    "supuesta": "suponer",
    "supuestas": "suponer",
    "supuesto": "suponer",
    "supuestos": "supuesto",
    "supura": "supurar",
    "supuraba": "supurar",
    "supurabais": "supurar",
    "supuraban": "supurar",
    "supurabas": "supurar",
    "supuraciones": "supuración",
    "supuraciónes": "supuración",
    "supurad": "supurar",
    "supurada": "supurar",
    "supuradas": "supurar",
    "supurado": "supurar",
    "supurados": "supurar",
    "supuramos": "supurar",
    "supuran": "supurar",
    "supurando": "supurar",
    "supurara": "supurar",
    "supurarais": "supurar",
    "supuraran": "supurar",
    "supuraras": "supurar",
    "supurare": "supurar",
    "supurareis": "supurar",
    "supuraremos": "supurar",
    "supuraren": "supurar",
    "supurares": "supurar",
    "supuraron": "supurar",
    "supurará": "supurar",
    "supurarán": "supurar",
    "supurarás": "supurar",
    "supuraré": "supurar",
    "supuraréis": "supurar",
    "supuraría": "supurar",
    "supuraríais": "supurar",
    "supuraríamos": "supurar",
    "supurarían": "supurar",
    "supurarías": "supurar",
    "supuras": "supurar",
    "supurase": "supurar",
    "supuraseis": "supurar",
    "supurasen": "supurar",
    "supurases": "supurar",
    "supuraste": "supurar",
    "supurasteis": "supurar",
    "supurativa": "supurativo",
    "supurativas": "supurativo",
    "supurativos": "supurativo",
    "supuratoria": "supuratorio",
    "supuratorias": "supuratorio",
    "supuratorios": "supuratorio",
    "supure": "supurar",
    "supuremos": "supurar",
    "supuren": "supurar",
    "supures": "supurar",
    "supuro": "supurar",
    "supurábamos": "supurar",
    "supuráis": "supurar",
    "supuráramos": "supurar",
    "supuráremos": "supurar",
    "supurásemos": "supurar",
    "supuré": "supurar",
    "supuréis": "supurar",
    "supuró": "supurar",
    "supuse": "suponer",
    "supusiera": "suponer",
    "supusierais": "suponer",
    "supusieran": "suponer",
    "supusieras": "suponer",
    "supusiere": "suponer",
    "supusiereis": "suponer",
    "supusieren": "suponer",
    "supusieres": "suponer",
    "supusieron": "suponer",
    "supusiese": "suponer",
    "supusieseis": "suponer",
    "supusiesen": "suponer",
    "supusieses": "suponer",
    "supusimos": "suponer",
    "supusiste": "suponer",
    "supusisteis": "suponer",
    "supusiéramos": "suponer",
    "supusiéremos": "suponer",
    "supusiésemos": "suponer",
    "supuso": "suponer",
    "suputa": "suputar",
    "suputaba": "suputar",
    "suputabais": "suputar",
    "suputaban": "suputar",
    "suputabas": "suputar",
    "suputaciones": "suputación",
    "suputad": "suputar",
    "suputada": "suputar",
    "suputadas": "suputar",
    "suputado": "suputar",
    "suputados": "suputar",
    "suputamos": "suputar",
    "suputan": "suputar",
    "suputando": "suputar",
    "suputara": "suputar",
    "suputarais": "suputar",
    "suputaran": "suputar",
    "suputaras": "suputar",
    "suputare": "suputar",
    "suputareis": "suputar",
    "suputaremos": "suputar",
    "suputaren": "suputar",
    "suputares": "suputar",
    "suputaron": "suputar",
    "suputará": "suputar",
    "suputarán": "suputar",
    "suputarás": "suputar",
    "suputaré": "suputar",
    "suputaréis": "suputar",
    "suputaría": "suputar",
    "suputaríais": "suputar",
    "suputaríamos": "suputar",
    "suputarían": "suputar",
    "suputarías": "suputar",
    "suputas": "suputar",
    "suputase": "suputar",
    "suputaseis": "suputar",
    "suputasen": "suputar",
    "suputases": "suputar",
    "suputaste": "suputar",
    "suputasteis": "suputar",
    "supute": "suputar",
    "suputemos": "suputar",
    "suputen": "suputar",
    "suputes": "suputar",
    "suputo": "suputar",
    "suputábamos": "suputar",
    "suputáis": "suputar",
    "suputáramos": "suputar",
    "suputáremos": "suputar",
    "suputásemos": "suputar",
    "suputé": "suputar",
    "suputéis": "suputar",
    "suputó": "suputar",
    "supérstites": "supérstite",
    "supón": "suponer",
    "supósitos": "supósito",
    "suraes": "surá",
    "surafricana": "surafricano",
    "surafricanas": "surafricano",
    "surafricanos": "surafricano",
    "surales": "sural",
    "suramericana": "suramericano",
    "suramericanas": "suramericano",
    "suramericanos": "suramericano",
    "suras": "sura",
    "surca": "surcar",
    "surcaba": "surcar",
    "surcabais": "surcar",
    "surcaban": "surcar",
    "surcabas": "surcar",
    "surcad": "surcar",
    "surcada": "surcar",
    "surcadas": "surcar",
    "surcado": "surcar",
    "surcadora": "surcador",
    "surcadoras": "surcador",
    "surcadores": "surcador",
    "surcados": "surcar",
    "surcamos": "surcar",
    "surcan": "surcar",
    "surcando": "surcar",
    "surcara": "surcar",
    "surcarais": "surcar",
    "surcaran": "surcar",
    "surcaras": "surcar",
    "surcare": "surcar",
    "surcareis": "surcar",
    "surcaremos": "surcar",
    "surcaren": "surcar",
    "surcares": "surcar",
    "surcaron": "surcar",
    "surcará": "surcar",
    "surcarán": "surcar",
    "surcarás": "surcar",
    "surcaré": "surcar",
    "surcaréis": "surcar",
    "surcaría": "surcar",
    "surcaríais": "surcar",
    "surcaríamos": "surcar",
    "surcarían": "surcar",
    "surcarías": "surcar",
    "surcas": "surcar",
    "surcase": "surcar",
    "surcaseis": "surcar",
    "surcasen": "surcar",
    "surcases": "surcar",
    "surcaste": "surcar",
    "surcasteis": "surcar",
    "surco": "surcar",
    "surcoreana": "surcoreano",
    "surcoreanas": "surcoreano",
    "surcoreanos": "surcoreano",
    "surcos": "surco",
    "surculada": "surculado",
    "surculadas": "surculado",
    "surculados": "surculado",
    "surculosa": "surculoso",
    "surculosas": "surculoso",
    "surculosos": "surculoso",
    "surcábamos": "surcar",
    "surcáis": "surcar",
    "surcáramos": "surcar",
    "surcáremos": "surcar",
    "surcásemos": "surcar",
    "surcó": "surcar",
    "sures": "sur",
    "surestes": "sureste",
    "sureña": "sureño",
    "sureñas": "sureño",
    "sureños": "sureño",
    "surfactantes": "surfactante",
    "surfera": "surfero",
    "surferas": "surfero",
    "surferos": "surfero",
    "surfistas": "surfista",
    "surge": "surgir",
    "surgen": "surgir",
    "surgentes": "surgente",
    "surges": "surgir",
    "surgid": "surgir",
    "surgida": "surgir",
    "surgidas": "surgir",
    "surgideros": "surgidero",
    "surgido": "surgir",
    "surgidora": "surgidor",
    "surgidoras": "surgidor",
    "surgidores": "surgidor",
    "surgidos": "surgir",
    "surgiendo": "surgir",
    "surgiera": "surgir",
    "surgierais": "surgir",
    "surgieran": "surgir",
    "surgieras": "surgir",
    "surgiere": "surgir",
    "surgiereis": "surgir",
    "surgieren": "surgir",
    "surgieres": "surgir",
    "surgieron": "surgir",
    "surgiese": "surgir",
    "surgieseis": "surgir",
    "surgiesen": "surgir",
    "surgieses": "surgir",
    "surgimientos": "surgimiento",
    "surgimos": "surgir",
    "surgiremos": "surgir",
    "surgirá": "surgir",
    "surgirán": "surgir",
    "surgirás": "surgir",
    "surgiré": "surgir",
    "surgiréis": "surgir",
    "surgiría": "surgir",
    "surgiríais": "surgir",
    "surgiríamos": "surgir",
    "surgirían": "surgir",
    "surgirías": "surgir",
    "surgiste": "surgir",
    "surgisteis": "surgir",
    "surgiéramos": "surgir",
    "surgiéremos": "surgir",
    "surgiésemos": "surgir",
    "surgió": "surgir",
    "surgí": "surgir",
    "surgía": "surgir",
    "surgíais": "surgir",
    "surgíamos": "surgir",
    "surgían": "surgir",
    "surgías": "surgir",
    "surgís": "surgir",
    "suricatas": "suricata",
    "suripantas": "suripanta",
    "surja": "surgir",
    "surjamos": "surgir",
    "surjan": "surgir",
    "surjas": "surgir",
    "surjo": "surgir",
    "surjáis": "surgir",
    "surmenages": "surmenage",
    "suroccidentales": "suroccidental",
    "suroestes": "suroeste",
    "surorientales": "suroriental",
    "surplaces": "surplace",
    "surque": "surcar",
    "surquemos": "surcar",
    "surquen": "surcar",
    "surques": "surcar",
    "surqué": "surcar",
    "surquéis": "surcar",
    "surreales": "surreal",
    "surrealismos": "surrealismo",
    "surrealistas": "surrealista",
    "sursudoestes": "sursudoeste",
    "sursuncordas": "sursuncorda",
    "surta": "surtir",
    "surtamos": "surtir",
    "surtan": "surtir",
    "surtas": "surtir",
    "surte": "surtir",
    "surten": "surtir",
    "surtes": "surtir",
    "surtid": "surtir",
    "surtida": "surtir",
    "surtidas": "surtir",
    "surtideros": "surtidero",
    "surtido": "surtir",
    "surtidora": "surtidor",
    "surtidoras": "surtidor",
    "surtidores": "surtidor",
    "surtidos": "surtir",
    "surtiendo": "surtir",
    "surtiera": "surtir",
    "surtierais": "surtir",
    "surtieran": "surtir",
    "surtieras": "surtir",
    "surtiere": "surtir",
    "surtiereis": "surtir",
    "surtieren": "surtir",
    "surtieres": "surtir",
    "surtieron": "surtir",
    "surtiese": "surtir",
    "surtieseis": "surtir",
    "surtiesen": "surtir",
    "surtieses": "surtir",
    "surtimientos": "surtimiento",
    "surtimos": "surtir",
    "surtiremos": "surtir",
    "surtirá": "surtir",
    "surtirán": "surtir",
    "surtirás": "surtir",
    "surtiré": "surtir",
    "surtiréis": "surtir",
    "surtiría": "surtir",
    "surtiríais": "surtir",
    "surtiríamos": "surtir",
    "surtirían": "surtir",
    "surtirías": "surtir",
    "surtiste": "surtir",
    "surtisteis": "surtir",
    "surtiéramos": "surtir",
    "surtiéremos": "surtir",
    "surtiésemos": "surtir",
    "surtió": "surtir",
    "surto": "surtir",
    "surtáis": "surtir",
    "surtí": "surtir",
    "surtía": "surtir",
    "surtíais": "surtir",
    "surtíamos": "surtir",
    "surtían": "surtir",
    "surtías": "surtir",
    "surtís": "surtir",
    "surás": "surá",
    "sus": "su",
    "suscepciones": "suscepción",
    "susceptancias": "susceptancia",
    "susceptibilidades": "susceptibilidad",
    "susceptibles": "susceptible",
    "susceptiva": "susceptivo",
    "susceptivas": "susceptivo",
    "susceptivos": "susceptivo",
    "suscita": "suscitar",
    "suscitaba": "suscitar",
    "suscitabais": "suscitar",
    "suscitaban": "suscitar",
    "suscitabas": "suscitar",
    "suscitaciones": "suscitación",
    "suscitad": "suscitar",
    "suscitada": "suscitar",
    "suscitadas": "suscitar",
    "suscitado": "suscitar",
    "suscitados": "suscitar",
    "suscitamos": "suscitar",
    "suscitan": "suscitar",
    "suscitando": "suscitar",
    "suscitara": "suscitar",
    "suscitarais": "suscitar",
    "suscitaran": "suscitar",
    "suscitaras": "suscitar",
    "suscitare": "suscitar",
    "suscitareis": "suscitar",
    "suscitaremos": "suscitar",
    "suscitaren": "suscitar",
    "suscitares": "suscitar",
    "suscitaron": "suscitar",
    "suscitará": "suscitar",
    "suscitarán": "suscitar",
    "suscitarás": "suscitar",
    "suscitaré": "suscitar",
    "suscitaréis": "suscitar",
    "suscitaría": "suscitar",
    "suscitaríais": "suscitar",
    "suscitaríamos": "suscitar",
    "suscitarían": "suscitar",
    "suscitarías": "suscitar",
    "suscitas": "suscitar",
    "suscitase": "suscitar",
    "suscitaseis": "suscitar",
    "suscitasen": "suscitar",
    "suscitases": "suscitar",
    "suscitaste": "suscitar",
    "suscitasteis": "suscitar",
    "suscite": "suscitar",
    "suscitemos": "suscitar",
    "susciten": "suscitar",
    "suscites": "suscitar",
    "suscito": "suscitar",
    "suscitábamos": "suscitar",
    "suscitáis": "suscitar",
    "suscitáramos": "suscitar",
    "suscitáremos": "suscitar",
    "suscitásemos": "suscitar",
    "suscité": "suscitar",
    "suscitéis": "suscitar",
    "suscitó": "suscitar",
    "suscriba": "suscribir",
    "suscribamos": "suscribir",
    "suscriban": "suscribir",
    "suscribas": "suscribir",
    "suscribe": "suscribir",
    "suscriben": "suscribir",
    "suscribes": "suscribir",
    "suscribid": "suscribir",
    "suscribiendo": "suscribir",
    "suscribiera": "suscribir",
    "suscribierais": "suscribir",
    "suscribieran": "suscribir",
    "suscribieras": "suscribir",
    "suscribiere": "suscribir",
    "suscribiereis": "suscribir",
    "suscribieren": "suscribir",
    "suscribieres": "suscribir",
    "suscribieron": "suscribir",
    "suscribiese": "suscribir",
    "suscribieseis": "suscribir",
    "suscribiesen": "suscribir",
    "suscribieses": "suscribir",
    "suscribimos": "suscribir",
    "suscribiremos": "suscribir",
    "suscribirá": "suscribir",
    "suscribirán": "suscribir",
    "suscribirás": "suscribir",
    "suscribiré": "suscribir",
    "suscribiréis": "suscribir",
    "suscribiría": "suscribir",
    "suscribiríais": "suscribir",
    "suscribiríamos": "suscribir",
    "suscribirían": "suscribir",
    "suscribirías": "suscribir",
    "suscribiste": "suscribir",
    "suscribisteis": "suscribir",
    "suscribiéramos": "suscribir",
    "suscribiéremos": "suscribir",
    "suscribiésemos": "suscribir",
    "suscribió": "suscribir",
    "suscribo": "suscribir",
    "suscribáis": "suscribir",
    "suscribí": "suscribir",
    "suscribía": "suscribir",
    "suscribíais": "suscribir",
    "suscribíamos": "suscribir",
    "suscribían": "suscribir",
    "suscribías": "suscribir",
    "suscribís": "suscribir",
    "suscripciones": "suscripción",
    "suscripta": "suscripto",
    "suscriptas": "suscripto",
    "suscriptora": "suscriptor",
    "suscriptoras": "suscriptor",
    "suscriptores": "suscriptor",
    "suscriptos": "suscripto",
    "suscrita": "suscribir",
    "suscritas": "suscribir",
    "suscrito": "suscribir",
    "suscritos": "suscribir",
    "susodicha": "susodicho",
    "susodichas": "susodicho",
    "susodichos": "susodicho",
    "suspenda": "suspender",
    "suspendamos": "suspender",
    "suspendan": "suspender",
    "suspendas": "suspender",
    "suspende": "suspender",
    "suspended": "suspender",
    "suspendedora": "suspendedor",
    "suspendedoras": "suspendedor",
    "suspendedores": "suspendedor",
    "suspendemos": "suspender",
    "suspenden": "suspender",
    "suspenderemos": "suspender",
    "suspenderá": "suspender",
    "suspenderán": "suspender",
    "suspenderás": "suspender",
    "suspenderé": "suspender",
    "suspenderéis": "suspender",
    "suspendería": "suspender",
    "suspenderíais": "suspender",
    "suspenderíamos": "suspender",
    "suspenderían": "suspender",
    "suspenderías": "suspender",
    "suspendes": "suspender",
    "suspendida": "suspender",
    "suspendidas": "suspender",
    "suspendido": "suspender",
    "suspendidos": "suspender",
    "suspendiendo": "suspender",
    "suspendiera": "suspender",
    "suspendierais": "suspender",
    "suspendieran": "suspender",
    "suspendieras": "suspender",
    "suspendiere": "suspender",
    "suspendiereis": "suspender",
    "suspendieren": "suspender",
    "suspendieres": "suspender",
    "suspendieron": "suspender",
    "suspendiese": "suspender",
    "suspendieseis": "suspender",
    "suspendiesen": "suspender",
    "suspendieses": "suspender",
    "suspendimos": "suspender",
    "suspendiste": "suspender",
    "suspendisteis": "suspender",
    "suspendiéramos": "suspender",
    "suspendiéremos": "suspender",
    "suspendiésemos": "suspender",
    "suspendió": "suspender",
    "suspendo": "suspender",
    "suspendáis": "suspender",
    "suspendéis": "suspender",
    "suspendí": "suspender",
    "suspendía": "suspender",
    "suspendíais": "suspender",
    "suspendíamos": "suspender",
    "suspendían": "suspender",
    "suspendías": "suspender",
    "suspensa": "suspender",
    "suspensas": "suspender",
    "suspensees": "suspense",
    "suspenses": "suspense",
    "suspensiones": "suspensión",
    "suspensiva": "suspensivo",
    "suspensivas": "suspensivo",
    "suspensivos": "suspensivo",
    "suspenso": "suspender",
    "suspensoides": "suspensoide",
    "suspensora": "suspensor",
    "suspensoras": "suspensor",
    "suspensores": "suspensor",
    "suspensoria": "suspensorio",
    "suspensorias": "suspensorio",
    "suspensorios": "suspensorio",
    "suspensos": "suspenso",
    "suspicaces": "suspicaz",
    "suspicacias": "suspicacia",
    "suspicazes": "suspicaz",
    "suspira": "suspirar",
    "suspiraba": "suspirar",
    "suspirabais": "suspirar",
    "suspiraban": "suspirar",
    "suspirabas": "suspirar",
    "suspirad": "suspirar",
    "suspirada": "suspirar",
    "suspiradas": "suspirar",
    "suspirado": "suspirar",
    "suspirados": "suspirar",
    "suspiramos": "suspirar",
    "suspiran": "suspirar",
    "suspirando": "suspirar",
    "suspirara": "suspirar",
    "suspirarais": "suspirar",
    "suspiraran": "suspirar",
    "suspiraras": "suspirar",
    "suspirare": "suspirar",
    "suspirareis": "suspirar",
    "suspiraremos": "suspirar",
    "suspiraren": "suspirar",
    "suspirares": "suspirar",
    "suspiraron": "suspirar",
    "suspirará": "suspirar",
    "suspirarán": "suspirar",
    "suspirarás": "suspirar",
    "suspiraré": "suspirar",
    "suspiraréis": "suspirar",
    "suspiraría": "suspirar",
    "suspiraríais": "suspirar",
    "suspiraríamos": "suspirar",
    "suspirarían": "suspirar",
    "suspirarías": "suspirar",
    "suspiras": "suspirar",
    "suspirase": "suspirar",
    "suspiraseis": "suspirar",
    "suspirasen": "suspirar",
    "suspirases": "suspirar",
    "suspiraste": "suspirar",
    "suspirasteis": "suspirar",
    "suspire": "suspirar",
    "suspiremos": "suspirar",
    "suspiren": "suspirar",
    "suspires": "suspirar",
    "suspiro": "suspirar",
    "suspiros": "suspiro",
    "suspirábamos": "suspirar",
    "suspiráis": "suspirar",
    "suspiráramos": "suspirar",
    "suspiráremos": "suspirar",
    "suspirásemos": "suspirar",
    "suspiré": "suspirar",
    "suspiréis": "suspirar",
    "suspiró": "suspirar",
    "sustancia": "sustanciar",
    "sustanciaba": "sustanciar",
    "sustanciabais": "sustanciar",
    "sustanciaban": "sustanciar",
    "sustanciabas": "sustanciar",
    "sustanciaciones": "sustanciación",
    "sustanciad": "sustanciar",
    "sustanciada": "sustanciar",
    "sustanciadas": "sustanciar",
    "sustanciado": "sustanciar",
    "sustanciados": "sustanciar",
    "sustanciales": "sustancial",
    "sustanciamos": "sustanciar",
    "sustancian": "sustanciar",
    "sustanciando": "sustanciar",
    "sustanciara": "sustanciar",
    "sustanciarais": "sustanciar",
    "sustanciaran": "sustanciar",
    "sustanciaras": "sustanciar",
    "sustanciare": "sustanciar",
    "sustanciareis": "sustanciar",
    "sustanciaremos": "sustanciar",
    "sustanciaren": "sustanciar",
    "sustanciares": "sustanciar",
    "sustanciaron": "sustanciar",
    "sustanciará": "sustanciar",
    "sustanciarán": "sustanciar",
    "sustanciarás": "sustanciar",
    "sustanciaré": "sustanciar",
    "sustanciaréis": "sustanciar",
    "sustanciaría": "sustanciar",
    "sustanciaríais": "sustanciar",
    "sustanciaríamos": "sustanciar",
    "sustanciarían": "sustanciar",
    "sustanciarías": "sustanciar",
    "sustancias": "sustanciar",
    "sustanciase": "sustanciar",
    "sustanciaseis": "sustanciar",
    "sustanciasen": "sustanciar",
    "sustanciases": "sustanciar",
    "sustanciaste": "sustanciar",
    "sustanciasteis": "sustanciar",
    "sustancie": "sustanciar",
    "sustanciemos": "sustanciar",
    "sustancien": "sustanciar",
    "sustancies": "sustanciar",
    "sustancio": "sustanciar",
    "sustanciosa": "sustancioso",
    "sustanciosas": "sustancioso",
    "sustanciosos": "sustancioso",
    "sustanciábamos": "sustanciar",
    "sustanciáis": "sustanciar",
    "sustanciáramos": "sustanciar",
    "sustanciáremos": "sustanciar",
    "sustanciásemos": "sustanciar",
    "sustancié": "sustanciar",
    "sustanciéis": "sustanciar",
    "sustanció": "sustanciar",
    "sustantiva": "sustantivo",
    "sustantivaba": "sustantivar",
    "sustantivabais": "sustantivar",
    "sustantivaban": "sustantivar",
    "sustantivabas": "sustantivar",
    "sustantivaciones": "sustantivación",
    "sustantivad": "sustantivar",
    "sustantivada": "sustantivar",
    "sustantivadas": "sustantivar",
    "sustantivado": "sustantivar",
    "sustantivados": "sustantivar",
    "sustantivamos": "sustantivar",
    "sustantivan": "sustantivar",
    "sustantivando": "sustantivar",
    "sustantivara": "sustantivar",
    "sustantivarais": "sustantivar",
    "sustantivaran": "sustantivar",
    "sustantivaras": "sustantivar",
    "sustantivare": "sustantivar",
    "sustantivareis": "sustantivar",
    "sustantivaremos": "sustantivar",
    "sustantivaren": "sustantivar",
    "sustantivares": "sustantivar",
    "sustantivaron": "sustantivar",
    "sustantivará": "sustantivar",
    "sustantivarán": "sustantivar",
    "sustantivarás": "sustantivar",
    "sustantivaré": "sustantivar",
    "sustantivaréis": "sustantivar",
    "sustantivaría": "sustantivar",
    "sustantivaríais": "sustantivar",
    "sustantivaríamos": "sustantivar",
    "sustantivarían": "sustantivar",
    "sustantivarías": "sustantivar",
    "sustantivas": "sustantivo",
    "sustantivase": "sustantivar",
    "sustantivaseis": "sustantivar",
    "sustantivasen": "sustantivar",
    "sustantivases": "sustantivar",
    "sustantivaste": "sustantivar",
    "sustantivasteis": "sustantivar",
    "sustantive": "sustantivar",
    "sustantivemos": "sustantivar",
    "sustantiven": "sustantivar",
    "sustantives": "sustantivar",
    "sustantividades": "sustantividad",
    "sustantivo": "sustantivar",
    "sustantivos": "sustantivo",
    "sustantivábamos": "sustantivar",
    "sustantiváis": "sustantivar",
    "sustantiváramos": "sustantivar",
    "sustantiváremos": "sustantivar",
    "sustantivásemos": "sustantivar",
    "sustantivé": "sustantivar",
    "sustantivéis": "sustantivar",
    "sustantivó": "sustantivar",
    "sustenida": "sustenido",
    "sustenidas": "sustenido",
    "sustenidos": "sustenido",
    "sustenta": "sustentar",
    "sustentaba": "sustentar",
    "sustentabais": "sustentar",
    "sustentaban": "sustentar",
    "sustentabas": "sustentar",
    "sustentabilidades": "sustentabilidad",
    "sustentables": "sustentable",
    "sustentaciones": "sustentación",
    "sustentad": "sustentar",
    "sustentada": "sustentar",
    "sustentadas": "sustentar",
    "sustentado": "sustentar",
    "sustentadora": "sustentador",
    "sustentadoras": "sustentador",
    "sustentadores": "sustentador",
    "sustentados": "sustentar",
    "sustentamientos": "sustentamiento",
    "sustentamos": "sustentar",
    "sustentan": "sustentar",
    "sustentando": "sustentar",
    "sustentantes": "sustentante",
    "sustentara": "sustentar",
    "sustentarais": "sustentar",
    "sustentaran": "sustentar",
    "sustentaras": "sustentar",
    "sustentare": "sustentar",
    "sustentareis": "sustentar",
    "sustentaremos": "sustentar",
    "sustentaren": "sustentar",
    "sustentares": "sustentar",
    "sustentaron": "sustentar",
    "sustentará": "sustentar",
    "sustentarán": "sustentar",
    "sustentarás": "sustentar",
    "sustentaré": "sustentar",
    "sustentaréis": "sustentar",
    "sustentaría": "sustentar",
    "sustentaríais": "sustentar",
    "sustentaríamos": "sustentar",
    "sustentarían": "sustentar",
    "sustentarías": "sustentar",
    "sustentas": "sustentar",
    "sustentase": "sustentar",
    "sustentaseis": "sustentar",
    "sustentasen": "sustentar",
    "sustentases": "sustentar",
    "sustentaste": "sustentar",
    "sustentasteis": "sustentar",
    "sustente": "sustentar",
    "sustentemos": "sustentar",
    "sustenten": "sustentar",
    "sustentes": "sustentar",
    "sustento": "sustentar",
    "sustentos": "sustento",
    "sustentábamos": "sustentar",
    "sustentáculos": "sustentáculo",
    "sustentáis": "sustentar",
    "sustentáramos": "sustentar",
    "sustentáremos": "sustentar",
    "sustentásemos": "sustentar",
    "sustenté": "sustentar",
    "sustentéis": "sustentar",
    "sustentó": "sustentar",
    "sustituciones": "sustitución",
    "sustituibles": "sustituible",
    "sustituid": "sustituir",
    "sustituida": "sustituir",
    "sustituidas": "sustituir",
    "sustituido": "sustituir",
    "sustituidora": "sustituidor",
    "sustituidoras": "sustituidor",
    "sustituidores": "sustituidor",
    "sustituidos": "sustituir",
    "sustituimos": "sustituir",
    "sustituiremos": "sustituir",
    "sustituirá": "sustituir",
    "sustituirán": "sustituir",
    "sustituirás": "sustituir",
    "sustituiré": "sustituir",
    "sustituiréis": "sustituir",
    "sustituiría": "sustituir",
    "sustituiríais": "sustituir",
    "sustituiríamos": "sustituir",
    "sustituirían": "sustituir",
    "sustituirías": "sustituir",
    "sustituiste": "sustituir",
    "sustituisteis": "sustituir",
    "sustituta": "sustituto",
    "sustitutas": "sustituto",
    "sustitutiva": "sustitutivo",
    "sustitutivas": "sustitutivo",
    "sustitutivos": "sustitutivo",
    "sustitutoria": "sustitutorio",
    "sustitutorias": "sustitutorio",
    "sustitutorios": "sustitutorio",
    "sustitutos": "sustituto",
    "sustituya": "sustituir",
    "sustituyamos": "sustituir",
    "sustituyan": "sustituir",
    "sustituyas": "sustituir",
    "sustituye": "sustituir",
    "sustituyen": "sustituir",
    "sustituyendo": "sustituir",
    "sustituyera": "sustituir",
    "sustituyerais": "sustituir",
    "sustituyeran": "sustituir",
    "sustituyeras": "sustituir",
    "sustituyere": "sustituir",
    "sustituyereis": "sustituir",
    "sustituyeren": "sustituir",
    "sustituyeres": "sustituir",
    "sustituyeron": "sustituir",
    "sustituyes": "sustituir",
    "sustituyese": "sustituir",
    "sustituyeseis": "sustituir",
    "sustituyesen": "sustituir",
    "sustituyeses": "sustituir",
    "sustituyo": "sustituir",
    "sustituyáis": "sustituir",
    "sustituyéramos": "sustituir",
    "sustituyéremos": "sustituir",
    "sustituyésemos": "sustituir",
    "sustituyó": "sustituir",
    "sustituí": "sustituir",
    "sustituía": "sustituir",
    "sustituíais": "sustituir",
    "sustituíamos": "sustituir",
    "sustituían": "sustituir",
    "sustituías": "sustituir",
    "sustituís": "sustituir",
    "sustos": "susto",
    "sustracciones": "sustracción",
    "sustractiva": "sustractivo",
    "sustractivas": "sustractivo",
    "sustractivos": "sustractivo",
    "sustractora": "sustractor",
    "sustractoras": "sustractor",
    "sustractores": "sustractor",
    "sustrae": "sustraer",
    "sustraed": "sustraer",
    "sustraemos": "sustraer",
    "sustraen": "sustraer",
    "sustraendos": "sustraendo",
    "sustraeremos": "sustraer",
    "sustraerá": "sustraer",
    "sustraerán": "sustraer",
    "sustraerás": "sustraer",
    "sustraeré": "sustraer",
    "sustraeréis": "sustraer",
    "sustraería": "sustraer",
    "sustraeríais": "sustraer",
    "sustraeríamos": "sustraer",
    "sustraerían": "sustraer",
    "sustraerías": "sustraer",
    "sustraes": "sustraer",
    "sustraiga": "sustraer",
    "sustraigamos": "sustraer",
    "sustraigan": "sustraer",
    "sustraigas": "sustraer",
    "sustraigo": "sustraer",
    "sustraigáis": "sustraer",
    "sustraje": "sustraer",
    "sustrajera": "sustraer",
    "sustrajerais": "sustraer",
    "sustrajeran": "sustraer",
    "sustrajeras": "sustraer",
    "sustrajere": "sustraer",
    "sustrajereis": "sustraer",
    "sustrajeren": "sustraer",
    "sustrajeres": "sustraer",
    "sustrajeron": "sustraer",
    "sustrajese": "sustraer",
    "sustrajeseis": "sustraer",
    "sustrajesen": "sustraer",
    "sustrajeses": "sustraer",
    "sustrajimos": "sustraer",
    "sustrajiste": "sustraer",
    "sustrajisteis": "sustraer",
    "sustrajo": "sustraer",
    "sustrajéramos": "sustraer",
    "sustrajéremos": "sustraer",
    "sustrajésemos": "sustraer",
    "sustratos": "sustrato",
    "sustrayendo": "sustraer",
    "sustraéis": "sustraer",
    "sustraía": "sustraer",
    "sustraíais": "sustraer",
    "sustraíamos": "sustraer",
    "sustraían": "sustraer",
    "sustraías": "sustraer",
    "sustraída": "sustraer",
    "sustraídas": "sustraer",
    "sustraído": "sustraer",
    "sustraídos": "sustraer",
    "susurra": "susurrar",
    "susurraba": "susurrar",
    "susurrabais": "susurrar",
    "susurraban": "susurrar",
    "susurrabas": "susurrar",
    "susurraciones": "susurración",
    "susurrad": "susurrar",
    "susurrada": "susurrar",
    "susurradas": "susurrar",
    "susurrado": "susurrar",
    "susurradora": "susurrador",
    "susurradoras": "susurrador",
    "susurradores": "susurrador",
    "susurrados": "susurrar",
    "susurramos": "susurrar",
    "susurran": "susurrar",
    "susurrando": "susurrar",
    "susurrantes": "susurrante",
    "susurrara": "susurrar",
    "susurrarais": "susurrar",
    "susurraran": "susurrar",
    "susurraras": "susurrar",
    "susurrare": "susurrar",
    "susurrareis": "susurrar",
    "susurraremos": "susurrar",
    "susurraren": "susurrar",
    "susurrares": "susurrar",
    "susurraron": "susurrar",
    "susurrará": "susurrar",
    "susurrarán": "susurrar",
    "susurrarás": "susurrar",
    "susurraré": "susurrar",
    "susurraréis": "susurrar",
    "susurraría": "susurrar",
    "susurraríais": "susurrar",
    "susurraríamos": "susurrar",
    "susurrarían": "susurrar",
    "susurrarías": "susurrar",
    "susurras": "susurrar",
    "susurrase": "susurrar",
    "susurraseis": "susurrar",
    "susurrasen": "susurrar",
    "susurrases": "susurrar",
    "susurraste": "susurrar",
    "susurrasteis": "susurrar",
    "susurre": "susurrar",
    "susurremos": "susurrar",
    "susurren": "susurrar",
    "susurres": "susurrar",
    "susurridos": "susurrido",
    "susurro": "susurrar",
    "susurrona": "susurrón",
    "susurronas": "susurrón",
    "susurrones": "susurrón",
    "susurros": "susurro",
    "susurrábamos": "susurrar",
    "susurráis": "susurrar",
    "susurráramos": "susurrar",
    "susurráremos": "susurrar",
    "susurrásemos": "susurrar",
    "susurré": "susurrar",
    "susurréis": "susurrar",
    "susurró": "susurrar",
    "sutases": "sutás",
    "sutiles": "sutil",
    "sutilezas": "sutileza",
    "sutilice": "sutilizar",
    "sutilicemos": "sutilizar",
    "sutilicen": "sutilizar",
    "sutilices": "sutilizar",
    "sutilicé": "sutilizar",
    "sutilicéis": "sutilizar",
    "sutilidades": "sutilidad",
    "sutiliza": "sutilizar",
    "sutilizaba": "sutilizar",
    "sutilizabais": "sutilizar",
    "sutilizaban": "sutilizar",
    "sutilizabas": "sutilizar",
    "sutilizad": "sutilizar",
    "sutilizada": "sutilizar",
    "sutilizadas": "sutilizar",
    "sutilizado": "sutilizar",
    "sutilizados": "sutilizar",
    "sutilizamos": "sutilizar",
    "sutilizan": "sutilizar",
    "sutilizando": "sutilizar",
    "sutilizara": "sutilizar",
    "sutilizarais": "sutilizar",
    "sutilizaran": "sutilizar",
    "sutilizaras": "sutilizar",
    "sutilizare": "sutilizar",
    "sutilizareis": "sutilizar",
    "sutilizaremos": "sutilizar",
    "sutilizaren": "sutilizar",
    "sutilizares": "sutilizar",
    "sutilizaron": "sutilizar",
    "sutilizará": "sutilizar",
    "sutilizarán": "sutilizar",
    "sutilizarás": "sutilizar",
    "sutilizaré": "sutilizar",
    "sutilizaréis": "sutilizar",
    "sutilizaría": "sutilizar",
    "sutilizaríais": "sutilizar",
    "sutilizaríamos": "sutilizar",
    "sutilizarían": "sutilizar",
    "sutilizarías": "sutilizar",
    "sutilizas": "sutilizar",
    "sutilizase": "sutilizar",
    "sutilizaseis": "sutilizar",
    "sutilizasen": "sutilizar",
    "sutilizases": "sutilizar",
    "sutilizaste": "sutilizar",
    "sutilizasteis": "sutilizar",
    "sutilizo": "sutilizar",
    "sutilizábamos": "sutilizar",
    "sutilizáis": "sutilizar",
    "sutilizáramos": "sutilizar",
    "sutilizáremos": "sutilizar",
    "sutilizásemos": "sutilizar",
    "sutilizó": "sutilizar",
    "sutras": "sutra",
    "sutura": "suturar",
    "suturaba": "suturar",
    "suturabais": "suturar",
    "suturaban": "suturar",
    "suturabas": "suturar",
    "suturad": "suturar",
    "suturada": "suturar",
    "suturadas": "suturar",
    "suturado": "suturar",
    "suturados": "suturar",
    "suturales": "sutural",
    "suturamos": "suturar",
    "suturan": "suturar",
    "suturando": "suturar",
    "suturara": "suturar",
    "suturarais": "suturar",
    "suturaran": "suturar",
    "suturaras": "suturar",
    "suturare": "suturar",
    "suturareis": "suturar",
    "suturaremos": "suturar",
    "suturaren": "suturar",
    "suturares": "suturar",
    "suturaron": "suturar",
    "suturará": "suturar",
    "suturarán": "suturar",
    "suturarás": "suturar",
    "suturaré": "suturar",
    "suturaréis": "suturar",
    "suturaría": "suturar",
    "suturaríais": "suturar",
    "suturaríamos": "suturar",
    "suturarían": "suturar",
    "suturarías": "suturar",
    "suturas": "suturar",
    "suturase": "suturar",
    "suturaseis": "suturar",
    "suturasen": "suturar",
    "suturases": "suturar",
    "suturaste": "suturar",
    "suturasteis": "suturar",
    "suture": "suturar",
    "suturemos": "suturar",
    "suturen": "suturar",
    "sutures": "suturar",
    "suturo": "suturar",
    "suturábamos": "suturar",
    "suturáis": "suturar",
    "suturáramos": "suturar",
    "suturáremos": "suturar",
    "suturásemos": "suturar",
    "suturé": "suturar",
    "suturéis": "suturar",
    "suturó": "suturar",
    "suya": "suyo",
    "suyas": "suyo",
    "suyos": "suyo",
    "suéteres": "suéter",
    "suévica": "suévico",
    "suévicas": "suévico",
    "suévicos": "suévico",
    "svásticas": "svástica",
    "swahelis": "swaheli",
    "swahilis": "swahili",
    "swappings": "swapping",
    "swings": "swing",
    "sábados": "sábado",
    "sábalos": "sábalo",
    "sábanas": "sábana",
    "sábulos": "sábulo",
    "sáculos": "sáculo",
    "sádica": "sádico",
    "sádicas": "sádico",
    "sádicos": "sádico",
    "sáfica": "sáfico",
    "sáficas": "sáfico",
    "sáficos": "sáfico",
    "sálica": "sálico",
    "sálicas": "sálico",
    "sálicos": "sálico",
    "sámagos": "sámago",
    "sámaras": "sámara",
    "sándalos": "sándalo",
    "sánscrita": "sánscrito",
    "sánscritas": "sánscrito",
    "sánscritos": "sánscrito",
    "sápida": "sápido",
    "sápidas": "sápido",
    "sápidos": "sápido",
    "sáprica": "sáprico",
    "sápricas": "sáprico",
    "sápricos": "sáprico",
    "sármatas": "sármata",
    "sátiras": "sátira",
    "sátiros": "sátiro",
    "sátrapas": "sátrapa",
    "sáxea": "sáxeo",
    "sáxeas": "sáxeo",
    "sáxeos": "sáxeo",
    "sèniors": "sènior",
    "sé": "ser",
    "sémenes": "semen",
    "sémolas": "sémola",
    "sénecas": "séneca",
    "séniores": "sénior",
    "séniors": "sénior",
    "sépalos": "sépalo",
    "séptica": "séptico",
    "sépticas": "séptico",
    "sépticos": "séptico",
    "séptima": "séptimo",
    "séptimas": "séptimo",
    "séptimo": "7",
    "séptimos": "séptimo",
    "séptupla": "séptuplo",
    "séptuplas": "séptuplo",
    "séptuples": "séptuple",
    "séptuplos": "séptuplo",
    "séquitos": "séquito",
    "sérica": "sérico",
    "séricas": "sérico",
    "séricos": "sérico",
    "sésamos": "sésamo",
    "sésiles": "sésil",
    "sétima": "7",
    "sétimas": "7",
    "sétimo": "7",
    "sétimos": "7",
    "séviros": "séviro",
    "séxtica": "séxtico",
    "séxticas": "séxtico",
    "séxticos": "séxtico",
    "séxtupla": "séxtuplo",
    "séxtuplas": "séxtuplo",
    "séxtuples": "séxtuple",
    "séxtuplos": "séxtuplo",
    "sícula": "sículo",
    "sículas": "sículo",
    "sículos": "sículo",
    "síes": "sí",
    "sílabas": "sílaba",
    "sílabos": "sílabo",
    "sílfides": "sílfide",
    "sílices": "sílice",
    "símbolos": "símbolo",
    "símica": "símico",
    "símicas": "símico",
    "símicos": "símico",
    "símiles": "símil",
    "símplexes": "símplex",
    "símplices": "símplice",
    "síncitos": "síncito",
    "síncopas": "síncopa",
    "síncopes": "síncope",
    "síncrona": "síncrono",
    "síncronas": "síncrono",
    "síncronos": "síncrono",
    "síndicos": "síndico",
    "síndromes": "síndrome",
    "sínfilos": "sínfilo",
    "sínodos": "sínodo",
    "síntomas": "síntoma",
    "síquica": "síquico",
    "síquicas": "síquico",
    "síquicos": "síquico",
    "sísmica": "sísmico",
    "sísmicas": "sísmico",
    "sísmicos": "sísmico",
    "sístila": "sístilo",
    "sístilas": "sístilo",
    "sístilos": "sístilo",
    "sístoles": "sístole",
    "sódica": "sódico",
    "sódicas": "sódico",
    "sódicos": "sódico",
    "sóforas": "sófora",
    "sólea": "sóleo",
    "sóleas": "sóleo",
    "sóleos": "sóleo",
    "sólida": "sólido",
    "sólidas": "sólido",
    "sólidos": "sólido",
    "sólita": "sólito",
    "sólitas": "sólito",
    "sólitos": "sólito",
    "sómbrica": "sómbrico",
    "sómbricas": "sómbrico",
    "sómbricos": "sómbrico",
    "sónica": "sónico",
    "sónicas": "sónico",
    "sónicos": "sónico",
    "sórbica": "sórbico",
    "sórbicas": "sórbico",
    "sórbicos": "sórbico",
    "sórdida": "sórdido",
    "sórdidas": "sórdido",
    "sórdidos": "sórdido",
    "sótanos": "sótano",
    "súbdita": "súbdito",
    "súbditas": "súbdito",
    "súbditos": "súbdito",
    "súberes": "súber",
    "súbita": "súbito",
    "súbitas": "súbito",
    "súbitos": "súbito",
    "súcuba": "súcubo",
    "súcubas": "súcubo",
    "súcubos": "súcubo",
    "súculas": "súcula",
    "súpera": "súpero",
    "súperas": "súpero",
    "súperes": "súper",
    "súperos": "súpero",
    "súpita": "súpito",
    "súpitas": "súpito",
    "súpitos": "súpito",
    "súplicas": "súplica",
    "súrculos": "súrculo",
    "taaffeitas": "taaffeita",
    "tabacalera": "tabacalero",
    "tabacaleras": "tabacalero",
    "tabacaleros": "tabacalero",
    "tabacales": "tabacal",
    "tabacos": "tabaco",
    "tabacosa": "tabacoso",
    "tabacosas": "tabacoso",
    "tabacosos": "tabacoso",
    "tabaibas": "tabaiba",
    "tabaladas": "tabalada",
    "tabalarios": "tabalario",
    "tabalea": "tabalear",
    "tabaleaba": "tabalear",
    "tabaleabais": "tabalear",
    "tabaleaban": "tabalear",
    "tabaleabas": "tabalear",
    "tabalead": "tabalear",
    "tabaleada": "tabalear",
    "tabaleadas": "tabalear",
    "tabaleado": "tabalear",
    "tabaleados": "tabalear",
    "tabaleamos": "tabalear",
    "tabalean": "tabalear",
    "tabaleando": "tabalear",
    "tabaleara": "tabalear",
    "tabalearais": "tabalear",
    "tabalearan": "tabalear",
    "tabalearas": "tabalear",
    "tabaleare": "tabalear",
    "tabaleareis": "tabalear",
    "tabalearemos": "tabalear",
    "tabalearen": "tabalear",
    "tabaleares": "tabalear",
    "tabalearon": "tabalear",
    "tabaleará": "tabalear",
    "tabalearán": "tabalear",
    "tabalearás": "tabalear",
    "tabalearé": "tabalear",
    "tabalearéis": "tabalear",
    "tabalearía": "tabalear",
    "tabalearíais": "tabalear",
    "tabalearíamos": "tabalear",
    "tabalearían": "tabalear",
    "tabalearías": "tabalear",
    "tabaleas": "tabalear",
    "tabalease": "tabalear",
    "tabaleaseis": "tabalear",
    "tabaleasen": "tabalear",
    "tabaleases": "tabalear",
    "tabaleaste": "tabalear",
    "tabaleasteis": "tabalear",
    "tabalee": "tabalear",
    "tabaleemos": "tabalear",
    "tabaleen": "tabalear",
    "tabalees": "tabalear",
    "tabaleo": "tabalear",
    "tabaleos": "tabaleo",
    "tabaleábamos": "tabalear",
    "tabaleáis": "tabalear",
    "tabaleáramos": "tabalear",
    "tabaleáremos": "tabalear",
    "tabaleásemos": "tabalear",
    "tabaleé": "tabalear",
    "tabaleéis": "tabalear",
    "tabaleó": "tabalear",
    "tabanazos": "tabanazo",
    "tabancos": "tabanco",
    "tabaneras": "tabanera",
    "tabanques": "tabanque",
    "tabaolas": "tabaola",
    "tabaquera": "tabaquero",
    "tabaqueras": "tabaquero",
    "tabaqueros": "tabaquero",
    "tabaquerías": "tabaquería",
    "tabaques": "tabaque",
    "tabaquismos": "tabaquismo",
    "tabaquistas": "tabaquista",
    "tabardetes": "tabardete",
    "tabardillos": "tabardillo",
    "tabardos": "tabardo",
    "tabarras": "tabarra",
    "tabas": "taba",
    "tabascos": "tabasco",
    "tabasqueña": "tabasqueño",
    "tabasqueñas": "tabasqueño",
    "tabasqueños": "tabasqueño",
    "tabella": "tabellar",
    "tabellaba": "tabellar",
    "tabellabais": "tabellar",
    "tabellaban": "tabellar",
    "tabellabas": "tabellar",
    "tabellad": "tabellar",
    "tabellada": "tabellar",
    "tabelladas": "tabellar",
    "tabellado": "tabellar",
    "tabellados": "tabellar",
    "tabelladuras": "tabelladura",
    "tabellamos": "tabellar",
    "tabellan": "tabellar",
    "tabellando": "tabellar",
    "tabellara": "tabellar",
    "tabellarais": "tabellar",
    "tabellaran": "tabellar",
    "tabellaras": "tabellar",
    "tabellare": "tabellar",
    "tabellareis": "tabellar",
    "tabellaremos": "tabellar",
    "tabellaren": "tabellar",
    "tabellares": "tabellar",
    "tabellaron": "tabellar",
    "tabellará": "tabellar",
    "tabellarán": "tabellar",
    "tabellarás": "tabellar",
    "tabellaré": "tabellar",
    "tabellaréis": "tabellar",
    "tabellaría": "tabellar",
    "tabellaríais": "tabellar",
    "tabellaríamos": "tabellar",
    "tabellarían": "tabellar",
    "tabellarías": "tabellar",
    "tabellas": "tabellar",
    "tabellase": "tabellar",
    "tabellaseis": "tabellar",
    "tabellasen": "tabellar",
    "tabellases": "tabellar",
    "tabellaste": "tabellar",
    "tabellasteis": "tabellar",
    "tabelle": "tabellar",
    "tabellemos": "tabellar",
    "tabellen": "tabellar",
    "tabelles": "tabellar",
    "tabello": "tabellar",
    "tabellábamos": "tabellar",
    "tabelláis": "tabellar",
    "tabelláramos": "tabellar",
    "tabelláremos": "tabellar",
    "tabellásemos": "tabellar",
    "tabellé": "tabellar",
    "tabelléis": "tabellar",
    "tabelló": "tabellar",
    "tabernaria": "tabernario",
    "tabernarias": "tabernario",
    "tabernarios": "tabernario",
    "tabernas": "taberna",
    "tabernera": "tabernero",
    "taberneras": "tabernero",
    "taberneros": "tabernero",
    "tabernerías": "tabernería",
    "tabernizada": "tabernizado",
    "tabernizadas": "tabernizado",
    "tabernizados": "tabernizado",
    "tabernáculos": "tabernáculo",
    "tabica": "tabicar",
    "tabicaba": "tabicar",
    "tabicabais": "tabicar",
    "tabicaban": "tabicar",
    "tabicabas": "tabicar",
    "tabicad": "tabicar",
    "tabicada": "tabicar",
    "tabicadas": "tabicar",
    "tabicado": "tabicar",
    "tabicados": "tabicar",
    "tabicamientos": "tabicamiento",
    "tabicamos": "tabicar",
    "tabican": "tabicar",
    "tabicando": "tabicar",
    "tabicara": "tabicar",
    "tabicarais": "tabicar",
    "tabicaran": "tabicar",
    "tabicaras": "tabicar",
    "tabicare": "tabicar",
    "tabicareis": "tabicar",
    "tabicaremos": "tabicar",
    "tabicaren": "tabicar",
    "tabicares": "tabicar",
    "tabicaron": "tabicar",
    "tabicará": "tabicar",
    "tabicarán": "tabicar",
    "tabicarás": "tabicar",
    "tabicaré": "tabicar",
    "tabicaréis": "tabicar",
    "tabicaría": "tabicar",
    "tabicaríais": "tabicar",
    "tabicaríamos": "tabicar",
    "tabicarían": "tabicar",
    "tabicarías": "tabicar",
    "tabicas": "tabicar",
    "tabicase": "tabicar",
    "tabicaseis": "tabicar",
    "tabicasen": "tabicar",
    "tabicases": "tabicar",
    "tabicaste": "tabicar",
    "tabicasteis": "tabicar",
    "tabico": "tabicar",
    "tabicábamos": "tabicar",
    "tabicáis": "tabicar",
    "tabicáramos": "tabicar",
    "tabicáremos": "tabicar",
    "tabicásemos": "tabicar",
    "tabicó": "tabicar",
    "tabique": "tabicar",
    "tabiquemos": "tabicar",
    "tabiquen": "tabicar",
    "tabiquera": "tabiquero",
    "tabiqueras": "tabiquero",
    "tabiqueros": "tabiquero",
    "tabiquerías": "tabiquería",
    "tabiques": "tabique",
    "tabiqué": "tabicar",
    "tabiquéis": "tabicar",
    "tablachinas": "tablachina",
    "tablachos": "tablacho",
    "tablados": "tablado",
    "tablajes": "tablaje",
    "tablaos": "tablao",
    "tablares": "tablar",
    "tablas": "tabla",
    "tablazones": "tablazón",
    "tablazos": "tablazo",
    "tablea": "tablear",
    "tableaba": "tablear",
    "tableabais": "tablear",
    "tableaban": "tablear",
    "tableabas": "tablear",
    "tablead": "tablear",
    "tableada": "tablear",
    "tableadas": "tablear",
    "tableado": "tablear",
    "tableados": "tablear",
    "tableamos": "tablear",
    "tablean": "tablear",
    "tableando": "tablear",
    "tableara": "tablear",
    "tablearais": "tablear",
    "tablearan": "tablear",
    "tablearas": "tablear",
    "tableare": "tablear",
    "tableareis": "tablear",
    "tablearemos": "tablear",
    "tablearen": "tablear",
    "tableares": "tablear",
    "tablearon": "tablear",
    "tableará": "tablear",
    "tablearán": "tablear",
    "tablearás": "tablear",
    "tablearé": "tablear",
    "tablearéis": "tablear",
    "tablearía": "tablear",
    "tablearíais": "tablear",
    "tablearíamos": "tablear",
    "tablearían": "tablear",
    "tablearías": "tablear",
    "tableas": "tablear",
    "tablease": "tablear",
    "tableaseis": "tablear",
    "tableasen": "tablear",
    "tableases": "tablear",
    "tableaste": "tablear",
    "tableasteis": "tablear",
    "tablee": "tablear",
    "tableemos": "tablear",
    "tableen": "tablear",
    "tablees": "tablear",
    "tableo": "tablear",
    "tableos": "tableo",
    "tableros": "tablero",
    "tablestacados": "tablestacado",
    "tablestacas": "tablestaca",
    "tabletas": "tableta",
    "tabletea": "tabletear",
    "tableteaba": "tabletear",
    "tableteabais": "tabletear",
    "tableteaban": "tabletear",
    "tableteabas": "tabletear",
    "tabletead": "tabletear",
    "tableteada": "tabletear",
    "tableteadas": "tabletear",
    "tableteado": "tabletear",
    "tableteados": "tabletear",
    "tableteamos": "tabletear",
    "tabletean": "tabletear",
    "tableteando": "tabletear",
    "tableteara": "tabletear",
    "tabletearais": "tabletear",
    "tabletearan": "tabletear",
    "tabletearas": "tabletear",
    "tableteare": "tabletear",
    "tableteareis": "tabletear",
    "tabletearemos": "tabletear",
    "tabletearen": "tabletear",
    "tableteares": "tabletear",
    "tabletearon": "tabletear",
    "tableteará": "tabletear",
    "tabletearán": "tabletear",
    "tabletearás": "tabletear",
    "tabletearé": "tabletear",
    "tabletearéis": "tabletear",
    "tabletearía": "tabletear",
    "tabletearíais": "tabletear",
    "tabletearíamos": "tabletear",
    "tabletearían": "tabletear",
    "tabletearías": "tabletear",
    "tableteas": "tabletear",
    "tabletease": "tabletear",
    "tableteaseis": "tabletear",
    "tableteasen": "tabletear",
    "tableteases": "tabletear",
    "tableteaste": "tabletear",
    "tableteasteis": "tabletear",
    "tabletee": "tabletear",
    "tableteemos": "tabletear",
    "tableteen": "tabletear",
    "tabletees": "tabletear",
    "tableteo": "tabletear",
    "tableteos": "tableteo",
    "tableteábamos": "tabletear",
    "tableteáis": "tabletear",
    "tableteáramos": "tabletear",
    "tableteáremos": "tabletear",
    "tableteásemos": "tabletear",
    "tableteé": "tabletear",
    "tableteéis": "tabletear",
    "tableteó": "tabletear",
    "tableábamos": "tablear",
    "tableáis": "tablear",
    "tableáramos": "tablear",
    "tableáremos": "tablear",
    "tableásemos": "tablear",
    "tableé": "tablear",
    "tableéis": "tablear",
    "tableó": "tablear",
    "tabliers": "tablier",
    "tablillas": "tablilla",
    "tablitas": "tablita",
    "tabloides": "tabloide",
    "tablonajes": "tablonaje",
    "tabloncillos": "tabloncillo",
    "tablones": "tablón",
    "tabores": "tabor",
    "taboritas": "taborita",
    "tabos": "tabo",
    "tabucos": "tabuco",
    "tabula": "tabular",
    "tabulaba": "tabular",
    "tabulabais": "tabular",
    "tabulaban": "tabular",
    "tabulabas": "tabular",
    "tabulaciones": "tabulación",
    "tabulad": "tabular",
    "tabulada": "tabular",
    "tabuladas": "tabular",
    "tabulado": "tabular",
    "tabuladora": "tabulador",
    "tabuladoras": "tabuladora",
    "tabuladores": "tabulador",
    "tabulados": "tabular",
    "tabulamos": "tabular",
    "tabulan": "tabular",
    "tabulando": "tabular",
    "tabulara": "tabular",
    "tabularais": "tabular",
    "tabularan": "tabular",
    "tabularas": "tabular",
    "tabulare": "tabular",
    "tabulareis": "tabular",
    "tabularemos": "tabular",
    "tabularen": "tabular",
    "tabulares": "tabular",
    "tabularon": "tabular",
    "tabulará": "tabular",
    "tabularán": "tabular",
    "tabularás": "tabular",
    "tabularé": "tabular",
    "tabularéis": "tabular",
    "tabularía": "tabular",
    "tabularíais": "tabular",
    "tabularíamos": "tabular",
    "tabularían": "tabular",
    "tabularías": "tabular",
    "tabulas": "tabular",
    "tabulase": "tabular",
    "tabulaseis": "tabular",
    "tabulasen": "tabular",
    "tabulases": "tabular",
    "tabulaste": "tabular",
    "tabulasteis": "tabular",
    "tabule": "tabular",
    "tabulemos": "tabular",
    "tabulen": "tabular",
    "tabules": "tabular",
    "tabulo": "tabular",
    "tabulábamos": "tabular",
    "tabuláis": "tabular",
    "tabuláramos": "tabular",
    "tabuláremos": "tabular",
    "tabulásemos": "tabular",
    "tabulé": "tabular",
    "tabuléis": "tabular",
    "tabuló": "tabular",
    "taburetes": "taburete",
    "tabuses": "tabú",
    "tabáquica": "tabáquico",
    "tabáquicas": "tabáquico",
    "tabáquicos": "tabáquico",
    "tabúes": "tabú",
    "tabús": "tabú",
    "tacadas": "tacada",
    "tacamacas": "tacamaca",
    "tacanas": "tacana",
    "tacanes": "tacán",
    "tacas": "taca",
    "tacatás": "tacatá",
    "tacazos": "tacazo",
    "tacaña": "tacaño",
    "tacañas": "tacaño",
    "tacañea": "tacañear",
    "tacañeaba": "tacañear",
    "tacañeabais": "tacañear",
    "tacañeaban": "tacañear",
    "tacañeabas": "tacañear",
    "tacañead": "tacañear",
    "tacañeada": "tacañear",
    "tacañeadas": "tacañear",
    "tacañeado": "tacañear",
    "tacañeados": "tacañear",
    "tacañeamos": "tacañear",
    "tacañean": "tacañear",
    "tacañeando": "tacañear",
    "tacañeara": "tacañear",
    "tacañearais": "tacañear",
    "tacañearan": "tacañear",
    "tacañearas": "tacañear",
    "tacañeare": "tacañear",
    "tacañeareis": "tacañear",
    "tacañearemos": "tacañear",
    "tacañearen": "tacañear",
    "tacañeares": "tacañear",
    "tacañearon": "tacañear",
    "tacañeará": "tacañear",
    "tacañearán": "tacañear",
    "tacañearás": "tacañear",
    "tacañearé": "tacañear",
    "tacañearéis": "tacañear",
    "tacañearía": "tacañear",
    "tacañearíais": "tacañear",
    "tacañearíamos": "tacañear",
    "tacañearían": "tacañear",
    "tacañearías": "tacañear",
    "tacañeas": "tacañear",
    "tacañease": "tacañear",
    "tacañeaseis": "tacañear",
    "tacañeasen": "tacañear",
    "tacañeases": "tacañear",
    "tacañeaste": "tacañear",
    "tacañeasteis": "tacañear",
    "tacañee": "tacañear",
    "tacañeemos": "tacañear",
    "tacañeen": "tacañear",
    "tacañees": "tacañear",
    "tacañeo": "tacañear",
    "tacañerías": "tacañería",
    "tacañeábamos": "tacañear",
    "tacañeáis": "tacañear",
    "tacañeáramos": "tacañear",
    "tacañeáremos": "tacañear",
    "tacañeásemos": "tacañear",
    "tacañeé": "tacañear",
    "tacañeéis": "tacañear",
    "tacañeó": "tacañear",
    "tacaños": "tacaño",
    "tacetas": "taceta",
    "tacha": "tachar",
    "tachaba": "tachar",
    "tachabais": "tachar",
    "tachaban": "tachar",
    "tachabas": "tachar",
    "tachables": "tachable",
    "tachad": "tachar",
    "tachada": "tachar",
    "tachadas": "tachar",
    "tachado": "tachar",
    "tachadora": "tachador",
    "tachadoras": "tachador",
    "tachadores": "tachador",
    "tachados": "tachar",
    "tachaduras": "tachadura",
    "tachamos": "tachar",
    "tachan": "tachar",
    "tachando": "tachar",
    "tachara": "tachar",
    "tacharais": "tachar",
    "tacharan": "tachar",
    "tacharas": "tachar",
    "tachare": "tachar",
    "tachareis": "tachar",
    "tacharemos": "tachar",
    "tacharen": "tachar",
    "tachares": "tachar",
    "tacharon": "tachar",
    "tachará": "tachar",
    "tacharán": "tachar",
    "tacharás": "tachar",
    "tacharé": "tachar",
    "tacharéis": "tachar",
    "tacharía": "tachar",
    "tacharíais": "tachar",
    "tacharíamos": "tachar",
    "tacharían": "tachar",
    "tacharías": "tachar",
    "tachas": "tachar",
    "tachase": "tachar",
    "tachaseis": "tachar",
    "tachasen": "tachar",
    "tachases": "tachar",
    "tachaste": "tachar",
    "tachasteis": "tachar",
    "tache": "tachar",
    "tachemos": "tachar",
    "tachen": "tachar",
    "taches": "tachar",
    "tacho": "tachar",
    "tachona": "tachonar",
    "tachonaba": "tachonar",
    "tachonabais": "tachonar",
    "tachonaban": "tachonar",
    "tachonabas": "tachonar",
    "tachonad": "tachonar",
    "tachonada": "tachonar",
    "tachonadas": "tachonar",
    "tachonado": "tachonar",
    "tachonados": "tachonar",
    "tachonamos": "tachonar",
    "tachonan": "tachonar",
    "tachonando": "tachonar",
    "tachonara": "tachonar",
    "tachonarais": "tachonar",
    "tachonaran": "tachonar",
    "tachonaras": "tachonar",
    "tachonare": "tachonar",
    "tachonareis": "tachonar",
    "tachonaremos": "tachonar",
    "tachonaren": "tachonar",
    "tachonares": "tachonar",
    "tachonaron": "tachonar",
    "tachonará": "tachonar",
    "tachonarán": "tachonar",
    "tachonarás": "tachonar",
    "tachonaré": "tachonar",
    "tachonaréis": "tachonar",
    "tachonaría": "tachonar",
    "tachonaríais": "tachonar",
    "tachonaríamos": "tachonar",
    "tachonarían": "tachonar",
    "tachonarías": "tachonar",
    "tachonas": "tachonar",
    "tachonase": "tachonar",
    "tachonaseis": "tachonar",
    "tachonasen": "tachonar",
    "tachonases": "tachonar",
    "tachonaste": "tachonar",
    "tachonasteis": "tachonar",
    "tachone": "tachonar",
    "tachonemos": "tachonar",
    "tachonen": "tachonar",
    "tachonerías": "tachonería",
    "tachones": "tachonar",
    "tachono": "tachonar",
    "tachonábamos": "tachonar",
    "tachonáis": "tachonar",
    "tachonáramos": "tachonar",
    "tachonáremos": "tachonar",
    "tachonásemos": "tachonar",
    "tachoné": "tachonar",
    "tachonéis": "tachonar",
    "tachonó": "tachonar",
    "tachos": "tacho",
    "tachosa": "tachoso",
    "tachosas": "tachoso",
    "tachosos": "tachoso",
    "tachuelas": "tachuela",
    "tachábamos": "tachar",
    "tacháis": "tachar",
    "tacháramos": "tachar",
    "tacháremos": "tachar",
    "tachásemos": "tachar",
    "taché": "tachar",
    "tachéis": "tachar",
    "tachó": "tachar",
    "taciturna": "taciturno",
    "taciturnas": "taciturno",
    "taciturnidades": "taciturnidad",
    "taciturnos": "taciturno",
    "taclobos": "taclobo",
    "tacnodos": "tacnodo",
    "tacogeneradores": "tacogenerador",
    "tacométrica": "tacométrico",
    "tacométricas": "tacométrico",
    "tacométricos": "tacométrico",
    "taconazos": "taconazo",
    "taconea": "taconear",
    "taconeaba": "taconear",
    "taconeabais": "taconear",
    "taconeaban": "taconear",
    "taconeabas": "taconear",
    "taconead": "taconear",
    "taconeada": "taconear",
    "taconeadas": "taconear",
    "taconeado": "taconear",
    "taconeados": "taconear",
    "taconeamos": "taconear",
    "taconean": "taconear",
    "taconeando": "taconear",
    "taconeara": "taconear",
    "taconearais": "taconear",
    "taconearan": "taconear",
    "taconearas": "taconear",
    "taconeare": "taconear",
    "taconeareis": "taconear",
    "taconearemos": "taconear",
    "taconearen": "taconear",
    "taconeares": "taconear",
    "taconearon": "taconear",
    "taconeará": "taconear",
    "taconearán": "taconear",
    "taconearás": "taconear",
    "taconearé": "taconear",
    "taconearéis": "taconear",
    "taconearía": "taconear",
    "taconearíais": "taconear",
    "taconearíamos": "taconear",
    "taconearían": "taconear",
    "taconearías": "taconear",
    "taconeas": "taconear",
    "taconease": "taconear",
    "taconeaseis": "taconear",
    "taconeasen": "taconear",
    "taconeases": "taconear",
    "taconeaste": "taconear",
    "taconeasteis": "taconear",
    "taconee": "taconear",
    "taconeemos": "taconear",
    "taconeen": "taconear",
    "taconees": "taconear",
    "taconeo": "taconear",
    "taconeos": "taconeo",
    "tacones": "tacón",
    "taconeábamos": "taconear",
    "taconeáis": "taconear",
    "taconeáramos": "taconear",
    "taconeáremos": "taconear",
    "taconeásemos": "taconear",
    "taconeé": "taconear",
    "taconeéis": "taconear",
    "taconeó": "taconear",
    "tacos": "taco",
    "tacticidades": "tacticidad",
    "tactismos": "tactismo",
    "tactos": "tacto",
    "tacógrafos": "tacógrafo",
    "tacómetros": "tacómetro",
    "tadornas": "tadorna",
    "tadziquistana": "tadziquistano",
    "tadziquistanas": "tadziquistano",
    "tadziquistanos": "tadziquistano",
    "taekwondistas": "taekwondista",
    "taekwondos": "taekwondo",
    "tafanarios": "tafanario",
    "tafes": "taf",
    "tafetanes": "tafetán",
    "tafias": "tafia",
    "tafiletea": "tafiletear",
    "tafileteaba": "tafiletear",
    "tafileteabais": "tafiletear",
    "tafileteaban": "tafiletear",
    "tafileteabas": "tafiletear",
    "tafiletead": "tafiletear",
    "tafileteada": "tafiletear",
    "tafileteadas": "tafiletear",
    "tafileteado": "tafiletear",
    "tafileteados": "tafiletear",
    "tafileteamos": "tafiletear",
    "tafiletean": "tafiletear",
    "tafileteando": "tafiletear",
    "tafileteara": "tafiletear",
    "tafiletearais": "tafiletear",
    "tafiletearan": "tafiletear",
    "tafiletearas": "tafiletear",
    "tafileteare": "tafiletear",
    "tafileteareis": "tafiletear",
    "tafiletearemos": "tafiletear",
    "tafiletearen": "tafiletear",
    "tafileteares": "tafiletear",
    "tafiletearon": "tafiletear",
    "tafileteará": "tafiletear",
    "tafiletearán": "tafiletear",
    "tafiletearás": "tafiletear",
    "tafiletearé": "tafiletear",
    "tafiletearéis": "tafiletear",
    "tafiletearía": "tafiletear",
    "tafiletearíais": "tafiletear",
    "tafiletearíamos": "tafiletear",
    "tafiletearían": "tafiletear",
    "tafiletearías": "tafiletear",
    "tafileteas": "tafiletear",
    "tafiletease": "tafiletear",
    "tafileteaseis": "tafiletear",
    "tafileteasen": "tafiletear",
    "tafileteases": "tafiletear",
    "tafileteaste": "tafiletear",
    "tafileteasteis": "tafiletear",
    "tafiletee": "tafiletear",
    "tafileteemos": "tafiletear",
    "tafileteen": "tafiletear",
    "tafiletees": "tafiletear",
    "tafileteo": "tafiletear",
    "tafileterías": "tafiletería",
    "tafiletes": "tafilete",
    "tafileteábamos": "tafiletear",
    "tafileteáis": "tafiletear",
    "tafileteáramos": "tafiletear",
    "tafileteáremos": "tafiletear",
    "tafileteásemos": "tafiletear",
    "tafileteé": "tafiletear",
    "tafileteéis": "tafiletear",
    "tafileteó": "tafiletear",
    "tafonomías": "tafonomía",
    "tafs": "taf",
    "tagala": "tagalo",
    "tagalas": "tagalo",
    "tagalos": "tagalo",
    "tagarina": "tagarino",
    "tagarinas": "tagarino",
    "tagarinos": "tagarino",
    "tahalíes": "tahalí",
    "tahalís": "tahalí",
    "taheña": "taheño",
    "taheñas": "taheño",
    "taheños": "taheño",
    "tahitiana": "tahitiano",
    "tahitianas": "tahitiano",
    "tahitianos": "tahitiano",
    "tahonas": "tahona",
    "tahonera": "tahonero",
    "tahoneras": "tahonero",
    "tahoneros": "tahonero",
    "tahurerías": "tahurería",
    "tahures": "tahúr",
    "tahúra": "tahúr",
    "tahúras": "tahúr",
    "tahúres": "tahúr",
    "taifas": "taifa",
    "taigas": "taiga",
    "tailandesa": "tailandés",
    "tailandesas": "tailandés",
    "tailandeses": "tailandés",
    "taimaba": "taimar",
    "taimabais": "taimar",
    "taimaban": "taimar",
    "taimabas": "taimar",
    "taimad": "taimar",
    "taimada": "taimar",
    "taimadas": "taimar",
    "taimado": "taimar",
    "taimados": "taimar",
    "taimamos": "taimar",
    "taimando": "taimar",
    "taimara": "taimar",
    "taimarais": "taimar",
    "taimaran": "taimar",
    "taimaras": "taimar",
    "taimare": "taimar",
    "taimareis": "taimar",
    "taimaremos": "taimar",
    "taimaren": "taimar",
    "taimares": "taimar",
    "taimaron": "taimar",
    "taimará": "taimar",
    "taimarán": "taimar",
    "taimarás": "taimar",
    "taimaré": "taimar",
    "taimaréis": "taimar",
    "taimaría": "taimar",
    "taimaríais": "taimar",
    "taimaríamos": "taimar",
    "taimarían": "taimar",
    "taimarías": "taimar",
    "taimase": "taimar",
    "taimaseis": "taimar",
    "taimasen": "taimar",
    "taimases": "taimar",
    "taimaste": "taimar",
    "taimasteis": "taimar",
    "taimemos": "taimar",
    "taimerías": "taimería",
    "taimábamos": "taimar",
    "taimáis": "taimar",
    "taimáramos": "taimar",
    "taimáremos": "taimar",
    "taimásemos": "taimar",
    "taimé": "taimar",
    "taiméis": "taimar",
    "taimó": "taimar",
    "tairas": "taira",
    "tais": "tai",
    "taiwanesa": "taiwanés",
    "taiwanesas": "taiwanés",
    "taiwaneses": "taiwanés",
    "taja": "tajar",
    "tajaba": "tajar",
    "tajabais": "tajar",
    "tajaban": "tajar",
    "tajabas": "tajar",
    "tajad": "tajar",
    "tajada": "tajar",
    "tajadas": "tajar",
    "tajaderas": "tajadera",
    "tajaderos": "tajadero",
    "tajadillas": "tajadilla",
    "tajado": "tajar",
    "tajadora": "tajador",
    "tajadoras": "tajadora",
    "tajadores": "tajador",
    "tajados": "tajar",
    "tajaduras": "tajadura",
    "tajamares": "tajamar",
    "tajamientos": "tajamiento",
    "tajamos": "tajar",
    "tajan": "tajar",
    "tajando": "tajar",
    "tajantes": "tajante",
    "tajara": "tajar",
    "tajarais": "tajar",
    "tajaran": "tajar",
    "tajaras": "tajar",
    "tajare": "tajar",
    "tajareis": "tajar",
    "tajaremos": "tajar",
    "tajaren": "tajar",
    "tajares": "tajar",
    "tajaron": "tajar",
    "tajará": "tajar",
    "tajarán": "tajar",
    "tajarás": "tajar",
    "tajaré": "tajar",
    "tajaréis": "tajar",
    "tajaría": "tajar",
    "tajaríais": "tajar",
    "tajaríamos": "tajar",
    "tajarían": "tajar",
    "tajarías": "tajar",
    "tajas": "tajar",
    "tajase": "tajar",
    "tajaseis": "tajar",
    "tajasen": "tajar",
    "tajases": "tajar",
    "tajaste": "tajar",
    "tajasteis": "tajar",
    "taje": "tajar",
    "tajeas": "tajea",
    "tajemos": "tajar",
    "tajen": "tajar",
    "tajes": "tajar",
    "tajo": "tajar",
    "tajones": "tajón",
    "tajos": "tajo",
    "tajuelas": "tajuela",
    "tajuelos": "tajuelo",
    "tajábamos": "tajar",
    "tajáis": "tajar",
    "tajáramos": "tajar",
    "tajáremos": "tajar",
    "tajásemos": "tajar",
    "tajé": "tajar",
    "tajéis": "tajar",
    "tajó": "tajar",
    "takas": "taka",
    "takines": "takín",
    "tala": "talar",
    "talaba": "talar",
    "talabais": "talar",
    "talaban": "talar",
    "talabartera": "talabartero",
    "talabarteras": "talabartero",
    "talabarteros": "talabartero",
    "talabarterías": "talabartería",
    "talabartes": "talabarte",
    "talabas": "talar",
    "talabricenses": "talabricense",
    "talad": "talar",
    "talada": "talar",
    "taladas": "talar",
    "talado": "talar",
    "taladora": "talador",
    "taladoras": "talador",
    "taladores": "talador",
    "talados": "talar",
    "taladra": "taladrar",
    "taladraba": "taladrar",
    "taladrabais": "taladrar",
    "taladraban": "taladrar",
    "taladrabas": "taladrar",
    "taladrad": "taladrar",
    "taladrada": "taladrar",
    "taladradas": "taladrar",
    "taladrado": "taladrar",
    "taladradora": "taladrador",
    "taladradoras": "taladradora",
    "taladradores": "taladrador",
    "taladrados": "taladrar",
    "taladramos": "taladrar",
    "taladran": "taladrar",
    "taladrando": "taladrar",
    "taladrara": "taladrar",
    "taladrarais": "taladrar",
    "taladraran": "taladrar",
    "taladraras": "taladrar",
    "taladrare": "taladrar",
    "taladrareis": "taladrar",
    "taladraremos": "taladrar",
    "taladraren": "taladrar",
    "taladrares": "taladrar",
    "taladraron": "taladrar",
    "taladrará": "taladrar",
    "taladrarán": "taladrar",
    "taladrarás": "taladrar",
    "taladraré": "taladrar",
    "taladraréis": "taladrar",
    "taladraría": "taladrar",
    "taladraríais": "taladrar",
    "taladraríamos": "taladrar",
    "taladrarían": "taladrar",
    "taladrarías": "taladrar",
    "taladras": "taladrar",
    "taladrase": "taladrar",
    "taladraseis": "taladrar",
    "taladrasen": "taladrar",
    "taladrases": "taladrar",
    "taladraste": "taladrar",
    "taladrasteis": "taladrar",
    "taladre": "taladrar",
    "taladremos": "taladrar",
    "taladren": "taladrar",
    "taladres": "taladrar",
    "taladrillas": "taladrilla",
    "taladro": "taladrar",
    "taladros": "taladro",
    "taladrábamos": "taladrar",
    "taladráis": "taladrar",
    "taladráramos": "taladrar",
    "taladráremos": "taladrar",
    "taladrásemos": "taladrar",
    "taladré": "taladrar",
    "taladréis": "taladrar",
    "taladró": "taladrar",
    "talametes": "talamete",
    "talamifloras": "talamiflora",
    "talamoestriada": "talamoestriado",
    "talamoestriadas": "talamoestriado",
    "talamoestriados": "talamoestriado",
    "talamoparietales": "talamoparietal",
    "talamos": "talar",
    "talan": "talar",
    "talando": "talar",
    "talanes": "talán",
    "talanqueras": "talanquera",
    "talantes": "talante",
    "talantosa": "talantoso",
    "talantosas": "talantoso",
    "talantosos": "talantoso",
    "talapoines": "talapoin",
    "talara": "talar",
    "talarais": "talar",
    "talaran": "talar",
    "talaras": "talar",
    "talare": "talar",
    "talareis": "talar",
    "talaremos": "talar",
    "talaren": "talar",
    "talares": "talar",
    "talaron": "talar",
    "talará": "talar",
    "talarán": "talar",
    "talarás": "talar",
    "talaré": "talar",
    "talaréis": "talar",
    "talaría": "talar",
    "talaríais": "talar",
    "talaríamos": "talar",
    "talarían": "talar",
    "talarías": "talar",
    "talas": "talar",
    "talase": "talar",
    "talaseis": "talar",
    "talasemias": "talasemia",
    "talasen": "talar",
    "talases": "talar",
    "talasocracias": "talasocracia",
    "talasoterapias": "talasoterapia",
    "talaste": "talar",
    "talasteis": "talar",
    "talatos": "talato",
    "talayotes": "talayote",
    "talcos": "talco",
    "talcosa": "talcoso",
    "talcosas": "talcoso",
    "talcosos": "talcoso",
    "talcualilla": "talcualillo",
    "talcualillas": "talcualillo",
    "talcualillos": "talcualillo",
    "tale": "talar",
    "taledes": "taled",
    "talegadas": "talegada",
    "talegas": "talega",
    "talegazos": "talegazo",
    "talegos": "talego",
    "taleguillas": "taleguilla",
    "talemos": "talar",
    "talen": "talar",
    "talentos": "talento",
    "talentosa": "talentoso",
    "talentosas": "talentoso",
    "talentosos": "talentoso",
    "talentuda": "talentudo",
    "talentudas": "talentudo",
    "talentudos": "talentudo",
    "tales": "talar",
    "talgos": "talgo",
    "talinas": "talina",
    "taliones": "talión",
    "talios": "talio",
    "talismanes": "talismán",
    "talismánica": "talismánico",
    "talismánicas": "talismánico",
    "talismánicos": "talismánico",
    "talla": "tallar",
    "tallaba": "tallar",
    "tallabais": "tallar",
    "tallaban": "tallar",
    "tallabas": "tallar",
    "tallad": "tallar",
    "tallada": "tallar",
    "talladas": "tallar",
    "tallado": "tallar",
    "talladoras": "talladora",
    "talladores": "tallador",
    "tallados": "tallar",
    "talladuras": "talladura",
    "tallajes": "tallaje",
    "tallamos": "tallar",
    "tallan": "tallar",
    "tallando": "tallar",
    "tallara": "tallar",
    "tallarais": "tallar",
    "tallaran": "tallar",
    "tallaras": "tallar",
    "tallare": "tallar",
    "tallareis": "tallar",
    "tallaremos": "tallar",
    "tallaren": "tallar",
    "tallares": "tallar",
    "tallarines": "tallarín",
    "tallarolas": "tallarola",
    "tallaron": "tallar",
    "tallará": "tallar",
    "tallarán": "tallar",
    "tallarás": "tallar",
    "tallaré": "tallar",
    "tallaréis": "tallar",
    "tallaría": "tallar",
    "tallaríais": "tallar",
    "tallaríamos": "tallar",
    "tallarían": "tallar",
    "tallarías": "tallar",
    "tallas": "tallar",
    "tallase": "tallar",
    "tallaseis": "tallar",
    "tallasen": "tallar",
    "tallases": "tallar",
    "tallaste": "tallar",
    "tallasteis": "tallar",
    "talle": "tallar",
    "tallece": "tallecer",
    "talleced": "tallecer",
    "tallecemos": "tallecer",
    "tallecen": "tallecer",
    "talleceremos": "tallecer",
    "tallecerá": "tallecer",
    "tallecerán": "tallecer",
    "tallecerás": "tallecer",
    "talleceré": "tallecer",
    "talleceréis": "tallecer",
    "tallecería": "tallecer",
    "talleceríais": "tallecer",
    "talleceríamos": "tallecer",
    "tallecerían": "tallecer",
    "tallecerías": "tallecer",
    "talleces": "tallecer",
    "tallecida": "tallecer",
    "tallecidas": "tallecer",
    "tallecido": "tallecer",
    "tallecidos": "tallecer",
    "talleciendo": "tallecer",
    "talleciera": "tallecer",
    "tallecierais": "tallecer",
    "tallecieran": "tallecer",
    "tallecieras": "tallecer",
    "talleciere": "tallecer",
    "talleciereis": "tallecer",
    "tallecieren": "tallecer",
    "tallecieres": "tallecer",
    "tallecieron": "tallecer",
    "talleciese": "tallecer",
    "tallecieseis": "tallecer",
    "talleciesen": "tallecer",
    "tallecieses": "tallecer",
    "tallecimos": "tallecer",
    "talleciste": "tallecer",
    "tallecisteis": "tallecer",
    "talleciéramos": "tallecer",
    "talleciéremos": "tallecer",
    "talleciésemos": "tallecer",
    "talleció": "tallecer",
    "tallecéis": "tallecer",
    "tallecí": "tallecer",
    "tallecía": "tallecer",
    "tallecíais": "tallecer",
    "tallecíamos": "tallecer",
    "tallecían": "tallecer",
    "tallecías": "tallecer",
    "tallemos": "tallar",
    "tallen": "tallar",
    "talleres": "taller",
    "tallerinas": "tallerina",
    "talles": "talle",
    "tallezca": "tallecer",
    "tallezcamos": "tallecer",
    "tallezcan": "tallecer",
    "tallezcas": "tallecer",
    "tallezco": "tallecer",
    "tallezcáis": "tallecer",
    "tallistas": "tallista",
    "tallo": "tallar",
    "tallos": "tallo",
    "talluda": "talludo",
    "talludas": "talludo",
    "talludos": "talludo",
    "tallábamos": "tallar",
    "talláis": "tallar",
    "talláramos": "tallar",
    "talláremos": "tallar",
    "tallásemos": "tallar",
    "tallé": "tallar",
    "talléis": "tallar",
    "talló": "tallar",
    "talmas": "talma",
    "talmudistas": "talmudista",
    "talmúdica": "talmúdico",
    "talmúdicas": "talmúdico",
    "talmúdicos": "talmúdico",
    "talo": "talar",
    "talocrurales": "talocrural",
    "talofita": "talófito",
    "talofitas": "talófito",
    "talofitos": "talófito",
    "talonadas": "talonada",
    "talonadora": "talonador",
    "talonadoras": "talonador",
    "talonadores": "talonador",
    "talonarios": "talonario",
    "talonazos": "talonazo",
    "talonea": "talonear",
    "taloneaba": "talonear",
    "taloneabais": "talonear",
    "taloneaban": "talonear",
    "taloneabas": "talonear",
    "talonead": "talonear",
    "taloneada": "talonear",
    "taloneadas": "talonear",
    "taloneado": "talonear",
    "taloneados": "talonear",
    "taloneamos": "talonear",
    "talonean": "talonear",
    "taloneando": "talonear",
    "taloneara": "talonear",
    "talonearais": "talonear",
    "talonearan": "talonear",
    "talonearas": "talonear",
    "taloneare": "talonear",
    "taloneareis": "talonear",
    "talonearemos": "talonear",
    "talonearen": "talonear",
    "taloneares": "talonear",
    "talonearon": "talonear",
    "taloneará": "talonear",
    "talonearán": "talonear",
    "talonearás": "talonear",
    "talonearé": "talonear",
    "talonearéis": "talonear",
    "talonearía": "talonear",
    "talonearíais": "talonear",
    "talonearíamos": "talonear",
    "talonearían": "talonear",
    "talonearías": "talonear",
    "taloneas": "talonear",
    "talonease": "talonear",
    "taloneaseis": "talonear",
    "taloneasen": "talonear",
    "taloneases": "talonear",
    "taloneaste": "talonear",
    "taloneasteis": "talonear",
    "talonee": "talonear",
    "taloneemos": "talonear",
    "taloneen": "talonear",
    "talonees": "talonear",
    "taloneo": "talonear",
    "taloneras": "talonera",
    "talones": "talón",
    "talonesca": "talonesco",
    "talonescas": "talonesco",
    "talonescos": "talonesco",
    "taloneábamos": "talonear",
    "taloneáis": "talonear",
    "taloneáramos": "talonear",
    "taloneáremos": "talonear",
    "taloneásemos": "talonear",
    "taloneé": "talonear",
    "taloneéis": "talonear",
    "taloneó": "talonear",
    "talos": "talo",
    "talparias": "talparia",
    "talpas": "talpa",
    "talqueras": "talquera",
    "talques": "talque",
    "talquitas": "talquita",
    "taltuzas": "taltuza",
    "taludes": "talud",
    "talvinas": "talvina",
    "talábamos": "talar",
    "taláis": "talar",
    "talámica": "talámico",
    "talámicas": "talámico",
    "talámicos": "talámico",
    "taláramos": "talar",
    "taláremos": "talar",
    "talásemos": "talar",
    "talé": "talar",
    "taléis": "talar",
    "taló": "talar",
    "talófitos": "talófito",
    "tamagases": "tamagás",
    "tamalera": "tamalero",
    "tamaleras": "tamalero",
    "tamaleros": "tamalero",
    "tamalerías": "tamalería",
    "tamales": "tamal",
    "tamanaca": "tamanaco",
    "tamanacas": "tamanaco",
    "tamanacos": "tamanaco",
    "tamanduaes": "tamanduá",
    "tamanduás": "tamanduá",
    "tamaricácea": "tamaricáceo",
    "tamaricáceas": "tamaricáceo",
    "tamaricáceos": "tamaricáceo",
    "tamarigales": "tamarigal",
    "tamarillas": "tamarilla",
    "tamarindos": "tamarindo",
    "tamarinos": "tamarino",
    "tamariscos": "tamarisco",
    "tamarrizquita": "tamarrizquito",
    "tamarrizquitas": "tamarrizquito",
    "tamarrizquitos": "tamarrizquito",
    "tamarrusquita": "tamarrusquito",
    "tamarrusquitas": "tamarrusquito",
    "tamarrusquitos": "tamarrusquito",
    "tamazules": "tamazul",
    "tamaña": "tamaño",
    "tamañas": "tamaño",
    "tamañita": "tamañito",
    "tamañitas": "tamañito",
    "tamañitos": "tamañito",
    "tamaños": "tamaño",
    "tambalea": "tambalear",
    "tambaleaba": "tambalear",
    "tambaleabais": "tambalear",
    "tambaleaban": "tambalear",
    "tambaleabas": "tambalear",
    "tambalead": "tambalear",
    "tambaleada": "tambalear",
    "tambaleadas": "tambalear",
    "tambaleado": "tambalear",
    "tambaleados": "tambalear",
    "tambaleamos": "tambalear",
    "tambalean": "tambalear",
    "tambaleando": "tambalear",
    "tambaleantes": "tambaleante",
    "tambaleara": "tambalear",
    "tambalearais": "tambalear",
    "tambalearan": "tambalear",
    "tambalearas": "tambalear",
    "tambaleare": "tambalear",
    "tambaleareis": "tambalear",
    "tambalearemos": "tambalear",
    "tambalearen": "tambalear",
    "tambaleares": "tambalear",
    "tambalearon": "tambalear",
    "tambaleará": "tambalear",
    "tambalearán": "tambalear",
    "tambalearás": "tambalear",
    "tambalearé": "tambalear",
    "tambalearéis": "tambalear",
    "tambalearía": "tambalear",
    "tambalearíais": "tambalear",
    "tambalearíamos": "tambalear",
    "tambalearían": "tambalear",
    "tambalearías": "tambalear",
    "tambaleas": "tambalear",
    "tambalease": "tambalear",
    "tambaleaseis": "tambalear",
    "tambaleasen": "tambalear",
    "tambaleases": "tambalear",
    "tambaleaste": "tambalear",
    "tambaleasteis": "tambalear",
    "tambalee": "tambalear",
    "tambaleemos": "tambalear",
    "tambaleen": "tambalear",
    "tambalees": "tambalear",
    "tambaleo": "tambalear",
    "tambaleos": "tambaleo",
    "tambaleábamos": "tambalear",
    "tambaleáis": "tambalear",
    "tambaleáramos": "tambalear",
    "tambaleáremos": "tambalear",
    "tambaleásemos": "tambalear",
    "tambaleé": "tambalear",
    "tambaleéis": "tambalear",
    "tambaleó": "tambalear",
    "tambanillos": "tambanillo",
    "tambarillas": "tambarilla",
    "tambarillos": "tambarillo",
    "tambera": "tambero",
    "tamberas": "tambero",
    "tamberos": "tambero",
    "tamboras": "tambora",
    "tamborea": "tamborear",
    "tamboreaba": "tamborear",
    "tamboreabais": "tamborear",
    "tamboreaban": "tamborear",
    "tamboreabas": "tamborear",
    "tamboread": "tamborear",
    "tamboreada": "tamborear",
    "tamboreadas": "tamborear",
    "tamboreado": "tamborear",
    "tamboreados": "tamborear",
    "tamboreamos": "tamborear",
    "tamborean": "tamborear",
    "tamboreando": "tamborear",
    "tamboreara": "tamborear",
    "tamborearais": "tamborear",
    "tamborearan": "tamborear",
    "tamborearas": "tamborear",
    "tamboreare": "tamborear",
    "tamboreareis": "tamborear",
    "tamborearemos": "tamborear",
    "tamborearen": "tamborear",
    "tamboreares": "tamborear",
    "tamborearon": "tamborear",
    "tamboreará": "tamborear",
    "tamborearán": "tamborear",
    "tamborearás": "tamborear",
    "tamborearé": "tamborear",
    "tamborearéis": "tamborear",
    "tamborearía": "tamborear",
    "tamborearíais": "tamborear",
    "tamborearíamos": "tamborear",
    "tamborearían": "tamborear",
    "tamborearías": "tamborear",
    "tamboreas": "tamborear",
    "tamborease": "tamborear",
    "tamboreaseis": "tamborear",
    "tamboreasen": "tamborear",
    "tamboreases": "tamborear",
    "tamboreaste": "tamborear",
    "tamboreasteis": "tamborear",
    "tamboree": "tamborear",
    "tamboreemos": "tamborear",
    "tamboreen": "tamborear",
    "tamborees": "tamborear",
    "tamboreo": "tamborear",
    "tamboreos": "tamboreo",
    "tambores": "tambor",
    "tamboretes": "tamborete",
    "tamboreábamos": "tamborear",
    "tamboreáis": "tamborear",
    "tamboreáramos": "tamborear",
    "tamboreáremos": "tamborear",
    "tamboreásemos": "tamborear",
    "tamboreé": "tamborear",
    "tamboreéis": "tamborear",
    "tamboreó": "tamborear",
    "tamboriladas": "tamborilada",
    "tamborilazos": "tamborilazo",
    "tamborilea": "tamborilear",
    "tamborileaba": "tamborilear",
    "tamborileabais": "tamborilear",
    "tamborileaban": "tamborilear",
    "tamborileabas": "tamborilear",
    "tamborilead": "tamborilear",
    "tamborileada": "tamborilear",
    "tamborileadas": "tamborilear",
    "tamborileado": "tamborilear",
    "tamborileados": "tamborilear",
    "tamborileamos": "tamborilear",
    "tamborilean": "tamborilear",
    "tamborileando": "tamborilear",
    "tamborileara": "tamborilear",
    "tamborilearais": "tamborilear",
    "tamborilearan": "tamborilear",
    "tamborilearas": "tamborilear",
    "tamborileare": "tamborilear",
    "tamborileareis": "tamborilear",
    "tamborilearemos": "tamborilear",
    "tamborilearen": "tamborilear",
    "tamborileares": "tamborilear",
    "tamborilearon": "tamborilear",
    "tamborileará": "tamborilear",
    "tamborilearán": "tamborilear",
    "tamborilearás": "tamborilear",
    "tamborilearé": "tamborilear",
    "tamborilearéis": "tamborilear",
    "tamborilearía": "tamborilear",
    "tamborilearíais": "tamborilear",
    "tamborilearíamos": "tamborilear",
    "tamborilearían": "tamborilear",
    "tamborilearías": "tamborilear",
    "tamborileas": "tamborilear",
    "tamborilease": "tamborilear",
    "tamborileaseis": "tamborilear",
    "tamborileasen": "tamborilear",
    "tamborileases": "tamborilear",
    "tamborileaste": "tamborilear",
    "tamborileasteis": "tamborilear",
    "tamborilee": "tamborilear",
    "tamborileemos": "tamborilear",
    "tamborileen": "tamborilear",
    "tamborilees": "tamborilear",
    "tamborileo": "tamborilear",
    "tamborilera": "tamborilero",
    "tamborileras": "tamborilero",
    "tamborileros": "tamborilero",
    "tamboriles": "tamboril",
    "tamboriletes": "tamborilete",
    "tamborileábamos": "tamborilear",
    "tamborileáis": "tamborilear",
    "tamborileáramos": "tamborilear",
    "tamborileáremos": "tamborilear",
    "tamborileásemos": "tamborilear",
    "tamborileé": "tamborilear",
    "tamborileéis": "tamborilear",
    "tamborileó": "tamborilear",
    "tamborines": "tamborín",
    "tamboritera": "tamboritero",
    "tamboriteras": "tamboritero",
    "tamboriteros": "tamboritero",
    "tambos": "tambo",
    "tamice": "tamizar",
    "tamicemos": "tamizar",
    "tamicen": "tamizar",
    "tamices": "tamizar",
    "tamicé": "tamizar",
    "tamicéis": "tamizar",
    "tamiles": "tamil",
    "tamiza": "tamizar",
    "tamizaba": "tamizar",
    "tamizabais": "tamizar",
    "tamizaban": "tamizar",
    "tamizabas": "tamizar",
    "tamizaciones": "tamización",
    "tamizad": "tamizar",
    "tamizada": "tamizar",
    "tamizadas": "tamizar",
    "tamizado": "tamizar",
    "tamizadora": "tamizador",
    "tamizadoras": "tamizador",
    "tamizadores": "tamizador",
    "tamizados": "tamizar",
    "tamizamos": "tamizar",
    "tamizan": "tamizar",
    "tamizando": "tamizar",
    "tamizara": "tamizar",
    "tamizarais": "tamizar",
    "tamizaran": "tamizar",
    "tamizaras": "tamizar",
    "tamizare": "tamizar",
    "tamizareis": "tamizar",
    "tamizaremos": "tamizar",
    "tamizaren": "tamizar",
    "tamizares": "tamizar",
    "tamizaron": "tamizar",
    "tamizará": "tamizar",
    "tamizarán": "tamizar",
    "tamizarás": "tamizar",
    "tamizaré": "tamizar",
    "tamizaréis": "tamizar",
    "tamizaría": "tamizar",
    "tamizaríais": "tamizar",
    "tamizaríamos": "tamizar",
    "tamizarían": "tamizar",
    "tamizarías": "tamizar",
    "tamizas": "tamizar",
    "tamizase": "tamizar",
    "tamizaseis": "tamizar",
    "tamizasen": "tamizar",
    "tamizases": "tamizar",
    "tamizaste": "tamizar",
    "tamizasteis": "tamizar",
    "tamizo": "tamizar",
    "tamizábamos": "tamizar",
    "tamizáis": "tamizar",
    "tamizáramos": "tamizar",
    "tamizáremos": "tamizar",
    "tamizásemos": "tamizar",
    "tamizó": "tamizar",
    "tamoataes": "tamoatá",
    "tamoatás": "tamoatá",
    "tamos": "tamo",
    "tamponajes": "tamponaje",
    "tampones": "tampón",
    "tamtames": "tamtam",
    "tamtams": "tamtam",
    "tanagras": "tanagra",
    "tanatorios": "tanatorio",
    "tancrinas": "tancrina",
    "tandas": "tanda",
    "tandeos": "tandeo",
    "tanganillos": "tanganillo",
    "tangas": "tanga",
    "tangenciales": "tangencial",
    "tangencias": "tangencia",
    "tangentes": "tangente",
    "tangerina": "tangerino",
    "tangerinas": "tangerino",
    "tangerinos": "tangerino",
    "tangibilidades": "tangibilidad",
    "tangibles": "tangible",
    "tangos": "tango",
    "tanguera": "tanguero",
    "tangueras": "tanguero",
    "tangueros": "tanguero",
    "tanguistas": "tanguista",
    "tangónes": "tangón",
    "taninos": "tanino",
    "tankas": "tanka",
    "tanora": "tanor",
    "tanoras": "tanor",
    "tanores": "tanor",
    "tanorías": "tanoría",
    "tanques": "tanque",
    "tanquetas": "tanqueta",
    "tanta": "tanto",
    "tantalios": "tantalio",
    "tantalitas": "tantalita",
    "tantanes": "tantán",
    "tantaranes": "tantarán",
    "tantarantanes": "tantarantán",
    "tantas": "tanto",
    "tantea": "tantear",
    "tanteaba": "tantear",
    "tanteabais": "tantear",
    "tanteaban": "tantear",
    "tanteabas": "tantear",
    "tantead": "tantear",
    "tanteada": "tantear",
    "tanteadas": "tantear",
    "tanteado": "tantear",
    "tanteadora": "tanteador",
    "tanteadoras": "tanteador",
    "tanteadores": "tanteador",
    "tanteados": "tantear",
    "tanteamos": "tantear",
    "tantean": "tantear",
    "tanteando": "tantear",
    "tanteara": "tantear",
    "tantearais": "tantear",
    "tantearan": "tantear",
    "tantearas": "tantear",
    "tanteare": "tantear",
    "tanteareis": "tantear",
    "tantearemos": "tantear",
    "tantearen": "tantear",
    "tanteares": "tantear",
    "tantearon": "tantear",
    "tanteará": "tantear",
    "tantearán": "tantear",
    "tantearás": "tantear",
    "tantearé": "tantear",
    "tantearéis": "tantear",
    "tantearía": "tantear",
    "tantearíais": "tantear",
    "tantearíamos": "tantear",
    "tantearían": "tantear",
    "tantearías": "tantear",
    "tanteas": "tantear",
    "tantease": "tantear",
    "tanteaseis": "tantear",
    "tanteasen": "tantear",
    "tanteases": "tantear",
    "tanteaste": "tantear",
    "tanteasteis": "tantear",
    "tantee": "tantear",
    "tanteemos": "tantear",
    "tanteen": "tantear",
    "tantees": "tantear",
    "tanteo": "tantear",
    "tanteos": "tanteo",
    "tanteábamos": "tantear",
    "tanteáis": "tantear",
    "tanteáramos": "tantear",
    "tanteáremos": "tantear",
    "tanteásemos": "tantear",
    "tanteé": "tantear",
    "tanteéis": "tantear",
    "tanteó": "tantear",
    "tantica": "tantico",
    "tanticas": "tantico",
    "tanticos": "tantico",
    "tantos": "tanto",
    "tantras": "tantra",
    "tantrismos": "tantrismo",
    "tanzana": "tanzano",
    "tanzanas": "tanzano",
    "tanzanitas": "tanzanita",
    "tanzanos": "tanzano",
    "taos": "tao",
    "taoísmos": "taoísmo",
    "taoístas": "taoísta",
    "tapa": "tapar",
    "tapaba": "tapar",
    "tapabais": "tapar",
    "tapaban": "tapar",
    "tapabas": "tapar",
    "tapabocas": "tapaboca",
    "tapacaminos": "tapacamino",
    "tapacetes": "tapacete",
    "tapaculos": "tapaculo",
    "tapad": "tapar",
    "tapada": "tapar",
    "tapadas": "tapar",
    "tapaderas": "tapadera",
    "tapado": "tapar",
    "tapadora": "tapador",
    "tapadoras": "tapador",
    "tapadores": "tapador",
    "tapados": "tapar",
    "tapaduras": "tapadura",
    "tapafundas": "tapafunda",
    "tapajuntas": "tapajunta",
    "tapamientos": "tapamiento",
    "tapamos": "tapar",
    "tapan": "tapar",
    "tapando": "tapar",
    "tapara": "tapar",
    "taparais": "tapar",
    "taparan": "tapar",
    "taparas": "tapar",
    "tapare": "tapar",
    "tapareis": "tapar",
    "taparemos": "tapar",
    "taparen": "tapar",
    "tapares": "tapar",
    "taparon": "tapar",
    "tapará": "tapar",
    "taparán": "tapar",
    "taparás": "tapar",
    "taparé": "tapar",
    "taparéis": "tapar",
    "taparía": "tapar",
    "taparíais": "tapar",
    "taparíamos": "tapar",
    "taparían": "tapar",
    "taparías": "tapar",
    "tapas": "tapar",
    "tapase": "tapar",
    "tapaseis": "tapar",
    "tapasen": "tapar",
    "tapases": "tapar",
    "tapaste": "tapar",
    "tapasteis": "tapar",
    "tape": "tapar",
    "tapemos": "tapar",
    "tapen": "tapar",
    "taperibaes": "taperibá",
    "taperibás": "taperibá",
    "taperujos": "taperujo",
    "tapes": "tapar",
    "tapetes": "tapete",
    "tapetums": "tapetum",
    "tapia": "tapiar",
    "tapiaba": "tapiar",
    "tapiabais": "tapiar",
    "tapiaban": "tapiar",
    "tapiabas": "tapiar",
    "tapiad": "tapiar",
    "tapiada": "tapiar",
    "tapiadas": "tapiar",
    "tapiado": "tapiar",
    "tapiados": "tapiar",
    "tapiales": "tapial",
    "tapiamos": "tapiar",
    "tapian": "tapiar",
    "tapiando": "tapiar",
    "tapiara": "tapiar",
    "tapiarais": "tapiar",
    "tapiaran": "tapiar",
    "tapiaras": "tapiar",
    "tapiare": "tapiar",
    "tapiareis": "tapiar",
    "tapiaremos": "tapiar",
    "tapiaren": "tapiar",
    "tapiares": "tapiar",
    "tapiaron": "tapiar",
    "tapiará": "tapiar",
    "tapiarán": "tapiar",
    "tapiarás": "tapiar",
    "tapiaré": "tapiar",
    "tapiaréis": "tapiar",
    "tapiaría": "tapiar",
    "tapiaríais": "tapiar",
    "tapiaríamos": "tapiar",
    "tapiarían": "tapiar",
    "tapiarías": "tapiar",
    "tapias": "tapiar",
    "tapiase": "tapiar",
    "tapiaseis": "tapiar",
    "tapiasen": "tapiar",
    "tapiases": "tapiar",
    "tapiaste": "tapiar",
    "tapiasteis": "tapiar",
    "tapice": "tapizar",
    "tapicemos": "tapizar",
    "tapicen": "tapizar",
    "tapicera": "tapicero",
    "tapiceras": "tapicero",
    "tapiceros": "tapicero",
    "tapicerías": "tapicería",
    "tapices": "tapizar",
    "tapicé": "tapizar",
    "tapicéis": "tapizar",
    "tapida": "tapido",
    "tapidas": "tapido",
    "tapidos": "tapido",
    "tapie": "tapiar",
    "tapiemos": "tapiar",
    "tapien": "tapiar",
    "tapierías": "tapiería",
    "tapies": "tapiar",
    "tapines": "tapín",
    "tapinocránea": "tapinocráneo",
    "tapinocráneas": "tapinocráneo",
    "tapinocráneos": "tapinocráneo",
    "tapio": "tapiar",
    "tapiocas": "tapioca",
    "tapires": "tapir",
    "tapirujos": "tapirujo",
    "tapiza": "tapizar",
    "tapizaba": "tapizar",
    "tapizabais": "tapizar",
    "tapizaban": "tapizar",
    "tapizabas": "tapizar",
    "tapizad": "tapizar",
    "tapizada": "tapizar",
    "tapizadas": "tapizar",
    "tapizado": "tapizar",
    "tapizados": "tapizar",
    "tapizamos": "tapizar",
    "tapizan": "tapizar",
    "tapizando": "tapizar",
    "tapizantes": "tapizante",
    "tapizara": "tapizar",
    "tapizarais": "tapizar",
    "tapizaran": "tapizar",
    "tapizaras": "tapizar",
    "tapizare": "tapizar",
    "tapizareis": "tapizar",
    "tapizaremos": "tapizar",
    "tapizaren": "tapizar",
    "tapizares": "tapizar",
    "tapizaron": "tapizar",
    "tapizará": "tapizar",
    "tapizarán": "tapizar",
    "tapizarás": "tapizar",
    "tapizaré": "tapizar",
    "tapizaréis": "tapizar",
    "tapizaría": "tapizar",
    "tapizaríais": "tapizar",
    "tapizaríamos": "tapizar",
    "tapizarían": "tapizar",
    "tapizarías": "tapizar",
    "tapizas": "tapizar",
    "tapizase": "tapizar",
    "tapizaseis": "tapizar",
    "tapizasen": "tapizar",
    "tapizases": "tapizar",
    "tapizaste": "tapizar",
    "tapizasteis": "tapizar",
    "tapizo": "tapizar",
    "tapizábamos": "tapizar",
    "tapizáis": "tapizar",
    "tapizáramos": "tapizar",
    "tapizáremos": "tapizar",
    "tapizásemos": "tapizar",
    "tapizó": "tapizar",
    "tapiábamos": "tapiar",
    "tapiáis": "tapiar",
    "tapiáramos": "tapiar",
    "tapiáremos": "tapiar",
    "tapiásemos": "tapiar",
    "tapié": "tapiar",
    "tapiéis": "tapiar",
    "tapió": "tapiar",
    "tapo": "tapar",
    "tapona": "taponar",
    "taponaba": "taponar",
    "taponabais": "taponar",
    "taponaban": "taponar",
    "taponabas": "taponar",
    "taponad": "taponar",
    "taponada": "taponar",
    "taponadas": "taponar",
    "taponado": "taponar",
    "taponados": "taponar",
    "taponamientos": "taponamiento",
    "taponamos": "taponar",
    "taponan": "taponar",
    "taponando": "taponar",
    "taponara": "taponar",
    "taponarais": "taponar",
    "taponaran": "taponar",
    "taponaras": "taponar",
    "taponare": "taponar",
    "taponareis": "taponar",
    "taponaremos": "taponar",
    "taponaren": "taponar",
    "taponares": "taponar",
    "taponaron": "taponar",
    "taponará": "taponar",
    "taponarán": "taponar",
    "taponarás": "taponar",
    "taponaré": "taponar",
    "taponaréis": "taponar",
    "taponaría": "taponar",
    "taponaríais": "taponar",
    "taponaríamos": "taponar",
    "taponarían": "taponar",
    "taponarías": "taponar",
    "taponas": "taponar",
    "taponase": "taponar",
    "taponaseis": "taponar",
    "taponasen": "taponar",
    "taponases": "taponar",
    "taponaste": "taponar",
    "taponasteis": "taponar",
    "taponazos": "taponazo",
    "tapone": "taponar",
    "taponemos": "taponar",
    "taponen": "taponar",
    "taponera": "taponero",
    "taponeras": "taponero",
    "taponeros": "taponero",
    "taponerías": "taponería",
    "tapones": "taponar",
    "tapono": "taponar",
    "taponábamos": "taponar",
    "taponáis": "taponar",
    "taponáramos": "taponar",
    "taponáremos": "taponar",
    "taponásemos": "taponar",
    "taponé": "taponar",
    "taponéis": "taponar",
    "taponó": "taponar",
    "tapsias": "tapsia",
    "tapujos": "tapujo",
    "tapuyas": "tapuya",
    "tapábamos": "tapar",
    "tapáis": "tapar",
    "tapáramos": "tapar",
    "tapáremos": "tapar",
    "tapásemos": "tapar",
    "tapé": "tapar",
    "tapéis": "tapar",
    "tapó": "tapar",
    "taqueras": "taquera",
    "taques": "taque",
    "taquiarritmias": "taquiarritmia",
    "taquicardias": "taquicardia",
    "taquifagias": "taquifagia",
    "taquiglósidos": "taquiglósido",
    "taquigrafiaba": "taquigrafiar",
    "taquigrafiabais": "taquigrafiar",
    "taquigrafiaban": "taquigrafiar",
    "taquigrafiabas": "taquigrafiar",
    "taquigrafiad": "taquigrafiar",
    "taquigrafiada": "taquigrafiar",
    "taquigrafiadas": "taquigrafiar",
    "taquigrafiado": "taquigrafiar",
    "taquigrafiados": "taquigrafiar",
    "taquigrafiamos": "taquigrafiar",
    "taquigrafiando": "taquigrafiar",
    "taquigrafiara": "taquigrafiar",
    "taquigrafiarais": "taquigrafiar",
    "taquigrafiaran": "taquigrafiar",
    "taquigrafiaras": "taquigrafiar",
    "taquigrafiare": "taquigrafiar",
    "taquigrafiareis": "taquigrafiar",
    "taquigrafiaremos": "taquigrafiar",
    "taquigrafiaren": "taquigrafiar",
    "taquigrafiares": "taquigrafiar",
    "taquigrafiaron": "taquigrafiar",
    "taquigrafiará": "taquigrafiar",
    "taquigrafiarán": "taquigrafiar",
    "taquigrafiarás": "taquigrafiar",
    "taquigrafiaré": "taquigrafiar",
    "taquigrafiaréis": "taquigrafiar",
    "taquigrafiaría": "taquigrafiar",
    "taquigrafiaríais": "taquigrafiar",
    "taquigrafiaríamos": "taquigrafiar",
    "taquigrafiarían": "taquigrafiar",
    "taquigrafiarías": "taquigrafiar",
    "taquigrafiase": "taquigrafiar",
    "taquigrafiaseis": "taquigrafiar",
    "taquigrafiasen": "taquigrafiar",
    "taquigrafiases": "taquigrafiar",
    "taquigrafiaste": "taquigrafiar",
    "taquigrafiasteis": "taquigrafiar",
    "taquigrafiemos": "taquigrafiar",
    "taquigrafiábamos": "taquigrafiar",
    "taquigrafiáis": "taquigrafiar",
    "taquigrafiáramos": "taquigrafiar",
    "taquigrafiáremos": "taquigrafiar",
    "taquigrafiásemos": "taquigrafiar",
    "taquigrafié": "taquigrafiar",
    "taquigrafiéis": "taquigrafiar",
    "taquigrafió": "taquigrafiar",
    "taquigrafía": "taquigrafiar",
    "taquigrafían": "taquigrafiar",
    "taquigrafías": "taquigrafiar",
    "taquigrafíe": "taquigrafiar",
    "taquigrafíen": "taquigrafiar",
    "taquigrafíes": "taquigrafiar",
    "taquigrafío": "taquigrafiar",
    "taquigráfica": "taquigráfico",
    "taquigráficas": "taquigráfico",
    "taquigráficos": "taquigráfico",
    "taquillas": "taquilla",
    "taquillera": "taquillero",
    "taquilleras": "taquillero",
    "taquilleros": "taquillero",
    "taquillos": "taquillo",
    "taquimecanografías": "taquimecanografía",
    "taquimecanógrafa": "taquimecanógrafo",
    "taquimecanógrafas": "taquimecanógrafo",
    "taquimecanógrafos": "taquimecanógrafo",
    "taquimetrías": "taquimetría",
    "taquimétrica": "taquimétrico",
    "taquimétricas": "taquimétrico",
    "taquimétricos": "taquimétrico",
    "taquines": "taquín",
    "taquipneas": "taquipnea",
    "taquisteroles": "taquisterol",
    "taqués": "taqué",
    "taquígrafa": "taquígrafo",
    "taquígrafas": "taquígrafo",
    "taquígrafos": "taquígrafo",
    "taquímetros": "taquímetro",
    "tara": "tarar",
    "taraba": "tarar",
    "tarabais": "tarar",
    "taraban": "tarar",
    "tarabas": "tarar",
    "tarabillas": "tarabilla",
    "tarabitas": "tarabita",
    "taracea": "taracear",
    "taraceaba": "taracear",
    "taraceabais": "taracear",
    "taraceaban": "taracear",
    "taraceabas": "taracear",
    "taracead": "taracear",
    "taraceada": "taracear",
    "taraceadas": "taracear",
    "taraceado": "taracear",
    "taraceados": "taracear",
    "taraceamos": "taracear",
    "taracean": "taracear",
    "taraceando": "taracear",
    "taraceara": "taracear",
    "taracearais": "taracear",
    "taracearan": "taracear",
    "taracearas": "taracear",
    "taraceare": "taracear",
    "taraceareis": "taracear",
    "taracearemos": "taracear",
    "taracearen": "taracear",
    "taraceares": "taracear",
    "taracearon": "taracear",
    "taraceará": "taracear",
    "taracearán": "taracear",
    "taracearás": "taracear",
    "taracearé": "taracear",
    "taracearéis": "taracear",
    "taracearía": "taracear",
    "taracearíais": "taracear",
    "taracearíamos": "taracear",
    "taracearían": "taracear",
    "taracearías": "taracear",
    "taraceas": "taracear",
    "taracease": "taracear",
    "taraceaseis": "taracear",
    "taraceasen": "taracear",
    "taraceases": "taracear",
    "taraceaste": "taracear",
    "taraceasteis": "taracear",
    "taracee": "taracear",
    "taraceemos": "taracear",
    "taraceen": "taracear",
    "taracees": "taracear",
    "taraceo": "taracear",
    "taraceábamos": "taracear",
    "taraceáis": "taracear",
    "taraceáramos": "taracear",
    "taraceáremos": "taracear",
    "taraceásemos": "taracear",
    "taraceé": "taracear",
    "taraceéis": "taracear",
    "taraceó": "taracear",
    "tarad": "tarar",
    "tarada": "tarar",
    "taradas": "tarar",
    "tarado": "tarar",
    "tarados": "tarar",
    "taragallos": "taragallo",
    "tarambanas": "tarambana",
    "taramos": "tarar",
    "taran": "tarar",
    "tarando": "tarar",
    "taranta": "taranto",
    "tarantas": "taranto",
    "tarantelas": "tarantela",
    "tarantos": "taranto",
    "tarantulada": "tarantulado",
    "tarantuladas": "tarantulado",
    "tarantulados": "tarantulado",
    "tarara": "tarar",
    "tararaes": "tarará",
    "tararais": "tarar",
    "tararan": "tarar",
    "tararas": "tarara",
    "tarare": "tarar",
    "tararea": "tararear",
    "tarareaba": "tararear",
    "tarareabais": "tararear",
    "tarareaban": "tararear",
    "tarareabas": "tararear",
    "tararead": "tararear",
    "tarareada": "tararear",
    "tarareadas": "tararear",
    "tarareado": "tararear",
    "tarareados": "tararear",
    "tarareamos": "tararear",
    "tararean": "tararear",
    "tarareando": "tararear",
    "tarareara": "tararear",
    "tararearais": "tararear",
    "tararearan": "tararear",
    "tararearas": "tararear",
    "tarareare": "tararear",
    "tarareareis": "tararear",
    "tararearemos": "tararear",
    "tararearen": "tararear",
    "tarareares": "tararear",
    "tararearon": "tararear",
    "tarareará": "tararear",
    "tararearán": "tararear",
    "tararearás": "tararear",
    "tararearé": "tararear",
    "tararearéis": "tararear",
    "tararearía": "tararear",
    "tararearíais": "tararear",
    "tararearíamos": "tararear",
    "tararearían": "tararear",
    "tararearías": "tararear",
    "tarareas": "tararear",
    "tararease": "tararear",
    "tarareaseis": "tararear",
    "tarareasen": "tararear",
    "tarareases": "tararear",
    "tarareaste": "tararear",
    "tarareasteis": "tararear",
    "tararee": "tararear",
    "tarareemos": "tararear",
    "tarareen": "tararear",
    "tararees": "tararear",
    "tarareis": "tarar",
    "tararemos": "tarar",
    "tararen": "tarar",
    "tarareo": "tararear",
    "tarareos": "tarareo",
    "tarares": "tarar",
    "tarareábamos": "tararear",
    "tarareáis": "tararear",
    "tarareáramos": "tararear",
    "tarareáremos": "tararear",
    "tarareásemos": "tararear",
    "tarareé": "tararear",
    "tarareéis": "tararear",
    "tarareó": "tararear",
    "tarariras": "tararira",
    "tararon": "tarar",
    "tarará": "tarar",
    "tararán": "tarar",
    "tararás": "tarará",
    "tararé": "tarar",
    "tararéis": "tarar",
    "tararía": "tarar",
    "tararíais": "tarar",
    "tararíamos": "tarar",
    "tararían": "tarar",
    "tararías": "tarar",
    "tararíes": "tararí",
    "tararís": "tararí",
    "taras": "tarar",
    "tarasca": "tarasco",
    "tarascaba": "tarascar",
    "tarascabais": "tarascar",
    "tarascaban": "tarascar",
    "tarascabas": "tarascar",
    "tarascad": "tarascar",
    "tarascada": "tarascar",
    "tarascadas": "tarascar",
    "tarascado": "tarascar",
    "tarascados": "tarascar",
    "tarascamos": "tarascar",
    "tarascan": "tarascar",
    "tarascando": "tarascar",
    "tarascara": "tarascar",
    "tarascarais": "tarascar",
    "tarascaran": "tarascar",
    "tarascaras": "tarascar",
    "tarascare": "tarascar",
    "tarascareis": "tarascar",
    "tarascaremos": "tarascar",
    "tarascaren": "tarascar",
    "tarascares": "tarascar",
    "tarascaron": "tarascar",
    "tarascará": "tarascar",
    "tarascarán": "tarascar",
    "tarascarás": "tarascar",
    "tarascaré": "tarascar",
    "tarascaréis": "tarascar",
    "tarascaría": "tarascar",
    "tarascaríais": "tarascar",
    "tarascaríamos": "tarascar",
    "tarascarían": "tarascar",
    "tarascarías": "tarascar",
    "tarascas": "tarasco",
    "tarascase": "tarascar",
    "tarascaseis": "tarascar",
    "tarascasen": "tarascar",
    "tarascases": "tarascar",
    "tarascaste": "tarascar",
    "tarascasteis": "tarascar",
    "tarasco": "tarascar",
    "tarascos": "tarasco",
    "tarascábamos": "tarascar",
    "tarascáis": "tarascar",
    "tarascáramos": "tarascar",
    "tarascáremos": "tarascar",
    "tarascásemos": "tarascar",
    "tarascó": "tarascar",
    "tarase": "tarar",
    "taraseis": "tarar",
    "tarasen": "tarar",
    "tarases": "tarar",
    "tarasque": "tarascar",
    "tarasquemos": "tarascar",
    "tarasquen": "tarascar",
    "tarasques": "tarascar",
    "tarasqué": "tarascar",
    "tarasquéis": "tarascar",
    "taraste": "tarar",
    "tarasteis": "tarar",
    "taraxacones": "taraxacón",
    "tarayales": "tarayal",
    "tarayes": "taray",
    "tarazanales": "tarazanal",
    "tarazanas": "tarazana",
    "tarazas": "taraza",
    "tarazones": "tarazón",
    "tarda": "tardo",
    "tardaba": "tardar",
    "tardabais": "tardar",
    "tardaban": "tardar",
    "tardabas": "tardar",
    "tardad": "tardar",
    "tardada": "tardar",
    "tardadas": "tardar",
    "tardado": "tardar",
    "tardados": "tardar",
    "tardamos": "tardar",
    "tardan": "tardar",
    "tardando": "tardar",
    "tardanzas": "tardanza",
    "tardara": "tardar",
    "tardarais": "tardar",
    "tardaran": "tardar",
    "tardaras": "tardar",
    "tardare": "tardar",
    "tardareis": "tardar",
    "tardaremos": "tardar",
    "tardaren": "tardar",
    "tardares": "tardar",
    "tardaron": "tardar",
    "tardará": "tardar",
    "tardarán": "tardar",
    "tardarás": "tardar",
    "tardaré": "tardar",
    "tardaréis": "tardar",
    "tardaría": "tardar",
    "tardaríais": "tardar",
    "tardaríamos": "tardar",
    "tardarían": "tardar",
    "tardarías": "tardar",
    "tardas": "tardo",
    "tardase": "tardar",
    "tardaseis": "tardar",
    "tardasen": "tardar",
    "tardases": "tardar",
    "tardaste": "tardar",
    "tardasteis": "tardar",
    "tarde": "tardar",
    "tardece": "tardecer",
    "tardeced": "tardecer",
    "tardecemos": "tardecer",
    "tardecen": "tardecer",
    "tardeceremos": "tardecer",
    "tardecerá": "tardecer",
    "tardecerán": "tardecer",
    "tardecerás": "tardecer",
    "tardeceré": "tardecer",
    "tardeceréis": "tardecer",
    "tardecería": "tardecer",
    "tardeceríais": "tardecer",
    "tardeceríamos": "tardecer",
    "tardecerían": "tardecer",
    "tardecerías": "tardecer",
    "tardeces": "tardecer",
    "tardecida": "tardecer",
    "tardecidas": "tardecer",
    "tardecido": "tardecer",
    "tardecidos": "tardecer",
    "tardeciendo": "tardecer",
    "tardeciera": "tardecer",
    "tardecierais": "tardecer",
    "tardecieran": "tardecer",
    "tardecieras": "tardecer",
    "tardeciere": "tardecer",
    "tardeciereis": "tardecer",
    "tardecieren": "tardecer",
    "tardecieres": "tardecer",
    "tardecieron": "tardecer",
    "tardeciese": "tardecer",
    "tardecieseis": "tardecer",
    "tardeciesen": "tardecer",
    "tardecieses": "tardecer",
    "tardecimos": "tardecer",
    "tardeciste": "tardecer",
    "tardecisteis": "tardecer",
    "tardeciéramos": "tardecer",
    "tardeciéremos": "tardecer",
    "tardeciésemos": "tardecer",
    "tardeció": "tardecer",
    "tardecéis": "tardecer",
    "tardecí": "tardecer",
    "tardecía": "tardecer",
    "tardecíais": "tardecer",
    "tardecíamos": "tardecer",
    "tardecían": "tardecer",
    "tardecías": "tardecer",
    "tardemos": "tardar",
    "tarden": "tardar",
    "tardes": "tarde",
    "tardezca": "tardecer",
    "tardezcamos": "tardecer",
    "tardezcan": "tardecer",
    "tardezcas": "tardecer",
    "tardezco": "tardecer",
    "tardezcáis": "tardecer",
    "tardo": "tardar",
    "tardona": "tardón",
    "tardonas": "tardón",
    "tardones": "tardón",
    "tardos": "tardo",
    "tardábamos": "tardar",
    "tardáis": "tardar",
    "tardáramos": "tardar",
    "tardáremos": "tardar",
    "tardásemos": "tardar",
    "tardé": "tardar",
    "tardéis": "tardar",
    "tardía": "tardío",
    "tardías": "tardío",
    "tardígrados": "tardígrado",
    "tardíos": "tardío",
    "tardó": "tardar",
    "tare": "tarar",
    "tareas": "tarea",
    "taremos": "tarar",
    "taren": "tarar",
    "tarentina": "tarentino",
    "tarentinas": "tarentino",
    "tarentinos": "tarentino",
    "tares": "tarar",
    "tarifa": "tarifar",
    "tarifaba": "tarifar",
    "tarifabais": "tarifar",
    "tarifaban": "tarifar",
    "tarifabas": "tarifar",
    "tarifaciones": "tarifación",
    "tarifad": "tarifar",
    "tarifada": "tarifar",
    "tarifadas": "tarifar",
    "tarifado": "tarifar",
    "tarifados": "tarifar",
    "tarifamos": "tarifar",
    "tarifan": "tarifar",
    "tarifando": "tarifar",
    "tarifara": "tarifar",
    "tarifarais": "tarifar",
    "tarifaran": "tarifar",
    "tarifaras": "tarifar",
    "tarifare": "tarifar",
    "tarifareis": "tarifar",
    "tarifaremos": "tarifar",
    "tarifaren": "tarifar",
    "tarifares": "tarifar",
    "tarifaria": "tarifario",
    "tarifarias": "tarifario",
    "tarifarios": "tarifario",
    "tarifaron": "tarifar",
    "tarifará": "tarifar",
    "tarifarán": "tarifar",
    "tarifarás": "tarifar",
    "tarifaré": "tarifar",
    "tarifaréis": "tarifar",
    "tarifaría": "tarifar",
    "tarifaríais": "tarifar",
    "tarifaríamos": "tarifar",
    "tarifarían": "tarifar",
    "tarifarías": "tarifar",
    "tarifas": "tarifar",
    "tarifase": "tarifar",
    "tarifaseis": "tarifar",
    "tarifasen": "tarifar",
    "tarifases": "tarifar",
    "tarifaste": "tarifar",
    "tarifasteis": "tarifar",
    "tarife": "tarifar",
    "tarifemos": "tarifar",
    "tarifen": "tarifar",
    "tarifes": "tarifar",
    "tarifeña": "tarifeño",
    "tarifeñas": "tarifeño",
    "tarifeños": "tarifeño",
    "tarificaciones": "tarificación",
    "tarifo": "tarifar",
    "tarifábamos": "tarifar",
    "tarifáis": "tarifar",
    "tarifáramos": "tarifar",
    "tarifáremos": "tarifar",
    "tarifásemos": "tarifar",
    "tarifé": "tarifar",
    "tariféis": "tarifar",
    "tarifó": "tarifar",
    "tarimas": "tarima",
    "tarja": "tarjar",
    "tarjaba": "tarjar",
    "tarjabais": "tarjar",
    "tarjaban": "tarjar",
    "tarjabas": "tarjar",
    "tarjad": "tarjar",
    "tarjada": "tarjar",
    "tarjadas": "tarjar",
    "tarjado": "tarjar",
    "tarjadora": "tarjador",
    "tarjadoras": "tarjador",
    "tarjadores": "tarjador",
    "tarjados": "tarjar",
    "tarjamos": "tarjar",
    "tarjan": "tarjar",
    "tarjando": "tarjar",
    "tarjara": "tarjar",
    "tarjarais": "tarjar",
    "tarjaran": "tarjar",
    "tarjaras": "tarjar",
    "tarjare": "tarjar",
    "tarjareis": "tarjar",
    "tarjaremos": "tarjar",
    "tarjaren": "tarjar",
    "tarjares": "tarjar",
    "tarjaron": "tarjar",
    "tarjará": "tarjar",
    "tarjarán": "tarjar",
    "tarjarás": "tarjar",
    "tarjaré": "tarjar",
    "tarjaréis": "tarjar",
    "tarjaría": "tarjar",
    "tarjaríais": "tarjar",
    "tarjaríamos": "tarjar",
    "tarjarían": "tarjar",
    "tarjarías": "tarjar",
    "tarjas": "tarjar",
    "tarjase": "tarjar",
    "tarjaseis": "tarjar",
    "tarjasen": "tarjar",
    "tarjases": "tarjar",
    "tarjaste": "tarjar",
    "tarjasteis": "tarjar",
    "tarje": "tarjar",
    "tarjemos": "tarjar",
    "tarjen": "tarjar",
    "tarjera": "tarjero",
    "tarjeras": "tarjero",
    "tarjeros": "tarjero",
    "tarjes": "tarjar",
    "tarjetas": "tarjeta",
    "tarjeteos": "tarjeteo",
    "tarjetera": "tarjetero",
    "tarjeteras": "tarjetero",
    "tarjeteros": "tarjetero",
    "tarjo": "tarjar",
    "tarjábamos": "tarjar",
    "tarjáis": "tarjar",
    "tarjáramos": "tarjar",
    "tarjáremos": "tarjar",
    "tarjásemos": "tarjar",
    "tarjé": "tarjar",
    "tarjéis": "tarjar",
    "tarjó": "tarjar",
    "tarlatanas": "tarlatana",
    "taro": "tarar",
    "tarots": "tarot",
    "tarquina": "tarquino",
    "tarquinadas": "tarquinada",
    "tarquinas": "tarquino",
    "tarquines": "tarquín",
    "tarquinos": "tarquino",
    "tarraconenses": "tarraconense",
    "tarrajas": "tarraja",
    "tarrasenses": "tarrasense",
    "tarreñas": "tarreña",
    "tarros": "tarro",
    "tars": "tar",
    "tarsales": "tarsal",
    "tarseros": "tarsero",
    "tarsiana": "tarsiano",
    "tarsianas": "tarsiano",
    "tarsianos": "tarsiano",
    "tarsometatarsiana": "tarsometatarsiano",
    "tarsometatarsianas": "tarsometatarsiano",
    "tarsometatarsianos": "tarsometatarsiano",
    "tarsometatarsos": "tarsometatarso",
    "tarsoplastias": "tarsoplastia",
    "tarsorrafías": "tarsorrafía",
    "tarsos": "tarso",
    "tartajas": "tartaja",
    "tartajea": "tartajear",
    "tartajeaba": "tartajear",
    "tartajeabais": "tartajear",
    "tartajeaban": "tartajear",
    "tartajeabas": "tartajear",
    "tartajead": "tartajear",
    "tartajeada": "tartajear",
    "tartajeadas": "tartajear",
    "tartajeado": "tartajear",
    "tartajeados": "tartajear",
    "tartajeamos": "tartajear",
    "tartajean": "tartajear",
    "tartajeando": "tartajear",
    "tartajeara": "tartajear",
    "tartajearais": "tartajear",
    "tartajearan": "tartajear",
    "tartajearas": "tartajear",
    "tartajeare": "tartajear",
    "tartajeareis": "tartajear",
    "tartajearemos": "tartajear",
    "tartajearen": "tartajear",
    "tartajeares": "tartajear",
    "tartajearon": "tartajear",
    "tartajeará": "tartajear",
    "tartajearán": "tartajear",
    "tartajearás": "tartajear",
    "tartajearé": "tartajear",
    "tartajearéis": "tartajear",
    "tartajearía": "tartajear",
    "tartajearíais": "tartajear",
    "tartajearíamos": "tartajear",
    "tartajearían": "tartajear",
    "tartajearías": "tartajear",
    "tartajeas": "tartajear",
    "tartajease": "tartajear",
    "tartajeaseis": "tartajear",
    "tartajeasen": "tartajear",
    "tartajeases": "tartajear",
    "tartajeaste": "tartajear",
    "tartajeasteis": "tartajear",
    "tartajee": "tartajear",
    "tartajeemos": "tartajear",
    "tartajeen": "tartajear",
    "tartajees": "tartajear",
    "tartajeo": "tartajear",
    "tartajeos": "tartajeo",
    "tartajeábamos": "tartajear",
    "tartajeáis": "tartajear",
    "tartajeáramos": "tartajear",
    "tartajeáremos": "tartajear",
    "tartajeásemos": "tartajear",
    "tartajeé": "tartajear",
    "tartajeéis": "tartajear",
    "tartajeó": "tartajear",
    "tartajosa": "tartajoso",
    "tartajosas": "tartajoso",
    "tartajosos": "tartajoso",
    "tartaletas": "tartaleta",
    "tartamuda": "tartamudo",
    "tartamudas": "tartamudo",
    "tartamudea": "tartamudear",
    "tartamudeaba": "tartamudear",
    "tartamudeabais": "tartamudear",
    "tartamudeaban": "tartamudear",
    "tartamudeabas": "tartamudear",
    "tartamudead": "tartamudear",
    "tartamudeada": "tartamudear",
    "tartamudeadas": "tartamudear",
    "tartamudeado": "tartamudear",
    "tartamudeados": "tartamudear",
    "tartamudeamos": "tartamudear",
    "tartamudean": "tartamudear",
    "tartamudeando": "tartamudear",
    "tartamudeara": "tartamudear",
    "tartamudearais": "tartamudear",
    "tartamudearan": "tartamudear",
    "tartamudearas": "tartamudear",
    "tartamudeare": "tartamudear",
    "tartamudeareis": "tartamudear",
    "tartamudearemos": "tartamudear",
    "tartamudearen": "tartamudear",
    "tartamudeares": "tartamudear",
    "tartamudearon": "tartamudear",
    "tartamudeará": "tartamudear",
    "tartamudearán": "tartamudear",
    "tartamudearás": "tartamudear",
    "tartamudearé": "tartamudear",
    "tartamudearéis": "tartamudear",
    "tartamudearía": "tartamudear",
    "tartamudearíais": "tartamudear",
    "tartamudearíamos": "tartamudear",
    "tartamudearían": "tartamudear",
    "tartamudearías": "tartamudear",
    "tartamudeas": "tartamudear",
    "tartamudease": "tartamudear",
    "tartamudeaseis": "tartamudear",
    "tartamudeasen": "tartamudear",
    "tartamudeases": "tartamudear",
    "tartamudeaste": "tartamudear",
    "tartamudeasteis": "tartamudear",
    "tartamudeces": "tartamudez",
    "tartamudee": "tartamudear",
    "tartamudeemos": "tartamudear",
    "tartamudeen": "tartamudear",
    "tartamudees": "tartamudear",
    "tartamudeo": "tartamudear",
    "tartamudeos": "tartamudeo",
    "tartamudeábamos": "tartamudear",
    "tartamudeáis": "tartamudear",
    "tartamudeáramos": "tartamudear",
    "tartamudeáremos": "tartamudear",
    "tartamudeásemos": "tartamudear",
    "tartamudeé": "tartamudear",
    "tartamudeéis": "tartamudear",
    "tartamudeó": "tartamudear",
    "tartamudos": "tartamudo",
    "tartanas": "tartana",
    "tartanera": "tartanero",
    "tartaneras": "tartanero",
    "tartaneros": "tartanero",
    "tartanes": "tartán",
    "tartarice": "tartarizar",
    "tartaricemos": "tartarizar",
    "tartaricen": "tartarizar",
    "tartarices": "tartarizar",
    "tartaricé": "tartarizar",
    "tartaricéis": "tartarizar",
    "tartariza": "tartarizar",
    "tartarizaba": "tartarizar",
    "tartarizabais": "tartarizar",
    "tartarizaban": "tartarizar",
    "tartarizabas": "tartarizar",
    "tartarizad": "tartarizar",
    "tartarizada": "tartarizar",
    "tartarizadas": "tartarizar",
    "tartarizado": "tartarizar",
    "tartarizados": "tartarizar",
    "tartarizamos": "tartarizar",
    "tartarizan": "tartarizar",
    "tartarizando": "tartarizar",
    "tartarizara": "tartarizar",
    "tartarizarais": "tartarizar",
    "tartarizaran": "tartarizar",
    "tartarizaras": "tartarizar",
    "tartarizare": "tartarizar",
    "tartarizareis": "tartarizar",
    "tartarizaremos": "tartarizar",
    "tartarizaren": "tartarizar",
    "tartarizares": "tartarizar",
    "tartarizaron": "tartarizar",
    "tartarizará": "tartarizar",
    "tartarizarán": "tartarizar",
    "tartarizarás": "tartarizar",
    "tartarizaré": "tartarizar",
    "tartarizaréis": "tartarizar",
    "tartarizaría": "tartarizar",
    "tartarizaríais": "tartarizar",
    "tartarizaríamos": "tartarizar",
    "tartarizarían": "tartarizar",
    "tartarizarías": "tartarizar",
    "tartarizas": "tartarizar",
    "tartarizase": "tartarizar",
    "tartarizaseis": "tartarizar",
    "tartarizasen": "tartarizar",
    "tartarizases": "tartarizar",
    "tartarizaste": "tartarizar",
    "tartarizasteis": "tartarizar",
    "tartarizo": "tartarizar",
    "tartarizábamos": "tartarizar",
    "tartarizáis": "tartarizar",
    "tartarizáramos": "tartarizar",
    "tartarizáremos": "tartarizar",
    "tartarizásemos": "tartarizar",
    "tartarizó": "tartarizar",
    "tartas": "tarta",
    "tarteras": "tartera",
    "tartesia": "tartesio",
    "tartesias": "tartesio",
    "tartesios": "tartesio",
    "tartratos": "tartrato",
    "tartánes": "tartán",
    "tartárea": "tartáreo",
    "tartáreas": "tartáreo",
    "tartáreos": "tartáreo",
    "tartárica": "tartárico",
    "tartáricas": "tartárico",
    "tartáricos": "tartárico",
    "taruga": "tarugo",
    "tarugas": "tarugo",
    "tarugos": "tarugo",
    "tarábamos": "tarar",
    "taráis": "tarar",
    "taránganas": "tarángana",
    "tarántulas": "tarántula",
    "taráramos": "tarar",
    "taráremos": "tarar",
    "tarásemos": "tarar",
    "taré": "tarar",
    "taréis": "tarar",
    "taró": "tarar",
    "tasa": "tasar",
    "tasaba": "tasar",
    "tasabais": "tasar",
    "tasaban": "tasar",
    "tasabas": "tasar",
    "tasables": "tasable",
    "tasaciones": "tasación",
    "tasad": "tasar",
    "tasada": "tasar",
    "tasadas": "tasar",
    "tasado": "tasar",
    "tasadora": "tasador",
    "tasadoras": "tasador",
    "tasadores": "tasador",
    "tasados": "tasar",
    "tasajos": "tasajo",
    "tasamos": "tasar",
    "tasan": "tasar",
    "tasando": "tasar",
    "tasara": "tasar",
    "tasarais": "tasar",
    "tasaran": "tasar",
    "tasaras": "tasar",
    "tasare": "tasar",
    "tasareis": "tasar",
    "tasaremos": "tasar",
    "tasaren": "tasar",
    "tasares": "tasar",
    "tasaron": "tasar",
    "tasará": "tasar",
    "tasarán": "tasar",
    "tasarás": "tasar",
    "tasaré": "tasar",
    "tasaréis": "tasar",
    "tasaría": "tasar",
    "tasaríais": "tasar",
    "tasaríamos": "tasar",
    "tasarían": "tasar",
    "tasarías": "tasar",
    "tasas": "tasar",
    "tasase": "tasar",
    "tasaseis": "tasar",
    "tasasen": "tasar",
    "tasases": "tasar",
    "tasaste": "tasar",
    "tasasteis": "tasar",
    "tasca": "tascar",
    "tascaba": "tascar",
    "tascabais": "tascar",
    "tascaban": "tascar",
    "tascabas": "tascar",
    "tascad": "tascar",
    "tascada": "tascar",
    "tascadas": "tascar",
    "tascado": "tascar",
    "tascadores": "tascador",
    "tascados": "tascar",
    "tascamos": "tascar",
    "tascan": "tascar",
    "tascando": "tascar",
    "tascara": "tascar",
    "tascarais": "tascar",
    "tascaran": "tascar",
    "tascaras": "tascar",
    "tascare": "tascar",
    "tascareis": "tascar",
    "tascaremos": "tascar",
    "tascaren": "tascar",
    "tascares": "tascar",
    "tascaron": "tascar",
    "tascará": "tascar",
    "tascarán": "tascar",
    "tascarás": "tascar",
    "tascaré": "tascar",
    "tascaréis": "tascar",
    "tascaría": "tascar",
    "tascaríais": "tascar",
    "tascaríamos": "tascar",
    "tascarían": "tascar",
    "tascarías": "tascar",
    "tascas": "tascar",
    "tascase": "tascar",
    "tascaseis": "tascar",
    "tascasen": "tascar",
    "tascases": "tascar",
    "tascaste": "tascar",
    "tascasteis": "tascar",
    "tasco": "tascar",
    "tasconios": "tasconio",
    "tascos": "tasco",
    "tascábamos": "tascar",
    "tascáis": "tascar",
    "tascáramos": "tascar",
    "tascáremos": "tascar",
    "tascásemos": "tascar",
    "tascó": "tascar",
    "tase": "tasar",
    "tasemos": "tasar",
    "tasen": "tasar",
    "tases": "tasar",
    "tasis": "tasi",
    "taso": "tasar",
    "tasque": "tascar",
    "tasquemos": "tascar",
    "tasquen": "tascar",
    "tasqueras": "tasquera",
    "tasqueros": "tasquero",
    "tasques": "tascar",
    "tasquiles": "tasquil",
    "tasqué": "tascar",
    "tasquéis": "tascar",
    "tastaces": "tastaz",
    "tastanas": "tastana",
    "tastos": "tasto",
    "tasábamos": "tasar",
    "tasáis": "tasar",
    "tasáramos": "tasar",
    "tasáremos": "tasar",
    "tasásemos": "tasar",
    "tasé": "tasar",
    "taséis": "tasar",
    "tasíes": "tasi",
    "tasó": "tasar",
    "tata": "tato",
    "tatamis": "tatami",
    "tatarabuela": "tatarabuelo",
    "tatarabuelas": "tatarabuelo",
    "tatarabuelos": "tatarabuelo",
    "tataradeuda": "tataradeudo",
    "tataradeudas": "tataradeudo",
    "tataradeudos": "tataradeudo",
    "tataranieta": "tataranieto",
    "tataranietas": "tataranieto",
    "tataranietos": "tataranieto",
    "tatarés": "tataré",
    "tatas": "tato",
    "tatos": "tato",
    "tatuaba": "tatuar",
    "tatuabais": "tatuar",
    "tatuaban": "tatuar",
    "tatuabas": "tatuar",
    "tatuad": "tatuar",
    "tatuada": "tatuar",
    "tatuadas": "tatuar",
    "tatuado": "tatuar",
    "tatuadora": "tatuador",
    "tatuadoras": "tatuador",
    "tatuadores": "tatuador",
    "tatuados": "tatuar",
    "tatuajes": "tatuaje",
    "tatuamos": "tatuar",
    "tatuando": "tatuar",
    "tatuara": "tatuar",
    "tatuarais": "tatuar",
    "tatuaran": "tatuar",
    "tatuaras": "tatuar",
    "tatuare": "tatuar",
    "tatuareis": "tatuar",
    "tatuaremos": "tatuar",
    "tatuaren": "tatuar",
    "tatuares": "tatuar",
    "tatuaron": "tatuar",
    "tatuará": "tatuar",
    "tatuarán": "tatuar",
    "tatuarás": "tatuar",
    "tatuaré": "tatuar",
    "tatuaréis": "tatuar",
    "tatuaría": "tatuar",
    "tatuaríais": "tatuar",
    "tatuaríamos": "tatuar",
    "tatuarían": "tatuar",
    "tatuarías": "tatuar",
    "tatuase": "tatuar",
    "tatuaseis": "tatuar",
    "tatuasen": "tatuar",
    "tatuases": "tatuar",
    "tatuaste": "tatuar",
    "tatuasteis": "tatuar",
    "tatuemos": "tatuar",
    "tatuábamos": "tatuar",
    "tatuáis": "tatuar",
    "tatuáramos": "tatuar",
    "tatuáremos": "tatuar",
    "tatuásemos": "tatuar",
    "tatué": "tatuar",
    "tatuéis": "tatuar",
    "tatuó": "tatuar",
    "tatúa": "tatuar",
    "tatúan": "tatuar",
    "tatúas": "tatuar",
    "tatúe": "tatuar",
    "tatúen": "tatuar",
    "tatúes": "tatuar",
    "tatúo": "tatuar",
    "taucronismos": "taucronismo",
    "taujeles": "taujel",
    "taujías": "taujía",
    "taulas": "taula",
    "taumaturga": "taumaturgo",
    "taumaturgas": "taumaturgo",
    "taumaturgias": "taumaturgia",
    "taumaturgos": "taumaturgo",
    "taumatúrgica": "taumatúrgico",
    "taumatúrgicas": "taumatúrgico",
    "taumatúrgicos": "taumatúrgico",
    "taurina": "taurino",
    "taurinas": "taurino",
    "taurinos": "taurino",
    "taurodonta": "taurodonto",
    "taurodontas": "taurodonto",
    "taurodontismos": "taurodontismo",
    "taurodontos": "taurodonto",
    "tauromaquias": "tauromaquia",
    "tauromáquica": "tauromáquico",
    "tauromáquicas": "tauromáquico",
    "tauromáquicos": "tauromáquico",
    "taurófila": "taurófilo",
    "taurófilas": "taurófilo",
    "taurófilos": "taurófilo",
    "taurómaca": "taurómaco",
    "taurómacas": "taurómaco",
    "taurómacos": "taurómaco",
    "taus": "tau",
    "tautofonías": "tautofonía",
    "tautofónica": "tautofónico",
    "tautofónicas": "tautofónico",
    "tautofónicos": "tautofónico",
    "tautologías": "tautología",
    "tautológica": "tautológico",
    "tautológicas": "tautológico",
    "tautológicos": "tautológico",
    "tautomerismos": "tautomerismo",
    "tautomerías": "tautomería",
    "tautosilábica": "tautosilábico",
    "tautosilábicas": "tautosilábico",
    "tautosilábicos": "tautosilábico",
    "tautómera": "tautómero",
    "tautómeras": "tautómero",
    "tautómeros": "tautómero",
    "taxativa": "taxativo",
    "taxativas": "taxativo",
    "taxatividades": "taxatividad",
    "taxativos": "taxativo",
    "taxias": "taxia",
    "taxidermias": "taxidermia",
    "taxidermistas": "taxidermista",
    "taxis": "taxi",
    "taxistas": "taxista",
    "taxodiácea": "taxodiáceo",
    "taxodiáceas": "taxodiáceo",
    "taxodiáceos": "taxodiáceo",
    "taxones": "taxón",
    "taxonomistas": "taxonomista",
    "taxonomías": "taxonomía",
    "taxonómica": "taxonómico",
    "taxonómicas": "taxonómico",
    "taxonómicos": "taxonómico",
    "taxácea": "taxáceo",
    "taxáceas": "taxáceo",
    "taxáceos": "taxáceo",
    "taxímetros": "taxímetro",
    "taxónoma": "taxónomo",
    "taxónomas": "taxónomo",
    "taxónomos": "taxónomo",
    "tayas": "taya",
    "taylorismos": "taylorismo",
    "tazas": "taza",
    "tazañas": "tazaña",
    "tazmías": "tazmía",
    "tazones": "tazón",
    "taíma": "taimar",
    "taíman": "taimar",
    "taímas": "taimar",
    "taíme": "taimar",
    "taímen": "taimar",
    "taímes": "taimar",
    "taímo": "taimar",
    "taína": "taíno",
    "taínas": "taíno",
    "taínos": "taíno",
    "taña": "tañer",
    "tañamos": "tañer",
    "tañan": "tañer",
    "tañas": "tañer",
    "tañe": "tañer",
    "tañed": "tañer",
    "tañedora": "tañedor",
    "tañedoras": "tañedor",
    "tañedores": "tañedor",
    "tañemos": "tañer",
    "tañen": "tañer",
    "tañendo": "tañer",
    "tañera": "tañer",
    "tañerais": "tañer",
    "tañeran": "tañer",
    "tañeras": "tañer",
    "tañere": "tañer",
    "tañereis": "tañer",
    "tañeremos": "tañer",
    "tañeren": "tañer",
    "tañeres": "tañer",
    "tañeron": "tañer",
    "tañerá": "tañer",
    "tañerán": "tañer",
    "tañerás": "tañer",
    "tañeré": "tañer",
    "tañeréis": "tañer",
    "tañería": "tañer",
    "tañeríais": "tañer",
    "tañeríamos": "tañer",
    "tañerían": "tañer",
    "tañerías": "tañer",
    "tañes": "tañer",
    "tañese": "tañer",
    "tañeseis": "tañer",
    "tañesen": "tañer",
    "tañeses": "tañer",
    "tañida": "tañer",
    "tañidas": "tañer",
    "tañido": "tañer",
    "tañidos": "tañido",
    "tañimientos": "tañimiento",
    "tañimos": "tañer",
    "tañiste": "tañer",
    "tañisteis": "tañer",
    "taño": "tañer",
    "taños": "taño",
    "tañáis": "tañer",
    "tañéis": "tañer",
    "tañéramos": "tañer",
    "tañéremos": "tañer",
    "tañésemos": "tañer",
    "tañí": "tañer",
    "tañía": "tañer",
    "tañíais": "tañer",
    "tañíamos": "tañer",
    "tañían": "tañer",
    "tañías": "tañer",
    "tañó": "tañer",
    "teames": "teame",
    "teamides": "teamide",
    "teas": "tea",
    "teatina": "teatino",
    "teatinas": "teatino",
    "teatinos": "teatino",
    "teatrales": "teatral",
    "teatralidades": "teatralidad",
    "teatralizaciones": "teatralización",
    "teatrinos": "teatrino",
    "teatros": "teatro",
    "tebaica": "tebaico",
    "tebaicas": "tebaico",
    "tebaicos": "tebaico",
    "tebana": "tebano",
    "tebanas": "tebano",
    "tebanos": "tebano",
    "tebaínas": "tebaína",
    "tebea": "tebeo",
    "tebeas": "tebeo",
    "tebeos": "tebeo",
    "tecameboideos": "tecameboideo",
    "tecas": "teca",
    "teces": "tez",
    "techa": "techar",
    "techaba": "techar",
    "techabais": "techar",
    "techaban": "techar",
    "techabas": "techar",
    "techad": "techar",
    "techada": "techar",
    "techadas": "techar",
    "techado": "techar",
    "techadores": "techador",
    "techados": "techar",
    "techamos": "techar",
    "techan": "techar",
    "techando": "techar",
    "techara": "techar",
    "techarais": "techar",
    "techaran": "techar",
    "techaras": "techar",
    "techare": "techar",
    "techareis": "techar",
    "techaremos": "techar",
    "techaren": "techar",
    "techares": "techar",
    "techaron": "techar",
    "techará": "techar",
    "techarán": "techar",
    "techarás": "techar",
    "techaré": "techar",
    "techaréis": "techar",
    "techaría": "techar",
    "techaríais": "techar",
    "techaríamos": "techar",
    "techarían": "techar",
    "techarías": "techar",
    "techas": "techar",
    "techase": "techar",
    "techaseis": "techar",
    "techasen": "techar",
    "techases": "techar",
    "techaste": "techar",
    "techasteis": "techar",
    "teche": "techar",
    "techemos": "techar",
    "techen": "techar",
    "teches": "techar",
    "techo": "techar",
    "techos": "techo",
    "techumbres": "techumbre",
    "techábamos": "techar",
    "techáis": "techar",
    "techáramos": "techar",
    "techáremos": "techar",
    "techásemos": "techar",
    "teché": "techar",
    "techéis": "techar",
    "techó": "techar",
    "teckels": "teckel",
    "teclados": "teclado",
    "teclas": "tecla",
    "teclea": "teclear",
    "tecleaba": "teclear",
    "tecleabais": "teclear",
    "tecleaban": "teclear",
    "tecleabas": "teclear",
    "teclead": "teclear",
    "tecleada": "teclear",
    "tecleadas": "teclear",
    "tecleado": "teclear",
    "tecleados": "teclear",
    "tecleamos": "teclear",
    "teclean": "teclear",
    "tecleando": "teclear",
    "tecleara": "teclear",
    "teclearais": "teclear",
    "teclearan": "teclear",
    "teclearas": "teclear",
    "tecleare": "teclear",
    "tecleareis": "teclear",
    "teclearemos": "teclear",
    "teclearen": "teclear",
    "tecleares": "teclear",
    "teclearon": "teclear",
    "tecleará": "teclear",
    "teclearán": "teclear",
    "teclearás": "teclear",
    "teclearé": "teclear",
    "teclearéis": "teclear",
    "teclearía": "teclear",
    "teclearíais": "teclear",
    "teclearíamos": "teclear",
    "teclearían": "teclear",
    "teclearías": "teclear",
    "tecleas": "teclear",
    "teclease": "teclear",
    "tecleaseis": "teclear",
    "tecleasen": "teclear",
    "tecleases": "teclear",
    "tecleaste": "teclear",
    "tecleasteis": "teclear",
    "teclee": "teclear",
    "tecleemos": "teclear",
    "tecleen": "teclear",
    "teclees": "teclear",
    "tecleo": "teclear",
    "tecleos": "tecleo",
    "tecles": "tecle",
    "tecleábamos": "teclear",
    "tecleáis": "teclear",
    "tecleáramos": "teclear",
    "tecleáremos": "teclear",
    "tecleásemos": "teclear",
    "tecleé": "teclear",
    "tecleéis": "teclear",
    "tecleó": "teclear",
    "teclistas": "teclista",
    "tecnecios": "tecnecio",
    "tecnicismos": "tecnicismo",
    "tecnicolores": "tecnicolor",
    "tecnificaciones": "tecnificación",
    "tecnoambientales": "tecnoambiental",
    "tecnociencias": "tecnociencia",
    "tecnocracias": "tecnocracia",
    "tecnocrática": "tecnocrático",
    "tecnocráticas": "tecnocrático",
    "tecnocráticos": "tecnocrático",
    "tecnoeconómica": "tecnoeconómico",
    "tecnoeconómicas": "tecnoeconómico",
    "tecnoeconómicos": "tecnoeconómico",
    "tecnoestructuras": "tecnoestructura",
    "tecnoindustriales": "tecnoindustrial",
    "tecnologías": "tecnología",
    "tecnológica": "tecnológico",
    "tecnológicas": "tecnológico",
    "tecnológicos": "tecnológico",
    "tecnígrafos": "tecnígrafo",
    "tecnócratas": "tecnócrata",
    "tecnóloga": "tecnólogo",
    "tecnólogas": "tecnólogo",
    "tecnólogos": "tecnólogo",
    "tecodonta": "tecodonto",
    "tecodontas": "tecodonto",
    "tecodontos": "tecodonto",
    "tectitas": "tectita",
    "tectolabiales": "tectolabial",
    "tectologías": "tectología",
    "tectoria": "tectorio",
    "tectorias": "tectorio",
    "tectorios": "tectorio",
    "tectosferas": "tectosfera",
    "tectosilicatos": "tectosilicato",
    "tectrices": "tectriz",
    "tectónica": "tectónico",
    "tectónicas": "tectónico",
    "tectónicos": "tectónico",
    "tedas": "teda",
    "tederos": "tedero",
    "tedia": "tediar",
    "tediaba": "tediar",
    "tediabais": "tediar",
    "tediaban": "tediar",
    "tediabas": "tediar",
    "tediad": "tediar",
    "tediada": "tediar",
    "tediadas": "tediar",
    "tediado": "tediar",
    "tediados": "tediar",
    "tediamos": "tediar",
    "tedian": "tediar",
    "tediando": "tediar",
    "tediara": "tediar",
    "tediarais": "tediar",
    "tediaran": "tediar",
    "tediaras": "tediar",
    "tediare": "tediar",
    "tediareis": "tediar",
    "tediaremos": "tediar",
    "tediaren": "tediar",
    "tediares": "tediar",
    "tediaron": "tediar",
    "tediará": "tediar",
    "tediarán": "tediar",
    "tediarás": "tediar",
    "tediaré": "tediar",
    "tediaréis": "tediar",
    "tediaría": "tediar",
    "tediaríais": "tediar",
    "tediaríamos": "tediar",
    "tediarían": "tediar",
    "tediarías": "tediar",
    "tedias": "tediar",
    "tediase": "tediar",
    "tediaseis": "tediar",
    "tediasen": "tediar",
    "tediases": "tediar",
    "tediaste": "tediar",
    "tediasteis": "tediar",
    "tedie": "tediar",
    "tediemos": "tediar",
    "tedien": "tediar",
    "tedies": "tediar",
    "tedio": "tediar",
    "tedios": "tedio",
    "tediosa": "tedioso",
    "tediosas": "tedioso",
    "tediosos": "tedioso",
    "tediábamos": "tediar",
    "tediáis": "tediar",
    "tediáramos": "tediar",
    "tediáremos": "tediar",
    "tediásemos": "tediar",
    "tedié": "tediar",
    "tediéis": "tediar",
    "tedió": "tediar",
    "teflones": "teflón",
    "teguillos": "teguillo",
    "tegumentaria": "tegumentario",
    "tegumentarias": "tegumentario",
    "tegumentarios": "tegumentario",
    "tegumentos": "tegumento",
    "teguys": "téguy",
    "tehuelches": "tehuelche",
    "teja": "tejer",
    "tejaba": "tejar",
    "tejabais": "tejar",
    "tejaban": "tejar",
    "tejabas": "tejar",
    "tejad": "tejar",
    "tejada": "tejar",
    "tejadas": "tejar",
    "tejadillos": "tejadillo",
    "tejado": "tejar",
    "tejadora": "tejador",
    "tejadoras": "tejador",
    "tejadores": "tejador",
    "tejados": "tejar",
    "tejamos": "tejer",
    "tejan": "tejer",
    "tejana": "tejano",
    "tejanas": "tejano",
    "tejando": "tejar",
    "tejanos": "tejano",
    "tejara": "tejar",
    "tejarais": "tejar",
    "tejaran": "tejar",
    "tejaras": "tejar",
    "tejare": "tejar",
    "tejareis": "tejar",
    "tejaremos": "tejar",
    "tejaren": "tejar",
    "tejares": "tejar",
    "tejaroces": "tejaroz",
    "tejaron": "tejar",
    "tejará": "tejar",
    "tejarán": "tejar",
    "tejarás": "tejar",
    "tejaré": "tejar",
    "tejaréis": "tejar",
    "tejaría": "tejar",
    "tejaríais": "tejar",
    "tejaríamos": "tejar",
    "tejarían": "tejar",
    "tejarías": "tejar",
    "tejas": "tejer",
    "tejase": "tejar",
    "tejaseis": "tejar",
    "tejasen": "tejar",
    "tejases": "tejar",
    "tejaste": "tejar",
    "tejasteis": "tejar",
    "tejavanas": "tejavana",
    "teje": "tejer",
    "tejed": "tejer",
    "tejederas": "tejedera",
    "tejedora": "tejedor",
    "tejedoras": "tejedor",
    "tejedores": "tejedor",
    "tejeduras": "tejedura",
    "tejedurías": "tejeduría",
    "tejemanejes": "tejemaneje",
    "tejemos": "tejer",
    "tejen": "tejer",
    "tejeras": "tejera",
    "tejeremos": "tejer",
    "tejerá": "tejer",
    "tejerán": "tejer",
    "tejerás": "tejer",
    "tejeré": "tejer",
    "tejeréis": "tejer",
    "tejería": "tejer",
    "tejeríais": "tejer",
    "tejeríamos": "tejer",
    "tejerían": "tejer",
    "tejerías": "tejería",
    "tejes": "tejer",
    "tejibles": "tejible",
    "tejida": "tejer",
    "tejidas": "tejer",
    "tejido": "tejer",
    "tejidos": "tejido",
    "tejiendo": "tejer",
    "tejiera": "tejer",
    "tejierais": "tejer",
    "tejieran": "tejer",
    "tejieras": "tejer",
    "tejiere": "tejer",
    "tejiereis": "tejer",
    "tejieren": "tejer",
    "tejieres": "tejer",
    "tejieron": "tejer",
    "tejiese": "tejer",
    "tejieseis": "tejer",
    "tejiesen": "tejer",
    "tejieses": "tejer",
    "tejimos": "tejer",
    "tejiste": "tejer",
    "tejisteis": "tejer",
    "tejiéramos": "tejer",
    "tejiéremos": "tejer",
    "tejiésemos": "tejer",
    "tejió": "tejer",
    "tejo": "tejer",
    "tejoletas": "tejoleta",
    "tejoneras": "tejonera",
    "tejones": "tejón",
    "tejos": "tejo",
    "tejuelas": "tejuela",
    "tejuelos": "tejuelo",
    "tejábamos": "tejar",
    "tejáis": "tejer",
    "tejáramos": "tejar",
    "tejáremos": "tejar",
    "tejásemos": "tejar",
    "tejé": "tejar",
    "tejéis": "tejer",
    "tejí": "tejer",
    "tejía": "tejer",
    "tejíais": "tejer",
    "tejíamos": "tejer",
    "tejían": "tejer",
    "tejías": "tejer",
    "tejó": "tejar",
    "tejúes": "tejú",
    "tejús": "tejú",
    "telamones": "telamón",
    "telarañas": "telaraña",
    "telarañosa": "telarañoso",
    "telarañosas": "telarañoso",
    "telarañosos": "telarañoso",
    "telares": "telar",
    "telas": "tela",
    "telealarmas": "telealarma",
    "teleapuntadores": "teleapuntador",
    "telearrastres": "telearrastre",
    "telebabies": "telebaby",
    "telecabinas": "telecabina",
    "telecargas": "telecarga",
    "telecinematógrafos": "telecinematógrafo",
    "telecines": "telecine",
    "telecintas": "telecinta",
    "telecomandos": "telecomando",
    "telecomposiciones": "telecomposición",
    "telecompras": "telecompra",
    "telecomunicaciones": "telecomunicación",
    "teleconexiones": "teleconexión",
    "teleconferencias": "teleconferencia",
    "teleconmutadora": "teleconmutador",
    "teleconmutadoras": "teleconmutador",
    "teleconmutadores": "teleconmutador",
    "telecontroles": "telecontrol",
    "telecopiadoras": "telecopiadora",
    "telecopias": "telecopia",
    "telecracias": "telecracia",
    "telecromómera": "telecromómero",
    "telecromómeras": "telecromómero",
    "telecromómeros": "telecromómero",
    "teledetecciones": "teledetección",
    "telediafonías": "telediafonía",
    "telediarios": "telediario",
    "teledifusiones": "teledifusión",
    "teledirecciones": "teledirección",
    "teledirige": "teledirigir",
    "teledirigen": "teledirigir",
    "telediriges": "teledirigir",
    "teledirigid": "teledirigir",
    "teledirigida": "teledirigir",
    "teledirigidas": "teledirigir",
    "teledirigido": "teledirigir",
    "teledirigidos": "teledirigir",
    "teledirigiendo": "teledirigir",
    "teledirigiera": "teledirigir",
    "teledirigierais": "teledirigir",
    "teledirigieran": "teledirigir",
    "teledirigieras": "teledirigir",
    "teledirigiere": "teledirigir",
    "teledirigiereis": "teledirigir",
    "teledirigieren": "teledirigir",
    "teledirigieres": "teledirigir",
    "teledirigieron": "teledirigir",
    "teledirigiese": "teledirigir",
    "teledirigieseis": "teledirigir",
    "teledirigiesen": "teledirigir",
    "teledirigieses": "teledirigir",
    "teledirigimos": "teledirigir",
    "teledirigiremos": "teledirigir",
    "teledirigirá": "teledirigir",
    "teledirigirán": "teledirigir",
    "teledirigirás": "teledirigir",
    "teledirigiré": "teledirigir",
    "teledirigiréis": "teledirigir",
    "teledirigiría": "teledirigir",
    "teledirigiríais": "teledirigir",
    "teledirigiríamos": "teledirigir",
    "teledirigirían": "teledirigir",
    "teledirigirías": "teledirigir",
    "teledirigiste": "teledirigir",
    "teledirigisteis": "teledirigir",
    "teledirigiéramos": "teledirigir",
    "teledirigiéremos": "teledirigir",
    "teledirigiésemos": "teledirigir",
    "teledirigió": "teledirigir",
    "teledirigí": "teledirigir",
    "teledirigía": "teledirigir",
    "teledirigíais": "teledirigir",
    "teledirigíamos": "teledirigir",
    "teledirigían": "teledirigir",
    "teledirigías": "teledirigir",
    "teledirigís": "teledirigir",
    "teledirija": "teledirigir",
    "teledirijamos": "teledirigir",
    "teledirijan": "teledirigir",
    "teledirijas": "teledirigir",
    "teledirijo": "teledirigir",
    "teledirijáis": "teledirigir",
    "telees": "tele",
    "teleescrituras": "teleescritura",
    "telefaxes": "telefax",
    "telefilmes": "telefilm",
    "telefilms": "telefilm",
    "telefios": "telefio",
    "telefonazos": "telefonazo",
    "telefonea": "telefonear",
    "telefoneaba": "telefonear",
    "telefoneabais": "telefonear",
    "telefoneaban": "telefonear",
    "telefoneabas": "telefonear",
    "telefonead": "telefonear",
    "telefoneada": "telefonear",
    "telefoneadas": "telefonear",
    "telefoneado": "telefonear",
    "telefoneados": "telefonear",
    "telefoneamos": "telefonear",
    "telefonean": "telefonear",
    "telefoneando": "telefonear",
    "telefoneara": "telefonear",
    "telefonearais": "telefonear",
    "telefonearan": "telefonear",
    "telefonearas": "telefonear",
    "telefoneare": "telefonear",
    "telefoneareis": "telefonear",
    "telefonearemos": "telefonear",
    "telefonearen": "telefonear",
    "telefoneares": "telefonear",
    "telefonearon": "telefonear",
    "telefoneará": "telefonear",
    "telefonearán": "telefonear",
    "telefonearás": "telefonear",
    "telefonearé": "telefonear",
    "telefonearéis": "telefonear",
    "telefonearía": "telefonear",
    "telefonearíais": "telefonear",
    "telefonearíamos": "telefonear",
    "telefonearían": "telefonear",
    "telefonearías": "telefonear",
    "telefoneas": "telefonear",
    "telefonease": "telefonear",
    "telefoneaseis": "telefonear",
    "telefoneasen": "telefonear",
    "telefoneases": "telefonear",
    "telefoneaste": "telefonear",
    "telefoneasteis": "telefonear",
    "telefonee": "telefonear",
    "telefoneemos": "telefonear",
    "telefoneen": "telefonear",
    "telefonees": "telefonear",
    "telefonemas": "telefonema",
    "telefoneo": "telefonear",
    "telefoneábamos": "telefonear",
    "telefoneáis": "telefonear",
    "telefoneáramos": "telefonear",
    "telefoneáremos": "telefonear",
    "telefoneásemos": "telefonear",
    "telefoneé": "telefonear",
    "telefoneéis": "telefonear",
    "telefoneó": "telefonear",
    "telefonistas": "telefonista",
    "telefonías": "telefonía",
    "telefotografías": "telefotografía",
    "telefotogramas": "telefotograma",
    "telefotográfica": "telefotográfico",
    "telefotográficas": "telefotográfico",
    "telefotográficos": "telefotográfico",
    "telefotos": "telefoto",
    "telefotógrafa": "telefotógrafo",
    "telefotógrafas": "telefotógrafo",
    "telefotógrafos": "telefotógrafo",
    "teleféricos": "teleférico",
    "telefónica": "telefónico",
    "telefónicas": "telefónico",
    "telefónicos": "telefónico",
    "telegestiones": "telegestión",
    "telegrabaciones": "telegrabación",
    "telegrafiaba": "telegrafiar",
    "telegrafiabais": "telegrafiar",
    "telegrafiaban": "telegrafiar",
    "telegrafiabas": "telegrafiar",
    "telegrafiad": "telegrafiar",
    "telegrafiada": "telegrafiar",
    "telegrafiadas": "telegrafiar",
    "telegrafiado": "telegrafiar",
    "telegrafiados": "telegrafiar",
    "telegrafiamos": "telegrafiar",
    "telegrafiando": "telegrafiar",
    "telegrafiara": "telegrafiar",
    "telegrafiarais": "telegrafiar",
    "telegrafiaran": "telegrafiar",
    "telegrafiaras": "telegrafiar",
    "telegrafiare": "telegrafiar",
    "telegrafiareis": "telegrafiar",
    "telegrafiaremos": "telegrafiar",
    "telegrafiaren": "telegrafiar",
    "telegrafiares": "telegrafiar",
    "telegrafiaron": "telegrafiar",
    "telegrafiará": "telegrafiar",
    "telegrafiarán": "telegrafiar",
    "telegrafiarás": "telegrafiar",
    "telegrafiaré": "telegrafiar",
    "telegrafiaréis": "telegrafiar",
    "telegrafiaría": "telegrafiar",
    "telegrafiaríais": "telegrafiar",
    "telegrafiaríamos": "telegrafiar",
    "telegrafiarían": "telegrafiar",
    "telegrafiarías": "telegrafiar",
    "telegrafiase": "telegrafiar",
    "telegrafiaseis": "telegrafiar",
    "telegrafiasen": "telegrafiar",
    "telegrafiases": "telegrafiar",
    "telegrafiaste": "telegrafiar",
    "telegrafiasteis": "telegrafiar",
    "telegrafiemos": "telegrafiar",
    "telegrafistas": "telegrafista",
    "telegrafiábamos": "telegrafiar",
    "telegrafiáis": "telegrafiar",
    "telegrafiáramos": "telegrafiar",
    "telegrafiáremos": "telegrafiar",
    "telegrafiásemos": "telegrafiar",
    "telegrafié": "telegrafiar",
    "telegrafiéis": "telegrafiar",
    "telegrafió": "telegrafiar",
    "telegrafía": "telegrafiar",
    "telegrafían": "telegrafiar",
    "telegrafías": "telegrafiar",
    "telegrafíe": "telegrafiar",
    "telegrafíen": "telegrafiar",
    "telegrafíes": "telegrafiar",
    "telegrafío": "telegrafiar",
    "telegramas": "telegrama",
    "telegráfica": "telegráfico",
    "telegráficas": "telegráfico",
    "telegráficos": "telegráfico",
    "teleguiada": "teleguiado",
    "teleguiadas": "teleguiado",
    "teleguiados": "teleguiado",
    "teleguiajes": "teleguiaje",
    "teleimpresiones": "teleimpresión",
    "teleimpresoras": "teleimpresora",
    "teleimpresores": "teleimpresor",
    "teleinformática": "teleinformático",
    "teleinformáticas": "teleinformático",
    "teleinformáticos": "teleinformático",
    "teleinterruptores": "teleinterruptor",
    "telelecturas": "telelectura",
    "teleles": "telele",
    "telemandos": "telemando",
    "telemanipuladora": "telemanipulador",
    "telemanipuladoras": "telemanipulador",
    "telemanipuladores": "telemanipulador",
    "telemecánica": "telemecánico",
    "telemecánicas": "telemecánico",
    "telemecánicos": "telemecánico",
    "telemedicinas": "telemedicina",
    "telemediciones": "telemedición",
    "telemedidas": "telemedida",
    "telemetros": "telemetro",
    "telemetrías": "telemetría",
    "telemotora": "telemotor",
    "telemotoras": "telemotor",
    "telemotores": "telemotor",
    "telemática": "telemático",
    "telemáticas": "telemático",
    "telemáticos": "telemático",
    "telemétrica": "telemétrico",
    "telemétricas": "telemétrico",
    "telemétricos": "telemétrico",
    "telencéfalos": "telencéfalo",
    "telenda": "telendo",
    "telendas": "telendo",
    "telendos": "telendo",
    "telenovelas": "telenovela",
    "teleobjetivos": "teleobjetivo",
    "teleologías": "teleología",
    "teleológica": "teleológico",
    "teleológicas": "teleológico",
    "teleológicos": "teleológico",
    "teleoperaciones": "teleoperación",
    "telepatías": "telepatía",
    "telepeajes": "telepeaje",
    "telepedidos": "telepedido",
    "teleprocesamientos": "teleprocesamiento",
    "teleprocesos": "teleproceso",
    "telepuertos": "telepuerto",
    "telepática": "telepático",
    "telepáticas": "telepático",
    "telepáticos": "telepático",
    "telequinesias": "telequinesia",
    "telequélica": "telequélico",
    "telequélicas": "telequélico",
    "telequélicos": "telequélico",
    "teleras": "telera",
    "telerones": "telerón",
    "teleros": "telero",
    "telerradares": "telerradar",
    "telerradiografías": "telerradiografía",
    "telerradioscopias": "telerradioscopia",
    "telerregulaciones": "telerregulación",
    "telerreuniones": "telerreunión",
    "telerruptores": "telerruptor",
    "teles": "tele",
    "telescopios": "telescopio",
    "telescópica": "telescópico",
    "telescópicas": "telescópico",
    "telescópicos": "telescópico",
    "teleservicios": "teleservicio",
    "teleseñalizaciones": "teleseñalización",
    "telesillas": "telesilla",
    "telespectadora": "telespectador",
    "telespectadoras": "telespectador",
    "telespectadores": "telespectador",
    "telesquises": "telesquí",
    "telesquíes": "telesquí",
    "telesquís": "telesquí",
    "telesupervisiones": "telesupervisión",
    "telesísmica": "telesísmico",
    "telesísmicas": "telesísmico",
    "telesísmicos": "telesísmico",
    "teletaquímetros": "teletaquímetro",
    "teletas": "teleta",
    "teletextos": "teletexto",
    "teletiendas": "teletienda",
    "teletipias": "teletipia",
    "teletipos": "teletipo",
    "teletrabajadora": "teletrabajador",
    "teletrabajadoras": "teletrabajador",
    "teletrabajadores": "teletrabajador",
    "teletransmisiones": "teletransmisión",
    "teletratamientos": "teletratamiento",
    "televaciados": "televaciado",
    "televentas": "televenta",
    "televidentes": "televidente",
    "televisa": "televisar",
    "televisaba": "televisar",
    "televisabais": "televisar",
    "televisaban": "televisar",
    "televisabas": "televisar",
    "televisad": "televisar",
    "televisada": "televisar",
    "televisadas": "televisar",
    "televisado": "televisar",
    "televisados": "televisar",
    "televisamos": "televisar",
    "televisan": "televisar",
    "televisando": "televisar",
    "televisara": "televisar",
    "televisarais": "televisar",
    "televisaran": "televisar",
    "televisaras": "televisar",
    "televisare": "televisar",
    "televisareis": "televisar",
    "televisaremos": "televisar",
    "televisaren": "televisar",
    "televisares": "televisar",
    "televisaron": "televisar",
    "televisará": "televisar",
    "televisarán": "televisar",
    "televisarás": "televisar",
    "televisaré": "televisar",
    "televisaréis": "televisar",
    "televisaría": "televisar",
    "televisaríais": "televisar",
    "televisaríamos": "televisar",
    "televisarían": "televisar",
    "televisarías": "televisar",
    "televisas": "televisar",
    "televisase": "televisar",
    "televisaseis": "televisar",
    "televisasen": "televisar",
    "televisases": "televisar",
    "televisaste": "televisar",
    "televisasteis": "televisar",
    "televise": "televisar",
    "televisemos": "televisar",
    "televisen": "televisar",
    "televises": "televisar",
    "televisiones": "televisión",
    "televisiva": "televisivo",
    "televisivas": "televisivo",
    "televisivos": "televisivo",
    "televiso": "televisar",
    "televisores": "televisor",
    "televisuales": "televisual",
    "televisábamos": "televisar",
    "televisáis": "televisar",
    "televisáramos": "televisar",
    "televisáremos": "televisar",
    "televisásemos": "televisar",
    "televisé": "televisar",
    "televiséis": "televisar",
    "televisó": "televisar",
    "teleósteos": "teleósteo",
    "teleóstomos": "teleóstomo",
    "telfs": "telf",
    "telicariones": "telicarión",
    "telillas": "telilla",
    "tellices": "telliz",
    "tellizas": "telliza",
    "telnets": "telnet",
    "teloblastos": "teloblasto",
    "telocéntrica": "telocéntrico",
    "telocéntricas": "telocéntrico",
    "telocéntricos": "telocéntrico",
    "telofases": "telofase",
    "telogasteres": "telogaster",
    "telolecita": "telolecito",
    "telolecitas": "telolecito",
    "telolecitos": "telolecito",
    "telomerizaciones": "telomerización",
    "telonera": "telonero",
    "teloneras": "telonero",
    "teloneros": "telonero",
    "telones": "telón",
    "telosindética": "telosindético",
    "telosindéticas": "telosindético",
    "telosindéticos": "telosindético",
    "telurios": "telurio",
    "teluros": "teluro",
    "telurómetros": "telurómetro",
    "teléfonos": "teléfono",
    "telégrafos": "telégrafo",
    "telémetros": "telémetro",
    "telómera": "telómero",
    "telómeras": "telómero",
    "telómeros": "telómero",
    "telúrica": "telúrico",
    "telúricas": "telúrico",
    "telúricos": "telúrico",
    "tema": "temer",
    "temamos": "temer",
    "teman": "temer",
    "temarios": "temario",
    "temas": "temer",
    "tematizaciones": "tematización",
    "temblaba": "temblar",
    "temblabais": "temblar",
    "temblaban": "temblar",
    "temblabas": "temblar",
    "temblad": "temblar",
    "temblada": "temblar",
    "tembladales": "tembladal",
    "tembladas": "temblar",
    "tembladera": "tembladero",
    "tembladeras": "tembladero",
    "tembladeros": "tembladero",
    "temblado": "temblar",
    "tembladora": "temblador",
    "tembladoras": "temblador",
    "tembladores": "temblador",
    "temblados": "temblar",
    "temblamos": "temblar",
    "temblando": "temblar",
    "temblara": "temblar",
    "temblarais": "temblar",
    "temblaran": "temblar",
    "temblaras": "temblar",
    "temblare": "temblar",
    "temblareis": "temblar",
    "temblaremos": "temblar",
    "temblaren": "temblar",
    "temblares": "temblar",
    "temblaron": "temblar",
    "temblará": "temblar",
    "temblarán": "temblar",
    "temblarás": "temblar",
    "temblaré": "temblar",
    "temblaréis": "temblar",
    "temblaría": "temblar",
    "temblaríais": "temblar",
    "temblaríamos": "temblar",
    "temblarían": "temblar",
    "temblarías": "temblar",
    "temblase": "temblar",
    "temblaseis": "temblar",
    "temblasen": "temblar",
    "temblases": "temblar",
    "temblaste": "temblar",
    "temblasteis": "temblar",
    "temblemos": "temblar",
    "temblequea": "temblequear",
    "temblequeaba": "temblequear",
    "temblequeabais": "temblequear",
    "temblequeaban": "temblequear",
    "temblequeabas": "temblequear",
    "temblequead": "temblequear",
    "temblequeada": "temblequear",
    "temblequeadas": "temblequear",
    "temblequeado": "temblequear",
    "temblequeados": "temblequear",
    "temblequeamos": "temblequear",
    "temblequean": "temblequear",
    "temblequeando": "temblequear",
    "temblequeara": "temblequear",
    "temblequearais": "temblequear",
    "temblequearan": "temblequear",
    "temblequearas": "temblequear",
    "temblequeare": "temblequear",
    "temblequeareis": "temblequear",
    "temblequearemos": "temblequear",
    "temblequearen": "temblequear",
    "temblequeares": "temblequear",
    "temblequearon": "temblequear",
    "temblequeará": "temblequear",
    "temblequearán": "temblequear",
    "temblequearás": "temblequear",
    "temblequearé": "temblequear",
    "temblequearéis": "temblequear",
    "temblequearía": "temblequear",
    "temblequearíais": "temblequear",
    "temblequearíamos": "temblequear",
    "temblequearían": "temblequear",
    "temblequearías": "temblequear",
    "temblequeas": "temblequear",
    "temblequease": "temblequear",
    "temblequeaseis": "temblequear",
    "temblequeasen": "temblequear",
    "temblequeases": "temblequear",
    "temblequeaste": "temblequear",
    "temblequeasteis": "temblequear",
    "temblequee": "temblequear",
    "temblequeemos": "temblequear",
    "temblequeen": "temblequear",
    "temblequees": "temblequear",
    "temblequeo": "temblequear",
    "tembleques": "tembleque",
    "temblequeábamos": "temblequear",
    "temblequeáis": "temblequear",
    "temblequeáramos": "temblequear",
    "temblequeáremos": "temblequear",
    "temblequeásemos": "temblequear",
    "temblequeé": "temblequear",
    "temblequeéis": "temblequear",
    "temblequeó": "temblequear",
    "tembletea": "tembletear",
    "tembleteaba": "tembletear",
    "tembleteabais": "tembletear",
    "tembleteaban": "tembletear",
    "tembleteabas": "tembletear",
    "tembletead": "tembletear",
    "tembleteada": "tembletear",
    "tembleteadas": "tembletear",
    "tembleteado": "tembletear",
    "tembleteados": "tembletear",
    "tembleteamos": "tembletear",
    "tembletean": "tembletear",
    "tembleteando": "tembletear",
    "tembleteara": "tembletear",
    "tembletearais": "tembletear",
    "tembletearan": "tembletear",
    "tembletearas": "tembletear",
    "tembleteare": "tembletear",
    "tembleteareis": "tembletear",
    "tembletearemos": "tembletear",
    "tembletearen": "tembletear",
    "tembleteares": "tembletear",
    "tembletearon": "tembletear",
    "tembleteará": "tembletear",
    "tembletearán": "tembletear",
    "tembletearás": "tembletear",
    "tembletearé": "tembletear",
    "tembletearéis": "tembletear",
    "tembletearía": "tembletear",
    "tembletearíais": "tembletear",
    "tembletearíamos": "tembletear",
    "tembletearían": "tembletear",
    "tembletearías": "tembletear",
    "tembleteas": "tembletear",
    "tembletease": "tembletear",
    "tembleteaseis": "tembletear",
    "tembleteasen": "tembletear",
    "tembleteases": "tembletear",
    "tembleteaste": "tembletear",
    "tembleteasteis": "tembletear",
    "tembletee": "tembletear",
    "tembleteemos": "tembletear",
    "tembleteen": "tembletear",
    "tembletees": "tembletear",
    "tembleteo": "tembletear",
    "tembleteábamos": "tembletear",
    "tembleteáis": "tembletear",
    "tembleteáramos": "tembletear",
    "tembleteáremos": "tembletear",
    "tembleteásemos": "tembletear",
    "tembleteé": "tembletear",
    "tembleteéis": "tembletear",
    "tembleteó": "tembletear",
    "temblona": "temblón",
    "temblonas": "temblón",
    "temblones": "temblón",
    "temblores": "temblor",
    "temblorosa": "tembloroso",
    "temblorosas": "tembloroso",
    "temblorosos": "tembloroso",
    "temblosa": "tembloso",
    "temblosas": "tembloso",
    "temblosos": "tembloso",
    "temblábamos": "temblar",
    "tembláis": "temblar",
    "tembláramos": "temblar",
    "tembláremos": "temblar",
    "temblásemos": "temblar",
    "temblé": "temblar",
    "tembléis": "temblar",
    "tembló": "temblar",
    "teme": "temer",
    "temed": "temer",
    "temedera": "temedero",
    "temederas": "temedero",
    "temederos": "temedero",
    "temedora": "temedor",
    "temedoras": "temedor",
    "temedores": "temedor",
    "tememos": "temer",
    "temen": "temer",
    "temeraria": "temerario",
    "temerarias": "temerario",
    "temerarios": "temerario",
    "temeremos": "temer",
    "temeridades": "temeridad",
    "temerona": "temerón",
    "temeronas": "temerón",
    "temerones": "temerón",
    "temerosa": "temeroso",
    "temerosas": "temeroso",
    "temerosos": "temeroso",
    "temerá": "temer",
    "temerán": "temer",
    "temerás": "temer",
    "temeré": "temer",
    "temeréis": "temer",
    "temería": "temer",
    "temeríais": "temer",
    "temeríamos": "temer",
    "temerían": "temer",
    "temerías": "temer",
    "temes": "temer",
    "temibles": "temible",
    "temida": "temer",
    "temidas": "temer",
    "temido": "temer",
    "temidos": "temer",
    "temiendo": "temer",
    "temiera": "temer",
    "temierais": "temer",
    "temieran": "temer",
    "temieras": "temer",
    "temiere": "temer",
    "temiereis": "temer",
    "temieren": "temer",
    "temieres": "temer",
    "temieron": "temer",
    "temiese": "temer",
    "temieseis": "temer",
    "temiesen": "temer",
    "temieses": "temer",
    "temimos": "temer",
    "temiste": "temer",
    "temisteis": "temer",
    "temiéramos": "temer",
    "temiéremos": "temer",
    "temiésemos": "temer",
    "temió": "temer",
    "temo": "temer",
    "temores": "temor",
    "temosa": "temoso",
    "temosas": "temoso",
    "temosos": "temoso",
    "tempana": "tempanar",
    "tempanaba": "tempanar",
    "tempanabais": "tempanar",
    "tempanaban": "tempanar",
    "tempanabas": "tempanar",
    "tempanad": "tempanar",
    "tempanada": "tempanar",
    "tempanadas": "tempanar",
    "tempanado": "tempanar",
    "tempanados": "tempanar",
    "tempanamos": "tempanar",
    "tempanan": "tempanar",
    "tempanando": "tempanar",
    "tempanara": "tempanar",
    "tempanarais": "tempanar",
    "tempanaran": "tempanar",
    "tempanaras": "tempanar",
    "tempanare": "tempanar",
    "tempanareis": "tempanar",
    "tempanaremos": "tempanar",
    "tempanaren": "tempanar",
    "tempanares": "tempanar",
    "tempanaron": "tempanar",
    "tempanará": "tempanar",
    "tempanarán": "tempanar",
    "tempanarás": "tempanar",
    "tempanaré": "tempanar",
    "tempanaréis": "tempanar",
    "tempanaría": "tempanar",
    "tempanaríais": "tempanar",
    "tempanaríamos": "tempanar",
    "tempanarían": "tempanar",
    "tempanarías": "tempanar",
    "tempanas": "tempanar",
    "tempanase": "tempanar",
    "tempanaseis": "tempanar",
    "tempanasen": "tempanar",
    "tempanases": "tempanar",
    "tempanaste": "tempanar",
    "tempanasteis": "tempanar",
    "tempane": "tempanar",
    "tempanemos": "tempanar",
    "tempanen": "tempanar",
    "tempanes": "tempanar",
    "tempano": "tempanar",
    "tempanábamos": "tempanar",
    "tempanáis": "tempanar",
    "tempanáramos": "tempanar",
    "tempanáremos": "tempanar",
    "tempanásemos": "tempanar",
    "tempané": "tempanar",
    "tempanéis": "tempanar",
    "tempanó": "tempanar",
    "tempera": "temperar",
    "temperaba": "temperar",
    "temperabais": "temperar",
    "temperaban": "temperar",
    "temperabas": "temperar",
    "temperaciones": "temperación",
    "temperad": "temperar",
    "temperada": "temperar",
    "temperadas": "temperar",
    "temperado": "temperar",
    "temperados": "temperar",
    "temperamentales": "temperamental",
    "temperamentos": "temperamento",
    "temperamos": "temperar",
    "temperan": "temperar",
    "temperancias": "temperancia",
    "temperando": "temperar",
    "temperantes": "temperante",
    "temperara": "temperar",
    "temperarais": "temperar",
    "temperaran": "temperar",
    "temperaras": "temperar",
    "temperare": "temperar",
    "temperareis": "temperar",
    "temperaremos": "temperar",
    "temperaren": "temperar",
    "temperares": "temperar",
    "temperaron": "temperar",
    "temperará": "temperar",
    "temperarán": "temperar",
    "temperarás": "temperar",
    "temperaré": "temperar",
    "temperaréis": "temperar",
    "temperaría": "temperar",
    "temperaríais": "temperar",
    "temperaríamos": "temperar",
    "temperarían": "temperar",
    "temperarías": "temperar",
    "temperas": "temperar",
    "temperase": "temperar",
    "temperaseis": "temperar",
    "temperasen": "temperar",
    "temperases": "temperar",
    "temperaste": "temperar",
    "temperasteis": "temperar",
    "temperaturas": "temperatura",
    "tempere": "temperar",
    "temperemos": "temperar",
    "temperen": "temperar",
    "temperes": "temperar",
    "temperies": "temperie",
    "tempero": "temperar",
    "temperos": "tempero",
    "temperábamos": "temperar",
    "temperáis": "temperar",
    "temperáramos": "temperar",
    "temperáremos": "temperar",
    "temperásemos": "temperar",
    "temperé": "temperar",
    "temperéis": "temperar",
    "temperó": "temperar",
    "tempestades": "tempestad",
    "tempestea": "tempestear",
    "tempesteaba": "tempestear",
    "tempesteabais": "tempestear",
    "tempesteaban": "tempestear",
    "tempesteabas": "tempestear",
    "tempestead": "tempestear",
    "tempesteada": "tempestear",
    "tempesteadas": "tempestear",
    "tempesteado": "tempestear",
    "tempesteados": "tempestear",
    "tempesteamos": "tempestear",
    "tempestean": "tempestear",
    "tempesteando": "tempestear",
    "tempesteara": "tempestear",
    "tempestearais": "tempestear",
    "tempestearan": "tempestear",
    "tempestearas": "tempestear",
    "tempesteare": "tempestear",
    "tempesteareis": "tempestear",
    "tempestearemos": "tempestear",
    "tempestearen": "tempestear",
    "tempesteares": "tempestear",
    "tempestearon": "tempestear",
    "tempesteará": "tempestear",
    "tempestearán": "tempestear",
    "tempestearás": "tempestear",
    "tempestearé": "tempestear",
    "tempestearéis": "tempestear",
    "tempestearía": "tempestear",
    "tempestearíais": "tempestear",
    "tempestearíamos": "tempestear",
    "tempestearían": "tempestear",
    "tempestearías": "tempestear",
    "tempesteas": "tempestear",
    "tempestease": "tempestear",
    "tempesteaseis": "tempestear",
    "tempesteasen": "tempestear",
    "tempesteases": "tempestear",
    "tempesteaste": "tempestear",
    "tempesteasteis": "tempestear",
    "tempestee": "tempestear",
    "tempesteemos": "tempestear",
    "tempesteen": "tempestear",
    "tempestees": "tempestear",
    "tempesteo": "tempestear",
    "tempesteábamos": "tempestear",
    "tempesteáis": "tempestear",
    "tempesteáramos": "tempestear",
    "tempesteáremos": "tempestear",
    "tempesteásemos": "tempestear",
    "tempesteé": "tempestear",
    "tempesteéis": "tempestear",
    "tempesteó": "tempestear",
    "tempestiva": "tempestivo",
    "tempestivas": "tempestivo",
    "tempestividades": "tempestividad",
    "tempestivos": "tempestivo",
    "tempestuosa": "tempestuoso",
    "tempestuosas": "tempestuoso",
    "tempestuosos": "tempestuoso",
    "templa": "templar",
    "templaba": "templar",
    "templabais": "templar",
    "templaban": "templar",
    "templabas": "templar",
    "templabilidades": "templabilidad",
    "templables": "templable",
    "templad": "templar",
    "templada": "templar",
    "templadas": "templar",
    "templado": "templar",
    "templadora": "templador",
    "templadoras": "templador",
    "templadores": "templador",
    "templados": "templar",
    "templamos": "templar",
    "templan": "templar",
    "templando": "templar",
    "templanzas": "templanza",
    "templara": "templar",
    "templarais": "templar",
    "templaran": "templar",
    "templaras": "templar",
    "templare": "templar",
    "templareis": "templar",
    "templaremos": "templar",
    "templaren": "templar",
    "templares": "templar",
    "templaria": "templario",
    "templarias": "templario",
    "templarios": "templario",
    "templaron": "templar",
    "templará": "templar",
    "templarán": "templar",
    "templarás": "templar",
    "templaré": "templar",
    "templaréis": "templar",
    "templaría": "templar",
    "templaríais": "templar",
    "templaríamos": "templar",
    "templarían": "templar",
    "templarías": "templar",
    "templas": "templar",
    "templase": "templar",
    "templaseis": "templar",
    "templasen": "templar",
    "templases": "templar",
    "templaste": "templar",
    "templasteis": "templar",
    "temple": "templar",
    "templemos": "templar",
    "templen": "templar",
    "templenes": "templén",
    "temples": "temple",
    "templetes": "templete",
    "templistas": "templista",
    "templo": "templar",
    "templos": "templo",
    "templábamos": "templar",
    "templáis": "templar",
    "templáramos": "templar",
    "templáremos": "templar",
    "templásemos": "templar",
    "templé": "templar",
    "templéis": "templar",
    "templó": "templar",
    "temporadas": "temporada",
    "temporales": "temporal",
    "temporalice": "temporalizar",
    "temporalicemos": "temporalizar",
    "temporalicen": "temporalizar",
    "temporalices": "temporalizar",
    "temporalicé": "temporalizar",
    "temporalicéis": "temporalizar",
    "temporalidades": "temporalidad",
    "temporaliza": "temporalizar",
    "temporalizaba": "temporalizar",
    "temporalizabais": "temporalizar",
    "temporalizaban": "temporalizar",
    "temporalizabas": "temporalizar",
    "temporalizad": "temporalizar",
    "temporalizada": "temporalizar",
    "temporalizadas": "temporalizar",
    "temporalizado": "temporalizar",
    "temporalizados": "temporalizar",
    "temporalizamos": "temporalizar",
    "temporalizan": "temporalizar",
    "temporalizando": "temporalizar",
    "temporalizara": "temporalizar",
    "temporalizarais": "temporalizar",
    "temporalizaran": "temporalizar",
    "temporalizaras": "temporalizar",
    "temporalizare": "temporalizar",
    "temporalizareis": "temporalizar",
    "temporalizaremos": "temporalizar",
    "temporalizaren": "temporalizar",
    "temporalizares": "temporalizar",
    "temporalizaron": "temporalizar",
    "temporalizará": "temporalizar",
    "temporalizarán": "temporalizar",
    "temporalizarás": "temporalizar",
    "temporalizaré": "temporalizar",
    "temporalizaréis": "temporalizar",
    "temporalizaría": "temporalizar",
    "temporalizaríais": "temporalizar",
    "temporalizaríamos": "temporalizar",
    "temporalizarían": "temporalizar",
    "temporalizarías": "temporalizar",
    "temporalizas": "temporalizar",
    "temporalizase": "temporalizar",
    "temporalizaseis": "temporalizar",
    "temporalizasen": "temporalizar",
    "temporalizases": "temporalizar",
    "temporalizaste": "temporalizar",
    "temporalizasteis": "temporalizar",
    "temporalizo": "temporalizar",
    "temporalizábamos": "temporalizar",
    "temporalizáis": "temporalizar",
    "temporalizáramos": "temporalizar",
    "temporalizáremos": "temporalizar",
    "temporalizásemos": "temporalizar",
    "temporalizó": "temporalizar",
    "temporaria": "temporario",
    "temporarias": "temporario",
    "temporarios": "temporario",
    "temporera": "temporero",
    "temporeras": "temporero",
    "temporeros": "temporero",
    "temporice": "temporizar",
    "temporicemos": "temporizar",
    "temporicen": "temporizar",
    "temporices": "temporizar",
    "temporicé": "temporizar",
    "temporicéis": "temporizar",
    "temporiza": "temporizar",
    "temporizaba": "temporizar",
    "temporizabais": "temporizar",
    "temporizaban": "temporizar",
    "temporizabas": "temporizar",
    "temporizaciones": "temporización",
    "temporizad": "temporizar",
    "temporizada": "temporizar",
    "temporizadas": "temporizar",
    "temporizado": "temporizar",
    "temporizadores": "temporizador",
    "temporizados": "temporizar",
    "temporizamos": "temporizar",
    "temporizan": "temporizar",
    "temporizando": "temporizar",
    "temporizara": "temporizar",
    "temporizarais": "temporizar",
    "temporizaran": "temporizar",
    "temporizaras": "temporizar",
    "temporizare": "temporizar",
    "temporizareis": "temporizar",
    "temporizaremos": "temporizar",
    "temporizaren": "temporizar",
    "temporizares": "temporizar",
    "temporizaron": "temporizar",
    "temporizará": "temporizar",
    "temporizarán": "temporizar",
    "temporizarás": "temporizar",
    "temporizaré": "temporizar",
    "temporizaréis": "temporizar",
    "temporizaría": "temporizar",
    "temporizaríais": "temporizar",
    "temporizaríamos": "temporizar",
    "temporizarían": "temporizar",
    "temporizarías": "temporizar",
    "temporizas": "temporizar",
    "temporizase": "temporizar",
    "temporizaseis": "temporizar",
    "temporizasen": "temporizar",
    "temporizases": "temporizar",
    "temporizaste": "temporizar",
    "temporizasteis": "temporizar",
    "temporizo": "temporizar",
    "temporizábamos": "temporizar",
    "temporizáis": "temporizar",
    "temporizáramos": "temporizar",
    "temporizáremos": "temporizar",
    "temporizásemos": "temporizar",
    "temporizó": "temporizar",
    "temporoauriculares": "temporoauricular",
    "temporocigomática": "temporocigomático",
    "temporocigomáticas": "temporocigomático",
    "temporocigomáticos": "temporocigomático",
    "temporoesfenoidales": "temporoesfenoidal",
    "temporofaciales": "temporofacial",
    "temporofrontales": "temporofrontal",
    "temporohioidea": "temporohioideo",
    "temporohioideas": "temporohioideo",
    "temporohioideos": "temporohioideo",
    "temporomalares": "temporomalar",
    "temporomandibulares": "temporomandibular",
    "temporomaxilares": "temporomaxilar",
    "temporooccipitales": "temporooccipital",
    "temporoparietales": "temporoparietal",
    "temporopontina": "temporopontino",
    "temporopontinas": "temporopontino",
    "temporopontinos": "temporopontino",
    "temporánea": "temporáneo",
    "temporáneas": "temporáneo",
    "temporáneos": "temporáneo",
    "tempos": "tempo",
    "temprana": "temprano",
    "tempranales": "tempranal",
    "tempranas": "temprano",
    "tempranera": "tempranero",
    "tempraneras": "tempranero",
    "tempraneros": "tempranero",
    "tempranos": "temprano",
    "temtemes": "temtem",
    "temulenta": "temulento",
    "temulentas": "temulento",
    "temulentos": "temulento",
    "temáis": "temer",
    "temática": "temático",
    "temáticas": "temático",
    "temáticos": "temático",
    "teméis": "temer",
    "temí": "temer",
    "temía": "temer",
    "temíais": "temer",
    "temíamos": "temer",
    "temían": "temer",
    "temías": "temer",
    "ten": "tener",
    "tenacea": "tenacear",
    "tenaceaba": "tenacear",
    "tenaceabais": "tenacear",
    "tenaceaban": "tenacear",
    "tenaceabas": "tenacear",
    "tenacead": "tenacear",
    "tenaceada": "tenacear",
    "tenaceadas": "tenacear",
    "tenaceado": "tenacear",
    "tenaceados": "tenacear",
    "tenaceamos": "tenacear",
    "tenacean": "tenacear",
    "tenaceando": "tenacear",
    "tenaceara": "tenacear",
    "tenacearais": "tenacear",
    "tenacearan": "tenacear",
    "tenacearas": "tenacear",
    "tenaceare": "tenacear",
    "tenaceareis": "tenacear",
    "tenacearemos": "tenacear",
    "tenacearen": "tenacear",
    "tenaceares": "tenacear",
    "tenacearon": "tenacear",
    "tenaceará": "tenacear",
    "tenacearán": "tenacear",
    "tenacearás": "tenacear",
    "tenacearé": "tenacear",
    "tenacearéis": "tenacear",
    "tenacearía": "tenacear",
    "tenacearíais": "tenacear",
    "tenacearíamos": "tenacear",
    "tenacearían": "tenacear",
    "tenacearías": "tenacear",
    "tenaceas": "tenacear",
    "tenacease": "tenacear",
    "tenaceaseis": "tenacear",
    "tenaceasen": "tenacear",
    "tenaceases": "tenacear",
    "tenaceaste": "tenacear",
    "tenaceasteis": "tenacear",
    "tenacee": "tenacear",
    "tenaceemos": "tenacear",
    "tenaceen": "tenacear",
    "tenacees": "tenacear",
    "tenaceo": "tenacear",
    "tenaces": "tenaz",
    "tenaceábamos": "tenacear",
    "tenaceáis": "tenacear",
    "tenaceáramos": "tenacear",
    "tenaceáremos": "tenacear",
    "tenaceásemos": "tenacear",
    "tenaceé": "tenacear",
    "tenaceéis": "tenacear",
    "tenaceó": "tenacear",
    "tenacidades": "tenacidad",
    "tenadas": "tenada",
    "tenallones": "tenallón",
    "tenantes": "tenante",
    "tenantitas": "tenantita",
    "tenarditas": "tenardita",
    "tenares": "tenar",
    "tenas": "tena",
    "tenazadas": "tenazada",
    "tenazes": "tenaz",
    "tencas": "tenca",
    "tenciones": "tención",
    "tendajos": "tendajo",
    "tendaleras": "tendalera",
    "tendaleros": "tendalero",
    "tendales": "tendal",
    "tendamos": "tender",
    "tended": "tender",
    "tendederos": "tendedero",
    "tendedora": "tendedor",
    "tendedoras": "tendedor",
    "tendedores": "tendedor",
    "tendeduras": "tendedura",
    "tendejones": "tendejón",
    "tendeles": "tendel",
    "tendemos": "tender",
    "tendenciales": "tendencial",
    "tendencias": "tendencia",
    "tendenciosa": "tendencioso",
    "tendenciosas": "tendencioso",
    "tendenciosidades": "tendenciosidad",
    "tendenciosos": "tendencioso",
    "tendentes": "tendente",
    "tendera": "tendero",
    "tenderas": "tendero",
    "tenderemos": "tender",
    "tenderetes": "tenderete",
    "tenderos": "tendero",
    "tenders": "ténder",
    "tenderá": "tender",
    "tenderán": "tender",
    "tenderás": "tender",
    "tenderé": "tender",
    "tenderéis": "tender",
    "tendería": "tender",
    "tenderíais": "tender",
    "tenderíamos": "tender",
    "tenderían": "tender",
    "tenderías": "tender",
    "tendida": "tender",
    "tendidas": "tender",
    "tendido": "tender",
    "tendidos": "tendido",
    "tendiendo": "tender",
    "tendiera": "tender",
    "tendierais": "tender",
    "tendieran": "tender",
    "tendieras": "tender",
    "tendiere": "tender",
    "tendiereis": "tender",
    "tendieren": "tender",
    "tendieres": "tender",
    "tendieron": "tender",
    "tendiese": "tender",
    "tendieseis": "tender",
    "tendiesen": "tender",
    "tendieses": "tender",
    "tendimos": "tender",
    "tendinosa": "tendinoso",
    "tendinosas": "tendinoso",
    "tendinosos": "tendinoso",
    "tendiste": "tender",
    "tendisteis": "tender",
    "tendiéramos": "tender",
    "tendiéremos": "tender",
    "tendiésemos": "tender",
    "tendió": "tender",
    "tendones": "tendón",
    "tendremos": "tener",
    "tendrá": "tener",
    "tendrán": "tener",
    "tendrás": "tener",
    "tendré": "tener",
    "tendréis": "tener",
    "tendría": "tener",
    "tendríais": "tener",
    "tendríamos": "tener",
    "tendrían": "tener",
    "tendrías": "tener",
    "tendáis": "tender",
    "tendéis": "tender",
    "tendí": "tender",
    "tendía": "tender",
    "tendíais": "tender",
    "tendíamos": "tender",
    "tendían": "tender",
    "tendías": "tender",
    "tenebrarios": "tenebrario",
    "tenebrismos": "tenebrismo",
    "tenebristas": "tenebrista",
    "tenebriónidos": "tenebriónido",
    "tenebrosa": "tenebroso",
    "tenebrosas": "tenebroso",
    "tenebrosidades": "tenebrosidad",
    "tenebrosos": "tenebroso",
    "tened": "tener",
    "tenederos": "tenedero",
    "tenedoras": "tenedora",
    "tenedores": "tenedor",
    "tenedurías": "teneduría",
    "tenemos": "tener",
    "tenencias": "tenencia",
    "tenerifeña": "tenerifeño",
    "tenerifeñas": "tenerifeño",
    "tenerifeños": "tenerifeño",
    "tenerías": "tenería",
    "tenesmos": "tenesmo",
    "tenga": "tener",
    "tengamos": "tener",
    "tengan": "tener",
    "tengas": "tener",
    "tenges": "tenge",
    "tengo": "tener",
    "tengáis": "tener",
    "tenias": "tenia",
    "tenida": "tener",
    "tenidas": "tenida",
    "tenido": "tener",
    "tenidos": "tener",
    "tenidurías": "teniduría",
    "teniendo": "tener",
    "tenientes": "teniente",
    "tenistas": "tenista",
    "tennos": "tenno",
    "tenoras": "tenora",
    "tenores": "tenor",
    "tenorios": "tenorio",
    "tensa": "tenso",
    "tensaba": "tensar",
    "tensabais": "tensar",
    "tensaban": "tensar",
    "tensabas": "tensar",
    "tensad": "tensar",
    "tensada": "tensar",
    "tensadas": "tensar",
    "tensado": "tensar",
    "tensados": "tensar",
    "tensamos": "tensar",
    "tensan": "tensar",
    "tensando": "tensar",
    "tensara": "tensar",
    "tensarais": "tensar",
    "tensaran": "tensar",
    "tensaras": "tensar",
    "tensare": "tensar",
    "tensareis": "tensar",
    "tensaremos": "tensar",
    "tensaren": "tensar",
    "tensares": "tensar",
    "tensaron": "tensar",
    "tensará": "tensar",
    "tensarán": "tensar",
    "tensarás": "tensar",
    "tensaré": "tensar",
    "tensaréis": "tensar",
    "tensaría": "tensar",
    "tensaríais": "tensar",
    "tensaríamos": "tensar",
    "tensarían": "tensar",
    "tensarías": "tensar",
    "tensas": "tenso",
    "tensase": "tensar",
    "tensaseis": "tensar",
    "tensasen": "tensar",
    "tensases": "tensar",
    "tensaste": "tensar",
    "tensasteis": "tensar",
    "tense": "tensar",
    "tensemos": "tensar",
    "tensen": "tensar",
    "tenses": "tensar",
    "tensioactiva": "tensioactivo",
    "tensioactivas": "tensioactivo",
    "tensioactivos": "tensioactivo",
    "tensionales": "tensional",
    "tensiones": "tensión",
    "tensiómetros": "tensiómetro",
    "tenso": "tensar",
    "tensoactiva": "tensoactivo",
    "tensoactivas": "tensoactivo",
    "tensoactivos": "tensoactivo",
    "tensones": "tensón",
    "tensora": "tensor",
    "tensoras": "tensor",
    "tensores": "tensor",
    "tensoriales": "tensorial",
    "tensos": "tenso",
    "tensábamos": "tensar",
    "tensáis": "tensar",
    "tensáramos": "tensar",
    "tensáremos": "tensar",
    "tensásemos": "tensar",
    "tensé": "tensar",
    "tenséis": "tensar",
    "tensímetros": "tensímetro",
    "tensó": "tensar",
    "tensómetros": "tensómetro",
    "tentaba": "tentar",
    "tentabais": "tentar",
    "tentaban": "tentar",
    "tentabas": "tentar",
    "tentaciones": "tentación",
    "tentaculados": "tentaculado",
    "tentaculares": "tentacular",
    "tentad": "tentar",
    "tentada": "tentar",
    "tentadas": "tentar",
    "tentaderos": "tentadero",
    "tentado": "tentar",
    "tentadora": "tentador",
    "tentadoras": "tentador",
    "tentadores": "tentador",
    "tentados": "tentar",
    "tentaduras": "tentadura",
    "tentalea": "tentalear",
    "tentaleaba": "tentalear",
    "tentaleabais": "tentalear",
    "tentaleaban": "tentalear",
    "tentaleabas": "tentalear",
    "tentalead": "tentalear",
    "tentaleada": "tentalear",
    "tentaleadas": "tentalear",
    "tentaleado": "tentalear",
    "tentaleados": "tentalear",
    "tentaleamos": "tentalear",
    "tentalean": "tentalear",
    "tentaleando": "tentalear",
    "tentaleara": "tentalear",
    "tentalearais": "tentalear",
    "tentalearan": "tentalear",
    "tentalearas": "tentalear",
    "tentaleare": "tentalear",
    "tentaleareis": "tentalear",
    "tentalearemos": "tentalear",
    "tentalearen": "tentalear",
    "tentaleares": "tentalear",
    "tentalearon": "tentalear",
    "tentaleará": "tentalear",
    "tentalearán": "tentalear",
    "tentalearás": "tentalear",
    "tentalearé": "tentalear",
    "tentalearéis": "tentalear",
    "tentalearía": "tentalear",
    "tentalearíais": "tentalear",
    "tentalearíamos": "tentalear",
    "tentalearían": "tentalear",
    "tentalearías": "tentalear",
    "tentaleas": "tentalear",
    "tentalease": "tentalear",
    "tentaleaseis": "tentalear",
    "tentaleasen": "tentalear",
    "tentaleases": "tentalear",
    "tentaleaste": "tentalear",
    "tentaleasteis": "tentalear",
    "tentalee": "tentalear",
    "tentaleemos": "tentalear",
    "tentaleen": "tentalear",
    "tentalees": "tentalear",
    "tentaleo": "tentalear",
    "tentaleábamos": "tentalear",
    "tentaleáis": "tentalear",
    "tentaleáramos": "tentalear",
    "tentaleáremos": "tentalear",
    "tentaleásemos": "tentalear",
    "tentaleé": "tentalear",
    "tentaleéis": "tentalear",
    "tentaleó": "tentalear",
    "tentamos": "tentar",
    "tentando": "tentar",
    "tentara": "tentar",
    "tentarais": "tentar",
    "tentaran": "tentar",
    "tentaras": "tentar",
    "tentare": "tentar",
    "tentareis": "tentar",
    "tentaremos": "tentar",
    "tentaren": "tentar",
    "tentares": "tentar",
    "tentaron": "tentar",
    "tentarujas": "tentaruja",
    "tentará": "tentar",
    "tentarán": "tentar",
    "tentarás": "tentar",
    "tentaré": "tentar",
    "tentaréis": "tentar",
    "tentaría": "tentar",
    "tentaríais": "tentar",
    "tentaríamos": "tentar",
    "tentarían": "tentar",
    "tentarías": "tentar",
    "tentase": "tentar",
    "tentaseis": "tentar",
    "tentasen": "tentar",
    "tentases": "tentar",
    "tentaste": "tentar",
    "tentasteis": "tentar",
    "tentativa": "tentativo",
    "tentativas": "tentativo",
    "tentativos": "tentativo",
    "tentemos": "tentar",
    "tentemozos": "tentemozo",
    "tentempies": "tentempié",
    "tentempiés": "tentempié",
    "tentenelaires": "tentenelaire",
    "tentetiesos": "tentetieso",
    "tentones": "tentón",
    "tentábamos": "tentar",
    "tentáculos": "tentáculo",
    "tentáis": "tentar",
    "tentáramos": "tentar",
    "tentáremos": "tentar",
    "tentásemos": "tentar",
    "tenté": "tentar",
    "tentéis": "tentar",
    "tentó": "tentar",
    "tenues": "tenue",
    "tenuidades": "tenuidad",
    "tenutaria": "tenutario",
    "tenutarias": "tenutario",
    "tenutarios": "tenutario",
    "tenutas": "tenuta",
    "tenáculos": "tenáculo",
    "tenéis": "tener",
    "tenía": "tener",
    "teníais": "tener",
    "teníamos": "tener",
    "tenían": "tener",
    "tenías": "tener",
    "tenífuga": "tenífugo",
    "tenífugas": "tenífugo",
    "tenífugos": "tenífugo",
    "tenística": "tenístico",
    "tenísticas": "tenístico",
    "tenísticos": "tenístico",
    "teobrominas": "teobromina",
    "teocalis": "teocali",
    "teocalíes": "teocali",
    "teocracias": "teocracia",
    "teocrática": "teocrático",
    "teocráticas": "teocrático",
    "teocráticos": "teocrático",
    "teodiceas": "teodicea",
    "teodolitos": "teodolito",
    "teodosiana": "teodosiano",
    "teodosianas": "teodosiano",
    "teodosianos": "teodosiano",
    "teofanías": "teofanía",
    "teofilinametilglucaminas": "teofilinametilglucamina",
    "teofilinas": "teofilina",
    "teofrastácea": "teofrastáceo",
    "teofrastáceas": "teofrastáceo",
    "teofrastáceos": "teofrastáceo",
    "teogonías": "teogonía",
    "teogónica": "teogónico",
    "teogónicas": "teogónico",
    "teogónicos": "teogónico",
    "teologales": "teologal",
    "teologice": "teologizar",
    "teologicemos": "teologizar",
    "teologicen": "teologizar",
    "teologices": "teologizar",
    "teologicé": "teologizar",
    "teologicéis": "teologizar",
    "teologismos": "teologismo",
    "teologiza": "teologizar",
    "teologizaba": "teologizar",
    "teologizabais": "teologizar",
    "teologizaban": "teologizar",
    "teologizabas": "teologizar",
    "teologizad": "teologizar",
    "teologizada": "teologizar",
    "teologizadas": "teologizar",
    "teologizado": "teologizar",
    "teologizados": "teologizar",
    "teologizamos": "teologizar",
    "teologizan": "teologizar",
    "teologizando": "teologizar",
    "teologizara": "teologizar",
    "teologizarais": "teologizar",
    "teologizaran": "teologizar",
    "teologizaras": "teologizar",
    "teologizare": "teologizar",
    "teologizareis": "teologizar",
    "teologizaremos": "teologizar",
    "teologizaren": "teologizar",
    "teologizares": "teologizar",
    "teologizaron": "teologizar",
    "teologizará": "teologizar",
    "teologizarán": "teologizar",
    "teologizarás": "teologizar",
    "teologizaré": "teologizar",
    "teologizaréis": "teologizar",
    "teologizaría": "teologizar",
    "teologizaríais": "teologizar",
    "teologizaríamos": "teologizar",
    "teologizarían": "teologizar",
    "teologizarías": "teologizar",
    "teologizas": "teologizar",
    "teologizase": "teologizar",
    "teologizaseis": "teologizar",
    "teologizasen": "teologizar",
    "teologizases": "teologizar",
    "teologizaste": "teologizar",
    "teologizasteis": "teologizar",
    "teologizo": "teologizar",
    "teologizábamos": "teologizar",
    "teologizáis": "teologizar",
    "teologizáramos": "teologizar",
    "teologizáremos": "teologizar",
    "teologizásemos": "teologizar",
    "teologizó": "teologizar",
    "teologías": "teología",
    "teológica": "teológico",
    "teológicas": "teológico",
    "teológicos": "teológico",
    "teomanías": "teomanía",
    "teomaquias": "teomaquia",
    "teoremas": "teorema",
    "teoremática": "teoremático",
    "teoremáticas": "teoremático",
    "teoremáticos": "teoremático",
    "teorice": "teorizar",
    "teoricemos": "teorizar",
    "teoricen": "teorizar",
    "teorices": "teorizar",
    "teoricé": "teorizar",
    "teoricéis": "teorizar",
    "teoriza": "teorizar",
    "teorizaba": "teorizar",
    "teorizabais": "teorizar",
    "teorizaban": "teorizar",
    "teorizabas": "teorizar",
    "teorizad": "teorizar",
    "teorizada": "teorizar",
    "teorizadas": "teorizar",
    "teorizado": "teorizar",
    "teorizadora": "teorizador",
    "teorizadoras": "teorizador",
    "teorizadores": "teorizador",
    "teorizados": "teorizar",
    "teorizamos": "teorizar",
    "teorizan": "teorizar",
    "teorizando": "teorizar",
    "teorizantes": "teorizante",
    "teorizara": "teorizar",
    "teorizarais": "teorizar",
    "teorizaran": "teorizar",
    "teorizaras": "teorizar",
    "teorizare": "teorizar",
    "teorizareis": "teorizar",
    "teorizaremos": "teorizar",
    "teorizaren": "teorizar",
    "teorizares": "teorizar",
    "teorizaron": "teorizar",
    "teorizará": "teorizar",
    "teorizarán": "teorizar",
    "teorizarás": "teorizar",
    "teorizaré": "teorizar",
    "teorizaréis": "teorizar",
    "teorizaría": "teorizar",
    "teorizaríais": "teorizar",
    "teorizaríamos": "teorizar",
    "teorizarían": "teorizar",
    "teorizarías": "teorizar",
    "teorizas": "teorizar",
    "teorizase": "teorizar",
    "teorizaseis": "teorizar",
    "teorizasen": "teorizar",
    "teorizases": "teorizar",
    "teorizaste": "teorizar",
    "teorizasteis": "teorizar",
    "teorizo": "teorizar",
    "teorizábamos": "teorizar",
    "teorizáis": "teorizar",
    "teorizáramos": "teorizar",
    "teorizáremos": "teorizar",
    "teorizásemos": "teorizar",
    "teorizó": "teorizar",
    "teorías": "teoría",
    "teosa": "teoso",
    "teosas": "teoso",
    "teosofías": "teosofía",
    "teosos": "teoso",
    "teosófica": "teosófico",
    "teosóficas": "teosófico",
    "teosóficos": "teosófico",
    "tepanecas": "tepaneca",
    "tepes": "tepe",
    "tepidarios": "tepidario",
    "tepozanes": "tepozán",
    "tequilas": "tequila",
    "terabytes": "terabyte",
    "teraciclos": "teraciclo",
    "terapeutas": "terapeuta",
    "terapias": "terapia",
    "terapéutica": "terapéutico",
    "terapéuticas": "terapéutico",
    "terapéuticos": "terapéutico",
    "teras": "tera",
    "teratogénica": "teratogénico",
    "teratogénicas": "teratogénico",
    "teratogénicos": "teratogénico",
    "teratologías": "teratología",
    "teratológica": "teratológico",
    "teratológicas": "teratológico",
    "teratológicos": "teratológico",
    "teratomas": "teratoma",
    "teratospermias": "teratospermia",
    "teratógenos": "teratógeno",
    "terbios": "terbio",
    "terca": "terco",
    "tercas": "terco",
    "terceleta": "tercelete",
    "terceletas": "tercelete",
    "terceletes": "tercelete",
    "tercenas": "tercena",
    "tercenistas": "tercenista",
    "tercer": "3",
    "tercera": "tercero",
    "terceras": "tercero",
    "tercerillas": "tercerilla",
    "tercerismos": "tercerismo",
    "tercermundistas": "tercermundista",
    "tercero": "3",
    "tercerolas": "tercerola",
    "terceroles": "tercerol",
    "terceros": "tercero",
    "tercerías": "tercería",
    "tercetos": "terceto",
    "tercia": "tercio",
    "terciaba": "terciar",
    "terciabais": "terciar",
    "terciaban": "terciar",
    "terciabas": "terciar",
    "terciad": "terciar",
    "terciada": "terciar",
    "terciadas": "terciar",
    "terciado": "terciar",
    "terciadora": "terciador",
    "terciadoras": "terciador",
    "terciadores": "terciador",
    "terciados": "terciar",
    "terciamos": "terciar",
    "tercian": "terciar",
    "tercianaria": "tercianario",
    "tercianarias": "tercianario",
    "tercianarios": "tercianario",
    "tercianas": "terciana",
    "terciando": "terciar",
    "tercianelas": "tercianela",
    "terciara": "terciar",
    "terciarais": "terciar",
    "terciaran": "terciar",
    "terciaras": "terciar",
    "terciare": "terciar",
    "terciareis": "terciar",
    "terciaremos": "terciar",
    "terciaren": "terciar",
    "terciares": "terciar",
    "terciaria": "terciario",
    "terciarias": "terciario",
    "terciarios": "terciario",
    "terciaron": "terciar",
    "terciará": "terciar",
    "terciarán": "terciar",
    "terciarás": "terciar",
    "terciaré": "terciar",
    "terciaréis": "terciar",
    "terciaría": "terciar",
    "terciaríais": "terciar",
    "terciaríamos": "terciar",
    "terciarían": "terciar",
    "terciarías": "terciar",
    "tercias": "tercio",
    "terciase": "terciar",
    "terciaseis": "terciar",
    "terciasen": "terciar",
    "terciases": "terciar",
    "terciaste": "terciar",
    "terciasteis": "terciar",
    "terciazones": "terciazón",
    "tercie": "terciar",
    "terciemos": "terciar",
    "tercien": "terciar",
    "tercies": "terciar",
    "tercio": "terciar",
    "terciodécupla": "terciodécuplo",
    "terciodécuplas": "terciodécuplo",
    "terciodécuplos": "terciodécuplo",
    "terciopelada": "terciopelado",
    "terciopeladas": "terciopelado",
    "terciopelados": "terciopelado",
    "terciopelos": "terciopelo",
    "tercios": "tercio",
    "terciábamos": "terciar",
    "terciáis": "terciar",
    "terciáramos": "terciar",
    "terciáremos": "terciar",
    "terciásemos": "terciar",
    "tercié": "terciar",
    "terciéis": "terciar",
    "terció": "terciar",
    "tercos": "terco",
    "terebenos": "terebeno",
    "terebintos": "terebinto",
    "terebintácea": "terebintáceo",
    "terebintáceas": "terebintáceo",
    "terebintáceos": "terebintáceo",
    "terebrantes": "terebrante",
    "terebrátulas": "terebrátula",
    "teredones": "teredón",
    "tereferos": "terefero",
    "tereftalatos": "tereftalato",
    "tereftálica": "tereftálico",
    "tereftálicas": "tereftálico",
    "tereftálicos": "tereftálico",
    "teresas": "teresa",
    "teresiana": "teresiano",
    "teresianas": "teresiano",
    "teresianos": "teresiano",
    "terfenilos": "terfenilo",
    "tergales": "tergal",
    "tergiversa": "tergiversar",
    "tergiversaba": "tergiversar",
    "tergiversabais": "tergiversar",
    "tergiversaban": "tergiversar",
    "tergiversabas": "tergiversar",
    "tergiversables": "tergiversable",
    "tergiversaciones": "tergiversación",
    "tergiversad": "tergiversar",
    "tergiversada": "tergiversar",
    "tergiversadas": "tergiversar",
    "tergiversado": "tergiversar",
    "tergiversadora": "tergiversador",
    "tergiversadoras": "tergiversador",
    "tergiversadores": "tergiversador",
    "tergiversados": "tergiversar",
    "tergiversamos": "tergiversar",
    "tergiversan": "tergiversar",
    "tergiversando": "tergiversar",
    "tergiversara": "tergiversar",
    "tergiversarais": "tergiversar",
    "tergiversaran": "tergiversar",
    "tergiversaras": "tergiversar",
    "tergiversare": "tergiversar",
    "tergiversareis": "tergiversar",
    "tergiversaremos": "tergiversar",
    "tergiversaren": "tergiversar",
    "tergiversares": "tergiversar",
    "tergiversaron": "tergiversar",
    "tergiversará": "tergiversar",
    "tergiversarán": "tergiversar",
    "tergiversarás": "tergiversar",
    "tergiversaré": "tergiversar",
    "tergiversaréis": "tergiversar",
    "tergiversaría": "tergiversar",
    "tergiversaríais": "tergiversar",
    "tergiversaríamos": "tergiversar",
    "tergiversarían": "tergiversar",
    "tergiversarías": "tergiversar",
    "tergiversas": "tergiversar",
    "tergiversase": "tergiversar",
    "tergiversaseis": "tergiversar",
    "tergiversasen": "tergiversar",
    "tergiversases": "tergiversar",
    "tergiversaste": "tergiversar",
    "tergiversasteis": "tergiversar",
    "tergiverse": "tergiversar",
    "tergiversemos": "tergiversar",
    "tergiversen": "tergiversar",
    "tergiverses": "tergiversar",
    "tergiverso": "tergiversar",
    "tergiversábamos": "tergiversar",
    "tergiversáis": "tergiversar",
    "tergiversáramos": "tergiversar",
    "tergiversáremos": "tergiversar",
    "tergiversásemos": "tergiversar",
    "tergiversé": "tergiversar",
    "tergiverséis": "tergiversar",
    "tergiversó": "tergiversar",
    "terios": "terio",
    "terlices": "terliz",
    "termales": "termal",
    "termalgias": "termalgia",
    "termalismos": "termalismo",
    "termas": "terma",
    "termias": "termia",
    "termidores": "termidor",
    "termina": "terminar",
    "terminaba": "terminar",
    "terminabais": "terminar",
    "terminaban": "terminar",
    "terminabas": "terminar",
    "terminables": "terminable",
    "terminachos": "terminacho",
    "terminaciones": "terminación",
    "terminad": "terminar",
    "terminada": "terminar",
    "terminadas": "terminar",
    "terminado": "terminar",
    "terminadora": "terminador",
    "terminadoras": "terminador",
    "terminadores": "terminador",
    "terminados": "terminar",
    "terminajos": "terminajo",
    "terminales": "terminal",
    "terminamos": "terminar",
    "terminan": "terminar",
    "terminando": "terminar",
    "terminantes": "terminante",
    "terminara": "terminar",
    "terminarais": "terminar",
    "terminaran": "terminar",
    "terminaras": "terminar",
    "terminare": "terminar",
    "terminareis": "terminar",
    "terminaremos": "terminar",
    "terminaren": "terminar",
    "terminares": "terminar",
    "terminaron": "terminar",
    "terminará": "terminar",
    "terminarán": "terminar",
    "terminarás": "terminar",
    "terminaré": "terminar",
    "terminaréis": "terminar",
    "terminaría": "terminar",
    "terminaríais": "terminar",
    "terminaríamos": "terminar",
    "terminarían": "terminar",
    "terminarías": "terminar",
    "terminas": "terminar",
    "terminase": "terminar",
    "terminaseis": "terminar",
    "terminasen": "terminar",
    "terminases": "terminar",
    "terminaste": "terminar",
    "terminasteis": "terminar",
    "terminativa": "terminativo",
    "terminativas": "terminativo",
    "terminativos": "terminativo",
    "termine": "terminar",
    "terminemos": "terminar",
    "terminen": "terminar",
    "termines": "terminar",
    "terminismos": "terminismo",
    "terministas": "terminista",
    "termino": "terminar",
    "terminologías": "terminología",
    "terminológica": "terminológico",
    "terminológicas": "terminológico",
    "terminológicos": "terminológico",
    "terminotes": "terminote",
    "terminábamos": "terminar",
    "termináis": "terminar",
    "termináramos": "terminar",
    "termináremos": "terminar",
    "terminásemos": "terminar",
    "terminé": "terminar",
    "terminéis": "terminar",
    "terminó": "terminar",
    "terminóloga": "terminólogo",
    "terminólogas": "terminólogo",
    "terminólogos": "terminólogo",
    "termistancias": "termistancia",
    "termistencias": "termistencia",
    "termistores": "termistor",
    "termitas": "termita",
    "termitera": "termitero",
    "termiteras": "termitero",
    "termiteros": "termitero",
    "termites": "termite",
    "termiónica": "termiónico",
    "termiónicas": "termiónico",
    "termiónicos": "termiónico",
    "termoalgesias": "termoalgesia",
    "termoanalgesias": "termoanalgesia",
    "termobalanzas": "termobalanza",
    "termocatalítica": "termocatalítico",
    "termocatalíticas": "termocatalítico",
    "termocatalíticos": "termocatalítico",
    "termocauterios": "termocauterio",
    "termocauterizaciones": "termocauterización",
    "termoclinas": "termoclina",
    "termocoagulaciones": "termocoagulación",
    "termocompresiones": "termocompresión",
    "termocompresores": "termocompresor",
    "termoconformados": "termoconformado",
    "termoconvecciones": "termoconvección",
    "termoconvectores": "termoconvector",
    "termocroica": "termocroico",
    "termocroicas": "termocroico",
    "termocroicos": "termocroico",
    "termocroísmos": "termocroísmo",
    "termocáustica": "termocáustico",
    "termocáusticas": "termocáustico",
    "termocáusticos": "termocáustico",
    "termodifusiones": "termodifusión",
    "termodilatometrías": "termodilatometría",
    "termodinámica": "termodinámico",
    "termodinámicas": "termodinámico",
    "termodinámicos": "termodinámico",
    "termoelasticidades": "termoelasticidad",
    "termoelectricidades": "termoelectricidad",
    "termoelectrónica": "termoelectrónico",
    "termoelectrónicas": "termoelectrónico",
    "termoelectrónicos": "termoelectrónico",
    "termoelementos": "termoelemento",
    "termoelástica": "termoelástico",
    "termoelásticas": "termoelástico",
    "termoelásticos": "termoelástico",
    "termoeléctrica": "termoeléctrico",
    "termoeléctricas": "termoeléctrico",
    "termoeléctricos": "termoeléctrico",
    "termoespectrometrías": "termoespectrometría",
    "termoestables": "termoestable",
    "termofagias": "termofagia",
    "termofusibles": "termofusible",
    "termofónica": "termofónico",
    "termofónicas": "termofónico",
    "termofónicos": "termofónico",
    "termografías": "termografía",
    "termogramas": "termograma",
    "termogravimetrías": "termogravimetría",
    "termogénica": "termogénico",
    "termogénicas": "termogénico",
    "termogénicos": "termogénico",
    "termohalina": "termohalino",
    "termohalinas": "termohalino",
    "termohalinos": "termohalino",
    "termoiónica": "termoiónico",
    "termoiónicas": "termoiónico",
    "termoiónicos": "termoiónico",
    "termologías": "termología",
    "termoluminiscencias": "termoluminiscencia",
    "termoluminiscentes": "termoluminiscente",
    "termolábiles": "termolábil",
    "termológica": "termológico",
    "termológicas": "termológico",
    "termológicos": "termológico",
    "termomagnetismos": "termomagnetismo",
    "termomagnética": "termomagnético",
    "termomagnéticas": "termomagnético",
    "termomagnéticos": "termomagnético",
    "termomecánica": "termomecánico",
    "termomecánicas": "termomecánico",
    "termomecánicos": "termomecánico",
    "termometrías": "termometría",
    "termometrógrafos": "termometrógrafo",
    "termomultiplicadores": "termomultiplicador",
    "termométrica": "termométrico",
    "termométricas": "termométrico",
    "termométricos": "termométrico",
    "termonebulizadores": "termonebulizador",
    "termonucleares": "termonuclear",
    "termopares": "termopar",
    "termopausas": "termopausa",
    "termopegables": "termopegable",
    "termopilas": "termopila",
    "termoplasticidades": "termoplasticidad",
    "termoplástica": "termoplástico",
    "termoplásticas": "termoplástico",
    "termoplásticos": "termoplástico",
    "termopropulsiones": "termopropulsión",
    "termoquímica": "termoquímico",
    "termoquímicas": "termoquímico",
    "termoquímicos": "termoquímico",
    "termorregeneraciones": "termorregeneración",
    "termorregulaciones": "termorregulación",
    "termorreguladora": "termorregulador",
    "termorreguladoras": "termorregulador",
    "termorreguladores": "termorregulador",
    "termorretracciones": "termorretracción",
    "termos": "termo",
    "termoscopios": "termoscopio",
    "termoscópica": "termoscópico",
    "termoscópicas": "termoscópico",
    "termoscópicos": "termoscópico",
    "termosecuencias": "termosecuencia",
    "termosellados": "termosellado",
    "termosensibles": "termosensible",
    "termosferas": "termosfera",
    "termosifones": "termosifón",
    "termosolares": "termosolar",
    "termosoldables": "termosoldable",
    "termostables": "termostable",
    "termostatos": "termostato",
    "termostática": "termostático",
    "termostáticas": "termostático",
    "termostáticos": "termostático",
    "termotecnias": "termotecnia",
    "termoterapias": "termoterapia",
    "termoventiladora": "termoventilador",
    "termoventiladoras": "termoventilador",
    "termoventiladores": "termoventilador",
    "termófila": "termófilo",
    "termófilas": "termófilo",
    "termófilos": "termófilo",
    "termófonos": "termófono",
    "termógena": "termógeno",
    "termógenas": "termógeno",
    "termógenos": "termógeno",
    "termógrafos": "termógrafo",
    "termómetros": "termómetro",
    "termóstatos": "termóstato",
    "ternaria": "ternario",
    "ternarias": "ternario",
    "ternarios": "ternario",
    "ternas": "terna",
    "ternascos": "ternasco",
    "ternejales": "ternejal",
    "ternejona": "ternejón",
    "ternejonas": "ternejón",
    "ternejones": "ternejón",
    "ternera": "ternero",
    "terneras": "ternero",
    "ternerona": "ternerón",
    "terneronas": "ternerón",
    "ternerones": "ternerón",
    "terneros": "ternero",
    "ternes": "terne",
    "ternezas": "terneza",
    "ternillas": "ternilla",
    "ternillosa": "ternilloso",
    "ternillosas": "ternilloso",
    "ternillosos": "ternilloso",
    "ternos": "terno",
    "ternuras": "ternura",
    "terpenos": "terpeno",
    "terpinas": "terpina",
    "terpineoles": "terpineol",
    "terpinolenos": "terpinoleno",
    "terpolímera": "terpolímero",
    "terpolímeras": "terpolímero",
    "terpolímeros": "terpolímero",
    "terpénica": "terpénico",
    "terpénicas": "terpénico",
    "terpénicos": "terpénico",
    "terquea": "terquear",
    "terqueaba": "terquear",
    "terqueabais": "terquear",
    "terqueaban": "terquear",
    "terqueabas": "terquear",
    "terquead": "terquear",
    "terqueada": "terquear",
    "terqueadas": "terquear",
    "terqueado": "terquear",
    "terqueados": "terquear",
    "terqueamos": "terquear",
    "terquean": "terquear",
    "terqueando": "terquear",
    "terqueara": "terquear",
    "terquearais": "terquear",
    "terquearan": "terquear",
    "terquearas": "terquear",
    "terqueare": "terquear",
    "terqueareis": "terquear",
    "terquearemos": "terquear",
    "terquearen": "terquear",
    "terqueares": "terquear",
    "terquearon": "terquear",
    "terqueará": "terquear",
    "terquearán": "terquear",
    "terquearás": "terquear",
    "terquearé": "terquear",
    "terquearéis": "terquear",
    "terquearía": "terquear",
    "terquearíais": "terquear",
    "terquearíamos": "terquear",
    "terquearían": "terquear",
    "terquearías": "terquear",
    "terqueas": "terquear",
    "terquease": "terquear",
    "terqueaseis": "terquear",
    "terqueasen": "terquear",
    "terqueases": "terquear",
    "terqueaste": "terquear",
    "terqueasteis": "terquear",
    "terquedades": "terquedad",
    "terquee": "terquear",
    "terqueemos": "terquear",
    "terqueen": "terquear",
    "terquees": "terquear",
    "terqueo": "terquear",
    "terquerías": "terquería",
    "terquezas": "terqueza",
    "terqueábamos": "terquear",
    "terqueáis": "terquear",
    "terqueáramos": "terquear",
    "terqueáremos": "terquear",
    "terqueásemos": "terquear",
    "terqueé": "terquear",
    "terqueéis": "terquear",
    "terqueó": "terquear",
    "terracotas": "terracota",
    "terrados": "terrado",
    "terrajas": "terraja",
    "terrajes": "terraje",
    "terrales": "terral",
    "terranovas": "terranova",
    "terraplena": "terraplenar",
    "terraplenaba": "terraplenar",
    "terraplenabais": "terraplenar",
    "terraplenaban": "terraplenar",
    "terraplenabas": "terraplenar",
    "terraplenad": "terraplenar",
    "terraplenada": "terraplenar",
    "terraplenadas": "terraplenar",
    "terraplenado": "terraplenar",
    "terraplenados": "terraplenar",
    "terraplenamos": "terraplenar",
    "terraplenan": "terraplenar",
    "terraplenando": "terraplenar",
    "terraplenara": "terraplenar",
    "terraplenarais": "terraplenar",
    "terraplenaran": "terraplenar",
    "terraplenaras": "terraplenar",
    "terraplenare": "terraplenar",
    "terraplenareis": "terraplenar",
    "terraplenaremos": "terraplenar",
    "terraplenaren": "terraplenar",
    "terraplenares": "terraplenar",
    "terraplenaron": "terraplenar",
    "terraplenará": "terraplenar",
    "terraplenarán": "terraplenar",
    "terraplenarás": "terraplenar",
    "terraplenaré": "terraplenar",
    "terraplenaréis": "terraplenar",
    "terraplenaría": "terraplenar",
    "terraplenaríais": "terraplenar",
    "terraplenaríamos": "terraplenar",
    "terraplenarían": "terraplenar",
    "terraplenarías": "terraplenar",
    "terraplenas": "terraplenar",
    "terraplenase": "terraplenar",
    "terraplenaseis": "terraplenar",
    "terraplenasen": "terraplenar",
    "terraplenases": "terraplenar",
    "terraplenaste": "terraplenar",
    "terraplenasteis": "terraplenar",
    "terraplene": "terraplenar",
    "terraplenemos": "terraplenar",
    "terraplenen": "terraplenar",
    "terraplenes": "terraplenar",
    "terrapleno": "terraplenar",
    "terraplenábamos": "terraplenar",
    "terraplenáis": "terraplenar",
    "terraplenáramos": "terraplenar",
    "terraplenáremos": "terraplenar",
    "terraplenásemos": "terraplenar",
    "terraplené": "terraplenar",
    "terraplenéis": "terraplenar",
    "terraplenó": "terraplenar",
    "terrarios": "terrario",
    "terrariumes": "terrarium",
    "terrariums": "terrarium",
    "terratenientes": "terrateniente",
    "terrazas": "terraza",
    "terrazgos": "terrazgo",
    "terrazguera": "terrazguero",
    "terrazgueras": "terrazguero",
    "terrazgueros": "terrazguero",
    "terrazos": "terrazo",
    "terrea": "terrear",
    "terreaba": "terrear",
    "terreabais": "terrear",
    "terreaban": "terrear",
    "terreabas": "terrear",
    "terread": "terrear",
    "terreada": "terrear",
    "terreadas": "terrear",
    "terreado": "terrear",
    "terreados": "terrear",
    "terreamos": "terrear",
    "terrean": "terrear",
    "terreando": "terrear",
    "terreara": "terrear",
    "terrearais": "terrear",
    "terrearan": "terrear",
    "terrearas": "terrear",
    "terreare": "terrear",
    "terreareis": "terrear",
    "terrearemos": "terrear",
    "terrearen": "terrear",
    "terreares": "terrear",
    "terrearon": "terrear",
    "terreará": "terrear",
    "terrearán": "terrear",
    "terrearás": "terrear",
    "terrearé": "terrear",
    "terrearéis": "terrear",
    "terrearía": "terrear",
    "terrearíais": "terrear",
    "terrearíamos": "terrear",
    "terrearían": "terrear",
    "terrearías": "terrear",
    "terreas": "terrear",
    "terrease": "terrear",
    "terreaseis": "terrear",
    "terreasen": "terrear",
    "terreases": "terrear",
    "terreaste": "terrear",
    "terreasteis": "terrear",
    "terree": "terrear",
    "terreemos": "terrear",
    "terreen": "terrear",
    "terrees": "terrear",
    "terregosa": "terregoso",
    "terregosas": "terregoso",
    "terregosos": "terregoso",
    "terremotos": "terremoto",
    "terrena": "terreno",
    "terrenales": "terrenal",
    "terrenas": "terreno",
    "terrenos": "terreno",
    "terreo": "terrear",
    "terrera": "terrero",
    "terreras": "terrero",
    "terreros": "terrero",
    "terrestres": "terrestre",
    "terrezuelas": "terrezuela",
    "terreábamos": "terrear",
    "terreáis": "terrear",
    "terreáramos": "terrear",
    "terreáremos": "terrear",
    "terreásemos": "terrear",
    "terreé": "terrear",
    "terreéis": "terrear",
    "terreó": "terrear",
    "terribles": "terrible",
    "terriers": "terrier",
    "terrina": "terrino",
    "terrinas": "terrino",
    "terrinos": "terrino",
    "territoriales": "territorial",
    "territorialidades": "territorialidad",
    "territorializaciones": "territorialización",
    "territorios": "territorio",
    "terriza": "terrizo",
    "terrizas": "terrizo",
    "terrizos": "terrizo",
    "terromonteros": "terromontero",
    "terronazos": "terronazo",
    "terrones": "terrón",
    "terrores": "terror",
    "terrorismos": "terrorismo",
    "terroristas": "terrorista",
    "terrorífica": "terrorífico",
    "terroríficas": "terrorífico",
    "terroríficos": "terrorífico",
    "terrosa": "terroso",
    "terrosas": "terroso",
    "terrosidades": "terrosidad",
    "terrosos": "terroso",
    "terruños": "terruño",
    "terráquea": "terráqueo",
    "terráqueas": "terráqueo",
    "terráqueos": "terráqueo",
    "terrícolas": "terrícola",
    "terrífica": "terrífico",
    "terríficas": "terrífico",
    "terríficos": "terrífico",
    "terrígena": "terrígeno",
    "terrígenas": "terrígeno",
    "terrígenos": "terrígeno",
    "tersa": "terso",
    "tersaba": "tersar",
    "tersabais": "tersar",
    "tersaban": "tersar",
    "tersabas": "tersar",
    "tersad": "tersar",
    "tersada": "tersar",
    "tersadas": "tersar",
    "tersado": "tersar",
    "tersados": "tersar",
    "tersamos": "tersar",
    "tersan": "tersar",
    "tersando": "tersar",
    "tersara": "tersar",
    "tersarais": "tersar",
    "tersaran": "tersar",
    "tersaras": "tersar",
    "tersare": "tersar",
    "tersareis": "tersar",
    "tersaremos": "tersar",
    "tersaren": "tersar",
    "tersares": "tersar",
    "tersaron": "tersar",
    "tersará": "tersar",
    "tersarán": "tersar",
    "tersarás": "tersar",
    "tersaré": "tersar",
    "tersaréis": "tersar",
    "tersaría": "tersar",
    "tersaríais": "tersar",
    "tersaríamos": "tersar",
    "tersarían": "tersar",
    "tersarías": "tersar",
    "tersas": "terso",
    "tersase": "tersar",
    "tersaseis": "tersar",
    "tersasen": "tersar",
    "tersases": "tersar",
    "tersaste": "tersar",
    "tersasteis": "tersar",
    "terse": "tersar",
    "tersemos": "tersar",
    "tersen": "tersar",
    "terses": "tersar",
    "tersidades": "tersidad",
    "terso": "tersar",
    "tersos": "terso",
    "tersuras": "tersura",
    "tersábamos": "tersar",
    "tersáis": "tersar",
    "tersáramos": "tersar",
    "tersáremos": "tersar",
    "tersásemos": "tersar",
    "tersé": "tersar",
    "terséis": "tersar",
    "tersó": "tersar",
    "tertulia": "tertulio",
    "tertuliaba": "tertuliar",
    "tertuliabais": "tertuliar",
    "tertuliaban": "tertuliar",
    "tertuliabas": "tertuliar",
    "tertuliad": "tertuliar",
    "tertuliada": "tertuliar",
    "tertuliadas": "tertuliar",
    "tertuliado": "tertuliar",
    "tertuliados": "tertuliar",
    "tertuliamos": "tertuliar",
    "tertulian": "tertuliar",
    "tertuliana": "tertuliano",
    "tertulianas": "tertuliano",
    "tertuliando": "tertuliar",
    "tertulianos": "tertuliano",
    "tertuliantes": "tertuliante",
    "tertuliara": "tertuliar",
    "tertuliarais": "tertuliar",
    "tertuliaran": "tertuliar",
    "tertuliaras": "tertuliar",
    "tertuliare": "tertuliar",
    "tertuliareis": "tertuliar",
    "tertuliaremos": "tertuliar",
    "tertuliaren": "tertuliar",
    "tertuliares": "tertuliar",
    "tertuliaron": "tertuliar",
    "tertuliará": "tertuliar",
    "tertuliarán": "tertuliar",
    "tertuliarás": "tertuliar",
    "tertuliaré": "tertuliar",
    "tertuliaréis": "tertuliar",
    "tertuliaría": "tertuliar",
    "tertuliaríais": "tertuliar",
    "tertuliaríamos": "tertuliar",
    "tertuliarían": "tertuliar",
    "tertuliarías": "tertuliar",
    "tertulias": "tertulio",
    "tertuliase": "tertuliar",
    "tertuliaseis": "tertuliar",
    "tertuliasen": "tertuliar",
    "tertuliases": "tertuliar",
    "tertuliaste": "tertuliar",
    "tertuliasteis": "tertuliar",
    "tertulie": "tertuliar",
    "tertuliemos": "tertuliar",
    "tertulien": "tertuliar",
    "tertulies": "tertuliar",
    "tertulio": "tertuliar",
    "tertulios": "tertulio",
    "tertuliábamos": "tertuliar",
    "tertuliáis": "tertuliar",
    "tertuliáramos": "tertuliar",
    "tertuliáremos": "tertuliar",
    "tertuliásemos": "tertuliar",
    "tertulié": "tertuliar",
    "tertuliéis": "tertuliar",
    "tertulió": "tertuliar",
    "terylenes": "terylene",
    "terzuelas": "terzuela",
    "terzuelos": "terzuelo",
    "terédines": "terédine",
    "terófita": "terófito",
    "terófitas": "terófito",
    "terófitos": "terófito",
    "tesa": "teso",
    "tesaba": "tesar",
    "tesabais": "tesar",
    "tesaban": "tesar",
    "tesabas": "tesar",
    "tesad": "tesar",
    "tesada": "tesar",
    "tesadas": "tesar",
    "tesado": "tesar",
    "tesados": "tesar",
    "tesalia": "tesalio",
    "tesaliana": "tesaliano",
    "tesalianas": "tesaliano",
    "tesalianos": "tesaliano",
    "tesalias": "tesalio",
    "tesalienses": "tesaliense",
    "tesalios": "tesalio",
    "tesamos": "tesar",
    "tesan": "tesar",
    "tesando": "tesar",
    "tesara": "tesar",
    "tesarais": "tesar",
    "tesaran": "tesar",
    "tesaras": "tesar",
    "tesare": "tesar",
    "tesareis": "tesar",
    "tesaremos": "tesar",
    "tesaren": "tesar",
    "tesares": "tesar",
    "tesaron": "tesar",
    "tesará": "tesar",
    "tesarán": "tesar",
    "tesarás": "tesar",
    "tesaré": "tesar",
    "tesaréis": "tesar",
    "tesaría": "tesar",
    "tesaríais": "tesar",
    "tesaríamos": "tesar",
    "tesarían": "tesar",
    "tesarías": "tesar",
    "tesas": "teso",
    "tesase": "tesar",
    "tesaseis": "tesar",
    "tesasen": "tesar",
    "tesases": "tesar",
    "tesaste": "tesar",
    "tesasteis": "tesar",
    "tesauros": "tesauro",
    "tese": "tesar",
    "teselada": "teselado",
    "teseladas": "teselado",
    "teselados": "teselado",
    "teselas": "tesela",
    "tesemos": "tesar",
    "tesen": "tesar",
    "teses": "tesar",
    "tesinas": "tesina",
    "tesituras": "tesitura",
    "teslas": "tesla",
    "teso": "tesar",
    "tesonera": "tesonero",
    "tesoneras": "tesonero",
    "tesoneros": "tesonero",
    "tesonerías": "tesonería",
    "tesones": "tesón",
    "tesorera": "tesorero",
    "tesoreras": "tesorero",
    "tesoreros": "tesorero",
    "tesorerías": "tesorería",
    "tesoros": "tesoro",
    "tesos": "teso",
    "testa": "testar",
    "testaba": "testar",
    "testabais": "testar",
    "testaban": "testar",
    "testabas": "testar",
    "testaciones": "testación",
    "testad": "testar",
    "testada": "testar",
    "testadas": "testar",
    "testado": "testar",
    "testadora": "testador",
    "testadoras": "testador",
    "testadores": "testador",
    "testados": "testar",
    "testaduras": "testadura",
    "testaferros": "testaferro",
    "testamentaria": "testamentario",
    "testamentarias": "testamentario",
    "testamentarios": "testamentario",
    "testamentarías": "testamentaría",
    "testamentificaciones": "testamentificación",
    "testamentos": "testamento",
    "testamos": "testar",
    "testan": "testar",
    "testando": "testar",
    "testara": "testar",
    "testaradas": "testarada",
    "testarais": "testar",
    "testaran": "testar",
    "testaras": "testar",
    "testarazos": "testarazo",
    "testare": "testar",
    "testareis": "testar",
    "testaremos": "testar",
    "testaren": "testar",
    "testares": "testar",
    "testaron": "testar",
    "testarrona": "testarrón",
    "testarronas": "testarrón",
    "testarrones": "testarrón",
    "testaruda": "testarudo",
    "testarudas": "testarudo",
    "testarudeces": "testarudez",
    "testarudos": "testarudo",
    "testará": "testar",
    "testarán": "testar",
    "testarás": "testar",
    "testaré": "testar",
    "testaréis": "testar",
    "testaría": "testar",
    "testaríais": "testar",
    "testaríamos": "testar",
    "testarían": "testar",
    "testarías": "testar",
    "testas": "testar",
    "testase": "testar",
    "testaseis": "testar",
    "testasen": "testar",
    "testases": "testar",
    "testaste": "testar",
    "testasteis": "testar",
    "teste": "testar",
    "testemos": "testar",
    "testen": "testar",
    "testeos": "testeo",
    "testeradas": "testerada",
    "testeras": "testera",
    "testeros": "testero",
    "testes": "testar",
    "testiculares": "testicular",
    "testifica": "testificar",
    "testificaba": "testificar",
    "testificabais": "testificar",
    "testificaban": "testificar",
    "testificabas": "testificar",
    "testificaciones": "testificación",
    "testificad": "testificar",
    "testificada": "testificar",
    "testificadas": "testificar",
    "testificado": "testificar",
    "testificadora": "testificador",
    "testificadoras": "testificador",
    "testificadores": "testificador",
    "testificados": "testificar",
    "testificales": "testifical",
    "testificamos": "testificar",
    "testifican": "testificar",
    "testificando": "testificar",
    "testificantes": "testificante",
    "testificara": "testificar",
    "testificarais": "testificar",
    "testificaran": "testificar",
    "testificaras": "testificar",
    "testificare": "testificar",
    "testificareis": "testificar",
    "testificaremos": "testificar",
    "testificaren": "testificar",
    "testificares": "testificar",
    "testificaron": "testificar",
    "testificará": "testificar",
    "testificarán": "testificar",
    "testificarás": "testificar",
    "testificaré": "testificar",
    "testificaréis": "testificar",
    "testificaría": "testificar",
    "testificaríais": "testificar",
    "testificaríamos": "testificar",
    "testificarían": "testificar",
    "testificarías": "testificar",
    "testificas": "testificar",
    "testificase": "testificar",
    "testificaseis": "testificar",
    "testificasen": "testificar",
    "testificases": "testificar",
    "testificaste": "testificar",
    "testificasteis": "testificar",
    "testificativa": "testificativo",
    "testificativas": "testificativo",
    "testificativos": "testificativo",
    "testifico": "testificar",
    "testificábamos": "testificar",
    "testificáis": "testificar",
    "testificáramos": "testificar",
    "testificáremos": "testificar",
    "testificásemos": "testificar",
    "testificó": "testificar",
    "testifique": "testificar",
    "testifiquemos": "testificar",
    "testifiquen": "testificar",
    "testifiques": "testificar",
    "testifiqué": "testificar",
    "testifiquéis": "testificar",
    "testigos": "testigo",
    "testimonia": "testimoniar",
    "testimoniaba": "testimoniar",
    "testimoniabais": "testimoniar",
    "testimoniaban": "testimoniar",
    "testimoniabas": "testimoniar",
    "testimoniad": "testimoniar",
    "testimoniada": "testimoniar",
    "testimoniadas": "testimoniar",
    "testimoniado": "testimoniar",
    "testimoniados": "testimoniar",
    "testimoniales": "testimonial",
    "testimonials": "testimonial",
    "testimoniamos": "testimoniar",
    "testimonian": "testimoniar",
    "testimoniando": "testimoniar",
    "testimoniara": "testimoniar",
    "testimoniarais": "testimoniar",
    "testimoniaran": "testimoniar",
    "testimoniaras": "testimoniar",
    "testimoniare": "testimoniar",
    "testimoniareis": "testimoniar",
    "testimoniaremos": "testimoniar",
    "testimoniaren": "testimoniar",
    "testimoniares": "testimoniar",
    "testimoniaron": "testimoniar",
    "testimoniará": "testimoniar",
    "testimoniarán": "testimoniar",
    "testimoniarás": "testimoniar",
    "testimoniaré": "testimoniar",
    "testimoniaréis": "testimoniar",
    "testimoniaría": "testimoniar",
    "testimoniaríais": "testimoniar",
    "testimoniaríamos": "testimoniar",
    "testimoniarían": "testimoniar",
    "testimoniarías": "testimoniar",
    "testimonias": "testimoniar",
    "testimoniase": "testimoniar",
    "testimoniaseis": "testimoniar",
    "testimoniasen": "testimoniar",
    "testimoniases": "testimoniar",
    "testimoniaste": "testimoniar",
    "testimoniasteis": "testimoniar",
    "testimonie": "testimoniar",
    "testimoniemos": "testimoniar",
    "testimonien": "testimoniar",
    "testimoniera": "testimoniero",
    "testimonieras": "testimoniero",
    "testimonieros": "testimoniero",
    "testimonies": "testimoniar",
    "testimonio": "testimoniar",
    "testimonios": "testimonio",
    "testimoniábamos": "testimoniar",
    "testimoniáis": "testimoniar",
    "testimoniáramos": "testimoniar",
    "testimoniáremos": "testimoniar",
    "testimoniásemos": "testimoniar",
    "testimonié": "testimoniar",
    "testimoniéis": "testimoniar",
    "testimonió": "testimoniar",
    "testimoñera": "testimoñero",
    "testimoñeras": "testimoñero",
    "testimoñeros": "testimoñero",
    "testo": "testar",
    "testosteronas": "testosterona",
    "tests": "test",
    "testuces": "testuz",
    "testudos": "testudo",
    "testudínea": "testudíneo",
    "testudíneas": "testudíneo",
    "testudíneos": "testudíneo",
    "testudínidos": "testudínido",
    "testábamos": "testar",
    "testácea": "testáceo",
    "testáceas": "testáceo",
    "testáceos": "testáceo",
    "testáis": "testar",
    "testáramos": "testar",
    "testáremos": "testar",
    "testásemos": "testar",
    "testé": "testar",
    "testéis": "testar",
    "testículos": "testículo",
    "testó": "testar",
    "tesuras": "tesura",
    "tesábamos": "tesar",
    "tesáis": "tesar",
    "tesálica": "tesálico",
    "tesálicas": "tesálico",
    "tesálicos": "tesálico",
    "tesáramos": "tesar",
    "tesáremos": "tesar",
    "tesásemos": "tesar",
    "tesé": "tesar",
    "teséis": "tesar",
    "tesó": "tesar",
    "tetadas": "tetada",
    "tetanias": "tetania",
    "tetartoedros": "tetartoedro",
    "tetartoedrías": "tetartoedría",
    "tetartoides": "tetartoide",
    "tetartoédrica": "tetartoédrico",
    "tetartoédricas": "tetartoédrico",
    "tetartoédricos": "tetartoédrico",
    "tetartóidica": "tetartóidico",
    "tetartóidicas": "tetartóidico",
    "tetartóidicos": "tetartóidico",
    "tetas": "teta",
    "teteras": "tetera",
    "tetillas": "tetilla",
    "tetinas": "tetina",
    "tetonas": "tetona",
    "tetones": "tetón",
    "tetraacetatos": "tetraacetato",
    "tetraacetilacetonatos": "tetraacetilacetonato",
    "tetraboratos": "tetraborato",
    "tetrabranquios": "tetrabranquio",
    "tetrabromoetilenos": "tetrabromoetileno",
    "tetrabromuros": "tetrabromuro",
    "tetrabutilestaños": "tetrabutilestaño",
    "tetracampeona": "tetracampeón",
    "tetracampeonas": "tetracampeón",
    "tetracampeones": "tetracampeón",
    "tetracarboxibutanos": "tetracarboxibutano",
    "tetracenos": "tetraceno",
    "tetracianoetilenos": "tetracianoetileno",
    "tetraciclinas": "tetraciclina",
    "tetracinas": "tetracina",
    "tetraclorobencenos": "tetraclorobenceno",
    "tetraclorodifluoetanos": "tetraclorodifluoetano",
    "tetracloroetanos": "tetracloroetano",
    "tetraclorofenatos": "tetraclorofenato",
    "tetraclorofenoles": "tetraclorofenol",
    "tetracloroftálica": "tetracloroftálico",
    "tetracloroftálicas": "tetracloroftálico",
    "tetracloroftálicos": "tetracloroftálico",
    "tetracloruros": "tetracloruro",
    "tetracordios": "tetracordio",
    "tetracosanos": "tetracosano",
    "tetracíclica": "tetracíclico",
    "tetracíclicas": "tetracíclico",
    "tetracíclicos": "tetracíclico",
    "tetracúspides": "tetracúspide",
    "tetradas": "tetrada",
    "tetradecanos": "tetradecano",
    "tetradecenos": "tetradeceno",
    "tetradecilaminas": "tetradecilamina",
    "tetradimitas": "tetradimita",
    "tetradracmas": "tetradracma",
    "tetraedritas": "tetraedrita",
    "tetraedros": "tetraedro",
    "tetraetilenpentaminas": "tetraetilenpentamina",
    "tetraetilos": "tetraetilo",
    "tetraetilplomos": "tetraetilplomo",
    "tetraexaedros": "tetraexaedro",
    "tetrafenilestaños": "tetrafenilestaño",
    "tetrafenilsilanos": "tetrafenilsilano",
    "tetrafilares": "tetrafilar",
    "tetrafluoetilenos": "tetrafluoetileno",
    "tetrafluometanos": "tetrafluometano",
    "tetrafluorhidracinas": "tetrafluorhidracina",
    "tetrafluoruros": "tetrafluoruro",
    "tetrafosfatos": "tetrafosfato",
    "tetrafásica": "tetrafásico",
    "tetrafásicas": "tetrafásico",
    "tetrafásicos": "tetrafásico",
    "tetragonales": "tetragonal",
    "tetragramas": "tetragrama",
    "tetragrámatones": "tetragrámaton",
    "tetrahedritas": "tetrahedrita",
    "tetrahexaedros": "tetrahexaedro",
    "tetrahidroftálica": "tetrahidroftálico",
    "tetrahidroftálicas": "tetrahidroftálico",
    "tetrahidroftálicos": "tetrahidroftálico",
    "tetrahidrofuranos": "tetrahidrofurano",
    "tetrahidronaftalenos": "tetrahidronaftaleno",
    "tetrahidropiranmetanoles": "tetrahidropiranmetanol",
    "tetrahidropiridinas": "tetrahidropiridina",
    "tetrahidrotiofenos": "tetrahidrotiofeno",
    "tetrahidroxietiletilendiaminas": "tetrahidroxietiletilendiamina",
    "tetralitas": "tetralita",
    "tetralogías": "tetralogía",
    "tetramera": "tetramero",
    "tetrameras": "tetramero",
    "tetrameros": "tetramero",
    "tetrametilbutandiaminas": "tetrametilbutandiamina",
    "tetrametildiaminobenzhidroles": "tetrametildiaminobenzhidrol",
    "tetrametildiaminobenzofenonas": "tetrametildiaminobenzofenona",
    "tetrametildiaminodifenilmetanos": "tetrametildiaminodifenilmetano",
    "tetrametildiaminodifenilsulfonas": "tetrametildiaminodifenilsulfona",
    "tetrametilendiaminas": "tetrametilendiamina",
    "tetrametiletilendiaminas": "tetrametiletilendiamina",
    "tetramorfa": "tetramorfo",
    "tetramorfas": "tetramorfo",
    "tetramorfos": "tetramorfo",
    "tetramotores": "tetramotor",
    "tetranitratos": "tetranitrato",
    "tetranitroanilinas": "tetranitroanilina",
    "tetranitrometanos": "tetranitrometano",
    "tetraodontiformes": "tetraodontiforme",
    "tetraplejias": "tetraplejia",
    "tetrapléjicos": "tetrapléjico",
    "tetrapolares": "tetrapolar",
    "tetrapolos": "tetrapolo",
    "tetrapropenilsuccínica": "tetrapropenilsuccínico",
    "tetrapropenilsuccínicas": "tetrapropenilsuccínico",
    "tetrapropenilsuccínicos": "tetrapropenilsuccínico",
    "tetrapropilenos": "tetrapropileno",
    "tetraquisexaedros": "tetraquisexaedro",
    "tetraquishexaedros": "tetraquishexaedro",
    "tetrarcas": "tetrarca",
    "tetrarquías": "tetrarquía",
    "tetrasulfuros": "tetrasulfuro",
    "tetrasílaba": "tetrasílabo",
    "tetrasílabas": "tetrasílabo",
    "tetrasílabos": "tetrasílabo",
    "tetravalentes": "tetravalente",
    "tetrayodoetilenos": "tetrayodoetileno",
    "tetraédrica": "tetraédrico",
    "tetraédricas": "tetraédrico",
    "tetraédricos": "tetraédrico",
    "tetraónidos": "tetraónido",
    "tetriles": "tetril",
    "tetrilos": "tetrilo",
    "tetrodos": "tetrodo",
    "tetrosas": "tetrosa",
    "tetrágona": "tetrágono",
    "tetrágonas": "tetrágono",
    "tetrágonos": "tetrágono",
    "tetrámera": "tetrámero",
    "tetrámeras": "tetrámero",
    "tetrámeros": "tetrámero",
    "tetrápodos": "tetrápodo",
    "tetrástica": "tetrástico",
    "tetrásticas": "tetrástico",
    "tetrásticos": "tetrástico",
    "tetrástila": "tetrástilo",
    "tetrástilas": "tetrástilo",
    "tetrástilos": "tetrástilo",
    "tetrástrofa": "tetrástrofo",
    "tetrástrofas": "tetrástrofo",
    "tetrástrofos": "tetrástrofo",
    "tetróxidos": "tetróxido",
    "tetuaníes": "tetuaní",
    "tetuanís": "tetuaní",
    "tetuda": "tetudo",
    "tetudas": "tetudo",
    "tetudos": "tetudo",
    "tetánica": "tetánico",
    "tetánicas": "tetánico",
    "tetánicos": "tetánico",
    "teucalis": "teucali",
    "teucalíes": "teucali",
    "teucra": "teucro",
    "teucras": "teucro",
    "teucrios": "teucrio",
    "teucros": "teucro",
    "teutona": "teutón",
    "teutonas": "teutón",
    "teutones": "teutón",
    "teutóna": "teutón",
    "teutónas": "teutón",
    "teutónes": "teutón",
    "teutónica": "teutónico",
    "teutónicas": "teutónico",
    "teutónicos": "teutónico",
    "textiles": "textil",
    "textoria": "textorio",
    "textorias": "textorio",
    "textorios": "textorio",
    "textos": "texto",
    "textuales": "textual",
    "textualidades": "textualidad",
    "textualistas": "textualista",
    "texturas": "textura",
    "texturizaciones": "texturización",
    "texturizados": "texturizado",
    "teácea": "teáceo",
    "teáceas": "teáceo",
    "teáceos": "teáceo",
    "teínas": "teína",
    "teísmos": "teísmo",
    "teístas": "teísta",
    "teística": "teístico",
    "teísticas": "teístico",
    "teísticos": "teístico",
    "teñibles": "teñible",
    "teñid": "teñir",
    "teñida": "teñir",
    "teñidas": "teñir",
    "teñido": "teñir",
    "teñidos": "teñir",
    "teñiduras": "teñidura",
    "teñimos": "teñir",
    "teñiremos": "teñir",
    "teñirá": "teñir",
    "teñirán": "teñir",
    "teñirás": "teñir",
    "teñiré": "teñir",
    "teñiréis": "teñir",
    "teñiría": "teñir",
    "teñiríais": "teñir",
    "teñiríamos": "teñir",
    "teñirían": "teñir",
    "teñirías": "teñir",
    "teñiste": "teñir",
    "teñisteis": "teñir",
    "teñí": "teñir",
    "teñía": "teñir",
    "teñíais": "teñir",
    "teñíamos": "teñir",
    "teñían": "teñir",
    "teñías": "teñir",
    "teñís": "teñir",
    "teóloga": "teólogo",
    "teólogas": "teólogo",
    "teólogos": "teólogo",
    "teórica": "teórico",
    "teóricas": "teórico",
    "teóricos": "teórico",
    "teósofa": "teósofo",
    "teósofas": "teósofo",
    "teósofos": "teósofo",
    "teúrgias": "teúrgia",
    "teúrgica": "teúrgico",
    "teúrgicas": "teúrgico",
    "teúrgicos": "teúrgico",
    "teúrgos": "teúrgo",
    "thailandesa": "tailandés",
    "thailandesas": "tailandés",
    "thailandeses": "tailandés",
    "thailandés": "tailandés",
    "thais": "thai",
    "thenarditas": "thenardita",
    "thetas": "theta",
    "thomsonitas": "thomsonita",
    "thrillers": "thriller",
    "thulitas": "thulita",
    "tiacinas": "tiacina",
    "tialinas": "tialina",
    "tialismos": "tialismo",
    "tiaminas": "tiamina",
    "tiaras": "tiara",
    "tiazinas": "tiazina",
    "tiazoles": "tiazol",
    "tiazosulfonas": "tiazosulfona",
    "tiberina": "tiberino",
    "tiberinas": "tiberino",
    "tiberinos": "tiberino",
    "tiberios": "tiberio",
    "tibetana": "tibetano",
    "tibetanas": "tibetano",
    "tibetanos": "tibetano",
    "tibia": "tibio",
    "tibiales": "tibial",
    "tibias": "tibio",
    "tibieces": "tibiez",
    "tibiezas": "tibieza",
    "tibioperonea": "tibioperoneo",
    "tibioperoneales": "tibioperoneal",
    "tibioperoneas": "tibioperoneo",
    "tibioperoneos": "tibioperoneo",
    "tibios": "tibio",
    "tibores": "tibor",
    "tiburones": "tiburón",
    "ticketes": "ticket",
    "tickets": "ticket",
    "tics": "tic",
    "tictacs": "tictac",
    "tie-breáks": "tie-break",
    "tiembla": "temblar",
    "tiemblan": "temblar",
    "tiemblas": "temblar",
    "tiemble": "temblar",
    "tiemblen": "temblar",
    "tiembles": "temblar",
    "tiemblo": "temblar",
    "tiemblos": "tiemblo",
    "tiempos": "tiempo",
    "tienda": "tender",
    "tiendan": "tender",
    "tiendas": "tienda",
    "tiende": "tender",
    "tienden": "tender",
    "tiendes": "tender",
    "tiendo": "tender",
    "tiene": "tener",
    "tienen": "tener",
    "tienes": "tener",
    "tienta": "tentar",
    "tientagujas": "tientaguja",
    "tientan": "tentar",
    "tientas": "tienta",
    "tiente": "tentar",
    "tienten": "tentar",
    "tientes": "tentar",
    "tiento": "tentar",
    "tientos": "tiento",
    "tierna": "tierno",
    "tiernas": "tierno",
    "tiernos": "tierno",
    "tierras": "tierra",
    "tiesa": "tieso",
    "tiesas": "tieso",
    "tiesos": "tieso",
    "tiestas": "tiesta",
    "tiestos": "tiesto",
    "tiesuras": "tiesura",
    "tifa": "tifo",
    "tifas": "tifo",
    "tifoidea": "tifoideo",
    "tifoideas": "tifoideo",
    "tifoideos": "tifoideo",
    "tifoides": "tifoide",
    "tifones": "tifón",
    "tifos": "tifo",
    "tifácea": "tifáceo",
    "tifáceas": "tifáceo",
    "tifáceos": "tifáceo",
    "tigres": "tigre",
    "tigresas": "tigresa",
    "tigrillos": "tigrillo",
    "tigroides": "tigroide",
    "tijas": "tija",
    "tijeradas": "tijerada",
    "tijeras": "tijera",
    "tijeretadas": "tijeretada",
    "tijeretas": "tijereta",
    "tijeretazos": "tijeretazo",
    "tijeretea": "tijeretear",
    "tijereteaba": "tijeretear",
    "tijereteabais": "tijeretear",
    "tijereteaban": "tijeretear",
    "tijereteabas": "tijeretear",
    "tijeretead": "tijeretear",
    "tijereteada": "tijeretear",
    "tijereteadas": "tijeretear",
    "tijereteado": "tijeretear",
    "tijereteados": "tijeretear",
    "tijereteamos": "tijeretear",
    "tijeretean": "tijeretear",
    "tijereteando": "tijeretear",
    "tijereteara": "tijeretear",
    "tijeretearais": "tijeretear",
    "tijeretearan": "tijeretear",
    "tijeretearas": "tijeretear",
    "tijereteare": "tijeretear",
    "tijereteareis": "tijeretear",
    "tijeretearemos": "tijeretear",
    "tijeretearen": "tijeretear",
    "tijereteares": "tijeretear",
    "tijeretearon": "tijeretear",
    "tijereteará": "tijeretear",
    "tijeretearán": "tijeretear",
    "tijeretearás": "tijeretear",
    "tijeretearé": "tijeretear",
    "tijeretearéis": "tijeretear",
    "tijeretearía": "tijeretear",
    "tijeretearíais": "tijeretear",
    "tijeretearíamos": "tijeretear",
    "tijeretearían": "tijeretear",
    "tijeretearías": "tijeretear",
    "tijereteas": "tijeretear",
    "tijeretease": "tijeretear",
    "tijereteaseis": "tijeretear",
    "tijereteasen": "tijeretear",
    "tijereteases": "tijeretear",
    "tijereteaste": "tijeretear",
    "tijereteasteis": "tijeretear",
    "tijeretee": "tijeretear",
    "tijereteemos": "tijeretear",
    "tijereteen": "tijeretear",
    "tijeretees": "tijeretear",
    "tijereteo": "tijeretear",
    "tijereteos": "tijereteo",
    "tijereteábamos": "tijeretear",
    "tijereteáis": "tijeretear",
    "tijereteáramos": "tijeretear",
    "tijereteáremos": "tijeretear",
    "tijereteásemos": "tijeretear",
    "tijereteé": "tijeretear",
    "tijereteéis": "tijeretear",
    "tijereteó": "tijeretear",
    "tijerillas": "tijerilla",
    "tijucas": "tijuca",
    "tilacoidales": "tilacoidal",
    "tilas": "tila",
    "tilda": "tildar",
    "tildaba": "tildar",
    "tildabais": "tildar",
    "tildaban": "tildar",
    "tildabas": "tildar",
    "tildad": "tildar",
    "tildada": "tildar",
    "tildadas": "tildar",
    "tildado": "tildar",
    "tildados": "tildar",
    "tildamos": "tildar",
    "tildan": "tildar",
    "tildando": "tildar",
    "tildara": "tildar",
    "tildarais": "tildar",
    "tildaran": "tildar",
    "tildaras": "tildar",
    "tildare": "tildar",
    "tildareis": "tildar",
    "tildaremos": "tildar",
    "tildaren": "tildar",
    "tildares": "tildar",
    "tildaron": "tildar",
    "tildará": "tildar",
    "tildarán": "tildar",
    "tildarás": "tildar",
    "tildaré": "tildar",
    "tildaréis": "tildar",
    "tildaría": "tildar",
    "tildaríais": "tildar",
    "tildaríamos": "tildar",
    "tildarían": "tildar",
    "tildarías": "tildar",
    "tildas": "tildar",
    "tildase": "tildar",
    "tildaseis": "tildar",
    "tildasen": "tildar",
    "tildases": "tildar",
    "tildaste": "tildar",
    "tildasteis": "tildar",
    "tilde": "tildar",
    "tildemos": "tildar",
    "tilden": "tildar",
    "tildes": "tilde",
    "tildo": "tildar",
    "tildones": "tildón",
    "tildábamos": "tildar",
    "tildáis": "tildar",
    "tildáramos": "tildar",
    "tildáremos": "tildar",
    "tildásemos": "tildar",
    "tildé": "tildar",
    "tildéis": "tildar",
    "tildó": "tildar",
    "tilines": "tilín",
    "tiliácea": "tiliáceo",
    "tiliáceas": "tiliáceo",
    "tiliáceos": "tiliáceo",
    "tilla": "tillar",
    "tillaba": "tillar",
    "tillabais": "tillar",
    "tillaban": "tillar",
    "tillabas": "tillar",
    "tillad": "tillar",
    "tillada": "tillar",
    "tilladas": "tillar",
    "tillado": "tillar",
    "tillados": "tillar",
    "tillamos": "tillar",
    "tillan": "tillar",
    "tillando": "tillar",
    "tillara": "tillar",
    "tillarais": "tillar",
    "tillaran": "tillar",
    "tillaras": "tillar",
    "tillare": "tillar",
    "tillareis": "tillar",
    "tillaremos": "tillar",
    "tillaren": "tillar",
    "tillares": "tillar",
    "tillaron": "tillar",
    "tillará": "tillar",
    "tillarán": "tillar",
    "tillarás": "tillar",
    "tillaré": "tillar",
    "tillaréis": "tillar",
    "tillaría": "tillar",
    "tillaríais": "tillar",
    "tillaríamos": "tillar",
    "tillarían": "tillar",
    "tillarías": "tillar",
    "tillas": "tillar",
    "tillase": "tillar",
    "tillaseis": "tillar",
    "tillasen": "tillar",
    "tillases": "tillar",
    "tillaste": "tillar",
    "tillasteis": "tillar",
    "tille": "tillar",
    "tillemos": "tillar",
    "tillen": "tillar",
    "tilles": "tillar",
    "tillitas": "tillita",
    "tillo": "tillar",
    "tillábamos": "tillar",
    "tilláis": "tillar",
    "tilláramos": "tillar",
    "tilláremos": "tillar",
    "tillásemos": "tillar",
    "tillé": "tillar",
    "tilléis": "tillar",
    "tilló": "tillar",
    "tilos": "tilo",
    "tima": "timar",
    "timaba": "timar",
    "timabais": "timar",
    "timaban": "timar",
    "timabas": "timar",
    "timad": "timar",
    "timada": "timar",
    "timadas": "timar",
    "timado": "timar",
    "timadora": "timador",
    "timadoras": "timador",
    "timadores": "timador",
    "timados": "timar",
    "timamos": "timar",
    "timan": "timar",
    "timando": "timar",
    "timara": "timar",
    "timarais": "timar",
    "timaran": "timar",
    "timaras": "timar",
    "timare": "timar",
    "timareis": "timar",
    "timaremos": "timar",
    "timaren": "timar",
    "timares": "timar",
    "timaron": "timar",
    "timará": "timar",
    "timarán": "timar",
    "timarás": "timar",
    "timaré": "timar",
    "timaréis": "timar",
    "timaría": "timar",
    "timaríais": "timar",
    "timaríamos": "timar",
    "timarían": "timar",
    "timarías": "timar",
    "timas": "timar",
    "timase": "timar",
    "timaseis": "timar",
    "timasen": "timar",
    "timases": "timar",
    "timaste": "timar",
    "timasteis": "timar",
    "timbalera": "timbalero",
    "timbaleras": "timbalero",
    "timbaleros": "timbalero",
    "timbales": "timbal",
    "timbas": "timba",
    "timbirimbas": "timbirimba",
    "timboes": "timbó",
    "timbra": "timbrar",
    "timbraba": "timbrar",
    "timbrabais": "timbrar",
    "timbraban": "timbrar",
    "timbrabas": "timbrar",
    "timbrad": "timbrar",
    "timbrada": "timbrar",
    "timbradas": "timbrar",
    "timbrado": "timbrar",
    "timbradora": "timbrador",
    "timbradoras": "timbrador",
    "timbradores": "timbrador",
    "timbrados": "timbrar",
    "timbramos": "timbrar",
    "timbran": "timbrar",
    "timbrando": "timbrar",
    "timbrara": "timbrar",
    "timbrarais": "timbrar",
    "timbraran": "timbrar",
    "timbraras": "timbrar",
    "timbrare": "timbrar",
    "timbrareis": "timbrar",
    "timbraremos": "timbrar",
    "timbraren": "timbrar",
    "timbrares": "timbrar",
    "timbraron": "timbrar",
    "timbrará": "timbrar",
    "timbrarán": "timbrar",
    "timbrarás": "timbrar",
    "timbraré": "timbrar",
    "timbraréis": "timbrar",
    "timbraría": "timbrar",
    "timbraríais": "timbrar",
    "timbraríamos": "timbrar",
    "timbrarían": "timbrar",
    "timbrarías": "timbrar",
    "timbras": "timbrar",
    "timbrase": "timbrar",
    "timbraseis": "timbrar",
    "timbrasen": "timbrar",
    "timbrases": "timbrar",
    "timbraste": "timbrar",
    "timbrasteis": "timbrar",
    "timbrazos": "timbrazo",
    "timbre": "timbrar",
    "timbremos": "timbrar",
    "timbren": "timbrar",
    "timbres": "timbre",
    "timbro": "timbrar",
    "timbrofilias": "timbrofilia",
    "timbrábamos": "timbrar",
    "timbráis": "timbrar",
    "timbráramos": "timbrar",
    "timbráremos": "timbrar",
    "timbrásemos": "timbrar",
    "timbré": "timbrar",
    "timbréis": "timbrar",
    "timbró": "timbrar",
    "timbrófila": "timbrófilo",
    "timbrófilas": "timbrófilo",
    "timbrófilos": "timbrófilo",
    "timbós": "timbó",
    "time": "timar",
    "timectomías": "timectomía",
    "timemos": "timar",
    "timen": "timar",
    "times": "timar",
    "timiamas": "timiama",
    "timideces": "timidez",
    "timidinas": "timidina",
    "timinas": "timina",
    "timo": "timar",
    "timocitos": "timocito",
    "timocracias": "timocracia",
    "timocrática": "timocrático",
    "timocráticas": "timocrático",
    "timocráticos": "timocrático",
    "timodependientes": "timodependiente",
    "timoftaleínas": "timoftaleína",
    "timoindependientes": "timoindependiente",
    "timoles": "timol",
    "timonea": "timonear",
    "timoneaba": "timonear",
    "timoneabais": "timonear",
    "timoneaban": "timonear",
    "timoneabas": "timonear",
    "timonead": "timonear",
    "timoneada": "timonear",
    "timoneadas": "timonear",
    "timoneado": "timonear",
    "timoneados": "timonear",
    "timoneamos": "timonear",
    "timonean": "timonear",
    "timoneando": "timonear",
    "timoneara": "timonear",
    "timonearais": "timonear",
    "timonearan": "timonear",
    "timonearas": "timonear",
    "timoneare": "timonear",
    "timoneareis": "timonear",
    "timonearemos": "timonear",
    "timonearen": "timonear",
    "timoneares": "timonear",
    "timonearon": "timonear",
    "timoneará": "timonear",
    "timonearán": "timonear",
    "timonearás": "timonear",
    "timonearé": "timonear",
    "timonearéis": "timonear",
    "timonearía": "timonear",
    "timonearíais": "timonear",
    "timonearíamos": "timonear",
    "timonearían": "timonear",
    "timonearías": "timonear",
    "timoneas": "timonear",
    "timonease": "timonear",
    "timoneaseis": "timonear",
    "timoneasen": "timonear",
    "timoneases": "timonear",
    "timoneaste": "timonear",
    "timoneasteis": "timonear",
    "timonee": "timonear",
    "timoneemos": "timonear",
    "timoneen": "timonear",
    "timonees": "timonear",
    "timoneles": "timonel",
    "timoneo": "timonear",
    "timonera": "timonero",
    "timoneras": "timonero",
    "timoneros": "timonero",
    "timonerías": "timonería",
    "timones": "timón",
    "timoneábamos": "timonear",
    "timoneáis": "timonear",
    "timoneáramos": "timonear",
    "timoneáremos": "timonear",
    "timoneásemos": "timonear",
    "timoneé": "timonear",
    "timoneéis": "timonear",
    "timoneó": "timonear",
    "timoquinonas": "timoquinona",
    "timorata": "timorato",
    "timoratas": "timorato",
    "timoratos": "timorato",
    "timorenses": "timorense",
    "timoresa": "timorés",
    "timoresas": "timorés",
    "timoreses": "timorés",
    "timos": "timo",
    "timosinas": "timosina",
    "timpanice": "timpanizar",
    "timpanicemos": "timpanizar",
    "timpanicen": "timpanizar",
    "timpanices": "timpanizar",
    "timpanicé": "timpanizar",
    "timpanicéis": "timpanizar",
    "timpanismos": "timpanismo",
    "timpaniza": "timpanizar",
    "timpanizaba": "timpanizar",
    "timpanizabais": "timpanizar",
    "timpanizaban": "timpanizar",
    "timpanizabas": "timpanizar",
    "timpanizaciones": "timpanización",
    "timpanizad": "timpanizar",
    "timpanizada": "timpanizar",
    "timpanizadas": "timpanizar",
    "timpanizado": "timpanizar",
    "timpanizados": "timpanizar",
    "timpanizamos": "timpanizar",
    "timpanizan": "timpanizar",
    "timpanizando": "timpanizar",
    "timpanizara": "timpanizar",
    "timpanizarais": "timpanizar",
    "timpanizaran": "timpanizar",
    "timpanizaras": "timpanizar",
    "timpanizare": "timpanizar",
    "timpanizareis": "timpanizar",
    "timpanizaremos": "timpanizar",
    "timpanizaren": "timpanizar",
    "timpanizares": "timpanizar",
    "timpanizaron": "timpanizar",
    "timpanizará": "timpanizar",
    "timpanizarán": "timpanizar",
    "timpanizarás": "timpanizar",
    "timpanizaré": "timpanizar",
    "timpanizaréis": "timpanizar",
    "timpanizaría": "timpanizar",
    "timpanizaríais": "timpanizar",
    "timpanizaríamos": "timpanizar",
    "timpanizarían": "timpanizar",
    "timpanizarías": "timpanizar",
    "timpanizas": "timpanizar",
    "timpanizase": "timpanizar",
    "timpanizaseis": "timpanizar",
    "timpanizasen": "timpanizar",
    "timpanizases": "timpanizar",
    "timpanizaste": "timpanizar",
    "timpanizasteis": "timpanizar",
    "timpanizo": "timpanizar",
    "timpanizábamos": "timpanizar",
    "timpanizáis": "timpanizar",
    "timpanizáramos": "timpanizar",
    "timpanizáremos": "timpanizar",
    "timpanizásemos": "timpanizar",
    "timpanizó": "timpanizar",
    "timpanoescamosa": "timpanoescamoso",
    "timpanoescamosas": "timpanoescamoso",
    "timpanoescamosos": "timpanoescamoso",
    "timpanoestapedia": "timpanoestapedio",
    "timpanoestapedias": "timpanoestapedio",
    "timpanoestapedios": "timpanoestapedio",
    "timpanogramas": "timpanograma",
    "timpanomaleares": "timpanomalear",
    "timpanomastoidea": "timpanomastoideo",
    "timpanomastoideas": "timpanomastoideo",
    "timpanomastoideos": "timpanomastoideo",
    "timpanometrías": "timpanometría",
    "timpanoosiculares": "timpanoosicular",
    "timpanoplastias": "timpanoplastia",
    "timpanostapediopexias": "timpanostapediopexia",
    "timpanotomías": "timpanotomía",
    "timpanítica": "timpanítico",
    "timpaníticas": "timpanítico",
    "timpaníticos": "timpanítico",
    "timpas": "timpa",
    "timpánica": "timpánico",
    "timpánicas": "timpánico",
    "timpánicos": "timpánico",
    "timuríes": "timurí",
    "timurís": "timurí",
    "timábamos": "timar",
    "timáis": "timar",
    "timáramos": "timar",
    "timáremos": "timar",
    "timásemos": "timar",
    "timé": "timar",
    "timéis": "timar",
    "timó": "timar",
    "timócratas": "timócrata",
    "tinacos": "tinaco",
    "tinadas": "tinada",
    "tinadores": "tinador",
    "tinados": "tinado",
    "tinajas": "tinaja",
    "tinajera": "tinajero",
    "tinajeras": "tinajero",
    "tinajeros": "tinajero",
    "tinajones": "tinajón",
    "tinamúes": "tinamú",
    "tinamús": "tinamú",
    "tinas": "tina",
    "tinciones": "tinción",
    "tindalizaciones": "tindalización",
    "tinelares": "tinelar",
    "tinelera": "tinelero",
    "tineleras": "tinelero",
    "tineleros": "tinelero",
    "tinelos": "tinelo",
    "tinerfeña": "tinerfeño",
    "tinerfeñas": "tinerfeño",
    "tinerfeños": "tinerfeño",
    "tinges": "tinge",
    "tingitana": "tingitano",
    "tingitanas": "tingitano",
    "tingitanos": "tingitano",
    "tingladillos": "tingladillo",
    "tinglados": "tinglado",
    "tingles": "tingle",
    "tiniclas": "tinicla",
    "tinieblas": "tiniebla",
    "tinillos": "tinillo",
    "tinnítuses": "tinnitus",
    "tinos": "tino",
    "tinta": "tinto",
    "tintaba": "tintar",
    "tintabais": "tintar",
    "tintaban": "tintar",
    "tintabas": "tintar",
    "tintad": "tintar",
    "tintada": "tintar",
    "tintadas": "tintar",
    "tintado": "tintar",
    "tintadora": "tintador",
    "tintadoras": "tintador",
    "tintadores": "tintador",
    "tintados": "tintar",
    "tintamos": "tintar",
    "tintan": "tintar",
    "tintando": "tintar",
    "tintara": "tintar",
    "tintarais": "tintar",
    "tintaran": "tintar",
    "tintaras": "tintar",
    "tintare": "tintar",
    "tintareis": "tintar",
    "tintaremos": "tintar",
    "tintaren": "tintar",
    "tintares": "tintar",
    "tintaron": "tintar",
    "tintará": "tintar",
    "tintarán": "tintar",
    "tintarás": "tintar",
    "tintaré": "tintar",
    "tintaréis": "tintar",
    "tintaría": "tintar",
    "tintaríais": "tintar",
    "tintaríamos": "tintar",
    "tintarían": "tintar",
    "tintarías": "tintar",
    "tintas": "tinto",
    "tintase": "tintar",
    "tintaseis": "tintar",
    "tintasen": "tintar",
    "tintases": "tintar",
    "tintaste": "tintar",
    "tintasteis": "tintar",
    "tinte": "tintar",
    "tintemos": "tintar",
    "tinten": "tintar",
    "tinterazos": "tinterazo",
    "tinterilla": "tinterillar",
    "tinterillaba": "tinterillar",
    "tinterillabais": "tinterillar",
    "tinterillaban": "tinterillar",
    "tinterillabas": "tinterillar",
    "tinterillad": "tinterillar",
    "tinterillada": "tinterillar",
    "tinterilladas": "tinterillar",
    "tinterillado": "tinterillar",
    "tinterillados": "tinterillar",
    "tinterillamos": "tinterillar",
    "tinterillan": "tinterillar",
    "tinterillando": "tinterillar",
    "tinterillara": "tinterillar",
    "tinterillarais": "tinterillar",
    "tinterillaran": "tinterillar",
    "tinterillaras": "tinterillar",
    "tinterillare": "tinterillar",
    "tinterillareis": "tinterillar",
    "tinterillaremos": "tinterillar",
    "tinterillaren": "tinterillar",
    "tinterillares": "tinterillar",
    "tinterillaron": "tinterillar",
    "tinterillará": "tinterillar",
    "tinterillarán": "tinterillar",
    "tinterillarás": "tinterillar",
    "tinterillaré": "tinterillar",
    "tinterillaréis": "tinterillar",
    "tinterillaría": "tinterillar",
    "tinterillaríais": "tinterillar",
    "tinterillaríamos": "tinterillar",
    "tinterillarían": "tinterillar",
    "tinterillarías": "tinterillar",
    "tinterillas": "tinterillar",
    "tinterillase": "tinterillar",
    "tinterillaseis": "tinterillar",
    "tinterillasen": "tinterillar",
    "tinterillases": "tinterillar",
    "tinterillaste": "tinterillar",
    "tinterillasteis": "tinterillar",
    "tinterille": "tinterillar",
    "tinterillemos": "tinterillar",
    "tinterillen": "tinterillar",
    "tinterilles": "tinterillar",
    "tinterillo": "tinterillar",
    "tinterillos": "tinterillo",
    "tinterillábamos": "tinterillar",
    "tinterilláis": "tinterillar",
    "tinterilláramos": "tinterillar",
    "tinterilláremos": "tinterillar",
    "tinterillásemos": "tinterillar",
    "tinterillé": "tinterillar",
    "tinterilléis": "tinterillar",
    "tinterilló": "tinterillar",
    "tinteros": "tintero",
    "tintes": "tinte",
    "tintinea": "tintinear",
    "tintineaba": "tintinear",
    "tintineabais": "tintinear",
    "tintineaban": "tintinear",
    "tintineabas": "tintinear",
    "tintinead": "tintinear",
    "tintineada": "tintinear",
    "tintineadas": "tintinear",
    "tintineado": "tintinear",
    "tintineados": "tintinear",
    "tintineamos": "tintinear",
    "tintinean": "tintinear",
    "tintineando": "tintinear",
    "tintineara": "tintinear",
    "tintinearais": "tintinear",
    "tintinearan": "tintinear",
    "tintinearas": "tintinear",
    "tintineare": "tintinear",
    "tintineareis": "tintinear",
    "tintinearemos": "tintinear",
    "tintinearen": "tintinear",
    "tintineares": "tintinear",
    "tintinearon": "tintinear",
    "tintineará": "tintinear",
    "tintinearán": "tintinear",
    "tintinearás": "tintinear",
    "tintinearé": "tintinear",
    "tintinearéis": "tintinear",
    "tintinearía": "tintinear",
    "tintinearíais": "tintinear",
    "tintinearíamos": "tintinear",
    "tintinearían": "tintinear",
    "tintinearías": "tintinear",
    "tintineas": "tintinear",
    "tintinease": "tintinear",
    "tintineaseis": "tintinear",
    "tintineasen": "tintinear",
    "tintineases": "tintinear",
    "tintineaste": "tintinear",
    "tintineasteis": "tintinear",
    "tintinee": "tintinear",
    "tintineemos": "tintinear",
    "tintineen": "tintinear",
    "tintinees": "tintinear",
    "tintineo": "tintinear",
    "tintineos": "tintineo",
    "tintines": "tintín",
    "tintineábamos": "tintinear",
    "tintineáis": "tintinear",
    "tintineáramos": "tintinear",
    "tintineáremos": "tintinear",
    "tintineásemos": "tintinear",
    "tintineé": "tintinear",
    "tintineéis": "tintinear",
    "tintineó": "tintinear",
    "tinto": "tintar",
    "tintorera": "tintorero",
    "tintoreras": "tintorero",
    "tintoreros": "tintorero",
    "tintorerías": "tintorería",
    "tintorros": "tintorro",
    "tintos": "tinto",
    "tinturas": "tintura",
    "tintábamos": "tintar",
    "tintáis": "tintar",
    "tintáramos": "tintar",
    "tintáremos": "tintar",
    "tintásemos": "tintar",
    "tinté": "tintar",
    "tintéis": "tintar",
    "tintó": "tintar",
    "tintómetros": "tintómetro",
    "tintórea": "tintóreo",
    "tintóreas": "tintóreo",
    "tintóreos": "tintóreo",
    "tioacética": "tioacético",
    "tioacéticas": "tioacético",
    "tioacéticos": "tioacético",
    "tioalcoholes": "tioalcohol",
    "tiobenzoica": "tiobenzoico",
    "tiobenzoicas": "tiobenzoico",
    "tiobenzoicos": "tiobenzoico",
    "tiocarbanilidas": "tiocarbanilida",
    "tiocianoacetatos": "tiocianoacetato",
    "tiociánica": "tiociánico",
    "tiociánicas": "tiociánico",
    "tiociánicos": "tiociánico",
    "tiodiglicoles": "tiodiglicol",
    "tiodiglicólica": "tiodiglicólico",
    "tiodiglicólicas": "tiodiglicólico",
    "tiodiglicólicos": "tiodiglicólico",
    "tiodipropionatos": "tiodipropionato",
    "tiofenoles": "tiofenol",
    "tiofenos": "tiofeno",
    "tiofosgenos": "tiofosgeno",
    "tioglicerinas": "tioglicerina",
    "tioglicolatos": "tioglicolato",
    "tioglicólica": "tioglicólico",
    "tioglicólicas": "tioglicólico",
    "tioglicólicos": "tioglicólico",
    "tiohidantoínas": "tiohidantoína",
    "tioles": "tiol",
    "tionilos": "tionilo",
    "tiorbas": "tiorba",
    "tiosalicílica": "tiosalicílico",
    "tiosalicílicas": "tiosalicílico",
    "tiosalicílicos": "tiosalicílico",
    "tiosemicarbacidas": "tiosemicarbacida",
    "tiosemicarbazonas": "tiosemicarbazona",
    "tiosulfatos": "tiosulfato",
    "tiosulfúrica": "tiosulfúrico",
    "tiosulfúricas": "tiosulfúrico",
    "tiosulfúricos": "tiosulfúrico",
    "tiouracilos": "tiouracilo",
    "tiouramilos": "tiouramilo",
    "tioureas": "tiourea",
    "tiovivos": "tiovivo",
    "tioácidos": "tioácido",
    "tioéteres": "tioéter",
    "tiparraca": "tiparraco",
    "tiparracas": "tiparraco",
    "tiparracos": "tiparraco",
    "tipas": "tipa",
    "tipeja": "tipejo",
    "tipejas": "tipejo",
    "tipejos": "tipejo",
    "tipiadoras": "tipiadora",
    "tipicidades": "tipicidad",
    "tipifica": "tipificar",
    "tipificaba": "tipificar",
    "tipificabais": "tipificar",
    "tipificaban": "tipificar",
    "tipificabas": "tipificar",
    "tipificaciones": "tipificación",
    "tipificad": "tipificar",
    "tipificada": "tipificar",
    "tipificadas": "tipificar",
    "tipificado": "tipificar",
    "tipificadora": "tipificador",
    "tipificadoras": "tipificador",
    "tipificadores": "tipificador",
    "tipificados": "tipificar",
    "tipificamos": "tipificar",
    "tipifican": "tipificar",
    "tipificando": "tipificar",
    "tipificara": "tipificar",
    "tipificarais": "tipificar",
    "tipificaran": "tipificar",
    "tipificaras": "tipificar",
    "tipificare": "tipificar",
    "tipificareis": "tipificar",
    "tipificaremos": "tipificar",
    "tipificaren": "tipificar",
    "tipificares": "tipificar",
    "tipificaron": "tipificar",
    "tipificará": "tipificar",
    "tipificarán": "tipificar",
    "tipificarás": "tipificar",
    "tipificaré": "tipificar",
    "tipificaréis": "tipificar",
    "tipificaría": "tipificar",
    "tipificaríais": "tipificar",
    "tipificaríamos": "tipificar",
    "tipificarían": "tipificar",
    "tipificarías": "tipificar",
    "tipificas": "tipificar",
    "tipificase": "tipificar",
    "tipificaseis": "tipificar",
    "tipificasen": "tipificar",
    "tipificases": "tipificar",
    "tipificaste": "tipificar",
    "tipificasteis": "tipificar",
    "tipifico": "tipificar",
    "tipificábamos": "tipificar",
    "tipificáis": "tipificar",
    "tipificáramos": "tipificar",
    "tipificáremos": "tipificar",
    "tipificásemos": "tipificar",
    "tipificó": "tipificar",
    "tipifique": "tipificar",
    "tipifiquemos": "tipificar",
    "tipifiquen": "tipificar",
    "tipifiques": "tipificar",
    "tipifiqué": "tipificar",
    "tipifiquéis": "tipificar",
    "tipismos": "tipismo",
    "tipistas": "tipista",
    "tiples": "tiple",
    "tiplisonantes": "tiplisonante",
    "tipocromías": "tipocromía",
    "tipografías": "tipografía",
    "tipográfica": "tipográfico",
    "tipográficas": "tipográfico",
    "tipográficos": "tipográfico",
    "tipologías": "tipología",
    "tipológica": "tipológico",
    "tipológicas": "tipológico",
    "tipológicos": "tipológico",
    "tipometrías": "tipometría",
    "tipos": "tipo",
    "tipotelegrafías": "tipotelegrafía",
    "tipotelegráfica": "tipotelegráfico",
    "tipotelegráficas": "tipotelegráfico",
    "tipotelegráficos": "tipotelegráfico",
    "tipotelégrafos": "tipotelégrafo",
    "tipógrafa": "tipógrafo",
    "tipógrafas": "tipógrafo",
    "tipógrafos": "tipógrafo",
    "tipómetros": "tipómetro",
    "tipúlidos": "tipúlido",
    "tiques": "tique",
    "tiquetes": "tiquete",
    "tira": "tirar",
    "tiraba": "tirar",
    "tirabais": "tirar",
    "tirabalas": "tirabala",
    "tiraban": "tirar",
    "tirabas": "tirar",
    "tirabragueros": "tirabraguero",
    "tirabuzones": "tirabuzón",
    "tiracoles": "tiracol",
    "tiracuellos": "tiracuello",
    "tirad": "tirar",
    "tirada": "tirar",
    "tiradas": "tirar",
    "tiraderas": "tiradera",
    "tiraderos": "tiradero",
    "tirado": "tirar",
    "tiradora": "tirador",
    "tiradoras": "tirador",
    "tiradores": "tirador",
    "tirados": "tirar",
    "tirafondos": "tirafondo",
    "tirajes": "tiraje",
    "tiramientos": "tiramiento",
    "tiraminas": "tiramina",
    "tiramiras": "tiramira",
    "tiramolla": "tiramollar",
    "tiramollaba": "tiramollar",
    "tiramollabais": "tiramollar",
    "tiramollaban": "tiramollar",
    "tiramollabas": "tiramollar",
    "tiramollad": "tiramollar",
    "tiramollada": "tiramollar",
    "tiramolladas": "tiramollar",
    "tiramollado": "tiramollar",
    "tiramollados": "tiramollar",
    "tiramollamos": "tiramollar",
    "tiramollan": "tiramollar",
    "tiramollando": "tiramollar",
    "tiramollara": "tiramollar",
    "tiramollarais": "tiramollar",
    "tiramollaran": "tiramollar",
    "tiramollaras": "tiramollar",
    "tiramollare": "tiramollar",
    "tiramollareis": "tiramollar",
    "tiramollaremos": "tiramollar",
    "tiramollaren": "tiramollar",
    "tiramollares": "tiramollar",
    "tiramollaron": "tiramollar",
    "tiramollará": "tiramollar",
    "tiramollarán": "tiramollar",
    "tiramollarás": "tiramollar",
    "tiramollaré": "tiramollar",
    "tiramollaréis": "tiramollar",
    "tiramollaría": "tiramollar",
    "tiramollaríais": "tiramollar",
    "tiramollaríamos": "tiramollar",
    "tiramollarían": "tiramollar",
    "tiramollarías": "tiramollar",
    "tiramollas": "tiramollar",
    "tiramollase": "tiramollar",
    "tiramollaseis": "tiramollar",
    "tiramollasen": "tiramollar",
    "tiramollases": "tiramollar",
    "tiramollaste": "tiramollar",
    "tiramollasteis": "tiramollar",
    "tiramolle": "tiramollar",
    "tiramollemos": "tiramollar",
    "tiramollen": "tiramollar",
    "tiramolles": "tiramollar",
    "tiramollo": "tiramollar",
    "tiramollábamos": "tiramollar",
    "tiramolláis": "tiramollar",
    "tiramolláramos": "tiramollar",
    "tiramolláremos": "tiramollar",
    "tiramollásemos": "tiramollar",
    "tiramollé": "tiramollar",
    "tiramolléis": "tiramollar",
    "tiramolló": "tiramollar",
    "tiramos": "tirar",
    "tiran": "tirar",
    "tirana": "tirano",
    "tiranas": "tirano",
    "tirando": "tirar",
    "tiranice": "tiranizar",
    "tiranicemos": "tiranizar",
    "tiranicen": "tiranizar",
    "tiranices": "tiranizar",
    "tiranicidas": "tiranicida",
    "tiranicidios": "tiranicidio",
    "tiranicé": "tiranizar",
    "tiranicéis": "tiranizar",
    "tiraniza": "tiranizar",
    "tiranizaba": "tiranizar",
    "tiranizabais": "tiranizar",
    "tiranizaban": "tiranizar",
    "tiranizabas": "tiranizar",
    "tiranizaciones": "tiranización",
    "tiranizad": "tiranizar",
    "tiranizada": "tiranizar",
    "tiranizadas": "tiranizar",
    "tiranizado": "tiranizar",
    "tiranizados": "tiranizar",
    "tiranizamos": "tiranizar",
    "tiranizan": "tiranizar",
    "tiranizando": "tiranizar",
    "tiranizara": "tiranizar",
    "tiranizarais": "tiranizar",
    "tiranizaran": "tiranizar",
    "tiranizaras": "tiranizar",
    "tiranizare": "tiranizar",
    "tiranizareis": "tiranizar",
    "tiranizaremos": "tiranizar",
    "tiranizaren": "tiranizar",
    "tiranizares": "tiranizar",
    "tiranizaron": "tiranizar",
    "tiranizará": "tiranizar",
    "tiranizarán": "tiranizar",
    "tiranizarás": "tiranizar",
    "tiranizaré": "tiranizar",
    "tiranizaréis": "tiranizar",
    "tiranizaría": "tiranizar",
    "tiranizaríais": "tiranizar",
    "tiranizaríamos": "tiranizar",
    "tiranizarían": "tiranizar",
    "tiranizarías": "tiranizar",
    "tiranizas": "tiranizar",
    "tiranizase": "tiranizar",
    "tiranizaseis": "tiranizar",
    "tiranizasen": "tiranizar",
    "tiranizases": "tiranizar",
    "tiranizaste": "tiranizar",
    "tiranizasteis": "tiranizar",
    "tiranizo": "tiranizar",
    "tiranizábamos": "tiranizar",
    "tiranizáis": "tiranizar",
    "tiranizáramos": "tiranizar",
    "tiranizáremos": "tiranizar",
    "tiranizásemos": "tiranizar",
    "tiranizó": "tiranizar",
    "tiranos": "tirano",
    "tiranteces": "tirantez",
    "tirantes": "tirante",
    "tiranías": "tiranía",
    "tirapiés": "tirapié",
    "tirara": "tirar",
    "tirarais": "tirar",
    "tiraran": "tirar",
    "tiraras": "tirar",
    "tirare": "tirar",
    "tirareis": "tirar",
    "tiraremos": "tirar",
    "tiraren": "tirar",
    "tirares": "tirar",
    "tiraron": "tirar",
    "tirará": "tirar",
    "tirarán": "tirar",
    "tirarás": "tirar",
    "tiraré": "tirar",
    "tiraréis": "tirar",
    "tiraría": "tirar",
    "tiraríais": "tirar",
    "tiraríamos": "tirar",
    "tirarían": "tirar",
    "tirarías": "tirar",
    "tiras": "tirar",
    "tirase": "tirar",
    "tiraseis": "tirar",
    "tirasen": "tirar",
    "tirases": "tirar",
    "tiraste": "tirar",
    "tirasteis": "tirar",
    "tiratrones": "tiratrón",
    "tire": "tirar",
    "tirelas": "tirela",
    "tiremos": "tirar",
    "tiren": "tirar",
    "tires": "tirar",
    "tiria": "tirio",
    "tirias": "tirio",
    "tirillas": "tirilla",
    "tirios": "tirio",
    "tiristores": "tiristor",
    "tirita": "tiritar",
    "tiritaba": "tiritar",
    "tiritabais": "tiritar",
    "tiritaban": "tiritar",
    "tiritabas": "tiritar",
    "tiritad": "tiritar",
    "tiritada": "tiritar",
    "tiritadas": "tiritar",
    "tiritado": "tiritar",
    "tiritados": "tiritar",
    "tiritamos": "tiritar",
    "tiritan": "tiritar",
    "tiritando": "tiritar",
    "tiritara": "tiritar",
    "tiritarais": "tiritar",
    "tiritaran": "tiritar",
    "tiritaras": "tiritar",
    "tiritare": "tiritar",
    "tiritareis": "tiritar",
    "tiritaremos": "tiritar",
    "tiritaren": "tiritar",
    "tiritares": "tiritar",
    "tiritaron": "tiritar",
    "tiritará": "tiritar",
    "tiritarán": "tiritar",
    "tiritarás": "tiritar",
    "tiritaré": "tiritar",
    "tiritaréis": "tiritar",
    "tiritaría": "tiritar",
    "tiritaríais": "tiritar",
    "tiritaríamos": "tiritar",
    "tiritarían": "tiritar",
    "tiritarías": "tiritar",
    "tiritas": "tiritar",
    "tiritase": "tiritar",
    "tiritaseis": "tiritar",
    "tiritasen": "tiritar",
    "tiritases": "tiritar",
    "tiritaste": "tiritar",
    "tiritasteis": "tiritar",
    "tiritañas": "tiritaña",
    "tirite": "tiritar",
    "tiritemos": "tiritar",
    "tiriten": "tiritar",
    "tiriteras": "tiritera",
    "tirites": "tiritar",
    "tirito": "tiritar",
    "tiritonas": "tiritona",
    "tiritábamos": "tiritar",
    "tiritáis": "tiritar",
    "tiritáramos": "tiritar",
    "tiritáremos": "tiritar",
    "tiritásemos": "tiritar",
    "tirité": "tiritar",
    "tiritéis": "tiritar",
    "tiritó": "tiritar",
    "tiro": "tirar",
    "tiroaritenoidea": "tiroaritenoideo",
    "tiroaritenoideas": "tiroaritenoideo",
    "tiroaritenoideos": "tiroaritenoideo",
    "tirocervicales": "tirocervical",
    "tirocidinas": "tirocidina",
    "tiroepiglótica": "tiroepiglótico",
    "tiroepiglóticas": "tiroepiglótico",
    "tiroepiglóticos": "tiroepiglótico",
    "tiroglosa": "tirogloso",
    "tiroglosas": "tirogloso",
    "tiroglosos": "tirogloso",
    "tirohioidea": "tirohioideo",
    "tirohioideas": "tirohioideo",
    "tirohioideos": "tirohioideo",
    "tiroidea": "tiroideo",
    "tiroideas": "tiroideo",
    "tiroideos": "tiroideo",
    "tiroidinas": "tiroidina",
    "tirolesa": "tirolés",
    "tirolesas": "tirolés",
    "tiroleses": "tirolés",
    "tironera": "tironero",
    "tironeras": "tironero",
    "tironeros": "tironero",
    "tirones": "tirón",
    "tiroriros": "tiroriro",
    "tiros": "tiro",
    "tirosinas": "tirosina",
    "tirotea": "tirotear",
    "tiroteaba": "tirotear",
    "tiroteabais": "tirotear",
    "tiroteaban": "tirotear",
    "tiroteabas": "tirotear",
    "tirotead": "tirotear",
    "tiroteada": "tirotear",
    "tiroteadas": "tirotear",
    "tiroteado": "tirotear",
    "tiroteados": "tirotear",
    "tiroteamos": "tirotear",
    "tirotean": "tirotear",
    "tiroteando": "tirotear",
    "tiroteara": "tirotear",
    "tirotearais": "tirotear",
    "tirotearan": "tirotear",
    "tirotearas": "tirotear",
    "tiroteare": "tirotear",
    "tiroteareis": "tirotear",
    "tirotearemos": "tirotear",
    "tirotearen": "tirotear",
    "tiroteares": "tirotear",
    "tirotearon": "tirotear",
    "tiroteará": "tirotear",
    "tirotearán": "tirotear",
    "tirotearás": "tirotear",
    "tirotearé": "tirotear",
    "tirotearéis": "tirotear",
    "tirotearía": "tirotear",
    "tirotearíais": "tirotear",
    "tirotearíamos": "tirotear",
    "tirotearían": "tirotear",
    "tirotearías": "tirotear",
    "tiroteas": "tirotear",
    "tirotease": "tirotear",
    "tiroteaseis": "tirotear",
    "tiroteasen": "tirotear",
    "tiroteases": "tirotear",
    "tiroteaste": "tirotear",
    "tiroteasteis": "tirotear",
    "tirotee": "tirotear",
    "tiroteemos": "tirotear",
    "tiroteen": "tirotear",
    "tirotees": "tirotear",
    "tiroteo": "tirotear",
    "tiroteos": "tiroteo",
    "tiroteábamos": "tirotear",
    "tiroteáis": "tirotear",
    "tiroteáramos": "tirotear",
    "tiroteáremos": "tirotear",
    "tiroteásemos": "tirotear",
    "tiroteé": "tirotear",
    "tiroteéis": "tirotear",
    "tiroteó": "tirotear",
    "tirotomías": "tirotomía",
    "tirotricinas": "tirotricina",
    "tiroxinas": "tiroxina",
    "tirrias": "tirria",
    "tirsos": "tirso",
    "tirulata": "tirulato",
    "tirulatas": "tirulato",
    "tirulatos": "tirulato",
    "tirulos": "tirulo",
    "tirábamos": "tirar",
    "tiráis": "tirar",
    "tiránica": "tiránico",
    "tiránicas": "tiránico",
    "tiránicos": "tiránico",
    "tiráramos": "tirar",
    "tiráremos": "tirar",
    "tirásemos": "tirar",
    "tiré": "tirar",
    "tiréis": "tirar",
    "tiró": "tirar",
    "tisanas": "tisana",
    "tisanuros": "tisanuro",
    "tisanópteros": "tisanóptero",
    "tisulares": "tisular",
    "tisurias": "tisuria",
    "titanatos": "titanato",
    "titanes": "titán",
    "titania": "titanio",
    "titanias": "titanio",
    "titanios": "titanio",
    "titanitas": "titanita",
    "titanosa": "titanoso",
    "titanosas": "titanoso",
    "titanosos": "titanoso",
    "titas": "tita",
    "titea": "titear",
    "titeaba": "titear",
    "titeabais": "titear",
    "titeaban": "titear",
    "titeabas": "titear",
    "titead": "titear",
    "titeada": "titear",
    "titeadas": "titear",
    "titeado": "titear",
    "titeados": "titear",
    "titeamos": "titear",
    "titean": "titear",
    "titeando": "titear",
    "titeara": "titear",
    "titearais": "titear",
    "titearan": "titear",
    "titearas": "titear",
    "titeare": "titear",
    "titeareis": "titear",
    "titearemos": "titear",
    "titearen": "titear",
    "titeares": "titear",
    "titearon": "titear",
    "titeará": "titear",
    "titearán": "titear",
    "titearás": "titear",
    "titearé": "titear",
    "titearéis": "titear",
    "titearía": "titear",
    "titearíais": "titear",
    "titearíamos": "titear",
    "titearían": "titear",
    "titearías": "titear",
    "titeas": "titear",
    "titease": "titear",
    "titeaseis": "titear",
    "titeasen": "titear",
    "titeases": "titear",
    "titeaste": "titear",
    "titeasteis": "titear",
    "titee": "titear",
    "titeemos": "titear",
    "titeen": "titear",
    "titees": "titear",
    "titeo": "titear",
    "titeos": "titeo",
    "titeábamos": "titear",
    "titeáis": "titear",
    "titeáramos": "titear",
    "titeáremos": "titear",
    "titeásemos": "titear",
    "titeé": "titear",
    "titeéis": "titear",
    "titeó": "titear",
    "titila": "titilar",
    "titilaba": "titilar",
    "titilabais": "titilar",
    "titilaban": "titilar",
    "titilabas": "titilar",
    "titilad": "titilar",
    "titilada": "titilar",
    "titiladas": "titilar",
    "titilado": "titilar",
    "titilados": "titilar",
    "titilamos": "titilar",
    "titilan": "titilar",
    "titilando": "titilar",
    "titilara": "titilar",
    "titilarais": "titilar",
    "titilaran": "titilar",
    "titilaras": "titilar",
    "titilare": "titilar",
    "titilareis": "titilar",
    "titilaremos": "titilar",
    "titilaren": "titilar",
    "titilares": "titilar",
    "titilaron": "titilar",
    "titilará": "titilar",
    "titilarán": "titilar",
    "titilarás": "titilar",
    "titilaré": "titilar",
    "titilaréis": "titilar",
    "titilaría": "titilar",
    "titilaríais": "titilar",
    "titilaríamos": "titilar",
    "titilarían": "titilar",
    "titilarías": "titilar",
    "titilas": "titilar",
    "titilase": "titilar",
    "titilaseis": "titilar",
    "titilasen": "titilar",
    "titilases": "titilar",
    "titilaste": "titilar",
    "titilasteis": "titilar",
    "titile": "titilar",
    "titilemos": "titilar",
    "titilen": "titilar",
    "titileos": "titileo",
    "titiles": "titilar",
    "titilo": "titilar",
    "titilábamos": "titilar",
    "titiláis": "titilar",
    "titiláramos": "titilar",
    "titiláremos": "titilar",
    "titilásemos": "titilar",
    "titilé": "titilar",
    "titiléis": "titilar",
    "titiló": "titilar",
    "titirimundis": "titirimundi",
    "titirita": "titiritar",
    "titiritaba": "titiritar",
    "titiritabais": "titiritar",
    "titiritaban": "titiritar",
    "titiritabas": "titiritar",
    "titiritad": "titiritar",
    "titiritada": "titiritar",
    "titiritadas": "titiritar",
    "titiritado": "titiritar",
    "titiritados": "titiritar",
    "titiritainas": "titiritaina",
    "titiritamos": "titiritar",
    "titiritan": "titiritar",
    "titiritando": "titiritar",
    "titiritara": "titiritar",
    "titiritarais": "titiritar",
    "titiritaran": "titiritar",
    "titiritaras": "titiritar",
    "titiritare": "titiritar",
    "titiritareis": "titiritar",
    "titiritaremos": "titiritar",
    "titiritaren": "titiritar",
    "titiritares": "titiritar",
    "titiritaron": "titiritar",
    "titiritará": "titiritar",
    "titiritarán": "titiritar",
    "titiritarás": "titiritar",
    "titiritaré": "titiritar",
    "titiritaréis": "titiritar",
    "titiritaría": "titiritar",
    "titiritaríais": "titiritar",
    "titiritaríamos": "titiritar",
    "titiritarían": "titiritar",
    "titiritarías": "titiritar",
    "titiritas": "titiritar",
    "titiritase": "titiritar",
    "titiritaseis": "titiritar",
    "titiritasen": "titiritar",
    "titiritases": "titiritar",
    "titiritaste": "titiritar",
    "titiritasteis": "titiritar",
    "titirite": "titiritar",
    "titiritemos": "titiritar",
    "titiriten": "titiritar",
    "titiritera": "titiritero",
    "titiriteras": "titiritero",
    "titiriteros": "titiritero",
    "titirites": "titiritar",
    "titirito": "titiritar",
    "titiritábamos": "titiritar",
    "titiritáis": "titiritar",
    "titiritáramos": "titiritar",
    "titiritáremos": "titiritar",
    "titiritásemos": "titiritar",
    "titirité": "titiritar",
    "titiritéis": "titiritar",
    "titiritó": "titiritar",
    "titos": "tito",
    "titrimetrías": "titrimetría",
    "titubea": "titubear",
    "titubeaba": "titubear",
    "titubeabais": "titubear",
    "titubeaban": "titubear",
    "titubeabas": "titubear",
    "titubead": "titubear",
    "titubeada": "titubear",
    "titubeadas": "titubear",
    "titubeado": "titubear",
    "titubeados": "titubear",
    "titubeamos": "titubear",
    "titubean": "titubear",
    "titubeando": "titubear",
    "titubeantes": "titubeante",
    "titubeara": "titubear",
    "titubearais": "titubear",
    "titubearan": "titubear",
    "titubearas": "titubear",
    "titubeare": "titubear",
    "titubeareis": "titubear",
    "titubearemos": "titubear",
    "titubearen": "titubear",
    "titubeares": "titubear",
    "titubearon": "titubear",
    "titubeará": "titubear",
    "titubearán": "titubear",
    "titubearás": "titubear",
    "titubearé": "titubear",
    "titubearéis": "titubear",
    "titubearía": "titubear",
    "titubearíais": "titubear",
    "titubearíamos": "titubear",
    "titubearían": "titubear",
    "titubearías": "titubear",
    "titubeas": "titubear",
    "titubease": "titubear",
    "titubeaseis": "titubear",
    "titubeasen": "titubear",
    "titubeases": "titubear",
    "titubeaste": "titubear",
    "titubeasteis": "titubear",
    "titubee": "titubear",
    "titubeemos": "titubear",
    "titubeen": "titubear",
    "titubees": "titubear",
    "titubeo": "titubear",
    "titubeos": "titubeo",
    "titubeábamos": "titubear",
    "titubeáis": "titubear",
    "titubeáramos": "titubear",
    "titubeáremos": "titubear",
    "titubeásemos": "titubear",
    "titubeé": "titubear",
    "titubeéis": "titubear",
    "titubeó": "titubear",
    "titula": "titular",
    "titulaba": "titular",
    "titulabais": "titular",
    "titulaban": "titular",
    "titulabas": "titular",
    "titulaciones": "titulación",
    "titulad": "titular",
    "titulada": "titular",
    "tituladas": "titular",
    "titulado": "titular",
    "tituladora": "titulador",
    "tituladoras": "titulador",
    "tituladores": "titulador",
    "titulados": "titular",
    "titulamos": "titular",
    "titulan": "titular",
    "titulando": "titular",
    "titulara": "titular",
    "titularais": "titular",
    "titularan": "titular",
    "titularas": "titular",
    "titulare": "titular",
    "titulareis": "titular",
    "titularemos": "titular",
    "titularen": "titular",
    "titulares": "titular",
    "titularidades": "titularidad",
    "titularizaciones": "titularización",
    "titularon": "titular",
    "titulará": "titular",
    "titularán": "titular",
    "titularás": "titular",
    "titularé": "titular",
    "titularéis": "titular",
    "titularía": "titular",
    "titularíais": "titular",
    "titularíamos": "titular",
    "titularían": "titular",
    "titularías": "titular",
    "titulas": "titular",
    "titulase": "titular",
    "titulaseis": "titular",
    "titulasen": "titular",
    "titulases": "titular",
    "titulaste": "titular",
    "titulasteis": "titular",
    "titule": "titular",
    "titulemos": "titular",
    "titulen": "titular",
    "titules": "titular",
    "titulillos": "titulillo",
    "titulo": "titular",
    "titulometrías": "titulometría",
    "titulábamos": "titular",
    "tituláis": "titular",
    "tituláramos": "titular",
    "tituláremos": "titular",
    "titulásemos": "titular",
    "titulé": "titular",
    "tituléis": "titular",
    "tituló": "titular",
    "titánica": "titánico",
    "titánicas": "titánico",
    "titánicos": "titánico",
    "titíes": "tití",
    "titímalos": "titímalo",
    "titís": "tití",
    "titíses": "tití",
    "tiufados": "tiufado",
    "tixotropos": "tixotropo",
    "tixotropías": "tixotropía",
    "tizas": "tiza",
    "tizna": "tiznar",
    "tiznaba": "tiznar",
    "tiznabais": "tiznar",
    "tiznaban": "tiznar",
    "tiznabas": "tiznar",
    "tiznad": "tiznar",
    "tiznada": "tiznar",
    "tiznadas": "tiznar",
    "tiznado": "tiznar",
    "tiznados": "tiznar",
    "tiznaduras": "tiznadura",
    "tiznamos": "tiznar",
    "tiznan": "tiznar",
    "tiznando": "tiznar",
    "tiznara": "tiznar",
    "tiznarais": "tiznar",
    "tiznaran": "tiznar",
    "tiznaras": "tiznar",
    "tiznare": "tiznar",
    "tiznareis": "tiznar",
    "tiznaremos": "tiznar",
    "tiznaren": "tiznar",
    "tiznares": "tiznar",
    "tiznaron": "tiznar",
    "tiznará": "tiznar",
    "tiznarán": "tiznar",
    "tiznarás": "tiznar",
    "tiznaré": "tiznar",
    "tiznaréis": "tiznar",
    "tiznaría": "tiznar",
    "tiznaríais": "tiznar",
    "tiznaríamos": "tiznar",
    "tiznarían": "tiznar",
    "tiznarías": "tiznar",
    "tiznas": "tiznar",
    "tiznase": "tiznar",
    "tiznaseis": "tiznar",
    "tiznasen": "tiznar",
    "tiznases": "tiznar",
    "tiznaste": "tiznar",
    "tiznasteis": "tiznar",
    "tizne": "tiznar",
    "tiznemos": "tiznar",
    "tiznen": "tiznar",
    "tiznera": "tiznero",
    "tizneras": "tiznero",
    "tizneros": "tiznero",
    "tiznes": "tizne",
    "tizno": "tiznar",
    "tiznones": "tiznón",
    "tiznábamos": "tiznar",
    "tiznáis": "tiznar",
    "tiznáramos": "tiznar",
    "tiznáremos": "tiznar",
    "tiznásemos": "tiznar",
    "tizné": "tiznar",
    "tiznéis": "tiznar",
    "tiznó": "tiznar",
    "tizonadas": "tizonada",
    "tizonas": "tizona",
    "tizonazos": "tizonazo",
    "tizonea": "tizonear",
    "tizoneaba": "tizonear",
    "tizoneabais": "tizonear",
    "tizoneaban": "tizonear",
    "tizoneabas": "tizonear",
    "tizonead": "tizonear",
    "tizoneada": "tizonear",
    "tizoneadas": "tizonear",
    "tizoneado": "tizonear",
    "tizoneados": "tizonear",
    "tizoneamos": "tizonear",
    "tizonean": "tizonear",
    "tizoneando": "tizonear",
    "tizoneara": "tizonear",
    "tizonearais": "tizonear",
    "tizonearan": "tizonear",
    "tizonearas": "tizonear",
    "tizoneare": "tizonear",
    "tizoneareis": "tizonear",
    "tizonearemos": "tizonear",
    "tizonearen": "tizonear",
    "tizoneares": "tizonear",
    "tizonearon": "tizonear",
    "tizoneará": "tizonear",
    "tizonearán": "tizonear",
    "tizonearás": "tizonear",
    "tizonearé": "tizonear",
    "tizonearéis": "tizonear",
    "tizonearía": "tizonear",
    "tizonearíais": "tizonear",
    "tizonearíamos": "tizonear",
    "tizonearían": "tizonear",
    "tizonearías": "tizonear",
    "tizoneas": "tizonear",
    "tizonease": "tizonear",
    "tizoneaseis": "tizonear",
    "tizoneasen": "tizonear",
    "tizoneases": "tizonear",
    "tizoneaste": "tizonear",
    "tizoneasteis": "tizonear",
    "tizonee": "tizonear",
    "tizoneemos": "tizonear",
    "tizoneen": "tizonear",
    "tizonees": "tizonear",
    "tizoneo": "tizonear",
    "tizoneras": "tizonera",
    "tizones": "tizón",
    "tizoneábamos": "tizonear",
    "tizoneáis": "tizonear",
    "tizoneáramos": "tizonear",
    "tizoneáremos": "tizonear",
    "tizoneásemos": "tizonear",
    "tizoneé": "tizonear",
    "tizoneéis": "tizonear",
    "tizoneó": "tizonear",
    "tizos": "tizo",
    "tiña": "teñir",
    "tiñamos": "teñir",
    "tiñan": "teñir",
    "tiñas": "tiña",
    "tiñe": "teñir",
    "tiñen": "teñir",
    "tiñendo": "teñir",
    "tiñera": "teñir",
    "tiñerais": "teñir",
    "tiñeran": "teñir",
    "tiñeras": "teñir",
    "tiñere": "teñir",
    "tiñereis": "teñir",
    "tiñeren": "teñir",
    "tiñeres": "teñir",
    "tiñeron": "teñir",
    "tiñes": "teñir",
    "tiñese": "teñir",
    "tiñeseis": "teñir",
    "tiñesen": "teñir",
    "tiñeses": "teñir",
    "tiño": "teñir",
    "tiñosa": "tiñoso",
    "tiñosas": "tiñoso",
    "tiñosos": "tiñoso",
    "tiñuelas": "tiñuela",
    "tiñáis": "teñir",
    "tiñéramos": "teñir",
    "tiñéremos": "teñir",
    "tiñésemos": "teñir",
    "tiñó": "teñir",
    "tiónica": "tiónico",
    "tiónicas": "tiónico",
    "tiónicos": "tiónico",
    "tlatoanis": "tlatoani",
    "tlatoaníes": "tlatoani",
    "tlaxcaltecas": "tlaxcalteca",
    "toa": "toar",
    "toaba": "toar",
    "toabais": "toar",
    "toaban": "toar",
    "toabas": "toar",
    "toad": "toar",
    "toada": "toar",
    "toadas": "toar",
    "toado": "toar",
    "toados": "toar",
    "toallas": "toalla",
    "toalleros": "toallero",
    "toallitas": "toallita",
    "toamos": "toar",
    "toan": "toar",
    "toando": "toar",
    "toara": "toar",
    "toarais": "toar",
    "toaran": "toar",
    "toaras": "toar",
    "toare": "toar",
    "toareis": "toar",
    "toaremos": "toar",
    "toaren": "toar",
    "toares": "toar",
    "toaron": "toar",
    "toará": "toar",
    "toarán": "toar",
    "toarás": "toar",
    "toaré": "toar",
    "toaréis": "toar",
    "toaría": "toar",
    "toaríais": "toar",
    "toaríamos": "toar",
    "toarían": "toar",
    "toarías": "toar",
    "toas": "toar",
    "toase": "toar",
    "toaseis": "toar",
    "toasen": "toar",
    "toases": "toar",
    "toaste": "toar",
    "toasteis": "toar",
    "tobares": "tobar",
    "tobas": "toba",
    "toberas": "tobera",
    "tobillera": "tobillero",
    "tobilleras": "tobillero",
    "tobilleros": "tobillero",
    "tobillos": "tobillo",
    "toboganes": "tobogán",
    "tobosa": "toboso",
    "tobosas": "toboso",
    "tobosos": "toboso",
    "tobácea": "tobáceo",
    "tobáceas": "tobáceo",
    "tobáceos": "tobáceo",
    "toca": "tocar",
    "tocaba": "tocar",
    "tocabais": "tocar",
    "tocaban": "tocar",
    "tocabas": "tocar",
    "tocad": "tocar",
    "tocada": "tocar",
    "tocadas": "tocar",
    "tocado": "tocar",
    "tocadora": "tocador",
    "tocadoras": "tocador",
    "tocadores": "tocador",
    "tocados": "tocar",
    "tocaduras": "tocadura",
    "tocamientos": "tocamiento",
    "tocamos": "tocar",
    "tocan": "tocar",
    "tocando": "tocar",
    "tocara": "tocar",
    "tocarais": "tocar",
    "tocaran": "tocar",
    "tocaras": "tocar",
    "tocare": "tocar",
    "tocareis": "tocar",
    "tocaremos": "tocar",
    "tocaren": "tocar",
    "tocares": "tocar",
    "tocaron": "tocar",
    "tocará": "tocar",
    "tocarán": "tocar",
    "tocarás": "tocar",
    "tocaré": "tocar",
    "tocaréis": "tocar",
    "tocaría": "tocar",
    "tocaríais": "tocar",
    "tocaríamos": "tocar",
    "tocarían": "tocar",
    "tocarías": "tocar",
    "tocas": "tocar",
    "tocase": "tocar",
    "tocaseis": "tocar",
    "tocasen": "tocar",
    "tocases": "tocar",
    "tocaste": "tocar",
    "tocasteis": "tocar",
    "tocatas": "tocata",
    "tocaya": "tocayo",
    "tocayas": "tocayo",
    "tocayos": "tocayo",
    "toccatas": "toccata",
    "tocha": "tocho",
    "tochas": "tocho",
    "tochedades": "tochedad",
    "tochos": "tocho",
    "tocinera": "tocinero",
    "tocineras": "tocinero",
    "tocineros": "tocinero",
    "tocinerías": "tocinería",
    "tocinos": "tocino",
    "toco": "tocar",
    "tocoferoles": "tocoferol",
    "tocologías": "tocología",
    "tocomochos": "tocomocho",
    "tocona": "tocón",
    "toconales": "toconal",
    "toconas": "tocona",
    "tocones": "tocón",
    "tocororos": "tocororo",
    "tocros": "tocro",
    "tocábamos": "tocar",
    "tocáis": "tocar",
    "tocáramos": "tocar",
    "tocáremos": "tocar",
    "tocásemos": "tocar",
    "tocó": "tocar",
    "tocóloga": "tocólogo",
    "tocólogas": "tocólogo",
    "tocólogos": "tocólogo",
    "toda": "todo",
    "todabuenas": "todabuena",
    "todas": "todo",
    "todis": "todi",
    "todopoderosa": "todopoderoso",
    "todopoderosas": "todopoderoso",
    "todopoderosos": "todopoderoso",
    "todos": "todo",
    "todoterrenos": "todoterreno",
    "toe": "toar",
    "toemos": "toar",
    "toen": "toar",
    "toes": "toar",
    "toesas": "toesa",
    "tofahes": "tofah",
    "tofahs": "tofah",
    "toffees": "toffee",
    "tofos": "tofo",
    "togada": "togado",
    "togadas": "togado",
    "togados": "togado",
    "togas": "toga",
    "toilettes": "toilette",
    "toisones": "toisón",
    "tojales": "tojal",
    "tojinos": "tojino",
    "tojos": "tojo",
    "tolanos": "tolano",
    "tolda": "toldar",
    "toldaba": "toldar",
    "toldabais": "toldar",
    "toldaban": "toldar",
    "toldabas": "toldar",
    "toldad": "toldar",
    "toldada": "toldar",
    "toldadas": "toldar",
    "toldado": "toldar",
    "toldados": "toldar",
    "toldaduras": "toldadura",
    "toldamos": "toldar",
    "toldan": "toldar",
    "toldando": "toldar",
    "toldara": "toldar",
    "toldarais": "toldar",
    "toldaran": "toldar",
    "toldaras": "toldar",
    "toldare": "toldar",
    "toldareis": "toldar",
    "toldaremos": "toldar",
    "toldaren": "toldar",
    "toldares": "toldar",
    "toldaron": "toldar",
    "toldará": "toldar",
    "toldarán": "toldar",
    "toldarás": "toldar",
    "toldaré": "toldar",
    "toldaréis": "toldar",
    "toldaría": "toldar",
    "toldaríais": "toldar",
    "toldaríamos": "toldar",
    "toldarían": "toldar",
    "toldarías": "toldar",
    "toldas": "toldar",
    "toldase": "toldar",
    "toldaseis": "toldar",
    "toldasen": "toldar",
    "toldases": "toldar",
    "toldaste": "toldar",
    "toldasteis": "toldar",
    "tolde": "toldar",
    "toldemos": "toldar",
    "tolden": "toldar",
    "toldes": "toldar",
    "toldillas": "toldilla",
    "toldillos": "toldillo",
    "toldo": "toldar",
    "toldos": "toldo",
    "toldábamos": "toldar",
    "toldáis": "toldar",
    "toldáramos": "toldar",
    "toldáremos": "toldar",
    "toldásemos": "toldar",
    "toldé": "toldar",
    "toldéis": "toldar",
    "toldó": "toldar",
    "toledana": "toledano",
    "toledanas": "toledano",
    "toledanos": "toledano",
    "tolemaica": "tolemaico",
    "tolemaicas": "tolemaico",
    "tolemaicos": "tolemaico",
    "tolera": "tolerar",
    "toleraba": "tolerar",
    "tolerabais": "tolerar",
    "toleraban": "tolerar",
    "tolerabas": "tolerar",
    "tolerabilidades": "tolerabilidad",
    "tolerables": "tolerable",
    "tolerad": "tolerar",
    "tolerada": "tolerar",
    "toleradas": "tolerar",
    "tolerado": "tolerar",
    "tolerados": "tolerar",
    "toleramos": "tolerar",
    "toleran": "tolerar",
    "tolerancias": "tolerancia",
    "tolerando": "tolerar",
    "tolerantes": "tolerante",
    "tolerantismos": "tolerantismo",
    "tolerara": "tolerar",
    "tolerarais": "tolerar",
    "toleraran": "tolerar",
    "toleraras": "tolerar",
    "tolerare": "tolerar",
    "tolerareis": "tolerar",
    "toleraremos": "tolerar",
    "toleraren": "tolerar",
    "tolerares": "tolerar",
    "toleraron": "tolerar",
    "tolerará": "tolerar",
    "tolerarán": "tolerar",
    "tolerarás": "tolerar",
    "toleraré": "tolerar",
    "toleraréis": "tolerar",
    "toleraría": "tolerar",
    "toleraríais": "tolerar",
    "toleraríamos": "tolerar",
    "tolerarían": "tolerar",
    "tolerarías": "tolerar",
    "toleras": "tolerar",
    "tolerase": "tolerar",
    "toleraseis": "tolerar",
    "tolerasen": "tolerar",
    "tolerases": "tolerar",
    "toleraste": "tolerar",
    "tolerasteis": "tolerar",
    "tolere": "tolerar",
    "toleremos": "tolerar",
    "toleren": "tolerar",
    "toleres": "tolerar",
    "tolero": "tolerar",
    "tolerogénica": "tolerogénico",
    "tolerogénicas": "tolerogénico",
    "tolerogénicos": "tolerogénico",
    "tolerábamos": "tolerar",
    "toleráis": "tolerar",
    "toleráramos": "tolerar",
    "toleráremos": "tolerar",
    "tolerásemos": "tolerar",
    "toleré": "tolerar",
    "toleréis": "tolerar",
    "toleró": "tolerar",
    "tolerógenos": "tolerógeno",
    "toles": "tole",
    "toletes": "tolete",
    "tolidinas": "tolidina",
    "tolilaldehídos": "tolilaldehído",
    "tolildietanolaminas": "tolildietanolamina",
    "toliletanolaminas": "toliletanolamina",
    "tolladares": "tolladar",
    "tollas": "tolla",
    "tollinas": "tollina",
    "tollones": "tollón",
    "tollos": "tollo",
    "tolmetinas": "tolmetina",
    "tolmos": "tolmo",
    "tolondra": "tolondro",
    "tolondras": "tolondro",
    "tolondrona": "tolondrón",
    "tolondronas": "tolondrón",
    "tolondrones": "tolondrón",
    "tolondros": "tolondro",
    "toltecas": "tolteca",
    "toluenos": "tolueno",
    "toluensulfanilidas": "toluensulfanilida",
    "toluensulfonamidas": "toluensulfonamida",
    "toluensulfonatos": "toluensulfonato",
    "toluensulfónica": "toluensulfónico",
    "toluensulfónicas": "toluensulfónico",
    "toluensulfónicos": "toluensulfónico",
    "toluhidroquinonas": "toluhidroquinona",
    "toluica": "toluico",
    "toluicas": "toluico",
    "toluicos": "toluico",
    "toluidinas": "toluidina",
    "toluquinonas": "toluquinona",
    "tolvaneras": "tolvanera",
    "tolvas": "tolva",
    "toma": "tomar",
    "tomaba": "tomar",
    "tomabais": "tomar",
    "tomaban": "tomar",
    "tomabas": "tomar",
    "tomacorrientes": "tomacorriente",
    "tomad": "tomar",
    "tomada": "tomar",
    "tomadas": "tomar",
    "tomado": "tomar",
    "tomadora": "tomador",
    "tomadoras": "tomador",
    "tomadores": "tomador",
    "tomados": "tomar",
    "tomaduras": "tomadura",
    "tomahawks": "tomahawk",
    "tomajona": "tomajón",
    "tomajonas": "tomajón",
    "tomajones": "tomajón",
    "tomamos": "tomar",
    "toman": "tomar",
    "tomando": "tomar",
    "tomara": "tomar",
    "tomarais": "tomar",
    "tomaran": "tomar",
    "tomaras": "tomar",
    "tomare": "tomar",
    "tomareis": "tomar",
    "tomaremos": "tomar",
    "tomaren": "tomar",
    "tomares": "tomar",
    "tomaron": "tomar",
    "tomará": "tomar",
    "tomarán": "tomar",
    "tomarás": "tomar",
    "tomaré": "tomar",
    "tomaréis": "tomar",
    "tomaría": "tomar",
    "tomaríais": "tomar",
    "tomaríamos": "tomar",
    "tomarían": "tomar",
    "tomarías": "tomar",
    "tomas": "tomar",
    "tomase": "tomar",
    "tomaseis": "tomar",
    "tomasen": "tomar",
    "tomases": "tomar",
    "tomaste": "tomar",
    "tomasteis": "tomar",
    "tomatadas": "tomatada",
    "tomatazos": "tomatazo",
    "tomateras": "tomatera",
    "tomates": "tomate",
    "tomawaks": "tomawak",
    "tomaínas": "tomaína",
    "tome": "tomar",
    "tomemos": "tomar",
    "tomen": "tomar",
    "tomentos": "tomento",
    "tomentosa": "tomentoso",
    "tomentosas": "tomentoso",
    "tomentosos": "tomentoso",
    "tomes": "tomar",
    "tomillares": "tomillar",
    "tomillos": "tomillo",
    "tomismos": "tomismo",
    "tomistas": "tomista",
    "tomizas": "tomiza",
    "tomo": "tomar",
    "tomografías": "tomografía",
    "tomos": "tomo",
    "tomsonitas": "tomsonita",
    "tomábamos": "tomar",
    "tomáis": "tomar",
    "tomáramos": "tomar",
    "tomáremos": "tomar",
    "tomásemos": "tomar",
    "tomé": "tomar",
    "toméis": "tomar",
    "tomística": "tomístico",
    "tomísticas": "tomístico",
    "tomísticos": "tomístico",
    "tomó": "tomar",
    "tonaba": "tonar",
    "tonabais": "tonar",
    "tonaban": "tonar",
    "tonabas": "tonar",
    "tonad": "tonar",
    "tonada": "tonar",
    "tonadas": "tonar",
    "tonadillas": "tonadilla",
    "tonadillera": "tonadillero",
    "tonadilleras": "tonadillero",
    "tonadilleros": "tonadillero",
    "tonado": "tonar",
    "tonados": "tonar",
    "tonales": "tonal",
    "tonalidades": "tonalidad",
    "tonamos": "tonar",
    "tonando": "tonar",
    "tonantes": "tonante",
    "tonara": "tonar",
    "tonarais": "tonar",
    "tonaran": "tonar",
    "tonaras": "tonar",
    "tonare": "tonar",
    "tonareis": "tonar",
    "tonaremos": "tonar",
    "tonaren": "tonar",
    "tonares": "tonar",
    "tonaron": "tonar",
    "tonará": "tonar",
    "tonarán": "tonar",
    "tonarás": "tonar",
    "tonaré": "tonar",
    "tonaréis": "tonar",
    "tonaría": "tonar",
    "tonaríais": "tonar",
    "tonaríamos": "tonar",
    "tonarían": "tonar",
    "tonarías": "tonar",
    "tonase": "tonar",
    "tonaseis": "tonar",
    "tonasen": "tonar",
    "tonases": "tonar",
    "tonaste": "tonar",
    "tonasteis": "tonar",
    "tondices": "tondiz",
    "tondinos": "tondino",
    "tondos": "tondo",
    "toneladas": "tonelada",
    "tonelajes": "tonelaje",
    "tonelera": "tonelero",
    "toneleras": "tonelero",
    "toneleros": "tonelero",
    "tonelerías": "tonelería",
    "toneles": "tonel",
    "toneletes": "tonelete",
    "tonemas": "tonema",
    "tonemos": "tonar",
    "tongadas": "tongada",
    "tongas": "tonga",
    "tongos": "tongo",
    "tonicidades": "tonicidad",
    "tonifica": "tonificar",
    "tonificaba": "tonificar",
    "tonificabais": "tonificar",
    "tonificaban": "tonificar",
    "tonificabas": "tonificar",
    "tonificaciones": "tonificación",
    "tonificad": "tonificar",
    "tonificada": "tonificar",
    "tonificadas": "tonificar",
    "tonificado": "tonificar",
    "tonificadora": "tonificador",
    "tonificadoras": "tonificador",
    "tonificadores": "tonificador",
    "tonificados": "tonificar",
    "tonificamos": "tonificar",
    "tonifican": "tonificar",
    "tonificando": "tonificar",
    "tonificantes": "tonificante",
    "tonificara": "tonificar",
    "tonificarais": "tonificar",
    "tonificaran": "tonificar",
    "tonificaras": "tonificar",
    "tonificare": "tonificar",
    "tonificareis": "tonificar",
    "tonificaremos": "tonificar",
    "tonificaren": "tonificar",
    "tonificares": "tonificar",
    "tonificaron": "tonificar",
    "tonificará": "tonificar",
    "tonificarán": "tonificar",
    "tonificarás": "tonificar",
    "tonificaré": "tonificar",
    "tonificaréis": "tonificar",
    "tonificaría": "tonificar",
    "tonificaríais": "tonificar",
    "tonificaríamos": "tonificar",
    "tonificarían": "tonificar",
    "tonificarías": "tonificar",
    "tonificas": "tonificar",
    "tonificase": "tonificar",
    "tonificaseis": "tonificar",
    "tonificasen": "tonificar",
    "tonificases": "tonificar",
    "tonificaste": "tonificar",
    "tonificasteis": "tonificar",
    "tonifico": "tonificar",
    "tonificábamos": "tonificar",
    "tonificáis": "tonificar",
    "tonificáramos": "tonificar",
    "tonificáremos": "tonificar",
    "tonificásemos": "tonificar",
    "tonificó": "tonificar",
    "tonifique": "tonificar",
    "tonifiquemos": "tonificar",
    "tonifiquen": "tonificar",
    "tonifiques": "tonificar",
    "tonifiqué": "tonificar",
    "tonifiquéis": "tonificar",
    "tonillos": "tonillo",
    "tonofilamentos": "tonofilamento",
    "tonografías": "tonografía",
    "tonometrías": "tonometría",
    "tonoplastos": "tonoplasto",
    "tonos": "tono",
    "tonotropismos": "tonotropismo",
    "tonsilares": "tonsilar",
    "tonsilas": "tonsila",
    "tonsilectomías": "tonsilectomía",
    "tonsilotomías": "tonsilotomía",
    "tonsura": "tonsurar",
    "tonsuraba": "tonsurar",
    "tonsurabais": "tonsurar",
    "tonsuraban": "tonsurar",
    "tonsurabas": "tonsurar",
    "tonsurad": "tonsurar",
    "tonsurada": "tonsurar",
    "tonsuradas": "tonsurar",
    "tonsurado": "tonsurar",
    "tonsurados": "tonsurar",
    "tonsuramos": "tonsurar",
    "tonsuran": "tonsurar",
    "tonsurando": "tonsurar",
    "tonsurara": "tonsurar",
    "tonsurarais": "tonsurar",
    "tonsuraran": "tonsurar",
    "tonsuraras": "tonsurar",
    "tonsurare": "tonsurar",
    "tonsurareis": "tonsurar",
    "tonsuraremos": "tonsurar",
    "tonsuraren": "tonsurar",
    "tonsurares": "tonsurar",
    "tonsuraron": "tonsurar",
    "tonsurará": "tonsurar",
    "tonsurarán": "tonsurar",
    "tonsurarás": "tonsurar",
    "tonsuraré": "tonsurar",
    "tonsuraréis": "tonsurar",
    "tonsuraría": "tonsurar",
    "tonsuraríais": "tonsurar",
    "tonsuraríamos": "tonsurar",
    "tonsurarían": "tonsurar",
    "tonsurarías": "tonsurar",
    "tonsuras": "tonsurar",
    "tonsurase": "tonsurar",
    "tonsuraseis": "tonsurar",
    "tonsurasen": "tonsurar",
    "tonsurases": "tonsurar",
    "tonsuraste": "tonsurar",
    "tonsurasteis": "tonsurar",
    "tonsure": "tonsurar",
    "tonsuremos": "tonsurar",
    "tonsuren": "tonsurar",
    "tonsures": "tonsurar",
    "tonsuro": "tonsurar",
    "tonsurábamos": "tonsurar",
    "tonsuráis": "tonsurar",
    "tonsuráramos": "tonsurar",
    "tonsuráremos": "tonsurar",
    "tonsurásemos": "tonsurar",
    "tonsuré": "tonsurar",
    "tonsuréis": "tonsurar",
    "tonsuró": "tonsurar",
    "tonta": "tonto",
    "tontadas": "tontada",
    "tontainas": "tontaina",
    "tontas": "tonto",
    "tontea": "tontear",
    "tonteaba": "tontear",
    "tonteabais": "tontear",
    "tonteaban": "tontear",
    "tonteabas": "tontear",
    "tontead": "tontear",
    "tonteada": "tontear",
    "tonteadas": "tontear",
    "tonteado": "tontear",
    "tonteados": "tontear",
    "tonteamos": "tontear",
    "tontean": "tontear",
    "tonteando": "tontear",
    "tonteara": "tontear",
    "tontearais": "tontear",
    "tontearan": "tontear",
    "tontearas": "tontear",
    "tonteare": "tontear",
    "tonteareis": "tontear",
    "tontearemos": "tontear",
    "tontearen": "tontear",
    "tonteares": "tontear",
    "tontearon": "tontear",
    "tonteará": "tontear",
    "tontearán": "tontear",
    "tontearás": "tontear",
    "tontearé": "tontear",
    "tontearéis": "tontear",
    "tontearía": "tontear",
    "tontearíais": "tontear",
    "tontearíamos": "tontear",
    "tontearían": "tontear",
    "tontearías": "tontear",
    "tonteas": "tontear",
    "tontease": "tontear",
    "tonteaseis": "tontear",
    "tonteasen": "tontear",
    "tonteases": "tontear",
    "tonteaste": "tontear",
    "tonteasteis": "tontear",
    "tontedades": "tontedad",
    "tontee": "tontear",
    "tonteemos": "tontear",
    "tonteen": "tontear",
    "tontees": "tontear",
    "tonteo": "tontear",
    "tonteras": "tontera",
    "tonterías": "tontería",
    "tonteábamos": "tontear",
    "tonteáis": "tontear",
    "tonteáramos": "tontear",
    "tonteáremos": "tontear",
    "tonteásemos": "tontear",
    "tonteé": "tontear",
    "tonteéis": "tontear",
    "tonteó": "tontear",
    "tontillos": "tontillo",
    "tontinas": "tontina",
    "tontos": "tonto",
    "tontunas": "tontuna",
    "tonábamos": "tonar",
    "tonáis": "tonar",
    "tonáramos": "tonar",
    "tonáremos": "tonar",
    "tonásemos": "tonar",
    "toné": "tonar",
    "tonéis": "tonar",
    "tonó": "tonar",
    "tonómetros": "tonómetro",
    "too": "toar",
    "topa": "topar",
    "topaba": "topar",
    "topabais": "topar",
    "topaban": "topar",
    "topabas": "topar",
    "topacios": "topacio",
    "topad": "topar",
    "topada": "topar",
    "topadas": "topar",
    "topadiza": "topadizo",
    "topadizas": "topadizo",
    "topadizos": "topadizo",
    "topado": "topar",
    "topadora": "topador",
    "topadoras": "topador",
    "topadores": "topador",
    "topados": "topar",
    "topamos": "topar",
    "topan": "topar",
    "topando": "topar",
    "topara": "topar",
    "toparais": "topar",
    "toparan": "topar",
    "toparas": "topar",
    "toparcas": "toparca",
    "topare": "topar",
    "topareis": "topar",
    "toparemos": "topar",
    "toparen": "topar",
    "topares": "topar",
    "toparon": "topar",
    "toparquías": "toparquía",
    "topará": "topar",
    "toparán": "topar",
    "toparás": "topar",
    "toparé": "topar",
    "toparéis": "topar",
    "toparía": "topar",
    "toparíais": "topar",
    "toparíamos": "topar",
    "toparían": "topar",
    "toparías": "topar",
    "topas": "topar",
    "topase": "topar",
    "topaseis": "topar",
    "topasen": "topar",
    "topases": "topar",
    "topaste": "topar",
    "topasteis": "topar",
    "topatopas": "topatopa",
    "topazolitas": "topazolita",
    "tope": "topar",
    "topemos": "topar",
    "topen": "topar",
    "toperas": "topera",
    "topes": "tope",
    "topeta": "topetar",
    "topetaba": "topetar",
    "topetabais": "topetar",
    "topetaban": "topetar",
    "topetabas": "topetar",
    "topetad": "topetar",
    "topetada": "topetar",
    "topetadas": "topetar",
    "topetado": "topetar",
    "topetados": "topetar",
    "topetamos": "topetar",
    "topetan": "topetar",
    "topetando": "topetar",
    "topetara": "topetar",
    "topetarais": "topetar",
    "topetaran": "topetar",
    "topetaras": "topetar",
    "topetare": "topetar",
    "topetareis": "topetar",
    "topetaremos": "topetar",
    "topetaren": "topetar",
    "topetares": "topetar",
    "topetaron": "topetar",
    "topetará": "topetar",
    "topetarán": "topetar",
    "topetarás": "topetar",
    "topetaré": "topetar",
    "topetaréis": "topetar",
    "topetaría": "topetar",
    "topetaríais": "topetar",
    "topetaríamos": "topetar",
    "topetarían": "topetar",
    "topetarías": "topetar",
    "topetas": "topetar",
    "topetase": "topetar",
    "topetaseis": "topetar",
    "topetasen": "topetar",
    "topetases": "topetar",
    "topetaste": "topetar",
    "topetasteis": "topetar",
    "topetazos": "topetazo",
    "topete": "topetar",
    "topetemos": "topetar",
    "topeten": "topetar",
    "topetes": "topetar",
    "topeto": "topetar",
    "topetones": "topetón",
    "topetuda": "topetudo",
    "topetudas": "topetudo",
    "topetudos": "topetudo",
    "topetábamos": "topetar",
    "topetáis": "topetar",
    "topetáramos": "topetar",
    "topetáremos": "topetar",
    "topetásemos": "topetar",
    "topeté": "topetar",
    "topetéis": "topetar",
    "topetó": "topetar",
    "topiaria": "topiario",
    "topiarias": "topiario",
    "topiarios": "topiario",
    "topillos": "topillo",
    "topina": "topino",
    "topinas": "topino",
    "topineras": "topinera",
    "topinos": "topino",
    "topo": "topar",
    "topofilias": "topofilia",
    "topofobias": "topofobia",
    "topografías": "topografía",
    "topográfica": "topográfico",
    "topográficas": "topográfico",
    "topográficos": "topográfico",
    "topologías": "topología",
    "topológica": "topológico",
    "topológicas": "topológico",
    "topológicos": "topológico",
    "topometrías": "topometría",
    "toponimias": "toponimia",
    "toponimistas": "toponimista",
    "toponomásticas": "toponomástica",
    "toponímica": "toponímico",
    "toponímicas": "toponímico",
    "toponímicos": "toponímico",
    "topoquímicas": "topoquímica",
    "toporas": "topora",
    "topos": "topo",
    "toposecuencias": "toposecuencia",
    "topotaxias": "topotaxia",
    "topotáctica": "topotáctico",
    "topotácticas": "topotáctico",
    "topotácticos": "topotáctico",
    "tops": "top",
    "topábamos": "topar",
    "topáis": "topar",
    "topáramos": "topar",
    "topáremos": "topar",
    "topásemos": "topar",
    "topé": "topar",
    "topéis": "topar",
    "topó": "topar",
    "topógrafa": "topógrafo",
    "topógrafas": "topógrafo",
    "topógrafos": "topógrafo",
    "topónimos": "topónimo",
    "toque": "tocar",
    "toqueados": "toqueado",
    "toquemos": "tocar",
    "toquen": "tocar",
    "toques": "toque",
    "toquetea": "toquetear",
    "toqueteaba": "toquetear",
    "toqueteabais": "toquetear",
    "toqueteaban": "toquetear",
    "toqueteabas": "toquetear",
    "toquetead": "toquetear",
    "toqueteada": "toquetear",
    "toqueteadas": "toquetear",
    "toqueteado": "toquetear",
    "toqueteados": "toquetear",
    "toqueteamos": "toquetear",
    "toquetean": "toquetear",
    "toqueteando": "toquetear",
    "toqueteara": "toquetear",
    "toquetearais": "toquetear",
    "toquetearan": "toquetear",
    "toquetearas": "toquetear",
    "toqueteare": "toquetear",
    "toqueteareis": "toquetear",
    "toquetearemos": "toquetear",
    "toquetearen": "toquetear",
    "toqueteares": "toquetear",
    "toquetearon": "toquetear",
    "toqueteará": "toquetear",
    "toquetearán": "toquetear",
    "toquetearás": "toquetear",
    "toquetearé": "toquetear",
    "toquetearéis": "toquetear",
    "toquetearía": "toquetear",
    "toquetearíais": "toquetear",
    "toquetearíamos": "toquetear",
    "toquetearían": "toquetear",
    "toquetearías": "toquetear",
    "toqueteas": "toquetear",
    "toquetease": "toquetear",
    "toqueteaseis": "toquetear",
    "toqueteasen": "toquetear",
    "toqueteases": "toquetear",
    "toqueteaste": "toquetear",
    "toqueteasteis": "toquetear",
    "toquetee": "toquetear",
    "toqueteemos": "toquetear",
    "toqueteen": "toquetear",
    "toquetees": "toquetear",
    "toqueteo": "toquetear",
    "toqueteos": "toqueteo",
    "toqueteábamos": "toquetear",
    "toqueteáis": "toquetear",
    "toqueteáramos": "toquetear",
    "toqueteáremos": "toquetear",
    "toqueteásemos": "toquetear",
    "toqueteé": "toquetear",
    "toqueteéis": "toquetear",
    "toqueteó": "toquetear",
    "toquillas": "toquilla",
    "toqué": "tocar",
    "toquéis": "tocar",
    "toracoacromiales": "toracoacromial",
    "toracodorsales": "toracodorsal",
    "toracoepigástrica": "toracoepigástrico",
    "toracoepigástricas": "toracoepigástrico",
    "toracoepigástricos": "toracoepigástrico",
    "toracotomías": "toracotomía",
    "toradas": "torada",
    "torales": "toral",
    "toras": "tora",
    "torbanitas": "torbanita",
    "torbellinos": "torbellino",
    "torbenitas": "torbenita",
    "torbernitas": "torbernita",
    "torcales": "torcal",
    "torcas": "torca",
    "torcazes": "torcaz",
    "torcecuellos": "torcecuello",
    "torced": "torcer",
    "torcedera": "torcedero",
    "torcederas": "torcedero",
    "torcederos": "torcedero",
    "torcedora": "torcedor",
    "torcedoras": "torcedora",
    "torcedores": "torcedor",
    "torceduras": "torcedura",
    "torcemos": "torcer",
    "torceremos": "torcer",
    "torcerá": "torcer",
    "torcerán": "torcer",
    "torcerás": "torcer",
    "torceré": "torcer",
    "torceréis": "torcer",
    "torcería": "torcer",
    "torceríais": "torcer",
    "torceríamos": "torcer",
    "torcerían": "torcer",
    "torcerías": "torcer",
    "torces": "torce",
    "torcida": "torcer",
    "torcidas": "torcer",
    "torcido": "torcer",
    "torcidos": "torcido",
    "torciendo": "torcer",
    "torciera": "torcer",
    "torcierais": "torcer",
    "torcieran": "torcer",
    "torcieras": "torcer",
    "torciere": "torcer",
    "torciereis": "torcer",
    "torcieren": "torcer",
    "torcieres": "torcer",
    "torcieron": "torcer",
    "torciese": "torcer",
    "torcieseis": "torcer",
    "torciesen": "torcer",
    "torcieses": "torcer",
    "torcijones": "torcijón",
    "torcimientos": "torcimiento",
    "torcimos": "torcer",
    "torciste": "torcer",
    "torcisteis": "torcer",
    "torciéramos": "torcer",
    "torciéremos": "torcer",
    "torciésemos": "torcer",
    "torció": "torcer",
    "torculada": "torculado",
    "torculadas": "torculado",
    "torculados": "torculado",
    "torcéis": "torcer",
    "torcí": "torcer",
    "torcía": "torcer",
    "torcíais": "torcer",
    "torcíamos": "torcer",
    "torcían": "torcer",
    "torcías": "torcer",
    "torda": "tordo",
    "tordas": "tordo",
    "tordilla": "tordillo",
    "tordillas": "tordillo",
    "tordillos": "tordillo",
    "tordos": "tordo",
    "torea": "torear",
    "toreaba": "torear",
    "toreabais": "torear",
    "toreaban": "torear",
    "toreabas": "torear",
    "toread": "torear",
    "toreada": "torear",
    "toreadas": "torear",
    "toreado": "torear",
    "toreadora": "toreador",
    "toreadoras": "toreador",
    "toreadores": "toreador",
    "toreados": "torear",
    "toreamos": "torear",
    "torean": "torear",
    "toreando": "torear",
    "toreara": "torear",
    "torearais": "torear",
    "torearan": "torear",
    "torearas": "torear",
    "toreare": "torear",
    "toreareis": "torear",
    "torearemos": "torear",
    "torearen": "torear",
    "toreares": "torear",
    "torearon": "torear",
    "toreará": "torear",
    "torearán": "torear",
    "torearás": "torear",
    "torearé": "torear",
    "torearéis": "torear",
    "torearía": "torear",
    "torearíais": "torear",
    "torearíamos": "torear",
    "torearían": "torear",
    "torearías": "torear",
    "toreas": "torear",
    "torease": "torear",
    "toreaseis": "torear",
    "toreasen": "torear",
    "toreases": "torear",
    "toreaste": "torear",
    "toreasteis": "torear",
    "toree": "torear",
    "toreemos": "torear",
    "toreen": "torear",
    "torees": "torear",
    "toreo": "torear",
    "toreos": "toreo",
    "torera": "torero",
    "toreras": "torero",
    "toreros": "torero",
    "torerías": "torería",
    "toreses": "torés",
    "toretes": "torete",
    "toreábamos": "torear",
    "toreáis": "torear",
    "toreáramos": "torear",
    "toreáremos": "torear",
    "toreásemos": "torear",
    "toreé": "torear",
    "toreéis": "torear",
    "toreó": "torear",
    "toriada": "toriado",
    "toriadas": "toriado",
    "toriados": "toriado",
    "tories": "tory",
    "toriles": "toril",
    "torillos": "torillo",
    "torionda": "toriondo",
    "toriondas": "toriondo",
    "toriondeces": "toriondez",
    "toriondos": "toriondo",
    "torios": "torio",
    "toritas": "torita",
    "torlorotos": "torloroto",
    "tormentaria": "tormentario",
    "tormentarias": "tormentario",
    "tormentarios": "tormentario",
    "tormentas": "tormenta",
    "tormentilas": "tormentila",
    "tormentos": "tormento",
    "tormentosa": "tormentoso",
    "tormentosas": "tormentoso",
    "tormentosos": "tormentoso",
    "tormos": "tormo",
    "torna": "tornar",
    "tornaba": "tornar",
    "tornabais": "tornar",
    "tornaban": "tornar",
    "tornabas": "tornar",
    "tornabodas": "tornaboda",
    "tornad": "tornar",
    "tornada": "tornar",
    "tornadas": "tornar",
    "tornaderas": "tornadera",
    "tornadiza": "tornadizo",
    "tornadizas": "tornadizo",
    "tornadizos": "tornadizo",
    "tornado": "tornar",
    "tornados": "tornar",
    "tornaduras": "tornadura",
    "tornaguías": "tornaguía",
    "tornalechos": "tornalecho",
    "tornamientos": "tornamiento",
    "tornamos": "tornar",
    "tornan": "tornar",
    "tornando": "tornar",
    "tornapuntas": "tornapunta",
    "tornara": "tornar",
    "tornarais": "tornar",
    "tornaran": "tornar",
    "tornaras": "tornar",
    "tornare": "tornar",
    "tornareis": "tornar",
    "tornaremos": "tornar",
    "tornaren": "tornar",
    "tornares": "tornar",
    "tornaron": "tornar",
    "tornará": "tornar",
    "tornarán": "tornar",
    "tornarás": "tornar",
    "tornaré": "tornar",
    "tornaréis": "tornar",
    "tornaría": "tornar",
    "tornaríais": "tornar",
    "tornaríamos": "tornar",
    "tornarían": "tornar",
    "tornarías": "tornar",
    "tornas": "tornar",
    "tornase": "tornar",
    "tornaseis": "tornar",
    "tornasen": "tornar",
    "tornases": "tornar",
    "tornasola": "tornasolar",
    "tornasolaba": "tornasolar",
    "tornasolabais": "tornasolar",
    "tornasolaban": "tornasolar",
    "tornasolabas": "tornasolar",
    "tornasolad": "tornasolar",
    "tornasolada": "tornasolar",
    "tornasoladas": "tornasolar",
    "tornasolado": "tornasolar",
    "tornasolados": "tornasolar",
    "tornasolamos": "tornasolar",
    "tornasolan": "tornasolar",
    "tornasolando": "tornasolar",
    "tornasolara": "tornasolar",
    "tornasolarais": "tornasolar",
    "tornasolaran": "tornasolar",
    "tornasolaras": "tornasolar",
    "tornasolare": "tornasolar",
    "tornasolareis": "tornasolar",
    "tornasolaremos": "tornasolar",
    "tornasolaren": "tornasolar",
    "tornasolares": "tornasolar",
    "tornasolaron": "tornasolar",
    "tornasolará": "tornasolar",
    "tornasolarán": "tornasolar",
    "tornasolarás": "tornasolar",
    "tornasolaré": "tornasolar",
    "tornasolaréis": "tornasolar",
    "tornasolaría": "tornasolar",
    "tornasolaríais": "tornasolar",
    "tornasolaríamos": "tornasolar",
    "tornasolarían": "tornasolar",
    "tornasolarías": "tornasolar",
    "tornasolas": "tornasolar",
    "tornasolase": "tornasolar",
    "tornasolaseis": "tornasolar",
    "tornasolasen": "tornasolar",
    "tornasolases": "tornasolar",
    "tornasolaste": "tornasolar",
    "tornasolasteis": "tornasolar",
    "tornasole": "tornasolar",
    "tornasolemos": "tornasolar",
    "tornasolen": "tornasolar",
    "tornasoles": "tornasolar",
    "tornasolo": "tornasolar",
    "tornasolábamos": "tornasolar",
    "tornasoláis": "tornasolar",
    "tornasoláramos": "tornasolar",
    "tornasoláremos": "tornasolar",
    "tornasolásemos": "tornasolar",
    "tornasolé": "tornasolar",
    "tornasoléis": "tornasolar",
    "tornasoló": "tornasolar",
    "tornaste": "tornar",
    "tornasteis": "tornar",
    "tornatrases": "tornatrás",
    "tornaviajes": "tornaviaje",
    "tornavoces": "tornavoz",
    "torne": "tornar",
    "tornea": "tornear",
    "torneaba": "tornear",
    "torneabais": "tornear",
    "torneaban": "tornear",
    "torneabas": "tornear",
    "tornead": "tornear",
    "torneada": "tornear",
    "torneadas": "tornear",
    "torneado": "tornear",
    "torneados": "tornear",
    "torneamos": "tornear",
    "tornean": "tornear",
    "torneando": "tornear",
    "torneara": "tornear",
    "tornearais": "tornear",
    "tornearan": "tornear",
    "tornearas": "tornear",
    "torneare": "tornear",
    "torneareis": "tornear",
    "tornearemos": "tornear",
    "tornearen": "tornear",
    "torneares": "tornear",
    "tornearon": "tornear",
    "torneará": "tornear",
    "tornearán": "tornear",
    "tornearás": "tornear",
    "tornearé": "tornear",
    "tornearéis": "tornear",
    "tornearía": "tornear",
    "tornearíais": "tornear",
    "tornearíamos": "tornear",
    "tornearían": "tornear",
    "tornearías": "tornear",
    "torneas": "tornear",
    "tornease": "tornear",
    "torneaseis": "tornear",
    "torneasen": "tornear",
    "torneases": "tornear",
    "torneaste": "tornear",
    "torneasteis": "tornear",
    "tornee": "tornear",
    "torneemos": "tornear",
    "torneen": "tornear",
    "tornees": "tornear",
    "tornemos": "tornar",
    "tornen": "tornar",
    "torneo": "tornear",
    "torneos": "torneo",
    "tornera": "tornero",
    "torneras": "tornero",
    "torneros": "tornero",
    "tornerías": "tornería",
    "tornes": "tornar",
    "torneábamos": "tornear",
    "torneáis": "tornear",
    "torneáramos": "tornear",
    "torneáremos": "tornear",
    "torneásemos": "tornear",
    "torneé": "tornear",
    "torneéis": "tornear",
    "torneó": "tornear",
    "tornillerías": "tornillería",
    "tornillos": "tornillo",
    "torniquetes": "torniquete",
    "torniscones": "torniscón",
    "torno": "tornar",
    "tornos": "torno",
    "tornábamos": "tornar",
    "tornáis": "tornar",
    "tornáramos": "tornar",
    "tornáremos": "tornar",
    "tornásemos": "tornar",
    "tornátiles": "tornátil",
    "torné": "tornar",
    "tornéis": "tornar",
    "tornó": "tornar",
    "toroidales": "toroidal",
    "toroides": "toroide",
    "torones": "torón",
    "toronjas": "toronja",
    "toronjiles": "toronjil",
    "toronjinas": "toronjina",
    "toros": "toro",
    "torozones": "torozón",
    "torpedea": "torpedear",
    "torpedeaba": "torpedear",
    "torpedeabais": "torpedear",
    "torpedeaban": "torpedear",
    "torpedeabas": "torpedear",
    "torpedead": "torpedear",
    "torpedeada": "torpedear",
    "torpedeadas": "torpedear",
    "torpedeado": "torpedear",
    "torpedeados": "torpedear",
    "torpedeamos": "torpedear",
    "torpedean": "torpedear",
    "torpedeando": "torpedear",
    "torpedeara": "torpedear",
    "torpedearais": "torpedear",
    "torpedearan": "torpedear",
    "torpedearas": "torpedear",
    "torpedeare": "torpedear",
    "torpedeareis": "torpedear",
    "torpedearemos": "torpedear",
    "torpedearen": "torpedear",
    "torpedeares": "torpedear",
    "torpedearon": "torpedear",
    "torpedeará": "torpedear",
    "torpedearán": "torpedear",
    "torpedearás": "torpedear",
    "torpedearé": "torpedear",
    "torpedearéis": "torpedear",
    "torpedearía": "torpedear",
    "torpedearíais": "torpedear",
    "torpedearíamos": "torpedear",
    "torpedearían": "torpedear",
    "torpedearías": "torpedear",
    "torpedeas": "torpedear",
    "torpedease": "torpedear",
    "torpedeaseis": "torpedear",
    "torpedeasen": "torpedear",
    "torpedeases": "torpedear",
    "torpedeaste": "torpedear",
    "torpedeasteis": "torpedear",
    "torpedee": "torpedear",
    "torpedeemos": "torpedear",
    "torpedeen": "torpedear",
    "torpedees": "torpedear",
    "torpedeo": "torpedear",
    "torpedeos": "torpedeo",
    "torpedera": "torpedero",
    "torpederas": "torpedero",
    "torpederos": "torpedero",
    "torpedeábamos": "torpedear",
    "torpedeáis": "torpedear",
    "torpedeáramos": "torpedear",
    "torpedeáremos": "torpedear",
    "torpedeásemos": "torpedear",
    "torpedeé": "torpedear",
    "torpedeéis": "torpedear",
    "torpedeó": "torpedear",
    "torpedistas": "torpedista",
    "torpedos": "torpedo",
    "torpes": "torpe",
    "torpezas": "torpeza",
    "torpores": "torpor",
    "torra": "torrar",
    "torraba": "torrar",
    "torrabais": "torrar",
    "torraban": "torrar",
    "torrabas": "torrar",
    "torrad": "torrar",
    "torrada": "torrar",
    "torradas": "torrar",
    "torrado": "torrar",
    "torrados": "torrar",
    "torramos": "torrar",
    "torran": "torrar",
    "torrando": "torrar",
    "torrara": "torrar",
    "torrarais": "torrar",
    "torraran": "torrar",
    "torraras": "torrar",
    "torrare": "torrar",
    "torrareis": "torrar",
    "torraremos": "torrar",
    "torraren": "torrar",
    "torrares": "torrar",
    "torraron": "torrar",
    "torrará": "torrar",
    "torrarán": "torrar",
    "torrarás": "torrar",
    "torraré": "torrar",
    "torraréis": "torrar",
    "torraría": "torrar",
    "torraríais": "torrar",
    "torraríamos": "torrar",
    "torrarían": "torrar",
    "torrarías": "torrar",
    "torras": "torrar",
    "torrase": "torrar",
    "torraseis": "torrar",
    "torrasen": "torrar",
    "torrases": "torrar",
    "torraste": "torrar",
    "torrasteis": "torrar",
    "torre": "torrar",
    "torrea": "torrear",
    "torreaba": "torrear",
    "torreabais": "torrear",
    "torreaban": "torrear",
    "torreabas": "torrear",
    "torread": "torrear",
    "torreada": "torrear",
    "torreadas": "torrear",
    "torreado": "torrear",
    "torreados": "torrear",
    "torreamos": "torrear",
    "torrean": "torrear",
    "torreando": "torrear",
    "torreara": "torrear",
    "torrearais": "torrear",
    "torrearan": "torrear",
    "torrearas": "torrear",
    "torreare": "torrear",
    "torreareis": "torrear",
    "torrearemos": "torrear",
    "torrearen": "torrear",
    "torreares": "torrear",
    "torrearon": "torrear",
    "torreará": "torrear",
    "torrearán": "torrear",
    "torrearás": "torrear",
    "torrearé": "torrear",
    "torrearéis": "torrear",
    "torrearía": "torrear",
    "torrearíais": "torrear",
    "torrearíamos": "torrear",
    "torrearían": "torrear",
    "torrearías": "torrear",
    "torreas": "torrear",
    "torrease": "torrear",
    "torreaseis": "torrear",
    "torreasen": "torrear",
    "torreases": "torrear",
    "torreaste": "torrear",
    "torreasteis": "torrear",
    "torree": "torrear",
    "torreemos": "torrear",
    "torreen": "torrear",
    "torrees": "torrear",
    "torrefacciones": "torrefacción",
    "torrefacta": "torrefacto",
    "torrefactas": "torrefacto",
    "torrefactos": "torrefacto",
    "torrejones": "torrejón",
    "torremos": "torrar",
    "torren": "torrar",
    "torrenciales": "torrencial",
    "torrencialidades": "torrencialidad",
    "torrenteras": "torrentera",
    "torrentes": "torrente",
    "torrentosa": "torrentoso",
    "torrentosas": "torrentoso",
    "torrentosos": "torrentoso",
    "torreo": "torrear",
    "torreones": "torreón",
    "torrera": "torrero",
    "torreras": "torrero",
    "torreros": "torrero",
    "torres": "torre",
    "torretas": "torreta",
    "torreznadas": "torreznada",
    "torreznera": "torreznero",
    "torrezneras": "torreznero",
    "torrezneros": "torreznero",
    "torreznos": "torrezno",
    "torreábamos": "torrear",
    "torreáis": "torrear",
    "torreáramos": "torrear",
    "torreáremos": "torrear",
    "torreásemos": "torrear",
    "torreé": "torrear",
    "torreéis": "torrear",
    "torreó": "torrear",
    "torrijas": "torrija",
    "torro": "torrar",
    "torrontera": "torrontero",
    "torronteras": "torrontero",
    "torronteros": "torrontero",
    "torrs": "torr",
    "torrábamos": "torrar",
    "torráis": "torrar",
    "torráramos": "torrar",
    "torráremos": "torrar",
    "torrásemos": "torrar",
    "torré": "torrar",
    "torréis": "torrar",
    "torró": "torrar",
    "torsiones": "torsión",
    "torsiómetros": "torsiómetro",
    "torsores": "torsor",
    "torsos": "torso",
    "tortadas": "tortada",
    "tortas": "torta",
    "tortazos": "tortazo",
    "tortellinis": "tortellini",
    "tortera": "tortero",
    "torteras": "tortero",
    "torteros": "tortero",
    "torticera": "torticero",
    "torticeras": "torticero",
    "torticeros": "torticero",
    "tortillas": "tortilla",
    "tortillos": "tortillo",
    "tortitas": "tortita",
    "tortolita": "tortolito",
    "tortolitas": "tortolito",
    "tortolitos": "tortolito",
    "tortores": "tortor",
    "tortosina": "tortosino",
    "tortosinas": "tortosino",
    "tortosinos": "tortosino",
    "tortozones": "tortozón",
    "tortugas": "tortuga",
    "tortuosa": "tortuoso",
    "tortuosas": "tortuoso",
    "tortuosidades": "tortuosidad",
    "tortuosos": "tortuoso",
    "tortura": "torturar",
    "torturaba": "torturar",
    "torturabais": "torturar",
    "torturaban": "torturar",
    "torturabas": "torturar",
    "torturad": "torturar",
    "torturada": "torturar",
    "torturadas": "torturar",
    "torturado": "torturar",
    "torturadora": "torturador",
    "torturadoras": "torturador",
    "torturadores": "torturador",
    "torturados": "torturar",
    "torturamos": "torturar",
    "torturan": "torturar",
    "torturando": "torturar",
    "torturara": "torturar",
    "torturarais": "torturar",
    "torturaran": "torturar",
    "torturaras": "torturar",
    "torturare": "torturar",
    "torturareis": "torturar",
    "torturaremos": "torturar",
    "torturaren": "torturar",
    "torturares": "torturar",
    "torturaron": "torturar",
    "torturará": "torturar",
    "torturarán": "torturar",
    "torturarás": "torturar",
    "torturaré": "torturar",
    "torturaréis": "torturar",
    "torturaría": "torturar",
    "torturaríais": "torturar",
    "torturaríamos": "torturar",
    "torturarían": "torturar",
    "torturarías": "torturar",
    "torturas": "torturar",
    "torturase": "torturar",
    "torturaseis": "torturar",
    "torturasen": "torturar",
    "torturases": "torturar",
    "torturaste": "torturar",
    "torturasteis": "torturar",
    "torture": "torturar",
    "torturemos": "torturar",
    "torturen": "torturar",
    "tortures": "torturar",
    "torturo": "torturar",
    "torturábamos": "torturar",
    "torturáis": "torturar",
    "torturáramos": "torturar",
    "torturáremos": "torturar",
    "torturásemos": "torturar",
    "torturé": "torturar",
    "torturéis": "torturar",
    "torturó": "torturar",
    "torundas": "torunda",
    "torva": "torvo",
    "torvas": "torvo",
    "torviscos": "torvisco",
    "torvos": "torvo",
    "torzadillos": "torzadillo",
    "torzales": "torzal",
    "torzamos": "torcer",
    "torzones": "torzón",
    "torzáis": "torcer",
    "torácica": "torácico",
    "torácicas": "torácico",
    "torácicos": "torácico",
    "toréuticas": "toréutica",
    "tosa": "toser",
    "tosamos": "toser",
    "tosan": "toser",
    "tosas": "toser",
    "tosca": "tosco",
    "toscana": "toscano",
    "toscanas": "toscano",
    "toscanos": "toscano",
    "toscas": "tosco",
    "toscos": "tosco",
    "tose": "toser",
    "tosed": "toser",
    "tosemos": "toser",
    "tosen": "toser",
    "toseremos": "toser",
    "toserá": "toser",
    "toserán": "toser",
    "toserás": "toser",
    "toseré": "toser",
    "toseréis": "toser",
    "tosería": "toser",
    "toseríais": "toser",
    "toseríamos": "toser",
    "toserían": "toser",
    "toserías": "toser",
    "toses": "toser",
    "tosida": "toser",
    "tosidas": "toser",
    "tosido": "toser",
    "tosidos": "toser",
    "tosiduras": "tosidura",
    "tosiendo": "toser",
    "tosiera": "toser",
    "tosierais": "toser",
    "tosieran": "toser",
    "tosieras": "toser",
    "tosiere": "toser",
    "tosiereis": "toser",
    "tosieren": "toser",
    "tosieres": "toser",
    "tosieron": "toser",
    "tosiese": "toser",
    "tosieseis": "toser",
    "tosiesen": "toser",
    "tosieses": "toser",
    "tosigosa": "tosigoso",
    "tosigosas": "tosigoso",
    "tosigosos": "tosigoso",
    "tosimos": "toser",
    "tosiste": "toser",
    "tosisteis": "toser",
    "tosiéramos": "toser",
    "tosiéremos": "toser",
    "tosiésemos": "toser",
    "tosió": "toser",
    "toso": "toser",
    "tosquedades": "tosquedad",
    "tostaba": "tostar",
    "tostabais": "tostar",
    "tostaban": "tostar",
    "tostabas": "tostar",
    "tostaciones": "tostación",
    "tostad": "tostar",
    "tostada": "tostar",
    "tostadas": "tostar",
    "tostaderos": "tostadero",
    "tostado": "tostar",
    "tostadora": "tostador",
    "tostadoras": "tostador",
    "tostadores": "tostador",
    "tostados": "tostar",
    "tostaduras": "tostadura",
    "tostamos": "tostar",
    "tostando": "tostar",
    "tostara": "tostar",
    "tostarais": "tostar",
    "tostaran": "tostar",
    "tostaras": "tostar",
    "tostare": "tostar",
    "tostareis": "tostar",
    "tostaremos": "tostar",
    "tostaren": "tostar",
    "tostares": "tostar",
    "tostaron": "tostar",
    "tostará": "tostar",
    "tostarán": "tostar",
    "tostarás": "tostar",
    "tostaré": "tostar",
    "tostaréis": "tostar",
    "tostaría": "tostar",
    "tostaríais": "tostar",
    "tostaríamos": "tostar",
    "tostarían": "tostar",
    "tostarías": "tostar",
    "tostase": "tostar",
    "tostaseis": "tostar",
    "tostasen": "tostar",
    "tostases": "tostar",
    "tostaste": "tostar",
    "tostasteis": "tostar",
    "tostemos": "tostar",
    "tostones": "tostón",
    "tostábamos": "tostar",
    "tostáis": "tostar",
    "tostáramos": "tostar",
    "tostáremos": "tostar",
    "tostásemos": "tostar",
    "tosté": "tostar",
    "tostéis": "tostar",
    "tostó": "tostar",
    "tosáis": "toser",
    "toséis": "toser",
    "tosí": "toser",
    "tosía": "toser",
    "tosíais": "toser",
    "tosíamos": "toser",
    "tosían": "toser",
    "tosías": "toser",
    "totales": "total",
    "totalice": "totalizar",
    "totalicemos": "totalizar",
    "totalicen": "totalizar",
    "totalices": "totalizar",
    "totalicé": "totalizar",
    "totalicéis": "totalizar",
    "totalidades": "totalidad",
    "totalitaria": "totalitario",
    "totalitarias": "totalitario",
    "totalitarios": "totalitario",
    "totalitarismos": "totalitarismo",
    "totalitaristas": "totalitarista",
    "totaliza": "totalizar",
    "totalizaba": "totalizar",
    "totalizabais": "totalizar",
    "totalizaban": "totalizar",
    "totalizabas": "totalizar",
    "totalizaciones": "totalización",
    "totalizad": "totalizar",
    "totalizada": "totalizar",
    "totalizadas": "totalizar",
    "totalizado": "totalizar",
    "totalizadora": "totalizador",
    "totalizadoras": "totalizador",
    "totalizadores": "totalizador",
    "totalizados": "totalizar",
    "totalizamos": "totalizar",
    "totalizan": "totalizar",
    "totalizando": "totalizar",
    "totalizara": "totalizar",
    "totalizarais": "totalizar",
    "totalizaran": "totalizar",
    "totalizaras": "totalizar",
    "totalizare": "totalizar",
    "totalizareis": "totalizar",
    "totalizaremos": "totalizar",
    "totalizaren": "totalizar",
    "totalizares": "totalizar",
    "totalizaron": "totalizar",
    "totalizará": "totalizar",
    "totalizarán": "totalizar",
    "totalizarás": "totalizar",
    "totalizaré": "totalizar",
    "totalizaréis": "totalizar",
    "totalizaría": "totalizar",
    "totalizaríais": "totalizar",
    "totalizaríamos": "totalizar",
    "totalizarían": "totalizar",
    "totalizarías": "totalizar",
    "totalizas": "totalizar",
    "totalizase": "totalizar",
    "totalizaseis": "totalizar",
    "totalizasen": "totalizar",
    "totalizases": "totalizar",
    "totalizaste": "totalizar",
    "totalizasteis": "totalizar",
    "totalizo": "totalizar",
    "totalizábamos": "totalizar",
    "totalizáis": "totalizar",
    "totalizáramos": "totalizar",
    "totalizáremos": "totalizar",
    "totalizásemos": "totalizar",
    "totalizó": "totalizar",
    "totas": "tota",
    "totemismos": "totemismo",
    "totoloques": "totoloque",
    "totonaca": "totonaco",
    "totonacas": "totonaco",
    "totonacos": "totonaco",
    "totovías": "totovía",
    "totémica": "totémico",
    "totémicas": "totémico",
    "totémicos": "totémico",
    "touches": "touche",
    "tournées": "tournée",
    "tours": "tour",
    "toxafenos": "toxafeno",
    "toxemias": "toxemia",
    "toxias": "toxia",
    "toxicidades": "toxicidad",
    "toxicologías": "toxicología",
    "toxicológica": "toxicológico",
    "toxicológicas": "toxicológico",
    "toxicológicos": "toxicológico",
    "toxicomanías": "toxicomanía",
    "toxicófora": "toxicóforo",
    "toxicóforas": "toxicóforo",
    "toxicóforos": "toxicóforo",
    "toxicógena": "toxicógeno",
    "toxicógenas": "toxicógeno",
    "toxicógenos": "toxicógeno",
    "toxicóloga": "toxicólogo",
    "toxicólogas": "toxicólogo",
    "toxicólogos": "toxicólogo",
    "toxicómana": "toxicómano",
    "toxicómanas": "toxicómano",
    "toxicómanos": "toxicómano",
    "toxinas": "toxina",
    "toxoides": "toxoide",
    "toxígena": "toxígeno",
    "toxígenas": "toxígeno",
    "toxígenos": "toxígeno",
    "toyotismos": "toyotismo",
    "tozas": "toza",
    "tozoladas": "tozolada",
    "tozolones": "tozolón",
    "tozuda": "tozudo",
    "tozudas": "tozudo",
    "tozudeces": "tozudez",
    "tozudos": "tozudo",
    "tozuelos": "tozuelo",
    "toábamos": "toar",
    "toáis": "toar",
    "toáramos": "toar",
    "toáremos": "toar",
    "toásemos": "toar",
    "toé": "toar",
    "toéis": "toar",
    "toñas": "toña",
    "toó": "toar",
    "traba": "trabar",
    "trababa": "trabar",
    "trababais": "trabar",
    "trababan": "trabar",
    "trababas": "trabar",
    "trabacuentas": "trabacuenta",
    "trabad": "trabar",
    "trabada": "trabar",
    "trabadas": "trabar",
    "trabaderos": "trabadero",
    "trabado": "trabar",
    "trabados": "trabar",
    "trabaduras": "trabadura",
    "trabaja": "trabajar",
    "trabajaba": "trabajar",
    "trabajabais": "trabajar",
    "trabajaban": "trabajar",
    "trabajabas": "trabajar",
    "trabajad": "trabajar",
    "trabajada": "trabajar",
    "trabajadas": "trabajar",
    "trabajado": "trabajar",
    "trabajadora": "trabajador",
    "trabajadoras": "trabajador",
    "trabajadores": "trabajador",
    "trabajados": "trabajar",
    "trabajamos": "trabajar",
    "trabajan": "trabajar",
    "trabajando": "trabajar",
    "trabajara": "trabajar",
    "trabajarais": "trabajar",
    "trabajaran": "trabajar",
    "trabajaras": "trabajar",
    "trabajare": "trabajar",
    "trabajareis": "trabajar",
    "trabajaremos": "trabajar",
    "trabajaren": "trabajar",
    "trabajares": "trabajar",
    "trabajaron": "trabajar",
    "trabajará": "trabajar",
    "trabajarán": "trabajar",
    "trabajarás": "trabajar",
    "trabajaré": "trabajar",
    "trabajaréis": "trabajar",
    "trabajaría": "trabajar",
    "trabajaríais": "trabajar",
    "trabajaríamos": "trabajar",
    "trabajarían": "trabajar",
    "trabajarías": "trabajar",
    "trabajas": "trabajar",
    "trabajase": "trabajar",
    "trabajaseis": "trabajar",
    "trabajasen": "trabajar",
    "trabajases": "trabajar",
    "trabajaste": "trabajar",
    "trabajasteis": "trabajar",
    "trabaje": "trabajar",
    "trabajemos": "trabajar",
    "trabajen": "trabajar",
    "trabajeras": "trabajera",
    "trabajes": "trabajar",
    "trabajo": "trabajar",
    "trabajos": "trabajo",
    "trabajosa": "trabajoso",
    "trabajosas": "trabajoso",
    "trabajosos": "trabajoso",
    "trabajábamos": "trabajar",
    "trabajáis": "trabajar",
    "trabajáramos": "trabajar",
    "trabajáremos": "trabajar",
    "trabajásemos": "trabajar",
    "trabajé": "trabajar",
    "trabajéis": "trabajar",
    "trabajó": "trabajar",
    "trabamientos": "trabamiento",
    "trabamos": "trabar",
    "traban": "trabar",
    "trabancas": "trabanca",
    "trabando": "trabar",
    "trabara": "trabar",
    "trabarais": "trabar",
    "trabaran": "trabar",
    "trabaras": "trabar",
    "trabare": "trabar",
    "trabareis": "trabar",
    "trabaremos": "trabar",
    "trabaren": "trabar",
    "trabares": "trabar",
    "trabaron": "trabar",
    "trabará": "trabar",
    "trabarán": "trabar",
    "trabarás": "trabar",
    "trabaré": "trabar",
    "trabaréis": "trabar",
    "trabaría": "trabar",
    "trabaríais": "trabar",
    "trabaríamos": "trabar",
    "trabarían": "trabar",
    "trabarías": "trabar",
    "trabas": "trabar",
    "trabase": "trabar",
    "trabaseis": "trabar",
    "trabasen": "trabar",
    "trabases": "trabar",
    "trabaste": "trabar",
    "trabasteis": "trabar",
    "trabazones": "trabazón",
    "trabe": "trabar",
    "trabeculaciones": "trabeculación",
    "trabemos": "trabar",
    "traben": "trabar",
    "trabes": "trabe",
    "trabillas": "trabilla",
    "trabo": "trabar",
    "trabones": "trabón",
    "trabuca": "trabucar",
    "trabucaba": "trabucar",
    "trabucabais": "trabucar",
    "trabucaban": "trabucar",
    "trabucabas": "trabucar",
    "trabucaciones": "trabucación",
    "trabucad": "trabucar",
    "trabucada": "trabucar",
    "trabucadas": "trabucar",
    "trabucado": "trabucar",
    "trabucadora": "trabucador",
    "trabucadoras": "trabucador",
    "trabucadores": "trabucador",
    "trabucados": "trabucar",
    "trabucaires": "trabucaire",
    "trabucamos": "trabucar",
    "trabucan": "trabucar",
    "trabucando": "trabucar",
    "trabucara": "trabucar",
    "trabucarais": "trabucar",
    "trabucaran": "trabucar",
    "trabucaras": "trabucar",
    "trabucare": "trabucar",
    "trabucareis": "trabucar",
    "trabucaremos": "trabucar",
    "trabucaren": "trabucar",
    "trabucares": "trabucar",
    "trabucaron": "trabucar",
    "trabucará": "trabucar",
    "trabucarán": "trabucar",
    "trabucarás": "trabucar",
    "trabucaré": "trabucar",
    "trabucaréis": "trabucar",
    "trabucaría": "trabucar",
    "trabucaríais": "trabucar",
    "trabucaríamos": "trabucar",
    "trabucarían": "trabucar",
    "trabucarías": "trabucar",
    "trabucas": "trabucar",
    "trabucase": "trabucar",
    "trabucaseis": "trabucar",
    "trabucasen": "trabucar",
    "trabucases": "trabucar",
    "trabucaste": "trabucar",
    "trabucasteis": "trabucar",
    "trabucazos": "trabucazo",
    "trabuco": "trabucar",
    "trabucos": "trabuco",
    "trabucábamos": "trabucar",
    "trabucáis": "trabucar",
    "trabucáramos": "trabucar",
    "trabucáremos": "trabucar",
    "trabucásemos": "trabucar",
    "trabucó": "trabucar",
    "trabuque": "trabucar",
    "trabuquemos": "trabucar",
    "trabuquen": "trabucar",
    "trabuques": "trabucar",
    "trabuquetes": "trabuquete",
    "trabuqué": "trabucar",
    "trabuquéis": "trabucar",
    "trabábamos": "trabar",
    "trabáis": "trabar",
    "trabáramos": "trabar",
    "trabáremos": "trabar",
    "trabásemos": "trabar",
    "trabé": "trabar",
    "trabéculas": "trabécula",
    "trabéis": "trabar",
    "trabó": "trabar",
    "tracamundanas": "tracamundana",
    "tracas": "traca",
    "tracciones": "tracción",
    "trace": "trazar",
    "traceadora": "traceador",
    "traceadoras": "traceador",
    "traceadores": "traceador",
    "traceados": "traceado",
    "tracemos": "trazar",
    "tracen": "trazar",
    "tracerías": "tracería",
    "traces": "trazar",
    "tracia": "tracio",
    "traciana": "traciano",
    "tracianas": "traciano",
    "tracianos": "traciano",
    "tracias": "tracio",
    "tracios": "tracio",
    "tracistas": "tracista",
    "tracomas": "tracoma",
    "tractiva": "tractivo",
    "tractivas": "tractivo",
    "tractivos": "tractivo",
    "tractocamiones": "tractocamión",
    "tractocarriles": "tractocarril",
    "tractora": "tractor",
    "tractoras": "tractor",
    "tractores": "tractor",
    "tractoristas": "tractorista",
    "tractos": "tracto",
    "tracé": "trazar",
    "tracéis": "trazar",
    "trademarks": "trademark",
    "tradicionales": "tradicional",
    "tradicionalismos": "tradicionalismo",
    "tradicionalistas": "tradicionalista",
    "tradiciones": "tradición",
    "traducciones": "traducción",
    "traducciónes": "traducción",
    "traduce": "traducir",
    "traducen": "traducir",
    "traduces": "traducir",
    "traducibilidades": "traducibilidad",
    "traducibles": "traducible",
    "traducid": "traducir",
    "traducida": "traducir",
    "traducidas": "traducir",
    "traducido": "traducir",
    "traducidos": "traducir",
    "traduciendo": "traducir",
    "traducimos": "traducir",
    "traduciremos": "traducir",
    "traducirá": "traducir",
    "traducirán": "traducir",
    "traducirás": "traducir",
    "traduciré": "traducir",
    "traduciréis": "traducir",
    "traduciría": "traducir",
    "traduciríais": "traducir",
    "traduciríamos": "traducir",
    "traducirían": "traducir",
    "traducirías": "traducir",
    "traductora": "traductor",
    "traductoras": "traductor",
    "traductores": "traductor",
    "traducía": "traducir",
    "traducíais": "traducir",
    "traducíamos": "traducir",
    "traducían": "traducir",
    "traducías": "traducir",
    "traducís": "traducir",
    "traduje": "traducir",
    "tradujera": "traducir",
    "tradujerais": "traducir",
    "tradujeran": "traducir",
    "tradujeras": "traducir",
    "tradujere": "traducir",
    "tradujereis": "traducir",
    "tradujeren": "traducir",
    "tradujeres": "traducir",
    "tradujeron": "traducir",
    "tradujese": "traducir",
    "tradujeseis": "traducir",
    "tradujesen": "traducir",
    "tradujeses": "traducir",
    "tradujimos": "traducir",
    "tradujiste": "traducir",
    "tradujisteis": "traducir",
    "tradujo": "traducir",
    "tradujéramos": "traducir",
    "tradujéremos": "traducir",
    "tradujésemos": "traducir",
    "traduzca": "traducir",
    "traduzcamos": "traducir",
    "traduzcan": "traducir",
    "traduzcas": "traducir",
    "traduzco": "traducir",
    "traduzcáis": "traducir",
    "trae": "traer",
    "traed": "traer",
    "traediza": "traedizo",
    "traedizas": "traedizo",
    "traedizos": "traedizo",
    "traedora": "traedor",
    "traedoras": "traedor",
    "traedores": "traedor",
    "traeduras": "traedura",
    "traemos": "traer",
    "traen": "traer",
    "traeremos": "traer",
    "traerá": "traer",
    "traerán": "traer",
    "traerás": "traer",
    "traeré": "traer",
    "traeréis": "traer",
    "traería": "traer",
    "traeríais": "traer",
    "traeríamos": "traer",
    "traerían": "traer",
    "traerías": "traer",
    "traes": "traer",
    "trafaga": "trafagar",
    "trafagaba": "trafagar",
    "trafagabais": "trafagar",
    "trafagaban": "trafagar",
    "trafagabas": "trafagar",
    "trafagad": "trafagar",
    "trafagada": "trafagar",
    "trafagadas": "trafagar",
    "trafagado": "trafagar",
    "trafagados": "trafagar",
    "trafagamos": "trafagar",
    "trafagan": "trafagar",
    "trafagando": "trafagar",
    "trafagara": "trafagar",
    "trafagarais": "trafagar",
    "trafagaran": "trafagar",
    "trafagaras": "trafagar",
    "trafagare": "trafagar",
    "trafagareis": "trafagar",
    "trafagaremos": "trafagar",
    "trafagaren": "trafagar",
    "trafagares": "trafagar",
    "trafagaron": "trafagar",
    "trafagará": "trafagar",
    "trafagarán": "trafagar",
    "trafagarás": "trafagar",
    "trafagaré": "trafagar",
    "trafagaréis": "trafagar",
    "trafagaría": "trafagar",
    "trafagaríais": "trafagar",
    "trafagaríamos": "trafagar",
    "trafagarían": "trafagar",
    "trafagarías": "trafagar",
    "trafagas": "trafagar",
    "trafagase": "trafagar",
    "trafagaseis": "trafagar",
    "trafagasen": "trafagar",
    "trafagases": "trafagar",
    "trafagaste": "trafagar",
    "trafagasteis": "trafagar",
    "trafago": "trafagar",
    "trafagosa": "trafagoso",
    "trafagosas": "trafagoso",
    "trafagosos": "trafagoso",
    "trafague": "trafagar",
    "trafaguemos": "trafagar",
    "trafaguen": "trafagar",
    "trafagues": "trafagar",
    "trafagué": "trafagar",
    "trafaguéis": "trafagar",
    "trafagábamos": "trafagar",
    "trafagáis": "trafagar",
    "trafagáramos": "trafagar",
    "trafagáremos": "trafagar",
    "trafagásemos": "trafagar",
    "trafagó": "trafagar",
    "trafalgares": "trafalgar",
    "trafalmejas": "trafalmeja",
    "trafica": "traficar",
    "traficaba": "traficar",
    "traficabais": "traficar",
    "traficaban": "traficar",
    "traficabas": "traficar",
    "traficad": "traficar",
    "traficada": "traficar",
    "traficadas": "traficar",
    "traficado": "traficar",
    "traficados": "traficar",
    "traficamos": "traficar",
    "trafican": "traficar",
    "traficando": "traficar",
    "traficantes": "traficante",
    "traficara": "traficar",
    "traficarais": "traficar",
    "traficaran": "traficar",
    "traficaras": "traficar",
    "traficare": "traficar",
    "traficareis": "traficar",
    "traficaremos": "traficar",
    "traficaren": "traficar",
    "traficares": "traficar",
    "traficaron": "traficar",
    "traficará": "traficar",
    "traficarán": "traficar",
    "traficarás": "traficar",
    "traficaré": "traficar",
    "traficaréis": "traficar",
    "traficaría": "traficar",
    "traficaríais": "traficar",
    "traficaríamos": "traficar",
    "traficarían": "traficar",
    "traficarías": "traficar",
    "traficas": "traficar",
    "traficase": "traficar",
    "traficaseis": "traficar",
    "traficasen": "traficar",
    "traficases": "traficar",
    "traficaste": "traficar",
    "traficasteis": "traficar",
    "trafico": "traficar",
    "traficábamos": "traficar",
    "traficáis": "traficar",
    "traficáramos": "traficar",
    "traficáremos": "traficar",
    "traficásemos": "traficar",
    "traficó": "traficar",
    "trafique": "traficar",
    "trafiquemos": "traficar",
    "trafiquen": "traficar",
    "trafiques": "traficar",
    "trafiqué": "traficar",
    "trafiquéis": "traficar",
    "traga": "tragar",
    "tragaba": "tragar",
    "tragabais": "tragar",
    "tragaban": "tragar",
    "tragabas": "tragar",
    "tragables": "tragable",
    "tragacantas": "tragacanta",
    "tragacantos": "tragacanto",
    "tragad": "tragar",
    "tragada": "tragar",
    "tragadas": "tragar",
    "tragaderos": "tragadero",
    "tragado": "tragar",
    "tragadora": "tragador",
    "tragadoras": "tragador",
    "tragadores": "tragador",
    "tragados": "tragar",
    "tragaluces": "tragaluz",
    "tragamos": "tragar",
    "tragan": "tragar",
    "tragando": "tragar",
    "tragantadas": "tragantada",
    "tragantes": "tragante",
    "tragantona": "tragantón",
    "tragantonas": "tragantón",
    "tragantones": "tragantón",
    "tragara": "tragar",
    "tragarais": "tragar",
    "tragaran": "tragar",
    "tragaras": "tragar",
    "tragare": "tragar",
    "tragareis": "tragar",
    "tragaremos": "tragar",
    "tragaren": "tragar",
    "tragares": "tragar",
    "tragaron": "tragar",
    "tragará": "tragar",
    "tragarán": "tragar",
    "tragarás": "tragar",
    "tragaré": "tragar",
    "tragaréis": "tragar",
    "tragaría": "tragar",
    "tragaríais": "tragar",
    "tragaríamos": "tragar",
    "tragarían": "tragar",
    "tragarías": "tragar",
    "tragas": "tragar",
    "tragase": "tragar",
    "tragaseis": "tragar",
    "tragasen": "tragar",
    "tragases": "tragar",
    "tragaste": "tragar",
    "tragasteis": "tragar",
    "tragavinos": "tragavino",
    "tragazones": "tragazón",
    "tragedias": "tragedia",
    "tragiana": "tragiano",
    "tragianas": "tragiano",
    "tragianos": "tragiano",
    "tragicomedias": "tragicomedia",
    "tragicómica": "tragicómico",
    "tragicómicas": "tragicómico",
    "tragicómicos": "tragicómico",
    "trago": "tragar",
    "tragona": "tragón",
    "tragonas": "tragón",
    "tragonea": "tragonear",
    "tragoneaba": "tragonear",
    "tragoneabais": "tragonear",
    "tragoneaban": "tragonear",
    "tragoneabas": "tragonear",
    "tragonead": "tragonear",
    "tragoneada": "tragonear",
    "tragoneadas": "tragonear",
    "tragoneado": "tragonear",
    "tragoneados": "tragonear",
    "tragoneamos": "tragonear",
    "tragonean": "tragonear",
    "tragoneando": "tragonear",
    "tragoneara": "tragonear",
    "tragonearais": "tragonear",
    "tragonearan": "tragonear",
    "tragonearas": "tragonear",
    "tragoneare": "tragonear",
    "tragoneareis": "tragonear",
    "tragonearemos": "tragonear",
    "tragonearen": "tragonear",
    "tragoneares": "tragonear",
    "tragonearon": "tragonear",
    "tragoneará": "tragonear",
    "tragonearán": "tragonear",
    "tragonearás": "tragonear",
    "tragonearé": "tragonear",
    "tragonearéis": "tragonear",
    "tragonearía": "tragonear",
    "tragonearíais": "tragonear",
    "tragonearíamos": "tragonear",
    "tragonearían": "tragonear",
    "tragonearías": "tragonear",
    "tragoneas": "tragonear",
    "tragonease": "tragonear",
    "tragoneaseis": "tragonear",
    "tragoneasen": "tragonear",
    "tragoneases": "tragonear",
    "tragoneaste": "tragonear",
    "tragoneasteis": "tragonear",
    "tragonee": "tragonear",
    "tragoneemos": "tragonear",
    "tragoneen": "tragonear",
    "tragonees": "tragonear",
    "tragoneo": "tragonear",
    "tragonerías": "tragonería",
    "tragones": "tragón",
    "tragoneábamos": "tragonear",
    "tragoneáis": "tragonear",
    "tragoneáramos": "tragonear",
    "tragoneáremos": "tragonear",
    "tragoneásemos": "tragonear",
    "tragoneé": "tragonear",
    "tragoneéis": "tragonear",
    "tragoneó": "tragonear",
    "tragonías": "tragonía",
    "tragopanes": "tragopán",
    "tragos": "trago",
    "trague": "tragar",
    "traguemos": "tragar",
    "traguen": "tragar",
    "tragues": "tragar",
    "tragué": "tragar",
    "traguéis": "tragar",
    "tragábamos": "tragar",
    "tragáis": "tragar",
    "tragáramos": "tragar",
    "tragáremos": "tragar",
    "tragásemos": "tragar",
    "tragó": "tragar",
    "traiciona": "traicionar",
    "traicionaba": "traicionar",
    "traicionabais": "traicionar",
    "traicionaban": "traicionar",
    "traicionabas": "traicionar",
    "traicionad": "traicionar",
    "traicionada": "traicionar",
    "traicionadas": "traicionar",
    "traicionado": "traicionar",
    "traicionados": "traicionar",
    "traicionamos": "traicionar",
    "traicionan": "traicionar",
    "traicionando": "traicionar",
    "traicionara": "traicionar",
    "traicionarais": "traicionar",
    "traicionaran": "traicionar",
    "traicionaras": "traicionar",
    "traicionare": "traicionar",
    "traicionareis": "traicionar",
    "traicionaremos": "traicionar",
    "traicionaren": "traicionar",
    "traicionares": "traicionar",
    "traicionaron": "traicionar",
    "traicionará": "traicionar",
    "traicionarán": "traicionar",
    "traicionarás": "traicionar",
    "traicionaré": "traicionar",
    "traicionaréis": "traicionar",
    "traicionaría": "traicionar",
    "traicionaríais": "traicionar",
    "traicionaríamos": "traicionar",
    "traicionarían": "traicionar",
    "traicionarías": "traicionar",
    "traicionas": "traicionar",
    "traicionase": "traicionar",
    "traicionaseis": "traicionar",
    "traicionasen": "traicionar",
    "traicionases": "traicionar",
    "traicionaste": "traicionar",
    "traicionasteis": "traicionar",
    "traicione": "traicionar",
    "traicionemos": "traicionar",
    "traicionen": "traicionar",
    "traicionera": "traicionero",
    "traicioneras": "traicionero",
    "traicioneros": "traicionero",
    "traiciones": "traicionar",
    "traiciono": "traicionar",
    "traicionábamos": "traicionar",
    "traicionáis": "traicionar",
    "traicionáramos": "traicionar",
    "traicionáremos": "traicionar",
    "traicionásemos": "traicionar",
    "traicioné": "traicionar",
    "traicionéis": "traicionar",
    "traicionó": "traicionar",
    "traidora": "traidor",
    "traidoras": "traidor",
    "traidores": "traidor",
    "traiga": "traer",
    "traigamos": "traer",
    "traigan": "traer",
    "traigas": "traer",
    "traigo": "traer",
    "traigáis": "traer",
    "trailers": "trailer",
    "traillaba": "traillar",
    "traillabais": "traillar",
    "traillaban": "traillar",
    "traillabas": "traillar",
    "traillad": "traillar",
    "traillada": "traillar",
    "trailladas": "traillar",
    "traillado": "traillar",
    "traillados": "traillar",
    "traillamos": "traillar",
    "traillando": "traillar",
    "traillara": "traillar",
    "traillarais": "traillar",
    "traillaran": "traillar",
    "traillaras": "traillar",
    "traillare": "traillar",
    "traillareis": "traillar",
    "traillaremos": "traillar",
    "traillaren": "traillar",
    "traillares": "traillar",
    "traillaron": "traillar",
    "traillará": "traillar",
    "traillarán": "traillar",
    "traillarás": "traillar",
    "traillaré": "traillar",
    "traillaréis": "traillar",
    "traillaría": "traillar",
    "traillaríais": "traillar",
    "traillaríamos": "traillar",
    "traillarían": "traillar",
    "traillarías": "traillar",
    "traillase": "traillar",
    "traillaseis": "traillar",
    "traillasen": "traillar",
    "traillases": "traillar",
    "traillaste": "traillar",
    "traillasteis": "traillar",
    "traillemos": "traillar",
    "traillábamos": "traillar",
    "trailláis": "traillar",
    "trailláramos": "traillar",
    "trailláremos": "traillar",
    "traillásemos": "traillar",
    "traillé": "traillar",
    "trailléis": "traillar",
    "trailló": "traillar",
    "traineras": "trainera",
    "trainerillas": "trainerilla",
    "traites": "traite",
    "trajana": "trajano",
    "trajanas": "trajano",
    "trajanos": "trajano",
    "traje": "traer",
    "trajea": "trajear",
    "trajeaba": "trajear",
    "trajeabais": "trajear",
    "trajeaban": "trajear",
    "trajeabas": "trajear",
    "trajead": "trajear",
    "trajeada": "trajear",
    "trajeadas": "trajear",
    "trajeado": "trajear",
    "trajeados": "trajear",
    "trajeamos": "trajear",
    "trajean": "trajear",
    "trajeando": "trajear",
    "trajeara": "trajear",
    "trajearais": "trajear",
    "trajearan": "trajear",
    "trajearas": "trajear",
    "trajeare": "trajear",
    "trajeareis": "trajear",
    "trajearemos": "trajear",
    "trajearen": "trajear",
    "trajeares": "trajear",
    "trajearon": "trajear",
    "trajeará": "trajear",
    "trajearán": "trajear",
    "trajearás": "trajear",
    "trajearé": "trajear",
    "trajearéis": "trajear",
    "trajearía": "trajear",
    "trajearíais": "trajear",
    "trajearíamos": "trajear",
    "trajearían": "trajear",
    "trajearías": "trajear",
    "trajeas": "trajear",
    "trajease": "trajear",
    "trajeaseis": "trajear",
    "trajeasen": "trajear",
    "trajeases": "trajear",
    "trajeaste": "trajear",
    "trajeasteis": "trajear",
    "trajecitos": "trajecito",
    "trajee": "trajear",
    "trajeemos": "trajear",
    "trajeen": "trajear",
    "trajees": "trajear",
    "trajeo": "trajear",
    "trajera": "traer",
    "trajerais": "traer",
    "trajeran": "traer",
    "trajeras": "traer",
    "trajere": "traer",
    "trajereis": "traer",
    "trajeren": "traer",
    "trajeres": "traer",
    "trajeron": "traer",
    "trajes": "traje",
    "trajese": "traer",
    "trajeseis": "traer",
    "trajesen": "traer",
    "trajeses": "traer",
    "trajeábamos": "trajear",
    "trajeáis": "trajear",
    "trajeáramos": "trajear",
    "trajeáremos": "trajear",
    "trajeásemos": "trajear",
    "trajeé": "trajear",
    "trajeéis": "trajear",
    "trajeó": "trajear",
    "trajimos": "traer",
    "trajina": "trajinar",
    "trajinaba": "trajinar",
    "trajinabais": "trajinar",
    "trajinaban": "trajinar",
    "trajinabas": "trajinar",
    "trajinad": "trajinar",
    "trajinada": "trajinar",
    "trajinadas": "trajinar",
    "trajinado": "trajinar",
    "trajinados": "trajinar",
    "trajinamos": "trajinar",
    "trajinan": "trajinar",
    "trajinando": "trajinar",
    "trajinantes": "trajinante",
    "trajinara": "trajinar",
    "trajinarais": "trajinar",
    "trajinaran": "trajinar",
    "trajinaras": "trajinar",
    "trajinare": "trajinar",
    "trajinareis": "trajinar",
    "trajinaremos": "trajinar",
    "trajinaren": "trajinar",
    "trajinares": "trajinar",
    "trajinaron": "trajinar",
    "trajinará": "trajinar",
    "trajinarán": "trajinar",
    "trajinarás": "trajinar",
    "trajinaré": "trajinar",
    "trajinaréis": "trajinar",
    "trajinaría": "trajinar",
    "trajinaríais": "trajinar",
    "trajinaríamos": "trajinar",
    "trajinarían": "trajinar",
    "trajinarías": "trajinar",
    "trajinas": "trajinar",
    "trajinase": "trajinar",
    "trajinaseis": "trajinar",
    "trajinasen": "trajinar",
    "trajinases": "trajinar",
    "trajinaste": "trajinar",
    "trajinasteis": "trajinar",
    "trajine": "trajinar",
    "trajinemos": "trajinar",
    "trajinen": "trajinar",
    "trajinera": "trajinero",
    "trajineras": "trajinero",
    "trajineros": "trajinero",
    "trajinerías": "trajinería",
    "trajines": "trajinar",
    "trajino": "trajinar",
    "trajinábamos": "trajinar",
    "trajináis": "trajinar",
    "trajináramos": "trajinar",
    "trajináremos": "trajinar",
    "trajinásemos": "trajinar",
    "trajiné": "trajinar",
    "trajinéis": "trajinar",
    "trajinó": "trajinar",
    "trajiste": "traer",
    "trajisteis": "traer",
    "trajo": "traer",
    "trajéramos": "traer",
    "trajéremos": "traer",
    "trajésemos": "traer",
    "trallas": "tralla",
    "trallazos": "trallazo",
    "trama": "tramar",
    "tramaba": "tramar",
    "tramabais": "tramar",
    "tramaban": "tramar",
    "tramabas": "tramar",
    "tramad": "tramar",
    "tramada": "tramar",
    "tramadas": "tramar",
    "tramado": "tramar",
    "tramadora": "tramador",
    "tramadoras": "tramador",
    "tramadores": "tramador",
    "tramados": "tramar",
    "tramamos": "tramar",
    "traman": "tramar",
    "tramando": "tramar",
    "tramara": "tramar",
    "tramarais": "tramar",
    "tramaran": "tramar",
    "tramaras": "tramar",
    "tramare": "tramar",
    "tramareis": "tramar",
    "tramaremos": "tramar",
    "tramaren": "tramar",
    "tramares": "tramar",
    "tramaron": "tramar",
    "tramará": "tramar",
    "tramarán": "tramar",
    "tramarás": "tramar",
    "tramaré": "tramar",
    "tramaréis": "tramar",
    "tramaría": "tramar",
    "tramaríais": "tramar",
    "tramaríamos": "tramar",
    "tramarían": "tramar",
    "tramarías": "tramar",
    "tramas": "tramar",
    "tramase": "tramar",
    "tramaseis": "tramar",
    "tramasen": "tramar",
    "tramases": "tramar",
    "tramaste": "tramar",
    "tramasteis": "tramar",
    "trame": "tramar",
    "tramemos": "tramar",
    "tramen": "tramar",
    "trames": "tramar",
    "tramificaciones": "tramificación",
    "tramillas": "tramilla",
    "tramita": "tramitar",
    "tramitaba": "tramitar",
    "tramitabais": "tramitar",
    "tramitaban": "tramitar",
    "tramitabas": "tramitar",
    "tramitaciones": "tramitación",
    "tramitad": "tramitar",
    "tramitada": "tramitar",
    "tramitadas": "tramitar",
    "tramitado": "tramitar",
    "tramitadora": "tramitador",
    "tramitadoras": "tramitador",
    "tramitadores": "tramitador",
    "tramitados": "tramitar",
    "tramitamos": "tramitar",
    "tramitan": "tramitar",
    "tramitando": "tramitar",
    "tramitara": "tramitar",
    "tramitarais": "tramitar",
    "tramitaran": "tramitar",
    "tramitaras": "tramitar",
    "tramitare": "tramitar",
    "tramitareis": "tramitar",
    "tramitaremos": "tramitar",
    "tramitaren": "tramitar",
    "tramitares": "tramitar",
    "tramitaron": "tramitar",
    "tramitará": "tramitar",
    "tramitarán": "tramitar",
    "tramitarás": "tramitar",
    "tramitaré": "tramitar",
    "tramitaréis": "tramitar",
    "tramitaría": "tramitar",
    "tramitaríais": "tramitar",
    "tramitaríamos": "tramitar",
    "tramitarían": "tramitar",
    "tramitarías": "tramitar",
    "tramitas": "tramitar",
    "tramitase": "tramitar",
    "tramitaseis": "tramitar",
    "tramitasen": "tramitar",
    "tramitases": "tramitar",
    "tramitaste": "tramitar",
    "tramitasteis": "tramitar",
    "tramite": "tramitar",
    "tramitemos": "tramitar",
    "tramiten": "tramitar",
    "tramites": "tramitar",
    "tramito": "tramitar",
    "tramitábamos": "tramitar",
    "tramitáis": "tramitar",
    "tramitáramos": "tramitar",
    "tramitáremos": "tramitar",
    "tramitásemos": "tramitar",
    "tramité": "tramitar",
    "tramitéis": "tramitar",
    "tramitó": "tramitar",
    "tramlots": "tramlot",
    "tramo": "tramar",
    "tramojos": "tramojo",
    "tramonta": "tramontar",
    "tramontaba": "tramontar",
    "tramontabais": "tramontar",
    "tramontaban": "tramontar",
    "tramontabas": "tramontar",
    "tramontad": "tramontar",
    "tramontada": "tramontar",
    "tramontadas": "tramontar",
    "tramontado": "tramontar",
    "tramontados": "tramontar",
    "tramontamos": "tramontar",
    "tramontan": "tramontar",
    "tramontana": "tramontano",
    "tramontanas": "tramontano",
    "tramontando": "tramontar",
    "tramontanos": "tramontano",
    "tramontara": "tramontar",
    "tramontarais": "tramontar",
    "tramontaran": "tramontar",
    "tramontaras": "tramontar",
    "tramontare": "tramontar",
    "tramontareis": "tramontar",
    "tramontaremos": "tramontar",
    "tramontaren": "tramontar",
    "tramontares": "tramontar",
    "tramontaron": "tramontar",
    "tramontará": "tramontar",
    "tramontarán": "tramontar",
    "tramontarás": "tramontar",
    "tramontaré": "tramontar",
    "tramontaréis": "tramontar",
    "tramontaría": "tramontar",
    "tramontaríais": "tramontar",
    "tramontaríamos": "tramontar",
    "tramontarían": "tramontar",
    "tramontarías": "tramontar",
    "tramontas": "tramontar",
    "tramontase": "tramontar",
    "tramontaseis": "tramontar",
    "tramontasen": "tramontar",
    "tramontases": "tramontar",
    "tramontaste": "tramontar",
    "tramontasteis": "tramontar",
    "tramonte": "tramontar",
    "tramontemos": "tramontar",
    "tramonten": "tramontar",
    "tramontes": "tramontar",
    "tramonto": "tramontar",
    "tramontábamos": "tramontar",
    "tramontáis": "tramontar",
    "tramontáramos": "tramontar",
    "tramontáremos": "tramontar",
    "tramontásemos": "tramontar",
    "tramonté": "tramontar",
    "tramontéis": "tramontar",
    "tramontó": "tramontar",
    "tramos": "tramo",
    "tramoyas": "tramoya",
    "tramoyistas": "tramoyista",
    "trampales": "trampal",
    "trampantojos": "trampantojo",
    "trampas": "trampa",
    "trampea": "trampear",
    "trampeaba": "trampear",
    "trampeabais": "trampear",
    "trampeaban": "trampear",
    "trampeabas": "trampear",
    "trampead": "trampear",
    "trampeada": "trampear",
    "trampeadas": "trampear",
    "trampeado": "trampear",
    "trampeadora": "trampeador",
    "trampeadoras": "trampeador",
    "trampeadores": "trampeador",
    "trampeados": "trampear",
    "trampeamos": "trampear",
    "trampean": "trampear",
    "trampeando": "trampear",
    "trampeara": "trampear",
    "trampearais": "trampear",
    "trampearan": "trampear",
    "trampearas": "trampear",
    "trampeare": "trampear",
    "trampeareis": "trampear",
    "trampearemos": "trampear",
    "trampearen": "trampear",
    "trampeares": "trampear",
    "trampearon": "trampear",
    "trampeará": "trampear",
    "trampearán": "trampear",
    "trampearás": "trampear",
    "trampearé": "trampear",
    "trampearéis": "trampear",
    "trampearía": "trampear",
    "trampearíais": "trampear",
    "trampearíamos": "trampear",
    "trampearían": "trampear",
    "trampearías": "trampear",
    "trampeas": "trampear",
    "trampease": "trampear",
    "trampeaseis": "trampear",
    "trampeasen": "trampear",
    "trampeases": "trampear",
    "trampeaste": "trampear",
    "trampeasteis": "trampear",
    "trampee": "trampear",
    "trampeemos": "trampear",
    "trampeen": "trampear",
    "trampees": "trampear",
    "trampeo": "trampear",
    "trampera": "trampero",
    "tramperas": "trampero",
    "tramperos": "trampero",
    "trampeábamos": "trampear",
    "trampeáis": "trampear",
    "trampeáramos": "trampear",
    "trampeáremos": "trampear",
    "trampeásemos": "trampear",
    "trampeé": "trampear",
    "trampeéis": "trampear",
    "trampeó": "trampear",
    "trampillas": "trampilla",
    "trampistas": "trampista",
    "trampolines": "trampolín",
    "tramposa": "tramposo",
    "tramposas": "tramposo",
    "tramposos": "tramposo",
    "tramuntanas": "tramuntana",
    "tramábamos": "tramar",
    "tramáis": "tramar",
    "tramáramos": "tramar",
    "tramáremos": "tramar",
    "tramásemos": "tramar",
    "tramé": "tramar",
    "traméis": "tramar",
    "tramó": "tramar",
    "tranca": "trancar",
    "trancaba": "trancar",
    "trancabais": "trancar",
    "trancaban": "trancar",
    "trancabas": "trancar",
    "trancad": "trancar",
    "trancada": "trancar",
    "trancadas": "trancar",
    "trancado": "trancar",
    "trancados": "trancar",
    "trancahílos": "trancahílo",
    "trancamos": "trancar",
    "trancan": "trancar",
    "trancando": "trancar",
    "trancaniles": "trancanil",
    "trancara": "trancar",
    "trancarais": "trancar",
    "trancaran": "trancar",
    "trancaras": "trancar",
    "trancare": "trancar",
    "trancareis": "trancar",
    "trancaremos": "trancar",
    "trancaren": "trancar",
    "trancares": "trancar",
    "trancaron": "trancar",
    "trancará": "trancar",
    "trancarán": "trancar",
    "trancarás": "trancar",
    "trancaré": "trancar",
    "trancaréis": "trancar",
    "trancaría": "trancar",
    "trancaríais": "trancar",
    "trancaríamos": "trancar",
    "trancarían": "trancar",
    "trancarías": "trancar",
    "trancas": "trancar",
    "trancase": "trancar",
    "trancaseis": "trancar",
    "trancasen": "trancar",
    "trancases": "trancar",
    "trancaste": "trancar",
    "trancasteis": "trancar",
    "trancazos": "trancazo",
    "trance": "tranzar",
    "trancemos": "tranzar",
    "trancen": "tranzar",
    "trances": "tranzar",
    "tranchetes": "tranchete",
    "tranco": "trancar",
    "trancos": "tranco",
    "trancábamos": "trancar",
    "trancáis": "trancar",
    "trancáramos": "trancar",
    "trancáremos": "trancar",
    "trancásemos": "trancar",
    "trancé": "tranzar",
    "trancéis": "tranzar",
    "trancó": "trancar",
    "trangallos": "trangallo",
    "tranque": "trancar",
    "tranquemos": "trancar",
    "tranquen": "trancar",
    "tranqueras": "tranquera",
    "tranqueros": "tranquero",
    "tranques": "trancar",
    "tranquila": "tranquilo",
    "tranquilas": "tranquilo",
    "tranquiles": "tranquil",
    "tranquilice": "tranquilizar",
    "tranquilicemos": "tranquilizar",
    "tranquilicen": "tranquilizar",
    "tranquilices": "tranquilizar",
    "tranquilicé": "tranquilizar",
    "tranquilicéis": "tranquilizar",
    "tranquilidades": "tranquilidad",
    "tranquiliza": "tranquilizar",
    "tranquilizaba": "tranquilizar",
    "tranquilizabais": "tranquilizar",
    "tranquilizaban": "tranquilizar",
    "tranquilizabas": "tranquilizar",
    "tranquilizad": "tranquilizar",
    "tranquilizada": "tranquilizar",
    "tranquilizadas": "tranquilizar",
    "tranquilizado": "tranquilizar",
    "tranquilizadora": "tranquilizador",
    "tranquilizadoras": "tranquilizador",
    "tranquilizadores": "tranquilizador",
    "tranquilizados": "tranquilizar",
    "tranquilizamos": "tranquilizar",
    "tranquilizan": "tranquilizar",
    "tranquilizando": "tranquilizar",
    "tranquilizantes": "tranquilizante",
    "tranquilizara": "tranquilizar",
    "tranquilizarais": "tranquilizar",
    "tranquilizaran": "tranquilizar",
    "tranquilizaras": "tranquilizar",
    "tranquilizare": "tranquilizar",
    "tranquilizareis": "tranquilizar",
    "tranquilizaremos": "tranquilizar",
    "tranquilizaren": "tranquilizar",
    "tranquilizares": "tranquilizar",
    "tranquilizaron": "tranquilizar",
    "tranquilizará": "tranquilizar",
    "tranquilizarán": "tranquilizar",
    "tranquilizarás": "tranquilizar",
    "tranquilizaré": "tranquilizar",
    "tranquilizaréis": "tranquilizar",
    "tranquilizaría": "tranquilizar",
    "tranquilizaríais": "tranquilizar",
    "tranquilizaríamos": "tranquilizar",
    "tranquilizarían": "tranquilizar",
    "tranquilizarías": "tranquilizar",
    "tranquilizas": "tranquilizar",
    "tranquilizase": "tranquilizar",
    "tranquilizaseis": "tranquilizar",
    "tranquilizasen": "tranquilizar",
    "tranquilizases": "tranquilizar",
    "tranquilizaste": "tranquilizar",
    "tranquilizasteis": "tranquilizar",
    "tranquilizo": "tranquilizar",
    "tranquilizábamos": "tranquilizar",
    "tranquilizáis": "tranquilizar",
    "tranquilizáramos": "tranquilizar",
    "tranquilizáremos": "tranquilizar",
    "tranquilizásemos": "tranquilizar",
    "tranquilizó": "tranquilizar",
    "tranquillas": "tranquilla",
    "tranquillones": "tranquillón",
    "tranquillos": "tranquillo",
    "tranquilos": "tranquilo",
    "tranqué": "trancar",
    "tranquéis": "trancar",
    "transacciona": "transaccionar",
    "transaccionaba": "transaccionar",
    "transaccionabais": "transaccionar",
    "transaccionaban": "transaccionar",
    "transaccionabas": "transaccionar",
    "transaccionad": "transaccionar",
    "transaccionada": "transaccionar",
    "transaccionadas": "transaccionar",
    "transaccionado": "transaccionar",
    "transaccionados": "transaccionar",
    "transaccionales": "transaccional",
    "transaccionamos": "transaccionar",
    "transaccionan": "transaccionar",
    "transaccionando": "transaccionar",
    "transaccionara": "transaccionar",
    "transaccionarais": "transaccionar",
    "transaccionaran": "transaccionar",
    "transaccionaras": "transaccionar",
    "transaccionare": "transaccionar",
    "transaccionareis": "transaccionar",
    "transaccionaremos": "transaccionar",
    "transaccionaren": "transaccionar",
    "transaccionares": "transaccionar",
    "transaccionaron": "transaccionar",
    "transaccionará": "transaccionar",
    "transaccionarán": "transaccionar",
    "transaccionarás": "transaccionar",
    "transaccionaré": "transaccionar",
    "transaccionaréis": "transaccionar",
    "transaccionaría": "transaccionar",
    "transaccionaríais": "transaccionar",
    "transaccionaríamos": "transaccionar",
    "transaccionarían": "transaccionar",
    "transaccionarías": "transaccionar",
    "transaccionas": "transaccionar",
    "transaccionase": "transaccionar",
    "transaccionaseis": "transaccionar",
    "transaccionasen": "transaccionar",
    "transaccionases": "transaccionar",
    "transaccionaste": "transaccionar",
    "transaccionasteis": "transaccionar",
    "transaccione": "transaccionar",
    "transaccionemos": "transaccionar",
    "transaccionen": "transaccionar",
    "transacciones": "transaccionar",
    "transacciono": "transaccionar",
    "transaccionábamos": "transaccionar",
    "transaccionáis": "transaccionar",
    "transaccionáramos": "transaccionar",
    "transaccionáremos": "transaccionar",
    "transaccionásemos": "transaccionar",
    "transaccioné": "transaccionar",
    "transaccionéis": "transaccionar",
    "transaccionó": "transaccionar",
    "transadmitancias": "transadmitancia",
    "transalpina": "transalpino",
    "transalpinas": "transalpino",
    "transalpinos": "transalpino",
    "transalquilaciones": "transalquilación",
    "transamazónica": "transamazónico",
    "transamazónicas": "transamazónico",
    "transamazónicos": "transamazónico",
    "transamina": "transaminar",
    "transaminaba": "transaminar",
    "transaminabais": "transaminar",
    "transaminaban": "transaminar",
    "transaminabas": "transaminar",
    "transaminaciónes": "transaminación",
    "transaminad": "transaminar",
    "transaminada": "transaminar",
    "transaminadas": "transaminar",
    "transaminado": "transaminar",
    "transaminados": "transaminar",
    "transaminamos": "transaminar",
    "transaminan": "transaminar",
    "transaminando": "transaminar",
    "transaminara": "transaminar",
    "transaminarais": "transaminar",
    "transaminaran": "transaminar",
    "transaminaras": "transaminar",
    "transaminare": "transaminar",
    "transaminareis": "transaminar",
    "transaminaremos": "transaminar",
    "transaminaren": "transaminar",
    "transaminares": "transaminar",
    "transaminaron": "transaminar",
    "transaminará": "transaminar",
    "transaminarán": "transaminar",
    "transaminarás": "transaminar",
    "transaminaré": "transaminar",
    "transaminaréis": "transaminar",
    "transaminaría": "transaminar",
    "transaminaríais": "transaminar",
    "transaminaríamos": "transaminar",
    "transaminarían": "transaminar",
    "transaminarías": "transaminar",
    "transaminas": "transaminar",
    "transaminasas": "transaminasa",
    "transaminase": "transaminar",
    "transaminaseis": "transaminar",
    "transaminasen": "transaminar",
    "transaminases": "transaminar",
    "transaminaste": "transaminar",
    "transaminasteis": "transaminar",
    "transamine": "transaminar",
    "transaminemos": "transaminar",
    "transaminen": "transaminar",
    "transamines": "transaminar",
    "transamino": "transaminar",
    "transaminábamos": "transaminar",
    "transamináis": "transaminar",
    "transamináramos": "transaminar",
    "transamináremos": "transaminar",
    "transaminásemos": "transaminar",
    "transaminé": "transaminar",
    "transaminéis": "transaminar",
    "transaminó": "transaminar",
    "transandina": "transandino",
    "transandinas": "transandino",
    "transandinos": "transandino",
    "transatlántica": "transatlántico",
    "transatlánticas": "transatlántico",
    "transatlánticos": "transatlántico",
    "transborda": "transbordar",
    "transbordaba": "transbordar",
    "transbordabais": "transbordar",
    "transbordaban": "transbordar",
    "transbordabas": "transbordar",
    "transbordad": "transbordar",
    "transbordada": "transbordar",
    "transbordadas": "transbordar",
    "transbordado": "transbordar",
    "transbordadora": "transbordador",
    "transbordadoras": "transbordador",
    "transbordadores": "transbordador",
    "transbordados": "transbordar",
    "transbordamos": "transbordar",
    "transbordan": "transbordar",
    "transbordando": "transbordar",
    "transbordara": "transbordar",
    "transbordarais": "transbordar",
    "transbordaran": "transbordar",
    "transbordaras": "transbordar",
    "transbordare": "transbordar",
    "transbordareis": "transbordar",
    "transbordaremos": "transbordar",
    "transbordaren": "transbordar",
    "transbordares": "transbordar",
    "transbordaron": "transbordar",
    "transbordará": "transbordar",
    "transbordarán": "transbordar",
    "transbordarás": "transbordar",
    "transbordaré": "transbordar",
    "transbordaréis": "transbordar",
    "transbordaría": "transbordar",
    "transbordaríais": "transbordar",
    "transbordaríamos": "transbordar",
    "transbordarían": "transbordar",
    "transbordarías": "transbordar",
    "transbordas": "transbordar",
    "transbordase": "transbordar",
    "transbordaseis": "transbordar",
    "transbordasen": "transbordar",
    "transbordases": "transbordar",
    "transbordaste": "transbordar",
    "transbordasteis": "transbordar",
    "transborde": "transbordar",
    "transbordemos": "transbordar",
    "transborden": "transbordar",
    "transbordes": "transbordar",
    "transbordo": "transbordar",
    "transbordos": "transbordo",
    "transbordábamos": "transbordar",
    "transbordáis": "transbordar",
    "transbordáramos": "transbordar",
    "transbordáremos": "transbordar",
    "transbordásemos": "transbordar",
    "transbordé": "transbordar",
    "transbordéis": "transbordar",
    "transbordó": "transbordar",
    "transcategorizaciones": "transcategorización",
    "transcaucasiana": "transcaucasiano",
    "transcaucasianas": "transcaucasiano",
    "transcaucasianos": "transcaucasiano",
    "transcelulares": "transcelular",
    "transcendamos": "transcender",
    "transcended": "transcender",
    "transcendemos": "transcender",
    "transcendencias": "transcendencia",
    "transcendentales": "transcendental",
    "transcendentalismos": "transcendentalismo",
    "transcendentes": "transcendente",
    "transcenderemos": "transcender",
    "transcenderá": "transcender",
    "transcenderán": "transcender",
    "transcenderás": "transcender",
    "transcenderé": "transcender",
    "transcenderéis": "transcender",
    "transcendería": "transcender",
    "transcenderíais": "transcender",
    "transcenderíamos": "transcender",
    "transcenderían": "transcender",
    "transcenderías": "transcender",
    "transcendida": "transcender",
    "transcendidas": "transcender",
    "transcendido": "transcender",
    "transcendidos": "transcender",
    "transcendiendo": "transcender",
    "transcendiera": "transcender",
    "transcendierais": "transcender",
    "transcendieran": "transcender",
    "transcendieras": "transcender",
    "transcendiere": "transcender",
    "transcendiereis": "transcender",
    "transcendieren": "transcender",
    "transcendieres": "transcender",
    "transcendieron": "transcender",
    "transcendiese": "transcender",
    "transcendieseis": "transcender",
    "transcendiesen": "transcender",
    "transcendieses": "transcender",
    "transcendimos": "transcender",
    "transcendiste": "transcender",
    "transcendisteis": "transcender",
    "transcendiéramos": "transcender",
    "transcendiéremos": "transcender",
    "transcendiésemos": "transcender",
    "transcendió": "transcender",
    "transcendáis": "transcender",
    "transcendéis": "transcender",
    "transcendí": "transcender",
    "transcendía": "transcender",
    "transcendíais": "transcender",
    "transcendíamos": "transcender",
    "transcendían": "transcender",
    "transcendías": "transcender",
    "transceptores": "transceptor",
    "transcienda": "transcender",
    "transciendan": "transcender",
    "transciendas": "transcender",
    "transciende": "transcender",
    "transcienden": "transcender",
    "transciendes": "transcender",
    "transciendo": "transcender",
    "transcodificaciones": "transcodificación",
    "transcodificadores": "transcodificador",
    "transconductancias": "transconductancia",
    "transconectores": "transconector",
    "transconexiones": "transconexión",
    "transcontinentales": "transcontinental",
    "transcriba": "transcribir",
    "transcribamos": "transcribir",
    "transcriban": "transcribir",
    "transcribas": "transcribir",
    "transcribe": "transcribir",
    "transcriben": "transcribir",
    "transcribes": "transcribir",
    "transcribid": "transcribir",
    "transcribiendo": "transcribir",
    "transcribiera": "transcribir",
    "transcribierais": "transcribir",
    "transcribieran": "transcribir",
    "transcribieras": "transcribir",
    "transcribiere": "transcribir",
    "transcribiereis": "transcribir",
    "transcribieren": "transcribir",
    "transcribieres": "transcribir",
    "transcribieron": "transcribir",
    "transcribiese": "transcribir",
    "transcribieseis": "transcribir",
    "transcribiesen": "transcribir",
    "transcribieses": "transcribir",
    "transcribimos": "transcribir",
    "transcribiremos": "transcribir",
    "transcribirá": "transcribir",
    "transcribirán": "transcribir",
    "transcribirás": "transcribir",
    "transcribiré": "transcribir",
    "transcribiréis": "transcribir",
    "transcribiría": "transcribir",
    "transcribiríais": "transcribir",
    "transcribiríamos": "transcribir",
    "transcribirían": "transcribir",
    "transcribirías": "transcribir",
    "transcribiste": "transcribir",
    "transcribisteis": "transcribir",
    "transcribiéramos": "transcribir",
    "transcribiéremos": "transcribir",
    "transcribiésemos": "transcribir",
    "transcribió": "transcribir",
    "transcribo": "transcribir",
    "transcribáis": "transcribir",
    "transcribí": "transcribir",
    "transcribía": "transcribir",
    "transcribíais": "transcribir",
    "transcribíamos": "transcribir",
    "transcribían": "transcribir",
    "transcribías": "transcribir",
    "transcribís": "transcribir",
    "transcripciones": "transcripción",
    "transcriptora": "transcriptor",
    "transcriptoras": "transcriptor",
    "transcriptores": "transcriptor",
    "transcrita": "transcribir",
    "transcritas": "transcribir",
    "transcrito": "transcribir",
    "transcritos": "transcribir",
    "transculturales": "transcultural",
    "transcurra": "transcurrir",
    "transcurramos": "transcurrir",
    "transcurran": "transcurrir",
    "transcurras": "transcurrir",
    "transcurre": "transcurrir",
    "transcurren": "transcurrir",
    "transcurrentes": "transcurrente",
    "transcurres": "transcurrir",
    "transcurrid": "transcurrir",
    "transcurrida": "transcurrir",
    "transcurridas": "transcurrir",
    "transcurrido": "transcurrir",
    "transcurridos": "transcurrir",
    "transcurriendo": "transcurrir",
    "transcurriera": "transcurrir",
    "transcurrierais": "transcurrir",
    "transcurrieran": "transcurrir",
    "transcurrieras": "transcurrir",
    "transcurriere": "transcurrir",
    "transcurriereis": "transcurrir",
    "transcurrieren": "transcurrir",
    "transcurrieres": "transcurrir",
    "transcurrieron": "transcurrir",
    "transcurriese": "transcurrir",
    "transcurrieseis": "transcurrir",
    "transcurriesen": "transcurrir",
    "transcurrieses": "transcurrir",
    "transcurrimos": "transcurrir",
    "transcurriremos": "transcurrir",
    "transcurrirá": "transcurrir",
    "transcurrirán": "transcurrir",
    "transcurrirás": "transcurrir",
    "transcurriré": "transcurrir",
    "transcurriréis": "transcurrir",
    "transcurriría": "transcurrir",
    "transcurriríais": "transcurrir",
    "transcurriríamos": "transcurrir",
    "transcurrirían": "transcurrir",
    "transcurrirías": "transcurrir",
    "transcurriste": "transcurrir",
    "transcurristeis": "transcurrir",
    "transcurriéramos": "transcurrir",
    "transcurriéremos": "transcurrir",
    "transcurriésemos": "transcurrir",
    "transcurrió": "transcurrir",
    "transcurro": "transcurrir",
    "transcurráis": "transcurrir",
    "transcurrí": "transcurrir",
    "transcurría": "transcurrir",
    "transcurríais": "transcurrir",
    "transcurríamos": "transcurrir",
    "transcurrían": "transcurrir",
    "transcurrías": "transcurrir",
    "transcurrís": "transcurrir",
    "transcursos": "transcurso",
    "transdeterminaciones": "transdeterminación",
    "transdiferenciaciones": "transdiferenciación",
    "transdiodos": "transdiodo",
    "transducciones": "transducción",
    "transductores": "transductor",
    "transectos": "transecto",
    "transelevadores": "transelevador",
    "transepiteliales": "transepitelial",
    "transeptales": "transeptal",
    "transeptos": "transepto",
    "transespecífica": "transespecífico",
    "transespecíficas": "transespecífico",
    "transespecíficos": "transespecífico",
    "transeuropea": "transeuropeo",
    "transeuropeas": "transeuropeo",
    "transeuropeos": "transeuropeo",
    "transeúntes": "transeúnte",
    "transfecciones": "transfección",
    "transfectomas": "transfectoma",
    "transferasas": "transferasa",
    "transferencias": "transferencia",
    "transferentes": "transferente",
    "transferibilidades": "transferibilidad",
    "transferibles": "transferible",
    "transferid": "transferir",
    "transferida": "transferir",
    "transferidas": "transferir",
    "transferido": "transferir",
    "transferidora": "transferidor",
    "transferidoras": "transferidor",
    "transferidores": "transferidor",
    "transferidos": "transferir",
    "transferimos": "transferir",
    "transferiremos": "transferir",
    "transferirá": "transferir",
    "transferirán": "transferir",
    "transferirás": "transferir",
    "transferiré": "transferir",
    "transferiréis": "transferir",
    "transferiría": "transferir",
    "transferiríais": "transferir",
    "transferiríamos": "transferir",
    "transferirían": "transferir",
    "transferirías": "transferir",
    "transferiste": "transferir",
    "transferisteis": "transferir",
    "transferrinas": "transferrina",
    "transfers": "transfer",
    "transferí": "transferir",
    "transfería": "transferir",
    "transferíais": "transferir",
    "transferíamos": "transferir",
    "transferían": "transferir",
    "transferías": "transferir",
    "transferís": "transferir",
    "transfiera": "transferir",
    "transfieran": "transferir",
    "transfieras": "transferir",
    "transfiere": "transferir",
    "transfieren": "transferir",
    "transfieres": "transferir",
    "transfiero": "transferir",
    "transfigura": "transfigurar",
    "transfiguraba": "transfigurar",
    "transfigurabais": "transfigurar",
    "transfiguraban": "transfigurar",
    "transfigurabas": "transfigurar",
    "transfigurables": "transfigurable",
    "transfiguraciones": "transfiguración",
    "transfigurad": "transfigurar",
    "transfigurada": "transfigurar",
    "transfiguradas": "transfigurar",
    "transfigurado": "transfigurar",
    "transfigurados": "transfigurar",
    "transfiguramos": "transfigurar",
    "transfiguran": "transfigurar",
    "transfigurando": "transfigurar",
    "transfigurara": "transfigurar",
    "transfigurarais": "transfigurar",
    "transfiguraran": "transfigurar",
    "transfiguraras": "transfigurar",
    "transfigurare": "transfigurar",
    "transfigurareis": "transfigurar",
    "transfiguraremos": "transfigurar",
    "transfiguraren": "transfigurar",
    "transfigurares": "transfigurar",
    "transfiguraron": "transfigurar",
    "transfigurará": "transfigurar",
    "transfigurarán": "transfigurar",
    "transfigurarás": "transfigurar",
    "transfiguraré": "transfigurar",
    "transfiguraréis": "transfigurar",
    "transfiguraría": "transfigurar",
    "transfiguraríais": "transfigurar",
    "transfiguraríamos": "transfigurar",
    "transfigurarían": "transfigurar",
    "transfigurarías": "transfigurar",
    "transfiguras": "transfigurar",
    "transfigurase": "transfigurar",
    "transfiguraseis": "transfigurar",
    "transfigurasen": "transfigurar",
    "transfigurases": "transfigurar",
    "transfiguraste": "transfigurar",
    "transfigurasteis": "transfigurar",
    "transfigure": "transfigurar",
    "transfiguremos": "transfigurar",
    "transfiguren": "transfigurar",
    "transfigures": "transfigurar",
    "transfiguro": "transfigurar",
    "transfigurábamos": "transfigurar",
    "transfiguráis": "transfigurar",
    "transfiguráramos": "transfigurar",
    "transfiguráremos": "transfigurar",
    "transfigurásemos": "transfigurar",
    "transfiguré": "transfigurar",
    "transfiguréis": "transfigurar",
    "transfiguró": "transfigurar",
    "transfija": "transfijo",
    "transfijas": "transfijo",
    "transfijos": "transfijo",
    "transfinita": "transfinito",
    "transfinitas": "transfinito",
    "transfinitos": "transfinito",
    "transfiramos": "transferir",
    "transfiriendo": "transferir",
    "transfiriera": "transferir",
    "transfirierais": "transferir",
    "transfirieran": "transferir",
    "transfirieras": "transferir",
    "transfiriere": "transferir",
    "transfiriereis": "transferir",
    "transfirieren": "transferir",
    "transfirieres": "transferir",
    "transfirieron": "transferir",
    "transfiriese": "transferir",
    "transfirieseis": "transferir",
    "transfiriesen": "transferir",
    "transfirieses": "transferir",
    "transfiriéramos": "transferir",
    "transfiriéremos": "transferir",
    "transfiriésemos": "transferir",
    "transfirió": "transferir",
    "transfiráis": "transferir",
    "transfixiones": "transfixión",
    "transflores": "transflor",
    "transfluencias": "transfluencia",
    "transfluxora": "transfluxor",
    "transfluxoras": "transfluxor",
    "transfluxores": "transfluxor",
    "transfonologizaciones": "transfonologización",
    "transforma": "transformar",
    "transformaba": "transformar",
    "transformabais": "transformar",
    "transformaban": "transformar",
    "transformabas": "transformar",
    "transformables": "transformable",
    "transformacionales": "transformacional",
    "transformacionalismos": "transformacionalismo",
    "transformaciones": "transformación",
    "transformad": "transformar",
    "transformada": "transformar",
    "transformadas": "transformar",
    "transformado": "transformar",
    "transformadora": "transformador",
    "transformadoras": "transformador",
    "transformadores": "transformador",
    "transformados": "transformar",
    "transformamientos": "transformamiento",
    "transformamos": "transformar",
    "transforman": "transformar",
    "transformando": "transformar",
    "transformara": "transformar",
    "transformarais": "transformar",
    "transformaran": "transformar",
    "transformaras": "transformar",
    "transformare": "transformar",
    "transformareis": "transformar",
    "transformaremos": "transformar",
    "transformaren": "transformar",
    "transformares": "transformar",
    "transformaron": "transformar",
    "transformará": "transformar",
    "transformarán": "transformar",
    "transformarás": "transformar",
    "transformaré": "transformar",
    "transformaréis": "transformar",
    "transformaría": "transformar",
    "transformaríais": "transformar",
    "transformaríamos": "transformar",
    "transformarían": "transformar",
    "transformarías": "transformar",
    "transformas": "transformar",
    "transformase": "transformar",
    "transformaseis": "transformar",
    "transformasen": "transformar",
    "transformases": "transformar",
    "transformaste": "transformar",
    "transformasteis": "transformar",
    "transformativa": "transformativo",
    "transformativas": "transformativo",
    "transformativos": "transformativo",
    "transforme": "transformar",
    "transformemos": "transformar",
    "transformen": "transformar",
    "transformes": "transformar",
    "transformismos": "transformismo",
    "transformistas": "transformista",
    "transformo": "transformar",
    "transformábamos": "transformar",
    "transformáis": "transformar",
    "transformáramos": "transformar",
    "transformáremos": "transformar",
    "transformásemos": "transformar",
    "transformé": "transformar",
    "transforméis": "transformar",
    "transformó": "transformar",
    "transfretana": "transfretano",
    "transfretanas": "transfretano",
    "transfretanos": "transfretano",
    "transfronteriza": "transfronterizo",
    "transfronterizas": "transfronterizo",
    "transfronterizos": "transfronterizo",
    "transfunda": "transfundir",
    "transfundamos": "transfundir",
    "transfundan": "transfundir",
    "transfundas": "transfundir",
    "transfunde": "transfundir",
    "transfunden": "transfundir",
    "transfundes": "transfundir",
    "transfundiciones": "transfundición",
    "transfundid": "transfundir",
    "transfundida": "transfundir",
    "transfundidas": "transfundir",
    "transfundido": "transfundir",
    "transfundidos": "transfundir",
    "transfundiendo": "transfundir",
    "transfundiera": "transfundir",
    "transfundierais": "transfundir",
    "transfundieran": "transfundir",
    "transfundieras": "transfundir",
    "transfundiere": "transfundir",
    "transfundiereis": "transfundir",
    "transfundieren": "transfundir",
    "transfundieres": "transfundir",
    "transfundieron": "transfundir",
    "transfundiese": "transfundir",
    "transfundieseis": "transfundir",
    "transfundiesen": "transfundir",
    "transfundieses": "transfundir",
    "transfundimos": "transfundir",
    "transfundiremos": "transfundir",
    "transfundirá": "transfundir",
    "transfundirán": "transfundir",
    "transfundirás": "transfundir",
    "transfundiré": "transfundir",
    "transfundiréis": "transfundir",
    "transfundiría": "transfundir",
    "transfundiríais": "transfundir",
    "transfundiríamos": "transfundir",
    "transfundirían": "transfundir",
    "transfundirías": "transfundir",
    "transfundiste": "transfundir",
    "transfundisteis": "transfundir",
    "transfundiéramos": "transfundir",
    "transfundiéremos": "transfundir",
    "transfundiésemos": "transfundir",
    "transfundió": "transfundir",
    "transfundo": "transfundir",
    "transfundáis": "transfundir",
    "transfundí": "transfundir",
    "transfundía": "transfundir",
    "transfundíais": "transfundir",
    "transfundíamos": "transfundir",
    "transfundían": "transfundir",
    "transfundías": "transfundir",
    "transfundís": "transfundir",
    "transfusibles": "transfusible",
    "transfusionales": "transfusional",
    "transfusiones": "transfusión",
    "transfusora": "transfusor",
    "transfusoras": "transfusor",
    "transfusores": "transfusor",
    "transgangética": "transgangético",
    "transgangéticas": "transgangético",
    "transgangéticos": "transgangético",
    "transgreda": "transgredir",
    "transgredamos": "transgredir",
    "transgredan": "transgredir",
    "transgredas": "transgredir",
    "transgrede": "transgredir",
    "transgreden": "transgredir",
    "transgredes": "transgredir",
    "transgredid": "transgredir",
    "transgredida": "transgredir",
    "transgredidas": "transgredir",
    "transgredido": "transgredir",
    "transgredidos": "transgredir",
    "transgrediendo": "transgredir",
    "transgrediera": "transgredir",
    "transgredierais": "transgredir",
    "transgredieran": "transgredir",
    "transgredieras": "transgredir",
    "transgrediere": "transgredir",
    "transgrediereis": "transgredir",
    "transgredieren": "transgredir",
    "transgredieres": "transgredir",
    "transgredieron": "transgredir",
    "transgrediese": "transgredir",
    "transgredieseis": "transgredir",
    "transgrediesen": "transgredir",
    "transgredieses": "transgredir",
    "transgredimos": "transgredir",
    "transgrediremos": "transgredir",
    "transgredirá": "transgredir",
    "transgredirán": "transgredir",
    "transgredirás": "transgredir",
    "transgrediré": "transgredir",
    "transgrediréis": "transgredir",
    "transgrediría": "transgredir",
    "transgrediríais": "transgredir",
    "transgrediríamos": "transgredir",
    "transgredirían": "transgredir",
    "transgredirías": "transgredir",
    "transgrediste": "transgredir",
    "transgredisteis": "transgredir",
    "transgrediéramos": "transgredir",
    "transgrediéremos": "transgredir",
    "transgrediésemos": "transgredir",
    "transgredió": "transgredir",
    "transgredo": "transgredir",
    "transgredáis": "transgredir",
    "transgredí": "transgredir",
    "transgredía": "transgredir",
    "transgredíais": "transgredir",
    "transgredíamos": "transgredir",
    "transgredían": "transgredir",
    "transgredías": "transgredir",
    "transgredís": "transgredir",
    "transgresiones": "transgresión",
    "transgresiva": "transgresivo",
    "transgresivas": "transgresivo",
    "transgresivos": "transgresivo",
    "transgresora": "transgresor",
    "transgresoras": "transgresor",
    "transgresores": "transgresor",
    "transgénica": "transgénico",
    "transgénicas": "transgénico",
    "transgénicos": "transgénico",
    "transiberiana": "transiberiano",
    "transiberianas": "transiberiano",
    "transiberianos": "transiberiano",
    "transicionales": "transicional",
    "transiciones": "transición",
    "transiciónes": "transición",
    "transida": "transido",
    "transidas": "transido",
    "transidos": "transido",
    "transige": "transigir",
    "transigen": "transigir",
    "transigencias": "transigencia",
    "transigentes": "transigente",
    "transiges": "transigir",
    "transigid": "transigir",
    "transigida": "transigir",
    "transigidas": "transigir",
    "transigido": "transigir",
    "transigidos": "transigir",
    "transigiendo": "transigir",
    "transigiera": "transigir",
    "transigierais": "transigir",
    "transigieran": "transigir",
    "transigieras": "transigir",
    "transigiere": "transigir",
    "transigiereis": "transigir",
    "transigieren": "transigir",
    "transigieres": "transigir",
    "transigieron": "transigir",
    "transigiese": "transigir",
    "transigieseis": "transigir",
    "transigiesen": "transigir",
    "transigieses": "transigir",
    "transigimos": "transigir",
    "transigiremos": "transigir",
    "transigirá": "transigir",
    "transigirán": "transigir",
    "transigirás": "transigir",
    "transigiré": "transigir",
    "transigiréis": "transigir",
    "transigiría": "transigir",
    "transigiríais": "transigir",
    "transigiríamos": "transigir",
    "transigirían": "transigir",
    "transigirías": "transigir",
    "transigiste": "transigir",
    "transigisteis": "transigir",
    "transigiéramos": "transigir",
    "transigiéremos": "transigir",
    "transigiésemos": "transigir",
    "transigió": "transigir",
    "transigí": "transigir",
    "transigía": "transigir",
    "transigíais": "transigir",
    "transigíamos": "transigir",
    "transigían": "transigir",
    "transigías": "transigir",
    "transigís": "transigir",
    "transija": "transigir",
    "transijamos": "transigir",
    "transijan": "transigir",
    "transijas": "transigir",
    "transijo": "transigir",
    "transijáis": "transigir",
    "transilvana": "transilvano",
    "transilvanas": "transilvano",
    "transilvanos": "transilvano",
    "transistores": "transistor",
    "transistorice": "transistorizar",
    "transistoricemos": "transistorizar",
    "transistoricen": "transistorizar",
    "transistorices": "transistorizar",
    "transistoricé": "transistorizar",
    "transistoricéis": "transistorizar",
    "transistoriza": "transistorizar",
    "transistorizaba": "transistorizar",
    "transistorizabais": "transistorizar",
    "transistorizaban": "transistorizar",
    "transistorizabas": "transistorizar",
    "transistorizaciones": "transistorización",
    "transistorizad": "transistorizar",
    "transistorizada": "transistorizar",
    "transistorizadas": "transistorizar",
    "transistorizado": "transistorizar",
    "transistorizados": "transistorizar",
    "transistorizamos": "transistorizar",
    "transistorizan": "transistorizar",
    "transistorizando": "transistorizar",
    "transistorizara": "transistorizar",
    "transistorizarais": "transistorizar",
    "transistorizaran": "transistorizar",
    "transistorizaras": "transistorizar",
    "transistorizare": "transistorizar",
    "transistorizareis": "transistorizar",
    "transistorizaremos": "transistorizar",
    "transistorizaren": "transistorizar",
    "transistorizares": "transistorizar",
    "transistorizaron": "transistorizar",
    "transistorizará": "transistorizar",
    "transistorizarán": "transistorizar",
    "transistorizarás": "transistorizar",
    "transistorizaré": "transistorizar",
    "transistorizaréis": "transistorizar",
    "transistorizaría": "transistorizar",
    "transistorizaríais": "transistorizar",
    "transistorizaríamos": "transistorizar",
    "transistorizarían": "transistorizar",
    "transistorizarías": "transistorizar",
    "transistorizas": "transistorizar",
    "transistorizase": "transistorizar",
    "transistorizaseis": "transistorizar",
    "transistorizasen": "transistorizar",
    "transistorizases": "transistorizar",
    "transistorizaste": "transistorizar",
    "transistorizasteis": "transistorizar",
    "transistorizo": "transistorizar",
    "transistorizábamos": "transistorizar",
    "transistorizáis": "transistorizar",
    "transistorizáramos": "transistorizar",
    "transistorizáremos": "transistorizar",
    "transistorizásemos": "transistorizar",
    "transistorizó": "transistorizar",
    "transita": "transitar",
    "transitaba": "transitar",
    "transitabais": "transitar",
    "transitaban": "transitar",
    "transitabas": "transitar",
    "transitabilidades": "transitabilidad",
    "transitables": "transitable",
    "transitad": "transitar",
    "transitada": "transitar",
    "transitadas": "transitar",
    "transitado": "transitar",
    "transitados": "transitar",
    "transitamos": "transitar",
    "transitan": "transitar",
    "transitando": "transitar",
    "transitara": "transitar",
    "transitarais": "transitar",
    "transitaran": "transitar",
    "transitaras": "transitar",
    "transitare": "transitar",
    "transitareis": "transitar",
    "transitaremos": "transitar",
    "transitaren": "transitar",
    "transitares": "transitar",
    "transitaria": "transitario",
    "transitarias": "transitario",
    "transitarios": "transitario",
    "transitaron": "transitar",
    "transitará": "transitar",
    "transitarán": "transitar",
    "transitarás": "transitar",
    "transitaré": "transitar",
    "transitaréis": "transitar",
    "transitaría": "transitar",
    "transitaríais": "transitar",
    "transitaríamos": "transitar",
    "transitarían": "transitar",
    "transitarías": "transitar",
    "transitas": "transitar",
    "transitase": "transitar",
    "transitaseis": "transitar",
    "transitasen": "transitar",
    "transitases": "transitar",
    "transitaste": "transitar",
    "transitasteis": "transitar",
    "transite": "transitar",
    "transitemos": "transitar",
    "transiten": "transitar",
    "transites": "transitar",
    "transitiva": "transitivo",
    "transitivas": "transitivo",
    "transitividades": "transitividad",
    "transitivos": "transitivo",
    "transito": "transitar",
    "transitoria": "transitorio",
    "transitorias": "transitorio",
    "transitoriedades": "transitoriedad",
    "transitorios": "transitorio",
    "transitábamos": "transitar",
    "transitáis": "transitar",
    "transitáramos": "transitar",
    "transitáremos": "transitar",
    "transitásemos": "transitar",
    "transité": "transitar",
    "transitéis": "transitar",
    "transitó": "transitar",
    "translaciones": "translación",
    "translaticia": "translaticio",
    "translaticias": "translaticio",
    "translaticios": "translaticio",
    "translativa": "translativo",
    "translativas": "translativo",
    "translativos": "translativo",
    "translimita": "translimitar",
    "translimitaba": "translimitar",
    "translimitabais": "translimitar",
    "translimitaban": "translimitar",
    "translimitabas": "translimitar",
    "translimitaciones": "translimitación",
    "translimitad": "translimitar",
    "translimitada": "translimitar",
    "translimitadas": "translimitar",
    "translimitado": "translimitar",
    "translimitados": "translimitar",
    "translimitamos": "translimitar",
    "translimitan": "translimitar",
    "translimitando": "translimitar",
    "translimitara": "translimitar",
    "translimitarais": "translimitar",
    "translimitaran": "translimitar",
    "translimitaras": "translimitar",
    "translimitare": "translimitar",
    "translimitareis": "translimitar",
    "translimitaremos": "translimitar",
    "translimitaren": "translimitar",
    "translimitares": "translimitar",
    "translimitaron": "translimitar",
    "translimitará": "translimitar",
    "translimitarán": "translimitar",
    "translimitarás": "translimitar",
    "translimitaré": "translimitar",
    "translimitaréis": "translimitar",
    "translimitaría": "translimitar",
    "translimitaríais": "translimitar",
    "translimitaríamos": "translimitar",
    "translimitarían": "translimitar",
    "translimitarías": "translimitar",
    "translimitas": "translimitar",
    "translimitase": "translimitar",
    "translimitaseis": "translimitar",
    "translimitasen": "translimitar",
    "translimitases": "translimitar",
    "translimitaste": "translimitar",
    "translimitasteis": "translimitar",
    "translimite": "translimitar",
    "translimitemos": "translimitar",
    "translimiten": "translimitar",
    "translimites": "translimitar",
    "translimito": "translimitar",
    "translimitábamos": "translimitar",
    "translimitáis": "translimitar",
    "translimitáramos": "translimitar",
    "translimitáremos": "translimitar",
    "translimitásemos": "translimitar",
    "translimité": "translimitar",
    "translimitéis": "translimitar",
    "translimitó": "translimitar",
    "translitera": "transliterar",
    "transliteraba": "transliterar",
    "transliterabais": "transliterar",
    "transliteraban": "transliterar",
    "transliterabas": "transliterar",
    "transliteraciones": "transliteración",
    "transliterad": "transliterar",
    "transliterada": "transliterar",
    "transliteradas": "transliterar",
    "transliterado": "transliterar",
    "transliterados": "transliterar",
    "transliteramos": "transliterar",
    "transliteran": "transliterar",
    "transliterando": "transliterar",
    "transliterara": "transliterar",
    "transliterarais": "transliterar",
    "transliteraran": "transliterar",
    "transliteraras": "transliterar",
    "transliterare": "transliterar",
    "transliterareis": "transliterar",
    "transliteraremos": "transliterar",
    "transliteraren": "transliterar",
    "transliterares": "transliterar",
    "transliteraron": "transliterar",
    "transliterará": "transliterar",
    "transliterarán": "transliterar",
    "transliterarás": "transliterar",
    "transliteraré": "transliterar",
    "transliteraréis": "transliterar",
    "transliteraría": "transliterar",
    "transliteraríais": "transliterar",
    "transliteraríamos": "transliterar",
    "transliterarían": "transliterar",
    "transliterarías": "transliterar",
    "transliteras": "transliterar",
    "transliterase": "transliterar",
    "transliteraseis": "transliterar",
    "transliterasen": "transliterar",
    "transliterases": "transliterar",
    "transliteraste": "transliterar",
    "transliterasteis": "transliterar",
    "translitere": "transliterar",
    "transliteremos": "transliterar",
    "transliteren": "transliterar",
    "transliteres": "transliterar",
    "translitero": "transliterar",
    "transliterábamos": "transliterar",
    "transliteráis": "transliterar",
    "transliteráramos": "transliterar",
    "transliteráremos": "transliterar",
    "transliterásemos": "transliterar",
    "transliteré": "transliterar",
    "transliteréis": "transliterar",
    "transliteró": "transliterar",
    "translocaciones": "translocación",
    "translocaciónes": "translocación",
    "transluce": "translucir",
    "translucen": "translucir",
    "transluces": "translucir",
    "translucid": "translucir",
    "translucida": "translucir",
    "translucidas": "translucir",
    "translucideces": "translucidez",
    "translucido": "translucir",
    "translucidos": "translucir",
    "transluciendo": "translucir",
    "transluciera": "translucir",
    "translucierais": "translucir",
    "translucieran": "translucir",
    "translucieras": "translucir",
    "transluciere": "translucir",
    "transluciereis": "translucir",
    "translucieren": "translucir",
    "translucieres": "translucir",
    "translucieron": "translucir",
    "transluciese": "translucir",
    "translucieseis": "translucir",
    "transluciesen": "translucir",
    "translucieses": "translucir",
    "translucimos": "translucir",
    "transluciremos": "translucir",
    "translucirá": "translucir",
    "translucirán": "translucir",
    "translucirás": "translucir",
    "transluciré": "translucir",
    "transluciréis": "translucir",
    "transluciría": "translucir",
    "transluciríais": "translucir",
    "transluciríamos": "translucir",
    "translucirían": "translucir",
    "translucirías": "translucir",
    "transluciste": "translucir",
    "translucisteis": "translucir",
    "transluciéramos": "translucir",
    "transluciéremos": "translucir",
    "transluciésemos": "translucir",
    "translució": "translucir",
    "translucí": "translucir",
    "translucía": "translucir",
    "translucíais": "translucir",
    "translucíamos": "translucir",
    "translucían": "translucir",
    "translucías": "translucir",
    "translucís": "translucir",
    "transluzca": "translucir",
    "transluzcamos": "translucir",
    "transluzcan": "translucir",
    "transluzcas": "translucir",
    "transluzco": "translucir",
    "transluzcáis": "translucir",
    "translúcida": "translúcido",
    "translúcidas": "translúcido",
    "translúcidos": "translúcido",
    "transmarina": "transmarino",
    "transmarinas": "transmarino",
    "transmarinos": "transmarino",
    "transmediterránea": "transmediterráneo",
    "transmediterráneas": "transmediterráneo",
    "transmediterráneos": "transmediterráneo",
    "transmembranosa": "transmembranoso",
    "transmembranosas": "transmembranoso",
    "transmembranosos": "transmembranoso",
    "transmetilaciónes": "transmetilación",
    "transmigra": "transmigrar",
    "transmigraba": "transmigrar",
    "transmigrabais": "transmigrar",
    "transmigraban": "transmigrar",
    "transmigrabas": "transmigrar",
    "transmigraciones": "transmigración",
    "transmigrad": "transmigrar",
    "transmigrada": "transmigrar",
    "transmigradas": "transmigrar",
    "transmigrado": "transmigrar",
    "transmigrados": "transmigrar",
    "transmigramos": "transmigrar",
    "transmigran": "transmigrar",
    "transmigrando": "transmigrar",
    "transmigrara": "transmigrar",
    "transmigrarais": "transmigrar",
    "transmigraran": "transmigrar",
    "transmigraras": "transmigrar",
    "transmigrare": "transmigrar",
    "transmigrareis": "transmigrar",
    "transmigraremos": "transmigrar",
    "transmigraren": "transmigrar",
    "transmigrares": "transmigrar",
    "transmigraron": "transmigrar",
    "transmigrará": "transmigrar",
    "transmigrarán": "transmigrar",
    "transmigrarás": "transmigrar",
    "transmigraré": "transmigrar",
    "transmigraréis": "transmigrar",
    "transmigraría": "transmigrar",
    "transmigraríais": "transmigrar",
    "transmigraríamos": "transmigrar",
    "transmigrarían": "transmigrar",
    "transmigrarías": "transmigrar",
    "transmigras": "transmigrar",
    "transmigrase": "transmigrar",
    "transmigraseis": "transmigrar",
    "transmigrasen": "transmigrar",
    "transmigrases": "transmigrar",
    "transmigraste": "transmigrar",
    "transmigrasteis": "transmigrar",
    "transmigratoria": "transmigratorio",
    "transmigratorias": "transmigratorio",
    "transmigratorios": "transmigratorio",
    "transmigre": "transmigrar",
    "transmigremos": "transmigrar",
    "transmigren": "transmigrar",
    "transmigres": "transmigrar",
    "transmigro": "transmigrar",
    "transmigrábamos": "transmigrar",
    "transmigráis": "transmigrar",
    "transmigráramos": "transmigrar",
    "transmigráremos": "transmigrar",
    "transmigrásemos": "transmigrar",
    "transmigré": "transmigrar",
    "transmigréis": "transmigrar",
    "transmigró": "transmigrar",
    "transmisibilidades": "transmisibilidad",
    "transmisibles": "transmisible",
    "transmisiones": "transmisión",
    "transmisividades": "transmisividad",
    "transmisora": "transmisor",
    "transmisoras": "transmisor",
    "transmisores": "transmisor",
    "transmita": "transmitir",
    "transmitamos": "transmitir",
    "transmitan": "transmitir",
    "transmitancias": "transmitancia",
    "transmitas": "transmitir",
    "transmite": "transmitir",
    "transmiten": "transmitir",
    "transmites": "transmitir",
    "transmitid": "transmitir",
    "transmitida": "transmitir",
    "transmitidas": "transmitir",
    "transmitido": "transmitir",
    "transmitidos": "transmitir",
    "transmitiendo": "transmitir",
    "transmitiera": "transmitir",
    "transmitierais": "transmitir",
    "transmitieran": "transmitir",
    "transmitieras": "transmitir",
    "transmitiere": "transmitir",
    "transmitiereis": "transmitir",
    "transmitieren": "transmitir",
    "transmitieres": "transmitir",
    "transmitieron": "transmitir",
    "transmitiese": "transmitir",
    "transmitieseis": "transmitir",
    "transmitiesen": "transmitir",
    "transmitieses": "transmitir",
    "transmitimos": "transmitir",
    "transmitiremos": "transmitir",
    "transmitirá": "transmitir",
    "transmitirán": "transmitir",
    "transmitirás": "transmitir",
    "transmitiré": "transmitir",
    "transmitiréis": "transmitir",
    "transmitiría": "transmitir",
    "transmitiríais": "transmitir",
    "transmitiríamos": "transmitir",
    "transmitirían": "transmitir",
    "transmitirías": "transmitir",
    "transmitiste": "transmitir",
    "transmitisteis": "transmitir",
    "transmitiéramos": "transmitir",
    "transmitiéremos": "transmitir",
    "transmitiésemos": "transmitir",
    "transmitió": "transmitir",
    "transmito": "transmitir",
    "transmitáis": "transmitir",
    "transmití": "transmitir",
    "transmitía": "transmitir",
    "transmitíais": "transmitir",
    "transmitíamos": "transmitir",
    "transmitían": "transmitir",
    "transmitías": "transmitir",
    "transmitís": "transmitir",
    "transmodulaciones": "transmodulación",
    "transmontana": "transmontano",
    "transmontanas": "transmontano",
    "transmontanos": "transmontano",
    "transmontes": "transmonte",
    "transmuda": "transmudar",
    "transmudaba": "transmudar",
    "transmudabais": "transmudar",
    "transmudaban": "transmudar",
    "transmudabas": "transmudar",
    "transmudaciones": "transmudación",
    "transmudad": "transmudar",
    "transmudada": "transmudar",
    "transmudadas": "transmudar",
    "transmudado": "transmudar",
    "transmudados": "transmudar",
    "transmudamientos": "transmudamiento",
    "transmudamos": "transmudar",
    "transmudan": "transmudar",
    "transmudando": "transmudar",
    "transmudara": "transmudar",
    "transmudarais": "transmudar",
    "transmudaran": "transmudar",
    "transmudaras": "transmudar",
    "transmudare": "transmudar",
    "transmudareis": "transmudar",
    "transmudaremos": "transmudar",
    "transmudaren": "transmudar",
    "transmudares": "transmudar",
    "transmudaron": "transmudar",
    "transmudará": "transmudar",
    "transmudarán": "transmudar",
    "transmudarás": "transmudar",
    "transmudaré": "transmudar",
    "transmudaréis": "transmudar",
    "transmudaría": "transmudar",
    "transmudaríais": "transmudar",
    "transmudaríamos": "transmudar",
    "transmudarían": "transmudar",
    "transmudarías": "transmudar",
    "transmudas": "transmudar",
    "transmudase": "transmudar",
    "transmudaseis": "transmudar",
    "transmudasen": "transmudar",
    "transmudases": "transmudar",
    "transmudaste": "transmudar",
    "transmudasteis": "transmudar",
    "transmude": "transmudar",
    "transmudemos": "transmudar",
    "transmuden": "transmudar",
    "transmudes": "transmudar",
    "transmudo": "transmudar",
    "transmudábamos": "transmudar",
    "transmudáis": "transmudar",
    "transmudáramos": "transmudar",
    "transmudáremos": "transmudar",
    "transmudásemos": "transmudar",
    "transmudé": "transmudar",
    "transmudéis": "transmudar",
    "transmudó": "transmudar",
    "transmultiplexadores": "transmultiplexador",
    "transmultiplexados": "transmultiplexado",
    "transmuta": "transmutar",
    "transmutaba": "transmutar",
    "transmutabais": "transmutar",
    "transmutaban": "transmutar",
    "transmutabas": "transmutar",
    "transmutables": "transmutable",
    "transmutaciones": "transmutación",
    "transmutad": "transmutar",
    "transmutada": "transmutar",
    "transmutadas": "transmutar",
    "transmutado": "transmutar",
    "transmutados": "transmutar",
    "transmutamos": "transmutar",
    "transmutan": "transmutar",
    "transmutancias": "transmutancia",
    "transmutando": "transmutar",
    "transmutara": "transmutar",
    "transmutarais": "transmutar",
    "transmutaran": "transmutar",
    "transmutaras": "transmutar",
    "transmutare": "transmutar",
    "transmutareis": "transmutar",
    "transmutaremos": "transmutar",
    "transmutaren": "transmutar",
    "transmutares": "transmutar",
    "transmutaron": "transmutar",
    "transmutará": "transmutar",
    "transmutarán": "transmutar",
    "transmutarás": "transmutar",
    "transmutaré": "transmutar",
    "transmutaréis": "transmutar",
    "transmutaría": "transmutar",
    "transmutaríais": "transmutar",
    "transmutaríamos": "transmutar",
    "transmutarían": "transmutar",
    "transmutarías": "transmutar",
    "transmutas": "transmutar",
    "transmutase": "transmutar",
    "transmutaseis": "transmutar",
    "transmutasen": "transmutar",
    "transmutases": "transmutar",
    "transmutaste": "transmutar",
    "transmutasteis": "transmutar",
    "transmutativa": "transmutativo",
    "transmutativas": "transmutativo",
    "transmutativos": "transmutativo",
    "transmutatoria": "transmutatorio",
    "transmutatorias": "transmutatorio",
    "transmutatorios": "transmutatorio",
    "transmute": "transmutar",
    "transmutemos": "transmutar",
    "transmuten": "transmutar",
    "transmutes": "transmutar",
    "transmuto": "transmutar",
    "transmutábamos": "transmutar",
    "transmutáis": "transmutar",
    "transmutáramos": "transmutar",
    "transmutáremos": "transmutar",
    "transmutásemos": "transmutar",
    "transmuté": "transmutar",
    "transmutéis": "transmutar",
    "transmutó": "transmutar",
    "transnacionales": "transnacional",
    "transnacionalistas": "transnacionalista",
    "transnacionalizaciones": "transnacionalización",
    "transoceánica": "transoceánico",
    "transoceánicas": "transoceánico",
    "transoceánicos": "transoceánico",
    "transpacífica": "transpacífico",
    "transpacíficas": "transpacífico",
    "transpacíficos": "transpacífico",
    "transpadana": "transpadano",
    "transpadanas": "transpadano",
    "transpadanos": "transpadano",
    "transpalatales": "transpalatal",
    "transparencias": "transparencia",
    "transparenta": "transparentar",
    "transparentaba": "transparentar",
    "transparentabais": "transparentar",
    "transparentaban": "transparentar",
    "transparentabas": "transparentar",
    "transparentad": "transparentar",
    "transparentada": "transparentar",
    "transparentadas": "transparentar",
    "transparentado": "transparentar",
    "transparentados": "transparentar",
    "transparentamos": "transparentar",
    "transparentan": "transparentar",
    "transparentando": "transparentar",
    "transparentara": "transparentar",
    "transparentarais": "transparentar",
    "transparentaran": "transparentar",
    "transparentaras": "transparentar",
    "transparentare": "transparentar",
    "transparentareis": "transparentar",
    "transparentaremos": "transparentar",
    "transparentaren": "transparentar",
    "transparentares": "transparentar",
    "transparentaron": "transparentar",
    "transparentará": "transparentar",
    "transparentarán": "transparentar",
    "transparentarás": "transparentar",
    "transparentaré": "transparentar",
    "transparentaréis": "transparentar",
    "transparentaría": "transparentar",
    "transparentaríais": "transparentar",
    "transparentaríamos": "transparentar",
    "transparentarían": "transparentar",
    "transparentarías": "transparentar",
    "transparentas": "transparentar",
    "transparentase": "transparentar",
    "transparentaseis": "transparentar",
    "transparentasen": "transparentar",
    "transparentases": "transparentar",
    "transparentaste": "transparentar",
    "transparentasteis": "transparentar",
    "transparente": "transparentar",
    "transparentemos": "transparentar",
    "transparenten": "transparentar",
    "transparentes": "transparente",
    "transparento": "transparentar",
    "transparentábamos": "transparentar",
    "transparentáis": "transparentar",
    "transparentáramos": "transparentar",
    "transparentáremos": "transparentar",
    "transparentásemos": "transparentar",
    "transparenté": "transparentar",
    "transparentéis": "transparentar",
    "transparentó": "transparentar",
    "transparietales": "transparietal",
    "transpilórica": "transpilórico",
    "transpilóricas": "transpilórico",
    "transpilóricos": "transpilórico",
    "transpira": "transpirar",
    "transpiraba": "transpirar",
    "transpirabais": "transpirar",
    "transpiraban": "transpirar",
    "transpirabas": "transpirar",
    "transpirabilidades": "transpirabilidad",
    "transpirables": "transpirable",
    "transpiraciones": "transpiración",
    "transpirad": "transpirar",
    "transpirada": "transpirar",
    "transpiradas": "transpirar",
    "transpirado": "transpirar",
    "transpirados": "transpirar",
    "transpiramos": "transpirar",
    "transpiran": "transpirar",
    "transpirando": "transpirar",
    "transpirantes": "transpirante",
    "transpirara": "transpirar",
    "transpirarais": "transpirar",
    "transpiraran": "transpirar",
    "transpiraras": "transpirar",
    "transpirare": "transpirar",
    "transpirareis": "transpirar",
    "transpiraremos": "transpirar",
    "transpiraren": "transpirar",
    "transpirares": "transpirar",
    "transpiraron": "transpirar",
    "transpirará": "transpirar",
    "transpirarán": "transpirar",
    "transpirarás": "transpirar",
    "transpiraré": "transpirar",
    "transpiraréis": "transpirar",
    "transpiraría": "transpirar",
    "transpiraríais": "transpirar",
    "transpiraríamos": "transpirar",
    "transpirarían": "transpirar",
    "transpirarías": "transpirar",
    "transpiras": "transpirar",
    "transpirase": "transpirar",
    "transpiraseis": "transpirar",
    "transpirasen": "transpirar",
    "transpirases": "transpirar",
    "transpiraste": "transpirar",
    "transpirasteis": "transpirar",
    "transpire": "transpirar",
    "transpiremos": "transpirar",
    "transpiren": "transpirar",
    "transpirenaica": "transpirenaico",
    "transpirenaicas": "transpirenaico",
    "transpirenaicos": "transpirenaico",
    "transpires": "transpirar",
    "transpiro": "transpirar",
    "transpirábamos": "transpirar",
    "transpiráis": "transpirar",
    "transpiráramos": "transpirar",
    "transpiráremos": "transpirar",
    "transpirásemos": "transpirar",
    "transpiré": "transpirar",
    "transpiréis": "transpirar",
    "transpiró": "transpirar",
    "transplantes": "transplante",
    "transplutoniana": "transplutoniano",
    "transplutonianas": "transplutoniano",
    "transplutonianos": "transplutoniano",
    "transpolares": "transpolar",
    "transpondremos": "transponer",
    "transpondrá": "transponer",
    "transpondrán": "transponer",
    "transpondrás": "transponer",
    "transpondré": "transponer",
    "transpondréis": "transponer",
    "transpondría": "transponer",
    "transpondríais": "transponer",
    "transpondríamos": "transponer",
    "transpondrían": "transponer",
    "transpondrías": "transponer",
    "transpone": "transponer",
    "transponed": "transponer",
    "transponedora": "transponedor",
    "transponedoras": "transponedor",
    "transponedores": "transponedor",
    "transponemos": "transponer",
    "transponen": "transponer",
    "transpones": "transponer",
    "transponga": "transponer",
    "transpongamos": "transponer",
    "transpongan": "transponer",
    "transpongas": "transponer",
    "transpongo": "transponer",
    "transpongáis": "transponer",
    "transponiendo": "transponer",
    "transponéis": "transponer",
    "transponía": "transponer",
    "transponíais": "transponer",
    "transponíamos": "transponer",
    "transponían": "transponer",
    "transponías": "transponer",
    "transporta": "transportar",
    "transportaba": "transportar",
    "transportabais": "transportar",
    "transportaban": "transportar",
    "transportabas": "transportar",
    "transportabilidades": "transportabilidad",
    "transportables": "transportable",
    "transportaciones": "transportación",
    "transportad": "transportar",
    "transportada": "transportar",
    "transportadas": "transportar",
    "transportado": "transportar",
    "transportadora": "transportador",
    "transportadoras": "transportador",
    "transportadores": "transportador",
    "transportados": "transportar",
    "transportamientos": "transportamiento",
    "transportamos": "transportar",
    "transportan": "transportar",
    "transportando": "transportar",
    "transportara": "transportar",
    "transportarais": "transportar",
    "transportaran": "transportar",
    "transportaras": "transportar",
    "transportare": "transportar",
    "transportareis": "transportar",
    "transportaremos": "transportar",
    "transportaren": "transportar",
    "transportares": "transportar",
    "transportaron": "transportar",
    "transportará": "transportar",
    "transportarán": "transportar",
    "transportarás": "transportar",
    "transportaré": "transportar",
    "transportaréis": "transportar",
    "transportaría": "transportar",
    "transportaríais": "transportar",
    "transportaríamos": "transportar",
    "transportarían": "transportar",
    "transportarías": "transportar",
    "transportas": "transportar",
    "transportase": "transportar",
    "transportaseis": "transportar",
    "transportasen": "transportar",
    "transportases": "transportar",
    "transportaste": "transportar",
    "transportasteis": "transportar",
    "transporte": "transportar",
    "transportemos": "transportar",
    "transporten": "transportar",
    "transportes": "transporte",
    "transportistas": "transportista",
    "transporto": "transportar",
    "transportábamos": "transportar",
    "transportáis": "transportar",
    "transportáramos": "transportar",
    "transportáremos": "transportar",
    "transportásemos": "transportar",
    "transporté": "transportar",
    "transportéis": "transportar",
    "transportó": "transportar",
    "transposiciones": "transposición",
    "transpositiva": "transpositivo",
    "transpositivas": "transpositivo",
    "transpositivos": "transpositivo",
    "transposones": "transposón",
    "transpuesta": "transponer",
    "transpuestas": "transponer",
    "transpuesto": "transponer",
    "transpuestos": "transponer",
    "transpuse": "transponer",
    "transpusiera": "transponer",
    "transpusierais": "transponer",
    "transpusieran": "transponer",
    "transpusieras": "transponer",
    "transpusiere": "transponer",
    "transpusiereis": "transponer",
    "transpusieren": "transponer",
    "transpusieres": "transponer",
    "transpusieron": "transponer",
    "transpusiese": "transponer",
    "transpusieseis": "transponer",
    "transpusiesen": "transponer",
    "transpusieses": "transponer",
    "transpusimos": "transponer",
    "transpusiste": "transponer",
    "transpusisteis": "transponer",
    "transpusiéramos": "transponer",
    "transpusiéremos": "transponer",
    "transpusiésemos": "transponer",
    "transpuso": "transponer",
    "transpón": "transponer",
    "transtermina": "transterminar",
    "transterminaba": "transterminar",
    "transterminabais": "transterminar",
    "transterminaban": "transterminar",
    "transterminabas": "transterminar",
    "transterminad": "transterminar",
    "transterminada": "transterminar",
    "transterminadas": "transterminar",
    "transterminado": "transterminar",
    "transterminados": "transterminar",
    "transterminamos": "transterminar",
    "transterminan": "transterminar",
    "transterminando": "transterminar",
    "transterminara": "transterminar",
    "transterminarais": "transterminar",
    "transterminaran": "transterminar",
    "transterminaras": "transterminar",
    "transterminare": "transterminar",
    "transterminareis": "transterminar",
    "transterminaremos": "transterminar",
    "transterminaren": "transterminar",
    "transterminares": "transterminar",
    "transterminaron": "transterminar",
    "transterminará": "transterminar",
    "transterminarán": "transterminar",
    "transterminarás": "transterminar",
    "transterminaré": "transterminar",
    "transterminaréis": "transterminar",
    "transterminaría": "transterminar",
    "transterminaríais": "transterminar",
    "transterminaríamos": "transterminar",
    "transterminarían": "transterminar",
    "transterminarías": "transterminar",
    "transterminas": "transterminar",
    "transterminase": "transterminar",
    "transterminaseis": "transterminar",
    "transterminasen": "transterminar",
    "transterminases": "transterminar",
    "transterminaste": "transterminar",
    "transterminasteis": "transterminar",
    "transtermine": "transterminar",
    "transterminemos": "transterminar",
    "transterminen": "transterminar",
    "transtermines": "transterminar",
    "transtermino": "transterminar",
    "transterminábamos": "transterminar",
    "transtermináis": "transterminar",
    "transtermináramos": "transterminar",
    "transtermináremos": "transterminar",
    "transterminásemos": "transterminar",
    "transterminé": "transterminar",
    "transterminéis": "transterminar",
    "transterminó": "transterminar",
    "transtiberina": "transtiberino",
    "transtiberinas": "transtiberino",
    "transtiberinos": "transtiberino",
    "transtimpánica": "transtimpánico",
    "transtimpánicas": "transtimpánico",
    "transtimpánicos": "transtimpánico",
    "transubstancia": "transubstanciar",
    "transubstanciaba": "transubstanciar",
    "transubstanciabais": "transubstanciar",
    "transubstanciaban": "transubstanciar",
    "transubstanciabas": "transubstanciar",
    "transubstanciaciones": "transubstanciación",
    "transubstanciad": "transubstanciar",
    "transubstanciada": "transubstanciar",
    "transubstanciadas": "transubstanciar",
    "transubstanciado": "transubstanciar",
    "transubstanciados": "transubstanciar",
    "transubstanciales": "transubstancial",
    "transubstanciamos": "transubstanciar",
    "transubstancian": "transubstanciar",
    "transubstanciando": "transubstanciar",
    "transubstanciara": "transubstanciar",
    "transubstanciarais": "transubstanciar",
    "transubstanciaran": "transubstanciar",
    "transubstanciaras": "transubstanciar",
    "transubstanciare": "transubstanciar",
    "transubstanciareis": "transubstanciar",
    "transubstanciaremos": "transubstanciar",
    "transubstanciaren": "transubstanciar",
    "transubstanciares": "transubstanciar",
    "transubstanciaron": "transubstanciar",
    "transubstanciará": "transubstanciar",
    "transubstanciarán": "transubstanciar",
    "transubstanciarás": "transubstanciar",
    "transubstanciaré": "transubstanciar",
    "transubstanciaréis": "transubstanciar",
    "transubstanciaría": "transubstanciar",
    "transubstanciaríais": "transubstanciar",
    "transubstanciaríamos": "transubstanciar",
    "transubstanciarían": "transubstanciar",
    "transubstanciarías": "transubstanciar",
    "transubstancias": "transubstanciar",
    "transubstanciase": "transubstanciar",
    "transubstanciaseis": "transubstanciar",
    "transubstanciasen": "transubstanciar",
    "transubstanciases": "transubstanciar",
    "transubstanciaste": "transubstanciar",
    "transubstanciasteis": "transubstanciar",
    "transubstancie": "transubstanciar",
    "transubstanciemos": "transubstanciar",
    "transubstancien": "transubstanciar",
    "transubstancies": "transubstanciar",
    "transubstancio": "transubstanciar",
    "transubstanciábamos": "transubstanciar",
    "transubstanciáis": "transubstanciar",
    "transubstanciáramos": "transubstanciar",
    "transubstanciáremos": "transubstanciar",
    "transubstanciásemos": "transubstanciar",
    "transubstancié": "transubstanciar",
    "transubstanciéis": "transubstanciar",
    "transubstanció": "transubstanciar",
    "transuranios": "transuranio",
    "transuránica": "transuránico",
    "transuránicas": "transuránico",
    "transuránicos": "transuránico",
    "transuránida": "transuránido",
    "transuránidas": "transuránido",
    "transuránidos": "transuránido",
    "transvasa": "transvasar",
    "transvasaba": "transvasar",
    "transvasabais": "transvasar",
    "transvasaban": "transvasar",
    "transvasabas": "transvasar",
    "transvasad": "transvasar",
    "transvasada": "transvasar",
    "transvasadas": "transvasar",
    "transvasado": "transvasar",
    "transvasados": "transvasar",
    "transvasamos": "transvasar",
    "transvasan": "transvasar",
    "transvasando": "transvasar",
    "transvasara": "transvasar",
    "transvasarais": "transvasar",
    "transvasaran": "transvasar",
    "transvasaras": "transvasar",
    "transvasare": "transvasar",
    "transvasareis": "transvasar",
    "transvasaremos": "transvasar",
    "transvasaren": "transvasar",
    "transvasares": "transvasar",
    "transvasaron": "transvasar",
    "transvasará": "transvasar",
    "transvasarán": "transvasar",
    "transvasarás": "transvasar",
    "transvasaré": "transvasar",
    "transvasaréis": "transvasar",
    "transvasaría": "transvasar",
    "transvasaríais": "transvasar",
    "transvasaríamos": "transvasar",
    "transvasarían": "transvasar",
    "transvasarías": "transvasar",
    "transvasas": "transvasar",
    "transvasase": "transvasar",
    "transvasaseis": "transvasar",
    "transvasasen": "transvasar",
    "transvasases": "transvasar",
    "transvasaste": "transvasar",
    "transvasasteis": "transvasar",
    "transvase": "transvasar",
    "transvasemos": "transvasar",
    "transvasen": "transvasar",
    "transvases": "transvase",
    "transvaso": "transvasar",
    "transvasábamos": "transvasar",
    "transvasáis": "transvasar",
    "transvasáramos": "transvasar",
    "transvasáremos": "transvasar",
    "transvasásemos": "transvasar",
    "transvasé": "transvasar",
    "transvaséis": "transvasar",
    "transvasó": "transvasar",
    "transvecciones": "transvección",
    "transverberaciones": "transverberación",
    "transversa": "transverso",
    "transversales": "transversal",
    "transversas": "transverso",
    "transversos": "transverso",
    "transvestida": "transvestido",
    "transvestidas": "transvestido",
    "transvestidos": "transvestido",
    "transvestismos": "transvestismo",
    "transónica": "transónico",
    "transónicas": "transónico",
    "transónicos": "transónico",
    "transósea": "transóseo",
    "transóseas": "transóseo",
    "transóseos": "transóseo",
    "tranversales": "tranversal",
    "tranviaria": "tranviario",
    "tranviarias": "tranviario",
    "tranviarios": "tranviario",
    "tranvieros": "tranviero",
    "tranvías": "tranvía",
    "tranza": "tranzar",
    "tranzaba": "tranzar",
    "tranzabais": "tranzar",
    "tranzaban": "tranzar",
    "tranzabas": "tranzar",
    "tranzad": "tranzar",
    "tranzada": "tranzar",
    "tranzadas": "tranzar",
    "tranzado": "tranzar",
    "tranzados": "tranzar",
    "tranzamos": "tranzar",
    "tranzan": "tranzar",
    "tranzando": "tranzar",
    "tranzara": "tranzar",
    "tranzarais": "tranzar",
    "tranzaran": "tranzar",
    "tranzaras": "tranzar",
    "tranzare": "tranzar",
    "tranzareis": "tranzar",
    "tranzaremos": "tranzar",
    "tranzaren": "tranzar",
    "tranzares": "tranzar",
    "tranzaron": "tranzar",
    "tranzará": "tranzar",
    "tranzarán": "tranzar",
    "tranzarás": "tranzar",
    "tranzaré": "tranzar",
    "tranzaréis": "tranzar",
    "tranzaría": "tranzar",
    "tranzaríais": "tranzar",
    "tranzaríamos": "tranzar",
    "tranzarían": "tranzar",
    "tranzarías": "tranzar",
    "tranzas": "tranzar",
    "tranzase": "tranzar",
    "tranzaseis": "tranzar",
    "tranzasen": "tranzar",
    "tranzases": "tranzar",
    "tranzaste": "tranzar",
    "tranzasteis": "tranzar",
    "tranzo": "tranzar",
    "tranzones": "tranzón",
    "tranzábamos": "tranzar",
    "tranzáis": "tranzar",
    "tranzáramos": "tranzar",
    "tranzáremos": "tranzar",
    "tranzásemos": "tranzar",
    "tranzó": "tranzar",
    "trapacea": "trapacear",
    "trapaceaba": "trapacear",
    "trapaceabais": "trapacear",
    "trapaceaban": "trapacear",
    "trapaceabas": "trapacear",
    "trapacead": "trapacear",
    "trapaceada": "trapacear",
    "trapaceadas": "trapacear",
    "trapaceado": "trapacear",
    "trapaceados": "trapacear",
    "trapaceamos": "trapacear",
    "trapacean": "trapacear",
    "trapaceando": "trapacear",
    "trapaceara": "trapacear",
    "trapacearais": "trapacear",
    "trapacearan": "trapacear",
    "trapacearas": "trapacear",
    "trapaceare": "trapacear",
    "trapaceareis": "trapacear",
    "trapacearemos": "trapacear",
    "trapacearen": "trapacear",
    "trapaceares": "trapacear",
    "trapacearon": "trapacear",
    "trapaceará": "trapacear",
    "trapacearán": "trapacear",
    "trapacearás": "trapacear",
    "trapacearé": "trapacear",
    "trapacearéis": "trapacear",
    "trapacearía": "trapacear",
    "trapacearíais": "trapacear",
    "trapacearíamos": "trapacear",
    "trapacearían": "trapacear",
    "trapacearías": "trapacear",
    "trapaceas": "trapacear",
    "trapacease": "trapacear",
    "trapaceaseis": "trapacear",
    "trapaceasen": "trapacear",
    "trapaceases": "trapacear",
    "trapaceaste": "trapacear",
    "trapaceasteis": "trapacear",
    "trapacee": "trapacear",
    "trapaceemos": "trapacear",
    "trapaceen": "trapacear",
    "trapacees": "trapacear",
    "trapaceo": "trapacear",
    "trapacera": "trapacero",
    "trapaceras": "trapacero",
    "trapaceros": "trapacero",
    "trapacerías": "trapacería",
    "trapacetes": "trapacete",
    "trapaceábamos": "trapacear",
    "trapaceáis": "trapacear",
    "trapaceáramos": "trapacear",
    "trapaceáremos": "trapacear",
    "trapaceásemos": "trapacear",
    "trapaceé": "trapacear",
    "trapaceéis": "trapacear",
    "trapaceó": "trapacear",
    "trapacistas": "trapacista",
    "trapajosa": "trapajoso",
    "trapajosas": "trapajoso",
    "trapajosos": "trapajoso",
    "trapalas": "trapala",
    "trapalea": "trapalear",
    "trapaleaba": "trapalear",
    "trapaleabais": "trapalear",
    "trapaleaban": "trapalear",
    "trapaleabas": "trapalear",
    "trapalead": "trapalear",
    "trapaleada": "trapalear",
    "trapaleadas": "trapalear",
    "trapaleado": "trapalear",
    "trapaleados": "trapalear",
    "trapaleamos": "trapalear",
    "trapalean": "trapalear",
    "trapaleando": "trapalear",
    "trapaleara": "trapalear",
    "trapalearais": "trapalear",
    "trapalearan": "trapalear",
    "trapalearas": "trapalear",
    "trapaleare": "trapalear",
    "trapaleareis": "trapalear",
    "trapalearemos": "trapalear",
    "trapalearen": "trapalear",
    "trapaleares": "trapalear",
    "trapalearon": "trapalear",
    "trapaleará": "trapalear",
    "trapalearán": "trapalear",
    "trapalearás": "trapalear",
    "trapalearé": "trapalear",
    "trapalearéis": "trapalear",
    "trapalearía": "trapalear",
    "trapalearíais": "trapalear",
    "trapalearíamos": "trapalear",
    "trapalearían": "trapalear",
    "trapalearías": "trapalear",
    "trapaleas": "trapalear",
    "trapalease": "trapalear",
    "trapaleaseis": "trapalear",
    "trapaleasen": "trapalear",
    "trapaleases": "trapalear",
    "trapaleaste": "trapalear",
    "trapaleasteis": "trapalear",
    "trapalee": "trapalear",
    "trapaleemos": "trapalear",
    "trapaleen": "trapalear",
    "trapalees": "trapalear",
    "trapaleo": "trapalear",
    "trapaleábamos": "trapalear",
    "trapaleáis": "trapalear",
    "trapaleáramos": "trapalear",
    "trapaleáremos": "trapalear",
    "trapaleásemos": "trapalear",
    "trapaleé": "trapalear",
    "trapaleéis": "trapalear",
    "trapaleó": "trapalear",
    "trapalona": "trapalón",
    "trapalonas": "trapalón",
    "trapalones": "trapalón",
    "trapas": "trapa",
    "trapeciales": "trapecial",
    "trapecios": "trapecio",
    "trapecistas": "trapecista",
    "trapenses": "trapense",
    "trapera": "trapero",
    "traperas": "trapero",
    "traperos": "trapero",
    "traperías": "trapería",
    "trapeziales": "trapezial",
    "trapezoedros": "trapezoedro",
    "trapezoidales": "trapezoidal",
    "trapezoidea": "trapezoideo",
    "trapezoideas": "trapezoideo",
    "trapezoideos": "trapezoideo",
    "trapezoides": "trapezoide",
    "trapezoédrica": "trapezoédrico",
    "trapezoédricas": "trapezoédrico",
    "trapezoédricos": "trapezoédrico",
    "trapichea": "trapichear",
    "trapicheaba": "trapichear",
    "trapicheabais": "trapichear",
    "trapicheaban": "trapichear",
    "trapicheabas": "trapichear",
    "trapichead": "trapichear",
    "trapicheada": "trapichear",
    "trapicheadas": "trapichear",
    "trapicheado": "trapichear",
    "trapicheados": "trapichear",
    "trapicheamos": "trapichear",
    "trapichean": "trapichear",
    "trapicheando": "trapichear",
    "trapicheara": "trapichear",
    "trapichearais": "trapichear",
    "trapichearan": "trapichear",
    "trapichearas": "trapichear",
    "trapicheare": "trapichear",
    "trapicheareis": "trapichear",
    "trapichearemos": "trapichear",
    "trapichearen": "trapichear",
    "trapicheares": "trapichear",
    "trapichearon": "trapichear",
    "trapicheará": "trapichear",
    "trapichearán": "trapichear",
    "trapichearás": "trapichear",
    "trapichearé": "trapichear",
    "trapichearéis": "trapichear",
    "trapichearía": "trapichear",
    "trapichearíais": "trapichear",
    "trapichearíamos": "trapichear",
    "trapichearían": "trapichear",
    "trapichearías": "trapichear",
    "trapicheas": "trapichear",
    "trapichease": "trapichear",
    "trapicheaseis": "trapichear",
    "trapicheasen": "trapichear",
    "trapicheases": "trapichear",
    "trapicheaste": "trapichear",
    "trapicheasteis": "trapichear",
    "trapichee": "trapichear",
    "trapicheemos": "trapichear",
    "trapicheen": "trapichear",
    "trapichees": "trapichear",
    "trapicheo": "trapichear",
    "trapicheos": "trapicheo",
    "trapiches": "trapiche",
    "trapicheábamos": "trapichear",
    "trapicheáis": "trapichear",
    "trapicheáramos": "trapichear",
    "trapicheáremos": "trapichear",
    "trapicheásemos": "trapichear",
    "trapicheé": "trapichear",
    "trapicheéis": "trapichear",
    "trapicheó": "trapichear",
    "trapienta": "trapiento",
    "trapientas": "trapiento",
    "trapientos": "trapiento",
    "trapillos": "trapillo",
    "trapisondas": "trapisonda",
    "trapisondea": "trapisondear",
    "trapisondeaba": "trapisondear",
    "trapisondeabais": "trapisondear",
    "trapisondeaban": "trapisondear",
    "trapisondeabas": "trapisondear",
    "trapisondead": "trapisondear",
    "trapisondeada": "trapisondear",
    "trapisondeadas": "trapisondear",
    "trapisondeado": "trapisondear",
    "trapisondeados": "trapisondear",
    "trapisondeamos": "trapisondear",
    "trapisondean": "trapisondear",
    "trapisondeando": "trapisondear",
    "trapisondeara": "trapisondear",
    "trapisondearais": "trapisondear",
    "trapisondearan": "trapisondear",
    "trapisondearas": "trapisondear",
    "trapisondeare": "trapisondear",
    "trapisondeareis": "trapisondear",
    "trapisondearemos": "trapisondear",
    "trapisondearen": "trapisondear",
    "trapisondeares": "trapisondear",
    "trapisondearon": "trapisondear",
    "trapisondeará": "trapisondear",
    "trapisondearán": "trapisondear",
    "trapisondearás": "trapisondear",
    "trapisondearé": "trapisondear",
    "trapisondearéis": "trapisondear",
    "trapisondearía": "trapisondear",
    "trapisondearíais": "trapisondear",
    "trapisondearíamos": "trapisondear",
    "trapisondearían": "trapisondear",
    "trapisondearías": "trapisondear",
    "trapisondeas": "trapisondear",
    "trapisondease": "trapisondear",
    "trapisondeaseis": "trapisondear",
    "trapisondeasen": "trapisondear",
    "trapisondeases": "trapisondear",
    "trapisondeaste": "trapisondear",
    "trapisondeasteis": "trapisondear",
    "trapisondee": "trapisondear",
    "trapisondeemos": "trapisondear",
    "trapisondeen": "trapisondear",
    "trapisondees": "trapisondear",
    "trapisondeo": "trapisondear",
    "trapisondeábamos": "trapisondear",
    "trapisondeáis": "trapisondear",
    "trapisondeáramos": "trapisondear",
    "trapisondeáremos": "trapisondear",
    "trapisondeásemos": "trapisondear",
    "trapisondeé": "trapisondear",
    "trapisondeéis": "trapisondear",
    "trapisondeó": "trapisondear",
    "trapisondistas": "trapisondista",
    "trapitos": "trapito",
    "trapos": "trapo",
    "trapíos": "trapío",
    "traquea": "traquear",
    "traqueaba": "traquear",
    "traqueabais": "traquear",
    "traqueaban": "traquear",
    "traqueabas": "traquear",
    "traquead": "traquear",
    "traqueada": "traquear",
    "traqueadas": "traquear",
    "traqueado": "traquear",
    "traqueados": "traquear",
    "traqueales": "traqueal",
    "traqueamos": "traquear",
    "traquean": "traquear",
    "traqueando": "traquear",
    "traqueara": "traquear",
    "traquearais": "traquear",
    "traquearan": "traquear",
    "traquearas": "traquear",
    "traqueare": "traquear",
    "traqueareis": "traquear",
    "traquearemos": "traquear",
    "traquearen": "traquear",
    "traqueares": "traquear",
    "traquearon": "traquear",
    "traqueará": "traquear",
    "traquearán": "traquear",
    "traquearás": "traquear",
    "traquearé": "traquear",
    "traquearéis": "traquear",
    "traquearía": "traquear",
    "traquearíais": "traquear",
    "traquearíamos": "traquear",
    "traquearían": "traquear",
    "traquearías": "traquear",
    "traqueas": "traquear",
    "traquease": "traquear",
    "traqueaseis": "traquear",
    "traqueasen": "traquear",
    "traqueases": "traquear",
    "traqueaste": "traquear",
    "traqueasteis": "traquear",
    "traquee": "traquear",
    "traqueemos": "traquear",
    "traqueen": "traquear",
    "traquees": "traquear",
    "traqueidas": "traqueida",
    "traqueo": "traquear",
    "traqueobronquiales": "traqueobronquial",
    "traqueos": "traqueo",
    "traqueostomas": "traqueostoma",
    "traqueostomías": "traqueostomía",
    "traqueotomías": "traqueotomía",
    "traques": "traque",
    "traquetea": "traquetear",
    "traqueteaba": "traquetear",
    "traqueteabais": "traquetear",
    "traqueteaban": "traquetear",
    "traqueteabas": "traquetear",
    "traquetead": "traquetear",
    "traqueteada": "traquetear",
    "traqueteadas": "traquetear",
    "traqueteado": "traquetear",
    "traqueteados": "traquetear",
    "traqueteamos": "traquetear",
    "traquetean": "traquetear",
    "traqueteando": "traquetear",
    "traqueteara": "traquetear",
    "traquetearais": "traquetear",
    "traquetearan": "traquetear",
    "traquetearas": "traquetear",
    "traqueteare": "traquetear",
    "traqueteareis": "traquetear",
    "traquetearemos": "traquetear",
    "traquetearen": "traquetear",
    "traqueteares": "traquetear",
    "traquetearon": "traquetear",
    "traqueteará": "traquetear",
    "traquetearán": "traquetear",
    "traquetearás": "traquetear",
    "traquetearé": "traquetear",
    "traquetearéis": "traquetear",
    "traquetearía": "traquetear",
    "traquetearíais": "traquetear",
    "traquetearíamos": "traquetear",
    "traquetearían": "traquetear",
    "traquetearías": "traquetear",
    "traqueteas": "traquetear",
    "traquetease": "traquetear",
    "traqueteaseis": "traquetear",
    "traqueteasen": "traquetear",
    "traqueteases": "traquetear",
    "traqueteaste": "traquetear",
    "traqueteasteis": "traquetear",
    "traquetee": "traquetear",
    "traqueteemos": "traquetear",
    "traqueteen": "traquetear",
    "traquetees": "traquetear",
    "traqueteo": "traquetear",
    "traqueteos": "traqueteo",
    "traqueteábamos": "traquetear",
    "traqueteáis": "traquetear",
    "traqueteáramos": "traquetear",
    "traqueteáremos": "traquetear",
    "traqueteásemos": "traquetear",
    "traqueteé": "traquetear",
    "traqueteéis": "traquetear",
    "traqueteó": "traquetear",
    "traqueábamos": "traquear",
    "traqueáis": "traquear",
    "traqueáramos": "traquear",
    "traqueáremos": "traquear",
    "traqueásemos": "traquear",
    "traqueé": "traquear",
    "traqueéis": "traquear",
    "traqueó": "traquear",
    "traquidos": "traquido",
    "traquilinos": "traquilino",
    "traquitas": "traquita",
    "traros": "traro",
    "trasalcobas": "trasalcoba",
    "trasalpina": "trasalpino",
    "trasalpinas": "trasalpino",
    "trasalpinos": "trasalpino",
    "trasaltares": "trasaltar",
    "trasandina": "trasandino",
    "trasandinas": "trasandino",
    "trasandinos": "trasandino",
    "trasandosca": "trasandosco",
    "trasandoscas": "trasandosco",
    "trasandoscos": "trasandosco",
    "trasaneja": "trasanejo",
    "trasanejas": "trasanejo",
    "trasanejos": "trasanejo",
    "trasatlántica": "trasatlántico",
    "trasatlánticas": "trasatlántico",
    "trasatlánticos": "trasatlántico",
    "trasboca": "trasbocar",
    "trasbocaba": "trasbocar",
    "trasbocabais": "trasbocar",
    "trasbocaban": "trasbocar",
    "trasbocabas": "trasbocar",
    "trasbocad": "trasbocar",
    "trasbocada": "trasbocar",
    "trasbocadas": "trasbocar",
    "trasbocado": "trasbocar",
    "trasbocados": "trasbocar",
    "trasbocamos": "trasbocar",
    "trasbocan": "trasbocar",
    "trasbocando": "trasbocar",
    "trasbocara": "trasbocar",
    "trasbocarais": "trasbocar",
    "trasbocaran": "trasbocar",
    "trasbocaras": "trasbocar",
    "trasbocare": "trasbocar",
    "trasbocareis": "trasbocar",
    "trasbocaremos": "trasbocar",
    "trasbocaren": "trasbocar",
    "trasbocares": "trasbocar",
    "trasbocaron": "trasbocar",
    "trasbocará": "trasbocar",
    "trasbocarán": "trasbocar",
    "trasbocarás": "trasbocar",
    "trasbocaré": "trasbocar",
    "trasbocaréis": "trasbocar",
    "trasbocaría": "trasbocar",
    "trasbocaríais": "trasbocar",
    "trasbocaríamos": "trasbocar",
    "trasbocarían": "trasbocar",
    "trasbocarías": "trasbocar",
    "trasbocas": "trasbocar",
    "trasbocase": "trasbocar",
    "trasbocaseis": "trasbocar",
    "trasbocasen": "trasbocar",
    "trasbocases": "trasbocar",
    "trasbocaste": "trasbocar",
    "trasbocasteis": "trasbocar",
    "trasboco": "trasbocar",
    "trasbocábamos": "trasbocar",
    "trasbocáis": "trasbocar",
    "trasbocáramos": "trasbocar",
    "trasbocáremos": "trasbocar",
    "trasbocásemos": "trasbocar",
    "trasbocó": "trasbocar",
    "trasboque": "trasbocar",
    "trasboquemos": "trasbocar",
    "trasboquen": "trasbocar",
    "trasboques": "trasbocar",
    "trasboqué": "trasbocar",
    "trasboquéis": "trasbocar",
    "trasborda": "trasbordar",
    "trasbordaba": "trasbordar",
    "trasbordabais": "trasbordar",
    "trasbordaban": "trasbordar",
    "trasbordabas": "trasbordar",
    "trasbordad": "trasbordar",
    "trasbordada": "trasbordar",
    "trasbordadas": "trasbordar",
    "trasbordado": "trasbordar",
    "trasbordadora": "trasbordador",
    "trasbordadoras": "trasbordador",
    "trasbordadores": "trasbordador",
    "trasbordados": "trasbordar",
    "trasbordamos": "trasbordar",
    "trasbordan": "trasbordar",
    "trasbordando": "trasbordar",
    "trasbordara": "trasbordar",
    "trasbordarais": "trasbordar",
    "trasbordaran": "trasbordar",
    "trasbordaras": "trasbordar",
    "trasbordare": "trasbordar",
    "trasbordareis": "trasbordar",
    "trasbordaremos": "trasbordar",
    "trasbordaren": "trasbordar",
    "trasbordares": "trasbordar",
    "trasbordaron": "trasbordar",
    "trasbordará": "trasbordar",
    "trasbordarán": "trasbordar",
    "trasbordarás": "trasbordar",
    "trasbordaré": "trasbordar",
    "trasbordaréis": "trasbordar",
    "trasbordaría": "trasbordar",
    "trasbordaríais": "trasbordar",
    "trasbordaríamos": "trasbordar",
    "trasbordarían": "trasbordar",
    "trasbordarías": "trasbordar",
    "trasbordas": "trasbordar",
    "trasbordase": "trasbordar",
    "trasbordaseis": "trasbordar",
    "trasbordasen": "trasbordar",
    "trasbordases": "trasbordar",
    "trasbordaste": "trasbordar",
    "trasbordasteis": "trasbordar",
    "trasborde": "trasbordar",
    "trasbordemos": "trasbordar",
    "trasborden": "trasbordar",
    "trasbordes": "trasbordar",
    "trasbordo": "trasbordar",
    "trasbordos": "trasbordo",
    "trasbordábamos": "trasbordar",
    "trasbordáis": "trasbordar",
    "trasbordáramos": "trasbordar",
    "trasbordáremos": "trasbordar",
    "trasbordásemos": "trasbordar",
    "trasbordé": "trasbordar",
    "trasbordéis": "trasbordar",
    "trasbordó": "trasbordar",
    "trascachos": "trascacho",
    "trascantones": "trascantón",
    "trascas": "trasca",
    "trascendamos": "trascender",
    "trascended": "trascender",
    "trascendemos": "trascender",
    "trascendencias": "trascendencia",
    "trascendentales": "trascendental",
    "trascendentalismos": "trascendentalismo",
    "trascendentes": "trascendente",
    "trascenderemos": "trascender",
    "trascenderá": "trascender",
    "trascenderán": "trascender",
    "trascenderás": "trascender",
    "trascenderé": "trascender",
    "trascenderéis": "trascender",
    "trascendería": "trascender",
    "trascenderíais": "trascender",
    "trascenderíamos": "trascender",
    "trascenderían": "trascender",
    "trascenderías": "trascender",
    "trascendida": "trascender",
    "trascendidas": "trascender",
    "trascendido": "trascender",
    "trascendidos": "trascender",
    "trascendiendo": "trascender",
    "trascendiera": "trascender",
    "trascendierais": "trascender",
    "trascendieran": "trascender",
    "trascendieras": "trascender",
    "trascendiere": "trascender",
    "trascendiereis": "trascender",
    "trascendieren": "trascender",
    "trascendieres": "trascender",
    "trascendieron": "trascender",
    "trascendiese": "trascender",
    "trascendieseis": "trascender",
    "trascendiesen": "trascender",
    "trascendieses": "trascender",
    "trascendimos": "trascender",
    "trascendiste": "trascender",
    "trascendisteis": "trascender",
    "trascendiéramos": "trascender",
    "trascendiéremos": "trascender",
    "trascendiésemos": "trascender",
    "trascendió": "trascender",
    "trascendáis": "trascender",
    "trascendéis": "trascender",
    "trascendí": "trascender",
    "trascendía": "trascender",
    "trascendíais": "trascender",
    "trascendíamos": "trascender",
    "trascendían": "trascender",
    "trascendías": "trascender",
    "trascienda": "trascender",
    "trasciendan": "trascender",
    "trasciendas": "trascender",
    "trasciende": "trascender",
    "trascienden": "trascender",
    "trasciendes": "trascender",
    "trasciendo": "trascender",
    "trascocinas": "trascocina",
    "trascodas": "trascoda",
    "trascolaba": "trascolar",
    "trascolabais": "trascolar",
    "trascolaban": "trascolar",
    "trascolabas": "trascolar",
    "trascolad": "trascolar",
    "trascolada": "trascolar",
    "trascoladas": "trascolar",
    "trascolado": "trascolar",
    "trascolados": "trascolar",
    "trascolamos": "trascolar",
    "trascolando": "trascolar",
    "trascolara": "trascolar",
    "trascolarais": "trascolar",
    "trascolaran": "trascolar",
    "trascolaras": "trascolar",
    "trascolare": "trascolar",
    "trascolareis": "trascolar",
    "trascolaremos": "trascolar",
    "trascolaren": "trascolar",
    "trascolares": "trascolar",
    "trascolaron": "trascolar",
    "trascolará": "trascolar",
    "trascolarán": "trascolar",
    "trascolarás": "trascolar",
    "trascolaré": "trascolar",
    "trascolaréis": "trascolar",
    "trascolaría": "trascolar",
    "trascolaríais": "trascolar",
    "trascolaríamos": "trascolar",
    "trascolarían": "trascolar",
    "trascolarías": "trascolar",
    "trascolase": "trascolar",
    "trascolaseis": "trascolar",
    "trascolasen": "trascolar",
    "trascolases": "trascolar",
    "trascolaste": "trascolar",
    "trascolasteis": "trascolar",
    "trascolemos": "trascolar",
    "trascolábamos": "trascolar",
    "trascoláis": "trascolar",
    "trascoláramos": "trascolar",
    "trascoláremos": "trascolar",
    "trascolásemos": "trascolar",
    "trascolé": "trascolar",
    "trascoléis": "trascolar",
    "trascoló": "trascolar",
    "trasconeja": "trasconejar",
    "trasconejaba": "trasconejar",
    "trasconejabais": "trasconejar",
    "trasconejaban": "trasconejar",
    "trasconejabas": "trasconejar",
    "trasconejad": "trasconejar",
    "trasconejada": "trasconejar",
    "trasconejadas": "trasconejar",
    "trasconejado": "trasconejar",
    "trasconejados": "trasconejar",
    "trasconejamos": "trasconejar",
    "trasconejan": "trasconejar",
    "trasconejando": "trasconejar",
    "trasconejara": "trasconejar",
    "trasconejarais": "trasconejar",
    "trasconejaran": "trasconejar",
    "trasconejaras": "trasconejar",
    "trasconejare": "trasconejar",
    "trasconejareis": "trasconejar",
    "trasconejaremos": "trasconejar",
    "trasconejaren": "trasconejar",
    "trasconejares": "trasconejar",
    "trasconejaron": "trasconejar",
    "trasconejará": "trasconejar",
    "trasconejarán": "trasconejar",
    "trasconejarás": "trasconejar",
    "trasconejaré": "trasconejar",
    "trasconejaréis": "trasconejar",
    "trasconejaría": "trasconejar",
    "trasconejaríais": "trasconejar",
    "trasconejaríamos": "trasconejar",
    "trasconejarían": "trasconejar",
    "trasconejarías": "trasconejar",
    "trasconejas": "trasconejar",
    "trasconejase": "trasconejar",
    "trasconejaseis": "trasconejar",
    "trasconejasen": "trasconejar",
    "trasconejases": "trasconejar",
    "trasconejaste": "trasconejar",
    "trasconejasteis": "trasconejar",
    "trasconeje": "trasconejar",
    "trasconejemos": "trasconejar",
    "trasconejen": "trasconejar",
    "trasconejes": "trasconejar",
    "trasconejo": "trasconejar",
    "trasconejábamos": "trasconejar",
    "trasconejáis": "trasconejar",
    "trasconejáramos": "trasconejar",
    "trasconejáremos": "trasconejar",
    "trasconejásemos": "trasconejar",
    "trasconejé": "trasconejar",
    "trasconejéis": "trasconejar",
    "trasconejó": "trasconejar",
    "trascontinentales": "trascontinental",
    "trascordaba": "trascordar",
    "trascordabais": "trascordar",
    "trascordaban": "trascordar",
    "trascordabas": "trascordar",
    "trascordad": "trascordar",
    "trascordada": "trascordar",
    "trascordadas": "trascordar",
    "trascordado": "trascordar",
    "trascordados": "trascordar",
    "trascordamos": "trascordar",
    "trascordando": "trascordar",
    "trascordara": "trascordar",
    "trascordarais": "trascordar",
    "trascordaran": "trascordar",
    "trascordaras": "trascordar",
    "trascordare": "trascordar",
    "trascordareis": "trascordar",
    "trascordaremos": "trascordar",
    "trascordaren": "trascordar",
    "trascordares": "trascordar",
    "trascordaron": "trascordar",
    "trascordará": "trascordar",
    "trascordarán": "trascordar",
    "trascordarás": "trascordar",
    "trascordaré": "trascordar",
    "trascordaréis": "trascordar",
    "trascordaría": "trascordar",
    "trascordaríais": "trascordar",
    "trascordaríamos": "trascordar",
    "trascordarían": "trascordar",
    "trascordarías": "trascordar",
    "trascordase": "trascordar",
    "trascordaseis": "trascordar",
    "trascordasen": "trascordar",
    "trascordases": "trascordar",
    "trascordaste": "trascordar",
    "trascordasteis": "trascordar",
    "trascordemos": "trascordar",
    "trascordábamos": "trascordar",
    "trascordáis": "trascordar",
    "trascordáramos": "trascordar",
    "trascordáremos": "trascordar",
    "trascordásemos": "trascordar",
    "trascordé": "trascordar",
    "trascordéis": "trascordar",
    "trascordó": "trascordar",
    "trascoros": "trascoro",
    "trascorrales": "trascorral",
    "trascorva": "trascorvo",
    "trascorvas": "trascorvo",
    "trascorvos": "trascorvo",
    "trascriba": "trascribir",
    "trascribamos": "trascribir",
    "trascriban": "trascribir",
    "trascribas": "trascribir",
    "trascribe": "trascribir",
    "trascriben": "trascribir",
    "trascribes": "trascribir",
    "trascribid": "trascribir",
    "trascribiendo": "trascribir",
    "trascribiera": "trascribir",
    "trascribierais": "trascribir",
    "trascribieran": "trascribir",
    "trascribieras": "trascribir",
    "trascribiere": "trascribir",
    "trascribiereis": "trascribir",
    "trascribieren": "trascribir",
    "trascribieres": "trascribir",
    "trascribieron": "trascribir",
    "trascribiese": "trascribir",
    "trascribieseis": "trascribir",
    "trascribiesen": "trascribir",
    "trascribieses": "trascribir",
    "trascribimos": "trascribir",
    "trascribiremos": "trascribir",
    "trascribirá": "trascribir",
    "trascribirán": "trascribir",
    "trascribirás": "trascribir",
    "trascribiré": "trascribir",
    "trascribiréis": "trascribir",
    "trascribiría": "trascribir",
    "trascribiríais": "trascribir",
    "trascribiríamos": "trascribir",
    "trascribirían": "trascribir",
    "trascribirías": "trascribir",
    "trascribiste": "trascribir",
    "trascribisteis": "trascribir",
    "trascribiéramos": "trascribir",
    "trascribiéremos": "trascribir",
    "trascribiésemos": "trascribir",
    "trascribió": "trascribir",
    "trascribo": "trascribir",
    "trascribáis": "trascribir",
    "trascribí": "trascribir",
    "trascribía": "trascribir",
    "trascribíais": "trascribir",
    "trascribíamos": "trascribir",
    "trascribían": "trascribir",
    "trascribías": "trascribir",
    "trascribís": "trascribir",
    "trascripciones": "trascripción",
    "trascrita": "trascribir",
    "trascritas": "trascribir",
    "trascrito": "trascribir",
    "trascritos": "trascribir",
    "trascuartos": "trascuarto",
    "trascuela": "trascolar",
    "trascuelan": "trascolar",
    "trascuelas": "trascolar",
    "trascuele": "trascolar",
    "trascuelen": "trascolar",
    "trascueles": "trascolar",
    "trascuelo": "trascolar",
    "trascuentas": "trascuenta",
    "trascuerda": "trascordar",
    "trascuerdan": "trascordar",
    "trascuerdas": "trascordar",
    "trascuerde": "trascordar",
    "trascuerden": "trascordar",
    "trascuerdes": "trascordar",
    "trascuerdo": "trascordar",
    "trascurra": "trascurrir",
    "trascurramos": "trascurrir",
    "trascurran": "trascurrir",
    "trascurras": "trascurrir",
    "trascurre": "trascurrir",
    "trascurren": "trascurrir",
    "trascurres": "trascurrir",
    "trascurrid": "trascurrir",
    "trascurrida": "trascurrir",
    "trascurridas": "trascurrir",
    "trascurrido": "trascurrir",
    "trascurridos": "trascurrir",
    "trascurriendo": "trascurrir",
    "trascurriera": "trascurrir",
    "trascurrierais": "trascurrir",
    "trascurrieran": "trascurrir",
    "trascurrieras": "trascurrir",
    "trascurriere": "trascurrir",
    "trascurriereis": "trascurrir",
    "trascurrieren": "trascurrir",
    "trascurrieres": "trascurrir",
    "trascurrieron": "trascurrir",
    "trascurriese": "trascurrir",
    "trascurrieseis": "trascurrir",
    "trascurriesen": "trascurrir",
    "trascurrieses": "trascurrir",
    "trascurrimos": "trascurrir",
    "trascurriremos": "trascurrir",
    "trascurrirá": "trascurrir",
    "trascurrirán": "trascurrir",
    "trascurrirás": "trascurrir",
    "trascurriré": "trascurrir",
    "trascurriréis": "trascurrir",
    "trascurriría": "trascurrir",
    "trascurriríais": "trascurrir",
    "trascurriríamos": "trascurrir",
    "trascurrirían": "trascurrir",
    "trascurrirías": "trascurrir",
    "trascurriste": "trascurrir",
    "trascurristeis": "trascurrir",
    "trascurriéramos": "trascurrir",
    "trascurriéremos": "trascurrir",
    "trascurriésemos": "trascurrir",
    "trascurrió": "trascurrir",
    "trascurro": "trascurrir",
    "trascurráis": "trascurrir",
    "trascurrí": "trascurrir",
    "trascurría": "trascurrir",
    "trascurríais": "trascurrir",
    "trascurríamos": "trascurrir",
    "trascurrían": "trascurrir",
    "trascurrías": "trascurrir",
    "trascurrís": "trascurrir",
    "trascursos": "trascurso",
    "trasdosados": "trasdosado",
    "trasdosea": "trasdosear",
    "trasdoseaba": "trasdosear",
    "trasdoseabais": "trasdosear",
    "trasdoseaban": "trasdosear",
    "trasdoseabas": "trasdosear",
    "trasdosead": "trasdosear",
    "trasdoseada": "trasdosear",
    "trasdoseadas": "trasdosear",
    "trasdoseado": "trasdosear",
    "trasdoseados": "trasdosear",
    "trasdoseamos": "trasdosear",
    "trasdosean": "trasdosear",
    "trasdoseando": "trasdosear",
    "trasdoseara": "trasdosear",
    "trasdosearais": "trasdosear",
    "trasdosearan": "trasdosear",
    "trasdosearas": "trasdosear",
    "trasdoseare": "trasdosear",
    "trasdoseareis": "trasdosear",
    "trasdosearemos": "trasdosear",
    "trasdosearen": "trasdosear",
    "trasdoseares": "trasdosear",
    "trasdosearon": "trasdosear",
    "trasdoseará": "trasdosear",
    "trasdosearán": "trasdosear",
    "trasdosearás": "trasdosear",
    "trasdosearé": "trasdosear",
    "trasdosearéis": "trasdosear",
    "trasdosearía": "trasdosear",
    "trasdosearíais": "trasdosear",
    "trasdosearíamos": "trasdosear",
    "trasdosearían": "trasdosear",
    "trasdosearías": "trasdosear",
    "trasdoseas": "trasdosear",
    "trasdosease": "trasdosear",
    "trasdoseaseis": "trasdosear",
    "trasdoseasen": "trasdosear",
    "trasdoseases": "trasdosear",
    "trasdoseaste": "trasdosear",
    "trasdoseasteis": "trasdosear",
    "trasdosee": "trasdosear",
    "trasdoseemos": "trasdosear",
    "trasdoseen": "trasdosear",
    "trasdosees": "trasdosear",
    "trasdoseo": "trasdosear",
    "trasdoses": "trasdós",
    "trasdoseábamos": "trasdosear",
    "trasdoseáis": "trasdosear",
    "trasdoseáramos": "trasdosear",
    "trasdoseáremos": "trasdosear",
    "trasdoseásemos": "trasdosear",
    "trasdoseé": "trasdosear",
    "trasdoseéis": "trasdosear",
    "trasdoseó": "trasdosear",
    "trasegaba": "trasegar",
    "trasegabais": "trasegar",
    "trasegaban": "trasegar",
    "trasegabas": "trasegar",
    "trasegaciones": "trasegación",
    "trasegad": "trasegar",
    "trasegada": "trasegar",
    "trasegadas": "trasegar",
    "trasegado": "trasegar",
    "trasegadora": "trasegador",
    "trasegadoras": "trasegador",
    "trasegadores": "trasegador",
    "trasegados": "trasegar",
    "trasegamos": "trasegar",
    "trasegando": "trasegar",
    "trasegara": "trasegar",
    "trasegarais": "trasegar",
    "trasegaran": "trasegar",
    "trasegaras": "trasegar",
    "trasegare": "trasegar",
    "trasegareis": "trasegar",
    "trasegaremos": "trasegar",
    "trasegaren": "trasegar",
    "trasegares": "trasegar",
    "trasegaron": "trasegar",
    "trasegará": "trasegar",
    "trasegarán": "trasegar",
    "trasegarás": "trasegar",
    "trasegaré": "trasegar",
    "trasegaréis": "trasegar",
    "trasegaría": "trasegar",
    "trasegaríais": "trasegar",
    "trasegaríamos": "trasegar",
    "trasegarían": "trasegar",
    "trasegarías": "trasegar",
    "trasegase": "trasegar",
    "trasegaseis": "trasegar",
    "trasegasen": "trasegar",
    "trasegases": "trasegar",
    "trasegaste": "trasegar",
    "trasegasteis": "trasegar",
    "traseguemos": "trasegar",
    "trasegué": "trasegar",
    "traseguéis": "trasegar",
    "trasegábamos": "trasegar",
    "trasegáis": "trasegar",
    "trasegáramos": "trasegar",
    "trasegáremos": "trasegar",
    "trasegásemos": "trasegar",
    "trasegó": "trasegar",
    "trasera": "trasero",
    "traseras": "trasero",
    "traseros": "trasero",
    "traseñaladora": "traseñalador",
    "traseñaladoras": "traseñalador",
    "traseñaladores": "traseñalador",
    "trasferencias": "trasferencia",
    "trasferibles": "trasferible",
    "trasferid": "trasferir",
    "trasferida": "trasferir",
    "trasferidas": "trasferir",
    "trasferido": "trasferir",
    "trasferidora": "trasferidor",
    "trasferidoras": "trasferidor",
    "trasferidores": "trasferidor",
    "trasferidos": "trasferir",
    "trasferimos": "trasferir",
    "trasferiremos": "trasferir",
    "trasferirá": "trasferir",
    "trasferirán": "trasferir",
    "trasferirás": "trasferir",
    "trasferiré": "trasferir",
    "trasferiréis": "trasferir",
    "trasferiría": "trasferir",
    "trasferiríais": "trasferir",
    "trasferiríamos": "trasferir",
    "trasferirían": "trasferir",
    "trasferirías": "trasferir",
    "trasferiste": "trasferir",
    "trasferisteis": "trasferir",
    "trasferí": "trasferir",
    "trasfería": "trasferir",
    "trasferíais": "trasferir",
    "trasferíamos": "trasferir",
    "trasferían": "trasferir",
    "trasferías": "trasferir",
    "trasferís": "trasferir",
    "trasfiera": "trasferir",
    "trasfieran": "trasferir",
    "trasfieras": "trasferir",
    "trasfiere": "trasferir",
    "trasfieren": "trasferir",
    "trasfieres": "trasferir",
    "trasfiero": "trasferir",
    "trasfigura": "trasfigurar",
    "trasfiguraba": "trasfigurar",
    "trasfigurabais": "trasfigurar",
    "trasfiguraban": "trasfigurar",
    "trasfigurabas": "trasfigurar",
    "trasfigurables": "trasfigurable",
    "trasfiguraciones": "trasfiguración",
    "trasfigurad": "trasfigurar",
    "trasfigurada": "trasfigurar",
    "trasfiguradas": "trasfigurar",
    "trasfigurado": "trasfigurar",
    "trasfigurados": "trasfigurar",
    "trasfiguramos": "trasfigurar",
    "trasfiguran": "trasfigurar",
    "trasfigurando": "trasfigurar",
    "trasfigurara": "trasfigurar",
    "trasfigurarais": "trasfigurar",
    "trasfiguraran": "trasfigurar",
    "trasfiguraras": "trasfigurar",
    "trasfigurare": "trasfigurar",
    "trasfigurareis": "trasfigurar",
    "trasfiguraremos": "trasfigurar",
    "trasfiguraren": "trasfigurar",
    "trasfigurares": "trasfigurar",
    "trasfiguraron": "trasfigurar",
    "trasfigurará": "trasfigurar",
    "trasfigurarán": "trasfigurar",
    "trasfigurarás": "trasfigurar",
    "trasfiguraré": "trasfigurar",
    "trasfiguraréis": "trasfigurar",
    "trasfiguraría": "trasfigurar",
    "trasfiguraríais": "trasfigurar",
    "trasfiguraríamos": "trasfigurar",
    "trasfigurarían": "trasfigurar",
    "trasfigurarías": "trasfigurar",
    "trasfiguras": "trasfigurar",
    "trasfigurase": "trasfigurar",
    "trasfiguraseis": "trasfigurar",
    "trasfigurasen": "trasfigurar",
    "trasfigurases": "trasfigurar",
    "trasfiguraste": "trasfigurar",
    "trasfigurasteis": "trasfigurar",
    "trasfigure": "trasfigurar",
    "trasfiguremos": "trasfigurar",
    "trasfiguren": "trasfigurar",
    "trasfigures": "trasfigurar",
    "trasfiguro": "trasfigurar",
    "trasfigurábamos": "trasfigurar",
    "trasfiguráis": "trasfigurar",
    "trasfiguráramos": "trasfigurar",
    "trasfiguráremos": "trasfigurar",
    "trasfigurásemos": "trasfigurar",
    "trasfiguré": "trasfigurar",
    "trasfiguréis": "trasfigurar",
    "trasfiguró": "trasfigurar",
    "trasfija": "trasfijo",
    "trasfijas": "trasfijo",
    "trasfijos": "trasfijo",
    "trasfiramos": "trasferir",
    "trasfiriendo": "trasferir",
    "trasfiriera": "trasferir",
    "trasfirierais": "trasferir",
    "trasfirieran": "trasferir",
    "trasfirieras": "trasferir",
    "trasfiriere": "trasferir",
    "trasfiriereis": "trasferir",
    "trasfirieren": "trasferir",
    "trasfirieres": "trasferir",
    "trasfirieron": "trasferir",
    "trasfiriese": "trasferir",
    "trasfirieseis": "trasferir",
    "trasfiriesen": "trasferir",
    "trasfirieses": "trasferir",
    "trasfiriéramos": "trasferir",
    "trasfiriéremos": "trasferir",
    "trasfiriésemos": "trasferir",
    "trasfirió": "trasferir",
    "trasfiráis": "trasferir",
    "trasfixiones": "trasfixión",
    "trasflores": "trasflor",
    "trasfollada": "trasfollado",
    "trasfolladas": "trasfollado",
    "trasfollados": "trasfollado",
    "trasfollos": "trasfollo",
    "trasfondos": "trasfondo",
    "trasforma": "trasformar",
    "trasformaba": "trasformar",
    "trasformabais": "trasformar",
    "trasformaban": "trasformar",
    "trasformabas": "trasformar",
    "trasformaciones": "trasformación",
    "trasformad": "trasformar",
    "trasformada": "trasformar",
    "trasformadas": "trasformar",
    "trasformado": "trasformar",
    "trasformadora": "trasformador",
    "trasformadoras": "trasformador",
    "trasformadores": "trasformador",
    "trasformados": "trasformar",
    "trasformamientos": "trasformamiento",
    "trasformamos": "trasformar",
    "trasforman": "trasformar",
    "trasformando": "trasformar",
    "trasformara": "trasformar",
    "trasformarais": "trasformar",
    "trasformaran": "trasformar",
    "trasformaras": "trasformar",
    "trasformare": "trasformar",
    "trasformareis": "trasformar",
    "trasformaremos": "trasformar",
    "trasformaren": "trasformar",
    "trasformares": "trasformar",
    "trasformaron": "trasformar",
    "trasformará": "trasformar",
    "trasformarán": "trasformar",
    "trasformarás": "trasformar",
    "trasformaré": "trasformar",
    "trasformaréis": "trasformar",
    "trasformaría": "trasformar",
    "trasformaríais": "trasformar",
    "trasformaríamos": "trasformar",
    "trasformarían": "trasformar",
    "trasformarías": "trasformar",
    "trasformas": "trasformar",
    "trasformase": "trasformar",
    "trasformaseis": "trasformar",
    "trasformasen": "trasformar",
    "trasformases": "trasformar",
    "trasformaste": "trasformar",
    "trasformasteis": "trasformar",
    "trasformativa": "trasformativo",
    "trasformativas": "trasformativo",
    "trasformativos": "trasformativo",
    "trasforme": "trasformar",
    "trasformemos": "trasformar",
    "trasformen": "trasformar",
    "trasformes": "trasformar",
    "trasformo": "trasformar",
    "trasformábamos": "trasformar",
    "trasformáis": "trasformar",
    "trasformáramos": "trasformar",
    "trasformáremos": "trasformar",
    "trasformásemos": "trasformar",
    "trasformé": "trasformar",
    "trasforméis": "trasformar",
    "trasformó": "trasformar",
    "trasfretana": "trasfretano",
    "trasfretanas": "trasfretano",
    "trasfretanos": "trasfretano",
    "trasfunda": "trasfundir",
    "trasfundamos": "trasfundir",
    "trasfundan": "trasfundir",
    "trasfundas": "trasfundir",
    "trasfunde": "trasfundir",
    "trasfunden": "trasfundir",
    "trasfundes": "trasfundir",
    "trasfundiciones": "trasfundición",
    "trasfundid": "trasfundir",
    "trasfundida": "trasfundir",
    "trasfundidas": "trasfundir",
    "trasfundido": "trasfundir",
    "trasfundidos": "trasfundir",
    "trasfundiendo": "trasfundir",
    "trasfundiera": "trasfundir",
    "trasfundierais": "trasfundir",
    "trasfundieran": "trasfundir",
    "trasfundieras": "trasfundir",
    "trasfundiere": "trasfundir",
    "trasfundiereis": "trasfundir",
    "trasfundieren": "trasfundir",
    "trasfundieres": "trasfundir",
    "trasfundieron": "trasfundir",
    "trasfundiese": "trasfundir",
    "trasfundieseis": "trasfundir",
    "trasfundiesen": "trasfundir",
    "trasfundieses": "trasfundir",
    "trasfundimos": "trasfundir",
    "trasfundiremos": "trasfundir",
    "trasfundirá": "trasfundir",
    "trasfundirán": "trasfundir",
    "trasfundirás": "trasfundir",
    "trasfundiré": "trasfundir",
    "trasfundiréis": "trasfundir",
    "trasfundiría": "trasfundir",
    "trasfundiríais": "trasfundir",
    "trasfundiríamos": "trasfundir",
    "trasfundirían": "trasfundir",
    "trasfundirías": "trasfundir",
    "trasfundiste": "trasfundir",
    "trasfundisteis": "trasfundir",
    "trasfundiéramos": "trasfundir",
    "trasfundiéremos": "trasfundir",
    "trasfundiésemos": "trasfundir",
    "trasfundió": "trasfundir",
    "trasfundo": "trasfundir",
    "trasfundáis": "trasfundir",
    "trasfundí": "trasfundir",
    "trasfundía": "trasfundir",
    "trasfundíais": "trasfundir",
    "trasfundíamos": "trasfundir",
    "trasfundían": "trasfundir",
    "trasfundías": "trasfundir",
    "trasfundís": "trasfundir",
    "trasfusiones": "trasfusión",
    "trasfusora": "trasfusor",
    "trasfusoras": "trasfusor",
    "trasfusores": "trasfusor",
    "trasgos": "trasgo",
    "trasgreda": "trasgredir",
    "trasgredamos": "trasgredir",
    "trasgredan": "trasgredir",
    "trasgredas": "trasgredir",
    "trasgrede": "trasgredir",
    "trasgreden": "trasgredir",
    "trasgredes": "trasgredir",
    "trasgredid": "trasgredir",
    "trasgredida": "trasgredir",
    "trasgredidas": "trasgredir",
    "trasgredido": "trasgredir",
    "trasgredidos": "trasgredir",
    "trasgrediendo": "trasgredir",
    "trasgrediera": "trasgredir",
    "trasgredierais": "trasgredir",
    "trasgredieran": "trasgredir",
    "trasgredieras": "trasgredir",
    "trasgrediere": "trasgredir",
    "trasgrediereis": "trasgredir",
    "trasgredieren": "trasgredir",
    "trasgredieres": "trasgredir",
    "trasgredieron": "trasgredir",
    "trasgrediese": "trasgredir",
    "trasgredieseis": "trasgredir",
    "trasgrediesen": "trasgredir",
    "trasgredieses": "trasgredir",
    "trasgredimos": "trasgredir",
    "trasgrediremos": "trasgredir",
    "trasgredirá": "trasgredir",
    "trasgredirán": "trasgredir",
    "trasgredirás": "trasgredir",
    "trasgrediré": "trasgredir",
    "trasgrediréis": "trasgredir",
    "trasgrediría": "trasgredir",
    "trasgrediríais": "trasgredir",
    "trasgrediríamos": "trasgredir",
    "trasgredirían": "trasgredir",
    "trasgredirías": "trasgredir",
    "trasgrediste": "trasgredir",
    "trasgredisteis": "trasgredir",
    "trasgrediéramos": "trasgredir",
    "trasgrediéremos": "trasgredir",
    "trasgrediésemos": "trasgredir",
    "trasgredió": "trasgredir",
    "trasgredo": "trasgredir",
    "trasgredáis": "trasgredir",
    "trasgredí": "trasgredir",
    "trasgredía": "trasgredir",
    "trasgredíais": "trasgredir",
    "trasgredíamos": "trasgredir",
    "trasgredían": "trasgredir",
    "trasgredías": "trasgredir",
    "trasgredís": "trasgredir",
    "trasgresiones": "trasgresión",
    "trasgresora": "trasgresor",
    "trasgresoras": "trasgresor",
    "trasgresores": "trasgresor",
    "trasguea": "trasguear",
    "trasgueaba": "trasguear",
    "trasgueabais": "trasguear",
    "trasgueaban": "trasguear",
    "trasgueabas": "trasguear",
    "trasguead": "trasguear",
    "trasgueada": "trasguear",
    "trasgueadas": "trasguear",
    "trasgueado": "trasguear",
    "trasgueados": "trasguear",
    "trasgueamos": "trasguear",
    "trasguean": "trasguear",
    "trasgueando": "trasguear",
    "trasgueara": "trasguear",
    "trasguearais": "trasguear",
    "trasguearan": "trasguear",
    "trasguearas": "trasguear",
    "trasgueare": "trasguear",
    "trasgueareis": "trasguear",
    "trasguearemos": "trasguear",
    "trasguearen": "trasguear",
    "trasgueares": "trasguear",
    "trasguearon": "trasguear",
    "trasgueará": "trasguear",
    "trasguearán": "trasguear",
    "trasguearás": "trasguear",
    "trasguearé": "trasguear",
    "trasguearéis": "trasguear",
    "trasguearía": "trasguear",
    "trasguearíais": "trasguear",
    "trasguearíamos": "trasguear",
    "trasguearían": "trasguear",
    "trasguearías": "trasguear",
    "trasgueas": "trasguear",
    "trasguease": "trasguear",
    "trasgueaseis": "trasguear",
    "trasgueasen": "trasguear",
    "trasgueases": "trasguear",
    "trasgueaste": "trasguear",
    "trasgueasteis": "trasguear",
    "trasguee": "trasguear",
    "trasgueemos": "trasguear",
    "trasgueen": "trasguear",
    "trasguees": "trasguear",
    "trasgueo": "trasguear",
    "trasguera": "trasguero",
    "trasgueras": "trasguero",
    "trasgueros": "trasguero",
    "trasgueábamos": "trasguear",
    "trasgueáis": "trasguear",
    "trasgueáramos": "trasguear",
    "trasgueáremos": "trasguear",
    "trasgueásemos": "trasguear",
    "trasgueé": "trasguear",
    "trasgueéis": "trasguear",
    "trasgueó": "trasguear",
    "trashoguera": "trashoguero",
    "trashogueras": "trashoguero",
    "trashogueros": "trashoguero",
    "trashoja": "trashojar",
    "trashojaba": "trashojar",
    "trashojabais": "trashojar",
    "trashojaban": "trashojar",
    "trashojabas": "trashojar",
    "trashojad": "trashojar",
    "trashojada": "trashojar",
    "trashojadas": "trashojar",
    "trashojado": "trashojar",
    "trashojados": "trashojar",
    "trashojamos": "trashojar",
    "trashojan": "trashojar",
    "trashojando": "trashojar",
    "trashojara": "trashojar",
    "trashojarais": "trashojar",
    "trashojaran": "trashojar",
    "trashojaras": "trashojar",
    "trashojare": "trashojar",
    "trashojareis": "trashojar",
    "trashojaremos": "trashojar",
    "trashojaren": "trashojar",
    "trashojares": "trashojar",
    "trashojaron": "trashojar",
    "trashojará": "trashojar",
    "trashojarán": "trashojar",
    "trashojarás": "trashojar",
    "trashojaré": "trashojar",
    "trashojaréis": "trashojar",
    "trashojaría": "trashojar",
    "trashojaríais": "trashojar",
    "trashojaríamos": "trashojar",
    "trashojarían": "trashojar",
    "trashojarías": "trashojar",
    "trashojas": "trashojar",
    "trashojase": "trashojar",
    "trashojaseis": "trashojar",
    "trashojasen": "trashojar",
    "trashojases": "trashojar",
    "trashojaste": "trashojar",
    "trashojasteis": "trashojar",
    "trashoje": "trashojar",
    "trashojemos": "trashojar",
    "trashojen": "trashojar",
    "trashojes": "trashojar",
    "trashojo": "trashojar",
    "trashojábamos": "trashojar",
    "trashojáis": "trashojar",
    "trashojáramos": "trashojar",
    "trashojáremos": "trashojar",
    "trashojásemos": "trashojar",
    "trashojé": "trashojar",
    "trashojéis": "trashojar",
    "trashojó": "trashojar",
    "trashuma": "trashumar",
    "trashumaba": "trashumar",
    "trashumabais": "trashumar",
    "trashumaban": "trashumar",
    "trashumabas": "trashumar",
    "trashumaciones": "trashumación",
    "trashumad": "trashumar",
    "trashumada": "trashumar",
    "trashumadas": "trashumar",
    "trashumado": "trashumar",
    "trashumados": "trashumar",
    "trashumamos": "trashumar",
    "trashuman": "trashumar",
    "trashumancias": "trashumancia",
    "trashumando": "trashumar",
    "trashumantes": "trashumante",
    "trashumara": "trashumar",
    "trashumarais": "trashumar",
    "trashumaran": "trashumar",
    "trashumaras": "trashumar",
    "trashumare": "trashumar",
    "trashumareis": "trashumar",
    "trashumaremos": "trashumar",
    "trashumaren": "trashumar",
    "trashumares": "trashumar",
    "trashumaron": "trashumar",
    "trashumará": "trashumar",
    "trashumarán": "trashumar",
    "trashumarás": "trashumar",
    "trashumaré": "trashumar",
    "trashumaréis": "trashumar",
    "trashumaría": "trashumar",
    "trashumaríais": "trashumar",
    "trashumaríamos": "trashumar",
    "trashumarían": "trashumar",
    "trashumarías": "trashumar",
    "trashumas": "trashumar",
    "trashumase": "trashumar",
    "trashumaseis": "trashumar",
    "trashumasen": "trashumar",
    "trashumases": "trashumar",
    "trashumaste": "trashumar",
    "trashumasteis": "trashumar",
    "trashume": "trashumar",
    "trashumemos": "trashumar",
    "trashumen": "trashumar",
    "trashumes": "trashumar",
    "trashumo": "trashumar",
    "trashumábamos": "trashumar",
    "trashumáis": "trashumar",
    "trashumáramos": "trashumar",
    "trashumáremos": "trashumar",
    "trashumásemos": "trashumar",
    "trashumé": "trashumar",
    "trashuméis": "trashumar",
    "trashumó": "trashumar",
    "trasiega": "trasegar",
    "trasiegan": "trasegar",
    "trasiegas": "trasegar",
    "trasiego": "trasegar",
    "trasiegos": "trasiego",
    "trasiegue": "trasegar",
    "trasieguen": "trasegar",
    "trasiegues": "trasegar",
    "trasijada": "trasijado",
    "trasijadas": "trasijado",
    "trasijados": "trasijado",
    "traslacionales": "traslacional",
    "traslaciones": "traslación",
    "traslada": "trasladar",
    "trasladaba": "trasladar",
    "trasladabais": "trasladar",
    "trasladaban": "trasladar",
    "trasladabas": "trasladar",
    "trasladables": "trasladable",
    "trasladad": "trasladar",
    "trasladada": "trasladar",
    "trasladadas": "trasladar",
    "trasladado": "trasladar",
    "trasladadora": "trasladador",
    "trasladadoras": "trasladador",
    "trasladadores": "trasladador",
    "trasladados": "trasladar",
    "trasladamos": "trasladar",
    "trasladan": "trasladar",
    "trasladando": "trasladar",
    "trasladara": "trasladar",
    "trasladarais": "trasladar",
    "trasladaran": "trasladar",
    "trasladaras": "trasladar",
    "trasladare": "trasladar",
    "trasladareis": "trasladar",
    "trasladaremos": "trasladar",
    "trasladaren": "trasladar",
    "trasladares": "trasladar",
    "trasladaron": "trasladar",
    "trasladará": "trasladar",
    "trasladarán": "trasladar",
    "trasladarás": "trasladar",
    "trasladaré": "trasladar",
    "trasladaréis": "trasladar",
    "trasladaría": "trasladar",
    "trasladaríais": "trasladar",
    "trasladaríamos": "trasladar",
    "trasladarían": "trasladar",
    "trasladarías": "trasladar",
    "trasladas": "trasladar",
    "trasladase": "trasladar",
    "trasladaseis": "trasladar",
    "trasladasen": "trasladar",
    "trasladases": "trasladar",
    "trasladaste": "trasladar",
    "trasladasteis": "trasladar",
    "traslade": "trasladar",
    "traslademos": "trasladar",
    "trasladen": "trasladar",
    "traslades": "trasladar",
    "traslado": "trasladar",
    "traslados": "traslado",
    "trasladábamos": "trasladar",
    "trasladáis": "trasladar",
    "trasladáramos": "trasladar",
    "trasladáremos": "trasladar",
    "trasladásemos": "trasladar",
    "trasladé": "trasladar",
    "trasladéis": "trasladar",
    "trasladó": "trasladar",
    "traslapa": "traslapar",
    "traslapaba": "traslapar",
    "traslapabais": "traslapar",
    "traslapaban": "traslapar",
    "traslapabas": "traslapar",
    "traslapad": "traslapar",
    "traslapada": "traslapar",
    "traslapadas": "traslapar",
    "traslapado": "traslapar",
    "traslapados": "traslapar",
    "traslapamos": "traslapar",
    "traslapan": "traslapar",
    "traslapando": "traslapar",
    "traslapara": "traslapar",
    "traslaparais": "traslapar",
    "traslaparan": "traslapar",
    "traslaparas": "traslapar",
    "traslapare": "traslapar",
    "traslapareis": "traslapar",
    "traslaparemos": "traslapar",
    "traslaparen": "traslapar",
    "traslapares": "traslapar",
    "traslaparon": "traslapar",
    "traslapará": "traslapar",
    "traslaparán": "traslapar",
    "traslaparás": "traslapar",
    "traslaparé": "traslapar",
    "traslaparéis": "traslapar",
    "traslaparía": "traslapar",
    "traslaparíais": "traslapar",
    "traslaparíamos": "traslapar",
    "traslaparían": "traslapar",
    "traslaparías": "traslapar",
    "traslapas": "traslapar",
    "traslapase": "traslapar",
    "traslapaseis": "traslapar",
    "traslapasen": "traslapar",
    "traslapases": "traslapar",
    "traslapaste": "traslapar",
    "traslapasteis": "traslapar",
    "traslape": "traslapar",
    "traslapemos": "traslapar",
    "traslapen": "traslapar",
    "traslapes": "traslapar",
    "traslapo": "traslapar",
    "traslapos": "traslapo",
    "traslapábamos": "traslapar",
    "traslapáis": "traslapar",
    "traslapáramos": "traslapar",
    "traslapáremos": "traslapar",
    "traslapásemos": "traslapar",
    "traslapé": "traslapar",
    "traslapéis": "traslapar",
    "traslapó": "traslapar",
    "traslata": "traslato",
    "traslatas": "traslato",
    "traslaticia": "traslaticio",
    "traslaticias": "traslaticio",
    "traslaticios": "traslaticio",
    "traslativa": "traslativo",
    "traslativas": "traslativo",
    "traslativos": "traslativo",
    "traslatos": "traslato",
    "trasluce": "traslucir",
    "traslucen": "traslucir",
    "trasluces": "trasluz",
    "trasluchadas": "trasluchada",
    "trasluches": "trasluche",
    "traslucid": "traslucir",
    "traslucida": "traslucir",
    "traslucidas": "traslucir",
    "traslucideces": "traslucidez",
    "traslucido": "traslucir",
    "traslucidos": "traslucir",
    "trasluciendo": "traslucir",
    "trasluciera": "traslucir",
    "traslucierais": "traslucir",
    "traslucieran": "traslucir",
    "traslucieras": "traslucir",
    "trasluciere": "traslucir",
    "trasluciereis": "traslucir",
    "traslucieren": "traslucir",
    "traslucieres": "traslucir",
    "traslucieron": "traslucir",
    "trasluciese": "traslucir",
    "traslucieseis": "traslucir",
    "trasluciesen": "traslucir",
    "traslucieses": "traslucir",
    "traslucimientos": "traslucimiento",
    "traslucimos": "traslucir",
    "trasluciremos": "traslucir",
    "traslucirá": "traslucir",
    "traslucirán": "traslucir",
    "traslucirás": "traslucir",
    "trasluciré": "traslucir",
    "trasluciréis": "traslucir",
    "trasluciría": "traslucir",
    "trasluciríais": "traslucir",
    "trasluciríamos": "traslucir",
    "traslucirían": "traslucir",
    "traslucirías": "traslucir",
    "trasluciste": "traslucir",
    "traslucisteis": "traslucir",
    "trasluciéramos": "traslucir",
    "trasluciéremos": "traslucir",
    "trasluciésemos": "traslucir",
    "traslució": "traslucir",
    "traslucí": "traslucir",
    "traslucía": "traslucir",
    "traslucíais": "traslucir",
    "traslucíamos": "traslucir",
    "traslucían": "traslucir",
    "traslucías": "traslucir",
    "traslucís": "traslucir",
    "traslumbra": "traslumbrar",
    "traslumbraba": "traslumbrar",
    "traslumbrabais": "traslumbrar",
    "traslumbraban": "traslumbrar",
    "traslumbrabas": "traslumbrar",
    "traslumbrad": "traslumbrar",
    "traslumbrada": "traslumbrar",
    "traslumbradas": "traslumbrar",
    "traslumbrado": "traslumbrar",
    "traslumbrados": "traslumbrar",
    "traslumbramientos": "traslumbramiento",
    "traslumbramos": "traslumbrar",
    "traslumbran": "traslumbrar",
    "traslumbrando": "traslumbrar",
    "traslumbrara": "traslumbrar",
    "traslumbrarais": "traslumbrar",
    "traslumbraran": "traslumbrar",
    "traslumbraras": "traslumbrar",
    "traslumbrare": "traslumbrar",
    "traslumbrareis": "traslumbrar",
    "traslumbraremos": "traslumbrar",
    "traslumbraren": "traslumbrar",
    "traslumbrares": "traslumbrar",
    "traslumbraron": "traslumbrar",
    "traslumbrará": "traslumbrar",
    "traslumbrarán": "traslumbrar",
    "traslumbrarás": "traslumbrar",
    "traslumbraré": "traslumbrar",
    "traslumbraréis": "traslumbrar",
    "traslumbraría": "traslumbrar",
    "traslumbraríais": "traslumbrar",
    "traslumbraríamos": "traslumbrar",
    "traslumbrarían": "traslumbrar",
    "traslumbrarías": "traslumbrar",
    "traslumbras": "traslumbrar",
    "traslumbrase": "traslumbrar",
    "traslumbraseis": "traslumbrar",
    "traslumbrasen": "traslumbrar",
    "traslumbrases": "traslumbrar",
    "traslumbraste": "traslumbrar",
    "traslumbrasteis": "traslumbrar",
    "traslumbre": "traslumbrar",
    "traslumbremos": "traslumbrar",
    "traslumbren": "traslumbrar",
    "traslumbres": "traslumbrar",
    "traslumbro": "traslumbrar",
    "traslumbrábamos": "traslumbrar",
    "traslumbráis": "traslumbrar",
    "traslumbráramos": "traslumbrar",
    "traslumbráremos": "traslumbrar",
    "traslumbrásemos": "traslumbrar",
    "traslumbré": "traslumbrar",
    "traslumbréis": "traslumbrar",
    "traslumbró": "traslumbrar",
    "trasluzca": "traslucir",
    "trasluzcamos": "traslucir",
    "trasluzcan": "traslucir",
    "trasluzcas": "traslucir",
    "trasluzco": "traslucir",
    "trasluzcáis": "traslucir",
    "traslúcida": "traslúcido",
    "traslúcidas": "traslúcido",
    "traslúcidos": "traslúcido",
    "trasmallos": "trasmallo",
    "trasmanos": "trasmano",
    "trasmarina": "trasmarino",
    "trasmarinas": "trasmarino",
    "trasmarinos": "trasmarino",
    "trasmediterránea": "trasmediterráneo",
    "trasmediterráneas": "trasmediterráneo",
    "trasmediterráneos": "trasmediterráneo",
    "trasmigra": "trasmigrar",
    "trasmigraba": "trasmigrar",
    "trasmigrabais": "trasmigrar",
    "trasmigraban": "trasmigrar",
    "trasmigrabas": "trasmigrar",
    "trasmigraciones": "trasmigración",
    "trasmigrad": "trasmigrar",
    "trasmigrada": "trasmigrar",
    "trasmigradas": "trasmigrar",
    "trasmigrado": "trasmigrar",
    "trasmigrados": "trasmigrar",
    "trasmigramos": "trasmigrar",
    "trasmigran": "trasmigrar",
    "trasmigrando": "trasmigrar",
    "trasmigrara": "trasmigrar",
    "trasmigrarais": "trasmigrar",
    "trasmigraran": "trasmigrar",
    "trasmigraras": "trasmigrar",
    "trasmigrare": "trasmigrar",
    "trasmigrareis": "trasmigrar",
    "trasmigraremos": "trasmigrar",
    "trasmigraren": "trasmigrar",
    "trasmigrares": "trasmigrar",
    "trasmigraron": "trasmigrar",
    "trasmigrará": "trasmigrar",
    "trasmigrarán": "trasmigrar",
    "trasmigrarás": "trasmigrar",
    "trasmigraré": "trasmigrar",
    "trasmigraréis": "trasmigrar",
    "trasmigraría": "trasmigrar",
    "trasmigraríais": "trasmigrar",
    "trasmigraríamos": "trasmigrar",
    "trasmigrarían": "trasmigrar",
    "trasmigrarías": "trasmigrar",
    "trasmigras": "trasmigrar",
    "trasmigrase": "trasmigrar",
    "trasmigraseis": "trasmigrar",
    "trasmigrasen": "trasmigrar",
    "trasmigrases": "trasmigrar",
    "trasmigraste": "trasmigrar",
    "trasmigrasteis": "trasmigrar",
    "trasmigre": "trasmigrar",
    "trasmigremos": "trasmigrar",
    "trasmigren": "trasmigrar",
    "trasmigres": "trasmigrar",
    "trasmigro": "trasmigrar",
    "trasmigrábamos": "trasmigrar",
    "trasmigráis": "trasmigrar",
    "trasmigráramos": "trasmigrar",
    "trasmigráremos": "trasmigrar",
    "trasmigrásemos": "trasmigrar",
    "trasmigré": "trasmigrar",
    "trasmigréis": "trasmigrar",
    "trasmigró": "trasmigrar",
    "trasmina": "trasminar",
    "trasminaba": "trasminar",
    "trasminabais": "trasminar",
    "trasminaban": "trasminar",
    "trasminabas": "trasminar",
    "trasminad": "trasminar",
    "trasminada": "trasminar",
    "trasminadas": "trasminar",
    "trasminado": "trasminar",
    "trasminados": "trasminar",
    "trasminamos": "trasminar",
    "trasminan": "trasminar",
    "trasminando": "trasminar",
    "trasminara": "trasminar",
    "trasminarais": "trasminar",
    "trasminaran": "trasminar",
    "trasminaras": "trasminar",
    "trasminare": "trasminar",
    "trasminareis": "trasminar",
    "trasminaremos": "trasminar",
    "trasminaren": "trasminar",
    "trasminares": "trasminar",
    "trasminaron": "trasminar",
    "trasminará": "trasminar",
    "trasminarán": "trasminar",
    "trasminarás": "trasminar",
    "trasminaré": "trasminar",
    "trasminaréis": "trasminar",
    "trasminaría": "trasminar",
    "trasminaríais": "trasminar",
    "trasminaríamos": "trasminar",
    "trasminarían": "trasminar",
    "trasminarías": "trasminar",
    "trasminas": "trasminar",
    "trasminase": "trasminar",
    "trasminaseis": "trasminar",
    "trasminasen": "trasminar",
    "trasminases": "trasminar",
    "trasminaste": "trasminar",
    "trasminasteis": "trasminar",
    "trasmine": "trasminar",
    "trasminemos": "trasminar",
    "trasminen": "trasminar",
    "trasmines": "trasminar",
    "trasmino": "trasminar",
    "trasminábamos": "trasminar",
    "trasmináis": "trasminar",
    "trasmináramos": "trasminar",
    "trasmináremos": "trasminar",
    "trasminásemos": "trasminar",
    "trasminé": "trasminar",
    "trasminéis": "trasminar",
    "trasminó": "trasminar",
    "trasmisibles": "trasmisible",
    "trasmisiones": "trasmisión",
    "trasmisora": "trasmisor",
    "trasmisoras": "trasmisor",
    "trasmisores": "trasmisor",
    "trasmita": "trasmitir",
    "trasmitamos": "trasmitir",
    "trasmitan": "trasmitir",
    "trasmitas": "trasmitir",
    "trasmite": "trasmitir",
    "trasmiten": "trasmitir",
    "trasmites": "trasmitir",
    "trasmitid": "trasmitir",
    "trasmitida": "trasmitir",
    "trasmitidas": "trasmitir",
    "trasmitido": "trasmitir",
    "trasmitidos": "trasmitir",
    "trasmitiendo": "trasmitir",
    "trasmitiera": "trasmitir",
    "trasmitierais": "trasmitir",
    "trasmitieran": "trasmitir",
    "trasmitieras": "trasmitir",
    "trasmitiere": "trasmitir",
    "trasmitiereis": "trasmitir",
    "trasmitieren": "trasmitir",
    "trasmitieres": "trasmitir",
    "trasmitieron": "trasmitir",
    "trasmitiese": "trasmitir",
    "trasmitieseis": "trasmitir",
    "trasmitiesen": "trasmitir",
    "trasmitieses": "trasmitir",
    "trasmitimos": "trasmitir",
    "trasmitiremos": "trasmitir",
    "trasmitirá": "trasmitir",
    "trasmitirán": "trasmitir",
    "trasmitirás": "trasmitir",
    "trasmitiré": "trasmitir",
    "trasmitiréis": "trasmitir",
    "trasmitiría": "trasmitir",
    "trasmitiríais": "trasmitir",
    "trasmitiríamos": "trasmitir",
    "trasmitirían": "trasmitir",
    "trasmitirías": "trasmitir",
    "trasmitiste": "trasmitir",
    "trasmitisteis": "trasmitir",
    "trasmitiéramos": "trasmitir",
    "trasmitiéremos": "trasmitir",
    "trasmitiésemos": "trasmitir",
    "trasmitió": "trasmitir",
    "trasmito": "trasmitir",
    "trasmitáis": "trasmitir",
    "trasmití": "trasmitir",
    "trasmitía": "trasmitir",
    "trasmitíais": "trasmitir",
    "trasmitíamos": "trasmitir",
    "trasmitían": "trasmitir",
    "trasmitías": "trasmitir",
    "trasmitís": "trasmitir",
    "trasmocha": "trasmocho",
    "trasmochas": "trasmocho",
    "trasmochos": "trasmocho",
    "trasmonta": "trasmontar",
    "trasmontaba": "trasmontar",
    "trasmontabais": "trasmontar",
    "trasmontaban": "trasmontar",
    "trasmontabas": "trasmontar",
    "trasmontad": "trasmontar",
    "trasmontada": "trasmontar",
    "trasmontadas": "trasmontar",
    "trasmontado": "trasmontar",
    "trasmontados": "trasmontar",
    "trasmontamos": "trasmontar",
    "trasmontan": "trasmontar",
    "trasmontana": "trasmontano",
    "trasmontanas": "trasmontano",
    "trasmontando": "trasmontar",
    "trasmontanos": "trasmontano",
    "trasmontara": "trasmontar",
    "trasmontarais": "trasmontar",
    "trasmontaran": "trasmontar",
    "trasmontaras": "trasmontar",
    "trasmontare": "trasmontar",
    "trasmontareis": "trasmontar",
    "trasmontaremos": "trasmontar",
    "trasmontaren": "trasmontar",
    "trasmontares": "trasmontar",
    "trasmontaron": "trasmontar",
    "trasmontará": "trasmontar",
    "trasmontarán": "trasmontar",
    "trasmontarás": "trasmontar",
    "trasmontaré": "trasmontar",
    "trasmontaréis": "trasmontar",
    "trasmontaría": "trasmontar",
    "trasmontaríais": "trasmontar",
    "trasmontaríamos": "trasmontar",
    "trasmontarían": "trasmontar",
    "trasmontarías": "trasmontar",
    "trasmontas": "trasmontar",
    "trasmontase": "trasmontar",
    "trasmontaseis": "trasmontar",
    "trasmontasen": "trasmontar",
    "trasmontases": "trasmontar",
    "trasmontaste": "trasmontar",
    "trasmontasteis": "trasmontar",
    "trasmonte": "trasmontar",
    "trasmontemos": "trasmontar",
    "trasmonten": "trasmontar",
    "trasmontes": "trasmontar",
    "trasmonto": "trasmontar",
    "trasmontábamos": "trasmontar",
    "trasmontáis": "trasmontar",
    "trasmontáramos": "trasmontar",
    "trasmontáremos": "trasmontar",
    "trasmontásemos": "trasmontar",
    "trasmonté": "trasmontar",
    "trasmontéis": "trasmontar",
    "trasmontó": "trasmontar",
    "trasmuda": "trasmudar",
    "trasmudaba": "trasmudar",
    "trasmudabais": "trasmudar",
    "trasmudaban": "trasmudar",
    "trasmudabas": "trasmudar",
    "trasmudaciones": "trasmudación",
    "trasmudad": "trasmudar",
    "trasmudada": "trasmudar",
    "trasmudadas": "trasmudar",
    "trasmudado": "trasmudar",
    "trasmudados": "trasmudar",
    "trasmudamientos": "trasmudamiento",
    "trasmudamos": "trasmudar",
    "trasmudan": "trasmudar",
    "trasmudando": "trasmudar",
    "trasmudara": "trasmudar",
    "trasmudarais": "trasmudar",
    "trasmudaran": "trasmudar",
    "trasmudaras": "trasmudar",
    "trasmudare": "trasmudar",
    "trasmudareis": "trasmudar",
    "trasmudaremos": "trasmudar",
    "trasmudaren": "trasmudar",
    "trasmudares": "trasmudar",
    "trasmudaron": "trasmudar",
    "trasmudará": "trasmudar",
    "trasmudarán": "trasmudar",
    "trasmudarás": "trasmudar",
    "trasmudaré": "trasmudar",
    "trasmudaréis": "trasmudar",
    "trasmudaría": "trasmudar",
    "trasmudaríais": "trasmudar",
    "trasmudaríamos": "trasmudar",
    "trasmudarían": "trasmudar",
    "trasmudarías": "trasmudar",
    "trasmudas": "trasmudar",
    "trasmudase": "trasmudar",
    "trasmudaseis": "trasmudar",
    "trasmudasen": "trasmudar",
    "trasmudases": "trasmudar",
    "trasmudaste": "trasmudar",
    "trasmudasteis": "trasmudar",
    "trasmude": "trasmudar",
    "trasmudemos": "trasmudar",
    "trasmuden": "trasmudar",
    "trasmudes": "trasmudar",
    "trasmudo": "trasmudar",
    "trasmudábamos": "trasmudar",
    "trasmudáis": "trasmudar",
    "trasmudáramos": "trasmudar",
    "trasmudáremos": "trasmudar",
    "trasmudásemos": "trasmudar",
    "trasmudé": "trasmudar",
    "trasmudéis": "trasmudar",
    "trasmudó": "trasmudar",
    "trasmuta": "trasmutar",
    "trasmutaba": "trasmutar",
    "trasmutabais": "trasmutar",
    "trasmutaban": "trasmutar",
    "trasmutabas": "trasmutar",
    "trasmutables": "trasmutable",
    "trasmutaciones": "trasmutación",
    "trasmutad": "trasmutar",
    "trasmutada": "trasmutar",
    "trasmutadas": "trasmutar",
    "trasmutado": "trasmutar",
    "trasmutados": "trasmutar",
    "trasmutamos": "trasmutar",
    "trasmutan": "trasmutar",
    "trasmutando": "trasmutar",
    "trasmutara": "trasmutar",
    "trasmutarais": "trasmutar",
    "trasmutaran": "trasmutar",
    "trasmutaras": "trasmutar",
    "trasmutare": "trasmutar",
    "trasmutareis": "trasmutar",
    "trasmutaremos": "trasmutar",
    "trasmutaren": "trasmutar",
    "trasmutares": "trasmutar",
    "trasmutaron": "trasmutar",
    "trasmutará": "trasmutar",
    "trasmutarán": "trasmutar",
    "trasmutarás": "trasmutar",
    "trasmutaré": "trasmutar",
    "trasmutaréis": "trasmutar",
    "trasmutaría": "trasmutar",
    "trasmutaríais": "trasmutar",
    "trasmutaríamos": "trasmutar",
    "trasmutarían": "trasmutar",
    "trasmutarías": "trasmutar",
    "trasmutas": "trasmutar",
    "trasmutase": "trasmutar",
    "trasmutaseis": "trasmutar",
    "trasmutasen": "trasmutar",
    "trasmutases": "trasmutar",
    "trasmutaste": "trasmutar",
    "trasmutasteis": "trasmutar",
    "trasmutativa": "trasmutativo",
    "trasmutativas": "trasmutativo",
    "trasmutativos": "trasmutativo",
    "trasmutatoria": "trasmutatorio",
    "trasmutatorias": "trasmutatorio",
    "trasmutatorios": "trasmutatorio",
    "trasmute": "trasmutar",
    "trasmutemos": "trasmutar",
    "trasmuten": "trasmutar",
    "trasmutes": "trasmutar",
    "trasmuto": "trasmutar",
    "trasmutábamos": "trasmutar",
    "trasmutáis": "trasmutar",
    "trasmutáramos": "trasmutar",
    "trasmutáremos": "trasmutar",
    "trasmutásemos": "trasmutar",
    "trasmuté": "trasmutar",
    "trasmutéis": "trasmutar",
    "trasmutó": "trasmutar",
    "trasnacionales": "trasnacional",
    "trasnocha": "trasnochar",
    "trasnochaba": "trasnochar",
    "trasnochabais": "trasnochar",
    "trasnochaban": "trasnochar",
    "trasnochabas": "trasnochar",
    "trasnochad": "trasnochar",
    "trasnochada": "trasnochar",
    "trasnochadas": "trasnochar",
    "trasnochado": "trasnochar",
    "trasnochadora": "trasnochador",
    "trasnochadoras": "trasnochador",
    "trasnochadores": "trasnochador",
    "trasnochados": "trasnochar",
    "trasnochamos": "trasnochar",
    "trasnochan": "trasnochar",
    "trasnochando": "trasnochar",
    "trasnochara": "trasnochar",
    "trasnocharais": "trasnochar",
    "trasnocharan": "trasnochar",
    "trasnocharas": "trasnochar",
    "trasnochare": "trasnochar",
    "trasnochareis": "trasnochar",
    "trasnocharemos": "trasnochar",
    "trasnocharen": "trasnochar",
    "trasnochares": "trasnochar",
    "trasnocharon": "trasnochar",
    "trasnochará": "trasnochar",
    "trasnocharán": "trasnochar",
    "trasnocharás": "trasnochar",
    "trasnocharé": "trasnochar",
    "trasnocharéis": "trasnochar",
    "trasnocharía": "trasnochar",
    "trasnocharíais": "trasnochar",
    "trasnocharíamos": "trasnochar",
    "trasnocharían": "trasnochar",
    "trasnocharías": "trasnochar",
    "trasnochas": "trasnochar",
    "trasnochase": "trasnochar",
    "trasnochaseis": "trasnochar",
    "trasnochasen": "trasnochar",
    "trasnochases": "trasnochar",
    "trasnochaste": "trasnochar",
    "trasnochasteis": "trasnochar",
    "trasnoche": "trasnochar",
    "trasnochemos": "trasnochar",
    "trasnochen": "trasnochar",
    "trasnoches": "trasnoche",
    "trasnocho": "trasnochar",
    "trasnochos": "trasnocho",
    "trasnochábamos": "trasnochar",
    "trasnocháis": "trasnochar",
    "trasnocháramos": "trasnochar",
    "trasnocháremos": "trasnochar",
    "trasnochásemos": "trasnochar",
    "trasnoché": "trasnochar",
    "trasnochéis": "trasnochar",
    "trasnochó": "trasnochar",
    "trasnombra": "trasnombrar",
    "trasnombraba": "trasnombrar",
    "trasnombrabais": "trasnombrar",
    "trasnombraban": "trasnombrar",
    "trasnombrabas": "trasnombrar",
    "trasnombrad": "trasnombrar",
    "trasnombrada": "trasnombrar",
    "trasnombradas": "trasnombrar",
    "trasnombrado": "trasnombrar",
    "trasnombrados": "trasnombrar",
    "trasnombramos": "trasnombrar",
    "trasnombran": "trasnombrar",
    "trasnombrando": "trasnombrar",
    "trasnombrara": "trasnombrar",
    "trasnombrarais": "trasnombrar",
    "trasnombraran": "trasnombrar",
    "trasnombraras": "trasnombrar",
    "trasnombrare": "trasnombrar",
    "trasnombrareis": "trasnombrar",
    "trasnombraremos": "trasnombrar",
    "trasnombraren": "trasnombrar",
    "trasnombrares": "trasnombrar",
    "trasnombraron": "trasnombrar",
    "trasnombrará": "trasnombrar",
    "trasnombrarán": "trasnombrar",
    "trasnombrarás": "trasnombrar",
    "trasnombraré": "trasnombrar",
    "trasnombraréis": "trasnombrar",
    "trasnombraría": "trasnombrar",
    "trasnombraríais": "trasnombrar",
    "trasnombraríamos": "trasnombrar",
    "trasnombrarían": "trasnombrar",
    "trasnombrarías": "trasnombrar",
    "trasnombras": "trasnombrar",
    "trasnombrase": "trasnombrar",
    "trasnombraseis": "trasnombrar",
    "trasnombrasen": "trasnombrar",
    "trasnombrases": "trasnombrar",
    "trasnombraste": "trasnombrar",
    "trasnombrasteis": "trasnombrar",
    "trasnombre": "trasnombrar",
    "trasnombremos": "trasnombrar",
    "trasnombren": "trasnombrar",
    "trasnombres": "trasnombrar",
    "trasnombro": "trasnombrar",
    "trasnombrábamos": "trasnombrar",
    "trasnombráis": "trasnombrar",
    "trasnombráramos": "trasnombrar",
    "trasnombráremos": "trasnombrar",
    "trasnombrásemos": "trasnombrar",
    "trasnombré": "trasnombrar",
    "trasnombréis": "trasnombrar",
    "trasnombró": "trasnombrar",
    "trasnominaciones": "trasnominación",
    "trasojada": "trasojado",
    "trasojadas": "trasojado",
    "trasojados": "trasojado",
    "trasoñaba": "trasoñar",
    "trasoñabais": "trasoñar",
    "trasoñaban": "trasoñar",
    "trasoñabas": "trasoñar",
    "trasoñad": "trasoñar",
    "trasoñada": "trasoñar",
    "trasoñadas": "trasoñar",
    "trasoñado": "trasoñar",
    "trasoñados": "trasoñar",
    "trasoñamos": "trasoñar",
    "trasoñando": "trasoñar",
    "trasoñara": "trasoñar",
    "trasoñarais": "trasoñar",
    "trasoñaran": "trasoñar",
    "trasoñaras": "trasoñar",
    "trasoñare": "trasoñar",
    "trasoñareis": "trasoñar",
    "trasoñaremos": "trasoñar",
    "trasoñaren": "trasoñar",
    "trasoñares": "trasoñar",
    "trasoñaron": "trasoñar",
    "trasoñará": "trasoñar",
    "trasoñarán": "trasoñar",
    "trasoñarás": "trasoñar",
    "trasoñaré": "trasoñar",
    "trasoñaréis": "trasoñar",
    "trasoñaría": "trasoñar",
    "trasoñaríais": "trasoñar",
    "trasoñaríamos": "trasoñar",
    "trasoñarían": "trasoñar",
    "trasoñarías": "trasoñar",
    "trasoñase": "trasoñar",
    "trasoñaseis": "trasoñar",
    "trasoñasen": "trasoñar",
    "trasoñases": "trasoñar",
    "trasoñaste": "trasoñar",
    "trasoñasteis": "trasoñar",
    "trasoñemos": "trasoñar",
    "trasoñábamos": "trasoñar",
    "trasoñáis": "trasoñar",
    "trasoñáramos": "trasoñar",
    "trasoñáremos": "trasoñar",
    "trasoñásemos": "trasoñar",
    "trasoñé": "trasoñar",
    "trasoñéis": "trasoñar",
    "trasoñó": "trasoñar",
    "traspadana": "traspadano",
    "traspadanas": "traspadano",
    "traspadanos": "traspadano",
    "traspala": "traspalar",
    "traspalaba": "traspalar",
    "traspalabais": "traspalar",
    "traspalaban": "traspalar",
    "traspalabas": "traspalar",
    "traspalad": "traspalar",
    "traspalada": "traspalar",
    "traspaladas": "traspalar",
    "traspalado": "traspalar",
    "traspalados": "traspalar",
    "traspalamos": "traspalar",
    "traspalan": "traspalar",
    "traspalando": "traspalar",
    "traspalara": "traspalar",
    "traspalarais": "traspalar",
    "traspalaran": "traspalar",
    "traspalaras": "traspalar",
    "traspalare": "traspalar",
    "traspalareis": "traspalar",
    "traspalaremos": "traspalar",
    "traspalaren": "traspalar",
    "traspalares": "traspalar",
    "traspalaron": "traspalar",
    "traspalará": "traspalar",
    "traspalarán": "traspalar",
    "traspalarás": "traspalar",
    "traspalaré": "traspalar",
    "traspalaréis": "traspalar",
    "traspalaría": "traspalar",
    "traspalaríais": "traspalar",
    "traspalaríamos": "traspalar",
    "traspalarían": "traspalar",
    "traspalarías": "traspalar",
    "traspalas": "traspalar",
    "traspalase": "traspalar",
    "traspalaseis": "traspalar",
    "traspalasen": "traspalar",
    "traspalases": "traspalar",
    "traspalaste": "traspalar",
    "traspalasteis": "traspalar",
    "traspale": "traspalar",
    "traspalea": "traspalear",
    "traspaleaba": "traspalear",
    "traspaleabais": "traspalear",
    "traspaleaban": "traspalear",
    "traspaleabas": "traspalear",
    "traspalead": "traspalear",
    "traspaleada": "traspalear",
    "traspaleadas": "traspalear",
    "traspaleado": "traspalear",
    "traspaleados": "traspalear",
    "traspaleamos": "traspalear",
    "traspalean": "traspalear",
    "traspaleando": "traspalear",
    "traspaleara": "traspalear",
    "traspalearais": "traspalear",
    "traspalearan": "traspalear",
    "traspalearas": "traspalear",
    "traspaleare": "traspalear",
    "traspaleareis": "traspalear",
    "traspalearemos": "traspalear",
    "traspalearen": "traspalear",
    "traspaleares": "traspalear",
    "traspalearon": "traspalear",
    "traspaleará": "traspalear",
    "traspalearán": "traspalear",
    "traspalearás": "traspalear",
    "traspalearé": "traspalear",
    "traspalearéis": "traspalear",
    "traspalearía": "traspalear",
    "traspalearíais": "traspalear",
    "traspalearíamos": "traspalear",
    "traspalearían": "traspalear",
    "traspalearías": "traspalear",
    "traspaleas": "traspalear",
    "traspalease": "traspalear",
    "traspaleaseis": "traspalear",
    "traspaleasen": "traspalear",
    "traspaleases": "traspalear",
    "traspaleaste": "traspalear",
    "traspaleasteis": "traspalear",
    "traspalee": "traspalear",
    "traspaleemos": "traspalear",
    "traspaleen": "traspalear",
    "traspalees": "traspalear",
    "traspalemos": "traspalar",
    "traspalen": "traspalar",
    "traspaleo": "traspalear",
    "traspaleos": "traspaleo",
    "traspales": "traspalar",
    "traspaleábamos": "traspalear",
    "traspaleáis": "traspalear",
    "traspaleáramos": "traspalear",
    "traspaleáremos": "traspalear",
    "traspaleásemos": "traspalear",
    "traspaleé": "traspalear",
    "traspaleéis": "traspalear",
    "traspaleó": "traspalear",
    "traspalo": "traspalar",
    "traspalábamos": "traspalar",
    "traspaláis": "traspalar",
    "traspaláramos": "traspalar",
    "traspaláremos": "traspalar",
    "traspalásemos": "traspalar",
    "traspalé": "traspalar",
    "traspaléis": "traspalar",
    "traspaló": "traspalar",
    "traspapela": "traspapelar",
    "traspapelaba": "traspapelar",
    "traspapelabais": "traspapelar",
    "traspapelaban": "traspapelar",
    "traspapelabas": "traspapelar",
    "traspapelad": "traspapelar",
    "traspapelada": "traspapelar",
    "traspapeladas": "traspapelar",
    "traspapelado": "traspapelar",
    "traspapelados": "traspapelar",
    "traspapelamos": "traspapelar",
    "traspapelan": "traspapelar",
    "traspapelando": "traspapelar",
    "traspapelara": "traspapelar",
    "traspapelarais": "traspapelar",
    "traspapelaran": "traspapelar",
    "traspapelaras": "traspapelar",
    "traspapelare": "traspapelar",
    "traspapelareis": "traspapelar",
    "traspapelaremos": "traspapelar",
    "traspapelaren": "traspapelar",
    "traspapelares": "traspapelar",
    "traspapelaron": "traspapelar",
    "traspapelará": "traspapelar",
    "traspapelarán": "traspapelar",
    "traspapelarás": "traspapelar",
    "traspapelaré": "traspapelar",
    "traspapelaréis": "traspapelar",
    "traspapelaría": "traspapelar",
    "traspapelaríais": "traspapelar",
    "traspapelaríamos": "traspapelar",
    "traspapelarían": "traspapelar",
    "traspapelarías": "traspapelar",
    "traspapelas": "traspapelar",
    "traspapelase": "traspapelar",
    "traspapelaseis": "traspapelar",
    "traspapelasen": "traspapelar",
    "traspapelases": "traspapelar",
    "traspapelaste": "traspapelar",
    "traspapelasteis": "traspapelar",
    "traspapele": "traspapelar",
    "traspapelemos": "traspapelar",
    "traspapelen": "traspapelar",
    "traspapeles": "traspapelar",
    "traspapelo": "traspapelar",
    "traspapelábamos": "traspapelar",
    "traspapeláis": "traspapelar",
    "traspapeláramos": "traspapelar",
    "traspapeláremos": "traspapelar",
    "traspapelásemos": "traspapelar",
    "traspapelé": "traspapelar",
    "traspapeléis": "traspapelar",
    "traspapeló": "traspapelar",
    "trasparencias": "trasparencia",
    "trasparenta": "trasparentar",
    "trasparentaba": "trasparentar",
    "trasparentabais": "trasparentar",
    "trasparentaban": "trasparentar",
    "trasparentabas": "trasparentar",
    "trasparentad": "trasparentar",
    "trasparentada": "trasparentar",
    "trasparentadas": "trasparentar",
    "trasparentado": "trasparentar",
    "trasparentados": "trasparentar",
    "trasparentamos": "trasparentar",
    "trasparentan": "trasparentar",
    "trasparentando": "trasparentar",
    "trasparentara": "trasparentar",
    "trasparentarais": "trasparentar",
    "trasparentaran": "trasparentar",
    "trasparentaras": "trasparentar",
    "trasparentare": "trasparentar",
    "trasparentareis": "trasparentar",
    "trasparentaremos": "trasparentar",
    "trasparentaren": "trasparentar",
    "trasparentares": "trasparentar",
    "trasparentaron": "trasparentar",
    "trasparentará": "trasparentar",
    "trasparentarán": "trasparentar",
    "trasparentarás": "trasparentar",
    "trasparentaré": "trasparentar",
    "trasparentaréis": "trasparentar",
    "trasparentaría": "trasparentar",
    "trasparentaríais": "trasparentar",
    "trasparentaríamos": "trasparentar",
    "trasparentarían": "trasparentar",
    "trasparentarías": "trasparentar",
    "trasparentas": "trasparentar",
    "trasparentase": "trasparentar",
    "trasparentaseis": "trasparentar",
    "trasparentasen": "trasparentar",
    "trasparentases": "trasparentar",
    "trasparentaste": "trasparentar",
    "trasparentasteis": "trasparentar",
    "trasparente": "trasparentar",
    "trasparentemos": "trasparentar",
    "trasparenten": "trasparentar",
    "trasparentes": "trasparente",
    "trasparento": "trasparentar",
    "trasparentábamos": "trasparentar",
    "trasparentáis": "trasparentar",
    "trasparentáramos": "trasparentar",
    "trasparentáremos": "trasparentar",
    "trasparentásemos": "trasparentar",
    "trasparenté": "trasparentar",
    "trasparentéis": "trasparentar",
    "trasparentó": "trasparentar",
    "traspasa": "traspasar",
    "traspasaba": "traspasar",
    "traspasabais": "traspasar",
    "traspasaban": "traspasar",
    "traspasabas": "traspasar",
    "traspasables": "traspasable",
    "traspasad": "traspasar",
    "traspasada": "traspasar",
    "traspasadas": "traspasar",
    "traspasado": "traspasar",
    "traspasadora": "traspasador",
    "traspasadoras": "traspasador",
    "traspasadores": "traspasador",
    "traspasados": "traspasar",
    "traspasamos": "traspasar",
    "traspasan": "traspasar",
    "traspasando": "traspasar",
    "traspasara": "traspasar",
    "traspasarais": "traspasar",
    "traspasaran": "traspasar",
    "traspasaras": "traspasar",
    "traspasare": "traspasar",
    "traspasareis": "traspasar",
    "traspasaremos": "traspasar",
    "traspasaren": "traspasar",
    "traspasares": "traspasar",
    "traspasaron": "traspasar",
    "traspasará": "traspasar",
    "traspasarán": "traspasar",
    "traspasarás": "traspasar",
    "traspasaré": "traspasar",
    "traspasaréis": "traspasar",
    "traspasaría": "traspasar",
    "traspasaríais": "traspasar",
    "traspasaríamos": "traspasar",
    "traspasarían": "traspasar",
    "traspasarías": "traspasar",
    "traspasas": "traspasar",
    "traspasase": "traspasar",
    "traspasaseis": "traspasar",
    "traspasasen": "traspasar",
    "traspasases": "traspasar",
    "traspasaste": "traspasar",
    "traspasasteis": "traspasar",
    "traspase": "traspasar",
    "traspasemos": "traspasar",
    "traspasen": "traspasar",
    "traspases": "traspasar",
    "traspaso": "traspasar",
    "traspasos": "traspaso",
    "traspasábamos": "traspasar",
    "traspasáis": "traspasar",
    "traspasáramos": "traspasar",
    "traspasáremos": "traspasar",
    "traspasásemos": "traspasar",
    "traspasé": "traspasar",
    "traspaséis": "traspasar",
    "traspasó": "traspasar",
    "traspatios": "traspatio",
    "traspaíses": "traspaís",
    "traspilastras": "traspilastra",
    "traspira": "traspirar",
    "traspiraba": "traspirar",
    "traspirabais": "traspirar",
    "traspiraban": "traspirar",
    "traspirabas": "traspirar",
    "traspirables": "traspirable",
    "traspiraciones": "traspiración",
    "traspirad": "traspirar",
    "traspirada": "traspirar",
    "traspiradas": "traspirar",
    "traspirado": "traspirar",
    "traspirados": "traspirar",
    "traspiramos": "traspirar",
    "traspiran": "traspirar",
    "traspirando": "traspirar",
    "traspirara": "traspirar",
    "traspirarais": "traspirar",
    "traspiraran": "traspirar",
    "traspiraras": "traspirar",
    "traspirare": "traspirar",
    "traspirareis": "traspirar",
    "traspiraremos": "traspirar",
    "traspiraren": "traspirar",
    "traspirares": "traspirar",
    "traspiraron": "traspirar",
    "traspirará": "traspirar",
    "traspirarán": "traspirar",
    "traspirarás": "traspirar",
    "traspiraré": "traspirar",
    "traspiraréis": "traspirar",
    "traspiraría": "traspirar",
    "traspiraríais": "traspirar",
    "traspiraríamos": "traspirar",
    "traspirarían": "traspirar",
    "traspirarías": "traspirar",
    "traspiras": "traspirar",
    "traspirase": "traspirar",
    "traspiraseis": "traspirar",
    "traspirasen": "traspirar",
    "traspirases": "traspirar",
    "traspiraste": "traspirar",
    "traspirasteis": "traspirar",
    "traspire": "traspirar",
    "traspiremos": "traspirar",
    "traspiren": "traspirar",
    "traspirenaica": "traspirenaico",
    "traspirenaicas": "traspirenaico",
    "traspirenaicos": "traspirenaico",
    "traspires": "traspirar",
    "traspiro": "traspirar",
    "traspirábamos": "traspirar",
    "traspiráis": "traspirar",
    "traspiráramos": "traspirar",
    "traspiráremos": "traspirar",
    "traspirásemos": "traspirar",
    "traspiré": "traspirar",
    "traspiréis": "traspirar",
    "traspiró": "traspirar",
    "traspiés": "traspié",
    "trasplanta": "trasplantar",
    "trasplantaba": "trasplantar",
    "trasplantabais": "trasplantar",
    "trasplantaban": "trasplantar",
    "trasplantabas": "trasplantar",
    "trasplantables": "trasplantable",
    "trasplantad": "trasplantar",
    "trasplantada": "trasplantar",
    "trasplantadas": "trasplantar",
    "trasplantado": "trasplantar",
    "trasplantadoras": "trasplantadora",
    "trasplantados": "trasplantar",
    "trasplantamos": "trasplantar",
    "trasplantan": "trasplantar",
    "trasplantando": "trasplantar",
    "trasplantara": "trasplantar",
    "trasplantarais": "trasplantar",
    "trasplantaran": "trasplantar",
    "trasplantaras": "trasplantar",
    "trasplantare": "trasplantar",
    "trasplantareis": "trasplantar",
    "trasplantaremos": "trasplantar",
    "trasplantaren": "trasplantar",
    "trasplantares": "trasplantar",
    "trasplantaron": "trasplantar",
    "trasplantará": "trasplantar",
    "trasplantarán": "trasplantar",
    "trasplantarás": "trasplantar",
    "trasplantaré": "trasplantar",
    "trasplantaréis": "trasplantar",
    "trasplantaría": "trasplantar",
    "trasplantaríais": "trasplantar",
    "trasplantaríamos": "trasplantar",
    "trasplantarían": "trasplantar",
    "trasplantarías": "trasplantar",
    "trasplantas": "trasplantar",
    "trasplantase": "trasplantar",
    "trasplantaseis": "trasplantar",
    "trasplantasen": "trasplantar",
    "trasplantases": "trasplantar",
    "trasplantaste": "trasplantar",
    "trasplantasteis": "trasplantar",
    "trasplante": "trasplantar",
    "trasplantemos": "trasplantar",
    "trasplanten": "trasplantar",
    "trasplantes": "trasplante",
    "trasplanto": "trasplantar",
    "trasplantábamos": "trasplantar",
    "trasplantáis": "trasplantar",
    "trasplantáramos": "trasplantar",
    "trasplantáremos": "trasplantar",
    "trasplantásemos": "trasplantar",
    "trasplanté": "trasplantar",
    "trasplantéis": "trasplantar",
    "trasplantó": "trasplantar",
    "trasplayas": "trasplaya",
    "traspondremos": "trasponer",
    "traspondrá": "trasponer",
    "traspondrán": "trasponer",
    "traspondrás": "trasponer",
    "traspondré": "trasponer",
    "traspondréis": "trasponer",
    "traspondría": "trasponer",
    "traspondríais": "trasponer",
    "traspondríamos": "trasponer",
    "traspondrían": "trasponer",
    "traspondrías": "trasponer",
    "traspone": "trasponer",
    "trasponed": "trasponer",
    "trasponedora": "trasponedor",
    "trasponedoras": "trasponedor",
    "trasponedores": "trasponedor",
    "trasponemos": "trasponer",
    "trasponen": "trasponer",
    "traspones": "trasponer",
    "trasponga": "trasponer",
    "traspongamos": "trasponer",
    "traspongan": "trasponer",
    "traspongas": "trasponer",
    "traspongo": "trasponer",
    "traspongáis": "trasponer",
    "trasponiendo": "trasponer",
    "trasponéis": "trasponer",
    "trasponía": "trasponer",
    "trasponíais": "trasponer",
    "trasponíamos": "trasponer",
    "trasponían": "trasponer",
    "trasponías": "trasponer",
    "trasportaciones": "trasportación",
    "trasportadora": "trasportador",
    "trasportadoras": "trasportador",
    "trasportadores": "trasportador",
    "trasportamientos": "trasportamiento",
    "trasportes": "trasporte",
    "trasportistas": "trasportista",
    "trasposiciones": "trasposición",
    "traspositiva": "traspositivo",
    "traspositivas": "traspositivo",
    "traspositivos": "traspositivo",
    "traspuesta": "trasponer",
    "traspuestas": "trasponer",
    "traspuesto": "trasponer",
    "traspuestos": "trasponer",
    "traspuntes": "traspunte",
    "traspuntines": "traspuntín",
    "traspuse": "trasponer",
    "traspusiera": "trasponer",
    "traspusierais": "trasponer",
    "traspusieran": "trasponer",
    "traspusieras": "trasponer",
    "traspusiere": "trasponer",
    "traspusiereis": "trasponer",
    "traspusieren": "trasponer",
    "traspusieres": "trasponer",
    "traspusieron": "trasponer",
    "traspusiese": "trasponer",
    "traspusieseis": "trasponer",
    "traspusiesen": "trasponer",
    "traspusieses": "trasponer",
    "traspusimos": "trasponer",
    "traspusiste": "trasponer",
    "traspusisteis": "trasponer",
    "traspusiéramos": "trasponer",
    "traspusiéremos": "trasponer",
    "traspusiésemos": "trasponer",
    "traspuso": "trasponer",
    "traspón": "trasponer",
    "trasquila": "trasquilar",
    "trasquilaba": "trasquilar",
    "trasquilabais": "trasquilar",
    "trasquilaban": "trasquilar",
    "trasquilabas": "trasquilar",
    "trasquilad": "trasquilar",
    "trasquilada": "trasquilar",
    "trasquiladas": "trasquilar",
    "trasquilado": "trasquilar",
    "trasquiladora": "trasquilador",
    "trasquiladoras": "trasquilador",
    "trasquiladores": "trasquilador",
    "trasquilados": "trasquilar",
    "trasquiladuras": "trasquiladura",
    "trasquilamos": "trasquilar",
    "trasquilan": "trasquilar",
    "trasquilando": "trasquilar",
    "trasquilara": "trasquilar",
    "trasquilarais": "trasquilar",
    "trasquilaran": "trasquilar",
    "trasquilaras": "trasquilar",
    "trasquilare": "trasquilar",
    "trasquilareis": "trasquilar",
    "trasquilaremos": "trasquilar",
    "trasquilaren": "trasquilar",
    "trasquilares": "trasquilar",
    "trasquilaron": "trasquilar",
    "trasquilará": "trasquilar",
    "trasquilarán": "trasquilar",
    "trasquilarás": "trasquilar",
    "trasquilaré": "trasquilar",
    "trasquilaréis": "trasquilar",
    "trasquilaría": "trasquilar",
    "trasquilaríais": "trasquilar",
    "trasquilaríamos": "trasquilar",
    "trasquilarían": "trasquilar",
    "trasquilarías": "trasquilar",
    "trasquilas": "trasquilar",
    "trasquilase": "trasquilar",
    "trasquilaseis": "trasquilar",
    "trasquilasen": "trasquilar",
    "trasquilases": "trasquilar",
    "trasquilaste": "trasquilar",
    "trasquilasteis": "trasquilar",
    "trasquile": "trasquilar",
    "trasquilemos": "trasquilar",
    "trasquilen": "trasquilar",
    "trasquiles": "trasquilar",
    "trasquilimocha": "trasquilimocho",
    "trasquilimochas": "trasquilimocho",
    "trasquilimochos": "trasquilimocho",
    "trasquilo": "trasquilar",
    "trasquilones": "trasquilón",
    "trasquilábamos": "trasquilar",
    "trasquiláis": "trasquilar",
    "trasquiláramos": "trasquilar",
    "trasquiláremos": "trasquilar",
    "trasquilásemos": "trasquilar",
    "trasquilé": "trasquilar",
    "trasquiléis": "trasquilar",
    "trasquiló": "trasquilar",
    "trastabilla": "trastabillar",
    "trastabillaba": "trastabillar",
    "trastabillabais": "trastabillar",
    "trastabillaban": "trastabillar",
    "trastabillabas": "trastabillar",
    "trastabillad": "trastabillar",
    "trastabillada": "trastabillar",
    "trastabilladas": "trastabillar",
    "trastabillado": "trastabillar",
    "trastabillados": "trastabillar",
    "trastabillamos": "trastabillar",
    "trastabillan": "trastabillar",
    "trastabillando": "trastabillar",
    "trastabillara": "trastabillar",
    "trastabillarais": "trastabillar",
    "trastabillaran": "trastabillar",
    "trastabillaras": "trastabillar",
    "trastabillare": "trastabillar",
    "trastabillareis": "trastabillar",
    "trastabillaremos": "trastabillar",
    "trastabillaren": "trastabillar",
    "trastabillares": "trastabillar",
    "trastabillaron": "trastabillar",
    "trastabillará": "trastabillar",
    "trastabillarán": "trastabillar",
    "trastabillarás": "trastabillar",
    "trastabillaré": "trastabillar",
    "trastabillaréis": "trastabillar",
    "trastabillaría": "trastabillar",
    "trastabillaríais": "trastabillar",
    "trastabillaríamos": "trastabillar",
    "trastabillarían": "trastabillar",
    "trastabillarías": "trastabillar",
    "trastabillas": "trastabillar",
    "trastabillase": "trastabillar",
    "trastabillaseis": "trastabillar",
    "trastabillasen": "trastabillar",
    "trastabillases": "trastabillar",
    "trastabillaste": "trastabillar",
    "trastabillasteis": "trastabillar",
    "trastabille": "trastabillar",
    "trastabillemos": "trastabillar",
    "trastabillen": "trastabillar",
    "trastabilles": "trastabillar",
    "trastabillo": "trastabillar",
    "trastabillábamos": "trastabillar",
    "trastabilláis": "trastabillar",
    "trastabilláramos": "trastabillar",
    "trastabilláremos": "trastabillar",
    "trastabillásemos": "trastabillar",
    "trastabillé": "trastabillar",
    "trastabilléis": "trastabillar",
    "trastabilló": "trastabillar",
    "trastadas": "trastada",
    "trastajos": "trastajo",
    "trastazos": "trastazo",
    "trastea": "trastear",
    "trasteaba": "trastear",
    "trasteabais": "trastear",
    "trasteaban": "trastear",
    "trasteabas": "trastear",
    "trastead": "trastear",
    "trasteada": "trastear",
    "trasteadas": "trastear",
    "trasteado": "trastear",
    "trasteadora": "trasteador",
    "trasteadoras": "trasteador",
    "trasteadores": "trasteador",
    "trasteados": "trastear",
    "trasteamos": "trastear",
    "trastean": "trastear",
    "trasteando": "trastear",
    "trasteantes": "trasteante",
    "trasteara": "trastear",
    "trastearais": "trastear",
    "trastearan": "trastear",
    "trastearas": "trastear",
    "trasteare": "trastear",
    "trasteareis": "trastear",
    "trastearemos": "trastear",
    "trastearen": "trastear",
    "trasteares": "trastear",
    "trastearon": "trastear",
    "trasteará": "trastear",
    "trastearán": "trastear",
    "trastearás": "trastear",
    "trastearé": "trastear",
    "trastearéis": "trastear",
    "trastearía": "trastear",
    "trastearíais": "trastear",
    "trastearíamos": "trastear",
    "trastearían": "trastear",
    "trastearías": "trastear",
    "trasteas": "trastear",
    "trastease": "trastear",
    "trasteaseis": "trastear",
    "trasteasen": "trastear",
    "trasteases": "trastear",
    "trasteaste": "trastear",
    "trasteasteis": "trastear",
    "trastee": "trastear",
    "trasteemos": "trastear",
    "trasteen": "trastear",
    "trastees": "trastear",
    "trasteja": "trastejar",
    "trastejaba": "trastejar",
    "trastejabais": "trastejar",
    "trastejaban": "trastejar",
    "trastejabas": "trastejar",
    "trastejad": "trastejar",
    "trastejada": "trastejar",
    "trastejadas": "trastejar",
    "trastejado": "trastejar",
    "trastejadora": "trastejador",
    "trastejadoras": "trastejador",
    "trastejadores": "trastejador",
    "trastejados": "trastejar",
    "trastejaduras": "trastejadura",
    "trastejamos": "trastejar",
    "trastejan": "trastejar",
    "trastejando": "trastejar",
    "trastejara": "trastejar",
    "trastejarais": "trastejar",
    "trastejaran": "trastejar",
    "trastejaras": "trastejar",
    "trastejare": "trastejar",
    "trastejareis": "trastejar",
    "trastejaremos": "trastejar",
    "trastejaren": "trastejar",
    "trastejares": "trastejar",
    "trastejaron": "trastejar",
    "trastejará": "trastejar",
    "trastejarán": "trastejar",
    "trastejarás": "trastejar",
    "trastejaré": "trastejar",
    "trastejaréis": "trastejar",
    "trastejaría": "trastejar",
    "trastejaríais": "trastejar",
    "trastejaríamos": "trastejar",
    "trastejarían": "trastejar",
    "trastejarías": "trastejar",
    "trastejas": "trastejar",
    "trastejase": "trastejar",
    "trastejaseis": "trastejar",
    "trastejasen": "trastejar",
    "trastejases": "trastejar",
    "trastejaste": "trastejar",
    "trastejasteis": "trastejar",
    "trasteje": "trastejar",
    "trastejemos": "trastejar",
    "trastejen": "trastejar",
    "trastejes": "trastejar",
    "trastejo": "trastejar",
    "trastejos": "trastejo",
    "trastejábamos": "trastejar",
    "trastejáis": "trastejar",
    "trastejáramos": "trastejar",
    "trastejáremos": "trastejar",
    "trastejásemos": "trastejar",
    "trastejé": "trastejar",
    "trastejéis": "trastejar",
    "trastejó": "trastejar",
    "trasteo": "trastear",
    "trasteos": "trasteo",
    "trastera": "trastero",
    "trasteras": "trastero",
    "trasteros": "trastero",
    "trasterrada": "trasterrado",
    "trasterradas": "trasterrado",
    "trasterrados": "trasterrado",
    "trasterías": "trastería",
    "trastes": "traste",
    "trastesada": "trastesado",
    "trastesadas": "trastesado",
    "trastesados": "trastesado",
    "trastesones": "trastesón",
    "trasteábamos": "trastear",
    "trasteáis": "trastear",
    "trasteáramos": "trastear",
    "trasteáremos": "trastear",
    "trasteásemos": "trastear",
    "trasteé": "trastear",
    "trasteéis": "trastear",
    "trasteó": "trastear",
    "trastiendas": "trastienda",
    "trastoca": "trastocar",
    "trastocaba": "trastocar",
    "trastocabais": "trastocar",
    "trastocaban": "trastocar",
    "trastocabas": "trastocar",
    "trastocad": "trastocar",
    "trastocada": "trastocar",
    "trastocadas": "trastocar",
    "trastocado": "trastocar",
    "trastocados": "trastocar",
    "trastocamos": "trastocar",
    "trastocan": "trastocar",
    "trastocando": "trastocar",
    "trastocara": "trastocar",
    "trastocarais": "trastocar",
    "trastocaran": "trastocar",
    "trastocaras": "trastocar",
    "trastocare": "trastocar",
    "trastocareis": "trastocar",
    "trastocaremos": "trastocar",
    "trastocaren": "trastocar",
    "trastocares": "trastocar",
    "trastocaron": "trastocar",
    "trastocará": "trastocar",
    "trastocarán": "trastocar",
    "trastocarás": "trastocar",
    "trastocaré": "trastocar",
    "trastocaréis": "trastocar",
    "trastocaría": "trastocar",
    "trastocaríais": "trastocar",
    "trastocaríamos": "trastocar",
    "trastocarían": "trastocar",
    "trastocarías": "trastocar",
    "trastocas": "trastocar",
    "trastocase": "trastocar",
    "trastocaseis": "trastocar",
    "trastocasen": "trastocar",
    "trastocases": "trastocar",
    "trastocaste": "trastocar",
    "trastocasteis": "trastocar",
    "trastoco": "trastocar",
    "trastocábamos": "trastocar",
    "trastocáis": "trastocar",
    "trastocáramos": "trastocar",
    "trastocáremos": "trastocar",
    "trastocásemos": "trastocar",
    "trastocó": "trastocar",
    "trastoque": "trastocar",
    "trastoquemos": "trastocar",
    "trastoquen": "trastocar",
    "trastoques": "trastocar",
    "trastoqué": "trastocar",
    "trastoquéis": "trastocar",
    "trastorna": "trastornar",
    "trastornaba": "trastornar",
    "trastornabais": "trastornar",
    "trastornaban": "trastornar",
    "trastornabas": "trastornar",
    "trastornables": "trastornable",
    "trastornad": "trastornar",
    "trastornada": "trastornar",
    "trastornadas": "trastornar",
    "trastornado": "trastornar",
    "trastornadora": "trastornador",
    "trastornadoras": "trastornador",
    "trastornadores": "trastornador",
    "trastornados": "trastornar",
    "trastornaduras": "trastornadura",
    "trastornamientos": "trastornamiento",
    "trastornamos": "trastornar",
    "trastornan": "trastornar",
    "trastornando": "trastornar",
    "trastornara": "trastornar",
    "trastornarais": "trastornar",
    "trastornaran": "trastornar",
    "trastornaras": "trastornar",
    "trastornare": "trastornar",
    "trastornareis": "trastornar",
    "trastornaremos": "trastornar",
    "trastornaren": "trastornar",
    "trastornares": "trastornar",
    "trastornaron": "trastornar",
    "trastornará": "trastornar",
    "trastornarán": "trastornar",
    "trastornarás": "trastornar",
    "trastornaré": "trastornar",
    "trastornaréis": "trastornar",
    "trastornaría": "trastornar",
    "trastornaríais": "trastornar",
    "trastornaríamos": "trastornar",
    "trastornarían": "trastornar",
    "trastornarías": "trastornar",
    "trastornas": "trastornar",
    "trastornase": "trastornar",
    "trastornaseis": "trastornar",
    "trastornasen": "trastornar",
    "trastornases": "trastornar",
    "trastornaste": "trastornar",
    "trastornasteis": "trastornar",
    "trastorne": "trastornar",
    "trastornemos": "trastornar",
    "trastornen": "trastornar",
    "trastornes": "trastornar",
    "trastorno": "trastornar",
    "trastornos": "trastorno",
    "trastornábamos": "trastornar",
    "trastornáis": "trastornar",
    "trastornáramos": "trastornar",
    "trastornáremos": "trastornar",
    "trastornásemos": "trastornar",
    "trastorné": "trastornar",
    "trastornéis": "trastornar",
    "trastornó": "trastornar",
    "trastos": "trasto",
    "trastrabada": "trastrabado",
    "trastrabadas": "trastrabado",
    "trastrabados": "trastrabado",
    "trastrabilla": "trastrabillar",
    "trastrabillaba": "trastrabillar",
    "trastrabillabais": "trastrabillar",
    "trastrabillaban": "trastrabillar",
    "trastrabillabas": "trastrabillar",
    "trastrabillad": "trastrabillar",
    "trastrabillada": "trastrabillar",
    "trastrabilladas": "trastrabillar",
    "trastrabillado": "trastrabillar",
    "trastrabillados": "trastrabillar",
    "trastrabillamos": "trastrabillar",
    "trastrabillan": "trastrabillar",
    "trastrabillando": "trastrabillar",
    "trastrabillara": "trastrabillar",
    "trastrabillarais": "trastrabillar",
    "trastrabillaran": "trastrabillar",
    "trastrabillaras": "trastrabillar",
    "trastrabillare": "trastrabillar",
    "trastrabillareis": "trastrabillar",
    "trastrabillaremos": "trastrabillar",
    "trastrabillaren": "trastrabillar",
    "trastrabillares": "trastrabillar",
    "trastrabillaron": "trastrabillar",
    "trastrabillará": "trastrabillar",
    "trastrabillarán": "trastrabillar",
    "trastrabillarás": "trastrabillar",
    "trastrabillaré": "trastrabillar",
    "trastrabillaréis": "trastrabillar",
    "trastrabillaría": "trastrabillar",
    "trastrabillaríais": "trastrabillar",
    "trastrabillaríamos": "trastrabillar",
    "trastrabillarían": "trastrabillar",
    "trastrabillarías": "trastrabillar",
    "trastrabillas": "trastrabillar",
    "trastrabillase": "trastrabillar",
    "trastrabillaseis": "trastrabillar",
    "trastrabillasen": "trastrabillar",
    "trastrabillases": "trastrabillar",
    "trastrabillaste": "trastrabillar",
    "trastrabillasteis": "trastrabillar",
    "trastrabille": "trastrabillar",
    "trastrabillemos": "trastrabillar",
    "trastrabillen": "trastrabillar",
    "trastrabilles": "trastrabillar",
    "trastrabillo": "trastrabillar",
    "trastrabillábamos": "trastrabillar",
    "trastrabilláis": "trastrabillar",
    "trastrabilláramos": "trastrabillar",
    "trastrabilláremos": "trastrabillar",
    "trastrabillásemos": "trastrabillar",
    "trastrabillé": "trastrabillar",
    "trastrabilléis": "trastrabillar",
    "trastrabilló": "trastrabillar",
    "trastrases": "trastrás",
    "trastrocaba": "trastrocar",
    "trastrocabais": "trastrocar",
    "trastrocaban": "trastrocar",
    "trastrocabas": "trastrocar",
    "trastrocad": "trastrocar",
    "trastrocada": "trastrocar",
    "trastrocadas": "trastrocar",
    "trastrocado": "trastrocar",
    "trastrocados": "trastrocar",
    "trastrocamientos": "trastrocamiento",
    "trastrocamos": "trastrocar",
    "trastrocando": "trastrocar",
    "trastrocara": "trastrocar",
    "trastrocarais": "trastrocar",
    "trastrocaran": "trastrocar",
    "trastrocaras": "trastrocar",
    "trastrocare": "trastrocar",
    "trastrocareis": "trastrocar",
    "trastrocaremos": "trastrocar",
    "trastrocaren": "trastrocar",
    "trastrocares": "trastrocar",
    "trastrocaron": "trastrocar",
    "trastrocará": "trastrocar",
    "trastrocarán": "trastrocar",
    "trastrocarás": "trastrocar",
    "trastrocaré": "trastrocar",
    "trastrocaréis": "trastrocar",
    "trastrocaría": "trastrocar",
    "trastrocaríais": "trastrocar",
    "trastrocaríamos": "trastrocar",
    "trastrocarían": "trastrocar",
    "trastrocarías": "trastrocar",
    "trastrocase": "trastrocar",
    "trastrocaseis": "trastrocar",
    "trastrocasen": "trastrocar",
    "trastrocases": "trastrocar",
    "trastrocaste": "trastrocar",
    "trastrocasteis": "trastrocar",
    "trastrocábamos": "trastrocar",
    "trastrocáis": "trastrocar",
    "trastrocáramos": "trastrocar",
    "trastrocáremos": "trastrocar",
    "trastrocásemos": "trastrocar",
    "trastrocó": "trastrocar",
    "trastroquemos": "trastrocar",
    "trastroqué": "trastrocar",
    "trastroquéis": "trastrocar",
    "trastrueca": "trastrocar",
    "trastruecan": "trastrocar",
    "trastruecas": "trastrocar",
    "trastrueco": "trastrocar",
    "trastruecos": "trastrueco",
    "trastrueque": "trastrocar",
    "trastruequen": "trastrocar",
    "trastrueques": "trastrueque",
    "trasuda": "trasudar",
    "trasudaba": "trasudar",
    "trasudabais": "trasudar",
    "trasudaban": "trasudar",
    "trasudabas": "trasudar",
    "trasudaciones": "trasudación",
    "trasudad": "trasudar",
    "trasudada": "trasudar",
    "trasudadas": "trasudar",
    "trasudado": "trasudar",
    "trasudados": "trasudar",
    "trasudamos": "trasudar",
    "trasudan": "trasudar",
    "trasudando": "trasudar",
    "trasudara": "trasudar",
    "trasudarais": "trasudar",
    "trasudaran": "trasudar",
    "trasudaras": "trasudar",
    "trasudare": "trasudar",
    "trasudareis": "trasudar",
    "trasudaremos": "trasudar",
    "trasudaren": "trasudar",
    "trasudares": "trasudar",
    "trasudaron": "trasudar",
    "trasudará": "trasudar",
    "trasudarán": "trasudar",
    "trasudarás": "trasudar",
    "trasudaré": "trasudar",
    "trasudaréis": "trasudar",
    "trasudaría": "trasudar",
    "trasudaríais": "trasudar",
    "trasudaríamos": "trasudar",
    "trasudarían": "trasudar",
    "trasudarías": "trasudar",
    "trasudas": "trasudar",
    "trasudase": "trasudar",
    "trasudaseis": "trasudar",
    "trasudasen": "trasudar",
    "trasudases": "trasudar",
    "trasudaste": "trasudar",
    "trasudasteis": "trasudar",
    "trasude": "trasudar",
    "trasudemos": "trasudar",
    "trasuden": "trasudar",
    "trasudes": "trasudar",
    "trasudo": "trasudar",
    "trasudores": "trasudor",
    "trasudábamos": "trasudar",
    "trasudáis": "trasudar",
    "trasudáramos": "trasudar",
    "trasudáremos": "trasudar",
    "trasudásemos": "trasudar",
    "trasudé": "trasudar",
    "trasudéis": "trasudar",
    "trasudó": "trasudar",
    "trasueña": "trasoñar",
    "trasueñan": "trasoñar",
    "trasueñas": "trasoñar",
    "trasueñe": "trasoñar",
    "trasueñen": "trasoñar",
    "trasueñes": "trasoñar",
    "trasueño": "trasoñar",
    "trasunta": "trasuntar",
    "trasuntaba": "trasuntar",
    "trasuntabais": "trasuntar",
    "trasuntaban": "trasuntar",
    "trasuntabas": "trasuntar",
    "trasuntad": "trasuntar",
    "trasuntada": "trasuntar",
    "trasuntadas": "trasuntar",
    "trasuntado": "trasuntar",
    "trasuntados": "trasuntar",
    "trasuntamos": "trasuntar",
    "trasuntan": "trasuntar",
    "trasuntando": "trasuntar",
    "trasuntara": "trasuntar",
    "trasuntarais": "trasuntar",
    "trasuntaran": "trasuntar",
    "trasuntaras": "trasuntar",
    "trasuntare": "trasuntar",
    "trasuntareis": "trasuntar",
    "trasuntaremos": "trasuntar",
    "trasuntaren": "trasuntar",
    "trasuntares": "trasuntar",
    "trasuntaron": "trasuntar",
    "trasuntará": "trasuntar",
    "trasuntarán": "trasuntar",
    "trasuntarás": "trasuntar",
    "trasuntaré": "trasuntar",
    "trasuntaréis": "trasuntar",
    "trasuntaría": "trasuntar",
    "trasuntaríais": "trasuntar",
    "trasuntaríamos": "trasuntar",
    "trasuntarían": "trasuntar",
    "trasuntarías": "trasuntar",
    "trasuntas": "trasuntar",
    "trasuntase": "trasuntar",
    "trasuntaseis": "trasuntar",
    "trasuntasen": "trasuntar",
    "trasuntases": "trasuntar",
    "trasuntaste": "trasuntar",
    "trasuntasteis": "trasuntar",
    "trasunte": "trasuntar",
    "trasuntemos": "trasuntar",
    "trasunten": "trasuntar",
    "trasuntes": "trasuntar",
    "trasunto": "trasuntar",
    "trasuntos": "trasunto",
    "trasuntábamos": "trasuntar",
    "trasuntáis": "trasuntar",
    "trasuntáramos": "trasuntar",
    "trasuntáremos": "trasuntar",
    "trasuntásemos": "trasuntar",
    "trasunté": "trasuntar",
    "trasuntéis": "trasuntar",
    "trasuntó": "trasuntar",
    "trasvasa": "trasvasar",
    "trasvasaba": "trasvasar",
    "trasvasabais": "trasvasar",
    "trasvasaban": "trasvasar",
    "trasvasabas": "trasvasar",
    "trasvasad": "trasvasar",
    "trasvasada": "trasvasar",
    "trasvasadas": "trasvasar",
    "trasvasado": "trasvasar",
    "trasvasados": "trasvasar",
    "trasvasamos": "trasvasar",
    "trasvasan": "trasvasar",
    "trasvasando": "trasvasar",
    "trasvasara": "trasvasar",
    "trasvasarais": "trasvasar",
    "trasvasaran": "trasvasar",
    "trasvasaras": "trasvasar",
    "trasvasare": "trasvasar",
    "trasvasareis": "trasvasar",
    "trasvasaremos": "trasvasar",
    "trasvasaren": "trasvasar",
    "trasvasares": "trasvasar",
    "trasvasaron": "trasvasar",
    "trasvasará": "trasvasar",
    "trasvasarán": "trasvasar",
    "trasvasarás": "trasvasar",
    "trasvasaré": "trasvasar",
    "trasvasaréis": "trasvasar",
    "trasvasaría": "trasvasar",
    "trasvasaríais": "trasvasar",
    "trasvasaríamos": "trasvasar",
    "trasvasarían": "trasvasar",
    "trasvasarías": "trasvasar",
    "trasvasas": "trasvasar",
    "trasvasase": "trasvasar",
    "trasvasaseis": "trasvasar",
    "trasvasasen": "trasvasar",
    "trasvasases": "trasvasar",
    "trasvasaste": "trasvasar",
    "trasvasasteis": "trasvasar",
    "trasvase": "trasvasar",
    "trasvasemos": "trasvasar",
    "trasvasen": "trasvasar",
    "trasvases": "trasvase",
    "trasvaso": "trasvasar",
    "trasvasábamos": "trasvasar",
    "trasvasáis": "trasvasar",
    "trasvasáramos": "trasvasar",
    "trasvasáremos": "trasvasar",
    "trasvasásemos": "trasvasar",
    "trasvasé": "trasvasar",
    "trasvaséis": "trasvasar",
    "trasvasó": "trasvasar",
    "trasvea": "trasver",
    "trasveamos": "trasver",
    "trasvean": "trasver",
    "trasveas": "trasver",
    "trasved": "trasver",
    "trasvemos": "trasver",
    "trasvena": "trasvenar",
    "trasvenaba": "trasvenar",
    "trasvenabais": "trasvenar",
    "trasvenaban": "trasvenar",
    "trasvenabas": "trasvenar",
    "trasvenad": "trasvenar",
    "trasvenada": "trasvenar",
    "trasvenadas": "trasvenar",
    "trasvenado": "trasvenar",
    "trasvenados": "trasvenar",
    "trasvenamos": "trasvenar",
    "trasvenan": "trasvenar",
    "trasvenando": "trasvenar",
    "trasvenara": "trasvenar",
    "trasvenarais": "trasvenar",
    "trasvenaran": "trasvenar",
    "trasvenaras": "trasvenar",
    "trasvenare": "trasvenar",
    "trasvenareis": "trasvenar",
    "trasvenaremos": "trasvenar",
    "trasvenaren": "trasvenar",
    "trasvenares": "trasvenar",
    "trasvenaron": "trasvenar",
    "trasvenará": "trasvenar",
    "trasvenarán": "trasvenar",
    "trasvenarás": "trasvenar",
    "trasvenaré": "trasvenar",
    "trasvenaréis": "trasvenar",
    "trasvenaría": "trasvenar",
    "trasvenaríais": "trasvenar",
    "trasvenaríamos": "trasvenar",
    "trasvenarían": "trasvenar",
    "trasvenarías": "trasvenar",
    "trasvenas": "trasvenar",
    "trasvenase": "trasvenar",
    "trasvenaseis": "trasvenar",
    "trasvenasen": "trasvenar",
    "trasvenases": "trasvenar",
    "trasvenaste": "trasvenar",
    "trasvenasteis": "trasvenar",
    "trasvene": "trasvenar",
    "trasvenemos": "trasvenar",
    "trasvenen": "trasvenar",
    "trasvenes": "trasvenar",
    "trasveno": "trasvenar",
    "trasvenábamos": "trasvenar",
    "trasvenáis": "trasvenar",
    "trasvenáramos": "trasvenar",
    "trasvenáremos": "trasvenar",
    "trasvenásemos": "trasvenar",
    "trasvené": "trasvenar",
    "trasvenéis": "trasvenar",
    "trasvenó": "trasvenar",
    "trasveo": "trasver",
    "trasveremos": "trasver",
    "trasvertamos": "trasverter",
    "trasverted": "trasverter",
    "trasvertemos": "trasverter",
    "trasverteremos": "trasverter",
    "trasverterá": "trasverter",
    "trasverterán": "trasverter",
    "trasverterás": "trasverter",
    "trasverteré": "trasverter",
    "trasverteréis": "trasverter",
    "trasvertería": "trasverter",
    "trasverteríais": "trasverter",
    "trasverteríamos": "trasverter",
    "trasverterían": "trasverter",
    "trasverterías": "trasverter",
    "trasvertida": "trasverter",
    "trasvertidas": "trasverter",
    "trasvertido": "trasverter",
    "trasvertidos": "trasverter",
    "trasvertiendo": "trasverter",
    "trasvertiera": "trasverter",
    "trasvertierais": "trasverter",
    "trasvertieran": "trasverter",
    "trasvertieras": "trasverter",
    "trasvertiere": "trasverter",
    "trasvertiereis": "trasverter",
    "trasvertieren": "trasverter",
    "trasvertieres": "trasverter",
    "trasvertieron": "trasverter",
    "trasvertiese": "trasverter",
    "trasvertieseis": "trasverter",
    "trasvertiesen": "trasverter",
    "trasvertieses": "trasverter",
    "trasvertimos": "trasverter",
    "trasvertiste": "trasverter",
    "trasvertisteis": "trasverter",
    "trasvertiéramos": "trasverter",
    "trasvertiéremos": "trasverter",
    "trasvertiésemos": "trasverter",
    "trasvertió": "trasverter",
    "trasvertáis": "trasverter",
    "trasvertéis": "trasverter",
    "trasvertí": "trasverter",
    "trasvertía": "trasverter",
    "trasvertíais": "trasverter",
    "trasvertíamos": "trasverter",
    "trasvertían": "trasverter",
    "trasvertías": "trasverter",
    "trasverá": "trasver",
    "trasverán": "trasver",
    "trasverás": "trasver",
    "trasveré": "trasver",
    "trasveréis": "trasver",
    "trasvería": "trasver",
    "trasveríais": "trasver",
    "trasveríamos": "trasver",
    "trasverían": "trasver",
    "trasverías": "trasver",
    "trasveáis": "trasver",
    "trasveía": "trasver",
    "trasveíais": "trasver",
    "trasveíamos": "trasver",
    "trasveían": "trasver",
    "trasveías": "trasver",
    "trasviendo": "trasver",
    "trasviera": "trasver",
    "trasvierais": "trasver",
    "trasvieran": "trasver",
    "trasvieras": "trasver",
    "trasviere": "trasver",
    "trasviereis": "trasver",
    "trasvieren": "trasver",
    "trasvieres": "trasver",
    "trasvieron": "trasver",
    "trasvierta": "trasverter",
    "trasviertan": "trasverter",
    "trasviertas": "trasverter",
    "trasvierte": "trasverter",
    "trasvierten": "trasverter",
    "trasviertes": "trasverter",
    "trasvierto": "trasverter",
    "trasviese": "trasver",
    "trasvieseis": "trasver",
    "trasviesen": "trasver",
    "trasvieses": "trasver",
    "trasvimos": "trasver",
    "trasvina": "trasvinar",
    "trasvinaba": "trasvinar",
    "trasvinabais": "trasvinar",
    "trasvinaban": "trasvinar",
    "trasvinabas": "trasvinar",
    "trasvinad": "trasvinar",
    "trasvinada": "trasvinar",
    "trasvinadas": "trasvinar",
    "trasvinado": "trasvinar",
    "trasvinados": "trasvinar",
    "trasvinamos": "trasvinar",
    "trasvinan": "trasvinar",
    "trasvinando": "trasvinar",
    "trasvinara": "trasvinar",
    "trasvinarais": "trasvinar",
    "trasvinaran": "trasvinar",
    "trasvinaras": "trasvinar",
    "trasvinare": "trasvinar",
    "trasvinareis": "trasvinar",
    "trasvinaremos": "trasvinar",
    "trasvinaren": "trasvinar",
    "trasvinares": "trasvinar",
    "trasvinaron": "trasvinar",
    "trasvinará": "trasvinar",
    "trasvinarán": "trasvinar",
    "trasvinarás": "trasvinar",
    "trasvinaré": "trasvinar",
    "trasvinaréis": "trasvinar",
    "trasvinaría": "trasvinar",
    "trasvinaríais": "trasvinar",
    "trasvinaríamos": "trasvinar",
    "trasvinarían": "trasvinar",
    "trasvinarías": "trasvinar",
    "trasvinas": "trasvinar",
    "trasvinase": "trasvinar",
    "trasvinaseis": "trasvinar",
    "trasvinasen": "trasvinar",
    "trasvinases": "trasvinar",
    "trasvinaste": "trasvinar",
    "trasvinasteis": "trasvinar",
    "trasvine": "trasvinar",
    "trasvinemos": "trasvinar",
    "trasvinen": "trasvinar",
    "trasvines": "trasvinar",
    "trasvino": "trasvinar",
    "trasvinábamos": "trasvinar",
    "trasvináis": "trasvinar",
    "trasvináramos": "trasvinar",
    "trasvináremos": "trasvinar",
    "trasvinásemos": "trasvinar",
    "trasviné": "trasvinar",
    "trasvinéis": "trasvinar",
    "trasvinó": "trasvinar",
    "trasvista": "trasver",
    "trasvistas": "trasver",
    "trasviste": "trasver",
    "trasvisteis": "trasver",
    "trasvisto": "trasver",
    "trasvistos": "trasver",
    "trasviéramos": "trasver",
    "trasviéremos": "trasver",
    "trasviésemos": "trasver",
    "trasvió": "trasver",
    "trasvé": "trasver",
    "trasvéis": "trasver",
    "trasvén": "trasver",
    "trasvés": "trasver",
    "trasví": "trasver",
    "trata": "tratar",
    "trataba": "tratar",
    "tratabais": "tratar",
    "trataban": "tratar",
    "tratabas": "tratar",
    "tratables": "tratable",
    "tratad": "tratar",
    "tratada": "tratar",
    "tratadas": "tratar",
    "tratadistas": "tratadista",
    "tratado": "tratar",
    "tratadora": "tratador",
    "tratadoras": "tratador",
    "tratadores": "tratador",
    "tratados": "tratar",
    "tratados-ley": "tratado-ley",
    "tratamentales": "tratamental",
    "tratamientos": "tratamiento",
    "tratamos": "tratar",
    "tratan": "tratar",
    "tratando": "tratar",
    "tratantes": "tratante",
    "tratara": "tratar",
    "tratarais": "tratar",
    "trataran": "tratar",
    "trataras": "tratar",
    "tratare": "tratar",
    "tratareis": "tratar",
    "trataremos": "tratar",
    "trataren": "tratar",
    "tratares": "tratar",
    "trataron": "tratar",
    "tratará": "tratar",
    "tratarán": "tratar",
    "tratarás": "tratar",
    "trataré": "tratar",
    "trataréis": "tratar",
    "trataría": "tratar",
    "trataríais": "tratar",
    "trataríamos": "tratar",
    "tratarían": "tratar",
    "tratarías": "tratar",
    "tratas": "tratar",
    "tratase": "tratar",
    "trataseis": "tratar",
    "tratasen": "tratar",
    "tratases": "tratar",
    "trataste": "tratar",
    "tratasteis": "tratar",
    "trate": "tratar",
    "tratemos": "tratar",
    "traten": "tratar",
    "trates": "tratar",
    "trato": "tratar",
    "tratos": "trato",
    "tratábamos": "tratar",
    "tratáis": "tratar",
    "tratáramos": "tratar",
    "tratáremos": "tratar",
    "tratásemos": "tratar",
    "traté": "tratar",
    "tratéis": "tratar",
    "trató": "tratar",
    "traumas": "trauma",
    "traumatice": "traumatizar",
    "traumaticemos": "traumatizar",
    "traumaticen": "traumatizar",
    "traumatices": "traumatizar",
    "traumaticé": "traumatizar",
    "traumaticéis": "traumatizar",
    "traumatismos": "traumatismo",
    "traumatiza": "traumatizar",
    "traumatizaba": "traumatizar",
    "traumatizabais": "traumatizar",
    "traumatizaban": "traumatizar",
    "traumatizabas": "traumatizar",
    "traumatizad": "traumatizar",
    "traumatizada": "traumatizar",
    "traumatizadas": "traumatizar",
    "traumatizado": "traumatizar",
    "traumatizados": "traumatizar",
    "traumatizamos": "traumatizar",
    "traumatizan": "traumatizar",
    "traumatizando": "traumatizar",
    "traumatizara": "traumatizar",
    "traumatizarais": "traumatizar",
    "traumatizaran": "traumatizar",
    "traumatizaras": "traumatizar",
    "traumatizare": "traumatizar",
    "traumatizareis": "traumatizar",
    "traumatizaremos": "traumatizar",
    "traumatizaren": "traumatizar",
    "traumatizares": "traumatizar",
    "traumatizaron": "traumatizar",
    "traumatizará": "traumatizar",
    "traumatizarán": "traumatizar",
    "traumatizarás": "traumatizar",
    "traumatizaré": "traumatizar",
    "traumatizaréis": "traumatizar",
    "traumatizaría": "traumatizar",
    "traumatizaríais": "traumatizar",
    "traumatizaríamos": "traumatizar",
    "traumatizarían": "traumatizar",
    "traumatizarías": "traumatizar",
    "traumatizas": "traumatizar",
    "traumatizase": "traumatizar",
    "traumatizaseis": "traumatizar",
    "traumatizasen": "traumatizar",
    "traumatizases": "traumatizar",
    "traumatizaste": "traumatizar",
    "traumatizasteis": "traumatizar",
    "traumatizo": "traumatizar",
    "traumatizábamos": "traumatizar",
    "traumatizáis": "traumatizar",
    "traumatizáramos": "traumatizar",
    "traumatizáremos": "traumatizar",
    "traumatizásemos": "traumatizar",
    "traumatizó": "traumatizar",
    "traumatologías": "traumatología",
    "traumatológica": "traumatológico",
    "traumatológicas": "traumatológico",
    "traumatológicos": "traumatológico",
    "traumatóloga": "traumatólogo",
    "traumatólogas": "traumatólogo",
    "traumatólogos": "traumatólogo",
    "traumática": "traumático",
    "traumáticas": "traumático",
    "traumáticos": "traumático",
    "travelines": "travelín",
    "travelings": "traveling",
    "travellings": "travelling",
    "traversas": "traversa",
    "traversos": "traverso",
    "travertinos": "travertino",
    "travesaños": "travesaño",
    "travesea": "travesear",
    "traveseaba": "travesear",
    "traveseabais": "travesear",
    "traveseaban": "travesear",
    "traveseabas": "travesear",
    "travesead": "travesear",
    "traveseada": "travesear",
    "traveseadas": "travesear",
    "traveseado": "travesear",
    "traveseados": "travesear",
    "traveseamos": "travesear",
    "travesean": "travesear",
    "traveseando": "travesear",
    "traveseara": "travesear",
    "travesearais": "travesear",
    "travesearan": "travesear",
    "travesearas": "travesear",
    "traveseare": "travesear",
    "traveseareis": "travesear",
    "travesearemos": "travesear",
    "travesearen": "travesear",
    "traveseares": "travesear",
    "travesearon": "travesear",
    "traveseará": "travesear",
    "travesearán": "travesear",
    "travesearás": "travesear",
    "travesearé": "travesear",
    "travesearéis": "travesear",
    "travesearía": "travesear",
    "travesearíais": "travesear",
    "travesearíamos": "travesear",
    "travesearían": "travesear",
    "travesearías": "travesear",
    "traveseas": "travesear",
    "travesease": "travesear",
    "traveseaseis": "travesear",
    "traveseasen": "travesear",
    "traveseases": "travesear",
    "traveseaste": "travesear",
    "traveseasteis": "travesear",
    "travesee": "travesear",
    "traveseemos": "travesear",
    "traveseen": "travesear",
    "travesees": "travesear",
    "traveseo": "travesear",
    "travesera": "travesero",
    "traveseras": "travesero",
    "traveseros": "travesero",
    "traveses": "través",
    "traveseábamos": "travesear",
    "traveseáis": "travesear",
    "traveseáramos": "travesear",
    "traveseáremos": "travesear",
    "traveseásemos": "travesear",
    "traveseé": "travesear",
    "traveseéis": "travesear",
    "traveseó": "travesear",
    "travestid": "travestir",
    "travestida": "travestir",
    "travestidas": "travestir",
    "travestido": "travestir",
    "travestidos": "travestir",
    "travesties": "travestí",
    "travestimos": "travestir",
    "travestiremos": "travestir",
    "travestirá": "travestir",
    "travestirán": "travestir",
    "travestirás": "travestir",
    "travestiré": "travestir",
    "travestiréis": "travestir",
    "travestiría": "travestir",
    "travestiríais": "travestir",
    "travestiríamos": "travestir",
    "travestirían": "travestir",
    "travestirías": "travestir",
    "travestis": "travesti",
    "travestismos": "travestismo",
    "travestiste": "travestir",
    "travestisteis": "travestir",
    "travestí": "travestir",
    "travestía": "travestir",
    "travestíais": "travestir",
    "travestíamos": "travestir",
    "travestían": "travestir",
    "travestías": "travestir",
    "travestíes": "travestí",
    "travestís": "travestir",
    "travesuras": "travesura",
    "travesía": "travesío",
    "travesías": "travesío",
    "travesíos": "travesío",
    "traviesa": "travieso",
    "traviesas": "travieso",
    "traviesos": "travieso",
    "travista": "travestir",
    "travistamos": "travestir",
    "travistan": "travestir",
    "travistas": "travestir",
    "traviste": "travestir",
    "travisten": "travestir",
    "travistes": "travestir",
    "travistiendo": "travestir",
    "travistiera": "travestir",
    "travistierais": "travestir",
    "travistieran": "travestir",
    "travistieras": "travestir",
    "travistiere": "travestir",
    "travistiereis": "travestir",
    "travistieren": "travestir",
    "travistieres": "travestir",
    "travistieron": "travestir",
    "travistiese": "travestir",
    "travistieseis": "travestir",
    "travistiesen": "travestir",
    "travistieses": "travestir",
    "travistiéramos": "travestir",
    "travistiéremos": "travestir",
    "travistiésemos": "travestir",
    "travistió": "travestir",
    "travisto": "travestir",
    "travistáis": "travestir",
    "trayectorias": "trayectoria",
    "trayectos": "trayecto",
    "trayendo": "traer",
    "traza": "trazar",
    "trazaba": "trazar",
    "trazabais": "trazar",
    "trazaban": "trazar",
    "trazabas": "trazar",
    "trazabilidades": "trazabilidad",
    "trazables": "trazable",
    "trazad": "trazar",
    "trazada": "trazar",
    "trazadas": "trazar",
    "trazado": "trazar",
    "trazadora": "trazador",
    "trazadoras": "trazadora",
    "trazadores": "trazador",
    "trazados": "trazar",
    "trazamos": "trazar",
    "trazan": "trazar",
    "trazando": "trazar",
    "trazara": "trazar",
    "trazarais": "trazar",
    "trazaran": "trazar",
    "trazaras": "trazar",
    "trazare": "trazar",
    "trazareis": "trazar",
    "trazaremos": "trazar",
    "trazaren": "trazar",
    "trazares": "trazar",
    "trazaron": "trazar",
    "trazará": "trazar",
    "trazarán": "trazar",
    "trazarás": "trazar",
    "trazaré": "trazar",
    "trazaréis": "trazar",
    "trazaría": "trazar",
    "trazaríais": "trazar",
    "trazaríamos": "trazar",
    "trazarían": "trazar",
    "trazarías": "trazar",
    "trazas": "trazar",
    "trazase": "trazar",
    "trazaseis": "trazar",
    "trazasen": "trazar",
    "trazases": "trazar",
    "trazaste": "trazar",
    "trazasteis": "trazar",
    "trazo": "trazar",
    "trazos": "trazo",
    "trazuma": "trazumar",
    "trazumaba": "trazumar",
    "trazumabais": "trazumar",
    "trazumaban": "trazumar",
    "trazumabas": "trazumar",
    "trazumad": "trazumar",
    "trazumada": "trazumar",
    "trazumadas": "trazumar",
    "trazumado": "trazumar",
    "trazumados": "trazumar",
    "trazumamos": "trazumar",
    "trazuman": "trazumar",
    "trazumando": "trazumar",
    "trazumara": "trazumar",
    "trazumarais": "trazumar",
    "trazumaran": "trazumar",
    "trazumaras": "trazumar",
    "trazumare": "trazumar",
    "trazumareis": "trazumar",
    "trazumaremos": "trazumar",
    "trazumaren": "trazumar",
    "trazumares": "trazumar",
    "trazumaron": "trazumar",
    "trazumará": "trazumar",
    "trazumarán": "trazumar",
    "trazumarás": "trazumar",
    "trazumaré": "trazumar",
    "trazumaréis": "trazumar",
    "trazumaría": "trazumar",
    "trazumaríais": "trazumar",
    "trazumaríamos": "trazumar",
    "trazumarían": "trazumar",
    "trazumarías": "trazumar",
    "trazumas": "trazumar",
    "trazumase": "trazumar",
    "trazumaseis": "trazumar",
    "trazumasen": "trazumar",
    "trazumases": "trazumar",
    "trazumaste": "trazumar",
    "trazumasteis": "trazumar",
    "trazume": "trazumar",
    "trazumemos": "trazumar",
    "trazumen": "trazumar",
    "trazumes": "trazumar",
    "trazumo": "trazumar",
    "trazumábamos": "trazumar",
    "trazumáis": "trazumar",
    "trazumáramos": "trazumar",
    "trazumáremos": "trazumar",
    "trazumásemos": "trazumar",
    "trazumé": "trazumar",
    "trazuméis": "trazumar",
    "trazumó": "trazumar",
    "trazábamos": "trazar",
    "trazáis": "trazar",
    "trazáramos": "trazar",
    "trazáremos": "trazar",
    "trazásemos": "trazar",
    "trazó": "trazar",
    "traéis": "traer",
    "traía": "traer",
    "traíais": "traer",
    "traíamos": "traer",
    "traían": "traer",
    "traías": "traer",
    "traída": "traer",
    "traídas": "traer",
    "traído": "traer",
    "traídos": "traer",
    "traílla": "traillar",
    "traíllan": "traillar",
    "traíllas": "traillar",
    "traílle": "traillar",
    "traíllen": "traillar",
    "traílles": "traillar",
    "traíllo": "traillar",
    "traínas": "traína",
    "traíñas": "traíña",
    "trebejos": "trebejo",
    "trebeliánica": "trebeliánico",
    "trebeliánicas": "trebeliánico",
    "trebeliánicos": "trebeliánico",
    "trebolada": "trebolado",
    "treboladas": "trebolado",
    "trebolados": "trebolado",
    "trebolares": "trebolar",
    "treceava": "treceavo",
    "treceavas": "treceavo",
    "treceavo": "13",
    "treceavos": "treceavo",
    "trecenarios": "trecenario",
    "trecentistas": "trecentista",
    "trecentos": "trecento",
    "trechas": "trecha",
    "trecheadores": "trecheador",
    "trecheos": "trecheo",
    "trechores": "trechor",
    "trechos": "trecho",
    "trecésima": "13",
    "trecésimas": "13",
    "trecésimo": "13",
    "trecésimos": "13",
    "trefes": "trefe",
    "trefila": "trefilar",
    "trefilaba": "trefilar",
    "trefilabais": "trefilar",
    "trefilaban": "trefilar",
    "trefilabas": "trefilar",
    "trefilad": "trefilar",
    "trefilada": "trefilar",
    "trefiladas": "trefilar",
    "trefilado": "trefilar",
    "trefiladora": "trefilador",
    "trefiladoras": "trefiladora",
    "trefiladores": "trefilador",
    "trefilados": "trefilar",
    "trefilamos": "trefilar",
    "trefilan": "trefilar",
    "trefilando": "trefilar",
    "trefilara": "trefilar",
    "trefilarais": "trefilar",
    "trefilaran": "trefilar",
    "trefilaras": "trefilar",
    "trefilare": "trefilar",
    "trefilareis": "trefilar",
    "trefilaremos": "trefilar",
    "trefilaren": "trefilar",
    "trefilares": "trefilar",
    "trefilaron": "trefilar",
    "trefilará": "trefilar",
    "trefilarán": "trefilar",
    "trefilarás": "trefilar",
    "trefilaré": "trefilar",
    "trefilaréis": "trefilar",
    "trefilaría": "trefilar",
    "trefilaríais": "trefilar",
    "trefilaríamos": "trefilar",
    "trefilarían": "trefilar",
    "trefilarías": "trefilar",
    "trefilas": "trefilar",
    "trefilase": "trefilar",
    "trefilaseis": "trefilar",
    "trefilasen": "trefilar",
    "trefilases": "trefilar",
    "trefilaste": "trefilar",
    "trefilasteis": "trefilar",
    "trefile": "trefilar",
    "trefilemos": "trefilar",
    "trefilen": "trefilar",
    "trefilerías": "trefilería",
    "trefiles": "trefilar",
    "trefilo": "trefilar",
    "trefilábamos": "trefilar",
    "trefiláis": "trefilar",
    "trefiláramos": "trefilar",
    "trefiláremos": "trefilar",
    "trefilásemos": "trefilar",
    "trefilé": "trefilar",
    "trefiléis": "trefilar",
    "trefiló": "trefilar",
    "treguas": "tregua",
    "treintaicincoava": "treintaicincoavo",
    "treintaicincoavas": "treintaicincoavo",
    "treintaicincoavo": "35",
    "treintaicincoavos": "treintaicincoavo",
    "treintaicuatroava": "treintaicuatroavo",
    "treintaicuatroavas": "treintaicuatroavo",
    "treintaicuatroavo": "34",
    "treintaicuatroavos": "treintaicuatroavo",
    "treintaidosava": "treintaidosavo",
    "treintaidosavas": "treintaidosavo",
    "treintaidosavo": "32",
    "treintaidosavos": "treintaidosavo",
    "treintainueveava": "treintainueveavo",
    "treintainueveavas": "treintainueveavo",
    "treintainueveavo": "39",
    "treintainueveavos": "treintainueveavo",
    "treintaiochava": "treintaiochoavo",
    "treintaiochavas": "treintaiochoavo",
    "treintaiochavo": "treintaiochoavo",
    "treintaiochavos": "treintaiochoavo",
    "treintaiochoavas": "treintaiochoavo",
    "treintaiochoavo": "38",
    "treintaiochoavos": "treintaiochoavo",
    "treintaiseisava": "treintaiseisavo",
    "treintaiseisavas": "treintaiseisavo",
    "treintaiseisavo": "36",
    "treintaiseisavos": "treintaiseisavo",
    "treintaisieteava": "treintaisieteavo",
    "treintaisieteavas": "treintaisieteavo",
    "treintaisieteavo": "37",
    "treintaisieteavos": "treintaisieteavo",
    "treintaitresava": "treintaitresavo",
    "treintaitresavas": "treintaitresavo",
    "treintaitresavo": "33",
    "treintaitresavos": "treintaitresavo",
    "treintaiunava": "treintaiunavo",
    "treintaiunavas": "treintaiunavo",
    "treintaiunavo": "31",
    "treintaiunavos": "treintaiunavo",
    "treintanarios": "treintanario",
    "treintava": "treintavo",
    "treintavas": "treintavo",
    "treintavo": "30",
    "treintavos": "treintavo",
    "treintañales": "treintañal",
    "treintenas": "treintena",
    "trejas": "treja",
    "trekkings": "trekking",
    "tremadales": "tremadal",
    "trematodos": "trematodo",
    "tremebunda": "tremebundo",
    "tremebundas": "tremebundo",
    "tremebundos": "tremebundo",
    "tremedales": "tremedal",
    "tremenda": "tremendo",
    "tremendas": "tremendo",
    "tremendos": "tremendo",
    "trementinas": "trementina",
    "tremeses": "tremés",
    "tremesina": "tremesino",
    "tremesinas": "tremesino",
    "tremesinos": "tremesino",
    "tremielgas": "tremielga",
    "tremola": "tremolar",
    "tremolaba": "tremolar",
    "tremolabais": "tremolar",
    "tremolaban": "tremolar",
    "tremolabas": "tremolar",
    "tremolad": "tremolar",
    "tremolada": "tremolar",
    "tremoladas": "tremolar",
    "tremolado": "tremolar",
    "tremolados": "tremolar",
    "tremolamos": "tremolar",
    "tremolan": "tremolar",
    "tremolando": "tremolar",
    "tremolara": "tremolar",
    "tremolarais": "tremolar",
    "tremolaran": "tremolar",
    "tremolaras": "tremolar",
    "tremolare": "tremolar",
    "tremolareis": "tremolar",
    "tremolaremos": "tremolar",
    "tremolaren": "tremolar",
    "tremolares": "tremolar",
    "tremolaron": "tremolar",
    "tremolará": "tremolar",
    "tremolarán": "tremolar",
    "tremolarás": "tremolar",
    "tremolaré": "tremolar",
    "tremolaréis": "tremolar",
    "tremolaría": "tremolar",
    "tremolaríais": "tremolar",
    "tremolaríamos": "tremolar",
    "tremolarían": "tremolar",
    "tremolarías": "tremolar",
    "tremolas": "tremolar",
    "tremolase": "tremolar",
    "tremolaseis": "tremolar",
    "tremolasen": "tremolar",
    "tremolases": "tremolar",
    "tremolaste": "tremolar",
    "tremolasteis": "tremolar",
    "tremole": "tremolar",
    "tremolemos": "tremolar",
    "tremolen": "tremolar",
    "tremoles": "tremolar",
    "tremolinas": "tremolina",
    "tremolitas": "tremolita",
    "tremolo": "tremolar",
    "tremolábamos": "tremolar",
    "tremoláis": "tremolar",
    "tremoláramos": "tremolar",
    "tremoláremos": "tremolar",
    "tremolásemos": "tremolar",
    "tremolé": "tremolar",
    "tremoléis": "tremolar",
    "tremoló": "tremolar",
    "tremores": "tremor",
    "trempabilidades": "trempabilidad",
    "trempables": "trempable",
    "tremulantes": "tremulante",
    "tremulenta": "tremulento",
    "tremulentas": "tremulento",
    "tremulentos": "tremulento",
    "tremérica": "tremérico",
    "treméricas": "tremérico",
    "treméricos": "tremérico",
    "tremós": "tremó",
    "trenada": "trenado",
    "trenadas": "trenado",
    "trenados": "trenado",
    "trencas": "trenca",
    "trence": "trenzar",
    "trencemos": "trenzar",
    "trencen": "trenzar",
    "trences": "trenzar",
    "trencilla": "trencillar",
    "trencillaba": "trencillar",
    "trencillabais": "trencillar",
    "trencillaban": "trencillar",
    "trencillabas": "trencillar",
    "trencillad": "trencillar",
    "trencillada": "trencillar",
    "trencilladas": "trencillar",
    "trencillado": "trencillar",
    "trencillados": "trencillar",
    "trencillamos": "trencillar",
    "trencillan": "trencillar",
    "trencillando": "trencillar",
    "trencillara": "trencillar",
    "trencillarais": "trencillar",
    "trencillaran": "trencillar",
    "trencillaras": "trencillar",
    "trencillare": "trencillar",
    "trencillareis": "trencillar",
    "trencillaremos": "trencillar",
    "trencillaren": "trencillar",
    "trencillares": "trencillar",
    "trencillaron": "trencillar",
    "trencillará": "trencillar",
    "trencillarán": "trencillar",
    "trencillarás": "trencillar",
    "trencillaré": "trencillar",
    "trencillaréis": "trencillar",
    "trencillaría": "trencillar",
    "trencillaríais": "trencillar",
    "trencillaríamos": "trencillar",
    "trencillarían": "trencillar",
    "trencillarías": "trencillar",
    "trencillas": "trencillar",
    "trencillase": "trencillar",
    "trencillaseis": "trencillar",
    "trencillasen": "trencillar",
    "trencillases": "trencillar",
    "trencillaste": "trencillar",
    "trencillasteis": "trencillar",
    "trencille": "trencillar",
    "trencillemos": "trencillar",
    "trencillen": "trencillar",
    "trencilles": "trencillar",
    "trencillo": "trencillar",
    "trencillos": "trencillo",
    "trencillábamos": "trencillar",
    "trencilláis": "trencillar",
    "trencilláramos": "trencillar",
    "trencilláremos": "trencillar",
    "trencillásemos": "trencillar",
    "trencillé": "trencillar",
    "trencilléis": "trencillar",
    "trencilló": "trencillar",
    "trencé": "trenzar",
    "trencéis": "trenzar",
    "trenes": "tren",
    "trenos": "treno",
    "trenza": "trenzar",
    "trenzaba": "trenzar",
    "trenzabais": "trenzar",
    "trenzaban": "trenzar",
    "trenzabas": "trenzar",
    "trenzad": "trenzar",
    "trenzada": "trenzar",
    "trenzadas": "trenzar",
    "trenzado": "trenzar",
    "trenzados": "trenzar",
    "trenzamos": "trenzar",
    "trenzan": "trenzar",
    "trenzando": "trenzar",
    "trenzara": "trenzar",
    "trenzarais": "trenzar",
    "trenzaran": "trenzar",
    "trenzaras": "trenzar",
    "trenzare": "trenzar",
    "trenzareis": "trenzar",
    "trenzaremos": "trenzar",
    "trenzaren": "trenzar",
    "trenzares": "trenzar",
    "trenzaron": "trenzar",
    "trenzará": "trenzar",
    "trenzarán": "trenzar",
    "trenzarás": "trenzar",
    "trenzaré": "trenzar",
    "trenzaréis": "trenzar",
    "trenzaría": "trenzar",
    "trenzaríais": "trenzar",
    "trenzaríamos": "trenzar",
    "trenzarían": "trenzar",
    "trenzarías": "trenzar",
    "trenzas": "trenzar",
    "trenzase": "trenzar",
    "trenzaseis": "trenzar",
    "trenzasen": "trenzar",
    "trenzases": "trenzar",
    "trenzaste": "trenzar",
    "trenzasteis": "trenzar",
    "trenzo": "trenzar",
    "trenzábamos": "trenzar",
    "trenzáis": "trenzar",
    "trenzáramos": "trenzar",
    "trenzáremos": "trenzar",
    "trenzásemos": "trenzar",
    "trenzó": "trenzar",
    "treoninas": "treonina",
    "treos": "treo",
    "trepa": "trepar",
    "trepaba": "trepar",
    "trepabais": "trepar",
    "trepaban": "trepar",
    "trepabas": "trepar",
    "trepad": "trepar",
    "trepada": "trepar",
    "trepadas": "trepar",
    "trepado": "trepar",
    "trepadora": "trepador",
    "trepadoras": "trepador",
    "trepadores": "trepador",
    "trepados": "trepar",
    "trepamos": "trepar",
    "trepan": "trepar",
    "trepana": "trepanar",
    "trepanaba": "trepanar",
    "trepanabais": "trepanar",
    "trepanaban": "trepanar",
    "trepanabas": "trepanar",
    "trepanaciones": "trepanación",
    "trepanad": "trepanar",
    "trepanada": "trepanar",
    "trepanadas": "trepanar",
    "trepanado": "trepanar",
    "trepanados": "trepanar",
    "trepanamos": "trepanar",
    "trepanan": "trepanar",
    "trepanando": "trepanar",
    "trepanara": "trepanar",
    "trepanarais": "trepanar",
    "trepanaran": "trepanar",
    "trepanaras": "trepanar",
    "trepanare": "trepanar",
    "trepanareis": "trepanar",
    "trepanaremos": "trepanar",
    "trepanaren": "trepanar",
    "trepanares": "trepanar",
    "trepanaron": "trepanar",
    "trepanará": "trepanar",
    "trepanarán": "trepanar",
    "trepanarás": "trepanar",
    "trepanaré": "trepanar",
    "trepanaréis": "trepanar",
    "trepanaría": "trepanar",
    "trepanaríais": "trepanar",
    "trepanaríamos": "trepanar",
    "trepanarían": "trepanar",
    "trepanarías": "trepanar",
    "trepanas": "trepanar",
    "trepanase": "trepanar",
    "trepanaseis": "trepanar",
    "trepanasen": "trepanar",
    "trepanases": "trepanar",
    "trepanaste": "trepanar",
    "trepanasteis": "trepanar",
    "trepando": "trepar",
    "trepane": "trepanar",
    "trepanemos": "trepanar",
    "trepanen": "trepanar",
    "trepanes": "trepanar",
    "trepano": "trepanar",
    "trepanábamos": "trepanar",
    "trepanáis": "trepanar",
    "trepanáramos": "trepanar",
    "trepanáremos": "trepanar",
    "trepanásemos": "trepanar",
    "trepané": "trepanar",
    "trepanéis": "trepanar",
    "trepanó": "trepanar",
    "trepara": "trepar",
    "treparais": "trepar",
    "treparan": "trepar",
    "treparas": "trepar",
    "trepare": "trepar",
    "trepareis": "trepar",
    "treparemos": "trepar",
    "treparen": "trepar",
    "trepares": "trepar",
    "treparon": "trepar",
    "trepará": "trepar",
    "treparán": "trepar",
    "treparás": "trepar",
    "treparé": "trepar",
    "treparéis": "trepar",
    "treparía": "trepar",
    "treparíais": "trepar",
    "treparíamos": "trepar",
    "treparían": "trepar",
    "treparías": "trepar",
    "trepas": "trepar",
    "trepase": "trepar",
    "trepaseis": "trepar",
    "trepasen": "trepar",
    "trepases": "trepar",
    "trepaste": "trepar",
    "trepasteis": "trepar",
    "trepe": "trepar",
    "trepemos": "trepar",
    "trepen": "trepar",
    "trepes": "trepar",
    "trepida": "trepidar",
    "trepidaba": "trepidar",
    "trepidabais": "trepidar",
    "trepidaban": "trepidar",
    "trepidabas": "trepidar",
    "trepidaciones": "trepidación",
    "trepidad": "trepidar",
    "trepidada": "trepidar",
    "trepidadas": "trepidar",
    "trepidado": "trepidar",
    "trepidados": "trepidar",
    "trepidamos": "trepidar",
    "trepidan": "trepidar",
    "trepidando": "trepidar",
    "trepidantes": "trepidante",
    "trepidara": "trepidar",
    "trepidarais": "trepidar",
    "trepidaran": "trepidar",
    "trepidaras": "trepidar",
    "trepidare": "trepidar",
    "trepidareis": "trepidar",
    "trepidaremos": "trepidar",
    "trepidaren": "trepidar",
    "trepidares": "trepidar",
    "trepidaron": "trepidar",
    "trepidará": "trepidar",
    "trepidarán": "trepidar",
    "trepidarás": "trepidar",
    "trepidaré": "trepidar",
    "trepidaréis": "trepidar",
    "trepidaría": "trepidar",
    "trepidaríais": "trepidar",
    "trepidaríamos": "trepidar",
    "trepidarían": "trepidar",
    "trepidarías": "trepidar",
    "trepidas": "trepidar",
    "trepidase": "trepidar",
    "trepidaseis": "trepidar",
    "trepidasen": "trepidar",
    "trepidases": "trepidar",
    "trepidaste": "trepidar",
    "trepidasteis": "trepidar",
    "trepide": "trepidar",
    "trepidemos": "trepidar",
    "trepiden": "trepidar",
    "trepides": "trepidar",
    "trepido": "trepidar",
    "trepidábamos": "trepidar",
    "trepidáis": "trepidar",
    "trepidáramos": "trepidar",
    "trepidáremos": "trepidar",
    "trepidásemos": "trepidar",
    "trepidé": "trepidar",
    "trepidéis": "trepidar",
    "trepidó": "trepidar",
    "trepo": "trepar",
    "treponemas": "treponema",
    "trepábamos": "trepar",
    "trepáis": "trepar",
    "trepáramos": "trepar",
    "trepáremos": "trepar",
    "trepásemos": "trepar",
    "trepé": "trepar",
    "trepéis": "trepar",
    "trepó": "trepar",
    "tresados": "tresado",
    "tresalba": "tresalbo",
    "tresalbas": "tresalbo",
    "tresalbos": "tresalbo",
    "tresañales": "tresañal",
    "tresañeja": "tresañejo",
    "tresañejas": "tresañejo",
    "tresañejos": "tresañejo",
    "trescientosava": "trescientosavo",
    "trescientosavas": "trescientosavo",
    "trescientosavo": "300",
    "trescientosavos": "trescientosavo",
    "tresdobla": "tresdoblar",
    "tresdoblaba": "tresdoblar",
    "tresdoblabais": "tresdoblar",
    "tresdoblaban": "tresdoblar",
    "tresdoblabas": "tresdoblar",
    "tresdoblad": "tresdoblar",
    "tresdoblada": "tresdoblar",
    "tresdobladas": "tresdoblar",
    "tresdoblado": "tresdoblar",
    "tresdoblados": "tresdoblar",
    "tresdoblamos": "tresdoblar",
    "tresdoblan": "tresdoblar",
    "tresdoblando": "tresdoblar",
    "tresdoblara": "tresdoblar",
    "tresdoblarais": "tresdoblar",
    "tresdoblaran": "tresdoblar",
    "tresdoblaras": "tresdoblar",
    "tresdoblare": "tresdoblar",
    "tresdoblareis": "tresdoblar",
    "tresdoblaremos": "tresdoblar",
    "tresdoblaren": "tresdoblar",
    "tresdoblares": "tresdoblar",
    "tresdoblaron": "tresdoblar",
    "tresdoblará": "tresdoblar",
    "tresdoblarán": "tresdoblar",
    "tresdoblarás": "tresdoblar",
    "tresdoblaré": "tresdoblar",
    "tresdoblaréis": "tresdoblar",
    "tresdoblaría": "tresdoblar",
    "tresdoblaríais": "tresdoblar",
    "tresdoblaríamos": "tresdoblar",
    "tresdoblarían": "tresdoblar",
    "tresdoblarías": "tresdoblar",
    "tresdoblas": "tresdoblar",
    "tresdoblase": "tresdoblar",
    "tresdoblaseis": "tresdoblar",
    "tresdoblasen": "tresdoblar",
    "tresdoblases": "tresdoblar",
    "tresdoblaste": "tresdoblar",
    "tresdoblasteis": "tresdoblar",
    "tresdoble": "tresdoblar",
    "tresdoblemos": "tresdoblar",
    "tresdoblen": "tresdoblar",
    "tresdobles": "tresdoble",
    "tresdoblo": "tresdoblar",
    "tresdoblábamos": "tresdoblar",
    "tresdobláis": "tresdoblar",
    "tresdobláramos": "tresdoblar",
    "tresdobláremos": "tresdoblar",
    "tresdoblásemos": "tresdoblar",
    "tresdoblé": "tresdoblar",
    "tresdobléis": "tresdoblar",
    "tresdobló": "tresdoblar",
    "tresillistas": "tresillista",
    "tresillos": "tresillo",
    "tresmesina": "tresmesino",
    "tresmesinas": "tresmesino",
    "tresmesinos": "tresmesino",
    "trespuntada": "trespuntado",
    "trespuntadas": "trespuntado",
    "trespuntados": "trespuntado",
    "tretas": "treta",
    "triaba": "triar",
    "triabais": "triar",
    "triaban": "triar",
    "triabas": "triar",
    "triacales": "triacal",
    "triacas": "triaca",
    "triacetatos": "triacetato",
    "triacetiloleandomicinas": "triacetiloleandomicina",
    "triacetilricinoleatos": "triacetilricinoleato",
    "triacetinas": "triacetina",
    "triacetoxiestearatos": "triacetoxiestearato",
    "triaches": "triache",
    "triacilgliceroles": "triacilglicerol",
    "triacilglicéridos": "triacilglicérido",
    "triacs": "triac",
    "triad": "triar",
    "triada": "triar",
    "triadas": "triar",
    "triado": "triar",
    "triados": "triar",
    "triajes": "triaje",
    "triales": "trial",
    "triamidas": "triamida",
    "triamilaminas": "triamilamina",
    "triaminotrifenilmetanos": "triaminotrifenilmetano",
    "triamos": "triar",
    "triando": "triar",
    "triangula": "triangular",
    "triangulaba": "triangular",
    "triangulabais": "triangular",
    "triangulaban": "triangular",
    "triangulabas": "triangular",
    "triangulaciones": "triangulación",
    "triangulad": "triangular",
    "triangulada": "triangular",
    "trianguladas": "triangular",
    "triangulado": "triangular",
    "triangulados": "triangular",
    "triangulamos": "triangular",
    "triangulan": "triangular",
    "triangulando": "triangular",
    "triangulara": "triangular",
    "triangularais": "triangular",
    "triangularan": "triangular",
    "triangularas": "triangular",
    "triangulare": "triangular",
    "triangulareis": "triangular",
    "triangularemos": "triangular",
    "triangularen": "triangular",
    "triangulares": "triangular",
    "triangularon": "triangular",
    "triangulará": "triangular",
    "triangularán": "triangular",
    "triangularás": "triangular",
    "triangularé": "triangular",
    "triangularéis": "triangular",
    "triangularía": "triangular",
    "triangularíais": "triangular",
    "triangularíamos": "triangular",
    "triangularían": "triangular",
    "triangularías": "triangular",
    "triangulas": "triangular",
    "triangulase": "triangular",
    "triangulaseis": "triangular",
    "triangulasen": "triangular",
    "triangulases": "triangular",
    "triangulaste": "triangular",
    "triangulasteis": "triangular",
    "triangule": "triangular",
    "triangulemos": "triangular",
    "triangulen": "triangular",
    "triangules": "triangular",
    "triangulo": "triangular",
    "triangulábamos": "triangular",
    "trianguláis": "triangular",
    "trianguláramos": "triangular",
    "trianguláremos": "triangular",
    "triangulásemos": "triangular",
    "triangulé": "triangular",
    "trianguléis": "triangular",
    "trianguló": "triangular",
    "triaqueses": "triac",
    "triara": "triar",
    "triarais": "triar",
    "triaran": "triar",
    "triaras": "triar",
    "triare": "triar",
    "triareis": "triar",
    "triaremos": "triar",
    "triaren": "triar",
    "triares": "triar",
    "triarios": "triario",
    "triaron": "triar",
    "triará": "triar",
    "triarán": "triar",
    "triarás": "triar",
    "triaré": "triar",
    "triaréis": "triar",
    "triaría": "triar",
    "triaríais": "triar",
    "triaríamos": "triar",
    "triarían": "triar",
    "triarías": "triar",
    "triase": "triar",
    "triaseis": "triar",
    "triasen": "triar",
    "triases": "triar",
    "triaste": "triar",
    "triasteis": "triar",
    "triatlones": "triatlón",
    "triatómica": "triatómico",
    "triatómicas": "triatómico",
    "triatómicos": "triatómico",
    "triaxiales": "triaxial",
    "tribales": "tribal",
    "tribalismos": "tribalismo",
    "triboelectricidades": "triboelectricidad",
    "triboluminiscencias": "triboluminiscencia",
    "tribopulimentos": "tribopulimento",
    "tribraquios": "tribraquio",
    "tribromoacetaldehídos": "tribromoacetaldehído",
    "tribromoacética": "tribromoacético",
    "tribromoacéticas": "tribromoacético",
    "tribromoacéticos": "tribromoacético",
    "tribromoetanoles": "tribromoetanol",
    "tribromuros": "tribromuro",
    "tribuales": "tribual",
    "tribuid": "tribuir",
    "tribuida": "tribuir",
    "tribuidas": "tribuir",
    "tribuido": "tribuir",
    "tribuidos": "tribuir",
    "tribuimos": "tribuir",
    "tribuiremos": "tribuir",
    "tribuirá": "tribuir",
    "tribuirán": "tribuir",
    "tribuirás": "tribuir",
    "tribuiré": "tribuir",
    "tribuiréis": "tribuir",
    "tribuiría": "tribuir",
    "tribuiríais": "tribuir",
    "tribuiríamos": "tribuir",
    "tribuirían": "tribuir",
    "tribuirías": "tribuir",
    "tribuiste": "tribuir",
    "tribuisteis": "tribuir",
    "tribulaciones": "tribulación",
    "tribunados": "tribunado",
    "tribunales": "tribunal",
    "tribunas": "tribuna",
    "tribunicia": "tribunicio",
    "tribunicias": "tribunicio",
    "tribunicios": "tribunicio",
    "tribunos": "tribuno",
    "tribus": "tribu",
    "tributa": "tributar",
    "tributaba": "tributar",
    "tributabais": "tributar",
    "tributaban": "tributar",
    "tributabas": "tributar",
    "tributables": "tributable",
    "tributaciones": "tributación",
    "tributad": "tributar",
    "tributada": "tributar",
    "tributadas": "tributar",
    "tributado": "tributar",
    "tributados": "tributar",
    "tributamos": "tributar",
    "tributan": "tributar",
    "tributando": "tributar",
    "tributantes": "tributante",
    "tributara": "tributar",
    "tributarais": "tributar",
    "tributaran": "tributar",
    "tributaras": "tributar",
    "tributare": "tributar",
    "tributareis": "tributar",
    "tributaremos": "tributar",
    "tributaren": "tributar",
    "tributares": "tributar",
    "tributaria": "tributario",
    "tributarias": "tributario",
    "tributarios": "tributario",
    "tributaron": "tributar",
    "tributará": "tributar",
    "tributarán": "tributar",
    "tributarás": "tributar",
    "tributaré": "tributar",
    "tributaréis": "tributar",
    "tributaría": "tributar",
    "tributaríais": "tributar",
    "tributaríamos": "tributar",
    "tributarían": "tributar",
    "tributarías": "tributar",
    "tributas": "tributar",
    "tributase": "tributar",
    "tributaseis": "tributar",
    "tributasen": "tributar",
    "tributases": "tributar",
    "tributaste": "tributar",
    "tributasteis": "tributar",
    "tribute": "tributar",
    "tributemos": "tributar",
    "tributen": "tributar",
    "tributes": "tributar",
    "tributilaminas": "tributilamina",
    "tributiratos": "tributirato",
    "tributo": "tributar",
    "tributos": "tributo",
    "tributábamos": "tributar",
    "tributáis": "tributar",
    "tributáramos": "tributar",
    "tributáremos": "tributar",
    "tributásemos": "tributar",
    "tributé": "tributar",
    "tributéis": "tributar",
    "tributó": "tributar",
    "tribuya": "tribuir",
    "tribuyamos": "tribuir",
    "tribuyan": "tribuir",
    "tribuyas": "tribuir",
    "tribuye": "tribuir",
    "tribuyen": "tribuir",
    "tribuyendo": "tribuir",
    "tribuyera": "tribuir",
    "tribuyerais": "tribuir",
    "tribuyeran": "tribuir",
    "tribuyeras": "tribuir",
    "tribuyere": "tribuir",
    "tribuyereis": "tribuir",
    "tribuyeren": "tribuir",
    "tribuyeres": "tribuir",
    "tribuyeron": "tribuir",
    "tribuyes": "tribuir",
    "tribuyese": "tribuir",
    "tribuyeseis": "tribuir",
    "tribuyesen": "tribuir",
    "tribuyeses": "tribuir",
    "tribuyo": "tribuir",
    "tribuyáis": "tribuir",
    "tribuyéramos": "tribuir",
    "tribuyéremos": "tribuir",
    "tribuyésemos": "tribuir",
    "tribuyó": "tribuir",
    "tribuí": "tribuir",
    "tribuía": "tribuir",
    "tribuíais": "tribuir",
    "tribuíamos": "tribuir",
    "tribuían": "tribuir",
    "tribuías": "tribuir",
    "tribuís": "tribuir",
    "tribásica": "tribásico",
    "tribásicas": "tribásico",
    "tribásicos": "tribásico",
    "tribómetros": "tribómetro",
    "tribúnica": "tribúnico",
    "tribúnicas": "tribúnico",
    "tribúnicos": "tribúnico",
    "tricampeona": "tricampeón",
    "tricampeones": "tricampeón",
    "tricarbalilatos": "tricarbalilato",
    "trice": "trizar",
    "tricemos": "trizar",
    "tricen": "trizar",
    "tricenales": "tricenal",
    "tricentenaria": "tricentenario",
    "tricentenarias": "tricentenario",
    "tricentenarios": "tricentenario",
    "tricentésima": "300",
    "tricentésimas": "300",
    "tricentésimo": "300",
    "tricentésimos": "300",
    "trices": "trizar",
    "triciclos": "triciclo",
    "triclinios": "triclinio",
    "tricloroacetatos": "tricloroacetato",
    "tricloroacética": "tricloroacético",
    "tricloroacéticas": "tricloroacético",
    "tricloroacéticos": "tricloroacético",
    "triclorobencenos": "triclorobenceno",
    "tricloroborazoles": "tricloroborazol",
    "triclorocarbanilidas": "triclorocarbanilida",
    "tricloroetanoles": "tricloroetanol",
    "tricloroetilenos": "tricloroetileno",
    "triclorofenoles": "triclorofenol",
    "triclorofenoxiacética": "triclorofenoxiacético",
    "triclorofenoxiacéticas": "triclorofenoxiacético",
    "triclorofenoxiacéticos": "triclorofenoxiacético",
    "triclorofluometanos": "triclorofluometano",
    "tricloroisocianúrica": "tricloroisocianúrico",
    "tricloroisocianúricas": "tricloroisocianúrico",
    "tricloroisocianúricos": "tricloroisocianúrico",
    "tricloromelaminas": "tricloromelamina",
    "triclorometilfosfónica": "triclorometilfosfónico",
    "triclorometilfosfónicas": "triclorometilfosfónico",
    "triclorometilfosfónicos": "triclorometilfosfónico",
    "tricloronitrosometanos": "tricloronitrosometano",
    "tricloropropanos": "tricloropropano",
    "triclorosilanos": "triclorosilano",
    "triclorotrifluoacetonas": "triclorotrifluoacetona",
    "triclorotrifluoetanos": "triclorotrifluoetano",
    "tricloruros": "tricloruro",
    "triclínica": "triclínico",
    "triclínicas": "triclínico",
    "triclínicos": "triclínico",
    "tricocéfalos": "tricocéfalo",
    "tricolores": "tricolor",
    "tricomas": "tricoma",
    "triconodonta": "triconodonto",
    "triconodontas": "triconodonto",
    "triconodontos": "triconodonto",
    "tricornes": "tricorne",
    "tricornios": "tricornio",
    "tricosanos": "tricosano",
    "tricota": "tricotar",
    "tricotaba": "tricotar",
    "tricotabais": "tricotar",
    "tricotaban": "tricotar",
    "tricotabas": "tricotar",
    "tricotad": "tricotar",
    "tricotada": "tricotar",
    "tricotadas": "tricotar",
    "tricotado": "tricotar",
    "tricotados": "tricotar",
    "tricotamos": "tricotar",
    "tricotan": "tricotar",
    "tricotando": "tricotar",
    "tricotara": "tricotar",
    "tricotarais": "tricotar",
    "tricotaran": "tricotar",
    "tricotaras": "tricotar",
    "tricotare": "tricotar",
    "tricotareis": "tricotar",
    "tricotaremos": "tricotar",
    "tricotaren": "tricotar",
    "tricotares": "tricotar",
    "tricotaron": "tricotar",
    "tricotará": "tricotar",
    "tricotarán": "tricotar",
    "tricotarás": "tricotar",
    "tricotaré": "tricotar",
    "tricotaréis": "tricotar",
    "tricotaría": "tricotar",
    "tricotaríais": "tricotar",
    "tricotaríamos": "tricotar",
    "tricotarían": "tricotar",
    "tricotarías": "tricotar",
    "tricotas": "tricotar",
    "tricotase": "tricotar",
    "tricotaseis": "tricotar",
    "tricotasen": "tricotar",
    "tricotases": "tricotar",
    "tricotaste": "tricotar",
    "tricotasteis": "tricotar",
    "tricote": "tricotar",
    "tricotemos": "tricotar",
    "tricoten": "tricotar",
    "tricotes": "tricotar",
    "tricotinas": "tricotina",
    "tricoto": "tricotar",
    "tricotomías": "tricotomía",
    "tricotábamos": "tricotar",
    "tricotáis": "tricotar",
    "tricotáramos": "tricotar",
    "tricotáremos": "tricotar",
    "tricotásemos": "tricotar",
    "tricoté": "tricotar",
    "tricotéis": "tricotar",
    "tricotó": "tricotar",
    "tricotómica": "tricotómico",
    "tricotómicas": "tricotómico",
    "tricotómicos": "tricotómico",
    "tricroismos": "tricroismo",
    "tricromática": "tricromático",
    "tricromáticas": "tricromático",
    "tricromáticos": "tricromático",
    "tricromías": "tricromía",
    "tricuspidada": "tricuspidado",
    "tricuspidadas": "tricuspidado",
    "tricuspidados": "tricuspidado",
    "tricálcica": "tricálcico",
    "tricálcicas": "tricálcico",
    "tricálcicos": "tricálcico",
    "tricé": "trizar",
    "tricéfala": "tricéfalo",
    "tricéfalas": "tricéfalo",
    "tricéfalos": "tricéfalo",
    "tricéis": "trizar",
    "tricéntrica": "tricéntrico",
    "tricéntricas": "tricéntrico",
    "tricéntricos": "tricéntrico",
    "tricésima": "30",
    "tricésimas": "30",
    "tricésimo": "30",
    "tricésimos": "30",
    "tricíclica": "tricíclico",
    "tricíclicas": "tricíclico",
    "tricíclicos": "tricíclico",
    "tricípites": "tricípite",
    "tricópteros": "tricóptero",
    "tricótoma": "tricótomo",
    "tricótomas": "tricótomo",
    "tricótomos": "tricótomo",
    "tricúspides": "tricúspide",
    "tridacios": "tridacio",
    "tridecanoatos": "tridecanoato",
    "tridecanos": "tridecano",
    "tridecilaluminios": "tridecilaluminio",
    "tridelta": "tridelto",
    "trideltas": "tridelto",
    "trideltos": "tridelto",
    "tridemismos": "tridemismo",
    "tridentes": "tridente",
    "tridentina": "tridentino",
    "tridentinas": "tridentino",
    "tridentinos": "tridentino",
    "tridimensionales": "tridimensional",
    "tridimensiones": "tridimensión",
    "tridimitas": "tridimita",
    "tridireccionales": "tridireccional",
    "triduana": "triduano",
    "triduanas": "triduano",
    "triduanos": "triduano",
    "triduos": "triduo",
    "tridáctila": "tridáctilo",
    "tridáctilas": "tridáctilo",
    "tridáctilos": "tridáctilo",
    "triedra": "triedro",
    "triedras": "triedro",
    "triedros": "triedro",
    "triemos": "triar",
    "trienales": "trienal",
    "trienios": "trienio",
    "trientes": "triente",
    "triestables": "triestable",
    "triestina": "triestino",
    "triestinas": "triestino",
    "triestinos": "triestino",
    "trietanolaminas": "trietanolamina",
    "trietilaminas": "trietilamina",
    "trietilboranos": "trietilborano",
    "trietilenmelaminas": "trietilenmelamina",
    "trietilentetraminas": "trietilentetramina",
    "trietoxihexanos": "trietoxihexano",
    "trietoximetoxipropanos": "trietoximetoxipropano",
    "trieñales": "trieñal",
    "trifauces": "trifauce",
    "trifenilestibinas": "trifenilestibina",
    "trifenilguanidinas": "trifenilguanidina",
    "trifilares": "trifilar",
    "trifinios": "trifinio",
    "triflora": "trifloro",
    "trifloras": "trifloro",
    "trifloros": "trifloro",
    "trifluoacética": "trifluoacético",
    "trifluoacéticas": "trifluoacético",
    "trifluoacéticos": "trifluoacético",
    "trifluonitrosometanos": "trifluonitrosometano",
    "trifluoruros": "trifluoruro",
    "trifoliada": "trifoliado",
    "trifoliadas": "trifoliado",
    "trifoliados": "trifoliado",
    "trifoliolada": "trifoliolado",
    "trifolioladas": "trifoliolado",
    "trifoliolados": "trifoliolado",
    "trifolios": "trifolio",
    "triforios": "triforio",
    "triformes": "triforme",
    "triformiatos": "triformiato",
    "trifosfatos": "trifosfato",
    "trifulcas": "trifulca",
    "trifurca": "trifurcar",
    "trifurcaba": "trifurcar",
    "trifurcabais": "trifurcar",
    "trifurcaban": "trifurcar",
    "trifurcabas": "trifurcar",
    "trifurcaciones": "trifurcación",
    "trifurcad": "trifurcar",
    "trifurcada": "trifurcar",
    "trifurcadas": "trifurcar",
    "trifurcado": "trifurcar",
    "trifurcados": "trifurcar",
    "trifurcamos": "trifurcar",
    "trifurcan": "trifurcar",
    "trifurcando": "trifurcar",
    "trifurcara": "trifurcar",
    "trifurcarais": "trifurcar",
    "trifurcaran": "trifurcar",
    "trifurcaras": "trifurcar",
    "trifurcare": "trifurcar",
    "trifurcareis": "trifurcar",
    "trifurcaremos": "trifurcar",
    "trifurcaren": "trifurcar",
    "trifurcares": "trifurcar",
    "trifurcaron": "trifurcar",
    "trifurcará": "trifurcar",
    "trifurcarán": "trifurcar",
    "trifurcarás": "trifurcar",
    "trifurcaré": "trifurcar",
    "trifurcaréis": "trifurcar",
    "trifurcaría": "trifurcar",
    "trifurcaríais": "trifurcar",
    "trifurcaríamos": "trifurcar",
    "trifurcarían": "trifurcar",
    "trifurcarías": "trifurcar",
    "trifurcas": "trifurcar",
    "trifurcase": "trifurcar",
    "trifurcaseis": "trifurcar",
    "trifurcasen": "trifurcar",
    "trifurcases": "trifurcar",
    "trifurcaste": "trifurcar",
    "trifurcasteis": "trifurcar",
    "trifurco": "trifurcar",
    "trifurcábamos": "trifurcar",
    "trifurcáis": "trifurcar",
    "trifurcáramos": "trifurcar",
    "trifurcáremos": "trifurcar",
    "trifurcásemos": "trifurcar",
    "trifurcó": "trifurcar",
    "trifurque": "trifurcar",
    "trifurquemos": "trifurcar",
    "trifurquen": "trifurcar",
    "trifurques": "trifurcar",
    "trifurqué": "trifurcar",
    "trifurquéis": "trifurcar",
    "trifásica": "trifásico",
    "trifásicas": "trifásico",
    "trifásicos": "trifásico",
    "trigales": "trigal",
    "trigas": "triga",
    "trigeminales": "trigeminal",
    "trigesimoctava": "38",
    "trigesimoctavas": "38",
    "trigesimoctavo": "38",
    "trigesimoctavos": "38",
    "trigesimocuarta": "34",
    "trigesimocuartas": "34",
    "trigesimocuarto": "34",
    "trigesimocuartos": "34",
    "trigesimonona": "39",
    "trigesimononas": "39",
    "trigesimonono": "39",
    "trigesimononos": "39",
    "trigesimonovena": "39",
    "trigesimonovenas": "39",
    "trigesimonoveno": "39",
    "trigesimonovenos": "39",
    "trigesimoprimera": "31",
    "trigesimoprimeras": "31",
    "trigesimoprimero": "31",
    "trigesimoprimeros": "31",
    "trigesimoquinta": "35",
    "trigesimoquintas": "35",
    "trigesimoquinto": "35",
    "trigesimoquintos": "35",
    "trigesimosegunda": "32",
    "trigesimosegundas": "32",
    "trigesimosegundo": "32",
    "trigesimosegundos": "32",
    "trigesimosexta": "36",
    "trigesimosextas": "36",
    "trigesimosexto": "36",
    "trigesimosextos": "36",
    "trigesimoséptima": "37",
    "trigesimoséptimas": "37",
    "trigesimoséptimo": "37",
    "trigesimoséptimos": "37",
    "triglicéridos": "triglicérido",
    "triglifos": "triglifo",
    "trigonales": "trigonal",
    "trigonelinas": "trigonelina",
    "trigones": "trigón",
    "trigonometrías": "trigonometría",
    "trigonométrica": "trigonométrico",
    "trigonométricas": "trigonométrico",
    "trigonométricos": "trigonométrico",
    "trigos": "trigo",
    "triguera": "triguero",
    "trigueras": "triguero",
    "trigueros": "triguero",
    "trigueña": "trigueño",
    "trigueñas": "trigueño",
    "trigueños": "trigueño",
    "trigémina": "trigémino",
    "trigéminas": "trigémino",
    "trigéminos": "trigémino",
    "trigésima": "30",
    "trigésimas": "30",
    "trigésimo": "30",
    "trigésimos": "30",
    "trihidratos": "trihidrato",
    "trihidrocloruros": "trihidrocloruro",
    "trihidroxiestearatos": "trihidroxiestearato",
    "trihídrica": "trihídrico",
    "trihídricas": "trihídrico",
    "trihídricos": "trihídrico",
    "triisobutilaluminios": "triisobutilaluminio",
    "triisobutilenos": "triisobutileno",
    "triisopropanolaminas": "triisopropanolamina",
    "trilateraciones": "trilateración",
    "trilaterales": "trilateral",
    "trilera": "trilero",
    "trileras": "trilero",
    "trileros": "trilero",
    "trilingües": "trilingüe",
    "trilitas": "trilita",
    "trilitos": "trilito",
    "trilla": "trillar",
    "trillaba": "trillar",
    "trillabais": "trillar",
    "trillaban": "trillar",
    "trillabas": "trillar",
    "trillad": "trillar",
    "trillada": "trillar",
    "trilladas": "trillar",
    "trillado": "trillar",
    "trilladora": "trillador",
    "trilladoras": "trilladora",
    "trilladores": "trillador",
    "trillados": "trillar",
    "trilladuras": "trilladura",
    "trillamos": "trillar",
    "trillan": "trillar",
    "trillando": "trillar",
    "trillara": "trillar",
    "trillarais": "trillar",
    "trillaran": "trillar",
    "trillaras": "trillar",
    "trillare": "trillar",
    "trillareis": "trillar",
    "trillaremos": "trillar",
    "trillaren": "trillar",
    "trillares": "trillar",
    "trillaron": "trillar",
    "trillará": "trillar",
    "trillarán": "trillar",
    "trillarás": "trillar",
    "trillaré": "trillar",
    "trillaréis": "trillar",
    "trillaría": "trillar",
    "trillaríais": "trillar",
    "trillaríamos": "trillar",
    "trillarían": "trillar",
    "trillarías": "trillar",
    "trillas": "trillar",
    "trillase": "trillar",
    "trillaseis": "trillar",
    "trillasen": "trillar",
    "trillases": "trillar",
    "trillaste": "trillar",
    "trillasteis": "trillar",
    "trille": "trillar",
    "trillemos": "trillar",
    "trillen": "trillar",
    "trilles": "trillar",
    "trilliza": "trillizo",
    "trillizas": "trillizo",
    "trillizos": "trillizo",
    "trillo": "trillar",
    "trillones": "trillón",
    "trillos": "trillo",
    "trillábamos": "trillar",
    "trilláis": "trillar",
    "trilláramos": "trillar",
    "trilláremos": "trillar",
    "trillásemos": "trillar",
    "trillé": "trillar",
    "trilléis": "trillar",
    "trilló": "trillar",
    "trilobites": "trilobite",
    "trilobulada": "trilobulado",
    "trilobuladas": "trilobulado",
    "trilobulados": "trilobulado",
    "triloculares": "trilocular",
    "trilogías": "trilogía",
    "trilátera": "trilátero",
    "triláteras": "trilátero",
    "triláteros": "trilátero",
    "trilítera": "trilítero",
    "trilíteras": "trilítero",
    "trilíteros": "trilítero",
    "trimaranes": "trimarán",
    "trimembres": "trimembre",
    "trimerizaciones": "trimerización",
    "trimeros": "trimero",
    "trimerías": "trimería",
    "trimestrales": "trimestral",
    "trimestres": "trimestre",
    "trimetadionas": "trimetadiona",
    "trimetilacética": "trimetilacético",
    "trimetilacéticas": "trimetilacético",
    "trimetilacéticos": "trimetilacético",
    "trimetilaluminios": "trimetilaluminio",
    "trimetilaminas": "trimetilamina",
    "trimetilbutanos": "trimetilbutano",
    "trimetilclorosilanos": "trimetilclorosilano",
    "trimetilhexanos": "trimetilhexano",
    "trimetilnonanonas": "trimetilnonanona",
    "trimetiloletanos": "trimetiloletano",
    "trimetilolpropanos": "trimetilolpropano",
    "trimetilpentanos": "trimetilpentano",
    "trimetilpentenos": "trimetilpenteno",
    "trimetilpentilaluminios": "trimetilpentilaluminio",
    "trimetrías": "trimetría",
    "trimielgas": "trimielga",
    "trimmers": "trimmer",
    "trimoleculares": "trimolecular",
    "trimonoica": "trimonoico",
    "trimonoicas": "trimonoico",
    "trimonoicos": "trimonoico",
    "trimotores": "trimotor",
    "trimurtis": "trimurti",
    "trimétrica": "trimétrico",
    "trimétricas": "trimétrico",
    "trimétricos": "trimétrico",
    "trina": "trino",
    "trinaba": "trinar",
    "trinabais": "trinar",
    "trinaban": "trinar",
    "trinabas": "trinar",
    "trinacria": "trinacrio",
    "trinacrias": "trinacrio",
    "trinacrios": "trinacrio",
    "trinad": "trinar",
    "trinada": "trinar",
    "trinadas": "trinar",
    "trinado": "trinar",
    "trinados": "trinar",
    "trinamos": "trinar",
    "trinan": "trinar",
    "trinando": "trinar",
    "trinara": "trinar",
    "trinarais": "trinar",
    "trinaran": "trinar",
    "trinaras": "trinar",
    "trinare": "trinar",
    "trinareis": "trinar",
    "trinaremos": "trinar",
    "trinaren": "trinar",
    "trinares": "trinar",
    "trinaron": "trinar",
    "trinará": "trinar",
    "trinarán": "trinar",
    "trinarás": "trinar",
    "trinaré": "trinar",
    "trinaréis": "trinar",
    "trinaría": "trinar",
    "trinaríais": "trinar",
    "trinaríamos": "trinar",
    "trinarían": "trinar",
    "trinarías": "trinar",
    "trinas": "trino",
    "trinase": "trinar",
    "trinaseis": "trinar",
    "trinasen": "trinar",
    "trinases": "trinar",
    "trinaste": "trinar",
    "trinasteis": "trinar",
    "trinca": "trincar",
    "trincaba": "trincar",
    "trincabais": "trincar",
    "trincaban": "trincar",
    "trincabas": "trincar",
    "trincad": "trincar",
    "trincada": "trincar",
    "trincadas": "trincar",
    "trincado": "trincar",
    "trincados": "trincar",
    "trincaduras": "trincadura",
    "trincafías": "trincafía",
    "trincajes": "trincaje",
    "trincamos": "trincar",
    "trincan": "trincar",
    "trincando": "trincar",
    "trincara": "trincar",
    "trincarais": "trincar",
    "trincaran": "trincar",
    "trincaras": "trincar",
    "trincare": "trincar",
    "trincareis": "trincar",
    "trincaremos": "trincar",
    "trincaren": "trincar",
    "trincares": "trincar",
    "trincaron": "trincar",
    "trincará": "trincar",
    "trincarán": "trincar",
    "trincarás": "trincar",
    "trincaré": "trincar",
    "trincaréis": "trincar",
    "trincaría": "trincar",
    "trincaríais": "trincar",
    "trincaríamos": "trincar",
    "trincarían": "trincar",
    "trincarías": "trincar",
    "trincas": "trincar",
    "trincase": "trincar",
    "trincaseis": "trincar",
    "trincasen": "trincar",
    "trincases": "trincar",
    "trincaste": "trincar",
    "trincasteis": "trincar",
    "trincha": "trinchar",
    "trinchaba": "trinchar",
    "trinchabais": "trinchar",
    "trinchaban": "trinchar",
    "trinchabas": "trinchar",
    "trinchad": "trinchar",
    "trinchada": "trinchar",
    "trinchadas": "trinchar",
    "trinchado": "trinchar",
    "trinchados": "trinchar",
    "trinchamos": "trinchar",
    "trinchan": "trinchar",
    "trinchando": "trinchar",
    "trinchantes": "trinchante",
    "trinchara": "trinchar",
    "trincharais": "trinchar",
    "trincharan": "trinchar",
    "trincharas": "trinchar",
    "trinchare": "trinchar",
    "trinchareis": "trinchar",
    "trincharemos": "trinchar",
    "trincharen": "trinchar",
    "trinchares": "trinchar",
    "trincharon": "trinchar",
    "trinchará": "trinchar",
    "trincharán": "trinchar",
    "trincharás": "trinchar",
    "trincharé": "trinchar",
    "trincharéis": "trinchar",
    "trincharía": "trinchar",
    "trincharíais": "trinchar",
    "trincharíamos": "trinchar",
    "trincharían": "trinchar",
    "trincharías": "trinchar",
    "trinchas": "trinchar",
    "trinchase": "trinchar",
    "trinchaseis": "trinchar",
    "trinchasen": "trinchar",
    "trinchases": "trinchar",
    "trinchaste": "trinchar",
    "trinchasteis": "trinchar",
    "trinche": "trinchar",
    "trinchemos": "trinchar",
    "trinchen": "trinchar",
    "trinchera": "trinchero",
    "trincheras": "trinchero",
    "trincheros": "trinchero",
    "trinches": "trinchar",
    "trinchetes": "trinchete",
    "trincho": "trinchar",
    "trinchábamos": "trinchar",
    "trincháis": "trinchar",
    "trincháramos": "trinchar",
    "trincháremos": "trinchar",
    "trinchásemos": "trinchar",
    "trinché": "trinchar",
    "trinchéis": "trinchar",
    "trinchó": "trinchar",
    "trinco": "trincar",
    "trincábamos": "trincar",
    "trincáis": "trincar",
    "trincáramos": "trincar",
    "trincáremos": "trincar",
    "trincásemos": "trincar",
    "trincó": "trincar",
    "trine": "trinar",
    "trinemos": "trinar",
    "trinen": "trinar",
    "trineos": "trineo",
    "trines": "trinar",
    "trinitaria": "trinitario",
    "trinitarias": "trinitario",
    "trinitarios": "trinitario",
    "trinitrobencenos": "trinitrobenceno",
    "trinitrobenzoica": "trinitrobenzoico",
    "trinitrobenzoicas": "trinitrobenzoico",
    "trinitrobenzoicos": "trinitrobenzoico",
    "trinitrorresorcinatos": "trinitrorresorcinato",
    "trinitrotoluenos": "trinitrotolueno",
    "trino": "trinar",
    "trinomios": "trinomio",
    "trinos": "trino",
    "trinque": "trincar",
    "trinquemos": "trincar",
    "trinquen": "trincar",
    "trinques": "trincar",
    "trinquetadas": "trinquetada",
    "trinquetes": "trinquete",
    "trinquetillas": "trinquetilla",
    "trinqué": "trincar",
    "trinquéis": "trincar",
    "trinábamos": "trinar",
    "trináis": "trinar",
    "trináramos": "trinar",
    "trináremos": "trinar",
    "trinásemos": "trinar",
    "triné": "trinar",
    "trinéis": "trinar",
    "trinó": "trinar",
    "trioctaédrica": "trioctaédrico",
    "trioctaédricas": "trioctaédrico",
    "trioctaédricos": "trioctaédrico",
    "triodos": "triodo",
    "trioles": "triol",
    "triosas": "triosa",
    "trioxanos": "trioxano",
    "tripadas": "tripada",
    "tripalmitinas": "tripalmitina",
    "tripanosomas": "tripanosoma",
    "triparanoles": "triparanol",
    "triparsamidas": "triparsamida",
    "triparta": "tripartir",
    "tripartamos": "tripartir",
    "tripartan": "tripartir",
    "tripartas": "tripartir",
    "triparte": "tripartir",
    "triparten": "tripartir",
    "tripartes": "tripartir",
    "triparticiones": "tripartición",
    "tripartid": "tripartir",
    "tripartida": "tripartir",
    "tripartidas": "tripartir",
    "tripartido": "tripartir",
    "tripartidos": "tripartir",
    "tripartiendo": "tripartir",
    "tripartiera": "tripartir",
    "tripartierais": "tripartir",
    "tripartieran": "tripartir",
    "tripartieras": "tripartir",
    "tripartiere": "tripartir",
    "tripartiereis": "tripartir",
    "tripartieren": "tripartir",
    "tripartieres": "tripartir",
    "tripartieron": "tripartir",
    "tripartiese": "tripartir",
    "tripartieseis": "tripartir",
    "tripartiesen": "tripartir",
    "tripartieses": "tripartir",
    "tripartimos": "tripartir",
    "tripartiremos": "tripartir",
    "tripartirá": "tripartir",
    "tripartirán": "tripartir",
    "tripartirás": "tripartir",
    "tripartiré": "tripartir",
    "tripartiréis": "tripartir",
    "tripartiría": "tripartir",
    "tripartiríais": "tripartir",
    "tripartiríamos": "tripartir",
    "tripartirían": "tripartir",
    "tripartirías": "tripartir",
    "tripartiste": "tripartir",
    "tripartisteis": "tripartir",
    "tripartita": "tripartito",
    "tripartitas": "tripartito",
    "tripartitos": "tripartito",
    "tripartiéramos": "tripartir",
    "tripartiéremos": "tripartir",
    "tripartiésemos": "tripartir",
    "tripartió": "tripartir",
    "triparto": "tripartir",
    "tripartáis": "tripartir",
    "tripartí": "tripartir",
    "tripartía": "tripartir",
    "tripartíais": "tripartir",
    "tripartíamos": "tripartir",
    "tripartían": "tripartir",
    "tripartías": "tripartir",
    "tripartís": "tripartir",
    "tripas": "tripa",
    "tripera": "tripero",
    "triperas": "tripero",
    "triperos": "tripero",
    "triperías": "tripería",
    "tripicallera": "tripicallero",
    "tripicalleras": "tripicallero",
    "tripicalleros": "tripicallero",
    "tripla": "triplo",
    "triplanos": "triplano",
    "triplas": "triplo",
    "triples": "triple",
    "tripletes": "triplete",
    "triplica": "triplicar",
    "triplicaba": "triplicar",
    "triplicabais": "triplicar",
    "triplicaban": "triplicar",
    "triplicabas": "triplicar",
    "triplicaciones": "triplicación",
    "triplicad": "triplicar",
    "triplicada": "triplicar",
    "triplicadas": "triplicar",
    "triplicado": "triplicar",
    "triplicadores": "triplicador",
    "triplicados": "triplicar",
    "triplicamos": "triplicar",
    "triplican": "triplicar",
    "triplicando": "triplicar",
    "triplicara": "triplicar",
    "triplicarais": "triplicar",
    "triplicaran": "triplicar",
    "triplicaras": "triplicar",
    "triplicare": "triplicar",
    "triplicareis": "triplicar",
    "triplicaremos": "triplicar",
    "triplicaren": "triplicar",
    "triplicares": "triplicar",
    "triplicaron": "triplicar",
    "triplicará": "triplicar",
    "triplicarán": "triplicar",
    "triplicarás": "triplicar",
    "triplicaré": "triplicar",
    "triplicaréis": "triplicar",
    "triplicaría": "triplicar",
    "triplicaríais": "triplicar",
    "triplicaríamos": "triplicar",
    "triplicarían": "triplicar",
    "triplicarías": "triplicar",
    "triplicas": "triplicar",
    "triplicase": "triplicar",
    "triplicaseis": "triplicar",
    "triplicasen": "triplicar",
    "triplicases": "triplicar",
    "triplicaste": "triplicar",
    "triplicasteis": "triplicar",
    "triplicidades": "triplicidad",
    "triplico": "triplicar",
    "triplicábamos": "triplicar",
    "triplicáis": "triplicar",
    "triplicáramos": "triplicar",
    "triplicáremos": "triplicar",
    "triplicásemos": "triplicar",
    "triplicó": "triplicar",
    "triplique": "triplicar",
    "tripliquemos": "triplicar",
    "tripliquen": "triplicar",
    "tripliques": "triplicar",
    "tripliqué": "triplicar",
    "tripliquéis": "triplicar",
    "triplistas": "triplista",
    "triploblástica": "triploblástico",
    "triploblásticas": "triploblástico",
    "triploblásticos": "triploblástico",
    "triploides": "triploide",
    "triploidias": "triploidia",
    "triplos": "triplo",
    "tripolares": "tripolar",
    "tripolifosfatos": "tripolifosfato",
    "tripolina": "tripolino",
    "tripolinas": "tripolino",
    "tripolinos": "tripolino",
    "tripolitana": "tripolitano",
    "tripolitanas": "tripolitano",
    "tripolitanos": "tripolitano",
    "tripona": "tripón",
    "triponas": "tripón",
    "tripones": "tripón",
    "tripropilenos": "tripropileno",
    "tripropionatos": "tripropionato",
    "tripsinas": "tripsina",
    "tripsómetros": "tripsómetro",
    "triptones": "tripton",
    "triptonga": "triptongar",
    "triptongaba": "triptongar",
    "triptongabais": "triptongar",
    "triptongaban": "triptongar",
    "triptongabas": "triptongar",
    "triptongaciones": "triptongación",
    "triptongad": "triptongar",
    "triptongada": "triptongar",
    "triptongadas": "triptongar",
    "triptongado": "triptongar",
    "triptongados": "triptongar",
    "triptongamos": "triptongar",
    "triptongan": "triptongar",
    "triptongando": "triptongar",
    "triptongara": "triptongar",
    "triptongarais": "triptongar",
    "triptongaran": "triptongar",
    "triptongaras": "triptongar",
    "triptongare": "triptongar",
    "triptongareis": "triptongar",
    "triptongaremos": "triptongar",
    "triptongaren": "triptongar",
    "triptongares": "triptongar",
    "triptongaron": "triptongar",
    "triptongará": "triptongar",
    "triptongarán": "triptongar",
    "triptongarás": "triptongar",
    "triptongaré": "triptongar",
    "triptongaréis": "triptongar",
    "triptongaría": "triptongar",
    "triptongaríais": "triptongar",
    "triptongaríamos": "triptongar",
    "triptongarían": "triptongar",
    "triptongarías": "triptongar",
    "triptongas": "triptongar",
    "triptongase": "triptongar",
    "triptongaseis": "triptongar",
    "triptongasen": "triptongar",
    "triptongases": "triptongar",
    "triptongaste": "triptongar",
    "triptongasteis": "triptongar",
    "triptongo": "triptongar",
    "triptongos": "triptongo",
    "triptongue": "triptongar",
    "triptonguemos": "triptongar",
    "triptonguen": "triptongar",
    "triptongues": "triptongar",
    "triptongué": "triptongar",
    "triptonguéis": "triptongar",
    "triptongábamos": "triptongar",
    "triptongáis": "triptongar",
    "triptongáramos": "triptongar",
    "triptongáremos": "triptongar",
    "triptongásemos": "triptongar",
    "triptongó": "triptongar",
    "triptófanos": "triptófano",
    "tripuda": "tripudo",
    "tripudas": "tripudo",
    "tripudios": "tripudio",
    "tripudos": "tripudo",
    "tripula": "tripular",
    "tripulaba": "tripular",
    "tripulabais": "tripular",
    "tripulaban": "tripular",
    "tripulabas": "tripular",
    "tripulaciones": "tripulación",
    "tripulad": "tripular",
    "tripulada": "tripular",
    "tripuladas": "tripular",
    "tripulado": "tripular",
    "tripulados": "tripular",
    "tripulamos": "tripular",
    "tripulan": "tripular",
    "tripulando": "tripular",
    "tripulantes": "tripulante",
    "tripulara": "tripular",
    "tripularais": "tripular",
    "tripularan": "tripular",
    "tripularas": "tripular",
    "tripulare": "tripular",
    "tripulareis": "tripular",
    "tripularemos": "tripular",
    "tripularen": "tripular",
    "tripulares": "tripular",
    "tripularon": "tripular",
    "tripulará": "tripular",
    "tripularán": "tripular",
    "tripularás": "tripular",
    "tripularé": "tripular",
    "tripularéis": "tripular",
    "tripularía": "tripular",
    "tripularíais": "tripular",
    "tripularíamos": "tripular",
    "tripularían": "tripular",
    "tripularías": "tripular",
    "tripulas": "tripular",
    "tripulase": "tripular",
    "tripulaseis": "tripular",
    "tripulasen": "tripular",
    "tripulases": "tripular",
    "tripulaste": "tripular",
    "tripulasteis": "tripular",
    "tripule": "tripular",
    "tripulemos": "tripular",
    "tripulen": "tripular",
    "tripules": "tripular",
    "tripulo": "tripular",
    "tripulábamos": "tripular",
    "tripuláis": "tripular",
    "tripuláramos": "tripular",
    "tripuláremos": "tripular",
    "tripulásemos": "tripular",
    "tripulé": "tripular",
    "tripuléis": "tripular",
    "tripuló": "tripular",
    "triques": "trique",
    "triquetes": "triquete",
    "triquetrales": "triquetral",
    "triquinas": "triquina",
    "triquitraques": "triquitraque",
    "triquiñuelas": "triquiñuela",
    "trirrectángula": "trirrectángulo",
    "trirrectángulas": "trirrectángulo",
    "trirrectángulos": "trirrectángulo",
    "trirremes": "trirreme",
    "trirricinoleatos": "trirricinoleato",
    "trisa": "trisar",
    "trisaba": "trisar",
    "trisabais": "trisar",
    "trisaban": "trisar",
    "trisabas": "trisar",
    "trisad": "trisar",
    "trisada": "trisar",
    "trisadas": "trisar",
    "trisado": "trisar",
    "trisados": "trisar",
    "trisagios": "trisagio",
    "trisamos": "trisar",
    "trisan": "trisar",
    "trisando": "trisar",
    "trisara": "trisar",
    "trisarais": "trisar",
    "trisaran": "trisar",
    "trisaras": "trisar",
    "trisare": "trisar",
    "trisareis": "trisar",
    "trisaremos": "trisar",
    "trisaren": "trisar",
    "trisares": "trisar",
    "trisaron": "trisar",
    "trisará": "trisar",
    "trisarán": "trisar",
    "trisarás": "trisar",
    "trisaré": "trisar",
    "trisaréis": "trisar",
    "trisaría": "trisar",
    "trisaríais": "trisar",
    "trisaríamos": "trisar",
    "trisarían": "trisar",
    "trisarías": "trisar",
    "trisas": "trisar",
    "trisase": "trisar",
    "trisaseis": "trisar",
    "trisasen": "trisar",
    "trisases": "trisar",
    "trisaste": "trisar",
    "trisasteis": "trisar",
    "trisca": "triscar",
    "triscaba": "triscar",
    "triscabais": "triscar",
    "triscaban": "triscar",
    "triscabas": "triscar",
    "triscad": "triscar",
    "triscada": "triscar",
    "triscadas": "triscar",
    "triscado": "triscar",
    "triscadores": "triscador",
    "triscados": "triscar",
    "triscamos": "triscar",
    "triscan": "triscar",
    "triscando": "triscar",
    "triscara": "triscar",
    "triscarais": "triscar",
    "triscaran": "triscar",
    "triscaras": "triscar",
    "triscare": "triscar",
    "triscareis": "triscar",
    "triscaremos": "triscar",
    "triscaren": "triscar",
    "triscares": "triscar",
    "triscaron": "triscar",
    "triscará": "triscar",
    "triscarán": "triscar",
    "triscarás": "triscar",
    "triscaré": "triscar",
    "triscaréis": "triscar",
    "triscaría": "triscar",
    "triscaríais": "triscar",
    "triscaríamos": "triscar",
    "triscarían": "triscar",
    "triscarías": "triscar",
    "triscas": "triscar",
    "triscase": "triscar",
    "triscaseis": "triscar",
    "triscasen": "triscar",
    "triscases": "triscar",
    "triscaste": "triscar",
    "triscasteis": "triscar",
    "trisco": "triscar",
    "triscábamos": "triscar",
    "triscáis": "triscar",
    "triscáramos": "triscar",
    "triscáremos": "triscar",
    "triscásemos": "triscar",
    "triscó": "triscar",
    "trise": "trisar",
    "triseca": "trisecar",
    "trisecaba": "trisecar",
    "trisecabais": "trisecar",
    "trisecaban": "trisecar",
    "trisecabas": "trisecar",
    "trisecad": "trisecar",
    "trisecada": "trisecar",
    "trisecadas": "trisecar",
    "trisecado": "trisecar",
    "trisecados": "trisecar",
    "trisecamos": "trisecar",
    "trisecan": "trisecar",
    "trisecando": "trisecar",
    "trisecara": "trisecar",
    "trisecarais": "trisecar",
    "trisecaran": "trisecar",
    "trisecaras": "trisecar",
    "trisecare": "trisecar",
    "trisecareis": "trisecar",
    "trisecaremos": "trisecar",
    "trisecaren": "trisecar",
    "trisecares": "trisecar",
    "trisecaron": "trisecar",
    "trisecará": "trisecar",
    "trisecarán": "trisecar",
    "trisecarás": "trisecar",
    "trisecaré": "trisecar",
    "trisecaréis": "trisecar",
    "trisecaría": "trisecar",
    "trisecaríais": "trisecar",
    "trisecaríamos": "trisecar",
    "trisecarían": "trisecar",
    "trisecarías": "trisecar",
    "trisecas": "trisecar",
    "trisecase": "trisecar",
    "trisecaseis": "trisecar",
    "trisecasen": "trisecar",
    "trisecases": "trisecar",
    "trisecaste": "trisecar",
    "trisecasteis": "trisecar",
    "trisecciones": "trisección",
    "trisecciónes": "trisección",
    "triseco": "trisecar",
    "trisectrices": "trisectriz",
    "trisecábamos": "trisecar",
    "trisecáis": "trisecar",
    "trisecáramos": "trisecar",
    "trisecáremos": "trisecar",
    "trisecásemos": "trisecar",
    "trisecó": "trisecar",
    "trisemanales": "trisemanal",
    "trisemos": "trisar",
    "trisen": "trisar",
    "triseque": "trisecar",
    "trisequemos": "trisecar",
    "trisequen": "trisecar",
    "triseques": "trisecar",
    "trisequé": "trisecar",
    "trisequéis": "trisecar",
    "trises": "trisar",
    "trisilicatos": "trisilicato",
    "trisilábica": "trisilábico",
    "trisilábicas": "trisilábico",
    "trisilábicos": "trisilábico",
    "trisistores": "trisistor",
    "trismos": "trismo",
    "triso": "trisar",
    "trisoctaédrica": "trisoctaédrico",
    "trisoctaédricas": "trisoctaédrico",
    "trisoctaédricos": "trisoctaédrico",
    "trisomias": "trisomia",
    "trispastos": "trispasto",
    "trisque": "triscar",
    "trisqueliones": "trisquelión",
    "trisquemos": "triscar",
    "trisquen": "triscar",
    "trisques": "trisque",
    "trisqué": "triscar",
    "trisquéis": "triscar",
    "tristes": "triste",
    "tristezas": "tristeza",
    "tristona": "tristón",
    "tristonas": "tristón",
    "tristones": "tristón",
    "tristuras": "tristura",
    "trisulca": "trisulco",
    "trisulcas": "trisulco",
    "trisulcos": "trisulco",
    "trisulfuros": "trisulfuro",
    "trisábamos": "trisar",
    "trisáis": "trisar",
    "trisáramos": "trisar",
    "trisáremos": "trisar",
    "trisásemos": "trisar",
    "trisé": "trisar",
    "triséis": "trisar",
    "trisílaba": "trisílabo",
    "trisílabas": "trisílabo",
    "trisílabos": "trisílabo",
    "trisó": "trisar",
    "trisómica": "trisómico",
    "trisómicas": "trisómico",
    "trisómicos": "trisómico",
    "tritanomalopías": "tritanomalopía",
    "tritanomalías": "tritanomalía",
    "tritanopsias": "tritanopsia",
    "tritanopías": "tritanopía",
    "tritios": "tritio",
    "tritones": "tritón",
    "trituberculada": "trituberculado",
    "trituberculadas": "trituberculado",
    "trituberculados": "trituberculado",
    "tritura": "triturar",
    "trituraba": "triturar",
    "triturabais": "triturar",
    "trituraban": "triturar",
    "triturabas": "triturar",
    "triturables": "triturable",
    "trituraciones": "trituración",
    "triturad": "triturar",
    "triturada": "triturar",
    "trituradas": "triturar",
    "triturado": "triturar",
    "trituradora": "triturador",
    "trituradoras": "trituradora",
    "trituradores": "triturador",
    "triturados": "triturar",
    "trituramos": "triturar",
    "trituran": "triturar",
    "triturando": "triturar",
    "triturantes": "triturante",
    "triturara": "triturar",
    "triturarais": "triturar",
    "trituraran": "triturar",
    "trituraras": "triturar",
    "triturare": "triturar",
    "triturareis": "triturar",
    "trituraremos": "triturar",
    "trituraren": "triturar",
    "triturares": "triturar",
    "trituraron": "triturar",
    "triturará": "triturar",
    "triturarán": "triturar",
    "triturarás": "triturar",
    "trituraré": "triturar",
    "trituraréis": "triturar",
    "trituraría": "triturar",
    "trituraríais": "triturar",
    "trituraríamos": "triturar",
    "triturarían": "triturar",
    "triturarías": "triturar",
    "trituras": "triturar",
    "triturase": "triturar",
    "trituraseis": "triturar",
    "triturasen": "triturar",
    "triturases": "triturar",
    "trituraste": "triturar",
    "triturasteis": "triturar",
    "triture": "triturar",
    "trituremos": "triturar",
    "trituren": "triturar",
    "tritures": "triturar",
    "trituro": "triturar",
    "triturábamos": "triturar",
    "trituráis": "triturar",
    "trituráramos": "triturar",
    "trituráremos": "triturar",
    "triturásemos": "triturar",
    "trituré": "triturar",
    "trituréis": "triturar",
    "trituró": "triturar",
    "tritícea": "tritíceo",
    "tritíceas": "tritíceo",
    "tritíceos": "tritíceo",
    "tritóxidos": "tritóxido",
    "triunfa": "triunfar",
    "triunfaba": "triunfar",
    "triunfabais": "triunfar",
    "triunfaban": "triunfar",
    "triunfabas": "triunfar",
    "triunfad": "triunfar",
    "triunfada": "triunfar",
    "triunfadas": "triunfar",
    "triunfado": "triunfar",
    "triunfadora": "triunfador",
    "triunfadoras": "triunfador",
    "triunfadores": "triunfador",
    "triunfados": "triunfar",
    "triunfales": "triunfal",
    "triunfalismos": "triunfalismo",
    "triunfalistas": "triunfalista",
    "triunfamos": "triunfar",
    "triunfan": "triunfar",
    "triunfando": "triunfar",
    "triunfantes": "triunfante",
    "triunfara": "triunfar",
    "triunfarais": "triunfar",
    "triunfaran": "triunfar",
    "triunfaras": "triunfar",
    "triunfare": "triunfar",
    "triunfareis": "triunfar",
    "triunfaremos": "triunfar",
    "triunfaren": "triunfar",
    "triunfares": "triunfar",
    "triunfaron": "triunfar",
    "triunfará": "triunfar",
    "triunfarán": "triunfar",
    "triunfarás": "triunfar",
    "triunfaré": "triunfar",
    "triunfaréis": "triunfar",
    "triunfaría": "triunfar",
    "triunfaríais": "triunfar",
    "triunfaríamos": "triunfar",
    "triunfarían": "triunfar",
    "triunfarías": "triunfar",
    "triunfas": "triunfar",
    "triunfase": "triunfar",
    "triunfaseis": "triunfar",
    "triunfasen": "triunfar",
    "triunfases": "triunfar",
    "triunfaste": "triunfar",
    "triunfasteis": "triunfar",
    "triunfe": "triunfar",
    "triunfemos": "triunfar",
    "triunfen": "triunfar",
    "triunfes": "triunfar",
    "triunfo": "triunfar",
    "triunfos": "triunfo",
    "triunfábamos": "triunfar",
    "triunfáis": "triunfar",
    "triunfáramos": "triunfar",
    "triunfáremos": "triunfar",
    "triunfásemos": "triunfar",
    "triunfé": "triunfar",
    "triunféis": "triunfar",
    "triunfó": "triunfar",
    "triunvirales": "triunviral",
    "triunviratos": "triunvirato",
    "triunviros": "triunviro",
    "trivalentes": "trivalente",
    "triviales": "trivial",
    "trivialidades": "trivialidad",
    "triviaria": "triviario",
    "triviarias": "triviario",
    "triviarios": "triviario",
    "trivios": "trivio",
    "triyodotironinas": "triyodotironina",
    "triza": "trizar",
    "trizaba": "trizar",
    "trizabais": "trizar",
    "trizaban": "trizar",
    "trizabas": "trizar",
    "trizad": "trizar",
    "trizada": "trizar",
    "trizadas": "trizar",
    "trizado": "trizar",
    "trizados": "trizar",
    "trizamos": "trizar",
    "trizan": "trizar",
    "trizando": "trizar",
    "trizara": "trizar",
    "trizarais": "trizar",
    "trizaran": "trizar",
    "trizaras": "trizar",
    "trizare": "trizar",
    "trizareis": "trizar",
    "trizaremos": "trizar",
    "trizaren": "trizar",
    "trizares": "trizar",
    "trizaron": "trizar",
    "trizará": "trizar",
    "trizarán": "trizar",
    "trizarás": "trizar",
    "trizaré": "trizar",
    "trizaréis": "trizar",
    "trizaría": "trizar",
    "trizaríais": "trizar",
    "trizaríamos": "trizar",
    "trizarían": "trizar",
    "trizarías": "trizar",
    "trizas": "trizar",
    "trizase": "trizar",
    "trizaseis": "trizar",
    "trizasen": "trizar",
    "trizases": "trizar",
    "trizaste": "trizar",
    "trizasteis": "trizar",
    "trizo": "trizar",
    "trizábamos": "trizar",
    "trizáis": "trizar",
    "trizáramos": "trizar",
    "trizáremos": "trizar",
    "trizásemos": "trizar",
    "trizó": "trizar",
    "triábamos": "triar",
    "triácida": "triácido",
    "triácidas": "triácido",
    "triácidos": "triácido",
    "triádica": "triádico",
    "triádicas": "triádico",
    "triádicos": "triádico",
    "triáis": "triar",
    "triálsines": "trialsin",
    "triángula": "triángulo",
    "triángulas": "triángulo",
    "triángulos": "triángulo",
    "triáramos": "triar",
    "triáremos": "triar",
    "triásemos": "triar",
    "triásica": "triásico",
    "triásicas": "triásico",
    "triásicos": "triásico",
    "trié": "triar",
    "triédrica": "triédrico",
    "triédricas": "triédrico",
    "triédricos": "triédrico",
    "triéis": "triar",
    "trió": "triar",
    "trióxidos": "trióxido",
    "trocaba": "trocar",
    "trocabais": "trocar",
    "trocaban": "trocar",
    "trocabas": "trocar",
    "trocables": "trocable",
    "trocad": "trocar",
    "trocada": "trocar",
    "trocadas": "trocar",
    "trocado": "trocar",
    "trocadora": "trocador",
    "trocadoras": "trocador",
    "trocadores": "trocador",
    "trocados": "trocar",
    "trocaica": "trocaico",
    "trocaicas": "trocaico",
    "trocaicos": "trocaico",
    "trocamientos": "trocamiento",
    "trocamos": "trocar",
    "trocando": "trocar",
    "trocantérica": "trocantérico",
    "trocantéricas": "trocantérico",
    "trocantéricos": "trocantérico",
    "trocara": "trocar",
    "trocarais": "trocar",
    "trocaran": "trocar",
    "trocaras": "trocar",
    "trocare": "trocar",
    "trocareis": "trocar",
    "trocaremos": "trocar",
    "trocaren": "trocar",
    "trocares": "trocar",
    "trocaron": "trocar",
    "trocará": "trocar",
    "trocarán": "trocar",
    "trocarás": "trocar",
    "trocaré": "trocar",
    "trocaréis": "trocar",
    "trocaría": "trocar",
    "trocaríais": "trocar",
    "trocaríamos": "trocar",
    "trocarían": "trocar",
    "trocarías": "trocar",
    "trocase": "trocar",
    "trocaseis": "trocar",
    "trocasen": "trocar",
    "trocases": "trocar",
    "trocaste": "trocar",
    "trocasteis": "trocar",
    "trocatintas": "trocatinta",
    "trocatintes": "trocatinte",
    "troce": "trozar",
    "trocea": "trocear",
    "troceaba": "trocear",
    "troceabais": "trocear",
    "troceaban": "trocear",
    "troceabas": "trocear",
    "trocead": "trocear",
    "troceada": "trocear",
    "troceadas": "trocear",
    "troceado": "trocear",
    "troceadoras": "troceadora",
    "troceadores": "troceador",
    "troceados": "trocear",
    "troceamos": "trocear",
    "trocean": "trocear",
    "troceando": "trocear",
    "troceara": "trocear",
    "trocearais": "trocear",
    "trocearan": "trocear",
    "trocearas": "trocear",
    "troceare": "trocear",
    "troceareis": "trocear",
    "trocearemos": "trocear",
    "trocearen": "trocear",
    "troceares": "trocear",
    "trocearon": "trocear",
    "troceará": "trocear",
    "trocearán": "trocear",
    "trocearás": "trocear",
    "trocearé": "trocear",
    "trocearéis": "trocear",
    "trocearía": "trocear",
    "trocearíais": "trocear",
    "trocearíamos": "trocear",
    "trocearían": "trocear",
    "trocearías": "trocear",
    "troceas": "trocear",
    "trocease": "trocear",
    "troceaseis": "trocear",
    "troceasen": "trocear",
    "troceases": "trocear",
    "troceaste": "trocear",
    "troceasteis": "trocear",
    "trocee": "trocear",
    "troceemos": "trocear",
    "troceen": "trocear",
    "trocees": "trocear",
    "trocemos": "trozar",
    "trocen": "trozar",
    "troceo": "trocear",
    "troceos": "troceo",
    "troces": "trozar",
    "troceábamos": "trocear",
    "troceáis": "trocear",
    "troceáramos": "trocear",
    "troceáremos": "trocear",
    "troceásemos": "trocear",
    "troceé": "trocear",
    "troceéis": "trocear",
    "troceó": "trocear",
    "trochas": "trocha",
    "trociscos": "trocisco",
    "trocitos": "trocito",
    "troclas": "trocla",
    "trocleares": "troclear",
    "trocoidales": "trocoidal",
    "trocoides": "trocoide",
    "trocábamos": "trocar",
    "trocáis": "trocar",
    "trocánteres": "trocánter",
    "trocáramos": "trocar",
    "trocáremos": "trocar",
    "trocásemos": "trocar",
    "trocé": "trozar",
    "trocéis": "trozar",
    "trocó": "trocar",
    "trocóforas": "trocófora",
    "trofalaxias": "trofalaxia",
    "trofectodermos": "trofectodermo",
    "trofeos": "trofeo",
    "troficidades": "troficidad",
    "trofoblastos": "trofoblasto",
    "trofocitos": "trofocito",
    "trofonúcleos": "trofonúcleo",
    "trofoplasmas": "trofoplasma",
    "trofotropismos": "trofotropismo",
    "trogloditas": "troglodita",
    "troglodítica": "troglodítico",
    "troglodíticas": "troglodítico",
    "troglodíticos": "troglodítico",
    "troicas": "troica",
    "troikas": "troika",
    "trojera": "trojero",
    "trojeras": "trojero",
    "trojeros": "trojero",
    "trojes": "troje",
    "trolas": "trola",
    "trolebuses": "trolebús",
    "trolera": "trolero",
    "troleras": "trolero",
    "troleros": "trolero",
    "troles": "trole",
    "trombas": "tromba",
    "trombastenias": "trombastenia",
    "trombinas": "trombina",
    "tromboblastos": "tromboblasto",
    "trombocitemias": "trombocitemia",
    "trombocitopatías": "trombocitopatía",
    "trombocitos": "trombocito",
    "tromboelastografías": "tromboelastografía",
    "trombones": "trombón",
    "trombopatías": "trombopatía",
    "trombopenias": "trombopenia",
    "tromboplastinas": "tromboplastina",
    "trombopénica": "trombopénico",
    "trombopénicas": "trombopénico",
    "trombopénicos": "trombopénico",
    "trombos": "trombo",
    "tromboxanos": "tromboxano",
    "trompadas": "trompada",
    "trompas": "trompa",
    "trompazos": "trompazo",
    "trompea": "trompear",
    "trompeaba": "trompear",
    "trompeabais": "trompear",
    "trompeaban": "trompear",
    "trompeabas": "trompear",
    "trompead": "trompear",
    "trompeada": "trompear",
    "trompeadas": "trompear",
    "trompeado": "trompear",
    "trompeados": "trompear",
    "trompeamos": "trompear",
    "trompean": "trompear",
    "trompeando": "trompear",
    "trompeara": "trompear",
    "trompearais": "trompear",
    "trompearan": "trompear",
    "trompearas": "trompear",
    "trompeare": "trompear",
    "trompeareis": "trompear",
    "trompearemos": "trompear",
    "trompearen": "trompear",
    "trompeares": "trompear",
    "trompearon": "trompear",
    "trompeará": "trompear",
    "trompearán": "trompear",
    "trompearás": "trompear",
    "trompearé": "trompear",
    "trompearéis": "trompear",
    "trompearía": "trompear",
    "trompearíais": "trompear",
    "trompearíamos": "trompear",
    "trompearían": "trompear",
    "trompearías": "trompear",
    "trompeas": "trompear",
    "trompease": "trompear",
    "trompeaseis": "trompear",
    "trompeasen": "trompear",
    "trompeases": "trompear",
    "trompeaste": "trompear",
    "trompeasteis": "trompear",
    "trompee": "trompear",
    "trompeemos": "trompear",
    "trompeen": "trompear",
    "trompees": "trompear",
    "trompeo": "trompear",
    "trompetadas": "trompetada",
    "trompetas": "trompeta",
    "trompetazos": "trompetazo",
    "trompetea": "trompetear",
    "trompeteaba": "trompetear",
    "trompeteabais": "trompetear",
    "trompeteaban": "trompetear",
    "trompeteabas": "trompetear",
    "trompetead": "trompetear",
    "trompeteada": "trompetear",
    "trompeteadas": "trompetear",
    "trompeteado": "trompetear",
    "trompeteados": "trompetear",
    "trompeteamos": "trompetear",
    "trompetean": "trompetear",
    "trompeteando": "trompetear",
    "trompeteara": "trompetear",
    "trompetearais": "trompetear",
    "trompetearan": "trompetear",
    "trompetearas": "trompetear",
    "trompeteare": "trompetear",
    "trompeteareis": "trompetear",
    "trompetearemos": "trompetear",
    "trompetearen": "trompetear",
    "trompeteares": "trompetear",
    "trompetearon": "trompetear",
    "trompeteará": "trompetear",
    "trompetearán": "trompetear",
    "trompetearás": "trompetear",
    "trompetearé": "trompetear",
    "trompetearéis": "trompetear",
    "trompetearía": "trompetear",
    "trompetearíais": "trompetear",
    "trompetearíamos": "trompetear",
    "trompetearían": "trompetear",
    "trompetearías": "trompetear",
    "trompeteas": "trompetear",
    "trompetease": "trompetear",
    "trompeteaseis": "trompetear",
    "trompeteasen": "trompetear",
    "trompeteases": "trompetear",
    "trompeteaste": "trompetear",
    "trompeteasteis": "trompetear",
    "trompetee": "trompetear",
    "trompeteemos": "trompetear",
    "trompeteen": "trompetear",
    "trompetees": "trompetear",
    "trompeteo": "trompetear",
    "trompeteos": "trompeteo",
    "trompetera": "trompetero",
    "trompeteras": "trompetero",
    "trompeteros": "trompetero",
    "trompeterías": "trompetería",
    "trompeteábamos": "trompetear",
    "trompeteáis": "trompetear",
    "trompeteáramos": "trompetear",
    "trompeteáremos": "trompetear",
    "trompeteásemos": "trompetear",
    "trompeteé": "trompetear",
    "trompeteéis": "trompetear",
    "trompeteó": "trompetear",
    "trompetillas": "trompetilla",
    "trompetistas": "trompetista",
    "trompeábamos": "trompear",
    "trompeáis": "trompear",
    "trompeáramos": "trompear",
    "trompeáremos": "trompear",
    "trompeásemos": "trompear",
    "trompeé": "trompear",
    "trompeéis": "trompear",
    "trompeó": "trompear",
    "trompica": "trompicar",
    "trompicaba": "trompicar",
    "trompicabais": "trompicar",
    "trompicaban": "trompicar",
    "trompicabas": "trompicar",
    "trompicad": "trompicar",
    "trompicada": "trompicar",
    "trompicadas": "trompicar",
    "trompicado": "trompicar",
    "trompicados": "trompicar",
    "trompicamos": "trompicar",
    "trompican": "trompicar",
    "trompicando": "trompicar",
    "trompicara": "trompicar",
    "trompicarais": "trompicar",
    "trompicaran": "trompicar",
    "trompicaras": "trompicar",
    "trompicare": "trompicar",
    "trompicareis": "trompicar",
    "trompicaremos": "trompicar",
    "trompicaren": "trompicar",
    "trompicares": "trompicar",
    "trompicaron": "trompicar",
    "trompicará": "trompicar",
    "trompicarán": "trompicar",
    "trompicarás": "trompicar",
    "trompicaré": "trompicar",
    "trompicaréis": "trompicar",
    "trompicaría": "trompicar",
    "trompicaríais": "trompicar",
    "trompicaríamos": "trompicar",
    "trompicarían": "trompicar",
    "trompicarías": "trompicar",
    "trompicas": "trompicar",
    "trompicase": "trompicar",
    "trompicaseis": "trompicar",
    "trompicasen": "trompicar",
    "trompicases": "trompicar",
    "trompicaste": "trompicar",
    "trompicasteis": "trompicar",
    "trompico": "trompicar",
    "trompicones": "trompicón",
    "trompicos": "trompico",
    "trompicábamos": "trompicar",
    "trompicáis": "trompicar",
    "trompicáramos": "trompicar",
    "trompicáremos": "trompicar",
    "trompicásemos": "trompicar",
    "trompicó": "trompicar",
    "trompillones": "trompillón",
    "trompique": "trompicar",
    "trompiquemos": "trompicar",
    "trompiquen": "trompicar",
    "trompiques": "trompicar",
    "trompiqué": "trompicar",
    "trompiquéis": "trompicar",
    "trompones": "trompón",
    "trompos": "trompo",
    "tronaba": "tronar",
    "tronabais": "tronar",
    "tronaban": "tronar",
    "tronabas": "tronar",
    "tronad": "tronar",
    "tronada": "tronar",
    "tronadas": "tronar",
    "tronado": "tronar",
    "tronadora": "tronador",
    "tronadoras": "tronador",
    "tronadores": "tronador",
    "tronados": "tronar",
    "tronamos": "tronar",
    "tronando": "tronar",
    "tronara": "tronar",
    "tronarais": "tronar",
    "tronaran": "tronar",
    "tronaras": "tronar",
    "tronare": "tronar",
    "tronareis": "tronar",
    "tronaremos": "tronar",
    "tronaren": "tronar",
    "tronares": "tronar",
    "tronaron": "tronar",
    "tronará": "tronar",
    "tronarán": "tronar",
    "tronarás": "tronar",
    "tronaré": "tronar",
    "tronaréis": "tronar",
    "tronaría": "tronar",
    "tronaríais": "tronar",
    "tronaríamos": "tronar",
    "tronarían": "tronar",
    "tronarías": "tronar",
    "tronas": "trona",
    "tronase": "tronar",
    "tronaseis": "tronar",
    "tronasen": "tronar",
    "tronases": "tronar",
    "tronaste": "tronar",
    "tronasteis": "tronar",
    "tronca": "troncar",
    "troncaba": "troncar",
    "troncabais": "troncar",
    "troncaban": "troncar",
    "troncabas": "troncar",
    "troncad": "troncar",
    "troncada": "troncar",
    "troncadas": "troncar",
    "troncado": "troncar",
    "troncados": "troncar",
    "troncales": "troncal",
    "troncalidades": "troncalidad",
    "troncamos": "troncar",
    "troncan": "troncar",
    "troncando": "troncar",
    "troncara": "troncar",
    "troncarais": "troncar",
    "troncaran": "troncar",
    "troncaras": "troncar",
    "troncare": "troncar",
    "troncareis": "troncar",
    "troncaremos": "troncar",
    "troncaren": "troncar",
    "troncares": "troncar",
    "troncaron": "troncar",
    "troncará": "troncar",
    "troncarán": "troncar",
    "troncarás": "troncar",
    "troncaré": "troncar",
    "troncaréis": "troncar",
    "troncaría": "troncar",
    "troncaríais": "troncar",
    "troncaríamos": "troncar",
    "troncarían": "troncar",
    "troncarías": "troncar",
    "troncas": "troncar",
    "troncase": "troncar",
    "troncaseis": "troncar",
    "troncasen": "troncar",
    "troncases": "troncar",
    "troncaste": "troncar",
    "troncasteis": "troncar",
    "tronce": "tronzar",
    "troncemos": "tronzar",
    "troncen": "tronzar",
    "tronces": "tronzar",
    "troncha": "tronchar",
    "tronchaba": "tronchar",
    "tronchabais": "tronchar",
    "tronchaban": "tronchar",
    "tronchabas": "tronchar",
    "tronchad": "tronchar",
    "tronchada": "tronchar",
    "tronchadas": "tronchar",
    "tronchado": "tronchar",
    "tronchados": "tronchar",
    "tronchamos": "tronchar",
    "tronchan": "tronchar",
    "tronchando": "tronchar",
    "tronchara": "tronchar",
    "troncharais": "tronchar",
    "troncharan": "tronchar",
    "troncharas": "tronchar",
    "tronchare": "tronchar",
    "tronchareis": "tronchar",
    "troncharemos": "tronchar",
    "troncharen": "tronchar",
    "tronchares": "tronchar",
    "troncharon": "tronchar",
    "tronchará": "tronchar",
    "troncharán": "tronchar",
    "troncharás": "tronchar",
    "troncharé": "tronchar",
    "troncharéis": "tronchar",
    "troncharía": "tronchar",
    "troncharíais": "tronchar",
    "troncharíamos": "tronchar",
    "troncharían": "tronchar",
    "troncharías": "tronchar",
    "tronchas": "tronchar",
    "tronchase": "tronchar",
    "tronchaseis": "tronchar",
    "tronchasen": "tronchar",
    "tronchases": "tronchar",
    "tronchaste": "tronchar",
    "tronchasteis": "tronchar",
    "tronchazos": "tronchazo",
    "tronche": "tronchar",
    "tronchemos": "tronchar",
    "tronchen": "tronchar",
    "tronches": "tronchar",
    "troncho": "tronchar",
    "tronchos": "troncho",
    "tronchuda": "tronchudo",
    "tronchudas": "tronchudo",
    "tronchudos": "tronchudo",
    "tronchábamos": "tronchar",
    "troncháis": "tronchar",
    "troncháramos": "tronchar",
    "troncháremos": "tronchar",
    "tronchásemos": "tronchar",
    "tronché": "tronchar",
    "tronchéis": "tronchar",
    "tronchó": "tronchar",
    "tronco": "troncar",
    "troncocónica": "troncocónico",
    "troncocónicas": "troncocónico",
    "troncocónicos": "troncocónico",
    "troncos": "tronco",
    "tronculares": "troncular",
    "troncábamos": "troncar",
    "troncáis": "troncar",
    "troncáramos": "troncar",
    "troncáremos": "troncar",
    "troncásemos": "troncar",
    "troncé": "tronzar",
    "troncéis": "tronzar",
    "troncó": "troncar",
    "tronemos": "tronar",
    "troneras": "tronera",
    "tronidos": "tronido",
    "tronos": "trono",
    "tronque": "troncar",
    "tronquemos": "troncar",
    "tronquen": "troncar",
    "tronques": "troncar",
    "tronquistas": "tronquista",
    "tronqué": "troncar",
    "tronquéis": "troncar",
    "tronza": "tronzo",
    "tronzaba": "tronzar",
    "tronzabais": "tronzar",
    "tronzaban": "tronzar",
    "tronzabas": "tronzar",
    "tronzad": "tronzar",
    "tronzada": "tronzar",
    "tronzadas": "tronzar",
    "tronzado": "tronzar",
    "tronzadora": "tronzador",
    "tronzadoras": "tronzador",
    "tronzadores": "tronzador",
    "tronzados": "tronzar",
    "tronzamos": "tronzar",
    "tronzan": "tronzar",
    "tronzando": "tronzar",
    "tronzara": "tronzar",
    "tronzarais": "tronzar",
    "tronzaran": "tronzar",
    "tronzaras": "tronzar",
    "tronzare": "tronzar",
    "tronzareis": "tronzar",
    "tronzaremos": "tronzar",
    "tronzaren": "tronzar",
    "tronzares": "tronzar",
    "tronzaron": "tronzar",
    "tronzará": "tronzar",
    "tronzarán": "tronzar",
    "tronzarás": "tronzar",
    "tronzaré": "tronzar",
    "tronzaréis": "tronzar",
    "tronzaría": "tronzar",
    "tronzaríais": "tronzar",
    "tronzaríamos": "tronzar",
    "tronzarían": "tronzar",
    "tronzarías": "tronzar",
    "tronzas": "tronzo",
    "tronzase": "tronzar",
    "tronzaseis": "tronzar",
    "tronzasen": "tronzar",
    "tronzases": "tronzar",
    "tronzaste": "tronzar",
    "tronzasteis": "tronzar",
    "tronzo": "tronzar",
    "tronzos": "tronzo",
    "tronzábamos": "tronzar",
    "tronzáis": "tronzar",
    "tronzáramos": "tronzar",
    "tronzáremos": "tronzar",
    "tronzásemos": "tronzar",
    "tronzó": "tronzar",
    "tronábamos": "tronar",
    "tronáis": "tronar",
    "tronáramos": "tronar",
    "tronáremos": "tronar",
    "tronásemos": "tronar",
    "troné": "tronar",
    "tronéis": "tronar",
    "troníos": "tronío",
    "tronó": "tronar",
    "tropas": "tropa",
    "tropecemos": "tropezar",
    "tropecé": "tropezar",
    "tropecéis": "tropezar",
    "tropeles": "tropel",
    "tropelistas": "tropelista",
    "tropelías": "tropelía",
    "tropeolácea": "tropeoláceo",
    "tropeoláceas": "tropeoláceo",
    "tropeoláceos": "tropeoláceo",
    "tropezaba": "tropezar",
    "tropezabais": "tropezar",
    "tropezaban": "tropezar",
    "tropezabas": "tropezar",
    "tropezad": "tropezar",
    "tropezada": "tropezar",
    "tropezadas": "tropezar",
    "tropezaderos": "tropezadero",
    "tropezado": "tropezar",
    "tropezadora": "tropezador",
    "tropezadoras": "tropezador",
    "tropezadores": "tropezador",
    "tropezados": "tropezar",
    "tropezaduras": "tropezadura",
    "tropezamos": "tropezar",
    "tropezando": "tropezar",
    "tropezara": "tropezar",
    "tropezarais": "tropezar",
    "tropezaran": "tropezar",
    "tropezaras": "tropezar",
    "tropezare": "tropezar",
    "tropezareis": "tropezar",
    "tropezaremos": "tropezar",
    "tropezaren": "tropezar",
    "tropezares": "tropezar",
    "tropezaron": "tropezar",
    "tropezará": "tropezar",
    "tropezarán": "tropezar",
    "tropezarás": "tropezar",
    "tropezaré": "tropezar",
    "tropezaréis": "tropezar",
    "tropezaría": "tropezar",
    "tropezaríais": "tropezar",
    "tropezaríamos": "tropezar",
    "tropezarían": "tropezar",
    "tropezarías": "tropezar",
    "tropezase": "tropezar",
    "tropezaseis": "tropezar",
    "tropezasen": "tropezar",
    "tropezases": "tropezar",
    "tropezaste": "tropezar",
    "tropezasteis": "tropezar",
    "tropezona": "tropezón",
    "tropezonas": "tropezón",
    "tropezones": "tropezón",
    "tropezosa": "tropezoso",
    "tropezosas": "tropezoso",
    "tropezosos": "tropezoso",
    "tropezábamos": "tropezar",
    "tropezáis": "tropezar",
    "tropezáramos": "tropezar",
    "tropezáremos": "tropezar",
    "tropezásemos": "tropezar",
    "tropezó": "tropezar",
    "tropicales": "tropical",
    "tropicalizada": "tropicalizado",
    "tropicalizadas": "tropicalizado",
    "tropicalizados": "tropicalizado",
    "tropiece": "tropezar",
    "tropiecen": "tropezar",
    "tropieces": "tropezar",
    "tropieza": "tropezar",
    "tropiezan": "tropezar",
    "tropiezas": "tropezar",
    "tropiezo": "tropezar",
    "tropiezos": "tropiezo",
    "tropismos": "tropismo",
    "tropologías": "tropología",
    "tropológica": "tropológico",
    "tropológicas": "tropológico",
    "tropológicos": "tropológico",
    "tropopausas": "tropopausa",
    "tropos": "tropo",
    "troposferas": "troposfera",
    "troposférica": "troposférico",
    "troposféricas": "troposférico",
    "troposféricos": "troposférico",
    "tropías": "tropía",
    "troquela": "troquelar",
    "troquelaba": "troquelar",
    "troquelabais": "troquelar",
    "troquelaban": "troquelar",
    "troquelabas": "troquelar",
    "troquelad": "troquelar",
    "troquelada": "troquelar",
    "troqueladas": "troquelar",
    "troquelado": "troquelar",
    "troqueladora": "troquelador",
    "troqueladoras": "troquelador",
    "troqueladores": "troquelador",
    "troquelados": "troquelar",
    "troquelamos": "troquelar",
    "troquelan": "troquelar",
    "troquelando": "troquelar",
    "troquelara": "troquelar",
    "troquelarais": "troquelar",
    "troquelaran": "troquelar",
    "troquelaras": "troquelar",
    "troquelare": "troquelar",
    "troquelareis": "troquelar",
    "troquelaremos": "troquelar",
    "troquelaren": "troquelar",
    "troquelares": "troquelar",
    "troquelaron": "troquelar",
    "troquelará": "troquelar",
    "troquelarán": "troquelar",
    "troquelarás": "troquelar",
    "troquelaré": "troquelar",
    "troquelaréis": "troquelar",
    "troquelaría": "troquelar",
    "troquelaríais": "troquelar",
    "troquelaríamos": "troquelar",
    "troquelarían": "troquelar",
    "troquelarías": "troquelar",
    "troquelas": "troquelar",
    "troquelase": "troquelar",
    "troquelaseis": "troquelar",
    "troquelasen": "troquelar",
    "troquelases": "troquelar",
    "troquelaste": "troquelar",
    "troquelasteis": "troquelar",
    "troquele": "troquelar",
    "troquelemos": "troquelar",
    "troquelen": "troquelar",
    "troqueles": "troquelar",
    "troquelistas": "troquelista",
    "troquelo": "troquelar",
    "troquelábamos": "troquelar",
    "troqueláis": "troquelar",
    "troqueláramos": "troquelar",
    "troqueláremos": "troquelar",
    "troquelásemos": "troquelar",
    "troquelé": "troquelar",
    "troqueléis": "troquelar",
    "troqueló": "troquelar",
    "troquemos": "trocar",
    "troqueos": "troqueo",
    "troquilos": "troquilo",
    "troqué": "trocar",
    "troquéis": "trocar",
    "trota": "trotar",
    "trotaba": "trotar",
    "trotabais": "trotar",
    "trotaban": "trotar",
    "trotabas": "trotar",
    "trotad": "trotar",
    "trotada": "trotar",
    "trotadas": "trotar",
    "trotado": "trotar",
    "trotadora": "trotador",
    "trotadoras": "trotador",
    "trotadores": "trotador",
    "trotados": "trotar",
    "trotamos": "trotar",
    "trotan": "trotar",
    "trotando": "trotar",
    "trotara": "trotar",
    "trotarais": "trotar",
    "trotaran": "trotar",
    "trotaras": "trotar",
    "trotare": "trotar",
    "trotareis": "trotar",
    "trotaremos": "trotar",
    "trotaren": "trotar",
    "trotares": "trotar",
    "trotaron": "trotar",
    "trotará": "trotar",
    "trotarán": "trotar",
    "trotarás": "trotar",
    "trotaré": "trotar",
    "trotaréis": "trotar",
    "trotaría": "trotar",
    "trotaríais": "trotar",
    "trotaríamos": "trotar",
    "trotarían": "trotar",
    "trotarías": "trotar",
    "trotas": "trotar",
    "trotase": "trotar",
    "trotaseis": "trotar",
    "trotasen": "trotar",
    "trotases": "trotar",
    "trotaste": "trotar",
    "trotasteis": "trotar",
    "trote": "trotar",
    "trotemos": "trotar",
    "troten": "trotar",
    "trotes": "trote",
    "troto": "trotar",
    "trotona": "trotón",
    "trotonas": "trotón",
    "trotonerías": "trotonería",
    "trotones": "trotón",
    "trotskismos": "trotskismo",
    "trotskistas": "trotskista",
    "trotábamos": "trotar",
    "trotáis": "trotar",
    "trotáramos": "trotar",
    "trotáremos": "trotar",
    "trotásemos": "trotar",
    "troté": "trotar",
    "trotéis": "trotar",
    "trotó": "trotar",
    "troupes": "troupe",
    "trova": "trovar",
    "trovaba": "trovar",
    "trovabais": "trovar",
    "trovaban": "trovar",
    "trovabas": "trovar",
    "trovad": "trovar",
    "trovada": "trovar",
    "trovadas": "trovar",
    "trovado": "trovar",
    "trovadora": "trovador",
    "trovadoras": "trovador",
    "trovadores": "trovador",
    "trovadoresca": "trovadoresco",
    "trovadorescas": "trovadoresco",
    "trovadorescos": "trovadoresco",
    "trovados": "trovar",
    "trovamos": "trovar",
    "trovan": "trovar",
    "trovando": "trovar",
    "trovara": "trovar",
    "trovarais": "trovar",
    "trovaran": "trovar",
    "trovaras": "trovar",
    "trovare": "trovar",
    "trovareis": "trovar",
    "trovaremos": "trovar",
    "trovaren": "trovar",
    "trovares": "trovar",
    "trovaron": "trovar",
    "trovará": "trovar",
    "trovarán": "trovar",
    "trovarás": "trovar",
    "trovaré": "trovar",
    "trovaréis": "trovar",
    "trovaría": "trovar",
    "trovaríais": "trovar",
    "trovaríamos": "trovar",
    "trovarían": "trovar",
    "trovarías": "trovar",
    "trovas": "trovar",
    "trovase": "trovar",
    "trovaseis": "trovar",
    "trovasen": "trovar",
    "trovases": "trovar",
    "trovaste": "trovar",
    "trovasteis": "trovar",
    "trove": "trovar",
    "trovemos": "trovar",
    "troven": "trovar",
    "trovera": "trovero",
    "troveras": "trovero",
    "troveros": "trovero",
    "troves": "trovar",
    "trovistas": "trovista",
    "trovo": "trovar",
    "trovos": "trovo",
    "trovábamos": "trovar",
    "trováis": "trovar",
    "trováramos": "trovar",
    "trováremos": "trovar",
    "trovásemos": "trovar",
    "trové": "trovar",
    "trovéis": "trovar",
    "trovó": "trovar",
    "troyana": "troyano",
    "troyanas": "troyano",
    "troyanos": "troyano",
    "troza": "trozar",
    "trozaba": "trozar",
    "trozabais": "trozar",
    "trozaban": "trozar",
    "trozabas": "trozar",
    "trozad": "trozar",
    "trozada": "trozar",
    "trozadas": "trozar",
    "trozado": "trozar",
    "trozados": "trozar",
    "trozamientos": "trozamiento",
    "trozamos": "trozar",
    "trozan": "trozar",
    "trozando": "trozar",
    "trozara": "trozar",
    "trozarais": "trozar",
    "trozaran": "trozar",
    "trozaras": "trozar",
    "trozare": "trozar",
    "trozareis": "trozar",
    "trozaremos": "trozar",
    "trozaren": "trozar",
    "trozares": "trozar",
    "trozaron": "trozar",
    "trozará": "trozar",
    "trozarán": "trozar",
    "trozarás": "trozar",
    "trozaré": "trozar",
    "trozaréis": "trozar",
    "trozaría": "trozar",
    "trozaríais": "trozar",
    "trozaríamos": "trozar",
    "trozarían": "trozar",
    "trozarías": "trozar",
    "trozas": "trozar",
    "trozase": "trozar",
    "trozaseis": "trozar",
    "trozasen": "trozar",
    "trozases": "trozar",
    "trozaste": "trozar",
    "trozasteis": "trozar",
    "trozo": "trozar",
    "trozos": "trozo",
    "trozábamos": "trozar",
    "trozáis": "trozar",
    "trozáramos": "trozar",
    "trozáremos": "trozar",
    "trozásemos": "trozar",
    "trozó": "trozar",
    "truca": "trucar",
    "trucaba": "trucar",
    "trucabais": "trucar",
    "trucaban": "trucar",
    "trucabas": "trucar",
    "trucad": "trucar",
    "trucada": "trucar",
    "trucadas": "trucar",
    "trucado": "trucar",
    "trucados": "trucar",
    "trucajes": "trucaje",
    "trucamos": "trucar",
    "trucan": "trucar",
    "trucando": "trucar",
    "trucara": "trucar",
    "trucarais": "trucar",
    "trucaran": "trucar",
    "trucaras": "trucar",
    "trucare": "trucar",
    "trucareis": "trucar",
    "trucaremos": "trucar",
    "trucaren": "trucar",
    "trucares": "trucar",
    "trucaron": "trucar",
    "trucará": "trucar",
    "trucarán": "trucar",
    "trucarás": "trucar",
    "trucaré": "trucar",
    "trucaréis": "trucar",
    "trucaría": "trucar",
    "trucaríais": "trucar",
    "trucaríamos": "trucar",
    "trucarían": "trucar",
    "trucarías": "trucar",
    "trucas": "trucar",
    "trucase": "trucar",
    "trucaseis": "trucar",
    "trucasen": "trucar",
    "trucases": "trucar",
    "trucaste": "trucar",
    "trucasteis": "trucar",
    "truchas": "trucha",
    "truchera": "truchero",
    "trucheras": "truchero",
    "trucheros": "truchero",
    "truchimana": "truchimán",
    "truchimanas": "truchimán",
    "truchimanes": "truchimán",
    "truchuelas": "truchuela",
    "trucida": "trucidar",
    "trucidaba": "trucidar",
    "trucidabais": "trucidar",
    "trucidaban": "trucidar",
    "trucidabas": "trucidar",
    "trucidad": "trucidar",
    "trucidada": "trucidar",
    "trucidadas": "trucidar",
    "trucidado": "trucidar",
    "trucidados": "trucidar",
    "trucidamos": "trucidar",
    "trucidan": "trucidar",
    "trucidando": "trucidar",
    "trucidara": "trucidar",
    "trucidarais": "trucidar",
    "trucidaran": "trucidar",
    "trucidaras": "trucidar",
    "trucidare": "trucidar",
    "trucidareis": "trucidar",
    "trucidaremos": "trucidar",
    "trucidaren": "trucidar",
    "trucidares": "trucidar",
    "trucidaron": "trucidar",
    "trucidará": "trucidar",
    "trucidarán": "trucidar",
    "trucidarás": "trucidar",
    "trucidaré": "trucidar",
    "trucidaréis": "trucidar",
    "trucidaría": "trucidar",
    "trucidaríais": "trucidar",
    "trucidaríamos": "trucidar",
    "trucidarían": "trucidar",
    "trucidarías": "trucidar",
    "trucidas": "trucidar",
    "trucidase": "trucidar",
    "trucidaseis": "trucidar",
    "trucidasen": "trucidar",
    "trucidases": "trucidar",
    "trucidaste": "trucidar",
    "trucidasteis": "trucidar",
    "trucide": "trucidar",
    "trucidemos": "trucidar",
    "truciden": "trucidar",
    "trucides": "trucidar",
    "trucido": "trucidar",
    "trucidábamos": "trucidar",
    "trucidáis": "trucidar",
    "trucidáramos": "trucidar",
    "trucidáremos": "trucidar",
    "trucidásemos": "trucidar",
    "trucidé": "trucidar",
    "trucidéis": "trucidar",
    "trucidó": "trucidar",
    "truco": "trucar",
    "trucos": "truco",
    "truculencias": "truculencia",
    "truculenta": "truculento",
    "truculentas": "truculento",
    "truculentos": "truculento",
    "trucábamos": "trucar",
    "trucáis": "trucar",
    "trucáramos": "trucar",
    "trucáremos": "trucar",
    "trucásemos": "trucar",
    "trucó": "trucar",
    "trueca": "trocar",
    "truecan": "trocar",
    "truecas": "trocar",
    "trueco": "trocar",
    "truecos": "trueco",
    "truena": "tronar",
    "truenan": "tronar",
    "truenas": "tronar",
    "truene": "tronar",
    "truenen": "tronar",
    "truenes": "tronar",
    "trueno": "tronar",
    "truenos": "trueno",
    "trueque": "trocar",
    "truequen": "trocar",
    "trueques": "trueque",
    "trufa": "trufar",
    "trufaba": "trufar",
    "trufabais": "trufar",
    "trufaban": "trufar",
    "trufabas": "trufar",
    "trufad": "trufar",
    "trufada": "trufar",
    "trufadas": "trufar",
    "trufado": "trufar",
    "trufados": "trufar",
    "trufamos": "trufar",
    "trufan": "trufar",
    "trufando": "trufar",
    "trufara": "trufar",
    "trufarais": "trufar",
    "trufaran": "trufar",
    "trufaras": "trufar",
    "trufare": "trufar",
    "trufareis": "trufar",
    "trufaremos": "trufar",
    "trufaren": "trufar",
    "trufares": "trufar",
    "trufaron": "trufar",
    "trufará": "trufar",
    "trufarán": "trufar",
    "trufarás": "trufar",
    "trufaré": "trufar",
    "trufaréis": "trufar",
    "trufaría": "trufar",
    "trufaríais": "trufar",
    "trufaríamos": "trufar",
    "trufarían": "trufar",
    "trufarías": "trufar",
    "trufas": "trufar",
    "trufase": "trufar",
    "trufaseis": "trufar",
    "trufasen": "trufar",
    "trufases": "trufar",
    "trufaste": "trufar",
    "trufasteis": "trufar",
    "trufe": "trufar",
    "trufemos": "trufar",
    "trufen": "trufar",
    "trufes": "trufar",
    "trufo": "trufar",
    "trufábamos": "trufar",
    "trufáis": "trufar",
    "trufáramos": "trufar",
    "trufáremos": "trufar",
    "trufásemos": "trufar",
    "trufé": "trufar",
    "truféis": "trufar",
    "trufó": "trufar",
    "truhana": "truhán",
    "truhanadas": "truhanada",
    "truhanas": "truhán",
    "truhanea": "truhanear",
    "truhaneaba": "truhanear",
    "truhaneabais": "truhanear",
    "truhaneaban": "truhanear",
    "truhaneabas": "truhanear",
    "truhanead": "truhanear",
    "truhaneada": "truhanear",
    "truhaneadas": "truhanear",
    "truhaneado": "truhanear",
    "truhaneados": "truhanear",
    "truhaneamos": "truhanear",
    "truhanean": "truhanear",
    "truhaneando": "truhanear",
    "truhaneara": "truhanear",
    "truhanearais": "truhanear",
    "truhanearan": "truhanear",
    "truhanearas": "truhanear",
    "truhaneare": "truhanear",
    "truhaneareis": "truhanear",
    "truhanearemos": "truhanear",
    "truhanearen": "truhanear",
    "truhaneares": "truhanear",
    "truhanearon": "truhanear",
    "truhaneará": "truhanear",
    "truhanearán": "truhanear",
    "truhanearás": "truhanear",
    "truhanearé": "truhanear",
    "truhanearéis": "truhanear",
    "truhanearía": "truhanear",
    "truhanearíais": "truhanear",
    "truhanearíamos": "truhanear",
    "truhanearían": "truhanear",
    "truhanearías": "truhanear",
    "truhaneas": "truhanear",
    "truhanease": "truhanear",
    "truhaneaseis": "truhanear",
    "truhaneasen": "truhanear",
    "truhaneases": "truhanear",
    "truhaneaste": "truhanear",
    "truhaneasteis": "truhanear",
    "truhanee": "truhanear",
    "truhaneemos": "truhanear",
    "truhaneen": "truhanear",
    "truhanees": "truhanear",
    "truhaneo": "truhanear",
    "truhanerías": "truhanería",
    "truhanes": "truhán",
    "truhanesca": "truhanesco",
    "truhanescas": "truhanesco",
    "truhanescos": "truhanesco",
    "truhaneábamos": "truhanear",
    "truhaneáis": "truhanear",
    "truhaneáramos": "truhanear",
    "truhaneáremos": "truhanear",
    "truhaneásemos": "truhanear",
    "truhaneé": "truhanear",
    "truhaneéis": "truhanear",
    "truhaneó": "truhanear",
    "trujales": "trujal",
    "trujamana": "trujamán",
    "trujamanas": "trujamán",
    "trujamanea": "trujamanear",
    "trujamaneaba": "trujamanear",
    "trujamaneabais": "trujamanear",
    "trujamaneaban": "trujamanear",
    "trujamaneabas": "trujamanear",
    "trujamanead": "trujamanear",
    "trujamaneada": "trujamanear",
    "trujamaneadas": "trujamanear",
    "trujamaneado": "trujamanear",
    "trujamaneados": "trujamanear",
    "trujamaneamos": "trujamanear",
    "trujamanean": "trujamanear",
    "trujamaneando": "trujamanear",
    "trujamaneara": "trujamanear",
    "trujamanearais": "trujamanear",
    "trujamanearan": "trujamanear",
    "trujamanearas": "trujamanear",
    "trujamaneare": "trujamanear",
    "trujamaneareis": "trujamanear",
    "trujamanearemos": "trujamanear",
    "trujamanearen": "trujamanear",
    "trujamaneares": "trujamanear",
    "trujamanearon": "trujamanear",
    "trujamaneará": "trujamanear",
    "trujamanearán": "trujamanear",
    "trujamanearás": "trujamanear",
    "trujamanearé": "trujamanear",
    "trujamanearéis": "trujamanear",
    "trujamanearía": "trujamanear",
    "trujamanearíais": "trujamanear",
    "trujamanearíamos": "trujamanear",
    "trujamanearían": "trujamanear",
    "trujamanearías": "trujamanear",
    "trujamaneas": "trujamanear",
    "trujamanease": "trujamanear",
    "trujamaneaseis": "trujamanear",
    "trujamaneasen": "trujamanear",
    "trujamaneases": "trujamanear",
    "trujamaneaste": "trujamanear",
    "trujamaneasteis": "trujamanear",
    "trujamanee": "trujamanear",
    "trujamaneemos": "trujamanear",
    "trujamaneen": "trujamanear",
    "trujamanees": "trujamanear",
    "trujamaneo": "trujamanear",
    "trujamanes": "trujamán",
    "trujamaneábamos": "trujamanear",
    "trujamaneáis": "trujamanear",
    "trujamaneáramos": "trujamanear",
    "trujamaneáremos": "trujamanear",
    "trujamaneásemos": "trujamanear",
    "trujamaneé": "trujamanear",
    "trujamaneéis": "trujamanear",
    "trujamaneó": "trujamanear",
    "trujamanías": "trujamanía",
    "trullas": "trulla",
    "trullos": "trullo",
    "trunca": "trunco",
    "truncaba": "truncar",
    "truncabais": "truncar",
    "truncaban": "truncar",
    "truncabas": "truncar",
    "truncad": "truncar",
    "truncada": "truncar",
    "truncadas": "truncar",
    "truncado": "truncar",
    "truncados": "truncar",
    "truncamientos": "truncamiento",
    "truncamos": "truncar",
    "truncan": "truncar",
    "truncando": "truncar",
    "truncara": "truncar",
    "truncarais": "truncar",
    "truncaran": "truncar",
    "truncaras": "truncar",
    "truncare": "truncar",
    "truncareis": "truncar",
    "truncaremos": "truncar",
    "truncaren": "truncar",
    "truncares": "truncar",
    "truncaron": "truncar",
    "truncará": "truncar",
    "truncarán": "truncar",
    "truncarás": "truncar",
    "truncaré": "truncar",
    "truncaréis": "truncar",
    "truncaría": "truncar",
    "truncaríais": "truncar",
    "truncaríamos": "truncar",
    "truncarían": "truncar",
    "truncarías": "truncar",
    "truncas": "trunco",
    "truncase": "truncar",
    "truncaseis": "truncar",
    "truncasen": "truncar",
    "truncases": "truncar",
    "truncaste": "truncar",
    "truncasteis": "truncar",
    "trunco": "truncar",
    "truncos": "trunco",
    "truncábamos": "truncar",
    "truncáis": "truncar",
    "truncáramos": "truncar",
    "truncáremos": "truncar",
    "truncásemos": "truncar",
    "truncó": "truncar",
    "trunque": "truncar",
    "trunquemos": "truncar",
    "trunquen": "truncar",
    "trunques": "truncar",
    "trunqué": "truncar",
    "trunquéis": "truncar",
    "trupiales": "trupial",
    "truque": "trucar",
    "truquemos": "trucar",
    "truquen": "trucar",
    "truques": "truque",
    "truqué": "trucar",
    "truquéis": "trucar",
    "trustes": "trust",
    "trusts": "trust",
    "truticulturas": "truticultura",
    "trábeas": "trábea",
    "tráfagos": "tráfago",
    "tráficos": "tráfico",
    "trágalas": "trágala",
    "trágica": "trágico",
    "trágicas": "trágico",
    "trágicos": "trágico",
    "tráileres": "tráiler",
    "tráilers": "tráiler",
    "trámites": "trámite",
    "tránsfugas": "tránsfuga",
    "tránsfugos": "tránsfugo",
    "tránsitos": "tránsito",
    "tráqueas": "tráquea",
    "trásfuga": "trásfugo",
    "trásfugas": "trásfugo",
    "trásfugos": "trásfugo",
    "trébedes": "trébede",
    "tréboles": "trébol",
    "trémolos": "trémolo",
    "trémula": "trémulo",
    "trémulas": "trémulo",
    "trémulos": "trémulo",
    "trépanos": "trépano",
    "trépida": "trépido",
    "trépidas": "trépido",
    "trépidos": "trépido",
    "tría": "triar",
    "tríadas": "tríada",
    "trían": "triar",
    "trías": "triar",
    "tríbulos": "tríbulo",
    "tríclados": "tríclado",
    "tríe": "triar",
    "tríen": "triar",
    "tríes": "triar",
    "trífida": "trífido",
    "trífidas": "trífido",
    "trífidos": "trífido",
    "tríglifos": "tríglifo",
    "trígonos": "trígono",
    "trímera": "trímero",
    "trímeras": "trímero",
    "trímeros": "trímero",
    "trímetros": "trímetro",
    "trío": "triar",
    "tríos": "trío",
    "tríplices": "tríplice",
    "trípodes": "trípode",
    "trípolis": "trípoli",
    "tríptica": "tríptico",
    "trípticas": "tríptico",
    "trípticos": "tríptico",
    "tríptones": "tripton",
    "trítonos": "trítono",
    "trócalas": "trócala",
    "trócleas": "tróclea",
    "trócolas": "trócola",
    "trófica": "trófico",
    "tróficas": "trófico",
    "tróficos": "trófico",
    "trómeles": "trómel",
    "trónicas": "trónica",
    "trópica": "trópico",
    "trópicas": "trópico",
    "trópicos": "trópico",
    "tsavoritas": "tsavorita",
    "tsetsés": "tsetsé",
    "tuaregs": "tuareg",
    "tubaria": "tubario",
    "tubarias": "tubario",
    "tubarios": "tubario",
    "tubas": "tuba",
    "tuberales": "tuberal",
    "tuberculares": "tubercular",
    "tuberculice": "tuberculizar",
    "tuberculicemos": "tuberculizar",
    "tuberculicen": "tuberculizar",
    "tuberculices": "tuberculizar",
    "tuberculicé": "tuberculizar",
    "tuberculicéis": "tuberculizar",
    "tuberculinas": "tuberculina",
    "tuberculinismos": "tuberculinismo",
    "tuberculiza": "tuberculizar",
    "tuberculizaba": "tuberculizar",
    "tuberculizabais": "tuberculizar",
    "tuberculizaban": "tuberculizar",
    "tuberculizabas": "tuberculizar",
    "tuberculizaciones": "tuberculización",
    "tuberculizad": "tuberculizar",
    "tuberculizada": "tuberculizar",
    "tuberculizadas": "tuberculizar",
    "tuberculizado": "tuberculizar",
    "tuberculizados": "tuberculizar",
    "tuberculizamos": "tuberculizar",
    "tuberculizan": "tuberculizar",
    "tuberculizando": "tuberculizar",
    "tuberculizara": "tuberculizar",
    "tuberculizarais": "tuberculizar",
    "tuberculizaran": "tuberculizar",
    "tuberculizaras": "tuberculizar",
    "tuberculizare": "tuberculizar",
    "tuberculizareis": "tuberculizar",
    "tuberculizaremos": "tuberculizar",
    "tuberculizaren": "tuberculizar",
    "tuberculizares": "tuberculizar",
    "tuberculizaron": "tuberculizar",
    "tuberculizará": "tuberculizar",
    "tuberculizarán": "tuberculizar",
    "tuberculizarás": "tuberculizar",
    "tuberculizaré": "tuberculizar",
    "tuberculizaréis": "tuberculizar",
    "tuberculizaría": "tuberculizar",
    "tuberculizaríais": "tuberculizar",
    "tuberculizaríamos": "tuberculizar",
    "tuberculizarían": "tuberculizar",
    "tuberculizarías": "tuberculizar",
    "tuberculizas": "tuberculizar",
    "tuberculizase": "tuberculizar",
    "tuberculizaseis": "tuberculizar",
    "tuberculizasen": "tuberculizar",
    "tuberculizases": "tuberculizar",
    "tuberculizaste": "tuberculizar",
    "tuberculizasteis": "tuberculizar",
    "tuberculizo": "tuberculizar",
    "tuberculizábamos": "tuberculizar",
    "tuberculizáis": "tuberculizar",
    "tuberculizáramos": "tuberculizar",
    "tuberculizáremos": "tuberculizar",
    "tuberculizásemos": "tuberculizar",
    "tuberculizó": "tuberculizar",
    "tuberculomas": "tuberculoma",
    "tuberculosa": "tuberculoso",
    "tuberculosas": "tuberculoso",
    "tuberculosos": "tuberculoso",
    "tuberculínica": "tuberculínico",
    "tuberculínicas": "tuberculínico",
    "tuberculínicos": "tuberculínico",
    "tuberosa": "tuberoso",
    "tuberosas": "tuberoso",
    "tuberosidades": "tuberosidad",
    "tuberosos": "tuberoso",
    "tuberías": "tubería",
    "tubifloras": "tubiflora",
    "tubos": "tubo",
    "tubulares": "tubular",
    "tubuliformes": "tubuliforme",
    "tubulosa": "tubuloso",
    "tubulosas": "tubuloso",
    "tubulosos": "tubuloso",
    "tubárica": "tubárico",
    "tubáricas": "tubárico",
    "tubáricos": "tubárico",
    "tubérculos": "tubérculo",
    "tucanes": "tucán",
    "tuciorismos": "tuciorismo",
    "tucuiricos": "tucuirico",
    "tucumana": "tucumano",
    "tucumanas": "tucumano",
    "tucumanos": "tucumano",
    "tucutucos": "tucutuco",
    "tudelana": "tudelano",
    "tudelanas": "tudelano",
    "tudelanos": "tudelano",
    "tudeles": "tudel",
    "tudesca": "tudesco",
    "tudescas": "tudesco",
    "tudescos": "tudesco",
    "tueca": "tueco",
    "tuecas": "tueco",
    "tuecos": "tueco",
    "tuena": "tonar",
    "tuenan": "tonar",
    "tuenas": "tonar",
    "tuene": "tonar",
    "tuenen": "tonar",
    "tuenes": "tonar",
    "tueno": "tonar",
    "tuercas": "tuerca",
    "tuerce": "torcer",
    "tuercen": "torcer",
    "tuerces": "tuerce",
    "tueros": "tuero",
    "tuerta": "tuerto",
    "tuertas": "tuerto",
    "tuertos": "tuerto",
    "tuerza": "torcer",
    "tuerzan": "torcer",
    "tuerzas": "torcer",
    "tuerzo": "torcer",
    "tuesta": "tostar",
    "tuestan": "tostar",
    "tuestas": "tostar",
    "tueste": "tostar",
    "tuesten": "tostar",
    "tuestes": "tueste",
    "tuesto": "tostar",
    "tufaradas": "tufarada",
    "tufos": "tufo",
    "tugriks": "tugrik",
    "tugtupitas": "tugtupita",
    "tugurios": "tugurio",
    "tuiciones": "tuición",
    "tuistores": "tuistor",
    "tuitiva": "tuitivo",
    "tuitivas": "tuitivo",
    "tuitivos": "tuitivo",
    "tularemias": "tularemia",
    "tules": "tul",
    "tulios": "tulio",
    "tulipaneros": "tulipanero",
    "tulipanes": "tulipán",
    "tulipas": "tulipa",
    "tuliperos": "tulipero",
    "tulla": "tullir",
    "tullamos": "tullir",
    "tullan": "tullir",
    "tullas": "tullir",
    "tulle": "tullir",
    "tullece": "tullecer",
    "tulleced": "tullecer",
    "tullecemos": "tullecer",
    "tullecen": "tullecer",
    "tulleceremos": "tullecer",
    "tullecerá": "tullecer",
    "tullecerán": "tullecer",
    "tullecerás": "tullecer",
    "tulleceré": "tullecer",
    "tulleceréis": "tullecer",
    "tullecería": "tullecer",
    "tulleceríais": "tullecer",
    "tulleceríamos": "tullecer",
    "tullecerían": "tullecer",
    "tullecerías": "tullecer",
    "tulleces": "tullecer",
    "tullecida": "tullecer",
    "tullecidas": "tullecer",
    "tullecido": "tullecer",
    "tullecidos": "tullecer",
    "tulleciendo": "tullecer",
    "tulleciera": "tullecer",
    "tullecierais": "tullecer",
    "tullecieran": "tullecer",
    "tullecieras": "tullecer",
    "tulleciere": "tullecer",
    "tulleciereis": "tullecer",
    "tullecieren": "tullecer",
    "tullecieres": "tullecer",
    "tullecieron": "tullecer",
    "tulleciese": "tullecer",
    "tullecieseis": "tullecer",
    "tulleciesen": "tullecer",
    "tullecieses": "tullecer",
    "tullecimos": "tullecer",
    "tulleciste": "tullecer",
    "tullecisteis": "tullecer",
    "tulleciéramos": "tullecer",
    "tulleciéremos": "tullecer",
    "tulleciésemos": "tullecer",
    "tulleció": "tullecer",
    "tullecéis": "tullecer",
    "tullecí": "tullecer",
    "tullecía": "tullecer",
    "tullecíais": "tullecer",
    "tullecíamos": "tullecer",
    "tullecían": "tullecer",
    "tullecías": "tullecer",
    "tullen": "tullir",
    "tullendo": "tullir",
    "tullera": "tullir",
    "tullerais": "tullir",
    "tulleran": "tullir",
    "tulleras": "tullir",
    "tullere": "tullir",
    "tullereis": "tullir",
    "tulleren": "tullir",
    "tulleres": "tullir",
    "tulleron": "tullir",
    "tulles": "tullir",
    "tullese": "tullir",
    "tulleseis": "tullir",
    "tullesen": "tullir",
    "tulleses": "tullir",
    "tullezca": "tullecer",
    "tullezcamos": "tullecer",
    "tullezcan": "tullecer",
    "tullezcas": "tullecer",
    "tullezco": "tullecer",
    "tullezcáis": "tullecer",
    "tullid": "tullir",
    "tullida": "tullir",
    "tullidas": "tullir",
    "tullideces": "tullidez",
    "tullido": "tullir",
    "tullidos": "tullir",
    "tulliduras": "tullidura",
    "tullimientos": "tullimiento",
    "tullimos": "tullir",
    "tulliremos": "tullir",
    "tullirá": "tullir",
    "tullirán": "tullir",
    "tullirás": "tullir",
    "tulliré": "tullir",
    "tulliréis": "tullir",
    "tulliría": "tullir",
    "tulliríais": "tullir",
    "tulliríamos": "tullir",
    "tullirían": "tullir",
    "tullirías": "tullir",
    "tulliste": "tullir",
    "tullisteis": "tullir",
    "tullo": "tullir",
    "tulláis": "tullir",
    "tulléramos": "tullir",
    "tulléremos": "tullir",
    "tullésemos": "tullir",
    "tullí": "tullir",
    "tullía": "tullir",
    "tullíais": "tullir",
    "tullíamos": "tullir",
    "tullían": "tullir",
    "tullías": "tullir",
    "tullís": "tullir",
    "tulló": "tullir",
    "tuluníes": "tuluní",
    "tulunís": "tuluní",
    "tumba": "tumbar",
    "tumbaba": "tumbar",
    "tumbabais": "tumbar",
    "tumbaban": "tumbar",
    "tumbabas": "tumbar",
    "tumbad": "tumbar",
    "tumbada": "tumbar",
    "tumbadas": "tumbar",
    "tumbado": "tumbar",
    "tumbados": "tumbar",
    "tumbagas": "tumbaga",
    "tumbales": "tumbal",
    "tumbamos": "tumbar",
    "tumban": "tumbar",
    "tumbando": "tumbar",
    "tumbara": "tumbar",
    "tumbarais": "tumbar",
    "tumbaran": "tumbar",
    "tumbaras": "tumbar",
    "tumbare": "tumbar",
    "tumbareis": "tumbar",
    "tumbaremos": "tumbar",
    "tumbaren": "tumbar",
    "tumbares": "tumbar",
    "tumbaron": "tumbar",
    "tumbará": "tumbar",
    "tumbarán": "tumbar",
    "tumbarás": "tumbar",
    "tumbaré": "tumbar",
    "tumbaréis": "tumbar",
    "tumbaría": "tumbar",
    "tumbaríais": "tumbar",
    "tumbaríamos": "tumbar",
    "tumbarían": "tumbar",
    "tumbarías": "tumbar",
    "tumbas": "tumbar",
    "tumbase": "tumbar",
    "tumbaseis": "tumbar",
    "tumbasen": "tumbar",
    "tumbases": "tumbar",
    "tumbaste": "tumbar",
    "tumbasteis": "tumbar",
    "tumbe": "tumbar",
    "tumbemos": "tumbar",
    "tumben": "tumbar",
    "tumbes": "tumbar",
    "tumbillas": "tumbilla",
    "tumbo": "tumbar",
    "tumbona": "tumbón",
    "tumbonas": "tumbona",
    "tumbones": "tumbón",
    "tumbos": "tumbo",
    "tumbábamos": "tumbar",
    "tumbáis": "tumbar",
    "tumbáramos": "tumbar",
    "tumbáremos": "tumbar",
    "tumbásemos": "tumbar",
    "tumbé": "tumbar",
    "tumbéis": "tumbar",
    "tumbó": "tumbar",
    "tumefacciones": "tumefacción",
    "tumefacta": "tumefacto",
    "tumefactas": "tumefacto",
    "tumefactos": "tumefacto",
    "tumescencias": "tumescencia",
    "tumescentes": "tumescente",
    "tumoraciones": "tumoración",
    "tumorales": "tumoral",
    "tumores": "tumor",
    "tumorosa": "tumoroso",
    "tumorosas": "tumoroso",
    "tumorosos": "tumoroso",
    "tumularia": "tumulario",
    "tumularias": "tumulario",
    "tumularios": "tumulario",
    "tumultos": "tumulto",
    "tumultuaba": "tumultuar",
    "tumultuabais": "tumultuar",
    "tumultuaban": "tumultuar",
    "tumultuabas": "tumultuar",
    "tumultuad": "tumultuar",
    "tumultuada": "tumultuar",
    "tumultuadas": "tumultuar",
    "tumultuado": "tumultuar",
    "tumultuados": "tumultuar",
    "tumultuamos": "tumultuar",
    "tumultuando": "tumultuar",
    "tumultuara": "tumultuar",
    "tumultuarais": "tumultuar",
    "tumultuaran": "tumultuar",
    "tumultuaras": "tumultuar",
    "tumultuare": "tumultuar",
    "tumultuareis": "tumultuar",
    "tumultuaremos": "tumultuar",
    "tumultuaren": "tumultuar",
    "tumultuares": "tumultuar",
    "tumultuaria": "tumultuario",
    "tumultuarias": "tumultuario",
    "tumultuarios": "tumultuario",
    "tumultuaron": "tumultuar",
    "tumultuará": "tumultuar",
    "tumultuarán": "tumultuar",
    "tumultuarás": "tumultuar",
    "tumultuaré": "tumultuar",
    "tumultuaréis": "tumultuar",
    "tumultuaría": "tumultuar",
    "tumultuaríais": "tumultuar",
    "tumultuaríamos": "tumultuar",
    "tumultuarían": "tumultuar",
    "tumultuarías": "tumultuar",
    "tumultuase": "tumultuar",
    "tumultuaseis": "tumultuar",
    "tumultuasen": "tumultuar",
    "tumultuases": "tumultuar",
    "tumultuaste": "tumultuar",
    "tumultuasteis": "tumultuar",
    "tumultuemos": "tumultuar",
    "tumultuosa": "tumultuoso",
    "tumultuosas": "tumultuoso",
    "tumultuosos": "tumultuoso",
    "tumultuábamos": "tumultuar",
    "tumultuáis": "tumultuar",
    "tumultuáramos": "tumultuar",
    "tumultuáremos": "tumultuar",
    "tumultuásemos": "tumultuar",
    "tumultué": "tumultuar",
    "tumultuéis": "tumultuar",
    "tumultuó": "tumultuar",
    "tumultúa": "tumultuar",
    "tumultúan": "tumultuar",
    "tumultúas": "tumultuar",
    "tumultúe": "tumultuar",
    "tumultúen": "tumultuar",
    "tumultúes": "tumultuar",
    "tumultúo": "tumultuar",
    "tuna": "tuno",
    "tunaba": "tunar",
    "tunabais": "tunar",
    "tunaban": "tunar",
    "tunabas": "tunar",
    "tunad": "tunar",
    "tunada": "tunar",
    "tunadas": "tunar",
    "tunado": "tunar",
    "tunados": "tunar",
    "tunales": "tunal",
    "tunamos": "tunar",
    "tunan": "tunar",
    "tunando": "tunar",
    "tunanta": "tunante",
    "tunantadas": "tunantada",
    "tunantas": "tunante",
    "tunantea": "tunantear",
    "tunanteaba": "tunantear",
    "tunanteabais": "tunantear",
    "tunanteaban": "tunantear",
    "tunanteabas": "tunantear",
    "tunantead": "tunantear",
    "tunanteada": "tunantear",
    "tunanteadas": "tunantear",
    "tunanteado": "tunantear",
    "tunanteados": "tunantear",
    "tunanteamos": "tunantear",
    "tunantean": "tunantear",
    "tunanteando": "tunantear",
    "tunanteara": "tunantear",
    "tunantearais": "tunantear",
    "tunantearan": "tunantear",
    "tunantearas": "tunantear",
    "tunanteare": "tunantear",
    "tunanteareis": "tunantear",
    "tunantearemos": "tunantear",
    "tunantearen": "tunantear",
    "tunanteares": "tunantear",
    "tunantearon": "tunantear",
    "tunanteará": "tunantear",
    "tunantearán": "tunantear",
    "tunantearás": "tunantear",
    "tunantearé": "tunantear",
    "tunantearéis": "tunantear",
    "tunantearía": "tunantear",
    "tunantearíais": "tunantear",
    "tunantearíamos": "tunantear",
    "tunantearían": "tunantear",
    "tunantearías": "tunantear",
    "tunanteas": "tunantear",
    "tunantease": "tunantear",
    "tunanteaseis": "tunantear",
    "tunanteasen": "tunantear",
    "tunanteases": "tunantear",
    "tunanteaste": "tunantear",
    "tunanteasteis": "tunantear",
    "tunantee": "tunantear",
    "tunanteemos": "tunantear",
    "tunanteen": "tunantear",
    "tunantees": "tunantear",
    "tunanteo": "tunantear",
    "tunanterías": "tunantería",
    "tunantes": "tunante",
    "tunanteábamos": "tunantear",
    "tunanteáis": "tunantear",
    "tunanteáramos": "tunantear",
    "tunanteáremos": "tunantear",
    "tunanteásemos": "tunantear",
    "tunanteé": "tunantear",
    "tunanteéis": "tunantear",
    "tunanteó": "tunantear",
    "tunara": "tunar",
    "tunarais": "tunar",
    "tunaran": "tunar",
    "tunaras": "tunar",
    "tunare": "tunar",
    "tunareis": "tunar",
    "tunaremos": "tunar",
    "tunaren": "tunar",
    "tunares": "tunar",
    "tunaron": "tunar",
    "tunarras": "tunarra",
    "tunará": "tunar",
    "tunarán": "tunar",
    "tunarás": "tunar",
    "tunaré": "tunar",
    "tunaréis": "tunar",
    "tunaría": "tunar",
    "tunaríais": "tunar",
    "tunaríamos": "tunar",
    "tunarían": "tunar",
    "tunarías": "tunar",
    "tunas": "tuno",
    "tunase": "tunar",
    "tunaseis": "tunar",
    "tunasen": "tunar",
    "tunases": "tunar",
    "tunaste": "tunar",
    "tunasteis": "tunar",
    "tunda": "tundir",
    "tundamos": "tundir",
    "tundan": "tundir",
    "tundas": "tundir",
    "tunde": "tundir",
    "tunden": "tundir",
    "tundes": "tundir",
    "tundiciones": "tundición",
    "tundid": "tundir",
    "tundida": "tundir",
    "tundidas": "tundir",
    "tundido": "tundir",
    "tundidora": "tundidor",
    "tundidoras": "tundidora",
    "tundidores": "tundidor",
    "tundidos": "tundir",
    "tundiduras": "tundidura",
    "tundiendo": "tundir",
    "tundiera": "tundir",
    "tundierais": "tundir",
    "tundieran": "tundir",
    "tundieras": "tundir",
    "tundiere": "tundir",
    "tundiereis": "tundir",
    "tundieren": "tundir",
    "tundieres": "tundir",
    "tundieron": "tundir",
    "tundiese": "tundir",
    "tundieseis": "tundir",
    "tundiesen": "tundir",
    "tundieses": "tundir",
    "tundimos": "tundir",
    "tundiremos": "tundir",
    "tundirá": "tundir",
    "tundirán": "tundir",
    "tundirás": "tundir",
    "tundiré": "tundir",
    "tundiréis": "tundir",
    "tundiría": "tundir",
    "tundiríais": "tundir",
    "tundiríamos": "tundir",
    "tundirían": "tundir",
    "tundirías": "tundir",
    "tundiste": "tundir",
    "tundisteis": "tundir",
    "tundiznos": "tundizno",
    "tundiéramos": "tundir",
    "tundiéremos": "tundir",
    "tundiésemos": "tundir",
    "tundió": "tundir",
    "tundo": "tundir",
    "tundras": "tundra",
    "tundáis": "tundir",
    "tundí": "tundir",
    "tundía": "tundir",
    "tundíais": "tundir",
    "tundíamos": "tundir",
    "tundían": "tundir",
    "tundías": "tundir",
    "tundís": "tundir",
    "tune": "tunar",
    "tunea": "tunear",
    "tuneaba": "tunear",
    "tuneabais": "tunear",
    "tuneaban": "tunear",
    "tuneabas": "tunear",
    "tunead": "tunear",
    "tuneada": "tunear",
    "tuneadas": "tunear",
    "tuneado": "tunear",
    "tuneados": "tunear",
    "tuneamos": "tunear",
    "tunean": "tunear",
    "tuneando": "tunear",
    "tuneara": "tunear",
    "tunearais": "tunear",
    "tunearan": "tunear",
    "tunearas": "tunear",
    "tuneare": "tunear",
    "tuneareis": "tunear",
    "tunearemos": "tunear",
    "tunearen": "tunear",
    "tuneares": "tunear",
    "tunearon": "tunear",
    "tuneará": "tunear",
    "tunearán": "tunear",
    "tunearás": "tunear",
    "tunearé": "tunear",
    "tunearéis": "tunear",
    "tunearía": "tunear",
    "tunearíais": "tunear",
    "tunearíamos": "tunear",
    "tunearían": "tunear",
    "tunearías": "tunear",
    "tuneas": "tunear",
    "tunease": "tunear",
    "tuneaseis": "tunear",
    "tuneasen": "tunear",
    "tuneases": "tunear",
    "tuneaste": "tunear",
    "tuneasteis": "tunear",
    "tunecina": "tunecino",
    "tunecinas": "tunecino",
    "tunecinos": "tunecino",
    "tunecíes": "tunecí",
    "tunecís": "tunecí",
    "tunee": "tunear",
    "tuneemos": "tunear",
    "tuneen": "tunear",
    "tunees": "tunear",
    "tunemos": "tunar",
    "tunen": "tunar",
    "tuneo": "tunear",
    "tuneras": "tunera",
    "tunerías": "tunería",
    "tunes": "tunar",
    "tuneábamos": "tunear",
    "tuneáis": "tunear",
    "tuneáramos": "tunear",
    "tuneáremos": "tunear",
    "tuneásemos": "tunear",
    "tuneé": "tunear",
    "tuneéis": "tunear",
    "tuneó": "tunear",
    "tungros": "tungro",
    "tungstatos": "tungstato",
    "tungstenos": "tungsteno",
    "tunguses": "tungus",
    "tunicados": "tunicado",
    "tunicelas": "tunicela",
    "tuno": "tunar",
    "tunos": "tuno",
    "tunábamos": "tunar",
    "tunáis": "tunar",
    "tunáramos": "tunar",
    "tunáremos": "tunar",
    "tunásemos": "tunar",
    "tuné": "tunar",
    "tunéis": "tunar",
    "tunó": "tunar",
    "tupa": "tupir",
    "tupamara": "tupamaro",
    "tupamaras": "tupamaro",
    "tupamaros": "tupamaro",
    "tupamos": "tupir",
    "tupan": "tupir",
    "tupas": "tupir",
    "tupe": "tupir",
    "tupen": "tupir",
    "tupes": "tupir",
    "tupiciones": "tupición",
    "tupid": "tupir",
    "tupida": "tupir",
    "tupidas": "tupir",
    "tupideces": "tupidez",
    "tupido": "tupir",
    "tupidos": "tupir",
    "tupiendo": "tupir",
    "tupiera": "tupir",
    "tupierais": "tupir",
    "tupieran": "tupir",
    "tupieras": "tupir",
    "tupiere": "tupir",
    "tupiereis": "tupir",
    "tupieren": "tupir",
    "tupieres": "tupir",
    "tupieron": "tupir",
    "tupiese": "tupir",
    "tupieseis": "tupir",
    "tupiesen": "tupir",
    "tupieses": "tupir",
    "tupimos": "tupir",
    "tupiremos": "tupir",
    "tupirá": "tupir",
    "tupirán": "tupir",
    "tupirás": "tupir",
    "tupiré": "tupir",
    "tupiréis": "tupir",
    "tupiría": "tupir",
    "tupiríais": "tupir",
    "tupiríamos": "tupir",
    "tupirían": "tupir",
    "tupirías": "tupir",
    "tupiste": "tupir",
    "tupisteis": "tupir",
    "tupitinas": "tupitina",
    "tupiéramos": "tupir",
    "tupiéremos": "tupir",
    "tupiésemos": "tupir",
    "tupió": "tupir",
    "tupliperos": "tuplipero",
    "tupo": "tupir",
    "tupáis": "tupir",
    "tupés": "tupé",
    "tupí": "tupir",
    "tupí-guaraníes": "tupí-guaraní",
    "tupí-guaranís": "tupí-guaraní",
    "tupía": "tupir",
    "tupíais": "tupir",
    "tupíamos": "tupir",
    "tupían": "tupir",
    "tupías": "tupir",
    "tupíguaraníes": "tupíguaraní",
    "tupíguaranís": "tupíguaraní",
    "tupís": "tupir",
    "turania": "turanio",
    "turanias": "turanio",
    "turanios": "turanio",
    "turba": "turbar",
    "turbaba": "turbar",
    "turbabais": "turbar",
    "turbaban": "turbar",
    "turbabas": "turbar",
    "turbaciones": "turbación",
    "turbad": "turbar",
    "turbada": "turbar",
    "turbadas": "turbar",
    "turbado": "turbar",
    "turbadora": "turbador",
    "turbadoras": "turbador",
    "turbadores": "turbador",
    "turbados": "turbar",
    "turbales": "turbal",
    "turbamientos": "turbamiento",
    "turbamos": "turbar",
    "turbamultas": "turbamulta",
    "turban": "turbar",
    "turbando": "turbar",
    "turbantes": "turbante",
    "turbara": "turbar",
    "turbarais": "turbar",
    "turbaran": "turbar",
    "turbaras": "turbar",
    "turbare": "turbar",
    "turbareis": "turbar",
    "turbaremos": "turbar",
    "turbaren": "turbar",
    "turbares": "turbar",
    "turbaron": "turbar",
    "turbará": "turbar",
    "turbarán": "turbar",
    "turbarás": "turbar",
    "turbaré": "turbar",
    "turbaréis": "turbar",
    "turbaría": "turbar",
    "turbaríais": "turbar",
    "turbaríamos": "turbar",
    "turbarían": "turbar",
    "turbarías": "turbar",
    "turbas": "turbar",
    "turbase": "turbar",
    "turbaseis": "turbar",
    "turbasen": "turbar",
    "turbases": "turbar",
    "turbaste": "turbar",
    "turbasteis": "turbar",
    "turbativa": "turbativo",
    "turbativas": "turbativo",
    "turbativos": "turbativo",
    "turbe": "turbar",
    "turbelarios": "turbelario",
    "turbemos": "turbar",
    "turben": "turbar",
    "turberas": "turbera",
    "turbes": "turbar",
    "turbia": "turbio",
    "turbias": "turbio",
    "turbideces": "turbidez",
    "turbidimetrías": "turbidimetría",
    "turbidímetros": "turbidímetro",
    "turbiedades": "turbiedad",
    "turbiezas": "turbieza",
    "turbina": "turbinar",
    "turbinaba": "turbinar",
    "turbinabais": "turbinar",
    "turbinaban": "turbinar",
    "turbinabas": "turbinar",
    "turbinad": "turbinar",
    "turbinada": "turbinar",
    "turbinadas": "turbinar",
    "turbinado": "turbinar",
    "turbinados": "turbinar",
    "turbinales": "turbinal",
    "turbinamos": "turbinar",
    "turbinan": "turbinar",
    "turbinando": "turbinar",
    "turbinara": "turbinar",
    "turbinarais": "turbinar",
    "turbinaran": "turbinar",
    "turbinaras": "turbinar",
    "turbinare": "turbinar",
    "turbinareis": "turbinar",
    "turbinaremos": "turbinar",
    "turbinaren": "turbinar",
    "turbinares": "turbinar",
    "turbinaron": "turbinar",
    "turbinará": "turbinar",
    "turbinarán": "turbinar",
    "turbinarás": "turbinar",
    "turbinaré": "turbinar",
    "turbinaréis": "turbinar",
    "turbinaría": "turbinar",
    "turbinaríais": "turbinar",
    "turbinaríamos": "turbinar",
    "turbinarían": "turbinar",
    "turbinarías": "turbinar",
    "turbinas": "turbinar",
    "turbinase": "turbinar",
    "turbinaseis": "turbinar",
    "turbinasen": "turbinar",
    "turbinases": "turbinar",
    "turbinaste": "turbinar",
    "turbinasteis": "turbinar",
    "turbine": "turbinar",
    "turbinemos": "turbinar",
    "turbinen": "turbinar",
    "turbines": "turbinar",
    "turbino": "turbinar",
    "turbinábamos": "turbinar",
    "turbináis": "turbinar",
    "turbináramos": "turbinar",
    "turbináremos": "turbinar",
    "turbinásemos": "turbinar",
    "turbiné": "turbinar",
    "turbinéis": "turbinar",
    "turbinó": "turbinar",
    "turbiones": "turbión",
    "turbios": "turbio",
    "turbo": "turbar",
    "turboalternadores": "turboalternador",
    "turbobombas": "turbobomba",
    "turbocompresores": "turbocompresor",
    "turbodinamos": "turbodinamo",
    "turboextractores": "turboextractor",
    "turbogeneradores": "turbogenerador",
    "turbohélices": "turbohélice",
    "turbomotores": "turbomotor",
    "turbomáquinas": "turbomáquina",
    "turbonadas": "turbonada",
    "turbonaves": "turbonave",
    "turboo": "turbo",
    "turbopropulsores": "turbopropulsor",
    "turborreactores": "turborreactor",
    "turbosa": "turboso",
    "turbosas": "turboso",
    "turbosos": "turboso",
    "turbotrenes": "turbotrén",
    "turboventiladores": "turboventilador",
    "turbulencias": "turbulencia",
    "turbulenta": "turbulento",
    "turbulentas": "turbulento",
    "turbulentos": "turbulento",
    "turbábamos": "turbar",
    "turbácea": "turbáceo",
    "turbáceas": "turbáceo",
    "turbáceos": "turbáceo",
    "turbáis": "turbar",
    "turbáramos": "turbar",
    "turbáremos": "turbar",
    "turbásemos": "turbar",
    "turbé": "turbar",
    "turbéis": "turbar",
    "turbó": "turbar",
    "turca": "turco",
    "turcas": "turco",
    "turcoestadounidenses": "turcoestadounidense",
    "turcomana": "turcomano",
    "turcomanas": "turcomano",
    "turcomanos": "turcomano",
    "turcoples": "turcople",
    "turcos": "turco",
    "turcotártara": "turcotártaro",
    "turcotártaras": "turcotártaro",
    "turcotártaros": "turcotártaro",
    "turdetana": "turdetano",
    "turdetanas": "turdetano",
    "turdetanos": "turdetano",
    "turgencias": "turgencia",
    "turgentes": "turgente",
    "turgescencias": "turgescencia",
    "turifica": "turificar",
    "turificaba": "turificar",
    "turificabais": "turificar",
    "turificaban": "turificar",
    "turificabas": "turificar",
    "turificad": "turificar",
    "turificada": "turificar",
    "turificadas": "turificar",
    "turificado": "turificar",
    "turificados": "turificar",
    "turificamos": "turificar",
    "turifican": "turificar",
    "turificando": "turificar",
    "turificara": "turificar",
    "turificarais": "turificar",
    "turificaran": "turificar",
    "turificaras": "turificar",
    "turificare": "turificar",
    "turificareis": "turificar",
    "turificaremos": "turificar",
    "turificaren": "turificar",
    "turificares": "turificar",
    "turificaron": "turificar",
    "turificará": "turificar",
    "turificarán": "turificar",
    "turificarás": "turificar",
    "turificaré": "turificar",
    "turificaréis": "turificar",
    "turificaría": "turificar",
    "turificaríais": "turificar",
    "turificaríamos": "turificar",
    "turificarían": "turificar",
    "turificarías": "turificar",
    "turificas": "turificar",
    "turificase": "turificar",
    "turificaseis": "turificar",
    "turificasen": "turificar",
    "turificases": "turificar",
    "turificaste": "turificar",
    "turificasteis": "turificar",
    "turifico": "turificar",
    "turificábamos": "turificar",
    "turificáis": "turificar",
    "turificáramos": "turificar",
    "turificáremos": "turificar",
    "turificásemos": "turificar",
    "turificó": "turificar",
    "turifique": "turificar",
    "turifiquemos": "turificar",
    "turifiquen": "turificar",
    "turifiques": "turificar",
    "turifiqué": "turificar",
    "turifiquéis": "turificar",
    "turinenses": "turinense",
    "turinesa": "turinés",
    "turinesas": "turinés",
    "turineses": "turinés",
    "turinésa": "turinés",
    "turinésas": "turinés",
    "turinéses": "turinés",
    "turiones": "turión",
    "turismos": "turismo",
    "turistas": "turista",
    "turkmenistana": "turkmenistano",
    "turkmenistanas": "turkmenistano",
    "turkmenistanos": "turkmenistano",
    "turmalinas": "turmalina",
    "turmas": "turma",
    "turna": "turnar",
    "turnaba": "turnar",
    "turnabais": "turnar",
    "turnaban": "turnar",
    "turnabas": "turnar",
    "turnad": "turnar",
    "turnada": "turnar",
    "turnadas": "turnar",
    "turnado": "turnar",
    "turnados": "turnar",
    "turnamos": "turnar",
    "turnan": "turnar",
    "turnando": "turnar",
    "turnara": "turnar",
    "turnarais": "turnar",
    "turnaran": "turnar",
    "turnaras": "turnar",
    "turnare": "turnar",
    "turnareis": "turnar",
    "turnaremos": "turnar",
    "turnaren": "turnar",
    "turnares": "turnar",
    "turnaron": "turnar",
    "turnará": "turnar",
    "turnarán": "turnar",
    "turnarás": "turnar",
    "turnaré": "turnar",
    "turnaréis": "turnar",
    "turnaría": "turnar",
    "turnaríais": "turnar",
    "turnaríamos": "turnar",
    "turnarían": "turnar",
    "turnarías": "turnar",
    "turnas": "turnar",
    "turnase": "turnar",
    "turnaseis": "turnar",
    "turnasen": "turnar",
    "turnases": "turnar",
    "turnaste": "turnar",
    "turnasteis": "turnar",
    "turne": "turnar",
    "turnedós": "turnedó",
    "turnemos": "turnar",
    "turnen": "turnar",
    "turnes": "turnar",
    "turnia": "turnio",
    "turnias": "turnio",
    "turnios": "turnio",
    "turno": "turnar",
    "turnos": "turno",
    "turnábamos": "turnar",
    "turnáis": "turnar",
    "turnáramos": "turnar",
    "turnáremos": "turnar",
    "turnásemos": "turnar",
    "turné": "turnar",
    "turnéis": "turnar",
    "turnó": "turnar",
    "turolenses": "turolense",
    "turonenses": "turonense",
    "turones": "turón",
    "turquesas": "turquesa",
    "turquesca": "turquesco",
    "turquescas": "turquesco",
    "turquescos": "turquesco",
    "turra": "turrar",
    "turraba": "turrar",
    "turrabais": "turrar",
    "turraban": "turrar",
    "turrabas": "turrar",
    "turrad": "turrar",
    "turrada": "turrar",
    "turradas": "turrar",
    "turrado": "turrar",
    "turrados": "turrar",
    "turramos": "turrar",
    "turran": "turrar",
    "turrando": "turrar",
    "turrara": "turrar",
    "turrarais": "turrar",
    "turraran": "turrar",
    "turraras": "turrar",
    "turrare": "turrar",
    "turrareis": "turrar",
    "turraremos": "turrar",
    "turraren": "turrar",
    "turrares": "turrar",
    "turraron": "turrar",
    "turrará": "turrar",
    "turrarán": "turrar",
    "turrarás": "turrar",
    "turraré": "turrar",
    "turraréis": "turrar",
    "turraría": "turrar",
    "turraríais": "turrar",
    "turraríamos": "turrar",
    "turrarían": "turrar",
    "turrarías": "turrar",
    "turras": "turrar",
    "turrase": "turrar",
    "turraseis": "turrar",
    "turrasen": "turrar",
    "turrases": "turrar",
    "turraste": "turrar",
    "turrasteis": "turrar",
    "turre": "turrar",
    "turremos": "turrar",
    "turren": "turrar",
    "turres": "turrar",
    "turro": "turrar",
    "turrones": "turrón",
    "turrábamos": "turrar",
    "turráis": "turrar",
    "turráramos": "turrar",
    "turráremos": "turrar",
    "turrásemos": "turrar",
    "turré": "turrar",
    "turréis": "turrar",
    "turró": "turrar",
    "turulata": "turulato",
    "turulatas": "turulato",
    "turulatos": "turulato",
    "turullos": "turullo",
    "turumbones": "turumbón",
    "turíbulos": "turíbulo",
    "turística": "turístico",
    "turísticas": "turístico",
    "turísticos": "turístico",
    "tus": "tu",
    "tusa": "tusar",
    "tusaba": "tusar",
    "tusabais": "tusar",
    "tusaban": "tusar",
    "tusabas": "tusar",
    "tusad": "tusar",
    "tusada": "tusar",
    "tusadas": "tusar",
    "tusado": "tusar",
    "tusados": "tusar",
    "tusamos": "tusar",
    "tusan": "tusar",
    "tusando": "tusar",
    "tusara": "tusar",
    "tusarais": "tusar",
    "tusaran": "tusar",
    "tusaras": "tusar",
    "tusare": "tusar",
    "tusareis": "tusar",
    "tusaremos": "tusar",
    "tusaren": "tusar",
    "tusares": "tusar",
    "tusaron": "tusar",
    "tusará": "tusar",
    "tusarán": "tusar",
    "tusarás": "tusar",
    "tusaré": "tusar",
    "tusaréis": "tusar",
    "tusaría": "tusar",
    "tusaríais": "tusar",
    "tusaríamos": "tusar",
    "tusarían": "tusar",
    "tusarías": "tusar",
    "tusas": "tusar",
    "tusase": "tusar",
    "tusaseis": "tusar",
    "tusasen": "tusar",
    "tusases": "tusar",
    "tusaste": "tusar",
    "tusasteis": "tusar",
    "tusca": "tusco",
    "tuscas": "tusco",
    "tuscos": "tusco",
    "tuse": "tusar",
    "tusemos": "tusar",
    "tusen": "tusar",
    "tuses": "tusar",
    "tusilagos": "tusilago",
    "tuso": "tusar",
    "tusonas": "tusona",
    "tusones": "tusón",
    "tusábamos": "tusar",
    "tusáis": "tusar",
    "tusáramos": "tusar",
    "tusáremos": "tusar",
    "tusásemos": "tusar",
    "tusé": "tusar",
    "tuséis": "tusar",
    "tusó": "tusar",
    "tutea": "tutear",
    "tuteaba": "tutear",
    "tuteabais": "tutear",
    "tuteaban": "tutear",
    "tuteabas": "tutear",
    "tutead": "tutear",
    "tuteada": "tutear",
    "tuteadas": "tutear",
    "tuteado": "tutear",
    "tuteadora": "tuteador",
    "tuteadoras": "tuteador",
    "tuteadores": "tuteador",
    "tuteados": "tutear",
    "tuteamientos": "tuteamiento",
    "tuteamos": "tutear",
    "tutean": "tutear",
    "tuteando": "tutear",
    "tuteara": "tutear",
    "tutearais": "tutear",
    "tutearan": "tutear",
    "tutearas": "tutear",
    "tuteare": "tutear",
    "tuteareis": "tutear",
    "tutearemos": "tutear",
    "tutearen": "tutear",
    "tuteares": "tutear",
    "tutearon": "tutear",
    "tuteará": "tutear",
    "tutearán": "tutear",
    "tutearás": "tutear",
    "tutearé": "tutear",
    "tutearéis": "tutear",
    "tutearía": "tutear",
    "tutearíais": "tutear",
    "tutearíamos": "tutear",
    "tutearían": "tutear",
    "tutearías": "tutear",
    "tuteas": "tutear",
    "tutease": "tutear",
    "tuteaseis": "tutear",
    "tuteasen": "tutear",
    "tuteases": "tutear",
    "tuteaste": "tutear",
    "tuteasteis": "tutear",
    "tutee": "tutear",
    "tuteemos": "tutear",
    "tuteen": "tutear",
    "tutees": "tutear",
    "tutela": "tutelar",
    "tutelaba": "tutelar",
    "tutelabais": "tutelar",
    "tutelaban": "tutelar",
    "tutelabas": "tutelar",
    "tutelad": "tutelar",
    "tutelada": "tutelar",
    "tuteladas": "tutelar",
    "tutelado": "tutelar",
    "tuteladora": "tutelador",
    "tuteladoras": "tutelador",
    "tuteladores": "tutelador",
    "tutelados": "tutelar",
    "tutelajes": "tutelaje",
    "tutelamos": "tutelar",
    "tutelan": "tutelar",
    "tutelando": "tutelar",
    "tutelara": "tutelar",
    "tutelarais": "tutelar",
    "tutelaran": "tutelar",
    "tutelaras": "tutelar",
    "tutelare": "tutelar",
    "tutelareis": "tutelar",
    "tutelaremos": "tutelar",
    "tutelaren": "tutelar",
    "tutelares": "tutelar",
    "tutelaron": "tutelar",
    "tutelará": "tutelar",
    "tutelarán": "tutelar",
    "tutelarás": "tutelar",
    "tutelaré": "tutelar",
    "tutelaréis": "tutelar",
    "tutelaría": "tutelar",
    "tutelaríais": "tutelar",
    "tutelaríamos": "tutelar",
    "tutelarían": "tutelar",
    "tutelarías": "tutelar",
    "tutelas": "tutelar",
    "tutelase": "tutelar",
    "tutelaseis": "tutelar",
    "tutelasen": "tutelar",
    "tutelases": "tutelar",
    "tutelaste": "tutelar",
    "tutelasteis": "tutelar",
    "tutele": "tutelar",
    "tutelemos": "tutelar",
    "tutelen": "tutelar",
    "tuteles": "tutelar",
    "tutelo": "tutelar",
    "tutelábamos": "tutelar",
    "tuteláis": "tutelar",
    "tuteláramos": "tutelar",
    "tuteláremos": "tutelar",
    "tutelásemos": "tutelar",
    "tutelé": "tutelar",
    "tuteléis": "tutelar",
    "tuteló": "tutelar",
    "tuteo": "tutear",
    "tuteos": "tuteo",
    "tutes": "tute",
    "tuteábamos": "tutear",
    "tuteáis": "tutear",
    "tuteáramos": "tutear",
    "tuteáremos": "tutear",
    "tuteásemos": "tutear",
    "tuteé": "tutear",
    "tuteéis": "tutear",
    "tuteó": "tutear",
    "tutora": "tutor",
    "tutoras": "tutor",
    "tutores": "tutor",
    "tutoriales": "tutorial",
    "tutorías": "tutoría",
    "tutsis": "tutsi",
    "tutús": "tutú",
    "tuve": "tener",
    "tuviera": "tener",
    "tuvierais": "tener",
    "tuvieran": "tener",
    "tuvieras": "tener",
    "tuviere": "tener",
    "tuviereis": "tener",
    "tuvieren": "tener",
    "tuvieres": "tener",
    "tuvieron": "tener",
    "tuviese": "tener",
    "tuvieseis": "tener",
    "tuviesen": "tener",
    "tuvieses": "tener",
    "tuvimos": "tener",
    "tuviste": "tener",
    "tuvisteis": "tener",
    "tuviéramos": "tener",
    "tuviéremos": "tener",
    "tuviésemos": "tener",
    "tuvo": "tener",
    "tuya": "tuyo",
    "tuyas": "tuyo",
    "tuyonas": "tuyona",
    "tuyos": "tuyo",
    "tuátaras": "tuátara",
    "tuáutemes": "tuáutem",
    "tuétanos": "tuétano",
    "tweedes": "tweed",
    "tweeds": "tweed",
    "twists": "twist",
    "txisteras": "txistera",
    "tyuyamunitas": "tyuyamunita",
    "tábanos": "tábano",
    "tábida": "tábido",
    "tábidas": "tábido",
    "tábidos": "tábido",
    "tácita": "tácito",
    "tácitas": "tácito",
    "tácitos": "tácito",
    "táctica": "táctico",
    "tácticas": "táctico",
    "tácticos": "táctico",
    "táctiles": "táctil",
    "tálamos": "tálamo",
    "táleas": "tálea",
    "támaras": "támara",
    "tándems": "tándem",
    "tánganas": "tángana",
    "tánica": "tánico",
    "tánicas": "tánico",
    "tánicos": "tánico",
    "tántalos": "tántalo",
    "táparas": "tápara",
    "tárgumes": "targum",
    "társidos": "társido",
    "tártagos": "tártago",
    "tártara": "tártaro",
    "tártaras": "tártaro",
    "tártaros": "tártaro",
    "tártrica": "tártrico",
    "tártricas": "tártrico",
    "tártricos": "tártrico",
    "táxones": "taxon",
    "técnica": "técnico",
    "técnicas": "técnico",
    "técnicos": "técnico",
    "tégmenes": "tegmen",
    "téguyes": "téguy",
    "téguys": "téguy",
    "télexes": "télex",
    "télsones": "telson",
    "témpanos": "témpano",
    "témperas": "témpera",
    "témporas": "témpora",
    "ténderes": "ténder",
    "ténders": "ténder",
    "tépalos": "tépalo",
    "térmica": "térmico",
    "térmicas": "térmico",
    "térmicos": "térmico",
    "términos": "término",
    "térrea": "térreo",
    "térreas": "térreo",
    "térreos": "térreo",
    "térrica": "térrico",
    "térricas": "térrico",
    "térricos": "térrico",
    "tés": "té",
    "téseras": "tésera",
    "tétradas": "tétrada",
    "tétrica": "tétrico",
    "tétricas": "tétrico",
    "tétricos": "tétrico",
    "tía": "tío",
    "tías": "tío",
    "tífica": "tífico",
    "tíficas": "tífico",
    "tíficos": "tífico",
    "tíglica": "tíglico",
    "tíglicas": "tíglico",
    "tíglicos": "tíglico",
    "tílburis": "tílburi",
    "tímalos": "tímalo",
    "tímbrica": "tímbrico",
    "tímbricas": "tímbrico",
    "tímbricos": "tímbrico",
    "tímica": "tímico",
    "tímicas": "tímico",
    "tímicos": "tímico",
    "tímida": "tímido",
    "tímidas": "tímido",
    "tímidos": "tímido",
    "tímpanos": "tímpano",
    "tíos": "tío",
    "típica": "típico",
    "típicas": "típico",
    "típicos": "típico",
    "típulas": "típula",
    "tíquets": "tíquet",
    "tísica": "tísico",
    "tísicas": "tísico",
    "tísicos": "tísico",
    "títeres": "títere",
    "títulos": "título",
    "tólares": "tólar",
    "tómbolas": "tómbola",
    "tómbolos": "tómbolo",
    "tóners": "tóner",
    "tónica": "tónico",
    "tónicas": "tónico",
    "tónicos": "tónico",
    "tóp-spins": "top-spin",
    "tópica": "tópico",
    "tópicas": "tópico",
    "tópicos": "tópico",
    "tórculos": "tórculo",
    "tórdigas": "tórdiga",
    "tórica": "tórico",
    "tóricas": "tórico",
    "tóricos": "tórico",
    "tórpida": "tórpido",
    "tórpidas": "tórpido",
    "tórpidos": "tórpido",
    "tórrida": "tórrido",
    "tórridas": "tórrido",
    "tórridos": "tórrido",
    "tórtolas": "tórtola",
    "tórtolos": "tórtolo",
    "tósigos": "tósigo",
    "tótemes": "tótem",
    "tótems": "tótem",
    "tótemses": "tótem",
    "tóxica": "tóxico",
    "tóxicas": "tóxico",
    "tóxicos": "tóxico",
    "túbulos": "túbulo",
    "túmida": "túmido",
    "túmidas": "túmido",
    "túmidos": "túmido",
    "túmulos": "túmulo",
    "túneles": "túnel",
    "túngida": "túngido",
    "túngidas": "túngido",
    "túngidos": "túngido",
    "túngstica": "túngstico",
    "túngsticas": "túngstico",
    "túngsticos": "túngstico",
    "túnguses": "tungus",
    "túnicas": "túnica",
    "túrbida": "túrbido",
    "túrbidas": "túrbido",
    "túrbidos": "túrbido",
    "túrdigas": "túrdiga",
    "túrdula": "túrdulo",
    "túrdulas": "túrdulo",
    "túrdulos": "túrdulo",
    "túrgida": "túrgido",
    "túrgidas": "túrgido",
    "túrgidos": "túrgido",
    "uabaínas": "uabaína",
    "ualabíes": "ualabí",
    "ualabís": "ualabí",
    "uapitíes": "uapití",
    "uapitís": "uapití",
    "uapitíses": "uapití",
    "ubajayes": "ubajay",
    "ubica": "ubicar",
    "ubicaba": "ubicar",
    "ubicabais": "ubicar",
    "ubicaban": "ubicar",
    "ubicabas": "ubicar",
    "ubicaciones": "ubicación",
    "ubicad": "ubicar",
    "ubicada": "ubicar",
    "ubicadas": "ubicar",
    "ubicado": "ubicar",
    "ubicados": "ubicar",
    "ubicamos": "ubicar",
    "ubican": "ubicar",
    "ubicando": "ubicar",
    "ubicara": "ubicar",
    "ubicarais": "ubicar",
    "ubicaran": "ubicar",
    "ubicaras": "ubicar",
    "ubicare": "ubicar",
    "ubicareis": "ubicar",
    "ubicaremos": "ubicar",
    "ubicaren": "ubicar",
    "ubicares": "ubicar",
    "ubicaron": "ubicar",
    "ubicará": "ubicar",
    "ubicarán": "ubicar",
    "ubicarás": "ubicar",
    "ubicaré": "ubicar",
    "ubicaréis": "ubicar",
    "ubicaría": "ubicar",
    "ubicaríais": "ubicar",
    "ubicaríamos": "ubicar",
    "ubicarían": "ubicar",
    "ubicarías": "ubicar",
    "ubicas": "ubicar",
    "ubicase": "ubicar",
    "ubicaseis": "ubicar",
    "ubicasen": "ubicar",
    "ubicases": "ubicar",
    "ubicaste": "ubicar",
    "ubicasteis": "ubicar",
    "ubico": "ubicar",
    "ubicua": "ubicuo",
    "ubicuas": "ubicuo",
    "ubicuidades": "ubicuidad",
    "ubicuos": "ubicuo",
    "ubicábamos": "ubicar",
    "ubicáis": "ubicar",
    "ubicáramos": "ubicar",
    "ubicáremos": "ubicar",
    "ubicásemos": "ubicar",
    "ubicó": "ubicar",
    "ubios": "ubio",
    "ubique": "ubicar",
    "ubiquemos": "ubicar",
    "ubiquen": "ubicar",
    "ubiques": "ubicar",
    "ubiquidades": "ubiquidad",
    "ubiquismos": "ubiquismo",
    "ubiquitaria": "ubiquitario",
    "ubiquitarias": "ubiquitario",
    "ubiquitarios": "ubiquitario",
    "ubiqué": "ubicar",
    "ubiquéis": "ubicar",
    "ubreras": "ubrera",
    "ubres": "ubre",
    "ucases": "ucase",
    "ucrania": "ucranio",
    "ucraniana": "ucraniano",
    "ucranianas": "ucraniano",
    "ucranianos": "ucraniano",
    "ucranias": "ucranio",
    "ucranios": "ucranio",
    "ufana": "ufano",
    "ufanaba": "ufanar",
    "ufanabais": "ufanar",
    "ufanaban": "ufanar",
    "ufanabas": "ufanar",
    "ufanad": "ufanar",
    "ufanada": "ufanar",
    "ufanadas": "ufanar",
    "ufanado": "ufanar",
    "ufanados": "ufanar",
    "ufanamos": "ufanar",
    "ufanan": "ufanar",
    "ufanando": "ufanar",
    "ufanara": "ufanar",
    "ufanarais": "ufanar",
    "ufanaran": "ufanar",
    "ufanaras": "ufanar",
    "ufanare": "ufanar",
    "ufanareis": "ufanar",
    "ufanaremos": "ufanar",
    "ufanaren": "ufanar",
    "ufanares": "ufanar",
    "ufanaron": "ufanar",
    "ufanará": "ufanar",
    "ufanarán": "ufanar",
    "ufanarás": "ufanar",
    "ufanaré": "ufanar",
    "ufanaréis": "ufanar",
    "ufanaría": "ufanar",
    "ufanaríais": "ufanar",
    "ufanaríamos": "ufanar",
    "ufanarían": "ufanar",
    "ufanarías": "ufanar",
    "ufanas": "ufano",
    "ufanase": "ufanar",
    "ufanaseis": "ufanar",
    "ufanasen": "ufanar",
    "ufanases": "ufanar",
    "ufanaste": "ufanar",
    "ufanasteis": "ufanar",
    "ufane": "ufanar",
    "ufanemos": "ufanar",
    "ufanen": "ufanar",
    "ufanes": "ufanar",
    "ufano": "ufanar",
    "ufanos": "ufano",
    "ufanábamos": "ufanar",
    "ufanáis": "ufanar",
    "ufanáramos": "ufanar",
    "ufanáremos": "ufanar",
    "ufanásemos": "ufanar",
    "ufané": "ufanar",
    "ufanéis": "ufanar",
    "ufanó": "ufanar",
    "ufologías": "ufología",
    "ugandesa": "ugandés",
    "ugandesas": "ugandés",
    "ugandeses": "ugandés",
    "ugrofinesa": "ugrofinés",
    "ugrofinesas": "ugrofinés",
    "ugrofineses": "ugrofinés",
    "ujieres": "ujier",
    "ukeleles": "ukelele",
    "ulcera": "ulcerar",
    "ulceraba": "ulcerar",
    "ulcerabais": "ulcerar",
    "ulceraban": "ulcerar",
    "ulcerabas": "ulcerar",
    "ulceraciones": "ulceración",
    "ulcerad": "ulcerar",
    "ulcerada": "ulcerar",
    "ulceradas": "ulcerar",
    "ulcerado": "ulcerar",
    "ulcerados": "ulcerar",
    "ulceramos": "ulcerar",
    "ulceran": "ulcerar",
    "ulcerando": "ulcerar",
    "ulcerara": "ulcerar",
    "ulcerarais": "ulcerar",
    "ulceraran": "ulcerar",
    "ulceraras": "ulcerar",
    "ulcerare": "ulcerar",
    "ulcerareis": "ulcerar",
    "ulceraremos": "ulcerar",
    "ulceraren": "ulcerar",
    "ulcerares": "ulcerar",
    "ulceraron": "ulcerar",
    "ulcerará": "ulcerar",
    "ulcerarán": "ulcerar",
    "ulcerarás": "ulcerar",
    "ulceraré": "ulcerar",
    "ulceraréis": "ulcerar",
    "ulceraría": "ulcerar",
    "ulceraríais": "ulcerar",
    "ulceraríamos": "ulcerar",
    "ulcerarían": "ulcerar",
    "ulcerarías": "ulcerar",
    "ulceras": "ulcerar",
    "ulcerase": "ulcerar",
    "ulceraseis": "ulcerar",
    "ulcerasen": "ulcerar",
    "ulcerases": "ulcerar",
    "ulceraste": "ulcerar",
    "ulcerasteis": "ulcerar",
    "ulcerativa": "ulcerativo",
    "ulcerativas": "ulcerativo",
    "ulcerativos": "ulcerativo",
    "ulcere": "ulcerar",
    "ulceremos": "ulcerar",
    "ulceren": "ulcerar",
    "ulceres": "ulcerar",
    "ulcero": "ulcerar",
    "ulceronecrótica": "ulceronecrótico",
    "ulceronecróticas": "ulceronecrótico",
    "ulceronecróticos": "ulceronecrótico",
    "ulcerosa": "ulceroso",
    "ulcerosas": "ulceroso",
    "ulcerosos": "ulceroso",
    "ulcerábamos": "ulcerar",
    "ulceráis": "ulcerar",
    "ulceráramos": "ulcerar",
    "ulceráremos": "ulcerar",
    "ulcerásemos": "ulcerar",
    "ulceré": "ulcerar",
    "ulceréis": "ulcerar",
    "ulceró": "ulcerar",
    "ulcerógena": "ulcerógeno",
    "ulcerógenas": "ulcerógeno",
    "ulcerógenos": "ulcerógeno",
    "ulemas": "ulema",
    "ulexitas": "ulexita",
    "uliginosa": "uliginoso",
    "uliginosas": "uliginoso",
    "uliginosos": "uliginoso",
    "ulmácea": "ulmáceo",
    "ulmáceas": "ulmáceo",
    "ulmáceos": "ulmáceo",
    "ulteriores": "ulterior",
    "ultima": "ultimar",
    "ultimaba": "ultimar",
    "ultimabais": "ultimar",
    "ultimaban": "ultimar",
    "ultimabas": "ultimar",
    "ultimaciones": "ultimación",
    "ultimad": "ultimar",
    "ultimada": "ultimar",
    "ultimadas": "ultimar",
    "ultimado": "ultimar",
    "ultimadora": "ultimador",
    "ultimadoras": "ultimador",
    "ultimadores": "ultimador",
    "ultimados": "ultimar",
    "ultimamos": "ultimar",
    "ultiman": "ultimar",
    "ultimando": "ultimar",
    "ultimara": "ultimar",
    "ultimarais": "ultimar",
    "ultimaran": "ultimar",
    "ultimaras": "ultimar",
    "ultimare": "ultimar",
    "ultimareis": "ultimar",
    "ultimaremos": "ultimar",
    "ultimaren": "ultimar",
    "ultimares": "ultimar",
    "ultimaron": "ultimar",
    "ultimará": "ultimar",
    "ultimarán": "ultimar",
    "ultimarás": "ultimar",
    "ultimaré": "ultimar",
    "ultimaréis": "ultimar",
    "ultimaría": "ultimar",
    "ultimaríais": "ultimar",
    "ultimaríamos": "ultimar",
    "ultimarían": "ultimar",
    "ultimarías": "ultimar",
    "ultimas": "ultimar",
    "ultimase": "ultimar",
    "ultimaseis": "ultimar",
    "ultimasen": "ultimar",
    "ultimases": "ultimar",
    "ultimaste": "ultimar",
    "ultimasteis": "ultimar",
    "ultimatos": "ultimato",
    "ultimatums": "ultimatum",
    "ultime": "ultimar",
    "ultimemos": "ultimar",
    "ultimen": "ultimar",
    "ultimes": "ultimar",
    "ultimo": "ultimar",
    "ultimábamos": "ultimar",
    "ultimáis": "ultimar",
    "ultimáramos": "ultimar",
    "ultimáremos": "ultimar",
    "ultimásemos": "ultimar",
    "ultimátums": "ultimátum",
    "ultimé": "ultimar",
    "ultiméis": "ultimar",
    "ultimó": "ultimar",
    "ultraacústica": "ultraacústico",
    "ultraacústicas": "ultraacústico",
    "ultraacústicos": "ultraacústico",
    "ultraalta": "ultraalto",
    "ultraaltas": "ultraalto",
    "ultraaltos": "ultraalto",
    "ultrabásica": "ultrabásico",
    "ultrabásicas": "ultrabásico",
    "ultrabásicos": "ultrabásico",
    "ultracentrifugaciones": "ultracentrifugación",
    "ultracentrifugadora": "ultracentrifugador",
    "ultracentrifugadoras": "ultracentrifugadora",
    "ultracentrifugadores": "ultracentrifugador",
    "ultracentrífugas": "ultracentrífuga",
    "ultracorta": "ultracorto",
    "ultracortas": "ultracorto",
    "ultracortos": "ultracorto",
    "ultraderechas": "ultraderecha",
    "ultradirectiva": "ultradirectivo",
    "ultradirectivas": "ultradirectivo",
    "ultradirectivos": "ultradirectivo",
    "ultrafiltraciones": "ultrafiltración",
    "ultrafina": "ultrafino",
    "ultrafinas": "ultrafino",
    "ultrafinos": "ultrafino",
    "ultraja": "ultrajar",
    "ultrajaba": "ultrajar",
    "ultrajabais": "ultrajar",
    "ultrajaban": "ultrajar",
    "ultrajabas": "ultrajar",
    "ultrajad": "ultrajar",
    "ultrajada": "ultrajar",
    "ultrajadas": "ultrajar",
    "ultrajado": "ultrajar",
    "ultrajadora": "ultrajador",
    "ultrajadoras": "ultrajador",
    "ultrajadores": "ultrajador",
    "ultrajados": "ultrajar",
    "ultrajamos": "ultrajar",
    "ultrajan": "ultrajar",
    "ultrajando": "ultrajar",
    "ultrajantes": "ultrajante",
    "ultrajara": "ultrajar",
    "ultrajarais": "ultrajar",
    "ultrajaran": "ultrajar",
    "ultrajaras": "ultrajar",
    "ultrajare": "ultrajar",
    "ultrajareis": "ultrajar",
    "ultrajaremos": "ultrajar",
    "ultrajaren": "ultrajar",
    "ultrajares": "ultrajar",
    "ultrajaron": "ultrajar",
    "ultrajará": "ultrajar",
    "ultrajarán": "ultrajar",
    "ultrajarás": "ultrajar",
    "ultrajaré": "ultrajar",
    "ultrajaréis": "ultrajar",
    "ultrajaría": "ultrajar",
    "ultrajaríais": "ultrajar",
    "ultrajaríamos": "ultrajar",
    "ultrajarían": "ultrajar",
    "ultrajarías": "ultrajar",
    "ultrajas": "ultrajar",
    "ultrajase": "ultrajar",
    "ultrajaseis": "ultrajar",
    "ultrajasen": "ultrajar",
    "ultrajases": "ultrajar",
    "ultrajaste": "ultrajar",
    "ultrajasteis": "ultrajar",
    "ultraje": "ultrajar",
    "ultrajemos": "ultrajar",
    "ultrajen": "ultrajar",
    "ultrajes": "ultraje",
    "ultrajo": "ultrajar",
    "ultrajosa": "ultrajoso",
    "ultrajosas": "ultrajoso",
    "ultrajosos": "ultrajoso",
    "ultrajábamos": "ultrajar",
    "ultrajáis": "ultrajar",
    "ultrajáramos": "ultrajar",
    "ultrajáremos": "ultrajar",
    "ultrajásemos": "ultrajar",
    "ultrajé": "ultrajar",
    "ultrajéis": "ultrajar",
    "ultrajó": "ultrajar",
    "ultraligera": "ultraligero",
    "ultraligeras": "ultraligero",
    "ultraligeros": "ultraligero",
    "ultralineales": "ultralineal",
    "ultramares": "ultramar",
    "ultramarina": "ultramarino",
    "ultramarinas": "ultramarino",
    "ultramarinos": "ultramarino",
    "ultramicroscopios": "ultramicroscopio",
    "ultramicroscópica": "ultramicroscópico",
    "ultramicroscópicas": "ultramicroscópico",
    "ultramicroscópicos": "ultramicroscópico",
    "ultramontana": "ultramontano",
    "ultramontanas": "ultramontano",
    "ultramontanismos": "ultramontanismo",
    "ultramontanos": "ultramontano",
    "ultramundana": "ultramundano",
    "ultramundanas": "ultramundano",
    "ultramundanos": "ultramundano",
    "ultramétrica": "ultramétrico",
    "ultramétricas": "ultramétrico",
    "ultramétricos": "ultramétrico",
    "ultranacionalistas": "ultranacionalista",
    "ultrapasa": "ultrapasar",
    "ultrapasaba": "ultrapasar",
    "ultrapasabais": "ultrapasar",
    "ultrapasaban": "ultrapasar",
    "ultrapasabas": "ultrapasar",
    "ultrapasad": "ultrapasar",
    "ultrapasada": "ultrapasar",
    "ultrapasadas": "ultrapasar",
    "ultrapasado": "ultrapasar",
    "ultrapasados": "ultrapasar",
    "ultrapasamos": "ultrapasar",
    "ultrapasan": "ultrapasar",
    "ultrapasando": "ultrapasar",
    "ultrapasara": "ultrapasar",
    "ultrapasarais": "ultrapasar",
    "ultrapasaran": "ultrapasar",
    "ultrapasaras": "ultrapasar",
    "ultrapasare": "ultrapasar",
    "ultrapasareis": "ultrapasar",
    "ultrapasaremos": "ultrapasar",
    "ultrapasaren": "ultrapasar",
    "ultrapasares": "ultrapasar",
    "ultrapasaron": "ultrapasar",
    "ultrapasará": "ultrapasar",
    "ultrapasarán": "ultrapasar",
    "ultrapasarás": "ultrapasar",
    "ultrapasaré": "ultrapasar",
    "ultrapasaréis": "ultrapasar",
    "ultrapasaría": "ultrapasar",
    "ultrapasaríais": "ultrapasar",
    "ultrapasaríamos": "ultrapasar",
    "ultrapasarían": "ultrapasar",
    "ultrapasarías": "ultrapasar",
    "ultrapasas": "ultrapasar",
    "ultrapasase": "ultrapasar",
    "ultrapasaseis": "ultrapasar",
    "ultrapasasen": "ultrapasar",
    "ultrapasases": "ultrapasar",
    "ultrapasaste": "ultrapasar",
    "ultrapasasteis": "ultrapasar",
    "ultrapase": "ultrapasar",
    "ultrapasemos": "ultrapasar",
    "ultrapasen": "ultrapasar",
    "ultrapases": "ultrapasar",
    "ultrapaso": "ultrapasar",
    "ultrapasábamos": "ultrapasar",
    "ultrapasáis": "ultrapasar",
    "ultrapasáramos": "ultrapasar",
    "ultrapasáremos": "ultrapasar",
    "ultrapasásemos": "ultrapasar",
    "ultrapasé": "ultrapasar",
    "ultrapaséis": "ultrapasar",
    "ultrapasó": "ultrapasar",
    "ultrapura": "ultrapuro",
    "ultrapuras": "ultrapuro",
    "ultrapuros": "ultrapuro",
    "ultrarradiaciones": "ultrarradiación",
    "ultrarroja": "ultrarrojo",
    "ultrarrojas": "ultrarrojo",
    "ultrarrojos": "ultrarrojo",
    "ultrarrápida": "ultrarrápido",
    "ultrarrápidas": "ultrarrápido",
    "ultrarrápidos": "ultrarrápido",
    "ultrarápida": "ultrarápido",
    "ultrarápidas": "ultrarápido",
    "ultrarápidos": "ultrarápido",
    "ultras": "ultra",
    "ultrasonidos": "ultrasonido",
    "ultrasónica": "ultrasónico",
    "ultrasónicas": "ultrasónico",
    "ultrasónicos": "ultrasónico",
    "ultraterrestres": "ultraterrestre",
    "ultravacíos": "ultravacío",
    "ultravioletas": "ultravioleta",
    "ultraísmos": "ultraísmo",
    "ultraístas": "ultraísta",
    "ulula": "ulular",
    "ululaba": "ulular",
    "ululabais": "ulular",
    "ululaban": "ulular",
    "ululabas": "ulular",
    "ululad": "ulular",
    "ululada": "ulular",
    "ululadas": "ulular",
    "ululado": "ulular",
    "ululados": "ulular",
    "ululamos": "ulular",
    "ululan": "ulular",
    "ululando": "ulular",
    "ululara": "ulular",
    "ulularais": "ulular",
    "ulularan": "ulular",
    "ulularas": "ulular",
    "ululare": "ulular",
    "ululareis": "ulular",
    "ulularemos": "ulular",
    "ulularen": "ulular",
    "ululares": "ulular",
    "ulularon": "ulular",
    "ululará": "ulular",
    "ulularán": "ulular",
    "ulularás": "ulular",
    "ulularé": "ulular",
    "ulularéis": "ulular",
    "ulularía": "ulular",
    "ulularíais": "ulular",
    "ulularíamos": "ulular",
    "ulularían": "ulular",
    "ulularías": "ulular",
    "ululas": "ulular",
    "ululase": "ulular",
    "ululaseis": "ulular",
    "ululasen": "ulular",
    "ululases": "ulular",
    "ululaste": "ulular",
    "ululasteis": "ulular",
    "ululatos": "ululato",
    "ulule": "ulular",
    "ululemos": "ulular",
    "ululen": "ulular",
    "ulules": "ulular",
    "ululo": "ulular",
    "ululábamos": "ulular",
    "ululáis": "ulular",
    "ululáramos": "ulular",
    "ululáremos": "ulular",
    "ululásemos": "ulular",
    "ululé": "ulular",
    "ululéis": "ulular",
    "ululó": "ulular",
    "umas": "uma",
    "umbelas": "umbela",
    "umbeliferales": "umbeliferal",
    "umbeliferonas": "umbeliferona",
    "umbelífera": "umbelífero",
    "umbelíferas": "umbelífero",
    "umbelíferos": "umbelífero",
    "umbilicada": "umbilicado",
    "umbilicadas": "umbilicado",
    "umbilicados": "umbilicado",
    "umbilicales": "umbilical",
    "umbrales": "umbral",
    "umbrelas": "umbrela",
    "umbrosa": "umbroso",
    "umbrosas": "umbroso",
    "umbrosos": "umbroso",
    "umbráculos": "umbráculo",
    "umbrática": "umbrático",
    "umbráticas": "umbrático",
    "umbráticos": "umbrático",
    "umbrátiles": "umbrátil",
    "umbría": "umbrío",
    "umbrías": "umbrío",
    "umbríos": "umbrío",
    "un": "uno",
    "una": "uno",
    "unakitas": "unakita",
    "unalba": "unalbo",
    "unalbas": "unalbo",
    "unalbos": "unalbo",
    "unamos": "unir",
    "unan": "unir",
    "unanimidades": "unanimidad",
    "unanimismos": "unanimismo",
    "unaria": "unario",
    "unarias": "unario",
    "unarios": "unario",
    "unas": "uno",
    "unce": "uncir",
    "uncen": "uncir",
    "unces": "uncir",
    "unciales": "uncial",
    "uncias": "uncia",
    "uncid": "uncir",
    "uncida": "uncir",
    "uncidas": "uncir",
    "uncido": "uncir",
    "uncidora": "uncidor",
    "uncidoras": "uncidor",
    "uncidores": "uncidor",
    "uncidos": "uncir",
    "unciendo": "uncir",
    "unciera": "uncir",
    "uncierais": "uncir",
    "uncieran": "uncir",
    "uncieras": "uncir",
    "unciere": "uncir",
    "unciereis": "uncir",
    "uncieren": "uncir",
    "uncieres": "uncir",
    "uncieron": "uncir",
    "unciese": "uncir",
    "uncieseis": "uncir",
    "unciesen": "uncir",
    "uncieses": "uncir",
    "unciformes": "unciforme",
    "uncimos": "uncir",
    "uncinarias": "uncinaria",
    "unciones": "unción",
    "unciremos": "uncir",
    "uncirá": "uncir",
    "uncirán": "uncir",
    "uncirás": "uncir",
    "unciré": "uncir",
    "unciréis": "uncir",
    "unciría": "uncir",
    "unciríais": "uncir",
    "unciríamos": "uncir",
    "uncirían": "uncir",
    "uncirías": "uncir",
    "unciste": "uncir",
    "uncisteis": "uncir",
    "unciéramos": "uncir",
    "unciéremos": "uncir",
    "unciésemos": "uncir",
    "unció": "uncir",
    "uncí": "uncir",
    "uncía": "uncir",
    "uncíais": "uncir",
    "uncíamos": "uncir",
    "uncían": "uncir",
    "uncías": "uncir",
    "uncís": "uncir",
    "undecanoatos": "undecanoato",
    "undecanoica": "undecanoico",
    "undecanoicas": "undecanoico",
    "undecanoicos": "undecanoico",
    "undecanoles": "undecanol",
    "undecilenatos": "undecilenato",
    "undecilénica": "undecilénico",
    "undecilénicas": "undecilénico",
    "undecilénicos": "undecilénico",
    "undergrounds": "underground",
    "undosa": "undoso",
    "undosas": "undoso",
    "undosos": "undoso",
    "undula": "undular",
    "undulaba": "undular",
    "undulabais": "undular",
    "undulaban": "undular",
    "undulabas": "undular",
    "undulaciones": "undulación",
    "undulad": "undular",
    "undulada": "undular",
    "unduladas": "undular",
    "undulado": "undular",
    "undulados": "undular",
    "undulamos": "undular",
    "undulan": "undular",
    "undulando": "undular",
    "undulantes": "undulante",
    "undulara": "undular",
    "undularais": "undular",
    "undularan": "undular",
    "undularas": "undular",
    "undulare": "undular",
    "undulareis": "undular",
    "undularemos": "undular",
    "undularen": "undular",
    "undulares": "undular",
    "undularon": "undular",
    "undulará": "undular",
    "undularán": "undular",
    "undularás": "undular",
    "undularé": "undular",
    "undularéis": "undular",
    "undularía": "undular",
    "undularíais": "undular",
    "undularíamos": "undular",
    "undularían": "undular",
    "undularías": "undular",
    "undulas": "undular",
    "undulase": "undular",
    "undulaseis": "undular",
    "undulasen": "undular",
    "undulases": "undular",
    "undulaste": "undular",
    "undulasteis": "undular",
    "undulatoria": "undulatorio",
    "undulatorias": "undulatorio",
    "undulatorios": "undulatorio",
    "undule": "undular",
    "undulemos": "undular",
    "undulen": "undular",
    "undules": "undular",
    "undulo": "undular",
    "undulábamos": "undular",
    "unduláis": "undular",
    "unduláramos": "undular",
    "unduláremos": "undular",
    "undulásemos": "undular",
    "undulé": "undular",
    "unduléis": "undular",
    "unduló": "undular",
    "undécima": "11",
    "undécimas": "11",
    "undécimo": "11",
    "undécimos": "11",
    "undécupla": "undécuplo",
    "undécuplas": "undécuplo",
    "undécuplos": "undécuplo",
    "undísona": "undísono",
    "undísonas": "undísono",
    "undísonos": "undísono",
    "undívaga": "undívago",
    "undívagas": "undívago",
    "undívagos": "undívago",
    "une": "unir",
    "unen": "unir",
    "unes": "unir",
    "unge": "ungir",
    "ungen": "ungir",
    "unges": "ungir",
    "ungid": "ungir",
    "ungida": "ungir",
    "ungidas": "ungir",
    "ungido": "ungir",
    "ungidos": "ungir",
    "ungiendo": "ungir",
    "ungiera": "ungir",
    "ungierais": "ungir",
    "ungieran": "ungir",
    "ungieras": "ungir",
    "ungiere": "ungir",
    "ungiereis": "ungir",
    "ungieren": "ungir",
    "ungieres": "ungir",
    "ungieron": "ungir",
    "ungiese": "ungir",
    "ungieseis": "ungir",
    "ungiesen": "ungir",
    "ungieses": "ungir",
    "ungimientos": "ungimiento",
    "ungimos": "ungir",
    "ungiremos": "ungir",
    "ungirá": "ungir",
    "ungirán": "ungir",
    "ungirás": "ungir",
    "ungiré": "ungir",
    "ungiréis": "ungir",
    "ungiría": "ungir",
    "ungiríais": "ungir",
    "ungiríamos": "ungir",
    "ungirían": "ungir",
    "ungirías": "ungir",
    "ungiste": "ungir",
    "ungisteis": "ungir",
    "ungiéramos": "ungir",
    "ungiéremos": "ungir",
    "ungiésemos": "ungir",
    "ungió": "ungir",
    "ungueales": "ungueal",
    "unguiculada": "unguiculado",
    "unguiculadas": "unguiculado",
    "unguiculados": "unguiculado",
    "ungulada": "ungulado",
    "unguladas": "ungulado",
    "ungulados": "ungulado",
    "ungulares": "ungular",
    "ungulígrada": "ungulígrado",
    "ungulígradas": "ungulígrado",
    "ungulígrados": "ungulígrado",
    "ungí": "ungir",
    "ungía": "ungir",
    "ungíais": "ungir",
    "ungíamos": "ungir",
    "ungían": "ungir",
    "ungías": "ungir",
    "ungís": "ungir",
    "ungüentaria": "ungüentario",
    "ungüentarias": "ungüentario",
    "ungüentarios": "ungüentario",
    "ungüentos": "ungüento",
    "uniaxiales": "uniaxial",
    "unicajas": "unicaja",
    "unicelulares": "unicelular",
    "unicidades": "unicidad",
    "unicismos": "unicismo",
    "unicolores": "unicolor",
    "unicornios": "unicornio",
    "unicursales": "unicursal",
    "unid": "unir",
    "unida": "unir",
    "unidades": "unidad",
    "unidas": "unir",
    "unidimensionales": "unidimensional",
    "unidireccionales": "unidireccional",
    "unido": "unir",
    "unidos": "unir",
    "uniendo": "unir",
    "uniera": "unir",
    "unierais": "unir",
    "unieran": "unir",
    "unieras": "unir",
    "uniere": "unir",
    "uniereis": "unir",
    "unieren": "unir",
    "unieres": "unir",
    "unieron": "unir",
    "uniese": "unir",
    "unieseis": "unir",
    "uniesen": "unir",
    "unieses": "unir",
    "uniestatales": "uniestatal",
    "unifaciales": "unifacial",
    "unifamiliares": "unifamiliar",
    "unifica": "unificar",
    "unificaba": "unificar",
    "unificabais": "unificar",
    "unificaban": "unificar",
    "unificabas": "unificar",
    "unificaciones": "unificación",
    "unificad": "unificar",
    "unificada": "unificar",
    "unificadas": "unificar",
    "unificado": "unificar",
    "unificadora": "unificador",
    "unificadoras": "unificador",
    "unificadores": "unificador",
    "unificados": "unificar",
    "unificamos": "unificar",
    "unifican": "unificar",
    "unificando": "unificar",
    "unificara": "unificar",
    "unificarais": "unificar",
    "unificaran": "unificar",
    "unificaras": "unificar",
    "unificare": "unificar",
    "unificareis": "unificar",
    "unificaremos": "unificar",
    "unificaren": "unificar",
    "unificares": "unificar",
    "unificaron": "unificar",
    "unificará": "unificar",
    "unificarán": "unificar",
    "unificarás": "unificar",
    "unificaré": "unificar",
    "unificaréis": "unificar",
    "unificaría": "unificar",
    "unificaríais": "unificar",
    "unificaríamos": "unificar",
    "unificarían": "unificar",
    "unificarías": "unificar",
    "unificas": "unificar",
    "unificase": "unificar",
    "unificaseis": "unificar",
    "unificasen": "unificar",
    "unificases": "unificar",
    "unificaste": "unificar",
    "unificasteis": "unificar",
    "unifico": "unificar",
    "unificábamos": "unificar",
    "unificáis": "unificar",
    "unificáramos": "unificar",
    "unificáremos": "unificar",
    "unificásemos": "unificar",
    "unificó": "unificar",
    "unifilares": "unifilar",
    "unifique": "unificar",
    "unifiquemos": "unificar",
    "unifiquen": "unificar",
    "unifiques": "unificar",
    "unifiqué": "unificar",
    "unifiquéis": "unificar",
    "unifoliada": "unifoliado",
    "unifoliadas": "unifoliado",
    "unifoliados": "unifoliado",
    "uniforma": "uniformar",
    "uniformaba": "uniformar",
    "uniformabais": "uniformar",
    "uniformaban": "uniformar",
    "uniformabas": "uniformar",
    "uniformad": "uniformar",
    "uniformada": "uniformar",
    "uniformadas": "uniformar",
    "uniformado": "uniformar",
    "uniformadora": "uniformador",
    "uniformadoras": "uniformador",
    "uniformadores": "uniformador",
    "uniformados": "uniformar",
    "uniformamos": "uniformar",
    "uniforman": "uniformar",
    "uniformando": "uniformar",
    "uniformara": "uniformar",
    "uniformarais": "uniformar",
    "uniformaran": "uniformar",
    "uniformaras": "uniformar",
    "uniformare": "uniformar",
    "uniformareis": "uniformar",
    "uniformaremos": "uniformar",
    "uniformaren": "uniformar",
    "uniformares": "uniformar",
    "uniformaron": "uniformar",
    "uniformará": "uniformar",
    "uniformarán": "uniformar",
    "uniformarás": "uniformar",
    "uniformaré": "uniformar",
    "uniformaréis": "uniformar",
    "uniformaría": "uniformar",
    "uniformaríais": "uniformar",
    "uniformaríamos": "uniformar",
    "uniformarían": "uniformar",
    "uniformarías": "uniformar",
    "uniformas": "uniformar",
    "uniformase": "uniformar",
    "uniformaseis": "uniformar",
    "uniformasen": "uniformar",
    "uniformases": "uniformar",
    "uniformaste": "uniformar",
    "uniformasteis": "uniformar",
    "uniforme": "uniformar",
    "uniformemos": "uniformar",
    "uniformen": "uniformar",
    "uniformes": "uniforme",
    "uniformidades": "uniformidad",
    "uniformitarismos": "uniformitarismo",
    "uniformizables": "uniformizable",
    "uniformizaciones": "uniformización",
    "uniformizadora": "uniformizador",
    "uniformizadoras": "uniformizador",
    "uniformizadores": "uniformizador",
    "uniformo": "uniformar",
    "uniformábamos": "uniformar",
    "uniformáis": "uniformar",
    "uniformáramos": "uniformar",
    "uniformáremos": "uniformar",
    "uniformásemos": "uniformar",
    "uniformé": "uniformar",
    "uniforméis": "uniformar",
    "uniformó": "uniformar",
    "unigénita": "unigénito",
    "unigénitas": "unigénito",
    "unigénitos": "unigénito",
    "unilaterales": "unilateral",
    "unilateralismos": "unilateralismo",
    "unilineales": "unilineal",
    "unilinealidades": "unilinealidad",
    "uniloculares": "unilocular",
    "unimodales": "unimodal",
    "unimodulares": "unimodular",
    "unimoleculares": "unimolecular",
    "unimos": "unir",
    "unioculares": "uniocular",
    "uniones": "unión",
    "unionismos": "unionismo",
    "unionistas": "unionista",
    "unipenniformes": "unipenniforme",
    "unipersonales": "unipersonal",
    "unipersonalidades": "unipersonalidad",
    "uniplanares": "uniplanar",
    "unipolares": "unipolar",
    "unipotentes": "unipotente",
    "uniprocesadores": "uniprocesador",
    "uniremos": "unir",
    "unirradiculares": "unirradicular",
    "unirá": "unir",
    "unirán": "unir",
    "unirás": "unir",
    "uniré": "unir",
    "uniréis": "unir",
    "uniría": "unir",
    "uniríais": "unir",
    "uniríamos": "unir",
    "unirían": "unir",
    "unirías": "unir",
    "uniselectores": "uniselector",
    "unisexuales": "unisexual",
    "unisonaba": "unisonar",
    "unisonabais": "unisonar",
    "unisonaban": "unisonar",
    "unisonabas": "unisonar",
    "unisonad": "unisonar",
    "unisonada": "unisonar",
    "unisonadas": "unisonar",
    "unisonado": "unisonar",
    "unisonados": "unisonar",
    "unisonamos": "unisonar",
    "unisonancias": "unisonancia",
    "unisonando": "unisonar",
    "unisonara": "unisonar",
    "unisonarais": "unisonar",
    "unisonaran": "unisonar",
    "unisonaras": "unisonar",
    "unisonare": "unisonar",
    "unisonareis": "unisonar",
    "unisonaremos": "unisonar",
    "unisonaren": "unisonar",
    "unisonares": "unisonar",
    "unisonaron": "unisonar",
    "unisonará": "unisonar",
    "unisonarán": "unisonar",
    "unisonarás": "unisonar",
    "unisonaré": "unisonar",
    "unisonaréis": "unisonar",
    "unisonaría": "unisonar",
    "unisonaríais": "unisonar",
    "unisonaríamos": "unisonar",
    "unisonarían": "unisonar",
    "unisonarías": "unisonar",
    "unisonase": "unisonar",
    "unisonaseis": "unisonar",
    "unisonasen": "unisonar",
    "unisonases": "unisonar",
    "unisonaste": "unisonar",
    "unisonasteis": "unisonar",
    "unisonemos": "unisonar",
    "unisones": "unisón",
    "unisonábamos": "unisonar",
    "unisonáis": "unisonar",
    "unisonáramos": "unisonar",
    "unisonáremos": "unisonar",
    "unisonásemos": "unisonar",
    "unisoné": "unisonar",
    "unisonéis": "unisonar",
    "unisonó": "unisonar",
    "uniste": "unir",
    "unisteis": "unir",
    "unisuena": "unisonar",
    "unisuenan": "unisonar",
    "unisuenas": "unisonar",
    "unisuene": "unisonar",
    "unisuenen": "unisonar",
    "unisuenes": "unisonar",
    "unisueno": "unisonar",
    "unitaria": "unitario",
    "unitarias": "unitario",
    "unitarios": "unitario",
    "unitarismos": "unitarismo",
    "unitaristas": "unitarista",
    "unitiva": "unitivo",
    "unitivas": "unitivo",
    "unitivos": "unitivo",
    "unitizada": "unitizado",
    "unitizadas": "unitizado",
    "unitizados": "unitizado",
    "univalentes": "univalente",
    "univalva": "univalvo",
    "univalvas": "univalvo",
    "univalvos": "univalvo",
    "univariables": "univariable",
    "univariantes": "univariante",
    "universales": "universal",
    "universalice": "universalizar",
    "universalicemos": "universalizar",
    "universalicen": "universalizar",
    "universalices": "universalizar",
    "universalicé": "universalizar",
    "universalicéis": "universalizar",
    "universalidades": "universalidad",
    "universalismos": "universalismo",
    "universalistas": "universalista",
    "universaliza": "universalizar",
    "universalizaba": "universalizar",
    "universalizabais": "universalizar",
    "universalizaban": "universalizar",
    "universalizabas": "universalizar",
    "universalizaciones": "universalización",
    "universalizad": "universalizar",
    "universalizada": "universalizar",
    "universalizadas": "universalizar",
    "universalizado": "universalizar",
    "universalizados": "universalizar",
    "universalizamos": "universalizar",
    "universalizan": "universalizar",
    "universalizando": "universalizar",
    "universalizara": "universalizar",
    "universalizarais": "universalizar",
    "universalizaran": "universalizar",
    "universalizaras": "universalizar",
    "universalizare": "universalizar",
    "universalizareis": "universalizar",
    "universalizaremos": "universalizar",
    "universalizaren": "universalizar",
    "universalizares": "universalizar",
    "universalizaron": "universalizar",
    "universalizará": "universalizar",
    "universalizarán": "universalizar",
    "universalizarás": "universalizar",
    "universalizaré": "universalizar",
    "universalizaréis": "universalizar",
    "universalizaría": "universalizar",
    "universalizaríais": "universalizar",
    "universalizaríamos": "universalizar",
    "universalizarían": "universalizar",
    "universalizarías": "universalizar",
    "universalizas": "universalizar",
    "universalizase": "universalizar",
    "universalizaseis": "universalizar",
    "universalizasen": "universalizar",
    "universalizases": "universalizar",
    "universalizaste": "universalizar",
    "universalizasteis": "universalizar",
    "universalizo": "universalizar",
    "universalizábamos": "universalizar",
    "universalizáis": "universalizar",
    "universalizáramos": "universalizar",
    "universalizáremos": "universalizar",
    "universalizásemos": "universalizar",
    "universalizó": "universalizar",
    "universidades": "universidad",
    "universitaria": "universitario",
    "universitarias": "universitario",
    "universitarios": "universitario",
    "universos": "universo",
    "univibradores": "univibrador",
    "univocaciones": "univocación",
    "univocidades": "univocidad",
    "uniáxica": "uniáxico",
    "uniáxicas": "uniáxico",
    "uniáxicos": "uniáxico",
    "uniéramos": "unir",
    "uniéremos": "unir",
    "uniésemos": "unir",
    "unió": "unir",
    "uniónes": "unión",
    "unja": "ungir",
    "unjamos": "ungir",
    "unjan": "ungir",
    "unjas": "ungir",
    "unjo": "ungir",
    "unjáis": "ungir",
    "uno": "unir",
    "unos": "uno",
    "unques": "unque",
    "unta": "untar",
    "untaba": "untar",
    "untabais": "untar",
    "untaban": "untar",
    "untabas": "untar",
    "untad": "untar",
    "untada": "untar",
    "untadas": "untar",
    "untado": "untar",
    "untadora": "untador",
    "untadoras": "untador",
    "untadores": "untador",
    "untados": "untar",
    "untaduras": "untadura",
    "untamientos": "untamiento",
    "untamos": "untar",
    "untan": "untar",
    "untando": "untar",
    "untara": "untar",
    "untarais": "untar",
    "untaran": "untar",
    "untaras": "untar",
    "untare": "untar",
    "untareis": "untar",
    "untaremos": "untar",
    "untaren": "untar",
    "untares": "untar",
    "untaron": "untar",
    "untará": "untar",
    "untarán": "untar",
    "untarás": "untar",
    "untaré": "untar",
    "untaréis": "untar",
    "untaría": "untar",
    "untaríais": "untar",
    "untaríamos": "untar",
    "untarían": "untar",
    "untarías": "untar",
    "untas": "untar",
    "untase": "untar",
    "untaseis": "untar",
    "untasen": "untar",
    "untases": "untar",
    "untaste": "untar",
    "untasteis": "untar",
    "unte": "untar",
    "untemos": "untar",
    "unten": "untar",
    "untes": "untar",
    "unto": "untar",
    "untos": "unto",
    "untosa": "untoso",
    "untosas": "untoso",
    "untosos": "untoso",
    "untuosa": "untuoso",
    "untuosas": "untuoso",
    "untuosidades": "untuosidad",
    "untuosos": "untuoso",
    "unturas": "untura",
    "untábamos": "untar",
    "untáis": "untar",
    "untáramos": "untar",
    "untáremos": "untar",
    "untásemos": "untar",
    "unté": "untar",
    "untéis": "untar",
    "untó": "untar",
    "unza": "uncir",
    "unzamos": "uncir",
    "unzan": "uncir",
    "unzas": "uncir",
    "unzo": "uncir",
    "unzáis": "uncir",
    "unáis": "unir",
    "unánimes": "unánime",
    "uní": "unir",
    "unía": "unir",
    "uníais": "unir",
    "uníamos": "unir",
    "unían": "unir",
    "unías": "unir",
    "unípara": "uníparo",
    "uníparas": "uníparo",
    "uníparos": "uníparo",
    "unípedes": "unípede",
    "unís": "unir",
    "unísona": "unísono",
    "unísonas": "unísono",
    "unísonos": "unísono",
    "unívoca": "unívoco",
    "unívocas": "unívoco",
    "unívocos": "unívoco",
    "upa": "upar",
    "upaba": "upar",
    "upabais": "upar",
    "upaban": "upar",
    "upabas": "upar",
    "upad": "upar",
    "upada": "upar",
    "upadas": "upar",
    "upado": "upar",
    "upados": "upar",
    "upamos": "upar",
    "upan": "upar",
    "upando": "upar",
    "upara": "upar",
    "uparais": "upar",
    "uparan": "upar",
    "uparas": "upar",
    "upare": "upar",
    "upareis": "upar",
    "uparemos": "upar",
    "uparen": "upar",
    "upares": "upar",
    "uparon": "upar",
    "upará": "upar",
    "uparán": "upar",
    "uparás": "upar",
    "uparé": "upar",
    "uparéis": "upar",
    "uparía": "upar",
    "uparíais": "upar",
    "uparíamos": "upar",
    "uparían": "upar",
    "uparías": "upar",
    "upas": "upar",
    "upase": "upar",
    "upaseis": "upar",
    "upasen": "upar",
    "upases": "upar",
    "upaste": "upar",
    "upasteis": "upar",
    "upe": "upar",
    "upemos": "upar",
    "upen": "upar",
    "uperizaciones": "uperización",
    "upes": "upar",
    "upo": "upar",
    "upábamos": "upar",
    "upáis": "upar",
    "upáramos": "upar",
    "upáremos": "upar",
    "upásemos": "upar",
    "upé": "upar",
    "upéis": "upar",
    "upó": "upar",
    "uracilos": "uracilo",
    "uraloaltaica": "uraloaltaico",
    "uraloaltaicas": "uraloaltaico",
    "uraloaltaicos": "uraloaltaico",
    "uranatos": "uranato",
    "urania": "uranio",
    "uranias": "uranio",
    "uraninas": "uranina",
    "uraninitas": "uraninita",
    "uranios": "uranio",
    "uranocircitas": "uranocircita",
    "uranográfica": "uranográfico",
    "uranográficas": "uranográfico",
    "uranográficos": "uranográfico",
    "uranoplastias": "uranoplastia",
    "uranófanas": "uranófana",
    "urbana": "urbano",
    "urbanas": "urbano",
    "urbanice": "urbanizar",
    "urbanicemos": "urbanizar",
    "urbanicen": "urbanizar",
    "urbanices": "urbanizar",
    "urbanicé": "urbanizar",
    "urbanicéis": "urbanizar",
    "urbanidades": "urbanidad",
    "urbanismos": "urbanismo",
    "urbanistas": "urbanista",
    "urbaniza": "urbanizar",
    "urbanizaba": "urbanizar",
    "urbanizabais": "urbanizar",
    "urbanizaban": "urbanizar",
    "urbanizabas": "urbanizar",
    "urbanizables": "urbanizable",
    "urbanizaciones": "urbanización",
    "urbanizad": "urbanizar",
    "urbanizada": "urbanizar",
    "urbanizadas": "urbanizar",
    "urbanizado": "urbanizar",
    "urbanizadora": "urbanizador",
    "urbanizadoras": "urbanizador",
    "urbanizadores": "urbanizador",
    "urbanizados": "urbanizar",
    "urbanizamos": "urbanizar",
    "urbanizan": "urbanizar",
    "urbanizando": "urbanizar",
    "urbanizara": "urbanizar",
    "urbanizarais": "urbanizar",
    "urbanizaran": "urbanizar",
    "urbanizaras": "urbanizar",
    "urbanizare": "urbanizar",
    "urbanizareis": "urbanizar",
    "urbanizaremos": "urbanizar",
    "urbanizaren": "urbanizar",
    "urbanizares": "urbanizar",
    "urbanizaron": "urbanizar",
    "urbanizará": "urbanizar",
    "urbanizarán": "urbanizar",
    "urbanizarás": "urbanizar",
    "urbanizaré": "urbanizar",
    "urbanizaréis": "urbanizar",
    "urbanizaría": "urbanizar",
    "urbanizaríais": "urbanizar",
    "urbanizaríamos": "urbanizar",
    "urbanizarían": "urbanizar",
    "urbanizarías": "urbanizar",
    "urbanizas": "urbanizar",
    "urbanizase": "urbanizar",
    "urbanizaseis": "urbanizar",
    "urbanizasen": "urbanizar",
    "urbanizases": "urbanizar",
    "urbanizaste": "urbanizar",
    "urbanizasteis": "urbanizar",
    "urbanizo": "urbanizar",
    "urbanizábamos": "urbanizar",
    "urbanizáis": "urbanizar",
    "urbanizáramos": "urbanizar",
    "urbanizáremos": "urbanizar",
    "urbanizásemos": "urbanizar",
    "urbanizó": "urbanizar",
    "urbano-industriales": "urbano-industrial",
    "urbanos": "urbano",
    "urbanística": "urbanístico",
    "urbanísticas": "urbanístico",
    "urbanísticos": "urbanístico",
    "urbes": "urbe",
    "urcas": "urca",
    "urda": "urdir",
    "urdamos": "urdir",
    "urdan": "urdir",
    "urdas": "urdir",
    "urde": "urdir",
    "urden": "urdir",
    "urdes": "urdir",
    "urdid": "urdir",
    "urdida": "urdir",
    "urdidas": "urdir",
    "urdideras": "urdidera",
    "urdido": "urdir",
    "urdidora": "urdidor",
    "urdidoras": "urdidor",
    "urdidores": "urdidor",
    "urdidos": "urdir",
    "urdiduras": "urdidura",
    "urdiendo": "urdir",
    "urdiera": "urdir",
    "urdierais": "urdir",
    "urdieran": "urdir",
    "urdieras": "urdir",
    "urdiere": "urdir",
    "urdiereis": "urdir",
    "urdieren": "urdir",
    "urdieres": "urdir",
    "urdieron": "urdir",
    "urdiese": "urdir",
    "urdieseis": "urdir",
    "urdiesen": "urdir",
    "urdieses": "urdir",
    "urdimbres": "urdimbre",
    "urdimos": "urdir",
    "urdiremos": "urdir",
    "urdirá": "urdir",
    "urdirán": "urdir",
    "urdirás": "urdir",
    "urdiré": "urdir",
    "urdiréis": "urdir",
    "urdiría": "urdir",
    "urdiríais": "urdir",
    "urdiríamos": "urdir",
    "urdirían": "urdir",
    "urdirías": "urdir",
    "urdiste": "urdir",
    "urdisteis": "urdir",
    "urdiéramos": "urdir",
    "urdiéremos": "urdir",
    "urdiésemos": "urdir",
    "urdió": "urdir",
    "urdo": "urdir",
    "urdus": "urdu",
    "urdáis": "urdir",
    "urdí": "urdir",
    "urdía": "urdir",
    "urdíais": "urdir",
    "urdíamos": "urdir",
    "urdían": "urdir",
    "urdías": "urdir",
    "urdís": "urdir",
    "ureas": "urea",
    "ureasas": "ureasa",
    "uredinales": "uredinal",
    "uremias": "uremia",
    "urentes": "urente",
    "uretanos": "uretano",
    "uretrales": "uretral",
    "uretras": "uretra",
    "uretroscopios": "uretroscopio",
    "uretérica": "uretérico",
    "uretéricas": "uretérico",
    "uretéricos": "uretérico",
    "urge": "urgir",
    "urgen": "urgir",
    "urgencias": "urgencia",
    "urgentes": "urgente",
    "urges": "urgir",
    "urgid": "urgir",
    "urgida": "urgir",
    "urgidas": "urgir",
    "urgido": "urgir",
    "urgidos": "urgir",
    "urgiendo": "urgir",
    "urgiera": "urgir",
    "urgierais": "urgir",
    "urgieran": "urgir",
    "urgieras": "urgir",
    "urgiere": "urgir",
    "urgiereis": "urgir",
    "urgieren": "urgir",
    "urgieres": "urgir",
    "urgieron": "urgir",
    "urgiese": "urgir",
    "urgieseis": "urgir",
    "urgiesen": "urgir",
    "urgieses": "urgir",
    "urgimos": "urgir",
    "urgiremos": "urgir",
    "urgirá": "urgir",
    "urgirán": "urgir",
    "urgirás": "urgir",
    "urgiré": "urgir",
    "urgiréis": "urgir",
    "urgiría": "urgir",
    "urgiríais": "urgir",
    "urgiríamos": "urgir",
    "urgirían": "urgir",
    "urgirías": "urgir",
    "urgiste": "urgir",
    "urgisteis": "urgir",
    "urgiéramos": "urgir",
    "urgiéremos": "urgir",
    "urgiésemos": "urgir",
    "urgió": "urgir",
    "urgí": "urgir",
    "urgía": "urgir",
    "urgíais": "urgir",
    "urgíamos": "urgir",
    "urgían": "urgir",
    "urgías": "urgir",
    "urgís": "urgir",
    "uriales": "urial",
    "uridinas": "uridina",
    "uridílica": "uridílico",
    "uridílicas": "uridílico",
    "uridílicos": "uridílico",
    "urinales": "urinal",
    "urinaria": "urinario",
    "urinarias": "urinario",
    "urinarios": "urinario",
    "urinosa": "urinoso",
    "urinosas": "urinoso",
    "urinosos": "urinoso",
    "urinífera": "urinífero",
    "uriníferas": "urinífero",
    "uriníferos": "urinífero",
    "urja": "urgir",
    "urjamos": "urgir",
    "urjan": "urgir",
    "urjas": "urgir",
    "urjo": "urgir",
    "urjáis": "urgir",
    "urnas": "urna",
    "urobilinas": "urobilina",
    "urobilinemias": "urobilinemia",
    "urobilinógenos": "urobilinógeno",
    "urocordados": "urocordado",
    "urocromos": "urocromo",
    "urodelos": "urodelo",
    "urogallos": "urogallo",
    "urogenitales": "urogenital",
    "urografías": "urografía",
    "urologías": "urología",
    "urológica": "urológico",
    "urológicas": "urológico",
    "urológicos": "urológico",
    "uromancias": "uromancia",
    "uros": "uro",
    "uroscopias": "uroscopia",
    "urotricos": "urotrico",
    "urracas": "urraca",
    "ursones": "ursón",
    "ursulinas": "ursulina",
    "urticales": "urtical",
    "urticantes": "urticante",
    "urticarias": "urticaria",
    "urticácea": "urticáceo",
    "urticáceas": "urticáceo",
    "urticáceos": "urticáceo",
    "uruguaya": "uruguayo",
    "uruguayas": "uruguayo",
    "uruguayismos": "uruguayismo",
    "uruguayos": "uruguayo",
    "urutaúes": "urutaú",
    "urutaús": "urutaú",
    "uránica": "uránico",
    "uránicas": "uránico",
    "uránicos": "uránico",
    "urática": "urático",
    "uráticas": "urático",
    "uráticos": "urático",
    "urémica": "urémico",
    "urémicas": "urémico",
    "urémicos": "urémico",
    "uréteres": "uréter",
    "urética": "urético",
    "uréticas": "urético",
    "uréticos": "urético",
    "uróloga": "urólogo",
    "urólogas": "urólogo",
    "urólogos": "urólogo",
    "urópodos": "urópodo",
    "usa": "usar",
    "usaba": "usar",
    "usabais": "usar",
    "usaban": "usar",
    "usabas": "usar",
    "usabilidades": "usabilidad",
    "usad": "usar",
    "usada": "usar",
    "usadas": "usar",
    "usado": "usar",
    "usados": "usar",
    "usagres": "usagre",
    "usamos": "usar",
    "usan": "usar",
    "usando": "usar",
    "usanzas": "usanza",
    "usara": "usar",
    "usarais": "usar",
    "usaran": "usar",
    "usaras": "usar",
    "usare": "usar",
    "usareis": "usar",
    "usaremos": "usar",
    "usaren": "usar",
    "usares": "usar",
    "usaron": "usar",
    "usará": "usar",
    "usarán": "usar",
    "usarás": "usar",
    "usaré": "usar",
    "usaréis": "usar",
    "usaría": "usar",
    "usaríais": "usar",
    "usaríamos": "usar",
    "usarían": "usar",
    "usarías": "usar",
    "usas": "usar",
    "usase": "usar",
    "usaseis": "usar",
    "usasen": "usar",
    "usases": "usar",
    "usaste": "usar",
    "usasteis": "usar",
    "use": "usar",
    "usemos": "usar",
    "usen": "usar",
    "uses": "usar",
    "usieres": "usier",
    "uso": "usar",
    "usos": "uso",
    "ussingitas": "ussingita",
    "usted": "tú",
    "ustedes": "vosotros",
    "ustibles": "ustible",
    "ustilaginales": "ustilaginal",
    "ustiones": "ustión",
    "usuales": "usual",
    "usuaria": "usuario",
    "usuarias": "usuario",
    "usuarios": "usuario",
    "usucapa": "usucapir",
    "usucapamos": "usucapir",
    "usucapan": "usucapir",
    "usucapas": "usucapir",
    "usucape": "usucapir",
    "usucapen": "usucapir",
    "usucapes": "usucapir",
    "usucapid": "usucapir",
    "usucapida": "usucapir",
    "usucapidas": "usucapir",
    "usucapido": "usucapir",
    "usucapidos": "usucapir",
    "usucapiendo": "usucapir",
    "usucapiera": "usucapir",
    "usucapierais": "usucapir",
    "usucapieran": "usucapir",
    "usucapieras": "usucapir",
    "usucapiere": "usucapir",
    "usucapiereis": "usucapir",
    "usucapieren": "usucapir",
    "usucapieres": "usucapir",
    "usucapieron": "usucapir",
    "usucapiese": "usucapir",
    "usucapieseis": "usucapir",
    "usucapiesen": "usucapir",
    "usucapieses": "usucapir",
    "usucapimos": "usucapir",
    "usucapiones": "usucapión",
    "usucapiremos": "usucapir",
    "usucapirá": "usucapir",
    "usucapirán": "usucapir",
    "usucapirás": "usucapir",
    "usucapiré": "usucapir",
    "usucapiréis": "usucapir",
    "usucapiría": "usucapir",
    "usucapiríais": "usucapir",
    "usucapiríamos": "usucapir",
    "usucapirían": "usucapir",
    "usucapirías": "usucapir",
    "usucapiste": "usucapir",
    "usucapisteis": "usucapir",
    "usucapiéramos": "usucapir",
    "usucapiéremos": "usucapir",
    "usucapiésemos": "usucapir",
    "usucapió": "usucapir",
    "usucapo": "usucapir",
    "usucapáis": "usucapir",
    "usucapí": "usucapir",
    "usucapía": "usucapir",
    "usucapíais": "usucapir",
    "usucapíamos": "usucapir",
    "usucapían": "usucapir",
    "usucapías": "usucapir",
    "usucapís": "usucapir",
    "usufructos": "usufructo",
    "usufructuaba": "usufructuar",
    "usufructuabais": "usufructuar",
    "usufructuaban": "usufructuar",
    "usufructuabas": "usufructuar",
    "usufructuad": "usufructuar",
    "usufructuada": "usufructuar",
    "usufructuadas": "usufructuar",
    "usufructuado": "usufructuar",
    "usufructuados": "usufructuar",
    "usufructuamos": "usufructuar",
    "usufructuando": "usufructuar",
    "usufructuara": "usufructuar",
    "usufructuarais": "usufructuar",
    "usufructuaran": "usufructuar",
    "usufructuaras": "usufructuar",
    "usufructuare": "usufructuar",
    "usufructuareis": "usufructuar",
    "usufructuaremos": "usufructuar",
    "usufructuaren": "usufructuar",
    "usufructuares": "usufructuar",
    "usufructuaria": "usufructuario",
    "usufructuarias": "usufructuario",
    "usufructuarios": "usufructuario",
    "usufructuaron": "usufructuar",
    "usufructuará": "usufructuar",
    "usufructuarán": "usufructuar",
    "usufructuarás": "usufructuar",
    "usufructuaré": "usufructuar",
    "usufructuaréis": "usufructuar",
    "usufructuaría": "usufructuar",
    "usufructuaríais": "usufructuar",
    "usufructuaríamos": "usufructuar",
    "usufructuarían": "usufructuar",
    "usufructuarías": "usufructuar",
    "usufructuase": "usufructuar",
    "usufructuaseis": "usufructuar",
    "usufructuasen": "usufructuar",
    "usufructuases": "usufructuar",
    "usufructuaste": "usufructuar",
    "usufructuasteis": "usufructuar",
    "usufructuemos": "usufructuar",
    "usufructuábamos": "usufructuar",
    "usufructuáis": "usufructuar",
    "usufructuáramos": "usufructuar",
    "usufructuáremos": "usufructuar",
    "usufructuásemos": "usufructuar",
    "usufructué": "usufructuar",
    "usufructuéis": "usufructuar",
    "usufructuó": "usufructuar",
    "usufructúa": "usufructuar",
    "usufructúan": "usufructuar",
    "usufructúas": "usufructuar",
    "usufructúe": "usufructuar",
    "usufructúen": "usufructuar",
    "usufructúes": "usufructuar",
    "usufructúo": "usufructuar",
    "usuraria": "usurario",
    "usurarias": "usurario",
    "usurarios": "usurario",
    "usuras": "usura",
    "usurera": "usurero",
    "usureras": "usurero",
    "usureros": "usurero",
    "usurpa": "usurpar",
    "usurpaba": "usurpar",
    "usurpabais": "usurpar",
    "usurpaban": "usurpar",
    "usurpabas": "usurpar",
    "usurpaciones": "usurpación",
    "usurpad": "usurpar",
    "usurpada": "usurpar",
    "usurpadas": "usurpar",
    "usurpado": "usurpar",
    "usurpadora": "usurpador",
    "usurpadoras": "usurpador",
    "usurpadores": "usurpador",
    "usurpados": "usurpar",
    "usurpamos": "usurpar",
    "usurpan": "usurpar",
    "usurpando": "usurpar",
    "usurpara": "usurpar",
    "usurparais": "usurpar",
    "usurparan": "usurpar",
    "usurparas": "usurpar",
    "usurpare": "usurpar",
    "usurpareis": "usurpar",
    "usurparemos": "usurpar",
    "usurparen": "usurpar",
    "usurpares": "usurpar",
    "usurparon": "usurpar",
    "usurpará": "usurpar",
    "usurparán": "usurpar",
    "usurparás": "usurpar",
    "usurparé": "usurpar",
    "usurparéis": "usurpar",
    "usurparía": "usurpar",
    "usurparíais": "usurpar",
    "usurparíamos": "usurpar",
    "usurparían": "usurpar",
    "usurparías": "usurpar",
    "usurpas": "usurpar",
    "usurpase": "usurpar",
    "usurpaseis": "usurpar",
    "usurpasen": "usurpar",
    "usurpases": "usurpar",
    "usurpaste": "usurpar",
    "usurpasteis": "usurpar",
    "usurpe": "usurpar",
    "usurpemos": "usurpar",
    "usurpen": "usurpar",
    "usurpes": "usurpar",
    "usurpo": "usurpar",
    "usurpábamos": "usurpar",
    "usurpáis": "usurpar",
    "usurpáramos": "usurpar",
    "usurpáremos": "usurpar",
    "usurpásemos": "usurpar",
    "usurpé": "usurpar",
    "usurpéis": "usurpar",
    "usurpó": "usurpar",
    "usábamos": "usar",
    "usáis": "usar",
    "usáramos": "usar",
    "usáremos": "usar",
    "usásemos": "usar",
    "usé": "usar",
    "uséis": "usar",
    "usías": "usía",
    "usó": "usar",
    "utensilios": "utensilio",
    "uterina": "uterino",
    "uterinas": "uterino",
    "uterinos": "uterino",
    "uteroovárica": "uteroovárico",
    "uteroováricas": "uteroovárico",
    "uteroováricos": "uteroovárico",
    "uterosacra": "uterosacro",
    "uterosacras": "uterosacro",
    "uterosacros": "uterosacro",
    "uterovaginales": "uterovaginal",
    "utilancias": "utilancia",
    "utilera": "utilero",
    "utileras": "utilero",
    "utileros": "utilero",
    "utilice": "utilizar",
    "utilicemos": "utilizar",
    "utilicen": "utilizar",
    "utilices": "utilizar",
    "utilicé": "utilizar",
    "utilicéis": "utilizar",
    "utilidades": "utilidad",
    "utilitaria": "utilitario",
    "utilitarias": "utilitario",
    "utilitarios": "utilitario",
    "utilitarismos": "utilitarismo",
    "utilitaristas": "utilitarista",
    "utiliza": "utilizar",
    "utilizaba": "utilizar",
    "utilizabais": "utilizar",
    "utilizaban": "utilizar",
    "utilizabas": "utilizar",
    "utilizables": "utilizable",
    "utilizaciones": "utilización",
    "utilizad": "utilizar",
    "utilizada": "utilizar",
    "utilizadas": "utilizar",
    "utilizado": "utilizar",
    "utilizados": "utilizar",
    "utilizamos": "utilizar",
    "utilizan": "utilizar",
    "utilizando": "utilizar",
    "utilizara": "utilizar",
    "utilizarais": "utilizar",
    "utilizaran": "utilizar",
    "utilizaras": "utilizar",
    "utilizare": "utilizar",
    "utilizareis": "utilizar",
    "utilizaremos": "utilizar",
    "utilizaren": "utilizar",
    "utilizares": "utilizar",
    "utilizaron": "utilizar",
    "utilizará": "utilizar",
    "utilizarán": "utilizar",
    "utilizarás": "utilizar",
    "utilizaré": "utilizar",
    "utilizaréis": "utilizar",
    "utilizaría": "utilizar",
    "utilizaríais": "utilizar",
    "utilizaríamos": "utilizar",
    "utilizarían": "utilizar",
    "utilizarías": "utilizar",
    "utilizas": "utilizar",
    "utilizase": "utilizar",
    "utilizaseis": "utilizar",
    "utilizasen": "utilizar",
    "utilizases": "utilizar",
    "utilizaste": "utilizar",
    "utilizasteis": "utilizar",
    "utilizo": "utilizar",
    "utilizábamos": "utilizar",
    "utilizáis": "utilizar",
    "utilizáramos": "utilizar",
    "utilizáremos": "utilizar",
    "utilizásemos": "utilizar",
    "utilizó": "utilizar",
    "utillajes": "utillaje",
    "utillera": "utillero",
    "utilleras": "utillero",
    "utilleros": "utillero",
    "utopistas": "utopista",
    "utopías": "utopía",
    "utraquistas": "utraquista",
    "utrera": "utrero",
    "utreras": "utrero",
    "utreros": "utrero",
    "utriculares": "utricular",
    "utricularias": "utricularia",
    "utriculoampulares": "utriculoampular",
    "utriculosaculares": "utriculosacular",
    "utrículos": "utrículo",
    "utópica": "utópico",
    "utópicas": "utópico",
    "utópicos": "utópico",
    "uvadas": "uvada",
    "uvalas": "uvala",
    "uvales": "uval",
    "uvarovitas": "uvarovita",
    "uvas": "uva",
    "uvates": "uvate",
    "uveales": "uveal",
    "uvera": "uvero",
    "uverales": "uveral",
    "uveras": "uvero",
    "uveros": "uvero",
    "uves": "uve",
    "uvillos": "uvillo",
    "uvitas": "uvita",
    "uvulares": "uvular",
    "uvulectomías": "uvulectomía",
    "uvulotomías": "uvulotomía",
    "uxoricidas": "uxoricida",
    "uxoricidios": "uxoricidio",
    "uxorilocales": "uxorilocal",
    "uxorilocalidades": "uxorilocalidad",
    "uzbeca": "uzbeco",
    "uzbecas": "uzbeco",
    "uzbecos": "uzbeco",
    "uzbeka": "uzbeko",
    "uzbekas": "uzbeko",
    "uzbekos": "uzbeko",
    "uácaris": "uácari",
    "uñada": "uñado",
    "uñadas": "uñado",
    "uñados": "uñado",
    "uñaradas": "uñarada",
    "uñas": "uña",
    "uñates": "uñate",
    "uñeras": "uñera",
    "uñeros": "uñero",
    "uñetas": "uñeta",
    "uñosa": "uñoso",
    "uñosas": "uñoso",
    "uñosos": "uñoso",
    "va": "ir",
    "vaca": "vaco",
    "vacaba": "vacar",
    "vacabais": "vacar",
    "vacaban": "vacar",
    "vacabas": "vacar",
    "vacabueyes": "vacabuey",
    "vacacionales": "vacacional",
    "vacaciones": "vacación",
    "vacad": "vacar",
    "vacada": "vacar",
    "vacadas": "vacar",
    "vacado": "vacar",
    "vacados": "vacar",
    "vacamos": "vacar",
    "vacan": "vacar",
    "vacancias": "vacancia",
    "vacando": "vacar",
    "vacantes": "vacante",
    "vacara": "vacar",
    "vacarais": "vacar",
    "vacaran": "vacar",
    "vacaras": "vacar",
    "vacare": "vacar",
    "vacareis": "vacar",
    "vacaremos": "vacar",
    "vacaren": "vacar",
    "vacares": "vacar",
    "vacaron": "vacar",
    "vacará": "vacar",
    "vacarán": "vacar",
    "vacarás": "vacar",
    "vacaré": "vacar",
    "vacaréis": "vacar",
    "vacaría": "vacar",
    "vacaríais": "vacar",
    "vacaríamos": "vacar",
    "vacarían": "vacar",
    "vacarías": "vacar",
    "vacaríes": "vacarí",
    "vacarís": "vacarí",
    "vacas": "vaco",
    "vacase": "vacar",
    "vacaseis": "vacar",
    "vacasen": "vacar",
    "vacases": "vacar",
    "vacaste": "vacar",
    "vacasteis": "vacar",
    "vacaturas": "vacatura",
    "vacceos": "vacceo",
    "vaccinias": "vaccinia",
    "vaciaba": "vaciar",
    "vaciabais": "vaciar",
    "vaciaban": "vaciar",
    "vaciabas": "vaciar",
    "vaciad": "vaciar",
    "vaciada": "vaciar",
    "vaciadas": "vaciar",
    "vaciadera": "vaciadero",
    "vaciaderas": "vaciadero",
    "vaciaderos": "vaciadero",
    "vaciadiza": "vaciadizo",
    "vaciadizas": "vaciadizo",
    "vaciadizos": "vaciadizo",
    "vaciado": "vaciar",
    "vaciadora": "vaciador",
    "vaciadoras": "vaciador",
    "vaciadores": "vaciador",
    "vaciados": "vaciar",
    "vaciamientos": "vaciamiento",
    "vaciamos": "vaciar",
    "vaciando": "vaciar",
    "vaciantes": "vaciante",
    "vaciara": "vaciar",
    "vaciarais": "vaciar",
    "vaciaran": "vaciar",
    "vaciaras": "vaciar",
    "vaciare": "vaciar",
    "vaciareis": "vaciar",
    "vaciaremos": "vaciar",
    "vaciaren": "vaciar",
    "vaciares": "vaciar",
    "vaciaron": "vaciar",
    "vaciará": "vaciar",
    "vaciarán": "vaciar",
    "vaciarás": "vaciar",
    "vaciaré": "vaciar",
    "vaciaréis": "vaciar",
    "vaciaría": "vaciar",
    "vaciaríais": "vaciar",
    "vaciaríamos": "vaciar",
    "vaciarían": "vaciar",
    "vaciarías": "vaciar",
    "vaciase": "vaciar",
    "vaciaseis": "vaciar",
    "vaciasen": "vaciar",
    "vaciases": "vaciar",
    "vaciaste": "vaciar",
    "vaciasteis": "vaciar",
    "vaciedades": "vaciedad",
    "vaciemos": "vaciar",
    "vacieros": "vaciero",
    "vacila": "vacilar",
    "vacilaba": "vacilar",
    "vacilabais": "vacilar",
    "vacilaban": "vacilar",
    "vacilabas": "vacilar",
    "vacilaciones": "vacilación",
    "vacilad": "vacilar",
    "vacilada": "vacilar",
    "vaciladas": "vacilar",
    "vacilado": "vacilar",
    "vacilados": "vacilar",
    "vacilamos": "vacilar",
    "vacilan": "vacilar",
    "vacilando": "vacilar",
    "vacilantes": "vacilante",
    "vacilara": "vacilar",
    "vacilarais": "vacilar",
    "vacilaran": "vacilar",
    "vacilaras": "vacilar",
    "vacilare": "vacilar",
    "vacilareis": "vacilar",
    "vacilaremos": "vacilar",
    "vacilaren": "vacilar",
    "vacilares": "vacilar",
    "vacilaron": "vacilar",
    "vacilará": "vacilar",
    "vacilarán": "vacilar",
    "vacilarás": "vacilar",
    "vacilaré": "vacilar",
    "vacilaréis": "vacilar",
    "vacilaría": "vacilar",
    "vacilaríais": "vacilar",
    "vacilaríamos": "vacilar",
    "vacilarían": "vacilar",
    "vacilarías": "vacilar",
    "vacilas": "vacilar",
    "vacilase": "vacilar",
    "vacilaseis": "vacilar",
    "vacilasen": "vacilar",
    "vacilases": "vacilar",
    "vacilaste": "vacilar",
    "vacilasteis": "vacilar",
    "vacile": "vacilar",
    "vacilemos": "vacilar",
    "vacilen": "vacilar",
    "vaciles": "vacile",
    "vacilo": "vacilar",
    "vacilábamos": "vacilar",
    "vaciláis": "vacilar",
    "vaciláramos": "vacilar",
    "vaciláremos": "vacilar",
    "vacilásemos": "vacilar",
    "vacilé": "vacilar",
    "vaciléis": "vacilar",
    "vaciló": "vacilar",
    "vaciábamos": "vaciar",
    "vaciáis": "vaciar",
    "vaciáramos": "vaciar",
    "vaciáremos": "vaciar",
    "vaciásemos": "vaciar",
    "vacié": "vaciar",
    "vaciéis": "vaciar",
    "vació": "vaciar",
    "vaco": "vacar",
    "vacos": "vaco",
    "vacua": "vacuo",
    "vacuas": "vacuo",
    "vacuidades": "vacuidad",
    "vacuna": "vacuno",
    "vacunaba": "vacunar",
    "vacunabais": "vacunar",
    "vacunaban": "vacunar",
    "vacunabas": "vacunar",
    "vacunables": "vacunable",
    "vacunaciones": "vacunación",
    "vacunad": "vacunar",
    "vacunada": "vacunar",
    "vacunadas": "vacunar",
    "vacunado": "vacunar",
    "vacunadora": "vacunador",
    "vacunadoras": "vacunador",
    "vacunadores": "vacunador",
    "vacunados": "vacunar",
    "vacunamos": "vacunar",
    "vacunan": "vacunar",
    "vacunando": "vacunar",
    "vacunara": "vacunar",
    "vacunarais": "vacunar",
    "vacunaran": "vacunar",
    "vacunaras": "vacunar",
    "vacunare": "vacunar",
    "vacunareis": "vacunar",
    "vacunaremos": "vacunar",
    "vacunaren": "vacunar",
    "vacunares": "vacunar",
    "vacunaron": "vacunar",
    "vacunará": "vacunar",
    "vacunarán": "vacunar",
    "vacunarás": "vacunar",
    "vacunaré": "vacunar",
    "vacunaréis": "vacunar",
    "vacunaría": "vacunar",
    "vacunaríais": "vacunar",
    "vacunaríamos": "vacunar",
    "vacunarían": "vacunar",
    "vacunarías": "vacunar",
    "vacunas": "vacuno",
    "vacunase": "vacunar",
    "vacunaseis": "vacunar",
    "vacunasen": "vacunar",
    "vacunases": "vacunar",
    "vacunaste": "vacunar",
    "vacunasteis": "vacunar",
    "vacune": "vacunar",
    "vacunemos": "vacunar",
    "vacunen": "vacunar",
    "vacunes": "vacunar",
    "vacuno": "vacunar",
    "vacunos": "vacuno",
    "vacunoterapias": "vacunoterapia",
    "vacunábamos": "vacunar",
    "vacunáis": "vacunar",
    "vacunáramos": "vacunar",
    "vacunáremos": "vacunar",
    "vacunásemos": "vacunar",
    "vacuné": "vacunar",
    "vacunéis": "vacunar",
    "vacunó": "vacunar",
    "vacuolares": "vacuolar",
    "vacuolas": "vacuola",
    "vacuolos": "vacuolo",
    "vacuos": "vacuo",
    "vacuímetros": "vacuímetro",
    "vacuómetros": "vacuómetro",
    "vacábamos": "vacar",
    "vacáis": "vacar",
    "vacáramos": "vacar",
    "vacáremos": "vacar",
    "vacásemos": "vacar",
    "vacía": "vacío",
    "vacían": "vaciar",
    "vacías": "vacío",
    "vacíe": "vaciar",
    "vacíen": "vaciar",
    "vacíes": "vaciar",
    "vacío": "vaciar",
    "vacíos": "vacío",
    "vacó": "vacar",
    "vacúolos": "vacúolo",
    "vadea": "vadear",
    "vadeaba": "vadear",
    "vadeabais": "vadear",
    "vadeaban": "vadear",
    "vadeabas": "vadear",
    "vadeables": "vadeable",
    "vadead": "vadear",
    "vadeada": "vadear",
    "vadeadas": "vadear",
    "vadeado": "vadear",
    "vadeadora": "vadeador",
    "vadeadoras": "vadeador",
    "vadeadores": "vadeador",
    "vadeados": "vadear",
    "vadeamos": "vadear",
    "vadean": "vadear",
    "vadeando": "vadear",
    "vadeara": "vadear",
    "vadearais": "vadear",
    "vadearan": "vadear",
    "vadearas": "vadear",
    "vadeare": "vadear",
    "vadeareis": "vadear",
    "vadearemos": "vadear",
    "vadearen": "vadear",
    "vadeares": "vadear",
    "vadearon": "vadear",
    "vadeará": "vadear",
    "vadearán": "vadear",
    "vadearás": "vadear",
    "vadearé": "vadear",
    "vadearéis": "vadear",
    "vadearía": "vadear",
    "vadearíais": "vadear",
    "vadearíamos": "vadear",
    "vadearían": "vadear",
    "vadearías": "vadear",
    "vadeas": "vadear",
    "vadease": "vadear",
    "vadeaseis": "vadear",
    "vadeasen": "vadear",
    "vadeases": "vadear",
    "vadeaste": "vadear",
    "vadeasteis": "vadear",
    "vadee": "vadear",
    "vadeemos": "vadear",
    "vadeen": "vadear",
    "vadees": "vadear",
    "vademécums": "vademécum",
    "vadeo": "vadear",
    "vaderas": "vadera",
    "vades": "vade",
    "vadeábamos": "vadear",
    "vadeáis": "vadear",
    "vadeáramos": "vadear",
    "vadeáremos": "vadear",
    "vadeásemos": "vadear",
    "vadeé": "vadear",
    "vadeéis": "vadear",
    "vadeó": "vadear",
    "vados": "vado",
    "vadosa": "vadoso",
    "vadosas": "vadoso",
    "vadosos": "vadoso",
    "vaga": "vago",
    "vagaba": "vagar",
    "vagabais": "vagar",
    "vagaban": "vagar",
    "vagabas": "vagar",
    "vagabunda": "vagabundo",
    "vagabundas": "vagabundo",
    "vagabundea": "vagabundear",
    "vagabundeaba": "vagabundear",
    "vagabundeabais": "vagabundear",
    "vagabundeaban": "vagabundear",
    "vagabundeabas": "vagabundear",
    "vagabundead": "vagabundear",
    "vagabundeada": "vagabundear",
    "vagabundeadas": "vagabundear",
    "vagabundeado": "vagabundear",
    "vagabundeados": "vagabundear",
    "vagabundeamos": "vagabundear",
    "vagabundean": "vagabundear",
    "vagabundeando": "vagabundear",
    "vagabundeara": "vagabundear",
    "vagabundearais": "vagabundear",
    "vagabundearan": "vagabundear",
    "vagabundearas": "vagabundear",
    "vagabundeare": "vagabundear",
    "vagabundeareis": "vagabundear",
    "vagabundearemos": "vagabundear",
    "vagabundearen": "vagabundear",
    "vagabundeares": "vagabundear",
    "vagabundearon": "vagabundear",
    "vagabundeará": "vagabundear",
    "vagabundearán": "vagabundear",
    "vagabundearás": "vagabundear",
    "vagabundearé": "vagabundear",
    "vagabundearéis": "vagabundear",
    "vagabundearía": "vagabundear",
    "vagabundearíais": "vagabundear",
    "vagabundearíamos": "vagabundear",
    "vagabundearían": "vagabundear",
    "vagabundearías": "vagabundear",
    "vagabundeas": "vagabundear",
    "vagabundease": "vagabundear",
    "vagabundeaseis": "vagabundear",
    "vagabundeasen": "vagabundear",
    "vagabundeases": "vagabundear",
    "vagabundeaste": "vagabundear",
    "vagabundeasteis": "vagabundear",
    "vagabundee": "vagabundear",
    "vagabundeemos": "vagabundear",
    "vagabundeen": "vagabundear",
    "vagabundees": "vagabundear",
    "vagabundeo": "vagabundear",
    "vagabundeos": "vagabundeo",
    "vagabunderías": "vagabundería",
    "vagabundeábamos": "vagabundear",
    "vagabundeáis": "vagabundear",
    "vagabundeáramos": "vagabundear",
    "vagabundeáremos": "vagabundear",
    "vagabundeásemos": "vagabundear",
    "vagabundeé": "vagabundear",
    "vagabundeéis": "vagabundear",
    "vagabundeó": "vagabundear",
    "vagabundos": "vagabundo",
    "vagad": "vagar",
    "vagada": "vagar",
    "vagadas": "vagar",
    "vagado": "vagar",
    "vagados": "vagar",
    "vagales": "vagal",
    "vagamos": "vagar",
    "vagamundea": "vagamundear",
    "vagamundeaba": "vagamundear",
    "vagamundeabais": "vagamundear",
    "vagamundeaban": "vagamundear",
    "vagamundeabas": "vagamundear",
    "vagamundead": "vagamundear",
    "vagamundeada": "vagamundear",
    "vagamundeadas": "vagamundear",
    "vagamundeado": "vagamundear",
    "vagamundeados": "vagamundear",
    "vagamundeamos": "vagamundear",
    "vagamundean": "vagamundear",
    "vagamundeando": "vagamundear",
    "vagamundeara": "vagamundear",
    "vagamundearais": "vagamundear",
    "vagamundearan": "vagamundear",
    "vagamundearas": "vagamundear",
    "vagamundeare": "vagamundear",
    "vagamundeareis": "vagamundear",
    "vagamundearemos": "vagamundear",
    "vagamundearen": "vagamundear",
    "vagamundeares": "vagamundear",
    "vagamundearon": "vagamundear",
    "vagamundeará": "vagamundear",
    "vagamundearán": "vagamundear",
    "vagamundearás": "vagamundear",
    "vagamundearé": "vagamundear",
    "vagamundearéis": "vagamundear",
    "vagamundearía": "vagamundear",
    "vagamundearíais": "vagamundear",
    "vagamundearíamos": "vagamundear",
    "vagamundearían": "vagamundear",
    "vagamundearías": "vagamundear",
    "vagamundeas": "vagamundear",
    "vagamundease": "vagamundear",
    "vagamundeaseis": "vagamundear",
    "vagamundeasen": "vagamundear",
    "vagamundeases": "vagamundear",
    "vagamundeaste": "vagamundear",
    "vagamundeasteis": "vagamundear",
    "vagamundee": "vagamundear",
    "vagamundeemos": "vagamundear",
    "vagamundeen": "vagamundear",
    "vagamundees": "vagamundear",
    "vagamundeo": "vagamundear",
    "vagamundeábamos": "vagamundear",
    "vagamundeáis": "vagamundear",
    "vagamundeáramos": "vagamundear",
    "vagamundeáremos": "vagamundear",
    "vagamundeásemos": "vagamundear",
    "vagamundeé": "vagamundear",
    "vagamundeéis": "vagamundear",
    "vagamundeó": "vagamundear",
    "vagan": "vagar",
    "vagancias": "vagancia",
    "vagando": "vagar",
    "vagantes": "vagante",
    "vagara": "vagar",
    "vagarais": "vagar",
    "vagaran": "vagar",
    "vagaras": "vagar",
    "vagare": "vagar",
    "vagareis": "vagar",
    "vagaremos": "vagar",
    "vagaren": "vagar",
    "vagares": "vagar",
    "vagaron": "vagar",
    "vagarosa": "vagaroso",
    "vagarosas": "vagaroso",
    "vagarosidades": "vagarosidad",
    "vagarosos": "vagaroso",
    "vagará": "vagar",
    "vagarán": "vagar",
    "vagarás": "vagar",
    "vagaré": "vagar",
    "vagaréis": "vagar",
    "vagaría": "vagar",
    "vagaríais": "vagar",
    "vagaríamos": "vagar",
    "vagarían": "vagar",
    "vagarías": "vagar",
    "vagas": "vago",
    "vagase": "vagar",
    "vagaseis": "vagar",
    "vagasen": "vagar",
    "vagases": "vagar",
    "vagaste": "vagar",
    "vagasteis": "vagar",
    "vagidos": "vagido",
    "vaginales": "vaginal",
    "vaginas": "vagina",
    "vaginismos": "vaginismo",
    "vaginoplastias": "vaginoplastia",
    "vago": "vagar",
    "vagones": "vagón",
    "vagonetas": "vagoneta",
    "vagos": "vago",
    "vagotomías": "vagotomía",
    "vagotonías": "vagotonía",
    "vagotónica": "vagotónico",
    "vagotónicas": "vagotónico",
    "vagotónicos": "vagotónico",
    "vaguadas": "vaguada",
    "vague": "vagar",
    "vaguea": "vaguear",
    "vagueaba": "vaguear",
    "vagueabais": "vaguear",
    "vagueaban": "vaguear",
    "vagueabas": "vaguear",
    "vagueaciones": "vagueación",
    "vaguead": "vaguear",
    "vagueada": "vaguear",
    "vagueadas": "vaguear",
    "vagueado": "vaguear",
    "vagueados": "vaguear",
    "vagueamos": "vaguear",
    "vaguean": "vaguear",
    "vagueando": "vaguear",
    "vagueara": "vaguear",
    "vaguearais": "vaguear",
    "vaguearan": "vaguear",
    "vaguearas": "vaguear",
    "vagueare": "vaguear",
    "vagueareis": "vaguear",
    "vaguearemos": "vaguear",
    "vaguearen": "vaguear",
    "vagueares": "vaguear",
    "vaguearon": "vaguear",
    "vagueará": "vaguear",
    "vaguearán": "vaguear",
    "vaguearás": "vaguear",
    "vaguearé": "vaguear",
    "vaguearéis": "vaguear",
    "vaguearía": "vaguear",
    "vaguearíais": "vaguear",
    "vaguearíamos": "vaguear",
    "vaguearían": "vaguear",
    "vaguearías": "vaguear",
    "vagueas": "vaguear",
    "vaguease": "vaguear",
    "vagueaseis": "vaguear",
    "vagueasen": "vaguear",
    "vagueases": "vaguear",
    "vagueaste": "vaguear",
    "vagueasteis": "vaguear",
    "vaguedades": "vaguedad",
    "vaguee": "vaguear",
    "vagueemos": "vaguear",
    "vagueen": "vaguear",
    "vaguees": "vaguear",
    "vaguemos": "vagar",
    "vaguen": "vagar",
    "vagueo": "vaguear",
    "vagues": "vagar",
    "vagueábamos": "vaguear",
    "vagueáis": "vaguear",
    "vagueáramos": "vaguear",
    "vagueáremos": "vaguear",
    "vagueásemos": "vaguear",
    "vagueé": "vaguear",
    "vagueéis": "vaguear",
    "vagueó": "vaguear",
    "vagué": "vagar",
    "vaguéis": "vagar",
    "vagábamos": "vagar",
    "vagáis": "vagar",
    "vagáramos": "vagar",
    "vagáremos": "vagar",
    "vagásemos": "vagar",
    "vagó": "vagar",
    "vahajes": "vahaje",
    "vaharadas": "vaharada",
    "vahareras": "vaharera",
    "vaharinas": "vaharina",
    "vahea": "vahear",
    "vaheaba": "vahear",
    "vaheabais": "vahear",
    "vaheaban": "vahear",
    "vaheabas": "vahear",
    "vahead": "vahear",
    "vaheada": "vahear",
    "vaheadas": "vahear",
    "vaheado": "vahear",
    "vaheados": "vahear",
    "vaheamos": "vahear",
    "vahean": "vahear",
    "vaheando": "vahear",
    "vaheara": "vahear",
    "vahearais": "vahear",
    "vahearan": "vahear",
    "vahearas": "vahear",
    "vaheare": "vahear",
    "vaheareis": "vahear",
    "vahearemos": "vahear",
    "vahearen": "vahear",
    "vaheares": "vahear",
    "vahearon": "vahear",
    "vaheará": "vahear",
    "vahearán": "vahear",
    "vahearás": "vahear",
    "vahearé": "vahear",
    "vahearéis": "vahear",
    "vahearía": "vahear",
    "vahearíais": "vahear",
    "vahearíamos": "vahear",
    "vahearían": "vahear",
    "vahearías": "vahear",
    "vaheas": "vahear",
    "vahease": "vahear",
    "vaheaseis": "vahear",
    "vaheasen": "vahear",
    "vaheases": "vahear",
    "vaheaste": "vahear",
    "vaheasteis": "vahear",
    "vahee": "vahear",
    "vaheemos": "vahear",
    "vaheen": "vahear",
    "vahees": "vahear",
    "vaheo": "vahear",
    "vaheábamos": "vahear",
    "vaheáis": "vahear",
    "vaheáramos": "vahear",
    "vaheáremos": "vahear",
    "vaheásemos": "vahear",
    "vaheé": "vahear",
    "vaheéis": "vahear",
    "vaheó": "vahear",
    "vahos": "vaho",
    "vahídos": "vahído",
    "vainas": "vaina",
    "vainicas": "vainica",
    "vainillas": "vainilla",
    "vainillinas": "vainillina",
    "vainiqueras": "vainiquera",
    "vais": "ir",
    "vaisesikas": "vaisesika",
    "vaivenes": "vaivén",
    "vaivodas": "vaivoda",
    "vajillas": "vajilla",
    "valaca": "valaco",
    "valacas": "valaco",
    "valacos": "valaco",
    "valares": "valar",
    "valdenses": "valdense",
    "valdremos": "valer",
    "valdrá": "valer",
    "valdrán": "valer",
    "valdrás": "valer",
    "valdré": "valer",
    "valdréis": "valer",
    "valdría": "valer",
    "valdríais": "valer",
    "valdríamos": "valer",
    "valdrían": "valer",
    "valdrías": "valer",
    "vale": "valer",
    "valed": "valer",
    "valedera": "valedero",
    "valederas": "valedero",
    "valederos": "valedero",
    "valedora": "valedor",
    "valedoras": "valedor",
    "valedores": "valedor",
    "valemos": "valer",
    "valen": "valer",
    "valenciana": "valenciano",
    "valencianas": "valenciano",
    "valencianismos": "valencianismo",
    "valencianistas": "valencianista",
    "valencianizaciones": "valencianización",
    "valencianos": "valenciano",
    "valencias": "valencia",
    "valentina": "valentino",
    "valentinas": "valentino",
    "valentiniana": "valentiniano",
    "valentinianas": "valentiniano",
    "valentinianos": "valentiniano",
    "valentinitas": "valentinita",
    "valentinos": "valentino",
    "valentona": "valentón",
    "valentonadas": "valentonada",
    "valentonas": "valentón",
    "valentones": "valentón",
    "valentías": "valentía",
    "valeres": "valer",
    "valerianas": "valeriana",
    "valerianatos": "valerianato",
    "valerianácea": "valerianáceo",
    "valerianáceas": "valerianáceo",
    "valerianáceos": "valerianáceo",
    "valerolactonas": "valerolactona",
    "valerosa": "valeroso",
    "valerosas": "valeroso",
    "valerosidades": "valerosidad",
    "valerosos": "valeroso",
    "vales": "valer",
    "valetes": "valet",
    "valets": "valet",
    "valetudinaria": "valetudinario",
    "valetudinarias": "valetudinario",
    "valetudinarios": "valetudinario",
    "valga": "valer",
    "valgamos": "valer",
    "valgan": "valer",
    "valgas": "valer",
    "valgo": "valer",
    "valgáis": "valer",
    "valhallas": "valhalla",
    "valholls": "valholl",
    "valiatos": "valiato",
    "valida": "validar",
    "validaba": "validar",
    "validabais": "validar",
    "validaban": "validar",
    "validabas": "validar",
    "validaciones": "validación",
    "validaciónes": "validación",
    "validad": "validar",
    "validada": "validar",
    "validadas": "validar",
    "validado": "validar",
    "validados": "validar",
    "validamos": "validar",
    "validan": "validar",
    "validando": "validar",
    "validara": "validar",
    "validarais": "validar",
    "validaran": "validar",
    "validaras": "validar",
    "validare": "validar",
    "validareis": "validar",
    "validaremos": "validar",
    "validaren": "validar",
    "validares": "validar",
    "validaron": "validar",
    "validará": "validar",
    "validarán": "validar",
    "validarás": "validar",
    "validaré": "validar",
    "validaréis": "validar",
    "validaría": "validar",
    "validaríais": "validar",
    "validaríamos": "validar",
    "validarían": "validar",
    "validarías": "validar",
    "validas": "validar",
    "validase": "validar",
    "validaseis": "validar",
    "validasen": "validar",
    "validases": "validar",
    "validaste": "validar",
    "validasteis": "validar",
    "valide": "validar",
    "valideces": "validez",
    "validemos": "validar",
    "validen": "validar",
    "valides": "validar",
    "valido": "validar",
    "validos": "valido",
    "validábamos": "validar",
    "validáis": "validar",
    "validáramos": "validar",
    "validáremos": "validar",
    "validásemos": "validar",
    "validé": "validar",
    "validéis": "validar",
    "validó": "validar",
    "valiendo": "valer",
    "valientes": "valiente",
    "valiera": "valer",
    "valierais": "valer",
    "valieran": "valer",
    "valieras": "valer",
    "valiere": "valer",
    "valiereis": "valer",
    "valieren": "valer",
    "valieres": "valer",
    "valieron": "valer",
    "valiese": "valer",
    "valieseis": "valer",
    "valiesen": "valer",
    "valieses": "valer",
    "valijas": "valija",
    "valijera": "valijero",
    "valijeras": "valijero",
    "valijeros": "valijero",
    "valimientos": "valimiento",
    "valimos": "valer",
    "valinas": "valina",
    "valiosa": "valioso",
    "valiosas": "valioso",
    "valiosos": "valioso",
    "valisoletana": "valisoletano",
    "valisoletanas": "valisoletano",
    "valisoletanos": "valisoletano",
    "valiste": "valer",
    "valisteis": "valer",
    "valiéramos": "valer",
    "valiéremos": "valer",
    "valiésemos": "valer",
    "valió": "valer",
    "valkirias": "valkiria",
    "valla": "vallar",
    "vallaba": "vallar",
    "vallabais": "vallar",
    "vallaban": "vallar",
    "vallabas": "vallar",
    "vallad": "vallar",
    "vallada": "vallar",
    "valladares": "valladar",
    "valladas": "vallar",
    "vallado": "vallar",
    "vallados": "vallar",
    "vallamos": "vallar",
    "vallan": "vallar",
    "vallando": "vallar",
    "vallara": "vallar",
    "vallarais": "vallar",
    "vallaran": "vallar",
    "vallaras": "vallar",
    "vallare": "vallar",
    "vallareis": "vallar",
    "vallaremos": "vallar",
    "vallaren": "vallar",
    "vallares": "vallar",
    "vallaron": "vallar",
    "vallará": "vallar",
    "vallarán": "vallar",
    "vallarás": "vallar",
    "vallaré": "vallar",
    "vallaréis": "vallar",
    "vallaría": "vallar",
    "vallaríais": "vallar",
    "vallaríamos": "vallar",
    "vallarían": "vallar",
    "vallarías": "vallar",
    "vallas": "vallar",
    "vallase": "vallar",
    "vallaseis": "vallar",
    "vallasen": "vallar",
    "vallases": "vallar",
    "vallaste": "vallar",
    "vallasteis": "vallar",
    "valle": "vallar",
    "vallemos": "vallar",
    "vallen": "vallar",
    "valles": "valle",
    "vallesana": "vallesano",
    "vallesanas": "vallesano",
    "vallesanos": "vallesano",
    "vallicos": "vallico",
    "vallisoletana": "vallisoletano",
    "vallisoletanas": "vallisoletano",
    "vallisoletanos": "vallisoletano",
    "vallistas": "vallista",
    "vallo": "vallar",
    "vallábamos": "vallar",
    "valláis": "vallar",
    "valláramos": "vallar",
    "valláremos": "vallar",
    "vallásemos": "vallar",
    "vallé": "vallar",
    "valléis": "vallar",
    "valló": "vallar",
    "valona": "valón",
    "valonas": "valón",
    "valones": "valón",
    "valora": "valorar",
    "valoraba": "valorar",
    "valorabais": "valorar",
    "valoraban": "valorar",
    "valorabas": "valorar",
    "valorables": "valorable",
    "valoraciones": "valoración",
    "valorad": "valorar",
    "valorada": "valorar",
    "valoradas": "valorar",
    "valorado": "valorar",
    "valorados": "valorar",
    "valoramos": "valorar",
    "valoran": "valorar",
    "valorando": "valorar",
    "valorara": "valorar",
    "valorarais": "valorar",
    "valoraran": "valorar",
    "valoraras": "valorar",
    "valorare": "valorar",
    "valorareis": "valorar",
    "valoraremos": "valorar",
    "valoraren": "valorar",
    "valorares": "valorar",
    "valoraron": "valorar",
    "valorará": "valorar",
    "valorarán": "valorar",
    "valorarás": "valorar",
    "valoraré": "valorar",
    "valoraréis": "valorar",
    "valoraría": "valorar",
    "valoraríais": "valorar",
    "valoraríamos": "valorar",
    "valorarían": "valorar",
    "valorarías": "valorar",
    "valoras": "valorar",
    "valorase": "valorar",
    "valoraseis": "valorar",
    "valorasen": "valorar",
    "valorases": "valorar",
    "valoraste": "valorar",
    "valorasteis": "valorar",
    "valorativa": "valorativo",
    "valorativas": "valorativo",
    "valorativos": "valorativo",
    "valore": "valorar",
    "valoremos": "valorar",
    "valoren": "valorar",
    "valores": "valorar",
    "valorice": "valorizar",
    "valoricemos": "valorizar",
    "valoricen": "valorizar",
    "valorices": "valorizar",
    "valoricé": "valorizar",
    "valoricéis": "valorizar",
    "valoriza": "valorizar",
    "valorizaba": "valorizar",
    "valorizabais": "valorizar",
    "valorizaban": "valorizar",
    "valorizabas": "valorizar",
    "valorizables": "valorizable",
    "valorizaciones": "valorización",
    "valorizad": "valorizar",
    "valorizada": "valorizar",
    "valorizadas": "valorizar",
    "valorizado": "valorizar",
    "valorizadora": "valorizador",
    "valorizadoras": "valorizador",
    "valorizadores": "valorizador",
    "valorizados": "valorizar",
    "valorizamos": "valorizar",
    "valorizan": "valorizar",
    "valorizando": "valorizar",
    "valorizara": "valorizar",
    "valorizarais": "valorizar",
    "valorizaran": "valorizar",
    "valorizaras": "valorizar",
    "valorizare": "valorizar",
    "valorizareis": "valorizar",
    "valorizaremos": "valorizar",
    "valorizaren": "valorizar",
    "valorizares": "valorizar",
    "valorizaron": "valorizar",
    "valorizará": "valorizar",
    "valorizarán": "valorizar",
    "valorizarás": "valorizar",
    "valorizaré": "valorizar",
    "valorizaréis": "valorizar",
    "valorizaría": "valorizar",
    "valorizaríais": "valorizar",
    "valorizaríamos": "valorizar",
    "valorizarían": "valorizar",
    "valorizarías": "valorizar",
    "valorizas": "valorizar",
    "valorizase": "valorizar",
    "valorizaseis": "valorizar",
    "valorizasen": "valorizar",
    "valorizases": "valorizar",
    "valorizaste": "valorizar",
    "valorizasteis": "valorizar",
    "valorizo": "valorizar",
    "valorizábamos": "valorizar",
    "valorizáis": "valorizar",
    "valorizáramos": "valorizar",
    "valorizáremos": "valorizar",
    "valorizásemos": "valorizar",
    "valorizó": "valorizar",
    "valoro": "valorar",
    "valorábamos": "valorar",
    "valoráis": "valorar",
    "valoráramos": "valorar",
    "valoráremos": "valorar",
    "valorásemos": "valorar",
    "valoré": "valorar",
    "valoréis": "valorar",
    "valoró": "valorar",
    "valquirias": "valquiria",
    "valsa": "valsar",
    "valsaba": "valsar",
    "valsabais": "valsar",
    "valsaban": "valsar",
    "valsabas": "valsar",
    "valsad": "valsar",
    "valsada": "valsar",
    "valsadas": "valsar",
    "valsado": "valsar",
    "valsados": "valsar",
    "valsamos": "valsar",
    "valsan": "valsar",
    "valsando": "valsar",
    "valsara": "valsar",
    "valsarais": "valsar",
    "valsaran": "valsar",
    "valsaras": "valsar",
    "valsare": "valsar",
    "valsareis": "valsar",
    "valsaremos": "valsar",
    "valsaren": "valsar",
    "valsares": "valsar",
    "valsaron": "valsar",
    "valsará": "valsar",
    "valsarán": "valsar",
    "valsarás": "valsar",
    "valsaré": "valsar",
    "valsaréis": "valsar",
    "valsaría": "valsar",
    "valsaríais": "valsar",
    "valsaríamos": "valsar",
    "valsarían": "valsar",
    "valsarías": "valsar",
    "valsas": "valsar",
    "valsase": "valsar",
    "valsaseis": "valsar",
    "valsasen": "valsar",
    "valsases": "valsar",
    "valsaste": "valsar",
    "valsasteis": "valsar",
    "valse": "valsar",
    "valsemos": "valsar",
    "valsen": "valsar",
    "valses": "valse",
    "valso": "valsar",
    "valsábamos": "valsar",
    "valsáis": "valsar",
    "valsáramos": "valsar",
    "valsáremos": "valsar",
    "valsásemos": "valsar",
    "valsé": "valsar",
    "valséis": "valsar",
    "valsó": "valsar",
    "valuaba": "valuar",
    "valuabais": "valuar",
    "valuaban": "valuar",
    "valuabas": "valuar",
    "valuaciones": "valuación",
    "valuad": "valuar",
    "valuada": "valuar",
    "valuadas": "valuar",
    "valuado": "valuar",
    "valuados": "valuar",
    "valuamos": "valuar",
    "valuando": "valuar",
    "valuara": "valuar",
    "valuarais": "valuar",
    "valuaran": "valuar",
    "valuaras": "valuar",
    "valuare": "valuar",
    "valuareis": "valuar",
    "valuaremos": "valuar",
    "valuaren": "valuar",
    "valuares": "valuar",
    "valuaron": "valuar",
    "valuará": "valuar",
    "valuarán": "valuar",
    "valuarás": "valuar",
    "valuaré": "valuar",
    "valuaréis": "valuar",
    "valuaría": "valuar",
    "valuaríais": "valuar",
    "valuaríamos": "valuar",
    "valuarían": "valuar",
    "valuarías": "valuar",
    "valuase": "valuar",
    "valuaseis": "valuar",
    "valuasen": "valuar",
    "valuases": "valuar",
    "valuaste": "valuar",
    "valuasteis": "valuar",
    "valuemos": "valuar",
    "valuábamos": "valuar",
    "valuáis": "valuar",
    "valuáramos": "valuar",
    "valuáremos": "valuar",
    "valuásemos": "valuar",
    "valué": "valuar",
    "valuéis": "valuar",
    "valuó": "valuar",
    "valvas": "valva",
    "valvasores": "valvasor",
    "valvulares": "valvular",
    "valvulinas": "valvulina",
    "valvulopatías": "valvulopatía",
    "valéculas": "valécula",
    "valéis": "valer",
    "valérica": "valérico",
    "valéricas": "valérico",
    "valéricos": "valérico",
    "valí": "valer",
    "valía": "valer",
    "valíais": "valer",
    "valíamos": "valer",
    "valían": "valer",
    "valías": "valía",
    "valíes": "valí",
    "valís": "valí",
    "valóna": "valón",
    "valónas": "valón",
    "valónes": "valón",
    "valúa": "valuar",
    "valúan": "valuar",
    "valúas": "valuar",
    "valúe": "valuar",
    "valúen": "valuar",
    "valúes": "valuar",
    "valúo": "valuar",
    "vamos": "ir",
    "vampiresas": "vampiresa",
    "vampirismos": "vampirismo",
    "vampiros": "vampiro",
    "vamps": "vamp",
    "van": "ir",
    "vana": "vano",
    "vanadatos": "vanadato",
    "vanadinitas": "vanadinita",
    "vanadios": "vanadio",
    "vanagloria": "vanagloriar",
    "vanagloriaba": "vanagloriar",
    "vanagloriabais": "vanagloriar",
    "vanagloriaban": "vanagloriar",
    "vanagloriabas": "vanagloriar",
    "vanagloriad": "vanagloriar",
    "vanagloriada": "vanagloriar",
    "vanagloriadas": "vanagloriar",
    "vanagloriado": "vanagloriar",
    "vanagloriados": "vanagloriar",
    "vanagloriamos": "vanagloriar",
    "vanaglorian": "vanagloriar",
    "vanagloriando": "vanagloriar",
    "vanagloriara": "vanagloriar",
    "vanagloriarais": "vanagloriar",
    "vanagloriaran": "vanagloriar",
    "vanagloriaras": "vanagloriar",
    "vanagloriare": "vanagloriar",
    "vanagloriareis": "vanagloriar",
    "vanagloriaremos": "vanagloriar",
    "vanagloriaren": "vanagloriar",
    "vanagloriares": "vanagloriar",
    "vanagloriaron": "vanagloriar",
    "vanagloriará": "vanagloriar",
    "vanagloriarán": "vanagloriar",
    "vanagloriarás": "vanagloriar",
    "vanagloriaré": "vanagloriar",
    "vanagloriaréis": "vanagloriar",
    "vanagloriaría": "vanagloriar",
    "vanagloriaríais": "vanagloriar",
    "vanagloriaríamos": "vanagloriar",
    "vanagloriarían": "vanagloriar",
    "vanagloriarías": "vanagloriar",
    "vanaglorias": "vanagloriar",
    "vanagloriase": "vanagloriar",
    "vanagloriaseis": "vanagloriar",
    "vanagloriasen": "vanagloriar",
    "vanagloriases": "vanagloriar",
    "vanagloriaste": "vanagloriar",
    "vanagloriasteis": "vanagloriar",
    "vanaglorie": "vanagloriar",
    "vanagloriemos": "vanagloriar",
    "vanaglorien": "vanagloriar",
    "vanaglories": "vanagloriar",
    "vanaglorio": "vanagloriar",
    "vanagloriosa": "vanaglorioso",
    "vanagloriosas": "vanaglorioso",
    "vanagloriosos": "vanaglorioso",
    "vanagloriábamos": "vanagloriar",
    "vanagloriáis": "vanagloriar",
    "vanagloriáramos": "vanagloriar",
    "vanagloriáremos": "vanagloriar",
    "vanagloriásemos": "vanagloriar",
    "vanaglorié": "vanagloriar",
    "vanagloriéis": "vanagloriar",
    "vanaglorió": "vanagloriar",
    "vanas": "vano",
    "vandalismos": "vandalismo",
    "vandálica": "vandálico",
    "vandálicas": "vandálico",
    "vandálicos": "vandálico",
    "vanea": "vanear",
    "vaneaba": "vanear",
    "vaneabais": "vanear",
    "vaneaban": "vanear",
    "vaneabas": "vanear",
    "vanead": "vanear",
    "vaneada": "vanear",
    "vaneadas": "vanear",
    "vaneado": "vanear",
    "vaneados": "vanear",
    "vaneamos": "vanear",
    "vanean": "vanear",
    "vaneando": "vanear",
    "vaneara": "vanear",
    "vanearais": "vanear",
    "vanearan": "vanear",
    "vanearas": "vanear",
    "vaneare": "vanear",
    "vaneareis": "vanear",
    "vanearemos": "vanear",
    "vanearen": "vanear",
    "vaneares": "vanear",
    "vanearon": "vanear",
    "vaneará": "vanear",
    "vanearán": "vanear",
    "vanearás": "vanear",
    "vanearé": "vanear",
    "vanearéis": "vanear",
    "vanearía": "vanear",
    "vanearíais": "vanear",
    "vanearíamos": "vanear",
    "vanearían": "vanear",
    "vanearías": "vanear",
    "vaneas": "vanear",
    "vanease": "vanear",
    "vaneaseis": "vanear",
    "vaneasen": "vanear",
    "vaneases": "vanear",
    "vaneaste": "vanear",
    "vaneasteis": "vanear",
    "vanee": "vanear",
    "vaneemos": "vanear",
    "vaneen": "vanear",
    "vanees": "vanear",
    "vaneo": "vanear",
    "vanesas": "vanesa",
    "vaneábamos": "vanear",
    "vaneáis": "vanear",
    "vaneáramos": "vanear",
    "vaneáremos": "vanear",
    "vaneásemos": "vanear",
    "vaneé": "vanear",
    "vaneéis": "vanear",
    "vaneó": "vanear",
    "vanguardias": "vanguardia",
    "vanguardismos": "vanguardismo",
    "vanguardistas": "vanguardista",
    "vanidades": "vanidad",
    "vanidosa": "vanidoso",
    "vanidosas": "vanidoso",
    "vanidosos": "vanidoso",
    "vanilocuencias": "vanilocuencia",
    "vanilocuentes": "vanilocuente",
    "vaniloquios": "vaniloquio",
    "vanistorios": "vanistorio",
    "vanos": "vano",
    "vanádica": "vanádico",
    "vanádicas": "vanádico",
    "vanádicos": "vanádico",
    "vanílocua": "vanílocuo",
    "vanílocuas": "vanílocuo",
    "vanílocuos": "vanílocuo",
    "vapora": "vaporar",
    "vaporaba": "vaporar",
    "vaporabais": "vaporar",
    "vaporaban": "vaporar",
    "vaporabas": "vaporar",
    "vaporables": "vaporable",
    "vaporad": "vaporar",
    "vaporada": "vaporar",
    "vaporadas": "vaporar",
    "vaporado": "vaporar",
    "vaporados": "vaporar",
    "vaporamos": "vaporar",
    "vaporan": "vaporar",
    "vaporando": "vaporar",
    "vaporara": "vaporar",
    "vaporarais": "vaporar",
    "vaporaran": "vaporar",
    "vaporaras": "vaporar",
    "vaporare": "vaporar",
    "vaporareis": "vaporar",
    "vaporaremos": "vaporar",
    "vaporaren": "vaporar",
    "vaporares": "vaporar",
    "vaporarios": "vaporario",
    "vaporaron": "vaporar",
    "vaporará": "vaporar",
    "vaporarán": "vaporar",
    "vaporarás": "vaporar",
    "vaporaré": "vaporar",
    "vaporaréis": "vaporar",
    "vaporaría": "vaporar",
    "vaporaríais": "vaporar",
    "vaporaríamos": "vaporar",
    "vaporarían": "vaporar",
    "vaporarías": "vaporar",
    "vaporas": "vaporar",
    "vaporase": "vaporar",
    "vaporaseis": "vaporar",
    "vaporasen": "vaporar",
    "vaporases": "vaporar",
    "vaporaste": "vaporar",
    "vaporasteis": "vaporar",
    "vapore": "vaporar",
    "vaporemos": "vaporar",
    "vaporen": "vaporar",
    "vapores": "vaporar",
    "vaporice": "vaporizar",
    "vaporicemos": "vaporizar",
    "vaporicen": "vaporizar",
    "vaporices": "vaporizar",
    "vaporicé": "vaporizar",
    "vaporicéis": "vaporizar",
    "vaporiza": "vaporizar",
    "vaporizaba": "vaporizar",
    "vaporizabais": "vaporizar",
    "vaporizaban": "vaporizar",
    "vaporizabas": "vaporizar",
    "vaporizaciones": "vaporización",
    "vaporizad": "vaporizar",
    "vaporizada": "vaporizar",
    "vaporizadas": "vaporizar",
    "vaporizado": "vaporizar",
    "vaporizadores": "vaporizador",
    "vaporizados": "vaporizar",
    "vaporizamos": "vaporizar",
    "vaporizan": "vaporizar",
    "vaporizando": "vaporizar",
    "vaporizara": "vaporizar",
    "vaporizarais": "vaporizar",
    "vaporizaran": "vaporizar",
    "vaporizaras": "vaporizar",
    "vaporizare": "vaporizar",
    "vaporizareis": "vaporizar",
    "vaporizaremos": "vaporizar",
    "vaporizaren": "vaporizar",
    "vaporizares": "vaporizar",
    "vaporizaron": "vaporizar",
    "vaporizará": "vaporizar",
    "vaporizarán": "vaporizar",
    "vaporizarás": "vaporizar",
    "vaporizaré": "vaporizar",
    "vaporizaréis": "vaporizar",
    "vaporizaría": "vaporizar",
    "vaporizaríais": "vaporizar",
    "vaporizaríamos": "vaporizar",
    "vaporizarían": "vaporizar",
    "vaporizarías": "vaporizar",
    "vaporizas": "vaporizar",
    "vaporizase": "vaporizar",
    "vaporizaseis": "vaporizar",
    "vaporizasen": "vaporizar",
    "vaporizases": "vaporizar",
    "vaporizaste": "vaporizar",
    "vaporizasteis": "vaporizar",
    "vaporizo": "vaporizar",
    "vaporizábamos": "vaporizar",
    "vaporizáis": "vaporizar",
    "vaporizáramos": "vaporizar",
    "vaporizáremos": "vaporizar",
    "vaporizásemos": "vaporizar",
    "vaporizó": "vaporizar",
    "vaporo": "vaporar",
    "vaporosa": "vaporoso",
    "vaporosas": "vaporoso",
    "vaporosos": "vaporoso",
    "vaporábamos": "vaporar",
    "vaporáis": "vaporar",
    "vaporáramos": "vaporar",
    "vaporáremos": "vaporar",
    "vaporásemos": "vaporar",
    "vaporé": "vaporar",
    "vaporéis": "vaporar",
    "vaporímetros": "vaporímetro",
    "vaporó": "vaporar",
    "vapotrones": "vapotrón",
    "vapulea": "vapulear",
    "vapuleaba": "vapulear",
    "vapuleabais": "vapulear",
    "vapuleaban": "vapulear",
    "vapuleabas": "vapulear",
    "vapulead": "vapulear",
    "vapuleada": "vapulear",
    "vapuleadas": "vapulear",
    "vapuleado": "vapulear",
    "vapuleadora": "vapuleador",
    "vapuleadoras": "vapuleador",
    "vapuleadores": "vapuleador",
    "vapuleados": "vapulear",
    "vapuleamientos": "vapuleamiento",
    "vapuleamos": "vapulear",
    "vapulean": "vapulear",
    "vapuleando": "vapulear",
    "vapuleara": "vapulear",
    "vapulearais": "vapulear",
    "vapulearan": "vapulear",
    "vapulearas": "vapulear",
    "vapuleare": "vapulear",
    "vapuleareis": "vapulear",
    "vapulearemos": "vapulear",
    "vapulearen": "vapulear",
    "vapuleares": "vapulear",
    "vapulearon": "vapulear",
    "vapuleará": "vapulear",
    "vapulearán": "vapulear",
    "vapulearás": "vapulear",
    "vapulearé": "vapulear",
    "vapulearéis": "vapulear",
    "vapulearía": "vapulear",
    "vapulearíais": "vapulear",
    "vapulearíamos": "vapulear",
    "vapulearían": "vapulear",
    "vapulearías": "vapulear",
    "vapuleas": "vapulear",
    "vapulease": "vapulear",
    "vapuleaseis": "vapulear",
    "vapuleasen": "vapulear",
    "vapuleases": "vapulear",
    "vapuleaste": "vapulear",
    "vapuleasteis": "vapulear",
    "vapulee": "vapulear",
    "vapuleemos": "vapulear",
    "vapuleen": "vapulear",
    "vapulees": "vapulear",
    "vapuleo": "vapulear",
    "vapuleos": "vapuleo",
    "vapuleábamos": "vapulear",
    "vapuleáis": "vapulear",
    "vapuleáramos": "vapulear",
    "vapuleáremos": "vapulear",
    "vapuleásemos": "vapulear",
    "vapuleé": "vapulear",
    "vapuleéis": "vapulear",
    "vapuleó": "vapulear",
    "vaque": "vacar",
    "vaquemos": "vacar",
    "vaquen": "vacar",
    "vaquera": "vaquero",
    "vaqueras": "vaquero",
    "vaqueriza": "vaquerizo",
    "vaquerizas": "vaquerizo",
    "vaquerizos": "vaquerizo",
    "vaqueros": "vaquero",
    "vaquerías": "vaquería",
    "vaques": "vacar",
    "vaquetas": "vaqueta",
    "vaquillas": "vaquilla",
    "vaqué": "vacar",
    "vaquéis": "vacar",
    "vara": "varar",
    "varaba": "varar",
    "varabais": "varar",
    "varaban": "varar",
    "varabas": "varar",
    "varactores": "varactor",
    "varad": "varar",
    "varada": "varar",
    "varadas": "varar",
    "varadera": "varadero",
    "varaderas": "varadero",
    "varaderos": "varadero",
    "varado": "varar",
    "varados": "varar",
    "varaduras": "varadura",
    "varales": "varal",
    "varamientos": "varamiento",
    "varamos": "varar",
    "varan": "varar",
    "varando": "varar",
    "varanos": "varano",
    "varapalos": "varapalo",
    "varara": "varar",
    "vararais": "varar",
    "vararan": "varar",
    "vararas": "varar",
    "varare": "varar",
    "varareis": "varar",
    "vararemos": "varar",
    "vararen": "varar",
    "varares": "varar",
    "vararon": "varar",
    "varará": "varar",
    "vararán": "varar",
    "vararás": "varar",
    "vararé": "varar",
    "vararéis": "varar",
    "vararía": "varar",
    "vararíais": "varar",
    "vararíamos": "varar",
    "vararían": "varar",
    "vararías": "varar",
    "varas": "varar",
    "varase": "varar",
    "varaseis": "varar",
    "varasen": "varar",
    "varases": "varar",
    "varasetos": "varaseto",
    "varaste": "varar",
    "varasteis": "varar",
    "vare": "varar",
    "varea": "varear",
    "vareaba": "varear",
    "vareabais": "varear",
    "vareaban": "varear",
    "vareabas": "varear",
    "varead": "varear",
    "vareada": "varear",
    "vareadas": "varear",
    "vareado": "varear",
    "vareadora": "vareador",
    "vareadoras": "vareador",
    "vareadores": "vareador",
    "vareados": "varear",
    "vareajes": "vareaje",
    "vareamos": "varear",
    "varean": "varear",
    "vareando": "varear",
    "vareara": "varear",
    "varearais": "varear",
    "varearan": "varear",
    "varearas": "varear",
    "vareare": "varear",
    "vareareis": "varear",
    "varearemos": "varear",
    "varearen": "varear",
    "vareares": "varear",
    "varearon": "varear",
    "vareará": "varear",
    "varearán": "varear",
    "varearás": "varear",
    "varearé": "varear",
    "varearéis": "varear",
    "varearía": "varear",
    "varearíais": "varear",
    "varearíamos": "varear",
    "varearían": "varear",
    "varearías": "varear",
    "vareas": "varear",
    "varease": "varear",
    "vareaseis": "varear",
    "vareasen": "varear",
    "vareases": "varear",
    "vareaste": "varear",
    "vareasteis": "varear",
    "varee": "varear",
    "vareemos": "varear",
    "vareen": "varear",
    "varees": "varear",
    "varegos": "varego",
    "varejones": "varejón",
    "varemos": "varar",
    "varen": "varar",
    "varengas": "varenga",
    "vareo": "varear",
    "vareos": "vareo",
    "vares": "varar",
    "varetas": "vareta",
    "varetazos": "varetazo",
    "varetones": "varetón",
    "vareábamos": "varear",
    "vareáis": "varear",
    "vareáramos": "varear",
    "vareáremos": "varear",
    "vareásemos": "varear",
    "vareé": "varear",
    "vareéis": "varear",
    "vareó": "varear",
    "varganales": "varganal",
    "vargas": "varga",
    "vargueños": "vargueño",
    "varhorímetros": "varhorímetro",
    "varia": "vario",
    "variaba": "variar",
    "variabais": "variar",
    "variaban": "variar",
    "variabas": "variar",
    "variabilidades": "variabilidad",
    "variables": "variable",
    "variaciones": "variación",
    "variad": "variar",
    "variada": "variar",
    "variadas": "variar",
    "variado": "variar",
    "variadores": "variador",
    "variados": "variar",
    "variamos": "variar",
    "variancias": "variancia",
    "variando": "variar",
    "variantes": "variante",
    "varianzas": "varianza",
    "variara": "variar",
    "variarais": "variar",
    "variaran": "variar",
    "variaras": "variar",
    "variare": "variar",
    "variareis": "variar",
    "variaremos": "variar",
    "variaren": "variar",
    "variares": "variar",
    "variaron": "variar",
    "variará": "variar",
    "variarán": "variar",
    "variarás": "variar",
    "variaré": "variar",
    "variaréis": "variar",
    "variaría": "variar",
    "variaríais": "variar",
    "variaríamos": "variar",
    "variarían": "variar",
    "variarías": "variar",
    "varias": "varios",
    "variase": "variar",
    "variaseis": "variar",
    "variasen": "variar",
    "variases": "variar",
    "variaste": "variar",
    "variasteis": "variar",
    "varice": "variz",
    "varicelas": "varicela",
    "varices": "variz",
    "varicoceles": "varicocele",
    "varicosa": "varicoso",
    "varicosas": "varicoso",
    "varicosos": "varicoso",
    "varicélica": "varicélico",
    "varicélicas": "varicélico",
    "varicélicos": "varicélico",
    "variedades": "variedad",
    "variegada": "variegado",
    "variegadas": "variegado",
    "variegados": "variegado",
    "variemos": "variar",
    "varietales": "varietal",
    "varilargueros": "varilarguero",
    "varillajes": "varillaje",
    "varillas": "varilla",
    "variolaciones": "variolación",
    "variolizaciones": "variolización",
    "varioloides": "varioloide",
    "variolosa": "varioloso",
    "variolosas": "varioloso",
    "variolosos": "varioloso",
    "variolítica": "variolítico",
    "variolíticas": "variolítico",
    "variolíticos": "variolítico",
    "variopinta": "variopinto",
    "variopintas": "variopinto",
    "variopintos": "variopinto",
    "varios": "vario",
    "varis": "vari",
    "variscitas": "variscita",
    "varistancias": "varistancia",
    "varistores": "varistor",
    "varitas": "varita",
    "variábamos": "variar",
    "variáis": "variar",
    "variáramos": "variar",
    "variáremos": "variar",
    "variásemos": "variar",
    "varié": "variar",
    "variéis": "variar",
    "varió": "variar",
    "variólica": "variólico",
    "variólicas": "variólico",
    "variólicos": "variólico",
    "variómetros": "variómetro",
    "varmetros": "varmetro",
    "varo": "varar",
    "varonas": "varona",
    "varones": "varón",
    "varonesas": "varonesa",
    "varoniles": "varonil",
    "varonías": "varonía",
    "varsoviana": "varsoviano",
    "varsovianas": "varsoviano",
    "varsovianos": "varsoviano",
    "varvas": "varva",
    "varábamos": "varar",
    "varáis": "varar",
    "varáramos": "varar",
    "varáremos": "varar",
    "varásemos": "varar",
    "varé": "varar",
    "varéis": "varar",
    "varía": "variar",
    "varían": "variar",
    "varías": "variar",
    "varíe": "variar",
    "varíen": "variar",
    "varíes": "variar",
    "varío": "variar",
    "varó": "varar",
    "varónes": "varón",
    "vas": "ir",
    "vasalla": "vasallo",
    "vasallajes": "vasallaje",
    "vasallas": "vasallo",
    "vasallos": "vasallo",
    "vasares": "vasar",
    "vasca": "vasco",
    "vascas": "vasco",
    "vascona": "vascón",
    "vasconas": "vascón",
    "vascones": "vascón",
    "vascongada": "vascongado",
    "vascongadas": "vascongado",
    "vascongados": "vascongado",
    "vascos": "vasco",
    "vascuences": "vascuence",
    "vasculares": "vascular",
    "vasculopatías": "vasculopatía",
    "vasculosa": "vasculoso",
    "vasculosas": "vasculoso",
    "vasculosos": "vasculoso",
    "vascóloga": "vascólogo",
    "vascólogas": "vascólogo",
    "vascólogos": "vascólogo",
    "vasectomías": "vasectomía",
    "vaselinas": "vaselina",
    "vaseras": "vasera",
    "vasijas": "vasija",
    "vasijeros": "vasijero",
    "vasillos": "vasillo",
    "vasoactiva": "vasoactivo",
    "vasoactivas": "vasoactivo",
    "vasoactivos": "vasoactivo",
    "vasoconstricciones": "vasoconstricción",
    "vasoconstrictora": "vasoconstrictor",
    "vasoconstrictoras": "vasoconstrictor",
    "vasoconstrictores": "vasoconstrictor",
    "vasodentinas": "vasodentina",
    "vasodilataciones": "vasodilatación",
    "vasodilatadora": "vasodilatador",
    "vasodilatadoras": "vasodilatador",
    "vasodilatadores": "vasodilatador",
    "vasomotora": "vasomotor",
    "vasomotoras": "vasomotor",
    "vasomotores": "vasomotor",
    "vasopresinas": "vasopresina",
    "vasos": "vaso",
    "vasquismos": "vasquismo",
    "vasta": "vasto",
    "vastas": "vasto",
    "vastedades": "vastedad",
    "vastos": "vasto",
    "vatajes": "vataje",
    "vates": "vate",
    "vaticana": "vaticano",
    "vaticanas": "vaticano",
    "vaticanistas": "vaticanista",
    "vaticanos": "vaticano",
    "vaticina": "vaticinar",
    "vaticinaba": "vaticinar",
    "vaticinabais": "vaticinar",
    "vaticinaban": "vaticinar",
    "vaticinabas": "vaticinar",
    "vaticinad": "vaticinar",
    "vaticinada": "vaticinar",
    "vaticinadas": "vaticinar",
    "vaticinado": "vaticinar",
    "vaticinadora": "vaticinador",
    "vaticinadoras": "vaticinador",
    "vaticinadores": "vaticinador",
    "vaticinados": "vaticinar",
    "vaticinamos": "vaticinar",
    "vaticinan": "vaticinar",
    "vaticinando": "vaticinar",
    "vaticinara": "vaticinar",
    "vaticinarais": "vaticinar",
    "vaticinaran": "vaticinar",
    "vaticinaras": "vaticinar",
    "vaticinare": "vaticinar",
    "vaticinareis": "vaticinar",
    "vaticinaremos": "vaticinar",
    "vaticinaren": "vaticinar",
    "vaticinares": "vaticinar",
    "vaticinaron": "vaticinar",
    "vaticinará": "vaticinar",
    "vaticinarán": "vaticinar",
    "vaticinarás": "vaticinar",
    "vaticinaré": "vaticinar",
    "vaticinaréis": "vaticinar",
    "vaticinaría": "vaticinar",
    "vaticinaríais": "vaticinar",
    "vaticinaríamos": "vaticinar",
    "vaticinarían": "vaticinar",
    "vaticinarías": "vaticinar",
    "vaticinas": "vaticinar",
    "vaticinase": "vaticinar",
    "vaticinaseis": "vaticinar",
    "vaticinasen": "vaticinar",
    "vaticinases": "vaticinar",
    "vaticinaste": "vaticinar",
    "vaticinasteis": "vaticinar",
    "vaticine": "vaticinar",
    "vaticinemos": "vaticinar",
    "vaticinen": "vaticinar",
    "vaticines": "vaticinar",
    "vaticinios": "vaticinio",
    "vaticino": "vaticinar",
    "vaticinábamos": "vaticinar",
    "vaticináis": "vaticinar",
    "vaticináramos": "vaticinar",
    "vaticináremos": "vaticinar",
    "vaticinásemos": "vaticinar",
    "vaticiné": "vaticinar",
    "vaticinéis": "vaticinar",
    "vaticinó": "vaticinar",
    "vatihorímetros": "vatihorímetro",
    "vatios": "vatio",
    "vatiómetros": "vatiómetro",
    "vatus": "vatu",
    "vatímetros": "vatímetro",
    "vaudevilles": "vaudeville",
    "vaya": "ir",
    "vayamos": "ir",
    "vayan": "ir",
    "vayas": "vaya",
    "vayáis": "ir",
    "ve": "ver",
    "vea": "ver",
    "veamos": "ver",
    "vean": "ver",
    "veas": "ver",
    "vecera": "vecero",
    "veceras": "vecero",
    "veceros": "vecero",
    "vecerías": "vecería",
    "veces": "vez",
    "vecina": "vecino",
    "vecinales": "vecinal",
    "vecinas": "vecino",
    "vecindades": "vecindad",
    "vecindarios": "vecindario",
    "vecinos": "vecino",
    "vectores": "vector",
    "vectoriales": "vectorial",
    "vectorizaciones": "vectorización",
    "vectorizada": "vectorizado",
    "vectorizadas": "vectorizado",
    "vectorizados": "vectorizado",
    "ved": "ver",
    "veda": "vedar",
    "vedaba": "vedar",
    "vedabais": "vedar",
    "vedaban": "vedar",
    "vedabas": "vedar",
    "vedad": "vedar",
    "vedada": "vedar",
    "vedadas": "vedar",
    "vedado": "vedar",
    "vedados": "vedar",
    "vedamos": "vedar",
    "vedan": "vedar",
    "vedando": "vedar",
    "vedantas": "vedanta",
    "vedara": "vedar",
    "vedarais": "vedar",
    "vedaran": "vedar",
    "vedaras": "vedar",
    "vedare": "vedar",
    "vedareis": "vedar",
    "vedaremos": "vedar",
    "vedaren": "vedar",
    "vedares": "vedar",
    "vedaron": "vedar",
    "vedará": "vedar",
    "vedarán": "vedar",
    "vedarás": "vedar",
    "vedaré": "vedar",
    "vedaréis": "vedar",
    "vedaría": "vedar",
    "vedaríais": "vedar",
    "vedaríamos": "vedar",
    "vedarían": "vedar",
    "vedarías": "vedar",
    "vedas": "vedar",
    "vedase": "vedar",
    "vedaseis": "vedar",
    "vedasen": "vedar",
    "vedases": "vedar",
    "vedaste": "vedar",
    "vedasteis": "vedar",
    "vede": "vedar",
    "vedegambres": "vedegambre",
    "vedejas": "vedeja",
    "vedemos": "vedar",
    "veden": "vedar",
    "vedes": "vedar",
    "vedettes": "vedette",
    "vedijas": "vedija",
    "vedijera": "vedijero",
    "vedijeras": "vedijero",
    "vedijeros": "vedijero",
    "vedijosa": "vedijoso",
    "vedijosas": "vedijoso",
    "vedijosos": "vedijoso",
    "vedijuda": "vedijudo",
    "vedijudas": "vedijudo",
    "vedijudos": "vedijudo",
    "vedismos": "vedismo",
    "vedo": "vedar",
    "vedoides": "vedoide",
    "veduños": "veduño",
    "vedábamos": "vedar",
    "vedáis": "vedar",
    "vedáramos": "vedar",
    "vedáremos": "vedar",
    "vedásemos": "vedar",
    "vedé": "vedar",
    "vedéis": "vedar",
    "vedó": "vedar",
    "veedora": "veedor",
    "veedoras": "veedor",
    "veedores": "veedor",
    "veedurías": "veeduría",
    "vegas": "vega",
    "vegeta": "vegetar",
    "vegetaba": "vegetar",
    "vegetabais": "vegetar",
    "vegetaban": "vegetar",
    "vegetabas": "vegetar",
    "vegetaciones": "vegetación",
    "vegetad": "vegetar",
    "vegetada": "vegetar",
    "vegetadas": "vegetar",
    "vegetado": "vegetar",
    "vegetados": "vegetar",
    "vegetales": "vegetal",
    "vegetamos": "vegetar",
    "vegetan": "vegetar",
    "vegetando": "vegetar",
    "vegetara": "vegetar",
    "vegetarais": "vegetar",
    "vegetaran": "vegetar",
    "vegetaras": "vegetar",
    "vegetare": "vegetar",
    "vegetareis": "vegetar",
    "vegetaremos": "vegetar",
    "vegetaren": "vegetar",
    "vegetares": "vegetar",
    "vegetariana": "vegetariano",
    "vegetarianas": "vegetariano",
    "vegetarianismos": "vegetarianismo",
    "vegetarianos": "vegetariano",
    "vegetaron": "vegetar",
    "vegetará": "vegetar",
    "vegetarán": "vegetar",
    "vegetarás": "vegetar",
    "vegetaré": "vegetar",
    "vegetaréis": "vegetar",
    "vegetaría": "vegetar",
    "vegetaríais": "vegetar",
    "vegetaríamos": "vegetar",
    "vegetarían": "vegetar",
    "vegetarías": "vegetar",
    "vegetas": "vegetar",
    "vegetase": "vegetar",
    "vegetaseis": "vegetar",
    "vegetasen": "vegetar",
    "vegetases": "vegetar",
    "vegetaste": "vegetar",
    "vegetasteis": "vegetar",
    "vegetativa": "vegetativo",
    "vegetativas": "vegetativo",
    "vegetativos": "vegetativo",
    "vegete": "vegetar",
    "vegetemos": "vegetar",
    "vegeten": "vegetar",
    "vegetes": "vegetar",
    "vegeto": "vegetar",
    "vegetábamos": "vegetar",
    "vegetáis": "vegetar",
    "vegetáramos": "vegetar",
    "vegetáremos": "vegetar",
    "vegetásemos": "vegetar",
    "vegeté": "vegetar",
    "vegetéis": "vegetar",
    "vegetó": "vegetar",
    "veguera": "veguero",
    "vegueras": "veguero",
    "vegueres": "veguer",
    "vegueros": "veguero",
    "veguerías": "veguería",
    "vegueríos": "veguerío",
    "vehemencias": "vehemencia",
    "vehementes": "vehemente",
    "vehicula": "vehicular",
    "vehiculaba": "vehicular",
    "vehiculabais": "vehicular",
    "vehiculaban": "vehicular",
    "vehiculabas": "vehicular",
    "vehiculad": "vehicular",
    "vehiculada": "vehicular",
    "vehiculadas": "vehicular",
    "vehiculado": "vehicular",
    "vehiculados": "vehicular",
    "vehiculamos": "vehicular",
    "vehiculan": "vehicular",
    "vehiculando": "vehicular",
    "vehiculara": "vehicular",
    "vehicularais": "vehicular",
    "vehicularan": "vehicular",
    "vehicularas": "vehicular",
    "vehiculare": "vehicular",
    "vehiculareis": "vehicular",
    "vehicularemos": "vehicular",
    "vehicularen": "vehicular",
    "vehiculares": "vehicular",
    "vehicularon": "vehicular",
    "vehiculará": "vehicular",
    "vehicularán": "vehicular",
    "vehicularás": "vehicular",
    "vehicularé": "vehicular",
    "vehicularéis": "vehicular",
    "vehicularía": "vehicular",
    "vehicularíais": "vehicular",
    "vehicularíamos": "vehicular",
    "vehicularían": "vehicular",
    "vehicularías": "vehicular",
    "vehiculas": "vehicular",
    "vehiculase": "vehicular",
    "vehiculaseis": "vehicular",
    "vehiculasen": "vehicular",
    "vehiculases": "vehicular",
    "vehiculaste": "vehicular",
    "vehiculasteis": "vehicular",
    "vehicule": "vehicular",
    "vehiculemos": "vehicular",
    "vehiculen": "vehicular",
    "vehicules": "vehicular",
    "vehiculice": "vehiculizar",
    "vehiculicemos": "vehiculizar",
    "vehiculicen": "vehiculizar",
    "vehiculices": "vehiculizar",
    "vehiculicé": "vehiculizar",
    "vehiculicéis": "vehiculizar",
    "vehiculiza": "vehiculizar",
    "vehiculizaba": "vehiculizar",
    "vehiculizabais": "vehiculizar",
    "vehiculizaban": "vehiculizar",
    "vehiculizabas": "vehiculizar",
    "vehiculizad": "vehiculizar",
    "vehiculizada": "vehiculizar",
    "vehiculizadas": "vehiculizar",
    "vehiculizado": "vehiculizar",
    "vehiculizados": "vehiculizar",
    "vehiculizamos": "vehiculizar",
    "vehiculizan": "vehiculizar",
    "vehiculizando": "vehiculizar",
    "vehiculizara": "vehiculizar",
    "vehiculizarais": "vehiculizar",
    "vehiculizaran": "vehiculizar",
    "vehiculizaras": "vehiculizar",
    "vehiculizare": "vehiculizar",
    "vehiculizareis": "vehiculizar",
    "vehiculizaremos": "vehiculizar",
    "vehiculizaren": "vehiculizar",
    "vehiculizares": "vehiculizar",
    "vehiculizaron": "vehiculizar",
    "vehiculizará": "vehiculizar",
    "vehiculizarán": "vehiculizar",
    "vehiculizarás": "vehiculizar",
    "vehiculizaré": "vehiculizar",
    "vehiculizaréis": "vehiculizar",
    "vehiculizaría": "vehiculizar",
    "vehiculizaríais": "vehiculizar",
    "vehiculizaríamos": "vehiculizar",
    "vehiculizarían": "vehiculizar",
    "vehiculizarías": "vehiculizar",
    "vehiculizas": "vehiculizar",
    "vehiculizase": "vehiculizar",
    "vehiculizaseis": "vehiculizar",
    "vehiculizasen": "vehiculizar",
    "vehiculizases": "vehiculizar",
    "vehiculizaste": "vehiculizar",
    "vehiculizasteis": "vehiculizar",
    "vehiculizo": "vehiculizar",
    "vehiculizábamos": "vehiculizar",
    "vehiculizáis": "vehiculizar",
    "vehiculizáramos": "vehiculizar",
    "vehiculizáremos": "vehiculizar",
    "vehiculizásemos": "vehiculizar",
    "vehiculizó": "vehiculizar",
    "vehiculo": "vehicular",
    "vehiculábamos": "vehicular",
    "vehiculáis": "vehicular",
    "vehiculáramos": "vehicular",
    "vehiculáremos": "vehicular",
    "vehiculásemos": "vehicular",
    "vehiculé": "vehicular",
    "vehiculéis": "vehicular",
    "vehiculó": "vehicular",
    "vehículos": "vehículo",
    "veintava": "veintavo",
    "veintavas": "veintavo",
    "veintavo": "20",
    "veintavos": "veintavo",
    "veinteava": "veinteavo",
    "veinteavas": "veinteavo",
    "veinteavo": "20",
    "veinteavos": "veinteavo",
    "veintenares": "veintenar",
    "veintenaria": "veintenario",
    "veintenarias": "veintenario",
    "veintenarios": "veintenario",
    "veintenas": "veintena",
    "veinteñales": "veinteñal",
    "veinticincoava": "veinticincoavo",
    "veinticincoavas": "veinticincoavo",
    "veinticincoavo": "25",
    "veinticincoavos": "veinticincoavo",
    "veinticuatroava": "veinticuatroavo",
    "veinticuatroavas": "veinticuatroavo",
    "veinticuatroavo": "24",
    "veinticuatroavos": "veinticuatroavo",
    "veintidosava": "veintidosavo",
    "veintidosavas": "veintidosavo",
    "veintidosavo": "22",
    "veintidosavos": "veintidosavo",
    "veintinueveava": "veintinueveavo",
    "veintinueveavas": "veintinueveavo",
    "veintinueveavo": "29",
    "veintinueveavos": "veintinueveavo",
    "veintiochava": "veintiochoavo",
    "veintiochavas": "veintiochoavo",
    "veintiochavo": "veintiochoavo",
    "veintiochavos": "veintiochoavo",
    "veintiochoava": "veintiochoavo",
    "veintiochoavas": "veintiochoavo",
    "veintiochoavo": "28",
    "veintiochoavos": "veintiochoavo",
    "veintiseisava": "veintiseisavo",
    "veintiseisavas": "veintiseisavo",
    "veintiseisavo": "26",
    "veintiseisavos": "veintiseisavo",
    "veintisieteava": "veintisieteavo",
    "veintisieteavas": "veintisieteavo",
    "veintisieteavo": "27",
    "veintisieteavos": "veintisieteavo",
    "veintitantas": "veintitantos",
    "veintitodas": "veintitodos",
    "veintitresava": "veintitresavo",
    "veintitresavas": "veintitresavo",
    "veintitresavo": "23",
    "veintitresavos": "veintitresavo",
    "veintiunava": "veintiunavo",
    "veintiunavas": "veintiunavo",
    "veintiunavo": "21",
    "veintiunavos": "veintiunavo",
    "veintésima": "20",
    "veintésimas": "20",
    "veintésimo": "20",
    "veintésimos": "20",
    "veis": "ver",
    "veja": "vejar",
    "vejaba": "vejar",
    "vejabais": "vejar",
    "vejaban": "vejar",
    "vejabas": "vejar",
    "vejaciones": "vejación",
    "vejaciónes": "vejación",
    "vejad": "vejar",
    "vejada": "vejar",
    "vejadas": "vejar",
    "vejado": "vejar",
    "vejadora": "vejador",
    "vejadoras": "vejador",
    "vejadores": "vejador",
    "vejados": "vejar",
    "vejamos": "vejar",
    "vejan": "vejar",
    "vejando": "vejar",
    "vejara": "vejar",
    "vejarais": "vejar",
    "vejaran": "vejar",
    "vejaras": "vejar",
    "vejare": "vejar",
    "vejareis": "vejar",
    "vejaremos": "vejar",
    "vejaren": "vejar",
    "vejares": "vejar",
    "vejaron": "vejar",
    "vejará": "vejar",
    "vejarán": "vejar",
    "vejarás": "vejar",
    "vejaré": "vejar",
    "vejaréis": "vejar",
    "vejaría": "vejar",
    "vejaríais": "vejar",
    "vejaríamos": "vejar",
    "vejarían": "vejar",
    "vejarías": "vejar",
    "vejas": "vejar",
    "vejase": "vejar",
    "vejaseis": "vejar",
    "vejasen": "vejar",
    "vejases": "vejar",
    "vejaste": "vejar",
    "vejasteis": "vejar",
    "vejatoria": "vejatorio",
    "vejatorias": "vejatorio",
    "vejatorios": "vejatorio",
    "veje": "vejar",
    "vejeces": "vejez",
    "vejemos": "vejar",
    "vejen": "vejar",
    "vejes": "vejar",
    "vejigas": "vejiga",
    "vejigatoria": "vejigatorio",
    "vejigatorias": "vejigatorio",
    "vejigatorios": "vejigatorio",
    "vejigazos": "vejigazo",
    "vejigosa": "vejigoso",
    "vejigosas": "vejigoso",
    "vejigosos": "vejigoso",
    "vejiguillas": "vejiguilla",
    "vejo": "vejar",
    "vejota": "vejote",
    "vejotas": "vejote",
    "vejotes": "vejote",
    "vejábamos": "vejar",
    "vejáis": "vejar",
    "vejámenes": "vejamen",
    "vejáramos": "vejar",
    "vejáremos": "vejar",
    "vejásemos": "vejar",
    "vejé": "vejar",
    "vejéis": "vejar",
    "vejó": "vejar",
    "vela": "velar",
    "velaba": "velar",
    "velabais": "velar",
    "velaban": "velar",
    "velabas": "velar",
    "velachos": "velacho",
    "velaciones": "velación",
    "velad": "velar",
    "velada": "velar",
    "veladas": "velar",
    "velado": "velar",
    "veladora": "velador",
    "veladoras": "veladora",
    "veladores": "velador",
    "velados": "velar",
    "veladuras": "veladura",
    "velajes": "velaje",
    "velamos": "velar",
    "velan": "velar",
    "velando": "velar",
    "velara": "velar",
    "velarais": "velar",
    "velaran": "velar",
    "velaras": "velar",
    "velare": "velar",
    "velareis": "velar",
    "velaremos": "velar",
    "velaren": "velar",
    "velares": "velar",
    "velarizaciones": "velarización",
    "velaron": "velar",
    "velará": "velar",
    "velarán": "velar",
    "velarás": "velar",
    "velaré": "velar",
    "velaréis": "velar",
    "velaría": "velar",
    "velaríais": "velar",
    "velaríamos": "velar",
    "velarían": "velar",
    "velarías": "velar",
    "velas": "velar",
    "velase": "velar",
    "velaseis": "velar",
    "velasen": "velar",
    "velases": "velar",
    "velaste": "velar",
    "velasteis": "velar",
    "velatorios": "velatorio",
    "velazqueña": "velazqueño",
    "velazqueñas": "velazqueño",
    "velazqueños": "velazqueño",
    "velcros": "velcro",
    "vele": "velar",
    "veleidades": "veleidad",
    "veleidosa": "veleidoso",
    "veleidosas": "veleidoso",
    "veleidosos": "veleidoso",
    "velemos": "velar",
    "velen": "velar",
    "velera": "velero",
    "veleras": "velero",
    "veleros": "velero",
    "velerías": "velería",
    "veles": "velar",
    "veletas": "veleta",
    "veletes": "velete",
    "velica": "velicar",
    "velicaba": "velicar",
    "velicabais": "velicar",
    "velicaban": "velicar",
    "velicabas": "velicar",
    "velicaciones": "velicación",
    "velicad": "velicar",
    "velicada": "velicar",
    "velicadas": "velicar",
    "velicado": "velicar",
    "velicados": "velicar",
    "velicamos": "velicar",
    "velican": "velicar",
    "velicando": "velicar",
    "velicara": "velicar",
    "velicarais": "velicar",
    "velicaran": "velicar",
    "velicaras": "velicar",
    "velicare": "velicar",
    "velicareis": "velicar",
    "velicaremos": "velicar",
    "velicaren": "velicar",
    "velicares": "velicar",
    "velicaron": "velicar",
    "velicará": "velicar",
    "velicarán": "velicar",
    "velicarás": "velicar",
    "velicaré": "velicar",
    "velicaréis": "velicar",
    "velicaría": "velicar",
    "velicaríais": "velicar",
    "velicaríamos": "velicar",
    "velicarían": "velicar",
    "velicarías": "velicar",
    "velicas": "velicar",
    "velicase": "velicar",
    "velicaseis": "velicar",
    "velicasen": "velicar",
    "velicases": "velicar",
    "velicaste": "velicar",
    "velicasteis": "velicar",
    "velico": "velicar",
    "velicábamos": "velicar",
    "velicáis": "velicar",
    "velicáramos": "velicar",
    "velicáremos": "velicar",
    "velicásemos": "velicar",
    "velicó": "velicar",
    "velicómenes": "velicomen",
    "velillos": "velillo",
    "velintonias": "velintonia",
    "velique": "velicar",
    "veliquemos": "velicar",
    "veliquen": "velicar",
    "veliques": "velicar",
    "veliqué": "velicar",
    "veliquéis": "velicar",
    "vellida": "vellido",
    "vellidas": "vellido",
    "vellidos": "vellido",
    "vellocinos": "vellocino",
    "vellonera": "vellonero",
    "velloneras": "vellonero",
    "velloneros": "vellonero",
    "vellones": "vellón",
    "vellorines": "vellorín",
    "velloríes": "vellorí",
    "vellorís": "vellorí",
    "vellos": "vello",
    "vellosa": "velloso",
    "vellosas": "velloso",
    "vellosidades": "vellosidad",
    "vellosillas": "vellosilla",
    "vellosos": "velloso",
    "velluda": "velludo",
    "velludas": "velludo",
    "velludillos": "velludillo",
    "velludos": "velludo",
    "velmeces": "velmez",
    "velo": "velar",
    "veloces": "veloz",
    "velocidades": "velocidad",
    "velocipedismos": "velocipedismo",
    "velocipedistas": "velocipedista",
    "velocipédica": "velocipédico",
    "velocipédicas": "velocipédico",
    "velocipédicos": "velocipédico",
    "velocistas": "velocista",
    "velocímetros": "velocímetro",
    "velocípedos": "velocípedo",
    "velofaríngea": "velofaríngeo",
    "velofaríngeas": "velofaríngeo",
    "velofaríngeos": "velofaríngeo",
    "velomotores": "velomotor",
    "velonera": "velonero",
    "veloneras": "velonero",
    "veloneros": "velonero",
    "velones": "velón",
    "velorios": "velorio",
    "velos": "velo",
    "velozes": "veloz",
    "velvetones": "velvetón",
    "velábamos": "velar",
    "veláis": "velar",
    "velámenes": "velamen",
    "veláramos": "velar",
    "veláremos": "velar",
    "velásemos": "velar",
    "velé": "velar",
    "veléis": "velar",
    "veló": "velar",
    "velódromos": "velódromo",
    "vemos": "ver",
    "ven": "ver",
    "venablos": "venablo",
    "venadas": "venada",
    "venaderos": "venadero",
    "venados": "venado",
    "venajes": "venaje",
    "venales": "venal",
    "venalidades": "venalidad",
    "venas": "vena",
    "venatoria": "venatorio",
    "venatorias": "venatorio",
    "venatorios": "venatorio",
    "vence": "vencer",
    "venced": "vencer",
    "vencedera": "vencedero",
    "vencederas": "vencedero",
    "vencederos": "vencedero",
    "vencedora": "vencedor",
    "vencedoras": "vencedor",
    "vencedores": "vencedor",
    "vencejos": "vencejo",
    "vencemos": "vencer",
    "vencen": "vencer",
    "venceremos": "vencer",
    "vencerá": "vencer",
    "vencerán": "vencer",
    "vencerás": "vencer",
    "venceré": "vencer",
    "venceréis": "vencer",
    "vencería": "vencer",
    "venceríais": "vencer",
    "venceríamos": "vencer",
    "vencerían": "vencer",
    "vencerías": "vencer",
    "vences": "vencer",
    "vencetósigos": "vencetósigo",
    "vencibles": "vencible",
    "vencida": "vencer",
    "vencidas": "vencida",
    "vencido": "vencer",
    "vencidos": "vencer",
    "venciendo": "vencer",
    "venciera": "vencer",
    "vencierais": "vencer",
    "vencieran": "vencer",
    "vencieras": "vencer",
    "venciere": "vencer",
    "venciereis": "vencer",
    "vencieren": "vencer",
    "vencieres": "vencer",
    "vencieron": "vencer",
    "venciese": "vencer",
    "vencieseis": "vencer",
    "venciesen": "vencer",
    "vencieses": "vencer",
    "vencimientos": "vencimiento",
    "vencimos": "vencer",
    "venciste": "vencer",
    "vencisteis": "vencer",
    "venciéramos": "vencer",
    "venciéremos": "vencer",
    "venciésemos": "vencer",
    "venció": "vencer",
    "vencéis": "vencer",
    "vencí": "vencer",
    "vencía": "vencer",
    "vencíais": "vencer",
    "vencíamos": "vencer",
    "vencían": "vencer",
    "vencías": "vencer",
    "venda": "vender",
    "vendaba": "vendar",
    "vendabais": "vendar",
    "vendaban": "vendar",
    "vendabas": "vendar",
    "vendad": "vendar",
    "vendada": "vendar",
    "vendadas": "vendar",
    "vendado": "vendar",
    "vendados": "vendar",
    "vendajes": "vendaje",
    "vendamos": "vender",
    "vendan": "vender",
    "vendando": "vendar",
    "vendara": "vendar",
    "vendarais": "vendar",
    "vendaran": "vendar",
    "vendaras": "vendar",
    "vendare": "vendar",
    "vendareis": "vendar",
    "vendaremos": "vendar",
    "vendaren": "vendar",
    "vendares": "vendar",
    "vendaron": "vendar",
    "vendará": "vendar",
    "vendarán": "vendar",
    "vendarás": "vendar",
    "vendaré": "vendar",
    "vendaréis": "vendar",
    "vendaría": "vendar",
    "vendaríais": "vendar",
    "vendaríamos": "vendar",
    "vendarían": "vendar",
    "vendarías": "vendar",
    "vendas": "vender",
    "vendase": "vendar",
    "vendaseis": "vendar",
    "vendasen": "vendar",
    "vendases": "vendar",
    "vendaste": "vendar",
    "vendasteis": "vendar",
    "vendavales": "vendaval",
    "vende": "vender",
    "vended": "vender",
    "vendedora": "vendedor",
    "vendedoras": "vendedor",
    "vendedores": "vendedor",
    "vendejas": "vendeja",
    "vendemos": "vender",
    "venden": "vender",
    "venderemos": "vender",
    "venderá": "vender",
    "venderán": "vender",
    "venderás": "vender",
    "venderé": "vender",
    "venderéis": "vender",
    "vendería": "vender",
    "venderíais": "vender",
    "venderíamos": "vender",
    "venderían": "vender",
    "venderías": "vender",
    "vendes": "vender",
    "vendettas": "vendetta",
    "vendibles": "vendible",
    "vendida": "vender",
    "vendidas": "vender",
    "vendido": "vender",
    "vendidos": "vender",
    "vendiendo": "vender",
    "vendiera": "vender",
    "vendierais": "vender",
    "vendieran": "vender",
    "vendieras": "vender",
    "vendiere": "vender",
    "vendiereis": "vender",
    "vendieren": "vender",
    "vendieres": "vender",
    "vendieron": "vender",
    "vendiese": "vender",
    "vendieseis": "vender",
    "vendiesen": "vender",
    "vendieses": "vender",
    "vendimia": "vendimiar",
    "vendimiaba": "vendimiar",
    "vendimiabais": "vendimiar",
    "vendimiaban": "vendimiar",
    "vendimiabas": "vendimiar",
    "vendimiad": "vendimiar",
    "vendimiada": "vendimiar",
    "vendimiadas": "vendimiar",
    "vendimiado": "vendimiar",
    "vendimiadora": "vendimiador",
    "vendimiadoras": "vendimiador",
    "vendimiadores": "vendimiador",
    "vendimiados": "vendimiar",
    "vendimiamos": "vendimiar",
    "vendimian": "vendimiar",
    "vendimiando": "vendimiar",
    "vendimiara": "vendimiar",
    "vendimiarais": "vendimiar",
    "vendimiaran": "vendimiar",
    "vendimiaras": "vendimiar",
    "vendimiare": "vendimiar",
    "vendimiareis": "vendimiar",
    "vendimiaremos": "vendimiar",
    "vendimiaren": "vendimiar",
    "vendimiares": "vendimiar",
    "vendimiarios": "vendimiario",
    "vendimiaron": "vendimiar",
    "vendimiará": "vendimiar",
    "vendimiarán": "vendimiar",
    "vendimiarás": "vendimiar",
    "vendimiaré": "vendimiar",
    "vendimiaréis": "vendimiar",
    "vendimiaría": "vendimiar",
    "vendimiaríais": "vendimiar",
    "vendimiaríamos": "vendimiar",
    "vendimiarían": "vendimiar",
    "vendimiarías": "vendimiar",
    "vendimias": "vendimiar",
    "vendimiase": "vendimiar",
    "vendimiaseis": "vendimiar",
    "vendimiasen": "vendimiar",
    "vendimiases": "vendimiar",
    "vendimiaste": "vendimiar",
    "vendimiasteis": "vendimiar",
    "vendimie": "vendimiar",
    "vendimiemos": "vendimiar",
    "vendimien": "vendimiar",
    "vendimies": "vendimiar",
    "vendimio": "vendimiar",
    "vendimiábamos": "vendimiar",
    "vendimiáis": "vendimiar",
    "vendimiáramos": "vendimiar",
    "vendimiáremos": "vendimiar",
    "vendimiásemos": "vendimiar",
    "vendimié": "vendimiar",
    "vendimiéis": "vendimiar",
    "vendimió": "vendimiar",
    "vendimos": "vender",
    "vendis": "vendí",
    "vendiste": "vender",
    "vendisteis": "vender",
    "vendiéramos": "vender",
    "vendiéremos": "vender",
    "vendiésemos": "vender",
    "vendió": "vender",
    "vendo": "vender",
    "vendos": "vendo",
    "vendremos": "venir",
    "vendrá": "venir",
    "vendrán": "venir",
    "vendrás": "venir",
    "vendré": "venir",
    "vendréis": "venir",
    "vendría": "venir",
    "vendríais": "venir",
    "vendríamos": "venir",
    "vendrían": "venir",
    "vendrías": "venir",
    "vendábamos": "vendar",
    "vendáis": "vender",
    "vendáramos": "vendar",
    "vendáremos": "vendar",
    "vendásemos": "vendar",
    "vendé": "vendar",
    "vendéis": "vender",
    "vendí": "vender",
    "vendía": "vender",
    "vendíais": "vender",
    "vendíamos": "vender",
    "vendían": "vender",
    "vendías": "vender",
    "vendíes": "vendí",
    "vendís": "vendí",
    "vendó": "vendar",
    "veneciana": "veneciano",
    "venecianas": "veneciano",
    "venecianos": "veneciano",
    "venenos": "veneno",
    "venenosa": "venenoso",
    "venenosas": "venenoso",
    "venenosidades": "venenosidad",
    "venenosos": "venenoso",
    "venenífera": "venenífero",
    "veneníferas": "venenífero",
    "veneníferos": "venenífero",
    "venera": "venerar",
    "veneraba": "venerar",
    "venerabais": "venerar",
    "veneraban": "venerar",
    "venerabas": "venerar",
    "venerabilidades": "venerabilidad",
    "venerables": "venerable",
    "veneraciones": "veneración",
    "venerad": "venerar",
    "venerada": "venerar",
    "veneradas": "venerar",
    "venerado": "venerar",
    "veneradora": "venerador",
    "veneradoras": "venerador",
    "veneradores": "venerador",
    "venerados": "venerar",
    "veneramos": "venerar",
    "veneran": "venerar",
    "veneranda": "venerando",
    "venerandas": "venerando",
    "venerando": "venerar",
    "venerandos": "venerando",
    "venerara": "venerar",
    "venerarais": "venerar",
    "veneraran": "venerar",
    "veneraras": "venerar",
    "venerare": "venerar",
    "venerareis": "venerar",
    "veneraremos": "venerar",
    "veneraren": "venerar",
    "venerares": "venerar",
    "veneraron": "venerar",
    "venerará": "venerar",
    "venerarán": "venerar",
    "venerarás": "venerar",
    "veneraré": "venerar",
    "veneraréis": "venerar",
    "veneraría": "venerar",
    "veneraríais": "venerar",
    "veneraríamos": "venerar",
    "venerarían": "venerar",
    "venerarías": "venerar",
    "veneras": "venerar",
    "venerase": "venerar",
    "veneraseis": "venerar",
    "venerasen": "venerar",
    "venerases": "venerar",
    "veneraste": "venerar",
    "venerasteis": "venerar",
    "venere": "venerar",
    "veneremos": "venerar",
    "veneren": "venerar",
    "venereologías": "venereología",
    "veneres": "venerar",
    "venereóloga": "venereólogo",
    "venereólogas": "venereólogo",
    "venereólogos": "venereólogo",
    "venero": "venerar",
    "veneros": "venero",
    "venerábamos": "venerar",
    "veneráis": "venerar",
    "veneráramos": "venerar",
    "veneráremos": "venerar",
    "venerásemos": "venerar",
    "veneré": "venerar",
    "veneréis": "venerar",
    "veneró": "venerar",
    "venezolana": "venezolano",
    "venezolanas": "venezolano",
    "venezolanismos": "venezolanismo",
    "venezolanos": "venezolano",
    "venga": "venir",
    "vengaba": "vengar",
    "vengabais": "vengar",
    "vengaban": "vengar",
    "vengabas": "vengar",
    "vengables": "vengable",
    "vengad": "vengar",
    "vengada": "vengar",
    "vengadas": "vengar",
    "vengado": "vengar",
    "vengadora": "vengador",
    "vengadoras": "vengador",
    "vengadores": "vengador",
    "vengados": "vengar",
    "vengamos": "venir",
    "vengan": "venir",
    "vengando": "vengar",
    "venganzas": "venganza",
    "vengara": "vengar",
    "vengarais": "vengar",
    "vengaran": "vengar",
    "vengaras": "vengar",
    "vengare": "vengar",
    "vengareis": "vengar",
    "vengaremos": "vengar",
    "vengaren": "vengar",
    "vengares": "vengar",
    "vengaron": "vengar",
    "vengará": "vengar",
    "vengarán": "vengar",
    "vengarás": "vengar",
    "vengaré": "vengar",
    "vengaréis": "vengar",
    "vengaría": "vengar",
    "vengaríais": "vengar",
    "vengaríamos": "vengar",
    "vengarían": "vengar",
    "vengarías": "vengar",
    "vengas": "venir",
    "vengase": "vengar",
    "vengaseis": "vengar",
    "vengasen": "vengar",
    "vengases": "vengar",
    "vengaste": "vengar",
    "vengasteis": "vengar",
    "vengativa": "vengativo",
    "vengativas": "vengativo",
    "vengativos": "vengativo",
    "vengo": "venir",
    "vengue": "vengar",
    "venguemos": "vengar",
    "venguen": "vengar",
    "vengues": "vengar",
    "vengué": "vengar",
    "venguéis": "vengar",
    "vengábamos": "vengar",
    "vengáis": "venir",
    "vengáramos": "vengar",
    "vengáremos": "vengar",
    "vengásemos": "vengar",
    "vengó": "vengar",
    "veniales": "venial",
    "venialidades": "venialidad",
    "venias": "venia",
    "venid": "venir",
    "venida": "venir",
    "venidas": "venir",
    "venidera": "venidero",
    "venideras": "venidero",
    "venideros": "venidero",
    "venido": "venir",
    "venidos": "venir",
    "venimos": "venir",
    "venosa": "venoso",
    "venosas": "venoso",
    "venosidades": "venosidad",
    "venosos": "venoso",
    "ventadas": "ventada",
    "ventajas": "ventaja",
    "ventajistas": "ventajista",
    "ventajosa": "ventajoso",
    "ventajosas": "ventajoso",
    "ventajosos": "ventajoso",
    "ventallas": "ventalla",
    "ventalles": "ventalle",
    "ventanajes": "ventanaje",
    "ventanales": "ventanal",
    "ventanas": "ventana",
    "ventanazos": "ventanazo",
    "ventanea": "ventanear",
    "ventaneaba": "ventanear",
    "ventaneabais": "ventanear",
    "ventaneaban": "ventanear",
    "ventaneabas": "ventanear",
    "ventanead": "ventanear",
    "ventaneada": "ventanear",
    "ventaneadas": "ventanear",
    "ventaneado": "ventanear",
    "ventaneados": "ventanear",
    "ventaneamos": "ventanear",
    "ventanean": "ventanear",
    "ventaneando": "ventanear",
    "ventaneara": "ventanear",
    "ventanearais": "ventanear",
    "ventanearan": "ventanear",
    "ventanearas": "ventanear",
    "ventaneare": "ventanear",
    "ventaneareis": "ventanear",
    "ventanearemos": "ventanear",
    "ventanearen": "ventanear",
    "ventaneares": "ventanear",
    "ventanearon": "ventanear",
    "ventaneará": "ventanear",
    "ventanearán": "ventanear",
    "ventanearás": "ventanear",
    "ventanearé": "ventanear",
    "ventanearéis": "ventanear",
    "ventanearía": "ventanear",
    "ventanearíais": "ventanear",
    "ventanearíamos": "ventanear",
    "ventanearían": "ventanear",
    "ventanearías": "ventanear",
    "ventaneas": "ventanear",
    "ventanease": "ventanear",
    "ventaneaseis": "ventanear",
    "ventaneasen": "ventanear",
    "ventaneases": "ventanear",
    "ventaneaste": "ventanear",
    "ventaneasteis": "ventanear",
    "ventanee": "ventanear",
    "ventaneemos": "ventanear",
    "ventaneen": "ventanear",
    "ventanees": "ventanear",
    "ventaneo": "ventanear",
    "ventaneos": "ventaneo",
    "ventaneábamos": "ventanear",
    "ventaneáis": "ventanear",
    "ventaneáramos": "ventanear",
    "ventaneáremos": "ventanear",
    "ventaneásemos": "ventanear",
    "ventaneé": "ventanear",
    "ventaneéis": "ventanear",
    "ventaneó": "ventanear",
    "ventanicos": "ventanico",
    "ventanillas": "ventanilla",
    "ventanillos": "ventanillo",
    "ventanitas": "ventanita",
    "ventanos": "ventano",
    "ventarrones": "ventarrón",
    "ventas": "venta",
    "ventea": "ventear",
    "venteaba": "ventear",
    "venteabais": "ventear",
    "venteaban": "ventear",
    "venteabas": "ventear",
    "ventead": "ventear",
    "venteada": "ventear",
    "venteadas": "ventear",
    "venteado": "ventear",
    "venteados": "ventear",
    "venteaduras": "venteadura",
    "venteamos": "ventear",
    "ventean": "ventear",
    "venteando": "ventear",
    "venteara": "ventear",
    "ventearais": "ventear",
    "ventearan": "ventear",
    "ventearas": "ventear",
    "venteare": "ventear",
    "venteareis": "ventear",
    "ventearemos": "ventear",
    "ventearen": "ventear",
    "venteares": "ventear",
    "ventearon": "ventear",
    "venteará": "ventear",
    "ventearán": "ventear",
    "ventearás": "ventear",
    "ventearé": "ventear",
    "ventearéis": "ventear",
    "ventearía": "ventear",
    "ventearíais": "ventear",
    "ventearíamos": "ventear",
    "ventearían": "ventear",
    "ventearías": "ventear",
    "venteas": "ventear",
    "ventease": "ventear",
    "venteaseis": "ventear",
    "venteasen": "ventear",
    "venteases": "ventear",
    "venteaste": "ventear",
    "venteasteis": "ventear",
    "ventee": "ventear",
    "venteemos": "ventear",
    "venteen": "ventear",
    "ventees": "ventear",
    "venteo": "ventear",
    "ventera": "ventero",
    "venteras": "ventero",
    "venteriles": "venteril",
    "venteros": "ventero",
    "venteábamos": "ventear",
    "venteáis": "ventear",
    "venteáramos": "ventear",
    "venteáremos": "ventear",
    "venteásemos": "ventear",
    "venteé": "ventear",
    "venteéis": "ventear",
    "venteó": "ventear",
    "ventila": "ventilar",
    "ventilaba": "ventilar",
    "ventilabais": "ventilar",
    "ventilaban": "ventilar",
    "ventilabas": "ventilar",
    "ventilaciones": "ventilación",
    "ventilad": "ventilar",
    "ventilada": "ventilar",
    "ventiladas": "ventilar",
    "ventilado": "ventilar",
    "ventiladora": "ventilador",
    "ventiladoras": "ventilador",
    "ventiladores": "ventilador",
    "ventilados": "ventilar",
    "ventilamos": "ventilar",
    "ventilan": "ventilar",
    "ventilando": "ventilar",
    "ventilara": "ventilar",
    "ventilarais": "ventilar",
    "ventilaran": "ventilar",
    "ventilaras": "ventilar",
    "ventilare": "ventilar",
    "ventilareis": "ventilar",
    "ventilaremos": "ventilar",
    "ventilaren": "ventilar",
    "ventilares": "ventilar",
    "ventilaron": "ventilar",
    "ventilará": "ventilar",
    "ventilarán": "ventilar",
    "ventilarás": "ventilar",
    "ventilaré": "ventilar",
    "ventilaréis": "ventilar",
    "ventilaría": "ventilar",
    "ventilaríais": "ventilar",
    "ventilaríamos": "ventilar",
    "ventilarían": "ventilar",
    "ventilarías": "ventilar",
    "ventilas": "ventilar",
    "ventilase": "ventilar",
    "ventilaseis": "ventilar",
    "ventilasen": "ventilar",
    "ventilases": "ventilar",
    "ventilaste": "ventilar",
    "ventilasteis": "ventilar",
    "ventile": "ventilar",
    "ventilemos": "ventilar",
    "ventilen": "ventilar",
    "ventiles": "ventilar",
    "ventilo": "ventilar",
    "ventilábamos": "ventilar",
    "ventiláis": "ventilar",
    "ventiláramos": "ventilar",
    "ventiláremos": "ventilar",
    "ventilásemos": "ventilar",
    "ventilé": "ventilar",
    "ventiléis": "ventilar",
    "ventiló": "ventilar",
    "ventisca": "ventiscar",
    "ventiscaba": "ventiscar",
    "ventiscabais": "ventiscar",
    "ventiscaban": "ventiscar",
    "ventiscabas": "ventiscar",
    "ventiscad": "ventiscar",
    "ventiscada": "ventiscar",
    "ventiscadas": "ventiscar",
    "ventiscado": "ventiscar",
    "ventiscados": "ventiscar",
    "ventiscamos": "ventiscar",
    "ventiscan": "ventiscar",
    "ventiscando": "ventiscar",
    "ventiscara": "ventiscar",
    "ventiscarais": "ventiscar",
    "ventiscaran": "ventiscar",
    "ventiscaras": "ventiscar",
    "ventiscare": "ventiscar",
    "ventiscareis": "ventiscar",
    "ventiscaremos": "ventiscar",
    "ventiscaren": "ventiscar",
    "ventiscares": "ventiscar",
    "ventiscaron": "ventiscar",
    "ventiscará": "ventiscar",
    "ventiscarán": "ventiscar",
    "ventiscarás": "ventiscar",
    "ventiscaré": "ventiscar",
    "ventiscaréis": "ventiscar",
    "ventiscaría": "ventiscar",
    "ventiscaríais": "ventiscar",
    "ventiscaríamos": "ventiscar",
    "ventiscarían": "ventiscar",
    "ventiscarías": "ventiscar",
    "ventiscas": "ventiscar",
    "ventiscase": "ventiscar",
    "ventiscaseis": "ventiscar",
    "ventiscasen": "ventiscar",
    "ventiscases": "ventiscar",
    "ventiscaste": "ventiscar",
    "ventiscasteis": "ventiscar",
    "ventisco": "ventiscar",
    "ventiscos": "ventisco",
    "ventiscosa": "ventiscoso",
    "ventiscosas": "ventiscoso",
    "ventiscosos": "ventiscoso",
    "ventiscábamos": "ventiscar",
    "ventiscáis": "ventiscar",
    "ventiscáramos": "ventiscar",
    "ventiscáremos": "ventiscar",
    "ventiscásemos": "ventiscar",
    "ventiscó": "ventiscar",
    "ventisque": "ventiscar",
    "ventisquea": "ventisquear",
    "ventisqueaba": "ventisquear",
    "ventisqueado": "ventisquear",
    "ventisqueando": "ventisquear",
    "ventisqueara": "ventisquear",
    "ventisqueará": "ventisquear",
    "ventisquearía": "ventisquear",
    "ventisquease": "ventisquear",
    "ventisquee": "ventisquear",
    "ventisquemos": "ventiscar",
    "ventisquen": "ventiscar",
    "ventisqueros": "ventisquero",
    "ventisques": "ventiscar",
    "ventisqueó": "ventisquear",
    "ventisqué": "ventiscar",
    "ventisquéis": "ventiscar",
    "ventolas": "ventola",
    "ventoleras": "ventolera",
    "ventolinas": "ventolina",
    "ventora": "ventor",
    "ventoras": "ventor",
    "ventores": "ventor",
    "ventorreros": "ventorrero",
    "ventorrillera": "ventorrillero",
    "ventorrilleras": "ventorrillero",
    "ventorrilleros": "ventorrillero",
    "ventorrillos": "ventorrillo",
    "ventorros": "ventorro",
    "ventosa": "ventoso",
    "ventosas": "ventoso",
    "ventosea": "ventosear",
    "ventoseaba": "ventosear",
    "ventoseabais": "ventosear",
    "ventoseaban": "ventosear",
    "ventoseabas": "ventosear",
    "ventosead": "ventosear",
    "ventoseada": "ventosear",
    "ventoseadas": "ventosear",
    "ventoseado": "ventosear",
    "ventoseados": "ventosear",
    "ventoseamos": "ventosear",
    "ventosean": "ventosear",
    "ventoseando": "ventosear",
    "ventoseara": "ventosear",
    "ventosearais": "ventosear",
    "ventosearan": "ventosear",
    "ventosearas": "ventosear",
    "ventoseare": "ventosear",
    "ventoseareis": "ventosear",
    "ventosearemos": "ventosear",
    "ventosearen": "ventosear",
    "ventoseares": "ventosear",
    "ventosearon": "ventosear",
    "ventoseará": "ventosear",
    "ventosearán": "ventosear",
    "ventosearás": "ventosear",
    "ventosearé": "ventosear",
    "ventosearéis": "ventosear",
    "ventosearía": "ventosear",
    "ventosearíais": "ventosear",
    "ventosearíamos": "ventosear",
    "ventosearían": "ventosear",
    "ventosearías": "ventosear",
    "ventoseas": "ventosear",
    "ventosease": "ventosear",
    "ventoseaseis": "ventosear",
    "ventoseasen": "ventosear",
    "ventoseases": "ventosear",
    "ventoseaste": "ventosear",
    "ventoseasteis": "ventosear",
    "ventosee": "ventosear",
    "ventoseemos": "ventosear",
    "ventoseen": "ventosear",
    "ventosees": "ventosear",
    "ventoseo": "ventosear",
    "ventoseábamos": "ventosear",
    "ventoseáis": "ventosear",
    "ventoseáramos": "ventosear",
    "ventoseáremos": "ventosear",
    "ventoseásemos": "ventosear",
    "ventoseé": "ventosear",
    "ventoseéis": "ventosear",
    "ventoseó": "ventosear",
    "ventosidades": "ventosidad",
    "ventosos": "ventoso",
    "ventrales": "ventral",
    "ventrechas": "ventrecha",
    "ventregadas": "ventregada",
    "ventreras": "ventrera",
    "ventriculares": "ventricular",
    "ventriles": "ventril",
    "ventriloquias": "ventriloquia",
    "ventriscas": "ventrisca",
    "ventrones": "ventrón",
    "ventrosa": "ventroso",
    "ventrosas": "ventroso",
    "ventrosos": "ventroso",
    "ventruda": "ventrudo",
    "ventrudas": "ventrudo",
    "ventrudos": "ventrudo",
    "ventrículos": "ventrículo",
    "ventrílocua": "ventrílocuo",
    "ventrílocuas": "ventrílocuo",
    "ventrílocuos": "ventrílocuo",
    "ventura": "venturo",
    "venturada": "venturado",
    "venturadas": "venturado",
    "venturados": "venturado",
    "venturas": "venturo",
    "venturera": "venturero",
    "ventureras": "venturero",
    "ventureros": "venturero",
    "venturinas": "venturina",
    "venturis": "venturi",
    "venturos": "venturo",
    "venturosa": "venturoso",
    "venturosas": "venturoso",
    "venturosos": "venturoso",
    "venturímetros": "venturímetro",
    "venusina": "venusino",
    "venusinas": "venusino",
    "venusinos": "venusino",
    "venusta": "venusto",
    "venustas": "venusto",
    "venusteces": "venustez",
    "venustidades": "venustidad",
    "venustos": "venusto",
    "venza": "vencer",
    "venzamos": "vencer",
    "venzan": "vencer",
    "venzas": "vencer",
    "venzo": "vencer",
    "venzáis": "vencer",
    "venática": "venático",
    "venáticas": "venático",
    "venáticos": "venático",
    "venérea": "venéreo",
    "venéreas": "venéreo",
    "venéreos": "venéreo",
    "venía": "venir",
    "veníais": "venir",
    "veníamos": "venir",
    "venían": "venir",
    "venías": "venir",
    "venís": "venir",
    "veo": "ver",
    "veraces": "veraz",
    "veracidades": "veracidad",
    "veranadas": "veranada",
    "veranaderos": "veranadero",
    "verandas": "veranda",
    "veranea": "veranear",
    "veraneaba": "veranear",
    "veraneabais": "veranear",
    "veraneaban": "veranear",
    "veraneabas": "veranear",
    "veranead": "veranear",
    "veraneada": "veranear",
    "veraneadas": "veranear",
    "veraneado": "veranear",
    "veraneados": "veranear",
    "veraneamos": "veranear",
    "veranean": "veranear",
    "veraneando": "veranear",
    "veraneantes": "veraneante",
    "veraneara": "veranear",
    "veranearais": "veranear",
    "veranearan": "veranear",
    "veranearas": "veranear",
    "veraneare": "veranear",
    "veraneareis": "veranear",
    "veranearemos": "veranear",
    "veranearen": "veranear",
    "veraneares": "veranear",
    "veranearon": "veranear",
    "veraneará": "veranear",
    "veranearán": "veranear",
    "veranearás": "veranear",
    "veranearé": "veranear",
    "veranearéis": "veranear",
    "veranearía": "veranear",
    "veranearíais": "veranear",
    "veranearíamos": "veranear",
    "veranearían": "veranear",
    "veranearías": "veranear",
    "veraneas": "veranear",
    "veranease": "veranear",
    "veraneaseis": "veranear",
    "veraneasen": "veranear",
    "veraneases": "veranear",
    "veraneaste": "veranear",
    "veraneasteis": "veranear",
    "veranee": "veranear",
    "veraneemos": "veranear",
    "veraneen": "veranear",
    "veranees": "veranear",
    "veraneo": "veranear",
    "veraneos": "veraneo",
    "veraneros": "veranero",
    "veraneábamos": "veranear",
    "veraneáis": "veranear",
    "veraneáramos": "veranear",
    "veraneáremos": "veranear",
    "veraneásemos": "veranear",
    "veraneé": "veranear",
    "veraneéis": "veranear",
    "veraneó": "veranear",
    "veraniega": "veraniego",
    "veraniegas": "veraniego",
    "veraniegos": "veraniego",
    "veranillos": "veranillo",
    "veranos": "verano",
    "veras": "vera",
    "verascopios": "verascopio",
    "veratrinas": "veratrina",
    "veratroles": "veratrol",
    "verazes": "veraz",
    "verbales": "verbal",
    "verbalice": "verbalizar",
    "verbalicemos": "verbalizar",
    "verbalicen": "verbalizar",
    "verbalices": "verbalizar",
    "verbalicé": "verbalizar",
    "verbalicéis": "verbalizar",
    "verbalismos": "verbalismo",
    "verbalistas": "verbalista",
    "verbaliza": "verbalizar",
    "verbalizaba": "verbalizar",
    "verbalizabais": "verbalizar",
    "verbalizaban": "verbalizar",
    "verbalizabas": "verbalizar",
    "verbalizaciones": "verbalización",
    "verbalizad": "verbalizar",
    "verbalizada": "verbalizar",
    "verbalizadas": "verbalizar",
    "verbalizado": "verbalizar",
    "verbalizados": "verbalizar",
    "verbalizamos": "verbalizar",
    "verbalizan": "verbalizar",
    "verbalizando": "verbalizar",
    "verbalizara": "verbalizar",
    "verbalizarais": "verbalizar",
    "verbalizaran": "verbalizar",
    "verbalizaras": "verbalizar",
    "verbalizare": "verbalizar",
    "verbalizareis": "verbalizar",
    "verbalizaremos": "verbalizar",
    "verbalizaren": "verbalizar",
    "verbalizares": "verbalizar",
    "verbalizaron": "verbalizar",
    "verbalizará": "verbalizar",
    "verbalizarán": "verbalizar",
    "verbalizarás": "verbalizar",
    "verbalizaré": "verbalizar",
    "verbalizaréis": "verbalizar",
    "verbalizaría": "verbalizar",
    "verbalizaríais": "verbalizar",
    "verbalizaríamos": "verbalizar",
    "verbalizarían": "verbalizar",
    "verbalizarías": "verbalizar",
    "verbalizas": "verbalizar",
    "verbalizase": "verbalizar",
    "verbalizaseis": "verbalizar",
    "verbalizasen": "verbalizar",
    "verbalizases": "verbalizar",
    "verbalizaste": "verbalizar",
    "verbalizasteis": "verbalizar",
    "verbalizo": "verbalizar",
    "verbalizábamos": "verbalizar",
    "verbalizáis": "verbalizar",
    "verbalizáramos": "verbalizar",
    "verbalizáremos": "verbalizar",
    "verbalizásemos": "verbalizar",
    "verbalizó": "verbalizar",
    "verbas": "verba",
    "verbascos": "verbasco",
    "verbenas": "verbena",
    "verbenera": "verbenero",
    "verbeneras": "verbenero",
    "verbeneros": "verbenero",
    "verberaciones": "verberación",
    "verborreas": "verborrea",
    "verbos": "verbo",
    "verbosa": "verboso",
    "verbosas": "verboso",
    "verbosidades": "verbosidad",
    "verbosos": "verboso",
    "verbotonales": "verbotonal",
    "verdachos": "verdacho",
    "verdadera": "verdadero",
    "verdaderas": "verdadero",
    "verdaderos": "verdadero",
    "verdades": "verdad",
    "verdales": "verdal",
    "verdascas": "verdasca",
    "verdascazos": "verdascazo",
    "verdea": "verdear",
    "verdeaba": "verdear",
    "verdeabais": "verdear",
    "verdeaban": "verdear",
    "verdeabas": "verdear",
    "verdead": "verdear",
    "verdeada": "verdear",
    "verdeadas": "verdear",
    "verdeado": "verdear",
    "verdeados": "verdear",
    "verdeamos": "verdear",
    "verdean": "verdear",
    "verdeando": "verdear",
    "verdeara": "verdear",
    "verdearais": "verdear",
    "verdearan": "verdear",
    "verdearas": "verdear",
    "verdeare": "verdear",
    "verdeareis": "verdear",
    "verdearemos": "verdear",
    "verdearen": "verdear",
    "verdeares": "verdear",
    "verdearon": "verdear",
    "verdeará": "verdear",
    "verdearán": "verdear",
    "verdearás": "verdear",
    "verdearé": "verdear",
    "verdearéis": "verdear",
    "verdearía": "verdear",
    "verdearíais": "verdear",
    "verdearíamos": "verdear",
    "verdearían": "verdear",
    "verdearías": "verdear",
    "verdeas": "verdear",
    "verdease": "verdear",
    "verdeaseis": "verdear",
    "verdeasen": "verdear",
    "verdeases": "verdear",
    "verdeaste": "verdear",
    "verdeasteis": "verdear",
    "verdece": "verdecer",
    "verdeced": "verdecer",
    "verdeceladones": "verdeceladón",
    "verdeceledones": "verdeceledón",
    "verdecemos": "verdecer",
    "verdecen": "verdecer",
    "verdeceremos": "verdecer",
    "verdecerá": "verdecer",
    "verdecerán": "verdecer",
    "verdecerás": "verdecer",
    "verdeceré": "verdecer",
    "verdeceréis": "verdecer",
    "verdecería": "verdecer",
    "verdeceríais": "verdecer",
    "verdeceríamos": "verdecer",
    "verdecerían": "verdecer",
    "verdecerías": "verdecer",
    "verdeces": "verdecer",
    "verdecida": "verdecer",
    "verdecidas": "verdecer",
    "verdecido": "verdecer",
    "verdecidos": "verdecer",
    "verdeciendo": "verdecer",
    "verdeciera": "verdecer",
    "verdecierais": "verdecer",
    "verdecieran": "verdecer",
    "verdecieras": "verdecer",
    "verdeciere": "verdecer",
    "verdeciereis": "verdecer",
    "verdecieren": "verdecer",
    "verdecieres": "verdecer",
    "verdecieron": "verdecer",
    "verdeciese": "verdecer",
    "verdecieseis": "verdecer",
    "verdeciesen": "verdecer",
    "verdecieses": "verdecer",
    "verdecillos": "verdecillo",
    "verdecimos": "verdecer",
    "verdeciste": "verdecer",
    "verdecisteis": "verdecer",
    "verdeciéramos": "verdecer",
    "verdeciéremos": "verdecer",
    "verdeciésemos": "verdecer",
    "verdeció": "verdecer",
    "verdecéis": "verdecer",
    "verdecí": "verdecer",
    "verdecía": "verdecer",
    "verdecíais": "verdecer",
    "verdecíamos": "verdecer",
    "verdecían": "verdecer",
    "verdecías": "verdecer",
    "verdee": "verdear",
    "verdeemos": "verdear",
    "verdeen": "verdear",
    "verdees": "verdear",
    "verdegales": "verdegal",
    "verdegayes": "verdegay",
    "verdegays": "verdegay",
    "verdeja": "verdejo",
    "verdejas": "verdejo",
    "verdejos": "verdejo",
    "verdelitas": "verdelita",
    "verdemares": "verdemar",
    "verdeo": "verdear",
    "verdeos": "verdeo",
    "verderona": "verderón",
    "verderonas": "verderón",
    "verderones": "verderón",
    "verdes": "verde",
    "verdetes": "verdete",
    "verdevejigas": "verdevejiga",
    "verdezca": "verdecer",
    "verdezcamos": "verdecer",
    "verdezcan": "verdecer",
    "verdezcas": "verdecer",
    "verdezco": "verdecer",
    "verdezcáis": "verdecer",
    "verdeábamos": "verdear",
    "verdeáis": "verdear",
    "verdeáramos": "verdear",
    "verdeáremos": "verdear",
    "verdeásemos": "verdear",
    "verdeé": "verdear",
    "verdeéis": "verdear",
    "verdeó": "verdear",
    "verdiblanca": "verdiblanco",
    "verdiblancas": "verdiblanco",
    "verdiblancos": "verdiblanco",
    "verdina": "verdino",
    "verdinales": "verdinal",
    "verdinas": "verdino",
    "verdinegra": "verdinegro",
    "verdinegras": "verdinegro",
    "verdinegros": "verdinegro",
    "verdines": "verdín",
    "verdinos": "verdino",
    "verdiseca": "verdiseco",
    "verdisecas": "verdiseco",
    "verdisecos": "verdiseco",
    "verditas": "verdita",
    "verdoglobinas": "verdoglobina",
    "verdolagas": "verdolaga",
    "verdores": "verdor",
    "verdosa": "verdoso",
    "verdosas": "verdoso",
    "verdosos": "verdoso",
    "verdoyos": "verdoyo",
    "verdugadas": "verdugada",
    "verdugales": "verdugal",
    "verdugazos": "verdugazo",
    "verdugones": "verdugón",
    "verdugos": "verdugo",
    "verduguillos": "verduguillo",
    "verdulera": "verdulero",
    "verduleras": "verdulero",
    "verduleros": "verdulero",
    "verdulerías": "verdulería",
    "verdunizaciones": "verdunización",
    "verduras": "verdura",
    "verduritas": "verdurita",
    "verdusca": "verdusco",
    "verduscas": "verdusco",
    "verduscos": "verdusco",
    "verecunda": "verecundo",
    "verecundas": "verecundo",
    "verecundias": "verecundia",
    "verecundos": "verecundo",
    "veredas": "vereda",
    "veredictos": "veredicto",
    "veremos": "ver",
    "veres": "ver",
    "vergajazos": "vergajazo",
    "vergajos": "vergajo",
    "vergas": "verga",
    "vergeles": "vergel",
    "vergencias": "vergencia",
    "vergetas": "vergeta",
    "verglases": "verglás",
    "vergonzantes": "vergonzante",
    "vergonzosa": "vergonzoso",
    "vergonzosas": "vergonzoso",
    "vergonzosos": "vergonzoso",
    "verguea": "verguear",
    "vergueaba": "verguear",
    "vergueabais": "verguear",
    "vergueaban": "verguear",
    "vergueabas": "verguear",
    "verguead": "verguear",
    "vergueada": "verguear",
    "vergueadas": "verguear",
    "vergueado": "verguear",
    "vergueados": "verguear",
    "vergueamos": "verguear",
    "verguean": "verguear",
    "vergueando": "verguear",
    "vergueara": "verguear",
    "verguearais": "verguear",
    "verguearan": "verguear",
    "verguearas": "verguear",
    "vergueare": "verguear",
    "vergueareis": "verguear",
    "verguearemos": "verguear",
    "verguearen": "verguear",
    "vergueares": "verguear",
    "verguearon": "verguear",
    "vergueará": "verguear",
    "verguearán": "verguear",
    "verguearás": "verguear",
    "verguearé": "verguear",
    "verguearéis": "verguear",
    "verguearía": "verguear",
    "verguearíais": "verguear",
    "verguearíamos": "verguear",
    "verguearían": "verguear",
    "verguearías": "verguear",
    "vergueas": "verguear",
    "verguease": "verguear",
    "vergueaseis": "verguear",
    "vergueasen": "verguear",
    "vergueases": "verguear",
    "vergueaste": "verguear",
    "vergueasteis": "verguear",
    "verguee": "verguear",
    "vergueemos": "verguear",
    "vergueen": "verguear",
    "verguees": "verguear",
    "vergueo": "verguear",
    "verguetas": "vergueta",
    "vergueábamos": "verguear",
    "vergueáis": "verguear",
    "vergueáramos": "verguear",
    "vergueáremos": "verguear",
    "vergueásemos": "verguear",
    "vergueé": "verguear",
    "vergueéis": "verguear",
    "vergueó": "verguear",
    "verguía": "verguío",
    "verguías": "verguío",
    "verguíos": "verguío",
    "vergüenzas": "vergüenza",
    "vericuetos": "vericueto",
    "verifica": "verificar",
    "verificaba": "verificar",
    "verificabais": "verificar",
    "verificaban": "verificar",
    "verificabas": "verificar",
    "verificabilidades": "verificabilidad",
    "verificables": "verificable",
    "verificaciones": "verificación",
    "verificad": "verificar",
    "verificada": "verificar",
    "verificadas": "verificar",
    "verificado": "verificar",
    "verificadora": "verificador",
    "verificadoras": "verificadora",
    "verificadores": "verificador",
    "verificados": "verificar",
    "verificamos": "verificar",
    "verifican": "verificar",
    "verificando": "verificar",
    "verificara": "verificar",
    "verificarais": "verificar",
    "verificaran": "verificar",
    "verificaras": "verificar",
    "verificare": "verificar",
    "verificareis": "verificar",
    "verificaremos": "verificar",
    "verificaren": "verificar",
    "verificares": "verificar",
    "verificaron": "verificar",
    "verificará": "verificar",
    "verificarán": "verificar",
    "verificarás": "verificar",
    "verificaré": "verificar",
    "verificaréis": "verificar",
    "verificaría": "verificar",
    "verificaríais": "verificar",
    "verificaríamos": "verificar",
    "verificarían": "verificar",
    "verificarías": "verificar",
    "verificas": "verificar",
    "verificase": "verificar",
    "verificaseis": "verificar",
    "verificasen": "verificar",
    "verificases": "verificar",
    "verificaste": "verificar",
    "verificasteis": "verificar",
    "verificativa": "verificativo",
    "verificativas": "verificativo",
    "verificativos": "verificativo",
    "verifico": "verificar",
    "verificábamos": "verificar",
    "verificáis": "verificar",
    "verificáramos": "verificar",
    "verificáremos": "verificar",
    "verificásemos": "verificar",
    "verificó": "verificar",
    "verifique": "verificar",
    "verifiquemos": "verificar",
    "verifiquen": "verificar",
    "verifiques": "verificar",
    "verifiqué": "verificar",
    "verifiquéis": "verificar",
    "verijas": "verija",
    "verilea": "verilear",
    "verileaba": "verilear",
    "verileabais": "verilear",
    "verileaban": "verilear",
    "verileabas": "verilear",
    "verilead": "verilear",
    "verileada": "verilear",
    "verileadas": "verilear",
    "verileado": "verilear",
    "verileados": "verilear",
    "verileamos": "verilear",
    "verilean": "verilear",
    "verileando": "verilear",
    "verileara": "verilear",
    "verilearais": "verilear",
    "verilearan": "verilear",
    "verilearas": "verilear",
    "verileare": "verilear",
    "verileareis": "verilear",
    "verilearemos": "verilear",
    "verilearen": "verilear",
    "verileares": "verilear",
    "verilearon": "verilear",
    "verileará": "verilear",
    "verilearán": "verilear",
    "verilearás": "verilear",
    "verilearé": "verilear",
    "verilearéis": "verilear",
    "verilearía": "verilear",
    "verilearíais": "verilear",
    "verilearíamos": "verilear",
    "verilearían": "verilear",
    "verilearías": "verilear",
    "verileas": "verilear",
    "verilease": "verilear",
    "verileaseis": "verilear",
    "verileasen": "verilear",
    "verileases": "verilear",
    "verileaste": "verilear",
    "verileasteis": "verilear",
    "verilee": "verilear",
    "verileemos": "verilear",
    "verileen": "verilear",
    "verilees": "verilear",
    "verileo": "verilear",
    "veriles": "veril",
    "verileábamos": "verilear",
    "verileáis": "verilear",
    "verileáramos": "verilear",
    "verileáremos": "verilear",
    "verileásemos": "verilear",
    "verileé": "verilear",
    "verileéis": "verilear",
    "verileó": "verilear",
    "verismos": "verismo",
    "veristas": "verista",
    "verjas": "verja",
    "verjurada": "verjurado",
    "verjuradas": "verjurado",
    "verjurados": "verjurado",
    "vermes": "verme",
    "vermicidas": "vermicida",
    "vermiculares": "vermicular",
    "vermiculitas": "vermiculita",
    "vermiformes": "vermiforme",
    "verminosa": "verminoso",
    "verminosas": "verminoso",
    "verminosos": "verminoso",
    "vermuses": "vermú",
    "vermuts": "vermut",
    "vermífuga": "vermífugo",
    "vermífugas": "vermífugo",
    "vermífugos": "vermífugo",
    "vermús": "vermú",
    "vernales": "vernal",
    "vernalizaciones": "vernalización",
    "vernácula": "vernáculo",
    "vernáculas": "vernáculo",
    "vernáculos": "vernáculo",
    "veronales": "veronal",
    "veronenses": "veronense",
    "veronesa": "veronés",
    "veronesas": "veronés",
    "veroneses": "veronés",
    "veros": "vero",
    "verosimilitudes": "verosimilitud",
    "verosímiles": "verosímil",
    "verracos": "verraco",
    "verraquea": "verraquear",
    "verraqueaba": "verraquear",
    "verraqueabais": "verraquear",
    "verraqueaban": "verraquear",
    "verraqueabas": "verraquear",
    "verraquead": "verraquear",
    "verraqueada": "verraquear",
    "verraqueadas": "verraquear",
    "verraqueado": "verraquear",
    "verraqueados": "verraquear",
    "verraqueamos": "verraquear",
    "verraquean": "verraquear",
    "verraqueando": "verraquear",
    "verraqueara": "verraquear",
    "verraquearais": "verraquear",
    "verraquearan": "verraquear",
    "verraquearas": "verraquear",
    "verraqueare": "verraquear",
    "verraqueareis": "verraquear",
    "verraquearemos": "verraquear",
    "verraquearen": "verraquear",
    "verraqueares": "verraquear",
    "verraquearon": "verraquear",
    "verraqueará": "verraquear",
    "verraquearán": "verraquear",
    "verraquearás": "verraquear",
    "verraquearé": "verraquear",
    "verraquearéis": "verraquear",
    "verraquearía": "verraquear",
    "verraquearíais": "verraquear",
    "verraquearíamos": "verraquear",
    "verraquearían": "verraquear",
    "verraquearías": "verraquear",
    "verraqueas": "verraquear",
    "verraquease": "verraquear",
    "verraqueaseis": "verraquear",
    "verraqueasen": "verraquear",
    "verraqueases": "verraquear",
    "verraqueaste": "verraquear",
    "verraqueasteis": "verraquear",
    "verraquee": "verraquear",
    "verraqueemos": "verraquear",
    "verraqueen": "verraquear",
    "verraquees": "verraquear",
    "verraqueo": "verraquear",
    "verraqueras": "verraquera",
    "verraqueábamos": "verraquear",
    "verraqueáis": "verraquear",
    "verraqueáramos": "verraquear",
    "verraqueáremos": "verraquear",
    "verraqueásemos": "verraquear",
    "verraqueé": "verraquear",
    "verraqueéis": "verraquear",
    "verraqueó": "verraquear",
    "verrionda": "verriondo",
    "verriondas": "verriondo",
    "verriondeces": "verriondez",
    "verriondos": "verriondo",
    "verrones": "verrón",
    "verrugas": "verruga",
    "verrugatos": "verrugato",
    "verrugos": "verrugo",
    "verrugosa": "verrugoso",
    "verrugosas": "verrugoso",
    "verrugosos": "verrugoso",
    "versa": "verso",
    "versaba": "versar",
    "versabais": "versar",
    "versaban": "versar",
    "versabas": "versar",
    "versad": "versar",
    "versada": "versar",
    "versadas": "versar",
    "versado": "versar",
    "versados": "versar",
    "versales": "versal",
    "versalillas": "versalilla",
    "versalitas": "versalita",
    "versallesca": "versallesco",
    "versallescas": "versallesco",
    "versallescos": "versallesco",
    "versamos": "versar",
    "versan": "versar",
    "versando": "versar",
    "versara": "versar",
    "versarais": "versar",
    "versaran": "versar",
    "versaras": "versar",
    "versare": "versar",
    "versareis": "versar",
    "versaremos": "versar",
    "versaren": "versar",
    "versares": "versar",
    "versaron": "versar",
    "versará": "versar",
    "versarán": "versar",
    "versarás": "versar",
    "versaré": "versar",
    "versaréis": "versar",
    "versaría": "versar",
    "versaríais": "versar",
    "versaríamos": "versar",
    "versarían": "versar",
    "versarías": "versar",
    "versas": "verso",
    "versase": "versar",
    "versaseis": "versar",
    "versasen": "versar",
    "versases": "versar",
    "versaste": "versar",
    "versasteis": "versar",
    "versatilidades": "versatilidad",
    "verse": "versar",
    "versemos": "versar",
    "versen": "versar",
    "verses": "versar",
    "versicularios": "versiculario",
    "versifica": "versificar",
    "versificaba": "versificar",
    "versificabais": "versificar",
    "versificaban": "versificar",
    "versificabas": "versificar",
    "versificaciones": "versificación",
    "versificad": "versificar",
    "versificada": "versificar",
    "versificadas": "versificar",
    "versificado": "versificar",
    "versificadora": "versificador",
    "versificadoras": "versificador",
    "versificadores": "versificador",
    "versificados": "versificar",
    "versificamos": "versificar",
    "versifican": "versificar",
    "versificando": "versificar",
    "versificara": "versificar",
    "versificarais": "versificar",
    "versificaran": "versificar",
    "versificaras": "versificar",
    "versificare": "versificar",
    "versificareis": "versificar",
    "versificaremos": "versificar",
    "versificaren": "versificar",
    "versificares": "versificar",
    "versificaron": "versificar",
    "versificará": "versificar",
    "versificarán": "versificar",
    "versificarás": "versificar",
    "versificaré": "versificar",
    "versificaréis": "versificar",
    "versificaría": "versificar",
    "versificaríais": "versificar",
    "versificaríamos": "versificar",
    "versificarían": "versificar",
    "versificarías": "versificar",
    "versificas": "versificar",
    "versificase": "versificar",
    "versificaseis": "versificar",
    "versificasen": "versificar",
    "versificases": "versificar",
    "versificaste": "versificar",
    "versificasteis": "versificar",
    "versifico": "versificar",
    "versificábamos": "versificar",
    "versificáis": "versificar",
    "versificáramos": "versificar",
    "versificáremos": "versificar",
    "versificásemos": "versificar",
    "versificó": "versificar",
    "versifique": "versificar",
    "versifiquemos": "versificar",
    "versifiquen": "versificar",
    "versifiques": "versificar",
    "versifiqué": "versificar",
    "versifiquéis": "versificar",
    "versiona": "versionar",
    "versionaba": "versionar",
    "versionabais": "versionar",
    "versionaban": "versionar",
    "versionabas": "versionar",
    "versionad": "versionar",
    "versionada": "versionar",
    "versionadas": "versionar",
    "versionado": "versionar",
    "versionados": "versionar",
    "versionamos": "versionar",
    "versionan": "versionar",
    "versionando": "versionar",
    "versionara": "versionar",
    "versionarais": "versionar",
    "versionaran": "versionar",
    "versionaras": "versionar",
    "versionare": "versionar",
    "versionareis": "versionar",
    "versionaremos": "versionar",
    "versionaren": "versionar",
    "versionares": "versionar",
    "versionaron": "versionar",
    "versionará": "versionar",
    "versionarán": "versionar",
    "versionarás": "versionar",
    "versionaré": "versionar",
    "versionaréis": "versionar",
    "versionaría": "versionar",
    "versionaríais": "versionar",
    "versionaríamos": "versionar",
    "versionarían": "versionar",
    "versionarías": "versionar",
    "versionas": "versionar",
    "versionase": "versionar",
    "versionaseis": "versionar",
    "versionasen": "versionar",
    "versionases": "versionar",
    "versionaste": "versionar",
    "versionasteis": "versionar",
    "versione": "versionar",
    "versionemos": "versionar",
    "versionen": "versionar",
    "versiones": "versionar",
    "versiono": "versionar",
    "versionábamos": "versionar",
    "versionáis": "versionar",
    "versionáramos": "versionar",
    "versionáremos": "versionar",
    "versionásemos": "versionar",
    "versioné": "versionar",
    "versionéis": "versionar",
    "versionó": "versionar",
    "versistas": "versista",
    "verso": "versar",
    "versolaris": "versolari",
    "versos": "verso",
    "verstas": "versta",
    "versábamos": "versar",
    "versáis": "versar",
    "versáramos": "versar",
    "versáremos": "versar",
    "versásemos": "versar",
    "versátiles": "versátil",
    "versé": "versar",
    "verséis": "versar",
    "versículas": "versícula",
    "versículos": "versículo",
    "versó": "versar",
    "vertamos": "verter",
    "vertebra": "vertebrar",
    "vertebraba": "vertebrar",
    "vertebrabais": "vertebrar",
    "vertebraban": "vertebrar",
    "vertebrabas": "vertebrar",
    "vertebraciones": "vertebración",
    "vertebrad": "vertebrar",
    "vertebrada": "vertebrar",
    "vertebradas": "vertebrar",
    "vertebrado": "vertebrar",
    "vertebradora": "vertebrador",
    "vertebradoras": "vertebrador",
    "vertebradores": "vertebrador",
    "vertebrados": "vertebrar",
    "vertebrales": "vertebral",
    "vertebramos": "vertebrar",
    "vertebran": "vertebrar",
    "vertebrando": "vertebrar",
    "vertebrara": "vertebrar",
    "vertebrarais": "vertebrar",
    "vertebraran": "vertebrar",
    "vertebraras": "vertebrar",
    "vertebrare": "vertebrar",
    "vertebrareis": "vertebrar",
    "vertebraremos": "vertebrar",
    "vertebraren": "vertebrar",
    "vertebrares": "vertebrar",
    "vertebraron": "vertebrar",
    "vertebrará": "vertebrar",
    "vertebrarán": "vertebrar",
    "vertebrarás": "vertebrar",
    "vertebraré": "vertebrar",
    "vertebraréis": "vertebrar",
    "vertebraría": "vertebrar",
    "vertebraríais": "vertebrar",
    "vertebraríamos": "vertebrar",
    "vertebrarían": "vertebrar",
    "vertebrarías": "vertebrar",
    "vertebras": "vertebrar",
    "vertebrase": "vertebrar",
    "vertebraseis": "vertebrar",
    "vertebrasen": "vertebrar",
    "vertebrases": "vertebrar",
    "vertebraste": "vertebrar",
    "vertebrasteis": "vertebrar",
    "vertebre": "vertebrar",
    "vertebremos": "vertebrar",
    "vertebren": "vertebrar",
    "vertebres": "vertebrar",
    "vertebro": "vertebrar",
    "vertebroarteriales": "vertebroarterial",
    "vertebrábamos": "vertebrar",
    "vertebráis": "vertebrar",
    "vertebráramos": "vertebrar",
    "vertebráremos": "vertebrar",
    "vertebrásemos": "vertebrar",
    "vertebré": "vertebrar",
    "vertebréis": "vertebrar",
    "vertebró": "vertebrar",
    "verted": "verter",
    "vertederas": "vertedera",
    "vertederos": "vertedero",
    "vertedora": "vertedor",
    "vertedoras": "vertedor",
    "vertedores": "vertedor",
    "vertellos": "vertello",
    "vertemos": "verter",
    "verteremos": "verter",
    "verterá": "verter",
    "verterán": "verter",
    "verterás": "verter",
    "verteré": "verter",
    "verteréis": "verter",
    "vertería": "verter",
    "verteríais": "verter",
    "verteríamos": "verter",
    "verterían": "verter",
    "verterías": "verter",
    "vertibilidades": "vertibilidad",
    "vertibles": "vertible",
    "verticales": "vertical",
    "verticalidades": "verticalidad",
    "verticidades": "verticidad",
    "verticilada": "verticilado",
    "verticiladas": "verticilado",
    "verticilados": "verticilado",
    "verticilos": "verticilo",
    "vertida": "verter",
    "vertidas": "verter",
    "vertido": "verter",
    "vertidos": "vertido",
    "vertiendo": "verter",
    "vertientes": "vertiente",
    "vertiera": "verter",
    "vertierais": "verter",
    "vertieran": "verter",
    "vertieras": "verter",
    "vertiere": "verter",
    "vertiereis": "verter",
    "vertieren": "verter",
    "vertieres": "verter",
    "vertieron": "verter",
    "vertiese": "verter",
    "vertieseis": "verter",
    "vertiesen": "verter",
    "vertieses": "verter",
    "vertiginosa": "vertiginoso",
    "vertiginosas": "vertiginoso",
    "vertiginosidades": "vertiginosidad",
    "vertiginosos": "vertiginoso",
    "vertimientos": "vertimiento",
    "vertimos": "verter",
    "vertisoles": "vertisol",
    "vertiste": "verter",
    "vertisteis": "verter",
    "vertisuelos": "vertisuelo",
    "vertiéramos": "verter",
    "vertiéremos": "verter",
    "vertiésemos": "verter",
    "vertió": "verter",
    "vertáis": "verter",
    "vertéis": "verter",
    "vertí": "verter",
    "vertía": "verter",
    "vertíais": "verter",
    "vertíamos": "verter",
    "vertían": "verter",
    "vertías": "verter",
    "verá": "ver",
    "verán": "ver",
    "verás": "ver",
    "veré": "ver",
    "veréis": "ver",
    "vería": "ver",
    "veríais": "ver",
    "veríamos": "ver",
    "verían": "ver",
    "verías": "ver",
    "verídica": "verídico",
    "verídicas": "verídico",
    "verídicos": "verídico",
    "verónicas": "verónica",
    "ves": "ver",
    "vesanias": "vesania",
    "vesicales": "vesical",
    "vesicantes": "vesicante",
    "vesicatoria": "vesicatorio",
    "vesicatorias": "vesicatorio",
    "vesicatorios": "vesicatorio",
    "vesicouterina": "vesicouterino",
    "vesicouterinas": "vesicouterino",
    "vesicouterinos": "vesicouterino",
    "vesiculares": "vesicular",
    "vesiculosa": "vesiculoso",
    "vesiculosas": "vesiculoso",
    "vesiculosos": "vesiculoso",
    "vespas": "vespa",
    "vesperales": "vesperal",
    "vespertiliónidos": "vespertiliónido",
    "vespertina": "vespertino",
    "vespertinas": "vespertino",
    "vespertinos": "vespertino",
    "vestales": "vestal",
    "vestibulares": "vestibular",
    "vestibulococleares": "vestibulococlear",
    "vestibulofosales": "vestibulofosal",
    "vestibulometrías": "vestibulometría",
    "vestibulooculares": "vestibuloocular",
    "vestibuloplastias": "vestibuloplastia",
    "vestibulotomías": "vestibulotomía",
    "vestid": "vestir",
    "vestida": "vestir",
    "vestidas": "vestir",
    "vestido": "vestir",
    "vestidores": "vestidor",
    "vestidos": "vestir",
    "vestiduras": "vestidura",
    "vestigiales": "vestigial",
    "vestigios": "vestigio",
    "vestiglos": "vestiglo",
    "vestimentas": "vestimenta",
    "vestimos": "vestir",
    "vestiremos": "vestir",
    "vestirá": "vestir",
    "vestirán": "vestir",
    "vestirás": "vestir",
    "vestiré": "vestir",
    "vestiréis": "vestir",
    "vestiría": "vestir",
    "vestiríais": "vestir",
    "vestiríamos": "vestir",
    "vestirían": "vestir",
    "vestirías": "vestir",
    "vestiste": "vestir",
    "vestisteis": "vestir",
    "vestuarios": "vestuario",
    "vestugos": "vestugo",
    "vestí": "vestir",
    "vestía": "vestir",
    "vestíais": "vestir",
    "vestíamos": "vestir",
    "vestían": "vestir",
    "vestías": "vestir",
    "vestíbulos": "vestíbulo",
    "vestís": "vestir",
    "vesubianas": "vesubiana",
    "vesubianitas": "vesubianita",
    "vesánica": "vesánico",
    "vesánicas": "vesánico",
    "vesánicos": "vesánico",
    "vesículas": "vesícula",
    "veta": "vetar",
    "vetaba": "vetar",
    "vetabais": "vetar",
    "vetaban": "vetar",
    "vetabas": "vetar",
    "vetad": "vetar",
    "vetada": "vetar",
    "vetadas": "vetar",
    "vetado": "vetar",
    "vetados": "vetar",
    "vetamos": "vetar",
    "vetan": "vetar",
    "vetando": "vetar",
    "vetara": "vetar",
    "vetarais": "vetar",
    "vetaran": "vetar",
    "vetaras": "vetar",
    "vetare": "vetar",
    "vetareis": "vetar",
    "vetaremos": "vetar",
    "vetaren": "vetar",
    "vetares": "vetar",
    "vetaron": "vetar",
    "vetará": "vetar",
    "vetarán": "vetar",
    "vetarás": "vetar",
    "vetaré": "vetar",
    "vetaréis": "vetar",
    "vetaría": "vetar",
    "vetaríais": "vetar",
    "vetaríamos": "vetar",
    "vetarían": "vetar",
    "vetarías": "vetar",
    "vetas": "vetar",
    "vetase": "vetar",
    "vetaseis": "vetar",
    "vetasen": "vetar",
    "vetases": "vetar",
    "vetaste": "vetar",
    "vetasteis": "vetar",
    "vete": "vetar",
    "vetea": "vetear",
    "veteaba": "vetear",
    "veteabais": "vetear",
    "veteaban": "vetear",
    "veteabas": "vetear",
    "vetead": "vetear",
    "veteada": "vetear",
    "veteadas": "vetear",
    "veteado": "vetear",
    "veteados": "vetear",
    "veteamos": "vetear",
    "vetean": "vetear",
    "veteando": "vetear",
    "veteara": "vetear",
    "vetearais": "vetear",
    "vetearan": "vetear",
    "vetearas": "vetear",
    "veteare": "vetear",
    "veteareis": "vetear",
    "vetearemos": "vetear",
    "vetearen": "vetear",
    "veteares": "vetear",
    "vetearon": "vetear",
    "veteará": "vetear",
    "vetearán": "vetear",
    "vetearás": "vetear",
    "vetearé": "vetear",
    "vetearéis": "vetear",
    "vetearía": "vetear",
    "vetearíais": "vetear",
    "vetearíamos": "vetear",
    "vetearían": "vetear",
    "vetearías": "vetear",
    "veteas": "vetear",
    "vetease": "vetear",
    "veteaseis": "vetear",
    "veteasen": "vetear",
    "veteases": "vetear",
    "veteaste": "vetear",
    "veteasteis": "vetear",
    "vetee": "vetear",
    "veteemos": "vetear",
    "veteen": "vetear",
    "vetees": "vetear",
    "vetemos": "vetar",
    "veten": "vetar",
    "veteo": "vetear",
    "veterana": "veterano",
    "veteranas": "veterano",
    "veteranos": "veterano",
    "veteranías": "veteranía",
    "veterinaria": "veterinario",
    "veterinarias": "veterinario",
    "veterinarios": "veterinario",
    "vetes": "vetar",
    "veteábamos": "vetear",
    "veteáis": "vetear",
    "veteáramos": "vetear",
    "veteáremos": "vetear",
    "veteásemos": "vetear",
    "veteé": "vetear",
    "veteéis": "vetear",
    "veteó": "vetear",
    "vetiveroles": "vetiverol",
    "veto": "vetar",
    "vetos": "veto",
    "vetusta": "vetusto",
    "vetustas": "vetusto",
    "vetusteces": "vetustez",
    "vetustos": "vetusto",
    "vetábamos": "vetar",
    "vetáis": "vetar",
    "vetáramos": "vetar",
    "vetáremos": "vetar",
    "vetásemos": "vetar",
    "veté": "vetar",
    "vetéis": "vetar",
    "vetó": "vetar",
    "vexaciones": "vexación",
    "vexaciónes": "vexación",
    "vezas": "veza",
    "veáis": "ver",
    "veía": "ver",
    "veíais": "ver",
    "veíamos": "ver",
    "veían": "ver",
    "veías": "ver",
    "vi": "ver",
    "viabilice": "viabilizar",
    "viabilicemos": "viabilizar",
    "viabilicen": "viabilizar",
    "viabilices": "viabilizar",
    "viabilicé": "viabilizar",
    "viabilicéis": "viabilizar",
    "viabilidades": "viabilidad",
    "viabiliza": "viabilizar",
    "viabilizaba": "viabilizar",
    "viabilizabais": "viabilizar",
    "viabilizaban": "viabilizar",
    "viabilizabas": "viabilizar",
    "viabilizad": "viabilizar",
    "viabilizada": "viabilizar",
    "viabilizadas": "viabilizar",
    "viabilizado": "viabilizar",
    "viabilizados": "viabilizar",
    "viabilizamos": "viabilizar",
    "viabilizan": "viabilizar",
    "viabilizando": "viabilizar",
    "viabilizara": "viabilizar",
    "viabilizarais": "viabilizar",
    "viabilizaran": "viabilizar",
    "viabilizaras": "viabilizar",
    "viabilizare": "viabilizar",
    "viabilizareis": "viabilizar",
    "viabilizaremos": "viabilizar",
    "viabilizaren": "viabilizar",
    "viabilizares": "viabilizar",
    "viabilizaron": "viabilizar",
    "viabilizará": "viabilizar",
    "viabilizarán": "viabilizar",
    "viabilizarás": "viabilizar",
    "viabilizaré": "viabilizar",
    "viabilizaréis": "viabilizar",
    "viabilizaría": "viabilizar",
    "viabilizaríais": "viabilizar",
    "viabilizaríamos": "viabilizar",
    "viabilizarían": "viabilizar",
    "viabilizarías": "viabilizar",
    "viabilizas": "viabilizar",
    "viabilizase": "viabilizar",
    "viabilizaseis": "viabilizar",
    "viabilizasen": "viabilizar",
    "viabilizases": "viabilizar",
    "viabilizaste": "viabilizar",
    "viabilizasteis": "viabilizar",
    "viabilizo": "viabilizar",
    "viabilizábamos": "viabilizar",
    "viabilizáis": "viabilizar",
    "viabilizáramos": "viabilizar",
    "viabilizáremos": "viabilizar",
    "viabilizásemos": "viabilizar",
    "viabilizó": "viabilizar",
    "viables": "viable",
    "viadas": "viada",
    "viadores": "viador",
    "viaductos": "viaducto",
    "viaja": "viajar",
    "viajaba": "viajar",
    "viajabais": "viajar",
    "viajaban": "viajar",
    "viajabas": "viajar",
    "viajad": "viajar",
    "viajada": "viajar",
    "viajadas": "viajar",
    "viajado": "viajar",
    "viajados": "viajar",
    "viajamos": "viajar",
    "viajan": "viajar",
    "viajando": "viajar",
    "viajantes": "viajante",
    "viajara": "viajar",
    "viajarais": "viajar",
    "viajaran": "viajar",
    "viajaras": "viajar",
    "viajare": "viajar",
    "viajareis": "viajar",
    "viajaremos": "viajar",
    "viajaren": "viajar",
    "viajares": "viajar",
    "viajaron": "viajar",
    "viajará": "viajar",
    "viajarán": "viajar",
    "viajarás": "viajar",
    "viajaré": "viajar",
    "viajaréis": "viajar",
    "viajaría": "viajar",
    "viajaríais": "viajar",
    "viajaríamos": "viajar",
    "viajarían": "viajar",
    "viajarías": "viajar",
    "viajas": "viajar",
    "viajase": "viajar",
    "viajaseis": "viajar",
    "viajasen": "viajar",
    "viajases": "viajar",
    "viajaste": "viajar",
    "viajasteis": "viajar",
    "viaje": "viajar",
    "viajemos": "viajar",
    "viajen": "viajar",
    "viajera": "viajero",
    "viajeras": "viajero",
    "viajeros": "viajero",
    "viajes": "viaje",
    "viajo": "viajar",
    "viajábamos": "viajar",
    "viajáis": "viajar",
    "viajáramos": "viajar",
    "viajáremos": "viajar",
    "viajásemos": "viajar",
    "viajé": "viajar",
    "viajéis": "viajar",
    "viajó": "viajar",
    "viales": "vial",
    "vialidades": "vialidad",
    "viandantes": "viandante",
    "viandas": "vianda",
    "viaria": "viario",
    "viarias": "viario",
    "viarios": "viario",
    "viatica": "viaticar",
    "viaticaba": "viaticar",
    "viaticabais": "viaticar",
    "viaticaban": "viaticar",
    "viaticabas": "viaticar",
    "viaticad": "viaticar",
    "viaticada": "viaticar",
    "viaticadas": "viaticar",
    "viaticado": "viaticar",
    "viaticados": "viaticar",
    "viaticamos": "viaticar",
    "viatican": "viaticar",
    "viaticando": "viaticar",
    "viaticara": "viaticar",
    "viaticarais": "viaticar",
    "viaticaran": "viaticar",
    "viaticaras": "viaticar",
    "viaticare": "viaticar",
    "viaticareis": "viaticar",
    "viaticaremos": "viaticar",
    "viaticaren": "viaticar",
    "viaticares": "viaticar",
    "viaticaron": "viaticar",
    "viaticará": "viaticar",
    "viaticarán": "viaticar",
    "viaticarás": "viaticar",
    "viaticaré": "viaticar",
    "viaticaréis": "viaticar",
    "viaticaría": "viaticar",
    "viaticaríais": "viaticar",
    "viaticaríamos": "viaticar",
    "viaticarían": "viaticar",
    "viaticarías": "viaticar",
    "viaticas": "viaticar",
    "viaticase": "viaticar",
    "viaticaseis": "viaticar",
    "viaticasen": "viaticar",
    "viaticases": "viaticar",
    "viaticaste": "viaticar",
    "viaticasteis": "viaticar",
    "viatico": "viaticar",
    "viaticábamos": "viaticar",
    "viaticáis": "viaticar",
    "viaticáramos": "viaticar",
    "viaticáremos": "viaticar",
    "viaticásemos": "viaticar",
    "viaticó": "viaticar",
    "viatique": "viaticar",
    "viatiquemos": "viaticar",
    "viatiquen": "viaticar",
    "viatiques": "viaticar",
    "viatiqué": "viaticar",
    "viatiquéis": "viaticar",
    "viboranes": "viborán",
    "viborea": "viborear",
    "viboreaba": "viborear",
    "viboreabais": "viborear",
    "viboreaban": "viborear",
    "viboreabas": "viborear",
    "viboread": "viborear",
    "viboreada": "viborear",
    "viboreadas": "viborear",
    "viboreado": "viborear",
    "viboreados": "viborear",
    "viboreamos": "viborear",
    "viborean": "viborear",
    "viboreando": "viborear",
    "viboreara": "viborear",
    "viborearais": "viborear",
    "viborearan": "viborear",
    "viborearas": "viborear",
    "viboreare": "viborear",
    "viboreareis": "viborear",
    "viborearemos": "viborear",
    "viborearen": "viborear",
    "viboreares": "viborear",
    "viborearon": "viborear",
    "viboreará": "viborear",
    "viborearán": "viborear",
    "viborearás": "viborear",
    "viborearé": "viborear",
    "viborearéis": "viborear",
    "viborearía": "viborear",
    "viborearíais": "viborear",
    "viborearíamos": "viborear",
    "viborearían": "viborear",
    "viborearías": "viborear",
    "viboreas": "viborear",
    "viborease": "viborear",
    "viboreaseis": "viborear",
    "viboreasen": "viborear",
    "viboreases": "viborear",
    "viboreaste": "viborear",
    "viboreasteis": "viborear",
    "viboree": "viborear",
    "viboreemos": "viborear",
    "viboreen": "viborear",
    "viborees": "viborear",
    "viboreo": "viborear",
    "viboreras": "viborera",
    "viborezna": "viborezno",
    "viboreznas": "viborezno",
    "viboreznos": "viborezno",
    "viboreábamos": "viborear",
    "viboreáis": "viborear",
    "viboreáramos": "viborear",
    "viboreáremos": "viborear",
    "viboreásemos": "viborear",
    "viboreé": "viborear",
    "viboreéis": "viborear",
    "viboreó": "viborear",
    "vibra": "vibrar",
    "vibraba": "vibrar",
    "vibrabais": "vibrar",
    "vibraban": "vibrar",
    "vibrabas": "vibrar",
    "vibraciones": "vibración",
    "vibrad": "vibrar",
    "vibrada": "vibrar",
    "vibradas": "vibrar",
    "vibrado": "vibrar",
    "vibradora": "vibrador",
    "vibradoras": "vibrador",
    "vibradores": "vibrador",
    "vibrados": "vibrar",
    "vibramos": "vibrar",
    "vibran": "vibrar",
    "vibrando": "vibrar",
    "vibrantes": "vibrante",
    "vibrara": "vibrar",
    "vibrarais": "vibrar",
    "vibraran": "vibrar",
    "vibraras": "vibrar",
    "vibrare": "vibrar",
    "vibrareis": "vibrar",
    "vibraremos": "vibrar",
    "vibraren": "vibrar",
    "vibrares": "vibrar",
    "vibraron": "vibrar",
    "vibrará": "vibrar",
    "vibrarán": "vibrar",
    "vibrarás": "vibrar",
    "vibraré": "vibrar",
    "vibraréis": "vibrar",
    "vibraría": "vibrar",
    "vibraríais": "vibrar",
    "vibraríamos": "vibrar",
    "vibrarían": "vibrar",
    "vibrarías": "vibrar",
    "vibras": "vibrar",
    "vibrase": "vibrar",
    "vibraseis": "vibrar",
    "vibrasen": "vibrar",
    "vibrases": "vibrar",
    "vibraste": "vibrar",
    "vibrasteis": "vibrar",
    "vibratoria": "vibratorio",
    "vibratorias": "vibratorio",
    "vibratorios": "vibratorio",
    "vibre": "vibrar",
    "vibremos": "vibrar",
    "vibren": "vibrar",
    "vibres": "vibrar",
    "vibrisas": "vibrisa",
    "vibro": "vibrar",
    "vibroflotaciones": "vibroflotación",
    "vibroscopios": "vibroscopio",
    "vibrábamos": "vibrar",
    "vibráfonos": "vibráfono",
    "vibráis": "vibrar",
    "vibráramos": "vibrar",
    "vibráremos": "vibrar",
    "vibrásemos": "vibrar",
    "vibrátiles": "vibrátil",
    "vibré": "vibrar",
    "vibréis": "vibrar",
    "vibró": "vibrar",
    "vibrógrafos": "vibrógrafo",
    "vibrómetros": "vibrómetro",
    "vibróscopos": "vibróscopo",
    "viburnos": "viburno",
    "vicalvaradas": "vicalvarada",
    "vicaria": "vicario",
    "vicariales": "vicarial",
    "vicariantes": "vicariante",
    "vicarianzas": "vicarianza",
    "vicarias": "vicario",
    "vicariatos": "vicariato",
    "vicarios": "vicario",
    "vicarías": "vicaría",
    "vicealmirantas": "vicealmiranta",
    "vicealmirantazgos": "vicealmirantazgo",
    "vicealmirantes": "vicealmirante",
    "vicecancilleres": "vicecanciller",
    "vicecancillerías": "vicecancillería",
    "viceconsiliaria": "viceconsiliario",
    "viceconsiliarias": "viceconsiliario",
    "viceconsiliarios": "viceconsiliario",
    "viceconsulados": "viceconsulado",
    "vicecristos": "vicecristo",
    "vicecónsules": "vicecónsul",
    "vicedioses": "vicediós",
    "vicedirectora": "vicedirector",
    "vicedirectoras": "vicedirector",
    "vicedirectores": "vicedirector",
    "vicegerencias": "vicegerencia",
    "vicegerentes": "vicegerente",
    "vicegobernadora": "vicegobernador",
    "vicegobernadoras": "vicegobernador",
    "vicegobernadores": "vicegobernador",
    "viceministra": "viceministro",
    "viceministras": "viceministro",
    "viceministros": "viceministro",
    "vicenales": "vicenal",
    "vicepatrona": "vicepatrón",
    "vicepatronas": "vicepatrón",
    "vicepatrones": "vicepatrón",
    "vicepresidencias": "vicepresidencia",
    "vicepresidenta": "vicepresidente",
    "vicepresidentas": "vicepresidente",
    "vicepresidentes": "vicepresidente",
    "viceprimer": "viceprimero",
    "viceprimera": "viceprimero",
    "viceprimeras": "viceprimero",
    "viceprimeros": "viceprimero",
    "viceprovinciales": "viceprovincial",
    "viceprovincias": "viceprovincia",
    "vicerrectora": "vicerrector",
    "vicerrectorados": "vicerrectorado",
    "vicerrectoras": "vicerrector",
    "vicerrectores": "vicerrector",
    "vicesecretaria": "vicesecretario",
    "vicesecretarias": "vicesecretario",
    "vicesecretarios": "vicesecretario",
    "vicesecretarías": "vicesecretaría",
    "vicetesorera": "vicetesorero",
    "vicetesoreras": "vicetesorero",
    "vicetesoreros": "vicetesorero",
    "vicetiples": "vicetiple",
    "vicha": "vichar",
    "vichaba": "vichar",
    "vichabais": "vichar",
    "vichaban": "vichar",
    "vichabas": "vichar",
    "vichad": "vichar",
    "vichada": "vichar",
    "vichadas": "vichar",
    "vichado": "vichar",
    "vichados": "vichar",
    "vichamos": "vichar",
    "vichan": "vichar",
    "vichando": "vichar",
    "vichara": "vichar",
    "vicharais": "vichar",
    "vicharan": "vichar",
    "vicharas": "vichar",
    "vichare": "vichar",
    "vichareis": "vichar",
    "vicharemos": "vichar",
    "vicharen": "vichar",
    "vichares": "vichar",
    "vicharon": "vichar",
    "vichará": "vichar",
    "vicharán": "vichar",
    "vicharás": "vichar",
    "vicharé": "vichar",
    "vicharéis": "vichar",
    "vicharía": "vichar",
    "vicharíais": "vichar",
    "vicharíamos": "vichar",
    "vicharían": "vichar",
    "vicharías": "vichar",
    "vichas": "vichar",
    "vichase": "vichar",
    "vichaseis": "vichar",
    "vichasen": "vichar",
    "vichases": "vichar",
    "vichaste": "vichar",
    "vichasteis": "vichar",
    "viche": "vichar",
    "vichemos": "vichar",
    "vichen": "vichar",
    "viches": "vichar",
    "vichies": "vichy",
    "vicho": "vichar",
    "vichyssoises": "vichyssoise",
    "vichábamos": "vichar",
    "vicháis": "vichar",
    "vicháramos": "vichar",
    "vicháremos": "vichar",
    "vichásemos": "vichar",
    "viché": "vichar",
    "vichéis": "vichar",
    "vichó": "vichar",
    "vicia": "viciar",
    "viciaba": "viciar",
    "viciabais": "viciar",
    "viciaban": "viciar",
    "viciabas": "viciar",
    "viciad": "viciar",
    "viciada": "viciar",
    "viciadas": "viciar",
    "viciado": "viciar",
    "viciados": "viciar",
    "viciamos": "viciar",
    "vician": "viciar",
    "viciando": "viciar",
    "viciara": "viciar",
    "viciarais": "viciar",
    "viciaran": "viciar",
    "viciaras": "viciar",
    "viciare": "viciar",
    "viciareis": "viciar",
    "viciaremos": "viciar",
    "viciaren": "viciar",
    "viciares": "viciar",
    "viciaron": "viciar",
    "viciará": "viciar",
    "viciarán": "viciar",
    "viciarás": "viciar",
    "viciaré": "viciar",
    "viciaréis": "viciar",
    "viciaría": "viciar",
    "viciaríais": "viciar",
    "viciaríamos": "viciar",
    "viciarían": "viciar",
    "viciarías": "viciar",
    "vicias": "viciar",
    "viciase": "viciar",
    "viciaseis": "viciar",
    "viciasen": "viciar",
    "viciases": "viciar",
    "viciaste": "viciar",
    "viciasteis": "viciar",
    "vicie": "viciar",
    "viciemos": "viciar",
    "vicien": "viciar",
    "vicies": "viciar",
    "vicio": "viciar",
    "vicios": "vicio",
    "viciosa": "vicioso",
    "viciosas": "vicioso",
    "viciosos": "vicioso",
    "vicisitudes": "vicisitud",
    "vicisitudinaria": "vicisitudinario",
    "vicisitudinarias": "vicisitudinario",
    "vicisitudinarios": "vicisitudinario",
    "viciábamos": "viciar",
    "viciáis": "viciar",
    "viciáramos": "viciar",
    "viciáremos": "viciar",
    "viciásemos": "viciar",
    "vicié": "viciar",
    "viciéis": "viciar",
    "vició": "viciar",
    "victima": "victimar",
    "victimaba": "victimar",
    "victimabais": "victimar",
    "victimaban": "victimar",
    "victimabas": "victimar",
    "victimad": "victimar",
    "victimada": "victimar",
    "victimadas": "victimar",
    "victimado": "victimar",
    "victimados": "victimar",
    "victimamos": "victimar",
    "victiman": "victimar",
    "victimando": "victimar",
    "victimara": "victimar",
    "victimarais": "victimar",
    "victimaran": "victimar",
    "victimaras": "victimar",
    "victimare": "victimar",
    "victimareis": "victimar",
    "victimaremos": "victimar",
    "victimaren": "victimar",
    "victimares": "victimar",
    "victimarios": "victimario",
    "victimaron": "victimar",
    "victimará": "victimar",
    "victimarán": "victimar",
    "victimarás": "victimar",
    "victimaré": "victimar",
    "victimaréis": "victimar",
    "victimaría": "victimar",
    "victimaríais": "victimar",
    "victimaríamos": "victimar",
    "victimarían": "victimar",
    "victimarías": "victimar",
    "victimas": "victimar",
    "victimase": "victimar",
    "victimaseis": "victimar",
    "victimasen": "victimar",
    "victimases": "victimar",
    "victimaste": "victimar",
    "victimasteis": "victimar",
    "victime": "victimar",
    "victimemos": "victimar",
    "victimen": "victimar",
    "victimes": "victimar",
    "victimismos": "victimismo",
    "victimistas": "victimista",
    "victimizaciones": "victimización",
    "victimo": "victimar",
    "victimologías": "victimología",
    "victimábamos": "victimar",
    "victimáis": "victimar",
    "victimáramos": "victimar",
    "victimáremos": "victimar",
    "victimásemos": "victimar",
    "victimé": "victimar",
    "victiméis": "victimar",
    "victimó": "victimar",
    "victoriana": "victoriano",
    "victorianas": "victoriano",
    "victorianos": "victoriano",
    "victorias": "victoria",
    "victoriatos": "victoriato",
    "victoriosa": "victorioso",
    "victoriosas": "victorioso",
    "victoriosos": "victorioso",
    "vicuñas": "vicuña",
    "vicésima": "20",
    "vicésimas": "20",
    "vicésimo": "20",
    "vicésimos": "20",
    "vidas": "vida",
    "videntes": "vidente",
    "videoartes": "videoarte",
    "videocasetes": "videocasete",
    "videocassettes": "videocassette",
    "videocintas": "videocinta",
    "videoclips": "videoclip",
    "videoclubes": "videoclub",
    "videoclubs": "videoclub",
    "videoconferencias": "videoconferencia",
    "videocámaras": "videocámara",
    "videodiscos": "videodisco",
    "videofonías": "videofonía",
    "videofrecuencias": "videofrecuencia",
    "videografías": "videografía",
    "videojuegos": "videojuego",
    "videomagnética": "videomagnético",
    "videomagnéticas": "videomagnético",
    "videomagnéticos": "videomagnético",
    "videomarcadores": "videomarcador",
    "videomatones": "videomatón",
    "videomontajes": "videomontaje",
    "videorregistradores": "videorregistrador",
    "videotapes": "videotape",
    "videotarjetas": "videotarjeta",
    "videotecas": "videoteca",
    "videotecnologías": "videotecnología",
    "videotelefonías": "videotelefonía",
    "videoteléfonos": "videoteléfono",
    "videotexes": "videotex",
    "videotextos": "videotexto",
    "vides": "vid",
    "videófonos": "videófono",
    "vidiana": "vidiano",
    "vidianas": "vidiano",
    "vidianos": "vidiano",
    "vidicones": "vidicón",
    "vidorras": "vidorra",
    "vidria": "vidriar",
    "vidriaba": "vidriar",
    "vidriabais": "vidriar",
    "vidriaban": "vidriar",
    "vidriabas": "vidriar",
    "vidriad": "vidriar",
    "vidriada": "vidriar",
    "vidriadas": "vidriar",
    "vidriado": "vidriar",
    "vidriados": "vidriar",
    "vidriamos": "vidriar",
    "vidrian": "vidriar",
    "vidriando": "vidriar",
    "vidriara": "vidriar",
    "vidriarais": "vidriar",
    "vidriaran": "vidriar",
    "vidriaras": "vidriar",
    "vidriare": "vidriar",
    "vidriareis": "vidriar",
    "vidriaremos": "vidriar",
    "vidriaren": "vidriar",
    "vidriares": "vidriar",
    "vidriaron": "vidriar",
    "vidriará": "vidriar",
    "vidriarán": "vidriar",
    "vidriarás": "vidriar",
    "vidriaré": "vidriar",
    "vidriaréis": "vidriar",
    "vidriaría": "vidriar",
    "vidriaríais": "vidriar",
    "vidriaríamos": "vidriar",
    "vidriarían": "vidriar",
    "vidriarías": "vidriar",
    "vidrias": "vidriar",
    "vidriase": "vidriar",
    "vidriaseis": "vidriar",
    "vidriasen": "vidriar",
    "vidriases": "vidriar",
    "vidriaste": "vidriar",
    "vidriasteis": "vidriar",
    "vidrie": "vidriar",
    "vidriemos": "vidriar",
    "vidrien": "vidriar",
    "vidrieras": "vidriera",
    "vidrieros": "vidriero",
    "vidrierías": "vidriería",
    "vidries": "vidriar",
    "vidrio": "vidriar",
    "vidrios": "vidrio",
    "vidriosa": "vidrioso",
    "vidriosas": "vidrioso",
    "vidriosidades": "vidriosidad",
    "vidriosos": "vidrioso",
    "vidriábamos": "vidriar",
    "vidriáis": "vidriar",
    "vidriáramos": "vidriar",
    "vidriáremos": "vidriar",
    "vidriásemos": "vidriar",
    "vidrié": "vidriar",
    "vidriéis": "vidriar",
    "vidrió": "vidriar",
    "viduales": "vidual",
    "vieiras": "vieira",
    "vieja": "viejo",
    "viejas": "viejo",
    "viejecita": "viejecito",
    "viejecitas": "viejecito",
    "viejecitos": "viejecito",
    "viejos": "viejo",
    "viendo": "ver",
    "viene": "venir",
    "vienen": "venir",
    "vienes": "venir",
    "vienesa": "vienés",
    "vienesas": "vienés",
    "vieneses": "vienés",
    "vientos": "viento",
    "vientres": "vientre",
    "viera": "ver",
    "vierais": "ver",
    "vieran": "ver",
    "vieras": "ver",
    "viere": "ver",
    "viereis": "ver",
    "vieren": "ver",
    "vieres": "ver",
    "vieron": "ver",
    "vierta": "verter",
    "viertan": "verter",
    "viertas": "verter",
    "vierte": "verter",
    "vierteaguas": "vierteagua",
    "vierten": "verter",
    "viertes": "verter",
    "vierto": "verter",
    "viese": "ver",
    "vieseis": "ver",
    "viesen": "ver",
    "vieses": "ver",
    "vietnamitas": "vietnamita",
    "vigas": "viga",
    "vigencias": "vigencia",
    "vigentes": "vigente",
    "vigesimales": "vigesimal",
    "vigesimoctava": "28",
    "vigesimoctavas": "28",
    "vigesimoctavo": "28",
    "vigesimoctavos": "28",
    "vigesimocuarta": "24",
    "vigesimocuartas": "24",
    "vigesimocuarto": "24",
    "vigesimocuartos": "24",
    "vigesimonona": "29",
    "vigesimononas": "29",
    "vigesimonono": "29",
    "vigesimononos": "29",
    "vigesimonovena": "29",
    "vigesimonovenas": "29",
    "vigesimonoveno": "29",
    "vigesimonovenos": "29",
    "vigesimoprimera": "21",
    "vigesimoprimeras": "21",
    "vigesimoprimero": "21",
    "vigesimoprimeros": "21",
    "vigesimoquinta": "25",
    "vigesimoquintas": "25",
    "vigesimoquinto": "25",
    "vigesimoquintos": "25",
    "vigesimosegunda": "22",
    "vigesimosegundas": "22",
    "vigesimosegundo": "22",
    "vigesimosegundos": "22",
    "vigesimosexta": "26",
    "vigesimosextas": "26",
    "vigesimosexto": "26",
    "vigesimosextos": "26",
    "vigesimoséptima": "27",
    "vigesimoséptimas": "27",
    "vigesimoséptimo": "27",
    "vigesimoséptimos": "27",
    "vigesimotercera": "23",
    "vigesimoterceras": "23",
    "vigesimotercero": "23",
    "vigesimoterceros": "23",
    "vigila": "vigilar",
    "vigilaba": "vigilar",
    "vigilabais": "vigilar",
    "vigilaban": "vigilar",
    "vigilabas": "vigilar",
    "vigilad": "vigilar",
    "vigilada": "vigilar",
    "vigiladas": "vigilar",
    "vigilado": "vigilar",
    "vigilados": "vigilar",
    "vigilamos": "vigilar",
    "vigilan": "vigilar",
    "vigilancias": "vigilancia",
    "vigilando": "vigilar",
    "vigilantes": "vigilante",
    "vigilara": "vigilar",
    "vigilarais": "vigilar",
    "vigilaran": "vigilar",
    "vigilaras": "vigilar",
    "vigilare": "vigilar",
    "vigilareis": "vigilar",
    "vigilaremos": "vigilar",
    "vigilaren": "vigilar",
    "vigilares": "vigilar",
    "vigilaron": "vigilar",
    "vigilará": "vigilar",
    "vigilarán": "vigilar",
    "vigilarás": "vigilar",
    "vigilaré": "vigilar",
    "vigilaréis": "vigilar",
    "vigilaría": "vigilar",
    "vigilaríais": "vigilar",
    "vigilaríamos": "vigilar",
    "vigilarían": "vigilar",
    "vigilarías": "vigilar",
    "vigilas": "vigilar",
    "vigilase": "vigilar",
    "vigilaseis": "vigilar",
    "vigilasen": "vigilar",
    "vigilases": "vigilar",
    "vigilaste": "vigilar",
    "vigilasteis": "vigilar",
    "vigilativa": "vigilativo",
    "vigilativas": "vigilativo",
    "vigilativos": "vigilativo",
    "vigile": "vigilar",
    "vigilemos": "vigilar",
    "vigilen": "vigilar",
    "vigiles": "vigilar",
    "vigilias": "vigilia",
    "vigilo": "vigilar",
    "vigilábamos": "vigilar",
    "vigiláis": "vigilar",
    "vigiláramos": "vigilar",
    "vigiláremos": "vigilar",
    "vigilásemos": "vigilar",
    "vigilé": "vigilar",
    "vigiléis": "vigilar",
    "vigiló": "vigilar",
    "vigora": "vigorar",
    "vigoraba": "vigorar",
    "vigorabais": "vigorar",
    "vigoraban": "vigorar",
    "vigorabas": "vigorar",
    "vigorad": "vigorar",
    "vigorada": "vigorar",
    "vigoradas": "vigorar",
    "vigorado": "vigorar",
    "vigorados": "vigorar",
    "vigoramos": "vigorar",
    "vigoran": "vigorar",
    "vigorando": "vigorar",
    "vigorara": "vigorar",
    "vigorarais": "vigorar",
    "vigoraran": "vigorar",
    "vigoraras": "vigorar",
    "vigorare": "vigorar",
    "vigorareis": "vigorar",
    "vigoraremos": "vigorar",
    "vigoraren": "vigorar",
    "vigorares": "vigorar",
    "vigoraron": "vigorar",
    "vigorará": "vigorar",
    "vigorarán": "vigorar",
    "vigorarás": "vigorar",
    "vigoraré": "vigorar",
    "vigoraréis": "vigorar",
    "vigoraría": "vigorar",
    "vigoraríais": "vigorar",
    "vigoraríamos": "vigorar",
    "vigorarían": "vigorar",
    "vigorarías": "vigorar",
    "vigoras": "vigorar",
    "vigorase": "vigorar",
    "vigoraseis": "vigorar",
    "vigorasen": "vigorar",
    "vigorases": "vigorar",
    "vigoraste": "vigorar",
    "vigorasteis": "vigorar",
    "vigore": "vigorar",
    "vigoremos": "vigorar",
    "vigoren": "vigorar",
    "vigores": "vigorar",
    "vigorice": "vigorizar",
    "vigoricemos": "vigorizar",
    "vigoricen": "vigorizar",
    "vigorices": "vigorizar",
    "vigoricé": "vigorizar",
    "vigoricéis": "vigorizar",
    "vigoriza": "vigorizar",
    "vigorizaba": "vigorizar",
    "vigorizabais": "vigorizar",
    "vigorizaban": "vigorizar",
    "vigorizabas": "vigorizar",
    "vigorizad": "vigorizar",
    "vigorizada": "vigorizar",
    "vigorizadas": "vigorizar",
    "vigorizado": "vigorizar",
    "vigorizadora": "vigorizador",
    "vigorizadoras": "vigorizador",
    "vigorizadores": "vigorizador",
    "vigorizados": "vigorizar",
    "vigorizamos": "vigorizar",
    "vigorizan": "vigorizar",
    "vigorizando": "vigorizar",
    "vigorizantes": "vigorizante",
    "vigorizara": "vigorizar",
    "vigorizarais": "vigorizar",
    "vigorizaran": "vigorizar",
    "vigorizaras": "vigorizar",
    "vigorizare": "vigorizar",
    "vigorizareis": "vigorizar",
    "vigorizaremos": "vigorizar",
    "vigorizaren": "vigorizar",
    "vigorizares": "vigorizar",
    "vigorizaron": "vigorizar",
    "vigorizará": "vigorizar",
    "vigorizarán": "vigorizar",
    "vigorizarás": "vigorizar",
    "vigorizaré": "vigorizar",
    "vigorizaréis": "vigorizar",
    "vigorizaría": "vigorizar",
    "vigorizaríais": "vigorizar",
    "vigorizaríamos": "vigorizar",
    "vigorizarían": "vigorizar",
    "vigorizarías": "vigorizar",
    "vigorizas": "vigorizar",
    "vigorizase": "vigorizar",
    "vigorizaseis": "vigorizar",
    "vigorizasen": "vigorizar",
    "vigorizases": "vigorizar",
    "vigorizaste": "vigorizar",
    "vigorizasteis": "vigorizar",
    "vigorizo": "vigorizar",
    "vigorizábamos": "vigorizar",
    "vigorizáis": "vigorizar",
    "vigorizáramos": "vigorizar",
    "vigorizáremos": "vigorizar",
    "vigorizásemos": "vigorizar",
    "vigorizó": "vigorizar",
    "vigoro": "vigorar",
    "vigorosa": "vigoroso",
    "vigorosas": "vigoroso",
    "vigorosidades": "vigorosidad",
    "vigorosos": "vigoroso",
    "vigorábamos": "vigorar",
    "vigoráis": "vigorar",
    "vigoráramos": "vigorar",
    "vigoráremos": "vigorar",
    "vigorásemos": "vigorar",
    "vigoré": "vigorar",
    "vigoréis": "vigorar",
    "vigoró": "vigorar",
    "vigotas": "vigota",
    "viguerías": "viguería",
    "viguesa": "vigués",
    "viguesas": "vigués",
    "vigueses": "vigués",
    "viguetas": "vigueta",
    "vigésima": "20",
    "vigésimas": "20",
    "vigésimo": "20",
    "vigésimos": "20",
    "vigías": "vigía",
    "vihuelas": "vihuela",
    "vihuelistas": "vihuelista",
    "vikinga": "vikingo",
    "vikingas": "vikingo",
    "vikingos": "vikingo",
    "vilanos": "vilano",
    "viles": "vil",
    "vilezas": "vileza",
    "vilipendia": "vilipendiar",
    "vilipendiaba": "vilipendiar",
    "vilipendiabais": "vilipendiar",
    "vilipendiaban": "vilipendiar",
    "vilipendiabas": "vilipendiar",
    "vilipendiad": "vilipendiar",
    "vilipendiada": "vilipendiar",
    "vilipendiadas": "vilipendiar",
    "vilipendiado": "vilipendiar",
    "vilipendiadora": "vilipendiador",
    "vilipendiadoras": "vilipendiador",
    "vilipendiadores": "vilipendiador",
    "vilipendiados": "vilipendiar",
    "vilipendiamos": "vilipendiar",
    "vilipendian": "vilipendiar",
    "vilipendiando": "vilipendiar",
    "vilipendiara": "vilipendiar",
    "vilipendiarais": "vilipendiar",
    "vilipendiaran": "vilipendiar",
    "vilipendiaras": "vilipendiar",
    "vilipendiare": "vilipendiar",
    "vilipendiareis": "vilipendiar",
    "vilipendiaremos": "vilipendiar",
    "vilipendiaren": "vilipendiar",
    "vilipendiares": "vilipendiar",
    "vilipendiaron": "vilipendiar",
    "vilipendiará": "vilipendiar",
    "vilipendiarán": "vilipendiar",
    "vilipendiarás": "vilipendiar",
    "vilipendiaré": "vilipendiar",
    "vilipendiaréis": "vilipendiar",
    "vilipendiaría": "vilipendiar",
    "vilipendiaríais": "vilipendiar",
    "vilipendiaríamos": "vilipendiar",
    "vilipendiarían": "vilipendiar",
    "vilipendiarías": "vilipendiar",
    "vilipendias": "vilipendiar",
    "vilipendiase": "vilipendiar",
    "vilipendiaseis": "vilipendiar",
    "vilipendiasen": "vilipendiar",
    "vilipendiases": "vilipendiar",
    "vilipendiaste": "vilipendiar",
    "vilipendiasteis": "vilipendiar",
    "vilipendie": "vilipendiar",
    "vilipendiemos": "vilipendiar",
    "vilipendien": "vilipendiar",
    "vilipendies": "vilipendiar",
    "vilipendio": "vilipendiar",
    "vilipendios": "vilipendio",
    "vilipendiosa": "vilipendioso",
    "vilipendiosas": "vilipendioso",
    "vilipendiosos": "vilipendioso",
    "vilipendiábamos": "vilipendiar",
    "vilipendiáis": "vilipendiar",
    "vilipendiáramos": "vilipendiar",
    "vilipendiáremos": "vilipendiar",
    "vilipendiásemos": "vilipendiar",
    "vilipendié": "vilipendiar",
    "vilipendiéis": "vilipendiar",
    "vilipendió": "vilipendiar",
    "villana": "villano",
    "villanadas": "villanada",
    "villanajes": "villanaje",
    "villanas": "villano",
    "villancicos": "villancico",
    "villanciquera": "villanciquero",
    "villanciqueras": "villanciquero",
    "villanciqueros": "villanciquero",
    "villanerías": "villanería",
    "villanesca": "villanesco",
    "villanescas": "villanesco",
    "villanescos": "villanesco",
    "villanos": "villano",
    "villanías": "villanía",
    "villares": "villar",
    "villas": "villa",
    "villazgos": "villazgo",
    "villorines": "villorín",
    "villorrios": "villorrio",
    "villorías": "villoría",
    "vilorda": "vilordo",
    "vilordas": "vilordo",
    "vilordos": "vilordo",
    "vilortas": "vilorta",
    "vilortos": "vilorto",
    "viltrotea": "viltrotear",
    "viltroteaba": "viltrotear",
    "viltroteabais": "viltrotear",
    "viltroteaban": "viltrotear",
    "viltroteabas": "viltrotear",
    "viltrotead": "viltrotear",
    "viltroteada": "viltrotear",
    "viltroteadas": "viltrotear",
    "viltroteado": "viltrotear",
    "viltroteados": "viltrotear",
    "viltroteamos": "viltrotear",
    "viltrotean": "viltrotear",
    "viltroteando": "viltrotear",
    "viltroteara": "viltrotear",
    "viltrotearais": "viltrotear",
    "viltrotearan": "viltrotear",
    "viltrotearas": "viltrotear",
    "viltroteare": "viltrotear",
    "viltroteareis": "viltrotear",
    "viltrotearemos": "viltrotear",
    "viltrotearen": "viltrotear",
    "viltroteares": "viltrotear",
    "viltrotearon": "viltrotear",
    "viltroteará": "viltrotear",
    "viltrotearán": "viltrotear",
    "viltrotearás": "viltrotear",
    "viltrotearé": "viltrotear",
    "viltrotearéis": "viltrotear",
    "viltrotearía": "viltrotear",
    "viltrotearíais": "viltrotear",
    "viltrotearíamos": "viltrotear",
    "viltrotearían": "viltrotear",
    "viltrotearías": "viltrotear",
    "viltroteas": "viltrotear",
    "viltrotease": "viltrotear",
    "viltroteaseis": "viltrotear",
    "viltroteasen": "viltrotear",
    "viltroteases": "viltrotear",
    "viltroteaste": "viltrotear",
    "viltroteasteis": "viltrotear",
    "viltrotee": "viltrotear",
    "viltroteemos": "viltrotear",
    "viltroteen": "viltrotear",
    "viltrotees": "viltrotear",
    "viltroteo": "viltrotear",
    "viltroteábamos": "viltrotear",
    "viltroteáis": "viltrotear",
    "viltroteáramos": "viltrotear",
    "viltroteáremos": "viltrotear",
    "viltroteásemos": "viltrotear",
    "viltroteé": "viltrotear",
    "viltroteéis": "viltrotear",
    "viltroteó": "viltrotear",
    "vimbreras": "vimbrera",
    "vimbres": "vimbre",
    "vimos": "ver",
    "vinagradas": "vinagrada",
    "vinagreras": "vinagrera",
    "vinagreros": "vinagrero",
    "vinagres": "vinagre",
    "vinagretas": "vinagreta",
    "vinagrillos": "vinagrillo",
    "vinagrones": "vinagrón",
    "vinagrosa": "vinagroso",
    "vinagrosas": "vinagroso",
    "vinagrosos": "vinagroso",
    "vinajeras": "vinajera",
    "vinariega": "vinariego",
    "vinariegas": "vinariego",
    "vinariegos": "vinariego",
    "vinatera": "vinatero",
    "vinateras": "vinatero",
    "vinateros": "vinatero",
    "vinaterías": "vinatería",
    "vinazas": "vinaza",
    "vinazos": "vinazo",
    "vinbarbitales": "vinbarbital",
    "vinblastinas": "vinblastina",
    "vincapervincas": "vincapervinca",
    "vincula": "vincular",
    "vinculaba": "vincular",
    "vinculabais": "vincular",
    "vinculaban": "vincular",
    "vinculabas": "vincular",
    "vinculables": "vinculable",
    "vinculaciones": "vinculación",
    "vinculad": "vincular",
    "vinculada": "vincular",
    "vinculadas": "vincular",
    "vinculado": "vincular",
    "vinculados": "vincular",
    "vinculamos": "vincular",
    "vinculan": "vincular",
    "vinculando": "vincular",
    "vinculantes": "vinculante",
    "vinculara": "vincular",
    "vincularais": "vincular",
    "vincularan": "vincular",
    "vincularas": "vincular",
    "vinculare": "vincular",
    "vinculareis": "vincular",
    "vincularemos": "vincular",
    "vincularen": "vincular",
    "vinculares": "vincular",
    "vincularon": "vincular",
    "vinculará": "vincular",
    "vincularán": "vincular",
    "vincularás": "vincular",
    "vincularé": "vincular",
    "vincularéis": "vincular",
    "vincularía": "vincular",
    "vincularíais": "vincular",
    "vincularíamos": "vincular",
    "vincularían": "vincular",
    "vincularías": "vincular",
    "vinculas": "vincular",
    "vinculase": "vincular",
    "vinculaseis": "vincular",
    "vinculasen": "vincular",
    "vinculases": "vincular",
    "vinculaste": "vincular",
    "vinculasteis": "vincular",
    "vincule": "vincular",
    "vinculemos": "vincular",
    "vinculen": "vincular",
    "vincules": "vincular",
    "vinculo": "vincular",
    "vinculábamos": "vincular",
    "vinculáis": "vincular",
    "vinculáramos": "vincular",
    "vinculáremos": "vincular",
    "vinculásemos": "vincular",
    "vinculé": "vincular",
    "vinculéis": "vincular",
    "vinculó": "vincular",
    "vindica": "vindicar",
    "vindicaba": "vindicar",
    "vindicabais": "vindicar",
    "vindicaban": "vindicar",
    "vindicabas": "vindicar",
    "vindicaciones": "vindicación",
    "vindicad": "vindicar",
    "vindicada": "vindicar",
    "vindicadas": "vindicar",
    "vindicado": "vindicar",
    "vindicadora": "vindicador",
    "vindicadoras": "vindicador",
    "vindicadores": "vindicador",
    "vindicados": "vindicar",
    "vindicamos": "vindicar",
    "vindican": "vindicar",
    "vindicando": "vindicar",
    "vindicara": "vindicar",
    "vindicarais": "vindicar",
    "vindicaran": "vindicar",
    "vindicaras": "vindicar",
    "vindicare": "vindicar",
    "vindicareis": "vindicar",
    "vindicaremos": "vindicar",
    "vindicaren": "vindicar",
    "vindicares": "vindicar",
    "vindicaron": "vindicar",
    "vindicará": "vindicar",
    "vindicarán": "vindicar",
    "vindicarás": "vindicar",
    "vindicaré": "vindicar",
    "vindicaréis": "vindicar",
    "vindicaría": "vindicar",
    "vindicaríais": "vindicar",
    "vindicaríamos": "vindicar",
    "vindicarían": "vindicar",
    "vindicarías": "vindicar",
    "vindicas": "vindicar",
    "vindicase": "vindicar",
    "vindicaseis": "vindicar",
    "vindicasen": "vindicar",
    "vindicases": "vindicar",
    "vindicaste": "vindicar",
    "vindicasteis": "vindicar",
    "vindicativa": "vindicativo",
    "vindicativas": "vindicativo",
    "vindicativos": "vindicativo",
    "vindicatoria": "vindicatorio",
    "vindicatorias": "vindicatorio",
    "vindicatorios": "vindicatorio",
    "vindico": "vindicar",
    "vindictas": "vindicta",
    "vindicábamos": "vindicar",
    "vindicáis": "vindicar",
    "vindicáramos": "vindicar",
    "vindicáremos": "vindicar",
    "vindicásemos": "vindicar",
    "vindicó": "vindicar",
    "vindique": "vindicar",
    "vindiquemos": "vindicar",
    "vindiquen": "vindicar",
    "vindiques": "vindicar",
    "vindiqué": "vindicar",
    "vindiquéis": "vindicar",
    "vine": "venir",
    "vineales": "vineal",
    "vinicultora": "vinicultor",
    "vinicultoras": "vinicultor",
    "vinicultores": "vinicultor",
    "viniculturas": "vinicultura",
    "viniendo": "venir",
    "viniera": "venir",
    "vinierais": "venir",
    "vinieran": "venir",
    "vinieras": "venir",
    "viniere": "venir",
    "viniereis": "venir",
    "vinieren": "venir",
    "vinieres": "venir",
    "vinieron": "venir",
    "viniese": "venir",
    "vinieseis": "venir",
    "viniesen": "venir",
    "vinieses": "venir",
    "vinificaciones": "vinificación",
    "vinilacetilenos": "vinilacetileno",
    "vinilaciones": "vinilación",
    "vinilcarbazoles": "vinilcarbazol",
    "vinillos": "vinillo",
    "vinilos": "vinilo",
    "vinimos": "venir",
    "viniste": "venir",
    "vinisteis": "venir",
    "viniéramos": "venir",
    "viniéremos": "venir",
    "viniésemos": "venir",
    "vino": "venir",
    "vinolencias": "vinolencia",
    "vinolenta": "vinolento",
    "vinolentas": "vinolento",
    "vinolentos": "vinolento",
    "vinos": "vino",
    "vinosa": "vinoso",
    "vinosas": "vinoso",
    "vinosidades": "vinosidad",
    "vinosos": "vinoso",
    "vinotes": "vinote",
    "vinícolas": "vinícola",
    "vinífera": "vinífero",
    "viníferas": "vinífero",
    "viníferos": "vinífero",
    "vinílica": "vinílico",
    "vinílicas": "vinílico",
    "vinílicos": "vinílico",
    "vio": "ver",
    "viola": "violar",
    "violaba": "violar",
    "violabais": "violar",
    "violaban": "violar",
    "violabas": "violar",
    "violabilidades": "violabilidad",
    "violaciones": "violación",
    "violad": "violar",
    "violada": "violar",
    "violadas": "violar",
    "violado": "violar",
    "violadora": "violador",
    "violadoras": "violador",
    "violadores": "violador",
    "violados": "violar",
    "violamos": "violar",
    "violan": "violar",
    "violando": "violar",
    "violanes": "violán",
    "violara": "violar",
    "violarais": "violar",
    "violaran": "violar",
    "violaras": "violar",
    "violare": "violar",
    "violareis": "violar",
    "violaremos": "violar",
    "violaren": "violar",
    "violares": "violar",
    "violarios": "violario",
    "violaron": "violar",
    "violará": "violar",
    "violarán": "violar",
    "violarás": "violar",
    "violaré": "violar",
    "violaréis": "violar",
    "violaría": "violar",
    "violaríais": "violar",
    "violaríamos": "violar",
    "violarían": "violar",
    "violarías": "violar",
    "violas": "violar",
    "violase": "violar",
    "violaseis": "violar",
    "violasen": "violar",
    "violases": "violar",
    "violaste": "violar",
    "violasteis": "violar",
    "violatoria": "violatorio",
    "violatorias": "violatorio",
    "violatorios": "violatorio",
    "viole": "violar",
    "violemos": "violar",
    "violen": "violar",
    "violencias": "violencia",
    "violenta": "violento",
    "violentaba": "violentar",
    "violentabais": "violentar",
    "violentaban": "violentar",
    "violentabas": "violentar",
    "violentad": "violentar",
    "violentada": "violentar",
    "violentadas": "violentar",
    "violentado": "violentar",
    "violentados": "violentar",
    "violentamos": "violentar",
    "violentan": "violentar",
    "violentando": "violentar",
    "violentara": "violentar",
    "violentarais": "violentar",
    "violentaran": "violentar",
    "violentaras": "violentar",
    "violentare": "violentar",
    "violentareis": "violentar",
    "violentaremos": "violentar",
    "violentaren": "violentar",
    "violentares": "violentar",
    "violentaron": "violentar",
    "violentará": "violentar",
    "violentarán": "violentar",
    "violentarás": "violentar",
    "violentaré": "violentar",
    "violentaréis": "violentar",
    "violentaría": "violentar",
    "violentaríais": "violentar",
    "violentaríamos": "violentar",
    "violentarían": "violentar",
    "violentarías": "violentar",
    "violentas": "violento",
    "violentase": "violentar",
    "violentaseis": "violentar",
    "violentasen": "violentar",
    "violentases": "violentar",
    "violentaste": "violentar",
    "violentasteis": "violentar",
    "violente": "violentar",
    "violentemos": "violentar",
    "violenten": "violentar",
    "violentes": "violentar",
    "violento": "violentar",
    "violentos": "violento",
    "violentábamos": "violentar",
    "violentáis": "violentar",
    "violentáramos": "violentar",
    "violentáremos": "violentar",
    "violentásemos": "violentar",
    "violenté": "violentar",
    "violentéis": "violentar",
    "violentó": "violentar",
    "violera": "violero",
    "violeras": "violero",
    "violeros": "violero",
    "violes": "violar",
    "violetas": "violeta",
    "violeteras": "violetera",
    "violeterismos": "violeterismo",
    "violeteros": "violetero",
    "violines": "violín",
    "violinistas": "violinista",
    "violistas": "violista",
    "violles": "violle",
    "violo": "violar",
    "violoncelistas": "violoncelista",
    "violoncelos": "violoncelo",
    "violonchelistas": "violonchelista",
    "violonchelos": "violonchelo",
    "violones": "violón",
    "violábamos": "violar",
    "violácea": "violáceo",
    "violáceas": "violáceo",
    "violáceos": "violáceo",
    "violáis": "violar",
    "violáramos": "violar",
    "violáremos": "violar",
    "violásemos": "violar",
    "violé": "violar",
    "violéis": "violar",
    "violó": "violar",
    "viomicinas": "viomicina",
    "viones": "vion",
    "viperina": "viperino",
    "viperinas": "viperino",
    "viperinos": "viperino",
    "vipérea": "vipéreo",
    "vipéreas": "vipéreo",
    "vipéreos": "vipéreo",
    "vipéridos": "vipérido",
    "vira": "virar",
    "viraba": "virar",
    "virabais": "virar",
    "viraban": "virar",
    "virabas": "virar",
    "viracochas": "viracocha",
    "virad": "virar",
    "virada": "virar",
    "viradas": "virar",
    "virado": "virar",
    "viradores": "virador",
    "virados": "virar",
    "viragos": "virago",
    "virajes": "viraje",
    "virales": "viral",
    "viramos": "virar",
    "viran": "virar",
    "virando": "virar",
    "virara": "virar",
    "virarais": "virar",
    "viraran": "virar",
    "viraras": "virar",
    "virare": "virar",
    "virareis": "virar",
    "viraremos": "virar",
    "viraren": "virar",
    "virares": "virar",
    "viraron": "virar",
    "virará": "virar",
    "virarán": "virar",
    "virarás": "virar",
    "viraré": "virar",
    "viraréis": "virar",
    "viraría": "virar",
    "viraríais": "virar",
    "viraríamos": "virar",
    "virarían": "virar",
    "virarías": "virar",
    "viras": "virar",
    "virase": "virar",
    "viraseis": "virar",
    "virasen": "virar",
    "virases": "virar",
    "viraste": "virar",
    "virasteis": "virar",
    "viratones": "viratón",
    "viraviras": "viravira",
    "virazones": "virazón",
    "vire": "virar",
    "viremos": "virar",
    "viren": "virar",
    "vires": "virar",
    "virescencias": "virescencia",
    "virgenes": "virgen",
    "virgiliana": "virgiliano",
    "virgilianas": "virgiliano",
    "virgilianos": "virgiliano",
    "virginales": "virginal",
    "virginiana": "virginiano",
    "virginianas": "virginiano",
    "virginianos": "virginiano",
    "virginias": "virginia",
    "virginidades": "virginidad",
    "virgos": "virgo",
    "virguera": "virguero",
    "virgueras": "virguero",
    "virgueros": "virguero",
    "virguerías": "virguería",
    "virgulillas": "virgulilla",
    "virgínea": "virgíneo",
    "virgíneas": "virgíneo",
    "virgíneos": "virgíneo",
    "viricidas": "viricida",
    "viriles": "viril",
    "virilidades": "virilidad",
    "virilismos": "virilismo",
    "virilizaciones": "virilización",
    "virilocales": "virilocal",
    "viriones": "virión",
    "viripotentes": "viripotente",
    "viro": "virar",
    "virogenética": "virogenético",
    "virogenéticas": "virogenético",
    "virogenéticos": "virogenético",
    "virola": "virolo",
    "virolada": "virolado",
    "viroladas": "virolado",
    "virolados": "virolado",
    "virolas": "virolo",
    "virolenta": "virolento",
    "virolentas": "virolento",
    "virolentos": "virolento",
    "virologías": "virología",
    "virolos": "virolo",
    "virotazos": "virotazo",
    "virotes": "virote",
    "virotillos": "virotillo",
    "virotismos": "virotismo",
    "virreinales": "virreinal",
    "virreinas": "virreina",
    "virreinatos": "virreinato",
    "virreinos": "virreino",
    "virreyes": "virrey",
    "virtuales": "virtual",
    "virtualidades": "virtualidad",
    "virtualizaciones": "virtualización",
    "virtudes": "virtud",
    "virtuosa": "virtuoso",
    "virtuosas": "virtuoso",
    "virtuosismos": "virtuosismo",
    "virtuosos": "virtuoso",
    "viruelas": "viruela",
    "virulencias": "virulencia",
    "virulenta": "virulento",
    "virulentas": "virulento",
    "virulentos": "virulento",
    "virulífera": "virulífero",
    "virulíferas": "virulífero",
    "virulíferos": "virulífero",
    "virutas": "viruta",
    "virábamos": "virar",
    "viráis": "virar",
    "viráramos": "virar",
    "viráremos": "virar",
    "virásemos": "virar",
    "viré": "virar",
    "viréis": "virar",
    "viró": "virar",
    "viróloga": "virólogo",
    "virólogas": "virólogo",
    "virólogos": "virólogo",
    "virótica": "virótico",
    "viróticas": "virótico",
    "viróticos": "virótico",
    "visa": "visar",
    "visaba": "visar",
    "visabais": "visar",
    "visaban": "visar",
    "visabas": "visar",
    "visad": "visar",
    "visada": "visar",
    "visadas": "visar",
    "visado": "visar",
    "visados": "visar",
    "visajes": "visaje",
    "visamos": "visar",
    "visan": "visar",
    "visando": "visar",
    "visara": "visar",
    "visarais": "visar",
    "visaran": "visar",
    "visaras": "visar",
    "visare": "visar",
    "visareis": "visar",
    "visaremos": "visar",
    "visaren": "visar",
    "visares": "visar",
    "visaron": "visar",
    "visará": "visar",
    "visarán": "visar",
    "visarás": "visar",
    "visaré": "visar",
    "visaréis": "visar",
    "visaría": "visar",
    "visaríais": "visar",
    "visaríamos": "visar",
    "visarían": "visar",
    "visarías": "visar",
    "visas": "visar",
    "visase": "visar",
    "visaseis": "visar",
    "visasen": "visar",
    "visases": "visar",
    "visaste": "visar",
    "visasteis": "visar",
    "viscerales": "visceral",
    "viscoelasticidades": "viscoelasticidad",
    "viscoelástica": "viscoelástico",
    "viscoelásticas": "viscoelástico",
    "viscoelásticos": "viscoelástico",
    "viscos": "visco",
    "viscosa": "viscoso",
    "viscosas": "viscoso",
    "viscosidades": "viscosidad",
    "viscosillas": "viscosilla",
    "viscosimetrías": "viscosimetría",
    "viscosos": "viscoso",
    "viscosímetros": "viscosímetro",
    "vise": "visar",
    "visemos": "visar",
    "visen": "visar",
    "viseras": "visera",
    "vises": "visar",
    "visibilidades": "visibilidad",
    "visibles": "visible",
    "visigoda": "visigodo",
    "visigodas": "visigodo",
    "visigodos": "visigodo",
    "visigótica": "visigótico",
    "visigóticas": "visigótico",
    "visigóticos": "visigótico",
    "visillos": "visillo",
    "visiona": "visionar",
    "visionaba": "visionar",
    "visionabais": "visionar",
    "visionaban": "visionar",
    "visionabas": "visionar",
    "visionad": "visionar",
    "visionada": "visionar",
    "visionadas": "visionar",
    "visionado": "visionar",
    "visionados": "visionar",
    "visionamos": "visionar",
    "visionan": "visionar",
    "visionando": "visionar",
    "visionara": "visionar",
    "visionarais": "visionar",
    "visionaran": "visionar",
    "visionaras": "visionar",
    "visionare": "visionar",
    "visionareis": "visionar",
    "visionaremos": "visionar",
    "visionaren": "visionar",
    "visionares": "visionar",
    "visionaria": "visionario",
    "visionarias": "visionario",
    "visionarios": "visionario",
    "visionaron": "visionar",
    "visionará": "visionar",
    "visionarán": "visionar",
    "visionarás": "visionar",
    "visionaré": "visionar",
    "visionaréis": "visionar",
    "visionaría": "visionar",
    "visionaríais": "visionar",
    "visionaríamos": "visionar",
    "visionarían": "visionar",
    "visionarías": "visionar",
    "visionas": "visionar",
    "visionase": "visionar",
    "visionaseis": "visionar",
    "visionasen": "visionar",
    "visionases": "visionar",
    "visionaste": "visionar",
    "visionasteis": "visionar",
    "visione": "visionar",
    "visionemos": "visionar",
    "visionen": "visionar",
    "visiones": "visionar",
    "visiono": "visionar",
    "visionábamos": "visionar",
    "visionáis": "visionar",
    "visionáramos": "visionar",
    "visionáremos": "visionar",
    "visionásemos": "visionar",
    "visioné": "visionar",
    "visionéis": "visionar",
    "visionó": "visionar",
    "visiratos": "visirato",
    "visires": "visir",
    "visita": "visitar",
    "visitaba": "visitar",
    "visitabais": "visitar",
    "visitaban": "visitar",
    "visitabas": "visitar",
    "visitables": "visitable",
    "visitaciones": "visitación",
    "visitad": "visitar",
    "visitada": "visitar",
    "visitadas": "visitar",
    "visitado": "visitar",
    "visitadora": "visitador",
    "visitadoras": "visitador",
    "visitadores": "visitador",
    "visitados": "visitar",
    "visitamos": "visitar",
    "visitan": "visitar",
    "visitando": "visitar",
    "visitantes": "visitante",
    "visitara": "visitar",
    "visitarais": "visitar",
    "visitaran": "visitar",
    "visitaras": "visitar",
    "visitare": "visitar",
    "visitareis": "visitar",
    "visitaremos": "visitar",
    "visitaren": "visitar",
    "visitares": "visitar",
    "visitaron": "visitar",
    "visitará": "visitar",
    "visitarán": "visitar",
    "visitarás": "visitar",
    "visitaré": "visitar",
    "visitaréis": "visitar",
    "visitaría": "visitar",
    "visitaríais": "visitar",
    "visitaríamos": "visitar",
    "visitarían": "visitar",
    "visitarías": "visitar",
    "visitas": "visitar",
    "visitase": "visitar",
    "visitaseis": "visitar",
    "visitasen": "visitar",
    "visitases": "visitar",
    "visitaste": "visitar",
    "visitasteis": "visitar",
    "visite": "visitar",
    "visitemos": "visitar",
    "visiten": "visitar",
    "visiteos": "visiteo",
    "visitera": "visitero",
    "visiteras": "visitero",
    "visiteros": "visitero",
    "visites": "visitar",
    "visito": "visitar",
    "visitones": "visitón",
    "visitábamos": "visitar",
    "visitáis": "visitar",
    "visitáramos": "visitar",
    "visitáremos": "visitar",
    "visitásemos": "visitar",
    "visité": "visitar",
    "visitéis": "visitar",
    "visitó": "visitar",
    "visiva": "visivo",
    "visivas": "visivo",
    "visivos": "visivo",
    "vislumbra": "vislumbrar",
    "vislumbraba": "vislumbrar",
    "vislumbrabais": "vislumbrar",
    "vislumbraban": "vislumbrar",
    "vislumbrabas": "vislumbrar",
    "vislumbrad": "vislumbrar",
    "vislumbrada": "vislumbrar",
    "vislumbradas": "vislumbrar",
    "vislumbrado": "vislumbrar",
    "vislumbrados": "vislumbrar",
    "vislumbramos": "vislumbrar",
    "vislumbran": "vislumbrar",
    "vislumbrando": "vislumbrar",
    "vislumbrara": "vislumbrar",
    "vislumbrarais": "vislumbrar",
    "vislumbraran": "vislumbrar",
    "vislumbraras": "vislumbrar",
    "vislumbrare": "vislumbrar",
    "vislumbrareis": "vislumbrar",
    "vislumbraremos": "vislumbrar",
    "vislumbraren": "vislumbrar",
    "vislumbrares": "vislumbrar",
    "vislumbraron": "vislumbrar",
    "vislumbrará": "vislumbrar",
    "vislumbrarán": "vislumbrar",
    "vislumbrarás": "vislumbrar",
    "vislumbraré": "vislumbrar",
    "vislumbraréis": "vislumbrar",
    "vislumbraría": "vislumbrar",
    "vislumbraríais": "vislumbrar",
    "vislumbraríamos": "vislumbrar",
    "vislumbrarían": "vislumbrar",
    "vislumbrarías": "vislumbrar",
    "vislumbras": "vislumbrar",
    "vislumbrase": "vislumbrar",
    "vislumbraseis": "vislumbrar",
    "vislumbrasen": "vislumbrar",
    "vislumbrases": "vislumbrar",
    "vislumbraste": "vislumbrar",
    "vislumbrasteis": "vislumbrar",
    "vislumbre": "vislumbrar",
    "vislumbremos": "vislumbrar",
    "vislumbren": "vislumbrar",
    "vislumbres": "vislumbre",
    "vislumbro": "vislumbrar",
    "vislumbrábamos": "vislumbrar",
    "vislumbráis": "vislumbrar",
    "vislumbráramos": "vislumbrar",
    "vislumbráremos": "vislumbrar",
    "vislumbrásemos": "vislumbrar",
    "vislumbré": "vislumbrar",
    "vislumbréis": "vislumbrar",
    "vislumbró": "vislumbrar",
    "visnuismos": "visnuismo",
    "viso": "visar",
    "visones": "visón",
    "visora": "visor",
    "visoras": "visor",
    "visores": "visor",
    "visoria": "visorio",
    "visorias": "visorio",
    "visorios": "visorio",
    "visos": "viso",
    "vista": "vestir",
    "vistamos": "vestir",
    "vistan": "vestir",
    "vistas": "vista",
    "vistazos": "vistazo",
    "viste": "vestir",
    "visteis": "ver",
    "visten": "vestir",
    "vistes": "vestir",
    "vistiendo": "vestir",
    "vistiera": "vestir",
    "vistierais": "vestir",
    "vistieran": "vestir",
    "vistieras": "vestir",
    "vistiere": "vestir",
    "vistiereis": "vestir",
    "vistieren": "vestir",
    "vistieres": "vestir",
    "vistieron": "vestir",
    "vistiese": "vestir",
    "vistieseis": "vestir",
    "vistiesen": "vestir",
    "vistieses": "vestir",
    "vistiéramos": "vestir",
    "vistiéremos": "vestir",
    "vistiésemos": "vestir",
    "vistió": "vestir",
    "visto": "vestir",
    "vistobuenos": "vistobueno",
    "vistos": "ver",
    "vistosa": "vistoso",
    "vistosas": "vistoso",
    "vistosidades": "vistosidad",
    "vistosos": "vistoso",
    "vistáis": "vestir",
    "visuales": "visual",
    "visualice": "visualizar",
    "visualicemos": "visualizar",
    "visualicen": "visualizar",
    "visualices": "visualizar",
    "visualicé": "visualizar",
    "visualicéis": "visualizar",
    "visualidades": "visualidad",
    "visualiza": "visualizar",
    "visualizaba": "visualizar",
    "visualizabais": "visualizar",
    "visualizaban": "visualizar",
    "visualizabas": "visualizar",
    "visualizables": "visualizable",
    "visualizaciones": "visualización",
    "visualizad": "visualizar",
    "visualizada": "visualizar",
    "visualizadas": "visualizar",
    "visualizado": "visualizar",
    "visualizadora": "visualizador",
    "visualizadoras": "visualizador",
    "visualizadores": "visualizador",
    "visualizados": "visualizar",
    "visualizamos": "visualizar",
    "visualizan": "visualizar",
    "visualizando": "visualizar",
    "visualizara": "visualizar",
    "visualizarais": "visualizar",
    "visualizaran": "visualizar",
    "visualizaras": "visualizar",
    "visualizare": "visualizar",
    "visualizareis": "visualizar",
    "visualizaremos": "visualizar",
    "visualizaren": "visualizar",
    "visualizares": "visualizar",
    "visualizaron": "visualizar",
    "visualizará": "visualizar",
    "visualizarán": "visualizar",
    "visualizarás": "visualizar",
    "visualizaré": "visualizar",
    "visualizaréis": "visualizar",
    "visualizaría": "visualizar",
    "visualizaríais": "visualizar",
    "visualizaríamos": "visualizar",
    "visualizarían": "visualizar",
    "visualizarías": "visualizar",
    "visualizas": "visualizar",
    "visualizase": "visualizar",
    "visualizaseis": "visualizar",
    "visualizasen": "visualizar",
    "visualizases": "visualizar",
    "visualizaste": "visualizar",
    "visualizasteis": "visualizar",
    "visualizo": "visualizar",
    "visualizábamos": "visualizar",
    "visualizáis": "visualizar",
    "visualizáramos": "visualizar",
    "visualizáremos": "visualizar",
    "visualizásemos": "visualizar",
    "visualizó": "visualizar",
    "visuras": "visura",
    "visuscopios": "visuscopio",
    "visábamos": "visar",
    "visáis": "visar",
    "visáramos": "visar",
    "visáremos": "visar",
    "visásemos": "visar",
    "visé": "visar",
    "viséis": "visar",
    "visó": "visar",
    "vitales": "vital",
    "vitalice": "vitalizar",
    "vitalicemos": "vitalizar",
    "vitalicen": "vitalizar",
    "vitalices": "vitalizar",
    "vitalicia": "vitalicio",
    "vitalicias": "vitalicio",
    "vitalicios": "vitalicio",
    "vitalicistas": "vitalicista",
    "vitalicé": "vitalizar",
    "vitalicéis": "vitalizar",
    "vitalidades": "vitalidad",
    "vitalismos": "vitalismo",
    "vitalistas": "vitalista",
    "vitaliza": "vitalizar",
    "vitalizaba": "vitalizar",
    "vitalizabais": "vitalizar",
    "vitalizaban": "vitalizar",
    "vitalizabas": "vitalizar",
    "vitalizad": "vitalizar",
    "vitalizada": "vitalizar",
    "vitalizadas": "vitalizar",
    "vitalizado": "vitalizar",
    "vitalizados": "vitalizar",
    "vitalizamos": "vitalizar",
    "vitalizan": "vitalizar",
    "vitalizando": "vitalizar",
    "vitalizara": "vitalizar",
    "vitalizarais": "vitalizar",
    "vitalizaran": "vitalizar",
    "vitalizaras": "vitalizar",
    "vitalizare": "vitalizar",
    "vitalizareis": "vitalizar",
    "vitalizaremos": "vitalizar",
    "vitalizaren": "vitalizar",
    "vitalizares": "vitalizar",
    "vitalizaron": "vitalizar",
    "vitalizará": "vitalizar",
    "vitalizarán": "vitalizar",
    "vitalizarás": "vitalizar",
    "vitalizaré": "vitalizar",
    "vitalizaréis": "vitalizar",
    "vitalizaría": "vitalizar",
    "vitalizaríais": "vitalizar",
    "vitalizaríamos": "vitalizar",
    "vitalizarían": "vitalizar",
    "vitalizarías": "vitalizar",
    "vitalizas": "vitalizar",
    "vitalizase": "vitalizar",
    "vitalizaseis": "vitalizar",
    "vitalizasen": "vitalizar",
    "vitalizases": "vitalizar",
    "vitalizaste": "vitalizar",
    "vitalizasteis": "vitalizar",
    "vitalizo": "vitalizar",
    "vitalizábamos": "vitalizar",
    "vitalizáis": "vitalizar",
    "vitalizáramos": "vitalizar",
    "vitalizáremos": "vitalizar",
    "vitalizásemos": "vitalizar",
    "vitalizó": "vitalizar",
    "vitaminada": "vitaminado",
    "vitaminadas": "vitaminado",
    "vitaminados": "vitaminado",
    "vitaminas": "vitamina",
    "vitamínica": "vitamínico",
    "vitamínicas": "vitamínico",
    "vitamínicos": "vitamínico",
    "vitanda": "vitando",
    "vitandas": "vitando",
    "vitandos": "vitando",
    "vitelas": "vitela",
    "vitelina": "vitelino",
    "vitelinas": "vitelino",
    "vitelinos": "vitelino",
    "vitelos": "vitelo",
    "viticultora": "viticultor",
    "viticultoras": "viticultor",
    "viticultores": "viticultor",
    "viticulturas": "viticultura",
    "vitivinicultora": "vitivinicultor",
    "vitivinicultoras": "vitivinicultor",
    "vitivinicultores": "vitivinicultor",
    "vitiviniculturas": "vitivinicultura",
    "vitivinícolas": "vitivinícola",
    "vitolas": "vitola",
    "vitorea": "vitorear",
    "vitoreaba": "vitorear",
    "vitoreabais": "vitorear",
    "vitoreaban": "vitorear",
    "vitoreabas": "vitorear",
    "vitoread": "vitorear",
    "vitoreada": "vitorear",
    "vitoreadas": "vitorear",
    "vitoreado": "vitorear",
    "vitoreados": "vitorear",
    "vitoreamos": "vitorear",
    "vitorean": "vitorear",
    "vitoreando": "vitorear",
    "vitoreara": "vitorear",
    "vitorearais": "vitorear",
    "vitorearan": "vitorear",
    "vitorearas": "vitorear",
    "vitoreare": "vitorear",
    "vitoreareis": "vitorear",
    "vitorearemos": "vitorear",
    "vitorearen": "vitorear",
    "vitoreares": "vitorear",
    "vitorearon": "vitorear",
    "vitoreará": "vitorear",
    "vitorearán": "vitorear",
    "vitorearás": "vitorear",
    "vitorearé": "vitorear",
    "vitorearéis": "vitorear",
    "vitorearía": "vitorear",
    "vitorearíais": "vitorear",
    "vitorearíamos": "vitorear",
    "vitorearían": "vitorear",
    "vitorearías": "vitorear",
    "vitoreas": "vitorear",
    "vitorease": "vitorear",
    "vitoreaseis": "vitorear",
    "vitoreasen": "vitorear",
    "vitoreases": "vitorear",
    "vitoreaste": "vitorear",
    "vitoreasteis": "vitorear",
    "vitoree": "vitorear",
    "vitoreemos": "vitorear",
    "vitoreen": "vitorear",
    "vitorees": "vitorear",
    "vitoreo": "vitorear",
    "vitoreábamos": "vitorear",
    "vitoreáis": "vitorear",
    "vitoreáramos": "vitorear",
    "vitoreáremos": "vitorear",
    "vitoreásemos": "vitorear",
    "vitoreé": "vitorear",
    "vitoreéis": "vitorear",
    "vitoreó": "vitorear",
    "vitoriana": "vitoriano",
    "vitorianas": "vitoriano",
    "vitorianos": "vitoriano",
    "vitos": "vito",
    "vitrales": "vitral",
    "vitrectomías": "vitrectomía",
    "vitreorretiniana": "vitreorretiniano",
    "vitreorretinianas": "vitreorretiniano",
    "vitreorretinianos": "vitreorretiniano",
    "vitres": "vitre",
    "vitrifica": "vitrificar",
    "vitrificaba": "vitrificar",
    "vitrificabais": "vitrificar",
    "vitrificaban": "vitrificar",
    "vitrificabas": "vitrificar",
    "vitrificables": "vitrificable",
    "vitrificaciones": "vitrificación",
    "vitrificad": "vitrificar",
    "vitrificada": "vitrificar",
    "vitrificadas": "vitrificar",
    "vitrificado": "vitrificar",
    "vitrificados": "vitrificar",
    "vitrificamos": "vitrificar",
    "vitrifican": "vitrificar",
    "vitrificando": "vitrificar",
    "vitrificara": "vitrificar",
    "vitrificarais": "vitrificar",
    "vitrificaran": "vitrificar",
    "vitrificaras": "vitrificar",
    "vitrificare": "vitrificar",
    "vitrificareis": "vitrificar",
    "vitrificaremos": "vitrificar",
    "vitrificaren": "vitrificar",
    "vitrificares": "vitrificar",
    "vitrificaron": "vitrificar",
    "vitrificará": "vitrificar",
    "vitrificarán": "vitrificar",
    "vitrificarás": "vitrificar",
    "vitrificaré": "vitrificar",
    "vitrificaréis": "vitrificar",
    "vitrificaría": "vitrificar",
    "vitrificaríais": "vitrificar",
    "vitrificaríamos": "vitrificar",
    "vitrificarían": "vitrificar",
    "vitrificarías": "vitrificar",
    "vitrificas": "vitrificar",
    "vitrificase": "vitrificar",
    "vitrificaseis": "vitrificar",
    "vitrificasen": "vitrificar",
    "vitrificases": "vitrificar",
    "vitrificaste": "vitrificar",
    "vitrificasteis": "vitrificar",
    "vitrifico": "vitrificar",
    "vitrificábamos": "vitrificar",
    "vitrificáis": "vitrificar",
    "vitrificáramos": "vitrificar",
    "vitrificáremos": "vitrificar",
    "vitrificásemos": "vitrificar",
    "vitrificó": "vitrificar",
    "vitrifique": "vitrificar",
    "vitrifiquemos": "vitrificar",
    "vitrifiquen": "vitrificar",
    "vitrifiques": "vitrificar",
    "vitrifiqué": "vitrificar",
    "vitrifiquéis": "vitrificar",
    "vitrinas": "vitrina",
    "vitriolos": "vitriolo",
    "vitritas": "vitrita",
    "vitriólica": "vitriólico",
    "vitriólicas": "vitriólico",
    "vitriólicos": "vitriólico",
    "vitronectinas": "vitronectina",
    "vitualla": "vituallar",
    "vituallaba": "vituallar",
    "vituallabais": "vituallar",
    "vituallaban": "vituallar",
    "vituallabas": "vituallar",
    "vituallad": "vituallar",
    "vituallada": "vituallar",
    "vitualladas": "vituallar",
    "vituallado": "vituallar",
    "vituallados": "vituallar",
    "vituallamos": "vituallar",
    "vituallan": "vituallar",
    "vituallando": "vituallar",
    "vituallara": "vituallar",
    "vituallarais": "vituallar",
    "vituallaran": "vituallar",
    "vituallaras": "vituallar",
    "vituallare": "vituallar",
    "vituallareis": "vituallar",
    "vituallaremos": "vituallar",
    "vituallaren": "vituallar",
    "vituallares": "vituallar",
    "vituallaron": "vituallar",
    "vituallará": "vituallar",
    "vituallarán": "vituallar",
    "vituallarás": "vituallar",
    "vituallaré": "vituallar",
    "vituallaréis": "vituallar",
    "vituallaría": "vituallar",
    "vituallaríais": "vituallar",
    "vituallaríamos": "vituallar",
    "vituallarían": "vituallar",
    "vituallarías": "vituallar",
    "vituallas": "vituallar",
    "vituallase": "vituallar",
    "vituallaseis": "vituallar",
    "vituallasen": "vituallar",
    "vituallases": "vituallar",
    "vituallaste": "vituallar",
    "vituallasteis": "vituallar",
    "vitualle": "vituallar",
    "vituallemos": "vituallar",
    "vituallen": "vituallar",
    "vitualles": "vituallar",
    "vituallo": "vituallar",
    "vituallábamos": "vituallar",
    "vitualláis": "vituallar",
    "vitualláramos": "vituallar",
    "vitualláremos": "vituallar",
    "vituallásemos": "vituallar",
    "vituallé": "vituallar",
    "vitualléis": "vituallar",
    "vitualló": "vituallar",
    "vitupera": "vituperar",
    "vituperaba": "vituperar",
    "vituperabais": "vituperar",
    "vituperaban": "vituperar",
    "vituperabas": "vituperar",
    "vituperables": "vituperable",
    "vituperaciones": "vituperación",
    "vituperad": "vituperar",
    "vituperada": "vituperar",
    "vituperadas": "vituperar",
    "vituperado": "vituperar",
    "vituperadora": "vituperador",
    "vituperadoras": "vituperador",
    "vituperadores": "vituperador",
    "vituperados": "vituperar",
    "vituperamos": "vituperar",
    "vituperan": "vituperar",
    "vituperando": "vituperar",
    "vituperantes": "vituperante",
    "vituperara": "vituperar",
    "vituperarais": "vituperar",
    "vituperaran": "vituperar",
    "vituperaras": "vituperar",
    "vituperare": "vituperar",
    "vituperareis": "vituperar",
    "vituperaremos": "vituperar",
    "vituperaren": "vituperar",
    "vituperares": "vituperar",
    "vituperaron": "vituperar",
    "vituperará": "vituperar",
    "vituperarán": "vituperar",
    "vituperarás": "vituperar",
    "vituperaré": "vituperar",
    "vituperaréis": "vituperar",
    "vituperaría": "vituperar",
    "vituperaríais": "vituperar",
    "vituperaríamos": "vituperar",
    "vituperarían": "vituperar",
    "vituperarías": "vituperar",
    "vituperas": "vituperar",
    "vituperase": "vituperar",
    "vituperaseis": "vituperar",
    "vituperasen": "vituperar",
    "vituperases": "vituperar",
    "vituperaste": "vituperar",
    "vituperasteis": "vituperar",
    "vitupere": "vituperar",
    "vituperemos": "vituperar",
    "vituperen": "vituperar",
    "vituperes": "vituperar",
    "vituperios": "vituperio",
    "vituperiosa": "vituperioso",
    "vituperiosas": "vituperioso",
    "vituperiosos": "vituperioso",
    "vitupero": "vituperar",
    "vituperosa": "vituperoso",
    "vituperosas": "vituperoso",
    "vituperosos": "vituperoso",
    "vituperábamos": "vituperar",
    "vituperáis": "vituperar",
    "vituperáramos": "vituperar",
    "vituperáremos": "vituperar",
    "vituperásemos": "vituperar",
    "vituperé": "vituperar",
    "vituperéis": "vituperar",
    "vituperó": "vituperar",
    "vitácea": "vitáceo",
    "vitáceas": "vitáceo",
    "vitáceos": "vitáceo",
    "vitícolas": "vitícola",
    "vitíligos": "vitíligo",
    "viuda": "viudo",
    "viudas": "viudo",
    "viudeces": "viudez",
    "viudedades": "viudedad",
    "viudos": "viudo",
    "viva": "vivo",
    "vivaces": "vivaz",
    "vivacidades": "vivacidad",
    "vivamos": "vivir",
    "vivan": "vivir",
    "vivandera": "vivandero",
    "vivanderas": "vivandero",
    "vivanderos": "vivandero",
    "vivaquea": "vivaquear",
    "vivaqueaba": "vivaquear",
    "vivaqueabais": "vivaquear",
    "vivaqueaban": "vivaquear",
    "vivaqueabas": "vivaquear",
    "vivaquead": "vivaquear",
    "vivaqueada": "vivaquear",
    "vivaqueadas": "vivaquear",
    "vivaqueado": "vivaquear",
    "vivaqueados": "vivaquear",
    "vivaqueamos": "vivaquear",
    "vivaquean": "vivaquear",
    "vivaqueando": "vivaquear",
    "vivaqueara": "vivaquear",
    "vivaquearais": "vivaquear",
    "vivaquearan": "vivaquear",
    "vivaquearas": "vivaquear",
    "vivaqueare": "vivaquear",
    "vivaqueareis": "vivaquear",
    "vivaquearemos": "vivaquear",
    "vivaquearen": "vivaquear",
    "vivaqueares": "vivaquear",
    "vivaquearon": "vivaquear",
    "vivaqueará": "vivaquear",
    "vivaquearán": "vivaquear",
    "vivaquearás": "vivaquear",
    "vivaquearé": "vivaquear",
    "vivaquearéis": "vivaquear",
    "vivaquearía": "vivaquear",
    "vivaquearíais": "vivaquear",
    "vivaquearíamos": "vivaquear",
    "vivaquearían": "vivaquear",
    "vivaquearías": "vivaquear",
    "vivaqueas": "vivaquear",
    "vivaquease": "vivaquear",
    "vivaqueaseis": "vivaquear",
    "vivaqueasen": "vivaquear",
    "vivaqueases": "vivaquear",
    "vivaqueaste": "vivaquear",
    "vivaqueasteis": "vivaquear",
    "vivaquee": "vivaquear",
    "vivaqueemos": "vivaquear",
    "vivaqueen": "vivaquear",
    "vivaquees": "vivaquear",
    "vivaqueo": "vivaquear",
    "vivaques": "vivaque",
    "vivaqueábamos": "vivaquear",
    "vivaqueáis": "vivaquear",
    "vivaqueáramos": "vivaquear",
    "vivaqueáremos": "vivaquear",
    "vivaqueásemos": "vivaquear",
    "vivaqueé": "vivaquear",
    "vivaqueéis": "vivaquear",
    "vivaqueó": "vivaquear",
    "vivaracha": "vivaracho",
    "vivarachas": "vivaracho",
    "vivarachos": "vivaracho",
    "vivares": "vivar",
    "vivas": "vivo",
    "vivazes": "vivaz",
    "vive": "vivir",
    "viven": "vivir",
    "vivenciales": "vivencial",
    "vivencias": "vivencia",
    "viverales": "viveral",
    "viveras": "vivera",
    "viveristas": "viverista",
    "viveros": "vivero",
    "vives": "vivir",
    "vivezas": "viveza",
    "vivianitas": "vivianita",
    "vivid": "vivir",
    "vivida": "vivir",
    "vividas": "vivir",
    "vividera": "vividero",
    "vivideras": "vividero",
    "vivideros": "vividero",
    "vivido": "vivir",
    "vividora": "vividor",
    "vividoras": "vividor",
    "vividores": "vividor",
    "vividos": "vivir",
    "viviendas": "vivienda",
    "viviendo": "vivir",
    "vivientes": "viviente",
    "viviera": "vivir",
    "vivierais": "vivir",
    "vivieran": "vivir",
    "vivieras": "vivir",
    "viviere": "vivir",
    "viviereis": "vivir",
    "vivieren": "vivir",
    "vivieres": "vivir",
    "vivieron": "vivir",
    "viviese": "vivir",
    "vivieseis": "vivir",
    "viviesen": "vivir",
    "vivieses": "vivir",
    "vivifica": "vivificar",
    "vivificaba": "vivificar",
    "vivificabais": "vivificar",
    "vivificaban": "vivificar",
    "vivificabas": "vivificar",
    "vivificaciones": "vivificación",
    "vivificad": "vivificar",
    "vivificada": "vivificar",
    "vivificadas": "vivificar",
    "vivificado": "vivificar",
    "vivificadora": "vivificador",
    "vivificadoras": "vivificador",
    "vivificadores": "vivificador",
    "vivificados": "vivificar",
    "vivificamos": "vivificar",
    "vivifican": "vivificar",
    "vivificando": "vivificar",
    "vivificara": "vivificar",
    "vivificarais": "vivificar",
    "vivificaran": "vivificar",
    "vivificaras": "vivificar",
    "vivificare": "vivificar",
    "vivificareis": "vivificar",
    "vivificaremos": "vivificar",
    "vivificaren": "vivificar",
    "vivificares": "vivificar",
    "vivificaron": "vivificar",
    "vivificará": "vivificar",
    "vivificarán": "vivificar",
    "vivificarás": "vivificar",
    "vivificaré": "vivificar",
    "vivificaréis": "vivificar",
    "vivificaría": "vivificar",
    "vivificaríais": "vivificar",
    "vivificaríamos": "vivificar",
    "vivificarían": "vivificar",
    "vivificarías": "vivificar",
    "vivificas": "vivificar",
    "vivificase": "vivificar",
    "vivificaseis": "vivificar",
    "vivificasen": "vivificar",
    "vivificases": "vivificar",
    "vivificaste": "vivificar",
    "vivificasteis": "vivificar",
    "vivificativa": "vivificativo",
    "vivificativas": "vivificativo",
    "vivificativos": "vivificativo",
    "vivifico": "vivificar",
    "vivificábamos": "vivificar",
    "vivificáis": "vivificar",
    "vivificáramos": "vivificar",
    "vivificáremos": "vivificar",
    "vivificásemos": "vivificar",
    "vivificó": "vivificar",
    "vivifique": "vivificar",
    "vivifiquemos": "vivificar",
    "vivifiquen": "vivificar",
    "vivifiques": "vivificar",
    "vivifiqué": "vivificar",
    "vivifiquéis": "vivificar",
    "vivimos": "vivir",
    "viviparismos": "viviparismo",
    "viviremos": "vivir",
    "vivires": "vivir",
    "vivirá": "vivir",
    "vivirán": "vivir",
    "vivirás": "vivir",
    "viviré": "vivir",
    "viviréis": "vivir",
    "viviría": "vivir",
    "viviríais": "vivir",
    "viviríamos": "vivir",
    "vivirían": "vivir",
    "vivirías": "vivir",
    "viviseccionadora": "viviseccionador",
    "viviseccionadoras": "viviseccionador",
    "viviseccionadores": "viviseccionador",
    "vivisecciones": "vivisección",
    "vivismos": "vivismo",
    "vivistas": "vivista",
    "viviste": "vivir",
    "vivisteis": "vivir",
    "viviéramos": "vivir",
    "viviéremos": "vivir",
    "viviésemos": "vivir",
    "vivió": "vivir",
    "vivo": "vivir",
    "vivos": "vivo",
    "viváis": "vivir",
    "vivérridos": "vivérrido",
    "viví": "vivir",
    "vivía": "vivir",
    "vivíais": "vivir",
    "vivíamos": "vivir",
    "vivían": "vivir",
    "vivías": "vivir",
    "vivífica": "vivífico",
    "vivíficas": "vivífico",
    "vivíficos": "vivífico",
    "vivípara": "vivíparo",
    "vivíparas": "vivíparo",
    "vivíparos": "vivíparo",
    "vivís": "vivir",
    "vizcachas": "vizcacha",
    "vizcainadas": "vizcainada",
    "vizcaitarras": "vizcaitarra",
    "vizcaína": "vizcaíno",
    "vizcaínas": "vizcaíno",
    "vizcaínos": "vizcaíno",
    "vizcondados": "vizcondado",
    "vizcondes": "vizconde",
    "vizcondesas": "vizcondesa",
    "viáticos": "viático",
    "viéramos": "ver",
    "viéremos": "ver",
    "viésemos": "ver",
    "viñadera": "viñadero",
    "viñaderas": "viñadero",
    "viñaderos": "viñadero",
    "viñadora": "viñador",
    "viñadoras": "viñador",
    "viñadores": "viñador",
    "viñas": "viña",
    "viñedos": "viñedo",
    "viñera": "viñero",
    "viñeras": "viñero",
    "viñeros": "viñero",
    "viñetajes": "viñetaje",
    "viñetas": "viñeta",
    "viñeteados": "viñeteado",
    "viñeteros": "viñetero",
    "vobuladores": "vobulador",
    "vocablos": "vocablo",
    "vocabularios": "vocabulario",
    "vocabulistas": "vocabulista",
    "vocacionales": "vocacional",
    "vocaciones": "vocación",
    "vocales": "vocal",
    "vocalice": "vocalizar",
    "vocalicemos": "vocalizar",
    "vocalicen": "vocalizar",
    "vocalices": "vocalizar",
    "vocalicé": "vocalizar",
    "vocalicéis": "vocalizar",
    "vocalismos": "vocalismo",
    "vocalistas": "vocalista",
    "vocaliza": "vocalizar",
    "vocalizaba": "vocalizar",
    "vocalizabais": "vocalizar",
    "vocalizaban": "vocalizar",
    "vocalizabas": "vocalizar",
    "vocalizaciones": "vocalización",
    "vocalizad": "vocalizar",
    "vocalizada": "vocalizar",
    "vocalizadas": "vocalizar",
    "vocalizado": "vocalizar",
    "vocalizadora": "vocalizador",
    "vocalizadoras": "vocalizador",
    "vocalizadores": "vocalizador",
    "vocalizados": "vocalizar",
    "vocalizamos": "vocalizar",
    "vocalizan": "vocalizar",
    "vocalizando": "vocalizar",
    "vocalizara": "vocalizar",
    "vocalizarais": "vocalizar",
    "vocalizaran": "vocalizar",
    "vocalizaras": "vocalizar",
    "vocalizare": "vocalizar",
    "vocalizareis": "vocalizar",
    "vocalizaremos": "vocalizar",
    "vocalizaren": "vocalizar",
    "vocalizares": "vocalizar",
    "vocalizaron": "vocalizar",
    "vocalizará": "vocalizar",
    "vocalizarán": "vocalizar",
    "vocalizarás": "vocalizar",
    "vocalizaré": "vocalizar",
    "vocalizaréis": "vocalizar",
    "vocalizaría": "vocalizar",
    "vocalizaríais": "vocalizar",
    "vocalizaríamos": "vocalizar",
    "vocalizarían": "vocalizar",
    "vocalizarías": "vocalizar",
    "vocalizas": "vocalizar",
    "vocalizase": "vocalizar",
    "vocalizaseis": "vocalizar",
    "vocalizasen": "vocalizar",
    "vocalizases": "vocalizar",
    "vocalizaste": "vocalizar",
    "vocalizasteis": "vocalizar",
    "vocalizo": "vocalizar",
    "vocalizábamos": "vocalizar",
    "vocalizáis": "vocalizar",
    "vocalizáramos": "vocalizar",
    "vocalizáremos": "vocalizar",
    "vocalizásemos": "vocalizar",
    "vocalizó": "vocalizar",
    "vocalías": "vocalía",
    "vocativa": "vocativo",
    "vocativas": "vocativo",
    "vocativos": "vocativo",
    "vocea": "vocear",
    "voceaba": "vocear",
    "voceabais": "vocear",
    "voceaban": "vocear",
    "voceabas": "vocear",
    "vocead": "vocear",
    "voceada": "vocear",
    "voceadas": "vocear",
    "voceado": "vocear",
    "voceadora": "voceador",
    "voceadoras": "voceador",
    "voceadores": "voceador",
    "voceados": "vocear",
    "voceamos": "vocear",
    "vocean": "vocear",
    "voceando": "vocear",
    "voceara": "vocear",
    "vocearais": "vocear",
    "vocearan": "vocear",
    "vocearas": "vocear",
    "voceare": "vocear",
    "voceareis": "vocear",
    "vocearemos": "vocear",
    "vocearen": "vocear",
    "voceares": "vocear",
    "vocearon": "vocear",
    "voceará": "vocear",
    "vocearán": "vocear",
    "vocearás": "vocear",
    "vocearé": "vocear",
    "vocearéis": "vocear",
    "vocearía": "vocear",
    "vocearíais": "vocear",
    "vocearíamos": "vocear",
    "vocearían": "vocear",
    "vocearías": "vocear",
    "voceas": "vocear",
    "vocease": "vocear",
    "voceaseis": "vocear",
    "voceasen": "vocear",
    "voceases": "vocear",
    "voceaste": "vocear",
    "voceasteis": "vocear",
    "vocee": "vocear",
    "voceemos": "vocear",
    "voceen": "vocear",
    "vocees": "vocear",
    "vocejones": "vocejón",
    "voceo": "vocear",
    "vocera": "vocero",
    "voceras": "vocero",
    "voceros": "vocero",
    "vocerías": "vocería",
    "voceríos": "vocerío",
    "voces": "voz",
    "voceábamos": "vocear",
    "voceáis": "vocear",
    "voceáramos": "vocear",
    "voceáremos": "vocear",
    "voceásemos": "vocear",
    "voceé": "vocear",
    "voceéis": "vocear",
    "voceó": "vocear",
    "vocifera": "vociferar",
    "vociferaba": "vociferar",
    "vociferabais": "vociferar",
    "vociferaban": "vociferar",
    "vociferabas": "vociferar",
    "vociferaciones": "vociferación",
    "vociferad": "vociferar",
    "vociferada": "vociferar",
    "vociferadas": "vociferar",
    "vociferado": "vociferar",
    "vociferadora": "vociferador",
    "vociferadoras": "vociferador",
    "vociferadores": "vociferador",
    "vociferados": "vociferar",
    "vociferamos": "vociferar",
    "vociferan": "vociferar",
    "vociferando": "vociferar",
    "vociferantes": "vociferante",
    "vociferara": "vociferar",
    "vociferarais": "vociferar",
    "vociferaran": "vociferar",
    "vociferaras": "vociferar",
    "vociferare": "vociferar",
    "vociferareis": "vociferar",
    "vociferaremos": "vociferar",
    "vociferaren": "vociferar",
    "vociferares": "vociferar",
    "vociferaron": "vociferar",
    "vociferará": "vociferar",
    "vociferarán": "vociferar",
    "vociferarás": "vociferar",
    "vociferaré": "vociferar",
    "vociferaréis": "vociferar",
    "vociferaría": "vociferar",
    "vociferaríais": "vociferar",
    "vociferaríamos": "vociferar",
    "vociferarían": "vociferar",
    "vociferarías": "vociferar",
    "vociferas": "vociferar",
    "vociferase": "vociferar",
    "vociferaseis": "vociferar",
    "vociferasen": "vociferar",
    "vociferases": "vociferar",
    "vociferaste": "vociferar",
    "vociferasteis": "vociferar",
    "vocifere": "vociferar",
    "vociferemos": "vociferar",
    "vociferen": "vociferar",
    "vociferes": "vociferar",
    "vocifero": "vociferar",
    "vociferábamos": "vociferar",
    "vociferáis": "vociferar",
    "vociferáramos": "vociferar",
    "vociferáremos": "vociferar",
    "vociferásemos": "vociferar",
    "vociferé": "vociferar",
    "vociferéis": "vociferar",
    "vociferó": "vociferar",
    "vocinglera": "vocinglero",
    "vocingleras": "vocinglero",
    "vocingleros": "vocinglero",
    "vocinglerías": "vocinglería",
    "vocoderes": "vocoder",
    "vocálica": "vocálico",
    "vocálicas": "vocálico",
    "vocálicos": "vocálico",
    "vodcas": "vodca",
    "vodeviles": "vodevil",
    "vodevilesca": "vodevilesco",
    "vodevilescas": "vodevilesco",
    "vodevilescos": "vodevilesco",
    "vodkas": "vodka",
    "voivodas": "voivoda",
    "voivodatos": "voivodato",
    "volaba": "volar",
    "volabais": "volar",
    "volaban": "volar",
    "volabas": "volar",
    "volad": "volar",
    "volada": "volar",
    "voladas": "volar",
    "voladera": "voladero",
    "voladeras": "voladero",
    "voladeros": "voladero",
    "voladiza": "voladizo",
    "voladizas": "voladizo",
    "voladizos": "voladizo",
    "volado": "volar",
    "voladora": "volador",
    "voladoras": "volador",
    "voladores": "volador",
    "volados": "volar",
    "voladuras": "voladura",
    "volamos": "volar",
    "volandera": "volandero",
    "volanderas": "volandero",
    "volanderos": "volandero",
    "volando": "volar",
    "volantes": "volante",
    "volantina": "volantín",
    "volantinas": "volantín",
    "volantines": "volantín",
    "volantona": "volantón",
    "volantonas": "volantón",
    "volantones": "volantón",
    "volapiés": "volapié",
    "volapukes": "volapuk",
    "volapuks": "volapuk",
    "volara": "volar",
    "volarais": "volar",
    "volaran": "volar",
    "volaras": "volar",
    "volare": "volar",
    "volareis": "volar",
    "volaremos": "volar",
    "volaren": "volar",
    "volares": "volar",
    "volaron": "volar",
    "volará": "volar",
    "volarán": "volar",
    "volarás": "volar",
    "volaré": "volar",
    "volaréis": "volar",
    "volaría": "volar",
    "volaríais": "volar",
    "volaríamos": "volar",
    "volarían": "volar",
    "volarías": "volar",
    "volase": "volar",
    "volaseis": "volar",
    "volasen": "volar",
    "volases": "volar",
    "volaste": "volar",
    "volasteis": "volar",
    "volaterías": "volatería",
    "volatilice": "volatilizar",
    "volatilicemos": "volatilizar",
    "volatilicen": "volatilizar",
    "volatilices": "volatilizar",
    "volatilicé": "volatilizar",
    "volatilicéis": "volatilizar",
    "volatilidades": "volatilidad",
    "volatiliza": "volatilizar",
    "volatilizaba": "volatilizar",
    "volatilizabais": "volatilizar",
    "volatilizaban": "volatilizar",
    "volatilizabas": "volatilizar",
    "volatilizables": "volatilizable",
    "volatilizaciones": "volatilización",
    "volatilizad": "volatilizar",
    "volatilizada": "volatilizar",
    "volatilizadas": "volatilizar",
    "volatilizado": "volatilizar",
    "volatilizadores": "volatilizador",
    "volatilizados": "volatilizar",
    "volatilizamos": "volatilizar",
    "volatilizan": "volatilizar",
    "volatilizando": "volatilizar",
    "volatilizara": "volatilizar",
    "volatilizarais": "volatilizar",
    "volatilizaran": "volatilizar",
    "volatilizaras": "volatilizar",
    "volatilizare": "volatilizar",
    "volatilizareis": "volatilizar",
    "volatilizaremos": "volatilizar",
    "volatilizaren": "volatilizar",
    "volatilizares": "volatilizar",
    "volatilizaron": "volatilizar",
    "volatilizará": "volatilizar",
    "volatilizarán": "volatilizar",
    "volatilizarás": "volatilizar",
    "volatilizaré": "volatilizar",
    "volatilizaréis": "volatilizar",
    "volatilizaría": "volatilizar",
    "volatilizaríais": "volatilizar",
    "volatilizaríamos": "volatilizar",
    "volatilizarían": "volatilizar",
    "volatilizarías": "volatilizar",
    "volatilizas": "volatilizar",
    "volatilizase": "volatilizar",
    "volatilizaseis": "volatilizar",
    "volatilizasen": "volatilizar",
    "volatilizases": "volatilizar",
    "volatilizaste": "volatilizar",
    "volatilizasteis": "volatilizar",
    "volatilizo": "volatilizar",
    "volatilizábamos": "volatilizar",
    "volatilizáis": "volatilizar",
    "volatilizáramos": "volatilizar",
    "volatilizáremos": "volatilizar",
    "volatilizásemos": "volatilizar",
    "volatilizó": "volatilizar",
    "volatinera": "volatinero",
    "volatineras": "volatinero",
    "volatineros": "volatinero",
    "volatines": "volatín",
    "volauventes": "volauvent",
    "volauvents": "volauvent",
    "volcaba": "volcar",
    "volcabais": "volcar",
    "volcaban": "volcar",
    "volcabas": "volcar",
    "volcad": "volcar",
    "volcada": "volcar",
    "volcadas": "volcar",
    "volcado": "volcar",
    "volcadores": "volcador",
    "volcados": "volcar",
    "volcamos": "volcar",
    "volcando": "volcar",
    "volcanes": "volcán",
    "volcanismos": "volcanismo",
    "volcanologías": "volcanología",
    "volcanóloga": "volcanólogo",
    "volcanólogas": "volcanólogo",
    "volcanólogos": "volcanólogo",
    "volcara": "volcar",
    "volcarais": "volcar",
    "volcaran": "volcar",
    "volcaras": "volcar",
    "volcare": "volcar",
    "volcareis": "volcar",
    "volcaremos": "volcar",
    "volcaren": "volcar",
    "volcares": "volcar",
    "volcaron": "volcar",
    "volcará": "volcar",
    "volcarán": "volcar",
    "volcarás": "volcar",
    "volcaré": "volcar",
    "volcaréis": "volcar",
    "volcaría": "volcar",
    "volcaríais": "volcar",
    "volcaríamos": "volcar",
    "volcarían": "volcar",
    "volcarías": "volcar",
    "volcase": "volcar",
    "volcaseis": "volcar",
    "volcasen": "volcar",
    "volcases": "volcar",
    "volcaste": "volcar",
    "volcasteis": "volcar",
    "volcábamos": "volcar",
    "volcáis": "volcar",
    "volcánica": "volcánico",
    "volcánicas": "volcánico",
    "volcánicos": "volcánico",
    "volcáramos": "volcar",
    "volcáremos": "volcar",
    "volcásemos": "volcar",
    "volcó": "volcar",
    "volea": "volear",
    "voleaba": "volear",
    "voleabais": "volear",
    "voleaban": "volear",
    "voleabas": "volear",
    "volead": "volear",
    "voleada": "volear",
    "voleadas": "volear",
    "voleado": "volear",
    "voleados": "volear",
    "voleamos": "volear",
    "volean": "volear",
    "voleando": "volear",
    "voleara": "volear",
    "volearais": "volear",
    "volearan": "volear",
    "volearas": "volear",
    "voleare": "volear",
    "voleareis": "volear",
    "volearemos": "volear",
    "volearen": "volear",
    "voleares": "volear",
    "volearon": "volear",
    "voleará": "volear",
    "volearán": "volear",
    "volearás": "volear",
    "volearé": "volear",
    "volearéis": "volear",
    "volearía": "volear",
    "volearíais": "volear",
    "volearíamos": "volear",
    "volearían": "volear",
    "volearías": "volear",
    "voleas": "volear",
    "volease": "volear",
    "voleaseis": "volear",
    "voleasen": "volear",
    "voleases": "volear",
    "voleaste": "volear",
    "voleasteis": "volear",
    "volee": "volear",
    "voleemos": "volear",
    "voleen": "volear",
    "volees": "volear",
    "voleiboles": "voleibol",
    "volemias": "volemia",
    "volemos": "volar",
    "voleo": "volear",
    "voleos": "voleo",
    "voleábamos": "volear",
    "voleáis": "volear",
    "voleáramos": "volear",
    "voleáremos": "volear",
    "voleásemos": "volear",
    "voleé": "volear",
    "voleéis": "volear",
    "voleó": "volear",
    "volframatos": "volframato",
    "volframios": "volframio",
    "voliciones": "volición",
    "volita": "volitar",
    "volitaba": "volitar",
    "volitabais": "volitar",
    "volitaban": "volitar",
    "volitabas": "volitar",
    "volitad": "volitar",
    "volitada": "volitar",
    "volitadas": "volitar",
    "volitado": "volitar",
    "volitados": "volitar",
    "volitamos": "volitar",
    "volitan": "volitar",
    "volitando": "volitar",
    "volitara": "volitar",
    "volitarais": "volitar",
    "volitaran": "volitar",
    "volitaras": "volitar",
    "volitare": "volitar",
    "volitareis": "volitar",
    "volitaremos": "volitar",
    "volitaren": "volitar",
    "volitares": "volitar",
    "volitaron": "volitar",
    "volitará": "volitar",
    "volitarán": "volitar",
    "volitarás": "volitar",
    "volitaré": "volitar",
    "volitaréis": "volitar",
    "volitaría": "volitar",
    "volitaríais": "volitar",
    "volitaríamos": "volitar",
    "volitarían": "volitar",
    "volitarías": "volitar",
    "volitas": "volitar",
    "volitase": "volitar",
    "volitaseis": "volitar",
    "volitasen": "volitar",
    "volitases": "volitar",
    "volitaste": "volitar",
    "volitasteis": "volitar",
    "volite": "volitar",
    "volitemos": "volitar",
    "voliten": "volitar",
    "volites": "volitar",
    "volitiva": "volitivo",
    "volitivas": "volitivo",
    "volitivos": "volitivo",
    "volito": "volitar",
    "volitábamos": "volitar",
    "volitáis": "volitar",
    "volitáramos": "volitar",
    "volitáremos": "volitar",
    "volitásemos": "volitar",
    "volité": "volitar",
    "volitéis": "volitar",
    "volitó": "volitar",
    "volovanes": "volován",
    "volquemos": "volcar",
    "volquetazos": "volquetazo",
    "volquetera": "volquetero",
    "volqueteras": "volquetero",
    "volqueteros": "volquetero",
    "volquetes": "volquete",
    "volqué": "volcar",
    "volquéis": "volcar",
    "volscos": "volsco",
    "volt": "voltio",
    "voltaica": "voltaico",
    "voltaicas": "voltaico",
    "voltaicos": "voltaico",
    "voltajes": "voltaje",
    "voltametrías": "voltametría",
    "voltamperes": "voltampere",
    "voltamperios": "voltamperio",
    "voltamperímetros": "voltamperímetro",
    "voltaria": "voltario",
    "voltarias": "voltario",
    "voltariedades": "voltariedad",
    "voltarios": "voltario",
    "voltascopios": "voltascopio",
    "voltaísmos": "voltaísmo",
    "voltea": "voltear",
    "volteaba": "voltear",
    "volteabais": "voltear",
    "volteaban": "voltear",
    "volteabas": "voltear",
    "voltead": "voltear",
    "volteada": "voltear",
    "volteadas": "voltear",
    "volteado": "voltear",
    "volteadora": "volteador",
    "volteadoras": "volteador",
    "volteadores": "volteador",
    "volteados": "voltear",
    "volteamos": "voltear",
    "voltean": "voltear",
    "volteando": "voltear",
    "volteara": "voltear",
    "voltearais": "voltear",
    "voltearan": "voltear",
    "voltearas": "voltear",
    "volteare": "voltear",
    "volteareis": "voltear",
    "voltearemos": "voltear",
    "voltearen": "voltear",
    "volteares": "voltear",
    "voltearon": "voltear",
    "volteará": "voltear",
    "voltearán": "voltear",
    "voltearás": "voltear",
    "voltearé": "voltear",
    "voltearéis": "voltear",
    "voltearía": "voltear",
    "voltearíais": "voltear",
    "voltearíamos": "voltear",
    "voltearían": "voltear",
    "voltearías": "voltear",
    "volteas": "voltear",
    "voltease": "voltear",
    "volteaseis": "voltear",
    "volteasen": "voltear",
    "volteases": "voltear",
    "volteaste": "voltear",
    "volteasteis": "voltear",
    "voltee": "voltear",
    "volteemos": "voltear",
    "volteen": "voltear",
    "voltees": "voltear",
    "voltejea": "voltejear",
    "voltejeaba": "voltejear",
    "voltejeabais": "voltejear",
    "voltejeaban": "voltejear",
    "voltejeabas": "voltejear",
    "voltejead": "voltejear",
    "voltejeada": "voltejear",
    "voltejeadas": "voltejear",
    "voltejeado": "voltejear",
    "voltejeados": "voltejear",
    "voltejeamos": "voltejear",
    "voltejean": "voltejear",
    "voltejeando": "voltejear",
    "voltejeara": "voltejear",
    "voltejearais": "voltejear",
    "voltejearan": "voltejear",
    "voltejearas": "voltejear",
    "voltejeare": "voltejear",
    "voltejeareis": "voltejear",
    "voltejearemos": "voltejear",
    "voltejearen": "voltejear",
    "voltejeares": "voltejear",
    "voltejearon": "voltejear",
    "voltejeará": "voltejear",
    "voltejearán": "voltejear",
    "voltejearás": "voltejear",
    "voltejearé": "voltejear",
    "voltejearéis": "voltejear",
    "voltejearía": "voltejear",
    "voltejearíais": "voltejear",
    "voltejearíamos": "voltejear",
    "voltejearían": "voltejear",
    "voltejearías": "voltejear",
    "voltejeas": "voltejear",
    "voltejease": "voltejear",
    "voltejeaseis": "voltejear",
    "voltejeasen": "voltejear",
    "voltejeases": "voltejear",
    "voltejeaste": "voltejear",
    "voltejeasteis": "voltejear",
    "voltejee": "voltejear",
    "voltejeemos": "voltejear",
    "voltejeen": "voltejear",
    "voltejees": "voltejear",
    "voltejeo": "voltejear",
    "voltejeábamos": "voltejear",
    "voltejeáis": "voltejear",
    "voltejeáramos": "voltejear",
    "voltejeáremos": "voltejear",
    "voltejeásemos": "voltejear",
    "voltejeé": "voltejear",
    "voltejeéis": "voltejear",
    "voltejeó": "voltejear",
    "volteletas": "volteleta",
    "volteo": "voltear",
    "volteos": "volteo",
    "volteretas": "voltereta",
    "volteriana": "volteriano",
    "volterianas": "volteriano",
    "volterianismos": "volterianismo",
    "volterianos": "volteriano",
    "voltetas": "volteta",
    "volteábamos": "voltear",
    "volteáis": "voltear",
    "volteáramos": "voltear",
    "volteáremos": "voltear",
    "volteásemos": "voltear",
    "volteé": "voltear",
    "volteéis": "voltear",
    "volteó": "voltear",
    "voltio-amperio": "voltampere",
    "voltio-amperios": "voltampere",
    "voltios": "voltio",
    "voltiza": "voltizo",
    "voltizas": "voltizo",
    "voltizos": "voltizo",
    "volts": "volt",
    "voltsegundos": "voltsegundo",
    "voltámetros": "voltámetro",
    "voltímetros": "voltímetro",
    "volubilidades": "volubilidad",
    "volubles": "voluble",
    "volumenómetros": "volumenómetro",
    "volumetrías": "volumetría",
    "voluminosa": "voluminoso",
    "voluminosas": "voluminoso",
    "voluminosos": "voluminoso",
    "volumétrica": "volumétrico",
    "volumétricas": "volumétrico",
    "volumétricos": "volumétrico",
    "voluntades": "voluntad",
    "voluntaria": "voluntario",
    "voluntariados": "voluntariado",
    "voluntarias": "voluntario",
    "voluntariedades": "voluntariedad",
    "voluntarios": "voluntario",
    "voluntariosa": "voluntarioso",
    "voluntariosas": "voluntarioso",
    "voluntariosos": "voluntarioso",
    "voluntarismos": "voluntarismo",
    "voluntaristas": "voluntarista",
    "voluptuosa": "voluptuoso",
    "voluptuosas": "voluptuoso",
    "voluptuosidades": "voluptuosidad",
    "voluptuosos": "voluptuoso",
    "volutas": "voluta",
    "volvamos": "volver",
    "volved": "volver",
    "volvemos": "volver",
    "volveremos": "volver",
    "volverinas": "volverina",
    "volverá": "volver",
    "volverán": "volver",
    "volverás": "volver",
    "volveré": "volver",
    "volveréis": "volver",
    "volvería": "volver",
    "volveríais": "volver",
    "volveríamos": "volver",
    "volverían": "volver",
    "volverías": "volver",
    "volviendo": "volver",
    "volviera": "volver",
    "volvierais": "volver",
    "volvieran": "volver",
    "volvieras": "volver",
    "volviere": "volver",
    "volviereis": "volver",
    "volvieren": "volver",
    "volvieres": "volver",
    "volvieron": "volver",
    "volviese": "volver",
    "volvieseis": "volver",
    "volviesen": "volver",
    "volvieses": "volver",
    "volvimos": "volver",
    "volviste": "volver",
    "volvisteis": "volver",
    "volviéramos": "volver",
    "volviéremos": "volver",
    "volviésemos": "volver",
    "volvió": "volver",
    "volvocales": "volvocal",
    "volvos": "volvo",
    "volváis": "volver",
    "volvéis": "volver",
    "volví": "volver",
    "volvía": "volver",
    "volvíais": "volver",
    "volvíamos": "volver",
    "volvían": "volver",
    "volvías": "volver",
    "volábamos": "volar",
    "voláis": "volar",
    "voláramos": "volar",
    "voláremos": "volar",
    "volásemos": "volar",
    "volátiles": "volátil",
    "volé": "volar",
    "voléis": "volar",
    "voló": "volar",
    "volúmenes": "volumen",
    "volúmetros": "volúmetro",
    "vomeriana": "vomeriano",
    "vomerianas": "vomeriano",
    "vomerianos": "vomeriano",
    "vomeronasales": "vomeronasal",
    "vomipurgantes": "vomipurgante",
    "vomipurgativa": "vomipurgativo",
    "vomipurgativas": "vomipurgativo",
    "vomipurgativos": "vomipurgativo",
    "vomita": "vomitar",
    "vomitaba": "vomitar",
    "vomitabais": "vomitar",
    "vomitaban": "vomitar",
    "vomitabas": "vomitar",
    "vomitad": "vomitar",
    "vomitada": "vomitar",
    "vomitadas": "vomitar",
    "vomitado": "vomitar",
    "vomitados": "vomitar",
    "vomitamos": "vomitar",
    "vomitan": "vomitar",
    "vomitando": "vomitar",
    "vomitara": "vomitar",
    "vomitarais": "vomitar",
    "vomitaran": "vomitar",
    "vomitaras": "vomitar",
    "vomitare": "vomitar",
    "vomitareis": "vomitar",
    "vomitaremos": "vomitar",
    "vomitaren": "vomitar",
    "vomitares": "vomitar",
    "vomitaron": "vomitar",
    "vomitará": "vomitar",
    "vomitarán": "vomitar",
    "vomitarás": "vomitar",
    "vomitaré": "vomitar",
    "vomitaréis": "vomitar",
    "vomitaría": "vomitar",
    "vomitaríais": "vomitar",
    "vomitaríamos": "vomitar",
    "vomitarían": "vomitar",
    "vomitarías": "vomitar",
    "vomitas": "vomitar",
    "vomitase": "vomitar",
    "vomitaseis": "vomitar",
    "vomitasen": "vomitar",
    "vomitases": "vomitar",
    "vomitaste": "vomitar",
    "vomitasteis": "vomitar",
    "vomite": "vomitar",
    "vomiteles": "vomitel",
    "vomitemos": "vomitar",
    "vomiten": "vomitar",
    "vomiteras": "vomitera",
    "vomites": "vomitar",
    "vomitiva": "vomitivo",
    "vomitivas": "vomitivo",
    "vomitivos": "vomitivo",
    "vomito": "vomitar",
    "vomitona": "vomitón",
    "vomitonas": "vomitona",
    "vomitones": "vomitón",
    "vomitoria": "vomitorio",
    "vomitorias": "vomitorio",
    "vomitorios": "vomitorio",
    "vomitábamos": "vomitar",
    "vomitáis": "vomitar",
    "vomitáramos": "vomitar",
    "vomitáremos": "vomitar",
    "vomitásemos": "vomitar",
    "vomité": "vomitar",
    "vomitéis": "vomitar",
    "vomitó": "vomitar",
    "voquibles": "voquible",
    "voraces": "voraz",
    "voracidades": "voracidad",
    "voraginosa": "voraginoso",
    "voraginosas": "voraginoso",
    "voraginosos": "voraginoso",
    "vorahúndas": "vorahúnda",
    "vorazes": "voraz",
    "vores": "vor",
    "voroninas": "voronina",
    "vors": "vor",
    "vorticidades": "vorticidad",
    "vorticismos": "vorticismo",
    "vorticosa": "vorticoso",
    "vorticosas": "vorticoso",
    "vorticosos": "vorticoso",
    "vortiginosa": "vortiginoso",
    "vortiginosas": "vortiginoso",
    "vortiginosos": "vortiginoso",
    "vorágines": "vorágine",
    "vos": "tú",
    "vosea": "vosear",
    "voseaba": "vosear",
    "voseabais": "vosear",
    "voseaban": "vosear",
    "voseabas": "vosear",
    "vosead": "vosear",
    "voseada": "vosear",
    "voseadas": "vosear",
    "voseado": "vosear",
    "voseados": "vosear",
    "voseamos": "vosear",
    "vosean": "vosear",
    "voseando": "vosear",
    "voseara": "vosear",
    "vosearais": "vosear",
    "vosearan": "vosear",
    "vosearas": "vosear",
    "voseare": "vosear",
    "voseareis": "vosear",
    "vosearemos": "vosear",
    "vosearen": "vosear",
    "voseares": "vosear",
    "vosearon": "vosear",
    "voseará": "vosear",
    "vosearán": "vosear",
    "vosearás": "vosear",
    "vosearé": "vosear",
    "vosearéis": "vosear",
    "vosearía": "vosear",
    "vosearíais": "vosear",
    "vosearíamos": "vosear",
    "vosearían": "vosear",
    "vosearías": "vosear",
    "voseas": "vosear",
    "vosease": "vosear",
    "voseaseis": "vosear",
    "voseasen": "vosear",
    "voseases": "vosear",
    "voseaste": "vosear",
    "voseasteis": "vosear",
    "vosee": "vosear",
    "voseemos": "vosear",
    "voseen": "vosear",
    "vosees": "vosear",
    "voseo": "vosear",
    "voseos": "voseo",
    "voseábamos": "vosear",
    "voseáis": "vosear",
    "voseáramos": "vosear",
    "voseáremos": "vosear",
    "voseásemos": "vosear",
    "voseé": "vosear",
    "voseéis": "vosear",
    "voseó": "vosear",
    "vosotras": "vosotros",
    "vota": "votar",
    "votaba": "votar",
    "votabais": "votar",
    "votaban": "votar",
    "votabas": "votar",
    "votaciones": "votación",
    "votad": "votar",
    "votada": "votar",
    "votadas": "votar",
    "votado": "votar",
    "votados": "votar",
    "votamos": "votar",
    "votan": "votar",
    "votando": "votar",
    "votantes": "votante",
    "votara": "votar",
    "votarais": "votar",
    "votaran": "votar",
    "votaras": "votar",
    "votare": "votar",
    "votareis": "votar",
    "votaremos": "votar",
    "votaren": "votar",
    "votares": "votar",
    "votaron": "votar",
    "votará": "votar",
    "votarán": "votar",
    "votarás": "votar",
    "votaré": "votar",
    "votaréis": "votar",
    "votaría": "votar",
    "votaríais": "votar",
    "votaríamos": "votar",
    "votarían": "votar",
    "votarías": "votar",
    "votas": "votar",
    "votase": "votar",
    "votaseis": "votar",
    "votasen": "votar",
    "votases": "votar",
    "votaste": "votar",
    "votasteis": "votar",
    "vote": "votar",
    "votemos": "votar",
    "voten": "votar",
    "votes": "votar",
    "votiva": "votivo",
    "votivas": "votivo",
    "votivos": "votivo",
    "voto": "votar",
    "votos": "voto",
    "votábamos": "votar",
    "votáis": "votar",
    "votáramos": "votar",
    "votáremos": "votar",
    "votásemos": "votar",
    "voté": "votar",
    "votéis": "votar",
    "votó": "votar",
    "voy": "ir",
    "voyeures": "voyeur",
    "voyeurismos": "voyeurismo",
    "voyeurs": "voyeur",
    "vozarrona": "vozarrón",
    "vozarronas": "vozarrón",
    "vozarrones": "vozarrón",
    "vudúes": "vudú",
    "vudús": "vudú",
    "vuecelencias": "vuecelencia",
    "vuecencias": "vuecencia",
    "vuela": "volar",
    "vuelan": "volar",
    "vuelas": "volar",
    "vuelca": "volcar",
    "vuelcan": "volcar",
    "vuelcas": "volcar",
    "vuelco": "volcar",
    "vuelcos": "vuelco",
    "vuele": "volar",
    "vuelen": "volar",
    "vueles": "volar",
    "vuelillos": "vuelillo",
    "vuelo": "volar",
    "vuelos": "vuelo",
    "vuelque": "volcar",
    "vuelquen": "volcar",
    "vuelques": "volcar",
    "vuelta": "volver",
    "vueltas": "vuelta",
    "vuelto": "volver",
    "vueltos": "volver",
    "vueluda": "vueludo",
    "vueludas": "vueludo",
    "vueludos": "vueludo",
    "vuelva": "volver",
    "vuelvan": "volver",
    "vuelvas": "volver",
    "vuelve": "volver",
    "vuelven": "volver",
    "vuelves": "volver",
    "vuelvo": "volver",
    "vuestra": "vuestro",
    "vuestras": "vuestro",
    "vuestros": "vuestro",
    "vulcania": "vulcanio",
    "vulcanias": "vulcanio",
    "vulcanice": "vulcanizar",
    "vulcanicemos": "vulcanizar",
    "vulcanicen": "vulcanizar",
    "vulcanices": "vulcanizar",
    "vulcanicé": "vulcanizar",
    "vulcanicéis": "vulcanizar",
    "vulcanios": "vulcanio",
    "vulcanismos": "vulcanismo",
    "vulcanistas": "vulcanista",
    "vulcanitas": "vulcanita",
    "vulcaniza": "vulcanizar",
    "vulcanizaba": "vulcanizar",
    "vulcanizabais": "vulcanizar",
    "vulcanizaban": "vulcanizar",
    "vulcanizabas": "vulcanizar",
    "vulcanizaciones": "vulcanización",
    "vulcanizad": "vulcanizar",
    "vulcanizada": "vulcanizar",
    "vulcanizadas": "vulcanizar",
    "vulcanizado": "vulcanizar",
    "vulcanizadores": "vulcanizador",
    "vulcanizados": "vulcanizar",
    "vulcanizamos": "vulcanizar",
    "vulcanizan": "vulcanizar",
    "vulcanizando": "vulcanizar",
    "vulcanizantes": "vulcanizante",
    "vulcanizara": "vulcanizar",
    "vulcanizarais": "vulcanizar",
    "vulcanizaran": "vulcanizar",
    "vulcanizaras": "vulcanizar",
    "vulcanizare": "vulcanizar",
    "vulcanizareis": "vulcanizar",
    "vulcanizaremos": "vulcanizar",
    "vulcanizaren": "vulcanizar",
    "vulcanizares": "vulcanizar",
    "vulcanizaron": "vulcanizar",
    "vulcanizará": "vulcanizar",
    "vulcanizarán": "vulcanizar",
    "vulcanizarás": "vulcanizar",
    "vulcanizaré": "vulcanizar",
    "vulcanizaréis": "vulcanizar",
    "vulcanizaría": "vulcanizar",
    "vulcanizaríais": "vulcanizar",
    "vulcanizaríamos": "vulcanizar",
    "vulcanizarían": "vulcanizar",
    "vulcanizarías": "vulcanizar",
    "vulcanizas": "vulcanizar",
    "vulcanizase": "vulcanizar",
    "vulcanizaseis": "vulcanizar",
    "vulcanizasen": "vulcanizar",
    "vulcanizases": "vulcanizar",
    "vulcanizaste": "vulcanizar",
    "vulcanizasteis": "vulcanizar",
    "vulcanizo": "vulcanizar",
    "vulcanizábamos": "vulcanizar",
    "vulcanizáis": "vulcanizar",
    "vulcanizáramos": "vulcanizar",
    "vulcanizáremos": "vulcanizar",
    "vulcanizásemos": "vulcanizar",
    "vulcanizó": "vulcanizar",
    "vulcanologías": "vulcanología",
    "vulcanológica": "vulcanológico",
    "vulcanológicas": "vulcanológico",
    "vulcanológicos": "vulcanológico",
    "vulcanóloga": "vulcanólogo",
    "vulcanólogas": "vulcanólogo",
    "vulcanólogos": "vulcanólogo",
    "vulgachos": "vulgacho",
    "vulgares": "vulgar",
    "vulgarice": "vulgarizar",
    "vulgaricemos": "vulgarizar",
    "vulgaricen": "vulgarizar",
    "vulgarices": "vulgarizar",
    "vulgaricé": "vulgarizar",
    "vulgaricéis": "vulgarizar",
    "vulgaridades": "vulgaridad",
    "vulgarismos": "vulgarismo",
    "vulgariza": "vulgarizar",
    "vulgarizaba": "vulgarizar",
    "vulgarizabais": "vulgarizar",
    "vulgarizaban": "vulgarizar",
    "vulgarizabas": "vulgarizar",
    "vulgarizaciones": "vulgarización",
    "vulgarizad": "vulgarizar",
    "vulgarizada": "vulgarizar",
    "vulgarizadas": "vulgarizar",
    "vulgarizado": "vulgarizar",
    "vulgarizadora": "vulgarizador",
    "vulgarizadoras": "vulgarizador",
    "vulgarizadores": "vulgarizador",
    "vulgarizados": "vulgarizar",
    "vulgarizamos": "vulgarizar",
    "vulgarizan": "vulgarizar",
    "vulgarizando": "vulgarizar",
    "vulgarizara": "vulgarizar",
    "vulgarizarais": "vulgarizar",
    "vulgarizaran": "vulgarizar",
    "vulgarizaras": "vulgarizar",
    "vulgarizare": "vulgarizar",
    "vulgarizareis": "vulgarizar",
    "vulgarizaremos": "vulgarizar",
    "vulgarizaren": "vulgarizar",
    "vulgarizares": "vulgarizar",
    "vulgarizaron": "vulgarizar",
    "vulgarizará": "vulgarizar",
    "vulgarizarán": "vulgarizar",
    "vulgarizarás": "vulgarizar",
    "vulgarizaré": "vulgarizar",
    "vulgarizaréis": "vulgarizar",
    "vulgarizaría": "vulgarizar",
    "vulgarizaríais": "vulgarizar",
    "vulgarizaríamos": "vulgarizar",
    "vulgarizarían": "vulgarizar",
    "vulgarizarías": "vulgarizar",
    "vulgarizas": "vulgarizar",
    "vulgarizase": "vulgarizar",
    "vulgarizaseis": "vulgarizar",
    "vulgarizasen": "vulgarizar",
    "vulgarizases": "vulgarizar",
    "vulgarizaste": "vulgarizar",
    "vulgarizasteis": "vulgarizar",
    "vulgarizo": "vulgarizar",
    "vulgarizábamos": "vulgarizar",
    "vulgarizáis": "vulgarizar",
    "vulgarizáramos": "vulgarizar",
    "vulgarizáremos": "vulgarizar",
    "vulgarizásemos": "vulgarizar",
    "vulgarizó": "vulgarizar",
    "vulgatas": "vulgata",
    "vulgos": "vulgo",
    "vulnera": "vulnerar",
    "vulneraba": "vulnerar",
    "vulnerabais": "vulnerar",
    "vulneraban": "vulnerar",
    "vulnerabas": "vulnerar",
    "vulnerabilidades": "vulnerabilidad",
    "vulnerabilizaciones": "vulnerabilización",
    "vulnerables": "vulnerable",
    "vulneraciones": "vulneración",
    "vulnerad": "vulnerar",
    "vulnerada": "vulnerar",
    "vulneradas": "vulnerar",
    "vulnerado": "vulnerar",
    "vulnerados": "vulnerar",
    "vulneramos": "vulnerar",
    "vulneran": "vulnerar",
    "vulnerando": "vulnerar",
    "vulnerara": "vulnerar",
    "vulnerarais": "vulnerar",
    "vulneraran": "vulnerar",
    "vulneraras": "vulnerar",
    "vulnerare": "vulnerar",
    "vulnerareis": "vulnerar",
    "vulneraremos": "vulnerar",
    "vulneraren": "vulnerar",
    "vulnerares": "vulnerar",
    "vulneraria": "vulnerario",
    "vulnerarias": "vulnerario",
    "vulnerarios": "vulnerario",
    "vulneraron": "vulnerar",
    "vulnerará": "vulnerar",
    "vulnerarán": "vulnerar",
    "vulnerarás": "vulnerar",
    "vulneraré": "vulnerar",
    "vulneraréis": "vulnerar",
    "vulneraría": "vulnerar",
    "vulneraríais": "vulnerar",
    "vulneraríamos": "vulnerar",
    "vulnerarían": "vulnerar",
    "vulnerarías": "vulnerar",
    "vulneras": "vulnerar",
    "vulnerase": "vulnerar",
    "vulneraseis": "vulnerar",
    "vulnerasen": "vulnerar",
    "vulnerases": "vulnerar",
    "vulneraste": "vulnerar",
    "vulnerasteis": "vulnerar",
    "vulnere": "vulnerar",
    "vulneremos": "vulnerar",
    "vulneren": "vulnerar",
    "vulneres": "vulnerar",
    "vulnero": "vulnerar",
    "vulnerábamos": "vulnerar",
    "vulneráis": "vulnerar",
    "vulneráramos": "vulnerar",
    "vulneráremos": "vulnerar",
    "vulnerásemos": "vulnerar",
    "vulneré": "vulnerar",
    "vulneréis": "vulnerar",
    "vulneró": "vulnerar",
    "vulpejas": "vulpeja",
    "vulpina": "vulpino",
    "vulpinas": "vulpino",
    "vulpinos": "vulpino",
    "vulpéculas": "vulpécula",
    "vultuosa": "vultuoso",
    "vultuosas": "vultuoso",
    "vultuosos": "vultuoso",
    "vulvares": "vulvar",
    "vulvas": "vulva",
    "válida": "válido",
    "válidas": "válido",
    "válidos": "válido",
    "válvulas": "válvula",
    "vándala": "vándalo",
    "vándalas": "vándalo",
    "vándalos": "vándalo",
    "várganos": "várgano",
    "várices": "várice",
    "vástagos": "vástago",
    "védica": "védico",
    "védicas": "védico",
    "védicos": "védico",
    "vélica": "vélico",
    "vélicas": "vélico",
    "vélicos": "vélico",
    "vélites": "vélite",
    "vélloras": "véllora",
    "véneta": "véneto",
    "vénetas": "véneto",
    "vénetos": "véneto",
    "vénulas": "vénula",
    "vértebras": "vértebra",
    "vértices": "vértice",
    "vértigos": "vértigo",
    "vésperos": "véspero",
    "véspidos": "véspido",
    "vías": "vía",
    "víboras": "víbora",
    "víctimas": "víctima",
    "vídeoclubes": "vídeoclub",
    "vídeoclubs": "vídeoclub",
    "vídeos": "vídeo",
    "vínculos": "vínculo",
    "vínica": "vínico",
    "vínicas": "vínico",
    "vínicos": "vínico",
    "vírgenes": "virgen",
    "vírgulas": "vírgula",
    "vírica": "vírico",
    "víricas": "vírico",
    "víricos": "vírico",
    "vísceras": "víscera",
    "vísperas": "víspera",
    "vítores": "vítor",
    "vítrea": "vítreo",
    "vítreas": "vítreo",
    "vítreos": "vítreo",
    "vívida": "vívido",
    "vívidas": "vívido",
    "vívidos": "vívido",
    "vólvulos": "vólvulo",
    "vómeres": "vómer",
    "vómica": "vómico",
    "vómicas": "vómico",
    "vómicos": "vómico",
    "vómitos": "vómito",
    "vórtexes": "vórtex",
    "vórtices": "vórtice",
    "wagneriana": "wagneriano",
    "wagnerianas": "wagneriano",
    "wagnerianos": "wagneriano",
    "wahhabíes": "wahhabí",
    "wahhabís": "wahhabí",
    "walis": "wali",
    "walkie-talkies": "walkie-talkie",
    "walkietalkies": "walkietalkie",
    "warditas": "wardita",
    "warfarinas": "warfarina",
    "washingtoniana": "washingtoniano",
    "washingtonianas": "washingtoniano",
    "washingtonianos": "washingtoniano",
    "water-closets": "water-closet",
    "waterclosets": "watercloset",
    "waterpolistas": "waterpolista",
    "waterpolos": "waterpolo",
    "waters": "wáter",
    "watio": "watt",
    "watios": "watt",
    "watt": "vatio",
    "watts": "watt",
    "watímetros": "watímetro",
    "waues": "wau",
    "wavelitas": "wavelita",
    "wavellitas": "wavellita",
    "webers": "weber",
    "webs": "web",
    "websites": "website",
    "weekends": "weekend",
    "weigelias": "weigelia",
    "welters": "welter",
    "welwitsquiácea": "welwitsquiáceo",
    "welwitsquiáceas": "welwitsquiáceo",
    "welwitsquiáceos": "welwitsquiáceo",
    "werneritas": "wernerita",
    "westerns": "western",
    "westfaliana": "westfaliano",
    "westfalianas": "westfaliano",
    "westfalianos": "westfaliano",
    "whiskeys": "whiskey",
    "whiskies": "whisky",
    "whiskys": "whisky",
    "whists": "whist",
    "wichitas": "wichita",
    "wigwams": "wigwam",
    "willemitas": "willemita",
    "williamsitas": "williamsita",
    "winches": "winche",
    "winchesters": "winchester",
    "windsurfs": "windsurf",
    "witeritas": "witerita",
    "witheritas": "witherita",
    "wobuladora": "wobulador",
    "wobuladoras": "wobulador",
    "wobuladores": "wobulador",
    "wolastonitas": "wolastonita",
    "wolframatos": "wolframato",
    "wolframios": "wolframio",
    "wolframitas": "wolframita",
    "wollastonitas": "wollastonita",
    "wones": "won",
    "wons": "won",
    "wormiana": "wormiano",
    "wormianas": "wormiano",
    "wormianos": "wormiano",
    "wulfenitas": "wulfenita",
    "wurtzitas": "wurtzita",
    "wus": "wu",
    "wáteres": "wáter",
    "wáters": "wáter",
    "wéberes": "wéber",
    "wébers": "wéber",
    "wúes": "wu",
    "xantatos": "xantato",
    "xantelasmas": "xantelasma",
    "xantenos": "xanteno",
    "xantidroles": "xantidrol",
    "xantinas": "xantina",
    "xantocilinas": "xantocilina",
    "xantocones": "xantocón",
    "xantocromías": "xantocromía",
    "xantodonta": "xantodonto",
    "xantodontas": "xantodonto",
    "xantodontos": "xantodonto",
    "xantofilas": "xantofila",
    "xantofilitas": "xantofilita",
    "xantomas": "xantoma",
    "xantonas": "xantona",
    "xantopsias": "xantopsia",
    "xantopterinas": "xantopterina",
    "xantotoxinas": "xantotoxina",
    "xenagos": "xenago",
    "xenartros": "xenartro",
    "xenofilias": "xenofilia",
    "xenofobias": "xenofobia",
    "xenogamias": "xenogamia",
    "xenogénica": "xenogénico",
    "xenogénicas": "xenogénico",
    "xenogénicos": "xenogénico",
    "xenoinducciones": "xenoinducción",
    "xenoinjertos": "xenoinjerto",
    "xenomonedas": "xenomoneda",
    "xenomórfica": "xenomórfico",
    "xenomórficas": "xenomórfico",
    "xenomórficos": "xenomórfico",
    "xenones": "xenón",
    "xenoplástica": "xenoplástico",
    "xenoplásticas": "xenoplástico",
    "xenoplásticos": "xenoplástico",
    "xenosáuridos": "xenosáurido",
    "xenotimas": "xenotima",
    "xenófoba": "xenófobo",
    "xenófobas": "xenófobo",
    "xenófobos": "xenófobo",
    "xenógena": "xenógeno",
    "xenógenas": "xenógeno",
    "xenógenos": "xenógeno",
    "xerocopias": "xerocopia",
    "xerodermias": "xerodermia",
    "xeroftalmías": "xeroftalmía",
    "xerografías": "xerografía",
    "xerográfica": "xerográfico",
    "xerográficas": "xerográfico",
    "xerográficos": "xerográfico",
    "xeros": "xero",
    "xerostomías": "xerostomía",
    "xerotelerradiografías": "xerotelerradiografía",
    "xerotérmica": "xerotérmico",
    "xerotérmicas": "xerotérmico",
    "xerotérmicos": "xerotérmico",
    "xerófaga": "xerófago",
    "xerófagas": "xerófago",
    "xerófagos": "xerófago",
    "xerófila": "xerófilo",
    "xerófilas": "xerófilo",
    "xerófilos": "xerófilo",
    "xerófita": "xerófito",
    "xerófitas": "xerófito",
    "xerófitos": "xerófito",
    "xifoidea": "xifoideo",
    "xifoideas": "xifoideo",
    "xifoideos": "xifoideo",
    "xifos": "xifo",
    "xifosuros": "xifosuro",
    "xilemas": "xilema",
    "xilenoles": "xilenol",
    "xilenos": "xileno",
    "xilidinas": "xilidina",
    "xilitoles": "xilitol",
    "xilofones": "xilofón",
    "xilofonistas": "xilofonista",
    "xilografías": "xilografía",
    "xilográfica": "xilográfico",
    "xilográficas": "xilográfico",
    "xilográficos": "xilográfico",
    "xiloles": "xilol",
    "xilolitas": "xilolita",
    "xilologías": "xilología",
    "xilológica": "xilológico",
    "xilológicas": "xilológico",
    "xilológicos": "xilológico",
    "xilometrías": "xilometría",
    "xilométrica": "xilométrico",
    "xilométricas": "xilométrico",
    "xilométricos": "xilométrico",
    "xilosas": "xilosa",
    "xilénica": "xilénico",
    "xilénicas": "xilénico",
    "xilénicos": "xilénico",
    "xilófaga": "xilófago",
    "xilófagas": "xilófago",
    "xilófagos": "xilófago",
    "xilófila": "xilófilo",
    "xilófilas": "xilófilo",
    "xilófilos": "xilófilo",
    "xilófitos": "xilófito",
    "xilófonos": "xilófono",
    "xilógrafa": "xilógrafo",
    "xilógrafas": "xilógrafo",
    "xilógrafos": "xilógrafo",
    "xilómetros": "xilómetro",
    "xilópalos": "xilópalo",
    "xis": "xi",
    "xuetas": "xueta",
    "xántica": "xántico",
    "xánticas": "xántico",
    "xánticos": "xántico",
    "xífidos": "xífido",
    "xílica": "xílico",
    "xílicas": "xílico",
    "xílicos": "xílico",
    "yabunas": "yabuna",
    "yacales": "yacal",
    "yacarés": "yacaré",
    "yace": "yacer",
    "yaced": "yacer",
    "yacemos": "yacer",
    "yacen": "yacer",
    "yacentes": "yacente",
    "yaceremos": "yacer",
    "yacerá": "yacer",
    "yacerán": "yacer",
    "yacerás": "yacer",
    "yaceré": "yacer",
    "yaceréis": "yacer",
    "yacería": "yacer",
    "yaceríais": "yacer",
    "yaceríamos": "yacer",
    "yacerían": "yacer",
    "yacerías": "yacer",
    "yaces": "yacer",
    "yacida": "yacer",
    "yacidas": "yacer",
    "yacido": "yacer",
    "yacidos": "yacer",
    "yaciendo": "yacer",
    "yacientes": "yaciente",
    "yaciera": "yacer",
    "yacierais": "yacer",
    "yacieran": "yacer",
    "yacieras": "yacer",
    "yaciere": "yacer",
    "yaciereis": "yacer",
    "yacieren": "yacer",
    "yacieres": "yacer",
    "yacieron": "yacer",
    "yaciese": "yacer",
    "yacieseis": "yacer",
    "yaciesen": "yacer",
    "yacieses": "yacer",
    "yacijas": "yacija",
    "yacimientos": "yacimiento",
    "yacimos": "yacer",
    "yacios": "yacio",
    "yaciste": "yacer",
    "yacisteis": "yacer",
    "yaciéramos": "yacer",
    "yaciéremos": "yacer",
    "yaciésemos": "yacer",
    "yació": "yacer",
    "yacéis": "yacer",
    "yací": "yacer",
    "yacía": "yacer",
    "yacíais": "yacer",
    "yacíamos": "yacer",
    "yacían": "yacer",
    "yacías": "yacer",
    "yaga": "yacer",
    "yagamos": "yacer",
    "yagan": "yacer",
    "yagas": "yacer",
    "yago": "yacer",
    "yaguales": "yagual",
    "yaguares": "yaguar",
    "yaguas": "yagua",
    "yaguasas": "yaguasa",
    "yaguazas": "yaguaza",
    "yagáis": "yacer",
    "yakes": "yak",
    "yakimonos": "yakimono",
    "yakis": "yaki",
    "yaks": "yak",
    "yambos": "yambo",
    "yanaconas": "yanacona",
    "yankees": "yankee",
    "yanquis": "yanqui",
    "yantares": "yantar",
    "yapoes": "yapó",
    "yapós": "yapó",
    "yaquiles": "yaquil",
    "yaquis": "yaqui",
    "yararaes": "yarará",
    "yararás": "yarará",
    "yaravíes": "yaraví",
    "yaravís": "yaraví",
    "yardas": "yarda",
    "yareis": "yarey",
    "yares": "yare",
    "yareyes": "yarey",
    "yareys": "yarey",
    "yaros": "yaro",
    "yataganes": "yatagán",
    "yatais": "yatay",
    "yatayes": "yatay",
    "yatays": "yatay",
    "yates": "yate",
    "yatrofísicas": "yatrofísica",
    "yatrogenias": "yatrogenia",
    "yatrogénica": "yatrogénico",
    "yatrogénicas": "yatrogénico",
    "yatrogénicos": "yatrogénico",
    "yatroquímicas": "yatroquímica",
    "yatrógena": "yatrógeno",
    "yatrógenas": "yatrógeno",
    "yatrógenos": "yatrógeno",
    "yayas": "yaya",
    "yayos": "yayo",
    "yaz": "yacer",
    "yazca": "yacer",
    "yazcamos": "yacer",
    "yazcan": "yacer",
    "yazcas": "yacer",
    "yazco": "yacer",
    "yazcáis": "yacer",
    "yazga": "yacer",
    "yazgamos": "yacer",
    "yazgan": "yacer",
    "yazgas": "yacer",
    "yazgo": "yacer",
    "yazgáis": "yacer",
    "yedras": "yedra",
    "yeguadas": "yeguada",
    "yeguares": "yeguar",
    "yeguas": "yegua",
    "yeguata": "yeguato",
    "yeguatas": "yeguato",
    "yeguatos": "yeguato",
    "yegüera": "yegüero",
    "yegüeras": "yegüero",
    "yegüeriza": "yegüerizo",
    "yegüerizas": "yegüerizo",
    "yegüerizos": "yegüerizo",
    "yegüeros": "yegüero",
    "yelmos": "yelmo",
    "yemas": "yema",
    "yemenises": "yemení",
    "yemenitas": "yemenita",
    "yemeníes": "yemení",
    "yemenís": "yemení",
    "yemeníses": "yemení",
    "yendo": "ir",
    "yenes": "yen",
    "yerales": "yeral",
    "yerbas": "yerba",
    "yerbatera": "yerbatero",
    "yerbateras": "yerbatero",
    "yerbateros": "yerbatero",
    "yerbea": "yerbear",
    "yerbeaba": "yerbear",
    "yerbeabais": "yerbear",
    "yerbeaban": "yerbear",
    "yerbeabas": "yerbear",
    "yerbead": "yerbear",
    "yerbeada": "yerbear",
    "yerbeadas": "yerbear",
    "yerbeado": "yerbear",
    "yerbeados": "yerbear",
    "yerbeamos": "yerbear",
    "yerbean": "yerbear",
    "yerbeando": "yerbear",
    "yerbeara": "yerbear",
    "yerbearais": "yerbear",
    "yerbearan": "yerbear",
    "yerbearas": "yerbear",
    "yerbeare": "yerbear",
    "yerbeareis": "yerbear",
    "yerbearemos": "yerbear",
    "yerbearen": "yerbear",
    "yerbeares": "yerbear",
    "yerbearon": "yerbear",
    "yerbeará": "yerbear",
    "yerbearán": "yerbear",
    "yerbearás": "yerbear",
    "yerbearé": "yerbear",
    "yerbearéis": "yerbear",
    "yerbearía": "yerbear",
    "yerbearíais": "yerbear",
    "yerbearíamos": "yerbear",
    "yerbearían": "yerbear",
    "yerbearías": "yerbear",
    "yerbeas": "yerbear",
    "yerbease": "yerbear",
    "yerbeaseis": "yerbear",
    "yerbeasen": "yerbear",
    "yerbeases": "yerbear",
    "yerbeaste": "yerbear",
    "yerbeasteis": "yerbear",
    "yerbee": "yerbear",
    "yerbeemos": "yerbear",
    "yerbeen": "yerbear",
    "yerbees": "yerbear",
    "yerbeo": "yerbear",
    "yerberas": "yerbera",
    "yerbeábamos": "yerbear",
    "yerbeáis": "yerbear",
    "yerbeáramos": "yerbear",
    "yerbeáremos": "yerbear",
    "yerbeásemos": "yerbear",
    "yerbeé": "yerbear",
    "yerbeéis": "yerbear",
    "yerbeó": "yerbear",
    "yerga": "erguir",
    "yergan": "erguir",
    "yergas": "erguir",
    "yergo": "erguir",
    "yergue": "erguir",
    "yerguen": "erguir",
    "yergues": "erguir",
    "yerma": "yermo",
    "yermas": "yermo",
    "yermos": "yermo",
    "yernos": "yerno",
    "yeros": "yero",
    "yerra": "errar",
    "yerran": "errar",
    "yerras": "errar",
    "yerre": "errar",
    "yerren": "errar",
    "yerres": "errar",
    "yerro": "errar",
    "yerros": "yerro",
    "yerta": "yerto",
    "yertas": "yerto",
    "yertos": "yerto",
    "yes": "ye",
    "yesales": "yesal",
    "yesares": "yesar",
    "yescas": "yesca",
    "yesera": "yesero",
    "yeseras": "yesero",
    "yeseros": "yesero",
    "yeserías": "yesería",
    "yesones": "yesón",
    "yesos": "yeso",
    "yesosa": "yesoso",
    "yesosas": "yesoso",
    "yesosos": "yesoso",
    "yesquera": "yesquero",
    "yesqueras": "yesquero",
    "yesqueros": "yesquero",
    "yetis": "yeti",
    "yeyunales": "yeyunal",
    "yeyunos": "yeyuno",
    "yezgos": "yezgo",
    "yeísmos": "yeísmo",
    "yeístas": "yeísta",
    "yoda": "yodar",
    "yodaba": "yodar",
    "yodabais": "yodar",
    "yodaban": "yodar",
    "yodabas": "yodar",
    "yodaciones": "yodación",
    "yodad": "yodar",
    "yodada": "yodar",
    "yodadas": "yodar",
    "yodado": "yodar",
    "yodados": "yodar",
    "yodamos": "yodar",
    "yodan": "yodar",
    "yodando": "yodar",
    "yodara": "yodar",
    "yodarais": "yodar",
    "yodaran": "yodar",
    "yodaras": "yodar",
    "yodare": "yodar",
    "yodareis": "yodar",
    "yodaremos": "yodar",
    "yodaren": "yodar",
    "yodares": "yodar",
    "yodargiritas": "yodargirita",
    "yodaron": "yodar",
    "yodará": "yodar",
    "yodarán": "yodar",
    "yodarás": "yodar",
    "yodaré": "yodar",
    "yodaréis": "yodar",
    "yodaría": "yodar",
    "yodaríais": "yodar",
    "yodaríamos": "yodar",
    "yodarían": "yodar",
    "yodarías": "yodar",
    "yodas": "yodar",
    "yodase": "yodar",
    "yodaseis": "yodar",
    "yodasen": "yodar",
    "yodases": "yodar",
    "yodaste": "yodar",
    "yodasteis": "yodar",
    "yodatos": "yodato",
    "yode": "yodar",
    "yodemos": "yodar",
    "yoden": "yodar",
    "yodes": "yodar",
    "yodhídrica": "yodhídrico",
    "yodhídricas": "yodhídrico",
    "yodhídricos": "yodhídrico",
    "yodo": "yodar",
    "yodoeosinas": "yodoeosina",
    "yodofilias": "yodofilia",
    "yodoformos": "yodoformo",
    "yodometrías": "yodometría",
    "yodos": "yodo",
    "yodosuccinimidas": "yodosuccinimida",
    "yods": "yod",
    "yodura": "yodurar",
    "yoduraba": "yodurar",
    "yodurabais": "yodurar",
    "yoduraban": "yodurar",
    "yodurabas": "yodurar",
    "yodurad": "yodurar",
    "yodurada": "yodurar",
    "yoduradas": "yodurar",
    "yodurado": "yodurar",
    "yodurados": "yodurar",
    "yoduramos": "yodurar",
    "yoduran": "yodurar",
    "yodurando": "yodurar",
    "yodurara": "yodurar",
    "yodurarais": "yodurar",
    "yoduraran": "yodurar",
    "yoduraras": "yodurar",
    "yodurare": "yodurar",
    "yodurareis": "yodurar",
    "yoduraremos": "yodurar",
    "yoduraren": "yodurar",
    "yodurares": "yodurar",
    "yoduraron": "yodurar",
    "yodurará": "yodurar",
    "yodurarán": "yodurar",
    "yodurarás": "yodurar",
    "yoduraré": "yodurar",
    "yoduraréis": "yodurar",
    "yoduraría": "yodurar",
    "yoduraríais": "yodurar",
    "yoduraríamos": "yodurar",
    "yodurarían": "yodurar",
    "yodurarías": "yodurar",
    "yoduras": "yodurar",
    "yodurase": "yodurar",
    "yoduraseis": "yodurar",
    "yodurasen": "yodurar",
    "yodurases": "yodurar",
    "yoduraste": "yodurar",
    "yodurasteis": "yodurar",
    "yodure": "yodurar",
    "yoduremos": "yodurar",
    "yoduren": "yodurar",
    "yodures": "yodurar",
    "yoduro": "yodurar",
    "yoduros": "yoduro",
    "yodurábamos": "yodurar",
    "yoduráis": "yodurar",
    "yoduráramos": "yodurar",
    "yoduráremos": "yodurar",
    "yodurásemos": "yodurar",
    "yoduré": "yodurar",
    "yoduréis": "yodurar",
    "yoduró": "yodurar",
    "yodábamos": "yodar",
    "yodáis": "yodar",
    "yodáramos": "yodar",
    "yodáremos": "yodar",
    "yodásemos": "yodar",
    "yodé": "yodar",
    "yodéis": "yodar",
    "yodó": "yodar",
    "yogas": "yoga",
    "yoghourts": "yoghourt",
    "yoguis": "yogui",
    "yogures": "yogur",
    "yogurt": "yogur",
    "yogurts": "yogur",
    "yohimbinas": "yohimbina",
    "yolas": "yola",
    "yoles": "yol",
    "yoletas": "yoleta",
    "yolofs": "yolof",
    "yonquis": "yonqui",
    "yoqueis": "yóquey",
    "yoquis": "yoqui",
    "yorubas": "yoruba",
    "yotizaciones": "yotización",
    "yottabytes": "yottabyte",
    "yoyos": "yoyo",
    "yterbios": "yterbio",
    "ytrios": "ytrio",
    "yuanes": "yuan",
    "yuans": "yuan",
    "yubartas": "yubarta",
    "yucales": "yucal",
    "yucas": "yuca",
    "yucateca": "yucateco",
    "yucatecas": "yucateco",
    "yucatecos": "yucateco",
    "yudoguis": "yudogui",
    "yudokas": "yudoka",
    "yudos": "yudo",
    "yugadas": "yugada",
    "yugales": "yugal",
    "yugos": "yugo",
    "yugoslava": "yugoslavo",
    "yugoslavas": "yugoslavo",
    "yugoslavos": "yugoslavo",
    "yugueros": "yuguero",
    "yugula": "yugular",
    "yugulaba": "yugular",
    "yugulabais": "yugular",
    "yugulaban": "yugular",
    "yugulabas": "yugular",
    "yugulad": "yugular",
    "yugulada": "yugular",
    "yuguladas": "yugular",
    "yugulado": "yugular",
    "yugulados": "yugular",
    "yugulamos": "yugular",
    "yugulan": "yugular",
    "yugulando": "yugular",
    "yugulara": "yugular",
    "yugularais": "yugular",
    "yugularan": "yugular",
    "yugularas": "yugular",
    "yugulare": "yugular",
    "yugulareis": "yugular",
    "yugularemos": "yugular",
    "yugularen": "yugular",
    "yugulares": "yugular",
    "yugularon": "yugular",
    "yugulará": "yugular",
    "yugularán": "yugular",
    "yugularás": "yugular",
    "yugularé": "yugular",
    "yugularéis": "yugular",
    "yugularía": "yugular",
    "yugularíais": "yugular",
    "yugularíamos": "yugular",
    "yugularían": "yugular",
    "yugularías": "yugular",
    "yugulas": "yugular",
    "yugulase": "yugular",
    "yugulaseis": "yugular",
    "yugulasen": "yugular",
    "yugulases": "yugular",
    "yugulaste": "yugular",
    "yugulasteis": "yugular",
    "yugule": "yugular",
    "yugulemos": "yugular",
    "yugulen": "yugular",
    "yugules": "yugular",
    "yugulo": "yugular",
    "yugulodigástrica": "yugulodigástrico",
    "yugulodigástricas": "yugulodigástrico",
    "yugulodigástricos": "yugulodigástrico",
    "yuguloomohioidea": "yuguloomohioideo",
    "yuguloomohioideas": "yuguloomohioideo",
    "yuguloomohioideos": "yuguloomohioideo",
    "yugulábamos": "yugular",
    "yuguláis": "yugular",
    "yuguláramos": "yugular",
    "yuguláremos": "yugular",
    "yugulásemos": "yugular",
    "yugulé": "yugular",
    "yuguléis": "yugular",
    "yuguló": "yugular",
    "yumba": "yumbo",
    "yumbas": "yumbo",
    "yumbos": "yumbo",
    "yunques": "yunque",
    "yuntas": "yunta",
    "yunteros": "yuntero",
    "yunterías": "yuntería",
    "yuppies": "yuppy",
    "yurtas": "yurta",
    "yurumíes": "yurumí",
    "yurumís": "yurumí",
    "yuseras": "yusera",
    "yutes": "yute",
    "yuxtalineales": "yuxtalineal",
    "yuxtapapilares": "yuxtapapilar",
    "yuxtapondremos": "yuxtaponer",
    "yuxtapondrá": "yuxtaponer",
    "yuxtapondrán": "yuxtaponer",
    "yuxtapondrás": "yuxtaponer",
    "yuxtapondré": "yuxtaponer",
    "yuxtapondréis": "yuxtaponer",
    "yuxtapondría": "yuxtaponer",
    "yuxtapondríais": "yuxtaponer",
    "yuxtapondríamos": "yuxtaponer",
    "yuxtapondrían": "yuxtaponer",
    "yuxtapondrías": "yuxtaponer",
    "yuxtapone": "yuxtaponer",
    "yuxtaponed": "yuxtaponer",
    "yuxtaponemos": "yuxtaponer",
    "yuxtaponen": "yuxtaponer",
    "yuxtapones": "yuxtaponer",
    "yuxtaponga": "yuxtaponer",
    "yuxtapongamos": "yuxtaponer",
    "yuxtapongan": "yuxtaponer",
    "yuxtapongas": "yuxtaponer",
    "yuxtapongo": "yuxtaponer",
    "yuxtapongáis": "yuxtaponer",
    "yuxtaponiendo": "yuxtaponer",
    "yuxtaponéis": "yuxtaponer",
    "yuxtaponía": "yuxtaponer",
    "yuxtaponíais": "yuxtaponer",
    "yuxtaponíamos": "yuxtaponer",
    "yuxtaponían": "yuxtaponer",
    "yuxtaponías": "yuxtaponer",
    "yuxtaposiciones": "yuxtaposición",
    "yuxtapuesta": "yuxtaponer",
    "yuxtapuestas": "yuxtaponer",
    "yuxtapuesto": "yuxtaponer",
    "yuxtapuestos": "yuxtaponer",
    "yuxtapuse": "yuxtaponer",
    "yuxtapusiera": "yuxtaponer",
    "yuxtapusierais": "yuxtaponer",
    "yuxtapusieran": "yuxtaponer",
    "yuxtapusieras": "yuxtaponer",
    "yuxtapusiere": "yuxtaponer",
    "yuxtapusiereis": "yuxtaponer",
    "yuxtapusieren": "yuxtaponer",
    "yuxtapusieres": "yuxtaponer",
    "yuxtapusieron": "yuxtaponer",
    "yuxtapusiese": "yuxtaponer",
    "yuxtapusieseis": "yuxtaponer",
    "yuxtapusiesen": "yuxtaponer",
    "yuxtapusieses": "yuxtaponer",
    "yuxtapusimos": "yuxtaponer",
    "yuxtapusiste": "yuxtaponer",
    "yuxtapusisteis": "yuxtaponer",
    "yuxtapusiéramos": "yuxtaponer",
    "yuxtapusiéremos": "yuxtaponer",
    "yuxtapusiésemos": "yuxtaponer",
    "yuxtapuso": "yuxtaponer",
    "yuxtapón": "yuxtaponer",
    "yámbica": "yámbico",
    "yámbicas": "yámbico",
    "yámbicos": "yámbico",
    "yódica": "yódico",
    "yódicas": "yódico",
    "yódicos": "yódico",
    "zabazalas": "zabazala",
    "zabidas": "zabida",
    "zabilas": "zabila",
    "zaborda": "zabordar",
    "zabordaba": "zabordar",
    "zabordabais": "zabordar",
    "zabordaban": "zabordar",
    "zabordabas": "zabordar",
    "zabordad": "zabordar",
    "zabordada": "zabordar",
    "zabordadas": "zabordar",
    "zabordado": "zabordar",
    "zabordados": "zabordar",
    "zabordamientos": "zabordamiento",
    "zabordamos": "zabordar",
    "zabordan": "zabordar",
    "zabordando": "zabordar",
    "zabordara": "zabordar",
    "zabordarais": "zabordar",
    "zabordaran": "zabordar",
    "zabordaras": "zabordar",
    "zabordare": "zabordar",
    "zabordareis": "zabordar",
    "zabordaremos": "zabordar",
    "zabordaren": "zabordar",
    "zabordares": "zabordar",
    "zabordaron": "zabordar",
    "zabordará": "zabordar",
    "zabordarán": "zabordar",
    "zabordarás": "zabordar",
    "zabordaré": "zabordar",
    "zabordaréis": "zabordar",
    "zabordaría": "zabordar",
    "zabordaríais": "zabordar",
    "zabordaríamos": "zabordar",
    "zabordarían": "zabordar",
    "zabordarías": "zabordar",
    "zabordas": "zabordar",
    "zabordase": "zabordar",
    "zabordaseis": "zabordar",
    "zabordasen": "zabordar",
    "zabordases": "zabordar",
    "zabordaste": "zabordar",
    "zabordasteis": "zabordar",
    "zaborde": "zabordar",
    "zabordemos": "zabordar",
    "zaborden": "zabordar",
    "zabordes": "zabordar",
    "zabordo": "zabordar",
    "zabordos": "zabordo",
    "zabordábamos": "zabordar",
    "zabordáis": "zabordar",
    "zabordáramos": "zabordar",
    "zabordáremos": "zabordar",
    "zabordásemos": "zabordar",
    "zabordé": "zabordar",
    "zabordéis": "zabordar",
    "zabordó": "zabordar",
    "zacas": "zaca",
    "zacateca": "zacateco",
    "zacatecana": "zacatecano",
    "zacatecanas": "zacatecano",
    "zacatecanos": "zacatecano",
    "zacatecas": "zacateco",
    "zacatecos": "zacateco",
    "zaceos": "zaceo",
    "zaceosa": "zaceoso",
    "zaceosas": "zaceoso",
    "zaceosos": "zaceoso",
    "zafa": "zafar",
    "zafaba": "zafar",
    "zafabais": "zafar",
    "zafaban": "zafar",
    "zafabas": "zafar",
    "zafad": "zafar",
    "zafada": "zafar",
    "zafadas": "zafar",
    "zafado": "zafar",
    "zafados": "zafar",
    "zafaduras": "zafadura",
    "zafamos": "zafar",
    "zafan": "zafar",
    "zafando": "zafar",
    "zafara": "zafar",
    "zafarais": "zafar",
    "zafaran": "zafar",
    "zafaras": "zafar",
    "zafare": "zafar",
    "zafareis": "zafar",
    "zafaremos": "zafar",
    "zafaren": "zafar",
    "zafares": "zafar",
    "zafaron": "zafar",
    "zafarranchos": "zafarrancho",
    "zafará": "zafar",
    "zafarán": "zafar",
    "zafarás": "zafar",
    "zafaré": "zafar",
    "zafaréis": "zafar",
    "zafaría": "zafar",
    "zafaríais": "zafar",
    "zafaríamos": "zafar",
    "zafarían": "zafar",
    "zafarías": "zafar",
    "zafas": "zafar",
    "zafase": "zafar",
    "zafaseis": "zafar",
    "zafasen": "zafar",
    "zafases": "zafar",
    "zafaste": "zafar",
    "zafasteis": "zafar",
    "zafe": "zafar",
    "zafemos": "zafar",
    "zafen": "zafar",
    "zaferos": "zafero",
    "zafes": "zafar",
    "zafia": "zafio",
    "zafias": "zafio",
    "zafios": "zafio",
    "zafirina": "zafirino",
    "zafirinas": "zafirino",
    "zafirinos": "zafirino",
    "zafiros": "zafiro",
    "zafo": "zafar",
    "zafras": "zafra",
    "zafreros": "zafrero",
    "zafres": "zafre",
    "zafábamos": "zafar",
    "zafáis": "zafar",
    "zafáramos": "zafar",
    "zafáremos": "zafar",
    "zafásemos": "zafar",
    "zafé": "zafar",
    "zaféis": "zafar",
    "zafírea": "zafíreo",
    "zafíreas": "zafíreo",
    "zafíreos": "zafíreo",
    "zafó": "zafar",
    "zagalas": "zagala",
    "zagalejos": "zagalejo",
    "zagales": "zagal",
    "zagalesca": "zagalesco",
    "zagalescas": "zagalesco",
    "zagalescos": "zagalesco",
    "zagalona": "zagalón",
    "zagalonas": "zagalón",
    "zagalones": "zagalón",
    "zagas": "zaga",
    "zaguales": "zagual",
    "zaguanes": "zaguán",
    "zaguera": "zaguero",
    "zagueras": "zaguero",
    "zagueros": "zaguero",
    "zagüías": "zagüía",
    "zahareña": "zahareño",
    "zahareñas": "zahareño",
    "zahareños": "zahareño",
    "zaherid": "zaherir",
    "zaherida": "zaherir",
    "zaheridas": "zaherir",
    "zaherido": "zaherir",
    "zaheridora": "zaheridor",
    "zaheridoras": "zaheridor",
    "zaheridores": "zaheridor",
    "zaheridos": "zaherir",
    "zaherimientos": "zaherimiento",
    "zaherimos": "zaherir",
    "zaheriremos": "zaherir",
    "zaherirá": "zaherir",
    "zaherirán": "zaherir",
    "zaherirás": "zaherir",
    "zaheriré": "zaherir",
    "zaheriréis": "zaherir",
    "zaheriría": "zaherir",
    "zaheriríais": "zaherir",
    "zaheriríamos": "zaherir",
    "zaherirían": "zaherir",
    "zaherirías": "zaherir",
    "zaheriste": "zaherir",
    "zaheristeis": "zaherir",
    "zaherí": "zaherir",
    "zahería": "zaherir",
    "zaheríais": "zaherir",
    "zaheríamos": "zaherir",
    "zaherían": "zaherir",
    "zaherías": "zaherir",
    "zaherís": "zaherir",
    "zahiera": "zaherir",
    "zahieran": "zaherir",
    "zahieras": "zaherir",
    "zahiere": "zaherir",
    "zahieren": "zaherir",
    "zahieres": "zaherir",
    "zahiero": "zaherir",
    "zahinares": "zahinar",
    "zahiramos": "zaherir",
    "zahiriendo": "zaherir",
    "zahiriera": "zaherir",
    "zahirierais": "zaherir",
    "zahirieran": "zaherir",
    "zahirieras": "zaherir",
    "zahiriere": "zaherir",
    "zahiriereis": "zaherir",
    "zahirieren": "zaherir",
    "zahirieres": "zaherir",
    "zahirieron": "zaherir",
    "zahiriese": "zaherir",
    "zahirieseis": "zaherir",
    "zahiriesen": "zaherir",
    "zahirieses": "zaherir",
    "zahiriéramos": "zaherir",
    "zahiriéremos": "zaherir",
    "zahiriésemos": "zaherir",
    "zahirió": "zaherir",
    "zahiráis": "zaherir",
    "zahonada": "zahonado",
    "zahonadas": "zahonado",
    "zahonados": "zahonado",
    "zahonda": "zahondar",
    "zahondaba": "zahondar",
    "zahondabais": "zahondar",
    "zahondaban": "zahondar",
    "zahondabas": "zahondar",
    "zahondad": "zahondar",
    "zahondada": "zahondar",
    "zahondadas": "zahondar",
    "zahondado": "zahondar",
    "zahondados": "zahondar",
    "zahondamos": "zahondar",
    "zahondan": "zahondar",
    "zahondando": "zahondar",
    "zahondara": "zahondar",
    "zahondarais": "zahondar",
    "zahondaran": "zahondar",
    "zahondaras": "zahondar",
    "zahondare": "zahondar",
    "zahondareis": "zahondar",
    "zahondaremos": "zahondar",
    "zahondaren": "zahondar",
    "zahondares": "zahondar",
    "zahondaron": "zahondar",
    "zahondará": "zahondar",
    "zahondarán": "zahondar",
    "zahondarás": "zahondar",
    "zahondaré": "zahondar",
    "zahondaréis": "zahondar",
    "zahondaría": "zahondar",
    "zahondaríais": "zahondar",
    "zahondaríamos": "zahondar",
    "zahondarían": "zahondar",
    "zahondarías": "zahondar",
    "zahondas": "zahondar",
    "zahondase": "zahondar",
    "zahondaseis": "zahondar",
    "zahondasen": "zahondar",
    "zahondases": "zahondar",
    "zahondaste": "zahondar",
    "zahondasteis": "zahondar",
    "zahonde": "zahondar",
    "zahondemos": "zahondar",
    "zahonden": "zahondar",
    "zahondes": "zahondar",
    "zahondo": "zahondar",
    "zahondábamos": "zahondar",
    "zahondáis": "zahondar",
    "zahondáramos": "zahondar",
    "zahondáremos": "zahondar",
    "zahondásemos": "zahondar",
    "zahondé": "zahondar",
    "zahondéis": "zahondar",
    "zahondó": "zahondar",
    "zahones": "zahón",
    "zahorras": "zahorra",
    "zahoríes": "zahorí",
    "zahorís": "zahorí",
    "zahínas": "zahína",
    "zahúrdas": "zahúrda",
    "zaibatsus": "zaibatsu",
    "zaina": "zaino",
    "zainas": "zaino",
    "zainos": "zaino",
    "zairenses": "zairense",
    "zaireña": "zaireño",
    "zaireñas": "zaireño",
    "zaireños": "zaireño",
    "zalagardas": "zalagarda",
    "zalamas": "zalama",
    "zalamelés": "zalamelé",
    "zalamera": "zalamero",
    "zalameras": "zalamero",
    "zalameros": "zalamero",
    "zalamerías": "zalamería",
    "zalea": "zalear",
    "zaleaba": "zalear",
    "zaleabais": "zalear",
    "zaleaban": "zalear",
    "zaleabas": "zalear",
    "zalead": "zalear",
    "zaleada": "zalear",
    "zaleadas": "zalear",
    "zaleado": "zalear",
    "zaleados": "zalear",
    "zaleamos": "zalear",
    "zalean": "zalear",
    "zaleando": "zalear",
    "zaleara": "zalear",
    "zalearais": "zalear",
    "zalearan": "zalear",
    "zalearas": "zalear",
    "zaleare": "zalear",
    "zaleareis": "zalear",
    "zalearemos": "zalear",
    "zalearen": "zalear",
    "zaleares": "zalear",
    "zalearon": "zalear",
    "zaleará": "zalear",
    "zalearán": "zalear",
    "zalearás": "zalear",
    "zalearé": "zalear",
    "zalearéis": "zalear",
    "zalearía": "zalear",
    "zalearíais": "zalear",
    "zalearíamos": "zalear",
    "zalearían": "zalear",
    "zalearías": "zalear",
    "zaleas": "zalear",
    "zalease": "zalear",
    "zaleaseis": "zalear",
    "zaleasen": "zalear",
    "zaleases": "zalear",
    "zaleaste": "zalear",
    "zaleasteis": "zalear",
    "zalee": "zalear",
    "zaleemos": "zalear",
    "zaleen": "zalear",
    "zalees": "zalear",
    "zalemas": "zalema",
    "zaleo": "zalear",
    "zaleos": "zaleo",
    "zaleábamos": "zalear",
    "zaleáis": "zalear",
    "zaleáramos": "zalear",
    "zaleáremos": "zalear",
    "zaleásemos": "zalear",
    "zaleé": "zalear",
    "zaleéis": "zalear",
    "zaleó": "zalear",
    "zalomas": "zaloma",
    "zamacuca": "zamacuco",
    "zamacucas": "zamacuco",
    "zamacucos": "zamacuco",
    "zamacuecas": "zamacueca",
    "zamancas": "zamanca",
    "zamarradas": "zamarrada",
    "zamarras": "zamarra",
    "zamarrea": "zamarrear",
    "zamarreaba": "zamarrear",
    "zamarreabais": "zamarrear",
    "zamarreaban": "zamarrear",
    "zamarreabas": "zamarrear",
    "zamarread": "zamarrear",
    "zamarreada": "zamarrear",
    "zamarreadas": "zamarrear",
    "zamarreado": "zamarrear",
    "zamarreados": "zamarrear",
    "zamarreamos": "zamarrear",
    "zamarrean": "zamarrear",
    "zamarreando": "zamarrear",
    "zamarreara": "zamarrear",
    "zamarrearais": "zamarrear",
    "zamarrearan": "zamarrear",
    "zamarrearas": "zamarrear",
    "zamarreare": "zamarrear",
    "zamarreareis": "zamarrear",
    "zamarrearemos": "zamarrear",
    "zamarrearen": "zamarrear",
    "zamarreares": "zamarrear",
    "zamarrearon": "zamarrear",
    "zamarreará": "zamarrear",
    "zamarrearán": "zamarrear",
    "zamarrearás": "zamarrear",
    "zamarrearé": "zamarrear",
    "zamarrearéis": "zamarrear",
    "zamarrearía": "zamarrear",
    "zamarrearíais": "zamarrear",
    "zamarrearíamos": "zamarrear",
    "zamarrearían": "zamarrear",
    "zamarrearías": "zamarrear",
    "zamarreas": "zamarrear",
    "zamarrease": "zamarrear",
    "zamarreaseis": "zamarrear",
    "zamarreasen": "zamarrear",
    "zamarreases": "zamarrear",
    "zamarreaste": "zamarrear",
    "zamarreasteis": "zamarrear",
    "zamarree": "zamarrear",
    "zamarreemos": "zamarrear",
    "zamarreen": "zamarrear",
    "zamarrees": "zamarrear",
    "zamarreo": "zamarrear",
    "zamarreos": "zamarreo",
    "zamarreábamos": "zamarrear",
    "zamarreáis": "zamarrear",
    "zamarreáramos": "zamarrear",
    "zamarreáremos": "zamarrear",
    "zamarreásemos": "zamarrear",
    "zamarreé": "zamarrear",
    "zamarreéis": "zamarrear",
    "zamarreó": "zamarrear",
    "zamarricos": "zamarrico",
    "zamarrillas": "zamarrilla",
    "zamarros": "zamarro",
    "zamba": "zambo",
    "zambaiga": "zambaigo",
    "zambaigas": "zambaigo",
    "zambaigos": "zambaigo",
    "zambarcos": "zambarco",
    "zambas": "zambo",
    "zambombas": "zambomba",
    "zambombazos": "zambombazo",
    "zamborotuda": "zamborotudo",
    "zamborotudas": "zamborotudo",
    "zamborotudos": "zamborotudo",
    "zambos": "zambo",
    "zambras": "zambra",
    "zambucos": "zambuco",
    "zambulla": "zambullir",
    "zambullamos": "zambullir",
    "zambullan": "zambullir",
    "zambullas": "zambullir",
    "zambulle": "zambullir",
    "zambullen": "zambullir",
    "zambullendo": "zambullir",
    "zambullera": "zambullir",
    "zambullerais": "zambullir",
    "zambulleran": "zambullir",
    "zambulleras": "zambullir",
    "zambullere": "zambullir",
    "zambullereis": "zambullir",
    "zambulleren": "zambullir",
    "zambulleres": "zambullir",
    "zambulleron": "zambullir",
    "zambulles": "zambullir",
    "zambullese": "zambullir",
    "zambulleseis": "zambullir",
    "zambullesen": "zambullir",
    "zambulleses": "zambullir",
    "zambullid": "zambullir",
    "zambullida": "zambullir",
    "zambullidas": "zambullir",
    "zambullido": "zambullir",
    "zambullidora": "zambullidor",
    "zambullidoras": "zambullidor",
    "zambullidores": "zambullidor",
    "zambullidos": "zambullir",
    "zambulliduras": "zambullidura",
    "zambullimientos": "zambullimiento",
    "zambullimos": "zambullir",
    "zambulliremos": "zambullir",
    "zambullirá": "zambullir",
    "zambullirán": "zambullir",
    "zambullirás": "zambullir",
    "zambulliré": "zambullir",
    "zambulliréis": "zambullir",
    "zambulliría": "zambullir",
    "zambulliríais": "zambullir",
    "zambulliríamos": "zambullir",
    "zambullirían": "zambullir",
    "zambullirías": "zambullir",
    "zambulliste": "zambullir",
    "zambullisteis": "zambullir",
    "zambullo": "zambullir",
    "zambulláis": "zambullir",
    "zambulléramos": "zambullir",
    "zambulléremos": "zambullir",
    "zambullésemos": "zambullir",
    "zambullí": "zambullir",
    "zambullía": "zambullir",
    "zambullíais": "zambullir",
    "zambullíamos": "zambullir",
    "zambullían": "zambullir",
    "zambullías": "zambullir",
    "zambullís": "zambullir",
    "zambulló": "zambullir",
    "zamburiñas": "zamburiña",
    "zaminas": "zamina",
    "zamorana": "zamorano",
    "zamoranas": "zamorano",
    "zamoranos": "zamorano",
    "zampa": "zampar",
    "zampaba": "zampar",
    "zampabais": "zampar",
    "zampaban": "zampar",
    "zampabas": "zampar",
    "zampad": "zampar",
    "zampada": "zampar",
    "zampadas": "zampar",
    "zampado": "zampar",
    "zampados": "zampar",
    "zampamos": "zampar",
    "zampan": "zampar",
    "zampando": "zampar",
    "zampara": "zampar",
    "zamparais": "zampar",
    "zamparan": "zampar",
    "zamparas": "zampar",
    "zampare": "zampar",
    "zampareis": "zampar",
    "zamparemos": "zampar",
    "zamparen": "zampar",
    "zampares": "zampar",
    "zamparon": "zampar",
    "zampará": "zampar",
    "zamparán": "zampar",
    "zamparás": "zampar",
    "zamparé": "zampar",
    "zamparéis": "zampar",
    "zamparía": "zampar",
    "zamparíais": "zampar",
    "zamparíamos": "zampar",
    "zamparían": "zampar",
    "zamparías": "zampar",
    "zampas": "zampar",
    "zampase": "zampar",
    "zampaseis": "zampar",
    "zampasen": "zampar",
    "zampases": "zampar",
    "zampaste": "zampar",
    "zampasteis": "zampar",
    "zampe": "zampar",
    "zampea": "zampear",
    "zampeaba": "zampear",
    "zampeabais": "zampear",
    "zampeaban": "zampear",
    "zampeabas": "zampear",
    "zampead": "zampear",
    "zampeada": "zampear",
    "zampeadas": "zampear",
    "zampeado": "zampear",
    "zampeados": "zampear",
    "zampeamos": "zampear",
    "zampean": "zampear",
    "zampeando": "zampear",
    "zampeara": "zampear",
    "zampearais": "zampear",
    "zampearan": "zampear",
    "zampearas": "zampear",
    "zampeare": "zampear",
    "zampeareis": "zampear",
    "zampearemos": "zampear",
    "zampearen": "zampear",
    "zampeares": "zampear",
    "zampearon": "zampear",
    "zampeará": "zampear",
    "zampearán": "zampear",
    "zampearás": "zampear",
    "zampearé": "zampear",
    "zampearéis": "zampear",
    "zampearía": "zampear",
    "zampearíais": "zampear",
    "zampearíamos": "zampear",
    "zampearían": "zampear",
    "zampearías": "zampear",
    "zampeas": "zampear",
    "zampease": "zampear",
    "zampeaseis": "zampear",
    "zampeasen": "zampear",
    "zampeases": "zampear",
    "zampeaste": "zampear",
    "zampeasteis": "zampear",
    "zampee": "zampear",
    "zampeemos": "zampear",
    "zampeen": "zampear",
    "zampees": "zampear",
    "zampemos": "zampar",
    "zampen": "zampar",
    "zampeo": "zampear",
    "zampes": "zampar",
    "zampeábamos": "zampear",
    "zampeáis": "zampear",
    "zampeáramos": "zampear",
    "zampeáremos": "zampear",
    "zampeásemos": "zampear",
    "zampeé": "zampear",
    "zampeéis": "zampear",
    "zampeó": "zampear",
    "zampo": "zampar",
    "zampona": "zampón",
    "zamponas": "zampón",
    "zampones": "zampón",
    "zampoñas": "zampoña",
    "zampullines": "zampullín",
    "zampuzos": "zampuzo",
    "zampábamos": "zampar",
    "zampáis": "zampar",
    "zampáramos": "zampar",
    "zampáremos": "zampar",
    "zampásemos": "zampar",
    "zampé": "zampar",
    "zampéis": "zampar",
    "zampó": "zampar",
    "zamuja": "zamujo",
    "zamujas": "zamujo",
    "zamujos": "zamujo",
    "zanahorias": "zanahoria",
    "zanahoriates": "zanahoriate",
    "zancadas": "zancada",
    "zancadillas": "zancadilla",
    "zancadillea": "zancadillear",
    "zancadilleaba": "zancadillear",
    "zancadilleabais": "zancadillear",
    "zancadilleaban": "zancadillear",
    "zancadilleabas": "zancadillear",
    "zancadillead": "zancadillear",
    "zancadilleada": "zancadillear",
    "zancadilleadas": "zancadillear",
    "zancadilleado": "zancadillear",
    "zancadilleados": "zancadillear",
    "zancadilleamos": "zancadillear",
    "zancadillean": "zancadillear",
    "zancadilleando": "zancadillear",
    "zancadilleara": "zancadillear",
    "zancadillearais": "zancadillear",
    "zancadillearan": "zancadillear",
    "zancadillearas": "zancadillear",
    "zancadilleare": "zancadillear",
    "zancadilleareis": "zancadillear",
    "zancadillearemos": "zancadillear",
    "zancadillearen": "zancadillear",
    "zancadilleares": "zancadillear",
    "zancadillearon": "zancadillear",
    "zancadilleará": "zancadillear",
    "zancadillearán": "zancadillear",
    "zancadillearás": "zancadillear",
    "zancadillearé": "zancadillear",
    "zancadillearéis": "zancadillear",
    "zancadillearía": "zancadillear",
    "zancadillearíais": "zancadillear",
    "zancadillearíamos": "zancadillear",
    "zancadillearían": "zancadillear",
    "zancadillearías": "zancadillear",
    "zancadilleas": "zancadillear",
    "zancadillease": "zancadillear",
    "zancadilleaseis": "zancadillear",
    "zancadilleasen": "zancadillear",
    "zancadilleases": "zancadillear",
    "zancadilleaste": "zancadillear",
    "zancadilleasteis": "zancadillear",
    "zancadillee": "zancadillear",
    "zancadilleemos": "zancadillear",
    "zancadilleen": "zancadillear",
    "zancadillees": "zancadillear",
    "zancadilleo": "zancadillear",
    "zancadilleábamos": "zancadillear",
    "zancadilleáis": "zancadillear",
    "zancadilleáramos": "zancadillear",
    "zancadilleáremos": "zancadillear",
    "zancadilleásemos": "zancadillear",
    "zancadilleé": "zancadillear",
    "zancadilleéis": "zancadillear",
    "zancadilleó": "zancadillear",
    "zancajadas": "zancajada",
    "zancajea": "zancajear",
    "zancajeaba": "zancajear",
    "zancajeabais": "zancajear",
    "zancajeaban": "zancajear",
    "zancajeabas": "zancajear",
    "zancajead": "zancajear",
    "zancajeada": "zancajear",
    "zancajeadas": "zancajear",
    "zancajeado": "zancajear",
    "zancajeados": "zancajear",
    "zancajeamos": "zancajear",
    "zancajean": "zancajear",
    "zancajeando": "zancajear",
    "zancajeara": "zancajear",
    "zancajearais": "zancajear",
    "zancajearan": "zancajear",
    "zancajearas": "zancajear",
    "zancajeare": "zancajear",
    "zancajeareis": "zancajear",
    "zancajearemos": "zancajear",
    "zancajearen": "zancajear",
    "zancajeares": "zancajear",
    "zancajearon": "zancajear",
    "zancajeará": "zancajear",
    "zancajearán": "zancajear",
    "zancajearás": "zancajear",
    "zancajearé": "zancajear",
    "zancajearéis": "zancajear",
    "zancajearía": "zancajear",
    "zancajearíais": "zancajear",
    "zancajearíamos": "zancajear",
    "zancajearían": "zancajear",
    "zancajearías": "zancajear",
    "zancajeas": "zancajear",
    "zancajease": "zancajear",
    "zancajeaseis": "zancajear",
    "zancajeasen": "zancajear",
    "zancajeases": "zancajear",
    "zancajeaste": "zancajear",
    "zancajeasteis": "zancajear",
    "zancajee": "zancajear",
    "zancajeemos": "zancajear",
    "zancajeen": "zancajear",
    "zancajees": "zancajear",
    "zancajeo": "zancajear",
    "zancajeras": "zancajera",
    "zancajeábamos": "zancajear",
    "zancajeáis": "zancajear",
    "zancajeáramos": "zancajear",
    "zancajeáremos": "zancajear",
    "zancajeásemos": "zancajear",
    "zancajeé": "zancajear",
    "zancajeéis": "zancajear",
    "zancajeó": "zancajear",
    "zancajos": "zancajo",
    "zancajosa": "zancajoso",
    "zancajosas": "zancajoso",
    "zancajosos": "zancajoso",
    "zancarrones": "zancarrón",
    "zancas": "zanca",
    "zancona": "zancón",
    "zanconas": "zancón",
    "zancones": "zancón",
    "zancos": "zanco",
    "zancuda": "zancudo",
    "zancudas": "zancudo",
    "zancudos": "zancudo",
    "zanfonías": "zanfonía",
    "zangalas": "zangala",
    "zanganadas": "zanganada",
    "zanganea": "zanganear",
    "zanganeaba": "zanganear",
    "zanganeabais": "zanganear",
    "zanganeaban": "zanganear",
    "zanganeabas": "zanganear",
    "zanganead": "zanganear",
    "zanganeada": "zanganear",
    "zanganeadas": "zanganear",
    "zanganeado": "zanganear",
    "zanganeados": "zanganear",
    "zanganeamos": "zanganear",
    "zanganean": "zanganear",
    "zanganeando": "zanganear",
    "zanganeara": "zanganear",
    "zanganearais": "zanganear",
    "zanganearan": "zanganear",
    "zanganearas": "zanganear",
    "zanganeare": "zanganear",
    "zanganeareis": "zanganear",
    "zanganearemos": "zanganear",
    "zanganearen": "zanganear",
    "zanganeares": "zanganear",
    "zanganearon": "zanganear",
    "zanganeará": "zanganear",
    "zanganearán": "zanganear",
    "zanganearás": "zanganear",
    "zanganearé": "zanganear",
    "zanganearéis": "zanganear",
    "zanganearía": "zanganear",
    "zanganearíais": "zanganear",
    "zanganearíamos": "zanganear",
    "zanganearían": "zanganear",
    "zanganearías": "zanganear",
    "zanganeas": "zanganear",
    "zanganease": "zanganear",
    "zanganeaseis": "zanganear",
    "zanganeasen": "zanganear",
    "zanganeases": "zanganear",
    "zanganeaste": "zanganear",
    "zanganeasteis": "zanganear",
    "zanganee": "zanganear",
    "zanganeemos": "zanganear",
    "zanganeen": "zanganear",
    "zanganees": "zanganear",
    "zanganeo": "zanganear",
    "zanganerías": "zanganería",
    "zanganeábamos": "zanganear",
    "zanganeáis": "zanganear",
    "zanganeáramos": "zanganear",
    "zanganeáremos": "zanganear",
    "zanganeásemos": "zanganear",
    "zanganeé": "zanganear",
    "zanganeéis": "zanganear",
    "zanganeó": "zanganear",
    "zangarrea": "zangarrear",
    "zangarreaba": "zangarrear",
    "zangarreabais": "zangarrear",
    "zangarreaban": "zangarrear",
    "zangarreabas": "zangarrear",
    "zangarread": "zangarrear",
    "zangarreada": "zangarrear",
    "zangarreadas": "zangarrear",
    "zangarreado": "zangarrear",
    "zangarreados": "zangarrear",
    "zangarreamos": "zangarrear",
    "zangarrean": "zangarrear",
    "zangarreando": "zangarrear",
    "zangarreara": "zangarrear",
    "zangarrearais": "zangarrear",
    "zangarrearan": "zangarrear",
    "zangarrearas": "zangarrear",
    "zangarreare": "zangarrear",
    "zangarreareis": "zangarrear",
    "zangarrearemos": "zangarrear",
    "zangarrearen": "zangarrear",
    "zangarreares": "zangarrear",
    "zangarrearon": "zangarrear",
    "zangarreará": "zangarrear",
    "zangarrearán": "zangarrear",
    "zangarrearás": "zangarrear",
    "zangarrearé": "zangarrear",
    "zangarrearéis": "zangarrear",
    "zangarrearía": "zangarrear",
    "zangarrearíais": "zangarrear",
    "zangarrearíamos": "zangarrear",
    "zangarrearían": "zangarrear",
    "zangarrearías": "zangarrear",
    "zangarreas": "zangarrear",
    "zangarrease": "zangarrear",
    "zangarreaseis": "zangarrear",
    "zangarreasen": "zangarrear",
    "zangarreases": "zangarrear",
    "zangarreaste": "zangarrear",
    "zangarreasteis": "zangarrear",
    "zangarree": "zangarrear",
    "zangarreemos": "zangarrear",
    "zangarreen": "zangarrear",
    "zangarrees": "zangarrear",
    "zangarreo": "zangarrear",
    "zangarreábamos": "zangarrear",
    "zangarreáis": "zangarrear",
    "zangarreáramos": "zangarrear",
    "zangarreáremos": "zangarrear",
    "zangarreásemos": "zangarrear",
    "zangarreé": "zangarrear",
    "zangarreéis": "zangarrear",
    "zangarreó": "zangarrear",
    "zangarrianas": "zangarriana",
    "zangarullona": "zangarullón",
    "zangarullonas": "zangarullón",
    "zangarullones": "zangarullón",
    "zangas": "zanga",
    "zangolotea": "zangolotear",
    "zangoloteaba": "zangolotear",
    "zangoloteabais": "zangolotear",
    "zangoloteaban": "zangolotear",
    "zangoloteabas": "zangolotear",
    "zangolotead": "zangolotear",
    "zangoloteada": "zangolotear",
    "zangoloteadas": "zangolotear",
    "zangoloteado": "zangolotear",
    "zangoloteados": "zangolotear",
    "zangoloteamos": "zangolotear",
    "zangolotean": "zangolotear",
    "zangoloteando": "zangolotear",
    "zangoloteara": "zangolotear",
    "zangolotearais": "zangolotear",
    "zangolotearan": "zangolotear",
    "zangolotearas": "zangolotear",
    "zangoloteare": "zangolotear",
    "zangoloteareis": "zangolotear",
    "zangolotearemos": "zangolotear",
    "zangolotearen": "zangolotear",
    "zangoloteares": "zangolotear",
    "zangolotearon": "zangolotear",
    "zangoloteará": "zangolotear",
    "zangolotearán": "zangolotear",
    "zangolotearás": "zangolotear",
    "zangolotearé": "zangolotear",
    "zangolotearéis": "zangolotear",
    "zangolotearía": "zangolotear",
    "zangolotearíais": "zangolotear",
    "zangolotearíamos": "zangolotear",
    "zangolotearían": "zangolotear",
    "zangolotearías": "zangolotear",
    "zangoloteas": "zangolotear",
    "zangolotease": "zangolotear",
    "zangoloteaseis": "zangolotear",
    "zangoloteasen": "zangolotear",
    "zangoloteases": "zangolotear",
    "zangoloteaste": "zangolotear",
    "zangoloteasteis": "zangolotear",
    "zangolotee": "zangolotear",
    "zangoloteemos": "zangolotear",
    "zangoloteen": "zangolotear",
    "zangolotees": "zangolotear",
    "zangoloteo": "zangolotear",
    "zangoloteos": "zangoloteo",
    "zangoloteábamos": "zangolotear",
    "zangoloteáis": "zangolotear",
    "zangoloteáramos": "zangolotear",
    "zangoloteáremos": "zangolotear",
    "zangoloteásemos": "zangolotear",
    "zangoloteé": "zangolotear",
    "zangoloteéis": "zangolotear",
    "zangoloteó": "zangolotear",
    "zangona": "zangón",
    "zangonas": "zangón",
    "zangones": "zangón",
    "zangotea": "zangotear",
    "zangoteaba": "zangotear",
    "zangoteabais": "zangotear",
    "zangoteaban": "zangotear",
    "zangoteabas": "zangotear",
    "zangotead": "zangotear",
    "zangoteada": "zangotear",
    "zangoteadas": "zangotear",
    "zangoteado": "zangotear",
    "zangoteados": "zangotear",
    "zangoteamos": "zangotear",
    "zangotean": "zangotear",
    "zangoteando": "zangotear",
    "zangoteara": "zangotear",
    "zangotearais": "zangotear",
    "zangotearan": "zangotear",
    "zangotearas": "zangotear",
    "zangoteare": "zangotear",
    "zangoteareis": "zangotear",
    "zangotearemos": "zangotear",
    "zangotearen": "zangotear",
    "zangoteares": "zangotear",
    "zangotearon": "zangotear",
    "zangoteará": "zangotear",
    "zangotearán": "zangotear",
    "zangotearás": "zangotear",
    "zangotearé": "zangotear",
    "zangotearéis": "zangotear",
    "zangotearía": "zangotear",
    "zangotearíais": "zangotear",
    "zangotearíamos": "zangotear",
    "zangotearían": "zangotear",
    "zangotearías": "zangotear",
    "zangoteas": "zangotear",
    "zangotease": "zangotear",
    "zangoteaseis": "zangotear",
    "zangoteasen": "zangotear",
    "zangoteases": "zangotear",
    "zangoteaste": "zangotear",
    "zangoteasteis": "zangotear",
    "zangotee": "zangotear",
    "zangoteemos": "zangotear",
    "zangoteen": "zangotear",
    "zangotees": "zangotear",
    "zangoteo": "zangotear",
    "zangoteábamos": "zangotear",
    "zangoteáis": "zangotear",
    "zangoteáramos": "zangotear",
    "zangoteáremos": "zangotear",
    "zangoteásemos": "zangotear",
    "zangoteé": "zangotear",
    "zangoteéis": "zangotear",
    "zangoteó": "zangotear",
    "zanguanga": "zanguango",
    "zanguangadas": "zanguangada",
    "zanguangas": "zanguango",
    "zanguangos": "zanguango",
    "zanja": "zanjar",
    "zanjaba": "zanjar",
    "zanjabais": "zanjar",
    "zanjaban": "zanjar",
    "zanjabas": "zanjar",
    "zanjad": "zanjar",
    "zanjada": "zanjar",
    "zanjadas": "zanjar",
    "zanjado": "zanjar",
    "zanjados": "zanjar",
    "zanjamos": "zanjar",
    "zanjan": "zanjar",
    "zanjando": "zanjar",
    "zanjara": "zanjar",
    "zanjarais": "zanjar",
    "zanjaran": "zanjar",
    "zanjaras": "zanjar",
    "zanjare": "zanjar",
    "zanjareis": "zanjar",
    "zanjaremos": "zanjar",
    "zanjaren": "zanjar",
    "zanjares": "zanjar",
    "zanjaron": "zanjar",
    "zanjará": "zanjar",
    "zanjarán": "zanjar",
    "zanjarás": "zanjar",
    "zanjaré": "zanjar",
    "zanjaréis": "zanjar",
    "zanjaría": "zanjar",
    "zanjaríais": "zanjar",
    "zanjaríamos": "zanjar",
    "zanjarían": "zanjar",
    "zanjarías": "zanjar",
    "zanjas": "zanjar",
    "zanjase": "zanjar",
    "zanjaseis": "zanjar",
    "zanjasen": "zanjar",
    "zanjases": "zanjar",
    "zanjaste": "zanjar",
    "zanjasteis": "zanjar",
    "zanje": "zanjar",
    "zanjemos": "zanjar",
    "zanjen": "zanjar",
    "zanjes": "zanjar",
    "zanjo": "zanjar",
    "zanjones": "zanjón",
    "zanjábamos": "zanjar",
    "zanjáis": "zanjar",
    "zanjáramos": "zanjar",
    "zanjáremos": "zanjar",
    "zanjásemos": "zanjar",
    "zanjé": "zanjar",
    "zanjéis": "zanjar",
    "zanjó": "zanjar",
    "zanquea": "zanquear",
    "zanqueaba": "zanquear",
    "zanqueabais": "zanquear",
    "zanqueaban": "zanquear",
    "zanqueabas": "zanquear",
    "zanquead": "zanquear",
    "zanqueada": "zanquear",
    "zanqueadas": "zanquear",
    "zanqueado": "zanquear",
    "zanqueadora": "zanqueador",
    "zanqueadoras": "zanqueador",
    "zanqueadores": "zanqueador",
    "zanqueados": "zanquear",
    "zanqueamientos": "zanqueamiento",
    "zanqueamos": "zanquear",
    "zanquean": "zanquear",
    "zanqueando": "zanquear",
    "zanqueara": "zanquear",
    "zanquearais": "zanquear",
    "zanquearan": "zanquear",
    "zanquearas": "zanquear",
    "zanqueare": "zanquear",
    "zanqueareis": "zanquear",
    "zanquearemos": "zanquear",
    "zanquearen": "zanquear",
    "zanqueares": "zanquear",
    "zanquearon": "zanquear",
    "zanqueará": "zanquear",
    "zanquearán": "zanquear",
    "zanquearás": "zanquear",
    "zanquearé": "zanquear",
    "zanquearéis": "zanquear",
    "zanquearía": "zanquear",
    "zanquearíais": "zanquear",
    "zanquearíamos": "zanquear",
    "zanquearían": "zanquear",
    "zanquearías": "zanquear",
    "zanqueas": "zanquear",
    "zanquease": "zanquear",
    "zanqueaseis": "zanquear",
    "zanqueasen": "zanquear",
    "zanqueases": "zanquear",
    "zanqueaste": "zanquear",
    "zanqueasteis": "zanquear",
    "zanquee": "zanquear",
    "zanqueemos": "zanquear",
    "zanqueen": "zanquear",
    "zanquees": "zanquear",
    "zanqueo": "zanquear",
    "zanqueábamos": "zanquear",
    "zanqueáis": "zanquear",
    "zanqueáramos": "zanquear",
    "zanqueáremos": "zanquear",
    "zanqueásemos": "zanquear",
    "zanqueé": "zanquear",
    "zanqueéis": "zanquear",
    "zanqueó": "zanquear",
    "zanquilarga": "zanquilargo",
    "zanquilargas": "zanquilargo",
    "zanquilargos": "zanquilargo",
    "zanquituerta": "zanquituerto",
    "zanquituertas": "zanquituerto",
    "zanquituertos": "zanquituerto",
    "zanquivana": "zanquivano",
    "zanquivanas": "zanquivano",
    "zanquivanos": "zanquivano",
    "zapa": "zapar",
    "zapaba": "zapar",
    "zapabais": "zapar",
    "zapaban": "zapar",
    "zapabas": "zapar",
    "zapad": "zapar",
    "zapada": "zapar",
    "zapadas": "zapar",
    "zapado": "zapar",
    "zapadores": "zapador",
    "zapados": "zapar",
    "zapamos": "zapar",
    "zapan": "zapar",
    "zapando": "zapar",
    "zapapicos": "zapapico",
    "zapara": "zapar",
    "zaparais": "zapar",
    "zaparan": "zapar",
    "zaparas": "zapar",
    "zapare": "zapar",
    "zapareis": "zapar",
    "zaparemos": "zapar",
    "zaparen": "zapar",
    "zapares": "zapar",
    "zaparon": "zapar",
    "zaparrastra": "zaparrastrar",
    "zaparrastraba": "zaparrastrar",
    "zaparrastrabais": "zaparrastrar",
    "zaparrastraban": "zaparrastrar",
    "zaparrastrabas": "zaparrastrar",
    "zaparrastrad": "zaparrastrar",
    "zaparrastrada": "zaparrastrar",
    "zaparrastradas": "zaparrastrar",
    "zaparrastrado": "zaparrastrar",
    "zaparrastrados": "zaparrastrar",
    "zaparrastramos": "zaparrastrar",
    "zaparrastran": "zaparrastrar",
    "zaparrastrando": "zaparrastrar",
    "zaparrastrara": "zaparrastrar",
    "zaparrastrarais": "zaparrastrar",
    "zaparrastraran": "zaparrastrar",
    "zaparrastraras": "zaparrastrar",
    "zaparrastrare": "zaparrastrar",
    "zaparrastrareis": "zaparrastrar",
    "zaparrastraremos": "zaparrastrar",
    "zaparrastraren": "zaparrastrar",
    "zaparrastrares": "zaparrastrar",
    "zaparrastraron": "zaparrastrar",
    "zaparrastrará": "zaparrastrar",
    "zaparrastrarán": "zaparrastrar",
    "zaparrastrarás": "zaparrastrar",
    "zaparrastraré": "zaparrastrar",
    "zaparrastraréis": "zaparrastrar",
    "zaparrastraría": "zaparrastrar",
    "zaparrastraríais": "zaparrastrar",
    "zaparrastraríamos": "zaparrastrar",
    "zaparrastrarían": "zaparrastrar",
    "zaparrastrarías": "zaparrastrar",
    "zaparrastras": "zaparrastrar",
    "zaparrastrase": "zaparrastrar",
    "zaparrastraseis": "zaparrastrar",
    "zaparrastrasen": "zaparrastrar",
    "zaparrastrases": "zaparrastrar",
    "zaparrastraste": "zaparrastrar",
    "zaparrastrasteis": "zaparrastrar",
    "zaparrastre": "zaparrastrar",
    "zaparrastremos": "zaparrastrar",
    "zaparrastren": "zaparrastrar",
    "zaparrastres": "zaparrastrar",
    "zaparrastro": "zaparrastrar",
    "zaparrastrábamos": "zaparrastrar",
    "zaparrastráis": "zaparrastrar",
    "zaparrastráramos": "zaparrastrar",
    "zaparrastráremos": "zaparrastrar",
    "zaparrastrásemos": "zaparrastrar",
    "zaparrastré": "zaparrastrar",
    "zaparrastréis": "zaparrastrar",
    "zaparrastró": "zaparrastrar",
    "zapará": "zapar",
    "zaparán": "zapar",
    "zaparás": "zapar",
    "zaparé": "zapar",
    "zaparéis": "zapar",
    "zaparía": "zapar",
    "zaparíais": "zapar",
    "zaparíamos": "zapar",
    "zaparían": "zapar",
    "zaparías": "zapar",
    "zapas": "zapar",
    "zapase": "zapar",
    "zapaseis": "zapar",
    "zapasen": "zapar",
    "zapases": "zapar",
    "zapaste": "zapar",
    "zapasteis": "zapar",
    "zapatas": "zapata",
    "zapatazos": "zapatazo",
    "zapatea": "zapatear",
    "zapateaba": "zapatear",
    "zapateabais": "zapatear",
    "zapateaban": "zapatear",
    "zapateabas": "zapatear",
    "zapatead": "zapatear",
    "zapateada": "zapatear",
    "zapateadas": "zapatear",
    "zapateado": "zapatear",
    "zapateadora": "zapateador",
    "zapateadoras": "zapateador",
    "zapateadores": "zapateador",
    "zapateados": "zapatear",
    "zapateamos": "zapatear",
    "zapatean": "zapatear",
    "zapateando": "zapatear",
    "zapateara": "zapatear",
    "zapatearais": "zapatear",
    "zapatearan": "zapatear",
    "zapatearas": "zapatear",
    "zapateare": "zapatear",
    "zapateareis": "zapatear",
    "zapatearemos": "zapatear",
    "zapatearen": "zapatear",
    "zapateares": "zapatear",
    "zapatearon": "zapatear",
    "zapateará": "zapatear",
    "zapatearán": "zapatear",
    "zapatearás": "zapatear",
    "zapatearé": "zapatear",
    "zapatearéis": "zapatear",
    "zapatearía": "zapatear",
    "zapatearíais": "zapatear",
    "zapatearíamos": "zapatear",
    "zapatearían": "zapatear",
    "zapatearías": "zapatear",
    "zapateas": "zapatear",
    "zapatease": "zapatear",
    "zapateaseis": "zapatear",
    "zapateasen": "zapatear",
    "zapateases": "zapatear",
    "zapateaste": "zapatear",
    "zapateasteis": "zapatear",
    "zapatee": "zapatear",
    "zapateemos": "zapatear",
    "zapateen": "zapatear",
    "zapatees": "zapatear",
    "zapateo": "zapatear",
    "zapateos": "zapateo",
    "zapatera": "zapatero",
    "zapateras": "zapatero",
    "zapateros": "zapatero",
    "zapaterías": "zapatería",
    "zapatetas": "zapateta",
    "zapateábamos": "zapatear",
    "zapateáis": "zapatear",
    "zapateáramos": "zapatear",
    "zapateáremos": "zapatear",
    "zapateásemos": "zapatear",
    "zapateé": "zapatear",
    "zapateéis": "zapatear",
    "zapateó": "zapatear",
    "zapatillas": "zapatilla",
    "zapatillazos": "zapatillazo",
    "zapatillera": "zapatillero",
    "zapatilleras": "zapatillero",
    "zapatilleros": "zapatillero",
    "zapatistas": "zapatista",
    "zapatos": "zapato",
    "zapatuda": "zapatudo",
    "zapatudas": "zapatudo",
    "zapatudos": "zapatudo",
    "zape": "zapar",
    "zapea": "zapear",
    "zapeaba": "zapear",
    "zapeabais": "zapear",
    "zapeaban": "zapear",
    "zapeabas": "zapear",
    "zapead": "zapear",
    "zapeada": "zapear",
    "zapeadas": "zapear",
    "zapeado": "zapear",
    "zapeados": "zapear",
    "zapeamos": "zapear",
    "zapean": "zapear",
    "zapeando": "zapear",
    "zapeara": "zapear",
    "zapearais": "zapear",
    "zapearan": "zapear",
    "zapearas": "zapear",
    "zapeare": "zapear",
    "zapeareis": "zapear",
    "zapearemos": "zapear",
    "zapearen": "zapear",
    "zapeares": "zapear",
    "zapearon": "zapear",
    "zapeará": "zapear",
    "zapearán": "zapear",
    "zapearás": "zapear",
    "zapearé": "zapear",
    "zapearéis": "zapear",
    "zapearía": "zapear",
    "zapearíais": "zapear",
    "zapearíamos": "zapear",
    "zapearían": "zapear",
    "zapearías": "zapear",
    "zapeas": "zapear",
    "zapease": "zapear",
    "zapeaseis": "zapear",
    "zapeasen": "zapear",
    "zapeases": "zapear",
    "zapeaste": "zapear",
    "zapeasteis": "zapear",
    "zapee": "zapear",
    "zapeemos": "zapear",
    "zapeen": "zapear",
    "zapees": "zapear",
    "zapemos": "zapar",
    "zapen": "zapar",
    "zapeo": "zapear",
    "zapes": "zapar",
    "zapeábamos": "zapear",
    "zapeáis": "zapear",
    "zapeáramos": "zapear",
    "zapeáremos": "zapear",
    "zapeásemos": "zapear",
    "zapeé": "zapear",
    "zapeéis": "zapear",
    "zapeó": "zapear",
    "zapo": "zapar",
    "zapotales": "zapotal",
    "zapoteca": "zapoteco",
    "zapotecas": "zapoteco",
    "zapotecos": "zapoteco",
    "zapotes": "zapote",
    "zappings": "zapping",
    "zapábamos": "zapar",
    "zapáis": "zapar",
    "zapáramos": "zapar",
    "zapáremos": "zapar",
    "zapásemos": "zapar",
    "zapé": "zapar",
    "zapéis": "zapar",
    "zapó": "zapar",
    "zaquea": "zaquear",
    "zaqueaba": "zaquear",
    "zaqueabais": "zaquear",
    "zaqueaban": "zaquear",
    "zaqueabas": "zaquear",
    "zaquead": "zaquear",
    "zaqueada": "zaquear",
    "zaqueadas": "zaquear",
    "zaqueado": "zaquear",
    "zaqueados": "zaquear",
    "zaqueamos": "zaquear",
    "zaquean": "zaquear",
    "zaqueando": "zaquear",
    "zaqueara": "zaquear",
    "zaquearais": "zaquear",
    "zaquearan": "zaquear",
    "zaquearas": "zaquear",
    "zaqueare": "zaquear",
    "zaqueareis": "zaquear",
    "zaquearemos": "zaquear",
    "zaquearen": "zaquear",
    "zaqueares": "zaquear",
    "zaquearon": "zaquear",
    "zaqueará": "zaquear",
    "zaquearán": "zaquear",
    "zaquearás": "zaquear",
    "zaquearé": "zaquear",
    "zaquearéis": "zaquear",
    "zaquearía": "zaquear",
    "zaquearíais": "zaquear",
    "zaquearíamos": "zaquear",
    "zaquearían": "zaquear",
    "zaquearías": "zaquear",
    "zaqueas": "zaquear",
    "zaquease": "zaquear",
    "zaqueaseis": "zaquear",
    "zaqueasen": "zaquear",
    "zaqueases": "zaquear",
    "zaqueaste": "zaquear",
    "zaqueasteis": "zaquear",
    "zaquee": "zaquear",
    "zaqueemos": "zaquear",
    "zaqueen": "zaquear",
    "zaquees": "zaquear",
    "zaqueo": "zaquear",
    "zaqueos": "zaqueo",
    "zaques": "zaque",
    "zaqueábamos": "zaquear",
    "zaqueáis": "zaquear",
    "zaqueáramos": "zaquear",
    "zaqueáremos": "zaquear",
    "zaqueásemos": "zaquear",
    "zaqueé": "zaquear",
    "zaqueéis": "zaquear",
    "zaqueó": "zaquear",
    "zaquizamíes": "zaquizamí",
    "zaquizamís": "zaquizamí",
    "zarabandas": "zarabanda",
    "zarabandistas": "zarabandista",
    "zaragallas": "zaragalla",
    "zaragatas": "zaragata",
    "zaragatera": "zaragatero",
    "zaragateras": "zaragatero",
    "zaragateros": "zaragatero",
    "zaragates": "zaragate",
    "zaragatonas": "zaragatona",
    "zaragocistas": "zaragocista",
    "zaragozana": "zaragozano",
    "zaragozanas": "zaragozano",
    "zaragozanos": "zaragozano",
    "zarambeques": "zarambeque",
    "zarandadora": "zarandador",
    "zarandadoras": "zarandador",
    "zarandadores": "zarandador",
    "zarandas": "zaranda",
    "zarandea": "zarandear",
    "zarandeaba": "zarandear",
    "zarandeabais": "zarandear",
    "zarandeaban": "zarandear",
    "zarandeabas": "zarandear",
    "zarandead": "zarandear",
    "zarandeada": "zarandear",
    "zarandeadas": "zarandear",
    "zarandeado": "zarandear",
    "zarandeados": "zarandear",
    "zarandeamos": "zarandear",
    "zarandean": "zarandear",
    "zarandeando": "zarandear",
    "zarandeara": "zarandear",
    "zarandearais": "zarandear",
    "zarandearan": "zarandear",
    "zarandearas": "zarandear",
    "zarandeare": "zarandear",
    "zarandeareis": "zarandear",
    "zarandearemos": "zarandear",
    "zarandearen": "zarandear",
    "zarandeares": "zarandear",
    "zarandearon": "zarandear",
    "zarandeará": "zarandear",
    "zarandearán": "zarandear",
    "zarandearás": "zarandear",
    "zarandearé": "zarandear",
    "zarandearéis": "zarandear",
    "zarandearía": "zarandear",
    "zarandearíais": "zarandear",
    "zarandearíamos": "zarandear",
    "zarandearían": "zarandear",
    "zarandearías": "zarandear",
    "zarandeas": "zarandear",
    "zarandease": "zarandear",
    "zarandeaseis": "zarandear",
    "zarandeasen": "zarandear",
    "zarandeases": "zarandear",
    "zarandeaste": "zarandear",
    "zarandeasteis": "zarandear",
    "zarandee": "zarandear",
    "zarandeemos": "zarandear",
    "zarandeen": "zarandear",
    "zarandees": "zarandear",
    "zarandeo": "zarandear",
    "zarandeos": "zarandeo",
    "zarandera": "zarandero",
    "zaranderas": "zarandero",
    "zaranderos": "zarandero",
    "zarandeábamos": "zarandear",
    "zarandeáis": "zarandear",
    "zarandeáramos": "zarandear",
    "zarandeáremos": "zarandear",
    "zarandeásemos": "zarandear",
    "zarandeé": "zarandear",
    "zarandeéis": "zarandear",
    "zarandeó": "zarandear",
    "zarandillos": "zarandillo",
    "zarapateles": "zarapatel",
    "zarapitos": "zarapito",
    "zarca": "zarco",
    "zarcas": "zarco",
    "zarcea": "zarcear",
    "zarceaba": "zarcear",
    "zarceabais": "zarcear",
    "zarceaban": "zarcear",
    "zarceabas": "zarcear",
    "zarcead": "zarcear",
    "zarceada": "zarcear",
    "zarceadas": "zarcear",
    "zarceado": "zarcear",
    "zarceados": "zarcear",
    "zarceamos": "zarcear",
    "zarcean": "zarcear",
    "zarceando": "zarcear",
    "zarceara": "zarcear",
    "zarcearais": "zarcear",
    "zarcearan": "zarcear",
    "zarcearas": "zarcear",
    "zarceare": "zarcear",
    "zarceareis": "zarcear",
    "zarcearemos": "zarcear",
    "zarcearen": "zarcear",
    "zarceares": "zarcear",
    "zarcearon": "zarcear",
    "zarceará": "zarcear",
    "zarcearán": "zarcear",
    "zarcearás": "zarcear",
    "zarcearé": "zarcear",
    "zarcearéis": "zarcear",
    "zarcearía": "zarcear",
    "zarcearíais": "zarcear",
    "zarcearíamos": "zarcear",
    "zarcearían": "zarcear",
    "zarcearías": "zarcear",
    "zarceas": "zarcear",
    "zarcease": "zarcear",
    "zarceaseis": "zarcear",
    "zarceasen": "zarcear",
    "zarceases": "zarcear",
    "zarceaste": "zarcear",
    "zarceasteis": "zarcear",
    "zarcee": "zarcear",
    "zarceemos": "zarcear",
    "zarceen": "zarcear",
    "zarcees": "zarcear",
    "zarceo": "zarcear",
    "zarcera": "zarcero",
    "zarceras": "zarcero",
    "zarceros": "zarcero",
    "zarceábamos": "zarcear",
    "zarceáis": "zarcear",
    "zarceáramos": "zarcear",
    "zarceáremos": "zarcear",
    "zarceásemos": "zarcear",
    "zarceé": "zarcear",
    "zarceéis": "zarcear",
    "zarceña": "zarceño",
    "zarceñas": "zarceño",
    "zarceños": "zarceño",
    "zarceó": "zarcear",
    "zarcillos": "zarcillo",
    "zarcos": "zarco",
    "zares": "zar",
    "zariana": "zariano",
    "zarianas": "zariano",
    "zarianos": "zariano",
    "zarigüeyas": "zarigüeya",
    "zarinas": "zarina",
    "zarismos": "zarismo",
    "zaristas": "zarista",
    "zarjas": "zarja",
    "zarpa": "zarpar",
    "zarpaba": "zarpar",
    "zarpabais": "zarpar",
    "zarpaban": "zarpar",
    "zarpabas": "zarpar",
    "zarpad": "zarpar",
    "zarpada": "zarpar",
    "zarpadas": "zarpar",
    "zarpado": "zarpar",
    "zarpados": "zarpar",
    "zarpamos": "zarpar",
    "zarpan": "zarpar",
    "zarpando": "zarpar",
    "zarpaneles": "zarpanel",
    "zarpara": "zarpar",
    "zarparais": "zarpar",
    "zarparan": "zarpar",
    "zarparas": "zarpar",
    "zarpare": "zarpar",
    "zarpareis": "zarpar",
    "zarparemos": "zarpar",
    "zarparen": "zarpar",
    "zarpares": "zarpar",
    "zarparon": "zarpar",
    "zarpará": "zarpar",
    "zarparán": "zarpar",
    "zarparás": "zarpar",
    "zarparé": "zarpar",
    "zarparéis": "zarpar",
    "zarparía": "zarpar",
    "zarparíais": "zarpar",
    "zarparíamos": "zarpar",
    "zarparían": "zarpar",
    "zarparías": "zarpar",
    "zarpas": "zarpar",
    "zarpase": "zarpar",
    "zarpaseis": "zarpar",
    "zarpasen": "zarpar",
    "zarpases": "zarpar",
    "zarpaste": "zarpar",
    "zarpasteis": "zarpar",
    "zarpazos": "zarpazo",
    "zarpe": "zarpar",
    "zarpea": "zarpear",
    "zarpeaba": "zarpear",
    "zarpeabais": "zarpear",
    "zarpeaban": "zarpear",
    "zarpeabas": "zarpear",
    "zarpead": "zarpear",
    "zarpeada": "zarpear",
    "zarpeadas": "zarpear",
    "zarpeado": "zarpear",
    "zarpeados": "zarpear",
    "zarpeamos": "zarpear",
    "zarpean": "zarpear",
    "zarpeando": "zarpear",
    "zarpeara": "zarpear",
    "zarpearais": "zarpear",
    "zarpearan": "zarpear",
    "zarpearas": "zarpear",
    "zarpeare": "zarpear",
    "zarpeareis": "zarpear",
    "zarpearemos": "zarpear",
    "zarpearen": "zarpear",
    "zarpeares": "zarpear",
    "zarpearon": "zarpear",
    "zarpeará": "zarpear",
    "zarpearán": "zarpear",
    "zarpearás": "zarpear",
    "zarpearé": "zarpear",
    "zarpearéis": "zarpear",
    "zarpearía": "zarpear",
    "zarpearíais": "zarpear",
    "zarpearíamos": "zarpear",
    "zarpearían": "zarpear",
    "zarpearías": "zarpear",
    "zarpeas": "zarpear",
    "zarpease": "zarpear",
    "zarpeaseis": "zarpear",
    "zarpeasen": "zarpear",
    "zarpeases": "zarpear",
    "zarpeaste": "zarpear",
    "zarpeasteis": "zarpear",
    "zarpee": "zarpear",
    "zarpeemos": "zarpear",
    "zarpeen": "zarpear",
    "zarpees": "zarpear",
    "zarpemos": "zarpar",
    "zarpen": "zarpar",
    "zarpeo": "zarpear",
    "zarpes": "zarpar",
    "zarpeábamos": "zarpear",
    "zarpeáis": "zarpear",
    "zarpeáramos": "zarpear",
    "zarpeáremos": "zarpear",
    "zarpeásemos": "zarpear",
    "zarpeé": "zarpear",
    "zarpeéis": "zarpear",
    "zarpeó": "zarpear",
    "zarpo": "zarpar",
    "zarposa": "zarposo",
    "zarposas": "zarposo",
    "zarposos": "zarposo",
    "zarpábamos": "zarpar",
    "zarpáis": "zarpar",
    "zarpáramos": "zarpar",
    "zarpáremos": "zarpar",
    "zarpásemos": "zarpar",
    "zarpé": "zarpar",
    "zarpéis": "zarpar",
    "zarpó": "zarpar",
    "zarracinas": "zarracina",
    "zarramplinadas": "zarramplinada",
    "zarramplines": "zarramplín",
    "zarrapastras": "zarrapastra",
    "zarrapastrona": "zarrapastrón",
    "zarrapastronas": "zarrapastrón",
    "zarrapastrones": "zarrapastrón",
    "zarrapastros": "zarrapastro",
    "zarrapastrosa": "zarrapastroso",
    "zarrapastrosas": "zarrapastroso",
    "zarrapastrosos": "zarrapastroso",
    "zarrias": "zarria",
    "zarrienta": "zarriento",
    "zarrientas": "zarriento",
    "zarrientos": "zarriento",
    "zarriosa": "zarrioso",
    "zarriosas": "zarrioso",
    "zarriosos": "zarrioso",
    "zarzaganes": "zarzagán",
    "zarzaganillos": "zarzaganillo",
    "zarzales": "zarzal",
    "zarzaleña": "zarzaleño",
    "zarzaleñas": "zarzaleño",
    "zarzaleños": "zarzaleño",
    "zarzamoras": "zarzamora",
    "zarzaparrillares": "zarzaparrillar",
    "zarzaparrillas": "zarzaparrilla",
    "zarzaperrunas": "zarzaperruna",
    "zarzarrosas": "zarzarrosa",
    "zarzas": "zarza",
    "zarzos": "zarzo",
    "zarzosa": "zarzoso",
    "zarzosas": "zarzoso",
    "zarzosos": "zarzoso",
    "zarzuelas": "zarzuela",
    "zarzuelera": "zarzuelero",
    "zarzueleras": "zarzuelero",
    "zarzueleros": "zarzuelero",
    "zarzuelistas": "zarzuelista",
    "zascandilea": "zascandilear",
    "zascandileaba": "zascandilear",
    "zascandileabais": "zascandilear",
    "zascandileaban": "zascandilear",
    "zascandileabas": "zascandilear",
    "zascandilead": "zascandilear",
    "zascandileada": "zascandilear",
    "zascandileadas": "zascandilear",
    "zascandileado": "zascandilear",
    "zascandileados": "zascandilear",
    "zascandileamos": "zascandilear",
    "zascandilean": "zascandilear",
    "zascandileando": "zascandilear",
    "zascandileara": "zascandilear",
    "zascandilearais": "zascandilear",
    "zascandilearan": "zascandilear",
    "zascandilearas": "zascandilear",
    "zascandileare": "zascandilear",
    "zascandileareis": "zascandilear",
    "zascandilearemos": "zascandilear",
    "zascandilearen": "zascandilear",
    "zascandileares": "zascandilear",
    "zascandilearon": "zascandilear",
    "zascandileará": "zascandilear",
    "zascandilearán": "zascandilear",
    "zascandilearás": "zascandilear",
    "zascandilearé": "zascandilear",
    "zascandilearéis": "zascandilear",
    "zascandilearía": "zascandilear",
    "zascandilearíais": "zascandilear",
    "zascandilearíamos": "zascandilear",
    "zascandilearían": "zascandilear",
    "zascandilearías": "zascandilear",
    "zascandileas": "zascandilear",
    "zascandilease": "zascandilear",
    "zascandileaseis": "zascandilear",
    "zascandileasen": "zascandilear",
    "zascandileases": "zascandilear",
    "zascandileaste": "zascandilear",
    "zascandileasteis": "zascandilear",
    "zascandilee": "zascandilear",
    "zascandileemos": "zascandilear",
    "zascandileen": "zascandilear",
    "zascandilees": "zascandilear",
    "zascandileo": "zascandilear",
    "zascandiles": "zascandil",
    "zascandileábamos": "zascandilear",
    "zascandileáis": "zascandilear",
    "zascandileáramos": "zascandilear",
    "zascandileáremos": "zascandilear",
    "zascandileásemos": "zascandilear",
    "zascandileé": "zascandilear",
    "zascandileéis": "zascandilear",
    "zascandileó": "zascandilear",
    "zataras": "zatara",
    "zatas": "zata",
    "zaza": "zazo",
    "zazas": "zazo",
    "zazos": "zazo",
    "zazosa": "zazoso",
    "zazosas": "zazoso",
    "zazosos": "zazoso",
    "zebras": "zebra",
    "zedas": "zeda",
    "zedillas": "zedilla",
    "zegríes": "zegrí",
    "zegrís": "zegrí",
    "zelandesa": "zelandés",
    "zelandesas": "zelandés",
    "zelandeses": "zelandés",
    "zelotas": "zelota",
    "zelotes": "zelote",
    "zemstvos": "zemstvo",
    "zenda": "zendo",
    "zendas": "zendo",
    "zendos": "zendo",
    "zeneres": "zener",
    "zenes": "zen",
    "zenitales": "zenital",
    "zeolitas": "zeolita",
    "zepelines": "zepelín",
    "zeppelins": "zeppelin",
    "zetas": "zeta",
    "zettabytes": "zettabyte",
    "zeugmas": "zeugma",
    "zeumas": "zeuma",
    "zeínas": "zeína",
    "zidovudinas": "zidovudina",
    "zig-zags": "zig-zag",
    "zigofiláceos": "zigofiláceo",
    "zigomorfa": "zigomorfo",
    "zigomorfas": "zigomorfo",
    "zigomorfos": "zigomorfo",
    "zigotena": "zigoteno",
    "zigotenas": "zigoteno",
    "zigotenos": "zigoteno",
    "zigotos": "zigoto",
    "zigurates": "zigurat",
    "zigurats": "zigurat",
    "zigzags": "zigzag",
    "zigzaguea": "zigzaguear",
    "zigzagueaba": "zigzaguear",
    "zigzagueabais": "zigzaguear",
    "zigzagueaban": "zigzaguear",
    "zigzagueabas": "zigzaguear",
    "zigzaguead": "zigzaguear",
    "zigzagueada": "zigzaguear",
    "zigzagueadas": "zigzaguear",
    "zigzagueado": "zigzaguear",
    "zigzagueados": "zigzaguear",
    "zigzagueamos": "zigzaguear",
    "zigzaguean": "zigzaguear",
    "zigzagueando": "zigzaguear",
    "zigzagueantes": "zigzagueante",
    "zigzagueara": "zigzaguear",
    "zigzaguearais": "zigzaguear",
    "zigzaguearan": "zigzaguear",
    "zigzaguearas": "zigzaguear",
    "zigzagueare": "zigzaguear",
    "zigzagueareis": "zigzaguear",
    "zigzaguearemos": "zigzaguear",
    "zigzaguearen": "zigzaguear",
    "zigzagueares": "zigzaguear",
    "zigzaguearon": "zigzaguear",
    "zigzagueará": "zigzaguear",
    "zigzaguearán": "zigzaguear",
    "zigzaguearás": "zigzaguear",
    "zigzaguearé": "zigzaguear",
    "zigzaguearéis": "zigzaguear",
    "zigzaguearía": "zigzaguear",
    "zigzaguearíais": "zigzaguear",
    "zigzaguearíamos": "zigzaguear",
    "zigzaguearían": "zigzaguear",
    "zigzaguearías": "zigzaguear",
    "zigzagueas": "zigzaguear",
    "zigzaguease": "zigzaguear",
    "zigzagueaseis": "zigzaguear",
    "zigzagueasen": "zigzaguear",
    "zigzagueases": "zigzaguear",
    "zigzagueaste": "zigzaguear",
    "zigzagueasteis": "zigzaguear",
    "zigzaguee": "zigzaguear",
    "zigzagueemos": "zigzaguear",
    "zigzagueen": "zigzaguear",
    "zigzaguees": "zigzaguear",
    "zigzagueo": "zigzaguear",
    "zigzagueos": "zigzagueo",
    "zigzagueábamos": "zigzaguear",
    "zigzagueáis": "zigzaguear",
    "zigzagueáramos": "zigzaguear",
    "zigzagueáremos": "zigzaguear",
    "zigzagueásemos": "zigzaguear",
    "zigzagueé": "zigzaguear",
    "zigzagueéis": "zigzaguear",
    "zigzagueó": "zigzaguear",
    "zigósporas": "zigóspora",
    "zigótica": "zigótico",
    "zigóticas": "zigótico",
    "zigóticos": "zigótico",
    "zimasas": "zimasa",
    "zimbawesa": "zimbawés",
    "zimbawesas": "zimbawés",
    "zimbaweses": "zimbawés",
    "zimologías": "zimología",
    "zimosanes": "zimosán",
    "zimógenos": "zimógeno",
    "zimómetros": "zimómetro",
    "zincitas": "zincita",
    "zingiberácea": "zingiberáceo",
    "zingiberáceas": "zingiberáceo",
    "zingiberáceos": "zingiberáceo",
    "zinkenitas": "zinkenita",
    "zinnias": "zinnia",
    "zipas": "zipa",
    "zipizapes": "zipizape",
    "zirconatos": "zirconato",
    "zirconios": "zirconio",
    "zloties": "zloty",
    "zlotys": "zloty",
    "zoantropías": "zoantropía",
    "zocas": "zoca",
    "zocata": "zocato",
    "zocatas": "zocato",
    "zocatos": "zocato",
    "zoclos": "zoclo",
    "zocos": "zoco",
    "zodiacales": "zodiacal",
    "zodíacos": "zodíaco",
    "zofras": "zofra",
    "zoides": "zoide",
    "zoidiófila": "zoidiófilo",
    "zoidiófilas": "zoidiófilo",
    "zoidiófilos": "zoidiófilo",
    "zoilos": "zoilo",
    "zoisitas": "zoisita",
    "zollipa": "zollipar",
    "zollipaba": "zollipar",
    "zollipabais": "zollipar",
    "zollipaban": "zollipar",
    "zollipabas": "zollipar",
    "zollipad": "zollipar",
    "zollipada": "zollipar",
    "zollipadas": "zollipar",
    "zollipado": "zollipar",
    "zollipados": "zollipar",
    "zollipamos": "zollipar",
    "zollipan": "zollipar",
    "zollipando": "zollipar",
    "zollipara": "zollipar",
    "zolliparais": "zollipar",
    "zolliparan": "zollipar",
    "zolliparas": "zollipar",
    "zollipare": "zollipar",
    "zollipareis": "zollipar",
    "zolliparemos": "zollipar",
    "zolliparen": "zollipar",
    "zollipares": "zollipar",
    "zolliparon": "zollipar",
    "zollipará": "zollipar",
    "zolliparán": "zollipar",
    "zolliparás": "zollipar",
    "zolliparé": "zollipar",
    "zolliparéis": "zollipar",
    "zolliparía": "zollipar",
    "zolliparíais": "zollipar",
    "zolliparíamos": "zollipar",
    "zolliparían": "zollipar",
    "zolliparías": "zollipar",
    "zollipas": "zollipar",
    "zollipase": "zollipar",
    "zollipaseis": "zollipar",
    "zollipasen": "zollipar",
    "zollipases": "zollipar",
    "zollipaste": "zollipar",
    "zollipasteis": "zollipar",
    "zollipe": "zollipar",
    "zollipemos": "zollipar",
    "zollipen": "zollipar",
    "zollipes": "zollipar",
    "zollipo": "zollipar",
    "zollipábamos": "zollipar",
    "zollipáis": "zollipar",
    "zollipáramos": "zollipar",
    "zollipáremos": "zollipar",
    "zollipásemos": "zollipar",
    "zollipé": "zollipar",
    "zollipéis": "zollipar",
    "zollipó": "zollipar",
    "zombis": "zombi",
    "zompa": "zompo",
    "zompas": "zompo",
    "zompos": "zompo",
    "zonaciones": "zonación",
    "zonada": "zonado",
    "zonadas": "zonado",
    "zonados": "zonado",
    "zonales": "zonal",
    "zonas": "zona",
    "zoncerías": "zoncería",
    "zonificaciones": "zonificación",
    "zonulares": "zonular",
    "zonza": "zonzo",
    "zonzas": "zonzo",
    "zonzos": "zonzo",
    "zooarqueologías": "zooarqueología",
    "zoobentónica": "zoobentónico",
    "zoobentónicas": "zoobentónico",
    "zoobentónicos": "zoobentónico",
    "zoofagias": "zoofagia",
    "zoofobias": "zoofobia",
    "zoogenias": "zoogenia",
    "zoogeografías": "zoogeografía",
    "zoografías": "zoografía",
    "zoográfica": "zoográfico",
    "zoográficas": "zoográfico",
    "zoográficos": "zoográfico",
    "zoolatrías": "zoolatría",
    "zoologías": "zoología",
    "zoológica": "zoológico",
    "zoológicas": "zoológico",
    "zoológicos": "zoológico",
    "zoomes": "zoom",
    "zoonosologías": "zoonosología",
    "zooparásitos": "zooparásito",
    "zooplánctones": "zooplancton",
    "zoos": "zoo",
    "zoosemióticas": "zoosemiótica",
    "zoospermos": "zoospermo",
    "zoosporangios": "zoosporangio",
    "zootecnias": "zootecnia",
    "zootomías": "zootomía",
    "zootécnica": "zootécnico",
    "zootécnicas": "zootécnico",
    "zootécnicos": "zootécnico",
    "zopa": "zopo",
    "zopas": "zopo",
    "zopenca": "zopenco",
    "zopencas": "zopenco",
    "zopencos": "zopenco",
    "zopeteros": "zopetero",
    "zopilotes": "zopilote",
    "zopisas": "zopisa",
    "zopos": "zopo",
    "zoquetas": "zoqueta",
    "zoquetes": "zoquete",
    "zoquetuda": "zoquetudo",
    "zoquetudas": "zoquetudo",
    "zoquetudos": "zoquetudo",
    "zorcicos": "zorcico",
    "zoroastrismos": "zoroastrismo",
    "zorollos": "zorollo",
    "zorongos": "zorongo",
    "zoroástrica": "zoroástrico",
    "zoroástricas": "zoroástrico",
    "zoroástricos": "zoroástrico",
    "zorra": "zorro",
    "zorras": "zorro",
    "zorrastrona": "zorrastrón",
    "zorrastronas": "zorrastrón",
    "zorrastrones": "zorrastrón",
    "zorrera": "zorrero",
    "zorreras": "zorrero",
    "zorreros": "zorrero",
    "zorrillas": "zorrilla",
    "zorrinos": "zorrino",
    "zorroclocos": "zorrocloco",
    "zorros": "zorro",
    "zorruna": "zorruno",
    "zorrunas": "zorruno",
    "zorrunos": "zorruno",
    "zorzales": "zorzal",
    "zorápteros": "zoráptero",
    "zosteres": "zoster",
    "zotes": "zote",
    "zozobra": "zozobrar",
    "zozobraba": "zozobrar",
    "zozobrabais": "zozobrar",
    "zozobraban": "zozobrar",
    "zozobrabas": "zozobrar",
    "zozobrad": "zozobrar",
    "zozobrada": "zozobrar",
    "zozobradas": "zozobrar",
    "zozobrado": "zozobrar",
    "zozobrados": "zozobrar",
    "zozobramos": "zozobrar",
    "zozobran": "zozobrar",
    "zozobrando": "zozobrar",
    "zozobrara": "zozobrar",
    "zozobrarais": "zozobrar",
    "zozobraran": "zozobrar",
    "zozobraras": "zozobrar",
    "zozobrare": "zozobrar",
    "zozobrareis": "zozobrar",
    "zozobraremos": "zozobrar",
    "zozobraren": "zozobrar",
    "zozobrares": "zozobrar",
    "zozobraron": "zozobrar",
    "zozobrará": "zozobrar",
    "zozobrarán": "zozobrar",
    "zozobrarás": "zozobrar",
    "zozobraré": "zozobrar",
    "zozobraréis": "zozobrar",
    "zozobraría": "zozobrar",
    "zozobraríais": "zozobrar",
    "zozobraríamos": "zozobrar",
    "zozobrarían": "zozobrar",
    "zozobrarías": "zozobrar",
    "zozobras": "zozobrar",
    "zozobrase": "zozobrar",
    "zozobraseis": "zozobrar",
    "zozobrasen": "zozobrar",
    "zozobrases": "zozobrar",
    "zozobraste": "zozobrar",
    "zozobrasteis": "zozobrar",
    "zozobre": "zozobrar",
    "zozobremos": "zozobrar",
    "zozobren": "zozobrar",
    "zozobres": "zozobrar",
    "zozobro": "zozobrar",
    "zozobrosa": "zozobroso",
    "zozobrosas": "zozobroso",
    "zozobrosos": "zozobroso",
    "zozobrábamos": "zozobrar",
    "zozobráis": "zozobrar",
    "zozobráramos": "zozobrar",
    "zozobráremos": "zozobrar",
    "zozobrásemos": "zozobrar",
    "zozobré": "zozobrar",
    "zozobréis": "zozobrar",
    "zozobró": "zozobrar",
    "zoócora": "zoócoro",
    "zoócoras": "zoócoro",
    "zoócoros": "zoócoro",
    "zoófaga": "zoófago",
    "zoófagas": "zoófago",
    "zoófagos": "zoófago",
    "zoólatras": "zoólatra",
    "zoóloga": "zoólogo",
    "zoólogas": "zoólogo",
    "zoólogos": "zoólogo",
    "zoósporas": "zoóspora",
    "zuavos": "zuavo",
    "zubias": "zubia",
    "zucarina": "zucarino",
    "zucarinas": "zucarino",
    "zucarinos": "zucarino",
    "zudas": "zuda",
    "zuecos": "zueco",
    "zuelas": "zuela",
    "zulaca": "zulacar",
    "zulacaba": "zulacar",
    "zulacabais": "zulacar",
    "zulacaban": "zulacar",
    "zulacabas": "zulacar",
    "zulacad": "zulacar",
    "zulacada": "zulacar",
    "zulacadas": "zulacar",
    "zulacado": "zulacar",
    "zulacados": "zulacar",
    "zulacamos": "zulacar",
    "zulacan": "zulacar",
    "zulacando": "zulacar",
    "zulacara": "zulacar",
    "zulacarais": "zulacar",
    "zulacaran": "zulacar",
    "zulacaras": "zulacar",
    "zulacare": "zulacar",
    "zulacareis": "zulacar",
    "zulacaremos": "zulacar",
    "zulacaren": "zulacar",
    "zulacares": "zulacar",
    "zulacaron": "zulacar",
    "zulacará": "zulacar",
    "zulacarán": "zulacar",
    "zulacarás": "zulacar",
    "zulacaré": "zulacar",
    "zulacaréis": "zulacar",
    "zulacaría": "zulacar",
    "zulacaríais": "zulacar",
    "zulacaríamos": "zulacar",
    "zulacarían": "zulacar",
    "zulacarías": "zulacar",
    "zulacas": "zulacar",
    "zulacase": "zulacar",
    "zulacaseis": "zulacar",
    "zulacasen": "zulacar",
    "zulacases": "zulacar",
    "zulacaste": "zulacar",
    "zulacasteis": "zulacar",
    "zulaco": "zulacar",
    "zulacábamos": "zulacar",
    "zulacáis": "zulacar",
    "zulacáramos": "zulacar",
    "zulacáremos": "zulacar",
    "zulacásemos": "zulacar",
    "zulacó": "zulacar",
    "zulaque": "zulacar",
    "zulaquemos": "zulacar",
    "zulaquen": "zulacar",
    "zulaques": "zulaque",
    "zulaqué": "zulacar",
    "zulaquéis": "zulacar",
    "zulla": "zullar",
    "zullaba": "zullar",
    "zullabais": "zullar",
    "zullaban": "zullar",
    "zullabas": "zullar",
    "zullad": "zullar",
    "zullada": "zullar",
    "zulladas": "zullar",
    "zullado": "zullar",
    "zullados": "zullar",
    "zullamos": "zullar",
    "zullan": "zullar",
    "zullando": "zullar",
    "zullara": "zullar",
    "zullarais": "zullar",
    "zullaran": "zullar",
    "zullaras": "zullar",
    "zullare": "zullar",
    "zullareis": "zullar",
    "zullaremos": "zullar",
    "zullaren": "zullar",
    "zullares": "zullar",
    "zullaron": "zullar",
    "zullará": "zullar",
    "zullarán": "zullar",
    "zullarás": "zullar",
    "zullaré": "zullar",
    "zullaréis": "zullar",
    "zullaría": "zullar",
    "zullaríais": "zullar",
    "zullaríamos": "zullar",
    "zullarían": "zullar",
    "zullarías": "zullar",
    "zullas": "zullar",
    "zullase": "zullar",
    "zullaseis": "zullar",
    "zullasen": "zullar",
    "zullases": "zullar",
    "zullaste": "zullar",
    "zullasteis": "zullar",
    "zulle": "zullar",
    "zullemos": "zullar",
    "zullen": "zullar",
    "zulles": "zullar",
    "zullo": "zullar",
    "zullábamos": "zullar",
    "zulláis": "zullar",
    "zulláramos": "zullar",
    "zulláremos": "zullar",
    "zullásemos": "zullar",
    "zullé": "zullar",
    "zulléis": "zullar",
    "zulló": "zullar",
    "zulos": "zulo",
    "zuluses": "zulú",
    "zulúes": "zulú",
    "zulús": "zulú",
    "zulúses": "zulú",
    "zumacales": "zumacal",
    "zumacares": "zumacar",
    "zumaques": "zumaque",
    "zumayas": "zumaya",
    "zumba": "zumbar",
    "zumbaba": "zumbar",
    "zumbabais": "zumbar",
    "zumbaban": "zumbar",
    "zumbabas": "zumbar",
    "zumbad": "zumbar",
    "zumbada": "zumbar",
    "zumbadas": "zumbar",
    "zumbado": "zumbar",
    "zumbadores": "zumbador",
    "zumbados": "zumbar",
    "zumbamos": "zumbar",
    "zumban": "zumbar",
    "zumbando": "zumbar",
    "zumbara": "zumbar",
    "zumbarais": "zumbar",
    "zumbaran": "zumbar",
    "zumbaras": "zumbar",
    "zumbare": "zumbar",
    "zumbareis": "zumbar",
    "zumbaremos": "zumbar",
    "zumbaren": "zumbar",
    "zumbares": "zumbar",
    "zumbaron": "zumbar",
    "zumbará": "zumbar",
    "zumbarán": "zumbar",
    "zumbarás": "zumbar",
    "zumbaré": "zumbar",
    "zumbaréis": "zumbar",
    "zumbaría": "zumbar",
    "zumbaríais": "zumbar",
    "zumbaríamos": "zumbar",
    "zumbarían": "zumbar",
    "zumbarías": "zumbar",
    "zumbas": "zumbar",
    "zumbase": "zumbar",
    "zumbaseis": "zumbar",
    "zumbasen": "zumbar",
    "zumbases": "zumbar",
    "zumbaste": "zumbar",
    "zumbasteis": "zumbar",
    "zumbe": "zumbar",
    "zumbeles": "zumbel",
    "zumbemos": "zumbar",
    "zumben": "zumbar",
    "zumbes": "zumbar",
    "zumbidos": "zumbido",
    "zumbo": "zumbar",
    "zumbona": "zumbón",
    "zumbonas": "zumbón",
    "zumbones": "zumbón",
    "zumbos": "zumbo",
    "zumbábamos": "zumbar",
    "zumbáis": "zumbar",
    "zumbáramos": "zumbar",
    "zumbáremos": "zumbar",
    "zumbásemos": "zumbar",
    "zumbé": "zumbar",
    "zumbéis": "zumbar",
    "zumbó": "zumbar",
    "zumienta": "zumiento",
    "zumientas": "zumiento",
    "zumientos": "zumiento",
    "zumos": "zumo",
    "zumosa": "zumoso",
    "zumosas": "zumoso",
    "zumosos": "zumoso",
    "zunas": "zuna",
    "zuncha": "zunchar",
    "zunchaba": "zunchar",
    "zunchabais": "zunchar",
    "zunchaban": "zunchar",
    "zunchabas": "zunchar",
    "zunchad": "zunchar",
    "zunchada": "zunchar",
    "zunchadas": "zunchar",
    "zunchado": "zunchar",
    "zunchados": "zunchar",
    "zunchamos": "zunchar",
    "zunchan": "zunchar",
    "zunchando": "zunchar",
    "zunchara": "zunchar",
    "zuncharais": "zunchar",
    "zuncharan": "zunchar",
    "zuncharas": "zunchar",
    "zunchare": "zunchar",
    "zunchareis": "zunchar",
    "zuncharemos": "zunchar",
    "zuncharen": "zunchar",
    "zunchares": "zunchar",
    "zuncharon": "zunchar",
    "zunchará": "zunchar",
    "zuncharán": "zunchar",
    "zuncharás": "zunchar",
    "zuncharé": "zunchar",
    "zuncharéis": "zunchar",
    "zuncharía": "zunchar",
    "zuncharíais": "zunchar",
    "zuncharíamos": "zunchar",
    "zuncharían": "zunchar",
    "zuncharías": "zunchar",
    "zunchas": "zunchar",
    "zunchase": "zunchar",
    "zunchaseis": "zunchar",
    "zunchasen": "zunchar",
    "zunchases": "zunchar",
    "zunchaste": "zunchar",
    "zunchasteis": "zunchar",
    "zunche": "zunchar",
    "zunchemos": "zunchar",
    "zunchen": "zunchar",
    "zunches": "zunchar",
    "zuncho": "zunchar",
    "zunchos": "zuncho",
    "zunchábamos": "zunchar",
    "zuncháis": "zunchar",
    "zuncháramos": "zunchar",
    "zuncháremos": "zunchar",
    "zunchásemos": "zunchar",
    "zunché": "zunchar",
    "zunchéis": "zunchar",
    "zunchó": "zunchar",
    "zupias": "zupia",
    "zura": "zuro",
    "zurana": "zurano",
    "zuranas": "zurano",
    "zuranos": "zurano",
    "zuras": "zuro",
    "zurce": "zurcir",
    "zurcen": "zurcir",
    "zurces": "zurcir",
    "zurcid": "zurcir",
    "zurcida": "zurcir",
    "zurcidas": "zurcir",
    "zurcido": "zurcir",
    "zurcidora": "zurcidor",
    "zurcidoras": "zurcidor",
    "zurcidores": "zurcidor",
    "zurcidos": "zurcir",
    "zurciduras": "zurcidura",
    "zurciendo": "zurcir",
    "zurciera": "zurcir",
    "zurcierais": "zurcir",
    "zurcieran": "zurcir",
    "zurcieras": "zurcir",
    "zurciere": "zurcir",
    "zurciereis": "zurcir",
    "zurcieren": "zurcir",
    "zurcieres": "zurcir",
    "zurcieron": "zurcir",
    "zurciese": "zurcir",
    "zurcieseis": "zurcir",
    "zurciesen": "zurcir",
    "zurcieses": "zurcir",
    "zurcimos": "zurcir",
    "zurciremos": "zurcir",
    "zurcirá": "zurcir",
    "zurcirán": "zurcir",
    "zurcirás": "zurcir",
    "zurciré": "zurcir",
    "zurciréis": "zurcir",
    "zurciría": "zurcir",
    "zurciríais": "zurcir",
    "zurciríamos": "zurcir",
    "zurcirían": "zurcir",
    "zurcirías": "zurcir",
    "zurciste": "zurcir",
    "zurcisteis": "zurcir",
    "zurciéramos": "zurcir",
    "zurciéremos": "zurcir",
    "zurciésemos": "zurcir",
    "zurció": "zurcir",
    "zurcí": "zurcir",
    "zurcía": "zurcir",
    "zurcíais": "zurcir",
    "zurcíamos": "zurcir",
    "zurcían": "zurcir",
    "zurcías": "zurcir",
    "zurcís": "zurcir",
    "zurda": "zurdo",
    "zurdas": "zurdo",
    "zurdazos": "zurdazo",
    "zurderas": "zurdera",
    "zurderías": "zurdería",
    "zurdos": "zurdo",
    "zurea": "zurear",
    "zureaba": "zurear",
    "zureabais": "zurear",
    "zureaban": "zurear",
    "zureabas": "zurear",
    "zuread": "zurear",
    "zureada": "zurear",
    "zureadas": "zurear",
    "zureado": "zurear",
    "zureados": "zurear",
    "zureamos": "zurear",
    "zurean": "zurear",
    "zureando": "zurear",
    "zureara": "zurear",
    "zurearais": "zurear",
    "zurearan": "zurear",
    "zurearas": "zurear",
    "zureare": "zurear",
    "zureareis": "zurear",
    "zurearemos": "zurear",
    "zurearen": "zurear",
    "zureares": "zurear",
    "zurearon": "zurear",
    "zureará": "zurear",
    "zurearán": "zurear",
    "zurearás": "zurear",
    "zurearé": "zurear",
    "zurearéis": "zurear",
    "zurearía": "zurear",
    "zurearíais": "zurear",
    "zurearíamos": "zurear",
    "zurearían": "zurear",
    "zurearías": "zurear",
    "zureas": "zurear",
    "zurease": "zurear",
    "zureaseis": "zurear",
    "zureasen": "zurear",
    "zureases": "zurear",
    "zureaste": "zurear",
    "zureasteis": "zurear",
    "zuree": "zurear",
    "zureemos": "zurear",
    "zureen": "zurear",
    "zurees": "zurear",
    "zureo": "zurear",
    "zureos": "zureo",
    "zureábamos": "zurear",
    "zureáis": "zurear",
    "zureáramos": "zurear",
    "zureáremos": "zurear",
    "zureásemos": "zurear",
    "zureé": "zurear",
    "zureéis": "zurear",
    "zureó": "zurear",
    "zurita": "zurito",
    "zuritas": "zurito",
    "zuritos": "zurito",
    "zuros": "zuro",
    "zurra": "zurrar",
    "zurraba": "zurrar",
    "zurrabais": "zurrar",
    "zurraban": "zurrar",
    "zurrabas": "zurrar",
    "zurrad": "zurrar",
    "zurrada": "zurrar",
    "zurradas": "zurrar",
    "zurrado": "zurrar",
    "zurradores": "zurrador",
    "zurrados": "zurrar",
    "zurramos": "zurrar",
    "zurran": "zurrar",
    "zurrando": "zurrar",
    "zurrapas": "zurrapa",
    "zurrapienta": "zurrapiento",
    "zurrapientas": "zurrapiento",
    "zurrapientos": "zurrapiento",
    "zurraposa": "zurraposo",
    "zurraposas": "zurraposo",
    "zurraposos": "zurraposo",
    "zurrara": "zurrar",
    "zurrarais": "zurrar",
    "zurraran": "zurrar",
    "zurraras": "zurrar",
    "zurrare": "zurrar",
    "zurrareis": "zurrar",
    "zurraremos": "zurrar",
    "zurraren": "zurrar",
    "zurrares": "zurrar",
    "zurraron": "zurrar",
    "zurrará": "zurrar",
    "zurrarán": "zurrar",
    "zurrarás": "zurrar",
    "zurraré": "zurrar",
    "zurraréis": "zurrar",
    "zurraría": "zurrar",
    "zurraríais": "zurrar",
    "zurraríamos": "zurrar",
    "zurrarían": "zurrar",
    "zurrarías": "zurrar",
    "zurras": "zurrar",
    "zurrase": "zurrar",
    "zurraseis": "zurrar",
    "zurrasen": "zurrar",
    "zurrases": "zurrar",
    "zurraste": "zurrar",
    "zurrasteis": "zurrar",
    "zurre": "zurrar",
    "zurremos": "zurrar",
    "zurren": "zurrar",
    "zurres": "zurrar",
    "zurriaga": "zurriago",
    "zurriagaba": "zurriagar",
    "zurriagabais": "zurriagar",
    "zurriagaban": "zurriagar",
    "zurriagabas": "zurriagar",
    "zurriagad": "zurriagar",
    "zurriagada": "zurriagar",
    "zurriagadas": "zurriagar",
    "zurriagado": "zurriagar",
    "zurriagados": "zurriagar",
    "zurriagamos": "zurriagar",
    "zurriagan": "zurriagar",
    "zurriagando": "zurriagar",
    "zurriagara": "zurriagar",
    "zurriagarais": "zurriagar",
    "zurriagaran": "zurriagar",
    "zurriagaras": "zurriagar",
    "zurriagare": "zurriagar",
    "zurriagareis": "zurriagar",
    "zurriagaremos": "zurriagar",
    "zurriagaren": "zurriagar",
    "zurriagares": "zurriagar",
    "zurriagaron": "zurriagar",
    "zurriagará": "zurriagar",
    "zurriagarán": "zurriagar",
    "zurriagarás": "zurriagar",
    "zurriagaré": "zurriagar",
    "zurriagaréis": "zurriagar",
    "zurriagaría": "zurriagar",
    "zurriagaríais": "zurriagar",
    "zurriagaríamos": "zurriagar",
    "zurriagarían": "zurriagar",
    "zurriagarías": "zurriagar",
    "zurriagas": "zurriago",
    "zurriagase": "zurriagar",
    "zurriagaseis": "zurriagar",
    "zurriagasen": "zurriagar",
    "zurriagases": "zurriagar",
    "zurriagaste": "zurriagar",
    "zurriagasteis": "zurriagar",
    "zurriagazos": "zurriagazo",
    "zurriago": "zurriagar",
    "zurriagos": "zurriago",
    "zurriague": "zurriagar",
    "zurriaguemos": "zurriagar",
    "zurriaguen": "zurriagar",
    "zurriagues": "zurriagar",
    "zurriagué": "zurriagar",
    "zurriaguéis": "zurriagar",
    "zurriagábamos": "zurriagar",
    "zurriagáis": "zurriagar",
    "zurriagáramos": "zurriagar",
    "zurriagáremos": "zurriagar",
    "zurriagásemos": "zurriagar",
    "zurriagó": "zurriagar",
    "zurriburris": "zurriburri",
    "zurridos": "zurrido",
    "zurro": "zurrar",
    "zurronadas": "zurronada",
    "zurronas": "zurrona",
    "zurrones": "zurrón",
    "zurruscos": "zurrusco",
    "zurrábamos": "zurrar",
    "zurráis": "zurrar",
    "zurráramos": "zurrar",
    "zurráremos": "zurrar",
    "zurrásemos": "zurrar",
    "zurré": "zurrar",
    "zurréis": "zurrar",
    "zurró": "zurrar",
    "zurullos": "zurullo",
    "zurupetos": "zurupeto",
    "zurza": "zurcir",
    "zurzamos": "zurcir",
    "zurzan": "zurcir",
    "zurzas": "zurcir",
    "zurzo": "zurcir",
    "zurzáis": "zurcir",
    "zutana": "zutano",
    "zutanas": "zutano",
    "zutanos": "zutano",
    "zuños": "zuño",
    "zámbiga": "zámbigo",
    "zámbigas": "zámbigo",
    "zámbigos": "zámbigo",
    "zánganos": "zángano",
    "zéjeles": "zéjel",
    "zíngara": "zíngaro",
    "zíngaras": "zíngaro",
    "zíngaros": "zíngaro",
    "zócalos": "zócalo",
    "zónulas": "zónula",
    "zúas": "zúa",
    "Å": "ångström",
    "ábacos": "ábaco",
    "ábregos": "ábrego",
    "ábsidas": "ábsida",
    "ábsides": "ábside",
    "ácanas": "ácana",
    "ácaros": "ácaro",
    "ácida": "ácido",
    "ácidas": "ácido",
    "ácidos": "ácido",
    "ácima": "ácimo",
    "ácimas": "ácimo",
    "ácimos": "ácimo",
    "ácinos": "ácino",
    "ácoros": "ácoro",
    "ácratas": "ácrata",
    "ácrica": "ácrico",
    "ácricas": "ácrico",
    "ácricos": "ácrico",
    "ácrona": "ácrono",
    "ácronas": "ácrono",
    "ácronos": "ácrono",
    "ácuea": "ácueo",
    "ácueas": "ácueo",
    "ácueos": "ácueo",
    "áditos": "ádito",
    "áfidos": "áfido",
    "áfila": "áfilo",
    "áfilas": "áfilo",
    "áfilos": "áfilo",
    "ágama": "ágamo",
    "ágamas": "ágamo",
    "ágamos": "ágamo",
    "ágapes": "ágape",
    "ágatas": "ágata",
    "ágidas": "ágida",
    "ágiles": "ágil",
    "ágoras": "ágora",
    "ágrafa": "ágrafo",
    "ágrafas": "ágrafo",
    "ágrafes": "ágrafe",
    "ágrafos": "ágrafo",
    "ágrica": "ágrico",
    "ágricas": "ágrico",
    "ágricos": "ágrico",
    "águilas": "águila",
    "álabes": "álabe",
    "álagas": "álaga",
    "álala": "álalo",
    "álalas": "álalo",
    "álalos": "álalo",
    "álamos": "álamo",
    "álbica": "álbico",
    "álbicas": "álbico",
    "álbicos": "álbico",
    "álbumes": "álbum",
    "álcalis": "álcali",
    "álcidos": "álcido",
    "álfica": "álfico",
    "álficas": "álfico",
    "álficos": "álfico",
    "álgebras": "álgebra",
    "álgida": "álgido",
    "álgidas": "álgido",
    "álgidos": "álgido",
    "áloes": "áloe",
    "álteregos": "álterego",
    "álveos": "álveo",
    "ámbares": "ámbar",
    "ámbitos": "ámbito",
    "ánades": "ánade",
    "áncoras": "áncora",
    "ándica": "ándico",
    "ándicas": "ándico",
    "ándicos": "ándico",
    "ándidas": "ándida",
    "ánditos": "ándito",
    "ánforas": "ánfora",
    "ángeles": "ángel",
    "ángstroms": "ángstrom",
    "ángulos": "ángulo",
    "ánimas": "ánima",
    "ánimos": "ánimo",
    "ánodos": "ánodo",
    "ánsares": "ánsar",
    "ántraxes": "ántrax",
    "ánulos": "ánulo",
    "ápeirones": "ápeiron",
    "ápexes": "ápex",
    "ápices": "ápice",
    "ápidos": "ápido",
    "ápocas": "ápoca",
    "ápoda": "ápodo",
    "ápodas": "ápodo",
    "ápodos": "ápodo",
    "ápsides": "ápside",
    "áptera": "áptero",
    "ápteras": "áptero",
    "ápteros": "áptero",
    "árabes": "árabe",
    "árbitra": "árbitro",
    "árbitras": "árbitro",
    "árbitros": "árbitro",
    "árboles": "árbol",
    "áreas": "área",
    "árganas": "árgana",
    "árgica": "árgico",
    "árgicas": "árgico",
    "árgicos": "árgico",
    "árida": "árido",
    "áridas": "árido",
    "áridos": "árido",
    "árnicas": "árnica",
    "ártica": "ártico",
    "árticas": "ártico",
    "árticos": "ártico",
    "áscaris": "áscari",
    "áspera": "áspero",
    "ásperas": "áspero",
    "ásperos": "áspero",
    "áspides": "áspid",
    "ástacos": "ástaco",
    "ática": "ático",
    "áticas": "ático",
    "áticos": "ático",
    "átomos": "átomo",
    "átona": "átono",
    "átonas": "átono",
    "átonos": "átono",
    "áulica": "áulico",
    "áulicas": "áulico",
    "áulicos": "áulico",
    "áurea": "áureo",
    "áureas": "áureo",
    "áureos": "áureo",
    "áurica": "áurico",
    "áuricas": "áurico",
    "áuricos": "áurico",
    "ávaros": "ávaro",
    "ávida": "ávido",
    "ávidas": "ávido",
    "ávidos": "ávido",
    "ázima": "ázimo",
    "ázimas": "ázimo",
    "ázimos": "ázimo",
    "ébanos": "ébano",
    "écades": "écade",
    "éfetas": "éfeta",
    "éforos": "éforo",
    "égidas": "égida",
    "églogas": "égloga",
    "élites": "élite",
    "élitros": "élitro",
    "émbolos": "émbolo",
    "émica": "émico",
    "émicas": "émico",
    "émicos": "émico",
    "émula": "émulo",
    "émulas": "émulo",
    "émulos": "émulo",
    "énea": "éneo",
    "éneas": "éneo",
    "éneos": "éneo",
    "éntica": "éntico",
    "énticas": "éntico",
    "énticos": "éntico",
    "épica": "épico",
    "épicas": "épico",
    "épicos": "épico",
    "épocas": "época",
    "épsilones": "épsilon",
    "épsilons": "épsilon",
    "épulises": "épulis",
    "équidos": "équido",
    "équites": "équite",
    "éramos": "ser",
    "érebos": "érebo",
    "ésa": "ese",
    "ésas": "ese",
    "ése": "ese",
    "ésos": "ese",
    "ésta": "este",
    "éstas": "este",
    "éste": "este",
    "ésteres": "éster",
    "éstos": "este",
    "éteres": "éter",
    "ética": "ético",
    "éticas": "ético",
    "éticos": "ético",
    "étimos": "étimo",
    "étnica": "étnico",
    "étnicas": "étnico",
    "étnicos": "étnico",
    "éuscara": "éuscaro",
    "éuscaras": "éuscaro",
    "éuscaros": "éuscaro",
    "éustilos": "éustilo",
    "éxitos": "éxito",
    "éxodos": "éxodo",
    "íbamos": "ir",
    "íbera": "íbero",
    "íberas": "íbero",
    "íberos": "íbero",
    "íbices": "íbice",
    "íctica": "íctico",
    "ícticas": "íctico",
    "ícticos": "íctico",
    "ídolos": "ídolo",
    "íes": "i",
    "ígnea": "ígneo",
    "ígneas": "ígneo",
    "ígneos": "ígneo",
    "íleones": "íleon",
    "íleos": "íleo",
    "íliones": "ilion",
    "ímpetus": "ímpetu",
    "ímproba": "ímprobo",
    "ímprobas": "ímprobo",
    "ímprobos": "ímprobo",
    "ínclita": "ínclito",
    "ínclitas": "ínclito",
    "ínclitos": "ínclito",
    "íncubos": "íncubo",
    "índica": "índico",
    "índicas": "índico",
    "índices": "índice",
    "índicos": "índico",
    "índiga": "índigo",
    "índigas": "índigo",
    "índigos": "índigo",
    "índoles": "índole",
    "índrida": "índrido",
    "índridas": "índrido",
    "índridos": "índrido",
    "ínfima": "ínfimo",
    "ínfimas": "ínfimo",
    "ínfimos": "ínfimo",
    "ínfulas": "ínfula",
    "íngrima": "íngrimo",
    "íngrimas": "íngrimo",
    "íngrimos": "íngrimo",
    "ínsita": "ínsito",
    "ínsitas": "ínsito",
    "ínsitos": "ínsito",
    "ínsulas": "ínsula",
    "íntegra": "íntegro",
    "íntegras": "íntegro",
    "íntegros": "íntegro",
    "ínterines": "ínterin",
    "íntima": "íntimo",
    "íntimas": "íntimo",
    "íntimos": "íntimo",
    "ípsilones": "ípsilon",
    "ípsilons": "ípsilon",
    "írrita": "írrito",
    "írritas": "írrito",
    "írritos": "írrito",
    "ísquiones": "isquion",
    "ístmica": "ístmico",
    "ístmicas": "ístmico",
    "ístmicos": "ístmico",
    "ítala": "ítalo",
    "ítalas": "ítalo",
    "ítalos": "ítalo",
    "ítemes": "ítem",
    "ítems": "ítem",
    "ñamales": "ñamal",
    "ñames": "ñame",
    "ñandúes": "ñandú",
    "ñapas": "ñapa",
    "ñaques": "ñaque",
    "ñata": "ñato",
    "ñatas": "ñato",
    "ñatos": "ñato",
    "ñeques": "ñeque",
    "ñipes": "ñipe",
    "ñiquiñaques": "ñiquiñaque",
    "ñires": "ñire",
    "ñochas": "ñocha",
    "ñoclos": "ñoclo",
    "ñoña": "ñoño",
    "ñoñas": "ñoño",
    "ñoñeces": "ñoñez",
    "ñoñerías": "ñoñería",
    "ñoños": "ñoño",
    "ñudosa": "ñudoso",
    "ñudosas": "ñudoso",
    "ñudosos": "ñudoso",
    "ñus": "ñu",
    "ñúes": "ñu",
    "óbices": "óbice",
    "óbitos": "óbito",
    "óbolos": "óbolo",
    "ócrica": "ócrico",
    "ócricas": "ócrico",
    "ócricos": "ócrico",
    "óctupla": "óctuplo",
    "óctuplas": "óctuplo",
    "óctuples": "óctuple",
    "óctuplos": "óctuplo",
    "óhmetros": "óhmetro",
    "óhmica": "óhmico",
    "óhmicas": "óhmico",
    "óhmicos": "óhmico",
    "óleos": "óleo",
    "óleums": "óleum",
    "ómicrons": "ómicron",
    "ómnibuses": "ómnibus",
    "ómniums": "ómnium",
    "ópalos": "ópalo",
    "óperas": "ópera",
    "óptica": "óptico",
    "ópticas": "óptico",
    "ópticos": "óptico",
    "óptima": "bueno",
    "óptimas": "bueno",
    "óptimo": "bueno",
    "óptimos": "bueno",
    "órbitas": "órbita",
    "órdagos": "órdago",
    "órdenes": "orden",
    "órfica": "órfico",
    "órficas": "órfico",
    "órficos": "órfico",
    "órganos": "órgano",
    "órtica": "órtico",
    "órticas": "órtico",
    "órticos": "órtico",
    "ósculos": "ósculo",
    "ósea": "óseo",
    "óseas": "óseo",
    "óseos": "óseo",
    "ósmica": "ósmico",
    "ósmicas": "ósmico",
    "ósmicos": "ósmico",
    "óstracones": "óstracon",
    "ótica": "ótico",
    "óticas": "ótico",
    "óticos": "ótico",
    "óvalos": "óvalo",
    "óvolos": "óvolo",
    "óvulos": "óvulo",
    "óxica": "óxico",
    "óxicas": "óxico",
    "óxicos": "óxico",
    "óxidos": "óxido",
    "úes": "u",
    "úlceras": "úlcera",
    "última": "último",
    "últimas": "último",
    "últimos": "último",
    "úmbrica": "úmbrico",
    "úmbricas": "úmbrico",
    "úmbricos": "úmbrico",
    "única": "único",
    "únicas": "único",
    "únicos": "único",
    "úrica": "úrico",
    "úricas": "úrico",
    "úricos": "úrico",
    "úrsida": "úrsido",
    "úrsidas": "úrsido",
    "úrsidos": "úrsido",
    "úteros": "útero",
    "útiles": "útil",
    "úveas": "úvea",
    "úvulas": "úvula"
}
